/* Automatically generated by tree-sitter v0.25.3 (2a835ee029dca1c325e6f1c01dbce40396f6123e) */

#include "tree_sitter/parser.h"

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#ifdef _MSC_VER
#pragma optimize("", off)
#elif defined(__clang__)
#pragma clang optimize off
#elif defined(__GNUC__)
#pragma GCC optimize ("O0")
#endif

#define LANGUAGE_VERSION 14
#define STATE_COUNT 2265
#define LARGE_STATE_COUNT 972
#define SYMBOL_COUNT 842
#define ALIAS_COUNT 2
#define TOKEN_COUNT 585
#define EXTERNAL_TOKEN_COUNT 1
#define FIELD_COUNT 43
#define MAX_ALIAS_SEQUENCE_LENGTH 7
#define MAX_RESERVED_WORD_SET_SIZE 0
#define PRODUCTION_ID_COUNT 88
#define SUPERTYPE_COUNT 0

enum ts_symbol_identifiers {
  sym_identifier = 1,
  sym_program_name_keyword = 2,
  sym_module_name_keyword = 3,
  sym_define_device_keyword = 4,
  sym_define_combine_keyword = 5,
  sym_define_constant_keyword = 6,
  sym_define_type_keyword = 7,
  sym_define_variable_keyword = 8,
  sym_define_system_variable_keyword = 9,
  sym_define_connect_level_keyword = 10,
  sym_define_latching_keyword = 11,
  sym_define_mutually_exclusive_keyword = 12,
  sym_define_toggling_keyword = 13,
  sym_define_call_keyword = 14,
  sym_define_function_keyword = 15,
  sym_define_library_function_keyword = 16,
  sym_define_start_keyword = 17,
  sym_define_event_keyword = 18,
  sym_define_program_keyword = 19,
  sym_define_module_keyword = 20,
  sym_char_keyword = 21,
  sym_widechar_keyword = 22,
  sym_integer_keyword = 23,
  sym_sinteger_keyword = 24,
  sym_long_keyword = 25,
  sym_slong_keyword = 26,
  sym_float_keyword = 27,
  sym_double_keyword = 28,
  sym_variant_keyword = 29,
  sym_variantarray_keyword = 30,
  sym_local_var_keyword = 31,
  sym_stack_var_keyword = 32,
  sym_constant_keyword = 33,
  sym_volatile_keyword = 34,
  sym_non_volatile_keyword = 35,
  sym_persistent_keyword = 36,
  sym_dev_keyword = 37,
  sym_devlev_keyword = 38,
  sym_devchan_keyword = 39,
  sym_if_keyword = 40,
  sym_else_keyword = 41,
  sym_switch_keyword = 42,
  sym_case_keyword = 43,
  sym_default_keyword = 44,
  sym_while_keyword = 45,
  sym_for_keyword = 46,
  sym_break_keyword = 47,
  sym_continue_keyword = 48,
  sym_return_keyword = 49,
  sym_select_keyword = 50,
  sym_active_keyword = 51,
  sym_struct_keyword = 52,
  sym_structure_keyword = 53,
  sym_button_event_keyword = 54,
  sym_channel_event_keyword = 55,
  sym_level_event_keyword = 56,
  sym_data_event_keyword = 57,
  sym_timeline_event_keyword = 58,
  sym_custom_event_keyword = 59,
  sym_push_keyword = 60,
  sym_release_keyword = 61,
  sym_hold_keyword = 62,
  sym_repeat_keyword = 63,
  aux_sym_on_keyword_token1 = 64,
  aux_sym_off_keyword_token1 = 65,
  sym_online_keyword = 66,
  sym_offline_keyword = 67,
  sym_onerror_keyword = 68,
  sym_string_keyword = 69,
  sym_command_keyword = 70,
  sym_standby_keyword = 71,
  sym_awake_keyword = 72,
  sym_send_level_keyword = 73,
  sym_send_string_keyword = 74,
  sym_send_command_keyword = 75,
  sym_clear_buffer_keyword = 76,
  sym_create_buffer_keyword = 77,
  sym_create_multi_buffer_keyword = 78,
  sym_call_keyword = 79,
  sym_system_call_keyword = 80,
  sym_devchan_to_keyword = 81,
  sym_devchan_min_to_keyword = 82,
  sym_devchan_total_off_keyword = 83,
  sym_devchan_pulse_keyword = 84,
  sym_wait_keyword = 85,
  sym_cancel_wait_keyword = 86,
  sym_wait_until_keyword = 87,
  sym_cancel_wait_until_keyword = 88,
  sym_cancel_all_wait_keyword = 89,
  sym_cancel_all_wait_until_keyword = 90,
  sym_and = 91,
  sym_or = 92,
  sym_not = 93,
  sym_xor = 94,
  sym_band = 95,
  sym_bor = 96,
  sym_bxor = 97,
  sym_bnot = 98,
  sym_lshift = 99,
  sym_rshift = 100,
  sym___file__ = 101,
  sym___date__ = 102,
  sym___ldate__ = 103,
  sym___time__ = 104,
  sym___line__ = 105,
  sym___name__ = 106,
  sym___netlinx__ = 107,
  sym_netlinx_axi_version = 108,
  sym_url_flg_tcp = 109,
  sym_url_flg_accinfopresent = 110,
  sym_url_flg_temp = 111,
  sym_url_flg_stat_prgnetlinx = 112,
  sym_url_flg_stat_mask = 113,
  sym_url_flg_stat_lookup = 114,
  sym_url_flg_stat_connecting = 115,
  sym_url_flg_stat_waiting = 116,
  sym_url_flg_stat_connected = 117,
  sym_ip_addr_flg_dhcp = 118,
  sym_first_virtual_device = 119,
  sym_first_local_port = 120,
  sym_dynamic_virtual_device = 121,
  sym_timeline_once = 122,
  sym_timeline_repeat = 123,
  sym_timeline_absolute = 124,
  sym_timeline_relative = 125,
  sym_do_push_timed_infinite = 126,
  sym_source_type_no_address = 127,
  sym_source_type_neuron_id = 128,
  sym_source_type_ip_address = 129,
  sym_source_type_axlink = 130,
  sym_source_type_neuron_subnode_icsp = 131,
  sym_source_type_neuron_subnode_pl = 132,
  sym_source_type_ip_socket_address = 133,
  sym_source_type_rs232 = 134,
  sym_source_type_internal = 135,
  sym_source_type_mac_address = 136,
  sym_source_type_ipv4_port = 137,
  sym_source_type_ipv4_port_mac_address = 138,
  sym_source_type_ipv4_port_mac_ipv6 = 139,
  sym_internal_queue_size_index_interpreter = 140,
  sym_internal_queue_size_index_notification_mgr = 141,
  sym_internal_queue_size_index_connection_mgr = 142,
  sym_internal_queue_size_index_route_mgr = 143,
  sym_internal_queue_size_index_device_mgr = 144,
  sym_internal_queue_size_index_diagnostic_mgr = 145,
  sym_internal_queue_size_index_tcp_tx = 146,
  sym_internal_queue_size_index_ipconnection_mgr = 147,
  sym_internal_queue_size_index_message_dispatcher = 148,
  sym_internal_queue_size_index_axlink_tx = 149,
  sym_internal_queue_size_index_phastlink_tx = 150,
  sym_internal_queue_size_index_icsplontalk_tx = 151,
  sym_internal_queue_size_index_icsp232_tx = 152,
  sym_internal_queue_size_index_icspip_tx = 153,
  sym_internal_queue_size_index_ni_device = 154,
  sym_true = 155,
  sym_false = 156,
  sym_file_read_only = 157,
  sym_file_rw_new = 158,
  sym_file_rw_append = 159,
  sym_ip_tcp = 160,
  sym_ip_udp = 161,
  sym_ip_udp_2way = 162,
  sym_xml_encode_types = 163,
  sym_xml_encode_char_as_list = 164,
  sym_xml_encode_le = 165,
  sym_xml_decode_types = 166,
  sym_xml_decode_no_preserve = 167,
  sym_duet_dev_type_display_device = 168,
  sym_duet_dev_type_rfidsystem_device = 169,
  sym_duet_dev_type_lightsystem_device = 170,
  sym_duet_dev_type_io_device = 171,
  sym_duet_dev_type_relay_device = 172,
  sym_duet_dev_type_ups = 173,
  sym_duet_dev_type_amplifier = 174,
  sym_duet_dev_type_audio_conferencer = 175,
  sym_duet_dev_type_audio_mixer = 176,
  sym_duet_dev_type_audio_processor = 177,
  sym_duet_dev_type_audio_tuner_device = 178,
  sym_duet_dev_type_camera = 179,
  sym_duet_dev_type_digital_media_encoder = 180,
  sym_duet_dev_type_digital_media_decoder = 181,
  sym_duet_dev_type_digital_media_server = 182,
  sym_duet_dev_type_dss = 183,
  sym_duet_dev_type_dvr = 184,
  sym_duet_dev_type_disc_device = 185,
  sym_duet_dev_type_document_camera = 186,
  sym_duet_dev_type_audio_tape = 187,
  sym_duet_dev_type_hvac = 188,
  sym_duet_dev_type_keypad = 189,
  sym_duet_dev_type_light = 190,
  sym_duet_dev_type_monitor = 191,
  sym_duet_dev_type_motor = 192,
  sym_duet_dev_type_multi_window = 193,
  sym_duet_dev_type_pool_spa = 194,
  sym_duet_dev_type_preamp_surround_sound_processor = 195,
  sym_duet_dev_type_receiver = 196,
  sym_duet_dev_type_security_system = 197,
  sym_duet_dev_type_sensor_device = 198,
  sym_duet_dev_type_settop_box = 199,
  sym_duet_dev_type_slide_projector = 200,
  sym_duet_dev_type_switcher = 201,
  sym_duet_dev_type_text_keypad = 202,
  sym_duet_dev_type_tv = 203,
  sym_duet_dev_type_utility = 204,
  sym_duet_dev_type_vcr = 205,
  sym_duet_dev_type_video_conferencer = 206,
  sym_duet_dev_type_video_processor = 207,
  sym_duet_dev_type_video_projector = 208,
  sym_duet_dev_type_video_wall = 209,
  sym_duet_dev_type_volume_controller = 210,
  sym_duet_dev_type_weather = 211,
  sym_duet_dev_not_polled = 212,
  sym_duet_dev_polled = 213,
  sym_clkmgr_mode_network = 214,
  sym_clkmgr_mode_standalone = 215,
  sym_normal_standby = 216,
  sym_normal_wake = 217,
  sym_ip_multicast_ttl_option = 218,
  sym_ip_ttl_subnet = 219,
  sym_ip_ttl_site = 220,
  sym_ip_ttl_region = 221,
  sym_ip_ttl_continent = 222,
  sym_ip_tcp_nodelay_option = 223,
  sym_ip_nodelay_on = 224,
  sym_ip_nodelay_off = 225,
  sym_amx_error = 226,
  sym_amx_warning = 227,
  sym_amx_info = 228,
  sym_amx_debug = 229,
  sym_netlinx_login_success = 230,
  sym_netlinx_login_fail = 231,
  sym_netlinx_logout = 232,
  sym_null_str = 233,
  sym_smtp_address = 234,
  sym_smtp_port_number = 235,
  sym_smtp_username = 236,
  sym_smtp_password = 237,
  sym_smtp_require_tls = 238,
  sym_smtp_from = 239,
  sym_smtp_tls_true = 240,
  sym_smtp_tls_false = 241,
  sym_listview_on_row_select_event = 242,
  sym_data_structure_unknown = 243,
  sym_data_structure_datafeed = 244,
  sym_data_structure_datarecordset = 245,
  sym_data_structure_datarecord = 246,
  sym_data_structure_datafield = 247,
  sym_data_max_content_field_count = 248,
  sym_data_max_metadata_field_count = 249,
  sym_data_max_id_length = 250,
  sym_data_max_name_length = 251,
  sym_data_max_description_length = 252,
  sym_data_max_type_length = 253,
  sym_data_max_format_length = 254,
  sym_data_max_source_length = 255,
  sym_data_max_value_length = 256,
  sym_data_max_label_length = 257,
  sym_data_max_url_length = 258,
  sym_max_last_login_info_length = 259,
  sym_data_type_unknown = 260,
  sym_data_type_string = 261,
  sym_data_type_datetime = 262,
  sym_data_type_date = 263,
  sym_data_type_time = 264,
  sym_data_type_image = 265,
  sym_data_format_url = 266,
  sym_data_format_phone = 267,
  sym_data_format_email = 268,
  sym_data_format_iso8601 = 269,
  sym_auditlog_priv = 270,
  sym_configuration_priv = 271,
  sym_deviceconfig_priv = 272,
  sym_ftp_priv = 273,
  sym_http_priv = 274,
  sym_networkconfig_priv = 275,
  sym_programport_priv = 276,
  sym_remoteui_priv = 277,
  sym_securitycontrol_priv = 278,
  sym_softwaremanagement_priv = 279,
  sym_terminal_priv = 280,
  sym_tpadmin_priv = 281,
  sym_user1_priv = 282,
  sym_user2_priv = 283,
  sym_user3_priv = 284,
  sym_user4_priv = 285,
  sym_usermanagement_priv = 286,
  sym_valid_account = 287,
  sym_err_name_invalid = 288,
  sym_err_password_invalid = 289,
  sym_err_invalid_account = 290,
  sym_err_no_authorization = 291,
  sym_err_invalid_parameter = 292,
  sym_err_user_locked_out = 293,
  sym_err_user_account_expired = 294,
  sym_err_general_error = 295,
  sym_err_authentication_error = 296,
  sym_tls_ignore_certificate_errors = 297,
  sym_tls_validate_certificate = 298,
  sym_tdata = 299,
  sym_tchannel = 300,
  sym_tlevel = 301,
  sym_tbutton = 302,
  sym_ttimeline = 303,
  sym_tcustom = 304,
  sym_url_struct = 305,
  sym_dns_struct = 306,
  sym_ip_address_struct = 307,
  sym_dev_info_struct = 308,
  sym_clkmgr_timeserver_struct = 309,
  sym_data_feed = 310,
  sym_data_field = 311,
  sym_data_record = 312,
  sym_wc_data_feed = 313,
  sym_wc_data_field = 314,
  sym_wc_data_record = 315,
  sym_last_login_info = 316,
  sym____reserved___ = 317,
  sym_date = 318,
  sym_day = 319,
  sym_time = 320,
  sym_ldate = 321,
  sym_push_device = 322,
  sym_push_channel = 323,
  sym_push_devchan = 324,
  sym_release_device = 325,
  sym_release_channel = 326,
  sym_release_devchan = 327,
  sym_master_slot = 328,
  sym_get_pulse_time = 329,
  sym_get_timer = 330,
  sym_master_sn = 331,
  sym_system_number = 332,
  sym_button = 333,
  sym_dv_channel = 334,
  sym_data = 335,
  sym_level = 336,
  sym_timeline = 337,
  sym_channel = 338,
  sym_custom = 339,
  sym_atoi = 340,
  sym_atol = 341,
  sym_atof = 342,
  sym_itoa = 343,
  sym_itohex = 344,
  sym_hextoi = 345,
  sym_ftoa = 346,
  sym_find_string = 347,
  sym_left_string = 348,
  sym_right_string = 349,
  sym_mid_string = 350,
  sym_remove_string = 351,
  sym_get_buffer_string = 352,
  sym_get_buffer_char = 353,
  sym_get_multi_buffer_string = 354,
  sym_length_string = 355,
  sym_max_length_string = 356,
  sym_set_length_string = 357,
  sym_compare_string = 358,
  sym_upper_string = 359,
  sym_lower_string = 360,
  sym_get_last = 361,
  sym_date_to_day = 362,
  sym_date_to_month = 363,
  sym_date_to_year = 364,
  sym_time_to_hour = 365,
  sym_time_to_minute = 366,
  sym_time_to_second = 367,
  sym_day_of_week = 368,
  sym_set_pulse_time = 369,
  sym_set_timer = 370,
  sym_file_open = 371,
  sym_file_close = 372,
  sym_file_read = 373,
  sym_file_read_line = 374,
  sym_file_write = 375,
  sym_file_write_line = 376,
  sym_file_copy = 377,
  sym_file_delete = 378,
  sym_file_rename = 379,
  sym_file_dir = 380,
  sym_file_getdir = 381,
  sym_file_setdir = 382,
  sym_file_create_dir = 383,
  sym_file_remove_dir = 384,
  sym_file_seek = 385,
  sym_length_array = 386,
  sym_max_length_array = 387,
  sym_set_length_array = 388,
  sym_ip_client_open = 389,
  sym_ip_client_close = 390,
  sym_ip_server_open = 391,
  sym_ip_server_close = 392,
  sym_ip_bound_client_open = 393,
  sym_ip_mc_server_open = 394,
  sym_ip_set_option = 395,
  sym_ssh_client_open = 396,
  sym_ssh_client_close = 397,
  sym_tls_client_open = 398,
  sym_tls_client_close = 399,
  sym_do_push = 400,
  sym_do_push_timed = 401,
  sym_do_release = 402,
  sym_device_id = 403,
  sym_device_info = 404,
  sym_device_id_string = 405,
  sym_device_standby = 406,
  sym_device_wake = 407,
  sym_timeline_create = 408,
  sym_timeline_kill = 409,
  sym_timeline_reload = 410,
  sym_timeline_pause = 411,
  sym_timeline_restart = 412,
  sym_timeline_set = 413,
  sym_timeline_get = 414,
  sym_timeline_active = 415,
  sym_timeline_dynamic_id = 416,
  sym_variable_to_string = 417,
  sym_string_to_variable = 418,
  sym_length_variable_to_string = 419,
  sym_type_cast = 420,
  sym_variable_to_xml = 421,
  sym_xml_to_variable = 422,
  sym_length_variable_to_xml = 423,
  sym_redirect_string = 424,
  sym_random_number = 425,
  sym_get_unique_id = 426,
  sym_get_system_number = 427,
  sym_set_system_number = 428,
  sym_get_serial_number = 429,
  sym_get_master_build = 430,
  sym_set_validation_code = 431,
  sym_reboot = 432,
  sym_load_duet_module = 433,
  sym_unload_duet_module = 434,
  sym_duet_mem_size_set = 435,
  sym_duet_mem_size_get = 436,
  sym_dynamic_application_device = 437,
  sym_dynamic_polled_port = 438,
  sym_static_port_binding = 439,
  sym_static_ip_binding = 440,
  sym_get_dns_list = 441,
  sym_set_dns_list = 442,
  sym_get_ip_address = 443,
  sym_set_ip_address = 444,
  sym_get_url_list = 445,
  sym_add_url_entry = 446,
  sym_delete_url_entry = 447,
  sym_abs_value = 448,
  sym_max_value = 449,
  sym_min_value = 450,
  sym_exp_value = 451,
  sym_log_value = 452,
  sym_log10_value = 453,
  sym_power_value = 454,
  sym_sqrt_value = 455,
  sym_raw_be = 456,
  sym_raw_le = 457,
  sym_internal_queue_size_set = 458,
  sym_internal_queue_size_get = 459,
  sym_internal_threshold_set = 460,
  sym_internal_threshold_get = 461,
  sym_rebuilt_event = 462,
  sym_do_custom_event = 463,
  sym_clkmgr_is_network_sourced = 464,
  sym_clkmgr_set_clk_source = 465,
  sym_clkmgr_is_daylightsavings_on = 466,
  sym_clkmgr_set_daylightsavings_mode = 467,
  sym_clkmgr_get_timezone = 468,
  sym_clkmgr_set_timezone = 469,
  sym_clkmgr_get_resync_period = 470,
  sym_clkmgr_set_resync_period = 471,
  sym_clkmgr_get_daylightsavings_offset = 472,
  sym_clkmgr_set_daylightsavings_offset = 473,
  sym_clkmgr_get_active_timeserver = 474,
  sym_clkmgr_set_active_timeserver = 475,
  sym_clkmgr_get_timeservers = 476,
  sym_clkmgr_add_userdefined_timeserver = 477,
  sym_clkmgr_delete_userdefined_timeserver = 478,
  sym_clkmgr_get_start_daylightsavings_rule = 479,
  sym_clkmgr_set_start_daylightsavings_rule = 480,
  sym_clkmgr_get_end_daylightsavings_rule = 481,
  sym_clkmgr_set_end_daylightsavings_rule = 482,
  sym_set_log_level = 483,
  sym_get_log_level = 484,
  sym_amx_log = 485,
  sym_smtp_server_config_set = 486,
  sym_smtp_server_config_get = 487,
  sym_smtp_send = 488,
  sym_validate_netlinx_account = 489,
  sym_validate_netlinx_account_with_permission = 490,
  sym_authenticate_certificate = 491,
  sym_audit_netlinx_session_event = 492,
  sym_audit_netlinx_generic_event = 493,
  sym_get_available_flash_disk_space = 494,
  sym_get_max_flash_disk_space = 495,
  sym_data_create_feed = 496,
  sym_data_delete_feed = 497,
  sym_data_publish_feed = 498,
  sym_data_get_published_feed = 499,
  sym_data_add_record = 500,
  sym_data_get_event_record = 501,
  sym__wc_data_create_feed = 502,
  sym__wc_data_add_record = 503,
  sym__wc_data_get_event_record = 504,
  sym_format = 505,
  sym_astro_clock = 506,
  sym_set_outdoor_temperature = 507,
  sym_set_virtual_level_count = 508,
  sym_set_virtual_channel_count = 509,
  sym_set_virtual_port_count = 510,
  sym__wc_encode_int = 511,
  sym__wc_decode_int = 512,
  sym__wc_int = 513,
  sym__wc_to_ch_int = 514,
  sym__ch_to_wc_int = 515,
  sym__wc_find_string_int = 516,
  sym__wc_left_string_int = 517,
  sym__wc_lower_string_int = 518,
  sym__wc_mid_string_int = 519,
  sym__wc_remove_string_int = 520,
  sym__wc_right_string_int = 521,
  sym__wc_upper_string_int = 522,
  sym__wc_compare_string_int = 523,
  sym__wc_get_buffer_char_int = 524,
  sym__wc_get_buffer_string_int = 525,
  sym__wc_concat_string_int = 526,
  sym__wc_file_open_int = 527,
  sym__wc_file_close_int = 528,
  sym__wc_file_read_int = 529,
  sym__wc_file_read_line_int = 530,
  sym__wc_file_write_int = 531,
  sym__wc_file_write_line_int = 532,
  anon_sym_EQ = 533,
  aux_sym_preproc_include_token1 = 534,
  aux_sym_preproc_define_token1 = 535,
  aux_sym_preproc_warn_token1 = 536,
  aux_sym_preproc_disable_warning_token1 = 537,
  aux_sym_preproc_if_defined_token1 = 538,
  anon_sym_LF = 539,
  aux_sym_preproc_if_defined_token2 = 540,
  aux_sym_preproc_if_not_defined_token1 = 541,
  aux_sym_preproc_else_token1 = 542,
  anon_sym_COMMA = 543,
  sym_preproc_arg = 544,
  anon_sym_LBRACK = 545,
  anon_sym_RBRACK = 546,
  anon_sym_LBRACE = 547,
  anon_sym_RBRACE = 548,
  anon_sym_COLON = 549,
  anon_sym_LPAREN = 550,
  anon_sym_RPAREN = 551,
  anon_sym_SEMI = 552,
  anon_sym_BANG = 553,
  anon_sym_TILDE = 554,
  anon_sym_DASH = 555,
  anon_sym_PLUS = 556,
  anon_sym_STAR = 557,
  anon_sym_SLASH = 558,
  anon_sym_PERCENT = 559,
  anon_sym_PIPE_PIPE = 560,
  anon_sym_AMP_AMP = 561,
  anon_sym_PIPE = 562,
  anon_sym_CARET = 563,
  anon_sym_AMP = 564,
  anon_sym_EQ_EQ = 565,
  anon_sym_BANG_EQ = 566,
  anon_sym_LT_GT = 567,
  anon_sym_GT = 568,
  anon_sym_GT_EQ = 569,
  anon_sym_LT_EQ = 570,
  anon_sym_LT = 571,
  anon_sym_LT_LT = 572,
  anon_sym_GT_GT = 573,
  anon_sym_DASH_DASH = 574,
  anon_sym_PLUS_PLUS = 575,
  anon_sym_DOT = 576,
  sym_range_operator = 577,
  anon_sym_DQUOTE = 578,
  anon_sym_SQUOTE = 579,
  aux_sym_string_literal_token1 = 580,
  sym_escape_sequence = 581,
  sym_number_literal = 582,
  sym_comment = 583,
  sym__automatic_semicolon = 584,
  sym_source_file = 585,
  sym__top_level_item = 586,
  sym__block_item = 587,
  sym_on_keyword = 588,
  sym_off_keyword = 589,
  sym_devchan_on_keyword = 590,
  sym_devchan_off_keyword = 591,
  sym_program_name = 592,
  sym_module_name = 593,
  sym_preproc_include = 594,
  sym_preproc_define = 595,
  sym_preproc_warn = 596,
  sym_preproc_disable_warning = 597,
  sym_preproc_if_defined = 598,
  sym_preproc_if_not_defined = 599,
  sym_preproc_else = 600,
  sym_preproc_if_defined_in_field_declaration_list = 601,
  sym_preproc_if_not_defined_in_field_declaration_list = 602,
  sym_preproc_else_in_field_declaration_list = 603,
  sym_preproc_if_defined_in_initializer_list = 604,
  sym_preproc_if_not_defined_in_initializer_list = 605,
  sym_preproc_else_in_initializer_list = 606,
  sym_preproc_if_defined_in_initializer_list_no_comma = 607,
  sym_preproc_if_not_defined_in_initializer_list_no_comma = 608,
  sym_preproc_else_in_initializer_list_no_comma = 609,
  sym_preproc_if_defined_in_button_event_declarator = 610,
  sym_preproc_if_not_defined_in_button_event_declarator = 611,
  sym_preproc_else_in_button_event_declarator = 612,
  sym_preproc_if_defined_in_button_event_block = 613,
  sym_preproc_if_not_defined_in_button_event_block = 614,
  sym_preproc_else_in_button_event_block = 615,
  sym_preproc_if_defined_in_data_event_declarator = 616,
  sym_preproc_if_not_defined_in_data_event_declarator = 617,
  sym_preproc_else_in_data_event_declarator = 618,
  sym_preproc_if_defined_in_data_event_block = 619,
  sym_preproc_if_not_defined_in_data_event_block = 620,
  sym_preproc_else_in_data_event_block = 621,
  sym_preproc_if_defined_in_channel_event_declarator = 622,
  sym_preproc_if_not_defined_in_channel_event_declarator = 623,
  sym_preproc_else_in_channel_event_declarator = 624,
  sym_preproc_if_defined_in_channel_event_block = 625,
  sym_preproc_if_not_defined_in_channel_event_block = 626,
  sym_preproc_else_in_channel_event_block = 627,
  sym_preproc_if_defined_in_timeline_event_declarator = 628,
  sym_preproc_if_not_defined_in_timeline_event_declarator = 629,
  sym_preproc_else_in_timeline_event_declarator = 630,
  sym_preproc_if_defined_in_level_event_declarator = 631,
  sym_preproc_if_not_defined_in_level_event_declarator = 632,
  sym_preproc_else_in_level_event_declarator = 633,
  sym_preproc_if_defined_in_custom_event_declarator = 634,
  sym_preproc_if_not_defined_in_custom_event_declarator = 635,
  sym_preproc_else_in_custom_event_declarator = 636,
  sym_preproc_if_defined_in_push_event_declarator = 637,
  sym_preproc_if_not_defined_in_push_event_declarator = 638,
  sym_preproc_else_in_push_event_declarator = 639,
  sym_preproc_if_defined_in_release_event_declarator = 640,
  sym_preproc_if_not_defined_in_release_event_declarator = 641,
  sym_preproc_else_in_release_event_declarator = 642,
  sym_section = 643,
  sym_define_device_section = 644,
  sym_define_combine_section = 645,
  sym_define_connect_level_section = 646,
  sym_define_constant_section = 647,
  sym_define_type_section = 648,
  sym_define_mutually_exclusive_section = 649,
  sym_define_latching_section = 650,
  sym_define_toggling_section = 651,
  sym_define_variable_section = 652,
  sym_define_system_variable_section = 653,
  sym_define_event_section = 654,
  sym_define_start_section = 655,
  sym_define_program_section = 656,
  sym_define_function = 657,
  sym_define_call = 658,
  sym_define_module = 659,
  sym_module_definition = 660,
  sym_event_definition = 661,
  sym_data_event_definition = 662,
  sym__data_event_declarator_list = 663,
  sym_data_event_declarator = 664,
  sym_data_event_device_reference = 665,
  sym_data_event_block = 666,
  sym__data_event_block_item = 667,
  sym_data_event_handler = 668,
  sym_data_event_type = 669,
  sym_timeline_event_definition = 670,
  sym__timeline_event_declarator_list = 671,
  sym_timeline_event_declarator = 672,
  sym_timeline_event_id_reference = 673,
  sym_button_event_definition = 674,
  sym__button_event_declarator_list = 675,
  sym_button_event_declarator = 676,
  sym_button_event_devchan_reference = 677,
  sym_button_event_block = 678,
  sym__button_event_block_item = 679,
  sym_button_event_handler = 680,
  sym_button_event_type = 681,
  sym_button_event_hold_repeat = 682,
  sym_level_event_definition = 683,
  sym__level_event_declarator_list = 684,
  sym_level_event_declarator = 685,
  sym_level_event_devlev_reference = 686,
  sym_channel_event_definition = 687,
  sym__channel_event_declarator_list = 688,
  sym_channel_event_declarator = 689,
  sym_channel_event_devchan_reference = 690,
  sym_channel_event_block = 691,
  sym__channel_event_block_item = 692,
  sym_channel_event_handler = 693,
  sym_channel_event_type = 694,
  sym_custom_event_definition = 695,
  sym__custom_event_declarator_list = 696,
  sym_custom_event_declarator = 697,
  sym_custom_event_reference = 698,
  sym_push_event_definition = 699,
  sym__push_event_declarator_list = 700,
  sym_push_event_declarator = 701,
  sym_push_event_devchan_reference = 702,
  sym_release_event_definition = 703,
  sym__release_event_declarator_list = 704,
  sym_release_event_declarator = 705,
  sym_release_event_devchan_reference = 706,
  sym_array_declarator = 707,
  sym_array_field_declarator = 708,
  sym_abstract_array_declarator = 709,
  sym_init_declarator = 710,
  sym_compound_statement = 711,
  sym_storage_class_specifier = 712,
  sym_type_qualifier = 713,
  sym_type_specifier = 714,
  sym_struct_specifier = 715,
  sym_field_declaration_list = 716,
  sym__field_declaration_list_item = 717,
  sym_field_declaration = 718,
  sym__field_declaration_declarator = 719,
  sym_intrinsic_type = 720,
  sym_primitive_type = 721,
  sym_structured_type = 722,
  sym_array_return_type = 723,
  sym_function_definition = 724,
  sym_function_declaration = 725,
  sym_call_definition = 726,
  sym_declaration = 727,
  sym_type_definition = 728,
  sym__declaration_modifiers = 729,
  sym__declaration_specifiers = 730,
  sym__declarator = 731,
  sym__declaration_declarator = 732,
  sym__field_declarator = 733,
  sym__abstract_declarator = 734,
  sym_function_declarator = 735,
  sym__function_declaration_declarator = 736,
  sym_parenthesized_declarator = 737,
  sym_parenthesized_field_declarator = 738,
  sym_function_field_declarator = 739,
  sym_parameter_list = 740,
  sym_parameter_declaration = 741,
  sym_statement = 742,
  sym__top_level_statement = 743,
  sym_expression_statement = 744,
  sym_if_statement = 745,
  sym_else_clause = 746,
  sym_switch_statement = 747,
  sym_case_statement = 748,
  sym_while_statement = 749,
  sym_for_statement = 750,
  sym__for_statement_body = 751,
  sym_select_statement = 752,
  sym_active_statement = 753,
  sym_return_statement = 754,
  sym_break_statement = 755,
  sym_continue_statement = 756,
  sym_devchan_operation_statement = 757,
  sym_devchan_operation = 758,
  sym_send_string_statement = 759,
  sym_send_command_statement = 760,
  sym_send_level_statement = 761,
  sym_create_buffer_statement = 762,
  sym_create_multi_buffer_statement = 763,
  sym_clear_buffer_statement = 764,
  sym_wait_statement = 765,
  sym_wait_until_statement = 766,
  sym_cancel_all_wait_statement = 767,
  sym_cancel_all_wait_until_statement = 768,
  sym_cancel_wait_statement = 769,
  sym_cancel_wait_until_statement = 770,
  sym_call_statement = 771,
  sym_system_call_statement = 772,
  sym_expression = 773,
  sym_compiler_variable = 774,
  sym_system_variable = 775,
  sym_system_constant = 776,
  sym_system_function = 777,
  sym_system_type = 778,
  sym_assignment_expression = 779,
  sym_unary_expression = 780,
  sym_binary_expression = 781,
  sym_update_expression = 782,
  sym_subscript_expression = 783,
  sym_call_expression = 784,
  sym_comma_expression = 785,
  sym_argument_list = 786,
  sym_field_expression = 787,
  sym_parenthesized_expression = 788,
  sym_devlev_expression = 789,
  sym_devchan_expression = 790,
  sym_devchan_range_expression = 791,
  sym_string_expression = 792,
  sym_initializer_list = 793,
  sym_device_literal = 794,
  sym_string_literal = 795,
  aux_sym_source_file_repeat1 = 796,
  aux_sym_preproc_if_defined_in_field_declaration_list_repeat1 = 797,
  aux_sym_preproc_if_defined_in_initializer_list_repeat1 = 798,
  aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1 = 799,
  aux_sym_preproc_if_defined_in_button_event_declarator_repeat1 = 800,
  aux_sym_preproc_if_defined_in_button_event_declarator_repeat2 = 801,
  aux_sym_preproc_if_defined_in_button_event_block_repeat1 = 802,
  aux_sym_preproc_if_defined_in_button_event_block_repeat2 = 803,
  aux_sym_preproc_if_defined_in_data_event_declarator_repeat1 = 804,
  aux_sym_preproc_if_defined_in_data_event_declarator_repeat2 = 805,
  aux_sym_preproc_if_defined_in_data_event_block_repeat1 = 806,
  aux_sym_preproc_if_defined_in_data_event_block_repeat2 = 807,
  aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1 = 808,
  aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2 = 809,
  aux_sym_preproc_if_defined_in_channel_event_block_repeat1 = 810,
  aux_sym_preproc_if_defined_in_channel_event_block_repeat2 = 811,
  aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1 = 812,
  aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2 = 813,
  aux_sym_preproc_if_defined_in_level_event_declarator_repeat1 = 814,
  aux_sym_preproc_if_defined_in_level_event_declarator_repeat2 = 815,
  aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1 = 816,
  aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2 = 817,
  aux_sym_preproc_if_defined_in_push_event_declarator_repeat1 = 818,
  aux_sym_preproc_if_defined_in_push_event_declarator_repeat2 = 819,
  aux_sym_preproc_if_defined_in_release_event_declarator_repeat1 = 820,
  aux_sym_preproc_if_defined_in_release_event_declarator_repeat2 = 821,
  aux_sym__data_event_declarator_list_repeat1 = 822,
  aux_sym__timeline_event_declarator_list_repeat1 = 823,
  aux_sym__button_event_declarator_list_repeat1 = 824,
  aux_sym__level_event_declarator_list_repeat1 = 825,
  aux_sym__channel_event_declarator_list_repeat1 = 826,
  aux_sym__custom_event_declarator_list_repeat1 = 827,
  aux_sym__push_event_declarator_list_repeat1 = 828,
  aux_sym__release_event_declarator_list_repeat1 = 829,
  aux_sym_compound_statement_repeat1 = 830,
  aux_sym__field_declaration_declarator_repeat1 = 831,
  aux_sym_function_definition_repeat1 = 832,
  aux_sym_declaration_repeat1 = 833,
  aux_sym__declaration_specifiers_repeat1 = 834,
  aux_sym_parameter_list_repeat1 = 835,
  aux_sym_case_statement_repeat1 = 836,
  aux_sym_subscript_expression_repeat1 = 837,
  aux_sym_argument_list_repeat1 = 838,
  aux_sym_string_expression_repeat1 = 839,
  aux_sym_initializer_list_repeat1 = 840,
  aux_sym_string_literal_repeat1 = 841,
  alias_sym_field_identifier = 842,
  alias_sym_type_identifier = 843,
};

static const char * const ts_symbol_names[] = {
  [ts_builtin_sym_end] = "end",
  [sym_identifier] = "identifier",
  [sym_program_name_keyword] = "program_name_keyword",
  [sym_module_name_keyword] = "module_name_keyword",
  [sym_define_device_keyword] = "define_device_keyword",
  [sym_define_combine_keyword] = "define_combine_keyword",
  [sym_define_constant_keyword] = "define_constant_keyword",
  [sym_define_type_keyword] = "define_type_keyword",
  [sym_define_variable_keyword] = "define_variable_keyword",
  [sym_define_system_variable_keyword] = "define_system_variable_keyword",
  [sym_define_connect_level_keyword] = "define_connect_level_keyword",
  [sym_define_latching_keyword] = "define_latching_keyword",
  [sym_define_mutually_exclusive_keyword] = "define_mutually_exclusive_keyword",
  [sym_define_toggling_keyword] = "define_toggling_keyword",
  [sym_define_call_keyword] = "define_call_keyword",
  [sym_define_function_keyword] = "define_function_keyword",
  [sym_define_library_function_keyword] = "define_library_function_keyword",
  [sym_define_start_keyword] = "define_start_keyword",
  [sym_define_event_keyword] = "define_event_keyword",
  [sym_define_program_keyword] = "define_program_keyword",
  [sym_define_module_keyword] = "define_module_keyword",
  [sym_char_keyword] = "char_keyword",
  [sym_widechar_keyword] = "widechar_keyword",
  [sym_integer_keyword] = "integer_keyword",
  [sym_sinteger_keyword] = "sinteger_keyword",
  [sym_long_keyword] = "long_keyword",
  [sym_slong_keyword] = "slong_keyword",
  [sym_float_keyword] = "float_keyword",
  [sym_double_keyword] = "double_keyword",
  [sym_variant_keyword] = "variant_keyword",
  [sym_variantarray_keyword] = "variantarray_keyword",
  [sym_local_var_keyword] = "local_var_keyword",
  [sym_stack_var_keyword] = "stack_var_keyword",
  [sym_constant_keyword] = "constant_keyword",
  [sym_volatile_keyword] = "volatile_keyword",
  [sym_non_volatile_keyword] = "non_volatile_keyword",
  [sym_persistent_keyword] = "persistent_keyword",
  [sym_dev_keyword] = "dev_keyword",
  [sym_devlev_keyword] = "devlev_keyword",
  [sym_devchan_keyword] = "devchan_keyword",
  [sym_if_keyword] = "if_keyword",
  [sym_else_keyword] = "else_keyword",
  [sym_switch_keyword] = "switch_keyword",
  [sym_case_keyword] = "case_keyword",
  [sym_default_keyword] = "default_keyword",
  [sym_while_keyword] = "while_keyword",
  [sym_for_keyword] = "for_keyword",
  [sym_break_keyword] = "break_keyword",
  [sym_continue_keyword] = "continue_keyword",
  [sym_return_keyword] = "return_keyword",
  [sym_select_keyword] = "select_keyword",
  [sym_active_keyword] = "active_keyword",
  [sym_struct_keyword] = "struct_keyword",
  [sym_structure_keyword] = "structure_keyword",
  [sym_button_event_keyword] = "button_event_keyword",
  [sym_channel_event_keyword] = "channel_event_keyword",
  [sym_level_event_keyword] = "level_event_keyword",
  [sym_data_event_keyword] = "data_event_keyword",
  [sym_timeline_event_keyword] = "timeline_event_keyword",
  [sym_custom_event_keyword] = "custom_event_keyword",
  [sym_push_keyword] = "push_keyword",
  [sym_release_keyword] = "release_keyword",
  [sym_hold_keyword] = "hold_keyword",
  [sym_repeat_keyword] = "repeat_keyword",
  [aux_sym_on_keyword_token1] = "on_keyword_token1",
  [aux_sym_off_keyword_token1] = "off_keyword_token1",
  [sym_online_keyword] = "online_keyword",
  [sym_offline_keyword] = "offline_keyword",
  [sym_onerror_keyword] = "onerror_keyword",
  [sym_string_keyword] = "string_keyword",
  [sym_command_keyword] = "command_keyword",
  [sym_standby_keyword] = "standby_keyword",
  [sym_awake_keyword] = "awake_keyword",
  [sym_send_level_keyword] = "send_level_keyword",
  [sym_send_string_keyword] = "send_string_keyword",
  [sym_send_command_keyword] = "send_command_keyword",
  [sym_clear_buffer_keyword] = "clear_buffer_keyword",
  [sym_create_buffer_keyword] = "create_buffer_keyword",
  [sym_create_multi_buffer_keyword] = "create_multi_buffer_keyword",
  [sym_call_keyword] = "call_keyword",
  [sym_system_call_keyword] = "system_call_keyword",
  [sym_devchan_to_keyword] = "devchan_to_keyword",
  [sym_devchan_min_to_keyword] = "devchan_min_to_keyword",
  [sym_devchan_total_off_keyword] = "devchan_total_off_keyword",
  [sym_devchan_pulse_keyword] = "devchan_pulse_keyword",
  [sym_wait_keyword] = "wait_keyword",
  [sym_cancel_wait_keyword] = "cancel_wait_keyword",
  [sym_wait_until_keyword] = "wait_until_keyword",
  [sym_cancel_wait_until_keyword] = "cancel_wait_until_keyword",
  [sym_cancel_all_wait_keyword] = "cancel_all_wait_keyword",
  [sym_cancel_all_wait_until_keyword] = "cancel_all_wait_until_keyword",
  [sym_and] = "and",
  [sym_or] = "or",
  [sym_not] = "not",
  [sym_xor] = "xor",
  [sym_band] = "band",
  [sym_bor] = "bor",
  [sym_bxor] = "bxor",
  [sym_bnot] = "bnot",
  [sym_lshift] = "lshift",
  [sym_rshift] = "rshift",
  [sym___file__] = "__file__",
  [sym___date__] = "__date__",
  [sym___ldate__] = "__ldate__",
  [sym___time__] = "__time__",
  [sym___line__] = "__line__",
  [sym___name__] = "__name__",
  [sym___netlinx__] = "__netlinx__",
  [sym_netlinx_axi_version] = "netlinx_axi_version",
  [sym_url_flg_tcp] = "url_flg_tcp",
  [sym_url_flg_accinfopresent] = "url_flg_accinfopresent",
  [sym_url_flg_temp] = "url_flg_temp",
  [sym_url_flg_stat_prgnetlinx] = "url_flg_stat_prgnetlinx",
  [sym_url_flg_stat_mask] = "url_flg_stat_mask",
  [sym_url_flg_stat_lookup] = "url_flg_stat_lookup",
  [sym_url_flg_stat_connecting] = "url_flg_stat_connecting",
  [sym_url_flg_stat_waiting] = "url_flg_stat_waiting",
  [sym_url_flg_stat_connected] = "url_flg_stat_connected",
  [sym_ip_addr_flg_dhcp] = "ip_addr_flg_dhcp",
  [sym_first_virtual_device] = "first_virtual_device",
  [sym_first_local_port] = "first_local_port",
  [sym_dynamic_virtual_device] = "dynamic_virtual_device",
  [sym_timeline_once] = "timeline_once",
  [sym_timeline_repeat] = "timeline_repeat",
  [sym_timeline_absolute] = "timeline_absolute",
  [sym_timeline_relative] = "timeline_relative",
  [sym_do_push_timed_infinite] = "do_push_timed_infinite",
  [sym_source_type_no_address] = "source_type_no_address",
  [sym_source_type_neuron_id] = "source_type_neuron_id",
  [sym_source_type_ip_address] = "source_type_ip_address",
  [sym_source_type_axlink] = "source_type_axlink",
  [sym_source_type_neuron_subnode_icsp] = "source_type_neuron_subnode_icsp",
  [sym_source_type_neuron_subnode_pl] = "source_type_neuron_subnode_pl",
  [sym_source_type_ip_socket_address] = "source_type_ip_socket_address",
  [sym_source_type_rs232] = "source_type_rs232",
  [sym_source_type_internal] = "source_type_internal",
  [sym_source_type_mac_address] = "source_type_mac_address",
  [sym_source_type_ipv4_port] = "source_type_ipv4_port",
  [sym_source_type_ipv4_port_mac_address] = "source_type_ipv4_port_mac_address",
  [sym_source_type_ipv4_port_mac_ipv6] = "source_type_ipv4_port_mac_ipv6",
  [sym_internal_queue_size_index_interpreter] = "internal_queue_size_index_interpreter",
  [sym_internal_queue_size_index_notification_mgr] = "internal_queue_size_index_notification_mgr",
  [sym_internal_queue_size_index_connection_mgr] = "internal_queue_size_index_connection_mgr",
  [sym_internal_queue_size_index_route_mgr] = "internal_queue_size_index_route_mgr",
  [sym_internal_queue_size_index_device_mgr] = "internal_queue_size_index_device_mgr",
  [sym_internal_queue_size_index_diagnostic_mgr] = "internal_queue_size_index_diagnostic_mgr",
  [sym_internal_queue_size_index_tcp_tx] = "internal_queue_size_index_tcp_tx",
  [sym_internal_queue_size_index_ipconnection_mgr] = "internal_queue_size_index_ipconnection_mgr",
  [sym_internal_queue_size_index_message_dispatcher] = "internal_queue_size_index_message_dispatcher",
  [sym_internal_queue_size_index_axlink_tx] = "internal_queue_size_index_axlink_tx",
  [sym_internal_queue_size_index_phastlink_tx] = "internal_queue_size_index_phastlink_tx",
  [sym_internal_queue_size_index_icsplontalk_tx] = "internal_queue_size_index_icsplontalk_tx",
  [sym_internal_queue_size_index_icsp232_tx] = "internal_queue_size_index_icsp232_tx",
  [sym_internal_queue_size_index_icspip_tx] = "internal_queue_size_index_icspip_tx",
  [sym_internal_queue_size_index_ni_device] = "internal_queue_size_index_ni_device",
  [sym_true] = "true",
  [sym_false] = "false",
  [sym_file_read_only] = "file_read_only",
  [sym_file_rw_new] = "file_rw_new",
  [sym_file_rw_append] = "file_rw_append",
  [sym_ip_tcp] = "ip_tcp",
  [sym_ip_udp] = "ip_udp",
  [sym_ip_udp_2way] = "ip_udp_2way",
  [sym_xml_encode_types] = "xml_encode_types",
  [sym_xml_encode_char_as_list] = "xml_encode_char_as_list",
  [sym_xml_encode_le] = "xml_encode_le",
  [sym_xml_decode_types] = "xml_decode_types",
  [sym_xml_decode_no_preserve] = "xml_decode_no_preserve",
  [sym_duet_dev_type_display_device] = "duet_dev_type_display_device",
  [sym_duet_dev_type_rfidsystem_device] = "duet_dev_type_rfidsystem_device",
  [sym_duet_dev_type_lightsystem_device] = "duet_dev_type_lightsystem_device",
  [sym_duet_dev_type_io_device] = "duet_dev_type_io_device",
  [sym_duet_dev_type_relay_device] = "duet_dev_type_relay_device",
  [sym_duet_dev_type_ups] = "duet_dev_type_ups",
  [sym_duet_dev_type_amplifier] = "duet_dev_type_amplifier",
  [sym_duet_dev_type_audio_conferencer] = "duet_dev_type_audio_conferencer",
  [sym_duet_dev_type_audio_mixer] = "duet_dev_type_audio_mixer",
  [sym_duet_dev_type_audio_processor] = "duet_dev_type_audio_processor",
  [sym_duet_dev_type_audio_tuner_device] = "duet_dev_type_audio_tuner_device",
  [sym_duet_dev_type_camera] = "duet_dev_type_camera",
  [sym_duet_dev_type_digital_media_encoder] = "duet_dev_type_digital_media_encoder",
  [sym_duet_dev_type_digital_media_decoder] = "duet_dev_type_digital_media_decoder",
  [sym_duet_dev_type_digital_media_server] = "duet_dev_type_digital_media_server",
  [sym_duet_dev_type_dss] = "duet_dev_type_dss",
  [sym_duet_dev_type_dvr] = "duet_dev_type_dvr",
  [sym_duet_dev_type_disc_device] = "duet_dev_type_disc_device",
  [sym_duet_dev_type_document_camera] = "duet_dev_type_document_camera",
  [sym_duet_dev_type_audio_tape] = "duet_dev_type_audio_tape",
  [sym_duet_dev_type_hvac] = "duet_dev_type_hvac",
  [sym_duet_dev_type_keypad] = "duet_dev_type_keypad",
  [sym_duet_dev_type_light] = "duet_dev_type_light",
  [sym_duet_dev_type_monitor] = "duet_dev_type_monitor",
  [sym_duet_dev_type_motor] = "duet_dev_type_motor",
  [sym_duet_dev_type_multi_window] = "duet_dev_type_multi_window",
  [sym_duet_dev_type_pool_spa] = "duet_dev_type_pool_spa",
  [sym_duet_dev_type_preamp_surround_sound_processor] = "duet_dev_type_preamp_surround_sound_processor",
  [sym_duet_dev_type_receiver] = "duet_dev_type_receiver",
  [sym_duet_dev_type_security_system] = "duet_dev_type_security_system",
  [sym_duet_dev_type_sensor_device] = "duet_dev_type_sensor_device",
  [sym_duet_dev_type_settop_box] = "duet_dev_type_settop_box",
  [sym_duet_dev_type_slide_projector] = "duet_dev_type_slide_projector",
  [sym_duet_dev_type_switcher] = "duet_dev_type_switcher",
  [sym_duet_dev_type_text_keypad] = "duet_dev_type_text_keypad",
  [sym_duet_dev_type_tv] = "duet_dev_type_tv",
  [sym_duet_dev_type_utility] = "duet_dev_type_utility",
  [sym_duet_dev_type_vcr] = "duet_dev_type_vcr",
  [sym_duet_dev_type_video_conferencer] = "duet_dev_type_video_conferencer",
  [sym_duet_dev_type_video_processor] = "duet_dev_type_video_processor",
  [sym_duet_dev_type_video_projector] = "duet_dev_type_video_projector",
  [sym_duet_dev_type_video_wall] = "duet_dev_type_video_wall",
  [sym_duet_dev_type_volume_controller] = "duet_dev_type_volume_controller",
  [sym_duet_dev_type_weather] = "duet_dev_type_weather",
  [sym_duet_dev_not_polled] = "duet_dev_not_polled",
  [sym_duet_dev_polled] = "duet_dev_polled",
  [sym_clkmgr_mode_network] = "clkmgr_mode_network",
  [sym_clkmgr_mode_standalone] = "clkmgr_mode_standalone",
  [sym_normal_standby] = "normal_standby",
  [sym_normal_wake] = "normal_wake",
  [sym_ip_multicast_ttl_option] = "ip_multicast_ttl_option",
  [sym_ip_ttl_subnet] = "ip_ttl_subnet",
  [sym_ip_ttl_site] = "ip_ttl_site",
  [sym_ip_ttl_region] = "ip_ttl_region",
  [sym_ip_ttl_continent] = "ip_ttl_continent",
  [sym_ip_tcp_nodelay_option] = "ip_tcp_nodelay_option",
  [sym_ip_nodelay_on] = "ip_nodelay_on",
  [sym_ip_nodelay_off] = "ip_nodelay_off",
  [sym_amx_error] = "amx_error",
  [sym_amx_warning] = "amx_warning",
  [sym_amx_info] = "amx_info",
  [sym_amx_debug] = "amx_debug",
  [sym_netlinx_login_success] = "netlinx_login_success",
  [sym_netlinx_login_fail] = "netlinx_login_fail",
  [sym_netlinx_logout] = "netlinx_logout",
  [sym_null_str] = "null_str",
  [sym_smtp_address] = "smtp_address",
  [sym_smtp_port_number] = "smtp_port_number",
  [sym_smtp_username] = "smtp_username",
  [sym_smtp_password] = "smtp_password",
  [sym_smtp_require_tls] = "smtp_require_tls",
  [sym_smtp_from] = "smtp_from",
  [sym_smtp_tls_true] = "smtp_tls_true",
  [sym_smtp_tls_false] = "smtp_tls_false",
  [sym_listview_on_row_select_event] = "listview_on_row_select_event",
  [sym_data_structure_unknown] = "data_structure_unknown",
  [sym_data_structure_datafeed] = "data_structure_datafeed",
  [sym_data_structure_datarecordset] = "data_structure_datarecordset",
  [sym_data_structure_datarecord] = "data_structure_datarecord",
  [sym_data_structure_datafield] = "data_structure_datafield",
  [sym_data_max_content_field_count] = "data_max_content_field_count",
  [sym_data_max_metadata_field_count] = "data_max_metadata_field_count",
  [sym_data_max_id_length] = "data_max_id_length",
  [sym_data_max_name_length] = "data_max_name_length",
  [sym_data_max_description_length] = "data_max_description_length",
  [sym_data_max_type_length] = "data_max_type_length",
  [sym_data_max_format_length] = "data_max_format_length",
  [sym_data_max_source_length] = "data_max_source_length",
  [sym_data_max_value_length] = "data_max_value_length",
  [sym_data_max_label_length] = "data_max_label_length",
  [sym_data_max_url_length] = "data_max_url_length",
  [sym_max_last_login_info_length] = "max_last_login_info_length",
  [sym_data_type_unknown] = "data_type_unknown",
  [sym_data_type_string] = "data_type_string",
  [sym_data_type_datetime] = "data_type_datetime",
  [sym_data_type_date] = "data_type_date",
  [sym_data_type_time] = "data_type_time",
  [sym_data_type_image] = "data_type_image",
  [sym_data_format_url] = "data_format_url",
  [sym_data_format_phone] = "data_format_phone",
  [sym_data_format_email] = "data_format_email",
  [sym_data_format_iso8601] = "data_format_iso8601",
  [sym_auditlog_priv] = "auditlog_priv",
  [sym_configuration_priv] = "configuration_priv",
  [sym_deviceconfig_priv] = "deviceconfig_priv",
  [sym_ftp_priv] = "ftp_priv",
  [sym_http_priv] = "http_priv",
  [sym_networkconfig_priv] = "networkconfig_priv",
  [sym_programport_priv] = "programport_priv",
  [sym_remoteui_priv] = "remoteui_priv",
  [sym_securitycontrol_priv] = "securitycontrol_priv",
  [sym_softwaremanagement_priv] = "softwaremanagement_priv",
  [sym_terminal_priv] = "terminal_priv",
  [sym_tpadmin_priv] = "tpadmin_priv",
  [sym_user1_priv] = "user1_priv",
  [sym_user2_priv] = "user2_priv",
  [sym_user3_priv] = "user3_priv",
  [sym_user4_priv] = "user4_priv",
  [sym_usermanagement_priv] = "usermanagement_priv",
  [sym_valid_account] = "valid_account",
  [sym_err_name_invalid] = "err_name_invalid",
  [sym_err_password_invalid] = "err_password_invalid",
  [sym_err_invalid_account] = "err_invalid_account",
  [sym_err_no_authorization] = "err_no_authorization",
  [sym_err_invalid_parameter] = "err_invalid_parameter",
  [sym_err_user_locked_out] = "err_user_locked_out",
  [sym_err_user_account_expired] = "err_user_account_expired",
  [sym_err_general_error] = "err_general_error",
  [sym_err_authentication_error] = "err_authentication_error",
  [sym_tls_ignore_certificate_errors] = "tls_ignore_certificate_errors",
  [sym_tls_validate_certificate] = "tls_validate_certificate",
  [sym_tdata] = "tdata",
  [sym_tchannel] = "tchannel",
  [sym_tlevel] = "tlevel",
  [sym_tbutton] = "tbutton",
  [sym_ttimeline] = "ttimeline",
  [sym_tcustom] = "tcustom",
  [sym_url_struct] = "url_struct",
  [sym_dns_struct] = "dns_struct",
  [sym_ip_address_struct] = "ip_address_struct",
  [sym_dev_info_struct] = "dev_info_struct",
  [sym_clkmgr_timeserver_struct] = "clkmgr_timeserver_struct",
  [sym_data_feed] = "data_feed",
  [sym_data_field] = "data_field",
  [sym_data_record] = "data_record",
  [sym_wc_data_feed] = "wc_data_feed",
  [sym_wc_data_field] = "wc_data_field",
  [sym_wc_data_record] = "wc_data_record",
  [sym_last_login_info] = "last_login_info",
  [sym____reserved___] = "___reserved___",
  [sym_date] = "date",
  [sym_day] = "day",
  [sym_time] = "time",
  [sym_ldate] = "ldate",
  [sym_push_device] = "push_device",
  [sym_push_channel] = "push_channel",
  [sym_push_devchan] = "push_devchan",
  [sym_release_device] = "release_device",
  [sym_release_channel] = "release_channel",
  [sym_release_devchan] = "release_devchan",
  [sym_master_slot] = "master_slot",
  [sym_get_pulse_time] = "get_pulse_time",
  [sym_get_timer] = "get_timer",
  [sym_master_sn] = "master_sn",
  [sym_system_number] = "system_number",
  [sym_button] = "button",
  [sym_dv_channel] = "dv_channel",
  [sym_data] = "data",
  [sym_level] = "level",
  [sym_timeline] = "timeline",
  [sym_channel] = "channel",
  [sym_custom] = "custom",
  [sym_atoi] = "atoi",
  [sym_atol] = "atol",
  [sym_atof] = "atof",
  [sym_itoa] = "itoa",
  [sym_itohex] = "itohex",
  [sym_hextoi] = "hextoi",
  [sym_ftoa] = "ftoa",
  [sym_find_string] = "find_string",
  [sym_left_string] = "left_string",
  [sym_right_string] = "right_string",
  [sym_mid_string] = "mid_string",
  [sym_remove_string] = "remove_string",
  [sym_get_buffer_string] = "get_buffer_string",
  [sym_get_buffer_char] = "get_buffer_char",
  [sym_get_multi_buffer_string] = "get_multi_buffer_string",
  [sym_length_string] = "length_string",
  [sym_max_length_string] = "max_length_string",
  [sym_set_length_string] = "set_length_string",
  [sym_compare_string] = "compare_string",
  [sym_upper_string] = "upper_string",
  [sym_lower_string] = "lower_string",
  [sym_get_last] = "get_last",
  [sym_date_to_day] = "date_to_day",
  [sym_date_to_month] = "date_to_month",
  [sym_date_to_year] = "date_to_year",
  [sym_time_to_hour] = "time_to_hour",
  [sym_time_to_minute] = "time_to_minute",
  [sym_time_to_second] = "time_to_second",
  [sym_day_of_week] = "day_of_week",
  [sym_set_pulse_time] = "set_pulse_time",
  [sym_set_timer] = "set_timer",
  [sym_file_open] = "file_open",
  [sym_file_close] = "file_close",
  [sym_file_read] = "file_read",
  [sym_file_read_line] = "file_read_line",
  [sym_file_write] = "file_write",
  [sym_file_write_line] = "file_write_line",
  [sym_file_copy] = "file_copy",
  [sym_file_delete] = "file_delete",
  [sym_file_rename] = "file_rename",
  [sym_file_dir] = "file_dir",
  [sym_file_getdir] = "file_getdir",
  [sym_file_setdir] = "file_setdir",
  [sym_file_create_dir] = "file_create_dir",
  [sym_file_remove_dir] = "file_remove_dir",
  [sym_file_seek] = "file_seek",
  [sym_length_array] = "length_array",
  [sym_max_length_array] = "max_length_array",
  [sym_set_length_array] = "set_length_array",
  [sym_ip_client_open] = "ip_client_open",
  [sym_ip_client_close] = "ip_client_close",
  [sym_ip_server_open] = "ip_server_open",
  [sym_ip_server_close] = "ip_server_close",
  [sym_ip_bound_client_open] = "ip_bound_client_open",
  [sym_ip_mc_server_open] = "ip_mc_server_open",
  [sym_ip_set_option] = "ip_set_option",
  [sym_ssh_client_open] = "ssh_client_open",
  [sym_ssh_client_close] = "ssh_client_close",
  [sym_tls_client_open] = "tls_client_open",
  [sym_tls_client_close] = "tls_client_close",
  [sym_do_push] = "do_push",
  [sym_do_push_timed] = "do_push_timed",
  [sym_do_release] = "do_release",
  [sym_device_id] = "device_id",
  [sym_device_info] = "device_info",
  [sym_device_id_string] = "device_id_string",
  [sym_device_standby] = "device_standby",
  [sym_device_wake] = "device_wake",
  [sym_timeline_create] = "timeline_create",
  [sym_timeline_kill] = "timeline_kill",
  [sym_timeline_reload] = "timeline_reload",
  [sym_timeline_pause] = "timeline_pause",
  [sym_timeline_restart] = "timeline_restart",
  [sym_timeline_set] = "timeline_set",
  [sym_timeline_get] = "timeline_get",
  [sym_timeline_active] = "timeline_active",
  [sym_timeline_dynamic_id] = "timeline_dynamic_id",
  [sym_variable_to_string] = "variable_to_string",
  [sym_string_to_variable] = "string_to_variable",
  [sym_length_variable_to_string] = "length_variable_to_string",
  [sym_type_cast] = "type_cast",
  [sym_variable_to_xml] = "variable_to_xml",
  [sym_xml_to_variable] = "xml_to_variable",
  [sym_length_variable_to_xml] = "length_variable_to_xml",
  [sym_redirect_string] = "redirect_string",
  [sym_random_number] = "random_number",
  [sym_get_unique_id] = "get_unique_id",
  [sym_get_system_number] = "get_system_number",
  [sym_set_system_number] = "set_system_number",
  [sym_get_serial_number] = "get_serial_number",
  [sym_get_master_build] = "get_master_build",
  [sym_set_validation_code] = "set_validation_code",
  [sym_reboot] = "reboot",
  [sym_load_duet_module] = "load_duet_module",
  [sym_unload_duet_module] = "unload_duet_module",
  [sym_duet_mem_size_set] = "duet_mem_size_set",
  [sym_duet_mem_size_get] = "duet_mem_size_get",
  [sym_dynamic_application_device] = "dynamic_application_device",
  [sym_dynamic_polled_port] = "dynamic_polled_port",
  [sym_static_port_binding] = "static_port_binding",
  [sym_static_ip_binding] = "static_ip_binding",
  [sym_get_dns_list] = "get_dns_list",
  [sym_set_dns_list] = "set_dns_list",
  [sym_get_ip_address] = "get_ip_address",
  [sym_set_ip_address] = "set_ip_address",
  [sym_get_url_list] = "get_url_list",
  [sym_add_url_entry] = "add_url_entry",
  [sym_delete_url_entry] = "delete_url_entry",
  [sym_abs_value] = "abs_value",
  [sym_max_value] = "max_value",
  [sym_min_value] = "min_value",
  [sym_exp_value] = "exp_value",
  [sym_log_value] = "log_value",
  [sym_log10_value] = "log10_value",
  [sym_power_value] = "power_value",
  [sym_sqrt_value] = "sqrt_value",
  [sym_raw_be] = "raw_be",
  [sym_raw_le] = "raw_le",
  [sym_internal_queue_size_set] = "internal_queue_size_set",
  [sym_internal_queue_size_get] = "internal_queue_size_get",
  [sym_internal_threshold_set] = "internal_threshold_set",
  [sym_internal_threshold_get] = "internal_threshold_get",
  [sym_rebuilt_event] = "rebuilt_event",
  [sym_do_custom_event] = "do_custom_event",
  [sym_clkmgr_is_network_sourced] = "clkmgr_is_network_sourced",
  [sym_clkmgr_set_clk_source] = "clkmgr_set_clk_source",
  [sym_clkmgr_is_daylightsavings_on] = "clkmgr_is_daylightsavings_on",
  [sym_clkmgr_set_daylightsavings_mode] = "clkmgr_set_daylightsavings_mode",
  [sym_clkmgr_get_timezone] = "clkmgr_get_timezone",
  [sym_clkmgr_set_timezone] = "clkmgr_set_timezone",
  [sym_clkmgr_get_resync_period] = "clkmgr_get_resync_period",
  [sym_clkmgr_set_resync_period] = "clkmgr_set_resync_period",
  [sym_clkmgr_get_daylightsavings_offset] = "clkmgr_get_daylightsavings_offset",
  [sym_clkmgr_set_daylightsavings_offset] = "clkmgr_set_daylightsavings_offset",
  [sym_clkmgr_get_active_timeserver] = "clkmgr_get_active_timeserver",
  [sym_clkmgr_set_active_timeserver] = "clkmgr_set_active_timeserver",
  [sym_clkmgr_get_timeservers] = "clkmgr_get_timeservers",
  [sym_clkmgr_add_userdefined_timeserver] = "clkmgr_add_userdefined_timeserver",
  [sym_clkmgr_delete_userdefined_timeserver] = "clkmgr_delete_userdefined_timeserver",
  [sym_clkmgr_get_start_daylightsavings_rule] = "clkmgr_get_start_daylightsavings_rule",
  [sym_clkmgr_set_start_daylightsavings_rule] = "clkmgr_set_start_daylightsavings_rule",
  [sym_clkmgr_get_end_daylightsavings_rule] = "clkmgr_get_end_daylightsavings_rule",
  [sym_clkmgr_set_end_daylightsavings_rule] = "clkmgr_set_end_daylightsavings_rule",
  [sym_set_log_level] = "set_log_level",
  [sym_get_log_level] = "get_log_level",
  [sym_amx_log] = "amx_log",
  [sym_smtp_server_config_set] = "smtp_server_config_set",
  [sym_smtp_server_config_get] = "smtp_server_config_get",
  [sym_smtp_send] = "smtp_send",
  [sym_validate_netlinx_account] = "validate_netlinx_account",
  [sym_validate_netlinx_account_with_permission] = "validate_netlinx_account_with_permission",
  [sym_authenticate_certificate] = "authenticate_certificate",
  [sym_audit_netlinx_session_event] = "audit_netlinx_session_event",
  [sym_audit_netlinx_generic_event] = "audit_netlinx_generic_event",
  [sym_get_available_flash_disk_space] = "get_available_flash_disk_space",
  [sym_get_max_flash_disk_space] = "get_max_flash_disk_space",
  [sym_data_create_feed] = "data_create_feed",
  [sym_data_delete_feed] = "data_delete_feed",
  [sym_data_publish_feed] = "data_publish_feed",
  [sym_data_get_published_feed] = "data_get_published_feed",
  [sym_data_add_record] = "data_add_record",
  [sym_data_get_event_record] = "data_get_event_record",
  [sym__wc_data_create_feed] = "_wc_data_create_feed",
  [sym__wc_data_add_record] = "_wc_data_add_record",
  [sym__wc_data_get_event_record] = "_wc_data_get_event_record",
  [sym_format] = "format",
  [sym_astro_clock] = "astro_clock",
  [sym_set_outdoor_temperature] = "set_outdoor_temperature",
  [sym_set_virtual_level_count] = "set_virtual_level_count",
  [sym_set_virtual_channel_count] = "set_virtual_channel_count",
  [sym_set_virtual_port_count] = "set_virtual_port_count",
  [sym__wc_encode_int] = "_wc_encode_int",
  [sym__wc_decode_int] = "_wc_decode_int",
  [sym__wc_int] = "_wc_int",
  [sym__wc_to_ch_int] = "_wc_to_ch_int",
  [sym__ch_to_wc_int] = "_ch_to_wc_int",
  [sym__wc_find_string_int] = "_wc_find_string_int",
  [sym__wc_left_string_int] = "_wc_left_string_int",
  [sym__wc_lower_string_int] = "_wc_lower_string_int",
  [sym__wc_mid_string_int] = "_wc_mid_string_int",
  [sym__wc_remove_string_int] = "_wc_remove_string_int",
  [sym__wc_right_string_int] = "_wc_right_string_int",
  [sym__wc_upper_string_int] = "_wc_upper_string_int",
  [sym__wc_compare_string_int] = "_wc_compare_string_int",
  [sym__wc_get_buffer_char_int] = "_wc_get_buffer_char_int",
  [sym__wc_get_buffer_string_int] = "_wc_get_buffer_string_int",
  [sym__wc_concat_string_int] = "_wc_concat_string_int",
  [sym__wc_file_open_int] = "_wc_file_open_int",
  [sym__wc_file_close_int] = "_wc_file_close_int",
  [sym__wc_file_read_int] = "_wc_file_read_int",
  [sym__wc_file_read_line_int] = "_wc_file_read_line_int",
  [sym__wc_file_write_int] = "_wc_file_write_int",
  [sym__wc_file_write_line_int] = "_wc_file_write_line_int",
  [anon_sym_EQ] = "=",
  [aux_sym_preproc_include_token1] = "preproc_include_keyword",
  [aux_sym_preproc_define_token1] = "preproc_define_keyword",
  [aux_sym_preproc_warn_token1] = "preproc_warn_keyword",
  [aux_sym_preproc_disable_warning_token1] = "preproc_disable_warning_keyword",
  [aux_sym_preproc_if_defined_token1] = "preproc_if_defined_keyword",
  [anon_sym_LF] = "\n",
  [aux_sym_preproc_if_defined_token2] = "preproc_end_if_keyword",
  [aux_sym_preproc_if_not_defined_token1] = "preproc_if_not_defined_keyword",
  [aux_sym_preproc_else_token1] = "preproc_else_keyword",
  [anon_sym_COMMA] = ",",
  [sym_preproc_arg] = "preproc_arg",
  [anon_sym_LBRACK] = "[",
  [anon_sym_RBRACK] = "]",
  [anon_sym_LBRACE] = "{",
  [anon_sym_RBRACE] = "}",
  [anon_sym_COLON] = ":",
  [anon_sym_LPAREN] = "(",
  [anon_sym_RPAREN] = ")",
  [anon_sym_SEMI] = ";",
  [anon_sym_BANG] = "!",
  [anon_sym_TILDE] = "~",
  [anon_sym_DASH] = "-",
  [anon_sym_PLUS] = "+",
  [anon_sym_STAR] = "*",
  [anon_sym_SLASH] = "/",
  [anon_sym_PERCENT] = "%",
  [anon_sym_PIPE_PIPE] = "||",
  [anon_sym_AMP_AMP] = "&&",
  [anon_sym_PIPE] = "|",
  [anon_sym_CARET] = "^",
  [anon_sym_AMP] = "&",
  [anon_sym_EQ_EQ] = "==",
  [anon_sym_BANG_EQ] = "!=",
  [anon_sym_LT_GT] = "<>",
  [anon_sym_GT] = ">",
  [anon_sym_GT_EQ] = ">=",
  [anon_sym_LT_EQ] = "<=",
  [anon_sym_LT] = "<",
  [anon_sym_LT_LT] = "<<",
  [anon_sym_GT_GT] = ">>",
  [anon_sym_DASH_DASH] = "--",
  [anon_sym_PLUS_PLUS] = "++",
  [anon_sym_DOT] = ".",
  [sym_range_operator] = "range_operator",
  [anon_sym_DQUOTE] = "\"",
  [anon_sym_SQUOTE] = "'",
  [aux_sym_string_literal_token1] = "string_content",
  [sym_escape_sequence] = "escape_sequence",
  [sym_number_literal] = "number_literal",
  [sym_comment] = "comment",
  [sym__automatic_semicolon] = "_automatic_semicolon",
  [sym_source_file] = "source_file",
  [sym__top_level_item] = "_top_level_item",
  [sym__block_item] = "_block_item",
  [sym_on_keyword] = "on_keyword",
  [sym_off_keyword] = "off_keyword",
  [sym_devchan_on_keyword] = "devchan_on_keyword",
  [sym_devchan_off_keyword] = "devchan_off_keyword",
  [sym_program_name] = "program_name",
  [sym_module_name] = "module_name",
  [sym_preproc_include] = "preproc_include",
  [sym_preproc_define] = "preproc_define",
  [sym_preproc_warn] = "preproc_warn",
  [sym_preproc_disable_warning] = "preproc_disable_warning",
  [sym_preproc_if_defined] = "preproc_if_defined",
  [sym_preproc_if_not_defined] = "preproc_if_not_defined",
  [sym_preproc_else] = "preproc_else",
  [sym_preproc_if_defined_in_field_declaration_list] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_field_declaration_list] = "preproc_if_not_defined",
  [sym_preproc_else_in_field_declaration_list] = "preproc_else",
  [sym_preproc_if_defined_in_initializer_list] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_initializer_list] = "preproc_if_not_defined",
  [sym_preproc_else_in_initializer_list] = "preproc_else",
  [sym_preproc_if_defined_in_initializer_list_no_comma] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_initializer_list_no_comma] = "preproc_if_not_defined",
  [sym_preproc_else_in_initializer_list_no_comma] = "preproc_else",
  [sym_preproc_if_defined_in_button_event_declarator] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_button_event_declarator] = "preproc_if_not_defined",
  [sym_preproc_else_in_button_event_declarator] = "preproc_else",
  [sym_preproc_if_defined_in_button_event_block] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_button_event_block] = "preproc_if_not_defined",
  [sym_preproc_else_in_button_event_block] = "preproc_else",
  [sym_preproc_if_defined_in_data_event_declarator] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_data_event_declarator] = "preproc_if_not_defined",
  [sym_preproc_else_in_data_event_declarator] = "preproc_else",
  [sym_preproc_if_defined_in_data_event_block] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_data_event_block] = "preproc_if_not_defined",
  [sym_preproc_else_in_data_event_block] = "preproc_else",
  [sym_preproc_if_defined_in_channel_event_declarator] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_channel_event_declarator] = "preproc_if_not_defined",
  [sym_preproc_else_in_channel_event_declarator] = "preproc_else",
  [sym_preproc_if_defined_in_channel_event_block] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_channel_event_block] = "preproc_if_not_defined",
  [sym_preproc_else_in_channel_event_block] = "preproc_else",
  [sym_preproc_if_defined_in_timeline_event_declarator] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_timeline_event_declarator] = "preproc_if_not_defined",
  [sym_preproc_else_in_timeline_event_declarator] = "preproc_else",
  [sym_preproc_if_defined_in_level_event_declarator] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_level_event_declarator] = "preproc_if_not_defined",
  [sym_preproc_else_in_level_event_declarator] = "preproc_else",
  [sym_preproc_if_defined_in_custom_event_declarator] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_custom_event_declarator] = "preproc_if_not_defined",
  [sym_preproc_else_in_custom_event_declarator] = "preproc_else",
  [sym_preproc_if_defined_in_push_event_declarator] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_push_event_declarator] = "preproc_if_not_defined",
  [sym_preproc_else_in_push_event_declarator] = "preproc_else",
  [sym_preproc_if_defined_in_release_event_declarator] = "preproc_if_defined",
  [sym_preproc_if_not_defined_in_release_event_declarator] = "preproc_if_not_defined",
  [sym_preproc_else_in_release_event_declarator] = "preproc_else",
  [sym_section] = "section",
  [sym_define_device_section] = "define_device_section",
  [sym_define_combine_section] = "define_combine_section",
  [sym_define_connect_level_section] = "define_connect_level_section",
  [sym_define_constant_section] = "define_constant_section",
  [sym_define_type_section] = "define_type_section",
  [sym_define_mutually_exclusive_section] = "define_mutually_exclusive_section",
  [sym_define_latching_section] = "define_latching_section",
  [sym_define_toggling_section] = "define_toggling_section",
  [sym_define_variable_section] = "define_variable_section",
  [sym_define_system_variable_section] = "define_system_variable_section",
  [sym_define_event_section] = "define_event_section",
  [sym_define_start_section] = "define_start_section",
  [sym_define_program_section] = "define_program_section",
  [sym_define_function] = "define_function",
  [sym_define_call] = "define_call",
  [sym_define_module] = "define_module",
  [sym_module_definition] = "module_definition",
  [sym_event_definition] = "event_definition",
  [sym_data_event_definition] = "data_event_definition",
  [sym__data_event_declarator_list] = "_data_event_declarator_list",
  [sym_data_event_declarator] = "data_event_declarator",
  [sym_data_event_device_reference] = "data_event_device_reference",
  [sym_data_event_block] = "data_event_block",
  [sym__data_event_block_item] = "_data_event_block_item",
  [sym_data_event_handler] = "data_event_handler",
  [sym_data_event_type] = "data_event_type",
  [sym_timeline_event_definition] = "timeline_event_definition",
  [sym__timeline_event_declarator_list] = "_timeline_event_declarator_list",
  [sym_timeline_event_declarator] = "timeline_event_declarator",
  [sym_timeline_event_id_reference] = "timeline_event_id_reference",
  [sym_button_event_definition] = "button_event_definition",
  [sym__button_event_declarator_list] = "_button_event_declarator_list",
  [sym_button_event_declarator] = "button_event_declarator",
  [sym_button_event_devchan_reference] = "button_event_devchan_reference",
  [sym_button_event_block] = "button_event_block",
  [sym__button_event_block_item] = "_button_event_block_item",
  [sym_button_event_handler] = "button_event_handler",
  [sym_button_event_type] = "button_event_type",
  [sym_button_event_hold_repeat] = "button_event_hold_repeat",
  [sym_level_event_definition] = "level_event_definition",
  [sym__level_event_declarator_list] = "_level_event_declarator_list",
  [sym_level_event_declarator] = "level_event_declarator",
  [sym_level_event_devlev_reference] = "level_event_devlev_reference",
  [sym_channel_event_definition] = "channel_event_definition",
  [sym__channel_event_declarator_list] = "_channel_event_declarator_list",
  [sym_channel_event_declarator] = "channel_event_declarator",
  [sym_channel_event_devchan_reference] = "channel_event_devchan_reference",
  [sym_channel_event_block] = "channel_event_block",
  [sym__channel_event_block_item] = "_channel_event_block_item",
  [sym_channel_event_handler] = "channel_event_handler",
  [sym_channel_event_type] = "channel_event_type",
  [sym_custom_event_definition] = "custom_event_definition",
  [sym__custom_event_declarator_list] = "_custom_event_declarator_list",
  [sym_custom_event_declarator] = "custom_event_declarator",
  [sym_custom_event_reference] = "custom_event_reference",
  [sym_push_event_definition] = "push_event_definition",
  [sym__push_event_declarator_list] = "_push_event_declarator_list",
  [sym_push_event_declarator] = "push_event_declarator",
  [sym_push_event_devchan_reference] = "push_event_devchan_reference",
  [sym_release_event_definition] = "release_event_definition",
  [sym__release_event_declarator_list] = "_release_event_declarator_list",
  [sym_release_event_declarator] = "release_event_declarator",
  [sym_release_event_devchan_reference] = "release_event_devchan_reference",
  [sym_array_declarator] = "array_declarator",
  [sym_array_field_declarator] = "array_declarator",
  [sym_abstract_array_declarator] = "abstract_array_declarator",
  [sym_init_declarator] = "init_declarator",
  [sym_compound_statement] = "compound_statement",
  [sym_storage_class_specifier] = "storage_class_specifier",
  [sym_type_qualifier] = "type_qualifier",
  [sym_type_specifier] = "type_specifier",
  [sym_struct_specifier] = "struct_specifier",
  [sym_field_declaration_list] = "field_declaration_list",
  [sym__field_declaration_list_item] = "_field_declaration_list_item",
  [sym_field_declaration] = "field_declaration",
  [sym__field_declaration_declarator] = "_field_declaration_declarator",
  [sym_intrinsic_type] = "intrinsic_type",
  [sym_primitive_type] = "primitive_type",
  [sym_structured_type] = "structured_type",
  [sym_array_return_type] = "array_return_type",
  [sym_function_definition] = "function_definition",
  [sym_function_declaration] = "function_declaration",
  [sym_call_definition] = "call_definition",
  [sym_declaration] = "declaration",
  [sym_type_definition] = "type_definition",
  [sym__declaration_modifiers] = "_declaration_modifiers",
  [sym__declaration_specifiers] = "_declaration_specifiers",
  [sym__declarator] = "_declarator",
  [sym__declaration_declarator] = "_declaration_declarator",
  [sym__field_declarator] = "_field_declarator",
  [sym__abstract_declarator] = "_abstract_declarator",
  [sym_function_declarator] = "function_declarator",
  [sym__function_declaration_declarator] = "function_declarator",
  [sym_parenthesized_declarator] = "parenthesized_declarator",
  [sym_parenthesized_field_declarator] = "parenthesized_declarator",
  [sym_function_field_declarator] = "function_declarator",
  [sym_parameter_list] = "parameter_list",
  [sym_parameter_declaration] = "parameter_declaration",
  [sym_statement] = "statement",
  [sym__top_level_statement] = "_top_level_statement",
  [sym_expression_statement] = "expression_statement",
  [sym_if_statement] = "if_statement",
  [sym_else_clause] = "else_clause",
  [sym_switch_statement] = "switch_statement",
  [sym_case_statement] = "case_statement",
  [sym_while_statement] = "while_statement",
  [sym_for_statement] = "for_statement",
  [sym__for_statement_body] = "_for_statement_body",
  [sym_select_statement] = "select_statement",
  [sym_active_statement] = "active_statement",
  [sym_return_statement] = "return_statement",
  [sym_break_statement] = "break_statement",
  [sym_continue_statement] = "continue_statement",
  [sym_devchan_operation_statement] = "devchan_operation_statement",
  [sym_devchan_operation] = "devchan_operation",
  [sym_send_string_statement] = "send_string_statement",
  [sym_send_command_statement] = "send_command_statement",
  [sym_send_level_statement] = "send_level_statement",
  [sym_create_buffer_statement] = "create_buffer_statement",
  [sym_create_multi_buffer_statement] = "create_multi_buffer_statement",
  [sym_clear_buffer_statement] = "clear_buffer_statement",
  [sym_wait_statement] = "wait_statement",
  [sym_wait_until_statement] = "wait_until_statement",
  [sym_cancel_all_wait_statement] = "cancel_all_wait_statement",
  [sym_cancel_all_wait_until_statement] = "cancel_all_wait_until_statement",
  [sym_cancel_wait_statement] = "cancel_wait_statement",
  [sym_cancel_wait_until_statement] = "cancel_wait_until_statement",
  [sym_call_statement] = "call_statement",
  [sym_system_call_statement] = "system_call_statement",
  [sym_expression] = "expression",
  [sym_compiler_variable] = "compiler_variable",
  [sym_system_variable] = "system_variable",
  [sym_system_constant] = "system_constant",
  [sym_system_function] = "system_function",
  [sym_system_type] = "system_type",
  [sym_assignment_expression] = "assignment_expression",
  [sym_unary_expression] = "unary_expression",
  [sym_binary_expression] = "binary_expression",
  [sym_update_expression] = "update_expression",
  [sym_subscript_expression] = "subscript_expression",
  [sym_call_expression] = "call_expression",
  [sym_comma_expression] = "comma_expression",
  [sym_argument_list] = "argument_list",
  [sym_field_expression] = "field_expression",
  [sym_parenthesized_expression] = "parenthesized_expression",
  [sym_devlev_expression] = "devlev_expression",
  [sym_devchan_expression] = "devchan_expression",
  [sym_devchan_range_expression] = "devchan_range_expression",
  [sym_string_expression] = "string_expression",
  [sym_initializer_list] = "initializer_list",
  [sym_device_literal] = "device_literal",
  [sym_string_literal] = "string_literal",
  [aux_sym_source_file_repeat1] = "source_file_repeat1",
  [aux_sym_preproc_if_defined_in_field_declaration_list_repeat1] = "preproc_if_defined_in_field_declaration_list_repeat1",
  [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = "preproc_if_defined_in_initializer_list_repeat1",
  [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = "preproc_if_defined_in_initializer_list_no_comma_repeat1",
  [aux_sym_preproc_if_defined_in_button_event_declarator_repeat1] = "preproc_if_defined_in_button_event_declarator_repeat1",
  [aux_sym_preproc_if_defined_in_button_event_declarator_repeat2] = "preproc_if_defined_in_button_event_declarator_repeat2",
  [aux_sym_preproc_if_defined_in_button_event_block_repeat1] = "preproc_if_defined_in_button_event_block_repeat1",
  [aux_sym_preproc_if_defined_in_button_event_block_repeat2] = "preproc_if_defined_in_button_event_block_repeat2",
  [aux_sym_preproc_if_defined_in_data_event_declarator_repeat1] = "preproc_if_defined_in_data_event_declarator_repeat1",
  [aux_sym_preproc_if_defined_in_data_event_declarator_repeat2] = "preproc_if_defined_in_data_event_declarator_repeat2",
  [aux_sym_preproc_if_defined_in_data_event_block_repeat1] = "preproc_if_defined_in_data_event_block_repeat1",
  [aux_sym_preproc_if_defined_in_data_event_block_repeat2] = "preproc_if_defined_in_data_event_block_repeat2",
  [aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1] = "preproc_if_defined_in_channel_event_declarator_repeat1",
  [aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2] = "preproc_if_defined_in_channel_event_declarator_repeat2",
  [aux_sym_preproc_if_defined_in_channel_event_block_repeat1] = "preproc_if_defined_in_channel_event_block_repeat1",
  [aux_sym_preproc_if_defined_in_channel_event_block_repeat2] = "preproc_if_defined_in_channel_event_block_repeat2",
  [aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1] = "preproc_if_defined_in_timeline_event_declarator_repeat1",
  [aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2] = "preproc_if_defined_in_timeline_event_declarator_repeat2",
  [aux_sym_preproc_if_defined_in_level_event_declarator_repeat1] = "preproc_if_defined_in_level_event_declarator_repeat1",
  [aux_sym_preproc_if_defined_in_level_event_declarator_repeat2] = "preproc_if_defined_in_level_event_declarator_repeat2",
  [aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1] = "preproc_if_defined_in_custom_event_declarator_repeat1",
  [aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2] = "preproc_if_defined_in_custom_event_declarator_repeat2",
  [aux_sym_preproc_if_defined_in_push_event_declarator_repeat1] = "preproc_if_defined_in_push_event_declarator_repeat1",
  [aux_sym_preproc_if_defined_in_push_event_declarator_repeat2] = "preproc_if_defined_in_push_event_declarator_repeat2",
  [aux_sym_preproc_if_defined_in_release_event_declarator_repeat1] = "preproc_if_defined_in_release_event_declarator_repeat1",
  [aux_sym_preproc_if_defined_in_release_event_declarator_repeat2] = "preproc_if_defined_in_release_event_declarator_repeat2",
  [aux_sym__data_event_declarator_list_repeat1] = "_data_event_declarator_list_repeat1",
  [aux_sym__timeline_event_declarator_list_repeat1] = "_timeline_event_declarator_list_repeat1",
  [aux_sym__button_event_declarator_list_repeat1] = "_button_event_declarator_list_repeat1",
  [aux_sym__level_event_declarator_list_repeat1] = "_level_event_declarator_list_repeat1",
  [aux_sym__channel_event_declarator_list_repeat1] = "_channel_event_declarator_list_repeat1",
  [aux_sym__custom_event_declarator_list_repeat1] = "_custom_event_declarator_list_repeat1",
  [aux_sym__push_event_declarator_list_repeat1] = "_push_event_declarator_list_repeat1",
  [aux_sym__release_event_declarator_list_repeat1] = "_release_event_declarator_list_repeat1",
  [aux_sym_compound_statement_repeat1] = "compound_statement_repeat1",
  [aux_sym__field_declaration_declarator_repeat1] = "_field_declaration_declarator_repeat1",
  [aux_sym_function_definition_repeat1] = "function_definition_repeat1",
  [aux_sym_declaration_repeat1] = "declaration_repeat1",
  [aux_sym__declaration_specifiers_repeat1] = "_declaration_specifiers_repeat1",
  [aux_sym_parameter_list_repeat1] = "parameter_list_repeat1",
  [aux_sym_case_statement_repeat1] = "case_statement_repeat1",
  [aux_sym_subscript_expression_repeat1] = "subscript_expression_repeat1",
  [aux_sym_argument_list_repeat1] = "argument_list_repeat1",
  [aux_sym_string_expression_repeat1] = "string_expression_repeat1",
  [aux_sym_initializer_list_repeat1] = "initializer_list_repeat1",
  [aux_sym_string_literal_repeat1] = "string_literal_repeat1",
  [alias_sym_field_identifier] = "field_identifier",
  [alias_sym_type_identifier] = "type_identifier",
};

static const TSSymbol ts_symbol_map[] = {
  [ts_builtin_sym_end] = ts_builtin_sym_end,
  [sym_identifier] = sym_identifier,
  [sym_program_name_keyword] = sym_program_name_keyword,
  [sym_module_name_keyword] = sym_module_name_keyword,
  [sym_define_device_keyword] = sym_define_device_keyword,
  [sym_define_combine_keyword] = sym_define_combine_keyword,
  [sym_define_constant_keyword] = sym_define_constant_keyword,
  [sym_define_type_keyword] = sym_define_type_keyword,
  [sym_define_variable_keyword] = sym_define_variable_keyword,
  [sym_define_system_variable_keyword] = sym_define_system_variable_keyword,
  [sym_define_connect_level_keyword] = sym_define_connect_level_keyword,
  [sym_define_latching_keyword] = sym_define_latching_keyword,
  [sym_define_mutually_exclusive_keyword] = sym_define_mutually_exclusive_keyword,
  [sym_define_toggling_keyword] = sym_define_toggling_keyword,
  [sym_define_call_keyword] = sym_define_call_keyword,
  [sym_define_function_keyword] = sym_define_function_keyword,
  [sym_define_library_function_keyword] = sym_define_library_function_keyword,
  [sym_define_start_keyword] = sym_define_start_keyword,
  [sym_define_event_keyword] = sym_define_event_keyword,
  [sym_define_program_keyword] = sym_define_program_keyword,
  [sym_define_module_keyword] = sym_define_module_keyword,
  [sym_char_keyword] = sym_char_keyword,
  [sym_widechar_keyword] = sym_widechar_keyword,
  [sym_integer_keyword] = sym_integer_keyword,
  [sym_sinteger_keyword] = sym_sinteger_keyword,
  [sym_long_keyword] = sym_long_keyword,
  [sym_slong_keyword] = sym_slong_keyword,
  [sym_float_keyword] = sym_float_keyword,
  [sym_double_keyword] = sym_double_keyword,
  [sym_variant_keyword] = sym_variant_keyword,
  [sym_variantarray_keyword] = sym_variantarray_keyword,
  [sym_local_var_keyword] = sym_local_var_keyword,
  [sym_stack_var_keyword] = sym_stack_var_keyword,
  [sym_constant_keyword] = sym_constant_keyword,
  [sym_volatile_keyword] = sym_volatile_keyword,
  [sym_non_volatile_keyword] = sym_non_volatile_keyword,
  [sym_persistent_keyword] = sym_persistent_keyword,
  [sym_dev_keyword] = sym_dev_keyword,
  [sym_devlev_keyword] = sym_devlev_keyword,
  [sym_devchan_keyword] = sym_devchan_keyword,
  [sym_if_keyword] = sym_if_keyword,
  [sym_else_keyword] = sym_else_keyword,
  [sym_switch_keyword] = sym_switch_keyword,
  [sym_case_keyword] = sym_case_keyword,
  [sym_default_keyword] = sym_default_keyword,
  [sym_while_keyword] = sym_while_keyword,
  [sym_for_keyword] = sym_for_keyword,
  [sym_break_keyword] = sym_break_keyword,
  [sym_continue_keyword] = sym_continue_keyword,
  [sym_return_keyword] = sym_return_keyword,
  [sym_select_keyword] = sym_select_keyword,
  [sym_active_keyword] = sym_active_keyword,
  [sym_struct_keyword] = sym_struct_keyword,
  [sym_structure_keyword] = sym_structure_keyword,
  [sym_button_event_keyword] = sym_button_event_keyword,
  [sym_channel_event_keyword] = sym_channel_event_keyword,
  [sym_level_event_keyword] = sym_level_event_keyword,
  [sym_data_event_keyword] = sym_data_event_keyword,
  [sym_timeline_event_keyword] = sym_timeline_event_keyword,
  [sym_custom_event_keyword] = sym_custom_event_keyword,
  [sym_push_keyword] = sym_push_keyword,
  [sym_release_keyword] = sym_release_keyword,
  [sym_hold_keyword] = sym_hold_keyword,
  [sym_repeat_keyword] = sym_repeat_keyword,
  [aux_sym_on_keyword_token1] = aux_sym_on_keyword_token1,
  [aux_sym_off_keyword_token1] = aux_sym_off_keyword_token1,
  [sym_online_keyword] = sym_online_keyword,
  [sym_offline_keyword] = sym_offline_keyword,
  [sym_onerror_keyword] = sym_onerror_keyword,
  [sym_string_keyword] = sym_string_keyword,
  [sym_command_keyword] = sym_command_keyword,
  [sym_standby_keyword] = sym_standby_keyword,
  [sym_awake_keyword] = sym_awake_keyword,
  [sym_send_level_keyword] = sym_send_level_keyword,
  [sym_send_string_keyword] = sym_send_string_keyword,
  [sym_send_command_keyword] = sym_send_command_keyword,
  [sym_clear_buffer_keyword] = sym_clear_buffer_keyword,
  [sym_create_buffer_keyword] = sym_create_buffer_keyword,
  [sym_create_multi_buffer_keyword] = sym_create_multi_buffer_keyword,
  [sym_call_keyword] = sym_call_keyword,
  [sym_system_call_keyword] = sym_system_call_keyword,
  [sym_devchan_to_keyword] = sym_devchan_to_keyword,
  [sym_devchan_min_to_keyword] = sym_devchan_min_to_keyword,
  [sym_devchan_total_off_keyword] = sym_devchan_total_off_keyword,
  [sym_devchan_pulse_keyword] = sym_devchan_pulse_keyword,
  [sym_wait_keyword] = sym_wait_keyword,
  [sym_cancel_wait_keyword] = sym_cancel_wait_keyword,
  [sym_wait_until_keyword] = sym_wait_until_keyword,
  [sym_cancel_wait_until_keyword] = sym_cancel_wait_until_keyword,
  [sym_cancel_all_wait_keyword] = sym_cancel_all_wait_keyword,
  [sym_cancel_all_wait_until_keyword] = sym_cancel_all_wait_until_keyword,
  [sym_and] = sym_and,
  [sym_or] = sym_or,
  [sym_not] = sym_not,
  [sym_xor] = sym_xor,
  [sym_band] = sym_band,
  [sym_bor] = sym_bor,
  [sym_bxor] = sym_bxor,
  [sym_bnot] = sym_bnot,
  [sym_lshift] = sym_lshift,
  [sym_rshift] = sym_rshift,
  [sym___file__] = sym___file__,
  [sym___date__] = sym___date__,
  [sym___ldate__] = sym___ldate__,
  [sym___time__] = sym___time__,
  [sym___line__] = sym___line__,
  [sym___name__] = sym___name__,
  [sym___netlinx__] = sym___netlinx__,
  [sym_netlinx_axi_version] = sym_netlinx_axi_version,
  [sym_url_flg_tcp] = sym_url_flg_tcp,
  [sym_url_flg_accinfopresent] = sym_url_flg_accinfopresent,
  [sym_url_flg_temp] = sym_url_flg_temp,
  [sym_url_flg_stat_prgnetlinx] = sym_url_flg_stat_prgnetlinx,
  [sym_url_flg_stat_mask] = sym_url_flg_stat_mask,
  [sym_url_flg_stat_lookup] = sym_url_flg_stat_lookup,
  [sym_url_flg_stat_connecting] = sym_url_flg_stat_connecting,
  [sym_url_flg_stat_waiting] = sym_url_flg_stat_waiting,
  [sym_url_flg_stat_connected] = sym_url_flg_stat_connected,
  [sym_ip_addr_flg_dhcp] = sym_ip_addr_flg_dhcp,
  [sym_first_virtual_device] = sym_first_virtual_device,
  [sym_first_local_port] = sym_first_local_port,
  [sym_dynamic_virtual_device] = sym_dynamic_virtual_device,
  [sym_timeline_once] = sym_timeline_once,
  [sym_timeline_repeat] = sym_timeline_repeat,
  [sym_timeline_absolute] = sym_timeline_absolute,
  [sym_timeline_relative] = sym_timeline_relative,
  [sym_do_push_timed_infinite] = sym_do_push_timed_infinite,
  [sym_source_type_no_address] = sym_source_type_no_address,
  [sym_source_type_neuron_id] = sym_source_type_neuron_id,
  [sym_source_type_ip_address] = sym_source_type_ip_address,
  [sym_source_type_axlink] = sym_source_type_axlink,
  [sym_source_type_neuron_subnode_icsp] = sym_source_type_neuron_subnode_icsp,
  [sym_source_type_neuron_subnode_pl] = sym_source_type_neuron_subnode_pl,
  [sym_source_type_ip_socket_address] = sym_source_type_ip_socket_address,
  [sym_source_type_rs232] = sym_source_type_rs232,
  [sym_source_type_internal] = sym_source_type_internal,
  [sym_source_type_mac_address] = sym_source_type_mac_address,
  [sym_source_type_ipv4_port] = sym_source_type_ipv4_port,
  [sym_source_type_ipv4_port_mac_address] = sym_source_type_ipv4_port_mac_address,
  [sym_source_type_ipv4_port_mac_ipv6] = sym_source_type_ipv4_port_mac_ipv6,
  [sym_internal_queue_size_index_interpreter] = sym_internal_queue_size_index_interpreter,
  [sym_internal_queue_size_index_notification_mgr] = sym_internal_queue_size_index_notification_mgr,
  [sym_internal_queue_size_index_connection_mgr] = sym_internal_queue_size_index_connection_mgr,
  [sym_internal_queue_size_index_route_mgr] = sym_internal_queue_size_index_route_mgr,
  [sym_internal_queue_size_index_device_mgr] = sym_internal_queue_size_index_device_mgr,
  [sym_internal_queue_size_index_diagnostic_mgr] = sym_internal_queue_size_index_diagnostic_mgr,
  [sym_internal_queue_size_index_tcp_tx] = sym_internal_queue_size_index_tcp_tx,
  [sym_internal_queue_size_index_ipconnection_mgr] = sym_internal_queue_size_index_ipconnection_mgr,
  [sym_internal_queue_size_index_message_dispatcher] = sym_internal_queue_size_index_message_dispatcher,
  [sym_internal_queue_size_index_axlink_tx] = sym_internal_queue_size_index_axlink_tx,
  [sym_internal_queue_size_index_phastlink_tx] = sym_internal_queue_size_index_phastlink_tx,
  [sym_internal_queue_size_index_icsplontalk_tx] = sym_internal_queue_size_index_icsplontalk_tx,
  [sym_internal_queue_size_index_icsp232_tx] = sym_internal_queue_size_index_icsp232_tx,
  [sym_internal_queue_size_index_icspip_tx] = sym_internal_queue_size_index_icspip_tx,
  [sym_internal_queue_size_index_ni_device] = sym_internal_queue_size_index_ni_device,
  [sym_true] = sym_true,
  [sym_false] = sym_false,
  [sym_file_read_only] = sym_file_read_only,
  [sym_file_rw_new] = sym_file_rw_new,
  [sym_file_rw_append] = sym_file_rw_append,
  [sym_ip_tcp] = sym_ip_tcp,
  [sym_ip_udp] = sym_ip_udp,
  [sym_ip_udp_2way] = sym_ip_udp_2way,
  [sym_xml_encode_types] = sym_xml_encode_types,
  [sym_xml_encode_char_as_list] = sym_xml_encode_char_as_list,
  [sym_xml_encode_le] = sym_xml_encode_le,
  [sym_xml_decode_types] = sym_xml_decode_types,
  [sym_xml_decode_no_preserve] = sym_xml_decode_no_preserve,
  [sym_duet_dev_type_display_device] = sym_duet_dev_type_display_device,
  [sym_duet_dev_type_rfidsystem_device] = sym_duet_dev_type_rfidsystem_device,
  [sym_duet_dev_type_lightsystem_device] = sym_duet_dev_type_lightsystem_device,
  [sym_duet_dev_type_io_device] = sym_duet_dev_type_io_device,
  [sym_duet_dev_type_relay_device] = sym_duet_dev_type_relay_device,
  [sym_duet_dev_type_ups] = sym_duet_dev_type_ups,
  [sym_duet_dev_type_amplifier] = sym_duet_dev_type_amplifier,
  [sym_duet_dev_type_audio_conferencer] = sym_duet_dev_type_audio_conferencer,
  [sym_duet_dev_type_audio_mixer] = sym_duet_dev_type_audio_mixer,
  [sym_duet_dev_type_audio_processor] = sym_duet_dev_type_audio_processor,
  [sym_duet_dev_type_audio_tuner_device] = sym_duet_dev_type_audio_tuner_device,
  [sym_duet_dev_type_camera] = sym_duet_dev_type_camera,
  [sym_duet_dev_type_digital_media_encoder] = sym_duet_dev_type_digital_media_encoder,
  [sym_duet_dev_type_digital_media_decoder] = sym_duet_dev_type_digital_media_decoder,
  [sym_duet_dev_type_digital_media_server] = sym_duet_dev_type_digital_media_server,
  [sym_duet_dev_type_dss] = sym_duet_dev_type_dss,
  [sym_duet_dev_type_dvr] = sym_duet_dev_type_dvr,
  [sym_duet_dev_type_disc_device] = sym_duet_dev_type_disc_device,
  [sym_duet_dev_type_document_camera] = sym_duet_dev_type_document_camera,
  [sym_duet_dev_type_audio_tape] = sym_duet_dev_type_audio_tape,
  [sym_duet_dev_type_hvac] = sym_duet_dev_type_hvac,
  [sym_duet_dev_type_keypad] = sym_duet_dev_type_keypad,
  [sym_duet_dev_type_light] = sym_duet_dev_type_light,
  [sym_duet_dev_type_monitor] = sym_duet_dev_type_monitor,
  [sym_duet_dev_type_motor] = sym_duet_dev_type_motor,
  [sym_duet_dev_type_multi_window] = sym_duet_dev_type_multi_window,
  [sym_duet_dev_type_pool_spa] = sym_duet_dev_type_pool_spa,
  [sym_duet_dev_type_preamp_surround_sound_processor] = sym_duet_dev_type_preamp_surround_sound_processor,
  [sym_duet_dev_type_receiver] = sym_duet_dev_type_receiver,
  [sym_duet_dev_type_security_system] = sym_duet_dev_type_security_system,
  [sym_duet_dev_type_sensor_device] = sym_duet_dev_type_sensor_device,
  [sym_duet_dev_type_settop_box] = sym_duet_dev_type_settop_box,
  [sym_duet_dev_type_slide_projector] = sym_duet_dev_type_slide_projector,
  [sym_duet_dev_type_switcher] = sym_duet_dev_type_switcher,
  [sym_duet_dev_type_text_keypad] = sym_duet_dev_type_text_keypad,
  [sym_duet_dev_type_tv] = sym_duet_dev_type_tv,
  [sym_duet_dev_type_utility] = sym_duet_dev_type_utility,
  [sym_duet_dev_type_vcr] = sym_duet_dev_type_vcr,
  [sym_duet_dev_type_video_conferencer] = sym_duet_dev_type_video_conferencer,
  [sym_duet_dev_type_video_processor] = sym_duet_dev_type_video_processor,
  [sym_duet_dev_type_video_projector] = sym_duet_dev_type_video_projector,
  [sym_duet_dev_type_video_wall] = sym_duet_dev_type_video_wall,
  [sym_duet_dev_type_volume_controller] = sym_duet_dev_type_volume_controller,
  [sym_duet_dev_type_weather] = sym_duet_dev_type_weather,
  [sym_duet_dev_not_polled] = sym_duet_dev_not_polled,
  [sym_duet_dev_polled] = sym_duet_dev_polled,
  [sym_clkmgr_mode_network] = sym_clkmgr_mode_network,
  [sym_clkmgr_mode_standalone] = sym_clkmgr_mode_standalone,
  [sym_normal_standby] = sym_normal_standby,
  [sym_normal_wake] = sym_normal_wake,
  [sym_ip_multicast_ttl_option] = sym_ip_multicast_ttl_option,
  [sym_ip_ttl_subnet] = sym_ip_ttl_subnet,
  [sym_ip_ttl_site] = sym_ip_ttl_site,
  [sym_ip_ttl_region] = sym_ip_ttl_region,
  [sym_ip_ttl_continent] = sym_ip_ttl_continent,
  [sym_ip_tcp_nodelay_option] = sym_ip_tcp_nodelay_option,
  [sym_ip_nodelay_on] = sym_ip_nodelay_on,
  [sym_ip_nodelay_off] = sym_ip_nodelay_off,
  [sym_amx_error] = sym_amx_error,
  [sym_amx_warning] = sym_amx_warning,
  [sym_amx_info] = sym_amx_info,
  [sym_amx_debug] = sym_amx_debug,
  [sym_netlinx_login_success] = sym_netlinx_login_success,
  [sym_netlinx_login_fail] = sym_netlinx_login_fail,
  [sym_netlinx_logout] = sym_netlinx_logout,
  [sym_null_str] = sym_null_str,
  [sym_smtp_address] = sym_smtp_address,
  [sym_smtp_port_number] = sym_smtp_port_number,
  [sym_smtp_username] = sym_smtp_username,
  [sym_smtp_password] = sym_smtp_password,
  [sym_smtp_require_tls] = sym_smtp_require_tls,
  [sym_smtp_from] = sym_smtp_from,
  [sym_smtp_tls_true] = sym_smtp_tls_true,
  [sym_smtp_tls_false] = sym_smtp_tls_false,
  [sym_listview_on_row_select_event] = sym_listview_on_row_select_event,
  [sym_data_structure_unknown] = sym_data_structure_unknown,
  [sym_data_structure_datafeed] = sym_data_structure_datafeed,
  [sym_data_structure_datarecordset] = sym_data_structure_datarecordset,
  [sym_data_structure_datarecord] = sym_data_structure_datarecord,
  [sym_data_structure_datafield] = sym_data_structure_datafield,
  [sym_data_max_content_field_count] = sym_data_max_content_field_count,
  [sym_data_max_metadata_field_count] = sym_data_max_metadata_field_count,
  [sym_data_max_id_length] = sym_data_max_id_length,
  [sym_data_max_name_length] = sym_data_max_name_length,
  [sym_data_max_description_length] = sym_data_max_description_length,
  [sym_data_max_type_length] = sym_data_max_type_length,
  [sym_data_max_format_length] = sym_data_max_format_length,
  [sym_data_max_source_length] = sym_data_max_source_length,
  [sym_data_max_value_length] = sym_data_max_value_length,
  [sym_data_max_label_length] = sym_data_max_label_length,
  [sym_data_max_url_length] = sym_data_max_url_length,
  [sym_max_last_login_info_length] = sym_max_last_login_info_length,
  [sym_data_type_unknown] = sym_data_type_unknown,
  [sym_data_type_string] = sym_data_type_string,
  [sym_data_type_datetime] = sym_data_type_datetime,
  [sym_data_type_date] = sym_data_type_date,
  [sym_data_type_time] = sym_data_type_time,
  [sym_data_type_image] = sym_data_type_image,
  [sym_data_format_url] = sym_data_format_url,
  [sym_data_format_phone] = sym_data_format_phone,
  [sym_data_format_email] = sym_data_format_email,
  [sym_data_format_iso8601] = sym_data_format_iso8601,
  [sym_auditlog_priv] = sym_auditlog_priv,
  [sym_configuration_priv] = sym_configuration_priv,
  [sym_deviceconfig_priv] = sym_deviceconfig_priv,
  [sym_ftp_priv] = sym_ftp_priv,
  [sym_http_priv] = sym_http_priv,
  [sym_networkconfig_priv] = sym_networkconfig_priv,
  [sym_programport_priv] = sym_programport_priv,
  [sym_remoteui_priv] = sym_remoteui_priv,
  [sym_securitycontrol_priv] = sym_securitycontrol_priv,
  [sym_softwaremanagement_priv] = sym_softwaremanagement_priv,
  [sym_terminal_priv] = sym_terminal_priv,
  [sym_tpadmin_priv] = sym_tpadmin_priv,
  [sym_user1_priv] = sym_user1_priv,
  [sym_user2_priv] = sym_user2_priv,
  [sym_user3_priv] = sym_user3_priv,
  [sym_user4_priv] = sym_user4_priv,
  [sym_usermanagement_priv] = sym_usermanagement_priv,
  [sym_valid_account] = sym_valid_account,
  [sym_err_name_invalid] = sym_err_name_invalid,
  [sym_err_password_invalid] = sym_err_password_invalid,
  [sym_err_invalid_account] = sym_err_invalid_account,
  [sym_err_no_authorization] = sym_err_no_authorization,
  [sym_err_invalid_parameter] = sym_err_invalid_parameter,
  [sym_err_user_locked_out] = sym_err_user_locked_out,
  [sym_err_user_account_expired] = sym_err_user_account_expired,
  [sym_err_general_error] = sym_err_general_error,
  [sym_err_authentication_error] = sym_err_authentication_error,
  [sym_tls_ignore_certificate_errors] = sym_tls_ignore_certificate_errors,
  [sym_tls_validate_certificate] = sym_tls_validate_certificate,
  [sym_tdata] = sym_tdata,
  [sym_tchannel] = sym_tchannel,
  [sym_tlevel] = sym_tlevel,
  [sym_tbutton] = sym_tbutton,
  [sym_ttimeline] = sym_ttimeline,
  [sym_tcustom] = sym_tcustom,
  [sym_url_struct] = sym_url_struct,
  [sym_dns_struct] = sym_dns_struct,
  [sym_ip_address_struct] = sym_ip_address_struct,
  [sym_dev_info_struct] = sym_dev_info_struct,
  [sym_clkmgr_timeserver_struct] = sym_clkmgr_timeserver_struct,
  [sym_data_feed] = sym_data_feed,
  [sym_data_field] = sym_data_field,
  [sym_data_record] = sym_data_record,
  [sym_wc_data_feed] = sym_wc_data_feed,
  [sym_wc_data_field] = sym_wc_data_field,
  [sym_wc_data_record] = sym_wc_data_record,
  [sym_last_login_info] = sym_last_login_info,
  [sym____reserved___] = sym____reserved___,
  [sym_date] = sym_date,
  [sym_day] = sym_day,
  [sym_time] = sym_time,
  [sym_ldate] = sym_ldate,
  [sym_push_device] = sym_push_device,
  [sym_push_channel] = sym_push_channel,
  [sym_push_devchan] = sym_push_devchan,
  [sym_release_device] = sym_release_device,
  [sym_release_channel] = sym_release_channel,
  [sym_release_devchan] = sym_release_devchan,
  [sym_master_slot] = sym_master_slot,
  [sym_get_pulse_time] = sym_get_pulse_time,
  [sym_get_timer] = sym_get_timer,
  [sym_master_sn] = sym_master_sn,
  [sym_system_number] = sym_system_number,
  [sym_button] = sym_button,
  [sym_dv_channel] = sym_dv_channel,
  [sym_data] = sym_data,
  [sym_level] = sym_level,
  [sym_timeline] = sym_timeline,
  [sym_channel] = sym_channel,
  [sym_custom] = sym_custom,
  [sym_atoi] = sym_atoi,
  [sym_atol] = sym_atol,
  [sym_atof] = sym_atof,
  [sym_itoa] = sym_itoa,
  [sym_itohex] = sym_itohex,
  [sym_hextoi] = sym_hextoi,
  [sym_ftoa] = sym_ftoa,
  [sym_find_string] = sym_find_string,
  [sym_left_string] = sym_left_string,
  [sym_right_string] = sym_right_string,
  [sym_mid_string] = sym_mid_string,
  [sym_remove_string] = sym_remove_string,
  [sym_get_buffer_string] = sym_get_buffer_string,
  [sym_get_buffer_char] = sym_get_buffer_char,
  [sym_get_multi_buffer_string] = sym_get_multi_buffer_string,
  [sym_length_string] = sym_length_string,
  [sym_max_length_string] = sym_max_length_string,
  [sym_set_length_string] = sym_set_length_string,
  [sym_compare_string] = sym_compare_string,
  [sym_upper_string] = sym_upper_string,
  [sym_lower_string] = sym_lower_string,
  [sym_get_last] = sym_get_last,
  [sym_date_to_day] = sym_date_to_day,
  [sym_date_to_month] = sym_date_to_month,
  [sym_date_to_year] = sym_date_to_year,
  [sym_time_to_hour] = sym_time_to_hour,
  [sym_time_to_minute] = sym_time_to_minute,
  [sym_time_to_second] = sym_time_to_second,
  [sym_day_of_week] = sym_day_of_week,
  [sym_set_pulse_time] = sym_set_pulse_time,
  [sym_set_timer] = sym_set_timer,
  [sym_file_open] = sym_file_open,
  [sym_file_close] = sym_file_close,
  [sym_file_read] = sym_file_read,
  [sym_file_read_line] = sym_file_read_line,
  [sym_file_write] = sym_file_write,
  [sym_file_write_line] = sym_file_write_line,
  [sym_file_copy] = sym_file_copy,
  [sym_file_delete] = sym_file_delete,
  [sym_file_rename] = sym_file_rename,
  [sym_file_dir] = sym_file_dir,
  [sym_file_getdir] = sym_file_getdir,
  [sym_file_setdir] = sym_file_setdir,
  [sym_file_create_dir] = sym_file_create_dir,
  [sym_file_remove_dir] = sym_file_remove_dir,
  [sym_file_seek] = sym_file_seek,
  [sym_length_array] = sym_length_array,
  [sym_max_length_array] = sym_max_length_array,
  [sym_set_length_array] = sym_set_length_array,
  [sym_ip_client_open] = sym_ip_client_open,
  [sym_ip_client_close] = sym_ip_client_close,
  [sym_ip_server_open] = sym_ip_server_open,
  [sym_ip_server_close] = sym_ip_server_close,
  [sym_ip_bound_client_open] = sym_ip_bound_client_open,
  [sym_ip_mc_server_open] = sym_ip_mc_server_open,
  [sym_ip_set_option] = sym_ip_set_option,
  [sym_ssh_client_open] = sym_ssh_client_open,
  [sym_ssh_client_close] = sym_ssh_client_close,
  [sym_tls_client_open] = sym_tls_client_open,
  [sym_tls_client_close] = sym_tls_client_close,
  [sym_do_push] = sym_do_push,
  [sym_do_push_timed] = sym_do_push_timed,
  [sym_do_release] = sym_do_release,
  [sym_device_id] = sym_device_id,
  [sym_device_info] = sym_device_info,
  [sym_device_id_string] = sym_device_id_string,
  [sym_device_standby] = sym_device_standby,
  [sym_device_wake] = sym_device_wake,
  [sym_timeline_create] = sym_timeline_create,
  [sym_timeline_kill] = sym_timeline_kill,
  [sym_timeline_reload] = sym_timeline_reload,
  [sym_timeline_pause] = sym_timeline_pause,
  [sym_timeline_restart] = sym_timeline_restart,
  [sym_timeline_set] = sym_timeline_set,
  [sym_timeline_get] = sym_timeline_get,
  [sym_timeline_active] = sym_timeline_active,
  [sym_timeline_dynamic_id] = sym_timeline_dynamic_id,
  [sym_variable_to_string] = sym_variable_to_string,
  [sym_string_to_variable] = sym_string_to_variable,
  [sym_length_variable_to_string] = sym_length_variable_to_string,
  [sym_type_cast] = sym_type_cast,
  [sym_variable_to_xml] = sym_variable_to_xml,
  [sym_xml_to_variable] = sym_xml_to_variable,
  [sym_length_variable_to_xml] = sym_length_variable_to_xml,
  [sym_redirect_string] = sym_redirect_string,
  [sym_random_number] = sym_random_number,
  [sym_get_unique_id] = sym_get_unique_id,
  [sym_get_system_number] = sym_get_system_number,
  [sym_set_system_number] = sym_set_system_number,
  [sym_get_serial_number] = sym_get_serial_number,
  [sym_get_master_build] = sym_get_master_build,
  [sym_set_validation_code] = sym_set_validation_code,
  [sym_reboot] = sym_reboot,
  [sym_load_duet_module] = sym_load_duet_module,
  [sym_unload_duet_module] = sym_unload_duet_module,
  [sym_duet_mem_size_set] = sym_duet_mem_size_set,
  [sym_duet_mem_size_get] = sym_duet_mem_size_get,
  [sym_dynamic_application_device] = sym_dynamic_application_device,
  [sym_dynamic_polled_port] = sym_dynamic_polled_port,
  [sym_static_port_binding] = sym_static_port_binding,
  [sym_static_ip_binding] = sym_static_ip_binding,
  [sym_get_dns_list] = sym_get_dns_list,
  [sym_set_dns_list] = sym_set_dns_list,
  [sym_get_ip_address] = sym_get_ip_address,
  [sym_set_ip_address] = sym_set_ip_address,
  [sym_get_url_list] = sym_get_url_list,
  [sym_add_url_entry] = sym_add_url_entry,
  [sym_delete_url_entry] = sym_delete_url_entry,
  [sym_abs_value] = sym_abs_value,
  [sym_max_value] = sym_max_value,
  [sym_min_value] = sym_min_value,
  [sym_exp_value] = sym_exp_value,
  [sym_log_value] = sym_log_value,
  [sym_log10_value] = sym_log10_value,
  [sym_power_value] = sym_power_value,
  [sym_sqrt_value] = sym_sqrt_value,
  [sym_raw_be] = sym_raw_be,
  [sym_raw_le] = sym_raw_le,
  [sym_internal_queue_size_set] = sym_internal_queue_size_set,
  [sym_internal_queue_size_get] = sym_internal_queue_size_get,
  [sym_internal_threshold_set] = sym_internal_threshold_set,
  [sym_internal_threshold_get] = sym_internal_threshold_get,
  [sym_rebuilt_event] = sym_rebuilt_event,
  [sym_do_custom_event] = sym_do_custom_event,
  [sym_clkmgr_is_network_sourced] = sym_clkmgr_is_network_sourced,
  [sym_clkmgr_set_clk_source] = sym_clkmgr_set_clk_source,
  [sym_clkmgr_is_daylightsavings_on] = sym_clkmgr_is_daylightsavings_on,
  [sym_clkmgr_set_daylightsavings_mode] = sym_clkmgr_set_daylightsavings_mode,
  [sym_clkmgr_get_timezone] = sym_clkmgr_get_timezone,
  [sym_clkmgr_set_timezone] = sym_clkmgr_set_timezone,
  [sym_clkmgr_get_resync_period] = sym_clkmgr_get_resync_period,
  [sym_clkmgr_set_resync_period] = sym_clkmgr_set_resync_period,
  [sym_clkmgr_get_daylightsavings_offset] = sym_clkmgr_get_daylightsavings_offset,
  [sym_clkmgr_set_daylightsavings_offset] = sym_clkmgr_set_daylightsavings_offset,
  [sym_clkmgr_get_active_timeserver] = sym_clkmgr_get_active_timeserver,
  [sym_clkmgr_set_active_timeserver] = sym_clkmgr_set_active_timeserver,
  [sym_clkmgr_get_timeservers] = sym_clkmgr_get_timeservers,
  [sym_clkmgr_add_userdefined_timeserver] = sym_clkmgr_add_userdefined_timeserver,
  [sym_clkmgr_delete_userdefined_timeserver] = sym_clkmgr_delete_userdefined_timeserver,
  [sym_clkmgr_get_start_daylightsavings_rule] = sym_clkmgr_get_start_daylightsavings_rule,
  [sym_clkmgr_set_start_daylightsavings_rule] = sym_clkmgr_set_start_daylightsavings_rule,
  [sym_clkmgr_get_end_daylightsavings_rule] = sym_clkmgr_get_end_daylightsavings_rule,
  [sym_clkmgr_set_end_daylightsavings_rule] = sym_clkmgr_set_end_daylightsavings_rule,
  [sym_set_log_level] = sym_set_log_level,
  [sym_get_log_level] = sym_get_log_level,
  [sym_amx_log] = sym_amx_log,
  [sym_smtp_server_config_set] = sym_smtp_server_config_set,
  [sym_smtp_server_config_get] = sym_smtp_server_config_get,
  [sym_smtp_send] = sym_smtp_send,
  [sym_validate_netlinx_account] = sym_validate_netlinx_account,
  [sym_validate_netlinx_account_with_permission] = sym_validate_netlinx_account_with_permission,
  [sym_authenticate_certificate] = sym_authenticate_certificate,
  [sym_audit_netlinx_session_event] = sym_audit_netlinx_session_event,
  [sym_audit_netlinx_generic_event] = sym_audit_netlinx_generic_event,
  [sym_get_available_flash_disk_space] = sym_get_available_flash_disk_space,
  [sym_get_max_flash_disk_space] = sym_get_max_flash_disk_space,
  [sym_data_create_feed] = sym_data_create_feed,
  [sym_data_delete_feed] = sym_data_delete_feed,
  [sym_data_publish_feed] = sym_data_publish_feed,
  [sym_data_get_published_feed] = sym_data_get_published_feed,
  [sym_data_add_record] = sym_data_add_record,
  [sym_data_get_event_record] = sym_data_get_event_record,
  [sym__wc_data_create_feed] = sym__wc_data_create_feed,
  [sym__wc_data_add_record] = sym__wc_data_add_record,
  [sym__wc_data_get_event_record] = sym__wc_data_get_event_record,
  [sym_format] = sym_format,
  [sym_astro_clock] = sym_astro_clock,
  [sym_set_outdoor_temperature] = sym_set_outdoor_temperature,
  [sym_set_virtual_level_count] = sym_set_virtual_level_count,
  [sym_set_virtual_channel_count] = sym_set_virtual_channel_count,
  [sym_set_virtual_port_count] = sym_set_virtual_port_count,
  [sym__wc_encode_int] = sym__wc_encode_int,
  [sym__wc_decode_int] = sym__wc_decode_int,
  [sym__wc_int] = sym__wc_int,
  [sym__wc_to_ch_int] = sym__wc_to_ch_int,
  [sym__ch_to_wc_int] = sym__ch_to_wc_int,
  [sym__wc_find_string_int] = sym__wc_find_string_int,
  [sym__wc_left_string_int] = sym__wc_left_string_int,
  [sym__wc_lower_string_int] = sym__wc_lower_string_int,
  [sym__wc_mid_string_int] = sym__wc_mid_string_int,
  [sym__wc_remove_string_int] = sym__wc_remove_string_int,
  [sym__wc_right_string_int] = sym__wc_right_string_int,
  [sym__wc_upper_string_int] = sym__wc_upper_string_int,
  [sym__wc_compare_string_int] = sym__wc_compare_string_int,
  [sym__wc_get_buffer_char_int] = sym__wc_get_buffer_char_int,
  [sym__wc_get_buffer_string_int] = sym__wc_get_buffer_string_int,
  [sym__wc_concat_string_int] = sym__wc_concat_string_int,
  [sym__wc_file_open_int] = sym__wc_file_open_int,
  [sym__wc_file_close_int] = sym__wc_file_close_int,
  [sym__wc_file_read_int] = sym__wc_file_read_int,
  [sym__wc_file_read_line_int] = sym__wc_file_read_line_int,
  [sym__wc_file_write_int] = sym__wc_file_write_int,
  [sym__wc_file_write_line_int] = sym__wc_file_write_line_int,
  [anon_sym_EQ] = anon_sym_EQ,
  [aux_sym_preproc_include_token1] = aux_sym_preproc_include_token1,
  [aux_sym_preproc_define_token1] = aux_sym_preproc_define_token1,
  [aux_sym_preproc_warn_token1] = aux_sym_preproc_warn_token1,
  [aux_sym_preproc_disable_warning_token1] = aux_sym_preproc_disable_warning_token1,
  [aux_sym_preproc_if_defined_token1] = aux_sym_preproc_if_defined_token1,
  [anon_sym_LF] = anon_sym_LF,
  [aux_sym_preproc_if_defined_token2] = aux_sym_preproc_if_defined_token2,
  [aux_sym_preproc_if_not_defined_token1] = aux_sym_preproc_if_not_defined_token1,
  [aux_sym_preproc_else_token1] = aux_sym_preproc_else_token1,
  [anon_sym_COMMA] = anon_sym_COMMA,
  [sym_preproc_arg] = sym_preproc_arg,
  [anon_sym_LBRACK] = anon_sym_LBRACK,
  [anon_sym_RBRACK] = anon_sym_RBRACK,
  [anon_sym_LBRACE] = anon_sym_LBRACE,
  [anon_sym_RBRACE] = anon_sym_RBRACE,
  [anon_sym_COLON] = anon_sym_COLON,
  [anon_sym_LPAREN] = anon_sym_LPAREN,
  [anon_sym_RPAREN] = anon_sym_RPAREN,
  [anon_sym_SEMI] = anon_sym_SEMI,
  [anon_sym_BANG] = anon_sym_BANG,
  [anon_sym_TILDE] = anon_sym_TILDE,
  [anon_sym_DASH] = anon_sym_DASH,
  [anon_sym_PLUS] = anon_sym_PLUS,
  [anon_sym_STAR] = anon_sym_STAR,
  [anon_sym_SLASH] = anon_sym_SLASH,
  [anon_sym_PERCENT] = anon_sym_PERCENT,
  [anon_sym_PIPE_PIPE] = anon_sym_PIPE_PIPE,
  [anon_sym_AMP_AMP] = anon_sym_AMP_AMP,
  [anon_sym_PIPE] = anon_sym_PIPE,
  [anon_sym_CARET] = anon_sym_CARET,
  [anon_sym_AMP] = anon_sym_AMP,
  [anon_sym_EQ_EQ] = anon_sym_EQ_EQ,
  [anon_sym_BANG_EQ] = anon_sym_BANG_EQ,
  [anon_sym_LT_GT] = anon_sym_LT_GT,
  [anon_sym_GT] = anon_sym_GT,
  [anon_sym_GT_EQ] = anon_sym_GT_EQ,
  [anon_sym_LT_EQ] = anon_sym_LT_EQ,
  [anon_sym_LT] = anon_sym_LT,
  [anon_sym_LT_LT] = anon_sym_LT_LT,
  [anon_sym_GT_GT] = anon_sym_GT_GT,
  [anon_sym_DASH_DASH] = anon_sym_DASH_DASH,
  [anon_sym_PLUS_PLUS] = anon_sym_PLUS_PLUS,
  [anon_sym_DOT] = anon_sym_DOT,
  [sym_range_operator] = sym_range_operator,
  [anon_sym_DQUOTE] = anon_sym_DQUOTE,
  [anon_sym_SQUOTE] = anon_sym_SQUOTE,
  [aux_sym_string_literal_token1] = aux_sym_string_literal_token1,
  [sym_escape_sequence] = sym_escape_sequence,
  [sym_number_literal] = sym_number_literal,
  [sym_comment] = sym_comment,
  [sym__automatic_semicolon] = sym__automatic_semicolon,
  [sym_source_file] = sym_source_file,
  [sym__top_level_item] = sym__top_level_item,
  [sym__block_item] = sym__block_item,
  [sym_on_keyword] = sym_on_keyword,
  [sym_off_keyword] = sym_off_keyword,
  [sym_devchan_on_keyword] = sym_devchan_on_keyword,
  [sym_devchan_off_keyword] = sym_devchan_off_keyword,
  [sym_program_name] = sym_program_name,
  [sym_module_name] = sym_module_name,
  [sym_preproc_include] = sym_preproc_include,
  [sym_preproc_define] = sym_preproc_define,
  [sym_preproc_warn] = sym_preproc_warn,
  [sym_preproc_disable_warning] = sym_preproc_disable_warning,
  [sym_preproc_if_defined] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined] = sym_preproc_if_not_defined,
  [sym_preproc_else] = sym_preproc_else,
  [sym_preproc_if_defined_in_field_declaration_list] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_field_declaration_list] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_field_declaration_list] = sym_preproc_else,
  [sym_preproc_if_defined_in_initializer_list] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_initializer_list] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_initializer_list] = sym_preproc_else,
  [sym_preproc_if_defined_in_initializer_list_no_comma] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_initializer_list_no_comma] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_initializer_list_no_comma] = sym_preproc_else,
  [sym_preproc_if_defined_in_button_event_declarator] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_button_event_declarator] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_button_event_declarator] = sym_preproc_else,
  [sym_preproc_if_defined_in_button_event_block] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_button_event_block] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_button_event_block] = sym_preproc_else,
  [sym_preproc_if_defined_in_data_event_declarator] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_data_event_declarator] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_data_event_declarator] = sym_preproc_else,
  [sym_preproc_if_defined_in_data_event_block] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_data_event_block] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_data_event_block] = sym_preproc_else,
  [sym_preproc_if_defined_in_channel_event_declarator] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_channel_event_declarator] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_channel_event_declarator] = sym_preproc_else,
  [sym_preproc_if_defined_in_channel_event_block] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_channel_event_block] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_channel_event_block] = sym_preproc_else,
  [sym_preproc_if_defined_in_timeline_event_declarator] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_timeline_event_declarator] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_timeline_event_declarator] = sym_preproc_else,
  [sym_preproc_if_defined_in_level_event_declarator] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_level_event_declarator] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_level_event_declarator] = sym_preproc_else,
  [sym_preproc_if_defined_in_custom_event_declarator] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_custom_event_declarator] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_custom_event_declarator] = sym_preproc_else,
  [sym_preproc_if_defined_in_push_event_declarator] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_push_event_declarator] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_push_event_declarator] = sym_preproc_else,
  [sym_preproc_if_defined_in_release_event_declarator] = sym_preproc_if_defined,
  [sym_preproc_if_not_defined_in_release_event_declarator] = sym_preproc_if_not_defined,
  [sym_preproc_else_in_release_event_declarator] = sym_preproc_else,
  [sym_section] = sym_section,
  [sym_define_device_section] = sym_define_device_section,
  [sym_define_combine_section] = sym_define_combine_section,
  [sym_define_connect_level_section] = sym_define_connect_level_section,
  [sym_define_constant_section] = sym_define_constant_section,
  [sym_define_type_section] = sym_define_type_section,
  [sym_define_mutually_exclusive_section] = sym_define_mutually_exclusive_section,
  [sym_define_latching_section] = sym_define_latching_section,
  [sym_define_toggling_section] = sym_define_toggling_section,
  [sym_define_variable_section] = sym_define_variable_section,
  [sym_define_system_variable_section] = sym_define_system_variable_section,
  [sym_define_event_section] = sym_define_event_section,
  [sym_define_start_section] = sym_define_start_section,
  [sym_define_program_section] = sym_define_program_section,
  [sym_define_function] = sym_define_function,
  [sym_define_call] = sym_define_call,
  [sym_define_module] = sym_define_module,
  [sym_module_definition] = sym_module_definition,
  [sym_event_definition] = sym_event_definition,
  [sym_data_event_definition] = sym_data_event_definition,
  [sym__data_event_declarator_list] = sym__data_event_declarator_list,
  [sym_data_event_declarator] = sym_data_event_declarator,
  [sym_data_event_device_reference] = sym_data_event_device_reference,
  [sym_data_event_block] = sym_data_event_block,
  [sym__data_event_block_item] = sym__data_event_block_item,
  [sym_data_event_handler] = sym_data_event_handler,
  [sym_data_event_type] = sym_data_event_type,
  [sym_timeline_event_definition] = sym_timeline_event_definition,
  [sym__timeline_event_declarator_list] = sym__timeline_event_declarator_list,
  [sym_timeline_event_declarator] = sym_timeline_event_declarator,
  [sym_timeline_event_id_reference] = sym_timeline_event_id_reference,
  [sym_button_event_definition] = sym_button_event_definition,
  [sym__button_event_declarator_list] = sym__button_event_declarator_list,
  [sym_button_event_declarator] = sym_button_event_declarator,
  [sym_button_event_devchan_reference] = sym_button_event_devchan_reference,
  [sym_button_event_block] = sym_button_event_block,
  [sym__button_event_block_item] = sym__button_event_block_item,
  [sym_button_event_handler] = sym_button_event_handler,
  [sym_button_event_type] = sym_button_event_type,
  [sym_button_event_hold_repeat] = sym_button_event_hold_repeat,
  [sym_level_event_definition] = sym_level_event_definition,
  [sym__level_event_declarator_list] = sym__level_event_declarator_list,
  [sym_level_event_declarator] = sym_level_event_declarator,
  [sym_level_event_devlev_reference] = sym_level_event_devlev_reference,
  [sym_channel_event_definition] = sym_channel_event_definition,
  [sym__channel_event_declarator_list] = sym__channel_event_declarator_list,
  [sym_channel_event_declarator] = sym_channel_event_declarator,
  [sym_channel_event_devchan_reference] = sym_channel_event_devchan_reference,
  [sym_channel_event_block] = sym_channel_event_block,
  [sym__channel_event_block_item] = sym__channel_event_block_item,
  [sym_channel_event_handler] = sym_channel_event_handler,
  [sym_channel_event_type] = sym_channel_event_type,
  [sym_custom_event_definition] = sym_custom_event_definition,
  [sym__custom_event_declarator_list] = sym__custom_event_declarator_list,
  [sym_custom_event_declarator] = sym_custom_event_declarator,
  [sym_custom_event_reference] = sym_custom_event_reference,
  [sym_push_event_definition] = sym_push_event_definition,
  [sym__push_event_declarator_list] = sym__push_event_declarator_list,
  [sym_push_event_declarator] = sym_push_event_declarator,
  [sym_push_event_devchan_reference] = sym_push_event_devchan_reference,
  [sym_release_event_definition] = sym_release_event_definition,
  [sym__release_event_declarator_list] = sym__release_event_declarator_list,
  [sym_release_event_declarator] = sym_release_event_declarator,
  [sym_release_event_devchan_reference] = sym_release_event_devchan_reference,
  [sym_array_declarator] = sym_array_declarator,
  [sym_array_field_declarator] = sym_array_declarator,
  [sym_abstract_array_declarator] = sym_abstract_array_declarator,
  [sym_init_declarator] = sym_init_declarator,
  [sym_compound_statement] = sym_compound_statement,
  [sym_storage_class_specifier] = sym_storage_class_specifier,
  [sym_type_qualifier] = sym_type_qualifier,
  [sym_type_specifier] = sym_type_specifier,
  [sym_struct_specifier] = sym_struct_specifier,
  [sym_field_declaration_list] = sym_field_declaration_list,
  [sym__field_declaration_list_item] = sym__field_declaration_list_item,
  [sym_field_declaration] = sym_field_declaration,
  [sym__field_declaration_declarator] = sym__field_declaration_declarator,
  [sym_intrinsic_type] = sym_intrinsic_type,
  [sym_primitive_type] = sym_primitive_type,
  [sym_structured_type] = sym_structured_type,
  [sym_array_return_type] = sym_array_return_type,
  [sym_function_definition] = sym_function_definition,
  [sym_function_declaration] = sym_function_declaration,
  [sym_call_definition] = sym_call_definition,
  [sym_declaration] = sym_declaration,
  [sym_type_definition] = sym_type_definition,
  [sym__declaration_modifiers] = sym__declaration_modifiers,
  [sym__declaration_specifiers] = sym__declaration_specifiers,
  [sym__declarator] = sym__declarator,
  [sym__declaration_declarator] = sym__declaration_declarator,
  [sym__field_declarator] = sym__field_declarator,
  [sym__abstract_declarator] = sym__abstract_declarator,
  [sym_function_declarator] = sym_function_declarator,
  [sym__function_declaration_declarator] = sym_function_declarator,
  [sym_parenthesized_declarator] = sym_parenthesized_declarator,
  [sym_parenthesized_field_declarator] = sym_parenthesized_declarator,
  [sym_function_field_declarator] = sym_function_declarator,
  [sym_parameter_list] = sym_parameter_list,
  [sym_parameter_declaration] = sym_parameter_declaration,
  [sym_statement] = sym_statement,
  [sym__top_level_statement] = sym__top_level_statement,
  [sym_expression_statement] = sym_expression_statement,
  [sym_if_statement] = sym_if_statement,
  [sym_else_clause] = sym_else_clause,
  [sym_switch_statement] = sym_switch_statement,
  [sym_case_statement] = sym_case_statement,
  [sym_while_statement] = sym_while_statement,
  [sym_for_statement] = sym_for_statement,
  [sym__for_statement_body] = sym__for_statement_body,
  [sym_select_statement] = sym_select_statement,
  [sym_active_statement] = sym_active_statement,
  [sym_return_statement] = sym_return_statement,
  [sym_break_statement] = sym_break_statement,
  [sym_continue_statement] = sym_continue_statement,
  [sym_devchan_operation_statement] = sym_devchan_operation_statement,
  [sym_devchan_operation] = sym_devchan_operation,
  [sym_send_string_statement] = sym_send_string_statement,
  [sym_send_command_statement] = sym_send_command_statement,
  [sym_send_level_statement] = sym_send_level_statement,
  [sym_create_buffer_statement] = sym_create_buffer_statement,
  [sym_create_multi_buffer_statement] = sym_create_multi_buffer_statement,
  [sym_clear_buffer_statement] = sym_clear_buffer_statement,
  [sym_wait_statement] = sym_wait_statement,
  [sym_wait_until_statement] = sym_wait_until_statement,
  [sym_cancel_all_wait_statement] = sym_cancel_all_wait_statement,
  [sym_cancel_all_wait_until_statement] = sym_cancel_all_wait_until_statement,
  [sym_cancel_wait_statement] = sym_cancel_wait_statement,
  [sym_cancel_wait_until_statement] = sym_cancel_wait_until_statement,
  [sym_call_statement] = sym_call_statement,
  [sym_system_call_statement] = sym_system_call_statement,
  [sym_expression] = sym_expression,
  [sym_compiler_variable] = sym_compiler_variable,
  [sym_system_variable] = sym_system_variable,
  [sym_system_constant] = sym_system_constant,
  [sym_system_function] = sym_system_function,
  [sym_system_type] = sym_system_type,
  [sym_assignment_expression] = sym_assignment_expression,
  [sym_unary_expression] = sym_unary_expression,
  [sym_binary_expression] = sym_binary_expression,
  [sym_update_expression] = sym_update_expression,
  [sym_subscript_expression] = sym_subscript_expression,
  [sym_call_expression] = sym_call_expression,
  [sym_comma_expression] = sym_comma_expression,
  [sym_argument_list] = sym_argument_list,
  [sym_field_expression] = sym_field_expression,
  [sym_parenthesized_expression] = sym_parenthesized_expression,
  [sym_devlev_expression] = sym_devlev_expression,
  [sym_devchan_expression] = sym_devchan_expression,
  [sym_devchan_range_expression] = sym_devchan_range_expression,
  [sym_string_expression] = sym_string_expression,
  [sym_initializer_list] = sym_initializer_list,
  [sym_device_literal] = sym_device_literal,
  [sym_string_literal] = sym_string_literal,
  [aux_sym_source_file_repeat1] = aux_sym_source_file_repeat1,
  [aux_sym_preproc_if_defined_in_field_declaration_list_repeat1] = aux_sym_preproc_if_defined_in_field_declaration_list_repeat1,
  [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = aux_sym_preproc_if_defined_in_initializer_list_repeat1,
  [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1,
  [aux_sym_preproc_if_defined_in_button_event_declarator_repeat1] = aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [aux_sym_preproc_if_defined_in_button_event_declarator_repeat2] = aux_sym_preproc_if_defined_in_button_event_declarator_repeat2,
  [aux_sym_preproc_if_defined_in_button_event_block_repeat1] = aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [aux_sym_preproc_if_defined_in_button_event_block_repeat2] = aux_sym_preproc_if_defined_in_button_event_block_repeat2,
  [aux_sym_preproc_if_defined_in_data_event_declarator_repeat1] = aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [aux_sym_preproc_if_defined_in_data_event_declarator_repeat2] = aux_sym_preproc_if_defined_in_data_event_declarator_repeat2,
  [aux_sym_preproc_if_defined_in_data_event_block_repeat1] = aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [aux_sym_preproc_if_defined_in_data_event_block_repeat2] = aux_sym_preproc_if_defined_in_data_event_block_repeat2,
  [aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1] = aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2] = aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2,
  [aux_sym_preproc_if_defined_in_channel_event_block_repeat1] = aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [aux_sym_preproc_if_defined_in_channel_event_block_repeat2] = aux_sym_preproc_if_defined_in_channel_event_block_repeat2,
  [aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1] = aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2] = aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2,
  [aux_sym_preproc_if_defined_in_level_event_declarator_repeat1] = aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [aux_sym_preproc_if_defined_in_level_event_declarator_repeat2] = aux_sym_preproc_if_defined_in_level_event_declarator_repeat2,
  [aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1] = aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2] = aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2,
  [aux_sym_preproc_if_defined_in_push_event_declarator_repeat1] = aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [aux_sym_preproc_if_defined_in_push_event_declarator_repeat2] = aux_sym_preproc_if_defined_in_push_event_declarator_repeat2,
  [aux_sym_preproc_if_defined_in_release_event_declarator_repeat1] = aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [aux_sym_preproc_if_defined_in_release_event_declarator_repeat2] = aux_sym_preproc_if_defined_in_release_event_declarator_repeat2,
  [aux_sym__data_event_declarator_list_repeat1] = aux_sym__data_event_declarator_list_repeat1,
  [aux_sym__timeline_event_declarator_list_repeat1] = aux_sym__timeline_event_declarator_list_repeat1,
  [aux_sym__button_event_declarator_list_repeat1] = aux_sym__button_event_declarator_list_repeat1,
  [aux_sym__level_event_declarator_list_repeat1] = aux_sym__level_event_declarator_list_repeat1,
  [aux_sym__channel_event_declarator_list_repeat1] = aux_sym__channel_event_declarator_list_repeat1,
  [aux_sym__custom_event_declarator_list_repeat1] = aux_sym__custom_event_declarator_list_repeat1,
  [aux_sym__push_event_declarator_list_repeat1] = aux_sym__push_event_declarator_list_repeat1,
  [aux_sym__release_event_declarator_list_repeat1] = aux_sym__release_event_declarator_list_repeat1,
  [aux_sym_compound_statement_repeat1] = aux_sym_compound_statement_repeat1,
  [aux_sym__field_declaration_declarator_repeat1] = aux_sym__field_declaration_declarator_repeat1,
  [aux_sym_function_definition_repeat1] = aux_sym_function_definition_repeat1,
  [aux_sym_declaration_repeat1] = aux_sym_declaration_repeat1,
  [aux_sym__declaration_specifiers_repeat1] = aux_sym__declaration_specifiers_repeat1,
  [aux_sym_parameter_list_repeat1] = aux_sym_parameter_list_repeat1,
  [aux_sym_case_statement_repeat1] = aux_sym_case_statement_repeat1,
  [aux_sym_subscript_expression_repeat1] = aux_sym_subscript_expression_repeat1,
  [aux_sym_argument_list_repeat1] = aux_sym_argument_list_repeat1,
  [aux_sym_string_expression_repeat1] = aux_sym_string_expression_repeat1,
  [aux_sym_initializer_list_repeat1] = aux_sym_initializer_list_repeat1,
  [aux_sym_string_literal_repeat1] = aux_sym_string_literal_repeat1,
  [alias_sym_field_identifier] = alias_sym_field_identifier,
  [alias_sym_type_identifier] = alias_sym_type_identifier,
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym_program_name_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_module_name_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_device_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_combine_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_constant_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_type_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_variable_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_system_variable_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_connect_level_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_latching_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_mutually_exclusive_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_toggling_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_call_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_function_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_library_function_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_start_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_event_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_program_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_define_module_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_char_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_widechar_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_integer_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_sinteger_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_long_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_slong_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_float_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_double_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_variant_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_variantarray_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_local_var_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_stack_var_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_constant_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_volatile_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_non_volatile_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_persistent_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_dev_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_devlev_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_devchan_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_if_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_else_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_switch_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_case_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_default_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_while_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_for_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_break_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_continue_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_return_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_select_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_active_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_struct_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_structure_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_button_event_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_channel_event_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_level_event_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_data_event_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_event_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_custom_event_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_push_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_release_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_hold_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_repeat_keyword] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_on_keyword_token1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_off_keyword_token1] = {
    .visible = false,
    .named = false,
  },
  [sym_online_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_offline_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_onerror_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_string_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_command_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_standby_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_awake_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_send_level_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_send_string_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_send_command_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_clear_buffer_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_create_buffer_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_create_multi_buffer_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_call_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_system_call_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_devchan_to_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_devchan_min_to_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_devchan_total_off_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_devchan_pulse_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_wait_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_cancel_wait_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_wait_until_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_cancel_wait_until_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_cancel_all_wait_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_cancel_all_wait_until_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_and] = {
    .visible = true,
    .named = true,
  },
  [sym_or] = {
    .visible = true,
    .named = true,
  },
  [sym_not] = {
    .visible = true,
    .named = true,
  },
  [sym_xor] = {
    .visible = true,
    .named = true,
  },
  [sym_band] = {
    .visible = true,
    .named = true,
  },
  [sym_bor] = {
    .visible = true,
    .named = true,
  },
  [sym_bxor] = {
    .visible = true,
    .named = true,
  },
  [sym_bnot] = {
    .visible = true,
    .named = true,
  },
  [sym_lshift] = {
    .visible = true,
    .named = true,
  },
  [sym_rshift] = {
    .visible = true,
    .named = true,
  },
  [sym___file__] = {
    .visible = false,
    .named = true,
  },
  [sym___date__] = {
    .visible = false,
    .named = true,
  },
  [sym___ldate__] = {
    .visible = false,
    .named = true,
  },
  [sym___time__] = {
    .visible = false,
    .named = true,
  },
  [sym___line__] = {
    .visible = false,
    .named = true,
  },
  [sym___name__] = {
    .visible = false,
    .named = true,
  },
  [sym___netlinx__] = {
    .visible = false,
    .named = true,
  },
  [sym_netlinx_axi_version] = {
    .visible = true,
    .named = true,
  },
  [sym_url_flg_tcp] = {
    .visible = true,
    .named = true,
  },
  [sym_url_flg_accinfopresent] = {
    .visible = true,
    .named = true,
  },
  [sym_url_flg_temp] = {
    .visible = true,
    .named = true,
  },
  [sym_url_flg_stat_prgnetlinx] = {
    .visible = true,
    .named = true,
  },
  [sym_url_flg_stat_mask] = {
    .visible = true,
    .named = true,
  },
  [sym_url_flg_stat_lookup] = {
    .visible = true,
    .named = true,
  },
  [sym_url_flg_stat_connecting] = {
    .visible = true,
    .named = true,
  },
  [sym_url_flg_stat_waiting] = {
    .visible = true,
    .named = true,
  },
  [sym_url_flg_stat_connected] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_addr_flg_dhcp] = {
    .visible = true,
    .named = true,
  },
  [sym_first_virtual_device] = {
    .visible = true,
    .named = true,
  },
  [sym_first_local_port] = {
    .visible = true,
    .named = true,
  },
  [sym_dynamic_virtual_device] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_once] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_repeat] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_absolute] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_relative] = {
    .visible = true,
    .named = true,
  },
  [sym_do_push_timed_infinite] = {
    .visible = true,
    .named = true,
  },
  [sym_source_type_no_address] = {
    .visible = true,
    .named = true,
  },
  [sym_source_type_neuron_id] = {
    .visible = true,
    .named = true,
  },
  [sym_source_type_ip_address] = {
    .visible = true,
    .named = true,
  },
  [sym_source_type_axlink] = {
    .visible = true,
    .named = true,
  },
  [sym_source_type_neuron_subnode_icsp] = {
    .visible = true,
    .named = true,
  },
  [sym_source_type_neuron_subnode_pl] = {
    .visible = true,
    .named = true,
  },
  [sym_source_type_ip_socket_address] = {
    .visible = true,
    .named = true,
  },
  [sym_source_type_rs232] = {
    .visible = true,
    .named = true,
  },
  [sym_source_type_internal] = {
    .visible = true,
    .named = true,
  },
  [sym_source_type_mac_address] = {
    .visible = true,
    .named = true,
  },
  [sym_source_type_ipv4_port] = {
    .visible = true,
    .named = true,
  },
  [sym_source_type_ipv4_port_mac_address] = {
    .visible = true,
    .named = true,
  },
  [sym_source_type_ipv4_port_mac_ipv6] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_interpreter] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_notification_mgr] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_connection_mgr] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_route_mgr] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_device_mgr] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_diagnostic_mgr] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_tcp_tx] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_ipconnection_mgr] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_message_dispatcher] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_axlink_tx] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_phastlink_tx] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_icsplontalk_tx] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_icsp232_tx] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_icspip_tx] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_index_ni_device] = {
    .visible = true,
    .named = true,
  },
  [sym_true] = {
    .visible = true,
    .named = true,
  },
  [sym_false] = {
    .visible = true,
    .named = true,
  },
  [sym_file_read_only] = {
    .visible = true,
    .named = true,
  },
  [sym_file_rw_new] = {
    .visible = true,
    .named = true,
  },
  [sym_file_rw_append] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_tcp] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_udp] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_udp_2way] = {
    .visible = true,
    .named = true,
  },
  [sym_xml_encode_types] = {
    .visible = true,
    .named = true,
  },
  [sym_xml_encode_char_as_list] = {
    .visible = true,
    .named = true,
  },
  [sym_xml_encode_le] = {
    .visible = true,
    .named = true,
  },
  [sym_xml_decode_types] = {
    .visible = true,
    .named = true,
  },
  [sym_xml_decode_no_preserve] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_display_device] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_rfidsystem_device] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_lightsystem_device] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_io_device] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_relay_device] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_ups] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_amplifier] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_audio_conferencer] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_audio_mixer] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_audio_processor] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_audio_tuner_device] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_camera] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_digital_media_encoder] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_digital_media_decoder] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_digital_media_server] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_dss] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_dvr] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_disc_device] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_document_camera] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_audio_tape] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_hvac] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_keypad] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_light] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_monitor] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_motor] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_multi_window] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_pool_spa] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_preamp_surround_sound_processor] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_receiver] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_security_system] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_sensor_device] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_settop_box] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_slide_projector] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_switcher] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_text_keypad] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_tv] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_utility] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_vcr] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_video_conferencer] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_video_processor] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_video_projector] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_video_wall] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_volume_controller] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_type_weather] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_not_polled] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_dev_polled] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_mode_network] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_mode_standalone] = {
    .visible = true,
    .named = true,
  },
  [sym_normal_standby] = {
    .visible = true,
    .named = true,
  },
  [sym_normal_wake] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_multicast_ttl_option] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_ttl_subnet] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_ttl_site] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_ttl_region] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_ttl_continent] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_tcp_nodelay_option] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_nodelay_on] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_nodelay_off] = {
    .visible = true,
    .named = true,
  },
  [sym_amx_error] = {
    .visible = true,
    .named = true,
  },
  [sym_amx_warning] = {
    .visible = true,
    .named = true,
  },
  [sym_amx_info] = {
    .visible = true,
    .named = true,
  },
  [sym_amx_debug] = {
    .visible = true,
    .named = true,
  },
  [sym_netlinx_login_success] = {
    .visible = true,
    .named = true,
  },
  [sym_netlinx_login_fail] = {
    .visible = true,
    .named = true,
  },
  [sym_netlinx_logout] = {
    .visible = true,
    .named = true,
  },
  [sym_null_str] = {
    .visible = true,
    .named = true,
  },
  [sym_smtp_address] = {
    .visible = true,
    .named = true,
  },
  [sym_smtp_port_number] = {
    .visible = true,
    .named = true,
  },
  [sym_smtp_username] = {
    .visible = true,
    .named = true,
  },
  [sym_smtp_password] = {
    .visible = true,
    .named = true,
  },
  [sym_smtp_require_tls] = {
    .visible = true,
    .named = true,
  },
  [sym_smtp_from] = {
    .visible = true,
    .named = true,
  },
  [sym_smtp_tls_true] = {
    .visible = true,
    .named = true,
  },
  [sym_smtp_tls_false] = {
    .visible = true,
    .named = true,
  },
  [sym_listview_on_row_select_event] = {
    .visible = true,
    .named = true,
  },
  [sym_data_structure_unknown] = {
    .visible = true,
    .named = true,
  },
  [sym_data_structure_datafeed] = {
    .visible = true,
    .named = true,
  },
  [sym_data_structure_datarecordset] = {
    .visible = true,
    .named = true,
  },
  [sym_data_structure_datarecord] = {
    .visible = true,
    .named = true,
  },
  [sym_data_structure_datafield] = {
    .visible = true,
    .named = true,
  },
  [sym_data_max_content_field_count] = {
    .visible = true,
    .named = true,
  },
  [sym_data_max_metadata_field_count] = {
    .visible = true,
    .named = true,
  },
  [sym_data_max_id_length] = {
    .visible = true,
    .named = true,
  },
  [sym_data_max_name_length] = {
    .visible = true,
    .named = true,
  },
  [sym_data_max_description_length] = {
    .visible = true,
    .named = true,
  },
  [sym_data_max_type_length] = {
    .visible = true,
    .named = true,
  },
  [sym_data_max_format_length] = {
    .visible = true,
    .named = true,
  },
  [sym_data_max_source_length] = {
    .visible = true,
    .named = true,
  },
  [sym_data_max_value_length] = {
    .visible = true,
    .named = true,
  },
  [sym_data_max_label_length] = {
    .visible = true,
    .named = true,
  },
  [sym_data_max_url_length] = {
    .visible = true,
    .named = true,
  },
  [sym_max_last_login_info_length] = {
    .visible = true,
    .named = true,
  },
  [sym_data_type_unknown] = {
    .visible = true,
    .named = true,
  },
  [sym_data_type_string] = {
    .visible = true,
    .named = true,
  },
  [sym_data_type_datetime] = {
    .visible = true,
    .named = true,
  },
  [sym_data_type_date] = {
    .visible = true,
    .named = true,
  },
  [sym_data_type_time] = {
    .visible = true,
    .named = true,
  },
  [sym_data_type_image] = {
    .visible = true,
    .named = true,
  },
  [sym_data_format_url] = {
    .visible = true,
    .named = true,
  },
  [sym_data_format_phone] = {
    .visible = true,
    .named = true,
  },
  [sym_data_format_email] = {
    .visible = true,
    .named = true,
  },
  [sym_data_format_iso8601] = {
    .visible = true,
    .named = true,
  },
  [sym_auditlog_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_configuration_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_deviceconfig_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_ftp_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_http_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_networkconfig_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_programport_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_remoteui_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_securitycontrol_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_softwaremanagement_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_terminal_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_tpadmin_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_user1_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_user2_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_user3_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_user4_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_usermanagement_priv] = {
    .visible = true,
    .named = true,
  },
  [sym_valid_account] = {
    .visible = true,
    .named = true,
  },
  [sym_err_name_invalid] = {
    .visible = true,
    .named = true,
  },
  [sym_err_password_invalid] = {
    .visible = true,
    .named = true,
  },
  [sym_err_invalid_account] = {
    .visible = true,
    .named = true,
  },
  [sym_err_no_authorization] = {
    .visible = true,
    .named = true,
  },
  [sym_err_invalid_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_err_user_locked_out] = {
    .visible = true,
    .named = true,
  },
  [sym_err_user_account_expired] = {
    .visible = true,
    .named = true,
  },
  [sym_err_general_error] = {
    .visible = true,
    .named = true,
  },
  [sym_err_authentication_error] = {
    .visible = true,
    .named = true,
  },
  [sym_tls_ignore_certificate_errors] = {
    .visible = true,
    .named = true,
  },
  [sym_tls_validate_certificate] = {
    .visible = true,
    .named = true,
  },
  [sym_tdata] = {
    .visible = true,
    .named = true,
  },
  [sym_tchannel] = {
    .visible = true,
    .named = true,
  },
  [sym_tlevel] = {
    .visible = true,
    .named = true,
  },
  [sym_tbutton] = {
    .visible = true,
    .named = true,
  },
  [sym_ttimeline] = {
    .visible = true,
    .named = true,
  },
  [sym_tcustom] = {
    .visible = true,
    .named = true,
  },
  [sym_url_struct] = {
    .visible = true,
    .named = true,
  },
  [sym_dns_struct] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_address_struct] = {
    .visible = true,
    .named = true,
  },
  [sym_dev_info_struct] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_timeserver_struct] = {
    .visible = true,
    .named = true,
  },
  [sym_data_feed] = {
    .visible = true,
    .named = true,
  },
  [sym_data_field] = {
    .visible = true,
    .named = true,
  },
  [sym_data_record] = {
    .visible = true,
    .named = true,
  },
  [sym_wc_data_feed] = {
    .visible = true,
    .named = true,
  },
  [sym_wc_data_field] = {
    .visible = true,
    .named = true,
  },
  [sym_wc_data_record] = {
    .visible = true,
    .named = true,
  },
  [sym_last_login_info] = {
    .visible = true,
    .named = true,
  },
  [sym____reserved___] = {
    .visible = false,
    .named = true,
  },
  [sym_date] = {
    .visible = true,
    .named = true,
  },
  [sym_day] = {
    .visible = true,
    .named = true,
  },
  [sym_time] = {
    .visible = true,
    .named = true,
  },
  [sym_ldate] = {
    .visible = true,
    .named = true,
  },
  [sym_push_device] = {
    .visible = true,
    .named = true,
  },
  [sym_push_channel] = {
    .visible = true,
    .named = true,
  },
  [sym_push_devchan] = {
    .visible = true,
    .named = true,
  },
  [sym_release_device] = {
    .visible = true,
    .named = true,
  },
  [sym_release_channel] = {
    .visible = true,
    .named = true,
  },
  [sym_release_devchan] = {
    .visible = true,
    .named = true,
  },
  [sym_master_slot] = {
    .visible = true,
    .named = true,
  },
  [sym_get_pulse_time] = {
    .visible = true,
    .named = true,
  },
  [sym_get_timer] = {
    .visible = true,
    .named = true,
  },
  [sym_master_sn] = {
    .visible = true,
    .named = true,
  },
  [sym_system_number] = {
    .visible = true,
    .named = true,
  },
  [sym_button] = {
    .visible = true,
    .named = true,
  },
  [sym_dv_channel] = {
    .visible = true,
    .named = true,
  },
  [sym_data] = {
    .visible = true,
    .named = true,
  },
  [sym_level] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline] = {
    .visible = true,
    .named = true,
  },
  [sym_channel] = {
    .visible = true,
    .named = true,
  },
  [sym_custom] = {
    .visible = true,
    .named = true,
  },
  [sym_atoi] = {
    .visible = true,
    .named = true,
  },
  [sym_atol] = {
    .visible = true,
    .named = true,
  },
  [sym_atof] = {
    .visible = true,
    .named = true,
  },
  [sym_itoa] = {
    .visible = true,
    .named = true,
  },
  [sym_itohex] = {
    .visible = true,
    .named = true,
  },
  [sym_hextoi] = {
    .visible = true,
    .named = true,
  },
  [sym_ftoa] = {
    .visible = true,
    .named = true,
  },
  [sym_find_string] = {
    .visible = true,
    .named = true,
  },
  [sym_left_string] = {
    .visible = true,
    .named = true,
  },
  [sym_right_string] = {
    .visible = true,
    .named = true,
  },
  [sym_mid_string] = {
    .visible = true,
    .named = true,
  },
  [sym_remove_string] = {
    .visible = true,
    .named = true,
  },
  [sym_get_buffer_string] = {
    .visible = true,
    .named = true,
  },
  [sym_get_buffer_char] = {
    .visible = true,
    .named = true,
  },
  [sym_get_multi_buffer_string] = {
    .visible = true,
    .named = true,
  },
  [sym_length_string] = {
    .visible = true,
    .named = true,
  },
  [sym_max_length_string] = {
    .visible = true,
    .named = true,
  },
  [sym_set_length_string] = {
    .visible = true,
    .named = true,
  },
  [sym_compare_string] = {
    .visible = true,
    .named = true,
  },
  [sym_upper_string] = {
    .visible = true,
    .named = true,
  },
  [sym_lower_string] = {
    .visible = true,
    .named = true,
  },
  [sym_get_last] = {
    .visible = true,
    .named = true,
  },
  [sym_date_to_day] = {
    .visible = true,
    .named = true,
  },
  [sym_date_to_month] = {
    .visible = true,
    .named = true,
  },
  [sym_date_to_year] = {
    .visible = true,
    .named = true,
  },
  [sym_time_to_hour] = {
    .visible = true,
    .named = true,
  },
  [sym_time_to_minute] = {
    .visible = true,
    .named = true,
  },
  [sym_time_to_second] = {
    .visible = true,
    .named = true,
  },
  [sym_day_of_week] = {
    .visible = true,
    .named = true,
  },
  [sym_set_pulse_time] = {
    .visible = true,
    .named = true,
  },
  [sym_set_timer] = {
    .visible = true,
    .named = true,
  },
  [sym_file_open] = {
    .visible = true,
    .named = true,
  },
  [sym_file_close] = {
    .visible = true,
    .named = true,
  },
  [sym_file_read] = {
    .visible = true,
    .named = true,
  },
  [sym_file_read_line] = {
    .visible = true,
    .named = true,
  },
  [sym_file_write] = {
    .visible = true,
    .named = true,
  },
  [sym_file_write_line] = {
    .visible = true,
    .named = true,
  },
  [sym_file_copy] = {
    .visible = true,
    .named = true,
  },
  [sym_file_delete] = {
    .visible = true,
    .named = true,
  },
  [sym_file_rename] = {
    .visible = true,
    .named = true,
  },
  [sym_file_dir] = {
    .visible = true,
    .named = true,
  },
  [sym_file_getdir] = {
    .visible = true,
    .named = true,
  },
  [sym_file_setdir] = {
    .visible = true,
    .named = true,
  },
  [sym_file_create_dir] = {
    .visible = true,
    .named = true,
  },
  [sym_file_remove_dir] = {
    .visible = true,
    .named = true,
  },
  [sym_file_seek] = {
    .visible = true,
    .named = true,
  },
  [sym_length_array] = {
    .visible = true,
    .named = true,
  },
  [sym_max_length_array] = {
    .visible = true,
    .named = true,
  },
  [sym_set_length_array] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_client_open] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_client_close] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_server_open] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_server_close] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_bound_client_open] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_mc_server_open] = {
    .visible = true,
    .named = true,
  },
  [sym_ip_set_option] = {
    .visible = true,
    .named = true,
  },
  [sym_ssh_client_open] = {
    .visible = true,
    .named = true,
  },
  [sym_ssh_client_close] = {
    .visible = true,
    .named = true,
  },
  [sym_tls_client_open] = {
    .visible = true,
    .named = true,
  },
  [sym_tls_client_close] = {
    .visible = true,
    .named = true,
  },
  [sym_do_push] = {
    .visible = true,
    .named = true,
  },
  [sym_do_push_timed] = {
    .visible = true,
    .named = true,
  },
  [sym_do_release] = {
    .visible = true,
    .named = true,
  },
  [sym_device_id] = {
    .visible = true,
    .named = true,
  },
  [sym_device_info] = {
    .visible = true,
    .named = true,
  },
  [sym_device_id_string] = {
    .visible = true,
    .named = true,
  },
  [sym_device_standby] = {
    .visible = true,
    .named = true,
  },
  [sym_device_wake] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_create] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_kill] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_reload] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_pause] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_restart] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_set] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_get] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_active] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_dynamic_id] = {
    .visible = true,
    .named = true,
  },
  [sym_variable_to_string] = {
    .visible = true,
    .named = true,
  },
  [sym_string_to_variable] = {
    .visible = true,
    .named = true,
  },
  [sym_length_variable_to_string] = {
    .visible = true,
    .named = true,
  },
  [sym_type_cast] = {
    .visible = true,
    .named = true,
  },
  [sym_variable_to_xml] = {
    .visible = true,
    .named = true,
  },
  [sym_xml_to_variable] = {
    .visible = true,
    .named = true,
  },
  [sym_length_variable_to_xml] = {
    .visible = true,
    .named = true,
  },
  [sym_redirect_string] = {
    .visible = true,
    .named = true,
  },
  [sym_random_number] = {
    .visible = true,
    .named = true,
  },
  [sym_get_unique_id] = {
    .visible = true,
    .named = true,
  },
  [sym_get_system_number] = {
    .visible = true,
    .named = true,
  },
  [sym_set_system_number] = {
    .visible = true,
    .named = true,
  },
  [sym_get_serial_number] = {
    .visible = true,
    .named = true,
  },
  [sym_get_master_build] = {
    .visible = true,
    .named = true,
  },
  [sym_set_validation_code] = {
    .visible = true,
    .named = true,
  },
  [sym_reboot] = {
    .visible = true,
    .named = true,
  },
  [sym_load_duet_module] = {
    .visible = true,
    .named = true,
  },
  [sym_unload_duet_module] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_mem_size_set] = {
    .visible = true,
    .named = true,
  },
  [sym_duet_mem_size_get] = {
    .visible = true,
    .named = true,
  },
  [sym_dynamic_application_device] = {
    .visible = true,
    .named = true,
  },
  [sym_dynamic_polled_port] = {
    .visible = true,
    .named = true,
  },
  [sym_static_port_binding] = {
    .visible = true,
    .named = true,
  },
  [sym_static_ip_binding] = {
    .visible = true,
    .named = true,
  },
  [sym_get_dns_list] = {
    .visible = true,
    .named = true,
  },
  [sym_set_dns_list] = {
    .visible = true,
    .named = true,
  },
  [sym_get_ip_address] = {
    .visible = true,
    .named = true,
  },
  [sym_set_ip_address] = {
    .visible = true,
    .named = true,
  },
  [sym_get_url_list] = {
    .visible = true,
    .named = true,
  },
  [sym_add_url_entry] = {
    .visible = true,
    .named = true,
  },
  [sym_delete_url_entry] = {
    .visible = true,
    .named = true,
  },
  [sym_abs_value] = {
    .visible = true,
    .named = true,
  },
  [sym_max_value] = {
    .visible = true,
    .named = true,
  },
  [sym_min_value] = {
    .visible = true,
    .named = true,
  },
  [sym_exp_value] = {
    .visible = true,
    .named = true,
  },
  [sym_log_value] = {
    .visible = true,
    .named = true,
  },
  [sym_log10_value] = {
    .visible = true,
    .named = true,
  },
  [sym_power_value] = {
    .visible = true,
    .named = true,
  },
  [sym_sqrt_value] = {
    .visible = true,
    .named = true,
  },
  [sym_raw_be] = {
    .visible = true,
    .named = true,
  },
  [sym_raw_le] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_set] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_queue_size_get] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_threshold_set] = {
    .visible = true,
    .named = true,
  },
  [sym_internal_threshold_get] = {
    .visible = true,
    .named = true,
  },
  [sym_rebuilt_event] = {
    .visible = true,
    .named = true,
  },
  [sym_do_custom_event] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_is_network_sourced] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_set_clk_source] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_is_daylightsavings_on] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_set_daylightsavings_mode] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_get_timezone] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_set_timezone] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_get_resync_period] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_set_resync_period] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_get_daylightsavings_offset] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_set_daylightsavings_offset] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_get_active_timeserver] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_set_active_timeserver] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_get_timeservers] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_add_userdefined_timeserver] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_delete_userdefined_timeserver] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_get_start_daylightsavings_rule] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_set_start_daylightsavings_rule] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_get_end_daylightsavings_rule] = {
    .visible = true,
    .named = true,
  },
  [sym_clkmgr_set_end_daylightsavings_rule] = {
    .visible = true,
    .named = true,
  },
  [sym_set_log_level] = {
    .visible = true,
    .named = true,
  },
  [sym_get_log_level] = {
    .visible = true,
    .named = true,
  },
  [sym_amx_log] = {
    .visible = true,
    .named = true,
  },
  [sym_smtp_server_config_set] = {
    .visible = true,
    .named = true,
  },
  [sym_smtp_server_config_get] = {
    .visible = true,
    .named = true,
  },
  [sym_smtp_send] = {
    .visible = true,
    .named = true,
  },
  [sym_validate_netlinx_account] = {
    .visible = true,
    .named = true,
  },
  [sym_validate_netlinx_account_with_permission] = {
    .visible = true,
    .named = true,
  },
  [sym_authenticate_certificate] = {
    .visible = true,
    .named = true,
  },
  [sym_audit_netlinx_session_event] = {
    .visible = true,
    .named = true,
  },
  [sym_audit_netlinx_generic_event] = {
    .visible = true,
    .named = true,
  },
  [sym_get_available_flash_disk_space] = {
    .visible = true,
    .named = true,
  },
  [sym_get_max_flash_disk_space] = {
    .visible = true,
    .named = true,
  },
  [sym_data_create_feed] = {
    .visible = true,
    .named = true,
  },
  [sym_data_delete_feed] = {
    .visible = true,
    .named = true,
  },
  [sym_data_publish_feed] = {
    .visible = true,
    .named = true,
  },
  [sym_data_get_published_feed] = {
    .visible = true,
    .named = true,
  },
  [sym_data_add_record] = {
    .visible = true,
    .named = true,
  },
  [sym_data_get_event_record] = {
    .visible = true,
    .named = true,
  },
  [sym__wc_data_create_feed] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_data_add_record] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_data_get_event_record] = {
    .visible = false,
    .named = true,
  },
  [sym_format] = {
    .visible = true,
    .named = true,
  },
  [sym_astro_clock] = {
    .visible = true,
    .named = true,
  },
  [sym_set_outdoor_temperature] = {
    .visible = true,
    .named = true,
  },
  [sym_set_virtual_level_count] = {
    .visible = true,
    .named = true,
  },
  [sym_set_virtual_channel_count] = {
    .visible = true,
    .named = true,
  },
  [sym_set_virtual_port_count] = {
    .visible = true,
    .named = true,
  },
  [sym__wc_encode_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_decode_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_to_ch_int] = {
    .visible = false,
    .named = true,
  },
  [sym__ch_to_wc_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_find_string_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_left_string_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_lower_string_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_mid_string_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_remove_string_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_right_string_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_upper_string_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_compare_string_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_get_buffer_char_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_get_buffer_string_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_concat_string_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_file_open_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_file_close_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_file_read_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_file_read_line_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_file_write_int] = {
    .visible = false,
    .named = true,
  },
  [sym__wc_file_write_line_int] = {
    .visible = false,
    .named = true,
  },
  [anon_sym_EQ] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_preproc_include_token1] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_preproc_define_token1] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_preproc_warn_token1] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_preproc_disable_warning_token1] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_preproc_if_defined_token1] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_LF] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_preproc_if_defined_token2] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_preproc_if_not_defined_token1] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_preproc_else_token1] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_COMMA] = {
    .visible = true,
    .named = false,
  },
  [sym_preproc_arg] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SEMI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_TILDE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT] = {
    .visible = true,
    .named = false,
  },
  [sym_range_operator] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_DQUOTE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SQUOTE] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_string_literal_token1] = {
    .visible = true,
    .named = true,
  },
  [sym_escape_sequence] = {
    .visible = true,
    .named = true,
  },
  [sym_number_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_comment] = {
    .visible = true,
    .named = true,
  },
  [sym__automatic_semicolon] = {
    .visible = false,
    .named = true,
  },
  [sym_source_file] = {
    .visible = true,
    .named = true,
  },
  [sym__top_level_item] = {
    .visible = false,
    .named = true,
  },
  [sym__block_item] = {
    .visible = false,
    .named = true,
  },
  [sym_on_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_off_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_devchan_on_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_devchan_off_keyword] = {
    .visible = true,
    .named = true,
  },
  [sym_program_name] = {
    .visible = true,
    .named = true,
  },
  [sym_module_name] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_include] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_define] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_warn] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_disable_warning] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_initializer_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_initializer_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_initializer_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_initializer_list_no_comma] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_initializer_list_no_comma] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_initializer_list_no_comma] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_button_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_button_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_button_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_button_event_block] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_button_event_block] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_button_event_block] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_data_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_data_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_data_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_data_event_block] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_data_event_block] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_data_event_block] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_channel_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_channel_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_channel_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_channel_event_block] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_channel_event_block] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_channel_event_block] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_timeline_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_timeline_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_timeline_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_level_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_level_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_level_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_custom_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_custom_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_custom_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_push_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_push_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_push_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_defined_in_release_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_not_defined_in_release_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_release_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_device_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_combine_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_connect_level_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_constant_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_type_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_mutually_exclusive_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_latching_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_toggling_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_variable_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_system_variable_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_event_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_start_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_program_section] = {
    .visible = true,
    .named = true,
  },
  [sym_define_function] = {
    .visible = true,
    .named = true,
  },
  [sym_define_call] = {
    .visible = true,
    .named = true,
  },
  [sym_define_module] = {
    .visible = true,
    .named = true,
  },
  [sym_module_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_event_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_data_event_definition] = {
    .visible = true,
    .named = true,
  },
  [sym__data_event_declarator_list] = {
    .visible = false,
    .named = true,
  },
  [sym_data_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_data_event_device_reference] = {
    .visible = true,
    .named = true,
  },
  [sym_data_event_block] = {
    .visible = true,
    .named = true,
  },
  [sym__data_event_block_item] = {
    .visible = false,
    .named = true,
  },
  [sym_data_event_handler] = {
    .visible = true,
    .named = true,
  },
  [sym_data_event_type] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_event_definition] = {
    .visible = true,
    .named = true,
  },
  [sym__timeline_event_declarator_list] = {
    .visible = false,
    .named = true,
  },
  [sym_timeline_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_timeline_event_id_reference] = {
    .visible = true,
    .named = true,
  },
  [sym_button_event_definition] = {
    .visible = true,
    .named = true,
  },
  [sym__button_event_declarator_list] = {
    .visible = false,
    .named = true,
  },
  [sym_button_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_button_event_devchan_reference] = {
    .visible = true,
    .named = true,
  },
  [sym_button_event_block] = {
    .visible = true,
    .named = true,
  },
  [sym__button_event_block_item] = {
    .visible = false,
    .named = true,
  },
  [sym_button_event_handler] = {
    .visible = true,
    .named = true,
  },
  [sym_button_event_type] = {
    .visible = true,
    .named = true,
  },
  [sym_button_event_hold_repeat] = {
    .visible = true,
    .named = true,
  },
  [sym_level_event_definition] = {
    .visible = true,
    .named = true,
  },
  [sym__level_event_declarator_list] = {
    .visible = false,
    .named = true,
  },
  [sym_level_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_level_event_devlev_reference] = {
    .visible = true,
    .named = true,
  },
  [sym_channel_event_definition] = {
    .visible = true,
    .named = true,
  },
  [sym__channel_event_declarator_list] = {
    .visible = false,
    .named = true,
  },
  [sym_channel_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_channel_event_devchan_reference] = {
    .visible = true,
    .named = true,
  },
  [sym_channel_event_block] = {
    .visible = true,
    .named = true,
  },
  [sym__channel_event_block_item] = {
    .visible = false,
    .named = true,
  },
  [sym_channel_event_handler] = {
    .visible = true,
    .named = true,
  },
  [sym_channel_event_type] = {
    .visible = true,
    .named = true,
  },
  [sym_custom_event_definition] = {
    .visible = true,
    .named = true,
  },
  [sym__custom_event_declarator_list] = {
    .visible = false,
    .named = true,
  },
  [sym_custom_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_custom_event_reference] = {
    .visible = true,
    .named = true,
  },
  [sym_push_event_definition] = {
    .visible = true,
    .named = true,
  },
  [sym__push_event_declarator_list] = {
    .visible = false,
    .named = true,
  },
  [sym_push_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_push_event_devchan_reference] = {
    .visible = true,
    .named = true,
  },
  [sym_release_event_definition] = {
    .visible = true,
    .named = true,
  },
  [sym__release_event_declarator_list] = {
    .visible = false,
    .named = true,
  },
  [sym_release_event_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_release_event_devchan_reference] = {
    .visible = true,
    .named = true,
  },
  [sym_array_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_array_field_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_abstract_array_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_init_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_compound_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_storage_class_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_type_qualifier] = {
    .visible = true,
    .named = true,
  },
  [sym_type_specifier] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_struct_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym__field_declaration_list_item] = {
    .visible = false,
    .named = true,
  },
  [sym_field_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym__field_declaration_declarator] = {
    .visible = false,
    .named = true,
  },
  [sym_intrinsic_type] = {
    .visible = true,
    .named = true,
  },
  [sym_primitive_type] = {
    .visible = true,
    .named = true,
  },
  [sym_structured_type] = {
    .visible = true,
    .named = true,
  },
  [sym_array_return_type] = {
    .visible = true,
    .named = true,
  },
  [sym_function_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_function_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_call_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_type_definition] = {
    .visible = true,
    .named = true,
  },
  [sym__declaration_modifiers] = {
    .visible = false,
    .named = true,
  },
  [sym__declaration_specifiers] = {
    .visible = false,
    .named = true,
  },
  [sym__declarator] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__declaration_declarator] = {
    .visible = false,
    .named = true,
  },
  [sym__field_declarator] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__abstract_declarator] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_function_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym__function_declaration_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_field_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_function_field_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_parameter_list] = {
    .visible = true,
    .named = true,
  },
  [sym_parameter_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_statement] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__top_level_statement] = {
    .visible = false,
    .named = true,
  },
  [sym_expression_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_if_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_else_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_switch_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_case_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_while_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_for_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__for_statement_body] = {
    .visible = false,
    .named = true,
  },
  [sym_select_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_active_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_return_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_break_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_continue_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_devchan_operation_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_devchan_operation] = {
    .visible = true,
    .named = true,
  },
  [sym_send_string_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_send_command_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_send_level_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_create_buffer_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_create_multi_buffer_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_clear_buffer_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_wait_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_wait_until_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_cancel_all_wait_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_cancel_all_wait_until_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_cancel_wait_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_cancel_wait_until_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_call_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_system_call_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_expression] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_compiler_variable] = {
    .visible = true,
    .named = true,
  },
  [sym_system_variable] = {
    .visible = true,
    .named = true,
  },
  [sym_system_constant] = {
    .visible = true,
    .named = true,
  },
  [sym_system_function] = {
    .visible = true,
    .named = true,
  },
  [sym_system_type] = {
    .visible = true,
    .named = true,
  },
  [sym_assignment_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_unary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_binary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_update_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_subscript_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_call_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_comma_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_argument_list] = {
    .visible = true,
    .named = true,
  },
  [sym_field_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_devlev_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_devchan_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_devchan_range_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_string_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_initializer_list] = {
    .visible = true,
    .named = true,
  },
  [sym_device_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_string_literal] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_source_file_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_field_declaration_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_button_event_declarator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_button_event_declarator_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_button_event_block_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_button_event_block_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_data_event_declarator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_data_event_declarator_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_data_event_block_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_data_event_block_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_channel_event_block_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_channel_event_block_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_level_event_declarator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_level_event_declarator_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_push_event_declarator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_push_event_declarator_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_release_event_declarator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_defined_in_release_event_declarator_repeat2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__data_event_declarator_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__timeline_event_declarator_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__button_event_declarator_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__level_event_declarator_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__channel_event_declarator_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__custom_event_declarator_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__push_event_declarator_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__release_event_declarator_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_compound_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__field_declaration_declarator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_function_definition_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_declaration_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__declaration_specifiers_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_parameter_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_case_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_subscript_expression_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_argument_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_string_expression_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_initializer_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_string_literal_repeat1] = {
    .visible = false,
    .named = false,
  },
  [alias_sym_field_identifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_type_identifier] = {
    .visible = true,
    .named = true,
  },
};

enum ts_field_identifiers {
  field_alternative = 1,
  field_argument = 2,
  field_arguments = 3,
  field_body = 4,
  field_call = 5,
  field_channel = 6,
  field_code = 7,
  field_condition = 8,
  field_consequence = 9,
  field_declarator = 10,
  field_devchan = 11,
  field_device = 12,
  field_devlev = 13,
  field_end = 14,
  field_eventid = 15,
  field_field = 16,
  field_function = 17,
  field_id = 18,
  field_index = 19,
  field_initializer = 20,
  field_instance_name = 21,
  field_left = 22,
  field_level = 23,
  field_local_declarations = 24,
  field_message = 25,
  field_module_name = 26,
  field_name = 27,
  field_operation = 28,
  field_operator = 29,
  field_parameters = 30,
  field_path = 31,
  field_port = 32,
  field_reference = 33,
  field_return_type = 34,
  field_right = 35,
  field_size = 36,
  field_start = 37,
  field_system = 38,
  field_target = 39,
  field_time = 40,
  field_type = 41,
  field_update = 42,
  field_value = 43,
};

static const char * const ts_field_names[] = {
  [0] = NULL,
  [field_alternative] = "alternative",
  [field_argument] = "argument",
  [field_arguments] = "arguments",
  [field_body] = "body",
  [field_call] = "call",
  [field_channel] = "channel",
  [field_code] = "code",
  [field_condition] = "condition",
  [field_consequence] = "consequence",
  [field_declarator] = "declarator",
  [field_devchan] = "devchan",
  [field_device] = "device",
  [field_devlev] = "devlev",
  [field_end] = "end",
  [field_eventid] = "eventid",
  [field_field] = "field",
  [field_function] = "function",
  [field_id] = "id",
  [field_index] = "index",
  [field_initializer] = "initializer",
  [field_instance_name] = "instance_name",
  [field_left] = "left",
  [field_level] = "level",
  [field_local_declarations] = "local_declarations",
  [field_message] = "message",
  [field_module_name] = "module_name",
  [field_name] = "name",
  [field_operation] = "operation",
  [field_operator] = "operator",
  [field_parameters] = "parameters",
  [field_path] = "path",
  [field_port] = "port",
  [field_reference] = "reference",
  [field_return_type] = "return_type",
  [field_right] = "right",
  [field_size] = "size",
  [field_start] = "start",
  [field_system] = "system",
  [field_target] = "target",
  [field_time] = "time",
  [field_type] = "type",
  [field_update] = "update",
  [field_value] = "value",
};

static const TSMapSlice ts_field_map_slices[PRODUCTION_ID_COUNT] = {
  [2] = {.index = 0, .length = 1},
  [3] = {.index = 1, .length = 1},
  [4] = {.index = 2, .length = 1},
  [5] = {.index = 3, .length = 1},
  [6] = {.index = 4, .length = 1},
  [7] = {.index = 5, .length = 1},
  [8] = {.index = 6, .length = 1},
  [9] = {.index = 7, .length = 1},
  [10] = {.index = 8, .length = 1},
  [11] = {.index = 9, .length = 2},
  [12] = {.index = 11, .length = 2},
  [13] = {.index = 13, .length = 2},
  [14] = {.index = 15, .length = 2},
  [15] = {.index = 17, .length = 2},
  [16] = {.index = 19, .length = 1},
  [17] = {.index = 20, .length = 2},
  [18] = {.index = 22, .length = 2},
  [19] = {.index = 24, .length = 2},
  [20] = {.index = 26, .length = 1},
  [21] = {.index = 27, .length = 2},
  [22] = {.index = 29, .length = 1},
  [23] = {.index = 30, .length = 2},
  [24] = {.index = 32, .length = 1},
  [25] = {.index = 33, .length = 2},
  [26] = {.index = 35, .length = 1},
  [27] = {.index = 36, .length = 1},
  [28] = {.index = 37, .length = 1},
  [29] = {.index = 38, .length = 3},
  [30] = {.index = 41, .length = 1},
  [31] = {.index = 42, .length = 2},
  [32] = {.index = 44, .length = 2},
  [33] = {.index = 46, .length = 2},
  [34] = {.index = 48, .length = 2},
  [35] = {.index = 50, .length = 3},
  [36] = {.index = 53, .length = 2},
  [37] = {.index = 38, .length = 3},
  [38] = {.index = 55, .length = 2},
  [39] = {.index = 57, .length = 1},
  [40] = {.index = 58, .length = 1},
  [41] = {.index = 59, .length = 3},
  [42] = {.index = 62, .length = 3},
  [43] = {.index = 65, .length = 2},
  [44] = {.index = 67, .length = 3},
  [45] = {.index = 70, .length = 2},
  [46] = {.index = 72, .length = 2},
  [47] = {.index = 74, .length = 2},
  [48] = {.index = 76, .length = 2},
  [49] = {.index = 78, .length = 2},
  [50] = {.index = 80, .length = 2},
  [51] = {.index = 82, .length = 2},
  [52] = {.index = 84, .length = 2},
  [53] = {.index = 86, .length = 1},
  [54] = {.index = 87, .length = 3},
  [55] = {.index = 90, .length = 3},
  [56] = {.index = 93, .length = 1},
  [57] = {.index = 94, .length = 4},
  [58] = {.index = 98, .length = 1},
  [59] = {.index = 99, .length = 4},
  [60] = {.index = 103, .length = 3},
  [61] = {.index = 106, .length = 3},
  [62] = {.index = 109, .length = 1},
  [63] = {.index = 110, .length = 4},
  [64] = {.index = 114, .length = 1},
  [66] = {.index = 115, .length = 2},
  [67] = {.index = 117, .length = 2},
  [68] = {.index = 119, .length = 2},
  [69] = {.index = 121, .length = 2},
  [70] = {.index = 123, .length = 3},
  [71] = {.index = 126, .length = 2},
  [72] = {.index = 128, .length = 4},
  [73] = {.index = 132, .length = 3},
  [74] = {.index = 135, .length = 5},
  [75] = {.index = 140, .length = 2},
  [76] = {.index = 142, .length = 2},
  [77] = {.index = 144, .length = 2},
  [78] = {.index = 146, .length = 2},
  [79] = {.index = 148, .length = 2},
  [80] = {.index = 150, .length = 2},
  [81] = {.index = 152, .length = 2},
  [82] = {.index = 154, .length = 2},
  [83] = {.index = 156, .length = 1},
  [84] = {.index = 157, .length = 1},
  [85] = {.index = 158, .length = 3},
  [86] = {.index = 161, .length = 3},
  [87] = {.index = 164, .length = 3},
};

static const TSFieldMapEntry ts_field_map_entries[] = {
  [0] =
    {field_type, 0},
  [1] =
    {field_body, 1},
  [2] =
    {field_devchan, 1},
  [3] =
    {field_devlev, 1},
  [4] =
    {field_device, 1},
  [5] =
    {field_id, 1},
  [6] =
    {field_reference, 1},
  [7] =
    {field_time, 1},
  [8] =
    {field_condition, 1},
  [9] =
    {field_argument, 1},
    {field_operator, 0},
  [11] =
    {field_declarator, 0, .inherited = true},
    {field_parameters, 0, .inherited = true},
  [13] =
    {field_argument, 0},
    {field_operator, 1},
  [15] =
    {field_arguments, 1},
    {field_function, 0},
  [17] =
    {field_argument, 0},
    {field_index, 1, .inherited = true},
  [19] =
    {field_type, 1},
  [20] =
    {field_body, 1},
    {field_name, 0},
  [22] =
    {field_condition, 1},
    {field_consequence, 2},
  [24] =
    {field_body, 2},
    {field_condition, 1},
  [26] =
    {field_value, 1},
  [27] =
    {field_body, 2},
    {field_name, 1},
  [29] =
    {field_call, 1},
  [30] =
    {field_name, 2},
    {field_time, 1},
  [32] =
    {field_name, 1},
  [33] =
    {field_condition, 1},
    {field_name, 2},
  [35] =
    {field_path, 1},
  [36] =
    {field_message, 1},
  [37] =
    {field_code, 1},
  [38] =
    {field_left, 0},
    {field_operator, 1},
    {field_right, 2},
  [41] =
    {field_declarator, 1},
  [42] =
    {field_declarator, 0},
    {field_parameters, 1},
  [44] =
    {field_declarator, 1},
    {field_type, 0, .inherited = true},
  [46] =
    {field_operation, 0},
    {field_target, 1},
  [48] =
    {field_left, 0},
    {field_right, 2},
  [50] =
    {field_argument, 0},
    {field_field, 2},
    {field_operator, 1},
  [53] =
    {field_index, 0, .inherited = true},
    {field_index, 1, .inherited = true},
  [55] =
    {field_end, 2},
    {field_start, 0},
  [57] =
    {field_parameters, 3},
  [58] =
    {field_type, 0, .inherited = true},
  [59] =
    {field_body, 2},
    {field_name, 0},
    {field_parameters, 1},
  [62] =
    {field_body, 2},
    {field_local_declarations, 1},
    {field_name, 0},
  [65] =
    {field_name, 0},
    {field_parameters, 1},
  [67] =
    {field_alternative, 3},
    {field_condition, 1},
    {field_consequence, 2},
  [70] =
    {field_body, 3},
    {field_condition, 1},
  [72] =
    {field_arguments, 2},
    {field_call, 1},
  [74] =
    {field_name, 1},
    {field_value, 2},
  [76] =
    {field_alternative, 2},
    {field_name, 1},
  [78] =
    {field_declarator, 2},
    {field_type, 1},
  [80] =
    {field_declarator, 1},
    {field_declarator, 2, .inherited = true},
  [82] =
    {field_declarator, 0, .inherited = true},
    {field_declarator, 1, .inherited = true},
  [84] =
    {field_declarator, 0},
    {field_value, 2},
  [86] =
    {field_declarator, 0},
  [87] =
    {field_declarator, 2},
    {field_type, 0, .inherited = true},
    {field_type, 1},
  [90] =
    {field_declarator, 1},
    {field_declarator, 2, .inherited = true},
    {field_type, 0, .inherited = true},
  [93] =
    {field_index, 1},
  [94] =
    {field_body, 3},
    {field_local_declarations, 2},
    {field_name, 0},
    {field_parameters, 1},
  [98] =
    {field_size, 2},
  [99] =
    {field_body, 3},
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 0},
  [103] =
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 0},
  [106] =
    {field_instance_name, 1},
    {field_module_name, 0},
    {field_parameters, 2},
  [109] =
    {field_update, 2},
  [110] =
    {field_body, 4},
    {field_condition, 2, .inherited = true},
    {field_initializer, 2, .inherited = true},
    {field_update, 2, .inherited = true},
  [114] =
    {field_initializer, 0},
  [115] =
    {field_device, 1},
    {field_value, 3},
  [117] =
    {field_alternative, 3},
    {field_name, 1},
  [119] =
    {field_channel, 3},
    {field_device, 1},
  [121] =
    {field_body, 2},
    {field_type, 0},
  [123] =
    {field_declarator, 2},
    {field_declarator, 3, .inherited = true},
    {field_type, 1},
  [126] =
    {field_declarator, 0},
    {field_size, 2},
  [128] =
    {field_declarator, 2},
    {field_declarator, 3, .inherited = true},
    {field_type, 0, .inherited = true},
    {field_type, 1},
  [132] =
    {field_device, 0},
    {field_port, 2},
    {field_system, 4},
  [135] =
    {field_body, 4},
    {field_local_declarations, 3},
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 0},
  [140] =
    {field_condition, 1},
    {field_update, 3},
  [142] =
    {field_initializer, 0},
    {field_update, 3},
  [144] =
    {field_condition, 2},
    {field_initializer, 0},
  [146] =
    {field_declarator, 1, .inherited = true},
    {field_type, 0, .inherited = true},
  [148] =
    {field_declarator, 0},
    {field_declarator, 1, .inherited = true},
  [150] =
    {field_device, 1},
    {field_level, 3},
  [152] =
    {field_devchan, 1},
    {field_eventid, 3},
  [154] =
    {field_alternative, 4},
    {field_name, 1},
  [156] =
    {field_time, 2},
  [157] =
    {field_size, 1},
  [158] =
    {field_condition, 2},
    {field_initializer, 0},
    {field_update, 4},
  [161] =
    {field_device, 1},
    {field_level, 3},
    {field_value, 5},
  [164] =
    {field_device, 1},
    {field_id, 3},
    {field_type, 5},
};

static const TSSymbol ts_alias_sequences[PRODUCTION_ID_COUNT][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [0] = {0},
  [1] = {
    [0] = alias_sym_type_identifier,
  },
  [21] = {
    [1] = alias_sym_type_identifier,
  },
  [35] = {
    [2] = alias_sym_field_identifier,
  },
  [37] = {
    [0] = sym_identifier,
  },
  [49] = {
    [1] = alias_sym_type_identifier,
  },
  [54] = {
    [1] = alias_sym_type_identifier,
  },
  [65] = {
    [0] = alias_sym_field_identifier,
  },
  [70] = {
    [1] = alias_sym_type_identifier,
  },
  [72] = {
    [1] = alias_sym_type_identifier,
  },
};

static const uint16_t ts_non_terminal_alias_map[] = {
  sym_system_variable, 2,
    sym_system_variable,
    sym_identifier,
  0,
};

static const TSStateId ts_primary_state_ids[STATE_COUNT] = {
  [0] = 0,
  [1] = 1,
  [2] = 2,
  [3] = 3,
  [4] = 2,
  [5] = 3,
  [6] = 3,
  [7] = 2,
  [8] = 8,
  [9] = 9,
  [10] = 10,
  [11] = 11,
  [12] = 12,
  [13] = 13,
  [14] = 11,
  [15] = 10,
  [16] = 12,
  [17] = 13,
  [18] = 9,
  [19] = 8,
  [20] = 8,
  [21] = 12,
  [22] = 11,
  [23] = 10,
  [24] = 9,
  [25] = 13,
  [26] = 26,
  [27] = 27,
  [28] = 28,
  [29] = 26,
  [30] = 30,
  [31] = 31,
  [32] = 32,
  [33] = 33,
  [34] = 34,
  [35] = 35,
  [36] = 34,
  [37] = 35,
  [38] = 32,
  [39] = 33,
  [40] = 31,
  [41] = 41,
  [42] = 41,
  [43] = 41,
  [44] = 44,
  [45] = 45,
  [46] = 41,
  [47] = 44,
  [48] = 44,
  [49] = 44,
  [50] = 50,
  [51] = 50,
  [52] = 31,
  [53] = 33,
  [54] = 32,
  [55] = 34,
  [56] = 35,
  [57] = 57,
  [58] = 58,
  [59] = 59,
  [60] = 58,
  [61] = 57,
  [62] = 62,
  [63] = 63,
  [64] = 64,
  [65] = 65,
  [66] = 66,
  [67] = 67,
  [68] = 68,
  [69] = 69,
  [70] = 70,
  [71] = 71,
  [72] = 72,
  [73] = 73,
  [74] = 74,
  [75] = 75,
  [76] = 76,
  [77] = 77,
  [78] = 78,
  [79] = 79,
  [80] = 80,
  [81] = 75,
  [82] = 76,
  [83] = 77,
  [84] = 78,
  [85] = 79,
  [86] = 65,
  [87] = 87,
  [88] = 88,
  [89] = 89,
  [90] = 62,
  [91] = 63,
  [92] = 64,
  [93] = 66,
  [94] = 67,
  [95] = 68,
  [96] = 69,
  [97] = 97,
  [98] = 98,
  [99] = 70,
  [100] = 59,
  [101] = 71,
  [102] = 72,
  [103] = 103,
  [104] = 74,
  [105] = 105,
  [106] = 106,
  [107] = 73,
  [108] = 108,
  [109] = 109,
  [110] = 110,
  [111] = 106,
  [112] = 112,
  [113] = 103,
  [114] = 108,
  [115] = 88,
  [116] = 89,
  [117] = 109,
  [118] = 118,
  [119] = 105,
  [120] = 120,
  [121] = 97,
  [122] = 98,
  [123] = 123,
  [124] = 124,
  [125] = 125,
  [126] = 126,
  [127] = 127,
  [128] = 87,
  [129] = 129,
  [130] = 130,
  [131] = 80,
  [132] = 132,
  [133] = 133,
  [134] = 134,
  [135] = 118,
  [136] = 127,
  [137] = 130,
  [138] = 112,
  [139] = 124,
  [140] = 120,
  [141] = 132,
  [142] = 110,
  [143] = 126,
  [144] = 123,
  [145] = 133,
  [146] = 134,
  [147] = 125,
  [148] = 129,
  [149] = 50,
  [150] = 58,
  [151] = 57,
  [152] = 68,
  [153] = 74,
  [154] = 76,
  [155] = 77,
  [156] = 78,
  [157] = 79,
  [158] = 75,
  [159] = 62,
  [160] = 63,
  [161] = 64,
  [162] = 66,
  [163] = 67,
  [164] = 73,
  [165] = 69,
  [166] = 65,
  [167] = 70,
  [168] = 59,
  [169] = 71,
  [170] = 72,
  [171] = 80,
  [172] = 106,
  [173] = 103,
  [174] = 87,
  [175] = 88,
  [176] = 89,
  [177] = 108,
  [178] = 105,
  [179] = 109,
  [180] = 98,
  [181] = 97,
  [182] = 112,
  [183] = 124,
  [184] = 129,
  [185] = 118,
  [186] = 127,
  [187] = 125,
  [188] = 123,
  [189] = 130,
  [190] = 120,
  [191] = 132,
  [192] = 134,
  [193] = 126,
  [194] = 133,
  [195] = 110,
  [196] = 196,
  [197] = 197,
  [198] = 198,
  [199] = 199,
  [200] = 199,
  [201] = 201,
  [202] = 198,
  [203] = 203,
  [204] = 204,
  [205] = 205,
  [206] = 206,
  [207] = 207,
  [208] = 208,
  [209] = 209,
  [210] = 210,
  [211] = 211,
  [212] = 212,
  [213] = 213,
  [214] = 214,
  [215] = 215,
  [216] = 216,
  [217] = 217,
  [218] = 218,
  [219] = 219,
  [220] = 201,
  [221] = 221,
  [222] = 222,
  [223] = 223,
  [224] = 224,
  [225] = 225,
  [226] = 226,
  [227] = 227,
  [228] = 228,
  [229] = 229,
  [230] = 230,
  [231] = 231,
  [232] = 232,
  [233] = 233,
  [234] = 234,
  [235] = 235,
  [236] = 236,
  [237] = 237,
  [238] = 238,
  [239] = 239,
  [240] = 240,
  [241] = 241,
  [242] = 242,
  [243] = 243,
  [244] = 244,
  [245] = 245,
  [246] = 246,
  [247] = 247,
  [248] = 248,
  [249] = 249,
  [250] = 250,
  [251] = 197,
  [252] = 252,
  [253] = 253,
  [254] = 254,
  [255] = 255,
  [256] = 256,
  [257] = 257,
  [258] = 258,
  [259] = 259,
  [260] = 260,
  [261] = 261,
  [262] = 262,
  [263] = 263,
  [264] = 264,
  [265] = 265,
  [266] = 196,
  [267] = 267,
  [268] = 259,
  [269] = 269,
  [270] = 270,
  [271] = 271,
  [272] = 272,
  [273] = 214,
  [274] = 244,
  [275] = 275,
  [276] = 276,
  [277] = 277,
  [278] = 278,
  [279] = 279,
  [280] = 280,
  [281] = 281,
  [282] = 282,
  [283] = 283,
  [284] = 249,
  [285] = 285,
  [286] = 286,
  [287] = 255,
  [288] = 256,
  [289] = 289,
  [290] = 290,
  [291] = 291,
  [292] = 292,
  [293] = 257,
  [294] = 258,
  [295] = 295,
  [296] = 296,
  [297] = 297,
  [298] = 254,
  [299] = 208,
  [300] = 226,
  [301] = 227,
  [302] = 228,
  [303] = 229,
  [304] = 230,
  [305] = 305,
  [306] = 306,
  [307] = 307,
  [308] = 213,
  [309] = 309,
  [310] = 310,
  [311] = 252,
  [312] = 312,
  [313] = 260,
  [314] = 261,
  [315] = 315,
  [316] = 316,
  [317] = 231,
  [318] = 318,
  [319] = 319,
  [320] = 232,
  [321] = 262,
  [322] = 322,
  [323] = 323,
  [324] = 233,
  [325] = 234,
  [326] = 235,
  [327] = 327,
  [328] = 267,
  [329] = 329,
  [330] = 237,
  [331] = 263,
  [332] = 332,
  [333] = 333,
  [334] = 212,
  [335] = 335,
  [336] = 336,
  [337] = 337,
  [338] = 338,
  [339] = 339,
  [340] = 215,
  [341] = 264,
  [342] = 265,
  [343] = 203,
  [344] = 216,
  [345] = 345,
  [346] = 204,
  [347] = 206,
  [348] = 205,
  [349] = 207,
  [350] = 350,
  [351] = 351,
  [352] = 352,
  [353] = 217,
  [354] = 218,
  [355] = 238,
  [356] = 356,
  [357] = 239,
  [358] = 240,
  [359] = 241,
  [360] = 242,
  [361] = 243,
  [362] = 219,
  [363] = 363,
  [364] = 364,
  [365] = 365,
  [366] = 366,
  [367] = 367,
  [368] = 368,
  [369] = 369,
  [370] = 370,
  [371] = 133,
  [372] = 134,
  [373] = 373,
  [374] = 374,
  [375] = 375,
  [376] = 250,
  [377] = 246,
  [378] = 247,
  [379] = 379,
  [380] = 245,
  [381] = 381,
  [382] = 248,
  [383] = 221,
  [384] = 222,
  [385] = 223,
  [386] = 224,
  [387] = 225,
  [388] = 236,
  [389] = 209,
  [390] = 253,
  [391] = 391,
  [392] = 210,
  [393] = 211,
  [394] = 394,
  [395] = 322,
  [396] = 327,
  [397] = 364,
  [398] = 319,
  [399] = 323,
  [400] = 365,
  [401] = 305,
  [402] = 306,
  [403] = 350,
  [404] = 307,
  [405] = 366,
  [406] = 351,
  [407] = 367,
  [408] = 289,
  [409] = 290,
  [410] = 291,
  [411] = 368,
  [412] = 369,
  [413] = 370,
  [414] = 373,
  [415] = 292,
  [416] = 379,
  [417] = 381,
  [418] = 391,
  [419] = 332,
  [420] = 374,
  [421] = 345,
  [422] = 375,
  [423] = 309,
  [424] = 310,
  [425] = 339,
  [426] = 316,
  [427] = 295,
  [428] = 296,
  [429] = 363,
  [430] = 318,
  [431] = 394,
  [432] = 312,
  [433] = 315,
  [434] = 333,
  [435] = 272,
  [436] = 279,
  [437] = 281,
  [438] = 282,
  [439] = 329,
  [440] = 335,
  [441] = 336,
  [442] = 271,
  [443] = 337,
  [444] = 338,
  [445] = 283,
  [446] = 280,
  [447] = 352,
  [448] = 356,
  [449] = 269,
  [450] = 270,
  [451] = 275,
  [452] = 276,
  [453] = 133,
  [454] = 134,
  [455] = 277,
  [456] = 278,
  [457] = 286,
  [458] = 285,
  [459] = 297,
  [460] = 199,
  [461] = 198,
  [462] = 201,
  [463] = 249,
  [464] = 238,
  [465] = 239,
  [466] = 240,
  [467] = 241,
  [468] = 242,
  [469] = 243,
  [470] = 219,
  [471] = 258,
  [472] = 259,
  [473] = 260,
  [474] = 261,
  [475] = 262,
  [476] = 254,
  [477] = 263,
  [478] = 264,
  [479] = 255,
  [480] = 265,
  [481] = 196,
  [482] = 221,
  [483] = 222,
  [484] = 223,
  [485] = 224,
  [486] = 225,
  [487] = 236,
  [488] = 256,
  [489] = 209,
  [490] = 210,
  [491] = 218,
  [492] = 212,
  [493] = 213,
  [494] = 252,
  [495] = 214,
  [496] = 257,
  [497] = 208,
  [498] = 226,
  [499] = 227,
  [500] = 228,
  [501] = 229,
  [502] = 230,
  [503] = 231,
  [504] = 232,
  [505] = 233,
  [506] = 234,
  [507] = 235,
  [508] = 267,
  [509] = 237,
  [510] = 215,
  [511] = 216,
  [512] = 217,
  [513] = 245,
  [514] = 246,
  [515] = 247,
  [516] = 248,
  [517] = 250,
  [518] = 197,
  [519] = 253,
  [520] = 211,
  [521] = 278,
  [522] = 309,
  [523] = 310,
  [524] = 316,
  [525] = 351,
  [526] = 275,
  [527] = 276,
  [528] = 277,
  [529] = 327,
  [530] = 530,
  [531] = 531,
  [532] = 532,
  [533] = 286,
  [534] = 356,
  [535] = 535,
  [536] = 379,
  [537] = 381,
  [538] = 391,
  [539] = 352,
  [540] = 335,
  [541] = 336,
  [542] = 337,
  [543] = 338,
  [544] = 332,
  [545] = 269,
  [546] = 270,
  [547] = 305,
  [548] = 306,
  [549] = 307,
  [550] = 204,
  [551] = 206,
  [552] = 205,
  [553] = 207,
  [554] = 554,
  [555] = 350,
  [556] = 530,
  [557] = 535,
  [558] = 554,
  [559] = 531,
  [560] = 532,
  [561] = 530,
  [562] = 535,
  [563] = 322,
  [564] = 203,
  [565] = 554,
  [566] = 531,
  [567] = 532,
  [568] = 345,
  [569] = 329,
  [570] = 50,
  [571] = 50,
  [572] = 50,
  [573] = 50,
  [574] = 75,
  [575] = 78,
  [576] = 77,
  [577] = 63,
  [578] = 64,
  [579] = 76,
  [580] = 62,
  [581] = 66,
  [582] = 79,
  [583] = 67,
  [584] = 68,
  [585] = 69,
  [586] = 70,
  [587] = 59,
  [588] = 71,
  [589] = 65,
  [590] = 72,
  [591] = 73,
  [592] = 74,
  [593] = 80,
  [594] = 103,
  [595] = 87,
  [596] = 105,
  [597] = 106,
  [598] = 108,
  [599] = 88,
  [600] = 89,
  [601] = 109,
  [602] = 97,
  [603] = 98,
  [604] = 130,
  [605] = 132,
  [606] = 125,
  [607] = 607,
  [608] = 118,
  [609] = 133,
  [610] = 134,
  [611] = 120,
  [612] = 127,
  [613] = 123,
  [614] = 614,
  [615] = 129,
  [616] = 607,
  [617] = 614,
  [618] = 112,
  [619] = 110,
  [620] = 614,
  [621] = 124,
  [622] = 607,
  [623] = 614,
  [624] = 607,
  [625] = 614,
  [626] = 607,
  [627] = 627,
  [628] = 614,
  [629] = 607,
  [630] = 126,
  [631] = 631,
  [632] = 109,
  [633] = 633,
  [634] = 634,
  [635] = 635,
  [636] = 636,
  [637] = 637,
  [638] = 638,
  [639] = 636,
  [640] = 640,
  [641] = 637,
  [642] = 636,
  [643] = 643,
  [644] = 637,
  [645] = 637,
  [646] = 637,
  [647] = 637,
  [648] = 636,
  [649] = 636,
  [650] = 636,
  [651] = 651,
  [652] = 652,
  [653] = 653,
  [654] = 654,
  [655] = 655,
  [656] = 656,
  [657] = 657,
  [658] = 658,
  [659] = 659,
  [660] = 660,
  [661] = 661,
  [662] = 662,
  [663] = 663,
  [664] = 664,
  [665] = 665,
  [666] = 664,
  [667] = 667,
  [668] = 664,
  [669] = 669,
  [670] = 670,
  [671] = 671,
  [672] = 672,
  [673] = 672,
  [674] = 674,
  [675] = 672,
  [676] = 672,
  [677] = 674,
  [678] = 672,
  [679] = 679,
  [680] = 679,
  [681] = 672,
  [682] = 679,
  [683] = 679,
  [684] = 679,
  [685] = 679,
  [686] = 679,
  [687] = 674,
  [688] = 688,
  [689] = 689,
  [690] = 688,
  [691] = 691,
  [692] = 692,
  [693] = 693,
  [694] = 689,
  [695] = 688,
  [696] = 688,
  [697] = 689,
  [698] = 689,
  [699] = 688,
  [700] = 688,
  [701] = 701,
  [702] = 689,
  [703] = 703,
  [704] = 704,
  [705] = 705,
  [706] = 706,
  [707] = 689,
  [708] = 689,
  [709] = 688,
  [710] = 710,
  [711] = 711,
  [712] = 712,
  [713] = 713,
  [714] = 714,
  [715] = 713,
  [716] = 711,
  [717] = 717,
  [718] = 711,
  [719] = 719,
  [720] = 714,
  [721] = 721,
  [722] = 717,
  [723] = 711,
  [724] = 711,
  [725] = 711,
  [726] = 710,
  [727] = 712,
  [728] = 728,
  [729] = 710,
  [730] = 712,
  [731] = 731,
  [732] = 732,
  [733] = 733,
  [734] = 734,
  [735] = 735,
  [736] = 735,
  [737] = 737,
  [738] = 731,
  [739] = 739,
  [740] = 740,
  [741] = 741,
  [742] = 742,
  [743] = 737,
  [744] = 744,
  [745] = 745,
  [746] = 746,
  [747] = 747,
  [748] = 744,
  [749] = 745,
  [750] = 746,
  [751] = 751,
  [752] = 752,
  [753] = 747,
  [754] = 735,
  [755] = 731,
  [756] = 739,
  [757] = 740,
  [758] = 741,
  [759] = 742,
  [760] = 737,
  [761] = 744,
  [762] = 745,
  [763] = 746,
  [764] = 747,
  [765] = 765,
  [766] = 752,
  [767] = 744,
  [768] = 745,
  [769] = 746,
  [770] = 747,
  [771] = 771,
  [772] = 772,
  [773] = 773,
  [774] = 735,
  [775] = 775,
  [776] = 776,
  [777] = 752,
  [778] = 752,
  [779] = 779,
  [780] = 731,
  [781] = 781,
  [782] = 782,
  [783] = 783,
  [784] = 784,
  [785] = 785,
  [786] = 786,
  [787] = 775,
  [788] = 781,
  [789] = 789,
  [790] = 790,
  [791] = 791,
  [792] = 746,
  [793] = 793,
  [794] = 785,
  [795] = 731,
  [796] = 739,
  [797] = 735,
  [798] = 752,
  [799] = 752,
  [800] = 800,
  [801] = 785,
  [802] = 786,
  [803] = 775,
  [804] = 804,
  [805] = 781,
  [806] = 806,
  [807] = 807,
  [808] = 786,
  [809] = 747,
  [810] = 810,
  [811] = 811,
  [812] = 812,
  [813] = 739,
  [814] = 740,
  [815] = 741,
  [816] = 742,
  [817] = 817,
  [818] = 739,
  [819] = 737,
  [820] = 740,
  [821] = 752,
  [822] = 775,
  [823] = 744,
  [824] = 781,
  [825] = 745,
  [826] = 741,
  [827] = 752,
  [828] = 775,
  [829] = 781,
  [830] = 742,
  [831] = 752,
  [832] = 775,
  [833] = 781,
  [834] = 731,
  [835] = 737,
  [836] = 739,
  [837] = 740,
  [838] = 741,
  [839] = 752,
  [840] = 781,
  [841] = 752,
  [842] = 811,
  [843] = 784,
  [844] = 740,
  [845] = 741,
  [846] = 742,
  [847] = 765,
  [848] = 806,
  [849] = 773,
  [850] = 776,
  [851] = 789,
  [852] = 807,
  [853] = 779,
  [854] = 735,
  [855] = 742,
  [856] = 731,
  [857] = 739,
  [858] = 740,
  [859] = 741,
  [860] = 742,
  [861] = 737,
  [862] = 744,
  [863] = 745,
  [864] = 746,
  [865] = 747,
  [866] = 737,
  [867] = 775,
  [868] = 752,
  [869] = 811,
  [870] = 784,
  [871] = 752,
  [872] = 872,
  [873] = 744,
  [874] = 765,
  [875] = 806,
  [876] = 773,
  [877] = 776,
  [878] = 789,
  [879] = 807,
  [880] = 779,
  [881] = 745,
  [882] = 765,
  [883] = 776,
  [884] = 746,
  [885] = 765,
  [886] = 776,
  [887] = 735,
  [888] = 765,
  [889] = 776,
  [890] = 747,
  [891] = 765,
  [892] = 772,
  [893] = 782,
  [894] = 783,
  [895] = 804,
  [896] = 772,
  [897] = 782,
  [898] = 783,
  [899] = 804,
  [900] = 783,
  [901] = 783,
  [902] = 783,
  [903] = 771,
  [904] = 771,
  [905] = 905,
  [906] = 906,
  [907] = 906,
  [908] = 908,
  [909] = 908,
  [910] = 906,
  [911] = 908,
  [912] = 906,
  [913] = 908,
  [914] = 906,
  [915] = 908,
  [916] = 916,
  [917] = 906,
  [918] = 908,
  [919] = 919,
  [920] = 920,
  [921] = 921,
  [922] = 922,
  [923] = 923,
  [924] = 924,
  [925] = 925,
  [926] = 926,
  [927] = 927,
  [928] = 928,
  [929] = 929,
  [930] = 930,
  [931] = 931,
  [932] = 932,
  [933] = 933,
  [934] = 927,
  [935] = 935,
  [936] = 936,
  [937] = 937,
  [938] = 927,
  [939] = 939,
  [940] = 940,
  [941] = 941,
  [942] = 942,
  [943] = 943,
  [944] = 944,
  [945] = 927,
  [946] = 946,
  [947] = 925,
  [948] = 925,
  [949] = 949,
  [950] = 950,
  [951] = 927,
  [952] = 925,
  [953] = 925,
  [954] = 954,
  [955] = 955,
  [956] = 956,
  [957] = 957,
  [958] = 927,
  [959] = 959,
  [960] = 925,
  [961] = 134,
  [962] = 133,
  [963] = 963,
  [964] = 964,
  [965] = 965,
  [966] = 966,
  [967] = 967,
  [968] = 968,
  [969] = 969,
  [970] = 970,
  [971] = 971,
  [972] = 972,
  [973] = 973,
  [974] = 974,
  [975] = 975,
  [976] = 976,
  [977] = 976,
  [978] = 976,
  [979] = 979,
  [980] = 980,
  [981] = 981,
  [982] = 982,
  [983] = 979,
  [984] = 984,
  [985] = 979,
  [986] = 986,
  [987] = 980,
  [988] = 981,
  [989] = 984,
  [990] = 990,
  [991] = 990,
  [992] = 986,
  [993] = 993,
  [994] = 993,
  [995] = 982,
  [996] = 982,
  [997] = 979,
  [998] = 980,
  [999] = 981,
  [1000] = 984,
  [1001] = 990,
  [1002] = 986,
  [1003] = 993,
  [1004] = 1004,
  [1005] = 79,
  [1006] = 1006,
  [1007] = 1007,
  [1008] = 1007,
  [1009] = 1009,
  [1010] = 1010,
  [1011] = 1011,
  [1012] = 203,
  [1013] = 1011,
  [1014] = 1014,
  [1015] = 1015,
  [1016] = 206,
  [1017] = 205,
  [1018] = 207,
  [1019] = 1010,
  [1020] = 1011,
  [1021] = 1021,
  [1022] = 1022,
  [1023] = 1010,
  [1024] = 1011,
  [1025] = 1025,
  [1026] = 1026,
  [1027] = 1027,
  [1028] = 1010,
  [1029] = 1029,
  [1030] = 1030,
  [1031] = 1031,
  [1032] = 1032,
  [1033] = 1033,
  [1034] = 1034,
  [1035] = 1035,
  [1036] = 1036,
  [1037] = 1037,
  [1038] = 280,
  [1039] = 271,
  [1040] = 394,
  [1041] = 1041,
  [1042] = 77,
  [1043] = 64,
  [1044] = 74,
  [1045] = 66,
  [1046] = 63,
  [1047] = 108,
  [1048] = 267,
  [1049] = 237,
  [1050] = 240,
  [1051] = 72,
  [1052] = 1052,
  [1053] = 73,
  [1054] = 228,
  [1055] = 238,
  [1056] = 78,
  [1057] = 229,
  [1058] = 1058,
  [1059] = 1059,
  [1060] = 230,
  [1061] = 1058,
  [1062] = 1062,
  [1063] = 231,
  [1064] = 223,
  [1065] = 75,
  [1066] = 109,
  [1067] = 224,
  [1068] = 208,
  [1069] = 97,
  [1070] = 70,
  [1071] = 222,
  [1072] = 241,
  [1073] = 239,
  [1074] = 242,
  [1075] = 226,
  [1076] = 105,
  [1077] = 225,
  [1078] = 243,
  [1079] = 71,
  [1080] = 233,
  [1081] = 65,
  [1082] = 234,
  [1083] = 235,
  [1084] = 1059,
  [1085] = 68,
  [1086] = 69,
  [1087] = 59,
  [1088] = 98,
  [1089] = 221,
  [1090] = 62,
  [1091] = 80,
  [1092] = 87,
  [1093] = 106,
  [1094] = 67,
  [1095] = 1058,
  [1096] = 1059,
  [1097] = 76,
  [1098] = 227,
  [1099] = 103,
  [1100] = 232,
  [1101] = 1101,
  [1102] = 89,
  [1103] = 118,
  [1104] = 68,
  [1105] = 69,
  [1106] = 70,
  [1107] = 59,
  [1108] = 71,
  [1109] = 72,
  [1110] = 1110,
  [1111] = 1111,
  [1112] = 127,
  [1113] = 73,
  [1114] = 74,
  [1115] = 130,
  [1116] = 75,
  [1117] = 76,
  [1118] = 77,
  [1119] = 78,
  [1120] = 1110,
  [1121] = 110,
  [1122] = 124,
  [1123] = 1123,
  [1124] = 1123,
  [1125] = 63,
  [1126] = 126,
  [1127] = 64,
  [1128] = 1128,
  [1129] = 88,
  [1130] = 1130,
  [1131] = 1123,
  [1132] = 125,
  [1133] = 129,
  [1134] = 79,
  [1135] = 66,
  [1136] = 65,
  [1137] = 1137,
  [1138] = 62,
  [1139] = 1137,
  [1140] = 1101,
  [1141] = 1128,
  [1142] = 1111,
  [1143] = 1137,
  [1144] = 1101,
  [1145] = 1128,
  [1146] = 1111,
  [1147] = 1128,
  [1148] = 1111,
  [1149] = 1128,
  [1150] = 1111,
  [1151] = 1128,
  [1152] = 1111,
  [1153] = 67,
  [1154] = 112,
  [1155] = 120,
  [1156] = 132,
  [1157] = 123,
  [1158] = 71,
  [1159] = 1159,
  [1160] = 1160,
  [1161] = 1161,
  [1162] = 88,
  [1163] = 89,
  [1164] = 1164,
  [1165] = 1165,
  [1166] = 1166,
  [1167] = 1167,
  [1168] = 1168,
  [1169] = 1169,
  [1170] = 1170,
  [1171] = 87,
  [1172] = 1161,
  [1173] = 1173,
  [1174] = 98,
  [1175] = 97,
  [1176] = 1176,
  [1177] = 1177,
  [1178] = 1178,
  [1179] = 1179,
  [1180] = 1169,
  [1181] = 105,
  [1182] = 106,
  [1183] = 1161,
  [1184] = 1173,
  [1185] = 1178,
  [1186] = 1186,
  [1187] = 1176,
  [1188] = 1177,
  [1189] = 1178,
  [1190] = 1179,
  [1191] = 1169,
  [1192] = 1192,
  [1193] = 1193,
  [1194] = 1179,
  [1195] = 1195,
  [1196] = 1196,
  [1197] = 1176,
  [1198] = 1173,
  [1199] = 1199,
  [1200] = 1177,
  [1201] = 1173,
  [1202] = 1202,
  [1203] = 80,
  [1204] = 1173,
  [1205] = 67,
  [1206] = 68,
  [1207] = 69,
  [1208] = 70,
  [1209] = 59,
  [1210] = 1173,
  [1211] = 72,
  [1212] = 73,
  [1213] = 75,
  [1214] = 76,
  [1215] = 78,
  [1216] = 65,
  [1217] = 109,
  [1218] = 62,
  [1219] = 103,
  [1220] = 1220,
  [1221] = 108,
  [1222] = 1173,
  [1223] = 1223,
  [1224] = 1224,
  [1225] = 1223,
  [1226] = 130,
  [1227] = 1227,
  [1228] = 118,
  [1229] = 1229,
  [1230] = 132,
  [1231] = 1231,
  [1232] = 1232,
  [1233] = 125,
  [1234] = 123,
  [1235] = 1235,
  [1236] = 1236,
  [1237] = 1237,
  [1238] = 1238,
  [1239] = 1239,
  [1240] = 133,
  [1241] = 1232,
  [1242] = 1242,
  [1243] = 1242,
  [1244] = 1244,
  [1245] = 126,
  [1246] = 1246,
  [1247] = 1232,
  [1248] = 1242,
  [1249] = 1249,
  [1250] = 1232,
  [1251] = 1242,
  [1252] = 1232,
  [1253] = 1242,
  [1254] = 1246,
  [1255] = 1244,
  [1256] = 1256,
  [1257] = 1232,
  [1258] = 1242,
  [1259] = 110,
  [1260] = 124,
  [1261] = 120,
  [1262] = 112,
  [1263] = 1227,
  [1264] = 1264,
  [1265] = 1223,
  [1266] = 1235,
  [1267] = 1236,
  [1268] = 88,
  [1269] = 89,
  [1270] = 127,
  [1271] = 1264,
  [1272] = 1264,
  [1273] = 1235,
  [1274] = 1236,
  [1275] = 1235,
  [1276] = 1235,
  [1277] = 1235,
  [1278] = 134,
  [1279] = 129,
  [1280] = 1227,
  [1281] = 1281,
  [1282] = 65,
  [1283] = 65,
  [1284] = 1284,
  [1285] = 1284,
  [1286] = 1286,
  [1287] = 65,
  [1288] = 1284,
  [1289] = 65,
  [1290] = 1286,
  [1291] = 65,
  [1292] = 1286,
  [1293] = 1286,
  [1294] = 65,
  [1295] = 1286,
  [1296] = 1286,
  [1297] = 1286,
  [1298] = 1298,
  [1299] = 1299,
  [1300] = 1300,
  [1301] = 1301,
  [1302] = 1302,
  [1303] = 1303,
  [1304] = 1304,
  [1305] = 1305,
  [1306] = 1306,
  [1307] = 1307,
  [1308] = 1308,
  [1309] = 1309,
  [1310] = 1310,
  [1311] = 1311,
  [1312] = 1312,
  [1313] = 1313,
  [1314] = 1314,
  [1315] = 1315,
  [1316] = 1316,
  [1317] = 1317,
  [1318] = 1318,
  [1319] = 1319,
  [1320] = 1315,
  [1321] = 1318,
  [1322] = 1322,
  [1323] = 1323,
  [1324] = 1324,
  [1325] = 1325,
  [1326] = 1326,
  [1327] = 1327,
  [1328] = 1328,
  [1329] = 1329,
  [1330] = 1330,
  [1331] = 1331,
  [1332] = 1332,
  [1333] = 1333,
  [1334] = 1334,
  [1335] = 1335,
  [1336] = 1336,
  [1337] = 1337,
  [1338] = 1338,
  [1339] = 1339,
  [1340] = 1340,
  [1341] = 1341,
  [1342] = 1342,
  [1343] = 1340,
  [1344] = 1339,
  [1345] = 1345,
  [1346] = 1345,
  [1347] = 1347,
  [1348] = 1347,
  [1349] = 203,
  [1350] = 207,
  [1351] = 197,
  [1352] = 196,
  [1353] = 204,
  [1354] = 206,
  [1355] = 205,
  [1356] = 1356,
  [1357] = 1357,
  [1358] = 1358,
  [1359] = 1359,
  [1360] = 1360,
  [1361] = 1361,
  [1362] = 1362,
  [1363] = 1363,
  [1364] = 1364,
  [1365] = 1365,
  [1366] = 1366,
  [1367] = 1367,
  [1368] = 1368,
  [1369] = 1369,
  [1370] = 1041,
  [1371] = 1371,
  [1372] = 1372,
  [1373] = 1373,
  [1374] = 1369,
  [1375] = 1041,
  [1376] = 1372,
  [1377] = 1377,
  [1378] = 1377,
  [1379] = 1377,
  [1380] = 1380,
  [1381] = 1380,
  [1382] = 1371,
  [1383] = 1373,
  [1384] = 1377,
  [1385] = 1380,
  [1386] = 1380,
  [1387] = 1387,
  [1388] = 1388,
  [1389] = 1389,
  [1390] = 1390,
  [1391] = 1391,
  [1392] = 1392,
  [1393] = 1393,
  [1394] = 1394,
  [1395] = 1395,
  [1396] = 1396,
  [1397] = 1397,
  [1398] = 1398,
  [1399] = 1399,
  [1400] = 1400,
  [1401] = 1401,
  [1402] = 1402,
  [1403] = 1403,
  [1404] = 1404,
  [1405] = 1405,
  [1406] = 1406,
  [1407] = 1407,
  [1408] = 1408,
  [1409] = 1409,
  [1410] = 1410,
  [1411] = 1411,
  [1412] = 1412,
  [1413] = 1413,
  [1414] = 1414,
  [1415] = 1415,
  [1416] = 1416,
  [1417] = 1417,
  [1418] = 1418,
  [1419] = 1419,
  [1420] = 1420,
  [1421] = 1421,
  [1422] = 1422,
  [1423] = 1423,
  [1424] = 1424,
  [1425] = 1425,
  [1426] = 1426,
  [1427] = 1427,
  [1428] = 1428,
  [1429] = 1429,
  [1430] = 1430,
  [1431] = 1431,
  [1432] = 1432,
  [1433] = 1433,
  [1434] = 1434,
  [1435] = 1435,
  [1436] = 1436,
  [1437] = 1437,
  [1438] = 1438,
  [1439] = 1439,
  [1440] = 1440,
  [1441] = 1441,
  [1442] = 1062,
  [1443] = 1052,
  [1444] = 1444,
  [1445] = 1445,
  [1446] = 1424,
  [1447] = 1440,
  [1448] = 1429,
  [1449] = 1052,
  [1450] = 1444,
  [1451] = 1428,
  [1452] = 1062,
  [1453] = 1439,
  [1454] = 1454,
  [1455] = 1454,
  [1456] = 1454,
  [1457] = 1454,
  [1458] = 1458,
  [1459] = 1459,
  [1460] = 1460,
  [1461] = 1461,
  [1462] = 1462,
  [1463] = 1463,
  [1464] = 1464,
  [1465] = 1465,
  [1466] = 1466,
  [1467] = 1467,
  [1468] = 1468,
  [1469] = 1469,
  [1470] = 1470,
  [1471] = 1471,
  [1472] = 1472,
  [1473] = 1473,
  [1474] = 1474,
  [1475] = 1475,
  [1476] = 1476,
  [1477] = 1477,
  [1478] = 1478,
  [1479] = 1479,
  [1480] = 1480,
  [1481] = 1481,
  [1482] = 1482,
  [1483] = 1483,
  [1484] = 1484,
  [1485] = 1485,
  [1486] = 1486,
  [1487] = 1487,
  [1488] = 1488,
  [1489] = 1489,
  [1490] = 1490,
  [1491] = 1491,
  [1492] = 1492,
  [1493] = 1493,
  [1494] = 1494,
  [1495] = 1495,
  [1496] = 1496,
  [1497] = 1497,
  [1498] = 1498,
  [1499] = 1499,
  [1500] = 1500,
  [1501] = 1501,
  [1502] = 1502,
  [1503] = 1503,
  [1504] = 1504,
  [1505] = 1505,
  [1506] = 1506,
  [1507] = 1507,
  [1508] = 1508,
  [1509] = 1509,
  [1510] = 1510,
  [1511] = 1511,
  [1512] = 1512,
  [1513] = 1513,
  [1514] = 1514,
  [1515] = 1515,
  [1516] = 1516,
  [1517] = 1517,
  [1518] = 1518,
  [1519] = 1519,
  [1520] = 1520,
  [1521] = 1521,
  [1522] = 1522,
  [1523] = 1523,
  [1524] = 1524,
  [1525] = 1500,
  [1526] = 1526,
  [1527] = 1527,
  [1528] = 1528,
  [1529] = 1529,
  [1530] = 1530,
  [1531] = 1531,
  [1532] = 1532,
  [1533] = 1533,
  [1534] = 1534,
  [1535] = 1535,
  [1536] = 1536,
  [1537] = 1537,
  [1538] = 1538,
  [1539] = 1539,
  [1540] = 1540,
  [1541] = 1541,
  [1542] = 1542,
  [1543] = 1543,
  [1544] = 1544,
  [1545] = 1545,
  [1546] = 1546,
  [1547] = 1547,
  [1548] = 1548,
  [1549] = 1549,
  [1550] = 1550,
  [1551] = 1551,
  [1552] = 1552,
  [1553] = 1553,
  [1554] = 1554,
  [1555] = 1555,
  [1556] = 1556,
  [1557] = 1557,
  [1558] = 1558,
  [1559] = 1559,
  [1560] = 1560,
  [1561] = 1561,
  [1562] = 1562,
  [1563] = 1563,
  [1564] = 280,
  [1565] = 1565,
  [1566] = 1566,
  [1567] = 1567,
  [1568] = 1568,
  [1569] = 1569,
  [1570] = 1570,
  [1571] = 1571,
  [1572] = 1572,
  [1573] = 1573,
  [1574] = 1574,
  [1575] = 1575,
  [1576] = 1576,
  [1577] = 1577,
  [1578] = 1578,
  [1579] = 1579,
  [1580] = 1580,
  [1581] = 1581,
  [1582] = 1582,
  [1583] = 1583,
  [1584] = 1584,
  [1585] = 1585,
  [1586] = 1586,
  [1587] = 1587,
  [1588] = 394,
  [1589] = 1589,
  [1590] = 1590,
  [1591] = 1591,
  [1592] = 1592,
  [1593] = 1593,
  [1594] = 1594,
  [1595] = 1595,
  [1596] = 1596,
  [1597] = 1597,
  [1598] = 271,
  [1599] = 1599,
  [1600] = 1600,
  [1601] = 1601,
  [1602] = 1602,
  [1603] = 1603,
  [1604] = 1604,
  [1605] = 1605,
  [1606] = 1606,
  [1607] = 1607,
  [1608] = 1608,
  [1609] = 1609,
  [1610] = 1610,
  [1611] = 1611,
  [1612] = 1612,
  [1613] = 1613,
  [1614] = 1614,
  [1615] = 1615,
  [1616] = 1616,
  [1617] = 1617,
  [1618] = 1618,
  [1619] = 1619,
  [1620] = 1620,
  [1621] = 1621,
  [1622] = 1622,
  [1623] = 1623,
  [1624] = 1624,
  [1625] = 1625,
  [1626] = 1626,
  [1627] = 1627,
  [1628] = 1628,
  [1629] = 1629,
  [1630] = 1630,
  [1631] = 1631,
  [1632] = 1632,
  [1633] = 1633,
  [1634] = 1634,
  [1635] = 1635,
  [1636] = 1636,
  [1637] = 1637,
  [1638] = 1638,
  [1639] = 1639,
  [1640] = 1640,
  [1641] = 1641,
  [1642] = 1642,
  [1643] = 1643,
  [1644] = 1644,
  [1645] = 1645,
  [1646] = 1646,
  [1647] = 1647,
  [1648] = 1648,
  [1649] = 1649,
  [1650] = 1650,
  [1651] = 1651,
  [1652] = 1652,
  [1653] = 1653,
  [1654] = 1654,
  [1655] = 1655,
  [1656] = 1656,
  [1657] = 1657,
  [1658] = 1658,
  [1659] = 1659,
  [1660] = 1660,
  [1661] = 1661,
  [1662] = 1641,
  [1663] = 1663,
  [1664] = 1664,
  [1665] = 1665,
  [1666] = 1666,
  [1667] = 1667,
  [1668] = 1668,
  [1669] = 1669,
  [1670] = 1670,
  [1671] = 1671,
  [1672] = 1672,
  [1673] = 1673,
  [1674] = 1674,
  [1675] = 1675,
  [1676] = 1676,
  [1677] = 1677,
  [1678] = 1678,
  [1679] = 1679,
  [1680] = 1680,
  [1681] = 1681,
  [1682] = 1682,
  [1683] = 1683,
  [1684] = 1684,
  [1685] = 1685,
  [1686] = 1686,
  [1687] = 1687,
  [1688] = 1688,
  [1689] = 1689,
  [1690] = 1690,
  [1691] = 1691,
  [1692] = 1692,
  [1693] = 1693,
  [1694] = 1694,
  [1695] = 1695,
  [1696] = 1696,
  [1697] = 1697,
  [1698] = 1698,
  [1699] = 1699,
  [1700] = 1700,
  [1701] = 1701,
  [1702] = 1702,
  [1703] = 1703,
  [1704] = 1704,
  [1705] = 1705,
  [1706] = 1706,
  [1707] = 1707,
  [1708] = 1708,
  [1709] = 1709,
  [1710] = 1710,
  [1711] = 1711,
  [1712] = 1712,
  [1713] = 1713,
  [1714] = 1714,
  [1715] = 1715,
  [1716] = 1716,
  [1717] = 1711,
  [1718] = 1718,
  [1719] = 1719,
  [1720] = 1720,
  [1721] = 1710,
  [1722] = 1722,
  [1723] = 1723,
  [1724] = 1724,
  [1725] = 1711,
  [1726] = 1726,
  [1727] = 1712,
  [1728] = 1713,
  [1729] = 1729,
  [1730] = 1714,
  [1731] = 1715,
  [1732] = 1716,
  [1733] = 1733,
  [1734] = 1734,
  [1735] = 1726,
  [1736] = 1719,
  [1737] = 1737,
  [1738] = 1738,
  [1739] = 1710,
  [1740] = 1711,
  [1741] = 1712,
  [1742] = 1713,
  [1743] = 1714,
  [1744] = 1715,
  [1745] = 1716,
  [1746] = 1715,
  [1747] = 1722,
  [1748] = 1723,
  [1749] = 1724,
  [1750] = 1729,
  [1751] = 1751,
  [1752] = 1733,
  [1753] = 1714,
  [1754] = 1712,
  [1755] = 1755,
  [1756] = 1722,
  [1757] = 1757,
  [1758] = 1733,
  [1759] = 1723,
  [1760] = 1724,
  [1761] = 1718,
  [1762] = 1729,
  [1763] = 1716,
  [1764] = 1713,
  [1765] = 1729,
  [1766] = 1733,
  [1767] = 1729,
  [1768] = 1733,
  [1769] = 1729,
  [1770] = 1733,
  [1771] = 1737,
  [1772] = 1772,
  [1773] = 1722,
  [1774] = 1709,
  [1775] = 1738,
  [1776] = 1776,
  [1777] = 1757,
  [1778] = 1729,
  [1779] = 1779,
  [1780] = 1639,
  [1781] = 1655,
  [1782] = 1562,
  [1783] = 1776,
  [1784] = 1757,
  [1785] = 1733,
  [1786] = 1733,
  [1787] = 1723,
  [1788] = 1734,
  [1789] = 1724,
  [1790] = 1709,
  [1791] = 1738,
  [1792] = 1729,
  [1793] = 1726,
  [1794] = 1719,
  [1795] = 1720,
  [1796] = 1710,
  [1797] = 1737,
  [1798] = 1733,
  [1799] = 1734,
  [1800] = 1726,
  [1801] = 1776,
  [1802] = 1719,
  [1803] = 1737,
  [1804] = 1734,
  [1805] = 1729,
  [1806] = 1806,
  [1807] = 1807,
  [1808] = 1808,
  [1809] = 1809,
  [1810] = 1810,
  [1811] = 1811,
  [1812] = 1812,
  [1813] = 1811,
  [1814] = 1806,
  [1815] = 1806,
  [1816] = 1811,
  [1817] = 1817,
  [1818] = 1818,
  [1819] = 1806,
  [1820] = 1820,
  [1821] = 1807,
  [1822] = 1822,
  [1823] = 1823,
  [1824] = 1824,
  [1825] = 1825,
  [1826] = 1826,
  [1827] = 1827,
  [1828] = 1808,
  [1829] = 1706,
  [1830] = 1807,
  [1831] = 1820,
  [1832] = 1832,
  [1833] = 1808,
  [1834] = 1834,
  [1835] = 1823,
  [1836] = 1824,
  [1837] = 1837,
  [1838] = 1838,
  [1839] = 1811,
  [1840] = 1812,
  [1841] = 1808,
  [1842] = 1842,
  [1843] = 1843,
  [1844] = 1690,
  [1845] = 1845,
  [1846] = 1846,
  [1847] = 1812,
  [1848] = 1807,
  [1849] = 1693,
  [1850] = 1808,
  [1851] = 1822,
  [1852] = 1820,
  [1853] = 1684,
  [1854] = 1832,
  [1855] = 1824,
  [1856] = 1683,
  [1857] = 1672,
  [1858] = 1858,
  [1859] = 1807,
  [1860] = 1860,
  [1861] = 1861,
  [1862] = 1862,
  [1863] = 1817,
  [1864] = 1818,
  [1865] = 1807,
  [1866] = 1823,
  [1867] = 1823,
  [1868] = 1824,
  [1869] = 1694,
  [1870] = 1818,
  [1871] = 1824,
  [1872] = 1811,
  [1873] = 1832,
  [1874] = 1811,
  [1875] = 1875,
  [1876] = 1824,
  [1877] = 1817,
  [1878] = 1822,
  [1879] = 1879,
  [1880] = 1880,
  [1881] = 1881,
  [1882] = 1882,
  [1883] = 1883,
  [1884] = 1884,
  [1885] = 1883,
  [1886] = 1884,
  [1887] = 1887,
  [1888] = 1888,
  [1889] = 1889,
  [1890] = 1890,
  [1891] = 1891,
  [1892] = 1892,
  [1893] = 1893,
  [1894] = 1894,
  [1895] = 1895,
  [1896] = 1896,
  [1897] = 1897,
  [1898] = 1898,
  [1899] = 1899,
  [1900] = 1889,
  [1901] = 1901,
  [1902] = 1902,
  [1903] = 1893,
  [1904] = 1904,
  [1905] = 1905,
  [1906] = 1906,
  [1907] = 1907,
  [1908] = 1908,
  [1909] = 1909,
  [1910] = 1910,
  [1911] = 1911,
  [1912] = 1912,
  [1913] = 1913,
  [1914] = 1914,
  [1915] = 1915,
  [1916] = 1916,
  [1917] = 1905,
  [1918] = 1918,
  [1919] = 1919,
  [1920] = 1920,
  [1921] = 1921,
  [1922] = 1922,
  [1923] = 1913,
  [1924] = 1899,
  [1925] = 1881,
  [1926] = 1909,
  [1927] = 1883,
  [1928] = 1884,
  [1929] = 1919,
  [1930] = 1880,
  [1931] = 1881,
  [1932] = 1883,
  [1933] = 1884,
  [1934] = 1891,
  [1935] = 1880,
  [1936] = 1936,
  [1937] = 1879,
  [1938] = 1906,
  [1939] = 1904,
  [1940] = 1907,
  [1941] = 1910,
  [1942] = 1890,
  [1943] = 1891,
  [1944] = 1908,
  [1945] = 1894,
  [1946] = 1913,
  [1947] = 1883,
  [1948] = 1884,
  [1949] = 1949,
  [1950] = 1950,
  [1951] = 1895,
  [1952] = 1891,
  [1953] = 1918,
  [1954] = 1879,
  [1955] = 1894,
  [1956] = 1895,
  [1957] = 1879,
  [1958] = 1958,
  [1959] = 1913,
  [1960] = 1896,
  [1961] = 1911,
  [1962] = 1897,
  [1963] = 1963,
  [1964] = 1891,
  [1965] = 1891,
  [1966] = 1966,
  [1967] = 1444,
  [1968] = 1909,
  [1969] = 1969,
  [1970] = 1970,
  [1971] = 1919,
  [1972] = 1972,
  [1973] = 1973,
  [1974] = 1974,
  [1975] = 1901,
  [1976] = 1888,
  [1977] = 1977,
  [1978] = 1912,
  [1979] = 1881,
  [1980] = 1980,
  [1981] = 1440,
  [1982] = 1982,
  [1983] = 1983,
  [1984] = 1984,
  [1985] = 1985,
  [1986] = 1982,
  [1987] = 1983,
  [1988] = 1984,
  [1989] = 1913,
  [1990] = 1914,
  [1991] = 1991,
  [1992] = 1890,
  [1993] = 1898,
  [1994] = 1896,
  [1995] = 1969,
  [1996] = 1980,
  [1997] = 1970,
  [1998] = 1969,
  [1999] = 1970,
  [2000] = 1879,
  [2001] = 1949,
  [2002] = 1972,
  [2003] = 1973,
  [2004] = 1920,
  [2005] = 1972,
  [2006] = 1973,
  [2007] = 1974,
  [2008] = 1901,
  [2009] = 1888,
  [2010] = 1977,
  [2011] = 1897,
  [2012] = 1898,
  [2013] = 1982,
  [2014] = 1983,
  [2015] = 1984,
  [2016] = 1899,
  [2017] = 1889,
  [2018] = 1888,
  [2019] = 1977,
  [2020] = 2020,
  [2021] = 1915,
  [2022] = 1912,
  [2023] = 2023,
  [2024] = 1977,
  [2025] = 1913,
  [2026] = 1974,
  [2027] = 1914,
  [2028] = 2028,
  [2029] = 2029,
  [2030] = 1936,
  [2031] = 1916,
  [2032] = 1918,
  [2033] = 2028,
  [2034] = 1921,
  [2035] = 2028,
  [2036] = 2029,
  [2037] = 1936,
  [2038] = 1922,
  [2039] = 2039,
  [2040] = 1977,
  [2041] = 1428,
  [2042] = 1920,
  [2043] = 1424,
  [2044] = 1429,
  [2045] = 1921,
  [2046] = 1922,
  [2047] = 2029,
  [2048] = 1949,
  [2049] = 2049,
  [2050] = 2050,
  [2051] = 2051,
  [2052] = 2052,
  [2053] = 2053,
  [2054] = 2054,
  [2055] = 2052,
  [2056] = 2056,
  [2057] = 2057,
  [2058] = 2058,
  [2059] = 2059,
  [2060] = 2060,
  [2061] = 2061,
  [2062] = 2062,
  [2063] = 2063,
  [2064] = 2064,
  [2065] = 2065,
  [2066] = 2066,
  [2067] = 2067,
  [2068] = 2068,
  [2069] = 2069,
  [2070] = 2070,
  [2071] = 2071,
  [2072] = 2072,
  [2073] = 2073,
  [2074] = 2074,
  [2075] = 2075,
  [2076] = 2076,
  [2077] = 2077,
  [2078] = 2078,
  [2079] = 2079,
  [2080] = 2052,
  [2081] = 2081,
  [2082] = 2082,
  [2083] = 2083,
  [2084] = 2084,
  [2085] = 2085,
  [2086] = 2086,
  [2087] = 2087,
  [2088] = 2088,
  [2089] = 2089,
  [2090] = 2090,
  [2091] = 2091,
  [2092] = 2092,
  [2093] = 2061,
  [2094] = 2094,
  [2095] = 2095,
  [2096] = 2096,
  [2097] = 2097,
  [2098] = 2098,
  [2099] = 2099,
  [2100] = 2052,
  [2101] = 2101,
  [2102] = 2102,
  [2103] = 2103,
  [2104] = 2101,
  [2105] = 2091,
  [2106] = 2087,
  [2107] = 2107,
  [2108] = 2108,
  [2109] = 2109,
  [2110] = 133,
  [2111] = 2111,
  [2112] = 2112,
  [2113] = 2113,
  [2114] = 2114,
  [2115] = 2067,
  [2116] = 2078,
  [2117] = 2078,
  [2118] = 2118,
  [2119] = 2119,
  [2120] = 2120,
  [2121] = 2121,
  [2122] = 2122,
  [2123] = 2123,
  [2124] = 2061,
  [2125] = 2125,
  [2126] = 2126,
  [2127] = 2127,
  [2128] = 2128,
  [2129] = 2129,
  [2130] = 2130,
  [2131] = 2131,
  [2132] = 2132,
  [2133] = 2133,
  [2134] = 2134,
  [2135] = 2053,
  [2136] = 2054,
  [2137] = 2137,
  [2138] = 2138,
  [2139] = 2139,
  [2140] = 2140,
  [2141] = 2122,
  [2142] = 2142,
  [2143] = 2143,
  [2144] = 2144,
  [2145] = 2145,
  [2146] = 2146,
  [2147] = 2147,
  [2148] = 2052,
  [2149] = 2149,
  [2150] = 2150,
  [2151] = 2151,
  [2152] = 2152,
  [2153] = 2153,
  [2154] = 2154,
  [2155] = 2155,
  [2156] = 2156,
  [2157] = 2157,
  [2158] = 2158,
  [2159] = 2159,
  [2160] = 2053,
  [2161] = 2161,
  [2162] = 2162,
  [2163] = 2163,
  [2164] = 2078,
  [2165] = 2165,
  [2166] = 2166,
  [2167] = 2167,
  [2168] = 2168,
  [2169] = 2102,
  [2170] = 2112,
  [2171] = 2101,
  [2172] = 2091,
  [2173] = 2087,
  [2174] = 2174,
  [2175] = 2175,
  [2176] = 2114,
  [2177] = 2177,
  [2178] = 2061,
  [2179] = 2179,
  [2180] = 2061,
  [2181] = 2114,
  [2182] = 2182,
  [2183] = 2183,
  [2184] = 2184,
  [2185] = 2052,
  [2186] = 2186,
  [2187] = 2187,
  [2188] = 2122,
  [2189] = 2189,
  [2190] = 2190,
  [2191] = 2191,
  [2192] = 2192,
  [2193] = 2193,
  [2194] = 2194,
  [2195] = 2195,
  [2196] = 2196,
  [2197] = 2102,
  [2198] = 2198,
  [2199] = 2101,
  [2200] = 2200,
  [2201] = 2201,
  [2202] = 2202,
  [2203] = 2203,
  [2204] = 2204,
  [2205] = 2205,
  [2206] = 2206,
  [2207] = 2112,
  [2208] = 2102,
  [2209] = 2209,
  [2210] = 2101,
  [2211] = 2089,
  [2212] = 2212,
  [2213] = 2051,
  [2214] = 2052,
  [2215] = 2215,
  [2216] = 2067,
  [2217] = 2217,
  [2218] = 2218,
  [2219] = 2219,
  [2220] = 2220,
  [2221] = 2121,
  [2222] = 2150,
  [2223] = 2166,
  [2224] = 2168,
  [2225] = 2150,
  [2226] = 2061,
  [2227] = 2206,
  [2228] = 2102,
  [2229] = 2229,
  [2230] = 2230,
  [2231] = 2231,
  [2232] = 2232,
  [2233] = 2233,
  [2234] = 2234,
  [2235] = 2121,
  [2236] = 2236,
  [2237] = 2166,
  [2238] = 2168,
  [2239] = 2054,
  [2240] = 2240,
  [2241] = 2241,
  [2242] = 2242,
  [2243] = 2243,
  [2244] = 2244,
  [2245] = 2245,
  [2246] = 2246,
  [2247] = 2198,
  [2248] = 2248,
  [2249] = 2249,
  [2250] = 2250,
  [2251] = 2251,
  [2252] = 2198,
  [2253] = 2253,
  [2254] = 134,
  [2255] = 2051,
  [2256] = 2256,
  [2257] = 2257,
  [2258] = 2088,
  [2259] = 2259,
  [2260] = 2260,
  [2261] = 2206,
  [2262] = 2262,
  [2263] = 2263,
  [2264] = 2264,
};

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (eof) ADVANCE(589);
      ADVANCE_MAP(
        '!', 1242,
        '"', 1275,
        '#', 817,
        '$', 1229,
        '%', 1253,
        '&', 1258,
        '\'', 1277,
        '(', 1237,
        ')', 1239,
        '*', 1250,
        '+', 1248,
        ',', 720,
        '-', 1245,
        '.', 1272,
        '/', 1251,
        ':', 1236,
        ';', 1240,
        '<', 1265,
        '=', 709,
        '>', 1262,
        '[', 1232,
        ']', 1233,
        '^', 1257,
        '{', 1234,
        '|', 1256,
        '}', 1235,
        '~', 1243,
        'A', 2096,
        'a', 2096,
        'B', 2045,
        'b', 2045,
        'C', 1763,
        'c', 1763,
        'D', 1450,
        'd', 1450,
        'E', 2043,
        'e', 2043,
        'F', 1780,
        'f', 1780,
        'G', 1613,
        'g', 1613,
        'H', 1976,
        'h', 1976,
        'I', 1916,
        'i', 1916,
        'L', 1612,
        'l', 1612,
        'N', 1615,
        'n', 1615,
        'O', 1723,
        'o', 1723,
        'P', 2233,
        'p', 2233,
        'R', 1657,
        'r', 1657,
        'S', 1975,
        's', 1975,
        'T', 1782,
        't', 1782,
        'U', 2044,
        'u', 2044,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(0);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1289);
      if (('J' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('j' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      if (lookahead != 0) ADVANCE(1230);
      END_STATE();
    case 1:
      ADVANCE_MAP(
        '\n', 716,
        '!', 1241,
        '"', 1275,
        '#', 126,
        '$', 586,
        '\'', 1276,
        '(', 1238,
        '+', 1249,
        '-', 1246,
        '.', 585,
        '/', 25,
        '[', 1232,
        '{', 1234,
        '~', 1243,
        'A', 2096,
        'a', 2096,
        'B', 2045,
        'b', 2045,
        'C', 1764,
        'c', 1764,
        'D', 1510,
        'd', 1510,
        'E', 2089,
        'e', 2089,
        'F', 1780,
        'f', 1780,
        'G', 1709,
        'g', 1709,
        'I', 1974,
        'i', 1974,
        'L', 1612,
        'l', 1612,
        'N', 1694,
        'n', 1694,
        'O', 1729,
        'o', 1729,
        'P', 2233,
        'p', 2233,
        'R', 1657,
        'r', 1657,
        'S', 2013,
        's', 2013,
        'T', 1823,
        't', 1823,
        'U', 2093,
        'u', 2093,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(1);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1290);
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2:
      ADVANCE_MAP(
        '\n', 716,
        '!', 1241,
        '"', 1275,
        '#', 172,
        '$', 586,
        '\'', 1276,
        '(', 1238,
        '+', 1249,
        ',', 720,
        '-', 1246,
        '.', 585,
        '/', 25,
        '[', 1232,
        '{', 1234,
        '~', 1243,
        'A', 2096,
        'a', 2096,
        'C', 1873,
        'c', 1873,
        'D', 1492,
        'd', 1492,
        'E', 2089,
        'e', 2089,
        'F', 1780,
        'f', 1780,
        'G', 1709,
        'g', 1709,
        'I', 1942,
        'i', 1942,
        'N', 1694,
        'n', 1694,
        'S', 2014,
        's', 2014,
        'T', 1825,
        't', 1825,
        'U', 2093,
        'u', 2093,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(2);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1290);
      if (('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 3:
      ADVANCE_MAP(
        '\n', 716,
        '#', 126,
        '(', 18,
        '/', 25,
        'A', 559,
        'a', 559,
        'C', 389,
        'c', 389,
        'H', 395,
        'h', 395,
        'I', 355,
        'i', 355,
        'O', 241,
        'o', 241,
        'P', 532,
        'p', 532,
        'R', 215,
        'r', 215,
        'S', 498,
        's', 498,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(3);
      END_STATE();
    case 4:
      ADVANCE_MAP(
        '\n', 716,
        '#', 127,
        '(', 18,
        '/', 25,
        'C', 1877,
        'c', 1877,
        'S', 2186,
        's', 2186,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(4);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 5:
      if (lookahead == '\n') SKIP(15);
      if (lookahead == '\'') ADVANCE(1277);
      if (lookahead == '(') ADVANCE(1282);
      if (lookahead == '/') ADVANCE(1285);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(1279);
      if (lookahead != 0) ADVANCE(1287);
      END_STATE();
    case 6:
      if (lookahead == '\r') ADVANCE(2347);
      if (lookahead == '\\') ADVANCE(2341);
      if (lookahead != 0) ADVANCE(2346);
      END_STATE();
    case 7:
      ADVANCE_MAP(
        '!', 1242,
        '"', 1275,
        '#', 128,
        '$', 586,
        '%', 1253,
        '&', 1258,
        '\'', 1276,
        '(', 1238,
        '*', 1250,
        '+', 1249,
        '-', 1246,
        '.', 1273,
        '/', 1252,
        ':', 1236,
        ';', 1240,
        '<', 1265,
        '=', 709,
        '>', 1262,
        '[', 1232,
        '^', 1257,
        '{', 1234,
        '|', 1256,
        '}', 1235,
        '~', 1243,
        'A', 2096,
        'a', 2096,
        'B', 2046,
        'b', 2046,
        'C', 1852,
        'c', 1852,
        'D', 1492,
        'd', 1492,
        'E', 2089,
        'e', 2089,
        'F', 1780,
        'f', 1780,
        'G', 1709,
        'g', 1709,
        'I', 1974,
        'i', 1974,
        'N', 1694,
        'n', 1694,
        'O', 1729,
        'o', 1729,
        'S', 2013,
        's', 2013,
        'T', 1825,
        't', 1825,
        'U', 2093,
        'u', 2093,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(7);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1290);
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 8:
      ADVANCE_MAP(
        '!', 1242,
        '"', 1275,
        '#', 173,
        '$', 586,
        '%', 1253,
        '&', 1258,
        '\'', 1276,
        '(', 1238,
        ')', 1239,
        '*', 1250,
        '+', 1249,
        ',', 720,
        '-', 1246,
        '.', 1273,
        '/', 1252,
        ':', 1236,
        ';', 1240,
        '<', 1265,
        '=', 709,
        '>', 1262,
        '[', 1232,
        ']', 1233,
        '^', 1257,
        '{', 1234,
        '|', 1256,
        '}', 1235,
        '~', 1243,
        'A', 2096,
        'a', 2096,
        'C', 1873,
        'c', 1873,
        'D', 1492,
        'd', 1492,
        'E', 2089,
        'e', 2089,
        'F', 1780,
        'f', 1780,
        'G', 1709,
        'g', 1709,
        'I', 1942,
        'i', 1942,
        'N', 1694,
        'n', 1694,
        'S', 2014,
        's', 2014,
        'T', 1825,
        't', 1825,
        'U', 2093,
        'u', 2093,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(8);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1290);
      if (('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 9:
      ADVANCE_MAP(
        '!', 1241,
        '"', 1275,
        '$', 586,
        '\'', 1276,
        '(', 1238,
        '+', 1249,
        '-', 1246,
        '.', 585,
        '/', 25,
        ':', 1236,
        '[', 1232,
        '{', 1234,
        '~', 1243,
        'A', 2096,
        'a', 2096,
        'B', 2046,
        'b', 2046,
        'C', 1873,
        'c', 1873,
        'D', 1492,
        'd', 1492,
        'E', 2089,
        'e', 2089,
        'F', 1780,
        'f', 1780,
        'G', 1709,
        'g', 1709,
        'I', 1942,
        'i', 1942,
        'N', 1694,
        'n', 1694,
        'O', 1729,
        'o', 1729,
        'S', 2014,
        's', 2014,
        'T', 1825,
        't', 1825,
        'U', 2093,
        'u', 2093,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(9);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1290);
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 10:
      ADVANCE_MAP(
        '!', 26,
        '"', 1275,
        '#', 145,
        '%', 1253,
        '&', 1258,
        '(', 1238,
        ')', 1239,
        '*', 1250,
        '+', 1247,
        ',', 720,
        '-', 1244,
        '.', 1271,
        '/', 1252,
        ':', 1236,
        ';', 1240,
        '<', 1265,
        '=', 709,
        '>', 1262,
        '[', 1232,
        ']', 1233,
        '^', 1257,
        '{', 1234,
        '|', 1256,
        'B', 2234,
        'b', 2234,
        'C', 1765,
        'c', 1765,
        'I', 1947,
        'i', 1947,
        'P', 2233,
        'p', 2233,
        'R', 1657,
        'r', 1657,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(10);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 11:
      ADVANCE_MAP(
        '!', 26,
        '"', 1275,
        '#', 127,
        '%', 1253,
        '&', 1258,
        '(', 1238,
        ')', 1239,
        '*', 1250,
        '+', 1247,
        ',', 720,
        '-', 1244,
        '.', 1270,
        '/', 1252,
        ':', 1236,
        ';', 1240,
        '<', 1265,
        '=', 27,
        '>', 1262,
        '[', 1232,
        ']', 1233,
        '^', 1257,
        '{', 1234,
        '|', 1256,
        '}', 1235,
        'C', 1877,
        'c', 1877,
        'S', 2186,
        's', 2186,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(11);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 12:
      ADVANCE_MAP(
        '!', 26,
        '"', 1275,
        '%', 1253,
        '&', 1258,
        '(', 1238,
        ')', 1239,
        '*', 1250,
        '+', 1247,
        ',', 720,
        '-', 1244,
        '.', 1271,
        '/', 1252,
        ':', 1236,
        ';', 1240,
        '<', 1265,
        '=', 709,
        '>', 1262,
        '[', 1232,
        ']', 1233,
        '^', 1257,
        '|', 1256,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(12);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 13:
      ADVANCE_MAP(
        '!', 26,
        '%', 1253,
        '&', 1258,
        '(', 1238,
        ')', 1239,
        '*', 1250,
        '+', 1247,
        ',', 720,
        '-', 1244,
        '.', 1270,
        '/', 1252,
        ':', 1236,
        ';', 1240,
        '<', 1265,
        '=', 709,
        '>', 1262,
        '[', 1232,
        '^', 1257,
        '|', 1256,
        'S', 2186,
        's', 2186,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(13);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 14:
      ADVANCE_MAP(
        '#', 126,
        '(', 18,
        '/', 25,
        '}', 1235,
        'A', 559,
        'a', 559,
        'C', 389,
        'c', 389,
        'H', 395,
        'h', 395,
        'I', 355,
        'i', 355,
        'O', 241,
        'o', 241,
        'P', 532,
        'p', 532,
        'R', 215,
        'r', 215,
        'S', 498,
        's', 498,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(14);
      END_STATE();
    case 15:
      if (lookahead == '\'') ADVANCE(1277);
      if (lookahead == '(') ADVANCE(18);
      if (lookahead == '/') ADVANCE(25);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(15);
      END_STATE();
    case 16:
      if (lookahead == '(') ADVANCE(18);
      if (lookahead == '/') ADVANCE(25);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1877);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(16);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 17:
      if (lookahead == '(') ADVANCE(730);
      if (lookahead == '/') ADVANCE(732);
      if (lookahead == ';') ADVANCE(1240);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(17);
      if (lookahead != 0) ADVANCE(1230);
      END_STATE();
    case 18:
      if (lookahead == '*') ADVANCE(21);
      END_STATE();
    case 19:
      if (lookahead == '*') ADVANCE(19);
      if (lookahead == ')' ||
          lookahead == '/') ADVANCE(2339);
      if (lookahead != 0) ADVANCE(21);
      END_STATE();
    case 20:
      if (lookahead == '*') ADVANCE(19);
      if (lookahead != 0) ADVANCE(722);
      END_STATE();
    case 21:
      if (lookahead == '*') ADVANCE(19);
      if (lookahead != 0) ADVANCE(21);
      END_STATE();
    case 22:
      if (lookahead == '*') ADVANCE(22);
      if (lookahead == '/') ADVANCE(2339);
      if (lookahead != 0) ADVANCE(24);
      END_STATE();
    case 23:
      if (lookahead == '*') ADVANCE(22);
      if (lookahead != 0) ADVANCE(725);
      END_STATE();
    case 24:
      if (lookahead == '*') ADVANCE(22);
      if (lookahead != 0) ADVANCE(24);
      END_STATE();
    case 25:
      if (lookahead == '*') ADVANCE(24);
      if (lookahead == '/') ADVANCE(2346);
      END_STATE();
    case 26:
      if (lookahead == '=') ADVANCE(1260);
      END_STATE();
    case 27:
      if (lookahead == '=') ADVANCE(1259);
      END_STATE();
    case 28:
      if (lookahead == '_') ADVANCE(132);
      END_STATE();
    case 29:
      if (lookahead == '_') ADVANCE(548);
      END_STATE();
    case 30:
      if (lookahead == '_') ADVANCE(75);
      END_STATE();
    case 31:
      if (lookahead == '_') ADVANCE(563);
      END_STATE();
    case 32:
      if (lookahead == '_') ADVANCE(76);
      END_STATE();
    case 33:
      if (lookahead == '_') ADVANCE(104);
      END_STATE();
    case 34:
      if (lookahead == '_') ADVANCE(377);
      END_STATE();
    case 35:
      if (lookahead == '_') ADVANCE(411);
      END_STATE();
    case 36:
      if (lookahead == '_') ADVANCE(345);
      END_STATE();
    case 37:
      if (lookahead == '_') ADVANCE(74);
      END_STATE();
    case 38:
      if (lookahead == '_') ADVANCE(278);
      END_STATE();
    case 39:
      if (lookahead == '_') ADVANCE(134);
      END_STATE();
    case 40:
      if (lookahead == '_') ADVANCE(534);
      END_STATE();
    case 41:
      if (lookahead == '_') ADVANCE(412);
      END_STATE();
    case 42:
      if (lookahead == '_') ADVANCE(461);
      END_STATE();
    case 43:
      if (lookahead == '_') ADVANCE(151);
      END_STATE();
    case 44:
      if (lookahead == '_') ADVANCE(462);
      END_STATE();
    case 45:
      if (lookahead == '_') ADVANCE(474);
      END_STATE();
    case 46:
      if (lookahead == '_') ADVANCE(393);
      END_STATE();
    case 47:
      if (lookahead == '_') ADVANCE(514);
      END_STATE();
    case 48:
      if (lookahead == '_') ADVANCE(399);
      END_STATE();
    case 49:
      if (lookahead == '_') ADVANCE(388);
      END_STATE();
    case 50:
      if (lookahead == '_') ADVANCE(497);
      END_STATE();
    case 51:
      if (lookahead == '_') ADVANCE(500);
      END_STATE();
    case 52:
      if (lookahead == '_') ADVANCE(429);
      END_STATE();
    case 53:
      if (lookahead == '_') ADVANCE(502);
      END_STATE();
    case 54:
      if (lookahead == '_') ADVANCE(146);
      END_STATE();
    case 55:
      if (lookahead == '_') ADVANCE(84);
      END_STATE();
    case 56:
      if (lookahead == '_') ADVANCE(446);
      END_STATE();
    case 57:
      if (lookahead == '_') ADVANCE(447);
      END_STATE();
    case 58:
      if (lookahead == '_') ADVANCE(448);
      END_STATE();
    case 59:
      if (lookahead == '_') ADVANCE(415);
      END_STATE();
    case 60:
      if (lookahead == '_') ADVANCE(476);
      END_STATE();
    case 61:
      if (lookahead == '_') ADVANCE(478);
      END_STATE();
    case 62:
      if (lookahead == '_') ADVANCE(501);
      END_STATE();
    case 63:
      if (lookahead == '_') ADVANCE(516);
      END_STATE();
    case 64:
      if (lookahead == '_') ADVANCE(518);
      END_STATE();
    case 65:
      if (lookahead == '_') ADVANCE(520);
      END_STATE();
    case 66:
      if (lookahead == '_') ADVANCE(540);
      END_STATE();
    case 67:
      if (lookahead == '_') ADVANCE(152);
      END_STATE();
    case 68:
      if (lookahead == '_') ADVANCE(153);
      END_STATE();
    case 69:
      if (lookahead == '_') ADVANCE(154);
      END_STATE();
    case 70:
      if (lookahead == '+' ||
          lookahead == '-') ADVANCE(584);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1293);
      END_STATE();
    case 71:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(421);
      END_STATE();
    case 72:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(106);
      END_STATE();
    case 73:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(569);
      END_STATE();
    case 74:
      ADVANCE_MAP(
        'A', 143,
        'a', 143,
        'D', 207,
        'd', 207,
        'G', 205,
        'g', 205,
        'I', 473,
        'i', 473,
        'M', 402,
        'm', 402,
        'S', 210,
        's', 210,
      );
      END_STATE();
    case 75:
      ADVANCE_MAP(
        'A', 143,
        'a', 143,
        'D', 207,
        'd', 207,
        'G', 205,
        'g', 205,
        'I', 473,
        'i', 473,
        'M', 402,
        'm', 402,
        'S', 210,
        's', 210,
        'T', 279,
        't', 279,
      );
      END_STATE();
    case 76:
      ADVANCE_MAP(
        'A', 110,
        'a', 110,
        'D', 98,
        'd', 98,
        'E', 375,
        'e', 375,
        'R', 188,
        'r', 188,
        'S', 511,
        's', 511,
        'T', 290,
        't', 290,
      );
      END_STATE();
    case 77:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(416);
      END_STATE();
    case 78:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(582);
      END_STATE();
    case 79:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(549);
      END_STATE();
    case 80:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(441);
      END_STATE();
    case 81:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(330);
      END_STATE();
    case 82:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(121);
      END_STATE();
    case 83:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(362);
      END_STATE();
    case 84:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(140);
      END_STATE();
    case 85:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(125);
      END_STATE();
    case 86:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(364);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(289);
      END_STATE();
    case 87:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(426);
      END_STATE();
    case 88:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(366);
      END_STATE();
    case 89:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(115);
      END_STATE();
    case 90:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(116);
      END_STATE();
    case 91:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(472);
      END_STATE();
    case 92:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(551);
      END_STATE();
    case 93:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(552);
      END_STATE();
    case 94:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(553);
      END_STATE();
    case 95:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(554);
      END_STATE();
    case 96:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(555);
      END_STATE();
    case 97:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(556);
      END_STATE();
    case 98:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(573);
      END_STATE();
    case 99:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(574);
      END_STATE();
    case 100:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(575);
      END_STATE();
    case 101:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(576);
      END_STATE();
    case 102:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(577);
      END_STATE();
    case 103:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(578);
      END_STATE();
    case 104:
      ADVANCE_MAP(
        'A', 118,
        'a', 118,
        'C', 318,
        'c', 318,
        'D', 99,
        'd', 99,
        'E', 386,
        'e', 386,
        'R', 230,
        'r', 230,
        'S', 527,
        's', 527,
        'T', 293,
        't', 293,
      );
      END_STATE();
    case 105:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(453);
      END_STATE();
    case 106:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(320);
      END_STATE();
    case 107:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(567);
      END_STATE();
    case 108:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(317);
      END_STATE();
    case 109:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(390);
      END_STATE();
    case 110:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(505);
      END_STATE();
    case 111:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(494);
      END_STATE();
    case 112:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(41);
      END_STATE();
    case 113:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(163);
      END_STATE();
    case 114:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(196);
      END_STATE();
    case 115:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(165);
      END_STATE();
    case 116:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(166);
      END_STATE();
    case 117:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(59);
      END_STATE();
    case 118:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(524);
      END_STATE();
    case 119:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(715);
      END_STATE();
    case 120:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(718);
      END_STATE();
    case 121:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(637);
      END_STATE();
    case 122:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(677);
      END_STATE();
    case 123:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(679);
      END_STATE();
    case 124:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(665);
      END_STATE();
    case 125:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(639);
      END_STATE();
    case 126:
      ADVANCE_MAP(
        'D', 175,
        'd', 175,
        'E', 319,
        'e', 319,
        'I', 237,
        'i', 237,
        'W', 71,
        'w', 71,
      );
      END_STATE();
    case 127:
      ADVANCE_MAP(
        'D', 175,
        'd', 175,
        'E', 319,
        'e', 319,
        'I', 236,
        'i', 236,
        'W', 71,
        'w', 71,
      );
      END_STATE();
    case 128:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(175);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(237);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(71);
      END_STATE();
    case 129:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(604);
      END_STATE();
    case 130:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(618);
      END_STATE();
    case 131:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(38);
      END_STATE();
    case 132:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(223);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(387);
      END_STATE();
    case 133:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(107);
      END_STATE();
    case 134:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(73);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(187);
      END_STATE();
    case 135:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(81);
      END_STATE();
    case 136:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(158);
      END_STATE();
    case 137:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(40);
      END_STATE();
    case 138:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(43);
      END_STATE();
    case 139:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(436);
      END_STATE();
    case 140:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(139);
      END_STATE();
    case 141:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(46);
      END_STATE();
    case 142:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(167);
      END_STATE();
    case 143:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(137);
      END_STATE();
    case 144:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(193);
      END_STATE();
    case 145:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(174);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(319);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(237);
      END_STATE();
    case 146:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(225);
      END_STATE();
    case 147:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(227);
      END_STATE();
    case 148:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(64);
      END_STATE();
    case 149:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(65);
      END_STATE();
    case 150:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(232);
      END_STATE();
    case 151:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(100);
      END_STATE();
    case 152:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(101);
      END_STATE();
    case 153:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(102);
      END_STATE();
    case 154:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(103);
      END_STATE();
    case 155:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(67);
      END_STATE();
    case 156:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(719);
      END_STATE();
    case 157:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(712);
      END_STATE();
    case 158:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(710);
      END_STATE();
    case 159:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(661);
      END_STATE();
    case 160:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(645);
      END_STATE();
    case 161:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(673);
      END_STATE();
    case 162:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(675);
      END_STATE();
    case 163:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(667);
      END_STATE();
    case 164:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(643);
      END_STATE();
    case 165:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(705);
      END_STATE();
    case 166:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(703);
      END_STATE();
    case 167:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(671);
      END_STATE();
    case 168:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(699);
      END_STATE();
    case 169:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(701);
      END_STATE();
    case 170:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(695);
      END_STATE();
    case 171:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(697);
      END_STATE();
    case 172:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(319);
      END_STATE();
    case 173:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(319);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(236);
      END_STATE();
    case 174:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(238);
      END_STATE();
    case 175:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(238);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(457);
      END_STATE();
    case 176:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(475);
      END_STATE();
    case 177:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(579);
      END_STATE();
    case 178:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(622);
      END_STATE();
    case 179:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(610);
      END_STATE();
    case 180:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(612);
      END_STATE();
    case 181:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(602);
      END_STATE();
    case 182:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(512);
      END_STATE();
    case 183:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(568);
      END_STATE();
    case 184:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(464);
      END_STATE();
    case 185:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(119);
      END_STATE();
    case 186:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(423);
      END_STATE();
    case 187:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(499);
      END_STATE();
    case 188:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(458);
      END_STATE();
    case 189:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(120);
      END_STATE();
    case 190:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(451);
      END_STATE();
    case 191:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(31);
      END_STATE();
    case 192:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(91);
      END_STATE();
    case 193:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(34);
      END_STATE();
    case 194:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(463);
      END_STATE();
    case 195:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(431);
      END_STATE();
    case 196:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(124);
      END_STATE();
    case 197:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(495);
      END_STATE();
    case 198:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(496);
      END_STATE();
    case 199:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(417);
      END_STATE();
    case 200:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(47);
      END_STATE();
    case 201:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(418);
      END_STATE();
    case 202:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(419);
      END_STATE();
    case 203:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(420);
      END_STATE();
    case 204:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(434);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(296);
      END_STATE();
    case 205:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(506);
      END_STATE();
    case 206:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(433);
      END_STATE();
    case 207:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(323);
      END_STATE();
    case 208:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(439);
      END_STATE();
    case 209:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(432);
      END_STATE();
    case 210:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(507);
      END_STATE();
    case 211:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(438);
      END_STATE();
    case 212:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(141);
      END_STATE();
    case 213:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(148);
      END_STATE();
    case 214:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(509);
      END_STATE();
    case 215:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(329);
      END_STATE();
    case 216:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(571);
      END_STATE();
    case 217:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(479);
      END_STATE();
    case 218:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(513);
      END_STATE();
    case 219:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(442);
      END_STATE();
    case 220:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(443);
      END_STATE();
    case 221:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(444);
      END_STATE();
    case 222:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(445);
      END_STATE();
    case 223:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(245);
      END_STATE();
    case 224:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(481);
      END_STATE();
    case 225:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(247);
      END_STATE();
    case 226:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(483);
      END_STATE();
    case 227:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(248);
      END_STATE();
    case 228:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(484);
      END_STATE();
    case 229:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(63);
      END_STATE();
    case 230:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(491);
      END_STATE();
    case 231:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(149);
      END_STATE();
    case 232:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(249);
      END_STATE();
    case 233:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(452);
      END_STATE();
    case 234:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(66);
      END_STATE();
    case 235:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(717);
      END_STATE();
    case 236:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(28);
      END_STATE();
    case 237:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(28);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(108);
      END_STATE();
    case 238:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(280);
      END_STATE();
    case 239:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(243);
      END_STATE();
    case 240:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(281);
      END_STATE();
    case 241:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(242);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(204);
      END_STATE();
    case 242:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(331);
      END_STATE();
    case 243:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(470);
      END_STATE();
    case 244:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(471);
      END_STATE();
    case 245:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(291);
      END_STATE();
    case 246:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(244);
      END_STATE();
    case 247:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(294);
      END_STATE();
    case 248:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(295);
      END_STATE();
    case 249:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(315);
      END_STATE();
    case 250:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(714);
      END_STATE();
    case 251:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(271);
      END_STATE();
    case 252:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(616);
      END_STATE();
    case 253:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(425);
      END_STATE();
    case 254:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(466);
      END_STATE();
    case 255:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(35);
      END_STATE();
    case 256:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(467);
      END_STATE();
    case 257:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(435);
      END_STATE();
    case 258:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(468);
      END_STATE();
    case 259:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(449);
      END_STATE();
    case 260:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(469);
      END_STATE();
    case 261:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(477);
      END_STATE();
    case 262:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(480);
      END_STATE();
    case 263:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(482);
      END_STATE();
    case 264:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(272);
      END_STATE();
    case 265:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(273);
      END_STATE();
    case 266:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(274);
      END_STATE();
    case 267:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(275);
      END_STATE();
    case 268:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(276);
      END_STATE();
    case 269:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(277);
      END_STATE();
    case 270:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(600);
      END_STATE();
    case 271:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(510);
      END_STATE();
    case 272:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(515);
      END_STATE();
    case 273:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(517);
      END_STATE();
    case 274:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(519);
      END_STATE();
    case 275:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(521);
      END_STATE();
    case 276:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(522);
      END_STATE();
    case 277:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(523);
      END_STATE();
    case 278:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(235);
      END_STATE();
    case 279:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(340);
      END_STATE();
    case 280:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(356);
      END_STATE();
    case 281:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(255);
      END_STATE();
    case 282:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(321);
      END_STATE();
    case 283:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(541);
      END_STATE();
    case 284:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(251);
      END_STATE();
    case 285:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(357);
      END_STATE();
    case 286:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(396);
      END_STATE();
    case 287:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(398);
      END_STATE();
    case 288:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(359);
      END_STATE();
    case 289:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(361);
      END_STATE();
    case 290:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(341);
      END_STATE();
    case 291:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(363);
      END_STATE();
    case 292:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(550);
      END_STATE();
    case 293:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(342);
      END_STATE();
    case 294:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(365);
      END_STATE();
    case 295:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(378);
      END_STATE();
    case 296:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(370);
      END_STATE();
    case 297:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(371);
      END_STATE();
    case 298:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(376);
      END_STATE();
    case 299:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(379);
      END_STATE();
    case 300:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(380);
      END_STATE();
    case 301:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(381);
      END_STATE();
    case 302:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(382);
      END_STATE();
    case 303:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(383);
      END_STATE();
    case 304:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(346);
      END_STATE();
    case 305:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(348);
      END_STATE();
    case 306:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(349);
      END_STATE();
    case 307:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(350);
      END_STATE();
    case 308:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(264);
      END_STATE();
    case 309:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(557);
      END_STATE();
    case 310:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(265);
      END_STATE();
    case 311:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(266);
      END_STATE();
    case 312:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(267);
      END_STATE();
    case 313:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(268);
      END_STATE();
    case 314:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(269);
      END_STATE();
    case 315:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(385);
      END_STATE();
    case 316:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(663);
      END_STATE();
    case 317:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(528);
      END_STATE();
    case 318:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(581);
      END_STATE();
    case 319:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(456);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(131);
      END_STATE();
    case 320:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(191);
      END_STATE();
    case 321:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(316);
      END_STATE();
    case 322:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(284);
      END_STATE();
    case 323:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(182);
      END_STATE();
    case 324:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(129);
      END_STATE();
    case 325:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(168);
      END_STATE();
    case 326:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(169);
      END_STATE();
    case 327:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(170);
      END_STATE();
    case 328:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(171);
      END_STATE();
    case 329:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(192);
      END_STATE();
    case 330:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(406);
      END_STATE();
    case 331:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(297);
      END_STATE();
    case 332:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(308);
      END_STATE();
    case 333:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(310);
      END_STATE();
    case 334:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(311);
      END_STATE();
    case 335:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(312);
      END_STATE();
    case 336:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(313);
      END_STATE();
    case 337:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(314);
      END_STATE();
    case 338:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(253);
      END_STATE();
    case 339:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(343);
      END_STATE();
    case 340:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(184);
      END_STATE();
    case 341:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(176);
      END_STATE();
    case 342:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(177);
      END_STATE();
    case 343:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(88);
      END_STATE();
    case 344:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(257);
      END_STATE();
    case 345:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(403);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(246);
      END_STATE();
    case 346:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(217);
      END_STATE();
    case 347:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(259);
      END_STATE();
    case 348:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(224);
      END_STATE();
    case 349:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(226);
      END_STATE();
    case 350:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(228);
      END_STATE();
    case 351:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(713);
      END_STATE();
    case 352:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(649);
      END_STATE();
    case 353:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(647);
      END_STATE();
    case 354:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(669);
      END_STATE();
    case 355:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(108);
      END_STATE();
    case 356:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(157);
      END_STATE();
    case 357:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(250);
      END_STATE();
    case 358:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(112);
      END_STATE();
    case 359:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(254);
      END_STATE();
    case 360:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(392);
      END_STATE();
    case 361:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(252);
      END_STATE();
    case 362:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(135);
      END_STATE();
    case 363:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(185);
      END_STATE();
    case 364:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(133);
      END_STATE();
    case 365:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(189);
      END_STATE();
    case 366:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(130);
      END_STATE();
    case 367:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(161);
      END_STATE();
    case 368:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(162);
      END_STATE();
    case 369:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(164);
      END_STATE();
    case 370:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(179);
      END_STATE();
    case 371:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(180);
      END_STATE();
    case 372:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(240);
      END_STATE();
    case 373:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(285);
      END_STATE();
    case 374:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(401);
      END_STATE();
    case 375:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(138);
      END_STATE();
    case 376:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(256);
      END_STATE();
    case 377:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(218);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(503);
      END_STATE();
    case 378:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(213);
      END_STATE();
    case 379:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(258);
      END_STATE();
    case 380:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(260);
      END_STATE();
    case 381:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(261);
      END_STATE();
    case 382:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(262);
      END_STATE();
    case 383:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(263);
      END_STATE();
    case 384:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(117);
      END_STATE();
    case 385:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(231);
      END_STATE();
    case 386:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(155);
      END_STATE();
    case 387:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(504);
      END_STATE();
    case 388:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(239);
      END_STATE();
    case 389:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(339);
      END_STATE();
    case 390:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(372);
      END_STATE();
    case 391:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(531);
      END_STATE();
    case 392:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(560);
      END_STATE();
    case 393:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(530);
      END_STATE();
    case 394:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(450);
      END_STATE();
    case 395:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(324);
      END_STATE();
    case 396:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(122);
      END_STATE();
    case 397:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(424);
      END_STATE();
    case 398:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(123);
      END_STATE();
    case 399:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(354);
      END_STATE();
    case 400:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(422);
      END_STATE();
    case 401:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(561);
      END_STATE();
    case 402:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(144);
      END_STATE();
    case 403:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(142);
      END_STATE();
    case 404:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(367);
      END_STATE();
    case 405:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(368);
      END_STATE();
    case 406:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(369);
      END_STATE();
    case 407:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(535);
      END_STATE();
    case 408:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(625);
      END_STATE();
    case 409:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(82);
      END_STATE();
    case 410:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(89);
      END_STATE();
    case 411:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(430);
      END_STATE();
    case 412:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(209);
      END_STATE();
    case 413:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(85);
      END_STATE();
    case 414:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(90);
      END_STATE();
    case 415:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(211);
      END_STATE();
    case 416:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(590);
      END_STATE();
    case 417:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(685);
      END_STATE();
    case 418:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(687);
      END_STATE();
    case 419:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(691);
      END_STATE();
    case 420:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(693);
      END_STATE();
    case 421:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(351);
      END_STATE();
    case 422:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(614);
      END_STATE();
    case 423:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(542);
      END_STATE();
    case 424:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(580);
      END_STATE();
    case 425:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(30);
      END_STATE();
    case 426:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(373);
      END_STATE();
    case 427:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(113);
      END_STATE();
    case 428:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(533);
      END_STATE();
    case 429:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(536);
      END_STATE();
    case 430:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(283);
      END_STATE();
    case 431:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(454);
      END_STATE();
    case 432:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(286);
      END_STATE();
    case 433:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(44);
      END_STATE();
    case 434:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(440);
      END_STATE();
    case 435:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(37);
      END_STATE();
    case 436:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(194);
      END_STATE();
    case 437:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(114);
      END_STATE();
    case 438:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(287);
      END_STATE();
    case 439:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(543);
      END_STATE();
    case 440:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(400);
      END_STATE();
    case 441:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(525);
      END_STATE();
    case 442:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(544);
      END_STATE();
    case 443:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(545);
      END_STATE();
    case 444:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(546);
      END_STATE();
    case 445:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(547);
      END_STATE();
    case 446:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(537);
      END_STATE();
    case 447:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(538);
      END_STATE();
    case 448:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(539);
      END_STATE();
    case 449:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(62);
      END_STATE();
    case 450:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(583);
      END_STATE();
    case 451:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(147);
      END_STATE();
    case 452:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(150);
      END_STATE();
    case 453:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(526);
      END_STATE();
    case 454:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(689);
      END_STATE();
    case 455:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(629);
      END_STATE();
    case 456:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(156);
      END_STATE();
    case 457:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(72);
      END_STATE();
    case 458:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(570);
      END_STATE();
    case 459:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(270);
      END_STATE();
    case 460:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(79);
      END_STATE();
    case 461:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(391);
      END_STATE();
    case 462:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(508);
      END_STATE();
    case 463:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(455);
      END_STATE();
    case 464:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(186);
      END_STATE();
    case 465:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(190);
      END_STATE();
    case 466:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(48);
      END_STATE();
    case 467:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(49);
      END_STATE();
    case 468:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(36);
      END_STATE();
    case 469:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(52);
      END_STATE();
    case 470:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(197);
      END_STATE();
    case 471:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(198);
      END_STATE();
    case 472:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(181);
      END_STATE();
    case 473:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(39);
      END_STATE();
    case 474:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(410);
      END_STATE();
    case 475:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(208);
      if (lookahead == 'Z' ||
          lookahead == 'z') ADVANCE(404);
      END_STATE();
    case 476:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(407);
      END_STATE();
    case 477:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(56);
      END_STATE();
    case 478:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(414);
      END_STATE();
    case 479:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(219);
      END_STATE();
    case 480:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(57);
      END_STATE();
    case 481:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(220);
      END_STATE();
    case 482:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(58);
      END_STATE();
    case 483:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(221);
      END_STATE();
    case 484:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(222);
      END_STATE();
    case 485:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(92);
      END_STATE();
    case 486:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(93);
      END_STATE();
    case 487:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(94);
      END_STATE();
    case 488:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(95);
      END_STATE();
    case 489:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(96);
      END_STATE();
    case 490:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(97);
      END_STATE();
    case 491:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(572);
      END_STATE();
    case 492:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(233);
      END_STATE();
    case 493:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(653);
      END_STATE();
    case 494:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(655);
      END_STATE();
    case 495:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(681);
      END_STATE();
    case 496:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(683);
      END_STATE();
    case 497:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(564);
      END_STATE();
    case 498:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(86);
      END_STATE();
    case 499:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(558);
      END_STATE();
    case 500:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(565);
      END_STATE();
    case 501:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(279);
      END_STATE();
    case 502:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(566);
      END_STATE();
    case 503:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(83);
      END_STATE();
    case 504:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(54);
      END_STATE();
    case 505:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(292);
      END_STATE();
    case 506:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(32);
      END_STATE();
    case 507:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(33);
      END_STATE();
    case 508:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(428);
      END_STATE();
    case 509:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(55);
      END_STATE();
    case 510:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(460);
      END_STATE();
    case 511:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(80);
      END_STATE();
    case 512:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(234);
      END_STATE();
    case 513:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(562);
      END_STATE();
    case 514:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(304);
      END_STATE();
    case 515:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(485);
      END_STATE();
    case 516:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(305);
      END_STATE();
    case 517:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(486);
      END_STATE();
    case 518:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(306);
      END_STATE();
    case 519:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(487);
      END_STATE();
    case 520:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(307);
      END_STATE();
    case 521:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(488);
      END_STATE();
    case 522:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(489);
      END_STATE();
    case 523:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(490);
      END_STATE();
    case 524:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(309);
      END_STATE();
    case 525:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(68);
      END_STATE();
    case 526:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(69);
      END_STATE();
    case 527:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(105);
      END_STATE();
    case 528:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(136);
      END_STATE();
    case 529:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(408);
      END_STATE();
    case 530:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(493);
      END_STATE();
    case 531:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(427);
      END_STATE();
    case 532:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(459);
      END_STATE();
    case 533:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(111);
      END_STATE();
    case 534:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(465);
      END_STATE();
    case 535:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(437);
      END_STATE();
    case 536:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(325);
      END_STATE();
    case 537:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(326);
      END_STATE();
    case 538:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(327);
      END_STATE();
    case 539:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(328);
      END_STATE();
    case 540:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(492);
      END_STATE();
    case 541:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(651);
      END_STATE();
    case 542:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(206);
      END_STATE();
    case 543:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(195);
      END_STATE();
    case 544:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(199);
      END_STATE();
    case 545:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(201);
      END_STATE();
    case 546:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(202);
      END_STATE();
    case 547:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(203);
      END_STATE();
    case 548:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(77);
      END_STATE();
    case 549:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(288);
      END_STATE();
    case 550:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(200);
      END_STATE();
    case 551:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(298);
      END_STATE();
    case 552:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(299);
      END_STATE();
    case 553:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(300);
      END_STATE();
    case 554:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(301);
      END_STATE();
    case 555:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(302);
      END_STATE();
    case 556:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(303);
      END_STATE();
    case 557:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(229);
      END_STATE();
    case 558:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(394);
      END_STATE();
    case 559:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(78);
      END_STATE();
    case 560:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(352);
      END_STATE();
    case 561:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(353);
      END_STATE();
    case 562:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(397);
      END_STATE();
    case 563:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(87);
      END_STATE();
    case 564:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(631);
      END_STATE();
    case 565:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(633);
      END_STATE();
    case 566:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(635);
      END_STATE();
    case 567:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(620);
      END_STATE();
    case 568:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(409);
      END_STATE();
    case 569:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(322);
      END_STATE();
    case 570:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(358);
      END_STATE();
    case 571:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(413);
      END_STATE();
    case 572:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(384);
      END_STATE();
    case 573:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(332);
      END_STATE();
    case 574:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(333);
      END_STATE();
    case 575:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(334);
      END_STATE();
    case 576:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(335);
      END_STATE();
    case 577:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(336);
      END_STATE();
    case 578:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(337);
      END_STATE();
    case 579:
      if (lookahead == 'Z' ||
          lookahead == 'z') ADVANCE(405);
      END_STATE();
    case 580:
      if (lookahead == 'K' ||
          lookahead == 'k' ||
          lookahead == 0x212a) ADVANCE(641);
      END_STATE();
    case 581:
      if (lookahead == 'K' ||
          lookahead == 'k' ||
          lookahead == 0x212a) ADVANCE(42);
      END_STATE();
    case 582:
      if (lookahead == 'K' ||
          lookahead == 'k' ||
          lookahead == 0x212a) ADVANCE(178);
      END_STATE();
    case 583:
      if (lookahead == 'K' ||
          lookahead == 'k' ||
          lookahead == 0x212a) ADVANCE(60);
      END_STATE();
    case 584:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1293);
      END_STATE();
    case 585:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1292);
      END_STATE();
    case 586:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(1294);
      END_STATE();
    case 587:
      if (lookahead != 0 &&
          lookahead != '*') ADVANCE(1230);
      END_STATE();
    case 588:
      if (eof) ADVANCE(589);
      ADVANCE_MAP(
        '!', 1242,
        '"', 1275,
        '#', 126,
        '$', 586,
        '%', 1253,
        '&', 1258,
        '\'', 1276,
        '(', 1238,
        ')', 1239,
        '*', 1250,
        '+', 1249,
        ',', 720,
        '-', 1246,
        '.', 1273,
        '/', 1252,
        ':', 1236,
        ';', 1240,
        '<', 1265,
        '=', 709,
        '>', 1262,
        '[', 1232,
        ']', 1233,
        '^', 1257,
        '{', 1234,
        '|', 1256,
        '}', 1235,
        '~', 1243,
        'A', 2096,
        'a', 2096,
        'B', 2045,
        'b', 2045,
        'C', 1764,
        'c', 1764,
        'D', 1510,
        'd', 1510,
        'E', 2089,
        'e', 2089,
        'F', 1780,
        'f', 1780,
        'G', 1709,
        'g', 1709,
        'I', 1974,
        'i', 1974,
        'L', 1612,
        'l', 1612,
        'N', 1694,
        'n', 1694,
        'O', 1729,
        'o', 1729,
        'P', 2233,
        'p', 2233,
        'R', 1657,
        'r', 1657,
        'S', 2013,
        's', 2013,
        'T', 1823,
        't', 1823,
        'U', 2093,
        'u', 2093,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(588);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1290);
      if (('H' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('h' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 589:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    case 590:
      ACCEPT_TOKEN(sym_stack_var_keyword);
      END_STATE();
    case 591:
      ACCEPT_TOKEN(sym_stack_var_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 592:
      ACCEPT_TOKEN(sym_break_keyword);
      END_STATE();
    case 593:
      ACCEPT_TOKEN(sym_break_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 594:
      ACCEPT_TOKEN(sym_button_event_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 595:
      ACCEPT_TOKEN(sym_channel_event_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 596:
      ACCEPT_TOKEN(sym_level_event_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 597:
      ACCEPT_TOKEN(sym_data_event_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 598:
      ACCEPT_TOKEN(sym_timeline_event_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 599:
      ACCEPT_TOKEN(sym_custom_event_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 600:
      ACCEPT_TOKEN(sym_push_keyword);
      END_STATE();
    case 601:
      ACCEPT_TOKEN(sym_push_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 602:
      ACCEPT_TOKEN(sym_release_keyword);
      END_STATE();
    case 603:
      ACCEPT_TOKEN(sym_release_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 604:
      ACCEPT_TOKEN(sym_hold_keyword);
      END_STATE();
    case 605:
      ACCEPT_TOKEN(sym_hold_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 606:
      ACCEPT_TOKEN(aux_sym_on_keyword_token1);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2051);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1799);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 607:
      ACCEPT_TOKEN(aux_sym_on_keyword_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 608:
      ACCEPT_TOKEN(aux_sym_off_keyword_token1);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1804);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 609:
      ACCEPT_TOKEN(aux_sym_off_keyword_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 610:
      ACCEPT_TOKEN(sym_online_keyword);
      END_STATE();
    case 611:
      ACCEPT_TOKEN(sym_online_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 612:
      ACCEPT_TOKEN(sym_offline_keyword);
      END_STATE();
    case 613:
      ACCEPT_TOKEN(sym_offline_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 614:
      ACCEPT_TOKEN(sym_onerror_keyword);
      END_STATE();
    case 615:
      ACCEPT_TOKEN(sym_onerror_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 616:
      ACCEPT_TOKEN(sym_string_keyword);
      END_STATE();
    case 617:
      ACCEPT_TOKEN(sym_string_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 618:
      ACCEPT_TOKEN(sym_command_keyword);
      END_STATE();
    case 619:
      ACCEPT_TOKEN(sym_command_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 620:
      ACCEPT_TOKEN(sym_standby_keyword);
      END_STATE();
    case 621:
      ACCEPT_TOKEN(sym_standby_keyword);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 622:
      ACCEPT_TOKEN(sym_awake_keyword);
      END_STATE();
    case 623:
      ACCEPT_TOKEN(sym_url_flg_stat_mask);
      END_STATE();
    case 624:
      ACCEPT_TOKEN(sym_url_flg_stat_mask);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 625:
      ACCEPT_TOKEN(sym_url_flg_stat_lookup);
      END_STATE();
    case 626:
      ACCEPT_TOKEN(sym_url_flg_stat_lookup);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 627:
      ACCEPT_TOKEN(sym_source_type_axlink);
      END_STATE();
    case 628:
      ACCEPT_TOKEN(sym_source_type_axlink);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 629:
      ACCEPT_TOKEN(sym_source_type_ip_socket_address);
      END_STATE();
    case 630:
      ACCEPT_TOKEN(sym_source_type_ip_socket_address);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 631:
      ACCEPT_TOKEN(sym_internal_queue_size_index_axlink_tx);
      END_STATE();
    case 632:
      ACCEPT_TOKEN(sym_internal_queue_size_index_axlink_tx);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 633:
      ACCEPT_TOKEN(sym_internal_queue_size_index_phastlink_tx);
      END_STATE();
    case 634:
      ACCEPT_TOKEN(sym_internal_queue_size_index_phastlink_tx);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 635:
      ACCEPT_TOKEN(sym_internal_queue_size_index_icsplontalk_tx);
      END_STATE();
    case 636:
      ACCEPT_TOKEN(sym_internal_queue_size_index_icsplontalk_tx);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 637:
      ACCEPT_TOKEN(sym_duet_dev_type_keypad);
      END_STATE();
    case 638:
      ACCEPT_TOKEN(sym_duet_dev_type_keypad);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 639:
      ACCEPT_TOKEN(sym_duet_dev_type_text_keypad);
      END_STATE();
    case 640:
      ACCEPT_TOKEN(sym_duet_dev_type_text_keypad);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 641:
      ACCEPT_TOKEN(sym_clkmgr_mode_network);
      END_STATE();
    case 642:
      ACCEPT_TOKEN(sym_clkmgr_mode_network);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 643:
      ACCEPT_TOKEN(sym_clkmgr_mode_standalone);
      END_STATE();
    case 644:
      ACCEPT_TOKEN(sym_clkmgr_mode_standalone);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 645:
      ACCEPT_TOKEN(sym_normal_wake);
      END_STATE();
    case 646:
      ACCEPT_TOKEN(sym_normal_wake);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 647:
      ACCEPT_TOKEN(sym_data_structure_unknown);
      END_STATE();
    case 648:
      ACCEPT_TOKEN(sym_data_structure_unknown);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 649:
      ACCEPT_TOKEN(sym_data_type_unknown);
      END_STATE();
    case 650:
      ACCEPT_TOKEN(sym_data_type_unknown);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 651:
      ACCEPT_TOKEN(sym_networkconfig_priv);
      END_STATE();
    case 652:
      ACCEPT_TOKEN(sym_networkconfig_priv);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 653:
      ACCEPT_TOKEN(sym_err_user_locked_out);
      END_STATE();
    case 654:
      ACCEPT_TOKEN(sym_err_user_locked_out);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 655:
      ACCEPT_TOKEN(sym_clkmgr_timeserver_struct);
      END_STATE();
    case 656:
      ACCEPT_TOKEN(sym_clkmgr_timeserver_struct);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 657:
      ACCEPT_TOKEN(sym_day_of_week);
      END_STATE();
    case 658:
      ACCEPT_TOKEN(sym_day_of_week);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 659:
      ACCEPT_TOKEN(sym_file_seek);
      END_STATE();
    case 660:
      ACCEPT_TOKEN(sym_file_seek);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 661:
      ACCEPT_TOKEN(sym_device_wake);
      END_STATE();
    case 662:
      ACCEPT_TOKEN(sym_device_wake);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 663:
      ACCEPT_TOKEN(sym_timeline_kill);
      END_STATE();
    case 664:
      ACCEPT_TOKEN(sym_timeline_kill);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 665:
      ACCEPT_TOKEN(sym_clkmgr_is_network_sourced);
      END_STATE();
    case 666:
      ACCEPT_TOKEN(sym_clkmgr_is_network_sourced);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 667:
      ACCEPT_TOKEN(sym_clkmgr_set_clk_source);
      END_STATE();
    case 668:
      ACCEPT_TOKEN(sym_clkmgr_set_clk_source);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 669:
      ACCEPT_TOKEN(sym_clkmgr_is_daylightsavings_on);
      END_STATE();
    case 670:
      ACCEPT_TOKEN(sym_clkmgr_is_daylightsavings_on);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 671:
      ACCEPT_TOKEN(sym_clkmgr_set_daylightsavings_mode);
      END_STATE();
    case 672:
      ACCEPT_TOKEN(sym_clkmgr_set_daylightsavings_mode);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 673:
      ACCEPT_TOKEN(sym_clkmgr_get_timezone);
      END_STATE();
    case 674:
      ACCEPT_TOKEN(sym_clkmgr_get_timezone);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 675:
      ACCEPT_TOKEN(sym_clkmgr_set_timezone);
      END_STATE();
    case 676:
      ACCEPT_TOKEN(sym_clkmgr_set_timezone);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 677:
      ACCEPT_TOKEN(sym_clkmgr_get_resync_period);
      END_STATE();
    case 678:
      ACCEPT_TOKEN(sym_clkmgr_get_resync_period);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 679:
      ACCEPT_TOKEN(sym_clkmgr_set_resync_period);
      END_STATE();
    case 680:
      ACCEPT_TOKEN(sym_clkmgr_set_resync_period);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 681:
      ACCEPT_TOKEN(sym_clkmgr_get_daylightsavings_offset);
      END_STATE();
    case 682:
      ACCEPT_TOKEN(sym_clkmgr_get_daylightsavings_offset);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 683:
      ACCEPT_TOKEN(sym_clkmgr_set_daylightsavings_offset);
      END_STATE();
    case 684:
      ACCEPT_TOKEN(sym_clkmgr_set_daylightsavings_offset);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 685:
      ACCEPT_TOKEN(sym_clkmgr_get_active_timeserver);
      END_STATE();
    case 686:
      ACCEPT_TOKEN(sym_clkmgr_get_active_timeserver);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 687:
      ACCEPT_TOKEN(sym_clkmgr_set_active_timeserver);
      END_STATE();
    case 688:
      ACCEPT_TOKEN(sym_clkmgr_set_active_timeserver);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 689:
      ACCEPT_TOKEN(sym_clkmgr_get_timeservers);
      END_STATE();
    case 690:
      ACCEPT_TOKEN(sym_clkmgr_get_timeservers);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 691:
      ACCEPT_TOKEN(sym_clkmgr_add_userdefined_timeserver);
      END_STATE();
    case 692:
      ACCEPT_TOKEN(sym_clkmgr_add_userdefined_timeserver);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 693:
      ACCEPT_TOKEN(sym_clkmgr_delete_userdefined_timeserver);
      END_STATE();
    case 694:
      ACCEPT_TOKEN(sym_clkmgr_delete_userdefined_timeserver);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 695:
      ACCEPT_TOKEN(sym_clkmgr_get_start_daylightsavings_rule);
      END_STATE();
    case 696:
      ACCEPT_TOKEN(sym_clkmgr_get_start_daylightsavings_rule);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 697:
      ACCEPT_TOKEN(sym_clkmgr_set_start_daylightsavings_rule);
      END_STATE();
    case 698:
      ACCEPT_TOKEN(sym_clkmgr_set_start_daylightsavings_rule);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 699:
      ACCEPT_TOKEN(sym_clkmgr_get_end_daylightsavings_rule);
      END_STATE();
    case 700:
      ACCEPT_TOKEN(sym_clkmgr_get_end_daylightsavings_rule);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 701:
      ACCEPT_TOKEN(sym_clkmgr_set_end_daylightsavings_rule);
      END_STATE();
    case 702:
      ACCEPT_TOKEN(sym_clkmgr_set_end_daylightsavings_rule);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 703:
      ACCEPT_TOKEN(sym_get_available_flash_disk_space);
      END_STATE();
    case 704:
      ACCEPT_TOKEN(sym_get_available_flash_disk_space);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 705:
      ACCEPT_TOKEN(sym_get_max_flash_disk_space);
      END_STATE();
    case 706:
      ACCEPT_TOKEN(sym_get_max_flash_disk_space);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 707:
      ACCEPT_TOKEN(sym_astro_clock);
      END_STATE();
    case 708:
      ACCEPT_TOKEN(sym_astro_clock);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 709:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=') ADVANCE(1259);
      END_STATE();
    case 710:
      ACCEPT_TOKEN(aux_sym_preproc_include_token1);
      END_STATE();
    case 711:
      ACCEPT_TOKEN(aux_sym_preproc_include_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 712:
      ACCEPT_TOKEN(aux_sym_preproc_define_token1);
      END_STATE();
    case 713:
      ACCEPT_TOKEN(aux_sym_preproc_warn_token1);
      END_STATE();
    case 714:
      ACCEPT_TOKEN(aux_sym_preproc_disable_warning_token1);
      END_STATE();
    case 715:
      ACCEPT_TOKEN(aux_sym_preproc_if_defined_token1);
      END_STATE();
    case 716:
      ACCEPT_TOKEN(anon_sym_LF);
      if (lookahead == '\n') ADVANCE(716);
      END_STATE();
    case 717:
      ACCEPT_TOKEN(aux_sym_preproc_if_defined_token2);
      END_STATE();
    case 718:
      ACCEPT_TOKEN(aux_sym_preproc_if_not_defined_token1);
      END_STATE();
    case 719:
      ACCEPT_TOKEN(aux_sym_preproc_else_token1);
      END_STATE();
    case 720:
      ACCEPT_TOKEN(anon_sym_COMMA);
      END_STATE();
    case 721:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\n') ADVANCE(21);
      if (lookahead == ')') ADVANCE(2339);
      if (lookahead == '*') ADVANCE(721);
      if (lookahead == '/') ADVANCE(2339);
      if (lookahead == '\\') ADVANCE(727);
      if (lookahead != 0) ADVANCE(722);
      END_STATE();
    case 722:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\n') ADVANCE(21);
      if (lookahead == '*') ADVANCE(721);
      if (lookahead == '/') ADVANCE(20);
      if (lookahead == '\\') ADVANCE(727);
      if (lookahead != 0) ADVANCE(722);
      END_STATE();
    case 723:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\n') ADVANCE(2346);
      if (lookahead == '\r') ADVANCE(2340);
      if (lookahead == '/') ADVANCE(2343);
      if (lookahead == '\\') ADVANCE(2342);
      if (lookahead != 0) ADVANCE(2344);
      END_STATE();
    case 724:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\n') ADVANCE(24);
      if (lookahead == '*') ADVANCE(724);
      if (lookahead == '/') ADVANCE(2339);
      if (lookahead == '\\') ADVANCE(729);
      if (lookahead != 0) ADVANCE(725);
      END_STATE();
    case 725:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\n') ADVANCE(24);
      if (lookahead == '*') ADVANCE(724);
      if (lookahead == '/') ADVANCE(23);
      if (lookahead == '\\') ADVANCE(729);
      if (lookahead != 0) ADVANCE(725);
      END_STATE();
    case 726:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\r') ADVANCE(1231);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead != 0) ADVANCE(1230);
      END_STATE();
    case 727:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\r') ADVANCE(731);
      if (lookahead == '*') ADVANCE(721);
      if (lookahead == '/') ADVANCE(20);
      if (lookahead == '\\') ADVANCE(727);
      if (lookahead != 0) ADVANCE(722);
      END_STATE();
    case 728:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\r') ADVANCE(2345);
      if (lookahead == '/') ADVANCE(2343);
      if (lookahead == '\\') ADVANCE(2342);
      if (lookahead != 0) ADVANCE(2344);
      END_STATE();
    case 729:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\r') ADVANCE(733);
      if (lookahead == '*') ADVANCE(724);
      if (lookahead == '/') ADVANCE(23);
      if (lookahead == '\\') ADVANCE(729);
      if (lookahead != 0) ADVANCE(725);
      END_STATE();
    case 730:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '*') ADVANCE(722);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 731:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '*') ADVANCE(721);
      if (lookahead == '/') ADVANCE(20);
      if (lookahead == '\\') ADVANCE(727);
      if (lookahead != 0) ADVANCE(722);
      END_STATE();
    case 732:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '*') ADVANCE(725);
      if (lookahead == '/') ADVANCE(2343);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 733:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '*') ADVANCE(724);
      if (lookahead == '/') ADVANCE(23);
      if (lookahead == '\\') ADVANCE(729);
      if (lookahead != 0) ADVANCE(725);
      END_STATE();
    case 734:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(826);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 735:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1194);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 736:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(778);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 737:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1208);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 738:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(779);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 739:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(803);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 740:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1036);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 741:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1067);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 742:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1011);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 743:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(953);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 744:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(827);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 745:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1180);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 746:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1068);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 747:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1111);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 748:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(843);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 749:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1112);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 750:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1123);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 751:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1051);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 752:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1161);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 753:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1056);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 754:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1047);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 755:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1146);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 756:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1148);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 757:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1084);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 758:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1149);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 759:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(838);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 760:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(786);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 761:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1098);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 762:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1099);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 763:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1100);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 764:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1071);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 765:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1125);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 766:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1127);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 767:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1163);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 768:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1165);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 769:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1167);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 770:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(1186);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 771:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(844);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 772:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(845);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 773:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '_') ADVANCE(846);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 774:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == '+' ||
          lookahead == '-') ADVANCE(1228);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1293);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 775:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1072);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 776:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(805);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 777:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1213);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 778:
      ACCEPT_TOKEN(sym_preproc_arg);
      ADVANCE_MAP(
        '/', 587,
        '\\', 726,
        'A', 836,
        'a', 836,
        'D', 890,
        'd', 890,
        'G', 888,
        'g', 888,
        'I', 1122,
        'i', 1122,
        'M', 1058,
        'm', 1058,
        'S', 893,
        's', 893,
        'T', 954,
        't', 954,
      );
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 779:
      ACCEPT_TOKEN(sym_preproc_arg);
      ADVANCE_MAP(
        '/', 587,
        '\\', 726,
        'A', 808,
        'a', 808,
        'D', 797,
        'd', 797,
        'E', 1034,
        'e', 1034,
        'R', 873,
        'r', 873,
        'S', 1158,
        's', 1158,
        'T', 964,
        't', 964,
      );
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 780:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1073);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 781:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1195);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 782:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1093);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 783:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1000);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 784:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(820);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 785:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1025);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 786:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(833);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 787:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(824);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 788:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1081);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 789:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(813);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 790:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(814);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 791:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1197);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 792:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1198);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 793:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1199);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 794:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1200);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 795:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1201);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 796:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1202);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 797:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1217);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 798:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1218);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 799:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1219);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 800:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1220);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 801:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1221);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 802:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1222);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 803:
      ACCEPT_TOKEN(sym_preproc_arg);
      ADVANCE_MAP(
        '/', 587,
        '\\', 726,
        'A', 816,
        'a', 816,
        'C', 989,
        'c', 989,
        'D', 798,
        'd', 798,
        'E', 1045,
        'e', 1045,
        'R', 912,
        'r', 912,
        'S', 1174,
        's', 1174,
        'T', 967,
        't', 967,
      );
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 804:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1104);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 805:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(992);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 806:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(988);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 807:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1048);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 808:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1152);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 809:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1143);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 810:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(746);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 811:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(858);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 812:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(880);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 813:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(860);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 814:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(861);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 815:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(764);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 816:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1171);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 817:
      ACCEPT_TOKEN(sym_preproc_arg);
      ADVANCE_MAP(
        '/', 587,
        '\\', 726,
        'D', 848,
        'd', 848,
        'E', 991,
        'e', 991,
        'I', 917,
        'i', 917,
        'W', 775,
        'w', 775,
      );
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 818:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(715);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 819:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(718);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 820:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(637);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 821:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(677);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 822:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(679);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 823:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(665);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 824:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(639);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 825:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(743);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 826:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(905);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1046);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 827:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(777);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(872);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 828:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(783);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 829:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(851);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 830:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(745);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 831:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(748);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 832:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1089);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 833:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(832);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 834:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(751);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 835:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(862);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 836:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(830);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 837:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(877);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 838:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(907);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 839:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(909);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 840:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(768);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 841:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(769);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 842:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(914);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 843:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(799);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 844:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(800);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 845:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(801);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 846:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(802);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 847:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(771);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 848:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(918);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1108);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 849:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(719);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 850:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(712);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 851:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(710);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 852:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(661);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 853:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(645);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 854:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1124);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 855:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1223);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 856:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(673);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 857:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(675);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 858:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(667);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 859:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(643);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 860:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(705);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 861:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(703);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 862:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(671);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 863:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(699);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 864:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(701);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 865:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(695);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 866:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(697);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 867:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1159);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 868:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1212);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 869:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1114);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 870:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(818);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 871:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1078);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 872:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1147);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 873:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1109);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 874:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(819);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 875:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1102);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 876:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(737);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 877:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(740);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 878:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1113);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 879:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1086);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 880:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(823);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 881:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1144);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 882:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1145);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 883:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1074);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 884:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(752);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 885:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1075);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 886:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1076);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 887:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1077);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 888:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1153);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 889:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1088);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 890:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(995);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 891:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1092);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 892:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1087);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 893:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1154);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 894:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1091);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 895:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(834);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 896:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(840);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 897:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1156);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 898:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1215);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 899:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1128);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 900:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1160);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 901:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1094);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 902:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1095);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 903:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1096);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 904:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1097);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 905:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(924);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 906:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1130);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 907:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(926);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 908:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1132);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 909:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(927);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 910:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1133);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 911:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(767);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 912:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1140);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 913:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(841);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 914:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(928);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 915:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1103);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 916:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(770);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 917:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(734);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(806);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 918:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(955);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 919:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(717);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 920:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(922);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 921:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(956);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 922:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1120);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 923:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1121);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 924:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(965);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 925:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(923);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 926:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(968);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 927:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(969);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 928:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(987);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 929:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(714);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 930:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(946);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 931:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1080);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 932:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1116);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 933:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(741);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 934:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1117);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 935:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1118);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 936:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1119);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 937:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1126);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 938:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1129);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 939:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1131);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 940:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(947);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 941:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(948);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 942:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(949);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 943:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(950);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 944:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(951);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 945:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(952);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 946:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1157);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 947:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1162);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 948:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1164);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 949:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1166);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 950:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1168);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 951:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1169);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 952:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1170);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 953:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(919);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 954:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1008);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 955:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1020);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 956:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(933);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 957:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(993);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 958:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1187);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 959:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(930);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 960:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1021);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 961:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1053);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 962:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1055);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 963:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1023);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 964:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1009);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 965:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1026);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 966:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1196);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 967:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1010);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 968:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1027);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 969:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1037);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 970:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1035);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 971:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1038);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 972:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1039);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 973:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1040);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 974:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1041);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 975:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1042);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 976:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1012);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 977:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1013);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 978:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1014);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 979:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1015);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 980:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(940);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 981:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1203);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 982:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(941);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 983:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(942);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 984:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(943);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 985:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(944);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 986:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(945);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 987:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1044);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 988:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1175);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 989:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1225);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 990:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(663);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 991:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1105);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(825);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 992:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(876);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 993:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(990);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 994:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(959);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 995:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(867);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 996:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(863);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 997:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(864);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 998:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(865);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 999:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(866);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1000:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1062);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1001:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(980);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1002:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(982);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1003:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(983);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1004:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(984);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1005:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(985);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1006:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(986);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1007:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(931);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1008:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(869);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1009:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(854);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1010:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(855);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1011:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1059);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(925);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1012:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(899);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1013:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(906);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1014:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(908);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1015:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(910);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1016:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(713);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1017:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(649);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1018:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(647);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1019:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(669);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1020:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(850);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1021:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(929);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1022:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(810);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1023:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(932);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1024:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1050);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1025:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(828);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1026:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(870);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1027:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(874);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1028:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(856);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1029:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(857);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1030:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(859);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1031:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(921);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1032:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(960);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1033:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1057);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1034:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(831);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1035:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(934);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1036:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(900);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1150);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1037:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(896);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1038:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(935);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1039:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(936);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1040:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(937);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1041:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(938);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1042:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(939);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1043:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(815);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1044:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(913);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1045:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(847);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1046:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1151);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1047:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(920);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1048:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1031);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1049:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1178);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1050:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1205);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1051:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1177);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1052:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1101);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1053:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(821);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1054:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1079);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1055:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(822);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1056:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1019);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1057:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1206);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1058:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(837);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1059:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(835);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1060:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1028);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1061:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1029);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1062:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1030);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1063:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1181);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1064:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(625);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1065:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(784);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1066:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(789);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1067:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1085);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1068:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(892);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1069:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(787);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1070:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(790);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1071:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(894);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1072:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1016);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1073:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(590);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1074:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(685);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1075:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(687);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1076:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(691);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1077:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(693);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1078:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1188);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1079:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1224);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1080:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(736);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1081:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1032);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1082:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(811);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1083:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1179);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1084:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1182);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1085:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(958);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1086:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1106);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1087:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(961);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1088:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(749);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1089:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(878);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1090:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(812);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1091:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(962);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1092:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1189);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1093:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1172);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1094:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1190);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1095:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1191);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1096:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1192);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1097:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1193);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1098:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1183);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1099:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1184);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1100:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1185);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1101:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1226);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1102:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(839);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1103:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(842);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1104:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1173);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1105:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(849);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1106:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(689);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1107:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(629);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1108:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(776);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1109:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1214);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1110:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(781);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1111:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1049);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1112:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1155);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1113:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1107);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1114:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(871);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1115:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(875);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1116:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(753);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1117:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(754);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1118:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(742);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1119:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(757);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1120:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(881);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1121:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(882);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1122:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(744);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1123:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1066);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1124:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(891);
      if (lookahead == 'Z' ||
          lookahead == 'z') ADVANCE(1060);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1125:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1063);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1126:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(761);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1127:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1070);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1128:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(901);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1129:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(762);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1130:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(902);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1131:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(763);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1132:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(903);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1133:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(904);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1134:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(791);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1135:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(792);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1136:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(793);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1137:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(794);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1138:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(795);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1139:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(796);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1140:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1216);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1141:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(915);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1142:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(653);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1143:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(655);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1144:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(681);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1145:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(683);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1146:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1209);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1147:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1204);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1148:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1210);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1149:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1211);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1150:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(785);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1151:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(759);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1152:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(966);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1153:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(738);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1154:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(739);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1155:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1083);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1156:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(760);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1157:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1110);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1158:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(782);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1159:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(916);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1160:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1207);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1161:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(976);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1162:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1134);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1163:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(977);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1164:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1135);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1165:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(978);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1166:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1136);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1167:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(979);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1168:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1137);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1169:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1138);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1170:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1139);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1171:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(981);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1172:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(772);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1173:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(773);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1174:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(804);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1175:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(829);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1176:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1064);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1177:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1142);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1178:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1082);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1179:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(809);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1180:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1115);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1181:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1090);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1182:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(996);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1183:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(997);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1184:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(998);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1185:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(999);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1186:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1141);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1187:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(651);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1188:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(889);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1189:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(879);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1190:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(883);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1191:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(885);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1192:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(886);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1193:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(887);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1194:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(780);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1195:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(963);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1196:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(884);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1197:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(970);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1198:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(971);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1199:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(972);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1200:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(973);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1201:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(974);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1202:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(975);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1203:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(911);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1204:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1052);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1205:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1017);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1206:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1018);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1207:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1054);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1208:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(788);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1209:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(631);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1210:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(633);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1211:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(635);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1212:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1065);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1213:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(994);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1214:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1022);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1215:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1069);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1216:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1043);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1217:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1001);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1218:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1002);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1219:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1003);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1220:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1004);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1221:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1005);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1222:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1006);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1223:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'Z' ||
          lookahead == 'z') ADVANCE(1061);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1224:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'K' ||
          lookahead == 'k' ||
          lookahead == 0x212a) ADVANCE(641);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1225:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'K' ||
          lookahead == 'k' ||
          lookahead == 0x212a) ADVANCE(747);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1226:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead == 'K' ||
          lookahead == 'k' ||
          lookahead == 0x212a) ADVANCE(765);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1227:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1291);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1228:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1293);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1229:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(1294);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1230:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(1230);
      END_STATE();
    case 1231:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(587);
      if (lookahead == '\\') ADVANCE(726);
      if (lookahead != 0) ADVANCE(1230);
      END_STATE();
    case 1232:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      END_STATE();
    case 1233:
      ACCEPT_TOKEN(anon_sym_RBRACK);
      END_STATE();
    case 1234:
      ACCEPT_TOKEN(anon_sym_LBRACE);
      END_STATE();
    case 1235:
      ACCEPT_TOKEN(anon_sym_RBRACE);
      END_STATE();
    case 1236:
      ACCEPT_TOKEN(anon_sym_COLON);
      END_STATE();
    case 1237:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      if (lookahead == '*') ADVANCE(722);
      END_STATE();
    case 1238:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      if (lookahead == '*') ADVANCE(21);
      END_STATE();
    case 1239:
      ACCEPT_TOKEN(anon_sym_RPAREN);
      END_STATE();
    case 1240:
      ACCEPT_TOKEN(anon_sym_SEMI);
      END_STATE();
    case 1241:
      ACCEPT_TOKEN(anon_sym_BANG);
      END_STATE();
    case 1242:
      ACCEPT_TOKEN(anon_sym_BANG);
      if (lookahead == '=') ADVANCE(1260);
      END_STATE();
    case 1243:
      ACCEPT_TOKEN(anon_sym_TILDE);
      END_STATE();
    case 1244:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '-') ADVANCE(1268);
      END_STATE();
    case 1245:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '-') ADVANCE(1268);
      if (lookahead == '.') ADVANCE(1227);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1289);
      END_STATE();
    case 1246:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '-') ADVANCE(1268);
      if (lookahead == '.') ADVANCE(585);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1290);
      END_STATE();
    case 1247:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '+') ADVANCE(1269);
      END_STATE();
    case 1248:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '+') ADVANCE(1269);
      if (lookahead == '.') ADVANCE(1227);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1289);
      END_STATE();
    case 1249:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '+') ADVANCE(1269);
      if (lookahead == '.') ADVANCE(585);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1290);
      END_STATE();
    case 1250:
      ACCEPT_TOKEN(anon_sym_STAR);
      END_STATE();
    case 1251:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '*') ADVANCE(725);
      if (lookahead == '/') ADVANCE(2343);
      END_STATE();
    case 1252:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '*') ADVANCE(24);
      if (lookahead == '/') ADVANCE(2346);
      END_STATE();
    case 1253:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      END_STATE();
    case 1254:
      ACCEPT_TOKEN(anon_sym_PIPE_PIPE);
      END_STATE();
    case 1255:
      ACCEPT_TOKEN(anon_sym_AMP_AMP);
      END_STATE();
    case 1256:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (lookahead == '|') ADVANCE(1254);
      END_STATE();
    case 1257:
      ACCEPT_TOKEN(anon_sym_CARET);
      END_STATE();
    case 1258:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&') ADVANCE(1255);
      END_STATE();
    case 1259:
      ACCEPT_TOKEN(anon_sym_EQ_EQ);
      END_STATE();
    case 1260:
      ACCEPT_TOKEN(anon_sym_BANG_EQ);
      END_STATE();
    case 1261:
      ACCEPT_TOKEN(anon_sym_LT_GT);
      END_STATE();
    case 1262:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(1263);
      if (lookahead == '>') ADVANCE(1267);
      END_STATE();
    case 1263:
      ACCEPT_TOKEN(anon_sym_GT_EQ);
      END_STATE();
    case 1264:
      ACCEPT_TOKEN(anon_sym_LT_EQ);
      END_STATE();
    case 1265:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<') ADVANCE(1266);
      if (lookahead == '=') ADVANCE(1264);
      if (lookahead == '>') ADVANCE(1261);
      END_STATE();
    case 1266:
      ACCEPT_TOKEN(anon_sym_LT_LT);
      END_STATE();
    case 1267:
      ACCEPT_TOKEN(anon_sym_GT_GT);
      END_STATE();
    case 1268:
      ACCEPT_TOKEN(anon_sym_DASH_DASH);
      END_STATE();
    case 1269:
      ACCEPT_TOKEN(anon_sym_PLUS_PLUS);
      END_STATE();
    case 1270:
      ACCEPT_TOKEN(anon_sym_DOT);
      END_STATE();
    case 1271:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(1274);
      END_STATE();
    case 1272:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(1274);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1291);
      END_STATE();
    case 1273:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(1274);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1292);
      END_STATE();
    case 1274:
      ACCEPT_TOKEN(sym_range_operator);
      END_STATE();
    case 1275:
      ACCEPT_TOKEN(anon_sym_DQUOTE);
      END_STATE();
    case 1276:
      ACCEPT_TOKEN(anon_sym_SQUOTE);
      END_STATE();
    case 1277:
      ACCEPT_TOKEN(anon_sym_SQUOTE);
      if (lookahead == '\'') ADVANCE(1288);
      END_STATE();
    case 1278:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead == '\r') ADVANCE(1286);
      if (lookahead == '\\') ADVANCE(1278);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\'') ADVANCE(1286);
      END_STATE();
    case 1279:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead == '(') ADVANCE(1282);
      if (lookahead == '/') ADVANCE(1285);
      if (lookahead == '\t' ||
          (0x0b <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(1279);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead) &&
          lookahead != '\'' &&
          lookahead != '(') ADVANCE(1287);
      END_STATE();
    case 1280:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead == '*') ADVANCE(1280);
      if (lookahead == ')' ||
          lookahead == '/') ADVANCE(1287);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\'') ADVANCE(1281);
      END_STATE();
    case 1281:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead == '*') ADVANCE(1280);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\'') ADVANCE(1281);
      END_STATE();
    case 1282:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead == '*') ADVANCE(1281);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\'') ADVANCE(1287);
      END_STATE();
    case 1283:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead == '*') ADVANCE(1283);
      if (lookahead == '/') ADVANCE(1287);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\'') ADVANCE(1284);
      END_STATE();
    case 1284:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead == '*') ADVANCE(1283);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\'') ADVANCE(1284);
      END_STATE();
    case 1285:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead == '*') ADVANCE(1284);
      if (lookahead == '/') ADVANCE(1286);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\'') ADVANCE(1287);
      END_STATE();
    case 1286:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead == '\\') ADVANCE(1278);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\'') ADVANCE(1286);
      END_STATE();
    case 1287:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\'') ADVANCE(1287);
      END_STATE();
    case 1288:
      ACCEPT_TOKEN(sym_escape_sequence);
      END_STATE();
    case 1289:
      ACCEPT_TOKEN(sym_number_literal);
      if (lookahead == '.') ADVANCE(1291);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(774);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1289);
      END_STATE();
    case 1290:
      ACCEPT_TOKEN(sym_number_literal);
      if (lookahead == '.') ADVANCE(1292);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(70);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1290);
      END_STATE();
    case 1291:
      ACCEPT_TOKEN(sym_number_literal);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(774);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1291);
      END_STATE();
    case 1292:
      ACCEPT_TOKEN(sym_number_literal);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(70);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1292);
      END_STATE();
    case 1293:
      ACCEPT_TOKEN(sym_number_literal);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(1293);
      END_STATE();
    case 1294:
      ACCEPT_TOKEN(sym_number_literal);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(1294);
      END_STATE();
    case 1295:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1458);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1296:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1660);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1297:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1464);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1298:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2033);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1299:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1413);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1300:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1566);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1301:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1480);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1302:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1513);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1303:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1967);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1304:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1456);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1305:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1849);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1306:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1430);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1307:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1493);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1308:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1905);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1309:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2155);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1310:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1466);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1311:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1727);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1312:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2297);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1313:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1531);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1314:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1414);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1315:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1431);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1316:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1577);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1317:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2296);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1318:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2018);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1319:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1977);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1320:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2240);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1321:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2108);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1322:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2208);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1323:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2105);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1324:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1564);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1325:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1586);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1326:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2118);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1327:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1983);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1328:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1982);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1329:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1482);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1330:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1568);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1331:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2211);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1332:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2052);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1333:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1441);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1334:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2174);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1335:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2177);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1336:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2179);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1337:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1415);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1338:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2235);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1339:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2022);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1340:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1514);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1341:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1732);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1342:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2106);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1343:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2267);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1344:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2244);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1345:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2123);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1346:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1863);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1347:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1793);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1348:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2104);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1349:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2247);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1350:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1866);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1351:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1995);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1352:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1440);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1353:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2129);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1354:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1864);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1355:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1670);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1356:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2298);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1357:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2246);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1358:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2078);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1359:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2256);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1360:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2024);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1361:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1733);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1362:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2132);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1363:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1675);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1364:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2300);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1365:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2081);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1366:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1678);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1367:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2302);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1368:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2082);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1369:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1680);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1370:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1465);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1371:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1497);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1372:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2130);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1373:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1519);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1374:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1661);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1375:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1567);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1376:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1457);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1377:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2140);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1378:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2212);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1379:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1580);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1380:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2213);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1381:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2257);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1382:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2150);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1383:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1737);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1384:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2215);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1385:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2214);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1386:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2216);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1387:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2200);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1388:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2219);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1389:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1584);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1390:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1828);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1391:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1585);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1392:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1738);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1393:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1587);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1394:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1739);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1395:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1588);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1396:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(1589);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1397:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2157);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1398:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2261);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1399:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '_') ADVANCE(2034);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1400:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(1007);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1896);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1401:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(592);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(593);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1402:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(659);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(660);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1403:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(707);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(708);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1404:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(657);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(658);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1405:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(623);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(624);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1406:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(627);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(628);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1407:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(641);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(642);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1408:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(338);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1915);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1409:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(807);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1545);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1410:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(109);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1545);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1411:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(735);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1343);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1412:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(29);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1343);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1413:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(957);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2276);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1789);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1414:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(282);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2276);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1789);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1415:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(282);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1789);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1416:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(1176);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(2238);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1417:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(529);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(2238);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1418:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(1024);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1929);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1419:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(360);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1929);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1420:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(852);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1596);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1421:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(159);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1596);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1422:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(853);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1597);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1423:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(160);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1597);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1424:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(747);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1321);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1425:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(42);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1321);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1426:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(750);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1326);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1427:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(895);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1628);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1428:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(45);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1326);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1429:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(212);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1628);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1430:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(868);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1658);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1614);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1431:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(183);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1658);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1685);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1432:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(755);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1334);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1433:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(50);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1334);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1434:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(756);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1335);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1435:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(51);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1335);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1436:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(758);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1336);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1437:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(53);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1336);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1438:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(1033);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1948);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1439:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(374);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1948);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1440:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(898);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1669);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1441:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(216);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1669);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1442:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(897);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1682);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1443:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(214);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1682);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1444:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(344);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1904);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1445:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(765);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1377);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1446:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(60);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1377);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1447:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(766);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1372);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1448:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(61);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1372);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1449:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 0x212a) ADVANCE(347);
      if (lookahead == 'K' ||
          lookahead == 'k') ADVANCE(1906);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1450:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2171);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2264);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1662);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1451:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1526);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1784);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1452:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2310);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1453:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1921);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1454:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1401);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1455:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1524);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1456:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2308);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2020);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1457:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2308);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2030);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1458:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2266);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1452);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1459:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2321);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1460:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2100);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1461:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1296);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1462:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1923);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1463:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1420);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1464:
      ACCEPT_TOKEN(sym_identifier);
      ADVANCE_MAP(
        'A', 1563,
        'a', 1563,
        'D', 1627,
        'd', 1627,
        'G', 1671,
        'g', 1671,
        'I', 2112,
        'i', 2112,
        'M', 1999,
        'm', 1999,
        'S', 1676,
        's', 1676,
        'T', 1798,
        't', 1798,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1465:
      ACCEPT_TOKEN(sym_identifier);
      ADVANCE_MAP(
        'A', 1563,
        'a', 1563,
        'D', 1627,
        'd', 1627,
        'G', 1671,
        'g', 1671,
        'I', 2151,
        'i', 2151,
        'M', 1999,
        'm', 1999,
        'S', 1683,
        's', 1683,
        'T', 1798,
        't', 1798,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1466:
      ACCEPT_TOKEN(sym_identifier);
      ADVANCE_MAP(
        'A', 1563,
        'a', 1563,
        'D', 1627,
        'd', 1627,
        'G', 1671,
        'g', 1671,
        'I', 2151,
        'i', 2151,
        'M', 1999,
        'm', 1999,
        'S', 1683,
        's', 1683,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1467:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2107);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1468:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1558);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1469:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2036);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1470:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1951);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1471:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1878);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1472:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1562);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1473:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2068);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1474:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1421);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1475:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1423);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1476:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1309);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1477:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1787);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1478:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2270);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1479:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2117);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1480:
      ACCEPT_TOKEN(sym_identifier);
      ADVANCE_MAP(
        'A', 1536,
        'a', 1536,
        'D', 1520,
        'd', 1520,
        'E', 1928,
        'e', 1928,
        'R', 1621,
        'r', 1621,
        'S', 2195,
        's', 2195,
        'T', 1803,
        't', 1803,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1481:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1422);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1482:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1570);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1483:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1856);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1484:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2109);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1485:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1872);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1486:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2113);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1487:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1543);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1488:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1853);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1489:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1535);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1490:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1544);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1491:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2189);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1492:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2194);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2287);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1715);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1493:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2311);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1534);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1768);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1494:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1869);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1495:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2272);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1496:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2202);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1497:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2315);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1551);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1772);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1498:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2283);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1499:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2284);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1500:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2285);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1501:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2286);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1502:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2314);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1503:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1879);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1504:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2142);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1505:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1525);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1506:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1374);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1507:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2288);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1508:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2152);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1509:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1827);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1510:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2224);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2287);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1715);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1511:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2329);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1512:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2156);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1513:
      ACCEPT_TOKEN(sym_identifier);
      ADVANCE_MAP(
        'A', 1554,
        'a', 1554,
        'C', 1862,
        'c', 1862,
        'D', 1511,
        'd', 1511,
        'E', 1972,
        'e', 1972,
        'R', 1711,
        'r', 1711,
        'S', 2231,
        's', 2231,
        'T', 1805,
        't', 1805,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1514:
      ACCEPT_TOKEN(sym_identifier);
      ADVANCE_MAP(
        'A', 1554,
        'a', 1554,
        'C', 1854,
        'c', 1854,
        'D', 1511,
        'd', 1511,
        'E', 1972,
        'e', 1972,
        'R', 1711,
        'r', 1711,
        'S', 2231,
        's', 2231,
        'T', 1805,
        't', 1805,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1515:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2330);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1516:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2331);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1517:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2332);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1518:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2333);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1519:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2292);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1502);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1520:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2334);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1521:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2094);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1522:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1895);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('B' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('b' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1523:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2318);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1524:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1855);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1525:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1882);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1526:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1411);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1556);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1527:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1839);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1622);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1528:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1839);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1722);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1529:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1839);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1530:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1403);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1531:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1843);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1532:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2193);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1533:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1442);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1534:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2103);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1535:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1412);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1536:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2180);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1537:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1429);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1538:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2167);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1539:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1443);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1540:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1339);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1541:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1602);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1542:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1634);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1543:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1604);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1544:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1605);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1545:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1985);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1546:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1630);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1547:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1631);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1548:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1427);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1549:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1691);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1550:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2203);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1551:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2139);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1552:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1360);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1553:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1712);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1554:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2227);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1555:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(605);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1556:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1523);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1557:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(619);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1558:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(638);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1559:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(678);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1560:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(680);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1561:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(666);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1562:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(640);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1563:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1579);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1564:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1788);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1565:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1593);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1566:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1459);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1686);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1567:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1459);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1697);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1568:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1801);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1569:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1471);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1570:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1571);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1571:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2060);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1572:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1325);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1573:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1620);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1574:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1659);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1575:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1606);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1576:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1327);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1577:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1616);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1578:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1642);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1579:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1359);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1580:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1819);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1581:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1703);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1582:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1385);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1583:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1386);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1584:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1824);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1585:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1710);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1586:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1515);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1587:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1516);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1588:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1517);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1589:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1518);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1590:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1393);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1591:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1716);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1592:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(611);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1593:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(711);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1594:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(613);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1595:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(603);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1596:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(662);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1597:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(646);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1598:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2131);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1599:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2336);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1600:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(674);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1601:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(676);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1602:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(668);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1603:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(644);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1604:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(706);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1605:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(704);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1606:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(672);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1607:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(700);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1608:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(702);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1609:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(696);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1610:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(698);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1611:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1454);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1612:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2265);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1613:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2172);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1614:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2312);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1615:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2159);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2049);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1616:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2289);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1617:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1484);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1618:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1402);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1619:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1876);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1620:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1735);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1621:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2101);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1622:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2050);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1623:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2254);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1624:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1342);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1625:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1927);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1626:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1404);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1627:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1861);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1628:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1576);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1629:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2188);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1630:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1317);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1631:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1322);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1632:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2111);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1633:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2069);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1634:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1561);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1635:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2067);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1636:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1618);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1637:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1626);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1638:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1299);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1639:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1320);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1640:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2056);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1641:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2055);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1642:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1303);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1643:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2037);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1644:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1304);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1645:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2038);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1646:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2168);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1647:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2169);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1648:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1306);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1649:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2039);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1650:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2040);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1651:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1323);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1652:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1331);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1653:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1347);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1654:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1314);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1655:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1315);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1656:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1337);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1657:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1845);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1658:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2319);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1659:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2309);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1660:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2268);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2183);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2317);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1661:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2268);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2225);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2325);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1662:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2178);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1663:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1847);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1664:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1932);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1665:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1582);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1666:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2058);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1667:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1344);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1668:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2064);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1669:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2322);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1670:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2269);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1671:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2185);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1672:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1933);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1673:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2065);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1674:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1349);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1675:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2271);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1676:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2187);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1677:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1936);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1678:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2273);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1679:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1937);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1680:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2274);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1681:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1938);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1682:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2191);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1683:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2197);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1684:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2122);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1685:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2313);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1686:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2199);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1687:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2070);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1688:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1357);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1689:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2079);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1690:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2204);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1691:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1364);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1692:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2071);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1693:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2134);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1694:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2206);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2086);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1695:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2072);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1696:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2136);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1697:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2207);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1698:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2073);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1699:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2137);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1700:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2074);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1701:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2138);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1702:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1880);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1703:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2316);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1704:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1883);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1705:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1881);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1706:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1376);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1707:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1380);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1708:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1383);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1709:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2217);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1710:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2291);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1711:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2153);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1712:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1388);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1713:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1583);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1714:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1390);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1715:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2226);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1716:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1740);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1717:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1394);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1718:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2092);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1719:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1397);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1720:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2260);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1721:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1398);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1722:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2095);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1723:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1724);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(606);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1724:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(608);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1725:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(609);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1726:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1800);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1727:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1840);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1728:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1731);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1729:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1725);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(607);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1730:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1312);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1731:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2124);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1732:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1850);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1733:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1865);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1734:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2127);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1735:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1807);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1736:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1734);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1737:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1885);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1738:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1891);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1739:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1892);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1740:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1834);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1741:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(617);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1742:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1767);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1743:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2053);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1744:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2120);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1745:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1348);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1746:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1318);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1747:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2115);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1748:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2116);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1749:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2061);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1750:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2087);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1751:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2125);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1752:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2128);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1753:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2133);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1754:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2135);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1755:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2083);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1756:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1771);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1757:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1382);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1758:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1774);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1759:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1775);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1760:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1776);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1761:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1777);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1762:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1779);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1763:
      ACCEPT_TOKEN(sym_identifier);
      ADVANCE_MAP(
        'H', 1453,
        'h', 1453,
        'L', 1400,
        'l', 1400,
        'O', 1898,
        'o', 1898,
        'U', 2102,
        'u', 2102,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1764:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1453);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1408);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2102);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1765:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1453);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1766:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(601);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1767:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2181);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1768:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1486);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1769:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1324);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1770:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1330);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1771:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2218);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1772:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1504);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1773:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1379);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1774:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2220);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1775:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2221);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1776:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2222);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1777:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2223);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1778:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1389);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1779:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2228);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1780:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1842);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1781:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1546);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1782:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1899);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1783:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2335);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1784:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1920);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1785:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1742);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1786:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2263);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1787:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1846);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1788:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2121);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1789:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1851);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1790:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1990);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1791:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1991);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1792:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1934);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1793:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1955);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1794:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1924);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1795:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1925);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1796:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1954);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1797:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1940);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1798:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1900);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1799:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1931);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1800:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1746);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1801:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2114);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1802:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1941);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1803:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1901);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1804:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1935);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1805:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1902);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1806:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1956);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1807:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1945);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1808:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2279);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1809:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1961);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1810:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1957);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1811:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1958);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1812:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1962);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1813:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1959);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1814:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1963);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1815:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1964);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1816:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1549);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1817:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1965);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1818:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1908);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1819:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2143);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1820:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1911);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1821:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1912);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1822:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1913);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1823:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1907);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1824:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2145);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1825:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1909);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1826:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1756);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1827:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1884);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1828:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1970);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1829:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2290);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1830:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1758);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1831:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1759);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1832:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1760);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1833:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1761);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1834:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1971);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1835:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2337);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1836:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1762);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1837:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1555);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1838:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(664);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1839:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2236);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1840:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1745);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1841:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1311);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1842:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1624);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1843:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1979);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1844:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1785);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1845:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1617);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1846:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1455);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1847:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1355);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1848:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1792);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1849:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1989);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1467);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1850:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1460);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1851:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1838);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1852:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1408);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1853:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1298);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1854:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1425);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1855:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1717);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1856:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1437);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1857:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1607);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1858:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1608);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1859:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1609);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1860:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1610);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1861:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1629);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1862:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1424);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1863:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2001);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1864:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1992);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1467);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1865:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1479);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1866:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2005);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1867:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2008);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1868:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1796);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1869:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1436);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1870:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1797);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1871:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1802);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1872:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1356);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1873:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1444);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1874:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2010);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1875:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1810);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1876:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1806);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1877:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1449);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1878:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2009);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1879:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1367);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1880:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1811);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1881:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1813);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1882:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1708);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1883:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1369);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1884:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1505);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1885:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1508);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1886:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1826);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1887:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1830);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1888:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1831);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1889:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1832);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1890:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1833);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1891:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1757);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1892:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1512);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1893:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1392);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1894:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1836);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1895:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1399);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1896:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1743);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1897:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1485);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1898:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1903);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1899:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1619);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1900:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1684);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1901:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1598);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1902:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1599);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1903:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1462);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1904:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1749);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1905:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2004);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1736);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1906:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1750);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1907:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1702);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1908:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1693);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1909:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1705);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1910:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1366);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1911:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1696);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1912:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1699);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1913:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1701);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1914:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1503);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1915:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1755);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1916:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1527);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1917:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(650);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1918:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(648);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1919:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(670);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1920:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1741);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1921:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1969);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1922:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1726);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1923:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1557);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1924:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1744);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1925:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1747);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1926:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1540);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1927:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2160);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1928:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1572);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1929:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1978);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1930:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1418);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1931:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1592);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1932:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2161);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1933:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2162);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1934:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1406);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1935:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1594);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1936:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2163);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1937:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2164);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1938:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2165);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1939:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1419);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1940:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1433);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1941:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1435);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1942:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2232);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1943:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1600);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1944:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1601);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1945:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1665);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1946:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1603);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1947:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1529);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1948:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2000);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1949:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2184);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1950:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1438);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1951:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1569);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1952:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1439);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1953:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2196);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1954:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1432);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1955:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1574);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1956:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1638);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1957:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1434);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1958:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1654);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1959:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1656);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1960:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1488);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1961:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1748);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1962:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1752);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1963:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1753);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1964:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1754);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1965:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1751);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1966:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1363);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1967:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1690);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2198);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1968:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1552);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1969:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1704);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1970:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1581);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1971:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1713);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1972:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1590);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1973:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1522);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1974:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1528);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1975:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2239);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1451);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1976:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1837);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1977:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1730);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1978:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2294);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1979:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1530);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1980:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1966);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1981:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1910);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1982:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1728);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1983:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2243);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1984:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1313);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1985:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1922);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1986:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2048);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1987:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1416);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1988:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2035);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1989:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1987);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1990:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1559);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1991:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1560);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1992:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1996);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1993:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2085);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1994:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2057);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1995:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1919);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1996:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1417);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1997:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2059);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1998:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2084);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 1999:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1578);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2000:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2295);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2001:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1548);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2002:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1533);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2003:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1943);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2004:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1575);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2005:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1537);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2006:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1944);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2007:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1539);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2008:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1949);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2009:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1946);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2010:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1953);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2011:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2245);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2012:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2249);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2013:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2259);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1489);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2014:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2259);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2015:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(626);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2016:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1468);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2017:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1487);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2018:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2054);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2019:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1472);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2020:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1345);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2021:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1639);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2022:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1640);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2023:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1644);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2024:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1668);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2025:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1648);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2026:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1667);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2027:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1655);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2028:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1867);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2029:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1490);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2030:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1362);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2031:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1874);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2032:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1706);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2033:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(2242);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2034:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(2262);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2035:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(615);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2036:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(591);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2037:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(686);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2038:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(688);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2039:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(692);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2040:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(694);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2041:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1984);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2042:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1338);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2043:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2042);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2044:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1841);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2045:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1611);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2176);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2046:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1611);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2047:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2237);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2048:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1409);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2049:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1897);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2050:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1960);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2051:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2063);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2052:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2250);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2053:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1297);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2054:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1786);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2055:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2098);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2056:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1790);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2057:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1407);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2058:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1346);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2059:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1410);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2060:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1632);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2061:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1310);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2062:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1541);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2063:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1988);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2064:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1791);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2065:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1353);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2066:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1542);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2067:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1573);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2068:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2229);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2069:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2282);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2070:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2275);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2071:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2277);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2072:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2278);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2073:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2280);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2074:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2281);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2075:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1674);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2076:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1547);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2077:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2248);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2078:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2251);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2079:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1350);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2080:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1688);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2081:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2252);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2082:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2253);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2083:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1370);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2084:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1446);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2085:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1445);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2086:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1914);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2087:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1387);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2088:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1381);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2089:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2088);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2090:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2258);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2091:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1553);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2092:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1591);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2093:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1893);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2094:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2230);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2095:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1973);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2096:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2175);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2097:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1766);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2098:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(690);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2099:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(630);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2100:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1769);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2101:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2320);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2102:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2173);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2103:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2028);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2104:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2182);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2105:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1783);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2106:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1636);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2107:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1405);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2108:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2011);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2109:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1595);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2110:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1478);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2111:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2099);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2112:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1300);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2113:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2210);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2114:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1428);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2115:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1308);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2116:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1332);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2117:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1770);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2118:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2017);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2119:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1666);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2120:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1351);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2121:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1426);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2122:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1633);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2123:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2002);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2124:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1646);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2125:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1328);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2126:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1635);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2127:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1647);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2128:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1358);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2129:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2201);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2130:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2029);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2131:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1687);
      if (lookahead == 'Z' ||
          lookahead == 'z') ADVANCE(2003);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Y') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'y')) ADVANCE(2338);
      END_STATE();
    case 2132:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2007);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2133:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1365);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2134:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1692);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2135:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1368);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2136:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1695);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2137:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1698);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2138:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1700);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2139:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2031);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2140:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2012);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2141:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1495);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2142:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2205);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2143:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1448);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2144:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1689);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2145:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1447);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2146:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1498);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2147:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1499);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2148:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1500);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2149:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1501);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2150:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2209);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2151:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1375);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2152:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1773);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2153:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2327);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2154:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1507);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2155:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2225);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2325);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2156:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1778);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2157:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1835);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2158:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1718);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2159:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2293);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2160:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(597);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2161:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(596);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2162:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(594);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2163:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(599);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2164:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(595);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2165:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(598);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2166:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(654);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2167:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(656);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2168:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(682);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2169:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(684);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2170:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1980);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2171:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1461);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1319);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2172:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1295);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2173:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1981);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2174:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2305);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2175:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2041);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2176:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2170);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2177:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2306);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2178:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1316);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2179:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2307);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2180:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1808);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2181:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2110);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2182:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1491);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2183:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2047);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2184:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1494);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2185:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1301);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2186:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1489);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2187:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1302);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2188:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1721);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2189:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1305);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2190:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1352);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2191:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1329);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2192:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1333);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2193:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2241);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2194:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1476);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1319);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2195:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1473);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2196:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1483);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2197:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1340);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2198:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1470);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2199:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2299);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2200:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1798);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2201:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2077);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2202:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1354);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2203:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2255);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2204:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2301);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2205:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1871);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2206:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2303);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2207:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2304);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2208:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2323);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2209:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1496);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2210:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1875);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2211:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1818);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2212:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2324);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2213:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1820);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2214:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1821);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2215:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2326);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2216:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1822);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2217:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1373);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2218:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2141);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2219:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2328);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2220:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2146);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2221:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2147);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2222:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2148);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2223:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2149);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2224:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1506);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1319);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2225:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2090);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2226:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1391);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2227:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1829);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2228:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2154);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2229:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1395);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2230:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1396);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2231:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1521);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2232:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1722);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2233:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2097);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2234:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2176);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2235:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2119);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2236:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1565);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2237:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1532);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2238:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2015);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2239:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2076);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2240:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1930);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2241:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2080);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2242:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1623);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2243:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2166);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2244:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1939);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2245:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2062);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2246:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1950);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2247:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1952);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2248:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1538);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2249:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2066);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2250:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1857);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2251:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1858);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2252:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1859);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2253:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1860);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2254:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1651);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2255:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2075);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2256:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2126);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2257:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2144);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2258:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1550);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2259:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2091);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2260:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1719);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2261:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2158);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2262:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1720);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2263:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(652);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2264:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1781);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2265:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1663);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2266:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1477);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2267:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1469);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2268:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1625);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2269:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1664);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2270:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1794);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2271:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1672);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2272:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1795);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2273:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1677);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2274:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1679);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2275:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1641);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2276:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1681);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2277:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1643);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2278:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1645);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2279:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1652);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2280:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1649);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2281:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1650);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2282:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1673);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2283:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1809);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2284:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1812);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2285:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1814);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2286:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1815);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2287:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1816);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2288:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1817);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2289:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1378);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2290:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1707);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2291:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1384);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2292:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1509);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2293:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1986);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2294:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1917);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2295:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1918);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2296:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1463);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2297:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1637);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2298:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1481);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2299:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1993);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2300:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1474);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2301:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1994);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2302:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1475);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2303:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1997);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2304:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1998);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2305:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(632);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2306:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(634);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2307:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(636);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2308:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1848);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2309:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1307);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2310:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1341);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2311:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1868);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2312:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(2190);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2313:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(2192);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2314:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1361);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2315:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1870);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2316:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1371);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2317:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2021);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2318:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(621);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2319:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2016);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2320:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1926);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2321:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1844);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2322:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2019);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2323:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2023);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2324:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2025);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2325:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2026);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2326:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2027);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2327:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1968);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2328:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2032);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2329:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1886);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2330:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1887);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2331:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1888);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2332:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1889);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2333:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1890);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2334:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1894);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2335:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Z' ||
          lookahead == 'z') ADVANCE(1653);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Y') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'y')) ADVANCE(2338);
      END_STATE();
    case 2336:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Z' ||
          lookahead == 'z') ADVANCE(2006);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Y') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'y')) ADVANCE(2338);
      END_STATE();
    case 2337:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'Z' ||
          lookahead == 'z') ADVANCE(1714);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Y') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'y')) ADVANCE(2338);
      END_STATE();
    case 2338:
      ACCEPT_TOKEN(sym_identifier);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(2338);
      END_STATE();
    case 2339:
      ACCEPT_TOKEN(sym_comment);
      END_STATE();
    case 2340:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '\n') ADVANCE(2346);
      if (lookahead == '/') ADVANCE(2343);
      if (lookahead == '\\') ADVANCE(728);
      if (lookahead != 0) ADVANCE(2344);
      END_STATE();
    case 2341:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '\r') ADVANCE(2347);
      if (lookahead == '\\') ADVANCE(2341);
      if (lookahead != 0) ADVANCE(2346);
      END_STATE();
    case 2342:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '\r') ADVANCE(2345);
      if (lookahead == '/') ADVANCE(2343);
      if (lookahead == '\\') ADVANCE(2342);
      if (lookahead != 0) ADVANCE(2344);
      END_STATE();
    case 2343:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '*') ADVANCE(2346);
      if (lookahead == '\\') ADVANCE(723);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(2344);
      END_STATE();
    case 2344:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '/') ADVANCE(2343);
      if (lookahead == '\\') ADVANCE(728);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(2344);
      END_STATE();
    case 2345:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '/') ADVANCE(2343);
      if (lookahead == '\\') ADVANCE(728);
      if (lookahead != 0) ADVANCE(2344);
      END_STATE();
    case 2346:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '\\') ADVANCE(6);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(2346);
      END_STATE();
    case 2347:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '\\') ADVANCE(6);
      if (lookahead != 0) ADVANCE(2346);
      END_STATE();
    default:
      return false;
  }
}

static bool ts_lex_keywords(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      ADVANCE_MAP(
        '_', 1,
        'A', 2,
        'a', 2,
        'B', 3,
        'b', 3,
        'C', 4,
        'c', 4,
        'D', 5,
        'd', 5,
        'E', 6,
        'e', 6,
        'F', 7,
        'f', 7,
        'G', 8,
        'g', 8,
        'H', 9,
        'h', 9,
        'I', 10,
        'i', 10,
        'L', 11,
        'l', 11,
        'M', 12,
        'm', 12,
        'N', 13,
        'n', 13,
        'O', 14,
        'o', 14,
        'P', 15,
        'p', 15,
        'R', 16,
        'r', 16,
        'S', 17,
        's', 17,
        'T', 18,
        't', 18,
        'U', 19,
        'u', 19,
        'V', 20,
        'v', 20,
        'W', 21,
        'w', 21,
        'X', 22,
        'x', 22,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(0);
      END_STATE();
    case 1:
      if (lookahead == '_') ADVANCE(23);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(24);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(25);
      END_STATE();
    case 2:
      ADVANCE_MAP(
        'B', 26,
        'b', 26,
        'C', 27,
        'c', 27,
        'D', 28,
        'd', 28,
        'M', 29,
        'm', 29,
        'N', 30,
        'n', 30,
        'T', 31,
        't', 31,
        'U', 32,
        'u', 32,
      );
      END_STATE();
    case 3:
      ADVANCE_MAP(
        'A', 33,
        'a', 33,
        'N', 34,
        'n', 34,
        'O', 35,
        'o', 35,
        'U', 36,
        'u', 36,
        'X', 37,
        'x', 37,
      );
      END_STATE();
    case 4:
      ADVANCE_MAP(
        'A', 38,
        'a', 38,
        'H', 39,
        'h', 39,
        'L', 40,
        'l', 40,
        'O', 41,
        'o', 41,
        'R', 42,
        'r', 42,
        'U', 43,
        'u', 43,
      );
      END_STATE();
    case 5:
      ADVANCE_MAP(
        'A', 44,
        'a', 44,
        'E', 45,
        'e', 45,
        'N', 46,
        'n', 46,
        'O', 47,
        'o', 47,
        'U', 48,
        'u', 48,
        'V', 49,
        'v', 49,
        'Y', 50,
        'y', 50,
      );
      END_STATE();
    case 6:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(51);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(52);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(53);
      END_STATE();
    case 7:
      ADVANCE_MAP(
        'A', 54,
        'a', 54,
        'I', 55,
        'i', 55,
        'L', 56,
        'l', 56,
        'O', 57,
        'o', 57,
        'T', 58,
        't', 58,
      );
      END_STATE();
    case 8:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(59);
      END_STATE();
    case 9:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(60);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(61);
      END_STATE();
    case 10:
      ADVANCE_MAP(
        'F', 62,
        'f', 62,
        'N', 63,
        'n', 63,
        'P', 64,
        'p', 64,
        'T', 65,
        't', 65,
      );
      END_STATE();
    case 11:
      ADVANCE_MAP(
        'A', 66,
        'a', 66,
        'D', 67,
        'd', 67,
        'E', 68,
        'e', 68,
        'I', 69,
        'i', 69,
        'O', 70,
        'o', 70,
        'S', 71,
        's', 71,
      );
      END_STATE();
    case 12:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(72);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(73);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(74);
      END_STATE();
    case 13:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(75);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(76);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(77);
      END_STATE();
    case 14:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(78);
      END_STATE();
    case 15:
      ADVANCE_MAP(
        'E', 79,
        'e', 79,
        'O', 80,
        'o', 80,
        'R', 81,
        'r', 81,
        'U', 82,
        'u', 82,
      );
      END_STATE();
    case 16:
      ADVANCE_MAP(
        'A', 83,
        'a', 83,
        'E', 84,
        'e', 84,
        'I', 85,
        'i', 85,
        'S', 86,
        's', 86,
      );
      END_STATE();
    case 17:
      ADVANCE_MAP(
        'E', 87,
        'e', 87,
        'I', 88,
        'i', 88,
        'L', 89,
        'l', 89,
        'M', 90,
        'm', 90,
        'O', 91,
        'o', 91,
        'Q', 92,
        'q', 92,
        'S', 93,
        's', 93,
        'T', 94,
        't', 94,
        'W', 95,
        'w', 95,
        'Y', 96,
        'y', 96,
      );
      END_STATE();
    case 18:
      ADVANCE_MAP(
        'B', 97,
        'b', 97,
        'C', 98,
        'c', 98,
        'D', 99,
        'd', 99,
        'E', 100,
        'e', 100,
        'I', 101,
        'i', 101,
        'L', 102,
        'l', 102,
        'O', 103,
        'o', 103,
        'P', 104,
        'p', 104,
        'R', 105,
        'r', 105,
        'T', 106,
        't', 106,
        'Y', 107,
        'y', 107,
      );
      END_STATE();
    case 19:
      ADVANCE_MAP(
        'N', 108,
        'n', 108,
        'P', 109,
        'p', 109,
        'R', 110,
        'r', 110,
        'S', 111,
        's', 111,
      );
      END_STATE();
    case 20:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(112);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(113);
      END_STATE();
    case 21:
      ADVANCE_MAP(
        'A', 114,
        'a', 114,
        'C', 115,
        'c', 115,
        'H', 116,
        'h', 116,
        'I', 117,
        'i', 117,
      );
      END_STATE();
    case 22:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(118);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(119);
      END_STATE();
    case 23:
      ADVANCE_MAP(
        '_', 120,
        'D', 121,
        'd', 121,
        'F', 122,
        'f', 122,
        'L', 123,
        'l', 123,
        'N', 124,
        'n', 124,
        'T', 125,
        't', 125,
      );
      END_STATE();
    case 24:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(126);
      END_STATE();
    case 25:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(127);
      END_STATE();
    case 26:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(128);
      END_STATE();
    case 27:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(129);
      END_STATE();
    case 28:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(130);
      END_STATE();
    case 29:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(131);
      END_STATE();
    case 30:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(132);
      END_STATE();
    case 31:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(133);
      END_STATE();
    case 32:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(134);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(135);
      END_STATE();
    case 33:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(136);
      END_STATE();
    case 34:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(137);
      END_STATE();
    case 35:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(138);
      END_STATE();
    case 36:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(139);
      END_STATE();
    case 37:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(140);
      END_STATE();
    case 38:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(141);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(142);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(143);
      END_STATE();
    case 39:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(144);
      END_STATE();
    case 40:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(145);
      END_STATE();
    case 41:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(146);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(147);
      END_STATE();
    case 42:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(148);
      END_STATE();
    case 43:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(149);
      END_STATE();
    case 44:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(150);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(151);
      END_STATE();
    case 45:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(152);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(153);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(154);
      END_STATE();
    case 46:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(155);
      END_STATE();
    case 47:
      if (lookahead == '_') ADVANCE(156);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(157);
      END_STATE();
    case 48:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(158);
      END_STATE();
    case 49:
      if (lookahead == '_') ADVANCE(159);
      END_STATE();
    case 50:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(160);
      END_STATE();
    case 51:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(161);
      END_STATE();
    case 52:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(162);
      END_STATE();
    case 53:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(163);
      END_STATE();
    case 54:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(164);
      END_STATE();
    case 55:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(165);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(166);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(167);
      END_STATE();
    case 56:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(168);
      END_STATE();
    case 57:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(169);
      END_STATE();
    case 58:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(170);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(171);
      END_STATE();
    case 59:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(172);
      END_STATE();
    case 60:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(173);
      END_STATE();
    case 61:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(174);
      END_STATE();
    case 62:
      ACCEPT_TOKEN(sym_if_keyword);
      END_STATE();
    case 63:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(175);
      END_STATE();
    case 64:
      if (lookahead == '_') ADVANCE(176);
      END_STATE();
    case 65:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(177);
      END_STATE();
    case 66:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(178);
      END_STATE();
    case 67:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(179);
      END_STATE();
    case 68:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(180);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(181);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(182);
      END_STATE();
    case 69:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(183);
      END_STATE();
    case 70:
      ADVANCE_MAP(
        'A', 184,
        'a', 184,
        'C', 185,
        'c', 185,
        'G', 186,
        'g', 186,
        'N', 187,
        'n', 187,
        'W', 188,
        'w', 188,
      );
      END_STATE();
    case 71:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(189);
      END_STATE();
    case 72:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(190);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(191);
      END_STATE();
    case 73:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(192);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(193);
      END_STATE();
    case 74:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(194);
      END_STATE();
    case 75:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(195);
      END_STATE();
    case 76:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(196);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(197);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(198);
      END_STATE();
    case 77:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(199);
      END_STATE();
    case 78:
      ACCEPT_TOKEN(sym_or);
      END_STATE();
    case 79:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(200);
      END_STATE();
    case 80:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(201);
      END_STATE();
    case 81:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(202);
      END_STATE();
    case 82:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(203);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(204);
      END_STATE();
    case 83:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(205);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(206);
      END_STATE();
    case 84:
      ADVANCE_MAP(
        'B', 207,
        'b', 207,
        'D', 208,
        'd', 208,
        'L', 209,
        'l', 209,
        'M', 210,
        'm', 210,
        'P', 211,
        'p', 211,
        'T', 212,
        't', 212,
      );
      END_STATE();
    case 85:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(213);
      END_STATE();
    case 86:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(214);
      END_STATE();
    case 87:
      ADVANCE_MAP(
        'C', 215,
        'c', 215,
        'L', 216,
        'l', 216,
        'N', 217,
        'n', 217,
        'T', 218,
        't', 218,
      );
      END_STATE();
    case 88:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(219);
      END_STATE();
    case 89:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(220);
      END_STATE();
    case 90:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(221);
      END_STATE();
    case 91:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(222);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(223);
      END_STATE();
    case 92:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(224);
      END_STATE();
    case 93:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(225);
      END_STATE();
    case 94:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(226);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(227);
      END_STATE();
    case 95:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(228);
      END_STATE();
    case 96:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(229);
      END_STATE();
    case 97:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(230);
      END_STATE();
    case 98:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(231);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(232);
      END_STATE();
    case 99:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(233);
      END_STATE();
    case 100:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(234);
      END_STATE();
    case 101:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(235);
      END_STATE();
    case 102:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(236);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(237);
      END_STATE();
    case 103:
      ACCEPT_TOKEN(sym_devchan_to_keyword);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(238);
      END_STATE();
    case 104:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(239);
      END_STATE();
    case 105:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(240);
      END_STATE();
    case 106:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(241);
      END_STATE();
    case 107:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(242);
      END_STATE();
    case 108:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(243);
      END_STATE();
    case 109:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(244);
      END_STATE();
    case 110:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(245);
      END_STATE();
    case 111:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(246);
      END_STATE();
    case 112:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(247);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(248);
      END_STATE();
    case 113:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(249);
      END_STATE();
    case 114:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(250);
      END_STATE();
    case 115:
      if (lookahead == '_') ADVANCE(251);
      END_STATE();
    case 116:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(252);
      END_STATE();
    case 117:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(253);
      END_STATE();
    case 118:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(254);
      END_STATE();
    case 119:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(255);
      END_STATE();
    case 120:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(256);
      END_STATE();
    case 121:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(257);
      END_STATE();
    case 122:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(258);
      END_STATE();
    case 123:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(259);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(260);
      END_STATE();
    case 124:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(261);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(262);
      END_STATE();
    case 125:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(263);
      END_STATE();
    case 126:
      if (lookahead == '_') ADVANCE(264);
      END_STATE();
    case 127:
      if (lookahead == '_') ADVANCE(265);
      END_STATE();
    case 128:
      if (lookahead == '_') ADVANCE(266);
      END_STATE();
    case 129:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(267);
      END_STATE();
    case 130:
      if (lookahead == '_') ADVANCE(268);
      END_STATE();
    case 131:
      if (lookahead == '_') ADVANCE(269);
      END_STATE();
    case 132:
      ACCEPT_TOKEN(sym_and);
      END_STATE();
    case 133:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(270);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(271);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(272);
      END_STATE();
    case 134:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(273);
      END_STATE();
    case 135:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(274);
      END_STATE();
    case 136:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(275);
      END_STATE();
    case 137:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(276);
      END_STATE();
    case 138:
      ACCEPT_TOKEN(sym_bor);
      END_STATE();
    case 139:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(277);
      END_STATE();
    case 140:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(278);
      END_STATE();
    case 141:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(279);
      END_STATE();
    case 142:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(280);
      END_STATE();
    case 143:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(281);
      END_STATE();
    case 144:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(282);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(283);
      END_STATE();
    case 145:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(284);
      END_STATE();
    case 146:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(285);
      END_STATE();
    case 147:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(286);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(287);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(288);
      END_STATE();
    case 148:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(289);
      END_STATE();
    case 149:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(290);
      END_STATE();
    case 150:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(291);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(292);
      END_STATE();
    case 151:
      ACCEPT_TOKEN(sym_day);
      END_STATE();
    case 152:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(293);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(294);
      END_STATE();
    case 153:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(295);
      END_STATE();
    case 154:
      ACCEPT_TOKEN(sym_dev_keyword);
      if (lookahead == '_') ADVANCE(296);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(297);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(298);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(299);
      END_STATE();
    case 155:
      if (lookahead == '_') ADVANCE(300);
      END_STATE();
    case 156:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(301);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(302);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(303);
      END_STATE();
    case 157:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(304);
      END_STATE();
    case 158:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(305);
      END_STATE();
    case 159:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(306);
      END_STATE();
    case 160:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(307);
      END_STATE();
    case 161:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(308);
      END_STATE();
    case 162:
      if (lookahead == '_') ADVANCE(309);
      END_STATE();
    case 163:
      if (lookahead == '_') ADVANCE(310);
      END_STATE();
    case 164:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(311);
      END_STATE();
    case 165:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(312);
      END_STATE();
    case 166:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(313);
      END_STATE();
    case 167:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(314);
      END_STATE();
    case 168:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(315);
      END_STATE();
    case 169:
      ACCEPT_TOKEN(sym_for_keyword);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(316);
      END_STATE();
    case 170:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(317);
      END_STATE();
    case 171:
      if (lookahead == '_') ADVANCE(318);
      END_STATE();
    case 172:
      if (lookahead == '_') ADVANCE(319);
      END_STATE();
    case 173:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(320);
      END_STATE();
    case 174:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(321);
      END_STATE();
    case 175:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(322);
      END_STATE();
    case 176:
      ADVANCE_MAP(
        'A', 323,
        'a', 323,
        'B', 324,
        'b', 324,
        'C', 325,
        'c', 325,
        'M', 326,
        'm', 326,
        'N', 327,
        'n', 327,
        'S', 328,
        's', 328,
        'T', 329,
        't', 329,
        'U', 330,
        'u', 330,
      );
      END_STATE();
    case 177:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(331);
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(332);
      END_STATE();
    case 178:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(333);
      END_STATE();
    case 179:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(334);
      END_STATE();
    case 180:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(335);
      END_STATE();
    case 181:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(336);
      END_STATE();
    case 182:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(337);
      END_STATE();
    case 183:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(338);
      END_STATE();
    case 184:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(339);
      END_STATE();
    case 185:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(340);
      END_STATE();
    case 186:
      if (lookahead == '1') ADVANCE(341);
      if (lookahead == '_') ADVANCE(342);
      END_STATE();
    case 187:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(343);
      END_STATE();
    case 188:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(344);
      END_STATE();
    case 189:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(345);
      END_STATE();
    case 190:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(346);
      END_STATE();
    case 191:
      if (lookahead == '_') ADVANCE(347);
      END_STATE();
    case 192:
      if (lookahead == '_') ADVANCE(348);
      END_STATE();
    case 193:
      if (lookahead == '_') ADVANCE(349);
      END_STATE();
    case 194:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(350);
      END_STATE();
    case 195:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(351);
      END_STATE();
    case 196:
      if (lookahead == '_') ADVANCE(352);
      END_STATE();
    case 197:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(353);
      END_STATE();
    case 198:
      ACCEPT_TOKEN(sym_not);
      END_STATE();
    case 199:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(354);
      END_STATE();
    case 200:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(355);
      END_STATE();
    case 201:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(356);
      END_STATE();
    case 202:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(357);
      END_STATE();
    case 203:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(358);
      END_STATE();
    case 204:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(359);
      END_STATE();
    case 205:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(360);
      END_STATE();
    case 206:
      if (lookahead == '_') ADVANCE(361);
      END_STATE();
    case 207:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(362);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(363);
      END_STATE();
    case 208:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(364);
      END_STATE();
    case 209:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(365);
      END_STATE();
    case 210:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(366);
      END_STATE();
    case 211:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(367);
      END_STATE();
    case 212:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(368);
      END_STATE();
    case 213:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(369);
      END_STATE();
    case 214:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(370);
      END_STATE();
    case 215:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(371);
      END_STATE();
    case 216:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(372);
      END_STATE();
    case 217:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(373);
      END_STATE();
    case 218:
      if (lookahead == '_') ADVANCE(374);
      END_STATE();
    case 219:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(375);
      END_STATE();
    case 220:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(376);
      END_STATE();
    case 221:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(377);
      END_STATE();
    case 222:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(378);
      END_STATE();
    case 223:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(379);
      END_STATE();
    case 224:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(380);
      END_STATE();
    case 225:
      if (lookahead == '_') ADVANCE(381);
      END_STATE();
    case 226:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(382);
      END_STATE();
    case 227:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(383);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(384);
      END_STATE();
    case 228:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(385);
      END_STATE();
    case 229:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(386);
      END_STATE();
    case 230:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(387);
      END_STATE();
    case 231:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(388);
      END_STATE();
    case 232:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(389);
      END_STATE();
    case 233:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(390);
      END_STATE();
    case 234:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(391);
      END_STATE();
    case 235:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(392);
      END_STATE();
    case 236:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(393);
      END_STATE();
    case 237:
      if (lookahead == '_') ADVANCE(394);
      END_STATE();
    case 238:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(395);
      END_STATE();
    case 239:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(396);
      END_STATE();
    case 240:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(397);
      END_STATE();
    case 241:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(398);
      END_STATE();
    case 242:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(399);
      END_STATE();
    case 243:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(400);
      END_STATE();
    case 244:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(401);
      END_STATE();
    case 245:
      if (lookahead == '_') ADVANCE(402);
      END_STATE();
    case 246:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(403);
      END_STATE();
    case 247:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(404);
      END_STATE();
    case 248:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(405);
      END_STATE();
    case 249:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(406);
      END_STATE();
    case 250:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(407);
      END_STATE();
    case 251:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(408);
      END_STATE();
    case 252:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(409);
      END_STATE();
    case 253:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(410);
      END_STATE();
    case 254:
      if (lookahead == '_') ADVANCE(411);
      END_STATE();
    case 255:
      ACCEPT_TOKEN(sym_xor);
      END_STATE();
    case 256:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(412);
      END_STATE();
    case 257:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(413);
      END_STATE();
    case 258:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(414);
      END_STATE();
    case 259:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(415);
      END_STATE();
    case 260:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(416);
      END_STATE();
    case 261:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(417);
      END_STATE();
    case 262:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(418);
      END_STATE();
    case 263:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(419);
      END_STATE();
    case 264:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(420);
      END_STATE();
    case 265:
      ADVANCE_MAP(
        'C', 421,
        'c', 421,
        'D', 422,
        'd', 422,
        'E', 423,
        'e', 423,
        'F', 424,
        'f', 424,
        'G', 425,
        'g', 425,
        'I', 426,
        'i', 426,
        'L', 427,
        'l', 427,
        'M', 428,
        'm', 428,
        'R', 429,
        'r', 429,
        'T', 430,
        't', 430,
        'U', 431,
        'u', 431,
      );
      END_STATE();
    case 266:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(432);
      END_STATE();
    case 267:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(433);
      END_STATE();
    case 268:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(434);
      END_STATE();
    case 269:
      ADVANCE_MAP(
        'D', 435,
        'd', 435,
        'E', 436,
        'e', 436,
        'I', 437,
        'i', 437,
        'L', 438,
        'l', 438,
        'W', 439,
        'w', 439,
      );
      END_STATE();
    case 270:
      ACCEPT_TOKEN(sym_atof);
      END_STATE();
    case 271:
      ACCEPT_TOKEN(sym_atoi);
      END_STATE();
    case 272:
      ACCEPT_TOKEN(sym_atol);
      END_STATE();
    case 273:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(440);
      END_STATE();
    case 274:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(441);
      END_STATE();
    case 275:
      ACCEPT_TOKEN(sym_band);
      END_STATE();
    case 276:
      ACCEPT_TOKEN(sym_bnot);
      END_STATE();
    case 277:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(442);
      END_STATE();
    case 278:
      ACCEPT_TOKEN(sym_bxor);
      END_STATE();
    case 279:
      ACCEPT_TOKEN(sym_call_keyword);
      END_STATE();
    case 280:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(443);
      END_STATE();
    case 281:
      ACCEPT_TOKEN(sym_case_keyword);
      END_STATE();
    case 282:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(444);
      END_STATE();
    case 283:
      ACCEPT_TOKEN(sym_char_keyword);
      END_STATE();
    case 284:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(445);
      END_STATE();
    case 285:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(446);
      END_STATE();
    case 286:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(447);
      END_STATE();
    case 287:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(448);
      END_STATE();
    case 288:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(449);
      END_STATE();
    case 289:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(450);
      END_STATE();
    case 290:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(451);
      END_STATE();
    case 291:
      ACCEPT_TOKEN(sym_data);
      if (lookahead == '_') ADVANCE(452);
      END_STATE();
    case 292:
      ACCEPT_TOKEN(sym_date);
      if (lookahead == '_') ADVANCE(453);
      END_STATE();
    case 293:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(454);
      END_STATE();
    case 294:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(455);
      END_STATE();
    case 295:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(456);
      END_STATE();
    case 296:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(457);
      END_STATE();
    case 297:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(458);
      END_STATE();
    case 298:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(459);
      END_STATE();
    case 299:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(460);
      END_STATE();
    case 300:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(461);
      END_STATE();
    case 301:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(462);
      END_STATE();
    case 302:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(463);
      END_STATE();
    case 303:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(464);
      END_STATE();
    case 304:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(465);
      END_STATE();
    case 305:
      if (lookahead == '_') ADVANCE(466);
      END_STATE();
    case 306:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(467);
      END_STATE();
    case 307:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(468);
      END_STATE();
    case 308:
      ACCEPT_TOKEN(sym_else_keyword);
      END_STATE();
    case 309:
      ADVANCE_MAP(
        'A', 469,
        'a', 469,
        'G', 470,
        'g', 470,
        'I', 471,
        'i', 471,
        'N', 472,
        'n', 472,
        'P', 473,
        'p', 473,
        'U', 474,
        'u', 474,
      );
      END_STATE();
    case 310:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(475);
      END_STATE();
    case 311:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(476);
      END_STATE();
    case 312:
      if (lookahead == '_') ADVANCE(477);
      END_STATE();
    case 313:
      if (lookahead == '_') ADVANCE(478);
      END_STATE();
    case 314:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(479);
      END_STATE();
    case 315:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(480);
      END_STATE();
    case 316:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(481);
      END_STATE();
    case 317:
      ACCEPT_TOKEN(sym_ftoa);
      END_STATE();
    case 318:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(482);
      END_STATE();
    case 319:
      ADVANCE_MAP(
        'B', 483,
        'b', 483,
        'D', 484,
        'd', 484,
        'I', 485,
        'i', 485,
        'L', 486,
        'l', 486,
        'M', 487,
        'm', 487,
        'P', 488,
        'p', 488,
        'S', 489,
        's', 489,
        'T', 490,
        't', 490,
        'U', 491,
        'u', 491,
      );
      END_STATE();
    case 320:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(492);
      END_STATE();
    case 321:
      if (lookahead == '_') ADVANCE(493);
      END_STATE();
    case 322:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(494);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(495);
      END_STATE();
    case 323:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(496);
      END_STATE();
    case 324:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(497);
      END_STATE();
    case 325:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(498);
      END_STATE();
    case 326:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(499);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(500);
      END_STATE();
    case 327:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(501);
      END_STATE();
    case 328:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(502);
      END_STATE();
    case 329:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(503);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(504);
      END_STATE();
    case 330:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(505);
      END_STATE();
    case 331:
      ACCEPT_TOKEN(sym_itoa);
      END_STATE();
    case 332:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(506);
      END_STATE();
    case 333:
      if (lookahead == '_') ADVANCE(507);
      END_STATE();
    case 334:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(508);
      END_STATE();
    case 335:
      if (lookahead == '_') ADVANCE(509);
      END_STATE();
    case 336:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(510);
      END_STATE();
    case 337:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(511);
      END_STATE();
    case 338:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(512);
      END_STATE();
    case 339:
      if (lookahead == '_') ADVANCE(513);
      END_STATE();
    case 340:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(514);
      END_STATE();
    case 341:
      if (lookahead == '0') ADVANCE(515);
      END_STATE();
    case 342:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(516);
      END_STATE();
    case 343:
      ACCEPT_TOKEN(sym_long_keyword);
      END_STATE();
    case 344:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(517);
      END_STATE();
    case 345:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(518);
      END_STATE();
    case 346:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(519);
      END_STATE();
    case 347:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(520);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(521);
      END_STATE();
    case 348:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(522);
      END_STATE();
    case 349:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(523);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(524);
      END_STATE();
    case 350:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(525);
      END_STATE();
    case 351:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(526);
      END_STATE();
    case 352:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(527);
      END_STATE();
    case 353:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(528);
      END_STATE();
    case 354:
      if (lookahead == '_') ADVANCE(529);
      END_STATE();
    case 355:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(530);
      END_STATE();
    case 356:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(531);
      END_STATE();
    case 357:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(532);
      END_STATE();
    case 358:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(533);
      END_STATE();
    case 359:
      if (lookahead == '_') ADVANCE(534);
      END_STATE();
    case 360:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(535);
      END_STATE();
    case 361:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(536);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(537);
      END_STATE();
    case 362:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(538);
      END_STATE();
    case 363:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(539);
      END_STATE();
    case 364:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(540);
      END_STATE();
    case 365:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(541);
      END_STATE();
    case 366:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(542);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(543);
      END_STATE();
    case 367:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(544);
      END_STATE();
    case 368:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(545);
      END_STATE();
    case 369:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(546);
      END_STATE();
    case 370:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(547);
      END_STATE();
    case 371:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(548);
      END_STATE();
    case 372:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(549);
      END_STATE();
    case 373:
      if (lookahead == '_') ADVANCE(550);
      END_STATE();
    case 374:
      ADVANCE_MAP(
        'D', 551,
        'd', 551,
        'I', 552,
        'i', 552,
        'L', 553,
        'l', 553,
        'O', 554,
        'o', 554,
        'P', 555,
        'p', 555,
        'S', 556,
        's', 556,
        'T', 557,
        't', 557,
        'V', 558,
        'v', 558,
      );
      END_STATE();
    case 375:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(559);
      END_STATE();
    case 376:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(560);
      END_STATE();
    case 377:
      if (lookahead == '_') ADVANCE(561);
      END_STATE();
    case 378:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(562);
      END_STATE();
    case 379:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(563);
      END_STATE();
    case 380:
      if (lookahead == '_') ADVANCE(564);
      END_STATE();
    case 381:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(565);
      END_STATE();
    case 382:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(566);
      END_STATE();
    case 383:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(567);
      END_STATE();
    case 384:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(568);
      END_STATE();
    case 385:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(569);
      END_STATE();
    case 386:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(570);
      END_STATE();
    case 387:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(571);
      END_STATE();
    case 388:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(572);
      END_STATE();
    case 389:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(573);
      END_STATE();
    case 390:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(574);
      END_STATE();
    case 391:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(575);
      END_STATE();
    case 392:
      ACCEPT_TOKEN(sym_time);
      if (lookahead == '_') ADVANCE(576);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(577);
      END_STATE();
    case 393:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(578);
      END_STATE();
    case 394:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(579);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(580);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(581);
      END_STATE();
    case 395:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(582);
      END_STATE();
    case 396:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(583);
      END_STATE();
    case 397:
      ACCEPT_TOKEN(sym_true);
      END_STATE();
    case 398:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(584);
      END_STATE();
    case 399:
      if (lookahead == '_') ADVANCE(585);
      END_STATE();
    case 400:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(586);
      END_STATE();
    case 401:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(587);
      END_STATE();
    case 402:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(588);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(589);
      END_STATE();
    case 403:
      if (lookahead == '1') ADVANCE(590);
      if (lookahead == '2') ADVANCE(591);
      if (lookahead == '3') ADVANCE(592);
      if (lookahead == '4') ADVANCE(593);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(594);
      END_STATE();
    case 404:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(595);
      END_STATE();
    case 405:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(596);
      END_STATE();
    case 406:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(597);
      END_STATE();
    case 407:
      ACCEPT_TOKEN(sym_wait_keyword);
      if (lookahead == '_') ADVANCE(598);
      END_STATE();
    case 408:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(599);
      END_STATE();
    case 409:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(600);
      END_STATE();
    case 410:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(601);
      END_STATE();
    case 411:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(602);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(603);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(604);
      END_STATE();
    case 412:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(605);
      END_STATE();
    case 413:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(606);
      END_STATE();
    case 414:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(607);
      END_STATE();
    case 415:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(608);
      END_STATE();
    case 416:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(609);
      END_STATE();
    case 417:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(610);
      END_STATE();
    case 418:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(611);
      END_STATE();
    case 419:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(612);
      END_STATE();
    case 420:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(613);
      END_STATE();
    case 421:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(614);
      END_STATE();
    case 422:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(615);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(616);
      END_STATE();
    case 423:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(617);
      END_STATE();
    case 424:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(618);
      END_STATE();
    case 425:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(619);
      END_STATE();
    case 426:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(620);
      END_STATE();
    case 427:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(621);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(622);
      END_STATE();
    case 428:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(623);
      END_STATE();
    case 429:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(624);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(625);
      END_STATE();
    case 430:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(626);
      END_STATE();
    case 431:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(627);
      END_STATE();
    case 432:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(628);
      END_STATE();
    case 433:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(629);
      END_STATE();
    case 434:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(630);
      END_STATE();
    case 435:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(631);
      END_STATE();
    case 436:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(632);
      END_STATE();
    case 437:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(633);
      END_STATE();
    case 438:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(634);
      END_STATE();
    case 439:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(635);
      END_STATE();
    case 440:
      if (lookahead == '_') ADVANCE(636);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(637);
      END_STATE();
    case 441:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(638);
      END_STATE();
    case 442:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(639);
      END_STATE();
    case 443:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(640);
      END_STATE();
    case 444:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(641);
      END_STATE();
    case 445:
      if (lookahead == '_') ADVANCE(642);
      END_STATE();
    case 446:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(643);
      END_STATE();
    case 447:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(644);
      END_STATE();
    case 448:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(645);
      END_STATE();
    case 449:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(646);
      END_STATE();
    case 450:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(647);
      END_STATE();
    case 451:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(648);
      END_STATE();
    case 452:
      ADVANCE_MAP(
        'A', 649,
        'a', 649,
        'C', 650,
        'c', 650,
        'D', 651,
        'd', 651,
        'F', 652,
        'f', 652,
        'G', 653,
        'g', 653,
        'M', 654,
        'm', 654,
        'P', 655,
        'p', 655,
        'R', 656,
        'r', 656,
        'S', 657,
        's', 657,
        'T', 658,
        't', 658,
      );
      END_STATE();
    case 453:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(659);
      END_STATE();
    case 454:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(660);
      END_STATE();
    case 455:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(661);
      END_STATE();
    case 456:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(662);
      END_STATE();
    case 457:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(663);
      END_STATE();
    case 458:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(664);
      END_STATE();
    case 459:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(665);
      END_STATE();
    case 460:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(666);
      END_STATE();
    case 461:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(667);
      END_STATE();
    case 462:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(668);
      END_STATE();
    case 463:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(669);
      END_STATE();
    case 464:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(670);
      END_STATE();
    case 465:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(671);
      END_STATE();
    case 466:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(672);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(673);
      END_STATE();
    case 467:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(674);
      END_STATE();
    case 468:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(675);
      END_STATE();
    case 469:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(676);
      END_STATE();
    case 470:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(677);
      END_STATE();
    case 471:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(678);
      END_STATE();
    case 472:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(679);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(680);
      END_STATE();
    case 473:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(681);
      END_STATE();
    case 474:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(682);
      END_STATE();
    case 475:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(683);
      END_STATE();
    case 476:
      ACCEPT_TOKEN(sym_false);
      END_STATE();
    case 477:
      ADVANCE_MAP(
        'C', 684,
        'c', 684,
        'D', 685,
        'd', 685,
        'G', 686,
        'g', 686,
        'O', 687,
        'o', 687,
        'R', 688,
        'r', 688,
        'S', 689,
        's', 689,
        'W', 690,
        'w', 690,
      );
      END_STATE();
    case 478:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(691);
      END_STATE();
    case 479:
      if (lookahead == '_') ADVANCE(692);
      END_STATE();
    case 480:
      ACCEPT_TOKEN(sym_float_keyword);
      END_STATE();
    case 481:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(693);
      END_STATE();
    case 482:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(694);
      END_STATE();
    case 483:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(695);
      END_STATE();
    case 484:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(696);
      END_STATE();
    case 485:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(697);
      END_STATE();
    case 486:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(698);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(699);
      END_STATE();
    case 487:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(700);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(701);
      END_STATE();
    case 488:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(702);
      END_STATE();
    case 489:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(703);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(704);
      END_STATE();
    case 490:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(705);
      END_STATE();
    case 491:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(706);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(707);
      END_STATE();
    case 492:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(708);
      END_STATE();
    case 493:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(709);
      END_STATE();
    case 494:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(710);
      END_STATE();
    case 495:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(711);
      END_STATE();
    case 496:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(712);
      END_STATE();
    case 497:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(713);
      END_STATE();
    case 498:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(714);
      END_STATE();
    case 499:
      if (lookahead == '_') ADVANCE(715);
      END_STATE();
    case 500:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(716);
      END_STATE();
    case 501:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(717);
      END_STATE();
    case 502:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(718);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(719);
      END_STATE();
    case 503:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(720);
      END_STATE();
    case 504:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(721);
      END_STATE();
    case 505:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(722);
      END_STATE();
    case 506:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(723);
      END_STATE();
    case 507:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(724);
      END_STATE();
    case 508:
      ACCEPT_TOKEN(sym_ldate);
      END_STATE();
    case 509:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(725);
      END_STATE();
    case 510:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(726);
      END_STATE();
    case 511:
      ACCEPT_TOKEN(sym_level);
      END_STATE();
    case 512:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(727);
      END_STATE();
    case 513:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(728);
      END_STATE();
    case 514:
      if (lookahead == '_') ADVANCE(729);
      END_STATE();
    case 515:
      if (lookahead == '_') ADVANCE(730);
      END_STATE();
    case 516:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(731);
      END_STATE();
    case 517:
      if (lookahead == '_') ADVANCE(732);
      END_STATE();
    case 518:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(733);
      END_STATE();
    case 519:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(734);
      END_STATE();
    case 520:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(735);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(736);
      END_STATE();
    case 521:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(737);
      END_STATE();
    case 522:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(738);
      END_STATE();
    case 523:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(739);
      END_STATE();
    case 524:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(740);
      END_STATE();
    case 525:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(741);
      END_STATE();
    case 526:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(742);
      END_STATE();
    case 527:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(743);
      END_STATE();
    case 528:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(744);
      END_STATE();
    case 529:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(745);
      END_STATE();
    case 530:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(746);
      END_STATE();
    case 531:
      if (lookahead == '_') ADVANCE(747);
      END_STATE();
    case 532:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(748);
      END_STATE();
    case 533:
      ACCEPT_TOKEN(sym_devchan_pulse_keyword);
      END_STATE();
    case 534:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(749);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(750);
      END_STATE();
    case 535:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(751);
      END_STATE();
    case 536:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(752);
      END_STATE();
    case 537:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(753);
      END_STATE();
    case 538:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(754);
      END_STATE();
    case 539:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(755);
      END_STATE();
    case 540:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(756);
      END_STATE();
    case 541:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(757);
      END_STATE();
    case 542:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(758);
      END_STATE();
    case 543:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(759);
      END_STATE();
    case 544:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(760);
      END_STATE();
    case 545:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(761);
      END_STATE();
    case 546:
      if (lookahead == '_') ADVANCE(762);
      END_STATE();
    case 547:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(763);
      END_STATE();
    case 548:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(764);
      END_STATE();
    case 549:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(765);
      END_STATE();
    case 550:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(766);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(767);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(768);
      END_STATE();
    case 551:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(769);
      END_STATE();
    case 552:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(770);
      END_STATE();
    case 553:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(771);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(772);
      END_STATE();
    case 554:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(773);
      END_STATE();
    case 555:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(774);
      END_STATE();
    case 556:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(775);
      END_STATE();
    case 557:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(776);
      END_STATE();
    case 558:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(777);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(778);
      END_STATE();
    case 559:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(779);
      END_STATE();
    case 560:
      ACCEPT_TOKEN(sym_slong_keyword);
      END_STATE();
    case 561:
      ADVANCE_MAP(
        'A', 780,
        'a', 780,
        'F', 781,
        'f', 781,
        'P', 782,
        'p', 782,
        'R', 783,
        'r', 783,
        'S', 784,
        's', 784,
        'T', 785,
        't', 785,
        'U', 786,
        'u', 786,
      );
      END_STATE();
    case 562:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(787);
      END_STATE();
    case 563:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(788);
      END_STATE();
    case 564:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(789);
      END_STATE();
    case 565:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(790);
      END_STATE();
    case 566:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(791);
      END_STATE();
    case 567:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(792);
      END_STATE();
    case 568:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(793);
      END_STATE();
    case 569:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(794);
      END_STATE();
    case 570:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(795);
      END_STATE();
    case 571:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(796);
      END_STATE();
    case 572:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(797);
      END_STATE();
    case 573:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(798);
      END_STATE();
    case 574:
      ACCEPT_TOKEN(sym_tdata);
      END_STATE();
    case 575:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(799);
      END_STATE();
    case 576:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(800);
      END_STATE();
    case 577:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(801);
      END_STATE();
    case 578:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(802);
      END_STATE();
    case 579:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(803);
      END_STATE();
    case 580:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(804);
      END_STATE();
    case 581:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(805);
      END_STATE();
    case 582:
      if (lookahead == '_') ADVANCE(806);
      END_STATE();
    case 583:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(807);
      END_STATE();
    case 584:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(808);
      END_STATE();
    case 585:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(809);
      END_STATE();
    case 586:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(810);
      END_STATE();
    case 587:
      if (lookahead == '_') ADVANCE(811);
      END_STATE();
    case 588:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(812);
      END_STATE();
    case 589:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(813);
      END_STATE();
    case 590:
      if (lookahead == '_') ADVANCE(814);
      END_STATE();
    case 591:
      if (lookahead == '_') ADVANCE(815);
      END_STATE();
    case 592:
      if (lookahead == '_') ADVANCE(816);
      END_STATE();
    case 593:
      if (lookahead == '_') ADVANCE(817);
      END_STATE();
    case 594:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(818);
      END_STATE();
    case 595:
      if (lookahead == '_') ADVANCE(819);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(820);
      END_STATE();
    case 596:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(821);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(822);
      END_STATE();
    case 597:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(823);
      END_STATE();
    case 598:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(824);
      END_STATE();
    case 599:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(825);
      END_STATE();
    case 600:
      ACCEPT_TOKEN(sym_while_keyword);
      END_STATE();
    case 601:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(826);
      END_STATE();
    case 602:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(827);
      END_STATE();
    case 603:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(828);
      END_STATE();
    case 604:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(829);
      END_STATE();
    case 605:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(830);
      END_STATE();
    case 606:
      if (lookahead == '_') ADVANCE(831);
      END_STATE();
    case 607:
      if (lookahead == '_') ADVANCE(832);
      END_STATE();
    case 608:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(833);
      END_STATE();
    case 609:
      if (lookahead == '_') ADVANCE(834);
      END_STATE();
    case 610:
      if (lookahead == '_') ADVANCE(835);
      END_STATE();
    case 611:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(836);
      END_STATE();
    case 612:
      if (lookahead == '_') ADVANCE(837);
      END_STATE();
    case 613:
      if (lookahead == '_') ADVANCE(838);
      END_STATE();
    case 614:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(839);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(840);
      END_STATE();
    case 615:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(841);
      END_STATE();
    case 616:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(842);
      END_STATE();
    case 617:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(843);
      END_STATE();
    case 618:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(844);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(845);
      END_STATE();
    case 619:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(846);
      END_STATE();
    case 620:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(847);
      END_STATE();
    case 621:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(848);
      END_STATE();
    case 622:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(849);
      END_STATE();
    case 623:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(850);
      END_STATE();
    case 624:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(851);
      END_STATE();
    case 625:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(852);
      END_STATE();
    case 626:
      if (lookahead == '_') ADVANCE(853);
      END_STATE();
    case 627:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(854);
      END_STATE();
    case 628:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(855);
      END_STATE();
    case 629:
      ACCEPT_TOKEN(sym_active_keyword);
      END_STATE();
    case 630:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(856);
      END_STATE();
    case 631:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(857);
      END_STATE();
    case 632:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(858);
      END_STATE();
    case 633:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(859);
      END_STATE();
    case 634:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(860);
      END_STATE();
    case 635:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(861);
      END_STATE();
    case 636:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(862);
      END_STATE();
    case 637:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(863);
      END_STATE();
    case 638:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(864);
      END_STATE();
    case 639:
      ACCEPT_TOKEN(sym_button);
      END_STATE();
    case 640:
      if (lookahead == '_') ADVANCE(865);
      END_STATE();
    case 641:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(866);
      END_STATE();
    case 642:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(867);
      END_STATE();
    case 643:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(868);
      END_STATE();
    case 644:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(869);
      END_STATE();
    case 645:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(870);
      END_STATE();
    case 646:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(871);
      END_STATE();
    case 647:
      if (lookahead == '_') ADVANCE(872);
      END_STATE();
    case 648:
      ACCEPT_TOKEN(sym_custom);
      END_STATE();
    case 649:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(873);
      END_STATE();
    case 650:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(874);
      END_STATE();
    case 651:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(875);
      END_STATE();
    case 652:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(876);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(877);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(878);
      END_STATE();
    case 653:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(879);
      END_STATE();
    case 654:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(880);
      END_STATE();
    case 655:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(881);
      END_STATE();
    case 656:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(882);
      END_STATE();
    case 657:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(883);
      END_STATE();
    case 658:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(884);
      END_STATE();
    case 659:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(885);
      END_STATE();
    case 660:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(886);
      END_STATE();
    case 661:
      if (lookahead == '_') ADVANCE(887);
      END_STATE();
    case 662:
      if (lookahead == '_') ADVANCE(888);
      END_STATE();
    case 663:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(889);
      END_STATE();
    case 664:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(890);
      END_STATE();
    case 665:
      if (lookahead == '_') ADVANCE(891);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(892);
      END_STATE();
    case 666:
      ACCEPT_TOKEN(sym_devlev_keyword);
      END_STATE();
    case 667:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(893);
      END_STATE();
    case 668:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(894);
      END_STATE();
    case 669:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(895);
      END_STATE();
    case 670:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(896);
      END_STATE();
    case 671:
      ACCEPT_TOKEN(sym_double_keyword);
      END_STATE();
    case 672:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(897);
      END_STATE();
    case 673:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(898);
      END_STATE();
    case 674:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(899);
      END_STATE();
    case 675:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(900);
      END_STATE();
    case 676:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(901);
      END_STATE();
    case 677:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(902);
      END_STATE();
    case 678:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(903);
      END_STATE();
    case 679:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(904);
      END_STATE();
    case 680:
      if (lookahead == '_') ADVANCE(905);
      END_STATE();
    case 681:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(906);
      END_STATE();
    case 682:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(907);
      END_STATE();
    case 683:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(908);
      END_STATE();
    case 684:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(909);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(910);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(911);
      END_STATE();
    case 685:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(912);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(913);
      END_STATE();
    case 686:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(914);
      END_STATE();
    case 687:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(915);
      END_STATE();
    case 688:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(916);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(917);
      END_STATE();
    case 689:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(918);
      END_STATE();
    case 690:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(919);
      END_STATE();
    case 691:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(920);
      END_STATE();
    case 692:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(921);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(922);
      END_STATE();
    case 693:
      ACCEPT_TOKEN(sym_format);
      END_STATE();
    case 694:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(923);
      END_STATE();
    case 695:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(924);
      END_STATE();
    case 696:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(925);
      END_STATE();
    case 697:
      if (lookahead == '_') ADVANCE(926);
      END_STATE();
    case 698:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(927);
      END_STATE();
    case 699:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(928);
      END_STATE();
    case 700:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(929);
      END_STATE();
    case 701:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(930);
      END_STATE();
    case 702:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(931);
      END_STATE();
    case 703:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(932);
      END_STATE();
    case 704:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(933);
      END_STATE();
    case 705:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(934);
      END_STATE();
    case 706:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(935);
      END_STATE();
    case 707:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(936);
      END_STATE();
    case 708:
      ACCEPT_TOKEN(sym_hextoi);
      END_STATE();
    case 709:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(937);
      END_STATE();
    case 710:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(938);
      END_STATE();
    case 711:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(939);
      END_STATE();
    case 712:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(940);
      END_STATE();
    case 713:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(941);
      END_STATE();
    case 714:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(942);
      END_STATE();
    case 715:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(943);
      END_STATE();
    case 716:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(944);
      END_STATE();
    case 717:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(945);
      END_STATE();
    case 718:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(946);
      END_STATE();
    case 719:
      if (lookahead == '_') ADVANCE(947);
      END_STATE();
    case 720:
      ACCEPT_TOKEN(sym_ip_tcp);
      if (lookahead == '_') ADVANCE(948);
      END_STATE();
    case 721:
      if (lookahead == '_') ADVANCE(949);
      END_STATE();
    case 722:
      ACCEPT_TOKEN(sym_ip_udp);
      if (lookahead == '_') ADVANCE(950);
      END_STATE();
    case 723:
      ACCEPT_TOKEN(sym_itohex);
      END_STATE();
    case 724:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(951);
      END_STATE();
    case 725:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(952);
      END_STATE();
    case 726:
      if (lookahead == '_') ADVANCE(953);
      END_STATE();
    case 727:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(954);
      END_STATE();
    case 728:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(955);
      END_STATE();
    case 729:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(956);
      END_STATE();
    case 730:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(957);
      END_STATE();
    case 731:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(958);
      END_STATE();
    case 732:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(959);
      END_STATE();
    case 733:
      ACCEPT_TOKEN(sym_lshift);
      END_STATE();
    case 734:
      if (lookahead == '_') ADVANCE(960);
      END_STATE();
    case 735:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(961);
      END_STATE();
    case 736:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(962);
      END_STATE();
    case 737:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(963);
      END_STATE();
    case 738:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(964);
      END_STATE();
    case 739:
      ACCEPT_TOKEN(sym_devchan_min_to_keyword);
      END_STATE();
    case 740:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(965);
      END_STATE();
    case 741:
      if (lookahead == '_') ADVANCE(966);
      END_STATE();
    case 742:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(967);
      END_STATE();
    case 743:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(968);
      END_STATE();
    case 744:
      if (lookahead == '_') ADVANCE(969);
      END_STATE();
    case 745:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(970);
      END_STATE();
    case 746:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(971);
      END_STATE();
    case 747:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(972);
      END_STATE();
    case 748:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(973);
      END_STATE();
    case 749:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(974);
      END_STATE();
    case 750:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(975);
      END_STATE();
    case 751:
      if (lookahead == '_') ADVANCE(976);
      END_STATE();
    case 752:
      ACCEPT_TOKEN(sym_raw_be);
      END_STATE();
    case 753:
      ACCEPT_TOKEN(sym_raw_le);
      END_STATE();
    case 754:
      ACCEPT_TOKEN(sym_reboot);
      END_STATE();
    case 755:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(977);
      END_STATE();
    case 756:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(978);
      END_STATE();
    case 757:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(979);
      END_STATE();
    case 758:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(980);
      END_STATE();
    case 759:
      if (lookahead == '_') ADVANCE(981);
      END_STATE();
    case 760:
      ACCEPT_TOKEN(sym_repeat_keyword);
      END_STATE();
    case 761:
      ACCEPT_TOKEN(sym_return_keyword);
      END_STATE();
    case 762:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(982);
      END_STATE();
    case 763:
      ACCEPT_TOKEN(sym_rshift);
      END_STATE();
    case 764:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(983);
      END_STATE();
    case 765:
      ACCEPT_TOKEN(sym_select_keyword);
      END_STATE();
    case 766:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(984);
      END_STATE();
    case 767:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(985);
      END_STATE();
    case 768:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(986);
      END_STATE();
    case 769:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(987);
      END_STATE();
    case 770:
      if (lookahead == '_') ADVANCE(988);
      END_STATE();
    case 771:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(989);
      END_STATE();
    case 772:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(990);
      END_STATE();
    case 773:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(991);
      END_STATE();
    case 774:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(992);
      END_STATE();
    case 775:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(993);
      END_STATE();
    case 776:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(994);
      END_STATE();
    case 777:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(995);
      END_STATE();
    case 778:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(996);
      END_STATE();
    case 779:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(997);
      END_STATE();
    case 780:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(998);
      END_STATE();
    case 781:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(999);
      END_STATE();
    case 782:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1000);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1001);
      END_STATE();
    case 783:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1002);
      END_STATE();
    case 784:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1003);
      END_STATE();
    case 785:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1004);
      END_STATE();
    case 786:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1005);
      END_STATE();
    case 787:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1006);
      END_STATE();
    case 788:
      if (lookahead == '_') ADVANCE(1007);
      END_STATE();
    case 789:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1008);
      END_STATE();
    case 790:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1009);
      END_STATE();
    case 791:
      if (lookahead == '_') ADVANCE(1010);
      END_STATE();
    case 792:
      if (lookahead == '_') ADVANCE(1011);
      END_STATE();
    case 793:
      ACCEPT_TOKEN(sym_struct_keyword);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1012);
      END_STATE();
    case 794:
      ACCEPT_TOKEN(sym_switch_keyword);
      END_STATE();
    case 795:
      if (lookahead == '_') ADVANCE(1013);
      END_STATE();
    case 796:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1014);
      END_STATE();
    case 797:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1015);
      END_STATE();
    case 798:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1016);
      END_STATE();
    case 799:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1017);
      END_STATE();
    case 800:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1018);
      END_STATE();
    case 801:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1019);
      END_STATE();
    case 802:
      ACCEPT_TOKEN(sym_tlevel);
      END_STATE();
    case 803:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1020);
      END_STATE();
    case 804:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1021);
      END_STATE();
    case 805:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1022);
      END_STATE();
    case 806:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1023);
      END_STATE();
    case 807:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1024);
      END_STATE();
    case 808:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1025);
      END_STATE();
    case 809:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1026);
      END_STATE();
    case 810:
      if (lookahead == '_') ADVANCE(1027);
      END_STATE();
    case 811:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1028);
      END_STATE();
    case 812:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1029);
      END_STATE();
    case 813:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1030);
      END_STATE();
    case 814:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1031);
      END_STATE();
    case 815:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1032);
      END_STATE();
    case 816:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1033);
      END_STATE();
    case 817:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1034);
      END_STATE();
    case 818:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1035);
      END_STATE();
    case 819:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1036);
      END_STATE();
    case 820:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1037);
      END_STATE();
    case 821:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1038);
      END_STATE();
    case 822:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1039);
      END_STATE();
    case 823:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1040);
      END_STATE();
    case 824:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1041);
      END_STATE();
    case 825:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1042);
      END_STATE();
    case 826:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1043);
      END_STATE();
    case 827:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1044);
      END_STATE();
    case 828:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1045);
      END_STATE();
    case 829:
      if (lookahead == '_') ADVANCE(1046);
      END_STATE();
    case 830:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1047);
      END_STATE();
    case 831:
      if (lookahead == '_') ADVANCE(1048);
      END_STATE();
    case 832:
      if (lookahead == '_') ADVANCE(1049);
      END_STATE();
    case 833:
      if (lookahead == '_') ADVANCE(1050);
      END_STATE();
    case 834:
      if (lookahead == '_') ADVANCE(1051);
      END_STATE();
    case 835:
      if (lookahead == '_') ADVANCE(1052);
      END_STATE();
    case 836:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1053);
      END_STATE();
    case 837:
      if (lookahead == '_') ADVANCE(1054);
      END_STATE();
    case 838:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1055);
      END_STATE();
    case 839:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1056);
      END_STATE();
    case 840:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1057);
      END_STATE();
    case 841:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1058);
      END_STATE();
    case 842:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1059);
      END_STATE();
    case 843:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1060);
      END_STATE();
    case 844:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1061);
      END_STATE();
    case 845:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1062);
      END_STATE();
    case 846:
      if (lookahead == '_') ADVANCE(1063);
      END_STATE();
    case 847:
      ACCEPT_TOKEN(sym__wc_int);
      END_STATE();
    case 848:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1064);
      END_STATE();
    case 849:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1065);
      END_STATE();
    case 850:
      if (lookahead == '_') ADVANCE(1066);
      END_STATE();
    case 851:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1067);
      END_STATE();
    case 852:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1068);
      END_STATE();
    case 853:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1069);
      END_STATE();
    case 854:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1070);
      END_STATE();
    case 855:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1071);
      END_STATE();
    case 856:
      if (lookahead == '_') ADVANCE(1072);
      END_STATE();
    case 857:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1073);
      END_STATE();
    case 858:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1074);
      END_STATE();
    case 859:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1075);
      END_STATE();
    case 860:
      ACCEPT_TOKEN(sym_amx_log);
      END_STATE();
    case 861:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1076);
      END_STATE();
    case 862:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1077);
      END_STATE();
    case 863:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1078);
      END_STATE();
    case 864:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1079);
      END_STATE();
    case 865:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1080);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1081);
      END_STATE();
    case 866:
      ACCEPT_TOKEN(sym_channel);
      END_STATE();
    case 867:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1082);
      END_STATE();
    case 868:
      if (lookahead == '_') ADVANCE(1083);
      END_STATE();
    case 869:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1084);
      END_STATE();
    case 870:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1085);
      END_STATE();
    case 871:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1086);
      END_STATE();
    case 872:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1087);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1088);
      END_STATE();
    case 873:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1089);
      END_STATE();
    case 874:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1090);
      END_STATE();
    case 875:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1091);
      END_STATE();
    case 876:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1092);
      END_STATE();
    case 877:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1093);
      END_STATE();
    case 878:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1094);
      END_STATE();
    case 879:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1095);
      END_STATE();
    case 880:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1096);
      END_STATE();
    case 881:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1097);
      END_STATE();
    case 882:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1098);
      END_STATE();
    case 883:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1099);
      END_STATE();
    case 884:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1100);
      END_STATE();
    case 885:
      if (lookahead == '_') ADVANCE(1101);
      END_STATE();
    case 886:
      ACCEPT_TOKEN(sym_default_keyword);
      END_STATE();
    case 887:
      ADVANCE_MAP(
        'C', 1102,
        'c', 1102,
        'D', 1103,
        'd', 1103,
        'E', 1104,
        'e', 1104,
        'F', 1105,
        'f', 1105,
        'L', 1106,
        'l', 1106,
        'M', 1107,
        'm', 1107,
        'P', 1108,
        'p', 1108,
        'S', 1109,
        's', 1109,
        'T', 1110,
        't', 1110,
        'V', 1111,
        'v', 1111,
      );
      END_STATE();
    case 888:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1112);
      END_STATE();
    case 889:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1113);
      END_STATE();
    case 890:
      ACCEPT_TOKEN(sym_devchan_keyword);
      END_STATE();
    case 891:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1114);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1115);
      END_STATE();
    case 892:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1116);
      END_STATE();
    case 893:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1117);
      END_STATE();
    case 894:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1118);
      END_STATE();
    case 895:
      ACCEPT_TOKEN(sym_do_push);
      if (lookahead == '_') ADVANCE(1119);
      END_STATE();
    case 896:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1120);
      END_STATE();
    case 897:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1121);
      END_STATE();
    case 898:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1122);
      END_STATE();
    case 899:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1123);
      END_STATE();
    case 900:
      if (lookahead == '_') ADVANCE(1124);
      END_STATE();
    case 901:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1125);
      END_STATE();
    case 902:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1126);
      END_STATE();
    case 903:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1127);
      END_STATE();
    case 904:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1128);
      END_STATE();
    case 905:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1129);
      END_STATE();
    case 906:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1130);
      END_STATE();
    case 907:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1131);
      END_STATE();
    case 908:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1132);
      END_STATE();
    case 909:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1133);
      END_STATE();
    case 910:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1134);
      END_STATE();
    case 911:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1135);
      END_STATE();
    case 912:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1136);
      END_STATE();
    case 913:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1137);
      END_STATE();
    case 914:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1138);
      END_STATE();
    case 915:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1139);
      END_STATE();
    case 916:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1140);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1141);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1142);
      END_STATE();
    case 917:
      if (lookahead == '_') ADVANCE(1143);
      END_STATE();
    case 918:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1144);
      END_STATE();
    case 919:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1145);
      END_STATE();
    case 920:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1146);
      END_STATE();
    case 921:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1147);
      END_STATE();
    case 922:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1148);
      END_STATE();
    case 923:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1149);
      END_STATE();
    case 924:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1150);
      END_STATE();
    case 925:
      if (lookahead == '_') ADVANCE(1151);
      END_STATE();
    case 926:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1152);
      END_STATE();
    case 927:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1153);
      END_STATE();
    case 928:
      if (lookahead == '_') ADVANCE(1154);
      END_STATE();
    case 929:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1155);
      END_STATE();
    case 930:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1156);
      END_STATE();
    case 931:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1157);
      END_STATE();
    case 932:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1158);
      END_STATE();
    case 933:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1159);
      END_STATE();
    case 934:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1160);
      END_STATE();
    case 935:
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1161);
      END_STATE();
    case 936:
      if (lookahead == '_') ADVANCE(1162);
      END_STATE();
    case 937:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1163);
      END_STATE();
    case 938:
      ACCEPT_TOKEN(sym_integer_keyword);
      END_STATE();
    case 939:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1164);
      END_STATE();
    case 940:
      if (lookahead == '_') ADVANCE(1165);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1166);
      END_STATE();
    case 941:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1167);
      END_STATE();
    case 942:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1168);
      END_STATE();
    case 943:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1169);
      END_STATE();
    case 944:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1170);
      END_STATE();
    case 945:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1171);
      END_STATE();
    case 946:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1172);
      END_STATE();
    case 947:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1173);
      END_STATE();
    case 948:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1174);
      END_STATE();
    case 949:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1175);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1176);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1177);
      END_STATE();
    case 950:
      if (lookahead == '2') ADVANCE(1178);
      END_STATE();
    case 951:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1179);
      END_STATE();
    case 952:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1180);
      END_STATE();
    case 953:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1181);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1182);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1183);
      END_STATE();
    case 954:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1184);
      END_STATE();
    case 955:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1185);
      END_STATE();
    case 956:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1186);
      END_STATE();
    case 957:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1187);
      END_STATE();
    case 958:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1188);
      END_STATE();
    case 959:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1189);
      END_STATE();
    case 960:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1190);
      END_STATE();
    case 961:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1191);
      END_STATE();
    case 962:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1192);
      END_STATE();
    case 963:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1193);
      END_STATE();
    case 964:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1194);
      END_STATE();
    case 965:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1195);
      END_STATE();
    case 966:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1196);
      END_STATE();
    case 967:
      if (lookahead == '_') ADVANCE(1197);
      END_STATE();
    case 968:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1198);
      END_STATE();
    case 969:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1199);
      END_STATE();
    case 970:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1200);
      END_STATE();
    case 971:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1201);
      END_STATE();
    case 972:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1202);
      END_STATE();
    case 973:
      if (lookahead == '_') ADVANCE(1203);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1204);
      END_STATE();
    case 974:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1205);
      END_STATE();
    case 975:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1206);
      END_STATE();
    case 976:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1207);
      END_STATE();
    case 977:
      if (lookahead == '_') ADVANCE(1208);
      END_STATE();
    case 978:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1209);
      END_STATE();
    case 979:
      if (lookahead == '_') ADVANCE(1210);
      END_STATE();
    case 980:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1211);
      END_STATE();
    case 981:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1212);
      END_STATE();
    case 982:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1213);
      END_STATE();
    case 983:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1214);
      END_STATE();
    case 984:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1215);
      END_STATE();
    case 985:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1216);
      END_STATE();
    case 986:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1217);
      END_STATE();
    case 987:
      if (lookahead == '_') ADVANCE(1218);
      END_STATE();
    case 988:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1219);
      END_STATE();
    case 989:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1220);
      END_STATE();
    case 990:
      if (lookahead == '_') ADVANCE(1221);
      END_STATE();
    case 991:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1222);
      END_STATE();
    case 992:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1223);
      END_STATE();
    case 993:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1224);
      END_STATE();
    case 994:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1225);
      END_STATE();
    case 995:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1226);
      END_STATE();
    case 996:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1227);
      END_STATE();
    case 997:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1228);
      END_STATE();
    case 998:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1229);
      END_STATE();
    case 999:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1230);
      END_STATE();
    case 1000:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1231);
      END_STATE();
    case 1001:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1232);
      END_STATE();
    case 1002:
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1233);
      END_STATE();
    case 1003:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1234);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1235);
      END_STATE();
    case 1004:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1236);
      END_STATE();
    case 1005:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1237);
      END_STATE();
    case 1006:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1238);
      END_STATE();
    case 1007:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1239);
      END_STATE();
    case 1008:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1240);
      END_STATE();
    case 1009:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1241);
      END_STATE();
    case 1010:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1242);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1243);
      END_STATE();
    case 1011:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1244);
      END_STATE();
    case 1012:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1245);
      END_STATE();
    case 1013:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1246);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1247);
      END_STATE();
    case 1014:
      ACCEPT_TOKEN(sym_tbutton);
      END_STATE();
    case 1015:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1248);
      END_STATE();
    case 1016:
      ACCEPT_TOKEN(sym_tcustom);
      END_STATE();
    case 1017:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1249);
      END_STATE();
    case 1018:
      if (lookahead == '_') ADVANCE(1250);
      END_STATE();
    case 1019:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1251);
      END_STATE();
    case 1020:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1252);
      END_STATE();
    case 1021:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1253);
      END_STATE();
    case 1022:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1254);
      END_STATE();
    case 1023:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1255);
      END_STATE();
    case 1024:
      if (lookahead == '_') ADVANCE(1256);
      END_STATE();
    case 1025:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1257);
      END_STATE();
    case 1026:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1258);
      END_STATE();
    case 1027:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1259);
      END_STATE();
    case 1028:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1260);
      END_STATE();
    case 1029:
      if (lookahead == '_') ADVANCE(1261);
      END_STATE();
    case 1030:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1262);
      END_STATE();
    case 1031:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1263);
      END_STATE();
    case 1032:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1264);
      END_STATE();
    case 1033:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1265);
      END_STATE();
    case 1034:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1266);
      END_STATE();
    case 1035:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1267);
      END_STATE();
    case 1036:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1268);
      END_STATE();
    case 1037:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1269);
      END_STATE();
    case 1038:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1270);
      END_STATE();
    case 1039:
      ACCEPT_TOKEN(sym_variant_keyword);
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1271);
      END_STATE();
    case 1040:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1272);
      END_STATE();
    case 1041:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1273);
      END_STATE();
    case 1042:
      if (lookahead == '_') ADVANCE(1274);
      END_STATE();
    case 1043:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1275);
      END_STATE();
    case 1044:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1276);
      END_STATE();
    case 1045:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1277);
      END_STATE();
    case 1046:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1278);
      END_STATE();
    case 1047:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1279);
      END_STATE();
    case 1048:
      ACCEPT_TOKEN(sym___date__);
      END_STATE();
    case 1049:
      ACCEPT_TOKEN(sym___file__);
      END_STATE();
    case 1050:
      if (lookahead == '_') ADVANCE(1280);
      END_STATE();
    case 1051:
      ACCEPT_TOKEN(sym___line__);
      END_STATE();
    case 1052:
      ACCEPT_TOKEN(sym___name__);
      END_STATE();
    case 1053:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1281);
      END_STATE();
    case 1054:
      ACCEPT_TOKEN(sym___time__);
      END_STATE();
    case 1055:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1282);
      END_STATE();
    case 1056:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1283);
      END_STATE();
    case 1057:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1284);
      END_STATE();
    case 1058:
      if (lookahead == '_') ADVANCE(1285);
      END_STATE();
    case 1059:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1286);
      END_STATE();
    case 1060:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1287);
      END_STATE();
    case 1061:
      if (lookahead == '_') ADVANCE(1288);
      END_STATE();
    case 1062:
      if (lookahead == '_') ADVANCE(1289);
      END_STATE();
    case 1063:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1290);
      END_STATE();
    case 1064:
      if (lookahead == '_') ADVANCE(1291);
      END_STATE();
    case 1065:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1292);
      END_STATE();
    case 1066:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1293);
      END_STATE();
    case 1067:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1294);
      END_STATE();
    case 1068:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1295);
      END_STATE();
    case 1069:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1296);
      END_STATE();
    case 1070:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1297);
      END_STATE();
    case 1071:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1298);
      END_STATE();
    case 1072:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1299);
      END_STATE();
    case 1073:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1300);
      END_STATE();
    case 1074:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1301);
      END_STATE();
    case 1075:
      ACCEPT_TOKEN(sym_amx_info);
      END_STATE();
    case 1076:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1302);
      END_STATE();
    case 1077:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1303);
      END_STATE();
    case 1078:
      if (lookahead == '_') ADVANCE(1304);
      END_STATE();
    case 1079:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1305);
      END_STATE();
    case 1080:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1306);
      END_STATE();
    case 1081:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1307);
      END_STATE();
    case 1082:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1308);
      END_STATE();
    case 1083:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1309);
      END_STATE();
    case 1084:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1310);
      END_STATE();
    case 1085:
      ACCEPT_TOKEN(sym_constant_keyword);
      END_STATE();
    case 1086:
      ACCEPT_TOKEN(sym_continue_keyword);
      END_STATE();
    case 1087:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1311);
      END_STATE();
    case 1088:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1312);
      END_STATE();
    case 1089:
      if (lookahead == '_') ADVANCE(1313);
      END_STATE();
    case 1090:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1314);
      END_STATE();
    case 1091:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1315);
      END_STATE();
    case 1092:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1316);
      END_STATE();
    case 1093:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1317);
      END_STATE();
    case 1094:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1318);
      END_STATE();
    case 1095:
      if (lookahead == '_') ADVANCE(1319);
      END_STATE();
    case 1096:
      if (lookahead == '_') ADVANCE(1320);
      END_STATE();
    case 1097:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1321);
      END_STATE();
    case 1098:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1322);
      END_STATE();
    case 1099:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1323);
      END_STATE();
    case 1100:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1324);
      END_STATE();
    case 1101:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1325);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1326);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1327);
      END_STATE();
    case 1102:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1328);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1329);
      END_STATE();
    case 1103:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1330);
      END_STATE();
    case 1104:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1331);
      END_STATE();
    case 1105:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1332);
      END_STATE();
    case 1106:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1333);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1334);
      END_STATE();
    case 1107:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1335);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1336);
      END_STATE();
    case 1108:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1337);
      END_STATE();
    case 1109:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1338);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1339);
      END_STATE();
    case 1110:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1340);
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1341);
      END_STATE();
    case 1111:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1342);
      END_STATE();
    case 1112:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1343);
      END_STATE();
    case 1113:
      if (lookahead == '_') ADVANCE(1344);
      END_STATE();
    case 1114:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1345);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1346);
      END_STATE();
    case 1115:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1347);
      END_STATE();
    case 1116:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1348);
      END_STATE();
    case 1117:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1349);
      END_STATE();
    case 1118:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1350);
      END_STATE();
    case 1119:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1351);
      END_STATE();
    case 1120:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1352);
      END_STATE();
    case 1121:
      if (lookahead == '_') ADVANCE(1353);
      END_STATE();
    case 1122:
      if (lookahead == '_') ADVANCE(1354);
      END_STATE();
    case 1123:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1355);
      END_STATE();
    case 1124:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1356);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1357);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1358);
      END_STATE();
    case 1125:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1359);
      END_STATE();
    case 1126:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1360);
      END_STATE();
    case 1127:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1361);
      END_STATE();
    case 1128:
      if (lookahead == '_') ADVANCE(1362);
      END_STATE();
    case 1129:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1363);
      END_STATE();
    case 1130:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1364);
      END_STATE();
    case 1131:
      if (lookahead == '_') ADVANCE(1365);
      END_STATE();
    case 1132:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1366);
      END_STATE();
    case 1133:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1367);
      END_STATE();
    case 1134:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1368);
      END_STATE();
    case 1135:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1369);
      END_STATE();
    case 1136:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1370);
      END_STATE();
    case 1137:
      ACCEPT_TOKEN(sym_file_dir);
      END_STATE();
    case 1138:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1371);
      END_STATE();
    case 1139:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1372);
      END_STATE();
    case 1140:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1373);
      END_STATE();
    case 1141:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1374);
      END_STATE();
    case 1142:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1375);
      END_STATE();
    case 1143:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1376);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1377);
      END_STATE();
    case 1144:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1378);
      END_STATE();
    case 1145:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1379);
      END_STATE();
    case 1146:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1380);
      END_STATE();
    case 1147:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1381);
      END_STATE();
    case 1148:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1382);
      END_STATE();
    case 1149:
      ACCEPT_TOKEN(sym_ftp_priv);
      END_STATE();
    case 1150:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1383);
      END_STATE();
    case 1151:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1384);
      END_STATE();
    case 1152:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1385);
      END_STATE();
    case 1153:
      ACCEPT_TOKEN(sym_get_last);
      END_STATE();
    case 1154:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1386);
      END_STATE();
    case 1155:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1387);
      END_STATE();
    case 1156:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1388);
      END_STATE();
    case 1157:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1389);
      END_STATE();
    case 1158:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1390);
      END_STATE();
    case 1159:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1391);
      END_STATE();
    case 1160:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1392);
      END_STATE();
    case 1161:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1393);
      END_STATE();
    case 1162:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1394);
      END_STATE();
    case 1163:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1395);
      END_STATE();
    case 1164:
      if (lookahead == '_') ADVANCE(1396);
      END_STATE();
    case 1165:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1397);
      END_STATE();
    case 1166:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1398);
      END_STATE();
    case 1167:
      if (lookahead == '_') ADVANCE(1399);
      END_STATE();
    case 1168:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1400);
      END_STATE();
    case 1169:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1401);
      END_STATE();
    case 1170:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1402);
      END_STATE();
    case 1171:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1403);
      END_STATE();
    case 1172:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1404);
      END_STATE();
    case 1173:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1405);
      END_STATE();
    case 1174:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1406);
      END_STATE();
    case 1175:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1407);
      END_STATE();
    case 1176:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1408);
      END_STATE();
    case 1177:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1409);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1410);
      END_STATE();
    case 1178:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1411);
      END_STATE();
    case 1179:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1412);
      END_STATE();
    case 1180:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1413);
      END_STATE();
    case 1181:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1414);
      END_STATE();
    case 1182:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1415);
      END_STATE();
    case 1183:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1416);
      END_STATE();
    case 1184:
      if (lookahead == '_') ADVANCE(1417);
      END_STATE();
    case 1185:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1418);
      END_STATE();
    case 1186:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1419);
      END_STATE();
    case 1187:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1420);
      END_STATE();
    case 1188:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1421);
      END_STATE();
    case 1189:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1422);
      END_STATE();
    case 1190:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1423);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1424);
      END_STATE();
    case 1191:
      if (lookahead == '_') ADVANCE(1425);
      END_STATE();
    case 1192:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1426);
      END_STATE();
    case 1193:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1427);
      END_STATE();
    case 1194:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1428);
      END_STATE();
    case 1195:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1429);
      END_STATE();
    case 1196:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1430);
      END_STATE();
    case 1197:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1431);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1432);
      END_STATE();
    case 1198:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1433);
      END_STATE();
    case 1199:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1434);
      END_STATE();
    case 1200:
      ACCEPT_TOKEN(sym_null_str);
      END_STATE();
    case 1201:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1435);
      END_STATE();
    case 1202:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1436);
      END_STATE();
    case 1203:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1437);
      END_STATE();
    case 1204:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1438);
      END_STATE();
    case 1205:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1439);
      END_STATE();
    case 1206:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1440);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1441);
      END_STATE();
    case 1207:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1442);
      END_STATE();
    case 1208:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1443);
      END_STATE();
    case 1209:
      if (lookahead == '_') ADVANCE(1444);
      END_STATE();
    case 1210:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1445);
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1446);
      END_STATE();
    case 1211:
      if (lookahead == '_') ADVANCE(1447);
      END_STATE();
    case 1212:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1448);
      END_STATE();
    case 1213:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1449);
      END_STATE();
    case 1214:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1450);
      END_STATE();
    case 1215:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1451);
      END_STATE();
    case 1216:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1452);
      END_STATE();
    case 1217:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1453);
      END_STATE();
    case 1218:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1454);
      END_STATE();
    case 1219:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1455);
      END_STATE();
    case 1220:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1456);
      END_STATE();
    case 1221:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1457);
      END_STATE();
    case 1222:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1458);
      END_STATE();
    case 1223:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1459);
      END_STATE();
    case 1224:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1460);
      END_STATE();
    case 1225:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1461);
      END_STATE();
    case 1226:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1462);
      END_STATE();
    case 1227:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1463);
      END_STATE();
    case 1228:
      ACCEPT_TOKEN(sym_sinteger_keyword);
      END_STATE();
    case 1229:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1464);
      END_STATE();
    case 1230:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1465);
      END_STATE();
    case 1231:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1466);
      END_STATE();
    case 1232:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1467);
      END_STATE();
    case 1233:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1468);
      END_STATE();
    case 1234:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1469);
      END_STATE();
    case 1235:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1470);
      END_STATE();
    case 1236:
      if (lookahead == '_') ADVANCE(1471);
      END_STATE();
    case 1237:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1472);
      END_STATE();
    case 1238:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1473);
      END_STATE();
    case 1239:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1474);
      END_STATE();
    case 1240:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1475);
      END_STATE();
    case 1241:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1476);
      END_STATE();
    case 1242:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1477);
      END_STATE();
    case 1243:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1478);
      END_STATE();
    case 1244:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1479);
      END_STATE();
    case 1245:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1480);
      END_STATE();
    case 1246:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1481);
      END_STATE();
    case 1247:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1482);
      END_STATE();
    case 1248:
      ACCEPT_TOKEN(sym_tchannel);
      END_STATE();
    case 1249:
      if (lookahead == '_') ADVANCE(1483);
      END_STATE();
    case 1250:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1484);
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1485);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1486);
      END_STATE();
    case 1251:
      ACCEPT_TOKEN(sym_timeline);
      if (lookahead == '_') ADVANCE(1487);
      END_STATE();
    case 1252:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1488);
      END_STATE();
    case 1253:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1489);
      END_STATE();
    case 1254:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1490);
      END_STATE();
    case 1255:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1491);
      END_STATE();
    case 1256:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1492);
      END_STATE();
    case 1257:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1493);
      END_STATE();
    case 1258:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1494);
      END_STATE();
    case 1259:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1495);
      END_STATE();
    case 1260:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1496);
      END_STATE();
    case 1261:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1497);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1498);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1499);
      END_STATE();
    case 1262:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1500);
      END_STATE();
    case 1263:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1501);
      END_STATE();
    case 1264:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1502);
      END_STATE();
    case 1265:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1503);
      END_STATE();
    case 1266:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1504);
      END_STATE();
    case 1267:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1505);
      END_STATE();
    case 1268:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1506);
      END_STATE();
    case 1269:
      if (lookahead == '_') ADVANCE(1507);
      END_STATE();
    case 1270:
      if (lookahead == '_') ADVANCE(1508);
      END_STATE();
    case 1271:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1509);
      END_STATE();
    case 1272:
      ACCEPT_TOKEN(sym_volatile_keyword);
      END_STATE();
    case 1273:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1510);
      END_STATE();
    case 1274:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1511);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1512);
      END_STATE();
    case 1275:
      ACCEPT_TOKEN(sym_widechar_keyword);
      END_STATE();
    case 1276:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1513);
      END_STATE();
    case 1277:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1514);
      END_STATE();
    case 1278:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1515);
      END_STATE();
    case 1279:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1516);
      END_STATE();
    case 1280:
      ACCEPT_TOKEN(sym___ldate__);
      END_STATE();
    case 1281:
      if (lookahead == '_') ADVANCE(1517);
      END_STATE();
    case 1282:
      if (lookahead == '_') ADVANCE(1518);
      END_STATE();
    case 1283:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1519);
      END_STATE();
    case 1284:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1520);
      END_STATE();
    case 1285:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1521);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1522);
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1523);
      END_STATE();
    case 1286:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1524);
      END_STATE();
    case 1287:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1525);
      END_STATE();
    case 1288:
      ADVANCE_MAP(
        'C', 1526,
        'c', 1526,
        'O', 1527,
        'o', 1527,
        'R', 1528,
        'r', 1528,
        'W', 1529,
        'w', 1529,
      );
      END_STATE();
    case 1289:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1530);
      END_STATE();
    case 1290:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1531);
      END_STATE();
    case 1291:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1532);
      END_STATE();
    case 1292:
      if (lookahead == '_') ADVANCE(1533);
      END_STATE();
    case 1293:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1534);
      END_STATE();
    case 1294:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1535);
      END_STATE();
    case 1295:
      if (lookahead == '_') ADVANCE(1536);
      END_STATE();
    case 1296:
      if (lookahead == '_') ADVANCE(1537);
      END_STATE();
    case 1297:
      if (lookahead == '_') ADVANCE(1538);
      END_STATE();
    case 1298:
      ACCEPT_TOKEN(sym_abs_value);
      END_STATE();
    case 1299:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1539);
      END_STATE();
    case 1300:
      ACCEPT_TOKEN(sym_amx_debug);
      END_STATE();
    case 1301:
      ACCEPT_TOKEN(sym_amx_error);
      END_STATE();
    case 1302:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1540);
      END_STATE();
    case 1303:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1541);
      END_STATE();
    case 1304:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1542);
      END_STATE();
    case 1305:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1543);
      END_STATE();
    case 1306:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1544);
      END_STATE();
    case 1307:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1545);
      END_STATE();
    case 1308:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1546);
      END_STATE();
    case 1309:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1547);
      END_STATE();
    case 1310:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1548);
      END_STATE();
    case 1311:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1549);
      END_STATE();
    case 1312:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1550);
      END_STATE();
    case 1313:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1551);
      END_STATE();
    case 1314:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1552);
      END_STATE();
    case 1315:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1553);
      END_STATE();
    case 1316:
      ACCEPT_TOKEN(sym_data_feed);
      END_STATE();
    case 1317:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1554);
      END_STATE();
    case 1318:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1555);
      END_STATE();
    case 1319:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1556);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1557);
      END_STATE();
    case 1320:
      ADVANCE_MAP(
        'C', 1558,
        'c', 1558,
        'D', 1559,
        'd', 1559,
        'F', 1560,
        'f', 1560,
        'I', 1561,
        'i', 1561,
        'L', 1562,
        'l', 1562,
        'M', 1563,
        'm', 1563,
        'N', 1564,
        'n', 1564,
        'S', 1565,
        's', 1565,
        'T', 1566,
        't', 1566,
        'U', 1567,
        'u', 1567,
        'V', 1568,
        'v', 1568,
      );
      END_STATE();
    case 1321:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1569);
      END_STATE();
    case 1322:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1570);
      END_STATE();
    case 1323:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1571);
      END_STATE();
    case 1324:
      if (lookahead == '_') ADVANCE(1572);
      END_STATE();
    case 1325:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1573);
      END_STATE();
    case 1326:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1574);
      END_STATE();
    case 1327:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1575);
      END_STATE();
    case 1328:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1576);
      END_STATE();
    case 1329:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1577);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1578);
      END_STATE();
    case 1330:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1579);
      END_STATE();
    case 1331:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1580);
      END_STATE();
    case 1332:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1581);
      END_STATE();
    case 1333:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1582);
      END_STATE();
    case 1334:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1583);
      END_STATE();
    case 1335:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1584);
      END_STATE();
    case 1336:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1585);
      END_STATE();
    case 1337:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1586);
      END_STATE();
    case 1338:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1587);
      END_STATE();
    case 1339:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1588);
      END_STATE();
    case 1340:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1589);
      END_STATE();
    case 1341:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1590);
      END_STATE();
    case 1342:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1591);
      END_STATE();
    case 1343:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1592);
      END_STATE();
    case 1344:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1593);
      END_STATE();
    case 1345:
      ACCEPT_TOKEN(sym_device_id);
      if (lookahead == '_') ADVANCE(1594);
      END_STATE();
    case 1346:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1595);
      END_STATE();
    case 1347:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1596);
      END_STATE();
    case 1348:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1597);
      END_STATE();
    case 1349:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1598);
      END_STATE();
    case 1350:
      if (lookahead == '_') ADVANCE(1599);
      END_STATE();
    case 1351:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1600);
      END_STATE();
    case 1352:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1601);
      END_STATE();
    case 1353:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1602);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1603);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1604);
      END_STATE();
    case 1354:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1605);
      END_STATE();
    case 1355:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1606);
      END_STATE();
    case 1356:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1607);
      END_STATE();
    case 1357:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1608);
      END_STATE();
    case 1358:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1609);
      END_STATE();
    case 1359:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1610);
      END_STATE();
    case 1360:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1611);
      END_STATE();
    case 1361:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1612);
      END_STATE();
    case 1362:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1613);
      END_STATE();
    case 1363:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1614);
      END_STATE();
    case 1364:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1615);
      END_STATE();
    case 1365:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1616);
      END_STATE();
    case 1366:
      ACCEPT_TOKEN(sym_exp_value);
      END_STATE();
    case 1367:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1617);
      END_STATE();
    case 1368:
      ACCEPT_TOKEN(sym_file_copy);
      END_STATE();
    case 1369:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1618);
      END_STATE();
    case 1370:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1619);
      END_STATE();
    case 1371:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1620);
      END_STATE();
    case 1372:
      ACCEPT_TOKEN(sym_file_open);
      END_STATE();
    case 1373:
      ACCEPT_TOKEN(sym_file_read);
      if (lookahead == '_') ADVANCE(1621);
      END_STATE();
    case 1374:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1622);
      END_STATE();
    case 1375:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1623);
      END_STATE();
    case 1376:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1624);
      END_STATE();
    case 1377:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1625);
      END_STATE();
    case 1378:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1626);
      END_STATE();
    case 1379:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1627);
      END_STATE();
    case 1380:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1628);
      END_STATE();
    case 1381:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1629);
      END_STATE();
    case 1382:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1630);
      END_STATE();
    case 1383:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1631);
      END_STATE();
    case 1384:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1632);
      END_STATE();
    case 1385:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1633);
      END_STATE();
    case 1386:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1634);
      END_STATE();
    case 1387:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1635);
      END_STATE();
    case 1388:
      if (lookahead == '_') ADVANCE(1636);
      END_STATE();
    case 1389:
      if (lookahead == '_') ADVANCE(1637);
      END_STATE();
    case 1390:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1638);
      END_STATE();
    case 1391:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1639);
      END_STATE();
    case 1392:
      ACCEPT_TOKEN(sym_get_timer);
      END_STATE();
    case 1393:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1640);
      END_STATE();
    case 1394:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1641);
      END_STATE();
    case 1395:
      ACCEPT_TOKEN(sym_http_priv);
      END_STATE();
    case 1396:
      if (lookahead == 'Q' ||
          lookahead == 'q') ADVANCE(1642);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1643);
      END_STATE();
    case 1397:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1644);
      END_STATE();
    case 1398:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1645);
      END_STATE();
    case 1399:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1646);
      END_STATE();
    case 1400:
      if (lookahead == '_') ADVANCE(1647);
      END_STATE();
    case 1401:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1648);
      END_STATE();
    case 1402:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1649);
      END_STATE();
    case 1403:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1650);
      END_STATE();
    case 1404:
      if (lookahead == '_') ADVANCE(1651);
      END_STATE();
    case 1405:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1652);
      END_STATE();
    case 1406:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1653);
      END_STATE();
    case 1407:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1654);
      END_STATE();
    case 1408:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1655);
      END_STATE();
    case 1409:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1656);
      END_STATE();
    case 1410:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1657);
      END_STATE();
    case 1411:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1658);
      END_STATE();
    case 1412:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1659);
      END_STATE();
    case 1413:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1660);
      END_STATE();
    case 1414:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1661);
      END_STATE();
    case 1415:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1662);
      END_STATE();
    case 1416:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1663);
      END_STATE();
    case 1417:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1664);
      END_STATE();
    case 1418:
      if (lookahead == '_') ADVANCE(1665);
      END_STATE();
    case 1419:
      ACCEPT_TOKEN(sym_local_var_keyword);
      END_STATE();
    case 1420:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1666);
      END_STATE();
    case 1421:
      ACCEPT_TOKEN(sym_log_value);
      END_STATE();
    case 1422:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1667);
      END_STATE();
    case 1423:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1668);
      END_STATE();
    case 1424:
      ACCEPT_TOKEN(sym_master_sn);
      END_STATE();
    case 1425:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1669);
      END_STATE();
    case 1426:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1670);
      END_STATE();
    case 1427:
      ACCEPT_TOKEN(sym_max_value);
      END_STATE();
    case 1428:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1671);
      END_STATE();
    case 1429:
      ACCEPT_TOKEN(sym_min_value);
      END_STATE();
    case 1430:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1672);
      END_STATE();
    case 1431:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(1673);
      END_STATE();
    case 1432:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1674);
      END_STATE();
    case 1433:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1675);
      END_STATE();
    case 1434:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1676);
      END_STATE();
    case 1435:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1677);
      END_STATE();
    case 1436:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1678);
      END_STATE();
    case 1437:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1679);
      END_STATE();
    case 1438:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1680);
      END_STATE();
    case 1439:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1681);
      END_STATE();
    case 1440:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1682);
      END_STATE();
    case 1441:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1683);
      END_STATE();
    case 1442:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1684);
      END_STATE();
    case 1443:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1685);
      END_STATE();
    case 1444:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1686);
      END_STATE();
    case 1445:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1687);
      END_STATE();
    case 1446:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1688);
      END_STATE();
    case 1447:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1689);
      END_STATE();
    case 1448:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1690);
      END_STATE();
    case 1449:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1691);
      END_STATE();
    case 1450:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1692);
      END_STATE();
    case 1451:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1693);
      END_STATE();
    case 1452:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1694);
      END_STATE();
    case 1453:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1695);
      END_STATE();
    case 1454:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1696);
      END_STATE();
    case 1455:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1697);
      END_STATE();
    case 1456:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1698);
      END_STATE();
    case 1457:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1699);
      END_STATE();
    case 1458:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1700);
      END_STATE();
    case 1459:
      if (lookahead == '_') ADVANCE(1701);
      END_STATE();
    case 1460:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1702);
      END_STATE();
    case 1461:
      ACCEPT_TOKEN(sym_set_timer);
      END_STATE();
    case 1462:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1703);
      END_STATE();
    case 1463:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1704);
      END_STATE();
    case 1464:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1705);
      END_STATE();
    case 1465:
      ACCEPT_TOKEN(sym_smtp_from);
      END_STATE();
    case 1466:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1706);
      END_STATE();
    case 1467:
      if (lookahead == '_') ADVANCE(1707);
      END_STATE();
    case 1468:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1708);
      END_STATE();
    case 1469:
      ACCEPT_TOKEN(sym_smtp_send);
      END_STATE();
    case 1470:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1709);
      END_STATE();
    case 1471:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1710);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1711);
      END_STATE();
    case 1472:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1712);
      END_STATE();
    case 1473:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1713);
      END_STATE();
    case 1474:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1714);
      END_STATE();
    case 1475:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1715);
      END_STATE();
    case 1476:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1716);
      END_STATE();
    case 1477:
      if (lookahead == '_') ADVANCE(1717);
      END_STATE();
    case 1478:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1718);
      END_STATE();
    case 1479:
      if (lookahead == '_') ADVANCE(1719);
      END_STATE();
    case 1480:
      ACCEPT_TOKEN(sym_structure_keyword);
      END_STATE();
    case 1481:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1720);
      END_STATE();
    case 1482:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1721);
      END_STATE();
    case 1483:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1722);
      END_STATE();
    case 1484:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1723);
      END_STATE();
    case 1485:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1724);
      END_STATE();
    case 1486:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1725);
      END_STATE();
    case 1487:
      ADVANCE_MAP(
        'A', 1726,
        'a', 1726,
        'C', 1727,
        'c', 1727,
        'D', 1728,
        'd', 1728,
        'G', 1729,
        'g', 1729,
        'O', 1730,
        'o', 1730,
        'P', 1731,
        'p', 1731,
        'R', 1732,
        'r', 1732,
        'S', 1733,
        's', 1733,
      );
      END_STATE();
    case 1488:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1734);
      END_STATE();
    case 1489:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1735);
      END_STATE();
    case 1490:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1736);
      END_STATE();
    case 1491:
      ACCEPT_TOKEN(sym_devchan_total_off_keyword);
      END_STATE();
    case 1492:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1737);
      END_STATE();
    case 1493:
      ACCEPT_TOKEN(sym_ttimeline);
      END_STATE();
    case 1494:
      ACCEPT_TOKEN(sym_type_cast);
      END_STATE();
    case 1495:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1738);
      END_STATE();
    case 1496:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1739);
      END_STATE();
    case 1497:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1740);
      END_STATE();
    case 1498:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1741);
      END_STATE();
    case 1499:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1742);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1743);
      END_STATE();
    case 1500:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1744);
      END_STATE();
    case 1501:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1745);
      END_STATE();
    case 1502:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1746);
      END_STATE();
    case 1503:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1747);
      END_STATE();
    case 1504:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1748);
      END_STATE();
    case 1505:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1749);
      END_STATE();
    case 1506:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1750);
      END_STATE();
    case 1507:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1751);
      END_STATE();
    case 1508:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1752);
      END_STATE();
    case 1509:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1753);
      END_STATE();
    case 1510:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1754);
      END_STATE();
    case 1511:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1755);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1756);
      END_STATE();
    case 1512:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1757);
      END_STATE();
    case 1513:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1758);
      END_STATE();
    case 1514:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1759);
      END_STATE();
    case 1515:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1760);
      END_STATE();
    case 1516:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1761);
      END_STATE();
    case 1517:
      if (lookahead == '_') ADVANCE(1762);
      END_STATE();
    case 1518:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1763);
      END_STATE();
    case 1519:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1764);
      END_STATE();
    case 1520:
      if (lookahead == '_') ADVANCE(1765);
      END_STATE();
    case 1521:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1766);
      END_STATE();
    case 1522:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1767);
      END_STATE();
    case 1523:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1768);
      END_STATE();
    case 1524:
      if (lookahead == '_') ADVANCE(1769);
      END_STATE();
    case 1525:
      if (lookahead == '_') ADVANCE(1770);
      END_STATE();
    case 1526:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1771);
      END_STATE();
    case 1527:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1772);
      END_STATE();
    case 1528:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1773);
      END_STATE();
    case 1529:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1774);
      END_STATE();
    case 1530:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1775);
      END_STATE();
    case 1531:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1776);
      END_STATE();
    case 1532:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1777);
      END_STATE();
    case 1533:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1778);
      END_STATE();
    case 1534:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1779);
      END_STATE();
    case 1535:
      if (lookahead == '_') ADVANCE(1780);
      END_STATE();
    case 1536:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1781);
      END_STATE();
    case 1537:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1782);
      END_STATE();
    case 1538:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1783);
      END_STATE();
    case 1539:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1784);
      END_STATE();
    case 1540:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1785);
      END_STATE();
    case 1541:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1786);
      END_STATE();
    case 1542:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1787);
      END_STATE();
    case 1543:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1788);
      END_STATE();
    case 1544:
      if (lookahead == '_') ADVANCE(1789);
      END_STATE();
    case 1545:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1790);
      END_STATE();
    case 1546:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1791);
      END_STATE();
    case 1547:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1792);
      END_STATE();
    case 1548:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1793);
      END_STATE();
    case 1549:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(1794);
      END_STATE();
    case 1550:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1795);
      END_STATE();
    case 1551:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1796);
      END_STATE();
    case 1552:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1797);
      END_STATE();
    case 1553:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1798);
      END_STATE();
    case 1554:
      ACCEPT_TOKEN(sym_data_field);
      END_STATE();
    case 1555:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1799);
      END_STATE();
    case 1556:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1800);
      END_STATE();
    case 1557:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1801);
      END_STATE();
    case 1558:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1802);
      END_STATE();
    case 1559:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1803);
      END_STATE();
    case 1560:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1804);
      END_STATE();
    case 1561:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1805);
      END_STATE();
    case 1562:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1806);
      END_STATE();
    case 1563:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1807);
      END_STATE();
    case 1564:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1808);
      END_STATE();
    case 1565:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1809);
      END_STATE();
    case 1566:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1810);
      END_STATE();
    case 1567:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1811);
      END_STATE();
    case 1568:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1812);
      END_STATE();
    case 1569:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1813);
      END_STATE();
    case 1570:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1814);
      END_STATE();
    case 1571:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1815);
      END_STATE();
    case 1572:
      ADVANCE_MAP(
        'D', 1816,
        'd', 1816,
        'I', 1817,
        'i', 1817,
        'S', 1818,
        's', 1818,
        'T', 1819,
        't', 1819,
      );
      END_STATE();
    case 1573:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1820);
      END_STATE();
    case 1574:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1821);
      END_STATE();
    case 1575:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1822);
      END_STATE();
    case 1576:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1823);
      END_STATE();
    case 1577:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1824);
      END_STATE();
    case 1578:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1825);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1826);
      END_STATE();
    case 1579:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1827);
      END_STATE();
    case 1580:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1828);
      END_STATE();
    case 1581:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1829);
      END_STATE();
    case 1582:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1830);
      END_STATE();
    case 1583:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1831);
      END_STATE();
    case 1584:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1832);
      END_STATE();
    case 1585:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1833);
      END_STATE();
    case 1586:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1834);
      END_STATE();
    case 1587:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1835);
      END_STATE();
    case 1588:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1836);
      END_STATE();
    case 1589:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1837);
      END_STATE();
    case 1590:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1838);
      END_STATE();
    case 1591:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1839);
      END_STATE();
    case 1592:
      if (lookahead == '_') ADVANCE(1840);
      END_STATE();
    case 1593:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1841);
      END_STATE();
    case 1594:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1842);
      END_STATE();
    case 1595:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1843);
      END_STATE();
    case 1596:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1844);
      END_STATE();
    case 1597:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1845);
      END_STATE();
    case 1598:
      ACCEPT_TOKEN(sym_dns_struct);
      END_STATE();
    case 1599:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1846);
      END_STATE();
    case 1600:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1847);
      END_STATE();
    case 1601:
      ACCEPT_TOKEN(sym_do_release);
      END_STATE();
    case 1602:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1848);
      END_STATE();
    case 1603:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1849);
      END_STATE();
    case 1604:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1850);
      END_STATE();
    case 1605:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1851);
      END_STATE();
    case 1606:
      ACCEPT_TOKEN(sym_dv_channel);
      END_STATE();
    case 1607:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1852);
      END_STATE();
    case 1608:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1853);
      END_STATE();
    case 1609:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1854);
      END_STATE();
    case 1610:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1855);
      END_STATE();
    case 1611:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1856);
      END_STATE();
    case 1612:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(1857);
      END_STATE();
    case 1613:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1858);
      END_STATE();
    case 1614:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1859);
      END_STATE();
    case 1615:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1860);
      END_STATE();
    case 1616:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1861);
      END_STATE();
    case 1617:
      ACCEPT_TOKEN(sym_file_close);
      END_STATE();
    case 1618:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1862);
      END_STATE();
    case 1619:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1863);
      END_STATE();
    case 1620:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1864);
      END_STATE();
    case 1621:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1865);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1866);
      END_STATE();
    case 1622:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1867);
      END_STATE();
    case 1623:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1868);
      END_STATE();
    case 1624:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1869);
      END_STATE();
    case 1625:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(1870);
      END_STATE();
    case 1626:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1871);
      END_STATE();
    case 1627:
      ACCEPT_TOKEN(sym_file_write);
      if (lookahead == '_') ADVANCE(1872);
      END_STATE();
    case 1628:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1873);
      END_STATE();
    case 1629:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1874);
      END_STATE();
    case 1630:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1875);
      END_STATE();
    case 1631:
      if (lookahead == '_') ADVANCE(1876);
      END_STATE();
    case 1632:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1877);
      END_STATE();
    case 1633:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1878);
      END_STATE();
    case 1634:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1879);
      END_STATE();
    case 1635:
      if (lookahead == '_') ADVANCE(1880);
      END_STATE();
    case 1636:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1881);
      END_STATE();
    case 1637:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1882);
      END_STATE();
    case 1638:
      if (lookahead == '_') ADVANCE(1883);
      END_STATE();
    case 1639:
      if (lookahead == '_') ADVANCE(1884);
      END_STATE();
    case 1640:
      if (lookahead == '_') ADVANCE(1885);
      END_STATE();
    case 1641:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1886);
      END_STATE();
    case 1642:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1887);
      END_STATE();
    case 1643:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(1888);
      END_STATE();
    case 1644:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1889);
      END_STATE();
    case 1645:
      if (lookahead == '_') ADVANCE(1890);
      END_STATE();
    case 1646:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1891);
      END_STATE();
    case 1647:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1892);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1893);
      END_STATE();
    case 1648:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1894);
      END_STATE();
    case 1649:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1895);
      END_STATE();
    case 1650:
      if (lookahead == '_') ADVANCE(1896);
      END_STATE();
    case 1651:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1897);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1898);
      END_STATE();
    case 1652:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1899);
      END_STATE();
    case 1653:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1900);
      END_STATE();
    case 1654:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1901);
      END_STATE();
    case 1655:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1902);
      END_STATE();
    case 1656:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1903);
      END_STATE();
    case 1657:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1904);
      END_STATE();
    case 1658:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1905);
      END_STATE();
    case 1659:
      if (lookahead == '_') ADVANCE(1906);
      END_STATE();
    case 1660:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1907);
      END_STATE();
    case 1661:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1908);
      END_STATE();
    case 1662:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1909);
      END_STATE();
    case 1663:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1910);
      END_STATE();
    case 1664:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1911);
      END_STATE();
    case 1665:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1912);
      END_STATE();
    case 1666:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1913);
      END_STATE();
    case 1667:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1914);
      END_STATE();
    case 1668:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1915);
      END_STATE();
    case 1669:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1916);
      END_STATE();
    case 1670:
      if (lookahead == '_') ADVANCE(1917);
      END_STATE();
    case 1671:
      ACCEPT_TOKEN(sym_mid_string);
      END_STATE();
    case 1672:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1918);
      END_STATE();
    case 1673:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1919);
      END_STATE();
    case 1674:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1920);
      END_STATE();
    case 1675:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1921);
      END_STATE();
    case 1676:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1922);
      END_STATE();
    case 1677:
      ACCEPT_TOKEN(sym_persistent_keyword);
      END_STATE();
    case 1678:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1923);
      END_STATE();
    case 1679:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1924);
      END_STATE();
    case 1680:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1925);
      END_STATE();
    case 1681:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1926);
      END_STATE();
    case 1682:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1927);
      END_STATE();
    case 1683:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1928);
      END_STATE();
    case 1684:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1929);
      END_STATE();
    case 1685:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1930);
      END_STATE();
    case 1686:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1931);
      END_STATE();
    case 1687:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1932);
      END_STATE();
    case 1688:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1933);
      END_STATE();
    case 1689:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1934);
      END_STATE();
    case 1690:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1935);
      END_STATE();
    case 1691:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1936);
      END_STATE();
    case 1692:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1937);
      END_STATE();
    case 1693:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1938);
      END_STATE();
    case 1694:
      ACCEPT_TOKEN(sym_send_level_keyword);
      END_STATE();
    case 1695:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(1939);
      END_STATE();
    case 1696:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1940);
      END_STATE();
    case 1697:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1941);
      END_STATE();
    case 1698:
      if (lookahead == '_') ADVANCE(1942);
      END_STATE();
    case 1699:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1943);
      END_STATE();
    case 1700:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1944);
      END_STATE();
    case 1701:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1945);
      END_STATE();
    case 1702:
      if (lookahead == '_') ADVANCE(1946);
      END_STATE();
    case 1703:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1947);
      END_STATE();
    case 1704:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1948);
      END_STATE();
    case 1705:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(1949);
      END_STATE();
    case 1706:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1950);
      END_STATE();
    case 1707:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1951);
      END_STATE();
    case 1708:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1952);
      END_STATE();
    case 1709:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1953);
      END_STATE();
    case 1710:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1954);
      END_STATE();
    case 1711:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1955);
      END_STATE();
    case 1712:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1956);
      END_STATE();
    case 1713:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1957);
      END_STATE();
    case 1714:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1958);
      END_STATE();
    case 1715:
      ACCEPT_TOKEN(sym_sqrt_value);
      END_STATE();
    case 1716:
      if (lookahead == '_') ADVANCE(1959);
      END_STATE();
    case 1717:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1960);
      END_STATE();
    case 1718:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1961);
      END_STATE();
    case 1719:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(1962);
      END_STATE();
    case 1720:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(1963);
      END_STATE();
    case 1721:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1964);
      END_STATE();
    case 1722:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1965);
      END_STATE();
    case 1723:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1966);
      END_STATE();
    case 1724:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1967);
      END_STATE();
    case 1725:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1968);
      END_STATE();
    case 1726:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(1969);
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1970);
      END_STATE();
    case 1727:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(1971);
      END_STATE();
    case 1728:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(1972);
      END_STATE();
    case 1729:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1973);
      END_STATE();
    case 1730:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1974);
      END_STATE();
    case 1731:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1975);
      END_STATE();
    case 1732:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1976);
      END_STATE();
    case 1733:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1977);
      END_STATE();
    case 1734:
      if (lookahead == '_') ADVANCE(1978);
      END_STATE();
    case 1735:
      if (lookahead == '_') ADVANCE(1979);
      END_STATE();
    case 1736:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1980);
      END_STATE();
    case 1737:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1981);
      END_STATE();
    case 1738:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(1982);
      END_STATE();
    case 1739:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(1983);
      END_STATE();
    case 1740:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1984);
      END_STATE();
    case 1741:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1985);
      END_STATE();
    case 1742:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(1986);
      END_STATE();
    case 1743:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1987);
      END_STATE();
    case 1744:
      ACCEPT_TOKEN(sym_url_struct);
      END_STATE();
    case 1745:
      ACCEPT_TOKEN(sym_user1_priv);
      END_STATE();
    case 1746:
      ACCEPT_TOKEN(sym_user2_priv);
      END_STATE();
    case 1747:
      ACCEPT_TOKEN(sym_user3_priv);
      END_STATE();
    case 1748:
      ACCEPT_TOKEN(sym_user4_priv);
      END_STATE();
    case 1749:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(1988);
      END_STATE();
    case 1750:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(1989);
      END_STATE();
    case 1751:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1990);
      END_STATE();
    case 1752:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(1991);
      END_STATE();
    case 1753:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(1992);
      END_STATE();
    case 1754:
      ACCEPT_TOKEN(sym_wait_until_keyword);
      END_STATE();
    case 1755:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1993);
      END_STATE();
    case 1756:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(1994);
      END_STATE();
    case 1757:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(1995);
      END_STATE();
    case 1758:
      if (lookahead == '_') ADVANCE(1996);
      END_STATE();
    case 1759:
      if (lookahead == '_') ADVANCE(1997);
      END_STATE();
    case 1760:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(1998);
      END_STATE();
    case 1761:
      if (lookahead == '_') ADVANCE(1999);
      END_STATE();
    case 1762:
      ACCEPT_TOKEN(sym___netlinx__);
      END_STATE();
    case 1763:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2000);
      END_STATE();
    case 1764:
      if (lookahead == '_') ADVANCE(2001);
      END_STATE();
    case 1765:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2002);
      END_STATE();
    case 1766:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2003);
      END_STATE();
    case 1767:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2004);
      END_STATE();
    case 1768:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2005);
      END_STATE();
    case 1769:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2006);
      END_STATE();
    case 1770:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2007);
      END_STATE();
    case 1771:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2008);
      END_STATE();
    case 1772:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2009);
      END_STATE();
    case 1773:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2010);
      END_STATE();
    case 1774:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2011);
      END_STATE();
    case 1775:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2012);
      END_STATE();
    case 1776:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2013);
      END_STATE();
    case 1777:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2014);
      END_STATE();
    case 1778:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2015);
      END_STATE();
    case 1779:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2016);
      END_STATE();
    case 1780:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2017);
      END_STATE();
    case 1781:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2018);
      END_STATE();
    case 1782:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2019);
      END_STATE();
    case 1783:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2020);
      END_STATE();
    case 1784:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2021);
      END_STATE();
    case 1785:
      ACCEPT_TOKEN(sym_amx_warning);
      END_STATE();
    case 1786:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2022);
      END_STATE();
    case 1787:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2023);
      END_STATE();
    case 1788:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2024);
      END_STATE();
    case 1789:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(2025);
      END_STATE();
    case 1790:
      ACCEPT_TOKEN(sym_cancel_wait_keyword);
      if (lookahead == '_') ADVANCE(2026);
      END_STATE();
    case 1791:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2027);
      END_STATE();
    case 1792:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2028);
      END_STATE();
    case 1793:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2029);
      END_STATE();
    case 1794:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2030);
      END_STATE();
    case 1795:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2031);
      END_STATE();
    case 1796:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2032);
      END_STATE();
    case 1797:
      if (lookahead == '_') ADVANCE(2033);
      END_STATE();
    case 1798:
      if (lookahead == '_') ADVANCE(2034);
      END_STATE();
    case 1799:
      if (lookahead == '_') ADVANCE(2035);
      END_STATE();
    case 1800:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2036);
      END_STATE();
    case 1801:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2037);
      END_STATE();
    case 1802:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2038);
      END_STATE();
    case 1803:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2039);
      END_STATE();
    case 1804:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2040);
      END_STATE();
    case 1805:
      if (lookahead == '_') ADVANCE(2041);
      END_STATE();
    case 1806:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2042);
      END_STATE();
    case 1807:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2043);
      END_STATE();
    case 1808:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2044);
      END_STATE();
    case 1809:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2045);
      END_STATE();
    case 1810:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2046);
      END_STATE();
    case 1811:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2047);
      END_STATE();
    case 1812:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2048);
      END_STATE();
    case 1813:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2049);
      END_STATE();
    case 1814:
      ACCEPT_TOKEN(sym_data_record);
      END_STATE();
    case 1815:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2050);
      END_STATE();
    case 1816:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2051);
      END_STATE();
    case 1817:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2052);
      END_STATE();
    case 1818:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2053);
      END_STATE();
    case 1819:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2054);
      END_STATE();
    case 1820:
      ACCEPT_TOKEN(sym_date_to_day);
      END_STATE();
    case 1821:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2055);
      END_STATE();
    case 1822:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2056);
      END_STATE();
    case 1823:
      ACCEPT_TOKEN(sym_define_call_keyword);
      END_STATE();
    case 1824:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2057);
      END_STATE();
    case 1825:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2058);
      END_STATE();
    case 1826:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2059);
      END_STATE();
    case 1827:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2060);
      END_STATE();
    case 1828:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2061);
      END_STATE();
    case 1829:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2062);
      END_STATE();
    case 1830:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2063);
      END_STATE();
    case 1831:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2064);
      END_STATE();
    case 1832:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2065);
      END_STATE();
    case 1833:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2066);
      END_STATE();
    case 1834:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2067);
      END_STATE();
    case 1835:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2068);
      END_STATE();
    case 1836:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2069);
      END_STATE();
    case 1837:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2070);
      END_STATE();
    case 1838:
      ACCEPT_TOKEN(sym_define_type_keyword);
      END_STATE();
    case 1839:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2071);
      END_STATE();
    case 1840:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2072);
      END_STATE();
    case 1841:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2073);
      END_STATE();
    case 1842:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2074);
      END_STATE();
    case 1843:
      ACCEPT_TOKEN(sym_device_info);
      END_STATE();
    case 1844:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2075);
      END_STATE();
    case 1845:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2076);
      END_STATE();
    case 1846:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2077);
      END_STATE();
    case 1847:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2078);
      END_STATE();
    case 1848:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2079);
      END_STATE();
    case 1849:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2080);
      END_STATE();
    case 1850:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2081);
      END_STATE();
    case 1851:
      if (lookahead == 'Z' ||
          lookahead == 'z') ADVANCE(2082);
      END_STATE();
    case 1852:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2083);
      END_STATE();
    case 1853:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2084);
      END_STATE();
    case 1854:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2085);
      END_STATE();
    case 1855:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2086);
      END_STATE();
    case 1856:
      if (lookahead == '_') ADVANCE(2087);
      END_STATE();
    case 1857:
      if (lookahead == '_') ADVANCE(2088);
      END_STATE();
    case 1858:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2089);
      END_STATE();
    case 1859:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2090);
      END_STATE();
    case 1860:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2091);
      END_STATE();
    case 1861:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2092);
      END_STATE();
    case 1862:
      if (lookahead == '_') ADVANCE(2093);
      END_STATE();
    case 1863:
      ACCEPT_TOKEN(sym_file_delete);
      END_STATE();
    case 1864:
      ACCEPT_TOKEN(sym_file_getdir);
      END_STATE();
    case 1865:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2094);
      END_STATE();
    case 1866:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2095);
      END_STATE();
    case 1867:
      if (lookahead == '_') ADVANCE(2096);
      END_STATE();
    case 1868:
      ACCEPT_TOKEN(sym_file_rename);
      END_STATE();
    case 1869:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2097);
      END_STATE();
    case 1870:
      ACCEPT_TOKEN(sym_file_rw_new);
      END_STATE();
    case 1871:
      ACCEPT_TOKEN(sym_file_setdir);
      END_STATE();
    case 1872:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2098);
      END_STATE();
    case 1873:
      ACCEPT_TOKEN(sym_find_string);
      END_STATE();
    case 1874:
      if (lookahead == '_') ADVANCE(2099);
      END_STATE();
    case 1875:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2100);
      END_STATE();
    case 1876:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2101);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2102);
      END_STATE();
    case 1877:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2103);
      END_STATE();
    case 1878:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2104);
      END_STATE();
    case 1879:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2105);
      END_STATE();
    case 1880:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2106);
      END_STATE();
    case 1881:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2107);
      END_STATE();
    case 1882:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2108);
      END_STATE();
    case 1883:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2109);
      END_STATE();
    case 1884:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2110);
      END_STATE();
    case 1885:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2111);
      END_STATE();
    case 1886:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2112);
      END_STATE();
    case 1887:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2113);
      END_STATE();
    case 1888:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2114);
      END_STATE();
    case 1889:
      if (lookahead == '_') ADVANCE(2115);
      END_STATE();
    case 1890:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2116);
      END_STATE();
    case 1891:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2117);
      END_STATE();
    case 1892:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2118);
      END_STATE();
    case 1893:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2119);
      END_STATE();
    case 1894:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2120);
      END_STATE();
    case 1895:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2121);
      END_STATE();
    case 1896:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2122);
      END_STATE();
    case 1897:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2123);
      END_STATE();
    case 1898:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2124);
      END_STATE();
    case 1899:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2125);
      END_STATE();
    case 1900:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2126);
      END_STATE();
    case 1901:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2127);
      END_STATE();
    case 1902:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2128);
      END_STATE();
    case 1903:
      ACCEPT_TOKEN(sym_ip_ttl_site);
      END_STATE();
    case 1904:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2129);
      END_STATE();
    case 1905:
      ACCEPT_TOKEN(sym_ip_udp_2way);
      END_STATE();
    case 1906:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2130);
      END_STATE();
    case 1907:
      ACCEPT_TOKEN(sym_left_string);
      END_STATE();
    case 1908:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2131);
      END_STATE();
    case 1909:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2132);
      END_STATE();
    case 1910:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2133);
      END_STATE();
    case 1911:
      if (lookahead == '_') ADVANCE(2134);
      END_STATE();
    case 1912:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2135);
      END_STATE();
    case 1913:
      ACCEPT_TOKEN(sym_log10_value);
      END_STATE();
    case 1914:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2136);
      END_STATE();
    case 1915:
      ACCEPT_TOKEN(sym_master_slot);
      END_STATE();
    case 1916:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2137);
      END_STATE();
    case 1917:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2138);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2139);
      END_STATE();
    case 1918:
      ACCEPT_TOKEN(sym_module_name_keyword);
      END_STATE();
    case 1919:
      if (lookahead == '_') ADVANCE(2140);
      END_STATE();
    case 1920:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2141);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2142);
      END_STATE();
    case 1921:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2143);
      END_STATE();
    case 1922:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2144);
      END_STATE();
    case 1923:
      ACCEPT_TOKEN(sym_power_value);
      END_STATE();
    case 1924:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2145);
      END_STATE();
    case 1925:
      if (lookahead == '_') ADVANCE(2146);
      END_STATE();
    case 1926:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2147);
      END_STATE();
    case 1927:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2148);
      END_STATE();
    case 1928:
      ACCEPT_TOKEN(sym_push_device);
      END_STATE();
    case 1929:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2149);
      END_STATE();
    case 1930:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2150);
      END_STATE();
    case 1931:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2151);
      END_STATE();
    case 1932:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2152);
      END_STATE();
    case 1933:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2153);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2154);
      END_STATE();
    case 1934:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2155);
      END_STATE();
    case 1935:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2156);
      END_STATE();
    case 1936:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2157);
      END_STATE();
    case 1937:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2158);
      END_STATE();
    case 1938:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2159);
      END_STATE();
    case 1939:
      ACCEPT_TOKEN(sym_send_string_keyword);
      END_STATE();
    case 1940:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2160);
      END_STATE();
    case 1941:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2161);
      END_STATE();
    case 1942:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2162);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2163);
      END_STATE();
    case 1943:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2164);
      END_STATE();
    case 1944:
      if (lookahead == '_') ADVANCE(2165);
      END_STATE();
    case 1945:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2166);
      END_STATE();
    case 1946:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2167);
      END_STATE();
    case 1947:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2168);
      END_STATE();
    case 1948:
      if (lookahead == '_') ADVANCE(2169);
      END_STATE();
    case 1949:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2170);
      END_STATE();
    case 1950:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2171);
      END_STATE();
    case 1951:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2172);
      END_STATE();
    case 1952:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2173);
      END_STATE();
    case 1953:
      if (lookahead == '_') ADVANCE(2174);
      END_STATE();
    case 1954:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2175);
      END_STATE();
    case 1955:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2176);
      END_STATE();
    case 1956:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2177);
      END_STATE();
    case 1957:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2178);
      END_STATE();
    case 1958:
      if (lookahead == '_') ADVANCE(2179);
      END_STATE();
    case 1959:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2180);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2181);
      END_STATE();
    case 1960:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2182);
      END_STATE();
    case 1961:
      if (lookahead == '_') ADVANCE(2183);
      END_STATE();
    case 1962:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2184);
      END_STATE();
    case 1963:
      ACCEPT_TOKEN(sym_system_call_keyword);
      END_STATE();
    case 1964:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2185);
      END_STATE();
    case 1965:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2186);
      END_STATE();
    case 1966:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2187);
      END_STATE();
    case 1967:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2188);
      END_STATE();
    case 1968:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2189);
      END_STATE();
    case 1969:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2190);
      END_STATE();
    case 1970:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2191);
      END_STATE();
    case 1971:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2192);
      END_STATE();
    case 1972:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2193);
      END_STATE();
    case 1973:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2194);
      END_STATE();
    case 1974:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2195);
      END_STATE();
    case 1975:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2196);
      END_STATE();
    case 1976:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2197);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2198);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2199);
      END_STATE();
    case 1977:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2200);
      END_STATE();
    case 1978:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2201);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2202);
      END_STATE();
    case 1979:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2203);
      END_STATE();
    case 1980:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2204);
      END_STATE();
    case 1981:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2205);
      END_STATE();
    case 1982:
      if (lookahead == '_') ADVANCE(2206);
      END_STATE();
    case 1983:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2207);
      END_STATE();
    case 1984:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2208);
      END_STATE();
    case 1985:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2209);
      END_STATE();
    case 1986:
      ACCEPT_TOKEN(sym_url_flg_tcp);
      END_STATE();
    case 1987:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2210);
      END_STATE();
    case 1988:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2211);
      END_STATE();
    case 1989:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2212);
      END_STATE();
    case 1990:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2213);
      END_STATE();
    case 1991:
      if (lookahead == '_') ADVANCE(2214);
      END_STATE();
    case 1992:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2215);
      END_STATE();
    case 1993:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2216);
      END_STATE();
    case 1994:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2217);
      END_STATE();
    case 1995:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2218);
      END_STATE();
    case 1996:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2219);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2220);
      END_STATE();
    case 1997:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2221);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2222);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2223);
      END_STATE();
    case 1998:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2224);
      END_STATE();
    case 1999:
      if (lookahead == '_') ADVANCE(2225);
      END_STATE();
    case 2000:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2226);
      END_STATE();
    case 2001:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2227);
      END_STATE();
    case 2002:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2228);
      END_STATE();
    case 2003:
      if (lookahead == '_') ADVANCE(2229);
      END_STATE();
    case 2004:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2230);
      END_STATE();
    case 2005:
      if (lookahead == '_') ADVANCE(2231);
      END_STATE();
    case 2006:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2232);
      END_STATE();
    case 2007:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2233);
      END_STATE();
    case 2008:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2234);
      END_STATE();
    case 2009:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2235);
      END_STATE();
    case 2010:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2236);
      END_STATE();
    case 2011:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2237);
      END_STATE();
    case 2012:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2238);
      END_STATE();
    case 2013:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2239);
      END_STATE();
    case 2014:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2240);
      END_STATE();
    case 2015:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2241);
      END_STATE();
    case 2016:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2242);
      END_STATE();
    case 2017:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2243);
      END_STATE();
    case 2018:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2244);
      END_STATE();
    case 2019:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2245);
      END_STATE();
    case 2020:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2246);
      END_STATE();
    case 2021:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2247);
      END_STATE();
    case 2022:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(2248);
      END_STATE();
    case 2023:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2249);
      END_STATE();
    case 2024:
      if (lookahead == '_') ADVANCE(2250);
      END_STATE();
    case 2025:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2251);
      END_STATE();
    case 2026:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2252);
      END_STATE();
    case 2027:
      ACCEPT_TOKEN(sym_clear_buffer_keyword);
      END_STATE();
    case 2028:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2253);
      END_STATE();
    case 2029:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2254);
      END_STATE();
    case 2030:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2255);
      END_STATE();
    case 2031:
      if (lookahead == '_') ADVANCE(2256);
      END_STATE();
    case 2032:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2257);
      END_STATE();
    case 2033:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2258);
      END_STATE();
    case 2034:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2259);
      END_STATE();
    case 2035:
      ADVANCE_MAP(
        'E', 2260,
        'e', 2260,
        'I', 2261,
        'i', 2261,
        'P', 2262,
        'p', 2262,
        'U', 2263,
        'u', 2263,
      );
      END_STATE();
    case 2036:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2264);
      END_STATE();
    case 2037:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2265);
      END_STATE();
    case 2038:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2266);
      END_STATE();
    case 2039:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2267);
      END_STATE();
    case 2040:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2268);
      END_STATE();
    case 2041:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2269);
      END_STATE();
    case 2042:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2270);
      END_STATE();
    case 2043:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2271);
      END_STATE();
    case 2044:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2272);
      END_STATE();
    case 2045:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2273);
      END_STATE();
    case 2046:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2274);
      END_STATE();
    case 2047:
      if (lookahead == '_') ADVANCE(2275);
      END_STATE();
    case 2048:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2276);
      END_STATE();
    case 2049:
      if (lookahead == '_') ADVANCE(2277);
      END_STATE();
    case 2050:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2278);
      END_STATE();
    case 2051:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2279);
      END_STATE();
    case 2052:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2280);
      END_STATE();
    case 2053:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2281);
      END_STATE();
    case 2054:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2282);
      END_STATE();
    case 2055:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2283);
      END_STATE();
    case 2056:
      ACCEPT_TOKEN(sym_date_to_year);
      END_STATE();
    case 2057:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2284);
      END_STATE();
    case 2058:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2285);
      END_STATE();
    case 2059:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2286);
      END_STATE();
    case 2060:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2287);
      END_STATE();
    case 2061:
      ACCEPT_TOKEN(sym_define_event_keyword);
      END_STATE();
    case 2062:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2288);
      END_STATE();
    case 2063:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2289);
      END_STATE();
    case 2064:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2290);
      END_STATE();
    case 2065:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2291);
      END_STATE();
    case 2066:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2292);
      END_STATE();
    case 2067:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2293);
      END_STATE();
    case 2068:
      ACCEPT_TOKEN(sym_define_start_keyword);
      END_STATE();
    case 2069:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2294);
      END_STATE();
    case 2070:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2295);
      END_STATE();
    case 2071:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2296);
      END_STATE();
    case 2072:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2297);
      END_STATE();
    case 2073:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2298);
      END_STATE();
    case 2074:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2299);
      END_STATE();
    case 2075:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2300);
      END_STATE();
    case 2076:
      if (lookahead == '_') ADVANCE(2301);
      END_STATE();
    case 2077:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2302);
      END_STATE();
    case 2078:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2303);
      END_STATE();
    case 2079:
      if (lookahead == '_') ADVANCE(2304);
      END_STATE();
    case 2080:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2305);
      END_STATE();
    case 2081:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2306);
      END_STATE();
    case 2082:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2307);
      END_STATE();
    case 2083:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2308);
      END_STATE();
    case 2084:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2309);
      END_STATE();
    case 2085:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2310);
      END_STATE();
    case 2086:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2311);
      END_STATE();
    case 2087:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2312);
      END_STATE();
    case 2088:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2313);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2314);
      END_STATE();
    case 2089:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2315);
      END_STATE();
    case 2090:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2316);
      END_STATE();
    case 2091:
      if (lookahead == '_') ADVANCE(2317);
      END_STATE();
    case 2092:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2318);
      END_STATE();
    case 2093:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2319);
      END_STATE();
    case 2094:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2320);
      END_STATE();
    case 2095:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2321);
      END_STATE();
    case 2096:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2322);
      END_STATE();
    case 2097:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2323);
      END_STATE();
    case 2098:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2324);
      END_STATE();
    case 2099:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2325);
      END_STATE();
    case 2100:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2326);
      END_STATE();
    case 2101:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2327);
      END_STATE();
    case 2102:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2328);
      END_STATE();
    case 2103:
      ACCEPT_TOKEN(sym_get_dns_list);
      END_STATE();
    case 2104:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2329);
      END_STATE();
    case 2105:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2330);
      END_STATE();
    case 2106:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2331);
      END_STATE();
    case 2107:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2332);
      END_STATE();
    case 2108:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2333);
      END_STATE();
    case 2109:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2334);
      END_STATE();
    case 2110:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2335);
      END_STATE();
    case 2111:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2336);
      END_STATE();
    case 2112:
      ACCEPT_TOKEN(sym_get_url_list);
      END_STATE();
    case 2113:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2337);
      END_STATE();
    case 2114:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2338);
      END_STATE();
    case 2115:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2339);
      END_STATE();
    case 2116:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2340);
      END_STATE();
    case 2117:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2341);
      END_STATE();
    case 2118:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2342);
      END_STATE();
    case 2119:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2343);
      END_STATE();
    case 2120:
      if (lookahead == '_') ADVANCE(2344);
      END_STATE();
    case 2121:
      if (lookahead == '_') ADVANCE(2345);
      END_STATE();
    case 2122:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2346);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2347);
      END_STATE();
    case 2123:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2348);
      END_STATE();
    case 2124:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2349);
      END_STATE();
    case 2125:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2350);
      END_STATE();
    case 2126:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2351);
      END_STATE();
    case 2127:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2352);
      END_STATE();
    case 2128:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2353);
      END_STATE();
    case 2129:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2354);
      END_STATE();
    case 2130:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2355);
      END_STATE();
    case 2131:
      ACCEPT_TOKEN(sym_length_array);
      END_STATE();
    case 2132:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2356);
      END_STATE();
    case 2133:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2357);
      END_STATE();
    case 2134:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2358);
      END_STATE();
    case 2135:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2359);
      END_STATE();
    case 2136:
      ACCEPT_TOKEN(sym_lower_string);
      END_STATE();
    case 2137:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2360);
      END_STATE();
    case 2138:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2361);
      END_STATE();
    case 2139:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2362);
      END_STATE();
    case 2140:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2363);
      END_STATE();
    case 2141:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2364);
      END_STATE();
    case 2142:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2365);
      END_STATE();
    case 2143:
      ACCEPT_TOKEN(sym_non_volatile_keyword);
      END_STATE();
    case 2144:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2366);
      END_STATE();
    case 2145:
      ACCEPT_TOKEN(sym_program_name_keyword);
      END_STATE();
    case 2146:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2367);
      END_STATE();
    case 2147:
      ACCEPT_TOKEN(sym_push_channel);
      END_STATE();
    case 2148:
      ACCEPT_TOKEN(sym_push_devchan);
      END_STATE();
    case 2149:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2368);
      END_STATE();
    case 2150:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2369);
      END_STATE();
    case 2151:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2370);
      END_STATE();
    case 2152:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2371);
      END_STATE();
    case 2153:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2372);
      END_STATE();
    case 2154:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2373);
      END_STATE();
    case 2155:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2374);
      END_STATE();
    case 2156:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2375);
      END_STATE();
    case 2157:
      ACCEPT_TOKEN(sym_right_string);
      END_STATE();
    case 2158:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2376);
      END_STATE();
    case 2159:
      ACCEPT_TOKEN(sym_send_command_keyword);
      END_STATE();
    case 2160:
      ACCEPT_TOKEN(sym_set_dns_list);
      END_STATE();
    case 2161:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2377);
      END_STATE();
    case 2162:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2378);
      END_STATE();
    case 2163:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2379);
      END_STATE();
    case 2164:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2380);
      END_STATE();
    case 2165:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2381);
      END_STATE();
    case 2166:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2382);
      END_STATE();
    case 2167:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2383);
      END_STATE();
    case 2168:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2384);
      END_STATE();
    case 2169:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2385);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2386);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2387);
      END_STATE();
    case 2170:
      ACCEPT_TOKEN(sym_smtp_address);
      END_STATE();
    case 2171:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2388);
      END_STATE();
    case 2172:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2389);
      END_STATE();
    case 2173:
      if (lookahead == '_') ADVANCE(2390);
      END_STATE();
    case 2174:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2391);
      END_STATE();
    case 2175:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2392);
      END_STATE();
    case 2176:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2393);
      END_STATE();
    case 2177:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2394);
      END_STATE();
    case 2178:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2395);
      END_STATE();
    case 2179:
      ADVANCE_MAP(
        'I', 2396,
        'i', 2396,
        'M', 2397,
        'm', 2397,
        'N', 2398,
        'n', 2398,
        'R', 2399,
        'r', 2399,
      );
      END_STATE();
    case 2180:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2400);
      END_STATE();
    case 2181:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2401);
      END_STATE();
    case 2182:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2402);
      END_STATE();
    case 2183:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2403);
      END_STATE();
    case 2184:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2404);
      END_STATE();
    case 2185:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2405);
      END_STATE();
    case 2186:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2406);
      END_STATE();
    case 2187:
      ACCEPT_TOKEN(sym_time_to_hour);
      END_STATE();
    case 2188:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2407);
      END_STATE();
    case 2189:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2408);
      END_STATE();
    case 2190:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2409);
      END_STATE();
    case 2191:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2410);
      END_STATE();
    case 2192:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2411);
      END_STATE();
    case 2193:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2412);
      END_STATE();
    case 2194:
      ACCEPT_TOKEN(sym_timeline_get);
      END_STATE();
    case 2195:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2413);
      END_STATE();
    case 2196:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2414);
      END_STATE();
    case 2197:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2415);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2416);
      END_STATE();
    case 2198:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2417);
      END_STATE();
    case 2199:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2418);
      END_STATE();
    case 2200:
      ACCEPT_TOKEN(sym_timeline_set);
      END_STATE();
    case 2201:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2419);
      END_STATE();
    case 2202:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2420);
      END_STATE();
    case 2203:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2421);
      END_STATE();
    case 2204:
      if (lookahead == '_') ADVANCE(2422);
      END_STATE();
    case 2205:
      ACCEPT_TOKEN(sym_tpadmin_priv);
      END_STATE();
    case 2206:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2423);
      END_STATE();
    case 2207:
      ACCEPT_TOKEN(sym_upper_string);
      END_STATE();
    case 2208:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2424);
      END_STATE();
    case 2209:
      if (lookahead == '_') ADVANCE(2425);
      END_STATE();
    case 2210:
      ACCEPT_TOKEN(sym_url_flg_temp);
      END_STATE();
    case 2211:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2426);
      END_STATE();
    case 2212:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2427);
      END_STATE();
    case 2213:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2428);
      END_STATE();
    case 2214:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2429);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(2430);
      END_STATE();
    case 2215:
      ACCEPT_TOKEN(sym_variantarray_keyword);
      END_STATE();
    case 2216:
      ACCEPT_TOKEN(sym_wc_data_feed);
      END_STATE();
    case 2217:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2431);
      END_STATE();
    case 2218:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2432);
      END_STATE();
    case 2219:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2433);
      END_STATE();
    case 2220:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2434);
      END_STATE();
    case 2221:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2435);
      END_STATE();
    case 2222:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2436);
      END_STATE();
    case 2223:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2437);
      END_STATE();
    case 2224:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2438);
      END_STATE();
    case 2225:
      if (lookahead == '_') ADVANCE(2439);
      END_STATE();
    case 2226:
      ACCEPT_TOKEN(sym__ch_to_wc_int);
      END_STATE();
    case 2227:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2440);
      END_STATE();
    case 2228:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2441);
      END_STATE();
    case 2229:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2442);
      END_STATE();
    case 2230:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2443);
      END_STATE();
    case 2231:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2444);
      END_STATE();
    case 2232:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2445);
      END_STATE();
    case 2233:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2446);
      END_STATE();
    case 2234:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2447);
      END_STATE();
    case 2235:
      if (lookahead == '_') ADVANCE(2448);
      END_STATE();
    case 2236:
      if (lookahead == '_') ADVANCE(2449);
      END_STATE();
    case 2237:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2450);
      END_STATE();
    case 2238:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2451);
      END_STATE();
    case 2239:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2452);
      END_STATE();
    case 2240:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2453);
      END_STATE();
    case 2241:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2454);
      END_STATE();
    case 2242:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2455);
      END_STATE();
    case 2243:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2456);
      END_STATE();
    case 2244:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2457);
      END_STATE();
    case 2245:
      ACCEPT_TOKEN(sym__wc_to_ch_int);
      END_STATE();
    case 2246:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2458);
      END_STATE();
    case 2247:
      ACCEPT_TOKEN(sym_add_url_entry);
      END_STATE();
    case 2248:
      if (lookahead == '_') ADVANCE(2459);
      END_STATE();
    case 2249:
      ACCEPT_TOKEN(sym_auditlog_priv);
      END_STATE();
    case 2250:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2460);
      END_STATE();
    case 2251:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2461);
      END_STATE();
    case 2252:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2462);
      END_STATE();
    case 2253:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2463);
      END_STATE();
    case 2254:
      if (lookahead == '_') ADVANCE(2464);
      END_STATE();
    case 2255:
      ACCEPT_TOKEN(sym_create_buffer_keyword);
      END_STATE();
    case 2256:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2465);
      END_STATE();
    case 2257:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2466);
      END_STATE();
    case 2258:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2467);
      END_STATE();
    case 2259:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2468);
      END_STATE();
    case 2260:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2469);
      END_STATE();
    case 2261:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2470);
      END_STATE();
    case 2262:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2471);
      END_STATE();
    case 2263:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2472);
      END_STATE();
    case 2264:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2473);
      END_STATE();
    case 2265:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2474);
      END_STATE();
    case 2266:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2475);
      END_STATE();
    case 2267:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2476);
      END_STATE();
    case 2268:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2477);
      END_STATE();
    case 2269:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2478);
      END_STATE();
    case 2270:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2479);
      END_STATE();
    case 2271:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2480);
      END_STATE();
    case 2272:
      if (lookahead == '_') ADVANCE(2481);
      END_STATE();
    case 2273:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2482);
      END_STATE();
    case 2274:
      if (lookahead == '_') ADVANCE(2483);
      END_STATE();
    case 2275:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2484);
      END_STATE();
    case 2276:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2485);
      END_STATE();
    case 2277:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2486);
      END_STATE();
    case 2278:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2487);
      END_STATE();
    case 2279:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2488);
      END_STATE();
    case 2280:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2489);
      END_STATE();
    case 2281:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2490);
      END_STATE();
    case 2282:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2491);
      END_STATE();
    case 2283:
      ACCEPT_TOKEN(sym_date_to_month);
      END_STATE();
    case 2284:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2492);
      END_STATE();
    case 2285:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2493);
      END_STATE();
    case 2286:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2494);
      END_STATE();
    case 2287:
      ACCEPT_TOKEN(sym_define_device_keyword);
      END_STATE();
    case 2288:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2495);
      END_STATE();
    case 2289:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2496);
      END_STATE();
    case 2290:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2497);
      END_STATE();
    case 2291:
      ACCEPT_TOKEN(sym_define_module_keyword);
      END_STATE();
    case 2292:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2498);
      END_STATE();
    case 2293:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2499);
      END_STATE();
    case 2294:
      if (lookahead == '_') ADVANCE(2500);
      END_STATE();
    case 2295:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2501);
      END_STATE();
    case 2296:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2502);
      END_STATE();
    case 2297:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2503);
      END_STATE();
    case 2298:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2504);
      END_STATE();
    case 2299:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2505);
      END_STATE();
    case 2300:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2506);
      END_STATE();
    case 2301:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2507);
      END_STATE();
    case 2302:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2508);
      END_STATE();
    case 2303:
      ACCEPT_TOKEN(sym_do_push_timed);
      if (lookahead == '_') ADVANCE(2509);
      END_STATE();
    case 2304:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2510);
      END_STATE();
    case 2305:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2511);
      END_STATE();
    case 2306:
      if (lookahead == '_') ADVANCE(2512);
      END_STATE();
    case 2307:
      if (lookahead == '_') ADVANCE(2513);
      END_STATE();
    case 2308:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2514);
      END_STATE();
    case 2309:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2515);
      END_STATE();
    case 2310:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2516);
      END_STATE();
    case 2311:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2517);
      END_STATE();
    case 2312:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2518);
      END_STATE();
    case 2313:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2519);
      END_STATE();
    case 2314:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2520);
      END_STATE();
    case 2315:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2521);
      END_STATE();
    case 2316:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2522);
      END_STATE();
    case 2317:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2523);
      END_STATE();
    case 2318:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2524);
      END_STATE();
    case 2319:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2525);
      END_STATE();
    case 2320:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2526);
      END_STATE();
    case 2321:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2527);
      END_STATE();
    case 2322:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2528);
      END_STATE();
    case 2323:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2529);
      END_STATE();
    case 2324:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2530);
      END_STATE();
    case 2325:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2531);
      END_STATE();
    case 2326:
      if (lookahead == '_') ADVANCE(2532);
      END_STATE();
    case 2327:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2533);
      END_STATE();
    case 2328:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2534);
      END_STATE();
    case 2329:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2535);
      END_STATE();
    case 2330:
      ACCEPT_TOKEN(sym_get_log_level);
      END_STATE();
    case 2331:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2536);
      END_STATE();
    case 2332:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2537);
      END_STATE();
    case 2333:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2538);
      END_STATE();
    case 2334:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2539);
      END_STATE();
    case 2335:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2540);
      END_STATE();
    case 2336:
      ACCEPT_TOKEN(sym_get_unique_id);
      END_STATE();
    case 2337:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2541);
      END_STATE();
    case 2338:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2542);
      END_STATE();
    case 2339:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2543);
      END_STATE();
    case 2340:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2544);
      END_STATE();
    case 2341:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2545);
      END_STATE();
    case 2342:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2546);
      END_STATE();
    case 2343:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2547);
      END_STATE();
    case 2344:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2548);
      END_STATE();
    case 2345:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2549);
      END_STATE();
    case 2346:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2550);
      END_STATE();
    case 2347:
      ACCEPT_TOKEN(sym_ip_nodelay_on);
      END_STATE();
    case 2348:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2551);
      END_STATE();
    case 2349:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2552);
      END_STATE();
    case 2350:
      ACCEPT_TOKEN(sym_ip_set_option);
      END_STATE();
    case 2351:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2553);
      END_STATE();
    case 2352:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2554);
      END_STATE();
    case 2353:
      ACCEPT_TOKEN(sym_ip_ttl_region);
      END_STATE();
    case 2354:
      ACCEPT_TOKEN(sym_ip_ttl_subnet);
      END_STATE();
    case 2355:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2555);
      END_STATE();
    case 2356:
      ACCEPT_TOKEN(sym_length_string);
      END_STATE();
    case 2357:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2556);
      END_STATE();
    case 2358:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2557);
      END_STATE();
    case 2359:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2558);
      END_STATE();
    case 2360:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2559);
      END_STATE();
    case 2361:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2560);
      END_STATE();
    case 2362:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2561);
      END_STATE();
    case 2363:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2562);
      END_STATE();
    case 2364:
      if (lookahead == '_') ADVANCE(2563);
      END_STATE();
    case 2365:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2564);
      END_STATE();
    case 2366:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2565);
      END_STATE();
    case 2367:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2566);
      END_STATE();
    case 2368:
      ACCEPT_TOKEN(sym_random_number);
      END_STATE();
    case 2369:
      ACCEPT_TOKEN(sym_rebuilt_event);
      END_STATE();
    case 2370:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2567);
      END_STATE();
    case 2371:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2568);
      END_STATE();
    case 2372:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2569);
      END_STATE();
    case 2373:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2570);
      END_STATE();
    case 2374:
      ACCEPT_TOKEN(sym_remoteui_priv);
      END_STATE();
    case 2375:
      ACCEPT_TOKEN(sym_remove_string);
      END_STATE();
    case 2376:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2571);
      END_STATE();
    case 2377:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2572);
      END_STATE();
    case 2378:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2573);
      END_STATE();
    case 2379:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2574);
      END_STATE();
    case 2380:
      ACCEPT_TOKEN(sym_set_log_level);
      END_STATE();
    case 2381:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2575);
      END_STATE();
    case 2382:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2576);
      END_STATE();
    case 2383:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2577);
      END_STATE();
    case 2384:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2578);
      END_STATE();
    case 2385:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2579);
      END_STATE();
    case 2386:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2580);
      END_STATE();
    case 2387:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2581);
      END_STATE();
    case 2388:
      ACCEPT_TOKEN(sym_smtp_password);
      END_STATE();
    case 2389:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2582);
      END_STATE();
    case 2390:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2583);
      END_STATE();
    case 2391:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2584);
      END_STATE();
    case 2392:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2585);
      END_STATE();
    case 2393:
      ACCEPT_TOKEN(sym_smtp_tls_true);
      END_STATE();
    case 2394:
      ACCEPT_TOKEN(sym_smtp_username);
      END_STATE();
    case 2395:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2586);
      END_STATE();
    case 2396:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2587);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2588);
      END_STATE();
    case 2397:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2589);
      END_STATE();
    case 2398:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2590);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2591);
      END_STATE();
    case 2399:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2592);
      END_STATE();
    case 2400:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2593);
      END_STATE();
    case 2401:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2594);
      END_STATE();
    case 2402:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2595);
      END_STATE();
    case 2403:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2596);
      END_STATE();
    case 2404:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2597);
      END_STATE();
    case 2405:
      ACCEPT_TOKEN(sym_system_number);
      END_STATE();
    case 2406:
      ACCEPT_TOKEN(sym_terminal_priv);
      END_STATE();
    case 2407:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2598);
      END_STATE();
    case 2408:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2599);
      END_STATE();
    case 2409:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2600);
      END_STATE();
    case 2410:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2601);
      END_STATE();
    case 2411:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2602);
      END_STATE();
    case 2412:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2603);
      END_STATE();
    case 2413:
      ACCEPT_TOKEN(sym_timeline_once);
      END_STATE();
    case 2414:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2604);
      END_STATE();
    case 2415:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2605);
      END_STATE();
    case 2416:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2606);
      END_STATE();
    case 2417:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2607);
      END_STATE();
    case 2418:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2608);
      END_STATE();
    case 2419:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2609);
      END_STATE();
    case 2420:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2610);
      END_STATE();
    case 2421:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2611);
      END_STATE();
    case 2422:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2612);
      END_STATE();
    case 2423:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2613);
      END_STATE();
    case 2424:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2614);
      END_STATE();
    case 2425:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2615);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2616);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(2617);
      END_STATE();
    case 2426:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2618);
      END_STATE();
    case 2427:
      ACCEPT_TOKEN(sym_valid_account);
      END_STATE();
    case 2428:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2619);
      END_STATE();
    case 2429:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2620);
      END_STATE();
    case 2430:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2621);
      END_STATE();
    case 2431:
      ACCEPT_TOKEN(sym_wc_data_field);
      END_STATE();
    case 2432:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2622);
      END_STATE();
    case 2433:
      if (lookahead == '_') ADVANCE(2623);
      END_STATE();
    case 2434:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2624);
      END_STATE();
    case 2435:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2625);
      END_STATE();
    case 2436:
      ACCEPT_TOKEN(sym_xml_encode_le);
      END_STATE();
    case 2437:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2626);
      END_STATE();
    case 2438:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2627);
      END_STATE();
    case 2439:
      ACCEPT_TOKEN(sym____reserved___);
      END_STATE();
    case 2440:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2628);
      END_STATE();
    case 2441:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2629);
      END_STATE();
    case 2442:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2630);
      END_STATE();
    case 2443:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2631);
      END_STATE();
    case 2444:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2632);
      END_STATE();
    case 2445:
      ACCEPT_TOKEN(sym__wc_decode_int);
      END_STATE();
    case 2446:
      ACCEPT_TOKEN(sym__wc_encode_int);
      END_STATE();
    case 2447:
      if (lookahead == '_') ADVANCE(2633);
      END_STATE();
    case 2448:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2634);
      END_STATE();
    case 2449:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2635);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2636);
      END_STATE();
    case 2450:
      if (lookahead == '_') ADVANCE(2637);
      END_STATE();
    case 2451:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2638);
      END_STATE();
    case 2452:
      if (lookahead == '_') ADVANCE(2639);
      END_STATE();
    case 2453:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2640);
      END_STATE();
    case 2454:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2641);
      END_STATE();
    case 2455:
      if (lookahead == '_') ADVANCE(2642);
      END_STATE();
    case 2456:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2643);
      END_STATE();
    case 2457:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2644);
      END_STATE();
    case 2458:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2645);
      END_STATE();
    case 2459:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2646);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2647);
      END_STATE();
    case 2460:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2648);
      END_STATE();
    case 2461:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2649);
      END_STATE();
    case 2462:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2650);
      END_STATE();
    case 2463:
      ACCEPT_TOKEN(sym_compare_string);
      END_STATE();
    case 2464:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2651);
      END_STATE();
    case 2465:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2652);
      END_STATE();
    case 2466:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2653);
      END_STATE();
    case 2467:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2654);
      END_STATE();
    case 2468:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2655);
      END_STATE();
    case 2469:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2656);
      END_STATE();
    case 2470:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2657);
      END_STATE();
    case 2471:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2658);
      END_STATE();
    case 2472:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2659);
      END_STATE();
    case 2473:
      if (lookahead == '_') ADVANCE(2660);
      END_STATE();
    case 2474:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2661);
      END_STATE();
    case 2475:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2662);
      END_STATE();
    case 2476:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2663);
      END_STATE();
    case 2477:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2664);
      END_STATE();
    case 2478:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2665);
      END_STATE();
    case 2479:
      if (lookahead == '_') ADVANCE(2666);
      END_STATE();
    case 2480:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2667);
      END_STATE();
    case 2481:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2668);
      END_STATE();
    case 2482:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2669);
      END_STATE();
    case 2483:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2670);
      END_STATE();
    case 2484:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2671);
      END_STATE();
    case 2485:
      if (lookahead == '_') ADVANCE(2672);
      END_STATE();
    case 2486:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2673);
      END_STATE();
    case 2487:
      if (lookahead == '_') ADVANCE(2674);
      END_STATE();
    case 2488:
      ACCEPT_TOKEN(sym_data_type_date);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2675);
      END_STATE();
    case 2489:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2676);
      END_STATE();
    case 2490:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2677);
      END_STATE();
    case 2491:
      ACCEPT_TOKEN(sym_data_type_time);
      END_STATE();
    case 2492:
      ACCEPT_TOKEN(sym_define_combine_keyword);
      END_STATE();
    case 2493:
      if (lookahead == '_') ADVANCE(2678);
      END_STATE();
    case 2494:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2679);
      END_STATE();
    case 2495:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2680);
      END_STATE();
    case 2496:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2681);
      END_STATE();
    case 2497:
      if (lookahead == '_') ADVANCE(2682);
      END_STATE();
    case 2498:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2683);
      END_STATE();
    case 2499:
      ACCEPT_TOKEN(sym_define_program_keyword);
      END_STATE();
    case 2500:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2684);
      END_STATE();
    case 2501:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2685);
      END_STATE();
    case 2502:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2686);
      END_STATE();
    case 2503:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2687);
      END_STATE();
    case 2504:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2688);
      END_STATE();
    case 2505:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2689);
      END_STATE();
    case 2506:
      ACCEPT_TOKEN(sym_device_standby);
      END_STATE();
    case 2507:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2690);
      END_STATE();
    case 2508:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2691);
      END_STATE();
    case 2509:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2692);
      END_STATE();
    case 2510:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2693);
      END_STATE();
    case 2511:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2694);
      END_STATE();
    case 2512:
      ADVANCE_MAP(
        'A', 2695,
        'a', 2695,
        'C', 2696,
        'c', 2696,
        'D', 2697,
        'd', 2697,
        'H', 2698,
        'h', 2698,
        'I', 2699,
        'i', 2699,
        'L', 2700,
        'l', 2700,
        'M', 2701,
        'm', 2701,
        'P', 2702,
        'p', 2702,
        'R', 2703,
        'r', 2703,
        'S', 2704,
        's', 2704,
        'T', 2705,
        't', 2705,
        'U', 2706,
        'u', 2706,
        'V', 2707,
        'v', 2707,
        'W', 2708,
        'w', 2708,
      );
      END_STATE();
    case 2513:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2709);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2710);
      END_STATE();
    case 2514:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2711);
      END_STATE();
    case 2515:
      if (lookahead == '_') ADVANCE(2712);
      END_STATE();
    case 2516:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2713);
      END_STATE();
    case 2517:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2714);
      END_STATE();
    case 2518:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2715);
      END_STATE();
    case 2519:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2716);
      END_STATE();
    case 2520:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2717);
      END_STATE();
    case 2521:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2718);
      END_STATE();
    case 2522:
      if (lookahead == 'Z' ||
          lookahead == 'z') ADVANCE(2719);
      END_STATE();
    case 2523:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2720);
      END_STATE();
    case 2524:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2721);
      END_STATE();
    case 2525:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2722);
      END_STATE();
    case 2526:
      ACCEPT_TOKEN(sym_file_read_line);
      END_STATE();
    case 2527:
      ACCEPT_TOKEN(sym_file_read_only);
      END_STATE();
    case 2528:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2723);
      END_STATE();
    case 2529:
      ACCEPT_TOKEN(sym_file_rw_append);
      END_STATE();
    case 2530:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2724);
      END_STATE();
    case 2531:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2725);
      END_STATE();
    case 2532:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2726);
      END_STATE();
    case 2533:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2727);
      END_STATE();
    case 2534:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2728);
      END_STATE();
    case 2535:
      ACCEPT_TOKEN(sym_get_ip_address);
      END_STATE();
    case 2536:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2729);
      END_STATE();
    case 2537:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2730);
      END_STATE();
    case 2538:
      ACCEPT_TOKEN(sym_get_pulse_time);
      END_STATE();
    case 2539:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2731);
      END_STATE();
    case 2540:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2732);
      END_STATE();
    case 2541:
      if (lookahead == '_') ADVANCE(2733);
      END_STATE();
    case 2542:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2734);
      END_STATE();
    case 2543:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2735);
      END_STATE();
    case 2544:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2736);
      END_STATE();
    case 2545:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2737);
      END_STATE();
    case 2546:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2738);
      END_STATE();
    case 2547:
      ACCEPT_TOKEN(sym_ip_client_open);
      END_STATE();
    case 2548:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2739);
      END_STATE();
    case 2549:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2740);
      END_STATE();
    case 2550:
      ACCEPT_TOKEN(sym_ip_nodelay_off);
      END_STATE();
    case 2551:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2741);
      END_STATE();
    case 2552:
      ACCEPT_TOKEN(sym_ip_server_open);
      END_STATE();
    case 2553:
      if (lookahead == '_') ADVANCE(2742);
      END_STATE();
    case 2554:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2743);
      END_STATE();
    case 2555:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2744);
      END_STATE();
    case 2556:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2745);
      END_STATE();
    case 2557:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(2746);
      END_STATE();
    case 2558:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2747);
      END_STATE();
    case 2559:
      if (lookahead == '_') ADVANCE(2748);
      END_STATE();
    case 2560:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2749);
      END_STATE();
    case 2561:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2750);
      END_STATE();
    case 2562:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2751);
      END_STATE();
    case 2563:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2752);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2753);
      END_STATE();
    case 2564:
      ACCEPT_TOKEN(sym_netlinx_logout);
      END_STATE();
    case 2565:
      ACCEPT_TOKEN(sym_normal_standby);
      END_STATE();
    case 2566:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2754);
      END_STATE();
    case 2567:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2755);
      END_STATE();
    case 2568:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2756);
      END_STATE();
    case 2569:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2757);
      END_STATE();
    case 2570:
      ACCEPT_TOKEN(sym_release_device);
      END_STATE();
    case 2571:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2758);
      END_STATE();
    case 2572:
      ACCEPT_TOKEN(sym_set_ip_address);
      END_STATE();
    case 2573:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2759);
      END_STATE();
    case 2574:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2760);
      END_STATE();
    case 2575:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2761);
      END_STATE();
    case 2576:
      ACCEPT_TOKEN(sym_set_pulse_time);
      END_STATE();
    case 2577:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2762);
      END_STATE();
    case 2578:
      if (lookahead == '_') ADVANCE(2763);
      END_STATE();
    case 2579:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2764);
      END_STATE();
    case 2580:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2765);
      END_STATE();
    case 2581:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2766);
      END_STATE();
    case 2582:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2767);
      END_STATE();
    case 2583:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2768);
      END_STATE();
    case 2584:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2769);
      END_STATE();
    case 2585:
      ACCEPT_TOKEN(sym_smtp_tls_false);
      END_STATE();
    case 2586:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2770);
      END_STATE();
    case 2587:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2771);
      END_STATE();
    case 2588:
      if (lookahead == '_') ADVANCE(2772);
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2773);
      END_STATE();
    case 2589:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2774);
      END_STATE();
    case 2590:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2775);
      END_STATE();
    case 2591:
      if (lookahead == '_') ADVANCE(2776);
      END_STATE();
    case 2592:
      if (lookahead == '2') ADVANCE(2777);
      END_STATE();
    case 2593:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2778);
      END_STATE();
    case 2594:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2779);
      END_STATE();
    case 2595:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2780);
      END_STATE();
    case 2596:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2781);
      END_STATE();
    case 2597:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2782);
      END_STATE();
    case 2598:
      ACCEPT_TOKEN(sym_time_to_minute);
      END_STATE();
    case 2599:
      ACCEPT_TOKEN(sym_time_to_second);
      END_STATE();
    case 2600:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2783);
      END_STATE();
    case 2601:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2784);
      END_STATE();
    case 2602:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2785);
      END_STATE();
    case 2603:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2786);
      END_STATE();
    case 2604:
      ACCEPT_TOKEN(sym_timeline_pause);
      END_STATE();
    case 2605:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2787);
      END_STATE();
    case 2606:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2788);
      END_STATE();
    case 2607:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2789);
      END_STATE();
    case 2608:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2790);
      END_STATE();
    case 2609:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2791);
      END_STATE();
    case 2610:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2792);
      END_STATE();
    case 2611:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2793);
      END_STATE();
    case 2612:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2794);
      END_STATE();
    case 2613:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2795);
      END_STATE();
    case 2614:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2796);
      END_STATE();
    case 2615:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2797);
      END_STATE();
    case 2616:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2798);
      END_STATE();
    case 2617:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2799);
      END_STATE();
    case 2618:
      if (lookahead == '_') ADVANCE(2800);
      END_STATE();
    case 2619:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2801);
      END_STATE();
    case 2620:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2802);
      END_STATE();
    case 2621:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2803);
      END_STATE();
    case 2622:
      ACCEPT_TOKEN(sym_wc_data_record);
      END_STATE();
    case 2623:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2804);
      END_STATE();
    case 2624:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2805);
      END_STATE();
    case 2625:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2806);
      END_STATE();
    case 2626:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2807);
      END_STATE();
    case 2627:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2808);
      END_STATE();
    case 2628:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2809);
      END_STATE();
    case 2629:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2810);
      END_STATE();
    case 2630:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2811);
      END_STATE();
    case 2631:
      if (lookahead == '_') ADVANCE(2812);
      END_STATE();
    case 2632:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2813);
      END_STATE();
    case 2633:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2814);
      END_STATE();
    case 2634:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2815);
      END_STATE();
    case 2635:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2816);
      END_STATE();
    case 2636:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2817);
      END_STATE();
    case 2637:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2818);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2819);
      END_STATE();
    case 2638:
      if (lookahead == '_') ADVANCE(2820);
      END_STATE();
    case 2639:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2821);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2822);
      END_STATE();
    case 2640:
      if (lookahead == '_') ADVANCE(2823);
      END_STATE();
    case 2641:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2824);
      END_STATE();
    case 2642:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2825);
      END_STATE();
    case 2643:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2826);
      END_STATE();
    case 2644:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2827);
      END_STATE();
    case 2645:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2828);
      END_STATE();
    case 2646:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2829);
      END_STATE();
    case 2647:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2830);
      END_STATE();
    case 2648:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2831);
      END_STATE();
    case 2649:
      ACCEPT_TOKEN(sym_cancel_all_wait_keyword);
      if (lookahead == '_') ADVANCE(2832);
      END_STATE();
    case 2650:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2833);
      END_STATE();
    case 2651:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2834);
      END_STATE();
    case 2652:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2835);
      END_STATE();
    case 2653:
      ACCEPT_TOKEN(sym_data_add_record);
      END_STATE();
    case 2654:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2836);
      END_STATE();
    case 2655:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2837);
      END_STATE();
    case 2656:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2838);
      END_STATE();
    case 2657:
      if (lookahead == '8') ADVANCE(2839);
      END_STATE();
    case 2658:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2840);
      END_STATE();
    case 2659:
      ACCEPT_TOKEN(sym_data_format_url);
      END_STATE();
    case 2660:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2841);
      END_STATE();
    case 2661:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2842);
      END_STATE();
    case 2662:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2843);
      END_STATE();
    case 2663:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2844);
      END_STATE();
    case 2664:
      if (lookahead == '_') ADVANCE(2845);
      END_STATE();
    case 2665:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2846);
      END_STATE();
    case 2666:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2847);
      END_STATE();
    case 2667:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2848);
      END_STATE();
    case 2668:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2849);
      END_STATE();
    case 2669:
      if (lookahead == '_') ADVANCE(2850);
      END_STATE();
    case 2670:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2851);
      END_STATE();
    case 2671:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2852);
      END_STATE();
    case 2672:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2853);
      END_STATE();
    case 2673:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2854);
      END_STATE();
    case 2674:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2855);
      END_STATE();
    case 2675:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2856);
      END_STATE();
    case 2676:
      ACCEPT_TOKEN(sym_data_type_image);
      END_STATE();
    case 2677:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2857);
      END_STATE();
    case 2678:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2858);
      END_STATE();
    case 2679:
      ACCEPT_TOKEN(sym_define_constant_keyword);
      END_STATE();
    case 2680:
      ACCEPT_TOKEN(sym_define_function_keyword);
      END_STATE();
    case 2681:
      ACCEPT_TOKEN(sym_define_latching_keyword);
      END_STATE();
    case 2682:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2859);
      END_STATE();
    case 2683:
      if (lookahead == '_') ADVANCE(2860);
      END_STATE();
    case 2684:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2861);
      END_STATE();
    case 2685:
      ACCEPT_TOKEN(sym_define_toggling_keyword);
      END_STATE();
    case 2686:
      ACCEPT_TOKEN(sym_define_variable_keyword);
      END_STATE();
    case 2687:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2862);
      END_STATE();
    case 2688:
      ACCEPT_TOKEN(sym_dev_info_struct);
      END_STATE();
    case 2689:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2863);
      END_STATE();
    case 2690:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2864);
      END_STATE();
    case 2691:
      ACCEPT_TOKEN(sym_do_custom_event);
      END_STATE();
    case 2692:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2865);
      END_STATE();
    case 2693:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2866);
      END_STATE();
    case 2694:
      ACCEPT_TOKEN(sym_duet_dev_polled);
      END_STATE();
    case 2695:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(2867);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2868);
      END_STATE();
    case 2696:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2869);
      END_STATE();
    case 2697:
      ADVANCE_MAP(
        'I', 2870,
        'i', 2870,
        'O', 2871,
        'o', 2871,
        'S', 2872,
        's', 2872,
        'V', 2873,
        'v', 2873,
      );
      END_STATE();
    case 2698:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2874);
      END_STATE();
    case 2699:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2875);
      END_STATE();
    case 2700:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2876);
      END_STATE();
    case 2701:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2877);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2878);
      END_STATE();
    case 2702:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2879);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2880);
      END_STATE();
    case 2703:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2881);
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2882);
      END_STATE();
    case 2704:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2883);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2884);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(2885);
      END_STATE();
    case 2705:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2886);
      END_STATE();
    case 2706:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2887);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2888);
      END_STATE();
    case 2707:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2889);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2890);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2891);
      END_STATE();
    case 2708:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2892);
      END_STATE();
    case 2709:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2893);
      END_STATE();
    case 2710:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2894);
      END_STATE();
    case 2711:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2895);
      END_STATE();
    case 2712:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2896);
      END_STATE();
    case 2713:
      if (lookahead == '_') ADVANCE(2897);
      END_STATE();
    case 2714:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2898);
      END_STATE();
    case 2715:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2899);
      END_STATE();
    case 2716:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2900);
      END_STATE();
    case 2717:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2901);
      END_STATE();
    case 2718:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2902);
      END_STATE();
    case 2719:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2903);
      END_STATE();
    case 2720:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2904);
      END_STATE();
    case 2721:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2905);
      END_STATE();
    case 2722:
      ACCEPT_TOKEN(sym_file_create_dir);
      END_STATE();
    case 2723:
      ACCEPT_TOKEN(sym_file_remove_dir);
      END_STATE();
    case 2724:
      ACCEPT_TOKEN(sym_file_write_line);
      END_STATE();
    case 2725:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2906);
      END_STATE();
    case 2726:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2907);
      END_STATE();
    case 2727:
      ACCEPT_TOKEN(sym_get_buffer_char);
      END_STATE();
    case 2728:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2908);
      END_STATE();
    case 2729:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2909);
      END_STATE();
    case 2730:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2910);
      END_STATE();
    case 2731:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2911);
      END_STATE();
    case 2732:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2912);
      END_STATE();
    case 2733:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2913);
      END_STATE();
    case 2734:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2914);
      END_STATE();
    case 2735:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2915);
      END_STATE();
    case 2736:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2916);
      END_STATE();
    case 2737:
      if (lookahead == '_') ADVANCE(2917);
      END_STATE();
    case 2738:
      ACCEPT_TOKEN(sym_ip_client_close);
      END_STATE();
    case 2739:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2918);
      END_STATE();
    case 2740:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2919);
      END_STATE();
    case 2741:
      ACCEPT_TOKEN(sym_ip_server_close);
      END_STATE();
    case 2742:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2920);
      END_STATE();
    case 2743:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2921);
      END_STATE();
    case 2744:
      ACCEPT_TOKEN(sym_last_login_info);
      END_STATE();
    case 2745:
      if (lookahead == '_') ADVANCE(2922);
      END_STATE();
    case 2746:
      if (lookahead == '_') ADVANCE(2923);
      END_STATE();
    case 2747:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2924);
      END_STATE();
    case 2748:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2925);
      END_STATE();
    case 2749:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2926);
      END_STATE();
    case 2750:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2927);
      END_STATE();
    case 2751:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2928);
      END_STATE();
    case 2752:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2929);
      END_STATE();
    case 2753:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2930);
      END_STATE();
    case 2754:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2931);
      END_STATE();
    case 2755:
      ACCEPT_TOKEN(sym_redirect_string);
      END_STATE();
    case 2756:
      ACCEPT_TOKEN(sym_release_channel);
      END_STATE();
    case 2757:
      ACCEPT_TOKEN(sym_release_devchan);
      END_STATE();
    case 2758:
      if (lookahead == '_') ADVANCE(2932);
      END_STATE();
    case 2759:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(2933);
      END_STATE();
    case 2760:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2934);
      END_STATE();
    case 2761:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2935);
      END_STATE();
    case 2762:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2936);
      END_STATE();
    case 2763:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2937);
      END_STATE();
    case 2764:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2938);
      END_STATE();
    case 2765:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2939);
      END_STATE();
    case 2766:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2940);
      END_STATE();
    case 2767:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2941);
      END_STATE();
    case 2768:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2942);
      END_STATE();
    case 2769:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2943);
      END_STATE();
    case 2770:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2944);
      END_STATE();
    case 2771:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2945);
      END_STATE();
    case 2772:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2946);
      END_STATE();
    case 2773:
      if (lookahead == '4') ADVANCE(2947);
      END_STATE();
    case 2774:
      if (lookahead == '_') ADVANCE(2948);
      END_STATE();
    case 2775:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2949);
      END_STATE();
    case 2776:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(2950);
      END_STATE();
    case 2777:
      if (lookahead == '3') ADVANCE(2951);
      END_STATE();
    case 2778:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2952);
      END_STATE();
    case 2779:
      ACCEPT_TOKEN(sym_ssh_client_open);
      END_STATE();
    case 2780:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2953);
      END_STATE();
    case 2781:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(2954);
      END_STATE();
    case 2782:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(2955);
      END_STATE();
    case 2783:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2956);
      END_STATE();
    case 2784:
      ACCEPT_TOKEN(sym_timeline_active);
      END_STATE();
    case 2785:
      ACCEPT_TOKEN(sym_timeline_create);
      END_STATE();
    case 2786:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(2957);
      END_STATE();
    case 2787:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(2958);
      END_STATE();
    case 2788:
      ACCEPT_TOKEN(sym_timeline_reload);
      END_STATE();
    case 2789:
      ACCEPT_TOKEN(sym_timeline_repeat);
      END_STATE();
    case 2790:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2959);
      END_STATE();
    case 2791:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(2960);
      END_STATE();
    case 2792:
      ACCEPT_TOKEN(sym_tls_client_open);
      END_STATE();
    case 2793:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2961);
      END_STATE();
    case 2794:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2962);
      END_STATE();
    case 2795:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2963);
      END_STATE();
    case 2796:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2964);
      END_STATE();
    case 2797:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2965);
      END_STATE();
    case 2798:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2966);
      END_STATE();
    case 2799:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2967);
      END_STATE();
    case 2800:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(2968);
      END_STATE();
    case 2801:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(2969);
      END_STATE();
    case 2802:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2970);
      END_STATE();
    case 2803:
      ACCEPT_TOKEN(sym_variable_to_xml);
      END_STATE();
    case 2804:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(2971);
      END_STATE();
    case 2805:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2972);
      END_STATE();
    case 2806:
      if (lookahead == '_') ADVANCE(2973);
      END_STATE();
    case 2807:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2974);
      END_STATE();
    case 2808:
      ACCEPT_TOKEN(sym_xml_to_variable);
      END_STATE();
    case 2809:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2975);
      END_STATE();
    case 2810:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2976);
      END_STATE();
    case 2811:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(2977);
      END_STATE();
    case 2812:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(2978);
      END_STATE();
    case 2813:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2979);
      END_STATE();
    case 2814:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2980);
      END_STATE();
    case 2815:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2981);
      END_STATE();
    case 2816:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2982);
      END_STATE();
    case 2817:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2983);
      END_STATE();
    case 2818:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2984);
      END_STATE();
    case 2819:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2985);
      END_STATE();
    case 2820:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2986);
      END_STATE();
    case 2821:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(2987);
      END_STATE();
    case 2822:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2988);
      END_STATE();
    case 2823:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(2989);
      END_STATE();
    case 2824:
      if (lookahead == '_') ADVANCE(2990);
      END_STATE();
    case 2825:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2991);
      END_STATE();
    case 2826:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(2992);
      END_STATE();
    case 2827:
      if (lookahead == '_') ADVANCE(2993);
      END_STATE();
    case 2828:
      if (lookahead == '_') ADVANCE(2994);
      END_STATE();
    case 2829:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(2995);
      END_STATE();
    case 2830:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(2996);
      END_STATE();
    case 2831:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(2997);
      END_STATE();
    case 2832:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(2998);
      END_STATE();
    case 2833:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(2999);
      END_STATE();
    case 2834:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3000);
      END_STATE();
    case 2835:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3001);
      END_STATE();
    case 2836:
      ACCEPT_TOKEN(sym_data_create_feed);
      END_STATE();
    case 2837:
      ACCEPT_TOKEN(sym_data_delete_feed);
      END_STATE();
    case 2838:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3002);
      END_STATE();
    case 2839:
      if (lookahead == '6') ADVANCE(3003);
      END_STATE();
    case 2840:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3004);
      END_STATE();
    case 2841:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3005);
      END_STATE();
    case 2842:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3006);
      END_STATE();
    case 2843:
      if (lookahead == '_') ADVANCE(3007);
      END_STATE();
    case 2844:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3008);
      END_STATE();
    case 2845:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3009);
      END_STATE();
    case 2846:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3010);
      END_STATE();
    case 2847:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3011);
      END_STATE();
    case 2848:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3012);
      END_STATE();
    case 2849:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3013);
      END_STATE();
    case 2850:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3014);
      END_STATE();
    case 2851:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3015);
      END_STATE();
    case 2852:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3016);
      END_STATE();
    case 2853:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3017);
      END_STATE();
    case 2854:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3018);
      END_STATE();
    case 2855:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3019);
      END_STATE();
    case 2856:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(3020);
      END_STATE();
    case 2857:
      ACCEPT_TOKEN(sym_data_type_string);
      END_STATE();
    case 2858:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3021);
      END_STATE();
    case 2859:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3022);
      END_STATE();
    case 2860:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3023);
      END_STATE();
    case 2861:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3024);
      END_STATE();
    case 2862:
      ACCEPT_TOKEN(sym_delete_url_entry);
      END_STATE();
    case 2863:
      ACCEPT_TOKEN(sym_device_id_string);
      END_STATE();
    case 2864:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3025);
      END_STATE();
    case 2865:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3026);
      END_STATE();
    case 2866:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3027);
      END_STATE();
    case 2867:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3028);
      END_STATE();
    case 2868:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3029);
      END_STATE();
    case 2869:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(3030);
      END_STATE();
    case 2870:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3031);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3032);
      END_STATE();
    case 2871:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3033);
      END_STATE();
    case 2872:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3034);
      END_STATE();
    case 2873:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3035);
      END_STATE();
    case 2874:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3036);
      END_STATE();
    case 2875:
      if (lookahead == '_') ADVANCE(3037);
      END_STATE();
    case 2876:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3038);
      END_STATE();
    case 2877:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3039);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3040);
      END_STATE();
    case 2878:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3041);
      END_STATE();
    case 2879:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3042);
      END_STATE();
    case 2880:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3043);
      END_STATE();
    case 2881:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3044);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3045);
      END_STATE();
    case 2882:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3046);
      END_STATE();
    case 2883:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3047);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3048);
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3049);
      END_STATE();
    case 2884:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3050);
      END_STATE();
    case 2885:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3051);
      END_STATE();
    case 2886:
      ACCEPT_TOKEN(sym_duet_dev_type_tv);
      END_STATE();
    case 2887:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3052);
      END_STATE();
    case 2888:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3053);
      END_STATE();
    case 2889:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3054);
      END_STATE();
    case 2890:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3055);
      END_STATE();
    case 2891:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3056);
      END_STATE();
    case 2892:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3057);
      END_STATE();
    case 2893:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3058);
      END_STATE();
    case 2894:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3059);
      END_STATE();
    case 2895:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3060);
      END_STATE();
    case 2896:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3061);
      END_STATE();
    case 2897:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3062);
      END_STATE();
    case 2898:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3063);
      END_STATE();
    case 2899:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3064);
      END_STATE();
    case 2900:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3065);
      END_STATE();
    case 2901:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(3066);
      END_STATE();
    case 2902:
      ACCEPT_TOKEN(sym_err_name_invalid);
      END_STATE();
    case 2903:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3067);
      END_STATE();
    case 2904:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3068);
      END_STATE();
    case 2905:
      if (lookahead == '_') ADVANCE(3069);
      END_STATE();
    case 2906:
      ACCEPT_TOKEN(sym_first_local_port);
      END_STATE();
    case 2907:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3070);
      END_STATE();
    case 2908:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3071);
      END_STATE();
    case 2909:
      ACCEPT_TOKEN(sym_get_master_build);
      END_STATE();
    case 2910:
      if (lookahead == '_') ADVANCE(3072);
      END_STATE();
    case 2911:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3073);
      END_STATE();
    case 2912:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3074);
      END_STATE();
    case 2913:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3075);
      END_STATE();
    case 2914:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3076);
      END_STATE();
    case 2915:
      ACCEPT_TOKEN(sym_ip_addr_flg_dhcp);
      END_STATE();
    case 2916:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3077);
      END_STATE();
    case 2917:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3078);
      END_STATE();
    case 2918:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3079);
      END_STATE();
    case 2919:
      if (lookahead == '_') ADVANCE(3080);
      END_STATE();
    case 2920:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3081);
      END_STATE();
    case 2921:
      ACCEPT_TOKEN(sym_ip_ttl_continent);
      END_STATE();
    case 2922:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3082);
      END_STATE();
    case 2923:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3083);
      END_STATE();
    case 2924:
      ACCEPT_TOKEN(sym_load_duet_module);
      END_STATE();
    case 2925:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3084);
      END_STATE();
    case 2926:
      ACCEPT_TOKEN(sym_max_length_array);
      END_STATE();
    case 2927:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3085);
      END_STATE();
    case 2928:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3086);
      END_STATE();
    case 2929:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3087);
      END_STATE();
    case 2930:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3088);
      END_STATE();
    case 2931:
      ACCEPT_TOKEN(sym_programport_priv);
      END_STATE();
    case 2932:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3089);
      END_STATE();
    case 2933:
      ACCEPT_TOKEN(sym_set_length_array);
      END_STATE();
    case 2934:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3090);
      END_STATE();
    case 2935:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3091);
      END_STATE();
    case 2936:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3092);
      END_STATE();
    case 2937:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3093);
      END_STATE();
    case 2938:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3094);
      END_STATE();
    case 2939:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3095);
      END_STATE();
    case 2940:
      if (lookahead == '_') ADVANCE(3096);
      END_STATE();
    case 2941:
      ACCEPT_TOKEN(sym_smtp_port_number);
      END_STATE();
    case 2942:
      ACCEPT_TOKEN(sym_smtp_require_tls);
      END_STATE();
    case 2943:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3097);
      END_STATE();
    case 2944:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3098);
      END_STATE();
    case 2945:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3099);
      END_STATE();
    case 2946:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3100);
      END_STATE();
    case 2947:
      if (lookahead == '_') ADVANCE(3101);
      END_STATE();
    case 2948:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3102);
      END_STATE();
    case 2949:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3103);
      END_STATE();
    case 2950:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3104);
      END_STATE();
    case 2951:
      if (lookahead == '2') ADVANCE(3105);
      END_STATE();
    case 2952:
      ACCEPT_TOKEN(sym_ssh_client_close);
      END_STATE();
    case 2953:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3106);
      END_STATE();
    case 2954:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3107);
      END_STATE();
    case 2955:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3108);
      END_STATE();
    case 2956:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3109);
      END_STATE();
    case 2957:
      if (lookahead == '_') ADVANCE(3110);
      END_STATE();
    case 2958:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3111);
      END_STATE();
    case 2959:
      ACCEPT_TOKEN(sym_timeline_restart);
      END_STATE();
    case 2960:
      ACCEPT_TOKEN(sym_tls_client_close);
      END_STATE();
    case 2961:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3112);
      END_STATE();
    case 2962:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3113);
      END_STATE();
    case 2963:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3114);
      END_STATE();
    case 2964:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3115);
      END_STATE();
    case 2965:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3116);
      END_STATE();
    case 2966:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3117);
      END_STATE();
    case 2967:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3118);
      END_STATE();
    case 2968:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3119);
      END_STATE();
    case 2969:
      if (lookahead == '_') ADVANCE(3120);
      END_STATE();
    case 2970:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3121);
      END_STATE();
    case 2971:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3122);
      END_STATE();
    case 2972:
      ACCEPT_TOKEN(sym_xml_decode_types);
      END_STATE();
    case 2973:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3123);
      END_STATE();
    case 2974:
      ACCEPT_TOKEN(sym_xml_encode_types);
      END_STATE();
    case 2975:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3124);
      END_STATE();
    case 2976:
      if (lookahead == '_') ADVANCE(3125);
      END_STATE();
    case 2977:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3126);
      END_STATE();
    case 2978:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3127);
      END_STATE();
    case 2979:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3128);
      END_STATE();
    case 2980:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3129);
      END_STATE();
    case 2981:
      ACCEPT_TOKEN(sym__wc_file_open_int);
      END_STATE();
    case 2982:
      ACCEPT_TOKEN(sym__wc_file_read_int);
      END_STATE();
    case 2983:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3130);
      END_STATE();
    case 2984:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3131);
      END_STATE();
    case 2985:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3132);
      END_STATE();
    case 2986:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3133);
      END_STATE();
    case 2987:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3134);
      END_STATE();
    case 2988:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3135);
      END_STATE();
    case 2989:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3136);
      END_STATE();
    case 2990:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3137);
      END_STATE();
    case 2991:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3138);
      END_STATE();
    case 2992:
      if (lookahead == '_') ADVANCE(3139);
      END_STATE();
    case 2993:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3140);
      END_STATE();
    case 2994:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3141);
      END_STATE();
    case 2995:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3142);
      END_STATE();
    case 2996:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3143);
      END_STATE();
    case 2997:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3144);
      END_STATE();
    case 2998:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3145);
      END_STATE();
    case 2999:
      ACCEPT_TOKEN(sym_cancel_wait_until_keyword);
      END_STATE();
    case 3000:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3146);
      END_STATE();
    case 3001:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3147);
      END_STATE();
    case 3002:
      ACCEPT_TOKEN(sym_data_format_email);
      END_STATE();
    case 3003:
      if (lookahead == '0') ADVANCE(3148);
      END_STATE();
    case 3004:
      ACCEPT_TOKEN(sym_data_format_phone);
      END_STATE();
    case 3005:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3149);
      END_STATE();
    case 3006:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3150);
      END_STATE();
    case 3007:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3151);
      END_STATE();
    case 3008:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3152);
      END_STATE();
    case 3009:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3153);
      END_STATE();
    case 3010:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3154);
      END_STATE();
    case 3011:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3155);
      END_STATE();
    case 3012:
      if (lookahead == '_') ADVANCE(3156);
      END_STATE();
    case 3013:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3157);
      END_STATE();
    case 3014:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3158);
      END_STATE();
    case 3015:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3159);
      END_STATE();
    case 3016:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3160);
      END_STATE();
    case 3017:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3161);
      END_STATE();
    case 3018:
      ACCEPT_TOKEN(sym_data_publish_feed);
      END_STATE();
    case 3019:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3162);
      END_STATE();
    case 3020:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3163);
      END_STATE();
    case 3021:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3164);
      END_STATE();
    case 3022:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3165);
      END_STATE();
    case 3023:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(3166);
      END_STATE();
    case 3024:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3167);
      END_STATE();
    case 3025:
      ACCEPT_TOKEN(sym_deviceconfig_priv);
      END_STATE();
    case 3026:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3168);
      END_STATE();
    case 3027:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3169);
      END_STATE();
    case 3028:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3170);
      END_STATE();
    case 3029:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3171);
      END_STATE();
    case 3030:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3172);
      END_STATE();
    case 3031:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3173);
      END_STATE();
    case 3032:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3174);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3175);
      END_STATE();
    case 3033:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3176);
      END_STATE();
    case 3034:
      ACCEPT_TOKEN(sym_duet_dev_type_dss);
      END_STATE();
    case 3035:
      ACCEPT_TOKEN(sym_duet_dev_type_dvr);
      END_STATE();
    case 3036:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3177);
      END_STATE();
    case 3037:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3178);
      END_STATE();
    case 3038:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3179);
      END_STATE();
    case 3039:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3180);
      END_STATE();
    case 3040:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3181);
      END_STATE();
    case 3041:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3182);
      END_STATE();
    case 3042:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3183);
      END_STATE();
    case 3043:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3184);
      END_STATE();
    case 3044:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3185);
      END_STATE();
    case 3045:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3186);
      END_STATE();
    case 3046:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3187);
      END_STATE();
    case 3047:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3188);
      END_STATE();
    case 3048:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3189);
      END_STATE();
    case 3049:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3190);
      END_STATE();
    case 3050:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3191);
      END_STATE();
    case 3051:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3192);
      END_STATE();
    case 3052:
      ACCEPT_TOKEN(sym_duet_dev_type_ups);
      END_STATE();
    case 3053:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3193);
      END_STATE();
    case 3054:
      ACCEPT_TOKEN(sym_duet_dev_type_vcr);
      END_STATE();
    case 3055:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3194);
      END_STATE();
    case 3056:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3195);
      END_STATE();
    case 3057:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3196);
      END_STATE();
    case 3058:
      ACCEPT_TOKEN(sym_duet_mem_size_get);
      END_STATE();
    case 3059:
      ACCEPT_TOKEN(sym_duet_mem_size_set);
      END_STATE();
    case 3060:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3197);
      END_STATE();
    case 3061:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3198);
      END_STATE();
    case 3062:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3199);
      END_STATE();
    case 3063:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3200);
      END_STATE();
    case 3064:
      ACCEPT_TOKEN(sym_err_general_error);
      END_STATE();
    case 3065:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3201);
      END_STATE();
    case 3066:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3202);
      END_STATE();
    case 3067:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3203);
      END_STATE();
    case 3068:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3204);
      END_STATE();
    case 3069:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3205);
      END_STATE();
    case 3070:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3206);
      END_STATE();
    case 3071:
      ACCEPT_TOKEN(sym_get_buffer_string);
      END_STATE();
    case 3072:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3207);
      END_STATE();
    case 3073:
      ACCEPT_TOKEN(sym_get_serial_number);
      END_STATE();
    case 3074:
      ACCEPT_TOKEN(sym_get_system_number);
      END_STATE();
    case 3075:
      if (lookahead == 'Z' ||
          lookahead == 'z') ADVANCE(3208);
      END_STATE();
    case 3076:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3209);
      END_STATE();
    case 3077:
      ACCEPT_TOKEN(sym_ip_address_struct);
      END_STATE();
    case 3078:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3210);
      END_STATE();
    case 3079:
      ACCEPT_TOKEN(sym_ip_mc_server_open);
      END_STATE();
    case 3080:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3211);
      END_STATE();
    case 3081:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3212);
      END_STATE();
    case 3082:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3213);
      END_STATE();
    case 3083:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3214);
      END_STATE();
    case 3084:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3215);
      END_STATE();
    case 3085:
      ACCEPT_TOKEN(sym_max_length_string);
      END_STATE();
    case 3086:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3216);
      END_STATE();
    case 3087:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3217);
      END_STATE();
    case 3088:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3218);
      END_STATE();
    case 3089:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3219);
      END_STATE();
    case 3090:
      ACCEPT_TOKEN(sym_set_length_string);
      END_STATE();
    case 3091:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3220);
      END_STATE();
    case 3092:
      ACCEPT_TOKEN(sym_set_system_number);
      END_STATE();
    case 3093:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3221);
      END_STATE();
    case 3094:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3222);
      END_STATE();
    case 3095:
      if (lookahead == '_') ADVANCE(3223);
      END_STATE();
    case 3096:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3224);
      END_STATE();
    case 3097:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3225);
      END_STATE();
    case 3098:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3226);
      END_STATE();
    case 3099:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3227);
      END_STATE();
    case 3100:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3228);
      END_STATE();
    case 3101:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3229);
      END_STATE();
    case 3102:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3230);
      END_STATE();
    case 3103:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3231);
      END_STATE();
    case 3104:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3232);
      END_STATE();
    case 3105:
      ACCEPT_TOKEN(sym_source_type_rs232);
      END_STATE();
    case 3106:
      ACCEPT_TOKEN(sym_static_ip_binding);
      END_STATE();
    case 3107:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3233);
      END_STATE();
    case 3108:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3234);
      END_STATE();
    case 3109:
      ACCEPT_TOKEN(sym_timeline_absolute);
      END_STATE();
    case 3110:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3235);
      END_STATE();
    case 3111:
      ACCEPT_TOKEN(sym_timeline_relative);
      END_STATE();
    case 3112:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3236);
      END_STATE();
    case 3113:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3237);
      END_STATE();
    case 3114:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3238);
      END_STATE();
    case 3115:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3239);
      END_STATE();
    case 3116:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3240);
      END_STATE();
    case 3117:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3241);
      END_STATE();
    case 3118:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3242);
      END_STATE();
    case 3119:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3243);
      END_STATE();
    case 3120:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3244);
      END_STATE();
    case 3121:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3245);
      END_STATE();
    case 3122:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3246);
      END_STATE();
    case 3123:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3247);
      END_STATE();
    case 3124:
      if (lookahead == '_') ADVANCE(3248);
      END_STATE();
    case 3125:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3249);
      END_STATE();
    case 3126:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3250);
      END_STATE();
    case 3127:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3251);
      END_STATE();
    case 3128:
      if (lookahead == '_') ADVANCE(3252);
      END_STATE();
    case 3129:
      ACCEPT_TOKEN(sym__wc_file_close_int);
      END_STATE();
    case 3130:
      if (lookahead == '_') ADVANCE(3253);
      END_STATE();
    case 3131:
      ACCEPT_TOKEN(sym__wc_file_write_int);
      END_STATE();
    case 3132:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3254);
      END_STATE();
    case 3133:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3255);
      END_STATE();
    case 3134:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3256);
      END_STATE();
    case 3135:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3257);
      END_STATE();
    case 3136:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3258);
      END_STATE();
    case 3137:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3259);
      END_STATE();
    case 3138:
      ACCEPT_TOKEN(sym__wc_mid_string_int);
      END_STATE();
    case 3139:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3260);
      END_STATE();
    case 3140:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3261);
      END_STATE();
    case 3141:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3262);
      END_STATE();
    case 3142:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3263);
      END_STATE();
    case 3143:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3264);
      END_STATE();
    case 3144:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3265);
      END_STATE();
    case 3145:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3266);
      END_STATE();
    case 3146:
      ACCEPT_TOKEN(sym_configuration_priv);
      END_STATE();
    case 3147:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3267);
      END_STATE();
    case 3148:
      if (lookahead == '1') ADVANCE(3268);
      END_STATE();
    case 3149:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3269);
      END_STATE();
    case 3150:
      if (lookahead == '_') ADVANCE(3270);
      END_STATE();
    case 3151:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3271);
      END_STATE();
    case 3152:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3272);
      END_STATE();
    case 3153:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3273);
      END_STATE();
    case 3154:
      ACCEPT_TOKEN(sym_data_max_id_length);
      END_STATE();
    case 3155:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3274);
      END_STATE();
    case 3156:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3275);
      END_STATE();
    case 3157:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3276);
      END_STATE();
    case 3158:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3277);
      END_STATE();
    case 3159:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3278);
      END_STATE();
    case 3160:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3279);
      END_STATE();
    case 3161:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3280);
      END_STATE();
    case 3162:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3281);
      END_STATE();
    case 3163:
      ACCEPT_TOKEN(sym_data_type_datetime);
      END_STATE();
    case 3164:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3282);
      END_STATE();
    case 3165:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3283);
      END_STATE();
    case 3166:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3284);
      END_STATE();
    case 3167:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3285);
      END_STATE();
    case 3168:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3286);
      END_STATE();
    case 3169:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3287);
      END_STATE();
    case 3170:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3288);
      END_STATE();
    case 3171:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3289);
      END_STATE();
    case 3172:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3290);
      END_STATE();
    case 3173:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3291);
      END_STATE();
    case 3174:
      if (lookahead == '_') ADVANCE(3292);
      END_STATE();
    case 3175:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3293);
      END_STATE();
    case 3176:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(3294);
      END_STATE();
    case 3177:
      ACCEPT_TOKEN(sym_duet_dev_type_hvac);
      END_STATE();
    case 3178:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3295);
      END_STATE();
    case 3179:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3296);
      END_STATE();
    case 3180:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3297);
      END_STATE();
    case 3181:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3298);
      END_STATE();
    case 3182:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3299);
      END_STATE();
    case 3183:
      if (lookahead == '_') ADVANCE(3300);
      END_STATE();
    case 3184:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(3301);
      END_STATE();
    case 3185:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3302);
      END_STATE();
    case 3186:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(3303);
      END_STATE();
    case 3187:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3304);
      END_STATE();
    case 3188:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3305);
      END_STATE();
    case 3189:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3306);
      END_STATE();
    case 3190:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3307);
      END_STATE();
    case 3191:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3308);
      END_STATE();
    case 3192:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3309);
      END_STATE();
    case 3193:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3310);
      END_STATE();
    case 3194:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3311);
      END_STATE();
    case 3195:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(3312);
      END_STATE();
    case 3196:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3313);
      END_STATE();
    case 3197:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3314);
      END_STATE();
    case 3198:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3315);
      END_STATE();
    case 3199:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3316);
      END_STATE();
    case 3200:
      if (lookahead == '_') ADVANCE(3317);
      END_STATE();
    case 3201:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3318);
      END_STATE();
    case 3202:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3319);
      END_STATE();
    case 3203:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3320);
      END_STATE();
    case 3204:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3321);
      END_STATE();
    case 3205:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(3322);
      END_STATE();
    case 3206:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3323);
      END_STATE();
    case 3207:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3324);
      END_STATE();
    case 3208:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3325);
      END_STATE();
    case 3209:
      if (lookahead == '_') ADVANCE(3326);
      END_STATE();
    case 3210:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3327);
      END_STATE();
    case 3211:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3328);
      END_STATE();
    case 3212:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3329);
      END_STATE();
    case 3213:
      if (lookahead == '_') ADVANCE(3330);
      END_STATE();
    case 3214:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3331);
      END_STATE();
    case 3215:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3332);
      END_STATE();
    case 3216:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3333);
      END_STATE();
    case 3217:
      ACCEPT_TOKEN(sym_netlinx_login_fail);
      END_STATE();
    case 3218:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3334);
      END_STATE();
    case 3219:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3335);
      END_STATE();
    case 3220:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3336);
      END_STATE();
    case 3221:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3337);
      END_STATE();
    case 3222:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3338);
      END_STATE();
    case 3223:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3339);
      END_STATE();
    case 3224:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3340);
      END_STATE();
    case 3225:
      if (lookahead == '_') ADVANCE(3341);
      END_STATE();
    case 3226:
      if (lookahead == '_') ADVANCE(3342);
      END_STATE();
    case 3227:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3343);
      END_STATE();
    case 3228:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3344);
      END_STATE();
    case 3229:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3345);
      END_STATE();
    case 3230:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3346);
      END_STATE();
    case 3231:
      if (lookahead == '_') ADVANCE(3347);
      END_STATE();
    case 3232:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3348);
      END_STATE();
    case 3233:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3349);
      END_STATE();
    case 3234:
      ACCEPT_TOKEN(sym_string_to_variable);
      END_STATE();
    case 3235:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3350);
      END_STATE();
    case 3236:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3351);
      END_STATE();
    case 3237:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3352);
      END_STATE();
    case 3238:
      ACCEPT_TOKEN(sym_unload_duet_module);
      END_STATE();
    case 3239:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3353);
      END_STATE();
    case 3240:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3354);
      END_STATE();
    case 3241:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3355);
      END_STATE();
    case 3242:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3356);
      END_STATE();
    case 3243:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3357);
      END_STATE();
    case 3244:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3358);
      END_STATE();
    case 3245:
      ACCEPT_TOKEN(sym_variable_to_string);
      END_STATE();
    case 3246:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3359);
      END_STATE();
    case 3247:
      if (lookahead == '_') ADVANCE(3360);
      END_STATE();
    case 3248:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3361);
      END_STATE();
    case 3249:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3362);
      END_STATE();
    case 3250:
      ACCEPT_TOKEN(sym__wc_data_add_record);
      END_STATE();
    case 3251:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3363);
      END_STATE();
    case 3252:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3364);
      END_STATE();
    case 3253:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3365);
      END_STATE();
    case 3254:
      if (lookahead == '_') ADVANCE(3366);
      END_STATE();
    case 3255:
      ACCEPT_TOKEN(sym__wc_find_string_int);
      END_STATE();
    case 3256:
      if (lookahead == '_') ADVANCE(3367);
      END_STATE();
    case 3257:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3368);
      END_STATE();
    case 3258:
      ACCEPT_TOKEN(sym__wc_left_string_int);
      END_STATE();
    case 3259:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3369);
      END_STATE();
    case 3260:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3370);
      END_STATE();
    case 3261:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3371);
      END_STATE();
    case 3262:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3372);
      END_STATE();
    case 3263:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3373);
      END_STATE();
    case 3264:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3374);
      END_STATE();
    case 3265:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3375);
      END_STATE();
    case 3266:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3376);
      END_STATE();
    case 3267:
      ACCEPT_TOKEN(sym_create_multi_buffer_keyword);
      END_STATE();
    case 3268:
      ACCEPT_TOKEN(sym_data_format_iso8601);
      END_STATE();
    case 3269:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3377);
      END_STATE();
    case 3270:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3378);
      END_STATE();
    case 3271:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3379);
      END_STATE();
    case 3272:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3380);
      END_STATE();
    case 3273:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3381);
      END_STATE();
    case 3274:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3382);
      END_STATE();
    case 3275:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3383);
      END_STATE();
    case 3276:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3384);
      END_STATE();
    case 3277:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3385);
      END_STATE();
    case 3278:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3386);
      END_STATE();
    case 3279:
      ACCEPT_TOKEN(sym_data_max_url_length);
      END_STATE();
    case 3280:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3387);
      END_STATE();
    case 3281:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3388);
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3389);
      END_STATE();
    case 3282:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3390);
      END_STATE();
    case 3283:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3391);
      END_STATE();
    case 3284:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3392);
      END_STATE();
    case 3285:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(3393);
      END_STATE();
    case 3286:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3394);
      END_STATE();
    case 3287:
      ACCEPT_TOKEN(sym_duet_dev_not_polled);
      END_STATE();
    case 3288:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3395);
      END_STATE();
    case 3289:
      if (lookahead == '_') ADVANCE(3396);
      END_STATE();
    case 3290:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3397);
      END_STATE();
    case 3291:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3398);
      END_STATE();
    case 3292:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3399);
      END_STATE();
    case 3293:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3400);
      END_STATE();
    case 3294:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3401);
      END_STATE();
    case 3295:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3402);
      END_STATE();
    case 3296:
      ACCEPT_TOKEN(sym_duet_dev_type_light);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3403);
      END_STATE();
    case 3297:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3404);
      END_STATE();
    case 3298:
      ACCEPT_TOKEN(sym_duet_dev_type_motor);
      END_STATE();
    case 3299:
      if (lookahead == '_') ADVANCE(3405);
      END_STATE();
    case 3300:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3406);
      END_STATE();
    case 3301:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3407);
      END_STATE();
    case 3302:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3408);
      END_STATE();
    case 3303:
      if (lookahead == '_') ADVANCE(3409);
      END_STATE();
    case 3304:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(3410);
      END_STATE();
    case 3305:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3411);
      END_STATE();
    case 3306:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3412);
      END_STATE();
    case 3307:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3413);
      END_STATE();
    case 3308:
      if (lookahead == '_') ADVANCE(3414);
      END_STATE();
    case 3309:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3415);
      END_STATE();
    case 3310:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3416);
      END_STATE();
    case 3311:
      if (lookahead == '_') ADVANCE(3417);
      END_STATE();
    case 3312:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3418);
      END_STATE();
    case 3313:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3419);
      END_STATE();
    case 3314:
      if (lookahead == '_') ADVANCE(3420);
      END_STATE();
    case 3315:
      ACCEPT_TOKEN(sym_dynamic_polled_port);
      END_STATE();
    case 3316:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3421);
      END_STATE();
    case 3317:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3422);
      END_STATE();
    case 3318:
      ACCEPT_TOKEN(sym_err_invalid_account);
      END_STATE();
    case 3319:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3423);
      END_STATE();
    case 3320:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3424);
      END_STATE();
    case 3321:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3425);
      END_STATE();
    case 3322:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3426);
      END_STATE();
    case 3323:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3427);
      END_STATE();
    case 3324:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3428);
      END_STATE();
    case 3325:
      if (lookahead == '_') ADVANCE(3429);
      END_STATE();
    case 3326:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3430);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3431);
      END_STATE();
    case 3327:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3432);
      END_STATE();
    case 3328:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3433);
      END_STATE();
    case 3329:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3434);
      END_STATE();
    case 3330:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3435);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(3436);
      END_STATE();
    case 3331:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3437);
      END_STATE();
    case 3332:
      if (lookahead == '_') ADVANCE(3438);
      END_STATE();
    case 3333:
      ACCEPT_TOKEN(sym_netlinx_axi_version);
      END_STATE();
    case 3334:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3439);
      END_STATE();
    case 3335:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3440);
      END_STATE();
    case 3336:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3441);
      END_STATE();
    case 3337:
      ACCEPT_TOKEN(sym_set_validation_code);
      END_STATE();
    case 3338:
      if (lookahead == '_') ADVANCE(3442);
      END_STATE();
    case 3339:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3443);
      END_STATE();
    case 3340:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3444);
      END_STATE();
    case 3341:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3445);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3446);
      END_STATE();
    case 3342:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3447);
      END_STATE();
    case 3343:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3448);
      END_STATE();
    case 3344:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3449);
      END_STATE();
    case 3345:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3450);
      END_STATE();
    case 3346:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3451);
      END_STATE();
    case 3347:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3452);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3453);
      END_STATE();
    case 3348:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3454);
      END_STATE();
    case 3349:
      ACCEPT_TOKEN(sym_static_port_binding);
      END_STATE();
    case 3350:
      ACCEPT_TOKEN(sym_timeline_dynamic_id);
      END_STATE();
    case 3351:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3455);
      END_STATE();
    case 3352:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3456);
      END_STATE();
    case 3353:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3457);
      END_STATE();
    case 3354:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3458);
      END_STATE();
    case 3355:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3459);
      END_STATE();
    case 3356:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3460);
      END_STATE();
    case 3357:
      ACCEPT_TOKEN(sym_usermanagement_priv);
      END_STATE();
    case 3358:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3461);
      END_STATE();
    case 3359:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3462);
      END_STATE();
    case 3360:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3463);
      END_STATE();
    case 3361:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3464);
      END_STATE();
    case 3362:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3465);
      END_STATE();
    case 3363:
      ACCEPT_TOKEN(sym__wc_data_create_feed);
      END_STATE();
    case 3364:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3466);
      END_STATE();
    case 3365:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3467);
      END_STATE();
    case 3366:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3468);
      END_STATE();
    case 3367:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3469);
      END_STATE();
    case 3368:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3470);
      END_STATE();
    case 3369:
      ACCEPT_TOKEN(sym__wc_lower_string_int);
      END_STATE();
    case 3370:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3471);
      END_STATE();
    case 3371:
      ACCEPT_TOKEN(sym__wc_right_string_int);
      END_STATE();
    case 3372:
      ACCEPT_TOKEN(sym__wc_upper_string_int);
      END_STATE();
    case 3373:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3472);
      END_STATE();
    case 3374:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3473);
      END_STATE();
    case 3375:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3474);
      END_STATE();
    case 3376:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3475);
      END_STATE();
    case 3377:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3476);
      END_STATE();
    case 3378:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3477);
      END_STATE();
    case 3379:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3478);
      END_STATE();
    case 3380:
      if (lookahead == '_') ADVANCE(3479);
      END_STATE();
    case 3381:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3480);
      END_STATE();
    case 3382:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3481);
      END_STATE();
    case 3383:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3482);
      END_STATE();
    case 3384:
      ACCEPT_TOKEN(sym_data_max_name_length);
      END_STATE();
    case 3385:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3483);
      END_STATE();
    case 3386:
      ACCEPT_TOKEN(sym_data_max_type_length);
      END_STATE();
    case 3387:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3484);
      END_STATE();
    case 3388:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3485);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3486);
      END_STATE();
    case 3389:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3487);
      END_STATE();
    case 3390:
      ACCEPT_TOKEN(sym_define_connect_level_keyword);
      END_STATE();
    case 3391:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3488);
      END_STATE();
    case 3392:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3489);
      END_STATE();
    case 3393:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3490);
      END_STATE();
    case 3394:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3491);
      END_STATE();
    case 3395:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3492);
      END_STATE();
    case 3396:
      ADVANCE_MAP(
        'C', 3493,
        'c', 3493,
        'M', 3494,
        'm', 3494,
        'P', 3495,
        'p', 3495,
        'T', 3496,
        't', 3496,
      );
      END_STATE();
    case 3397:
      ACCEPT_TOKEN(sym_duet_dev_type_camera);
      END_STATE();
    case 3398:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3497);
      END_STATE();
    case 3399:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3498);
      END_STATE();
    case 3400:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(3499);
      END_STATE();
    case 3401:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3500);
      END_STATE();
    case 3402:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3501);
      END_STATE();
    case 3403:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(3502);
      END_STATE();
    case 3404:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3503);
      END_STATE();
    case 3405:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(3504);
      END_STATE();
    case 3406:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3505);
      END_STATE();
    case 3407:
      if (lookahead == '_') ADVANCE(3506);
      END_STATE();
    case 3408:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3507);
      END_STATE();
    case 3409:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3508);
      END_STATE();
    case 3410:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3509);
      END_STATE();
    case 3411:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3510);
      END_STATE();
    case 3412:
      if (lookahead == '_') ADVANCE(3511);
      END_STATE();
    case 3413:
      if (lookahead == '_') ADVANCE(3512);
      END_STATE();
    case 3414:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3513);
      END_STATE();
    case 3415:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3514);
      END_STATE();
    case 3416:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(3515);
      END_STATE();
    case 3417:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3516);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3517);
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(3518);
      END_STATE();
    case 3418:
      if (lookahead == '_') ADVANCE(3519);
      END_STATE();
    case 3419:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3520);
      END_STATE();
    case 3420:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3521);
      END_STATE();
    case 3421:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3522);
      END_STATE();
    case 3422:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3523);
      END_STATE();
    case 3423:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3524);
      END_STATE();
    case 3424:
      ACCEPT_TOKEN(sym_err_no_authorization);
      END_STATE();
    case 3425:
      ACCEPT_TOKEN(sym_err_password_invalid);
      END_STATE();
    case 3426:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3525);
      END_STATE();
    case 3427:
      ACCEPT_TOKEN(sym_first_virtual_device);
      END_STATE();
    case 3428:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3526);
      END_STATE();
    case 3429:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3527);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3528);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3529);
      END_STATE();
    case 3430:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3530);
      END_STATE();
    case 3431:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3531);
      END_STATE();
    case 3432:
      ACCEPT_TOKEN(sym_ip_bound_client_open);
      END_STATE();
    case 3433:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3532);
      END_STATE();
    case 3434:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3533);
      END_STATE();
    case 3435:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3534);
      END_STATE();
    case 3436:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(3535);
      END_STATE();
    case 3437:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3536);
      END_STATE();
    case 3438:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3537);
      END_STATE();
    case 3439:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3538);
      END_STATE();
    case 3440:
      ACCEPT_TOKEN(sym_securitycontrol_priv);
      END_STATE();
    case 3441:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3539);
      END_STATE();
    case 3442:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3540);
      END_STATE();
    case 3443:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3541);
      END_STATE();
    case 3444:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3542);
      END_STATE();
    case 3445:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3543);
      END_STATE();
    case 3446:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3544);
      END_STATE();
    case 3447:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3545);
      END_STATE();
    case 3448:
      ACCEPT_TOKEN(sym_source_type_internal);
      END_STATE();
    case 3449:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3546);
      END_STATE();
    case 3450:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3547);
      END_STATE();
    case 3451:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3548);
      END_STATE();
    case 3452:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3549);
      END_STATE();
    case 3453:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3550);
      END_STATE();
    case 3454:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3551);
      END_STATE();
    case 3455:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3552);
      END_STATE();
    case 3456:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3553);
      END_STATE();
    case 3457:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3554);
      END_STATE();
    case 3458:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3555);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3556);
      END_STATE();
    case 3459:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3557);
      END_STATE();
    case 3460:
      ACCEPT_TOKEN(sym_url_flg_stat_waiting);
      END_STATE();
    case 3461:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3558);
      END_STATE();
    case 3462:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3559);
      END_STATE();
    case 3463:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3560);
      END_STATE();
    case 3464:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3561);
      END_STATE();
    case 3465:
      ACCEPT_TOKEN(sym__wc_concat_string_int);
      END_STATE();
    case 3466:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3562);
      END_STATE();
    case 3467:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3563);
      END_STATE();
    case 3468:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3564);
      END_STATE();
    case 3469:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3565);
      END_STATE();
    case 3470:
      if (lookahead == '_') ADVANCE(3566);
      END_STATE();
    case 3471:
      ACCEPT_TOKEN(sym__wc_remove_string_int);
      END_STATE();
    case 3472:
      if (lookahead == '_') ADVANCE(3567);
      END_STATE();
    case 3473:
      if (lookahead == '_') ADVANCE(3568);
      END_STATE();
    case 3474:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3569);
      END_STATE();
    case 3475:
      ACCEPT_TOKEN(sym_cancel_all_wait_until_keyword);
      END_STATE();
    case 3476:
      ACCEPT_TOKEN(sym_data_get_event_record);
      END_STATE();
    case 3477:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3570);
      END_STATE();
    case 3478:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3571);
      END_STATE();
    case 3479:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3572);
      END_STATE();
    case 3480:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3573);
      END_STATE();
    case 3481:
      ACCEPT_TOKEN(sym_data_max_label_length);
      END_STATE();
    case 3482:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3574);
      END_STATE();
    case 3483:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3575);
      END_STATE();
    case 3484:
      ACCEPT_TOKEN(sym_data_max_value_length);
      END_STATE();
    case 3485:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3576);
      END_STATE();
    case 3486:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3577);
      END_STATE();
    case 3487:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3578);
      END_STATE();
    case 3488:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3579);
      END_STATE();
    case 3489:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3580);
      END_STATE();
    case 3490:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3581);
      END_STATE();
    case 3491:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3582);
      END_STATE();
    case 3492:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3583);
      END_STATE();
    case 3493:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3584);
      END_STATE();
    case 3494:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3585);
      END_STATE();
    case 3495:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3586);
      END_STATE();
    case 3496:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3587);
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3588);
      END_STATE();
    case 3497:
      if (lookahead == '_') ADVANCE(3589);
      END_STATE();
    case 3498:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3590);
      END_STATE();
    case 3499:
      if (lookahead == '_') ADVANCE(3591);
      END_STATE();
    case 3500:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3592);
      END_STATE();
    case 3501:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3593);
      END_STATE();
    case 3502:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3594);
      END_STATE();
    case 3503:
      ACCEPT_TOKEN(sym_duet_dev_type_monitor);
      END_STATE();
    case 3504:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3595);
      END_STATE();
    case 3505:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3596);
      END_STATE();
    case 3506:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3597);
      END_STATE();
    case 3507:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3598);
      END_STATE();
    case 3508:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3599);
      END_STATE();
    case 3509:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3600);
      END_STATE();
    case 3510:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(3601);
      END_STATE();
    case 3511:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3602);
      END_STATE();
    case 3512:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(3603);
      END_STATE();
    case 3513:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3604);
      END_STATE();
    case 3514:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3605);
      END_STATE();
    case 3515:
      ACCEPT_TOKEN(sym_duet_dev_type_utility);
      END_STATE();
    case 3516:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3606);
      END_STATE();
    case 3517:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3607);
      END_STATE();
    case 3518:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3608);
      END_STATE();
    case 3519:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3609);
      END_STATE();
    case 3520:
      ACCEPT_TOKEN(sym_duet_dev_type_weather);
      END_STATE();
    case 3521:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3610);
      END_STATE();
    case 3522:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3611);
      END_STATE();
    case 3523:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3612);
      END_STATE();
    case 3524:
      ACCEPT_TOKEN(sym_err_invalid_parameter);
      END_STATE();
    case 3525:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3613);
      END_STATE();
    case 3526:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3614);
      END_STATE();
    case 3527:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3615);
      END_STATE();
    case 3528:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3616);
      END_STATE();
    case 3529:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3617);
      END_STATE();
    case 3530:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3618);
      END_STATE();
    case 3531:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3619);
      END_STATE();
    case 3532:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3620);
      END_STATE();
    case 3533:
      ACCEPT_TOKEN(sym_ip_tcp_nodelay_option);
      END_STATE();
    case 3534:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3621);
      END_STATE();
    case 3535:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3622);
      END_STATE();
    case 3536:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3623);
      END_STATE();
    case 3537:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3624);
      END_STATE();
    case 3538:
      ACCEPT_TOKEN(sym_netlinx_login_success);
      END_STATE();
    case 3539:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3625);
      END_STATE();
    case 3540:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3626);
      END_STATE();
    case 3541:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3627);
      END_STATE();
    case 3542:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3628);
      END_STATE();
    case 3543:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3629);
      END_STATE();
    case 3544:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3630);
      END_STATE();
    case 3545:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3631);
      END_STATE();
    case 3546:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3632);
      END_STATE();
    case 3547:
      ACCEPT_TOKEN(sym_source_type_ipv4_port);
      if (lookahead == '_') ADVANCE(3633);
      END_STATE();
    case 3548:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3634);
      END_STATE();
    case 3549:
      ACCEPT_TOKEN(sym_source_type_neuron_id);
      END_STATE();
    case 3550:
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(3635);
      END_STATE();
    case 3551:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3636);
      END_STATE();
    case 3552:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3637);
      END_STATE();
    case 3553:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3638);
      END_STATE();
    case 3554:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3639);
      END_STATE();
    case 3555:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3640);
      END_STATE();
    case 3556:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3641);
      END_STATE();
    case 3557:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3642);
      END_STATE();
    case 3558:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3643);
      END_STATE();
    case 3559:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3644);
      END_STATE();
    case 3560:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3645);
      END_STATE();
    case 3561:
      ACCEPT_TOKEN(sym__wc_compare_string_int);
      END_STATE();
    case 3562:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3646);
      END_STATE();
    case 3563:
      ACCEPT_TOKEN(sym__wc_file_read_line_int);
      END_STATE();
    case 3564:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3647);
      END_STATE();
    case 3565:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3648);
      END_STATE();
    case 3566:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3649);
      END_STATE();
    case 3567:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3650);
      END_STATE();
    case 3568:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3651);
      END_STATE();
    case 3569:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3652);
      END_STATE();
    case 3570:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3653);
      END_STATE();
    case 3571:
      if (lookahead == '_') ADVANCE(3654);
      END_STATE();
    case 3572:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3655);
      END_STATE();
    case 3573:
      ACCEPT_TOKEN(sym_data_max_format_length);
      END_STATE();
    case 3574:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3656);
      END_STATE();
    case 3575:
      ACCEPT_TOKEN(sym_data_max_source_length);
      END_STATE();
    case 3576:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3657);
      END_STATE();
    case 3577:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3658);
      END_STATE();
    case 3578:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3659);
      END_STATE();
    case 3579:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3660);
      END_STATE();
    case 3580:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3661);
      END_STATE();
    case 3581:
      ACCEPT_TOKEN(sym_define_system_variable_keyword);
      END_STATE();
    case 3582:
      ACCEPT_TOKEN(sym_do_push_timed_infinite);
      END_STATE();
    case 3583:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3662);
      END_STATE();
    case 3584:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3663);
      END_STATE();
    case 3585:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(3664);
      END_STATE();
    case 3586:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3665);
      END_STATE();
    case 3587:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3666);
      END_STATE();
    case 3588:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3667);
      END_STATE();
    case 3589:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(3668);
      END_STATE();
    case 3590:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3669);
      END_STATE();
    case 3591:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3670);
      END_STATE();
    case 3592:
      if (lookahead == '_') ADVANCE(3671);
      END_STATE();
    case 3593:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3672);
      END_STATE();
    case 3594:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3673);
      END_STATE();
    case 3595:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3674);
      END_STATE();
    case 3596:
      ACCEPT_TOKEN(sym_duet_dev_type_pool_spa);
      END_STATE();
    case 3597:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3675);
      END_STATE();
    case 3598:
      ACCEPT_TOKEN(sym_duet_dev_type_receiver);
      END_STATE();
    case 3599:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3676);
      END_STATE();
    case 3600:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3677);
      END_STATE();
    case 3601:
      if (lookahead == '_') ADVANCE(3678);
      END_STATE();
    case 3602:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3679);
      END_STATE();
    case 3603:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3680);
      END_STATE();
    case 3604:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3681);
      END_STATE();
    case 3605:
      ACCEPT_TOKEN(sym_duet_dev_type_switcher);
      END_STATE();
    case 3606:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3682);
      END_STATE();
    case 3607:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3683);
      END_STATE();
    case 3608:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3684);
      END_STATE();
    case 3609:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3685);
      END_STATE();
    case 3610:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3686);
      END_STATE();
    case 3611:
      ACCEPT_TOKEN(sym_dynamic_virtual_device);
      END_STATE();
    case 3612:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3687);
      END_STATE();
    case 3613:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3688);
      END_STATE();
    case 3614:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3689);
      END_STATE();
    case 3615:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3690);
      END_STATE();
    case 3616:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3691);
      END_STATE();
    case 3617:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3692);
      END_STATE();
    case 3618:
      ACCEPT_TOKEN(sym_internal_threshold_get);
      END_STATE();
    case 3619:
      ACCEPT_TOKEN(sym_internal_threshold_set);
      END_STATE();
    case 3620:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3693);
      END_STATE();
    case 3621:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3694);
      END_STATE();
    case 3622:
      ACCEPT_TOKEN(sym_length_variable_to_xml);
      END_STATE();
    case 3623:
      if (lookahead == '_') ADVANCE(3695);
      END_STATE();
    case 3624:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3696);
      END_STATE();
    case 3625:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3697);
      END_STATE();
    case 3626:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3698);
      END_STATE();
    case 3627:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3699);
      END_STATE();
    case 3628:
      ACCEPT_TOKEN(sym_set_virtual_port_count);
      END_STATE();
    case 3629:
      ACCEPT_TOKEN(sym_smtp_server_config_get);
      END_STATE();
    case 3630:
      ACCEPT_TOKEN(sym_smtp_server_config_set);
      END_STATE();
    case 3631:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3700);
      END_STATE();
    case 3632:
      ACCEPT_TOKEN(sym_source_type_ip_address);
      END_STATE();
    case 3633:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(3701);
      END_STATE();
    case 3634:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3702);
      END_STATE();
    case 3635:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3703);
      END_STATE();
    case 3636:
      ACCEPT_TOKEN(sym_source_type_no_address);
      END_STATE();
    case 3637:
      if (lookahead == '_') ADVANCE(3704);
      END_STATE();
    case 3638:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3705);
      END_STATE();
    case 3639:
      ACCEPT_TOKEN(sym_url_flg_accinfopresent);
      END_STATE();
    case 3640:
      ACCEPT_TOKEN(sym_url_flg_stat_connected);
      END_STATE();
    case 3641:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3706);
      END_STATE();
    case 3642:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(3707);
      END_STATE();
    case 3643:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3708);
      END_STATE();
    case 3644:
      ACCEPT_TOKEN(sym_xml_decode_no_preserve);
      END_STATE();
    case 3645:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3709);
      END_STATE();
    case 3646:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3710);
      END_STATE();
    case 3647:
      ACCEPT_TOKEN(sym__wc_file_write_line_int);
      END_STATE();
    case 3648:
      ACCEPT_TOKEN(sym__wc_get_buffer_char_int);
      END_STATE();
    case 3649:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3711);
      END_STATE();
    case 3650:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3712);
      END_STATE();
    case 3651:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3713);
      END_STATE();
    case 3652:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3714);
      END_STATE();
    case 3653:
      ACCEPT_TOKEN(sym_data_get_published_feed);
      END_STATE();
    case 3654:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3715);
      END_STATE();
    case 3655:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3716);
      END_STATE();
    case 3656:
      if (lookahead == '_') ADVANCE(3717);
      END_STATE();
    case 3657:
      ACCEPT_TOKEN(sym_data_structure_datafeed);
      END_STATE();
    case 3658:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3718);
      END_STATE();
    case 3659:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3719);
      END_STATE();
    case 3660:
      ACCEPT_TOKEN(sym_define_library_function_keyword);
      END_STATE();
    case 3661:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3720);
      END_STATE();
    case 3662:
      ACCEPT_TOKEN(sym_duet_dev_type_amplifier);
      END_STATE();
    case 3663:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3721);
      END_STATE();
    case 3664:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3722);
      END_STATE();
    case 3665:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3723);
      END_STATE();
    case 3666:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3724);
      END_STATE();
    case 3667:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3725);
      END_STATE();
    case 3668:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3726);
      END_STATE();
    case 3669:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3727);
      END_STATE();
    case 3670:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3728);
      END_STATE();
    case 3671:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3729);
      END_STATE();
    case 3672:
      ACCEPT_TOKEN(sym_duet_dev_type_io_device);
      END_STATE();
    case 3673:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3730);
      END_STATE();
    case 3674:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3731);
      END_STATE();
    case 3675:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3732);
      END_STATE();
    case 3676:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3733);
      END_STATE();
    case 3677:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(3734);
      END_STATE();
    case 3678:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3735);
      END_STATE();
    case 3679:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3736);
      END_STATE();
    case 3680:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(3737);
      END_STATE();
    case 3681:
      if (lookahead == 'J' ||
          lookahead == 'j') ADVANCE(3738);
      END_STATE();
    case 3682:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3739);
      END_STATE();
    case 3683:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3740);
      if (lookahead == 'J' ||
          lookahead == 'j') ADVANCE(3741);
      END_STATE();
    case 3684:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3742);
      END_STATE();
    case 3685:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3743);
      END_STATE();
    case 3686:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3744);
      END_STATE();
    case 3687:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3745);
      END_STATE();
    case 3688:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3746);
      END_STATE();
    case 3689:
      ACCEPT_TOKEN(sym_get_multi_buffer_string);
      END_STATE();
    case 3690:
      ACCEPT_TOKEN(sym_internal_queue_size_get);
      END_STATE();
    case 3691:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3747);
      END_STATE();
    case 3692:
      ACCEPT_TOKEN(sym_internal_queue_size_set);
      END_STATE();
    case 3693:
      ACCEPT_TOKEN(sym_ip_multicast_ttl_option);
      END_STATE();
    case 3694:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3748);
      END_STATE();
    case 3695:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3749);
      END_STATE();
    case 3696:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3750);
      END_STATE();
    case 3697:
      ACCEPT_TOKEN(sym_set_outdoor_temperature);
      END_STATE();
    case 3698:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3751);
      END_STATE();
    case 3699:
      ACCEPT_TOKEN(sym_set_virtual_level_count);
      END_STATE();
    case 3700:
      ACCEPT_TOKEN(sym_softwaremanagement_priv);
      END_STATE();
    case 3701:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3752);
      END_STATE();
    case 3702:
      ACCEPT_TOKEN(sym_source_type_mac_address);
      END_STATE();
    case 3703:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3753);
      END_STATE();
    case 3704:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3754);
      END_STATE();
    case 3705:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3755);
      END_STATE();
    case 3706:
      ACCEPT_TOKEN(sym_url_flg_stat_connecting);
      END_STATE();
    case 3707:
      ACCEPT_TOKEN(sym_url_flg_stat_prgnetlinx);
      END_STATE();
    case 3708:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3756);
      END_STATE();
    case 3709:
      ACCEPT_TOKEN(sym_xml_encode_char_as_list);
      END_STATE();
    case 3710:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3757);
      END_STATE();
    case 3711:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3758);
      END_STATE();
    case 3712:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3759);
      END_STATE();
    case 3713:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3760);
      END_STATE();
    case 3714:
      ACCEPT_TOKEN(sym_authenticate_certificate);
      END_STATE();
    case 3715:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3761);
      END_STATE();
    case 3716:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3762);
      END_STATE();
    case 3717:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3763);
      END_STATE();
    case 3718:
      ACCEPT_TOKEN(sym_data_structure_datafield);
      END_STATE();
    case 3719:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3764);
      END_STATE();
    case 3720:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3765);
      END_STATE();
    case 3721:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3766);
      END_STATE();
    case 3722:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3767);
      END_STATE();
    case 3723:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3768);
      END_STATE();
    case 3724:
      ACCEPT_TOKEN(sym_duet_dev_type_audio_tape);
      END_STATE();
    case 3725:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3769);
      END_STATE();
    case 3726:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3770);
      END_STATE();
    case 3727:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3771);
      END_STATE();
    case 3728:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3772);
      END_STATE();
    case 3729:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3773);
      END_STATE();
    case 3730:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(3774);
      END_STATE();
    case 3731:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3775);
      END_STATE();
    case 3732:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3776);
      END_STATE();
    case 3733:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3777);
      END_STATE();
    case 3734:
      if (lookahead == '_') ADVANCE(3778);
      END_STATE();
    case 3735:
      if (lookahead == 'Y' ||
          lookahead == 'y') ADVANCE(3779);
      END_STATE();
    case 3736:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3780);
      END_STATE();
    case 3737:
      ACCEPT_TOKEN(sym_duet_dev_type_settop_box);
      END_STATE();
    case 3738:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3781);
      END_STATE();
    case 3739:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3782);
      END_STATE();
    case 3740:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3783);
      END_STATE();
    case 3741:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3784);
      END_STATE();
    case 3742:
      ACCEPT_TOKEN(sym_duet_dev_type_video_wall);
      END_STATE();
    case 3743:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3785);
      END_STATE();
    case 3744:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3786);
      END_STATE();
    case 3745:
      ACCEPT_TOKEN(sym_err_authentication_error);
      END_STATE();
    case 3746:
      ACCEPT_TOKEN(sym_err_user_account_expired);
      END_STATE();
    case 3747:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(3787);
      END_STATE();
    case 3748:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3788);
      END_STATE();
    case 3749:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3789);
      END_STATE();
    case 3750:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3790);
      END_STATE();
    case 3751:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3791);
      END_STATE();
    case 3752:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3792);
      END_STATE();
    case 3753:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3793);
      END_STATE();
    case 3754:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3794);
      END_STATE();
    case 3755:
      ACCEPT_TOKEN(sym_tls_validate_certificate);
      END_STATE();
    case 3756:
      ACCEPT_TOKEN(sym_validate_netlinx_account);
      if (lookahead == '_') ADVANCE(3795);
      END_STATE();
    case 3757:
      ACCEPT_TOKEN(sym__wc_data_get_event_record);
      END_STATE();
    case 3758:
      ACCEPT_TOKEN(sym__wc_get_buffer_string_int);
      END_STATE();
    case 3759:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3796);
      END_STATE();
    case 3760:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3797);
      END_STATE();
    case 3761:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3798);
      END_STATE();
    case 3762:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3799);
      END_STATE();
    case 3763:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3800);
      END_STATE();
    case 3764:
      ACCEPT_TOKEN(sym_data_structure_datarecord);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3801);
      END_STATE();
    case 3765:
      ACCEPT_TOKEN(sym_define_mutually_exclusive_keyword);
      END_STATE();
    case 3766:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3802);
      END_STATE();
    case 3767:
      ACCEPT_TOKEN(sym_duet_dev_type_audio_mixer);
      END_STATE();
    case 3768:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3803);
      END_STATE();
    case 3769:
      if (lookahead == '_') ADVANCE(3804);
      END_STATE();
    case 3770:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3805);
      END_STATE();
    case 3771:
      ACCEPT_TOKEN(sym_duet_dev_type_disc_device);
      END_STATE();
    case 3772:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3806);
      END_STATE();
    case 3773:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(3807);
      END_STATE();
    case 3774:
      if (lookahead == '_') ADVANCE(3808);
      END_STATE();
    case 3775:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(3809);
      END_STATE();
    case 3776:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3810);
      END_STATE();
    case 3777:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3811);
      END_STATE();
    case 3778:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3812);
      END_STATE();
    case 3779:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3813);
      END_STATE();
    case 3780:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3814);
      END_STATE();
    case 3781:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3815);
      END_STATE();
    case 3782:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3816);
      END_STATE();
    case 3783:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3817);
      END_STATE();
    case 3784:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3818);
      END_STATE();
    case 3785:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3819);
      END_STATE();
    case 3786:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3820);
      END_STATE();
    case 3787:
      if (lookahead == '_') ADVANCE(3821);
      END_STATE();
    case 3788:
      ACCEPT_TOKEN(sym_length_variable_to_string);
      END_STATE();
    case 3789:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3822);
      END_STATE();
    case 3790:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3823);
      END_STATE();
    case 3791:
      ACCEPT_TOKEN(sym_set_virtual_channel_count);
      END_STATE();
    case 3792:
      if (lookahead == '_') ADVANCE(3824);
      END_STATE();
    case 3793:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3825);
      END_STATE();
    case 3794:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3826);
      END_STATE();
    case 3795:
      if (lookahead == 'W' ||
          lookahead == 'w') ADVANCE(3827);
      END_STATE();
    case 3796:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3828);
      END_STATE();
    case 3797:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3829);
      END_STATE();
    case 3798:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3830);
      END_STATE();
    case 3799:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3831);
      END_STATE();
    case 3800:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3832);
      END_STATE();
    case 3801:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3833);
      END_STATE();
    case 3802:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3834);
      END_STATE();
    case 3803:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3835);
      END_STATE();
    case 3804:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3836);
      END_STATE();
    case 3805:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3837);
      END_STATE();
    case 3806:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3838);
      END_STATE();
    case 3807:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3839);
      END_STATE();
    case 3808:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3840);
      END_STATE();
    case 3809:
      ACCEPT_TOKEN(sym_duet_dev_type_multi_window);
      END_STATE();
    case 3810:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3841);
      END_STATE();
    case 3811:
      ACCEPT_TOKEN(sym_duet_dev_type_relay_device);
      END_STATE();
    case 3812:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3842);
      END_STATE();
    case 3813:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3843);
      END_STATE();
    case 3814:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3844);
      END_STATE();
    case 3815:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3845);
      END_STATE();
    case 3816:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3846);
      END_STATE();
    case 3817:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3847);
      END_STATE();
    case 3818:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3848);
      END_STATE();
    case 3819:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3849);
      END_STATE();
    case 3820:
      ACCEPT_TOKEN(sym_dynamic_application_device);
      END_STATE();
    case 3821:
      ADVANCE_MAP(
        'C', 3850,
        'c', 3850,
        'D', 3851,
        'd', 3851,
        'I', 3852,
        'i', 3852,
        'M', 3853,
        'm', 3853,
        'N', 3854,
        'n', 3854,
        'R', 3855,
        'r', 3855,
        'T', 3856,
        't', 3856,
      );
      END_STATE();
    case 3822:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3857);
      END_STATE();
    case 3823:
      ACCEPT_TOKEN(sym_max_last_login_info_length);
      END_STATE();
    case 3824:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3858);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3859);
      END_STATE();
    case 3825:
      if (lookahead == '_') ADVANCE(3860);
      END_STATE();
    case 3826:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3861);
      END_STATE();
    case 3827:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3862);
      END_STATE();
    case 3828:
      ACCEPT_TOKEN(sym_audit_netlinx_generic_event);
      END_STATE();
    case 3829:
      ACCEPT_TOKEN(sym_audit_netlinx_session_event);
      END_STATE();
    case 3830:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3863);
      END_STATE();
    case 3831:
      ACCEPT_TOKEN(sym_data_max_description_length);
      END_STATE();
    case 3832:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3864);
      END_STATE();
    case 3833:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3865);
      END_STATE();
    case 3834:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3866);
      END_STATE();
    case 3835:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3867);
      END_STATE();
    case 3836:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3868);
      END_STATE();
    case 3837:
      if (lookahead == '_') ADVANCE(3869);
      END_STATE();
    case 3838:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3870);
      END_STATE();
    case 3839:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3871);
      END_STATE();
    case 3840:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3872);
      END_STATE();
    case 3841:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3873);
      END_STATE();
    case 3842:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3874);
      END_STATE();
    case 3843:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3875);
      END_STATE();
    case 3844:
      ACCEPT_TOKEN(sym_duet_dev_type_sensor_device);
      END_STATE();
    case 3845:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3876);
      END_STATE();
    case 3846:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3877);
      END_STATE();
    case 3847:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3878);
      END_STATE();
    case 3848:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3879);
      END_STATE();
    case 3849:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3880);
      END_STATE();
    case 3850:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3881);
      END_STATE();
    case 3851:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3882);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3883);
      END_STATE();
    case 3852:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3884);
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3885);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3886);
      END_STATE();
    case 3853:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3887);
      END_STATE();
    case 3854:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3888);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3889);
      END_STATE();
    case 3855:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3890);
      END_STATE();
    case 3856:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3891);
      END_STATE();
    case 3857:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3892);
      END_STATE();
    case 3858:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3893);
      END_STATE();
    case 3859:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3894);
      END_STATE();
    case 3860:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3895);
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3896);
      END_STATE();
    case 3861:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3897);
      END_STATE();
    case 3862:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3898);
      END_STATE();
    case 3863:
      ACCEPT_TOKEN(sym_data_max_content_field_count);
      END_STATE();
    case 3864:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3899);
      END_STATE();
    case 3865:
      ACCEPT_TOKEN(sym_data_structure_datarecordset);
      END_STATE();
    case 3866:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3900);
      END_STATE();
    case 3867:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3901);
      END_STATE();
    case 3868:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3902);
      END_STATE();
    case 3869:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3903);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3904);
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3905);
      END_STATE();
    case 3870:
      ACCEPT_TOKEN(sym_duet_dev_type_display_device);
      END_STATE();
    case 3871:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3906);
      END_STATE();
    case 3872:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3907);
      END_STATE();
    case 3873:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3908);
      END_STATE();
    case 3874:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3909);
      END_STATE();
    case 3875:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(3910);
      END_STATE();
    case 3876:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3911);
      END_STATE();
    case 3877:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3912);
      END_STATE();
    case 3878:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3913);
      END_STATE();
    case 3879:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3914);
      END_STATE();
    case 3880:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3915);
      END_STATE();
    case 3881:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3916);
      END_STATE();
    case 3882:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3917);
      END_STATE();
    case 3883:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3918);
      END_STATE();
    case 3884:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3919);
      END_STATE();
    case 3885:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3920);
      END_STATE();
    case 3886:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3921);
      END_STATE();
    case 3887:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3922);
      END_STATE();
    case 3888:
      if (lookahead == '_') ADVANCE(3923);
      END_STATE();
    case 3889:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3924);
      END_STATE();
    case 3890:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(3925);
      END_STATE();
    case 3891:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3926);
      END_STATE();
    case 3892:
      ACCEPT_TOKEN(sym_listview_on_row_select_event);
      END_STATE();
    case 3893:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3927);
      END_STATE();
    case 3894:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3928);
      END_STATE();
    case 3895:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3929);
      END_STATE();
    case 3896:
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(3930);
      END_STATE();
    case 3897:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3931);
      END_STATE();
    case 3898:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(3932);
      END_STATE();
    case 3899:
      ACCEPT_TOKEN(sym_data_max_metadata_field_count);
      END_STATE();
    case 3900:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3933);
      END_STATE();
    case 3901:
      ACCEPT_TOKEN(sym_duet_dev_type_audio_processor);
      END_STATE();
    case 3902:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3934);
      END_STATE();
    case 3903:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3935);
      END_STATE();
    case 3904:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3936);
      END_STATE();
    case 3905:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3937);
      END_STATE();
    case 3906:
      ACCEPT_TOKEN(sym_duet_dev_type_document_camera);
      END_STATE();
    case 3907:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3938);
      END_STATE();
    case 3908:
      if (lookahead == '_') ADVANCE(3939);
      END_STATE();
    case 3909:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3940);
      END_STATE();
    case 3910:
      ACCEPT_TOKEN(sym_duet_dev_type_security_system);
      END_STATE();
    case 3911:
      ACCEPT_TOKEN(sym_duet_dev_type_slide_projector);
      END_STATE();
    case 3912:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3941);
      END_STATE();
    case 3913:
      ACCEPT_TOKEN(sym_duet_dev_type_video_processor);
      END_STATE();
    case 3914:
      ACCEPT_TOKEN(sym_duet_dev_type_video_projector);
      END_STATE();
    case 3915:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3942);
      END_STATE();
    case 3916:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3943);
      END_STATE();
    case 3917:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3944);
      END_STATE();
    case 3918:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3945);
      END_STATE();
    case 3919:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3946);
      END_STATE();
    case 3920:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3947);
      END_STATE();
    case 3921:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3948);
      END_STATE();
    case 3922:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3949);
      END_STATE();
    case 3923:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(3950);
      END_STATE();
    case 3924:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3951);
      END_STATE();
    case 3925:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3952);
      END_STATE();
    case 3926:
      if (lookahead == '_') ADVANCE(3953);
      END_STATE();
    case 3927:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3954);
      END_STATE();
    case 3928:
      if (lookahead == '6') ADVANCE(3955);
      END_STATE();
    case 3929:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3956);
      END_STATE();
    case 3930:
      ACCEPT_TOKEN(sym_source_type_neuron_subnode_pl);
      END_STATE();
    case 3931:
      ACCEPT_TOKEN(sym_tls_ignore_certificate_errors);
      END_STATE();
    case 3932:
      if (lookahead == '_') ADVANCE(3957);
      END_STATE();
    case 3933:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3958);
      END_STATE();
    case 3934:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3959);
      END_STATE();
    case 3935:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3960);
      END_STATE();
    case 3936:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3961);
      END_STATE();
    case 3937:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3962);
      END_STATE();
    case 3938:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3963);
      END_STATE();
    case 3939:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(3964);
      END_STATE();
    case 3940:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3965);
      END_STATE();
    case 3941:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3966);
      END_STATE();
    case 3942:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3967);
      END_STATE();
    case 3943:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3968);
      END_STATE();
    case 3944:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3969);
      END_STATE();
    case 3945:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3970);
      END_STATE();
    case 3946:
      if (lookahead == '2') ADVANCE(3971);
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3972);
      END_STATE();
    case 3947:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(3973);
      END_STATE();
    case 3948:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3974);
      END_STATE();
    case 3949:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(3975);
      END_STATE();
    case 3950:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3976);
      END_STATE();
    case 3951:
      if (lookahead == 'F' ||
          lookahead == 'f') ADVANCE(3977);
      END_STATE();
    case 3952:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3978);
      END_STATE();
    case 3953:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(3979);
      END_STATE();
    case 3954:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3980);
      END_STATE();
    case 3955:
      ACCEPT_TOKEN(sym_source_type_ipv4_port_mac_ipv6);
      END_STATE();
    case 3956:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3981);
      END_STATE();
    case 3957:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3982);
      END_STATE();
    case 3958:
      ACCEPT_TOKEN(sym_duet_dev_type_audio_conferencer);
      END_STATE();
    case 3959:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3983);
      END_STATE();
    case 3960:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3984);
      END_STATE();
    case 3961:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3985);
      END_STATE();
    case 3962:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3986);
      END_STATE();
    case 3963:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3987);
      END_STATE();
    case 3964:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3988);
      END_STATE();
    case 3965:
      ACCEPT_TOKEN(sym_duet_dev_type_rfidsystem_device);
      END_STATE();
    case 3966:
      ACCEPT_TOKEN(sym_duet_dev_type_video_conferencer);
      END_STATE();
    case 3967:
      ACCEPT_TOKEN(sym_duet_dev_type_volume_controller);
      END_STATE();
    case 3968:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(3989);
      END_STATE();
    case 3969:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(3990);
      END_STATE();
    case 3970:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(3991);
      END_STATE();
    case 3971:
      if (lookahead == '3') ADVANCE(3992);
      END_STATE();
    case 3972:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3993);
      END_STATE();
    case 3973:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(3994);
      END_STATE();
    case 3974:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(3995);
      END_STATE();
    case 3975:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(3996);
      END_STATE();
    case 3976:
      if (lookahead == 'V' ||
          lookahead == 'v') ADVANCE(3997);
      END_STATE();
    case 3977:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(3998);
      END_STATE();
    case 3978:
      if (lookahead == '_') ADVANCE(3999);
      END_STATE();
    case 3979:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(4000);
      END_STATE();
    case 3980:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(4001);
      END_STATE();
    case 3981:
      ACCEPT_TOKEN(sym_source_type_neuron_subnode_icsp);
      END_STATE();
    case 3982:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(4002);
      END_STATE();
    case 3983:
      ACCEPT_TOKEN(sym_duet_dev_type_audio_tuner_device);
      END_STATE();
    case 3984:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(4003);
      END_STATE();
    case 3985:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(4004);
      END_STATE();
    case 3986:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(4005);
      END_STATE();
    case 3987:
      ACCEPT_TOKEN(sym_duet_dev_type_lightsystem_device);
      END_STATE();
    case 3988:
      if (lookahead == 'U' ||
          lookahead == 'u') ADVANCE(4006);
      END_STATE();
    case 3989:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(4007);
      END_STATE();
    case 3990:
      if (lookahead == '_') ADVANCE(4008);
      END_STATE();
    case 3991:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(4009);
      END_STATE();
    case 3992:
      if (lookahead == '2') ADVANCE(4010);
      END_STATE();
    case 3993:
      if (lookahead == '_') ADVANCE(4011);
      END_STATE();
    case 3994:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4012);
      END_STATE();
    case 3995:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(4013);
      END_STATE();
    case 3996:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(4014);
      END_STATE();
    case 3997:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(4015);
      END_STATE();
    case 3998:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(4016);
      END_STATE();
    case 3999:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(4017);
      END_STATE();
    case 4000:
      ACCEPT_TOKEN(sym_internal_queue_size_index_tcp_tx);
      END_STATE();
    case 4001:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(4018);
      END_STATE();
    case 4002:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4019);
      END_STATE();
    case 4003:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(4020);
      END_STATE();
    case 4004:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(4021);
      END_STATE();
    case 4005:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4022);
      END_STATE();
    case 4006:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(4023);
      END_STATE();
    case 4007:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(4024);
      END_STATE();
    case 4008:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(4025);
      END_STATE();
    case 4009:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(4026);
      END_STATE();
    case 4010:
      if (lookahead == '_') ADVANCE(4027);
      END_STATE();
    case 4011:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(4028);
      END_STATE();
    case 4012:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(4029);
      END_STATE();
    case 4013:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(4030);
      END_STATE();
    case 4014:
      if (lookahead == '_') ADVANCE(4031);
      END_STATE();
    case 4015:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(4032);
      END_STATE();
    case 4016:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(4033);
      END_STATE();
    case 4017:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(4034);
      END_STATE();
    case 4018:
      ACCEPT_TOKEN(sym_source_type_ipv4_port_mac_address);
      END_STATE();
    case 4019:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(4035);
      END_STATE();
    case 4020:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4036);
      END_STATE();
    case 4021:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4037);
      END_STATE();
    case 4022:
      ACCEPT_TOKEN(sym_duet_dev_type_digital_media_server);
      END_STATE();
    case 4023:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(4038);
      END_STATE();
    case 4024:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(4039);
      END_STATE();
    case 4025:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(4040);
      END_STATE();
    case 4026:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(4041);
      END_STATE();
    case 4027:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(4042);
      END_STATE();
    case 4028:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(4043);
      END_STATE();
    case 4029:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(4044);
      END_STATE();
    case 4030:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(4045);
      END_STATE();
    case 4031:
      if (lookahead == 'D' ||
          lookahead == 'd') ADVANCE(4046);
      END_STATE();
    case 4032:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(4047);
      END_STATE();
    case 4033:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(4048);
      END_STATE();
    case 4034:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4049);
      END_STATE();
    case 4035:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(4050);
      END_STATE();
    case 4036:
      ACCEPT_TOKEN(sym_duet_dev_type_digital_media_decoder);
      END_STATE();
    case 4037:
      ACCEPT_TOKEN(sym_duet_dev_type_digital_media_encoder);
      END_STATE();
    case 4038:
      if (lookahead == '_') ADVANCE(4051);
      END_STATE();
    case 4039:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(4052);
      END_STATE();
    case 4040:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4053);
      END_STATE();
    case 4041:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(4054);
      END_STATE();
    case 4042:
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(4055);
      END_STATE();
    case 4043:
      ACCEPT_TOKEN(sym_internal_queue_size_index_icspip_tx);
      END_STATE();
    case 4044:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(4056);
      END_STATE();
    case 4045:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(4057);
      END_STATE();
    case 4046:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(4058);
      END_STATE();
    case 4047:
      ACCEPT_TOKEN(sym_internal_queue_size_index_ni_device);
      END_STATE();
    case 4048:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(4059);
      END_STATE();
    case 4049:
      ACCEPT_TOKEN(sym_internal_queue_size_index_route_mgr);
      END_STATE();
    case 4050:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(4060);
      END_STATE();
    case 4051:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(4061);
      END_STATE();
    case 4052:
      if (lookahead == '_') ADVANCE(4062);
      END_STATE();
    case 4053:
      ACCEPT_TOKEN(sym_internal_queue_size_index_device_mgr);
      END_STATE();
    case 4054:
      if (lookahead == '_') ADVANCE(4063);
      END_STATE();
    case 4055:
      ACCEPT_TOKEN(sym_internal_queue_size_index_icsp232_tx);
      END_STATE();
    case 4056:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4064);
      END_STATE();
    case 4057:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(4065);
      END_STATE();
    case 4058:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(4066);
      END_STATE();
    case 4059:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(4067);
      END_STATE();
    case 4060:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(4068);
      END_STATE();
    case 4061:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4069);
      END_STATE();
    case 4062:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(4070);
      END_STATE();
    case 4063:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(4071);
      END_STATE();
    case 4064:
      ACCEPT_TOKEN(sym_internal_queue_size_index_interpreter);
      END_STATE();
    case 4065:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(4072);
      END_STATE();
    case 4066:
      if (lookahead == 'P' ||
          lookahead == 'p') ADVANCE(4073);
      END_STATE();
    case 4067:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(4074);
      END_STATE();
    case 4068:
      if (lookahead == 'I' ||
          lookahead == 'i') ADVANCE(4075);
      END_STATE();
    case 4069:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(4076);
      END_STATE();
    case 4070:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(4077);
      END_STATE();
    case 4071:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(4078);
      END_STATE();
    case 4072:
      if (lookahead == '_') ADVANCE(4079);
      END_STATE();
    case 4073:
      if (lookahead == 'A' ||
          lookahead == 'a') ADVANCE(4080);
      END_STATE();
    case 4074:
      if (lookahead == '_') ADVANCE(4081);
      END_STATE();
    case 4075:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(4082);
      END_STATE();
    case 4076:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(4083);
      END_STATE();
    case 4077:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4084);
      END_STATE();
    case 4078:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4085);
      END_STATE();
    case 4079:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(4086);
      END_STATE();
    case 4080:
      if (lookahead == 'T' ||
          lookahead == 't') ADVANCE(4087);
      END_STATE();
    case 4081:
      if (lookahead == 'M' ||
          lookahead == 'm') ADVANCE(4088);
      END_STATE();
    case 4082:
      if (lookahead == 'N' ||
          lookahead == 'n') ADVANCE(4089);
      END_STATE();
    case 4083:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(4090);
      END_STATE();
    case 4084:
      ACCEPT_TOKEN(sym_internal_queue_size_index_connection_mgr);
      END_STATE();
    case 4085:
      ACCEPT_TOKEN(sym_internal_queue_size_index_diagnostic_mgr);
      END_STATE();
    case 4086:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(4091);
      END_STATE();
    case 4087:
      if (lookahead == 'C' ||
          lookahead == 'c') ADVANCE(4092);
      END_STATE();
    case 4088:
      if (lookahead == 'G' ||
          lookahead == 'g') ADVANCE(4093);
      END_STATE();
    case 4089:
      ACCEPT_TOKEN(sym_validate_netlinx_account_with_permission);
      END_STATE();
    case 4090:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(4094);
      END_STATE();
    case 4091:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4095);
      END_STATE();
    case 4092:
      if (lookahead == 'H' ||
          lookahead == 'h') ADVANCE(4096);
      END_STATE();
    case 4093:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4097);
      END_STATE();
    case 4094:
      if (lookahead == 'S' ||
          lookahead == 's') ADVANCE(4098);
      END_STATE();
    case 4095:
      ACCEPT_TOKEN(sym_internal_queue_size_index_ipconnection_mgr);
      END_STATE();
    case 4096:
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(4099);
      END_STATE();
    case 4097:
      ACCEPT_TOKEN(sym_internal_queue_size_index_notification_mgr);
      END_STATE();
    case 4098:
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(4100);
      END_STATE();
    case 4099:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4101);
      END_STATE();
    case 4100:
      if (lookahead == 'R' ||
          lookahead == 'r') ADVANCE(4102);
      END_STATE();
    case 4101:
      ACCEPT_TOKEN(sym_internal_queue_size_index_message_dispatcher);
      END_STATE();
    case 4102:
      ACCEPT_TOKEN(sym_duet_dev_type_preamp_surround_sound_processor);
      END_STATE();
    default:
      return false;
  }
}

static const TSLexMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0, .external_lex_state = 1},
  [1] = {.lex_state = 588},
  [2] = {.lex_state = 1},
  [3] = {.lex_state = 1},
  [4] = {.lex_state = 1},
  [5] = {.lex_state = 1},
  [6] = {.lex_state = 1},
  [7] = {.lex_state = 1},
  [8] = {.lex_state = 588},
  [9] = {.lex_state = 588},
  [10] = {.lex_state = 588},
  [11] = {.lex_state = 588},
  [12] = {.lex_state = 588},
  [13] = {.lex_state = 588},
  [14] = {.lex_state = 588},
  [15] = {.lex_state = 588},
  [16] = {.lex_state = 588},
  [17] = {.lex_state = 588},
  [18] = {.lex_state = 588},
  [19] = {.lex_state = 588},
  [20] = {.lex_state = 588},
  [21] = {.lex_state = 588},
  [22] = {.lex_state = 588},
  [23] = {.lex_state = 588},
  [24] = {.lex_state = 588},
  [25] = {.lex_state = 588},
  [26] = {.lex_state = 588},
  [27] = {.lex_state = 588},
  [28] = {.lex_state = 588},
  [29] = {.lex_state = 588},
  [30] = {.lex_state = 588},
  [31] = {.lex_state = 588},
  [32] = {.lex_state = 588},
  [33] = {.lex_state = 588},
  [34] = {.lex_state = 588},
  [35] = {.lex_state = 588},
  [36] = {.lex_state = 588},
  [37] = {.lex_state = 588},
  [38] = {.lex_state = 588},
  [39] = {.lex_state = 588},
  [40] = {.lex_state = 588},
  [41] = {.lex_state = 7},
  [42] = {.lex_state = 7},
  [43] = {.lex_state = 7},
  [44] = {.lex_state = 7},
  [45] = {.lex_state = 7},
  [46] = {.lex_state = 7},
  [47] = {.lex_state = 7},
  [48] = {.lex_state = 7},
  [49] = {.lex_state = 7},
  [50] = {.lex_state = 588, .external_lex_state = 1},
  [51] = {.lex_state = 588, .external_lex_state = 1},
  [52] = {.lex_state = 7},
  [53] = {.lex_state = 7},
  [54] = {.lex_state = 7},
  [55] = {.lex_state = 7},
  [56] = {.lex_state = 7},
  [57] = {.lex_state = 588, .external_lex_state = 1},
  [58] = {.lex_state = 588, .external_lex_state = 1},
  [59] = {.lex_state = 588, .external_lex_state = 1},
  [60] = {.lex_state = 588, .external_lex_state = 1},
  [61] = {.lex_state = 588, .external_lex_state = 1},
  [62] = {.lex_state = 588, .external_lex_state = 1},
  [63] = {.lex_state = 588, .external_lex_state = 1},
  [64] = {.lex_state = 588, .external_lex_state = 1},
  [65] = {.lex_state = 588, .external_lex_state = 1},
  [66] = {.lex_state = 588, .external_lex_state = 1},
  [67] = {.lex_state = 588, .external_lex_state = 1},
  [68] = {.lex_state = 588, .external_lex_state = 1},
  [69] = {.lex_state = 588, .external_lex_state = 1},
  [70] = {.lex_state = 588, .external_lex_state = 1},
  [71] = {.lex_state = 588, .external_lex_state = 1},
  [72] = {.lex_state = 588, .external_lex_state = 1},
  [73] = {.lex_state = 588, .external_lex_state = 1},
  [74] = {.lex_state = 588, .external_lex_state = 1},
  [75] = {.lex_state = 588, .external_lex_state = 1},
  [76] = {.lex_state = 588, .external_lex_state = 1},
  [77] = {.lex_state = 588, .external_lex_state = 1},
  [78] = {.lex_state = 588, .external_lex_state = 1},
  [79] = {.lex_state = 588, .external_lex_state = 1},
  [80] = {.lex_state = 588, .external_lex_state = 1},
  [81] = {.lex_state = 588, .external_lex_state = 1},
  [82] = {.lex_state = 588, .external_lex_state = 1},
  [83] = {.lex_state = 588, .external_lex_state = 1},
  [84] = {.lex_state = 588, .external_lex_state = 1},
  [85] = {.lex_state = 588, .external_lex_state = 1},
  [86] = {.lex_state = 588, .external_lex_state = 1},
  [87] = {.lex_state = 588, .external_lex_state = 1},
  [88] = {.lex_state = 588, .external_lex_state = 1},
  [89] = {.lex_state = 588, .external_lex_state = 1},
  [90] = {.lex_state = 588, .external_lex_state = 1},
  [91] = {.lex_state = 588, .external_lex_state = 1},
  [92] = {.lex_state = 588, .external_lex_state = 1},
  [93] = {.lex_state = 588, .external_lex_state = 1},
  [94] = {.lex_state = 588, .external_lex_state = 1},
  [95] = {.lex_state = 588, .external_lex_state = 1},
  [96] = {.lex_state = 588, .external_lex_state = 1},
  [97] = {.lex_state = 588, .external_lex_state = 1},
  [98] = {.lex_state = 588, .external_lex_state = 1},
  [99] = {.lex_state = 588, .external_lex_state = 1},
  [100] = {.lex_state = 588, .external_lex_state = 1},
  [101] = {.lex_state = 588, .external_lex_state = 1},
  [102] = {.lex_state = 588, .external_lex_state = 1},
  [103] = {.lex_state = 588, .external_lex_state = 1},
  [104] = {.lex_state = 588, .external_lex_state = 1},
  [105] = {.lex_state = 588, .external_lex_state = 1},
  [106] = {.lex_state = 588, .external_lex_state = 1},
  [107] = {.lex_state = 588, .external_lex_state = 1},
  [108] = {.lex_state = 588, .external_lex_state = 1},
  [109] = {.lex_state = 588, .external_lex_state = 1},
  [110] = {.lex_state = 588, .external_lex_state = 1},
  [111] = {.lex_state = 588, .external_lex_state = 1},
  [112] = {.lex_state = 588, .external_lex_state = 1},
  [113] = {.lex_state = 588, .external_lex_state = 1},
  [114] = {.lex_state = 588, .external_lex_state = 1},
  [115] = {.lex_state = 588, .external_lex_state = 1},
  [116] = {.lex_state = 588, .external_lex_state = 1},
  [117] = {.lex_state = 588, .external_lex_state = 1},
  [118] = {.lex_state = 588, .external_lex_state = 1},
  [119] = {.lex_state = 588, .external_lex_state = 1},
  [120] = {.lex_state = 588, .external_lex_state = 1},
  [121] = {.lex_state = 588, .external_lex_state = 1},
  [122] = {.lex_state = 588, .external_lex_state = 1},
  [123] = {.lex_state = 588, .external_lex_state = 1},
  [124] = {.lex_state = 588, .external_lex_state = 1},
  [125] = {.lex_state = 588, .external_lex_state = 1},
  [126] = {.lex_state = 588, .external_lex_state = 1},
  [127] = {.lex_state = 588, .external_lex_state = 1},
  [128] = {.lex_state = 588, .external_lex_state = 1},
  [129] = {.lex_state = 588, .external_lex_state = 1},
  [130] = {.lex_state = 588, .external_lex_state = 1},
  [131] = {.lex_state = 588, .external_lex_state = 1},
  [132] = {.lex_state = 588, .external_lex_state = 1},
  [133] = {.lex_state = 588, .external_lex_state = 1},
  [134] = {.lex_state = 588, .external_lex_state = 1},
  [135] = {.lex_state = 588, .external_lex_state = 1},
  [136] = {.lex_state = 588, .external_lex_state = 1},
  [137] = {.lex_state = 588, .external_lex_state = 1},
  [138] = {.lex_state = 588, .external_lex_state = 1},
  [139] = {.lex_state = 588, .external_lex_state = 1},
  [140] = {.lex_state = 588, .external_lex_state = 1},
  [141] = {.lex_state = 588, .external_lex_state = 1},
  [142] = {.lex_state = 588, .external_lex_state = 1},
  [143] = {.lex_state = 588, .external_lex_state = 1},
  [144] = {.lex_state = 588, .external_lex_state = 1},
  [145] = {.lex_state = 588, .external_lex_state = 1},
  [146] = {.lex_state = 588, .external_lex_state = 1},
  [147] = {.lex_state = 588, .external_lex_state = 1},
  [148] = {.lex_state = 588, .external_lex_state = 1},
  [149] = {.lex_state = 7, .external_lex_state = 1},
  [150] = {.lex_state = 7, .external_lex_state = 1},
  [151] = {.lex_state = 7, .external_lex_state = 1},
  [152] = {.lex_state = 7, .external_lex_state = 1},
  [153] = {.lex_state = 7, .external_lex_state = 1},
  [154] = {.lex_state = 7, .external_lex_state = 1},
  [155] = {.lex_state = 7, .external_lex_state = 1},
  [156] = {.lex_state = 7, .external_lex_state = 1},
  [157] = {.lex_state = 7, .external_lex_state = 1},
  [158] = {.lex_state = 7, .external_lex_state = 1},
  [159] = {.lex_state = 7, .external_lex_state = 1},
  [160] = {.lex_state = 7, .external_lex_state = 1},
  [161] = {.lex_state = 7, .external_lex_state = 1},
  [162] = {.lex_state = 7, .external_lex_state = 1},
  [163] = {.lex_state = 7, .external_lex_state = 1},
  [164] = {.lex_state = 7, .external_lex_state = 1},
  [165] = {.lex_state = 7, .external_lex_state = 1},
  [166] = {.lex_state = 7, .external_lex_state = 1},
  [167] = {.lex_state = 7, .external_lex_state = 1},
  [168] = {.lex_state = 7, .external_lex_state = 1},
  [169] = {.lex_state = 7, .external_lex_state = 1},
  [170] = {.lex_state = 7, .external_lex_state = 1},
  [171] = {.lex_state = 7, .external_lex_state = 1},
  [172] = {.lex_state = 7, .external_lex_state = 1},
  [173] = {.lex_state = 7, .external_lex_state = 1},
  [174] = {.lex_state = 7, .external_lex_state = 1},
  [175] = {.lex_state = 7, .external_lex_state = 1},
  [176] = {.lex_state = 7, .external_lex_state = 1},
  [177] = {.lex_state = 7, .external_lex_state = 1},
  [178] = {.lex_state = 7, .external_lex_state = 1},
  [179] = {.lex_state = 7, .external_lex_state = 1},
  [180] = {.lex_state = 7, .external_lex_state = 1},
  [181] = {.lex_state = 7, .external_lex_state = 1},
  [182] = {.lex_state = 7, .external_lex_state = 1},
  [183] = {.lex_state = 7, .external_lex_state = 1},
  [184] = {.lex_state = 7, .external_lex_state = 1},
  [185] = {.lex_state = 7, .external_lex_state = 1},
  [186] = {.lex_state = 7, .external_lex_state = 1},
  [187] = {.lex_state = 7, .external_lex_state = 1},
  [188] = {.lex_state = 7, .external_lex_state = 1},
  [189] = {.lex_state = 7, .external_lex_state = 1},
  [190] = {.lex_state = 7, .external_lex_state = 1},
  [191] = {.lex_state = 7, .external_lex_state = 1},
  [192] = {.lex_state = 7, .external_lex_state = 1},
  [193] = {.lex_state = 7, .external_lex_state = 1},
  [194] = {.lex_state = 7, .external_lex_state = 1},
  [195] = {.lex_state = 7, .external_lex_state = 1},
  [196] = {.lex_state = 588},
  [197] = {.lex_state = 588},
  [198] = {.lex_state = 588, .external_lex_state = 1},
  [199] = {.lex_state = 588, .external_lex_state = 1},
  [200] = {.lex_state = 588, .external_lex_state = 1},
  [201] = {.lex_state = 588},
  [202] = {.lex_state = 588, .external_lex_state = 1},
  [203] = {.lex_state = 588},
  [204] = {.lex_state = 588},
  [205] = {.lex_state = 588},
  [206] = {.lex_state = 588},
  [207] = {.lex_state = 588},
  [208] = {.lex_state = 588},
  [209] = {.lex_state = 588},
  [210] = {.lex_state = 588},
  [211] = {.lex_state = 588},
  [212] = {.lex_state = 588},
  [213] = {.lex_state = 588},
  [214] = {.lex_state = 588},
  [215] = {.lex_state = 588},
  [216] = {.lex_state = 588},
  [217] = {.lex_state = 588},
  [218] = {.lex_state = 588},
  [219] = {.lex_state = 588},
  [220] = {.lex_state = 588},
  [221] = {.lex_state = 588},
  [222] = {.lex_state = 588},
  [223] = {.lex_state = 588},
  [224] = {.lex_state = 588},
  [225] = {.lex_state = 588},
  [226] = {.lex_state = 588},
  [227] = {.lex_state = 588},
  [228] = {.lex_state = 588},
  [229] = {.lex_state = 588},
  [230] = {.lex_state = 588},
  [231] = {.lex_state = 588},
  [232] = {.lex_state = 588},
  [233] = {.lex_state = 588},
  [234] = {.lex_state = 588},
  [235] = {.lex_state = 588},
  [236] = {.lex_state = 588},
  [237] = {.lex_state = 588},
  [238] = {.lex_state = 588},
  [239] = {.lex_state = 588},
  [240] = {.lex_state = 588},
  [241] = {.lex_state = 588},
  [242] = {.lex_state = 588},
  [243] = {.lex_state = 588},
  [244] = {.lex_state = 588},
  [245] = {.lex_state = 588},
  [246] = {.lex_state = 588},
  [247] = {.lex_state = 588},
  [248] = {.lex_state = 588},
  [249] = {.lex_state = 588},
  [250] = {.lex_state = 588},
  [251] = {.lex_state = 588},
  [252] = {.lex_state = 588},
  [253] = {.lex_state = 588},
  [254] = {.lex_state = 588},
  [255] = {.lex_state = 588},
  [256] = {.lex_state = 588},
  [257] = {.lex_state = 588},
  [258] = {.lex_state = 588},
  [259] = {.lex_state = 588},
  [260] = {.lex_state = 588},
  [261] = {.lex_state = 588},
  [262] = {.lex_state = 588},
  [263] = {.lex_state = 588},
  [264] = {.lex_state = 588},
  [265] = {.lex_state = 588},
  [266] = {.lex_state = 588},
  [267] = {.lex_state = 588},
  [268] = {.lex_state = 588},
  [269] = {.lex_state = 588},
  [270] = {.lex_state = 588},
  [271] = {.lex_state = 588},
  [272] = {.lex_state = 588},
  [273] = {.lex_state = 588},
  [274] = {.lex_state = 588},
  [275] = {.lex_state = 588},
  [276] = {.lex_state = 588},
  [277] = {.lex_state = 588},
  [278] = {.lex_state = 588},
  [279] = {.lex_state = 588},
  [280] = {.lex_state = 588},
  [281] = {.lex_state = 588},
  [282] = {.lex_state = 588},
  [283] = {.lex_state = 588},
  [284] = {.lex_state = 588},
  [285] = {.lex_state = 588},
  [286] = {.lex_state = 588},
  [287] = {.lex_state = 588},
  [288] = {.lex_state = 588},
  [289] = {.lex_state = 588},
  [290] = {.lex_state = 588},
  [291] = {.lex_state = 588},
  [292] = {.lex_state = 588},
  [293] = {.lex_state = 588},
  [294] = {.lex_state = 588},
  [295] = {.lex_state = 588},
  [296] = {.lex_state = 588},
  [297] = {.lex_state = 588},
  [298] = {.lex_state = 588},
  [299] = {.lex_state = 588},
  [300] = {.lex_state = 588},
  [301] = {.lex_state = 588},
  [302] = {.lex_state = 588},
  [303] = {.lex_state = 588},
  [304] = {.lex_state = 588},
  [305] = {.lex_state = 588},
  [306] = {.lex_state = 588},
  [307] = {.lex_state = 588},
  [308] = {.lex_state = 588},
  [309] = {.lex_state = 588},
  [310] = {.lex_state = 588},
  [311] = {.lex_state = 588},
  [312] = {.lex_state = 588},
  [313] = {.lex_state = 588},
  [314] = {.lex_state = 588},
  [315] = {.lex_state = 588},
  [316] = {.lex_state = 588},
  [317] = {.lex_state = 588},
  [318] = {.lex_state = 588},
  [319] = {.lex_state = 588},
  [320] = {.lex_state = 588},
  [321] = {.lex_state = 588},
  [322] = {.lex_state = 588},
  [323] = {.lex_state = 588},
  [324] = {.lex_state = 588},
  [325] = {.lex_state = 588},
  [326] = {.lex_state = 588},
  [327] = {.lex_state = 588},
  [328] = {.lex_state = 588},
  [329] = {.lex_state = 588},
  [330] = {.lex_state = 588},
  [331] = {.lex_state = 588},
  [332] = {.lex_state = 588},
  [333] = {.lex_state = 588},
  [334] = {.lex_state = 588},
  [335] = {.lex_state = 588},
  [336] = {.lex_state = 588},
  [337] = {.lex_state = 588},
  [338] = {.lex_state = 588},
  [339] = {.lex_state = 588},
  [340] = {.lex_state = 588},
  [341] = {.lex_state = 588},
  [342] = {.lex_state = 588},
  [343] = {.lex_state = 588},
  [344] = {.lex_state = 588},
  [345] = {.lex_state = 588},
  [346] = {.lex_state = 588},
  [347] = {.lex_state = 588},
  [348] = {.lex_state = 588},
  [349] = {.lex_state = 588},
  [350] = {.lex_state = 588},
  [351] = {.lex_state = 588},
  [352] = {.lex_state = 588},
  [353] = {.lex_state = 588},
  [354] = {.lex_state = 588},
  [355] = {.lex_state = 588},
  [356] = {.lex_state = 588},
  [357] = {.lex_state = 588},
  [358] = {.lex_state = 588},
  [359] = {.lex_state = 588},
  [360] = {.lex_state = 588},
  [361] = {.lex_state = 588},
  [362] = {.lex_state = 588},
  [363] = {.lex_state = 588},
  [364] = {.lex_state = 588},
  [365] = {.lex_state = 588},
  [366] = {.lex_state = 588},
  [367] = {.lex_state = 588},
  [368] = {.lex_state = 588},
  [369] = {.lex_state = 588},
  [370] = {.lex_state = 588},
  [371] = {.lex_state = 588},
  [372] = {.lex_state = 588},
  [373] = {.lex_state = 588},
  [374] = {.lex_state = 588},
  [375] = {.lex_state = 588},
  [376] = {.lex_state = 588},
  [377] = {.lex_state = 588},
  [378] = {.lex_state = 588},
  [379] = {.lex_state = 588},
  [380] = {.lex_state = 588},
  [381] = {.lex_state = 588},
  [382] = {.lex_state = 588},
  [383] = {.lex_state = 588},
  [384] = {.lex_state = 588},
  [385] = {.lex_state = 588},
  [386] = {.lex_state = 588},
  [387] = {.lex_state = 588},
  [388] = {.lex_state = 588},
  [389] = {.lex_state = 588},
  [390] = {.lex_state = 588},
  [391] = {.lex_state = 588},
  [392] = {.lex_state = 588},
  [393] = {.lex_state = 588},
  [394] = {.lex_state = 588},
  [395] = {.lex_state = 588},
  [396] = {.lex_state = 588},
  [397] = {.lex_state = 588},
  [398] = {.lex_state = 588},
  [399] = {.lex_state = 588},
  [400] = {.lex_state = 588},
  [401] = {.lex_state = 588},
  [402] = {.lex_state = 588},
  [403] = {.lex_state = 588},
  [404] = {.lex_state = 588},
  [405] = {.lex_state = 588},
  [406] = {.lex_state = 588},
  [407] = {.lex_state = 588},
  [408] = {.lex_state = 588},
  [409] = {.lex_state = 588},
  [410] = {.lex_state = 588},
  [411] = {.lex_state = 588},
  [412] = {.lex_state = 588},
  [413] = {.lex_state = 588},
  [414] = {.lex_state = 588},
  [415] = {.lex_state = 588},
  [416] = {.lex_state = 588},
  [417] = {.lex_state = 588},
  [418] = {.lex_state = 588},
  [419] = {.lex_state = 588},
  [420] = {.lex_state = 588},
  [421] = {.lex_state = 588},
  [422] = {.lex_state = 588},
  [423] = {.lex_state = 588},
  [424] = {.lex_state = 588},
  [425] = {.lex_state = 588},
  [426] = {.lex_state = 588},
  [427] = {.lex_state = 588},
  [428] = {.lex_state = 588},
  [429] = {.lex_state = 588},
  [430] = {.lex_state = 588},
  [431] = {.lex_state = 588},
  [432] = {.lex_state = 588},
  [433] = {.lex_state = 588},
  [434] = {.lex_state = 588},
  [435] = {.lex_state = 588},
  [436] = {.lex_state = 588},
  [437] = {.lex_state = 588},
  [438] = {.lex_state = 588},
  [439] = {.lex_state = 588},
  [440] = {.lex_state = 588},
  [441] = {.lex_state = 588},
  [442] = {.lex_state = 588},
  [443] = {.lex_state = 588},
  [444] = {.lex_state = 588},
  [445] = {.lex_state = 588},
  [446] = {.lex_state = 588},
  [447] = {.lex_state = 588},
  [448] = {.lex_state = 588},
  [449] = {.lex_state = 588},
  [450] = {.lex_state = 588},
  [451] = {.lex_state = 588},
  [452] = {.lex_state = 588},
  [453] = {.lex_state = 588},
  [454] = {.lex_state = 588},
  [455] = {.lex_state = 588},
  [456] = {.lex_state = 588},
  [457] = {.lex_state = 588},
  [458] = {.lex_state = 588},
  [459] = {.lex_state = 588},
  [460] = {.lex_state = 7, .external_lex_state = 1},
  [461] = {.lex_state = 7, .external_lex_state = 1},
  [462] = {.lex_state = 7},
  [463] = {.lex_state = 7},
  [464] = {.lex_state = 7},
  [465] = {.lex_state = 7},
  [466] = {.lex_state = 7},
  [467] = {.lex_state = 7},
  [468] = {.lex_state = 7},
  [469] = {.lex_state = 7},
  [470] = {.lex_state = 7},
  [471] = {.lex_state = 7},
  [472] = {.lex_state = 7},
  [473] = {.lex_state = 7},
  [474] = {.lex_state = 7},
  [475] = {.lex_state = 7},
  [476] = {.lex_state = 7},
  [477] = {.lex_state = 7},
  [478] = {.lex_state = 7},
  [479] = {.lex_state = 7},
  [480] = {.lex_state = 7},
  [481] = {.lex_state = 7},
  [482] = {.lex_state = 7},
  [483] = {.lex_state = 7},
  [484] = {.lex_state = 7},
  [485] = {.lex_state = 7},
  [486] = {.lex_state = 7},
  [487] = {.lex_state = 7},
  [488] = {.lex_state = 7},
  [489] = {.lex_state = 7},
  [490] = {.lex_state = 7},
  [491] = {.lex_state = 7},
  [492] = {.lex_state = 7},
  [493] = {.lex_state = 7},
  [494] = {.lex_state = 7},
  [495] = {.lex_state = 7},
  [496] = {.lex_state = 7},
  [497] = {.lex_state = 7},
  [498] = {.lex_state = 7},
  [499] = {.lex_state = 7},
  [500] = {.lex_state = 7},
  [501] = {.lex_state = 7},
  [502] = {.lex_state = 7},
  [503] = {.lex_state = 7},
  [504] = {.lex_state = 7},
  [505] = {.lex_state = 7},
  [506] = {.lex_state = 7},
  [507] = {.lex_state = 7},
  [508] = {.lex_state = 7},
  [509] = {.lex_state = 7},
  [510] = {.lex_state = 7},
  [511] = {.lex_state = 7},
  [512] = {.lex_state = 7},
  [513] = {.lex_state = 7},
  [514] = {.lex_state = 7},
  [515] = {.lex_state = 7},
  [516] = {.lex_state = 7},
  [517] = {.lex_state = 7},
  [518] = {.lex_state = 7},
  [519] = {.lex_state = 7},
  [520] = {.lex_state = 7},
  [521] = {.lex_state = 7},
  [522] = {.lex_state = 7},
  [523] = {.lex_state = 7},
  [524] = {.lex_state = 7},
  [525] = {.lex_state = 7},
  [526] = {.lex_state = 7},
  [527] = {.lex_state = 7},
  [528] = {.lex_state = 7},
  [529] = {.lex_state = 7},
  [530] = {.lex_state = 9},
  [531] = {.lex_state = 9},
  [532] = {.lex_state = 9},
  [533] = {.lex_state = 7},
  [534] = {.lex_state = 7},
  [535] = {.lex_state = 9},
  [536] = {.lex_state = 7},
  [537] = {.lex_state = 7},
  [538] = {.lex_state = 7},
  [539] = {.lex_state = 7},
  [540] = {.lex_state = 7},
  [541] = {.lex_state = 7},
  [542] = {.lex_state = 7},
  [543] = {.lex_state = 7},
  [544] = {.lex_state = 7},
  [545] = {.lex_state = 7},
  [546] = {.lex_state = 7},
  [547] = {.lex_state = 7},
  [548] = {.lex_state = 7},
  [549] = {.lex_state = 7},
  [550] = {.lex_state = 7},
  [551] = {.lex_state = 7},
  [552] = {.lex_state = 7},
  [553] = {.lex_state = 7},
  [554] = {.lex_state = 9},
  [555] = {.lex_state = 7},
  [556] = {.lex_state = 9},
  [557] = {.lex_state = 9},
  [558] = {.lex_state = 9},
  [559] = {.lex_state = 9},
  [560] = {.lex_state = 9},
  [561] = {.lex_state = 9},
  [562] = {.lex_state = 9},
  [563] = {.lex_state = 7},
  [564] = {.lex_state = 7},
  [565] = {.lex_state = 9},
  [566] = {.lex_state = 9},
  [567] = {.lex_state = 9},
  [568] = {.lex_state = 7},
  [569] = {.lex_state = 7},
  [570] = {.lex_state = 8},
  [571] = {.lex_state = 8},
  [572] = {.lex_state = 8, .external_lex_state = 1},
  [573] = {.lex_state = 8},
  [574] = {.lex_state = 8},
  [575] = {.lex_state = 8},
  [576] = {.lex_state = 8},
  [577] = {.lex_state = 8},
  [578] = {.lex_state = 8},
  [579] = {.lex_state = 8},
  [580] = {.lex_state = 8},
  [581] = {.lex_state = 8},
  [582] = {.lex_state = 8},
  [583] = {.lex_state = 8},
  [584] = {.lex_state = 8},
  [585] = {.lex_state = 8},
  [586] = {.lex_state = 8},
  [587] = {.lex_state = 8},
  [588] = {.lex_state = 8},
  [589] = {.lex_state = 8},
  [590] = {.lex_state = 8},
  [591] = {.lex_state = 8},
  [592] = {.lex_state = 8},
  [593] = {.lex_state = 8},
  [594] = {.lex_state = 8},
  [595] = {.lex_state = 8},
  [596] = {.lex_state = 8},
  [597] = {.lex_state = 8},
  [598] = {.lex_state = 8},
  [599] = {.lex_state = 8},
  [600] = {.lex_state = 8},
  [601] = {.lex_state = 8},
  [602] = {.lex_state = 8},
  [603] = {.lex_state = 8},
  [604] = {.lex_state = 8},
  [605] = {.lex_state = 8},
  [606] = {.lex_state = 8},
  [607] = {.lex_state = 8},
  [608] = {.lex_state = 8},
  [609] = {.lex_state = 8},
  [610] = {.lex_state = 8},
  [611] = {.lex_state = 8},
  [612] = {.lex_state = 8},
  [613] = {.lex_state = 8},
  [614] = {.lex_state = 8},
  [615] = {.lex_state = 8},
  [616] = {.lex_state = 8},
  [617] = {.lex_state = 8},
  [618] = {.lex_state = 8},
  [619] = {.lex_state = 8},
  [620] = {.lex_state = 8},
  [621] = {.lex_state = 8},
  [622] = {.lex_state = 8},
  [623] = {.lex_state = 8},
  [624] = {.lex_state = 8},
  [625] = {.lex_state = 8},
  [626] = {.lex_state = 8},
  [627] = {.lex_state = 8},
  [628] = {.lex_state = 8},
  [629] = {.lex_state = 8},
  [630] = {.lex_state = 8},
  [631] = {.lex_state = 8},
  [632] = {.lex_state = 9},
  [633] = {.lex_state = 8},
  [634] = {.lex_state = 8},
  [635] = {.lex_state = 8},
  [636] = {.lex_state = 8},
  [637] = {.lex_state = 8},
  [638] = {.lex_state = 2},
  [639] = {.lex_state = 8},
  [640] = {.lex_state = 2},
  [641] = {.lex_state = 8},
  [642] = {.lex_state = 8},
  [643] = {.lex_state = 8},
  [644] = {.lex_state = 8},
  [645] = {.lex_state = 8},
  [646] = {.lex_state = 8},
  [647] = {.lex_state = 8},
  [648] = {.lex_state = 8},
  [649] = {.lex_state = 8},
  [650] = {.lex_state = 8},
  [651] = {.lex_state = 8},
  [652] = {.lex_state = 8},
  [653] = {.lex_state = 8},
  [654] = {.lex_state = 8},
  [655] = {.lex_state = 8},
  [656] = {.lex_state = 8},
  [657] = {.lex_state = 8},
  [658] = {.lex_state = 8},
  [659] = {.lex_state = 8},
  [660] = {.lex_state = 8},
  [661] = {.lex_state = 8},
  [662] = {.lex_state = 8},
  [663] = {.lex_state = 8},
  [664] = {.lex_state = 8, .external_lex_state = 1},
  [665] = {.lex_state = 8},
  [666] = {.lex_state = 8, .external_lex_state = 1},
  [667] = {.lex_state = 8},
  [668] = {.lex_state = 8, .external_lex_state = 1},
  [669] = {.lex_state = 8},
  [670] = {.lex_state = 8},
  [671] = {.lex_state = 8},
  [672] = {.lex_state = 8},
  [673] = {.lex_state = 8},
  [674] = {.lex_state = 8},
  [675] = {.lex_state = 8},
  [676] = {.lex_state = 8},
  [677] = {.lex_state = 8},
  [678] = {.lex_state = 8},
  [679] = {.lex_state = 8},
  [680] = {.lex_state = 8},
  [681] = {.lex_state = 8},
  [682] = {.lex_state = 8},
  [683] = {.lex_state = 8},
  [684] = {.lex_state = 8},
  [685] = {.lex_state = 8},
  [686] = {.lex_state = 8},
  [687] = {.lex_state = 8},
  [688] = {.lex_state = 8},
  [689] = {.lex_state = 8},
  [690] = {.lex_state = 8},
  [691] = {.lex_state = 8},
  [692] = {.lex_state = 8},
  [693] = {.lex_state = 8},
  [694] = {.lex_state = 8},
  [695] = {.lex_state = 8},
  [696] = {.lex_state = 8},
  [697] = {.lex_state = 8},
  [698] = {.lex_state = 8},
  [699] = {.lex_state = 8},
  [700] = {.lex_state = 8},
  [701] = {.lex_state = 8},
  [702] = {.lex_state = 8},
  [703] = {.lex_state = 8},
  [704] = {.lex_state = 8},
  [705] = {.lex_state = 8},
  [706] = {.lex_state = 8},
  [707] = {.lex_state = 8},
  [708] = {.lex_state = 8},
  [709] = {.lex_state = 8},
  [710] = {.lex_state = 8},
  [711] = {.lex_state = 8},
  [712] = {.lex_state = 8},
  [713] = {.lex_state = 8},
  [714] = {.lex_state = 8},
  [715] = {.lex_state = 8},
  [716] = {.lex_state = 8},
  [717] = {.lex_state = 8},
  [718] = {.lex_state = 8},
  [719] = {.lex_state = 8},
  [720] = {.lex_state = 8},
  [721] = {.lex_state = 8},
  [722] = {.lex_state = 8},
  [723] = {.lex_state = 8},
  [724] = {.lex_state = 8},
  [725] = {.lex_state = 8},
  [726] = {.lex_state = 8},
  [727] = {.lex_state = 8},
  [728] = {.lex_state = 8},
  [729] = {.lex_state = 8},
  [730] = {.lex_state = 8},
  [731] = {.lex_state = 8},
  [732] = {.lex_state = 8},
  [733] = {.lex_state = 8},
  [734] = {.lex_state = 8},
  [735] = {.lex_state = 8},
  [736] = {.lex_state = 8},
  [737] = {.lex_state = 8},
  [738] = {.lex_state = 8},
  [739] = {.lex_state = 8},
  [740] = {.lex_state = 8},
  [741] = {.lex_state = 8},
  [742] = {.lex_state = 8},
  [743] = {.lex_state = 8},
  [744] = {.lex_state = 8},
  [745] = {.lex_state = 8},
  [746] = {.lex_state = 8},
  [747] = {.lex_state = 8},
  [748] = {.lex_state = 8},
  [749] = {.lex_state = 8},
  [750] = {.lex_state = 8},
  [751] = {.lex_state = 8},
  [752] = {.lex_state = 8},
  [753] = {.lex_state = 8},
  [754] = {.lex_state = 8},
  [755] = {.lex_state = 8},
  [756] = {.lex_state = 8},
  [757] = {.lex_state = 8},
  [758] = {.lex_state = 8},
  [759] = {.lex_state = 8},
  [760] = {.lex_state = 8},
  [761] = {.lex_state = 8},
  [762] = {.lex_state = 8},
  [763] = {.lex_state = 8},
  [764] = {.lex_state = 8},
  [765] = {.lex_state = 8},
  [766] = {.lex_state = 8},
  [767] = {.lex_state = 8},
  [768] = {.lex_state = 8},
  [769] = {.lex_state = 8},
  [770] = {.lex_state = 8},
  [771] = {.lex_state = 8},
  [772] = {.lex_state = 8},
  [773] = {.lex_state = 8},
  [774] = {.lex_state = 8},
  [775] = {.lex_state = 8},
  [776] = {.lex_state = 8},
  [777] = {.lex_state = 8},
  [778] = {.lex_state = 8},
  [779] = {.lex_state = 8},
  [780] = {.lex_state = 8},
  [781] = {.lex_state = 8},
  [782] = {.lex_state = 8},
  [783] = {.lex_state = 8},
  [784] = {.lex_state = 8},
  [785] = {.lex_state = 8},
  [786] = {.lex_state = 8},
  [787] = {.lex_state = 8},
  [788] = {.lex_state = 8},
  [789] = {.lex_state = 8},
  [790] = {.lex_state = 8},
  [791] = {.lex_state = 8},
  [792] = {.lex_state = 8},
  [793] = {.lex_state = 8},
  [794] = {.lex_state = 8},
  [795] = {.lex_state = 8},
  [796] = {.lex_state = 8},
  [797] = {.lex_state = 8},
  [798] = {.lex_state = 8},
  [799] = {.lex_state = 8},
  [800] = {.lex_state = 8},
  [801] = {.lex_state = 8},
  [802] = {.lex_state = 8},
  [803] = {.lex_state = 8},
  [804] = {.lex_state = 8},
  [805] = {.lex_state = 8},
  [806] = {.lex_state = 8},
  [807] = {.lex_state = 8},
  [808] = {.lex_state = 8},
  [809] = {.lex_state = 8},
  [810] = {.lex_state = 8},
  [811] = {.lex_state = 8},
  [812] = {.lex_state = 8},
  [813] = {.lex_state = 8},
  [814] = {.lex_state = 8},
  [815] = {.lex_state = 8},
  [816] = {.lex_state = 8},
  [817] = {.lex_state = 8},
  [818] = {.lex_state = 8},
  [819] = {.lex_state = 8},
  [820] = {.lex_state = 8},
  [821] = {.lex_state = 8},
  [822] = {.lex_state = 8},
  [823] = {.lex_state = 8},
  [824] = {.lex_state = 8},
  [825] = {.lex_state = 8},
  [826] = {.lex_state = 8},
  [827] = {.lex_state = 8},
  [828] = {.lex_state = 8},
  [829] = {.lex_state = 8},
  [830] = {.lex_state = 8},
  [831] = {.lex_state = 8},
  [832] = {.lex_state = 8},
  [833] = {.lex_state = 8},
  [834] = {.lex_state = 8},
  [835] = {.lex_state = 8},
  [836] = {.lex_state = 8},
  [837] = {.lex_state = 8},
  [838] = {.lex_state = 8},
  [839] = {.lex_state = 8},
  [840] = {.lex_state = 8},
  [841] = {.lex_state = 8},
  [842] = {.lex_state = 8},
  [843] = {.lex_state = 8},
  [844] = {.lex_state = 8},
  [845] = {.lex_state = 8},
  [846] = {.lex_state = 8},
  [847] = {.lex_state = 8},
  [848] = {.lex_state = 8},
  [849] = {.lex_state = 8},
  [850] = {.lex_state = 8},
  [851] = {.lex_state = 8},
  [852] = {.lex_state = 8},
  [853] = {.lex_state = 8},
  [854] = {.lex_state = 8},
  [855] = {.lex_state = 8},
  [856] = {.lex_state = 8},
  [857] = {.lex_state = 8},
  [858] = {.lex_state = 8},
  [859] = {.lex_state = 8},
  [860] = {.lex_state = 8},
  [861] = {.lex_state = 8},
  [862] = {.lex_state = 8},
  [863] = {.lex_state = 8},
  [864] = {.lex_state = 8},
  [865] = {.lex_state = 8},
  [866] = {.lex_state = 8},
  [867] = {.lex_state = 8},
  [868] = {.lex_state = 8},
  [869] = {.lex_state = 8},
  [870] = {.lex_state = 8},
  [871] = {.lex_state = 8},
  [872] = {.lex_state = 8},
  [873] = {.lex_state = 8},
  [874] = {.lex_state = 8},
  [875] = {.lex_state = 8},
  [876] = {.lex_state = 8},
  [877] = {.lex_state = 8},
  [878] = {.lex_state = 8},
  [879] = {.lex_state = 8},
  [880] = {.lex_state = 8},
  [881] = {.lex_state = 8},
  [882] = {.lex_state = 8},
  [883] = {.lex_state = 8},
  [884] = {.lex_state = 8},
  [885] = {.lex_state = 8},
  [886] = {.lex_state = 8},
  [887] = {.lex_state = 8},
  [888] = {.lex_state = 8},
  [889] = {.lex_state = 8},
  [890] = {.lex_state = 8},
  [891] = {.lex_state = 8},
  [892] = {.lex_state = 8},
  [893] = {.lex_state = 8},
  [894] = {.lex_state = 8},
  [895] = {.lex_state = 8},
  [896] = {.lex_state = 8},
  [897] = {.lex_state = 8},
  [898] = {.lex_state = 8},
  [899] = {.lex_state = 8},
  [900] = {.lex_state = 8},
  [901] = {.lex_state = 8},
  [902] = {.lex_state = 8},
  [903] = {.lex_state = 8},
  [904] = {.lex_state = 8},
  [905] = {.lex_state = 8},
  [906] = {.lex_state = 8},
  [907] = {.lex_state = 8},
  [908] = {.lex_state = 8},
  [909] = {.lex_state = 8},
  [910] = {.lex_state = 8},
  [911] = {.lex_state = 8},
  [912] = {.lex_state = 8},
  [913] = {.lex_state = 8},
  [914] = {.lex_state = 8},
  [915] = {.lex_state = 8},
  [916] = {.lex_state = 8},
  [917] = {.lex_state = 8},
  [918] = {.lex_state = 8},
  [919] = {.lex_state = 8},
  [920] = {.lex_state = 8},
  [921] = {.lex_state = 8},
  [922] = {.lex_state = 8},
  [923] = {.lex_state = 8},
  [924] = {.lex_state = 8},
  [925] = {.lex_state = 8},
  [926] = {.lex_state = 8},
  [927] = {.lex_state = 8},
  [928] = {.lex_state = 8},
  [929] = {.lex_state = 8},
  [930] = {.lex_state = 8},
  [931] = {.lex_state = 8},
  [932] = {.lex_state = 8},
  [933] = {.lex_state = 8},
  [934] = {.lex_state = 8},
  [935] = {.lex_state = 8},
  [936] = {.lex_state = 8},
  [937] = {.lex_state = 8},
  [938] = {.lex_state = 8},
  [939] = {.lex_state = 8},
  [940] = {.lex_state = 8},
  [941] = {.lex_state = 8},
  [942] = {.lex_state = 8},
  [943] = {.lex_state = 8},
  [944] = {.lex_state = 8},
  [945] = {.lex_state = 8},
  [946] = {.lex_state = 8},
  [947] = {.lex_state = 8},
  [948] = {.lex_state = 8},
  [949] = {.lex_state = 8},
  [950] = {.lex_state = 8},
  [951] = {.lex_state = 8},
  [952] = {.lex_state = 8},
  [953] = {.lex_state = 8},
  [954] = {.lex_state = 8},
  [955] = {.lex_state = 8},
  [956] = {.lex_state = 8},
  [957] = {.lex_state = 8},
  [958] = {.lex_state = 8},
  [959] = {.lex_state = 8},
  [960] = {.lex_state = 8},
  [961] = {.lex_state = 11},
  [962] = {.lex_state = 11},
  [963] = {.lex_state = 4},
  [964] = {.lex_state = 4},
  [965] = {.lex_state = 11},
  [966] = {.lex_state = 11},
  [967] = {.lex_state = 11},
  [968] = {.lex_state = 11},
  [969] = {.lex_state = 11},
  [970] = {.lex_state = 11},
  [971] = {.lex_state = 11},
  [972] = {.lex_state = 11},
  [973] = {.lex_state = 11},
  [974] = {.lex_state = 11},
  [975] = {.lex_state = 11},
  [976] = {.lex_state = 11},
  [977] = {.lex_state = 11},
  [978] = {.lex_state = 11},
  [979] = {.lex_state = 11},
  [980] = {.lex_state = 11},
  [981] = {.lex_state = 11},
  [982] = {.lex_state = 11},
  [983] = {.lex_state = 11},
  [984] = {.lex_state = 11},
  [985] = {.lex_state = 11},
  [986] = {.lex_state = 11},
  [987] = {.lex_state = 11},
  [988] = {.lex_state = 11},
  [989] = {.lex_state = 11},
  [990] = {.lex_state = 11},
  [991] = {.lex_state = 11},
  [992] = {.lex_state = 11},
  [993] = {.lex_state = 11},
  [994] = {.lex_state = 11},
  [995] = {.lex_state = 11},
  [996] = {.lex_state = 11},
  [997] = {.lex_state = 11},
  [998] = {.lex_state = 11},
  [999] = {.lex_state = 11},
  [1000] = {.lex_state = 11},
  [1001] = {.lex_state = 11},
  [1002] = {.lex_state = 11},
  [1003] = {.lex_state = 11},
  [1004] = {.lex_state = 11},
  [1005] = {.lex_state = 10},
  [1006] = {.lex_state = 11},
  [1007] = {.lex_state = 11},
  [1008] = {.lex_state = 11},
  [1009] = {.lex_state = 11},
  [1010] = {.lex_state = 11},
  [1011] = {.lex_state = 11},
  [1012] = {.lex_state = 11},
  [1013] = {.lex_state = 11},
  [1014] = {.lex_state = 11},
  [1015] = {.lex_state = 11},
  [1016] = {.lex_state = 11},
  [1017] = {.lex_state = 11},
  [1018] = {.lex_state = 11},
  [1019] = {.lex_state = 11},
  [1020] = {.lex_state = 11},
  [1021] = {.lex_state = 11},
  [1022] = {.lex_state = 11},
  [1023] = {.lex_state = 11},
  [1024] = {.lex_state = 11},
  [1025] = {.lex_state = 11},
  [1026] = {.lex_state = 11},
  [1027] = {.lex_state = 11},
  [1028] = {.lex_state = 11},
  [1029] = {.lex_state = 11},
  [1030] = {.lex_state = 11},
  [1031] = {.lex_state = 11},
  [1032] = {.lex_state = 11},
  [1033] = {.lex_state = 11},
  [1034] = {.lex_state = 11},
  [1035] = {.lex_state = 11},
  [1036] = {.lex_state = 11},
  [1037] = {.lex_state = 13, .external_lex_state = 1},
  [1038] = {.lex_state = 11},
  [1039] = {.lex_state = 11},
  [1040] = {.lex_state = 11},
  [1041] = {.lex_state = 11},
  [1042] = {.lex_state = 12},
  [1043] = {.lex_state = 12},
  [1044] = {.lex_state = 12},
  [1045] = {.lex_state = 12},
  [1046] = {.lex_state = 12},
  [1047] = {.lex_state = 12},
  [1048] = {.lex_state = 11},
  [1049] = {.lex_state = 11},
  [1050] = {.lex_state = 11},
  [1051] = {.lex_state = 12},
  [1052] = {.lex_state = 11},
  [1053] = {.lex_state = 12},
  [1054] = {.lex_state = 11},
  [1055] = {.lex_state = 11},
  [1056] = {.lex_state = 12},
  [1057] = {.lex_state = 11},
  [1058] = {.lex_state = 16},
  [1059] = {.lex_state = 16},
  [1060] = {.lex_state = 11},
  [1061] = {.lex_state = 16},
  [1062] = {.lex_state = 11},
  [1063] = {.lex_state = 11},
  [1064] = {.lex_state = 11},
  [1065] = {.lex_state = 12},
  [1066] = {.lex_state = 12},
  [1067] = {.lex_state = 11},
  [1068] = {.lex_state = 11},
  [1069] = {.lex_state = 12},
  [1070] = {.lex_state = 12},
  [1071] = {.lex_state = 11},
  [1072] = {.lex_state = 11},
  [1073] = {.lex_state = 11},
  [1074] = {.lex_state = 11},
  [1075] = {.lex_state = 11},
  [1076] = {.lex_state = 12},
  [1077] = {.lex_state = 11},
  [1078] = {.lex_state = 11},
  [1079] = {.lex_state = 12},
  [1080] = {.lex_state = 11},
  [1081] = {.lex_state = 12},
  [1082] = {.lex_state = 11},
  [1083] = {.lex_state = 11},
  [1084] = {.lex_state = 16},
  [1085] = {.lex_state = 12},
  [1086] = {.lex_state = 12},
  [1087] = {.lex_state = 12},
  [1088] = {.lex_state = 12},
  [1089] = {.lex_state = 11},
  [1090] = {.lex_state = 12},
  [1091] = {.lex_state = 12},
  [1092] = {.lex_state = 12},
  [1093] = {.lex_state = 12},
  [1094] = {.lex_state = 12},
  [1095] = {.lex_state = 16},
  [1096] = {.lex_state = 16},
  [1097] = {.lex_state = 12},
  [1098] = {.lex_state = 11},
  [1099] = {.lex_state = 12},
  [1100] = {.lex_state = 11},
  [1101] = {.lex_state = 12, .external_lex_state = 1},
  [1102] = {.lex_state = 12},
  [1103] = {.lex_state = 12},
  [1104] = {.lex_state = 12, .external_lex_state = 1},
  [1105] = {.lex_state = 12, .external_lex_state = 1},
  [1106] = {.lex_state = 12, .external_lex_state = 1},
  [1107] = {.lex_state = 12, .external_lex_state = 1},
  [1108] = {.lex_state = 12, .external_lex_state = 1},
  [1109] = {.lex_state = 12, .external_lex_state = 1},
  [1110] = {.lex_state = 12, .external_lex_state = 1},
  [1111] = {.lex_state = 12},
  [1112] = {.lex_state = 12},
  [1113] = {.lex_state = 12, .external_lex_state = 1},
  [1114] = {.lex_state = 12, .external_lex_state = 1},
  [1115] = {.lex_state = 12},
  [1116] = {.lex_state = 12, .external_lex_state = 1},
  [1117] = {.lex_state = 12, .external_lex_state = 1},
  [1118] = {.lex_state = 12, .external_lex_state = 1},
  [1119] = {.lex_state = 12, .external_lex_state = 1},
  [1120] = {.lex_state = 12},
  [1121] = {.lex_state = 12},
  [1122] = {.lex_state = 12},
  [1123] = {.lex_state = 12, .external_lex_state = 1},
  [1124] = {.lex_state = 12, .external_lex_state = 1},
  [1125] = {.lex_state = 12, .external_lex_state = 1},
  [1126] = {.lex_state = 12},
  [1127] = {.lex_state = 12, .external_lex_state = 1},
  [1128] = {.lex_state = 12},
  [1129] = {.lex_state = 12},
  [1130] = {.lex_state = 12, .external_lex_state = 1},
  [1131] = {.lex_state = 12, .external_lex_state = 1},
  [1132] = {.lex_state = 12},
  [1133] = {.lex_state = 12},
  [1134] = {.lex_state = 12, .external_lex_state = 1},
  [1135] = {.lex_state = 12, .external_lex_state = 1},
  [1136] = {.lex_state = 12, .external_lex_state = 1},
  [1137] = {.lex_state = 12, .external_lex_state = 1},
  [1138] = {.lex_state = 12, .external_lex_state = 1},
  [1139] = {.lex_state = 12, .external_lex_state = 1},
  [1140] = {.lex_state = 12, .external_lex_state = 1},
  [1141] = {.lex_state = 12},
  [1142] = {.lex_state = 12},
  [1143] = {.lex_state = 12, .external_lex_state = 1},
  [1144] = {.lex_state = 12, .external_lex_state = 1},
  [1145] = {.lex_state = 12},
  [1146] = {.lex_state = 12},
  [1147] = {.lex_state = 12},
  [1148] = {.lex_state = 12},
  [1149] = {.lex_state = 12},
  [1150] = {.lex_state = 12},
  [1151] = {.lex_state = 12},
  [1152] = {.lex_state = 12},
  [1153] = {.lex_state = 12, .external_lex_state = 1},
  [1154] = {.lex_state = 12},
  [1155] = {.lex_state = 12},
  [1156] = {.lex_state = 12},
  [1157] = {.lex_state = 12},
  [1158] = {.lex_state = 12},
  [1159] = {.lex_state = 12},
  [1160] = {.lex_state = 12},
  [1161] = {.lex_state = 12, .external_lex_state = 1},
  [1162] = {.lex_state = 12, .external_lex_state = 1},
  [1163] = {.lex_state = 12, .external_lex_state = 1},
  [1164] = {.lex_state = 12},
  [1165] = {.lex_state = 12},
  [1166] = {.lex_state = 12},
  [1167] = {.lex_state = 12},
  [1168] = {.lex_state = 12},
  [1169] = {.lex_state = 12, .external_lex_state = 1},
  [1170] = {.lex_state = 12},
  [1171] = {.lex_state = 12, .external_lex_state = 1},
  [1172] = {.lex_state = 12, .external_lex_state = 1},
  [1173] = {.lex_state = 12},
  [1174] = {.lex_state = 12, .external_lex_state = 1},
  [1175] = {.lex_state = 12, .external_lex_state = 1},
  [1176] = {.lex_state = 12, .external_lex_state = 1},
  [1177] = {.lex_state = 12, .external_lex_state = 1},
  [1178] = {.lex_state = 12, .external_lex_state = 1},
  [1179] = {.lex_state = 12, .external_lex_state = 1},
  [1180] = {.lex_state = 12, .external_lex_state = 1},
  [1181] = {.lex_state = 12, .external_lex_state = 1},
  [1182] = {.lex_state = 12, .external_lex_state = 1},
  [1183] = {.lex_state = 12, .external_lex_state = 1},
  [1184] = {.lex_state = 12},
  [1185] = {.lex_state = 12, .external_lex_state = 1},
  [1186] = {.lex_state = 12},
  [1187] = {.lex_state = 12, .external_lex_state = 1},
  [1188] = {.lex_state = 12, .external_lex_state = 1},
  [1189] = {.lex_state = 12, .external_lex_state = 1},
  [1190] = {.lex_state = 12, .external_lex_state = 1},
  [1191] = {.lex_state = 12, .external_lex_state = 1},
  [1192] = {.lex_state = 12},
  [1193] = {.lex_state = 12},
  [1194] = {.lex_state = 12, .external_lex_state = 1},
  [1195] = {.lex_state = 12},
  [1196] = {.lex_state = 12},
  [1197] = {.lex_state = 12, .external_lex_state = 1},
  [1198] = {.lex_state = 12},
  [1199] = {.lex_state = 12},
  [1200] = {.lex_state = 12, .external_lex_state = 1},
  [1201] = {.lex_state = 12},
  [1202] = {.lex_state = 12},
  [1203] = {.lex_state = 12, .external_lex_state = 1},
  [1204] = {.lex_state = 12},
  [1205] = {.lex_state = 12},
  [1206] = {.lex_state = 12},
  [1207] = {.lex_state = 12},
  [1208] = {.lex_state = 12},
  [1209] = {.lex_state = 12},
  [1210] = {.lex_state = 12},
  [1211] = {.lex_state = 12},
  [1212] = {.lex_state = 12},
  [1213] = {.lex_state = 12},
  [1214] = {.lex_state = 12},
  [1215] = {.lex_state = 12},
  [1216] = {.lex_state = 12},
  [1217] = {.lex_state = 12, .external_lex_state = 1},
  [1218] = {.lex_state = 12},
  [1219] = {.lex_state = 12, .external_lex_state = 1},
  [1220] = {.lex_state = 12},
  [1221] = {.lex_state = 12, .external_lex_state = 1},
  [1222] = {.lex_state = 12},
  [1223] = {.lex_state = 12},
  [1224] = {.lex_state = 12},
  [1225] = {.lex_state = 12},
  [1226] = {.lex_state = 12, .external_lex_state = 1},
  [1227] = {.lex_state = 12},
  [1228] = {.lex_state = 12, .external_lex_state = 1},
  [1229] = {.lex_state = 12},
  [1230] = {.lex_state = 12, .external_lex_state = 1},
  [1231] = {.lex_state = 12},
  [1232] = {.lex_state = 12},
  [1233] = {.lex_state = 12, .external_lex_state = 1},
  [1234] = {.lex_state = 12, .external_lex_state = 1},
  [1235] = {.lex_state = 12},
  [1236] = {.lex_state = 12},
  [1237] = {.lex_state = 12},
  [1238] = {.lex_state = 12},
  [1239] = {.lex_state = 12},
  [1240] = {.lex_state = 12, .external_lex_state = 1},
  [1241] = {.lex_state = 12},
  [1242] = {.lex_state = 12},
  [1243] = {.lex_state = 12},
  [1244] = {.lex_state = 12},
  [1245] = {.lex_state = 12, .external_lex_state = 1},
  [1246] = {.lex_state = 12},
  [1247] = {.lex_state = 12},
  [1248] = {.lex_state = 12},
  [1249] = {.lex_state = 12},
  [1250] = {.lex_state = 12},
  [1251] = {.lex_state = 12},
  [1252] = {.lex_state = 12},
  [1253] = {.lex_state = 12},
  [1254] = {.lex_state = 12},
  [1255] = {.lex_state = 12},
  [1256] = {.lex_state = 12},
  [1257] = {.lex_state = 12},
  [1258] = {.lex_state = 12},
  [1259] = {.lex_state = 12, .external_lex_state = 1},
  [1260] = {.lex_state = 12, .external_lex_state = 1},
  [1261] = {.lex_state = 12, .external_lex_state = 1},
  [1262] = {.lex_state = 12, .external_lex_state = 1},
  [1263] = {.lex_state = 12},
  [1264] = {.lex_state = 12},
  [1265] = {.lex_state = 12},
  [1266] = {.lex_state = 12},
  [1267] = {.lex_state = 12},
  [1268] = {.lex_state = 12},
  [1269] = {.lex_state = 12},
  [1270] = {.lex_state = 12, .external_lex_state = 1},
  [1271] = {.lex_state = 12},
  [1272] = {.lex_state = 12},
  [1273] = {.lex_state = 12},
  [1274] = {.lex_state = 12},
  [1275] = {.lex_state = 12},
  [1276] = {.lex_state = 12},
  [1277] = {.lex_state = 12},
  [1278] = {.lex_state = 12, .external_lex_state = 1},
  [1279] = {.lex_state = 12, .external_lex_state = 1},
  [1280] = {.lex_state = 12},
  [1281] = {.lex_state = 12},
  [1282] = {.lex_state = 12},
  [1283] = {.lex_state = 12},
  [1284] = {.lex_state = 12},
  [1285] = {.lex_state = 12},
  [1286] = {.lex_state = 12},
  [1287] = {.lex_state = 12},
  [1288] = {.lex_state = 12},
  [1289] = {.lex_state = 12},
  [1290] = {.lex_state = 12},
  [1291] = {.lex_state = 12},
  [1292] = {.lex_state = 12},
  [1293] = {.lex_state = 12},
  [1294] = {.lex_state = 12},
  [1295] = {.lex_state = 12},
  [1296] = {.lex_state = 12},
  [1297] = {.lex_state = 12},
  [1298] = {.lex_state = 3},
  [1299] = {.lex_state = 3},
  [1300] = {.lex_state = 14},
  [1301] = {.lex_state = 14},
  [1302] = {.lex_state = 14},
  [1303] = {.lex_state = 14},
  [1304] = {.lex_state = 14},
  [1305] = {.lex_state = 14},
  [1306] = {.lex_state = 14},
  [1307] = {.lex_state = 14},
  [1308] = {.lex_state = 1},
  [1309] = {.lex_state = 1},
  [1310] = {.lex_state = 14},
  [1311] = {.lex_state = 14},
  [1312] = {.lex_state = 14},
  [1313] = {.lex_state = 3},
  [1314] = {.lex_state = 588},
  [1315] = {.lex_state = 14},
  [1316] = {.lex_state = 3},
  [1317] = {.lex_state = 588},
  [1318] = {.lex_state = 14},
  [1319] = {.lex_state = 588},
  [1320] = {.lex_state = 14},
  [1321] = {.lex_state = 14},
  [1322] = {.lex_state = 588},
  [1323] = {.lex_state = 588},
  [1324] = {.lex_state = 588},
  [1325] = {.lex_state = 588},
  [1326] = {.lex_state = 14},
  [1327] = {.lex_state = 588},
  [1328] = {.lex_state = 14},
  [1329] = {.lex_state = 14},
  [1330] = {.lex_state = 14},
  [1331] = {.lex_state = 14},
  [1332] = {.lex_state = 14},
  [1333] = {.lex_state = 14},
  [1334] = {.lex_state = 588},
  [1335] = {.lex_state = 14},
  [1336] = {.lex_state = 588},
  [1337] = {.lex_state = 588},
  [1338] = {.lex_state = 14},
  [1339] = {.lex_state = 588},
  [1340] = {.lex_state = 588},
  [1341] = {.lex_state = 14},
  [1342] = {.lex_state = 14},
  [1343] = {.lex_state = 588},
  [1344] = {.lex_state = 588},
  [1345] = {.lex_state = 14},
  [1346] = {.lex_state = 14},
  [1347] = {.lex_state = 14},
  [1348] = {.lex_state = 14},
  [1349] = {.lex_state = 14},
  [1350] = {.lex_state = 14},
  [1351] = {.lex_state = 14},
  [1352] = {.lex_state = 14},
  [1353] = {.lex_state = 14},
  [1354] = {.lex_state = 14},
  [1355] = {.lex_state = 14},
  [1356] = {.lex_state = 14},
  [1357] = {.lex_state = 14},
  [1358] = {.lex_state = 14},
  [1359] = {.lex_state = 14},
  [1360] = {.lex_state = 14},
  [1361] = {.lex_state = 14},
  [1362] = {.lex_state = 14},
  [1363] = {.lex_state = 14},
  [1364] = {.lex_state = 14},
  [1365] = {.lex_state = 14},
  [1366] = {.lex_state = 14},
  [1367] = {.lex_state = 14},
  [1368] = {.lex_state = 14},
  [1369] = {.lex_state = 13},
  [1370] = {.lex_state = 13},
  [1371] = {.lex_state = 13},
  [1372] = {.lex_state = 13},
  [1373] = {.lex_state = 13},
  [1374] = {.lex_state = 13, .external_lex_state = 1},
  [1375] = {.lex_state = 13, .external_lex_state = 1},
  [1376] = {.lex_state = 13, .external_lex_state = 1},
  [1377] = {.lex_state = 12, .external_lex_state = 1},
  [1378] = {.lex_state = 12, .external_lex_state = 1},
  [1379] = {.lex_state = 12, .external_lex_state = 1},
  [1380] = {.lex_state = 12, .external_lex_state = 1},
  [1381] = {.lex_state = 12, .external_lex_state = 1},
  [1382] = {.lex_state = 13, .external_lex_state = 1},
  [1383] = {.lex_state = 13, .external_lex_state = 1},
  [1384] = {.lex_state = 12, .external_lex_state = 1},
  [1385] = {.lex_state = 12, .external_lex_state = 1},
  [1386] = {.lex_state = 12, .external_lex_state = 1},
  [1387] = {.lex_state = 14},
  [1388] = {.lex_state = 588},
  [1389] = {.lex_state = 588},
  [1390] = {.lex_state = 588},
  [1391] = {.lex_state = 588},
  [1392] = {.lex_state = 14},
  [1393] = {.lex_state = 14},
  [1394] = {.lex_state = 14},
  [1395] = {.lex_state = 14},
  [1396] = {.lex_state = 14},
  [1397] = {.lex_state = 588},
  [1398] = {.lex_state = 588},
  [1399] = {.lex_state = 588},
  [1400] = {.lex_state = 588},
  [1401] = {.lex_state = 14},
  [1402] = {.lex_state = 588},
  [1403] = {.lex_state = 588},
  [1404] = {.lex_state = 588},
  [1405] = {.lex_state = 588},
  [1406] = {.lex_state = 588},
  [1407] = {.lex_state = 14},
  [1408] = {.lex_state = 588},
  [1409] = {.lex_state = 588},
  [1410] = {.lex_state = 588},
  [1411] = {.lex_state = 14},
  [1412] = {.lex_state = 14},
  [1413] = {.lex_state = 14},
  [1414] = {.lex_state = 14},
  [1415] = {.lex_state = 588},
  [1416] = {.lex_state = 14},
  [1417] = {.lex_state = 588},
  [1418] = {.lex_state = 588},
  [1419] = {.lex_state = 588},
  [1420] = {.lex_state = 588},
  [1421] = {.lex_state = 588},
  [1422] = {.lex_state = 588},
  [1423] = {.lex_state = 588},
  [1424] = {.lex_state = 13},
  [1425] = {.lex_state = 588},
  [1426] = {.lex_state = 588},
  [1427] = {.lex_state = 588},
  [1428] = {.lex_state = 13},
  [1429] = {.lex_state = 13},
  [1430] = {.lex_state = 588},
  [1431] = {.lex_state = 12},
  [1432] = {.lex_state = 588},
  [1433] = {.lex_state = 588},
  [1434] = {.lex_state = 588},
  [1435] = {.lex_state = 588},
  [1436] = {.lex_state = 588},
  [1437] = {.lex_state = 588},
  [1438] = {.lex_state = 588},
  [1439] = {.lex_state = 13},
  [1440] = {.lex_state = 13},
  [1441] = {.lex_state = 588},
  [1442] = {.lex_state = 13},
  [1443] = {.lex_state = 13},
  [1444] = {.lex_state = 13},
  [1445] = {.lex_state = 588},
  [1446] = {.lex_state = 13, .external_lex_state = 1},
  [1447] = {.lex_state = 13, .external_lex_state = 1},
  [1448] = {.lex_state = 13, .external_lex_state = 1},
  [1449] = {.lex_state = 13, .external_lex_state = 1},
  [1450] = {.lex_state = 13, .external_lex_state = 1},
  [1451] = {.lex_state = 13, .external_lex_state = 1},
  [1452] = {.lex_state = 13, .external_lex_state = 1},
  [1453] = {.lex_state = 13, .external_lex_state = 1},
  [1454] = {.lex_state = 12},
  [1455] = {.lex_state = 12},
  [1456] = {.lex_state = 12},
  [1457] = {.lex_state = 12},
  [1458] = {.lex_state = 12},
  [1459] = {.lex_state = 12, .external_lex_state = 1},
  [1460] = {.lex_state = 588},
  [1461] = {.lex_state = 588},
  [1462] = {.lex_state = 588},
  [1463] = {.lex_state = 588},
  [1464] = {.lex_state = 588},
  [1465] = {.lex_state = 588},
  [1466] = {.lex_state = 588},
  [1467] = {.lex_state = 588},
  [1468] = {.lex_state = 588},
  [1469] = {.lex_state = 588},
  [1470] = {.lex_state = 588},
  [1471] = {.lex_state = 588},
  [1472] = {.lex_state = 588},
  [1473] = {.lex_state = 1},
  [1474] = {.lex_state = 1},
  [1475] = {.lex_state = 1},
  [1476] = {.lex_state = 588},
  [1477] = {.lex_state = 588},
  [1478] = {.lex_state = 588},
  [1479] = {.lex_state = 1},
  [1480] = {.lex_state = 1},
  [1481] = {.lex_state = 1},
  [1482] = {.lex_state = 1},
  [1483] = {.lex_state = 1},
  [1484] = {.lex_state = 1},
  [1485] = {.lex_state = 1},
  [1486] = {.lex_state = 1},
  [1487] = {.lex_state = 1},
  [1488] = {.lex_state = 588},
  [1489] = {.lex_state = 1},
  [1490] = {.lex_state = 588},
  [1491] = {.lex_state = 1},
  [1492] = {.lex_state = 1},
  [1493] = {.lex_state = 588},
  [1494] = {.lex_state = 588},
  [1495] = {.lex_state = 588},
  [1496] = {.lex_state = 588},
  [1497] = {.lex_state = 588},
  [1498] = {.lex_state = 1},
  [1499] = {.lex_state = 588},
  [1500] = {.lex_state = 12},
  [1501] = {.lex_state = 588},
  [1502] = {.lex_state = 588},
  [1503] = {.lex_state = 588},
  [1504] = {.lex_state = 588},
  [1505] = {.lex_state = 588, .external_lex_state = 1},
  [1506] = {.lex_state = 588},
  [1507] = {.lex_state = 588},
  [1508] = {.lex_state = 588},
  [1509] = {.lex_state = 588},
  [1510] = {.lex_state = 588},
  [1511] = {.lex_state = 588},
  [1512] = {.lex_state = 588},
  [1513] = {.lex_state = 588},
  [1514] = {.lex_state = 588},
  [1515] = {.lex_state = 588},
  [1516] = {.lex_state = 588},
  [1517] = {.lex_state = 588},
  [1518] = {.lex_state = 588},
  [1519] = {.lex_state = 588},
  [1520] = {.lex_state = 588},
  [1521] = {.lex_state = 588},
  [1522] = {.lex_state = 588},
  [1523] = {.lex_state = 588},
  [1524] = {.lex_state = 588},
  [1525] = {.lex_state = 12},
  [1526] = {.lex_state = 588},
  [1527] = {.lex_state = 588},
  [1528] = {.lex_state = 588},
  [1529] = {.lex_state = 588},
  [1530] = {.lex_state = 588},
  [1531] = {.lex_state = 588},
  [1532] = {.lex_state = 588},
  [1533] = {.lex_state = 588},
  [1534] = {.lex_state = 588},
  [1535] = {.lex_state = 588},
  [1536] = {.lex_state = 588},
  [1537] = {.lex_state = 588},
  [1538] = {.lex_state = 588},
  [1539] = {.lex_state = 588},
  [1540] = {.lex_state = 588},
  [1541] = {.lex_state = 588},
  [1542] = {.lex_state = 588},
  [1543] = {.lex_state = 588},
  [1544] = {.lex_state = 588},
  [1545] = {.lex_state = 588},
  [1546] = {.lex_state = 588},
  [1547] = {.lex_state = 588},
  [1548] = {.lex_state = 588},
  [1549] = {.lex_state = 588},
  [1550] = {.lex_state = 588},
  [1551] = {.lex_state = 588},
  [1552] = {.lex_state = 588},
  [1553] = {.lex_state = 588},
  [1554] = {.lex_state = 588},
  [1555] = {.lex_state = 588},
  [1556] = {.lex_state = 588},
  [1557] = {.lex_state = 588},
  [1558] = {.lex_state = 588},
  [1559] = {.lex_state = 588},
  [1560] = {.lex_state = 588},
  [1561] = {.lex_state = 588},
  [1562] = {.lex_state = 588, .external_lex_state = 1},
  [1563] = {.lex_state = 588, .external_lex_state = 1},
  [1564] = {.lex_state = 588, .external_lex_state = 1},
  [1565] = {.lex_state = 588},
  [1566] = {.lex_state = 588},
  [1567] = {.lex_state = 12},
  [1568] = {.lex_state = 588},
  [1569] = {.lex_state = 588},
  [1570] = {.lex_state = 588},
  [1571] = {.lex_state = 588},
  [1572] = {.lex_state = 588},
  [1573] = {.lex_state = 588},
  [1574] = {.lex_state = 588},
  [1575] = {.lex_state = 588},
  [1576] = {.lex_state = 588},
  [1577] = {.lex_state = 588},
  [1578] = {.lex_state = 588},
  [1579] = {.lex_state = 588},
  [1580] = {.lex_state = 588},
  [1581] = {.lex_state = 588},
  [1582] = {.lex_state = 588},
  [1583] = {.lex_state = 588},
  [1584] = {.lex_state = 588},
  [1585] = {.lex_state = 588},
  [1586] = {.lex_state = 588},
  [1587] = {.lex_state = 588},
  [1588] = {.lex_state = 588, .external_lex_state = 1},
  [1589] = {.lex_state = 588},
  [1590] = {.lex_state = 588},
  [1591] = {.lex_state = 588},
  [1592] = {.lex_state = 588},
  [1593] = {.lex_state = 588},
  [1594] = {.lex_state = 588},
  [1595] = {.lex_state = 588},
  [1596] = {.lex_state = 588},
  [1597] = {.lex_state = 588},
  [1598] = {.lex_state = 588, .external_lex_state = 1},
  [1599] = {.lex_state = 588},
  [1600] = {.lex_state = 588},
  [1601] = {.lex_state = 588},
  [1602] = {.lex_state = 588},
  [1603] = {.lex_state = 588},
  [1604] = {.lex_state = 588},
  [1605] = {.lex_state = 588},
  [1606] = {.lex_state = 588},
  [1607] = {.lex_state = 588},
  [1608] = {.lex_state = 588},
  [1609] = {.lex_state = 588},
  [1610] = {.lex_state = 588},
  [1611] = {.lex_state = 588},
  [1612] = {.lex_state = 588},
  [1613] = {.lex_state = 588},
  [1614] = {.lex_state = 588},
  [1615] = {.lex_state = 588},
  [1616] = {.lex_state = 588},
  [1617] = {.lex_state = 588},
  [1618] = {.lex_state = 588},
  [1619] = {.lex_state = 588},
  [1620] = {.lex_state = 588},
  [1621] = {.lex_state = 588},
  [1622] = {.lex_state = 588},
  [1623] = {.lex_state = 588},
  [1624] = {.lex_state = 588},
  [1625] = {.lex_state = 588},
  [1626] = {.lex_state = 588},
  [1627] = {.lex_state = 588, .external_lex_state = 1},
  [1628] = {.lex_state = 588},
  [1629] = {.lex_state = 588},
  [1630] = {.lex_state = 588},
  [1631] = {.lex_state = 588},
  [1632] = {.lex_state = 588},
  [1633] = {.lex_state = 588},
  [1634] = {.lex_state = 588},
  [1635] = {.lex_state = 588},
  [1636] = {.lex_state = 588},
  [1637] = {.lex_state = 588},
  [1638] = {.lex_state = 588},
  [1639] = {.lex_state = 588, .external_lex_state = 1},
  [1640] = {.lex_state = 588},
  [1641] = {.lex_state = 12},
  [1642] = {.lex_state = 588},
  [1643] = {.lex_state = 588},
  [1644] = {.lex_state = 588},
  [1645] = {.lex_state = 588},
  [1646] = {.lex_state = 588},
  [1647] = {.lex_state = 588},
  [1648] = {.lex_state = 588},
  [1649] = {.lex_state = 588},
  [1650] = {.lex_state = 588},
  [1651] = {.lex_state = 588},
  [1652] = {.lex_state = 588},
  [1653] = {.lex_state = 588},
  [1654] = {.lex_state = 588},
  [1655] = {.lex_state = 588, .external_lex_state = 1},
  [1656] = {.lex_state = 588},
  [1657] = {.lex_state = 588},
  [1658] = {.lex_state = 588},
  [1659] = {.lex_state = 588},
  [1660] = {.lex_state = 588},
  [1661] = {.lex_state = 588},
  [1662] = {.lex_state = 12},
  [1663] = {.lex_state = 588},
  [1664] = {.lex_state = 588, .external_lex_state = 1},
  [1665] = {.lex_state = 588},
  [1666] = {.lex_state = 588},
  [1667] = {.lex_state = 588},
  [1668] = {.lex_state = 588},
  [1669] = {.lex_state = 588},
  [1670] = {.lex_state = 588},
  [1671] = {.lex_state = 588},
  [1672] = {.lex_state = 588, .external_lex_state = 1},
  [1673] = {.lex_state = 588},
  [1674] = {.lex_state = 588},
  [1675] = {.lex_state = 588},
  [1676] = {.lex_state = 588},
  [1677] = {.lex_state = 588},
  [1678] = {.lex_state = 588},
  [1679] = {.lex_state = 588},
  [1680] = {.lex_state = 588},
  [1681] = {.lex_state = 588},
  [1682] = {.lex_state = 588},
  [1683] = {.lex_state = 588, .external_lex_state = 1},
  [1684] = {.lex_state = 588, .external_lex_state = 1},
  [1685] = {.lex_state = 588},
  [1686] = {.lex_state = 588},
  [1687] = {.lex_state = 588},
  [1688] = {.lex_state = 588},
  [1689] = {.lex_state = 588},
  [1690] = {.lex_state = 588, .external_lex_state = 1},
  [1691] = {.lex_state = 588},
  [1692] = {.lex_state = 588},
  [1693] = {.lex_state = 588, .external_lex_state = 1},
  [1694] = {.lex_state = 588, .external_lex_state = 1},
  [1695] = {.lex_state = 588},
  [1696] = {.lex_state = 588},
  [1697] = {.lex_state = 588},
  [1698] = {.lex_state = 588},
  [1699] = {.lex_state = 588},
  [1700] = {.lex_state = 588},
  [1701] = {.lex_state = 588},
  [1702] = {.lex_state = 588},
  [1703] = {.lex_state = 588},
  [1704] = {.lex_state = 588},
  [1705] = {.lex_state = 588},
  [1706] = {.lex_state = 588, .external_lex_state = 1},
  [1707] = {.lex_state = 5},
  [1708] = {.lex_state = 588},
  [1709] = {.lex_state = 12},
  [1710] = {.lex_state = 588, .external_lex_state = 1},
  [1711] = {.lex_state = 588, .external_lex_state = 1},
  [1712] = {.lex_state = 588, .external_lex_state = 1},
  [1713] = {.lex_state = 588, .external_lex_state = 1},
  [1714] = {.lex_state = 588, .external_lex_state = 1},
  [1715] = {.lex_state = 588, .external_lex_state = 1},
  [1716] = {.lex_state = 588, .external_lex_state = 1},
  [1717] = {.lex_state = 588, .external_lex_state = 1},
  [1718] = {.lex_state = 588},
  [1719] = {.lex_state = 588, .external_lex_state = 1},
  [1720] = {.lex_state = 588},
  [1721] = {.lex_state = 588, .external_lex_state = 1},
  [1722] = {.lex_state = 588, .external_lex_state = 1},
  [1723] = {.lex_state = 588, .external_lex_state = 1},
  [1724] = {.lex_state = 588, .external_lex_state = 1},
  [1725] = {.lex_state = 588, .external_lex_state = 1},
  [1726] = {.lex_state = 588, .external_lex_state = 1},
  [1727] = {.lex_state = 588, .external_lex_state = 1},
  [1728] = {.lex_state = 588, .external_lex_state = 1},
  [1729] = {.lex_state = 5},
  [1730] = {.lex_state = 588, .external_lex_state = 1},
  [1731] = {.lex_state = 588, .external_lex_state = 1},
  [1732] = {.lex_state = 588, .external_lex_state = 1},
  [1733] = {.lex_state = 5},
  [1734] = {.lex_state = 588, .external_lex_state = 1},
  [1735] = {.lex_state = 588, .external_lex_state = 1},
  [1736] = {.lex_state = 588, .external_lex_state = 1},
  [1737] = {.lex_state = 588, .external_lex_state = 1},
  [1738] = {.lex_state = 12},
  [1739] = {.lex_state = 588, .external_lex_state = 1},
  [1740] = {.lex_state = 588, .external_lex_state = 1},
  [1741] = {.lex_state = 588, .external_lex_state = 1},
  [1742] = {.lex_state = 588, .external_lex_state = 1},
  [1743] = {.lex_state = 588, .external_lex_state = 1},
  [1744] = {.lex_state = 588, .external_lex_state = 1},
  [1745] = {.lex_state = 588, .external_lex_state = 1},
  [1746] = {.lex_state = 588, .external_lex_state = 1},
  [1747] = {.lex_state = 588, .external_lex_state = 1},
  [1748] = {.lex_state = 588, .external_lex_state = 1},
  [1749] = {.lex_state = 588, .external_lex_state = 1},
  [1750] = {.lex_state = 5},
  [1751] = {.lex_state = 588, .external_lex_state = 1},
  [1752] = {.lex_state = 5},
  [1753] = {.lex_state = 588, .external_lex_state = 1},
  [1754] = {.lex_state = 588, .external_lex_state = 1},
  [1755] = {.lex_state = 588, .external_lex_state = 1},
  [1756] = {.lex_state = 588, .external_lex_state = 1},
  [1757] = {.lex_state = 588, .external_lex_state = 1},
  [1758] = {.lex_state = 5},
  [1759] = {.lex_state = 588, .external_lex_state = 1},
  [1760] = {.lex_state = 588, .external_lex_state = 1},
  [1761] = {.lex_state = 588},
  [1762] = {.lex_state = 5},
  [1763] = {.lex_state = 588, .external_lex_state = 1},
  [1764] = {.lex_state = 588, .external_lex_state = 1},
  [1765] = {.lex_state = 5},
  [1766] = {.lex_state = 5},
  [1767] = {.lex_state = 5},
  [1768] = {.lex_state = 5},
  [1769] = {.lex_state = 5},
  [1770] = {.lex_state = 5},
  [1771] = {.lex_state = 588, .external_lex_state = 1},
  [1772] = {.lex_state = 588},
  [1773] = {.lex_state = 588, .external_lex_state = 1},
  [1774] = {.lex_state = 12},
  [1775] = {.lex_state = 12},
  [1776] = {.lex_state = 588, .external_lex_state = 1},
  [1777] = {.lex_state = 588, .external_lex_state = 1},
  [1778] = {.lex_state = 5},
  [1779] = {.lex_state = 588, .external_lex_state = 1},
  [1780] = {.lex_state = 588},
  [1781] = {.lex_state = 588},
  [1782] = {.lex_state = 588},
  [1783] = {.lex_state = 588, .external_lex_state = 1},
  [1784] = {.lex_state = 588, .external_lex_state = 1},
  [1785] = {.lex_state = 5},
  [1786] = {.lex_state = 5},
  [1787] = {.lex_state = 588, .external_lex_state = 1},
  [1788] = {.lex_state = 588, .external_lex_state = 1},
  [1789] = {.lex_state = 588, .external_lex_state = 1},
  [1790] = {.lex_state = 12},
  [1791] = {.lex_state = 12},
  [1792] = {.lex_state = 5},
  [1793] = {.lex_state = 588, .external_lex_state = 1},
  [1794] = {.lex_state = 588, .external_lex_state = 1},
  [1795] = {.lex_state = 588},
  [1796] = {.lex_state = 588, .external_lex_state = 1},
  [1797] = {.lex_state = 588, .external_lex_state = 1},
  [1798] = {.lex_state = 5},
  [1799] = {.lex_state = 588, .external_lex_state = 1},
  [1800] = {.lex_state = 588, .external_lex_state = 1},
  [1801] = {.lex_state = 588, .external_lex_state = 1},
  [1802] = {.lex_state = 588, .external_lex_state = 1},
  [1803] = {.lex_state = 588, .external_lex_state = 1},
  [1804] = {.lex_state = 588, .external_lex_state = 1},
  [1805] = {.lex_state = 5},
  [1806] = {.lex_state = 588},
  [1807] = {.lex_state = 588},
  [1808] = {.lex_state = 17, .external_lex_state = 1},
  [1809] = {.lex_state = 588},
  [1810] = {.lex_state = 588},
  [1811] = {.lex_state = 588},
  [1812] = {.lex_state = 588},
  [1813] = {.lex_state = 588},
  [1814] = {.lex_state = 588},
  [1815] = {.lex_state = 588},
  [1816] = {.lex_state = 588},
  [1817] = {.lex_state = 588, .external_lex_state = 1},
  [1818] = {.lex_state = 588, .external_lex_state = 1},
  [1819] = {.lex_state = 588},
  [1820] = {.lex_state = 588},
  [1821] = {.lex_state = 588},
  [1822] = {.lex_state = 588},
  [1823] = {.lex_state = 588},
  [1824] = {.lex_state = 588},
  [1825] = {.lex_state = 588},
  [1826] = {.lex_state = 588, .external_lex_state = 1},
  [1827] = {.lex_state = 588},
  [1828] = {.lex_state = 17, .external_lex_state = 1},
  [1829] = {.lex_state = 588},
  [1830] = {.lex_state = 588},
  [1831] = {.lex_state = 588},
  [1832] = {.lex_state = 588, .external_lex_state = 1},
  [1833] = {.lex_state = 17, .external_lex_state = 1},
  [1834] = {.lex_state = 588, .external_lex_state = 1},
  [1835] = {.lex_state = 588},
  [1836] = {.lex_state = 588},
  [1837] = {.lex_state = 588},
  [1838] = {.lex_state = 588},
  [1839] = {.lex_state = 588},
  [1840] = {.lex_state = 588},
  [1841] = {.lex_state = 17, .external_lex_state = 1},
  [1842] = {.lex_state = 588, .external_lex_state = 1},
  [1843] = {.lex_state = 588},
  [1844] = {.lex_state = 588},
  [1845] = {.lex_state = 588},
  [1846] = {.lex_state = 588},
  [1847] = {.lex_state = 588},
  [1848] = {.lex_state = 588},
  [1849] = {.lex_state = 588},
  [1850] = {.lex_state = 17, .external_lex_state = 1},
  [1851] = {.lex_state = 588},
  [1852] = {.lex_state = 588},
  [1853] = {.lex_state = 588},
  [1854] = {.lex_state = 588, .external_lex_state = 1},
  [1855] = {.lex_state = 588},
  [1856] = {.lex_state = 588},
  [1857] = {.lex_state = 588},
  [1858] = {.lex_state = 588},
  [1859] = {.lex_state = 588},
  [1860] = {.lex_state = 588},
  [1861] = {.lex_state = 588},
  [1862] = {.lex_state = 588},
  [1863] = {.lex_state = 588, .external_lex_state = 1},
  [1864] = {.lex_state = 588, .external_lex_state = 1},
  [1865] = {.lex_state = 588},
  [1866] = {.lex_state = 588},
  [1867] = {.lex_state = 588},
  [1868] = {.lex_state = 588},
  [1869] = {.lex_state = 588},
  [1870] = {.lex_state = 588, .external_lex_state = 1},
  [1871] = {.lex_state = 588},
  [1872] = {.lex_state = 588},
  [1873] = {.lex_state = 588, .external_lex_state = 1},
  [1874] = {.lex_state = 588},
  [1875] = {.lex_state = 588},
  [1876] = {.lex_state = 588},
  [1877] = {.lex_state = 588, .external_lex_state = 1},
  [1878] = {.lex_state = 588},
  [1879] = {.lex_state = 0, .external_lex_state = 1},
  [1880] = {.lex_state = 0, .external_lex_state = 1},
  [1881] = {.lex_state = 0, .external_lex_state = 1},
  [1882] = {.lex_state = 0, .external_lex_state = 1},
  [1883] = {.lex_state = 0, .external_lex_state = 1},
  [1884] = {.lex_state = 0, .external_lex_state = 1},
  [1885] = {.lex_state = 0, .external_lex_state = 1},
  [1886] = {.lex_state = 0, .external_lex_state = 1},
  [1887] = {.lex_state = 588},
  [1888] = {.lex_state = 588},
  [1889] = {.lex_state = 0, .external_lex_state = 1},
  [1890] = {.lex_state = 0, .external_lex_state = 1},
  [1891] = {.lex_state = 588},
  [1892] = {.lex_state = 0, .external_lex_state = 1},
  [1893] = {.lex_state = 588},
  [1894] = {.lex_state = 0, .external_lex_state = 1},
  [1895] = {.lex_state = 0, .external_lex_state = 1},
  [1896] = {.lex_state = 0, .external_lex_state = 1},
  [1897] = {.lex_state = 0, .external_lex_state = 1},
  [1898] = {.lex_state = 588},
  [1899] = {.lex_state = 0, .external_lex_state = 1},
  [1900] = {.lex_state = 0, .external_lex_state = 1},
  [1901] = {.lex_state = 588},
  [1902] = {.lex_state = 0, .external_lex_state = 1},
  [1903] = {.lex_state = 588},
  [1904] = {.lex_state = 588},
  [1905] = {.lex_state = 588},
  [1906] = {.lex_state = 588},
  [1907] = {.lex_state = 588},
  [1908] = {.lex_state = 588},
  [1909] = {.lex_state = 0, .external_lex_state = 1},
  [1910] = {.lex_state = 588},
  [1911] = {.lex_state = 588},
  [1912] = {.lex_state = 0, .external_lex_state = 1},
  [1913] = {.lex_state = 588},
  [1914] = {.lex_state = 0, .external_lex_state = 1},
  [1915] = {.lex_state = 588},
  [1916] = {.lex_state = 588},
  [1917] = {.lex_state = 588},
  [1918] = {.lex_state = 588},
  [1919] = {.lex_state = 0, .external_lex_state = 1},
  [1920] = {.lex_state = 0, .external_lex_state = 1},
  [1921] = {.lex_state = 0, .external_lex_state = 1},
  [1922] = {.lex_state = 0, .external_lex_state = 1},
  [1923] = {.lex_state = 588},
  [1924] = {.lex_state = 0, .external_lex_state = 1},
  [1925] = {.lex_state = 0, .external_lex_state = 1},
  [1926] = {.lex_state = 0, .external_lex_state = 1},
  [1927] = {.lex_state = 0, .external_lex_state = 1},
  [1928] = {.lex_state = 0, .external_lex_state = 1},
  [1929] = {.lex_state = 0, .external_lex_state = 1},
  [1930] = {.lex_state = 0, .external_lex_state = 1},
  [1931] = {.lex_state = 0, .external_lex_state = 1},
  [1932] = {.lex_state = 0, .external_lex_state = 1},
  [1933] = {.lex_state = 0, .external_lex_state = 1},
  [1934] = {.lex_state = 588},
  [1935] = {.lex_state = 0, .external_lex_state = 1},
  [1936] = {.lex_state = 12},
  [1937] = {.lex_state = 0, .external_lex_state = 1},
  [1938] = {.lex_state = 588},
  [1939] = {.lex_state = 588},
  [1940] = {.lex_state = 588},
  [1941] = {.lex_state = 588},
  [1942] = {.lex_state = 0, .external_lex_state = 1},
  [1943] = {.lex_state = 588},
  [1944] = {.lex_state = 588},
  [1945] = {.lex_state = 0, .external_lex_state = 1},
  [1946] = {.lex_state = 588},
  [1947] = {.lex_state = 0, .external_lex_state = 1},
  [1948] = {.lex_state = 0, .external_lex_state = 1},
  [1949] = {.lex_state = 588},
  [1950] = {.lex_state = 588},
  [1951] = {.lex_state = 0, .external_lex_state = 1},
  [1952] = {.lex_state = 588},
  [1953] = {.lex_state = 588},
  [1954] = {.lex_state = 0, .external_lex_state = 1},
  [1955] = {.lex_state = 0, .external_lex_state = 1},
  [1956] = {.lex_state = 0, .external_lex_state = 1},
  [1957] = {.lex_state = 0, .external_lex_state = 1},
  [1958] = {.lex_state = 12},
  [1959] = {.lex_state = 588},
  [1960] = {.lex_state = 0, .external_lex_state = 1},
  [1961] = {.lex_state = 588},
  [1962] = {.lex_state = 0, .external_lex_state = 1},
  [1963] = {.lex_state = 588},
  [1964] = {.lex_state = 588},
  [1965] = {.lex_state = 588},
  [1966] = {.lex_state = 588},
  [1967] = {.lex_state = 12},
  [1968] = {.lex_state = 0, .external_lex_state = 1},
  [1969] = {.lex_state = 588},
  [1970] = {.lex_state = 588},
  [1971] = {.lex_state = 0, .external_lex_state = 1},
  [1972] = {.lex_state = 588},
  [1973] = {.lex_state = 588},
  [1974] = {.lex_state = 588},
  [1975] = {.lex_state = 588},
  [1976] = {.lex_state = 588},
  [1977] = {.lex_state = 588},
  [1978] = {.lex_state = 0, .external_lex_state = 1},
  [1979] = {.lex_state = 0, .external_lex_state = 1},
  [1980] = {.lex_state = 0, .external_lex_state = 1},
  [1981] = {.lex_state = 12},
  [1982] = {.lex_state = 588},
  [1983] = {.lex_state = 588},
  [1984] = {.lex_state = 588},
  [1985] = {.lex_state = 588},
  [1986] = {.lex_state = 588},
  [1987] = {.lex_state = 588},
  [1988] = {.lex_state = 588},
  [1989] = {.lex_state = 588},
  [1990] = {.lex_state = 0, .external_lex_state = 1},
  [1991] = {.lex_state = 588},
  [1992] = {.lex_state = 0, .external_lex_state = 1},
  [1993] = {.lex_state = 588},
  [1994] = {.lex_state = 0, .external_lex_state = 1},
  [1995] = {.lex_state = 588},
  [1996] = {.lex_state = 588},
  [1997] = {.lex_state = 588},
  [1998] = {.lex_state = 588},
  [1999] = {.lex_state = 588},
  [2000] = {.lex_state = 0, .external_lex_state = 1},
  [2001] = {.lex_state = 588},
  [2002] = {.lex_state = 588},
  [2003] = {.lex_state = 588},
  [2004] = {.lex_state = 0, .external_lex_state = 1},
  [2005] = {.lex_state = 588},
  [2006] = {.lex_state = 588},
  [2007] = {.lex_state = 588},
  [2008] = {.lex_state = 588},
  [2009] = {.lex_state = 588},
  [2010] = {.lex_state = 588},
  [2011] = {.lex_state = 0, .external_lex_state = 1},
  [2012] = {.lex_state = 588},
  [2013] = {.lex_state = 588},
  [2014] = {.lex_state = 588},
  [2015] = {.lex_state = 588},
  [2016] = {.lex_state = 0, .external_lex_state = 1},
  [2017] = {.lex_state = 0, .external_lex_state = 1},
  [2018] = {.lex_state = 588},
  [2019] = {.lex_state = 588},
  [2020] = {.lex_state = 588},
  [2021] = {.lex_state = 588},
  [2022] = {.lex_state = 0, .external_lex_state = 1},
  [2023] = {.lex_state = 588},
  [2024] = {.lex_state = 588},
  [2025] = {.lex_state = 588},
  [2026] = {.lex_state = 588},
  [2027] = {.lex_state = 0, .external_lex_state = 1},
  [2028] = {.lex_state = 588},
  [2029] = {.lex_state = 12},
  [2030] = {.lex_state = 12},
  [2031] = {.lex_state = 588},
  [2032] = {.lex_state = 588},
  [2033] = {.lex_state = 588},
  [2034] = {.lex_state = 0, .external_lex_state = 1},
  [2035] = {.lex_state = 588},
  [2036] = {.lex_state = 12},
  [2037] = {.lex_state = 12},
  [2038] = {.lex_state = 0, .external_lex_state = 1},
  [2039] = {.lex_state = 588},
  [2040] = {.lex_state = 588},
  [2041] = {.lex_state = 12},
  [2042] = {.lex_state = 0, .external_lex_state = 1},
  [2043] = {.lex_state = 12},
  [2044] = {.lex_state = 12},
  [2045] = {.lex_state = 0, .external_lex_state = 1},
  [2046] = {.lex_state = 0, .external_lex_state = 1},
  [2047] = {.lex_state = 12},
  [2048] = {.lex_state = 588},
  [2049] = {.lex_state = 588},
  [2050] = {.lex_state = 588},
  [2051] = {.lex_state = 588},
  [2052] = {.lex_state = 588},
  [2053] = {.lex_state = 588},
  [2054] = {.lex_state = 588},
  [2055] = {.lex_state = 588},
  [2056] = {.lex_state = 588},
  [2057] = {.lex_state = 588},
  [2058] = {.lex_state = 588},
  [2059] = {.lex_state = 588},
  [2060] = {.lex_state = 588},
  [2061] = {.lex_state = 12},
  [2062] = {.lex_state = 588},
  [2063] = {.lex_state = 588},
  [2064] = {.lex_state = 588},
  [2065] = {.lex_state = 588},
  [2066] = {.lex_state = 588},
  [2067] = {.lex_state = 588},
  [2068] = {.lex_state = 588},
  [2069] = {.lex_state = 12},
  [2070] = {.lex_state = 12},
  [2071] = {.lex_state = 588},
  [2072] = {.lex_state = 588},
  [2073] = {.lex_state = 588},
  [2074] = {.lex_state = 588},
  [2075] = {.lex_state = 588},
  [2076] = {.lex_state = 588},
  [2077] = {.lex_state = 588},
  [2078] = {.lex_state = 588},
  [2079] = {.lex_state = 588},
  [2080] = {.lex_state = 588},
  [2081] = {.lex_state = 588},
  [2082] = {.lex_state = 12},
  [2083] = {.lex_state = 12},
  [2084] = {.lex_state = 588},
  [2085] = {.lex_state = 588},
  [2086] = {.lex_state = 588},
  [2087] = {.lex_state = 12},
  [2088] = {.lex_state = 588},
  [2089] = {.lex_state = 588},
  [2090] = {.lex_state = 12},
  [2091] = {.lex_state = 12},
  [2092] = {.lex_state = 588},
  [2093] = {.lex_state = 12},
  [2094] = {.lex_state = 588},
  [2095] = {.lex_state = 588},
  [2096] = {.lex_state = 588},
  [2097] = {.lex_state = 12},
  [2098] = {.lex_state = 588},
  [2099] = {.lex_state = 588},
  [2100] = {.lex_state = 588},
  [2101] = {.lex_state = 588},
  [2102] = {.lex_state = 12},
  [2103] = {.lex_state = 588},
  [2104] = {.lex_state = 588},
  [2105] = {.lex_state = 12},
  [2106] = {.lex_state = 12},
  [2107] = {.lex_state = 588},
  [2108] = {.lex_state = 588},
  [2109] = {.lex_state = 588},
  [2110] = {.lex_state = 12},
  [2111] = {.lex_state = 588},
  [2112] = {.lex_state = 588},
  [2113] = {.lex_state = 588},
  [2114] = {.lex_state = 588},
  [2115] = {.lex_state = 588},
  [2116] = {.lex_state = 588},
  [2117] = {.lex_state = 588},
  [2118] = {.lex_state = 588},
  [2119] = {.lex_state = 588},
  [2120] = {.lex_state = 588},
  [2121] = {.lex_state = 12},
  [2122] = {.lex_state = 588},
  [2123] = {.lex_state = 588},
  [2124] = {.lex_state = 12},
  [2125] = {.lex_state = 588},
  [2126] = {.lex_state = 588},
  [2127] = {.lex_state = 588},
  [2128] = {.lex_state = 588},
  [2129] = {.lex_state = 588},
  [2130] = {.lex_state = 588},
  [2131] = {.lex_state = 588},
  [2132] = {.lex_state = 588},
  [2133] = {.lex_state = 588},
  [2134] = {.lex_state = 588},
  [2135] = {.lex_state = 588},
  [2136] = {.lex_state = 588},
  [2137] = {.lex_state = 588},
  [2138] = {.lex_state = 588},
  [2139] = {.lex_state = 12},
  [2140] = {.lex_state = 588},
  [2141] = {.lex_state = 588},
  [2142] = {.lex_state = 588},
  [2143] = {.lex_state = 588},
  [2144] = {.lex_state = 12},
  [2145] = {.lex_state = 12},
  [2146] = {.lex_state = 588},
  [2147] = {.lex_state = 12},
  [2148] = {.lex_state = 588},
  [2149] = {.lex_state = 0},
  [2150] = {.lex_state = 588},
  [2151] = {.lex_state = 12},
  [2152] = {.lex_state = 12},
  [2153] = {.lex_state = 588},
  [2154] = {.lex_state = 588},
  [2155] = {.lex_state = 588},
  [2156] = {.lex_state = 588},
  [2157] = {.lex_state = 588},
  [2158] = {.lex_state = 588},
  [2159] = {.lex_state = 0},
  [2160] = {.lex_state = 588},
  [2161] = {.lex_state = 588},
  [2162] = {.lex_state = 588},
  [2163] = {.lex_state = 588},
  [2164] = {.lex_state = 588},
  [2165] = {.lex_state = 588},
  [2166] = {.lex_state = 12},
  [2167] = {.lex_state = 588},
  [2168] = {.lex_state = 12},
  [2169] = {.lex_state = 12},
  [2170] = {.lex_state = 588},
  [2171] = {.lex_state = 588},
  [2172] = {.lex_state = 12},
  [2173] = {.lex_state = 12},
  [2174] = {.lex_state = 588},
  [2175] = {.lex_state = 588},
  [2176] = {.lex_state = 588},
  [2177] = {.lex_state = 12},
  [2178] = {.lex_state = 12},
  [2179] = {.lex_state = 12},
  [2180] = {.lex_state = 12},
  [2181] = {.lex_state = 588},
  [2182] = {.lex_state = 588},
  [2183] = {.lex_state = 12},
  [2184] = {.lex_state = 588},
  [2185] = {.lex_state = 588},
  [2186] = {.lex_state = 588},
  [2187] = {.lex_state = 588},
  [2188] = {.lex_state = 588},
  [2189] = {.lex_state = 12},
  [2190] = {.lex_state = 588},
  [2191] = {.lex_state = 588},
  [2192] = {.lex_state = 12},
  [2193] = {.lex_state = 588},
  [2194] = {.lex_state = 588},
  [2195] = {.lex_state = 588},
  [2196] = {.lex_state = 588},
  [2197] = {.lex_state = 12},
  [2198] = {.lex_state = 588},
  [2199] = {.lex_state = 588},
  [2200] = {.lex_state = 588},
  [2201] = {.lex_state = 12},
  [2202] = {.lex_state = 12},
  [2203] = {.lex_state = 588},
  [2204] = {.lex_state = 588},
  [2205] = {.lex_state = 588},
  [2206] = {.lex_state = 588},
  [2207] = {.lex_state = 588},
  [2208] = {.lex_state = 12},
  [2209] = {.lex_state = 588},
  [2210] = {.lex_state = 588},
  [2211] = {.lex_state = 588},
  [2212] = {.lex_state = 588},
  [2213] = {.lex_state = 588},
  [2214] = {.lex_state = 588},
  [2215] = {.lex_state = 588},
  [2216] = {.lex_state = 588},
  [2217] = {.lex_state = 588},
  [2218] = {.lex_state = 12},
  [2219] = {.lex_state = 12},
  [2220] = {.lex_state = 588},
  [2221] = {.lex_state = 12},
  [2222] = {.lex_state = 588},
  [2223] = {.lex_state = 12},
  [2224] = {.lex_state = 12},
  [2225] = {.lex_state = 588},
  [2226] = {.lex_state = 12},
  [2227] = {.lex_state = 588},
  [2228] = {.lex_state = 12},
  [2229] = {.lex_state = 12},
  [2230] = {.lex_state = 12},
  [2231] = {.lex_state = 12},
  [2232] = {.lex_state = 12},
  [2233] = {.lex_state = 588},
  [2234] = {.lex_state = 588},
  [2235] = {.lex_state = 12},
  [2236] = {.lex_state = 588},
  [2237] = {.lex_state = 12},
  [2238] = {.lex_state = 12},
  [2239] = {.lex_state = 588},
  [2240] = {.lex_state = 0},
  [2241] = {.lex_state = 12},
  [2242] = {.lex_state = 12},
  [2243] = {.lex_state = 588},
  [2244] = {.lex_state = 588},
  [2245] = {.lex_state = 588},
  [2246] = {.lex_state = 588},
  [2247] = {.lex_state = 588},
  [2248] = {.lex_state = 588},
  [2249] = {.lex_state = 12},
  [2250] = {.lex_state = 588},
  [2251] = {.lex_state = 588},
  [2252] = {.lex_state = 588},
  [2253] = {.lex_state = 12},
  [2254] = {.lex_state = 12},
  [2255] = {.lex_state = 588},
  [2256] = {.lex_state = 0},
  [2257] = {.lex_state = 588},
  [2258] = {.lex_state = 588},
  [2259] = {.lex_state = 588},
  [2260] = {.lex_state = 588},
  [2261] = {.lex_state = 588},
  [2262] = {.lex_state = 588},
  [2263] = {.lex_state = 588},
  [2264] = {.lex_state = 588},
};

static const uint16_t ts_parse_table[LARGE_STATE_COUNT][SYMBOL_COUNT] = {
  [STATE(0)] = {
    [ts_builtin_sym_end] = ACTIONS(1),
    [sym_identifier] = ACTIONS(1),
    [sym_program_name_keyword] = ACTIONS(1),
    [sym_module_name_keyword] = ACTIONS(1),
    [sym_define_device_keyword] = ACTIONS(1),
    [sym_define_combine_keyword] = ACTIONS(1),
    [sym_define_constant_keyword] = ACTIONS(1),
    [sym_define_type_keyword] = ACTIONS(1),
    [sym_define_variable_keyword] = ACTIONS(1),
    [sym_define_system_variable_keyword] = ACTIONS(1),
    [sym_define_connect_level_keyword] = ACTIONS(1),
    [sym_define_latching_keyword] = ACTIONS(1),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1),
    [sym_define_toggling_keyword] = ACTIONS(1),
    [sym_define_call_keyword] = ACTIONS(1),
    [sym_define_function_keyword] = ACTIONS(1),
    [sym_define_library_function_keyword] = ACTIONS(1),
    [sym_define_start_keyword] = ACTIONS(1),
    [sym_define_event_keyword] = ACTIONS(1),
    [sym_define_program_keyword] = ACTIONS(1),
    [sym_define_module_keyword] = ACTIONS(1),
    [sym_char_keyword] = ACTIONS(1),
    [sym_widechar_keyword] = ACTIONS(1),
    [sym_integer_keyword] = ACTIONS(1),
    [sym_sinteger_keyword] = ACTIONS(1),
    [sym_long_keyword] = ACTIONS(1),
    [sym_slong_keyword] = ACTIONS(1),
    [sym_float_keyword] = ACTIONS(1),
    [sym_double_keyword] = ACTIONS(1),
    [sym_variant_keyword] = ACTIONS(1),
    [sym_variantarray_keyword] = ACTIONS(1),
    [sym_local_var_keyword] = ACTIONS(1),
    [sym_stack_var_keyword] = ACTIONS(1),
    [sym_constant_keyword] = ACTIONS(1),
    [sym_volatile_keyword] = ACTIONS(1),
    [sym_non_volatile_keyword] = ACTIONS(1),
    [sym_persistent_keyword] = ACTIONS(1),
    [sym_dev_keyword] = ACTIONS(1),
    [sym_devlev_keyword] = ACTIONS(1),
    [sym_devchan_keyword] = ACTIONS(1),
    [sym_if_keyword] = ACTIONS(1),
    [sym_else_keyword] = ACTIONS(1),
    [sym_switch_keyword] = ACTIONS(1),
    [sym_case_keyword] = ACTIONS(1),
    [sym_default_keyword] = ACTIONS(1),
    [sym_while_keyword] = ACTIONS(1),
    [sym_for_keyword] = ACTIONS(1),
    [sym_break_keyword] = ACTIONS(1),
    [sym_continue_keyword] = ACTIONS(1),
    [sym_return_keyword] = ACTIONS(1),
    [sym_select_keyword] = ACTIONS(1),
    [sym_active_keyword] = ACTIONS(1),
    [sym_struct_keyword] = ACTIONS(1),
    [sym_structure_keyword] = ACTIONS(1),
    [sym_button_event_keyword] = ACTIONS(1),
    [sym_channel_event_keyword] = ACTIONS(1),
    [sym_level_event_keyword] = ACTIONS(1),
    [sym_data_event_keyword] = ACTIONS(1),
    [sym_timeline_event_keyword] = ACTIONS(1),
    [sym_custom_event_keyword] = ACTIONS(1),
    [sym_push_keyword] = ACTIONS(1),
    [sym_release_keyword] = ACTIONS(1),
    [sym_hold_keyword] = ACTIONS(1),
    [sym_repeat_keyword] = ACTIONS(1),
    [aux_sym_on_keyword_token1] = ACTIONS(1),
    [aux_sym_off_keyword_token1] = ACTIONS(1),
    [sym_online_keyword] = ACTIONS(1),
    [sym_offline_keyword] = ACTIONS(1),
    [sym_onerror_keyword] = ACTIONS(1),
    [sym_string_keyword] = ACTIONS(1),
    [sym_command_keyword] = ACTIONS(1),
    [sym_standby_keyword] = ACTIONS(1),
    [sym_send_level_keyword] = ACTIONS(1),
    [sym_send_string_keyword] = ACTIONS(1),
    [sym_send_command_keyword] = ACTIONS(1),
    [sym_clear_buffer_keyword] = ACTIONS(1),
    [sym_create_buffer_keyword] = ACTIONS(1),
    [sym_create_multi_buffer_keyword] = ACTIONS(1),
    [sym_call_keyword] = ACTIONS(1),
    [sym_system_call_keyword] = ACTIONS(1),
    [sym_devchan_to_keyword] = ACTIONS(1),
    [sym_devchan_min_to_keyword] = ACTIONS(1),
    [sym_devchan_total_off_keyword] = ACTIONS(1),
    [sym_devchan_pulse_keyword] = ACTIONS(1),
    [sym_wait_keyword] = ACTIONS(1),
    [sym_cancel_wait_keyword] = ACTIONS(1),
    [sym_wait_until_keyword] = ACTIONS(1),
    [sym_cancel_wait_until_keyword] = ACTIONS(1),
    [sym_cancel_all_wait_keyword] = ACTIONS(1),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1),
    [sym_and] = ACTIONS(1),
    [sym_or] = ACTIONS(1),
    [sym_not] = ACTIONS(1),
    [sym_xor] = ACTIONS(1),
    [sym_band] = ACTIONS(1),
    [sym_bor] = ACTIONS(1),
    [sym_bxor] = ACTIONS(1),
    [sym_bnot] = ACTIONS(1),
    [sym_lshift] = ACTIONS(1),
    [sym_rshift] = ACTIONS(1),
    [sym___file__] = ACTIONS(1),
    [sym___date__] = ACTIONS(1),
    [sym___ldate__] = ACTIONS(1),
    [sym___time__] = ACTIONS(1),
    [sym___line__] = ACTIONS(1),
    [sym___name__] = ACTIONS(1),
    [sym___netlinx__] = ACTIONS(1),
    [sym_netlinx_axi_version] = ACTIONS(1),
    [sym_url_flg_tcp] = ACTIONS(1),
    [sym_url_flg_accinfopresent] = ACTIONS(1),
    [sym_url_flg_temp] = ACTIONS(1),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1),
    [sym_url_flg_stat_mask] = ACTIONS(1),
    [sym_url_flg_stat_lookup] = ACTIONS(1),
    [sym_url_flg_stat_connecting] = ACTIONS(1),
    [sym_url_flg_stat_waiting] = ACTIONS(1),
    [sym_url_flg_stat_connected] = ACTIONS(1),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1),
    [sym_first_virtual_device] = ACTIONS(1),
    [sym_first_local_port] = ACTIONS(1),
    [sym_dynamic_virtual_device] = ACTIONS(1),
    [sym_timeline_once] = ACTIONS(1),
    [sym_timeline_repeat] = ACTIONS(1),
    [sym_timeline_absolute] = ACTIONS(1),
    [sym_timeline_relative] = ACTIONS(1),
    [sym_do_push_timed_infinite] = ACTIONS(1),
    [sym_source_type_no_address] = ACTIONS(1),
    [sym_source_type_neuron_id] = ACTIONS(1),
    [sym_source_type_ip_address] = ACTIONS(1),
    [sym_source_type_axlink] = ACTIONS(1),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1),
    [sym_source_type_ip_socket_address] = ACTIONS(1),
    [sym_source_type_rs232] = ACTIONS(1),
    [sym_source_type_internal] = ACTIONS(1),
    [sym_source_type_mac_address] = ACTIONS(1),
    [sym_source_type_ipv4_port] = ACTIONS(1),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1),
    [sym_true] = ACTIONS(1),
    [sym_false] = ACTIONS(1),
    [sym_file_read_only] = ACTIONS(1),
    [sym_file_rw_new] = ACTIONS(1),
    [sym_file_rw_append] = ACTIONS(1),
    [sym_ip_tcp] = ACTIONS(1),
    [sym_ip_udp] = ACTIONS(1),
    [sym_ip_udp_2way] = ACTIONS(1),
    [sym_xml_encode_types] = ACTIONS(1),
    [sym_xml_encode_char_as_list] = ACTIONS(1),
    [sym_xml_encode_le] = ACTIONS(1),
    [sym_xml_decode_types] = ACTIONS(1),
    [sym_xml_decode_no_preserve] = ACTIONS(1),
    [sym_duet_dev_type_display_device] = ACTIONS(1),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1),
    [sym_duet_dev_type_io_device] = ACTIONS(1),
    [sym_duet_dev_type_relay_device] = ACTIONS(1),
    [sym_duet_dev_type_ups] = ACTIONS(1),
    [sym_duet_dev_type_amplifier] = ACTIONS(1),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1),
    [sym_duet_dev_type_camera] = ACTIONS(1),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1),
    [sym_duet_dev_type_dss] = ACTIONS(1),
    [sym_duet_dev_type_dvr] = ACTIONS(1),
    [sym_duet_dev_type_disc_device] = ACTIONS(1),
    [sym_duet_dev_type_document_camera] = ACTIONS(1),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1),
    [sym_duet_dev_type_hvac] = ACTIONS(1),
    [sym_duet_dev_type_keypad] = ACTIONS(1),
    [sym_duet_dev_type_light] = ACTIONS(1),
    [sym_duet_dev_type_monitor] = ACTIONS(1),
    [sym_duet_dev_type_motor] = ACTIONS(1),
    [sym_duet_dev_type_multi_window] = ACTIONS(1),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1),
    [sym_duet_dev_type_receiver] = ACTIONS(1),
    [sym_duet_dev_type_security_system] = ACTIONS(1),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1),
    [sym_duet_dev_type_settop_box] = ACTIONS(1),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1),
    [sym_duet_dev_type_switcher] = ACTIONS(1),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1),
    [sym_duet_dev_type_tv] = ACTIONS(1),
    [sym_duet_dev_type_utility] = ACTIONS(1),
    [sym_duet_dev_type_vcr] = ACTIONS(1),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1),
    [sym_duet_dev_type_video_processor] = ACTIONS(1),
    [sym_duet_dev_type_video_projector] = ACTIONS(1),
    [sym_duet_dev_type_video_wall] = ACTIONS(1),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1),
    [sym_duet_dev_type_weather] = ACTIONS(1),
    [sym_duet_dev_not_polled] = ACTIONS(1),
    [sym_duet_dev_polled] = ACTIONS(1),
    [sym_clkmgr_mode_network] = ACTIONS(1),
    [sym_clkmgr_mode_standalone] = ACTIONS(1),
    [sym_normal_standby] = ACTIONS(1),
    [sym_normal_wake] = ACTIONS(1),
    [sym_ip_multicast_ttl_option] = ACTIONS(1),
    [sym_ip_ttl_subnet] = ACTIONS(1),
    [sym_ip_ttl_site] = ACTIONS(1),
    [sym_ip_ttl_region] = ACTIONS(1),
    [sym_ip_ttl_continent] = ACTIONS(1),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1),
    [sym_ip_nodelay_on] = ACTIONS(1),
    [sym_ip_nodelay_off] = ACTIONS(1),
    [sym_amx_error] = ACTIONS(1),
    [sym_amx_warning] = ACTIONS(1),
    [sym_amx_info] = ACTIONS(1),
    [sym_amx_debug] = ACTIONS(1),
    [sym_netlinx_login_success] = ACTIONS(1),
    [sym_netlinx_login_fail] = ACTIONS(1),
    [sym_netlinx_logout] = ACTIONS(1),
    [sym_null_str] = ACTIONS(1),
    [sym_smtp_address] = ACTIONS(1),
    [sym_smtp_port_number] = ACTIONS(1),
    [sym_smtp_username] = ACTIONS(1),
    [sym_smtp_password] = ACTIONS(1),
    [sym_smtp_require_tls] = ACTIONS(1),
    [sym_smtp_from] = ACTIONS(1),
    [sym_smtp_tls_true] = ACTIONS(1),
    [sym_smtp_tls_false] = ACTIONS(1),
    [sym_listview_on_row_select_event] = ACTIONS(1),
    [sym_data_structure_unknown] = ACTIONS(1),
    [sym_data_structure_datafeed] = ACTIONS(1),
    [sym_data_structure_datarecordset] = ACTIONS(1),
    [sym_data_structure_datarecord] = ACTIONS(1),
    [sym_data_structure_datafield] = ACTIONS(1),
    [sym_data_max_content_field_count] = ACTIONS(1),
    [sym_data_max_metadata_field_count] = ACTIONS(1),
    [sym_data_max_id_length] = ACTIONS(1),
    [sym_data_max_name_length] = ACTIONS(1),
    [sym_data_max_description_length] = ACTIONS(1),
    [sym_data_max_type_length] = ACTIONS(1),
    [sym_data_max_format_length] = ACTIONS(1),
    [sym_data_max_source_length] = ACTIONS(1),
    [sym_data_max_value_length] = ACTIONS(1),
    [sym_data_max_label_length] = ACTIONS(1),
    [sym_data_max_url_length] = ACTIONS(1),
    [sym_max_last_login_info_length] = ACTIONS(1),
    [sym_data_type_unknown] = ACTIONS(1),
    [sym_data_type_string] = ACTIONS(1),
    [sym_data_type_datetime] = ACTIONS(1),
    [sym_data_type_date] = ACTIONS(1),
    [sym_data_type_time] = ACTIONS(1),
    [sym_data_type_image] = ACTIONS(1),
    [sym_data_format_url] = ACTIONS(1),
    [sym_data_format_phone] = ACTIONS(1),
    [sym_data_format_email] = ACTIONS(1),
    [sym_data_format_iso8601] = ACTIONS(1),
    [sym_auditlog_priv] = ACTIONS(1),
    [sym_configuration_priv] = ACTIONS(1),
    [sym_deviceconfig_priv] = ACTIONS(1),
    [sym_ftp_priv] = ACTIONS(1),
    [sym_http_priv] = ACTIONS(1),
    [sym_networkconfig_priv] = ACTIONS(1),
    [sym_programport_priv] = ACTIONS(1),
    [sym_remoteui_priv] = ACTIONS(1),
    [sym_securitycontrol_priv] = ACTIONS(1),
    [sym_softwaremanagement_priv] = ACTIONS(1),
    [sym_terminal_priv] = ACTIONS(1),
    [sym_tpadmin_priv] = ACTIONS(1),
    [sym_user1_priv] = ACTIONS(1),
    [sym_user2_priv] = ACTIONS(1),
    [sym_user3_priv] = ACTIONS(1),
    [sym_user4_priv] = ACTIONS(1),
    [sym_usermanagement_priv] = ACTIONS(1),
    [sym_valid_account] = ACTIONS(1),
    [sym_err_name_invalid] = ACTIONS(1),
    [sym_err_password_invalid] = ACTIONS(1),
    [sym_err_invalid_account] = ACTIONS(1),
    [sym_err_no_authorization] = ACTIONS(1),
    [sym_err_invalid_parameter] = ACTIONS(1),
    [sym_err_user_locked_out] = ACTIONS(1),
    [sym_err_user_account_expired] = ACTIONS(1),
    [sym_err_general_error] = ACTIONS(1),
    [sym_err_authentication_error] = ACTIONS(1),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1),
    [sym_tls_validate_certificate] = ACTIONS(1),
    [sym_tdata] = ACTIONS(1),
    [sym_tchannel] = ACTIONS(1),
    [sym_tlevel] = ACTIONS(1),
    [sym_tbutton] = ACTIONS(1),
    [sym_ttimeline] = ACTIONS(1),
    [sym_tcustom] = ACTIONS(1),
    [sym_url_struct] = ACTIONS(1),
    [sym_dns_struct] = ACTIONS(1),
    [sym_ip_address_struct] = ACTIONS(1),
    [sym_dev_info_struct] = ACTIONS(1),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1),
    [sym_data_feed] = ACTIONS(1),
    [sym_data_field] = ACTIONS(1),
    [sym_data_record] = ACTIONS(1),
    [sym_wc_data_feed] = ACTIONS(1),
    [sym_wc_data_field] = ACTIONS(1),
    [sym_wc_data_record] = ACTIONS(1),
    [sym_last_login_info] = ACTIONS(1),
    [sym____reserved___] = ACTIONS(1),
    [sym_date] = ACTIONS(1),
    [sym_day] = ACTIONS(1),
    [sym_time] = ACTIONS(1),
    [sym_ldate] = ACTIONS(1),
    [sym_push_device] = ACTIONS(1),
    [sym_push_channel] = ACTIONS(1),
    [sym_push_devchan] = ACTIONS(1),
    [sym_release_device] = ACTIONS(1),
    [sym_release_channel] = ACTIONS(1),
    [sym_release_devchan] = ACTIONS(1),
    [sym_master_slot] = ACTIONS(1),
    [sym_get_pulse_time] = ACTIONS(1),
    [sym_get_timer] = ACTIONS(1),
    [sym_master_sn] = ACTIONS(1),
    [sym_system_number] = ACTIONS(1),
    [sym_button] = ACTIONS(1),
    [sym_dv_channel] = ACTIONS(1),
    [sym_data] = ACTIONS(1),
    [sym_level] = ACTIONS(1),
    [sym_timeline] = ACTIONS(1),
    [sym_channel] = ACTIONS(1),
    [sym_custom] = ACTIONS(1),
    [sym_atoi] = ACTIONS(1),
    [sym_atol] = ACTIONS(1),
    [sym_atof] = ACTIONS(1),
    [sym_itoa] = ACTIONS(1),
    [sym_itohex] = ACTIONS(1),
    [sym_hextoi] = ACTIONS(1),
    [sym_ftoa] = ACTIONS(1),
    [sym_find_string] = ACTIONS(1),
    [sym_left_string] = ACTIONS(1),
    [sym_right_string] = ACTIONS(1),
    [sym_mid_string] = ACTIONS(1),
    [sym_remove_string] = ACTIONS(1),
    [sym_get_buffer_string] = ACTIONS(1),
    [sym_get_buffer_char] = ACTIONS(1),
    [sym_get_multi_buffer_string] = ACTIONS(1),
    [sym_length_string] = ACTIONS(1),
    [sym_max_length_string] = ACTIONS(1),
    [sym_set_length_string] = ACTIONS(1),
    [sym_compare_string] = ACTIONS(1),
    [sym_upper_string] = ACTIONS(1),
    [sym_lower_string] = ACTIONS(1),
    [sym_get_last] = ACTIONS(1),
    [sym_date_to_day] = ACTIONS(1),
    [sym_date_to_month] = ACTIONS(1),
    [sym_date_to_year] = ACTIONS(1),
    [sym_time_to_hour] = ACTIONS(1),
    [sym_time_to_minute] = ACTIONS(1),
    [sym_time_to_second] = ACTIONS(1),
    [sym_day_of_week] = ACTIONS(1),
    [sym_set_pulse_time] = ACTIONS(1),
    [sym_set_timer] = ACTIONS(1),
    [sym_file_open] = ACTIONS(1),
    [sym_file_close] = ACTIONS(1),
    [sym_file_read] = ACTIONS(1),
    [sym_file_read_line] = ACTIONS(1),
    [sym_file_write] = ACTIONS(1),
    [sym_file_write_line] = ACTIONS(1),
    [sym_file_copy] = ACTIONS(1),
    [sym_file_delete] = ACTIONS(1),
    [sym_file_rename] = ACTIONS(1),
    [sym_file_dir] = ACTIONS(1),
    [sym_file_getdir] = ACTIONS(1),
    [sym_file_setdir] = ACTIONS(1),
    [sym_file_create_dir] = ACTIONS(1),
    [sym_file_remove_dir] = ACTIONS(1),
    [sym_file_seek] = ACTIONS(1),
    [sym_length_array] = ACTIONS(1),
    [sym_max_length_array] = ACTIONS(1),
    [sym_set_length_array] = ACTIONS(1),
    [sym_ip_client_open] = ACTIONS(1),
    [sym_ip_client_close] = ACTIONS(1),
    [sym_ip_server_open] = ACTIONS(1),
    [sym_ip_server_close] = ACTIONS(1),
    [sym_ip_bound_client_open] = ACTIONS(1),
    [sym_ip_mc_server_open] = ACTIONS(1),
    [sym_ip_set_option] = ACTIONS(1),
    [sym_ssh_client_open] = ACTIONS(1),
    [sym_ssh_client_close] = ACTIONS(1),
    [sym_tls_client_open] = ACTIONS(1),
    [sym_tls_client_close] = ACTIONS(1),
    [sym_do_push] = ACTIONS(1),
    [sym_do_push_timed] = ACTIONS(1),
    [sym_do_release] = ACTIONS(1),
    [sym_device_id] = ACTIONS(1),
    [sym_device_info] = ACTIONS(1),
    [sym_device_id_string] = ACTIONS(1),
    [sym_device_standby] = ACTIONS(1),
    [sym_device_wake] = ACTIONS(1),
    [sym_timeline_create] = ACTIONS(1),
    [sym_timeline_kill] = ACTIONS(1),
    [sym_timeline_reload] = ACTIONS(1),
    [sym_timeline_pause] = ACTIONS(1),
    [sym_timeline_restart] = ACTIONS(1),
    [sym_timeline_set] = ACTIONS(1),
    [sym_timeline_get] = ACTIONS(1),
    [sym_timeline_active] = ACTIONS(1),
    [sym_timeline_dynamic_id] = ACTIONS(1),
    [sym_variable_to_string] = ACTIONS(1),
    [sym_string_to_variable] = ACTIONS(1),
    [sym_length_variable_to_string] = ACTIONS(1),
    [sym_type_cast] = ACTIONS(1),
    [sym_variable_to_xml] = ACTIONS(1),
    [sym_xml_to_variable] = ACTIONS(1),
    [sym_length_variable_to_xml] = ACTIONS(1),
    [sym_redirect_string] = ACTIONS(1),
    [sym_random_number] = ACTIONS(1),
    [sym_get_unique_id] = ACTIONS(1),
    [sym_get_system_number] = ACTIONS(1),
    [sym_set_system_number] = ACTIONS(1),
    [sym_get_serial_number] = ACTIONS(1),
    [sym_get_master_build] = ACTIONS(1),
    [sym_set_validation_code] = ACTIONS(1),
    [sym_reboot] = ACTIONS(1),
    [sym_load_duet_module] = ACTIONS(1),
    [sym_unload_duet_module] = ACTIONS(1),
    [sym_duet_mem_size_set] = ACTIONS(1),
    [sym_duet_mem_size_get] = ACTIONS(1),
    [sym_dynamic_application_device] = ACTIONS(1),
    [sym_dynamic_polled_port] = ACTIONS(1),
    [sym_static_port_binding] = ACTIONS(1),
    [sym_static_ip_binding] = ACTIONS(1),
    [sym_get_dns_list] = ACTIONS(1),
    [sym_set_dns_list] = ACTIONS(1),
    [sym_get_ip_address] = ACTIONS(1),
    [sym_set_ip_address] = ACTIONS(1),
    [sym_get_url_list] = ACTIONS(1),
    [sym_add_url_entry] = ACTIONS(1),
    [sym_delete_url_entry] = ACTIONS(1),
    [sym_abs_value] = ACTIONS(1),
    [sym_max_value] = ACTIONS(1),
    [sym_min_value] = ACTIONS(1),
    [sym_exp_value] = ACTIONS(1),
    [sym_log_value] = ACTIONS(1),
    [sym_log10_value] = ACTIONS(1),
    [sym_power_value] = ACTIONS(1),
    [sym_sqrt_value] = ACTIONS(1),
    [sym_raw_be] = ACTIONS(1),
    [sym_raw_le] = ACTIONS(1),
    [sym_internal_queue_size_set] = ACTIONS(1),
    [sym_internal_queue_size_get] = ACTIONS(1),
    [sym_internal_threshold_set] = ACTIONS(1),
    [sym_internal_threshold_get] = ACTIONS(1),
    [sym_rebuilt_event] = ACTIONS(1),
    [sym_do_custom_event] = ACTIONS(1),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1),
    [sym_clkmgr_set_clk_source] = ACTIONS(1),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1),
    [sym_clkmgr_get_timezone] = ACTIONS(1),
    [sym_clkmgr_set_timezone] = ACTIONS(1),
    [sym_clkmgr_get_resync_period] = ACTIONS(1),
    [sym_clkmgr_set_resync_period] = ACTIONS(1),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1),
    [sym_clkmgr_get_timeservers] = ACTIONS(1),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1),
    [sym_set_log_level] = ACTIONS(1),
    [sym_get_log_level] = ACTIONS(1),
    [sym_amx_log] = ACTIONS(1),
    [sym_smtp_server_config_set] = ACTIONS(1),
    [sym_smtp_server_config_get] = ACTIONS(1),
    [sym_smtp_send] = ACTIONS(1),
    [sym_validate_netlinx_account] = ACTIONS(1),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1),
    [sym_authenticate_certificate] = ACTIONS(1),
    [sym_audit_netlinx_session_event] = ACTIONS(1),
    [sym_audit_netlinx_generic_event] = ACTIONS(1),
    [sym_get_available_flash_disk_space] = ACTIONS(1),
    [sym_get_max_flash_disk_space] = ACTIONS(1),
    [sym_data_create_feed] = ACTIONS(1),
    [sym_data_delete_feed] = ACTIONS(1),
    [sym_data_publish_feed] = ACTIONS(1),
    [sym_data_get_published_feed] = ACTIONS(1),
    [sym_data_add_record] = ACTIONS(1),
    [sym_data_get_event_record] = ACTIONS(1),
    [sym__wc_data_create_feed] = ACTIONS(1),
    [sym__wc_data_add_record] = ACTIONS(1),
    [sym__wc_data_get_event_record] = ACTIONS(1),
    [sym_format] = ACTIONS(1),
    [sym_astro_clock] = ACTIONS(1),
    [sym_set_outdoor_temperature] = ACTIONS(1),
    [sym_set_virtual_level_count] = ACTIONS(1),
    [sym_set_virtual_channel_count] = ACTIONS(1),
    [sym_set_virtual_port_count] = ACTIONS(1),
    [sym__wc_encode_int] = ACTIONS(1),
    [sym__wc_decode_int] = ACTIONS(1),
    [sym__wc_int] = ACTIONS(1),
    [sym__wc_to_ch_int] = ACTIONS(1),
    [sym__ch_to_wc_int] = ACTIONS(1),
    [sym__wc_find_string_int] = ACTIONS(1),
    [sym__wc_left_string_int] = ACTIONS(1),
    [sym__wc_lower_string_int] = ACTIONS(1),
    [sym__wc_mid_string_int] = ACTIONS(1),
    [sym__wc_remove_string_int] = ACTIONS(1),
    [sym__wc_right_string_int] = ACTIONS(1),
    [sym__wc_upper_string_int] = ACTIONS(1),
    [sym__wc_compare_string_int] = ACTIONS(1),
    [sym__wc_get_buffer_char_int] = ACTIONS(1),
    [sym__wc_get_buffer_string_int] = ACTIONS(1),
    [sym__wc_concat_string_int] = ACTIONS(1),
    [sym__wc_file_open_int] = ACTIONS(1),
    [sym__wc_file_close_int] = ACTIONS(1),
    [sym__wc_file_read_int] = ACTIONS(1),
    [sym__wc_file_read_line_int] = ACTIONS(1),
    [sym__wc_file_write_int] = ACTIONS(1),
    [sym__wc_file_write_line_int] = ACTIONS(1),
    [anon_sym_EQ] = ACTIONS(1),
    [aux_sym_preproc_include_token1] = ACTIONS(1),
    [aux_sym_preproc_define_token1] = ACTIONS(1),
    [aux_sym_preproc_warn_token1] = ACTIONS(1),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1),
    [aux_sym_preproc_else_token1] = ACTIONS(1),
    [anon_sym_COMMA] = ACTIONS(1),
    [sym_preproc_arg] = ACTIONS(1),
    [anon_sym_LBRACK] = ACTIONS(1),
    [anon_sym_RBRACK] = ACTIONS(1),
    [anon_sym_LBRACE] = ACTIONS(1),
    [anon_sym_RBRACE] = ACTIONS(1),
    [anon_sym_COLON] = ACTIONS(1),
    [anon_sym_LPAREN] = ACTIONS(1),
    [anon_sym_RPAREN] = ACTIONS(1),
    [anon_sym_SEMI] = ACTIONS(1),
    [anon_sym_BANG] = ACTIONS(1),
    [anon_sym_TILDE] = ACTIONS(1),
    [anon_sym_DASH] = ACTIONS(1),
    [anon_sym_PLUS] = ACTIONS(1),
    [anon_sym_STAR] = ACTIONS(1),
    [anon_sym_SLASH] = ACTIONS(1),
    [anon_sym_PERCENT] = ACTIONS(1),
    [anon_sym_PIPE_PIPE] = ACTIONS(1),
    [anon_sym_AMP_AMP] = ACTIONS(1),
    [anon_sym_PIPE] = ACTIONS(1),
    [anon_sym_CARET] = ACTIONS(1),
    [anon_sym_AMP] = ACTIONS(1),
    [anon_sym_EQ_EQ] = ACTIONS(1),
    [anon_sym_BANG_EQ] = ACTIONS(1),
    [anon_sym_LT_GT] = ACTIONS(1),
    [anon_sym_GT] = ACTIONS(1),
    [anon_sym_GT_EQ] = ACTIONS(1),
    [anon_sym_LT_EQ] = ACTIONS(1),
    [anon_sym_LT] = ACTIONS(1),
    [anon_sym_LT_LT] = ACTIONS(1),
    [anon_sym_GT_GT] = ACTIONS(1),
    [anon_sym_DASH_DASH] = ACTIONS(1),
    [anon_sym_PLUS_PLUS] = ACTIONS(1),
    [anon_sym_DOT] = ACTIONS(1),
    [sym_range_operator] = ACTIONS(1),
    [anon_sym_DQUOTE] = ACTIONS(1),
    [anon_sym_SQUOTE] = ACTIONS(1),
    [sym_escape_sequence] = ACTIONS(1),
    [sym_number_literal] = ACTIONS(1),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1),
  },
  [STATE(1)] = {
    [sym_source_file] = STATE(2240),
    [sym__top_level_item] = STATE(27),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(27),
    [sym_module_name] = STATE(27),
    [sym_preproc_include] = STATE(27),
    [sym_preproc_define] = STATE(27),
    [sym_preproc_warn] = STATE(27),
    [sym_preproc_disable_warning] = STATE(27),
    [sym_preproc_if_defined] = STATE(27),
    [sym_preproc_if_not_defined] = STATE(27),
    [sym_section] = STATE(27),
    [sym_define_device_section] = STATE(398),
    [sym_define_combine_section] = STATE(398),
    [sym_define_connect_level_section] = STATE(398),
    [sym_define_constant_section] = STATE(398),
    [sym_define_type_section] = STATE(398),
    [sym_define_mutually_exclusive_section] = STATE(398),
    [sym_define_latching_section] = STATE(398),
    [sym_define_toggling_section] = STATE(398),
    [sym_define_variable_section] = STATE(398),
    [sym_define_system_variable_section] = STATE(398),
    [sym_define_event_section] = STATE(398),
    [sym_define_start_section] = STATE(398),
    [sym_define_program_section] = STATE(398),
    [sym_define_function] = STATE(27),
    [sym_define_call] = STATE(27),
    [sym_define_module] = STATE(27),
    [sym_event_definition] = STATE(27),
    [sym_data_event_definition] = STATE(399),
    [sym__data_event_declarator_list] = STATE(1893),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(399),
    [sym__timeline_event_declarator_list] = STATE(1939),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(399),
    [sym__button_event_declarator_list] = STATE(1917),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(399),
    [sym__level_event_declarator_list] = STATE(1938),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(399),
    [sym__channel_event_declarator_list] = STATE(1940),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(399),
    [sym__custom_event_declarator_list] = STATE(1944),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(399),
    [sym__push_event_declarator_list] = STATE(1941),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(399),
    [sym__release_event_declarator_list] = STATE(1961),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(27),
    [sym_storage_class_specifier] = STATE(1028),
    [sym_type_qualifier] = STATE(1013),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(1978),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(27),
    [sym_type_definition] = STATE(27),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1457),
    [sym__top_level_statement] = STATE(27),
    [sym_expression_statement] = STATE(27),
    [sym_if_statement] = STATE(27),
    [sym_switch_statement] = STATE(27),
    [sym_case_statement] = STATE(27),
    [sym_while_statement] = STATE(27),
    [sym_for_statement] = STATE(27),
    [sym_select_statement] = STATE(27),
    [sym_active_statement] = STATE(27),
    [sym_return_statement] = STATE(27),
    [sym_break_statement] = STATE(27),
    [sym_continue_statement] = STATE(27),
    [sym_devchan_operation_statement] = STATE(27),
    [sym_devchan_operation] = STATE(1878),
    [sym_send_string_statement] = STATE(27),
    [sym_send_command_statement] = STATE(27),
    [sym_send_level_statement] = STATE(27),
    [sym_create_buffer_statement] = STATE(27),
    [sym_create_multi_buffer_statement] = STATE(27),
    [sym_clear_buffer_statement] = STATE(27),
    [sym_wait_statement] = STATE(27),
    [sym_wait_until_statement] = STATE(27),
    [sym_cancel_all_wait_statement] = STATE(27),
    [sym_cancel_all_wait_until_statement] = STATE(27),
    [sym_cancel_wait_statement] = STATE(27),
    [sym_cancel_wait_until_statement] = STATE(27),
    [sym_call_statement] = STATE(27),
    [sym_system_call_statement] = STATE(27),
    [sym_expression] = STATE(1137),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1990),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(27),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [ts_builtin_sym_end] = ACTIONS(5),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(9),
    [sym_module_name_keyword] = ACTIONS(11),
    [sym_define_device_keyword] = ACTIONS(13),
    [sym_define_combine_keyword] = ACTIONS(15),
    [sym_define_constant_keyword] = ACTIONS(17),
    [sym_define_type_keyword] = ACTIONS(19),
    [sym_define_variable_keyword] = ACTIONS(21),
    [sym_define_system_variable_keyword] = ACTIONS(23),
    [sym_define_connect_level_keyword] = ACTIONS(25),
    [sym_define_latching_keyword] = ACTIONS(27),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(29),
    [sym_define_toggling_keyword] = ACTIONS(31),
    [sym_define_call_keyword] = ACTIONS(33),
    [sym_define_function_keyword] = ACTIONS(35),
    [sym_define_library_function_keyword] = ACTIONS(37),
    [sym_define_start_keyword] = ACTIONS(39),
    [sym_define_event_keyword] = ACTIONS(41),
    [sym_define_program_keyword] = ACTIONS(43),
    [sym_define_module_keyword] = ACTIONS(45),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(55),
    [sym_switch_keyword] = ACTIONS(57),
    [sym_case_keyword] = ACTIONS(59),
    [sym_default_keyword] = ACTIONS(61),
    [sym_while_keyword] = ACTIONS(63),
    [sym_for_keyword] = ACTIONS(65),
    [sym_break_keyword] = ACTIONS(67),
    [sym_continue_keyword] = ACTIONS(69),
    [sym_return_keyword] = ACTIONS(71),
    [sym_select_keyword] = ACTIONS(73),
    [sym_active_keyword] = ACTIONS(75),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(99),
    [sym_send_string_keyword] = ACTIONS(101),
    [sym_send_command_keyword] = ACTIONS(103),
    [sym_clear_buffer_keyword] = ACTIONS(105),
    [sym_create_buffer_keyword] = ACTIONS(107),
    [sym_create_multi_buffer_keyword] = ACTIONS(109),
    [sym_call_keyword] = ACTIONS(111),
    [sym_system_call_keyword] = ACTIONS(113),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(117),
    [sym_cancel_wait_keyword] = ACTIONS(119),
    [sym_wait_until_keyword] = ACTIONS(121),
    [sym_cancel_wait_until_keyword] = ACTIONS(123),
    [sym_cancel_all_wait_keyword] = ACTIONS(125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(127),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(141),
    [aux_sym_preproc_define_token1] = ACTIONS(143),
    [aux_sym_preproc_warn_token1] = ACTIONS(145),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(147),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(149),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(151),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(2)] = {
    [sym__top_level_item] = STATE(12),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(12),
    [sym_module_name] = STATE(12),
    [sym_preproc_include] = STATE(12),
    [sym_preproc_define] = STATE(12),
    [sym_preproc_warn] = STATE(12),
    [sym_preproc_disable_warning] = STATE(12),
    [sym_preproc_if_defined] = STATE(12),
    [sym_preproc_if_not_defined] = STATE(12),
    [sym_preproc_else] = STATE(2206),
    [sym_section] = STATE(12),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(12),
    [sym_define_call] = STATE(12),
    [sym_define_module] = STATE(12),
    [sym_event_definition] = STATE(12),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(12),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(12),
    [sym_type_definition] = STATE(12),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(12),
    [sym_expression_statement] = STATE(12),
    [sym_if_statement] = STATE(12),
    [sym_switch_statement] = STATE(12),
    [sym_case_statement] = STATE(12),
    [sym_while_statement] = STATE(12),
    [sym_for_statement] = STATE(12),
    [sym_select_statement] = STATE(12),
    [sym_active_statement] = STATE(12),
    [sym_return_statement] = STATE(12),
    [sym_break_statement] = STATE(12),
    [sym_continue_statement] = STATE(12),
    [sym_devchan_operation_statement] = STATE(12),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(12),
    [sym_send_command_statement] = STATE(12),
    [sym_send_level_statement] = STATE(12),
    [sym_create_buffer_statement] = STATE(12),
    [sym_create_multi_buffer_statement] = STATE(12),
    [sym_clear_buffer_statement] = STATE(12),
    [sym_wait_statement] = STATE(12),
    [sym_wait_until_statement] = STATE(12),
    [sym_cancel_all_wait_statement] = STATE(12),
    [sym_cancel_all_wait_until_statement] = STATE(12),
    [sym_cancel_wait_statement] = STATE(12),
    [sym_cancel_wait_until_statement] = STATE(12),
    [sym_call_statement] = STATE(12),
    [sym_system_call_statement] = STATE(12),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(12),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(259),
    [aux_sym_preproc_warn_token1] = ACTIONS(261),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(263),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(265),
    [anon_sym_LF] = ACTIONS(267),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(269),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(271),
    [aux_sym_preproc_else_token1] = ACTIONS(273),
    [anon_sym_LBRACK] = ACTIONS(275),
    [anon_sym_LBRACE] = ACTIONS(277),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(129),
    [anon_sym_TILDE] = ACTIONS(129),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(279),
    [anon_sym_PLUS_PLUS] = ACTIONS(279),
    [anon_sym_DQUOTE] = ACTIONS(281),
    [anon_sym_SQUOTE] = ACTIONS(283),
    [sym_number_literal] = ACTIONS(285),
    [sym_comment] = ACTIONS(287),
  },
  [STATE(3)] = {
    [sym__top_level_item] = STATE(10),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(10),
    [sym_module_name] = STATE(10),
    [sym_preproc_include] = STATE(10),
    [sym_preproc_define] = STATE(10),
    [sym_preproc_warn] = STATE(10),
    [sym_preproc_disable_warning] = STATE(10),
    [sym_preproc_if_defined] = STATE(10),
    [sym_preproc_if_not_defined] = STATE(10),
    [sym_preproc_else] = STATE(2213),
    [sym_section] = STATE(10),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(10),
    [sym_define_call] = STATE(10),
    [sym_define_module] = STATE(10),
    [sym_event_definition] = STATE(10),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(10),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(10),
    [sym_type_definition] = STATE(10),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(10),
    [sym_expression_statement] = STATE(10),
    [sym_if_statement] = STATE(10),
    [sym_switch_statement] = STATE(10),
    [sym_case_statement] = STATE(10),
    [sym_while_statement] = STATE(10),
    [sym_for_statement] = STATE(10),
    [sym_select_statement] = STATE(10),
    [sym_active_statement] = STATE(10),
    [sym_return_statement] = STATE(10),
    [sym_break_statement] = STATE(10),
    [sym_continue_statement] = STATE(10),
    [sym_devchan_operation_statement] = STATE(10),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(10),
    [sym_send_command_statement] = STATE(10),
    [sym_send_level_statement] = STATE(10),
    [sym_create_buffer_statement] = STATE(10),
    [sym_create_multi_buffer_statement] = STATE(10),
    [sym_clear_buffer_statement] = STATE(10),
    [sym_wait_statement] = STATE(10),
    [sym_wait_until_statement] = STATE(10),
    [sym_cancel_all_wait_statement] = STATE(10),
    [sym_cancel_all_wait_until_statement] = STATE(10),
    [sym_cancel_wait_statement] = STATE(10),
    [sym_cancel_wait_until_statement] = STATE(10),
    [sym_call_statement] = STATE(10),
    [sym_system_call_statement] = STATE(10),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(10),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(259),
    [aux_sym_preproc_warn_token1] = ACTIONS(261),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(263),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(265),
    [anon_sym_LF] = ACTIONS(289),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(291),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(271),
    [aux_sym_preproc_else_token1] = ACTIONS(273),
    [anon_sym_LBRACK] = ACTIONS(275),
    [anon_sym_LBRACE] = ACTIONS(277),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(129),
    [anon_sym_TILDE] = ACTIONS(129),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(279),
    [anon_sym_PLUS_PLUS] = ACTIONS(279),
    [anon_sym_DQUOTE] = ACTIONS(281),
    [anon_sym_SQUOTE] = ACTIONS(283),
    [sym_number_literal] = ACTIONS(285),
    [sym_comment] = ACTIONS(287),
  },
  [STATE(4)] = {
    [sym__top_level_item] = STATE(16),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(16),
    [sym_module_name] = STATE(16),
    [sym_preproc_include] = STATE(16),
    [sym_preproc_define] = STATE(16),
    [sym_preproc_warn] = STATE(16),
    [sym_preproc_disable_warning] = STATE(16),
    [sym_preproc_if_defined] = STATE(16),
    [sym_preproc_if_not_defined] = STATE(16),
    [sym_preproc_else] = STATE(2261),
    [sym_section] = STATE(16),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(16),
    [sym_define_call] = STATE(16),
    [sym_define_module] = STATE(16),
    [sym_event_definition] = STATE(16),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(16),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(16),
    [sym_type_definition] = STATE(16),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(16),
    [sym_expression_statement] = STATE(16),
    [sym_if_statement] = STATE(16),
    [sym_switch_statement] = STATE(16),
    [sym_case_statement] = STATE(16),
    [sym_while_statement] = STATE(16),
    [sym_for_statement] = STATE(16),
    [sym_select_statement] = STATE(16),
    [sym_active_statement] = STATE(16),
    [sym_return_statement] = STATE(16),
    [sym_break_statement] = STATE(16),
    [sym_continue_statement] = STATE(16),
    [sym_devchan_operation_statement] = STATE(16),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(16),
    [sym_send_command_statement] = STATE(16),
    [sym_send_level_statement] = STATE(16),
    [sym_create_buffer_statement] = STATE(16),
    [sym_create_multi_buffer_statement] = STATE(16),
    [sym_clear_buffer_statement] = STATE(16),
    [sym_wait_statement] = STATE(16),
    [sym_wait_until_statement] = STATE(16),
    [sym_cancel_all_wait_statement] = STATE(16),
    [sym_cancel_all_wait_until_statement] = STATE(16),
    [sym_cancel_wait_statement] = STATE(16),
    [sym_cancel_wait_until_statement] = STATE(16),
    [sym_call_statement] = STATE(16),
    [sym_system_call_statement] = STATE(16),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(16),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(259),
    [aux_sym_preproc_warn_token1] = ACTIONS(261),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(263),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(265),
    [anon_sym_LF] = ACTIONS(293),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(295),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(271),
    [aux_sym_preproc_else_token1] = ACTIONS(273),
    [anon_sym_LBRACK] = ACTIONS(275),
    [anon_sym_LBRACE] = ACTIONS(277),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(129),
    [anon_sym_TILDE] = ACTIONS(129),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(279),
    [anon_sym_PLUS_PLUS] = ACTIONS(279),
    [anon_sym_DQUOTE] = ACTIONS(281),
    [anon_sym_SQUOTE] = ACTIONS(283),
    [sym_number_literal] = ACTIONS(285),
    [sym_comment] = ACTIONS(287),
  },
  [STATE(5)] = {
    [sym__top_level_item] = STATE(15),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(15),
    [sym_module_name] = STATE(15),
    [sym_preproc_include] = STATE(15),
    [sym_preproc_define] = STATE(15),
    [sym_preproc_warn] = STATE(15),
    [sym_preproc_disable_warning] = STATE(15),
    [sym_preproc_if_defined] = STATE(15),
    [sym_preproc_if_not_defined] = STATE(15),
    [sym_preproc_else] = STATE(2051),
    [sym_section] = STATE(15),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(15),
    [sym_define_call] = STATE(15),
    [sym_define_module] = STATE(15),
    [sym_event_definition] = STATE(15),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(15),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(15),
    [sym_type_definition] = STATE(15),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(15),
    [sym_expression_statement] = STATE(15),
    [sym_if_statement] = STATE(15),
    [sym_switch_statement] = STATE(15),
    [sym_case_statement] = STATE(15),
    [sym_while_statement] = STATE(15),
    [sym_for_statement] = STATE(15),
    [sym_select_statement] = STATE(15),
    [sym_active_statement] = STATE(15),
    [sym_return_statement] = STATE(15),
    [sym_break_statement] = STATE(15),
    [sym_continue_statement] = STATE(15),
    [sym_devchan_operation_statement] = STATE(15),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(15),
    [sym_send_command_statement] = STATE(15),
    [sym_send_level_statement] = STATE(15),
    [sym_create_buffer_statement] = STATE(15),
    [sym_create_multi_buffer_statement] = STATE(15),
    [sym_clear_buffer_statement] = STATE(15),
    [sym_wait_statement] = STATE(15),
    [sym_wait_until_statement] = STATE(15),
    [sym_cancel_all_wait_statement] = STATE(15),
    [sym_cancel_all_wait_until_statement] = STATE(15),
    [sym_cancel_wait_statement] = STATE(15),
    [sym_cancel_wait_until_statement] = STATE(15),
    [sym_call_statement] = STATE(15),
    [sym_system_call_statement] = STATE(15),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(15),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(259),
    [aux_sym_preproc_warn_token1] = ACTIONS(261),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(263),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(265),
    [anon_sym_LF] = ACTIONS(297),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(299),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(271),
    [aux_sym_preproc_else_token1] = ACTIONS(273),
    [anon_sym_LBRACK] = ACTIONS(275),
    [anon_sym_LBRACE] = ACTIONS(277),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(129),
    [anon_sym_TILDE] = ACTIONS(129),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(279),
    [anon_sym_PLUS_PLUS] = ACTIONS(279),
    [anon_sym_DQUOTE] = ACTIONS(281),
    [anon_sym_SQUOTE] = ACTIONS(283),
    [sym_number_literal] = ACTIONS(285),
    [sym_comment] = ACTIONS(287),
  },
  [STATE(6)] = {
    [sym__top_level_item] = STATE(23),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(23),
    [sym_module_name] = STATE(23),
    [sym_preproc_include] = STATE(23),
    [sym_preproc_define] = STATE(23),
    [sym_preproc_warn] = STATE(23),
    [sym_preproc_disable_warning] = STATE(23),
    [sym_preproc_if_defined] = STATE(23),
    [sym_preproc_if_not_defined] = STATE(23),
    [sym_preproc_else] = STATE(2255),
    [sym_section] = STATE(23),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(23),
    [sym_define_call] = STATE(23),
    [sym_define_module] = STATE(23),
    [sym_event_definition] = STATE(23),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(23),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(23),
    [sym_type_definition] = STATE(23),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(23),
    [sym_expression_statement] = STATE(23),
    [sym_if_statement] = STATE(23),
    [sym_switch_statement] = STATE(23),
    [sym_case_statement] = STATE(23),
    [sym_while_statement] = STATE(23),
    [sym_for_statement] = STATE(23),
    [sym_select_statement] = STATE(23),
    [sym_active_statement] = STATE(23),
    [sym_return_statement] = STATE(23),
    [sym_break_statement] = STATE(23),
    [sym_continue_statement] = STATE(23),
    [sym_devchan_operation_statement] = STATE(23),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(23),
    [sym_send_command_statement] = STATE(23),
    [sym_send_level_statement] = STATE(23),
    [sym_create_buffer_statement] = STATE(23),
    [sym_create_multi_buffer_statement] = STATE(23),
    [sym_clear_buffer_statement] = STATE(23),
    [sym_wait_statement] = STATE(23),
    [sym_wait_until_statement] = STATE(23),
    [sym_cancel_all_wait_statement] = STATE(23),
    [sym_cancel_all_wait_until_statement] = STATE(23),
    [sym_cancel_wait_statement] = STATE(23),
    [sym_cancel_wait_until_statement] = STATE(23),
    [sym_call_statement] = STATE(23),
    [sym_system_call_statement] = STATE(23),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(23),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(259),
    [aux_sym_preproc_warn_token1] = ACTIONS(261),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(263),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(265),
    [anon_sym_LF] = ACTIONS(301),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(303),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(271),
    [aux_sym_preproc_else_token1] = ACTIONS(273),
    [anon_sym_LBRACK] = ACTIONS(275),
    [anon_sym_LBRACE] = ACTIONS(277),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(129),
    [anon_sym_TILDE] = ACTIONS(129),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(279),
    [anon_sym_PLUS_PLUS] = ACTIONS(279),
    [anon_sym_DQUOTE] = ACTIONS(281),
    [anon_sym_SQUOTE] = ACTIONS(283),
    [sym_number_literal] = ACTIONS(285),
    [sym_comment] = ACTIONS(287),
  },
  [STATE(7)] = {
    [sym__top_level_item] = STATE(21),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(21),
    [sym_module_name] = STATE(21),
    [sym_preproc_include] = STATE(21),
    [sym_preproc_define] = STATE(21),
    [sym_preproc_warn] = STATE(21),
    [sym_preproc_disable_warning] = STATE(21),
    [sym_preproc_if_defined] = STATE(21),
    [sym_preproc_if_not_defined] = STATE(21),
    [sym_preproc_else] = STATE(2227),
    [sym_section] = STATE(21),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(21),
    [sym_define_call] = STATE(21),
    [sym_define_module] = STATE(21),
    [sym_event_definition] = STATE(21),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(21),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(21),
    [sym_type_definition] = STATE(21),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(21),
    [sym_expression_statement] = STATE(21),
    [sym_if_statement] = STATE(21),
    [sym_switch_statement] = STATE(21),
    [sym_case_statement] = STATE(21),
    [sym_while_statement] = STATE(21),
    [sym_for_statement] = STATE(21),
    [sym_select_statement] = STATE(21),
    [sym_active_statement] = STATE(21),
    [sym_return_statement] = STATE(21),
    [sym_break_statement] = STATE(21),
    [sym_continue_statement] = STATE(21),
    [sym_devchan_operation_statement] = STATE(21),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(21),
    [sym_send_command_statement] = STATE(21),
    [sym_send_level_statement] = STATE(21),
    [sym_create_buffer_statement] = STATE(21),
    [sym_create_multi_buffer_statement] = STATE(21),
    [sym_clear_buffer_statement] = STATE(21),
    [sym_wait_statement] = STATE(21),
    [sym_wait_until_statement] = STATE(21),
    [sym_cancel_all_wait_statement] = STATE(21),
    [sym_cancel_all_wait_until_statement] = STATE(21),
    [sym_cancel_wait_statement] = STATE(21),
    [sym_cancel_wait_until_statement] = STATE(21),
    [sym_call_statement] = STATE(21),
    [sym_system_call_statement] = STATE(21),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(21),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(259),
    [aux_sym_preproc_warn_token1] = ACTIONS(261),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(263),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(265),
    [anon_sym_LF] = ACTIONS(305),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(307),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(271),
    [aux_sym_preproc_else_token1] = ACTIONS(273),
    [anon_sym_LBRACK] = ACTIONS(275),
    [anon_sym_LBRACE] = ACTIONS(277),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(129),
    [anon_sym_TILDE] = ACTIONS(129),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(279),
    [anon_sym_PLUS_PLUS] = ACTIONS(279),
    [anon_sym_DQUOTE] = ACTIONS(281),
    [anon_sym_SQUOTE] = ACTIONS(283),
    [sym_number_literal] = ACTIONS(285),
    [sym_comment] = ACTIONS(287),
  },
  [STATE(8)] = {
    [sym__top_level_item] = STATE(9),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(9),
    [sym_module_name] = STATE(9),
    [sym_preproc_include] = STATE(9),
    [sym_preproc_define] = STATE(9),
    [sym_preproc_warn] = STATE(9),
    [sym_preproc_disable_warning] = STATE(9),
    [sym_preproc_if_defined] = STATE(9),
    [sym_preproc_if_not_defined] = STATE(9),
    [sym_preproc_else] = STATE(2170),
    [sym_section] = STATE(9),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(9),
    [sym_define_call] = STATE(9),
    [sym_define_module] = STATE(9),
    [sym_event_definition] = STATE(9),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(9),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(9),
    [sym_type_definition] = STATE(9),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(9),
    [sym_expression_statement] = STATE(9),
    [sym_if_statement] = STATE(9),
    [sym_switch_statement] = STATE(9),
    [sym_case_statement] = STATE(9),
    [sym_while_statement] = STATE(9),
    [sym_for_statement] = STATE(9),
    [sym_select_statement] = STATE(9),
    [sym_active_statement] = STATE(9),
    [sym_return_statement] = STATE(9),
    [sym_break_statement] = STATE(9),
    [sym_continue_statement] = STATE(9),
    [sym_devchan_operation_statement] = STATE(9),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(9),
    [sym_send_command_statement] = STATE(9),
    [sym_send_level_statement] = STATE(9),
    [sym_create_buffer_statement] = STATE(9),
    [sym_create_multi_buffer_statement] = STATE(9),
    [sym_clear_buffer_statement] = STATE(9),
    [sym_wait_statement] = STATE(9),
    [sym_wait_until_statement] = STATE(9),
    [sym_cancel_all_wait_statement] = STATE(9),
    [sym_cancel_all_wait_until_statement] = STATE(9),
    [sym_cancel_wait_statement] = STATE(9),
    [sym_cancel_wait_until_statement] = STATE(9),
    [sym_call_statement] = STATE(9),
    [sym_system_call_statement] = STATE(9),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(9),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(317),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(9)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_preproc_else] = STATE(2160),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(325),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(10)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_preproc_else] = STATE(2067),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(327),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(11)] = {
    [sym__top_level_item] = STATE(13),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(13),
    [sym_module_name] = STATE(13),
    [sym_preproc_include] = STATE(13),
    [sym_preproc_define] = STATE(13),
    [sym_preproc_warn] = STATE(13),
    [sym_preproc_disable_warning] = STATE(13),
    [sym_preproc_if_defined] = STATE(13),
    [sym_preproc_if_not_defined] = STATE(13),
    [sym_preproc_else] = STATE(2067),
    [sym_section] = STATE(13),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(13),
    [sym_define_call] = STATE(13),
    [sym_define_module] = STATE(13),
    [sym_event_definition] = STATE(13),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(13),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(13),
    [sym_type_definition] = STATE(13),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(13),
    [sym_expression_statement] = STATE(13),
    [sym_if_statement] = STATE(13),
    [sym_switch_statement] = STATE(13),
    [sym_case_statement] = STATE(13),
    [sym_while_statement] = STATE(13),
    [sym_for_statement] = STATE(13),
    [sym_select_statement] = STATE(13),
    [sym_active_statement] = STATE(13),
    [sym_return_statement] = STATE(13),
    [sym_break_statement] = STATE(13),
    [sym_continue_statement] = STATE(13),
    [sym_devchan_operation_statement] = STATE(13),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(13),
    [sym_send_command_statement] = STATE(13),
    [sym_send_level_statement] = STATE(13),
    [sym_create_buffer_statement] = STATE(13),
    [sym_create_multi_buffer_statement] = STATE(13),
    [sym_clear_buffer_statement] = STATE(13),
    [sym_wait_statement] = STATE(13),
    [sym_wait_until_statement] = STATE(13),
    [sym_cancel_all_wait_statement] = STATE(13),
    [sym_cancel_all_wait_until_statement] = STATE(13),
    [sym_cancel_wait_statement] = STATE(13),
    [sym_cancel_wait_until_statement] = STATE(13),
    [sym_call_statement] = STATE(13),
    [sym_system_call_statement] = STATE(13),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(13),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(327),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(12)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_preproc_else] = STATE(2170),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(317),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(13)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_preproc_else] = STATE(2239),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(329),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(14)] = {
    [sym__top_level_item] = STATE(17),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(17),
    [sym_module_name] = STATE(17),
    [sym_preproc_include] = STATE(17),
    [sym_preproc_define] = STATE(17),
    [sym_preproc_warn] = STATE(17),
    [sym_preproc_disable_warning] = STATE(17),
    [sym_preproc_if_defined] = STATE(17),
    [sym_preproc_if_not_defined] = STATE(17),
    [sym_preproc_else] = STATE(2115),
    [sym_section] = STATE(17),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(17),
    [sym_define_call] = STATE(17),
    [sym_define_module] = STATE(17),
    [sym_event_definition] = STATE(17),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(17),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(17),
    [sym_type_definition] = STATE(17),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(17),
    [sym_expression_statement] = STATE(17),
    [sym_if_statement] = STATE(17),
    [sym_switch_statement] = STATE(17),
    [sym_case_statement] = STATE(17),
    [sym_while_statement] = STATE(17),
    [sym_for_statement] = STATE(17),
    [sym_select_statement] = STATE(17),
    [sym_active_statement] = STATE(17),
    [sym_return_statement] = STATE(17),
    [sym_break_statement] = STATE(17),
    [sym_continue_statement] = STATE(17),
    [sym_devchan_operation_statement] = STATE(17),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(17),
    [sym_send_command_statement] = STATE(17),
    [sym_send_level_statement] = STATE(17),
    [sym_create_buffer_statement] = STATE(17),
    [sym_create_multi_buffer_statement] = STATE(17),
    [sym_clear_buffer_statement] = STATE(17),
    [sym_wait_statement] = STATE(17),
    [sym_wait_until_statement] = STATE(17),
    [sym_cancel_all_wait_statement] = STATE(17),
    [sym_cancel_all_wait_until_statement] = STATE(17),
    [sym_cancel_wait_statement] = STATE(17),
    [sym_cancel_wait_until_statement] = STATE(17),
    [sym_call_statement] = STATE(17),
    [sym_system_call_statement] = STATE(17),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(17),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(331),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(15)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_preproc_else] = STATE(2115),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(331),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(16)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_preproc_else] = STATE(2112),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(333),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(17)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_preproc_else] = STATE(2136),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(335),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(18)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_preproc_else] = STATE(2135),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(337),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(19)] = {
    [sym__top_level_item] = STATE(18),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(18),
    [sym_module_name] = STATE(18),
    [sym_preproc_include] = STATE(18),
    [sym_preproc_define] = STATE(18),
    [sym_preproc_warn] = STATE(18),
    [sym_preproc_disable_warning] = STATE(18),
    [sym_preproc_if_defined] = STATE(18),
    [sym_preproc_if_not_defined] = STATE(18),
    [sym_preproc_else] = STATE(2112),
    [sym_section] = STATE(18),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(18),
    [sym_define_call] = STATE(18),
    [sym_define_module] = STATE(18),
    [sym_event_definition] = STATE(18),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(18),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(18),
    [sym_type_definition] = STATE(18),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(18),
    [sym_expression_statement] = STATE(18),
    [sym_if_statement] = STATE(18),
    [sym_switch_statement] = STATE(18),
    [sym_case_statement] = STATE(18),
    [sym_while_statement] = STATE(18),
    [sym_for_statement] = STATE(18),
    [sym_select_statement] = STATE(18),
    [sym_active_statement] = STATE(18),
    [sym_return_statement] = STATE(18),
    [sym_break_statement] = STATE(18),
    [sym_continue_statement] = STATE(18),
    [sym_devchan_operation_statement] = STATE(18),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(18),
    [sym_send_command_statement] = STATE(18),
    [sym_send_level_statement] = STATE(18),
    [sym_create_buffer_statement] = STATE(18),
    [sym_create_multi_buffer_statement] = STATE(18),
    [sym_clear_buffer_statement] = STATE(18),
    [sym_wait_statement] = STATE(18),
    [sym_wait_until_statement] = STATE(18),
    [sym_cancel_all_wait_statement] = STATE(18),
    [sym_cancel_all_wait_until_statement] = STATE(18),
    [sym_cancel_wait_statement] = STATE(18),
    [sym_cancel_wait_until_statement] = STATE(18),
    [sym_call_statement] = STATE(18),
    [sym_system_call_statement] = STATE(18),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(18),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(333),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(20)] = {
    [sym__top_level_item] = STATE(24),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(24),
    [sym_module_name] = STATE(24),
    [sym_preproc_include] = STATE(24),
    [sym_preproc_define] = STATE(24),
    [sym_preproc_warn] = STATE(24),
    [sym_preproc_disable_warning] = STATE(24),
    [sym_preproc_if_defined] = STATE(24),
    [sym_preproc_if_not_defined] = STATE(24),
    [sym_preproc_else] = STATE(2207),
    [sym_section] = STATE(24),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(24),
    [sym_define_call] = STATE(24),
    [sym_define_module] = STATE(24),
    [sym_event_definition] = STATE(24),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(24),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(24),
    [sym_type_definition] = STATE(24),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(24),
    [sym_expression_statement] = STATE(24),
    [sym_if_statement] = STATE(24),
    [sym_switch_statement] = STATE(24),
    [sym_case_statement] = STATE(24),
    [sym_while_statement] = STATE(24),
    [sym_for_statement] = STATE(24),
    [sym_select_statement] = STATE(24),
    [sym_active_statement] = STATE(24),
    [sym_return_statement] = STATE(24),
    [sym_break_statement] = STATE(24),
    [sym_continue_statement] = STATE(24),
    [sym_devchan_operation_statement] = STATE(24),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(24),
    [sym_send_command_statement] = STATE(24),
    [sym_send_level_statement] = STATE(24),
    [sym_create_buffer_statement] = STATE(24),
    [sym_create_multi_buffer_statement] = STATE(24),
    [sym_clear_buffer_statement] = STATE(24),
    [sym_wait_statement] = STATE(24),
    [sym_wait_until_statement] = STATE(24),
    [sym_cancel_all_wait_statement] = STATE(24),
    [sym_cancel_all_wait_until_statement] = STATE(24),
    [sym_cancel_wait_statement] = STATE(24),
    [sym_cancel_wait_until_statement] = STATE(24),
    [sym_call_statement] = STATE(24),
    [sym_system_call_statement] = STATE(24),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(24),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(339),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(21)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_preproc_else] = STATE(2207),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(339),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(22)] = {
    [sym__top_level_item] = STATE(25),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(25),
    [sym_module_name] = STATE(25),
    [sym_preproc_include] = STATE(25),
    [sym_preproc_define] = STATE(25),
    [sym_preproc_warn] = STATE(25),
    [sym_preproc_disable_warning] = STATE(25),
    [sym_preproc_if_defined] = STATE(25),
    [sym_preproc_if_not_defined] = STATE(25),
    [sym_preproc_else] = STATE(2216),
    [sym_section] = STATE(25),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(25),
    [sym_define_call] = STATE(25),
    [sym_define_module] = STATE(25),
    [sym_event_definition] = STATE(25),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(25),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(25),
    [sym_type_definition] = STATE(25),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(25),
    [sym_expression_statement] = STATE(25),
    [sym_if_statement] = STATE(25),
    [sym_switch_statement] = STATE(25),
    [sym_case_statement] = STATE(25),
    [sym_while_statement] = STATE(25),
    [sym_for_statement] = STATE(25),
    [sym_select_statement] = STATE(25),
    [sym_active_statement] = STATE(25),
    [sym_return_statement] = STATE(25),
    [sym_break_statement] = STATE(25),
    [sym_continue_statement] = STATE(25),
    [sym_devchan_operation_statement] = STATE(25),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(25),
    [sym_send_command_statement] = STATE(25),
    [sym_send_level_statement] = STATE(25),
    [sym_create_buffer_statement] = STATE(25),
    [sym_create_multi_buffer_statement] = STATE(25),
    [sym_clear_buffer_statement] = STATE(25),
    [sym_wait_statement] = STATE(25),
    [sym_wait_until_statement] = STATE(25),
    [sym_cancel_all_wait_statement] = STATE(25),
    [sym_cancel_all_wait_until_statement] = STATE(25),
    [sym_cancel_wait_statement] = STATE(25),
    [sym_cancel_wait_until_statement] = STATE(25),
    [sym_call_statement] = STATE(25),
    [sym_system_call_statement] = STATE(25),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(25),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(341),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(23)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_preproc_else] = STATE(2216),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(341),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(24)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_preproc_else] = STATE(2053),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(343),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(25)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_preproc_else] = STATE(2054),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(345),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [aux_sym_preproc_else_token1] = ACTIONS(321),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(26)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(347),
    [sym_program_name_keyword] = ACTIONS(350),
    [sym_module_name_keyword] = ACTIONS(353),
    [sym_define_device_keyword] = ACTIONS(356),
    [sym_define_combine_keyword] = ACTIONS(359),
    [sym_define_constant_keyword] = ACTIONS(362),
    [sym_define_type_keyword] = ACTIONS(365),
    [sym_define_variable_keyword] = ACTIONS(368),
    [sym_define_system_variable_keyword] = ACTIONS(371),
    [sym_define_connect_level_keyword] = ACTIONS(374),
    [sym_define_latching_keyword] = ACTIONS(377),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(380),
    [sym_define_toggling_keyword] = ACTIONS(383),
    [sym_define_call_keyword] = ACTIONS(386),
    [sym_define_function_keyword] = ACTIONS(389),
    [sym_define_library_function_keyword] = ACTIONS(392),
    [sym_define_start_keyword] = ACTIONS(395),
    [sym_define_event_keyword] = ACTIONS(398),
    [sym_define_program_keyword] = ACTIONS(401),
    [sym_define_module_keyword] = ACTIONS(404),
    [sym_char_keyword] = ACTIONS(407),
    [sym_widechar_keyword] = ACTIONS(407),
    [sym_integer_keyword] = ACTIONS(407),
    [sym_sinteger_keyword] = ACTIONS(407),
    [sym_long_keyword] = ACTIONS(407),
    [sym_slong_keyword] = ACTIONS(407),
    [sym_float_keyword] = ACTIONS(407),
    [sym_double_keyword] = ACTIONS(407),
    [sym_variant_keyword] = ACTIONS(407),
    [sym_variantarray_keyword] = ACTIONS(407),
    [sym_local_var_keyword] = ACTIONS(410),
    [sym_stack_var_keyword] = ACTIONS(410),
    [sym_constant_keyword] = ACTIONS(413),
    [sym_volatile_keyword] = ACTIONS(413),
    [sym_non_volatile_keyword] = ACTIONS(413),
    [sym_persistent_keyword] = ACTIONS(413),
    [sym_dev_keyword] = ACTIONS(416),
    [sym_devlev_keyword] = ACTIONS(416),
    [sym_devchan_keyword] = ACTIONS(416),
    [sym_if_keyword] = ACTIONS(419),
    [sym_switch_keyword] = ACTIONS(422),
    [sym_case_keyword] = ACTIONS(425),
    [sym_default_keyword] = ACTIONS(428),
    [sym_while_keyword] = ACTIONS(431),
    [sym_for_keyword] = ACTIONS(434),
    [sym_break_keyword] = ACTIONS(437),
    [sym_continue_keyword] = ACTIONS(440),
    [sym_return_keyword] = ACTIONS(443),
    [sym_select_keyword] = ACTIONS(446),
    [sym_active_keyword] = ACTIONS(449),
    [sym_struct_keyword] = ACTIONS(452),
    [sym_structure_keyword] = ACTIONS(452),
    [sym_button_event_keyword] = ACTIONS(455),
    [sym_channel_event_keyword] = ACTIONS(458),
    [sym_level_event_keyword] = ACTIONS(461),
    [sym_data_event_keyword] = ACTIONS(464),
    [sym_timeline_event_keyword] = ACTIONS(467),
    [sym_custom_event_keyword] = ACTIONS(470),
    [sym_push_keyword] = ACTIONS(473),
    [sym_release_keyword] = ACTIONS(476),
    [aux_sym_on_keyword_token1] = ACTIONS(479),
    [aux_sym_off_keyword_token1] = ACTIONS(482),
    [sym_send_level_keyword] = ACTIONS(485),
    [sym_send_string_keyword] = ACTIONS(488),
    [sym_send_command_keyword] = ACTIONS(491),
    [sym_clear_buffer_keyword] = ACTIONS(494),
    [sym_create_buffer_keyword] = ACTIONS(497),
    [sym_create_multi_buffer_keyword] = ACTIONS(500),
    [sym_call_keyword] = ACTIONS(503),
    [sym_system_call_keyword] = ACTIONS(506),
    [sym_devchan_to_keyword] = ACTIONS(509),
    [sym_devchan_min_to_keyword] = ACTIONS(509),
    [sym_devchan_total_off_keyword] = ACTIONS(509),
    [sym_devchan_pulse_keyword] = ACTIONS(509),
    [sym_wait_keyword] = ACTIONS(512),
    [sym_cancel_wait_keyword] = ACTIONS(515),
    [sym_wait_until_keyword] = ACTIONS(518),
    [sym_cancel_wait_until_keyword] = ACTIONS(521),
    [sym_cancel_all_wait_keyword] = ACTIONS(524),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(527),
    [sym_not] = ACTIONS(530),
    [sym_bnot] = ACTIONS(530),
    [sym___file__] = ACTIONS(533),
    [sym___date__] = ACTIONS(533),
    [sym___ldate__] = ACTIONS(533),
    [sym___time__] = ACTIONS(533),
    [sym___line__] = ACTIONS(533),
    [sym___name__] = ACTIONS(533),
    [sym___netlinx__] = ACTIONS(536),
    [sym_netlinx_axi_version] = ACTIONS(536),
    [sym_url_flg_tcp] = ACTIONS(536),
    [sym_url_flg_accinfopresent] = ACTIONS(536),
    [sym_url_flg_temp] = ACTIONS(536),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(536),
    [sym_url_flg_stat_mask] = ACTIONS(536),
    [sym_url_flg_stat_lookup] = ACTIONS(536),
    [sym_url_flg_stat_connecting] = ACTIONS(536),
    [sym_url_flg_stat_waiting] = ACTIONS(536),
    [sym_url_flg_stat_connected] = ACTIONS(536),
    [sym_ip_addr_flg_dhcp] = ACTIONS(536),
    [sym_first_virtual_device] = ACTIONS(536),
    [sym_first_local_port] = ACTIONS(536),
    [sym_dynamic_virtual_device] = ACTIONS(536),
    [sym_timeline_once] = ACTIONS(536),
    [sym_timeline_repeat] = ACTIONS(536),
    [sym_timeline_absolute] = ACTIONS(536),
    [sym_timeline_relative] = ACTIONS(536),
    [sym_do_push_timed_infinite] = ACTIONS(536),
    [sym_source_type_no_address] = ACTIONS(536),
    [sym_source_type_neuron_id] = ACTIONS(536),
    [sym_source_type_ip_address] = ACTIONS(536),
    [sym_source_type_axlink] = ACTIONS(536),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(536),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(536),
    [sym_source_type_ip_socket_address] = ACTIONS(536),
    [sym_source_type_rs232] = ACTIONS(536),
    [sym_source_type_internal] = ACTIONS(536),
    [sym_source_type_mac_address] = ACTIONS(536),
    [sym_source_type_ipv4_port] = ACTIONS(536),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(536),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(536),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(536),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(536),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(536),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(536),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(536),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(536),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(536),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(536),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(536),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(536),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(536),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(536),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(536),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(536),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(536),
    [sym_true] = ACTIONS(536),
    [sym_false] = ACTIONS(536),
    [sym_file_read_only] = ACTIONS(536),
    [sym_file_rw_new] = ACTIONS(536),
    [sym_file_rw_append] = ACTIONS(536),
    [sym_ip_tcp] = ACTIONS(536),
    [sym_ip_udp] = ACTIONS(536),
    [sym_ip_udp_2way] = ACTIONS(536),
    [sym_xml_encode_types] = ACTIONS(536),
    [sym_xml_encode_char_as_list] = ACTIONS(536),
    [sym_xml_encode_le] = ACTIONS(536),
    [sym_xml_decode_types] = ACTIONS(536),
    [sym_xml_decode_no_preserve] = ACTIONS(536),
    [sym_duet_dev_type_display_device] = ACTIONS(536),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(536),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(536),
    [sym_duet_dev_type_io_device] = ACTIONS(536),
    [sym_duet_dev_type_relay_device] = ACTIONS(536),
    [sym_duet_dev_type_ups] = ACTIONS(536),
    [sym_duet_dev_type_amplifier] = ACTIONS(536),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(536),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(536),
    [sym_duet_dev_type_audio_processor] = ACTIONS(536),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(536),
    [sym_duet_dev_type_camera] = ACTIONS(536),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(536),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(536),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(536),
    [sym_duet_dev_type_dss] = ACTIONS(536),
    [sym_duet_dev_type_dvr] = ACTIONS(536),
    [sym_duet_dev_type_disc_device] = ACTIONS(536),
    [sym_duet_dev_type_document_camera] = ACTIONS(536),
    [sym_duet_dev_type_audio_tape] = ACTIONS(536),
    [sym_duet_dev_type_hvac] = ACTIONS(536),
    [sym_duet_dev_type_keypad] = ACTIONS(536),
    [sym_duet_dev_type_light] = ACTIONS(536),
    [sym_duet_dev_type_monitor] = ACTIONS(536),
    [sym_duet_dev_type_motor] = ACTIONS(536),
    [sym_duet_dev_type_multi_window] = ACTIONS(536),
    [sym_duet_dev_type_pool_spa] = ACTIONS(536),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(536),
    [sym_duet_dev_type_receiver] = ACTIONS(536),
    [sym_duet_dev_type_security_system] = ACTIONS(536),
    [sym_duet_dev_type_sensor_device] = ACTIONS(536),
    [sym_duet_dev_type_settop_box] = ACTIONS(536),
    [sym_duet_dev_type_slide_projector] = ACTIONS(536),
    [sym_duet_dev_type_switcher] = ACTIONS(536),
    [sym_duet_dev_type_text_keypad] = ACTIONS(536),
    [sym_duet_dev_type_tv] = ACTIONS(536),
    [sym_duet_dev_type_utility] = ACTIONS(536),
    [sym_duet_dev_type_vcr] = ACTIONS(536),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(536),
    [sym_duet_dev_type_video_processor] = ACTIONS(536),
    [sym_duet_dev_type_video_projector] = ACTIONS(536),
    [sym_duet_dev_type_video_wall] = ACTIONS(536),
    [sym_duet_dev_type_volume_controller] = ACTIONS(536),
    [sym_duet_dev_type_weather] = ACTIONS(536),
    [sym_duet_dev_not_polled] = ACTIONS(536),
    [sym_duet_dev_polled] = ACTIONS(536),
    [sym_clkmgr_mode_network] = ACTIONS(536),
    [sym_clkmgr_mode_standalone] = ACTIONS(536),
    [sym_normal_standby] = ACTIONS(536),
    [sym_normal_wake] = ACTIONS(536),
    [sym_ip_multicast_ttl_option] = ACTIONS(536),
    [sym_ip_ttl_subnet] = ACTIONS(536),
    [sym_ip_ttl_site] = ACTIONS(536),
    [sym_ip_ttl_region] = ACTIONS(536),
    [sym_ip_ttl_continent] = ACTIONS(536),
    [sym_ip_tcp_nodelay_option] = ACTIONS(536),
    [sym_ip_nodelay_on] = ACTIONS(536),
    [sym_ip_nodelay_off] = ACTIONS(536),
    [sym_amx_error] = ACTIONS(536),
    [sym_amx_warning] = ACTIONS(536),
    [sym_amx_info] = ACTIONS(536),
    [sym_amx_debug] = ACTIONS(536),
    [sym_netlinx_login_success] = ACTIONS(536),
    [sym_netlinx_login_fail] = ACTIONS(536),
    [sym_netlinx_logout] = ACTIONS(536),
    [sym_null_str] = ACTIONS(536),
    [sym_smtp_address] = ACTIONS(536),
    [sym_smtp_port_number] = ACTIONS(536),
    [sym_smtp_username] = ACTIONS(536),
    [sym_smtp_password] = ACTIONS(536),
    [sym_smtp_require_tls] = ACTIONS(536),
    [sym_smtp_from] = ACTIONS(536),
    [sym_smtp_tls_true] = ACTIONS(536),
    [sym_smtp_tls_false] = ACTIONS(536),
    [sym_listview_on_row_select_event] = ACTIONS(536),
    [sym_data_structure_unknown] = ACTIONS(536),
    [sym_data_structure_datafeed] = ACTIONS(536),
    [sym_data_structure_datarecordset] = ACTIONS(536),
    [sym_data_structure_datarecord] = ACTIONS(536),
    [sym_data_structure_datafield] = ACTIONS(536),
    [sym_data_max_content_field_count] = ACTIONS(536),
    [sym_data_max_metadata_field_count] = ACTIONS(536),
    [sym_data_max_id_length] = ACTIONS(536),
    [sym_data_max_name_length] = ACTIONS(536),
    [sym_data_max_description_length] = ACTIONS(536),
    [sym_data_max_type_length] = ACTIONS(536),
    [sym_data_max_format_length] = ACTIONS(536),
    [sym_data_max_source_length] = ACTIONS(536),
    [sym_data_max_value_length] = ACTIONS(536),
    [sym_data_max_label_length] = ACTIONS(536),
    [sym_data_max_url_length] = ACTIONS(536),
    [sym_max_last_login_info_length] = ACTIONS(536),
    [sym_data_type_unknown] = ACTIONS(536),
    [sym_data_type_string] = ACTIONS(536),
    [sym_data_type_datetime] = ACTIONS(536),
    [sym_data_type_date] = ACTIONS(536),
    [sym_data_type_time] = ACTIONS(536),
    [sym_data_type_image] = ACTIONS(536),
    [sym_data_format_url] = ACTIONS(536),
    [sym_data_format_phone] = ACTIONS(536),
    [sym_data_format_email] = ACTIONS(536),
    [sym_data_format_iso8601] = ACTIONS(536),
    [sym_auditlog_priv] = ACTIONS(536),
    [sym_configuration_priv] = ACTIONS(536),
    [sym_deviceconfig_priv] = ACTIONS(536),
    [sym_ftp_priv] = ACTIONS(536),
    [sym_http_priv] = ACTIONS(536),
    [sym_networkconfig_priv] = ACTIONS(536),
    [sym_programport_priv] = ACTIONS(536),
    [sym_remoteui_priv] = ACTIONS(536),
    [sym_securitycontrol_priv] = ACTIONS(536),
    [sym_softwaremanagement_priv] = ACTIONS(536),
    [sym_terminal_priv] = ACTIONS(536),
    [sym_tpadmin_priv] = ACTIONS(536),
    [sym_user1_priv] = ACTIONS(536),
    [sym_user2_priv] = ACTIONS(536),
    [sym_user3_priv] = ACTIONS(536),
    [sym_user4_priv] = ACTIONS(536),
    [sym_usermanagement_priv] = ACTIONS(536),
    [sym_valid_account] = ACTIONS(536),
    [sym_err_name_invalid] = ACTIONS(536),
    [sym_err_password_invalid] = ACTIONS(536),
    [sym_err_invalid_account] = ACTIONS(536),
    [sym_err_no_authorization] = ACTIONS(536),
    [sym_err_invalid_parameter] = ACTIONS(536),
    [sym_err_user_locked_out] = ACTIONS(536),
    [sym_err_user_account_expired] = ACTIONS(536),
    [sym_err_general_error] = ACTIONS(536),
    [sym_err_authentication_error] = ACTIONS(536),
    [sym_tls_ignore_certificate_errors] = ACTIONS(536),
    [sym_tls_validate_certificate] = ACTIONS(536),
    [sym_tdata] = ACTIONS(539),
    [sym_tchannel] = ACTIONS(539),
    [sym_tlevel] = ACTIONS(539),
    [sym_tbutton] = ACTIONS(539),
    [sym_ttimeline] = ACTIONS(539),
    [sym_tcustom] = ACTIONS(539),
    [sym_url_struct] = ACTIONS(539),
    [sym_dns_struct] = ACTIONS(539),
    [sym_ip_address_struct] = ACTIONS(539),
    [sym_dev_info_struct] = ACTIONS(539),
    [sym_clkmgr_timeserver_struct] = ACTIONS(539),
    [sym_data_feed] = ACTIONS(539),
    [sym_data_field] = ACTIONS(539),
    [sym_data_record] = ACTIONS(539),
    [sym_wc_data_feed] = ACTIONS(539),
    [sym_wc_data_field] = ACTIONS(539),
    [sym_wc_data_record] = ACTIONS(539),
    [sym_last_login_info] = ACTIONS(539),
    [sym____reserved___] = ACTIONS(542),
    [sym_date] = ACTIONS(542),
    [sym_day] = ACTIONS(542),
    [sym_time] = ACTIONS(542),
    [sym_ldate] = ACTIONS(542),
    [sym_push_device] = ACTIONS(542),
    [sym_push_channel] = ACTIONS(542),
    [sym_push_devchan] = ACTIONS(542),
    [sym_release_device] = ACTIONS(542),
    [sym_release_channel] = ACTIONS(542),
    [sym_release_devchan] = ACTIONS(542),
    [sym_master_slot] = ACTIONS(542),
    [sym_get_pulse_time] = ACTIONS(542),
    [sym_get_timer] = ACTIONS(542),
    [sym_master_sn] = ACTIONS(542),
    [sym_system_number] = ACTIONS(542),
    [sym_button] = ACTIONS(542),
    [sym_dv_channel] = ACTIONS(542),
    [sym_data] = ACTIONS(542),
    [sym_level] = ACTIONS(542),
    [sym_timeline] = ACTIONS(542),
    [sym_channel] = ACTIONS(542),
    [sym_custom] = ACTIONS(542),
    [sym_atoi] = ACTIONS(545),
    [sym_atol] = ACTIONS(545),
    [sym_atof] = ACTIONS(545),
    [sym_itoa] = ACTIONS(545),
    [sym_itohex] = ACTIONS(545),
    [sym_hextoi] = ACTIONS(545),
    [sym_ftoa] = ACTIONS(545),
    [sym_find_string] = ACTIONS(545),
    [sym_left_string] = ACTIONS(545),
    [sym_right_string] = ACTIONS(545),
    [sym_mid_string] = ACTIONS(545),
    [sym_remove_string] = ACTIONS(545),
    [sym_get_buffer_string] = ACTIONS(545),
    [sym_get_buffer_char] = ACTIONS(545),
    [sym_get_multi_buffer_string] = ACTIONS(545),
    [sym_length_string] = ACTIONS(545),
    [sym_max_length_string] = ACTIONS(545),
    [sym_set_length_string] = ACTIONS(545),
    [sym_compare_string] = ACTIONS(545),
    [sym_upper_string] = ACTIONS(545),
    [sym_lower_string] = ACTIONS(545),
    [sym_get_last] = ACTIONS(545),
    [sym_date_to_day] = ACTIONS(545),
    [sym_date_to_month] = ACTIONS(545),
    [sym_date_to_year] = ACTIONS(545),
    [sym_time_to_hour] = ACTIONS(545),
    [sym_time_to_minute] = ACTIONS(545),
    [sym_time_to_second] = ACTIONS(545),
    [sym_day_of_week] = ACTIONS(545),
    [sym_set_pulse_time] = ACTIONS(545),
    [sym_set_timer] = ACTIONS(545),
    [sym_file_open] = ACTIONS(545),
    [sym_file_close] = ACTIONS(545),
    [sym_file_read] = ACTIONS(545),
    [sym_file_read_line] = ACTIONS(545),
    [sym_file_write] = ACTIONS(545),
    [sym_file_write_line] = ACTIONS(545),
    [sym_file_copy] = ACTIONS(545),
    [sym_file_delete] = ACTIONS(545),
    [sym_file_rename] = ACTIONS(545),
    [sym_file_dir] = ACTIONS(545),
    [sym_file_getdir] = ACTIONS(545),
    [sym_file_setdir] = ACTIONS(545),
    [sym_file_create_dir] = ACTIONS(545),
    [sym_file_remove_dir] = ACTIONS(545),
    [sym_file_seek] = ACTIONS(545),
    [sym_length_array] = ACTIONS(545),
    [sym_max_length_array] = ACTIONS(545),
    [sym_set_length_array] = ACTIONS(545),
    [sym_ip_client_open] = ACTIONS(545),
    [sym_ip_client_close] = ACTIONS(545),
    [sym_ip_server_open] = ACTIONS(545),
    [sym_ip_server_close] = ACTIONS(545),
    [sym_ip_bound_client_open] = ACTIONS(545),
    [sym_ip_mc_server_open] = ACTIONS(545),
    [sym_ip_set_option] = ACTIONS(545),
    [sym_ssh_client_open] = ACTIONS(545),
    [sym_ssh_client_close] = ACTIONS(545),
    [sym_tls_client_open] = ACTIONS(545),
    [sym_tls_client_close] = ACTIONS(545),
    [sym_do_push] = ACTIONS(545),
    [sym_do_push_timed] = ACTIONS(545),
    [sym_do_release] = ACTIONS(545),
    [sym_device_id] = ACTIONS(545),
    [sym_device_info] = ACTIONS(545),
    [sym_device_id_string] = ACTIONS(545),
    [sym_device_standby] = ACTIONS(545),
    [sym_device_wake] = ACTIONS(545),
    [sym_timeline_create] = ACTIONS(545),
    [sym_timeline_kill] = ACTIONS(545),
    [sym_timeline_reload] = ACTIONS(545),
    [sym_timeline_pause] = ACTIONS(545),
    [sym_timeline_restart] = ACTIONS(545),
    [sym_timeline_set] = ACTIONS(545),
    [sym_timeline_get] = ACTIONS(545),
    [sym_timeline_active] = ACTIONS(545),
    [sym_timeline_dynamic_id] = ACTIONS(545),
    [sym_variable_to_string] = ACTIONS(545),
    [sym_string_to_variable] = ACTIONS(545),
    [sym_length_variable_to_string] = ACTIONS(545),
    [sym_type_cast] = ACTIONS(545),
    [sym_variable_to_xml] = ACTIONS(545),
    [sym_xml_to_variable] = ACTIONS(545),
    [sym_length_variable_to_xml] = ACTIONS(545),
    [sym_redirect_string] = ACTIONS(545),
    [sym_random_number] = ACTIONS(545),
    [sym_get_unique_id] = ACTIONS(545),
    [sym_get_system_number] = ACTIONS(545),
    [sym_set_system_number] = ACTIONS(545),
    [sym_get_serial_number] = ACTIONS(545),
    [sym_get_master_build] = ACTIONS(545),
    [sym_set_validation_code] = ACTIONS(545),
    [sym_reboot] = ACTIONS(545),
    [sym_load_duet_module] = ACTIONS(545),
    [sym_unload_duet_module] = ACTIONS(545),
    [sym_duet_mem_size_set] = ACTIONS(545),
    [sym_duet_mem_size_get] = ACTIONS(545),
    [sym_dynamic_application_device] = ACTIONS(545),
    [sym_dynamic_polled_port] = ACTIONS(545),
    [sym_static_port_binding] = ACTIONS(545),
    [sym_static_ip_binding] = ACTIONS(545),
    [sym_get_dns_list] = ACTIONS(545),
    [sym_set_dns_list] = ACTIONS(545),
    [sym_get_ip_address] = ACTIONS(545),
    [sym_set_ip_address] = ACTIONS(545),
    [sym_get_url_list] = ACTIONS(545),
    [sym_add_url_entry] = ACTIONS(545),
    [sym_delete_url_entry] = ACTIONS(545),
    [sym_abs_value] = ACTIONS(545),
    [sym_max_value] = ACTIONS(545),
    [sym_min_value] = ACTIONS(545),
    [sym_exp_value] = ACTIONS(545),
    [sym_log_value] = ACTIONS(545),
    [sym_log10_value] = ACTIONS(545),
    [sym_power_value] = ACTIONS(545),
    [sym_sqrt_value] = ACTIONS(545),
    [sym_raw_be] = ACTIONS(545),
    [sym_raw_le] = ACTIONS(545),
    [sym_internal_queue_size_set] = ACTIONS(545),
    [sym_internal_queue_size_get] = ACTIONS(545),
    [sym_internal_threshold_set] = ACTIONS(545),
    [sym_internal_threshold_get] = ACTIONS(545),
    [sym_rebuilt_event] = ACTIONS(545),
    [sym_do_custom_event] = ACTIONS(545),
    [sym_clkmgr_is_network_sourced] = ACTIONS(545),
    [sym_clkmgr_set_clk_source] = ACTIONS(545),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(545),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(545),
    [sym_clkmgr_get_timezone] = ACTIONS(545),
    [sym_clkmgr_set_timezone] = ACTIONS(545),
    [sym_clkmgr_get_resync_period] = ACTIONS(545),
    [sym_clkmgr_set_resync_period] = ACTIONS(545),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(545),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(545),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(545),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(545),
    [sym_clkmgr_get_timeservers] = ACTIONS(545),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(545),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(545),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(545),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(545),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(545),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(545),
    [sym_set_log_level] = ACTIONS(545),
    [sym_get_log_level] = ACTIONS(545),
    [sym_amx_log] = ACTIONS(545),
    [sym_smtp_server_config_set] = ACTIONS(545),
    [sym_smtp_server_config_get] = ACTIONS(545),
    [sym_smtp_send] = ACTIONS(545),
    [sym_validate_netlinx_account] = ACTIONS(545),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(545),
    [sym_authenticate_certificate] = ACTIONS(545),
    [sym_audit_netlinx_session_event] = ACTIONS(545),
    [sym_audit_netlinx_generic_event] = ACTIONS(545),
    [sym_get_available_flash_disk_space] = ACTIONS(545),
    [sym_get_max_flash_disk_space] = ACTIONS(545),
    [sym_data_create_feed] = ACTIONS(545),
    [sym_data_delete_feed] = ACTIONS(545),
    [sym_data_publish_feed] = ACTIONS(545),
    [sym_data_get_published_feed] = ACTIONS(545),
    [sym_data_add_record] = ACTIONS(545),
    [sym_data_get_event_record] = ACTIONS(545),
    [sym__wc_data_create_feed] = ACTIONS(545),
    [sym__wc_data_add_record] = ACTIONS(545),
    [sym__wc_data_get_event_record] = ACTIONS(545),
    [sym_format] = ACTIONS(545),
    [sym_astro_clock] = ACTIONS(545),
    [sym_set_outdoor_temperature] = ACTIONS(545),
    [sym_set_virtual_level_count] = ACTIONS(545),
    [sym_set_virtual_channel_count] = ACTIONS(545),
    [sym_set_virtual_port_count] = ACTIONS(545),
    [sym__wc_encode_int] = ACTIONS(545),
    [sym__wc_decode_int] = ACTIONS(545),
    [sym__wc_int] = ACTIONS(545),
    [sym__wc_to_ch_int] = ACTIONS(545),
    [sym__ch_to_wc_int] = ACTIONS(545),
    [sym__wc_find_string_int] = ACTIONS(545),
    [sym__wc_left_string_int] = ACTIONS(545),
    [sym__wc_lower_string_int] = ACTIONS(545),
    [sym__wc_mid_string_int] = ACTIONS(545),
    [sym__wc_remove_string_int] = ACTIONS(545),
    [sym__wc_right_string_int] = ACTIONS(545),
    [sym__wc_upper_string_int] = ACTIONS(545),
    [sym__wc_compare_string_int] = ACTIONS(545),
    [sym__wc_get_buffer_char_int] = ACTIONS(545),
    [sym__wc_get_buffer_string_int] = ACTIONS(545),
    [sym__wc_concat_string_int] = ACTIONS(545),
    [sym__wc_file_open_int] = ACTIONS(545),
    [sym__wc_file_close_int] = ACTIONS(545),
    [sym__wc_file_read_int] = ACTIONS(545),
    [sym__wc_file_read_line_int] = ACTIONS(545),
    [sym__wc_file_write_int] = ACTIONS(545),
    [sym__wc_file_write_line_int] = ACTIONS(545),
    [aux_sym_preproc_include_token1] = ACTIONS(548),
    [aux_sym_preproc_define_token1] = ACTIONS(551),
    [aux_sym_preproc_warn_token1] = ACTIONS(554),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(557),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(560),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(563),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(565),
    [aux_sym_preproc_else_token1] = ACTIONS(563),
    [anon_sym_LBRACK] = ACTIONS(568),
    [anon_sym_LBRACE] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(574),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_TILDE] = ACTIONS(577),
    [anon_sym_DASH] = ACTIONS(530),
    [anon_sym_PLUS] = ACTIONS(530),
    [anon_sym_DASH_DASH] = ACTIONS(580),
    [anon_sym_PLUS_PLUS] = ACTIONS(580),
    [anon_sym_DQUOTE] = ACTIONS(583),
    [anon_sym_SQUOTE] = ACTIONS(586),
    [sym_number_literal] = ACTIONS(589),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(27)] = {
    [sym__top_level_item] = STATE(29),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(29),
    [sym_module_name] = STATE(29),
    [sym_preproc_include] = STATE(29),
    [sym_preproc_define] = STATE(29),
    [sym_preproc_warn] = STATE(29),
    [sym_preproc_disable_warning] = STATE(29),
    [sym_preproc_if_defined] = STATE(29),
    [sym_preproc_if_not_defined] = STATE(29),
    [sym_section] = STATE(29),
    [sym_define_device_section] = STATE(398),
    [sym_define_combine_section] = STATE(398),
    [sym_define_connect_level_section] = STATE(398),
    [sym_define_constant_section] = STATE(398),
    [sym_define_type_section] = STATE(398),
    [sym_define_mutually_exclusive_section] = STATE(398),
    [sym_define_latching_section] = STATE(398),
    [sym_define_toggling_section] = STATE(398),
    [sym_define_variable_section] = STATE(398),
    [sym_define_system_variable_section] = STATE(398),
    [sym_define_event_section] = STATE(398),
    [sym_define_start_section] = STATE(398),
    [sym_define_program_section] = STATE(398),
    [sym_define_function] = STATE(29),
    [sym_define_call] = STATE(29),
    [sym_define_module] = STATE(29),
    [sym_event_definition] = STATE(29),
    [sym_data_event_definition] = STATE(399),
    [sym__data_event_declarator_list] = STATE(1893),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(399),
    [sym__timeline_event_declarator_list] = STATE(1939),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(399),
    [sym__button_event_declarator_list] = STATE(1917),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(399),
    [sym__level_event_declarator_list] = STATE(1938),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(399),
    [sym__channel_event_declarator_list] = STATE(1940),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(399),
    [sym__custom_event_declarator_list] = STATE(1944),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(399),
    [sym__push_event_declarator_list] = STATE(1941),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(399),
    [sym__release_event_declarator_list] = STATE(1961),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(29),
    [sym_storage_class_specifier] = STATE(1028),
    [sym_type_qualifier] = STATE(1013),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(1978),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(29),
    [sym_type_definition] = STATE(29),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1457),
    [sym__top_level_statement] = STATE(29),
    [sym_expression_statement] = STATE(29),
    [sym_if_statement] = STATE(29),
    [sym_switch_statement] = STATE(29),
    [sym_case_statement] = STATE(29),
    [sym_while_statement] = STATE(29),
    [sym_for_statement] = STATE(29),
    [sym_select_statement] = STATE(29),
    [sym_active_statement] = STATE(29),
    [sym_return_statement] = STATE(29),
    [sym_break_statement] = STATE(29),
    [sym_continue_statement] = STATE(29),
    [sym_devchan_operation_statement] = STATE(29),
    [sym_devchan_operation] = STATE(1878),
    [sym_send_string_statement] = STATE(29),
    [sym_send_command_statement] = STATE(29),
    [sym_send_level_statement] = STATE(29),
    [sym_create_buffer_statement] = STATE(29),
    [sym_create_multi_buffer_statement] = STATE(29),
    [sym_clear_buffer_statement] = STATE(29),
    [sym_wait_statement] = STATE(29),
    [sym_wait_until_statement] = STATE(29),
    [sym_cancel_all_wait_statement] = STATE(29),
    [sym_cancel_all_wait_until_statement] = STATE(29),
    [sym_cancel_wait_statement] = STATE(29),
    [sym_cancel_wait_until_statement] = STATE(29),
    [sym_call_statement] = STATE(29),
    [sym_system_call_statement] = STATE(29),
    [sym_expression] = STATE(1137),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1990),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(29),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [ts_builtin_sym_end] = ACTIONS(592),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(9),
    [sym_module_name_keyword] = ACTIONS(11),
    [sym_define_device_keyword] = ACTIONS(13),
    [sym_define_combine_keyword] = ACTIONS(15),
    [sym_define_constant_keyword] = ACTIONS(17),
    [sym_define_type_keyword] = ACTIONS(19),
    [sym_define_variable_keyword] = ACTIONS(21),
    [sym_define_system_variable_keyword] = ACTIONS(23),
    [sym_define_connect_level_keyword] = ACTIONS(25),
    [sym_define_latching_keyword] = ACTIONS(27),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(29),
    [sym_define_toggling_keyword] = ACTIONS(31),
    [sym_define_call_keyword] = ACTIONS(33),
    [sym_define_function_keyword] = ACTIONS(35),
    [sym_define_library_function_keyword] = ACTIONS(37),
    [sym_define_start_keyword] = ACTIONS(39),
    [sym_define_event_keyword] = ACTIONS(41),
    [sym_define_program_keyword] = ACTIONS(43),
    [sym_define_module_keyword] = ACTIONS(45),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(55),
    [sym_switch_keyword] = ACTIONS(57),
    [sym_case_keyword] = ACTIONS(59),
    [sym_default_keyword] = ACTIONS(61),
    [sym_while_keyword] = ACTIONS(63),
    [sym_for_keyword] = ACTIONS(65),
    [sym_break_keyword] = ACTIONS(67),
    [sym_continue_keyword] = ACTIONS(69),
    [sym_return_keyword] = ACTIONS(71),
    [sym_select_keyword] = ACTIONS(73),
    [sym_active_keyword] = ACTIONS(75),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(99),
    [sym_send_string_keyword] = ACTIONS(101),
    [sym_send_command_keyword] = ACTIONS(103),
    [sym_clear_buffer_keyword] = ACTIONS(105),
    [sym_create_buffer_keyword] = ACTIONS(107),
    [sym_create_multi_buffer_keyword] = ACTIONS(109),
    [sym_call_keyword] = ACTIONS(111),
    [sym_system_call_keyword] = ACTIONS(113),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(117),
    [sym_cancel_wait_keyword] = ACTIONS(119),
    [sym_wait_until_keyword] = ACTIONS(121),
    [sym_cancel_wait_until_keyword] = ACTIONS(123),
    [sym_cancel_all_wait_keyword] = ACTIONS(125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(127),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(141),
    [aux_sym_preproc_define_token1] = ACTIONS(143),
    [aux_sym_preproc_warn_token1] = ACTIONS(145),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(147),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(149),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(151),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(28)] = {
    [sym__top_level_item] = STATE(30),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(30),
    [sym_module_name] = STATE(30),
    [sym_preproc_include] = STATE(30),
    [sym_preproc_define] = STATE(30),
    [sym_preproc_warn] = STATE(30),
    [sym_preproc_disable_warning] = STATE(30),
    [sym_preproc_if_defined] = STATE(30),
    [sym_preproc_if_not_defined] = STATE(30),
    [sym_section] = STATE(30),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(30),
    [sym_define_call] = STATE(30),
    [sym_define_module] = STATE(30),
    [sym_event_definition] = STATE(30),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(30),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(30),
    [sym_type_definition] = STATE(30),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(30),
    [sym_expression_statement] = STATE(30),
    [sym_if_statement] = STATE(30),
    [sym_switch_statement] = STATE(30),
    [sym_case_statement] = STATE(30),
    [sym_while_statement] = STATE(30),
    [sym_for_statement] = STATE(30),
    [sym_select_statement] = STATE(30),
    [sym_active_statement] = STATE(30),
    [sym_return_statement] = STATE(30),
    [sym_break_statement] = STATE(30),
    [sym_continue_statement] = STATE(30),
    [sym_devchan_operation_statement] = STATE(30),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(30),
    [sym_send_command_statement] = STATE(30),
    [sym_send_level_statement] = STATE(30),
    [sym_create_buffer_statement] = STATE(30),
    [sym_create_multi_buffer_statement] = STATE(30),
    [sym_clear_buffer_statement] = STATE(30),
    [sym_wait_statement] = STATE(30),
    [sym_wait_until_statement] = STATE(30),
    [sym_cancel_all_wait_statement] = STATE(30),
    [sym_cancel_all_wait_until_statement] = STATE(30),
    [sym_cancel_wait_statement] = STATE(30),
    [sym_cancel_wait_until_statement] = STATE(30),
    [sym_call_statement] = STATE(30),
    [sym_system_call_statement] = STATE(30),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(30),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(594),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(29)] = {
    [sym__top_level_item] = STATE(29),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(29),
    [sym_module_name] = STATE(29),
    [sym_preproc_include] = STATE(29),
    [sym_preproc_define] = STATE(29),
    [sym_preproc_warn] = STATE(29),
    [sym_preproc_disable_warning] = STATE(29),
    [sym_preproc_if_defined] = STATE(29),
    [sym_preproc_if_not_defined] = STATE(29),
    [sym_section] = STATE(29),
    [sym_define_device_section] = STATE(398),
    [sym_define_combine_section] = STATE(398),
    [sym_define_connect_level_section] = STATE(398),
    [sym_define_constant_section] = STATE(398),
    [sym_define_type_section] = STATE(398),
    [sym_define_mutually_exclusive_section] = STATE(398),
    [sym_define_latching_section] = STATE(398),
    [sym_define_toggling_section] = STATE(398),
    [sym_define_variable_section] = STATE(398),
    [sym_define_system_variable_section] = STATE(398),
    [sym_define_event_section] = STATE(398),
    [sym_define_start_section] = STATE(398),
    [sym_define_program_section] = STATE(398),
    [sym_define_function] = STATE(29),
    [sym_define_call] = STATE(29),
    [sym_define_module] = STATE(29),
    [sym_event_definition] = STATE(29),
    [sym_data_event_definition] = STATE(399),
    [sym__data_event_declarator_list] = STATE(1893),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(399),
    [sym__timeline_event_declarator_list] = STATE(1939),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(399),
    [sym__button_event_declarator_list] = STATE(1917),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(399),
    [sym__level_event_declarator_list] = STATE(1938),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(399),
    [sym__channel_event_declarator_list] = STATE(1940),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(399),
    [sym__custom_event_declarator_list] = STATE(1944),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(399),
    [sym__push_event_declarator_list] = STATE(1941),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(399),
    [sym__release_event_declarator_list] = STATE(1961),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(29),
    [sym_storage_class_specifier] = STATE(1028),
    [sym_type_qualifier] = STATE(1013),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(1978),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(29),
    [sym_type_definition] = STATE(29),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1457),
    [sym__top_level_statement] = STATE(29),
    [sym_expression_statement] = STATE(29),
    [sym_if_statement] = STATE(29),
    [sym_switch_statement] = STATE(29),
    [sym_case_statement] = STATE(29),
    [sym_while_statement] = STATE(29),
    [sym_for_statement] = STATE(29),
    [sym_select_statement] = STATE(29),
    [sym_active_statement] = STATE(29),
    [sym_return_statement] = STATE(29),
    [sym_break_statement] = STATE(29),
    [sym_continue_statement] = STATE(29),
    [sym_devchan_operation_statement] = STATE(29),
    [sym_devchan_operation] = STATE(1878),
    [sym_send_string_statement] = STATE(29),
    [sym_send_command_statement] = STATE(29),
    [sym_send_level_statement] = STATE(29),
    [sym_create_buffer_statement] = STATE(29),
    [sym_create_multi_buffer_statement] = STATE(29),
    [sym_clear_buffer_statement] = STATE(29),
    [sym_wait_statement] = STATE(29),
    [sym_wait_until_statement] = STATE(29),
    [sym_cancel_all_wait_statement] = STATE(29),
    [sym_cancel_all_wait_until_statement] = STATE(29),
    [sym_cancel_wait_statement] = STATE(29),
    [sym_cancel_wait_until_statement] = STATE(29),
    [sym_call_statement] = STATE(29),
    [sym_system_call_statement] = STATE(29),
    [sym_expression] = STATE(1137),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1990),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(29),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [ts_builtin_sym_end] = ACTIONS(563),
    [sym_identifier] = ACTIONS(347),
    [sym_program_name_keyword] = ACTIONS(596),
    [sym_module_name_keyword] = ACTIONS(599),
    [sym_define_device_keyword] = ACTIONS(602),
    [sym_define_combine_keyword] = ACTIONS(605),
    [sym_define_constant_keyword] = ACTIONS(608),
    [sym_define_type_keyword] = ACTIONS(611),
    [sym_define_variable_keyword] = ACTIONS(614),
    [sym_define_system_variable_keyword] = ACTIONS(617),
    [sym_define_connect_level_keyword] = ACTIONS(620),
    [sym_define_latching_keyword] = ACTIONS(623),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(626),
    [sym_define_toggling_keyword] = ACTIONS(629),
    [sym_define_call_keyword] = ACTIONS(632),
    [sym_define_function_keyword] = ACTIONS(635),
    [sym_define_library_function_keyword] = ACTIONS(638),
    [sym_define_start_keyword] = ACTIONS(641),
    [sym_define_event_keyword] = ACTIONS(644),
    [sym_define_program_keyword] = ACTIONS(647),
    [sym_define_module_keyword] = ACTIONS(650),
    [sym_char_keyword] = ACTIONS(407),
    [sym_widechar_keyword] = ACTIONS(407),
    [sym_integer_keyword] = ACTIONS(407),
    [sym_sinteger_keyword] = ACTIONS(407),
    [sym_long_keyword] = ACTIONS(407),
    [sym_slong_keyword] = ACTIONS(407),
    [sym_float_keyword] = ACTIONS(407),
    [sym_double_keyword] = ACTIONS(407),
    [sym_variant_keyword] = ACTIONS(407),
    [sym_variantarray_keyword] = ACTIONS(407),
    [sym_local_var_keyword] = ACTIONS(410),
    [sym_stack_var_keyword] = ACTIONS(410),
    [sym_constant_keyword] = ACTIONS(413),
    [sym_volatile_keyword] = ACTIONS(413),
    [sym_non_volatile_keyword] = ACTIONS(413),
    [sym_persistent_keyword] = ACTIONS(413),
    [sym_dev_keyword] = ACTIONS(416),
    [sym_devlev_keyword] = ACTIONS(416),
    [sym_devchan_keyword] = ACTIONS(416),
    [sym_if_keyword] = ACTIONS(653),
    [sym_switch_keyword] = ACTIONS(656),
    [sym_case_keyword] = ACTIONS(659),
    [sym_default_keyword] = ACTIONS(662),
    [sym_while_keyword] = ACTIONS(665),
    [sym_for_keyword] = ACTIONS(668),
    [sym_break_keyword] = ACTIONS(671),
    [sym_continue_keyword] = ACTIONS(674),
    [sym_return_keyword] = ACTIONS(677),
    [sym_select_keyword] = ACTIONS(680),
    [sym_active_keyword] = ACTIONS(683),
    [sym_struct_keyword] = ACTIONS(452),
    [sym_structure_keyword] = ACTIONS(452),
    [sym_button_event_keyword] = ACTIONS(455),
    [sym_channel_event_keyword] = ACTIONS(458),
    [sym_level_event_keyword] = ACTIONS(461),
    [sym_data_event_keyword] = ACTIONS(464),
    [sym_timeline_event_keyword] = ACTIONS(467),
    [sym_custom_event_keyword] = ACTIONS(470),
    [sym_push_keyword] = ACTIONS(473),
    [sym_release_keyword] = ACTIONS(476),
    [aux_sym_on_keyword_token1] = ACTIONS(479),
    [aux_sym_off_keyword_token1] = ACTIONS(482),
    [sym_send_level_keyword] = ACTIONS(686),
    [sym_send_string_keyword] = ACTIONS(689),
    [sym_send_command_keyword] = ACTIONS(692),
    [sym_clear_buffer_keyword] = ACTIONS(695),
    [sym_create_buffer_keyword] = ACTIONS(698),
    [sym_create_multi_buffer_keyword] = ACTIONS(701),
    [sym_call_keyword] = ACTIONS(704),
    [sym_system_call_keyword] = ACTIONS(707),
    [sym_devchan_to_keyword] = ACTIONS(509),
    [sym_devchan_min_to_keyword] = ACTIONS(509),
    [sym_devchan_total_off_keyword] = ACTIONS(509),
    [sym_devchan_pulse_keyword] = ACTIONS(509),
    [sym_wait_keyword] = ACTIONS(710),
    [sym_cancel_wait_keyword] = ACTIONS(713),
    [sym_wait_until_keyword] = ACTIONS(716),
    [sym_cancel_wait_until_keyword] = ACTIONS(719),
    [sym_cancel_all_wait_keyword] = ACTIONS(722),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(725),
    [sym_not] = ACTIONS(530),
    [sym_bnot] = ACTIONS(530),
    [sym___file__] = ACTIONS(533),
    [sym___date__] = ACTIONS(533),
    [sym___ldate__] = ACTIONS(533),
    [sym___time__] = ACTIONS(533),
    [sym___line__] = ACTIONS(533),
    [sym___name__] = ACTIONS(533),
    [sym___netlinx__] = ACTIONS(536),
    [sym_netlinx_axi_version] = ACTIONS(536),
    [sym_url_flg_tcp] = ACTIONS(536),
    [sym_url_flg_accinfopresent] = ACTIONS(536),
    [sym_url_flg_temp] = ACTIONS(536),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(536),
    [sym_url_flg_stat_mask] = ACTIONS(536),
    [sym_url_flg_stat_lookup] = ACTIONS(536),
    [sym_url_flg_stat_connecting] = ACTIONS(536),
    [sym_url_flg_stat_waiting] = ACTIONS(536),
    [sym_url_flg_stat_connected] = ACTIONS(536),
    [sym_ip_addr_flg_dhcp] = ACTIONS(536),
    [sym_first_virtual_device] = ACTIONS(536),
    [sym_first_local_port] = ACTIONS(536),
    [sym_dynamic_virtual_device] = ACTIONS(536),
    [sym_timeline_once] = ACTIONS(536),
    [sym_timeline_repeat] = ACTIONS(536),
    [sym_timeline_absolute] = ACTIONS(536),
    [sym_timeline_relative] = ACTIONS(536),
    [sym_do_push_timed_infinite] = ACTIONS(536),
    [sym_source_type_no_address] = ACTIONS(536),
    [sym_source_type_neuron_id] = ACTIONS(536),
    [sym_source_type_ip_address] = ACTIONS(536),
    [sym_source_type_axlink] = ACTIONS(536),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(536),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(536),
    [sym_source_type_ip_socket_address] = ACTIONS(536),
    [sym_source_type_rs232] = ACTIONS(536),
    [sym_source_type_internal] = ACTIONS(536),
    [sym_source_type_mac_address] = ACTIONS(536),
    [sym_source_type_ipv4_port] = ACTIONS(536),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(536),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(536),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(536),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(536),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(536),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(536),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(536),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(536),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(536),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(536),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(536),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(536),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(536),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(536),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(536),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(536),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(536),
    [sym_true] = ACTIONS(536),
    [sym_false] = ACTIONS(536),
    [sym_file_read_only] = ACTIONS(536),
    [sym_file_rw_new] = ACTIONS(536),
    [sym_file_rw_append] = ACTIONS(536),
    [sym_ip_tcp] = ACTIONS(536),
    [sym_ip_udp] = ACTIONS(536),
    [sym_ip_udp_2way] = ACTIONS(536),
    [sym_xml_encode_types] = ACTIONS(536),
    [sym_xml_encode_char_as_list] = ACTIONS(536),
    [sym_xml_encode_le] = ACTIONS(536),
    [sym_xml_decode_types] = ACTIONS(536),
    [sym_xml_decode_no_preserve] = ACTIONS(536),
    [sym_duet_dev_type_display_device] = ACTIONS(536),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(536),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(536),
    [sym_duet_dev_type_io_device] = ACTIONS(536),
    [sym_duet_dev_type_relay_device] = ACTIONS(536),
    [sym_duet_dev_type_ups] = ACTIONS(536),
    [sym_duet_dev_type_amplifier] = ACTIONS(536),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(536),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(536),
    [sym_duet_dev_type_audio_processor] = ACTIONS(536),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(536),
    [sym_duet_dev_type_camera] = ACTIONS(536),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(536),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(536),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(536),
    [sym_duet_dev_type_dss] = ACTIONS(536),
    [sym_duet_dev_type_dvr] = ACTIONS(536),
    [sym_duet_dev_type_disc_device] = ACTIONS(536),
    [sym_duet_dev_type_document_camera] = ACTIONS(536),
    [sym_duet_dev_type_audio_tape] = ACTIONS(536),
    [sym_duet_dev_type_hvac] = ACTIONS(536),
    [sym_duet_dev_type_keypad] = ACTIONS(536),
    [sym_duet_dev_type_light] = ACTIONS(536),
    [sym_duet_dev_type_monitor] = ACTIONS(536),
    [sym_duet_dev_type_motor] = ACTIONS(536),
    [sym_duet_dev_type_multi_window] = ACTIONS(536),
    [sym_duet_dev_type_pool_spa] = ACTIONS(536),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(536),
    [sym_duet_dev_type_receiver] = ACTIONS(536),
    [sym_duet_dev_type_security_system] = ACTIONS(536),
    [sym_duet_dev_type_sensor_device] = ACTIONS(536),
    [sym_duet_dev_type_settop_box] = ACTIONS(536),
    [sym_duet_dev_type_slide_projector] = ACTIONS(536),
    [sym_duet_dev_type_switcher] = ACTIONS(536),
    [sym_duet_dev_type_text_keypad] = ACTIONS(536),
    [sym_duet_dev_type_tv] = ACTIONS(536),
    [sym_duet_dev_type_utility] = ACTIONS(536),
    [sym_duet_dev_type_vcr] = ACTIONS(536),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(536),
    [sym_duet_dev_type_video_processor] = ACTIONS(536),
    [sym_duet_dev_type_video_projector] = ACTIONS(536),
    [sym_duet_dev_type_video_wall] = ACTIONS(536),
    [sym_duet_dev_type_volume_controller] = ACTIONS(536),
    [sym_duet_dev_type_weather] = ACTIONS(536),
    [sym_duet_dev_not_polled] = ACTIONS(536),
    [sym_duet_dev_polled] = ACTIONS(536),
    [sym_clkmgr_mode_network] = ACTIONS(536),
    [sym_clkmgr_mode_standalone] = ACTIONS(536),
    [sym_normal_standby] = ACTIONS(536),
    [sym_normal_wake] = ACTIONS(536),
    [sym_ip_multicast_ttl_option] = ACTIONS(536),
    [sym_ip_ttl_subnet] = ACTIONS(536),
    [sym_ip_ttl_site] = ACTIONS(536),
    [sym_ip_ttl_region] = ACTIONS(536),
    [sym_ip_ttl_continent] = ACTIONS(536),
    [sym_ip_tcp_nodelay_option] = ACTIONS(536),
    [sym_ip_nodelay_on] = ACTIONS(536),
    [sym_ip_nodelay_off] = ACTIONS(536),
    [sym_amx_error] = ACTIONS(536),
    [sym_amx_warning] = ACTIONS(536),
    [sym_amx_info] = ACTIONS(536),
    [sym_amx_debug] = ACTIONS(536),
    [sym_netlinx_login_success] = ACTIONS(536),
    [sym_netlinx_login_fail] = ACTIONS(536),
    [sym_netlinx_logout] = ACTIONS(536),
    [sym_null_str] = ACTIONS(536),
    [sym_smtp_address] = ACTIONS(536),
    [sym_smtp_port_number] = ACTIONS(536),
    [sym_smtp_username] = ACTIONS(536),
    [sym_smtp_password] = ACTIONS(536),
    [sym_smtp_require_tls] = ACTIONS(536),
    [sym_smtp_from] = ACTIONS(536),
    [sym_smtp_tls_true] = ACTIONS(536),
    [sym_smtp_tls_false] = ACTIONS(536),
    [sym_listview_on_row_select_event] = ACTIONS(536),
    [sym_data_structure_unknown] = ACTIONS(536),
    [sym_data_structure_datafeed] = ACTIONS(536),
    [sym_data_structure_datarecordset] = ACTIONS(536),
    [sym_data_structure_datarecord] = ACTIONS(536),
    [sym_data_structure_datafield] = ACTIONS(536),
    [sym_data_max_content_field_count] = ACTIONS(536),
    [sym_data_max_metadata_field_count] = ACTIONS(536),
    [sym_data_max_id_length] = ACTIONS(536),
    [sym_data_max_name_length] = ACTIONS(536),
    [sym_data_max_description_length] = ACTIONS(536),
    [sym_data_max_type_length] = ACTIONS(536),
    [sym_data_max_format_length] = ACTIONS(536),
    [sym_data_max_source_length] = ACTIONS(536),
    [sym_data_max_value_length] = ACTIONS(536),
    [sym_data_max_label_length] = ACTIONS(536),
    [sym_data_max_url_length] = ACTIONS(536),
    [sym_max_last_login_info_length] = ACTIONS(536),
    [sym_data_type_unknown] = ACTIONS(536),
    [sym_data_type_string] = ACTIONS(536),
    [sym_data_type_datetime] = ACTIONS(536),
    [sym_data_type_date] = ACTIONS(536),
    [sym_data_type_time] = ACTIONS(536),
    [sym_data_type_image] = ACTIONS(536),
    [sym_data_format_url] = ACTIONS(536),
    [sym_data_format_phone] = ACTIONS(536),
    [sym_data_format_email] = ACTIONS(536),
    [sym_data_format_iso8601] = ACTIONS(536),
    [sym_auditlog_priv] = ACTIONS(536),
    [sym_configuration_priv] = ACTIONS(536),
    [sym_deviceconfig_priv] = ACTIONS(536),
    [sym_ftp_priv] = ACTIONS(536),
    [sym_http_priv] = ACTIONS(536),
    [sym_networkconfig_priv] = ACTIONS(536),
    [sym_programport_priv] = ACTIONS(536),
    [sym_remoteui_priv] = ACTIONS(536),
    [sym_securitycontrol_priv] = ACTIONS(536),
    [sym_softwaremanagement_priv] = ACTIONS(536),
    [sym_terminal_priv] = ACTIONS(536),
    [sym_tpadmin_priv] = ACTIONS(536),
    [sym_user1_priv] = ACTIONS(536),
    [sym_user2_priv] = ACTIONS(536),
    [sym_user3_priv] = ACTIONS(536),
    [sym_user4_priv] = ACTIONS(536),
    [sym_usermanagement_priv] = ACTIONS(536),
    [sym_valid_account] = ACTIONS(536),
    [sym_err_name_invalid] = ACTIONS(536),
    [sym_err_password_invalid] = ACTIONS(536),
    [sym_err_invalid_account] = ACTIONS(536),
    [sym_err_no_authorization] = ACTIONS(536),
    [sym_err_invalid_parameter] = ACTIONS(536),
    [sym_err_user_locked_out] = ACTIONS(536),
    [sym_err_user_account_expired] = ACTIONS(536),
    [sym_err_general_error] = ACTIONS(536),
    [sym_err_authentication_error] = ACTIONS(536),
    [sym_tls_ignore_certificate_errors] = ACTIONS(536),
    [sym_tls_validate_certificate] = ACTIONS(536),
    [sym_tdata] = ACTIONS(539),
    [sym_tchannel] = ACTIONS(539),
    [sym_tlevel] = ACTIONS(539),
    [sym_tbutton] = ACTIONS(539),
    [sym_ttimeline] = ACTIONS(539),
    [sym_tcustom] = ACTIONS(539),
    [sym_url_struct] = ACTIONS(539),
    [sym_dns_struct] = ACTIONS(539),
    [sym_ip_address_struct] = ACTIONS(539),
    [sym_dev_info_struct] = ACTIONS(539),
    [sym_clkmgr_timeserver_struct] = ACTIONS(539),
    [sym_data_feed] = ACTIONS(539),
    [sym_data_field] = ACTIONS(539),
    [sym_data_record] = ACTIONS(539),
    [sym_wc_data_feed] = ACTIONS(539),
    [sym_wc_data_field] = ACTIONS(539),
    [sym_wc_data_record] = ACTIONS(539),
    [sym_last_login_info] = ACTIONS(539),
    [sym____reserved___] = ACTIONS(542),
    [sym_date] = ACTIONS(542),
    [sym_day] = ACTIONS(542),
    [sym_time] = ACTIONS(542),
    [sym_ldate] = ACTIONS(542),
    [sym_push_device] = ACTIONS(542),
    [sym_push_channel] = ACTIONS(542),
    [sym_push_devchan] = ACTIONS(542),
    [sym_release_device] = ACTIONS(542),
    [sym_release_channel] = ACTIONS(542),
    [sym_release_devchan] = ACTIONS(542),
    [sym_master_slot] = ACTIONS(542),
    [sym_get_pulse_time] = ACTIONS(542),
    [sym_get_timer] = ACTIONS(542),
    [sym_master_sn] = ACTIONS(542),
    [sym_system_number] = ACTIONS(542),
    [sym_button] = ACTIONS(542),
    [sym_dv_channel] = ACTIONS(542),
    [sym_data] = ACTIONS(542),
    [sym_level] = ACTIONS(542),
    [sym_timeline] = ACTIONS(542),
    [sym_channel] = ACTIONS(542),
    [sym_custom] = ACTIONS(542),
    [sym_atoi] = ACTIONS(545),
    [sym_atol] = ACTIONS(545),
    [sym_atof] = ACTIONS(545),
    [sym_itoa] = ACTIONS(545),
    [sym_itohex] = ACTIONS(545),
    [sym_hextoi] = ACTIONS(545),
    [sym_ftoa] = ACTIONS(545),
    [sym_find_string] = ACTIONS(545),
    [sym_left_string] = ACTIONS(545),
    [sym_right_string] = ACTIONS(545),
    [sym_mid_string] = ACTIONS(545),
    [sym_remove_string] = ACTIONS(545),
    [sym_get_buffer_string] = ACTIONS(545),
    [sym_get_buffer_char] = ACTIONS(545),
    [sym_get_multi_buffer_string] = ACTIONS(545),
    [sym_length_string] = ACTIONS(545),
    [sym_max_length_string] = ACTIONS(545),
    [sym_set_length_string] = ACTIONS(545),
    [sym_compare_string] = ACTIONS(545),
    [sym_upper_string] = ACTIONS(545),
    [sym_lower_string] = ACTIONS(545),
    [sym_get_last] = ACTIONS(545),
    [sym_date_to_day] = ACTIONS(545),
    [sym_date_to_month] = ACTIONS(545),
    [sym_date_to_year] = ACTIONS(545),
    [sym_time_to_hour] = ACTIONS(545),
    [sym_time_to_minute] = ACTIONS(545),
    [sym_time_to_second] = ACTIONS(545),
    [sym_day_of_week] = ACTIONS(545),
    [sym_set_pulse_time] = ACTIONS(545),
    [sym_set_timer] = ACTIONS(545),
    [sym_file_open] = ACTIONS(545),
    [sym_file_close] = ACTIONS(545),
    [sym_file_read] = ACTIONS(545),
    [sym_file_read_line] = ACTIONS(545),
    [sym_file_write] = ACTIONS(545),
    [sym_file_write_line] = ACTIONS(545),
    [sym_file_copy] = ACTIONS(545),
    [sym_file_delete] = ACTIONS(545),
    [sym_file_rename] = ACTIONS(545),
    [sym_file_dir] = ACTIONS(545),
    [sym_file_getdir] = ACTIONS(545),
    [sym_file_setdir] = ACTIONS(545),
    [sym_file_create_dir] = ACTIONS(545),
    [sym_file_remove_dir] = ACTIONS(545),
    [sym_file_seek] = ACTIONS(545),
    [sym_length_array] = ACTIONS(545),
    [sym_max_length_array] = ACTIONS(545),
    [sym_set_length_array] = ACTIONS(545),
    [sym_ip_client_open] = ACTIONS(545),
    [sym_ip_client_close] = ACTIONS(545),
    [sym_ip_server_open] = ACTIONS(545),
    [sym_ip_server_close] = ACTIONS(545),
    [sym_ip_bound_client_open] = ACTIONS(545),
    [sym_ip_mc_server_open] = ACTIONS(545),
    [sym_ip_set_option] = ACTIONS(545),
    [sym_ssh_client_open] = ACTIONS(545),
    [sym_ssh_client_close] = ACTIONS(545),
    [sym_tls_client_open] = ACTIONS(545),
    [sym_tls_client_close] = ACTIONS(545),
    [sym_do_push] = ACTIONS(545),
    [sym_do_push_timed] = ACTIONS(545),
    [sym_do_release] = ACTIONS(545),
    [sym_device_id] = ACTIONS(545),
    [sym_device_info] = ACTIONS(545),
    [sym_device_id_string] = ACTIONS(545),
    [sym_device_standby] = ACTIONS(545),
    [sym_device_wake] = ACTIONS(545),
    [sym_timeline_create] = ACTIONS(545),
    [sym_timeline_kill] = ACTIONS(545),
    [sym_timeline_reload] = ACTIONS(545),
    [sym_timeline_pause] = ACTIONS(545),
    [sym_timeline_restart] = ACTIONS(545),
    [sym_timeline_set] = ACTIONS(545),
    [sym_timeline_get] = ACTIONS(545),
    [sym_timeline_active] = ACTIONS(545),
    [sym_timeline_dynamic_id] = ACTIONS(545),
    [sym_variable_to_string] = ACTIONS(545),
    [sym_string_to_variable] = ACTIONS(545),
    [sym_length_variable_to_string] = ACTIONS(545),
    [sym_type_cast] = ACTIONS(545),
    [sym_variable_to_xml] = ACTIONS(545),
    [sym_xml_to_variable] = ACTIONS(545),
    [sym_length_variable_to_xml] = ACTIONS(545),
    [sym_redirect_string] = ACTIONS(545),
    [sym_random_number] = ACTIONS(545),
    [sym_get_unique_id] = ACTIONS(545),
    [sym_get_system_number] = ACTIONS(545),
    [sym_set_system_number] = ACTIONS(545),
    [sym_get_serial_number] = ACTIONS(545),
    [sym_get_master_build] = ACTIONS(545),
    [sym_set_validation_code] = ACTIONS(545),
    [sym_reboot] = ACTIONS(545),
    [sym_load_duet_module] = ACTIONS(545),
    [sym_unload_duet_module] = ACTIONS(545),
    [sym_duet_mem_size_set] = ACTIONS(545),
    [sym_duet_mem_size_get] = ACTIONS(545),
    [sym_dynamic_application_device] = ACTIONS(545),
    [sym_dynamic_polled_port] = ACTIONS(545),
    [sym_static_port_binding] = ACTIONS(545),
    [sym_static_ip_binding] = ACTIONS(545),
    [sym_get_dns_list] = ACTIONS(545),
    [sym_set_dns_list] = ACTIONS(545),
    [sym_get_ip_address] = ACTIONS(545),
    [sym_set_ip_address] = ACTIONS(545),
    [sym_get_url_list] = ACTIONS(545),
    [sym_add_url_entry] = ACTIONS(545),
    [sym_delete_url_entry] = ACTIONS(545),
    [sym_abs_value] = ACTIONS(545),
    [sym_max_value] = ACTIONS(545),
    [sym_min_value] = ACTIONS(545),
    [sym_exp_value] = ACTIONS(545),
    [sym_log_value] = ACTIONS(545),
    [sym_log10_value] = ACTIONS(545),
    [sym_power_value] = ACTIONS(545),
    [sym_sqrt_value] = ACTIONS(545),
    [sym_raw_be] = ACTIONS(545),
    [sym_raw_le] = ACTIONS(545),
    [sym_internal_queue_size_set] = ACTIONS(545),
    [sym_internal_queue_size_get] = ACTIONS(545),
    [sym_internal_threshold_set] = ACTIONS(545),
    [sym_internal_threshold_get] = ACTIONS(545),
    [sym_rebuilt_event] = ACTIONS(545),
    [sym_do_custom_event] = ACTIONS(545),
    [sym_clkmgr_is_network_sourced] = ACTIONS(545),
    [sym_clkmgr_set_clk_source] = ACTIONS(545),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(545),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(545),
    [sym_clkmgr_get_timezone] = ACTIONS(545),
    [sym_clkmgr_set_timezone] = ACTIONS(545),
    [sym_clkmgr_get_resync_period] = ACTIONS(545),
    [sym_clkmgr_set_resync_period] = ACTIONS(545),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(545),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(545),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(545),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(545),
    [sym_clkmgr_get_timeservers] = ACTIONS(545),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(545),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(545),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(545),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(545),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(545),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(545),
    [sym_set_log_level] = ACTIONS(545),
    [sym_get_log_level] = ACTIONS(545),
    [sym_amx_log] = ACTIONS(545),
    [sym_smtp_server_config_set] = ACTIONS(545),
    [sym_smtp_server_config_get] = ACTIONS(545),
    [sym_smtp_send] = ACTIONS(545),
    [sym_validate_netlinx_account] = ACTIONS(545),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(545),
    [sym_authenticate_certificate] = ACTIONS(545),
    [sym_audit_netlinx_session_event] = ACTIONS(545),
    [sym_audit_netlinx_generic_event] = ACTIONS(545),
    [sym_get_available_flash_disk_space] = ACTIONS(545),
    [sym_get_max_flash_disk_space] = ACTIONS(545),
    [sym_data_create_feed] = ACTIONS(545),
    [sym_data_delete_feed] = ACTIONS(545),
    [sym_data_publish_feed] = ACTIONS(545),
    [sym_data_get_published_feed] = ACTIONS(545),
    [sym_data_add_record] = ACTIONS(545),
    [sym_data_get_event_record] = ACTIONS(545),
    [sym__wc_data_create_feed] = ACTIONS(545),
    [sym__wc_data_add_record] = ACTIONS(545),
    [sym__wc_data_get_event_record] = ACTIONS(545),
    [sym_format] = ACTIONS(545),
    [sym_astro_clock] = ACTIONS(545),
    [sym_set_outdoor_temperature] = ACTIONS(545),
    [sym_set_virtual_level_count] = ACTIONS(545),
    [sym_set_virtual_channel_count] = ACTIONS(545),
    [sym_set_virtual_port_count] = ACTIONS(545),
    [sym__wc_encode_int] = ACTIONS(545),
    [sym__wc_decode_int] = ACTIONS(545),
    [sym__wc_int] = ACTIONS(545),
    [sym__wc_to_ch_int] = ACTIONS(545),
    [sym__ch_to_wc_int] = ACTIONS(545),
    [sym__wc_find_string_int] = ACTIONS(545),
    [sym__wc_left_string_int] = ACTIONS(545),
    [sym__wc_lower_string_int] = ACTIONS(545),
    [sym__wc_mid_string_int] = ACTIONS(545),
    [sym__wc_remove_string_int] = ACTIONS(545),
    [sym__wc_right_string_int] = ACTIONS(545),
    [sym__wc_upper_string_int] = ACTIONS(545),
    [sym__wc_compare_string_int] = ACTIONS(545),
    [sym__wc_get_buffer_char_int] = ACTIONS(545),
    [sym__wc_get_buffer_string_int] = ACTIONS(545),
    [sym__wc_concat_string_int] = ACTIONS(545),
    [sym__wc_file_open_int] = ACTIONS(545),
    [sym__wc_file_close_int] = ACTIONS(545),
    [sym__wc_file_read_int] = ACTIONS(545),
    [sym__wc_file_read_line_int] = ACTIONS(545),
    [sym__wc_file_write_int] = ACTIONS(545),
    [sym__wc_file_write_line_int] = ACTIONS(545),
    [aux_sym_preproc_include_token1] = ACTIONS(728),
    [aux_sym_preproc_define_token1] = ACTIONS(731),
    [aux_sym_preproc_warn_token1] = ACTIONS(734),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(737),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(740),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(568),
    [anon_sym_LBRACE] = ACTIONS(746),
    [anon_sym_LPAREN] = ACTIONS(574),
    [anon_sym_BANG] = ACTIONS(577),
    [anon_sym_TILDE] = ACTIONS(577),
    [anon_sym_DASH] = ACTIONS(530),
    [anon_sym_PLUS] = ACTIONS(530),
    [anon_sym_DASH_DASH] = ACTIONS(580),
    [anon_sym_PLUS_PLUS] = ACTIONS(580),
    [anon_sym_DQUOTE] = ACTIONS(583),
    [anon_sym_SQUOTE] = ACTIONS(586),
    [sym_number_literal] = ACTIONS(589),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(30)] = {
    [sym__top_level_item] = STATE(26),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_program_name] = STATE(26),
    [sym_module_name] = STATE(26),
    [sym_preproc_include] = STATE(26),
    [sym_preproc_define] = STATE(26),
    [sym_preproc_warn] = STATE(26),
    [sym_preproc_disable_warning] = STATE(26),
    [sym_preproc_if_defined] = STATE(26),
    [sym_preproc_if_not_defined] = STATE(26),
    [sym_section] = STATE(26),
    [sym_define_device_section] = STATE(319),
    [sym_define_combine_section] = STATE(319),
    [sym_define_connect_level_section] = STATE(319),
    [sym_define_constant_section] = STATE(319),
    [sym_define_type_section] = STATE(319),
    [sym_define_mutually_exclusive_section] = STATE(319),
    [sym_define_latching_section] = STATE(319),
    [sym_define_toggling_section] = STATE(319),
    [sym_define_variable_section] = STATE(319),
    [sym_define_system_variable_section] = STATE(319),
    [sym_define_event_section] = STATE(319),
    [sym_define_start_section] = STATE(319),
    [sym_define_program_section] = STATE(319),
    [sym_define_function] = STATE(26),
    [sym_define_call] = STATE(26),
    [sym_define_module] = STATE(26),
    [sym_event_definition] = STATE(26),
    [sym_data_event_definition] = STATE(323),
    [sym__data_event_declarator_list] = STATE(1903),
    [sym_data_event_declarator] = STATE(1397),
    [sym_timeline_event_definition] = STATE(323),
    [sym__timeline_event_declarator_list] = STATE(1904),
    [sym_timeline_event_declarator] = STATE(1415),
    [sym_button_event_definition] = STATE(323),
    [sym__button_event_declarator_list] = STATE(1905),
    [sym_button_event_declarator] = STATE(1420),
    [sym_level_event_definition] = STATE(323),
    [sym__level_event_declarator_list] = STATE(1906),
    [sym_level_event_declarator] = STATE(1422),
    [sym_channel_event_definition] = STATE(323),
    [sym__channel_event_declarator_list] = STATE(1907),
    [sym_channel_event_declarator] = STATE(1403),
    [sym_custom_event_definition] = STATE(323),
    [sym__custom_event_declarator_list] = STATE(1908),
    [sym_custom_event_declarator] = STATE(1399),
    [sym_push_event_definition] = STATE(323),
    [sym__push_event_declarator_list] = STATE(1910),
    [sym_push_event_declarator] = STATE(1419),
    [sym_release_event_definition] = STATE(323),
    [sym__release_event_declarator_list] = STATE(1911),
    [sym_release_event_declarator] = STATE(1388),
    [sym_compound_statement] = STATE(26),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(2022),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(26),
    [sym_type_definition] = STATE(26),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym__top_level_statement] = STATE(26),
    [sym_expression_statement] = STATE(26),
    [sym_if_statement] = STATE(26),
    [sym_switch_statement] = STATE(26),
    [sym_case_statement] = STATE(26),
    [sym_while_statement] = STATE(26),
    [sym_for_statement] = STATE(26),
    [sym_select_statement] = STATE(26),
    [sym_active_statement] = STATE(26),
    [sym_return_statement] = STATE(26),
    [sym_break_statement] = STATE(26),
    [sym_continue_statement] = STATE(26),
    [sym_devchan_operation_statement] = STATE(26),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(26),
    [sym_send_command_statement] = STATE(26),
    [sym_send_level_statement] = STATE(26),
    [sym_create_buffer_statement] = STATE(26),
    [sym_create_multi_buffer_statement] = STATE(26),
    [sym_clear_buffer_statement] = STATE(26),
    [sym_wait_statement] = STATE(26),
    [sym_wait_until_statement] = STATE(26),
    [sym_cancel_all_wait_statement] = STATE(26),
    [sym_cancel_all_wait_until_statement] = STATE(26),
    [sym_cancel_wait_statement] = STATE(26),
    [sym_cancel_wait_until_statement] = STATE(26),
    [sym_call_statement] = STATE(26),
    [sym_system_call_statement] = STATE(26),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_source_file_repeat1] = STATE(26),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(169),
    [sym_module_name_keyword] = ACTIONS(171),
    [sym_define_device_keyword] = ACTIONS(173),
    [sym_define_combine_keyword] = ACTIONS(175),
    [sym_define_constant_keyword] = ACTIONS(177),
    [sym_define_type_keyword] = ACTIONS(179),
    [sym_define_variable_keyword] = ACTIONS(181),
    [sym_define_system_variable_keyword] = ACTIONS(183),
    [sym_define_connect_level_keyword] = ACTIONS(185),
    [sym_define_latching_keyword] = ACTIONS(187),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(189),
    [sym_define_toggling_keyword] = ACTIONS(191),
    [sym_define_call_keyword] = ACTIONS(193),
    [sym_define_function_keyword] = ACTIONS(195),
    [sym_define_library_function_keyword] = ACTIONS(197),
    [sym_define_start_keyword] = ACTIONS(199),
    [sym_define_event_keyword] = ACTIONS(201),
    [sym_define_program_keyword] = ACTIONS(203),
    [sym_define_module_keyword] = ACTIONS(205),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [sym_button_event_keyword] = ACTIONS(79),
    [sym_channel_event_keyword] = ACTIONS(81),
    [sym_level_event_keyword] = ACTIONS(83),
    [sym_data_event_keyword] = ACTIONS(85),
    [sym_timeline_event_keyword] = ACTIONS(87),
    [sym_custom_event_keyword] = ACTIONS(89),
    [sym_push_keyword] = ACTIONS(91),
    [sym_release_keyword] = ACTIONS(93),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(257),
    [aux_sym_preproc_define_token1] = ACTIONS(309),
    [aux_sym_preproc_warn_token1] = ACTIONS(311),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(313),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(315),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(749),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(319),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(31)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(33),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(33),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym_expression_statement] = STATE(33),
    [sym_if_statement] = STATE(33),
    [sym_switch_statement] = STATE(33),
    [sym_while_statement] = STATE(33),
    [sym_for_statement] = STATE(33),
    [sym_select_statement] = STATE(33),
    [sym_active_statement] = STATE(33),
    [sym_return_statement] = STATE(33),
    [sym_break_statement] = STATE(33),
    [sym_continue_statement] = STATE(33),
    [sym_devchan_operation_statement] = STATE(33),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(33),
    [sym_send_command_statement] = STATE(33),
    [sym_send_level_statement] = STATE(33),
    [sym_create_buffer_statement] = STATE(33),
    [sym_create_multi_buffer_statement] = STATE(33),
    [sym_clear_buffer_statement] = STATE(33),
    [sym_wait_statement] = STATE(33),
    [sym_wait_until_statement] = STATE(33),
    [sym_cancel_all_wait_statement] = STATE(33),
    [sym_cancel_all_wait_until_statement] = STATE(33),
    [sym_cancel_wait_statement] = STATE(33),
    [sym_cancel_wait_until_statement] = STATE(33),
    [sym_call_statement] = STATE(33),
    [sym_system_call_statement] = STATE(33),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(33),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(751),
    [sym_module_name_keyword] = ACTIONS(751),
    [sym_define_device_keyword] = ACTIONS(751),
    [sym_define_combine_keyword] = ACTIONS(751),
    [sym_define_constant_keyword] = ACTIONS(751),
    [sym_define_type_keyword] = ACTIONS(751),
    [sym_define_variable_keyword] = ACTIONS(751),
    [sym_define_system_variable_keyword] = ACTIONS(751),
    [sym_define_connect_level_keyword] = ACTIONS(751),
    [sym_define_latching_keyword] = ACTIONS(751),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(751),
    [sym_define_toggling_keyword] = ACTIONS(751),
    [sym_define_call_keyword] = ACTIONS(751),
    [sym_define_function_keyword] = ACTIONS(751),
    [sym_define_library_function_keyword] = ACTIONS(751),
    [sym_define_start_keyword] = ACTIONS(751),
    [sym_define_event_keyword] = ACTIONS(751),
    [sym_define_program_keyword] = ACTIONS(751),
    [sym_define_module_keyword] = ACTIONS(751),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_else_keyword] = ACTIONS(751),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(751),
    [sym_default_keyword] = ACTIONS(751),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(751),
    [sym_structure_keyword] = ACTIONS(751),
    [sym_button_event_keyword] = ACTIONS(751),
    [sym_channel_event_keyword] = ACTIONS(751),
    [sym_level_event_keyword] = ACTIONS(751),
    [sym_data_event_keyword] = ACTIONS(751),
    [sym_timeline_event_keyword] = ACTIONS(751),
    [sym_custom_event_keyword] = ACTIONS(751),
    [sym_push_keyword] = ACTIONS(751),
    [sym_release_keyword] = ACTIONS(751),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(751),
    [aux_sym_preproc_define_token1] = ACTIONS(753),
    [aux_sym_preproc_warn_token1] = ACTIONS(753),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(753),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(753),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(753),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(753),
    [aux_sym_preproc_else_token1] = ACTIONS(753),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(32)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(34),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(34),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym_expression_statement] = STATE(34),
    [sym_if_statement] = STATE(34),
    [sym_switch_statement] = STATE(34),
    [sym_while_statement] = STATE(34),
    [sym_for_statement] = STATE(34),
    [sym_select_statement] = STATE(34),
    [sym_active_statement] = STATE(34),
    [sym_return_statement] = STATE(34),
    [sym_break_statement] = STATE(34),
    [sym_continue_statement] = STATE(34),
    [sym_devchan_operation_statement] = STATE(34),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(34),
    [sym_send_command_statement] = STATE(34),
    [sym_send_level_statement] = STATE(34),
    [sym_create_buffer_statement] = STATE(34),
    [sym_create_multi_buffer_statement] = STATE(34),
    [sym_clear_buffer_statement] = STATE(34),
    [sym_wait_statement] = STATE(34),
    [sym_wait_until_statement] = STATE(34),
    [sym_cancel_all_wait_statement] = STATE(34),
    [sym_cancel_all_wait_until_statement] = STATE(34),
    [sym_cancel_wait_statement] = STATE(34),
    [sym_cancel_wait_until_statement] = STATE(34),
    [sym_call_statement] = STATE(34),
    [sym_system_call_statement] = STATE(34),
    [sym_expression] = STATE(1131),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(34),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(755),
    [sym_module_name_keyword] = ACTIONS(755),
    [sym_define_device_keyword] = ACTIONS(755),
    [sym_define_combine_keyword] = ACTIONS(755),
    [sym_define_constant_keyword] = ACTIONS(755),
    [sym_define_type_keyword] = ACTIONS(755),
    [sym_define_variable_keyword] = ACTIONS(755),
    [sym_define_system_variable_keyword] = ACTIONS(755),
    [sym_define_connect_level_keyword] = ACTIONS(755),
    [sym_define_latching_keyword] = ACTIONS(755),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(755),
    [sym_define_toggling_keyword] = ACTIONS(755),
    [sym_define_call_keyword] = ACTIONS(755),
    [sym_define_function_keyword] = ACTIONS(755),
    [sym_define_library_function_keyword] = ACTIONS(755),
    [sym_define_start_keyword] = ACTIONS(755),
    [sym_define_event_keyword] = ACTIONS(755),
    [sym_define_program_keyword] = ACTIONS(755),
    [sym_define_module_keyword] = ACTIONS(755),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_else_keyword] = ACTIONS(755),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(755),
    [sym_default_keyword] = ACTIONS(755),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(755),
    [sym_structure_keyword] = ACTIONS(755),
    [sym_button_event_keyword] = ACTIONS(755),
    [sym_channel_event_keyword] = ACTIONS(755),
    [sym_level_event_keyword] = ACTIONS(755),
    [sym_data_event_keyword] = ACTIONS(755),
    [sym_timeline_event_keyword] = ACTIONS(755),
    [sym_custom_event_keyword] = ACTIONS(755),
    [sym_push_keyword] = ACTIONS(755),
    [sym_release_keyword] = ACTIONS(755),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(755),
    [aux_sym_preproc_define_token1] = ACTIONS(757),
    [aux_sym_preproc_warn_token1] = ACTIONS(757),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(757),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(757),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(757),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(757),
    [aux_sym_preproc_else_token1] = ACTIONS(757),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(33)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(35),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(35),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym_expression_statement] = STATE(35),
    [sym_if_statement] = STATE(35),
    [sym_switch_statement] = STATE(35),
    [sym_while_statement] = STATE(35),
    [sym_for_statement] = STATE(35),
    [sym_select_statement] = STATE(35),
    [sym_active_statement] = STATE(35),
    [sym_return_statement] = STATE(35),
    [sym_break_statement] = STATE(35),
    [sym_continue_statement] = STATE(35),
    [sym_devchan_operation_statement] = STATE(35),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(35),
    [sym_send_command_statement] = STATE(35),
    [sym_send_level_statement] = STATE(35),
    [sym_create_buffer_statement] = STATE(35),
    [sym_create_multi_buffer_statement] = STATE(35),
    [sym_clear_buffer_statement] = STATE(35),
    [sym_wait_statement] = STATE(35),
    [sym_wait_until_statement] = STATE(35),
    [sym_cancel_all_wait_statement] = STATE(35),
    [sym_cancel_all_wait_until_statement] = STATE(35),
    [sym_cancel_wait_statement] = STATE(35),
    [sym_cancel_wait_until_statement] = STATE(35),
    [sym_call_statement] = STATE(35),
    [sym_system_call_statement] = STATE(35),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(35),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(759),
    [sym_module_name_keyword] = ACTIONS(759),
    [sym_define_device_keyword] = ACTIONS(759),
    [sym_define_combine_keyword] = ACTIONS(759),
    [sym_define_constant_keyword] = ACTIONS(759),
    [sym_define_type_keyword] = ACTIONS(759),
    [sym_define_variable_keyword] = ACTIONS(759),
    [sym_define_system_variable_keyword] = ACTIONS(759),
    [sym_define_connect_level_keyword] = ACTIONS(759),
    [sym_define_latching_keyword] = ACTIONS(759),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(759),
    [sym_define_toggling_keyword] = ACTIONS(759),
    [sym_define_call_keyword] = ACTIONS(759),
    [sym_define_function_keyword] = ACTIONS(759),
    [sym_define_library_function_keyword] = ACTIONS(759),
    [sym_define_start_keyword] = ACTIONS(759),
    [sym_define_event_keyword] = ACTIONS(759),
    [sym_define_program_keyword] = ACTIONS(759),
    [sym_define_module_keyword] = ACTIONS(759),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_else_keyword] = ACTIONS(759),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(759),
    [sym_default_keyword] = ACTIONS(759),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(759),
    [sym_structure_keyword] = ACTIONS(759),
    [sym_button_event_keyword] = ACTIONS(759),
    [sym_channel_event_keyword] = ACTIONS(759),
    [sym_level_event_keyword] = ACTIONS(759),
    [sym_data_event_keyword] = ACTIONS(759),
    [sym_timeline_event_keyword] = ACTIONS(759),
    [sym_custom_event_keyword] = ACTIONS(759),
    [sym_push_keyword] = ACTIONS(759),
    [sym_release_keyword] = ACTIONS(759),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(759),
    [aux_sym_preproc_define_token1] = ACTIONS(761),
    [aux_sym_preproc_warn_token1] = ACTIONS(761),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(761),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(761),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(761),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(761),
    [aux_sym_preproc_else_token1] = ACTIONS(761),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(34)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(35),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(35),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym_expression_statement] = STATE(35),
    [sym_if_statement] = STATE(35),
    [sym_switch_statement] = STATE(35),
    [sym_while_statement] = STATE(35),
    [sym_for_statement] = STATE(35),
    [sym_select_statement] = STATE(35),
    [sym_active_statement] = STATE(35),
    [sym_return_statement] = STATE(35),
    [sym_break_statement] = STATE(35),
    [sym_continue_statement] = STATE(35),
    [sym_devchan_operation_statement] = STATE(35),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(35),
    [sym_send_command_statement] = STATE(35),
    [sym_send_level_statement] = STATE(35),
    [sym_create_buffer_statement] = STATE(35),
    [sym_create_multi_buffer_statement] = STATE(35),
    [sym_clear_buffer_statement] = STATE(35),
    [sym_wait_statement] = STATE(35),
    [sym_wait_until_statement] = STATE(35),
    [sym_cancel_all_wait_statement] = STATE(35),
    [sym_cancel_all_wait_until_statement] = STATE(35),
    [sym_cancel_wait_statement] = STATE(35),
    [sym_cancel_wait_until_statement] = STATE(35),
    [sym_call_statement] = STATE(35),
    [sym_system_call_statement] = STATE(35),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(35),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(763),
    [sym_module_name_keyword] = ACTIONS(763),
    [sym_define_device_keyword] = ACTIONS(763),
    [sym_define_combine_keyword] = ACTIONS(763),
    [sym_define_constant_keyword] = ACTIONS(763),
    [sym_define_type_keyword] = ACTIONS(763),
    [sym_define_variable_keyword] = ACTIONS(763),
    [sym_define_system_variable_keyword] = ACTIONS(763),
    [sym_define_connect_level_keyword] = ACTIONS(763),
    [sym_define_latching_keyword] = ACTIONS(763),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(763),
    [sym_define_toggling_keyword] = ACTIONS(763),
    [sym_define_call_keyword] = ACTIONS(763),
    [sym_define_function_keyword] = ACTIONS(763),
    [sym_define_library_function_keyword] = ACTIONS(763),
    [sym_define_start_keyword] = ACTIONS(763),
    [sym_define_event_keyword] = ACTIONS(763),
    [sym_define_program_keyword] = ACTIONS(763),
    [sym_define_module_keyword] = ACTIONS(763),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(207),
    [sym_else_keyword] = ACTIONS(763),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(763),
    [sym_default_keyword] = ACTIONS(763),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [sym_struct_keyword] = ACTIONS(763),
    [sym_structure_keyword] = ACTIONS(763),
    [sym_button_event_keyword] = ACTIONS(763),
    [sym_channel_event_keyword] = ACTIONS(763),
    [sym_level_event_keyword] = ACTIONS(763),
    [sym_data_event_keyword] = ACTIONS(763),
    [sym_timeline_event_keyword] = ACTIONS(763),
    [sym_custom_event_keyword] = ACTIONS(763),
    [sym_push_keyword] = ACTIONS(763),
    [sym_release_keyword] = ACTIONS(763),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(763),
    [aux_sym_preproc_define_token1] = ACTIONS(765),
    [aux_sym_preproc_warn_token1] = ACTIONS(765),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(765),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(765),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(765),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(765),
    [aux_sym_preproc_else_token1] = ACTIONS(765),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(35)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(35),
    [sym_storage_class_specifier] = STATE(1023),
    [sym_type_qualifier] = STATE(1024),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(35),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1456),
    [sym_expression_statement] = STATE(35),
    [sym_if_statement] = STATE(35),
    [sym_switch_statement] = STATE(35),
    [sym_while_statement] = STATE(35),
    [sym_for_statement] = STATE(35),
    [sym_select_statement] = STATE(35),
    [sym_active_statement] = STATE(35),
    [sym_return_statement] = STATE(35),
    [sym_break_statement] = STATE(35),
    [sym_continue_statement] = STATE(35),
    [sym_devchan_operation_statement] = STATE(35),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(35),
    [sym_send_command_statement] = STATE(35),
    [sym_send_level_statement] = STATE(35),
    [sym_create_buffer_statement] = STATE(35),
    [sym_create_multi_buffer_statement] = STATE(35),
    [sym_clear_buffer_statement] = STATE(35),
    [sym_wait_statement] = STATE(35),
    [sym_wait_until_statement] = STATE(35),
    [sym_cancel_all_wait_statement] = STATE(35),
    [sym_cancel_all_wait_until_statement] = STATE(35),
    [sym_cancel_wait_statement] = STATE(35),
    [sym_cancel_wait_until_statement] = STATE(35),
    [sym_call_statement] = STATE(35),
    [sym_system_call_statement] = STATE(35),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(35),
    [sym_identifier] = ACTIONS(767),
    [sym_program_name_keyword] = ACTIONS(770),
    [sym_module_name_keyword] = ACTIONS(770),
    [sym_define_device_keyword] = ACTIONS(770),
    [sym_define_combine_keyword] = ACTIONS(770),
    [sym_define_constant_keyword] = ACTIONS(770),
    [sym_define_type_keyword] = ACTIONS(770),
    [sym_define_variable_keyword] = ACTIONS(770),
    [sym_define_system_variable_keyword] = ACTIONS(770),
    [sym_define_connect_level_keyword] = ACTIONS(770),
    [sym_define_latching_keyword] = ACTIONS(770),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(770),
    [sym_define_toggling_keyword] = ACTIONS(770),
    [sym_define_call_keyword] = ACTIONS(770),
    [sym_define_function_keyword] = ACTIONS(770),
    [sym_define_library_function_keyword] = ACTIONS(770),
    [sym_define_start_keyword] = ACTIONS(770),
    [sym_define_event_keyword] = ACTIONS(770),
    [sym_define_program_keyword] = ACTIONS(770),
    [sym_define_module_keyword] = ACTIONS(770),
    [sym_char_keyword] = ACTIONS(772),
    [sym_widechar_keyword] = ACTIONS(772),
    [sym_integer_keyword] = ACTIONS(772),
    [sym_sinteger_keyword] = ACTIONS(772),
    [sym_long_keyword] = ACTIONS(772),
    [sym_slong_keyword] = ACTIONS(772),
    [sym_float_keyword] = ACTIONS(772),
    [sym_double_keyword] = ACTIONS(772),
    [sym_variant_keyword] = ACTIONS(772),
    [sym_variantarray_keyword] = ACTIONS(772),
    [sym_local_var_keyword] = ACTIONS(775),
    [sym_stack_var_keyword] = ACTIONS(775),
    [sym_constant_keyword] = ACTIONS(778),
    [sym_volatile_keyword] = ACTIONS(778),
    [sym_non_volatile_keyword] = ACTIONS(778),
    [sym_persistent_keyword] = ACTIONS(778),
    [sym_dev_keyword] = ACTIONS(781),
    [sym_devlev_keyword] = ACTIONS(781),
    [sym_devchan_keyword] = ACTIONS(781),
    [sym_if_keyword] = ACTIONS(784),
    [sym_else_keyword] = ACTIONS(770),
    [sym_switch_keyword] = ACTIONS(787),
    [sym_case_keyword] = ACTIONS(770),
    [sym_default_keyword] = ACTIONS(770),
    [sym_while_keyword] = ACTIONS(790),
    [sym_for_keyword] = ACTIONS(793),
    [sym_break_keyword] = ACTIONS(796),
    [sym_continue_keyword] = ACTIONS(799),
    [sym_return_keyword] = ACTIONS(802),
    [sym_select_keyword] = ACTIONS(805),
    [sym_active_keyword] = ACTIONS(808),
    [sym_struct_keyword] = ACTIONS(770),
    [sym_structure_keyword] = ACTIONS(770),
    [sym_button_event_keyword] = ACTIONS(770),
    [sym_channel_event_keyword] = ACTIONS(770),
    [sym_level_event_keyword] = ACTIONS(770),
    [sym_data_event_keyword] = ACTIONS(770),
    [sym_timeline_event_keyword] = ACTIONS(770),
    [sym_custom_event_keyword] = ACTIONS(770),
    [sym_push_keyword] = ACTIONS(770),
    [sym_release_keyword] = ACTIONS(770),
    [aux_sym_on_keyword_token1] = ACTIONS(811),
    [aux_sym_off_keyword_token1] = ACTIONS(814),
    [sym_send_level_keyword] = ACTIONS(817),
    [sym_send_string_keyword] = ACTIONS(820),
    [sym_send_command_keyword] = ACTIONS(823),
    [sym_clear_buffer_keyword] = ACTIONS(826),
    [sym_create_buffer_keyword] = ACTIONS(829),
    [sym_create_multi_buffer_keyword] = ACTIONS(832),
    [sym_call_keyword] = ACTIONS(835),
    [sym_system_call_keyword] = ACTIONS(838),
    [sym_devchan_to_keyword] = ACTIONS(841),
    [sym_devchan_min_to_keyword] = ACTIONS(841),
    [sym_devchan_total_off_keyword] = ACTIONS(841),
    [sym_devchan_pulse_keyword] = ACTIONS(841),
    [sym_wait_keyword] = ACTIONS(844),
    [sym_cancel_wait_keyword] = ACTIONS(847),
    [sym_wait_until_keyword] = ACTIONS(850),
    [sym_cancel_wait_until_keyword] = ACTIONS(853),
    [sym_cancel_all_wait_keyword] = ACTIONS(856),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(859),
    [sym_not] = ACTIONS(862),
    [sym_bnot] = ACTIONS(862),
    [sym___file__] = ACTIONS(865),
    [sym___date__] = ACTIONS(865),
    [sym___ldate__] = ACTIONS(865),
    [sym___time__] = ACTIONS(865),
    [sym___line__] = ACTIONS(865),
    [sym___name__] = ACTIONS(865),
    [sym___netlinx__] = ACTIONS(868),
    [sym_netlinx_axi_version] = ACTIONS(868),
    [sym_url_flg_tcp] = ACTIONS(868),
    [sym_url_flg_accinfopresent] = ACTIONS(868),
    [sym_url_flg_temp] = ACTIONS(868),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(868),
    [sym_url_flg_stat_mask] = ACTIONS(868),
    [sym_url_flg_stat_lookup] = ACTIONS(868),
    [sym_url_flg_stat_connecting] = ACTIONS(868),
    [sym_url_flg_stat_waiting] = ACTIONS(868),
    [sym_url_flg_stat_connected] = ACTIONS(868),
    [sym_ip_addr_flg_dhcp] = ACTIONS(868),
    [sym_first_virtual_device] = ACTIONS(868),
    [sym_first_local_port] = ACTIONS(868),
    [sym_dynamic_virtual_device] = ACTIONS(868),
    [sym_timeline_once] = ACTIONS(868),
    [sym_timeline_repeat] = ACTIONS(868),
    [sym_timeline_absolute] = ACTIONS(868),
    [sym_timeline_relative] = ACTIONS(868),
    [sym_do_push_timed_infinite] = ACTIONS(868),
    [sym_source_type_no_address] = ACTIONS(868),
    [sym_source_type_neuron_id] = ACTIONS(868),
    [sym_source_type_ip_address] = ACTIONS(868),
    [sym_source_type_axlink] = ACTIONS(868),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(868),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(868),
    [sym_source_type_ip_socket_address] = ACTIONS(868),
    [sym_source_type_rs232] = ACTIONS(868),
    [sym_source_type_internal] = ACTIONS(868),
    [sym_source_type_mac_address] = ACTIONS(868),
    [sym_source_type_ipv4_port] = ACTIONS(868),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(868),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(868),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(868),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(868),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(868),
    [sym_true] = ACTIONS(868),
    [sym_false] = ACTIONS(868),
    [sym_file_read_only] = ACTIONS(868),
    [sym_file_rw_new] = ACTIONS(868),
    [sym_file_rw_append] = ACTIONS(868),
    [sym_ip_tcp] = ACTIONS(868),
    [sym_ip_udp] = ACTIONS(868),
    [sym_ip_udp_2way] = ACTIONS(868),
    [sym_xml_encode_types] = ACTIONS(868),
    [sym_xml_encode_char_as_list] = ACTIONS(868),
    [sym_xml_encode_le] = ACTIONS(868),
    [sym_xml_decode_types] = ACTIONS(868),
    [sym_xml_decode_no_preserve] = ACTIONS(868),
    [sym_duet_dev_type_display_device] = ACTIONS(868),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(868),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(868),
    [sym_duet_dev_type_io_device] = ACTIONS(868),
    [sym_duet_dev_type_relay_device] = ACTIONS(868),
    [sym_duet_dev_type_ups] = ACTIONS(868),
    [sym_duet_dev_type_amplifier] = ACTIONS(868),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(868),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(868),
    [sym_duet_dev_type_audio_processor] = ACTIONS(868),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(868),
    [sym_duet_dev_type_camera] = ACTIONS(868),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(868),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(868),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(868),
    [sym_duet_dev_type_dss] = ACTIONS(868),
    [sym_duet_dev_type_dvr] = ACTIONS(868),
    [sym_duet_dev_type_disc_device] = ACTIONS(868),
    [sym_duet_dev_type_document_camera] = ACTIONS(868),
    [sym_duet_dev_type_audio_tape] = ACTIONS(868),
    [sym_duet_dev_type_hvac] = ACTIONS(868),
    [sym_duet_dev_type_keypad] = ACTIONS(868),
    [sym_duet_dev_type_light] = ACTIONS(868),
    [sym_duet_dev_type_monitor] = ACTIONS(868),
    [sym_duet_dev_type_motor] = ACTIONS(868),
    [sym_duet_dev_type_multi_window] = ACTIONS(868),
    [sym_duet_dev_type_pool_spa] = ACTIONS(868),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(868),
    [sym_duet_dev_type_receiver] = ACTIONS(868),
    [sym_duet_dev_type_security_system] = ACTIONS(868),
    [sym_duet_dev_type_sensor_device] = ACTIONS(868),
    [sym_duet_dev_type_settop_box] = ACTIONS(868),
    [sym_duet_dev_type_slide_projector] = ACTIONS(868),
    [sym_duet_dev_type_switcher] = ACTIONS(868),
    [sym_duet_dev_type_text_keypad] = ACTIONS(868),
    [sym_duet_dev_type_tv] = ACTIONS(868),
    [sym_duet_dev_type_utility] = ACTIONS(868),
    [sym_duet_dev_type_vcr] = ACTIONS(868),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(868),
    [sym_duet_dev_type_video_processor] = ACTIONS(868),
    [sym_duet_dev_type_video_projector] = ACTIONS(868),
    [sym_duet_dev_type_video_wall] = ACTIONS(868),
    [sym_duet_dev_type_volume_controller] = ACTIONS(868),
    [sym_duet_dev_type_weather] = ACTIONS(868),
    [sym_duet_dev_not_polled] = ACTIONS(868),
    [sym_duet_dev_polled] = ACTIONS(868),
    [sym_clkmgr_mode_network] = ACTIONS(868),
    [sym_clkmgr_mode_standalone] = ACTIONS(868),
    [sym_normal_standby] = ACTIONS(868),
    [sym_normal_wake] = ACTIONS(868),
    [sym_ip_multicast_ttl_option] = ACTIONS(868),
    [sym_ip_ttl_subnet] = ACTIONS(868),
    [sym_ip_ttl_site] = ACTIONS(868),
    [sym_ip_ttl_region] = ACTIONS(868),
    [sym_ip_ttl_continent] = ACTIONS(868),
    [sym_ip_tcp_nodelay_option] = ACTIONS(868),
    [sym_ip_nodelay_on] = ACTIONS(868),
    [sym_ip_nodelay_off] = ACTIONS(868),
    [sym_amx_error] = ACTIONS(868),
    [sym_amx_warning] = ACTIONS(868),
    [sym_amx_info] = ACTIONS(868),
    [sym_amx_debug] = ACTIONS(868),
    [sym_netlinx_login_success] = ACTIONS(868),
    [sym_netlinx_login_fail] = ACTIONS(868),
    [sym_netlinx_logout] = ACTIONS(868),
    [sym_null_str] = ACTIONS(868),
    [sym_smtp_address] = ACTIONS(868),
    [sym_smtp_port_number] = ACTIONS(868),
    [sym_smtp_username] = ACTIONS(868),
    [sym_smtp_password] = ACTIONS(868),
    [sym_smtp_require_tls] = ACTIONS(868),
    [sym_smtp_from] = ACTIONS(868),
    [sym_smtp_tls_true] = ACTIONS(868),
    [sym_smtp_tls_false] = ACTIONS(868),
    [sym_listview_on_row_select_event] = ACTIONS(868),
    [sym_data_structure_unknown] = ACTIONS(868),
    [sym_data_structure_datafeed] = ACTIONS(868),
    [sym_data_structure_datarecordset] = ACTIONS(868),
    [sym_data_structure_datarecord] = ACTIONS(868),
    [sym_data_structure_datafield] = ACTIONS(868),
    [sym_data_max_content_field_count] = ACTIONS(868),
    [sym_data_max_metadata_field_count] = ACTIONS(868),
    [sym_data_max_id_length] = ACTIONS(868),
    [sym_data_max_name_length] = ACTIONS(868),
    [sym_data_max_description_length] = ACTIONS(868),
    [sym_data_max_type_length] = ACTIONS(868),
    [sym_data_max_format_length] = ACTIONS(868),
    [sym_data_max_source_length] = ACTIONS(868),
    [sym_data_max_value_length] = ACTIONS(868),
    [sym_data_max_label_length] = ACTIONS(868),
    [sym_data_max_url_length] = ACTIONS(868),
    [sym_max_last_login_info_length] = ACTIONS(868),
    [sym_data_type_unknown] = ACTIONS(868),
    [sym_data_type_string] = ACTIONS(868),
    [sym_data_type_datetime] = ACTIONS(868),
    [sym_data_type_date] = ACTIONS(868),
    [sym_data_type_time] = ACTIONS(868),
    [sym_data_type_image] = ACTIONS(868),
    [sym_data_format_url] = ACTIONS(868),
    [sym_data_format_phone] = ACTIONS(868),
    [sym_data_format_email] = ACTIONS(868),
    [sym_data_format_iso8601] = ACTIONS(868),
    [sym_auditlog_priv] = ACTIONS(868),
    [sym_configuration_priv] = ACTIONS(868),
    [sym_deviceconfig_priv] = ACTIONS(868),
    [sym_ftp_priv] = ACTIONS(868),
    [sym_http_priv] = ACTIONS(868),
    [sym_networkconfig_priv] = ACTIONS(868),
    [sym_programport_priv] = ACTIONS(868),
    [sym_remoteui_priv] = ACTIONS(868),
    [sym_securitycontrol_priv] = ACTIONS(868),
    [sym_softwaremanagement_priv] = ACTIONS(868),
    [sym_terminal_priv] = ACTIONS(868),
    [sym_tpadmin_priv] = ACTIONS(868),
    [sym_user1_priv] = ACTIONS(868),
    [sym_user2_priv] = ACTIONS(868),
    [sym_user3_priv] = ACTIONS(868),
    [sym_user4_priv] = ACTIONS(868),
    [sym_usermanagement_priv] = ACTIONS(868),
    [sym_valid_account] = ACTIONS(868),
    [sym_err_name_invalid] = ACTIONS(868),
    [sym_err_password_invalid] = ACTIONS(868),
    [sym_err_invalid_account] = ACTIONS(868),
    [sym_err_no_authorization] = ACTIONS(868),
    [sym_err_invalid_parameter] = ACTIONS(868),
    [sym_err_user_locked_out] = ACTIONS(868),
    [sym_err_user_account_expired] = ACTIONS(868),
    [sym_err_general_error] = ACTIONS(868),
    [sym_err_authentication_error] = ACTIONS(868),
    [sym_tls_ignore_certificate_errors] = ACTIONS(868),
    [sym_tls_validate_certificate] = ACTIONS(868),
    [sym_tdata] = ACTIONS(871),
    [sym_tchannel] = ACTIONS(871),
    [sym_tlevel] = ACTIONS(871),
    [sym_tbutton] = ACTIONS(871),
    [sym_ttimeline] = ACTIONS(871),
    [sym_tcustom] = ACTIONS(871),
    [sym_url_struct] = ACTIONS(871),
    [sym_dns_struct] = ACTIONS(871),
    [sym_ip_address_struct] = ACTIONS(871),
    [sym_dev_info_struct] = ACTIONS(871),
    [sym_clkmgr_timeserver_struct] = ACTIONS(871),
    [sym_data_feed] = ACTIONS(871),
    [sym_data_field] = ACTIONS(871),
    [sym_data_record] = ACTIONS(871),
    [sym_wc_data_feed] = ACTIONS(871),
    [sym_wc_data_field] = ACTIONS(871),
    [sym_wc_data_record] = ACTIONS(871),
    [sym_last_login_info] = ACTIONS(871),
    [sym____reserved___] = ACTIONS(874),
    [sym_date] = ACTIONS(874),
    [sym_day] = ACTIONS(874),
    [sym_time] = ACTIONS(874),
    [sym_ldate] = ACTIONS(874),
    [sym_push_device] = ACTIONS(874),
    [sym_push_channel] = ACTIONS(874),
    [sym_push_devchan] = ACTIONS(874),
    [sym_release_device] = ACTIONS(874),
    [sym_release_channel] = ACTIONS(874),
    [sym_release_devchan] = ACTIONS(874),
    [sym_master_slot] = ACTIONS(874),
    [sym_get_pulse_time] = ACTIONS(874),
    [sym_get_timer] = ACTIONS(874),
    [sym_master_sn] = ACTIONS(874),
    [sym_system_number] = ACTIONS(874),
    [sym_button] = ACTIONS(874),
    [sym_dv_channel] = ACTIONS(874),
    [sym_data] = ACTIONS(874),
    [sym_level] = ACTIONS(874),
    [sym_timeline] = ACTIONS(874),
    [sym_channel] = ACTIONS(874),
    [sym_custom] = ACTIONS(874),
    [sym_atoi] = ACTIONS(877),
    [sym_atol] = ACTIONS(877),
    [sym_atof] = ACTIONS(877),
    [sym_itoa] = ACTIONS(877),
    [sym_itohex] = ACTIONS(877),
    [sym_hextoi] = ACTIONS(877),
    [sym_ftoa] = ACTIONS(877),
    [sym_find_string] = ACTIONS(877),
    [sym_left_string] = ACTIONS(877),
    [sym_right_string] = ACTIONS(877),
    [sym_mid_string] = ACTIONS(877),
    [sym_remove_string] = ACTIONS(877),
    [sym_get_buffer_string] = ACTIONS(877),
    [sym_get_buffer_char] = ACTIONS(877),
    [sym_get_multi_buffer_string] = ACTIONS(877),
    [sym_length_string] = ACTIONS(877),
    [sym_max_length_string] = ACTIONS(877),
    [sym_set_length_string] = ACTIONS(877),
    [sym_compare_string] = ACTIONS(877),
    [sym_upper_string] = ACTIONS(877),
    [sym_lower_string] = ACTIONS(877),
    [sym_get_last] = ACTIONS(877),
    [sym_date_to_day] = ACTIONS(877),
    [sym_date_to_month] = ACTIONS(877),
    [sym_date_to_year] = ACTIONS(877),
    [sym_time_to_hour] = ACTIONS(877),
    [sym_time_to_minute] = ACTIONS(877),
    [sym_time_to_second] = ACTIONS(877),
    [sym_day_of_week] = ACTIONS(877),
    [sym_set_pulse_time] = ACTIONS(877),
    [sym_set_timer] = ACTIONS(877),
    [sym_file_open] = ACTIONS(877),
    [sym_file_close] = ACTIONS(877),
    [sym_file_read] = ACTIONS(877),
    [sym_file_read_line] = ACTIONS(877),
    [sym_file_write] = ACTIONS(877),
    [sym_file_write_line] = ACTIONS(877),
    [sym_file_copy] = ACTIONS(877),
    [sym_file_delete] = ACTIONS(877),
    [sym_file_rename] = ACTIONS(877),
    [sym_file_dir] = ACTIONS(877),
    [sym_file_getdir] = ACTIONS(877),
    [sym_file_setdir] = ACTIONS(877),
    [sym_file_create_dir] = ACTIONS(877),
    [sym_file_remove_dir] = ACTIONS(877),
    [sym_file_seek] = ACTIONS(877),
    [sym_length_array] = ACTIONS(877),
    [sym_max_length_array] = ACTIONS(877),
    [sym_set_length_array] = ACTIONS(877),
    [sym_ip_client_open] = ACTIONS(877),
    [sym_ip_client_close] = ACTIONS(877),
    [sym_ip_server_open] = ACTIONS(877),
    [sym_ip_server_close] = ACTIONS(877),
    [sym_ip_bound_client_open] = ACTIONS(877),
    [sym_ip_mc_server_open] = ACTIONS(877),
    [sym_ip_set_option] = ACTIONS(877),
    [sym_ssh_client_open] = ACTIONS(877),
    [sym_ssh_client_close] = ACTIONS(877),
    [sym_tls_client_open] = ACTIONS(877),
    [sym_tls_client_close] = ACTIONS(877),
    [sym_do_push] = ACTIONS(877),
    [sym_do_push_timed] = ACTIONS(877),
    [sym_do_release] = ACTIONS(877),
    [sym_device_id] = ACTIONS(877),
    [sym_device_info] = ACTIONS(877),
    [sym_device_id_string] = ACTIONS(877),
    [sym_device_standby] = ACTIONS(877),
    [sym_device_wake] = ACTIONS(877),
    [sym_timeline_create] = ACTIONS(877),
    [sym_timeline_kill] = ACTIONS(877),
    [sym_timeline_reload] = ACTIONS(877),
    [sym_timeline_pause] = ACTIONS(877),
    [sym_timeline_restart] = ACTIONS(877),
    [sym_timeline_set] = ACTIONS(877),
    [sym_timeline_get] = ACTIONS(877),
    [sym_timeline_active] = ACTIONS(877),
    [sym_timeline_dynamic_id] = ACTIONS(877),
    [sym_variable_to_string] = ACTIONS(877),
    [sym_string_to_variable] = ACTIONS(877),
    [sym_length_variable_to_string] = ACTIONS(877),
    [sym_type_cast] = ACTIONS(877),
    [sym_variable_to_xml] = ACTIONS(877),
    [sym_xml_to_variable] = ACTIONS(877),
    [sym_length_variable_to_xml] = ACTIONS(877),
    [sym_redirect_string] = ACTIONS(877),
    [sym_random_number] = ACTIONS(877),
    [sym_get_unique_id] = ACTIONS(877),
    [sym_get_system_number] = ACTIONS(877),
    [sym_set_system_number] = ACTIONS(877),
    [sym_get_serial_number] = ACTIONS(877),
    [sym_get_master_build] = ACTIONS(877),
    [sym_set_validation_code] = ACTIONS(877),
    [sym_reboot] = ACTIONS(877),
    [sym_load_duet_module] = ACTIONS(877),
    [sym_unload_duet_module] = ACTIONS(877),
    [sym_duet_mem_size_set] = ACTIONS(877),
    [sym_duet_mem_size_get] = ACTIONS(877),
    [sym_dynamic_application_device] = ACTIONS(877),
    [sym_dynamic_polled_port] = ACTIONS(877),
    [sym_static_port_binding] = ACTIONS(877),
    [sym_static_ip_binding] = ACTIONS(877),
    [sym_get_dns_list] = ACTIONS(877),
    [sym_set_dns_list] = ACTIONS(877),
    [sym_get_ip_address] = ACTIONS(877),
    [sym_set_ip_address] = ACTIONS(877),
    [sym_get_url_list] = ACTIONS(877),
    [sym_add_url_entry] = ACTIONS(877),
    [sym_delete_url_entry] = ACTIONS(877),
    [sym_abs_value] = ACTIONS(877),
    [sym_max_value] = ACTIONS(877),
    [sym_min_value] = ACTIONS(877),
    [sym_exp_value] = ACTIONS(877),
    [sym_log_value] = ACTIONS(877),
    [sym_log10_value] = ACTIONS(877),
    [sym_power_value] = ACTIONS(877),
    [sym_sqrt_value] = ACTIONS(877),
    [sym_raw_be] = ACTIONS(877),
    [sym_raw_le] = ACTIONS(877),
    [sym_internal_queue_size_set] = ACTIONS(877),
    [sym_internal_queue_size_get] = ACTIONS(877),
    [sym_internal_threshold_set] = ACTIONS(877),
    [sym_internal_threshold_get] = ACTIONS(877),
    [sym_rebuilt_event] = ACTIONS(877),
    [sym_do_custom_event] = ACTIONS(877),
    [sym_clkmgr_is_network_sourced] = ACTIONS(877),
    [sym_clkmgr_set_clk_source] = ACTIONS(877),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(877),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(877),
    [sym_clkmgr_get_timezone] = ACTIONS(877),
    [sym_clkmgr_set_timezone] = ACTIONS(877),
    [sym_clkmgr_get_resync_period] = ACTIONS(877),
    [sym_clkmgr_set_resync_period] = ACTIONS(877),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(877),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(877),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(877),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(877),
    [sym_clkmgr_get_timeservers] = ACTIONS(877),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(877),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(877),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(877),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(877),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(877),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(877),
    [sym_set_log_level] = ACTIONS(877),
    [sym_get_log_level] = ACTIONS(877),
    [sym_amx_log] = ACTIONS(877),
    [sym_smtp_server_config_set] = ACTIONS(877),
    [sym_smtp_server_config_get] = ACTIONS(877),
    [sym_smtp_send] = ACTIONS(877),
    [sym_validate_netlinx_account] = ACTIONS(877),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(877),
    [sym_authenticate_certificate] = ACTIONS(877),
    [sym_audit_netlinx_session_event] = ACTIONS(877),
    [sym_audit_netlinx_generic_event] = ACTIONS(877),
    [sym_get_available_flash_disk_space] = ACTIONS(877),
    [sym_get_max_flash_disk_space] = ACTIONS(877),
    [sym_data_create_feed] = ACTIONS(877),
    [sym_data_delete_feed] = ACTIONS(877),
    [sym_data_publish_feed] = ACTIONS(877),
    [sym_data_get_published_feed] = ACTIONS(877),
    [sym_data_add_record] = ACTIONS(877),
    [sym_data_get_event_record] = ACTIONS(877),
    [sym__wc_data_create_feed] = ACTIONS(877),
    [sym__wc_data_add_record] = ACTIONS(877),
    [sym__wc_data_get_event_record] = ACTIONS(877),
    [sym_format] = ACTIONS(877),
    [sym_astro_clock] = ACTIONS(877),
    [sym_set_outdoor_temperature] = ACTIONS(877),
    [sym_set_virtual_level_count] = ACTIONS(877),
    [sym_set_virtual_channel_count] = ACTIONS(877),
    [sym_set_virtual_port_count] = ACTIONS(877),
    [sym__wc_encode_int] = ACTIONS(877),
    [sym__wc_decode_int] = ACTIONS(877),
    [sym__wc_int] = ACTIONS(877),
    [sym__wc_to_ch_int] = ACTIONS(877),
    [sym__ch_to_wc_int] = ACTIONS(877),
    [sym__wc_find_string_int] = ACTIONS(877),
    [sym__wc_left_string_int] = ACTIONS(877),
    [sym__wc_lower_string_int] = ACTIONS(877),
    [sym__wc_mid_string_int] = ACTIONS(877),
    [sym__wc_remove_string_int] = ACTIONS(877),
    [sym__wc_right_string_int] = ACTIONS(877),
    [sym__wc_upper_string_int] = ACTIONS(877),
    [sym__wc_compare_string_int] = ACTIONS(877),
    [sym__wc_get_buffer_char_int] = ACTIONS(877),
    [sym__wc_get_buffer_string_int] = ACTIONS(877),
    [sym__wc_concat_string_int] = ACTIONS(877),
    [sym__wc_file_open_int] = ACTIONS(877),
    [sym__wc_file_close_int] = ACTIONS(877),
    [sym__wc_file_read_int] = ACTIONS(877),
    [sym__wc_file_read_line_int] = ACTIONS(877),
    [sym__wc_file_write_int] = ACTIONS(877),
    [sym__wc_file_write_line_int] = ACTIONS(877),
    [aux_sym_preproc_include_token1] = ACTIONS(770),
    [aux_sym_preproc_define_token1] = ACTIONS(880),
    [aux_sym_preproc_warn_token1] = ACTIONS(880),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(880),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(880),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(880),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(880),
    [aux_sym_preproc_else_token1] = ACTIONS(880),
    [anon_sym_LBRACK] = ACTIONS(882),
    [anon_sym_LBRACE] = ACTIONS(885),
    [anon_sym_LPAREN] = ACTIONS(888),
    [anon_sym_BANG] = ACTIONS(891),
    [anon_sym_TILDE] = ACTIONS(891),
    [anon_sym_DASH] = ACTIONS(862),
    [anon_sym_PLUS] = ACTIONS(862),
    [anon_sym_DASH_DASH] = ACTIONS(894),
    [anon_sym_PLUS_PLUS] = ACTIONS(894),
    [anon_sym_DQUOTE] = ACTIONS(897),
    [anon_sym_SQUOTE] = ACTIONS(900),
    [sym_number_literal] = ACTIONS(903),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(36)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(37),
    [sym_storage_class_specifier] = STATE(1028),
    [sym_type_qualifier] = STATE(1013),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(37),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1457),
    [sym_expression_statement] = STATE(37),
    [sym_if_statement] = STATE(37),
    [sym_switch_statement] = STATE(37),
    [sym_while_statement] = STATE(37),
    [sym_for_statement] = STATE(37),
    [sym_select_statement] = STATE(37),
    [sym_active_statement] = STATE(37),
    [sym_return_statement] = STATE(37),
    [sym_break_statement] = STATE(37),
    [sym_continue_statement] = STATE(37),
    [sym_devchan_operation_statement] = STATE(37),
    [sym_devchan_operation] = STATE(1878),
    [sym_send_string_statement] = STATE(37),
    [sym_send_command_statement] = STATE(37),
    [sym_send_level_statement] = STATE(37),
    [sym_create_buffer_statement] = STATE(37),
    [sym_create_multi_buffer_statement] = STATE(37),
    [sym_clear_buffer_statement] = STATE(37),
    [sym_wait_statement] = STATE(37),
    [sym_wait_until_statement] = STATE(37),
    [sym_cancel_all_wait_statement] = STATE(37),
    [sym_cancel_all_wait_until_statement] = STATE(37),
    [sym_cancel_wait_statement] = STATE(37),
    [sym_cancel_wait_until_statement] = STATE(37),
    [sym_call_statement] = STATE(37),
    [sym_system_call_statement] = STATE(37),
    [sym_expression] = STATE(1137),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1990),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(37),
    [ts_builtin_sym_end] = ACTIONS(765),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(763),
    [sym_module_name_keyword] = ACTIONS(763),
    [sym_define_device_keyword] = ACTIONS(763),
    [sym_define_combine_keyword] = ACTIONS(763),
    [sym_define_constant_keyword] = ACTIONS(763),
    [sym_define_type_keyword] = ACTIONS(763),
    [sym_define_variable_keyword] = ACTIONS(763),
    [sym_define_system_variable_keyword] = ACTIONS(763),
    [sym_define_connect_level_keyword] = ACTIONS(763),
    [sym_define_latching_keyword] = ACTIONS(763),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(763),
    [sym_define_toggling_keyword] = ACTIONS(763),
    [sym_define_call_keyword] = ACTIONS(763),
    [sym_define_function_keyword] = ACTIONS(763),
    [sym_define_library_function_keyword] = ACTIONS(763),
    [sym_define_start_keyword] = ACTIONS(763),
    [sym_define_event_keyword] = ACTIONS(763),
    [sym_define_program_keyword] = ACTIONS(763),
    [sym_define_module_keyword] = ACTIONS(763),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(55),
    [sym_else_keyword] = ACTIONS(763),
    [sym_switch_keyword] = ACTIONS(57),
    [sym_case_keyword] = ACTIONS(763),
    [sym_default_keyword] = ACTIONS(763),
    [sym_while_keyword] = ACTIONS(63),
    [sym_for_keyword] = ACTIONS(65),
    [sym_break_keyword] = ACTIONS(67),
    [sym_continue_keyword] = ACTIONS(69),
    [sym_return_keyword] = ACTIONS(71),
    [sym_select_keyword] = ACTIONS(73),
    [sym_active_keyword] = ACTIONS(75),
    [sym_struct_keyword] = ACTIONS(763),
    [sym_structure_keyword] = ACTIONS(763),
    [sym_button_event_keyword] = ACTIONS(763),
    [sym_channel_event_keyword] = ACTIONS(763),
    [sym_level_event_keyword] = ACTIONS(763),
    [sym_data_event_keyword] = ACTIONS(763),
    [sym_timeline_event_keyword] = ACTIONS(763),
    [sym_custom_event_keyword] = ACTIONS(763),
    [sym_push_keyword] = ACTIONS(763),
    [sym_release_keyword] = ACTIONS(763),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(99),
    [sym_send_string_keyword] = ACTIONS(101),
    [sym_send_command_keyword] = ACTIONS(103),
    [sym_clear_buffer_keyword] = ACTIONS(105),
    [sym_create_buffer_keyword] = ACTIONS(107),
    [sym_create_multi_buffer_keyword] = ACTIONS(109),
    [sym_call_keyword] = ACTIONS(111),
    [sym_system_call_keyword] = ACTIONS(113),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(117),
    [sym_cancel_wait_keyword] = ACTIONS(119),
    [sym_wait_until_keyword] = ACTIONS(121),
    [sym_cancel_wait_until_keyword] = ACTIONS(123),
    [sym_cancel_all_wait_keyword] = ACTIONS(125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(127),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(763),
    [aux_sym_preproc_define_token1] = ACTIONS(765),
    [aux_sym_preproc_warn_token1] = ACTIONS(765),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(765),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(765),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(765),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(37)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(37),
    [sym_storage_class_specifier] = STATE(1028),
    [sym_type_qualifier] = STATE(1013),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(37),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1457),
    [sym_expression_statement] = STATE(37),
    [sym_if_statement] = STATE(37),
    [sym_switch_statement] = STATE(37),
    [sym_while_statement] = STATE(37),
    [sym_for_statement] = STATE(37),
    [sym_select_statement] = STATE(37),
    [sym_active_statement] = STATE(37),
    [sym_return_statement] = STATE(37),
    [sym_break_statement] = STATE(37),
    [sym_continue_statement] = STATE(37),
    [sym_devchan_operation_statement] = STATE(37),
    [sym_devchan_operation] = STATE(1878),
    [sym_send_string_statement] = STATE(37),
    [sym_send_command_statement] = STATE(37),
    [sym_send_level_statement] = STATE(37),
    [sym_create_buffer_statement] = STATE(37),
    [sym_create_multi_buffer_statement] = STATE(37),
    [sym_clear_buffer_statement] = STATE(37),
    [sym_wait_statement] = STATE(37),
    [sym_wait_until_statement] = STATE(37),
    [sym_cancel_all_wait_statement] = STATE(37),
    [sym_cancel_all_wait_until_statement] = STATE(37),
    [sym_cancel_wait_statement] = STATE(37),
    [sym_cancel_wait_until_statement] = STATE(37),
    [sym_call_statement] = STATE(37),
    [sym_system_call_statement] = STATE(37),
    [sym_expression] = STATE(1137),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1990),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(37),
    [ts_builtin_sym_end] = ACTIONS(880),
    [sym_identifier] = ACTIONS(767),
    [sym_program_name_keyword] = ACTIONS(770),
    [sym_module_name_keyword] = ACTIONS(770),
    [sym_define_device_keyword] = ACTIONS(770),
    [sym_define_combine_keyword] = ACTIONS(770),
    [sym_define_constant_keyword] = ACTIONS(770),
    [sym_define_type_keyword] = ACTIONS(770),
    [sym_define_variable_keyword] = ACTIONS(770),
    [sym_define_system_variable_keyword] = ACTIONS(770),
    [sym_define_connect_level_keyword] = ACTIONS(770),
    [sym_define_latching_keyword] = ACTIONS(770),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(770),
    [sym_define_toggling_keyword] = ACTIONS(770),
    [sym_define_call_keyword] = ACTIONS(770),
    [sym_define_function_keyword] = ACTIONS(770),
    [sym_define_library_function_keyword] = ACTIONS(770),
    [sym_define_start_keyword] = ACTIONS(770),
    [sym_define_event_keyword] = ACTIONS(770),
    [sym_define_program_keyword] = ACTIONS(770),
    [sym_define_module_keyword] = ACTIONS(770),
    [sym_char_keyword] = ACTIONS(772),
    [sym_widechar_keyword] = ACTIONS(772),
    [sym_integer_keyword] = ACTIONS(772),
    [sym_sinteger_keyword] = ACTIONS(772),
    [sym_long_keyword] = ACTIONS(772),
    [sym_slong_keyword] = ACTIONS(772),
    [sym_float_keyword] = ACTIONS(772),
    [sym_double_keyword] = ACTIONS(772),
    [sym_variant_keyword] = ACTIONS(772),
    [sym_variantarray_keyword] = ACTIONS(772),
    [sym_local_var_keyword] = ACTIONS(775),
    [sym_stack_var_keyword] = ACTIONS(775),
    [sym_constant_keyword] = ACTIONS(778),
    [sym_volatile_keyword] = ACTIONS(778),
    [sym_non_volatile_keyword] = ACTIONS(778),
    [sym_persistent_keyword] = ACTIONS(778),
    [sym_dev_keyword] = ACTIONS(781),
    [sym_devlev_keyword] = ACTIONS(781),
    [sym_devchan_keyword] = ACTIONS(781),
    [sym_if_keyword] = ACTIONS(906),
    [sym_else_keyword] = ACTIONS(770),
    [sym_switch_keyword] = ACTIONS(909),
    [sym_case_keyword] = ACTIONS(770),
    [sym_default_keyword] = ACTIONS(770),
    [sym_while_keyword] = ACTIONS(912),
    [sym_for_keyword] = ACTIONS(915),
    [sym_break_keyword] = ACTIONS(918),
    [sym_continue_keyword] = ACTIONS(921),
    [sym_return_keyword] = ACTIONS(924),
    [sym_select_keyword] = ACTIONS(927),
    [sym_active_keyword] = ACTIONS(930),
    [sym_struct_keyword] = ACTIONS(770),
    [sym_structure_keyword] = ACTIONS(770),
    [sym_button_event_keyword] = ACTIONS(770),
    [sym_channel_event_keyword] = ACTIONS(770),
    [sym_level_event_keyword] = ACTIONS(770),
    [sym_data_event_keyword] = ACTIONS(770),
    [sym_timeline_event_keyword] = ACTIONS(770),
    [sym_custom_event_keyword] = ACTIONS(770),
    [sym_push_keyword] = ACTIONS(770),
    [sym_release_keyword] = ACTIONS(770),
    [aux_sym_on_keyword_token1] = ACTIONS(811),
    [aux_sym_off_keyword_token1] = ACTIONS(814),
    [sym_send_level_keyword] = ACTIONS(933),
    [sym_send_string_keyword] = ACTIONS(936),
    [sym_send_command_keyword] = ACTIONS(939),
    [sym_clear_buffer_keyword] = ACTIONS(942),
    [sym_create_buffer_keyword] = ACTIONS(945),
    [sym_create_multi_buffer_keyword] = ACTIONS(948),
    [sym_call_keyword] = ACTIONS(951),
    [sym_system_call_keyword] = ACTIONS(954),
    [sym_devchan_to_keyword] = ACTIONS(841),
    [sym_devchan_min_to_keyword] = ACTIONS(841),
    [sym_devchan_total_off_keyword] = ACTIONS(841),
    [sym_devchan_pulse_keyword] = ACTIONS(841),
    [sym_wait_keyword] = ACTIONS(957),
    [sym_cancel_wait_keyword] = ACTIONS(960),
    [sym_wait_until_keyword] = ACTIONS(963),
    [sym_cancel_wait_until_keyword] = ACTIONS(966),
    [sym_cancel_all_wait_keyword] = ACTIONS(969),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(972),
    [sym_not] = ACTIONS(862),
    [sym_bnot] = ACTIONS(862),
    [sym___file__] = ACTIONS(865),
    [sym___date__] = ACTIONS(865),
    [sym___ldate__] = ACTIONS(865),
    [sym___time__] = ACTIONS(865),
    [sym___line__] = ACTIONS(865),
    [sym___name__] = ACTIONS(865),
    [sym___netlinx__] = ACTIONS(868),
    [sym_netlinx_axi_version] = ACTIONS(868),
    [sym_url_flg_tcp] = ACTIONS(868),
    [sym_url_flg_accinfopresent] = ACTIONS(868),
    [sym_url_flg_temp] = ACTIONS(868),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(868),
    [sym_url_flg_stat_mask] = ACTIONS(868),
    [sym_url_flg_stat_lookup] = ACTIONS(868),
    [sym_url_flg_stat_connecting] = ACTIONS(868),
    [sym_url_flg_stat_waiting] = ACTIONS(868),
    [sym_url_flg_stat_connected] = ACTIONS(868),
    [sym_ip_addr_flg_dhcp] = ACTIONS(868),
    [sym_first_virtual_device] = ACTIONS(868),
    [sym_first_local_port] = ACTIONS(868),
    [sym_dynamic_virtual_device] = ACTIONS(868),
    [sym_timeline_once] = ACTIONS(868),
    [sym_timeline_repeat] = ACTIONS(868),
    [sym_timeline_absolute] = ACTIONS(868),
    [sym_timeline_relative] = ACTIONS(868),
    [sym_do_push_timed_infinite] = ACTIONS(868),
    [sym_source_type_no_address] = ACTIONS(868),
    [sym_source_type_neuron_id] = ACTIONS(868),
    [sym_source_type_ip_address] = ACTIONS(868),
    [sym_source_type_axlink] = ACTIONS(868),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(868),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(868),
    [sym_source_type_ip_socket_address] = ACTIONS(868),
    [sym_source_type_rs232] = ACTIONS(868),
    [sym_source_type_internal] = ACTIONS(868),
    [sym_source_type_mac_address] = ACTIONS(868),
    [sym_source_type_ipv4_port] = ACTIONS(868),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(868),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(868),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(868),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(868),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(868),
    [sym_true] = ACTIONS(868),
    [sym_false] = ACTIONS(868),
    [sym_file_read_only] = ACTIONS(868),
    [sym_file_rw_new] = ACTIONS(868),
    [sym_file_rw_append] = ACTIONS(868),
    [sym_ip_tcp] = ACTIONS(868),
    [sym_ip_udp] = ACTIONS(868),
    [sym_ip_udp_2way] = ACTIONS(868),
    [sym_xml_encode_types] = ACTIONS(868),
    [sym_xml_encode_char_as_list] = ACTIONS(868),
    [sym_xml_encode_le] = ACTIONS(868),
    [sym_xml_decode_types] = ACTIONS(868),
    [sym_xml_decode_no_preserve] = ACTIONS(868),
    [sym_duet_dev_type_display_device] = ACTIONS(868),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(868),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(868),
    [sym_duet_dev_type_io_device] = ACTIONS(868),
    [sym_duet_dev_type_relay_device] = ACTIONS(868),
    [sym_duet_dev_type_ups] = ACTIONS(868),
    [sym_duet_dev_type_amplifier] = ACTIONS(868),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(868),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(868),
    [sym_duet_dev_type_audio_processor] = ACTIONS(868),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(868),
    [sym_duet_dev_type_camera] = ACTIONS(868),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(868),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(868),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(868),
    [sym_duet_dev_type_dss] = ACTIONS(868),
    [sym_duet_dev_type_dvr] = ACTIONS(868),
    [sym_duet_dev_type_disc_device] = ACTIONS(868),
    [sym_duet_dev_type_document_camera] = ACTIONS(868),
    [sym_duet_dev_type_audio_tape] = ACTIONS(868),
    [sym_duet_dev_type_hvac] = ACTIONS(868),
    [sym_duet_dev_type_keypad] = ACTIONS(868),
    [sym_duet_dev_type_light] = ACTIONS(868),
    [sym_duet_dev_type_monitor] = ACTIONS(868),
    [sym_duet_dev_type_motor] = ACTIONS(868),
    [sym_duet_dev_type_multi_window] = ACTIONS(868),
    [sym_duet_dev_type_pool_spa] = ACTIONS(868),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(868),
    [sym_duet_dev_type_receiver] = ACTIONS(868),
    [sym_duet_dev_type_security_system] = ACTIONS(868),
    [sym_duet_dev_type_sensor_device] = ACTIONS(868),
    [sym_duet_dev_type_settop_box] = ACTIONS(868),
    [sym_duet_dev_type_slide_projector] = ACTIONS(868),
    [sym_duet_dev_type_switcher] = ACTIONS(868),
    [sym_duet_dev_type_text_keypad] = ACTIONS(868),
    [sym_duet_dev_type_tv] = ACTIONS(868),
    [sym_duet_dev_type_utility] = ACTIONS(868),
    [sym_duet_dev_type_vcr] = ACTIONS(868),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(868),
    [sym_duet_dev_type_video_processor] = ACTIONS(868),
    [sym_duet_dev_type_video_projector] = ACTIONS(868),
    [sym_duet_dev_type_video_wall] = ACTIONS(868),
    [sym_duet_dev_type_volume_controller] = ACTIONS(868),
    [sym_duet_dev_type_weather] = ACTIONS(868),
    [sym_duet_dev_not_polled] = ACTIONS(868),
    [sym_duet_dev_polled] = ACTIONS(868),
    [sym_clkmgr_mode_network] = ACTIONS(868),
    [sym_clkmgr_mode_standalone] = ACTIONS(868),
    [sym_normal_standby] = ACTIONS(868),
    [sym_normal_wake] = ACTIONS(868),
    [sym_ip_multicast_ttl_option] = ACTIONS(868),
    [sym_ip_ttl_subnet] = ACTIONS(868),
    [sym_ip_ttl_site] = ACTIONS(868),
    [sym_ip_ttl_region] = ACTIONS(868),
    [sym_ip_ttl_continent] = ACTIONS(868),
    [sym_ip_tcp_nodelay_option] = ACTIONS(868),
    [sym_ip_nodelay_on] = ACTIONS(868),
    [sym_ip_nodelay_off] = ACTIONS(868),
    [sym_amx_error] = ACTIONS(868),
    [sym_amx_warning] = ACTIONS(868),
    [sym_amx_info] = ACTIONS(868),
    [sym_amx_debug] = ACTIONS(868),
    [sym_netlinx_login_success] = ACTIONS(868),
    [sym_netlinx_login_fail] = ACTIONS(868),
    [sym_netlinx_logout] = ACTIONS(868),
    [sym_null_str] = ACTIONS(868),
    [sym_smtp_address] = ACTIONS(868),
    [sym_smtp_port_number] = ACTIONS(868),
    [sym_smtp_username] = ACTIONS(868),
    [sym_smtp_password] = ACTIONS(868),
    [sym_smtp_require_tls] = ACTIONS(868),
    [sym_smtp_from] = ACTIONS(868),
    [sym_smtp_tls_true] = ACTIONS(868),
    [sym_smtp_tls_false] = ACTIONS(868),
    [sym_listview_on_row_select_event] = ACTIONS(868),
    [sym_data_structure_unknown] = ACTIONS(868),
    [sym_data_structure_datafeed] = ACTIONS(868),
    [sym_data_structure_datarecordset] = ACTIONS(868),
    [sym_data_structure_datarecord] = ACTIONS(868),
    [sym_data_structure_datafield] = ACTIONS(868),
    [sym_data_max_content_field_count] = ACTIONS(868),
    [sym_data_max_metadata_field_count] = ACTIONS(868),
    [sym_data_max_id_length] = ACTIONS(868),
    [sym_data_max_name_length] = ACTIONS(868),
    [sym_data_max_description_length] = ACTIONS(868),
    [sym_data_max_type_length] = ACTIONS(868),
    [sym_data_max_format_length] = ACTIONS(868),
    [sym_data_max_source_length] = ACTIONS(868),
    [sym_data_max_value_length] = ACTIONS(868),
    [sym_data_max_label_length] = ACTIONS(868),
    [sym_data_max_url_length] = ACTIONS(868),
    [sym_max_last_login_info_length] = ACTIONS(868),
    [sym_data_type_unknown] = ACTIONS(868),
    [sym_data_type_string] = ACTIONS(868),
    [sym_data_type_datetime] = ACTIONS(868),
    [sym_data_type_date] = ACTIONS(868),
    [sym_data_type_time] = ACTIONS(868),
    [sym_data_type_image] = ACTIONS(868),
    [sym_data_format_url] = ACTIONS(868),
    [sym_data_format_phone] = ACTIONS(868),
    [sym_data_format_email] = ACTIONS(868),
    [sym_data_format_iso8601] = ACTIONS(868),
    [sym_auditlog_priv] = ACTIONS(868),
    [sym_configuration_priv] = ACTIONS(868),
    [sym_deviceconfig_priv] = ACTIONS(868),
    [sym_ftp_priv] = ACTIONS(868),
    [sym_http_priv] = ACTIONS(868),
    [sym_networkconfig_priv] = ACTIONS(868),
    [sym_programport_priv] = ACTIONS(868),
    [sym_remoteui_priv] = ACTIONS(868),
    [sym_securitycontrol_priv] = ACTIONS(868),
    [sym_softwaremanagement_priv] = ACTIONS(868),
    [sym_terminal_priv] = ACTIONS(868),
    [sym_tpadmin_priv] = ACTIONS(868),
    [sym_user1_priv] = ACTIONS(868),
    [sym_user2_priv] = ACTIONS(868),
    [sym_user3_priv] = ACTIONS(868),
    [sym_user4_priv] = ACTIONS(868),
    [sym_usermanagement_priv] = ACTIONS(868),
    [sym_valid_account] = ACTIONS(868),
    [sym_err_name_invalid] = ACTIONS(868),
    [sym_err_password_invalid] = ACTIONS(868),
    [sym_err_invalid_account] = ACTIONS(868),
    [sym_err_no_authorization] = ACTIONS(868),
    [sym_err_invalid_parameter] = ACTIONS(868),
    [sym_err_user_locked_out] = ACTIONS(868),
    [sym_err_user_account_expired] = ACTIONS(868),
    [sym_err_general_error] = ACTIONS(868),
    [sym_err_authentication_error] = ACTIONS(868),
    [sym_tls_ignore_certificate_errors] = ACTIONS(868),
    [sym_tls_validate_certificate] = ACTIONS(868),
    [sym_tdata] = ACTIONS(871),
    [sym_tchannel] = ACTIONS(871),
    [sym_tlevel] = ACTIONS(871),
    [sym_tbutton] = ACTIONS(871),
    [sym_ttimeline] = ACTIONS(871),
    [sym_tcustom] = ACTIONS(871),
    [sym_url_struct] = ACTIONS(871),
    [sym_dns_struct] = ACTIONS(871),
    [sym_ip_address_struct] = ACTIONS(871),
    [sym_dev_info_struct] = ACTIONS(871),
    [sym_clkmgr_timeserver_struct] = ACTIONS(871),
    [sym_data_feed] = ACTIONS(871),
    [sym_data_field] = ACTIONS(871),
    [sym_data_record] = ACTIONS(871),
    [sym_wc_data_feed] = ACTIONS(871),
    [sym_wc_data_field] = ACTIONS(871),
    [sym_wc_data_record] = ACTIONS(871),
    [sym_last_login_info] = ACTIONS(871),
    [sym____reserved___] = ACTIONS(874),
    [sym_date] = ACTIONS(874),
    [sym_day] = ACTIONS(874),
    [sym_time] = ACTIONS(874),
    [sym_ldate] = ACTIONS(874),
    [sym_push_device] = ACTIONS(874),
    [sym_push_channel] = ACTIONS(874),
    [sym_push_devchan] = ACTIONS(874),
    [sym_release_device] = ACTIONS(874),
    [sym_release_channel] = ACTIONS(874),
    [sym_release_devchan] = ACTIONS(874),
    [sym_master_slot] = ACTIONS(874),
    [sym_get_pulse_time] = ACTIONS(874),
    [sym_get_timer] = ACTIONS(874),
    [sym_master_sn] = ACTIONS(874),
    [sym_system_number] = ACTIONS(874),
    [sym_button] = ACTIONS(874),
    [sym_dv_channel] = ACTIONS(874),
    [sym_data] = ACTIONS(874),
    [sym_level] = ACTIONS(874),
    [sym_timeline] = ACTIONS(874),
    [sym_channel] = ACTIONS(874),
    [sym_custom] = ACTIONS(874),
    [sym_atoi] = ACTIONS(877),
    [sym_atol] = ACTIONS(877),
    [sym_atof] = ACTIONS(877),
    [sym_itoa] = ACTIONS(877),
    [sym_itohex] = ACTIONS(877),
    [sym_hextoi] = ACTIONS(877),
    [sym_ftoa] = ACTIONS(877),
    [sym_find_string] = ACTIONS(877),
    [sym_left_string] = ACTIONS(877),
    [sym_right_string] = ACTIONS(877),
    [sym_mid_string] = ACTIONS(877),
    [sym_remove_string] = ACTIONS(877),
    [sym_get_buffer_string] = ACTIONS(877),
    [sym_get_buffer_char] = ACTIONS(877),
    [sym_get_multi_buffer_string] = ACTIONS(877),
    [sym_length_string] = ACTIONS(877),
    [sym_max_length_string] = ACTIONS(877),
    [sym_set_length_string] = ACTIONS(877),
    [sym_compare_string] = ACTIONS(877),
    [sym_upper_string] = ACTIONS(877),
    [sym_lower_string] = ACTIONS(877),
    [sym_get_last] = ACTIONS(877),
    [sym_date_to_day] = ACTIONS(877),
    [sym_date_to_month] = ACTIONS(877),
    [sym_date_to_year] = ACTIONS(877),
    [sym_time_to_hour] = ACTIONS(877),
    [sym_time_to_minute] = ACTIONS(877),
    [sym_time_to_second] = ACTIONS(877),
    [sym_day_of_week] = ACTIONS(877),
    [sym_set_pulse_time] = ACTIONS(877),
    [sym_set_timer] = ACTIONS(877),
    [sym_file_open] = ACTIONS(877),
    [sym_file_close] = ACTIONS(877),
    [sym_file_read] = ACTIONS(877),
    [sym_file_read_line] = ACTIONS(877),
    [sym_file_write] = ACTIONS(877),
    [sym_file_write_line] = ACTIONS(877),
    [sym_file_copy] = ACTIONS(877),
    [sym_file_delete] = ACTIONS(877),
    [sym_file_rename] = ACTIONS(877),
    [sym_file_dir] = ACTIONS(877),
    [sym_file_getdir] = ACTIONS(877),
    [sym_file_setdir] = ACTIONS(877),
    [sym_file_create_dir] = ACTIONS(877),
    [sym_file_remove_dir] = ACTIONS(877),
    [sym_file_seek] = ACTIONS(877),
    [sym_length_array] = ACTIONS(877),
    [sym_max_length_array] = ACTIONS(877),
    [sym_set_length_array] = ACTIONS(877),
    [sym_ip_client_open] = ACTIONS(877),
    [sym_ip_client_close] = ACTIONS(877),
    [sym_ip_server_open] = ACTIONS(877),
    [sym_ip_server_close] = ACTIONS(877),
    [sym_ip_bound_client_open] = ACTIONS(877),
    [sym_ip_mc_server_open] = ACTIONS(877),
    [sym_ip_set_option] = ACTIONS(877),
    [sym_ssh_client_open] = ACTIONS(877),
    [sym_ssh_client_close] = ACTIONS(877),
    [sym_tls_client_open] = ACTIONS(877),
    [sym_tls_client_close] = ACTIONS(877),
    [sym_do_push] = ACTIONS(877),
    [sym_do_push_timed] = ACTIONS(877),
    [sym_do_release] = ACTIONS(877),
    [sym_device_id] = ACTIONS(877),
    [sym_device_info] = ACTIONS(877),
    [sym_device_id_string] = ACTIONS(877),
    [sym_device_standby] = ACTIONS(877),
    [sym_device_wake] = ACTIONS(877),
    [sym_timeline_create] = ACTIONS(877),
    [sym_timeline_kill] = ACTIONS(877),
    [sym_timeline_reload] = ACTIONS(877),
    [sym_timeline_pause] = ACTIONS(877),
    [sym_timeline_restart] = ACTIONS(877),
    [sym_timeline_set] = ACTIONS(877),
    [sym_timeline_get] = ACTIONS(877),
    [sym_timeline_active] = ACTIONS(877),
    [sym_timeline_dynamic_id] = ACTIONS(877),
    [sym_variable_to_string] = ACTIONS(877),
    [sym_string_to_variable] = ACTIONS(877),
    [sym_length_variable_to_string] = ACTIONS(877),
    [sym_type_cast] = ACTIONS(877),
    [sym_variable_to_xml] = ACTIONS(877),
    [sym_xml_to_variable] = ACTIONS(877),
    [sym_length_variable_to_xml] = ACTIONS(877),
    [sym_redirect_string] = ACTIONS(877),
    [sym_random_number] = ACTIONS(877),
    [sym_get_unique_id] = ACTIONS(877),
    [sym_get_system_number] = ACTIONS(877),
    [sym_set_system_number] = ACTIONS(877),
    [sym_get_serial_number] = ACTIONS(877),
    [sym_get_master_build] = ACTIONS(877),
    [sym_set_validation_code] = ACTIONS(877),
    [sym_reboot] = ACTIONS(877),
    [sym_load_duet_module] = ACTIONS(877),
    [sym_unload_duet_module] = ACTIONS(877),
    [sym_duet_mem_size_set] = ACTIONS(877),
    [sym_duet_mem_size_get] = ACTIONS(877),
    [sym_dynamic_application_device] = ACTIONS(877),
    [sym_dynamic_polled_port] = ACTIONS(877),
    [sym_static_port_binding] = ACTIONS(877),
    [sym_static_ip_binding] = ACTIONS(877),
    [sym_get_dns_list] = ACTIONS(877),
    [sym_set_dns_list] = ACTIONS(877),
    [sym_get_ip_address] = ACTIONS(877),
    [sym_set_ip_address] = ACTIONS(877),
    [sym_get_url_list] = ACTIONS(877),
    [sym_add_url_entry] = ACTIONS(877),
    [sym_delete_url_entry] = ACTIONS(877),
    [sym_abs_value] = ACTIONS(877),
    [sym_max_value] = ACTIONS(877),
    [sym_min_value] = ACTIONS(877),
    [sym_exp_value] = ACTIONS(877),
    [sym_log_value] = ACTIONS(877),
    [sym_log10_value] = ACTIONS(877),
    [sym_power_value] = ACTIONS(877),
    [sym_sqrt_value] = ACTIONS(877),
    [sym_raw_be] = ACTIONS(877),
    [sym_raw_le] = ACTIONS(877),
    [sym_internal_queue_size_set] = ACTIONS(877),
    [sym_internal_queue_size_get] = ACTIONS(877),
    [sym_internal_threshold_set] = ACTIONS(877),
    [sym_internal_threshold_get] = ACTIONS(877),
    [sym_rebuilt_event] = ACTIONS(877),
    [sym_do_custom_event] = ACTIONS(877),
    [sym_clkmgr_is_network_sourced] = ACTIONS(877),
    [sym_clkmgr_set_clk_source] = ACTIONS(877),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(877),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(877),
    [sym_clkmgr_get_timezone] = ACTIONS(877),
    [sym_clkmgr_set_timezone] = ACTIONS(877),
    [sym_clkmgr_get_resync_period] = ACTIONS(877),
    [sym_clkmgr_set_resync_period] = ACTIONS(877),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(877),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(877),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(877),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(877),
    [sym_clkmgr_get_timeservers] = ACTIONS(877),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(877),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(877),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(877),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(877),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(877),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(877),
    [sym_set_log_level] = ACTIONS(877),
    [sym_get_log_level] = ACTIONS(877),
    [sym_amx_log] = ACTIONS(877),
    [sym_smtp_server_config_set] = ACTIONS(877),
    [sym_smtp_server_config_get] = ACTIONS(877),
    [sym_smtp_send] = ACTIONS(877),
    [sym_validate_netlinx_account] = ACTIONS(877),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(877),
    [sym_authenticate_certificate] = ACTIONS(877),
    [sym_audit_netlinx_session_event] = ACTIONS(877),
    [sym_audit_netlinx_generic_event] = ACTIONS(877),
    [sym_get_available_flash_disk_space] = ACTIONS(877),
    [sym_get_max_flash_disk_space] = ACTIONS(877),
    [sym_data_create_feed] = ACTIONS(877),
    [sym_data_delete_feed] = ACTIONS(877),
    [sym_data_publish_feed] = ACTIONS(877),
    [sym_data_get_published_feed] = ACTIONS(877),
    [sym_data_add_record] = ACTIONS(877),
    [sym_data_get_event_record] = ACTIONS(877),
    [sym__wc_data_create_feed] = ACTIONS(877),
    [sym__wc_data_add_record] = ACTIONS(877),
    [sym__wc_data_get_event_record] = ACTIONS(877),
    [sym_format] = ACTIONS(877),
    [sym_astro_clock] = ACTIONS(877),
    [sym_set_outdoor_temperature] = ACTIONS(877),
    [sym_set_virtual_level_count] = ACTIONS(877),
    [sym_set_virtual_channel_count] = ACTIONS(877),
    [sym_set_virtual_port_count] = ACTIONS(877),
    [sym__wc_encode_int] = ACTIONS(877),
    [sym__wc_decode_int] = ACTIONS(877),
    [sym__wc_int] = ACTIONS(877),
    [sym__wc_to_ch_int] = ACTIONS(877),
    [sym__ch_to_wc_int] = ACTIONS(877),
    [sym__wc_find_string_int] = ACTIONS(877),
    [sym__wc_left_string_int] = ACTIONS(877),
    [sym__wc_lower_string_int] = ACTIONS(877),
    [sym__wc_mid_string_int] = ACTIONS(877),
    [sym__wc_remove_string_int] = ACTIONS(877),
    [sym__wc_right_string_int] = ACTIONS(877),
    [sym__wc_upper_string_int] = ACTIONS(877),
    [sym__wc_compare_string_int] = ACTIONS(877),
    [sym__wc_get_buffer_char_int] = ACTIONS(877),
    [sym__wc_get_buffer_string_int] = ACTIONS(877),
    [sym__wc_concat_string_int] = ACTIONS(877),
    [sym__wc_file_open_int] = ACTIONS(877),
    [sym__wc_file_close_int] = ACTIONS(877),
    [sym__wc_file_read_int] = ACTIONS(877),
    [sym__wc_file_read_line_int] = ACTIONS(877),
    [sym__wc_file_write_int] = ACTIONS(877),
    [sym__wc_file_write_line_int] = ACTIONS(877),
    [aux_sym_preproc_include_token1] = ACTIONS(770),
    [aux_sym_preproc_define_token1] = ACTIONS(880),
    [aux_sym_preproc_warn_token1] = ACTIONS(880),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(880),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(880),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(880),
    [anon_sym_LBRACK] = ACTIONS(882),
    [anon_sym_LBRACE] = ACTIONS(975),
    [anon_sym_LPAREN] = ACTIONS(888),
    [anon_sym_BANG] = ACTIONS(891),
    [anon_sym_TILDE] = ACTIONS(891),
    [anon_sym_DASH] = ACTIONS(862),
    [anon_sym_PLUS] = ACTIONS(862),
    [anon_sym_DASH_DASH] = ACTIONS(894),
    [anon_sym_PLUS_PLUS] = ACTIONS(894),
    [anon_sym_DQUOTE] = ACTIONS(897),
    [anon_sym_SQUOTE] = ACTIONS(900),
    [sym_number_literal] = ACTIONS(903),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(38)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(36),
    [sym_storage_class_specifier] = STATE(1028),
    [sym_type_qualifier] = STATE(1013),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(36),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1457),
    [sym_expression_statement] = STATE(36),
    [sym_if_statement] = STATE(36),
    [sym_switch_statement] = STATE(36),
    [sym_while_statement] = STATE(36),
    [sym_for_statement] = STATE(36),
    [sym_select_statement] = STATE(36),
    [sym_active_statement] = STATE(36),
    [sym_return_statement] = STATE(36),
    [sym_break_statement] = STATE(36),
    [sym_continue_statement] = STATE(36),
    [sym_devchan_operation_statement] = STATE(36),
    [sym_devchan_operation] = STATE(1878),
    [sym_send_string_statement] = STATE(36),
    [sym_send_command_statement] = STATE(36),
    [sym_send_level_statement] = STATE(36),
    [sym_create_buffer_statement] = STATE(36),
    [sym_create_multi_buffer_statement] = STATE(36),
    [sym_clear_buffer_statement] = STATE(36),
    [sym_wait_statement] = STATE(36),
    [sym_wait_until_statement] = STATE(36),
    [sym_cancel_all_wait_statement] = STATE(36),
    [sym_cancel_all_wait_until_statement] = STATE(36),
    [sym_cancel_wait_statement] = STATE(36),
    [sym_cancel_wait_until_statement] = STATE(36),
    [sym_call_statement] = STATE(36),
    [sym_system_call_statement] = STATE(36),
    [sym_expression] = STATE(1123),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1990),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(36),
    [ts_builtin_sym_end] = ACTIONS(757),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(755),
    [sym_module_name_keyword] = ACTIONS(755),
    [sym_define_device_keyword] = ACTIONS(755),
    [sym_define_combine_keyword] = ACTIONS(755),
    [sym_define_constant_keyword] = ACTIONS(755),
    [sym_define_type_keyword] = ACTIONS(755),
    [sym_define_variable_keyword] = ACTIONS(755),
    [sym_define_system_variable_keyword] = ACTIONS(755),
    [sym_define_connect_level_keyword] = ACTIONS(755),
    [sym_define_latching_keyword] = ACTIONS(755),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(755),
    [sym_define_toggling_keyword] = ACTIONS(755),
    [sym_define_call_keyword] = ACTIONS(755),
    [sym_define_function_keyword] = ACTIONS(755),
    [sym_define_library_function_keyword] = ACTIONS(755),
    [sym_define_start_keyword] = ACTIONS(755),
    [sym_define_event_keyword] = ACTIONS(755),
    [sym_define_program_keyword] = ACTIONS(755),
    [sym_define_module_keyword] = ACTIONS(755),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(55),
    [sym_else_keyword] = ACTIONS(755),
    [sym_switch_keyword] = ACTIONS(57),
    [sym_case_keyword] = ACTIONS(755),
    [sym_default_keyword] = ACTIONS(755),
    [sym_while_keyword] = ACTIONS(63),
    [sym_for_keyword] = ACTIONS(65),
    [sym_break_keyword] = ACTIONS(67),
    [sym_continue_keyword] = ACTIONS(69),
    [sym_return_keyword] = ACTIONS(71),
    [sym_select_keyword] = ACTIONS(73),
    [sym_active_keyword] = ACTIONS(75),
    [sym_struct_keyword] = ACTIONS(755),
    [sym_structure_keyword] = ACTIONS(755),
    [sym_button_event_keyword] = ACTIONS(755),
    [sym_channel_event_keyword] = ACTIONS(755),
    [sym_level_event_keyword] = ACTIONS(755),
    [sym_data_event_keyword] = ACTIONS(755),
    [sym_timeline_event_keyword] = ACTIONS(755),
    [sym_custom_event_keyword] = ACTIONS(755),
    [sym_push_keyword] = ACTIONS(755),
    [sym_release_keyword] = ACTIONS(755),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(99),
    [sym_send_string_keyword] = ACTIONS(101),
    [sym_send_command_keyword] = ACTIONS(103),
    [sym_clear_buffer_keyword] = ACTIONS(105),
    [sym_create_buffer_keyword] = ACTIONS(107),
    [sym_create_multi_buffer_keyword] = ACTIONS(109),
    [sym_call_keyword] = ACTIONS(111),
    [sym_system_call_keyword] = ACTIONS(113),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(117),
    [sym_cancel_wait_keyword] = ACTIONS(119),
    [sym_wait_until_keyword] = ACTIONS(121),
    [sym_cancel_wait_until_keyword] = ACTIONS(123),
    [sym_cancel_all_wait_keyword] = ACTIONS(125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(127),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(755),
    [aux_sym_preproc_define_token1] = ACTIONS(757),
    [aux_sym_preproc_warn_token1] = ACTIONS(757),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(757),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(757),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(757),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(39)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(37),
    [sym_storage_class_specifier] = STATE(1028),
    [sym_type_qualifier] = STATE(1013),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(37),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1457),
    [sym_expression_statement] = STATE(37),
    [sym_if_statement] = STATE(37),
    [sym_switch_statement] = STATE(37),
    [sym_while_statement] = STATE(37),
    [sym_for_statement] = STATE(37),
    [sym_select_statement] = STATE(37),
    [sym_active_statement] = STATE(37),
    [sym_return_statement] = STATE(37),
    [sym_break_statement] = STATE(37),
    [sym_continue_statement] = STATE(37),
    [sym_devchan_operation_statement] = STATE(37),
    [sym_devchan_operation] = STATE(1878),
    [sym_send_string_statement] = STATE(37),
    [sym_send_command_statement] = STATE(37),
    [sym_send_level_statement] = STATE(37),
    [sym_create_buffer_statement] = STATE(37),
    [sym_create_multi_buffer_statement] = STATE(37),
    [sym_clear_buffer_statement] = STATE(37),
    [sym_wait_statement] = STATE(37),
    [sym_wait_until_statement] = STATE(37),
    [sym_cancel_all_wait_statement] = STATE(37),
    [sym_cancel_all_wait_until_statement] = STATE(37),
    [sym_cancel_wait_statement] = STATE(37),
    [sym_cancel_wait_until_statement] = STATE(37),
    [sym_call_statement] = STATE(37),
    [sym_system_call_statement] = STATE(37),
    [sym_expression] = STATE(1137),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1990),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(37),
    [ts_builtin_sym_end] = ACTIONS(761),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(759),
    [sym_module_name_keyword] = ACTIONS(759),
    [sym_define_device_keyword] = ACTIONS(759),
    [sym_define_combine_keyword] = ACTIONS(759),
    [sym_define_constant_keyword] = ACTIONS(759),
    [sym_define_type_keyword] = ACTIONS(759),
    [sym_define_variable_keyword] = ACTIONS(759),
    [sym_define_system_variable_keyword] = ACTIONS(759),
    [sym_define_connect_level_keyword] = ACTIONS(759),
    [sym_define_latching_keyword] = ACTIONS(759),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(759),
    [sym_define_toggling_keyword] = ACTIONS(759),
    [sym_define_call_keyword] = ACTIONS(759),
    [sym_define_function_keyword] = ACTIONS(759),
    [sym_define_library_function_keyword] = ACTIONS(759),
    [sym_define_start_keyword] = ACTIONS(759),
    [sym_define_event_keyword] = ACTIONS(759),
    [sym_define_program_keyword] = ACTIONS(759),
    [sym_define_module_keyword] = ACTIONS(759),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(55),
    [sym_else_keyword] = ACTIONS(759),
    [sym_switch_keyword] = ACTIONS(57),
    [sym_case_keyword] = ACTIONS(759),
    [sym_default_keyword] = ACTIONS(759),
    [sym_while_keyword] = ACTIONS(63),
    [sym_for_keyword] = ACTIONS(65),
    [sym_break_keyword] = ACTIONS(67),
    [sym_continue_keyword] = ACTIONS(69),
    [sym_return_keyword] = ACTIONS(71),
    [sym_select_keyword] = ACTIONS(73),
    [sym_active_keyword] = ACTIONS(75),
    [sym_struct_keyword] = ACTIONS(759),
    [sym_structure_keyword] = ACTIONS(759),
    [sym_button_event_keyword] = ACTIONS(759),
    [sym_channel_event_keyword] = ACTIONS(759),
    [sym_level_event_keyword] = ACTIONS(759),
    [sym_data_event_keyword] = ACTIONS(759),
    [sym_timeline_event_keyword] = ACTIONS(759),
    [sym_custom_event_keyword] = ACTIONS(759),
    [sym_push_keyword] = ACTIONS(759),
    [sym_release_keyword] = ACTIONS(759),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(99),
    [sym_send_string_keyword] = ACTIONS(101),
    [sym_send_command_keyword] = ACTIONS(103),
    [sym_clear_buffer_keyword] = ACTIONS(105),
    [sym_create_buffer_keyword] = ACTIONS(107),
    [sym_create_multi_buffer_keyword] = ACTIONS(109),
    [sym_call_keyword] = ACTIONS(111),
    [sym_system_call_keyword] = ACTIONS(113),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(117),
    [sym_cancel_wait_keyword] = ACTIONS(119),
    [sym_wait_until_keyword] = ACTIONS(121),
    [sym_cancel_wait_until_keyword] = ACTIONS(123),
    [sym_cancel_all_wait_keyword] = ACTIONS(125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(127),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(759),
    [aux_sym_preproc_define_token1] = ACTIONS(761),
    [aux_sym_preproc_warn_token1] = ACTIONS(761),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(761),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(761),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(761),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(40)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(39),
    [sym_storage_class_specifier] = STATE(1028),
    [sym_type_qualifier] = STATE(1013),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(39),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1457),
    [sym_expression_statement] = STATE(39),
    [sym_if_statement] = STATE(39),
    [sym_switch_statement] = STATE(39),
    [sym_while_statement] = STATE(39),
    [sym_for_statement] = STATE(39),
    [sym_select_statement] = STATE(39),
    [sym_active_statement] = STATE(39),
    [sym_return_statement] = STATE(39),
    [sym_break_statement] = STATE(39),
    [sym_continue_statement] = STATE(39),
    [sym_devchan_operation_statement] = STATE(39),
    [sym_devchan_operation] = STATE(1878),
    [sym_send_string_statement] = STATE(39),
    [sym_send_command_statement] = STATE(39),
    [sym_send_level_statement] = STATE(39),
    [sym_create_buffer_statement] = STATE(39),
    [sym_create_multi_buffer_statement] = STATE(39),
    [sym_clear_buffer_statement] = STATE(39),
    [sym_wait_statement] = STATE(39),
    [sym_wait_until_statement] = STATE(39),
    [sym_cancel_all_wait_statement] = STATE(39),
    [sym_cancel_all_wait_until_statement] = STATE(39),
    [sym_cancel_wait_statement] = STATE(39),
    [sym_cancel_wait_until_statement] = STATE(39),
    [sym_call_statement] = STATE(39),
    [sym_system_call_statement] = STATE(39),
    [sym_expression] = STATE(1137),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1990),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(39),
    [ts_builtin_sym_end] = ACTIONS(753),
    [sym_identifier] = ACTIONS(7),
    [sym_program_name_keyword] = ACTIONS(751),
    [sym_module_name_keyword] = ACTIONS(751),
    [sym_define_device_keyword] = ACTIONS(751),
    [sym_define_combine_keyword] = ACTIONS(751),
    [sym_define_constant_keyword] = ACTIONS(751),
    [sym_define_type_keyword] = ACTIONS(751),
    [sym_define_variable_keyword] = ACTIONS(751),
    [sym_define_system_variable_keyword] = ACTIONS(751),
    [sym_define_connect_level_keyword] = ACTIONS(751),
    [sym_define_latching_keyword] = ACTIONS(751),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(751),
    [sym_define_toggling_keyword] = ACTIONS(751),
    [sym_define_call_keyword] = ACTIONS(751),
    [sym_define_function_keyword] = ACTIONS(751),
    [sym_define_library_function_keyword] = ACTIONS(751),
    [sym_define_start_keyword] = ACTIONS(751),
    [sym_define_event_keyword] = ACTIONS(751),
    [sym_define_program_keyword] = ACTIONS(751),
    [sym_define_module_keyword] = ACTIONS(751),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(55),
    [sym_else_keyword] = ACTIONS(751),
    [sym_switch_keyword] = ACTIONS(57),
    [sym_case_keyword] = ACTIONS(751),
    [sym_default_keyword] = ACTIONS(751),
    [sym_while_keyword] = ACTIONS(63),
    [sym_for_keyword] = ACTIONS(65),
    [sym_break_keyword] = ACTIONS(67),
    [sym_continue_keyword] = ACTIONS(69),
    [sym_return_keyword] = ACTIONS(71),
    [sym_select_keyword] = ACTIONS(73),
    [sym_active_keyword] = ACTIONS(75),
    [sym_struct_keyword] = ACTIONS(751),
    [sym_structure_keyword] = ACTIONS(751),
    [sym_button_event_keyword] = ACTIONS(751),
    [sym_channel_event_keyword] = ACTIONS(751),
    [sym_level_event_keyword] = ACTIONS(751),
    [sym_data_event_keyword] = ACTIONS(751),
    [sym_timeline_event_keyword] = ACTIONS(751),
    [sym_custom_event_keyword] = ACTIONS(751),
    [sym_push_keyword] = ACTIONS(751),
    [sym_release_keyword] = ACTIONS(751),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(99),
    [sym_send_string_keyword] = ACTIONS(101),
    [sym_send_command_keyword] = ACTIONS(103),
    [sym_clear_buffer_keyword] = ACTIONS(105),
    [sym_create_buffer_keyword] = ACTIONS(107),
    [sym_create_multi_buffer_keyword] = ACTIONS(109),
    [sym_call_keyword] = ACTIONS(111),
    [sym_system_call_keyword] = ACTIONS(113),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(117),
    [sym_cancel_wait_keyword] = ACTIONS(119),
    [sym_wait_until_keyword] = ACTIONS(121),
    [sym_cancel_wait_until_keyword] = ACTIONS(123),
    [sym_cancel_all_wait_keyword] = ACTIONS(125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(127),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(751),
    [aux_sym_preproc_define_token1] = ACTIONS(753),
    [aux_sym_preproc_warn_token1] = ACTIONS(753),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(753),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(753),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(753),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(41)] = {
    [sym__block_item] = STATE(45),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_preproc_include] = STATE(45),
    [sym_preproc_define] = STATE(45),
    [sym_preproc_warn] = STATE(45),
    [sym_preproc_disable_warning] = STATE(45),
    [sym_preproc_if_defined] = STATE(45),
    [sym_preproc_if_not_defined] = STATE(45),
    [sym_define_function] = STATE(45),
    [sym_define_call] = STATE(45),
    [sym_define_module] = STATE(45),
    [sym_compound_statement] = STATE(494),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(1912),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(45),
    [sym_type_definition] = STATE(45),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_statement] = STATE(45),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_compound_statement_repeat1] = STATE(45),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_define_call_keyword] = ACTIONS(978),
    [sym_define_function_keyword] = ACTIONS(980),
    [sym_define_library_function_keyword] = ACTIONS(982),
    [sym_define_module_keyword] = ACTIONS(984),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(986),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(990),
    [sym_default_keyword] = ACTIONS(992),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(1036),
    [aux_sym_preproc_define_token1] = ACTIONS(1038),
    [aux_sym_preproc_warn_token1] = ACTIONS(1040),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1042),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1044),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1046),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(1050),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(42)] = {
    [sym__block_item] = STATE(45),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_preproc_include] = STATE(45),
    [sym_preproc_define] = STATE(45),
    [sym_preproc_warn] = STATE(45),
    [sym_preproc_disable_warning] = STATE(45),
    [sym_preproc_if_defined] = STATE(45),
    [sym_preproc_if_not_defined] = STATE(45),
    [sym_define_function] = STATE(45),
    [sym_define_call] = STATE(45),
    [sym_define_module] = STATE(45),
    [sym_compound_statement] = STATE(494),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(1912),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(45),
    [sym_type_definition] = STATE(45),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_statement] = STATE(45),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_compound_statement_repeat1] = STATE(45),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_define_call_keyword] = ACTIONS(978),
    [sym_define_function_keyword] = ACTIONS(980),
    [sym_define_library_function_keyword] = ACTIONS(982),
    [sym_define_module_keyword] = ACTIONS(984),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(986),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(990),
    [sym_default_keyword] = ACTIONS(992),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(1036),
    [aux_sym_preproc_define_token1] = ACTIONS(1038),
    [aux_sym_preproc_warn_token1] = ACTIONS(1040),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1042),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1044),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1046),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(1052),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(43)] = {
    [sym__block_item] = STATE(45),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_preproc_include] = STATE(45),
    [sym_preproc_define] = STATE(45),
    [sym_preproc_warn] = STATE(45),
    [sym_preproc_disable_warning] = STATE(45),
    [sym_preproc_if_defined] = STATE(45),
    [sym_preproc_if_not_defined] = STATE(45),
    [sym_define_function] = STATE(45),
    [sym_define_call] = STATE(45),
    [sym_define_module] = STATE(45),
    [sym_compound_statement] = STATE(494),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(1912),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(45),
    [sym_type_definition] = STATE(45),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_statement] = STATE(45),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_compound_statement_repeat1] = STATE(45),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_define_call_keyword] = ACTIONS(978),
    [sym_define_function_keyword] = ACTIONS(980),
    [sym_define_library_function_keyword] = ACTIONS(982),
    [sym_define_module_keyword] = ACTIONS(984),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(986),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(990),
    [sym_default_keyword] = ACTIONS(992),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(1036),
    [aux_sym_preproc_define_token1] = ACTIONS(1038),
    [aux_sym_preproc_warn_token1] = ACTIONS(1040),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1042),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1044),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1046),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(1054),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(44)] = {
    [sym__block_item] = STATE(43),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_preproc_include] = STATE(43),
    [sym_preproc_define] = STATE(43),
    [sym_preproc_warn] = STATE(43),
    [sym_preproc_disable_warning] = STATE(43),
    [sym_preproc_if_defined] = STATE(43),
    [sym_preproc_if_not_defined] = STATE(43),
    [sym_define_function] = STATE(43),
    [sym_define_call] = STATE(43),
    [sym_define_module] = STATE(43),
    [sym_compound_statement] = STATE(494),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(1912),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(43),
    [sym_type_definition] = STATE(43),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_statement] = STATE(43),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_compound_statement_repeat1] = STATE(43),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_define_call_keyword] = ACTIONS(978),
    [sym_define_function_keyword] = ACTIONS(980),
    [sym_define_library_function_keyword] = ACTIONS(982),
    [sym_define_module_keyword] = ACTIONS(984),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(986),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(990),
    [sym_default_keyword] = ACTIONS(992),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(1036),
    [aux_sym_preproc_define_token1] = ACTIONS(1038),
    [aux_sym_preproc_warn_token1] = ACTIONS(1040),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1042),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1044),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1046),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(1056),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(45)] = {
    [sym__block_item] = STATE(45),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_preproc_include] = STATE(45),
    [sym_preproc_define] = STATE(45),
    [sym_preproc_warn] = STATE(45),
    [sym_preproc_disable_warning] = STATE(45),
    [sym_preproc_if_defined] = STATE(45),
    [sym_preproc_if_not_defined] = STATE(45),
    [sym_define_function] = STATE(45),
    [sym_define_call] = STATE(45),
    [sym_define_module] = STATE(45),
    [sym_compound_statement] = STATE(494),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(1912),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(45),
    [sym_type_definition] = STATE(45),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_statement] = STATE(45),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_compound_statement_repeat1] = STATE(45),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(1058),
    [sym_define_call_keyword] = ACTIONS(1061),
    [sym_define_function_keyword] = ACTIONS(1064),
    [sym_define_library_function_keyword] = ACTIONS(1067),
    [sym_define_module_keyword] = ACTIONS(1070),
    [sym_char_keyword] = ACTIONS(1073),
    [sym_widechar_keyword] = ACTIONS(1073),
    [sym_integer_keyword] = ACTIONS(1073),
    [sym_sinteger_keyword] = ACTIONS(1073),
    [sym_long_keyword] = ACTIONS(1073),
    [sym_slong_keyword] = ACTIONS(1073),
    [sym_float_keyword] = ACTIONS(1073),
    [sym_double_keyword] = ACTIONS(1073),
    [sym_variant_keyword] = ACTIONS(1073),
    [sym_variantarray_keyword] = ACTIONS(1073),
    [sym_local_var_keyword] = ACTIONS(1076),
    [sym_stack_var_keyword] = ACTIONS(1076),
    [sym_constant_keyword] = ACTIONS(1079),
    [sym_volatile_keyword] = ACTIONS(1079),
    [sym_non_volatile_keyword] = ACTIONS(1079),
    [sym_persistent_keyword] = ACTIONS(1079),
    [sym_dev_keyword] = ACTIONS(1082),
    [sym_devlev_keyword] = ACTIONS(1082),
    [sym_devchan_keyword] = ACTIONS(1082),
    [sym_if_keyword] = ACTIONS(1085),
    [sym_switch_keyword] = ACTIONS(1088),
    [sym_case_keyword] = ACTIONS(1091),
    [sym_default_keyword] = ACTIONS(1094),
    [sym_while_keyword] = ACTIONS(1097),
    [sym_for_keyword] = ACTIONS(1100),
    [sym_break_keyword] = ACTIONS(1103),
    [sym_continue_keyword] = ACTIONS(1106),
    [sym_return_keyword] = ACTIONS(1109),
    [sym_select_keyword] = ACTIONS(1112),
    [sym_active_keyword] = ACTIONS(1115),
    [sym_struct_keyword] = ACTIONS(1118),
    [sym_structure_keyword] = ACTIONS(1118),
    [aux_sym_on_keyword_token1] = ACTIONS(1121),
    [aux_sym_off_keyword_token1] = ACTIONS(1124),
    [sym_send_level_keyword] = ACTIONS(1127),
    [sym_send_string_keyword] = ACTIONS(1130),
    [sym_send_command_keyword] = ACTIONS(1133),
    [sym_clear_buffer_keyword] = ACTIONS(1136),
    [sym_create_buffer_keyword] = ACTIONS(1139),
    [sym_create_multi_buffer_keyword] = ACTIONS(1142),
    [sym_call_keyword] = ACTIONS(1145),
    [sym_system_call_keyword] = ACTIONS(1148),
    [sym_devchan_to_keyword] = ACTIONS(1151),
    [sym_devchan_min_to_keyword] = ACTIONS(1151),
    [sym_devchan_total_off_keyword] = ACTIONS(1151),
    [sym_devchan_pulse_keyword] = ACTIONS(1151),
    [sym_wait_keyword] = ACTIONS(1154),
    [sym_cancel_wait_keyword] = ACTIONS(1157),
    [sym_wait_until_keyword] = ACTIONS(1160),
    [sym_cancel_wait_until_keyword] = ACTIONS(1163),
    [sym_cancel_all_wait_keyword] = ACTIONS(1166),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1169),
    [sym_not] = ACTIONS(1172),
    [sym_bnot] = ACTIONS(1172),
    [sym___file__] = ACTIONS(1175),
    [sym___date__] = ACTIONS(1175),
    [sym___ldate__] = ACTIONS(1175),
    [sym___time__] = ACTIONS(1175),
    [sym___line__] = ACTIONS(1175),
    [sym___name__] = ACTIONS(1175),
    [sym___netlinx__] = ACTIONS(1178),
    [sym_netlinx_axi_version] = ACTIONS(1178),
    [sym_url_flg_tcp] = ACTIONS(1178),
    [sym_url_flg_accinfopresent] = ACTIONS(1178),
    [sym_url_flg_temp] = ACTIONS(1178),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1178),
    [sym_url_flg_stat_mask] = ACTIONS(1178),
    [sym_url_flg_stat_lookup] = ACTIONS(1178),
    [sym_url_flg_stat_connecting] = ACTIONS(1178),
    [sym_url_flg_stat_waiting] = ACTIONS(1178),
    [sym_url_flg_stat_connected] = ACTIONS(1178),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1178),
    [sym_first_virtual_device] = ACTIONS(1178),
    [sym_first_local_port] = ACTIONS(1178),
    [sym_dynamic_virtual_device] = ACTIONS(1178),
    [sym_timeline_once] = ACTIONS(1178),
    [sym_timeline_repeat] = ACTIONS(1178),
    [sym_timeline_absolute] = ACTIONS(1178),
    [sym_timeline_relative] = ACTIONS(1178),
    [sym_do_push_timed_infinite] = ACTIONS(1178),
    [sym_source_type_no_address] = ACTIONS(1178),
    [sym_source_type_neuron_id] = ACTIONS(1178),
    [sym_source_type_ip_address] = ACTIONS(1178),
    [sym_source_type_axlink] = ACTIONS(1178),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1178),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1178),
    [sym_source_type_ip_socket_address] = ACTIONS(1178),
    [sym_source_type_rs232] = ACTIONS(1178),
    [sym_source_type_internal] = ACTIONS(1178),
    [sym_source_type_mac_address] = ACTIONS(1178),
    [sym_source_type_ipv4_port] = ACTIONS(1178),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1178),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1178),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1178),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1178),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1178),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1178),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1178),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1178),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1178),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1178),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1178),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1178),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1178),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1178),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1178),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1178),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1178),
    [sym_true] = ACTIONS(1178),
    [sym_false] = ACTIONS(1178),
    [sym_file_read_only] = ACTIONS(1178),
    [sym_file_rw_new] = ACTIONS(1178),
    [sym_file_rw_append] = ACTIONS(1178),
    [sym_ip_tcp] = ACTIONS(1178),
    [sym_ip_udp] = ACTIONS(1178),
    [sym_ip_udp_2way] = ACTIONS(1178),
    [sym_xml_encode_types] = ACTIONS(1178),
    [sym_xml_encode_char_as_list] = ACTIONS(1178),
    [sym_xml_encode_le] = ACTIONS(1178),
    [sym_xml_decode_types] = ACTIONS(1178),
    [sym_xml_decode_no_preserve] = ACTIONS(1178),
    [sym_duet_dev_type_display_device] = ACTIONS(1178),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1178),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1178),
    [sym_duet_dev_type_io_device] = ACTIONS(1178),
    [sym_duet_dev_type_relay_device] = ACTIONS(1178),
    [sym_duet_dev_type_ups] = ACTIONS(1178),
    [sym_duet_dev_type_amplifier] = ACTIONS(1178),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1178),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1178),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1178),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1178),
    [sym_duet_dev_type_camera] = ACTIONS(1178),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1178),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1178),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1178),
    [sym_duet_dev_type_dss] = ACTIONS(1178),
    [sym_duet_dev_type_dvr] = ACTIONS(1178),
    [sym_duet_dev_type_disc_device] = ACTIONS(1178),
    [sym_duet_dev_type_document_camera] = ACTIONS(1178),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1178),
    [sym_duet_dev_type_hvac] = ACTIONS(1178),
    [sym_duet_dev_type_keypad] = ACTIONS(1178),
    [sym_duet_dev_type_light] = ACTIONS(1178),
    [sym_duet_dev_type_monitor] = ACTIONS(1178),
    [sym_duet_dev_type_motor] = ACTIONS(1178),
    [sym_duet_dev_type_multi_window] = ACTIONS(1178),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1178),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1178),
    [sym_duet_dev_type_receiver] = ACTIONS(1178),
    [sym_duet_dev_type_security_system] = ACTIONS(1178),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1178),
    [sym_duet_dev_type_settop_box] = ACTIONS(1178),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1178),
    [sym_duet_dev_type_switcher] = ACTIONS(1178),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1178),
    [sym_duet_dev_type_tv] = ACTIONS(1178),
    [sym_duet_dev_type_utility] = ACTIONS(1178),
    [sym_duet_dev_type_vcr] = ACTIONS(1178),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1178),
    [sym_duet_dev_type_video_processor] = ACTIONS(1178),
    [sym_duet_dev_type_video_projector] = ACTIONS(1178),
    [sym_duet_dev_type_video_wall] = ACTIONS(1178),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1178),
    [sym_duet_dev_type_weather] = ACTIONS(1178),
    [sym_duet_dev_not_polled] = ACTIONS(1178),
    [sym_duet_dev_polled] = ACTIONS(1178),
    [sym_clkmgr_mode_network] = ACTIONS(1178),
    [sym_clkmgr_mode_standalone] = ACTIONS(1178),
    [sym_normal_standby] = ACTIONS(1178),
    [sym_normal_wake] = ACTIONS(1178),
    [sym_ip_multicast_ttl_option] = ACTIONS(1178),
    [sym_ip_ttl_subnet] = ACTIONS(1178),
    [sym_ip_ttl_site] = ACTIONS(1178),
    [sym_ip_ttl_region] = ACTIONS(1178),
    [sym_ip_ttl_continent] = ACTIONS(1178),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1178),
    [sym_ip_nodelay_on] = ACTIONS(1178),
    [sym_ip_nodelay_off] = ACTIONS(1178),
    [sym_amx_error] = ACTIONS(1178),
    [sym_amx_warning] = ACTIONS(1178),
    [sym_amx_info] = ACTIONS(1178),
    [sym_amx_debug] = ACTIONS(1178),
    [sym_netlinx_login_success] = ACTIONS(1178),
    [sym_netlinx_login_fail] = ACTIONS(1178),
    [sym_netlinx_logout] = ACTIONS(1178),
    [sym_null_str] = ACTIONS(1178),
    [sym_smtp_address] = ACTIONS(1178),
    [sym_smtp_port_number] = ACTIONS(1178),
    [sym_smtp_username] = ACTIONS(1178),
    [sym_smtp_password] = ACTIONS(1178),
    [sym_smtp_require_tls] = ACTIONS(1178),
    [sym_smtp_from] = ACTIONS(1178),
    [sym_smtp_tls_true] = ACTIONS(1178),
    [sym_smtp_tls_false] = ACTIONS(1178),
    [sym_listview_on_row_select_event] = ACTIONS(1178),
    [sym_data_structure_unknown] = ACTIONS(1178),
    [sym_data_structure_datafeed] = ACTIONS(1178),
    [sym_data_structure_datarecordset] = ACTIONS(1178),
    [sym_data_structure_datarecord] = ACTIONS(1178),
    [sym_data_structure_datafield] = ACTIONS(1178),
    [sym_data_max_content_field_count] = ACTIONS(1178),
    [sym_data_max_metadata_field_count] = ACTIONS(1178),
    [sym_data_max_id_length] = ACTIONS(1178),
    [sym_data_max_name_length] = ACTIONS(1178),
    [sym_data_max_description_length] = ACTIONS(1178),
    [sym_data_max_type_length] = ACTIONS(1178),
    [sym_data_max_format_length] = ACTIONS(1178),
    [sym_data_max_source_length] = ACTIONS(1178),
    [sym_data_max_value_length] = ACTIONS(1178),
    [sym_data_max_label_length] = ACTIONS(1178),
    [sym_data_max_url_length] = ACTIONS(1178),
    [sym_max_last_login_info_length] = ACTIONS(1178),
    [sym_data_type_unknown] = ACTIONS(1178),
    [sym_data_type_string] = ACTIONS(1178),
    [sym_data_type_datetime] = ACTIONS(1178),
    [sym_data_type_date] = ACTIONS(1178),
    [sym_data_type_time] = ACTIONS(1178),
    [sym_data_type_image] = ACTIONS(1178),
    [sym_data_format_url] = ACTIONS(1178),
    [sym_data_format_phone] = ACTIONS(1178),
    [sym_data_format_email] = ACTIONS(1178),
    [sym_data_format_iso8601] = ACTIONS(1178),
    [sym_auditlog_priv] = ACTIONS(1178),
    [sym_configuration_priv] = ACTIONS(1178),
    [sym_deviceconfig_priv] = ACTIONS(1178),
    [sym_ftp_priv] = ACTIONS(1178),
    [sym_http_priv] = ACTIONS(1178),
    [sym_networkconfig_priv] = ACTIONS(1178),
    [sym_programport_priv] = ACTIONS(1178),
    [sym_remoteui_priv] = ACTIONS(1178),
    [sym_securitycontrol_priv] = ACTIONS(1178),
    [sym_softwaremanagement_priv] = ACTIONS(1178),
    [sym_terminal_priv] = ACTIONS(1178),
    [sym_tpadmin_priv] = ACTIONS(1178),
    [sym_user1_priv] = ACTIONS(1178),
    [sym_user2_priv] = ACTIONS(1178),
    [sym_user3_priv] = ACTIONS(1178),
    [sym_user4_priv] = ACTIONS(1178),
    [sym_usermanagement_priv] = ACTIONS(1178),
    [sym_valid_account] = ACTIONS(1178),
    [sym_err_name_invalid] = ACTIONS(1178),
    [sym_err_password_invalid] = ACTIONS(1178),
    [sym_err_invalid_account] = ACTIONS(1178),
    [sym_err_no_authorization] = ACTIONS(1178),
    [sym_err_invalid_parameter] = ACTIONS(1178),
    [sym_err_user_locked_out] = ACTIONS(1178),
    [sym_err_user_account_expired] = ACTIONS(1178),
    [sym_err_general_error] = ACTIONS(1178),
    [sym_err_authentication_error] = ACTIONS(1178),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1178),
    [sym_tls_validate_certificate] = ACTIONS(1178),
    [sym_tdata] = ACTIONS(1181),
    [sym_tchannel] = ACTIONS(1181),
    [sym_tlevel] = ACTIONS(1181),
    [sym_tbutton] = ACTIONS(1181),
    [sym_ttimeline] = ACTIONS(1181),
    [sym_tcustom] = ACTIONS(1181),
    [sym_url_struct] = ACTIONS(1181),
    [sym_dns_struct] = ACTIONS(1181),
    [sym_ip_address_struct] = ACTIONS(1181),
    [sym_dev_info_struct] = ACTIONS(1181),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1181),
    [sym_data_feed] = ACTIONS(1181),
    [sym_data_field] = ACTIONS(1181),
    [sym_data_record] = ACTIONS(1181),
    [sym_wc_data_feed] = ACTIONS(1181),
    [sym_wc_data_field] = ACTIONS(1181),
    [sym_wc_data_record] = ACTIONS(1181),
    [sym_last_login_info] = ACTIONS(1181),
    [sym____reserved___] = ACTIONS(1184),
    [sym_date] = ACTIONS(1184),
    [sym_day] = ACTIONS(1184),
    [sym_time] = ACTIONS(1184),
    [sym_ldate] = ACTIONS(1184),
    [sym_push_device] = ACTIONS(1184),
    [sym_push_channel] = ACTIONS(1184),
    [sym_push_devchan] = ACTIONS(1184),
    [sym_release_device] = ACTIONS(1184),
    [sym_release_channel] = ACTIONS(1184),
    [sym_release_devchan] = ACTIONS(1184),
    [sym_master_slot] = ACTIONS(1184),
    [sym_get_pulse_time] = ACTIONS(1184),
    [sym_get_timer] = ACTIONS(1184),
    [sym_master_sn] = ACTIONS(1184),
    [sym_system_number] = ACTIONS(1184),
    [sym_button] = ACTIONS(1184),
    [sym_dv_channel] = ACTIONS(1184),
    [sym_data] = ACTIONS(1184),
    [sym_level] = ACTIONS(1184),
    [sym_timeline] = ACTIONS(1184),
    [sym_channel] = ACTIONS(1184),
    [sym_custom] = ACTIONS(1184),
    [sym_atoi] = ACTIONS(1187),
    [sym_atol] = ACTIONS(1187),
    [sym_atof] = ACTIONS(1187),
    [sym_itoa] = ACTIONS(1187),
    [sym_itohex] = ACTIONS(1187),
    [sym_hextoi] = ACTIONS(1187),
    [sym_ftoa] = ACTIONS(1187),
    [sym_find_string] = ACTIONS(1187),
    [sym_left_string] = ACTIONS(1187),
    [sym_right_string] = ACTIONS(1187),
    [sym_mid_string] = ACTIONS(1187),
    [sym_remove_string] = ACTIONS(1187),
    [sym_get_buffer_string] = ACTIONS(1187),
    [sym_get_buffer_char] = ACTIONS(1187),
    [sym_get_multi_buffer_string] = ACTIONS(1187),
    [sym_length_string] = ACTIONS(1187),
    [sym_max_length_string] = ACTIONS(1187),
    [sym_set_length_string] = ACTIONS(1187),
    [sym_compare_string] = ACTIONS(1187),
    [sym_upper_string] = ACTIONS(1187),
    [sym_lower_string] = ACTIONS(1187),
    [sym_get_last] = ACTIONS(1187),
    [sym_date_to_day] = ACTIONS(1187),
    [sym_date_to_month] = ACTIONS(1187),
    [sym_date_to_year] = ACTIONS(1187),
    [sym_time_to_hour] = ACTIONS(1187),
    [sym_time_to_minute] = ACTIONS(1187),
    [sym_time_to_second] = ACTIONS(1187),
    [sym_day_of_week] = ACTIONS(1187),
    [sym_set_pulse_time] = ACTIONS(1187),
    [sym_set_timer] = ACTIONS(1187),
    [sym_file_open] = ACTIONS(1187),
    [sym_file_close] = ACTIONS(1187),
    [sym_file_read] = ACTIONS(1187),
    [sym_file_read_line] = ACTIONS(1187),
    [sym_file_write] = ACTIONS(1187),
    [sym_file_write_line] = ACTIONS(1187),
    [sym_file_copy] = ACTIONS(1187),
    [sym_file_delete] = ACTIONS(1187),
    [sym_file_rename] = ACTIONS(1187),
    [sym_file_dir] = ACTIONS(1187),
    [sym_file_getdir] = ACTIONS(1187),
    [sym_file_setdir] = ACTIONS(1187),
    [sym_file_create_dir] = ACTIONS(1187),
    [sym_file_remove_dir] = ACTIONS(1187),
    [sym_file_seek] = ACTIONS(1187),
    [sym_length_array] = ACTIONS(1187),
    [sym_max_length_array] = ACTIONS(1187),
    [sym_set_length_array] = ACTIONS(1187),
    [sym_ip_client_open] = ACTIONS(1187),
    [sym_ip_client_close] = ACTIONS(1187),
    [sym_ip_server_open] = ACTIONS(1187),
    [sym_ip_server_close] = ACTIONS(1187),
    [sym_ip_bound_client_open] = ACTIONS(1187),
    [sym_ip_mc_server_open] = ACTIONS(1187),
    [sym_ip_set_option] = ACTIONS(1187),
    [sym_ssh_client_open] = ACTIONS(1187),
    [sym_ssh_client_close] = ACTIONS(1187),
    [sym_tls_client_open] = ACTIONS(1187),
    [sym_tls_client_close] = ACTIONS(1187),
    [sym_do_push] = ACTIONS(1187),
    [sym_do_push_timed] = ACTIONS(1187),
    [sym_do_release] = ACTIONS(1187),
    [sym_device_id] = ACTIONS(1187),
    [sym_device_info] = ACTIONS(1187),
    [sym_device_id_string] = ACTIONS(1187),
    [sym_device_standby] = ACTIONS(1187),
    [sym_device_wake] = ACTIONS(1187),
    [sym_timeline_create] = ACTIONS(1187),
    [sym_timeline_kill] = ACTIONS(1187),
    [sym_timeline_reload] = ACTIONS(1187),
    [sym_timeline_pause] = ACTIONS(1187),
    [sym_timeline_restart] = ACTIONS(1187),
    [sym_timeline_set] = ACTIONS(1187),
    [sym_timeline_get] = ACTIONS(1187),
    [sym_timeline_active] = ACTIONS(1187),
    [sym_timeline_dynamic_id] = ACTIONS(1187),
    [sym_variable_to_string] = ACTIONS(1187),
    [sym_string_to_variable] = ACTIONS(1187),
    [sym_length_variable_to_string] = ACTIONS(1187),
    [sym_type_cast] = ACTIONS(1187),
    [sym_variable_to_xml] = ACTIONS(1187),
    [sym_xml_to_variable] = ACTIONS(1187),
    [sym_length_variable_to_xml] = ACTIONS(1187),
    [sym_redirect_string] = ACTIONS(1187),
    [sym_random_number] = ACTIONS(1187),
    [sym_get_unique_id] = ACTIONS(1187),
    [sym_get_system_number] = ACTIONS(1187),
    [sym_set_system_number] = ACTIONS(1187),
    [sym_get_serial_number] = ACTIONS(1187),
    [sym_get_master_build] = ACTIONS(1187),
    [sym_set_validation_code] = ACTIONS(1187),
    [sym_reboot] = ACTIONS(1187),
    [sym_load_duet_module] = ACTIONS(1187),
    [sym_unload_duet_module] = ACTIONS(1187),
    [sym_duet_mem_size_set] = ACTIONS(1187),
    [sym_duet_mem_size_get] = ACTIONS(1187),
    [sym_dynamic_application_device] = ACTIONS(1187),
    [sym_dynamic_polled_port] = ACTIONS(1187),
    [sym_static_port_binding] = ACTIONS(1187),
    [sym_static_ip_binding] = ACTIONS(1187),
    [sym_get_dns_list] = ACTIONS(1187),
    [sym_set_dns_list] = ACTIONS(1187),
    [sym_get_ip_address] = ACTIONS(1187),
    [sym_set_ip_address] = ACTIONS(1187),
    [sym_get_url_list] = ACTIONS(1187),
    [sym_add_url_entry] = ACTIONS(1187),
    [sym_delete_url_entry] = ACTIONS(1187),
    [sym_abs_value] = ACTIONS(1187),
    [sym_max_value] = ACTIONS(1187),
    [sym_min_value] = ACTIONS(1187),
    [sym_exp_value] = ACTIONS(1187),
    [sym_log_value] = ACTIONS(1187),
    [sym_log10_value] = ACTIONS(1187),
    [sym_power_value] = ACTIONS(1187),
    [sym_sqrt_value] = ACTIONS(1187),
    [sym_raw_be] = ACTIONS(1187),
    [sym_raw_le] = ACTIONS(1187),
    [sym_internal_queue_size_set] = ACTIONS(1187),
    [sym_internal_queue_size_get] = ACTIONS(1187),
    [sym_internal_threshold_set] = ACTIONS(1187),
    [sym_internal_threshold_get] = ACTIONS(1187),
    [sym_rebuilt_event] = ACTIONS(1187),
    [sym_do_custom_event] = ACTIONS(1187),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1187),
    [sym_clkmgr_set_clk_source] = ACTIONS(1187),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1187),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1187),
    [sym_clkmgr_get_timezone] = ACTIONS(1187),
    [sym_clkmgr_set_timezone] = ACTIONS(1187),
    [sym_clkmgr_get_resync_period] = ACTIONS(1187),
    [sym_clkmgr_set_resync_period] = ACTIONS(1187),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1187),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1187),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1187),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1187),
    [sym_clkmgr_get_timeservers] = ACTIONS(1187),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1187),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1187),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1187),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1187),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1187),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1187),
    [sym_set_log_level] = ACTIONS(1187),
    [sym_get_log_level] = ACTIONS(1187),
    [sym_amx_log] = ACTIONS(1187),
    [sym_smtp_server_config_set] = ACTIONS(1187),
    [sym_smtp_server_config_get] = ACTIONS(1187),
    [sym_smtp_send] = ACTIONS(1187),
    [sym_validate_netlinx_account] = ACTIONS(1187),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1187),
    [sym_authenticate_certificate] = ACTIONS(1187),
    [sym_audit_netlinx_session_event] = ACTIONS(1187),
    [sym_audit_netlinx_generic_event] = ACTIONS(1187),
    [sym_get_available_flash_disk_space] = ACTIONS(1187),
    [sym_get_max_flash_disk_space] = ACTIONS(1187),
    [sym_data_create_feed] = ACTIONS(1187),
    [sym_data_delete_feed] = ACTIONS(1187),
    [sym_data_publish_feed] = ACTIONS(1187),
    [sym_data_get_published_feed] = ACTIONS(1187),
    [sym_data_add_record] = ACTIONS(1187),
    [sym_data_get_event_record] = ACTIONS(1187),
    [sym__wc_data_create_feed] = ACTIONS(1187),
    [sym__wc_data_add_record] = ACTIONS(1187),
    [sym__wc_data_get_event_record] = ACTIONS(1187),
    [sym_format] = ACTIONS(1187),
    [sym_astro_clock] = ACTIONS(1187),
    [sym_set_outdoor_temperature] = ACTIONS(1187),
    [sym_set_virtual_level_count] = ACTIONS(1187),
    [sym_set_virtual_channel_count] = ACTIONS(1187),
    [sym_set_virtual_port_count] = ACTIONS(1187),
    [sym__wc_encode_int] = ACTIONS(1187),
    [sym__wc_decode_int] = ACTIONS(1187),
    [sym__wc_int] = ACTIONS(1187),
    [sym__wc_to_ch_int] = ACTIONS(1187),
    [sym__ch_to_wc_int] = ACTIONS(1187),
    [sym__wc_find_string_int] = ACTIONS(1187),
    [sym__wc_left_string_int] = ACTIONS(1187),
    [sym__wc_lower_string_int] = ACTIONS(1187),
    [sym__wc_mid_string_int] = ACTIONS(1187),
    [sym__wc_remove_string_int] = ACTIONS(1187),
    [sym__wc_right_string_int] = ACTIONS(1187),
    [sym__wc_upper_string_int] = ACTIONS(1187),
    [sym__wc_compare_string_int] = ACTIONS(1187),
    [sym__wc_get_buffer_char_int] = ACTIONS(1187),
    [sym__wc_get_buffer_string_int] = ACTIONS(1187),
    [sym__wc_concat_string_int] = ACTIONS(1187),
    [sym__wc_file_open_int] = ACTIONS(1187),
    [sym__wc_file_close_int] = ACTIONS(1187),
    [sym__wc_file_read_int] = ACTIONS(1187),
    [sym__wc_file_read_line_int] = ACTIONS(1187),
    [sym__wc_file_write_int] = ACTIONS(1187),
    [sym__wc_file_write_line_int] = ACTIONS(1187),
    [aux_sym_preproc_include_token1] = ACTIONS(1190),
    [aux_sym_preproc_define_token1] = ACTIONS(1193),
    [aux_sym_preproc_warn_token1] = ACTIONS(1196),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1199),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1202),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1205),
    [anon_sym_LBRACK] = ACTIONS(1208),
    [anon_sym_LBRACE] = ACTIONS(1211),
    [anon_sym_RBRACE] = ACTIONS(1214),
    [anon_sym_LPAREN] = ACTIONS(1216),
    [anon_sym_BANG] = ACTIONS(1219),
    [anon_sym_TILDE] = ACTIONS(1219),
    [anon_sym_DASH] = ACTIONS(1172),
    [anon_sym_PLUS] = ACTIONS(1172),
    [anon_sym_DASH_DASH] = ACTIONS(1222),
    [anon_sym_PLUS_PLUS] = ACTIONS(1222),
    [anon_sym_DQUOTE] = ACTIONS(1225),
    [anon_sym_SQUOTE] = ACTIONS(1228),
    [sym_number_literal] = ACTIONS(1231),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(46)] = {
    [sym__block_item] = STATE(45),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_preproc_include] = STATE(45),
    [sym_preproc_define] = STATE(45),
    [sym_preproc_warn] = STATE(45),
    [sym_preproc_disable_warning] = STATE(45),
    [sym_preproc_if_defined] = STATE(45),
    [sym_preproc_if_not_defined] = STATE(45),
    [sym_define_function] = STATE(45),
    [sym_define_call] = STATE(45),
    [sym_define_module] = STATE(45),
    [sym_compound_statement] = STATE(494),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(1912),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(45),
    [sym_type_definition] = STATE(45),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_statement] = STATE(45),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_compound_statement_repeat1] = STATE(45),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_define_call_keyword] = ACTIONS(978),
    [sym_define_function_keyword] = ACTIONS(980),
    [sym_define_library_function_keyword] = ACTIONS(982),
    [sym_define_module_keyword] = ACTIONS(984),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(986),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(990),
    [sym_default_keyword] = ACTIONS(992),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(1036),
    [aux_sym_preproc_define_token1] = ACTIONS(1038),
    [aux_sym_preproc_warn_token1] = ACTIONS(1040),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1042),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1044),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1046),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(1234),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(47)] = {
    [sym__block_item] = STATE(46),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_preproc_include] = STATE(46),
    [sym_preproc_define] = STATE(46),
    [sym_preproc_warn] = STATE(46),
    [sym_preproc_disable_warning] = STATE(46),
    [sym_preproc_if_defined] = STATE(46),
    [sym_preproc_if_not_defined] = STATE(46),
    [sym_define_function] = STATE(46),
    [sym_define_call] = STATE(46),
    [sym_define_module] = STATE(46),
    [sym_compound_statement] = STATE(494),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(1912),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(46),
    [sym_type_definition] = STATE(46),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_statement] = STATE(46),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_compound_statement_repeat1] = STATE(46),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_define_call_keyword] = ACTIONS(978),
    [sym_define_function_keyword] = ACTIONS(980),
    [sym_define_library_function_keyword] = ACTIONS(982),
    [sym_define_module_keyword] = ACTIONS(984),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(986),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(990),
    [sym_default_keyword] = ACTIONS(992),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(1036),
    [aux_sym_preproc_define_token1] = ACTIONS(1038),
    [aux_sym_preproc_warn_token1] = ACTIONS(1040),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1042),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1044),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1046),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(1236),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(48)] = {
    [sym__block_item] = STATE(41),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_preproc_include] = STATE(41),
    [sym_preproc_define] = STATE(41),
    [sym_preproc_warn] = STATE(41),
    [sym_preproc_disable_warning] = STATE(41),
    [sym_preproc_if_defined] = STATE(41),
    [sym_preproc_if_not_defined] = STATE(41),
    [sym_define_function] = STATE(41),
    [sym_define_call] = STATE(41),
    [sym_define_module] = STATE(41),
    [sym_compound_statement] = STATE(494),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(1912),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(41),
    [sym_type_definition] = STATE(41),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_statement] = STATE(41),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_compound_statement_repeat1] = STATE(41),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_define_call_keyword] = ACTIONS(978),
    [sym_define_function_keyword] = ACTIONS(980),
    [sym_define_library_function_keyword] = ACTIONS(982),
    [sym_define_module_keyword] = ACTIONS(984),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(986),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(990),
    [sym_default_keyword] = ACTIONS(992),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(1036),
    [aux_sym_preproc_define_token1] = ACTIONS(1038),
    [aux_sym_preproc_warn_token1] = ACTIONS(1040),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1042),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1044),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1046),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(1238),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(49)] = {
    [sym__block_item] = STATE(42),
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_preproc_include] = STATE(42),
    [sym_preproc_define] = STATE(42),
    [sym_preproc_warn] = STATE(42),
    [sym_preproc_disable_warning] = STATE(42),
    [sym_preproc_if_defined] = STATE(42),
    [sym_preproc_if_not_defined] = STATE(42),
    [sym_define_function] = STATE(42),
    [sym_define_call] = STATE(42),
    [sym_define_module] = STATE(42),
    [sym_compound_statement] = STATE(494),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_struct_specifier] = STATE(1912),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(42),
    [sym_type_definition] = STATE(42),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_statement] = STATE(42),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym_compound_statement_repeat1] = STATE(42),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [sym_identifier] = ACTIONS(7),
    [sym_define_call_keyword] = ACTIONS(978),
    [sym_define_function_keyword] = ACTIONS(980),
    [sym_define_library_function_keyword] = ACTIONS(982),
    [sym_define_module_keyword] = ACTIONS(984),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(986),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(990),
    [sym_default_keyword] = ACTIONS(992),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [sym_struct_keyword] = ACTIONS(77),
    [sym_structure_keyword] = ACTIONS(77),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(1036),
    [aux_sym_preproc_define_token1] = ACTIONS(1038),
    [aux_sym_preproc_warn_token1] = ACTIONS(1040),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1042),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1044),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1046),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(1240),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(50)] = {
    [sym_expression] = STATE(1141),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(1242),
    [sym_program_name_keyword] = ACTIONS(1245),
    [sym_module_name_keyword] = ACTIONS(1245),
    [sym_define_device_keyword] = ACTIONS(1245),
    [sym_define_combine_keyword] = ACTIONS(1245),
    [sym_define_constant_keyword] = ACTIONS(1245),
    [sym_define_type_keyword] = ACTIONS(1245),
    [sym_define_variable_keyword] = ACTIONS(1245),
    [sym_define_system_variable_keyword] = ACTIONS(1245),
    [sym_define_connect_level_keyword] = ACTIONS(1245),
    [sym_define_latching_keyword] = ACTIONS(1245),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1245),
    [sym_define_toggling_keyword] = ACTIONS(1245),
    [sym_define_call_keyword] = ACTIONS(1245),
    [sym_define_function_keyword] = ACTIONS(1245),
    [sym_define_library_function_keyword] = ACTIONS(1245),
    [sym_define_start_keyword] = ACTIONS(1245),
    [sym_define_event_keyword] = ACTIONS(1245),
    [sym_define_program_keyword] = ACTIONS(1245),
    [sym_define_module_keyword] = ACTIONS(1245),
    [sym_char_keyword] = ACTIONS(1245),
    [sym_widechar_keyword] = ACTIONS(1245),
    [sym_integer_keyword] = ACTIONS(1245),
    [sym_sinteger_keyword] = ACTIONS(1245),
    [sym_long_keyword] = ACTIONS(1245),
    [sym_slong_keyword] = ACTIONS(1245),
    [sym_float_keyword] = ACTIONS(1245),
    [sym_double_keyword] = ACTIONS(1245),
    [sym_variant_keyword] = ACTIONS(1245),
    [sym_variantarray_keyword] = ACTIONS(1245),
    [sym_local_var_keyword] = ACTIONS(1245),
    [sym_stack_var_keyword] = ACTIONS(1245),
    [sym_constant_keyword] = ACTIONS(1245),
    [sym_volatile_keyword] = ACTIONS(1245),
    [sym_non_volatile_keyword] = ACTIONS(1245),
    [sym_persistent_keyword] = ACTIONS(1245),
    [sym_dev_keyword] = ACTIONS(1245),
    [sym_devlev_keyword] = ACTIONS(1245),
    [sym_devchan_keyword] = ACTIONS(1245),
    [sym_if_keyword] = ACTIONS(1245),
    [sym_else_keyword] = ACTIONS(1245),
    [sym_switch_keyword] = ACTIONS(1245),
    [sym_case_keyword] = ACTIONS(1245),
    [sym_default_keyword] = ACTIONS(1245),
    [sym_while_keyword] = ACTIONS(1245),
    [sym_for_keyword] = ACTIONS(1245),
    [sym_break_keyword] = ACTIONS(1245),
    [sym_continue_keyword] = ACTIONS(1245),
    [sym_return_keyword] = ACTIONS(1245),
    [sym_select_keyword] = ACTIONS(1245),
    [sym_active_keyword] = ACTIONS(1245),
    [sym_struct_keyword] = ACTIONS(1245),
    [sym_structure_keyword] = ACTIONS(1245),
    [sym_button_event_keyword] = ACTIONS(1245),
    [sym_channel_event_keyword] = ACTIONS(1245),
    [sym_level_event_keyword] = ACTIONS(1245),
    [sym_data_event_keyword] = ACTIONS(1245),
    [sym_timeline_event_keyword] = ACTIONS(1245),
    [sym_custom_event_keyword] = ACTIONS(1245),
    [sym_push_keyword] = ACTIONS(1245),
    [sym_release_keyword] = ACTIONS(1245),
    [aux_sym_on_keyword_token1] = ACTIONS(1245),
    [aux_sym_off_keyword_token1] = ACTIONS(1245),
    [sym_send_level_keyword] = ACTIONS(1245),
    [sym_send_string_keyword] = ACTIONS(1245),
    [sym_send_command_keyword] = ACTIONS(1245),
    [sym_clear_buffer_keyword] = ACTIONS(1245),
    [sym_create_buffer_keyword] = ACTIONS(1245),
    [sym_create_multi_buffer_keyword] = ACTIONS(1245),
    [sym_call_keyword] = ACTIONS(1245),
    [sym_system_call_keyword] = ACTIONS(1245),
    [sym_devchan_to_keyword] = ACTIONS(1245),
    [sym_devchan_min_to_keyword] = ACTIONS(1245),
    [sym_devchan_total_off_keyword] = ACTIONS(1245),
    [sym_devchan_pulse_keyword] = ACTIONS(1245),
    [sym_wait_keyword] = ACTIONS(1245),
    [sym_cancel_wait_keyword] = ACTIONS(1245),
    [sym_wait_until_keyword] = ACTIONS(1245),
    [sym_cancel_wait_until_keyword] = ACTIONS(1245),
    [sym_cancel_all_wait_keyword] = ACTIONS(1245),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1245),
    [sym_and] = ACTIONS(1245),
    [sym_or] = ACTIONS(1245),
    [sym_not] = ACTIONS(1247),
    [sym_xor] = ACTIONS(1245),
    [sym_band] = ACTIONS(1245),
    [sym_bor] = ACTIONS(1245),
    [sym_bxor] = ACTIONS(1245),
    [sym_bnot] = ACTIONS(1247),
    [sym_lshift] = ACTIONS(1245),
    [sym_rshift] = ACTIONS(1245),
    [sym___file__] = ACTIONS(1250),
    [sym___date__] = ACTIONS(1250),
    [sym___ldate__] = ACTIONS(1250),
    [sym___time__] = ACTIONS(1250),
    [sym___line__] = ACTIONS(1250),
    [sym___name__] = ACTIONS(1250),
    [sym___netlinx__] = ACTIONS(1253),
    [sym_netlinx_axi_version] = ACTIONS(1253),
    [sym_url_flg_tcp] = ACTIONS(1253),
    [sym_url_flg_accinfopresent] = ACTIONS(1253),
    [sym_url_flg_temp] = ACTIONS(1253),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1253),
    [sym_url_flg_stat_mask] = ACTIONS(1253),
    [sym_url_flg_stat_lookup] = ACTIONS(1253),
    [sym_url_flg_stat_connecting] = ACTIONS(1253),
    [sym_url_flg_stat_waiting] = ACTIONS(1253),
    [sym_url_flg_stat_connected] = ACTIONS(1253),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1253),
    [sym_first_virtual_device] = ACTIONS(1253),
    [sym_first_local_port] = ACTIONS(1253),
    [sym_dynamic_virtual_device] = ACTIONS(1253),
    [sym_timeline_once] = ACTIONS(1253),
    [sym_timeline_repeat] = ACTIONS(1253),
    [sym_timeline_absolute] = ACTIONS(1253),
    [sym_timeline_relative] = ACTIONS(1253),
    [sym_do_push_timed_infinite] = ACTIONS(1253),
    [sym_source_type_no_address] = ACTIONS(1253),
    [sym_source_type_neuron_id] = ACTIONS(1253),
    [sym_source_type_ip_address] = ACTIONS(1253),
    [sym_source_type_axlink] = ACTIONS(1253),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1253),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1253),
    [sym_source_type_ip_socket_address] = ACTIONS(1253),
    [sym_source_type_rs232] = ACTIONS(1253),
    [sym_source_type_internal] = ACTIONS(1253),
    [sym_source_type_mac_address] = ACTIONS(1253),
    [sym_source_type_ipv4_port] = ACTIONS(1253),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1253),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1253),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1253),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1253),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1253),
    [sym_true] = ACTIONS(1253),
    [sym_false] = ACTIONS(1253),
    [sym_file_read_only] = ACTIONS(1253),
    [sym_file_rw_new] = ACTIONS(1253),
    [sym_file_rw_append] = ACTIONS(1253),
    [sym_ip_tcp] = ACTIONS(1253),
    [sym_ip_udp] = ACTIONS(1253),
    [sym_ip_udp_2way] = ACTIONS(1253),
    [sym_xml_encode_types] = ACTIONS(1253),
    [sym_xml_encode_char_as_list] = ACTIONS(1253),
    [sym_xml_encode_le] = ACTIONS(1253),
    [sym_xml_decode_types] = ACTIONS(1253),
    [sym_xml_decode_no_preserve] = ACTIONS(1253),
    [sym_duet_dev_type_display_device] = ACTIONS(1253),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1253),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1253),
    [sym_duet_dev_type_io_device] = ACTIONS(1253),
    [sym_duet_dev_type_relay_device] = ACTIONS(1253),
    [sym_duet_dev_type_ups] = ACTIONS(1253),
    [sym_duet_dev_type_amplifier] = ACTIONS(1253),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1253),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1253),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1253),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1253),
    [sym_duet_dev_type_camera] = ACTIONS(1253),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1253),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1253),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1253),
    [sym_duet_dev_type_dss] = ACTIONS(1253),
    [sym_duet_dev_type_dvr] = ACTIONS(1253),
    [sym_duet_dev_type_disc_device] = ACTIONS(1253),
    [sym_duet_dev_type_document_camera] = ACTIONS(1253),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1253),
    [sym_duet_dev_type_hvac] = ACTIONS(1253),
    [sym_duet_dev_type_keypad] = ACTIONS(1253),
    [sym_duet_dev_type_light] = ACTIONS(1253),
    [sym_duet_dev_type_monitor] = ACTIONS(1253),
    [sym_duet_dev_type_motor] = ACTIONS(1253),
    [sym_duet_dev_type_multi_window] = ACTIONS(1253),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1253),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1253),
    [sym_duet_dev_type_receiver] = ACTIONS(1253),
    [sym_duet_dev_type_security_system] = ACTIONS(1253),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1253),
    [sym_duet_dev_type_settop_box] = ACTIONS(1253),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1253),
    [sym_duet_dev_type_switcher] = ACTIONS(1253),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1253),
    [sym_duet_dev_type_tv] = ACTIONS(1253),
    [sym_duet_dev_type_utility] = ACTIONS(1253),
    [sym_duet_dev_type_vcr] = ACTIONS(1253),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1253),
    [sym_duet_dev_type_video_processor] = ACTIONS(1253),
    [sym_duet_dev_type_video_projector] = ACTIONS(1253),
    [sym_duet_dev_type_video_wall] = ACTIONS(1253),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1253),
    [sym_duet_dev_type_weather] = ACTIONS(1253),
    [sym_duet_dev_not_polled] = ACTIONS(1253),
    [sym_duet_dev_polled] = ACTIONS(1253),
    [sym_clkmgr_mode_network] = ACTIONS(1253),
    [sym_clkmgr_mode_standalone] = ACTIONS(1253),
    [sym_normal_standby] = ACTIONS(1253),
    [sym_normal_wake] = ACTIONS(1253),
    [sym_ip_multicast_ttl_option] = ACTIONS(1253),
    [sym_ip_ttl_subnet] = ACTIONS(1253),
    [sym_ip_ttl_site] = ACTIONS(1253),
    [sym_ip_ttl_region] = ACTIONS(1253),
    [sym_ip_ttl_continent] = ACTIONS(1253),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1253),
    [sym_ip_nodelay_on] = ACTIONS(1253),
    [sym_ip_nodelay_off] = ACTIONS(1253),
    [sym_amx_error] = ACTIONS(1253),
    [sym_amx_warning] = ACTIONS(1253),
    [sym_amx_info] = ACTIONS(1253),
    [sym_amx_debug] = ACTIONS(1253),
    [sym_netlinx_login_success] = ACTIONS(1253),
    [sym_netlinx_login_fail] = ACTIONS(1253),
    [sym_netlinx_logout] = ACTIONS(1253),
    [sym_null_str] = ACTIONS(1253),
    [sym_smtp_address] = ACTIONS(1253),
    [sym_smtp_port_number] = ACTIONS(1253),
    [sym_smtp_username] = ACTIONS(1253),
    [sym_smtp_password] = ACTIONS(1253),
    [sym_smtp_require_tls] = ACTIONS(1253),
    [sym_smtp_from] = ACTIONS(1253),
    [sym_smtp_tls_true] = ACTIONS(1253),
    [sym_smtp_tls_false] = ACTIONS(1253),
    [sym_listview_on_row_select_event] = ACTIONS(1253),
    [sym_data_structure_unknown] = ACTIONS(1253),
    [sym_data_structure_datafeed] = ACTIONS(1253),
    [sym_data_structure_datarecordset] = ACTIONS(1253),
    [sym_data_structure_datarecord] = ACTIONS(1253),
    [sym_data_structure_datafield] = ACTIONS(1253),
    [sym_data_max_content_field_count] = ACTIONS(1253),
    [sym_data_max_metadata_field_count] = ACTIONS(1253),
    [sym_data_max_id_length] = ACTIONS(1253),
    [sym_data_max_name_length] = ACTIONS(1253),
    [sym_data_max_description_length] = ACTIONS(1253),
    [sym_data_max_type_length] = ACTIONS(1253),
    [sym_data_max_format_length] = ACTIONS(1253),
    [sym_data_max_source_length] = ACTIONS(1253),
    [sym_data_max_value_length] = ACTIONS(1253),
    [sym_data_max_label_length] = ACTIONS(1253),
    [sym_data_max_url_length] = ACTIONS(1253),
    [sym_max_last_login_info_length] = ACTIONS(1253),
    [sym_data_type_unknown] = ACTIONS(1253),
    [sym_data_type_string] = ACTIONS(1253),
    [sym_data_type_datetime] = ACTIONS(1253),
    [sym_data_type_date] = ACTIONS(1253),
    [sym_data_type_time] = ACTIONS(1253),
    [sym_data_type_image] = ACTIONS(1253),
    [sym_data_format_url] = ACTIONS(1253),
    [sym_data_format_phone] = ACTIONS(1253),
    [sym_data_format_email] = ACTIONS(1253),
    [sym_data_format_iso8601] = ACTIONS(1253),
    [sym_auditlog_priv] = ACTIONS(1253),
    [sym_configuration_priv] = ACTIONS(1253),
    [sym_deviceconfig_priv] = ACTIONS(1253),
    [sym_ftp_priv] = ACTIONS(1253),
    [sym_http_priv] = ACTIONS(1253),
    [sym_networkconfig_priv] = ACTIONS(1253),
    [sym_programport_priv] = ACTIONS(1253),
    [sym_remoteui_priv] = ACTIONS(1253),
    [sym_securitycontrol_priv] = ACTIONS(1253),
    [sym_softwaremanagement_priv] = ACTIONS(1253),
    [sym_terminal_priv] = ACTIONS(1253),
    [sym_tpadmin_priv] = ACTIONS(1253),
    [sym_user1_priv] = ACTIONS(1253),
    [sym_user2_priv] = ACTIONS(1253),
    [sym_user3_priv] = ACTIONS(1253),
    [sym_user4_priv] = ACTIONS(1253),
    [sym_usermanagement_priv] = ACTIONS(1253),
    [sym_valid_account] = ACTIONS(1253),
    [sym_err_name_invalid] = ACTIONS(1253),
    [sym_err_password_invalid] = ACTIONS(1253),
    [sym_err_invalid_account] = ACTIONS(1253),
    [sym_err_no_authorization] = ACTIONS(1253),
    [sym_err_invalid_parameter] = ACTIONS(1253),
    [sym_err_user_locked_out] = ACTIONS(1253),
    [sym_err_user_account_expired] = ACTIONS(1253),
    [sym_err_general_error] = ACTIONS(1253),
    [sym_err_authentication_error] = ACTIONS(1253),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1253),
    [sym_tls_validate_certificate] = ACTIONS(1253),
    [sym_tdata] = ACTIONS(1245),
    [sym_tchannel] = ACTIONS(1245),
    [sym_tlevel] = ACTIONS(1245),
    [sym_tbutton] = ACTIONS(1245),
    [sym_ttimeline] = ACTIONS(1245),
    [sym_tcustom] = ACTIONS(1245),
    [sym_url_struct] = ACTIONS(1245),
    [sym_dns_struct] = ACTIONS(1245),
    [sym_ip_address_struct] = ACTIONS(1245),
    [sym_dev_info_struct] = ACTIONS(1245),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1245),
    [sym_data_feed] = ACTIONS(1245),
    [sym_data_field] = ACTIONS(1245),
    [sym_data_record] = ACTIONS(1245),
    [sym_wc_data_feed] = ACTIONS(1245),
    [sym_wc_data_field] = ACTIONS(1245),
    [sym_wc_data_record] = ACTIONS(1245),
    [sym_last_login_info] = ACTIONS(1245),
    [sym____reserved___] = ACTIONS(1256),
    [sym_date] = ACTIONS(1256),
    [sym_day] = ACTIONS(1256),
    [sym_time] = ACTIONS(1256),
    [sym_ldate] = ACTIONS(1256),
    [sym_push_device] = ACTIONS(1256),
    [sym_push_channel] = ACTIONS(1256),
    [sym_push_devchan] = ACTIONS(1256),
    [sym_release_device] = ACTIONS(1256),
    [sym_release_channel] = ACTIONS(1256),
    [sym_release_devchan] = ACTIONS(1256),
    [sym_master_slot] = ACTIONS(1256),
    [sym_get_pulse_time] = ACTIONS(1256),
    [sym_get_timer] = ACTIONS(1256),
    [sym_master_sn] = ACTIONS(1256),
    [sym_system_number] = ACTIONS(1256),
    [sym_button] = ACTIONS(1256),
    [sym_dv_channel] = ACTIONS(1256),
    [sym_data] = ACTIONS(1256),
    [sym_level] = ACTIONS(1256),
    [sym_timeline] = ACTIONS(1256),
    [sym_channel] = ACTIONS(1256),
    [sym_custom] = ACTIONS(1256),
    [sym_atoi] = ACTIONS(1259),
    [sym_atol] = ACTIONS(1259),
    [sym_atof] = ACTIONS(1259),
    [sym_itoa] = ACTIONS(1259),
    [sym_itohex] = ACTIONS(1259),
    [sym_hextoi] = ACTIONS(1259),
    [sym_ftoa] = ACTIONS(1259),
    [sym_find_string] = ACTIONS(1259),
    [sym_left_string] = ACTIONS(1259),
    [sym_right_string] = ACTIONS(1259),
    [sym_mid_string] = ACTIONS(1259),
    [sym_remove_string] = ACTIONS(1259),
    [sym_get_buffer_string] = ACTIONS(1259),
    [sym_get_buffer_char] = ACTIONS(1259),
    [sym_get_multi_buffer_string] = ACTIONS(1259),
    [sym_length_string] = ACTIONS(1259),
    [sym_max_length_string] = ACTIONS(1259),
    [sym_set_length_string] = ACTIONS(1259),
    [sym_compare_string] = ACTIONS(1259),
    [sym_upper_string] = ACTIONS(1259),
    [sym_lower_string] = ACTIONS(1259),
    [sym_get_last] = ACTIONS(1259),
    [sym_date_to_day] = ACTIONS(1259),
    [sym_date_to_month] = ACTIONS(1259),
    [sym_date_to_year] = ACTIONS(1259),
    [sym_time_to_hour] = ACTIONS(1259),
    [sym_time_to_minute] = ACTIONS(1259),
    [sym_time_to_second] = ACTIONS(1259),
    [sym_day_of_week] = ACTIONS(1259),
    [sym_set_pulse_time] = ACTIONS(1259),
    [sym_set_timer] = ACTIONS(1259),
    [sym_file_open] = ACTIONS(1259),
    [sym_file_close] = ACTIONS(1259),
    [sym_file_read] = ACTIONS(1259),
    [sym_file_read_line] = ACTIONS(1259),
    [sym_file_write] = ACTIONS(1259),
    [sym_file_write_line] = ACTIONS(1259),
    [sym_file_copy] = ACTIONS(1259),
    [sym_file_delete] = ACTIONS(1259),
    [sym_file_rename] = ACTIONS(1259),
    [sym_file_dir] = ACTIONS(1259),
    [sym_file_getdir] = ACTIONS(1259),
    [sym_file_setdir] = ACTIONS(1259),
    [sym_file_create_dir] = ACTIONS(1259),
    [sym_file_remove_dir] = ACTIONS(1259),
    [sym_file_seek] = ACTIONS(1259),
    [sym_length_array] = ACTIONS(1259),
    [sym_max_length_array] = ACTIONS(1259),
    [sym_set_length_array] = ACTIONS(1259),
    [sym_ip_client_open] = ACTIONS(1259),
    [sym_ip_client_close] = ACTIONS(1259),
    [sym_ip_server_open] = ACTIONS(1259),
    [sym_ip_server_close] = ACTIONS(1259),
    [sym_ip_bound_client_open] = ACTIONS(1259),
    [sym_ip_mc_server_open] = ACTIONS(1259),
    [sym_ip_set_option] = ACTIONS(1259),
    [sym_ssh_client_open] = ACTIONS(1259),
    [sym_ssh_client_close] = ACTIONS(1259),
    [sym_tls_client_open] = ACTIONS(1259),
    [sym_tls_client_close] = ACTIONS(1259),
    [sym_do_push] = ACTIONS(1259),
    [sym_do_push_timed] = ACTIONS(1259),
    [sym_do_release] = ACTIONS(1259),
    [sym_device_id] = ACTIONS(1259),
    [sym_device_info] = ACTIONS(1259),
    [sym_device_id_string] = ACTIONS(1259),
    [sym_device_standby] = ACTIONS(1259),
    [sym_device_wake] = ACTIONS(1259),
    [sym_timeline_create] = ACTIONS(1259),
    [sym_timeline_kill] = ACTIONS(1259),
    [sym_timeline_reload] = ACTIONS(1259),
    [sym_timeline_pause] = ACTIONS(1259),
    [sym_timeline_restart] = ACTIONS(1259),
    [sym_timeline_set] = ACTIONS(1259),
    [sym_timeline_get] = ACTIONS(1259),
    [sym_timeline_active] = ACTIONS(1259),
    [sym_timeline_dynamic_id] = ACTIONS(1259),
    [sym_variable_to_string] = ACTIONS(1259),
    [sym_string_to_variable] = ACTIONS(1259),
    [sym_length_variable_to_string] = ACTIONS(1259),
    [sym_type_cast] = ACTIONS(1259),
    [sym_variable_to_xml] = ACTIONS(1259),
    [sym_xml_to_variable] = ACTIONS(1259),
    [sym_length_variable_to_xml] = ACTIONS(1259),
    [sym_redirect_string] = ACTIONS(1259),
    [sym_random_number] = ACTIONS(1259),
    [sym_get_unique_id] = ACTIONS(1259),
    [sym_get_system_number] = ACTIONS(1259),
    [sym_set_system_number] = ACTIONS(1259),
    [sym_get_serial_number] = ACTIONS(1259),
    [sym_get_master_build] = ACTIONS(1259),
    [sym_set_validation_code] = ACTIONS(1259),
    [sym_reboot] = ACTIONS(1259),
    [sym_load_duet_module] = ACTIONS(1259),
    [sym_unload_duet_module] = ACTIONS(1259),
    [sym_duet_mem_size_set] = ACTIONS(1259),
    [sym_duet_mem_size_get] = ACTIONS(1259),
    [sym_dynamic_application_device] = ACTIONS(1259),
    [sym_dynamic_polled_port] = ACTIONS(1259),
    [sym_static_port_binding] = ACTIONS(1259),
    [sym_static_ip_binding] = ACTIONS(1259),
    [sym_get_dns_list] = ACTIONS(1259),
    [sym_set_dns_list] = ACTIONS(1259),
    [sym_get_ip_address] = ACTIONS(1259),
    [sym_set_ip_address] = ACTIONS(1259),
    [sym_get_url_list] = ACTIONS(1259),
    [sym_add_url_entry] = ACTIONS(1259),
    [sym_delete_url_entry] = ACTIONS(1259),
    [sym_abs_value] = ACTIONS(1259),
    [sym_max_value] = ACTIONS(1259),
    [sym_min_value] = ACTIONS(1259),
    [sym_exp_value] = ACTIONS(1259),
    [sym_log_value] = ACTIONS(1259),
    [sym_log10_value] = ACTIONS(1259),
    [sym_power_value] = ACTIONS(1259),
    [sym_sqrt_value] = ACTIONS(1259),
    [sym_raw_be] = ACTIONS(1259),
    [sym_raw_le] = ACTIONS(1259),
    [sym_internal_queue_size_set] = ACTIONS(1259),
    [sym_internal_queue_size_get] = ACTIONS(1259),
    [sym_internal_threshold_set] = ACTIONS(1259),
    [sym_internal_threshold_get] = ACTIONS(1259),
    [sym_rebuilt_event] = ACTIONS(1259),
    [sym_do_custom_event] = ACTIONS(1259),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1259),
    [sym_clkmgr_set_clk_source] = ACTIONS(1259),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1259),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1259),
    [sym_clkmgr_get_timezone] = ACTIONS(1259),
    [sym_clkmgr_set_timezone] = ACTIONS(1259),
    [sym_clkmgr_get_resync_period] = ACTIONS(1259),
    [sym_clkmgr_set_resync_period] = ACTIONS(1259),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1259),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1259),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1259),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1259),
    [sym_clkmgr_get_timeservers] = ACTIONS(1259),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1259),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1259),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1259),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1259),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1259),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1259),
    [sym_set_log_level] = ACTIONS(1259),
    [sym_get_log_level] = ACTIONS(1259),
    [sym_amx_log] = ACTIONS(1259),
    [sym_smtp_server_config_set] = ACTIONS(1259),
    [sym_smtp_server_config_get] = ACTIONS(1259),
    [sym_smtp_send] = ACTIONS(1259),
    [sym_validate_netlinx_account] = ACTIONS(1259),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1259),
    [sym_authenticate_certificate] = ACTIONS(1259),
    [sym_audit_netlinx_session_event] = ACTIONS(1259),
    [sym_audit_netlinx_generic_event] = ACTIONS(1259),
    [sym_get_available_flash_disk_space] = ACTIONS(1259),
    [sym_get_max_flash_disk_space] = ACTIONS(1259),
    [sym_data_create_feed] = ACTIONS(1259),
    [sym_data_delete_feed] = ACTIONS(1259),
    [sym_data_publish_feed] = ACTIONS(1259),
    [sym_data_get_published_feed] = ACTIONS(1259),
    [sym_data_add_record] = ACTIONS(1259),
    [sym_data_get_event_record] = ACTIONS(1259),
    [sym__wc_data_create_feed] = ACTIONS(1259),
    [sym__wc_data_add_record] = ACTIONS(1259),
    [sym__wc_data_get_event_record] = ACTIONS(1259),
    [sym_format] = ACTIONS(1259),
    [sym_astro_clock] = ACTIONS(1259),
    [sym_set_outdoor_temperature] = ACTIONS(1259),
    [sym_set_virtual_level_count] = ACTIONS(1259),
    [sym_set_virtual_channel_count] = ACTIONS(1259),
    [sym_set_virtual_port_count] = ACTIONS(1259),
    [sym__wc_encode_int] = ACTIONS(1259),
    [sym__wc_decode_int] = ACTIONS(1259),
    [sym__wc_int] = ACTIONS(1259),
    [sym__wc_to_ch_int] = ACTIONS(1259),
    [sym__ch_to_wc_int] = ACTIONS(1259),
    [sym__wc_find_string_int] = ACTIONS(1259),
    [sym__wc_left_string_int] = ACTIONS(1259),
    [sym__wc_lower_string_int] = ACTIONS(1259),
    [sym__wc_mid_string_int] = ACTIONS(1259),
    [sym__wc_remove_string_int] = ACTIONS(1259),
    [sym__wc_right_string_int] = ACTIONS(1259),
    [sym__wc_upper_string_int] = ACTIONS(1259),
    [sym__wc_compare_string_int] = ACTIONS(1259),
    [sym__wc_get_buffer_char_int] = ACTIONS(1259),
    [sym__wc_get_buffer_string_int] = ACTIONS(1259),
    [sym__wc_concat_string_int] = ACTIONS(1259),
    [sym__wc_file_open_int] = ACTIONS(1259),
    [sym__wc_file_close_int] = ACTIONS(1259),
    [sym__wc_file_read_int] = ACTIONS(1259),
    [sym__wc_file_read_line_int] = ACTIONS(1259),
    [sym__wc_file_write_int] = ACTIONS(1259),
    [sym__wc_file_write_line_int] = ACTIONS(1259),
    [aux_sym_preproc_include_token1] = ACTIONS(1245),
    [aux_sym_preproc_define_token1] = ACTIONS(1262),
    [aux_sym_preproc_warn_token1] = ACTIONS(1262),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1262),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1262),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1262),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1262),
    [aux_sym_preproc_else_token1] = ACTIONS(1262),
    [anon_sym_LBRACK] = ACTIONS(1264),
    [anon_sym_LBRACE] = ACTIONS(1262),
    [anon_sym_COLON] = ACTIONS(1262),
    [anon_sym_LPAREN] = ACTIONS(1267),
    [anon_sym_SEMI] = ACTIONS(1262),
    [anon_sym_BANG] = ACTIONS(1247),
    [anon_sym_TILDE] = ACTIONS(1270),
    [anon_sym_DASH] = ACTIONS(1247),
    [anon_sym_PLUS] = ACTIONS(1247),
    [anon_sym_STAR] = ACTIONS(1262),
    [anon_sym_SLASH] = ACTIONS(1245),
    [anon_sym_PERCENT] = ACTIONS(1262),
    [anon_sym_PIPE_PIPE] = ACTIONS(1262),
    [anon_sym_AMP_AMP] = ACTIONS(1262),
    [anon_sym_PIPE] = ACTIONS(1245),
    [anon_sym_CARET] = ACTIONS(1262),
    [anon_sym_AMP] = ACTIONS(1245),
    [anon_sym_EQ_EQ] = ACTIONS(1262),
    [anon_sym_BANG_EQ] = ACTIONS(1262),
    [anon_sym_LT_GT] = ACTIONS(1262),
    [anon_sym_GT] = ACTIONS(1245),
    [anon_sym_GT_EQ] = ACTIONS(1262),
    [anon_sym_LT_EQ] = ACTIONS(1262),
    [anon_sym_LT] = ACTIONS(1245),
    [anon_sym_LT_LT] = ACTIONS(1262),
    [anon_sym_GT_GT] = ACTIONS(1262),
    [anon_sym_DASH_DASH] = ACTIONS(1273),
    [anon_sym_PLUS_PLUS] = ACTIONS(1273),
    [anon_sym_DOT] = ACTIONS(1245),
    [anon_sym_DQUOTE] = ACTIONS(1276),
    [anon_sym_SQUOTE] = ACTIONS(1279),
    [sym_number_literal] = ACTIONS(1282),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1262),
  },
  [STATE(51)] = {
    [sym_expression] = STATE(1141),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [ts_builtin_sym_end] = ACTIONS(1262),
    [sym_identifier] = ACTIONS(1242),
    [sym_program_name_keyword] = ACTIONS(1245),
    [sym_module_name_keyword] = ACTIONS(1245),
    [sym_define_device_keyword] = ACTIONS(1245),
    [sym_define_combine_keyword] = ACTIONS(1245),
    [sym_define_constant_keyword] = ACTIONS(1245),
    [sym_define_type_keyword] = ACTIONS(1245),
    [sym_define_variable_keyword] = ACTIONS(1245),
    [sym_define_system_variable_keyword] = ACTIONS(1245),
    [sym_define_connect_level_keyword] = ACTIONS(1245),
    [sym_define_latching_keyword] = ACTIONS(1245),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1245),
    [sym_define_toggling_keyword] = ACTIONS(1245),
    [sym_define_call_keyword] = ACTIONS(1245),
    [sym_define_function_keyword] = ACTIONS(1245),
    [sym_define_library_function_keyword] = ACTIONS(1245),
    [sym_define_start_keyword] = ACTIONS(1245),
    [sym_define_event_keyword] = ACTIONS(1245),
    [sym_define_program_keyword] = ACTIONS(1245),
    [sym_define_module_keyword] = ACTIONS(1245),
    [sym_char_keyword] = ACTIONS(1245),
    [sym_widechar_keyword] = ACTIONS(1245),
    [sym_integer_keyword] = ACTIONS(1245),
    [sym_sinteger_keyword] = ACTIONS(1245),
    [sym_long_keyword] = ACTIONS(1245),
    [sym_slong_keyword] = ACTIONS(1245),
    [sym_float_keyword] = ACTIONS(1245),
    [sym_double_keyword] = ACTIONS(1245),
    [sym_variant_keyword] = ACTIONS(1245),
    [sym_variantarray_keyword] = ACTIONS(1245),
    [sym_local_var_keyword] = ACTIONS(1245),
    [sym_stack_var_keyword] = ACTIONS(1245),
    [sym_constant_keyword] = ACTIONS(1245),
    [sym_volatile_keyword] = ACTIONS(1245),
    [sym_non_volatile_keyword] = ACTIONS(1245),
    [sym_persistent_keyword] = ACTIONS(1245),
    [sym_dev_keyword] = ACTIONS(1245),
    [sym_devlev_keyword] = ACTIONS(1245),
    [sym_devchan_keyword] = ACTIONS(1245),
    [sym_if_keyword] = ACTIONS(1245),
    [sym_else_keyword] = ACTIONS(1245),
    [sym_switch_keyword] = ACTIONS(1245),
    [sym_case_keyword] = ACTIONS(1245),
    [sym_default_keyword] = ACTIONS(1245),
    [sym_while_keyword] = ACTIONS(1245),
    [sym_for_keyword] = ACTIONS(1245),
    [sym_break_keyword] = ACTIONS(1245),
    [sym_continue_keyword] = ACTIONS(1245),
    [sym_return_keyword] = ACTIONS(1245),
    [sym_select_keyword] = ACTIONS(1245),
    [sym_active_keyword] = ACTIONS(1245),
    [sym_struct_keyword] = ACTIONS(1245),
    [sym_structure_keyword] = ACTIONS(1245),
    [sym_button_event_keyword] = ACTIONS(1245),
    [sym_channel_event_keyword] = ACTIONS(1245),
    [sym_level_event_keyword] = ACTIONS(1245),
    [sym_data_event_keyword] = ACTIONS(1245),
    [sym_timeline_event_keyword] = ACTIONS(1245),
    [sym_custom_event_keyword] = ACTIONS(1245),
    [sym_push_keyword] = ACTIONS(1245),
    [sym_release_keyword] = ACTIONS(1245),
    [aux_sym_on_keyword_token1] = ACTIONS(1245),
    [aux_sym_off_keyword_token1] = ACTIONS(1245),
    [sym_send_level_keyword] = ACTIONS(1245),
    [sym_send_string_keyword] = ACTIONS(1245),
    [sym_send_command_keyword] = ACTIONS(1245),
    [sym_clear_buffer_keyword] = ACTIONS(1245),
    [sym_create_buffer_keyword] = ACTIONS(1245),
    [sym_create_multi_buffer_keyword] = ACTIONS(1245),
    [sym_call_keyword] = ACTIONS(1245),
    [sym_system_call_keyword] = ACTIONS(1245),
    [sym_devchan_to_keyword] = ACTIONS(1245),
    [sym_devchan_min_to_keyword] = ACTIONS(1245),
    [sym_devchan_total_off_keyword] = ACTIONS(1245),
    [sym_devchan_pulse_keyword] = ACTIONS(1245),
    [sym_wait_keyword] = ACTIONS(1245),
    [sym_cancel_wait_keyword] = ACTIONS(1245),
    [sym_wait_until_keyword] = ACTIONS(1245),
    [sym_cancel_wait_until_keyword] = ACTIONS(1245),
    [sym_cancel_all_wait_keyword] = ACTIONS(1245),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1245),
    [sym_and] = ACTIONS(1245),
    [sym_or] = ACTIONS(1245),
    [sym_not] = ACTIONS(1247),
    [sym_xor] = ACTIONS(1245),
    [sym_band] = ACTIONS(1245),
    [sym_bor] = ACTIONS(1245),
    [sym_bxor] = ACTIONS(1245),
    [sym_bnot] = ACTIONS(1247),
    [sym_lshift] = ACTIONS(1245),
    [sym_rshift] = ACTIONS(1245),
    [sym___file__] = ACTIONS(1250),
    [sym___date__] = ACTIONS(1250),
    [sym___ldate__] = ACTIONS(1250),
    [sym___time__] = ACTIONS(1250),
    [sym___line__] = ACTIONS(1250),
    [sym___name__] = ACTIONS(1250),
    [sym___netlinx__] = ACTIONS(1253),
    [sym_netlinx_axi_version] = ACTIONS(1253),
    [sym_url_flg_tcp] = ACTIONS(1253),
    [sym_url_flg_accinfopresent] = ACTIONS(1253),
    [sym_url_flg_temp] = ACTIONS(1253),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1253),
    [sym_url_flg_stat_mask] = ACTIONS(1253),
    [sym_url_flg_stat_lookup] = ACTIONS(1253),
    [sym_url_flg_stat_connecting] = ACTIONS(1253),
    [sym_url_flg_stat_waiting] = ACTIONS(1253),
    [sym_url_flg_stat_connected] = ACTIONS(1253),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1253),
    [sym_first_virtual_device] = ACTIONS(1253),
    [sym_first_local_port] = ACTIONS(1253),
    [sym_dynamic_virtual_device] = ACTIONS(1253),
    [sym_timeline_once] = ACTIONS(1253),
    [sym_timeline_repeat] = ACTIONS(1253),
    [sym_timeline_absolute] = ACTIONS(1253),
    [sym_timeline_relative] = ACTIONS(1253),
    [sym_do_push_timed_infinite] = ACTIONS(1253),
    [sym_source_type_no_address] = ACTIONS(1253),
    [sym_source_type_neuron_id] = ACTIONS(1253),
    [sym_source_type_ip_address] = ACTIONS(1253),
    [sym_source_type_axlink] = ACTIONS(1253),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1253),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1253),
    [sym_source_type_ip_socket_address] = ACTIONS(1253),
    [sym_source_type_rs232] = ACTIONS(1253),
    [sym_source_type_internal] = ACTIONS(1253),
    [sym_source_type_mac_address] = ACTIONS(1253),
    [sym_source_type_ipv4_port] = ACTIONS(1253),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1253),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1253),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1253),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1253),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1253),
    [sym_true] = ACTIONS(1253),
    [sym_false] = ACTIONS(1253),
    [sym_file_read_only] = ACTIONS(1253),
    [sym_file_rw_new] = ACTIONS(1253),
    [sym_file_rw_append] = ACTIONS(1253),
    [sym_ip_tcp] = ACTIONS(1253),
    [sym_ip_udp] = ACTIONS(1253),
    [sym_ip_udp_2way] = ACTIONS(1253),
    [sym_xml_encode_types] = ACTIONS(1253),
    [sym_xml_encode_char_as_list] = ACTIONS(1253),
    [sym_xml_encode_le] = ACTIONS(1253),
    [sym_xml_decode_types] = ACTIONS(1253),
    [sym_xml_decode_no_preserve] = ACTIONS(1253),
    [sym_duet_dev_type_display_device] = ACTIONS(1253),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1253),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1253),
    [sym_duet_dev_type_io_device] = ACTIONS(1253),
    [sym_duet_dev_type_relay_device] = ACTIONS(1253),
    [sym_duet_dev_type_ups] = ACTIONS(1253),
    [sym_duet_dev_type_amplifier] = ACTIONS(1253),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1253),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1253),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1253),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1253),
    [sym_duet_dev_type_camera] = ACTIONS(1253),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1253),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1253),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1253),
    [sym_duet_dev_type_dss] = ACTIONS(1253),
    [sym_duet_dev_type_dvr] = ACTIONS(1253),
    [sym_duet_dev_type_disc_device] = ACTIONS(1253),
    [sym_duet_dev_type_document_camera] = ACTIONS(1253),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1253),
    [sym_duet_dev_type_hvac] = ACTIONS(1253),
    [sym_duet_dev_type_keypad] = ACTIONS(1253),
    [sym_duet_dev_type_light] = ACTIONS(1253),
    [sym_duet_dev_type_monitor] = ACTIONS(1253),
    [sym_duet_dev_type_motor] = ACTIONS(1253),
    [sym_duet_dev_type_multi_window] = ACTIONS(1253),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1253),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1253),
    [sym_duet_dev_type_receiver] = ACTIONS(1253),
    [sym_duet_dev_type_security_system] = ACTIONS(1253),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1253),
    [sym_duet_dev_type_settop_box] = ACTIONS(1253),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1253),
    [sym_duet_dev_type_switcher] = ACTIONS(1253),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1253),
    [sym_duet_dev_type_tv] = ACTIONS(1253),
    [sym_duet_dev_type_utility] = ACTIONS(1253),
    [sym_duet_dev_type_vcr] = ACTIONS(1253),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1253),
    [sym_duet_dev_type_video_processor] = ACTIONS(1253),
    [sym_duet_dev_type_video_projector] = ACTIONS(1253),
    [sym_duet_dev_type_video_wall] = ACTIONS(1253),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1253),
    [sym_duet_dev_type_weather] = ACTIONS(1253),
    [sym_duet_dev_not_polled] = ACTIONS(1253),
    [sym_duet_dev_polled] = ACTIONS(1253),
    [sym_clkmgr_mode_network] = ACTIONS(1253),
    [sym_clkmgr_mode_standalone] = ACTIONS(1253),
    [sym_normal_standby] = ACTIONS(1253),
    [sym_normal_wake] = ACTIONS(1253),
    [sym_ip_multicast_ttl_option] = ACTIONS(1253),
    [sym_ip_ttl_subnet] = ACTIONS(1253),
    [sym_ip_ttl_site] = ACTIONS(1253),
    [sym_ip_ttl_region] = ACTIONS(1253),
    [sym_ip_ttl_continent] = ACTIONS(1253),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1253),
    [sym_ip_nodelay_on] = ACTIONS(1253),
    [sym_ip_nodelay_off] = ACTIONS(1253),
    [sym_amx_error] = ACTIONS(1253),
    [sym_amx_warning] = ACTIONS(1253),
    [sym_amx_info] = ACTIONS(1253),
    [sym_amx_debug] = ACTIONS(1253),
    [sym_netlinx_login_success] = ACTIONS(1253),
    [sym_netlinx_login_fail] = ACTIONS(1253),
    [sym_netlinx_logout] = ACTIONS(1253),
    [sym_null_str] = ACTIONS(1253),
    [sym_smtp_address] = ACTIONS(1253),
    [sym_smtp_port_number] = ACTIONS(1253),
    [sym_smtp_username] = ACTIONS(1253),
    [sym_smtp_password] = ACTIONS(1253),
    [sym_smtp_require_tls] = ACTIONS(1253),
    [sym_smtp_from] = ACTIONS(1253),
    [sym_smtp_tls_true] = ACTIONS(1253),
    [sym_smtp_tls_false] = ACTIONS(1253),
    [sym_listview_on_row_select_event] = ACTIONS(1253),
    [sym_data_structure_unknown] = ACTIONS(1253),
    [sym_data_structure_datafeed] = ACTIONS(1253),
    [sym_data_structure_datarecordset] = ACTIONS(1253),
    [sym_data_structure_datarecord] = ACTIONS(1253),
    [sym_data_structure_datafield] = ACTIONS(1253),
    [sym_data_max_content_field_count] = ACTIONS(1253),
    [sym_data_max_metadata_field_count] = ACTIONS(1253),
    [sym_data_max_id_length] = ACTIONS(1253),
    [sym_data_max_name_length] = ACTIONS(1253),
    [sym_data_max_description_length] = ACTIONS(1253),
    [sym_data_max_type_length] = ACTIONS(1253),
    [sym_data_max_format_length] = ACTIONS(1253),
    [sym_data_max_source_length] = ACTIONS(1253),
    [sym_data_max_value_length] = ACTIONS(1253),
    [sym_data_max_label_length] = ACTIONS(1253),
    [sym_data_max_url_length] = ACTIONS(1253),
    [sym_max_last_login_info_length] = ACTIONS(1253),
    [sym_data_type_unknown] = ACTIONS(1253),
    [sym_data_type_string] = ACTIONS(1253),
    [sym_data_type_datetime] = ACTIONS(1253),
    [sym_data_type_date] = ACTIONS(1253),
    [sym_data_type_time] = ACTIONS(1253),
    [sym_data_type_image] = ACTIONS(1253),
    [sym_data_format_url] = ACTIONS(1253),
    [sym_data_format_phone] = ACTIONS(1253),
    [sym_data_format_email] = ACTIONS(1253),
    [sym_data_format_iso8601] = ACTIONS(1253),
    [sym_auditlog_priv] = ACTIONS(1253),
    [sym_configuration_priv] = ACTIONS(1253),
    [sym_deviceconfig_priv] = ACTIONS(1253),
    [sym_ftp_priv] = ACTIONS(1253),
    [sym_http_priv] = ACTIONS(1253),
    [sym_networkconfig_priv] = ACTIONS(1253),
    [sym_programport_priv] = ACTIONS(1253),
    [sym_remoteui_priv] = ACTIONS(1253),
    [sym_securitycontrol_priv] = ACTIONS(1253),
    [sym_softwaremanagement_priv] = ACTIONS(1253),
    [sym_terminal_priv] = ACTIONS(1253),
    [sym_tpadmin_priv] = ACTIONS(1253),
    [sym_user1_priv] = ACTIONS(1253),
    [sym_user2_priv] = ACTIONS(1253),
    [sym_user3_priv] = ACTIONS(1253),
    [sym_user4_priv] = ACTIONS(1253),
    [sym_usermanagement_priv] = ACTIONS(1253),
    [sym_valid_account] = ACTIONS(1253),
    [sym_err_name_invalid] = ACTIONS(1253),
    [sym_err_password_invalid] = ACTIONS(1253),
    [sym_err_invalid_account] = ACTIONS(1253),
    [sym_err_no_authorization] = ACTIONS(1253),
    [sym_err_invalid_parameter] = ACTIONS(1253),
    [sym_err_user_locked_out] = ACTIONS(1253),
    [sym_err_user_account_expired] = ACTIONS(1253),
    [sym_err_general_error] = ACTIONS(1253),
    [sym_err_authentication_error] = ACTIONS(1253),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1253),
    [sym_tls_validate_certificate] = ACTIONS(1253),
    [sym_tdata] = ACTIONS(1245),
    [sym_tchannel] = ACTIONS(1245),
    [sym_tlevel] = ACTIONS(1245),
    [sym_tbutton] = ACTIONS(1245),
    [sym_ttimeline] = ACTIONS(1245),
    [sym_tcustom] = ACTIONS(1245),
    [sym_url_struct] = ACTIONS(1245),
    [sym_dns_struct] = ACTIONS(1245),
    [sym_ip_address_struct] = ACTIONS(1245),
    [sym_dev_info_struct] = ACTIONS(1245),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1245),
    [sym_data_feed] = ACTIONS(1245),
    [sym_data_field] = ACTIONS(1245),
    [sym_data_record] = ACTIONS(1245),
    [sym_wc_data_feed] = ACTIONS(1245),
    [sym_wc_data_field] = ACTIONS(1245),
    [sym_wc_data_record] = ACTIONS(1245),
    [sym_last_login_info] = ACTIONS(1245),
    [sym____reserved___] = ACTIONS(1256),
    [sym_date] = ACTIONS(1256),
    [sym_day] = ACTIONS(1256),
    [sym_time] = ACTIONS(1256),
    [sym_ldate] = ACTIONS(1256),
    [sym_push_device] = ACTIONS(1256),
    [sym_push_channel] = ACTIONS(1256),
    [sym_push_devchan] = ACTIONS(1256),
    [sym_release_device] = ACTIONS(1256),
    [sym_release_channel] = ACTIONS(1256),
    [sym_release_devchan] = ACTIONS(1256),
    [sym_master_slot] = ACTIONS(1256),
    [sym_get_pulse_time] = ACTIONS(1256),
    [sym_get_timer] = ACTIONS(1256),
    [sym_master_sn] = ACTIONS(1256),
    [sym_system_number] = ACTIONS(1256),
    [sym_button] = ACTIONS(1256),
    [sym_dv_channel] = ACTIONS(1256),
    [sym_data] = ACTIONS(1256),
    [sym_level] = ACTIONS(1256),
    [sym_timeline] = ACTIONS(1256),
    [sym_channel] = ACTIONS(1256),
    [sym_custom] = ACTIONS(1256),
    [sym_atoi] = ACTIONS(1259),
    [sym_atol] = ACTIONS(1259),
    [sym_atof] = ACTIONS(1259),
    [sym_itoa] = ACTIONS(1259),
    [sym_itohex] = ACTIONS(1259),
    [sym_hextoi] = ACTIONS(1259),
    [sym_ftoa] = ACTIONS(1259),
    [sym_find_string] = ACTIONS(1259),
    [sym_left_string] = ACTIONS(1259),
    [sym_right_string] = ACTIONS(1259),
    [sym_mid_string] = ACTIONS(1259),
    [sym_remove_string] = ACTIONS(1259),
    [sym_get_buffer_string] = ACTIONS(1259),
    [sym_get_buffer_char] = ACTIONS(1259),
    [sym_get_multi_buffer_string] = ACTIONS(1259),
    [sym_length_string] = ACTIONS(1259),
    [sym_max_length_string] = ACTIONS(1259),
    [sym_set_length_string] = ACTIONS(1259),
    [sym_compare_string] = ACTIONS(1259),
    [sym_upper_string] = ACTIONS(1259),
    [sym_lower_string] = ACTIONS(1259),
    [sym_get_last] = ACTIONS(1259),
    [sym_date_to_day] = ACTIONS(1259),
    [sym_date_to_month] = ACTIONS(1259),
    [sym_date_to_year] = ACTIONS(1259),
    [sym_time_to_hour] = ACTIONS(1259),
    [sym_time_to_minute] = ACTIONS(1259),
    [sym_time_to_second] = ACTIONS(1259),
    [sym_day_of_week] = ACTIONS(1259),
    [sym_set_pulse_time] = ACTIONS(1259),
    [sym_set_timer] = ACTIONS(1259),
    [sym_file_open] = ACTIONS(1259),
    [sym_file_close] = ACTIONS(1259),
    [sym_file_read] = ACTIONS(1259),
    [sym_file_read_line] = ACTIONS(1259),
    [sym_file_write] = ACTIONS(1259),
    [sym_file_write_line] = ACTIONS(1259),
    [sym_file_copy] = ACTIONS(1259),
    [sym_file_delete] = ACTIONS(1259),
    [sym_file_rename] = ACTIONS(1259),
    [sym_file_dir] = ACTIONS(1259),
    [sym_file_getdir] = ACTIONS(1259),
    [sym_file_setdir] = ACTIONS(1259),
    [sym_file_create_dir] = ACTIONS(1259),
    [sym_file_remove_dir] = ACTIONS(1259),
    [sym_file_seek] = ACTIONS(1259),
    [sym_length_array] = ACTIONS(1259),
    [sym_max_length_array] = ACTIONS(1259),
    [sym_set_length_array] = ACTIONS(1259),
    [sym_ip_client_open] = ACTIONS(1259),
    [sym_ip_client_close] = ACTIONS(1259),
    [sym_ip_server_open] = ACTIONS(1259),
    [sym_ip_server_close] = ACTIONS(1259),
    [sym_ip_bound_client_open] = ACTIONS(1259),
    [sym_ip_mc_server_open] = ACTIONS(1259),
    [sym_ip_set_option] = ACTIONS(1259),
    [sym_ssh_client_open] = ACTIONS(1259),
    [sym_ssh_client_close] = ACTIONS(1259),
    [sym_tls_client_open] = ACTIONS(1259),
    [sym_tls_client_close] = ACTIONS(1259),
    [sym_do_push] = ACTIONS(1259),
    [sym_do_push_timed] = ACTIONS(1259),
    [sym_do_release] = ACTIONS(1259),
    [sym_device_id] = ACTIONS(1259),
    [sym_device_info] = ACTIONS(1259),
    [sym_device_id_string] = ACTIONS(1259),
    [sym_device_standby] = ACTIONS(1259),
    [sym_device_wake] = ACTIONS(1259),
    [sym_timeline_create] = ACTIONS(1259),
    [sym_timeline_kill] = ACTIONS(1259),
    [sym_timeline_reload] = ACTIONS(1259),
    [sym_timeline_pause] = ACTIONS(1259),
    [sym_timeline_restart] = ACTIONS(1259),
    [sym_timeline_set] = ACTIONS(1259),
    [sym_timeline_get] = ACTIONS(1259),
    [sym_timeline_active] = ACTIONS(1259),
    [sym_timeline_dynamic_id] = ACTIONS(1259),
    [sym_variable_to_string] = ACTIONS(1259),
    [sym_string_to_variable] = ACTIONS(1259),
    [sym_length_variable_to_string] = ACTIONS(1259),
    [sym_type_cast] = ACTIONS(1259),
    [sym_variable_to_xml] = ACTIONS(1259),
    [sym_xml_to_variable] = ACTIONS(1259),
    [sym_length_variable_to_xml] = ACTIONS(1259),
    [sym_redirect_string] = ACTIONS(1259),
    [sym_random_number] = ACTIONS(1259),
    [sym_get_unique_id] = ACTIONS(1259),
    [sym_get_system_number] = ACTIONS(1259),
    [sym_set_system_number] = ACTIONS(1259),
    [sym_get_serial_number] = ACTIONS(1259),
    [sym_get_master_build] = ACTIONS(1259),
    [sym_set_validation_code] = ACTIONS(1259),
    [sym_reboot] = ACTIONS(1259),
    [sym_load_duet_module] = ACTIONS(1259),
    [sym_unload_duet_module] = ACTIONS(1259),
    [sym_duet_mem_size_set] = ACTIONS(1259),
    [sym_duet_mem_size_get] = ACTIONS(1259),
    [sym_dynamic_application_device] = ACTIONS(1259),
    [sym_dynamic_polled_port] = ACTIONS(1259),
    [sym_static_port_binding] = ACTIONS(1259),
    [sym_static_ip_binding] = ACTIONS(1259),
    [sym_get_dns_list] = ACTIONS(1259),
    [sym_set_dns_list] = ACTIONS(1259),
    [sym_get_ip_address] = ACTIONS(1259),
    [sym_set_ip_address] = ACTIONS(1259),
    [sym_get_url_list] = ACTIONS(1259),
    [sym_add_url_entry] = ACTIONS(1259),
    [sym_delete_url_entry] = ACTIONS(1259),
    [sym_abs_value] = ACTIONS(1259),
    [sym_max_value] = ACTIONS(1259),
    [sym_min_value] = ACTIONS(1259),
    [sym_exp_value] = ACTIONS(1259),
    [sym_log_value] = ACTIONS(1259),
    [sym_log10_value] = ACTIONS(1259),
    [sym_power_value] = ACTIONS(1259),
    [sym_sqrt_value] = ACTIONS(1259),
    [sym_raw_be] = ACTIONS(1259),
    [sym_raw_le] = ACTIONS(1259),
    [sym_internal_queue_size_set] = ACTIONS(1259),
    [sym_internal_queue_size_get] = ACTIONS(1259),
    [sym_internal_threshold_set] = ACTIONS(1259),
    [sym_internal_threshold_get] = ACTIONS(1259),
    [sym_rebuilt_event] = ACTIONS(1259),
    [sym_do_custom_event] = ACTIONS(1259),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1259),
    [sym_clkmgr_set_clk_source] = ACTIONS(1259),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1259),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1259),
    [sym_clkmgr_get_timezone] = ACTIONS(1259),
    [sym_clkmgr_set_timezone] = ACTIONS(1259),
    [sym_clkmgr_get_resync_period] = ACTIONS(1259),
    [sym_clkmgr_set_resync_period] = ACTIONS(1259),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1259),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1259),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1259),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1259),
    [sym_clkmgr_get_timeservers] = ACTIONS(1259),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1259),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1259),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1259),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1259),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1259),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1259),
    [sym_set_log_level] = ACTIONS(1259),
    [sym_get_log_level] = ACTIONS(1259),
    [sym_amx_log] = ACTIONS(1259),
    [sym_smtp_server_config_set] = ACTIONS(1259),
    [sym_smtp_server_config_get] = ACTIONS(1259),
    [sym_smtp_send] = ACTIONS(1259),
    [sym_validate_netlinx_account] = ACTIONS(1259),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1259),
    [sym_authenticate_certificate] = ACTIONS(1259),
    [sym_audit_netlinx_session_event] = ACTIONS(1259),
    [sym_audit_netlinx_generic_event] = ACTIONS(1259),
    [sym_get_available_flash_disk_space] = ACTIONS(1259),
    [sym_get_max_flash_disk_space] = ACTIONS(1259),
    [sym_data_create_feed] = ACTIONS(1259),
    [sym_data_delete_feed] = ACTIONS(1259),
    [sym_data_publish_feed] = ACTIONS(1259),
    [sym_data_get_published_feed] = ACTIONS(1259),
    [sym_data_add_record] = ACTIONS(1259),
    [sym_data_get_event_record] = ACTIONS(1259),
    [sym__wc_data_create_feed] = ACTIONS(1259),
    [sym__wc_data_add_record] = ACTIONS(1259),
    [sym__wc_data_get_event_record] = ACTIONS(1259),
    [sym_format] = ACTIONS(1259),
    [sym_astro_clock] = ACTIONS(1259),
    [sym_set_outdoor_temperature] = ACTIONS(1259),
    [sym_set_virtual_level_count] = ACTIONS(1259),
    [sym_set_virtual_channel_count] = ACTIONS(1259),
    [sym_set_virtual_port_count] = ACTIONS(1259),
    [sym__wc_encode_int] = ACTIONS(1259),
    [sym__wc_decode_int] = ACTIONS(1259),
    [sym__wc_int] = ACTIONS(1259),
    [sym__wc_to_ch_int] = ACTIONS(1259),
    [sym__ch_to_wc_int] = ACTIONS(1259),
    [sym__wc_find_string_int] = ACTIONS(1259),
    [sym__wc_left_string_int] = ACTIONS(1259),
    [sym__wc_lower_string_int] = ACTIONS(1259),
    [sym__wc_mid_string_int] = ACTIONS(1259),
    [sym__wc_remove_string_int] = ACTIONS(1259),
    [sym__wc_right_string_int] = ACTIONS(1259),
    [sym__wc_upper_string_int] = ACTIONS(1259),
    [sym__wc_compare_string_int] = ACTIONS(1259),
    [sym__wc_get_buffer_char_int] = ACTIONS(1259),
    [sym__wc_get_buffer_string_int] = ACTIONS(1259),
    [sym__wc_concat_string_int] = ACTIONS(1259),
    [sym__wc_file_open_int] = ACTIONS(1259),
    [sym__wc_file_close_int] = ACTIONS(1259),
    [sym__wc_file_read_int] = ACTIONS(1259),
    [sym__wc_file_read_line_int] = ACTIONS(1259),
    [sym__wc_file_write_int] = ACTIONS(1259),
    [sym__wc_file_write_line_int] = ACTIONS(1259),
    [aux_sym_preproc_include_token1] = ACTIONS(1245),
    [aux_sym_preproc_define_token1] = ACTIONS(1262),
    [aux_sym_preproc_warn_token1] = ACTIONS(1262),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1262),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1262),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1262),
    [anon_sym_LBRACK] = ACTIONS(1264),
    [anon_sym_LBRACE] = ACTIONS(1262),
    [anon_sym_COLON] = ACTIONS(1262),
    [anon_sym_LPAREN] = ACTIONS(1267),
    [anon_sym_SEMI] = ACTIONS(1262),
    [anon_sym_BANG] = ACTIONS(1247),
    [anon_sym_TILDE] = ACTIONS(1270),
    [anon_sym_DASH] = ACTIONS(1247),
    [anon_sym_PLUS] = ACTIONS(1247),
    [anon_sym_STAR] = ACTIONS(1262),
    [anon_sym_SLASH] = ACTIONS(1245),
    [anon_sym_PERCENT] = ACTIONS(1262),
    [anon_sym_PIPE_PIPE] = ACTIONS(1262),
    [anon_sym_AMP_AMP] = ACTIONS(1262),
    [anon_sym_PIPE] = ACTIONS(1245),
    [anon_sym_CARET] = ACTIONS(1262),
    [anon_sym_AMP] = ACTIONS(1245),
    [anon_sym_EQ_EQ] = ACTIONS(1262),
    [anon_sym_BANG_EQ] = ACTIONS(1262),
    [anon_sym_LT_GT] = ACTIONS(1262),
    [anon_sym_GT] = ACTIONS(1245),
    [anon_sym_GT_EQ] = ACTIONS(1262),
    [anon_sym_LT_EQ] = ACTIONS(1262),
    [anon_sym_LT] = ACTIONS(1245),
    [anon_sym_LT_LT] = ACTIONS(1262),
    [anon_sym_GT_GT] = ACTIONS(1262),
    [anon_sym_DASH_DASH] = ACTIONS(1273),
    [anon_sym_PLUS_PLUS] = ACTIONS(1273),
    [anon_sym_DOT] = ACTIONS(1245),
    [anon_sym_DQUOTE] = ACTIONS(1276),
    [anon_sym_SQUOTE] = ACTIONS(1279),
    [sym_number_literal] = ACTIONS(1282),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1262),
  },
  [STATE(52)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(53),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(53),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_expression_statement] = STATE(53),
    [sym_if_statement] = STATE(53),
    [sym_switch_statement] = STATE(53),
    [sym_while_statement] = STATE(53),
    [sym_for_statement] = STATE(53),
    [sym_select_statement] = STATE(53),
    [sym_active_statement] = STATE(53),
    [sym_return_statement] = STATE(53),
    [sym_break_statement] = STATE(53),
    [sym_continue_statement] = STATE(53),
    [sym_devchan_operation_statement] = STATE(53),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(53),
    [sym_send_command_statement] = STATE(53),
    [sym_send_level_statement] = STATE(53),
    [sym_create_buffer_statement] = STATE(53),
    [sym_create_multi_buffer_statement] = STATE(53),
    [sym_clear_buffer_statement] = STATE(53),
    [sym_wait_statement] = STATE(53),
    [sym_wait_until_statement] = STATE(53),
    [sym_cancel_all_wait_statement] = STATE(53),
    [sym_cancel_all_wait_until_statement] = STATE(53),
    [sym_cancel_wait_statement] = STATE(53),
    [sym_cancel_wait_until_statement] = STATE(53),
    [sym_call_statement] = STATE(53),
    [sym_system_call_statement] = STATE(53),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(53),
    [sym_identifier] = ACTIONS(7),
    [sym_define_call_keyword] = ACTIONS(751),
    [sym_define_function_keyword] = ACTIONS(751),
    [sym_define_library_function_keyword] = ACTIONS(751),
    [sym_define_module_keyword] = ACTIONS(751),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(986),
    [sym_else_keyword] = ACTIONS(751),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(751),
    [sym_default_keyword] = ACTIONS(751),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [sym_struct_keyword] = ACTIONS(751),
    [sym_structure_keyword] = ACTIONS(751),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(751),
    [aux_sym_preproc_define_token1] = ACTIONS(753),
    [aux_sym_preproc_warn_token1] = ACTIONS(753),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(753),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(753),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(753),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(753),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(53)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(56),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(56),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_expression_statement] = STATE(56),
    [sym_if_statement] = STATE(56),
    [sym_switch_statement] = STATE(56),
    [sym_while_statement] = STATE(56),
    [sym_for_statement] = STATE(56),
    [sym_select_statement] = STATE(56),
    [sym_active_statement] = STATE(56),
    [sym_return_statement] = STATE(56),
    [sym_break_statement] = STATE(56),
    [sym_continue_statement] = STATE(56),
    [sym_devchan_operation_statement] = STATE(56),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(56),
    [sym_send_command_statement] = STATE(56),
    [sym_send_level_statement] = STATE(56),
    [sym_create_buffer_statement] = STATE(56),
    [sym_create_multi_buffer_statement] = STATE(56),
    [sym_clear_buffer_statement] = STATE(56),
    [sym_wait_statement] = STATE(56),
    [sym_wait_until_statement] = STATE(56),
    [sym_cancel_all_wait_statement] = STATE(56),
    [sym_cancel_all_wait_until_statement] = STATE(56),
    [sym_cancel_wait_statement] = STATE(56),
    [sym_cancel_wait_until_statement] = STATE(56),
    [sym_call_statement] = STATE(56),
    [sym_system_call_statement] = STATE(56),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(56),
    [sym_identifier] = ACTIONS(7),
    [sym_define_call_keyword] = ACTIONS(759),
    [sym_define_function_keyword] = ACTIONS(759),
    [sym_define_library_function_keyword] = ACTIONS(759),
    [sym_define_module_keyword] = ACTIONS(759),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(986),
    [sym_else_keyword] = ACTIONS(759),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(759),
    [sym_default_keyword] = ACTIONS(759),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [sym_struct_keyword] = ACTIONS(759),
    [sym_structure_keyword] = ACTIONS(759),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(759),
    [aux_sym_preproc_define_token1] = ACTIONS(761),
    [aux_sym_preproc_warn_token1] = ACTIONS(761),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(761),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(761),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(761),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(761),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(54)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(55),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(55),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_expression_statement] = STATE(55),
    [sym_if_statement] = STATE(55),
    [sym_switch_statement] = STATE(55),
    [sym_while_statement] = STATE(55),
    [sym_for_statement] = STATE(55),
    [sym_select_statement] = STATE(55),
    [sym_active_statement] = STATE(55),
    [sym_return_statement] = STATE(55),
    [sym_break_statement] = STATE(55),
    [sym_continue_statement] = STATE(55),
    [sym_devchan_operation_statement] = STATE(55),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(55),
    [sym_send_command_statement] = STATE(55),
    [sym_send_level_statement] = STATE(55),
    [sym_create_buffer_statement] = STATE(55),
    [sym_create_multi_buffer_statement] = STATE(55),
    [sym_clear_buffer_statement] = STATE(55),
    [sym_wait_statement] = STATE(55),
    [sym_wait_until_statement] = STATE(55),
    [sym_cancel_all_wait_statement] = STATE(55),
    [sym_cancel_all_wait_until_statement] = STATE(55),
    [sym_cancel_wait_statement] = STATE(55),
    [sym_cancel_wait_until_statement] = STATE(55),
    [sym_call_statement] = STATE(55),
    [sym_system_call_statement] = STATE(55),
    [sym_expression] = STATE(1124),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(55),
    [sym_identifier] = ACTIONS(7),
    [sym_define_call_keyword] = ACTIONS(755),
    [sym_define_function_keyword] = ACTIONS(755),
    [sym_define_library_function_keyword] = ACTIONS(755),
    [sym_define_module_keyword] = ACTIONS(755),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(986),
    [sym_else_keyword] = ACTIONS(755),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(755),
    [sym_default_keyword] = ACTIONS(755),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [sym_struct_keyword] = ACTIONS(755),
    [sym_structure_keyword] = ACTIONS(755),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(755),
    [aux_sym_preproc_define_token1] = ACTIONS(757),
    [aux_sym_preproc_warn_token1] = ACTIONS(757),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(757),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(757),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(757),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(757),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(55)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(56),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(56),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_expression_statement] = STATE(56),
    [sym_if_statement] = STATE(56),
    [sym_switch_statement] = STATE(56),
    [sym_while_statement] = STATE(56),
    [sym_for_statement] = STATE(56),
    [sym_select_statement] = STATE(56),
    [sym_active_statement] = STATE(56),
    [sym_return_statement] = STATE(56),
    [sym_break_statement] = STATE(56),
    [sym_continue_statement] = STATE(56),
    [sym_devchan_operation_statement] = STATE(56),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(56),
    [sym_send_command_statement] = STATE(56),
    [sym_send_level_statement] = STATE(56),
    [sym_create_buffer_statement] = STATE(56),
    [sym_create_multi_buffer_statement] = STATE(56),
    [sym_clear_buffer_statement] = STATE(56),
    [sym_wait_statement] = STATE(56),
    [sym_wait_until_statement] = STATE(56),
    [sym_cancel_all_wait_statement] = STATE(56),
    [sym_cancel_all_wait_until_statement] = STATE(56),
    [sym_cancel_wait_statement] = STATE(56),
    [sym_cancel_wait_until_statement] = STATE(56),
    [sym_call_statement] = STATE(56),
    [sym_system_call_statement] = STATE(56),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(56),
    [sym_identifier] = ACTIONS(7),
    [sym_define_call_keyword] = ACTIONS(763),
    [sym_define_function_keyword] = ACTIONS(763),
    [sym_define_library_function_keyword] = ACTIONS(763),
    [sym_define_module_keyword] = ACTIONS(763),
    [sym_char_keyword] = ACTIONS(47),
    [sym_widechar_keyword] = ACTIONS(47),
    [sym_integer_keyword] = ACTIONS(47),
    [sym_sinteger_keyword] = ACTIONS(47),
    [sym_long_keyword] = ACTIONS(47),
    [sym_slong_keyword] = ACTIONS(47),
    [sym_float_keyword] = ACTIONS(47),
    [sym_double_keyword] = ACTIONS(47),
    [sym_variant_keyword] = ACTIONS(47),
    [sym_variantarray_keyword] = ACTIONS(47),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(53),
    [sym_devlev_keyword] = ACTIONS(53),
    [sym_devchan_keyword] = ACTIONS(53),
    [sym_if_keyword] = ACTIONS(986),
    [sym_else_keyword] = ACTIONS(763),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(763),
    [sym_default_keyword] = ACTIONS(763),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [sym_struct_keyword] = ACTIONS(763),
    [sym_structure_keyword] = ACTIONS(763),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym_tdata] = ACTIONS(135),
    [sym_tchannel] = ACTIONS(135),
    [sym_tlevel] = ACTIONS(135),
    [sym_tbutton] = ACTIONS(135),
    [sym_ttimeline] = ACTIONS(135),
    [sym_tcustom] = ACTIONS(135),
    [sym_url_struct] = ACTIONS(135),
    [sym_dns_struct] = ACTIONS(135),
    [sym_ip_address_struct] = ACTIONS(135),
    [sym_dev_info_struct] = ACTIONS(135),
    [sym_clkmgr_timeserver_struct] = ACTIONS(135),
    [sym_data_feed] = ACTIONS(135),
    [sym_data_field] = ACTIONS(135),
    [sym_data_record] = ACTIONS(135),
    [sym_wc_data_feed] = ACTIONS(135),
    [sym_wc_data_field] = ACTIONS(135),
    [sym_wc_data_record] = ACTIONS(135),
    [sym_last_login_info] = ACTIONS(135),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_include_token1] = ACTIONS(763),
    [aux_sym_preproc_define_token1] = ACTIONS(765),
    [aux_sym_preproc_warn_token1] = ACTIONS(765),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(765),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(765),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(765),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(765),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(56)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(56),
    [sym_storage_class_specifier] = STATE(1010),
    [sym_type_qualifier] = STATE(1011),
    [sym_type_specifier] = STATE(1371),
    [sym_intrinsic_type] = STATE(1440),
    [sym_primitive_type] = STATE(1444),
    [sym_structured_type] = STATE(1444),
    [sym_declaration] = STATE(56),
    [sym__declaration_modifiers] = STATE(1008),
    [sym__declaration_specifiers] = STATE(1454),
    [sym_expression_statement] = STATE(56),
    [sym_if_statement] = STATE(56),
    [sym_switch_statement] = STATE(56),
    [sym_while_statement] = STATE(56),
    [sym_for_statement] = STATE(56),
    [sym_select_statement] = STATE(56),
    [sym_active_statement] = STATE(56),
    [sym_return_statement] = STATE(56),
    [sym_break_statement] = STATE(56),
    [sym_continue_statement] = STATE(56),
    [sym_devchan_operation_statement] = STATE(56),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(56),
    [sym_send_command_statement] = STATE(56),
    [sym_send_level_statement] = STATE(56),
    [sym_create_buffer_statement] = STATE(56),
    [sym_create_multi_buffer_statement] = STATE(56),
    [sym_clear_buffer_statement] = STATE(56),
    [sym_wait_statement] = STATE(56),
    [sym_wait_until_statement] = STATE(56),
    [sym_cancel_all_wait_statement] = STATE(56),
    [sym_cancel_all_wait_until_statement] = STATE(56),
    [sym_cancel_wait_statement] = STATE(56),
    [sym_cancel_wait_until_statement] = STATE(56),
    [sym_call_statement] = STATE(56),
    [sym_system_call_statement] = STATE(56),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_system_type] = STATE(1440),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1008),
    [aux_sym_case_statement_repeat1] = STATE(56),
    [sym_identifier] = ACTIONS(767),
    [sym_define_call_keyword] = ACTIONS(770),
    [sym_define_function_keyword] = ACTIONS(770),
    [sym_define_library_function_keyword] = ACTIONS(770),
    [sym_define_module_keyword] = ACTIONS(770),
    [sym_char_keyword] = ACTIONS(772),
    [sym_widechar_keyword] = ACTIONS(772),
    [sym_integer_keyword] = ACTIONS(772),
    [sym_sinteger_keyword] = ACTIONS(772),
    [sym_long_keyword] = ACTIONS(772),
    [sym_slong_keyword] = ACTIONS(772),
    [sym_float_keyword] = ACTIONS(772),
    [sym_double_keyword] = ACTIONS(772),
    [sym_variant_keyword] = ACTIONS(772),
    [sym_variantarray_keyword] = ACTIONS(772),
    [sym_local_var_keyword] = ACTIONS(775),
    [sym_stack_var_keyword] = ACTIONS(775),
    [sym_constant_keyword] = ACTIONS(778),
    [sym_volatile_keyword] = ACTIONS(778),
    [sym_non_volatile_keyword] = ACTIONS(778),
    [sym_persistent_keyword] = ACTIONS(778),
    [sym_dev_keyword] = ACTIONS(781),
    [sym_devlev_keyword] = ACTIONS(781),
    [sym_devchan_keyword] = ACTIONS(781),
    [sym_if_keyword] = ACTIONS(1285),
    [sym_else_keyword] = ACTIONS(770),
    [sym_switch_keyword] = ACTIONS(1288),
    [sym_case_keyword] = ACTIONS(770),
    [sym_default_keyword] = ACTIONS(770),
    [sym_while_keyword] = ACTIONS(1291),
    [sym_for_keyword] = ACTIONS(1294),
    [sym_break_keyword] = ACTIONS(1297),
    [sym_continue_keyword] = ACTIONS(1300),
    [sym_return_keyword] = ACTIONS(1303),
    [sym_select_keyword] = ACTIONS(1306),
    [sym_active_keyword] = ACTIONS(1309),
    [sym_struct_keyword] = ACTIONS(770),
    [sym_structure_keyword] = ACTIONS(770),
    [aux_sym_on_keyword_token1] = ACTIONS(811),
    [aux_sym_off_keyword_token1] = ACTIONS(814),
    [sym_send_level_keyword] = ACTIONS(1312),
    [sym_send_string_keyword] = ACTIONS(1315),
    [sym_send_command_keyword] = ACTIONS(1318),
    [sym_clear_buffer_keyword] = ACTIONS(1321),
    [sym_create_buffer_keyword] = ACTIONS(1324),
    [sym_create_multi_buffer_keyword] = ACTIONS(1327),
    [sym_call_keyword] = ACTIONS(1330),
    [sym_system_call_keyword] = ACTIONS(1333),
    [sym_devchan_to_keyword] = ACTIONS(841),
    [sym_devchan_min_to_keyword] = ACTIONS(841),
    [sym_devchan_total_off_keyword] = ACTIONS(841),
    [sym_devchan_pulse_keyword] = ACTIONS(841),
    [sym_wait_keyword] = ACTIONS(1336),
    [sym_cancel_wait_keyword] = ACTIONS(1339),
    [sym_wait_until_keyword] = ACTIONS(1342),
    [sym_cancel_wait_until_keyword] = ACTIONS(1345),
    [sym_cancel_all_wait_keyword] = ACTIONS(1348),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1351),
    [sym_not] = ACTIONS(862),
    [sym_bnot] = ACTIONS(862),
    [sym___file__] = ACTIONS(865),
    [sym___date__] = ACTIONS(865),
    [sym___ldate__] = ACTIONS(865),
    [sym___time__] = ACTIONS(865),
    [sym___line__] = ACTIONS(865),
    [sym___name__] = ACTIONS(865),
    [sym___netlinx__] = ACTIONS(868),
    [sym_netlinx_axi_version] = ACTIONS(868),
    [sym_url_flg_tcp] = ACTIONS(868),
    [sym_url_flg_accinfopresent] = ACTIONS(868),
    [sym_url_flg_temp] = ACTIONS(868),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(868),
    [sym_url_flg_stat_mask] = ACTIONS(868),
    [sym_url_flg_stat_lookup] = ACTIONS(868),
    [sym_url_flg_stat_connecting] = ACTIONS(868),
    [sym_url_flg_stat_waiting] = ACTIONS(868),
    [sym_url_flg_stat_connected] = ACTIONS(868),
    [sym_ip_addr_flg_dhcp] = ACTIONS(868),
    [sym_first_virtual_device] = ACTIONS(868),
    [sym_first_local_port] = ACTIONS(868),
    [sym_dynamic_virtual_device] = ACTIONS(868),
    [sym_timeline_once] = ACTIONS(868),
    [sym_timeline_repeat] = ACTIONS(868),
    [sym_timeline_absolute] = ACTIONS(868),
    [sym_timeline_relative] = ACTIONS(868),
    [sym_do_push_timed_infinite] = ACTIONS(868),
    [sym_source_type_no_address] = ACTIONS(868),
    [sym_source_type_neuron_id] = ACTIONS(868),
    [sym_source_type_ip_address] = ACTIONS(868),
    [sym_source_type_axlink] = ACTIONS(868),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(868),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(868),
    [sym_source_type_ip_socket_address] = ACTIONS(868),
    [sym_source_type_rs232] = ACTIONS(868),
    [sym_source_type_internal] = ACTIONS(868),
    [sym_source_type_mac_address] = ACTIONS(868),
    [sym_source_type_ipv4_port] = ACTIONS(868),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(868),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(868),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(868),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(868),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(868),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(868),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(868),
    [sym_true] = ACTIONS(868),
    [sym_false] = ACTIONS(868),
    [sym_file_read_only] = ACTIONS(868),
    [sym_file_rw_new] = ACTIONS(868),
    [sym_file_rw_append] = ACTIONS(868),
    [sym_ip_tcp] = ACTIONS(868),
    [sym_ip_udp] = ACTIONS(868),
    [sym_ip_udp_2way] = ACTIONS(868),
    [sym_xml_encode_types] = ACTIONS(868),
    [sym_xml_encode_char_as_list] = ACTIONS(868),
    [sym_xml_encode_le] = ACTIONS(868),
    [sym_xml_decode_types] = ACTIONS(868),
    [sym_xml_decode_no_preserve] = ACTIONS(868),
    [sym_duet_dev_type_display_device] = ACTIONS(868),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(868),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(868),
    [sym_duet_dev_type_io_device] = ACTIONS(868),
    [sym_duet_dev_type_relay_device] = ACTIONS(868),
    [sym_duet_dev_type_ups] = ACTIONS(868),
    [sym_duet_dev_type_amplifier] = ACTIONS(868),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(868),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(868),
    [sym_duet_dev_type_audio_processor] = ACTIONS(868),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(868),
    [sym_duet_dev_type_camera] = ACTIONS(868),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(868),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(868),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(868),
    [sym_duet_dev_type_dss] = ACTIONS(868),
    [sym_duet_dev_type_dvr] = ACTIONS(868),
    [sym_duet_dev_type_disc_device] = ACTIONS(868),
    [sym_duet_dev_type_document_camera] = ACTIONS(868),
    [sym_duet_dev_type_audio_tape] = ACTIONS(868),
    [sym_duet_dev_type_hvac] = ACTIONS(868),
    [sym_duet_dev_type_keypad] = ACTIONS(868),
    [sym_duet_dev_type_light] = ACTIONS(868),
    [sym_duet_dev_type_monitor] = ACTIONS(868),
    [sym_duet_dev_type_motor] = ACTIONS(868),
    [sym_duet_dev_type_multi_window] = ACTIONS(868),
    [sym_duet_dev_type_pool_spa] = ACTIONS(868),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(868),
    [sym_duet_dev_type_receiver] = ACTIONS(868),
    [sym_duet_dev_type_security_system] = ACTIONS(868),
    [sym_duet_dev_type_sensor_device] = ACTIONS(868),
    [sym_duet_dev_type_settop_box] = ACTIONS(868),
    [sym_duet_dev_type_slide_projector] = ACTIONS(868),
    [sym_duet_dev_type_switcher] = ACTIONS(868),
    [sym_duet_dev_type_text_keypad] = ACTIONS(868),
    [sym_duet_dev_type_tv] = ACTIONS(868),
    [sym_duet_dev_type_utility] = ACTIONS(868),
    [sym_duet_dev_type_vcr] = ACTIONS(868),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(868),
    [sym_duet_dev_type_video_processor] = ACTIONS(868),
    [sym_duet_dev_type_video_projector] = ACTIONS(868),
    [sym_duet_dev_type_video_wall] = ACTIONS(868),
    [sym_duet_dev_type_volume_controller] = ACTIONS(868),
    [sym_duet_dev_type_weather] = ACTIONS(868),
    [sym_duet_dev_not_polled] = ACTIONS(868),
    [sym_duet_dev_polled] = ACTIONS(868),
    [sym_clkmgr_mode_network] = ACTIONS(868),
    [sym_clkmgr_mode_standalone] = ACTIONS(868),
    [sym_normal_standby] = ACTIONS(868),
    [sym_normal_wake] = ACTIONS(868),
    [sym_ip_multicast_ttl_option] = ACTIONS(868),
    [sym_ip_ttl_subnet] = ACTIONS(868),
    [sym_ip_ttl_site] = ACTIONS(868),
    [sym_ip_ttl_region] = ACTIONS(868),
    [sym_ip_ttl_continent] = ACTIONS(868),
    [sym_ip_tcp_nodelay_option] = ACTIONS(868),
    [sym_ip_nodelay_on] = ACTIONS(868),
    [sym_ip_nodelay_off] = ACTIONS(868),
    [sym_amx_error] = ACTIONS(868),
    [sym_amx_warning] = ACTIONS(868),
    [sym_amx_info] = ACTIONS(868),
    [sym_amx_debug] = ACTIONS(868),
    [sym_netlinx_login_success] = ACTIONS(868),
    [sym_netlinx_login_fail] = ACTIONS(868),
    [sym_netlinx_logout] = ACTIONS(868),
    [sym_null_str] = ACTIONS(868),
    [sym_smtp_address] = ACTIONS(868),
    [sym_smtp_port_number] = ACTIONS(868),
    [sym_smtp_username] = ACTIONS(868),
    [sym_smtp_password] = ACTIONS(868),
    [sym_smtp_require_tls] = ACTIONS(868),
    [sym_smtp_from] = ACTIONS(868),
    [sym_smtp_tls_true] = ACTIONS(868),
    [sym_smtp_tls_false] = ACTIONS(868),
    [sym_listview_on_row_select_event] = ACTIONS(868),
    [sym_data_structure_unknown] = ACTIONS(868),
    [sym_data_structure_datafeed] = ACTIONS(868),
    [sym_data_structure_datarecordset] = ACTIONS(868),
    [sym_data_structure_datarecord] = ACTIONS(868),
    [sym_data_structure_datafield] = ACTIONS(868),
    [sym_data_max_content_field_count] = ACTIONS(868),
    [sym_data_max_metadata_field_count] = ACTIONS(868),
    [sym_data_max_id_length] = ACTIONS(868),
    [sym_data_max_name_length] = ACTIONS(868),
    [sym_data_max_description_length] = ACTIONS(868),
    [sym_data_max_type_length] = ACTIONS(868),
    [sym_data_max_format_length] = ACTIONS(868),
    [sym_data_max_source_length] = ACTIONS(868),
    [sym_data_max_value_length] = ACTIONS(868),
    [sym_data_max_label_length] = ACTIONS(868),
    [sym_data_max_url_length] = ACTIONS(868),
    [sym_max_last_login_info_length] = ACTIONS(868),
    [sym_data_type_unknown] = ACTIONS(868),
    [sym_data_type_string] = ACTIONS(868),
    [sym_data_type_datetime] = ACTIONS(868),
    [sym_data_type_date] = ACTIONS(868),
    [sym_data_type_time] = ACTIONS(868),
    [sym_data_type_image] = ACTIONS(868),
    [sym_data_format_url] = ACTIONS(868),
    [sym_data_format_phone] = ACTIONS(868),
    [sym_data_format_email] = ACTIONS(868),
    [sym_data_format_iso8601] = ACTIONS(868),
    [sym_auditlog_priv] = ACTIONS(868),
    [sym_configuration_priv] = ACTIONS(868),
    [sym_deviceconfig_priv] = ACTIONS(868),
    [sym_ftp_priv] = ACTIONS(868),
    [sym_http_priv] = ACTIONS(868),
    [sym_networkconfig_priv] = ACTIONS(868),
    [sym_programport_priv] = ACTIONS(868),
    [sym_remoteui_priv] = ACTIONS(868),
    [sym_securitycontrol_priv] = ACTIONS(868),
    [sym_softwaremanagement_priv] = ACTIONS(868),
    [sym_terminal_priv] = ACTIONS(868),
    [sym_tpadmin_priv] = ACTIONS(868),
    [sym_user1_priv] = ACTIONS(868),
    [sym_user2_priv] = ACTIONS(868),
    [sym_user3_priv] = ACTIONS(868),
    [sym_user4_priv] = ACTIONS(868),
    [sym_usermanagement_priv] = ACTIONS(868),
    [sym_valid_account] = ACTIONS(868),
    [sym_err_name_invalid] = ACTIONS(868),
    [sym_err_password_invalid] = ACTIONS(868),
    [sym_err_invalid_account] = ACTIONS(868),
    [sym_err_no_authorization] = ACTIONS(868),
    [sym_err_invalid_parameter] = ACTIONS(868),
    [sym_err_user_locked_out] = ACTIONS(868),
    [sym_err_user_account_expired] = ACTIONS(868),
    [sym_err_general_error] = ACTIONS(868),
    [sym_err_authentication_error] = ACTIONS(868),
    [sym_tls_ignore_certificate_errors] = ACTIONS(868),
    [sym_tls_validate_certificate] = ACTIONS(868),
    [sym_tdata] = ACTIONS(871),
    [sym_tchannel] = ACTIONS(871),
    [sym_tlevel] = ACTIONS(871),
    [sym_tbutton] = ACTIONS(871),
    [sym_ttimeline] = ACTIONS(871),
    [sym_tcustom] = ACTIONS(871),
    [sym_url_struct] = ACTIONS(871),
    [sym_dns_struct] = ACTIONS(871),
    [sym_ip_address_struct] = ACTIONS(871),
    [sym_dev_info_struct] = ACTIONS(871),
    [sym_clkmgr_timeserver_struct] = ACTIONS(871),
    [sym_data_feed] = ACTIONS(871),
    [sym_data_field] = ACTIONS(871),
    [sym_data_record] = ACTIONS(871),
    [sym_wc_data_feed] = ACTIONS(871),
    [sym_wc_data_field] = ACTIONS(871),
    [sym_wc_data_record] = ACTIONS(871),
    [sym_last_login_info] = ACTIONS(871),
    [sym____reserved___] = ACTIONS(874),
    [sym_date] = ACTIONS(874),
    [sym_day] = ACTIONS(874),
    [sym_time] = ACTIONS(874),
    [sym_ldate] = ACTIONS(874),
    [sym_push_device] = ACTIONS(874),
    [sym_push_channel] = ACTIONS(874),
    [sym_push_devchan] = ACTIONS(874),
    [sym_release_device] = ACTIONS(874),
    [sym_release_channel] = ACTIONS(874),
    [sym_release_devchan] = ACTIONS(874),
    [sym_master_slot] = ACTIONS(874),
    [sym_get_pulse_time] = ACTIONS(874),
    [sym_get_timer] = ACTIONS(874),
    [sym_master_sn] = ACTIONS(874),
    [sym_system_number] = ACTIONS(874),
    [sym_button] = ACTIONS(874),
    [sym_dv_channel] = ACTIONS(874),
    [sym_data] = ACTIONS(874),
    [sym_level] = ACTIONS(874),
    [sym_timeline] = ACTIONS(874),
    [sym_channel] = ACTIONS(874),
    [sym_custom] = ACTIONS(874),
    [sym_atoi] = ACTIONS(877),
    [sym_atol] = ACTIONS(877),
    [sym_atof] = ACTIONS(877),
    [sym_itoa] = ACTIONS(877),
    [sym_itohex] = ACTIONS(877),
    [sym_hextoi] = ACTIONS(877),
    [sym_ftoa] = ACTIONS(877),
    [sym_find_string] = ACTIONS(877),
    [sym_left_string] = ACTIONS(877),
    [sym_right_string] = ACTIONS(877),
    [sym_mid_string] = ACTIONS(877),
    [sym_remove_string] = ACTIONS(877),
    [sym_get_buffer_string] = ACTIONS(877),
    [sym_get_buffer_char] = ACTIONS(877),
    [sym_get_multi_buffer_string] = ACTIONS(877),
    [sym_length_string] = ACTIONS(877),
    [sym_max_length_string] = ACTIONS(877),
    [sym_set_length_string] = ACTIONS(877),
    [sym_compare_string] = ACTIONS(877),
    [sym_upper_string] = ACTIONS(877),
    [sym_lower_string] = ACTIONS(877),
    [sym_get_last] = ACTIONS(877),
    [sym_date_to_day] = ACTIONS(877),
    [sym_date_to_month] = ACTIONS(877),
    [sym_date_to_year] = ACTIONS(877),
    [sym_time_to_hour] = ACTIONS(877),
    [sym_time_to_minute] = ACTIONS(877),
    [sym_time_to_second] = ACTIONS(877),
    [sym_day_of_week] = ACTIONS(877),
    [sym_set_pulse_time] = ACTIONS(877),
    [sym_set_timer] = ACTIONS(877),
    [sym_file_open] = ACTIONS(877),
    [sym_file_close] = ACTIONS(877),
    [sym_file_read] = ACTIONS(877),
    [sym_file_read_line] = ACTIONS(877),
    [sym_file_write] = ACTIONS(877),
    [sym_file_write_line] = ACTIONS(877),
    [sym_file_copy] = ACTIONS(877),
    [sym_file_delete] = ACTIONS(877),
    [sym_file_rename] = ACTIONS(877),
    [sym_file_dir] = ACTIONS(877),
    [sym_file_getdir] = ACTIONS(877),
    [sym_file_setdir] = ACTIONS(877),
    [sym_file_create_dir] = ACTIONS(877),
    [sym_file_remove_dir] = ACTIONS(877),
    [sym_file_seek] = ACTIONS(877),
    [sym_length_array] = ACTIONS(877),
    [sym_max_length_array] = ACTIONS(877),
    [sym_set_length_array] = ACTIONS(877),
    [sym_ip_client_open] = ACTIONS(877),
    [sym_ip_client_close] = ACTIONS(877),
    [sym_ip_server_open] = ACTIONS(877),
    [sym_ip_server_close] = ACTIONS(877),
    [sym_ip_bound_client_open] = ACTIONS(877),
    [sym_ip_mc_server_open] = ACTIONS(877),
    [sym_ip_set_option] = ACTIONS(877),
    [sym_ssh_client_open] = ACTIONS(877),
    [sym_ssh_client_close] = ACTIONS(877),
    [sym_tls_client_open] = ACTIONS(877),
    [sym_tls_client_close] = ACTIONS(877),
    [sym_do_push] = ACTIONS(877),
    [sym_do_push_timed] = ACTIONS(877),
    [sym_do_release] = ACTIONS(877),
    [sym_device_id] = ACTIONS(877),
    [sym_device_info] = ACTIONS(877),
    [sym_device_id_string] = ACTIONS(877),
    [sym_device_standby] = ACTIONS(877),
    [sym_device_wake] = ACTIONS(877),
    [sym_timeline_create] = ACTIONS(877),
    [sym_timeline_kill] = ACTIONS(877),
    [sym_timeline_reload] = ACTIONS(877),
    [sym_timeline_pause] = ACTIONS(877),
    [sym_timeline_restart] = ACTIONS(877),
    [sym_timeline_set] = ACTIONS(877),
    [sym_timeline_get] = ACTIONS(877),
    [sym_timeline_active] = ACTIONS(877),
    [sym_timeline_dynamic_id] = ACTIONS(877),
    [sym_variable_to_string] = ACTIONS(877),
    [sym_string_to_variable] = ACTIONS(877),
    [sym_length_variable_to_string] = ACTIONS(877),
    [sym_type_cast] = ACTIONS(877),
    [sym_variable_to_xml] = ACTIONS(877),
    [sym_xml_to_variable] = ACTIONS(877),
    [sym_length_variable_to_xml] = ACTIONS(877),
    [sym_redirect_string] = ACTIONS(877),
    [sym_random_number] = ACTIONS(877),
    [sym_get_unique_id] = ACTIONS(877),
    [sym_get_system_number] = ACTIONS(877),
    [sym_set_system_number] = ACTIONS(877),
    [sym_get_serial_number] = ACTIONS(877),
    [sym_get_master_build] = ACTIONS(877),
    [sym_set_validation_code] = ACTIONS(877),
    [sym_reboot] = ACTIONS(877),
    [sym_load_duet_module] = ACTIONS(877),
    [sym_unload_duet_module] = ACTIONS(877),
    [sym_duet_mem_size_set] = ACTIONS(877),
    [sym_duet_mem_size_get] = ACTIONS(877),
    [sym_dynamic_application_device] = ACTIONS(877),
    [sym_dynamic_polled_port] = ACTIONS(877),
    [sym_static_port_binding] = ACTIONS(877),
    [sym_static_ip_binding] = ACTIONS(877),
    [sym_get_dns_list] = ACTIONS(877),
    [sym_set_dns_list] = ACTIONS(877),
    [sym_get_ip_address] = ACTIONS(877),
    [sym_set_ip_address] = ACTIONS(877),
    [sym_get_url_list] = ACTIONS(877),
    [sym_add_url_entry] = ACTIONS(877),
    [sym_delete_url_entry] = ACTIONS(877),
    [sym_abs_value] = ACTIONS(877),
    [sym_max_value] = ACTIONS(877),
    [sym_min_value] = ACTIONS(877),
    [sym_exp_value] = ACTIONS(877),
    [sym_log_value] = ACTIONS(877),
    [sym_log10_value] = ACTIONS(877),
    [sym_power_value] = ACTIONS(877),
    [sym_sqrt_value] = ACTIONS(877),
    [sym_raw_be] = ACTIONS(877),
    [sym_raw_le] = ACTIONS(877),
    [sym_internal_queue_size_set] = ACTIONS(877),
    [sym_internal_queue_size_get] = ACTIONS(877),
    [sym_internal_threshold_set] = ACTIONS(877),
    [sym_internal_threshold_get] = ACTIONS(877),
    [sym_rebuilt_event] = ACTIONS(877),
    [sym_do_custom_event] = ACTIONS(877),
    [sym_clkmgr_is_network_sourced] = ACTIONS(877),
    [sym_clkmgr_set_clk_source] = ACTIONS(877),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(877),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(877),
    [sym_clkmgr_get_timezone] = ACTIONS(877),
    [sym_clkmgr_set_timezone] = ACTIONS(877),
    [sym_clkmgr_get_resync_period] = ACTIONS(877),
    [sym_clkmgr_set_resync_period] = ACTIONS(877),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(877),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(877),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(877),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(877),
    [sym_clkmgr_get_timeservers] = ACTIONS(877),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(877),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(877),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(877),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(877),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(877),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(877),
    [sym_set_log_level] = ACTIONS(877),
    [sym_get_log_level] = ACTIONS(877),
    [sym_amx_log] = ACTIONS(877),
    [sym_smtp_server_config_set] = ACTIONS(877),
    [sym_smtp_server_config_get] = ACTIONS(877),
    [sym_smtp_send] = ACTIONS(877),
    [sym_validate_netlinx_account] = ACTIONS(877),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(877),
    [sym_authenticate_certificate] = ACTIONS(877),
    [sym_audit_netlinx_session_event] = ACTIONS(877),
    [sym_audit_netlinx_generic_event] = ACTIONS(877),
    [sym_get_available_flash_disk_space] = ACTIONS(877),
    [sym_get_max_flash_disk_space] = ACTIONS(877),
    [sym_data_create_feed] = ACTIONS(877),
    [sym_data_delete_feed] = ACTIONS(877),
    [sym_data_publish_feed] = ACTIONS(877),
    [sym_data_get_published_feed] = ACTIONS(877),
    [sym_data_add_record] = ACTIONS(877),
    [sym_data_get_event_record] = ACTIONS(877),
    [sym__wc_data_create_feed] = ACTIONS(877),
    [sym__wc_data_add_record] = ACTIONS(877),
    [sym__wc_data_get_event_record] = ACTIONS(877),
    [sym_format] = ACTIONS(877),
    [sym_astro_clock] = ACTIONS(877),
    [sym_set_outdoor_temperature] = ACTIONS(877),
    [sym_set_virtual_level_count] = ACTIONS(877),
    [sym_set_virtual_channel_count] = ACTIONS(877),
    [sym_set_virtual_port_count] = ACTIONS(877),
    [sym__wc_encode_int] = ACTIONS(877),
    [sym__wc_decode_int] = ACTIONS(877),
    [sym__wc_int] = ACTIONS(877),
    [sym__wc_to_ch_int] = ACTIONS(877),
    [sym__ch_to_wc_int] = ACTIONS(877),
    [sym__wc_find_string_int] = ACTIONS(877),
    [sym__wc_left_string_int] = ACTIONS(877),
    [sym__wc_lower_string_int] = ACTIONS(877),
    [sym__wc_mid_string_int] = ACTIONS(877),
    [sym__wc_remove_string_int] = ACTIONS(877),
    [sym__wc_right_string_int] = ACTIONS(877),
    [sym__wc_upper_string_int] = ACTIONS(877),
    [sym__wc_compare_string_int] = ACTIONS(877),
    [sym__wc_get_buffer_char_int] = ACTIONS(877),
    [sym__wc_get_buffer_string_int] = ACTIONS(877),
    [sym__wc_concat_string_int] = ACTIONS(877),
    [sym__wc_file_open_int] = ACTIONS(877),
    [sym__wc_file_close_int] = ACTIONS(877),
    [sym__wc_file_read_int] = ACTIONS(877),
    [sym__wc_file_read_line_int] = ACTIONS(877),
    [sym__wc_file_write_int] = ACTIONS(877),
    [sym__wc_file_write_line_int] = ACTIONS(877),
    [aux_sym_preproc_include_token1] = ACTIONS(770),
    [aux_sym_preproc_define_token1] = ACTIONS(880),
    [aux_sym_preproc_warn_token1] = ACTIONS(880),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(880),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(880),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(880),
    [anon_sym_LBRACK] = ACTIONS(882),
    [anon_sym_LBRACE] = ACTIONS(1354),
    [anon_sym_RBRACE] = ACTIONS(880),
    [anon_sym_LPAREN] = ACTIONS(888),
    [anon_sym_BANG] = ACTIONS(891),
    [anon_sym_TILDE] = ACTIONS(891),
    [anon_sym_DASH] = ACTIONS(862),
    [anon_sym_PLUS] = ACTIONS(862),
    [anon_sym_DASH_DASH] = ACTIONS(894),
    [anon_sym_PLUS_PLUS] = ACTIONS(894),
    [anon_sym_DQUOTE] = ACTIONS(897),
    [anon_sym_SQUOTE] = ACTIONS(900),
    [sym_number_literal] = ACTIONS(903),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(57)] = {
    [sym_argument_list] = STATE(80),
    [sym_string_literal] = STATE(198),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1357),
    [sym_program_name_keyword] = ACTIONS(1357),
    [sym_module_name_keyword] = ACTIONS(1357),
    [sym_define_device_keyword] = ACTIONS(1357),
    [sym_define_combine_keyword] = ACTIONS(1357),
    [sym_define_constant_keyword] = ACTIONS(1357),
    [sym_define_type_keyword] = ACTIONS(1357),
    [sym_define_variable_keyword] = ACTIONS(1357),
    [sym_define_system_variable_keyword] = ACTIONS(1357),
    [sym_define_connect_level_keyword] = ACTIONS(1357),
    [sym_define_latching_keyword] = ACTIONS(1357),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1357),
    [sym_define_toggling_keyword] = ACTIONS(1357),
    [sym_define_call_keyword] = ACTIONS(1357),
    [sym_define_function_keyword] = ACTIONS(1357),
    [sym_define_library_function_keyword] = ACTIONS(1357),
    [sym_define_start_keyword] = ACTIONS(1357),
    [sym_define_event_keyword] = ACTIONS(1357),
    [sym_define_program_keyword] = ACTIONS(1357),
    [sym_define_module_keyword] = ACTIONS(1357),
    [sym_char_keyword] = ACTIONS(1357),
    [sym_widechar_keyword] = ACTIONS(1357),
    [sym_integer_keyword] = ACTIONS(1357),
    [sym_sinteger_keyword] = ACTIONS(1357),
    [sym_long_keyword] = ACTIONS(1357),
    [sym_slong_keyword] = ACTIONS(1357),
    [sym_float_keyword] = ACTIONS(1357),
    [sym_double_keyword] = ACTIONS(1357),
    [sym_variant_keyword] = ACTIONS(1357),
    [sym_variantarray_keyword] = ACTIONS(1357),
    [sym_local_var_keyword] = ACTIONS(1357),
    [sym_stack_var_keyword] = ACTIONS(1357),
    [sym_constant_keyword] = ACTIONS(1357),
    [sym_volatile_keyword] = ACTIONS(1357),
    [sym_non_volatile_keyword] = ACTIONS(1357),
    [sym_persistent_keyword] = ACTIONS(1357),
    [sym_dev_keyword] = ACTIONS(1357),
    [sym_devlev_keyword] = ACTIONS(1357),
    [sym_devchan_keyword] = ACTIONS(1357),
    [sym_if_keyword] = ACTIONS(1357),
    [sym_else_keyword] = ACTIONS(1357),
    [sym_switch_keyword] = ACTIONS(1357),
    [sym_case_keyword] = ACTIONS(1357),
    [sym_default_keyword] = ACTIONS(1357),
    [sym_while_keyword] = ACTIONS(1357),
    [sym_for_keyword] = ACTIONS(1357),
    [sym_break_keyword] = ACTIONS(1357),
    [sym_continue_keyword] = ACTIONS(1357),
    [sym_return_keyword] = ACTIONS(1357),
    [sym_select_keyword] = ACTIONS(1357),
    [sym_active_keyword] = ACTIONS(1357),
    [sym_struct_keyword] = ACTIONS(1357),
    [sym_structure_keyword] = ACTIONS(1357),
    [sym_button_event_keyword] = ACTIONS(1357),
    [sym_channel_event_keyword] = ACTIONS(1357),
    [sym_level_event_keyword] = ACTIONS(1357),
    [sym_data_event_keyword] = ACTIONS(1357),
    [sym_timeline_event_keyword] = ACTIONS(1357),
    [sym_custom_event_keyword] = ACTIONS(1357),
    [sym_push_keyword] = ACTIONS(1357),
    [sym_release_keyword] = ACTIONS(1357),
    [aux_sym_on_keyword_token1] = ACTIONS(1357),
    [aux_sym_off_keyword_token1] = ACTIONS(1357),
    [sym_send_level_keyword] = ACTIONS(1357),
    [sym_send_string_keyword] = ACTIONS(1357),
    [sym_send_command_keyword] = ACTIONS(1357),
    [sym_clear_buffer_keyword] = ACTIONS(1357),
    [sym_create_buffer_keyword] = ACTIONS(1357),
    [sym_create_multi_buffer_keyword] = ACTIONS(1357),
    [sym_call_keyword] = ACTIONS(1357),
    [sym_system_call_keyword] = ACTIONS(1357),
    [sym_devchan_to_keyword] = ACTIONS(1357),
    [sym_devchan_min_to_keyword] = ACTIONS(1357),
    [sym_devchan_total_off_keyword] = ACTIONS(1357),
    [sym_devchan_pulse_keyword] = ACTIONS(1357),
    [sym_wait_keyword] = ACTIONS(1357),
    [sym_cancel_wait_keyword] = ACTIONS(1357),
    [sym_wait_until_keyword] = ACTIONS(1357),
    [sym_cancel_wait_until_keyword] = ACTIONS(1357),
    [sym_cancel_all_wait_keyword] = ACTIONS(1357),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1357),
    [sym_and] = ACTIONS(1359),
    [sym_or] = ACTIONS(1361),
    [sym_not] = ACTIONS(1357),
    [sym_xor] = ACTIONS(1363),
    [sym_band] = ACTIONS(1365),
    [sym_bor] = ACTIONS(1367),
    [sym_bxor] = ACTIONS(1363),
    [sym_bnot] = ACTIONS(1357),
    [sym_lshift] = ACTIONS(1369),
    [sym_rshift] = ACTIONS(1369),
    [sym___file__] = ACTIONS(1357),
    [sym___date__] = ACTIONS(1357),
    [sym___ldate__] = ACTIONS(1357),
    [sym___time__] = ACTIONS(1357),
    [sym___line__] = ACTIONS(1357),
    [sym___name__] = ACTIONS(1357),
    [sym___netlinx__] = ACTIONS(1357),
    [sym_netlinx_axi_version] = ACTIONS(1357),
    [sym_url_flg_tcp] = ACTIONS(1357),
    [sym_url_flg_accinfopresent] = ACTIONS(1357),
    [sym_url_flg_temp] = ACTIONS(1357),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1357),
    [sym_url_flg_stat_mask] = ACTIONS(1357),
    [sym_url_flg_stat_lookup] = ACTIONS(1357),
    [sym_url_flg_stat_connecting] = ACTIONS(1357),
    [sym_url_flg_stat_waiting] = ACTIONS(1357),
    [sym_url_flg_stat_connected] = ACTIONS(1357),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1357),
    [sym_first_virtual_device] = ACTIONS(1357),
    [sym_first_local_port] = ACTIONS(1357),
    [sym_dynamic_virtual_device] = ACTIONS(1357),
    [sym_timeline_once] = ACTIONS(1357),
    [sym_timeline_repeat] = ACTIONS(1357),
    [sym_timeline_absolute] = ACTIONS(1357),
    [sym_timeline_relative] = ACTIONS(1357),
    [sym_do_push_timed_infinite] = ACTIONS(1357),
    [sym_source_type_no_address] = ACTIONS(1357),
    [sym_source_type_neuron_id] = ACTIONS(1357),
    [sym_source_type_ip_address] = ACTIONS(1357),
    [sym_source_type_axlink] = ACTIONS(1357),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1357),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1357),
    [sym_source_type_ip_socket_address] = ACTIONS(1357),
    [sym_source_type_rs232] = ACTIONS(1357),
    [sym_source_type_internal] = ACTIONS(1357),
    [sym_source_type_mac_address] = ACTIONS(1357),
    [sym_source_type_ipv4_port] = ACTIONS(1357),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1357),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1357),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1357),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1357),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1357),
    [sym_true] = ACTIONS(1357),
    [sym_false] = ACTIONS(1357),
    [sym_file_read_only] = ACTIONS(1357),
    [sym_file_rw_new] = ACTIONS(1357),
    [sym_file_rw_append] = ACTIONS(1357),
    [sym_ip_tcp] = ACTIONS(1357),
    [sym_ip_udp] = ACTIONS(1357),
    [sym_ip_udp_2way] = ACTIONS(1357),
    [sym_xml_encode_types] = ACTIONS(1357),
    [sym_xml_encode_char_as_list] = ACTIONS(1357),
    [sym_xml_encode_le] = ACTIONS(1357),
    [sym_xml_decode_types] = ACTIONS(1357),
    [sym_xml_decode_no_preserve] = ACTIONS(1357),
    [sym_duet_dev_type_display_device] = ACTIONS(1357),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1357),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1357),
    [sym_duet_dev_type_io_device] = ACTIONS(1357),
    [sym_duet_dev_type_relay_device] = ACTIONS(1357),
    [sym_duet_dev_type_ups] = ACTIONS(1357),
    [sym_duet_dev_type_amplifier] = ACTIONS(1357),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1357),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1357),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1357),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1357),
    [sym_duet_dev_type_camera] = ACTIONS(1357),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1357),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1357),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1357),
    [sym_duet_dev_type_dss] = ACTIONS(1357),
    [sym_duet_dev_type_dvr] = ACTIONS(1357),
    [sym_duet_dev_type_disc_device] = ACTIONS(1357),
    [sym_duet_dev_type_document_camera] = ACTIONS(1357),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1357),
    [sym_duet_dev_type_hvac] = ACTIONS(1357),
    [sym_duet_dev_type_keypad] = ACTIONS(1357),
    [sym_duet_dev_type_light] = ACTIONS(1357),
    [sym_duet_dev_type_monitor] = ACTIONS(1357),
    [sym_duet_dev_type_motor] = ACTIONS(1357),
    [sym_duet_dev_type_multi_window] = ACTIONS(1357),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1357),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1357),
    [sym_duet_dev_type_receiver] = ACTIONS(1357),
    [sym_duet_dev_type_security_system] = ACTIONS(1357),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1357),
    [sym_duet_dev_type_settop_box] = ACTIONS(1357),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1357),
    [sym_duet_dev_type_switcher] = ACTIONS(1357),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1357),
    [sym_duet_dev_type_tv] = ACTIONS(1357),
    [sym_duet_dev_type_utility] = ACTIONS(1357),
    [sym_duet_dev_type_vcr] = ACTIONS(1357),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1357),
    [sym_duet_dev_type_video_processor] = ACTIONS(1357),
    [sym_duet_dev_type_video_projector] = ACTIONS(1357),
    [sym_duet_dev_type_video_wall] = ACTIONS(1357),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1357),
    [sym_duet_dev_type_weather] = ACTIONS(1357),
    [sym_duet_dev_not_polled] = ACTIONS(1357),
    [sym_duet_dev_polled] = ACTIONS(1357),
    [sym_clkmgr_mode_network] = ACTIONS(1357),
    [sym_clkmgr_mode_standalone] = ACTIONS(1357),
    [sym_normal_standby] = ACTIONS(1357),
    [sym_normal_wake] = ACTIONS(1357),
    [sym_ip_multicast_ttl_option] = ACTIONS(1357),
    [sym_ip_ttl_subnet] = ACTIONS(1357),
    [sym_ip_ttl_site] = ACTIONS(1357),
    [sym_ip_ttl_region] = ACTIONS(1357),
    [sym_ip_ttl_continent] = ACTIONS(1357),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1357),
    [sym_ip_nodelay_on] = ACTIONS(1357),
    [sym_ip_nodelay_off] = ACTIONS(1357),
    [sym_amx_error] = ACTIONS(1357),
    [sym_amx_warning] = ACTIONS(1357),
    [sym_amx_info] = ACTIONS(1357),
    [sym_amx_debug] = ACTIONS(1357),
    [sym_netlinx_login_success] = ACTIONS(1357),
    [sym_netlinx_login_fail] = ACTIONS(1357),
    [sym_netlinx_logout] = ACTIONS(1357),
    [sym_null_str] = ACTIONS(1357),
    [sym_smtp_address] = ACTIONS(1357),
    [sym_smtp_port_number] = ACTIONS(1357),
    [sym_smtp_username] = ACTIONS(1357),
    [sym_smtp_password] = ACTIONS(1357),
    [sym_smtp_require_tls] = ACTIONS(1357),
    [sym_smtp_from] = ACTIONS(1357),
    [sym_smtp_tls_true] = ACTIONS(1357),
    [sym_smtp_tls_false] = ACTIONS(1357),
    [sym_listview_on_row_select_event] = ACTIONS(1357),
    [sym_data_structure_unknown] = ACTIONS(1357),
    [sym_data_structure_datafeed] = ACTIONS(1357),
    [sym_data_structure_datarecordset] = ACTIONS(1357),
    [sym_data_structure_datarecord] = ACTIONS(1357),
    [sym_data_structure_datafield] = ACTIONS(1357),
    [sym_data_max_content_field_count] = ACTIONS(1357),
    [sym_data_max_metadata_field_count] = ACTIONS(1357),
    [sym_data_max_id_length] = ACTIONS(1357),
    [sym_data_max_name_length] = ACTIONS(1357),
    [sym_data_max_description_length] = ACTIONS(1357),
    [sym_data_max_type_length] = ACTIONS(1357),
    [sym_data_max_format_length] = ACTIONS(1357),
    [sym_data_max_source_length] = ACTIONS(1357),
    [sym_data_max_value_length] = ACTIONS(1357),
    [sym_data_max_label_length] = ACTIONS(1357),
    [sym_data_max_url_length] = ACTIONS(1357),
    [sym_max_last_login_info_length] = ACTIONS(1357),
    [sym_data_type_unknown] = ACTIONS(1357),
    [sym_data_type_string] = ACTIONS(1357),
    [sym_data_type_datetime] = ACTIONS(1357),
    [sym_data_type_date] = ACTIONS(1357),
    [sym_data_type_time] = ACTIONS(1357),
    [sym_data_type_image] = ACTIONS(1357),
    [sym_data_format_url] = ACTIONS(1357),
    [sym_data_format_phone] = ACTIONS(1357),
    [sym_data_format_email] = ACTIONS(1357),
    [sym_data_format_iso8601] = ACTIONS(1357),
    [sym_auditlog_priv] = ACTIONS(1357),
    [sym_configuration_priv] = ACTIONS(1357),
    [sym_deviceconfig_priv] = ACTIONS(1357),
    [sym_ftp_priv] = ACTIONS(1357),
    [sym_http_priv] = ACTIONS(1357),
    [sym_networkconfig_priv] = ACTIONS(1357),
    [sym_programport_priv] = ACTIONS(1357),
    [sym_remoteui_priv] = ACTIONS(1357),
    [sym_securitycontrol_priv] = ACTIONS(1357),
    [sym_softwaremanagement_priv] = ACTIONS(1357),
    [sym_terminal_priv] = ACTIONS(1357),
    [sym_tpadmin_priv] = ACTIONS(1357),
    [sym_user1_priv] = ACTIONS(1357),
    [sym_user2_priv] = ACTIONS(1357),
    [sym_user3_priv] = ACTIONS(1357),
    [sym_user4_priv] = ACTIONS(1357),
    [sym_usermanagement_priv] = ACTIONS(1357),
    [sym_valid_account] = ACTIONS(1357),
    [sym_err_name_invalid] = ACTIONS(1357),
    [sym_err_password_invalid] = ACTIONS(1357),
    [sym_err_invalid_account] = ACTIONS(1357),
    [sym_err_no_authorization] = ACTIONS(1357),
    [sym_err_invalid_parameter] = ACTIONS(1357),
    [sym_err_user_locked_out] = ACTIONS(1357),
    [sym_err_user_account_expired] = ACTIONS(1357),
    [sym_err_general_error] = ACTIONS(1357),
    [sym_err_authentication_error] = ACTIONS(1357),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1357),
    [sym_tls_validate_certificate] = ACTIONS(1357),
    [sym_tdata] = ACTIONS(1357),
    [sym_tchannel] = ACTIONS(1357),
    [sym_tlevel] = ACTIONS(1357),
    [sym_tbutton] = ACTIONS(1357),
    [sym_ttimeline] = ACTIONS(1357),
    [sym_tcustom] = ACTIONS(1357),
    [sym_url_struct] = ACTIONS(1357),
    [sym_dns_struct] = ACTIONS(1357),
    [sym_ip_address_struct] = ACTIONS(1357),
    [sym_dev_info_struct] = ACTIONS(1357),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1357),
    [sym_data_feed] = ACTIONS(1357),
    [sym_data_field] = ACTIONS(1357),
    [sym_data_record] = ACTIONS(1357),
    [sym_wc_data_feed] = ACTIONS(1357),
    [sym_wc_data_field] = ACTIONS(1357),
    [sym_wc_data_record] = ACTIONS(1357),
    [sym_last_login_info] = ACTIONS(1357),
    [sym____reserved___] = ACTIONS(1357),
    [sym_date] = ACTIONS(1357),
    [sym_day] = ACTIONS(1357),
    [sym_time] = ACTIONS(1357),
    [sym_ldate] = ACTIONS(1357),
    [sym_push_device] = ACTIONS(1357),
    [sym_push_channel] = ACTIONS(1357),
    [sym_push_devchan] = ACTIONS(1357),
    [sym_release_device] = ACTIONS(1357),
    [sym_release_channel] = ACTIONS(1357),
    [sym_release_devchan] = ACTIONS(1357),
    [sym_master_slot] = ACTIONS(1357),
    [sym_get_pulse_time] = ACTIONS(1357),
    [sym_get_timer] = ACTIONS(1357),
    [sym_master_sn] = ACTIONS(1357),
    [sym_system_number] = ACTIONS(1357),
    [sym_button] = ACTIONS(1357),
    [sym_dv_channel] = ACTIONS(1357),
    [sym_data] = ACTIONS(1357),
    [sym_level] = ACTIONS(1357),
    [sym_timeline] = ACTIONS(1357),
    [sym_channel] = ACTIONS(1357),
    [sym_custom] = ACTIONS(1357),
    [sym_atoi] = ACTIONS(1357),
    [sym_atol] = ACTIONS(1357),
    [sym_atof] = ACTIONS(1357),
    [sym_itoa] = ACTIONS(1357),
    [sym_itohex] = ACTIONS(1357),
    [sym_hextoi] = ACTIONS(1357),
    [sym_ftoa] = ACTIONS(1357),
    [sym_find_string] = ACTIONS(1357),
    [sym_left_string] = ACTIONS(1357),
    [sym_right_string] = ACTIONS(1357),
    [sym_mid_string] = ACTIONS(1357),
    [sym_remove_string] = ACTIONS(1357),
    [sym_get_buffer_string] = ACTIONS(1357),
    [sym_get_buffer_char] = ACTIONS(1357),
    [sym_get_multi_buffer_string] = ACTIONS(1357),
    [sym_length_string] = ACTIONS(1357),
    [sym_max_length_string] = ACTIONS(1357),
    [sym_set_length_string] = ACTIONS(1357),
    [sym_compare_string] = ACTIONS(1357),
    [sym_upper_string] = ACTIONS(1357),
    [sym_lower_string] = ACTIONS(1357),
    [sym_get_last] = ACTIONS(1357),
    [sym_date_to_day] = ACTIONS(1357),
    [sym_date_to_month] = ACTIONS(1357),
    [sym_date_to_year] = ACTIONS(1357),
    [sym_time_to_hour] = ACTIONS(1357),
    [sym_time_to_minute] = ACTIONS(1357),
    [sym_time_to_second] = ACTIONS(1357),
    [sym_day_of_week] = ACTIONS(1357),
    [sym_set_pulse_time] = ACTIONS(1357),
    [sym_set_timer] = ACTIONS(1357),
    [sym_file_open] = ACTIONS(1357),
    [sym_file_close] = ACTIONS(1357),
    [sym_file_read] = ACTIONS(1357),
    [sym_file_read_line] = ACTIONS(1357),
    [sym_file_write] = ACTIONS(1357),
    [sym_file_write_line] = ACTIONS(1357),
    [sym_file_copy] = ACTIONS(1357),
    [sym_file_delete] = ACTIONS(1357),
    [sym_file_rename] = ACTIONS(1357),
    [sym_file_dir] = ACTIONS(1357),
    [sym_file_getdir] = ACTIONS(1357),
    [sym_file_setdir] = ACTIONS(1357),
    [sym_file_create_dir] = ACTIONS(1357),
    [sym_file_remove_dir] = ACTIONS(1357),
    [sym_file_seek] = ACTIONS(1357),
    [sym_length_array] = ACTIONS(1357),
    [sym_max_length_array] = ACTIONS(1357),
    [sym_set_length_array] = ACTIONS(1357),
    [sym_ip_client_open] = ACTIONS(1357),
    [sym_ip_client_close] = ACTIONS(1357),
    [sym_ip_server_open] = ACTIONS(1357),
    [sym_ip_server_close] = ACTIONS(1357),
    [sym_ip_bound_client_open] = ACTIONS(1357),
    [sym_ip_mc_server_open] = ACTIONS(1357),
    [sym_ip_set_option] = ACTIONS(1357),
    [sym_ssh_client_open] = ACTIONS(1357),
    [sym_ssh_client_close] = ACTIONS(1357),
    [sym_tls_client_open] = ACTIONS(1357),
    [sym_tls_client_close] = ACTIONS(1357),
    [sym_do_push] = ACTIONS(1357),
    [sym_do_push_timed] = ACTIONS(1357),
    [sym_do_release] = ACTIONS(1357),
    [sym_device_id] = ACTIONS(1357),
    [sym_device_info] = ACTIONS(1357),
    [sym_device_id_string] = ACTIONS(1357),
    [sym_device_standby] = ACTIONS(1357),
    [sym_device_wake] = ACTIONS(1357),
    [sym_timeline_create] = ACTIONS(1357),
    [sym_timeline_kill] = ACTIONS(1357),
    [sym_timeline_reload] = ACTIONS(1357),
    [sym_timeline_pause] = ACTIONS(1357),
    [sym_timeline_restart] = ACTIONS(1357),
    [sym_timeline_set] = ACTIONS(1357),
    [sym_timeline_get] = ACTIONS(1357),
    [sym_timeline_active] = ACTIONS(1357),
    [sym_timeline_dynamic_id] = ACTIONS(1357),
    [sym_variable_to_string] = ACTIONS(1357),
    [sym_string_to_variable] = ACTIONS(1357),
    [sym_length_variable_to_string] = ACTIONS(1357),
    [sym_type_cast] = ACTIONS(1357),
    [sym_variable_to_xml] = ACTIONS(1357),
    [sym_xml_to_variable] = ACTIONS(1357),
    [sym_length_variable_to_xml] = ACTIONS(1357),
    [sym_redirect_string] = ACTIONS(1357),
    [sym_random_number] = ACTIONS(1357),
    [sym_get_unique_id] = ACTIONS(1357),
    [sym_get_system_number] = ACTIONS(1357),
    [sym_set_system_number] = ACTIONS(1357),
    [sym_get_serial_number] = ACTIONS(1357),
    [sym_get_master_build] = ACTIONS(1357),
    [sym_set_validation_code] = ACTIONS(1357),
    [sym_reboot] = ACTIONS(1357),
    [sym_load_duet_module] = ACTIONS(1357),
    [sym_unload_duet_module] = ACTIONS(1357),
    [sym_duet_mem_size_set] = ACTIONS(1357),
    [sym_duet_mem_size_get] = ACTIONS(1357),
    [sym_dynamic_application_device] = ACTIONS(1357),
    [sym_dynamic_polled_port] = ACTIONS(1357),
    [sym_static_port_binding] = ACTIONS(1357),
    [sym_static_ip_binding] = ACTIONS(1357),
    [sym_get_dns_list] = ACTIONS(1357),
    [sym_set_dns_list] = ACTIONS(1357),
    [sym_get_ip_address] = ACTIONS(1357),
    [sym_set_ip_address] = ACTIONS(1357),
    [sym_get_url_list] = ACTIONS(1357),
    [sym_add_url_entry] = ACTIONS(1357),
    [sym_delete_url_entry] = ACTIONS(1357),
    [sym_abs_value] = ACTIONS(1357),
    [sym_max_value] = ACTIONS(1357),
    [sym_min_value] = ACTIONS(1357),
    [sym_exp_value] = ACTIONS(1357),
    [sym_log_value] = ACTIONS(1357),
    [sym_log10_value] = ACTIONS(1357),
    [sym_power_value] = ACTIONS(1357),
    [sym_sqrt_value] = ACTIONS(1357),
    [sym_raw_be] = ACTIONS(1357),
    [sym_raw_le] = ACTIONS(1357),
    [sym_internal_queue_size_set] = ACTIONS(1357),
    [sym_internal_queue_size_get] = ACTIONS(1357),
    [sym_internal_threshold_set] = ACTIONS(1357),
    [sym_internal_threshold_get] = ACTIONS(1357),
    [sym_rebuilt_event] = ACTIONS(1357),
    [sym_do_custom_event] = ACTIONS(1357),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1357),
    [sym_clkmgr_set_clk_source] = ACTIONS(1357),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1357),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1357),
    [sym_clkmgr_get_timezone] = ACTIONS(1357),
    [sym_clkmgr_set_timezone] = ACTIONS(1357),
    [sym_clkmgr_get_resync_period] = ACTIONS(1357),
    [sym_clkmgr_set_resync_period] = ACTIONS(1357),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1357),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1357),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1357),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1357),
    [sym_clkmgr_get_timeservers] = ACTIONS(1357),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1357),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1357),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1357),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1357),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1357),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1357),
    [sym_set_log_level] = ACTIONS(1357),
    [sym_get_log_level] = ACTIONS(1357),
    [sym_amx_log] = ACTIONS(1357),
    [sym_smtp_server_config_set] = ACTIONS(1357),
    [sym_smtp_server_config_get] = ACTIONS(1357),
    [sym_smtp_send] = ACTIONS(1357),
    [sym_validate_netlinx_account] = ACTIONS(1357),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1357),
    [sym_authenticate_certificate] = ACTIONS(1357),
    [sym_audit_netlinx_session_event] = ACTIONS(1357),
    [sym_audit_netlinx_generic_event] = ACTIONS(1357),
    [sym_get_available_flash_disk_space] = ACTIONS(1357),
    [sym_get_max_flash_disk_space] = ACTIONS(1357),
    [sym_data_create_feed] = ACTIONS(1357),
    [sym_data_delete_feed] = ACTIONS(1357),
    [sym_data_publish_feed] = ACTIONS(1357),
    [sym_data_get_published_feed] = ACTIONS(1357),
    [sym_data_add_record] = ACTIONS(1357),
    [sym_data_get_event_record] = ACTIONS(1357),
    [sym__wc_data_create_feed] = ACTIONS(1357),
    [sym__wc_data_add_record] = ACTIONS(1357),
    [sym__wc_data_get_event_record] = ACTIONS(1357),
    [sym_format] = ACTIONS(1357),
    [sym_astro_clock] = ACTIONS(1357),
    [sym_set_outdoor_temperature] = ACTIONS(1357),
    [sym_set_virtual_level_count] = ACTIONS(1357),
    [sym_set_virtual_channel_count] = ACTIONS(1357),
    [sym_set_virtual_port_count] = ACTIONS(1357),
    [sym__wc_encode_int] = ACTIONS(1357),
    [sym__wc_decode_int] = ACTIONS(1357),
    [sym__wc_int] = ACTIONS(1357),
    [sym__wc_to_ch_int] = ACTIONS(1357),
    [sym__ch_to_wc_int] = ACTIONS(1357),
    [sym__wc_find_string_int] = ACTIONS(1357),
    [sym__wc_left_string_int] = ACTIONS(1357),
    [sym__wc_lower_string_int] = ACTIONS(1357),
    [sym__wc_mid_string_int] = ACTIONS(1357),
    [sym__wc_remove_string_int] = ACTIONS(1357),
    [sym__wc_right_string_int] = ACTIONS(1357),
    [sym__wc_upper_string_int] = ACTIONS(1357),
    [sym__wc_compare_string_int] = ACTIONS(1357),
    [sym__wc_get_buffer_char_int] = ACTIONS(1357),
    [sym__wc_get_buffer_string_int] = ACTIONS(1357),
    [sym__wc_concat_string_int] = ACTIONS(1357),
    [sym__wc_file_open_int] = ACTIONS(1357),
    [sym__wc_file_close_int] = ACTIONS(1357),
    [sym__wc_file_read_int] = ACTIONS(1357),
    [sym__wc_file_read_line_int] = ACTIONS(1357),
    [sym__wc_file_write_int] = ACTIONS(1357),
    [sym__wc_file_write_line_int] = ACTIONS(1357),
    [aux_sym_preproc_include_token1] = ACTIONS(1357),
    [aux_sym_preproc_define_token1] = ACTIONS(1371),
    [aux_sym_preproc_warn_token1] = ACTIONS(1371),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1371),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1371),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1371),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1371),
    [aux_sym_preproc_else_token1] = ACTIONS(1371),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1371),
    [anon_sym_COLON] = ACTIONS(1375),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1379),
    [anon_sym_BANG] = ACTIONS(1357),
    [anon_sym_TILDE] = ACTIONS(1371),
    [anon_sym_DASH] = ACTIONS(1381),
    [anon_sym_PLUS] = ACTIONS(1381),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1387),
    [anon_sym_AMP_AMP] = ACTIONS(1389),
    [anon_sym_PIPE] = ACTIONS(1367),
    [anon_sym_CARET] = ACTIONS(1391),
    [anon_sym_AMP] = ACTIONS(1365),
    [anon_sym_EQ_EQ] = ACTIONS(1393),
    [anon_sym_BANG_EQ] = ACTIONS(1393),
    [anon_sym_LT_GT] = ACTIONS(1393),
    [anon_sym_GT] = ACTIONS(1395),
    [anon_sym_GT_EQ] = ACTIONS(1397),
    [anon_sym_LT_EQ] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1395),
    [anon_sym_LT_LT] = ACTIONS(1399),
    [anon_sym_GT_GT] = ACTIONS(1399),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1371),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(1371),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1379),
  },
  [STATE(58)] = {
    [sym_argument_list] = STATE(80),
    [sym_string_literal] = STATE(199),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1407),
    [sym_program_name_keyword] = ACTIONS(1407),
    [sym_module_name_keyword] = ACTIONS(1407),
    [sym_define_device_keyword] = ACTIONS(1407),
    [sym_define_combine_keyword] = ACTIONS(1407),
    [sym_define_constant_keyword] = ACTIONS(1407),
    [sym_define_type_keyword] = ACTIONS(1407),
    [sym_define_variable_keyword] = ACTIONS(1407),
    [sym_define_system_variable_keyword] = ACTIONS(1407),
    [sym_define_connect_level_keyword] = ACTIONS(1407),
    [sym_define_latching_keyword] = ACTIONS(1407),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1407),
    [sym_define_toggling_keyword] = ACTIONS(1407),
    [sym_define_call_keyword] = ACTIONS(1407),
    [sym_define_function_keyword] = ACTIONS(1407),
    [sym_define_library_function_keyword] = ACTIONS(1407),
    [sym_define_start_keyword] = ACTIONS(1407),
    [sym_define_event_keyword] = ACTIONS(1407),
    [sym_define_program_keyword] = ACTIONS(1407),
    [sym_define_module_keyword] = ACTIONS(1407),
    [sym_char_keyword] = ACTIONS(1407),
    [sym_widechar_keyword] = ACTIONS(1407),
    [sym_integer_keyword] = ACTIONS(1407),
    [sym_sinteger_keyword] = ACTIONS(1407),
    [sym_long_keyword] = ACTIONS(1407),
    [sym_slong_keyword] = ACTIONS(1407),
    [sym_float_keyword] = ACTIONS(1407),
    [sym_double_keyword] = ACTIONS(1407),
    [sym_variant_keyword] = ACTIONS(1407),
    [sym_variantarray_keyword] = ACTIONS(1407),
    [sym_local_var_keyword] = ACTIONS(1407),
    [sym_stack_var_keyword] = ACTIONS(1407),
    [sym_constant_keyword] = ACTIONS(1407),
    [sym_volatile_keyword] = ACTIONS(1407),
    [sym_non_volatile_keyword] = ACTIONS(1407),
    [sym_persistent_keyword] = ACTIONS(1407),
    [sym_dev_keyword] = ACTIONS(1407),
    [sym_devlev_keyword] = ACTIONS(1407),
    [sym_devchan_keyword] = ACTIONS(1407),
    [sym_if_keyword] = ACTIONS(1407),
    [sym_else_keyword] = ACTIONS(1407),
    [sym_switch_keyword] = ACTIONS(1407),
    [sym_case_keyword] = ACTIONS(1407),
    [sym_default_keyword] = ACTIONS(1407),
    [sym_while_keyword] = ACTIONS(1407),
    [sym_for_keyword] = ACTIONS(1407),
    [sym_break_keyword] = ACTIONS(1407),
    [sym_continue_keyword] = ACTIONS(1407),
    [sym_return_keyword] = ACTIONS(1407),
    [sym_select_keyword] = ACTIONS(1407),
    [sym_active_keyword] = ACTIONS(1407),
    [sym_struct_keyword] = ACTIONS(1407),
    [sym_structure_keyword] = ACTIONS(1407),
    [sym_button_event_keyword] = ACTIONS(1407),
    [sym_channel_event_keyword] = ACTIONS(1407),
    [sym_level_event_keyword] = ACTIONS(1407),
    [sym_data_event_keyword] = ACTIONS(1407),
    [sym_timeline_event_keyword] = ACTIONS(1407),
    [sym_custom_event_keyword] = ACTIONS(1407),
    [sym_push_keyword] = ACTIONS(1407),
    [sym_release_keyword] = ACTIONS(1407),
    [aux_sym_on_keyword_token1] = ACTIONS(1407),
    [aux_sym_off_keyword_token1] = ACTIONS(1407),
    [sym_send_level_keyword] = ACTIONS(1407),
    [sym_send_string_keyword] = ACTIONS(1407),
    [sym_send_command_keyword] = ACTIONS(1407),
    [sym_clear_buffer_keyword] = ACTIONS(1407),
    [sym_create_buffer_keyword] = ACTIONS(1407),
    [sym_create_multi_buffer_keyword] = ACTIONS(1407),
    [sym_call_keyword] = ACTIONS(1407),
    [sym_system_call_keyword] = ACTIONS(1407),
    [sym_devchan_to_keyword] = ACTIONS(1407),
    [sym_devchan_min_to_keyword] = ACTIONS(1407),
    [sym_devchan_total_off_keyword] = ACTIONS(1407),
    [sym_devchan_pulse_keyword] = ACTIONS(1407),
    [sym_wait_keyword] = ACTIONS(1407),
    [sym_cancel_wait_keyword] = ACTIONS(1407),
    [sym_wait_until_keyword] = ACTIONS(1407),
    [sym_cancel_wait_until_keyword] = ACTIONS(1407),
    [sym_cancel_all_wait_keyword] = ACTIONS(1407),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1407),
    [sym_and] = ACTIONS(1359),
    [sym_or] = ACTIONS(1361),
    [sym_not] = ACTIONS(1407),
    [sym_xor] = ACTIONS(1363),
    [sym_band] = ACTIONS(1365),
    [sym_bor] = ACTIONS(1367),
    [sym_bxor] = ACTIONS(1363),
    [sym_bnot] = ACTIONS(1407),
    [sym_lshift] = ACTIONS(1369),
    [sym_rshift] = ACTIONS(1369),
    [sym___file__] = ACTIONS(1407),
    [sym___date__] = ACTIONS(1407),
    [sym___ldate__] = ACTIONS(1407),
    [sym___time__] = ACTIONS(1407),
    [sym___line__] = ACTIONS(1407),
    [sym___name__] = ACTIONS(1407),
    [sym___netlinx__] = ACTIONS(1407),
    [sym_netlinx_axi_version] = ACTIONS(1407),
    [sym_url_flg_tcp] = ACTIONS(1407),
    [sym_url_flg_accinfopresent] = ACTIONS(1407),
    [sym_url_flg_temp] = ACTIONS(1407),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1407),
    [sym_url_flg_stat_mask] = ACTIONS(1407),
    [sym_url_flg_stat_lookup] = ACTIONS(1407),
    [sym_url_flg_stat_connecting] = ACTIONS(1407),
    [sym_url_flg_stat_waiting] = ACTIONS(1407),
    [sym_url_flg_stat_connected] = ACTIONS(1407),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1407),
    [sym_first_virtual_device] = ACTIONS(1407),
    [sym_first_local_port] = ACTIONS(1407),
    [sym_dynamic_virtual_device] = ACTIONS(1407),
    [sym_timeline_once] = ACTIONS(1407),
    [sym_timeline_repeat] = ACTIONS(1407),
    [sym_timeline_absolute] = ACTIONS(1407),
    [sym_timeline_relative] = ACTIONS(1407),
    [sym_do_push_timed_infinite] = ACTIONS(1407),
    [sym_source_type_no_address] = ACTIONS(1407),
    [sym_source_type_neuron_id] = ACTIONS(1407),
    [sym_source_type_ip_address] = ACTIONS(1407),
    [sym_source_type_axlink] = ACTIONS(1407),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1407),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1407),
    [sym_source_type_ip_socket_address] = ACTIONS(1407),
    [sym_source_type_rs232] = ACTIONS(1407),
    [sym_source_type_internal] = ACTIONS(1407),
    [sym_source_type_mac_address] = ACTIONS(1407),
    [sym_source_type_ipv4_port] = ACTIONS(1407),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1407),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1407),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1407),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1407),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1407),
    [sym_true] = ACTIONS(1407),
    [sym_false] = ACTIONS(1407),
    [sym_file_read_only] = ACTIONS(1407),
    [sym_file_rw_new] = ACTIONS(1407),
    [sym_file_rw_append] = ACTIONS(1407),
    [sym_ip_tcp] = ACTIONS(1407),
    [sym_ip_udp] = ACTIONS(1407),
    [sym_ip_udp_2way] = ACTIONS(1407),
    [sym_xml_encode_types] = ACTIONS(1407),
    [sym_xml_encode_char_as_list] = ACTIONS(1407),
    [sym_xml_encode_le] = ACTIONS(1407),
    [sym_xml_decode_types] = ACTIONS(1407),
    [sym_xml_decode_no_preserve] = ACTIONS(1407),
    [sym_duet_dev_type_display_device] = ACTIONS(1407),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1407),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1407),
    [sym_duet_dev_type_io_device] = ACTIONS(1407),
    [sym_duet_dev_type_relay_device] = ACTIONS(1407),
    [sym_duet_dev_type_ups] = ACTIONS(1407),
    [sym_duet_dev_type_amplifier] = ACTIONS(1407),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1407),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1407),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1407),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1407),
    [sym_duet_dev_type_camera] = ACTIONS(1407),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1407),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1407),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1407),
    [sym_duet_dev_type_dss] = ACTIONS(1407),
    [sym_duet_dev_type_dvr] = ACTIONS(1407),
    [sym_duet_dev_type_disc_device] = ACTIONS(1407),
    [sym_duet_dev_type_document_camera] = ACTIONS(1407),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1407),
    [sym_duet_dev_type_hvac] = ACTIONS(1407),
    [sym_duet_dev_type_keypad] = ACTIONS(1407),
    [sym_duet_dev_type_light] = ACTIONS(1407),
    [sym_duet_dev_type_monitor] = ACTIONS(1407),
    [sym_duet_dev_type_motor] = ACTIONS(1407),
    [sym_duet_dev_type_multi_window] = ACTIONS(1407),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1407),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1407),
    [sym_duet_dev_type_receiver] = ACTIONS(1407),
    [sym_duet_dev_type_security_system] = ACTIONS(1407),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1407),
    [sym_duet_dev_type_settop_box] = ACTIONS(1407),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1407),
    [sym_duet_dev_type_switcher] = ACTIONS(1407),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1407),
    [sym_duet_dev_type_tv] = ACTIONS(1407),
    [sym_duet_dev_type_utility] = ACTIONS(1407),
    [sym_duet_dev_type_vcr] = ACTIONS(1407),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1407),
    [sym_duet_dev_type_video_processor] = ACTIONS(1407),
    [sym_duet_dev_type_video_projector] = ACTIONS(1407),
    [sym_duet_dev_type_video_wall] = ACTIONS(1407),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1407),
    [sym_duet_dev_type_weather] = ACTIONS(1407),
    [sym_duet_dev_not_polled] = ACTIONS(1407),
    [sym_duet_dev_polled] = ACTIONS(1407),
    [sym_clkmgr_mode_network] = ACTIONS(1407),
    [sym_clkmgr_mode_standalone] = ACTIONS(1407),
    [sym_normal_standby] = ACTIONS(1407),
    [sym_normal_wake] = ACTIONS(1407),
    [sym_ip_multicast_ttl_option] = ACTIONS(1407),
    [sym_ip_ttl_subnet] = ACTIONS(1407),
    [sym_ip_ttl_site] = ACTIONS(1407),
    [sym_ip_ttl_region] = ACTIONS(1407),
    [sym_ip_ttl_continent] = ACTIONS(1407),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1407),
    [sym_ip_nodelay_on] = ACTIONS(1407),
    [sym_ip_nodelay_off] = ACTIONS(1407),
    [sym_amx_error] = ACTIONS(1407),
    [sym_amx_warning] = ACTIONS(1407),
    [sym_amx_info] = ACTIONS(1407),
    [sym_amx_debug] = ACTIONS(1407),
    [sym_netlinx_login_success] = ACTIONS(1407),
    [sym_netlinx_login_fail] = ACTIONS(1407),
    [sym_netlinx_logout] = ACTIONS(1407),
    [sym_null_str] = ACTIONS(1407),
    [sym_smtp_address] = ACTIONS(1407),
    [sym_smtp_port_number] = ACTIONS(1407),
    [sym_smtp_username] = ACTIONS(1407),
    [sym_smtp_password] = ACTIONS(1407),
    [sym_smtp_require_tls] = ACTIONS(1407),
    [sym_smtp_from] = ACTIONS(1407),
    [sym_smtp_tls_true] = ACTIONS(1407),
    [sym_smtp_tls_false] = ACTIONS(1407),
    [sym_listview_on_row_select_event] = ACTIONS(1407),
    [sym_data_structure_unknown] = ACTIONS(1407),
    [sym_data_structure_datafeed] = ACTIONS(1407),
    [sym_data_structure_datarecordset] = ACTIONS(1407),
    [sym_data_structure_datarecord] = ACTIONS(1407),
    [sym_data_structure_datafield] = ACTIONS(1407),
    [sym_data_max_content_field_count] = ACTIONS(1407),
    [sym_data_max_metadata_field_count] = ACTIONS(1407),
    [sym_data_max_id_length] = ACTIONS(1407),
    [sym_data_max_name_length] = ACTIONS(1407),
    [sym_data_max_description_length] = ACTIONS(1407),
    [sym_data_max_type_length] = ACTIONS(1407),
    [sym_data_max_format_length] = ACTIONS(1407),
    [sym_data_max_source_length] = ACTIONS(1407),
    [sym_data_max_value_length] = ACTIONS(1407),
    [sym_data_max_label_length] = ACTIONS(1407),
    [sym_data_max_url_length] = ACTIONS(1407),
    [sym_max_last_login_info_length] = ACTIONS(1407),
    [sym_data_type_unknown] = ACTIONS(1407),
    [sym_data_type_string] = ACTIONS(1407),
    [sym_data_type_datetime] = ACTIONS(1407),
    [sym_data_type_date] = ACTIONS(1407),
    [sym_data_type_time] = ACTIONS(1407),
    [sym_data_type_image] = ACTIONS(1407),
    [sym_data_format_url] = ACTIONS(1407),
    [sym_data_format_phone] = ACTIONS(1407),
    [sym_data_format_email] = ACTIONS(1407),
    [sym_data_format_iso8601] = ACTIONS(1407),
    [sym_auditlog_priv] = ACTIONS(1407),
    [sym_configuration_priv] = ACTIONS(1407),
    [sym_deviceconfig_priv] = ACTIONS(1407),
    [sym_ftp_priv] = ACTIONS(1407),
    [sym_http_priv] = ACTIONS(1407),
    [sym_networkconfig_priv] = ACTIONS(1407),
    [sym_programport_priv] = ACTIONS(1407),
    [sym_remoteui_priv] = ACTIONS(1407),
    [sym_securitycontrol_priv] = ACTIONS(1407),
    [sym_softwaremanagement_priv] = ACTIONS(1407),
    [sym_terminal_priv] = ACTIONS(1407),
    [sym_tpadmin_priv] = ACTIONS(1407),
    [sym_user1_priv] = ACTIONS(1407),
    [sym_user2_priv] = ACTIONS(1407),
    [sym_user3_priv] = ACTIONS(1407),
    [sym_user4_priv] = ACTIONS(1407),
    [sym_usermanagement_priv] = ACTIONS(1407),
    [sym_valid_account] = ACTIONS(1407),
    [sym_err_name_invalid] = ACTIONS(1407),
    [sym_err_password_invalid] = ACTIONS(1407),
    [sym_err_invalid_account] = ACTIONS(1407),
    [sym_err_no_authorization] = ACTIONS(1407),
    [sym_err_invalid_parameter] = ACTIONS(1407),
    [sym_err_user_locked_out] = ACTIONS(1407),
    [sym_err_user_account_expired] = ACTIONS(1407),
    [sym_err_general_error] = ACTIONS(1407),
    [sym_err_authentication_error] = ACTIONS(1407),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1407),
    [sym_tls_validate_certificate] = ACTIONS(1407),
    [sym_tdata] = ACTIONS(1407),
    [sym_tchannel] = ACTIONS(1407),
    [sym_tlevel] = ACTIONS(1407),
    [sym_tbutton] = ACTIONS(1407),
    [sym_ttimeline] = ACTIONS(1407),
    [sym_tcustom] = ACTIONS(1407),
    [sym_url_struct] = ACTIONS(1407),
    [sym_dns_struct] = ACTIONS(1407),
    [sym_ip_address_struct] = ACTIONS(1407),
    [sym_dev_info_struct] = ACTIONS(1407),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1407),
    [sym_data_feed] = ACTIONS(1407),
    [sym_data_field] = ACTIONS(1407),
    [sym_data_record] = ACTIONS(1407),
    [sym_wc_data_feed] = ACTIONS(1407),
    [sym_wc_data_field] = ACTIONS(1407),
    [sym_wc_data_record] = ACTIONS(1407),
    [sym_last_login_info] = ACTIONS(1407),
    [sym____reserved___] = ACTIONS(1407),
    [sym_date] = ACTIONS(1407),
    [sym_day] = ACTIONS(1407),
    [sym_time] = ACTIONS(1407),
    [sym_ldate] = ACTIONS(1407),
    [sym_push_device] = ACTIONS(1407),
    [sym_push_channel] = ACTIONS(1407),
    [sym_push_devchan] = ACTIONS(1407),
    [sym_release_device] = ACTIONS(1407),
    [sym_release_channel] = ACTIONS(1407),
    [sym_release_devchan] = ACTIONS(1407),
    [sym_master_slot] = ACTIONS(1407),
    [sym_get_pulse_time] = ACTIONS(1407),
    [sym_get_timer] = ACTIONS(1407),
    [sym_master_sn] = ACTIONS(1407),
    [sym_system_number] = ACTIONS(1407),
    [sym_button] = ACTIONS(1407),
    [sym_dv_channel] = ACTIONS(1407),
    [sym_data] = ACTIONS(1407),
    [sym_level] = ACTIONS(1407),
    [sym_timeline] = ACTIONS(1407),
    [sym_channel] = ACTIONS(1407),
    [sym_custom] = ACTIONS(1407),
    [sym_atoi] = ACTIONS(1407),
    [sym_atol] = ACTIONS(1407),
    [sym_atof] = ACTIONS(1407),
    [sym_itoa] = ACTIONS(1407),
    [sym_itohex] = ACTIONS(1407),
    [sym_hextoi] = ACTIONS(1407),
    [sym_ftoa] = ACTIONS(1407),
    [sym_find_string] = ACTIONS(1407),
    [sym_left_string] = ACTIONS(1407),
    [sym_right_string] = ACTIONS(1407),
    [sym_mid_string] = ACTIONS(1407),
    [sym_remove_string] = ACTIONS(1407),
    [sym_get_buffer_string] = ACTIONS(1407),
    [sym_get_buffer_char] = ACTIONS(1407),
    [sym_get_multi_buffer_string] = ACTIONS(1407),
    [sym_length_string] = ACTIONS(1407),
    [sym_max_length_string] = ACTIONS(1407),
    [sym_set_length_string] = ACTIONS(1407),
    [sym_compare_string] = ACTIONS(1407),
    [sym_upper_string] = ACTIONS(1407),
    [sym_lower_string] = ACTIONS(1407),
    [sym_get_last] = ACTIONS(1407),
    [sym_date_to_day] = ACTIONS(1407),
    [sym_date_to_month] = ACTIONS(1407),
    [sym_date_to_year] = ACTIONS(1407),
    [sym_time_to_hour] = ACTIONS(1407),
    [sym_time_to_minute] = ACTIONS(1407),
    [sym_time_to_second] = ACTIONS(1407),
    [sym_day_of_week] = ACTIONS(1407),
    [sym_set_pulse_time] = ACTIONS(1407),
    [sym_set_timer] = ACTIONS(1407),
    [sym_file_open] = ACTIONS(1407),
    [sym_file_close] = ACTIONS(1407),
    [sym_file_read] = ACTIONS(1407),
    [sym_file_read_line] = ACTIONS(1407),
    [sym_file_write] = ACTIONS(1407),
    [sym_file_write_line] = ACTIONS(1407),
    [sym_file_copy] = ACTIONS(1407),
    [sym_file_delete] = ACTIONS(1407),
    [sym_file_rename] = ACTIONS(1407),
    [sym_file_dir] = ACTIONS(1407),
    [sym_file_getdir] = ACTIONS(1407),
    [sym_file_setdir] = ACTIONS(1407),
    [sym_file_create_dir] = ACTIONS(1407),
    [sym_file_remove_dir] = ACTIONS(1407),
    [sym_file_seek] = ACTIONS(1407),
    [sym_length_array] = ACTIONS(1407),
    [sym_max_length_array] = ACTIONS(1407),
    [sym_set_length_array] = ACTIONS(1407),
    [sym_ip_client_open] = ACTIONS(1407),
    [sym_ip_client_close] = ACTIONS(1407),
    [sym_ip_server_open] = ACTIONS(1407),
    [sym_ip_server_close] = ACTIONS(1407),
    [sym_ip_bound_client_open] = ACTIONS(1407),
    [sym_ip_mc_server_open] = ACTIONS(1407),
    [sym_ip_set_option] = ACTIONS(1407),
    [sym_ssh_client_open] = ACTIONS(1407),
    [sym_ssh_client_close] = ACTIONS(1407),
    [sym_tls_client_open] = ACTIONS(1407),
    [sym_tls_client_close] = ACTIONS(1407),
    [sym_do_push] = ACTIONS(1407),
    [sym_do_push_timed] = ACTIONS(1407),
    [sym_do_release] = ACTIONS(1407),
    [sym_device_id] = ACTIONS(1407),
    [sym_device_info] = ACTIONS(1407),
    [sym_device_id_string] = ACTIONS(1407),
    [sym_device_standby] = ACTIONS(1407),
    [sym_device_wake] = ACTIONS(1407),
    [sym_timeline_create] = ACTIONS(1407),
    [sym_timeline_kill] = ACTIONS(1407),
    [sym_timeline_reload] = ACTIONS(1407),
    [sym_timeline_pause] = ACTIONS(1407),
    [sym_timeline_restart] = ACTIONS(1407),
    [sym_timeline_set] = ACTIONS(1407),
    [sym_timeline_get] = ACTIONS(1407),
    [sym_timeline_active] = ACTIONS(1407),
    [sym_timeline_dynamic_id] = ACTIONS(1407),
    [sym_variable_to_string] = ACTIONS(1407),
    [sym_string_to_variable] = ACTIONS(1407),
    [sym_length_variable_to_string] = ACTIONS(1407),
    [sym_type_cast] = ACTIONS(1407),
    [sym_variable_to_xml] = ACTIONS(1407),
    [sym_xml_to_variable] = ACTIONS(1407),
    [sym_length_variable_to_xml] = ACTIONS(1407),
    [sym_redirect_string] = ACTIONS(1407),
    [sym_random_number] = ACTIONS(1407),
    [sym_get_unique_id] = ACTIONS(1407),
    [sym_get_system_number] = ACTIONS(1407),
    [sym_set_system_number] = ACTIONS(1407),
    [sym_get_serial_number] = ACTIONS(1407),
    [sym_get_master_build] = ACTIONS(1407),
    [sym_set_validation_code] = ACTIONS(1407),
    [sym_reboot] = ACTIONS(1407),
    [sym_load_duet_module] = ACTIONS(1407),
    [sym_unload_duet_module] = ACTIONS(1407),
    [sym_duet_mem_size_set] = ACTIONS(1407),
    [sym_duet_mem_size_get] = ACTIONS(1407),
    [sym_dynamic_application_device] = ACTIONS(1407),
    [sym_dynamic_polled_port] = ACTIONS(1407),
    [sym_static_port_binding] = ACTIONS(1407),
    [sym_static_ip_binding] = ACTIONS(1407),
    [sym_get_dns_list] = ACTIONS(1407),
    [sym_set_dns_list] = ACTIONS(1407),
    [sym_get_ip_address] = ACTIONS(1407),
    [sym_set_ip_address] = ACTIONS(1407),
    [sym_get_url_list] = ACTIONS(1407),
    [sym_add_url_entry] = ACTIONS(1407),
    [sym_delete_url_entry] = ACTIONS(1407),
    [sym_abs_value] = ACTIONS(1407),
    [sym_max_value] = ACTIONS(1407),
    [sym_min_value] = ACTIONS(1407),
    [sym_exp_value] = ACTIONS(1407),
    [sym_log_value] = ACTIONS(1407),
    [sym_log10_value] = ACTIONS(1407),
    [sym_power_value] = ACTIONS(1407),
    [sym_sqrt_value] = ACTIONS(1407),
    [sym_raw_be] = ACTIONS(1407),
    [sym_raw_le] = ACTIONS(1407),
    [sym_internal_queue_size_set] = ACTIONS(1407),
    [sym_internal_queue_size_get] = ACTIONS(1407),
    [sym_internal_threshold_set] = ACTIONS(1407),
    [sym_internal_threshold_get] = ACTIONS(1407),
    [sym_rebuilt_event] = ACTIONS(1407),
    [sym_do_custom_event] = ACTIONS(1407),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1407),
    [sym_clkmgr_set_clk_source] = ACTIONS(1407),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1407),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1407),
    [sym_clkmgr_get_timezone] = ACTIONS(1407),
    [sym_clkmgr_set_timezone] = ACTIONS(1407),
    [sym_clkmgr_get_resync_period] = ACTIONS(1407),
    [sym_clkmgr_set_resync_period] = ACTIONS(1407),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1407),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1407),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1407),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1407),
    [sym_clkmgr_get_timeservers] = ACTIONS(1407),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1407),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1407),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1407),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1407),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1407),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1407),
    [sym_set_log_level] = ACTIONS(1407),
    [sym_get_log_level] = ACTIONS(1407),
    [sym_amx_log] = ACTIONS(1407),
    [sym_smtp_server_config_set] = ACTIONS(1407),
    [sym_smtp_server_config_get] = ACTIONS(1407),
    [sym_smtp_send] = ACTIONS(1407),
    [sym_validate_netlinx_account] = ACTIONS(1407),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1407),
    [sym_authenticate_certificate] = ACTIONS(1407),
    [sym_audit_netlinx_session_event] = ACTIONS(1407),
    [sym_audit_netlinx_generic_event] = ACTIONS(1407),
    [sym_get_available_flash_disk_space] = ACTIONS(1407),
    [sym_get_max_flash_disk_space] = ACTIONS(1407),
    [sym_data_create_feed] = ACTIONS(1407),
    [sym_data_delete_feed] = ACTIONS(1407),
    [sym_data_publish_feed] = ACTIONS(1407),
    [sym_data_get_published_feed] = ACTIONS(1407),
    [sym_data_add_record] = ACTIONS(1407),
    [sym_data_get_event_record] = ACTIONS(1407),
    [sym__wc_data_create_feed] = ACTIONS(1407),
    [sym__wc_data_add_record] = ACTIONS(1407),
    [sym__wc_data_get_event_record] = ACTIONS(1407),
    [sym_format] = ACTIONS(1407),
    [sym_astro_clock] = ACTIONS(1407),
    [sym_set_outdoor_temperature] = ACTIONS(1407),
    [sym_set_virtual_level_count] = ACTIONS(1407),
    [sym_set_virtual_channel_count] = ACTIONS(1407),
    [sym_set_virtual_port_count] = ACTIONS(1407),
    [sym__wc_encode_int] = ACTIONS(1407),
    [sym__wc_decode_int] = ACTIONS(1407),
    [sym__wc_int] = ACTIONS(1407),
    [sym__wc_to_ch_int] = ACTIONS(1407),
    [sym__ch_to_wc_int] = ACTIONS(1407),
    [sym__wc_find_string_int] = ACTIONS(1407),
    [sym__wc_left_string_int] = ACTIONS(1407),
    [sym__wc_lower_string_int] = ACTIONS(1407),
    [sym__wc_mid_string_int] = ACTIONS(1407),
    [sym__wc_remove_string_int] = ACTIONS(1407),
    [sym__wc_right_string_int] = ACTIONS(1407),
    [sym__wc_upper_string_int] = ACTIONS(1407),
    [sym__wc_compare_string_int] = ACTIONS(1407),
    [sym__wc_get_buffer_char_int] = ACTIONS(1407),
    [sym__wc_get_buffer_string_int] = ACTIONS(1407),
    [sym__wc_concat_string_int] = ACTIONS(1407),
    [sym__wc_file_open_int] = ACTIONS(1407),
    [sym__wc_file_close_int] = ACTIONS(1407),
    [sym__wc_file_read_int] = ACTIONS(1407),
    [sym__wc_file_read_line_int] = ACTIONS(1407),
    [sym__wc_file_write_int] = ACTIONS(1407),
    [sym__wc_file_write_line_int] = ACTIONS(1407),
    [aux_sym_preproc_include_token1] = ACTIONS(1407),
    [aux_sym_preproc_define_token1] = ACTIONS(1409),
    [aux_sym_preproc_warn_token1] = ACTIONS(1409),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1409),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1409),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1409),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1409),
    [aux_sym_preproc_else_token1] = ACTIONS(1409),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1409),
    [anon_sym_COLON] = ACTIONS(1375),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1411),
    [anon_sym_BANG] = ACTIONS(1407),
    [anon_sym_TILDE] = ACTIONS(1409),
    [anon_sym_DASH] = ACTIONS(1381),
    [anon_sym_PLUS] = ACTIONS(1381),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1387),
    [anon_sym_AMP_AMP] = ACTIONS(1389),
    [anon_sym_PIPE] = ACTIONS(1367),
    [anon_sym_CARET] = ACTIONS(1391),
    [anon_sym_AMP] = ACTIONS(1365),
    [anon_sym_EQ_EQ] = ACTIONS(1393),
    [anon_sym_BANG_EQ] = ACTIONS(1393),
    [anon_sym_LT_GT] = ACTIONS(1393),
    [anon_sym_GT] = ACTIONS(1395),
    [anon_sym_GT_EQ] = ACTIONS(1397),
    [anon_sym_LT_EQ] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1395),
    [anon_sym_LT_LT] = ACTIONS(1399),
    [anon_sym_GT_GT] = ACTIONS(1399),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1409),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(1409),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1411),
  },
  [STATE(59)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1369),
    [sym_rshift] = ACTIONS(1369),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1381),
    [anon_sym_PLUS] = ACTIONS(1381),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1393),
    [anon_sym_BANG_EQ] = ACTIONS(1393),
    [anon_sym_LT_GT] = ACTIONS(1393),
    [anon_sym_GT] = ACTIONS(1395),
    [anon_sym_GT_EQ] = ACTIONS(1397),
    [anon_sym_LT_EQ] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1395),
    [anon_sym_LT_LT] = ACTIONS(1399),
    [anon_sym_GT_GT] = ACTIONS(1399),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(60)] = {
    [sym_argument_list] = STATE(131),
    [sym_string_literal] = STATE(200),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1409),
    [sym_identifier] = ACTIONS(1407),
    [sym_program_name_keyword] = ACTIONS(1407),
    [sym_module_name_keyword] = ACTIONS(1407),
    [sym_define_device_keyword] = ACTIONS(1407),
    [sym_define_combine_keyword] = ACTIONS(1407),
    [sym_define_constant_keyword] = ACTIONS(1407),
    [sym_define_type_keyword] = ACTIONS(1407),
    [sym_define_variable_keyword] = ACTIONS(1407),
    [sym_define_system_variable_keyword] = ACTIONS(1407),
    [sym_define_connect_level_keyword] = ACTIONS(1407),
    [sym_define_latching_keyword] = ACTIONS(1407),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1407),
    [sym_define_toggling_keyword] = ACTIONS(1407),
    [sym_define_call_keyword] = ACTIONS(1407),
    [sym_define_function_keyword] = ACTIONS(1407),
    [sym_define_library_function_keyword] = ACTIONS(1407),
    [sym_define_start_keyword] = ACTIONS(1407),
    [sym_define_event_keyword] = ACTIONS(1407),
    [sym_define_program_keyword] = ACTIONS(1407),
    [sym_define_module_keyword] = ACTIONS(1407),
    [sym_char_keyword] = ACTIONS(1407),
    [sym_widechar_keyword] = ACTIONS(1407),
    [sym_integer_keyword] = ACTIONS(1407),
    [sym_sinteger_keyword] = ACTIONS(1407),
    [sym_long_keyword] = ACTIONS(1407),
    [sym_slong_keyword] = ACTIONS(1407),
    [sym_float_keyword] = ACTIONS(1407),
    [sym_double_keyword] = ACTIONS(1407),
    [sym_variant_keyword] = ACTIONS(1407),
    [sym_variantarray_keyword] = ACTIONS(1407),
    [sym_local_var_keyword] = ACTIONS(1407),
    [sym_stack_var_keyword] = ACTIONS(1407),
    [sym_constant_keyword] = ACTIONS(1407),
    [sym_volatile_keyword] = ACTIONS(1407),
    [sym_non_volatile_keyword] = ACTIONS(1407),
    [sym_persistent_keyword] = ACTIONS(1407),
    [sym_dev_keyword] = ACTIONS(1407),
    [sym_devlev_keyword] = ACTIONS(1407),
    [sym_devchan_keyword] = ACTIONS(1407),
    [sym_if_keyword] = ACTIONS(1407),
    [sym_else_keyword] = ACTIONS(1407),
    [sym_switch_keyword] = ACTIONS(1407),
    [sym_case_keyword] = ACTIONS(1407),
    [sym_default_keyword] = ACTIONS(1407),
    [sym_while_keyword] = ACTIONS(1407),
    [sym_for_keyword] = ACTIONS(1407),
    [sym_break_keyword] = ACTIONS(1407),
    [sym_continue_keyword] = ACTIONS(1407),
    [sym_return_keyword] = ACTIONS(1407),
    [sym_select_keyword] = ACTIONS(1407),
    [sym_active_keyword] = ACTIONS(1407),
    [sym_struct_keyword] = ACTIONS(1407),
    [sym_structure_keyword] = ACTIONS(1407),
    [sym_button_event_keyword] = ACTIONS(1407),
    [sym_channel_event_keyword] = ACTIONS(1407),
    [sym_level_event_keyword] = ACTIONS(1407),
    [sym_data_event_keyword] = ACTIONS(1407),
    [sym_timeline_event_keyword] = ACTIONS(1407),
    [sym_custom_event_keyword] = ACTIONS(1407),
    [sym_push_keyword] = ACTIONS(1407),
    [sym_release_keyword] = ACTIONS(1407),
    [aux_sym_on_keyword_token1] = ACTIONS(1407),
    [aux_sym_off_keyword_token1] = ACTIONS(1407),
    [sym_send_level_keyword] = ACTIONS(1407),
    [sym_send_string_keyword] = ACTIONS(1407),
    [sym_send_command_keyword] = ACTIONS(1407),
    [sym_clear_buffer_keyword] = ACTIONS(1407),
    [sym_create_buffer_keyword] = ACTIONS(1407),
    [sym_create_multi_buffer_keyword] = ACTIONS(1407),
    [sym_call_keyword] = ACTIONS(1407),
    [sym_system_call_keyword] = ACTIONS(1407),
    [sym_devchan_to_keyword] = ACTIONS(1407),
    [sym_devchan_min_to_keyword] = ACTIONS(1407),
    [sym_devchan_total_off_keyword] = ACTIONS(1407),
    [sym_devchan_pulse_keyword] = ACTIONS(1407),
    [sym_wait_keyword] = ACTIONS(1407),
    [sym_cancel_wait_keyword] = ACTIONS(1407),
    [sym_wait_until_keyword] = ACTIONS(1407),
    [sym_cancel_wait_until_keyword] = ACTIONS(1407),
    [sym_cancel_all_wait_keyword] = ACTIONS(1407),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1407),
    [sym_and] = ACTIONS(1417),
    [sym_or] = ACTIONS(1419),
    [sym_not] = ACTIONS(1407),
    [sym_xor] = ACTIONS(1421),
    [sym_band] = ACTIONS(1423),
    [sym_bor] = ACTIONS(1425),
    [sym_bxor] = ACTIONS(1421),
    [sym_bnot] = ACTIONS(1407),
    [sym_lshift] = ACTIONS(1427),
    [sym_rshift] = ACTIONS(1427),
    [sym___file__] = ACTIONS(1407),
    [sym___date__] = ACTIONS(1407),
    [sym___ldate__] = ACTIONS(1407),
    [sym___time__] = ACTIONS(1407),
    [sym___line__] = ACTIONS(1407),
    [sym___name__] = ACTIONS(1407),
    [sym___netlinx__] = ACTIONS(1407),
    [sym_netlinx_axi_version] = ACTIONS(1407),
    [sym_url_flg_tcp] = ACTIONS(1407),
    [sym_url_flg_accinfopresent] = ACTIONS(1407),
    [sym_url_flg_temp] = ACTIONS(1407),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1407),
    [sym_url_flg_stat_mask] = ACTIONS(1407),
    [sym_url_flg_stat_lookup] = ACTIONS(1407),
    [sym_url_flg_stat_connecting] = ACTIONS(1407),
    [sym_url_flg_stat_waiting] = ACTIONS(1407),
    [sym_url_flg_stat_connected] = ACTIONS(1407),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1407),
    [sym_first_virtual_device] = ACTIONS(1407),
    [sym_first_local_port] = ACTIONS(1407),
    [sym_dynamic_virtual_device] = ACTIONS(1407),
    [sym_timeline_once] = ACTIONS(1407),
    [sym_timeline_repeat] = ACTIONS(1407),
    [sym_timeline_absolute] = ACTIONS(1407),
    [sym_timeline_relative] = ACTIONS(1407),
    [sym_do_push_timed_infinite] = ACTIONS(1407),
    [sym_source_type_no_address] = ACTIONS(1407),
    [sym_source_type_neuron_id] = ACTIONS(1407),
    [sym_source_type_ip_address] = ACTIONS(1407),
    [sym_source_type_axlink] = ACTIONS(1407),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1407),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1407),
    [sym_source_type_ip_socket_address] = ACTIONS(1407),
    [sym_source_type_rs232] = ACTIONS(1407),
    [sym_source_type_internal] = ACTIONS(1407),
    [sym_source_type_mac_address] = ACTIONS(1407),
    [sym_source_type_ipv4_port] = ACTIONS(1407),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1407),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1407),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1407),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1407),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1407),
    [sym_true] = ACTIONS(1407),
    [sym_false] = ACTIONS(1407),
    [sym_file_read_only] = ACTIONS(1407),
    [sym_file_rw_new] = ACTIONS(1407),
    [sym_file_rw_append] = ACTIONS(1407),
    [sym_ip_tcp] = ACTIONS(1407),
    [sym_ip_udp] = ACTIONS(1407),
    [sym_ip_udp_2way] = ACTIONS(1407),
    [sym_xml_encode_types] = ACTIONS(1407),
    [sym_xml_encode_char_as_list] = ACTIONS(1407),
    [sym_xml_encode_le] = ACTIONS(1407),
    [sym_xml_decode_types] = ACTIONS(1407),
    [sym_xml_decode_no_preserve] = ACTIONS(1407),
    [sym_duet_dev_type_display_device] = ACTIONS(1407),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1407),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1407),
    [sym_duet_dev_type_io_device] = ACTIONS(1407),
    [sym_duet_dev_type_relay_device] = ACTIONS(1407),
    [sym_duet_dev_type_ups] = ACTIONS(1407),
    [sym_duet_dev_type_amplifier] = ACTIONS(1407),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1407),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1407),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1407),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1407),
    [sym_duet_dev_type_camera] = ACTIONS(1407),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1407),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1407),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1407),
    [sym_duet_dev_type_dss] = ACTIONS(1407),
    [sym_duet_dev_type_dvr] = ACTIONS(1407),
    [sym_duet_dev_type_disc_device] = ACTIONS(1407),
    [sym_duet_dev_type_document_camera] = ACTIONS(1407),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1407),
    [sym_duet_dev_type_hvac] = ACTIONS(1407),
    [sym_duet_dev_type_keypad] = ACTIONS(1407),
    [sym_duet_dev_type_light] = ACTIONS(1407),
    [sym_duet_dev_type_monitor] = ACTIONS(1407),
    [sym_duet_dev_type_motor] = ACTIONS(1407),
    [sym_duet_dev_type_multi_window] = ACTIONS(1407),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1407),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1407),
    [sym_duet_dev_type_receiver] = ACTIONS(1407),
    [sym_duet_dev_type_security_system] = ACTIONS(1407),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1407),
    [sym_duet_dev_type_settop_box] = ACTIONS(1407),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1407),
    [sym_duet_dev_type_switcher] = ACTIONS(1407),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1407),
    [sym_duet_dev_type_tv] = ACTIONS(1407),
    [sym_duet_dev_type_utility] = ACTIONS(1407),
    [sym_duet_dev_type_vcr] = ACTIONS(1407),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1407),
    [sym_duet_dev_type_video_processor] = ACTIONS(1407),
    [sym_duet_dev_type_video_projector] = ACTIONS(1407),
    [sym_duet_dev_type_video_wall] = ACTIONS(1407),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1407),
    [sym_duet_dev_type_weather] = ACTIONS(1407),
    [sym_duet_dev_not_polled] = ACTIONS(1407),
    [sym_duet_dev_polled] = ACTIONS(1407),
    [sym_clkmgr_mode_network] = ACTIONS(1407),
    [sym_clkmgr_mode_standalone] = ACTIONS(1407),
    [sym_normal_standby] = ACTIONS(1407),
    [sym_normal_wake] = ACTIONS(1407),
    [sym_ip_multicast_ttl_option] = ACTIONS(1407),
    [sym_ip_ttl_subnet] = ACTIONS(1407),
    [sym_ip_ttl_site] = ACTIONS(1407),
    [sym_ip_ttl_region] = ACTIONS(1407),
    [sym_ip_ttl_continent] = ACTIONS(1407),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1407),
    [sym_ip_nodelay_on] = ACTIONS(1407),
    [sym_ip_nodelay_off] = ACTIONS(1407),
    [sym_amx_error] = ACTIONS(1407),
    [sym_amx_warning] = ACTIONS(1407),
    [sym_amx_info] = ACTIONS(1407),
    [sym_amx_debug] = ACTIONS(1407),
    [sym_netlinx_login_success] = ACTIONS(1407),
    [sym_netlinx_login_fail] = ACTIONS(1407),
    [sym_netlinx_logout] = ACTIONS(1407),
    [sym_null_str] = ACTIONS(1407),
    [sym_smtp_address] = ACTIONS(1407),
    [sym_smtp_port_number] = ACTIONS(1407),
    [sym_smtp_username] = ACTIONS(1407),
    [sym_smtp_password] = ACTIONS(1407),
    [sym_smtp_require_tls] = ACTIONS(1407),
    [sym_smtp_from] = ACTIONS(1407),
    [sym_smtp_tls_true] = ACTIONS(1407),
    [sym_smtp_tls_false] = ACTIONS(1407),
    [sym_listview_on_row_select_event] = ACTIONS(1407),
    [sym_data_structure_unknown] = ACTIONS(1407),
    [sym_data_structure_datafeed] = ACTIONS(1407),
    [sym_data_structure_datarecordset] = ACTIONS(1407),
    [sym_data_structure_datarecord] = ACTIONS(1407),
    [sym_data_structure_datafield] = ACTIONS(1407),
    [sym_data_max_content_field_count] = ACTIONS(1407),
    [sym_data_max_metadata_field_count] = ACTIONS(1407),
    [sym_data_max_id_length] = ACTIONS(1407),
    [sym_data_max_name_length] = ACTIONS(1407),
    [sym_data_max_description_length] = ACTIONS(1407),
    [sym_data_max_type_length] = ACTIONS(1407),
    [sym_data_max_format_length] = ACTIONS(1407),
    [sym_data_max_source_length] = ACTIONS(1407),
    [sym_data_max_value_length] = ACTIONS(1407),
    [sym_data_max_label_length] = ACTIONS(1407),
    [sym_data_max_url_length] = ACTIONS(1407),
    [sym_max_last_login_info_length] = ACTIONS(1407),
    [sym_data_type_unknown] = ACTIONS(1407),
    [sym_data_type_string] = ACTIONS(1407),
    [sym_data_type_datetime] = ACTIONS(1407),
    [sym_data_type_date] = ACTIONS(1407),
    [sym_data_type_time] = ACTIONS(1407),
    [sym_data_type_image] = ACTIONS(1407),
    [sym_data_format_url] = ACTIONS(1407),
    [sym_data_format_phone] = ACTIONS(1407),
    [sym_data_format_email] = ACTIONS(1407),
    [sym_data_format_iso8601] = ACTIONS(1407),
    [sym_auditlog_priv] = ACTIONS(1407),
    [sym_configuration_priv] = ACTIONS(1407),
    [sym_deviceconfig_priv] = ACTIONS(1407),
    [sym_ftp_priv] = ACTIONS(1407),
    [sym_http_priv] = ACTIONS(1407),
    [sym_networkconfig_priv] = ACTIONS(1407),
    [sym_programport_priv] = ACTIONS(1407),
    [sym_remoteui_priv] = ACTIONS(1407),
    [sym_securitycontrol_priv] = ACTIONS(1407),
    [sym_softwaremanagement_priv] = ACTIONS(1407),
    [sym_terminal_priv] = ACTIONS(1407),
    [sym_tpadmin_priv] = ACTIONS(1407),
    [sym_user1_priv] = ACTIONS(1407),
    [sym_user2_priv] = ACTIONS(1407),
    [sym_user3_priv] = ACTIONS(1407),
    [sym_user4_priv] = ACTIONS(1407),
    [sym_usermanagement_priv] = ACTIONS(1407),
    [sym_valid_account] = ACTIONS(1407),
    [sym_err_name_invalid] = ACTIONS(1407),
    [sym_err_password_invalid] = ACTIONS(1407),
    [sym_err_invalid_account] = ACTIONS(1407),
    [sym_err_no_authorization] = ACTIONS(1407),
    [sym_err_invalid_parameter] = ACTIONS(1407),
    [sym_err_user_locked_out] = ACTIONS(1407),
    [sym_err_user_account_expired] = ACTIONS(1407),
    [sym_err_general_error] = ACTIONS(1407),
    [sym_err_authentication_error] = ACTIONS(1407),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1407),
    [sym_tls_validate_certificate] = ACTIONS(1407),
    [sym_tdata] = ACTIONS(1407),
    [sym_tchannel] = ACTIONS(1407),
    [sym_tlevel] = ACTIONS(1407),
    [sym_tbutton] = ACTIONS(1407),
    [sym_ttimeline] = ACTIONS(1407),
    [sym_tcustom] = ACTIONS(1407),
    [sym_url_struct] = ACTIONS(1407),
    [sym_dns_struct] = ACTIONS(1407),
    [sym_ip_address_struct] = ACTIONS(1407),
    [sym_dev_info_struct] = ACTIONS(1407),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1407),
    [sym_data_feed] = ACTIONS(1407),
    [sym_data_field] = ACTIONS(1407),
    [sym_data_record] = ACTIONS(1407),
    [sym_wc_data_feed] = ACTIONS(1407),
    [sym_wc_data_field] = ACTIONS(1407),
    [sym_wc_data_record] = ACTIONS(1407),
    [sym_last_login_info] = ACTIONS(1407),
    [sym____reserved___] = ACTIONS(1407),
    [sym_date] = ACTIONS(1407),
    [sym_day] = ACTIONS(1407),
    [sym_time] = ACTIONS(1407),
    [sym_ldate] = ACTIONS(1407),
    [sym_push_device] = ACTIONS(1407),
    [sym_push_channel] = ACTIONS(1407),
    [sym_push_devchan] = ACTIONS(1407),
    [sym_release_device] = ACTIONS(1407),
    [sym_release_channel] = ACTIONS(1407),
    [sym_release_devchan] = ACTIONS(1407),
    [sym_master_slot] = ACTIONS(1407),
    [sym_get_pulse_time] = ACTIONS(1407),
    [sym_get_timer] = ACTIONS(1407),
    [sym_master_sn] = ACTIONS(1407),
    [sym_system_number] = ACTIONS(1407),
    [sym_button] = ACTIONS(1407),
    [sym_dv_channel] = ACTIONS(1407),
    [sym_data] = ACTIONS(1407),
    [sym_level] = ACTIONS(1407),
    [sym_timeline] = ACTIONS(1407),
    [sym_channel] = ACTIONS(1407),
    [sym_custom] = ACTIONS(1407),
    [sym_atoi] = ACTIONS(1407),
    [sym_atol] = ACTIONS(1407),
    [sym_atof] = ACTIONS(1407),
    [sym_itoa] = ACTIONS(1407),
    [sym_itohex] = ACTIONS(1407),
    [sym_hextoi] = ACTIONS(1407),
    [sym_ftoa] = ACTIONS(1407),
    [sym_find_string] = ACTIONS(1407),
    [sym_left_string] = ACTIONS(1407),
    [sym_right_string] = ACTIONS(1407),
    [sym_mid_string] = ACTIONS(1407),
    [sym_remove_string] = ACTIONS(1407),
    [sym_get_buffer_string] = ACTIONS(1407),
    [sym_get_buffer_char] = ACTIONS(1407),
    [sym_get_multi_buffer_string] = ACTIONS(1407),
    [sym_length_string] = ACTIONS(1407),
    [sym_max_length_string] = ACTIONS(1407),
    [sym_set_length_string] = ACTIONS(1407),
    [sym_compare_string] = ACTIONS(1407),
    [sym_upper_string] = ACTIONS(1407),
    [sym_lower_string] = ACTIONS(1407),
    [sym_get_last] = ACTIONS(1407),
    [sym_date_to_day] = ACTIONS(1407),
    [sym_date_to_month] = ACTIONS(1407),
    [sym_date_to_year] = ACTIONS(1407),
    [sym_time_to_hour] = ACTIONS(1407),
    [sym_time_to_minute] = ACTIONS(1407),
    [sym_time_to_second] = ACTIONS(1407),
    [sym_day_of_week] = ACTIONS(1407),
    [sym_set_pulse_time] = ACTIONS(1407),
    [sym_set_timer] = ACTIONS(1407),
    [sym_file_open] = ACTIONS(1407),
    [sym_file_close] = ACTIONS(1407),
    [sym_file_read] = ACTIONS(1407),
    [sym_file_read_line] = ACTIONS(1407),
    [sym_file_write] = ACTIONS(1407),
    [sym_file_write_line] = ACTIONS(1407),
    [sym_file_copy] = ACTIONS(1407),
    [sym_file_delete] = ACTIONS(1407),
    [sym_file_rename] = ACTIONS(1407),
    [sym_file_dir] = ACTIONS(1407),
    [sym_file_getdir] = ACTIONS(1407),
    [sym_file_setdir] = ACTIONS(1407),
    [sym_file_create_dir] = ACTIONS(1407),
    [sym_file_remove_dir] = ACTIONS(1407),
    [sym_file_seek] = ACTIONS(1407),
    [sym_length_array] = ACTIONS(1407),
    [sym_max_length_array] = ACTIONS(1407),
    [sym_set_length_array] = ACTIONS(1407),
    [sym_ip_client_open] = ACTIONS(1407),
    [sym_ip_client_close] = ACTIONS(1407),
    [sym_ip_server_open] = ACTIONS(1407),
    [sym_ip_server_close] = ACTIONS(1407),
    [sym_ip_bound_client_open] = ACTIONS(1407),
    [sym_ip_mc_server_open] = ACTIONS(1407),
    [sym_ip_set_option] = ACTIONS(1407),
    [sym_ssh_client_open] = ACTIONS(1407),
    [sym_ssh_client_close] = ACTIONS(1407),
    [sym_tls_client_open] = ACTIONS(1407),
    [sym_tls_client_close] = ACTIONS(1407),
    [sym_do_push] = ACTIONS(1407),
    [sym_do_push_timed] = ACTIONS(1407),
    [sym_do_release] = ACTIONS(1407),
    [sym_device_id] = ACTIONS(1407),
    [sym_device_info] = ACTIONS(1407),
    [sym_device_id_string] = ACTIONS(1407),
    [sym_device_standby] = ACTIONS(1407),
    [sym_device_wake] = ACTIONS(1407),
    [sym_timeline_create] = ACTIONS(1407),
    [sym_timeline_kill] = ACTIONS(1407),
    [sym_timeline_reload] = ACTIONS(1407),
    [sym_timeline_pause] = ACTIONS(1407),
    [sym_timeline_restart] = ACTIONS(1407),
    [sym_timeline_set] = ACTIONS(1407),
    [sym_timeline_get] = ACTIONS(1407),
    [sym_timeline_active] = ACTIONS(1407),
    [sym_timeline_dynamic_id] = ACTIONS(1407),
    [sym_variable_to_string] = ACTIONS(1407),
    [sym_string_to_variable] = ACTIONS(1407),
    [sym_length_variable_to_string] = ACTIONS(1407),
    [sym_type_cast] = ACTIONS(1407),
    [sym_variable_to_xml] = ACTIONS(1407),
    [sym_xml_to_variable] = ACTIONS(1407),
    [sym_length_variable_to_xml] = ACTIONS(1407),
    [sym_redirect_string] = ACTIONS(1407),
    [sym_random_number] = ACTIONS(1407),
    [sym_get_unique_id] = ACTIONS(1407),
    [sym_get_system_number] = ACTIONS(1407),
    [sym_set_system_number] = ACTIONS(1407),
    [sym_get_serial_number] = ACTIONS(1407),
    [sym_get_master_build] = ACTIONS(1407),
    [sym_set_validation_code] = ACTIONS(1407),
    [sym_reboot] = ACTIONS(1407),
    [sym_load_duet_module] = ACTIONS(1407),
    [sym_unload_duet_module] = ACTIONS(1407),
    [sym_duet_mem_size_set] = ACTIONS(1407),
    [sym_duet_mem_size_get] = ACTIONS(1407),
    [sym_dynamic_application_device] = ACTIONS(1407),
    [sym_dynamic_polled_port] = ACTIONS(1407),
    [sym_static_port_binding] = ACTIONS(1407),
    [sym_static_ip_binding] = ACTIONS(1407),
    [sym_get_dns_list] = ACTIONS(1407),
    [sym_set_dns_list] = ACTIONS(1407),
    [sym_get_ip_address] = ACTIONS(1407),
    [sym_set_ip_address] = ACTIONS(1407),
    [sym_get_url_list] = ACTIONS(1407),
    [sym_add_url_entry] = ACTIONS(1407),
    [sym_delete_url_entry] = ACTIONS(1407),
    [sym_abs_value] = ACTIONS(1407),
    [sym_max_value] = ACTIONS(1407),
    [sym_min_value] = ACTIONS(1407),
    [sym_exp_value] = ACTIONS(1407),
    [sym_log_value] = ACTIONS(1407),
    [sym_log10_value] = ACTIONS(1407),
    [sym_power_value] = ACTIONS(1407),
    [sym_sqrt_value] = ACTIONS(1407),
    [sym_raw_be] = ACTIONS(1407),
    [sym_raw_le] = ACTIONS(1407),
    [sym_internal_queue_size_set] = ACTIONS(1407),
    [sym_internal_queue_size_get] = ACTIONS(1407),
    [sym_internal_threshold_set] = ACTIONS(1407),
    [sym_internal_threshold_get] = ACTIONS(1407),
    [sym_rebuilt_event] = ACTIONS(1407),
    [sym_do_custom_event] = ACTIONS(1407),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1407),
    [sym_clkmgr_set_clk_source] = ACTIONS(1407),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1407),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1407),
    [sym_clkmgr_get_timezone] = ACTIONS(1407),
    [sym_clkmgr_set_timezone] = ACTIONS(1407),
    [sym_clkmgr_get_resync_period] = ACTIONS(1407),
    [sym_clkmgr_set_resync_period] = ACTIONS(1407),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1407),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1407),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1407),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1407),
    [sym_clkmgr_get_timeservers] = ACTIONS(1407),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1407),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1407),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1407),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1407),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1407),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1407),
    [sym_set_log_level] = ACTIONS(1407),
    [sym_get_log_level] = ACTIONS(1407),
    [sym_amx_log] = ACTIONS(1407),
    [sym_smtp_server_config_set] = ACTIONS(1407),
    [sym_smtp_server_config_get] = ACTIONS(1407),
    [sym_smtp_send] = ACTIONS(1407),
    [sym_validate_netlinx_account] = ACTIONS(1407),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1407),
    [sym_authenticate_certificate] = ACTIONS(1407),
    [sym_audit_netlinx_session_event] = ACTIONS(1407),
    [sym_audit_netlinx_generic_event] = ACTIONS(1407),
    [sym_get_available_flash_disk_space] = ACTIONS(1407),
    [sym_get_max_flash_disk_space] = ACTIONS(1407),
    [sym_data_create_feed] = ACTIONS(1407),
    [sym_data_delete_feed] = ACTIONS(1407),
    [sym_data_publish_feed] = ACTIONS(1407),
    [sym_data_get_published_feed] = ACTIONS(1407),
    [sym_data_add_record] = ACTIONS(1407),
    [sym_data_get_event_record] = ACTIONS(1407),
    [sym__wc_data_create_feed] = ACTIONS(1407),
    [sym__wc_data_add_record] = ACTIONS(1407),
    [sym__wc_data_get_event_record] = ACTIONS(1407),
    [sym_format] = ACTIONS(1407),
    [sym_astro_clock] = ACTIONS(1407),
    [sym_set_outdoor_temperature] = ACTIONS(1407),
    [sym_set_virtual_level_count] = ACTIONS(1407),
    [sym_set_virtual_channel_count] = ACTIONS(1407),
    [sym_set_virtual_port_count] = ACTIONS(1407),
    [sym__wc_encode_int] = ACTIONS(1407),
    [sym__wc_decode_int] = ACTIONS(1407),
    [sym__wc_int] = ACTIONS(1407),
    [sym__wc_to_ch_int] = ACTIONS(1407),
    [sym__ch_to_wc_int] = ACTIONS(1407),
    [sym__wc_find_string_int] = ACTIONS(1407),
    [sym__wc_left_string_int] = ACTIONS(1407),
    [sym__wc_lower_string_int] = ACTIONS(1407),
    [sym__wc_mid_string_int] = ACTIONS(1407),
    [sym__wc_remove_string_int] = ACTIONS(1407),
    [sym__wc_right_string_int] = ACTIONS(1407),
    [sym__wc_upper_string_int] = ACTIONS(1407),
    [sym__wc_compare_string_int] = ACTIONS(1407),
    [sym__wc_get_buffer_char_int] = ACTIONS(1407),
    [sym__wc_get_buffer_string_int] = ACTIONS(1407),
    [sym__wc_concat_string_int] = ACTIONS(1407),
    [sym__wc_file_open_int] = ACTIONS(1407),
    [sym__wc_file_close_int] = ACTIONS(1407),
    [sym__wc_file_read_int] = ACTIONS(1407),
    [sym__wc_file_read_line_int] = ACTIONS(1407),
    [sym__wc_file_write_int] = ACTIONS(1407),
    [sym__wc_file_write_line_int] = ACTIONS(1407),
    [aux_sym_preproc_include_token1] = ACTIONS(1407),
    [aux_sym_preproc_define_token1] = ACTIONS(1409),
    [aux_sym_preproc_warn_token1] = ACTIONS(1409),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1409),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1409),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1409),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1409),
    [anon_sym_COLON] = ACTIONS(1431),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1435),
    [anon_sym_BANG] = ACTIONS(1407),
    [anon_sym_TILDE] = ACTIONS(1409),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1443),
    [anon_sym_AMP_AMP] = ACTIONS(1445),
    [anon_sym_PIPE] = ACTIONS(1425),
    [anon_sym_CARET] = ACTIONS(1447),
    [anon_sym_AMP] = ACTIONS(1423),
    [anon_sym_EQ_EQ] = ACTIONS(1449),
    [anon_sym_BANG_EQ] = ACTIONS(1449),
    [anon_sym_LT_GT] = ACTIONS(1449),
    [anon_sym_GT] = ACTIONS(1451),
    [anon_sym_GT_EQ] = ACTIONS(1453),
    [anon_sym_LT_EQ] = ACTIONS(1453),
    [anon_sym_LT] = ACTIONS(1451),
    [anon_sym_LT_LT] = ACTIONS(1455),
    [anon_sym_GT_GT] = ACTIONS(1455),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1409),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(1409),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1435),
  },
  [STATE(61)] = {
    [sym_argument_list] = STATE(131),
    [sym_string_literal] = STATE(202),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1371),
    [sym_identifier] = ACTIONS(1357),
    [sym_program_name_keyword] = ACTIONS(1357),
    [sym_module_name_keyword] = ACTIONS(1357),
    [sym_define_device_keyword] = ACTIONS(1357),
    [sym_define_combine_keyword] = ACTIONS(1357),
    [sym_define_constant_keyword] = ACTIONS(1357),
    [sym_define_type_keyword] = ACTIONS(1357),
    [sym_define_variable_keyword] = ACTIONS(1357),
    [sym_define_system_variable_keyword] = ACTIONS(1357),
    [sym_define_connect_level_keyword] = ACTIONS(1357),
    [sym_define_latching_keyword] = ACTIONS(1357),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1357),
    [sym_define_toggling_keyword] = ACTIONS(1357),
    [sym_define_call_keyword] = ACTIONS(1357),
    [sym_define_function_keyword] = ACTIONS(1357),
    [sym_define_library_function_keyword] = ACTIONS(1357),
    [sym_define_start_keyword] = ACTIONS(1357),
    [sym_define_event_keyword] = ACTIONS(1357),
    [sym_define_program_keyword] = ACTIONS(1357),
    [sym_define_module_keyword] = ACTIONS(1357),
    [sym_char_keyword] = ACTIONS(1357),
    [sym_widechar_keyword] = ACTIONS(1357),
    [sym_integer_keyword] = ACTIONS(1357),
    [sym_sinteger_keyword] = ACTIONS(1357),
    [sym_long_keyword] = ACTIONS(1357),
    [sym_slong_keyword] = ACTIONS(1357),
    [sym_float_keyword] = ACTIONS(1357),
    [sym_double_keyword] = ACTIONS(1357),
    [sym_variant_keyword] = ACTIONS(1357),
    [sym_variantarray_keyword] = ACTIONS(1357),
    [sym_local_var_keyword] = ACTIONS(1357),
    [sym_stack_var_keyword] = ACTIONS(1357),
    [sym_constant_keyword] = ACTIONS(1357),
    [sym_volatile_keyword] = ACTIONS(1357),
    [sym_non_volatile_keyword] = ACTIONS(1357),
    [sym_persistent_keyword] = ACTIONS(1357),
    [sym_dev_keyword] = ACTIONS(1357),
    [sym_devlev_keyword] = ACTIONS(1357),
    [sym_devchan_keyword] = ACTIONS(1357),
    [sym_if_keyword] = ACTIONS(1357),
    [sym_else_keyword] = ACTIONS(1357),
    [sym_switch_keyword] = ACTIONS(1357),
    [sym_case_keyword] = ACTIONS(1357),
    [sym_default_keyword] = ACTIONS(1357),
    [sym_while_keyword] = ACTIONS(1357),
    [sym_for_keyword] = ACTIONS(1357),
    [sym_break_keyword] = ACTIONS(1357),
    [sym_continue_keyword] = ACTIONS(1357),
    [sym_return_keyword] = ACTIONS(1357),
    [sym_select_keyword] = ACTIONS(1357),
    [sym_active_keyword] = ACTIONS(1357),
    [sym_struct_keyword] = ACTIONS(1357),
    [sym_structure_keyword] = ACTIONS(1357),
    [sym_button_event_keyword] = ACTIONS(1357),
    [sym_channel_event_keyword] = ACTIONS(1357),
    [sym_level_event_keyword] = ACTIONS(1357),
    [sym_data_event_keyword] = ACTIONS(1357),
    [sym_timeline_event_keyword] = ACTIONS(1357),
    [sym_custom_event_keyword] = ACTIONS(1357),
    [sym_push_keyword] = ACTIONS(1357),
    [sym_release_keyword] = ACTIONS(1357),
    [aux_sym_on_keyword_token1] = ACTIONS(1357),
    [aux_sym_off_keyword_token1] = ACTIONS(1357),
    [sym_send_level_keyword] = ACTIONS(1357),
    [sym_send_string_keyword] = ACTIONS(1357),
    [sym_send_command_keyword] = ACTIONS(1357),
    [sym_clear_buffer_keyword] = ACTIONS(1357),
    [sym_create_buffer_keyword] = ACTIONS(1357),
    [sym_create_multi_buffer_keyword] = ACTIONS(1357),
    [sym_call_keyword] = ACTIONS(1357),
    [sym_system_call_keyword] = ACTIONS(1357),
    [sym_devchan_to_keyword] = ACTIONS(1357),
    [sym_devchan_min_to_keyword] = ACTIONS(1357),
    [sym_devchan_total_off_keyword] = ACTIONS(1357),
    [sym_devchan_pulse_keyword] = ACTIONS(1357),
    [sym_wait_keyword] = ACTIONS(1357),
    [sym_cancel_wait_keyword] = ACTIONS(1357),
    [sym_wait_until_keyword] = ACTIONS(1357),
    [sym_cancel_wait_until_keyword] = ACTIONS(1357),
    [sym_cancel_all_wait_keyword] = ACTIONS(1357),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1357),
    [sym_and] = ACTIONS(1417),
    [sym_or] = ACTIONS(1419),
    [sym_not] = ACTIONS(1357),
    [sym_xor] = ACTIONS(1421),
    [sym_band] = ACTIONS(1423),
    [sym_bor] = ACTIONS(1425),
    [sym_bxor] = ACTIONS(1421),
    [sym_bnot] = ACTIONS(1357),
    [sym_lshift] = ACTIONS(1427),
    [sym_rshift] = ACTIONS(1427),
    [sym___file__] = ACTIONS(1357),
    [sym___date__] = ACTIONS(1357),
    [sym___ldate__] = ACTIONS(1357),
    [sym___time__] = ACTIONS(1357),
    [sym___line__] = ACTIONS(1357),
    [sym___name__] = ACTIONS(1357),
    [sym___netlinx__] = ACTIONS(1357),
    [sym_netlinx_axi_version] = ACTIONS(1357),
    [sym_url_flg_tcp] = ACTIONS(1357),
    [sym_url_flg_accinfopresent] = ACTIONS(1357),
    [sym_url_flg_temp] = ACTIONS(1357),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1357),
    [sym_url_flg_stat_mask] = ACTIONS(1357),
    [sym_url_flg_stat_lookup] = ACTIONS(1357),
    [sym_url_flg_stat_connecting] = ACTIONS(1357),
    [sym_url_flg_stat_waiting] = ACTIONS(1357),
    [sym_url_flg_stat_connected] = ACTIONS(1357),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1357),
    [sym_first_virtual_device] = ACTIONS(1357),
    [sym_first_local_port] = ACTIONS(1357),
    [sym_dynamic_virtual_device] = ACTIONS(1357),
    [sym_timeline_once] = ACTIONS(1357),
    [sym_timeline_repeat] = ACTIONS(1357),
    [sym_timeline_absolute] = ACTIONS(1357),
    [sym_timeline_relative] = ACTIONS(1357),
    [sym_do_push_timed_infinite] = ACTIONS(1357),
    [sym_source_type_no_address] = ACTIONS(1357),
    [sym_source_type_neuron_id] = ACTIONS(1357),
    [sym_source_type_ip_address] = ACTIONS(1357),
    [sym_source_type_axlink] = ACTIONS(1357),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1357),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1357),
    [sym_source_type_ip_socket_address] = ACTIONS(1357),
    [sym_source_type_rs232] = ACTIONS(1357),
    [sym_source_type_internal] = ACTIONS(1357),
    [sym_source_type_mac_address] = ACTIONS(1357),
    [sym_source_type_ipv4_port] = ACTIONS(1357),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1357),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1357),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1357),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1357),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1357),
    [sym_true] = ACTIONS(1357),
    [sym_false] = ACTIONS(1357),
    [sym_file_read_only] = ACTIONS(1357),
    [sym_file_rw_new] = ACTIONS(1357),
    [sym_file_rw_append] = ACTIONS(1357),
    [sym_ip_tcp] = ACTIONS(1357),
    [sym_ip_udp] = ACTIONS(1357),
    [sym_ip_udp_2way] = ACTIONS(1357),
    [sym_xml_encode_types] = ACTIONS(1357),
    [sym_xml_encode_char_as_list] = ACTIONS(1357),
    [sym_xml_encode_le] = ACTIONS(1357),
    [sym_xml_decode_types] = ACTIONS(1357),
    [sym_xml_decode_no_preserve] = ACTIONS(1357),
    [sym_duet_dev_type_display_device] = ACTIONS(1357),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1357),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1357),
    [sym_duet_dev_type_io_device] = ACTIONS(1357),
    [sym_duet_dev_type_relay_device] = ACTIONS(1357),
    [sym_duet_dev_type_ups] = ACTIONS(1357),
    [sym_duet_dev_type_amplifier] = ACTIONS(1357),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1357),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1357),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1357),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1357),
    [sym_duet_dev_type_camera] = ACTIONS(1357),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1357),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1357),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1357),
    [sym_duet_dev_type_dss] = ACTIONS(1357),
    [sym_duet_dev_type_dvr] = ACTIONS(1357),
    [sym_duet_dev_type_disc_device] = ACTIONS(1357),
    [sym_duet_dev_type_document_camera] = ACTIONS(1357),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1357),
    [sym_duet_dev_type_hvac] = ACTIONS(1357),
    [sym_duet_dev_type_keypad] = ACTIONS(1357),
    [sym_duet_dev_type_light] = ACTIONS(1357),
    [sym_duet_dev_type_monitor] = ACTIONS(1357),
    [sym_duet_dev_type_motor] = ACTIONS(1357),
    [sym_duet_dev_type_multi_window] = ACTIONS(1357),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1357),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1357),
    [sym_duet_dev_type_receiver] = ACTIONS(1357),
    [sym_duet_dev_type_security_system] = ACTIONS(1357),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1357),
    [sym_duet_dev_type_settop_box] = ACTIONS(1357),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1357),
    [sym_duet_dev_type_switcher] = ACTIONS(1357),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1357),
    [sym_duet_dev_type_tv] = ACTIONS(1357),
    [sym_duet_dev_type_utility] = ACTIONS(1357),
    [sym_duet_dev_type_vcr] = ACTIONS(1357),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1357),
    [sym_duet_dev_type_video_processor] = ACTIONS(1357),
    [sym_duet_dev_type_video_projector] = ACTIONS(1357),
    [sym_duet_dev_type_video_wall] = ACTIONS(1357),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1357),
    [sym_duet_dev_type_weather] = ACTIONS(1357),
    [sym_duet_dev_not_polled] = ACTIONS(1357),
    [sym_duet_dev_polled] = ACTIONS(1357),
    [sym_clkmgr_mode_network] = ACTIONS(1357),
    [sym_clkmgr_mode_standalone] = ACTIONS(1357),
    [sym_normal_standby] = ACTIONS(1357),
    [sym_normal_wake] = ACTIONS(1357),
    [sym_ip_multicast_ttl_option] = ACTIONS(1357),
    [sym_ip_ttl_subnet] = ACTIONS(1357),
    [sym_ip_ttl_site] = ACTIONS(1357),
    [sym_ip_ttl_region] = ACTIONS(1357),
    [sym_ip_ttl_continent] = ACTIONS(1357),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1357),
    [sym_ip_nodelay_on] = ACTIONS(1357),
    [sym_ip_nodelay_off] = ACTIONS(1357),
    [sym_amx_error] = ACTIONS(1357),
    [sym_amx_warning] = ACTIONS(1357),
    [sym_amx_info] = ACTIONS(1357),
    [sym_amx_debug] = ACTIONS(1357),
    [sym_netlinx_login_success] = ACTIONS(1357),
    [sym_netlinx_login_fail] = ACTIONS(1357),
    [sym_netlinx_logout] = ACTIONS(1357),
    [sym_null_str] = ACTIONS(1357),
    [sym_smtp_address] = ACTIONS(1357),
    [sym_smtp_port_number] = ACTIONS(1357),
    [sym_smtp_username] = ACTIONS(1357),
    [sym_smtp_password] = ACTIONS(1357),
    [sym_smtp_require_tls] = ACTIONS(1357),
    [sym_smtp_from] = ACTIONS(1357),
    [sym_smtp_tls_true] = ACTIONS(1357),
    [sym_smtp_tls_false] = ACTIONS(1357),
    [sym_listview_on_row_select_event] = ACTIONS(1357),
    [sym_data_structure_unknown] = ACTIONS(1357),
    [sym_data_structure_datafeed] = ACTIONS(1357),
    [sym_data_structure_datarecordset] = ACTIONS(1357),
    [sym_data_structure_datarecord] = ACTIONS(1357),
    [sym_data_structure_datafield] = ACTIONS(1357),
    [sym_data_max_content_field_count] = ACTIONS(1357),
    [sym_data_max_metadata_field_count] = ACTIONS(1357),
    [sym_data_max_id_length] = ACTIONS(1357),
    [sym_data_max_name_length] = ACTIONS(1357),
    [sym_data_max_description_length] = ACTIONS(1357),
    [sym_data_max_type_length] = ACTIONS(1357),
    [sym_data_max_format_length] = ACTIONS(1357),
    [sym_data_max_source_length] = ACTIONS(1357),
    [sym_data_max_value_length] = ACTIONS(1357),
    [sym_data_max_label_length] = ACTIONS(1357),
    [sym_data_max_url_length] = ACTIONS(1357),
    [sym_max_last_login_info_length] = ACTIONS(1357),
    [sym_data_type_unknown] = ACTIONS(1357),
    [sym_data_type_string] = ACTIONS(1357),
    [sym_data_type_datetime] = ACTIONS(1357),
    [sym_data_type_date] = ACTIONS(1357),
    [sym_data_type_time] = ACTIONS(1357),
    [sym_data_type_image] = ACTIONS(1357),
    [sym_data_format_url] = ACTIONS(1357),
    [sym_data_format_phone] = ACTIONS(1357),
    [sym_data_format_email] = ACTIONS(1357),
    [sym_data_format_iso8601] = ACTIONS(1357),
    [sym_auditlog_priv] = ACTIONS(1357),
    [sym_configuration_priv] = ACTIONS(1357),
    [sym_deviceconfig_priv] = ACTIONS(1357),
    [sym_ftp_priv] = ACTIONS(1357),
    [sym_http_priv] = ACTIONS(1357),
    [sym_networkconfig_priv] = ACTIONS(1357),
    [sym_programport_priv] = ACTIONS(1357),
    [sym_remoteui_priv] = ACTIONS(1357),
    [sym_securitycontrol_priv] = ACTIONS(1357),
    [sym_softwaremanagement_priv] = ACTIONS(1357),
    [sym_terminal_priv] = ACTIONS(1357),
    [sym_tpadmin_priv] = ACTIONS(1357),
    [sym_user1_priv] = ACTIONS(1357),
    [sym_user2_priv] = ACTIONS(1357),
    [sym_user3_priv] = ACTIONS(1357),
    [sym_user4_priv] = ACTIONS(1357),
    [sym_usermanagement_priv] = ACTIONS(1357),
    [sym_valid_account] = ACTIONS(1357),
    [sym_err_name_invalid] = ACTIONS(1357),
    [sym_err_password_invalid] = ACTIONS(1357),
    [sym_err_invalid_account] = ACTIONS(1357),
    [sym_err_no_authorization] = ACTIONS(1357),
    [sym_err_invalid_parameter] = ACTIONS(1357),
    [sym_err_user_locked_out] = ACTIONS(1357),
    [sym_err_user_account_expired] = ACTIONS(1357),
    [sym_err_general_error] = ACTIONS(1357),
    [sym_err_authentication_error] = ACTIONS(1357),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1357),
    [sym_tls_validate_certificate] = ACTIONS(1357),
    [sym_tdata] = ACTIONS(1357),
    [sym_tchannel] = ACTIONS(1357),
    [sym_tlevel] = ACTIONS(1357),
    [sym_tbutton] = ACTIONS(1357),
    [sym_ttimeline] = ACTIONS(1357),
    [sym_tcustom] = ACTIONS(1357),
    [sym_url_struct] = ACTIONS(1357),
    [sym_dns_struct] = ACTIONS(1357),
    [sym_ip_address_struct] = ACTIONS(1357),
    [sym_dev_info_struct] = ACTIONS(1357),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1357),
    [sym_data_feed] = ACTIONS(1357),
    [sym_data_field] = ACTIONS(1357),
    [sym_data_record] = ACTIONS(1357),
    [sym_wc_data_feed] = ACTIONS(1357),
    [sym_wc_data_field] = ACTIONS(1357),
    [sym_wc_data_record] = ACTIONS(1357),
    [sym_last_login_info] = ACTIONS(1357),
    [sym____reserved___] = ACTIONS(1357),
    [sym_date] = ACTIONS(1357),
    [sym_day] = ACTIONS(1357),
    [sym_time] = ACTIONS(1357),
    [sym_ldate] = ACTIONS(1357),
    [sym_push_device] = ACTIONS(1357),
    [sym_push_channel] = ACTIONS(1357),
    [sym_push_devchan] = ACTIONS(1357),
    [sym_release_device] = ACTIONS(1357),
    [sym_release_channel] = ACTIONS(1357),
    [sym_release_devchan] = ACTIONS(1357),
    [sym_master_slot] = ACTIONS(1357),
    [sym_get_pulse_time] = ACTIONS(1357),
    [sym_get_timer] = ACTIONS(1357),
    [sym_master_sn] = ACTIONS(1357),
    [sym_system_number] = ACTIONS(1357),
    [sym_button] = ACTIONS(1357),
    [sym_dv_channel] = ACTIONS(1357),
    [sym_data] = ACTIONS(1357),
    [sym_level] = ACTIONS(1357),
    [sym_timeline] = ACTIONS(1357),
    [sym_channel] = ACTIONS(1357),
    [sym_custom] = ACTIONS(1357),
    [sym_atoi] = ACTIONS(1357),
    [sym_atol] = ACTIONS(1357),
    [sym_atof] = ACTIONS(1357),
    [sym_itoa] = ACTIONS(1357),
    [sym_itohex] = ACTIONS(1357),
    [sym_hextoi] = ACTIONS(1357),
    [sym_ftoa] = ACTIONS(1357),
    [sym_find_string] = ACTIONS(1357),
    [sym_left_string] = ACTIONS(1357),
    [sym_right_string] = ACTIONS(1357),
    [sym_mid_string] = ACTIONS(1357),
    [sym_remove_string] = ACTIONS(1357),
    [sym_get_buffer_string] = ACTIONS(1357),
    [sym_get_buffer_char] = ACTIONS(1357),
    [sym_get_multi_buffer_string] = ACTIONS(1357),
    [sym_length_string] = ACTIONS(1357),
    [sym_max_length_string] = ACTIONS(1357),
    [sym_set_length_string] = ACTIONS(1357),
    [sym_compare_string] = ACTIONS(1357),
    [sym_upper_string] = ACTIONS(1357),
    [sym_lower_string] = ACTIONS(1357),
    [sym_get_last] = ACTIONS(1357),
    [sym_date_to_day] = ACTIONS(1357),
    [sym_date_to_month] = ACTIONS(1357),
    [sym_date_to_year] = ACTIONS(1357),
    [sym_time_to_hour] = ACTIONS(1357),
    [sym_time_to_minute] = ACTIONS(1357),
    [sym_time_to_second] = ACTIONS(1357),
    [sym_day_of_week] = ACTIONS(1357),
    [sym_set_pulse_time] = ACTIONS(1357),
    [sym_set_timer] = ACTIONS(1357),
    [sym_file_open] = ACTIONS(1357),
    [sym_file_close] = ACTIONS(1357),
    [sym_file_read] = ACTIONS(1357),
    [sym_file_read_line] = ACTIONS(1357),
    [sym_file_write] = ACTIONS(1357),
    [sym_file_write_line] = ACTIONS(1357),
    [sym_file_copy] = ACTIONS(1357),
    [sym_file_delete] = ACTIONS(1357),
    [sym_file_rename] = ACTIONS(1357),
    [sym_file_dir] = ACTIONS(1357),
    [sym_file_getdir] = ACTIONS(1357),
    [sym_file_setdir] = ACTIONS(1357),
    [sym_file_create_dir] = ACTIONS(1357),
    [sym_file_remove_dir] = ACTIONS(1357),
    [sym_file_seek] = ACTIONS(1357),
    [sym_length_array] = ACTIONS(1357),
    [sym_max_length_array] = ACTIONS(1357),
    [sym_set_length_array] = ACTIONS(1357),
    [sym_ip_client_open] = ACTIONS(1357),
    [sym_ip_client_close] = ACTIONS(1357),
    [sym_ip_server_open] = ACTIONS(1357),
    [sym_ip_server_close] = ACTIONS(1357),
    [sym_ip_bound_client_open] = ACTIONS(1357),
    [sym_ip_mc_server_open] = ACTIONS(1357),
    [sym_ip_set_option] = ACTIONS(1357),
    [sym_ssh_client_open] = ACTIONS(1357),
    [sym_ssh_client_close] = ACTIONS(1357),
    [sym_tls_client_open] = ACTIONS(1357),
    [sym_tls_client_close] = ACTIONS(1357),
    [sym_do_push] = ACTIONS(1357),
    [sym_do_push_timed] = ACTIONS(1357),
    [sym_do_release] = ACTIONS(1357),
    [sym_device_id] = ACTIONS(1357),
    [sym_device_info] = ACTIONS(1357),
    [sym_device_id_string] = ACTIONS(1357),
    [sym_device_standby] = ACTIONS(1357),
    [sym_device_wake] = ACTIONS(1357),
    [sym_timeline_create] = ACTIONS(1357),
    [sym_timeline_kill] = ACTIONS(1357),
    [sym_timeline_reload] = ACTIONS(1357),
    [sym_timeline_pause] = ACTIONS(1357),
    [sym_timeline_restart] = ACTIONS(1357),
    [sym_timeline_set] = ACTIONS(1357),
    [sym_timeline_get] = ACTIONS(1357),
    [sym_timeline_active] = ACTIONS(1357),
    [sym_timeline_dynamic_id] = ACTIONS(1357),
    [sym_variable_to_string] = ACTIONS(1357),
    [sym_string_to_variable] = ACTIONS(1357),
    [sym_length_variable_to_string] = ACTIONS(1357),
    [sym_type_cast] = ACTIONS(1357),
    [sym_variable_to_xml] = ACTIONS(1357),
    [sym_xml_to_variable] = ACTIONS(1357),
    [sym_length_variable_to_xml] = ACTIONS(1357),
    [sym_redirect_string] = ACTIONS(1357),
    [sym_random_number] = ACTIONS(1357),
    [sym_get_unique_id] = ACTIONS(1357),
    [sym_get_system_number] = ACTIONS(1357),
    [sym_set_system_number] = ACTIONS(1357),
    [sym_get_serial_number] = ACTIONS(1357),
    [sym_get_master_build] = ACTIONS(1357),
    [sym_set_validation_code] = ACTIONS(1357),
    [sym_reboot] = ACTIONS(1357),
    [sym_load_duet_module] = ACTIONS(1357),
    [sym_unload_duet_module] = ACTIONS(1357),
    [sym_duet_mem_size_set] = ACTIONS(1357),
    [sym_duet_mem_size_get] = ACTIONS(1357),
    [sym_dynamic_application_device] = ACTIONS(1357),
    [sym_dynamic_polled_port] = ACTIONS(1357),
    [sym_static_port_binding] = ACTIONS(1357),
    [sym_static_ip_binding] = ACTIONS(1357),
    [sym_get_dns_list] = ACTIONS(1357),
    [sym_set_dns_list] = ACTIONS(1357),
    [sym_get_ip_address] = ACTIONS(1357),
    [sym_set_ip_address] = ACTIONS(1357),
    [sym_get_url_list] = ACTIONS(1357),
    [sym_add_url_entry] = ACTIONS(1357),
    [sym_delete_url_entry] = ACTIONS(1357),
    [sym_abs_value] = ACTIONS(1357),
    [sym_max_value] = ACTIONS(1357),
    [sym_min_value] = ACTIONS(1357),
    [sym_exp_value] = ACTIONS(1357),
    [sym_log_value] = ACTIONS(1357),
    [sym_log10_value] = ACTIONS(1357),
    [sym_power_value] = ACTIONS(1357),
    [sym_sqrt_value] = ACTIONS(1357),
    [sym_raw_be] = ACTIONS(1357),
    [sym_raw_le] = ACTIONS(1357),
    [sym_internal_queue_size_set] = ACTIONS(1357),
    [sym_internal_queue_size_get] = ACTIONS(1357),
    [sym_internal_threshold_set] = ACTIONS(1357),
    [sym_internal_threshold_get] = ACTIONS(1357),
    [sym_rebuilt_event] = ACTIONS(1357),
    [sym_do_custom_event] = ACTIONS(1357),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1357),
    [sym_clkmgr_set_clk_source] = ACTIONS(1357),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1357),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1357),
    [sym_clkmgr_get_timezone] = ACTIONS(1357),
    [sym_clkmgr_set_timezone] = ACTIONS(1357),
    [sym_clkmgr_get_resync_period] = ACTIONS(1357),
    [sym_clkmgr_set_resync_period] = ACTIONS(1357),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1357),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1357),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1357),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1357),
    [sym_clkmgr_get_timeservers] = ACTIONS(1357),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1357),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1357),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1357),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1357),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1357),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1357),
    [sym_set_log_level] = ACTIONS(1357),
    [sym_get_log_level] = ACTIONS(1357),
    [sym_amx_log] = ACTIONS(1357),
    [sym_smtp_server_config_set] = ACTIONS(1357),
    [sym_smtp_server_config_get] = ACTIONS(1357),
    [sym_smtp_send] = ACTIONS(1357),
    [sym_validate_netlinx_account] = ACTIONS(1357),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1357),
    [sym_authenticate_certificate] = ACTIONS(1357),
    [sym_audit_netlinx_session_event] = ACTIONS(1357),
    [sym_audit_netlinx_generic_event] = ACTIONS(1357),
    [sym_get_available_flash_disk_space] = ACTIONS(1357),
    [sym_get_max_flash_disk_space] = ACTIONS(1357),
    [sym_data_create_feed] = ACTIONS(1357),
    [sym_data_delete_feed] = ACTIONS(1357),
    [sym_data_publish_feed] = ACTIONS(1357),
    [sym_data_get_published_feed] = ACTIONS(1357),
    [sym_data_add_record] = ACTIONS(1357),
    [sym_data_get_event_record] = ACTIONS(1357),
    [sym__wc_data_create_feed] = ACTIONS(1357),
    [sym__wc_data_add_record] = ACTIONS(1357),
    [sym__wc_data_get_event_record] = ACTIONS(1357),
    [sym_format] = ACTIONS(1357),
    [sym_astro_clock] = ACTIONS(1357),
    [sym_set_outdoor_temperature] = ACTIONS(1357),
    [sym_set_virtual_level_count] = ACTIONS(1357),
    [sym_set_virtual_channel_count] = ACTIONS(1357),
    [sym_set_virtual_port_count] = ACTIONS(1357),
    [sym__wc_encode_int] = ACTIONS(1357),
    [sym__wc_decode_int] = ACTIONS(1357),
    [sym__wc_int] = ACTIONS(1357),
    [sym__wc_to_ch_int] = ACTIONS(1357),
    [sym__ch_to_wc_int] = ACTIONS(1357),
    [sym__wc_find_string_int] = ACTIONS(1357),
    [sym__wc_left_string_int] = ACTIONS(1357),
    [sym__wc_lower_string_int] = ACTIONS(1357),
    [sym__wc_mid_string_int] = ACTIONS(1357),
    [sym__wc_remove_string_int] = ACTIONS(1357),
    [sym__wc_right_string_int] = ACTIONS(1357),
    [sym__wc_upper_string_int] = ACTIONS(1357),
    [sym__wc_compare_string_int] = ACTIONS(1357),
    [sym__wc_get_buffer_char_int] = ACTIONS(1357),
    [sym__wc_get_buffer_string_int] = ACTIONS(1357),
    [sym__wc_concat_string_int] = ACTIONS(1357),
    [sym__wc_file_open_int] = ACTIONS(1357),
    [sym__wc_file_close_int] = ACTIONS(1357),
    [sym__wc_file_read_int] = ACTIONS(1357),
    [sym__wc_file_read_line_int] = ACTIONS(1357),
    [sym__wc_file_write_int] = ACTIONS(1357),
    [sym__wc_file_write_line_int] = ACTIONS(1357),
    [aux_sym_preproc_include_token1] = ACTIONS(1357),
    [aux_sym_preproc_define_token1] = ACTIONS(1371),
    [aux_sym_preproc_warn_token1] = ACTIONS(1371),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1371),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1371),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1371),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1371),
    [anon_sym_COLON] = ACTIONS(1431),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1463),
    [anon_sym_BANG] = ACTIONS(1357),
    [anon_sym_TILDE] = ACTIONS(1371),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1443),
    [anon_sym_AMP_AMP] = ACTIONS(1445),
    [anon_sym_PIPE] = ACTIONS(1425),
    [anon_sym_CARET] = ACTIONS(1447),
    [anon_sym_AMP] = ACTIONS(1423),
    [anon_sym_EQ_EQ] = ACTIONS(1449),
    [anon_sym_BANG_EQ] = ACTIONS(1449),
    [anon_sym_LT_GT] = ACTIONS(1449),
    [anon_sym_GT] = ACTIONS(1451),
    [anon_sym_GT_EQ] = ACTIONS(1453),
    [anon_sym_LT_EQ] = ACTIONS(1453),
    [anon_sym_LT] = ACTIONS(1451),
    [anon_sym_LT_LT] = ACTIONS(1455),
    [anon_sym_GT_GT] = ACTIONS(1455),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1371),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(1371),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1463),
  },
  [STATE(62)] = {
    [sym_identifier] = ACTIONS(1465),
    [sym_program_name_keyword] = ACTIONS(1465),
    [sym_module_name_keyword] = ACTIONS(1465),
    [sym_define_device_keyword] = ACTIONS(1465),
    [sym_define_combine_keyword] = ACTIONS(1465),
    [sym_define_constant_keyword] = ACTIONS(1465),
    [sym_define_type_keyword] = ACTIONS(1465),
    [sym_define_variable_keyword] = ACTIONS(1465),
    [sym_define_system_variable_keyword] = ACTIONS(1465),
    [sym_define_connect_level_keyword] = ACTIONS(1465),
    [sym_define_latching_keyword] = ACTIONS(1465),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1465),
    [sym_define_toggling_keyword] = ACTIONS(1465),
    [sym_define_call_keyword] = ACTIONS(1465),
    [sym_define_function_keyword] = ACTIONS(1465),
    [sym_define_library_function_keyword] = ACTIONS(1465),
    [sym_define_start_keyword] = ACTIONS(1465),
    [sym_define_event_keyword] = ACTIONS(1465),
    [sym_define_program_keyword] = ACTIONS(1465),
    [sym_define_module_keyword] = ACTIONS(1465),
    [sym_char_keyword] = ACTIONS(1465),
    [sym_widechar_keyword] = ACTIONS(1465),
    [sym_integer_keyword] = ACTIONS(1465),
    [sym_sinteger_keyword] = ACTIONS(1465),
    [sym_long_keyword] = ACTIONS(1465),
    [sym_slong_keyword] = ACTIONS(1465),
    [sym_float_keyword] = ACTIONS(1465),
    [sym_double_keyword] = ACTIONS(1465),
    [sym_variant_keyword] = ACTIONS(1465),
    [sym_variantarray_keyword] = ACTIONS(1465),
    [sym_local_var_keyword] = ACTIONS(1465),
    [sym_stack_var_keyword] = ACTIONS(1465),
    [sym_constant_keyword] = ACTIONS(1465),
    [sym_volatile_keyword] = ACTIONS(1465),
    [sym_non_volatile_keyword] = ACTIONS(1465),
    [sym_persistent_keyword] = ACTIONS(1465),
    [sym_dev_keyword] = ACTIONS(1465),
    [sym_devlev_keyword] = ACTIONS(1465),
    [sym_devchan_keyword] = ACTIONS(1465),
    [sym_if_keyword] = ACTIONS(1465),
    [sym_else_keyword] = ACTIONS(1465),
    [sym_switch_keyword] = ACTIONS(1465),
    [sym_case_keyword] = ACTIONS(1465),
    [sym_default_keyword] = ACTIONS(1465),
    [sym_while_keyword] = ACTIONS(1465),
    [sym_for_keyword] = ACTIONS(1465),
    [sym_break_keyword] = ACTIONS(1465),
    [sym_continue_keyword] = ACTIONS(1465),
    [sym_return_keyword] = ACTIONS(1465),
    [sym_select_keyword] = ACTIONS(1465),
    [sym_active_keyword] = ACTIONS(1465),
    [sym_struct_keyword] = ACTIONS(1465),
    [sym_structure_keyword] = ACTIONS(1465),
    [sym_button_event_keyword] = ACTIONS(1465),
    [sym_channel_event_keyword] = ACTIONS(1465),
    [sym_level_event_keyword] = ACTIONS(1465),
    [sym_data_event_keyword] = ACTIONS(1465),
    [sym_timeline_event_keyword] = ACTIONS(1465),
    [sym_custom_event_keyword] = ACTIONS(1465),
    [sym_push_keyword] = ACTIONS(1465),
    [sym_release_keyword] = ACTIONS(1465),
    [aux_sym_on_keyword_token1] = ACTIONS(1465),
    [aux_sym_off_keyword_token1] = ACTIONS(1465),
    [sym_send_level_keyword] = ACTIONS(1465),
    [sym_send_string_keyword] = ACTIONS(1465),
    [sym_send_command_keyword] = ACTIONS(1465),
    [sym_clear_buffer_keyword] = ACTIONS(1465),
    [sym_create_buffer_keyword] = ACTIONS(1465),
    [sym_create_multi_buffer_keyword] = ACTIONS(1465),
    [sym_call_keyword] = ACTIONS(1465),
    [sym_system_call_keyword] = ACTIONS(1465),
    [sym_devchan_to_keyword] = ACTIONS(1465),
    [sym_devchan_min_to_keyword] = ACTIONS(1465),
    [sym_devchan_total_off_keyword] = ACTIONS(1465),
    [sym_devchan_pulse_keyword] = ACTIONS(1465),
    [sym_wait_keyword] = ACTIONS(1465),
    [sym_cancel_wait_keyword] = ACTIONS(1465),
    [sym_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym_tdata] = ACTIONS(1465),
    [sym_tchannel] = ACTIONS(1465),
    [sym_tlevel] = ACTIONS(1465),
    [sym_tbutton] = ACTIONS(1465),
    [sym_ttimeline] = ACTIONS(1465),
    [sym_tcustom] = ACTIONS(1465),
    [sym_url_struct] = ACTIONS(1465),
    [sym_dns_struct] = ACTIONS(1465),
    [sym_ip_address_struct] = ACTIONS(1465),
    [sym_dev_info_struct] = ACTIONS(1465),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1465),
    [sym_data_feed] = ACTIONS(1465),
    [sym_data_field] = ACTIONS(1465),
    [sym_data_record] = ACTIONS(1465),
    [sym_wc_data_feed] = ACTIONS(1465),
    [sym_wc_data_field] = ACTIONS(1465),
    [sym_wc_data_record] = ACTIONS(1465),
    [sym_last_login_info] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [anon_sym_EQ] = ACTIONS(1467),
    [aux_sym_preproc_include_token1] = ACTIONS(1465),
    [aux_sym_preproc_define_token1] = ACTIONS(1469),
    [aux_sym_preproc_warn_token1] = ACTIONS(1469),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_else_token1] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1469),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [sym_range_operator] = ACTIONS(1471),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1469),
  },
  [STATE(63)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1473),
    [sym_program_name_keyword] = ACTIONS(1473),
    [sym_module_name_keyword] = ACTIONS(1473),
    [sym_define_device_keyword] = ACTIONS(1473),
    [sym_define_combine_keyword] = ACTIONS(1473),
    [sym_define_constant_keyword] = ACTIONS(1473),
    [sym_define_type_keyword] = ACTIONS(1473),
    [sym_define_variable_keyword] = ACTIONS(1473),
    [sym_define_system_variable_keyword] = ACTIONS(1473),
    [sym_define_connect_level_keyword] = ACTIONS(1473),
    [sym_define_latching_keyword] = ACTIONS(1473),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1473),
    [sym_define_toggling_keyword] = ACTIONS(1473),
    [sym_define_call_keyword] = ACTIONS(1473),
    [sym_define_function_keyword] = ACTIONS(1473),
    [sym_define_library_function_keyword] = ACTIONS(1473),
    [sym_define_start_keyword] = ACTIONS(1473),
    [sym_define_event_keyword] = ACTIONS(1473),
    [sym_define_program_keyword] = ACTIONS(1473),
    [sym_define_module_keyword] = ACTIONS(1473),
    [sym_char_keyword] = ACTIONS(1473),
    [sym_widechar_keyword] = ACTIONS(1473),
    [sym_integer_keyword] = ACTIONS(1473),
    [sym_sinteger_keyword] = ACTIONS(1473),
    [sym_long_keyword] = ACTIONS(1473),
    [sym_slong_keyword] = ACTIONS(1473),
    [sym_float_keyword] = ACTIONS(1473),
    [sym_double_keyword] = ACTIONS(1473),
    [sym_variant_keyword] = ACTIONS(1473),
    [sym_variantarray_keyword] = ACTIONS(1473),
    [sym_local_var_keyword] = ACTIONS(1473),
    [sym_stack_var_keyword] = ACTIONS(1473),
    [sym_constant_keyword] = ACTIONS(1473),
    [sym_volatile_keyword] = ACTIONS(1473),
    [sym_non_volatile_keyword] = ACTIONS(1473),
    [sym_persistent_keyword] = ACTIONS(1473),
    [sym_dev_keyword] = ACTIONS(1473),
    [sym_devlev_keyword] = ACTIONS(1473),
    [sym_devchan_keyword] = ACTIONS(1473),
    [sym_if_keyword] = ACTIONS(1473),
    [sym_else_keyword] = ACTIONS(1473),
    [sym_switch_keyword] = ACTIONS(1473),
    [sym_case_keyword] = ACTIONS(1473),
    [sym_default_keyword] = ACTIONS(1473),
    [sym_while_keyword] = ACTIONS(1473),
    [sym_for_keyword] = ACTIONS(1473),
    [sym_break_keyword] = ACTIONS(1473),
    [sym_continue_keyword] = ACTIONS(1473),
    [sym_return_keyword] = ACTIONS(1473),
    [sym_select_keyword] = ACTIONS(1473),
    [sym_active_keyword] = ACTIONS(1473),
    [sym_struct_keyword] = ACTIONS(1473),
    [sym_structure_keyword] = ACTIONS(1473),
    [sym_button_event_keyword] = ACTIONS(1473),
    [sym_channel_event_keyword] = ACTIONS(1473),
    [sym_level_event_keyword] = ACTIONS(1473),
    [sym_data_event_keyword] = ACTIONS(1473),
    [sym_timeline_event_keyword] = ACTIONS(1473),
    [sym_custom_event_keyword] = ACTIONS(1473),
    [sym_push_keyword] = ACTIONS(1473),
    [sym_release_keyword] = ACTIONS(1473),
    [aux_sym_on_keyword_token1] = ACTIONS(1473),
    [aux_sym_off_keyword_token1] = ACTIONS(1473),
    [sym_send_level_keyword] = ACTIONS(1473),
    [sym_send_string_keyword] = ACTIONS(1473),
    [sym_send_command_keyword] = ACTIONS(1473),
    [sym_clear_buffer_keyword] = ACTIONS(1473),
    [sym_create_buffer_keyword] = ACTIONS(1473),
    [sym_create_multi_buffer_keyword] = ACTIONS(1473),
    [sym_call_keyword] = ACTIONS(1473),
    [sym_system_call_keyword] = ACTIONS(1473),
    [sym_devchan_to_keyword] = ACTIONS(1473),
    [sym_devchan_min_to_keyword] = ACTIONS(1473),
    [sym_devchan_total_off_keyword] = ACTIONS(1473),
    [sym_devchan_pulse_keyword] = ACTIONS(1473),
    [sym_wait_keyword] = ACTIONS(1473),
    [sym_cancel_wait_keyword] = ACTIONS(1473),
    [sym_wait_until_keyword] = ACTIONS(1473),
    [sym_cancel_wait_until_keyword] = ACTIONS(1473),
    [sym_cancel_all_wait_keyword] = ACTIONS(1473),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1473),
    [sym_and] = ACTIONS(1473),
    [sym_or] = ACTIONS(1473),
    [sym_not] = ACTIONS(1473),
    [sym_xor] = ACTIONS(1473),
    [sym_band] = ACTIONS(1473),
    [sym_bor] = ACTIONS(1473),
    [sym_bxor] = ACTIONS(1473),
    [sym_bnot] = ACTIONS(1473),
    [sym_lshift] = ACTIONS(1473),
    [sym_rshift] = ACTIONS(1473),
    [sym___file__] = ACTIONS(1473),
    [sym___date__] = ACTIONS(1473),
    [sym___ldate__] = ACTIONS(1473),
    [sym___time__] = ACTIONS(1473),
    [sym___line__] = ACTIONS(1473),
    [sym___name__] = ACTIONS(1473),
    [sym___netlinx__] = ACTIONS(1473),
    [sym_netlinx_axi_version] = ACTIONS(1473),
    [sym_url_flg_tcp] = ACTIONS(1473),
    [sym_url_flg_accinfopresent] = ACTIONS(1473),
    [sym_url_flg_temp] = ACTIONS(1473),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1473),
    [sym_url_flg_stat_mask] = ACTIONS(1473),
    [sym_url_flg_stat_lookup] = ACTIONS(1473),
    [sym_url_flg_stat_connecting] = ACTIONS(1473),
    [sym_url_flg_stat_waiting] = ACTIONS(1473),
    [sym_url_flg_stat_connected] = ACTIONS(1473),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1473),
    [sym_first_virtual_device] = ACTIONS(1473),
    [sym_first_local_port] = ACTIONS(1473),
    [sym_dynamic_virtual_device] = ACTIONS(1473),
    [sym_timeline_once] = ACTIONS(1473),
    [sym_timeline_repeat] = ACTIONS(1473),
    [sym_timeline_absolute] = ACTIONS(1473),
    [sym_timeline_relative] = ACTIONS(1473),
    [sym_do_push_timed_infinite] = ACTIONS(1473),
    [sym_source_type_no_address] = ACTIONS(1473),
    [sym_source_type_neuron_id] = ACTIONS(1473),
    [sym_source_type_ip_address] = ACTIONS(1473),
    [sym_source_type_axlink] = ACTIONS(1473),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1473),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1473),
    [sym_source_type_ip_socket_address] = ACTIONS(1473),
    [sym_source_type_rs232] = ACTIONS(1473),
    [sym_source_type_internal] = ACTIONS(1473),
    [sym_source_type_mac_address] = ACTIONS(1473),
    [sym_source_type_ipv4_port] = ACTIONS(1473),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1473),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1473),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1473),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1473),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1473),
    [sym_true] = ACTIONS(1473),
    [sym_false] = ACTIONS(1473),
    [sym_file_read_only] = ACTIONS(1473),
    [sym_file_rw_new] = ACTIONS(1473),
    [sym_file_rw_append] = ACTIONS(1473),
    [sym_ip_tcp] = ACTIONS(1473),
    [sym_ip_udp] = ACTIONS(1473),
    [sym_ip_udp_2way] = ACTIONS(1473),
    [sym_xml_encode_types] = ACTIONS(1473),
    [sym_xml_encode_char_as_list] = ACTIONS(1473),
    [sym_xml_encode_le] = ACTIONS(1473),
    [sym_xml_decode_types] = ACTIONS(1473),
    [sym_xml_decode_no_preserve] = ACTIONS(1473),
    [sym_duet_dev_type_display_device] = ACTIONS(1473),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1473),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1473),
    [sym_duet_dev_type_io_device] = ACTIONS(1473),
    [sym_duet_dev_type_relay_device] = ACTIONS(1473),
    [sym_duet_dev_type_ups] = ACTIONS(1473),
    [sym_duet_dev_type_amplifier] = ACTIONS(1473),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1473),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1473),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1473),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1473),
    [sym_duet_dev_type_camera] = ACTIONS(1473),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1473),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1473),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1473),
    [sym_duet_dev_type_dss] = ACTIONS(1473),
    [sym_duet_dev_type_dvr] = ACTIONS(1473),
    [sym_duet_dev_type_disc_device] = ACTIONS(1473),
    [sym_duet_dev_type_document_camera] = ACTIONS(1473),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1473),
    [sym_duet_dev_type_hvac] = ACTIONS(1473),
    [sym_duet_dev_type_keypad] = ACTIONS(1473),
    [sym_duet_dev_type_light] = ACTIONS(1473),
    [sym_duet_dev_type_monitor] = ACTIONS(1473),
    [sym_duet_dev_type_motor] = ACTIONS(1473),
    [sym_duet_dev_type_multi_window] = ACTIONS(1473),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1473),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1473),
    [sym_duet_dev_type_receiver] = ACTIONS(1473),
    [sym_duet_dev_type_security_system] = ACTIONS(1473),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1473),
    [sym_duet_dev_type_settop_box] = ACTIONS(1473),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1473),
    [sym_duet_dev_type_switcher] = ACTIONS(1473),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1473),
    [sym_duet_dev_type_tv] = ACTIONS(1473),
    [sym_duet_dev_type_utility] = ACTIONS(1473),
    [sym_duet_dev_type_vcr] = ACTIONS(1473),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1473),
    [sym_duet_dev_type_video_processor] = ACTIONS(1473),
    [sym_duet_dev_type_video_projector] = ACTIONS(1473),
    [sym_duet_dev_type_video_wall] = ACTIONS(1473),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1473),
    [sym_duet_dev_type_weather] = ACTIONS(1473),
    [sym_duet_dev_not_polled] = ACTIONS(1473),
    [sym_duet_dev_polled] = ACTIONS(1473),
    [sym_clkmgr_mode_network] = ACTIONS(1473),
    [sym_clkmgr_mode_standalone] = ACTIONS(1473),
    [sym_normal_standby] = ACTIONS(1473),
    [sym_normal_wake] = ACTIONS(1473),
    [sym_ip_multicast_ttl_option] = ACTIONS(1473),
    [sym_ip_ttl_subnet] = ACTIONS(1473),
    [sym_ip_ttl_site] = ACTIONS(1473),
    [sym_ip_ttl_region] = ACTIONS(1473),
    [sym_ip_ttl_continent] = ACTIONS(1473),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1473),
    [sym_ip_nodelay_on] = ACTIONS(1473),
    [sym_ip_nodelay_off] = ACTIONS(1473),
    [sym_amx_error] = ACTIONS(1473),
    [sym_amx_warning] = ACTIONS(1473),
    [sym_amx_info] = ACTIONS(1473),
    [sym_amx_debug] = ACTIONS(1473),
    [sym_netlinx_login_success] = ACTIONS(1473),
    [sym_netlinx_login_fail] = ACTIONS(1473),
    [sym_netlinx_logout] = ACTIONS(1473),
    [sym_null_str] = ACTIONS(1473),
    [sym_smtp_address] = ACTIONS(1473),
    [sym_smtp_port_number] = ACTIONS(1473),
    [sym_smtp_username] = ACTIONS(1473),
    [sym_smtp_password] = ACTIONS(1473),
    [sym_smtp_require_tls] = ACTIONS(1473),
    [sym_smtp_from] = ACTIONS(1473),
    [sym_smtp_tls_true] = ACTIONS(1473),
    [sym_smtp_tls_false] = ACTIONS(1473),
    [sym_listview_on_row_select_event] = ACTIONS(1473),
    [sym_data_structure_unknown] = ACTIONS(1473),
    [sym_data_structure_datafeed] = ACTIONS(1473),
    [sym_data_structure_datarecordset] = ACTIONS(1473),
    [sym_data_structure_datarecord] = ACTIONS(1473),
    [sym_data_structure_datafield] = ACTIONS(1473),
    [sym_data_max_content_field_count] = ACTIONS(1473),
    [sym_data_max_metadata_field_count] = ACTIONS(1473),
    [sym_data_max_id_length] = ACTIONS(1473),
    [sym_data_max_name_length] = ACTIONS(1473),
    [sym_data_max_description_length] = ACTIONS(1473),
    [sym_data_max_type_length] = ACTIONS(1473),
    [sym_data_max_format_length] = ACTIONS(1473),
    [sym_data_max_source_length] = ACTIONS(1473),
    [sym_data_max_value_length] = ACTIONS(1473),
    [sym_data_max_label_length] = ACTIONS(1473),
    [sym_data_max_url_length] = ACTIONS(1473),
    [sym_max_last_login_info_length] = ACTIONS(1473),
    [sym_data_type_unknown] = ACTIONS(1473),
    [sym_data_type_string] = ACTIONS(1473),
    [sym_data_type_datetime] = ACTIONS(1473),
    [sym_data_type_date] = ACTIONS(1473),
    [sym_data_type_time] = ACTIONS(1473),
    [sym_data_type_image] = ACTIONS(1473),
    [sym_data_format_url] = ACTIONS(1473),
    [sym_data_format_phone] = ACTIONS(1473),
    [sym_data_format_email] = ACTIONS(1473),
    [sym_data_format_iso8601] = ACTIONS(1473),
    [sym_auditlog_priv] = ACTIONS(1473),
    [sym_configuration_priv] = ACTIONS(1473),
    [sym_deviceconfig_priv] = ACTIONS(1473),
    [sym_ftp_priv] = ACTIONS(1473),
    [sym_http_priv] = ACTIONS(1473),
    [sym_networkconfig_priv] = ACTIONS(1473),
    [sym_programport_priv] = ACTIONS(1473),
    [sym_remoteui_priv] = ACTIONS(1473),
    [sym_securitycontrol_priv] = ACTIONS(1473),
    [sym_softwaremanagement_priv] = ACTIONS(1473),
    [sym_terminal_priv] = ACTIONS(1473),
    [sym_tpadmin_priv] = ACTIONS(1473),
    [sym_user1_priv] = ACTIONS(1473),
    [sym_user2_priv] = ACTIONS(1473),
    [sym_user3_priv] = ACTIONS(1473),
    [sym_user4_priv] = ACTIONS(1473),
    [sym_usermanagement_priv] = ACTIONS(1473),
    [sym_valid_account] = ACTIONS(1473),
    [sym_err_name_invalid] = ACTIONS(1473),
    [sym_err_password_invalid] = ACTIONS(1473),
    [sym_err_invalid_account] = ACTIONS(1473),
    [sym_err_no_authorization] = ACTIONS(1473),
    [sym_err_invalid_parameter] = ACTIONS(1473),
    [sym_err_user_locked_out] = ACTIONS(1473),
    [sym_err_user_account_expired] = ACTIONS(1473),
    [sym_err_general_error] = ACTIONS(1473),
    [sym_err_authentication_error] = ACTIONS(1473),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1473),
    [sym_tls_validate_certificate] = ACTIONS(1473),
    [sym_tdata] = ACTIONS(1473),
    [sym_tchannel] = ACTIONS(1473),
    [sym_tlevel] = ACTIONS(1473),
    [sym_tbutton] = ACTIONS(1473),
    [sym_ttimeline] = ACTIONS(1473),
    [sym_tcustom] = ACTIONS(1473),
    [sym_url_struct] = ACTIONS(1473),
    [sym_dns_struct] = ACTIONS(1473),
    [sym_ip_address_struct] = ACTIONS(1473),
    [sym_dev_info_struct] = ACTIONS(1473),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1473),
    [sym_data_feed] = ACTIONS(1473),
    [sym_data_field] = ACTIONS(1473),
    [sym_data_record] = ACTIONS(1473),
    [sym_wc_data_feed] = ACTIONS(1473),
    [sym_wc_data_field] = ACTIONS(1473),
    [sym_wc_data_record] = ACTIONS(1473),
    [sym_last_login_info] = ACTIONS(1473),
    [sym____reserved___] = ACTIONS(1473),
    [sym_date] = ACTIONS(1473),
    [sym_day] = ACTIONS(1473),
    [sym_time] = ACTIONS(1473),
    [sym_ldate] = ACTIONS(1473),
    [sym_push_device] = ACTIONS(1473),
    [sym_push_channel] = ACTIONS(1473),
    [sym_push_devchan] = ACTIONS(1473),
    [sym_release_device] = ACTIONS(1473),
    [sym_release_channel] = ACTIONS(1473),
    [sym_release_devchan] = ACTIONS(1473),
    [sym_master_slot] = ACTIONS(1473),
    [sym_get_pulse_time] = ACTIONS(1473),
    [sym_get_timer] = ACTIONS(1473),
    [sym_master_sn] = ACTIONS(1473),
    [sym_system_number] = ACTIONS(1473),
    [sym_button] = ACTIONS(1473),
    [sym_dv_channel] = ACTIONS(1473),
    [sym_data] = ACTIONS(1473),
    [sym_level] = ACTIONS(1473),
    [sym_timeline] = ACTIONS(1473),
    [sym_channel] = ACTIONS(1473),
    [sym_custom] = ACTIONS(1473),
    [sym_atoi] = ACTIONS(1473),
    [sym_atol] = ACTIONS(1473),
    [sym_atof] = ACTIONS(1473),
    [sym_itoa] = ACTIONS(1473),
    [sym_itohex] = ACTIONS(1473),
    [sym_hextoi] = ACTIONS(1473),
    [sym_ftoa] = ACTIONS(1473),
    [sym_find_string] = ACTIONS(1473),
    [sym_left_string] = ACTIONS(1473),
    [sym_right_string] = ACTIONS(1473),
    [sym_mid_string] = ACTIONS(1473),
    [sym_remove_string] = ACTIONS(1473),
    [sym_get_buffer_string] = ACTIONS(1473),
    [sym_get_buffer_char] = ACTIONS(1473),
    [sym_get_multi_buffer_string] = ACTIONS(1473),
    [sym_length_string] = ACTIONS(1473),
    [sym_max_length_string] = ACTIONS(1473),
    [sym_set_length_string] = ACTIONS(1473),
    [sym_compare_string] = ACTIONS(1473),
    [sym_upper_string] = ACTIONS(1473),
    [sym_lower_string] = ACTIONS(1473),
    [sym_get_last] = ACTIONS(1473),
    [sym_date_to_day] = ACTIONS(1473),
    [sym_date_to_month] = ACTIONS(1473),
    [sym_date_to_year] = ACTIONS(1473),
    [sym_time_to_hour] = ACTIONS(1473),
    [sym_time_to_minute] = ACTIONS(1473),
    [sym_time_to_second] = ACTIONS(1473),
    [sym_day_of_week] = ACTIONS(1473),
    [sym_set_pulse_time] = ACTIONS(1473),
    [sym_set_timer] = ACTIONS(1473),
    [sym_file_open] = ACTIONS(1473),
    [sym_file_close] = ACTIONS(1473),
    [sym_file_read] = ACTIONS(1473),
    [sym_file_read_line] = ACTIONS(1473),
    [sym_file_write] = ACTIONS(1473),
    [sym_file_write_line] = ACTIONS(1473),
    [sym_file_copy] = ACTIONS(1473),
    [sym_file_delete] = ACTIONS(1473),
    [sym_file_rename] = ACTIONS(1473),
    [sym_file_dir] = ACTIONS(1473),
    [sym_file_getdir] = ACTIONS(1473),
    [sym_file_setdir] = ACTIONS(1473),
    [sym_file_create_dir] = ACTIONS(1473),
    [sym_file_remove_dir] = ACTIONS(1473),
    [sym_file_seek] = ACTIONS(1473),
    [sym_length_array] = ACTIONS(1473),
    [sym_max_length_array] = ACTIONS(1473),
    [sym_set_length_array] = ACTIONS(1473),
    [sym_ip_client_open] = ACTIONS(1473),
    [sym_ip_client_close] = ACTIONS(1473),
    [sym_ip_server_open] = ACTIONS(1473),
    [sym_ip_server_close] = ACTIONS(1473),
    [sym_ip_bound_client_open] = ACTIONS(1473),
    [sym_ip_mc_server_open] = ACTIONS(1473),
    [sym_ip_set_option] = ACTIONS(1473),
    [sym_ssh_client_open] = ACTIONS(1473),
    [sym_ssh_client_close] = ACTIONS(1473),
    [sym_tls_client_open] = ACTIONS(1473),
    [sym_tls_client_close] = ACTIONS(1473),
    [sym_do_push] = ACTIONS(1473),
    [sym_do_push_timed] = ACTIONS(1473),
    [sym_do_release] = ACTIONS(1473),
    [sym_device_id] = ACTIONS(1473),
    [sym_device_info] = ACTIONS(1473),
    [sym_device_id_string] = ACTIONS(1473),
    [sym_device_standby] = ACTIONS(1473),
    [sym_device_wake] = ACTIONS(1473),
    [sym_timeline_create] = ACTIONS(1473),
    [sym_timeline_kill] = ACTIONS(1473),
    [sym_timeline_reload] = ACTIONS(1473),
    [sym_timeline_pause] = ACTIONS(1473),
    [sym_timeline_restart] = ACTIONS(1473),
    [sym_timeline_set] = ACTIONS(1473),
    [sym_timeline_get] = ACTIONS(1473),
    [sym_timeline_active] = ACTIONS(1473),
    [sym_timeline_dynamic_id] = ACTIONS(1473),
    [sym_variable_to_string] = ACTIONS(1473),
    [sym_string_to_variable] = ACTIONS(1473),
    [sym_length_variable_to_string] = ACTIONS(1473),
    [sym_type_cast] = ACTIONS(1473),
    [sym_variable_to_xml] = ACTIONS(1473),
    [sym_xml_to_variable] = ACTIONS(1473),
    [sym_length_variable_to_xml] = ACTIONS(1473),
    [sym_redirect_string] = ACTIONS(1473),
    [sym_random_number] = ACTIONS(1473),
    [sym_get_unique_id] = ACTIONS(1473),
    [sym_get_system_number] = ACTIONS(1473),
    [sym_set_system_number] = ACTIONS(1473),
    [sym_get_serial_number] = ACTIONS(1473),
    [sym_get_master_build] = ACTIONS(1473),
    [sym_set_validation_code] = ACTIONS(1473),
    [sym_reboot] = ACTIONS(1473),
    [sym_load_duet_module] = ACTIONS(1473),
    [sym_unload_duet_module] = ACTIONS(1473),
    [sym_duet_mem_size_set] = ACTIONS(1473),
    [sym_duet_mem_size_get] = ACTIONS(1473),
    [sym_dynamic_application_device] = ACTIONS(1473),
    [sym_dynamic_polled_port] = ACTIONS(1473),
    [sym_static_port_binding] = ACTIONS(1473),
    [sym_static_ip_binding] = ACTIONS(1473),
    [sym_get_dns_list] = ACTIONS(1473),
    [sym_set_dns_list] = ACTIONS(1473),
    [sym_get_ip_address] = ACTIONS(1473),
    [sym_set_ip_address] = ACTIONS(1473),
    [sym_get_url_list] = ACTIONS(1473),
    [sym_add_url_entry] = ACTIONS(1473),
    [sym_delete_url_entry] = ACTIONS(1473),
    [sym_abs_value] = ACTIONS(1473),
    [sym_max_value] = ACTIONS(1473),
    [sym_min_value] = ACTIONS(1473),
    [sym_exp_value] = ACTIONS(1473),
    [sym_log_value] = ACTIONS(1473),
    [sym_log10_value] = ACTIONS(1473),
    [sym_power_value] = ACTIONS(1473),
    [sym_sqrt_value] = ACTIONS(1473),
    [sym_raw_be] = ACTIONS(1473),
    [sym_raw_le] = ACTIONS(1473),
    [sym_internal_queue_size_set] = ACTIONS(1473),
    [sym_internal_queue_size_get] = ACTIONS(1473),
    [sym_internal_threshold_set] = ACTIONS(1473),
    [sym_internal_threshold_get] = ACTIONS(1473),
    [sym_rebuilt_event] = ACTIONS(1473),
    [sym_do_custom_event] = ACTIONS(1473),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1473),
    [sym_clkmgr_set_clk_source] = ACTIONS(1473),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1473),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1473),
    [sym_clkmgr_get_timezone] = ACTIONS(1473),
    [sym_clkmgr_set_timezone] = ACTIONS(1473),
    [sym_clkmgr_get_resync_period] = ACTIONS(1473),
    [sym_clkmgr_set_resync_period] = ACTIONS(1473),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1473),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1473),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1473),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1473),
    [sym_clkmgr_get_timeservers] = ACTIONS(1473),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1473),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1473),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1473),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1473),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1473),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1473),
    [sym_set_log_level] = ACTIONS(1473),
    [sym_get_log_level] = ACTIONS(1473),
    [sym_amx_log] = ACTIONS(1473),
    [sym_smtp_server_config_set] = ACTIONS(1473),
    [sym_smtp_server_config_get] = ACTIONS(1473),
    [sym_smtp_send] = ACTIONS(1473),
    [sym_validate_netlinx_account] = ACTIONS(1473),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1473),
    [sym_authenticate_certificate] = ACTIONS(1473),
    [sym_audit_netlinx_session_event] = ACTIONS(1473),
    [sym_audit_netlinx_generic_event] = ACTIONS(1473),
    [sym_get_available_flash_disk_space] = ACTIONS(1473),
    [sym_get_max_flash_disk_space] = ACTIONS(1473),
    [sym_data_create_feed] = ACTIONS(1473),
    [sym_data_delete_feed] = ACTIONS(1473),
    [sym_data_publish_feed] = ACTIONS(1473),
    [sym_data_get_published_feed] = ACTIONS(1473),
    [sym_data_add_record] = ACTIONS(1473),
    [sym_data_get_event_record] = ACTIONS(1473),
    [sym__wc_data_create_feed] = ACTIONS(1473),
    [sym__wc_data_add_record] = ACTIONS(1473),
    [sym__wc_data_get_event_record] = ACTIONS(1473),
    [sym_format] = ACTIONS(1473),
    [sym_astro_clock] = ACTIONS(1473),
    [sym_set_outdoor_temperature] = ACTIONS(1473),
    [sym_set_virtual_level_count] = ACTIONS(1473),
    [sym_set_virtual_channel_count] = ACTIONS(1473),
    [sym_set_virtual_port_count] = ACTIONS(1473),
    [sym__wc_encode_int] = ACTIONS(1473),
    [sym__wc_decode_int] = ACTIONS(1473),
    [sym__wc_int] = ACTIONS(1473),
    [sym__wc_to_ch_int] = ACTIONS(1473),
    [sym__ch_to_wc_int] = ACTIONS(1473),
    [sym__wc_find_string_int] = ACTIONS(1473),
    [sym__wc_left_string_int] = ACTIONS(1473),
    [sym__wc_lower_string_int] = ACTIONS(1473),
    [sym__wc_mid_string_int] = ACTIONS(1473),
    [sym__wc_remove_string_int] = ACTIONS(1473),
    [sym__wc_right_string_int] = ACTIONS(1473),
    [sym__wc_upper_string_int] = ACTIONS(1473),
    [sym__wc_compare_string_int] = ACTIONS(1473),
    [sym__wc_get_buffer_char_int] = ACTIONS(1473),
    [sym__wc_get_buffer_string_int] = ACTIONS(1473),
    [sym__wc_concat_string_int] = ACTIONS(1473),
    [sym__wc_file_open_int] = ACTIONS(1473),
    [sym__wc_file_close_int] = ACTIONS(1473),
    [sym__wc_file_read_int] = ACTIONS(1473),
    [sym__wc_file_read_line_int] = ACTIONS(1473),
    [sym__wc_file_write_int] = ACTIONS(1473),
    [sym__wc_file_write_line_int] = ACTIONS(1473),
    [aux_sym_preproc_include_token1] = ACTIONS(1473),
    [aux_sym_preproc_define_token1] = ACTIONS(1475),
    [aux_sym_preproc_warn_token1] = ACTIONS(1475),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1475),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1475),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1475),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1475),
    [aux_sym_preproc_else_token1] = ACTIONS(1475),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1475),
    [anon_sym_COLON] = ACTIONS(1475),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1475),
    [anon_sym_BANG] = ACTIONS(1473),
    [anon_sym_TILDE] = ACTIONS(1475),
    [anon_sym_DASH] = ACTIONS(1473),
    [anon_sym_PLUS] = ACTIONS(1473),
    [anon_sym_STAR] = ACTIONS(1475),
    [anon_sym_SLASH] = ACTIONS(1473),
    [anon_sym_PERCENT] = ACTIONS(1475),
    [anon_sym_PIPE_PIPE] = ACTIONS(1475),
    [anon_sym_AMP_AMP] = ACTIONS(1475),
    [anon_sym_PIPE] = ACTIONS(1473),
    [anon_sym_CARET] = ACTIONS(1475),
    [anon_sym_AMP] = ACTIONS(1473),
    [anon_sym_EQ_EQ] = ACTIONS(1475),
    [anon_sym_BANG_EQ] = ACTIONS(1475),
    [anon_sym_LT_GT] = ACTIONS(1475),
    [anon_sym_GT] = ACTIONS(1473),
    [anon_sym_GT_EQ] = ACTIONS(1475),
    [anon_sym_LT_EQ] = ACTIONS(1475),
    [anon_sym_LT] = ACTIONS(1473),
    [anon_sym_LT_LT] = ACTIONS(1475),
    [anon_sym_GT_GT] = ACTIONS(1475),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1475),
    [anon_sym_SQUOTE] = ACTIONS(1475),
    [sym_number_literal] = ACTIONS(1475),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1475),
  },
  [STATE(64)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1477),
    [sym_program_name_keyword] = ACTIONS(1477),
    [sym_module_name_keyword] = ACTIONS(1477),
    [sym_define_device_keyword] = ACTIONS(1477),
    [sym_define_combine_keyword] = ACTIONS(1477),
    [sym_define_constant_keyword] = ACTIONS(1477),
    [sym_define_type_keyword] = ACTIONS(1477),
    [sym_define_variable_keyword] = ACTIONS(1477),
    [sym_define_system_variable_keyword] = ACTIONS(1477),
    [sym_define_connect_level_keyword] = ACTIONS(1477),
    [sym_define_latching_keyword] = ACTIONS(1477),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1477),
    [sym_define_toggling_keyword] = ACTIONS(1477),
    [sym_define_call_keyword] = ACTIONS(1477),
    [sym_define_function_keyword] = ACTIONS(1477),
    [sym_define_library_function_keyword] = ACTIONS(1477),
    [sym_define_start_keyword] = ACTIONS(1477),
    [sym_define_event_keyword] = ACTIONS(1477),
    [sym_define_program_keyword] = ACTIONS(1477),
    [sym_define_module_keyword] = ACTIONS(1477),
    [sym_char_keyword] = ACTIONS(1477),
    [sym_widechar_keyword] = ACTIONS(1477),
    [sym_integer_keyword] = ACTIONS(1477),
    [sym_sinteger_keyword] = ACTIONS(1477),
    [sym_long_keyword] = ACTIONS(1477),
    [sym_slong_keyword] = ACTIONS(1477),
    [sym_float_keyword] = ACTIONS(1477),
    [sym_double_keyword] = ACTIONS(1477),
    [sym_variant_keyword] = ACTIONS(1477),
    [sym_variantarray_keyword] = ACTIONS(1477),
    [sym_local_var_keyword] = ACTIONS(1477),
    [sym_stack_var_keyword] = ACTIONS(1477),
    [sym_constant_keyword] = ACTIONS(1477),
    [sym_volatile_keyword] = ACTIONS(1477),
    [sym_non_volatile_keyword] = ACTIONS(1477),
    [sym_persistent_keyword] = ACTIONS(1477),
    [sym_dev_keyword] = ACTIONS(1477),
    [sym_devlev_keyword] = ACTIONS(1477),
    [sym_devchan_keyword] = ACTIONS(1477),
    [sym_if_keyword] = ACTIONS(1477),
    [sym_else_keyword] = ACTIONS(1477),
    [sym_switch_keyword] = ACTIONS(1477),
    [sym_case_keyword] = ACTIONS(1477),
    [sym_default_keyword] = ACTIONS(1477),
    [sym_while_keyword] = ACTIONS(1477),
    [sym_for_keyword] = ACTIONS(1477),
    [sym_break_keyword] = ACTIONS(1477),
    [sym_continue_keyword] = ACTIONS(1477),
    [sym_return_keyword] = ACTIONS(1477),
    [sym_select_keyword] = ACTIONS(1477),
    [sym_active_keyword] = ACTIONS(1477),
    [sym_struct_keyword] = ACTIONS(1477),
    [sym_structure_keyword] = ACTIONS(1477),
    [sym_button_event_keyword] = ACTIONS(1477),
    [sym_channel_event_keyword] = ACTIONS(1477),
    [sym_level_event_keyword] = ACTIONS(1477),
    [sym_data_event_keyword] = ACTIONS(1477),
    [sym_timeline_event_keyword] = ACTIONS(1477),
    [sym_custom_event_keyword] = ACTIONS(1477),
    [sym_push_keyword] = ACTIONS(1477),
    [sym_release_keyword] = ACTIONS(1477),
    [aux_sym_on_keyword_token1] = ACTIONS(1477),
    [aux_sym_off_keyword_token1] = ACTIONS(1477),
    [sym_send_level_keyword] = ACTIONS(1477),
    [sym_send_string_keyword] = ACTIONS(1477),
    [sym_send_command_keyword] = ACTIONS(1477),
    [sym_clear_buffer_keyword] = ACTIONS(1477),
    [sym_create_buffer_keyword] = ACTIONS(1477),
    [sym_create_multi_buffer_keyword] = ACTIONS(1477),
    [sym_call_keyword] = ACTIONS(1477),
    [sym_system_call_keyword] = ACTIONS(1477),
    [sym_devchan_to_keyword] = ACTIONS(1477),
    [sym_devchan_min_to_keyword] = ACTIONS(1477),
    [sym_devchan_total_off_keyword] = ACTIONS(1477),
    [sym_devchan_pulse_keyword] = ACTIONS(1477),
    [sym_wait_keyword] = ACTIONS(1477),
    [sym_cancel_wait_keyword] = ACTIONS(1477),
    [sym_wait_until_keyword] = ACTIONS(1477),
    [sym_cancel_wait_until_keyword] = ACTIONS(1477),
    [sym_cancel_all_wait_keyword] = ACTIONS(1477),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1477),
    [sym_and] = ACTIONS(1477),
    [sym_or] = ACTIONS(1477),
    [sym_not] = ACTIONS(1477),
    [sym_xor] = ACTIONS(1477),
    [sym_band] = ACTIONS(1477),
    [sym_bor] = ACTIONS(1477),
    [sym_bxor] = ACTIONS(1477),
    [sym_bnot] = ACTIONS(1477),
    [sym_lshift] = ACTIONS(1477),
    [sym_rshift] = ACTIONS(1477),
    [sym___file__] = ACTIONS(1477),
    [sym___date__] = ACTIONS(1477),
    [sym___ldate__] = ACTIONS(1477),
    [sym___time__] = ACTIONS(1477),
    [sym___line__] = ACTIONS(1477),
    [sym___name__] = ACTIONS(1477),
    [sym___netlinx__] = ACTIONS(1477),
    [sym_netlinx_axi_version] = ACTIONS(1477),
    [sym_url_flg_tcp] = ACTIONS(1477),
    [sym_url_flg_accinfopresent] = ACTIONS(1477),
    [sym_url_flg_temp] = ACTIONS(1477),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1477),
    [sym_url_flg_stat_mask] = ACTIONS(1477),
    [sym_url_flg_stat_lookup] = ACTIONS(1477),
    [sym_url_flg_stat_connecting] = ACTIONS(1477),
    [sym_url_flg_stat_waiting] = ACTIONS(1477),
    [sym_url_flg_stat_connected] = ACTIONS(1477),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1477),
    [sym_first_virtual_device] = ACTIONS(1477),
    [sym_first_local_port] = ACTIONS(1477),
    [sym_dynamic_virtual_device] = ACTIONS(1477),
    [sym_timeline_once] = ACTIONS(1477),
    [sym_timeline_repeat] = ACTIONS(1477),
    [sym_timeline_absolute] = ACTIONS(1477),
    [sym_timeline_relative] = ACTIONS(1477),
    [sym_do_push_timed_infinite] = ACTIONS(1477),
    [sym_source_type_no_address] = ACTIONS(1477),
    [sym_source_type_neuron_id] = ACTIONS(1477),
    [sym_source_type_ip_address] = ACTIONS(1477),
    [sym_source_type_axlink] = ACTIONS(1477),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1477),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1477),
    [sym_source_type_ip_socket_address] = ACTIONS(1477),
    [sym_source_type_rs232] = ACTIONS(1477),
    [sym_source_type_internal] = ACTIONS(1477),
    [sym_source_type_mac_address] = ACTIONS(1477),
    [sym_source_type_ipv4_port] = ACTIONS(1477),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1477),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1477),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1477),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1477),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1477),
    [sym_true] = ACTIONS(1477),
    [sym_false] = ACTIONS(1477),
    [sym_file_read_only] = ACTIONS(1477),
    [sym_file_rw_new] = ACTIONS(1477),
    [sym_file_rw_append] = ACTIONS(1477),
    [sym_ip_tcp] = ACTIONS(1477),
    [sym_ip_udp] = ACTIONS(1477),
    [sym_ip_udp_2way] = ACTIONS(1477),
    [sym_xml_encode_types] = ACTIONS(1477),
    [sym_xml_encode_char_as_list] = ACTIONS(1477),
    [sym_xml_encode_le] = ACTIONS(1477),
    [sym_xml_decode_types] = ACTIONS(1477),
    [sym_xml_decode_no_preserve] = ACTIONS(1477),
    [sym_duet_dev_type_display_device] = ACTIONS(1477),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1477),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1477),
    [sym_duet_dev_type_io_device] = ACTIONS(1477),
    [sym_duet_dev_type_relay_device] = ACTIONS(1477),
    [sym_duet_dev_type_ups] = ACTIONS(1477),
    [sym_duet_dev_type_amplifier] = ACTIONS(1477),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1477),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1477),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1477),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1477),
    [sym_duet_dev_type_camera] = ACTIONS(1477),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1477),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1477),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1477),
    [sym_duet_dev_type_dss] = ACTIONS(1477),
    [sym_duet_dev_type_dvr] = ACTIONS(1477),
    [sym_duet_dev_type_disc_device] = ACTIONS(1477),
    [sym_duet_dev_type_document_camera] = ACTIONS(1477),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1477),
    [sym_duet_dev_type_hvac] = ACTIONS(1477),
    [sym_duet_dev_type_keypad] = ACTIONS(1477),
    [sym_duet_dev_type_light] = ACTIONS(1477),
    [sym_duet_dev_type_monitor] = ACTIONS(1477),
    [sym_duet_dev_type_motor] = ACTIONS(1477),
    [sym_duet_dev_type_multi_window] = ACTIONS(1477),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1477),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1477),
    [sym_duet_dev_type_receiver] = ACTIONS(1477),
    [sym_duet_dev_type_security_system] = ACTIONS(1477),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1477),
    [sym_duet_dev_type_settop_box] = ACTIONS(1477),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1477),
    [sym_duet_dev_type_switcher] = ACTIONS(1477),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1477),
    [sym_duet_dev_type_tv] = ACTIONS(1477),
    [sym_duet_dev_type_utility] = ACTIONS(1477),
    [sym_duet_dev_type_vcr] = ACTIONS(1477),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1477),
    [sym_duet_dev_type_video_processor] = ACTIONS(1477),
    [sym_duet_dev_type_video_projector] = ACTIONS(1477),
    [sym_duet_dev_type_video_wall] = ACTIONS(1477),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1477),
    [sym_duet_dev_type_weather] = ACTIONS(1477),
    [sym_duet_dev_not_polled] = ACTIONS(1477),
    [sym_duet_dev_polled] = ACTIONS(1477),
    [sym_clkmgr_mode_network] = ACTIONS(1477),
    [sym_clkmgr_mode_standalone] = ACTIONS(1477),
    [sym_normal_standby] = ACTIONS(1477),
    [sym_normal_wake] = ACTIONS(1477),
    [sym_ip_multicast_ttl_option] = ACTIONS(1477),
    [sym_ip_ttl_subnet] = ACTIONS(1477),
    [sym_ip_ttl_site] = ACTIONS(1477),
    [sym_ip_ttl_region] = ACTIONS(1477),
    [sym_ip_ttl_continent] = ACTIONS(1477),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1477),
    [sym_ip_nodelay_on] = ACTIONS(1477),
    [sym_ip_nodelay_off] = ACTIONS(1477),
    [sym_amx_error] = ACTIONS(1477),
    [sym_amx_warning] = ACTIONS(1477),
    [sym_amx_info] = ACTIONS(1477),
    [sym_amx_debug] = ACTIONS(1477),
    [sym_netlinx_login_success] = ACTIONS(1477),
    [sym_netlinx_login_fail] = ACTIONS(1477),
    [sym_netlinx_logout] = ACTIONS(1477),
    [sym_null_str] = ACTIONS(1477),
    [sym_smtp_address] = ACTIONS(1477),
    [sym_smtp_port_number] = ACTIONS(1477),
    [sym_smtp_username] = ACTIONS(1477),
    [sym_smtp_password] = ACTIONS(1477),
    [sym_smtp_require_tls] = ACTIONS(1477),
    [sym_smtp_from] = ACTIONS(1477),
    [sym_smtp_tls_true] = ACTIONS(1477),
    [sym_smtp_tls_false] = ACTIONS(1477),
    [sym_listview_on_row_select_event] = ACTIONS(1477),
    [sym_data_structure_unknown] = ACTIONS(1477),
    [sym_data_structure_datafeed] = ACTIONS(1477),
    [sym_data_structure_datarecordset] = ACTIONS(1477),
    [sym_data_structure_datarecord] = ACTIONS(1477),
    [sym_data_structure_datafield] = ACTIONS(1477),
    [sym_data_max_content_field_count] = ACTIONS(1477),
    [sym_data_max_metadata_field_count] = ACTIONS(1477),
    [sym_data_max_id_length] = ACTIONS(1477),
    [sym_data_max_name_length] = ACTIONS(1477),
    [sym_data_max_description_length] = ACTIONS(1477),
    [sym_data_max_type_length] = ACTIONS(1477),
    [sym_data_max_format_length] = ACTIONS(1477),
    [sym_data_max_source_length] = ACTIONS(1477),
    [sym_data_max_value_length] = ACTIONS(1477),
    [sym_data_max_label_length] = ACTIONS(1477),
    [sym_data_max_url_length] = ACTIONS(1477),
    [sym_max_last_login_info_length] = ACTIONS(1477),
    [sym_data_type_unknown] = ACTIONS(1477),
    [sym_data_type_string] = ACTIONS(1477),
    [sym_data_type_datetime] = ACTIONS(1477),
    [sym_data_type_date] = ACTIONS(1477),
    [sym_data_type_time] = ACTIONS(1477),
    [sym_data_type_image] = ACTIONS(1477),
    [sym_data_format_url] = ACTIONS(1477),
    [sym_data_format_phone] = ACTIONS(1477),
    [sym_data_format_email] = ACTIONS(1477),
    [sym_data_format_iso8601] = ACTIONS(1477),
    [sym_auditlog_priv] = ACTIONS(1477),
    [sym_configuration_priv] = ACTIONS(1477),
    [sym_deviceconfig_priv] = ACTIONS(1477),
    [sym_ftp_priv] = ACTIONS(1477),
    [sym_http_priv] = ACTIONS(1477),
    [sym_networkconfig_priv] = ACTIONS(1477),
    [sym_programport_priv] = ACTIONS(1477),
    [sym_remoteui_priv] = ACTIONS(1477),
    [sym_securitycontrol_priv] = ACTIONS(1477),
    [sym_softwaremanagement_priv] = ACTIONS(1477),
    [sym_terminal_priv] = ACTIONS(1477),
    [sym_tpadmin_priv] = ACTIONS(1477),
    [sym_user1_priv] = ACTIONS(1477),
    [sym_user2_priv] = ACTIONS(1477),
    [sym_user3_priv] = ACTIONS(1477),
    [sym_user4_priv] = ACTIONS(1477),
    [sym_usermanagement_priv] = ACTIONS(1477),
    [sym_valid_account] = ACTIONS(1477),
    [sym_err_name_invalid] = ACTIONS(1477),
    [sym_err_password_invalid] = ACTIONS(1477),
    [sym_err_invalid_account] = ACTIONS(1477),
    [sym_err_no_authorization] = ACTIONS(1477),
    [sym_err_invalid_parameter] = ACTIONS(1477),
    [sym_err_user_locked_out] = ACTIONS(1477),
    [sym_err_user_account_expired] = ACTIONS(1477),
    [sym_err_general_error] = ACTIONS(1477),
    [sym_err_authentication_error] = ACTIONS(1477),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1477),
    [sym_tls_validate_certificate] = ACTIONS(1477),
    [sym_tdata] = ACTIONS(1477),
    [sym_tchannel] = ACTIONS(1477),
    [sym_tlevel] = ACTIONS(1477),
    [sym_tbutton] = ACTIONS(1477),
    [sym_ttimeline] = ACTIONS(1477),
    [sym_tcustom] = ACTIONS(1477),
    [sym_url_struct] = ACTIONS(1477),
    [sym_dns_struct] = ACTIONS(1477),
    [sym_ip_address_struct] = ACTIONS(1477),
    [sym_dev_info_struct] = ACTIONS(1477),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1477),
    [sym_data_feed] = ACTIONS(1477),
    [sym_data_field] = ACTIONS(1477),
    [sym_data_record] = ACTIONS(1477),
    [sym_wc_data_feed] = ACTIONS(1477),
    [sym_wc_data_field] = ACTIONS(1477),
    [sym_wc_data_record] = ACTIONS(1477),
    [sym_last_login_info] = ACTIONS(1477),
    [sym____reserved___] = ACTIONS(1477),
    [sym_date] = ACTIONS(1477),
    [sym_day] = ACTIONS(1477),
    [sym_time] = ACTIONS(1477),
    [sym_ldate] = ACTIONS(1477),
    [sym_push_device] = ACTIONS(1477),
    [sym_push_channel] = ACTIONS(1477),
    [sym_push_devchan] = ACTIONS(1477),
    [sym_release_device] = ACTIONS(1477),
    [sym_release_channel] = ACTIONS(1477),
    [sym_release_devchan] = ACTIONS(1477),
    [sym_master_slot] = ACTIONS(1477),
    [sym_get_pulse_time] = ACTIONS(1477),
    [sym_get_timer] = ACTIONS(1477),
    [sym_master_sn] = ACTIONS(1477),
    [sym_system_number] = ACTIONS(1477),
    [sym_button] = ACTIONS(1477),
    [sym_dv_channel] = ACTIONS(1477),
    [sym_data] = ACTIONS(1477),
    [sym_level] = ACTIONS(1477),
    [sym_timeline] = ACTIONS(1477),
    [sym_channel] = ACTIONS(1477),
    [sym_custom] = ACTIONS(1477),
    [sym_atoi] = ACTIONS(1477),
    [sym_atol] = ACTIONS(1477),
    [sym_atof] = ACTIONS(1477),
    [sym_itoa] = ACTIONS(1477),
    [sym_itohex] = ACTIONS(1477),
    [sym_hextoi] = ACTIONS(1477),
    [sym_ftoa] = ACTIONS(1477),
    [sym_find_string] = ACTIONS(1477),
    [sym_left_string] = ACTIONS(1477),
    [sym_right_string] = ACTIONS(1477),
    [sym_mid_string] = ACTIONS(1477),
    [sym_remove_string] = ACTIONS(1477),
    [sym_get_buffer_string] = ACTIONS(1477),
    [sym_get_buffer_char] = ACTIONS(1477),
    [sym_get_multi_buffer_string] = ACTIONS(1477),
    [sym_length_string] = ACTIONS(1477),
    [sym_max_length_string] = ACTIONS(1477),
    [sym_set_length_string] = ACTIONS(1477),
    [sym_compare_string] = ACTIONS(1477),
    [sym_upper_string] = ACTIONS(1477),
    [sym_lower_string] = ACTIONS(1477),
    [sym_get_last] = ACTIONS(1477),
    [sym_date_to_day] = ACTIONS(1477),
    [sym_date_to_month] = ACTIONS(1477),
    [sym_date_to_year] = ACTIONS(1477),
    [sym_time_to_hour] = ACTIONS(1477),
    [sym_time_to_minute] = ACTIONS(1477),
    [sym_time_to_second] = ACTIONS(1477),
    [sym_day_of_week] = ACTIONS(1477),
    [sym_set_pulse_time] = ACTIONS(1477),
    [sym_set_timer] = ACTIONS(1477),
    [sym_file_open] = ACTIONS(1477),
    [sym_file_close] = ACTIONS(1477),
    [sym_file_read] = ACTIONS(1477),
    [sym_file_read_line] = ACTIONS(1477),
    [sym_file_write] = ACTIONS(1477),
    [sym_file_write_line] = ACTIONS(1477),
    [sym_file_copy] = ACTIONS(1477),
    [sym_file_delete] = ACTIONS(1477),
    [sym_file_rename] = ACTIONS(1477),
    [sym_file_dir] = ACTIONS(1477),
    [sym_file_getdir] = ACTIONS(1477),
    [sym_file_setdir] = ACTIONS(1477),
    [sym_file_create_dir] = ACTIONS(1477),
    [sym_file_remove_dir] = ACTIONS(1477),
    [sym_file_seek] = ACTIONS(1477),
    [sym_length_array] = ACTIONS(1477),
    [sym_max_length_array] = ACTIONS(1477),
    [sym_set_length_array] = ACTIONS(1477),
    [sym_ip_client_open] = ACTIONS(1477),
    [sym_ip_client_close] = ACTIONS(1477),
    [sym_ip_server_open] = ACTIONS(1477),
    [sym_ip_server_close] = ACTIONS(1477),
    [sym_ip_bound_client_open] = ACTIONS(1477),
    [sym_ip_mc_server_open] = ACTIONS(1477),
    [sym_ip_set_option] = ACTIONS(1477),
    [sym_ssh_client_open] = ACTIONS(1477),
    [sym_ssh_client_close] = ACTIONS(1477),
    [sym_tls_client_open] = ACTIONS(1477),
    [sym_tls_client_close] = ACTIONS(1477),
    [sym_do_push] = ACTIONS(1477),
    [sym_do_push_timed] = ACTIONS(1477),
    [sym_do_release] = ACTIONS(1477),
    [sym_device_id] = ACTIONS(1477),
    [sym_device_info] = ACTIONS(1477),
    [sym_device_id_string] = ACTIONS(1477),
    [sym_device_standby] = ACTIONS(1477),
    [sym_device_wake] = ACTIONS(1477),
    [sym_timeline_create] = ACTIONS(1477),
    [sym_timeline_kill] = ACTIONS(1477),
    [sym_timeline_reload] = ACTIONS(1477),
    [sym_timeline_pause] = ACTIONS(1477),
    [sym_timeline_restart] = ACTIONS(1477),
    [sym_timeline_set] = ACTIONS(1477),
    [sym_timeline_get] = ACTIONS(1477),
    [sym_timeline_active] = ACTIONS(1477),
    [sym_timeline_dynamic_id] = ACTIONS(1477),
    [sym_variable_to_string] = ACTIONS(1477),
    [sym_string_to_variable] = ACTIONS(1477),
    [sym_length_variable_to_string] = ACTIONS(1477),
    [sym_type_cast] = ACTIONS(1477),
    [sym_variable_to_xml] = ACTIONS(1477),
    [sym_xml_to_variable] = ACTIONS(1477),
    [sym_length_variable_to_xml] = ACTIONS(1477),
    [sym_redirect_string] = ACTIONS(1477),
    [sym_random_number] = ACTIONS(1477),
    [sym_get_unique_id] = ACTIONS(1477),
    [sym_get_system_number] = ACTIONS(1477),
    [sym_set_system_number] = ACTIONS(1477),
    [sym_get_serial_number] = ACTIONS(1477),
    [sym_get_master_build] = ACTIONS(1477),
    [sym_set_validation_code] = ACTIONS(1477),
    [sym_reboot] = ACTIONS(1477),
    [sym_load_duet_module] = ACTIONS(1477),
    [sym_unload_duet_module] = ACTIONS(1477),
    [sym_duet_mem_size_set] = ACTIONS(1477),
    [sym_duet_mem_size_get] = ACTIONS(1477),
    [sym_dynamic_application_device] = ACTIONS(1477),
    [sym_dynamic_polled_port] = ACTIONS(1477),
    [sym_static_port_binding] = ACTIONS(1477),
    [sym_static_ip_binding] = ACTIONS(1477),
    [sym_get_dns_list] = ACTIONS(1477),
    [sym_set_dns_list] = ACTIONS(1477),
    [sym_get_ip_address] = ACTIONS(1477),
    [sym_set_ip_address] = ACTIONS(1477),
    [sym_get_url_list] = ACTIONS(1477),
    [sym_add_url_entry] = ACTIONS(1477),
    [sym_delete_url_entry] = ACTIONS(1477),
    [sym_abs_value] = ACTIONS(1477),
    [sym_max_value] = ACTIONS(1477),
    [sym_min_value] = ACTIONS(1477),
    [sym_exp_value] = ACTIONS(1477),
    [sym_log_value] = ACTIONS(1477),
    [sym_log10_value] = ACTIONS(1477),
    [sym_power_value] = ACTIONS(1477),
    [sym_sqrt_value] = ACTIONS(1477),
    [sym_raw_be] = ACTIONS(1477),
    [sym_raw_le] = ACTIONS(1477),
    [sym_internal_queue_size_set] = ACTIONS(1477),
    [sym_internal_queue_size_get] = ACTIONS(1477),
    [sym_internal_threshold_set] = ACTIONS(1477),
    [sym_internal_threshold_get] = ACTIONS(1477),
    [sym_rebuilt_event] = ACTIONS(1477),
    [sym_do_custom_event] = ACTIONS(1477),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1477),
    [sym_clkmgr_set_clk_source] = ACTIONS(1477),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1477),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1477),
    [sym_clkmgr_get_timezone] = ACTIONS(1477),
    [sym_clkmgr_set_timezone] = ACTIONS(1477),
    [sym_clkmgr_get_resync_period] = ACTIONS(1477),
    [sym_clkmgr_set_resync_period] = ACTIONS(1477),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1477),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1477),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1477),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1477),
    [sym_clkmgr_get_timeservers] = ACTIONS(1477),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1477),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1477),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1477),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1477),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1477),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1477),
    [sym_set_log_level] = ACTIONS(1477),
    [sym_get_log_level] = ACTIONS(1477),
    [sym_amx_log] = ACTIONS(1477),
    [sym_smtp_server_config_set] = ACTIONS(1477),
    [sym_smtp_server_config_get] = ACTIONS(1477),
    [sym_smtp_send] = ACTIONS(1477),
    [sym_validate_netlinx_account] = ACTIONS(1477),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1477),
    [sym_authenticate_certificate] = ACTIONS(1477),
    [sym_audit_netlinx_session_event] = ACTIONS(1477),
    [sym_audit_netlinx_generic_event] = ACTIONS(1477),
    [sym_get_available_flash_disk_space] = ACTIONS(1477),
    [sym_get_max_flash_disk_space] = ACTIONS(1477),
    [sym_data_create_feed] = ACTIONS(1477),
    [sym_data_delete_feed] = ACTIONS(1477),
    [sym_data_publish_feed] = ACTIONS(1477),
    [sym_data_get_published_feed] = ACTIONS(1477),
    [sym_data_add_record] = ACTIONS(1477),
    [sym_data_get_event_record] = ACTIONS(1477),
    [sym__wc_data_create_feed] = ACTIONS(1477),
    [sym__wc_data_add_record] = ACTIONS(1477),
    [sym__wc_data_get_event_record] = ACTIONS(1477),
    [sym_format] = ACTIONS(1477),
    [sym_astro_clock] = ACTIONS(1477),
    [sym_set_outdoor_temperature] = ACTIONS(1477),
    [sym_set_virtual_level_count] = ACTIONS(1477),
    [sym_set_virtual_channel_count] = ACTIONS(1477),
    [sym_set_virtual_port_count] = ACTIONS(1477),
    [sym__wc_encode_int] = ACTIONS(1477),
    [sym__wc_decode_int] = ACTIONS(1477),
    [sym__wc_int] = ACTIONS(1477),
    [sym__wc_to_ch_int] = ACTIONS(1477),
    [sym__ch_to_wc_int] = ACTIONS(1477),
    [sym__wc_find_string_int] = ACTIONS(1477),
    [sym__wc_left_string_int] = ACTIONS(1477),
    [sym__wc_lower_string_int] = ACTIONS(1477),
    [sym__wc_mid_string_int] = ACTIONS(1477),
    [sym__wc_remove_string_int] = ACTIONS(1477),
    [sym__wc_right_string_int] = ACTIONS(1477),
    [sym__wc_upper_string_int] = ACTIONS(1477),
    [sym__wc_compare_string_int] = ACTIONS(1477),
    [sym__wc_get_buffer_char_int] = ACTIONS(1477),
    [sym__wc_get_buffer_string_int] = ACTIONS(1477),
    [sym__wc_concat_string_int] = ACTIONS(1477),
    [sym__wc_file_open_int] = ACTIONS(1477),
    [sym__wc_file_close_int] = ACTIONS(1477),
    [sym__wc_file_read_int] = ACTIONS(1477),
    [sym__wc_file_read_line_int] = ACTIONS(1477),
    [sym__wc_file_write_int] = ACTIONS(1477),
    [sym__wc_file_write_line_int] = ACTIONS(1477),
    [aux_sym_preproc_include_token1] = ACTIONS(1477),
    [aux_sym_preproc_define_token1] = ACTIONS(1479),
    [aux_sym_preproc_warn_token1] = ACTIONS(1479),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1479),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1479),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1479),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1479),
    [aux_sym_preproc_else_token1] = ACTIONS(1479),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1479),
    [anon_sym_COLON] = ACTIONS(1479),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1479),
    [anon_sym_BANG] = ACTIONS(1477),
    [anon_sym_TILDE] = ACTIONS(1479),
    [anon_sym_DASH] = ACTIONS(1477),
    [anon_sym_PLUS] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(1479),
    [anon_sym_SLASH] = ACTIONS(1477),
    [anon_sym_PERCENT] = ACTIONS(1479),
    [anon_sym_PIPE_PIPE] = ACTIONS(1479),
    [anon_sym_AMP_AMP] = ACTIONS(1479),
    [anon_sym_PIPE] = ACTIONS(1477),
    [anon_sym_CARET] = ACTIONS(1479),
    [anon_sym_AMP] = ACTIONS(1477),
    [anon_sym_EQ_EQ] = ACTIONS(1479),
    [anon_sym_BANG_EQ] = ACTIONS(1479),
    [anon_sym_LT_GT] = ACTIONS(1479),
    [anon_sym_GT] = ACTIONS(1477),
    [anon_sym_GT_EQ] = ACTIONS(1479),
    [anon_sym_LT_EQ] = ACTIONS(1479),
    [anon_sym_LT] = ACTIONS(1477),
    [anon_sym_LT_LT] = ACTIONS(1479),
    [anon_sym_GT_GT] = ACTIONS(1479),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1479),
    [anon_sym_SQUOTE] = ACTIONS(1479),
    [sym_number_literal] = ACTIONS(1479),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1479),
  },
  [STATE(65)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1481),
    [sym_program_name_keyword] = ACTIONS(1481),
    [sym_module_name_keyword] = ACTIONS(1481),
    [sym_define_device_keyword] = ACTIONS(1481),
    [sym_define_combine_keyword] = ACTIONS(1481),
    [sym_define_constant_keyword] = ACTIONS(1481),
    [sym_define_type_keyword] = ACTIONS(1481),
    [sym_define_variable_keyword] = ACTIONS(1481),
    [sym_define_system_variable_keyword] = ACTIONS(1481),
    [sym_define_connect_level_keyword] = ACTIONS(1481),
    [sym_define_latching_keyword] = ACTIONS(1481),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1481),
    [sym_define_toggling_keyword] = ACTIONS(1481),
    [sym_define_call_keyword] = ACTIONS(1481),
    [sym_define_function_keyword] = ACTIONS(1481),
    [sym_define_library_function_keyword] = ACTIONS(1481),
    [sym_define_start_keyword] = ACTIONS(1481),
    [sym_define_event_keyword] = ACTIONS(1481),
    [sym_define_program_keyword] = ACTIONS(1481),
    [sym_define_module_keyword] = ACTIONS(1481),
    [sym_char_keyword] = ACTIONS(1481),
    [sym_widechar_keyword] = ACTIONS(1481),
    [sym_integer_keyword] = ACTIONS(1481),
    [sym_sinteger_keyword] = ACTIONS(1481),
    [sym_long_keyword] = ACTIONS(1481),
    [sym_slong_keyword] = ACTIONS(1481),
    [sym_float_keyword] = ACTIONS(1481),
    [sym_double_keyword] = ACTIONS(1481),
    [sym_variant_keyword] = ACTIONS(1481),
    [sym_variantarray_keyword] = ACTIONS(1481),
    [sym_local_var_keyword] = ACTIONS(1481),
    [sym_stack_var_keyword] = ACTIONS(1481),
    [sym_constant_keyword] = ACTIONS(1481),
    [sym_volatile_keyword] = ACTIONS(1481),
    [sym_non_volatile_keyword] = ACTIONS(1481),
    [sym_persistent_keyword] = ACTIONS(1481),
    [sym_dev_keyword] = ACTIONS(1481),
    [sym_devlev_keyword] = ACTIONS(1481),
    [sym_devchan_keyword] = ACTIONS(1481),
    [sym_if_keyword] = ACTIONS(1481),
    [sym_else_keyword] = ACTIONS(1481),
    [sym_switch_keyword] = ACTIONS(1481),
    [sym_case_keyword] = ACTIONS(1481),
    [sym_default_keyword] = ACTIONS(1481),
    [sym_while_keyword] = ACTIONS(1481),
    [sym_for_keyword] = ACTIONS(1481),
    [sym_break_keyword] = ACTIONS(1481),
    [sym_continue_keyword] = ACTIONS(1481),
    [sym_return_keyword] = ACTIONS(1481),
    [sym_select_keyword] = ACTIONS(1481),
    [sym_active_keyword] = ACTIONS(1481),
    [sym_struct_keyword] = ACTIONS(1481),
    [sym_structure_keyword] = ACTIONS(1481),
    [sym_button_event_keyword] = ACTIONS(1481),
    [sym_channel_event_keyword] = ACTIONS(1481),
    [sym_level_event_keyword] = ACTIONS(1481),
    [sym_data_event_keyword] = ACTIONS(1481),
    [sym_timeline_event_keyword] = ACTIONS(1481),
    [sym_custom_event_keyword] = ACTIONS(1481),
    [sym_push_keyword] = ACTIONS(1481),
    [sym_release_keyword] = ACTIONS(1481),
    [aux_sym_on_keyword_token1] = ACTIONS(1481),
    [aux_sym_off_keyword_token1] = ACTIONS(1481),
    [sym_send_level_keyword] = ACTIONS(1481),
    [sym_send_string_keyword] = ACTIONS(1481),
    [sym_send_command_keyword] = ACTIONS(1481),
    [sym_clear_buffer_keyword] = ACTIONS(1481),
    [sym_create_buffer_keyword] = ACTIONS(1481),
    [sym_create_multi_buffer_keyword] = ACTIONS(1481),
    [sym_call_keyword] = ACTIONS(1481),
    [sym_system_call_keyword] = ACTIONS(1481),
    [sym_devchan_to_keyword] = ACTIONS(1481),
    [sym_devchan_min_to_keyword] = ACTIONS(1481),
    [sym_devchan_total_off_keyword] = ACTIONS(1481),
    [sym_devchan_pulse_keyword] = ACTIONS(1481),
    [sym_wait_keyword] = ACTIONS(1481),
    [sym_cancel_wait_keyword] = ACTIONS(1481),
    [sym_wait_until_keyword] = ACTIONS(1481),
    [sym_cancel_wait_until_keyword] = ACTIONS(1481),
    [sym_cancel_all_wait_keyword] = ACTIONS(1481),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1481),
    [sym_and] = ACTIONS(1359),
    [sym_or] = ACTIONS(1361),
    [sym_not] = ACTIONS(1481),
    [sym_xor] = ACTIONS(1363),
    [sym_band] = ACTIONS(1365),
    [sym_bor] = ACTIONS(1367),
    [sym_bxor] = ACTIONS(1363),
    [sym_bnot] = ACTIONS(1481),
    [sym_lshift] = ACTIONS(1369),
    [sym_rshift] = ACTIONS(1369),
    [sym___file__] = ACTIONS(1481),
    [sym___date__] = ACTIONS(1481),
    [sym___ldate__] = ACTIONS(1481),
    [sym___time__] = ACTIONS(1481),
    [sym___line__] = ACTIONS(1481),
    [sym___name__] = ACTIONS(1481),
    [sym___netlinx__] = ACTIONS(1481),
    [sym_netlinx_axi_version] = ACTIONS(1481),
    [sym_url_flg_tcp] = ACTIONS(1481),
    [sym_url_flg_accinfopresent] = ACTIONS(1481),
    [sym_url_flg_temp] = ACTIONS(1481),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1481),
    [sym_url_flg_stat_mask] = ACTIONS(1481),
    [sym_url_flg_stat_lookup] = ACTIONS(1481),
    [sym_url_flg_stat_connecting] = ACTIONS(1481),
    [sym_url_flg_stat_waiting] = ACTIONS(1481),
    [sym_url_flg_stat_connected] = ACTIONS(1481),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1481),
    [sym_first_virtual_device] = ACTIONS(1481),
    [sym_first_local_port] = ACTIONS(1481),
    [sym_dynamic_virtual_device] = ACTIONS(1481),
    [sym_timeline_once] = ACTIONS(1481),
    [sym_timeline_repeat] = ACTIONS(1481),
    [sym_timeline_absolute] = ACTIONS(1481),
    [sym_timeline_relative] = ACTIONS(1481),
    [sym_do_push_timed_infinite] = ACTIONS(1481),
    [sym_source_type_no_address] = ACTIONS(1481),
    [sym_source_type_neuron_id] = ACTIONS(1481),
    [sym_source_type_ip_address] = ACTIONS(1481),
    [sym_source_type_axlink] = ACTIONS(1481),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1481),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1481),
    [sym_source_type_ip_socket_address] = ACTIONS(1481),
    [sym_source_type_rs232] = ACTIONS(1481),
    [sym_source_type_internal] = ACTIONS(1481),
    [sym_source_type_mac_address] = ACTIONS(1481),
    [sym_source_type_ipv4_port] = ACTIONS(1481),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1481),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1481),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1481),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1481),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1481),
    [sym_true] = ACTIONS(1481),
    [sym_false] = ACTIONS(1481),
    [sym_file_read_only] = ACTIONS(1481),
    [sym_file_rw_new] = ACTIONS(1481),
    [sym_file_rw_append] = ACTIONS(1481),
    [sym_ip_tcp] = ACTIONS(1481),
    [sym_ip_udp] = ACTIONS(1481),
    [sym_ip_udp_2way] = ACTIONS(1481),
    [sym_xml_encode_types] = ACTIONS(1481),
    [sym_xml_encode_char_as_list] = ACTIONS(1481),
    [sym_xml_encode_le] = ACTIONS(1481),
    [sym_xml_decode_types] = ACTIONS(1481),
    [sym_xml_decode_no_preserve] = ACTIONS(1481),
    [sym_duet_dev_type_display_device] = ACTIONS(1481),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1481),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1481),
    [sym_duet_dev_type_io_device] = ACTIONS(1481),
    [sym_duet_dev_type_relay_device] = ACTIONS(1481),
    [sym_duet_dev_type_ups] = ACTIONS(1481),
    [sym_duet_dev_type_amplifier] = ACTIONS(1481),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1481),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1481),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1481),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1481),
    [sym_duet_dev_type_camera] = ACTIONS(1481),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1481),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1481),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1481),
    [sym_duet_dev_type_dss] = ACTIONS(1481),
    [sym_duet_dev_type_dvr] = ACTIONS(1481),
    [sym_duet_dev_type_disc_device] = ACTIONS(1481),
    [sym_duet_dev_type_document_camera] = ACTIONS(1481),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1481),
    [sym_duet_dev_type_hvac] = ACTIONS(1481),
    [sym_duet_dev_type_keypad] = ACTIONS(1481),
    [sym_duet_dev_type_light] = ACTIONS(1481),
    [sym_duet_dev_type_monitor] = ACTIONS(1481),
    [sym_duet_dev_type_motor] = ACTIONS(1481),
    [sym_duet_dev_type_multi_window] = ACTIONS(1481),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1481),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1481),
    [sym_duet_dev_type_receiver] = ACTIONS(1481),
    [sym_duet_dev_type_security_system] = ACTIONS(1481),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1481),
    [sym_duet_dev_type_settop_box] = ACTIONS(1481),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1481),
    [sym_duet_dev_type_switcher] = ACTIONS(1481),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1481),
    [sym_duet_dev_type_tv] = ACTIONS(1481),
    [sym_duet_dev_type_utility] = ACTIONS(1481),
    [sym_duet_dev_type_vcr] = ACTIONS(1481),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1481),
    [sym_duet_dev_type_video_processor] = ACTIONS(1481),
    [sym_duet_dev_type_video_projector] = ACTIONS(1481),
    [sym_duet_dev_type_video_wall] = ACTIONS(1481),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1481),
    [sym_duet_dev_type_weather] = ACTIONS(1481),
    [sym_duet_dev_not_polled] = ACTIONS(1481),
    [sym_duet_dev_polled] = ACTIONS(1481),
    [sym_clkmgr_mode_network] = ACTIONS(1481),
    [sym_clkmgr_mode_standalone] = ACTIONS(1481),
    [sym_normal_standby] = ACTIONS(1481),
    [sym_normal_wake] = ACTIONS(1481),
    [sym_ip_multicast_ttl_option] = ACTIONS(1481),
    [sym_ip_ttl_subnet] = ACTIONS(1481),
    [sym_ip_ttl_site] = ACTIONS(1481),
    [sym_ip_ttl_region] = ACTIONS(1481),
    [sym_ip_ttl_continent] = ACTIONS(1481),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1481),
    [sym_ip_nodelay_on] = ACTIONS(1481),
    [sym_ip_nodelay_off] = ACTIONS(1481),
    [sym_amx_error] = ACTIONS(1481),
    [sym_amx_warning] = ACTIONS(1481),
    [sym_amx_info] = ACTIONS(1481),
    [sym_amx_debug] = ACTIONS(1481),
    [sym_netlinx_login_success] = ACTIONS(1481),
    [sym_netlinx_login_fail] = ACTIONS(1481),
    [sym_netlinx_logout] = ACTIONS(1481),
    [sym_null_str] = ACTIONS(1481),
    [sym_smtp_address] = ACTIONS(1481),
    [sym_smtp_port_number] = ACTIONS(1481),
    [sym_smtp_username] = ACTIONS(1481),
    [sym_smtp_password] = ACTIONS(1481),
    [sym_smtp_require_tls] = ACTIONS(1481),
    [sym_smtp_from] = ACTIONS(1481),
    [sym_smtp_tls_true] = ACTIONS(1481),
    [sym_smtp_tls_false] = ACTIONS(1481),
    [sym_listview_on_row_select_event] = ACTIONS(1481),
    [sym_data_structure_unknown] = ACTIONS(1481),
    [sym_data_structure_datafeed] = ACTIONS(1481),
    [sym_data_structure_datarecordset] = ACTIONS(1481),
    [sym_data_structure_datarecord] = ACTIONS(1481),
    [sym_data_structure_datafield] = ACTIONS(1481),
    [sym_data_max_content_field_count] = ACTIONS(1481),
    [sym_data_max_metadata_field_count] = ACTIONS(1481),
    [sym_data_max_id_length] = ACTIONS(1481),
    [sym_data_max_name_length] = ACTIONS(1481),
    [sym_data_max_description_length] = ACTIONS(1481),
    [sym_data_max_type_length] = ACTIONS(1481),
    [sym_data_max_format_length] = ACTIONS(1481),
    [sym_data_max_source_length] = ACTIONS(1481),
    [sym_data_max_value_length] = ACTIONS(1481),
    [sym_data_max_label_length] = ACTIONS(1481),
    [sym_data_max_url_length] = ACTIONS(1481),
    [sym_max_last_login_info_length] = ACTIONS(1481),
    [sym_data_type_unknown] = ACTIONS(1481),
    [sym_data_type_string] = ACTIONS(1481),
    [sym_data_type_datetime] = ACTIONS(1481),
    [sym_data_type_date] = ACTIONS(1481),
    [sym_data_type_time] = ACTIONS(1481),
    [sym_data_type_image] = ACTIONS(1481),
    [sym_data_format_url] = ACTIONS(1481),
    [sym_data_format_phone] = ACTIONS(1481),
    [sym_data_format_email] = ACTIONS(1481),
    [sym_data_format_iso8601] = ACTIONS(1481),
    [sym_auditlog_priv] = ACTIONS(1481),
    [sym_configuration_priv] = ACTIONS(1481),
    [sym_deviceconfig_priv] = ACTIONS(1481),
    [sym_ftp_priv] = ACTIONS(1481),
    [sym_http_priv] = ACTIONS(1481),
    [sym_networkconfig_priv] = ACTIONS(1481),
    [sym_programport_priv] = ACTIONS(1481),
    [sym_remoteui_priv] = ACTIONS(1481),
    [sym_securitycontrol_priv] = ACTIONS(1481),
    [sym_softwaremanagement_priv] = ACTIONS(1481),
    [sym_terminal_priv] = ACTIONS(1481),
    [sym_tpadmin_priv] = ACTIONS(1481),
    [sym_user1_priv] = ACTIONS(1481),
    [sym_user2_priv] = ACTIONS(1481),
    [sym_user3_priv] = ACTIONS(1481),
    [sym_user4_priv] = ACTIONS(1481),
    [sym_usermanagement_priv] = ACTIONS(1481),
    [sym_valid_account] = ACTIONS(1481),
    [sym_err_name_invalid] = ACTIONS(1481),
    [sym_err_password_invalid] = ACTIONS(1481),
    [sym_err_invalid_account] = ACTIONS(1481),
    [sym_err_no_authorization] = ACTIONS(1481),
    [sym_err_invalid_parameter] = ACTIONS(1481),
    [sym_err_user_locked_out] = ACTIONS(1481),
    [sym_err_user_account_expired] = ACTIONS(1481),
    [sym_err_general_error] = ACTIONS(1481),
    [sym_err_authentication_error] = ACTIONS(1481),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1481),
    [sym_tls_validate_certificate] = ACTIONS(1481),
    [sym_tdata] = ACTIONS(1481),
    [sym_tchannel] = ACTIONS(1481),
    [sym_tlevel] = ACTIONS(1481),
    [sym_tbutton] = ACTIONS(1481),
    [sym_ttimeline] = ACTIONS(1481),
    [sym_tcustom] = ACTIONS(1481),
    [sym_url_struct] = ACTIONS(1481),
    [sym_dns_struct] = ACTIONS(1481),
    [sym_ip_address_struct] = ACTIONS(1481),
    [sym_dev_info_struct] = ACTIONS(1481),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1481),
    [sym_data_feed] = ACTIONS(1481),
    [sym_data_field] = ACTIONS(1481),
    [sym_data_record] = ACTIONS(1481),
    [sym_wc_data_feed] = ACTIONS(1481),
    [sym_wc_data_field] = ACTIONS(1481),
    [sym_wc_data_record] = ACTIONS(1481),
    [sym_last_login_info] = ACTIONS(1481),
    [sym____reserved___] = ACTIONS(1481),
    [sym_date] = ACTIONS(1481),
    [sym_day] = ACTIONS(1481),
    [sym_time] = ACTIONS(1481),
    [sym_ldate] = ACTIONS(1481),
    [sym_push_device] = ACTIONS(1481),
    [sym_push_channel] = ACTIONS(1481),
    [sym_push_devchan] = ACTIONS(1481),
    [sym_release_device] = ACTIONS(1481),
    [sym_release_channel] = ACTIONS(1481),
    [sym_release_devchan] = ACTIONS(1481),
    [sym_master_slot] = ACTIONS(1481),
    [sym_get_pulse_time] = ACTIONS(1481),
    [sym_get_timer] = ACTIONS(1481),
    [sym_master_sn] = ACTIONS(1481),
    [sym_system_number] = ACTIONS(1481),
    [sym_button] = ACTIONS(1481),
    [sym_dv_channel] = ACTIONS(1481),
    [sym_data] = ACTIONS(1481),
    [sym_level] = ACTIONS(1481),
    [sym_timeline] = ACTIONS(1481),
    [sym_channel] = ACTIONS(1481),
    [sym_custom] = ACTIONS(1481),
    [sym_atoi] = ACTIONS(1481),
    [sym_atol] = ACTIONS(1481),
    [sym_atof] = ACTIONS(1481),
    [sym_itoa] = ACTIONS(1481),
    [sym_itohex] = ACTIONS(1481),
    [sym_hextoi] = ACTIONS(1481),
    [sym_ftoa] = ACTIONS(1481),
    [sym_find_string] = ACTIONS(1481),
    [sym_left_string] = ACTIONS(1481),
    [sym_right_string] = ACTIONS(1481),
    [sym_mid_string] = ACTIONS(1481),
    [sym_remove_string] = ACTIONS(1481),
    [sym_get_buffer_string] = ACTIONS(1481),
    [sym_get_buffer_char] = ACTIONS(1481),
    [sym_get_multi_buffer_string] = ACTIONS(1481),
    [sym_length_string] = ACTIONS(1481),
    [sym_max_length_string] = ACTIONS(1481),
    [sym_set_length_string] = ACTIONS(1481),
    [sym_compare_string] = ACTIONS(1481),
    [sym_upper_string] = ACTIONS(1481),
    [sym_lower_string] = ACTIONS(1481),
    [sym_get_last] = ACTIONS(1481),
    [sym_date_to_day] = ACTIONS(1481),
    [sym_date_to_month] = ACTIONS(1481),
    [sym_date_to_year] = ACTIONS(1481),
    [sym_time_to_hour] = ACTIONS(1481),
    [sym_time_to_minute] = ACTIONS(1481),
    [sym_time_to_second] = ACTIONS(1481),
    [sym_day_of_week] = ACTIONS(1481),
    [sym_set_pulse_time] = ACTIONS(1481),
    [sym_set_timer] = ACTIONS(1481),
    [sym_file_open] = ACTIONS(1481),
    [sym_file_close] = ACTIONS(1481),
    [sym_file_read] = ACTIONS(1481),
    [sym_file_read_line] = ACTIONS(1481),
    [sym_file_write] = ACTIONS(1481),
    [sym_file_write_line] = ACTIONS(1481),
    [sym_file_copy] = ACTIONS(1481),
    [sym_file_delete] = ACTIONS(1481),
    [sym_file_rename] = ACTIONS(1481),
    [sym_file_dir] = ACTIONS(1481),
    [sym_file_getdir] = ACTIONS(1481),
    [sym_file_setdir] = ACTIONS(1481),
    [sym_file_create_dir] = ACTIONS(1481),
    [sym_file_remove_dir] = ACTIONS(1481),
    [sym_file_seek] = ACTIONS(1481),
    [sym_length_array] = ACTIONS(1481),
    [sym_max_length_array] = ACTIONS(1481),
    [sym_set_length_array] = ACTIONS(1481),
    [sym_ip_client_open] = ACTIONS(1481),
    [sym_ip_client_close] = ACTIONS(1481),
    [sym_ip_server_open] = ACTIONS(1481),
    [sym_ip_server_close] = ACTIONS(1481),
    [sym_ip_bound_client_open] = ACTIONS(1481),
    [sym_ip_mc_server_open] = ACTIONS(1481),
    [sym_ip_set_option] = ACTIONS(1481),
    [sym_ssh_client_open] = ACTIONS(1481),
    [sym_ssh_client_close] = ACTIONS(1481),
    [sym_tls_client_open] = ACTIONS(1481),
    [sym_tls_client_close] = ACTIONS(1481),
    [sym_do_push] = ACTIONS(1481),
    [sym_do_push_timed] = ACTIONS(1481),
    [sym_do_release] = ACTIONS(1481),
    [sym_device_id] = ACTIONS(1481),
    [sym_device_info] = ACTIONS(1481),
    [sym_device_id_string] = ACTIONS(1481),
    [sym_device_standby] = ACTIONS(1481),
    [sym_device_wake] = ACTIONS(1481),
    [sym_timeline_create] = ACTIONS(1481),
    [sym_timeline_kill] = ACTIONS(1481),
    [sym_timeline_reload] = ACTIONS(1481),
    [sym_timeline_pause] = ACTIONS(1481),
    [sym_timeline_restart] = ACTIONS(1481),
    [sym_timeline_set] = ACTIONS(1481),
    [sym_timeline_get] = ACTIONS(1481),
    [sym_timeline_active] = ACTIONS(1481),
    [sym_timeline_dynamic_id] = ACTIONS(1481),
    [sym_variable_to_string] = ACTIONS(1481),
    [sym_string_to_variable] = ACTIONS(1481),
    [sym_length_variable_to_string] = ACTIONS(1481),
    [sym_type_cast] = ACTIONS(1481),
    [sym_variable_to_xml] = ACTIONS(1481),
    [sym_xml_to_variable] = ACTIONS(1481),
    [sym_length_variable_to_xml] = ACTIONS(1481),
    [sym_redirect_string] = ACTIONS(1481),
    [sym_random_number] = ACTIONS(1481),
    [sym_get_unique_id] = ACTIONS(1481),
    [sym_get_system_number] = ACTIONS(1481),
    [sym_set_system_number] = ACTIONS(1481),
    [sym_get_serial_number] = ACTIONS(1481),
    [sym_get_master_build] = ACTIONS(1481),
    [sym_set_validation_code] = ACTIONS(1481),
    [sym_reboot] = ACTIONS(1481),
    [sym_load_duet_module] = ACTIONS(1481),
    [sym_unload_duet_module] = ACTIONS(1481),
    [sym_duet_mem_size_set] = ACTIONS(1481),
    [sym_duet_mem_size_get] = ACTIONS(1481),
    [sym_dynamic_application_device] = ACTIONS(1481),
    [sym_dynamic_polled_port] = ACTIONS(1481),
    [sym_static_port_binding] = ACTIONS(1481),
    [sym_static_ip_binding] = ACTIONS(1481),
    [sym_get_dns_list] = ACTIONS(1481),
    [sym_set_dns_list] = ACTIONS(1481),
    [sym_get_ip_address] = ACTIONS(1481),
    [sym_set_ip_address] = ACTIONS(1481),
    [sym_get_url_list] = ACTIONS(1481),
    [sym_add_url_entry] = ACTIONS(1481),
    [sym_delete_url_entry] = ACTIONS(1481),
    [sym_abs_value] = ACTIONS(1481),
    [sym_max_value] = ACTIONS(1481),
    [sym_min_value] = ACTIONS(1481),
    [sym_exp_value] = ACTIONS(1481),
    [sym_log_value] = ACTIONS(1481),
    [sym_log10_value] = ACTIONS(1481),
    [sym_power_value] = ACTIONS(1481),
    [sym_sqrt_value] = ACTIONS(1481),
    [sym_raw_be] = ACTIONS(1481),
    [sym_raw_le] = ACTIONS(1481),
    [sym_internal_queue_size_set] = ACTIONS(1481),
    [sym_internal_queue_size_get] = ACTIONS(1481),
    [sym_internal_threshold_set] = ACTIONS(1481),
    [sym_internal_threshold_get] = ACTIONS(1481),
    [sym_rebuilt_event] = ACTIONS(1481),
    [sym_do_custom_event] = ACTIONS(1481),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1481),
    [sym_clkmgr_set_clk_source] = ACTIONS(1481),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1481),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1481),
    [sym_clkmgr_get_timezone] = ACTIONS(1481),
    [sym_clkmgr_set_timezone] = ACTIONS(1481),
    [sym_clkmgr_get_resync_period] = ACTIONS(1481),
    [sym_clkmgr_set_resync_period] = ACTIONS(1481),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1481),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1481),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1481),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1481),
    [sym_clkmgr_get_timeservers] = ACTIONS(1481),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1481),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1481),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1481),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1481),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1481),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1481),
    [sym_set_log_level] = ACTIONS(1481),
    [sym_get_log_level] = ACTIONS(1481),
    [sym_amx_log] = ACTIONS(1481),
    [sym_smtp_server_config_set] = ACTIONS(1481),
    [sym_smtp_server_config_get] = ACTIONS(1481),
    [sym_smtp_send] = ACTIONS(1481),
    [sym_validate_netlinx_account] = ACTIONS(1481),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1481),
    [sym_authenticate_certificate] = ACTIONS(1481),
    [sym_audit_netlinx_session_event] = ACTIONS(1481),
    [sym_audit_netlinx_generic_event] = ACTIONS(1481),
    [sym_get_available_flash_disk_space] = ACTIONS(1481),
    [sym_get_max_flash_disk_space] = ACTIONS(1481),
    [sym_data_create_feed] = ACTIONS(1481),
    [sym_data_delete_feed] = ACTIONS(1481),
    [sym_data_publish_feed] = ACTIONS(1481),
    [sym_data_get_published_feed] = ACTIONS(1481),
    [sym_data_add_record] = ACTIONS(1481),
    [sym_data_get_event_record] = ACTIONS(1481),
    [sym__wc_data_create_feed] = ACTIONS(1481),
    [sym__wc_data_add_record] = ACTIONS(1481),
    [sym__wc_data_get_event_record] = ACTIONS(1481),
    [sym_format] = ACTIONS(1481),
    [sym_astro_clock] = ACTIONS(1481),
    [sym_set_outdoor_temperature] = ACTIONS(1481),
    [sym_set_virtual_level_count] = ACTIONS(1481),
    [sym_set_virtual_channel_count] = ACTIONS(1481),
    [sym_set_virtual_port_count] = ACTIONS(1481),
    [sym__wc_encode_int] = ACTIONS(1481),
    [sym__wc_decode_int] = ACTIONS(1481),
    [sym__wc_int] = ACTIONS(1481),
    [sym__wc_to_ch_int] = ACTIONS(1481),
    [sym__ch_to_wc_int] = ACTIONS(1481),
    [sym__wc_find_string_int] = ACTIONS(1481),
    [sym__wc_left_string_int] = ACTIONS(1481),
    [sym__wc_lower_string_int] = ACTIONS(1481),
    [sym__wc_mid_string_int] = ACTIONS(1481),
    [sym__wc_remove_string_int] = ACTIONS(1481),
    [sym__wc_right_string_int] = ACTIONS(1481),
    [sym__wc_upper_string_int] = ACTIONS(1481),
    [sym__wc_compare_string_int] = ACTIONS(1481),
    [sym__wc_get_buffer_char_int] = ACTIONS(1481),
    [sym__wc_get_buffer_string_int] = ACTIONS(1481),
    [sym__wc_concat_string_int] = ACTIONS(1481),
    [sym__wc_file_open_int] = ACTIONS(1481),
    [sym__wc_file_close_int] = ACTIONS(1481),
    [sym__wc_file_read_int] = ACTIONS(1481),
    [sym__wc_file_read_line_int] = ACTIONS(1481),
    [sym__wc_file_write_int] = ACTIONS(1481),
    [sym__wc_file_write_line_int] = ACTIONS(1481),
    [aux_sym_preproc_include_token1] = ACTIONS(1481),
    [aux_sym_preproc_define_token1] = ACTIONS(1483),
    [aux_sym_preproc_warn_token1] = ACTIONS(1483),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1483),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1483),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1483),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1483),
    [aux_sym_preproc_else_token1] = ACTIONS(1483),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1483),
    [anon_sym_COLON] = ACTIONS(1485),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1483),
    [anon_sym_BANG] = ACTIONS(1481),
    [anon_sym_TILDE] = ACTIONS(1483),
    [anon_sym_DASH] = ACTIONS(1381),
    [anon_sym_PLUS] = ACTIONS(1381),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1387),
    [anon_sym_AMP_AMP] = ACTIONS(1389),
    [anon_sym_PIPE] = ACTIONS(1367),
    [anon_sym_CARET] = ACTIONS(1391),
    [anon_sym_AMP] = ACTIONS(1365),
    [anon_sym_EQ_EQ] = ACTIONS(1393),
    [anon_sym_BANG_EQ] = ACTIONS(1393),
    [anon_sym_LT_GT] = ACTIONS(1393),
    [anon_sym_GT] = ACTIONS(1395),
    [anon_sym_GT_EQ] = ACTIONS(1397),
    [anon_sym_LT_EQ] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1395),
    [anon_sym_LT_LT] = ACTIONS(1399),
    [anon_sym_GT_GT] = ACTIONS(1399),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1483),
    [anon_sym_SQUOTE] = ACTIONS(1483),
    [sym_number_literal] = ACTIONS(1483),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1483),
  },
  [STATE(66)] = {
    [aux_sym_subscript_expression_repeat1] = STATE(77),
    [sym_identifier] = ACTIONS(1488),
    [sym_program_name_keyword] = ACTIONS(1488),
    [sym_module_name_keyword] = ACTIONS(1488),
    [sym_define_device_keyword] = ACTIONS(1488),
    [sym_define_combine_keyword] = ACTIONS(1488),
    [sym_define_constant_keyword] = ACTIONS(1488),
    [sym_define_type_keyword] = ACTIONS(1488),
    [sym_define_variable_keyword] = ACTIONS(1488),
    [sym_define_system_variable_keyword] = ACTIONS(1488),
    [sym_define_connect_level_keyword] = ACTIONS(1488),
    [sym_define_latching_keyword] = ACTIONS(1488),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1488),
    [sym_define_toggling_keyword] = ACTIONS(1488),
    [sym_define_call_keyword] = ACTIONS(1488),
    [sym_define_function_keyword] = ACTIONS(1488),
    [sym_define_library_function_keyword] = ACTIONS(1488),
    [sym_define_start_keyword] = ACTIONS(1488),
    [sym_define_event_keyword] = ACTIONS(1488),
    [sym_define_program_keyword] = ACTIONS(1488),
    [sym_define_module_keyword] = ACTIONS(1488),
    [sym_char_keyword] = ACTIONS(1488),
    [sym_widechar_keyword] = ACTIONS(1488),
    [sym_integer_keyword] = ACTIONS(1488),
    [sym_sinteger_keyword] = ACTIONS(1488),
    [sym_long_keyword] = ACTIONS(1488),
    [sym_slong_keyword] = ACTIONS(1488),
    [sym_float_keyword] = ACTIONS(1488),
    [sym_double_keyword] = ACTIONS(1488),
    [sym_variant_keyword] = ACTIONS(1488),
    [sym_variantarray_keyword] = ACTIONS(1488),
    [sym_local_var_keyword] = ACTIONS(1488),
    [sym_stack_var_keyword] = ACTIONS(1488),
    [sym_constant_keyword] = ACTIONS(1488),
    [sym_volatile_keyword] = ACTIONS(1488),
    [sym_non_volatile_keyword] = ACTIONS(1488),
    [sym_persistent_keyword] = ACTIONS(1488),
    [sym_dev_keyword] = ACTIONS(1488),
    [sym_devlev_keyword] = ACTIONS(1488),
    [sym_devchan_keyword] = ACTIONS(1488),
    [sym_if_keyword] = ACTIONS(1488),
    [sym_else_keyword] = ACTIONS(1488),
    [sym_switch_keyword] = ACTIONS(1488),
    [sym_case_keyword] = ACTIONS(1488),
    [sym_default_keyword] = ACTIONS(1488),
    [sym_while_keyword] = ACTIONS(1488),
    [sym_for_keyword] = ACTIONS(1488),
    [sym_break_keyword] = ACTIONS(1488),
    [sym_continue_keyword] = ACTIONS(1488),
    [sym_return_keyword] = ACTIONS(1488),
    [sym_select_keyword] = ACTIONS(1488),
    [sym_active_keyword] = ACTIONS(1488),
    [sym_struct_keyword] = ACTIONS(1488),
    [sym_structure_keyword] = ACTIONS(1488),
    [sym_button_event_keyword] = ACTIONS(1488),
    [sym_channel_event_keyword] = ACTIONS(1488),
    [sym_level_event_keyword] = ACTIONS(1488),
    [sym_data_event_keyword] = ACTIONS(1488),
    [sym_timeline_event_keyword] = ACTIONS(1488),
    [sym_custom_event_keyword] = ACTIONS(1488),
    [sym_push_keyword] = ACTIONS(1488),
    [sym_release_keyword] = ACTIONS(1488),
    [aux_sym_on_keyword_token1] = ACTIONS(1488),
    [aux_sym_off_keyword_token1] = ACTIONS(1488),
    [sym_send_level_keyword] = ACTIONS(1488),
    [sym_send_string_keyword] = ACTIONS(1488),
    [sym_send_command_keyword] = ACTIONS(1488),
    [sym_clear_buffer_keyword] = ACTIONS(1488),
    [sym_create_buffer_keyword] = ACTIONS(1488),
    [sym_create_multi_buffer_keyword] = ACTIONS(1488),
    [sym_call_keyword] = ACTIONS(1488),
    [sym_system_call_keyword] = ACTIONS(1488),
    [sym_devchan_to_keyword] = ACTIONS(1488),
    [sym_devchan_min_to_keyword] = ACTIONS(1488),
    [sym_devchan_total_off_keyword] = ACTIONS(1488),
    [sym_devchan_pulse_keyword] = ACTIONS(1488),
    [sym_wait_keyword] = ACTIONS(1488),
    [sym_cancel_wait_keyword] = ACTIONS(1488),
    [sym_wait_until_keyword] = ACTIONS(1488),
    [sym_cancel_wait_until_keyword] = ACTIONS(1488),
    [sym_cancel_all_wait_keyword] = ACTIONS(1488),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1488),
    [sym_and] = ACTIONS(1488),
    [sym_or] = ACTIONS(1488),
    [sym_not] = ACTIONS(1488),
    [sym_xor] = ACTIONS(1488),
    [sym_band] = ACTIONS(1488),
    [sym_bor] = ACTIONS(1488),
    [sym_bxor] = ACTIONS(1488),
    [sym_bnot] = ACTIONS(1488),
    [sym_lshift] = ACTIONS(1488),
    [sym_rshift] = ACTIONS(1488),
    [sym___file__] = ACTIONS(1488),
    [sym___date__] = ACTIONS(1488),
    [sym___ldate__] = ACTIONS(1488),
    [sym___time__] = ACTIONS(1488),
    [sym___line__] = ACTIONS(1488),
    [sym___name__] = ACTIONS(1488),
    [sym___netlinx__] = ACTIONS(1488),
    [sym_netlinx_axi_version] = ACTIONS(1488),
    [sym_url_flg_tcp] = ACTIONS(1488),
    [sym_url_flg_accinfopresent] = ACTIONS(1488),
    [sym_url_flg_temp] = ACTIONS(1488),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1488),
    [sym_url_flg_stat_mask] = ACTIONS(1488),
    [sym_url_flg_stat_lookup] = ACTIONS(1488),
    [sym_url_flg_stat_connecting] = ACTIONS(1488),
    [sym_url_flg_stat_waiting] = ACTIONS(1488),
    [sym_url_flg_stat_connected] = ACTIONS(1488),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1488),
    [sym_first_virtual_device] = ACTIONS(1488),
    [sym_first_local_port] = ACTIONS(1488),
    [sym_dynamic_virtual_device] = ACTIONS(1488),
    [sym_timeline_once] = ACTIONS(1488),
    [sym_timeline_repeat] = ACTIONS(1488),
    [sym_timeline_absolute] = ACTIONS(1488),
    [sym_timeline_relative] = ACTIONS(1488),
    [sym_do_push_timed_infinite] = ACTIONS(1488),
    [sym_source_type_no_address] = ACTIONS(1488),
    [sym_source_type_neuron_id] = ACTIONS(1488),
    [sym_source_type_ip_address] = ACTIONS(1488),
    [sym_source_type_axlink] = ACTIONS(1488),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1488),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1488),
    [sym_source_type_ip_socket_address] = ACTIONS(1488),
    [sym_source_type_rs232] = ACTIONS(1488),
    [sym_source_type_internal] = ACTIONS(1488),
    [sym_source_type_mac_address] = ACTIONS(1488),
    [sym_source_type_ipv4_port] = ACTIONS(1488),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1488),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1488),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1488),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1488),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1488),
    [sym_true] = ACTIONS(1488),
    [sym_false] = ACTIONS(1488),
    [sym_file_read_only] = ACTIONS(1488),
    [sym_file_rw_new] = ACTIONS(1488),
    [sym_file_rw_append] = ACTIONS(1488),
    [sym_ip_tcp] = ACTIONS(1488),
    [sym_ip_udp] = ACTIONS(1488),
    [sym_ip_udp_2way] = ACTIONS(1488),
    [sym_xml_encode_types] = ACTIONS(1488),
    [sym_xml_encode_char_as_list] = ACTIONS(1488),
    [sym_xml_encode_le] = ACTIONS(1488),
    [sym_xml_decode_types] = ACTIONS(1488),
    [sym_xml_decode_no_preserve] = ACTIONS(1488),
    [sym_duet_dev_type_display_device] = ACTIONS(1488),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1488),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1488),
    [sym_duet_dev_type_io_device] = ACTIONS(1488),
    [sym_duet_dev_type_relay_device] = ACTIONS(1488),
    [sym_duet_dev_type_ups] = ACTIONS(1488),
    [sym_duet_dev_type_amplifier] = ACTIONS(1488),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1488),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1488),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1488),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1488),
    [sym_duet_dev_type_camera] = ACTIONS(1488),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1488),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1488),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1488),
    [sym_duet_dev_type_dss] = ACTIONS(1488),
    [sym_duet_dev_type_dvr] = ACTIONS(1488),
    [sym_duet_dev_type_disc_device] = ACTIONS(1488),
    [sym_duet_dev_type_document_camera] = ACTIONS(1488),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1488),
    [sym_duet_dev_type_hvac] = ACTIONS(1488),
    [sym_duet_dev_type_keypad] = ACTIONS(1488),
    [sym_duet_dev_type_light] = ACTIONS(1488),
    [sym_duet_dev_type_monitor] = ACTIONS(1488),
    [sym_duet_dev_type_motor] = ACTIONS(1488),
    [sym_duet_dev_type_multi_window] = ACTIONS(1488),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1488),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1488),
    [sym_duet_dev_type_receiver] = ACTIONS(1488),
    [sym_duet_dev_type_security_system] = ACTIONS(1488),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1488),
    [sym_duet_dev_type_settop_box] = ACTIONS(1488),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1488),
    [sym_duet_dev_type_switcher] = ACTIONS(1488),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1488),
    [sym_duet_dev_type_tv] = ACTIONS(1488),
    [sym_duet_dev_type_utility] = ACTIONS(1488),
    [sym_duet_dev_type_vcr] = ACTIONS(1488),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1488),
    [sym_duet_dev_type_video_processor] = ACTIONS(1488),
    [sym_duet_dev_type_video_projector] = ACTIONS(1488),
    [sym_duet_dev_type_video_wall] = ACTIONS(1488),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1488),
    [sym_duet_dev_type_weather] = ACTIONS(1488),
    [sym_duet_dev_not_polled] = ACTIONS(1488),
    [sym_duet_dev_polled] = ACTIONS(1488),
    [sym_clkmgr_mode_network] = ACTIONS(1488),
    [sym_clkmgr_mode_standalone] = ACTIONS(1488),
    [sym_normal_standby] = ACTIONS(1488),
    [sym_normal_wake] = ACTIONS(1488),
    [sym_ip_multicast_ttl_option] = ACTIONS(1488),
    [sym_ip_ttl_subnet] = ACTIONS(1488),
    [sym_ip_ttl_site] = ACTIONS(1488),
    [sym_ip_ttl_region] = ACTIONS(1488),
    [sym_ip_ttl_continent] = ACTIONS(1488),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1488),
    [sym_ip_nodelay_on] = ACTIONS(1488),
    [sym_ip_nodelay_off] = ACTIONS(1488),
    [sym_amx_error] = ACTIONS(1488),
    [sym_amx_warning] = ACTIONS(1488),
    [sym_amx_info] = ACTIONS(1488),
    [sym_amx_debug] = ACTIONS(1488),
    [sym_netlinx_login_success] = ACTIONS(1488),
    [sym_netlinx_login_fail] = ACTIONS(1488),
    [sym_netlinx_logout] = ACTIONS(1488),
    [sym_null_str] = ACTIONS(1488),
    [sym_smtp_address] = ACTIONS(1488),
    [sym_smtp_port_number] = ACTIONS(1488),
    [sym_smtp_username] = ACTIONS(1488),
    [sym_smtp_password] = ACTIONS(1488),
    [sym_smtp_require_tls] = ACTIONS(1488),
    [sym_smtp_from] = ACTIONS(1488),
    [sym_smtp_tls_true] = ACTIONS(1488),
    [sym_smtp_tls_false] = ACTIONS(1488),
    [sym_listview_on_row_select_event] = ACTIONS(1488),
    [sym_data_structure_unknown] = ACTIONS(1488),
    [sym_data_structure_datafeed] = ACTIONS(1488),
    [sym_data_structure_datarecordset] = ACTIONS(1488),
    [sym_data_structure_datarecord] = ACTIONS(1488),
    [sym_data_structure_datafield] = ACTIONS(1488),
    [sym_data_max_content_field_count] = ACTIONS(1488),
    [sym_data_max_metadata_field_count] = ACTIONS(1488),
    [sym_data_max_id_length] = ACTIONS(1488),
    [sym_data_max_name_length] = ACTIONS(1488),
    [sym_data_max_description_length] = ACTIONS(1488),
    [sym_data_max_type_length] = ACTIONS(1488),
    [sym_data_max_format_length] = ACTIONS(1488),
    [sym_data_max_source_length] = ACTIONS(1488),
    [sym_data_max_value_length] = ACTIONS(1488),
    [sym_data_max_label_length] = ACTIONS(1488),
    [sym_data_max_url_length] = ACTIONS(1488),
    [sym_max_last_login_info_length] = ACTIONS(1488),
    [sym_data_type_unknown] = ACTIONS(1488),
    [sym_data_type_string] = ACTIONS(1488),
    [sym_data_type_datetime] = ACTIONS(1488),
    [sym_data_type_date] = ACTIONS(1488),
    [sym_data_type_time] = ACTIONS(1488),
    [sym_data_type_image] = ACTIONS(1488),
    [sym_data_format_url] = ACTIONS(1488),
    [sym_data_format_phone] = ACTIONS(1488),
    [sym_data_format_email] = ACTIONS(1488),
    [sym_data_format_iso8601] = ACTIONS(1488),
    [sym_auditlog_priv] = ACTIONS(1488),
    [sym_configuration_priv] = ACTIONS(1488),
    [sym_deviceconfig_priv] = ACTIONS(1488),
    [sym_ftp_priv] = ACTIONS(1488),
    [sym_http_priv] = ACTIONS(1488),
    [sym_networkconfig_priv] = ACTIONS(1488),
    [sym_programport_priv] = ACTIONS(1488),
    [sym_remoteui_priv] = ACTIONS(1488),
    [sym_securitycontrol_priv] = ACTIONS(1488),
    [sym_softwaremanagement_priv] = ACTIONS(1488),
    [sym_terminal_priv] = ACTIONS(1488),
    [sym_tpadmin_priv] = ACTIONS(1488),
    [sym_user1_priv] = ACTIONS(1488),
    [sym_user2_priv] = ACTIONS(1488),
    [sym_user3_priv] = ACTIONS(1488),
    [sym_user4_priv] = ACTIONS(1488),
    [sym_usermanagement_priv] = ACTIONS(1488),
    [sym_valid_account] = ACTIONS(1488),
    [sym_err_name_invalid] = ACTIONS(1488),
    [sym_err_password_invalid] = ACTIONS(1488),
    [sym_err_invalid_account] = ACTIONS(1488),
    [sym_err_no_authorization] = ACTIONS(1488),
    [sym_err_invalid_parameter] = ACTIONS(1488),
    [sym_err_user_locked_out] = ACTIONS(1488),
    [sym_err_user_account_expired] = ACTIONS(1488),
    [sym_err_general_error] = ACTIONS(1488),
    [sym_err_authentication_error] = ACTIONS(1488),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1488),
    [sym_tls_validate_certificate] = ACTIONS(1488),
    [sym_tdata] = ACTIONS(1488),
    [sym_tchannel] = ACTIONS(1488),
    [sym_tlevel] = ACTIONS(1488),
    [sym_tbutton] = ACTIONS(1488),
    [sym_ttimeline] = ACTIONS(1488),
    [sym_tcustom] = ACTIONS(1488),
    [sym_url_struct] = ACTIONS(1488),
    [sym_dns_struct] = ACTIONS(1488),
    [sym_ip_address_struct] = ACTIONS(1488),
    [sym_dev_info_struct] = ACTIONS(1488),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1488),
    [sym_data_feed] = ACTIONS(1488),
    [sym_data_field] = ACTIONS(1488),
    [sym_data_record] = ACTIONS(1488),
    [sym_wc_data_feed] = ACTIONS(1488),
    [sym_wc_data_field] = ACTIONS(1488),
    [sym_wc_data_record] = ACTIONS(1488),
    [sym_last_login_info] = ACTIONS(1488),
    [sym____reserved___] = ACTIONS(1488),
    [sym_date] = ACTIONS(1488),
    [sym_day] = ACTIONS(1488),
    [sym_time] = ACTIONS(1488),
    [sym_ldate] = ACTIONS(1488),
    [sym_push_device] = ACTIONS(1488),
    [sym_push_channel] = ACTIONS(1488),
    [sym_push_devchan] = ACTIONS(1488),
    [sym_release_device] = ACTIONS(1488),
    [sym_release_channel] = ACTIONS(1488),
    [sym_release_devchan] = ACTIONS(1488),
    [sym_master_slot] = ACTIONS(1488),
    [sym_get_pulse_time] = ACTIONS(1488),
    [sym_get_timer] = ACTIONS(1488),
    [sym_master_sn] = ACTIONS(1488),
    [sym_system_number] = ACTIONS(1488),
    [sym_button] = ACTIONS(1488),
    [sym_dv_channel] = ACTIONS(1488),
    [sym_data] = ACTIONS(1488),
    [sym_level] = ACTIONS(1488),
    [sym_timeline] = ACTIONS(1488),
    [sym_channel] = ACTIONS(1488),
    [sym_custom] = ACTIONS(1488),
    [sym_atoi] = ACTIONS(1488),
    [sym_atol] = ACTIONS(1488),
    [sym_atof] = ACTIONS(1488),
    [sym_itoa] = ACTIONS(1488),
    [sym_itohex] = ACTIONS(1488),
    [sym_hextoi] = ACTIONS(1488),
    [sym_ftoa] = ACTIONS(1488),
    [sym_find_string] = ACTIONS(1488),
    [sym_left_string] = ACTIONS(1488),
    [sym_right_string] = ACTIONS(1488),
    [sym_mid_string] = ACTIONS(1488),
    [sym_remove_string] = ACTIONS(1488),
    [sym_get_buffer_string] = ACTIONS(1488),
    [sym_get_buffer_char] = ACTIONS(1488),
    [sym_get_multi_buffer_string] = ACTIONS(1488),
    [sym_length_string] = ACTIONS(1488),
    [sym_max_length_string] = ACTIONS(1488),
    [sym_set_length_string] = ACTIONS(1488),
    [sym_compare_string] = ACTIONS(1488),
    [sym_upper_string] = ACTIONS(1488),
    [sym_lower_string] = ACTIONS(1488),
    [sym_get_last] = ACTIONS(1488),
    [sym_date_to_day] = ACTIONS(1488),
    [sym_date_to_month] = ACTIONS(1488),
    [sym_date_to_year] = ACTIONS(1488),
    [sym_time_to_hour] = ACTIONS(1488),
    [sym_time_to_minute] = ACTIONS(1488),
    [sym_time_to_second] = ACTIONS(1488),
    [sym_day_of_week] = ACTIONS(1488),
    [sym_set_pulse_time] = ACTIONS(1488),
    [sym_set_timer] = ACTIONS(1488),
    [sym_file_open] = ACTIONS(1488),
    [sym_file_close] = ACTIONS(1488),
    [sym_file_read] = ACTIONS(1488),
    [sym_file_read_line] = ACTIONS(1488),
    [sym_file_write] = ACTIONS(1488),
    [sym_file_write_line] = ACTIONS(1488),
    [sym_file_copy] = ACTIONS(1488),
    [sym_file_delete] = ACTIONS(1488),
    [sym_file_rename] = ACTIONS(1488),
    [sym_file_dir] = ACTIONS(1488),
    [sym_file_getdir] = ACTIONS(1488),
    [sym_file_setdir] = ACTIONS(1488),
    [sym_file_create_dir] = ACTIONS(1488),
    [sym_file_remove_dir] = ACTIONS(1488),
    [sym_file_seek] = ACTIONS(1488),
    [sym_length_array] = ACTIONS(1488),
    [sym_max_length_array] = ACTIONS(1488),
    [sym_set_length_array] = ACTIONS(1488),
    [sym_ip_client_open] = ACTIONS(1488),
    [sym_ip_client_close] = ACTIONS(1488),
    [sym_ip_server_open] = ACTIONS(1488),
    [sym_ip_server_close] = ACTIONS(1488),
    [sym_ip_bound_client_open] = ACTIONS(1488),
    [sym_ip_mc_server_open] = ACTIONS(1488),
    [sym_ip_set_option] = ACTIONS(1488),
    [sym_ssh_client_open] = ACTIONS(1488),
    [sym_ssh_client_close] = ACTIONS(1488),
    [sym_tls_client_open] = ACTIONS(1488),
    [sym_tls_client_close] = ACTIONS(1488),
    [sym_do_push] = ACTIONS(1488),
    [sym_do_push_timed] = ACTIONS(1488),
    [sym_do_release] = ACTIONS(1488),
    [sym_device_id] = ACTIONS(1488),
    [sym_device_info] = ACTIONS(1488),
    [sym_device_id_string] = ACTIONS(1488),
    [sym_device_standby] = ACTIONS(1488),
    [sym_device_wake] = ACTIONS(1488),
    [sym_timeline_create] = ACTIONS(1488),
    [sym_timeline_kill] = ACTIONS(1488),
    [sym_timeline_reload] = ACTIONS(1488),
    [sym_timeline_pause] = ACTIONS(1488),
    [sym_timeline_restart] = ACTIONS(1488),
    [sym_timeline_set] = ACTIONS(1488),
    [sym_timeline_get] = ACTIONS(1488),
    [sym_timeline_active] = ACTIONS(1488),
    [sym_timeline_dynamic_id] = ACTIONS(1488),
    [sym_variable_to_string] = ACTIONS(1488),
    [sym_string_to_variable] = ACTIONS(1488),
    [sym_length_variable_to_string] = ACTIONS(1488),
    [sym_type_cast] = ACTIONS(1488),
    [sym_variable_to_xml] = ACTIONS(1488),
    [sym_xml_to_variable] = ACTIONS(1488),
    [sym_length_variable_to_xml] = ACTIONS(1488),
    [sym_redirect_string] = ACTIONS(1488),
    [sym_random_number] = ACTIONS(1488),
    [sym_get_unique_id] = ACTIONS(1488),
    [sym_get_system_number] = ACTIONS(1488),
    [sym_set_system_number] = ACTIONS(1488),
    [sym_get_serial_number] = ACTIONS(1488),
    [sym_get_master_build] = ACTIONS(1488),
    [sym_set_validation_code] = ACTIONS(1488),
    [sym_reboot] = ACTIONS(1488),
    [sym_load_duet_module] = ACTIONS(1488),
    [sym_unload_duet_module] = ACTIONS(1488),
    [sym_duet_mem_size_set] = ACTIONS(1488),
    [sym_duet_mem_size_get] = ACTIONS(1488),
    [sym_dynamic_application_device] = ACTIONS(1488),
    [sym_dynamic_polled_port] = ACTIONS(1488),
    [sym_static_port_binding] = ACTIONS(1488),
    [sym_static_ip_binding] = ACTIONS(1488),
    [sym_get_dns_list] = ACTIONS(1488),
    [sym_set_dns_list] = ACTIONS(1488),
    [sym_get_ip_address] = ACTIONS(1488),
    [sym_set_ip_address] = ACTIONS(1488),
    [sym_get_url_list] = ACTIONS(1488),
    [sym_add_url_entry] = ACTIONS(1488),
    [sym_delete_url_entry] = ACTIONS(1488),
    [sym_abs_value] = ACTIONS(1488),
    [sym_max_value] = ACTIONS(1488),
    [sym_min_value] = ACTIONS(1488),
    [sym_exp_value] = ACTIONS(1488),
    [sym_log_value] = ACTIONS(1488),
    [sym_log10_value] = ACTIONS(1488),
    [sym_power_value] = ACTIONS(1488),
    [sym_sqrt_value] = ACTIONS(1488),
    [sym_raw_be] = ACTIONS(1488),
    [sym_raw_le] = ACTIONS(1488),
    [sym_internal_queue_size_set] = ACTIONS(1488),
    [sym_internal_queue_size_get] = ACTIONS(1488),
    [sym_internal_threshold_set] = ACTIONS(1488),
    [sym_internal_threshold_get] = ACTIONS(1488),
    [sym_rebuilt_event] = ACTIONS(1488),
    [sym_do_custom_event] = ACTIONS(1488),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1488),
    [sym_clkmgr_set_clk_source] = ACTIONS(1488),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1488),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1488),
    [sym_clkmgr_get_timezone] = ACTIONS(1488),
    [sym_clkmgr_set_timezone] = ACTIONS(1488),
    [sym_clkmgr_get_resync_period] = ACTIONS(1488),
    [sym_clkmgr_set_resync_period] = ACTIONS(1488),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1488),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1488),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1488),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1488),
    [sym_clkmgr_get_timeservers] = ACTIONS(1488),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1488),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1488),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1488),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1488),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1488),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1488),
    [sym_set_log_level] = ACTIONS(1488),
    [sym_get_log_level] = ACTIONS(1488),
    [sym_amx_log] = ACTIONS(1488),
    [sym_smtp_server_config_set] = ACTIONS(1488),
    [sym_smtp_server_config_get] = ACTIONS(1488),
    [sym_smtp_send] = ACTIONS(1488),
    [sym_validate_netlinx_account] = ACTIONS(1488),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1488),
    [sym_authenticate_certificate] = ACTIONS(1488),
    [sym_audit_netlinx_session_event] = ACTIONS(1488),
    [sym_audit_netlinx_generic_event] = ACTIONS(1488),
    [sym_get_available_flash_disk_space] = ACTIONS(1488),
    [sym_get_max_flash_disk_space] = ACTIONS(1488),
    [sym_data_create_feed] = ACTIONS(1488),
    [sym_data_delete_feed] = ACTIONS(1488),
    [sym_data_publish_feed] = ACTIONS(1488),
    [sym_data_get_published_feed] = ACTIONS(1488),
    [sym_data_add_record] = ACTIONS(1488),
    [sym_data_get_event_record] = ACTIONS(1488),
    [sym__wc_data_create_feed] = ACTIONS(1488),
    [sym__wc_data_add_record] = ACTIONS(1488),
    [sym__wc_data_get_event_record] = ACTIONS(1488),
    [sym_format] = ACTIONS(1488),
    [sym_astro_clock] = ACTIONS(1488),
    [sym_set_outdoor_temperature] = ACTIONS(1488),
    [sym_set_virtual_level_count] = ACTIONS(1488),
    [sym_set_virtual_channel_count] = ACTIONS(1488),
    [sym_set_virtual_port_count] = ACTIONS(1488),
    [sym__wc_encode_int] = ACTIONS(1488),
    [sym__wc_decode_int] = ACTIONS(1488),
    [sym__wc_int] = ACTIONS(1488),
    [sym__wc_to_ch_int] = ACTIONS(1488),
    [sym__ch_to_wc_int] = ACTIONS(1488),
    [sym__wc_find_string_int] = ACTIONS(1488),
    [sym__wc_left_string_int] = ACTIONS(1488),
    [sym__wc_lower_string_int] = ACTIONS(1488),
    [sym__wc_mid_string_int] = ACTIONS(1488),
    [sym__wc_remove_string_int] = ACTIONS(1488),
    [sym__wc_right_string_int] = ACTIONS(1488),
    [sym__wc_upper_string_int] = ACTIONS(1488),
    [sym__wc_compare_string_int] = ACTIONS(1488),
    [sym__wc_get_buffer_char_int] = ACTIONS(1488),
    [sym__wc_get_buffer_string_int] = ACTIONS(1488),
    [sym__wc_concat_string_int] = ACTIONS(1488),
    [sym__wc_file_open_int] = ACTIONS(1488),
    [sym__wc_file_close_int] = ACTIONS(1488),
    [sym__wc_file_read_int] = ACTIONS(1488),
    [sym__wc_file_read_line_int] = ACTIONS(1488),
    [sym__wc_file_write_int] = ACTIONS(1488),
    [sym__wc_file_write_line_int] = ACTIONS(1488),
    [anon_sym_EQ] = ACTIONS(1488),
    [aux_sym_preproc_include_token1] = ACTIONS(1488),
    [aux_sym_preproc_define_token1] = ACTIONS(1490),
    [aux_sym_preproc_warn_token1] = ACTIONS(1490),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1490),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1490),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1490),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1490),
    [aux_sym_preproc_else_token1] = ACTIONS(1490),
    [anon_sym_LBRACK] = ACTIONS(1490),
    [anon_sym_LBRACE] = ACTIONS(1490),
    [anon_sym_COLON] = ACTIONS(1490),
    [anon_sym_LPAREN] = ACTIONS(1488),
    [anon_sym_SEMI] = ACTIONS(1490),
    [anon_sym_BANG] = ACTIONS(1488),
    [anon_sym_TILDE] = ACTIONS(1490),
    [anon_sym_DASH] = ACTIONS(1488),
    [anon_sym_PLUS] = ACTIONS(1488),
    [anon_sym_STAR] = ACTIONS(1490),
    [anon_sym_SLASH] = ACTIONS(1488),
    [anon_sym_PERCENT] = ACTIONS(1490),
    [anon_sym_PIPE_PIPE] = ACTIONS(1490),
    [anon_sym_AMP_AMP] = ACTIONS(1490),
    [anon_sym_PIPE] = ACTIONS(1488),
    [anon_sym_CARET] = ACTIONS(1490),
    [anon_sym_AMP] = ACTIONS(1488),
    [anon_sym_EQ_EQ] = ACTIONS(1490),
    [anon_sym_BANG_EQ] = ACTIONS(1490),
    [anon_sym_LT_GT] = ACTIONS(1490),
    [anon_sym_GT] = ACTIONS(1488),
    [anon_sym_GT_EQ] = ACTIONS(1490),
    [anon_sym_LT_EQ] = ACTIONS(1490),
    [anon_sym_LT] = ACTIONS(1488),
    [anon_sym_LT_LT] = ACTIONS(1490),
    [anon_sym_GT_GT] = ACTIONS(1490),
    [anon_sym_DASH_DASH] = ACTIONS(1490),
    [anon_sym_PLUS_PLUS] = ACTIONS(1490),
    [anon_sym_DOT] = ACTIONS(1488),
    [anon_sym_DQUOTE] = ACTIONS(1490),
    [anon_sym_SQUOTE] = ACTIONS(1490),
    [sym_number_literal] = ACTIONS(1490),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1490),
  },
  [STATE(67)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1492),
    [sym_program_name_keyword] = ACTIONS(1492),
    [sym_module_name_keyword] = ACTIONS(1492),
    [sym_define_device_keyword] = ACTIONS(1492),
    [sym_define_combine_keyword] = ACTIONS(1492),
    [sym_define_constant_keyword] = ACTIONS(1492),
    [sym_define_type_keyword] = ACTIONS(1492),
    [sym_define_variable_keyword] = ACTIONS(1492),
    [sym_define_system_variable_keyword] = ACTIONS(1492),
    [sym_define_connect_level_keyword] = ACTIONS(1492),
    [sym_define_latching_keyword] = ACTIONS(1492),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1492),
    [sym_define_toggling_keyword] = ACTIONS(1492),
    [sym_define_call_keyword] = ACTIONS(1492),
    [sym_define_function_keyword] = ACTIONS(1492),
    [sym_define_library_function_keyword] = ACTIONS(1492),
    [sym_define_start_keyword] = ACTIONS(1492),
    [sym_define_event_keyword] = ACTIONS(1492),
    [sym_define_program_keyword] = ACTIONS(1492),
    [sym_define_module_keyword] = ACTIONS(1492),
    [sym_char_keyword] = ACTIONS(1492),
    [sym_widechar_keyword] = ACTIONS(1492),
    [sym_integer_keyword] = ACTIONS(1492),
    [sym_sinteger_keyword] = ACTIONS(1492),
    [sym_long_keyword] = ACTIONS(1492),
    [sym_slong_keyword] = ACTIONS(1492),
    [sym_float_keyword] = ACTIONS(1492),
    [sym_double_keyword] = ACTIONS(1492),
    [sym_variant_keyword] = ACTIONS(1492),
    [sym_variantarray_keyword] = ACTIONS(1492),
    [sym_local_var_keyword] = ACTIONS(1492),
    [sym_stack_var_keyword] = ACTIONS(1492),
    [sym_constant_keyword] = ACTIONS(1492),
    [sym_volatile_keyword] = ACTIONS(1492),
    [sym_non_volatile_keyword] = ACTIONS(1492),
    [sym_persistent_keyword] = ACTIONS(1492),
    [sym_dev_keyword] = ACTIONS(1492),
    [sym_devlev_keyword] = ACTIONS(1492),
    [sym_devchan_keyword] = ACTIONS(1492),
    [sym_if_keyword] = ACTIONS(1492),
    [sym_else_keyword] = ACTIONS(1492),
    [sym_switch_keyword] = ACTIONS(1492),
    [sym_case_keyword] = ACTIONS(1492),
    [sym_default_keyword] = ACTIONS(1492),
    [sym_while_keyword] = ACTIONS(1492),
    [sym_for_keyword] = ACTIONS(1492),
    [sym_break_keyword] = ACTIONS(1492),
    [sym_continue_keyword] = ACTIONS(1492),
    [sym_return_keyword] = ACTIONS(1492),
    [sym_select_keyword] = ACTIONS(1492),
    [sym_active_keyword] = ACTIONS(1492),
    [sym_struct_keyword] = ACTIONS(1492),
    [sym_structure_keyword] = ACTIONS(1492),
    [sym_button_event_keyword] = ACTIONS(1492),
    [sym_channel_event_keyword] = ACTIONS(1492),
    [sym_level_event_keyword] = ACTIONS(1492),
    [sym_data_event_keyword] = ACTIONS(1492),
    [sym_timeline_event_keyword] = ACTIONS(1492),
    [sym_custom_event_keyword] = ACTIONS(1492),
    [sym_push_keyword] = ACTIONS(1492),
    [sym_release_keyword] = ACTIONS(1492),
    [aux_sym_on_keyword_token1] = ACTIONS(1492),
    [aux_sym_off_keyword_token1] = ACTIONS(1492),
    [sym_send_level_keyword] = ACTIONS(1492),
    [sym_send_string_keyword] = ACTIONS(1492),
    [sym_send_command_keyword] = ACTIONS(1492),
    [sym_clear_buffer_keyword] = ACTIONS(1492),
    [sym_create_buffer_keyword] = ACTIONS(1492),
    [sym_create_multi_buffer_keyword] = ACTIONS(1492),
    [sym_call_keyword] = ACTIONS(1492),
    [sym_system_call_keyword] = ACTIONS(1492),
    [sym_devchan_to_keyword] = ACTIONS(1492),
    [sym_devchan_min_to_keyword] = ACTIONS(1492),
    [sym_devchan_total_off_keyword] = ACTIONS(1492),
    [sym_devchan_pulse_keyword] = ACTIONS(1492),
    [sym_wait_keyword] = ACTIONS(1492),
    [sym_cancel_wait_keyword] = ACTIONS(1492),
    [sym_wait_until_keyword] = ACTIONS(1492),
    [sym_cancel_wait_until_keyword] = ACTIONS(1492),
    [sym_cancel_all_wait_keyword] = ACTIONS(1492),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1492),
    [sym_and] = ACTIONS(1359),
    [sym_or] = ACTIONS(1361),
    [sym_not] = ACTIONS(1492),
    [sym_xor] = ACTIONS(1363),
    [sym_band] = ACTIONS(1365),
    [sym_bor] = ACTIONS(1367),
    [sym_bxor] = ACTIONS(1363),
    [sym_bnot] = ACTIONS(1492),
    [sym_lshift] = ACTIONS(1369),
    [sym_rshift] = ACTIONS(1369),
    [sym___file__] = ACTIONS(1492),
    [sym___date__] = ACTIONS(1492),
    [sym___ldate__] = ACTIONS(1492),
    [sym___time__] = ACTIONS(1492),
    [sym___line__] = ACTIONS(1492),
    [sym___name__] = ACTIONS(1492),
    [sym___netlinx__] = ACTIONS(1492),
    [sym_netlinx_axi_version] = ACTIONS(1492),
    [sym_url_flg_tcp] = ACTIONS(1492),
    [sym_url_flg_accinfopresent] = ACTIONS(1492),
    [sym_url_flg_temp] = ACTIONS(1492),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1492),
    [sym_url_flg_stat_mask] = ACTIONS(1492),
    [sym_url_flg_stat_lookup] = ACTIONS(1492),
    [sym_url_flg_stat_connecting] = ACTIONS(1492),
    [sym_url_flg_stat_waiting] = ACTIONS(1492),
    [sym_url_flg_stat_connected] = ACTIONS(1492),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1492),
    [sym_first_virtual_device] = ACTIONS(1492),
    [sym_first_local_port] = ACTIONS(1492),
    [sym_dynamic_virtual_device] = ACTIONS(1492),
    [sym_timeline_once] = ACTIONS(1492),
    [sym_timeline_repeat] = ACTIONS(1492),
    [sym_timeline_absolute] = ACTIONS(1492),
    [sym_timeline_relative] = ACTIONS(1492),
    [sym_do_push_timed_infinite] = ACTIONS(1492),
    [sym_source_type_no_address] = ACTIONS(1492),
    [sym_source_type_neuron_id] = ACTIONS(1492),
    [sym_source_type_ip_address] = ACTIONS(1492),
    [sym_source_type_axlink] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1492),
    [sym_source_type_ip_socket_address] = ACTIONS(1492),
    [sym_source_type_rs232] = ACTIONS(1492),
    [sym_source_type_internal] = ACTIONS(1492),
    [sym_source_type_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1492),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1492),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1492),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1492),
    [sym_true] = ACTIONS(1492),
    [sym_false] = ACTIONS(1492),
    [sym_file_read_only] = ACTIONS(1492),
    [sym_file_rw_new] = ACTIONS(1492),
    [sym_file_rw_append] = ACTIONS(1492),
    [sym_ip_tcp] = ACTIONS(1492),
    [sym_ip_udp] = ACTIONS(1492),
    [sym_ip_udp_2way] = ACTIONS(1492),
    [sym_xml_encode_types] = ACTIONS(1492),
    [sym_xml_encode_char_as_list] = ACTIONS(1492),
    [sym_xml_encode_le] = ACTIONS(1492),
    [sym_xml_decode_types] = ACTIONS(1492),
    [sym_xml_decode_no_preserve] = ACTIONS(1492),
    [sym_duet_dev_type_display_device] = ACTIONS(1492),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_io_device] = ACTIONS(1492),
    [sym_duet_dev_type_relay_device] = ACTIONS(1492),
    [sym_duet_dev_type_ups] = ACTIONS(1492),
    [sym_duet_dev_type_amplifier] = ACTIONS(1492),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1492),
    [sym_duet_dev_type_camera] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1492),
    [sym_duet_dev_type_dss] = ACTIONS(1492),
    [sym_duet_dev_type_dvr] = ACTIONS(1492),
    [sym_duet_dev_type_disc_device] = ACTIONS(1492),
    [sym_duet_dev_type_document_camera] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1492),
    [sym_duet_dev_type_hvac] = ACTIONS(1492),
    [sym_duet_dev_type_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_light] = ACTIONS(1492),
    [sym_duet_dev_type_monitor] = ACTIONS(1492),
    [sym_duet_dev_type_motor] = ACTIONS(1492),
    [sym_duet_dev_type_multi_window] = ACTIONS(1492),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1492),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1492),
    [sym_duet_dev_type_receiver] = ACTIONS(1492),
    [sym_duet_dev_type_security_system] = ACTIONS(1492),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1492),
    [sym_duet_dev_type_settop_box] = ACTIONS(1492),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1492),
    [sym_duet_dev_type_switcher] = ACTIONS(1492),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_tv] = ACTIONS(1492),
    [sym_duet_dev_type_utility] = ACTIONS(1492),
    [sym_duet_dev_type_vcr] = ACTIONS(1492),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_video_processor] = ACTIONS(1492),
    [sym_duet_dev_type_video_projector] = ACTIONS(1492),
    [sym_duet_dev_type_video_wall] = ACTIONS(1492),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1492),
    [sym_duet_dev_type_weather] = ACTIONS(1492),
    [sym_duet_dev_not_polled] = ACTIONS(1492),
    [sym_duet_dev_polled] = ACTIONS(1492),
    [sym_clkmgr_mode_network] = ACTIONS(1492),
    [sym_clkmgr_mode_standalone] = ACTIONS(1492),
    [sym_normal_standby] = ACTIONS(1492),
    [sym_normal_wake] = ACTIONS(1492),
    [sym_ip_multicast_ttl_option] = ACTIONS(1492),
    [sym_ip_ttl_subnet] = ACTIONS(1492),
    [sym_ip_ttl_site] = ACTIONS(1492),
    [sym_ip_ttl_region] = ACTIONS(1492),
    [sym_ip_ttl_continent] = ACTIONS(1492),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1492),
    [sym_ip_nodelay_on] = ACTIONS(1492),
    [sym_ip_nodelay_off] = ACTIONS(1492),
    [sym_amx_error] = ACTIONS(1492),
    [sym_amx_warning] = ACTIONS(1492),
    [sym_amx_info] = ACTIONS(1492),
    [sym_amx_debug] = ACTIONS(1492),
    [sym_netlinx_login_success] = ACTIONS(1492),
    [sym_netlinx_login_fail] = ACTIONS(1492),
    [sym_netlinx_logout] = ACTIONS(1492),
    [sym_null_str] = ACTIONS(1492),
    [sym_smtp_address] = ACTIONS(1492),
    [sym_smtp_port_number] = ACTIONS(1492),
    [sym_smtp_username] = ACTIONS(1492),
    [sym_smtp_password] = ACTIONS(1492),
    [sym_smtp_require_tls] = ACTIONS(1492),
    [sym_smtp_from] = ACTIONS(1492),
    [sym_smtp_tls_true] = ACTIONS(1492),
    [sym_smtp_tls_false] = ACTIONS(1492),
    [sym_listview_on_row_select_event] = ACTIONS(1492),
    [sym_data_structure_unknown] = ACTIONS(1492),
    [sym_data_structure_datafeed] = ACTIONS(1492),
    [sym_data_structure_datarecordset] = ACTIONS(1492),
    [sym_data_structure_datarecord] = ACTIONS(1492),
    [sym_data_structure_datafield] = ACTIONS(1492),
    [sym_data_max_content_field_count] = ACTIONS(1492),
    [sym_data_max_metadata_field_count] = ACTIONS(1492),
    [sym_data_max_id_length] = ACTIONS(1492),
    [sym_data_max_name_length] = ACTIONS(1492),
    [sym_data_max_description_length] = ACTIONS(1492),
    [sym_data_max_type_length] = ACTIONS(1492),
    [sym_data_max_format_length] = ACTIONS(1492),
    [sym_data_max_source_length] = ACTIONS(1492),
    [sym_data_max_value_length] = ACTIONS(1492),
    [sym_data_max_label_length] = ACTIONS(1492),
    [sym_data_max_url_length] = ACTIONS(1492),
    [sym_max_last_login_info_length] = ACTIONS(1492),
    [sym_data_type_unknown] = ACTIONS(1492),
    [sym_data_type_string] = ACTIONS(1492),
    [sym_data_type_datetime] = ACTIONS(1492),
    [sym_data_type_date] = ACTIONS(1492),
    [sym_data_type_time] = ACTIONS(1492),
    [sym_data_type_image] = ACTIONS(1492),
    [sym_data_format_url] = ACTIONS(1492),
    [sym_data_format_phone] = ACTIONS(1492),
    [sym_data_format_email] = ACTIONS(1492),
    [sym_data_format_iso8601] = ACTIONS(1492),
    [sym_auditlog_priv] = ACTIONS(1492),
    [sym_configuration_priv] = ACTIONS(1492),
    [sym_deviceconfig_priv] = ACTIONS(1492),
    [sym_ftp_priv] = ACTIONS(1492),
    [sym_http_priv] = ACTIONS(1492),
    [sym_networkconfig_priv] = ACTIONS(1492),
    [sym_programport_priv] = ACTIONS(1492),
    [sym_remoteui_priv] = ACTIONS(1492),
    [sym_securitycontrol_priv] = ACTIONS(1492),
    [sym_softwaremanagement_priv] = ACTIONS(1492),
    [sym_terminal_priv] = ACTIONS(1492),
    [sym_tpadmin_priv] = ACTIONS(1492),
    [sym_user1_priv] = ACTIONS(1492),
    [sym_user2_priv] = ACTIONS(1492),
    [sym_user3_priv] = ACTIONS(1492),
    [sym_user4_priv] = ACTIONS(1492),
    [sym_usermanagement_priv] = ACTIONS(1492),
    [sym_valid_account] = ACTIONS(1492),
    [sym_err_name_invalid] = ACTIONS(1492),
    [sym_err_password_invalid] = ACTIONS(1492),
    [sym_err_invalid_account] = ACTIONS(1492),
    [sym_err_no_authorization] = ACTIONS(1492),
    [sym_err_invalid_parameter] = ACTIONS(1492),
    [sym_err_user_locked_out] = ACTIONS(1492),
    [sym_err_user_account_expired] = ACTIONS(1492),
    [sym_err_general_error] = ACTIONS(1492),
    [sym_err_authentication_error] = ACTIONS(1492),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1492),
    [sym_tls_validate_certificate] = ACTIONS(1492),
    [sym_tdata] = ACTIONS(1492),
    [sym_tchannel] = ACTIONS(1492),
    [sym_tlevel] = ACTIONS(1492),
    [sym_tbutton] = ACTIONS(1492),
    [sym_ttimeline] = ACTIONS(1492),
    [sym_tcustom] = ACTIONS(1492),
    [sym_url_struct] = ACTIONS(1492),
    [sym_dns_struct] = ACTIONS(1492),
    [sym_ip_address_struct] = ACTIONS(1492),
    [sym_dev_info_struct] = ACTIONS(1492),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1492),
    [sym_data_feed] = ACTIONS(1492),
    [sym_data_field] = ACTIONS(1492),
    [sym_data_record] = ACTIONS(1492),
    [sym_wc_data_feed] = ACTIONS(1492),
    [sym_wc_data_field] = ACTIONS(1492),
    [sym_wc_data_record] = ACTIONS(1492),
    [sym_last_login_info] = ACTIONS(1492),
    [sym____reserved___] = ACTIONS(1492),
    [sym_date] = ACTIONS(1492),
    [sym_day] = ACTIONS(1492),
    [sym_time] = ACTIONS(1492),
    [sym_ldate] = ACTIONS(1492),
    [sym_push_device] = ACTIONS(1492),
    [sym_push_channel] = ACTIONS(1492),
    [sym_push_devchan] = ACTIONS(1492),
    [sym_release_device] = ACTIONS(1492),
    [sym_release_channel] = ACTIONS(1492),
    [sym_release_devchan] = ACTIONS(1492),
    [sym_master_slot] = ACTIONS(1492),
    [sym_get_pulse_time] = ACTIONS(1492),
    [sym_get_timer] = ACTIONS(1492),
    [sym_master_sn] = ACTIONS(1492),
    [sym_system_number] = ACTIONS(1492),
    [sym_button] = ACTIONS(1492),
    [sym_dv_channel] = ACTIONS(1492),
    [sym_data] = ACTIONS(1492),
    [sym_level] = ACTIONS(1492),
    [sym_timeline] = ACTIONS(1492),
    [sym_channel] = ACTIONS(1492),
    [sym_custom] = ACTIONS(1492),
    [sym_atoi] = ACTIONS(1492),
    [sym_atol] = ACTIONS(1492),
    [sym_atof] = ACTIONS(1492),
    [sym_itoa] = ACTIONS(1492),
    [sym_itohex] = ACTIONS(1492),
    [sym_hextoi] = ACTIONS(1492),
    [sym_ftoa] = ACTIONS(1492),
    [sym_find_string] = ACTIONS(1492),
    [sym_left_string] = ACTIONS(1492),
    [sym_right_string] = ACTIONS(1492),
    [sym_mid_string] = ACTIONS(1492),
    [sym_remove_string] = ACTIONS(1492),
    [sym_get_buffer_string] = ACTIONS(1492),
    [sym_get_buffer_char] = ACTIONS(1492),
    [sym_get_multi_buffer_string] = ACTIONS(1492),
    [sym_length_string] = ACTIONS(1492),
    [sym_max_length_string] = ACTIONS(1492),
    [sym_set_length_string] = ACTIONS(1492),
    [sym_compare_string] = ACTIONS(1492),
    [sym_upper_string] = ACTIONS(1492),
    [sym_lower_string] = ACTIONS(1492),
    [sym_get_last] = ACTIONS(1492),
    [sym_date_to_day] = ACTIONS(1492),
    [sym_date_to_month] = ACTIONS(1492),
    [sym_date_to_year] = ACTIONS(1492),
    [sym_time_to_hour] = ACTIONS(1492),
    [sym_time_to_minute] = ACTIONS(1492),
    [sym_time_to_second] = ACTIONS(1492),
    [sym_day_of_week] = ACTIONS(1492),
    [sym_set_pulse_time] = ACTIONS(1492),
    [sym_set_timer] = ACTIONS(1492),
    [sym_file_open] = ACTIONS(1492),
    [sym_file_close] = ACTIONS(1492),
    [sym_file_read] = ACTIONS(1492),
    [sym_file_read_line] = ACTIONS(1492),
    [sym_file_write] = ACTIONS(1492),
    [sym_file_write_line] = ACTIONS(1492),
    [sym_file_copy] = ACTIONS(1492),
    [sym_file_delete] = ACTIONS(1492),
    [sym_file_rename] = ACTIONS(1492),
    [sym_file_dir] = ACTIONS(1492),
    [sym_file_getdir] = ACTIONS(1492),
    [sym_file_setdir] = ACTIONS(1492),
    [sym_file_create_dir] = ACTIONS(1492),
    [sym_file_remove_dir] = ACTIONS(1492),
    [sym_file_seek] = ACTIONS(1492),
    [sym_length_array] = ACTIONS(1492),
    [sym_max_length_array] = ACTIONS(1492),
    [sym_set_length_array] = ACTIONS(1492),
    [sym_ip_client_open] = ACTIONS(1492),
    [sym_ip_client_close] = ACTIONS(1492),
    [sym_ip_server_open] = ACTIONS(1492),
    [sym_ip_server_close] = ACTIONS(1492),
    [sym_ip_bound_client_open] = ACTIONS(1492),
    [sym_ip_mc_server_open] = ACTIONS(1492),
    [sym_ip_set_option] = ACTIONS(1492),
    [sym_ssh_client_open] = ACTIONS(1492),
    [sym_ssh_client_close] = ACTIONS(1492),
    [sym_tls_client_open] = ACTIONS(1492),
    [sym_tls_client_close] = ACTIONS(1492),
    [sym_do_push] = ACTIONS(1492),
    [sym_do_push_timed] = ACTIONS(1492),
    [sym_do_release] = ACTIONS(1492),
    [sym_device_id] = ACTIONS(1492),
    [sym_device_info] = ACTIONS(1492),
    [sym_device_id_string] = ACTIONS(1492),
    [sym_device_standby] = ACTIONS(1492),
    [sym_device_wake] = ACTIONS(1492),
    [sym_timeline_create] = ACTIONS(1492),
    [sym_timeline_kill] = ACTIONS(1492),
    [sym_timeline_reload] = ACTIONS(1492),
    [sym_timeline_pause] = ACTIONS(1492),
    [sym_timeline_restart] = ACTIONS(1492),
    [sym_timeline_set] = ACTIONS(1492),
    [sym_timeline_get] = ACTIONS(1492),
    [sym_timeline_active] = ACTIONS(1492),
    [sym_timeline_dynamic_id] = ACTIONS(1492),
    [sym_variable_to_string] = ACTIONS(1492),
    [sym_string_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_string] = ACTIONS(1492),
    [sym_type_cast] = ACTIONS(1492),
    [sym_variable_to_xml] = ACTIONS(1492),
    [sym_xml_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_xml] = ACTIONS(1492),
    [sym_redirect_string] = ACTIONS(1492),
    [sym_random_number] = ACTIONS(1492),
    [sym_get_unique_id] = ACTIONS(1492),
    [sym_get_system_number] = ACTIONS(1492),
    [sym_set_system_number] = ACTIONS(1492),
    [sym_get_serial_number] = ACTIONS(1492),
    [sym_get_master_build] = ACTIONS(1492),
    [sym_set_validation_code] = ACTIONS(1492),
    [sym_reboot] = ACTIONS(1492),
    [sym_load_duet_module] = ACTIONS(1492),
    [sym_unload_duet_module] = ACTIONS(1492),
    [sym_duet_mem_size_set] = ACTIONS(1492),
    [sym_duet_mem_size_get] = ACTIONS(1492),
    [sym_dynamic_application_device] = ACTIONS(1492),
    [sym_dynamic_polled_port] = ACTIONS(1492),
    [sym_static_port_binding] = ACTIONS(1492),
    [sym_static_ip_binding] = ACTIONS(1492),
    [sym_get_dns_list] = ACTIONS(1492),
    [sym_set_dns_list] = ACTIONS(1492),
    [sym_get_ip_address] = ACTIONS(1492),
    [sym_set_ip_address] = ACTIONS(1492),
    [sym_get_url_list] = ACTIONS(1492),
    [sym_add_url_entry] = ACTIONS(1492),
    [sym_delete_url_entry] = ACTIONS(1492),
    [sym_abs_value] = ACTIONS(1492),
    [sym_max_value] = ACTIONS(1492),
    [sym_min_value] = ACTIONS(1492),
    [sym_exp_value] = ACTIONS(1492),
    [sym_log_value] = ACTIONS(1492),
    [sym_log10_value] = ACTIONS(1492),
    [sym_power_value] = ACTIONS(1492),
    [sym_sqrt_value] = ACTIONS(1492),
    [sym_raw_be] = ACTIONS(1492),
    [sym_raw_le] = ACTIONS(1492),
    [sym_internal_queue_size_set] = ACTIONS(1492),
    [sym_internal_queue_size_get] = ACTIONS(1492),
    [sym_internal_threshold_set] = ACTIONS(1492),
    [sym_internal_threshold_get] = ACTIONS(1492),
    [sym_rebuilt_event] = ACTIONS(1492),
    [sym_do_custom_event] = ACTIONS(1492),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1492),
    [sym_clkmgr_set_clk_source] = ACTIONS(1492),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1492),
    [sym_clkmgr_get_timezone] = ACTIONS(1492),
    [sym_clkmgr_set_timezone] = ACTIONS(1492),
    [sym_clkmgr_get_resync_period] = ACTIONS(1492),
    [sym_clkmgr_set_resync_period] = ACTIONS(1492),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_timeservers] = ACTIONS(1492),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_set_log_level] = ACTIONS(1492),
    [sym_get_log_level] = ACTIONS(1492),
    [sym_amx_log] = ACTIONS(1492),
    [sym_smtp_server_config_set] = ACTIONS(1492),
    [sym_smtp_server_config_get] = ACTIONS(1492),
    [sym_smtp_send] = ACTIONS(1492),
    [sym_validate_netlinx_account] = ACTIONS(1492),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1492),
    [sym_authenticate_certificate] = ACTIONS(1492),
    [sym_audit_netlinx_session_event] = ACTIONS(1492),
    [sym_audit_netlinx_generic_event] = ACTIONS(1492),
    [sym_get_available_flash_disk_space] = ACTIONS(1492),
    [sym_get_max_flash_disk_space] = ACTIONS(1492),
    [sym_data_create_feed] = ACTIONS(1492),
    [sym_data_delete_feed] = ACTIONS(1492),
    [sym_data_publish_feed] = ACTIONS(1492),
    [sym_data_get_published_feed] = ACTIONS(1492),
    [sym_data_add_record] = ACTIONS(1492),
    [sym_data_get_event_record] = ACTIONS(1492),
    [sym__wc_data_create_feed] = ACTIONS(1492),
    [sym__wc_data_add_record] = ACTIONS(1492),
    [sym__wc_data_get_event_record] = ACTIONS(1492),
    [sym_format] = ACTIONS(1492),
    [sym_astro_clock] = ACTIONS(1492),
    [sym_set_outdoor_temperature] = ACTIONS(1492),
    [sym_set_virtual_level_count] = ACTIONS(1492),
    [sym_set_virtual_channel_count] = ACTIONS(1492),
    [sym_set_virtual_port_count] = ACTIONS(1492),
    [sym__wc_encode_int] = ACTIONS(1492),
    [sym__wc_decode_int] = ACTIONS(1492),
    [sym__wc_int] = ACTIONS(1492),
    [sym__wc_to_ch_int] = ACTIONS(1492),
    [sym__ch_to_wc_int] = ACTIONS(1492),
    [sym__wc_find_string_int] = ACTIONS(1492),
    [sym__wc_left_string_int] = ACTIONS(1492),
    [sym__wc_lower_string_int] = ACTIONS(1492),
    [sym__wc_mid_string_int] = ACTIONS(1492),
    [sym__wc_remove_string_int] = ACTIONS(1492),
    [sym__wc_right_string_int] = ACTIONS(1492),
    [sym__wc_upper_string_int] = ACTIONS(1492),
    [sym__wc_compare_string_int] = ACTIONS(1492),
    [sym__wc_get_buffer_char_int] = ACTIONS(1492),
    [sym__wc_get_buffer_string_int] = ACTIONS(1492),
    [sym__wc_concat_string_int] = ACTIONS(1492),
    [sym__wc_file_open_int] = ACTIONS(1492),
    [sym__wc_file_close_int] = ACTIONS(1492),
    [sym__wc_file_read_int] = ACTIONS(1492),
    [sym__wc_file_read_line_int] = ACTIONS(1492),
    [sym__wc_file_write_int] = ACTIONS(1492),
    [sym__wc_file_write_line_int] = ACTIONS(1492),
    [aux_sym_preproc_include_token1] = ACTIONS(1492),
    [aux_sym_preproc_define_token1] = ACTIONS(1494),
    [aux_sym_preproc_warn_token1] = ACTIONS(1494),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1494),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1494),
    [aux_sym_preproc_else_token1] = ACTIONS(1494),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1494),
    [anon_sym_COLON] = ACTIONS(1375),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1494),
    [anon_sym_BANG] = ACTIONS(1492),
    [anon_sym_TILDE] = ACTIONS(1494),
    [anon_sym_DASH] = ACTIONS(1381),
    [anon_sym_PLUS] = ACTIONS(1381),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1387),
    [anon_sym_AMP_AMP] = ACTIONS(1389),
    [anon_sym_PIPE] = ACTIONS(1367),
    [anon_sym_CARET] = ACTIONS(1391),
    [anon_sym_AMP] = ACTIONS(1365),
    [anon_sym_EQ_EQ] = ACTIONS(1393),
    [anon_sym_BANG_EQ] = ACTIONS(1393),
    [anon_sym_LT_GT] = ACTIONS(1393),
    [anon_sym_GT] = ACTIONS(1395),
    [anon_sym_GT_EQ] = ACTIONS(1397),
    [anon_sym_LT_EQ] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1395),
    [anon_sym_LT_LT] = ACTIONS(1399),
    [anon_sym_GT_GT] = ACTIONS(1399),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1494),
    [anon_sym_SQUOTE] = ACTIONS(1494),
    [sym_number_literal] = ACTIONS(1494),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1494),
  },
  [STATE(68)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1363),
    [sym_band] = ACTIONS(1365),
    [sym_bor] = ACTIONS(1367),
    [sym_bxor] = ACTIONS(1363),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1369),
    [sym_rshift] = ACTIONS(1369),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1381),
    [anon_sym_PLUS] = ACTIONS(1381),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1367),
    [anon_sym_CARET] = ACTIONS(1391),
    [anon_sym_AMP] = ACTIONS(1365),
    [anon_sym_EQ_EQ] = ACTIONS(1393),
    [anon_sym_BANG_EQ] = ACTIONS(1393),
    [anon_sym_LT_GT] = ACTIONS(1393),
    [anon_sym_GT] = ACTIONS(1395),
    [anon_sym_GT_EQ] = ACTIONS(1397),
    [anon_sym_LT_EQ] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1395),
    [anon_sym_LT_LT] = ACTIONS(1399),
    [anon_sym_GT_GT] = ACTIONS(1399),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(69)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1359),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1363),
    [sym_band] = ACTIONS(1365),
    [sym_bor] = ACTIONS(1367),
    [sym_bxor] = ACTIONS(1363),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1369),
    [sym_rshift] = ACTIONS(1369),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1381),
    [anon_sym_PLUS] = ACTIONS(1381),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1389),
    [anon_sym_PIPE] = ACTIONS(1367),
    [anon_sym_CARET] = ACTIONS(1391),
    [anon_sym_AMP] = ACTIONS(1365),
    [anon_sym_EQ_EQ] = ACTIONS(1393),
    [anon_sym_BANG_EQ] = ACTIONS(1393),
    [anon_sym_LT_GT] = ACTIONS(1393),
    [anon_sym_GT] = ACTIONS(1395),
    [anon_sym_GT_EQ] = ACTIONS(1397),
    [anon_sym_LT_EQ] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1395),
    [anon_sym_LT_LT] = ACTIONS(1399),
    [anon_sym_GT_GT] = ACTIONS(1399),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(70)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1365),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1369),
    [sym_rshift] = ACTIONS(1369),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1381),
    [anon_sym_PLUS] = ACTIONS(1381),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1365),
    [anon_sym_EQ_EQ] = ACTIONS(1393),
    [anon_sym_BANG_EQ] = ACTIONS(1393),
    [anon_sym_LT_GT] = ACTIONS(1393),
    [anon_sym_GT] = ACTIONS(1395),
    [anon_sym_GT_EQ] = ACTIONS(1397),
    [anon_sym_LT_EQ] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1395),
    [anon_sym_LT_LT] = ACTIONS(1399),
    [anon_sym_GT_GT] = ACTIONS(1399),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(71)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1363),
    [sym_band] = ACTIONS(1365),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1363),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1369),
    [sym_rshift] = ACTIONS(1369),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1381),
    [anon_sym_PLUS] = ACTIONS(1381),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1391),
    [anon_sym_AMP] = ACTIONS(1365),
    [anon_sym_EQ_EQ] = ACTIONS(1393),
    [anon_sym_BANG_EQ] = ACTIONS(1393),
    [anon_sym_LT_GT] = ACTIONS(1393),
    [anon_sym_GT] = ACTIONS(1395),
    [anon_sym_GT_EQ] = ACTIONS(1397),
    [anon_sym_LT_EQ] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1395),
    [anon_sym_LT_LT] = ACTIONS(1399),
    [anon_sym_GT_GT] = ACTIONS(1399),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(72)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1413),
    [sym_rshift] = ACTIONS(1413),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1381),
    [anon_sym_PLUS] = ACTIONS(1381),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1415),
    [anon_sym_GT_GT] = ACTIONS(1415),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(73)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1413),
    [sym_rshift] = ACTIONS(1413),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1415),
    [anon_sym_GT_GT] = ACTIONS(1415),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(74)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1413),
    [sym_rshift] = ACTIONS(1413),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_STAR] = ACTIONS(1415),
    [anon_sym_SLASH] = ACTIONS(1413),
    [anon_sym_PERCENT] = ACTIONS(1415),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1415),
    [anon_sym_GT_GT] = ACTIONS(1415),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(75)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1369),
    [sym_rshift] = ACTIONS(1369),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1381),
    [anon_sym_PLUS] = ACTIONS(1381),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1395),
    [anon_sym_GT_EQ] = ACTIONS(1397),
    [anon_sym_LT_EQ] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1395),
    [anon_sym_LT_LT] = ACTIONS(1399),
    [anon_sym_GT_GT] = ACTIONS(1399),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(76)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1369),
    [sym_rshift] = ACTIONS(1369),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1381),
    [anon_sym_PLUS] = ACTIONS(1381),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1399),
    [anon_sym_GT_GT] = ACTIONS(1399),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(77)] = {
    [aux_sym_subscript_expression_repeat1] = STATE(77),
    [sym_identifier] = ACTIONS(1496),
    [sym_program_name_keyword] = ACTIONS(1496),
    [sym_module_name_keyword] = ACTIONS(1496),
    [sym_define_device_keyword] = ACTIONS(1496),
    [sym_define_combine_keyword] = ACTIONS(1496),
    [sym_define_constant_keyword] = ACTIONS(1496),
    [sym_define_type_keyword] = ACTIONS(1496),
    [sym_define_variable_keyword] = ACTIONS(1496),
    [sym_define_system_variable_keyword] = ACTIONS(1496),
    [sym_define_connect_level_keyword] = ACTIONS(1496),
    [sym_define_latching_keyword] = ACTIONS(1496),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1496),
    [sym_define_toggling_keyword] = ACTIONS(1496),
    [sym_define_call_keyword] = ACTIONS(1496),
    [sym_define_function_keyword] = ACTIONS(1496),
    [sym_define_library_function_keyword] = ACTIONS(1496),
    [sym_define_start_keyword] = ACTIONS(1496),
    [sym_define_event_keyword] = ACTIONS(1496),
    [sym_define_program_keyword] = ACTIONS(1496),
    [sym_define_module_keyword] = ACTIONS(1496),
    [sym_char_keyword] = ACTIONS(1496),
    [sym_widechar_keyword] = ACTIONS(1496),
    [sym_integer_keyword] = ACTIONS(1496),
    [sym_sinteger_keyword] = ACTIONS(1496),
    [sym_long_keyword] = ACTIONS(1496),
    [sym_slong_keyword] = ACTIONS(1496),
    [sym_float_keyword] = ACTIONS(1496),
    [sym_double_keyword] = ACTIONS(1496),
    [sym_variant_keyword] = ACTIONS(1496),
    [sym_variantarray_keyword] = ACTIONS(1496),
    [sym_local_var_keyword] = ACTIONS(1496),
    [sym_stack_var_keyword] = ACTIONS(1496),
    [sym_constant_keyword] = ACTIONS(1496),
    [sym_volatile_keyword] = ACTIONS(1496),
    [sym_non_volatile_keyword] = ACTIONS(1496),
    [sym_persistent_keyword] = ACTIONS(1496),
    [sym_dev_keyword] = ACTIONS(1496),
    [sym_devlev_keyword] = ACTIONS(1496),
    [sym_devchan_keyword] = ACTIONS(1496),
    [sym_if_keyword] = ACTIONS(1496),
    [sym_else_keyword] = ACTIONS(1496),
    [sym_switch_keyword] = ACTIONS(1496),
    [sym_case_keyword] = ACTIONS(1496),
    [sym_default_keyword] = ACTIONS(1496),
    [sym_while_keyword] = ACTIONS(1496),
    [sym_for_keyword] = ACTIONS(1496),
    [sym_break_keyword] = ACTIONS(1496),
    [sym_continue_keyword] = ACTIONS(1496),
    [sym_return_keyword] = ACTIONS(1496),
    [sym_select_keyword] = ACTIONS(1496),
    [sym_active_keyword] = ACTIONS(1496),
    [sym_struct_keyword] = ACTIONS(1496),
    [sym_structure_keyword] = ACTIONS(1496),
    [sym_button_event_keyword] = ACTIONS(1496),
    [sym_channel_event_keyword] = ACTIONS(1496),
    [sym_level_event_keyword] = ACTIONS(1496),
    [sym_data_event_keyword] = ACTIONS(1496),
    [sym_timeline_event_keyword] = ACTIONS(1496),
    [sym_custom_event_keyword] = ACTIONS(1496),
    [sym_push_keyword] = ACTIONS(1496),
    [sym_release_keyword] = ACTIONS(1496),
    [aux_sym_on_keyword_token1] = ACTIONS(1496),
    [aux_sym_off_keyword_token1] = ACTIONS(1496),
    [sym_send_level_keyword] = ACTIONS(1496),
    [sym_send_string_keyword] = ACTIONS(1496),
    [sym_send_command_keyword] = ACTIONS(1496),
    [sym_clear_buffer_keyword] = ACTIONS(1496),
    [sym_create_buffer_keyword] = ACTIONS(1496),
    [sym_create_multi_buffer_keyword] = ACTIONS(1496),
    [sym_call_keyword] = ACTIONS(1496),
    [sym_system_call_keyword] = ACTIONS(1496),
    [sym_devchan_to_keyword] = ACTIONS(1496),
    [sym_devchan_min_to_keyword] = ACTIONS(1496),
    [sym_devchan_total_off_keyword] = ACTIONS(1496),
    [sym_devchan_pulse_keyword] = ACTIONS(1496),
    [sym_wait_keyword] = ACTIONS(1496),
    [sym_cancel_wait_keyword] = ACTIONS(1496),
    [sym_wait_until_keyword] = ACTIONS(1496),
    [sym_cancel_wait_until_keyword] = ACTIONS(1496),
    [sym_cancel_all_wait_keyword] = ACTIONS(1496),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1496),
    [sym_and] = ACTIONS(1496),
    [sym_or] = ACTIONS(1496),
    [sym_not] = ACTIONS(1496),
    [sym_xor] = ACTIONS(1496),
    [sym_band] = ACTIONS(1496),
    [sym_bor] = ACTIONS(1496),
    [sym_bxor] = ACTIONS(1496),
    [sym_bnot] = ACTIONS(1496),
    [sym_lshift] = ACTIONS(1496),
    [sym_rshift] = ACTIONS(1496),
    [sym___file__] = ACTIONS(1496),
    [sym___date__] = ACTIONS(1496),
    [sym___ldate__] = ACTIONS(1496),
    [sym___time__] = ACTIONS(1496),
    [sym___line__] = ACTIONS(1496),
    [sym___name__] = ACTIONS(1496),
    [sym___netlinx__] = ACTIONS(1496),
    [sym_netlinx_axi_version] = ACTIONS(1496),
    [sym_url_flg_tcp] = ACTIONS(1496),
    [sym_url_flg_accinfopresent] = ACTIONS(1496),
    [sym_url_flg_temp] = ACTIONS(1496),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1496),
    [sym_url_flg_stat_mask] = ACTIONS(1496),
    [sym_url_flg_stat_lookup] = ACTIONS(1496),
    [sym_url_flg_stat_connecting] = ACTIONS(1496),
    [sym_url_flg_stat_waiting] = ACTIONS(1496),
    [sym_url_flg_stat_connected] = ACTIONS(1496),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1496),
    [sym_first_virtual_device] = ACTIONS(1496),
    [sym_first_local_port] = ACTIONS(1496),
    [sym_dynamic_virtual_device] = ACTIONS(1496),
    [sym_timeline_once] = ACTIONS(1496),
    [sym_timeline_repeat] = ACTIONS(1496),
    [sym_timeline_absolute] = ACTIONS(1496),
    [sym_timeline_relative] = ACTIONS(1496),
    [sym_do_push_timed_infinite] = ACTIONS(1496),
    [sym_source_type_no_address] = ACTIONS(1496),
    [sym_source_type_neuron_id] = ACTIONS(1496),
    [sym_source_type_ip_address] = ACTIONS(1496),
    [sym_source_type_axlink] = ACTIONS(1496),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1496),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1496),
    [sym_source_type_ip_socket_address] = ACTIONS(1496),
    [sym_source_type_rs232] = ACTIONS(1496),
    [sym_source_type_internal] = ACTIONS(1496),
    [sym_source_type_mac_address] = ACTIONS(1496),
    [sym_source_type_ipv4_port] = ACTIONS(1496),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1496),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1496),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1496),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1496),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1496),
    [sym_true] = ACTIONS(1496),
    [sym_false] = ACTIONS(1496),
    [sym_file_read_only] = ACTIONS(1496),
    [sym_file_rw_new] = ACTIONS(1496),
    [sym_file_rw_append] = ACTIONS(1496),
    [sym_ip_tcp] = ACTIONS(1496),
    [sym_ip_udp] = ACTIONS(1496),
    [sym_ip_udp_2way] = ACTIONS(1496),
    [sym_xml_encode_types] = ACTIONS(1496),
    [sym_xml_encode_char_as_list] = ACTIONS(1496),
    [sym_xml_encode_le] = ACTIONS(1496),
    [sym_xml_decode_types] = ACTIONS(1496),
    [sym_xml_decode_no_preserve] = ACTIONS(1496),
    [sym_duet_dev_type_display_device] = ACTIONS(1496),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1496),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1496),
    [sym_duet_dev_type_io_device] = ACTIONS(1496),
    [sym_duet_dev_type_relay_device] = ACTIONS(1496),
    [sym_duet_dev_type_ups] = ACTIONS(1496),
    [sym_duet_dev_type_amplifier] = ACTIONS(1496),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1496),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1496),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1496),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1496),
    [sym_duet_dev_type_camera] = ACTIONS(1496),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1496),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1496),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1496),
    [sym_duet_dev_type_dss] = ACTIONS(1496),
    [sym_duet_dev_type_dvr] = ACTIONS(1496),
    [sym_duet_dev_type_disc_device] = ACTIONS(1496),
    [sym_duet_dev_type_document_camera] = ACTIONS(1496),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1496),
    [sym_duet_dev_type_hvac] = ACTIONS(1496),
    [sym_duet_dev_type_keypad] = ACTIONS(1496),
    [sym_duet_dev_type_light] = ACTIONS(1496),
    [sym_duet_dev_type_monitor] = ACTIONS(1496),
    [sym_duet_dev_type_motor] = ACTIONS(1496),
    [sym_duet_dev_type_multi_window] = ACTIONS(1496),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1496),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1496),
    [sym_duet_dev_type_receiver] = ACTIONS(1496),
    [sym_duet_dev_type_security_system] = ACTIONS(1496),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1496),
    [sym_duet_dev_type_settop_box] = ACTIONS(1496),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1496),
    [sym_duet_dev_type_switcher] = ACTIONS(1496),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1496),
    [sym_duet_dev_type_tv] = ACTIONS(1496),
    [sym_duet_dev_type_utility] = ACTIONS(1496),
    [sym_duet_dev_type_vcr] = ACTIONS(1496),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1496),
    [sym_duet_dev_type_video_processor] = ACTIONS(1496),
    [sym_duet_dev_type_video_projector] = ACTIONS(1496),
    [sym_duet_dev_type_video_wall] = ACTIONS(1496),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1496),
    [sym_duet_dev_type_weather] = ACTIONS(1496),
    [sym_duet_dev_not_polled] = ACTIONS(1496),
    [sym_duet_dev_polled] = ACTIONS(1496),
    [sym_clkmgr_mode_network] = ACTIONS(1496),
    [sym_clkmgr_mode_standalone] = ACTIONS(1496),
    [sym_normal_standby] = ACTIONS(1496),
    [sym_normal_wake] = ACTIONS(1496),
    [sym_ip_multicast_ttl_option] = ACTIONS(1496),
    [sym_ip_ttl_subnet] = ACTIONS(1496),
    [sym_ip_ttl_site] = ACTIONS(1496),
    [sym_ip_ttl_region] = ACTIONS(1496),
    [sym_ip_ttl_continent] = ACTIONS(1496),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1496),
    [sym_ip_nodelay_on] = ACTIONS(1496),
    [sym_ip_nodelay_off] = ACTIONS(1496),
    [sym_amx_error] = ACTIONS(1496),
    [sym_amx_warning] = ACTIONS(1496),
    [sym_amx_info] = ACTIONS(1496),
    [sym_amx_debug] = ACTIONS(1496),
    [sym_netlinx_login_success] = ACTIONS(1496),
    [sym_netlinx_login_fail] = ACTIONS(1496),
    [sym_netlinx_logout] = ACTIONS(1496),
    [sym_null_str] = ACTIONS(1496),
    [sym_smtp_address] = ACTIONS(1496),
    [sym_smtp_port_number] = ACTIONS(1496),
    [sym_smtp_username] = ACTIONS(1496),
    [sym_smtp_password] = ACTIONS(1496),
    [sym_smtp_require_tls] = ACTIONS(1496),
    [sym_smtp_from] = ACTIONS(1496),
    [sym_smtp_tls_true] = ACTIONS(1496),
    [sym_smtp_tls_false] = ACTIONS(1496),
    [sym_listview_on_row_select_event] = ACTIONS(1496),
    [sym_data_structure_unknown] = ACTIONS(1496),
    [sym_data_structure_datafeed] = ACTIONS(1496),
    [sym_data_structure_datarecordset] = ACTIONS(1496),
    [sym_data_structure_datarecord] = ACTIONS(1496),
    [sym_data_structure_datafield] = ACTIONS(1496),
    [sym_data_max_content_field_count] = ACTIONS(1496),
    [sym_data_max_metadata_field_count] = ACTIONS(1496),
    [sym_data_max_id_length] = ACTIONS(1496),
    [sym_data_max_name_length] = ACTIONS(1496),
    [sym_data_max_description_length] = ACTIONS(1496),
    [sym_data_max_type_length] = ACTIONS(1496),
    [sym_data_max_format_length] = ACTIONS(1496),
    [sym_data_max_source_length] = ACTIONS(1496),
    [sym_data_max_value_length] = ACTIONS(1496),
    [sym_data_max_label_length] = ACTIONS(1496),
    [sym_data_max_url_length] = ACTIONS(1496),
    [sym_max_last_login_info_length] = ACTIONS(1496),
    [sym_data_type_unknown] = ACTIONS(1496),
    [sym_data_type_string] = ACTIONS(1496),
    [sym_data_type_datetime] = ACTIONS(1496),
    [sym_data_type_date] = ACTIONS(1496),
    [sym_data_type_time] = ACTIONS(1496),
    [sym_data_type_image] = ACTIONS(1496),
    [sym_data_format_url] = ACTIONS(1496),
    [sym_data_format_phone] = ACTIONS(1496),
    [sym_data_format_email] = ACTIONS(1496),
    [sym_data_format_iso8601] = ACTIONS(1496),
    [sym_auditlog_priv] = ACTIONS(1496),
    [sym_configuration_priv] = ACTIONS(1496),
    [sym_deviceconfig_priv] = ACTIONS(1496),
    [sym_ftp_priv] = ACTIONS(1496),
    [sym_http_priv] = ACTIONS(1496),
    [sym_networkconfig_priv] = ACTIONS(1496),
    [sym_programport_priv] = ACTIONS(1496),
    [sym_remoteui_priv] = ACTIONS(1496),
    [sym_securitycontrol_priv] = ACTIONS(1496),
    [sym_softwaremanagement_priv] = ACTIONS(1496),
    [sym_terminal_priv] = ACTIONS(1496),
    [sym_tpadmin_priv] = ACTIONS(1496),
    [sym_user1_priv] = ACTIONS(1496),
    [sym_user2_priv] = ACTIONS(1496),
    [sym_user3_priv] = ACTIONS(1496),
    [sym_user4_priv] = ACTIONS(1496),
    [sym_usermanagement_priv] = ACTIONS(1496),
    [sym_valid_account] = ACTIONS(1496),
    [sym_err_name_invalid] = ACTIONS(1496),
    [sym_err_password_invalid] = ACTIONS(1496),
    [sym_err_invalid_account] = ACTIONS(1496),
    [sym_err_no_authorization] = ACTIONS(1496),
    [sym_err_invalid_parameter] = ACTIONS(1496),
    [sym_err_user_locked_out] = ACTIONS(1496),
    [sym_err_user_account_expired] = ACTIONS(1496),
    [sym_err_general_error] = ACTIONS(1496),
    [sym_err_authentication_error] = ACTIONS(1496),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1496),
    [sym_tls_validate_certificate] = ACTIONS(1496),
    [sym_tdata] = ACTIONS(1496),
    [sym_tchannel] = ACTIONS(1496),
    [sym_tlevel] = ACTIONS(1496),
    [sym_tbutton] = ACTIONS(1496),
    [sym_ttimeline] = ACTIONS(1496),
    [sym_tcustom] = ACTIONS(1496),
    [sym_url_struct] = ACTIONS(1496),
    [sym_dns_struct] = ACTIONS(1496),
    [sym_ip_address_struct] = ACTIONS(1496),
    [sym_dev_info_struct] = ACTIONS(1496),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1496),
    [sym_data_feed] = ACTIONS(1496),
    [sym_data_field] = ACTIONS(1496),
    [sym_data_record] = ACTIONS(1496),
    [sym_wc_data_feed] = ACTIONS(1496),
    [sym_wc_data_field] = ACTIONS(1496),
    [sym_wc_data_record] = ACTIONS(1496),
    [sym_last_login_info] = ACTIONS(1496),
    [sym____reserved___] = ACTIONS(1496),
    [sym_date] = ACTIONS(1496),
    [sym_day] = ACTIONS(1496),
    [sym_time] = ACTIONS(1496),
    [sym_ldate] = ACTIONS(1496),
    [sym_push_device] = ACTIONS(1496),
    [sym_push_channel] = ACTIONS(1496),
    [sym_push_devchan] = ACTIONS(1496),
    [sym_release_device] = ACTIONS(1496),
    [sym_release_channel] = ACTIONS(1496),
    [sym_release_devchan] = ACTIONS(1496),
    [sym_master_slot] = ACTIONS(1496),
    [sym_get_pulse_time] = ACTIONS(1496),
    [sym_get_timer] = ACTIONS(1496),
    [sym_master_sn] = ACTIONS(1496),
    [sym_system_number] = ACTIONS(1496),
    [sym_button] = ACTIONS(1496),
    [sym_dv_channel] = ACTIONS(1496),
    [sym_data] = ACTIONS(1496),
    [sym_level] = ACTIONS(1496),
    [sym_timeline] = ACTIONS(1496),
    [sym_channel] = ACTIONS(1496),
    [sym_custom] = ACTIONS(1496),
    [sym_atoi] = ACTIONS(1496),
    [sym_atol] = ACTIONS(1496),
    [sym_atof] = ACTIONS(1496),
    [sym_itoa] = ACTIONS(1496),
    [sym_itohex] = ACTIONS(1496),
    [sym_hextoi] = ACTIONS(1496),
    [sym_ftoa] = ACTIONS(1496),
    [sym_find_string] = ACTIONS(1496),
    [sym_left_string] = ACTIONS(1496),
    [sym_right_string] = ACTIONS(1496),
    [sym_mid_string] = ACTIONS(1496),
    [sym_remove_string] = ACTIONS(1496),
    [sym_get_buffer_string] = ACTIONS(1496),
    [sym_get_buffer_char] = ACTIONS(1496),
    [sym_get_multi_buffer_string] = ACTIONS(1496),
    [sym_length_string] = ACTIONS(1496),
    [sym_max_length_string] = ACTIONS(1496),
    [sym_set_length_string] = ACTIONS(1496),
    [sym_compare_string] = ACTIONS(1496),
    [sym_upper_string] = ACTIONS(1496),
    [sym_lower_string] = ACTIONS(1496),
    [sym_get_last] = ACTIONS(1496),
    [sym_date_to_day] = ACTIONS(1496),
    [sym_date_to_month] = ACTIONS(1496),
    [sym_date_to_year] = ACTIONS(1496),
    [sym_time_to_hour] = ACTIONS(1496),
    [sym_time_to_minute] = ACTIONS(1496),
    [sym_time_to_second] = ACTIONS(1496),
    [sym_day_of_week] = ACTIONS(1496),
    [sym_set_pulse_time] = ACTIONS(1496),
    [sym_set_timer] = ACTIONS(1496),
    [sym_file_open] = ACTIONS(1496),
    [sym_file_close] = ACTIONS(1496),
    [sym_file_read] = ACTIONS(1496),
    [sym_file_read_line] = ACTIONS(1496),
    [sym_file_write] = ACTIONS(1496),
    [sym_file_write_line] = ACTIONS(1496),
    [sym_file_copy] = ACTIONS(1496),
    [sym_file_delete] = ACTIONS(1496),
    [sym_file_rename] = ACTIONS(1496),
    [sym_file_dir] = ACTIONS(1496),
    [sym_file_getdir] = ACTIONS(1496),
    [sym_file_setdir] = ACTIONS(1496),
    [sym_file_create_dir] = ACTIONS(1496),
    [sym_file_remove_dir] = ACTIONS(1496),
    [sym_file_seek] = ACTIONS(1496),
    [sym_length_array] = ACTIONS(1496),
    [sym_max_length_array] = ACTIONS(1496),
    [sym_set_length_array] = ACTIONS(1496),
    [sym_ip_client_open] = ACTIONS(1496),
    [sym_ip_client_close] = ACTIONS(1496),
    [sym_ip_server_open] = ACTIONS(1496),
    [sym_ip_server_close] = ACTIONS(1496),
    [sym_ip_bound_client_open] = ACTIONS(1496),
    [sym_ip_mc_server_open] = ACTIONS(1496),
    [sym_ip_set_option] = ACTIONS(1496),
    [sym_ssh_client_open] = ACTIONS(1496),
    [sym_ssh_client_close] = ACTIONS(1496),
    [sym_tls_client_open] = ACTIONS(1496),
    [sym_tls_client_close] = ACTIONS(1496),
    [sym_do_push] = ACTIONS(1496),
    [sym_do_push_timed] = ACTIONS(1496),
    [sym_do_release] = ACTIONS(1496),
    [sym_device_id] = ACTIONS(1496),
    [sym_device_info] = ACTIONS(1496),
    [sym_device_id_string] = ACTIONS(1496),
    [sym_device_standby] = ACTIONS(1496),
    [sym_device_wake] = ACTIONS(1496),
    [sym_timeline_create] = ACTIONS(1496),
    [sym_timeline_kill] = ACTIONS(1496),
    [sym_timeline_reload] = ACTIONS(1496),
    [sym_timeline_pause] = ACTIONS(1496),
    [sym_timeline_restart] = ACTIONS(1496),
    [sym_timeline_set] = ACTIONS(1496),
    [sym_timeline_get] = ACTIONS(1496),
    [sym_timeline_active] = ACTIONS(1496),
    [sym_timeline_dynamic_id] = ACTIONS(1496),
    [sym_variable_to_string] = ACTIONS(1496),
    [sym_string_to_variable] = ACTIONS(1496),
    [sym_length_variable_to_string] = ACTIONS(1496),
    [sym_type_cast] = ACTIONS(1496),
    [sym_variable_to_xml] = ACTIONS(1496),
    [sym_xml_to_variable] = ACTIONS(1496),
    [sym_length_variable_to_xml] = ACTIONS(1496),
    [sym_redirect_string] = ACTIONS(1496),
    [sym_random_number] = ACTIONS(1496),
    [sym_get_unique_id] = ACTIONS(1496),
    [sym_get_system_number] = ACTIONS(1496),
    [sym_set_system_number] = ACTIONS(1496),
    [sym_get_serial_number] = ACTIONS(1496),
    [sym_get_master_build] = ACTIONS(1496),
    [sym_set_validation_code] = ACTIONS(1496),
    [sym_reboot] = ACTIONS(1496),
    [sym_load_duet_module] = ACTIONS(1496),
    [sym_unload_duet_module] = ACTIONS(1496),
    [sym_duet_mem_size_set] = ACTIONS(1496),
    [sym_duet_mem_size_get] = ACTIONS(1496),
    [sym_dynamic_application_device] = ACTIONS(1496),
    [sym_dynamic_polled_port] = ACTIONS(1496),
    [sym_static_port_binding] = ACTIONS(1496),
    [sym_static_ip_binding] = ACTIONS(1496),
    [sym_get_dns_list] = ACTIONS(1496),
    [sym_set_dns_list] = ACTIONS(1496),
    [sym_get_ip_address] = ACTIONS(1496),
    [sym_set_ip_address] = ACTIONS(1496),
    [sym_get_url_list] = ACTIONS(1496),
    [sym_add_url_entry] = ACTIONS(1496),
    [sym_delete_url_entry] = ACTIONS(1496),
    [sym_abs_value] = ACTIONS(1496),
    [sym_max_value] = ACTIONS(1496),
    [sym_min_value] = ACTIONS(1496),
    [sym_exp_value] = ACTIONS(1496),
    [sym_log_value] = ACTIONS(1496),
    [sym_log10_value] = ACTIONS(1496),
    [sym_power_value] = ACTIONS(1496),
    [sym_sqrt_value] = ACTIONS(1496),
    [sym_raw_be] = ACTIONS(1496),
    [sym_raw_le] = ACTIONS(1496),
    [sym_internal_queue_size_set] = ACTIONS(1496),
    [sym_internal_queue_size_get] = ACTIONS(1496),
    [sym_internal_threshold_set] = ACTIONS(1496),
    [sym_internal_threshold_get] = ACTIONS(1496),
    [sym_rebuilt_event] = ACTIONS(1496),
    [sym_do_custom_event] = ACTIONS(1496),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1496),
    [sym_clkmgr_set_clk_source] = ACTIONS(1496),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1496),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1496),
    [sym_clkmgr_get_timezone] = ACTIONS(1496),
    [sym_clkmgr_set_timezone] = ACTIONS(1496),
    [sym_clkmgr_get_resync_period] = ACTIONS(1496),
    [sym_clkmgr_set_resync_period] = ACTIONS(1496),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1496),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1496),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1496),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1496),
    [sym_clkmgr_get_timeservers] = ACTIONS(1496),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1496),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1496),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1496),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1496),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1496),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1496),
    [sym_set_log_level] = ACTIONS(1496),
    [sym_get_log_level] = ACTIONS(1496),
    [sym_amx_log] = ACTIONS(1496),
    [sym_smtp_server_config_set] = ACTIONS(1496),
    [sym_smtp_server_config_get] = ACTIONS(1496),
    [sym_smtp_send] = ACTIONS(1496),
    [sym_validate_netlinx_account] = ACTIONS(1496),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1496),
    [sym_authenticate_certificate] = ACTIONS(1496),
    [sym_audit_netlinx_session_event] = ACTIONS(1496),
    [sym_audit_netlinx_generic_event] = ACTIONS(1496),
    [sym_get_available_flash_disk_space] = ACTIONS(1496),
    [sym_get_max_flash_disk_space] = ACTIONS(1496),
    [sym_data_create_feed] = ACTIONS(1496),
    [sym_data_delete_feed] = ACTIONS(1496),
    [sym_data_publish_feed] = ACTIONS(1496),
    [sym_data_get_published_feed] = ACTIONS(1496),
    [sym_data_add_record] = ACTIONS(1496),
    [sym_data_get_event_record] = ACTIONS(1496),
    [sym__wc_data_create_feed] = ACTIONS(1496),
    [sym__wc_data_add_record] = ACTIONS(1496),
    [sym__wc_data_get_event_record] = ACTIONS(1496),
    [sym_format] = ACTIONS(1496),
    [sym_astro_clock] = ACTIONS(1496),
    [sym_set_outdoor_temperature] = ACTIONS(1496),
    [sym_set_virtual_level_count] = ACTIONS(1496),
    [sym_set_virtual_channel_count] = ACTIONS(1496),
    [sym_set_virtual_port_count] = ACTIONS(1496),
    [sym__wc_encode_int] = ACTIONS(1496),
    [sym__wc_decode_int] = ACTIONS(1496),
    [sym__wc_int] = ACTIONS(1496),
    [sym__wc_to_ch_int] = ACTIONS(1496),
    [sym__ch_to_wc_int] = ACTIONS(1496),
    [sym__wc_find_string_int] = ACTIONS(1496),
    [sym__wc_left_string_int] = ACTIONS(1496),
    [sym__wc_lower_string_int] = ACTIONS(1496),
    [sym__wc_mid_string_int] = ACTIONS(1496),
    [sym__wc_remove_string_int] = ACTIONS(1496),
    [sym__wc_right_string_int] = ACTIONS(1496),
    [sym__wc_upper_string_int] = ACTIONS(1496),
    [sym__wc_compare_string_int] = ACTIONS(1496),
    [sym__wc_get_buffer_char_int] = ACTIONS(1496),
    [sym__wc_get_buffer_string_int] = ACTIONS(1496),
    [sym__wc_concat_string_int] = ACTIONS(1496),
    [sym__wc_file_open_int] = ACTIONS(1496),
    [sym__wc_file_close_int] = ACTIONS(1496),
    [sym__wc_file_read_int] = ACTIONS(1496),
    [sym__wc_file_read_line_int] = ACTIONS(1496),
    [sym__wc_file_write_int] = ACTIONS(1496),
    [sym__wc_file_write_line_int] = ACTIONS(1496),
    [anon_sym_EQ] = ACTIONS(1496),
    [aux_sym_preproc_include_token1] = ACTIONS(1496),
    [aux_sym_preproc_define_token1] = ACTIONS(1498),
    [aux_sym_preproc_warn_token1] = ACTIONS(1498),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1498),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1498),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1498),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1498),
    [aux_sym_preproc_else_token1] = ACTIONS(1498),
    [anon_sym_LBRACK] = ACTIONS(1500),
    [anon_sym_LBRACE] = ACTIONS(1498),
    [anon_sym_COLON] = ACTIONS(1498),
    [anon_sym_LPAREN] = ACTIONS(1496),
    [anon_sym_SEMI] = ACTIONS(1498),
    [anon_sym_BANG] = ACTIONS(1496),
    [anon_sym_TILDE] = ACTIONS(1498),
    [anon_sym_DASH] = ACTIONS(1496),
    [anon_sym_PLUS] = ACTIONS(1496),
    [anon_sym_STAR] = ACTIONS(1498),
    [anon_sym_SLASH] = ACTIONS(1496),
    [anon_sym_PERCENT] = ACTIONS(1498),
    [anon_sym_PIPE_PIPE] = ACTIONS(1498),
    [anon_sym_AMP_AMP] = ACTIONS(1498),
    [anon_sym_PIPE] = ACTIONS(1496),
    [anon_sym_CARET] = ACTIONS(1498),
    [anon_sym_AMP] = ACTIONS(1496),
    [anon_sym_EQ_EQ] = ACTIONS(1498),
    [anon_sym_BANG_EQ] = ACTIONS(1498),
    [anon_sym_LT_GT] = ACTIONS(1498),
    [anon_sym_GT] = ACTIONS(1496),
    [anon_sym_GT_EQ] = ACTIONS(1498),
    [anon_sym_LT_EQ] = ACTIONS(1498),
    [anon_sym_LT] = ACTIONS(1496),
    [anon_sym_LT_LT] = ACTIONS(1498),
    [anon_sym_GT_GT] = ACTIONS(1498),
    [anon_sym_DASH_DASH] = ACTIONS(1498),
    [anon_sym_PLUS_PLUS] = ACTIONS(1498),
    [anon_sym_DOT] = ACTIONS(1496),
    [anon_sym_DQUOTE] = ACTIONS(1498),
    [anon_sym_SQUOTE] = ACTIONS(1498),
    [sym_number_literal] = ACTIONS(1498),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1498),
  },
  [STATE(78)] = {
    [sym_argument_list] = STATE(80),
    [aux_sym_subscript_expression_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1503),
    [sym_program_name_keyword] = ACTIONS(1503),
    [sym_module_name_keyword] = ACTIONS(1503),
    [sym_define_device_keyword] = ACTIONS(1503),
    [sym_define_combine_keyword] = ACTIONS(1503),
    [sym_define_constant_keyword] = ACTIONS(1503),
    [sym_define_type_keyword] = ACTIONS(1503),
    [sym_define_variable_keyword] = ACTIONS(1503),
    [sym_define_system_variable_keyword] = ACTIONS(1503),
    [sym_define_connect_level_keyword] = ACTIONS(1503),
    [sym_define_latching_keyword] = ACTIONS(1503),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1503),
    [sym_define_toggling_keyword] = ACTIONS(1503),
    [sym_define_call_keyword] = ACTIONS(1503),
    [sym_define_function_keyword] = ACTIONS(1503),
    [sym_define_library_function_keyword] = ACTIONS(1503),
    [sym_define_start_keyword] = ACTIONS(1503),
    [sym_define_event_keyword] = ACTIONS(1503),
    [sym_define_program_keyword] = ACTIONS(1503),
    [sym_define_module_keyword] = ACTIONS(1503),
    [sym_char_keyword] = ACTIONS(1503),
    [sym_widechar_keyword] = ACTIONS(1503),
    [sym_integer_keyword] = ACTIONS(1503),
    [sym_sinteger_keyword] = ACTIONS(1503),
    [sym_long_keyword] = ACTIONS(1503),
    [sym_slong_keyword] = ACTIONS(1503),
    [sym_float_keyword] = ACTIONS(1503),
    [sym_double_keyword] = ACTIONS(1503),
    [sym_variant_keyword] = ACTIONS(1503),
    [sym_variantarray_keyword] = ACTIONS(1503),
    [sym_local_var_keyword] = ACTIONS(1503),
    [sym_stack_var_keyword] = ACTIONS(1503),
    [sym_constant_keyword] = ACTIONS(1503),
    [sym_volatile_keyword] = ACTIONS(1503),
    [sym_non_volatile_keyword] = ACTIONS(1503),
    [sym_persistent_keyword] = ACTIONS(1503),
    [sym_dev_keyword] = ACTIONS(1503),
    [sym_devlev_keyword] = ACTIONS(1503),
    [sym_devchan_keyword] = ACTIONS(1503),
    [sym_if_keyword] = ACTIONS(1503),
    [sym_else_keyword] = ACTIONS(1503),
    [sym_switch_keyword] = ACTIONS(1503),
    [sym_case_keyword] = ACTIONS(1503),
    [sym_default_keyword] = ACTIONS(1503),
    [sym_while_keyword] = ACTIONS(1503),
    [sym_for_keyword] = ACTIONS(1503),
    [sym_break_keyword] = ACTIONS(1503),
    [sym_continue_keyword] = ACTIONS(1503),
    [sym_return_keyword] = ACTIONS(1503),
    [sym_select_keyword] = ACTIONS(1503),
    [sym_active_keyword] = ACTIONS(1503),
    [sym_struct_keyword] = ACTIONS(1503),
    [sym_structure_keyword] = ACTIONS(1503),
    [sym_button_event_keyword] = ACTIONS(1503),
    [sym_channel_event_keyword] = ACTIONS(1503),
    [sym_level_event_keyword] = ACTIONS(1503),
    [sym_data_event_keyword] = ACTIONS(1503),
    [sym_timeline_event_keyword] = ACTIONS(1503),
    [sym_custom_event_keyword] = ACTIONS(1503),
    [sym_push_keyword] = ACTIONS(1503),
    [sym_release_keyword] = ACTIONS(1503),
    [aux_sym_on_keyword_token1] = ACTIONS(1503),
    [aux_sym_off_keyword_token1] = ACTIONS(1503),
    [sym_send_level_keyword] = ACTIONS(1503),
    [sym_send_string_keyword] = ACTIONS(1503),
    [sym_send_command_keyword] = ACTIONS(1503),
    [sym_clear_buffer_keyword] = ACTIONS(1503),
    [sym_create_buffer_keyword] = ACTIONS(1503),
    [sym_create_multi_buffer_keyword] = ACTIONS(1503),
    [sym_call_keyword] = ACTIONS(1503),
    [sym_system_call_keyword] = ACTIONS(1503),
    [sym_devchan_to_keyword] = ACTIONS(1503),
    [sym_devchan_min_to_keyword] = ACTIONS(1503),
    [sym_devchan_total_off_keyword] = ACTIONS(1503),
    [sym_devchan_pulse_keyword] = ACTIONS(1503),
    [sym_wait_keyword] = ACTIONS(1503),
    [sym_cancel_wait_keyword] = ACTIONS(1503),
    [sym_wait_until_keyword] = ACTIONS(1503),
    [sym_cancel_wait_until_keyword] = ACTIONS(1503),
    [sym_cancel_all_wait_keyword] = ACTIONS(1503),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1503),
    [sym_and] = ACTIONS(1359),
    [sym_or] = ACTIONS(1361),
    [sym_not] = ACTIONS(1503),
    [sym_xor] = ACTIONS(1363),
    [sym_band] = ACTIONS(1365),
    [sym_bor] = ACTIONS(1367),
    [sym_bxor] = ACTIONS(1363),
    [sym_bnot] = ACTIONS(1503),
    [sym_lshift] = ACTIONS(1369),
    [sym_rshift] = ACTIONS(1369),
    [sym___file__] = ACTIONS(1503),
    [sym___date__] = ACTIONS(1503),
    [sym___ldate__] = ACTIONS(1503),
    [sym___time__] = ACTIONS(1503),
    [sym___line__] = ACTIONS(1503),
    [sym___name__] = ACTIONS(1503),
    [sym___netlinx__] = ACTIONS(1503),
    [sym_netlinx_axi_version] = ACTIONS(1503),
    [sym_url_flg_tcp] = ACTIONS(1503),
    [sym_url_flg_accinfopresent] = ACTIONS(1503),
    [sym_url_flg_temp] = ACTIONS(1503),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1503),
    [sym_url_flg_stat_mask] = ACTIONS(1503),
    [sym_url_flg_stat_lookup] = ACTIONS(1503),
    [sym_url_flg_stat_connecting] = ACTIONS(1503),
    [sym_url_flg_stat_waiting] = ACTIONS(1503),
    [sym_url_flg_stat_connected] = ACTIONS(1503),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1503),
    [sym_first_virtual_device] = ACTIONS(1503),
    [sym_first_local_port] = ACTIONS(1503),
    [sym_dynamic_virtual_device] = ACTIONS(1503),
    [sym_timeline_once] = ACTIONS(1503),
    [sym_timeline_repeat] = ACTIONS(1503),
    [sym_timeline_absolute] = ACTIONS(1503),
    [sym_timeline_relative] = ACTIONS(1503),
    [sym_do_push_timed_infinite] = ACTIONS(1503),
    [sym_source_type_no_address] = ACTIONS(1503),
    [sym_source_type_neuron_id] = ACTIONS(1503),
    [sym_source_type_ip_address] = ACTIONS(1503),
    [sym_source_type_axlink] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1503),
    [sym_source_type_ip_socket_address] = ACTIONS(1503),
    [sym_source_type_rs232] = ACTIONS(1503),
    [sym_source_type_internal] = ACTIONS(1503),
    [sym_source_type_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1503),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1503),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1503),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1503),
    [sym_true] = ACTIONS(1503),
    [sym_false] = ACTIONS(1503),
    [sym_file_read_only] = ACTIONS(1503),
    [sym_file_rw_new] = ACTIONS(1503),
    [sym_file_rw_append] = ACTIONS(1503),
    [sym_ip_tcp] = ACTIONS(1503),
    [sym_ip_udp] = ACTIONS(1503),
    [sym_ip_udp_2way] = ACTIONS(1503),
    [sym_xml_encode_types] = ACTIONS(1503),
    [sym_xml_encode_char_as_list] = ACTIONS(1503),
    [sym_xml_encode_le] = ACTIONS(1503),
    [sym_xml_decode_types] = ACTIONS(1503),
    [sym_xml_decode_no_preserve] = ACTIONS(1503),
    [sym_duet_dev_type_display_device] = ACTIONS(1503),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_io_device] = ACTIONS(1503),
    [sym_duet_dev_type_relay_device] = ACTIONS(1503),
    [sym_duet_dev_type_ups] = ACTIONS(1503),
    [sym_duet_dev_type_amplifier] = ACTIONS(1503),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1503),
    [sym_duet_dev_type_camera] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1503),
    [sym_duet_dev_type_dss] = ACTIONS(1503),
    [sym_duet_dev_type_dvr] = ACTIONS(1503),
    [sym_duet_dev_type_disc_device] = ACTIONS(1503),
    [sym_duet_dev_type_document_camera] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1503),
    [sym_duet_dev_type_hvac] = ACTIONS(1503),
    [sym_duet_dev_type_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_light] = ACTIONS(1503),
    [sym_duet_dev_type_monitor] = ACTIONS(1503),
    [sym_duet_dev_type_motor] = ACTIONS(1503),
    [sym_duet_dev_type_multi_window] = ACTIONS(1503),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1503),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1503),
    [sym_duet_dev_type_receiver] = ACTIONS(1503),
    [sym_duet_dev_type_security_system] = ACTIONS(1503),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1503),
    [sym_duet_dev_type_settop_box] = ACTIONS(1503),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1503),
    [sym_duet_dev_type_switcher] = ACTIONS(1503),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_tv] = ACTIONS(1503),
    [sym_duet_dev_type_utility] = ACTIONS(1503),
    [sym_duet_dev_type_vcr] = ACTIONS(1503),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_video_processor] = ACTIONS(1503),
    [sym_duet_dev_type_video_projector] = ACTIONS(1503),
    [sym_duet_dev_type_video_wall] = ACTIONS(1503),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1503),
    [sym_duet_dev_type_weather] = ACTIONS(1503),
    [sym_duet_dev_not_polled] = ACTIONS(1503),
    [sym_duet_dev_polled] = ACTIONS(1503),
    [sym_clkmgr_mode_network] = ACTIONS(1503),
    [sym_clkmgr_mode_standalone] = ACTIONS(1503),
    [sym_normal_standby] = ACTIONS(1503),
    [sym_normal_wake] = ACTIONS(1503),
    [sym_ip_multicast_ttl_option] = ACTIONS(1503),
    [sym_ip_ttl_subnet] = ACTIONS(1503),
    [sym_ip_ttl_site] = ACTIONS(1503),
    [sym_ip_ttl_region] = ACTIONS(1503),
    [sym_ip_ttl_continent] = ACTIONS(1503),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1503),
    [sym_ip_nodelay_on] = ACTIONS(1503),
    [sym_ip_nodelay_off] = ACTIONS(1503),
    [sym_amx_error] = ACTIONS(1503),
    [sym_amx_warning] = ACTIONS(1503),
    [sym_amx_info] = ACTIONS(1503),
    [sym_amx_debug] = ACTIONS(1503),
    [sym_netlinx_login_success] = ACTIONS(1503),
    [sym_netlinx_login_fail] = ACTIONS(1503),
    [sym_netlinx_logout] = ACTIONS(1503),
    [sym_null_str] = ACTIONS(1503),
    [sym_smtp_address] = ACTIONS(1503),
    [sym_smtp_port_number] = ACTIONS(1503),
    [sym_smtp_username] = ACTIONS(1503),
    [sym_smtp_password] = ACTIONS(1503),
    [sym_smtp_require_tls] = ACTIONS(1503),
    [sym_smtp_from] = ACTIONS(1503),
    [sym_smtp_tls_true] = ACTIONS(1503),
    [sym_smtp_tls_false] = ACTIONS(1503),
    [sym_listview_on_row_select_event] = ACTIONS(1503),
    [sym_data_structure_unknown] = ACTIONS(1503),
    [sym_data_structure_datafeed] = ACTIONS(1503),
    [sym_data_structure_datarecordset] = ACTIONS(1503),
    [sym_data_structure_datarecord] = ACTIONS(1503),
    [sym_data_structure_datafield] = ACTIONS(1503),
    [sym_data_max_content_field_count] = ACTIONS(1503),
    [sym_data_max_metadata_field_count] = ACTIONS(1503),
    [sym_data_max_id_length] = ACTIONS(1503),
    [sym_data_max_name_length] = ACTIONS(1503),
    [sym_data_max_description_length] = ACTIONS(1503),
    [sym_data_max_type_length] = ACTIONS(1503),
    [sym_data_max_format_length] = ACTIONS(1503),
    [sym_data_max_source_length] = ACTIONS(1503),
    [sym_data_max_value_length] = ACTIONS(1503),
    [sym_data_max_label_length] = ACTIONS(1503),
    [sym_data_max_url_length] = ACTIONS(1503),
    [sym_max_last_login_info_length] = ACTIONS(1503),
    [sym_data_type_unknown] = ACTIONS(1503),
    [sym_data_type_string] = ACTIONS(1503),
    [sym_data_type_datetime] = ACTIONS(1503),
    [sym_data_type_date] = ACTIONS(1503),
    [sym_data_type_time] = ACTIONS(1503),
    [sym_data_type_image] = ACTIONS(1503),
    [sym_data_format_url] = ACTIONS(1503),
    [sym_data_format_phone] = ACTIONS(1503),
    [sym_data_format_email] = ACTIONS(1503),
    [sym_data_format_iso8601] = ACTIONS(1503),
    [sym_auditlog_priv] = ACTIONS(1503),
    [sym_configuration_priv] = ACTIONS(1503),
    [sym_deviceconfig_priv] = ACTIONS(1503),
    [sym_ftp_priv] = ACTIONS(1503),
    [sym_http_priv] = ACTIONS(1503),
    [sym_networkconfig_priv] = ACTIONS(1503),
    [sym_programport_priv] = ACTIONS(1503),
    [sym_remoteui_priv] = ACTIONS(1503),
    [sym_securitycontrol_priv] = ACTIONS(1503),
    [sym_softwaremanagement_priv] = ACTIONS(1503),
    [sym_terminal_priv] = ACTIONS(1503),
    [sym_tpadmin_priv] = ACTIONS(1503),
    [sym_user1_priv] = ACTIONS(1503),
    [sym_user2_priv] = ACTIONS(1503),
    [sym_user3_priv] = ACTIONS(1503),
    [sym_user4_priv] = ACTIONS(1503),
    [sym_usermanagement_priv] = ACTIONS(1503),
    [sym_valid_account] = ACTIONS(1503),
    [sym_err_name_invalid] = ACTIONS(1503),
    [sym_err_password_invalid] = ACTIONS(1503),
    [sym_err_invalid_account] = ACTIONS(1503),
    [sym_err_no_authorization] = ACTIONS(1503),
    [sym_err_invalid_parameter] = ACTIONS(1503),
    [sym_err_user_locked_out] = ACTIONS(1503),
    [sym_err_user_account_expired] = ACTIONS(1503),
    [sym_err_general_error] = ACTIONS(1503),
    [sym_err_authentication_error] = ACTIONS(1503),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1503),
    [sym_tls_validate_certificate] = ACTIONS(1503),
    [sym_tdata] = ACTIONS(1503),
    [sym_tchannel] = ACTIONS(1503),
    [sym_tlevel] = ACTIONS(1503),
    [sym_tbutton] = ACTIONS(1503),
    [sym_ttimeline] = ACTIONS(1503),
    [sym_tcustom] = ACTIONS(1503),
    [sym_url_struct] = ACTIONS(1503),
    [sym_dns_struct] = ACTIONS(1503),
    [sym_ip_address_struct] = ACTIONS(1503),
    [sym_dev_info_struct] = ACTIONS(1503),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1503),
    [sym_data_feed] = ACTIONS(1503),
    [sym_data_field] = ACTIONS(1503),
    [sym_data_record] = ACTIONS(1503),
    [sym_wc_data_feed] = ACTIONS(1503),
    [sym_wc_data_field] = ACTIONS(1503),
    [sym_wc_data_record] = ACTIONS(1503),
    [sym_last_login_info] = ACTIONS(1503),
    [sym____reserved___] = ACTIONS(1503),
    [sym_date] = ACTIONS(1503),
    [sym_day] = ACTIONS(1503),
    [sym_time] = ACTIONS(1503),
    [sym_ldate] = ACTIONS(1503),
    [sym_push_device] = ACTIONS(1503),
    [sym_push_channel] = ACTIONS(1503),
    [sym_push_devchan] = ACTIONS(1503),
    [sym_release_device] = ACTIONS(1503),
    [sym_release_channel] = ACTIONS(1503),
    [sym_release_devchan] = ACTIONS(1503),
    [sym_master_slot] = ACTIONS(1503),
    [sym_get_pulse_time] = ACTIONS(1503),
    [sym_get_timer] = ACTIONS(1503),
    [sym_master_sn] = ACTIONS(1503),
    [sym_system_number] = ACTIONS(1503),
    [sym_button] = ACTIONS(1503),
    [sym_dv_channel] = ACTIONS(1503),
    [sym_data] = ACTIONS(1503),
    [sym_level] = ACTIONS(1503),
    [sym_timeline] = ACTIONS(1503),
    [sym_channel] = ACTIONS(1503),
    [sym_custom] = ACTIONS(1503),
    [sym_atoi] = ACTIONS(1503),
    [sym_atol] = ACTIONS(1503),
    [sym_atof] = ACTIONS(1503),
    [sym_itoa] = ACTIONS(1503),
    [sym_itohex] = ACTIONS(1503),
    [sym_hextoi] = ACTIONS(1503),
    [sym_ftoa] = ACTIONS(1503),
    [sym_find_string] = ACTIONS(1503),
    [sym_left_string] = ACTIONS(1503),
    [sym_right_string] = ACTIONS(1503),
    [sym_mid_string] = ACTIONS(1503),
    [sym_remove_string] = ACTIONS(1503),
    [sym_get_buffer_string] = ACTIONS(1503),
    [sym_get_buffer_char] = ACTIONS(1503),
    [sym_get_multi_buffer_string] = ACTIONS(1503),
    [sym_length_string] = ACTIONS(1503),
    [sym_max_length_string] = ACTIONS(1503),
    [sym_set_length_string] = ACTIONS(1503),
    [sym_compare_string] = ACTIONS(1503),
    [sym_upper_string] = ACTIONS(1503),
    [sym_lower_string] = ACTIONS(1503),
    [sym_get_last] = ACTIONS(1503),
    [sym_date_to_day] = ACTIONS(1503),
    [sym_date_to_month] = ACTIONS(1503),
    [sym_date_to_year] = ACTIONS(1503),
    [sym_time_to_hour] = ACTIONS(1503),
    [sym_time_to_minute] = ACTIONS(1503),
    [sym_time_to_second] = ACTIONS(1503),
    [sym_day_of_week] = ACTIONS(1503),
    [sym_set_pulse_time] = ACTIONS(1503),
    [sym_set_timer] = ACTIONS(1503),
    [sym_file_open] = ACTIONS(1503),
    [sym_file_close] = ACTIONS(1503),
    [sym_file_read] = ACTIONS(1503),
    [sym_file_read_line] = ACTIONS(1503),
    [sym_file_write] = ACTIONS(1503),
    [sym_file_write_line] = ACTIONS(1503),
    [sym_file_copy] = ACTIONS(1503),
    [sym_file_delete] = ACTIONS(1503),
    [sym_file_rename] = ACTIONS(1503),
    [sym_file_dir] = ACTIONS(1503),
    [sym_file_getdir] = ACTIONS(1503),
    [sym_file_setdir] = ACTIONS(1503),
    [sym_file_create_dir] = ACTIONS(1503),
    [sym_file_remove_dir] = ACTIONS(1503),
    [sym_file_seek] = ACTIONS(1503),
    [sym_length_array] = ACTIONS(1503),
    [sym_max_length_array] = ACTIONS(1503),
    [sym_set_length_array] = ACTIONS(1503),
    [sym_ip_client_open] = ACTIONS(1503),
    [sym_ip_client_close] = ACTIONS(1503),
    [sym_ip_server_open] = ACTIONS(1503),
    [sym_ip_server_close] = ACTIONS(1503),
    [sym_ip_bound_client_open] = ACTIONS(1503),
    [sym_ip_mc_server_open] = ACTIONS(1503),
    [sym_ip_set_option] = ACTIONS(1503),
    [sym_ssh_client_open] = ACTIONS(1503),
    [sym_ssh_client_close] = ACTIONS(1503),
    [sym_tls_client_open] = ACTIONS(1503),
    [sym_tls_client_close] = ACTIONS(1503),
    [sym_do_push] = ACTIONS(1503),
    [sym_do_push_timed] = ACTIONS(1503),
    [sym_do_release] = ACTIONS(1503),
    [sym_device_id] = ACTIONS(1503),
    [sym_device_info] = ACTIONS(1503),
    [sym_device_id_string] = ACTIONS(1503),
    [sym_device_standby] = ACTIONS(1503),
    [sym_device_wake] = ACTIONS(1503),
    [sym_timeline_create] = ACTIONS(1503),
    [sym_timeline_kill] = ACTIONS(1503),
    [sym_timeline_reload] = ACTIONS(1503),
    [sym_timeline_pause] = ACTIONS(1503),
    [sym_timeline_restart] = ACTIONS(1503),
    [sym_timeline_set] = ACTIONS(1503),
    [sym_timeline_get] = ACTIONS(1503),
    [sym_timeline_active] = ACTIONS(1503),
    [sym_timeline_dynamic_id] = ACTIONS(1503),
    [sym_variable_to_string] = ACTIONS(1503),
    [sym_string_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_string] = ACTIONS(1503),
    [sym_type_cast] = ACTIONS(1503),
    [sym_variable_to_xml] = ACTIONS(1503),
    [sym_xml_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_xml] = ACTIONS(1503),
    [sym_redirect_string] = ACTIONS(1503),
    [sym_random_number] = ACTIONS(1503),
    [sym_get_unique_id] = ACTIONS(1503),
    [sym_get_system_number] = ACTIONS(1503),
    [sym_set_system_number] = ACTIONS(1503),
    [sym_get_serial_number] = ACTIONS(1503),
    [sym_get_master_build] = ACTIONS(1503),
    [sym_set_validation_code] = ACTIONS(1503),
    [sym_reboot] = ACTIONS(1503),
    [sym_load_duet_module] = ACTIONS(1503),
    [sym_unload_duet_module] = ACTIONS(1503),
    [sym_duet_mem_size_set] = ACTIONS(1503),
    [sym_duet_mem_size_get] = ACTIONS(1503),
    [sym_dynamic_application_device] = ACTIONS(1503),
    [sym_dynamic_polled_port] = ACTIONS(1503),
    [sym_static_port_binding] = ACTIONS(1503),
    [sym_static_ip_binding] = ACTIONS(1503),
    [sym_get_dns_list] = ACTIONS(1503),
    [sym_set_dns_list] = ACTIONS(1503),
    [sym_get_ip_address] = ACTIONS(1503),
    [sym_set_ip_address] = ACTIONS(1503),
    [sym_get_url_list] = ACTIONS(1503),
    [sym_add_url_entry] = ACTIONS(1503),
    [sym_delete_url_entry] = ACTIONS(1503),
    [sym_abs_value] = ACTIONS(1503),
    [sym_max_value] = ACTIONS(1503),
    [sym_min_value] = ACTIONS(1503),
    [sym_exp_value] = ACTIONS(1503),
    [sym_log_value] = ACTIONS(1503),
    [sym_log10_value] = ACTIONS(1503),
    [sym_power_value] = ACTIONS(1503),
    [sym_sqrt_value] = ACTIONS(1503),
    [sym_raw_be] = ACTIONS(1503),
    [sym_raw_le] = ACTIONS(1503),
    [sym_internal_queue_size_set] = ACTIONS(1503),
    [sym_internal_queue_size_get] = ACTIONS(1503),
    [sym_internal_threshold_set] = ACTIONS(1503),
    [sym_internal_threshold_get] = ACTIONS(1503),
    [sym_rebuilt_event] = ACTIONS(1503),
    [sym_do_custom_event] = ACTIONS(1503),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1503),
    [sym_clkmgr_set_clk_source] = ACTIONS(1503),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1503),
    [sym_clkmgr_get_timezone] = ACTIONS(1503),
    [sym_clkmgr_set_timezone] = ACTIONS(1503),
    [sym_clkmgr_get_resync_period] = ACTIONS(1503),
    [sym_clkmgr_set_resync_period] = ACTIONS(1503),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_timeservers] = ACTIONS(1503),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_set_log_level] = ACTIONS(1503),
    [sym_get_log_level] = ACTIONS(1503),
    [sym_amx_log] = ACTIONS(1503),
    [sym_smtp_server_config_set] = ACTIONS(1503),
    [sym_smtp_server_config_get] = ACTIONS(1503),
    [sym_smtp_send] = ACTIONS(1503),
    [sym_validate_netlinx_account] = ACTIONS(1503),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1503),
    [sym_authenticate_certificate] = ACTIONS(1503),
    [sym_audit_netlinx_session_event] = ACTIONS(1503),
    [sym_audit_netlinx_generic_event] = ACTIONS(1503),
    [sym_get_available_flash_disk_space] = ACTIONS(1503),
    [sym_get_max_flash_disk_space] = ACTIONS(1503),
    [sym_data_create_feed] = ACTIONS(1503),
    [sym_data_delete_feed] = ACTIONS(1503),
    [sym_data_publish_feed] = ACTIONS(1503),
    [sym_data_get_published_feed] = ACTIONS(1503),
    [sym_data_add_record] = ACTIONS(1503),
    [sym_data_get_event_record] = ACTIONS(1503),
    [sym__wc_data_create_feed] = ACTIONS(1503),
    [sym__wc_data_add_record] = ACTIONS(1503),
    [sym__wc_data_get_event_record] = ACTIONS(1503),
    [sym_format] = ACTIONS(1503),
    [sym_astro_clock] = ACTIONS(1503),
    [sym_set_outdoor_temperature] = ACTIONS(1503),
    [sym_set_virtual_level_count] = ACTIONS(1503),
    [sym_set_virtual_channel_count] = ACTIONS(1503),
    [sym_set_virtual_port_count] = ACTIONS(1503),
    [sym__wc_encode_int] = ACTIONS(1503),
    [sym__wc_decode_int] = ACTIONS(1503),
    [sym__wc_int] = ACTIONS(1503),
    [sym__wc_to_ch_int] = ACTIONS(1503),
    [sym__ch_to_wc_int] = ACTIONS(1503),
    [sym__wc_find_string_int] = ACTIONS(1503),
    [sym__wc_left_string_int] = ACTIONS(1503),
    [sym__wc_lower_string_int] = ACTIONS(1503),
    [sym__wc_mid_string_int] = ACTIONS(1503),
    [sym__wc_remove_string_int] = ACTIONS(1503),
    [sym__wc_right_string_int] = ACTIONS(1503),
    [sym__wc_upper_string_int] = ACTIONS(1503),
    [sym__wc_compare_string_int] = ACTIONS(1503),
    [sym__wc_get_buffer_char_int] = ACTIONS(1503),
    [sym__wc_get_buffer_string_int] = ACTIONS(1503),
    [sym__wc_concat_string_int] = ACTIONS(1503),
    [sym__wc_file_open_int] = ACTIONS(1503),
    [sym__wc_file_close_int] = ACTIONS(1503),
    [sym__wc_file_read_int] = ACTIONS(1503),
    [sym__wc_file_read_line_int] = ACTIONS(1503),
    [sym__wc_file_write_int] = ACTIONS(1503),
    [sym__wc_file_write_line_int] = ACTIONS(1503),
    [aux_sym_preproc_include_token1] = ACTIONS(1503),
    [aux_sym_preproc_define_token1] = ACTIONS(1505),
    [aux_sym_preproc_warn_token1] = ACTIONS(1505),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1505),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1505),
    [aux_sym_preproc_else_token1] = ACTIONS(1505),
    [anon_sym_LBRACK] = ACTIONS(1373),
    [anon_sym_LBRACE] = ACTIONS(1505),
    [anon_sym_COLON] = ACTIONS(1375),
    [anon_sym_LPAREN] = ACTIONS(1377),
    [anon_sym_SEMI] = ACTIONS(1505),
    [anon_sym_BANG] = ACTIONS(1503),
    [anon_sym_TILDE] = ACTIONS(1505),
    [anon_sym_DASH] = ACTIONS(1381),
    [anon_sym_PLUS] = ACTIONS(1381),
    [anon_sym_STAR] = ACTIONS(1383),
    [anon_sym_SLASH] = ACTIONS(1385),
    [anon_sym_PERCENT] = ACTIONS(1383),
    [anon_sym_PIPE_PIPE] = ACTIONS(1387),
    [anon_sym_AMP_AMP] = ACTIONS(1389),
    [anon_sym_PIPE] = ACTIONS(1367),
    [anon_sym_CARET] = ACTIONS(1391),
    [anon_sym_AMP] = ACTIONS(1365),
    [anon_sym_EQ_EQ] = ACTIONS(1393),
    [anon_sym_BANG_EQ] = ACTIONS(1393),
    [anon_sym_LT_GT] = ACTIONS(1393),
    [anon_sym_GT] = ACTIONS(1395),
    [anon_sym_GT_EQ] = ACTIONS(1397),
    [anon_sym_LT_EQ] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1395),
    [anon_sym_LT_LT] = ACTIONS(1399),
    [anon_sym_GT_GT] = ACTIONS(1399),
    [anon_sym_DASH_DASH] = ACTIONS(1401),
    [anon_sym_PLUS_PLUS] = ACTIONS(1401),
    [anon_sym_DOT] = ACTIONS(1403),
    [anon_sym_DQUOTE] = ACTIONS(1505),
    [anon_sym_SQUOTE] = ACTIONS(1505),
    [sym_number_literal] = ACTIONS(1505),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1505),
  },
  [STATE(79)] = {
    [sym_identifier] = ACTIONS(1507),
    [sym_program_name_keyword] = ACTIONS(1507),
    [sym_module_name_keyword] = ACTIONS(1507),
    [sym_define_device_keyword] = ACTIONS(1507),
    [sym_define_combine_keyword] = ACTIONS(1507),
    [sym_define_constant_keyword] = ACTIONS(1507),
    [sym_define_type_keyword] = ACTIONS(1507),
    [sym_define_variable_keyword] = ACTIONS(1507),
    [sym_define_system_variable_keyword] = ACTIONS(1507),
    [sym_define_connect_level_keyword] = ACTIONS(1507),
    [sym_define_latching_keyword] = ACTIONS(1507),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1507),
    [sym_define_toggling_keyword] = ACTIONS(1507),
    [sym_define_call_keyword] = ACTIONS(1507),
    [sym_define_function_keyword] = ACTIONS(1507),
    [sym_define_library_function_keyword] = ACTIONS(1507),
    [sym_define_start_keyword] = ACTIONS(1507),
    [sym_define_event_keyword] = ACTIONS(1507),
    [sym_define_program_keyword] = ACTIONS(1507),
    [sym_define_module_keyword] = ACTIONS(1507),
    [sym_char_keyword] = ACTIONS(1507),
    [sym_widechar_keyword] = ACTIONS(1507),
    [sym_integer_keyword] = ACTIONS(1507),
    [sym_sinteger_keyword] = ACTIONS(1507),
    [sym_long_keyword] = ACTIONS(1507),
    [sym_slong_keyword] = ACTIONS(1507),
    [sym_float_keyword] = ACTIONS(1507),
    [sym_double_keyword] = ACTIONS(1507),
    [sym_variant_keyword] = ACTIONS(1507),
    [sym_variantarray_keyword] = ACTIONS(1507),
    [sym_local_var_keyword] = ACTIONS(1507),
    [sym_stack_var_keyword] = ACTIONS(1507),
    [sym_constant_keyword] = ACTIONS(1507),
    [sym_volatile_keyword] = ACTIONS(1507),
    [sym_non_volatile_keyword] = ACTIONS(1507),
    [sym_persistent_keyword] = ACTIONS(1507),
    [sym_dev_keyword] = ACTIONS(1507),
    [sym_devlev_keyword] = ACTIONS(1507),
    [sym_devchan_keyword] = ACTIONS(1507),
    [sym_if_keyword] = ACTIONS(1507),
    [sym_else_keyword] = ACTIONS(1507),
    [sym_switch_keyword] = ACTIONS(1507),
    [sym_case_keyword] = ACTIONS(1507),
    [sym_default_keyword] = ACTIONS(1507),
    [sym_while_keyword] = ACTIONS(1507),
    [sym_for_keyword] = ACTIONS(1507),
    [sym_break_keyword] = ACTIONS(1507),
    [sym_continue_keyword] = ACTIONS(1507),
    [sym_return_keyword] = ACTIONS(1507),
    [sym_select_keyword] = ACTIONS(1507),
    [sym_active_keyword] = ACTIONS(1507),
    [sym_struct_keyword] = ACTIONS(1507),
    [sym_structure_keyword] = ACTIONS(1507),
    [sym_button_event_keyword] = ACTIONS(1507),
    [sym_channel_event_keyword] = ACTIONS(1507),
    [sym_level_event_keyword] = ACTIONS(1507),
    [sym_data_event_keyword] = ACTIONS(1507),
    [sym_timeline_event_keyword] = ACTIONS(1507),
    [sym_custom_event_keyword] = ACTIONS(1507),
    [sym_push_keyword] = ACTIONS(1507),
    [sym_release_keyword] = ACTIONS(1507),
    [aux_sym_on_keyword_token1] = ACTIONS(1507),
    [aux_sym_off_keyword_token1] = ACTIONS(1507),
    [sym_send_level_keyword] = ACTIONS(1507),
    [sym_send_string_keyword] = ACTIONS(1507),
    [sym_send_command_keyword] = ACTIONS(1507),
    [sym_clear_buffer_keyword] = ACTIONS(1507),
    [sym_create_buffer_keyword] = ACTIONS(1507),
    [sym_create_multi_buffer_keyword] = ACTIONS(1507),
    [sym_call_keyword] = ACTIONS(1507),
    [sym_system_call_keyword] = ACTIONS(1507),
    [sym_devchan_to_keyword] = ACTIONS(1507),
    [sym_devchan_min_to_keyword] = ACTIONS(1507),
    [sym_devchan_total_off_keyword] = ACTIONS(1507),
    [sym_devchan_pulse_keyword] = ACTIONS(1507),
    [sym_wait_keyword] = ACTIONS(1507),
    [sym_cancel_wait_keyword] = ACTIONS(1507),
    [sym_wait_until_keyword] = ACTIONS(1507),
    [sym_cancel_wait_until_keyword] = ACTIONS(1507),
    [sym_cancel_all_wait_keyword] = ACTIONS(1507),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1507),
    [sym_and] = ACTIONS(1507),
    [sym_or] = ACTIONS(1507),
    [sym_not] = ACTIONS(1507),
    [sym_xor] = ACTIONS(1507),
    [sym_band] = ACTIONS(1507),
    [sym_bor] = ACTIONS(1507),
    [sym_bxor] = ACTIONS(1507),
    [sym_bnot] = ACTIONS(1507),
    [sym_lshift] = ACTIONS(1507),
    [sym_rshift] = ACTIONS(1507),
    [sym___file__] = ACTIONS(1507),
    [sym___date__] = ACTIONS(1507),
    [sym___ldate__] = ACTIONS(1507),
    [sym___time__] = ACTIONS(1507),
    [sym___line__] = ACTIONS(1507),
    [sym___name__] = ACTIONS(1507),
    [sym___netlinx__] = ACTIONS(1507),
    [sym_netlinx_axi_version] = ACTIONS(1507),
    [sym_url_flg_tcp] = ACTIONS(1507),
    [sym_url_flg_accinfopresent] = ACTIONS(1507),
    [sym_url_flg_temp] = ACTIONS(1507),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1507),
    [sym_url_flg_stat_mask] = ACTIONS(1507),
    [sym_url_flg_stat_lookup] = ACTIONS(1507),
    [sym_url_flg_stat_connecting] = ACTIONS(1507),
    [sym_url_flg_stat_waiting] = ACTIONS(1507),
    [sym_url_flg_stat_connected] = ACTIONS(1507),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1507),
    [sym_first_virtual_device] = ACTIONS(1507),
    [sym_first_local_port] = ACTIONS(1507),
    [sym_dynamic_virtual_device] = ACTIONS(1507),
    [sym_timeline_once] = ACTIONS(1507),
    [sym_timeline_repeat] = ACTIONS(1507),
    [sym_timeline_absolute] = ACTIONS(1507),
    [sym_timeline_relative] = ACTIONS(1507),
    [sym_do_push_timed_infinite] = ACTIONS(1507),
    [sym_source_type_no_address] = ACTIONS(1507),
    [sym_source_type_neuron_id] = ACTIONS(1507),
    [sym_source_type_ip_address] = ACTIONS(1507),
    [sym_source_type_axlink] = ACTIONS(1507),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1507),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1507),
    [sym_source_type_ip_socket_address] = ACTIONS(1507),
    [sym_source_type_rs232] = ACTIONS(1507),
    [sym_source_type_internal] = ACTIONS(1507),
    [sym_source_type_mac_address] = ACTIONS(1507),
    [sym_source_type_ipv4_port] = ACTIONS(1507),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1507),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1507),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1507),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1507),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1507),
    [sym_true] = ACTIONS(1507),
    [sym_false] = ACTIONS(1507),
    [sym_file_read_only] = ACTIONS(1507),
    [sym_file_rw_new] = ACTIONS(1507),
    [sym_file_rw_append] = ACTIONS(1507),
    [sym_ip_tcp] = ACTIONS(1507),
    [sym_ip_udp] = ACTIONS(1507),
    [sym_ip_udp_2way] = ACTIONS(1507),
    [sym_xml_encode_types] = ACTIONS(1507),
    [sym_xml_encode_char_as_list] = ACTIONS(1507),
    [sym_xml_encode_le] = ACTIONS(1507),
    [sym_xml_decode_types] = ACTIONS(1507),
    [sym_xml_decode_no_preserve] = ACTIONS(1507),
    [sym_duet_dev_type_display_device] = ACTIONS(1507),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1507),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1507),
    [sym_duet_dev_type_io_device] = ACTIONS(1507),
    [sym_duet_dev_type_relay_device] = ACTIONS(1507),
    [sym_duet_dev_type_ups] = ACTIONS(1507),
    [sym_duet_dev_type_amplifier] = ACTIONS(1507),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1507),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1507),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1507),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1507),
    [sym_duet_dev_type_camera] = ACTIONS(1507),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1507),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1507),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1507),
    [sym_duet_dev_type_dss] = ACTIONS(1507),
    [sym_duet_dev_type_dvr] = ACTIONS(1507),
    [sym_duet_dev_type_disc_device] = ACTIONS(1507),
    [sym_duet_dev_type_document_camera] = ACTIONS(1507),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1507),
    [sym_duet_dev_type_hvac] = ACTIONS(1507),
    [sym_duet_dev_type_keypad] = ACTIONS(1507),
    [sym_duet_dev_type_light] = ACTIONS(1507),
    [sym_duet_dev_type_monitor] = ACTIONS(1507),
    [sym_duet_dev_type_motor] = ACTIONS(1507),
    [sym_duet_dev_type_multi_window] = ACTIONS(1507),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1507),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1507),
    [sym_duet_dev_type_receiver] = ACTIONS(1507),
    [sym_duet_dev_type_security_system] = ACTIONS(1507),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1507),
    [sym_duet_dev_type_settop_box] = ACTIONS(1507),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1507),
    [sym_duet_dev_type_switcher] = ACTIONS(1507),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1507),
    [sym_duet_dev_type_tv] = ACTIONS(1507),
    [sym_duet_dev_type_utility] = ACTIONS(1507),
    [sym_duet_dev_type_vcr] = ACTIONS(1507),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1507),
    [sym_duet_dev_type_video_processor] = ACTIONS(1507),
    [sym_duet_dev_type_video_projector] = ACTIONS(1507),
    [sym_duet_dev_type_video_wall] = ACTIONS(1507),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1507),
    [sym_duet_dev_type_weather] = ACTIONS(1507),
    [sym_duet_dev_not_polled] = ACTIONS(1507),
    [sym_duet_dev_polled] = ACTIONS(1507),
    [sym_clkmgr_mode_network] = ACTIONS(1507),
    [sym_clkmgr_mode_standalone] = ACTIONS(1507),
    [sym_normal_standby] = ACTIONS(1507),
    [sym_normal_wake] = ACTIONS(1507),
    [sym_ip_multicast_ttl_option] = ACTIONS(1507),
    [sym_ip_ttl_subnet] = ACTIONS(1507),
    [sym_ip_ttl_site] = ACTIONS(1507),
    [sym_ip_ttl_region] = ACTIONS(1507),
    [sym_ip_ttl_continent] = ACTIONS(1507),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1507),
    [sym_ip_nodelay_on] = ACTIONS(1507),
    [sym_ip_nodelay_off] = ACTIONS(1507),
    [sym_amx_error] = ACTIONS(1507),
    [sym_amx_warning] = ACTIONS(1507),
    [sym_amx_info] = ACTIONS(1507),
    [sym_amx_debug] = ACTIONS(1507),
    [sym_netlinx_login_success] = ACTIONS(1507),
    [sym_netlinx_login_fail] = ACTIONS(1507),
    [sym_netlinx_logout] = ACTIONS(1507),
    [sym_null_str] = ACTIONS(1507),
    [sym_smtp_address] = ACTIONS(1507),
    [sym_smtp_port_number] = ACTIONS(1507),
    [sym_smtp_username] = ACTIONS(1507),
    [sym_smtp_password] = ACTIONS(1507),
    [sym_smtp_require_tls] = ACTIONS(1507),
    [sym_smtp_from] = ACTIONS(1507),
    [sym_smtp_tls_true] = ACTIONS(1507),
    [sym_smtp_tls_false] = ACTIONS(1507),
    [sym_listview_on_row_select_event] = ACTIONS(1507),
    [sym_data_structure_unknown] = ACTIONS(1507),
    [sym_data_structure_datafeed] = ACTIONS(1507),
    [sym_data_structure_datarecordset] = ACTIONS(1507),
    [sym_data_structure_datarecord] = ACTIONS(1507),
    [sym_data_structure_datafield] = ACTIONS(1507),
    [sym_data_max_content_field_count] = ACTIONS(1507),
    [sym_data_max_metadata_field_count] = ACTIONS(1507),
    [sym_data_max_id_length] = ACTIONS(1507),
    [sym_data_max_name_length] = ACTIONS(1507),
    [sym_data_max_description_length] = ACTIONS(1507),
    [sym_data_max_type_length] = ACTIONS(1507),
    [sym_data_max_format_length] = ACTIONS(1507),
    [sym_data_max_source_length] = ACTIONS(1507),
    [sym_data_max_value_length] = ACTIONS(1507),
    [sym_data_max_label_length] = ACTIONS(1507),
    [sym_data_max_url_length] = ACTIONS(1507),
    [sym_max_last_login_info_length] = ACTIONS(1507),
    [sym_data_type_unknown] = ACTIONS(1507),
    [sym_data_type_string] = ACTIONS(1507),
    [sym_data_type_datetime] = ACTIONS(1507),
    [sym_data_type_date] = ACTIONS(1507),
    [sym_data_type_time] = ACTIONS(1507),
    [sym_data_type_image] = ACTIONS(1507),
    [sym_data_format_url] = ACTIONS(1507),
    [sym_data_format_phone] = ACTIONS(1507),
    [sym_data_format_email] = ACTIONS(1507),
    [sym_data_format_iso8601] = ACTIONS(1507),
    [sym_auditlog_priv] = ACTIONS(1507),
    [sym_configuration_priv] = ACTIONS(1507),
    [sym_deviceconfig_priv] = ACTIONS(1507),
    [sym_ftp_priv] = ACTIONS(1507),
    [sym_http_priv] = ACTIONS(1507),
    [sym_networkconfig_priv] = ACTIONS(1507),
    [sym_programport_priv] = ACTIONS(1507),
    [sym_remoteui_priv] = ACTIONS(1507),
    [sym_securitycontrol_priv] = ACTIONS(1507),
    [sym_softwaremanagement_priv] = ACTIONS(1507),
    [sym_terminal_priv] = ACTIONS(1507),
    [sym_tpadmin_priv] = ACTIONS(1507),
    [sym_user1_priv] = ACTIONS(1507),
    [sym_user2_priv] = ACTIONS(1507),
    [sym_user3_priv] = ACTIONS(1507),
    [sym_user4_priv] = ACTIONS(1507),
    [sym_usermanagement_priv] = ACTIONS(1507),
    [sym_valid_account] = ACTIONS(1507),
    [sym_err_name_invalid] = ACTIONS(1507),
    [sym_err_password_invalid] = ACTIONS(1507),
    [sym_err_invalid_account] = ACTIONS(1507),
    [sym_err_no_authorization] = ACTIONS(1507),
    [sym_err_invalid_parameter] = ACTIONS(1507),
    [sym_err_user_locked_out] = ACTIONS(1507),
    [sym_err_user_account_expired] = ACTIONS(1507),
    [sym_err_general_error] = ACTIONS(1507),
    [sym_err_authentication_error] = ACTIONS(1507),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1507),
    [sym_tls_validate_certificate] = ACTIONS(1507),
    [sym_tdata] = ACTIONS(1507),
    [sym_tchannel] = ACTIONS(1507),
    [sym_tlevel] = ACTIONS(1507),
    [sym_tbutton] = ACTIONS(1507),
    [sym_ttimeline] = ACTIONS(1507),
    [sym_tcustom] = ACTIONS(1507),
    [sym_url_struct] = ACTIONS(1507),
    [sym_dns_struct] = ACTIONS(1507),
    [sym_ip_address_struct] = ACTIONS(1507),
    [sym_dev_info_struct] = ACTIONS(1507),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1507),
    [sym_data_feed] = ACTIONS(1507),
    [sym_data_field] = ACTIONS(1507),
    [sym_data_record] = ACTIONS(1507),
    [sym_wc_data_feed] = ACTIONS(1507),
    [sym_wc_data_field] = ACTIONS(1507),
    [sym_wc_data_record] = ACTIONS(1507),
    [sym_last_login_info] = ACTIONS(1507),
    [sym____reserved___] = ACTIONS(1507),
    [sym_date] = ACTIONS(1507),
    [sym_day] = ACTIONS(1507),
    [sym_time] = ACTIONS(1507),
    [sym_ldate] = ACTIONS(1507),
    [sym_push_device] = ACTIONS(1507),
    [sym_push_channel] = ACTIONS(1507),
    [sym_push_devchan] = ACTIONS(1507),
    [sym_release_device] = ACTIONS(1507),
    [sym_release_channel] = ACTIONS(1507),
    [sym_release_devchan] = ACTIONS(1507),
    [sym_master_slot] = ACTIONS(1507),
    [sym_get_pulse_time] = ACTIONS(1507),
    [sym_get_timer] = ACTIONS(1507),
    [sym_master_sn] = ACTIONS(1507),
    [sym_system_number] = ACTIONS(1507),
    [sym_button] = ACTIONS(1507),
    [sym_dv_channel] = ACTIONS(1507),
    [sym_data] = ACTIONS(1507),
    [sym_level] = ACTIONS(1507),
    [sym_timeline] = ACTIONS(1507),
    [sym_channel] = ACTIONS(1507),
    [sym_custom] = ACTIONS(1507),
    [sym_atoi] = ACTIONS(1507),
    [sym_atol] = ACTIONS(1507),
    [sym_atof] = ACTIONS(1507),
    [sym_itoa] = ACTIONS(1507),
    [sym_itohex] = ACTIONS(1507),
    [sym_hextoi] = ACTIONS(1507),
    [sym_ftoa] = ACTIONS(1507),
    [sym_find_string] = ACTIONS(1507),
    [sym_left_string] = ACTIONS(1507),
    [sym_right_string] = ACTIONS(1507),
    [sym_mid_string] = ACTIONS(1507),
    [sym_remove_string] = ACTIONS(1507),
    [sym_get_buffer_string] = ACTIONS(1507),
    [sym_get_buffer_char] = ACTIONS(1507),
    [sym_get_multi_buffer_string] = ACTIONS(1507),
    [sym_length_string] = ACTIONS(1507),
    [sym_max_length_string] = ACTIONS(1507),
    [sym_set_length_string] = ACTIONS(1507),
    [sym_compare_string] = ACTIONS(1507),
    [sym_upper_string] = ACTIONS(1507),
    [sym_lower_string] = ACTIONS(1507),
    [sym_get_last] = ACTIONS(1507),
    [sym_date_to_day] = ACTIONS(1507),
    [sym_date_to_month] = ACTIONS(1507),
    [sym_date_to_year] = ACTIONS(1507),
    [sym_time_to_hour] = ACTIONS(1507),
    [sym_time_to_minute] = ACTIONS(1507),
    [sym_time_to_second] = ACTIONS(1507),
    [sym_day_of_week] = ACTIONS(1507),
    [sym_set_pulse_time] = ACTIONS(1507),
    [sym_set_timer] = ACTIONS(1507),
    [sym_file_open] = ACTIONS(1507),
    [sym_file_close] = ACTIONS(1507),
    [sym_file_read] = ACTIONS(1507),
    [sym_file_read_line] = ACTIONS(1507),
    [sym_file_write] = ACTIONS(1507),
    [sym_file_write_line] = ACTIONS(1507),
    [sym_file_copy] = ACTIONS(1507),
    [sym_file_delete] = ACTIONS(1507),
    [sym_file_rename] = ACTIONS(1507),
    [sym_file_dir] = ACTIONS(1507),
    [sym_file_getdir] = ACTIONS(1507),
    [sym_file_setdir] = ACTIONS(1507),
    [sym_file_create_dir] = ACTIONS(1507),
    [sym_file_remove_dir] = ACTIONS(1507),
    [sym_file_seek] = ACTIONS(1507),
    [sym_length_array] = ACTIONS(1507),
    [sym_max_length_array] = ACTIONS(1507),
    [sym_set_length_array] = ACTIONS(1507),
    [sym_ip_client_open] = ACTIONS(1507),
    [sym_ip_client_close] = ACTIONS(1507),
    [sym_ip_server_open] = ACTIONS(1507),
    [sym_ip_server_close] = ACTIONS(1507),
    [sym_ip_bound_client_open] = ACTIONS(1507),
    [sym_ip_mc_server_open] = ACTIONS(1507),
    [sym_ip_set_option] = ACTIONS(1507),
    [sym_ssh_client_open] = ACTIONS(1507),
    [sym_ssh_client_close] = ACTIONS(1507),
    [sym_tls_client_open] = ACTIONS(1507),
    [sym_tls_client_close] = ACTIONS(1507),
    [sym_do_push] = ACTIONS(1507),
    [sym_do_push_timed] = ACTIONS(1507),
    [sym_do_release] = ACTIONS(1507),
    [sym_device_id] = ACTIONS(1507),
    [sym_device_info] = ACTIONS(1507),
    [sym_device_id_string] = ACTIONS(1507),
    [sym_device_standby] = ACTIONS(1507),
    [sym_device_wake] = ACTIONS(1507),
    [sym_timeline_create] = ACTIONS(1507),
    [sym_timeline_kill] = ACTIONS(1507),
    [sym_timeline_reload] = ACTIONS(1507),
    [sym_timeline_pause] = ACTIONS(1507),
    [sym_timeline_restart] = ACTIONS(1507),
    [sym_timeline_set] = ACTIONS(1507),
    [sym_timeline_get] = ACTIONS(1507),
    [sym_timeline_active] = ACTIONS(1507),
    [sym_timeline_dynamic_id] = ACTIONS(1507),
    [sym_variable_to_string] = ACTIONS(1507),
    [sym_string_to_variable] = ACTIONS(1507),
    [sym_length_variable_to_string] = ACTIONS(1507),
    [sym_type_cast] = ACTIONS(1507),
    [sym_variable_to_xml] = ACTIONS(1507),
    [sym_xml_to_variable] = ACTIONS(1507),
    [sym_length_variable_to_xml] = ACTIONS(1507),
    [sym_redirect_string] = ACTIONS(1507),
    [sym_random_number] = ACTIONS(1507),
    [sym_get_unique_id] = ACTIONS(1507),
    [sym_get_system_number] = ACTIONS(1507),
    [sym_set_system_number] = ACTIONS(1507),
    [sym_get_serial_number] = ACTIONS(1507),
    [sym_get_master_build] = ACTIONS(1507),
    [sym_set_validation_code] = ACTIONS(1507),
    [sym_reboot] = ACTIONS(1507),
    [sym_load_duet_module] = ACTIONS(1507),
    [sym_unload_duet_module] = ACTIONS(1507),
    [sym_duet_mem_size_set] = ACTIONS(1507),
    [sym_duet_mem_size_get] = ACTIONS(1507),
    [sym_dynamic_application_device] = ACTIONS(1507),
    [sym_dynamic_polled_port] = ACTIONS(1507),
    [sym_static_port_binding] = ACTIONS(1507),
    [sym_static_ip_binding] = ACTIONS(1507),
    [sym_get_dns_list] = ACTIONS(1507),
    [sym_set_dns_list] = ACTIONS(1507),
    [sym_get_ip_address] = ACTIONS(1507),
    [sym_set_ip_address] = ACTIONS(1507),
    [sym_get_url_list] = ACTIONS(1507),
    [sym_add_url_entry] = ACTIONS(1507),
    [sym_delete_url_entry] = ACTIONS(1507),
    [sym_abs_value] = ACTIONS(1507),
    [sym_max_value] = ACTIONS(1507),
    [sym_min_value] = ACTIONS(1507),
    [sym_exp_value] = ACTIONS(1507),
    [sym_log_value] = ACTIONS(1507),
    [sym_log10_value] = ACTIONS(1507),
    [sym_power_value] = ACTIONS(1507),
    [sym_sqrt_value] = ACTIONS(1507),
    [sym_raw_be] = ACTIONS(1507),
    [sym_raw_le] = ACTIONS(1507),
    [sym_internal_queue_size_set] = ACTIONS(1507),
    [sym_internal_queue_size_get] = ACTIONS(1507),
    [sym_internal_threshold_set] = ACTIONS(1507),
    [sym_internal_threshold_get] = ACTIONS(1507),
    [sym_rebuilt_event] = ACTIONS(1507),
    [sym_do_custom_event] = ACTIONS(1507),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1507),
    [sym_clkmgr_set_clk_source] = ACTIONS(1507),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1507),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1507),
    [sym_clkmgr_get_timezone] = ACTIONS(1507),
    [sym_clkmgr_set_timezone] = ACTIONS(1507),
    [sym_clkmgr_get_resync_period] = ACTIONS(1507),
    [sym_clkmgr_set_resync_period] = ACTIONS(1507),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1507),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1507),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1507),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1507),
    [sym_clkmgr_get_timeservers] = ACTIONS(1507),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1507),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1507),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1507),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1507),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1507),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1507),
    [sym_set_log_level] = ACTIONS(1507),
    [sym_get_log_level] = ACTIONS(1507),
    [sym_amx_log] = ACTIONS(1507),
    [sym_smtp_server_config_set] = ACTIONS(1507),
    [sym_smtp_server_config_get] = ACTIONS(1507),
    [sym_smtp_send] = ACTIONS(1507),
    [sym_validate_netlinx_account] = ACTIONS(1507),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1507),
    [sym_authenticate_certificate] = ACTIONS(1507),
    [sym_audit_netlinx_session_event] = ACTIONS(1507),
    [sym_audit_netlinx_generic_event] = ACTIONS(1507),
    [sym_get_available_flash_disk_space] = ACTIONS(1507),
    [sym_get_max_flash_disk_space] = ACTIONS(1507),
    [sym_data_create_feed] = ACTIONS(1507),
    [sym_data_delete_feed] = ACTIONS(1507),
    [sym_data_publish_feed] = ACTIONS(1507),
    [sym_data_get_published_feed] = ACTIONS(1507),
    [sym_data_add_record] = ACTIONS(1507),
    [sym_data_get_event_record] = ACTIONS(1507),
    [sym__wc_data_create_feed] = ACTIONS(1507),
    [sym__wc_data_add_record] = ACTIONS(1507),
    [sym__wc_data_get_event_record] = ACTIONS(1507),
    [sym_format] = ACTIONS(1507),
    [sym_astro_clock] = ACTIONS(1507),
    [sym_set_outdoor_temperature] = ACTIONS(1507),
    [sym_set_virtual_level_count] = ACTIONS(1507),
    [sym_set_virtual_channel_count] = ACTIONS(1507),
    [sym_set_virtual_port_count] = ACTIONS(1507),
    [sym__wc_encode_int] = ACTIONS(1507),
    [sym__wc_decode_int] = ACTIONS(1507),
    [sym__wc_int] = ACTIONS(1507),
    [sym__wc_to_ch_int] = ACTIONS(1507),
    [sym__ch_to_wc_int] = ACTIONS(1507),
    [sym__wc_find_string_int] = ACTIONS(1507),
    [sym__wc_left_string_int] = ACTIONS(1507),
    [sym__wc_lower_string_int] = ACTIONS(1507),
    [sym__wc_mid_string_int] = ACTIONS(1507),
    [sym__wc_remove_string_int] = ACTIONS(1507),
    [sym__wc_right_string_int] = ACTIONS(1507),
    [sym__wc_upper_string_int] = ACTIONS(1507),
    [sym__wc_compare_string_int] = ACTIONS(1507),
    [sym__wc_get_buffer_char_int] = ACTIONS(1507),
    [sym__wc_get_buffer_string_int] = ACTIONS(1507),
    [sym__wc_concat_string_int] = ACTIONS(1507),
    [sym__wc_file_open_int] = ACTIONS(1507),
    [sym__wc_file_close_int] = ACTIONS(1507),
    [sym__wc_file_read_int] = ACTIONS(1507),
    [sym__wc_file_read_line_int] = ACTIONS(1507),
    [sym__wc_file_write_int] = ACTIONS(1507),
    [sym__wc_file_write_line_int] = ACTIONS(1507),
    [anon_sym_EQ] = ACTIONS(1507),
    [aux_sym_preproc_include_token1] = ACTIONS(1507),
    [aux_sym_preproc_define_token1] = ACTIONS(1509),
    [aux_sym_preproc_warn_token1] = ACTIONS(1509),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1509),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1509),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1509),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1509),
    [aux_sym_preproc_else_token1] = ACTIONS(1509),
    [anon_sym_LBRACK] = ACTIONS(1509),
    [anon_sym_LBRACE] = ACTIONS(1509),
    [anon_sym_COLON] = ACTIONS(1509),
    [anon_sym_LPAREN] = ACTIONS(1507),
    [anon_sym_SEMI] = ACTIONS(1509),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1509),
    [anon_sym_DASH] = ACTIONS(1507),
    [anon_sym_PLUS] = ACTIONS(1507),
    [anon_sym_STAR] = ACTIONS(1509),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_PERCENT] = ACTIONS(1509),
    [anon_sym_PIPE_PIPE] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1509),
    [anon_sym_PIPE] = ACTIONS(1507),
    [anon_sym_CARET] = ACTIONS(1509),
    [anon_sym_AMP] = ACTIONS(1507),
    [anon_sym_EQ_EQ] = ACTIONS(1509),
    [anon_sym_BANG_EQ] = ACTIONS(1509),
    [anon_sym_LT_GT] = ACTIONS(1509),
    [anon_sym_GT] = ACTIONS(1507),
    [anon_sym_GT_EQ] = ACTIONS(1509),
    [anon_sym_LT_EQ] = ACTIONS(1509),
    [anon_sym_LT] = ACTIONS(1507),
    [anon_sym_LT_LT] = ACTIONS(1509),
    [anon_sym_GT_GT] = ACTIONS(1509),
    [anon_sym_DASH_DASH] = ACTIONS(1509),
    [anon_sym_PLUS_PLUS] = ACTIONS(1509),
    [anon_sym_DOT] = ACTIONS(1507),
    [sym_range_operator] = ACTIONS(1509),
    [anon_sym_DQUOTE] = ACTIONS(1509),
    [anon_sym_SQUOTE] = ACTIONS(1509),
    [sym_number_literal] = ACTIONS(1509),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1509),
  },
  [STATE(80)] = {
    [sym_identifier] = ACTIONS(1511),
    [sym_program_name_keyword] = ACTIONS(1511),
    [sym_module_name_keyword] = ACTIONS(1511),
    [sym_define_device_keyword] = ACTIONS(1511),
    [sym_define_combine_keyword] = ACTIONS(1511),
    [sym_define_constant_keyword] = ACTIONS(1511),
    [sym_define_type_keyword] = ACTIONS(1511),
    [sym_define_variable_keyword] = ACTIONS(1511),
    [sym_define_system_variable_keyword] = ACTIONS(1511),
    [sym_define_connect_level_keyword] = ACTIONS(1511),
    [sym_define_latching_keyword] = ACTIONS(1511),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1511),
    [sym_define_toggling_keyword] = ACTIONS(1511),
    [sym_define_call_keyword] = ACTIONS(1511),
    [sym_define_function_keyword] = ACTIONS(1511),
    [sym_define_library_function_keyword] = ACTIONS(1511),
    [sym_define_start_keyword] = ACTIONS(1511),
    [sym_define_event_keyword] = ACTIONS(1511),
    [sym_define_program_keyword] = ACTIONS(1511),
    [sym_define_module_keyword] = ACTIONS(1511),
    [sym_char_keyword] = ACTIONS(1511),
    [sym_widechar_keyword] = ACTIONS(1511),
    [sym_integer_keyword] = ACTIONS(1511),
    [sym_sinteger_keyword] = ACTIONS(1511),
    [sym_long_keyword] = ACTIONS(1511),
    [sym_slong_keyword] = ACTIONS(1511),
    [sym_float_keyword] = ACTIONS(1511),
    [sym_double_keyword] = ACTIONS(1511),
    [sym_variant_keyword] = ACTIONS(1511),
    [sym_variantarray_keyword] = ACTIONS(1511),
    [sym_local_var_keyword] = ACTIONS(1511),
    [sym_stack_var_keyword] = ACTIONS(1511),
    [sym_constant_keyword] = ACTIONS(1511),
    [sym_volatile_keyword] = ACTIONS(1511),
    [sym_non_volatile_keyword] = ACTIONS(1511),
    [sym_persistent_keyword] = ACTIONS(1511),
    [sym_dev_keyword] = ACTIONS(1511),
    [sym_devlev_keyword] = ACTIONS(1511),
    [sym_devchan_keyword] = ACTIONS(1511),
    [sym_if_keyword] = ACTIONS(1511),
    [sym_else_keyword] = ACTIONS(1511),
    [sym_switch_keyword] = ACTIONS(1511),
    [sym_case_keyword] = ACTIONS(1511),
    [sym_default_keyword] = ACTIONS(1511),
    [sym_while_keyword] = ACTIONS(1511),
    [sym_for_keyword] = ACTIONS(1511),
    [sym_break_keyword] = ACTIONS(1511),
    [sym_continue_keyword] = ACTIONS(1511),
    [sym_return_keyword] = ACTIONS(1511),
    [sym_select_keyword] = ACTIONS(1511),
    [sym_active_keyword] = ACTIONS(1511),
    [sym_struct_keyword] = ACTIONS(1511),
    [sym_structure_keyword] = ACTIONS(1511),
    [sym_button_event_keyword] = ACTIONS(1511),
    [sym_channel_event_keyword] = ACTIONS(1511),
    [sym_level_event_keyword] = ACTIONS(1511),
    [sym_data_event_keyword] = ACTIONS(1511),
    [sym_timeline_event_keyword] = ACTIONS(1511),
    [sym_custom_event_keyword] = ACTIONS(1511),
    [sym_push_keyword] = ACTIONS(1511),
    [sym_release_keyword] = ACTIONS(1511),
    [aux_sym_on_keyword_token1] = ACTIONS(1511),
    [aux_sym_off_keyword_token1] = ACTIONS(1511),
    [sym_send_level_keyword] = ACTIONS(1511),
    [sym_send_string_keyword] = ACTIONS(1511),
    [sym_send_command_keyword] = ACTIONS(1511),
    [sym_clear_buffer_keyword] = ACTIONS(1511),
    [sym_create_buffer_keyword] = ACTIONS(1511),
    [sym_create_multi_buffer_keyword] = ACTIONS(1511),
    [sym_call_keyword] = ACTIONS(1511),
    [sym_system_call_keyword] = ACTIONS(1511),
    [sym_devchan_to_keyword] = ACTIONS(1511),
    [sym_devchan_min_to_keyword] = ACTIONS(1511),
    [sym_devchan_total_off_keyword] = ACTIONS(1511),
    [sym_devchan_pulse_keyword] = ACTIONS(1511),
    [sym_wait_keyword] = ACTIONS(1511),
    [sym_cancel_wait_keyword] = ACTIONS(1511),
    [sym_wait_until_keyword] = ACTIONS(1511),
    [sym_cancel_wait_until_keyword] = ACTIONS(1511),
    [sym_cancel_all_wait_keyword] = ACTIONS(1511),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1511),
    [sym_and] = ACTIONS(1511),
    [sym_or] = ACTIONS(1511),
    [sym_not] = ACTIONS(1511),
    [sym_xor] = ACTIONS(1511),
    [sym_band] = ACTIONS(1511),
    [sym_bor] = ACTIONS(1511),
    [sym_bxor] = ACTIONS(1511),
    [sym_bnot] = ACTIONS(1511),
    [sym_lshift] = ACTIONS(1511),
    [sym_rshift] = ACTIONS(1511),
    [sym___file__] = ACTIONS(1511),
    [sym___date__] = ACTIONS(1511),
    [sym___ldate__] = ACTIONS(1511),
    [sym___time__] = ACTIONS(1511),
    [sym___line__] = ACTIONS(1511),
    [sym___name__] = ACTIONS(1511),
    [sym___netlinx__] = ACTIONS(1511),
    [sym_netlinx_axi_version] = ACTIONS(1511),
    [sym_url_flg_tcp] = ACTIONS(1511),
    [sym_url_flg_accinfopresent] = ACTIONS(1511),
    [sym_url_flg_temp] = ACTIONS(1511),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1511),
    [sym_url_flg_stat_mask] = ACTIONS(1511),
    [sym_url_flg_stat_lookup] = ACTIONS(1511),
    [sym_url_flg_stat_connecting] = ACTIONS(1511),
    [sym_url_flg_stat_waiting] = ACTIONS(1511),
    [sym_url_flg_stat_connected] = ACTIONS(1511),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1511),
    [sym_first_virtual_device] = ACTIONS(1511),
    [sym_first_local_port] = ACTIONS(1511),
    [sym_dynamic_virtual_device] = ACTIONS(1511),
    [sym_timeline_once] = ACTIONS(1511),
    [sym_timeline_repeat] = ACTIONS(1511),
    [sym_timeline_absolute] = ACTIONS(1511),
    [sym_timeline_relative] = ACTIONS(1511),
    [sym_do_push_timed_infinite] = ACTIONS(1511),
    [sym_source_type_no_address] = ACTIONS(1511),
    [sym_source_type_neuron_id] = ACTIONS(1511),
    [sym_source_type_ip_address] = ACTIONS(1511),
    [sym_source_type_axlink] = ACTIONS(1511),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1511),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1511),
    [sym_source_type_ip_socket_address] = ACTIONS(1511),
    [sym_source_type_rs232] = ACTIONS(1511),
    [sym_source_type_internal] = ACTIONS(1511),
    [sym_source_type_mac_address] = ACTIONS(1511),
    [sym_source_type_ipv4_port] = ACTIONS(1511),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1511),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1511),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1511),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1511),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1511),
    [sym_true] = ACTIONS(1511),
    [sym_false] = ACTIONS(1511),
    [sym_file_read_only] = ACTIONS(1511),
    [sym_file_rw_new] = ACTIONS(1511),
    [sym_file_rw_append] = ACTIONS(1511),
    [sym_ip_tcp] = ACTIONS(1511),
    [sym_ip_udp] = ACTIONS(1511),
    [sym_ip_udp_2way] = ACTIONS(1511),
    [sym_xml_encode_types] = ACTIONS(1511),
    [sym_xml_encode_char_as_list] = ACTIONS(1511),
    [sym_xml_encode_le] = ACTIONS(1511),
    [sym_xml_decode_types] = ACTIONS(1511),
    [sym_xml_decode_no_preserve] = ACTIONS(1511),
    [sym_duet_dev_type_display_device] = ACTIONS(1511),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1511),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1511),
    [sym_duet_dev_type_io_device] = ACTIONS(1511),
    [sym_duet_dev_type_relay_device] = ACTIONS(1511),
    [sym_duet_dev_type_ups] = ACTIONS(1511),
    [sym_duet_dev_type_amplifier] = ACTIONS(1511),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1511),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1511),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1511),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1511),
    [sym_duet_dev_type_camera] = ACTIONS(1511),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1511),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1511),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1511),
    [sym_duet_dev_type_dss] = ACTIONS(1511),
    [sym_duet_dev_type_dvr] = ACTIONS(1511),
    [sym_duet_dev_type_disc_device] = ACTIONS(1511),
    [sym_duet_dev_type_document_camera] = ACTIONS(1511),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1511),
    [sym_duet_dev_type_hvac] = ACTIONS(1511),
    [sym_duet_dev_type_keypad] = ACTIONS(1511),
    [sym_duet_dev_type_light] = ACTIONS(1511),
    [sym_duet_dev_type_monitor] = ACTIONS(1511),
    [sym_duet_dev_type_motor] = ACTIONS(1511),
    [sym_duet_dev_type_multi_window] = ACTIONS(1511),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1511),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1511),
    [sym_duet_dev_type_receiver] = ACTIONS(1511),
    [sym_duet_dev_type_security_system] = ACTIONS(1511),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1511),
    [sym_duet_dev_type_settop_box] = ACTIONS(1511),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1511),
    [sym_duet_dev_type_switcher] = ACTIONS(1511),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1511),
    [sym_duet_dev_type_tv] = ACTIONS(1511),
    [sym_duet_dev_type_utility] = ACTIONS(1511),
    [sym_duet_dev_type_vcr] = ACTIONS(1511),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1511),
    [sym_duet_dev_type_video_processor] = ACTIONS(1511),
    [sym_duet_dev_type_video_projector] = ACTIONS(1511),
    [sym_duet_dev_type_video_wall] = ACTIONS(1511),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1511),
    [sym_duet_dev_type_weather] = ACTIONS(1511),
    [sym_duet_dev_not_polled] = ACTIONS(1511),
    [sym_duet_dev_polled] = ACTIONS(1511),
    [sym_clkmgr_mode_network] = ACTIONS(1511),
    [sym_clkmgr_mode_standalone] = ACTIONS(1511),
    [sym_normal_standby] = ACTIONS(1511),
    [sym_normal_wake] = ACTIONS(1511),
    [sym_ip_multicast_ttl_option] = ACTIONS(1511),
    [sym_ip_ttl_subnet] = ACTIONS(1511),
    [sym_ip_ttl_site] = ACTIONS(1511),
    [sym_ip_ttl_region] = ACTIONS(1511),
    [sym_ip_ttl_continent] = ACTIONS(1511),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1511),
    [sym_ip_nodelay_on] = ACTIONS(1511),
    [sym_ip_nodelay_off] = ACTIONS(1511),
    [sym_amx_error] = ACTIONS(1511),
    [sym_amx_warning] = ACTIONS(1511),
    [sym_amx_info] = ACTIONS(1511),
    [sym_amx_debug] = ACTIONS(1511),
    [sym_netlinx_login_success] = ACTIONS(1511),
    [sym_netlinx_login_fail] = ACTIONS(1511),
    [sym_netlinx_logout] = ACTIONS(1511),
    [sym_null_str] = ACTIONS(1511),
    [sym_smtp_address] = ACTIONS(1511),
    [sym_smtp_port_number] = ACTIONS(1511),
    [sym_smtp_username] = ACTIONS(1511),
    [sym_smtp_password] = ACTIONS(1511),
    [sym_smtp_require_tls] = ACTIONS(1511),
    [sym_smtp_from] = ACTIONS(1511),
    [sym_smtp_tls_true] = ACTIONS(1511),
    [sym_smtp_tls_false] = ACTIONS(1511),
    [sym_listview_on_row_select_event] = ACTIONS(1511),
    [sym_data_structure_unknown] = ACTIONS(1511),
    [sym_data_structure_datafeed] = ACTIONS(1511),
    [sym_data_structure_datarecordset] = ACTIONS(1511),
    [sym_data_structure_datarecord] = ACTIONS(1511),
    [sym_data_structure_datafield] = ACTIONS(1511),
    [sym_data_max_content_field_count] = ACTIONS(1511),
    [sym_data_max_metadata_field_count] = ACTIONS(1511),
    [sym_data_max_id_length] = ACTIONS(1511),
    [sym_data_max_name_length] = ACTIONS(1511),
    [sym_data_max_description_length] = ACTIONS(1511),
    [sym_data_max_type_length] = ACTIONS(1511),
    [sym_data_max_format_length] = ACTIONS(1511),
    [sym_data_max_source_length] = ACTIONS(1511),
    [sym_data_max_value_length] = ACTIONS(1511),
    [sym_data_max_label_length] = ACTIONS(1511),
    [sym_data_max_url_length] = ACTIONS(1511),
    [sym_max_last_login_info_length] = ACTIONS(1511),
    [sym_data_type_unknown] = ACTIONS(1511),
    [sym_data_type_string] = ACTIONS(1511),
    [sym_data_type_datetime] = ACTIONS(1511),
    [sym_data_type_date] = ACTIONS(1511),
    [sym_data_type_time] = ACTIONS(1511),
    [sym_data_type_image] = ACTIONS(1511),
    [sym_data_format_url] = ACTIONS(1511),
    [sym_data_format_phone] = ACTIONS(1511),
    [sym_data_format_email] = ACTIONS(1511),
    [sym_data_format_iso8601] = ACTIONS(1511),
    [sym_auditlog_priv] = ACTIONS(1511),
    [sym_configuration_priv] = ACTIONS(1511),
    [sym_deviceconfig_priv] = ACTIONS(1511),
    [sym_ftp_priv] = ACTIONS(1511),
    [sym_http_priv] = ACTIONS(1511),
    [sym_networkconfig_priv] = ACTIONS(1511),
    [sym_programport_priv] = ACTIONS(1511),
    [sym_remoteui_priv] = ACTIONS(1511),
    [sym_securitycontrol_priv] = ACTIONS(1511),
    [sym_softwaremanagement_priv] = ACTIONS(1511),
    [sym_terminal_priv] = ACTIONS(1511),
    [sym_tpadmin_priv] = ACTIONS(1511),
    [sym_user1_priv] = ACTIONS(1511),
    [sym_user2_priv] = ACTIONS(1511),
    [sym_user3_priv] = ACTIONS(1511),
    [sym_user4_priv] = ACTIONS(1511),
    [sym_usermanagement_priv] = ACTIONS(1511),
    [sym_valid_account] = ACTIONS(1511),
    [sym_err_name_invalid] = ACTIONS(1511),
    [sym_err_password_invalid] = ACTIONS(1511),
    [sym_err_invalid_account] = ACTIONS(1511),
    [sym_err_no_authorization] = ACTIONS(1511),
    [sym_err_invalid_parameter] = ACTIONS(1511),
    [sym_err_user_locked_out] = ACTIONS(1511),
    [sym_err_user_account_expired] = ACTIONS(1511),
    [sym_err_general_error] = ACTIONS(1511),
    [sym_err_authentication_error] = ACTIONS(1511),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1511),
    [sym_tls_validate_certificate] = ACTIONS(1511),
    [sym_tdata] = ACTIONS(1511),
    [sym_tchannel] = ACTIONS(1511),
    [sym_tlevel] = ACTIONS(1511),
    [sym_tbutton] = ACTIONS(1511),
    [sym_ttimeline] = ACTIONS(1511),
    [sym_tcustom] = ACTIONS(1511),
    [sym_url_struct] = ACTIONS(1511),
    [sym_dns_struct] = ACTIONS(1511),
    [sym_ip_address_struct] = ACTIONS(1511),
    [sym_dev_info_struct] = ACTIONS(1511),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1511),
    [sym_data_feed] = ACTIONS(1511),
    [sym_data_field] = ACTIONS(1511),
    [sym_data_record] = ACTIONS(1511),
    [sym_wc_data_feed] = ACTIONS(1511),
    [sym_wc_data_field] = ACTIONS(1511),
    [sym_wc_data_record] = ACTIONS(1511),
    [sym_last_login_info] = ACTIONS(1511),
    [sym____reserved___] = ACTIONS(1511),
    [sym_date] = ACTIONS(1511),
    [sym_day] = ACTIONS(1511),
    [sym_time] = ACTIONS(1511),
    [sym_ldate] = ACTIONS(1511),
    [sym_push_device] = ACTIONS(1511),
    [sym_push_channel] = ACTIONS(1511),
    [sym_push_devchan] = ACTIONS(1511),
    [sym_release_device] = ACTIONS(1511),
    [sym_release_channel] = ACTIONS(1511),
    [sym_release_devchan] = ACTIONS(1511),
    [sym_master_slot] = ACTIONS(1511),
    [sym_get_pulse_time] = ACTIONS(1511),
    [sym_get_timer] = ACTIONS(1511),
    [sym_master_sn] = ACTIONS(1511),
    [sym_system_number] = ACTIONS(1511),
    [sym_button] = ACTIONS(1511),
    [sym_dv_channel] = ACTIONS(1511),
    [sym_data] = ACTIONS(1511),
    [sym_level] = ACTIONS(1511),
    [sym_timeline] = ACTIONS(1511),
    [sym_channel] = ACTIONS(1511),
    [sym_custom] = ACTIONS(1511),
    [sym_atoi] = ACTIONS(1511),
    [sym_atol] = ACTIONS(1511),
    [sym_atof] = ACTIONS(1511),
    [sym_itoa] = ACTIONS(1511),
    [sym_itohex] = ACTIONS(1511),
    [sym_hextoi] = ACTIONS(1511),
    [sym_ftoa] = ACTIONS(1511),
    [sym_find_string] = ACTIONS(1511),
    [sym_left_string] = ACTIONS(1511),
    [sym_right_string] = ACTIONS(1511),
    [sym_mid_string] = ACTIONS(1511),
    [sym_remove_string] = ACTIONS(1511),
    [sym_get_buffer_string] = ACTIONS(1511),
    [sym_get_buffer_char] = ACTIONS(1511),
    [sym_get_multi_buffer_string] = ACTIONS(1511),
    [sym_length_string] = ACTIONS(1511),
    [sym_max_length_string] = ACTIONS(1511),
    [sym_set_length_string] = ACTIONS(1511),
    [sym_compare_string] = ACTIONS(1511),
    [sym_upper_string] = ACTIONS(1511),
    [sym_lower_string] = ACTIONS(1511),
    [sym_get_last] = ACTIONS(1511),
    [sym_date_to_day] = ACTIONS(1511),
    [sym_date_to_month] = ACTIONS(1511),
    [sym_date_to_year] = ACTIONS(1511),
    [sym_time_to_hour] = ACTIONS(1511),
    [sym_time_to_minute] = ACTIONS(1511),
    [sym_time_to_second] = ACTIONS(1511),
    [sym_day_of_week] = ACTIONS(1511),
    [sym_set_pulse_time] = ACTIONS(1511),
    [sym_set_timer] = ACTIONS(1511),
    [sym_file_open] = ACTIONS(1511),
    [sym_file_close] = ACTIONS(1511),
    [sym_file_read] = ACTIONS(1511),
    [sym_file_read_line] = ACTIONS(1511),
    [sym_file_write] = ACTIONS(1511),
    [sym_file_write_line] = ACTIONS(1511),
    [sym_file_copy] = ACTIONS(1511),
    [sym_file_delete] = ACTIONS(1511),
    [sym_file_rename] = ACTIONS(1511),
    [sym_file_dir] = ACTIONS(1511),
    [sym_file_getdir] = ACTIONS(1511),
    [sym_file_setdir] = ACTIONS(1511),
    [sym_file_create_dir] = ACTIONS(1511),
    [sym_file_remove_dir] = ACTIONS(1511),
    [sym_file_seek] = ACTIONS(1511),
    [sym_length_array] = ACTIONS(1511),
    [sym_max_length_array] = ACTIONS(1511),
    [sym_set_length_array] = ACTIONS(1511),
    [sym_ip_client_open] = ACTIONS(1511),
    [sym_ip_client_close] = ACTIONS(1511),
    [sym_ip_server_open] = ACTIONS(1511),
    [sym_ip_server_close] = ACTIONS(1511),
    [sym_ip_bound_client_open] = ACTIONS(1511),
    [sym_ip_mc_server_open] = ACTIONS(1511),
    [sym_ip_set_option] = ACTIONS(1511),
    [sym_ssh_client_open] = ACTIONS(1511),
    [sym_ssh_client_close] = ACTIONS(1511),
    [sym_tls_client_open] = ACTIONS(1511),
    [sym_tls_client_close] = ACTIONS(1511),
    [sym_do_push] = ACTIONS(1511),
    [sym_do_push_timed] = ACTIONS(1511),
    [sym_do_release] = ACTIONS(1511),
    [sym_device_id] = ACTIONS(1511),
    [sym_device_info] = ACTIONS(1511),
    [sym_device_id_string] = ACTIONS(1511),
    [sym_device_standby] = ACTIONS(1511),
    [sym_device_wake] = ACTIONS(1511),
    [sym_timeline_create] = ACTIONS(1511),
    [sym_timeline_kill] = ACTIONS(1511),
    [sym_timeline_reload] = ACTIONS(1511),
    [sym_timeline_pause] = ACTIONS(1511),
    [sym_timeline_restart] = ACTIONS(1511),
    [sym_timeline_set] = ACTIONS(1511),
    [sym_timeline_get] = ACTIONS(1511),
    [sym_timeline_active] = ACTIONS(1511),
    [sym_timeline_dynamic_id] = ACTIONS(1511),
    [sym_variable_to_string] = ACTIONS(1511),
    [sym_string_to_variable] = ACTIONS(1511),
    [sym_length_variable_to_string] = ACTIONS(1511),
    [sym_type_cast] = ACTIONS(1511),
    [sym_variable_to_xml] = ACTIONS(1511),
    [sym_xml_to_variable] = ACTIONS(1511),
    [sym_length_variable_to_xml] = ACTIONS(1511),
    [sym_redirect_string] = ACTIONS(1511),
    [sym_random_number] = ACTIONS(1511),
    [sym_get_unique_id] = ACTIONS(1511),
    [sym_get_system_number] = ACTIONS(1511),
    [sym_set_system_number] = ACTIONS(1511),
    [sym_get_serial_number] = ACTIONS(1511),
    [sym_get_master_build] = ACTIONS(1511),
    [sym_set_validation_code] = ACTIONS(1511),
    [sym_reboot] = ACTIONS(1511),
    [sym_load_duet_module] = ACTIONS(1511),
    [sym_unload_duet_module] = ACTIONS(1511),
    [sym_duet_mem_size_set] = ACTIONS(1511),
    [sym_duet_mem_size_get] = ACTIONS(1511),
    [sym_dynamic_application_device] = ACTIONS(1511),
    [sym_dynamic_polled_port] = ACTIONS(1511),
    [sym_static_port_binding] = ACTIONS(1511),
    [sym_static_ip_binding] = ACTIONS(1511),
    [sym_get_dns_list] = ACTIONS(1511),
    [sym_set_dns_list] = ACTIONS(1511),
    [sym_get_ip_address] = ACTIONS(1511),
    [sym_set_ip_address] = ACTIONS(1511),
    [sym_get_url_list] = ACTIONS(1511),
    [sym_add_url_entry] = ACTIONS(1511),
    [sym_delete_url_entry] = ACTIONS(1511),
    [sym_abs_value] = ACTIONS(1511),
    [sym_max_value] = ACTIONS(1511),
    [sym_min_value] = ACTIONS(1511),
    [sym_exp_value] = ACTIONS(1511),
    [sym_log_value] = ACTIONS(1511),
    [sym_log10_value] = ACTIONS(1511),
    [sym_power_value] = ACTIONS(1511),
    [sym_sqrt_value] = ACTIONS(1511),
    [sym_raw_be] = ACTIONS(1511),
    [sym_raw_le] = ACTIONS(1511),
    [sym_internal_queue_size_set] = ACTIONS(1511),
    [sym_internal_queue_size_get] = ACTIONS(1511),
    [sym_internal_threshold_set] = ACTIONS(1511),
    [sym_internal_threshold_get] = ACTIONS(1511),
    [sym_rebuilt_event] = ACTIONS(1511),
    [sym_do_custom_event] = ACTIONS(1511),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1511),
    [sym_clkmgr_set_clk_source] = ACTIONS(1511),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1511),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1511),
    [sym_clkmgr_get_timezone] = ACTIONS(1511),
    [sym_clkmgr_set_timezone] = ACTIONS(1511),
    [sym_clkmgr_get_resync_period] = ACTIONS(1511),
    [sym_clkmgr_set_resync_period] = ACTIONS(1511),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1511),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1511),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1511),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1511),
    [sym_clkmgr_get_timeservers] = ACTIONS(1511),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1511),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1511),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1511),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1511),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1511),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1511),
    [sym_set_log_level] = ACTIONS(1511),
    [sym_get_log_level] = ACTIONS(1511),
    [sym_amx_log] = ACTIONS(1511),
    [sym_smtp_server_config_set] = ACTIONS(1511),
    [sym_smtp_server_config_get] = ACTIONS(1511),
    [sym_smtp_send] = ACTIONS(1511),
    [sym_validate_netlinx_account] = ACTIONS(1511),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1511),
    [sym_authenticate_certificate] = ACTIONS(1511),
    [sym_audit_netlinx_session_event] = ACTIONS(1511),
    [sym_audit_netlinx_generic_event] = ACTIONS(1511),
    [sym_get_available_flash_disk_space] = ACTIONS(1511),
    [sym_get_max_flash_disk_space] = ACTIONS(1511),
    [sym_data_create_feed] = ACTIONS(1511),
    [sym_data_delete_feed] = ACTIONS(1511),
    [sym_data_publish_feed] = ACTIONS(1511),
    [sym_data_get_published_feed] = ACTIONS(1511),
    [sym_data_add_record] = ACTIONS(1511),
    [sym_data_get_event_record] = ACTIONS(1511),
    [sym__wc_data_create_feed] = ACTIONS(1511),
    [sym__wc_data_add_record] = ACTIONS(1511),
    [sym__wc_data_get_event_record] = ACTIONS(1511),
    [sym_format] = ACTIONS(1511),
    [sym_astro_clock] = ACTIONS(1511),
    [sym_set_outdoor_temperature] = ACTIONS(1511),
    [sym_set_virtual_level_count] = ACTIONS(1511),
    [sym_set_virtual_channel_count] = ACTIONS(1511),
    [sym_set_virtual_port_count] = ACTIONS(1511),
    [sym__wc_encode_int] = ACTIONS(1511),
    [sym__wc_decode_int] = ACTIONS(1511),
    [sym__wc_int] = ACTIONS(1511),
    [sym__wc_to_ch_int] = ACTIONS(1511),
    [sym__ch_to_wc_int] = ACTIONS(1511),
    [sym__wc_find_string_int] = ACTIONS(1511),
    [sym__wc_left_string_int] = ACTIONS(1511),
    [sym__wc_lower_string_int] = ACTIONS(1511),
    [sym__wc_mid_string_int] = ACTIONS(1511),
    [sym__wc_remove_string_int] = ACTIONS(1511),
    [sym__wc_right_string_int] = ACTIONS(1511),
    [sym__wc_upper_string_int] = ACTIONS(1511),
    [sym__wc_compare_string_int] = ACTIONS(1511),
    [sym__wc_get_buffer_char_int] = ACTIONS(1511),
    [sym__wc_get_buffer_string_int] = ACTIONS(1511),
    [sym__wc_concat_string_int] = ACTIONS(1511),
    [sym__wc_file_open_int] = ACTIONS(1511),
    [sym__wc_file_close_int] = ACTIONS(1511),
    [sym__wc_file_read_int] = ACTIONS(1511),
    [sym__wc_file_read_line_int] = ACTIONS(1511),
    [sym__wc_file_write_int] = ACTIONS(1511),
    [sym__wc_file_write_line_int] = ACTIONS(1511),
    [anon_sym_EQ] = ACTIONS(1511),
    [aux_sym_preproc_include_token1] = ACTIONS(1511),
    [aux_sym_preproc_define_token1] = ACTIONS(1513),
    [aux_sym_preproc_warn_token1] = ACTIONS(1513),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1513),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1513),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1513),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1513),
    [aux_sym_preproc_else_token1] = ACTIONS(1513),
    [anon_sym_LBRACK] = ACTIONS(1513),
    [anon_sym_LBRACE] = ACTIONS(1513),
    [anon_sym_COLON] = ACTIONS(1513),
    [anon_sym_LPAREN] = ACTIONS(1511),
    [anon_sym_SEMI] = ACTIONS(1513),
    [anon_sym_BANG] = ACTIONS(1511),
    [anon_sym_TILDE] = ACTIONS(1513),
    [anon_sym_DASH] = ACTIONS(1511),
    [anon_sym_PLUS] = ACTIONS(1511),
    [anon_sym_STAR] = ACTIONS(1513),
    [anon_sym_SLASH] = ACTIONS(1511),
    [anon_sym_PERCENT] = ACTIONS(1513),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_AMP_AMP] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1511),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_AMP] = ACTIONS(1511),
    [anon_sym_EQ_EQ] = ACTIONS(1513),
    [anon_sym_BANG_EQ] = ACTIONS(1513),
    [anon_sym_LT_GT] = ACTIONS(1513),
    [anon_sym_GT] = ACTIONS(1511),
    [anon_sym_GT_EQ] = ACTIONS(1513),
    [anon_sym_LT_EQ] = ACTIONS(1513),
    [anon_sym_LT] = ACTIONS(1511),
    [anon_sym_LT_LT] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1513),
    [anon_sym_DASH_DASH] = ACTIONS(1513),
    [anon_sym_PLUS_PLUS] = ACTIONS(1513),
    [anon_sym_DOT] = ACTIONS(1511),
    [anon_sym_DQUOTE] = ACTIONS(1513),
    [anon_sym_SQUOTE] = ACTIONS(1513),
    [sym_number_literal] = ACTIONS(1513),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1513),
  },
  [STATE(81)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1415),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1427),
    [sym_rshift] = ACTIONS(1427),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1451),
    [anon_sym_GT_EQ] = ACTIONS(1453),
    [anon_sym_LT_EQ] = ACTIONS(1453),
    [anon_sym_LT] = ACTIONS(1451),
    [anon_sym_LT_LT] = ACTIONS(1455),
    [anon_sym_GT_GT] = ACTIONS(1455),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(82)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1415),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1427),
    [sym_rshift] = ACTIONS(1427),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1455),
    [anon_sym_GT_GT] = ACTIONS(1455),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(83)] = {
    [aux_sym_subscript_expression_repeat1] = STATE(83),
    [ts_builtin_sym_end] = ACTIONS(1498),
    [sym_identifier] = ACTIONS(1496),
    [sym_program_name_keyword] = ACTIONS(1496),
    [sym_module_name_keyword] = ACTIONS(1496),
    [sym_define_device_keyword] = ACTIONS(1496),
    [sym_define_combine_keyword] = ACTIONS(1496),
    [sym_define_constant_keyword] = ACTIONS(1496),
    [sym_define_type_keyword] = ACTIONS(1496),
    [sym_define_variable_keyword] = ACTIONS(1496),
    [sym_define_system_variable_keyword] = ACTIONS(1496),
    [sym_define_connect_level_keyword] = ACTIONS(1496),
    [sym_define_latching_keyword] = ACTIONS(1496),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1496),
    [sym_define_toggling_keyword] = ACTIONS(1496),
    [sym_define_call_keyword] = ACTIONS(1496),
    [sym_define_function_keyword] = ACTIONS(1496),
    [sym_define_library_function_keyword] = ACTIONS(1496),
    [sym_define_start_keyword] = ACTIONS(1496),
    [sym_define_event_keyword] = ACTIONS(1496),
    [sym_define_program_keyword] = ACTIONS(1496),
    [sym_define_module_keyword] = ACTIONS(1496),
    [sym_char_keyword] = ACTIONS(1496),
    [sym_widechar_keyword] = ACTIONS(1496),
    [sym_integer_keyword] = ACTIONS(1496),
    [sym_sinteger_keyword] = ACTIONS(1496),
    [sym_long_keyword] = ACTIONS(1496),
    [sym_slong_keyword] = ACTIONS(1496),
    [sym_float_keyword] = ACTIONS(1496),
    [sym_double_keyword] = ACTIONS(1496),
    [sym_variant_keyword] = ACTIONS(1496),
    [sym_variantarray_keyword] = ACTIONS(1496),
    [sym_local_var_keyword] = ACTIONS(1496),
    [sym_stack_var_keyword] = ACTIONS(1496),
    [sym_constant_keyword] = ACTIONS(1496),
    [sym_volatile_keyword] = ACTIONS(1496),
    [sym_non_volatile_keyword] = ACTIONS(1496),
    [sym_persistent_keyword] = ACTIONS(1496),
    [sym_dev_keyword] = ACTIONS(1496),
    [sym_devlev_keyword] = ACTIONS(1496),
    [sym_devchan_keyword] = ACTIONS(1496),
    [sym_if_keyword] = ACTIONS(1496),
    [sym_else_keyword] = ACTIONS(1496),
    [sym_switch_keyword] = ACTIONS(1496),
    [sym_case_keyword] = ACTIONS(1496),
    [sym_default_keyword] = ACTIONS(1496),
    [sym_while_keyword] = ACTIONS(1496),
    [sym_for_keyword] = ACTIONS(1496),
    [sym_break_keyword] = ACTIONS(1496),
    [sym_continue_keyword] = ACTIONS(1496),
    [sym_return_keyword] = ACTIONS(1496),
    [sym_select_keyword] = ACTIONS(1496),
    [sym_active_keyword] = ACTIONS(1496),
    [sym_struct_keyword] = ACTIONS(1496),
    [sym_structure_keyword] = ACTIONS(1496),
    [sym_button_event_keyword] = ACTIONS(1496),
    [sym_channel_event_keyword] = ACTIONS(1496),
    [sym_level_event_keyword] = ACTIONS(1496),
    [sym_data_event_keyword] = ACTIONS(1496),
    [sym_timeline_event_keyword] = ACTIONS(1496),
    [sym_custom_event_keyword] = ACTIONS(1496),
    [sym_push_keyword] = ACTIONS(1496),
    [sym_release_keyword] = ACTIONS(1496),
    [aux_sym_on_keyword_token1] = ACTIONS(1496),
    [aux_sym_off_keyword_token1] = ACTIONS(1496),
    [sym_send_level_keyword] = ACTIONS(1496),
    [sym_send_string_keyword] = ACTIONS(1496),
    [sym_send_command_keyword] = ACTIONS(1496),
    [sym_clear_buffer_keyword] = ACTIONS(1496),
    [sym_create_buffer_keyword] = ACTIONS(1496),
    [sym_create_multi_buffer_keyword] = ACTIONS(1496),
    [sym_call_keyword] = ACTIONS(1496),
    [sym_system_call_keyword] = ACTIONS(1496),
    [sym_devchan_to_keyword] = ACTIONS(1496),
    [sym_devchan_min_to_keyword] = ACTIONS(1496),
    [sym_devchan_total_off_keyword] = ACTIONS(1496),
    [sym_devchan_pulse_keyword] = ACTIONS(1496),
    [sym_wait_keyword] = ACTIONS(1496),
    [sym_cancel_wait_keyword] = ACTIONS(1496),
    [sym_wait_until_keyword] = ACTIONS(1496),
    [sym_cancel_wait_until_keyword] = ACTIONS(1496),
    [sym_cancel_all_wait_keyword] = ACTIONS(1496),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1496),
    [sym_and] = ACTIONS(1496),
    [sym_or] = ACTIONS(1496),
    [sym_not] = ACTIONS(1496),
    [sym_xor] = ACTIONS(1496),
    [sym_band] = ACTIONS(1496),
    [sym_bor] = ACTIONS(1496),
    [sym_bxor] = ACTIONS(1496),
    [sym_bnot] = ACTIONS(1496),
    [sym_lshift] = ACTIONS(1496),
    [sym_rshift] = ACTIONS(1496),
    [sym___file__] = ACTIONS(1496),
    [sym___date__] = ACTIONS(1496),
    [sym___ldate__] = ACTIONS(1496),
    [sym___time__] = ACTIONS(1496),
    [sym___line__] = ACTIONS(1496),
    [sym___name__] = ACTIONS(1496),
    [sym___netlinx__] = ACTIONS(1496),
    [sym_netlinx_axi_version] = ACTIONS(1496),
    [sym_url_flg_tcp] = ACTIONS(1496),
    [sym_url_flg_accinfopresent] = ACTIONS(1496),
    [sym_url_flg_temp] = ACTIONS(1496),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1496),
    [sym_url_flg_stat_mask] = ACTIONS(1496),
    [sym_url_flg_stat_lookup] = ACTIONS(1496),
    [sym_url_flg_stat_connecting] = ACTIONS(1496),
    [sym_url_flg_stat_waiting] = ACTIONS(1496),
    [sym_url_flg_stat_connected] = ACTIONS(1496),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1496),
    [sym_first_virtual_device] = ACTIONS(1496),
    [sym_first_local_port] = ACTIONS(1496),
    [sym_dynamic_virtual_device] = ACTIONS(1496),
    [sym_timeline_once] = ACTIONS(1496),
    [sym_timeline_repeat] = ACTIONS(1496),
    [sym_timeline_absolute] = ACTIONS(1496),
    [sym_timeline_relative] = ACTIONS(1496),
    [sym_do_push_timed_infinite] = ACTIONS(1496),
    [sym_source_type_no_address] = ACTIONS(1496),
    [sym_source_type_neuron_id] = ACTIONS(1496),
    [sym_source_type_ip_address] = ACTIONS(1496),
    [sym_source_type_axlink] = ACTIONS(1496),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1496),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1496),
    [sym_source_type_ip_socket_address] = ACTIONS(1496),
    [sym_source_type_rs232] = ACTIONS(1496),
    [sym_source_type_internal] = ACTIONS(1496),
    [sym_source_type_mac_address] = ACTIONS(1496),
    [sym_source_type_ipv4_port] = ACTIONS(1496),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1496),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1496),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1496),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1496),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1496),
    [sym_true] = ACTIONS(1496),
    [sym_false] = ACTIONS(1496),
    [sym_file_read_only] = ACTIONS(1496),
    [sym_file_rw_new] = ACTIONS(1496),
    [sym_file_rw_append] = ACTIONS(1496),
    [sym_ip_tcp] = ACTIONS(1496),
    [sym_ip_udp] = ACTIONS(1496),
    [sym_ip_udp_2way] = ACTIONS(1496),
    [sym_xml_encode_types] = ACTIONS(1496),
    [sym_xml_encode_char_as_list] = ACTIONS(1496),
    [sym_xml_encode_le] = ACTIONS(1496),
    [sym_xml_decode_types] = ACTIONS(1496),
    [sym_xml_decode_no_preserve] = ACTIONS(1496),
    [sym_duet_dev_type_display_device] = ACTIONS(1496),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1496),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1496),
    [sym_duet_dev_type_io_device] = ACTIONS(1496),
    [sym_duet_dev_type_relay_device] = ACTIONS(1496),
    [sym_duet_dev_type_ups] = ACTIONS(1496),
    [sym_duet_dev_type_amplifier] = ACTIONS(1496),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1496),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1496),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1496),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1496),
    [sym_duet_dev_type_camera] = ACTIONS(1496),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1496),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1496),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1496),
    [sym_duet_dev_type_dss] = ACTIONS(1496),
    [sym_duet_dev_type_dvr] = ACTIONS(1496),
    [sym_duet_dev_type_disc_device] = ACTIONS(1496),
    [sym_duet_dev_type_document_camera] = ACTIONS(1496),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1496),
    [sym_duet_dev_type_hvac] = ACTIONS(1496),
    [sym_duet_dev_type_keypad] = ACTIONS(1496),
    [sym_duet_dev_type_light] = ACTIONS(1496),
    [sym_duet_dev_type_monitor] = ACTIONS(1496),
    [sym_duet_dev_type_motor] = ACTIONS(1496),
    [sym_duet_dev_type_multi_window] = ACTIONS(1496),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1496),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1496),
    [sym_duet_dev_type_receiver] = ACTIONS(1496),
    [sym_duet_dev_type_security_system] = ACTIONS(1496),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1496),
    [sym_duet_dev_type_settop_box] = ACTIONS(1496),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1496),
    [sym_duet_dev_type_switcher] = ACTIONS(1496),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1496),
    [sym_duet_dev_type_tv] = ACTIONS(1496),
    [sym_duet_dev_type_utility] = ACTIONS(1496),
    [sym_duet_dev_type_vcr] = ACTIONS(1496),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1496),
    [sym_duet_dev_type_video_processor] = ACTIONS(1496),
    [sym_duet_dev_type_video_projector] = ACTIONS(1496),
    [sym_duet_dev_type_video_wall] = ACTIONS(1496),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1496),
    [sym_duet_dev_type_weather] = ACTIONS(1496),
    [sym_duet_dev_not_polled] = ACTIONS(1496),
    [sym_duet_dev_polled] = ACTIONS(1496),
    [sym_clkmgr_mode_network] = ACTIONS(1496),
    [sym_clkmgr_mode_standalone] = ACTIONS(1496),
    [sym_normal_standby] = ACTIONS(1496),
    [sym_normal_wake] = ACTIONS(1496),
    [sym_ip_multicast_ttl_option] = ACTIONS(1496),
    [sym_ip_ttl_subnet] = ACTIONS(1496),
    [sym_ip_ttl_site] = ACTIONS(1496),
    [sym_ip_ttl_region] = ACTIONS(1496),
    [sym_ip_ttl_continent] = ACTIONS(1496),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1496),
    [sym_ip_nodelay_on] = ACTIONS(1496),
    [sym_ip_nodelay_off] = ACTIONS(1496),
    [sym_amx_error] = ACTIONS(1496),
    [sym_amx_warning] = ACTIONS(1496),
    [sym_amx_info] = ACTIONS(1496),
    [sym_amx_debug] = ACTIONS(1496),
    [sym_netlinx_login_success] = ACTIONS(1496),
    [sym_netlinx_login_fail] = ACTIONS(1496),
    [sym_netlinx_logout] = ACTIONS(1496),
    [sym_null_str] = ACTIONS(1496),
    [sym_smtp_address] = ACTIONS(1496),
    [sym_smtp_port_number] = ACTIONS(1496),
    [sym_smtp_username] = ACTIONS(1496),
    [sym_smtp_password] = ACTIONS(1496),
    [sym_smtp_require_tls] = ACTIONS(1496),
    [sym_smtp_from] = ACTIONS(1496),
    [sym_smtp_tls_true] = ACTIONS(1496),
    [sym_smtp_tls_false] = ACTIONS(1496),
    [sym_listview_on_row_select_event] = ACTIONS(1496),
    [sym_data_structure_unknown] = ACTIONS(1496),
    [sym_data_structure_datafeed] = ACTIONS(1496),
    [sym_data_structure_datarecordset] = ACTIONS(1496),
    [sym_data_structure_datarecord] = ACTIONS(1496),
    [sym_data_structure_datafield] = ACTIONS(1496),
    [sym_data_max_content_field_count] = ACTIONS(1496),
    [sym_data_max_metadata_field_count] = ACTIONS(1496),
    [sym_data_max_id_length] = ACTIONS(1496),
    [sym_data_max_name_length] = ACTIONS(1496),
    [sym_data_max_description_length] = ACTIONS(1496),
    [sym_data_max_type_length] = ACTIONS(1496),
    [sym_data_max_format_length] = ACTIONS(1496),
    [sym_data_max_source_length] = ACTIONS(1496),
    [sym_data_max_value_length] = ACTIONS(1496),
    [sym_data_max_label_length] = ACTIONS(1496),
    [sym_data_max_url_length] = ACTIONS(1496),
    [sym_max_last_login_info_length] = ACTIONS(1496),
    [sym_data_type_unknown] = ACTIONS(1496),
    [sym_data_type_string] = ACTIONS(1496),
    [sym_data_type_datetime] = ACTIONS(1496),
    [sym_data_type_date] = ACTIONS(1496),
    [sym_data_type_time] = ACTIONS(1496),
    [sym_data_type_image] = ACTIONS(1496),
    [sym_data_format_url] = ACTIONS(1496),
    [sym_data_format_phone] = ACTIONS(1496),
    [sym_data_format_email] = ACTIONS(1496),
    [sym_data_format_iso8601] = ACTIONS(1496),
    [sym_auditlog_priv] = ACTIONS(1496),
    [sym_configuration_priv] = ACTIONS(1496),
    [sym_deviceconfig_priv] = ACTIONS(1496),
    [sym_ftp_priv] = ACTIONS(1496),
    [sym_http_priv] = ACTIONS(1496),
    [sym_networkconfig_priv] = ACTIONS(1496),
    [sym_programport_priv] = ACTIONS(1496),
    [sym_remoteui_priv] = ACTIONS(1496),
    [sym_securitycontrol_priv] = ACTIONS(1496),
    [sym_softwaremanagement_priv] = ACTIONS(1496),
    [sym_terminal_priv] = ACTIONS(1496),
    [sym_tpadmin_priv] = ACTIONS(1496),
    [sym_user1_priv] = ACTIONS(1496),
    [sym_user2_priv] = ACTIONS(1496),
    [sym_user3_priv] = ACTIONS(1496),
    [sym_user4_priv] = ACTIONS(1496),
    [sym_usermanagement_priv] = ACTIONS(1496),
    [sym_valid_account] = ACTIONS(1496),
    [sym_err_name_invalid] = ACTIONS(1496),
    [sym_err_password_invalid] = ACTIONS(1496),
    [sym_err_invalid_account] = ACTIONS(1496),
    [sym_err_no_authorization] = ACTIONS(1496),
    [sym_err_invalid_parameter] = ACTIONS(1496),
    [sym_err_user_locked_out] = ACTIONS(1496),
    [sym_err_user_account_expired] = ACTIONS(1496),
    [sym_err_general_error] = ACTIONS(1496),
    [sym_err_authentication_error] = ACTIONS(1496),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1496),
    [sym_tls_validate_certificate] = ACTIONS(1496),
    [sym_tdata] = ACTIONS(1496),
    [sym_tchannel] = ACTIONS(1496),
    [sym_tlevel] = ACTIONS(1496),
    [sym_tbutton] = ACTIONS(1496),
    [sym_ttimeline] = ACTIONS(1496),
    [sym_tcustom] = ACTIONS(1496),
    [sym_url_struct] = ACTIONS(1496),
    [sym_dns_struct] = ACTIONS(1496),
    [sym_ip_address_struct] = ACTIONS(1496),
    [sym_dev_info_struct] = ACTIONS(1496),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1496),
    [sym_data_feed] = ACTIONS(1496),
    [sym_data_field] = ACTIONS(1496),
    [sym_data_record] = ACTIONS(1496),
    [sym_wc_data_feed] = ACTIONS(1496),
    [sym_wc_data_field] = ACTIONS(1496),
    [sym_wc_data_record] = ACTIONS(1496),
    [sym_last_login_info] = ACTIONS(1496),
    [sym____reserved___] = ACTIONS(1496),
    [sym_date] = ACTIONS(1496),
    [sym_day] = ACTIONS(1496),
    [sym_time] = ACTIONS(1496),
    [sym_ldate] = ACTIONS(1496),
    [sym_push_device] = ACTIONS(1496),
    [sym_push_channel] = ACTIONS(1496),
    [sym_push_devchan] = ACTIONS(1496),
    [sym_release_device] = ACTIONS(1496),
    [sym_release_channel] = ACTIONS(1496),
    [sym_release_devchan] = ACTIONS(1496),
    [sym_master_slot] = ACTIONS(1496),
    [sym_get_pulse_time] = ACTIONS(1496),
    [sym_get_timer] = ACTIONS(1496),
    [sym_master_sn] = ACTIONS(1496),
    [sym_system_number] = ACTIONS(1496),
    [sym_button] = ACTIONS(1496),
    [sym_dv_channel] = ACTIONS(1496),
    [sym_data] = ACTIONS(1496),
    [sym_level] = ACTIONS(1496),
    [sym_timeline] = ACTIONS(1496),
    [sym_channel] = ACTIONS(1496),
    [sym_custom] = ACTIONS(1496),
    [sym_atoi] = ACTIONS(1496),
    [sym_atol] = ACTIONS(1496),
    [sym_atof] = ACTIONS(1496),
    [sym_itoa] = ACTIONS(1496),
    [sym_itohex] = ACTIONS(1496),
    [sym_hextoi] = ACTIONS(1496),
    [sym_ftoa] = ACTIONS(1496),
    [sym_find_string] = ACTIONS(1496),
    [sym_left_string] = ACTIONS(1496),
    [sym_right_string] = ACTIONS(1496),
    [sym_mid_string] = ACTIONS(1496),
    [sym_remove_string] = ACTIONS(1496),
    [sym_get_buffer_string] = ACTIONS(1496),
    [sym_get_buffer_char] = ACTIONS(1496),
    [sym_get_multi_buffer_string] = ACTIONS(1496),
    [sym_length_string] = ACTIONS(1496),
    [sym_max_length_string] = ACTIONS(1496),
    [sym_set_length_string] = ACTIONS(1496),
    [sym_compare_string] = ACTIONS(1496),
    [sym_upper_string] = ACTIONS(1496),
    [sym_lower_string] = ACTIONS(1496),
    [sym_get_last] = ACTIONS(1496),
    [sym_date_to_day] = ACTIONS(1496),
    [sym_date_to_month] = ACTIONS(1496),
    [sym_date_to_year] = ACTIONS(1496),
    [sym_time_to_hour] = ACTIONS(1496),
    [sym_time_to_minute] = ACTIONS(1496),
    [sym_time_to_second] = ACTIONS(1496),
    [sym_day_of_week] = ACTIONS(1496),
    [sym_set_pulse_time] = ACTIONS(1496),
    [sym_set_timer] = ACTIONS(1496),
    [sym_file_open] = ACTIONS(1496),
    [sym_file_close] = ACTIONS(1496),
    [sym_file_read] = ACTIONS(1496),
    [sym_file_read_line] = ACTIONS(1496),
    [sym_file_write] = ACTIONS(1496),
    [sym_file_write_line] = ACTIONS(1496),
    [sym_file_copy] = ACTIONS(1496),
    [sym_file_delete] = ACTIONS(1496),
    [sym_file_rename] = ACTIONS(1496),
    [sym_file_dir] = ACTIONS(1496),
    [sym_file_getdir] = ACTIONS(1496),
    [sym_file_setdir] = ACTIONS(1496),
    [sym_file_create_dir] = ACTIONS(1496),
    [sym_file_remove_dir] = ACTIONS(1496),
    [sym_file_seek] = ACTIONS(1496),
    [sym_length_array] = ACTIONS(1496),
    [sym_max_length_array] = ACTIONS(1496),
    [sym_set_length_array] = ACTIONS(1496),
    [sym_ip_client_open] = ACTIONS(1496),
    [sym_ip_client_close] = ACTIONS(1496),
    [sym_ip_server_open] = ACTIONS(1496),
    [sym_ip_server_close] = ACTIONS(1496),
    [sym_ip_bound_client_open] = ACTIONS(1496),
    [sym_ip_mc_server_open] = ACTIONS(1496),
    [sym_ip_set_option] = ACTIONS(1496),
    [sym_ssh_client_open] = ACTIONS(1496),
    [sym_ssh_client_close] = ACTIONS(1496),
    [sym_tls_client_open] = ACTIONS(1496),
    [sym_tls_client_close] = ACTIONS(1496),
    [sym_do_push] = ACTIONS(1496),
    [sym_do_push_timed] = ACTIONS(1496),
    [sym_do_release] = ACTIONS(1496),
    [sym_device_id] = ACTIONS(1496),
    [sym_device_info] = ACTIONS(1496),
    [sym_device_id_string] = ACTIONS(1496),
    [sym_device_standby] = ACTIONS(1496),
    [sym_device_wake] = ACTIONS(1496),
    [sym_timeline_create] = ACTIONS(1496),
    [sym_timeline_kill] = ACTIONS(1496),
    [sym_timeline_reload] = ACTIONS(1496),
    [sym_timeline_pause] = ACTIONS(1496),
    [sym_timeline_restart] = ACTIONS(1496),
    [sym_timeline_set] = ACTIONS(1496),
    [sym_timeline_get] = ACTIONS(1496),
    [sym_timeline_active] = ACTIONS(1496),
    [sym_timeline_dynamic_id] = ACTIONS(1496),
    [sym_variable_to_string] = ACTIONS(1496),
    [sym_string_to_variable] = ACTIONS(1496),
    [sym_length_variable_to_string] = ACTIONS(1496),
    [sym_type_cast] = ACTIONS(1496),
    [sym_variable_to_xml] = ACTIONS(1496),
    [sym_xml_to_variable] = ACTIONS(1496),
    [sym_length_variable_to_xml] = ACTIONS(1496),
    [sym_redirect_string] = ACTIONS(1496),
    [sym_random_number] = ACTIONS(1496),
    [sym_get_unique_id] = ACTIONS(1496),
    [sym_get_system_number] = ACTIONS(1496),
    [sym_set_system_number] = ACTIONS(1496),
    [sym_get_serial_number] = ACTIONS(1496),
    [sym_get_master_build] = ACTIONS(1496),
    [sym_set_validation_code] = ACTIONS(1496),
    [sym_reboot] = ACTIONS(1496),
    [sym_load_duet_module] = ACTIONS(1496),
    [sym_unload_duet_module] = ACTIONS(1496),
    [sym_duet_mem_size_set] = ACTIONS(1496),
    [sym_duet_mem_size_get] = ACTIONS(1496),
    [sym_dynamic_application_device] = ACTIONS(1496),
    [sym_dynamic_polled_port] = ACTIONS(1496),
    [sym_static_port_binding] = ACTIONS(1496),
    [sym_static_ip_binding] = ACTIONS(1496),
    [sym_get_dns_list] = ACTIONS(1496),
    [sym_set_dns_list] = ACTIONS(1496),
    [sym_get_ip_address] = ACTIONS(1496),
    [sym_set_ip_address] = ACTIONS(1496),
    [sym_get_url_list] = ACTIONS(1496),
    [sym_add_url_entry] = ACTIONS(1496),
    [sym_delete_url_entry] = ACTIONS(1496),
    [sym_abs_value] = ACTIONS(1496),
    [sym_max_value] = ACTIONS(1496),
    [sym_min_value] = ACTIONS(1496),
    [sym_exp_value] = ACTIONS(1496),
    [sym_log_value] = ACTIONS(1496),
    [sym_log10_value] = ACTIONS(1496),
    [sym_power_value] = ACTIONS(1496),
    [sym_sqrt_value] = ACTIONS(1496),
    [sym_raw_be] = ACTIONS(1496),
    [sym_raw_le] = ACTIONS(1496),
    [sym_internal_queue_size_set] = ACTIONS(1496),
    [sym_internal_queue_size_get] = ACTIONS(1496),
    [sym_internal_threshold_set] = ACTIONS(1496),
    [sym_internal_threshold_get] = ACTIONS(1496),
    [sym_rebuilt_event] = ACTIONS(1496),
    [sym_do_custom_event] = ACTIONS(1496),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1496),
    [sym_clkmgr_set_clk_source] = ACTIONS(1496),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1496),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1496),
    [sym_clkmgr_get_timezone] = ACTIONS(1496),
    [sym_clkmgr_set_timezone] = ACTIONS(1496),
    [sym_clkmgr_get_resync_period] = ACTIONS(1496),
    [sym_clkmgr_set_resync_period] = ACTIONS(1496),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1496),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1496),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1496),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1496),
    [sym_clkmgr_get_timeservers] = ACTIONS(1496),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1496),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1496),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1496),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1496),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1496),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1496),
    [sym_set_log_level] = ACTIONS(1496),
    [sym_get_log_level] = ACTIONS(1496),
    [sym_amx_log] = ACTIONS(1496),
    [sym_smtp_server_config_set] = ACTIONS(1496),
    [sym_smtp_server_config_get] = ACTIONS(1496),
    [sym_smtp_send] = ACTIONS(1496),
    [sym_validate_netlinx_account] = ACTIONS(1496),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1496),
    [sym_authenticate_certificate] = ACTIONS(1496),
    [sym_audit_netlinx_session_event] = ACTIONS(1496),
    [sym_audit_netlinx_generic_event] = ACTIONS(1496),
    [sym_get_available_flash_disk_space] = ACTIONS(1496),
    [sym_get_max_flash_disk_space] = ACTIONS(1496),
    [sym_data_create_feed] = ACTIONS(1496),
    [sym_data_delete_feed] = ACTIONS(1496),
    [sym_data_publish_feed] = ACTIONS(1496),
    [sym_data_get_published_feed] = ACTIONS(1496),
    [sym_data_add_record] = ACTIONS(1496),
    [sym_data_get_event_record] = ACTIONS(1496),
    [sym__wc_data_create_feed] = ACTIONS(1496),
    [sym__wc_data_add_record] = ACTIONS(1496),
    [sym__wc_data_get_event_record] = ACTIONS(1496),
    [sym_format] = ACTIONS(1496),
    [sym_astro_clock] = ACTIONS(1496),
    [sym_set_outdoor_temperature] = ACTIONS(1496),
    [sym_set_virtual_level_count] = ACTIONS(1496),
    [sym_set_virtual_channel_count] = ACTIONS(1496),
    [sym_set_virtual_port_count] = ACTIONS(1496),
    [sym__wc_encode_int] = ACTIONS(1496),
    [sym__wc_decode_int] = ACTIONS(1496),
    [sym__wc_int] = ACTIONS(1496),
    [sym__wc_to_ch_int] = ACTIONS(1496),
    [sym__ch_to_wc_int] = ACTIONS(1496),
    [sym__wc_find_string_int] = ACTIONS(1496),
    [sym__wc_left_string_int] = ACTIONS(1496),
    [sym__wc_lower_string_int] = ACTIONS(1496),
    [sym__wc_mid_string_int] = ACTIONS(1496),
    [sym__wc_remove_string_int] = ACTIONS(1496),
    [sym__wc_right_string_int] = ACTIONS(1496),
    [sym__wc_upper_string_int] = ACTIONS(1496),
    [sym__wc_compare_string_int] = ACTIONS(1496),
    [sym__wc_get_buffer_char_int] = ACTIONS(1496),
    [sym__wc_get_buffer_string_int] = ACTIONS(1496),
    [sym__wc_concat_string_int] = ACTIONS(1496),
    [sym__wc_file_open_int] = ACTIONS(1496),
    [sym__wc_file_close_int] = ACTIONS(1496),
    [sym__wc_file_read_int] = ACTIONS(1496),
    [sym__wc_file_read_line_int] = ACTIONS(1496),
    [sym__wc_file_write_int] = ACTIONS(1496),
    [sym__wc_file_write_line_int] = ACTIONS(1496),
    [anon_sym_EQ] = ACTIONS(1496),
    [aux_sym_preproc_include_token1] = ACTIONS(1496),
    [aux_sym_preproc_define_token1] = ACTIONS(1498),
    [aux_sym_preproc_warn_token1] = ACTIONS(1498),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1498),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1498),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1498),
    [anon_sym_LBRACK] = ACTIONS(1515),
    [anon_sym_LBRACE] = ACTIONS(1498),
    [anon_sym_COLON] = ACTIONS(1498),
    [anon_sym_LPAREN] = ACTIONS(1496),
    [anon_sym_SEMI] = ACTIONS(1498),
    [anon_sym_BANG] = ACTIONS(1496),
    [anon_sym_TILDE] = ACTIONS(1498),
    [anon_sym_DASH] = ACTIONS(1496),
    [anon_sym_PLUS] = ACTIONS(1496),
    [anon_sym_STAR] = ACTIONS(1498),
    [anon_sym_SLASH] = ACTIONS(1496),
    [anon_sym_PERCENT] = ACTIONS(1498),
    [anon_sym_PIPE_PIPE] = ACTIONS(1498),
    [anon_sym_AMP_AMP] = ACTIONS(1498),
    [anon_sym_PIPE] = ACTIONS(1496),
    [anon_sym_CARET] = ACTIONS(1498),
    [anon_sym_AMP] = ACTIONS(1496),
    [anon_sym_EQ_EQ] = ACTIONS(1498),
    [anon_sym_BANG_EQ] = ACTIONS(1498),
    [anon_sym_LT_GT] = ACTIONS(1498),
    [anon_sym_GT] = ACTIONS(1496),
    [anon_sym_GT_EQ] = ACTIONS(1498),
    [anon_sym_LT_EQ] = ACTIONS(1498),
    [anon_sym_LT] = ACTIONS(1496),
    [anon_sym_LT_LT] = ACTIONS(1498),
    [anon_sym_GT_GT] = ACTIONS(1498),
    [anon_sym_DASH_DASH] = ACTIONS(1498),
    [anon_sym_PLUS_PLUS] = ACTIONS(1498),
    [anon_sym_DOT] = ACTIONS(1496),
    [anon_sym_DQUOTE] = ACTIONS(1498),
    [anon_sym_SQUOTE] = ACTIONS(1498),
    [sym_number_literal] = ACTIONS(1498),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1498),
  },
  [STATE(84)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1505),
    [sym_identifier] = ACTIONS(1503),
    [sym_program_name_keyword] = ACTIONS(1503),
    [sym_module_name_keyword] = ACTIONS(1503),
    [sym_define_device_keyword] = ACTIONS(1503),
    [sym_define_combine_keyword] = ACTIONS(1503),
    [sym_define_constant_keyword] = ACTIONS(1503),
    [sym_define_type_keyword] = ACTIONS(1503),
    [sym_define_variable_keyword] = ACTIONS(1503),
    [sym_define_system_variable_keyword] = ACTIONS(1503),
    [sym_define_connect_level_keyword] = ACTIONS(1503),
    [sym_define_latching_keyword] = ACTIONS(1503),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1503),
    [sym_define_toggling_keyword] = ACTIONS(1503),
    [sym_define_call_keyword] = ACTIONS(1503),
    [sym_define_function_keyword] = ACTIONS(1503),
    [sym_define_library_function_keyword] = ACTIONS(1503),
    [sym_define_start_keyword] = ACTIONS(1503),
    [sym_define_event_keyword] = ACTIONS(1503),
    [sym_define_program_keyword] = ACTIONS(1503),
    [sym_define_module_keyword] = ACTIONS(1503),
    [sym_char_keyword] = ACTIONS(1503),
    [sym_widechar_keyword] = ACTIONS(1503),
    [sym_integer_keyword] = ACTIONS(1503),
    [sym_sinteger_keyword] = ACTIONS(1503),
    [sym_long_keyword] = ACTIONS(1503),
    [sym_slong_keyword] = ACTIONS(1503),
    [sym_float_keyword] = ACTIONS(1503),
    [sym_double_keyword] = ACTIONS(1503),
    [sym_variant_keyword] = ACTIONS(1503),
    [sym_variantarray_keyword] = ACTIONS(1503),
    [sym_local_var_keyword] = ACTIONS(1503),
    [sym_stack_var_keyword] = ACTIONS(1503),
    [sym_constant_keyword] = ACTIONS(1503),
    [sym_volatile_keyword] = ACTIONS(1503),
    [sym_non_volatile_keyword] = ACTIONS(1503),
    [sym_persistent_keyword] = ACTIONS(1503),
    [sym_dev_keyword] = ACTIONS(1503),
    [sym_devlev_keyword] = ACTIONS(1503),
    [sym_devchan_keyword] = ACTIONS(1503),
    [sym_if_keyword] = ACTIONS(1503),
    [sym_else_keyword] = ACTIONS(1503),
    [sym_switch_keyword] = ACTIONS(1503),
    [sym_case_keyword] = ACTIONS(1503),
    [sym_default_keyword] = ACTIONS(1503),
    [sym_while_keyword] = ACTIONS(1503),
    [sym_for_keyword] = ACTIONS(1503),
    [sym_break_keyword] = ACTIONS(1503),
    [sym_continue_keyword] = ACTIONS(1503),
    [sym_return_keyword] = ACTIONS(1503),
    [sym_select_keyword] = ACTIONS(1503),
    [sym_active_keyword] = ACTIONS(1503),
    [sym_struct_keyword] = ACTIONS(1503),
    [sym_structure_keyword] = ACTIONS(1503),
    [sym_button_event_keyword] = ACTIONS(1503),
    [sym_channel_event_keyword] = ACTIONS(1503),
    [sym_level_event_keyword] = ACTIONS(1503),
    [sym_data_event_keyword] = ACTIONS(1503),
    [sym_timeline_event_keyword] = ACTIONS(1503),
    [sym_custom_event_keyword] = ACTIONS(1503),
    [sym_push_keyword] = ACTIONS(1503),
    [sym_release_keyword] = ACTIONS(1503),
    [aux_sym_on_keyword_token1] = ACTIONS(1503),
    [aux_sym_off_keyword_token1] = ACTIONS(1503),
    [sym_send_level_keyword] = ACTIONS(1503),
    [sym_send_string_keyword] = ACTIONS(1503),
    [sym_send_command_keyword] = ACTIONS(1503),
    [sym_clear_buffer_keyword] = ACTIONS(1503),
    [sym_create_buffer_keyword] = ACTIONS(1503),
    [sym_create_multi_buffer_keyword] = ACTIONS(1503),
    [sym_call_keyword] = ACTIONS(1503),
    [sym_system_call_keyword] = ACTIONS(1503),
    [sym_devchan_to_keyword] = ACTIONS(1503),
    [sym_devchan_min_to_keyword] = ACTIONS(1503),
    [sym_devchan_total_off_keyword] = ACTIONS(1503),
    [sym_devchan_pulse_keyword] = ACTIONS(1503),
    [sym_wait_keyword] = ACTIONS(1503),
    [sym_cancel_wait_keyword] = ACTIONS(1503),
    [sym_wait_until_keyword] = ACTIONS(1503),
    [sym_cancel_wait_until_keyword] = ACTIONS(1503),
    [sym_cancel_all_wait_keyword] = ACTIONS(1503),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1503),
    [sym_and] = ACTIONS(1417),
    [sym_or] = ACTIONS(1419),
    [sym_not] = ACTIONS(1503),
    [sym_xor] = ACTIONS(1421),
    [sym_band] = ACTIONS(1423),
    [sym_bor] = ACTIONS(1425),
    [sym_bxor] = ACTIONS(1421),
    [sym_bnot] = ACTIONS(1503),
    [sym_lshift] = ACTIONS(1427),
    [sym_rshift] = ACTIONS(1427),
    [sym___file__] = ACTIONS(1503),
    [sym___date__] = ACTIONS(1503),
    [sym___ldate__] = ACTIONS(1503),
    [sym___time__] = ACTIONS(1503),
    [sym___line__] = ACTIONS(1503),
    [sym___name__] = ACTIONS(1503),
    [sym___netlinx__] = ACTIONS(1503),
    [sym_netlinx_axi_version] = ACTIONS(1503),
    [sym_url_flg_tcp] = ACTIONS(1503),
    [sym_url_flg_accinfopresent] = ACTIONS(1503),
    [sym_url_flg_temp] = ACTIONS(1503),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1503),
    [sym_url_flg_stat_mask] = ACTIONS(1503),
    [sym_url_flg_stat_lookup] = ACTIONS(1503),
    [sym_url_flg_stat_connecting] = ACTIONS(1503),
    [sym_url_flg_stat_waiting] = ACTIONS(1503),
    [sym_url_flg_stat_connected] = ACTIONS(1503),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1503),
    [sym_first_virtual_device] = ACTIONS(1503),
    [sym_first_local_port] = ACTIONS(1503),
    [sym_dynamic_virtual_device] = ACTIONS(1503),
    [sym_timeline_once] = ACTIONS(1503),
    [sym_timeline_repeat] = ACTIONS(1503),
    [sym_timeline_absolute] = ACTIONS(1503),
    [sym_timeline_relative] = ACTIONS(1503),
    [sym_do_push_timed_infinite] = ACTIONS(1503),
    [sym_source_type_no_address] = ACTIONS(1503),
    [sym_source_type_neuron_id] = ACTIONS(1503),
    [sym_source_type_ip_address] = ACTIONS(1503),
    [sym_source_type_axlink] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1503),
    [sym_source_type_ip_socket_address] = ACTIONS(1503),
    [sym_source_type_rs232] = ACTIONS(1503),
    [sym_source_type_internal] = ACTIONS(1503),
    [sym_source_type_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1503),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1503),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1503),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1503),
    [sym_true] = ACTIONS(1503),
    [sym_false] = ACTIONS(1503),
    [sym_file_read_only] = ACTIONS(1503),
    [sym_file_rw_new] = ACTIONS(1503),
    [sym_file_rw_append] = ACTIONS(1503),
    [sym_ip_tcp] = ACTIONS(1503),
    [sym_ip_udp] = ACTIONS(1503),
    [sym_ip_udp_2way] = ACTIONS(1503),
    [sym_xml_encode_types] = ACTIONS(1503),
    [sym_xml_encode_char_as_list] = ACTIONS(1503),
    [sym_xml_encode_le] = ACTIONS(1503),
    [sym_xml_decode_types] = ACTIONS(1503),
    [sym_xml_decode_no_preserve] = ACTIONS(1503),
    [sym_duet_dev_type_display_device] = ACTIONS(1503),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_io_device] = ACTIONS(1503),
    [sym_duet_dev_type_relay_device] = ACTIONS(1503),
    [sym_duet_dev_type_ups] = ACTIONS(1503),
    [sym_duet_dev_type_amplifier] = ACTIONS(1503),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1503),
    [sym_duet_dev_type_camera] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1503),
    [sym_duet_dev_type_dss] = ACTIONS(1503),
    [sym_duet_dev_type_dvr] = ACTIONS(1503),
    [sym_duet_dev_type_disc_device] = ACTIONS(1503),
    [sym_duet_dev_type_document_camera] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1503),
    [sym_duet_dev_type_hvac] = ACTIONS(1503),
    [sym_duet_dev_type_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_light] = ACTIONS(1503),
    [sym_duet_dev_type_monitor] = ACTIONS(1503),
    [sym_duet_dev_type_motor] = ACTIONS(1503),
    [sym_duet_dev_type_multi_window] = ACTIONS(1503),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1503),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1503),
    [sym_duet_dev_type_receiver] = ACTIONS(1503),
    [sym_duet_dev_type_security_system] = ACTIONS(1503),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1503),
    [sym_duet_dev_type_settop_box] = ACTIONS(1503),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1503),
    [sym_duet_dev_type_switcher] = ACTIONS(1503),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_tv] = ACTIONS(1503),
    [sym_duet_dev_type_utility] = ACTIONS(1503),
    [sym_duet_dev_type_vcr] = ACTIONS(1503),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_video_processor] = ACTIONS(1503),
    [sym_duet_dev_type_video_projector] = ACTIONS(1503),
    [sym_duet_dev_type_video_wall] = ACTIONS(1503),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1503),
    [sym_duet_dev_type_weather] = ACTIONS(1503),
    [sym_duet_dev_not_polled] = ACTIONS(1503),
    [sym_duet_dev_polled] = ACTIONS(1503),
    [sym_clkmgr_mode_network] = ACTIONS(1503),
    [sym_clkmgr_mode_standalone] = ACTIONS(1503),
    [sym_normal_standby] = ACTIONS(1503),
    [sym_normal_wake] = ACTIONS(1503),
    [sym_ip_multicast_ttl_option] = ACTIONS(1503),
    [sym_ip_ttl_subnet] = ACTIONS(1503),
    [sym_ip_ttl_site] = ACTIONS(1503),
    [sym_ip_ttl_region] = ACTIONS(1503),
    [sym_ip_ttl_continent] = ACTIONS(1503),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1503),
    [sym_ip_nodelay_on] = ACTIONS(1503),
    [sym_ip_nodelay_off] = ACTIONS(1503),
    [sym_amx_error] = ACTIONS(1503),
    [sym_amx_warning] = ACTIONS(1503),
    [sym_amx_info] = ACTIONS(1503),
    [sym_amx_debug] = ACTIONS(1503),
    [sym_netlinx_login_success] = ACTIONS(1503),
    [sym_netlinx_login_fail] = ACTIONS(1503),
    [sym_netlinx_logout] = ACTIONS(1503),
    [sym_null_str] = ACTIONS(1503),
    [sym_smtp_address] = ACTIONS(1503),
    [sym_smtp_port_number] = ACTIONS(1503),
    [sym_smtp_username] = ACTIONS(1503),
    [sym_smtp_password] = ACTIONS(1503),
    [sym_smtp_require_tls] = ACTIONS(1503),
    [sym_smtp_from] = ACTIONS(1503),
    [sym_smtp_tls_true] = ACTIONS(1503),
    [sym_smtp_tls_false] = ACTIONS(1503),
    [sym_listview_on_row_select_event] = ACTIONS(1503),
    [sym_data_structure_unknown] = ACTIONS(1503),
    [sym_data_structure_datafeed] = ACTIONS(1503),
    [sym_data_structure_datarecordset] = ACTIONS(1503),
    [sym_data_structure_datarecord] = ACTIONS(1503),
    [sym_data_structure_datafield] = ACTIONS(1503),
    [sym_data_max_content_field_count] = ACTIONS(1503),
    [sym_data_max_metadata_field_count] = ACTIONS(1503),
    [sym_data_max_id_length] = ACTIONS(1503),
    [sym_data_max_name_length] = ACTIONS(1503),
    [sym_data_max_description_length] = ACTIONS(1503),
    [sym_data_max_type_length] = ACTIONS(1503),
    [sym_data_max_format_length] = ACTIONS(1503),
    [sym_data_max_source_length] = ACTIONS(1503),
    [sym_data_max_value_length] = ACTIONS(1503),
    [sym_data_max_label_length] = ACTIONS(1503),
    [sym_data_max_url_length] = ACTIONS(1503),
    [sym_max_last_login_info_length] = ACTIONS(1503),
    [sym_data_type_unknown] = ACTIONS(1503),
    [sym_data_type_string] = ACTIONS(1503),
    [sym_data_type_datetime] = ACTIONS(1503),
    [sym_data_type_date] = ACTIONS(1503),
    [sym_data_type_time] = ACTIONS(1503),
    [sym_data_type_image] = ACTIONS(1503),
    [sym_data_format_url] = ACTIONS(1503),
    [sym_data_format_phone] = ACTIONS(1503),
    [sym_data_format_email] = ACTIONS(1503),
    [sym_data_format_iso8601] = ACTIONS(1503),
    [sym_auditlog_priv] = ACTIONS(1503),
    [sym_configuration_priv] = ACTIONS(1503),
    [sym_deviceconfig_priv] = ACTIONS(1503),
    [sym_ftp_priv] = ACTIONS(1503),
    [sym_http_priv] = ACTIONS(1503),
    [sym_networkconfig_priv] = ACTIONS(1503),
    [sym_programport_priv] = ACTIONS(1503),
    [sym_remoteui_priv] = ACTIONS(1503),
    [sym_securitycontrol_priv] = ACTIONS(1503),
    [sym_softwaremanagement_priv] = ACTIONS(1503),
    [sym_terminal_priv] = ACTIONS(1503),
    [sym_tpadmin_priv] = ACTIONS(1503),
    [sym_user1_priv] = ACTIONS(1503),
    [sym_user2_priv] = ACTIONS(1503),
    [sym_user3_priv] = ACTIONS(1503),
    [sym_user4_priv] = ACTIONS(1503),
    [sym_usermanagement_priv] = ACTIONS(1503),
    [sym_valid_account] = ACTIONS(1503),
    [sym_err_name_invalid] = ACTIONS(1503),
    [sym_err_password_invalid] = ACTIONS(1503),
    [sym_err_invalid_account] = ACTIONS(1503),
    [sym_err_no_authorization] = ACTIONS(1503),
    [sym_err_invalid_parameter] = ACTIONS(1503),
    [sym_err_user_locked_out] = ACTIONS(1503),
    [sym_err_user_account_expired] = ACTIONS(1503),
    [sym_err_general_error] = ACTIONS(1503),
    [sym_err_authentication_error] = ACTIONS(1503),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1503),
    [sym_tls_validate_certificate] = ACTIONS(1503),
    [sym_tdata] = ACTIONS(1503),
    [sym_tchannel] = ACTIONS(1503),
    [sym_tlevel] = ACTIONS(1503),
    [sym_tbutton] = ACTIONS(1503),
    [sym_ttimeline] = ACTIONS(1503),
    [sym_tcustom] = ACTIONS(1503),
    [sym_url_struct] = ACTIONS(1503),
    [sym_dns_struct] = ACTIONS(1503),
    [sym_ip_address_struct] = ACTIONS(1503),
    [sym_dev_info_struct] = ACTIONS(1503),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1503),
    [sym_data_feed] = ACTIONS(1503),
    [sym_data_field] = ACTIONS(1503),
    [sym_data_record] = ACTIONS(1503),
    [sym_wc_data_feed] = ACTIONS(1503),
    [sym_wc_data_field] = ACTIONS(1503),
    [sym_wc_data_record] = ACTIONS(1503),
    [sym_last_login_info] = ACTIONS(1503),
    [sym____reserved___] = ACTIONS(1503),
    [sym_date] = ACTIONS(1503),
    [sym_day] = ACTIONS(1503),
    [sym_time] = ACTIONS(1503),
    [sym_ldate] = ACTIONS(1503),
    [sym_push_device] = ACTIONS(1503),
    [sym_push_channel] = ACTIONS(1503),
    [sym_push_devchan] = ACTIONS(1503),
    [sym_release_device] = ACTIONS(1503),
    [sym_release_channel] = ACTIONS(1503),
    [sym_release_devchan] = ACTIONS(1503),
    [sym_master_slot] = ACTIONS(1503),
    [sym_get_pulse_time] = ACTIONS(1503),
    [sym_get_timer] = ACTIONS(1503),
    [sym_master_sn] = ACTIONS(1503),
    [sym_system_number] = ACTIONS(1503),
    [sym_button] = ACTIONS(1503),
    [sym_dv_channel] = ACTIONS(1503),
    [sym_data] = ACTIONS(1503),
    [sym_level] = ACTIONS(1503),
    [sym_timeline] = ACTIONS(1503),
    [sym_channel] = ACTIONS(1503),
    [sym_custom] = ACTIONS(1503),
    [sym_atoi] = ACTIONS(1503),
    [sym_atol] = ACTIONS(1503),
    [sym_atof] = ACTIONS(1503),
    [sym_itoa] = ACTIONS(1503),
    [sym_itohex] = ACTIONS(1503),
    [sym_hextoi] = ACTIONS(1503),
    [sym_ftoa] = ACTIONS(1503),
    [sym_find_string] = ACTIONS(1503),
    [sym_left_string] = ACTIONS(1503),
    [sym_right_string] = ACTIONS(1503),
    [sym_mid_string] = ACTIONS(1503),
    [sym_remove_string] = ACTIONS(1503),
    [sym_get_buffer_string] = ACTIONS(1503),
    [sym_get_buffer_char] = ACTIONS(1503),
    [sym_get_multi_buffer_string] = ACTIONS(1503),
    [sym_length_string] = ACTIONS(1503),
    [sym_max_length_string] = ACTIONS(1503),
    [sym_set_length_string] = ACTIONS(1503),
    [sym_compare_string] = ACTIONS(1503),
    [sym_upper_string] = ACTIONS(1503),
    [sym_lower_string] = ACTIONS(1503),
    [sym_get_last] = ACTIONS(1503),
    [sym_date_to_day] = ACTIONS(1503),
    [sym_date_to_month] = ACTIONS(1503),
    [sym_date_to_year] = ACTIONS(1503),
    [sym_time_to_hour] = ACTIONS(1503),
    [sym_time_to_minute] = ACTIONS(1503),
    [sym_time_to_second] = ACTIONS(1503),
    [sym_day_of_week] = ACTIONS(1503),
    [sym_set_pulse_time] = ACTIONS(1503),
    [sym_set_timer] = ACTIONS(1503),
    [sym_file_open] = ACTIONS(1503),
    [sym_file_close] = ACTIONS(1503),
    [sym_file_read] = ACTIONS(1503),
    [sym_file_read_line] = ACTIONS(1503),
    [sym_file_write] = ACTIONS(1503),
    [sym_file_write_line] = ACTIONS(1503),
    [sym_file_copy] = ACTIONS(1503),
    [sym_file_delete] = ACTIONS(1503),
    [sym_file_rename] = ACTIONS(1503),
    [sym_file_dir] = ACTIONS(1503),
    [sym_file_getdir] = ACTIONS(1503),
    [sym_file_setdir] = ACTIONS(1503),
    [sym_file_create_dir] = ACTIONS(1503),
    [sym_file_remove_dir] = ACTIONS(1503),
    [sym_file_seek] = ACTIONS(1503),
    [sym_length_array] = ACTIONS(1503),
    [sym_max_length_array] = ACTIONS(1503),
    [sym_set_length_array] = ACTIONS(1503),
    [sym_ip_client_open] = ACTIONS(1503),
    [sym_ip_client_close] = ACTIONS(1503),
    [sym_ip_server_open] = ACTIONS(1503),
    [sym_ip_server_close] = ACTIONS(1503),
    [sym_ip_bound_client_open] = ACTIONS(1503),
    [sym_ip_mc_server_open] = ACTIONS(1503),
    [sym_ip_set_option] = ACTIONS(1503),
    [sym_ssh_client_open] = ACTIONS(1503),
    [sym_ssh_client_close] = ACTIONS(1503),
    [sym_tls_client_open] = ACTIONS(1503),
    [sym_tls_client_close] = ACTIONS(1503),
    [sym_do_push] = ACTIONS(1503),
    [sym_do_push_timed] = ACTIONS(1503),
    [sym_do_release] = ACTIONS(1503),
    [sym_device_id] = ACTIONS(1503),
    [sym_device_info] = ACTIONS(1503),
    [sym_device_id_string] = ACTIONS(1503),
    [sym_device_standby] = ACTIONS(1503),
    [sym_device_wake] = ACTIONS(1503),
    [sym_timeline_create] = ACTIONS(1503),
    [sym_timeline_kill] = ACTIONS(1503),
    [sym_timeline_reload] = ACTIONS(1503),
    [sym_timeline_pause] = ACTIONS(1503),
    [sym_timeline_restart] = ACTIONS(1503),
    [sym_timeline_set] = ACTIONS(1503),
    [sym_timeline_get] = ACTIONS(1503),
    [sym_timeline_active] = ACTIONS(1503),
    [sym_timeline_dynamic_id] = ACTIONS(1503),
    [sym_variable_to_string] = ACTIONS(1503),
    [sym_string_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_string] = ACTIONS(1503),
    [sym_type_cast] = ACTIONS(1503),
    [sym_variable_to_xml] = ACTIONS(1503),
    [sym_xml_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_xml] = ACTIONS(1503),
    [sym_redirect_string] = ACTIONS(1503),
    [sym_random_number] = ACTIONS(1503),
    [sym_get_unique_id] = ACTIONS(1503),
    [sym_get_system_number] = ACTIONS(1503),
    [sym_set_system_number] = ACTIONS(1503),
    [sym_get_serial_number] = ACTIONS(1503),
    [sym_get_master_build] = ACTIONS(1503),
    [sym_set_validation_code] = ACTIONS(1503),
    [sym_reboot] = ACTIONS(1503),
    [sym_load_duet_module] = ACTIONS(1503),
    [sym_unload_duet_module] = ACTIONS(1503),
    [sym_duet_mem_size_set] = ACTIONS(1503),
    [sym_duet_mem_size_get] = ACTIONS(1503),
    [sym_dynamic_application_device] = ACTIONS(1503),
    [sym_dynamic_polled_port] = ACTIONS(1503),
    [sym_static_port_binding] = ACTIONS(1503),
    [sym_static_ip_binding] = ACTIONS(1503),
    [sym_get_dns_list] = ACTIONS(1503),
    [sym_set_dns_list] = ACTIONS(1503),
    [sym_get_ip_address] = ACTIONS(1503),
    [sym_set_ip_address] = ACTIONS(1503),
    [sym_get_url_list] = ACTIONS(1503),
    [sym_add_url_entry] = ACTIONS(1503),
    [sym_delete_url_entry] = ACTIONS(1503),
    [sym_abs_value] = ACTIONS(1503),
    [sym_max_value] = ACTIONS(1503),
    [sym_min_value] = ACTIONS(1503),
    [sym_exp_value] = ACTIONS(1503),
    [sym_log_value] = ACTIONS(1503),
    [sym_log10_value] = ACTIONS(1503),
    [sym_power_value] = ACTIONS(1503),
    [sym_sqrt_value] = ACTIONS(1503),
    [sym_raw_be] = ACTIONS(1503),
    [sym_raw_le] = ACTIONS(1503),
    [sym_internal_queue_size_set] = ACTIONS(1503),
    [sym_internal_queue_size_get] = ACTIONS(1503),
    [sym_internal_threshold_set] = ACTIONS(1503),
    [sym_internal_threshold_get] = ACTIONS(1503),
    [sym_rebuilt_event] = ACTIONS(1503),
    [sym_do_custom_event] = ACTIONS(1503),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1503),
    [sym_clkmgr_set_clk_source] = ACTIONS(1503),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1503),
    [sym_clkmgr_get_timezone] = ACTIONS(1503),
    [sym_clkmgr_set_timezone] = ACTIONS(1503),
    [sym_clkmgr_get_resync_period] = ACTIONS(1503),
    [sym_clkmgr_set_resync_period] = ACTIONS(1503),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_timeservers] = ACTIONS(1503),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_set_log_level] = ACTIONS(1503),
    [sym_get_log_level] = ACTIONS(1503),
    [sym_amx_log] = ACTIONS(1503),
    [sym_smtp_server_config_set] = ACTIONS(1503),
    [sym_smtp_server_config_get] = ACTIONS(1503),
    [sym_smtp_send] = ACTIONS(1503),
    [sym_validate_netlinx_account] = ACTIONS(1503),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1503),
    [sym_authenticate_certificate] = ACTIONS(1503),
    [sym_audit_netlinx_session_event] = ACTIONS(1503),
    [sym_audit_netlinx_generic_event] = ACTIONS(1503),
    [sym_get_available_flash_disk_space] = ACTIONS(1503),
    [sym_get_max_flash_disk_space] = ACTIONS(1503),
    [sym_data_create_feed] = ACTIONS(1503),
    [sym_data_delete_feed] = ACTIONS(1503),
    [sym_data_publish_feed] = ACTIONS(1503),
    [sym_data_get_published_feed] = ACTIONS(1503),
    [sym_data_add_record] = ACTIONS(1503),
    [sym_data_get_event_record] = ACTIONS(1503),
    [sym__wc_data_create_feed] = ACTIONS(1503),
    [sym__wc_data_add_record] = ACTIONS(1503),
    [sym__wc_data_get_event_record] = ACTIONS(1503),
    [sym_format] = ACTIONS(1503),
    [sym_astro_clock] = ACTIONS(1503),
    [sym_set_outdoor_temperature] = ACTIONS(1503),
    [sym_set_virtual_level_count] = ACTIONS(1503),
    [sym_set_virtual_channel_count] = ACTIONS(1503),
    [sym_set_virtual_port_count] = ACTIONS(1503),
    [sym__wc_encode_int] = ACTIONS(1503),
    [sym__wc_decode_int] = ACTIONS(1503),
    [sym__wc_int] = ACTIONS(1503),
    [sym__wc_to_ch_int] = ACTIONS(1503),
    [sym__ch_to_wc_int] = ACTIONS(1503),
    [sym__wc_find_string_int] = ACTIONS(1503),
    [sym__wc_left_string_int] = ACTIONS(1503),
    [sym__wc_lower_string_int] = ACTIONS(1503),
    [sym__wc_mid_string_int] = ACTIONS(1503),
    [sym__wc_remove_string_int] = ACTIONS(1503),
    [sym__wc_right_string_int] = ACTIONS(1503),
    [sym__wc_upper_string_int] = ACTIONS(1503),
    [sym__wc_compare_string_int] = ACTIONS(1503),
    [sym__wc_get_buffer_char_int] = ACTIONS(1503),
    [sym__wc_get_buffer_string_int] = ACTIONS(1503),
    [sym__wc_concat_string_int] = ACTIONS(1503),
    [sym__wc_file_open_int] = ACTIONS(1503),
    [sym__wc_file_close_int] = ACTIONS(1503),
    [sym__wc_file_read_int] = ACTIONS(1503),
    [sym__wc_file_read_line_int] = ACTIONS(1503),
    [sym__wc_file_write_int] = ACTIONS(1503),
    [sym__wc_file_write_line_int] = ACTIONS(1503),
    [aux_sym_preproc_include_token1] = ACTIONS(1503),
    [aux_sym_preproc_define_token1] = ACTIONS(1505),
    [aux_sym_preproc_warn_token1] = ACTIONS(1505),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1505),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1505),
    [anon_sym_COLON] = ACTIONS(1431),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1505),
    [anon_sym_BANG] = ACTIONS(1503),
    [anon_sym_TILDE] = ACTIONS(1505),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1443),
    [anon_sym_AMP_AMP] = ACTIONS(1445),
    [anon_sym_PIPE] = ACTIONS(1425),
    [anon_sym_CARET] = ACTIONS(1447),
    [anon_sym_AMP] = ACTIONS(1423),
    [anon_sym_EQ_EQ] = ACTIONS(1449),
    [anon_sym_BANG_EQ] = ACTIONS(1449),
    [anon_sym_LT_GT] = ACTIONS(1449),
    [anon_sym_GT] = ACTIONS(1451),
    [anon_sym_GT_EQ] = ACTIONS(1453),
    [anon_sym_LT_EQ] = ACTIONS(1453),
    [anon_sym_LT] = ACTIONS(1451),
    [anon_sym_LT_LT] = ACTIONS(1455),
    [anon_sym_GT_GT] = ACTIONS(1455),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1505),
    [anon_sym_SQUOTE] = ACTIONS(1505),
    [sym_number_literal] = ACTIONS(1505),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1505),
  },
  [STATE(85)] = {
    [ts_builtin_sym_end] = ACTIONS(1509),
    [sym_identifier] = ACTIONS(1507),
    [sym_program_name_keyword] = ACTIONS(1507),
    [sym_module_name_keyword] = ACTIONS(1507),
    [sym_define_device_keyword] = ACTIONS(1507),
    [sym_define_combine_keyword] = ACTIONS(1507),
    [sym_define_constant_keyword] = ACTIONS(1507),
    [sym_define_type_keyword] = ACTIONS(1507),
    [sym_define_variable_keyword] = ACTIONS(1507),
    [sym_define_system_variable_keyword] = ACTIONS(1507),
    [sym_define_connect_level_keyword] = ACTIONS(1507),
    [sym_define_latching_keyword] = ACTIONS(1507),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1507),
    [sym_define_toggling_keyword] = ACTIONS(1507),
    [sym_define_call_keyword] = ACTIONS(1507),
    [sym_define_function_keyword] = ACTIONS(1507),
    [sym_define_library_function_keyword] = ACTIONS(1507),
    [sym_define_start_keyword] = ACTIONS(1507),
    [sym_define_event_keyword] = ACTIONS(1507),
    [sym_define_program_keyword] = ACTIONS(1507),
    [sym_define_module_keyword] = ACTIONS(1507),
    [sym_char_keyword] = ACTIONS(1507),
    [sym_widechar_keyword] = ACTIONS(1507),
    [sym_integer_keyword] = ACTIONS(1507),
    [sym_sinteger_keyword] = ACTIONS(1507),
    [sym_long_keyword] = ACTIONS(1507),
    [sym_slong_keyword] = ACTIONS(1507),
    [sym_float_keyword] = ACTIONS(1507),
    [sym_double_keyword] = ACTIONS(1507),
    [sym_variant_keyword] = ACTIONS(1507),
    [sym_variantarray_keyword] = ACTIONS(1507),
    [sym_local_var_keyword] = ACTIONS(1507),
    [sym_stack_var_keyword] = ACTIONS(1507),
    [sym_constant_keyword] = ACTIONS(1507),
    [sym_volatile_keyword] = ACTIONS(1507),
    [sym_non_volatile_keyword] = ACTIONS(1507),
    [sym_persistent_keyword] = ACTIONS(1507),
    [sym_dev_keyword] = ACTIONS(1507),
    [sym_devlev_keyword] = ACTIONS(1507),
    [sym_devchan_keyword] = ACTIONS(1507),
    [sym_if_keyword] = ACTIONS(1507),
    [sym_else_keyword] = ACTIONS(1507),
    [sym_switch_keyword] = ACTIONS(1507),
    [sym_case_keyword] = ACTIONS(1507),
    [sym_default_keyword] = ACTIONS(1507),
    [sym_while_keyword] = ACTIONS(1507),
    [sym_for_keyword] = ACTIONS(1507),
    [sym_break_keyword] = ACTIONS(1507),
    [sym_continue_keyword] = ACTIONS(1507),
    [sym_return_keyword] = ACTIONS(1507),
    [sym_select_keyword] = ACTIONS(1507),
    [sym_active_keyword] = ACTIONS(1507),
    [sym_struct_keyword] = ACTIONS(1507),
    [sym_structure_keyword] = ACTIONS(1507),
    [sym_button_event_keyword] = ACTIONS(1507),
    [sym_channel_event_keyword] = ACTIONS(1507),
    [sym_level_event_keyword] = ACTIONS(1507),
    [sym_data_event_keyword] = ACTIONS(1507),
    [sym_timeline_event_keyword] = ACTIONS(1507),
    [sym_custom_event_keyword] = ACTIONS(1507),
    [sym_push_keyword] = ACTIONS(1507),
    [sym_release_keyword] = ACTIONS(1507),
    [aux_sym_on_keyword_token1] = ACTIONS(1507),
    [aux_sym_off_keyword_token1] = ACTIONS(1507),
    [sym_send_level_keyword] = ACTIONS(1507),
    [sym_send_string_keyword] = ACTIONS(1507),
    [sym_send_command_keyword] = ACTIONS(1507),
    [sym_clear_buffer_keyword] = ACTIONS(1507),
    [sym_create_buffer_keyword] = ACTIONS(1507),
    [sym_create_multi_buffer_keyword] = ACTIONS(1507),
    [sym_call_keyword] = ACTIONS(1507),
    [sym_system_call_keyword] = ACTIONS(1507),
    [sym_devchan_to_keyword] = ACTIONS(1507),
    [sym_devchan_min_to_keyword] = ACTIONS(1507),
    [sym_devchan_total_off_keyword] = ACTIONS(1507),
    [sym_devchan_pulse_keyword] = ACTIONS(1507),
    [sym_wait_keyword] = ACTIONS(1507),
    [sym_cancel_wait_keyword] = ACTIONS(1507),
    [sym_wait_until_keyword] = ACTIONS(1507),
    [sym_cancel_wait_until_keyword] = ACTIONS(1507),
    [sym_cancel_all_wait_keyword] = ACTIONS(1507),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1507),
    [sym_and] = ACTIONS(1507),
    [sym_or] = ACTIONS(1507),
    [sym_not] = ACTIONS(1507),
    [sym_xor] = ACTIONS(1507),
    [sym_band] = ACTIONS(1507),
    [sym_bor] = ACTIONS(1507),
    [sym_bxor] = ACTIONS(1507),
    [sym_bnot] = ACTIONS(1507),
    [sym_lshift] = ACTIONS(1507),
    [sym_rshift] = ACTIONS(1507),
    [sym___file__] = ACTIONS(1507),
    [sym___date__] = ACTIONS(1507),
    [sym___ldate__] = ACTIONS(1507),
    [sym___time__] = ACTIONS(1507),
    [sym___line__] = ACTIONS(1507),
    [sym___name__] = ACTIONS(1507),
    [sym___netlinx__] = ACTIONS(1507),
    [sym_netlinx_axi_version] = ACTIONS(1507),
    [sym_url_flg_tcp] = ACTIONS(1507),
    [sym_url_flg_accinfopresent] = ACTIONS(1507),
    [sym_url_flg_temp] = ACTIONS(1507),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1507),
    [sym_url_flg_stat_mask] = ACTIONS(1507),
    [sym_url_flg_stat_lookup] = ACTIONS(1507),
    [sym_url_flg_stat_connecting] = ACTIONS(1507),
    [sym_url_flg_stat_waiting] = ACTIONS(1507),
    [sym_url_flg_stat_connected] = ACTIONS(1507),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1507),
    [sym_first_virtual_device] = ACTIONS(1507),
    [sym_first_local_port] = ACTIONS(1507),
    [sym_dynamic_virtual_device] = ACTIONS(1507),
    [sym_timeline_once] = ACTIONS(1507),
    [sym_timeline_repeat] = ACTIONS(1507),
    [sym_timeline_absolute] = ACTIONS(1507),
    [sym_timeline_relative] = ACTIONS(1507),
    [sym_do_push_timed_infinite] = ACTIONS(1507),
    [sym_source_type_no_address] = ACTIONS(1507),
    [sym_source_type_neuron_id] = ACTIONS(1507),
    [sym_source_type_ip_address] = ACTIONS(1507),
    [sym_source_type_axlink] = ACTIONS(1507),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1507),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1507),
    [sym_source_type_ip_socket_address] = ACTIONS(1507),
    [sym_source_type_rs232] = ACTIONS(1507),
    [sym_source_type_internal] = ACTIONS(1507),
    [sym_source_type_mac_address] = ACTIONS(1507),
    [sym_source_type_ipv4_port] = ACTIONS(1507),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1507),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1507),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1507),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1507),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1507),
    [sym_true] = ACTIONS(1507),
    [sym_false] = ACTIONS(1507),
    [sym_file_read_only] = ACTIONS(1507),
    [sym_file_rw_new] = ACTIONS(1507),
    [sym_file_rw_append] = ACTIONS(1507),
    [sym_ip_tcp] = ACTIONS(1507),
    [sym_ip_udp] = ACTIONS(1507),
    [sym_ip_udp_2way] = ACTIONS(1507),
    [sym_xml_encode_types] = ACTIONS(1507),
    [sym_xml_encode_char_as_list] = ACTIONS(1507),
    [sym_xml_encode_le] = ACTIONS(1507),
    [sym_xml_decode_types] = ACTIONS(1507),
    [sym_xml_decode_no_preserve] = ACTIONS(1507),
    [sym_duet_dev_type_display_device] = ACTIONS(1507),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1507),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1507),
    [sym_duet_dev_type_io_device] = ACTIONS(1507),
    [sym_duet_dev_type_relay_device] = ACTIONS(1507),
    [sym_duet_dev_type_ups] = ACTIONS(1507),
    [sym_duet_dev_type_amplifier] = ACTIONS(1507),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1507),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1507),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1507),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1507),
    [sym_duet_dev_type_camera] = ACTIONS(1507),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1507),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1507),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1507),
    [sym_duet_dev_type_dss] = ACTIONS(1507),
    [sym_duet_dev_type_dvr] = ACTIONS(1507),
    [sym_duet_dev_type_disc_device] = ACTIONS(1507),
    [sym_duet_dev_type_document_camera] = ACTIONS(1507),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1507),
    [sym_duet_dev_type_hvac] = ACTIONS(1507),
    [sym_duet_dev_type_keypad] = ACTIONS(1507),
    [sym_duet_dev_type_light] = ACTIONS(1507),
    [sym_duet_dev_type_monitor] = ACTIONS(1507),
    [sym_duet_dev_type_motor] = ACTIONS(1507),
    [sym_duet_dev_type_multi_window] = ACTIONS(1507),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1507),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1507),
    [sym_duet_dev_type_receiver] = ACTIONS(1507),
    [sym_duet_dev_type_security_system] = ACTIONS(1507),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1507),
    [sym_duet_dev_type_settop_box] = ACTIONS(1507),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1507),
    [sym_duet_dev_type_switcher] = ACTIONS(1507),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1507),
    [sym_duet_dev_type_tv] = ACTIONS(1507),
    [sym_duet_dev_type_utility] = ACTIONS(1507),
    [sym_duet_dev_type_vcr] = ACTIONS(1507),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1507),
    [sym_duet_dev_type_video_processor] = ACTIONS(1507),
    [sym_duet_dev_type_video_projector] = ACTIONS(1507),
    [sym_duet_dev_type_video_wall] = ACTIONS(1507),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1507),
    [sym_duet_dev_type_weather] = ACTIONS(1507),
    [sym_duet_dev_not_polled] = ACTIONS(1507),
    [sym_duet_dev_polled] = ACTIONS(1507),
    [sym_clkmgr_mode_network] = ACTIONS(1507),
    [sym_clkmgr_mode_standalone] = ACTIONS(1507),
    [sym_normal_standby] = ACTIONS(1507),
    [sym_normal_wake] = ACTIONS(1507),
    [sym_ip_multicast_ttl_option] = ACTIONS(1507),
    [sym_ip_ttl_subnet] = ACTIONS(1507),
    [sym_ip_ttl_site] = ACTIONS(1507),
    [sym_ip_ttl_region] = ACTIONS(1507),
    [sym_ip_ttl_continent] = ACTIONS(1507),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1507),
    [sym_ip_nodelay_on] = ACTIONS(1507),
    [sym_ip_nodelay_off] = ACTIONS(1507),
    [sym_amx_error] = ACTIONS(1507),
    [sym_amx_warning] = ACTIONS(1507),
    [sym_amx_info] = ACTIONS(1507),
    [sym_amx_debug] = ACTIONS(1507),
    [sym_netlinx_login_success] = ACTIONS(1507),
    [sym_netlinx_login_fail] = ACTIONS(1507),
    [sym_netlinx_logout] = ACTIONS(1507),
    [sym_null_str] = ACTIONS(1507),
    [sym_smtp_address] = ACTIONS(1507),
    [sym_smtp_port_number] = ACTIONS(1507),
    [sym_smtp_username] = ACTIONS(1507),
    [sym_smtp_password] = ACTIONS(1507),
    [sym_smtp_require_tls] = ACTIONS(1507),
    [sym_smtp_from] = ACTIONS(1507),
    [sym_smtp_tls_true] = ACTIONS(1507),
    [sym_smtp_tls_false] = ACTIONS(1507),
    [sym_listview_on_row_select_event] = ACTIONS(1507),
    [sym_data_structure_unknown] = ACTIONS(1507),
    [sym_data_structure_datafeed] = ACTIONS(1507),
    [sym_data_structure_datarecordset] = ACTIONS(1507),
    [sym_data_structure_datarecord] = ACTIONS(1507),
    [sym_data_structure_datafield] = ACTIONS(1507),
    [sym_data_max_content_field_count] = ACTIONS(1507),
    [sym_data_max_metadata_field_count] = ACTIONS(1507),
    [sym_data_max_id_length] = ACTIONS(1507),
    [sym_data_max_name_length] = ACTIONS(1507),
    [sym_data_max_description_length] = ACTIONS(1507),
    [sym_data_max_type_length] = ACTIONS(1507),
    [sym_data_max_format_length] = ACTIONS(1507),
    [sym_data_max_source_length] = ACTIONS(1507),
    [sym_data_max_value_length] = ACTIONS(1507),
    [sym_data_max_label_length] = ACTIONS(1507),
    [sym_data_max_url_length] = ACTIONS(1507),
    [sym_max_last_login_info_length] = ACTIONS(1507),
    [sym_data_type_unknown] = ACTIONS(1507),
    [sym_data_type_string] = ACTIONS(1507),
    [sym_data_type_datetime] = ACTIONS(1507),
    [sym_data_type_date] = ACTIONS(1507),
    [sym_data_type_time] = ACTIONS(1507),
    [sym_data_type_image] = ACTIONS(1507),
    [sym_data_format_url] = ACTIONS(1507),
    [sym_data_format_phone] = ACTIONS(1507),
    [sym_data_format_email] = ACTIONS(1507),
    [sym_data_format_iso8601] = ACTIONS(1507),
    [sym_auditlog_priv] = ACTIONS(1507),
    [sym_configuration_priv] = ACTIONS(1507),
    [sym_deviceconfig_priv] = ACTIONS(1507),
    [sym_ftp_priv] = ACTIONS(1507),
    [sym_http_priv] = ACTIONS(1507),
    [sym_networkconfig_priv] = ACTIONS(1507),
    [sym_programport_priv] = ACTIONS(1507),
    [sym_remoteui_priv] = ACTIONS(1507),
    [sym_securitycontrol_priv] = ACTIONS(1507),
    [sym_softwaremanagement_priv] = ACTIONS(1507),
    [sym_terminal_priv] = ACTIONS(1507),
    [sym_tpadmin_priv] = ACTIONS(1507),
    [sym_user1_priv] = ACTIONS(1507),
    [sym_user2_priv] = ACTIONS(1507),
    [sym_user3_priv] = ACTIONS(1507),
    [sym_user4_priv] = ACTIONS(1507),
    [sym_usermanagement_priv] = ACTIONS(1507),
    [sym_valid_account] = ACTIONS(1507),
    [sym_err_name_invalid] = ACTIONS(1507),
    [sym_err_password_invalid] = ACTIONS(1507),
    [sym_err_invalid_account] = ACTIONS(1507),
    [sym_err_no_authorization] = ACTIONS(1507),
    [sym_err_invalid_parameter] = ACTIONS(1507),
    [sym_err_user_locked_out] = ACTIONS(1507),
    [sym_err_user_account_expired] = ACTIONS(1507),
    [sym_err_general_error] = ACTIONS(1507),
    [sym_err_authentication_error] = ACTIONS(1507),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1507),
    [sym_tls_validate_certificate] = ACTIONS(1507),
    [sym_tdata] = ACTIONS(1507),
    [sym_tchannel] = ACTIONS(1507),
    [sym_tlevel] = ACTIONS(1507),
    [sym_tbutton] = ACTIONS(1507),
    [sym_ttimeline] = ACTIONS(1507),
    [sym_tcustom] = ACTIONS(1507),
    [sym_url_struct] = ACTIONS(1507),
    [sym_dns_struct] = ACTIONS(1507),
    [sym_ip_address_struct] = ACTIONS(1507),
    [sym_dev_info_struct] = ACTIONS(1507),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1507),
    [sym_data_feed] = ACTIONS(1507),
    [sym_data_field] = ACTIONS(1507),
    [sym_data_record] = ACTIONS(1507),
    [sym_wc_data_feed] = ACTIONS(1507),
    [sym_wc_data_field] = ACTIONS(1507),
    [sym_wc_data_record] = ACTIONS(1507),
    [sym_last_login_info] = ACTIONS(1507),
    [sym____reserved___] = ACTIONS(1507),
    [sym_date] = ACTIONS(1507),
    [sym_day] = ACTIONS(1507),
    [sym_time] = ACTIONS(1507),
    [sym_ldate] = ACTIONS(1507),
    [sym_push_device] = ACTIONS(1507),
    [sym_push_channel] = ACTIONS(1507),
    [sym_push_devchan] = ACTIONS(1507),
    [sym_release_device] = ACTIONS(1507),
    [sym_release_channel] = ACTIONS(1507),
    [sym_release_devchan] = ACTIONS(1507),
    [sym_master_slot] = ACTIONS(1507),
    [sym_get_pulse_time] = ACTIONS(1507),
    [sym_get_timer] = ACTIONS(1507),
    [sym_master_sn] = ACTIONS(1507),
    [sym_system_number] = ACTIONS(1507),
    [sym_button] = ACTIONS(1507),
    [sym_dv_channel] = ACTIONS(1507),
    [sym_data] = ACTIONS(1507),
    [sym_level] = ACTIONS(1507),
    [sym_timeline] = ACTIONS(1507),
    [sym_channel] = ACTIONS(1507),
    [sym_custom] = ACTIONS(1507),
    [sym_atoi] = ACTIONS(1507),
    [sym_atol] = ACTIONS(1507),
    [sym_atof] = ACTIONS(1507),
    [sym_itoa] = ACTIONS(1507),
    [sym_itohex] = ACTIONS(1507),
    [sym_hextoi] = ACTIONS(1507),
    [sym_ftoa] = ACTIONS(1507),
    [sym_find_string] = ACTIONS(1507),
    [sym_left_string] = ACTIONS(1507),
    [sym_right_string] = ACTIONS(1507),
    [sym_mid_string] = ACTIONS(1507),
    [sym_remove_string] = ACTIONS(1507),
    [sym_get_buffer_string] = ACTIONS(1507),
    [sym_get_buffer_char] = ACTIONS(1507),
    [sym_get_multi_buffer_string] = ACTIONS(1507),
    [sym_length_string] = ACTIONS(1507),
    [sym_max_length_string] = ACTIONS(1507),
    [sym_set_length_string] = ACTIONS(1507),
    [sym_compare_string] = ACTIONS(1507),
    [sym_upper_string] = ACTIONS(1507),
    [sym_lower_string] = ACTIONS(1507),
    [sym_get_last] = ACTIONS(1507),
    [sym_date_to_day] = ACTIONS(1507),
    [sym_date_to_month] = ACTIONS(1507),
    [sym_date_to_year] = ACTIONS(1507),
    [sym_time_to_hour] = ACTIONS(1507),
    [sym_time_to_minute] = ACTIONS(1507),
    [sym_time_to_second] = ACTIONS(1507),
    [sym_day_of_week] = ACTIONS(1507),
    [sym_set_pulse_time] = ACTIONS(1507),
    [sym_set_timer] = ACTIONS(1507),
    [sym_file_open] = ACTIONS(1507),
    [sym_file_close] = ACTIONS(1507),
    [sym_file_read] = ACTIONS(1507),
    [sym_file_read_line] = ACTIONS(1507),
    [sym_file_write] = ACTIONS(1507),
    [sym_file_write_line] = ACTIONS(1507),
    [sym_file_copy] = ACTIONS(1507),
    [sym_file_delete] = ACTIONS(1507),
    [sym_file_rename] = ACTIONS(1507),
    [sym_file_dir] = ACTIONS(1507),
    [sym_file_getdir] = ACTIONS(1507),
    [sym_file_setdir] = ACTIONS(1507),
    [sym_file_create_dir] = ACTIONS(1507),
    [sym_file_remove_dir] = ACTIONS(1507),
    [sym_file_seek] = ACTIONS(1507),
    [sym_length_array] = ACTIONS(1507),
    [sym_max_length_array] = ACTIONS(1507),
    [sym_set_length_array] = ACTIONS(1507),
    [sym_ip_client_open] = ACTIONS(1507),
    [sym_ip_client_close] = ACTIONS(1507),
    [sym_ip_server_open] = ACTIONS(1507),
    [sym_ip_server_close] = ACTIONS(1507),
    [sym_ip_bound_client_open] = ACTIONS(1507),
    [sym_ip_mc_server_open] = ACTIONS(1507),
    [sym_ip_set_option] = ACTIONS(1507),
    [sym_ssh_client_open] = ACTIONS(1507),
    [sym_ssh_client_close] = ACTIONS(1507),
    [sym_tls_client_open] = ACTIONS(1507),
    [sym_tls_client_close] = ACTIONS(1507),
    [sym_do_push] = ACTIONS(1507),
    [sym_do_push_timed] = ACTIONS(1507),
    [sym_do_release] = ACTIONS(1507),
    [sym_device_id] = ACTIONS(1507),
    [sym_device_info] = ACTIONS(1507),
    [sym_device_id_string] = ACTIONS(1507),
    [sym_device_standby] = ACTIONS(1507),
    [sym_device_wake] = ACTIONS(1507),
    [sym_timeline_create] = ACTIONS(1507),
    [sym_timeline_kill] = ACTIONS(1507),
    [sym_timeline_reload] = ACTIONS(1507),
    [sym_timeline_pause] = ACTIONS(1507),
    [sym_timeline_restart] = ACTIONS(1507),
    [sym_timeline_set] = ACTIONS(1507),
    [sym_timeline_get] = ACTIONS(1507),
    [sym_timeline_active] = ACTIONS(1507),
    [sym_timeline_dynamic_id] = ACTIONS(1507),
    [sym_variable_to_string] = ACTIONS(1507),
    [sym_string_to_variable] = ACTIONS(1507),
    [sym_length_variable_to_string] = ACTIONS(1507),
    [sym_type_cast] = ACTIONS(1507),
    [sym_variable_to_xml] = ACTIONS(1507),
    [sym_xml_to_variable] = ACTIONS(1507),
    [sym_length_variable_to_xml] = ACTIONS(1507),
    [sym_redirect_string] = ACTIONS(1507),
    [sym_random_number] = ACTIONS(1507),
    [sym_get_unique_id] = ACTIONS(1507),
    [sym_get_system_number] = ACTIONS(1507),
    [sym_set_system_number] = ACTIONS(1507),
    [sym_get_serial_number] = ACTIONS(1507),
    [sym_get_master_build] = ACTIONS(1507),
    [sym_set_validation_code] = ACTIONS(1507),
    [sym_reboot] = ACTIONS(1507),
    [sym_load_duet_module] = ACTIONS(1507),
    [sym_unload_duet_module] = ACTIONS(1507),
    [sym_duet_mem_size_set] = ACTIONS(1507),
    [sym_duet_mem_size_get] = ACTIONS(1507),
    [sym_dynamic_application_device] = ACTIONS(1507),
    [sym_dynamic_polled_port] = ACTIONS(1507),
    [sym_static_port_binding] = ACTIONS(1507),
    [sym_static_ip_binding] = ACTIONS(1507),
    [sym_get_dns_list] = ACTIONS(1507),
    [sym_set_dns_list] = ACTIONS(1507),
    [sym_get_ip_address] = ACTIONS(1507),
    [sym_set_ip_address] = ACTIONS(1507),
    [sym_get_url_list] = ACTIONS(1507),
    [sym_add_url_entry] = ACTIONS(1507),
    [sym_delete_url_entry] = ACTIONS(1507),
    [sym_abs_value] = ACTIONS(1507),
    [sym_max_value] = ACTIONS(1507),
    [sym_min_value] = ACTIONS(1507),
    [sym_exp_value] = ACTIONS(1507),
    [sym_log_value] = ACTIONS(1507),
    [sym_log10_value] = ACTIONS(1507),
    [sym_power_value] = ACTIONS(1507),
    [sym_sqrt_value] = ACTIONS(1507),
    [sym_raw_be] = ACTIONS(1507),
    [sym_raw_le] = ACTIONS(1507),
    [sym_internal_queue_size_set] = ACTIONS(1507),
    [sym_internal_queue_size_get] = ACTIONS(1507),
    [sym_internal_threshold_set] = ACTIONS(1507),
    [sym_internal_threshold_get] = ACTIONS(1507),
    [sym_rebuilt_event] = ACTIONS(1507),
    [sym_do_custom_event] = ACTIONS(1507),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1507),
    [sym_clkmgr_set_clk_source] = ACTIONS(1507),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1507),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1507),
    [sym_clkmgr_get_timezone] = ACTIONS(1507),
    [sym_clkmgr_set_timezone] = ACTIONS(1507),
    [sym_clkmgr_get_resync_period] = ACTIONS(1507),
    [sym_clkmgr_set_resync_period] = ACTIONS(1507),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1507),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1507),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1507),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1507),
    [sym_clkmgr_get_timeservers] = ACTIONS(1507),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1507),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1507),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1507),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1507),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1507),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1507),
    [sym_set_log_level] = ACTIONS(1507),
    [sym_get_log_level] = ACTIONS(1507),
    [sym_amx_log] = ACTIONS(1507),
    [sym_smtp_server_config_set] = ACTIONS(1507),
    [sym_smtp_server_config_get] = ACTIONS(1507),
    [sym_smtp_send] = ACTIONS(1507),
    [sym_validate_netlinx_account] = ACTIONS(1507),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1507),
    [sym_authenticate_certificate] = ACTIONS(1507),
    [sym_audit_netlinx_session_event] = ACTIONS(1507),
    [sym_audit_netlinx_generic_event] = ACTIONS(1507),
    [sym_get_available_flash_disk_space] = ACTIONS(1507),
    [sym_get_max_flash_disk_space] = ACTIONS(1507),
    [sym_data_create_feed] = ACTIONS(1507),
    [sym_data_delete_feed] = ACTIONS(1507),
    [sym_data_publish_feed] = ACTIONS(1507),
    [sym_data_get_published_feed] = ACTIONS(1507),
    [sym_data_add_record] = ACTIONS(1507),
    [sym_data_get_event_record] = ACTIONS(1507),
    [sym__wc_data_create_feed] = ACTIONS(1507),
    [sym__wc_data_add_record] = ACTIONS(1507),
    [sym__wc_data_get_event_record] = ACTIONS(1507),
    [sym_format] = ACTIONS(1507),
    [sym_astro_clock] = ACTIONS(1507),
    [sym_set_outdoor_temperature] = ACTIONS(1507),
    [sym_set_virtual_level_count] = ACTIONS(1507),
    [sym_set_virtual_channel_count] = ACTIONS(1507),
    [sym_set_virtual_port_count] = ACTIONS(1507),
    [sym__wc_encode_int] = ACTIONS(1507),
    [sym__wc_decode_int] = ACTIONS(1507),
    [sym__wc_int] = ACTIONS(1507),
    [sym__wc_to_ch_int] = ACTIONS(1507),
    [sym__ch_to_wc_int] = ACTIONS(1507),
    [sym__wc_find_string_int] = ACTIONS(1507),
    [sym__wc_left_string_int] = ACTIONS(1507),
    [sym__wc_lower_string_int] = ACTIONS(1507),
    [sym__wc_mid_string_int] = ACTIONS(1507),
    [sym__wc_remove_string_int] = ACTIONS(1507),
    [sym__wc_right_string_int] = ACTIONS(1507),
    [sym__wc_upper_string_int] = ACTIONS(1507),
    [sym__wc_compare_string_int] = ACTIONS(1507),
    [sym__wc_get_buffer_char_int] = ACTIONS(1507),
    [sym__wc_get_buffer_string_int] = ACTIONS(1507),
    [sym__wc_concat_string_int] = ACTIONS(1507),
    [sym__wc_file_open_int] = ACTIONS(1507),
    [sym__wc_file_close_int] = ACTIONS(1507),
    [sym__wc_file_read_int] = ACTIONS(1507),
    [sym__wc_file_read_line_int] = ACTIONS(1507),
    [sym__wc_file_write_int] = ACTIONS(1507),
    [sym__wc_file_write_line_int] = ACTIONS(1507),
    [anon_sym_EQ] = ACTIONS(1507),
    [aux_sym_preproc_include_token1] = ACTIONS(1507),
    [aux_sym_preproc_define_token1] = ACTIONS(1509),
    [aux_sym_preproc_warn_token1] = ACTIONS(1509),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1509),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1509),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1509),
    [anon_sym_LBRACK] = ACTIONS(1509),
    [anon_sym_LBRACE] = ACTIONS(1509),
    [anon_sym_COLON] = ACTIONS(1509),
    [anon_sym_LPAREN] = ACTIONS(1507),
    [anon_sym_SEMI] = ACTIONS(1509),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1509),
    [anon_sym_DASH] = ACTIONS(1507),
    [anon_sym_PLUS] = ACTIONS(1507),
    [anon_sym_STAR] = ACTIONS(1509),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_PERCENT] = ACTIONS(1509),
    [anon_sym_PIPE_PIPE] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1509),
    [anon_sym_PIPE] = ACTIONS(1507),
    [anon_sym_CARET] = ACTIONS(1509),
    [anon_sym_AMP] = ACTIONS(1507),
    [anon_sym_EQ_EQ] = ACTIONS(1509),
    [anon_sym_BANG_EQ] = ACTIONS(1509),
    [anon_sym_LT_GT] = ACTIONS(1509),
    [anon_sym_GT] = ACTIONS(1507),
    [anon_sym_GT_EQ] = ACTIONS(1509),
    [anon_sym_LT_EQ] = ACTIONS(1509),
    [anon_sym_LT] = ACTIONS(1507),
    [anon_sym_LT_LT] = ACTIONS(1509),
    [anon_sym_GT_GT] = ACTIONS(1509),
    [anon_sym_DASH_DASH] = ACTIONS(1509),
    [anon_sym_PLUS_PLUS] = ACTIONS(1509),
    [anon_sym_DOT] = ACTIONS(1507),
    [sym_range_operator] = ACTIONS(1509),
    [anon_sym_DQUOTE] = ACTIONS(1509),
    [anon_sym_SQUOTE] = ACTIONS(1509),
    [sym_number_literal] = ACTIONS(1509),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1509),
  },
  [STATE(86)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1483),
    [sym_identifier] = ACTIONS(1481),
    [sym_program_name_keyword] = ACTIONS(1481),
    [sym_module_name_keyword] = ACTIONS(1481),
    [sym_define_device_keyword] = ACTIONS(1481),
    [sym_define_combine_keyword] = ACTIONS(1481),
    [sym_define_constant_keyword] = ACTIONS(1481),
    [sym_define_type_keyword] = ACTIONS(1481),
    [sym_define_variable_keyword] = ACTIONS(1481),
    [sym_define_system_variable_keyword] = ACTIONS(1481),
    [sym_define_connect_level_keyword] = ACTIONS(1481),
    [sym_define_latching_keyword] = ACTIONS(1481),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1481),
    [sym_define_toggling_keyword] = ACTIONS(1481),
    [sym_define_call_keyword] = ACTIONS(1481),
    [sym_define_function_keyword] = ACTIONS(1481),
    [sym_define_library_function_keyword] = ACTIONS(1481),
    [sym_define_start_keyword] = ACTIONS(1481),
    [sym_define_event_keyword] = ACTIONS(1481),
    [sym_define_program_keyword] = ACTIONS(1481),
    [sym_define_module_keyword] = ACTIONS(1481),
    [sym_char_keyword] = ACTIONS(1481),
    [sym_widechar_keyword] = ACTIONS(1481),
    [sym_integer_keyword] = ACTIONS(1481),
    [sym_sinteger_keyword] = ACTIONS(1481),
    [sym_long_keyword] = ACTIONS(1481),
    [sym_slong_keyword] = ACTIONS(1481),
    [sym_float_keyword] = ACTIONS(1481),
    [sym_double_keyword] = ACTIONS(1481),
    [sym_variant_keyword] = ACTIONS(1481),
    [sym_variantarray_keyword] = ACTIONS(1481),
    [sym_local_var_keyword] = ACTIONS(1481),
    [sym_stack_var_keyword] = ACTIONS(1481),
    [sym_constant_keyword] = ACTIONS(1481),
    [sym_volatile_keyword] = ACTIONS(1481),
    [sym_non_volatile_keyword] = ACTIONS(1481),
    [sym_persistent_keyword] = ACTIONS(1481),
    [sym_dev_keyword] = ACTIONS(1481),
    [sym_devlev_keyword] = ACTIONS(1481),
    [sym_devchan_keyword] = ACTIONS(1481),
    [sym_if_keyword] = ACTIONS(1481),
    [sym_else_keyword] = ACTIONS(1481),
    [sym_switch_keyword] = ACTIONS(1481),
    [sym_case_keyword] = ACTIONS(1481),
    [sym_default_keyword] = ACTIONS(1481),
    [sym_while_keyword] = ACTIONS(1481),
    [sym_for_keyword] = ACTIONS(1481),
    [sym_break_keyword] = ACTIONS(1481),
    [sym_continue_keyword] = ACTIONS(1481),
    [sym_return_keyword] = ACTIONS(1481),
    [sym_select_keyword] = ACTIONS(1481),
    [sym_active_keyword] = ACTIONS(1481),
    [sym_struct_keyword] = ACTIONS(1481),
    [sym_structure_keyword] = ACTIONS(1481),
    [sym_button_event_keyword] = ACTIONS(1481),
    [sym_channel_event_keyword] = ACTIONS(1481),
    [sym_level_event_keyword] = ACTIONS(1481),
    [sym_data_event_keyword] = ACTIONS(1481),
    [sym_timeline_event_keyword] = ACTIONS(1481),
    [sym_custom_event_keyword] = ACTIONS(1481),
    [sym_push_keyword] = ACTIONS(1481),
    [sym_release_keyword] = ACTIONS(1481),
    [aux_sym_on_keyword_token1] = ACTIONS(1481),
    [aux_sym_off_keyword_token1] = ACTIONS(1481),
    [sym_send_level_keyword] = ACTIONS(1481),
    [sym_send_string_keyword] = ACTIONS(1481),
    [sym_send_command_keyword] = ACTIONS(1481),
    [sym_clear_buffer_keyword] = ACTIONS(1481),
    [sym_create_buffer_keyword] = ACTIONS(1481),
    [sym_create_multi_buffer_keyword] = ACTIONS(1481),
    [sym_call_keyword] = ACTIONS(1481),
    [sym_system_call_keyword] = ACTIONS(1481),
    [sym_devchan_to_keyword] = ACTIONS(1481),
    [sym_devchan_min_to_keyword] = ACTIONS(1481),
    [sym_devchan_total_off_keyword] = ACTIONS(1481),
    [sym_devchan_pulse_keyword] = ACTIONS(1481),
    [sym_wait_keyword] = ACTIONS(1481),
    [sym_cancel_wait_keyword] = ACTIONS(1481),
    [sym_wait_until_keyword] = ACTIONS(1481),
    [sym_cancel_wait_until_keyword] = ACTIONS(1481),
    [sym_cancel_all_wait_keyword] = ACTIONS(1481),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1481),
    [sym_and] = ACTIONS(1417),
    [sym_or] = ACTIONS(1419),
    [sym_not] = ACTIONS(1481),
    [sym_xor] = ACTIONS(1421),
    [sym_band] = ACTIONS(1423),
    [sym_bor] = ACTIONS(1425),
    [sym_bxor] = ACTIONS(1421),
    [sym_bnot] = ACTIONS(1481),
    [sym_lshift] = ACTIONS(1427),
    [sym_rshift] = ACTIONS(1427),
    [sym___file__] = ACTIONS(1481),
    [sym___date__] = ACTIONS(1481),
    [sym___ldate__] = ACTIONS(1481),
    [sym___time__] = ACTIONS(1481),
    [sym___line__] = ACTIONS(1481),
    [sym___name__] = ACTIONS(1481),
    [sym___netlinx__] = ACTIONS(1481),
    [sym_netlinx_axi_version] = ACTIONS(1481),
    [sym_url_flg_tcp] = ACTIONS(1481),
    [sym_url_flg_accinfopresent] = ACTIONS(1481),
    [sym_url_flg_temp] = ACTIONS(1481),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1481),
    [sym_url_flg_stat_mask] = ACTIONS(1481),
    [sym_url_flg_stat_lookup] = ACTIONS(1481),
    [sym_url_flg_stat_connecting] = ACTIONS(1481),
    [sym_url_flg_stat_waiting] = ACTIONS(1481),
    [sym_url_flg_stat_connected] = ACTIONS(1481),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1481),
    [sym_first_virtual_device] = ACTIONS(1481),
    [sym_first_local_port] = ACTIONS(1481),
    [sym_dynamic_virtual_device] = ACTIONS(1481),
    [sym_timeline_once] = ACTIONS(1481),
    [sym_timeline_repeat] = ACTIONS(1481),
    [sym_timeline_absolute] = ACTIONS(1481),
    [sym_timeline_relative] = ACTIONS(1481),
    [sym_do_push_timed_infinite] = ACTIONS(1481),
    [sym_source_type_no_address] = ACTIONS(1481),
    [sym_source_type_neuron_id] = ACTIONS(1481),
    [sym_source_type_ip_address] = ACTIONS(1481),
    [sym_source_type_axlink] = ACTIONS(1481),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1481),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1481),
    [sym_source_type_ip_socket_address] = ACTIONS(1481),
    [sym_source_type_rs232] = ACTIONS(1481),
    [sym_source_type_internal] = ACTIONS(1481),
    [sym_source_type_mac_address] = ACTIONS(1481),
    [sym_source_type_ipv4_port] = ACTIONS(1481),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1481),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1481),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1481),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1481),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1481),
    [sym_true] = ACTIONS(1481),
    [sym_false] = ACTIONS(1481),
    [sym_file_read_only] = ACTIONS(1481),
    [sym_file_rw_new] = ACTIONS(1481),
    [sym_file_rw_append] = ACTIONS(1481),
    [sym_ip_tcp] = ACTIONS(1481),
    [sym_ip_udp] = ACTIONS(1481),
    [sym_ip_udp_2way] = ACTIONS(1481),
    [sym_xml_encode_types] = ACTIONS(1481),
    [sym_xml_encode_char_as_list] = ACTIONS(1481),
    [sym_xml_encode_le] = ACTIONS(1481),
    [sym_xml_decode_types] = ACTIONS(1481),
    [sym_xml_decode_no_preserve] = ACTIONS(1481),
    [sym_duet_dev_type_display_device] = ACTIONS(1481),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1481),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1481),
    [sym_duet_dev_type_io_device] = ACTIONS(1481),
    [sym_duet_dev_type_relay_device] = ACTIONS(1481),
    [sym_duet_dev_type_ups] = ACTIONS(1481),
    [sym_duet_dev_type_amplifier] = ACTIONS(1481),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1481),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1481),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1481),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1481),
    [sym_duet_dev_type_camera] = ACTIONS(1481),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1481),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1481),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1481),
    [sym_duet_dev_type_dss] = ACTIONS(1481),
    [sym_duet_dev_type_dvr] = ACTIONS(1481),
    [sym_duet_dev_type_disc_device] = ACTIONS(1481),
    [sym_duet_dev_type_document_camera] = ACTIONS(1481),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1481),
    [sym_duet_dev_type_hvac] = ACTIONS(1481),
    [sym_duet_dev_type_keypad] = ACTIONS(1481),
    [sym_duet_dev_type_light] = ACTIONS(1481),
    [sym_duet_dev_type_monitor] = ACTIONS(1481),
    [sym_duet_dev_type_motor] = ACTIONS(1481),
    [sym_duet_dev_type_multi_window] = ACTIONS(1481),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1481),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1481),
    [sym_duet_dev_type_receiver] = ACTIONS(1481),
    [sym_duet_dev_type_security_system] = ACTIONS(1481),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1481),
    [sym_duet_dev_type_settop_box] = ACTIONS(1481),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1481),
    [sym_duet_dev_type_switcher] = ACTIONS(1481),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1481),
    [sym_duet_dev_type_tv] = ACTIONS(1481),
    [sym_duet_dev_type_utility] = ACTIONS(1481),
    [sym_duet_dev_type_vcr] = ACTIONS(1481),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1481),
    [sym_duet_dev_type_video_processor] = ACTIONS(1481),
    [sym_duet_dev_type_video_projector] = ACTIONS(1481),
    [sym_duet_dev_type_video_wall] = ACTIONS(1481),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1481),
    [sym_duet_dev_type_weather] = ACTIONS(1481),
    [sym_duet_dev_not_polled] = ACTIONS(1481),
    [sym_duet_dev_polled] = ACTIONS(1481),
    [sym_clkmgr_mode_network] = ACTIONS(1481),
    [sym_clkmgr_mode_standalone] = ACTIONS(1481),
    [sym_normal_standby] = ACTIONS(1481),
    [sym_normal_wake] = ACTIONS(1481),
    [sym_ip_multicast_ttl_option] = ACTIONS(1481),
    [sym_ip_ttl_subnet] = ACTIONS(1481),
    [sym_ip_ttl_site] = ACTIONS(1481),
    [sym_ip_ttl_region] = ACTIONS(1481),
    [sym_ip_ttl_continent] = ACTIONS(1481),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1481),
    [sym_ip_nodelay_on] = ACTIONS(1481),
    [sym_ip_nodelay_off] = ACTIONS(1481),
    [sym_amx_error] = ACTIONS(1481),
    [sym_amx_warning] = ACTIONS(1481),
    [sym_amx_info] = ACTIONS(1481),
    [sym_amx_debug] = ACTIONS(1481),
    [sym_netlinx_login_success] = ACTIONS(1481),
    [sym_netlinx_login_fail] = ACTIONS(1481),
    [sym_netlinx_logout] = ACTIONS(1481),
    [sym_null_str] = ACTIONS(1481),
    [sym_smtp_address] = ACTIONS(1481),
    [sym_smtp_port_number] = ACTIONS(1481),
    [sym_smtp_username] = ACTIONS(1481),
    [sym_smtp_password] = ACTIONS(1481),
    [sym_smtp_require_tls] = ACTIONS(1481),
    [sym_smtp_from] = ACTIONS(1481),
    [sym_smtp_tls_true] = ACTIONS(1481),
    [sym_smtp_tls_false] = ACTIONS(1481),
    [sym_listview_on_row_select_event] = ACTIONS(1481),
    [sym_data_structure_unknown] = ACTIONS(1481),
    [sym_data_structure_datafeed] = ACTIONS(1481),
    [sym_data_structure_datarecordset] = ACTIONS(1481),
    [sym_data_structure_datarecord] = ACTIONS(1481),
    [sym_data_structure_datafield] = ACTIONS(1481),
    [sym_data_max_content_field_count] = ACTIONS(1481),
    [sym_data_max_metadata_field_count] = ACTIONS(1481),
    [sym_data_max_id_length] = ACTIONS(1481),
    [sym_data_max_name_length] = ACTIONS(1481),
    [sym_data_max_description_length] = ACTIONS(1481),
    [sym_data_max_type_length] = ACTIONS(1481),
    [sym_data_max_format_length] = ACTIONS(1481),
    [sym_data_max_source_length] = ACTIONS(1481),
    [sym_data_max_value_length] = ACTIONS(1481),
    [sym_data_max_label_length] = ACTIONS(1481),
    [sym_data_max_url_length] = ACTIONS(1481),
    [sym_max_last_login_info_length] = ACTIONS(1481),
    [sym_data_type_unknown] = ACTIONS(1481),
    [sym_data_type_string] = ACTIONS(1481),
    [sym_data_type_datetime] = ACTIONS(1481),
    [sym_data_type_date] = ACTIONS(1481),
    [sym_data_type_time] = ACTIONS(1481),
    [sym_data_type_image] = ACTIONS(1481),
    [sym_data_format_url] = ACTIONS(1481),
    [sym_data_format_phone] = ACTIONS(1481),
    [sym_data_format_email] = ACTIONS(1481),
    [sym_data_format_iso8601] = ACTIONS(1481),
    [sym_auditlog_priv] = ACTIONS(1481),
    [sym_configuration_priv] = ACTIONS(1481),
    [sym_deviceconfig_priv] = ACTIONS(1481),
    [sym_ftp_priv] = ACTIONS(1481),
    [sym_http_priv] = ACTIONS(1481),
    [sym_networkconfig_priv] = ACTIONS(1481),
    [sym_programport_priv] = ACTIONS(1481),
    [sym_remoteui_priv] = ACTIONS(1481),
    [sym_securitycontrol_priv] = ACTIONS(1481),
    [sym_softwaremanagement_priv] = ACTIONS(1481),
    [sym_terminal_priv] = ACTIONS(1481),
    [sym_tpadmin_priv] = ACTIONS(1481),
    [sym_user1_priv] = ACTIONS(1481),
    [sym_user2_priv] = ACTIONS(1481),
    [sym_user3_priv] = ACTIONS(1481),
    [sym_user4_priv] = ACTIONS(1481),
    [sym_usermanagement_priv] = ACTIONS(1481),
    [sym_valid_account] = ACTIONS(1481),
    [sym_err_name_invalid] = ACTIONS(1481),
    [sym_err_password_invalid] = ACTIONS(1481),
    [sym_err_invalid_account] = ACTIONS(1481),
    [sym_err_no_authorization] = ACTIONS(1481),
    [sym_err_invalid_parameter] = ACTIONS(1481),
    [sym_err_user_locked_out] = ACTIONS(1481),
    [sym_err_user_account_expired] = ACTIONS(1481),
    [sym_err_general_error] = ACTIONS(1481),
    [sym_err_authentication_error] = ACTIONS(1481),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1481),
    [sym_tls_validate_certificate] = ACTIONS(1481),
    [sym_tdata] = ACTIONS(1481),
    [sym_tchannel] = ACTIONS(1481),
    [sym_tlevel] = ACTIONS(1481),
    [sym_tbutton] = ACTIONS(1481),
    [sym_ttimeline] = ACTIONS(1481),
    [sym_tcustom] = ACTIONS(1481),
    [sym_url_struct] = ACTIONS(1481),
    [sym_dns_struct] = ACTIONS(1481),
    [sym_ip_address_struct] = ACTIONS(1481),
    [sym_dev_info_struct] = ACTIONS(1481),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1481),
    [sym_data_feed] = ACTIONS(1481),
    [sym_data_field] = ACTIONS(1481),
    [sym_data_record] = ACTIONS(1481),
    [sym_wc_data_feed] = ACTIONS(1481),
    [sym_wc_data_field] = ACTIONS(1481),
    [sym_wc_data_record] = ACTIONS(1481),
    [sym_last_login_info] = ACTIONS(1481),
    [sym____reserved___] = ACTIONS(1481),
    [sym_date] = ACTIONS(1481),
    [sym_day] = ACTIONS(1481),
    [sym_time] = ACTIONS(1481),
    [sym_ldate] = ACTIONS(1481),
    [sym_push_device] = ACTIONS(1481),
    [sym_push_channel] = ACTIONS(1481),
    [sym_push_devchan] = ACTIONS(1481),
    [sym_release_device] = ACTIONS(1481),
    [sym_release_channel] = ACTIONS(1481),
    [sym_release_devchan] = ACTIONS(1481),
    [sym_master_slot] = ACTIONS(1481),
    [sym_get_pulse_time] = ACTIONS(1481),
    [sym_get_timer] = ACTIONS(1481),
    [sym_master_sn] = ACTIONS(1481),
    [sym_system_number] = ACTIONS(1481),
    [sym_button] = ACTIONS(1481),
    [sym_dv_channel] = ACTIONS(1481),
    [sym_data] = ACTIONS(1481),
    [sym_level] = ACTIONS(1481),
    [sym_timeline] = ACTIONS(1481),
    [sym_channel] = ACTIONS(1481),
    [sym_custom] = ACTIONS(1481),
    [sym_atoi] = ACTIONS(1481),
    [sym_atol] = ACTIONS(1481),
    [sym_atof] = ACTIONS(1481),
    [sym_itoa] = ACTIONS(1481),
    [sym_itohex] = ACTIONS(1481),
    [sym_hextoi] = ACTIONS(1481),
    [sym_ftoa] = ACTIONS(1481),
    [sym_find_string] = ACTIONS(1481),
    [sym_left_string] = ACTIONS(1481),
    [sym_right_string] = ACTIONS(1481),
    [sym_mid_string] = ACTIONS(1481),
    [sym_remove_string] = ACTIONS(1481),
    [sym_get_buffer_string] = ACTIONS(1481),
    [sym_get_buffer_char] = ACTIONS(1481),
    [sym_get_multi_buffer_string] = ACTIONS(1481),
    [sym_length_string] = ACTIONS(1481),
    [sym_max_length_string] = ACTIONS(1481),
    [sym_set_length_string] = ACTIONS(1481),
    [sym_compare_string] = ACTIONS(1481),
    [sym_upper_string] = ACTIONS(1481),
    [sym_lower_string] = ACTIONS(1481),
    [sym_get_last] = ACTIONS(1481),
    [sym_date_to_day] = ACTIONS(1481),
    [sym_date_to_month] = ACTIONS(1481),
    [sym_date_to_year] = ACTIONS(1481),
    [sym_time_to_hour] = ACTIONS(1481),
    [sym_time_to_minute] = ACTIONS(1481),
    [sym_time_to_second] = ACTIONS(1481),
    [sym_day_of_week] = ACTIONS(1481),
    [sym_set_pulse_time] = ACTIONS(1481),
    [sym_set_timer] = ACTIONS(1481),
    [sym_file_open] = ACTIONS(1481),
    [sym_file_close] = ACTIONS(1481),
    [sym_file_read] = ACTIONS(1481),
    [sym_file_read_line] = ACTIONS(1481),
    [sym_file_write] = ACTIONS(1481),
    [sym_file_write_line] = ACTIONS(1481),
    [sym_file_copy] = ACTIONS(1481),
    [sym_file_delete] = ACTIONS(1481),
    [sym_file_rename] = ACTIONS(1481),
    [sym_file_dir] = ACTIONS(1481),
    [sym_file_getdir] = ACTIONS(1481),
    [sym_file_setdir] = ACTIONS(1481),
    [sym_file_create_dir] = ACTIONS(1481),
    [sym_file_remove_dir] = ACTIONS(1481),
    [sym_file_seek] = ACTIONS(1481),
    [sym_length_array] = ACTIONS(1481),
    [sym_max_length_array] = ACTIONS(1481),
    [sym_set_length_array] = ACTIONS(1481),
    [sym_ip_client_open] = ACTIONS(1481),
    [sym_ip_client_close] = ACTIONS(1481),
    [sym_ip_server_open] = ACTIONS(1481),
    [sym_ip_server_close] = ACTIONS(1481),
    [sym_ip_bound_client_open] = ACTIONS(1481),
    [sym_ip_mc_server_open] = ACTIONS(1481),
    [sym_ip_set_option] = ACTIONS(1481),
    [sym_ssh_client_open] = ACTIONS(1481),
    [sym_ssh_client_close] = ACTIONS(1481),
    [sym_tls_client_open] = ACTIONS(1481),
    [sym_tls_client_close] = ACTIONS(1481),
    [sym_do_push] = ACTIONS(1481),
    [sym_do_push_timed] = ACTIONS(1481),
    [sym_do_release] = ACTIONS(1481),
    [sym_device_id] = ACTIONS(1481),
    [sym_device_info] = ACTIONS(1481),
    [sym_device_id_string] = ACTIONS(1481),
    [sym_device_standby] = ACTIONS(1481),
    [sym_device_wake] = ACTIONS(1481),
    [sym_timeline_create] = ACTIONS(1481),
    [sym_timeline_kill] = ACTIONS(1481),
    [sym_timeline_reload] = ACTIONS(1481),
    [sym_timeline_pause] = ACTIONS(1481),
    [sym_timeline_restart] = ACTIONS(1481),
    [sym_timeline_set] = ACTIONS(1481),
    [sym_timeline_get] = ACTIONS(1481),
    [sym_timeline_active] = ACTIONS(1481),
    [sym_timeline_dynamic_id] = ACTIONS(1481),
    [sym_variable_to_string] = ACTIONS(1481),
    [sym_string_to_variable] = ACTIONS(1481),
    [sym_length_variable_to_string] = ACTIONS(1481),
    [sym_type_cast] = ACTIONS(1481),
    [sym_variable_to_xml] = ACTIONS(1481),
    [sym_xml_to_variable] = ACTIONS(1481),
    [sym_length_variable_to_xml] = ACTIONS(1481),
    [sym_redirect_string] = ACTIONS(1481),
    [sym_random_number] = ACTIONS(1481),
    [sym_get_unique_id] = ACTIONS(1481),
    [sym_get_system_number] = ACTIONS(1481),
    [sym_set_system_number] = ACTIONS(1481),
    [sym_get_serial_number] = ACTIONS(1481),
    [sym_get_master_build] = ACTIONS(1481),
    [sym_set_validation_code] = ACTIONS(1481),
    [sym_reboot] = ACTIONS(1481),
    [sym_load_duet_module] = ACTIONS(1481),
    [sym_unload_duet_module] = ACTIONS(1481),
    [sym_duet_mem_size_set] = ACTIONS(1481),
    [sym_duet_mem_size_get] = ACTIONS(1481),
    [sym_dynamic_application_device] = ACTIONS(1481),
    [sym_dynamic_polled_port] = ACTIONS(1481),
    [sym_static_port_binding] = ACTIONS(1481),
    [sym_static_ip_binding] = ACTIONS(1481),
    [sym_get_dns_list] = ACTIONS(1481),
    [sym_set_dns_list] = ACTIONS(1481),
    [sym_get_ip_address] = ACTIONS(1481),
    [sym_set_ip_address] = ACTIONS(1481),
    [sym_get_url_list] = ACTIONS(1481),
    [sym_add_url_entry] = ACTIONS(1481),
    [sym_delete_url_entry] = ACTIONS(1481),
    [sym_abs_value] = ACTIONS(1481),
    [sym_max_value] = ACTIONS(1481),
    [sym_min_value] = ACTIONS(1481),
    [sym_exp_value] = ACTIONS(1481),
    [sym_log_value] = ACTIONS(1481),
    [sym_log10_value] = ACTIONS(1481),
    [sym_power_value] = ACTIONS(1481),
    [sym_sqrt_value] = ACTIONS(1481),
    [sym_raw_be] = ACTIONS(1481),
    [sym_raw_le] = ACTIONS(1481),
    [sym_internal_queue_size_set] = ACTIONS(1481),
    [sym_internal_queue_size_get] = ACTIONS(1481),
    [sym_internal_threshold_set] = ACTIONS(1481),
    [sym_internal_threshold_get] = ACTIONS(1481),
    [sym_rebuilt_event] = ACTIONS(1481),
    [sym_do_custom_event] = ACTIONS(1481),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1481),
    [sym_clkmgr_set_clk_source] = ACTIONS(1481),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1481),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1481),
    [sym_clkmgr_get_timezone] = ACTIONS(1481),
    [sym_clkmgr_set_timezone] = ACTIONS(1481),
    [sym_clkmgr_get_resync_period] = ACTIONS(1481),
    [sym_clkmgr_set_resync_period] = ACTIONS(1481),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1481),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1481),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1481),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1481),
    [sym_clkmgr_get_timeservers] = ACTIONS(1481),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1481),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1481),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1481),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1481),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1481),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1481),
    [sym_set_log_level] = ACTIONS(1481),
    [sym_get_log_level] = ACTIONS(1481),
    [sym_amx_log] = ACTIONS(1481),
    [sym_smtp_server_config_set] = ACTIONS(1481),
    [sym_smtp_server_config_get] = ACTIONS(1481),
    [sym_smtp_send] = ACTIONS(1481),
    [sym_validate_netlinx_account] = ACTIONS(1481),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1481),
    [sym_authenticate_certificate] = ACTIONS(1481),
    [sym_audit_netlinx_session_event] = ACTIONS(1481),
    [sym_audit_netlinx_generic_event] = ACTIONS(1481),
    [sym_get_available_flash_disk_space] = ACTIONS(1481),
    [sym_get_max_flash_disk_space] = ACTIONS(1481),
    [sym_data_create_feed] = ACTIONS(1481),
    [sym_data_delete_feed] = ACTIONS(1481),
    [sym_data_publish_feed] = ACTIONS(1481),
    [sym_data_get_published_feed] = ACTIONS(1481),
    [sym_data_add_record] = ACTIONS(1481),
    [sym_data_get_event_record] = ACTIONS(1481),
    [sym__wc_data_create_feed] = ACTIONS(1481),
    [sym__wc_data_add_record] = ACTIONS(1481),
    [sym__wc_data_get_event_record] = ACTIONS(1481),
    [sym_format] = ACTIONS(1481),
    [sym_astro_clock] = ACTIONS(1481),
    [sym_set_outdoor_temperature] = ACTIONS(1481),
    [sym_set_virtual_level_count] = ACTIONS(1481),
    [sym_set_virtual_channel_count] = ACTIONS(1481),
    [sym_set_virtual_port_count] = ACTIONS(1481),
    [sym__wc_encode_int] = ACTIONS(1481),
    [sym__wc_decode_int] = ACTIONS(1481),
    [sym__wc_int] = ACTIONS(1481),
    [sym__wc_to_ch_int] = ACTIONS(1481),
    [sym__ch_to_wc_int] = ACTIONS(1481),
    [sym__wc_find_string_int] = ACTIONS(1481),
    [sym__wc_left_string_int] = ACTIONS(1481),
    [sym__wc_lower_string_int] = ACTIONS(1481),
    [sym__wc_mid_string_int] = ACTIONS(1481),
    [sym__wc_remove_string_int] = ACTIONS(1481),
    [sym__wc_right_string_int] = ACTIONS(1481),
    [sym__wc_upper_string_int] = ACTIONS(1481),
    [sym__wc_compare_string_int] = ACTIONS(1481),
    [sym__wc_get_buffer_char_int] = ACTIONS(1481),
    [sym__wc_get_buffer_string_int] = ACTIONS(1481),
    [sym__wc_concat_string_int] = ACTIONS(1481),
    [sym__wc_file_open_int] = ACTIONS(1481),
    [sym__wc_file_close_int] = ACTIONS(1481),
    [sym__wc_file_read_int] = ACTIONS(1481),
    [sym__wc_file_read_line_int] = ACTIONS(1481),
    [sym__wc_file_write_int] = ACTIONS(1481),
    [sym__wc_file_write_line_int] = ACTIONS(1481),
    [aux_sym_preproc_include_token1] = ACTIONS(1481),
    [aux_sym_preproc_define_token1] = ACTIONS(1483),
    [aux_sym_preproc_warn_token1] = ACTIONS(1483),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1483),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1483),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1483),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1483),
    [anon_sym_COLON] = ACTIONS(1518),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1483),
    [anon_sym_BANG] = ACTIONS(1481),
    [anon_sym_TILDE] = ACTIONS(1483),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1443),
    [anon_sym_AMP_AMP] = ACTIONS(1445),
    [anon_sym_PIPE] = ACTIONS(1425),
    [anon_sym_CARET] = ACTIONS(1447),
    [anon_sym_AMP] = ACTIONS(1423),
    [anon_sym_EQ_EQ] = ACTIONS(1449),
    [anon_sym_BANG_EQ] = ACTIONS(1449),
    [anon_sym_LT_GT] = ACTIONS(1449),
    [anon_sym_GT] = ACTIONS(1451),
    [anon_sym_GT_EQ] = ACTIONS(1453),
    [anon_sym_LT_EQ] = ACTIONS(1453),
    [anon_sym_LT] = ACTIONS(1451),
    [anon_sym_LT_LT] = ACTIONS(1455),
    [anon_sym_GT_GT] = ACTIONS(1455),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1483),
    [anon_sym_SQUOTE] = ACTIONS(1483),
    [sym_number_literal] = ACTIONS(1483),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1483),
  },
  [STATE(87)] = {
    [sym_identifier] = ACTIONS(1521),
    [sym_program_name_keyword] = ACTIONS(1521),
    [sym_module_name_keyword] = ACTIONS(1521),
    [sym_define_device_keyword] = ACTIONS(1521),
    [sym_define_combine_keyword] = ACTIONS(1521),
    [sym_define_constant_keyword] = ACTIONS(1521),
    [sym_define_type_keyword] = ACTIONS(1521),
    [sym_define_variable_keyword] = ACTIONS(1521),
    [sym_define_system_variable_keyword] = ACTIONS(1521),
    [sym_define_connect_level_keyword] = ACTIONS(1521),
    [sym_define_latching_keyword] = ACTIONS(1521),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1521),
    [sym_define_toggling_keyword] = ACTIONS(1521),
    [sym_define_call_keyword] = ACTIONS(1521),
    [sym_define_function_keyword] = ACTIONS(1521),
    [sym_define_library_function_keyword] = ACTIONS(1521),
    [sym_define_start_keyword] = ACTIONS(1521),
    [sym_define_event_keyword] = ACTIONS(1521),
    [sym_define_program_keyword] = ACTIONS(1521),
    [sym_define_module_keyword] = ACTIONS(1521),
    [sym_char_keyword] = ACTIONS(1521),
    [sym_widechar_keyword] = ACTIONS(1521),
    [sym_integer_keyword] = ACTIONS(1521),
    [sym_sinteger_keyword] = ACTIONS(1521),
    [sym_long_keyword] = ACTIONS(1521),
    [sym_slong_keyword] = ACTIONS(1521),
    [sym_float_keyword] = ACTIONS(1521),
    [sym_double_keyword] = ACTIONS(1521),
    [sym_variant_keyword] = ACTIONS(1521),
    [sym_variantarray_keyword] = ACTIONS(1521),
    [sym_local_var_keyword] = ACTIONS(1521),
    [sym_stack_var_keyword] = ACTIONS(1521),
    [sym_constant_keyword] = ACTIONS(1521),
    [sym_volatile_keyword] = ACTIONS(1521),
    [sym_non_volatile_keyword] = ACTIONS(1521),
    [sym_persistent_keyword] = ACTIONS(1521),
    [sym_dev_keyword] = ACTIONS(1521),
    [sym_devlev_keyword] = ACTIONS(1521),
    [sym_devchan_keyword] = ACTIONS(1521),
    [sym_if_keyword] = ACTIONS(1521),
    [sym_else_keyword] = ACTIONS(1521),
    [sym_switch_keyword] = ACTIONS(1521),
    [sym_case_keyword] = ACTIONS(1521),
    [sym_default_keyword] = ACTIONS(1521),
    [sym_while_keyword] = ACTIONS(1521),
    [sym_for_keyword] = ACTIONS(1521),
    [sym_break_keyword] = ACTIONS(1521),
    [sym_continue_keyword] = ACTIONS(1521),
    [sym_return_keyword] = ACTIONS(1521),
    [sym_select_keyword] = ACTIONS(1521),
    [sym_active_keyword] = ACTIONS(1521),
    [sym_struct_keyword] = ACTIONS(1521),
    [sym_structure_keyword] = ACTIONS(1521),
    [sym_button_event_keyword] = ACTIONS(1521),
    [sym_channel_event_keyword] = ACTIONS(1521),
    [sym_level_event_keyword] = ACTIONS(1521),
    [sym_data_event_keyword] = ACTIONS(1521),
    [sym_timeline_event_keyword] = ACTIONS(1521),
    [sym_custom_event_keyword] = ACTIONS(1521),
    [sym_push_keyword] = ACTIONS(1521),
    [sym_release_keyword] = ACTIONS(1521),
    [aux_sym_on_keyword_token1] = ACTIONS(1521),
    [aux_sym_off_keyword_token1] = ACTIONS(1521),
    [sym_send_level_keyword] = ACTIONS(1521),
    [sym_send_string_keyword] = ACTIONS(1521),
    [sym_send_command_keyword] = ACTIONS(1521),
    [sym_clear_buffer_keyword] = ACTIONS(1521),
    [sym_create_buffer_keyword] = ACTIONS(1521),
    [sym_create_multi_buffer_keyword] = ACTIONS(1521),
    [sym_call_keyword] = ACTIONS(1521),
    [sym_system_call_keyword] = ACTIONS(1521),
    [sym_devchan_to_keyword] = ACTIONS(1521),
    [sym_devchan_min_to_keyword] = ACTIONS(1521),
    [sym_devchan_total_off_keyword] = ACTIONS(1521),
    [sym_devchan_pulse_keyword] = ACTIONS(1521),
    [sym_wait_keyword] = ACTIONS(1521),
    [sym_cancel_wait_keyword] = ACTIONS(1521),
    [sym_wait_until_keyword] = ACTIONS(1521),
    [sym_cancel_wait_until_keyword] = ACTIONS(1521),
    [sym_cancel_all_wait_keyword] = ACTIONS(1521),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1521),
    [sym_and] = ACTIONS(1521),
    [sym_or] = ACTIONS(1521),
    [sym_not] = ACTIONS(1521),
    [sym_xor] = ACTIONS(1521),
    [sym_band] = ACTIONS(1521),
    [sym_bor] = ACTIONS(1521),
    [sym_bxor] = ACTIONS(1521),
    [sym_bnot] = ACTIONS(1521),
    [sym_lshift] = ACTIONS(1521),
    [sym_rshift] = ACTIONS(1521),
    [sym___file__] = ACTIONS(1521),
    [sym___date__] = ACTIONS(1521),
    [sym___ldate__] = ACTIONS(1521),
    [sym___time__] = ACTIONS(1521),
    [sym___line__] = ACTIONS(1521),
    [sym___name__] = ACTIONS(1521),
    [sym___netlinx__] = ACTIONS(1521),
    [sym_netlinx_axi_version] = ACTIONS(1521),
    [sym_url_flg_tcp] = ACTIONS(1521),
    [sym_url_flg_accinfopresent] = ACTIONS(1521),
    [sym_url_flg_temp] = ACTIONS(1521),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1521),
    [sym_url_flg_stat_mask] = ACTIONS(1521),
    [sym_url_flg_stat_lookup] = ACTIONS(1521),
    [sym_url_flg_stat_connecting] = ACTIONS(1521),
    [sym_url_flg_stat_waiting] = ACTIONS(1521),
    [sym_url_flg_stat_connected] = ACTIONS(1521),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1521),
    [sym_first_virtual_device] = ACTIONS(1521),
    [sym_first_local_port] = ACTIONS(1521),
    [sym_dynamic_virtual_device] = ACTIONS(1521),
    [sym_timeline_once] = ACTIONS(1521),
    [sym_timeline_repeat] = ACTIONS(1521),
    [sym_timeline_absolute] = ACTIONS(1521),
    [sym_timeline_relative] = ACTIONS(1521),
    [sym_do_push_timed_infinite] = ACTIONS(1521),
    [sym_source_type_no_address] = ACTIONS(1521),
    [sym_source_type_neuron_id] = ACTIONS(1521),
    [sym_source_type_ip_address] = ACTIONS(1521),
    [sym_source_type_axlink] = ACTIONS(1521),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1521),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1521),
    [sym_source_type_ip_socket_address] = ACTIONS(1521),
    [sym_source_type_rs232] = ACTIONS(1521),
    [sym_source_type_internal] = ACTIONS(1521),
    [sym_source_type_mac_address] = ACTIONS(1521),
    [sym_source_type_ipv4_port] = ACTIONS(1521),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1521),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1521),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1521),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1521),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1521),
    [sym_true] = ACTIONS(1521),
    [sym_false] = ACTIONS(1521),
    [sym_file_read_only] = ACTIONS(1521),
    [sym_file_rw_new] = ACTIONS(1521),
    [sym_file_rw_append] = ACTIONS(1521),
    [sym_ip_tcp] = ACTIONS(1521),
    [sym_ip_udp] = ACTIONS(1521),
    [sym_ip_udp_2way] = ACTIONS(1521),
    [sym_xml_encode_types] = ACTIONS(1521),
    [sym_xml_encode_char_as_list] = ACTIONS(1521),
    [sym_xml_encode_le] = ACTIONS(1521),
    [sym_xml_decode_types] = ACTIONS(1521),
    [sym_xml_decode_no_preserve] = ACTIONS(1521),
    [sym_duet_dev_type_display_device] = ACTIONS(1521),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1521),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1521),
    [sym_duet_dev_type_io_device] = ACTIONS(1521),
    [sym_duet_dev_type_relay_device] = ACTIONS(1521),
    [sym_duet_dev_type_ups] = ACTIONS(1521),
    [sym_duet_dev_type_amplifier] = ACTIONS(1521),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1521),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1521),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1521),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1521),
    [sym_duet_dev_type_camera] = ACTIONS(1521),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1521),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1521),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1521),
    [sym_duet_dev_type_dss] = ACTIONS(1521),
    [sym_duet_dev_type_dvr] = ACTIONS(1521),
    [sym_duet_dev_type_disc_device] = ACTIONS(1521),
    [sym_duet_dev_type_document_camera] = ACTIONS(1521),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1521),
    [sym_duet_dev_type_hvac] = ACTIONS(1521),
    [sym_duet_dev_type_keypad] = ACTIONS(1521),
    [sym_duet_dev_type_light] = ACTIONS(1521),
    [sym_duet_dev_type_monitor] = ACTIONS(1521),
    [sym_duet_dev_type_motor] = ACTIONS(1521),
    [sym_duet_dev_type_multi_window] = ACTIONS(1521),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1521),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1521),
    [sym_duet_dev_type_receiver] = ACTIONS(1521),
    [sym_duet_dev_type_security_system] = ACTIONS(1521),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1521),
    [sym_duet_dev_type_settop_box] = ACTIONS(1521),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1521),
    [sym_duet_dev_type_switcher] = ACTIONS(1521),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1521),
    [sym_duet_dev_type_tv] = ACTIONS(1521),
    [sym_duet_dev_type_utility] = ACTIONS(1521),
    [sym_duet_dev_type_vcr] = ACTIONS(1521),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1521),
    [sym_duet_dev_type_video_processor] = ACTIONS(1521),
    [sym_duet_dev_type_video_projector] = ACTIONS(1521),
    [sym_duet_dev_type_video_wall] = ACTIONS(1521),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1521),
    [sym_duet_dev_type_weather] = ACTIONS(1521),
    [sym_duet_dev_not_polled] = ACTIONS(1521),
    [sym_duet_dev_polled] = ACTIONS(1521),
    [sym_clkmgr_mode_network] = ACTIONS(1521),
    [sym_clkmgr_mode_standalone] = ACTIONS(1521),
    [sym_normal_standby] = ACTIONS(1521),
    [sym_normal_wake] = ACTIONS(1521),
    [sym_ip_multicast_ttl_option] = ACTIONS(1521),
    [sym_ip_ttl_subnet] = ACTIONS(1521),
    [sym_ip_ttl_site] = ACTIONS(1521),
    [sym_ip_ttl_region] = ACTIONS(1521),
    [sym_ip_ttl_continent] = ACTIONS(1521),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1521),
    [sym_ip_nodelay_on] = ACTIONS(1521),
    [sym_ip_nodelay_off] = ACTIONS(1521),
    [sym_amx_error] = ACTIONS(1521),
    [sym_amx_warning] = ACTIONS(1521),
    [sym_amx_info] = ACTIONS(1521),
    [sym_amx_debug] = ACTIONS(1521),
    [sym_netlinx_login_success] = ACTIONS(1521),
    [sym_netlinx_login_fail] = ACTIONS(1521),
    [sym_netlinx_logout] = ACTIONS(1521),
    [sym_null_str] = ACTIONS(1521),
    [sym_smtp_address] = ACTIONS(1521),
    [sym_smtp_port_number] = ACTIONS(1521),
    [sym_smtp_username] = ACTIONS(1521),
    [sym_smtp_password] = ACTIONS(1521),
    [sym_smtp_require_tls] = ACTIONS(1521),
    [sym_smtp_from] = ACTIONS(1521),
    [sym_smtp_tls_true] = ACTIONS(1521),
    [sym_smtp_tls_false] = ACTIONS(1521),
    [sym_listview_on_row_select_event] = ACTIONS(1521),
    [sym_data_structure_unknown] = ACTIONS(1521),
    [sym_data_structure_datafeed] = ACTIONS(1521),
    [sym_data_structure_datarecordset] = ACTIONS(1521),
    [sym_data_structure_datarecord] = ACTIONS(1521),
    [sym_data_structure_datafield] = ACTIONS(1521),
    [sym_data_max_content_field_count] = ACTIONS(1521),
    [sym_data_max_metadata_field_count] = ACTIONS(1521),
    [sym_data_max_id_length] = ACTIONS(1521),
    [sym_data_max_name_length] = ACTIONS(1521),
    [sym_data_max_description_length] = ACTIONS(1521),
    [sym_data_max_type_length] = ACTIONS(1521),
    [sym_data_max_format_length] = ACTIONS(1521),
    [sym_data_max_source_length] = ACTIONS(1521),
    [sym_data_max_value_length] = ACTIONS(1521),
    [sym_data_max_label_length] = ACTIONS(1521),
    [sym_data_max_url_length] = ACTIONS(1521),
    [sym_max_last_login_info_length] = ACTIONS(1521),
    [sym_data_type_unknown] = ACTIONS(1521),
    [sym_data_type_string] = ACTIONS(1521),
    [sym_data_type_datetime] = ACTIONS(1521),
    [sym_data_type_date] = ACTIONS(1521),
    [sym_data_type_time] = ACTIONS(1521),
    [sym_data_type_image] = ACTIONS(1521),
    [sym_data_format_url] = ACTIONS(1521),
    [sym_data_format_phone] = ACTIONS(1521),
    [sym_data_format_email] = ACTIONS(1521),
    [sym_data_format_iso8601] = ACTIONS(1521),
    [sym_auditlog_priv] = ACTIONS(1521),
    [sym_configuration_priv] = ACTIONS(1521),
    [sym_deviceconfig_priv] = ACTIONS(1521),
    [sym_ftp_priv] = ACTIONS(1521),
    [sym_http_priv] = ACTIONS(1521),
    [sym_networkconfig_priv] = ACTIONS(1521),
    [sym_programport_priv] = ACTIONS(1521),
    [sym_remoteui_priv] = ACTIONS(1521),
    [sym_securitycontrol_priv] = ACTIONS(1521),
    [sym_softwaremanagement_priv] = ACTIONS(1521),
    [sym_terminal_priv] = ACTIONS(1521),
    [sym_tpadmin_priv] = ACTIONS(1521),
    [sym_user1_priv] = ACTIONS(1521),
    [sym_user2_priv] = ACTIONS(1521),
    [sym_user3_priv] = ACTIONS(1521),
    [sym_user4_priv] = ACTIONS(1521),
    [sym_usermanagement_priv] = ACTIONS(1521),
    [sym_valid_account] = ACTIONS(1521),
    [sym_err_name_invalid] = ACTIONS(1521),
    [sym_err_password_invalid] = ACTIONS(1521),
    [sym_err_invalid_account] = ACTIONS(1521),
    [sym_err_no_authorization] = ACTIONS(1521),
    [sym_err_invalid_parameter] = ACTIONS(1521),
    [sym_err_user_locked_out] = ACTIONS(1521),
    [sym_err_user_account_expired] = ACTIONS(1521),
    [sym_err_general_error] = ACTIONS(1521),
    [sym_err_authentication_error] = ACTIONS(1521),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1521),
    [sym_tls_validate_certificate] = ACTIONS(1521),
    [sym_tdata] = ACTIONS(1521),
    [sym_tchannel] = ACTIONS(1521),
    [sym_tlevel] = ACTIONS(1521),
    [sym_tbutton] = ACTIONS(1521),
    [sym_ttimeline] = ACTIONS(1521),
    [sym_tcustom] = ACTIONS(1521),
    [sym_url_struct] = ACTIONS(1521),
    [sym_dns_struct] = ACTIONS(1521),
    [sym_ip_address_struct] = ACTIONS(1521),
    [sym_dev_info_struct] = ACTIONS(1521),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1521),
    [sym_data_feed] = ACTIONS(1521),
    [sym_data_field] = ACTIONS(1521),
    [sym_data_record] = ACTIONS(1521),
    [sym_wc_data_feed] = ACTIONS(1521),
    [sym_wc_data_field] = ACTIONS(1521),
    [sym_wc_data_record] = ACTIONS(1521),
    [sym_last_login_info] = ACTIONS(1521),
    [sym____reserved___] = ACTIONS(1521),
    [sym_date] = ACTIONS(1521),
    [sym_day] = ACTIONS(1521),
    [sym_time] = ACTIONS(1521),
    [sym_ldate] = ACTIONS(1521),
    [sym_push_device] = ACTIONS(1521),
    [sym_push_channel] = ACTIONS(1521),
    [sym_push_devchan] = ACTIONS(1521),
    [sym_release_device] = ACTIONS(1521),
    [sym_release_channel] = ACTIONS(1521),
    [sym_release_devchan] = ACTIONS(1521),
    [sym_master_slot] = ACTIONS(1521),
    [sym_get_pulse_time] = ACTIONS(1521),
    [sym_get_timer] = ACTIONS(1521),
    [sym_master_sn] = ACTIONS(1521),
    [sym_system_number] = ACTIONS(1521),
    [sym_button] = ACTIONS(1521),
    [sym_dv_channel] = ACTIONS(1521),
    [sym_data] = ACTIONS(1521),
    [sym_level] = ACTIONS(1521),
    [sym_timeline] = ACTIONS(1521),
    [sym_channel] = ACTIONS(1521),
    [sym_custom] = ACTIONS(1521),
    [sym_atoi] = ACTIONS(1521),
    [sym_atol] = ACTIONS(1521),
    [sym_atof] = ACTIONS(1521),
    [sym_itoa] = ACTIONS(1521),
    [sym_itohex] = ACTIONS(1521),
    [sym_hextoi] = ACTIONS(1521),
    [sym_ftoa] = ACTIONS(1521),
    [sym_find_string] = ACTIONS(1521),
    [sym_left_string] = ACTIONS(1521),
    [sym_right_string] = ACTIONS(1521),
    [sym_mid_string] = ACTIONS(1521),
    [sym_remove_string] = ACTIONS(1521),
    [sym_get_buffer_string] = ACTIONS(1521),
    [sym_get_buffer_char] = ACTIONS(1521),
    [sym_get_multi_buffer_string] = ACTIONS(1521),
    [sym_length_string] = ACTIONS(1521),
    [sym_max_length_string] = ACTIONS(1521),
    [sym_set_length_string] = ACTIONS(1521),
    [sym_compare_string] = ACTIONS(1521),
    [sym_upper_string] = ACTIONS(1521),
    [sym_lower_string] = ACTIONS(1521),
    [sym_get_last] = ACTIONS(1521),
    [sym_date_to_day] = ACTIONS(1521),
    [sym_date_to_month] = ACTIONS(1521),
    [sym_date_to_year] = ACTIONS(1521),
    [sym_time_to_hour] = ACTIONS(1521),
    [sym_time_to_minute] = ACTIONS(1521),
    [sym_time_to_second] = ACTIONS(1521),
    [sym_day_of_week] = ACTIONS(1521),
    [sym_set_pulse_time] = ACTIONS(1521),
    [sym_set_timer] = ACTIONS(1521),
    [sym_file_open] = ACTIONS(1521),
    [sym_file_close] = ACTIONS(1521),
    [sym_file_read] = ACTIONS(1521),
    [sym_file_read_line] = ACTIONS(1521),
    [sym_file_write] = ACTIONS(1521),
    [sym_file_write_line] = ACTIONS(1521),
    [sym_file_copy] = ACTIONS(1521),
    [sym_file_delete] = ACTIONS(1521),
    [sym_file_rename] = ACTIONS(1521),
    [sym_file_dir] = ACTIONS(1521),
    [sym_file_getdir] = ACTIONS(1521),
    [sym_file_setdir] = ACTIONS(1521),
    [sym_file_create_dir] = ACTIONS(1521),
    [sym_file_remove_dir] = ACTIONS(1521),
    [sym_file_seek] = ACTIONS(1521),
    [sym_length_array] = ACTIONS(1521),
    [sym_max_length_array] = ACTIONS(1521),
    [sym_set_length_array] = ACTIONS(1521),
    [sym_ip_client_open] = ACTIONS(1521),
    [sym_ip_client_close] = ACTIONS(1521),
    [sym_ip_server_open] = ACTIONS(1521),
    [sym_ip_server_close] = ACTIONS(1521),
    [sym_ip_bound_client_open] = ACTIONS(1521),
    [sym_ip_mc_server_open] = ACTIONS(1521),
    [sym_ip_set_option] = ACTIONS(1521),
    [sym_ssh_client_open] = ACTIONS(1521),
    [sym_ssh_client_close] = ACTIONS(1521),
    [sym_tls_client_open] = ACTIONS(1521),
    [sym_tls_client_close] = ACTIONS(1521),
    [sym_do_push] = ACTIONS(1521),
    [sym_do_push_timed] = ACTIONS(1521),
    [sym_do_release] = ACTIONS(1521),
    [sym_device_id] = ACTIONS(1521),
    [sym_device_info] = ACTIONS(1521),
    [sym_device_id_string] = ACTIONS(1521),
    [sym_device_standby] = ACTIONS(1521),
    [sym_device_wake] = ACTIONS(1521),
    [sym_timeline_create] = ACTIONS(1521),
    [sym_timeline_kill] = ACTIONS(1521),
    [sym_timeline_reload] = ACTIONS(1521),
    [sym_timeline_pause] = ACTIONS(1521),
    [sym_timeline_restart] = ACTIONS(1521),
    [sym_timeline_set] = ACTIONS(1521),
    [sym_timeline_get] = ACTIONS(1521),
    [sym_timeline_active] = ACTIONS(1521),
    [sym_timeline_dynamic_id] = ACTIONS(1521),
    [sym_variable_to_string] = ACTIONS(1521),
    [sym_string_to_variable] = ACTIONS(1521),
    [sym_length_variable_to_string] = ACTIONS(1521),
    [sym_type_cast] = ACTIONS(1521),
    [sym_variable_to_xml] = ACTIONS(1521),
    [sym_xml_to_variable] = ACTIONS(1521),
    [sym_length_variable_to_xml] = ACTIONS(1521),
    [sym_redirect_string] = ACTIONS(1521),
    [sym_random_number] = ACTIONS(1521),
    [sym_get_unique_id] = ACTIONS(1521),
    [sym_get_system_number] = ACTIONS(1521),
    [sym_set_system_number] = ACTIONS(1521),
    [sym_get_serial_number] = ACTIONS(1521),
    [sym_get_master_build] = ACTIONS(1521),
    [sym_set_validation_code] = ACTIONS(1521),
    [sym_reboot] = ACTIONS(1521),
    [sym_load_duet_module] = ACTIONS(1521),
    [sym_unload_duet_module] = ACTIONS(1521),
    [sym_duet_mem_size_set] = ACTIONS(1521),
    [sym_duet_mem_size_get] = ACTIONS(1521),
    [sym_dynamic_application_device] = ACTIONS(1521),
    [sym_dynamic_polled_port] = ACTIONS(1521),
    [sym_static_port_binding] = ACTIONS(1521),
    [sym_static_ip_binding] = ACTIONS(1521),
    [sym_get_dns_list] = ACTIONS(1521),
    [sym_set_dns_list] = ACTIONS(1521),
    [sym_get_ip_address] = ACTIONS(1521),
    [sym_set_ip_address] = ACTIONS(1521),
    [sym_get_url_list] = ACTIONS(1521),
    [sym_add_url_entry] = ACTIONS(1521),
    [sym_delete_url_entry] = ACTIONS(1521),
    [sym_abs_value] = ACTIONS(1521),
    [sym_max_value] = ACTIONS(1521),
    [sym_min_value] = ACTIONS(1521),
    [sym_exp_value] = ACTIONS(1521),
    [sym_log_value] = ACTIONS(1521),
    [sym_log10_value] = ACTIONS(1521),
    [sym_power_value] = ACTIONS(1521),
    [sym_sqrt_value] = ACTIONS(1521),
    [sym_raw_be] = ACTIONS(1521),
    [sym_raw_le] = ACTIONS(1521),
    [sym_internal_queue_size_set] = ACTIONS(1521),
    [sym_internal_queue_size_get] = ACTIONS(1521),
    [sym_internal_threshold_set] = ACTIONS(1521),
    [sym_internal_threshold_get] = ACTIONS(1521),
    [sym_rebuilt_event] = ACTIONS(1521),
    [sym_do_custom_event] = ACTIONS(1521),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1521),
    [sym_clkmgr_set_clk_source] = ACTIONS(1521),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1521),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1521),
    [sym_clkmgr_get_timezone] = ACTIONS(1521),
    [sym_clkmgr_set_timezone] = ACTIONS(1521),
    [sym_clkmgr_get_resync_period] = ACTIONS(1521),
    [sym_clkmgr_set_resync_period] = ACTIONS(1521),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1521),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1521),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1521),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1521),
    [sym_clkmgr_get_timeservers] = ACTIONS(1521),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1521),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1521),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1521),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1521),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1521),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1521),
    [sym_set_log_level] = ACTIONS(1521),
    [sym_get_log_level] = ACTIONS(1521),
    [sym_amx_log] = ACTIONS(1521),
    [sym_smtp_server_config_set] = ACTIONS(1521),
    [sym_smtp_server_config_get] = ACTIONS(1521),
    [sym_smtp_send] = ACTIONS(1521),
    [sym_validate_netlinx_account] = ACTIONS(1521),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1521),
    [sym_authenticate_certificate] = ACTIONS(1521),
    [sym_audit_netlinx_session_event] = ACTIONS(1521),
    [sym_audit_netlinx_generic_event] = ACTIONS(1521),
    [sym_get_available_flash_disk_space] = ACTIONS(1521),
    [sym_get_max_flash_disk_space] = ACTIONS(1521),
    [sym_data_create_feed] = ACTIONS(1521),
    [sym_data_delete_feed] = ACTIONS(1521),
    [sym_data_publish_feed] = ACTIONS(1521),
    [sym_data_get_published_feed] = ACTIONS(1521),
    [sym_data_add_record] = ACTIONS(1521),
    [sym_data_get_event_record] = ACTIONS(1521),
    [sym__wc_data_create_feed] = ACTIONS(1521),
    [sym__wc_data_add_record] = ACTIONS(1521),
    [sym__wc_data_get_event_record] = ACTIONS(1521),
    [sym_format] = ACTIONS(1521),
    [sym_astro_clock] = ACTIONS(1521),
    [sym_set_outdoor_temperature] = ACTIONS(1521),
    [sym_set_virtual_level_count] = ACTIONS(1521),
    [sym_set_virtual_channel_count] = ACTIONS(1521),
    [sym_set_virtual_port_count] = ACTIONS(1521),
    [sym__wc_encode_int] = ACTIONS(1521),
    [sym__wc_decode_int] = ACTIONS(1521),
    [sym__wc_int] = ACTIONS(1521),
    [sym__wc_to_ch_int] = ACTIONS(1521),
    [sym__ch_to_wc_int] = ACTIONS(1521),
    [sym__wc_find_string_int] = ACTIONS(1521),
    [sym__wc_left_string_int] = ACTIONS(1521),
    [sym__wc_lower_string_int] = ACTIONS(1521),
    [sym__wc_mid_string_int] = ACTIONS(1521),
    [sym__wc_remove_string_int] = ACTIONS(1521),
    [sym__wc_right_string_int] = ACTIONS(1521),
    [sym__wc_upper_string_int] = ACTIONS(1521),
    [sym__wc_compare_string_int] = ACTIONS(1521),
    [sym__wc_get_buffer_char_int] = ACTIONS(1521),
    [sym__wc_get_buffer_string_int] = ACTIONS(1521),
    [sym__wc_concat_string_int] = ACTIONS(1521),
    [sym__wc_file_open_int] = ACTIONS(1521),
    [sym__wc_file_close_int] = ACTIONS(1521),
    [sym__wc_file_read_int] = ACTIONS(1521),
    [sym__wc_file_read_line_int] = ACTIONS(1521),
    [sym__wc_file_write_int] = ACTIONS(1521),
    [sym__wc_file_write_line_int] = ACTIONS(1521),
    [anon_sym_EQ] = ACTIONS(1521),
    [aux_sym_preproc_include_token1] = ACTIONS(1521),
    [aux_sym_preproc_define_token1] = ACTIONS(1523),
    [aux_sym_preproc_warn_token1] = ACTIONS(1523),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1523),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1523),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1523),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1523),
    [aux_sym_preproc_else_token1] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(1523),
    [anon_sym_LBRACE] = ACTIONS(1523),
    [anon_sym_COLON] = ACTIONS(1523),
    [anon_sym_LPAREN] = ACTIONS(1521),
    [anon_sym_SEMI] = ACTIONS(1523),
    [anon_sym_BANG] = ACTIONS(1521),
    [anon_sym_TILDE] = ACTIONS(1523),
    [anon_sym_DASH] = ACTIONS(1521),
    [anon_sym_PLUS] = ACTIONS(1521),
    [anon_sym_STAR] = ACTIONS(1523),
    [anon_sym_SLASH] = ACTIONS(1521),
    [anon_sym_PERCENT] = ACTIONS(1523),
    [anon_sym_PIPE_PIPE] = ACTIONS(1523),
    [anon_sym_AMP_AMP] = ACTIONS(1523),
    [anon_sym_PIPE] = ACTIONS(1521),
    [anon_sym_CARET] = ACTIONS(1523),
    [anon_sym_AMP] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1523),
    [anon_sym_BANG_EQ] = ACTIONS(1523),
    [anon_sym_LT_GT] = ACTIONS(1523),
    [anon_sym_GT] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1523),
    [anon_sym_LT_EQ] = ACTIONS(1523),
    [anon_sym_LT] = ACTIONS(1521),
    [anon_sym_LT_LT] = ACTIONS(1523),
    [anon_sym_GT_GT] = ACTIONS(1523),
    [anon_sym_DASH_DASH] = ACTIONS(1523),
    [anon_sym_PLUS_PLUS] = ACTIONS(1523),
    [anon_sym_DOT] = ACTIONS(1521),
    [anon_sym_DQUOTE] = ACTIONS(1523),
    [anon_sym_SQUOTE] = ACTIONS(1523),
    [sym_number_literal] = ACTIONS(1523),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1523),
  },
  [STATE(88)] = {
    [sym_identifier] = ACTIONS(1465),
    [sym_program_name_keyword] = ACTIONS(1465),
    [sym_module_name_keyword] = ACTIONS(1465),
    [sym_define_device_keyword] = ACTIONS(1465),
    [sym_define_combine_keyword] = ACTIONS(1465),
    [sym_define_constant_keyword] = ACTIONS(1465),
    [sym_define_type_keyword] = ACTIONS(1465),
    [sym_define_variable_keyword] = ACTIONS(1465),
    [sym_define_system_variable_keyword] = ACTIONS(1465),
    [sym_define_connect_level_keyword] = ACTIONS(1465),
    [sym_define_latching_keyword] = ACTIONS(1465),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1465),
    [sym_define_toggling_keyword] = ACTIONS(1465),
    [sym_define_call_keyword] = ACTIONS(1465),
    [sym_define_function_keyword] = ACTIONS(1465),
    [sym_define_library_function_keyword] = ACTIONS(1465),
    [sym_define_start_keyword] = ACTIONS(1465),
    [sym_define_event_keyword] = ACTIONS(1465),
    [sym_define_program_keyword] = ACTIONS(1465),
    [sym_define_module_keyword] = ACTIONS(1465),
    [sym_char_keyword] = ACTIONS(1465),
    [sym_widechar_keyword] = ACTIONS(1465),
    [sym_integer_keyword] = ACTIONS(1465),
    [sym_sinteger_keyword] = ACTIONS(1465),
    [sym_long_keyword] = ACTIONS(1465),
    [sym_slong_keyword] = ACTIONS(1465),
    [sym_float_keyword] = ACTIONS(1465),
    [sym_double_keyword] = ACTIONS(1465),
    [sym_variant_keyword] = ACTIONS(1465),
    [sym_variantarray_keyword] = ACTIONS(1465),
    [sym_local_var_keyword] = ACTIONS(1465),
    [sym_stack_var_keyword] = ACTIONS(1465),
    [sym_constant_keyword] = ACTIONS(1465),
    [sym_volatile_keyword] = ACTIONS(1465),
    [sym_non_volatile_keyword] = ACTIONS(1465),
    [sym_persistent_keyword] = ACTIONS(1465),
    [sym_dev_keyword] = ACTIONS(1465),
    [sym_devlev_keyword] = ACTIONS(1465),
    [sym_devchan_keyword] = ACTIONS(1465),
    [sym_if_keyword] = ACTIONS(1465),
    [sym_else_keyword] = ACTIONS(1465),
    [sym_switch_keyword] = ACTIONS(1465),
    [sym_case_keyword] = ACTIONS(1465),
    [sym_default_keyword] = ACTIONS(1465),
    [sym_while_keyword] = ACTIONS(1465),
    [sym_for_keyword] = ACTIONS(1465),
    [sym_break_keyword] = ACTIONS(1465),
    [sym_continue_keyword] = ACTIONS(1465),
    [sym_return_keyword] = ACTIONS(1465),
    [sym_select_keyword] = ACTIONS(1465),
    [sym_active_keyword] = ACTIONS(1465),
    [sym_struct_keyword] = ACTIONS(1465),
    [sym_structure_keyword] = ACTIONS(1465),
    [sym_button_event_keyword] = ACTIONS(1465),
    [sym_channel_event_keyword] = ACTIONS(1465),
    [sym_level_event_keyword] = ACTIONS(1465),
    [sym_data_event_keyword] = ACTIONS(1465),
    [sym_timeline_event_keyword] = ACTIONS(1465),
    [sym_custom_event_keyword] = ACTIONS(1465),
    [sym_push_keyword] = ACTIONS(1465),
    [sym_release_keyword] = ACTIONS(1465),
    [aux_sym_on_keyword_token1] = ACTIONS(1465),
    [aux_sym_off_keyword_token1] = ACTIONS(1465),
    [sym_send_level_keyword] = ACTIONS(1465),
    [sym_send_string_keyword] = ACTIONS(1465),
    [sym_send_command_keyword] = ACTIONS(1465),
    [sym_clear_buffer_keyword] = ACTIONS(1465),
    [sym_create_buffer_keyword] = ACTIONS(1465),
    [sym_create_multi_buffer_keyword] = ACTIONS(1465),
    [sym_call_keyword] = ACTIONS(1465),
    [sym_system_call_keyword] = ACTIONS(1465),
    [sym_devchan_to_keyword] = ACTIONS(1465),
    [sym_devchan_min_to_keyword] = ACTIONS(1465),
    [sym_devchan_total_off_keyword] = ACTIONS(1465),
    [sym_devchan_pulse_keyword] = ACTIONS(1465),
    [sym_wait_keyword] = ACTIONS(1465),
    [sym_cancel_wait_keyword] = ACTIONS(1465),
    [sym_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym_tdata] = ACTIONS(1465),
    [sym_tchannel] = ACTIONS(1465),
    [sym_tlevel] = ACTIONS(1465),
    [sym_tbutton] = ACTIONS(1465),
    [sym_ttimeline] = ACTIONS(1465),
    [sym_tcustom] = ACTIONS(1465),
    [sym_url_struct] = ACTIONS(1465),
    [sym_dns_struct] = ACTIONS(1465),
    [sym_ip_address_struct] = ACTIONS(1465),
    [sym_dev_info_struct] = ACTIONS(1465),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1465),
    [sym_data_feed] = ACTIONS(1465),
    [sym_data_field] = ACTIONS(1465),
    [sym_data_record] = ACTIONS(1465),
    [sym_wc_data_feed] = ACTIONS(1465),
    [sym_wc_data_field] = ACTIONS(1465),
    [sym_wc_data_record] = ACTIONS(1465),
    [sym_last_login_info] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [anon_sym_EQ] = ACTIONS(1525),
    [aux_sym_preproc_include_token1] = ACTIONS(1465),
    [aux_sym_preproc_define_token1] = ACTIONS(1469),
    [aux_sym_preproc_warn_token1] = ACTIONS(1469),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_else_token1] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1469),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1469),
  },
  [STATE(89)] = {
    [sym_identifier] = ACTIONS(1465),
    [sym_program_name_keyword] = ACTIONS(1465),
    [sym_module_name_keyword] = ACTIONS(1465),
    [sym_define_device_keyword] = ACTIONS(1465),
    [sym_define_combine_keyword] = ACTIONS(1465),
    [sym_define_constant_keyword] = ACTIONS(1465),
    [sym_define_type_keyword] = ACTIONS(1465),
    [sym_define_variable_keyword] = ACTIONS(1465),
    [sym_define_system_variable_keyword] = ACTIONS(1465),
    [sym_define_connect_level_keyword] = ACTIONS(1465),
    [sym_define_latching_keyword] = ACTIONS(1465),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1465),
    [sym_define_toggling_keyword] = ACTIONS(1465),
    [sym_define_call_keyword] = ACTIONS(1465),
    [sym_define_function_keyword] = ACTIONS(1465),
    [sym_define_library_function_keyword] = ACTIONS(1465),
    [sym_define_start_keyword] = ACTIONS(1465),
    [sym_define_event_keyword] = ACTIONS(1465),
    [sym_define_program_keyword] = ACTIONS(1465),
    [sym_define_module_keyword] = ACTIONS(1465),
    [sym_char_keyword] = ACTIONS(1465),
    [sym_widechar_keyword] = ACTIONS(1465),
    [sym_integer_keyword] = ACTIONS(1465),
    [sym_sinteger_keyword] = ACTIONS(1465),
    [sym_long_keyword] = ACTIONS(1465),
    [sym_slong_keyword] = ACTIONS(1465),
    [sym_float_keyword] = ACTIONS(1465),
    [sym_double_keyword] = ACTIONS(1465),
    [sym_variant_keyword] = ACTIONS(1465),
    [sym_variantarray_keyword] = ACTIONS(1465),
    [sym_local_var_keyword] = ACTIONS(1465),
    [sym_stack_var_keyword] = ACTIONS(1465),
    [sym_constant_keyword] = ACTIONS(1465),
    [sym_volatile_keyword] = ACTIONS(1465),
    [sym_non_volatile_keyword] = ACTIONS(1465),
    [sym_persistent_keyword] = ACTIONS(1465),
    [sym_dev_keyword] = ACTIONS(1465),
    [sym_devlev_keyword] = ACTIONS(1465),
    [sym_devchan_keyword] = ACTIONS(1465),
    [sym_if_keyword] = ACTIONS(1465),
    [sym_else_keyword] = ACTIONS(1465),
    [sym_switch_keyword] = ACTIONS(1465),
    [sym_case_keyword] = ACTIONS(1465),
    [sym_default_keyword] = ACTIONS(1465),
    [sym_while_keyword] = ACTIONS(1465),
    [sym_for_keyword] = ACTIONS(1465),
    [sym_break_keyword] = ACTIONS(1465),
    [sym_continue_keyword] = ACTIONS(1465),
    [sym_return_keyword] = ACTIONS(1465),
    [sym_select_keyword] = ACTIONS(1465),
    [sym_active_keyword] = ACTIONS(1465),
    [sym_struct_keyword] = ACTIONS(1465),
    [sym_structure_keyword] = ACTIONS(1465),
    [sym_button_event_keyword] = ACTIONS(1465),
    [sym_channel_event_keyword] = ACTIONS(1465),
    [sym_level_event_keyword] = ACTIONS(1465),
    [sym_data_event_keyword] = ACTIONS(1465),
    [sym_timeline_event_keyword] = ACTIONS(1465),
    [sym_custom_event_keyword] = ACTIONS(1465),
    [sym_push_keyword] = ACTIONS(1465),
    [sym_release_keyword] = ACTIONS(1465),
    [aux_sym_on_keyword_token1] = ACTIONS(1465),
    [aux_sym_off_keyword_token1] = ACTIONS(1465),
    [sym_send_level_keyword] = ACTIONS(1465),
    [sym_send_string_keyword] = ACTIONS(1465),
    [sym_send_command_keyword] = ACTIONS(1465),
    [sym_clear_buffer_keyword] = ACTIONS(1465),
    [sym_create_buffer_keyword] = ACTIONS(1465),
    [sym_create_multi_buffer_keyword] = ACTIONS(1465),
    [sym_call_keyword] = ACTIONS(1465),
    [sym_system_call_keyword] = ACTIONS(1465),
    [sym_devchan_to_keyword] = ACTIONS(1465),
    [sym_devchan_min_to_keyword] = ACTIONS(1465),
    [sym_devchan_total_off_keyword] = ACTIONS(1465),
    [sym_devchan_pulse_keyword] = ACTIONS(1465),
    [sym_wait_keyword] = ACTIONS(1465),
    [sym_cancel_wait_keyword] = ACTIONS(1465),
    [sym_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym_tdata] = ACTIONS(1465),
    [sym_tchannel] = ACTIONS(1465),
    [sym_tlevel] = ACTIONS(1465),
    [sym_tbutton] = ACTIONS(1465),
    [sym_ttimeline] = ACTIONS(1465),
    [sym_tcustom] = ACTIONS(1465),
    [sym_url_struct] = ACTIONS(1465),
    [sym_dns_struct] = ACTIONS(1465),
    [sym_ip_address_struct] = ACTIONS(1465),
    [sym_dev_info_struct] = ACTIONS(1465),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1465),
    [sym_data_feed] = ACTIONS(1465),
    [sym_data_field] = ACTIONS(1465),
    [sym_data_record] = ACTIONS(1465),
    [sym_wc_data_feed] = ACTIONS(1465),
    [sym_wc_data_field] = ACTIONS(1465),
    [sym_wc_data_record] = ACTIONS(1465),
    [sym_last_login_info] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [anon_sym_EQ] = ACTIONS(1467),
    [aux_sym_preproc_include_token1] = ACTIONS(1465),
    [aux_sym_preproc_define_token1] = ACTIONS(1469),
    [aux_sym_preproc_warn_token1] = ACTIONS(1469),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_else_token1] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1469),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1469),
  },
  [STATE(90)] = {
    [ts_builtin_sym_end] = ACTIONS(1469),
    [sym_identifier] = ACTIONS(1465),
    [sym_program_name_keyword] = ACTIONS(1465),
    [sym_module_name_keyword] = ACTIONS(1465),
    [sym_define_device_keyword] = ACTIONS(1465),
    [sym_define_combine_keyword] = ACTIONS(1465),
    [sym_define_constant_keyword] = ACTIONS(1465),
    [sym_define_type_keyword] = ACTIONS(1465),
    [sym_define_variable_keyword] = ACTIONS(1465),
    [sym_define_system_variable_keyword] = ACTIONS(1465),
    [sym_define_connect_level_keyword] = ACTIONS(1465),
    [sym_define_latching_keyword] = ACTIONS(1465),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1465),
    [sym_define_toggling_keyword] = ACTIONS(1465),
    [sym_define_call_keyword] = ACTIONS(1465),
    [sym_define_function_keyword] = ACTIONS(1465),
    [sym_define_library_function_keyword] = ACTIONS(1465),
    [sym_define_start_keyword] = ACTIONS(1465),
    [sym_define_event_keyword] = ACTIONS(1465),
    [sym_define_program_keyword] = ACTIONS(1465),
    [sym_define_module_keyword] = ACTIONS(1465),
    [sym_char_keyword] = ACTIONS(1465),
    [sym_widechar_keyword] = ACTIONS(1465),
    [sym_integer_keyword] = ACTIONS(1465),
    [sym_sinteger_keyword] = ACTIONS(1465),
    [sym_long_keyword] = ACTIONS(1465),
    [sym_slong_keyword] = ACTIONS(1465),
    [sym_float_keyword] = ACTIONS(1465),
    [sym_double_keyword] = ACTIONS(1465),
    [sym_variant_keyword] = ACTIONS(1465),
    [sym_variantarray_keyword] = ACTIONS(1465),
    [sym_local_var_keyword] = ACTIONS(1465),
    [sym_stack_var_keyword] = ACTIONS(1465),
    [sym_constant_keyword] = ACTIONS(1465),
    [sym_volatile_keyword] = ACTIONS(1465),
    [sym_non_volatile_keyword] = ACTIONS(1465),
    [sym_persistent_keyword] = ACTIONS(1465),
    [sym_dev_keyword] = ACTIONS(1465),
    [sym_devlev_keyword] = ACTIONS(1465),
    [sym_devchan_keyword] = ACTIONS(1465),
    [sym_if_keyword] = ACTIONS(1465),
    [sym_else_keyword] = ACTIONS(1465),
    [sym_switch_keyword] = ACTIONS(1465),
    [sym_case_keyword] = ACTIONS(1465),
    [sym_default_keyword] = ACTIONS(1465),
    [sym_while_keyword] = ACTIONS(1465),
    [sym_for_keyword] = ACTIONS(1465),
    [sym_break_keyword] = ACTIONS(1465),
    [sym_continue_keyword] = ACTIONS(1465),
    [sym_return_keyword] = ACTIONS(1465),
    [sym_select_keyword] = ACTIONS(1465),
    [sym_active_keyword] = ACTIONS(1465),
    [sym_struct_keyword] = ACTIONS(1465),
    [sym_structure_keyword] = ACTIONS(1465),
    [sym_button_event_keyword] = ACTIONS(1465),
    [sym_channel_event_keyword] = ACTIONS(1465),
    [sym_level_event_keyword] = ACTIONS(1465),
    [sym_data_event_keyword] = ACTIONS(1465),
    [sym_timeline_event_keyword] = ACTIONS(1465),
    [sym_custom_event_keyword] = ACTIONS(1465),
    [sym_push_keyword] = ACTIONS(1465),
    [sym_release_keyword] = ACTIONS(1465),
    [aux_sym_on_keyword_token1] = ACTIONS(1465),
    [aux_sym_off_keyword_token1] = ACTIONS(1465),
    [sym_send_level_keyword] = ACTIONS(1465),
    [sym_send_string_keyword] = ACTIONS(1465),
    [sym_send_command_keyword] = ACTIONS(1465),
    [sym_clear_buffer_keyword] = ACTIONS(1465),
    [sym_create_buffer_keyword] = ACTIONS(1465),
    [sym_create_multi_buffer_keyword] = ACTIONS(1465),
    [sym_call_keyword] = ACTIONS(1465),
    [sym_system_call_keyword] = ACTIONS(1465),
    [sym_devchan_to_keyword] = ACTIONS(1465),
    [sym_devchan_min_to_keyword] = ACTIONS(1465),
    [sym_devchan_total_off_keyword] = ACTIONS(1465),
    [sym_devchan_pulse_keyword] = ACTIONS(1465),
    [sym_wait_keyword] = ACTIONS(1465),
    [sym_cancel_wait_keyword] = ACTIONS(1465),
    [sym_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym_tdata] = ACTIONS(1465),
    [sym_tchannel] = ACTIONS(1465),
    [sym_tlevel] = ACTIONS(1465),
    [sym_tbutton] = ACTIONS(1465),
    [sym_ttimeline] = ACTIONS(1465),
    [sym_tcustom] = ACTIONS(1465),
    [sym_url_struct] = ACTIONS(1465),
    [sym_dns_struct] = ACTIONS(1465),
    [sym_ip_address_struct] = ACTIONS(1465),
    [sym_dev_info_struct] = ACTIONS(1465),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1465),
    [sym_data_feed] = ACTIONS(1465),
    [sym_data_field] = ACTIONS(1465),
    [sym_data_record] = ACTIONS(1465),
    [sym_wc_data_feed] = ACTIONS(1465),
    [sym_wc_data_field] = ACTIONS(1465),
    [sym_wc_data_record] = ACTIONS(1465),
    [sym_last_login_info] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [anon_sym_EQ] = ACTIONS(1527),
    [aux_sym_preproc_include_token1] = ACTIONS(1465),
    [aux_sym_preproc_define_token1] = ACTIONS(1469),
    [aux_sym_preproc_warn_token1] = ACTIONS(1469),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1469),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [sym_range_operator] = ACTIONS(1529),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1469),
  },
  [STATE(91)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1475),
    [sym_identifier] = ACTIONS(1473),
    [sym_program_name_keyword] = ACTIONS(1473),
    [sym_module_name_keyword] = ACTIONS(1473),
    [sym_define_device_keyword] = ACTIONS(1473),
    [sym_define_combine_keyword] = ACTIONS(1473),
    [sym_define_constant_keyword] = ACTIONS(1473),
    [sym_define_type_keyword] = ACTIONS(1473),
    [sym_define_variable_keyword] = ACTIONS(1473),
    [sym_define_system_variable_keyword] = ACTIONS(1473),
    [sym_define_connect_level_keyword] = ACTIONS(1473),
    [sym_define_latching_keyword] = ACTIONS(1473),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1473),
    [sym_define_toggling_keyword] = ACTIONS(1473),
    [sym_define_call_keyword] = ACTIONS(1473),
    [sym_define_function_keyword] = ACTIONS(1473),
    [sym_define_library_function_keyword] = ACTIONS(1473),
    [sym_define_start_keyword] = ACTIONS(1473),
    [sym_define_event_keyword] = ACTIONS(1473),
    [sym_define_program_keyword] = ACTIONS(1473),
    [sym_define_module_keyword] = ACTIONS(1473),
    [sym_char_keyword] = ACTIONS(1473),
    [sym_widechar_keyword] = ACTIONS(1473),
    [sym_integer_keyword] = ACTIONS(1473),
    [sym_sinteger_keyword] = ACTIONS(1473),
    [sym_long_keyword] = ACTIONS(1473),
    [sym_slong_keyword] = ACTIONS(1473),
    [sym_float_keyword] = ACTIONS(1473),
    [sym_double_keyword] = ACTIONS(1473),
    [sym_variant_keyword] = ACTIONS(1473),
    [sym_variantarray_keyword] = ACTIONS(1473),
    [sym_local_var_keyword] = ACTIONS(1473),
    [sym_stack_var_keyword] = ACTIONS(1473),
    [sym_constant_keyword] = ACTIONS(1473),
    [sym_volatile_keyword] = ACTIONS(1473),
    [sym_non_volatile_keyword] = ACTIONS(1473),
    [sym_persistent_keyword] = ACTIONS(1473),
    [sym_dev_keyword] = ACTIONS(1473),
    [sym_devlev_keyword] = ACTIONS(1473),
    [sym_devchan_keyword] = ACTIONS(1473),
    [sym_if_keyword] = ACTIONS(1473),
    [sym_else_keyword] = ACTIONS(1473),
    [sym_switch_keyword] = ACTIONS(1473),
    [sym_case_keyword] = ACTIONS(1473),
    [sym_default_keyword] = ACTIONS(1473),
    [sym_while_keyword] = ACTIONS(1473),
    [sym_for_keyword] = ACTIONS(1473),
    [sym_break_keyword] = ACTIONS(1473),
    [sym_continue_keyword] = ACTIONS(1473),
    [sym_return_keyword] = ACTIONS(1473),
    [sym_select_keyword] = ACTIONS(1473),
    [sym_active_keyword] = ACTIONS(1473),
    [sym_struct_keyword] = ACTIONS(1473),
    [sym_structure_keyword] = ACTIONS(1473),
    [sym_button_event_keyword] = ACTIONS(1473),
    [sym_channel_event_keyword] = ACTIONS(1473),
    [sym_level_event_keyword] = ACTIONS(1473),
    [sym_data_event_keyword] = ACTIONS(1473),
    [sym_timeline_event_keyword] = ACTIONS(1473),
    [sym_custom_event_keyword] = ACTIONS(1473),
    [sym_push_keyword] = ACTIONS(1473),
    [sym_release_keyword] = ACTIONS(1473),
    [aux_sym_on_keyword_token1] = ACTIONS(1473),
    [aux_sym_off_keyword_token1] = ACTIONS(1473),
    [sym_send_level_keyword] = ACTIONS(1473),
    [sym_send_string_keyword] = ACTIONS(1473),
    [sym_send_command_keyword] = ACTIONS(1473),
    [sym_clear_buffer_keyword] = ACTIONS(1473),
    [sym_create_buffer_keyword] = ACTIONS(1473),
    [sym_create_multi_buffer_keyword] = ACTIONS(1473),
    [sym_call_keyword] = ACTIONS(1473),
    [sym_system_call_keyword] = ACTIONS(1473),
    [sym_devchan_to_keyword] = ACTIONS(1473),
    [sym_devchan_min_to_keyword] = ACTIONS(1473),
    [sym_devchan_total_off_keyword] = ACTIONS(1473),
    [sym_devchan_pulse_keyword] = ACTIONS(1473),
    [sym_wait_keyword] = ACTIONS(1473),
    [sym_cancel_wait_keyword] = ACTIONS(1473),
    [sym_wait_until_keyword] = ACTIONS(1473),
    [sym_cancel_wait_until_keyword] = ACTIONS(1473),
    [sym_cancel_all_wait_keyword] = ACTIONS(1473),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1473),
    [sym_and] = ACTIONS(1473),
    [sym_or] = ACTIONS(1473),
    [sym_not] = ACTIONS(1473),
    [sym_xor] = ACTIONS(1473),
    [sym_band] = ACTIONS(1473),
    [sym_bor] = ACTIONS(1473),
    [sym_bxor] = ACTIONS(1473),
    [sym_bnot] = ACTIONS(1473),
    [sym_lshift] = ACTIONS(1473),
    [sym_rshift] = ACTIONS(1473),
    [sym___file__] = ACTIONS(1473),
    [sym___date__] = ACTIONS(1473),
    [sym___ldate__] = ACTIONS(1473),
    [sym___time__] = ACTIONS(1473),
    [sym___line__] = ACTIONS(1473),
    [sym___name__] = ACTIONS(1473),
    [sym___netlinx__] = ACTIONS(1473),
    [sym_netlinx_axi_version] = ACTIONS(1473),
    [sym_url_flg_tcp] = ACTIONS(1473),
    [sym_url_flg_accinfopresent] = ACTIONS(1473),
    [sym_url_flg_temp] = ACTIONS(1473),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1473),
    [sym_url_flg_stat_mask] = ACTIONS(1473),
    [sym_url_flg_stat_lookup] = ACTIONS(1473),
    [sym_url_flg_stat_connecting] = ACTIONS(1473),
    [sym_url_flg_stat_waiting] = ACTIONS(1473),
    [sym_url_flg_stat_connected] = ACTIONS(1473),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1473),
    [sym_first_virtual_device] = ACTIONS(1473),
    [sym_first_local_port] = ACTIONS(1473),
    [sym_dynamic_virtual_device] = ACTIONS(1473),
    [sym_timeline_once] = ACTIONS(1473),
    [sym_timeline_repeat] = ACTIONS(1473),
    [sym_timeline_absolute] = ACTIONS(1473),
    [sym_timeline_relative] = ACTIONS(1473),
    [sym_do_push_timed_infinite] = ACTIONS(1473),
    [sym_source_type_no_address] = ACTIONS(1473),
    [sym_source_type_neuron_id] = ACTIONS(1473),
    [sym_source_type_ip_address] = ACTIONS(1473),
    [sym_source_type_axlink] = ACTIONS(1473),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1473),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1473),
    [sym_source_type_ip_socket_address] = ACTIONS(1473),
    [sym_source_type_rs232] = ACTIONS(1473),
    [sym_source_type_internal] = ACTIONS(1473),
    [sym_source_type_mac_address] = ACTIONS(1473),
    [sym_source_type_ipv4_port] = ACTIONS(1473),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1473),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1473),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1473),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1473),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1473),
    [sym_true] = ACTIONS(1473),
    [sym_false] = ACTIONS(1473),
    [sym_file_read_only] = ACTIONS(1473),
    [sym_file_rw_new] = ACTIONS(1473),
    [sym_file_rw_append] = ACTIONS(1473),
    [sym_ip_tcp] = ACTIONS(1473),
    [sym_ip_udp] = ACTIONS(1473),
    [sym_ip_udp_2way] = ACTIONS(1473),
    [sym_xml_encode_types] = ACTIONS(1473),
    [sym_xml_encode_char_as_list] = ACTIONS(1473),
    [sym_xml_encode_le] = ACTIONS(1473),
    [sym_xml_decode_types] = ACTIONS(1473),
    [sym_xml_decode_no_preserve] = ACTIONS(1473),
    [sym_duet_dev_type_display_device] = ACTIONS(1473),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1473),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1473),
    [sym_duet_dev_type_io_device] = ACTIONS(1473),
    [sym_duet_dev_type_relay_device] = ACTIONS(1473),
    [sym_duet_dev_type_ups] = ACTIONS(1473),
    [sym_duet_dev_type_amplifier] = ACTIONS(1473),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1473),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1473),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1473),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1473),
    [sym_duet_dev_type_camera] = ACTIONS(1473),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1473),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1473),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1473),
    [sym_duet_dev_type_dss] = ACTIONS(1473),
    [sym_duet_dev_type_dvr] = ACTIONS(1473),
    [sym_duet_dev_type_disc_device] = ACTIONS(1473),
    [sym_duet_dev_type_document_camera] = ACTIONS(1473),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1473),
    [sym_duet_dev_type_hvac] = ACTIONS(1473),
    [sym_duet_dev_type_keypad] = ACTIONS(1473),
    [sym_duet_dev_type_light] = ACTIONS(1473),
    [sym_duet_dev_type_monitor] = ACTIONS(1473),
    [sym_duet_dev_type_motor] = ACTIONS(1473),
    [sym_duet_dev_type_multi_window] = ACTIONS(1473),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1473),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1473),
    [sym_duet_dev_type_receiver] = ACTIONS(1473),
    [sym_duet_dev_type_security_system] = ACTIONS(1473),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1473),
    [sym_duet_dev_type_settop_box] = ACTIONS(1473),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1473),
    [sym_duet_dev_type_switcher] = ACTIONS(1473),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1473),
    [sym_duet_dev_type_tv] = ACTIONS(1473),
    [sym_duet_dev_type_utility] = ACTIONS(1473),
    [sym_duet_dev_type_vcr] = ACTIONS(1473),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1473),
    [sym_duet_dev_type_video_processor] = ACTIONS(1473),
    [sym_duet_dev_type_video_projector] = ACTIONS(1473),
    [sym_duet_dev_type_video_wall] = ACTIONS(1473),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1473),
    [sym_duet_dev_type_weather] = ACTIONS(1473),
    [sym_duet_dev_not_polled] = ACTIONS(1473),
    [sym_duet_dev_polled] = ACTIONS(1473),
    [sym_clkmgr_mode_network] = ACTIONS(1473),
    [sym_clkmgr_mode_standalone] = ACTIONS(1473),
    [sym_normal_standby] = ACTIONS(1473),
    [sym_normal_wake] = ACTIONS(1473),
    [sym_ip_multicast_ttl_option] = ACTIONS(1473),
    [sym_ip_ttl_subnet] = ACTIONS(1473),
    [sym_ip_ttl_site] = ACTIONS(1473),
    [sym_ip_ttl_region] = ACTIONS(1473),
    [sym_ip_ttl_continent] = ACTIONS(1473),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1473),
    [sym_ip_nodelay_on] = ACTIONS(1473),
    [sym_ip_nodelay_off] = ACTIONS(1473),
    [sym_amx_error] = ACTIONS(1473),
    [sym_amx_warning] = ACTIONS(1473),
    [sym_amx_info] = ACTIONS(1473),
    [sym_amx_debug] = ACTIONS(1473),
    [sym_netlinx_login_success] = ACTIONS(1473),
    [sym_netlinx_login_fail] = ACTIONS(1473),
    [sym_netlinx_logout] = ACTIONS(1473),
    [sym_null_str] = ACTIONS(1473),
    [sym_smtp_address] = ACTIONS(1473),
    [sym_smtp_port_number] = ACTIONS(1473),
    [sym_smtp_username] = ACTIONS(1473),
    [sym_smtp_password] = ACTIONS(1473),
    [sym_smtp_require_tls] = ACTIONS(1473),
    [sym_smtp_from] = ACTIONS(1473),
    [sym_smtp_tls_true] = ACTIONS(1473),
    [sym_smtp_tls_false] = ACTIONS(1473),
    [sym_listview_on_row_select_event] = ACTIONS(1473),
    [sym_data_structure_unknown] = ACTIONS(1473),
    [sym_data_structure_datafeed] = ACTIONS(1473),
    [sym_data_structure_datarecordset] = ACTIONS(1473),
    [sym_data_structure_datarecord] = ACTIONS(1473),
    [sym_data_structure_datafield] = ACTIONS(1473),
    [sym_data_max_content_field_count] = ACTIONS(1473),
    [sym_data_max_metadata_field_count] = ACTIONS(1473),
    [sym_data_max_id_length] = ACTIONS(1473),
    [sym_data_max_name_length] = ACTIONS(1473),
    [sym_data_max_description_length] = ACTIONS(1473),
    [sym_data_max_type_length] = ACTIONS(1473),
    [sym_data_max_format_length] = ACTIONS(1473),
    [sym_data_max_source_length] = ACTIONS(1473),
    [sym_data_max_value_length] = ACTIONS(1473),
    [sym_data_max_label_length] = ACTIONS(1473),
    [sym_data_max_url_length] = ACTIONS(1473),
    [sym_max_last_login_info_length] = ACTIONS(1473),
    [sym_data_type_unknown] = ACTIONS(1473),
    [sym_data_type_string] = ACTIONS(1473),
    [sym_data_type_datetime] = ACTIONS(1473),
    [sym_data_type_date] = ACTIONS(1473),
    [sym_data_type_time] = ACTIONS(1473),
    [sym_data_type_image] = ACTIONS(1473),
    [sym_data_format_url] = ACTIONS(1473),
    [sym_data_format_phone] = ACTIONS(1473),
    [sym_data_format_email] = ACTIONS(1473),
    [sym_data_format_iso8601] = ACTIONS(1473),
    [sym_auditlog_priv] = ACTIONS(1473),
    [sym_configuration_priv] = ACTIONS(1473),
    [sym_deviceconfig_priv] = ACTIONS(1473),
    [sym_ftp_priv] = ACTIONS(1473),
    [sym_http_priv] = ACTIONS(1473),
    [sym_networkconfig_priv] = ACTIONS(1473),
    [sym_programport_priv] = ACTIONS(1473),
    [sym_remoteui_priv] = ACTIONS(1473),
    [sym_securitycontrol_priv] = ACTIONS(1473),
    [sym_softwaremanagement_priv] = ACTIONS(1473),
    [sym_terminal_priv] = ACTIONS(1473),
    [sym_tpadmin_priv] = ACTIONS(1473),
    [sym_user1_priv] = ACTIONS(1473),
    [sym_user2_priv] = ACTIONS(1473),
    [sym_user3_priv] = ACTIONS(1473),
    [sym_user4_priv] = ACTIONS(1473),
    [sym_usermanagement_priv] = ACTIONS(1473),
    [sym_valid_account] = ACTIONS(1473),
    [sym_err_name_invalid] = ACTIONS(1473),
    [sym_err_password_invalid] = ACTIONS(1473),
    [sym_err_invalid_account] = ACTIONS(1473),
    [sym_err_no_authorization] = ACTIONS(1473),
    [sym_err_invalid_parameter] = ACTIONS(1473),
    [sym_err_user_locked_out] = ACTIONS(1473),
    [sym_err_user_account_expired] = ACTIONS(1473),
    [sym_err_general_error] = ACTIONS(1473),
    [sym_err_authentication_error] = ACTIONS(1473),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1473),
    [sym_tls_validate_certificate] = ACTIONS(1473),
    [sym_tdata] = ACTIONS(1473),
    [sym_tchannel] = ACTIONS(1473),
    [sym_tlevel] = ACTIONS(1473),
    [sym_tbutton] = ACTIONS(1473),
    [sym_ttimeline] = ACTIONS(1473),
    [sym_tcustom] = ACTIONS(1473),
    [sym_url_struct] = ACTIONS(1473),
    [sym_dns_struct] = ACTIONS(1473),
    [sym_ip_address_struct] = ACTIONS(1473),
    [sym_dev_info_struct] = ACTIONS(1473),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1473),
    [sym_data_feed] = ACTIONS(1473),
    [sym_data_field] = ACTIONS(1473),
    [sym_data_record] = ACTIONS(1473),
    [sym_wc_data_feed] = ACTIONS(1473),
    [sym_wc_data_field] = ACTIONS(1473),
    [sym_wc_data_record] = ACTIONS(1473),
    [sym_last_login_info] = ACTIONS(1473),
    [sym____reserved___] = ACTIONS(1473),
    [sym_date] = ACTIONS(1473),
    [sym_day] = ACTIONS(1473),
    [sym_time] = ACTIONS(1473),
    [sym_ldate] = ACTIONS(1473),
    [sym_push_device] = ACTIONS(1473),
    [sym_push_channel] = ACTIONS(1473),
    [sym_push_devchan] = ACTIONS(1473),
    [sym_release_device] = ACTIONS(1473),
    [sym_release_channel] = ACTIONS(1473),
    [sym_release_devchan] = ACTIONS(1473),
    [sym_master_slot] = ACTIONS(1473),
    [sym_get_pulse_time] = ACTIONS(1473),
    [sym_get_timer] = ACTIONS(1473),
    [sym_master_sn] = ACTIONS(1473),
    [sym_system_number] = ACTIONS(1473),
    [sym_button] = ACTIONS(1473),
    [sym_dv_channel] = ACTIONS(1473),
    [sym_data] = ACTIONS(1473),
    [sym_level] = ACTIONS(1473),
    [sym_timeline] = ACTIONS(1473),
    [sym_channel] = ACTIONS(1473),
    [sym_custom] = ACTIONS(1473),
    [sym_atoi] = ACTIONS(1473),
    [sym_atol] = ACTIONS(1473),
    [sym_atof] = ACTIONS(1473),
    [sym_itoa] = ACTIONS(1473),
    [sym_itohex] = ACTIONS(1473),
    [sym_hextoi] = ACTIONS(1473),
    [sym_ftoa] = ACTIONS(1473),
    [sym_find_string] = ACTIONS(1473),
    [sym_left_string] = ACTIONS(1473),
    [sym_right_string] = ACTIONS(1473),
    [sym_mid_string] = ACTIONS(1473),
    [sym_remove_string] = ACTIONS(1473),
    [sym_get_buffer_string] = ACTIONS(1473),
    [sym_get_buffer_char] = ACTIONS(1473),
    [sym_get_multi_buffer_string] = ACTIONS(1473),
    [sym_length_string] = ACTIONS(1473),
    [sym_max_length_string] = ACTIONS(1473),
    [sym_set_length_string] = ACTIONS(1473),
    [sym_compare_string] = ACTIONS(1473),
    [sym_upper_string] = ACTIONS(1473),
    [sym_lower_string] = ACTIONS(1473),
    [sym_get_last] = ACTIONS(1473),
    [sym_date_to_day] = ACTIONS(1473),
    [sym_date_to_month] = ACTIONS(1473),
    [sym_date_to_year] = ACTIONS(1473),
    [sym_time_to_hour] = ACTIONS(1473),
    [sym_time_to_minute] = ACTIONS(1473),
    [sym_time_to_second] = ACTIONS(1473),
    [sym_day_of_week] = ACTIONS(1473),
    [sym_set_pulse_time] = ACTIONS(1473),
    [sym_set_timer] = ACTIONS(1473),
    [sym_file_open] = ACTIONS(1473),
    [sym_file_close] = ACTIONS(1473),
    [sym_file_read] = ACTIONS(1473),
    [sym_file_read_line] = ACTIONS(1473),
    [sym_file_write] = ACTIONS(1473),
    [sym_file_write_line] = ACTIONS(1473),
    [sym_file_copy] = ACTIONS(1473),
    [sym_file_delete] = ACTIONS(1473),
    [sym_file_rename] = ACTIONS(1473),
    [sym_file_dir] = ACTIONS(1473),
    [sym_file_getdir] = ACTIONS(1473),
    [sym_file_setdir] = ACTIONS(1473),
    [sym_file_create_dir] = ACTIONS(1473),
    [sym_file_remove_dir] = ACTIONS(1473),
    [sym_file_seek] = ACTIONS(1473),
    [sym_length_array] = ACTIONS(1473),
    [sym_max_length_array] = ACTIONS(1473),
    [sym_set_length_array] = ACTIONS(1473),
    [sym_ip_client_open] = ACTIONS(1473),
    [sym_ip_client_close] = ACTIONS(1473),
    [sym_ip_server_open] = ACTIONS(1473),
    [sym_ip_server_close] = ACTIONS(1473),
    [sym_ip_bound_client_open] = ACTIONS(1473),
    [sym_ip_mc_server_open] = ACTIONS(1473),
    [sym_ip_set_option] = ACTIONS(1473),
    [sym_ssh_client_open] = ACTIONS(1473),
    [sym_ssh_client_close] = ACTIONS(1473),
    [sym_tls_client_open] = ACTIONS(1473),
    [sym_tls_client_close] = ACTIONS(1473),
    [sym_do_push] = ACTIONS(1473),
    [sym_do_push_timed] = ACTIONS(1473),
    [sym_do_release] = ACTIONS(1473),
    [sym_device_id] = ACTIONS(1473),
    [sym_device_info] = ACTIONS(1473),
    [sym_device_id_string] = ACTIONS(1473),
    [sym_device_standby] = ACTIONS(1473),
    [sym_device_wake] = ACTIONS(1473),
    [sym_timeline_create] = ACTIONS(1473),
    [sym_timeline_kill] = ACTIONS(1473),
    [sym_timeline_reload] = ACTIONS(1473),
    [sym_timeline_pause] = ACTIONS(1473),
    [sym_timeline_restart] = ACTIONS(1473),
    [sym_timeline_set] = ACTIONS(1473),
    [sym_timeline_get] = ACTIONS(1473),
    [sym_timeline_active] = ACTIONS(1473),
    [sym_timeline_dynamic_id] = ACTIONS(1473),
    [sym_variable_to_string] = ACTIONS(1473),
    [sym_string_to_variable] = ACTIONS(1473),
    [sym_length_variable_to_string] = ACTIONS(1473),
    [sym_type_cast] = ACTIONS(1473),
    [sym_variable_to_xml] = ACTIONS(1473),
    [sym_xml_to_variable] = ACTIONS(1473),
    [sym_length_variable_to_xml] = ACTIONS(1473),
    [sym_redirect_string] = ACTIONS(1473),
    [sym_random_number] = ACTIONS(1473),
    [sym_get_unique_id] = ACTIONS(1473),
    [sym_get_system_number] = ACTIONS(1473),
    [sym_set_system_number] = ACTIONS(1473),
    [sym_get_serial_number] = ACTIONS(1473),
    [sym_get_master_build] = ACTIONS(1473),
    [sym_set_validation_code] = ACTIONS(1473),
    [sym_reboot] = ACTIONS(1473),
    [sym_load_duet_module] = ACTIONS(1473),
    [sym_unload_duet_module] = ACTIONS(1473),
    [sym_duet_mem_size_set] = ACTIONS(1473),
    [sym_duet_mem_size_get] = ACTIONS(1473),
    [sym_dynamic_application_device] = ACTIONS(1473),
    [sym_dynamic_polled_port] = ACTIONS(1473),
    [sym_static_port_binding] = ACTIONS(1473),
    [sym_static_ip_binding] = ACTIONS(1473),
    [sym_get_dns_list] = ACTIONS(1473),
    [sym_set_dns_list] = ACTIONS(1473),
    [sym_get_ip_address] = ACTIONS(1473),
    [sym_set_ip_address] = ACTIONS(1473),
    [sym_get_url_list] = ACTIONS(1473),
    [sym_add_url_entry] = ACTIONS(1473),
    [sym_delete_url_entry] = ACTIONS(1473),
    [sym_abs_value] = ACTIONS(1473),
    [sym_max_value] = ACTIONS(1473),
    [sym_min_value] = ACTIONS(1473),
    [sym_exp_value] = ACTIONS(1473),
    [sym_log_value] = ACTIONS(1473),
    [sym_log10_value] = ACTIONS(1473),
    [sym_power_value] = ACTIONS(1473),
    [sym_sqrt_value] = ACTIONS(1473),
    [sym_raw_be] = ACTIONS(1473),
    [sym_raw_le] = ACTIONS(1473),
    [sym_internal_queue_size_set] = ACTIONS(1473),
    [sym_internal_queue_size_get] = ACTIONS(1473),
    [sym_internal_threshold_set] = ACTIONS(1473),
    [sym_internal_threshold_get] = ACTIONS(1473),
    [sym_rebuilt_event] = ACTIONS(1473),
    [sym_do_custom_event] = ACTIONS(1473),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1473),
    [sym_clkmgr_set_clk_source] = ACTIONS(1473),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1473),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1473),
    [sym_clkmgr_get_timezone] = ACTIONS(1473),
    [sym_clkmgr_set_timezone] = ACTIONS(1473),
    [sym_clkmgr_get_resync_period] = ACTIONS(1473),
    [sym_clkmgr_set_resync_period] = ACTIONS(1473),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1473),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1473),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1473),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1473),
    [sym_clkmgr_get_timeservers] = ACTIONS(1473),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1473),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1473),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1473),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1473),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1473),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1473),
    [sym_set_log_level] = ACTIONS(1473),
    [sym_get_log_level] = ACTIONS(1473),
    [sym_amx_log] = ACTIONS(1473),
    [sym_smtp_server_config_set] = ACTIONS(1473),
    [sym_smtp_server_config_get] = ACTIONS(1473),
    [sym_smtp_send] = ACTIONS(1473),
    [sym_validate_netlinx_account] = ACTIONS(1473),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1473),
    [sym_authenticate_certificate] = ACTIONS(1473),
    [sym_audit_netlinx_session_event] = ACTIONS(1473),
    [sym_audit_netlinx_generic_event] = ACTIONS(1473),
    [sym_get_available_flash_disk_space] = ACTIONS(1473),
    [sym_get_max_flash_disk_space] = ACTIONS(1473),
    [sym_data_create_feed] = ACTIONS(1473),
    [sym_data_delete_feed] = ACTIONS(1473),
    [sym_data_publish_feed] = ACTIONS(1473),
    [sym_data_get_published_feed] = ACTIONS(1473),
    [sym_data_add_record] = ACTIONS(1473),
    [sym_data_get_event_record] = ACTIONS(1473),
    [sym__wc_data_create_feed] = ACTIONS(1473),
    [sym__wc_data_add_record] = ACTIONS(1473),
    [sym__wc_data_get_event_record] = ACTIONS(1473),
    [sym_format] = ACTIONS(1473),
    [sym_astro_clock] = ACTIONS(1473),
    [sym_set_outdoor_temperature] = ACTIONS(1473),
    [sym_set_virtual_level_count] = ACTIONS(1473),
    [sym_set_virtual_channel_count] = ACTIONS(1473),
    [sym_set_virtual_port_count] = ACTIONS(1473),
    [sym__wc_encode_int] = ACTIONS(1473),
    [sym__wc_decode_int] = ACTIONS(1473),
    [sym__wc_int] = ACTIONS(1473),
    [sym__wc_to_ch_int] = ACTIONS(1473),
    [sym__ch_to_wc_int] = ACTIONS(1473),
    [sym__wc_find_string_int] = ACTIONS(1473),
    [sym__wc_left_string_int] = ACTIONS(1473),
    [sym__wc_lower_string_int] = ACTIONS(1473),
    [sym__wc_mid_string_int] = ACTIONS(1473),
    [sym__wc_remove_string_int] = ACTIONS(1473),
    [sym__wc_right_string_int] = ACTIONS(1473),
    [sym__wc_upper_string_int] = ACTIONS(1473),
    [sym__wc_compare_string_int] = ACTIONS(1473),
    [sym__wc_get_buffer_char_int] = ACTIONS(1473),
    [sym__wc_get_buffer_string_int] = ACTIONS(1473),
    [sym__wc_concat_string_int] = ACTIONS(1473),
    [sym__wc_file_open_int] = ACTIONS(1473),
    [sym__wc_file_close_int] = ACTIONS(1473),
    [sym__wc_file_read_int] = ACTIONS(1473),
    [sym__wc_file_read_line_int] = ACTIONS(1473),
    [sym__wc_file_write_int] = ACTIONS(1473),
    [sym__wc_file_write_line_int] = ACTIONS(1473),
    [aux_sym_preproc_include_token1] = ACTIONS(1473),
    [aux_sym_preproc_define_token1] = ACTIONS(1475),
    [aux_sym_preproc_warn_token1] = ACTIONS(1475),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1475),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1475),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1475),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1475),
    [anon_sym_COLON] = ACTIONS(1475),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1475),
    [anon_sym_BANG] = ACTIONS(1473),
    [anon_sym_TILDE] = ACTIONS(1475),
    [anon_sym_DASH] = ACTIONS(1473),
    [anon_sym_PLUS] = ACTIONS(1473),
    [anon_sym_STAR] = ACTIONS(1475),
    [anon_sym_SLASH] = ACTIONS(1473),
    [anon_sym_PERCENT] = ACTIONS(1475),
    [anon_sym_PIPE_PIPE] = ACTIONS(1475),
    [anon_sym_AMP_AMP] = ACTIONS(1475),
    [anon_sym_PIPE] = ACTIONS(1473),
    [anon_sym_CARET] = ACTIONS(1475),
    [anon_sym_AMP] = ACTIONS(1473),
    [anon_sym_EQ_EQ] = ACTIONS(1475),
    [anon_sym_BANG_EQ] = ACTIONS(1475),
    [anon_sym_LT_GT] = ACTIONS(1475),
    [anon_sym_GT] = ACTIONS(1473),
    [anon_sym_GT_EQ] = ACTIONS(1475),
    [anon_sym_LT_EQ] = ACTIONS(1475),
    [anon_sym_LT] = ACTIONS(1473),
    [anon_sym_LT_LT] = ACTIONS(1475),
    [anon_sym_GT_GT] = ACTIONS(1475),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1475),
    [anon_sym_SQUOTE] = ACTIONS(1475),
    [sym_number_literal] = ACTIONS(1475),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1475),
  },
  [STATE(92)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1479),
    [sym_identifier] = ACTIONS(1477),
    [sym_program_name_keyword] = ACTIONS(1477),
    [sym_module_name_keyword] = ACTIONS(1477),
    [sym_define_device_keyword] = ACTIONS(1477),
    [sym_define_combine_keyword] = ACTIONS(1477),
    [sym_define_constant_keyword] = ACTIONS(1477),
    [sym_define_type_keyword] = ACTIONS(1477),
    [sym_define_variable_keyword] = ACTIONS(1477),
    [sym_define_system_variable_keyword] = ACTIONS(1477),
    [sym_define_connect_level_keyword] = ACTIONS(1477),
    [sym_define_latching_keyword] = ACTIONS(1477),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1477),
    [sym_define_toggling_keyword] = ACTIONS(1477),
    [sym_define_call_keyword] = ACTIONS(1477),
    [sym_define_function_keyword] = ACTIONS(1477),
    [sym_define_library_function_keyword] = ACTIONS(1477),
    [sym_define_start_keyword] = ACTIONS(1477),
    [sym_define_event_keyword] = ACTIONS(1477),
    [sym_define_program_keyword] = ACTIONS(1477),
    [sym_define_module_keyword] = ACTIONS(1477),
    [sym_char_keyword] = ACTIONS(1477),
    [sym_widechar_keyword] = ACTIONS(1477),
    [sym_integer_keyword] = ACTIONS(1477),
    [sym_sinteger_keyword] = ACTIONS(1477),
    [sym_long_keyword] = ACTIONS(1477),
    [sym_slong_keyword] = ACTIONS(1477),
    [sym_float_keyword] = ACTIONS(1477),
    [sym_double_keyword] = ACTIONS(1477),
    [sym_variant_keyword] = ACTIONS(1477),
    [sym_variantarray_keyword] = ACTIONS(1477),
    [sym_local_var_keyword] = ACTIONS(1477),
    [sym_stack_var_keyword] = ACTIONS(1477),
    [sym_constant_keyword] = ACTIONS(1477),
    [sym_volatile_keyword] = ACTIONS(1477),
    [sym_non_volatile_keyword] = ACTIONS(1477),
    [sym_persistent_keyword] = ACTIONS(1477),
    [sym_dev_keyword] = ACTIONS(1477),
    [sym_devlev_keyword] = ACTIONS(1477),
    [sym_devchan_keyword] = ACTIONS(1477),
    [sym_if_keyword] = ACTIONS(1477),
    [sym_else_keyword] = ACTIONS(1477),
    [sym_switch_keyword] = ACTIONS(1477),
    [sym_case_keyword] = ACTIONS(1477),
    [sym_default_keyword] = ACTIONS(1477),
    [sym_while_keyword] = ACTIONS(1477),
    [sym_for_keyword] = ACTIONS(1477),
    [sym_break_keyword] = ACTIONS(1477),
    [sym_continue_keyword] = ACTIONS(1477),
    [sym_return_keyword] = ACTIONS(1477),
    [sym_select_keyword] = ACTIONS(1477),
    [sym_active_keyword] = ACTIONS(1477),
    [sym_struct_keyword] = ACTIONS(1477),
    [sym_structure_keyword] = ACTIONS(1477),
    [sym_button_event_keyword] = ACTIONS(1477),
    [sym_channel_event_keyword] = ACTIONS(1477),
    [sym_level_event_keyword] = ACTIONS(1477),
    [sym_data_event_keyword] = ACTIONS(1477),
    [sym_timeline_event_keyword] = ACTIONS(1477),
    [sym_custom_event_keyword] = ACTIONS(1477),
    [sym_push_keyword] = ACTIONS(1477),
    [sym_release_keyword] = ACTIONS(1477),
    [aux_sym_on_keyword_token1] = ACTIONS(1477),
    [aux_sym_off_keyword_token1] = ACTIONS(1477),
    [sym_send_level_keyword] = ACTIONS(1477),
    [sym_send_string_keyword] = ACTIONS(1477),
    [sym_send_command_keyword] = ACTIONS(1477),
    [sym_clear_buffer_keyword] = ACTIONS(1477),
    [sym_create_buffer_keyword] = ACTIONS(1477),
    [sym_create_multi_buffer_keyword] = ACTIONS(1477),
    [sym_call_keyword] = ACTIONS(1477),
    [sym_system_call_keyword] = ACTIONS(1477),
    [sym_devchan_to_keyword] = ACTIONS(1477),
    [sym_devchan_min_to_keyword] = ACTIONS(1477),
    [sym_devchan_total_off_keyword] = ACTIONS(1477),
    [sym_devchan_pulse_keyword] = ACTIONS(1477),
    [sym_wait_keyword] = ACTIONS(1477),
    [sym_cancel_wait_keyword] = ACTIONS(1477),
    [sym_wait_until_keyword] = ACTIONS(1477),
    [sym_cancel_wait_until_keyword] = ACTIONS(1477),
    [sym_cancel_all_wait_keyword] = ACTIONS(1477),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1477),
    [sym_and] = ACTIONS(1477),
    [sym_or] = ACTIONS(1477),
    [sym_not] = ACTIONS(1477),
    [sym_xor] = ACTIONS(1477),
    [sym_band] = ACTIONS(1477),
    [sym_bor] = ACTIONS(1477),
    [sym_bxor] = ACTIONS(1477),
    [sym_bnot] = ACTIONS(1477),
    [sym_lshift] = ACTIONS(1477),
    [sym_rshift] = ACTIONS(1477),
    [sym___file__] = ACTIONS(1477),
    [sym___date__] = ACTIONS(1477),
    [sym___ldate__] = ACTIONS(1477),
    [sym___time__] = ACTIONS(1477),
    [sym___line__] = ACTIONS(1477),
    [sym___name__] = ACTIONS(1477),
    [sym___netlinx__] = ACTIONS(1477),
    [sym_netlinx_axi_version] = ACTIONS(1477),
    [sym_url_flg_tcp] = ACTIONS(1477),
    [sym_url_flg_accinfopresent] = ACTIONS(1477),
    [sym_url_flg_temp] = ACTIONS(1477),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1477),
    [sym_url_flg_stat_mask] = ACTIONS(1477),
    [sym_url_flg_stat_lookup] = ACTIONS(1477),
    [sym_url_flg_stat_connecting] = ACTIONS(1477),
    [sym_url_flg_stat_waiting] = ACTIONS(1477),
    [sym_url_flg_stat_connected] = ACTIONS(1477),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1477),
    [sym_first_virtual_device] = ACTIONS(1477),
    [sym_first_local_port] = ACTIONS(1477),
    [sym_dynamic_virtual_device] = ACTIONS(1477),
    [sym_timeline_once] = ACTIONS(1477),
    [sym_timeline_repeat] = ACTIONS(1477),
    [sym_timeline_absolute] = ACTIONS(1477),
    [sym_timeline_relative] = ACTIONS(1477),
    [sym_do_push_timed_infinite] = ACTIONS(1477),
    [sym_source_type_no_address] = ACTIONS(1477),
    [sym_source_type_neuron_id] = ACTIONS(1477),
    [sym_source_type_ip_address] = ACTIONS(1477),
    [sym_source_type_axlink] = ACTIONS(1477),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1477),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1477),
    [sym_source_type_ip_socket_address] = ACTIONS(1477),
    [sym_source_type_rs232] = ACTIONS(1477),
    [sym_source_type_internal] = ACTIONS(1477),
    [sym_source_type_mac_address] = ACTIONS(1477),
    [sym_source_type_ipv4_port] = ACTIONS(1477),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1477),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1477),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1477),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1477),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1477),
    [sym_true] = ACTIONS(1477),
    [sym_false] = ACTIONS(1477),
    [sym_file_read_only] = ACTIONS(1477),
    [sym_file_rw_new] = ACTIONS(1477),
    [sym_file_rw_append] = ACTIONS(1477),
    [sym_ip_tcp] = ACTIONS(1477),
    [sym_ip_udp] = ACTIONS(1477),
    [sym_ip_udp_2way] = ACTIONS(1477),
    [sym_xml_encode_types] = ACTIONS(1477),
    [sym_xml_encode_char_as_list] = ACTIONS(1477),
    [sym_xml_encode_le] = ACTIONS(1477),
    [sym_xml_decode_types] = ACTIONS(1477),
    [sym_xml_decode_no_preserve] = ACTIONS(1477),
    [sym_duet_dev_type_display_device] = ACTIONS(1477),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1477),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1477),
    [sym_duet_dev_type_io_device] = ACTIONS(1477),
    [sym_duet_dev_type_relay_device] = ACTIONS(1477),
    [sym_duet_dev_type_ups] = ACTIONS(1477),
    [sym_duet_dev_type_amplifier] = ACTIONS(1477),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1477),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1477),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1477),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1477),
    [sym_duet_dev_type_camera] = ACTIONS(1477),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1477),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1477),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1477),
    [sym_duet_dev_type_dss] = ACTIONS(1477),
    [sym_duet_dev_type_dvr] = ACTIONS(1477),
    [sym_duet_dev_type_disc_device] = ACTIONS(1477),
    [sym_duet_dev_type_document_camera] = ACTIONS(1477),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1477),
    [sym_duet_dev_type_hvac] = ACTIONS(1477),
    [sym_duet_dev_type_keypad] = ACTIONS(1477),
    [sym_duet_dev_type_light] = ACTIONS(1477),
    [sym_duet_dev_type_monitor] = ACTIONS(1477),
    [sym_duet_dev_type_motor] = ACTIONS(1477),
    [sym_duet_dev_type_multi_window] = ACTIONS(1477),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1477),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1477),
    [sym_duet_dev_type_receiver] = ACTIONS(1477),
    [sym_duet_dev_type_security_system] = ACTIONS(1477),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1477),
    [sym_duet_dev_type_settop_box] = ACTIONS(1477),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1477),
    [sym_duet_dev_type_switcher] = ACTIONS(1477),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1477),
    [sym_duet_dev_type_tv] = ACTIONS(1477),
    [sym_duet_dev_type_utility] = ACTIONS(1477),
    [sym_duet_dev_type_vcr] = ACTIONS(1477),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1477),
    [sym_duet_dev_type_video_processor] = ACTIONS(1477),
    [sym_duet_dev_type_video_projector] = ACTIONS(1477),
    [sym_duet_dev_type_video_wall] = ACTIONS(1477),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1477),
    [sym_duet_dev_type_weather] = ACTIONS(1477),
    [sym_duet_dev_not_polled] = ACTIONS(1477),
    [sym_duet_dev_polled] = ACTIONS(1477),
    [sym_clkmgr_mode_network] = ACTIONS(1477),
    [sym_clkmgr_mode_standalone] = ACTIONS(1477),
    [sym_normal_standby] = ACTIONS(1477),
    [sym_normal_wake] = ACTIONS(1477),
    [sym_ip_multicast_ttl_option] = ACTIONS(1477),
    [sym_ip_ttl_subnet] = ACTIONS(1477),
    [sym_ip_ttl_site] = ACTIONS(1477),
    [sym_ip_ttl_region] = ACTIONS(1477),
    [sym_ip_ttl_continent] = ACTIONS(1477),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1477),
    [sym_ip_nodelay_on] = ACTIONS(1477),
    [sym_ip_nodelay_off] = ACTIONS(1477),
    [sym_amx_error] = ACTIONS(1477),
    [sym_amx_warning] = ACTIONS(1477),
    [sym_amx_info] = ACTIONS(1477),
    [sym_amx_debug] = ACTIONS(1477),
    [sym_netlinx_login_success] = ACTIONS(1477),
    [sym_netlinx_login_fail] = ACTIONS(1477),
    [sym_netlinx_logout] = ACTIONS(1477),
    [sym_null_str] = ACTIONS(1477),
    [sym_smtp_address] = ACTIONS(1477),
    [sym_smtp_port_number] = ACTIONS(1477),
    [sym_smtp_username] = ACTIONS(1477),
    [sym_smtp_password] = ACTIONS(1477),
    [sym_smtp_require_tls] = ACTIONS(1477),
    [sym_smtp_from] = ACTIONS(1477),
    [sym_smtp_tls_true] = ACTIONS(1477),
    [sym_smtp_tls_false] = ACTIONS(1477),
    [sym_listview_on_row_select_event] = ACTIONS(1477),
    [sym_data_structure_unknown] = ACTIONS(1477),
    [sym_data_structure_datafeed] = ACTIONS(1477),
    [sym_data_structure_datarecordset] = ACTIONS(1477),
    [sym_data_structure_datarecord] = ACTIONS(1477),
    [sym_data_structure_datafield] = ACTIONS(1477),
    [sym_data_max_content_field_count] = ACTIONS(1477),
    [sym_data_max_metadata_field_count] = ACTIONS(1477),
    [sym_data_max_id_length] = ACTIONS(1477),
    [sym_data_max_name_length] = ACTIONS(1477),
    [sym_data_max_description_length] = ACTIONS(1477),
    [sym_data_max_type_length] = ACTIONS(1477),
    [sym_data_max_format_length] = ACTIONS(1477),
    [sym_data_max_source_length] = ACTIONS(1477),
    [sym_data_max_value_length] = ACTIONS(1477),
    [sym_data_max_label_length] = ACTIONS(1477),
    [sym_data_max_url_length] = ACTIONS(1477),
    [sym_max_last_login_info_length] = ACTIONS(1477),
    [sym_data_type_unknown] = ACTIONS(1477),
    [sym_data_type_string] = ACTIONS(1477),
    [sym_data_type_datetime] = ACTIONS(1477),
    [sym_data_type_date] = ACTIONS(1477),
    [sym_data_type_time] = ACTIONS(1477),
    [sym_data_type_image] = ACTIONS(1477),
    [sym_data_format_url] = ACTIONS(1477),
    [sym_data_format_phone] = ACTIONS(1477),
    [sym_data_format_email] = ACTIONS(1477),
    [sym_data_format_iso8601] = ACTIONS(1477),
    [sym_auditlog_priv] = ACTIONS(1477),
    [sym_configuration_priv] = ACTIONS(1477),
    [sym_deviceconfig_priv] = ACTIONS(1477),
    [sym_ftp_priv] = ACTIONS(1477),
    [sym_http_priv] = ACTIONS(1477),
    [sym_networkconfig_priv] = ACTIONS(1477),
    [sym_programport_priv] = ACTIONS(1477),
    [sym_remoteui_priv] = ACTIONS(1477),
    [sym_securitycontrol_priv] = ACTIONS(1477),
    [sym_softwaremanagement_priv] = ACTIONS(1477),
    [sym_terminal_priv] = ACTIONS(1477),
    [sym_tpadmin_priv] = ACTIONS(1477),
    [sym_user1_priv] = ACTIONS(1477),
    [sym_user2_priv] = ACTIONS(1477),
    [sym_user3_priv] = ACTIONS(1477),
    [sym_user4_priv] = ACTIONS(1477),
    [sym_usermanagement_priv] = ACTIONS(1477),
    [sym_valid_account] = ACTIONS(1477),
    [sym_err_name_invalid] = ACTIONS(1477),
    [sym_err_password_invalid] = ACTIONS(1477),
    [sym_err_invalid_account] = ACTIONS(1477),
    [sym_err_no_authorization] = ACTIONS(1477),
    [sym_err_invalid_parameter] = ACTIONS(1477),
    [sym_err_user_locked_out] = ACTIONS(1477),
    [sym_err_user_account_expired] = ACTIONS(1477),
    [sym_err_general_error] = ACTIONS(1477),
    [sym_err_authentication_error] = ACTIONS(1477),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1477),
    [sym_tls_validate_certificate] = ACTIONS(1477),
    [sym_tdata] = ACTIONS(1477),
    [sym_tchannel] = ACTIONS(1477),
    [sym_tlevel] = ACTIONS(1477),
    [sym_tbutton] = ACTIONS(1477),
    [sym_ttimeline] = ACTIONS(1477),
    [sym_tcustom] = ACTIONS(1477),
    [sym_url_struct] = ACTIONS(1477),
    [sym_dns_struct] = ACTIONS(1477),
    [sym_ip_address_struct] = ACTIONS(1477),
    [sym_dev_info_struct] = ACTIONS(1477),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1477),
    [sym_data_feed] = ACTIONS(1477),
    [sym_data_field] = ACTIONS(1477),
    [sym_data_record] = ACTIONS(1477),
    [sym_wc_data_feed] = ACTIONS(1477),
    [sym_wc_data_field] = ACTIONS(1477),
    [sym_wc_data_record] = ACTIONS(1477),
    [sym_last_login_info] = ACTIONS(1477),
    [sym____reserved___] = ACTIONS(1477),
    [sym_date] = ACTIONS(1477),
    [sym_day] = ACTIONS(1477),
    [sym_time] = ACTIONS(1477),
    [sym_ldate] = ACTIONS(1477),
    [sym_push_device] = ACTIONS(1477),
    [sym_push_channel] = ACTIONS(1477),
    [sym_push_devchan] = ACTIONS(1477),
    [sym_release_device] = ACTIONS(1477),
    [sym_release_channel] = ACTIONS(1477),
    [sym_release_devchan] = ACTIONS(1477),
    [sym_master_slot] = ACTIONS(1477),
    [sym_get_pulse_time] = ACTIONS(1477),
    [sym_get_timer] = ACTIONS(1477),
    [sym_master_sn] = ACTIONS(1477),
    [sym_system_number] = ACTIONS(1477),
    [sym_button] = ACTIONS(1477),
    [sym_dv_channel] = ACTIONS(1477),
    [sym_data] = ACTIONS(1477),
    [sym_level] = ACTIONS(1477),
    [sym_timeline] = ACTIONS(1477),
    [sym_channel] = ACTIONS(1477),
    [sym_custom] = ACTIONS(1477),
    [sym_atoi] = ACTIONS(1477),
    [sym_atol] = ACTIONS(1477),
    [sym_atof] = ACTIONS(1477),
    [sym_itoa] = ACTIONS(1477),
    [sym_itohex] = ACTIONS(1477),
    [sym_hextoi] = ACTIONS(1477),
    [sym_ftoa] = ACTIONS(1477),
    [sym_find_string] = ACTIONS(1477),
    [sym_left_string] = ACTIONS(1477),
    [sym_right_string] = ACTIONS(1477),
    [sym_mid_string] = ACTIONS(1477),
    [sym_remove_string] = ACTIONS(1477),
    [sym_get_buffer_string] = ACTIONS(1477),
    [sym_get_buffer_char] = ACTIONS(1477),
    [sym_get_multi_buffer_string] = ACTIONS(1477),
    [sym_length_string] = ACTIONS(1477),
    [sym_max_length_string] = ACTIONS(1477),
    [sym_set_length_string] = ACTIONS(1477),
    [sym_compare_string] = ACTIONS(1477),
    [sym_upper_string] = ACTIONS(1477),
    [sym_lower_string] = ACTIONS(1477),
    [sym_get_last] = ACTIONS(1477),
    [sym_date_to_day] = ACTIONS(1477),
    [sym_date_to_month] = ACTIONS(1477),
    [sym_date_to_year] = ACTIONS(1477),
    [sym_time_to_hour] = ACTIONS(1477),
    [sym_time_to_minute] = ACTIONS(1477),
    [sym_time_to_second] = ACTIONS(1477),
    [sym_day_of_week] = ACTIONS(1477),
    [sym_set_pulse_time] = ACTIONS(1477),
    [sym_set_timer] = ACTIONS(1477),
    [sym_file_open] = ACTIONS(1477),
    [sym_file_close] = ACTIONS(1477),
    [sym_file_read] = ACTIONS(1477),
    [sym_file_read_line] = ACTIONS(1477),
    [sym_file_write] = ACTIONS(1477),
    [sym_file_write_line] = ACTIONS(1477),
    [sym_file_copy] = ACTIONS(1477),
    [sym_file_delete] = ACTIONS(1477),
    [sym_file_rename] = ACTIONS(1477),
    [sym_file_dir] = ACTIONS(1477),
    [sym_file_getdir] = ACTIONS(1477),
    [sym_file_setdir] = ACTIONS(1477),
    [sym_file_create_dir] = ACTIONS(1477),
    [sym_file_remove_dir] = ACTIONS(1477),
    [sym_file_seek] = ACTIONS(1477),
    [sym_length_array] = ACTIONS(1477),
    [sym_max_length_array] = ACTIONS(1477),
    [sym_set_length_array] = ACTIONS(1477),
    [sym_ip_client_open] = ACTIONS(1477),
    [sym_ip_client_close] = ACTIONS(1477),
    [sym_ip_server_open] = ACTIONS(1477),
    [sym_ip_server_close] = ACTIONS(1477),
    [sym_ip_bound_client_open] = ACTIONS(1477),
    [sym_ip_mc_server_open] = ACTIONS(1477),
    [sym_ip_set_option] = ACTIONS(1477),
    [sym_ssh_client_open] = ACTIONS(1477),
    [sym_ssh_client_close] = ACTIONS(1477),
    [sym_tls_client_open] = ACTIONS(1477),
    [sym_tls_client_close] = ACTIONS(1477),
    [sym_do_push] = ACTIONS(1477),
    [sym_do_push_timed] = ACTIONS(1477),
    [sym_do_release] = ACTIONS(1477),
    [sym_device_id] = ACTIONS(1477),
    [sym_device_info] = ACTIONS(1477),
    [sym_device_id_string] = ACTIONS(1477),
    [sym_device_standby] = ACTIONS(1477),
    [sym_device_wake] = ACTIONS(1477),
    [sym_timeline_create] = ACTIONS(1477),
    [sym_timeline_kill] = ACTIONS(1477),
    [sym_timeline_reload] = ACTIONS(1477),
    [sym_timeline_pause] = ACTIONS(1477),
    [sym_timeline_restart] = ACTIONS(1477),
    [sym_timeline_set] = ACTIONS(1477),
    [sym_timeline_get] = ACTIONS(1477),
    [sym_timeline_active] = ACTIONS(1477),
    [sym_timeline_dynamic_id] = ACTIONS(1477),
    [sym_variable_to_string] = ACTIONS(1477),
    [sym_string_to_variable] = ACTIONS(1477),
    [sym_length_variable_to_string] = ACTIONS(1477),
    [sym_type_cast] = ACTIONS(1477),
    [sym_variable_to_xml] = ACTIONS(1477),
    [sym_xml_to_variable] = ACTIONS(1477),
    [sym_length_variable_to_xml] = ACTIONS(1477),
    [sym_redirect_string] = ACTIONS(1477),
    [sym_random_number] = ACTIONS(1477),
    [sym_get_unique_id] = ACTIONS(1477),
    [sym_get_system_number] = ACTIONS(1477),
    [sym_set_system_number] = ACTIONS(1477),
    [sym_get_serial_number] = ACTIONS(1477),
    [sym_get_master_build] = ACTIONS(1477),
    [sym_set_validation_code] = ACTIONS(1477),
    [sym_reboot] = ACTIONS(1477),
    [sym_load_duet_module] = ACTIONS(1477),
    [sym_unload_duet_module] = ACTIONS(1477),
    [sym_duet_mem_size_set] = ACTIONS(1477),
    [sym_duet_mem_size_get] = ACTIONS(1477),
    [sym_dynamic_application_device] = ACTIONS(1477),
    [sym_dynamic_polled_port] = ACTIONS(1477),
    [sym_static_port_binding] = ACTIONS(1477),
    [sym_static_ip_binding] = ACTIONS(1477),
    [sym_get_dns_list] = ACTIONS(1477),
    [sym_set_dns_list] = ACTIONS(1477),
    [sym_get_ip_address] = ACTIONS(1477),
    [sym_set_ip_address] = ACTIONS(1477),
    [sym_get_url_list] = ACTIONS(1477),
    [sym_add_url_entry] = ACTIONS(1477),
    [sym_delete_url_entry] = ACTIONS(1477),
    [sym_abs_value] = ACTIONS(1477),
    [sym_max_value] = ACTIONS(1477),
    [sym_min_value] = ACTIONS(1477),
    [sym_exp_value] = ACTIONS(1477),
    [sym_log_value] = ACTIONS(1477),
    [sym_log10_value] = ACTIONS(1477),
    [sym_power_value] = ACTIONS(1477),
    [sym_sqrt_value] = ACTIONS(1477),
    [sym_raw_be] = ACTIONS(1477),
    [sym_raw_le] = ACTIONS(1477),
    [sym_internal_queue_size_set] = ACTIONS(1477),
    [sym_internal_queue_size_get] = ACTIONS(1477),
    [sym_internal_threshold_set] = ACTIONS(1477),
    [sym_internal_threshold_get] = ACTIONS(1477),
    [sym_rebuilt_event] = ACTIONS(1477),
    [sym_do_custom_event] = ACTIONS(1477),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1477),
    [sym_clkmgr_set_clk_source] = ACTIONS(1477),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1477),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1477),
    [sym_clkmgr_get_timezone] = ACTIONS(1477),
    [sym_clkmgr_set_timezone] = ACTIONS(1477),
    [sym_clkmgr_get_resync_period] = ACTIONS(1477),
    [sym_clkmgr_set_resync_period] = ACTIONS(1477),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1477),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1477),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1477),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1477),
    [sym_clkmgr_get_timeservers] = ACTIONS(1477),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1477),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1477),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1477),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1477),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1477),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1477),
    [sym_set_log_level] = ACTIONS(1477),
    [sym_get_log_level] = ACTIONS(1477),
    [sym_amx_log] = ACTIONS(1477),
    [sym_smtp_server_config_set] = ACTIONS(1477),
    [sym_smtp_server_config_get] = ACTIONS(1477),
    [sym_smtp_send] = ACTIONS(1477),
    [sym_validate_netlinx_account] = ACTIONS(1477),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1477),
    [sym_authenticate_certificate] = ACTIONS(1477),
    [sym_audit_netlinx_session_event] = ACTIONS(1477),
    [sym_audit_netlinx_generic_event] = ACTIONS(1477),
    [sym_get_available_flash_disk_space] = ACTIONS(1477),
    [sym_get_max_flash_disk_space] = ACTIONS(1477),
    [sym_data_create_feed] = ACTIONS(1477),
    [sym_data_delete_feed] = ACTIONS(1477),
    [sym_data_publish_feed] = ACTIONS(1477),
    [sym_data_get_published_feed] = ACTIONS(1477),
    [sym_data_add_record] = ACTIONS(1477),
    [sym_data_get_event_record] = ACTIONS(1477),
    [sym__wc_data_create_feed] = ACTIONS(1477),
    [sym__wc_data_add_record] = ACTIONS(1477),
    [sym__wc_data_get_event_record] = ACTIONS(1477),
    [sym_format] = ACTIONS(1477),
    [sym_astro_clock] = ACTIONS(1477),
    [sym_set_outdoor_temperature] = ACTIONS(1477),
    [sym_set_virtual_level_count] = ACTIONS(1477),
    [sym_set_virtual_channel_count] = ACTIONS(1477),
    [sym_set_virtual_port_count] = ACTIONS(1477),
    [sym__wc_encode_int] = ACTIONS(1477),
    [sym__wc_decode_int] = ACTIONS(1477),
    [sym__wc_int] = ACTIONS(1477),
    [sym__wc_to_ch_int] = ACTIONS(1477),
    [sym__ch_to_wc_int] = ACTIONS(1477),
    [sym__wc_find_string_int] = ACTIONS(1477),
    [sym__wc_left_string_int] = ACTIONS(1477),
    [sym__wc_lower_string_int] = ACTIONS(1477),
    [sym__wc_mid_string_int] = ACTIONS(1477),
    [sym__wc_remove_string_int] = ACTIONS(1477),
    [sym__wc_right_string_int] = ACTIONS(1477),
    [sym__wc_upper_string_int] = ACTIONS(1477),
    [sym__wc_compare_string_int] = ACTIONS(1477),
    [sym__wc_get_buffer_char_int] = ACTIONS(1477),
    [sym__wc_get_buffer_string_int] = ACTIONS(1477),
    [sym__wc_concat_string_int] = ACTIONS(1477),
    [sym__wc_file_open_int] = ACTIONS(1477),
    [sym__wc_file_close_int] = ACTIONS(1477),
    [sym__wc_file_read_int] = ACTIONS(1477),
    [sym__wc_file_read_line_int] = ACTIONS(1477),
    [sym__wc_file_write_int] = ACTIONS(1477),
    [sym__wc_file_write_line_int] = ACTIONS(1477),
    [aux_sym_preproc_include_token1] = ACTIONS(1477),
    [aux_sym_preproc_define_token1] = ACTIONS(1479),
    [aux_sym_preproc_warn_token1] = ACTIONS(1479),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1479),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1479),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1479),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1479),
    [anon_sym_COLON] = ACTIONS(1479),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1479),
    [anon_sym_BANG] = ACTIONS(1477),
    [anon_sym_TILDE] = ACTIONS(1479),
    [anon_sym_DASH] = ACTIONS(1477),
    [anon_sym_PLUS] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(1479),
    [anon_sym_SLASH] = ACTIONS(1477),
    [anon_sym_PERCENT] = ACTIONS(1479),
    [anon_sym_PIPE_PIPE] = ACTIONS(1479),
    [anon_sym_AMP_AMP] = ACTIONS(1479),
    [anon_sym_PIPE] = ACTIONS(1477),
    [anon_sym_CARET] = ACTIONS(1479),
    [anon_sym_AMP] = ACTIONS(1477),
    [anon_sym_EQ_EQ] = ACTIONS(1479),
    [anon_sym_BANG_EQ] = ACTIONS(1479),
    [anon_sym_LT_GT] = ACTIONS(1479),
    [anon_sym_GT] = ACTIONS(1477),
    [anon_sym_GT_EQ] = ACTIONS(1479),
    [anon_sym_LT_EQ] = ACTIONS(1479),
    [anon_sym_LT] = ACTIONS(1477),
    [anon_sym_LT_LT] = ACTIONS(1479),
    [anon_sym_GT_GT] = ACTIONS(1479),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1479),
    [anon_sym_SQUOTE] = ACTIONS(1479),
    [sym_number_literal] = ACTIONS(1479),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1479),
  },
  [STATE(93)] = {
    [aux_sym_subscript_expression_repeat1] = STATE(83),
    [ts_builtin_sym_end] = ACTIONS(1490),
    [sym_identifier] = ACTIONS(1488),
    [sym_program_name_keyword] = ACTIONS(1488),
    [sym_module_name_keyword] = ACTIONS(1488),
    [sym_define_device_keyword] = ACTIONS(1488),
    [sym_define_combine_keyword] = ACTIONS(1488),
    [sym_define_constant_keyword] = ACTIONS(1488),
    [sym_define_type_keyword] = ACTIONS(1488),
    [sym_define_variable_keyword] = ACTIONS(1488),
    [sym_define_system_variable_keyword] = ACTIONS(1488),
    [sym_define_connect_level_keyword] = ACTIONS(1488),
    [sym_define_latching_keyword] = ACTIONS(1488),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1488),
    [sym_define_toggling_keyword] = ACTIONS(1488),
    [sym_define_call_keyword] = ACTIONS(1488),
    [sym_define_function_keyword] = ACTIONS(1488),
    [sym_define_library_function_keyword] = ACTIONS(1488),
    [sym_define_start_keyword] = ACTIONS(1488),
    [sym_define_event_keyword] = ACTIONS(1488),
    [sym_define_program_keyword] = ACTIONS(1488),
    [sym_define_module_keyword] = ACTIONS(1488),
    [sym_char_keyword] = ACTIONS(1488),
    [sym_widechar_keyword] = ACTIONS(1488),
    [sym_integer_keyword] = ACTIONS(1488),
    [sym_sinteger_keyword] = ACTIONS(1488),
    [sym_long_keyword] = ACTIONS(1488),
    [sym_slong_keyword] = ACTIONS(1488),
    [sym_float_keyword] = ACTIONS(1488),
    [sym_double_keyword] = ACTIONS(1488),
    [sym_variant_keyword] = ACTIONS(1488),
    [sym_variantarray_keyword] = ACTIONS(1488),
    [sym_local_var_keyword] = ACTIONS(1488),
    [sym_stack_var_keyword] = ACTIONS(1488),
    [sym_constant_keyword] = ACTIONS(1488),
    [sym_volatile_keyword] = ACTIONS(1488),
    [sym_non_volatile_keyword] = ACTIONS(1488),
    [sym_persistent_keyword] = ACTIONS(1488),
    [sym_dev_keyword] = ACTIONS(1488),
    [sym_devlev_keyword] = ACTIONS(1488),
    [sym_devchan_keyword] = ACTIONS(1488),
    [sym_if_keyword] = ACTIONS(1488),
    [sym_else_keyword] = ACTIONS(1488),
    [sym_switch_keyword] = ACTIONS(1488),
    [sym_case_keyword] = ACTIONS(1488),
    [sym_default_keyword] = ACTIONS(1488),
    [sym_while_keyword] = ACTIONS(1488),
    [sym_for_keyword] = ACTIONS(1488),
    [sym_break_keyword] = ACTIONS(1488),
    [sym_continue_keyword] = ACTIONS(1488),
    [sym_return_keyword] = ACTIONS(1488),
    [sym_select_keyword] = ACTIONS(1488),
    [sym_active_keyword] = ACTIONS(1488),
    [sym_struct_keyword] = ACTIONS(1488),
    [sym_structure_keyword] = ACTIONS(1488),
    [sym_button_event_keyword] = ACTIONS(1488),
    [sym_channel_event_keyword] = ACTIONS(1488),
    [sym_level_event_keyword] = ACTIONS(1488),
    [sym_data_event_keyword] = ACTIONS(1488),
    [sym_timeline_event_keyword] = ACTIONS(1488),
    [sym_custom_event_keyword] = ACTIONS(1488),
    [sym_push_keyword] = ACTIONS(1488),
    [sym_release_keyword] = ACTIONS(1488),
    [aux_sym_on_keyword_token1] = ACTIONS(1488),
    [aux_sym_off_keyword_token1] = ACTIONS(1488),
    [sym_send_level_keyword] = ACTIONS(1488),
    [sym_send_string_keyword] = ACTIONS(1488),
    [sym_send_command_keyword] = ACTIONS(1488),
    [sym_clear_buffer_keyword] = ACTIONS(1488),
    [sym_create_buffer_keyword] = ACTIONS(1488),
    [sym_create_multi_buffer_keyword] = ACTIONS(1488),
    [sym_call_keyword] = ACTIONS(1488),
    [sym_system_call_keyword] = ACTIONS(1488),
    [sym_devchan_to_keyword] = ACTIONS(1488),
    [sym_devchan_min_to_keyword] = ACTIONS(1488),
    [sym_devchan_total_off_keyword] = ACTIONS(1488),
    [sym_devchan_pulse_keyword] = ACTIONS(1488),
    [sym_wait_keyword] = ACTIONS(1488),
    [sym_cancel_wait_keyword] = ACTIONS(1488),
    [sym_wait_until_keyword] = ACTIONS(1488),
    [sym_cancel_wait_until_keyword] = ACTIONS(1488),
    [sym_cancel_all_wait_keyword] = ACTIONS(1488),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1488),
    [sym_and] = ACTIONS(1488),
    [sym_or] = ACTIONS(1488),
    [sym_not] = ACTIONS(1488),
    [sym_xor] = ACTIONS(1488),
    [sym_band] = ACTIONS(1488),
    [sym_bor] = ACTIONS(1488),
    [sym_bxor] = ACTIONS(1488),
    [sym_bnot] = ACTIONS(1488),
    [sym_lshift] = ACTIONS(1488),
    [sym_rshift] = ACTIONS(1488),
    [sym___file__] = ACTIONS(1488),
    [sym___date__] = ACTIONS(1488),
    [sym___ldate__] = ACTIONS(1488),
    [sym___time__] = ACTIONS(1488),
    [sym___line__] = ACTIONS(1488),
    [sym___name__] = ACTIONS(1488),
    [sym___netlinx__] = ACTIONS(1488),
    [sym_netlinx_axi_version] = ACTIONS(1488),
    [sym_url_flg_tcp] = ACTIONS(1488),
    [sym_url_flg_accinfopresent] = ACTIONS(1488),
    [sym_url_flg_temp] = ACTIONS(1488),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1488),
    [sym_url_flg_stat_mask] = ACTIONS(1488),
    [sym_url_flg_stat_lookup] = ACTIONS(1488),
    [sym_url_flg_stat_connecting] = ACTIONS(1488),
    [sym_url_flg_stat_waiting] = ACTIONS(1488),
    [sym_url_flg_stat_connected] = ACTIONS(1488),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1488),
    [sym_first_virtual_device] = ACTIONS(1488),
    [sym_first_local_port] = ACTIONS(1488),
    [sym_dynamic_virtual_device] = ACTIONS(1488),
    [sym_timeline_once] = ACTIONS(1488),
    [sym_timeline_repeat] = ACTIONS(1488),
    [sym_timeline_absolute] = ACTIONS(1488),
    [sym_timeline_relative] = ACTIONS(1488),
    [sym_do_push_timed_infinite] = ACTIONS(1488),
    [sym_source_type_no_address] = ACTIONS(1488),
    [sym_source_type_neuron_id] = ACTIONS(1488),
    [sym_source_type_ip_address] = ACTIONS(1488),
    [sym_source_type_axlink] = ACTIONS(1488),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1488),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1488),
    [sym_source_type_ip_socket_address] = ACTIONS(1488),
    [sym_source_type_rs232] = ACTIONS(1488),
    [sym_source_type_internal] = ACTIONS(1488),
    [sym_source_type_mac_address] = ACTIONS(1488),
    [sym_source_type_ipv4_port] = ACTIONS(1488),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1488),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1488),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1488),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1488),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1488),
    [sym_true] = ACTIONS(1488),
    [sym_false] = ACTIONS(1488),
    [sym_file_read_only] = ACTIONS(1488),
    [sym_file_rw_new] = ACTIONS(1488),
    [sym_file_rw_append] = ACTIONS(1488),
    [sym_ip_tcp] = ACTIONS(1488),
    [sym_ip_udp] = ACTIONS(1488),
    [sym_ip_udp_2way] = ACTIONS(1488),
    [sym_xml_encode_types] = ACTIONS(1488),
    [sym_xml_encode_char_as_list] = ACTIONS(1488),
    [sym_xml_encode_le] = ACTIONS(1488),
    [sym_xml_decode_types] = ACTIONS(1488),
    [sym_xml_decode_no_preserve] = ACTIONS(1488),
    [sym_duet_dev_type_display_device] = ACTIONS(1488),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1488),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1488),
    [sym_duet_dev_type_io_device] = ACTIONS(1488),
    [sym_duet_dev_type_relay_device] = ACTIONS(1488),
    [sym_duet_dev_type_ups] = ACTIONS(1488),
    [sym_duet_dev_type_amplifier] = ACTIONS(1488),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1488),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1488),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1488),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1488),
    [sym_duet_dev_type_camera] = ACTIONS(1488),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1488),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1488),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1488),
    [sym_duet_dev_type_dss] = ACTIONS(1488),
    [sym_duet_dev_type_dvr] = ACTIONS(1488),
    [sym_duet_dev_type_disc_device] = ACTIONS(1488),
    [sym_duet_dev_type_document_camera] = ACTIONS(1488),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1488),
    [sym_duet_dev_type_hvac] = ACTIONS(1488),
    [sym_duet_dev_type_keypad] = ACTIONS(1488),
    [sym_duet_dev_type_light] = ACTIONS(1488),
    [sym_duet_dev_type_monitor] = ACTIONS(1488),
    [sym_duet_dev_type_motor] = ACTIONS(1488),
    [sym_duet_dev_type_multi_window] = ACTIONS(1488),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1488),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1488),
    [sym_duet_dev_type_receiver] = ACTIONS(1488),
    [sym_duet_dev_type_security_system] = ACTIONS(1488),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1488),
    [sym_duet_dev_type_settop_box] = ACTIONS(1488),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1488),
    [sym_duet_dev_type_switcher] = ACTIONS(1488),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1488),
    [sym_duet_dev_type_tv] = ACTIONS(1488),
    [sym_duet_dev_type_utility] = ACTIONS(1488),
    [sym_duet_dev_type_vcr] = ACTIONS(1488),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1488),
    [sym_duet_dev_type_video_processor] = ACTIONS(1488),
    [sym_duet_dev_type_video_projector] = ACTIONS(1488),
    [sym_duet_dev_type_video_wall] = ACTIONS(1488),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1488),
    [sym_duet_dev_type_weather] = ACTIONS(1488),
    [sym_duet_dev_not_polled] = ACTIONS(1488),
    [sym_duet_dev_polled] = ACTIONS(1488),
    [sym_clkmgr_mode_network] = ACTIONS(1488),
    [sym_clkmgr_mode_standalone] = ACTIONS(1488),
    [sym_normal_standby] = ACTIONS(1488),
    [sym_normal_wake] = ACTIONS(1488),
    [sym_ip_multicast_ttl_option] = ACTIONS(1488),
    [sym_ip_ttl_subnet] = ACTIONS(1488),
    [sym_ip_ttl_site] = ACTIONS(1488),
    [sym_ip_ttl_region] = ACTIONS(1488),
    [sym_ip_ttl_continent] = ACTIONS(1488),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1488),
    [sym_ip_nodelay_on] = ACTIONS(1488),
    [sym_ip_nodelay_off] = ACTIONS(1488),
    [sym_amx_error] = ACTIONS(1488),
    [sym_amx_warning] = ACTIONS(1488),
    [sym_amx_info] = ACTIONS(1488),
    [sym_amx_debug] = ACTIONS(1488),
    [sym_netlinx_login_success] = ACTIONS(1488),
    [sym_netlinx_login_fail] = ACTIONS(1488),
    [sym_netlinx_logout] = ACTIONS(1488),
    [sym_null_str] = ACTIONS(1488),
    [sym_smtp_address] = ACTIONS(1488),
    [sym_smtp_port_number] = ACTIONS(1488),
    [sym_smtp_username] = ACTIONS(1488),
    [sym_smtp_password] = ACTIONS(1488),
    [sym_smtp_require_tls] = ACTIONS(1488),
    [sym_smtp_from] = ACTIONS(1488),
    [sym_smtp_tls_true] = ACTIONS(1488),
    [sym_smtp_tls_false] = ACTIONS(1488),
    [sym_listview_on_row_select_event] = ACTIONS(1488),
    [sym_data_structure_unknown] = ACTIONS(1488),
    [sym_data_structure_datafeed] = ACTIONS(1488),
    [sym_data_structure_datarecordset] = ACTIONS(1488),
    [sym_data_structure_datarecord] = ACTIONS(1488),
    [sym_data_structure_datafield] = ACTIONS(1488),
    [sym_data_max_content_field_count] = ACTIONS(1488),
    [sym_data_max_metadata_field_count] = ACTIONS(1488),
    [sym_data_max_id_length] = ACTIONS(1488),
    [sym_data_max_name_length] = ACTIONS(1488),
    [sym_data_max_description_length] = ACTIONS(1488),
    [sym_data_max_type_length] = ACTIONS(1488),
    [sym_data_max_format_length] = ACTIONS(1488),
    [sym_data_max_source_length] = ACTIONS(1488),
    [sym_data_max_value_length] = ACTIONS(1488),
    [sym_data_max_label_length] = ACTIONS(1488),
    [sym_data_max_url_length] = ACTIONS(1488),
    [sym_max_last_login_info_length] = ACTIONS(1488),
    [sym_data_type_unknown] = ACTIONS(1488),
    [sym_data_type_string] = ACTIONS(1488),
    [sym_data_type_datetime] = ACTIONS(1488),
    [sym_data_type_date] = ACTIONS(1488),
    [sym_data_type_time] = ACTIONS(1488),
    [sym_data_type_image] = ACTIONS(1488),
    [sym_data_format_url] = ACTIONS(1488),
    [sym_data_format_phone] = ACTIONS(1488),
    [sym_data_format_email] = ACTIONS(1488),
    [sym_data_format_iso8601] = ACTIONS(1488),
    [sym_auditlog_priv] = ACTIONS(1488),
    [sym_configuration_priv] = ACTIONS(1488),
    [sym_deviceconfig_priv] = ACTIONS(1488),
    [sym_ftp_priv] = ACTIONS(1488),
    [sym_http_priv] = ACTIONS(1488),
    [sym_networkconfig_priv] = ACTIONS(1488),
    [sym_programport_priv] = ACTIONS(1488),
    [sym_remoteui_priv] = ACTIONS(1488),
    [sym_securitycontrol_priv] = ACTIONS(1488),
    [sym_softwaremanagement_priv] = ACTIONS(1488),
    [sym_terminal_priv] = ACTIONS(1488),
    [sym_tpadmin_priv] = ACTIONS(1488),
    [sym_user1_priv] = ACTIONS(1488),
    [sym_user2_priv] = ACTIONS(1488),
    [sym_user3_priv] = ACTIONS(1488),
    [sym_user4_priv] = ACTIONS(1488),
    [sym_usermanagement_priv] = ACTIONS(1488),
    [sym_valid_account] = ACTIONS(1488),
    [sym_err_name_invalid] = ACTIONS(1488),
    [sym_err_password_invalid] = ACTIONS(1488),
    [sym_err_invalid_account] = ACTIONS(1488),
    [sym_err_no_authorization] = ACTIONS(1488),
    [sym_err_invalid_parameter] = ACTIONS(1488),
    [sym_err_user_locked_out] = ACTIONS(1488),
    [sym_err_user_account_expired] = ACTIONS(1488),
    [sym_err_general_error] = ACTIONS(1488),
    [sym_err_authentication_error] = ACTIONS(1488),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1488),
    [sym_tls_validate_certificate] = ACTIONS(1488),
    [sym_tdata] = ACTIONS(1488),
    [sym_tchannel] = ACTIONS(1488),
    [sym_tlevel] = ACTIONS(1488),
    [sym_tbutton] = ACTIONS(1488),
    [sym_ttimeline] = ACTIONS(1488),
    [sym_tcustom] = ACTIONS(1488),
    [sym_url_struct] = ACTIONS(1488),
    [sym_dns_struct] = ACTIONS(1488),
    [sym_ip_address_struct] = ACTIONS(1488),
    [sym_dev_info_struct] = ACTIONS(1488),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1488),
    [sym_data_feed] = ACTIONS(1488),
    [sym_data_field] = ACTIONS(1488),
    [sym_data_record] = ACTIONS(1488),
    [sym_wc_data_feed] = ACTIONS(1488),
    [sym_wc_data_field] = ACTIONS(1488),
    [sym_wc_data_record] = ACTIONS(1488),
    [sym_last_login_info] = ACTIONS(1488),
    [sym____reserved___] = ACTIONS(1488),
    [sym_date] = ACTIONS(1488),
    [sym_day] = ACTIONS(1488),
    [sym_time] = ACTIONS(1488),
    [sym_ldate] = ACTIONS(1488),
    [sym_push_device] = ACTIONS(1488),
    [sym_push_channel] = ACTIONS(1488),
    [sym_push_devchan] = ACTIONS(1488),
    [sym_release_device] = ACTIONS(1488),
    [sym_release_channel] = ACTIONS(1488),
    [sym_release_devchan] = ACTIONS(1488),
    [sym_master_slot] = ACTIONS(1488),
    [sym_get_pulse_time] = ACTIONS(1488),
    [sym_get_timer] = ACTIONS(1488),
    [sym_master_sn] = ACTIONS(1488),
    [sym_system_number] = ACTIONS(1488),
    [sym_button] = ACTIONS(1488),
    [sym_dv_channel] = ACTIONS(1488),
    [sym_data] = ACTIONS(1488),
    [sym_level] = ACTIONS(1488),
    [sym_timeline] = ACTIONS(1488),
    [sym_channel] = ACTIONS(1488),
    [sym_custom] = ACTIONS(1488),
    [sym_atoi] = ACTIONS(1488),
    [sym_atol] = ACTIONS(1488),
    [sym_atof] = ACTIONS(1488),
    [sym_itoa] = ACTIONS(1488),
    [sym_itohex] = ACTIONS(1488),
    [sym_hextoi] = ACTIONS(1488),
    [sym_ftoa] = ACTIONS(1488),
    [sym_find_string] = ACTIONS(1488),
    [sym_left_string] = ACTIONS(1488),
    [sym_right_string] = ACTIONS(1488),
    [sym_mid_string] = ACTIONS(1488),
    [sym_remove_string] = ACTIONS(1488),
    [sym_get_buffer_string] = ACTIONS(1488),
    [sym_get_buffer_char] = ACTIONS(1488),
    [sym_get_multi_buffer_string] = ACTIONS(1488),
    [sym_length_string] = ACTIONS(1488),
    [sym_max_length_string] = ACTIONS(1488),
    [sym_set_length_string] = ACTIONS(1488),
    [sym_compare_string] = ACTIONS(1488),
    [sym_upper_string] = ACTIONS(1488),
    [sym_lower_string] = ACTIONS(1488),
    [sym_get_last] = ACTIONS(1488),
    [sym_date_to_day] = ACTIONS(1488),
    [sym_date_to_month] = ACTIONS(1488),
    [sym_date_to_year] = ACTIONS(1488),
    [sym_time_to_hour] = ACTIONS(1488),
    [sym_time_to_minute] = ACTIONS(1488),
    [sym_time_to_second] = ACTIONS(1488),
    [sym_day_of_week] = ACTIONS(1488),
    [sym_set_pulse_time] = ACTIONS(1488),
    [sym_set_timer] = ACTIONS(1488),
    [sym_file_open] = ACTIONS(1488),
    [sym_file_close] = ACTIONS(1488),
    [sym_file_read] = ACTIONS(1488),
    [sym_file_read_line] = ACTIONS(1488),
    [sym_file_write] = ACTIONS(1488),
    [sym_file_write_line] = ACTIONS(1488),
    [sym_file_copy] = ACTIONS(1488),
    [sym_file_delete] = ACTIONS(1488),
    [sym_file_rename] = ACTIONS(1488),
    [sym_file_dir] = ACTIONS(1488),
    [sym_file_getdir] = ACTIONS(1488),
    [sym_file_setdir] = ACTIONS(1488),
    [sym_file_create_dir] = ACTIONS(1488),
    [sym_file_remove_dir] = ACTIONS(1488),
    [sym_file_seek] = ACTIONS(1488),
    [sym_length_array] = ACTIONS(1488),
    [sym_max_length_array] = ACTIONS(1488),
    [sym_set_length_array] = ACTIONS(1488),
    [sym_ip_client_open] = ACTIONS(1488),
    [sym_ip_client_close] = ACTIONS(1488),
    [sym_ip_server_open] = ACTIONS(1488),
    [sym_ip_server_close] = ACTIONS(1488),
    [sym_ip_bound_client_open] = ACTIONS(1488),
    [sym_ip_mc_server_open] = ACTIONS(1488),
    [sym_ip_set_option] = ACTIONS(1488),
    [sym_ssh_client_open] = ACTIONS(1488),
    [sym_ssh_client_close] = ACTIONS(1488),
    [sym_tls_client_open] = ACTIONS(1488),
    [sym_tls_client_close] = ACTIONS(1488),
    [sym_do_push] = ACTIONS(1488),
    [sym_do_push_timed] = ACTIONS(1488),
    [sym_do_release] = ACTIONS(1488),
    [sym_device_id] = ACTIONS(1488),
    [sym_device_info] = ACTIONS(1488),
    [sym_device_id_string] = ACTIONS(1488),
    [sym_device_standby] = ACTIONS(1488),
    [sym_device_wake] = ACTIONS(1488),
    [sym_timeline_create] = ACTIONS(1488),
    [sym_timeline_kill] = ACTIONS(1488),
    [sym_timeline_reload] = ACTIONS(1488),
    [sym_timeline_pause] = ACTIONS(1488),
    [sym_timeline_restart] = ACTIONS(1488),
    [sym_timeline_set] = ACTIONS(1488),
    [sym_timeline_get] = ACTIONS(1488),
    [sym_timeline_active] = ACTIONS(1488),
    [sym_timeline_dynamic_id] = ACTIONS(1488),
    [sym_variable_to_string] = ACTIONS(1488),
    [sym_string_to_variable] = ACTIONS(1488),
    [sym_length_variable_to_string] = ACTIONS(1488),
    [sym_type_cast] = ACTIONS(1488),
    [sym_variable_to_xml] = ACTIONS(1488),
    [sym_xml_to_variable] = ACTIONS(1488),
    [sym_length_variable_to_xml] = ACTIONS(1488),
    [sym_redirect_string] = ACTIONS(1488),
    [sym_random_number] = ACTIONS(1488),
    [sym_get_unique_id] = ACTIONS(1488),
    [sym_get_system_number] = ACTIONS(1488),
    [sym_set_system_number] = ACTIONS(1488),
    [sym_get_serial_number] = ACTIONS(1488),
    [sym_get_master_build] = ACTIONS(1488),
    [sym_set_validation_code] = ACTIONS(1488),
    [sym_reboot] = ACTIONS(1488),
    [sym_load_duet_module] = ACTIONS(1488),
    [sym_unload_duet_module] = ACTIONS(1488),
    [sym_duet_mem_size_set] = ACTIONS(1488),
    [sym_duet_mem_size_get] = ACTIONS(1488),
    [sym_dynamic_application_device] = ACTIONS(1488),
    [sym_dynamic_polled_port] = ACTIONS(1488),
    [sym_static_port_binding] = ACTIONS(1488),
    [sym_static_ip_binding] = ACTIONS(1488),
    [sym_get_dns_list] = ACTIONS(1488),
    [sym_set_dns_list] = ACTIONS(1488),
    [sym_get_ip_address] = ACTIONS(1488),
    [sym_set_ip_address] = ACTIONS(1488),
    [sym_get_url_list] = ACTIONS(1488),
    [sym_add_url_entry] = ACTIONS(1488),
    [sym_delete_url_entry] = ACTIONS(1488),
    [sym_abs_value] = ACTIONS(1488),
    [sym_max_value] = ACTIONS(1488),
    [sym_min_value] = ACTIONS(1488),
    [sym_exp_value] = ACTIONS(1488),
    [sym_log_value] = ACTIONS(1488),
    [sym_log10_value] = ACTIONS(1488),
    [sym_power_value] = ACTIONS(1488),
    [sym_sqrt_value] = ACTIONS(1488),
    [sym_raw_be] = ACTIONS(1488),
    [sym_raw_le] = ACTIONS(1488),
    [sym_internal_queue_size_set] = ACTIONS(1488),
    [sym_internal_queue_size_get] = ACTIONS(1488),
    [sym_internal_threshold_set] = ACTIONS(1488),
    [sym_internal_threshold_get] = ACTIONS(1488),
    [sym_rebuilt_event] = ACTIONS(1488),
    [sym_do_custom_event] = ACTIONS(1488),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1488),
    [sym_clkmgr_set_clk_source] = ACTIONS(1488),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1488),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1488),
    [sym_clkmgr_get_timezone] = ACTIONS(1488),
    [sym_clkmgr_set_timezone] = ACTIONS(1488),
    [sym_clkmgr_get_resync_period] = ACTIONS(1488),
    [sym_clkmgr_set_resync_period] = ACTIONS(1488),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1488),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1488),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1488),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1488),
    [sym_clkmgr_get_timeservers] = ACTIONS(1488),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1488),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1488),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1488),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1488),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1488),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1488),
    [sym_set_log_level] = ACTIONS(1488),
    [sym_get_log_level] = ACTIONS(1488),
    [sym_amx_log] = ACTIONS(1488),
    [sym_smtp_server_config_set] = ACTIONS(1488),
    [sym_smtp_server_config_get] = ACTIONS(1488),
    [sym_smtp_send] = ACTIONS(1488),
    [sym_validate_netlinx_account] = ACTIONS(1488),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1488),
    [sym_authenticate_certificate] = ACTIONS(1488),
    [sym_audit_netlinx_session_event] = ACTIONS(1488),
    [sym_audit_netlinx_generic_event] = ACTIONS(1488),
    [sym_get_available_flash_disk_space] = ACTIONS(1488),
    [sym_get_max_flash_disk_space] = ACTIONS(1488),
    [sym_data_create_feed] = ACTIONS(1488),
    [sym_data_delete_feed] = ACTIONS(1488),
    [sym_data_publish_feed] = ACTIONS(1488),
    [sym_data_get_published_feed] = ACTIONS(1488),
    [sym_data_add_record] = ACTIONS(1488),
    [sym_data_get_event_record] = ACTIONS(1488),
    [sym__wc_data_create_feed] = ACTIONS(1488),
    [sym__wc_data_add_record] = ACTIONS(1488),
    [sym__wc_data_get_event_record] = ACTIONS(1488),
    [sym_format] = ACTIONS(1488),
    [sym_astro_clock] = ACTIONS(1488),
    [sym_set_outdoor_temperature] = ACTIONS(1488),
    [sym_set_virtual_level_count] = ACTIONS(1488),
    [sym_set_virtual_channel_count] = ACTIONS(1488),
    [sym_set_virtual_port_count] = ACTIONS(1488),
    [sym__wc_encode_int] = ACTIONS(1488),
    [sym__wc_decode_int] = ACTIONS(1488),
    [sym__wc_int] = ACTIONS(1488),
    [sym__wc_to_ch_int] = ACTIONS(1488),
    [sym__ch_to_wc_int] = ACTIONS(1488),
    [sym__wc_find_string_int] = ACTIONS(1488),
    [sym__wc_left_string_int] = ACTIONS(1488),
    [sym__wc_lower_string_int] = ACTIONS(1488),
    [sym__wc_mid_string_int] = ACTIONS(1488),
    [sym__wc_remove_string_int] = ACTIONS(1488),
    [sym__wc_right_string_int] = ACTIONS(1488),
    [sym__wc_upper_string_int] = ACTIONS(1488),
    [sym__wc_compare_string_int] = ACTIONS(1488),
    [sym__wc_get_buffer_char_int] = ACTIONS(1488),
    [sym__wc_get_buffer_string_int] = ACTIONS(1488),
    [sym__wc_concat_string_int] = ACTIONS(1488),
    [sym__wc_file_open_int] = ACTIONS(1488),
    [sym__wc_file_close_int] = ACTIONS(1488),
    [sym__wc_file_read_int] = ACTIONS(1488),
    [sym__wc_file_read_line_int] = ACTIONS(1488),
    [sym__wc_file_write_int] = ACTIONS(1488),
    [sym__wc_file_write_line_int] = ACTIONS(1488),
    [anon_sym_EQ] = ACTIONS(1488),
    [aux_sym_preproc_include_token1] = ACTIONS(1488),
    [aux_sym_preproc_define_token1] = ACTIONS(1490),
    [aux_sym_preproc_warn_token1] = ACTIONS(1490),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1490),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1490),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1490),
    [anon_sym_LBRACK] = ACTIONS(1490),
    [anon_sym_LBRACE] = ACTIONS(1490),
    [anon_sym_COLON] = ACTIONS(1490),
    [anon_sym_LPAREN] = ACTIONS(1488),
    [anon_sym_SEMI] = ACTIONS(1490),
    [anon_sym_BANG] = ACTIONS(1488),
    [anon_sym_TILDE] = ACTIONS(1490),
    [anon_sym_DASH] = ACTIONS(1488),
    [anon_sym_PLUS] = ACTIONS(1488),
    [anon_sym_STAR] = ACTIONS(1490),
    [anon_sym_SLASH] = ACTIONS(1488),
    [anon_sym_PERCENT] = ACTIONS(1490),
    [anon_sym_PIPE_PIPE] = ACTIONS(1490),
    [anon_sym_AMP_AMP] = ACTIONS(1490),
    [anon_sym_PIPE] = ACTIONS(1488),
    [anon_sym_CARET] = ACTIONS(1490),
    [anon_sym_AMP] = ACTIONS(1488),
    [anon_sym_EQ_EQ] = ACTIONS(1490),
    [anon_sym_BANG_EQ] = ACTIONS(1490),
    [anon_sym_LT_GT] = ACTIONS(1490),
    [anon_sym_GT] = ACTIONS(1488),
    [anon_sym_GT_EQ] = ACTIONS(1490),
    [anon_sym_LT_EQ] = ACTIONS(1490),
    [anon_sym_LT] = ACTIONS(1488),
    [anon_sym_LT_LT] = ACTIONS(1490),
    [anon_sym_GT_GT] = ACTIONS(1490),
    [anon_sym_DASH_DASH] = ACTIONS(1490),
    [anon_sym_PLUS_PLUS] = ACTIONS(1490),
    [anon_sym_DOT] = ACTIONS(1488),
    [anon_sym_DQUOTE] = ACTIONS(1490),
    [anon_sym_SQUOTE] = ACTIONS(1490),
    [sym_number_literal] = ACTIONS(1490),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1490),
  },
  [STATE(94)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1494),
    [sym_identifier] = ACTIONS(1492),
    [sym_program_name_keyword] = ACTIONS(1492),
    [sym_module_name_keyword] = ACTIONS(1492),
    [sym_define_device_keyword] = ACTIONS(1492),
    [sym_define_combine_keyword] = ACTIONS(1492),
    [sym_define_constant_keyword] = ACTIONS(1492),
    [sym_define_type_keyword] = ACTIONS(1492),
    [sym_define_variable_keyword] = ACTIONS(1492),
    [sym_define_system_variable_keyword] = ACTIONS(1492),
    [sym_define_connect_level_keyword] = ACTIONS(1492),
    [sym_define_latching_keyword] = ACTIONS(1492),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1492),
    [sym_define_toggling_keyword] = ACTIONS(1492),
    [sym_define_call_keyword] = ACTIONS(1492),
    [sym_define_function_keyword] = ACTIONS(1492),
    [sym_define_library_function_keyword] = ACTIONS(1492),
    [sym_define_start_keyword] = ACTIONS(1492),
    [sym_define_event_keyword] = ACTIONS(1492),
    [sym_define_program_keyword] = ACTIONS(1492),
    [sym_define_module_keyword] = ACTIONS(1492),
    [sym_char_keyword] = ACTIONS(1492),
    [sym_widechar_keyword] = ACTIONS(1492),
    [sym_integer_keyword] = ACTIONS(1492),
    [sym_sinteger_keyword] = ACTIONS(1492),
    [sym_long_keyword] = ACTIONS(1492),
    [sym_slong_keyword] = ACTIONS(1492),
    [sym_float_keyword] = ACTIONS(1492),
    [sym_double_keyword] = ACTIONS(1492),
    [sym_variant_keyword] = ACTIONS(1492),
    [sym_variantarray_keyword] = ACTIONS(1492),
    [sym_local_var_keyword] = ACTIONS(1492),
    [sym_stack_var_keyword] = ACTIONS(1492),
    [sym_constant_keyword] = ACTIONS(1492),
    [sym_volatile_keyword] = ACTIONS(1492),
    [sym_non_volatile_keyword] = ACTIONS(1492),
    [sym_persistent_keyword] = ACTIONS(1492),
    [sym_dev_keyword] = ACTIONS(1492),
    [sym_devlev_keyword] = ACTIONS(1492),
    [sym_devchan_keyword] = ACTIONS(1492),
    [sym_if_keyword] = ACTIONS(1492),
    [sym_else_keyword] = ACTIONS(1492),
    [sym_switch_keyword] = ACTIONS(1492),
    [sym_case_keyword] = ACTIONS(1492),
    [sym_default_keyword] = ACTIONS(1492),
    [sym_while_keyword] = ACTIONS(1492),
    [sym_for_keyword] = ACTIONS(1492),
    [sym_break_keyword] = ACTIONS(1492),
    [sym_continue_keyword] = ACTIONS(1492),
    [sym_return_keyword] = ACTIONS(1492),
    [sym_select_keyword] = ACTIONS(1492),
    [sym_active_keyword] = ACTIONS(1492),
    [sym_struct_keyword] = ACTIONS(1492),
    [sym_structure_keyword] = ACTIONS(1492),
    [sym_button_event_keyword] = ACTIONS(1492),
    [sym_channel_event_keyword] = ACTIONS(1492),
    [sym_level_event_keyword] = ACTIONS(1492),
    [sym_data_event_keyword] = ACTIONS(1492),
    [sym_timeline_event_keyword] = ACTIONS(1492),
    [sym_custom_event_keyword] = ACTIONS(1492),
    [sym_push_keyword] = ACTIONS(1492),
    [sym_release_keyword] = ACTIONS(1492),
    [aux_sym_on_keyword_token1] = ACTIONS(1492),
    [aux_sym_off_keyword_token1] = ACTIONS(1492),
    [sym_send_level_keyword] = ACTIONS(1492),
    [sym_send_string_keyword] = ACTIONS(1492),
    [sym_send_command_keyword] = ACTIONS(1492),
    [sym_clear_buffer_keyword] = ACTIONS(1492),
    [sym_create_buffer_keyword] = ACTIONS(1492),
    [sym_create_multi_buffer_keyword] = ACTIONS(1492),
    [sym_call_keyword] = ACTIONS(1492),
    [sym_system_call_keyword] = ACTIONS(1492),
    [sym_devchan_to_keyword] = ACTIONS(1492),
    [sym_devchan_min_to_keyword] = ACTIONS(1492),
    [sym_devchan_total_off_keyword] = ACTIONS(1492),
    [sym_devchan_pulse_keyword] = ACTIONS(1492),
    [sym_wait_keyword] = ACTIONS(1492),
    [sym_cancel_wait_keyword] = ACTIONS(1492),
    [sym_wait_until_keyword] = ACTIONS(1492),
    [sym_cancel_wait_until_keyword] = ACTIONS(1492),
    [sym_cancel_all_wait_keyword] = ACTIONS(1492),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1492),
    [sym_and] = ACTIONS(1417),
    [sym_or] = ACTIONS(1419),
    [sym_not] = ACTIONS(1492),
    [sym_xor] = ACTIONS(1421),
    [sym_band] = ACTIONS(1423),
    [sym_bor] = ACTIONS(1425),
    [sym_bxor] = ACTIONS(1421),
    [sym_bnot] = ACTIONS(1492),
    [sym_lshift] = ACTIONS(1427),
    [sym_rshift] = ACTIONS(1427),
    [sym___file__] = ACTIONS(1492),
    [sym___date__] = ACTIONS(1492),
    [sym___ldate__] = ACTIONS(1492),
    [sym___time__] = ACTIONS(1492),
    [sym___line__] = ACTIONS(1492),
    [sym___name__] = ACTIONS(1492),
    [sym___netlinx__] = ACTIONS(1492),
    [sym_netlinx_axi_version] = ACTIONS(1492),
    [sym_url_flg_tcp] = ACTIONS(1492),
    [sym_url_flg_accinfopresent] = ACTIONS(1492),
    [sym_url_flg_temp] = ACTIONS(1492),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1492),
    [sym_url_flg_stat_mask] = ACTIONS(1492),
    [sym_url_flg_stat_lookup] = ACTIONS(1492),
    [sym_url_flg_stat_connecting] = ACTIONS(1492),
    [sym_url_flg_stat_waiting] = ACTIONS(1492),
    [sym_url_flg_stat_connected] = ACTIONS(1492),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1492),
    [sym_first_virtual_device] = ACTIONS(1492),
    [sym_first_local_port] = ACTIONS(1492),
    [sym_dynamic_virtual_device] = ACTIONS(1492),
    [sym_timeline_once] = ACTIONS(1492),
    [sym_timeline_repeat] = ACTIONS(1492),
    [sym_timeline_absolute] = ACTIONS(1492),
    [sym_timeline_relative] = ACTIONS(1492),
    [sym_do_push_timed_infinite] = ACTIONS(1492),
    [sym_source_type_no_address] = ACTIONS(1492),
    [sym_source_type_neuron_id] = ACTIONS(1492),
    [sym_source_type_ip_address] = ACTIONS(1492),
    [sym_source_type_axlink] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1492),
    [sym_source_type_ip_socket_address] = ACTIONS(1492),
    [sym_source_type_rs232] = ACTIONS(1492),
    [sym_source_type_internal] = ACTIONS(1492),
    [sym_source_type_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1492),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1492),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1492),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1492),
    [sym_true] = ACTIONS(1492),
    [sym_false] = ACTIONS(1492),
    [sym_file_read_only] = ACTIONS(1492),
    [sym_file_rw_new] = ACTIONS(1492),
    [sym_file_rw_append] = ACTIONS(1492),
    [sym_ip_tcp] = ACTIONS(1492),
    [sym_ip_udp] = ACTIONS(1492),
    [sym_ip_udp_2way] = ACTIONS(1492),
    [sym_xml_encode_types] = ACTIONS(1492),
    [sym_xml_encode_char_as_list] = ACTIONS(1492),
    [sym_xml_encode_le] = ACTIONS(1492),
    [sym_xml_decode_types] = ACTIONS(1492),
    [sym_xml_decode_no_preserve] = ACTIONS(1492),
    [sym_duet_dev_type_display_device] = ACTIONS(1492),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_io_device] = ACTIONS(1492),
    [sym_duet_dev_type_relay_device] = ACTIONS(1492),
    [sym_duet_dev_type_ups] = ACTIONS(1492),
    [sym_duet_dev_type_amplifier] = ACTIONS(1492),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1492),
    [sym_duet_dev_type_camera] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1492),
    [sym_duet_dev_type_dss] = ACTIONS(1492),
    [sym_duet_dev_type_dvr] = ACTIONS(1492),
    [sym_duet_dev_type_disc_device] = ACTIONS(1492),
    [sym_duet_dev_type_document_camera] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1492),
    [sym_duet_dev_type_hvac] = ACTIONS(1492),
    [sym_duet_dev_type_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_light] = ACTIONS(1492),
    [sym_duet_dev_type_monitor] = ACTIONS(1492),
    [sym_duet_dev_type_motor] = ACTIONS(1492),
    [sym_duet_dev_type_multi_window] = ACTIONS(1492),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1492),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1492),
    [sym_duet_dev_type_receiver] = ACTIONS(1492),
    [sym_duet_dev_type_security_system] = ACTIONS(1492),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1492),
    [sym_duet_dev_type_settop_box] = ACTIONS(1492),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1492),
    [sym_duet_dev_type_switcher] = ACTIONS(1492),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_tv] = ACTIONS(1492),
    [sym_duet_dev_type_utility] = ACTIONS(1492),
    [sym_duet_dev_type_vcr] = ACTIONS(1492),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_video_processor] = ACTIONS(1492),
    [sym_duet_dev_type_video_projector] = ACTIONS(1492),
    [sym_duet_dev_type_video_wall] = ACTIONS(1492),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1492),
    [sym_duet_dev_type_weather] = ACTIONS(1492),
    [sym_duet_dev_not_polled] = ACTIONS(1492),
    [sym_duet_dev_polled] = ACTIONS(1492),
    [sym_clkmgr_mode_network] = ACTIONS(1492),
    [sym_clkmgr_mode_standalone] = ACTIONS(1492),
    [sym_normal_standby] = ACTIONS(1492),
    [sym_normal_wake] = ACTIONS(1492),
    [sym_ip_multicast_ttl_option] = ACTIONS(1492),
    [sym_ip_ttl_subnet] = ACTIONS(1492),
    [sym_ip_ttl_site] = ACTIONS(1492),
    [sym_ip_ttl_region] = ACTIONS(1492),
    [sym_ip_ttl_continent] = ACTIONS(1492),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1492),
    [sym_ip_nodelay_on] = ACTIONS(1492),
    [sym_ip_nodelay_off] = ACTIONS(1492),
    [sym_amx_error] = ACTIONS(1492),
    [sym_amx_warning] = ACTIONS(1492),
    [sym_amx_info] = ACTIONS(1492),
    [sym_amx_debug] = ACTIONS(1492),
    [sym_netlinx_login_success] = ACTIONS(1492),
    [sym_netlinx_login_fail] = ACTIONS(1492),
    [sym_netlinx_logout] = ACTIONS(1492),
    [sym_null_str] = ACTIONS(1492),
    [sym_smtp_address] = ACTIONS(1492),
    [sym_smtp_port_number] = ACTIONS(1492),
    [sym_smtp_username] = ACTIONS(1492),
    [sym_smtp_password] = ACTIONS(1492),
    [sym_smtp_require_tls] = ACTIONS(1492),
    [sym_smtp_from] = ACTIONS(1492),
    [sym_smtp_tls_true] = ACTIONS(1492),
    [sym_smtp_tls_false] = ACTIONS(1492),
    [sym_listview_on_row_select_event] = ACTIONS(1492),
    [sym_data_structure_unknown] = ACTIONS(1492),
    [sym_data_structure_datafeed] = ACTIONS(1492),
    [sym_data_structure_datarecordset] = ACTIONS(1492),
    [sym_data_structure_datarecord] = ACTIONS(1492),
    [sym_data_structure_datafield] = ACTIONS(1492),
    [sym_data_max_content_field_count] = ACTIONS(1492),
    [sym_data_max_metadata_field_count] = ACTIONS(1492),
    [sym_data_max_id_length] = ACTIONS(1492),
    [sym_data_max_name_length] = ACTIONS(1492),
    [sym_data_max_description_length] = ACTIONS(1492),
    [sym_data_max_type_length] = ACTIONS(1492),
    [sym_data_max_format_length] = ACTIONS(1492),
    [sym_data_max_source_length] = ACTIONS(1492),
    [sym_data_max_value_length] = ACTIONS(1492),
    [sym_data_max_label_length] = ACTIONS(1492),
    [sym_data_max_url_length] = ACTIONS(1492),
    [sym_max_last_login_info_length] = ACTIONS(1492),
    [sym_data_type_unknown] = ACTIONS(1492),
    [sym_data_type_string] = ACTIONS(1492),
    [sym_data_type_datetime] = ACTIONS(1492),
    [sym_data_type_date] = ACTIONS(1492),
    [sym_data_type_time] = ACTIONS(1492),
    [sym_data_type_image] = ACTIONS(1492),
    [sym_data_format_url] = ACTIONS(1492),
    [sym_data_format_phone] = ACTIONS(1492),
    [sym_data_format_email] = ACTIONS(1492),
    [sym_data_format_iso8601] = ACTIONS(1492),
    [sym_auditlog_priv] = ACTIONS(1492),
    [sym_configuration_priv] = ACTIONS(1492),
    [sym_deviceconfig_priv] = ACTIONS(1492),
    [sym_ftp_priv] = ACTIONS(1492),
    [sym_http_priv] = ACTIONS(1492),
    [sym_networkconfig_priv] = ACTIONS(1492),
    [sym_programport_priv] = ACTIONS(1492),
    [sym_remoteui_priv] = ACTIONS(1492),
    [sym_securitycontrol_priv] = ACTIONS(1492),
    [sym_softwaremanagement_priv] = ACTIONS(1492),
    [sym_terminal_priv] = ACTIONS(1492),
    [sym_tpadmin_priv] = ACTIONS(1492),
    [sym_user1_priv] = ACTIONS(1492),
    [sym_user2_priv] = ACTIONS(1492),
    [sym_user3_priv] = ACTIONS(1492),
    [sym_user4_priv] = ACTIONS(1492),
    [sym_usermanagement_priv] = ACTIONS(1492),
    [sym_valid_account] = ACTIONS(1492),
    [sym_err_name_invalid] = ACTIONS(1492),
    [sym_err_password_invalid] = ACTIONS(1492),
    [sym_err_invalid_account] = ACTIONS(1492),
    [sym_err_no_authorization] = ACTIONS(1492),
    [sym_err_invalid_parameter] = ACTIONS(1492),
    [sym_err_user_locked_out] = ACTIONS(1492),
    [sym_err_user_account_expired] = ACTIONS(1492),
    [sym_err_general_error] = ACTIONS(1492),
    [sym_err_authentication_error] = ACTIONS(1492),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1492),
    [sym_tls_validate_certificate] = ACTIONS(1492),
    [sym_tdata] = ACTIONS(1492),
    [sym_tchannel] = ACTIONS(1492),
    [sym_tlevel] = ACTIONS(1492),
    [sym_tbutton] = ACTIONS(1492),
    [sym_ttimeline] = ACTIONS(1492),
    [sym_tcustom] = ACTIONS(1492),
    [sym_url_struct] = ACTIONS(1492),
    [sym_dns_struct] = ACTIONS(1492),
    [sym_ip_address_struct] = ACTIONS(1492),
    [sym_dev_info_struct] = ACTIONS(1492),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1492),
    [sym_data_feed] = ACTIONS(1492),
    [sym_data_field] = ACTIONS(1492),
    [sym_data_record] = ACTIONS(1492),
    [sym_wc_data_feed] = ACTIONS(1492),
    [sym_wc_data_field] = ACTIONS(1492),
    [sym_wc_data_record] = ACTIONS(1492),
    [sym_last_login_info] = ACTIONS(1492),
    [sym____reserved___] = ACTIONS(1492),
    [sym_date] = ACTIONS(1492),
    [sym_day] = ACTIONS(1492),
    [sym_time] = ACTIONS(1492),
    [sym_ldate] = ACTIONS(1492),
    [sym_push_device] = ACTIONS(1492),
    [sym_push_channel] = ACTIONS(1492),
    [sym_push_devchan] = ACTIONS(1492),
    [sym_release_device] = ACTIONS(1492),
    [sym_release_channel] = ACTIONS(1492),
    [sym_release_devchan] = ACTIONS(1492),
    [sym_master_slot] = ACTIONS(1492),
    [sym_get_pulse_time] = ACTIONS(1492),
    [sym_get_timer] = ACTIONS(1492),
    [sym_master_sn] = ACTIONS(1492),
    [sym_system_number] = ACTIONS(1492),
    [sym_button] = ACTIONS(1492),
    [sym_dv_channel] = ACTIONS(1492),
    [sym_data] = ACTIONS(1492),
    [sym_level] = ACTIONS(1492),
    [sym_timeline] = ACTIONS(1492),
    [sym_channel] = ACTIONS(1492),
    [sym_custom] = ACTIONS(1492),
    [sym_atoi] = ACTIONS(1492),
    [sym_atol] = ACTIONS(1492),
    [sym_atof] = ACTIONS(1492),
    [sym_itoa] = ACTIONS(1492),
    [sym_itohex] = ACTIONS(1492),
    [sym_hextoi] = ACTIONS(1492),
    [sym_ftoa] = ACTIONS(1492),
    [sym_find_string] = ACTIONS(1492),
    [sym_left_string] = ACTIONS(1492),
    [sym_right_string] = ACTIONS(1492),
    [sym_mid_string] = ACTIONS(1492),
    [sym_remove_string] = ACTIONS(1492),
    [sym_get_buffer_string] = ACTIONS(1492),
    [sym_get_buffer_char] = ACTIONS(1492),
    [sym_get_multi_buffer_string] = ACTIONS(1492),
    [sym_length_string] = ACTIONS(1492),
    [sym_max_length_string] = ACTIONS(1492),
    [sym_set_length_string] = ACTIONS(1492),
    [sym_compare_string] = ACTIONS(1492),
    [sym_upper_string] = ACTIONS(1492),
    [sym_lower_string] = ACTIONS(1492),
    [sym_get_last] = ACTIONS(1492),
    [sym_date_to_day] = ACTIONS(1492),
    [sym_date_to_month] = ACTIONS(1492),
    [sym_date_to_year] = ACTIONS(1492),
    [sym_time_to_hour] = ACTIONS(1492),
    [sym_time_to_minute] = ACTIONS(1492),
    [sym_time_to_second] = ACTIONS(1492),
    [sym_day_of_week] = ACTIONS(1492),
    [sym_set_pulse_time] = ACTIONS(1492),
    [sym_set_timer] = ACTIONS(1492),
    [sym_file_open] = ACTIONS(1492),
    [sym_file_close] = ACTIONS(1492),
    [sym_file_read] = ACTIONS(1492),
    [sym_file_read_line] = ACTIONS(1492),
    [sym_file_write] = ACTIONS(1492),
    [sym_file_write_line] = ACTIONS(1492),
    [sym_file_copy] = ACTIONS(1492),
    [sym_file_delete] = ACTIONS(1492),
    [sym_file_rename] = ACTIONS(1492),
    [sym_file_dir] = ACTIONS(1492),
    [sym_file_getdir] = ACTIONS(1492),
    [sym_file_setdir] = ACTIONS(1492),
    [sym_file_create_dir] = ACTIONS(1492),
    [sym_file_remove_dir] = ACTIONS(1492),
    [sym_file_seek] = ACTIONS(1492),
    [sym_length_array] = ACTIONS(1492),
    [sym_max_length_array] = ACTIONS(1492),
    [sym_set_length_array] = ACTIONS(1492),
    [sym_ip_client_open] = ACTIONS(1492),
    [sym_ip_client_close] = ACTIONS(1492),
    [sym_ip_server_open] = ACTIONS(1492),
    [sym_ip_server_close] = ACTIONS(1492),
    [sym_ip_bound_client_open] = ACTIONS(1492),
    [sym_ip_mc_server_open] = ACTIONS(1492),
    [sym_ip_set_option] = ACTIONS(1492),
    [sym_ssh_client_open] = ACTIONS(1492),
    [sym_ssh_client_close] = ACTIONS(1492),
    [sym_tls_client_open] = ACTIONS(1492),
    [sym_tls_client_close] = ACTIONS(1492),
    [sym_do_push] = ACTIONS(1492),
    [sym_do_push_timed] = ACTIONS(1492),
    [sym_do_release] = ACTIONS(1492),
    [sym_device_id] = ACTIONS(1492),
    [sym_device_info] = ACTIONS(1492),
    [sym_device_id_string] = ACTIONS(1492),
    [sym_device_standby] = ACTIONS(1492),
    [sym_device_wake] = ACTIONS(1492),
    [sym_timeline_create] = ACTIONS(1492),
    [sym_timeline_kill] = ACTIONS(1492),
    [sym_timeline_reload] = ACTIONS(1492),
    [sym_timeline_pause] = ACTIONS(1492),
    [sym_timeline_restart] = ACTIONS(1492),
    [sym_timeline_set] = ACTIONS(1492),
    [sym_timeline_get] = ACTIONS(1492),
    [sym_timeline_active] = ACTIONS(1492),
    [sym_timeline_dynamic_id] = ACTIONS(1492),
    [sym_variable_to_string] = ACTIONS(1492),
    [sym_string_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_string] = ACTIONS(1492),
    [sym_type_cast] = ACTIONS(1492),
    [sym_variable_to_xml] = ACTIONS(1492),
    [sym_xml_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_xml] = ACTIONS(1492),
    [sym_redirect_string] = ACTIONS(1492),
    [sym_random_number] = ACTIONS(1492),
    [sym_get_unique_id] = ACTIONS(1492),
    [sym_get_system_number] = ACTIONS(1492),
    [sym_set_system_number] = ACTIONS(1492),
    [sym_get_serial_number] = ACTIONS(1492),
    [sym_get_master_build] = ACTIONS(1492),
    [sym_set_validation_code] = ACTIONS(1492),
    [sym_reboot] = ACTIONS(1492),
    [sym_load_duet_module] = ACTIONS(1492),
    [sym_unload_duet_module] = ACTIONS(1492),
    [sym_duet_mem_size_set] = ACTIONS(1492),
    [sym_duet_mem_size_get] = ACTIONS(1492),
    [sym_dynamic_application_device] = ACTIONS(1492),
    [sym_dynamic_polled_port] = ACTIONS(1492),
    [sym_static_port_binding] = ACTIONS(1492),
    [sym_static_ip_binding] = ACTIONS(1492),
    [sym_get_dns_list] = ACTIONS(1492),
    [sym_set_dns_list] = ACTIONS(1492),
    [sym_get_ip_address] = ACTIONS(1492),
    [sym_set_ip_address] = ACTIONS(1492),
    [sym_get_url_list] = ACTIONS(1492),
    [sym_add_url_entry] = ACTIONS(1492),
    [sym_delete_url_entry] = ACTIONS(1492),
    [sym_abs_value] = ACTIONS(1492),
    [sym_max_value] = ACTIONS(1492),
    [sym_min_value] = ACTIONS(1492),
    [sym_exp_value] = ACTIONS(1492),
    [sym_log_value] = ACTIONS(1492),
    [sym_log10_value] = ACTIONS(1492),
    [sym_power_value] = ACTIONS(1492),
    [sym_sqrt_value] = ACTIONS(1492),
    [sym_raw_be] = ACTIONS(1492),
    [sym_raw_le] = ACTIONS(1492),
    [sym_internal_queue_size_set] = ACTIONS(1492),
    [sym_internal_queue_size_get] = ACTIONS(1492),
    [sym_internal_threshold_set] = ACTIONS(1492),
    [sym_internal_threshold_get] = ACTIONS(1492),
    [sym_rebuilt_event] = ACTIONS(1492),
    [sym_do_custom_event] = ACTIONS(1492),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1492),
    [sym_clkmgr_set_clk_source] = ACTIONS(1492),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1492),
    [sym_clkmgr_get_timezone] = ACTIONS(1492),
    [sym_clkmgr_set_timezone] = ACTIONS(1492),
    [sym_clkmgr_get_resync_period] = ACTIONS(1492),
    [sym_clkmgr_set_resync_period] = ACTIONS(1492),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_timeservers] = ACTIONS(1492),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_set_log_level] = ACTIONS(1492),
    [sym_get_log_level] = ACTIONS(1492),
    [sym_amx_log] = ACTIONS(1492),
    [sym_smtp_server_config_set] = ACTIONS(1492),
    [sym_smtp_server_config_get] = ACTIONS(1492),
    [sym_smtp_send] = ACTIONS(1492),
    [sym_validate_netlinx_account] = ACTIONS(1492),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1492),
    [sym_authenticate_certificate] = ACTIONS(1492),
    [sym_audit_netlinx_session_event] = ACTIONS(1492),
    [sym_audit_netlinx_generic_event] = ACTIONS(1492),
    [sym_get_available_flash_disk_space] = ACTIONS(1492),
    [sym_get_max_flash_disk_space] = ACTIONS(1492),
    [sym_data_create_feed] = ACTIONS(1492),
    [sym_data_delete_feed] = ACTIONS(1492),
    [sym_data_publish_feed] = ACTIONS(1492),
    [sym_data_get_published_feed] = ACTIONS(1492),
    [sym_data_add_record] = ACTIONS(1492),
    [sym_data_get_event_record] = ACTIONS(1492),
    [sym__wc_data_create_feed] = ACTIONS(1492),
    [sym__wc_data_add_record] = ACTIONS(1492),
    [sym__wc_data_get_event_record] = ACTIONS(1492),
    [sym_format] = ACTIONS(1492),
    [sym_astro_clock] = ACTIONS(1492),
    [sym_set_outdoor_temperature] = ACTIONS(1492),
    [sym_set_virtual_level_count] = ACTIONS(1492),
    [sym_set_virtual_channel_count] = ACTIONS(1492),
    [sym_set_virtual_port_count] = ACTIONS(1492),
    [sym__wc_encode_int] = ACTIONS(1492),
    [sym__wc_decode_int] = ACTIONS(1492),
    [sym__wc_int] = ACTIONS(1492),
    [sym__wc_to_ch_int] = ACTIONS(1492),
    [sym__ch_to_wc_int] = ACTIONS(1492),
    [sym__wc_find_string_int] = ACTIONS(1492),
    [sym__wc_left_string_int] = ACTIONS(1492),
    [sym__wc_lower_string_int] = ACTIONS(1492),
    [sym__wc_mid_string_int] = ACTIONS(1492),
    [sym__wc_remove_string_int] = ACTIONS(1492),
    [sym__wc_right_string_int] = ACTIONS(1492),
    [sym__wc_upper_string_int] = ACTIONS(1492),
    [sym__wc_compare_string_int] = ACTIONS(1492),
    [sym__wc_get_buffer_char_int] = ACTIONS(1492),
    [sym__wc_get_buffer_string_int] = ACTIONS(1492),
    [sym__wc_concat_string_int] = ACTIONS(1492),
    [sym__wc_file_open_int] = ACTIONS(1492),
    [sym__wc_file_close_int] = ACTIONS(1492),
    [sym__wc_file_read_int] = ACTIONS(1492),
    [sym__wc_file_read_line_int] = ACTIONS(1492),
    [sym__wc_file_write_int] = ACTIONS(1492),
    [sym__wc_file_write_line_int] = ACTIONS(1492),
    [aux_sym_preproc_include_token1] = ACTIONS(1492),
    [aux_sym_preproc_define_token1] = ACTIONS(1494),
    [aux_sym_preproc_warn_token1] = ACTIONS(1494),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1494),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1494),
    [anon_sym_COLON] = ACTIONS(1431),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1494),
    [anon_sym_BANG] = ACTIONS(1492),
    [anon_sym_TILDE] = ACTIONS(1494),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1443),
    [anon_sym_AMP_AMP] = ACTIONS(1445),
    [anon_sym_PIPE] = ACTIONS(1425),
    [anon_sym_CARET] = ACTIONS(1447),
    [anon_sym_AMP] = ACTIONS(1423),
    [anon_sym_EQ_EQ] = ACTIONS(1449),
    [anon_sym_BANG_EQ] = ACTIONS(1449),
    [anon_sym_LT_GT] = ACTIONS(1449),
    [anon_sym_GT] = ACTIONS(1451),
    [anon_sym_GT_EQ] = ACTIONS(1453),
    [anon_sym_LT_EQ] = ACTIONS(1453),
    [anon_sym_LT] = ACTIONS(1451),
    [anon_sym_LT_LT] = ACTIONS(1455),
    [anon_sym_GT_GT] = ACTIONS(1455),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1494),
    [anon_sym_SQUOTE] = ACTIONS(1494),
    [sym_number_literal] = ACTIONS(1494),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1494),
  },
  [STATE(95)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1415),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1421),
    [sym_band] = ACTIONS(1423),
    [sym_bor] = ACTIONS(1425),
    [sym_bxor] = ACTIONS(1421),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1427),
    [sym_rshift] = ACTIONS(1427),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1425),
    [anon_sym_CARET] = ACTIONS(1447),
    [anon_sym_AMP] = ACTIONS(1423),
    [anon_sym_EQ_EQ] = ACTIONS(1449),
    [anon_sym_BANG_EQ] = ACTIONS(1449),
    [anon_sym_LT_GT] = ACTIONS(1449),
    [anon_sym_GT] = ACTIONS(1451),
    [anon_sym_GT_EQ] = ACTIONS(1453),
    [anon_sym_LT_EQ] = ACTIONS(1453),
    [anon_sym_LT] = ACTIONS(1451),
    [anon_sym_LT_LT] = ACTIONS(1455),
    [anon_sym_GT_GT] = ACTIONS(1455),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(96)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1415),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1417),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1421),
    [sym_band] = ACTIONS(1423),
    [sym_bor] = ACTIONS(1425),
    [sym_bxor] = ACTIONS(1421),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1427),
    [sym_rshift] = ACTIONS(1427),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1445),
    [anon_sym_PIPE] = ACTIONS(1425),
    [anon_sym_CARET] = ACTIONS(1447),
    [anon_sym_AMP] = ACTIONS(1423),
    [anon_sym_EQ_EQ] = ACTIONS(1449),
    [anon_sym_BANG_EQ] = ACTIONS(1449),
    [anon_sym_LT_GT] = ACTIONS(1449),
    [anon_sym_GT] = ACTIONS(1451),
    [anon_sym_GT_EQ] = ACTIONS(1453),
    [anon_sym_LT_EQ] = ACTIONS(1453),
    [anon_sym_LT] = ACTIONS(1451),
    [anon_sym_LT_LT] = ACTIONS(1455),
    [anon_sym_GT_GT] = ACTIONS(1455),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(97)] = {
    [sym_identifier] = ACTIONS(1531),
    [sym_program_name_keyword] = ACTIONS(1531),
    [sym_module_name_keyword] = ACTIONS(1531),
    [sym_define_device_keyword] = ACTIONS(1531),
    [sym_define_combine_keyword] = ACTIONS(1531),
    [sym_define_constant_keyword] = ACTIONS(1531),
    [sym_define_type_keyword] = ACTIONS(1531),
    [sym_define_variable_keyword] = ACTIONS(1531),
    [sym_define_system_variable_keyword] = ACTIONS(1531),
    [sym_define_connect_level_keyword] = ACTIONS(1531),
    [sym_define_latching_keyword] = ACTIONS(1531),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1531),
    [sym_define_toggling_keyword] = ACTIONS(1531),
    [sym_define_call_keyword] = ACTIONS(1531),
    [sym_define_function_keyword] = ACTIONS(1531),
    [sym_define_library_function_keyword] = ACTIONS(1531),
    [sym_define_start_keyword] = ACTIONS(1531),
    [sym_define_event_keyword] = ACTIONS(1531),
    [sym_define_program_keyword] = ACTIONS(1531),
    [sym_define_module_keyword] = ACTIONS(1531),
    [sym_char_keyword] = ACTIONS(1531),
    [sym_widechar_keyword] = ACTIONS(1531),
    [sym_integer_keyword] = ACTIONS(1531),
    [sym_sinteger_keyword] = ACTIONS(1531),
    [sym_long_keyword] = ACTIONS(1531),
    [sym_slong_keyword] = ACTIONS(1531),
    [sym_float_keyword] = ACTIONS(1531),
    [sym_double_keyword] = ACTIONS(1531),
    [sym_variant_keyword] = ACTIONS(1531),
    [sym_variantarray_keyword] = ACTIONS(1531),
    [sym_local_var_keyword] = ACTIONS(1531),
    [sym_stack_var_keyword] = ACTIONS(1531),
    [sym_constant_keyword] = ACTIONS(1531),
    [sym_volatile_keyword] = ACTIONS(1531),
    [sym_non_volatile_keyword] = ACTIONS(1531),
    [sym_persistent_keyword] = ACTIONS(1531),
    [sym_dev_keyword] = ACTIONS(1531),
    [sym_devlev_keyword] = ACTIONS(1531),
    [sym_devchan_keyword] = ACTIONS(1531),
    [sym_if_keyword] = ACTIONS(1531),
    [sym_else_keyword] = ACTIONS(1531),
    [sym_switch_keyword] = ACTIONS(1531),
    [sym_case_keyword] = ACTIONS(1531),
    [sym_default_keyword] = ACTIONS(1531),
    [sym_while_keyword] = ACTIONS(1531),
    [sym_for_keyword] = ACTIONS(1531),
    [sym_break_keyword] = ACTIONS(1531),
    [sym_continue_keyword] = ACTIONS(1531),
    [sym_return_keyword] = ACTIONS(1531),
    [sym_select_keyword] = ACTIONS(1531),
    [sym_active_keyword] = ACTIONS(1531),
    [sym_struct_keyword] = ACTIONS(1531),
    [sym_structure_keyword] = ACTIONS(1531),
    [sym_button_event_keyword] = ACTIONS(1531),
    [sym_channel_event_keyword] = ACTIONS(1531),
    [sym_level_event_keyword] = ACTIONS(1531),
    [sym_data_event_keyword] = ACTIONS(1531),
    [sym_timeline_event_keyword] = ACTIONS(1531),
    [sym_custom_event_keyword] = ACTIONS(1531),
    [sym_push_keyword] = ACTIONS(1531),
    [sym_release_keyword] = ACTIONS(1531),
    [aux_sym_on_keyword_token1] = ACTIONS(1531),
    [aux_sym_off_keyword_token1] = ACTIONS(1531),
    [sym_send_level_keyword] = ACTIONS(1531),
    [sym_send_string_keyword] = ACTIONS(1531),
    [sym_send_command_keyword] = ACTIONS(1531),
    [sym_clear_buffer_keyword] = ACTIONS(1531),
    [sym_create_buffer_keyword] = ACTIONS(1531),
    [sym_create_multi_buffer_keyword] = ACTIONS(1531),
    [sym_call_keyword] = ACTIONS(1531),
    [sym_system_call_keyword] = ACTIONS(1531),
    [sym_devchan_to_keyword] = ACTIONS(1531),
    [sym_devchan_min_to_keyword] = ACTIONS(1531),
    [sym_devchan_total_off_keyword] = ACTIONS(1531),
    [sym_devchan_pulse_keyword] = ACTIONS(1531),
    [sym_wait_keyword] = ACTIONS(1531),
    [sym_cancel_wait_keyword] = ACTIONS(1531),
    [sym_wait_until_keyword] = ACTIONS(1531),
    [sym_cancel_wait_until_keyword] = ACTIONS(1531),
    [sym_cancel_all_wait_keyword] = ACTIONS(1531),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1531),
    [sym_and] = ACTIONS(1531),
    [sym_or] = ACTIONS(1531),
    [sym_not] = ACTIONS(1531),
    [sym_xor] = ACTIONS(1531),
    [sym_band] = ACTIONS(1531),
    [sym_bor] = ACTIONS(1531),
    [sym_bxor] = ACTIONS(1531),
    [sym_bnot] = ACTIONS(1531),
    [sym_lshift] = ACTIONS(1531),
    [sym_rshift] = ACTIONS(1531),
    [sym___file__] = ACTIONS(1531),
    [sym___date__] = ACTIONS(1531),
    [sym___ldate__] = ACTIONS(1531),
    [sym___time__] = ACTIONS(1531),
    [sym___line__] = ACTIONS(1531),
    [sym___name__] = ACTIONS(1531),
    [sym___netlinx__] = ACTIONS(1531),
    [sym_netlinx_axi_version] = ACTIONS(1531),
    [sym_url_flg_tcp] = ACTIONS(1531),
    [sym_url_flg_accinfopresent] = ACTIONS(1531),
    [sym_url_flg_temp] = ACTIONS(1531),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1531),
    [sym_url_flg_stat_mask] = ACTIONS(1531),
    [sym_url_flg_stat_lookup] = ACTIONS(1531),
    [sym_url_flg_stat_connecting] = ACTIONS(1531),
    [sym_url_flg_stat_waiting] = ACTIONS(1531),
    [sym_url_flg_stat_connected] = ACTIONS(1531),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1531),
    [sym_first_virtual_device] = ACTIONS(1531),
    [sym_first_local_port] = ACTIONS(1531),
    [sym_dynamic_virtual_device] = ACTIONS(1531),
    [sym_timeline_once] = ACTIONS(1531),
    [sym_timeline_repeat] = ACTIONS(1531),
    [sym_timeline_absolute] = ACTIONS(1531),
    [sym_timeline_relative] = ACTIONS(1531),
    [sym_do_push_timed_infinite] = ACTIONS(1531),
    [sym_source_type_no_address] = ACTIONS(1531),
    [sym_source_type_neuron_id] = ACTIONS(1531),
    [sym_source_type_ip_address] = ACTIONS(1531),
    [sym_source_type_axlink] = ACTIONS(1531),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1531),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1531),
    [sym_source_type_ip_socket_address] = ACTIONS(1531),
    [sym_source_type_rs232] = ACTIONS(1531),
    [sym_source_type_internal] = ACTIONS(1531),
    [sym_source_type_mac_address] = ACTIONS(1531),
    [sym_source_type_ipv4_port] = ACTIONS(1531),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1531),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1531),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1531),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1531),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1531),
    [sym_true] = ACTIONS(1531),
    [sym_false] = ACTIONS(1531),
    [sym_file_read_only] = ACTIONS(1531),
    [sym_file_rw_new] = ACTIONS(1531),
    [sym_file_rw_append] = ACTIONS(1531),
    [sym_ip_tcp] = ACTIONS(1531),
    [sym_ip_udp] = ACTIONS(1531),
    [sym_ip_udp_2way] = ACTIONS(1531),
    [sym_xml_encode_types] = ACTIONS(1531),
    [sym_xml_encode_char_as_list] = ACTIONS(1531),
    [sym_xml_encode_le] = ACTIONS(1531),
    [sym_xml_decode_types] = ACTIONS(1531),
    [sym_xml_decode_no_preserve] = ACTIONS(1531),
    [sym_duet_dev_type_display_device] = ACTIONS(1531),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1531),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1531),
    [sym_duet_dev_type_io_device] = ACTIONS(1531),
    [sym_duet_dev_type_relay_device] = ACTIONS(1531),
    [sym_duet_dev_type_ups] = ACTIONS(1531),
    [sym_duet_dev_type_amplifier] = ACTIONS(1531),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1531),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1531),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1531),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1531),
    [sym_duet_dev_type_camera] = ACTIONS(1531),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1531),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1531),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1531),
    [sym_duet_dev_type_dss] = ACTIONS(1531),
    [sym_duet_dev_type_dvr] = ACTIONS(1531),
    [sym_duet_dev_type_disc_device] = ACTIONS(1531),
    [sym_duet_dev_type_document_camera] = ACTIONS(1531),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1531),
    [sym_duet_dev_type_hvac] = ACTIONS(1531),
    [sym_duet_dev_type_keypad] = ACTIONS(1531),
    [sym_duet_dev_type_light] = ACTIONS(1531),
    [sym_duet_dev_type_monitor] = ACTIONS(1531),
    [sym_duet_dev_type_motor] = ACTIONS(1531),
    [sym_duet_dev_type_multi_window] = ACTIONS(1531),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1531),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1531),
    [sym_duet_dev_type_receiver] = ACTIONS(1531),
    [sym_duet_dev_type_security_system] = ACTIONS(1531),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1531),
    [sym_duet_dev_type_settop_box] = ACTIONS(1531),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1531),
    [sym_duet_dev_type_switcher] = ACTIONS(1531),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1531),
    [sym_duet_dev_type_tv] = ACTIONS(1531),
    [sym_duet_dev_type_utility] = ACTIONS(1531),
    [sym_duet_dev_type_vcr] = ACTIONS(1531),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1531),
    [sym_duet_dev_type_video_processor] = ACTIONS(1531),
    [sym_duet_dev_type_video_projector] = ACTIONS(1531),
    [sym_duet_dev_type_video_wall] = ACTIONS(1531),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1531),
    [sym_duet_dev_type_weather] = ACTIONS(1531),
    [sym_duet_dev_not_polled] = ACTIONS(1531),
    [sym_duet_dev_polled] = ACTIONS(1531),
    [sym_clkmgr_mode_network] = ACTIONS(1531),
    [sym_clkmgr_mode_standalone] = ACTIONS(1531),
    [sym_normal_standby] = ACTIONS(1531),
    [sym_normal_wake] = ACTIONS(1531),
    [sym_ip_multicast_ttl_option] = ACTIONS(1531),
    [sym_ip_ttl_subnet] = ACTIONS(1531),
    [sym_ip_ttl_site] = ACTIONS(1531),
    [sym_ip_ttl_region] = ACTIONS(1531),
    [sym_ip_ttl_continent] = ACTIONS(1531),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1531),
    [sym_ip_nodelay_on] = ACTIONS(1531),
    [sym_ip_nodelay_off] = ACTIONS(1531),
    [sym_amx_error] = ACTIONS(1531),
    [sym_amx_warning] = ACTIONS(1531),
    [sym_amx_info] = ACTIONS(1531),
    [sym_amx_debug] = ACTIONS(1531),
    [sym_netlinx_login_success] = ACTIONS(1531),
    [sym_netlinx_login_fail] = ACTIONS(1531),
    [sym_netlinx_logout] = ACTIONS(1531),
    [sym_null_str] = ACTIONS(1531),
    [sym_smtp_address] = ACTIONS(1531),
    [sym_smtp_port_number] = ACTIONS(1531),
    [sym_smtp_username] = ACTIONS(1531),
    [sym_smtp_password] = ACTIONS(1531),
    [sym_smtp_require_tls] = ACTIONS(1531),
    [sym_smtp_from] = ACTIONS(1531),
    [sym_smtp_tls_true] = ACTIONS(1531),
    [sym_smtp_tls_false] = ACTIONS(1531),
    [sym_listview_on_row_select_event] = ACTIONS(1531),
    [sym_data_structure_unknown] = ACTIONS(1531),
    [sym_data_structure_datafeed] = ACTIONS(1531),
    [sym_data_structure_datarecordset] = ACTIONS(1531),
    [sym_data_structure_datarecord] = ACTIONS(1531),
    [sym_data_structure_datafield] = ACTIONS(1531),
    [sym_data_max_content_field_count] = ACTIONS(1531),
    [sym_data_max_metadata_field_count] = ACTIONS(1531),
    [sym_data_max_id_length] = ACTIONS(1531),
    [sym_data_max_name_length] = ACTIONS(1531),
    [sym_data_max_description_length] = ACTIONS(1531),
    [sym_data_max_type_length] = ACTIONS(1531),
    [sym_data_max_format_length] = ACTIONS(1531),
    [sym_data_max_source_length] = ACTIONS(1531),
    [sym_data_max_value_length] = ACTIONS(1531),
    [sym_data_max_label_length] = ACTIONS(1531),
    [sym_data_max_url_length] = ACTIONS(1531),
    [sym_max_last_login_info_length] = ACTIONS(1531),
    [sym_data_type_unknown] = ACTIONS(1531),
    [sym_data_type_string] = ACTIONS(1531),
    [sym_data_type_datetime] = ACTIONS(1531),
    [sym_data_type_date] = ACTIONS(1531),
    [sym_data_type_time] = ACTIONS(1531),
    [sym_data_type_image] = ACTIONS(1531),
    [sym_data_format_url] = ACTIONS(1531),
    [sym_data_format_phone] = ACTIONS(1531),
    [sym_data_format_email] = ACTIONS(1531),
    [sym_data_format_iso8601] = ACTIONS(1531),
    [sym_auditlog_priv] = ACTIONS(1531),
    [sym_configuration_priv] = ACTIONS(1531),
    [sym_deviceconfig_priv] = ACTIONS(1531),
    [sym_ftp_priv] = ACTIONS(1531),
    [sym_http_priv] = ACTIONS(1531),
    [sym_networkconfig_priv] = ACTIONS(1531),
    [sym_programport_priv] = ACTIONS(1531),
    [sym_remoteui_priv] = ACTIONS(1531),
    [sym_securitycontrol_priv] = ACTIONS(1531),
    [sym_softwaremanagement_priv] = ACTIONS(1531),
    [sym_terminal_priv] = ACTIONS(1531),
    [sym_tpadmin_priv] = ACTIONS(1531),
    [sym_user1_priv] = ACTIONS(1531),
    [sym_user2_priv] = ACTIONS(1531),
    [sym_user3_priv] = ACTIONS(1531),
    [sym_user4_priv] = ACTIONS(1531),
    [sym_usermanagement_priv] = ACTIONS(1531),
    [sym_valid_account] = ACTIONS(1531),
    [sym_err_name_invalid] = ACTIONS(1531),
    [sym_err_password_invalid] = ACTIONS(1531),
    [sym_err_invalid_account] = ACTIONS(1531),
    [sym_err_no_authorization] = ACTIONS(1531),
    [sym_err_invalid_parameter] = ACTIONS(1531),
    [sym_err_user_locked_out] = ACTIONS(1531),
    [sym_err_user_account_expired] = ACTIONS(1531),
    [sym_err_general_error] = ACTIONS(1531),
    [sym_err_authentication_error] = ACTIONS(1531),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1531),
    [sym_tls_validate_certificate] = ACTIONS(1531),
    [sym_tdata] = ACTIONS(1531),
    [sym_tchannel] = ACTIONS(1531),
    [sym_tlevel] = ACTIONS(1531),
    [sym_tbutton] = ACTIONS(1531),
    [sym_ttimeline] = ACTIONS(1531),
    [sym_tcustom] = ACTIONS(1531),
    [sym_url_struct] = ACTIONS(1531),
    [sym_dns_struct] = ACTIONS(1531),
    [sym_ip_address_struct] = ACTIONS(1531),
    [sym_dev_info_struct] = ACTIONS(1531),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1531),
    [sym_data_feed] = ACTIONS(1531),
    [sym_data_field] = ACTIONS(1531),
    [sym_data_record] = ACTIONS(1531),
    [sym_wc_data_feed] = ACTIONS(1531),
    [sym_wc_data_field] = ACTIONS(1531),
    [sym_wc_data_record] = ACTIONS(1531),
    [sym_last_login_info] = ACTIONS(1531),
    [sym____reserved___] = ACTIONS(1531),
    [sym_date] = ACTIONS(1531),
    [sym_day] = ACTIONS(1531),
    [sym_time] = ACTIONS(1531),
    [sym_ldate] = ACTIONS(1531),
    [sym_push_device] = ACTIONS(1531),
    [sym_push_channel] = ACTIONS(1531),
    [sym_push_devchan] = ACTIONS(1531),
    [sym_release_device] = ACTIONS(1531),
    [sym_release_channel] = ACTIONS(1531),
    [sym_release_devchan] = ACTIONS(1531),
    [sym_master_slot] = ACTIONS(1531),
    [sym_get_pulse_time] = ACTIONS(1531),
    [sym_get_timer] = ACTIONS(1531),
    [sym_master_sn] = ACTIONS(1531),
    [sym_system_number] = ACTIONS(1531),
    [sym_button] = ACTIONS(1531),
    [sym_dv_channel] = ACTIONS(1531),
    [sym_data] = ACTIONS(1531),
    [sym_level] = ACTIONS(1531),
    [sym_timeline] = ACTIONS(1531),
    [sym_channel] = ACTIONS(1531),
    [sym_custom] = ACTIONS(1531),
    [sym_atoi] = ACTIONS(1531),
    [sym_atol] = ACTIONS(1531),
    [sym_atof] = ACTIONS(1531),
    [sym_itoa] = ACTIONS(1531),
    [sym_itohex] = ACTIONS(1531),
    [sym_hextoi] = ACTIONS(1531),
    [sym_ftoa] = ACTIONS(1531),
    [sym_find_string] = ACTIONS(1531),
    [sym_left_string] = ACTIONS(1531),
    [sym_right_string] = ACTIONS(1531),
    [sym_mid_string] = ACTIONS(1531),
    [sym_remove_string] = ACTIONS(1531),
    [sym_get_buffer_string] = ACTIONS(1531),
    [sym_get_buffer_char] = ACTIONS(1531),
    [sym_get_multi_buffer_string] = ACTIONS(1531),
    [sym_length_string] = ACTIONS(1531),
    [sym_max_length_string] = ACTIONS(1531),
    [sym_set_length_string] = ACTIONS(1531),
    [sym_compare_string] = ACTIONS(1531),
    [sym_upper_string] = ACTIONS(1531),
    [sym_lower_string] = ACTIONS(1531),
    [sym_get_last] = ACTIONS(1531),
    [sym_date_to_day] = ACTIONS(1531),
    [sym_date_to_month] = ACTIONS(1531),
    [sym_date_to_year] = ACTIONS(1531),
    [sym_time_to_hour] = ACTIONS(1531),
    [sym_time_to_minute] = ACTIONS(1531),
    [sym_time_to_second] = ACTIONS(1531),
    [sym_day_of_week] = ACTIONS(1531),
    [sym_set_pulse_time] = ACTIONS(1531),
    [sym_set_timer] = ACTIONS(1531),
    [sym_file_open] = ACTIONS(1531),
    [sym_file_close] = ACTIONS(1531),
    [sym_file_read] = ACTIONS(1531),
    [sym_file_read_line] = ACTIONS(1531),
    [sym_file_write] = ACTIONS(1531),
    [sym_file_write_line] = ACTIONS(1531),
    [sym_file_copy] = ACTIONS(1531),
    [sym_file_delete] = ACTIONS(1531),
    [sym_file_rename] = ACTIONS(1531),
    [sym_file_dir] = ACTIONS(1531),
    [sym_file_getdir] = ACTIONS(1531),
    [sym_file_setdir] = ACTIONS(1531),
    [sym_file_create_dir] = ACTIONS(1531),
    [sym_file_remove_dir] = ACTIONS(1531),
    [sym_file_seek] = ACTIONS(1531),
    [sym_length_array] = ACTIONS(1531),
    [sym_max_length_array] = ACTIONS(1531),
    [sym_set_length_array] = ACTIONS(1531),
    [sym_ip_client_open] = ACTIONS(1531),
    [sym_ip_client_close] = ACTIONS(1531),
    [sym_ip_server_open] = ACTIONS(1531),
    [sym_ip_server_close] = ACTIONS(1531),
    [sym_ip_bound_client_open] = ACTIONS(1531),
    [sym_ip_mc_server_open] = ACTIONS(1531),
    [sym_ip_set_option] = ACTIONS(1531),
    [sym_ssh_client_open] = ACTIONS(1531),
    [sym_ssh_client_close] = ACTIONS(1531),
    [sym_tls_client_open] = ACTIONS(1531),
    [sym_tls_client_close] = ACTIONS(1531),
    [sym_do_push] = ACTIONS(1531),
    [sym_do_push_timed] = ACTIONS(1531),
    [sym_do_release] = ACTIONS(1531),
    [sym_device_id] = ACTIONS(1531),
    [sym_device_info] = ACTIONS(1531),
    [sym_device_id_string] = ACTIONS(1531),
    [sym_device_standby] = ACTIONS(1531),
    [sym_device_wake] = ACTIONS(1531),
    [sym_timeline_create] = ACTIONS(1531),
    [sym_timeline_kill] = ACTIONS(1531),
    [sym_timeline_reload] = ACTIONS(1531),
    [sym_timeline_pause] = ACTIONS(1531),
    [sym_timeline_restart] = ACTIONS(1531),
    [sym_timeline_set] = ACTIONS(1531),
    [sym_timeline_get] = ACTIONS(1531),
    [sym_timeline_active] = ACTIONS(1531),
    [sym_timeline_dynamic_id] = ACTIONS(1531),
    [sym_variable_to_string] = ACTIONS(1531),
    [sym_string_to_variable] = ACTIONS(1531),
    [sym_length_variable_to_string] = ACTIONS(1531),
    [sym_type_cast] = ACTIONS(1531),
    [sym_variable_to_xml] = ACTIONS(1531),
    [sym_xml_to_variable] = ACTIONS(1531),
    [sym_length_variable_to_xml] = ACTIONS(1531),
    [sym_redirect_string] = ACTIONS(1531),
    [sym_random_number] = ACTIONS(1531),
    [sym_get_unique_id] = ACTIONS(1531),
    [sym_get_system_number] = ACTIONS(1531),
    [sym_set_system_number] = ACTIONS(1531),
    [sym_get_serial_number] = ACTIONS(1531),
    [sym_get_master_build] = ACTIONS(1531),
    [sym_set_validation_code] = ACTIONS(1531),
    [sym_reboot] = ACTIONS(1531),
    [sym_load_duet_module] = ACTIONS(1531),
    [sym_unload_duet_module] = ACTIONS(1531),
    [sym_duet_mem_size_set] = ACTIONS(1531),
    [sym_duet_mem_size_get] = ACTIONS(1531),
    [sym_dynamic_application_device] = ACTIONS(1531),
    [sym_dynamic_polled_port] = ACTIONS(1531),
    [sym_static_port_binding] = ACTIONS(1531),
    [sym_static_ip_binding] = ACTIONS(1531),
    [sym_get_dns_list] = ACTIONS(1531),
    [sym_set_dns_list] = ACTIONS(1531),
    [sym_get_ip_address] = ACTIONS(1531),
    [sym_set_ip_address] = ACTIONS(1531),
    [sym_get_url_list] = ACTIONS(1531),
    [sym_add_url_entry] = ACTIONS(1531),
    [sym_delete_url_entry] = ACTIONS(1531),
    [sym_abs_value] = ACTIONS(1531),
    [sym_max_value] = ACTIONS(1531),
    [sym_min_value] = ACTIONS(1531),
    [sym_exp_value] = ACTIONS(1531),
    [sym_log_value] = ACTIONS(1531),
    [sym_log10_value] = ACTIONS(1531),
    [sym_power_value] = ACTIONS(1531),
    [sym_sqrt_value] = ACTIONS(1531),
    [sym_raw_be] = ACTIONS(1531),
    [sym_raw_le] = ACTIONS(1531),
    [sym_internal_queue_size_set] = ACTIONS(1531),
    [sym_internal_queue_size_get] = ACTIONS(1531),
    [sym_internal_threshold_set] = ACTIONS(1531),
    [sym_internal_threshold_get] = ACTIONS(1531),
    [sym_rebuilt_event] = ACTIONS(1531),
    [sym_do_custom_event] = ACTIONS(1531),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1531),
    [sym_clkmgr_set_clk_source] = ACTIONS(1531),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1531),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1531),
    [sym_clkmgr_get_timezone] = ACTIONS(1531),
    [sym_clkmgr_set_timezone] = ACTIONS(1531),
    [sym_clkmgr_get_resync_period] = ACTIONS(1531),
    [sym_clkmgr_set_resync_period] = ACTIONS(1531),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1531),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1531),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1531),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1531),
    [sym_clkmgr_get_timeservers] = ACTIONS(1531),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1531),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1531),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1531),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1531),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1531),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1531),
    [sym_set_log_level] = ACTIONS(1531),
    [sym_get_log_level] = ACTIONS(1531),
    [sym_amx_log] = ACTIONS(1531),
    [sym_smtp_server_config_set] = ACTIONS(1531),
    [sym_smtp_server_config_get] = ACTIONS(1531),
    [sym_smtp_send] = ACTIONS(1531),
    [sym_validate_netlinx_account] = ACTIONS(1531),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1531),
    [sym_authenticate_certificate] = ACTIONS(1531),
    [sym_audit_netlinx_session_event] = ACTIONS(1531),
    [sym_audit_netlinx_generic_event] = ACTIONS(1531),
    [sym_get_available_flash_disk_space] = ACTIONS(1531),
    [sym_get_max_flash_disk_space] = ACTIONS(1531),
    [sym_data_create_feed] = ACTIONS(1531),
    [sym_data_delete_feed] = ACTIONS(1531),
    [sym_data_publish_feed] = ACTIONS(1531),
    [sym_data_get_published_feed] = ACTIONS(1531),
    [sym_data_add_record] = ACTIONS(1531),
    [sym_data_get_event_record] = ACTIONS(1531),
    [sym__wc_data_create_feed] = ACTIONS(1531),
    [sym__wc_data_add_record] = ACTIONS(1531),
    [sym__wc_data_get_event_record] = ACTIONS(1531),
    [sym_format] = ACTIONS(1531),
    [sym_astro_clock] = ACTIONS(1531),
    [sym_set_outdoor_temperature] = ACTIONS(1531),
    [sym_set_virtual_level_count] = ACTIONS(1531),
    [sym_set_virtual_channel_count] = ACTIONS(1531),
    [sym_set_virtual_port_count] = ACTIONS(1531),
    [sym__wc_encode_int] = ACTIONS(1531),
    [sym__wc_decode_int] = ACTIONS(1531),
    [sym__wc_int] = ACTIONS(1531),
    [sym__wc_to_ch_int] = ACTIONS(1531),
    [sym__ch_to_wc_int] = ACTIONS(1531),
    [sym__wc_find_string_int] = ACTIONS(1531),
    [sym__wc_left_string_int] = ACTIONS(1531),
    [sym__wc_lower_string_int] = ACTIONS(1531),
    [sym__wc_mid_string_int] = ACTIONS(1531),
    [sym__wc_remove_string_int] = ACTIONS(1531),
    [sym__wc_right_string_int] = ACTIONS(1531),
    [sym__wc_upper_string_int] = ACTIONS(1531),
    [sym__wc_compare_string_int] = ACTIONS(1531),
    [sym__wc_get_buffer_char_int] = ACTIONS(1531),
    [sym__wc_get_buffer_string_int] = ACTIONS(1531),
    [sym__wc_concat_string_int] = ACTIONS(1531),
    [sym__wc_file_open_int] = ACTIONS(1531),
    [sym__wc_file_close_int] = ACTIONS(1531),
    [sym__wc_file_read_int] = ACTIONS(1531),
    [sym__wc_file_read_line_int] = ACTIONS(1531),
    [sym__wc_file_write_int] = ACTIONS(1531),
    [sym__wc_file_write_line_int] = ACTIONS(1531),
    [anon_sym_EQ] = ACTIONS(1531),
    [aux_sym_preproc_include_token1] = ACTIONS(1531),
    [aux_sym_preproc_define_token1] = ACTIONS(1533),
    [aux_sym_preproc_warn_token1] = ACTIONS(1533),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1533),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1533),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1533),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1533),
    [aux_sym_preproc_else_token1] = ACTIONS(1533),
    [anon_sym_LBRACK] = ACTIONS(1533),
    [anon_sym_LBRACE] = ACTIONS(1533),
    [anon_sym_COLON] = ACTIONS(1533),
    [anon_sym_LPAREN] = ACTIONS(1531),
    [anon_sym_SEMI] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1531),
    [anon_sym_TILDE] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(1533),
    [anon_sym_SLASH] = ACTIONS(1531),
    [anon_sym_PERCENT] = ACTIONS(1533),
    [anon_sym_PIPE_PIPE] = ACTIONS(1533),
    [anon_sym_AMP_AMP] = ACTIONS(1533),
    [anon_sym_PIPE] = ACTIONS(1531),
    [anon_sym_CARET] = ACTIONS(1533),
    [anon_sym_AMP] = ACTIONS(1531),
    [anon_sym_EQ_EQ] = ACTIONS(1533),
    [anon_sym_BANG_EQ] = ACTIONS(1533),
    [anon_sym_LT_GT] = ACTIONS(1533),
    [anon_sym_GT] = ACTIONS(1531),
    [anon_sym_GT_EQ] = ACTIONS(1533),
    [anon_sym_LT_EQ] = ACTIONS(1533),
    [anon_sym_LT] = ACTIONS(1531),
    [anon_sym_LT_LT] = ACTIONS(1533),
    [anon_sym_GT_GT] = ACTIONS(1533),
    [anon_sym_DASH_DASH] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1533),
    [anon_sym_DOT] = ACTIONS(1531),
    [anon_sym_DQUOTE] = ACTIONS(1533),
    [anon_sym_SQUOTE] = ACTIONS(1533),
    [sym_number_literal] = ACTIONS(1533),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1533),
  },
  [STATE(98)] = {
    [sym_identifier] = ACTIONS(1535),
    [sym_program_name_keyword] = ACTIONS(1535),
    [sym_module_name_keyword] = ACTIONS(1535),
    [sym_define_device_keyword] = ACTIONS(1535),
    [sym_define_combine_keyword] = ACTIONS(1535),
    [sym_define_constant_keyword] = ACTIONS(1535),
    [sym_define_type_keyword] = ACTIONS(1535),
    [sym_define_variable_keyword] = ACTIONS(1535),
    [sym_define_system_variable_keyword] = ACTIONS(1535),
    [sym_define_connect_level_keyword] = ACTIONS(1535),
    [sym_define_latching_keyword] = ACTIONS(1535),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1535),
    [sym_define_toggling_keyword] = ACTIONS(1535),
    [sym_define_call_keyword] = ACTIONS(1535),
    [sym_define_function_keyword] = ACTIONS(1535),
    [sym_define_library_function_keyword] = ACTIONS(1535),
    [sym_define_start_keyword] = ACTIONS(1535),
    [sym_define_event_keyword] = ACTIONS(1535),
    [sym_define_program_keyword] = ACTIONS(1535),
    [sym_define_module_keyword] = ACTIONS(1535),
    [sym_char_keyword] = ACTIONS(1535),
    [sym_widechar_keyword] = ACTIONS(1535),
    [sym_integer_keyword] = ACTIONS(1535),
    [sym_sinteger_keyword] = ACTIONS(1535),
    [sym_long_keyword] = ACTIONS(1535),
    [sym_slong_keyword] = ACTIONS(1535),
    [sym_float_keyword] = ACTIONS(1535),
    [sym_double_keyword] = ACTIONS(1535),
    [sym_variant_keyword] = ACTIONS(1535),
    [sym_variantarray_keyword] = ACTIONS(1535),
    [sym_local_var_keyword] = ACTIONS(1535),
    [sym_stack_var_keyword] = ACTIONS(1535),
    [sym_constant_keyword] = ACTIONS(1535),
    [sym_volatile_keyword] = ACTIONS(1535),
    [sym_non_volatile_keyword] = ACTIONS(1535),
    [sym_persistent_keyword] = ACTIONS(1535),
    [sym_dev_keyword] = ACTIONS(1535),
    [sym_devlev_keyword] = ACTIONS(1535),
    [sym_devchan_keyword] = ACTIONS(1535),
    [sym_if_keyword] = ACTIONS(1535),
    [sym_else_keyword] = ACTIONS(1535),
    [sym_switch_keyword] = ACTIONS(1535),
    [sym_case_keyword] = ACTIONS(1535),
    [sym_default_keyword] = ACTIONS(1535),
    [sym_while_keyword] = ACTIONS(1535),
    [sym_for_keyword] = ACTIONS(1535),
    [sym_break_keyword] = ACTIONS(1535),
    [sym_continue_keyword] = ACTIONS(1535),
    [sym_return_keyword] = ACTIONS(1535),
    [sym_select_keyword] = ACTIONS(1535),
    [sym_active_keyword] = ACTIONS(1535),
    [sym_struct_keyword] = ACTIONS(1535),
    [sym_structure_keyword] = ACTIONS(1535),
    [sym_button_event_keyword] = ACTIONS(1535),
    [sym_channel_event_keyword] = ACTIONS(1535),
    [sym_level_event_keyword] = ACTIONS(1535),
    [sym_data_event_keyword] = ACTIONS(1535),
    [sym_timeline_event_keyword] = ACTIONS(1535),
    [sym_custom_event_keyword] = ACTIONS(1535),
    [sym_push_keyword] = ACTIONS(1535),
    [sym_release_keyword] = ACTIONS(1535),
    [aux_sym_on_keyword_token1] = ACTIONS(1535),
    [aux_sym_off_keyword_token1] = ACTIONS(1535),
    [sym_send_level_keyword] = ACTIONS(1535),
    [sym_send_string_keyword] = ACTIONS(1535),
    [sym_send_command_keyword] = ACTIONS(1535),
    [sym_clear_buffer_keyword] = ACTIONS(1535),
    [sym_create_buffer_keyword] = ACTIONS(1535),
    [sym_create_multi_buffer_keyword] = ACTIONS(1535),
    [sym_call_keyword] = ACTIONS(1535),
    [sym_system_call_keyword] = ACTIONS(1535),
    [sym_devchan_to_keyword] = ACTIONS(1535),
    [sym_devchan_min_to_keyword] = ACTIONS(1535),
    [sym_devchan_total_off_keyword] = ACTIONS(1535),
    [sym_devchan_pulse_keyword] = ACTIONS(1535),
    [sym_wait_keyword] = ACTIONS(1535),
    [sym_cancel_wait_keyword] = ACTIONS(1535),
    [sym_wait_until_keyword] = ACTIONS(1535),
    [sym_cancel_wait_until_keyword] = ACTIONS(1535),
    [sym_cancel_all_wait_keyword] = ACTIONS(1535),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1535),
    [sym_and] = ACTIONS(1535),
    [sym_or] = ACTIONS(1535),
    [sym_not] = ACTIONS(1535),
    [sym_xor] = ACTIONS(1535),
    [sym_band] = ACTIONS(1535),
    [sym_bor] = ACTIONS(1535),
    [sym_bxor] = ACTIONS(1535),
    [sym_bnot] = ACTIONS(1535),
    [sym_lshift] = ACTIONS(1535),
    [sym_rshift] = ACTIONS(1535),
    [sym___file__] = ACTIONS(1535),
    [sym___date__] = ACTIONS(1535),
    [sym___ldate__] = ACTIONS(1535),
    [sym___time__] = ACTIONS(1535),
    [sym___line__] = ACTIONS(1535),
    [sym___name__] = ACTIONS(1535),
    [sym___netlinx__] = ACTIONS(1535),
    [sym_netlinx_axi_version] = ACTIONS(1535),
    [sym_url_flg_tcp] = ACTIONS(1535),
    [sym_url_flg_accinfopresent] = ACTIONS(1535),
    [sym_url_flg_temp] = ACTIONS(1535),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1535),
    [sym_url_flg_stat_mask] = ACTIONS(1535),
    [sym_url_flg_stat_lookup] = ACTIONS(1535),
    [sym_url_flg_stat_connecting] = ACTIONS(1535),
    [sym_url_flg_stat_waiting] = ACTIONS(1535),
    [sym_url_flg_stat_connected] = ACTIONS(1535),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1535),
    [sym_first_virtual_device] = ACTIONS(1535),
    [sym_first_local_port] = ACTIONS(1535),
    [sym_dynamic_virtual_device] = ACTIONS(1535),
    [sym_timeline_once] = ACTIONS(1535),
    [sym_timeline_repeat] = ACTIONS(1535),
    [sym_timeline_absolute] = ACTIONS(1535),
    [sym_timeline_relative] = ACTIONS(1535),
    [sym_do_push_timed_infinite] = ACTIONS(1535),
    [sym_source_type_no_address] = ACTIONS(1535),
    [sym_source_type_neuron_id] = ACTIONS(1535),
    [sym_source_type_ip_address] = ACTIONS(1535),
    [sym_source_type_axlink] = ACTIONS(1535),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1535),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1535),
    [sym_source_type_ip_socket_address] = ACTIONS(1535),
    [sym_source_type_rs232] = ACTIONS(1535),
    [sym_source_type_internal] = ACTIONS(1535),
    [sym_source_type_mac_address] = ACTIONS(1535),
    [sym_source_type_ipv4_port] = ACTIONS(1535),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1535),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1535),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1535),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1535),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1535),
    [sym_true] = ACTIONS(1535),
    [sym_false] = ACTIONS(1535),
    [sym_file_read_only] = ACTIONS(1535),
    [sym_file_rw_new] = ACTIONS(1535),
    [sym_file_rw_append] = ACTIONS(1535),
    [sym_ip_tcp] = ACTIONS(1535),
    [sym_ip_udp] = ACTIONS(1535),
    [sym_ip_udp_2way] = ACTIONS(1535),
    [sym_xml_encode_types] = ACTIONS(1535),
    [sym_xml_encode_char_as_list] = ACTIONS(1535),
    [sym_xml_encode_le] = ACTIONS(1535),
    [sym_xml_decode_types] = ACTIONS(1535),
    [sym_xml_decode_no_preserve] = ACTIONS(1535),
    [sym_duet_dev_type_display_device] = ACTIONS(1535),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1535),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1535),
    [sym_duet_dev_type_io_device] = ACTIONS(1535),
    [sym_duet_dev_type_relay_device] = ACTIONS(1535),
    [sym_duet_dev_type_ups] = ACTIONS(1535),
    [sym_duet_dev_type_amplifier] = ACTIONS(1535),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1535),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1535),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1535),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1535),
    [sym_duet_dev_type_camera] = ACTIONS(1535),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1535),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1535),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1535),
    [sym_duet_dev_type_dss] = ACTIONS(1535),
    [sym_duet_dev_type_dvr] = ACTIONS(1535),
    [sym_duet_dev_type_disc_device] = ACTIONS(1535),
    [sym_duet_dev_type_document_camera] = ACTIONS(1535),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1535),
    [sym_duet_dev_type_hvac] = ACTIONS(1535),
    [sym_duet_dev_type_keypad] = ACTIONS(1535),
    [sym_duet_dev_type_light] = ACTIONS(1535),
    [sym_duet_dev_type_monitor] = ACTIONS(1535),
    [sym_duet_dev_type_motor] = ACTIONS(1535),
    [sym_duet_dev_type_multi_window] = ACTIONS(1535),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1535),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1535),
    [sym_duet_dev_type_receiver] = ACTIONS(1535),
    [sym_duet_dev_type_security_system] = ACTIONS(1535),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1535),
    [sym_duet_dev_type_settop_box] = ACTIONS(1535),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1535),
    [sym_duet_dev_type_switcher] = ACTIONS(1535),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1535),
    [sym_duet_dev_type_tv] = ACTIONS(1535),
    [sym_duet_dev_type_utility] = ACTIONS(1535),
    [sym_duet_dev_type_vcr] = ACTIONS(1535),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1535),
    [sym_duet_dev_type_video_processor] = ACTIONS(1535),
    [sym_duet_dev_type_video_projector] = ACTIONS(1535),
    [sym_duet_dev_type_video_wall] = ACTIONS(1535),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1535),
    [sym_duet_dev_type_weather] = ACTIONS(1535),
    [sym_duet_dev_not_polled] = ACTIONS(1535),
    [sym_duet_dev_polled] = ACTIONS(1535),
    [sym_clkmgr_mode_network] = ACTIONS(1535),
    [sym_clkmgr_mode_standalone] = ACTIONS(1535),
    [sym_normal_standby] = ACTIONS(1535),
    [sym_normal_wake] = ACTIONS(1535),
    [sym_ip_multicast_ttl_option] = ACTIONS(1535),
    [sym_ip_ttl_subnet] = ACTIONS(1535),
    [sym_ip_ttl_site] = ACTIONS(1535),
    [sym_ip_ttl_region] = ACTIONS(1535),
    [sym_ip_ttl_continent] = ACTIONS(1535),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1535),
    [sym_ip_nodelay_on] = ACTIONS(1535),
    [sym_ip_nodelay_off] = ACTIONS(1535),
    [sym_amx_error] = ACTIONS(1535),
    [sym_amx_warning] = ACTIONS(1535),
    [sym_amx_info] = ACTIONS(1535),
    [sym_amx_debug] = ACTIONS(1535),
    [sym_netlinx_login_success] = ACTIONS(1535),
    [sym_netlinx_login_fail] = ACTIONS(1535),
    [sym_netlinx_logout] = ACTIONS(1535),
    [sym_null_str] = ACTIONS(1535),
    [sym_smtp_address] = ACTIONS(1535),
    [sym_smtp_port_number] = ACTIONS(1535),
    [sym_smtp_username] = ACTIONS(1535),
    [sym_smtp_password] = ACTIONS(1535),
    [sym_smtp_require_tls] = ACTIONS(1535),
    [sym_smtp_from] = ACTIONS(1535),
    [sym_smtp_tls_true] = ACTIONS(1535),
    [sym_smtp_tls_false] = ACTIONS(1535),
    [sym_listview_on_row_select_event] = ACTIONS(1535),
    [sym_data_structure_unknown] = ACTIONS(1535),
    [sym_data_structure_datafeed] = ACTIONS(1535),
    [sym_data_structure_datarecordset] = ACTIONS(1535),
    [sym_data_structure_datarecord] = ACTIONS(1535),
    [sym_data_structure_datafield] = ACTIONS(1535),
    [sym_data_max_content_field_count] = ACTIONS(1535),
    [sym_data_max_metadata_field_count] = ACTIONS(1535),
    [sym_data_max_id_length] = ACTIONS(1535),
    [sym_data_max_name_length] = ACTIONS(1535),
    [sym_data_max_description_length] = ACTIONS(1535),
    [sym_data_max_type_length] = ACTIONS(1535),
    [sym_data_max_format_length] = ACTIONS(1535),
    [sym_data_max_source_length] = ACTIONS(1535),
    [sym_data_max_value_length] = ACTIONS(1535),
    [sym_data_max_label_length] = ACTIONS(1535),
    [sym_data_max_url_length] = ACTIONS(1535),
    [sym_max_last_login_info_length] = ACTIONS(1535),
    [sym_data_type_unknown] = ACTIONS(1535),
    [sym_data_type_string] = ACTIONS(1535),
    [sym_data_type_datetime] = ACTIONS(1535),
    [sym_data_type_date] = ACTIONS(1535),
    [sym_data_type_time] = ACTIONS(1535),
    [sym_data_type_image] = ACTIONS(1535),
    [sym_data_format_url] = ACTIONS(1535),
    [sym_data_format_phone] = ACTIONS(1535),
    [sym_data_format_email] = ACTIONS(1535),
    [sym_data_format_iso8601] = ACTIONS(1535),
    [sym_auditlog_priv] = ACTIONS(1535),
    [sym_configuration_priv] = ACTIONS(1535),
    [sym_deviceconfig_priv] = ACTIONS(1535),
    [sym_ftp_priv] = ACTIONS(1535),
    [sym_http_priv] = ACTIONS(1535),
    [sym_networkconfig_priv] = ACTIONS(1535),
    [sym_programport_priv] = ACTIONS(1535),
    [sym_remoteui_priv] = ACTIONS(1535),
    [sym_securitycontrol_priv] = ACTIONS(1535),
    [sym_softwaremanagement_priv] = ACTIONS(1535),
    [sym_terminal_priv] = ACTIONS(1535),
    [sym_tpadmin_priv] = ACTIONS(1535),
    [sym_user1_priv] = ACTIONS(1535),
    [sym_user2_priv] = ACTIONS(1535),
    [sym_user3_priv] = ACTIONS(1535),
    [sym_user4_priv] = ACTIONS(1535),
    [sym_usermanagement_priv] = ACTIONS(1535),
    [sym_valid_account] = ACTIONS(1535),
    [sym_err_name_invalid] = ACTIONS(1535),
    [sym_err_password_invalid] = ACTIONS(1535),
    [sym_err_invalid_account] = ACTIONS(1535),
    [sym_err_no_authorization] = ACTIONS(1535),
    [sym_err_invalid_parameter] = ACTIONS(1535),
    [sym_err_user_locked_out] = ACTIONS(1535),
    [sym_err_user_account_expired] = ACTIONS(1535),
    [sym_err_general_error] = ACTIONS(1535),
    [sym_err_authentication_error] = ACTIONS(1535),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1535),
    [sym_tls_validate_certificate] = ACTIONS(1535),
    [sym_tdata] = ACTIONS(1535),
    [sym_tchannel] = ACTIONS(1535),
    [sym_tlevel] = ACTIONS(1535),
    [sym_tbutton] = ACTIONS(1535),
    [sym_ttimeline] = ACTIONS(1535),
    [sym_tcustom] = ACTIONS(1535),
    [sym_url_struct] = ACTIONS(1535),
    [sym_dns_struct] = ACTIONS(1535),
    [sym_ip_address_struct] = ACTIONS(1535),
    [sym_dev_info_struct] = ACTIONS(1535),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1535),
    [sym_data_feed] = ACTIONS(1535),
    [sym_data_field] = ACTIONS(1535),
    [sym_data_record] = ACTIONS(1535),
    [sym_wc_data_feed] = ACTIONS(1535),
    [sym_wc_data_field] = ACTIONS(1535),
    [sym_wc_data_record] = ACTIONS(1535),
    [sym_last_login_info] = ACTIONS(1535),
    [sym____reserved___] = ACTIONS(1535),
    [sym_date] = ACTIONS(1535),
    [sym_day] = ACTIONS(1535),
    [sym_time] = ACTIONS(1535),
    [sym_ldate] = ACTIONS(1535),
    [sym_push_device] = ACTIONS(1535),
    [sym_push_channel] = ACTIONS(1535),
    [sym_push_devchan] = ACTIONS(1535),
    [sym_release_device] = ACTIONS(1535),
    [sym_release_channel] = ACTIONS(1535),
    [sym_release_devchan] = ACTIONS(1535),
    [sym_master_slot] = ACTIONS(1535),
    [sym_get_pulse_time] = ACTIONS(1535),
    [sym_get_timer] = ACTIONS(1535),
    [sym_master_sn] = ACTIONS(1535),
    [sym_system_number] = ACTIONS(1535),
    [sym_button] = ACTIONS(1535),
    [sym_dv_channel] = ACTIONS(1535),
    [sym_data] = ACTIONS(1535),
    [sym_level] = ACTIONS(1535),
    [sym_timeline] = ACTIONS(1535),
    [sym_channel] = ACTIONS(1535),
    [sym_custom] = ACTIONS(1535),
    [sym_atoi] = ACTIONS(1535),
    [sym_atol] = ACTIONS(1535),
    [sym_atof] = ACTIONS(1535),
    [sym_itoa] = ACTIONS(1535),
    [sym_itohex] = ACTIONS(1535),
    [sym_hextoi] = ACTIONS(1535),
    [sym_ftoa] = ACTIONS(1535),
    [sym_find_string] = ACTIONS(1535),
    [sym_left_string] = ACTIONS(1535),
    [sym_right_string] = ACTIONS(1535),
    [sym_mid_string] = ACTIONS(1535),
    [sym_remove_string] = ACTIONS(1535),
    [sym_get_buffer_string] = ACTIONS(1535),
    [sym_get_buffer_char] = ACTIONS(1535),
    [sym_get_multi_buffer_string] = ACTIONS(1535),
    [sym_length_string] = ACTIONS(1535),
    [sym_max_length_string] = ACTIONS(1535),
    [sym_set_length_string] = ACTIONS(1535),
    [sym_compare_string] = ACTIONS(1535),
    [sym_upper_string] = ACTIONS(1535),
    [sym_lower_string] = ACTIONS(1535),
    [sym_get_last] = ACTIONS(1535),
    [sym_date_to_day] = ACTIONS(1535),
    [sym_date_to_month] = ACTIONS(1535),
    [sym_date_to_year] = ACTIONS(1535),
    [sym_time_to_hour] = ACTIONS(1535),
    [sym_time_to_minute] = ACTIONS(1535),
    [sym_time_to_second] = ACTIONS(1535),
    [sym_day_of_week] = ACTIONS(1535),
    [sym_set_pulse_time] = ACTIONS(1535),
    [sym_set_timer] = ACTIONS(1535),
    [sym_file_open] = ACTIONS(1535),
    [sym_file_close] = ACTIONS(1535),
    [sym_file_read] = ACTIONS(1535),
    [sym_file_read_line] = ACTIONS(1535),
    [sym_file_write] = ACTIONS(1535),
    [sym_file_write_line] = ACTIONS(1535),
    [sym_file_copy] = ACTIONS(1535),
    [sym_file_delete] = ACTIONS(1535),
    [sym_file_rename] = ACTIONS(1535),
    [sym_file_dir] = ACTIONS(1535),
    [sym_file_getdir] = ACTIONS(1535),
    [sym_file_setdir] = ACTIONS(1535),
    [sym_file_create_dir] = ACTIONS(1535),
    [sym_file_remove_dir] = ACTIONS(1535),
    [sym_file_seek] = ACTIONS(1535),
    [sym_length_array] = ACTIONS(1535),
    [sym_max_length_array] = ACTIONS(1535),
    [sym_set_length_array] = ACTIONS(1535),
    [sym_ip_client_open] = ACTIONS(1535),
    [sym_ip_client_close] = ACTIONS(1535),
    [sym_ip_server_open] = ACTIONS(1535),
    [sym_ip_server_close] = ACTIONS(1535),
    [sym_ip_bound_client_open] = ACTIONS(1535),
    [sym_ip_mc_server_open] = ACTIONS(1535),
    [sym_ip_set_option] = ACTIONS(1535),
    [sym_ssh_client_open] = ACTIONS(1535),
    [sym_ssh_client_close] = ACTIONS(1535),
    [sym_tls_client_open] = ACTIONS(1535),
    [sym_tls_client_close] = ACTIONS(1535),
    [sym_do_push] = ACTIONS(1535),
    [sym_do_push_timed] = ACTIONS(1535),
    [sym_do_release] = ACTIONS(1535),
    [sym_device_id] = ACTIONS(1535),
    [sym_device_info] = ACTIONS(1535),
    [sym_device_id_string] = ACTIONS(1535),
    [sym_device_standby] = ACTIONS(1535),
    [sym_device_wake] = ACTIONS(1535),
    [sym_timeline_create] = ACTIONS(1535),
    [sym_timeline_kill] = ACTIONS(1535),
    [sym_timeline_reload] = ACTIONS(1535),
    [sym_timeline_pause] = ACTIONS(1535),
    [sym_timeline_restart] = ACTIONS(1535),
    [sym_timeline_set] = ACTIONS(1535),
    [sym_timeline_get] = ACTIONS(1535),
    [sym_timeline_active] = ACTIONS(1535),
    [sym_timeline_dynamic_id] = ACTIONS(1535),
    [sym_variable_to_string] = ACTIONS(1535),
    [sym_string_to_variable] = ACTIONS(1535),
    [sym_length_variable_to_string] = ACTIONS(1535),
    [sym_type_cast] = ACTIONS(1535),
    [sym_variable_to_xml] = ACTIONS(1535),
    [sym_xml_to_variable] = ACTIONS(1535),
    [sym_length_variable_to_xml] = ACTIONS(1535),
    [sym_redirect_string] = ACTIONS(1535),
    [sym_random_number] = ACTIONS(1535),
    [sym_get_unique_id] = ACTIONS(1535),
    [sym_get_system_number] = ACTIONS(1535),
    [sym_set_system_number] = ACTIONS(1535),
    [sym_get_serial_number] = ACTIONS(1535),
    [sym_get_master_build] = ACTIONS(1535),
    [sym_set_validation_code] = ACTIONS(1535),
    [sym_reboot] = ACTIONS(1535),
    [sym_load_duet_module] = ACTIONS(1535),
    [sym_unload_duet_module] = ACTIONS(1535),
    [sym_duet_mem_size_set] = ACTIONS(1535),
    [sym_duet_mem_size_get] = ACTIONS(1535),
    [sym_dynamic_application_device] = ACTIONS(1535),
    [sym_dynamic_polled_port] = ACTIONS(1535),
    [sym_static_port_binding] = ACTIONS(1535),
    [sym_static_ip_binding] = ACTIONS(1535),
    [sym_get_dns_list] = ACTIONS(1535),
    [sym_set_dns_list] = ACTIONS(1535),
    [sym_get_ip_address] = ACTIONS(1535),
    [sym_set_ip_address] = ACTIONS(1535),
    [sym_get_url_list] = ACTIONS(1535),
    [sym_add_url_entry] = ACTIONS(1535),
    [sym_delete_url_entry] = ACTIONS(1535),
    [sym_abs_value] = ACTIONS(1535),
    [sym_max_value] = ACTIONS(1535),
    [sym_min_value] = ACTIONS(1535),
    [sym_exp_value] = ACTIONS(1535),
    [sym_log_value] = ACTIONS(1535),
    [sym_log10_value] = ACTIONS(1535),
    [sym_power_value] = ACTIONS(1535),
    [sym_sqrt_value] = ACTIONS(1535),
    [sym_raw_be] = ACTIONS(1535),
    [sym_raw_le] = ACTIONS(1535),
    [sym_internal_queue_size_set] = ACTIONS(1535),
    [sym_internal_queue_size_get] = ACTIONS(1535),
    [sym_internal_threshold_set] = ACTIONS(1535),
    [sym_internal_threshold_get] = ACTIONS(1535),
    [sym_rebuilt_event] = ACTIONS(1535),
    [sym_do_custom_event] = ACTIONS(1535),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1535),
    [sym_clkmgr_set_clk_source] = ACTIONS(1535),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1535),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1535),
    [sym_clkmgr_get_timezone] = ACTIONS(1535),
    [sym_clkmgr_set_timezone] = ACTIONS(1535),
    [sym_clkmgr_get_resync_period] = ACTIONS(1535),
    [sym_clkmgr_set_resync_period] = ACTIONS(1535),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1535),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1535),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1535),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1535),
    [sym_clkmgr_get_timeservers] = ACTIONS(1535),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1535),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1535),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1535),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1535),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1535),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1535),
    [sym_set_log_level] = ACTIONS(1535),
    [sym_get_log_level] = ACTIONS(1535),
    [sym_amx_log] = ACTIONS(1535),
    [sym_smtp_server_config_set] = ACTIONS(1535),
    [sym_smtp_server_config_get] = ACTIONS(1535),
    [sym_smtp_send] = ACTIONS(1535),
    [sym_validate_netlinx_account] = ACTIONS(1535),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1535),
    [sym_authenticate_certificate] = ACTIONS(1535),
    [sym_audit_netlinx_session_event] = ACTIONS(1535),
    [sym_audit_netlinx_generic_event] = ACTIONS(1535),
    [sym_get_available_flash_disk_space] = ACTIONS(1535),
    [sym_get_max_flash_disk_space] = ACTIONS(1535),
    [sym_data_create_feed] = ACTIONS(1535),
    [sym_data_delete_feed] = ACTIONS(1535),
    [sym_data_publish_feed] = ACTIONS(1535),
    [sym_data_get_published_feed] = ACTIONS(1535),
    [sym_data_add_record] = ACTIONS(1535),
    [sym_data_get_event_record] = ACTIONS(1535),
    [sym__wc_data_create_feed] = ACTIONS(1535),
    [sym__wc_data_add_record] = ACTIONS(1535),
    [sym__wc_data_get_event_record] = ACTIONS(1535),
    [sym_format] = ACTIONS(1535),
    [sym_astro_clock] = ACTIONS(1535),
    [sym_set_outdoor_temperature] = ACTIONS(1535),
    [sym_set_virtual_level_count] = ACTIONS(1535),
    [sym_set_virtual_channel_count] = ACTIONS(1535),
    [sym_set_virtual_port_count] = ACTIONS(1535),
    [sym__wc_encode_int] = ACTIONS(1535),
    [sym__wc_decode_int] = ACTIONS(1535),
    [sym__wc_int] = ACTIONS(1535),
    [sym__wc_to_ch_int] = ACTIONS(1535),
    [sym__ch_to_wc_int] = ACTIONS(1535),
    [sym__wc_find_string_int] = ACTIONS(1535),
    [sym__wc_left_string_int] = ACTIONS(1535),
    [sym__wc_lower_string_int] = ACTIONS(1535),
    [sym__wc_mid_string_int] = ACTIONS(1535),
    [sym__wc_remove_string_int] = ACTIONS(1535),
    [sym__wc_right_string_int] = ACTIONS(1535),
    [sym__wc_upper_string_int] = ACTIONS(1535),
    [sym__wc_compare_string_int] = ACTIONS(1535),
    [sym__wc_get_buffer_char_int] = ACTIONS(1535),
    [sym__wc_get_buffer_string_int] = ACTIONS(1535),
    [sym__wc_concat_string_int] = ACTIONS(1535),
    [sym__wc_file_open_int] = ACTIONS(1535),
    [sym__wc_file_close_int] = ACTIONS(1535),
    [sym__wc_file_read_int] = ACTIONS(1535),
    [sym__wc_file_read_line_int] = ACTIONS(1535),
    [sym__wc_file_write_int] = ACTIONS(1535),
    [sym__wc_file_write_line_int] = ACTIONS(1535),
    [anon_sym_EQ] = ACTIONS(1535),
    [aux_sym_preproc_include_token1] = ACTIONS(1535),
    [aux_sym_preproc_define_token1] = ACTIONS(1537),
    [aux_sym_preproc_warn_token1] = ACTIONS(1537),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1537),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1537),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1537),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1537),
    [aux_sym_preproc_else_token1] = ACTIONS(1537),
    [anon_sym_LBRACK] = ACTIONS(1537),
    [anon_sym_LBRACE] = ACTIONS(1537),
    [anon_sym_COLON] = ACTIONS(1537),
    [anon_sym_LPAREN] = ACTIONS(1535),
    [anon_sym_SEMI] = ACTIONS(1537),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1537),
    [anon_sym_DASH] = ACTIONS(1535),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_STAR] = ACTIONS(1537),
    [anon_sym_SLASH] = ACTIONS(1535),
    [anon_sym_PERCENT] = ACTIONS(1537),
    [anon_sym_PIPE_PIPE] = ACTIONS(1537),
    [anon_sym_AMP_AMP] = ACTIONS(1537),
    [anon_sym_PIPE] = ACTIONS(1535),
    [anon_sym_CARET] = ACTIONS(1537),
    [anon_sym_AMP] = ACTIONS(1535),
    [anon_sym_EQ_EQ] = ACTIONS(1537),
    [anon_sym_BANG_EQ] = ACTIONS(1537),
    [anon_sym_LT_GT] = ACTIONS(1537),
    [anon_sym_GT] = ACTIONS(1535),
    [anon_sym_GT_EQ] = ACTIONS(1537),
    [anon_sym_LT_EQ] = ACTIONS(1537),
    [anon_sym_LT] = ACTIONS(1535),
    [anon_sym_LT_LT] = ACTIONS(1537),
    [anon_sym_GT_GT] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DOT] = ACTIONS(1535),
    [anon_sym_DQUOTE] = ACTIONS(1537),
    [anon_sym_SQUOTE] = ACTIONS(1537),
    [sym_number_literal] = ACTIONS(1537),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1537),
  },
  [STATE(99)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1415),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1423),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1427),
    [sym_rshift] = ACTIONS(1427),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1423),
    [anon_sym_EQ_EQ] = ACTIONS(1449),
    [anon_sym_BANG_EQ] = ACTIONS(1449),
    [anon_sym_LT_GT] = ACTIONS(1449),
    [anon_sym_GT] = ACTIONS(1451),
    [anon_sym_GT_EQ] = ACTIONS(1453),
    [anon_sym_LT_EQ] = ACTIONS(1453),
    [anon_sym_LT] = ACTIONS(1451),
    [anon_sym_LT_LT] = ACTIONS(1455),
    [anon_sym_GT_GT] = ACTIONS(1455),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(100)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1415),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1427),
    [sym_rshift] = ACTIONS(1427),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1449),
    [anon_sym_BANG_EQ] = ACTIONS(1449),
    [anon_sym_LT_GT] = ACTIONS(1449),
    [anon_sym_GT] = ACTIONS(1451),
    [anon_sym_GT_EQ] = ACTIONS(1453),
    [anon_sym_LT_EQ] = ACTIONS(1453),
    [anon_sym_LT] = ACTIONS(1451),
    [anon_sym_LT_LT] = ACTIONS(1455),
    [anon_sym_GT_GT] = ACTIONS(1455),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(101)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1415),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1421),
    [sym_band] = ACTIONS(1423),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1421),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1427),
    [sym_rshift] = ACTIONS(1427),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1447),
    [anon_sym_AMP] = ACTIONS(1423),
    [anon_sym_EQ_EQ] = ACTIONS(1449),
    [anon_sym_BANG_EQ] = ACTIONS(1449),
    [anon_sym_LT_GT] = ACTIONS(1449),
    [anon_sym_GT] = ACTIONS(1451),
    [anon_sym_GT_EQ] = ACTIONS(1453),
    [anon_sym_LT_EQ] = ACTIONS(1453),
    [anon_sym_LT] = ACTIONS(1451),
    [anon_sym_LT_LT] = ACTIONS(1455),
    [anon_sym_GT_GT] = ACTIONS(1455),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(102)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1415),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1413),
    [sym_rshift] = ACTIONS(1413),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1437),
    [anon_sym_PLUS] = ACTIONS(1437),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1415),
    [anon_sym_GT_GT] = ACTIONS(1415),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(103)] = {
    [sym_identifier] = ACTIONS(1539),
    [sym_program_name_keyword] = ACTIONS(1539),
    [sym_module_name_keyword] = ACTIONS(1539),
    [sym_define_device_keyword] = ACTIONS(1539),
    [sym_define_combine_keyword] = ACTIONS(1539),
    [sym_define_constant_keyword] = ACTIONS(1539),
    [sym_define_type_keyword] = ACTIONS(1539),
    [sym_define_variable_keyword] = ACTIONS(1539),
    [sym_define_system_variable_keyword] = ACTIONS(1539),
    [sym_define_connect_level_keyword] = ACTIONS(1539),
    [sym_define_latching_keyword] = ACTIONS(1539),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1539),
    [sym_define_toggling_keyword] = ACTIONS(1539),
    [sym_define_call_keyword] = ACTIONS(1539),
    [sym_define_function_keyword] = ACTIONS(1539),
    [sym_define_library_function_keyword] = ACTIONS(1539),
    [sym_define_start_keyword] = ACTIONS(1539),
    [sym_define_event_keyword] = ACTIONS(1539),
    [sym_define_program_keyword] = ACTIONS(1539),
    [sym_define_module_keyword] = ACTIONS(1539),
    [sym_char_keyword] = ACTIONS(1539),
    [sym_widechar_keyword] = ACTIONS(1539),
    [sym_integer_keyword] = ACTIONS(1539),
    [sym_sinteger_keyword] = ACTIONS(1539),
    [sym_long_keyword] = ACTIONS(1539),
    [sym_slong_keyword] = ACTIONS(1539),
    [sym_float_keyword] = ACTIONS(1539),
    [sym_double_keyword] = ACTIONS(1539),
    [sym_variant_keyword] = ACTIONS(1539),
    [sym_variantarray_keyword] = ACTIONS(1539),
    [sym_local_var_keyword] = ACTIONS(1539),
    [sym_stack_var_keyword] = ACTIONS(1539),
    [sym_constant_keyword] = ACTIONS(1539),
    [sym_volatile_keyword] = ACTIONS(1539),
    [sym_non_volatile_keyword] = ACTIONS(1539),
    [sym_persistent_keyword] = ACTIONS(1539),
    [sym_dev_keyword] = ACTIONS(1539),
    [sym_devlev_keyword] = ACTIONS(1539),
    [sym_devchan_keyword] = ACTIONS(1539),
    [sym_if_keyword] = ACTIONS(1539),
    [sym_else_keyword] = ACTIONS(1539),
    [sym_switch_keyword] = ACTIONS(1539),
    [sym_case_keyword] = ACTIONS(1539),
    [sym_default_keyword] = ACTIONS(1539),
    [sym_while_keyword] = ACTIONS(1539),
    [sym_for_keyword] = ACTIONS(1539),
    [sym_break_keyword] = ACTIONS(1539),
    [sym_continue_keyword] = ACTIONS(1539),
    [sym_return_keyword] = ACTIONS(1539),
    [sym_select_keyword] = ACTIONS(1539),
    [sym_active_keyword] = ACTIONS(1539),
    [sym_struct_keyword] = ACTIONS(1539),
    [sym_structure_keyword] = ACTIONS(1539),
    [sym_button_event_keyword] = ACTIONS(1539),
    [sym_channel_event_keyword] = ACTIONS(1539),
    [sym_level_event_keyword] = ACTIONS(1539),
    [sym_data_event_keyword] = ACTIONS(1539),
    [sym_timeline_event_keyword] = ACTIONS(1539),
    [sym_custom_event_keyword] = ACTIONS(1539),
    [sym_push_keyword] = ACTIONS(1539),
    [sym_release_keyword] = ACTIONS(1539),
    [aux_sym_on_keyword_token1] = ACTIONS(1539),
    [aux_sym_off_keyword_token1] = ACTIONS(1539),
    [sym_send_level_keyword] = ACTIONS(1539),
    [sym_send_string_keyword] = ACTIONS(1539),
    [sym_send_command_keyword] = ACTIONS(1539),
    [sym_clear_buffer_keyword] = ACTIONS(1539),
    [sym_create_buffer_keyword] = ACTIONS(1539),
    [sym_create_multi_buffer_keyword] = ACTIONS(1539),
    [sym_call_keyword] = ACTIONS(1539),
    [sym_system_call_keyword] = ACTIONS(1539),
    [sym_devchan_to_keyword] = ACTIONS(1539),
    [sym_devchan_min_to_keyword] = ACTIONS(1539),
    [sym_devchan_total_off_keyword] = ACTIONS(1539),
    [sym_devchan_pulse_keyword] = ACTIONS(1539),
    [sym_wait_keyword] = ACTIONS(1539),
    [sym_cancel_wait_keyword] = ACTIONS(1539),
    [sym_wait_until_keyword] = ACTIONS(1539),
    [sym_cancel_wait_until_keyword] = ACTIONS(1539),
    [sym_cancel_all_wait_keyword] = ACTIONS(1539),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1539),
    [sym_and] = ACTIONS(1539),
    [sym_or] = ACTIONS(1539),
    [sym_not] = ACTIONS(1539),
    [sym_xor] = ACTIONS(1539),
    [sym_band] = ACTIONS(1539),
    [sym_bor] = ACTIONS(1539),
    [sym_bxor] = ACTIONS(1539),
    [sym_bnot] = ACTIONS(1539),
    [sym_lshift] = ACTIONS(1539),
    [sym_rshift] = ACTIONS(1539),
    [sym___file__] = ACTIONS(1539),
    [sym___date__] = ACTIONS(1539),
    [sym___ldate__] = ACTIONS(1539),
    [sym___time__] = ACTIONS(1539),
    [sym___line__] = ACTIONS(1539),
    [sym___name__] = ACTIONS(1539),
    [sym___netlinx__] = ACTIONS(1539),
    [sym_netlinx_axi_version] = ACTIONS(1539),
    [sym_url_flg_tcp] = ACTIONS(1539),
    [sym_url_flg_accinfopresent] = ACTIONS(1539),
    [sym_url_flg_temp] = ACTIONS(1539),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1539),
    [sym_url_flg_stat_mask] = ACTIONS(1539),
    [sym_url_flg_stat_lookup] = ACTIONS(1539),
    [sym_url_flg_stat_connecting] = ACTIONS(1539),
    [sym_url_flg_stat_waiting] = ACTIONS(1539),
    [sym_url_flg_stat_connected] = ACTIONS(1539),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1539),
    [sym_first_virtual_device] = ACTIONS(1539),
    [sym_first_local_port] = ACTIONS(1539),
    [sym_dynamic_virtual_device] = ACTIONS(1539),
    [sym_timeline_once] = ACTIONS(1539),
    [sym_timeline_repeat] = ACTIONS(1539),
    [sym_timeline_absolute] = ACTIONS(1539),
    [sym_timeline_relative] = ACTIONS(1539),
    [sym_do_push_timed_infinite] = ACTIONS(1539),
    [sym_source_type_no_address] = ACTIONS(1539),
    [sym_source_type_neuron_id] = ACTIONS(1539),
    [sym_source_type_ip_address] = ACTIONS(1539),
    [sym_source_type_axlink] = ACTIONS(1539),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1539),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1539),
    [sym_source_type_ip_socket_address] = ACTIONS(1539),
    [sym_source_type_rs232] = ACTIONS(1539),
    [sym_source_type_internal] = ACTIONS(1539),
    [sym_source_type_mac_address] = ACTIONS(1539),
    [sym_source_type_ipv4_port] = ACTIONS(1539),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1539),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1539),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1539),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1539),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1539),
    [sym_true] = ACTIONS(1539),
    [sym_false] = ACTIONS(1539),
    [sym_file_read_only] = ACTIONS(1539),
    [sym_file_rw_new] = ACTIONS(1539),
    [sym_file_rw_append] = ACTIONS(1539),
    [sym_ip_tcp] = ACTIONS(1539),
    [sym_ip_udp] = ACTIONS(1539),
    [sym_ip_udp_2way] = ACTIONS(1539),
    [sym_xml_encode_types] = ACTIONS(1539),
    [sym_xml_encode_char_as_list] = ACTIONS(1539),
    [sym_xml_encode_le] = ACTIONS(1539),
    [sym_xml_decode_types] = ACTIONS(1539),
    [sym_xml_decode_no_preserve] = ACTIONS(1539),
    [sym_duet_dev_type_display_device] = ACTIONS(1539),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1539),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1539),
    [sym_duet_dev_type_io_device] = ACTIONS(1539),
    [sym_duet_dev_type_relay_device] = ACTIONS(1539),
    [sym_duet_dev_type_ups] = ACTIONS(1539),
    [sym_duet_dev_type_amplifier] = ACTIONS(1539),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1539),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1539),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1539),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1539),
    [sym_duet_dev_type_camera] = ACTIONS(1539),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1539),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1539),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1539),
    [sym_duet_dev_type_dss] = ACTIONS(1539),
    [sym_duet_dev_type_dvr] = ACTIONS(1539),
    [sym_duet_dev_type_disc_device] = ACTIONS(1539),
    [sym_duet_dev_type_document_camera] = ACTIONS(1539),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1539),
    [sym_duet_dev_type_hvac] = ACTIONS(1539),
    [sym_duet_dev_type_keypad] = ACTIONS(1539),
    [sym_duet_dev_type_light] = ACTIONS(1539),
    [sym_duet_dev_type_monitor] = ACTIONS(1539),
    [sym_duet_dev_type_motor] = ACTIONS(1539),
    [sym_duet_dev_type_multi_window] = ACTIONS(1539),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1539),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1539),
    [sym_duet_dev_type_receiver] = ACTIONS(1539),
    [sym_duet_dev_type_security_system] = ACTIONS(1539),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1539),
    [sym_duet_dev_type_settop_box] = ACTIONS(1539),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1539),
    [sym_duet_dev_type_switcher] = ACTIONS(1539),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1539),
    [sym_duet_dev_type_tv] = ACTIONS(1539),
    [sym_duet_dev_type_utility] = ACTIONS(1539),
    [sym_duet_dev_type_vcr] = ACTIONS(1539),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1539),
    [sym_duet_dev_type_video_processor] = ACTIONS(1539),
    [sym_duet_dev_type_video_projector] = ACTIONS(1539),
    [sym_duet_dev_type_video_wall] = ACTIONS(1539),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1539),
    [sym_duet_dev_type_weather] = ACTIONS(1539),
    [sym_duet_dev_not_polled] = ACTIONS(1539),
    [sym_duet_dev_polled] = ACTIONS(1539),
    [sym_clkmgr_mode_network] = ACTIONS(1539),
    [sym_clkmgr_mode_standalone] = ACTIONS(1539),
    [sym_normal_standby] = ACTIONS(1539),
    [sym_normal_wake] = ACTIONS(1539),
    [sym_ip_multicast_ttl_option] = ACTIONS(1539),
    [sym_ip_ttl_subnet] = ACTIONS(1539),
    [sym_ip_ttl_site] = ACTIONS(1539),
    [sym_ip_ttl_region] = ACTIONS(1539),
    [sym_ip_ttl_continent] = ACTIONS(1539),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1539),
    [sym_ip_nodelay_on] = ACTIONS(1539),
    [sym_ip_nodelay_off] = ACTIONS(1539),
    [sym_amx_error] = ACTIONS(1539),
    [sym_amx_warning] = ACTIONS(1539),
    [sym_amx_info] = ACTIONS(1539),
    [sym_amx_debug] = ACTIONS(1539),
    [sym_netlinx_login_success] = ACTIONS(1539),
    [sym_netlinx_login_fail] = ACTIONS(1539),
    [sym_netlinx_logout] = ACTIONS(1539),
    [sym_null_str] = ACTIONS(1539),
    [sym_smtp_address] = ACTIONS(1539),
    [sym_smtp_port_number] = ACTIONS(1539),
    [sym_smtp_username] = ACTIONS(1539),
    [sym_smtp_password] = ACTIONS(1539),
    [sym_smtp_require_tls] = ACTIONS(1539),
    [sym_smtp_from] = ACTIONS(1539),
    [sym_smtp_tls_true] = ACTIONS(1539),
    [sym_smtp_tls_false] = ACTIONS(1539),
    [sym_listview_on_row_select_event] = ACTIONS(1539),
    [sym_data_structure_unknown] = ACTIONS(1539),
    [sym_data_structure_datafeed] = ACTIONS(1539),
    [sym_data_structure_datarecordset] = ACTIONS(1539),
    [sym_data_structure_datarecord] = ACTIONS(1539),
    [sym_data_structure_datafield] = ACTIONS(1539),
    [sym_data_max_content_field_count] = ACTIONS(1539),
    [sym_data_max_metadata_field_count] = ACTIONS(1539),
    [sym_data_max_id_length] = ACTIONS(1539),
    [sym_data_max_name_length] = ACTIONS(1539),
    [sym_data_max_description_length] = ACTIONS(1539),
    [sym_data_max_type_length] = ACTIONS(1539),
    [sym_data_max_format_length] = ACTIONS(1539),
    [sym_data_max_source_length] = ACTIONS(1539),
    [sym_data_max_value_length] = ACTIONS(1539),
    [sym_data_max_label_length] = ACTIONS(1539),
    [sym_data_max_url_length] = ACTIONS(1539),
    [sym_max_last_login_info_length] = ACTIONS(1539),
    [sym_data_type_unknown] = ACTIONS(1539),
    [sym_data_type_string] = ACTIONS(1539),
    [sym_data_type_datetime] = ACTIONS(1539),
    [sym_data_type_date] = ACTIONS(1539),
    [sym_data_type_time] = ACTIONS(1539),
    [sym_data_type_image] = ACTIONS(1539),
    [sym_data_format_url] = ACTIONS(1539),
    [sym_data_format_phone] = ACTIONS(1539),
    [sym_data_format_email] = ACTIONS(1539),
    [sym_data_format_iso8601] = ACTIONS(1539),
    [sym_auditlog_priv] = ACTIONS(1539),
    [sym_configuration_priv] = ACTIONS(1539),
    [sym_deviceconfig_priv] = ACTIONS(1539),
    [sym_ftp_priv] = ACTIONS(1539),
    [sym_http_priv] = ACTIONS(1539),
    [sym_networkconfig_priv] = ACTIONS(1539),
    [sym_programport_priv] = ACTIONS(1539),
    [sym_remoteui_priv] = ACTIONS(1539),
    [sym_securitycontrol_priv] = ACTIONS(1539),
    [sym_softwaremanagement_priv] = ACTIONS(1539),
    [sym_terminal_priv] = ACTIONS(1539),
    [sym_tpadmin_priv] = ACTIONS(1539),
    [sym_user1_priv] = ACTIONS(1539),
    [sym_user2_priv] = ACTIONS(1539),
    [sym_user3_priv] = ACTIONS(1539),
    [sym_user4_priv] = ACTIONS(1539),
    [sym_usermanagement_priv] = ACTIONS(1539),
    [sym_valid_account] = ACTIONS(1539),
    [sym_err_name_invalid] = ACTIONS(1539),
    [sym_err_password_invalid] = ACTIONS(1539),
    [sym_err_invalid_account] = ACTIONS(1539),
    [sym_err_no_authorization] = ACTIONS(1539),
    [sym_err_invalid_parameter] = ACTIONS(1539),
    [sym_err_user_locked_out] = ACTIONS(1539),
    [sym_err_user_account_expired] = ACTIONS(1539),
    [sym_err_general_error] = ACTIONS(1539),
    [sym_err_authentication_error] = ACTIONS(1539),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1539),
    [sym_tls_validate_certificate] = ACTIONS(1539),
    [sym_tdata] = ACTIONS(1539),
    [sym_tchannel] = ACTIONS(1539),
    [sym_tlevel] = ACTIONS(1539),
    [sym_tbutton] = ACTIONS(1539),
    [sym_ttimeline] = ACTIONS(1539),
    [sym_tcustom] = ACTIONS(1539),
    [sym_url_struct] = ACTIONS(1539),
    [sym_dns_struct] = ACTIONS(1539),
    [sym_ip_address_struct] = ACTIONS(1539),
    [sym_dev_info_struct] = ACTIONS(1539),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1539),
    [sym_data_feed] = ACTIONS(1539),
    [sym_data_field] = ACTIONS(1539),
    [sym_data_record] = ACTIONS(1539),
    [sym_wc_data_feed] = ACTIONS(1539),
    [sym_wc_data_field] = ACTIONS(1539),
    [sym_wc_data_record] = ACTIONS(1539),
    [sym_last_login_info] = ACTIONS(1539),
    [sym____reserved___] = ACTIONS(1539),
    [sym_date] = ACTIONS(1539),
    [sym_day] = ACTIONS(1539),
    [sym_time] = ACTIONS(1539),
    [sym_ldate] = ACTIONS(1539),
    [sym_push_device] = ACTIONS(1539),
    [sym_push_channel] = ACTIONS(1539),
    [sym_push_devchan] = ACTIONS(1539),
    [sym_release_device] = ACTIONS(1539),
    [sym_release_channel] = ACTIONS(1539),
    [sym_release_devchan] = ACTIONS(1539),
    [sym_master_slot] = ACTIONS(1539),
    [sym_get_pulse_time] = ACTIONS(1539),
    [sym_get_timer] = ACTIONS(1539),
    [sym_master_sn] = ACTIONS(1539),
    [sym_system_number] = ACTIONS(1539),
    [sym_button] = ACTIONS(1539),
    [sym_dv_channel] = ACTIONS(1539),
    [sym_data] = ACTIONS(1539),
    [sym_level] = ACTIONS(1539),
    [sym_timeline] = ACTIONS(1539),
    [sym_channel] = ACTIONS(1539),
    [sym_custom] = ACTIONS(1539),
    [sym_atoi] = ACTIONS(1539),
    [sym_atol] = ACTIONS(1539),
    [sym_atof] = ACTIONS(1539),
    [sym_itoa] = ACTIONS(1539),
    [sym_itohex] = ACTIONS(1539),
    [sym_hextoi] = ACTIONS(1539),
    [sym_ftoa] = ACTIONS(1539),
    [sym_find_string] = ACTIONS(1539),
    [sym_left_string] = ACTIONS(1539),
    [sym_right_string] = ACTIONS(1539),
    [sym_mid_string] = ACTIONS(1539),
    [sym_remove_string] = ACTIONS(1539),
    [sym_get_buffer_string] = ACTIONS(1539),
    [sym_get_buffer_char] = ACTIONS(1539),
    [sym_get_multi_buffer_string] = ACTIONS(1539),
    [sym_length_string] = ACTIONS(1539),
    [sym_max_length_string] = ACTIONS(1539),
    [sym_set_length_string] = ACTIONS(1539),
    [sym_compare_string] = ACTIONS(1539),
    [sym_upper_string] = ACTIONS(1539),
    [sym_lower_string] = ACTIONS(1539),
    [sym_get_last] = ACTIONS(1539),
    [sym_date_to_day] = ACTIONS(1539),
    [sym_date_to_month] = ACTIONS(1539),
    [sym_date_to_year] = ACTIONS(1539),
    [sym_time_to_hour] = ACTIONS(1539),
    [sym_time_to_minute] = ACTIONS(1539),
    [sym_time_to_second] = ACTIONS(1539),
    [sym_day_of_week] = ACTIONS(1539),
    [sym_set_pulse_time] = ACTIONS(1539),
    [sym_set_timer] = ACTIONS(1539),
    [sym_file_open] = ACTIONS(1539),
    [sym_file_close] = ACTIONS(1539),
    [sym_file_read] = ACTIONS(1539),
    [sym_file_read_line] = ACTIONS(1539),
    [sym_file_write] = ACTIONS(1539),
    [sym_file_write_line] = ACTIONS(1539),
    [sym_file_copy] = ACTIONS(1539),
    [sym_file_delete] = ACTIONS(1539),
    [sym_file_rename] = ACTIONS(1539),
    [sym_file_dir] = ACTIONS(1539),
    [sym_file_getdir] = ACTIONS(1539),
    [sym_file_setdir] = ACTIONS(1539),
    [sym_file_create_dir] = ACTIONS(1539),
    [sym_file_remove_dir] = ACTIONS(1539),
    [sym_file_seek] = ACTIONS(1539),
    [sym_length_array] = ACTIONS(1539),
    [sym_max_length_array] = ACTIONS(1539),
    [sym_set_length_array] = ACTIONS(1539),
    [sym_ip_client_open] = ACTIONS(1539),
    [sym_ip_client_close] = ACTIONS(1539),
    [sym_ip_server_open] = ACTIONS(1539),
    [sym_ip_server_close] = ACTIONS(1539),
    [sym_ip_bound_client_open] = ACTIONS(1539),
    [sym_ip_mc_server_open] = ACTIONS(1539),
    [sym_ip_set_option] = ACTIONS(1539),
    [sym_ssh_client_open] = ACTIONS(1539),
    [sym_ssh_client_close] = ACTIONS(1539),
    [sym_tls_client_open] = ACTIONS(1539),
    [sym_tls_client_close] = ACTIONS(1539),
    [sym_do_push] = ACTIONS(1539),
    [sym_do_push_timed] = ACTIONS(1539),
    [sym_do_release] = ACTIONS(1539),
    [sym_device_id] = ACTIONS(1539),
    [sym_device_info] = ACTIONS(1539),
    [sym_device_id_string] = ACTIONS(1539),
    [sym_device_standby] = ACTIONS(1539),
    [sym_device_wake] = ACTIONS(1539),
    [sym_timeline_create] = ACTIONS(1539),
    [sym_timeline_kill] = ACTIONS(1539),
    [sym_timeline_reload] = ACTIONS(1539),
    [sym_timeline_pause] = ACTIONS(1539),
    [sym_timeline_restart] = ACTIONS(1539),
    [sym_timeline_set] = ACTIONS(1539),
    [sym_timeline_get] = ACTIONS(1539),
    [sym_timeline_active] = ACTIONS(1539),
    [sym_timeline_dynamic_id] = ACTIONS(1539),
    [sym_variable_to_string] = ACTIONS(1539),
    [sym_string_to_variable] = ACTIONS(1539),
    [sym_length_variable_to_string] = ACTIONS(1539),
    [sym_type_cast] = ACTIONS(1539),
    [sym_variable_to_xml] = ACTIONS(1539),
    [sym_xml_to_variable] = ACTIONS(1539),
    [sym_length_variable_to_xml] = ACTIONS(1539),
    [sym_redirect_string] = ACTIONS(1539),
    [sym_random_number] = ACTIONS(1539),
    [sym_get_unique_id] = ACTIONS(1539),
    [sym_get_system_number] = ACTIONS(1539),
    [sym_set_system_number] = ACTIONS(1539),
    [sym_get_serial_number] = ACTIONS(1539),
    [sym_get_master_build] = ACTIONS(1539),
    [sym_set_validation_code] = ACTIONS(1539),
    [sym_reboot] = ACTIONS(1539),
    [sym_load_duet_module] = ACTIONS(1539),
    [sym_unload_duet_module] = ACTIONS(1539),
    [sym_duet_mem_size_set] = ACTIONS(1539),
    [sym_duet_mem_size_get] = ACTIONS(1539),
    [sym_dynamic_application_device] = ACTIONS(1539),
    [sym_dynamic_polled_port] = ACTIONS(1539),
    [sym_static_port_binding] = ACTIONS(1539),
    [sym_static_ip_binding] = ACTIONS(1539),
    [sym_get_dns_list] = ACTIONS(1539),
    [sym_set_dns_list] = ACTIONS(1539),
    [sym_get_ip_address] = ACTIONS(1539),
    [sym_set_ip_address] = ACTIONS(1539),
    [sym_get_url_list] = ACTIONS(1539),
    [sym_add_url_entry] = ACTIONS(1539),
    [sym_delete_url_entry] = ACTIONS(1539),
    [sym_abs_value] = ACTIONS(1539),
    [sym_max_value] = ACTIONS(1539),
    [sym_min_value] = ACTIONS(1539),
    [sym_exp_value] = ACTIONS(1539),
    [sym_log_value] = ACTIONS(1539),
    [sym_log10_value] = ACTIONS(1539),
    [sym_power_value] = ACTIONS(1539),
    [sym_sqrt_value] = ACTIONS(1539),
    [sym_raw_be] = ACTIONS(1539),
    [sym_raw_le] = ACTIONS(1539),
    [sym_internal_queue_size_set] = ACTIONS(1539),
    [sym_internal_queue_size_get] = ACTIONS(1539),
    [sym_internal_threshold_set] = ACTIONS(1539),
    [sym_internal_threshold_get] = ACTIONS(1539),
    [sym_rebuilt_event] = ACTIONS(1539),
    [sym_do_custom_event] = ACTIONS(1539),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1539),
    [sym_clkmgr_set_clk_source] = ACTIONS(1539),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1539),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1539),
    [sym_clkmgr_get_timezone] = ACTIONS(1539),
    [sym_clkmgr_set_timezone] = ACTIONS(1539),
    [sym_clkmgr_get_resync_period] = ACTIONS(1539),
    [sym_clkmgr_set_resync_period] = ACTIONS(1539),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1539),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1539),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1539),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1539),
    [sym_clkmgr_get_timeservers] = ACTIONS(1539),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1539),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1539),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1539),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1539),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1539),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1539),
    [sym_set_log_level] = ACTIONS(1539),
    [sym_get_log_level] = ACTIONS(1539),
    [sym_amx_log] = ACTIONS(1539),
    [sym_smtp_server_config_set] = ACTIONS(1539),
    [sym_smtp_server_config_get] = ACTIONS(1539),
    [sym_smtp_send] = ACTIONS(1539),
    [sym_validate_netlinx_account] = ACTIONS(1539),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1539),
    [sym_authenticate_certificate] = ACTIONS(1539),
    [sym_audit_netlinx_session_event] = ACTIONS(1539),
    [sym_audit_netlinx_generic_event] = ACTIONS(1539),
    [sym_get_available_flash_disk_space] = ACTIONS(1539),
    [sym_get_max_flash_disk_space] = ACTIONS(1539),
    [sym_data_create_feed] = ACTIONS(1539),
    [sym_data_delete_feed] = ACTIONS(1539),
    [sym_data_publish_feed] = ACTIONS(1539),
    [sym_data_get_published_feed] = ACTIONS(1539),
    [sym_data_add_record] = ACTIONS(1539),
    [sym_data_get_event_record] = ACTIONS(1539),
    [sym__wc_data_create_feed] = ACTIONS(1539),
    [sym__wc_data_add_record] = ACTIONS(1539),
    [sym__wc_data_get_event_record] = ACTIONS(1539),
    [sym_format] = ACTIONS(1539),
    [sym_astro_clock] = ACTIONS(1539),
    [sym_set_outdoor_temperature] = ACTIONS(1539),
    [sym_set_virtual_level_count] = ACTIONS(1539),
    [sym_set_virtual_channel_count] = ACTIONS(1539),
    [sym_set_virtual_port_count] = ACTIONS(1539),
    [sym__wc_encode_int] = ACTIONS(1539),
    [sym__wc_decode_int] = ACTIONS(1539),
    [sym__wc_int] = ACTIONS(1539),
    [sym__wc_to_ch_int] = ACTIONS(1539),
    [sym__ch_to_wc_int] = ACTIONS(1539),
    [sym__wc_find_string_int] = ACTIONS(1539),
    [sym__wc_left_string_int] = ACTIONS(1539),
    [sym__wc_lower_string_int] = ACTIONS(1539),
    [sym__wc_mid_string_int] = ACTIONS(1539),
    [sym__wc_remove_string_int] = ACTIONS(1539),
    [sym__wc_right_string_int] = ACTIONS(1539),
    [sym__wc_upper_string_int] = ACTIONS(1539),
    [sym__wc_compare_string_int] = ACTIONS(1539),
    [sym__wc_get_buffer_char_int] = ACTIONS(1539),
    [sym__wc_get_buffer_string_int] = ACTIONS(1539),
    [sym__wc_concat_string_int] = ACTIONS(1539),
    [sym__wc_file_open_int] = ACTIONS(1539),
    [sym__wc_file_close_int] = ACTIONS(1539),
    [sym__wc_file_read_int] = ACTIONS(1539),
    [sym__wc_file_read_line_int] = ACTIONS(1539),
    [sym__wc_file_write_int] = ACTIONS(1539),
    [sym__wc_file_write_line_int] = ACTIONS(1539),
    [anon_sym_EQ] = ACTIONS(1539),
    [aux_sym_preproc_include_token1] = ACTIONS(1539),
    [aux_sym_preproc_define_token1] = ACTIONS(1541),
    [aux_sym_preproc_warn_token1] = ACTIONS(1541),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1541),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1541),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1541),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1541),
    [aux_sym_preproc_else_token1] = ACTIONS(1541),
    [anon_sym_LBRACK] = ACTIONS(1541),
    [anon_sym_LBRACE] = ACTIONS(1541),
    [anon_sym_COLON] = ACTIONS(1541),
    [anon_sym_LPAREN] = ACTIONS(1539),
    [anon_sym_SEMI] = ACTIONS(1541),
    [anon_sym_BANG] = ACTIONS(1539),
    [anon_sym_TILDE] = ACTIONS(1541),
    [anon_sym_DASH] = ACTIONS(1539),
    [anon_sym_PLUS] = ACTIONS(1539),
    [anon_sym_STAR] = ACTIONS(1541),
    [anon_sym_SLASH] = ACTIONS(1539),
    [anon_sym_PERCENT] = ACTIONS(1541),
    [anon_sym_PIPE_PIPE] = ACTIONS(1541),
    [anon_sym_AMP_AMP] = ACTIONS(1541),
    [anon_sym_PIPE] = ACTIONS(1539),
    [anon_sym_CARET] = ACTIONS(1541),
    [anon_sym_AMP] = ACTIONS(1539),
    [anon_sym_EQ_EQ] = ACTIONS(1541),
    [anon_sym_BANG_EQ] = ACTIONS(1541),
    [anon_sym_LT_GT] = ACTIONS(1541),
    [anon_sym_GT] = ACTIONS(1539),
    [anon_sym_GT_EQ] = ACTIONS(1541),
    [anon_sym_LT_EQ] = ACTIONS(1541),
    [anon_sym_LT] = ACTIONS(1539),
    [anon_sym_LT_LT] = ACTIONS(1541),
    [anon_sym_GT_GT] = ACTIONS(1541),
    [anon_sym_DASH_DASH] = ACTIONS(1541),
    [anon_sym_PLUS_PLUS] = ACTIONS(1541),
    [anon_sym_DOT] = ACTIONS(1539),
    [anon_sym_DQUOTE] = ACTIONS(1541),
    [anon_sym_SQUOTE] = ACTIONS(1541),
    [sym_number_literal] = ACTIONS(1541),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1541),
  },
  [STATE(104)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1415),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1413),
    [sym_rshift] = ACTIONS(1413),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_STAR] = ACTIONS(1415),
    [anon_sym_SLASH] = ACTIONS(1413),
    [anon_sym_PERCENT] = ACTIONS(1415),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1415),
    [anon_sym_GT_GT] = ACTIONS(1415),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(105)] = {
    [sym_identifier] = ACTIONS(1543),
    [sym_program_name_keyword] = ACTIONS(1543),
    [sym_module_name_keyword] = ACTIONS(1543),
    [sym_define_device_keyword] = ACTIONS(1543),
    [sym_define_combine_keyword] = ACTIONS(1543),
    [sym_define_constant_keyword] = ACTIONS(1543),
    [sym_define_type_keyword] = ACTIONS(1543),
    [sym_define_variable_keyword] = ACTIONS(1543),
    [sym_define_system_variable_keyword] = ACTIONS(1543),
    [sym_define_connect_level_keyword] = ACTIONS(1543),
    [sym_define_latching_keyword] = ACTIONS(1543),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1543),
    [sym_define_toggling_keyword] = ACTIONS(1543),
    [sym_define_call_keyword] = ACTIONS(1543),
    [sym_define_function_keyword] = ACTIONS(1543),
    [sym_define_library_function_keyword] = ACTIONS(1543),
    [sym_define_start_keyword] = ACTIONS(1543),
    [sym_define_event_keyword] = ACTIONS(1543),
    [sym_define_program_keyword] = ACTIONS(1543),
    [sym_define_module_keyword] = ACTIONS(1543),
    [sym_char_keyword] = ACTIONS(1543),
    [sym_widechar_keyword] = ACTIONS(1543),
    [sym_integer_keyword] = ACTIONS(1543),
    [sym_sinteger_keyword] = ACTIONS(1543),
    [sym_long_keyword] = ACTIONS(1543),
    [sym_slong_keyword] = ACTIONS(1543),
    [sym_float_keyword] = ACTIONS(1543),
    [sym_double_keyword] = ACTIONS(1543),
    [sym_variant_keyword] = ACTIONS(1543),
    [sym_variantarray_keyword] = ACTIONS(1543),
    [sym_local_var_keyword] = ACTIONS(1543),
    [sym_stack_var_keyword] = ACTIONS(1543),
    [sym_constant_keyword] = ACTIONS(1543),
    [sym_volatile_keyword] = ACTIONS(1543),
    [sym_non_volatile_keyword] = ACTIONS(1543),
    [sym_persistent_keyword] = ACTIONS(1543),
    [sym_dev_keyword] = ACTIONS(1543),
    [sym_devlev_keyword] = ACTIONS(1543),
    [sym_devchan_keyword] = ACTIONS(1543),
    [sym_if_keyword] = ACTIONS(1543),
    [sym_else_keyword] = ACTIONS(1543),
    [sym_switch_keyword] = ACTIONS(1543),
    [sym_case_keyword] = ACTIONS(1543),
    [sym_default_keyword] = ACTIONS(1543),
    [sym_while_keyword] = ACTIONS(1543),
    [sym_for_keyword] = ACTIONS(1543),
    [sym_break_keyword] = ACTIONS(1543),
    [sym_continue_keyword] = ACTIONS(1543),
    [sym_return_keyword] = ACTIONS(1543),
    [sym_select_keyword] = ACTIONS(1543),
    [sym_active_keyword] = ACTIONS(1543),
    [sym_struct_keyword] = ACTIONS(1543),
    [sym_structure_keyword] = ACTIONS(1543),
    [sym_button_event_keyword] = ACTIONS(1543),
    [sym_channel_event_keyword] = ACTIONS(1543),
    [sym_level_event_keyword] = ACTIONS(1543),
    [sym_data_event_keyword] = ACTIONS(1543),
    [sym_timeline_event_keyword] = ACTIONS(1543),
    [sym_custom_event_keyword] = ACTIONS(1543),
    [sym_push_keyword] = ACTIONS(1543),
    [sym_release_keyword] = ACTIONS(1543),
    [aux_sym_on_keyword_token1] = ACTIONS(1543),
    [aux_sym_off_keyword_token1] = ACTIONS(1543),
    [sym_send_level_keyword] = ACTIONS(1543),
    [sym_send_string_keyword] = ACTIONS(1543),
    [sym_send_command_keyword] = ACTIONS(1543),
    [sym_clear_buffer_keyword] = ACTIONS(1543),
    [sym_create_buffer_keyword] = ACTIONS(1543),
    [sym_create_multi_buffer_keyword] = ACTIONS(1543),
    [sym_call_keyword] = ACTIONS(1543),
    [sym_system_call_keyword] = ACTIONS(1543),
    [sym_devchan_to_keyword] = ACTIONS(1543),
    [sym_devchan_min_to_keyword] = ACTIONS(1543),
    [sym_devchan_total_off_keyword] = ACTIONS(1543),
    [sym_devchan_pulse_keyword] = ACTIONS(1543),
    [sym_wait_keyword] = ACTIONS(1543),
    [sym_cancel_wait_keyword] = ACTIONS(1543),
    [sym_wait_until_keyword] = ACTIONS(1543),
    [sym_cancel_wait_until_keyword] = ACTIONS(1543),
    [sym_cancel_all_wait_keyword] = ACTIONS(1543),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1543),
    [sym_and] = ACTIONS(1543),
    [sym_or] = ACTIONS(1543),
    [sym_not] = ACTIONS(1543),
    [sym_xor] = ACTIONS(1543),
    [sym_band] = ACTIONS(1543),
    [sym_bor] = ACTIONS(1543),
    [sym_bxor] = ACTIONS(1543),
    [sym_bnot] = ACTIONS(1543),
    [sym_lshift] = ACTIONS(1543),
    [sym_rshift] = ACTIONS(1543),
    [sym___file__] = ACTIONS(1543),
    [sym___date__] = ACTIONS(1543),
    [sym___ldate__] = ACTIONS(1543),
    [sym___time__] = ACTIONS(1543),
    [sym___line__] = ACTIONS(1543),
    [sym___name__] = ACTIONS(1543),
    [sym___netlinx__] = ACTIONS(1543),
    [sym_netlinx_axi_version] = ACTIONS(1543),
    [sym_url_flg_tcp] = ACTIONS(1543),
    [sym_url_flg_accinfopresent] = ACTIONS(1543),
    [sym_url_flg_temp] = ACTIONS(1543),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1543),
    [sym_url_flg_stat_mask] = ACTIONS(1543),
    [sym_url_flg_stat_lookup] = ACTIONS(1543),
    [sym_url_flg_stat_connecting] = ACTIONS(1543),
    [sym_url_flg_stat_waiting] = ACTIONS(1543),
    [sym_url_flg_stat_connected] = ACTIONS(1543),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1543),
    [sym_first_virtual_device] = ACTIONS(1543),
    [sym_first_local_port] = ACTIONS(1543),
    [sym_dynamic_virtual_device] = ACTIONS(1543),
    [sym_timeline_once] = ACTIONS(1543),
    [sym_timeline_repeat] = ACTIONS(1543),
    [sym_timeline_absolute] = ACTIONS(1543),
    [sym_timeline_relative] = ACTIONS(1543),
    [sym_do_push_timed_infinite] = ACTIONS(1543),
    [sym_source_type_no_address] = ACTIONS(1543),
    [sym_source_type_neuron_id] = ACTIONS(1543),
    [sym_source_type_ip_address] = ACTIONS(1543),
    [sym_source_type_axlink] = ACTIONS(1543),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1543),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1543),
    [sym_source_type_ip_socket_address] = ACTIONS(1543),
    [sym_source_type_rs232] = ACTIONS(1543),
    [sym_source_type_internal] = ACTIONS(1543),
    [sym_source_type_mac_address] = ACTIONS(1543),
    [sym_source_type_ipv4_port] = ACTIONS(1543),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1543),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1543),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1543),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1543),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1543),
    [sym_true] = ACTIONS(1543),
    [sym_false] = ACTIONS(1543),
    [sym_file_read_only] = ACTIONS(1543),
    [sym_file_rw_new] = ACTIONS(1543),
    [sym_file_rw_append] = ACTIONS(1543),
    [sym_ip_tcp] = ACTIONS(1543),
    [sym_ip_udp] = ACTIONS(1543),
    [sym_ip_udp_2way] = ACTIONS(1543),
    [sym_xml_encode_types] = ACTIONS(1543),
    [sym_xml_encode_char_as_list] = ACTIONS(1543),
    [sym_xml_encode_le] = ACTIONS(1543),
    [sym_xml_decode_types] = ACTIONS(1543),
    [sym_xml_decode_no_preserve] = ACTIONS(1543),
    [sym_duet_dev_type_display_device] = ACTIONS(1543),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1543),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1543),
    [sym_duet_dev_type_io_device] = ACTIONS(1543),
    [sym_duet_dev_type_relay_device] = ACTIONS(1543),
    [sym_duet_dev_type_ups] = ACTIONS(1543),
    [sym_duet_dev_type_amplifier] = ACTIONS(1543),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1543),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1543),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1543),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1543),
    [sym_duet_dev_type_camera] = ACTIONS(1543),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1543),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1543),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1543),
    [sym_duet_dev_type_dss] = ACTIONS(1543),
    [sym_duet_dev_type_dvr] = ACTIONS(1543),
    [sym_duet_dev_type_disc_device] = ACTIONS(1543),
    [sym_duet_dev_type_document_camera] = ACTIONS(1543),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1543),
    [sym_duet_dev_type_hvac] = ACTIONS(1543),
    [sym_duet_dev_type_keypad] = ACTIONS(1543),
    [sym_duet_dev_type_light] = ACTIONS(1543),
    [sym_duet_dev_type_monitor] = ACTIONS(1543),
    [sym_duet_dev_type_motor] = ACTIONS(1543),
    [sym_duet_dev_type_multi_window] = ACTIONS(1543),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1543),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1543),
    [sym_duet_dev_type_receiver] = ACTIONS(1543),
    [sym_duet_dev_type_security_system] = ACTIONS(1543),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1543),
    [sym_duet_dev_type_settop_box] = ACTIONS(1543),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1543),
    [sym_duet_dev_type_switcher] = ACTIONS(1543),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1543),
    [sym_duet_dev_type_tv] = ACTIONS(1543),
    [sym_duet_dev_type_utility] = ACTIONS(1543),
    [sym_duet_dev_type_vcr] = ACTIONS(1543),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1543),
    [sym_duet_dev_type_video_processor] = ACTIONS(1543),
    [sym_duet_dev_type_video_projector] = ACTIONS(1543),
    [sym_duet_dev_type_video_wall] = ACTIONS(1543),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1543),
    [sym_duet_dev_type_weather] = ACTIONS(1543),
    [sym_duet_dev_not_polled] = ACTIONS(1543),
    [sym_duet_dev_polled] = ACTIONS(1543),
    [sym_clkmgr_mode_network] = ACTIONS(1543),
    [sym_clkmgr_mode_standalone] = ACTIONS(1543),
    [sym_normal_standby] = ACTIONS(1543),
    [sym_normal_wake] = ACTIONS(1543),
    [sym_ip_multicast_ttl_option] = ACTIONS(1543),
    [sym_ip_ttl_subnet] = ACTIONS(1543),
    [sym_ip_ttl_site] = ACTIONS(1543),
    [sym_ip_ttl_region] = ACTIONS(1543),
    [sym_ip_ttl_continent] = ACTIONS(1543),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1543),
    [sym_ip_nodelay_on] = ACTIONS(1543),
    [sym_ip_nodelay_off] = ACTIONS(1543),
    [sym_amx_error] = ACTIONS(1543),
    [sym_amx_warning] = ACTIONS(1543),
    [sym_amx_info] = ACTIONS(1543),
    [sym_amx_debug] = ACTIONS(1543),
    [sym_netlinx_login_success] = ACTIONS(1543),
    [sym_netlinx_login_fail] = ACTIONS(1543),
    [sym_netlinx_logout] = ACTIONS(1543),
    [sym_null_str] = ACTIONS(1543),
    [sym_smtp_address] = ACTIONS(1543),
    [sym_smtp_port_number] = ACTIONS(1543),
    [sym_smtp_username] = ACTIONS(1543),
    [sym_smtp_password] = ACTIONS(1543),
    [sym_smtp_require_tls] = ACTIONS(1543),
    [sym_smtp_from] = ACTIONS(1543),
    [sym_smtp_tls_true] = ACTIONS(1543),
    [sym_smtp_tls_false] = ACTIONS(1543),
    [sym_listview_on_row_select_event] = ACTIONS(1543),
    [sym_data_structure_unknown] = ACTIONS(1543),
    [sym_data_structure_datafeed] = ACTIONS(1543),
    [sym_data_structure_datarecordset] = ACTIONS(1543),
    [sym_data_structure_datarecord] = ACTIONS(1543),
    [sym_data_structure_datafield] = ACTIONS(1543),
    [sym_data_max_content_field_count] = ACTIONS(1543),
    [sym_data_max_metadata_field_count] = ACTIONS(1543),
    [sym_data_max_id_length] = ACTIONS(1543),
    [sym_data_max_name_length] = ACTIONS(1543),
    [sym_data_max_description_length] = ACTIONS(1543),
    [sym_data_max_type_length] = ACTIONS(1543),
    [sym_data_max_format_length] = ACTIONS(1543),
    [sym_data_max_source_length] = ACTIONS(1543),
    [sym_data_max_value_length] = ACTIONS(1543),
    [sym_data_max_label_length] = ACTIONS(1543),
    [sym_data_max_url_length] = ACTIONS(1543),
    [sym_max_last_login_info_length] = ACTIONS(1543),
    [sym_data_type_unknown] = ACTIONS(1543),
    [sym_data_type_string] = ACTIONS(1543),
    [sym_data_type_datetime] = ACTIONS(1543),
    [sym_data_type_date] = ACTIONS(1543),
    [sym_data_type_time] = ACTIONS(1543),
    [sym_data_type_image] = ACTIONS(1543),
    [sym_data_format_url] = ACTIONS(1543),
    [sym_data_format_phone] = ACTIONS(1543),
    [sym_data_format_email] = ACTIONS(1543),
    [sym_data_format_iso8601] = ACTIONS(1543),
    [sym_auditlog_priv] = ACTIONS(1543),
    [sym_configuration_priv] = ACTIONS(1543),
    [sym_deviceconfig_priv] = ACTIONS(1543),
    [sym_ftp_priv] = ACTIONS(1543),
    [sym_http_priv] = ACTIONS(1543),
    [sym_networkconfig_priv] = ACTIONS(1543),
    [sym_programport_priv] = ACTIONS(1543),
    [sym_remoteui_priv] = ACTIONS(1543),
    [sym_securitycontrol_priv] = ACTIONS(1543),
    [sym_softwaremanagement_priv] = ACTIONS(1543),
    [sym_terminal_priv] = ACTIONS(1543),
    [sym_tpadmin_priv] = ACTIONS(1543),
    [sym_user1_priv] = ACTIONS(1543),
    [sym_user2_priv] = ACTIONS(1543),
    [sym_user3_priv] = ACTIONS(1543),
    [sym_user4_priv] = ACTIONS(1543),
    [sym_usermanagement_priv] = ACTIONS(1543),
    [sym_valid_account] = ACTIONS(1543),
    [sym_err_name_invalid] = ACTIONS(1543),
    [sym_err_password_invalid] = ACTIONS(1543),
    [sym_err_invalid_account] = ACTIONS(1543),
    [sym_err_no_authorization] = ACTIONS(1543),
    [sym_err_invalid_parameter] = ACTIONS(1543),
    [sym_err_user_locked_out] = ACTIONS(1543),
    [sym_err_user_account_expired] = ACTIONS(1543),
    [sym_err_general_error] = ACTIONS(1543),
    [sym_err_authentication_error] = ACTIONS(1543),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1543),
    [sym_tls_validate_certificate] = ACTIONS(1543),
    [sym_tdata] = ACTIONS(1543),
    [sym_tchannel] = ACTIONS(1543),
    [sym_tlevel] = ACTIONS(1543),
    [sym_tbutton] = ACTIONS(1543),
    [sym_ttimeline] = ACTIONS(1543),
    [sym_tcustom] = ACTIONS(1543),
    [sym_url_struct] = ACTIONS(1543),
    [sym_dns_struct] = ACTIONS(1543),
    [sym_ip_address_struct] = ACTIONS(1543),
    [sym_dev_info_struct] = ACTIONS(1543),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1543),
    [sym_data_feed] = ACTIONS(1543),
    [sym_data_field] = ACTIONS(1543),
    [sym_data_record] = ACTIONS(1543),
    [sym_wc_data_feed] = ACTIONS(1543),
    [sym_wc_data_field] = ACTIONS(1543),
    [sym_wc_data_record] = ACTIONS(1543),
    [sym_last_login_info] = ACTIONS(1543),
    [sym____reserved___] = ACTIONS(1543),
    [sym_date] = ACTIONS(1543),
    [sym_day] = ACTIONS(1543),
    [sym_time] = ACTIONS(1543),
    [sym_ldate] = ACTIONS(1543),
    [sym_push_device] = ACTIONS(1543),
    [sym_push_channel] = ACTIONS(1543),
    [sym_push_devchan] = ACTIONS(1543),
    [sym_release_device] = ACTIONS(1543),
    [sym_release_channel] = ACTIONS(1543),
    [sym_release_devchan] = ACTIONS(1543),
    [sym_master_slot] = ACTIONS(1543),
    [sym_get_pulse_time] = ACTIONS(1543),
    [sym_get_timer] = ACTIONS(1543),
    [sym_master_sn] = ACTIONS(1543),
    [sym_system_number] = ACTIONS(1543),
    [sym_button] = ACTIONS(1543),
    [sym_dv_channel] = ACTIONS(1543),
    [sym_data] = ACTIONS(1543),
    [sym_level] = ACTIONS(1543),
    [sym_timeline] = ACTIONS(1543),
    [sym_channel] = ACTIONS(1543),
    [sym_custom] = ACTIONS(1543),
    [sym_atoi] = ACTIONS(1543),
    [sym_atol] = ACTIONS(1543),
    [sym_atof] = ACTIONS(1543),
    [sym_itoa] = ACTIONS(1543),
    [sym_itohex] = ACTIONS(1543),
    [sym_hextoi] = ACTIONS(1543),
    [sym_ftoa] = ACTIONS(1543),
    [sym_find_string] = ACTIONS(1543),
    [sym_left_string] = ACTIONS(1543),
    [sym_right_string] = ACTIONS(1543),
    [sym_mid_string] = ACTIONS(1543),
    [sym_remove_string] = ACTIONS(1543),
    [sym_get_buffer_string] = ACTIONS(1543),
    [sym_get_buffer_char] = ACTIONS(1543),
    [sym_get_multi_buffer_string] = ACTIONS(1543),
    [sym_length_string] = ACTIONS(1543),
    [sym_max_length_string] = ACTIONS(1543),
    [sym_set_length_string] = ACTIONS(1543),
    [sym_compare_string] = ACTIONS(1543),
    [sym_upper_string] = ACTIONS(1543),
    [sym_lower_string] = ACTIONS(1543),
    [sym_get_last] = ACTIONS(1543),
    [sym_date_to_day] = ACTIONS(1543),
    [sym_date_to_month] = ACTIONS(1543),
    [sym_date_to_year] = ACTIONS(1543),
    [sym_time_to_hour] = ACTIONS(1543),
    [sym_time_to_minute] = ACTIONS(1543),
    [sym_time_to_second] = ACTIONS(1543),
    [sym_day_of_week] = ACTIONS(1543),
    [sym_set_pulse_time] = ACTIONS(1543),
    [sym_set_timer] = ACTIONS(1543),
    [sym_file_open] = ACTIONS(1543),
    [sym_file_close] = ACTIONS(1543),
    [sym_file_read] = ACTIONS(1543),
    [sym_file_read_line] = ACTIONS(1543),
    [sym_file_write] = ACTIONS(1543),
    [sym_file_write_line] = ACTIONS(1543),
    [sym_file_copy] = ACTIONS(1543),
    [sym_file_delete] = ACTIONS(1543),
    [sym_file_rename] = ACTIONS(1543),
    [sym_file_dir] = ACTIONS(1543),
    [sym_file_getdir] = ACTIONS(1543),
    [sym_file_setdir] = ACTIONS(1543),
    [sym_file_create_dir] = ACTIONS(1543),
    [sym_file_remove_dir] = ACTIONS(1543),
    [sym_file_seek] = ACTIONS(1543),
    [sym_length_array] = ACTIONS(1543),
    [sym_max_length_array] = ACTIONS(1543),
    [sym_set_length_array] = ACTIONS(1543),
    [sym_ip_client_open] = ACTIONS(1543),
    [sym_ip_client_close] = ACTIONS(1543),
    [sym_ip_server_open] = ACTIONS(1543),
    [sym_ip_server_close] = ACTIONS(1543),
    [sym_ip_bound_client_open] = ACTIONS(1543),
    [sym_ip_mc_server_open] = ACTIONS(1543),
    [sym_ip_set_option] = ACTIONS(1543),
    [sym_ssh_client_open] = ACTIONS(1543),
    [sym_ssh_client_close] = ACTIONS(1543),
    [sym_tls_client_open] = ACTIONS(1543),
    [sym_tls_client_close] = ACTIONS(1543),
    [sym_do_push] = ACTIONS(1543),
    [sym_do_push_timed] = ACTIONS(1543),
    [sym_do_release] = ACTIONS(1543),
    [sym_device_id] = ACTIONS(1543),
    [sym_device_info] = ACTIONS(1543),
    [sym_device_id_string] = ACTIONS(1543),
    [sym_device_standby] = ACTIONS(1543),
    [sym_device_wake] = ACTIONS(1543),
    [sym_timeline_create] = ACTIONS(1543),
    [sym_timeline_kill] = ACTIONS(1543),
    [sym_timeline_reload] = ACTIONS(1543),
    [sym_timeline_pause] = ACTIONS(1543),
    [sym_timeline_restart] = ACTIONS(1543),
    [sym_timeline_set] = ACTIONS(1543),
    [sym_timeline_get] = ACTIONS(1543),
    [sym_timeline_active] = ACTIONS(1543),
    [sym_timeline_dynamic_id] = ACTIONS(1543),
    [sym_variable_to_string] = ACTIONS(1543),
    [sym_string_to_variable] = ACTIONS(1543),
    [sym_length_variable_to_string] = ACTIONS(1543),
    [sym_type_cast] = ACTIONS(1543),
    [sym_variable_to_xml] = ACTIONS(1543),
    [sym_xml_to_variable] = ACTIONS(1543),
    [sym_length_variable_to_xml] = ACTIONS(1543),
    [sym_redirect_string] = ACTIONS(1543),
    [sym_random_number] = ACTIONS(1543),
    [sym_get_unique_id] = ACTIONS(1543),
    [sym_get_system_number] = ACTIONS(1543),
    [sym_set_system_number] = ACTIONS(1543),
    [sym_get_serial_number] = ACTIONS(1543),
    [sym_get_master_build] = ACTIONS(1543),
    [sym_set_validation_code] = ACTIONS(1543),
    [sym_reboot] = ACTIONS(1543),
    [sym_load_duet_module] = ACTIONS(1543),
    [sym_unload_duet_module] = ACTIONS(1543),
    [sym_duet_mem_size_set] = ACTIONS(1543),
    [sym_duet_mem_size_get] = ACTIONS(1543),
    [sym_dynamic_application_device] = ACTIONS(1543),
    [sym_dynamic_polled_port] = ACTIONS(1543),
    [sym_static_port_binding] = ACTIONS(1543),
    [sym_static_ip_binding] = ACTIONS(1543),
    [sym_get_dns_list] = ACTIONS(1543),
    [sym_set_dns_list] = ACTIONS(1543),
    [sym_get_ip_address] = ACTIONS(1543),
    [sym_set_ip_address] = ACTIONS(1543),
    [sym_get_url_list] = ACTIONS(1543),
    [sym_add_url_entry] = ACTIONS(1543),
    [sym_delete_url_entry] = ACTIONS(1543),
    [sym_abs_value] = ACTIONS(1543),
    [sym_max_value] = ACTIONS(1543),
    [sym_min_value] = ACTIONS(1543),
    [sym_exp_value] = ACTIONS(1543),
    [sym_log_value] = ACTIONS(1543),
    [sym_log10_value] = ACTIONS(1543),
    [sym_power_value] = ACTIONS(1543),
    [sym_sqrt_value] = ACTIONS(1543),
    [sym_raw_be] = ACTIONS(1543),
    [sym_raw_le] = ACTIONS(1543),
    [sym_internal_queue_size_set] = ACTIONS(1543),
    [sym_internal_queue_size_get] = ACTIONS(1543),
    [sym_internal_threshold_set] = ACTIONS(1543),
    [sym_internal_threshold_get] = ACTIONS(1543),
    [sym_rebuilt_event] = ACTIONS(1543),
    [sym_do_custom_event] = ACTIONS(1543),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1543),
    [sym_clkmgr_set_clk_source] = ACTIONS(1543),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1543),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1543),
    [sym_clkmgr_get_timezone] = ACTIONS(1543),
    [sym_clkmgr_set_timezone] = ACTIONS(1543),
    [sym_clkmgr_get_resync_period] = ACTIONS(1543),
    [sym_clkmgr_set_resync_period] = ACTIONS(1543),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1543),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1543),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1543),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1543),
    [sym_clkmgr_get_timeservers] = ACTIONS(1543),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1543),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1543),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1543),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1543),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1543),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1543),
    [sym_set_log_level] = ACTIONS(1543),
    [sym_get_log_level] = ACTIONS(1543),
    [sym_amx_log] = ACTIONS(1543),
    [sym_smtp_server_config_set] = ACTIONS(1543),
    [sym_smtp_server_config_get] = ACTIONS(1543),
    [sym_smtp_send] = ACTIONS(1543),
    [sym_validate_netlinx_account] = ACTIONS(1543),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1543),
    [sym_authenticate_certificate] = ACTIONS(1543),
    [sym_audit_netlinx_session_event] = ACTIONS(1543),
    [sym_audit_netlinx_generic_event] = ACTIONS(1543),
    [sym_get_available_flash_disk_space] = ACTIONS(1543),
    [sym_get_max_flash_disk_space] = ACTIONS(1543),
    [sym_data_create_feed] = ACTIONS(1543),
    [sym_data_delete_feed] = ACTIONS(1543),
    [sym_data_publish_feed] = ACTIONS(1543),
    [sym_data_get_published_feed] = ACTIONS(1543),
    [sym_data_add_record] = ACTIONS(1543),
    [sym_data_get_event_record] = ACTIONS(1543),
    [sym__wc_data_create_feed] = ACTIONS(1543),
    [sym__wc_data_add_record] = ACTIONS(1543),
    [sym__wc_data_get_event_record] = ACTIONS(1543),
    [sym_format] = ACTIONS(1543),
    [sym_astro_clock] = ACTIONS(1543),
    [sym_set_outdoor_temperature] = ACTIONS(1543),
    [sym_set_virtual_level_count] = ACTIONS(1543),
    [sym_set_virtual_channel_count] = ACTIONS(1543),
    [sym_set_virtual_port_count] = ACTIONS(1543),
    [sym__wc_encode_int] = ACTIONS(1543),
    [sym__wc_decode_int] = ACTIONS(1543),
    [sym__wc_int] = ACTIONS(1543),
    [sym__wc_to_ch_int] = ACTIONS(1543),
    [sym__ch_to_wc_int] = ACTIONS(1543),
    [sym__wc_find_string_int] = ACTIONS(1543),
    [sym__wc_left_string_int] = ACTIONS(1543),
    [sym__wc_lower_string_int] = ACTIONS(1543),
    [sym__wc_mid_string_int] = ACTIONS(1543),
    [sym__wc_remove_string_int] = ACTIONS(1543),
    [sym__wc_right_string_int] = ACTIONS(1543),
    [sym__wc_upper_string_int] = ACTIONS(1543),
    [sym__wc_compare_string_int] = ACTIONS(1543),
    [sym__wc_get_buffer_char_int] = ACTIONS(1543),
    [sym__wc_get_buffer_string_int] = ACTIONS(1543),
    [sym__wc_concat_string_int] = ACTIONS(1543),
    [sym__wc_file_open_int] = ACTIONS(1543),
    [sym__wc_file_close_int] = ACTIONS(1543),
    [sym__wc_file_read_int] = ACTIONS(1543),
    [sym__wc_file_read_line_int] = ACTIONS(1543),
    [sym__wc_file_write_int] = ACTIONS(1543),
    [sym__wc_file_write_line_int] = ACTIONS(1543),
    [anon_sym_EQ] = ACTIONS(1543),
    [aux_sym_preproc_include_token1] = ACTIONS(1543),
    [aux_sym_preproc_define_token1] = ACTIONS(1545),
    [aux_sym_preproc_warn_token1] = ACTIONS(1545),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1545),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1545),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1545),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1545),
    [aux_sym_preproc_else_token1] = ACTIONS(1545),
    [anon_sym_LBRACK] = ACTIONS(1545),
    [anon_sym_LBRACE] = ACTIONS(1545),
    [anon_sym_COLON] = ACTIONS(1545),
    [anon_sym_LPAREN] = ACTIONS(1543),
    [anon_sym_SEMI] = ACTIONS(1545),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1545),
    [anon_sym_DASH] = ACTIONS(1543),
    [anon_sym_PLUS] = ACTIONS(1543),
    [anon_sym_STAR] = ACTIONS(1545),
    [anon_sym_SLASH] = ACTIONS(1543),
    [anon_sym_PERCENT] = ACTIONS(1545),
    [anon_sym_PIPE_PIPE] = ACTIONS(1545),
    [anon_sym_AMP_AMP] = ACTIONS(1545),
    [anon_sym_PIPE] = ACTIONS(1543),
    [anon_sym_CARET] = ACTIONS(1545),
    [anon_sym_AMP] = ACTIONS(1543),
    [anon_sym_EQ_EQ] = ACTIONS(1545),
    [anon_sym_BANG_EQ] = ACTIONS(1545),
    [anon_sym_LT_GT] = ACTIONS(1545),
    [anon_sym_GT] = ACTIONS(1543),
    [anon_sym_GT_EQ] = ACTIONS(1545),
    [anon_sym_LT_EQ] = ACTIONS(1545),
    [anon_sym_LT] = ACTIONS(1543),
    [anon_sym_LT_LT] = ACTIONS(1545),
    [anon_sym_GT_GT] = ACTIONS(1545),
    [anon_sym_DASH_DASH] = ACTIONS(1545),
    [anon_sym_PLUS_PLUS] = ACTIONS(1545),
    [anon_sym_DOT] = ACTIONS(1543),
    [anon_sym_DQUOTE] = ACTIONS(1545),
    [anon_sym_SQUOTE] = ACTIONS(1545),
    [sym_number_literal] = ACTIONS(1545),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1545),
  },
  [STATE(106)] = {
    [sym_identifier] = ACTIONS(1547),
    [sym_program_name_keyword] = ACTIONS(1547),
    [sym_module_name_keyword] = ACTIONS(1547),
    [sym_define_device_keyword] = ACTIONS(1547),
    [sym_define_combine_keyword] = ACTIONS(1547),
    [sym_define_constant_keyword] = ACTIONS(1547),
    [sym_define_type_keyword] = ACTIONS(1547),
    [sym_define_variable_keyword] = ACTIONS(1547),
    [sym_define_system_variable_keyword] = ACTIONS(1547),
    [sym_define_connect_level_keyword] = ACTIONS(1547),
    [sym_define_latching_keyword] = ACTIONS(1547),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1547),
    [sym_define_toggling_keyword] = ACTIONS(1547),
    [sym_define_call_keyword] = ACTIONS(1547),
    [sym_define_function_keyword] = ACTIONS(1547),
    [sym_define_library_function_keyword] = ACTIONS(1547),
    [sym_define_start_keyword] = ACTIONS(1547),
    [sym_define_event_keyword] = ACTIONS(1547),
    [sym_define_program_keyword] = ACTIONS(1547),
    [sym_define_module_keyword] = ACTIONS(1547),
    [sym_char_keyword] = ACTIONS(1547),
    [sym_widechar_keyword] = ACTIONS(1547),
    [sym_integer_keyword] = ACTIONS(1547),
    [sym_sinteger_keyword] = ACTIONS(1547),
    [sym_long_keyword] = ACTIONS(1547),
    [sym_slong_keyword] = ACTIONS(1547),
    [sym_float_keyword] = ACTIONS(1547),
    [sym_double_keyword] = ACTIONS(1547),
    [sym_variant_keyword] = ACTIONS(1547),
    [sym_variantarray_keyword] = ACTIONS(1547),
    [sym_local_var_keyword] = ACTIONS(1547),
    [sym_stack_var_keyword] = ACTIONS(1547),
    [sym_constant_keyword] = ACTIONS(1547),
    [sym_volatile_keyword] = ACTIONS(1547),
    [sym_non_volatile_keyword] = ACTIONS(1547),
    [sym_persistent_keyword] = ACTIONS(1547),
    [sym_dev_keyword] = ACTIONS(1547),
    [sym_devlev_keyword] = ACTIONS(1547),
    [sym_devchan_keyword] = ACTIONS(1547),
    [sym_if_keyword] = ACTIONS(1547),
    [sym_else_keyword] = ACTIONS(1547),
    [sym_switch_keyword] = ACTIONS(1547),
    [sym_case_keyword] = ACTIONS(1547),
    [sym_default_keyword] = ACTIONS(1547),
    [sym_while_keyword] = ACTIONS(1547),
    [sym_for_keyword] = ACTIONS(1547),
    [sym_break_keyword] = ACTIONS(1547),
    [sym_continue_keyword] = ACTIONS(1547),
    [sym_return_keyword] = ACTIONS(1547),
    [sym_select_keyword] = ACTIONS(1547),
    [sym_active_keyword] = ACTIONS(1547),
    [sym_struct_keyword] = ACTIONS(1547),
    [sym_structure_keyword] = ACTIONS(1547),
    [sym_button_event_keyword] = ACTIONS(1547),
    [sym_channel_event_keyword] = ACTIONS(1547),
    [sym_level_event_keyword] = ACTIONS(1547),
    [sym_data_event_keyword] = ACTIONS(1547),
    [sym_timeline_event_keyword] = ACTIONS(1547),
    [sym_custom_event_keyword] = ACTIONS(1547),
    [sym_push_keyword] = ACTIONS(1547),
    [sym_release_keyword] = ACTIONS(1547),
    [aux_sym_on_keyword_token1] = ACTIONS(1547),
    [aux_sym_off_keyword_token1] = ACTIONS(1547),
    [sym_send_level_keyword] = ACTIONS(1547),
    [sym_send_string_keyword] = ACTIONS(1547),
    [sym_send_command_keyword] = ACTIONS(1547),
    [sym_clear_buffer_keyword] = ACTIONS(1547),
    [sym_create_buffer_keyword] = ACTIONS(1547),
    [sym_create_multi_buffer_keyword] = ACTIONS(1547),
    [sym_call_keyword] = ACTIONS(1547),
    [sym_system_call_keyword] = ACTIONS(1547),
    [sym_devchan_to_keyword] = ACTIONS(1547),
    [sym_devchan_min_to_keyword] = ACTIONS(1547),
    [sym_devchan_total_off_keyword] = ACTIONS(1547),
    [sym_devchan_pulse_keyword] = ACTIONS(1547),
    [sym_wait_keyword] = ACTIONS(1547),
    [sym_cancel_wait_keyword] = ACTIONS(1547),
    [sym_wait_until_keyword] = ACTIONS(1547),
    [sym_cancel_wait_until_keyword] = ACTIONS(1547),
    [sym_cancel_all_wait_keyword] = ACTIONS(1547),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1547),
    [sym_and] = ACTIONS(1547),
    [sym_or] = ACTIONS(1547),
    [sym_not] = ACTIONS(1547),
    [sym_xor] = ACTIONS(1547),
    [sym_band] = ACTIONS(1547),
    [sym_bor] = ACTIONS(1547),
    [sym_bxor] = ACTIONS(1547),
    [sym_bnot] = ACTIONS(1547),
    [sym_lshift] = ACTIONS(1547),
    [sym_rshift] = ACTIONS(1547),
    [sym___file__] = ACTIONS(1547),
    [sym___date__] = ACTIONS(1547),
    [sym___ldate__] = ACTIONS(1547),
    [sym___time__] = ACTIONS(1547),
    [sym___line__] = ACTIONS(1547),
    [sym___name__] = ACTIONS(1547),
    [sym___netlinx__] = ACTIONS(1547),
    [sym_netlinx_axi_version] = ACTIONS(1547),
    [sym_url_flg_tcp] = ACTIONS(1547),
    [sym_url_flg_accinfopresent] = ACTIONS(1547),
    [sym_url_flg_temp] = ACTIONS(1547),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1547),
    [sym_url_flg_stat_mask] = ACTIONS(1547),
    [sym_url_flg_stat_lookup] = ACTIONS(1547),
    [sym_url_flg_stat_connecting] = ACTIONS(1547),
    [sym_url_flg_stat_waiting] = ACTIONS(1547),
    [sym_url_flg_stat_connected] = ACTIONS(1547),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1547),
    [sym_first_virtual_device] = ACTIONS(1547),
    [sym_first_local_port] = ACTIONS(1547),
    [sym_dynamic_virtual_device] = ACTIONS(1547),
    [sym_timeline_once] = ACTIONS(1547),
    [sym_timeline_repeat] = ACTIONS(1547),
    [sym_timeline_absolute] = ACTIONS(1547),
    [sym_timeline_relative] = ACTIONS(1547),
    [sym_do_push_timed_infinite] = ACTIONS(1547),
    [sym_source_type_no_address] = ACTIONS(1547),
    [sym_source_type_neuron_id] = ACTIONS(1547),
    [sym_source_type_ip_address] = ACTIONS(1547),
    [sym_source_type_axlink] = ACTIONS(1547),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1547),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1547),
    [sym_source_type_ip_socket_address] = ACTIONS(1547),
    [sym_source_type_rs232] = ACTIONS(1547),
    [sym_source_type_internal] = ACTIONS(1547),
    [sym_source_type_mac_address] = ACTIONS(1547),
    [sym_source_type_ipv4_port] = ACTIONS(1547),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1547),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1547),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1547),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1547),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1547),
    [sym_true] = ACTIONS(1547),
    [sym_false] = ACTIONS(1547),
    [sym_file_read_only] = ACTIONS(1547),
    [sym_file_rw_new] = ACTIONS(1547),
    [sym_file_rw_append] = ACTIONS(1547),
    [sym_ip_tcp] = ACTIONS(1547),
    [sym_ip_udp] = ACTIONS(1547),
    [sym_ip_udp_2way] = ACTIONS(1547),
    [sym_xml_encode_types] = ACTIONS(1547),
    [sym_xml_encode_char_as_list] = ACTIONS(1547),
    [sym_xml_encode_le] = ACTIONS(1547),
    [sym_xml_decode_types] = ACTIONS(1547),
    [sym_xml_decode_no_preserve] = ACTIONS(1547),
    [sym_duet_dev_type_display_device] = ACTIONS(1547),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1547),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1547),
    [sym_duet_dev_type_io_device] = ACTIONS(1547),
    [sym_duet_dev_type_relay_device] = ACTIONS(1547),
    [sym_duet_dev_type_ups] = ACTIONS(1547),
    [sym_duet_dev_type_amplifier] = ACTIONS(1547),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1547),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1547),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1547),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1547),
    [sym_duet_dev_type_camera] = ACTIONS(1547),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1547),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1547),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1547),
    [sym_duet_dev_type_dss] = ACTIONS(1547),
    [sym_duet_dev_type_dvr] = ACTIONS(1547),
    [sym_duet_dev_type_disc_device] = ACTIONS(1547),
    [sym_duet_dev_type_document_camera] = ACTIONS(1547),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1547),
    [sym_duet_dev_type_hvac] = ACTIONS(1547),
    [sym_duet_dev_type_keypad] = ACTIONS(1547),
    [sym_duet_dev_type_light] = ACTIONS(1547),
    [sym_duet_dev_type_monitor] = ACTIONS(1547),
    [sym_duet_dev_type_motor] = ACTIONS(1547),
    [sym_duet_dev_type_multi_window] = ACTIONS(1547),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1547),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1547),
    [sym_duet_dev_type_receiver] = ACTIONS(1547),
    [sym_duet_dev_type_security_system] = ACTIONS(1547),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1547),
    [sym_duet_dev_type_settop_box] = ACTIONS(1547),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1547),
    [sym_duet_dev_type_switcher] = ACTIONS(1547),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1547),
    [sym_duet_dev_type_tv] = ACTIONS(1547),
    [sym_duet_dev_type_utility] = ACTIONS(1547),
    [sym_duet_dev_type_vcr] = ACTIONS(1547),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1547),
    [sym_duet_dev_type_video_processor] = ACTIONS(1547),
    [sym_duet_dev_type_video_projector] = ACTIONS(1547),
    [sym_duet_dev_type_video_wall] = ACTIONS(1547),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1547),
    [sym_duet_dev_type_weather] = ACTIONS(1547),
    [sym_duet_dev_not_polled] = ACTIONS(1547),
    [sym_duet_dev_polled] = ACTIONS(1547),
    [sym_clkmgr_mode_network] = ACTIONS(1547),
    [sym_clkmgr_mode_standalone] = ACTIONS(1547),
    [sym_normal_standby] = ACTIONS(1547),
    [sym_normal_wake] = ACTIONS(1547),
    [sym_ip_multicast_ttl_option] = ACTIONS(1547),
    [sym_ip_ttl_subnet] = ACTIONS(1547),
    [sym_ip_ttl_site] = ACTIONS(1547),
    [sym_ip_ttl_region] = ACTIONS(1547),
    [sym_ip_ttl_continent] = ACTIONS(1547),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1547),
    [sym_ip_nodelay_on] = ACTIONS(1547),
    [sym_ip_nodelay_off] = ACTIONS(1547),
    [sym_amx_error] = ACTIONS(1547),
    [sym_amx_warning] = ACTIONS(1547),
    [sym_amx_info] = ACTIONS(1547),
    [sym_amx_debug] = ACTIONS(1547),
    [sym_netlinx_login_success] = ACTIONS(1547),
    [sym_netlinx_login_fail] = ACTIONS(1547),
    [sym_netlinx_logout] = ACTIONS(1547),
    [sym_null_str] = ACTIONS(1547),
    [sym_smtp_address] = ACTIONS(1547),
    [sym_smtp_port_number] = ACTIONS(1547),
    [sym_smtp_username] = ACTIONS(1547),
    [sym_smtp_password] = ACTIONS(1547),
    [sym_smtp_require_tls] = ACTIONS(1547),
    [sym_smtp_from] = ACTIONS(1547),
    [sym_smtp_tls_true] = ACTIONS(1547),
    [sym_smtp_tls_false] = ACTIONS(1547),
    [sym_listview_on_row_select_event] = ACTIONS(1547),
    [sym_data_structure_unknown] = ACTIONS(1547),
    [sym_data_structure_datafeed] = ACTIONS(1547),
    [sym_data_structure_datarecordset] = ACTIONS(1547),
    [sym_data_structure_datarecord] = ACTIONS(1547),
    [sym_data_structure_datafield] = ACTIONS(1547),
    [sym_data_max_content_field_count] = ACTIONS(1547),
    [sym_data_max_metadata_field_count] = ACTIONS(1547),
    [sym_data_max_id_length] = ACTIONS(1547),
    [sym_data_max_name_length] = ACTIONS(1547),
    [sym_data_max_description_length] = ACTIONS(1547),
    [sym_data_max_type_length] = ACTIONS(1547),
    [sym_data_max_format_length] = ACTIONS(1547),
    [sym_data_max_source_length] = ACTIONS(1547),
    [sym_data_max_value_length] = ACTIONS(1547),
    [sym_data_max_label_length] = ACTIONS(1547),
    [sym_data_max_url_length] = ACTIONS(1547),
    [sym_max_last_login_info_length] = ACTIONS(1547),
    [sym_data_type_unknown] = ACTIONS(1547),
    [sym_data_type_string] = ACTIONS(1547),
    [sym_data_type_datetime] = ACTIONS(1547),
    [sym_data_type_date] = ACTIONS(1547),
    [sym_data_type_time] = ACTIONS(1547),
    [sym_data_type_image] = ACTIONS(1547),
    [sym_data_format_url] = ACTIONS(1547),
    [sym_data_format_phone] = ACTIONS(1547),
    [sym_data_format_email] = ACTIONS(1547),
    [sym_data_format_iso8601] = ACTIONS(1547),
    [sym_auditlog_priv] = ACTIONS(1547),
    [sym_configuration_priv] = ACTIONS(1547),
    [sym_deviceconfig_priv] = ACTIONS(1547),
    [sym_ftp_priv] = ACTIONS(1547),
    [sym_http_priv] = ACTIONS(1547),
    [sym_networkconfig_priv] = ACTIONS(1547),
    [sym_programport_priv] = ACTIONS(1547),
    [sym_remoteui_priv] = ACTIONS(1547),
    [sym_securitycontrol_priv] = ACTIONS(1547),
    [sym_softwaremanagement_priv] = ACTIONS(1547),
    [sym_terminal_priv] = ACTIONS(1547),
    [sym_tpadmin_priv] = ACTIONS(1547),
    [sym_user1_priv] = ACTIONS(1547),
    [sym_user2_priv] = ACTIONS(1547),
    [sym_user3_priv] = ACTIONS(1547),
    [sym_user4_priv] = ACTIONS(1547),
    [sym_usermanagement_priv] = ACTIONS(1547),
    [sym_valid_account] = ACTIONS(1547),
    [sym_err_name_invalid] = ACTIONS(1547),
    [sym_err_password_invalid] = ACTIONS(1547),
    [sym_err_invalid_account] = ACTIONS(1547),
    [sym_err_no_authorization] = ACTIONS(1547),
    [sym_err_invalid_parameter] = ACTIONS(1547),
    [sym_err_user_locked_out] = ACTIONS(1547),
    [sym_err_user_account_expired] = ACTIONS(1547),
    [sym_err_general_error] = ACTIONS(1547),
    [sym_err_authentication_error] = ACTIONS(1547),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1547),
    [sym_tls_validate_certificate] = ACTIONS(1547),
    [sym_tdata] = ACTIONS(1547),
    [sym_tchannel] = ACTIONS(1547),
    [sym_tlevel] = ACTIONS(1547),
    [sym_tbutton] = ACTIONS(1547),
    [sym_ttimeline] = ACTIONS(1547),
    [sym_tcustom] = ACTIONS(1547),
    [sym_url_struct] = ACTIONS(1547),
    [sym_dns_struct] = ACTIONS(1547),
    [sym_ip_address_struct] = ACTIONS(1547),
    [sym_dev_info_struct] = ACTIONS(1547),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1547),
    [sym_data_feed] = ACTIONS(1547),
    [sym_data_field] = ACTIONS(1547),
    [sym_data_record] = ACTIONS(1547),
    [sym_wc_data_feed] = ACTIONS(1547),
    [sym_wc_data_field] = ACTIONS(1547),
    [sym_wc_data_record] = ACTIONS(1547),
    [sym_last_login_info] = ACTIONS(1547),
    [sym____reserved___] = ACTIONS(1547),
    [sym_date] = ACTIONS(1547),
    [sym_day] = ACTIONS(1547),
    [sym_time] = ACTIONS(1547),
    [sym_ldate] = ACTIONS(1547),
    [sym_push_device] = ACTIONS(1547),
    [sym_push_channel] = ACTIONS(1547),
    [sym_push_devchan] = ACTIONS(1547),
    [sym_release_device] = ACTIONS(1547),
    [sym_release_channel] = ACTIONS(1547),
    [sym_release_devchan] = ACTIONS(1547),
    [sym_master_slot] = ACTIONS(1547),
    [sym_get_pulse_time] = ACTIONS(1547),
    [sym_get_timer] = ACTIONS(1547),
    [sym_master_sn] = ACTIONS(1547),
    [sym_system_number] = ACTIONS(1547),
    [sym_button] = ACTIONS(1547),
    [sym_dv_channel] = ACTIONS(1547),
    [sym_data] = ACTIONS(1547),
    [sym_level] = ACTIONS(1547),
    [sym_timeline] = ACTIONS(1547),
    [sym_channel] = ACTIONS(1547),
    [sym_custom] = ACTIONS(1547),
    [sym_atoi] = ACTIONS(1547),
    [sym_atol] = ACTIONS(1547),
    [sym_atof] = ACTIONS(1547),
    [sym_itoa] = ACTIONS(1547),
    [sym_itohex] = ACTIONS(1547),
    [sym_hextoi] = ACTIONS(1547),
    [sym_ftoa] = ACTIONS(1547),
    [sym_find_string] = ACTIONS(1547),
    [sym_left_string] = ACTIONS(1547),
    [sym_right_string] = ACTIONS(1547),
    [sym_mid_string] = ACTIONS(1547),
    [sym_remove_string] = ACTIONS(1547),
    [sym_get_buffer_string] = ACTIONS(1547),
    [sym_get_buffer_char] = ACTIONS(1547),
    [sym_get_multi_buffer_string] = ACTIONS(1547),
    [sym_length_string] = ACTIONS(1547),
    [sym_max_length_string] = ACTIONS(1547),
    [sym_set_length_string] = ACTIONS(1547),
    [sym_compare_string] = ACTIONS(1547),
    [sym_upper_string] = ACTIONS(1547),
    [sym_lower_string] = ACTIONS(1547),
    [sym_get_last] = ACTIONS(1547),
    [sym_date_to_day] = ACTIONS(1547),
    [sym_date_to_month] = ACTIONS(1547),
    [sym_date_to_year] = ACTIONS(1547),
    [sym_time_to_hour] = ACTIONS(1547),
    [sym_time_to_minute] = ACTIONS(1547),
    [sym_time_to_second] = ACTIONS(1547),
    [sym_day_of_week] = ACTIONS(1547),
    [sym_set_pulse_time] = ACTIONS(1547),
    [sym_set_timer] = ACTIONS(1547),
    [sym_file_open] = ACTIONS(1547),
    [sym_file_close] = ACTIONS(1547),
    [sym_file_read] = ACTIONS(1547),
    [sym_file_read_line] = ACTIONS(1547),
    [sym_file_write] = ACTIONS(1547),
    [sym_file_write_line] = ACTIONS(1547),
    [sym_file_copy] = ACTIONS(1547),
    [sym_file_delete] = ACTIONS(1547),
    [sym_file_rename] = ACTIONS(1547),
    [sym_file_dir] = ACTIONS(1547),
    [sym_file_getdir] = ACTIONS(1547),
    [sym_file_setdir] = ACTIONS(1547),
    [sym_file_create_dir] = ACTIONS(1547),
    [sym_file_remove_dir] = ACTIONS(1547),
    [sym_file_seek] = ACTIONS(1547),
    [sym_length_array] = ACTIONS(1547),
    [sym_max_length_array] = ACTIONS(1547),
    [sym_set_length_array] = ACTIONS(1547),
    [sym_ip_client_open] = ACTIONS(1547),
    [sym_ip_client_close] = ACTIONS(1547),
    [sym_ip_server_open] = ACTIONS(1547),
    [sym_ip_server_close] = ACTIONS(1547),
    [sym_ip_bound_client_open] = ACTIONS(1547),
    [sym_ip_mc_server_open] = ACTIONS(1547),
    [sym_ip_set_option] = ACTIONS(1547),
    [sym_ssh_client_open] = ACTIONS(1547),
    [sym_ssh_client_close] = ACTIONS(1547),
    [sym_tls_client_open] = ACTIONS(1547),
    [sym_tls_client_close] = ACTIONS(1547),
    [sym_do_push] = ACTIONS(1547),
    [sym_do_push_timed] = ACTIONS(1547),
    [sym_do_release] = ACTIONS(1547),
    [sym_device_id] = ACTIONS(1547),
    [sym_device_info] = ACTIONS(1547),
    [sym_device_id_string] = ACTIONS(1547),
    [sym_device_standby] = ACTIONS(1547),
    [sym_device_wake] = ACTIONS(1547),
    [sym_timeline_create] = ACTIONS(1547),
    [sym_timeline_kill] = ACTIONS(1547),
    [sym_timeline_reload] = ACTIONS(1547),
    [sym_timeline_pause] = ACTIONS(1547),
    [sym_timeline_restart] = ACTIONS(1547),
    [sym_timeline_set] = ACTIONS(1547),
    [sym_timeline_get] = ACTIONS(1547),
    [sym_timeline_active] = ACTIONS(1547),
    [sym_timeline_dynamic_id] = ACTIONS(1547),
    [sym_variable_to_string] = ACTIONS(1547),
    [sym_string_to_variable] = ACTIONS(1547),
    [sym_length_variable_to_string] = ACTIONS(1547),
    [sym_type_cast] = ACTIONS(1547),
    [sym_variable_to_xml] = ACTIONS(1547),
    [sym_xml_to_variable] = ACTIONS(1547),
    [sym_length_variable_to_xml] = ACTIONS(1547),
    [sym_redirect_string] = ACTIONS(1547),
    [sym_random_number] = ACTIONS(1547),
    [sym_get_unique_id] = ACTIONS(1547),
    [sym_get_system_number] = ACTIONS(1547),
    [sym_set_system_number] = ACTIONS(1547),
    [sym_get_serial_number] = ACTIONS(1547),
    [sym_get_master_build] = ACTIONS(1547),
    [sym_set_validation_code] = ACTIONS(1547),
    [sym_reboot] = ACTIONS(1547),
    [sym_load_duet_module] = ACTIONS(1547),
    [sym_unload_duet_module] = ACTIONS(1547),
    [sym_duet_mem_size_set] = ACTIONS(1547),
    [sym_duet_mem_size_get] = ACTIONS(1547),
    [sym_dynamic_application_device] = ACTIONS(1547),
    [sym_dynamic_polled_port] = ACTIONS(1547),
    [sym_static_port_binding] = ACTIONS(1547),
    [sym_static_ip_binding] = ACTIONS(1547),
    [sym_get_dns_list] = ACTIONS(1547),
    [sym_set_dns_list] = ACTIONS(1547),
    [sym_get_ip_address] = ACTIONS(1547),
    [sym_set_ip_address] = ACTIONS(1547),
    [sym_get_url_list] = ACTIONS(1547),
    [sym_add_url_entry] = ACTIONS(1547),
    [sym_delete_url_entry] = ACTIONS(1547),
    [sym_abs_value] = ACTIONS(1547),
    [sym_max_value] = ACTIONS(1547),
    [sym_min_value] = ACTIONS(1547),
    [sym_exp_value] = ACTIONS(1547),
    [sym_log_value] = ACTIONS(1547),
    [sym_log10_value] = ACTIONS(1547),
    [sym_power_value] = ACTIONS(1547),
    [sym_sqrt_value] = ACTIONS(1547),
    [sym_raw_be] = ACTIONS(1547),
    [sym_raw_le] = ACTIONS(1547),
    [sym_internal_queue_size_set] = ACTIONS(1547),
    [sym_internal_queue_size_get] = ACTIONS(1547),
    [sym_internal_threshold_set] = ACTIONS(1547),
    [sym_internal_threshold_get] = ACTIONS(1547),
    [sym_rebuilt_event] = ACTIONS(1547),
    [sym_do_custom_event] = ACTIONS(1547),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1547),
    [sym_clkmgr_set_clk_source] = ACTIONS(1547),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1547),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1547),
    [sym_clkmgr_get_timezone] = ACTIONS(1547),
    [sym_clkmgr_set_timezone] = ACTIONS(1547),
    [sym_clkmgr_get_resync_period] = ACTIONS(1547),
    [sym_clkmgr_set_resync_period] = ACTIONS(1547),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1547),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1547),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1547),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1547),
    [sym_clkmgr_get_timeservers] = ACTIONS(1547),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1547),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1547),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1547),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1547),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1547),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1547),
    [sym_set_log_level] = ACTIONS(1547),
    [sym_get_log_level] = ACTIONS(1547),
    [sym_amx_log] = ACTIONS(1547),
    [sym_smtp_server_config_set] = ACTIONS(1547),
    [sym_smtp_server_config_get] = ACTIONS(1547),
    [sym_smtp_send] = ACTIONS(1547),
    [sym_validate_netlinx_account] = ACTIONS(1547),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1547),
    [sym_authenticate_certificate] = ACTIONS(1547),
    [sym_audit_netlinx_session_event] = ACTIONS(1547),
    [sym_audit_netlinx_generic_event] = ACTIONS(1547),
    [sym_get_available_flash_disk_space] = ACTIONS(1547),
    [sym_get_max_flash_disk_space] = ACTIONS(1547),
    [sym_data_create_feed] = ACTIONS(1547),
    [sym_data_delete_feed] = ACTIONS(1547),
    [sym_data_publish_feed] = ACTIONS(1547),
    [sym_data_get_published_feed] = ACTIONS(1547),
    [sym_data_add_record] = ACTIONS(1547),
    [sym_data_get_event_record] = ACTIONS(1547),
    [sym__wc_data_create_feed] = ACTIONS(1547),
    [sym__wc_data_add_record] = ACTIONS(1547),
    [sym__wc_data_get_event_record] = ACTIONS(1547),
    [sym_format] = ACTIONS(1547),
    [sym_astro_clock] = ACTIONS(1547),
    [sym_set_outdoor_temperature] = ACTIONS(1547),
    [sym_set_virtual_level_count] = ACTIONS(1547),
    [sym_set_virtual_channel_count] = ACTIONS(1547),
    [sym_set_virtual_port_count] = ACTIONS(1547),
    [sym__wc_encode_int] = ACTIONS(1547),
    [sym__wc_decode_int] = ACTIONS(1547),
    [sym__wc_int] = ACTIONS(1547),
    [sym__wc_to_ch_int] = ACTIONS(1547),
    [sym__ch_to_wc_int] = ACTIONS(1547),
    [sym__wc_find_string_int] = ACTIONS(1547),
    [sym__wc_left_string_int] = ACTIONS(1547),
    [sym__wc_lower_string_int] = ACTIONS(1547),
    [sym__wc_mid_string_int] = ACTIONS(1547),
    [sym__wc_remove_string_int] = ACTIONS(1547),
    [sym__wc_right_string_int] = ACTIONS(1547),
    [sym__wc_upper_string_int] = ACTIONS(1547),
    [sym__wc_compare_string_int] = ACTIONS(1547),
    [sym__wc_get_buffer_char_int] = ACTIONS(1547),
    [sym__wc_get_buffer_string_int] = ACTIONS(1547),
    [sym__wc_concat_string_int] = ACTIONS(1547),
    [sym__wc_file_open_int] = ACTIONS(1547),
    [sym__wc_file_close_int] = ACTIONS(1547),
    [sym__wc_file_read_int] = ACTIONS(1547),
    [sym__wc_file_read_line_int] = ACTIONS(1547),
    [sym__wc_file_write_int] = ACTIONS(1547),
    [sym__wc_file_write_line_int] = ACTIONS(1547),
    [anon_sym_EQ] = ACTIONS(1547),
    [aux_sym_preproc_include_token1] = ACTIONS(1547),
    [aux_sym_preproc_define_token1] = ACTIONS(1549),
    [aux_sym_preproc_warn_token1] = ACTIONS(1549),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1549),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1549),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1549),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1549),
    [aux_sym_preproc_else_token1] = ACTIONS(1549),
    [anon_sym_LBRACK] = ACTIONS(1549),
    [anon_sym_LBRACE] = ACTIONS(1549),
    [anon_sym_COLON] = ACTIONS(1549),
    [anon_sym_LPAREN] = ACTIONS(1547),
    [anon_sym_SEMI] = ACTIONS(1549),
    [anon_sym_BANG] = ACTIONS(1547),
    [anon_sym_TILDE] = ACTIONS(1549),
    [anon_sym_DASH] = ACTIONS(1547),
    [anon_sym_PLUS] = ACTIONS(1547),
    [anon_sym_STAR] = ACTIONS(1549),
    [anon_sym_SLASH] = ACTIONS(1547),
    [anon_sym_PERCENT] = ACTIONS(1549),
    [anon_sym_PIPE_PIPE] = ACTIONS(1549),
    [anon_sym_AMP_AMP] = ACTIONS(1549),
    [anon_sym_PIPE] = ACTIONS(1547),
    [anon_sym_CARET] = ACTIONS(1549),
    [anon_sym_AMP] = ACTIONS(1547),
    [anon_sym_EQ_EQ] = ACTIONS(1549),
    [anon_sym_BANG_EQ] = ACTIONS(1549),
    [anon_sym_LT_GT] = ACTIONS(1549),
    [anon_sym_GT] = ACTIONS(1547),
    [anon_sym_GT_EQ] = ACTIONS(1549),
    [anon_sym_LT_EQ] = ACTIONS(1549),
    [anon_sym_LT] = ACTIONS(1547),
    [anon_sym_LT_LT] = ACTIONS(1549),
    [anon_sym_GT_GT] = ACTIONS(1549),
    [anon_sym_DASH_DASH] = ACTIONS(1549),
    [anon_sym_PLUS_PLUS] = ACTIONS(1549),
    [anon_sym_DOT] = ACTIONS(1547),
    [anon_sym_DQUOTE] = ACTIONS(1549),
    [anon_sym_SQUOTE] = ACTIONS(1549),
    [sym_number_literal] = ACTIONS(1549),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1549),
  },
  [STATE(107)] = {
    [sym_argument_list] = STATE(131),
    [aux_sym_subscript_expression_repeat1] = STATE(93),
    [ts_builtin_sym_end] = ACTIONS(1415),
    [sym_identifier] = ACTIONS(1413),
    [sym_program_name_keyword] = ACTIONS(1413),
    [sym_module_name_keyword] = ACTIONS(1413),
    [sym_define_device_keyword] = ACTIONS(1413),
    [sym_define_combine_keyword] = ACTIONS(1413),
    [sym_define_constant_keyword] = ACTIONS(1413),
    [sym_define_type_keyword] = ACTIONS(1413),
    [sym_define_variable_keyword] = ACTIONS(1413),
    [sym_define_system_variable_keyword] = ACTIONS(1413),
    [sym_define_connect_level_keyword] = ACTIONS(1413),
    [sym_define_latching_keyword] = ACTIONS(1413),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1413),
    [sym_define_toggling_keyword] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_start_keyword] = ACTIONS(1413),
    [sym_define_event_keyword] = ACTIONS(1413),
    [sym_define_program_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [sym_button_event_keyword] = ACTIONS(1413),
    [sym_channel_event_keyword] = ACTIONS(1413),
    [sym_level_event_keyword] = ACTIONS(1413),
    [sym_data_event_keyword] = ACTIONS(1413),
    [sym_timeline_event_keyword] = ACTIONS(1413),
    [sym_custom_event_keyword] = ACTIONS(1413),
    [sym_push_keyword] = ACTIONS(1413),
    [sym_release_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1413),
    [sym_rshift] = ACTIONS(1413),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1429),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1433),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_STAR] = ACTIONS(1439),
    [anon_sym_SLASH] = ACTIONS(1441),
    [anon_sym_PERCENT] = ACTIONS(1439),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1415),
    [anon_sym_GT_GT] = ACTIONS(1415),
    [anon_sym_DASH_DASH] = ACTIONS(1457),
    [anon_sym_PLUS_PLUS] = ACTIONS(1457),
    [anon_sym_DOT] = ACTIONS(1459),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(108)] = {
    [sym_identifier] = ACTIONS(1551),
    [sym_program_name_keyword] = ACTIONS(1551),
    [sym_module_name_keyword] = ACTIONS(1551),
    [sym_define_device_keyword] = ACTIONS(1551),
    [sym_define_combine_keyword] = ACTIONS(1551),
    [sym_define_constant_keyword] = ACTIONS(1551),
    [sym_define_type_keyword] = ACTIONS(1551),
    [sym_define_variable_keyword] = ACTIONS(1551),
    [sym_define_system_variable_keyword] = ACTIONS(1551),
    [sym_define_connect_level_keyword] = ACTIONS(1551),
    [sym_define_latching_keyword] = ACTIONS(1551),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1551),
    [sym_define_toggling_keyword] = ACTIONS(1551),
    [sym_define_call_keyword] = ACTIONS(1551),
    [sym_define_function_keyword] = ACTIONS(1551),
    [sym_define_library_function_keyword] = ACTIONS(1551),
    [sym_define_start_keyword] = ACTIONS(1551),
    [sym_define_event_keyword] = ACTIONS(1551),
    [sym_define_program_keyword] = ACTIONS(1551),
    [sym_define_module_keyword] = ACTIONS(1551),
    [sym_char_keyword] = ACTIONS(1551),
    [sym_widechar_keyword] = ACTIONS(1551),
    [sym_integer_keyword] = ACTIONS(1551),
    [sym_sinteger_keyword] = ACTIONS(1551),
    [sym_long_keyword] = ACTIONS(1551),
    [sym_slong_keyword] = ACTIONS(1551),
    [sym_float_keyword] = ACTIONS(1551),
    [sym_double_keyword] = ACTIONS(1551),
    [sym_variant_keyword] = ACTIONS(1551),
    [sym_variantarray_keyword] = ACTIONS(1551),
    [sym_local_var_keyword] = ACTIONS(1551),
    [sym_stack_var_keyword] = ACTIONS(1551),
    [sym_constant_keyword] = ACTIONS(1551),
    [sym_volatile_keyword] = ACTIONS(1551),
    [sym_non_volatile_keyword] = ACTIONS(1551),
    [sym_persistent_keyword] = ACTIONS(1551),
    [sym_dev_keyword] = ACTIONS(1551),
    [sym_devlev_keyword] = ACTIONS(1551),
    [sym_devchan_keyword] = ACTIONS(1551),
    [sym_if_keyword] = ACTIONS(1551),
    [sym_else_keyword] = ACTIONS(1551),
    [sym_switch_keyword] = ACTIONS(1551),
    [sym_case_keyword] = ACTIONS(1551),
    [sym_default_keyword] = ACTIONS(1551),
    [sym_while_keyword] = ACTIONS(1551),
    [sym_for_keyword] = ACTIONS(1551),
    [sym_break_keyword] = ACTIONS(1551),
    [sym_continue_keyword] = ACTIONS(1551),
    [sym_return_keyword] = ACTIONS(1551),
    [sym_select_keyword] = ACTIONS(1551),
    [sym_active_keyword] = ACTIONS(1551),
    [sym_struct_keyword] = ACTIONS(1551),
    [sym_structure_keyword] = ACTIONS(1551),
    [sym_button_event_keyword] = ACTIONS(1551),
    [sym_channel_event_keyword] = ACTIONS(1551),
    [sym_level_event_keyword] = ACTIONS(1551),
    [sym_data_event_keyword] = ACTIONS(1551),
    [sym_timeline_event_keyword] = ACTIONS(1551),
    [sym_custom_event_keyword] = ACTIONS(1551),
    [sym_push_keyword] = ACTIONS(1551),
    [sym_release_keyword] = ACTIONS(1551),
    [aux_sym_on_keyword_token1] = ACTIONS(1551),
    [aux_sym_off_keyword_token1] = ACTIONS(1551),
    [sym_send_level_keyword] = ACTIONS(1551),
    [sym_send_string_keyword] = ACTIONS(1551),
    [sym_send_command_keyword] = ACTIONS(1551),
    [sym_clear_buffer_keyword] = ACTIONS(1551),
    [sym_create_buffer_keyword] = ACTIONS(1551),
    [sym_create_multi_buffer_keyword] = ACTIONS(1551),
    [sym_call_keyword] = ACTIONS(1551),
    [sym_system_call_keyword] = ACTIONS(1551),
    [sym_devchan_to_keyword] = ACTIONS(1551),
    [sym_devchan_min_to_keyword] = ACTIONS(1551),
    [sym_devchan_total_off_keyword] = ACTIONS(1551),
    [sym_devchan_pulse_keyword] = ACTIONS(1551),
    [sym_wait_keyword] = ACTIONS(1551),
    [sym_cancel_wait_keyword] = ACTIONS(1551),
    [sym_wait_until_keyword] = ACTIONS(1551),
    [sym_cancel_wait_until_keyword] = ACTIONS(1551),
    [sym_cancel_all_wait_keyword] = ACTIONS(1551),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1551),
    [sym_and] = ACTIONS(1551),
    [sym_or] = ACTIONS(1551),
    [sym_not] = ACTIONS(1551),
    [sym_xor] = ACTIONS(1551),
    [sym_band] = ACTIONS(1551),
    [sym_bor] = ACTIONS(1551),
    [sym_bxor] = ACTIONS(1551),
    [sym_bnot] = ACTIONS(1551),
    [sym_lshift] = ACTIONS(1551),
    [sym_rshift] = ACTIONS(1551),
    [sym___file__] = ACTIONS(1551),
    [sym___date__] = ACTIONS(1551),
    [sym___ldate__] = ACTIONS(1551),
    [sym___time__] = ACTIONS(1551),
    [sym___line__] = ACTIONS(1551),
    [sym___name__] = ACTIONS(1551),
    [sym___netlinx__] = ACTIONS(1551),
    [sym_netlinx_axi_version] = ACTIONS(1551),
    [sym_url_flg_tcp] = ACTIONS(1551),
    [sym_url_flg_accinfopresent] = ACTIONS(1551),
    [sym_url_flg_temp] = ACTIONS(1551),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1551),
    [sym_url_flg_stat_mask] = ACTIONS(1551),
    [sym_url_flg_stat_lookup] = ACTIONS(1551),
    [sym_url_flg_stat_connecting] = ACTIONS(1551),
    [sym_url_flg_stat_waiting] = ACTIONS(1551),
    [sym_url_flg_stat_connected] = ACTIONS(1551),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1551),
    [sym_first_virtual_device] = ACTIONS(1551),
    [sym_first_local_port] = ACTIONS(1551),
    [sym_dynamic_virtual_device] = ACTIONS(1551),
    [sym_timeline_once] = ACTIONS(1551),
    [sym_timeline_repeat] = ACTIONS(1551),
    [sym_timeline_absolute] = ACTIONS(1551),
    [sym_timeline_relative] = ACTIONS(1551),
    [sym_do_push_timed_infinite] = ACTIONS(1551),
    [sym_source_type_no_address] = ACTIONS(1551),
    [sym_source_type_neuron_id] = ACTIONS(1551),
    [sym_source_type_ip_address] = ACTIONS(1551),
    [sym_source_type_axlink] = ACTIONS(1551),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1551),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1551),
    [sym_source_type_ip_socket_address] = ACTIONS(1551),
    [sym_source_type_rs232] = ACTIONS(1551),
    [sym_source_type_internal] = ACTIONS(1551),
    [sym_source_type_mac_address] = ACTIONS(1551),
    [sym_source_type_ipv4_port] = ACTIONS(1551),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1551),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1551),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1551),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1551),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1551),
    [sym_true] = ACTIONS(1551),
    [sym_false] = ACTIONS(1551),
    [sym_file_read_only] = ACTIONS(1551),
    [sym_file_rw_new] = ACTIONS(1551),
    [sym_file_rw_append] = ACTIONS(1551),
    [sym_ip_tcp] = ACTIONS(1551),
    [sym_ip_udp] = ACTIONS(1551),
    [sym_ip_udp_2way] = ACTIONS(1551),
    [sym_xml_encode_types] = ACTIONS(1551),
    [sym_xml_encode_char_as_list] = ACTIONS(1551),
    [sym_xml_encode_le] = ACTIONS(1551),
    [sym_xml_decode_types] = ACTIONS(1551),
    [sym_xml_decode_no_preserve] = ACTIONS(1551),
    [sym_duet_dev_type_display_device] = ACTIONS(1551),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1551),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1551),
    [sym_duet_dev_type_io_device] = ACTIONS(1551),
    [sym_duet_dev_type_relay_device] = ACTIONS(1551),
    [sym_duet_dev_type_ups] = ACTIONS(1551),
    [sym_duet_dev_type_amplifier] = ACTIONS(1551),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1551),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1551),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1551),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1551),
    [sym_duet_dev_type_camera] = ACTIONS(1551),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1551),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1551),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1551),
    [sym_duet_dev_type_dss] = ACTIONS(1551),
    [sym_duet_dev_type_dvr] = ACTIONS(1551),
    [sym_duet_dev_type_disc_device] = ACTIONS(1551),
    [sym_duet_dev_type_document_camera] = ACTIONS(1551),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1551),
    [sym_duet_dev_type_hvac] = ACTIONS(1551),
    [sym_duet_dev_type_keypad] = ACTIONS(1551),
    [sym_duet_dev_type_light] = ACTIONS(1551),
    [sym_duet_dev_type_monitor] = ACTIONS(1551),
    [sym_duet_dev_type_motor] = ACTIONS(1551),
    [sym_duet_dev_type_multi_window] = ACTIONS(1551),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1551),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1551),
    [sym_duet_dev_type_receiver] = ACTIONS(1551),
    [sym_duet_dev_type_security_system] = ACTIONS(1551),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1551),
    [sym_duet_dev_type_settop_box] = ACTIONS(1551),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1551),
    [sym_duet_dev_type_switcher] = ACTIONS(1551),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1551),
    [sym_duet_dev_type_tv] = ACTIONS(1551),
    [sym_duet_dev_type_utility] = ACTIONS(1551),
    [sym_duet_dev_type_vcr] = ACTIONS(1551),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1551),
    [sym_duet_dev_type_video_processor] = ACTIONS(1551),
    [sym_duet_dev_type_video_projector] = ACTIONS(1551),
    [sym_duet_dev_type_video_wall] = ACTIONS(1551),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1551),
    [sym_duet_dev_type_weather] = ACTIONS(1551),
    [sym_duet_dev_not_polled] = ACTIONS(1551),
    [sym_duet_dev_polled] = ACTIONS(1551),
    [sym_clkmgr_mode_network] = ACTIONS(1551),
    [sym_clkmgr_mode_standalone] = ACTIONS(1551),
    [sym_normal_standby] = ACTIONS(1551),
    [sym_normal_wake] = ACTIONS(1551),
    [sym_ip_multicast_ttl_option] = ACTIONS(1551),
    [sym_ip_ttl_subnet] = ACTIONS(1551),
    [sym_ip_ttl_site] = ACTIONS(1551),
    [sym_ip_ttl_region] = ACTIONS(1551),
    [sym_ip_ttl_continent] = ACTIONS(1551),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1551),
    [sym_ip_nodelay_on] = ACTIONS(1551),
    [sym_ip_nodelay_off] = ACTIONS(1551),
    [sym_amx_error] = ACTIONS(1551),
    [sym_amx_warning] = ACTIONS(1551),
    [sym_amx_info] = ACTIONS(1551),
    [sym_amx_debug] = ACTIONS(1551),
    [sym_netlinx_login_success] = ACTIONS(1551),
    [sym_netlinx_login_fail] = ACTIONS(1551),
    [sym_netlinx_logout] = ACTIONS(1551),
    [sym_null_str] = ACTIONS(1551),
    [sym_smtp_address] = ACTIONS(1551),
    [sym_smtp_port_number] = ACTIONS(1551),
    [sym_smtp_username] = ACTIONS(1551),
    [sym_smtp_password] = ACTIONS(1551),
    [sym_smtp_require_tls] = ACTIONS(1551),
    [sym_smtp_from] = ACTIONS(1551),
    [sym_smtp_tls_true] = ACTIONS(1551),
    [sym_smtp_tls_false] = ACTIONS(1551),
    [sym_listview_on_row_select_event] = ACTIONS(1551),
    [sym_data_structure_unknown] = ACTIONS(1551),
    [sym_data_structure_datafeed] = ACTIONS(1551),
    [sym_data_structure_datarecordset] = ACTIONS(1551),
    [sym_data_structure_datarecord] = ACTIONS(1551),
    [sym_data_structure_datafield] = ACTIONS(1551),
    [sym_data_max_content_field_count] = ACTIONS(1551),
    [sym_data_max_metadata_field_count] = ACTIONS(1551),
    [sym_data_max_id_length] = ACTIONS(1551),
    [sym_data_max_name_length] = ACTIONS(1551),
    [sym_data_max_description_length] = ACTIONS(1551),
    [sym_data_max_type_length] = ACTIONS(1551),
    [sym_data_max_format_length] = ACTIONS(1551),
    [sym_data_max_source_length] = ACTIONS(1551),
    [sym_data_max_value_length] = ACTIONS(1551),
    [sym_data_max_label_length] = ACTIONS(1551),
    [sym_data_max_url_length] = ACTIONS(1551),
    [sym_max_last_login_info_length] = ACTIONS(1551),
    [sym_data_type_unknown] = ACTIONS(1551),
    [sym_data_type_string] = ACTIONS(1551),
    [sym_data_type_datetime] = ACTIONS(1551),
    [sym_data_type_date] = ACTIONS(1551),
    [sym_data_type_time] = ACTIONS(1551),
    [sym_data_type_image] = ACTIONS(1551),
    [sym_data_format_url] = ACTIONS(1551),
    [sym_data_format_phone] = ACTIONS(1551),
    [sym_data_format_email] = ACTIONS(1551),
    [sym_data_format_iso8601] = ACTIONS(1551),
    [sym_auditlog_priv] = ACTIONS(1551),
    [sym_configuration_priv] = ACTIONS(1551),
    [sym_deviceconfig_priv] = ACTIONS(1551),
    [sym_ftp_priv] = ACTIONS(1551),
    [sym_http_priv] = ACTIONS(1551),
    [sym_networkconfig_priv] = ACTIONS(1551),
    [sym_programport_priv] = ACTIONS(1551),
    [sym_remoteui_priv] = ACTIONS(1551),
    [sym_securitycontrol_priv] = ACTIONS(1551),
    [sym_softwaremanagement_priv] = ACTIONS(1551),
    [sym_terminal_priv] = ACTIONS(1551),
    [sym_tpadmin_priv] = ACTIONS(1551),
    [sym_user1_priv] = ACTIONS(1551),
    [sym_user2_priv] = ACTIONS(1551),
    [sym_user3_priv] = ACTIONS(1551),
    [sym_user4_priv] = ACTIONS(1551),
    [sym_usermanagement_priv] = ACTIONS(1551),
    [sym_valid_account] = ACTIONS(1551),
    [sym_err_name_invalid] = ACTIONS(1551),
    [sym_err_password_invalid] = ACTIONS(1551),
    [sym_err_invalid_account] = ACTIONS(1551),
    [sym_err_no_authorization] = ACTIONS(1551),
    [sym_err_invalid_parameter] = ACTIONS(1551),
    [sym_err_user_locked_out] = ACTIONS(1551),
    [sym_err_user_account_expired] = ACTIONS(1551),
    [sym_err_general_error] = ACTIONS(1551),
    [sym_err_authentication_error] = ACTIONS(1551),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1551),
    [sym_tls_validate_certificate] = ACTIONS(1551),
    [sym_tdata] = ACTIONS(1551),
    [sym_tchannel] = ACTIONS(1551),
    [sym_tlevel] = ACTIONS(1551),
    [sym_tbutton] = ACTIONS(1551),
    [sym_ttimeline] = ACTIONS(1551),
    [sym_tcustom] = ACTIONS(1551),
    [sym_url_struct] = ACTIONS(1551),
    [sym_dns_struct] = ACTIONS(1551),
    [sym_ip_address_struct] = ACTIONS(1551),
    [sym_dev_info_struct] = ACTIONS(1551),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1551),
    [sym_data_feed] = ACTIONS(1551),
    [sym_data_field] = ACTIONS(1551),
    [sym_data_record] = ACTIONS(1551),
    [sym_wc_data_feed] = ACTIONS(1551),
    [sym_wc_data_field] = ACTIONS(1551),
    [sym_wc_data_record] = ACTIONS(1551),
    [sym_last_login_info] = ACTIONS(1551),
    [sym____reserved___] = ACTIONS(1551),
    [sym_date] = ACTIONS(1551),
    [sym_day] = ACTIONS(1551),
    [sym_time] = ACTIONS(1551),
    [sym_ldate] = ACTIONS(1551),
    [sym_push_device] = ACTIONS(1551),
    [sym_push_channel] = ACTIONS(1551),
    [sym_push_devchan] = ACTIONS(1551),
    [sym_release_device] = ACTIONS(1551),
    [sym_release_channel] = ACTIONS(1551),
    [sym_release_devchan] = ACTIONS(1551),
    [sym_master_slot] = ACTIONS(1551),
    [sym_get_pulse_time] = ACTIONS(1551),
    [sym_get_timer] = ACTIONS(1551),
    [sym_master_sn] = ACTIONS(1551),
    [sym_system_number] = ACTIONS(1551),
    [sym_button] = ACTIONS(1551),
    [sym_dv_channel] = ACTIONS(1551),
    [sym_data] = ACTIONS(1551),
    [sym_level] = ACTIONS(1551),
    [sym_timeline] = ACTIONS(1551),
    [sym_channel] = ACTIONS(1551),
    [sym_custom] = ACTIONS(1551),
    [sym_atoi] = ACTIONS(1551),
    [sym_atol] = ACTIONS(1551),
    [sym_atof] = ACTIONS(1551),
    [sym_itoa] = ACTIONS(1551),
    [sym_itohex] = ACTIONS(1551),
    [sym_hextoi] = ACTIONS(1551),
    [sym_ftoa] = ACTIONS(1551),
    [sym_find_string] = ACTIONS(1551),
    [sym_left_string] = ACTIONS(1551),
    [sym_right_string] = ACTIONS(1551),
    [sym_mid_string] = ACTIONS(1551),
    [sym_remove_string] = ACTIONS(1551),
    [sym_get_buffer_string] = ACTIONS(1551),
    [sym_get_buffer_char] = ACTIONS(1551),
    [sym_get_multi_buffer_string] = ACTIONS(1551),
    [sym_length_string] = ACTIONS(1551),
    [sym_max_length_string] = ACTIONS(1551),
    [sym_set_length_string] = ACTIONS(1551),
    [sym_compare_string] = ACTIONS(1551),
    [sym_upper_string] = ACTIONS(1551),
    [sym_lower_string] = ACTIONS(1551),
    [sym_get_last] = ACTIONS(1551),
    [sym_date_to_day] = ACTIONS(1551),
    [sym_date_to_month] = ACTIONS(1551),
    [sym_date_to_year] = ACTIONS(1551),
    [sym_time_to_hour] = ACTIONS(1551),
    [sym_time_to_minute] = ACTIONS(1551),
    [sym_time_to_second] = ACTIONS(1551),
    [sym_day_of_week] = ACTIONS(1551),
    [sym_set_pulse_time] = ACTIONS(1551),
    [sym_set_timer] = ACTIONS(1551),
    [sym_file_open] = ACTIONS(1551),
    [sym_file_close] = ACTIONS(1551),
    [sym_file_read] = ACTIONS(1551),
    [sym_file_read_line] = ACTIONS(1551),
    [sym_file_write] = ACTIONS(1551),
    [sym_file_write_line] = ACTIONS(1551),
    [sym_file_copy] = ACTIONS(1551),
    [sym_file_delete] = ACTIONS(1551),
    [sym_file_rename] = ACTIONS(1551),
    [sym_file_dir] = ACTIONS(1551),
    [sym_file_getdir] = ACTIONS(1551),
    [sym_file_setdir] = ACTIONS(1551),
    [sym_file_create_dir] = ACTIONS(1551),
    [sym_file_remove_dir] = ACTIONS(1551),
    [sym_file_seek] = ACTIONS(1551),
    [sym_length_array] = ACTIONS(1551),
    [sym_max_length_array] = ACTIONS(1551),
    [sym_set_length_array] = ACTIONS(1551),
    [sym_ip_client_open] = ACTIONS(1551),
    [sym_ip_client_close] = ACTIONS(1551),
    [sym_ip_server_open] = ACTIONS(1551),
    [sym_ip_server_close] = ACTIONS(1551),
    [sym_ip_bound_client_open] = ACTIONS(1551),
    [sym_ip_mc_server_open] = ACTIONS(1551),
    [sym_ip_set_option] = ACTIONS(1551),
    [sym_ssh_client_open] = ACTIONS(1551),
    [sym_ssh_client_close] = ACTIONS(1551),
    [sym_tls_client_open] = ACTIONS(1551),
    [sym_tls_client_close] = ACTIONS(1551),
    [sym_do_push] = ACTIONS(1551),
    [sym_do_push_timed] = ACTIONS(1551),
    [sym_do_release] = ACTIONS(1551),
    [sym_device_id] = ACTIONS(1551),
    [sym_device_info] = ACTIONS(1551),
    [sym_device_id_string] = ACTIONS(1551),
    [sym_device_standby] = ACTIONS(1551),
    [sym_device_wake] = ACTIONS(1551),
    [sym_timeline_create] = ACTIONS(1551),
    [sym_timeline_kill] = ACTIONS(1551),
    [sym_timeline_reload] = ACTIONS(1551),
    [sym_timeline_pause] = ACTIONS(1551),
    [sym_timeline_restart] = ACTIONS(1551),
    [sym_timeline_set] = ACTIONS(1551),
    [sym_timeline_get] = ACTIONS(1551),
    [sym_timeline_active] = ACTIONS(1551),
    [sym_timeline_dynamic_id] = ACTIONS(1551),
    [sym_variable_to_string] = ACTIONS(1551),
    [sym_string_to_variable] = ACTIONS(1551),
    [sym_length_variable_to_string] = ACTIONS(1551),
    [sym_type_cast] = ACTIONS(1551),
    [sym_variable_to_xml] = ACTIONS(1551),
    [sym_xml_to_variable] = ACTIONS(1551),
    [sym_length_variable_to_xml] = ACTIONS(1551),
    [sym_redirect_string] = ACTIONS(1551),
    [sym_random_number] = ACTIONS(1551),
    [sym_get_unique_id] = ACTIONS(1551),
    [sym_get_system_number] = ACTIONS(1551),
    [sym_set_system_number] = ACTIONS(1551),
    [sym_get_serial_number] = ACTIONS(1551),
    [sym_get_master_build] = ACTIONS(1551),
    [sym_set_validation_code] = ACTIONS(1551),
    [sym_reboot] = ACTIONS(1551),
    [sym_load_duet_module] = ACTIONS(1551),
    [sym_unload_duet_module] = ACTIONS(1551),
    [sym_duet_mem_size_set] = ACTIONS(1551),
    [sym_duet_mem_size_get] = ACTIONS(1551),
    [sym_dynamic_application_device] = ACTIONS(1551),
    [sym_dynamic_polled_port] = ACTIONS(1551),
    [sym_static_port_binding] = ACTIONS(1551),
    [sym_static_ip_binding] = ACTIONS(1551),
    [sym_get_dns_list] = ACTIONS(1551),
    [sym_set_dns_list] = ACTIONS(1551),
    [sym_get_ip_address] = ACTIONS(1551),
    [sym_set_ip_address] = ACTIONS(1551),
    [sym_get_url_list] = ACTIONS(1551),
    [sym_add_url_entry] = ACTIONS(1551),
    [sym_delete_url_entry] = ACTIONS(1551),
    [sym_abs_value] = ACTIONS(1551),
    [sym_max_value] = ACTIONS(1551),
    [sym_min_value] = ACTIONS(1551),
    [sym_exp_value] = ACTIONS(1551),
    [sym_log_value] = ACTIONS(1551),
    [sym_log10_value] = ACTIONS(1551),
    [sym_power_value] = ACTIONS(1551),
    [sym_sqrt_value] = ACTIONS(1551),
    [sym_raw_be] = ACTIONS(1551),
    [sym_raw_le] = ACTIONS(1551),
    [sym_internal_queue_size_set] = ACTIONS(1551),
    [sym_internal_queue_size_get] = ACTIONS(1551),
    [sym_internal_threshold_set] = ACTIONS(1551),
    [sym_internal_threshold_get] = ACTIONS(1551),
    [sym_rebuilt_event] = ACTIONS(1551),
    [sym_do_custom_event] = ACTIONS(1551),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1551),
    [sym_clkmgr_set_clk_source] = ACTIONS(1551),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1551),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1551),
    [sym_clkmgr_get_timezone] = ACTIONS(1551),
    [sym_clkmgr_set_timezone] = ACTIONS(1551),
    [sym_clkmgr_get_resync_period] = ACTIONS(1551),
    [sym_clkmgr_set_resync_period] = ACTIONS(1551),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1551),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1551),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1551),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1551),
    [sym_clkmgr_get_timeservers] = ACTIONS(1551),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1551),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1551),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1551),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1551),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1551),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1551),
    [sym_set_log_level] = ACTIONS(1551),
    [sym_get_log_level] = ACTIONS(1551),
    [sym_amx_log] = ACTIONS(1551),
    [sym_smtp_server_config_set] = ACTIONS(1551),
    [sym_smtp_server_config_get] = ACTIONS(1551),
    [sym_smtp_send] = ACTIONS(1551),
    [sym_validate_netlinx_account] = ACTIONS(1551),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1551),
    [sym_authenticate_certificate] = ACTIONS(1551),
    [sym_audit_netlinx_session_event] = ACTIONS(1551),
    [sym_audit_netlinx_generic_event] = ACTIONS(1551),
    [sym_get_available_flash_disk_space] = ACTIONS(1551),
    [sym_get_max_flash_disk_space] = ACTIONS(1551),
    [sym_data_create_feed] = ACTIONS(1551),
    [sym_data_delete_feed] = ACTIONS(1551),
    [sym_data_publish_feed] = ACTIONS(1551),
    [sym_data_get_published_feed] = ACTIONS(1551),
    [sym_data_add_record] = ACTIONS(1551),
    [sym_data_get_event_record] = ACTIONS(1551),
    [sym__wc_data_create_feed] = ACTIONS(1551),
    [sym__wc_data_add_record] = ACTIONS(1551),
    [sym__wc_data_get_event_record] = ACTIONS(1551),
    [sym_format] = ACTIONS(1551),
    [sym_astro_clock] = ACTIONS(1551),
    [sym_set_outdoor_temperature] = ACTIONS(1551),
    [sym_set_virtual_level_count] = ACTIONS(1551),
    [sym_set_virtual_channel_count] = ACTIONS(1551),
    [sym_set_virtual_port_count] = ACTIONS(1551),
    [sym__wc_encode_int] = ACTIONS(1551),
    [sym__wc_decode_int] = ACTIONS(1551),
    [sym__wc_int] = ACTIONS(1551),
    [sym__wc_to_ch_int] = ACTIONS(1551),
    [sym__ch_to_wc_int] = ACTIONS(1551),
    [sym__wc_find_string_int] = ACTIONS(1551),
    [sym__wc_left_string_int] = ACTIONS(1551),
    [sym__wc_lower_string_int] = ACTIONS(1551),
    [sym__wc_mid_string_int] = ACTIONS(1551),
    [sym__wc_remove_string_int] = ACTIONS(1551),
    [sym__wc_right_string_int] = ACTIONS(1551),
    [sym__wc_upper_string_int] = ACTIONS(1551),
    [sym__wc_compare_string_int] = ACTIONS(1551),
    [sym__wc_get_buffer_char_int] = ACTIONS(1551),
    [sym__wc_get_buffer_string_int] = ACTIONS(1551),
    [sym__wc_concat_string_int] = ACTIONS(1551),
    [sym__wc_file_open_int] = ACTIONS(1551),
    [sym__wc_file_close_int] = ACTIONS(1551),
    [sym__wc_file_read_int] = ACTIONS(1551),
    [sym__wc_file_read_line_int] = ACTIONS(1551),
    [sym__wc_file_write_int] = ACTIONS(1551),
    [sym__wc_file_write_line_int] = ACTIONS(1551),
    [anon_sym_EQ] = ACTIONS(1551),
    [aux_sym_preproc_include_token1] = ACTIONS(1551),
    [aux_sym_preproc_define_token1] = ACTIONS(1553),
    [aux_sym_preproc_warn_token1] = ACTIONS(1553),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1553),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1553),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1553),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1553),
    [aux_sym_preproc_else_token1] = ACTIONS(1553),
    [anon_sym_LBRACK] = ACTIONS(1553),
    [anon_sym_LBRACE] = ACTIONS(1553),
    [anon_sym_COLON] = ACTIONS(1553),
    [anon_sym_LPAREN] = ACTIONS(1551),
    [anon_sym_SEMI] = ACTIONS(1553),
    [anon_sym_BANG] = ACTIONS(1551),
    [anon_sym_TILDE] = ACTIONS(1553),
    [anon_sym_DASH] = ACTIONS(1551),
    [anon_sym_PLUS] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(1553),
    [anon_sym_SLASH] = ACTIONS(1551),
    [anon_sym_PERCENT] = ACTIONS(1553),
    [anon_sym_PIPE_PIPE] = ACTIONS(1553),
    [anon_sym_AMP_AMP] = ACTIONS(1553),
    [anon_sym_PIPE] = ACTIONS(1551),
    [anon_sym_CARET] = ACTIONS(1553),
    [anon_sym_AMP] = ACTIONS(1551),
    [anon_sym_EQ_EQ] = ACTIONS(1553),
    [anon_sym_BANG_EQ] = ACTIONS(1553),
    [anon_sym_LT_GT] = ACTIONS(1553),
    [anon_sym_GT] = ACTIONS(1551),
    [anon_sym_GT_EQ] = ACTIONS(1553),
    [anon_sym_LT_EQ] = ACTIONS(1553),
    [anon_sym_LT] = ACTIONS(1551),
    [anon_sym_LT_LT] = ACTIONS(1553),
    [anon_sym_GT_GT] = ACTIONS(1553),
    [anon_sym_DASH_DASH] = ACTIONS(1553),
    [anon_sym_PLUS_PLUS] = ACTIONS(1553),
    [anon_sym_DOT] = ACTIONS(1551),
    [anon_sym_DQUOTE] = ACTIONS(1553),
    [anon_sym_SQUOTE] = ACTIONS(1553),
    [sym_number_literal] = ACTIONS(1553),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1553),
  },
  [STATE(109)] = {
    [sym_identifier] = ACTIONS(1555),
    [sym_program_name_keyword] = ACTIONS(1555),
    [sym_module_name_keyword] = ACTIONS(1555),
    [sym_define_device_keyword] = ACTIONS(1555),
    [sym_define_combine_keyword] = ACTIONS(1555),
    [sym_define_constant_keyword] = ACTIONS(1555),
    [sym_define_type_keyword] = ACTIONS(1555),
    [sym_define_variable_keyword] = ACTIONS(1555),
    [sym_define_system_variable_keyword] = ACTIONS(1555),
    [sym_define_connect_level_keyword] = ACTIONS(1555),
    [sym_define_latching_keyword] = ACTIONS(1555),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1555),
    [sym_define_toggling_keyword] = ACTIONS(1555),
    [sym_define_call_keyword] = ACTIONS(1555),
    [sym_define_function_keyword] = ACTIONS(1555),
    [sym_define_library_function_keyword] = ACTIONS(1555),
    [sym_define_start_keyword] = ACTIONS(1555),
    [sym_define_event_keyword] = ACTIONS(1555),
    [sym_define_program_keyword] = ACTIONS(1555),
    [sym_define_module_keyword] = ACTIONS(1555),
    [sym_char_keyword] = ACTIONS(1555),
    [sym_widechar_keyword] = ACTIONS(1555),
    [sym_integer_keyword] = ACTIONS(1555),
    [sym_sinteger_keyword] = ACTIONS(1555),
    [sym_long_keyword] = ACTIONS(1555),
    [sym_slong_keyword] = ACTIONS(1555),
    [sym_float_keyword] = ACTIONS(1555),
    [sym_double_keyword] = ACTIONS(1555),
    [sym_variant_keyword] = ACTIONS(1555),
    [sym_variantarray_keyword] = ACTIONS(1555),
    [sym_local_var_keyword] = ACTIONS(1555),
    [sym_stack_var_keyword] = ACTIONS(1555),
    [sym_constant_keyword] = ACTIONS(1555),
    [sym_volatile_keyword] = ACTIONS(1555),
    [sym_non_volatile_keyword] = ACTIONS(1555),
    [sym_persistent_keyword] = ACTIONS(1555),
    [sym_dev_keyword] = ACTIONS(1555),
    [sym_devlev_keyword] = ACTIONS(1555),
    [sym_devchan_keyword] = ACTIONS(1555),
    [sym_if_keyword] = ACTIONS(1555),
    [sym_else_keyword] = ACTIONS(1555),
    [sym_switch_keyword] = ACTIONS(1555),
    [sym_case_keyword] = ACTIONS(1555),
    [sym_default_keyword] = ACTIONS(1555),
    [sym_while_keyword] = ACTIONS(1555),
    [sym_for_keyword] = ACTIONS(1555),
    [sym_break_keyword] = ACTIONS(1555),
    [sym_continue_keyword] = ACTIONS(1555),
    [sym_return_keyword] = ACTIONS(1555),
    [sym_select_keyword] = ACTIONS(1555),
    [sym_active_keyword] = ACTIONS(1555),
    [sym_struct_keyword] = ACTIONS(1555),
    [sym_structure_keyword] = ACTIONS(1555),
    [sym_button_event_keyword] = ACTIONS(1555),
    [sym_channel_event_keyword] = ACTIONS(1555),
    [sym_level_event_keyword] = ACTIONS(1555),
    [sym_data_event_keyword] = ACTIONS(1555),
    [sym_timeline_event_keyword] = ACTIONS(1555),
    [sym_custom_event_keyword] = ACTIONS(1555),
    [sym_push_keyword] = ACTIONS(1555),
    [sym_release_keyword] = ACTIONS(1555),
    [aux_sym_on_keyword_token1] = ACTIONS(1555),
    [aux_sym_off_keyword_token1] = ACTIONS(1555),
    [sym_send_level_keyword] = ACTIONS(1555),
    [sym_send_string_keyword] = ACTIONS(1555),
    [sym_send_command_keyword] = ACTIONS(1555),
    [sym_clear_buffer_keyword] = ACTIONS(1555),
    [sym_create_buffer_keyword] = ACTIONS(1555),
    [sym_create_multi_buffer_keyword] = ACTIONS(1555),
    [sym_call_keyword] = ACTIONS(1555),
    [sym_system_call_keyword] = ACTIONS(1555),
    [sym_devchan_to_keyword] = ACTIONS(1555),
    [sym_devchan_min_to_keyword] = ACTIONS(1555),
    [sym_devchan_total_off_keyword] = ACTIONS(1555),
    [sym_devchan_pulse_keyword] = ACTIONS(1555),
    [sym_wait_keyword] = ACTIONS(1555),
    [sym_cancel_wait_keyword] = ACTIONS(1555),
    [sym_wait_until_keyword] = ACTIONS(1555),
    [sym_cancel_wait_until_keyword] = ACTIONS(1555),
    [sym_cancel_all_wait_keyword] = ACTIONS(1555),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1555),
    [sym_and] = ACTIONS(1555),
    [sym_or] = ACTIONS(1555),
    [sym_not] = ACTIONS(1555),
    [sym_xor] = ACTIONS(1555),
    [sym_band] = ACTIONS(1555),
    [sym_bor] = ACTIONS(1555),
    [sym_bxor] = ACTIONS(1555),
    [sym_bnot] = ACTIONS(1555),
    [sym_lshift] = ACTIONS(1555),
    [sym_rshift] = ACTIONS(1555),
    [sym___file__] = ACTIONS(1555),
    [sym___date__] = ACTIONS(1555),
    [sym___ldate__] = ACTIONS(1555),
    [sym___time__] = ACTIONS(1555),
    [sym___line__] = ACTIONS(1555),
    [sym___name__] = ACTIONS(1555),
    [sym___netlinx__] = ACTIONS(1555),
    [sym_netlinx_axi_version] = ACTIONS(1555),
    [sym_url_flg_tcp] = ACTIONS(1555),
    [sym_url_flg_accinfopresent] = ACTIONS(1555),
    [sym_url_flg_temp] = ACTIONS(1555),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1555),
    [sym_url_flg_stat_mask] = ACTIONS(1555),
    [sym_url_flg_stat_lookup] = ACTIONS(1555),
    [sym_url_flg_stat_connecting] = ACTIONS(1555),
    [sym_url_flg_stat_waiting] = ACTIONS(1555),
    [sym_url_flg_stat_connected] = ACTIONS(1555),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1555),
    [sym_first_virtual_device] = ACTIONS(1555),
    [sym_first_local_port] = ACTIONS(1555),
    [sym_dynamic_virtual_device] = ACTIONS(1555),
    [sym_timeline_once] = ACTIONS(1555),
    [sym_timeline_repeat] = ACTIONS(1555),
    [sym_timeline_absolute] = ACTIONS(1555),
    [sym_timeline_relative] = ACTIONS(1555),
    [sym_do_push_timed_infinite] = ACTIONS(1555),
    [sym_source_type_no_address] = ACTIONS(1555),
    [sym_source_type_neuron_id] = ACTIONS(1555),
    [sym_source_type_ip_address] = ACTIONS(1555),
    [sym_source_type_axlink] = ACTIONS(1555),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1555),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1555),
    [sym_source_type_ip_socket_address] = ACTIONS(1555),
    [sym_source_type_rs232] = ACTIONS(1555),
    [sym_source_type_internal] = ACTIONS(1555),
    [sym_source_type_mac_address] = ACTIONS(1555),
    [sym_source_type_ipv4_port] = ACTIONS(1555),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1555),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1555),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1555),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1555),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1555),
    [sym_true] = ACTIONS(1555),
    [sym_false] = ACTIONS(1555),
    [sym_file_read_only] = ACTIONS(1555),
    [sym_file_rw_new] = ACTIONS(1555),
    [sym_file_rw_append] = ACTIONS(1555),
    [sym_ip_tcp] = ACTIONS(1555),
    [sym_ip_udp] = ACTIONS(1555),
    [sym_ip_udp_2way] = ACTIONS(1555),
    [sym_xml_encode_types] = ACTIONS(1555),
    [sym_xml_encode_char_as_list] = ACTIONS(1555),
    [sym_xml_encode_le] = ACTIONS(1555),
    [sym_xml_decode_types] = ACTIONS(1555),
    [sym_xml_decode_no_preserve] = ACTIONS(1555),
    [sym_duet_dev_type_display_device] = ACTIONS(1555),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1555),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1555),
    [sym_duet_dev_type_io_device] = ACTIONS(1555),
    [sym_duet_dev_type_relay_device] = ACTIONS(1555),
    [sym_duet_dev_type_ups] = ACTIONS(1555),
    [sym_duet_dev_type_amplifier] = ACTIONS(1555),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1555),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1555),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1555),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1555),
    [sym_duet_dev_type_camera] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1555),
    [sym_duet_dev_type_dss] = ACTIONS(1555),
    [sym_duet_dev_type_dvr] = ACTIONS(1555),
    [sym_duet_dev_type_disc_device] = ACTIONS(1555),
    [sym_duet_dev_type_document_camera] = ACTIONS(1555),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1555),
    [sym_duet_dev_type_hvac] = ACTIONS(1555),
    [sym_duet_dev_type_keypad] = ACTIONS(1555),
    [sym_duet_dev_type_light] = ACTIONS(1555),
    [sym_duet_dev_type_monitor] = ACTIONS(1555),
    [sym_duet_dev_type_motor] = ACTIONS(1555),
    [sym_duet_dev_type_multi_window] = ACTIONS(1555),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1555),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1555),
    [sym_duet_dev_type_receiver] = ACTIONS(1555),
    [sym_duet_dev_type_security_system] = ACTIONS(1555),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1555),
    [sym_duet_dev_type_settop_box] = ACTIONS(1555),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1555),
    [sym_duet_dev_type_switcher] = ACTIONS(1555),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1555),
    [sym_duet_dev_type_tv] = ACTIONS(1555),
    [sym_duet_dev_type_utility] = ACTIONS(1555),
    [sym_duet_dev_type_vcr] = ACTIONS(1555),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1555),
    [sym_duet_dev_type_video_processor] = ACTIONS(1555),
    [sym_duet_dev_type_video_projector] = ACTIONS(1555),
    [sym_duet_dev_type_video_wall] = ACTIONS(1555),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1555),
    [sym_duet_dev_type_weather] = ACTIONS(1555),
    [sym_duet_dev_not_polled] = ACTIONS(1555),
    [sym_duet_dev_polled] = ACTIONS(1555),
    [sym_clkmgr_mode_network] = ACTIONS(1555),
    [sym_clkmgr_mode_standalone] = ACTIONS(1555),
    [sym_normal_standby] = ACTIONS(1555),
    [sym_normal_wake] = ACTIONS(1555),
    [sym_ip_multicast_ttl_option] = ACTIONS(1555),
    [sym_ip_ttl_subnet] = ACTIONS(1555),
    [sym_ip_ttl_site] = ACTIONS(1555),
    [sym_ip_ttl_region] = ACTIONS(1555),
    [sym_ip_ttl_continent] = ACTIONS(1555),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1555),
    [sym_ip_nodelay_on] = ACTIONS(1555),
    [sym_ip_nodelay_off] = ACTIONS(1555),
    [sym_amx_error] = ACTIONS(1555),
    [sym_amx_warning] = ACTIONS(1555),
    [sym_amx_info] = ACTIONS(1555),
    [sym_amx_debug] = ACTIONS(1555),
    [sym_netlinx_login_success] = ACTIONS(1555),
    [sym_netlinx_login_fail] = ACTIONS(1555),
    [sym_netlinx_logout] = ACTIONS(1555),
    [sym_null_str] = ACTIONS(1555),
    [sym_smtp_address] = ACTIONS(1555),
    [sym_smtp_port_number] = ACTIONS(1555),
    [sym_smtp_username] = ACTIONS(1555),
    [sym_smtp_password] = ACTIONS(1555),
    [sym_smtp_require_tls] = ACTIONS(1555),
    [sym_smtp_from] = ACTIONS(1555),
    [sym_smtp_tls_true] = ACTIONS(1555),
    [sym_smtp_tls_false] = ACTIONS(1555),
    [sym_listview_on_row_select_event] = ACTIONS(1555),
    [sym_data_structure_unknown] = ACTIONS(1555),
    [sym_data_structure_datafeed] = ACTIONS(1555),
    [sym_data_structure_datarecordset] = ACTIONS(1555),
    [sym_data_structure_datarecord] = ACTIONS(1555),
    [sym_data_structure_datafield] = ACTIONS(1555),
    [sym_data_max_content_field_count] = ACTIONS(1555),
    [sym_data_max_metadata_field_count] = ACTIONS(1555),
    [sym_data_max_id_length] = ACTIONS(1555),
    [sym_data_max_name_length] = ACTIONS(1555),
    [sym_data_max_description_length] = ACTIONS(1555),
    [sym_data_max_type_length] = ACTIONS(1555),
    [sym_data_max_format_length] = ACTIONS(1555),
    [sym_data_max_source_length] = ACTIONS(1555),
    [sym_data_max_value_length] = ACTIONS(1555),
    [sym_data_max_label_length] = ACTIONS(1555),
    [sym_data_max_url_length] = ACTIONS(1555),
    [sym_max_last_login_info_length] = ACTIONS(1555),
    [sym_data_type_unknown] = ACTIONS(1555),
    [sym_data_type_string] = ACTIONS(1555),
    [sym_data_type_datetime] = ACTIONS(1555),
    [sym_data_type_date] = ACTIONS(1555),
    [sym_data_type_time] = ACTIONS(1555),
    [sym_data_type_image] = ACTIONS(1555),
    [sym_data_format_url] = ACTIONS(1555),
    [sym_data_format_phone] = ACTIONS(1555),
    [sym_data_format_email] = ACTIONS(1555),
    [sym_data_format_iso8601] = ACTIONS(1555),
    [sym_auditlog_priv] = ACTIONS(1555),
    [sym_configuration_priv] = ACTIONS(1555),
    [sym_deviceconfig_priv] = ACTIONS(1555),
    [sym_ftp_priv] = ACTIONS(1555),
    [sym_http_priv] = ACTIONS(1555),
    [sym_networkconfig_priv] = ACTIONS(1555),
    [sym_programport_priv] = ACTIONS(1555),
    [sym_remoteui_priv] = ACTIONS(1555),
    [sym_securitycontrol_priv] = ACTIONS(1555),
    [sym_softwaremanagement_priv] = ACTIONS(1555),
    [sym_terminal_priv] = ACTIONS(1555),
    [sym_tpadmin_priv] = ACTIONS(1555),
    [sym_user1_priv] = ACTIONS(1555),
    [sym_user2_priv] = ACTIONS(1555),
    [sym_user3_priv] = ACTIONS(1555),
    [sym_user4_priv] = ACTIONS(1555),
    [sym_usermanagement_priv] = ACTIONS(1555),
    [sym_valid_account] = ACTIONS(1555),
    [sym_err_name_invalid] = ACTIONS(1555),
    [sym_err_password_invalid] = ACTIONS(1555),
    [sym_err_invalid_account] = ACTIONS(1555),
    [sym_err_no_authorization] = ACTIONS(1555),
    [sym_err_invalid_parameter] = ACTIONS(1555),
    [sym_err_user_locked_out] = ACTIONS(1555),
    [sym_err_user_account_expired] = ACTIONS(1555),
    [sym_err_general_error] = ACTIONS(1555),
    [sym_err_authentication_error] = ACTIONS(1555),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1555),
    [sym_tls_validate_certificate] = ACTIONS(1555),
    [sym_tdata] = ACTIONS(1555),
    [sym_tchannel] = ACTIONS(1555),
    [sym_tlevel] = ACTIONS(1555),
    [sym_tbutton] = ACTIONS(1555),
    [sym_ttimeline] = ACTIONS(1555),
    [sym_tcustom] = ACTIONS(1555),
    [sym_url_struct] = ACTIONS(1555),
    [sym_dns_struct] = ACTIONS(1555),
    [sym_ip_address_struct] = ACTIONS(1555),
    [sym_dev_info_struct] = ACTIONS(1555),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1555),
    [sym_data_feed] = ACTIONS(1555),
    [sym_data_field] = ACTIONS(1555),
    [sym_data_record] = ACTIONS(1555),
    [sym_wc_data_feed] = ACTIONS(1555),
    [sym_wc_data_field] = ACTIONS(1555),
    [sym_wc_data_record] = ACTIONS(1555),
    [sym_last_login_info] = ACTIONS(1555),
    [sym____reserved___] = ACTIONS(1555),
    [sym_date] = ACTIONS(1555),
    [sym_day] = ACTIONS(1555),
    [sym_time] = ACTIONS(1555),
    [sym_ldate] = ACTIONS(1555),
    [sym_push_device] = ACTIONS(1555),
    [sym_push_channel] = ACTIONS(1555),
    [sym_push_devchan] = ACTIONS(1555),
    [sym_release_device] = ACTIONS(1555),
    [sym_release_channel] = ACTIONS(1555),
    [sym_release_devchan] = ACTIONS(1555),
    [sym_master_slot] = ACTIONS(1555),
    [sym_get_pulse_time] = ACTIONS(1555),
    [sym_get_timer] = ACTIONS(1555),
    [sym_master_sn] = ACTIONS(1555),
    [sym_system_number] = ACTIONS(1555),
    [sym_button] = ACTIONS(1555),
    [sym_dv_channel] = ACTIONS(1555),
    [sym_data] = ACTIONS(1555),
    [sym_level] = ACTIONS(1555),
    [sym_timeline] = ACTIONS(1555),
    [sym_channel] = ACTIONS(1555),
    [sym_custom] = ACTIONS(1555),
    [sym_atoi] = ACTIONS(1555),
    [sym_atol] = ACTIONS(1555),
    [sym_atof] = ACTIONS(1555),
    [sym_itoa] = ACTIONS(1555),
    [sym_itohex] = ACTIONS(1555),
    [sym_hextoi] = ACTIONS(1555),
    [sym_ftoa] = ACTIONS(1555),
    [sym_find_string] = ACTIONS(1555),
    [sym_left_string] = ACTIONS(1555),
    [sym_right_string] = ACTIONS(1555),
    [sym_mid_string] = ACTIONS(1555),
    [sym_remove_string] = ACTIONS(1555),
    [sym_get_buffer_string] = ACTIONS(1555),
    [sym_get_buffer_char] = ACTIONS(1555),
    [sym_get_multi_buffer_string] = ACTIONS(1555),
    [sym_length_string] = ACTIONS(1555),
    [sym_max_length_string] = ACTIONS(1555),
    [sym_set_length_string] = ACTIONS(1555),
    [sym_compare_string] = ACTIONS(1555),
    [sym_upper_string] = ACTIONS(1555),
    [sym_lower_string] = ACTIONS(1555),
    [sym_get_last] = ACTIONS(1555),
    [sym_date_to_day] = ACTIONS(1555),
    [sym_date_to_month] = ACTIONS(1555),
    [sym_date_to_year] = ACTIONS(1555),
    [sym_time_to_hour] = ACTIONS(1555),
    [sym_time_to_minute] = ACTIONS(1555),
    [sym_time_to_second] = ACTIONS(1555),
    [sym_day_of_week] = ACTIONS(1555),
    [sym_set_pulse_time] = ACTIONS(1555),
    [sym_set_timer] = ACTIONS(1555),
    [sym_file_open] = ACTIONS(1555),
    [sym_file_close] = ACTIONS(1555),
    [sym_file_read] = ACTIONS(1555),
    [sym_file_read_line] = ACTIONS(1555),
    [sym_file_write] = ACTIONS(1555),
    [sym_file_write_line] = ACTIONS(1555),
    [sym_file_copy] = ACTIONS(1555),
    [sym_file_delete] = ACTIONS(1555),
    [sym_file_rename] = ACTIONS(1555),
    [sym_file_dir] = ACTIONS(1555),
    [sym_file_getdir] = ACTIONS(1555),
    [sym_file_setdir] = ACTIONS(1555),
    [sym_file_create_dir] = ACTIONS(1555),
    [sym_file_remove_dir] = ACTIONS(1555),
    [sym_file_seek] = ACTIONS(1555),
    [sym_length_array] = ACTIONS(1555),
    [sym_max_length_array] = ACTIONS(1555),
    [sym_set_length_array] = ACTIONS(1555),
    [sym_ip_client_open] = ACTIONS(1555),
    [sym_ip_client_close] = ACTIONS(1555),
    [sym_ip_server_open] = ACTIONS(1555),
    [sym_ip_server_close] = ACTIONS(1555),
    [sym_ip_bound_client_open] = ACTIONS(1555),
    [sym_ip_mc_server_open] = ACTIONS(1555),
    [sym_ip_set_option] = ACTIONS(1555),
    [sym_ssh_client_open] = ACTIONS(1555),
    [sym_ssh_client_close] = ACTIONS(1555),
    [sym_tls_client_open] = ACTIONS(1555),
    [sym_tls_client_close] = ACTIONS(1555),
    [sym_do_push] = ACTIONS(1555),
    [sym_do_push_timed] = ACTIONS(1555),
    [sym_do_release] = ACTIONS(1555),
    [sym_device_id] = ACTIONS(1555),
    [sym_device_info] = ACTIONS(1555),
    [sym_device_id_string] = ACTIONS(1555),
    [sym_device_standby] = ACTIONS(1555),
    [sym_device_wake] = ACTIONS(1555),
    [sym_timeline_create] = ACTIONS(1555),
    [sym_timeline_kill] = ACTIONS(1555),
    [sym_timeline_reload] = ACTIONS(1555),
    [sym_timeline_pause] = ACTIONS(1555),
    [sym_timeline_restart] = ACTIONS(1555),
    [sym_timeline_set] = ACTIONS(1555),
    [sym_timeline_get] = ACTIONS(1555),
    [sym_timeline_active] = ACTIONS(1555),
    [sym_timeline_dynamic_id] = ACTIONS(1555),
    [sym_variable_to_string] = ACTIONS(1555),
    [sym_string_to_variable] = ACTIONS(1555),
    [sym_length_variable_to_string] = ACTIONS(1555),
    [sym_type_cast] = ACTIONS(1555),
    [sym_variable_to_xml] = ACTIONS(1555),
    [sym_xml_to_variable] = ACTIONS(1555),
    [sym_length_variable_to_xml] = ACTIONS(1555),
    [sym_redirect_string] = ACTIONS(1555),
    [sym_random_number] = ACTIONS(1555),
    [sym_get_unique_id] = ACTIONS(1555),
    [sym_get_system_number] = ACTIONS(1555),
    [sym_set_system_number] = ACTIONS(1555),
    [sym_get_serial_number] = ACTIONS(1555),
    [sym_get_master_build] = ACTIONS(1555),
    [sym_set_validation_code] = ACTIONS(1555),
    [sym_reboot] = ACTIONS(1555),
    [sym_load_duet_module] = ACTIONS(1555),
    [sym_unload_duet_module] = ACTIONS(1555),
    [sym_duet_mem_size_set] = ACTIONS(1555),
    [sym_duet_mem_size_get] = ACTIONS(1555),
    [sym_dynamic_application_device] = ACTIONS(1555),
    [sym_dynamic_polled_port] = ACTIONS(1555),
    [sym_static_port_binding] = ACTIONS(1555),
    [sym_static_ip_binding] = ACTIONS(1555),
    [sym_get_dns_list] = ACTIONS(1555),
    [sym_set_dns_list] = ACTIONS(1555),
    [sym_get_ip_address] = ACTIONS(1555),
    [sym_set_ip_address] = ACTIONS(1555),
    [sym_get_url_list] = ACTIONS(1555),
    [sym_add_url_entry] = ACTIONS(1555),
    [sym_delete_url_entry] = ACTIONS(1555),
    [sym_abs_value] = ACTIONS(1555),
    [sym_max_value] = ACTIONS(1555),
    [sym_min_value] = ACTIONS(1555),
    [sym_exp_value] = ACTIONS(1555),
    [sym_log_value] = ACTIONS(1555),
    [sym_log10_value] = ACTIONS(1555),
    [sym_power_value] = ACTIONS(1555),
    [sym_sqrt_value] = ACTIONS(1555),
    [sym_raw_be] = ACTIONS(1555),
    [sym_raw_le] = ACTIONS(1555),
    [sym_internal_queue_size_set] = ACTIONS(1555),
    [sym_internal_queue_size_get] = ACTIONS(1555),
    [sym_internal_threshold_set] = ACTIONS(1555),
    [sym_internal_threshold_get] = ACTIONS(1555),
    [sym_rebuilt_event] = ACTIONS(1555),
    [sym_do_custom_event] = ACTIONS(1555),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1555),
    [sym_clkmgr_set_clk_source] = ACTIONS(1555),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1555),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1555),
    [sym_clkmgr_get_timezone] = ACTIONS(1555),
    [sym_clkmgr_set_timezone] = ACTIONS(1555),
    [sym_clkmgr_get_resync_period] = ACTIONS(1555),
    [sym_clkmgr_set_resync_period] = ACTIONS(1555),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1555),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1555),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1555),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1555),
    [sym_clkmgr_get_timeservers] = ACTIONS(1555),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1555),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1555),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1555),
    [sym_set_log_level] = ACTIONS(1555),
    [sym_get_log_level] = ACTIONS(1555),
    [sym_amx_log] = ACTIONS(1555),
    [sym_smtp_server_config_set] = ACTIONS(1555),
    [sym_smtp_server_config_get] = ACTIONS(1555),
    [sym_smtp_send] = ACTIONS(1555),
    [sym_validate_netlinx_account] = ACTIONS(1555),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1555),
    [sym_authenticate_certificate] = ACTIONS(1555),
    [sym_audit_netlinx_session_event] = ACTIONS(1555),
    [sym_audit_netlinx_generic_event] = ACTIONS(1555),
    [sym_get_available_flash_disk_space] = ACTIONS(1555),
    [sym_get_max_flash_disk_space] = ACTIONS(1555),
    [sym_data_create_feed] = ACTIONS(1555),
    [sym_data_delete_feed] = ACTIONS(1555),
    [sym_data_publish_feed] = ACTIONS(1555),
    [sym_data_get_published_feed] = ACTIONS(1555),
    [sym_data_add_record] = ACTIONS(1555),
    [sym_data_get_event_record] = ACTIONS(1555),
    [sym__wc_data_create_feed] = ACTIONS(1555),
    [sym__wc_data_add_record] = ACTIONS(1555),
    [sym__wc_data_get_event_record] = ACTIONS(1555),
    [sym_format] = ACTIONS(1555),
    [sym_astro_clock] = ACTIONS(1555),
    [sym_set_outdoor_temperature] = ACTIONS(1555),
    [sym_set_virtual_level_count] = ACTIONS(1555),
    [sym_set_virtual_channel_count] = ACTIONS(1555),
    [sym_set_virtual_port_count] = ACTIONS(1555),
    [sym__wc_encode_int] = ACTIONS(1555),
    [sym__wc_decode_int] = ACTIONS(1555),
    [sym__wc_int] = ACTIONS(1555),
    [sym__wc_to_ch_int] = ACTIONS(1555),
    [sym__ch_to_wc_int] = ACTIONS(1555),
    [sym__wc_find_string_int] = ACTIONS(1555),
    [sym__wc_left_string_int] = ACTIONS(1555),
    [sym__wc_lower_string_int] = ACTIONS(1555),
    [sym__wc_mid_string_int] = ACTIONS(1555),
    [sym__wc_remove_string_int] = ACTIONS(1555),
    [sym__wc_right_string_int] = ACTIONS(1555),
    [sym__wc_upper_string_int] = ACTIONS(1555),
    [sym__wc_compare_string_int] = ACTIONS(1555),
    [sym__wc_get_buffer_char_int] = ACTIONS(1555),
    [sym__wc_get_buffer_string_int] = ACTIONS(1555),
    [sym__wc_concat_string_int] = ACTIONS(1555),
    [sym__wc_file_open_int] = ACTIONS(1555),
    [sym__wc_file_close_int] = ACTIONS(1555),
    [sym__wc_file_read_int] = ACTIONS(1555),
    [sym__wc_file_read_line_int] = ACTIONS(1555),
    [sym__wc_file_write_int] = ACTIONS(1555),
    [sym__wc_file_write_line_int] = ACTIONS(1555),
    [anon_sym_EQ] = ACTIONS(1555),
    [aux_sym_preproc_include_token1] = ACTIONS(1555),
    [aux_sym_preproc_define_token1] = ACTIONS(1557),
    [aux_sym_preproc_warn_token1] = ACTIONS(1557),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1557),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1557),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1557),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1557),
    [aux_sym_preproc_else_token1] = ACTIONS(1557),
    [anon_sym_LBRACK] = ACTIONS(1557),
    [anon_sym_LBRACE] = ACTIONS(1557),
    [anon_sym_COLON] = ACTIONS(1557),
    [anon_sym_LPAREN] = ACTIONS(1555),
    [anon_sym_SEMI] = ACTIONS(1557),
    [anon_sym_BANG] = ACTIONS(1555),
    [anon_sym_TILDE] = ACTIONS(1557),
    [anon_sym_DASH] = ACTIONS(1555),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_STAR] = ACTIONS(1557),
    [anon_sym_SLASH] = ACTIONS(1555),
    [anon_sym_PERCENT] = ACTIONS(1557),
    [anon_sym_PIPE_PIPE] = ACTIONS(1557),
    [anon_sym_AMP_AMP] = ACTIONS(1557),
    [anon_sym_PIPE] = ACTIONS(1555),
    [anon_sym_CARET] = ACTIONS(1557),
    [anon_sym_AMP] = ACTIONS(1555),
    [anon_sym_EQ_EQ] = ACTIONS(1557),
    [anon_sym_BANG_EQ] = ACTIONS(1557),
    [anon_sym_LT_GT] = ACTIONS(1557),
    [anon_sym_GT] = ACTIONS(1555),
    [anon_sym_GT_EQ] = ACTIONS(1557),
    [anon_sym_LT_EQ] = ACTIONS(1557),
    [anon_sym_LT] = ACTIONS(1555),
    [anon_sym_LT_LT] = ACTIONS(1557),
    [anon_sym_GT_GT] = ACTIONS(1557),
    [anon_sym_DASH_DASH] = ACTIONS(1557),
    [anon_sym_PLUS_PLUS] = ACTIONS(1557),
    [anon_sym_DOT] = ACTIONS(1555),
    [anon_sym_DQUOTE] = ACTIONS(1557),
    [anon_sym_SQUOTE] = ACTIONS(1557),
    [sym_number_literal] = ACTIONS(1557),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1557),
  },
  [STATE(110)] = {
    [sym_identifier] = ACTIONS(1503),
    [sym_program_name_keyword] = ACTIONS(1503),
    [sym_module_name_keyword] = ACTIONS(1503),
    [sym_define_device_keyword] = ACTIONS(1503),
    [sym_define_combine_keyword] = ACTIONS(1503),
    [sym_define_constant_keyword] = ACTIONS(1503),
    [sym_define_type_keyword] = ACTIONS(1503),
    [sym_define_variable_keyword] = ACTIONS(1503),
    [sym_define_system_variable_keyword] = ACTIONS(1503),
    [sym_define_connect_level_keyword] = ACTIONS(1503),
    [sym_define_latching_keyword] = ACTIONS(1503),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1503),
    [sym_define_toggling_keyword] = ACTIONS(1503),
    [sym_define_call_keyword] = ACTIONS(1503),
    [sym_define_function_keyword] = ACTIONS(1503),
    [sym_define_library_function_keyword] = ACTIONS(1503),
    [sym_define_start_keyword] = ACTIONS(1503),
    [sym_define_event_keyword] = ACTIONS(1503),
    [sym_define_program_keyword] = ACTIONS(1503),
    [sym_define_module_keyword] = ACTIONS(1503),
    [sym_char_keyword] = ACTIONS(1503),
    [sym_widechar_keyword] = ACTIONS(1503),
    [sym_integer_keyword] = ACTIONS(1503),
    [sym_sinteger_keyword] = ACTIONS(1503),
    [sym_long_keyword] = ACTIONS(1503),
    [sym_slong_keyword] = ACTIONS(1503),
    [sym_float_keyword] = ACTIONS(1503),
    [sym_double_keyword] = ACTIONS(1503),
    [sym_variant_keyword] = ACTIONS(1503),
    [sym_variantarray_keyword] = ACTIONS(1503),
    [sym_local_var_keyword] = ACTIONS(1503),
    [sym_stack_var_keyword] = ACTIONS(1503),
    [sym_constant_keyword] = ACTIONS(1503),
    [sym_volatile_keyword] = ACTIONS(1503),
    [sym_non_volatile_keyword] = ACTIONS(1503),
    [sym_persistent_keyword] = ACTIONS(1503),
    [sym_dev_keyword] = ACTIONS(1503),
    [sym_devlev_keyword] = ACTIONS(1503),
    [sym_devchan_keyword] = ACTIONS(1503),
    [sym_if_keyword] = ACTIONS(1503),
    [sym_else_keyword] = ACTIONS(1503),
    [sym_switch_keyword] = ACTIONS(1503),
    [sym_case_keyword] = ACTIONS(1503),
    [sym_default_keyword] = ACTIONS(1503),
    [sym_while_keyword] = ACTIONS(1503),
    [sym_for_keyword] = ACTIONS(1503),
    [sym_break_keyword] = ACTIONS(1503),
    [sym_continue_keyword] = ACTIONS(1503),
    [sym_return_keyword] = ACTIONS(1503),
    [sym_select_keyword] = ACTIONS(1503),
    [sym_active_keyword] = ACTIONS(1503),
    [sym_struct_keyword] = ACTIONS(1503),
    [sym_structure_keyword] = ACTIONS(1503),
    [sym_button_event_keyword] = ACTIONS(1503),
    [sym_channel_event_keyword] = ACTIONS(1503),
    [sym_level_event_keyword] = ACTIONS(1503),
    [sym_data_event_keyword] = ACTIONS(1503),
    [sym_timeline_event_keyword] = ACTIONS(1503),
    [sym_custom_event_keyword] = ACTIONS(1503),
    [sym_push_keyword] = ACTIONS(1503),
    [sym_release_keyword] = ACTIONS(1503),
    [aux_sym_on_keyword_token1] = ACTIONS(1503),
    [aux_sym_off_keyword_token1] = ACTIONS(1503),
    [sym_send_level_keyword] = ACTIONS(1503),
    [sym_send_string_keyword] = ACTIONS(1503),
    [sym_send_command_keyword] = ACTIONS(1503),
    [sym_clear_buffer_keyword] = ACTIONS(1503),
    [sym_create_buffer_keyword] = ACTIONS(1503),
    [sym_create_multi_buffer_keyword] = ACTIONS(1503),
    [sym_call_keyword] = ACTIONS(1503),
    [sym_system_call_keyword] = ACTIONS(1503),
    [sym_devchan_to_keyword] = ACTIONS(1503),
    [sym_devchan_min_to_keyword] = ACTIONS(1503),
    [sym_devchan_total_off_keyword] = ACTIONS(1503),
    [sym_devchan_pulse_keyword] = ACTIONS(1503),
    [sym_wait_keyword] = ACTIONS(1503),
    [sym_cancel_wait_keyword] = ACTIONS(1503),
    [sym_wait_until_keyword] = ACTIONS(1503),
    [sym_cancel_wait_until_keyword] = ACTIONS(1503),
    [sym_cancel_all_wait_keyword] = ACTIONS(1503),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1503),
    [sym_and] = ACTIONS(1503),
    [sym_or] = ACTIONS(1503),
    [sym_not] = ACTIONS(1503),
    [sym_xor] = ACTIONS(1503),
    [sym_band] = ACTIONS(1503),
    [sym_bor] = ACTIONS(1503),
    [sym_bxor] = ACTIONS(1503),
    [sym_bnot] = ACTIONS(1503),
    [sym_lshift] = ACTIONS(1503),
    [sym_rshift] = ACTIONS(1503),
    [sym___file__] = ACTIONS(1503),
    [sym___date__] = ACTIONS(1503),
    [sym___ldate__] = ACTIONS(1503),
    [sym___time__] = ACTIONS(1503),
    [sym___line__] = ACTIONS(1503),
    [sym___name__] = ACTIONS(1503),
    [sym___netlinx__] = ACTIONS(1503),
    [sym_netlinx_axi_version] = ACTIONS(1503),
    [sym_url_flg_tcp] = ACTIONS(1503),
    [sym_url_flg_accinfopresent] = ACTIONS(1503),
    [sym_url_flg_temp] = ACTIONS(1503),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1503),
    [sym_url_flg_stat_mask] = ACTIONS(1503),
    [sym_url_flg_stat_lookup] = ACTIONS(1503),
    [sym_url_flg_stat_connecting] = ACTIONS(1503),
    [sym_url_flg_stat_waiting] = ACTIONS(1503),
    [sym_url_flg_stat_connected] = ACTIONS(1503),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1503),
    [sym_first_virtual_device] = ACTIONS(1503),
    [sym_first_local_port] = ACTIONS(1503),
    [sym_dynamic_virtual_device] = ACTIONS(1503),
    [sym_timeline_once] = ACTIONS(1503),
    [sym_timeline_repeat] = ACTIONS(1503),
    [sym_timeline_absolute] = ACTIONS(1503),
    [sym_timeline_relative] = ACTIONS(1503),
    [sym_do_push_timed_infinite] = ACTIONS(1503),
    [sym_source_type_no_address] = ACTIONS(1503),
    [sym_source_type_neuron_id] = ACTIONS(1503),
    [sym_source_type_ip_address] = ACTIONS(1503),
    [sym_source_type_axlink] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1503),
    [sym_source_type_ip_socket_address] = ACTIONS(1503),
    [sym_source_type_rs232] = ACTIONS(1503),
    [sym_source_type_internal] = ACTIONS(1503),
    [sym_source_type_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1503),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1503),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1503),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1503),
    [sym_true] = ACTIONS(1503),
    [sym_false] = ACTIONS(1503),
    [sym_file_read_only] = ACTIONS(1503),
    [sym_file_rw_new] = ACTIONS(1503),
    [sym_file_rw_append] = ACTIONS(1503),
    [sym_ip_tcp] = ACTIONS(1503),
    [sym_ip_udp] = ACTIONS(1503),
    [sym_ip_udp_2way] = ACTIONS(1503),
    [sym_xml_encode_types] = ACTIONS(1503),
    [sym_xml_encode_char_as_list] = ACTIONS(1503),
    [sym_xml_encode_le] = ACTIONS(1503),
    [sym_xml_decode_types] = ACTIONS(1503),
    [sym_xml_decode_no_preserve] = ACTIONS(1503),
    [sym_duet_dev_type_display_device] = ACTIONS(1503),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_io_device] = ACTIONS(1503),
    [sym_duet_dev_type_relay_device] = ACTIONS(1503),
    [sym_duet_dev_type_ups] = ACTIONS(1503),
    [sym_duet_dev_type_amplifier] = ACTIONS(1503),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1503),
    [sym_duet_dev_type_camera] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1503),
    [sym_duet_dev_type_dss] = ACTIONS(1503),
    [sym_duet_dev_type_dvr] = ACTIONS(1503),
    [sym_duet_dev_type_disc_device] = ACTIONS(1503),
    [sym_duet_dev_type_document_camera] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1503),
    [sym_duet_dev_type_hvac] = ACTIONS(1503),
    [sym_duet_dev_type_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_light] = ACTIONS(1503),
    [sym_duet_dev_type_monitor] = ACTIONS(1503),
    [sym_duet_dev_type_motor] = ACTIONS(1503),
    [sym_duet_dev_type_multi_window] = ACTIONS(1503),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1503),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1503),
    [sym_duet_dev_type_receiver] = ACTIONS(1503),
    [sym_duet_dev_type_security_system] = ACTIONS(1503),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1503),
    [sym_duet_dev_type_settop_box] = ACTIONS(1503),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1503),
    [sym_duet_dev_type_switcher] = ACTIONS(1503),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_tv] = ACTIONS(1503),
    [sym_duet_dev_type_utility] = ACTIONS(1503),
    [sym_duet_dev_type_vcr] = ACTIONS(1503),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_video_processor] = ACTIONS(1503),
    [sym_duet_dev_type_video_projector] = ACTIONS(1503),
    [sym_duet_dev_type_video_wall] = ACTIONS(1503),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1503),
    [sym_duet_dev_type_weather] = ACTIONS(1503),
    [sym_duet_dev_not_polled] = ACTIONS(1503),
    [sym_duet_dev_polled] = ACTIONS(1503),
    [sym_clkmgr_mode_network] = ACTIONS(1503),
    [sym_clkmgr_mode_standalone] = ACTIONS(1503),
    [sym_normal_standby] = ACTIONS(1503),
    [sym_normal_wake] = ACTIONS(1503),
    [sym_ip_multicast_ttl_option] = ACTIONS(1503),
    [sym_ip_ttl_subnet] = ACTIONS(1503),
    [sym_ip_ttl_site] = ACTIONS(1503),
    [sym_ip_ttl_region] = ACTIONS(1503),
    [sym_ip_ttl_continent] = ACTIONS(1503),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1503),
    [sym_ip_nodelay_on] = ACTIONS(1503),
    [sym_ip_nodelay_off] = ACTIONS(1503),
    [sym_amx_error] = ACTIONS(1503),
    [sym_amx_warning] = ACTIONS(1503),
    [sym_amx_info] = ACTIONS(1503),
    [sym_amx_debug] = ACTIONS(1503),
    [sym_netlinx_login_success] = ACTIONS(1503),
    [sym_netlinx_login_fail] = ACTIONS(1503),
    [sym_netlinx_logout] = ACTIONS(1503),
    [sym_null_str] = ACTIONS(1503),
    [sym_smtp_address] = ACTIONS(1503),
    [sym_smtp_port_number] = ACTIONS(1503),
    [sym_smtp_username] = ACTIONS(1503),
    [sym_smtp_password] = ACTIONS(1503),
    [sym_smtp_require_tls] = ACTIONS(1503),
    [sym_smtp_from] = ACTIONS(1503),
    [sym_smtp_tls_true] = ACTIONS(1503),
    [sym_smtp_tls_false] = ACTIONS(1503),
    [sym_listview_on_row_select_event] = ACTIONS(1503),
    [sym_data_structure_unknown] = ACTIONS(1503),
    [sym_data_structure_datafeed] = ACTIONS(1503),
    [sym_data_structure_datarecordset] = ACTIONS(1503),
    [sym_data_structure_datarecord] = ACTIONS(1503),
    [sym_data_structure_datafield] = ACTIONS(1503),
    [sym_data_max_content_field_count] = ACTIONS(1503),
    [sym_data_max_metadata_field_count] = ACTIONS(1503),
    [sym_data_max_id_length] = ACTIONS(1503),
    [sym_data_max_name_length] = ACTIONS(1503),
    [sym_data_max_description_length] = ACTIONS(1503),
    [sym_data_max_type_length] = ACTIONS(1503),
    [sym_data_max_format_length] = ACTIONS(1503),
    [sym_data_max_source_length] = ACTIONS(1503),
    [sym_data_max_value_length] = ACTIONS(1503),
    [sym_data_max_label_length] = ACTIONS(1503),
    [sym_data_max_url_length] = ACTIONS(1503),
    [sym_max_last_login_info_length] = ACTIONS(1503),
    [sym_data_type_unknown] = ACTIONS(1503),
    [sym_data_type_string] = ACTIONS(1503),
    [sym_data_type_datetime] = ACTIONS(1503),
    [sym_data_type_date] = ACTIONS(1503),
    [sym_data_type_time] = ACTIONS(1503),
    [sym_data_type_image] = ACTIONS(1503),
    [sym_data_format_url] = ACTIONS(1503),
    [sym_data_format_phone] = ACTIONS(1503),
    [sym_data_format_email] = ACTIONS(1503),
    [sym_data_format_iso8601] = ACTIONS(1503),
    [sym_auditlog_priv] = ACTIONS(1503),
    [sym_configuration_priv] = ACTIONS(1503),
    [sym_deviceconfig_priv] = ACTIONS(1503),
    [sym_ftp_priv] = ACTIONS(1503),
    [sym_http_priv] = ACTIONS(1503),
    [sym_networkconfig_priv] = ACTIONS(1503),
    [sym_programport_priv] = ACTIONS(1503),
    [sym_remoteui_priv] = ACTIONS(1503),
    [sym_securitycontrol_priv] = ACTIONS(1503),
    [sym_softwaremanagement_priv] = ACTIONS(1503),
    [sym_terminal_priv] = ACTIONS(1503),
    [sym_tpadmin_priv] = ACTIONS(1503),
    [sym_user1_priv] = ACTIONS(1503),
    [sym_user2_priv] = ACTIONS(1503),
    [sym_user3_priv] = ACTIONS(1503),
    [sym_user4_priv] = ACTIONS(1503),
    [sym_usermanagement_priv] = ACTIONS(1503),
    [sym_valid_account] = ACTIONS(1503),
    [sym_err_name_invalid] = ACTIONS(1503),
    [sym_err_password_invalid] = ACTIONS(1503),
    [sym_err_invalid_account] = ACTIONS(1503),
    [sym_err_no_authorization] = ACTIONS(1503),
    [sym_err_invalid_parameter] = ACTIONS(1503),
    [sym_err_user_locked_out] = ACTIONS(1503),
    [sym_err_user_account_expired] = ACTIONS(1503),
    [sym_err_general_error] = ACTIONS(1503),
    [sym_err_authentication_error] = ACTIONS(1503),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1503),
    [sym_tls_validate_certificate] = ACTIONS(1503),
    [sym_tdata] = ACTIONS(1503),
    [sym_tchannel] = ACTIONS(1503),
    [sym_tlevel] = ACTIONS(1503),
    [sym_tbutton] = ACTIONS(1503),
    [sym_ttimeline] = ACTIONS(1503),
    [sym_tcustom] = ACTIONS(1503),
    [sym_url_struct] = ACTIONS(1503),
    [sym_dns_struct] = ACTIONS(1503),
    [sym_ip_address_struct] = ACTIONS(1503),
    [sym_dev_info_struct] = ACTIONS(1503),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1503),
    [sym_data_feed] = ACTIONS(1503),
    [sym_data_field] = ACTIONS(1503),
    [sym_data_record] = ACTIONS(1503),
    [sym_wc_data_feed] = ACTIONS(1503),
    [sym_wc_data_field] = ACTIONS(1503),
    [sym_wc_data_record] = ACTIONS(1503),
    [sym_last_login_info] = ACTIONS(1503),
    [sym____reserved___] = ACTIONS(1503),
    [sym_date] = ACTIONS(1503),
    [sym_day] = ACTIONS(1503),
    [sym_time] = ACTIONS(1503),
    [sym_ldate] = ACTIONS(1503),
    [sym_push_device] = ACTIONS(1503),
    [sym_push_channel] = ACTIONS(1503),
    [sym_push_devchan] = ACTIONS(1503),
    [sym_release_device] = ACTIONS(1503),
    [sym_release_channel] = ACTIONS(1503),
    [sym_release_devchan] = ACTIONS(1503),
    [sym_master_slot] = ACTIONS(1503),
    [sym_get_pulse_time] = ACTIONS(1503),
    [sym_get_timer] = ACTIONS(1503),
    [sym_master_sn] = ACTIONS(1503),
    [sym_system_number] = ACTIONS(1503),
    [sym_button] = ACTIONS(1503),
    [sym_dv_channel] = ACTIONS(1503),
    [sym_data] = ACTIONS(1503),
    [sym_level] = ACTIONS(1503),
    [sym_timeline] = ACTIONS(1503),
    [sym_channel] = ACTIONS(1503),
    [sym_custom] = ACTIONS(1503),
    [sym_atoi] = ACTIONS(1503),
    [sym_atol] = ACTIONS(1503),
    [sym_atof] = ACTIONS(1503),
    [sym_itoa] = ACTIONS(1503),
    [sym_itohex] = ACTIONS(1503),
    [sym_hextoi] = ACTIONS(1503),
    [sym_ftoa] = ACTIONS(1503),
    [sym_find_string] = ACTIONS(1503),
    [sym_left_string] = ACTIONS(1503),
    [sym_right_string] = ACTIONS(1503),
    [sym_mid_string] = ACTIONS(1503),
    [sym_remove_string] = ACTIONS(1503),
    [sym_get_buffer_string] = ACTIONS(1503),
    [sym_get_buffer_char] = ACTIONS(1503),
    [sym_get_multi_buffer_string] = ACTIONS(1503),
    [sym_length_string] = ACTIONS(1503),
    [sym_max_length_string] = ACTIONS(1503),
    [sym_set_length_string] = ACTIONS(1503),
    [sym_compare_string] = ACTIONS(1503),
    [sym_upper_string] = ACTIONS(1503),
    [sym_lower_string] = ACTIONS(1503),
    [sym_get_last] = ACTIONS(1503),
    [sym_date_to_day] = ACTIONS(1503),
    [sym_date_to_month] = ACTIONS(1503),
    [sym_date_to_year] = ACTIONS(1503),
    [sym_time_to_hour] = ACTIONS(1503),
    [sym_time_to_minute] = ACTIONS(1503),
    [sym_time_to_second] = ACTIONS(1503),
    [sym_day_of_week] = ACTIONS(1503),
    [sym_set_pulse_time] = ACTIONS(1503),
    [sym_set_timer] = ACTIONS(1503),
    [sym_file_open] = ACTIONS(1503),
    [sym_file_close] = ACTIONS(1503),
    [sym_file_read] = ACTIONS(1503),
    [sym_file_read_line] = ACTIONS(1503),
    [sym_file_write] = ACTIONS(1503),
    [sym_file_write_line] = ACTIONS(1503),
    [sym_file_copy] = ACTIONS(1503),
    [sym_file_delete] = ACTIONS(1503),
    [sym_file_rename] = ACTIONS(1503),
    [sym_file_dir] = ACTIONS(1503),
    [sym_file_getdir] = ACTIONS(1503),
    [sym_file_setdir] = ACTIONS(1503),
    [sym_file_create_dir] = ACTIONS(1503),
    [sym_file_remove_dir] = ACTIONS(1503),
    [sym_file_seek] = ACTIONS(1503),
    [sym_length_array] = ACTIONS(1503),
    [sym_max_length_array] = ACTIONS(1503),
    [sym_set_length_array] = ACTIONS(1503),
    [sym_ip_client_open] = ACTIONS(1503),
    [sym_ip_client_close] = ACTIONS(1503),
    [sym_ip_server_open] = ACTIONS(1503),
    [sym_ip_server_close] = ACTIONS(1503),
    [sym_ip_bound_client_open] = ACTIONS(1503),
    [sym_ip_mc_server_open] = ACTIONS(1503),
    [sym_ip_set_option] = ACTIONS(1503),
    [sym_ssh_client_open] = ACTIONS(1503),
    [sym_ssh_client_close] = ACTIONS(1503),
    [sym_tls_client_open] = ACTIONS(1503),
    [sym_tls_client_close] = ACTIONS(1503),
    [sym_do_push] = ACTIONS(1503),
    [sym_do_push_timed] = ACTIONS(1503),
    [sym_do_release] = ACTIONS(1503),
    [sym_device_id] = ACTIONS(1503),
    [sym_device_info] = ACTIONS(1503),
    [sym_device_id_string] = ACTIONS(1503),
    [sym_device_standby] = ACTIONS(1503),
    [sym_device_wake] = ACTIONS(1503),
    [sym_timeline_create] = ACTIONS(1503),
    [sym_timeline_kill] = ACTIONS(1503),
    [sym_timeline_reload] = ACTIONS(1503),
    [sym_timeline_pause] = ACTIONS(1503),
    [sym_timeline_restart] = ACTIONS(1503),
    [sym_timeline_set] = ACTIONS(1503),
    [sym_timeline_get] = ACTIONS(1503),
    [sym_timeline_active] = ACTIONS(1503),
    [sym_timeline_dynamic_id] = ACTIONS(1503),
    [sym_variable_to_string] = ACTIONS(1503),
    [sym_string_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_string] = ACTIONS(1503),
    [sym_type_cast] = ACTIONS(1503),
    [sym_variable_to_xml] = ACTIONS(1503),
    [sym_xml_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_xml] = ACTIONS(1503),
    [sym_redirect_string] = ACTIONS(1503),
    [sym_random_number] = ACTIONS(1503),
    [sym_get_unique_id] = ACTIONS(1503),
    [sym_get_system_number] = ACTIONS(1503),
    [sym_set_system_number] = ACTIONS(1503),
    [sym_get_serial_number] = ACTIONS(1503),
    [sym_get_master_build] = ACTIONS(1503),
    [sym_set_validation_code] = ACTIONS(1503),
    [sym_reboot] = ACTIONS(1503),
    [sym_load_duet_module] = ACTIONS(1503),
    [sym_unload_duet_module] = ACTIONS(1503),
    [sym_duet_mem_size_set] = ACTIONS(1503),
    [sym_duet_mem_size_get] = ACTIONS(1503),
    [sym_dynamic_application_device] = ACTIONS(1503),
    [sym_dynamic_polled_port] = ACTIONS(1503),
    [sym_static_port_binding] = ACTIONS(1503),
    [sym_static_ip_binding] = ACTIONS(1503),
    [sym_get_dns_list] = ACTIONS(1503),
    [sym_set_dns_list] = ACTIONS(1503),
    [sym_get_ip_address] = ACTIONS(1503),
    [sym_set_ip_address] = ACTIONS(1503),
    [sym_get_url_list] = ACTIONS(1503),
    [sym_add_url_entry] = ACTIONS(1503),
    [sym_delete_url_entry] = ACTIONS(1503),
    [sym_abs_value] = ACTIONS(1503),
    [sym_max_value] = ACTIONS(1503),
    [sym_min_value] = ACTIONS(1503),
    [sym_exp_value] = ACTIONS(1503),
    [sym_log_value] = ACTIONS(1503),
    [sym_log10_value] = ACTIONS(1503),
    [sym_power_value] = ACTIONS(1503),
    [sym_sqrt_value] = ACTIONS(1503),
    [sym_raw_be] = ACTIONS(1503),
    [sym_raw_le] = ACTIONS(1503),
    [sym_internal_queue_size_set] = ACTIONS(1503),
    [sym_internal_queue_size_get] = ACTIONS(1503),
    [sym_internal_threshold_set] = ACTIONS(1503),
    [sym_internal_threshold_get] = ACTIONS(1503),
    [sym_rebuilt_event] = ACTIONS(1503),
    [sym_do_custom_event] = ACTIONS(1503),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1503),
    [sym_clkmgr_set_clk_source] = ACTIONS(1503),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1503),
    [sym_clkmgr_get_timezone] = ACTIONS(1503),
    [sym_clkmgr_set_timezone] = ACTIONS(1503),
    [sym_clkmgr_get_resync_period] = ACTIONS(1503),
    [sym_clkmgr_set_resync_period] = ACTIONS(1503),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_timeservers] = ACTIONS(1503),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_set_log_level] = ACTIONS(1503),
    [sym_get_log_level] = ACTIONS(1503),
    [sym_amx_log] = ACTIONS(1503),
    [sym_smtp_server_config_set] = ACTIONS(1503),
    [sym_smtp_server_config_get] = ACTIONS(1503),
    [sym_smtp_send] = ACTIONS(1503),
    [sym_validate_netlinx_account] = ACTIONS(1503),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1503),
    [sym_authenticate_certificate] = ACTIONS(1503),
    [sym_audit_netlinx_session_event] = ACTIONS(1503),
    [sym_audit_netlinx_generic_event] = ACTIONS(1503),
    [sym_get_available_flash_disk_space] = ACTIONS(1503),
    [sym_get_max_flash_disk_space] = ACTIONS(1503),
    [sym_data_create_feed] = ACTIONS(1503),
    [sym_data_delete_feed] = ACTIONS(1503),
    [sym_data_publish_feed] = ACTIONS(1503),
    [sym_data_get_published_feed] = ACTIONS(1503),
    [sym_data_add_record] = ACTIONS(1503),
    [sym_data_get_event_record] = ACTIONS(1503),
    [sym__wc_data_create_feed] = ACTIONS(1503),
    [sym__wc_data_add_record] = ACTIONS(1503),
    [sym__wc_data_get_event_record] = ACTIONS(1503),
    [sym_format] = ACTIONS(1503),
    [sym_astro_clock] = ACTIONS(1503),
    [sym_set_outdoor_temperature] = ACTIONS(1503),
    [sym_set_virtual_level_count] = ACTIONS(1503),
    [sym_set_virtual_channel_count] = ACTIONS(1503),
    [sym_set_virtual_port_count] = ACTIONS(1503),
    [sym__wc_encode_int] = ACTIONS(1503),
    [sym__wc_decode_int] = ACTIONS(1503),
    [sym__wc_int] = ACTIONS(1503),
    [sym__wc_to_ch_int] = ACTIONS(1503),
    [sym__ch_to_wc_int] = ACTIONS(1503),
    [sym__wc_find_string_int] = ACTIONS(1503),
    [sym__wc_left_string_int] = ACTIONS(1503),
    [sym__wc_lower_string_int] = ACTIONS(1503),
    [sym__wc_mid_string_int] = ACTIONS(1503),
    [sym__wc_remove_string_int] = ACTIONS(1503),
    [sym__wc_right_string_int] = ACTIONS(1503),
    [sym__wc_upper_string_int] = ACTIONS(1503),
    [sym__wc_compare_string_int] = ACTIONS(1503),
    [sym__wc_get_buffer_char_int] = ACTIONS(1503),
    [sym__wc_get_buffer_string_int] = ACTIONS(1503),
    [sym__wc_concat_string_int] = ACTIONS(1503),
    [sym__wc_file_open_int] = ACTIONS(1503),
    [sym__wc_file_close_int] = ACTIONS(1503),
    [sym__wc_file_read_int] = ACTIONS(1503),
    [sym__wc_file_read_line_int] = ACTIONS(1503),
    [sym__wc_file_write_int] = ACTIONS(1503),
    [sym__wc_file_write_line_int] = ACTIONS(1503),
    [aux_sym_preproc_include_token1] = ACTIONS(1503),
    [aux_sym_preproc_define_token1] = ACTIONS(1505),
    [aux_sym_preproc_warn_token1] = ACTIONS(1505),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1505),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1505),
    [aux_sym_preproc_else_token1] = ACTIONS(1505),
    [anon_sym_LBRACK] = ACTIONS(1505),
    [anon_sym_LBRACE] = ACTIONS(1505),
    [anon_sym_COLON] = ACTIONS(1505),
    [anon_sym_LPAREN] = ACTIONS(1503),
    [anon_sym_SEMI] = ACTIONS(1505),
    [anon_sym_BANG] = ACTIONS(1503),
    [anon_sym_TILDE] = ACTIONS(1505),
    [anon_sym_DASH] = ACTIONS(1503),
    [anon_sym_PLUS] = ACTIONS(1503),
    [anon_sym_STAR] = ACTIONS(1505),
    [anon_sym_SLASH] = ACTIONS(1503),
    [anon_sym_PERCENT] = ACTIONS(1505),
    [anon_sym_PIPE_PIPE] = ACTIONS(1505),
    [anon_sym_AMP_AMP] = ACTIONS(1505),
    [anon_sym_PIPE] = ACTIONS(1503),
    [anon_sym_CARET] = ACTIONS(1505),
    [anon_sym_AMP] = ACTIONS(1503),
    [anon_sym_EQ_EQ] = ACTIONS(1505),
    [anon_sym_BANG_EQ] = ACTIONS(1505),
    [anon_sym_LT_GT] = ACTIONS(1505),
    [anon_sym_GT] = ACTIONS(1503),
    [anon_sym_GT_EQ] = ACTIONS(1505),
    [anon_sym_LT_EQ] = ACTIONS(1505),
    [anon_sym_LT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1505),
    [anon_sym_GT_GT] = ACTIONS(1505),
    [anon_sym_DASH_DASH] = ACTIONS(1505),
    [anon_sym_PLUS_PLUS] = ACTIONS(1505),
    [anon_sym_DOT] = ACTIONS(1503),
    [anon_sym_DQUOTE] = ACTIONS(1505),
    [anon_sym_SQUOTE] = ACTIONS(1505),
    [sym_number_literal] = ACTIONS(1505),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1505),
  },
  [STATE(111)] = {
    [ts_builtin_sym_end] = ACTIONS(1549),
    [sym_identifier] = ACTIONS(1547),
    [sym_program_name_keyword] = ACTIONS(1547),
    [sym_module_name_keyword] = ACTIONS(1547),
    [sym_define_device_keyword] = ACTIONS(1547),
    [sym_define_combine_keyword] = ACTIONS(1547),
    [sym_define_constant_keyword] = ACTIONS(1547),
    [sym_define_type_keyword] = ACTIONS(1547),
    [sym_define_variable_keyword] = ACTIONS(1547),
    [sym_define_system_variable_keyword] = ACTIONS(1547),
    [sym_define_connect_level_keyword] = ACTIONS(1547),
    [sym_define_latching_keyword] = ACTIONS(1547),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1547),
    [sym_define_toggling_keyword] = ACTIONS(1547),
    [sym_define_call_keyword] = ACTIONS(1547),
    [sym_define_function_keyword] = ACTIONS(1547),
    [sym_define_library_function_keyword] = ACTIONS(1547),
    [sym_define_start_keyword] = ACTIONS(1547),
    [sym_define_event_keyword] = ACTIONS(1547),
    [sym_define_program_keyword] = ACTIONS(1547),
    [sym_define_module_keyword] = ACTIONS(1547),
    [sym_char_keyword] = ACTIONS(1547),
    [sym_widechar_keyword] = ACTIONS(1547),
    [sym_integer_keyword] = ACTIONS(1547),
    [sym_sinteger_keyword] = ACTIONS(1547),
    [sym_long_keyword] = ACTIONS(1547),
    [sym_slong_keyword] = ACTIONS(1547),
    [sym_float_keyword] = ACTIONS(1547),
    [sym_double_keyword] = ACTIONS(1547),
    [sym_variant_keyword] = ACTIONS(1547),
    [sym_variantarray_keyword] = ACTIONS(1547),
    [sym_local_var_keyword] = ACTIONS(1547),
    [sym_stack_var_keyword] = ACTIONS(1547),
    [sym_constant_keyword] = ACTIONS(1547),
    [sym_volatile_keyword] = ACTIONS(1547),
    [sym_non_volatile_keyword] = ACTIONS(1547),
    [sym_persistent_keyword] = ACTIONS(1547),
    [sym_dev_keyword] = ACTIONS(1547),
    [sym_devlev_keyword] = ACTIONS(1547),
    [sym_devchan_keyword] = ACTIONS(1547),
    [sym_if_keyword] = ACTIONS(1547),
    [sym_else_keyword] = ACTIONS(1547),
    [sym_switch_keyword] = ACTIONS(1547),
    [sym_case_keyword] = ACTIONS(1547),
    [sym_default_keyword] = ACTIONS(1547),
    [sym_while_keyword] = ACTIONS(1547),
    [sym_for_keyword] = ACTIONS(1547),
    [sym_break_keyword] = ACTIONS(1547),
    [sym_continue_keyword] = ACTIONS(1547),
    [sym_return_keyword] = ACTIONS(1547),
    [sym_select_keyword] = ACTIONS(1547),
    [sym_active_keyword] = ACTIONS(1547),
    [sym_struct_keyword] = ACTIONS(1547),
    [sym_structure_keyword] = ACTIONS(1547),
    [sym_button_event_keyword] = ACTIONS(1547),
    [sym_channel_event_keyword] = ACTIONS(1547),
    [sym_level_event_keyword] = ACTIONS(1547),
    [sym_data_event_keyword] = ACTIONS(1547),
    [sym_timeline_event_keyword] = ACTIONS(1547),
    [sym_custom_event_keyword] = ACTIONS(1547),
    [sym_push_keyword] = ACTIONS(1547),
    [sym_release_keyword] = ACTIONS(1547),
    [aux_sym_on_keyword_token1] = ACTIONS(1547),
    [aux_sym_off_keyword_token1] = ACTIONS(1547),
    [sym_send_level_keyword] = ACTIONS(1547),
    [sym_send_string_keyword] = ACTIONS(1547),
    [sym_send_command_keyword] = ACTIONS(1547),
    [sym_clear_buffer_keyword] = ACTIONS(1547),
    [sym_create_buffer_keyword] = ACTIONS(1547),
    [sym_create_multi_buffer_keyword] = ACTIONS(1547),
    [sym_call_keyword] = ACTIONS(1547),
    [sym_system_call_keyword] = ACTIONS(1547),
    [sym_devchan_to_keyword] = ACTIONS(1547),
    [sym_devchan_min_to_keyword] = ACTIONS(1547),
    [sym_devchan_total_off_keyword] = ACTIONS(1547),
    [sym_devchan_pulse_keyword] = ACTIONS(1547),
    [sym_wait_keyword] = ACTIONS(1547),
    [sym_cancel_wait_keyword] = ACTIONS(1547),
    [sym_wait_until_keyword] = ACTIONS(1547),
    [sym_cancel_wait_until_keyword] = ACTIONS(1547),
    [sym_cancel_all_wait_keyword] = ACTIONS(1547),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1547),
    [sym_and] = ACTIONS(1547),
    [sym_or] = ACTIONS(1547),
    [sym_not] = ACTIONS(1547),
    [sym_xor] = ACTIONS(1547),
    [sym_band] = ACTIONS(1547),
    [sym_bor] = ACTIONS(1547),
    [sym_bxor] = ACTIONS(1547),
    [sym_bnot] = ACTIONS(1547),
    [sym_lshift] = ACTIONS(1547),
    [sym_rshift] = ACTIONS(1547),
    [sym___file__] = ACTIONS(1547),
    [sym___date__] = ACTIONS(1547),
    [sym___ldate__] = ACTIONS(1547),
    [sym___time__] = ACTIONS(1547),
    [sym___line__] = ACTIONS(1547),
    [sym___name__] = ACTIONS(1547),
    [sym___netlinx__] = ACTIONS(1547),
    [sym_netlinx_axi_version] = ACTIONS(1547),
    [sym_url_flg_tcp] = ACTIONS(1547),
    [sym_url_flg_accinfopresent] = ACTIONS(1547),
    [sym_url_flg_temp] = ACTIONS(1547),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1547),
    [sym_url_flg_stat_mask] = ACTIONS(1547),
    [sym_url_flg_stat_lookup] = ACTIONS(1547),
    [sym_url_flg_stat_connecting] = ACTIONS(1547),
    [sym_url_flg_stat_waiting] = ACTIONS(1547),
    [sym_url_flg_stat_connected] = ACTIONS(1547),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1547),
    [sym_first_virtual_device] = ACTIONS(1547),
    [sym_first_local_port] = ACTIONS(1547),
    [sym_dynamic_virtual_device] = ACTIONS(1547),
    [sym_timeline_once] = ACTIONS(1547),
    [sym_timeline_repeat] = ACTIONS(1547),
    [sym_timeline_absolute] = ACTIONS(1547),
    [sym_timeline_relative] = ACTIONS(1547),
    [sym_do_push_timed_infinite] = ACTIONS(1547),
    [sym_source_type_no_address] = ACTIONS(1547),
    [sym_source_type_neuron_id] = ACTIONS(1547),
    [sym_source_type_ip_address] = ACTIONS(1547),
    [sym_source_type_axlink] = ACTIONS(1547),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1547),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1547),
    [sym_source_type_ip_socket_address] = ACTIONS(1547),
    [sym_source_type_rs232] = ACTIONS(1547),
    [sym_source_type_internal] = ACTIONS(1547),
    [sym_source_type_mac_address] = ACTIONS(1547),
    [sym_source_type_ipv4_port] = ACTIONS(1547),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1547),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1547),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1547),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1547),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1547),
    [sym_true] = ACTIONS(1547),
    [sym_false] = ACTIONS(1547),
    [sym_file_read_only] = ACTIONS(1547),
    [sym_file_rw_new] = ACTIONS(1547),
    [sym_file_rw_append] = ACTIONS(1547),
    [sym_ip_tcp] = ACTIONS(1547),
    [sym_ip_udp] = ACTIONS(1547),
    [sym_ip_udp_2way] = ACTIONS(1547),
    [sym_xml_encode_types] = ACTIONS(1547),
    [sym_xml_encode_char_as_list] = ACTIONS(1547),
    [sym_xml_encode_le] = ACTIONS(1547),
    [sym_xml_decode_types] = ACTIONS(1547),
    [sym_xml_decode_no_preserve] = ACTIONS(1547),
    [sym_duet_dev_type_display_device] = ACTIONS(1547),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1547),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1547),
    [sym_duet_dev_type_io_device] = ACTIONS(1547),
    [sym_duet_dev_type_relay_device] = ACTIONS(1547),
    [sym_duet_dev_type_ups] = ACTIONS(1547),
    [sym_duet_dev_type_amplifier] = ACTIONS(1547),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1547),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1547),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1547),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1547),
    [sym_duet_dev_type_camera] = ACTIONS(1547),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1547),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1547),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1547),
    [sym_duet_dev_type_dss] = ACTIONS(1547),
    [sym_duet_dev_type_dvr] = ACTIONS(1547),
    [sym_duet_dev_type_disc_device] = ACTIONS(1547),
    [sym_duet_dev_type_document_camera] = ACTIONS(1547),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1547),
    [sym_duet_dev_type_hvac] = ACTIONS(1547),
    [sym_duet_dev_type_keypad] = ACTIONS(1547),
    [sym_duet_dev_type_light] = ACTIONS(1547),
    [sym_duet_dev_type_monitor] = ACTIONS(1547),
    [sym_duet_dev_type_motor] = ACTIONS(1547),
    [sym_duet_dev_type_multi_window] = ACTIONS(1547),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1547),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1547),
    [sym_duet_dev_type_receiver] = ACTIONS(1547),
    [sym_duet_dev_type_security_system] = ACTIONS(1547),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1547),
    [sym_duet_dev_type_settop_box] = ACTIONS(1547),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1547),
    [sym_duet_dev_type_switcher] = ACTIONS(1547),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1547),
    [sym_duet_dev_type_tv] = ACTIONS(1547),
    [sym_duet_dev_type_utility] = ACTIONS(1547),
    [sym_duet_dev_type_vcr] = ACTIONS(1547),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1547),
    [sym_duet_dev_type_video_processor] = ACTIONS(1547),
    [sym_duet_dev_type_video_projector] = ACTIONS(1547),
    [sym_duet_dev_type_video_wall] = ACTIONS(1547),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1547),
    [sym_duet_dev_type_weather] = ACTIONS(1547),
    [sym_duet_dev_not_polled] = ACTIONS(1547),
    [sym_duet_dev_polled] = ACTIONS(1547),
    [sym_clkmgr_mode_network] = ACTIONS(1547),
    [sym_clkmgr_mode_standalone] = ACTIONS(1547),
    [sym_normal_standby] = ACTIONS(1547),
    [sym_normal_wake] = ACTIONS(1547),
    [sym_ip_multicast_ttl_option] = ACTIONS(1547),
    [sym_ip_ttl_subnet] = ACTIONS(1547),
    [sym_ip_ttl_site] = ACTIONS(1547),
    [sym_ip_ttl_region] = ACTIONS(1547),
    [sym_ip_ttl_continent] = ACTIONS(1547),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1547),
    [sym_ip_nodelay_on] = ACTIONS(1547),
    [sym_ip_nodelay_off] = ACTIONS(1547),
    [sym_amx_error] = ACTIONS(1547),
    [sym_amx_warning] = ACTIONS(1547),
    [sym_amx_info] = ACTIONS(1547),
    [sym_amx_debug] = ACTIONS(1547),
    [sym_netlinx_login_success] = ACTIONS(1547),
    [sym_netlinx_login_fail] = ACTIONS(1547),
    [sym_netlinx_logout] = ACTIONS(1547),
    [sym_null_str] = ACTIONS(1547),
    [sym_smtp_address] = ACTIONS(1547),
    [sym_smtp_port_number] = ACTIONS(1547),
    [sym_smtp_username] = ACTIONS(1547),
    [sym_smtp_password] = ACTIONS(1547),
    [sym_smtp_require_tls] = ACTIONS(1547),
    [sym_smtp_from] = ACTIONS(1547),
    [sym_smtp_tls_true] = ACTIONS(1547),
    [sym_smtp_tls_false] = ACTIONS(1547),
    [sym_listview_on_row_select_event] = ACTIONS(1547),
    [sym_data_structure_unknown] = ACTIONS(1547),
    [sym_data_structure_datafeed] = ACTIONS(1547),
    [sym_data_structure_datarecordset] = ACTIONS(1547),
    [sym_data_structure_datarecord] = ACTIONS(1547),
    [sym_data_structure_datafield] = ACTIONS(1547),
    [sym_data_max_content_field_count] = ACTIONS(1547),
    [sym_data_max_metadata_field_count] = ACTIONS(1547),
    [sym_data_max_id_length] = ACTIONS(1547),
    [sym_data_max_name_length] = ACTIONS(1547),
    [sym_data_max_description_length] = ACTIONS(1547),
    [sym_data_max_type_length] = ACTIONS(1547),
    [sym_data_max_format_length] = ACTIONS(1547),
    [sym_data_max_source_length] = ACTIONS(1547),
    [sym_data_max_value_length] = ACTIONS(1547),
    [sym_data_max_label_length] = ACTIONS(1547),
    [sym_data_max_url_length] = ACTIONS(1547),
    [sym_max_last_login_info_length] = ACTIONS(1547),
    [sym_data_type_unknown] = ACTIONS(1547),
    [sym_data_type_string] = ACTIONS(1547),
    [sym_data_type_datetime] = ACTIONS(1547),
    [sym_data_type_date] = ACTIONS(1547),
    [sym_data_type_time] = ACTIONS(1547),
    [sym_data_type_image] = ACTIONS(1547),
    [sym_data_format_url] = ACTIONS(1547),
    [sym_data_format_phone] = ACTIONS(1547),
    [sym_data_format_email] = ACTIONS(1547),
    [sym_data_format_iso8601] = ACTIONS(1547),
    [sym_auditlog_priv] = ACTIONS(1547),
    [sym_configuration_priv] = ACTIONS(1547),
    [sym_deviceconfig_priv] = ACTIONS(1547),
    [sym_ftp_priv] = ACTIONS(1547),
    [sym_http_priv] = ACTIONS(1547),
    [sym_networkconfig_priv] = ACTIONS(1547),
    [sym_programport_priv] = ACTIONS(1547),
    [sym_remoteui_priv] = ACTIONS(1547),
    [sym_securitycontrol_priv] = ACTIONS(1547),
    [sym_softwaremanagement_priv] = ACTIONS(1547),
    [sym_terminal_priv] = ACTIONS(1547),
    [sym_tpadmin_priv] = ACTIONS(1547),
    [sym_user1_priv] = ACTIONS(1547),
    [sym_user2_priv] = ACTIONS(1547),
    [sym_user3_priv] = ACTIONS(1547),
    [sym_user4_priv] = ACTIONS(1547),
    [sym_usermanagement_priv] = ACTIONS(1547),
    [sym_valid_account] = ACTIONS(1547),
    [sym_err_name_invalid] = ACTIONS(1547),
    [sym_err_password_invalid] = ACTIONS(1547),
    [sym_err_invalid_account] = ACTIONS(1547),
    [sym_err_no_authorization] = ACTIONS(1547),
    [sym_err_invalid_parameter] = ACTIONS(1547),
    [sym_err_user_locked_out] = ACTIONS(1547),
    [sym_err_user_account_expired] = ACTIONS(1547),
    [sym_err_general_error] = ACTIONS(1547),
    [sym_err_authentication_error] = ACTIONS(1547),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1547),
    [sym_tls_validate_certificate] = ACTIONS(1547),
    [sym_tdata] = ACTIONS(1547),
    [sym_tchannel] = ACTIONS(1547),
    [sym_tlevel] = ACTIONS(1547),
    [sym_tbutton] = ACTIONS(1547),
    [sym_ttimeline] = ACTIONS(1547),
    [sym_tcustom] = ACTIONS(1547),
    [sym_url_struct] = ACTIONS(1547),
    [sym_dns_struct] = ACTIONS(1547),
    [sym_ip_address_struct] = ACTIONS(1547),
    [sym_dev_info_struct] = ACTIONS(1547),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1547),
    [sym_data_feed] = ACTIONS(1547),
    [sym_data_field] = ACTIONS(1547),
    [sym_data_record] = ACTIONS(1547),
    [sym_wc_data_feed] = ACTIONS(1547),
    [sym_wc_data_field] = ACTIONS(1547),
    [sym_wc_data_record] = ACTIONS(1547),
    [sym_last_login_info] = ACTIONS(1547),
    [sym____reserved___] = ACTIONS(1547),
    [sym_date] = ACTIONS(1547),
    [sym_day] = ACTIONS(1547),
    [sym_time] = ACTIONS(1547),
    [sym_ldate] = ACTIONS(1547),
    [sym_push_device] = ACTIONS(1547),
    [sym_push_channel] = ACTIONS(1547),
    [sym_push_devchan] = ACTIONS(1547),
    [sym_release_device] = ACTIONS(1547),
    [sym_release_channel] = ACTIONS(1547),
    [sym_release_devchan] = ACTIONS(1547),
    [sym_master_slot] = ACTIONS(1547),
    [sym_get_pulse_time] = ACTIONS(1547),
    [sym_get_timer] = ACTIONS(1547),
    [sym_master_sn] = ACTIONS(1547),
    [sym_system_number] = ACTIONS(1547),
    [sym_button] = ACTIONS(1547),
    [sym_dv_channel] = ACTIONS(1547),
    [sym_data] = ACTIONS(1547),
    [sym_level] = ACTIONS(1547),
    [sym_timeline] = ACTIONS(1547),
    [sym_channel] = ACTIONS(1547),
    [sym_custom] = ACTIONS(1547),
    [sym_atoi] = ACTIONS(1547),
    [sym_atol] = ACTIONS(1547),
    [sym_atof] = ACTIONS(1547),
    [sym_itoa] = ACTIONS(1547),
    [sym_itohex] = ACTIONS(1547),
    [sym_hextoi] = ACTIONS(1547),
    [sym_ftoa] = ACTIONS(1547),
    [sym_find_string] = ACTIONS(1547),
    [sym_left_string] = ACTIONS(1547),
    [sym_right_string] = ACTIONS(1547),
    [sym_mid_string] = ACTIONS(1547),
    [sym_remove_string] = ACTIONS(1547),
    [sym_get_buffer_string] = ACTIONS(1547),
    [sym_get_buffer_char] = ACTIONS(1547),
    [sym_get_multi_buffer_string] = ACTIONS(1547),
    [sym_length_string] = ACTIONS(1547),
    [sym_max_length_string] = ACTIONS(1547),
    [sym_set_length_string] = ACTIONS(1547),
    [sym_compare_string] = ACTIONS(1547),
    [sym_upper_string] = ACTIONS(1547),
    [sym_lower_string] = ACTIONS(1547),
    [sym_get_last] = ACTIONS(1547),
    [sym_date_to_day] = ACTIONS(1547),
    [sym_date_to_month] = ACTIONS(1547),
    [sym_date_to_year] = ACTIONS(1547),
    [sym_time_to_hour] = ACTIONS(1547),
    [sym_time_to_minute] = ACTIONS(1547),
    [sym_time_to_second] = ACTIONS(1547),
    [sym_day_of_week] = ACTIONS(1547),
    [sym_set_pulse_time] = ACTIONS(1547),
    [sym_set_timer] = ACTIONS(1547),
    [sym_file_open] = ACTIONS(1547),
    [sym_file_close] = ACTIONS(1547),
    [sym_file_read] = ACTIONS(1547),
    [sym_file_read_line] = ACTIONS(1547),
    [sym_file_write] = ACTIONS(1547),
    [sym_file_write_line] = ACTIONS(1547),
    [sym_file_copy] = ACTIONS(1547),
    [sym_file_delete] = ACTIONS(1547),
    [sym_file_rename] = ACTIONS(1547),
    [sym_file_dir] = ACTIONS(1547),
    [sym_file_getdir] = ACTIONS(1547),
    [sym_file_setdir] = ACTIONS(1547),
    [sym_file_create_dir] = ACTIONS(1547),
    [sym_file_remove_dir] = ACTIONS(1547),
    [sym_file_seek] = ACTIONS(1547),
    [sym_length_array] = ACTIONS(1547),
    [sym_max_length_array] = ACTIONS(1547),
    [sym_set_length_array] = ACTIONS(1547),
    [sym_ip_client_open] = ACTIONS(1547),
    [sym_ip_client_close] = ACTIONS(1547),
    [sym_ip_server_open] = ACTIONS(1547),
    [sym_ip_server_close] = ACTIONS(1547),
    [sym_ip_bound_client_open] = ACTIONS(1547),
    [sym_ip_mc_server_open] = ACTIONS(1547),
    [sym_ip_set_option] = ACTIONS(1547),
    [sym_ssh_client_open] = ACTIONS(1547),
    [sym_ssh_client_close] = ACTIONS(1547),
    [sym_tls_client_open] = ACTIONS(1547),
    [sym_tls_client_close] = ACTIONS(1547),
    [sym_do_push] = ACTIONS(1547),
    [sym_do_push_timed] = ACTIONS(1547),
    [sym_do_release] = ACTIONS(1547),
    [sym_device_id] = ACTIONS(1547),
    [sym_device_info] = ACTIONS(1547),
    [sym_device_id_string] = ACTIONS(1547),
    [sym_device_standby] = ACTIONS(1547),
    [sym_device_wake] = ACTIONS(1547),
    [sym_timeline_create] = ACTIONS(1547),
    [sym_timeline_kill] = ACTIONS(1547),
    [sym_timeline_reload] = ACTIONS(1547),
    [sym_timeline_pause] = ACTIONS(1547),
    [sym_timeline_restart] = ACTIONS(1547),
    [sym_timeline_set] = ACTIONS(1547),
    [sym_timeline_get] = ACTIONS(1547),
    [sym_timeline_active] = ACTIONS(1547),
    [sym_timeline_dynamic_id] = ACTIONS(1547),
    [sym_variable_to_string] = ACTIONS(1547),
    [sym_string_to_variable] = ACTIONS(1547),
    [sym_length_variable_to_string] = ACTIONS(1547),
    [sym_type_cast] = ACTIONS(1547),
    [sym_variable_to_xml] = ACTIONS(1547),
    [sym_xml_to_variable] = ACTIONS(1547),
    [sym_length_variable_to_xml] = ACTIONS(1547),
    [sym_redirect_string] = ACTIONS(1547),
    [sym_random_number] = ACTIONS(1547),
    [sym_get_unique_id] = ACTIONS(1547),
    [sym_get_system_number] = ACTIONS(1547),
    [sym_set_system_number] = ACTIONS(1547),
    [sym_get_serial_number] = ACTIONS(1547),
    [sym_get_master_build] = ACTIONS(1547),
    [sym_set_validation_code] = ACTIONS(1547),
    [sym_reboot] = ACTIONS(1547),
    [sym_load_duet_module] = ACTIONS(1547),
    [sym_unload_duet_module] = ACTIONS(1547),
    [sym_duet_mem_size_set] = ACTIONS(1547),
    [sym_duet_mem_size_get] = ACTIONS(1547),
    [sym_dynamic_application_device] = ACTIONS(1547),
    [sym_dynamic_polled_port] = ACTIONS(1547),
    [sym_static_port_binding] = ACTIONS(1547),
    [sym_static_ip_binding] = ACTIONS(1547),
    [sym_get_dns_list] = ACTIONS(1547),
    [sym_set_dns_list] = ACTIONS(1547),
    [sym_get_ip_address] = ACTIONS(1547),
    [sym_set_ip_address] = ACTIONS(1547),
    [sym_get_url_list] = ACTIONS(1547),
    [sym_add_url_entry] = ACTIONS(1547),
    [sym_delete_url_entry] = ACTIONS(1547),
    [sym_abs_value] = ACTIONS(1547),
    [sym_max_value] = ACTIONS(1547),
    [sym_min_value] = ACTIONS(1547),
    [sym_exp_value] = ACTIONS(1547),
    [sym_log_value] = ACTIONS(1547),
    [sym_log10_value] = ACTIONS(1547),
    [sym_power_value] = ACTIONS(1547),
    [sym_sqrt_value] = ACTIONS(1547),
    [sym_raw_be] = ACTIONS(1547),
    [sym_raw_le] = ACTIONS(1547),
    [sym_internal_queue_size_set] = ACTIONS(1547),
    [sym_internal_queue_size_get] = ACTIONS(1547),
    [sym_internal_threshold_set] = ACTIONS(1547),
    [sym_internal_threshold_get] = ACTIONS(1547),
    [sym_rebuilt_event] = ACTIONS(1547),
    [sym_do_custom_event] = ACTIONS(1547),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1547),
    [sym_clkmgr_set_clk_source] = ACTIONS(1547),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1547),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1547),
    [sym_clkmgr_get_timezone] = ACTIONS(1547),
    [sym_clkmgr_set_timezone] = ACTIONS(1547),
    [sym_clkmgr_get_resync_period] = ACTIONS(1547),
    [sym_clkmgr_set_resync_period] = ACTIONS(1547),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1547),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1547),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1547),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1547),
    [sym_clkmgr_get_timeservers] = ACTIONS(1547),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1547),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1547),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1547),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1547),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1547),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1547),
    [sym_set_log_level] = ACTIONS(1547),
    [sym_get_log_level] = ACTIONS(1547),
    [sym_amx_log] = ACTIONS(1547),
    [sym_smtp_server_config_set] = ACTIONS(1547),
    [sym_smtp_server_config_get] = ACTIONS(1547),
    [sym_smtp_send] = ACTIONS(1547),
    [sym_validate_netlinx_account] = ACTIONS(1547),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1547),
    [sym_authenticate_certificate] = ACTIONS(1547),
    [sym_audit_netlinx_session_event] = ACTIONS(1547),
    [sym_audit_netlinx_generic_event] = ACTIONS(1547),
    [sym_get_available_flash_disk_space] = ACTIONS(1547),
    [sym_get_max_flash_disk_space] = ACTIONS(1547),
    [sym_data_create_feed] = ACTIONS(1547),
    [sym_data_delete_feed] = ACTIONS(1547),
    [sym_data_publish_feed] = ACTIONS(1547),
    [sym_data_get_published_feed] = ACTIONS(1547),
    [sym_data_add_record] = ACTIONS(1547),
    [sym_data_get_event_record] = ACTIONS(1547),
    [sym__wc_data_create_feed] = ACTIONS(1547),
    [sym__wc_data_add_record] = ACTIONS(1547),
    [sym__wc_data_get_event_record] = ACTIONS(1547),
    [sym_format] = ACTIONS(1547),
    [sym_astro_clock] = ACTIONS(1547),
    [sym_set_outdoor_temperature] = ACTIONS(1547),
    [sym_set_virtual_level_count] = ACTIONS(1547),
    [sym_set_virtual_channel_count] = ACTIONS(1547),
    [sym_set_virtual_port_count] = ACTIONS(1547),
    [sym__wc_encode_int] = ACTIONS(1547),
    [sym__wc_decode_int] = ACTIONS(1547),
    [sym__wc_int] = ACTIONS(1547),
    [sym__wc_to_ch_int] = ACTIONS(1547),
    [sym__ch_to_wc_int] = ACTIONS(1547),
    [sym__wc_find_string_int] = ACTIONS(1547),
    [sym__wc_left_string_int] = ACTIONS(1547),
    [sym__wc_lower_string_int] = ACTIONS(1547),
    [sym__wc_mid_string_int] = ACTIONS(1547),
    [sym__wc_remove_string_int] = ACTIONS(1547),
    [sym__wc_right_string_int] = ACTIONS(1547),
    [sym__wc_upper_string_int] = ACTIONS(1547),
    [sym__wc_compare_string_int] = ACTIONS(1547),
    [sym__wc_get_buffer_char_int] = ACTIONS(1547),
    [sym__wc_get_buffer_string_int] = ACTIONS(1547),
    [sym__wc_concat_string_int] = ACTIONS(1547),
    [sym__wc_file_open_int] = ACTIONS(1547),
    [sym__wc_file_close_int] = ACTIONS(1547),
    [sym__wc_file_read_int] = ACTIONS(1547),
    [sym__wc_file_read_line_int] = ACTIONS(1547),
    [sym__wc_file_write_int] = ACTIONS(1547),
    [sym__wc_file_write_line_int] = ACTIONS(1547),
    [anon_sym_EQ] = ACTIONS(1547),
    [aux_sym_preproc_include_token1] = ACTIONS(1547),
    [aux_sym_preproc_define_token1] = ACTIONS(1549),
    [aux_sym_preproc_warn_token1] = ACTIONS(1549),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1549),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1549),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1549),
    [anon_sym_LBRACK] = ACTIONS(1549),
    [anon_sym_LBRACE] = ACTIONS(1549),
    [anon_sym_COLON] = ACTIONS(1549),
    [anon_sym_LPAREN] = ACTIONS(1547),
    [anon_sym_SEMI] = ACTIONS(1549),
    [anon_sym_BANG] = ACTIONS(1547),
    [anon_sym_TILDE] = ACTIONS(1549),
    [anon_sym_DASH] = ACTIONS(1547),
    [anon_sym_PLUS] = ACTIONS(1547),
    [anon_sym_STAR] = ACTIONS(1549),
    [anon_sym_SLASH] = ACTIONS(1547),
    [anon_sym_PERCENT] = ACTIONS(1549),
    [anon_sym_PIPE_PIPE] = ACTIONS(1549),
    [anon_sym_AMP_AMP] = ACTIONS(1549),
    [anon_sym_PIPE] = ACTIONS(1547),
    [anon_sym_CARET] = ACTIONS(1549),
    [anon_sym_AMP] = ACTIONS(1547),
    [anon_sym_EQ_EQ] = ACTIONS(1549),
    [anon_sym_BANG_EQ] = ACTIONS(1549),
    [anon_sym_LT_GT] = ACTIONS(1549),
    [anon_sym_GT] = ACTIONS(1547),
    [anon_sym_GT_EQ] = ACTIONS(1549),
    [anon_sym_LT_EQ] = ACTIONS(1549),
    [anon_sym_LT] = ACTIONS(1547),
    [anon_sym_LT_LT] = ACTIONS(1549),
    [anon_sym_GT_GT] = ACTIONS(1549),
    [anon_sym_DASH_DASH] = ACTIONS(1549),
    [anon_sym_PLUS_PLUS] = ACTIONS(1549),
    [anon_sym_DOT] = ACTIONS(1547),
    [anon_sym_DQUOTE] = ACTIONS(1549),
    [anon_sym_SQUOTE] = ACTIONS(1549),
    [sym_number_literal] = ACTIONS(1549),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1549),
  },
  [STATE(112)] = {
    [sym_identifier] = ACTIONS(1465),
    [sym_program_name_keyword] = ACTIONS(1465),
    [sym_module_name_keyword] = ACTIONS(1465),
    [sym_define_device_keyword] = ACTIONS(1465),
    [sym_define_combine_keyword] = ACTIONS(1465),
    [sym_define_constant_keyword] = ACTIONS(1465),
    [sym_define_type_keyword] = ACTIONS(1465),
    [sym_define_variable_keyword] = ACTIONS(1465),
    [sym_define_system_variable_keyword] = ACTIONS(1465),
    [sym_define_connect_level_keyword] = ACTIONS(1465),
    [sym_define_latching_keyword] = ACTIONS(1465),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1465),
    [sym_define_toggling_keyword] = ACTIONS(1465),
    [sym_define_call_keyword] = ACTIONS(1465),
    [sym_define_function_keyword] = ACTIONS(1465),
    [sym_define_library_function_keyword] = ACTIONS(1465),
    [sym_define_start_keyword] = ACTIONS(1465),
    [sym_define_event_keyword] = ACTIONS(1465),
    [sym_define_program_keyword] = ACTIONS(1465),
    [sym_define_module_keyword] = ACTIONS(1465),
    [sym_char_keyword] = ACTIONS(1465),
    [sym_widechar_keyword] = ACTIONS(1465),
    [sym_integer_keyword] = ACTIONS(1465),
    [sym_sinteger_keyword] = ACTIONS(1465),
    [sym_long_keyword] = ACTIONS(1465),
    [sym_slong_keyword] = ACTIONS(1465),
    [sym_float_keyword] = ACTIONS(1465),
    [sym_double_keyword] = ACTIONS(1465),
    [sym_variant_keyword] = ACTIONS(1465),
    [sym_variantarray_keyword] = ACTIONS(1465),
    [sym_local_var_keyword] = ACTIONS(1465),
    [sym_stack_var_keyword] = ACTIONS(1465),
    [sym_constant_keyword] = ACTIONS(1465),
    [sym_volatile_keyword] = ACTIONS(1465),
    [sym_non_volatile_keyword] = ACTIONS(1465),
    [sym_persistent_keyword] = ACTIONS(1465),
    [sym_dev_keyword] = ACTIONS(1465),
    [sym_devlev_keyword] = ACTIONS(1465),
    [sym_devchan_keyword] = ACTIONS(1465),
    [sym_if_keyword] = ACTIONS(1465),
    [sym_else_keyword] = ACTIONS(1465),
    [sym_switch_keyword] = ACTIONS(1465),
    [sym_case_keyword] = ACTIONS(1465),
    [sym_default_keyword] = ACTIONS(1465),
    [sym_while_keyword] = ACTIONS(1465),
    [sym_for_keyword] = ACTIONS(1465),
    [sym_break_keyword] = ACTIONS(1465),
    [sym_continue_keyword] = ACTIONS(1465),
    [sym_return_keyword] = ACTIONS(1465),
    [sym_select_keyword] = ACTIONS(1465),
    [sym_active_keyword] = ACTIONS(1465),
    [sym_struct_keyword] = ACTIONS(1465),
    [sym_structure_keyword] = ACTIONS(1465),
    [sym_button_event_keyword] = ACTIONS(1465),
    [sym_channel_event_keyword] = ACTIONS(1465),
    [sym_level_event_keyword] = ACTIONS(1465),
    [sym_data_event_keyword] = ACTIONS(1465),
    [sym_timeline_event_keyword] = ACTIONS(1465),
    [sym_custom_event_keyword] = ACTIONS(1465),
    [sym_push_keyword] = ACTIONS(1465),
    [sym_release_keyword] = ACTIONS(1465),
    [aux_sym_on_keyword_token1] = ACTIONS(1465),
    [aux_sym_off_keyword_token1] = ACTIONS(1465),
    [sym_send_level_keyword] = ACTIONS(1465),
    [sym_send_string_keyword] = ACTIONS(1465),
    [sym_send_command_keyword] = ACTIONS(1465),
    [sym_clear_buffer_keyword] = ACTIONS(1465),
    [sym_create_buffer_keyword] = ACTIONS(1465),
    [sym_create_multi_buffer_keyword] = ACTIONS(1465),
    [sym_call_keyword] = ACTIONS(1465),
    [sym_system_call_keyword] = ACTIONS(1465),
    [sym_devchan_to_keyword] = ACTIONS(1465),
    [sym_devchan_min_to_keyword] = ACTIONS(1465),
    [sym_devchan_total_off_keyword] = ACTIONS(1465),
    [sym_devchan_pulse_keyword] = ACTIONS(1465),
    [sym_wait_keyword] = ACTIONS(1465),
    [sym_cancel_wait_keyword] = ACTIONS(1465),
    [sym_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym_tdata] = ACTIONS(1465),
    [sym_tchannel] = ACTIONS(1465),
    [sym_tlevel] = ACTIONS(1465),
    [sym_tbutton] = ACTIONS(1465),
    [sym_ttimeline] = ACTIONS(1465),
    [sym_tcustom] = ACTIONS(1465),
    [sym_url_struct] = ACTIONS(1465),
    [sym_dns_struct] = ACTIONS(1465),
    [sym_ip_address_struct] = ACTIONS(1465),
    [sym_dev_info_struct] = ACTIONS(1465),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1465),
    [sym_data_feed] = ACTIONS(1465),
    [sym_data_field] = ACTIONS(1465),
    [sym_data_record] = ACTIONS(1465),
    [sym_wc_data_feed] = ACTIONS(1465),
    [sym_wc_data_field] = ACTIONS(1465),
    [sym_wc_data_record] = ACTIONS(1465),
    [sym_last_login_info] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [aux_sym_preproc_include_token1] = ACTIONS(1465),
    [aux_sym_preproc_define_token1] = ACTIONS(1469),
    [aux_sym_preproc_warn_token1] = ACTIONS(1469),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_else_token1] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1469),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1469),
  },
  [STATE(113)] = {
    [ts_builtin_sym_end] = ACTIONS(1541),
    [sym_identifier] = ACTIONS(1539),
    [sym_program_name_keyword] = ACTIONS(1539),
    [sym_module_name_keyword] = ACTIONS(1539),
    [sym_define_device_keyword] = ACTIONS(1539),
    [sym_define_combine_keyword] = ACTIONS(1539),
    [sym_define_constant_keyword] = ACTIONS(1539),
    [sym_define_type_keyword] = ACTIONS(1539),
    [sym_define_variable_keyword] = ACTIONS(1539),
    [sym_define_system_variable_keyword] = ACTIONS(1539),
    [sym_define_connect_level_keyword] = ACTIONS(1539),
    [sym_define_latching_keyword] = ACTIONS(1539),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1539),
    [sym_define_toggling_keyword] = ACTIONS(1539),
    [sym_define_call_keyword] = ACTIONS(1539),
    [sym_define_function_keyword] = ACTIONS(1539),
    [sym_define_library_function_keyword] = ACTIONS(1539),
    [sym_define_start_keyword] = ACTIONS(1539),
    [sym_define_event_keyword] = ACTIONS(1539),
    [sym_define_program_keyword] = ACTIONS(1539),
    [sym_define_module_keyword] = ACTIONS(1539),
    [sym_char_keyword] = ACTIONS(1539),
    [sym_widechar_keyword] = ACTIONS(1539),
    [sym_integer_keyword] = ACTIONS(1539),
    [sym_sinteger_keyword] = ACTIONS(1539),
    [sym_long_keyword] = ACTIONS(1539),
    [sym_slong_keyword] = ACTIONS(1539),
    [sym_float_keyword] = ACTIONS(1539),
    [sym_double_keyword] = ACTIONS(1539),
    [sym_variant_keyword] = ACTIONS(1539),
    [sym_variantarray_keyword] = ACTIONS(1539),
    [sym_local_var_keyword] = ACTIONS(1539),
    [sym_stack_var_keyword] = ACTIONS(1539),
    [sym_constant_keyword] = ACTIONS(1539),
    [sym_volatile_keyword] = ACTIONS(1539),
    [sym_non_volatile_keyword] = ACTIONS(1539),
    [sym_persistent_keyword] = ACTIONS(1539),
    [sym_dev_keyword] = ACTIONS(1539),
    [sym_devlev_keyword] = ACTIONS(1539),
    [sym_devchan_keyword] = ACTIONS(1539),
    [sym_if_keyword] = ACTIONS(1539),
    [sym_else_keyword] = ACTIONS(1539),
    [sym_switch_keyword] = ACTIONS(1539),
    [sym_case_keyword] = ACTIONS(1539),
    [sym_default_keyword] = ACTIONS(1539),
    [sym_while_keyword] = ACTIONS(1539),
    [sym_for_keyword] = ACTIONS(1539),
    [sym_break_keyword] = ACTIONS(1539),
    [sym_continue_keyword] = ACTIONS(1539),
    [sym_return_keyword] = ACTIONS(1539),
    [sym_select_keyword] = ACTIONS(1539),
    [sym_active_keyword] = ACTIONS(1539),
    [sym_struct_keyword] = ACTIONS(1539),
    [sym_structure_keyword] = ACTIONS(1539),
    [sym_button_event_keyword] = ACTIONS(1539),
    [sym_channel_event_keyword] = ACTIONS(1539),
    [sym_level_event_keyword] = ACTIONS(1539),
    [sym_data_event_keyword] = ACTIONS(1539),
    [sym_timeline_event_keyword] = ACTIONS(1539),
    [sym_custom_event_keyword] = ACTIONS(1539),
    [sym_push_keyword] = ACTIONS(1539),
    [sym_release_keyword] = ACTIONS(1539),
    [aux_sym_on_keyword_token1] = ACTIONS(1539),
    [aux_sym_off_keyword_token1] = ACTIONS(1539),
    [sym_send_level_keyword] = ACTIONS(1539),
    [sym_send_string_keyword] = ACTIONS(1539),
    [sym_send_command_keyword] = ACTIONS(1539),
    [sym_clear_buffer_keyword] = ACTIONS(1539),
    [sym_create_buffer_keyword] = ACTIONS(1539),
    [sym_create_multi_buffer_keyword] = ACTIONS(1539),
    [sym_call_keyword] = ACTIONS(1539),
    [sym_system_call_keyword] = ACTIONS(1539),
    [sym_devchan_to_keyword] = ACTIONS(1539),
    [sym_devchan_min_to_keyword] = ACTIONS(1539),
    [sym_devchan_total_off_keyword] = ACTIONS(1539),
    [sym_devchan_pulse_keyword] = ACTIONS(1539),
    [sym_wait_keyword] = ACTIONS(1539),
    [sym_cancel_wait_keyword] = ACTIONS(1539),
    [sym_wait_until_keyword] = ACTIONS(1539),
    [sym_cancel_wait_until_keyword] = ACTIONS(1539),
    [sym_cancel_all_wait_keyword] = ACTIONS(1539),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1539),
    [sym_and] = ACTIONS(1539),
    [sym_or] = ACTIONS(1539),
    [sym_not] = ACTIONS(1539),
    [sym_xor] = ACTIONS(1539),
    [sym_band] = ACTIONS(1539),
    [sym_bor] = ACTIONS(1539),
    [sym_bxor] = ACTIONS(1539),
    [sym_bnot] = ACTIONS(1539),
    [sym_lshift] = ACTIONS(1539),
    [sym_rshift] = ACTIONS(1539),
    [sym___file__] = ACTIONS(1539),
    [sym___date__] = ACTIONS(1539),
    [sym___ldate__] = ACTIONS(1539),
    [sym___time__] = ACTIONS(1539),
    [sym___line__] = ACTIONS(1539),
    [sym___name__] = ACTIONS(1539),
    [sym___netlinx__] = ACTIONS(1539),
    [sym_netlinx_axi_version] = ACTIONS(1539),
    [sym_url_flg_tcp] = ACTIONS(1539),
    [sym_url_flg_accinfopresent] = ACTIONS(1539),
    [sym_url_flg_temp] = ACTIONS(1539),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1539),
    [sym_url_flg_stat_mask] = ACTIONS(1539),
    [sym_url_flg_stat_lookup] = ACTIONS(1539),
    [sym_url_flg_stat_connecting] = ACTIONS(1539),
    [sym_url_flg_stat_waiting] = ACTIONS(1539),
    [sym_url_flg_stat_connected] = ACTIONS(1539),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1539),
    [sym_first_virtual_device] = ACTIONS(1539),
    [sym_first_local_port] = ACTIONS(1539),
    [sym_dynamic_virtual_device] = ACTIONS(1539),
    [sym_timeline_once] = ACTIONS(1539),
    [sym_timeline_repeat] = ACTIONS(1539),
    [sym_timeline_absolute] = ACTIONS(1539),
    [sym_timeline_relative] = ACTIONS(1539),
    [sym_do_push_timed_infinite] = ACTIONS(1539),
    [sym_source_type_no_address] = ACTIONS(1539),
    [sym_source_type_neuron_id] = ACTIONS(1539),
    [sym_source_type_ip_address] = ACTIONS(1539),
    [sym_source_type_axlink] = ACTIONS(1539),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1539),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1539),
    [sym_source_type_ip_socket_address] = ACTIONS(1539),
    [sym_source_type_rs232] = ACTIONS(1539),
    [sym_source_type_internal] = ACTIONS(1539),
    [sym_source_type_mac_address] = ACTIONS(1539),
    [sym_source_type_ipv4_port] = ACTIONS(1539),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1539),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1539),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1539),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1539),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1539),
    [sym_true] = ACTIONS(1539),
    [sym_false] = ACTIONS(1539),
    [sym_file_read_only] = ACTIONS(1539),
    [sym_file_rw_new] = ACTIONS(1539),
    [sym_file_rw_append] = ACTIONS(1539),
    [sym_ip_tcp] = ACTIONS(1539),
    [sym_ip_udp] = ACTIONS(1539),
    [sym_ip_udp_2way] = ACTIONS(1539),
    [sym_xml_encode_types] = ACTIONS(1539),
    [sym_xml_encode_char_as_list] = ACTIONS(1539),
    [sym_xml_encode_le] = ACTIONS(1539),
    [sym_xml_decode_types] = ACTIONS(1539),
    [sym_xml_decode_no_preserve] = ACTIONS(1539),
    [sym_duet_dev_type_display_device] = ACTIONS(1539),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1539),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1539),
    [sym_duet_dev_type_io_device] = ACTIONS(1539),
    [sym_duet_dev_type_relay_device] = ACTIONS(1539),
    [sym_duet_dev_type_ups] = ACTIONS(1539),
    [sym_duet_dev_type_amplifier] = ACTIONS(1539),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1539),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1539),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1539),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1539),
    [sym_duet_dev_type_camera] = ACTIONS(1539),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1539),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1539),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1539),
    [sym_duet_dev_type_dss] = ACTIONS(1539),
    [sym_duet_dev_type_dvr] = ACTIONS(1539),
    [sym_duet_dev_type_disc_device] = ACTIONS(1539),
    [sym_duet_dev_type_document_camera] = ACTIONS(1539),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1539),
    [sym_duet_dev_type_hvac] = ACTIONS(1539),
    [sym_duet_dev_type_keypad] = ACTIONS(1539),
    [sym_duet_dev_type_light] = ACTIONS(1539),
    [sym_duet_dev_type_monitor] = ACTIONS(1539),
    [sym_duet_dev_type_motor] = ACTIONS(1539),
    [sym_duet_dev_type_multi_window] = ACTIONS(1539),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1539),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1539),
    [sym_duet_dev_type_receiver] = ACTIONS(1539),
    [sym_duet_dev_type_security_system] = ACTIONS(1539),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1539),
    [sym_duet_dev_type_settop_box] = ACTIONS(1539),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1539),
    [sym_duet_dev_type_switcher] = ACTIONS(1539),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1539),
    [sym_duet_dev_type_tv] = ACTIONS(1539),
    [sym_duet_dev_type_utility] = ACTIONS(1539),
    [sym_duet_dev_type_vcr] = ACTIONS(1539),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1539),
    [sym_duet_dev_type_video_processor] = ACTIONS(1539),
    [sym_duet_dev_type_video_projector] = ACTIONS(1539),
    [sym_duet_dev_type_video_wall] = ACTIONS(1539),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1539),
    [sym_duet_dev_type_weather] = ACTIONS(1539),
    [sym_duet_dev_not_polled] = ACTIONS(1539),
    [sym_duet_dev_polled] = ACTIONS(1539),
    [sym_clkmgr_mode_network] = ACTIONS(1539),
    [sym_clkmgr_mode_standalone] = ACTIONS(1539),
    [sym_normal_standby] = ACTIONS(1539),
    [sym_normal_wake] = ACTIONS(1539),
    [sym_ip_multicast_ttl_option] = ACTIONS(1539),
    [sym_ip_ttl_subnet] = ACTIONS(1539),
    [sym_ip_ttl_site] = ACTIONS(1539),
    [sym_ip_ttl_region] = ACTIONS(1539),
    [sym_ip_ttl_continent] = ACTIONS(1539),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1539),
    [sym_ip_nodelay_on] = ACTIONS(1539),
    [sym_ip_nodelay_off] = ACTIONS(1539),
    [sym_amx_error] = ACTIONS(1539),
    [sym_amx_warning] = ACTIONS(1539),
    [sym_amx_info] = ACTIONS(1539),
    [sym_amx_debug] = ACTIONS(1539),
    [sym_netlinx_login_success] = ACTIONS(1539),
    [sym_netlinx_login_fail] = ACTIONS(1539),
    [sym_netlinx_logout] = ACTIONS(1539),
    [sym_null_str] = ACTIONS(1539),
    [sym_smtp_address] = ACTIONS(1539),
    [sym_smtp_port_number] = ACTIONS(1539),
    [sym_smtp_username] = ACTIONS(1539),
    [sym_smtp_password] = ACTIONS(1539),
    [sym_smtp_require_tls] = ACTIONS(1539),
    [sym_smtp_from] = ACTIONS(1539),
    [sym_smtp_tls_true] = ACTIONS(1539),
    [sym_smtp_tls_false] = ACTIONS(1539),
    [sym_listview_on_row_select_event] = ACTIONS(1539),
    [sym_data_structure_unknown] = ACTIONS(1539),
    [sym_data_structure_datafeed] = ACTIONS(1539),
    [sym_data_structure_datarecordset] = ACTIONS(1539),
    [sym_data_structure_datarecord] = ACTIONS(1539),
    [sym_data_structure_datafield] = ACTIONS(1539),
    [sym_data_max_content_field_count] = ACTIONS(1539),
    [sym_data_max_metadata_field_count] = ACTIONS(1539),
    [sym_data_max_id_length] = ACTIONS(1539),
    [sym_data_max_name_length] = ACTIONS(1539),
    [sym_data_max_description_length] = ACTIONS(1539),
    [sym_data_max_type_length] = ACTIONS(1539),
    [sym_data_max_format_length] = ACTIONS(1539),
    [sym_data_max_source_length] = ACTIONS(1539),
    [sym_data_max_value_length] = ACTIONS(1539),
    [sym_data_max_label_length] = ACTIONS(1539),
    [sym_data_max_url_length] = ACTIONS(1539),
    [sym_max_last_login_info_length] = ACTIONS(1539),
    [sym_data_type_unknown] = ACTIONS(1539),
    [sym_data_type_string] = ACTIONS(1539),
    [sym_data_type_datetime] = ACTIONS(1539),
    [sym_data_type_date] = ACTIONS(1539),
    [sym_data_type_time] = ACTIONS(1539),
    [sym_data_type_image] = ACTIONS(1539),
    [sym_data_format_url] = ACTIONS(1539),
    [sym_data_format_phone] = ACTIONS(1539),
    [sym_data_format_email] = ACTIONS(1539),
    [sym_data_format_iso8601] = ACTIONS(1539),
    [sym_auditlog_priv] = ACTIONS(1539),
    [sym_configuration_priv] = ACTIONS(1539),
    [sym_deviceconfig_priv] = ACTIONS(1539),
    [sym_ftp_priv] = ACTIONS(1539),
    [sym_http_priv] = ACTIONS(1539),
    [sym_networkconfig_priv] = ACTIONS(1539),
    [sym_programport_priv] = ACTIONS(1539),
    [sym_remoteui_priv] = ACTIONS(1539),
    [sym_securitycontrol_priv] = ACTIONS(1539),
    [sym_softwaremanagement_priv] = ACTIONS(1539),
    [sym_terminal_priv] = ACTIONS(1539),
    [sym_tpadmin_priv] = ACTIONS(1539),
    [sym_user1_priv] = ACTIONS(1539),
    [sym_user2_priv] = ACTIONS(1539),
    [sym_user3_priv] = ACTIONS(1539),
    [sym_user4_priv] = ACTIONS(1539),
    [sym_usermanagement_priv] = ACTIONS(1539),
    [sym_valid_account] = ACTIONS(1539),
    [sym_err_name_invalid] = ACTIONS(1539),
    [sym_err_password_invalid] = ACTIONS(1539),
    [sym_err_invalid_account] = ACTIONS(1539),
    [sym_err_no_authorization] = ACTIONS(1539),
    [sym_err_invalid_parameter] = ACTIONS(1539),
    [sym_err_user_locked_out] = ACTIONS(1539),
    [sym_err_user_account_expired] = ACTIONS(1539),
    [sym_err_general_error] = ACTIONS(1539),
    [sym_err_authentication_error] = ACTIONS(1539),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1539),
    [sym_tls_validate_certificate] = ACTIONS(1539),
    [sym_tdata] = ACTIONS(1539),
    [sym_tchannel] = ACTIONS(1539),
    [sym_tlevel] = ACTIONS(1539),
    [sym_tbutton] = ACTIONS(1539),
    [sym_ttimeline] = ACTIONS(1539),
    [sym_tcustom] = ACTIONS(1539),
    [sym_url_struct] = ACTIONS(1539),
    [sym_dns_struct] = ACTIONS(1539),
    [sym_ip_address_struct] = ACTIONS(1539),
    [sym_dev_info_struct] = ACTIONS(1539),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1539),
    [sym_data_feed] = ACTIONS(1539),
    [sym_data_field] = ACTIONS(1539),
    [sym_data_record] = ACTIONS(1539),
    [sym_wc_data_feed] = ACTIONS(1539),
    [sym_wc_data_field] = ACTIONS(1539),
    [sym_wc_data_record] = ACTIONS(1539),
    [sym_last_login_info] = ACTIONS(1539),
    [sym____reserved___] = ACTIONS(1539),
    [sym_date] = ACTIONS(1539),
    [sym_day] = ACTIONS(1539),
    [sym_time] = ACTIONS(1539),
    [sym_ldate] = ACTIONS(1539),
    [sym_push_device] = ACTIONS(1539),
    [sym_push_channel] = ACTIONS(1539),
    [sym_push_devchan] = ACTIONS(1539),
    [sym_release_device] = ACTIONS(1539),
    [sym_release_channel] = ACTIONS(1539),
    [sym_release_devchan] = ACTIONS(1539),
    [sym_master_slot] = ACTIONS(1539),
    [sym_get_pulse_time] = ACTIONS(1539),
    [sym_get_timer] = ACTIONS(1539),
    [sym_master_sn] = ACTIONS(1539),
    [sym_system_number] = ACTIONS(1539),
    [sym_button] = ACTIONS(1539),
    [sym_dv_channel] = ACTIONS(1539),
    [sym_data] = ACTIONS(1539),
    [sym_level] = ACTIONS(1539),
    [sym_timeline] = ACTIONS(1539),
    [sym_channel] = ACTIONS(1539),
    [sym_custom] = ACTIONS(1539),
    [sym_atoi] = ACTIONS(1539),
    [sym_atol] = ACTIONS(1539),
    [sym_atof] = ACTIONS(1539),
    [sym_itoa] = ACTIONS(1539),
    [sym_itohex] = ACTIONS(1539),
    [sym_hextoi] = ACTIONS(1539),
    [sym_ftoa] = ACTIONS(1539),
    [sym_find_string] = ACTIONS(1539),
    [sym_left_string] = ACTIONS(1539),
    [sym_right_string] = ACTIONS(1539),
    [sym_mid_string] = ACTIONS(1539),
    [sym_remove_string] = ACTIONS(1539),
    [sym_get_buffer_string] = ACTIONS(1539),
    [sym_get_buffer_char] = ACTIONS(1539),
    [sym_get_multi_buffer_string] = ACTIONS(1539),
    [sym_length_string] = ACTIONS(1539),
    [sym_max_length_string] = ACTIONS(1539),
    [sym_set_length_string] = ACTIONS(1539),
    [sym_compare_string] = ACTIONS(1539),
    [sym_upper_string] = ACTIONS(1539),
    [sym_lower_string] = ACTIONS(1539),
    [sym_get_last] = ACTIONS(1539),
    [sym_date_to_day] = ACTIONS(1539),
    [sym_date_to_month] = ACTIONS(1539),
    [sym_date_to_year] = ACTIONS(1539),
    [sym_time_to_hour] = ACTIONS(1539),
    [sym_time_to_minute] = ACTIONS(1539),
    [sym_time_to_second] = ACTIONS(1539),
    [sym_day_of_week] = ACTIONS(1539),
    [sym_set_pulse_time] = ACTIONS(1539),
    [sym_set_timer] = ACTIONS(1539),
    [sym_file_open] = ACTIONS(1539),
    [sym_file_close] = ACTIONS(1539),
    [sym_file_read] = ACTIONS(1539),
    [sym_file_read_line] = ACTIONS(1539),
    [sym_file_write] = ACTIONS(1539),
    [sym_file_write_line] = ACTIONS(1539),
    [sym_file_copy] = ACTIONS(1539),
    [sym_file_delete] = ACTIONS(1539),
    [sym_file_rename] = ACTIONS(1539),
    [sym_file_dir] = ACTIONS(1539),
    [sym_file_getdir] = ACTIONS(1539),
    [sym_file_setdir] = ACTIONS(1539),
    [sym_file_create_dir] = ACTIONS(1539),
    [sym_file_remove_dir] = ACTIONS(1539),
    [sym_file_seek] = ACTIONS(1539),
    [sym_length_array] = ACTIONS(1539),
    [sym_max_length_array] = ACTIONS(1539),
    [sym_set_length_array] = ACTIONS(1539),
    [sym_ip_client_open] = ACTIONS(1539),
    [sym_ip_client_close] = ACTIONS(1539),
    [sym_ip_server_open] = ACTIONS(1539),
    [sym_ip_server_close] = ACTIONS(1539),
    [sym_ip_bound_client_open] = ACTIONS(1539),
    [sym_ip_mc_server_open] = ACTIONS(1539),
    [sym_ip_set_option] = ACTIONS(1539),
    [sym_ssh_client_open] = ACTIONS(1539),
    [sym_ssh_client_close] = ACTIONS(1539),
    [sym_tls_client_open] = ACTIONS(1539),
    [sym_tls_client_close] = ACTIONS(1539),
    [sym_do_push] = ACTIONS(1539),
    [sym_do_push_timed] = ACTIONS(1539),
    [sym_do_release] = ACTIONS(1539),
    [sym_device_id] = ACTIONS(1539),
    [sym_device_info] = ACTIONS(1539),
    [sym_device_id_string] = ACTIONS(1539),
    [sym_device_standby] = ACTIONS(1539),
    [sym_device_wake] = ACTIONS(1539),
    [sym_timeline_create] = ACTIONS(1539),
    [sym_timeline_kill] = ACTIONS(1539),
    [sym_timeline_reload] = ACTIONS(1539),
    [sym_timeline_pause] = ACTIONS(1539),
    [sym_timeline_restart] = ACTIONS(1539),
    [sym_timeline_set] = ACTIONS(1539),
    [sym_timeline_get] = ACTIONS(1539),
    [sym_timeline_active] = ACTIONS(1539),
    [sym_timeline_dynamic_id] = ACTIONS(1539),
    [sym_variable_to_string] = ACTIONS(1539),
    [sym_string_to_variable] = ACTIONS(1539),
    [sym_length_variable_to_string] = ACTIONS(1539),
    [sym_type_cast] = ACTIONS(1539),
    [sym_variable_to_xml] = ACTIONS(1539),
    [sym_xml_to_variable] = ACTIONS(1539),
    [sym_length_variable_to_xml] = ACTIONS(1539),
    [sym_redirect_string] = ACTIONS(1539),
    [sym_random_number] = ACTIONS(1539),
    [sym_get_unique_id] = ACTIONS(1539),
    [sym_get_system_number] = ACTIONS(1539),
    [sym_set_system_number] = ACTIONS(1539),
    [sym_get_serial_number] = ACTIONS(1539),
    [sym_get_master_build] = ACTIONS(1539),
    [sym_set_validation_code] = ACTIONS(1539),
    [sym_reboot] = ACTIONS(1539),
    [sym_load_duet_module] = ACTIONS(1539),
    [sym_unload_duet_module] = ACTIONS(1539),
    [sym_duet_mem_size_set] = ACTIONS(1539),
    [sym_duet_mem_size_get] = ACTIONS(1539),
    [sym_dynamic_application_device] = ACTIONS(1539),
    [sym_dynamic_polled_port] = ACTIONS(1539),
    [sym_static_port_binding] = ACTIONS(1539),
    [sym_static_ip_binding] = ACTIONS(1539),
    [sym_get_dns_list] = ACTIONS(1539),
    [sym_set_dns_list] = ACTIONS(1539),
    [sym_get_ip_address] = ACTIONS(1539),
    [sym_set_ip_address] = ACTIONS(1539),
    [sym_get_url_list] = ACTIONS(1539),
    [sym_add_url_entry] = ACTIONS(1539),
    [sym_delete_url_entry] = ACTIONS(1539),
    [sym_abs_value] = ACTIONS(1539),
    [sym_max_value] = ACTIONS(1539),
    [sym_min_value] = ACTIONS(1539),
    [sym_exp_value] = ACTIONS(1539),
    [sym_log_value] = ACTIONS(1539),
    [sym_log10_value] = ACTIONS(1539),
    [sym_power_value] = ACTIONS(1539),
    [sym_sqrt_value] = ACTIONS(1539),
    [sym_raw_be] = ACTIONS(1539),
    [sym_raw_le] = ACTIONS(1539),
    [sym_internal_queue_size_set] = ACTIONS(1539),
    [sym_internal_queue_size_get] = ACTIONS(1539),
    [sym_internal_threshold_set] = ACTIONS(1539),
    [sym_internal_threshold_get] = ACTIONS(1539),
    [sym_rebuilt_event] = ACTIONS(1539),
    [sym_do_custom_event] = ACTIONS(1539),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1539),
    [sym_clkmgr_set_clk_source] = ACTIONS(1539),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1539),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1539),
    [sym_clkmgr_get_timezone] = ACTIONS(1539),
    [sym_clkmgr_set_timezone] = ACTIONS(1539),
    [sym_clkmgr_get_resync_period] = ACTIONS(1539),
    [sym_clkmgr_set_resync_period] = ACTIONS(1539),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1539),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1539),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1539),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1539),
    [sym_clkmgr_get_timeservers] = ACTIONS(1539),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1539),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1539),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1539),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1539),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1539),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1539),
    [sym_set_log_level] = ACTIONS(1539),
    [sym_get_log_level] = ACTIONS(1539),
    [sym_amx_log] = ACTIONS(1539),
    [sym_smtp_server_config_set] = ACTIONS(1539),
    [sym_smtp_server_config_get] = ACTIONS(1539),
    [sym_smtp_send] = ACTIONS(1539),
    [sym_validate_netlinx_account] = ACTIONS(1539),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1539),
    [sym_authenticate_certificate] = ACTIONS(1539),
    [sym_audit_netlinx_session_event] = ACTIONS(1539),
    [sym_audit_netlinx_generic_event] = ACTIONS(1539),
    [sym_get_available_flash_disk_space] = ACTIONS(1539),
    [sym_get_max_flash_disk_space] = ACTIONS(1539),
    [sym_data_create_feed] = ACTIONS(1539),
    [sym_data_delete_feed] = ACTIONS(1539),
    [sym_data_publish_feed] = ACTIONS(1539),
    [sym_data_get_published_feed] = ACTIONS(1539),
    [sym_data_add_record] = ACTIONS(1539),
    [sym_data_get_event_record] = ACTIONS(1539),
    [sym__wc_data_create_feed] = ACTIONS(1539),
    [sym__wc_data_add_record] = ACTIONS(1539),
    [sym__wc_data_get_event_record] = ACTIONS(1539),
    [sym_format] = ACTIONS(1539),
    [sym_astro_clock] = ACTIONS(1539),
    [sym_set_outdoor_temperature] = ACTIONS(1539),
    [sym_set_virtual_level_count] = ACTIONS(1539),
    [sym_set_virtual_channel_count] = ACTIONS(1539),
    [sym_set_virtual_port_count] = ACTIONS(1539),
    [sym__wc_encode_int] = ACTIONS(1539),
    [sym__wc_decode_int] = ACTIONS(1539),
    [sym__wc_int] = ACTIONS(1539),
    [sym__wc_to_ch_int] = ACTIONS(1539),
    [sym__ch_to_wc_int] = ACTIONS(1539),
    [sym__wc_find_string_int] = ACTIONS(1539),
    [sym__wc_left_string_int] = ACTIONS(1539),
    [sym__wc_lower_string_int] = ACTIONS(1539),
    [sym__wc_mid_string_int] = ACTIONS(1539),
    [sym__wc_remove_string_int] = ACTIONS(1539),
    [sym__wc_right_string_int] = ACTIONS(1539),
    [sym__wc_upper_string_int] = ACTIONS(1539),
    [sym__wc_compare_string_int] = ACTIONS(1539),
    [sym__wc_get_buffer_char_int] = ACTIONS(1539),
    [sym__wc_get_buffer_string_int] = ACTIONS(1539),
    [sym__wc_concat_string_int] = ACTIONS(1539),
    [sym__wc_file_open_int] = ACTIONS(1539),
    [sym__wc_file_close_int] = ACTIONS(1539),
    [sym__wc_file_read_int] = ACTIONS(1539),
    [sym__wc_file_read_line_int] = ACTIONS(1539),
    [sym__wc_file_write_int] = ACTIONS(1539),
    [sym__wc_file_write_line_int] = ACTIONS(1539),
    [anon_sym_EQ] = ACTIONS(1539),
    [aux_sym_preproc_include_token1] = ACTIONS(1539),
    [aux_sym_preproc_define_token1] = ACTIONS(1541),
    [aux_sym_preproc_warn_token1] = ACTIONS(1541),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1541),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1541),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1541),
    [anon_sym_LBRACK] = ACTIONS(1541),
    [anon_sym_LBRACE] = ACTIONS(1541),
    [anon_sym_COLON] = ACTIONS(1541),
    [anon_sym_LPAREN] = ACTIONS(1539),
    [anon_sym_SEMI] = ACTIONS(1541),
    [anon_sym_BANG] = ACTIONS(1539),
    [anon_sym_TILDE] = ACTIONS(1541),
    [anon_sym_DASH] = ACTIONS(1539),
    [anon_sym_PLUS] = ACTIONS(1539),
    [anon_sym_STAR] = ACTIONS(1541),
    [anon_sym_SLASH] = ACTIONS(1539),
    [anon_sym_PERCENT] = ACTIONS(1541),
    [anon_sym_PIPE_PIPE] = ACTIONS(1541),
    [anon_sym_AMP_AMP] = ACTIONS(1541),
    [anon_sym_PIPE] = ACTIONS(1539),
    [anon_sym_CARET] = ACTIONS(1541),
    [anon_sym_AMP] = ACTIONS(1539),
    [anon_sym_EQ_EQ] = ACTIONS(1541),
    [anon_sym_BANG_EQ] = ACTIONS(1541),
    [anon_sym_LT_GT] = ACTIONS(1541),
    [anon_sym_GT] = ACTIONS(1539),
    [anon_sym_GT_EQ] = ACTIONS(1541),
    [anon_sym_LT_EQ] = ACTIONS(1541),
    [anon_sym_LT] = ACTIONS(1539),
    [anon_sym_LT_LT] = ACTIONS(1541),
    [anon_sym_GT_GT] = ACTIONS(1541),
    [anon_sym_DASH_DASH] = ACTIONS(1541),
    [anon_sym_PLUS_PLUS] = ACTIONS(1541),
    [anon_sym_DOT] = ACTIONS(1539),
    [anon_sym_DQUOTE] = ACTIONS(1541),
    [anon_sym_SQUOTE] = ACTIONS(1541),
    [sym_number_literal] = ACTIONS(1541),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1541),
  },
  [STATE(114)] = {
    [ts_builtin_sym_end] = ACTIONS(1553),
    [sym_identifier] = ACTIONS(1551),
    [sym_program_name_keyword] = ACTIONS(1551),
    [sym_module_name_keyword] = ACTIONS(1551),
    [sym_define_device_keyword] = ACTIONS(1551),
    [sym_define_combine_keyword] = ACTIONS(1551),
    [sym_define_constant_keyword] = ACTIONS(1551),
    [sym_define_type_keyword] = ACTIONS(1551),
    [sym_define_variable_keyword] = ACTIONS(1551),
    [sym_define_system_variable_keyword] = ACTIONS(1551),
    [sym_define_connect_level_keyword] = ACTIONS(1551),
    [sym_define_latching_keyword] = ACTIONS(1551),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1551),
    [sym_define_toggling_keyword] = ACTIONS(1551),
    [sym_define_call_keyword] = ACTIONS(1551),
    [sym_define_function_keyword] = ACTIONS(1551),
    [sym_define_library_function_keyword] = ACTIONS(1551),
    [sym_define_start_keyword] = ACTIONS(1551),
    [sym_define_event_keyword] = ACTIONS(1551),
    [sym_define_program_keyword] = ACTIONS(1551),
    [sym_define_module_keyword] = ACTIONS(1551),
    [sym_char_keyword] = ACTIONS(1551),
    [sym_widechar_keyword] = ACTIONS(1551),
    [sym_integer_keyword] = ACTIONS(1551),
    [sym_sinteger_keyword] = ACTIONS(1551),
    [sym_long_keyword] = ACTIONS(1551),
    [sym_slong_keyword] = ACTIONS(1551),
    [sym_float_keyword] = ACTIONS(1551),
    [sym_double_keyword] = ACTIONS(1551),
    [sym_variant_keyword] = ACTIONS(1551),
    [sym_variantarray_keyword] = ACTIONS(1551),
    [sym_local_var_keyword] = ACTIONS(1551),
    [sym_stack_var_keyword] = ACTIONS(1551),
    [sym_constant_keyword] = ACTIONS(1551),
    [sym_volatile_keyword] = ACTIONS(1551),
    [sym_non_volatile_keyword] = ACTIONS(1551),
    [sym_persistent_keyword] = ACTIONS(1551),
    [sym_dev_keyword] = ACTIONS(1551),
    [sym_devlev_keyword] = ACTIONS(1551),
    [sym_devchan_keyword] = ACTIONS(1551),
    [sym_if_keyword] = ACTIONS(1551),
    [sym_else_keyword] = ACTIONS(1551),
    [sym_switch_keyword] = ACTIONS(1551),
    [sym_case_keyword] = ACTIONS(1551),
    [sym_default_keyword] = ACTIONS(1551),
    [sym_while_keyword] = ACTIONS(1551),
    [sym_for_keyword] = ACTIONS(1551),
    [sym_break_keyword] = ACTIONS(1551),
    [sym_continue_keyword] = ACTIONS(1551),
    [sym_return_keyword] = ACTIONS(1551),
    [sym_select_keyword] = ACTIONS(1551),
    [sym_active_keyword] = ACTIONS(1551),
    [sym_struct_keyword] = ACTIONS(1551),
    [sym_structure_keyword] = ACTIONS(1551),
    [sym_button_event_keyword] = ACTIONS(1551),
    [sym_channel_event_keyword] = ACTIONS(1551),
    [sym_level_event_keyword] = ACTIONS(1551),
    [sym_data_event_keyword] = ACTIONS(1551),
    [sym_timeline_event_keyword] = ACTIONS(1551),
    [sym_custom_event_keyword] = ACTIONS(1551),
    [sym_push_keyword] = ACTIONS(1551),
    [sym_release_keyword] = ACTIONS(1551),
    [aux_sym_on_keyword_token1] = ACTIONS(1551),
    [aux_sym_off_keyword_token1] = ACTIONS(1551),
    [sym_send_level_keyword] = ACTIONS(1551),
    [sym_send_string_keyword] = ACTIONS(1551),
    [sym_send_command_keyword] = ACTIONS(1551),
    [sym_clear_buffer_keyword] = ACTIONS(1551),
    [sym_create_buffer_keyword] = ACTIONS(1551),
    [sym_create_multi_buffer_keyword] = ACTIONS(1551),
    [sym_call_keyword] = ACTIONS(1551),
    [sym_system_call_keyword] = ACTIONS(1551),
    [sym_devchan_to_keyword] = ACTIONS(1551),
    [sym_devchan_min_to_keyword] = ACTIONS(1551),
    [sym_devchan_total_off_keyword] = ACTIONS(1551),
    [sym_devchan_pulse_keyword] = ACTIONS(1551),
    [sym_wait_keyword] = ACTIONS(1551),
    [sym_cancel_wait_keyword] = ACTIONS(1551),
    [sym_wait_until_keyword] = ACTIONS(1551),
    [sym_cancel_wait_until_keyword] = ACTIONS(1551),
    [sym_cancel_all_wait_keyword] = ACTIONS(1551),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1551),
    [sym_and] = ACTIONS(1551),
    [sym_or] = ACTIONS(1551),
    [sym_not] = ACTIONS(1551),
    [sym_xor] = ACTIONS(1551),
    [sym_band] = ACTIONS(1551),
    [sym_bor] = ACTIONS(1551),
    [sym_bxor] = ACTIONS(1551),
    [sym_bnot] = ACTIONS(1551),
    [sym_lshift] = ACTIONS(1551),
    [sym_rshift] = ACTIONS(1551),
    [sym___file__] = ACTIONS(1551),
    [sym___date__] = ACTIONS(1551),
    [sym___ldate__] = ACTIONS(1551),
    [sym___time__] = ACTIONS(1551),
    [sym___line__] = ACTIONS(1551),
    [sym___name__] = ACTIONS(1551),
    [sym___netlinx__] = ACTIONS(1551),
    [sym_netlinx_axi_version] = ACTIONS(1551),
    [sym_url_flg_tcp] = ACTIONS(1551),
    [sym_url_flg_accinfopresent] = ACTIONS(1551),
    [sym_url_flg_temp] = ACTIONS(1551),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1551),
    [sym_url_flg_stat_mask] = ACTIONS(1551),
    [sym_url_flg_stat_lookup] = ACTIONS(1551),
    [sym_url_flg_stat_connecting] = ACTIONS(1551),
    [sym_url_flg_stat_waiting] = ACTIONS(1551),
    [sym_url_flg_stat_connected] = ACTIONS(1551),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1551),
    [sym_first_virtual_device] = ACTIONS(1551),
    [sym_first_local_port] = ACTIONS(1551),
    [sym_dynamic_virtual_device] = ACTIONS(1551),
    [sym_timeline_once] = ACTIONS(1551),
    [sym_timeline_repeat] = ACTIONS(1551),
    [sym_timeline_absolute] = ACTIONS(1551),
    [sym_timeline_relative] = ACTIONS(1551),
    [sym_do_push_timed_infinite] = ACTIONS(1551),
    [sym_source_type_no_address] = ACTIONS(1551),
    [sym_source_type_neuron_id] = ACTIONS(1551),
    [sym_source_type_ip_address] = ACTIONS(1551),
    [sym_source_type_axlink] = ACTIONS(1551),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1551),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1551),
    [sym_source_type_ip_socket_address] = ACTIONS(1551),
    [sym_source_type_rs232] = ACTIONS(1551),
    [sym_source_type_internal] = ACTIONS(1551),
    [sym_source_type_mac_address] = ACTIONS(1551),
    [sym_source_type_ipv4_port] = ACTIONS(1551),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1551),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1551),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1551),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1551),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1551),
    [sym_true] = ACTIONS(1551),
    [sym_false] = ACTIONS(1551),
    [sym_file_read_only] = ACTIONS(1551),
    [sym_file_rw_new] = ACTIONS(1551),
    [sym_file_rw_append] = ACTIONS(1551),
    [sym_ip_tcp] = ACTIONS(1551),
    [sym_ip_udp] = ACTIONS(1551),
    [sym_ip_udp_2way] = ACTIONS(1551),
    [sym_xml_encode_types] = ACTIONS(1551),
    [sym_xml_encode_char_as_list] = ACTIONS(1551),
    [sym_xml_encode_le] = ACTIONS(1551),
    [sym_xml_decode_types] = ACTIONS(1551),
    [sym_xml_decode_no_preserve] = ACTIONS(1551),
    [sym_duet_dev_type_display_device] = ACTIONS(1551),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1551),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1551),
    [sym_duet_dev_type_io_device] = ACTIONS(1551),
    [sym_duet_dev_type_relay_device] = ACTIONS(1551),
    [sym_duet_dev_type_ups] = ACTIONS(1551),
    [sym_duet_dev_type_amplifier] = ACTIONS(1551),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1551),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1551),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1551),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1551),
    [sym_duet_dev_type_camera] = ACTIONS(1551),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1551),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1551),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1551),
    [sym_duet_dev_type_dss] = ACTIONS(1551),
    [sym_duet_dev_type_dvr] = ACTIONS(1551),
    [sym_duet_dev_type_disc_device] = ACTIONS(1551),
    [sym_duet_dev_type_document_camera] = ACTIONS(1551),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1551),
    [sym_duet_dev_type_hvac] = ACTIONS(1551),
    [sym_duet_dev_type_keypad] = ACTIONS(1551),
    [sym_duet_dev_type_light] = ACTIONS(1551),
    [sym_duet_dev_type_monitor] = ACTIONS(1551),
    [sym_duet_dev_type_motor] = ACTIONS(1551),
    [sym_duet_dev_type_multi_window] = ACTIONS(1551),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1551),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1551),
    [sym_duet_dev_type_receiver] = ACTIONS(1551),
    [sym_duet_dev_type_security_system] = ACTIONS(1551),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1551),
    [sym_duet_dev_type_settop_box] = ACTIONS(1551),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1551),
    [sym_duet_dev_type_switcher] = ACTIONS(1551),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1551),
    [sym_duet_dev_type_tv] = ACTIONS(1551),
    [sym_duet_dev_type_utility] = ACTIONS(1551),
    [sym_duet_dev_type_vcr] = ACTIONS(1551),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1551),
    [sym_duet_dev_type_video_processor] = ACTIONS(1551),
    [sym_duet_dev_type_video_projector] = ACTIONS(1551),
    [sym_duet_dev_type_video_wall] = ACTIONS(1551),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1551),
    [sym_duet_dev_type_weather] = ACTIONS(1551),
    [sym_duet_dev_not_polled] = ACTIONS(1551),
    [sym_duet_dev_polled] = ACTIONS(1551),
    [sym_clkmgr_mode_network] = ACTIONS(1551),
    [sym_clkmgr_mode_standalone] = ACTIONS(1551),
    [sym_normal_standby] = ACTIONS(1551),
    [sym_normal_wake] = ACTIONS(1551),
    [sym_ip_multicast_ttl_option] = ACTIONS(1551),
    [sym_ip_ttl_subnet] = ACTIONS(1551),
    [sym_ip_ttl_site] = ACTIONS(1551),
    [sym_ip_ttl_region] = ACTIONS(1551),
    [sym_ip_ttl_continent] = ACTIONS(1551),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1551),
    [sym_ip_nodelay_on] = ACTIONS(1551),
    [sym_ip_nodelay_off] = ACTIONS(1551),
    [sym_amx_error] = ACTIONS(1551),
    [sym_amx_warning] = ACTIONS(1551),
    [sym_amx_info] = ACTIONS(1551),
    [sym_amx_debug] = ACTIONS(1551),
    [sym_netlinx_login_success] = ACTIONS(1551),
    [sym_netlinx_login_fail] = ACTIONS(1551),
    [sym_netlinx_logout] = ACTIONS(1551),
    [sym_null_str] = ACTIONS(1551),
    [sym_smtp_address] = ACTIONS(1551),
    [sym_smtp_port_number] = ACTIONS(1551),
    [sym_smtp_username] = ACTIONS(1551),
    [sym_smtp_password] = ACTIONS(1551),
    [sym_smtp_require_tls] = ACTIONS(1551),
    [sym_smtp_from] = ACTIONS(1551),
    [sym_smtp_tls_true] = ACTIONS(1551),
    [sym_smtp_tls_false] = ACTIONS(1551),
    [sym_listview_on_row_select_event] = ACTIONS(1551),
    [sym_data_structure_unknown] = ACTIONS(1551),
    [sym_data_structure_datafeed] = ACTIONS(1551),
    [sym_data_structure_datarecordset] = ACTIONS(1551),
    [sym_data_structure_datarecord] = ACTIONS(1551),
    [sym_data_structure_datafield] = ACTIONS(1551),
    [sym_data_max_content_field_count] = ACTIONS(1551),
    [sym_data_max_metadata_field_count] = ACTIONS(1551),
    [sym_data_max_id_length] = ACTIONS(1551),
    [sym_data_max_name_length] = ACTIONS(1551),
    [sym_data_max_description_length] = ACTIONS(1551),
    [sym_data_max_type_length] = ACTIONS(1551),
    [sym_data_max_format_length] = ACTIONS(1551),
    [sym_data_max_source_length] = ACTIONS(1551),
    [sym_data_max_value_length] = ACTIONS(1551),
    [sym_data_max_label_length] = ACTIONS(1551),
    [sym_data_max_url_length] = ACTIONS(1551),
    [sym_max_last_login_info_length] = ACTIONS(1551),
    [sym_data_type_unknown] = ACTIONS(1551),
    [sym_data_type_string] = ACTIONS(1551),
    [sym_data_type_datetime] = ACTIONS(1551),
    [sym_data_type_date] = ACTIONS(1551),
    [sym_data_type_time] = ACTIONS(1551),
    [sym_data_type_image] = ACTIONS(1551),
    [sym_data_format_url] = ACTIONS(1551),
    [sym_data_format_phone] = ACTIONS(1551),
    [sym_data_format_email] = ACTIONS(1551),
    [sym_data_format_iso8601] = ACTIONS(1551),
    [sym_auditlog_priv] = ACTIONS(1551),
    [sym_configuration_priv] = ACTIONS(1551),
    [sym_deviceconfig_priv] = ACTIONS(1551),
    [sym_ftp_priv] = ACTIONS(1551),
    [sym_http_priv] = ACTIONS(1551),
    [sym_networkconfig_priv] = ACTIONS(1551),
    [sym_programport_priv] = ACTIONS(1551),
    [sym_remoteui_priv] = ACTIONS(1551),
    [sym_securitycontrol_priv] = ACTIONS(1551),
    [sym_softwaremanagement_priv] = ACTIONS(1551),
    [sym_terminal_priv] = ACTIONS(1551),
    [sym_tpadmin_priv] = ACTIONS(1551),
    [sym_user1_priv] = ACTIONS(1551),
    [sym_user2_priv] = ACTIONS(1551),
    [sym_user3_priv] = ACTIONS(1551),
    [sym_user4_priv] = ACTIONS(1551),
    [sym_usermanagement_priv] = ACTIONS(1551),
    [sym_valid_account] = ACTIONS(1551),
    [sym_err_name_invalid] = ACTIONS(1551),
    [sym_err_password_invalid] = ACTIONS(1551),
    [sym_err_invalid_account] = ACTIONS(1551),
    [sym_err_no_authorization] = ACTIONS(1551),
    [sym_err_invalid_parameter] = ACTIONS(1551),
    [sym_err_user_locked_out] = ACTIONS(1551),
    [sym_err_user_account_expired] = ACTIONS(1551),
    [sym_err_general_error] = ACTIONS(1551),
    [sym_err_authentication_error] = ACTIONS(1551),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1551),
    [sym_tls_validate_certificate] = ACTIONS(1551),
    [sym_tdata] = ACTIONS(1551),
    [sym_tchannel] = ACTIONS(1551),
    [sym_tlevel] = ACTIONS(1551),
    [sym_tbutton] = ACTIONS(1551),
    [sym_ttimeline] = ACTIONS(1551),
    [sym_tcustom] = ACTIONS(1551),
    [sym_url_struct] = ACTIONS(1551),
    [sym_dns_struct] = ACTIONS(1551),
    [sym_ip_address_struct] = ACTIONS(1551),
    [sym_dev_info_struct] = ACTIONS(1551),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1551),
    [sym_data_feed] = ACTIONS(1551),
    [sym_data_field] = ACTIONS(1551),
    [sym_data_record] = ACTIONS(1551),
    [sym_wc_data_feed] = ACTIONS(1551),
    [sym_wc_data_field] = ACTIONS(1551),
    [sym_wc_data_record] = ACTIONS(1551),
    [sym_last_login_info] = ACTIONS(1551),
    [sym____reserved___] = ACTIONS(1551),
    [sym_date] = ACTIONS(1551),
    [sym_day] = ACTIONS(1551),
    [sym_time] = ACTIONS(1551),
    [sym_ldate] = ACTIONS(1551),
    [sym_push_device] = ACTIONS(1551),
    [sym_push_channel] = ACTIONS(1551),
    [sym_push_devchan] = ACTIONS(1551),
    [sym_release_device] = ACTIONS(1551),
    [sym_release_channel] = ACTIONS(1551),
    [sym_release_devchan] = ACTIONS(1551),
    [sym_master_slot] = ACTIONS(1551),
    [sym_get_pulse_time] = ACTIONS(1551),
    [sym_get_timer] = ACTIONS(1551),
    [sym_master_sn] = ACTIONS(1551),
    [sym_system_number] = ACTIONS(1551),
    [sym_button] = ACTIONS(1551),
    [sym_dv_channel] = ACTIONS(1551),
    [sym_data] = ACTIONS(1551),
    [sym_level] = ACTIONS(1551),
    [sym_timeline] = ACTIONS(1551),
    [sym_channel] = ACTIONS(1551),
    [sym_custom] = ACTIONS(1551),
    [sym_atoi] = ACTIONS(1551),
    [sym_atol] = ACTIONS(1551),
    [sym_atof] = ACTIONS(1551),
    [sym_itoa] = ACTIONS(1551),
    [sym_itohex] = ACTIONS(1551),
    [sym_hextoi] = ACTIONS(1551),
    [sym_ftoa] = ACTIONS(1551),
    [sym_find_string] = ACTIONS(1551),
    [sym_left_string] = ACTIONS(1551),
    [sym_right_string] = ACTIONS(1551),
    [sym_mid_string] = ACTIONS(1551),
    [sym_remove_string] = ACTIONS(1551),
    [sym_get_buffer_string] = ACTIONS(1551),
    [sym_get_buffer_char] = ACTIONS(1551),
    [sym_get_multi_buffer_string] = ACTIONS(1551),
    [sym_length_string] = ACTIONS(1551),
    [sym_max_length_string] = ACTIONS(1551),
    [sym_set_length_string] = ACTIONS(1551),
    [sym_compare_string] = ACTIONS(1551),
    [sym_upper_string] = ACTIONS(1551),
    [sym_lower_string] = ACTIONS(1551),
    [sym_get_last] = ACTIONS(1551),
    [sym_date_to_day] = ACTIONS(1551),
    [sym_date_to_month] = ACTIONS(1551),
    [sym_date_to_year] = ACTIONS(1551),
    [sym_time_to_hour] = ACTIONS(1551),
    [sym_time_to_minute] = ACTIONS(1551),
    [sym_time_to_second] = ACTIONS(1551),
    [sym_day_of_week] = ACTIONS(1551),
    [sym_set_pulse_time] = ACTIONS(1551),
    [sym_set_timer] = ACTIONS(1551),
    [sym_file_open] = ACTIONS(1551),
    [sym_file_close] = ACTIONS(1551),
    [sym_file_read] = ACTIONS(1551),
    [sym_file_read_line] = ACTIONS(1551),
    [sym_file_write] = ACTIONS(1551),
    [sym_file_write_line] = ACTIONS(1551),
    [sym_file_copy] = ACTIONS(1551),
    [sym_file_delete] = ACTIONS(1551),
    [sym_file_rename] = ACTIONS(1551),
    [sym_file_dir] = ACTIONS(1551),
    [sym_file_getdir] = ACTIONS(1551),
    [sym_file_setdir] = ACTIONS(1551),
    [sym_file_create_dir] = ACTIONS(1551),
    [sym_file_remove_dir] = ACTIONS(1551),
    [sym_file_seek] = ACTIONS(1551),
    [sym_length_array] = ACTIONS(1551),
    [sym_max_length_array] = ACTIONS(1551),
    [sym_set_length_array] = ACTIONS(1551),
    [sym_ip_client_open] = ACTIONS(1551),
    [sym_ip_client_close] = ACTIONS(1551),
    [sym_ip_server_open] = ACTIONS(1551),
    [sym_ip_server_close] = ACTIONS(1551),
    [sym_ip_bound_client_open] = ACTIONS(1551),
    [sym_ip_mc_server_open] = ACTIONS(1551),
    [sym_ip_set_option] = ACTIONS(1551),
    [sym_ssh_client_open] = ACTIONS(1551),
    [sym_ssh_client_close] = ACTIONS(1551),
    [sym_tls_client_open] = ACTIONS(1551),
    [sym_tls_client_close] = ACTIONS(1551),
    [sym_do_push] = ACTIONS(1551),
    [sym_do_push_timed] = ACTIONS(1551),
    [sym_do_release] = ACTIONS(1551),
    [sym_device_id] = ACTIONS(1551),
    [sym_device_info] = ACTIONS(1551),
    [sym_device_id_string] = ACTIONS(1551),
    [sym_device_standby] = ACTIONS(1551),
    [sym_device_wake] = ACTIONS(1551),
    [sym_timeline_create] = ACTIONS(1551),
    [sym_timeline_kill] = ACTIONS(1551),
    [sym_timeline_reload] = ACTIONS(1551),
    [sym_timeline_pause] = ACTIONS(1551),
    [sym_timeline_restart] = ACTIONS(1551),
    [sym_timeline_set] = ACTIONS(1551),
    [sym_timeline_get] = ACTIONS(1551),
    [sym_timeline_active] = ACTIONS(1551),
    [sym_timeline_dynamic_id] = ACTIONS(1551),
    [sym_variable_to_string] = ACTIONS(1551),
    [sym_string_to_variable] = ACTIONS(1551),
    [sym_length_variable_to_string] = ACTIONS(1551),
    [sym_type_cast] = ACTIONS(1551),
    [sym_variable_to_xml] = ACTIONS(1551),
    [sym_xml_to_variable] = ACTIONS(1551),
    [sym_length_variable_to_xml] = ACTIONS(1551),
    [sym_redirect_string] = ACTIONS(1551),
    [sym_random_number] = ACTIONS(1551),
    [sym_get_unique_id] = ACTIONS(1551),
    [sym_get_system_number] = ACTIONS(1551),
    [sym_set_system_number] = ACTIONS(1551),
    [sym_get_serial_number] = ACTIONS(1551),
    [sym_get_master_build] = ACTIONS(1551),
    [sym_set_validation_code] = ACTIONS(1551),
    [sym_reboot] = ACTIONS(1551),
    [sym_load_duet_module] = ACTIONS(1551),
    [sym_unload_duet_module] = ACTIONS(1551),
    [sym_duet_mem_size_set] = ACTIONS(1551),
    [sym_duet_mem_size_get] = ACTIONS(1551),
    [sym_dynamic_application_device] = ACTIONS(1551),
    [sym_dynamic_polled_port] = ACTIONS(1551),
    [sym_static_port_binding] = ACTIONS(1551),
    [sym_static_ip_binding] = ACTIONS(1551),
    [sym_get_dns_list] = ACTIONS(1551),
    [sym_set_dns_list] = ACTIONS(1551),
    [sym_get_ip_address] = ACTIONS(1551),
    [sym_set_ip_address] = ACTIONS(1551),
    [sym_get_url_list] = ACTIONS(1551),
    [sym_add_url_entry] = ACTIONS(1551),
    [sym_delete_url_entry] = ACTIONS(1551),
    [sym_abs_value] = ACTIONS(1551),
    [sym_max_value] = ACTIONS(1551),
    [sym_min_value] = ACTIONS(1551),
    [sym_exp_value] = ACTIONS(1551),
    [sym_log_value] = ACTIONS(1551),
    [sym_log10_value] = ACTIONS(1551),
    [sym_power_value] = ACTIONS(1551),
    [sym_sqrt_value] = ACTIONS(1551),
    [sym_raw_be] = ACTIONS(1551),
    [sym_raw_le] = ACTIONS(1551),
    [sym_internal_queue_size_set] = ACTIONS(1551),
    [sym_internal_queue_size_get] = ACTIONS(1551),
    [sym_internal_threshold_set] = ACTIONS(1551),
    [sym_internal_threshold_get] = ACTIONS(1551),
    [sym_rebuilt_event] = ACTIONS(1551),
    [sym_do_custom_event] = ACTIONS(1551),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1551),
    [sym_clkmgr_set_clk_source] = ACTIONS(1551),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1551),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1551),
    [sym_clkmgr_get_timezone] = ACTIONS(1551),
    [sym_clkmgr_set_timezone] = ACTIONS(1551),
    [sym_clkmgr_get_resync_period] = ACTIONS(1551),
    [sym_clkmgr_set_resync_period] = ACTIONS(1551),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1551),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1551),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1551),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1551),
    [sym_clkmgr_get_timeservers] = ACTIONS(1551),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1551),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1551),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1551),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1551),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1551),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1551),
    [sym_set_log_level] = ACTIONS(1551),
    [sym_get_log_level] = ACTIONS(1551),
    [sym_amx_log] = ACTIONS(1551),
    [sym_smtp_server_config_set] = ACTIONS(1551),
    [sym_smtp_server_config_get] = ACTIONS(1551),
    [sym_smtp_send] = ACTIONS(1551),
    [sym_validate_netlinx_account] = ACTIONS(1551),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1551),
    [sym_authenticate_certificate] = ACTIONS(1551),
    [sym_audit_netlinx_session_event] = ACTIONS(1551),
    [sym_audit_netlinx_generic_event] = ACTIONS(1551),
    [sym_get_available_flash_disk_space] = ACTIONS(1551),
    [sym_get_max_flash_disk_space] = ACTIONS(1551),
    [sym_data_create_feed] = ACTIONS(1551),
    [sym_data_delete_feed] = ACTIONS(1551),
    [sym_data_publish_feed] = ACTIONS(1551),
    [sym_data_get_published_feed] = ACTIONS(1551),
    [sym_data_add_record] = ACTIONS(1551),
    [sym_data_get_event_record] = ACTIONS(1551),
    [sym__wc_data_create_feed] = ACTIONS(1551),
    [sym__wc_data_add_record] = ACTIONS(1551),
    [sym__wc_data_get_event_record] = ACTIONS(1551),
    [sym_format] = ACTIONS(1551),
    [sym_astro_clock] = ACTIONS(1551),
    [sym_set_outdoor_temperature] = ACTIONS(1551),
    [sym_set_virtual_level_count] = ACTIONS(1551),
    [sym_set_virtual_channel_count] = ACTIONS(1551),
    [sym_set_virtual_port_count] = ACTIONS(1551),
    [sym__wc_encode_int] = ACTIONS(1551),
    [sym__wc_decode_int] = ACTIONS(1551),
    [sym__wc_int] = ACTIONS(1551),
    [sym__wc_to_ch_int] = ACTIONS(1551),
    [sym__ch_to_wc_int] = ACTIONS(1551),
    [sym__wc_find_string_int] = ACTIONS(1551),
    [sym__wc_left_string_int] = ACTIONS(1551),
    [sym__wc_lower_string_int] = ACTIONS(1551),
    [sym__wc_mid_string_int] = ACTIONS(1551),
    [sym__wc_remove_string_int] = ACTIONS(1551),
    [sym__wc_right_string_int] = ACTIONS(1551),
    [sym__wc_upper_string_int] = ACTIONS(1551),
    [sym__wc_compare_string_int] = ACTIONS(1551),
    [sym__wc_get_buffer_char_int] = ACTIONS(1551),
    [sym__wc_get_buffer_string_int] = ACTIONS(1551),
    [sym__wc_concat_string_int] = ACTIONS(1551),
    [sym__wc_file_open_int] = ACTIONS(1551),
    [sym__wc_file_close_int] = ACTIONS(1551),
    [sym__wc_file_read_int] = ACTIONS(1551),
    [sym__wc_file_read_line_int] = ACTIONS(1551),
    [sym__wc_file_write_int] = ACTIONS(1551),
    [sym__wc_file_write_line_int] = ACTIONS(1551),
    [anon_sym_EQ] = ACTIONS(1551),
    [aux_sym_preproc_include_token1] = ACTIONS(1551),
    [aux_sym_preproc_define_token1] = ACTIONS(1553),
    [aux_sym_preproc_warn_token1] = ACTIONS(1553),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1553),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1553),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1553),
    [anon_sym_LBRACK] = ACTIONS(1553),
    [anon_sym_LBRACE] = ACTIONS(1553),
    [anon_sym_COLON] = ACTIONS(1553),
    [anon_sym_LPAREN] = ACTIONS(1551),
    [anon_sym_SEMI] = ACTIONS(1553),
    [anon_sym_BANG] = ACTIONS(1551),
    [anon_sym_TILDE] = ACTIONS(1553),
    [anon_sym_DASH] = ACTIONS(1551),
    [anon_sym_PLUS] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(1553),
    [anon_sym_SLASH] = ACTIONS(1551),
    [anon_sym_PERCENT] = ACTIONS(1553),
    [anon_sym_PIPE_PIPE] = ACTIONS(1553),
    [anon_sym_AMP_AMP] = ACTIONS(1553),
    [anon_sym_PIPE] = ACTIONS(1551),
    [anon_sym_CARET] = ACTIONS(1553),
    [anon_sym_AMP] = ACTIONS(1551),
    [anon_sym_EQ_EQ] = ACTIONS(1553),
    [anon_sym_BANG_EQ] = ACTIONS(1553),
    [anon_sym_LT_GT] = ACTIONS(1553),
    [anon_sym_GT] = ACTIONS(1551),
    [anon_sym_GT_EQ] = ACTIONS(1553),
    [anon_sym_LT_EQ] = ACTIONS(1553),
    [anon_sym_LT] = ACTIONS(1551),
    [anon_sym_LT_LT] = ACTIONS(1553),
    [anon_sym_GT_GT] = ACTIONS(1553),
    [anon_sym_DASH_DASH] = ACTIONS(1553),
    [anon_sym_PLUS_PLUS] = ACTIONS(1553),
    [anon_sym_DOT] = ACTIONS(1551),
    [anon_sym_DQUOTE] = ACTIONS(1553),
    [anon_sym_SQUOTE] = ACTIONS(1553),
    [sym_number_literal] = ACTIONS(1553),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1553),
  },
  [STATE(115)] = {
    [ts_builtin_sym_end] = ACTIONS(1469),
    [sym_identifier] = ACTIONS(1465),
    [sym_program_name_keyword] = ACTIONS(1465),
    [sym_module_name_keyword] = ACTIONS(1465),
    [sym_define_device_keyword] = ACTIONS(1465),
    [sym_define_combine_keyword] = ACTIONS(1465),
    [sym_define_constant_keyword] = ACTIONS(1465),
    [sym_define_type_keyword] = ACTIONS(1465),
    [sym_define_variable_keyword] = ACTIONS(1465),
    [sym_define_system_variable_keyword] = ACTIONS(1465),
    [sym_define_connect_level_keyword] = ACTIONS(1465),
    [sym_define_latching_keyword] = ACTIONS(1465),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1465),
    [sym_define_toggling_keyword] = ACTIONS(1465),
    [sym_define_call_keyword] = ACTIONS(1465),
    [sym_define_function_keyword] = ACTIONS(1465),
    [sym_define_library_function_keyword] = ACTIONS(1465),
    [sym_define_start_keyword] = ACTIONS(1465),
    [sym_define_event_keyword] = ACTIONS(1465),
    [sym_define_program_keyword] = ACTIONS(1465),
    [sym_define_module_keyword] = ACTIONS(1465),
    [sym_char_keyword] = ACTIONS(1465),
    [sym_widechar_keyword] = ACTIONS(1465),
    [sym_integer_keyword] = ACTIONS(1465),
    [sym_sinteger_keyword] = ACTIONS(1465),
    [sym_long_keyword] = ACTIONS(1465),
    [sym_slong_keyword] = ACTIONS(1465),
    [sym_float_keyword] = ACTIONS(1465),
    [sym_double_keyword] = ACTIONS(1465),
    [sym_variant_keyword] = ACTIONS(1465),
    [sym_variantarray_keyword] = ACTIONS(1465),
    [sym_local_var_keyword] = ACTIONS(1465),
    [sym_stack_var_keyword] = ACTIONS(1465),
    [sym_constant_keyword] = ACTIONS(1465),
    [sym_volatile_keyword] = ACTIONS(1465),
    [sym_non_volatile_keyword] = ACTIONS(1465),
    [sym_persistent_keyword] = ACTIONS(1465),
    [sym_dev_keyword] = ACTIONS(1465),
    [sym_devlev_keyword] = ACTIONS(1465),
    [sym_devchan_keyword] = ACTIONS(1465),
    [sym_if_keyword] = ACTIONS(1465),
    [sym_else_keyword] = ACTIONS(1465),
    [sym_switch_keyword] = ACTIONS(1465),
    [sym_case_keyword] = ACTIONS(1465),
    [sym_default_keyword] = ACTIONS(1465),
    [sym_while_keyword] = ACTIONS(1465),
    [sym_for_keyword] = ACTIONS(1465),
    [sym_break_keyword] = ACTIONS(1465),
    [sym_continue_keyword] = ACTIONS(1465),
    [sym_return_keyword] = ACTIONS(1465),
    [sym_select_keyword] = ACTIONS(1465),
    [sym_active_keyword] = ACTIONS(1465),
    [sym_struct_keyword] = ACTIONS(1465),
    [sym_structure_keyword] = ACTIONS(1465),
    [sym_button_event_keyword] = ACTIONS(1465),
    [sym_channel_event_keyword] = ACTIONS(1465),
    [sym_level_event_keyword] = ACTIONS(1465),
    [sym_data_event_keyword] = ACTIONS(1465),
    [sym_timeline_event_keyword] = ACTIONS(1465),
    [sym_custom_event_keyword] = ACTIONS(1465),
    [sym_push_keyword] = ACTIONS(1465),
    [sym_release_keyword] = ACTIONS(1465),
    [aux_sym_on_keyword_token1] = ACTIONS(1465),
    [aux_sym_off_keyword_token1] = ACTIONS(1465),
    [sym_send_level_keyword] = ACTIONS(1465),
    [sym_send_string_keyword] = ACTIONS(1465),
    [sym_send_command_keyword] = ACTIONS(1465),
    [sym_clear_buffer_keyword] = ACTIONS(1465),
    [sym_create_buffer_keyword] = ACTIONS(1465),
    [sym_create_multi_buffer_keyword] = ACTIONS(1465),
    [sym_call_keyword] = ACTIONS(1465),
    [sym_system_call_keyword] = ACTIONS(1465),
    [sym_devchan_to_keyword] = ACTIONS(1465),
    [sym_devchan_min_to_keyword] = ACTIONS(1465),
    [sym_devchan_total_off_keyword] = ACTIONS(1465),
    [sym_devchan_pulse_keyword] = ACTIONS(1465),
    [sym_wait_keyword] = ACTIONS(1465),
    [sym_cancel_wait_keyword] = ACTIONS(1465),
    [sym_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym_tdata] = ACTIONS(1465),
    [sym_tchannel] = ACTIONS(1465),
    [sym_tlevel] = ACTIONS(1465),
    [sym_tbutton] = ACTIONS(1465),
    [sym_ttimeline] = ACTIONS(1465),
    [sym_tcustom] = ACTIONS(1465),
    [sym_url_struct] = ACTIONS(1465),
    [sym_dns_struct] = ACTIONS(1465),
    [sym_ip_address_struct] = ACTIONS(1465),
    [sym_dev_info_struct] = ACTIONS(1465),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1465),
    [sym_data_feed] = ACTIONS(1465),
    [sym_data_field] = ACTIONS(1465),
    [sym_data_record] = ACTIONS(1465),
    [sym_wc_data_feed] = ACTIONS(1465),
    [sym_wc_data_field] = ACTIONS(1465),
    [sym_wc_data_record] = ACTIONS(1465),
    [sym_last_login_info] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [anon_sym_EQ] = ACTIONS(1559),
    [aux_sym_preproc_include_token1] = ACTIONS(1465),
    [aux_sym_preproc_define_token1] = ACTIONS(1469),
    [aux_sym_preproc_warn_token1] = ACTIONS(1469),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1469),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1469),
  },
  [STATE(116)] = {
    [ts_builtin_sym_end] = ACTIONS(1469),
    [sym_identifier] = ACTIONS(1465),
    [sym_program_name_keyword] = ACTIONS(1465),
    [sym_module_name_keyword] = ACTIONS(1465),
    [sym_define_device_keyword] = ACTIONS(1465),
    [sym_define_combine_keyword] = ACTIONS(1465),
    [sym_define_constant_keyword] = ACTIONS(1465),
    [sym_define_type_keyword] = ACTIONS(1465),
    [sym_define_variable_keyword] = ACTIONS(1465),
    [sym_define_system_variable_keyword] = ACTIONS(1465),
    [sym_define_connect_level_keyword] = ACTIONS(1465),
    [sym_define_latching_keyword] = ACTIONS(1465),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1465),
    [sym_define_toggling_keyword] = ACTIONS(1465),
    [sym_define_call_keyword] = ACTIONS(1465),
    [sym_define_function_keyword] = ACTIONS(1465),
    [sym_define_library_function_keyword] = ACTIONS(1465),
    [sym_define_start_keyword] = ACTIONS(1465),
    [sym_define_event_keyword] = ACTIONS(1465),
    [sym_define_program_keyword] = ACTIONS(1465),
    [sym_define_module_keyword] = ACTIONS(1465),
    [sym_char_keyword] = ACTIONS(1465),
    [sym_widechar_keyword] = ACTIONS(1465),
    [sym_integer_keyword] = ACTIONS(1465),
    [sym_sinteger_keyword] = ACTIONS(1465),
    [sym_long_keyword] = ACTIONS(1465),
    [sym_slong_keyword] = ACTIONS(1465),
    [sym_float_keyword] = ACTIONS(1465),
    [sym_double_keyword] = ACTIONS(1465),
    [sym_variant_keyword] = ACTIONS(1465),
    [sym_variantarray_keyword] = ACTIONS(1465),
    [sym_local_var_keyword] = ACTIONS(1465),
    [sym_stack_var_keyword] = ACTIONS(1465),
    [sym_constant_keyword] = ACTIONS(1465),
    [sym_volatile_keyword] = ACTIONS(1465),
    [sym_non_volatile_keyword] = ACTIONS(1465),
    [sym_persistent_keyword] = ACTIONS(1465),
    [sym_dev_keyword] = ACTIONS(1465),
    [sym_devlev_keyword] = ACTIONS(1465),
    [sym_devchan_keyword] = ACTIONS(1465),
    [sym_if_keyword] = ACTIONS(1465),
    [sym_else_keyword] = ACTIONS(1465),
    [sym_switch_keyword] = ACTIONS(1465),
    [sym_case_keyword] = ACTIONS(1465),
    [sym_default_keyword] = ACTIONS(1465),
    [sym_while_keyword] = ACTIONS(1465),
    [sym_for_keyword] = ACTIONS(1465),
    [sym_break_keyword] = ACTIONS(1465),
    [sym_continue_keyword] = ACTIONS(1465),
    [sym_return_keyword] = ACTIONS(1465),
    [sym_select_keyword] = ACTIONS(1465),
    [sym_active_keyword] = ACTIONS(1465),
    [sym_struct_keyword] = ACTIONS(1465),
    [sym_structure_keyword] = ACTIONS(1465),
    [sym_button_event_keyword] = ACTIONS(1465),
    [sym_channel_event_keyword] = ACTIONS(1465),
    [sym_level_event_keyword] = ACTIONS(1465),
    [sym_data_event_keyword] = ACTIONS(1465),
    [sym_timeline_event_keyword] = ACTIONS(1465),
    [sym_custom_event_keyword] = ACTIONS(1465),
    [sym_push_keyword] = ACTIONS(1465),
    [sym_release_keyword] = ACTIONS(1465),
    [aux_sym_on_keyword_token1] = ACTIONS(1465),
    [aux_sym_off_keyword_token1] = ACTIONS(1465),
    [sym_send_level_keyword] = ACTIONS(1465),
    [sym_send_string_keyword] = ACTIONS(1465),
    [sym_send_command_keyword] = ACTIONS(1465),
    [sym_clear_buffer_keyword] = ACTIONS(1465),
    [sym_create_buffer_keyword] = ACTIONS(1465),
    [sym_create_multi_buffer_keyword] = ACTIONS(1465),
    [sym_call_keyword] = ACTIONS(1465),
    [sym_system_call_keyword] = ACTIONS(1465),
    [sym_devchan_to_keyword] = ACTIONS(1465),
    [sym_devchan_min_to_keyword] = ACTIONS(1465),
    [sym_devchan_total_off_keyword] = ACTIONS(1465),
    [sym_devchan_pulse_keyword] = ACTIONS(1465),
    [sym_wait_keyword] = ACTIONS(1465),
    [sym_cancel_wait_keyword] = ACTIONS(1465),
    [sym_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym_tdata] = ACTIONS(1465),
    [sym_tchannel] = ACTIONS(1465),
    [sym_tlevel] = ACTIONS(1465),
    [sym_tbutton] = ACTIONS(1465),
    [sym_ttimeline] = ACTIONS(1465),
    [sym_tcustom] = ACTIONS(1465),
    [sym_url_struct] = ACTIONS(1465),
    [sym_dns_struct] = ACTIONS(1465),
    [sym_ip_address_struct] = ACTIONS(1465),
    [sym_dev_info_struct] = ACTIONS(1465),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1465),
    [sym_data_feed] = ACTIONS(1465),
    [sym_data_field] = ACTIONS(1465),
    [sym_data_record] = ACTIONS(1465),
    [sym_wc_data_feed] = ACTIONS(1465),
    [sym_wc_data_field] = ACTIONS(1465),
    [sym_wc_data_record] = ACTIONS(1465),
    [sym_last_login_info] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [anon_sym_EQ] = ACTIONS(1527),
    [aux_sym_preproc_include_token1] = ACTIONS(1465),
    [aux_sym_preproc_define_token1] = ACTIONS(1469),
    [aux_sym_preproc_warn_token1] = ACTIONS(1469),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1469),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1469),
  },
  [STATE(117)] = {
    [ts_builtin_sym_end] = ACTIONS(1557),
    [sym_identifier] = ACTIONS(1555),
    [sym_program_name_keyword] = ACTIONS(1555),
    [sym_module_name_keyword] = ACTIONS(1555),
    [sym_define_device_keyword] = ACTIONS(1555),
    [sym_define_combine_keyword] = ACTIONS(1555),
    [sym_define_constant_keyword] = ACTIONS(1555),
    [sym_define_type_keyword] = ACTIONS(1555),
    [sym_define_variable_keyword] = ACTIONS(1555),
    [sym_define_system_variable_keyword] = ACTIONS(1555),
    [sym_define_connect_level_keyword] = ACTIONS(1555),
    [sym_define_latching_keyword] = ACTIONS(1555),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1555),
    [sym_define_toggling_keyword] = ACTIONS(1555),
    [sym_define_call_keyword] = ACTIONS(1555),
    [sym_define_function_keyword] = ACTIONS(1555),
    [sym_define_library_function_keyword] = ACTIONS(1555),
    [sym_define_start_keyword] = ACTIONS(1555),
    [sym_define_event_keyword] = ACTIONS(1555),
    [sym_define_program_keyword] = ACTIONS(1555),
    [sym_define_module_keyword] = ACTIONS(1555),
    [sym_char_keyword] = ACTIONS(1555),
    [sym_widechar_keyword] = ACTIONS(1555),
    [sym_integer_keyword] = ACTIONS(1555),
    [sym_sinteger_keyword] = ACTIONS(1555),
    [sym_long_keyword] = ACTIONS(1555),
    [sym_slong_keyword] = ACTIONS(1555),
    [sym_float_keyword] = ACTIONS(1555),
    [sym_double_keyword] = ACTIONS(1555),
    [sym_variant_keyword] = ACTIONS(1555),
    [sym_variantarray_keyword] = ACTIONS(1555),
    [sym_local_var_keyword] = ACTIONS(1555),
    [sym_stack_var_keyword] = ACTIONS(1555),
    [sym_constant_keyword] = ACTIONS(1555),
    [sym_volatile_keyword] = ACTIONS(1555),
    [sym_non_volatile_keyword] = ACTIONS(1555),
    [sym_persistent_keyword] = ACTIONS(1555),
    [sym_dev_keyword] = ACTIONS(1555),
    [sym_devlev_keyword] = ACTIONS(1555),
    [sym_devchan_keyword] = ACTIONS(1555),
    [sym_if_keyword] = ACTIONS(1555),
    [sym_else_keyword] = ACTIONS(1555),
    [sym_switch_keyword] = ACTIONS(1555),
    [sym_case_keyword] = ACTIONS(1555),
    [sym_default_keyword] = ACTIONS(1555),
    [sym_while_keyword] = ACTIONS(1555),
    [sym_for_keyword] = ACTIONS(1555),
    [sym_break_keyword] = ACTIONS(1555),
    [sym_continue_keyword] = ACTIONS(1555),
    [sym_return_keyword] = ACTIONS(1555),
    [sym_select_keyword] = ACTIONS(1555),
    [sym_active_keyword] = ACTIONS(1555),
    [sym_struct_keyword] = ACTIONS(1555),
    [sym_structure_keyword] = ACTIONS(1555),
    [sym_button_event_keyword] = ACTIONS(1555),
    [sym_channel_event_keyword] = ACTIONS(1555),
    [sym_level_event_keyword] = ACTIONS(1555),
    [sym_data_event_keyword] = ACTIONS(1555),
    [sym_timeline_event_keyword] = ACTIONS(1555),
    [sym_custom_event_keyword] = ACTIONS(1555),
    [sym_push_keyword] = ACTIONS(1555),
    [sym_release_keyword] = ACTIONS(1555),
    [aux_sym_on_keyword_token1] = ACTIONS(1555),
    [aux_sym_off_keyword_token1] = ACTIONS(1555),
    [sym_send_level_keyword] = ACTIONS(1555),
    [sym_send_string_keyword] = ACTIONS(1555),
    [sym_send_command_keyword] = ACTIONS(1555),
    [sym_clear_buffer_keyword] = ACTIONS(1555),
    [sym_create_buffer_keyword] = ACTIONS(1555),
    [sym_create_multi_buffer_keyword] = ACTIONS(1555),
    [sym_call_keyword] = ACTIONS(1555),
    [sym_system_call_keyword] = ACTIONS(1555),
    [sym_devchan_to_keyword] = ACTIONS(1555),
    [sym_devchan_min_to_keyword] = ACTIONS(1555),
    [sym_devchan_total_off_keyword] = ACTIONS(1555),
    [sym_devchan_pulse_keyword] = ACTIONS(1555),
    [sym_wait_keyword] = ACTIONS(1555),
    [sym_cancel_wait_keyword] = ACTIONS(1555),
    [sym_wait_until_keyword] = ACTIONS(1555),
    [sym_cancel_wait_until_keyword] = ACTIONS(1555),
    [sym_cancel_all_wait_keyword] = ACTIONS(1555),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1555),
    [sym_and] = ACTIONS(1555),
    [sym_or] = ACTIONS(1555),
    [sym_not] = ACTIONS(1555),
    [sym_xor] = ACTIONS(1555),
    [sym_band] = ACTIONS(1555),
    [sym_bor] = ACTIONS(1555),
    [sym_bxor] = ACTIONS(1555),
    [sym_bnot] = ACTIONS(1555),
    [sym_lshift] = ACTIONS(1555),
    [sym_rshift] = ACTIONS(1555),
    [sym___file__] = ACTIONS(1555),
    [sym___date__] = ACTIONS(1555),
    [sym___ldate__] = ACTIONS(1555),
    [sym___time__] = ACTIONS(1555),
    [sym___line__] = ACTIONS(1555),
    [sym___name__] = ACTIONS(1555),
    [sym___netlinx__] = ACTIONS(1555),
    [sym_netlinx_axi_version] = ACTIONS(1555),
    [sym_url_flg_tcp] = ACTIONS(1555),
    [sym_url_flg_accinfopresent] = ACTIONS(1555),
    [sym_url_flg_temp] = ACTIONS(1555),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1555),
    [sym_url_flg_stat_mask] = ACTIONS(1555),
    [sym_url_flg_stat_lookup] = ACTIONS(1555),
    [sym_url_flg_stat_connecting] = ACTIONS(1555),
    [sym_url_flg_stat_waiting] = ACTIONS(1555),
    [sym_url_flg_stat_connected] = ACTIONS(1555),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1555),
    [sym_first_virtual_device] = ACTIONS(1555),
    [sym_first_local_port] = ACTIONS(1555),
    [sym_dynamic_virtual_device] = ACTIONS(1555),
    [sym_timeline_once] = ACTIONS(1555),
    [sym_timeline_repeat] = ACTIONS(1555),
    [sym_timeline_absolute] = ACTIONS(1555),
    [sym_timeline_relative] = ACTIONS(1555),
    [sym_do_push_timed_infinite] = ACTIONS(1555),
    [sym_source_type_no_address] = ACTIONS(1555),
    [sym_source_type_neuron_id] = ACTIONS(1555),
    [sym_source_type_ip_address] = ACTIONS(1555),
    [sym_source_type_axlink] = ACTIONS(1555),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1555),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1555),
    [sym_source_type_ip_socket_address] = ACTIONS(1555),
    [sym_source_type_rs232] = ACTIONS(1555),
    [sym_source_type_internal] = ACTIONS(1555),
    [sym_source_type_mac_address] = ACTIONS(1555),
    [sym_source_type_ipv4_port] = ACTIONS(1555),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1555),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1555),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1555),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1555),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1555),
    [sym_true] = ACTIONS(1555),
    [sym_false] = ACTIONS(1555),
    [sym_file_read_only] = ACTIONS(1555),
    [sym_file_rw_new] = ACTIONS(1555),
    [sym_file_rw_append] = ACTIONS(1555),
    [sym_ip_tcp] = ACTIONS(1555),
    [sym_ip_udp] = ACTIONS(1555),
    [sym_ip_udp_2way] = ACTIONS(1555),
    [sym_xml_encode_types] = ACTIONS(1555),
    [sym_xml_encode_char_as_list] = ACTIONS(1555),
    [sym_xml_encode_le] = ACTIONS(1555),
    [sym_xml_decode_types] = ACTIONS(1555),
    [sym_xml_decode_no_preserve] = ACTIONS(1555),
    [sym_duet_dev_type_display_device] = ACTIONS(1555),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1555),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1555),
    [sym_duet_dev_type_io_device] = ACTIONS(1555),
    [sym_duet_dev_type_relay_device] = ACTIONS(1555),
    [sym_duet_dev_type_ups] = ACTIONS(1555),
    [sym_duet_dev_type_amplifier] = ACTIONS(1555),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1555),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1555),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1555),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1555),
    [sym_duet_dev_type_camera] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1555),
    [sym_duet_dev_type_dss] = ACTIONS(1555),
    [sym_duet_dev_type_dvr] = ACTIONS(1555),
    [sym_duet_dev_type_disc_device] = ACTIONS(1555),
    [sym_duet_dev_type_document_camera] = ACTIONS(1555),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1555),
    [sym_duet_dev_type_hvac] = ACTIONS(1555),
    [sym_duet_dev_type_keypad] = ACTIONS(1555),
    [sym_duet_dev_type_light] = ACTIONS(1555),
    [sym_duet_dev_type_monitor] = ACTIONS(1555),
    [sym_duet_dev_type_motor] = ACTIONS(1555),
    [sym_duet_dev_type_multi_window] = ACTIONS(1555),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1555),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1555),
    [sym_duet_dev_type_receiver] = ACTIONS(1555),
    [sym_duet_dev_type_security_system] = ACTIONS(1555),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1555),
    [sym_duet_dev_type_settop_box] = ACTIONS(1555),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1555),
    [sym_duet_dev_type_switcher] = ACTIONS(1555),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1555),
    [sym_duet_dev_type_tv] = ACTIONS(1555),
    [sym_duet_dev_type_utility] = ACTIONS(1555),
    [sym_duet_dev_type_vcr] = ACTIONS(1555),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1555),
    [sym_duet_dev_type_video_processor] = ACTIONS(1555),
    [sym_duet_dev_type_video_projector] = ACTIONS(1555),
    [sym_duet_dev_type_video_wall] = ACTIONS(1555),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1555),
    [sym_duet_dev_type_weather] = ACTIONS(1555),
    [sym_duet_dev_not_polled] = ACTIONS(1555),
    [sym_duet_dev_polled] = ACTIONS(1555),
    [sym_clkmgr_mode_network] = ACTIONS(1555),
    [sym_clkmgr_mode_standalone] = ACTIONS(1555),
    [sym_normal_standby] = ACTIONS(1555),
    [sym_normal_wake] = ACTIONS(1555),
    [sym_ip_multicast_ttl_option] = ACTIONS(1555),
    [sym_ip_ttl_subnet] = ACTIONS(1555),
    [sym_ip_ttl_site] = ACTIONS(1555),
    [sym_ip_ttl_region] = ACTIONS(1555),
    [sym_ip_ttl_continent] = ACTIONS(1555),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1555),
    [sym_ip_nodelay_on] = ACTIONS(1555),
    [sym_ip_nodelay_off] = ACTIONS(1555),
    [sym_amx_error] = ACTIONS(1555),
    [sym_amx_warning] = ACTIONS(1555),
    [sym_amx_info] = ACTIONS(1555),
    [sym_amx_debug] = ACTIONS(1555),
    [sym_netlinx_login_success] = ACTIONS(1555),
    [sym_netlinx_login_fail] = ACTIONS(1555),
    [sym_netlinx_logout] = ACTIONS(1555),
    [sym_null_str] = ACTIONS(1555),
    [sym_smtp_address] = ACTIONS(1555),
    [sym_smtp_port_number] = ACTIONS(1555),
    [sym_smtp_username] = ACTIONS(1555),
    [sym_smtp_password] = ACTIONS(1555),
    [sym_smtp_require_tls] = ACTIONS(1555),
    [sym_smtp_from] = ACTIONS(1555),
    [sym_smtp_tls_true] = ACTIONS(1555),
    [sym_smtp_tls_false] = ACTIONS(1555),
    [sym_listview_on_row_select_event] = ACTIONS(1555),
    [sym_data_structure_unknown] = ACTIONS(1555),
    [sym_data_structure_datafeed] = ACTIONS(1555),
    [sym_data_structure_datarecordset] = ACTIONS(1555),
    [sym_data_structure_datarecord] = ACTIONS(1555),
    [sym_data_structure_datafield] = ACTIONS(1555),
    [sym_data_max_content_field_count] = ACTIONS(1555),
    [sym_data_max_metadata_field_count] = ACTIONS(1555),
    [sym_data_max_id_length] = ACTIONS(1555),
    [sym_data_max_name_length] = ACTIONS(1555),
    [sym_data_max_description_length] = ACTIONS(1555),
    [sym_data_max_type_length] = ACTIONS(1555),
    [sym_data_max_format_length] = ACTIONS(1555),
    [sym_data_max_source_length] = ACTIONS(1555),
    [sym_data_max_value_length] = ACTIONS(1555),
    [sym_data_max_label_length] = ACTIONS(1555),
    [sym_data_max_url_length] = ACTIONS(1555),
    [sym_max_last_login_info_length] = ACTIONS(1555),
    [sym_data_type_unknown] = ACTIONS(1555),
    [sym_data_type_string] = ACTIONS(1555),
    [sym_data_type_datetime] = ACTIONS(1555),
    [sym_data_type_date] = ACTIONS(1555),
    [sym_data_type_time] = ACTIONS(1555),
    [sym_data_type_image] = ACTIONS(1555),
    [sym_data_format_url] = ACTIONS(1555),
    [sym_data_format_phone] = ACTIONS(1555),
    [sym_data_format_email] = ACTIONS(1555),
    [sym_data_format_iso8601] = ACTIONS(1555),
    [sym_auditlog_priv] = ACTIONS(1555),
    [sym_configuration_priv] = ACTIONS(1555),
    [sym_deviceconfig_priv] = ACTIONS(1555),
    [sym_ftp_priv] = ACTIONS(1555),
    [sym_http_priv] = ACTIONS(1555),
    [sym_networkconfig_priv] = ACTIONS(1555),
    [sym_programport_priv] = ACTIONS(1555),
    [sym_remoteui_priv] = ACTIONS(1555),
    [sym_securitycontrol_priv] = ACTIONS(1555),
    [sym_softwaremanagement_priv] = ACTIONS(1555),
    [sym_terminal_priv] = ACTIONS(1555),
    [sym_tpadmin_priv] = ACTIONS(1555),
    [sym_user1_priv] = ACTIONS(1555),
    [sym_user2_priv] = ACTIONS(1555),
    [sym_user3_priv] = ACTIONS(1555),
    [sym_user4_priv] = ACTIONS(1555),
    [sym_usermanagement_priv] = ACTIONS(1555),
    [sym_valid_account] = ACTIONS(1555),
    [sym_err_name_invalid] = ACTIONS(1555),
    [sym_err_password_invalid] = ACTIONS(1555),
    [sym_err_invalid_account] = ACTIONS(1555),
    [sym_err_no_authorization] = ACTIONS(1555),
    [sym_err_invalid_parameter] = ACTIONS(1555),
    [sym_err_user_locked_out] = ACTIONS(1555),
    [sym_err_user_account_expired] = ACTIONS(1555),
    [sym_err_general_error] = ACTIONS(1555),
    [sym_err_authentication_error] = ACTIONS(1555),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1555),
    [sym_tls_validate_certificate] = ACTIONS(1555),
    [sym_tdata] = ACTIONS(1555),
    [sym_tchannel] = ACTIONS(1555),
    [sym_tlevel] = ACTIONS(1555),
    [sym_tbutton] = ACTIONS(1555),
    [sym_ttimeline] = ACTIONS(1555),
    [sym_tcustom] = ACTIONS(1555),
    [sym_url_struct] = ACTIONS(1555),
    [sym_dns_struct] = ACTIONS(1555),
    [sym_ip_address_struct] = ACTIONS(1555),
    [sym_dev_info_struct] = ACTIONS(1555),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1555),
    [sym_data_feed] = ACTIONS(1555),
    [sym_data_field] = ACTIONS(1555),
    [sym_data_record] = ACTIONS(1555),
    [sym_wc_data_feed] = ACTIONS(1555),
    [sym_wc_data_field] = ACTIONS(1555),
    [sym_wc_data_record] = ACTIONS(1555),
    [sym_last_login_info] = ACTIONS(1555),
    [sym____reserved___] = ACTIONS(1555),
    [sym_date] = ACTIONS(1555),
    [sym_day] = ACTIONS(1555),
    [sym_time] = ACTIONS(1555),
    [sym_ldate] = ACTIONS(1555),
    [sym_push_device] = ACTIONS(1555),
    [sym_push_channel] = ACTIONS(1555),
    [sym_push_devchan] = ACTIONS(1555),
    [sym_release_device] = ACTIONS(1555),
    [sym_release_channel] = ACTIONS(1555),
    [sym_release_devchan] = ACTIONS(1555),
    [sym_master_slot] = ACTIONS(1555),
    [sym_get_pulse_time] = ACTIONS(1555),
    [sym_get_timer] = ACTIONS(1555),
    [sym_master_sn] = ACTIONS(1555),
    [sym_system_number] = ACTIONS(1555),
    [sym_button] = ACTIONS(1555),
    [sym_dv_channel] = ACTIONS(1555),
    [sym_data] = ACTIONS(1555),
    [sym_level] = ACTIONS(1555),
    [sym_timeline] = ACTIONS(1555),
    [sym_channel] = ACTIONS(1555),
    [sym_custom] = ACTIONS(1555),
    [sym_atoi] = ACTIONS(1555),
    [sym_atol] = ACTIONS(1555),
    [sym_atof] = ACTIONS(1555),
    [sym_itoa] = ACTIONS(1555),
    [sym_itohex] = ACTIONS(1555),
    [sym_hextoi] = ACTIONS(1555),
    [sym_ftoa] = ACTIONS(1555),
    [sym_find_string] = ACTIONS(1555),
    [sym_left_string] = ACTIONS(1555),
    [sym_right_string] = ACTIONS(1555),
    [sym_mid_string] = ACTIONS(1555),
    [sym_remove_string] = ACTIONS(1555),
    [sym_get_buffer_string] = ACTIONS(1555),
    [sym_get_buffer_char] = ACTIONS(1555),
    [sym_get_multi_buffer_string] = ACTIONS(1555),
    [sym_length_string] = ACTIONS(1555),
    [sym_max_length_string] = ACTIONS(1555),
    [sym_set_length_string] = ACTIONS(1555),
    [sym_compare_string] = ACTIONS(1555),
    [sym_upper_string] = ACTIONS(1555),
    [sym_lower_string] = ACTIONS(1555),
    [sym_get_last] = ACTIONS(1555),
    [sym_date_to_day] = ACTIONS(1555),
    [sym_date_to_month] = ACTIONS(1555),
    [sym_date_to_year] = ACTIONS(1555),
    [sym_time_to_hour] = ACTIONS(1555),
    [sym_time_to_minute] = ACTIONS(1555),
    [sym_time_to_second] = ACTIONS(1555),
    [sym_day_of_week] = ACTIONS(1555),
    [sym_set_pulse_time] = ACTIONS(1555),
    [sym_set_timer] = ACTIONS(1555),
    [sym_file_open] = ACTIONS(1555),
    [sym_file_close] = ACTIONS(1555),
    [sym_file_read] = ACTIONS(1555),
    [sym_file_read_line] = ACTIONS(1555),
    [sym_file_write] = ACTIONS(1555),
    [sym_file_write_line] = ACTIONS(1555),
    [sym_file_copy] = ACTIONS(1555),
    [sym_file_delete] = ACTIONS(1555),
    [sym_file_rename] = ACTIONS(1555),
    [sym_file_dir] = ACTIONS(1555),
    [sym_file_getdir] = ACTIONS(1555),
    [sym_file_setdir] = ACTIONS(1555),
    [sym_file_create_dir] = ACTIONS(1555),
    [sym_file_remove_dir] = ACTIONS(1555),
    [sym_file_seek] = ACTIONS(1555),
    [sym_length_array] = ACTIONS(1555),
    [sym_max_length_array] = ACTIONS(1555),
    [sym_set_length_array] = ACTIONS(1555),
    [sym_ip_client_open] = ACTIONS(1555),
    [sym_ip_client_close] = ACTIONS(1555),
    [sym_ip_server_open] = ACTIONS(1555),
    [sym_ip_server_close] = ACTIONS(1555),
    [sym_ip_bound_client_open] = ACTIONS(1555),
    [sym_ip_mc_server_open] = ACTIONS(1555),
    [sym_ip_set_option] = ACTIONS(1555),
    [sym_ssh_client_open] = ACTIONS(1555),
    [sym_ssh_client_close] = ACTIONS(1555),
    [sym_tls_client_open] = ACTIONS(1555),
    [sym_tls_client_close] = ACTIONS(1555),
    [sym_do_push] = ACTIONS(1555),
    [sym_do_push_timed] = ACTIONS(1555),
    [sym_do_release] = ACTIONS(1555),
    [sym_device_id] = ACTIONS(1555),
    [sym_device_info] = ACTIONS(1555),
    [sym_device_id_string] = ACTIONS(1555),
    [sym_device_standby] = ACTIONS(1555),
    [sym_device_wake] = ACTIONS(1555),
    [sym_timeline_create] = ACTIONS(1555),
    [sym_timeline_kill] = ACTIONS(1555),
    [sym_timeline_reload] = ACTIONS(1555),
    [sym_timeline_pause] = ACTIONS(1555),
    [sym_timeline_restart] = ACTIONS(1555),
    [sym_timeline_set] = ACTIONS(1555),
    [sym_timeline_get] = ACTIONS(1555),
    [sym_timeline_active] = ACTIONS(1555),
    [sym_timeline_dynamic_id] = ACTIONS(1555),
    [sym_variable_to_string] = ACTIONS(1555),
    [sym_string_to_variable] = ACTIONS(1555),
    [sym_length_variable_to_string] = ACTIONS(1555),
    [sym_type_cast] = ACTIONS(1555),
    [sym_variable_to_xml] = ACTIONS(1555),
    [sym_xml_to_variable] = ACTIONS(1555),
    [sym_length_variable_to_xml] = ACTIONS(1555),
    [sym_redirect_string] = ACTIONS(1555),
    [sym_random_number] = ACTIONS(1555),
    [sym_get_unique_id] = ACTIONS(1555),
    [sym_get_system_number] = ACTIONS(1555),
    [sym_set_system_number] = ACTIONS(1555),
    [sym_get_serial_number] = ACTIONS(1555),
    [sym_get_master_build] = ACTIONS(1555),
    [sym_set_validation_code] = ACTIONS(1555),
    [sym_reboot] = ACTIONS(1555),
    [sym_load_duet_module] = ACTIONS(1555),
    [sym_unload_duet_module] = ACTIONS(1555),
    [sym_duet_mem_size_set] = ACTIONS(1555),
    [sym_duet_mem_size_get] = ACTIONS(1555),
    [sym_dynamic_application_device] = ACTIONS(1555),
    [sym_dynamic_polled_port] = ACTIONS(1555),
    [sym_static_port_binding] = ACTIONS(1555),
    [sym_static_ip_binding] = ACTIONS(1555),
    [sym_get_dns_list] = ACTIONS(1555),
    [sym_set_dns_list] = ACTIONS(1555),
    [sym_get_ip_address] = ACTIONS(1555),
    [sym_set_ip_address] = ACTIONS(1555),
    [sym_get_url_list] = ACTIONS(1555),
    [sym_add_url_entry] = ACTIONS(1555),
    [sym_delete_url_entry] = ACTIONS(1555),
    [sym_abs_value] = ACTIONS(1555),
    [sym_max_value] = ACTIONS(1555),
    [sym_min_value] = ACTIONS(1555),
    [sym_exp_value] = ACTIONS(1555),
    [sym_log_value] = ACTIONS(1555),
    [sym_log10_value] = ACTIONS(1555),
    [sym_power_value] = ACTIONS(1555),
    [sym_sqrt_value] = ACTIONS(1555),
    [sym_raw_be] = ACTIONS(1555),
    [sym_raw_le] = ACTIONS(1555),
    [sym_internal_queue_size_set] = ACTIONS(1555),
    [sym_internal_queue_size_get] = ACTIONS(1555),
    [sym_internal_threshold_set] = ACTIONS(1555),
    [sym_internal_threshold_get] = ACTIONS(1555),
    [sym_rebuilt_event] = ACTIONS(1555),
    [sym_do_custom_event] = ACTIONS(1555),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1555),
    [sym_clkmgr_set_clk_source] = ACTIONS(1555),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1555),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1555),
    [sym_clkmgr_get_timezone] = ACTIONS(1555),
    [sym_clkmgr_set_timezone] = ACTIONS(1555),
    [sym_clkmgr_get_resync_period] = ACTIONS(1555),
    [sym_clkmgr_set_resync_period] = ACTIONS(1555),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1555),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1555),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1555),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1555),
    [sym_clkmgr_get_timeservers] = ACTIONS(1555),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1555),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1555),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1555),
    [sym_set_log_level] = ACTIONS(1555),
    [sym_get_log_level] = ACTIONS(1555),
    [sym_amx_log] = ACTIONS(1555),
    [sym_smtp_server_config_set] = ACTIONS(1555),
    [sym_smtp_server_config_get] = ACTIONS(1555),
    [sym_smtp_send] = ACTIONS(1555),
    [sym_validate_netlinx_account] = ACTIONS(1555),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1555),
    [sym_authenticate_certificate] = ACTIONS(1555),
    [sym_audit_netlinx_session_event] = ACTIONS(1555),
    [sym_audit_netlinx_generic_event] = ACTIONS(1555),
    [sym_get_available_flash_disk_space] = ACTIONS(1555),
    [sym_get_max_flash_disk_space] = ACTIONS(1555),
    [sym_data_create_feed] = ACTIONS(1555),
    [sym_data_delete_feed] = ACTIONS(1555),
    [sym_data_publish_feed] = ACTIONS(1555),
    [sym_data_get_published_feed] = ACTIONS(1555),
    [sym_data_add_record] = ACTIONS(1555),
    [sym_data_get_event_record] = ACTIONS(1555),
    [sym__wc_data_create_feed] = ACTIONS(1555),
    [sym__wc_data_add_record] = ACTIONS(1555),
    [sym__wc_data_get_event_record] = ACTIONS(1555),
    [sym_format] = ACTIONS(1555),
    [sym_astro_clock] = ACTIONS(1555),
    [sym_set_outdoor_temperature] = ACTIONS(1555),
    [sym_set_virtual_level_count] = ACTIONS(1555),
    [sym_set_virtual_channel_count] = ACTIONS(1555),
    [sym_set_virtual_port_count] = ACTIONS(1555),
    [sym__wc_encode_int] = ACTIONS(1555),
    [sym__wc_decode_int] = ACTIONS(1555),
    [sym__wc_int] = ACTIONS(1555),
    [sym__wc_to_ch_int] = ACTIONS(1555),
    [sym__ch_to_wc_int] = ACTIONS(1555),
    [sym__wc_find_string_int] = ACTIONS(1555),
    [sym__wc_left_string_int] = ACTIONS(1555),
    [sym__wc_lower_string_int] = ACTIONS(1555),
    [sym__wc_mid_string_int] = ACTIONS(1555),
    [sym__wc_remove_string_int] = ACTIONS(1555),
    [sym__wc_right_string_int] = ACTIONS(1555),
    [sym__wc_upper_string_int] = ACTIONS(1555),
    [sym__wc_compare_string_int] = ACTIONS(1555),
    [sym__wc_get_buffer_char_int] = ACTIONS(1555),
    [sym__wc_get_buffer_string_int] = ACTIONS(1555),
    [sym__wc_concat_string_int] = ACTIONS(1555),
    [sym__wc_file_open_int] = ACTIONS(1555),
    [sym__wc_file_close_int] = ACTIONS(1555),
    [sym__wc_file_read_int] = ACTIONS(1555),
    [sym__wc_file_read_line_int] = ACTIONS(1555),
    [sym__wc_file_write_int] = ACTIONS(1555),
    [sym__wc_file_write_line_int] = ACTIONS(1555),
    [anon_sym_EQ] = ACTIONS(1555),
    [aux_sym_preproc_include_token1] = ACTIONS(1555),
    [aux_sym_preproc_define_token1] = ACTIONS(1557),
    [aux_sym_preproc_warn_token1] = ACTIONS(1557),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1557),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1557),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1557),
    [anon_sym_LBRACK] = ACTIONS(1557),
    [anon_sym_LBRACE] = ACTIONS(1557),
    [anon_sym_COLON] = ACTIONS(1557),
    [anon_sym_LPAREN] = ACTIONS(1555),
    [anon_sym_SEMI] = ACTIONS(1557),
    [anon_sym_BANG] = ACTIONS(1555),
    [anon_sym_TILDE] = ACTIONS(1557),
    [anon_sym_DASH] = ACTIONS(1555),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_STAR] = ACTIONS(1557),
    [anon_sym_SLASH] = ACTIONS(1555),
    [anon_sym_PERCENT] = ACTIONS(1557),
    [anon_sym_PIPE_PIPE] = ACTIONS(1557),
    [anon_sym_AMP_AMP] = ACTIONS(1557),
    [anon_sym_PIPE] = ACTIONS(1555),
    [anon_sym_CARET] = ACTIONS(1557),
    [anon_sym_AMP] = ACTIONS(1555),
    [anon_sym_EQ_EQ] = ACTIONS(1557),
    [anon_sym_BANG_EQ] = ACTIONS(1557),
    [anon_sym_LT_GT] = ACTIONS(1557),
    [anon_sym_GT] = ACTIONS(1555),
    [anon_sym_GT_EQ] = ACTIONS(1557),
    [anon_sym_LT_EQ] = ACTIONS(1557),
    [anon_sym_LT] = ACTIONS(1555),
    [anon_sym_LT_LT] = ACTIONS(1557),
    [anon_sym_GT_GT] = ACTIONS(1557),
    [anon_sym_DASH_DASH] = ACTIONS(1557),
    [anon_sym_PLUS_PLUS] = ACTIONS(1557),
    [anon_sym_DOT] = ACTIONS(1555),
    [anon_sym_DQUOTE] = ACTIONS(1557),
    [anon_sym_SQUOTE] = ACTIONS(1557),
    [sym_number_literal] = ACTIONS(1557),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1557),
  },
  [STATE(118)] = {
    [sym_identifier] = ACTIONS(1561),
    [sym_program_name_keyword] = ACTIONS(1561),
    [sym_module_name_keyword] = ACTIONS(1561),
    [sym_define_device_keyword] = ACTIONS(1561),
    [sym_define_combine_keyword] = ACTIONS(1561),
    [sym_define_constant_keyword] = ACTIONS(1561),
    [sym_define_type_keyword] = ACTIONS(1561),
    [sym_define_variable_keyword] = ACTIONS(1561),
    [sym_define_system_variable_keyword] = ACTIONS(1561),
    [sym_define_connect_level_keyword] = ACTIONS(1561),
    [sym_define_latching_keyword] = ACTIONS(1561),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1561),
    [sym_define_toggling_keyword] = ACTIONS(1561),
    [sym_define_call_keyword] = ACTIONS(1561),
    [sym_define_function_keyword] = ACTIONS(1561),
    [sym_define_library_function_keyword] = ACTIONS(1561),
    [sym_define_start_keyword] = ACTIONS(1561),
    [sym_define_event_keyword] = ACTIONS(1561),
    [sym_define_program_keyword] = ACTIONS(1561),
    [sym_define_module_keyword] = ACTIONS(1561),
    [sym_char_keyword] = ACTIONS(1561),
    [sym_widechar_keyword] = ACTIONS(1561),
    [sym_integer_keyword] = ACTIONS(1561),
    [sym_sinteger_keyword] = ACTIONS(1561),
    [sym_long_keyword] = ACTIONS(1561),
    [sym_slong_keyword] = ACTIONS(1561),
    [sym_float_keyword] = ACTIONS(1561),
    [sym_double_keyword] = ACTIONS(1561),
    [sym_variant_keyword] = ACTIONS(1561),
    [sym_variantarray_keyword] = ACTIONS(1561),
    [sym_local_var_keyword] = ACTIONS(1561),
    [sym_stack_var_keyword] = ACTIONS(1561),
    [sym_constant_keyword] = ACTIONS(1561),
    [sym_volatile_keyword] = ACTIONS(1561),
    [sym_non_volatile_keyword] = ACTIONS(1561),
    [sym_persistent_keyword] = ACTIONS(1561),
    [sym_dev_keyword] = ACTIONS(1561),
    [sym_devlev_keyword] = ACTIONS(1561),
    [sym_devchan_keyword] = ACTIONS(1561),
    [sym_if_keyword] = ACTIONS(1561),
    [sym_else_keyword] = ACTIONS(1561),
    [sym_switch_keyword] = ACTIONS(1561),
    [sym_case_keyword] = ACTIONS(1561),
    [sym_default_keyword] = ACTIONS(1561),
    [sym_while_keyword] = ACTIONS(1561),
    [sym_for_keyword] = ACTIONS(1561),
    [sym_break_keyword] = ACTIONS(1561),
    [sym_continue_keyword] = ACTIONS(1561),
    [sym_return_keyword] = ACTIONS(1561),
    [sym_select_keyword] = ACTIONS(1561),
    [sym_active_keyword] = ACTIONS(1561),
    [sym_struct_keyword] = ACTIONS(1561),
    [sym_structure_keyword] = ACTIONS(1561),
    [sym_button_event_keyword] = ACTIONS(1561),
    [sym_channel_event_keyword] = ACTIONS(1561),
    [sym_level_event_keyword] = ACTIONS(1561),
    [sym_data_event_keyword] = ACTIONS(1561),
    [sym_timeline_event_keyword] = ACTIONS(1561),
    [sym_custom_event_keyword] = ACTIONS(1561),
    [sym_push_keyword] = ACTIONS(1561),
    [sym_release_keyword] = ACTIONS(1561),
    [aux_sym_on_keyword_token1] = ACTIONS(1561),
    [aux_sym_off_keyword_token1] = ACTIONS(1561),
    [sym_send_level_keyword] = ACTIONS(1561),
    [sym_send_string_keyword] = ACTIONS(1561),
    [sym_send_command_keyword] = ACTIONS(1561),
    [sym_clear_buffer_keyword] = ACTIONS(1561),
    [sym_create_buffer_keyword] = ACTIONS(1561),
    [sym_create_multi_buffer_keyword] = ACTIONS(1561),
    [sym_call_keyword] = ACTIONS(1561),
    [sym_system_call_keyword] = ACTIONS(1561),
    [sym_devchan_to_keyword] = ACTIONS(1561),
    [sym_devchan_min_to_keyword] = ACTIONS(1561),
    [sym_devchan_total_off_keyword] = ACTIONS(1561),
    [sym_devchan_pulse_keyword] = ACTIONS(1561),
    [sym_wait_keyword] = ACTIONS(1561),
    [sym_cancel_wait_keyword] = ACTIONS(1561),
    [sym_wait_until_keyword] = ACTIONS(1561),
    [sym_cancel_wait_until_keyword] = ACTIONS(1561),
    [sym_cancel_all_wait_keyword] = ACTIONS(1561),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1561),
    [sym_and] = ACTIONS(1561),
    [sym_or] = ACTIONS(1561),
    [sym_not] = ACTIONS(1561),
    [sym_xor] = ACTIONS(1561),
    [sym_band] = ACTIONS(1561),
    [sym_bor] = ACTIONS(1561),
    [sym_bxor] = ACTIONS(1561),
    [sym_bnot] = ACTIONS(1561),
    [sym_lshift] = ACTIONS(1561),
    [sym_rshift] = ACTIONS(1561),
    [sym___file__] = ACTIONS(1561),
    [sym___date__] = ACTIONS(1561),
    [sym___ldate__] = ACTIONS(1561),
    [sym___time__] = ACTIONS(1561),
    [sym___line__] = ACTIONS(1561),
    [sym___name__] = ACTIONS(1561),
    [sym___netlinx__] = ACTIONS(1561),
    [sym_netlinx_axi_version] = ACTIONS(1561),
    [sym_url_flg_tcp] = ACTIONS(1561),
    [sym_url_flg_accinfopresent] = ACTIONS(1561),
    [sym_url_flg_temp] = ACTIONS(1561),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1561),
    [sym_url_flg_stat_mask] = ACTIONS(1561),
    [sym_url_flg_stat_lookup] = ACTIONS(1561),
    [sym_url_flg_stat_connecting] = ACTIONS(1561),
    [sym_url_flg_stat_waiting] = ACTIONS(1561),
    [sym_url_flg_stat_connected] = ACTIONS(1561),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1561),
    [sym_first_virtual_device] = ACTIONS(1561),
    [sym_first_local_port] = ACTIONS(1561),
    [sym_dynamic_virtual_device] = ACTIONS(1561),
    [sym_timeline_once] = ACTIONS(1561),
    [sym_timeline_repeat] = ACTIONS(1561),
    [sym_timeline_absolute] = ACTIONS(1561),
    [sym_timeline_relative] = ACTIONS(1561),
    [sym_do_push_timed_infinite] = ACTIONS(1561),
    [sym_source_type_no_address] = ACTIONS(1561),
    [sym_source_type_neuron_id] = ACTIONS(1561),
    [sym_source_type_ip_address] = ACTIONS(1561),
    [sym_source_type_axlink] = ACTIONS(1561),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1561),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1561),
    [sym_source_type_ip_socket_address] = ACTIONS(1561),
    [sym_source_type_rs232] = ACTIONS(1561),
    [sym_source_type_internal] = ACTIONS(1561),
    [sym_source_type_mac_address] = ACTIONS(1561),
    [sym_source_type_ipv4_port] = ACTIONS(1561),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1561),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1561),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1561),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1561),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1561),
    [sym_true] = ACTIONS(1561),
    [sym_false] = ACTIONS(1561),
    [sym_file_read_only] = ACTIONS(1561),
    [sym_file_rw_new] = ACTIONS(1561),
    [sym_file_rw_append] = ACTIONS(1561),
    [sym_ip_tcp] = ACTIONS(1561),
    [sym_ip_udp] = ACTIONS(1561),
    [sym_ip_udp_2way] = ACTIONS(1561),
    [sym_xml_encode_types] = ACTIONS(1561),
    [sym_xml_encode_char_as_list] = ACTIONS(1561),
    [sym_xml_encode_le] = ACTIONS(1561),
    [sym_xml_decode_types] = ACTIONS(1561),
    [sym_xml_decode_no_preserve] = ACTIONS(1561),
    [sym_duet_dev_type_display_device] = ACTIONS(1561),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1561),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1561),
    [sym_duet_dev_type_io_device] = ACTIONS(1561),
    [sym_duet_dev_type_relay_device] = ACTIONS(1561),
    [sym_duet_dev_type_ups] = ACTIONS(1561),
    [sym_duet_dev_type_amplifier] = ACTIONS(1561),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1561),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1561),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1561),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1561),
    [sym_duet_dev_type_camera] = ACTIONS(1561),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1561),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1561),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1561),
    [sym_duet_dev_type_dss] = ACTIONS(1561),
    [sym_duet_dev_type_dvr] = ACTIONS(1561),
    [sym_duet_dev_type_disc_device] = ACTIONS(1561),
    [sym_duet_dev_type_document_camera] = ACTIONS(1561),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1561),
    [sym_duet_dev_type_hvac] = ACTIONS(1561),
    [sym_duet_dev_type_keypad] = ACTIONS(1561),
    [sym_duet_dev_type_light] = ACTIONS(1561),
    [sym_duet_dev_type_monitor] = ACTIONS(1561),
    [sym_duet_dev_type_motor] = ACTIONS(1561),
    [sym_duet_dev_type_multi_window] = ACTIONS(1561),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1561),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1561),
    [sym_duet_dev_type_receiver] = ACTIONS(1561),
    [sym_duet_dev_type_security_system] = ACTIONS(1561),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1561),
    [sym_duet_dev_type_settop_box] = ACTIONS(1561),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1561),
    [sym_duet_dev_type_switcher] = ACTIONS(1561),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1561),
    [sym_duet_dev_type_tv] = ACTIONS(1561),
    [sym_duet_dev_type_utility] = ACTIONS(1561),
    [sym_duet_dev_type_vcr] = ACTIONS(1561),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1561),
    [sym_duet_dev_type_video_processor] = ACTIONS(1561),
    [sym_duet_dev_type_video_projector] = ACTIONS(1561),
    [sym_duet_dev_type_video_wall] = ACTIONS(1561),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1561),
    [sym_duet_dev_type_weather] = ACTIONS(1561),
    [sym_duet_dev_not_polled] = ACTIONS(1561),
    [sym_duet_dev_polled] = ACTIONS(1561),
    [sym_clkmgr_mode_network] = ACTIONS(1561),
    [sym_clkmgr_mode_standalone] = ACTIONS(1561),
    [sym_normal_standby] = ACTIONS(1561),
    [sym_normal_wake] = ACTIONS(1561),
    [sym_ip_multicast_ttl_option] = ACTIONS(1561),
    [sym_ip_ttl_subnet] = ACTIONS(1561),
    [sym_ip_ttl_site] = ACTIONS(1561),
    [sym_ip_ttl_region] = ACTIONS(1561),
    [sym_ip_ttl_continent] = ACTIONS(1561),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1561),
    [sym_ip_nodelay_on] = ACTIONS(1561),
    [sym_ip_nodelay_off] = ACTIONS(1561),
    [sym_amx_error] = ACTIONS(1561),
    [sym_amx_warning] = ACTIONS(1561),
    [sym_amx_info] = ACTIONS(1561),
    [sym_amx_debug] = ACTIONS(1561),
    [sym_netlinx_login_success] = ACTIONS(1561),
    [sym_netlinx_login_fail] = ACTIONS(1561),
    [sym_netlinx_logout] = ACTIONS(1561),
    [sym_null_str] = ACTIONS(1561),
    [sym_smtp_address] = ACTIONS(1561),
    [sym_smtp_port_number] = ACTIONS(1561),
    [sym_smtp_username] = ACTIONS(1561),
    [sym_smtp_password] = ACTIONS(1561),
    [sym_smtp_require_tls] = ACTIONS(1561),
    [sym_smtp_from] = ACTIONS(1561),
    [sym_smtp_tls_true] = ACTIONS(1561),
    [sym_smtp_tls_false] = ACTIONS(1561),
    [sym_listview_on_row_select_event] = ACTIONS(1561),
    [sym_data_structure_unknown] = ACTIONS(1561),
    [sym_data_structure_datafeed] = ACTIONS(1561),
    [sym_data_structure_datarecordset] = ACTIONS(1561),
    [sym_data_structure_datarecord] = ACTIONS(1561),
    [sym_data_structure_datafield] = ACTIONS(1561),
    [sym_data_max_content_field_count] = ACTIONS(1561),
    [sym_data_max_metadata_field_count] = ACTIONS(1561),
    [sym_data_max_id_length] = ACTIONS(1561),
    [sym_data_max_name_length] = ACTIONS(1561),
    [sym_data_max_description_length] = ACTIONS(1561),
    [sym_data_max_type_length] = ACTIONS(1561),
    [sym_data_max_format_length] = ACTIONS(1561),
    [sym_data_max_source_length] = ACTIONS(1561),
    [sym_data_max_value_length] = ACTIONS(1561),
    [sym_data_max_label_length] = ACTIONS(1561),
    [sym_data_max_url_length] = ACTIONS(1561),
    [sym_max_last_login_info_length] = ACTIONS(1561),
    [sym_data_type_unknown] = ACTIONS(1561),
    [sym_data_type_string] = ACTIONS(1561),
    [sym_data_type_datetime] = ACTIONS(1561),
    [sym_data_type_date] = ACTIONS(1561),
    [sym_data_type_time] = ACTIONS(1561),
    [sym_data_type_image] = ACTIONS(1561),
    [sym_data_format_url] = ACTIONS(1561),
    [sym_data_format_phone] = ACTIONS(1561),
    [sym_data_format_email] = ACTIONS(1561),
    [sym_data_format_iso8601] = ACTIONS(1561),
    [sym_auditlog_priv] = ACTIONS(1561),
    [sym_configuration_priv] = ACTIONS(1561),
    [sym_deviceconfig_priv] = ACTIONS(1561),
    [sym_ftp_priv] = ACTIONS(1561),
    [sym_http_priv] = ACTIONS(1561),
    [sym_networkconfig_priv] = ACTIONS(1561),
    [sym_programport_priv] = ACTIONS(1561),
    [sym_remoteui_priv] = ACTIONS(1561),
    [sym_securitycontrol_priv] = ACTIONS(1561),
    [sym_softwaremanagement_priv] = ACTIONS(1561),
    [sym_terminal_priv] = ACTIONS(1561),
    [sym_tpadmin_priv] = ACTIONS(1561),
    [sym_user1_priv] = ACTIONS(1561),
    [sym_user2_priv] = ACTIONS(1561),
    [sym_user3_priv] = ACTIONS(1561),
    [sym_user4_priv] = ACTIONS(1561),
    [sym_usermanagement_priv] = ACTIONS(1561),
    [sym_valid_account] = ACTIONS(1561),
    [sym_err_name_invalid] = ACTIONS(1561),
    [sym_err_password_invalid] = ACTIONS(1561),
    [sym_err_invalid_account] = ACTIONS(1561),
    [sym_err_no_authorization] = ACTIONS(1561),
    [sym_err_invalid_parameter] = ACTIONS(1561),
    [sym_err_user_locked_out] = ACTIONS(1561),
    [sym_err_user_account_expired] = ACTIONS(1561),
    [sym_err_general_error] = ACTIONS(1561),
    [sym_err_authentication_error] = ACTIONS(1561),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1561),
    [sym_tls_validate_certificate] = ACTIONS(1561),
    [sym_tdata] = ACTIONS(1561),
    [sym_tchannel] = ACTIONS(1561),
    [sym_tlevel] = ACTIONS(1561),
    [sym_tbutton] = ACTIONS(1561),
    [sym_ttimeline] = ACTIONS(1561),
    [sym_tcustom] = ACTIONS(1561),
    [sym_url_struct] = ACTIONS(1561),
    [sym_dns_struct] = ACTIONS(1561),
    [sym_ip_address_struct] = ACTIONS(1561),
    [sym_dev_info_struct] = ACTIONS(1561),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1561),
    [sym_data_feed] = ACTIONS(1561),
    [sym_data_field] = ACTIONS(1561),
    [sym_data_record] = ACTIONS(1561),
    [sym_wc_data_feed] = ACTIONS(1561),
    [sym_wc_data_field] = ACTIONS(1561),
    [sym_wc_data_record] = ACTIONS(1561),
    [sym_last_login_info] = ACTIONS(1561),
    [sym____reserved___] = ACTIONS(1561),
    [sym_date] = ACTIONS(1561),
    [sym_day] = ACTIONS(1561),
    [sym_time] = ACTIONS(1561),
    [sym_ldate] = ACTIONS(1561),
    [sym_push_device] = ACTIONS(1561),
    [sym_push_channel] = ACTIONS(1561),
    [sym_push_devchan] = ACTIONS(1561),
    [sym_release_device] = ACTIONS(1561),
    [sym_release_channel] = ACTIONS(1561),
    [sym_release_devchan] = ACTIONS(1561),
    [sym_master_slot] = ACTIONS(1561),
    [sym_get_pulse_time] = ACTIONS(1561),
    [sym_get_timer] = ACTIONS(1561),
    [sym_master_sn] = ACTIONS(1561),
    [sym_system_number] = ACTIONS(1561),
    [sym_button] = ACTIONS(1561),
    [sym_dv_channel] = ACTIONS(1561),
    [sym_data] = ACTIONS(1561),
    [sym_level] = ACTIONS(1561),
    [sym_timeline] = ACTIONS(1561),
    [sym_channel] = ACTIONS(1561),
    [sym_custom] = ACTIONS(1561),
    [sym_atoi] = ACTIONS(1561),
    [sym_atol] = ACTIONS(1561),
    [sym_atof] = ACTIONS(1561),
    [sym_itoa] = ACTIONS(1561),
    [sym_itohex] = ACTIONS(1561),
    [sym_hextoi] = ACTIONS(1561),
    [sym_ftoa] = ACTIONS(1561),
    [sym_find_string] = ACTIONS(1561),
    [sym_left_string] = ACTIONS(1561),
    [sym_right_string] = ACTIONS(1561),
    [sym_mid_string] = ACTIONS(1561),
    [sym_remove_string] = ACTIONS(1561),
    [sym_get_buffer_string] = ACTIONS(1561),
    [sym_get_buffer_char] = ACTIONS(1561),
    [sym_get_multi_buffer_string] = ACTIONS(1561),
    [sym_length_string] = ACTIONS(1561),
    [sym_max_length_string] = ACTIONS(1561),
    [sym_set_length_string] = ACTIONS(1561),
    [sym_compare_string] = ACTIONS(1561),
    [sym_upper_string] = ACTIONS(1561),
    [sym_lower_string] = ACTIONS(1561),
    [sym_get_last] = ACTIONS(1561),
    [sym_date_to_day] = ACTIONS(1561),
    [sym_date_to_month] = ACTIONS(1561),
    [sym_date_to_year] = ACTIONS(1561),
    [sym_time_to_hour] = ACTIONS(1561),
    [sym_time_to_minute] = ACTIONS(1561),
    [sym_time_to_second] = ACTIONS(1561),
    [sym_day_of_week] = ACTIONS(1561),
    [sym_set_pulse_time] = ACTIONS(1561),
    [sym_set_timer] = ACTIONS(1561),
    [sym_file_open] = ACTIONS(1561),
    [sym_file_close] = ACTIONS(1561),
    [sym_file_read] = ACTIONS(1561),
    [sym_file_read_line] = ACTIONS(1561),
    [sym_file_write] = ACTIONS(1561),
    [sym_file_write_line] = ACTIONS(1561),
    [sym_file_copy] = ACTIONS(1561),
    [sym_file_delete] = ACTIONS(1561),
    [sym_file_rename] = ACTIONS(1561),
    [sym_file_dir] = ACTIONS(1561),
    [sym_file_getdir] = ACTIONS(1561),
    [sym_file_setdir] = ACTIONS(1561),
    [sym_file_create_dir] = ACTIONS(1561),
    [sym_file_remove_dir] = ACTIONS(1561),
    [sym_file_seek] = ACTIONS(1561),
    [sym_length_array] = ACTIONS(1561),
    [sym_max_length_array] = ACTIONS(1561),
    [sym_set_length_array] = ACTIONS(1561),
    [sym_ip_client_open] = ACTIONS(1561),
    [sym_ip_client_close] = ACTIONS(1561),
    [sym_ip_server_open] = ACTIONS(1561),
    [sym_ip_server_close] = ACTIONS(1561),
    [sym_ip_bound_client_open] = ACTIONS(1561),
    [sym_ip_mc_server_open] = ACTIONS(1561),
    [sym_ip_set_option] = ACTIONS(1561),
    [sym_ssh_client_open] = ACTIONS(1561),
    [sym_ssh_client_close] = ACTIONS(1561),
    [sym_tls_client_open] = ACTIONS(1561),
    [sym_tls_client_close] = ACTIONS(1561),
    [sym_do_push] = ACTIONS(1561),
    [sym_do_push_timed] = ACTIONS(1561),
    [sym_do_release] = ACTIONS(1561),
    [sym_device_id] = ACTIONS(1561),
    [sym_device_info] = ACTIONS(1561),
    [sym_device_id_string] = ACTIONS(1561),
    [sym_device_standby] = ACTIONS(1561),
    [sym_device_wake] = ACTIONS(1561),
    [sym_timeline_create] = ACTIONS(1561),
    [sym_timeline_kill] = ACTIONS(1561),
    [sym_timeline_reload] = ACTIONS(1561),
    [sym_timeline_pause] = ACTIONS(1561),
    [sym_timeline_restart] = ACTIONS(1561),
    [sym_timeline_set] = ACTIONS(1561),
    [sym_timeline_get] = ACTIONS(1561),
    [sym_timeline_active] = ACTIONS(1561),
    [sym_timeline_dynamic_id] = ACTIONS(1561),
    [sym_variable_to_string] = ACTIONS(1561),
    [sym_string_to_variable] = ACTIONS(1561),
    [sym_length_variable_to_string] = ACTIONS(1561),
    [sym_type_cast] = ACTIONS(1561),
    [sym_variable_to_xml] = ACTIONS(1561),
    [sym_xml_to_variable] = ACTIONS(1561),
    [sym_length_variable_to_xml] = ACTIONS(1561),
    [sym_redirect_string] = ACTIONS(1561),
    [sym_random_number] = ACTIONS(1561),
    [sym_get_unique_id] = ACTIONS(1561),
    [sym_get_system_number] = ACTIONS(1561),
    [sym_set_system_number] = ACTIONS(1561),
    [sym_get_serial_number] = ACTIONS(1561),
    [sym_get_master_build] = ACTIONS(1561),
    [sym_set_validation_code] = ACTIONS(1561),
    [sym_reboot] = ACTIONS(1561),
    [sym_load_duet_module] = ACTIONS(1561),
    [sym_unload_duet_module] = ACTIONS(1561),
    [sym_duet_mem_size_set] = ACTIONS(1561),
    [sym_duet_mem_size_get] = ACTIONS(1561),
    [sym_dynamic_application_device] = ACTIONS(1561),
    [sym_dynamic_polled_port] = ACTIONS(1561),
    [sym_static_port_binding] = ACTIONS(1561),
    [sym_static_ip_binding] = ACTIONS(1561),
    [sym_get_dns_list] = ACTIONS(1561),
    [sym_set_dns_list] = ACTIONS(1561),
    [sym_get_ip_address] = ACTIONS(1561),
    [sym_set_ip_address] = ACTIONS(1561),
    [sym_get_url_list] = ACTIONS(1561),
    [sym_add_url_entry] = ACTIONS(1561),
    [sym_delete_url_entry] = ACTIONS(1561),
    [sym_abs_value] = ACTIONS(1561),
    [sym_max_value] = ACTIONS(1561),
    [sym_min_value] = ACTIONS(1561),
    [sym_exp_value] = ACTIONS(1561),
    [sym_log_value] = ACTIONS(1561),
    [sym_log10_value] = ACTIONS(1561),
    [sym_power_value] = ACTIONS(1561),
    [sym_sqrt_value] = ACTIONS(1561),
    [sym_raw_be] = ACTIONS(1561),
    [sym_raw_le] = ACTIONS(1561),
    [sym_internal_queue_size_set] = ACTIONS(1561),
    [sym_internal_queue_size_get] = ACTIONS(1561),
    [sym_internal_threshold_set] = ACTIONS(1561),
    [sym_internal_threshold_get] = ACTIONS(1561),
    [sym_rebuilt_event] = ACTIONS(1561),
    [sym_do_custom_event] = ACTIONS(1561),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1561),
    [sym_clkmgr_set_clk_source] = ACTIONS(1561),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1561),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1561),
    [sym_clkmgr_get_timezone] = ACTIONS(1561),
    [sym_clkmgr_set_timezone] = ACTIONS(1561),
    [sym_clkmgr_get_resync_period] = ACTIONS(1561),
    [sym_clkmgr_set_resync_period] = ACTIONS(1561),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1561),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1561),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1561),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1561),
    [sym_clkmgr_get_timeservers] = ACTIONS(1561),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1561),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1561),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1561),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1561),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1561),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1561),
    [sym_set_log_level] = ACTIONS(1561),
    [sym_get_log_level] = ACTIONS(1561),
    [sym_amx_log] = ACTIONS(1561),
    [sym_smtp_server_config_set] = ACTIONS(1561),
    [sym_smtp_server_config_get] = ACTIONS(1561),
    [sym_smtp_send] = ACTIONS(1561),
    [sym_validate_netlinx_account] = ACTIONS(1561),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1561),
    [sym_authenticate_certificate] = ACTIONS(1561),
    [sym_audit_netlinx_session_event] = ACTIONS(1561),
    [sym_audit_netlinx_generic_event] = ACTIONS(1561),
    [sym_get_available_flash_disk_space] = ACTIONS(1561),
    [sym_get_max_flash_disk_space] = ACTIONS(1561),
    [sym_data_create_feed] = ACTIONS(1561),
    [sym_data_delete_feed] = ACTIONS(1561),
    [sym_data_publish_feed] = ACTIONS(1561),
    [sym_data_get_published_feed] = ACTIONS(1561),
    [sym_data_add_record] = ACTIONS(1561),
    [sym_data_get_event_record] = ACTIONS(1561),
    [sym__wc_data_create_feed] = ACTIONS(1561),
    [sym__wc_data_add_record] = ACTIONS(1561),
    [sym__wc_data_get_event_record] = ACTIONS(1561),
    [sym_format] = ACTIONS(1561),
    [sym_astro_clock] = ACTIONS(1561),
    [sym_set_outdoor_temperature] = ACTIONS(1561),
    [sym_set_virtual_level_count] = ACTIONS(1561),
    [sym_set_virtual_channel_count] = ACTIONS(1561),
    [sym_set_virtual_port_count] = ACTIONS(1561),
    [sym__wc_encode_int] = ACTIONS(1561),
    [sym__wc_decode_int] = ACTIONS(1561),
    [sym__wc_int] = ACTIONS(1561),
    [sym__wc_to_ch_int] = ACTIONS(1561),
    [sym__ch_to_wc_int] = ACTIONS(1561),
    [sym__wc_find_string_int] = ACTIONS(1561),
    [sym__wc_left_string_int] = ACTIONS(1561),
    [sym__wc_lower_string_int] = ACTIONS(1561),
    [sym__wc_mid_string_int] = ACTIONS(1561),
    [sym__wc_remove_string_int] = ACTIONS(1561),
    [sym__wc_right_string_int] = ACTIONS(1561),
    [sym__wc_upper_string_int] = ACTIONS(1561),
    [sym__wc_compare_string_int] = ACTIONS(1561),
    [sym__wc_get_buffer_char_int] = ACTIONS(1561),
    [sym__wc_get_buffer_string_int] = ACTIONS(1561),
    [sym__wc_concat_string_int] = ACTIONS(1561),
    [sym__wc_file_open_int] = ACTIONS(1561),
    [sym__wc_file_close_int] = ACTIONS(1561),
    [sym__wc_file_read_int] = ACTIONS(1561),
    [sym__wc_file_read_line_int] = ACTIONS(1561),
    [sym__wc_file_write_int] = ACTIONS(1561),
    [sym__wc_file_write_line_int] = ACTIONS(1561),
    [aux_sym_preproc_include_token1] = ACTIONS(1561),
    [aux_sym_preproc_define_token1] = ACTIONS(1563),
    [aux_sym_preproc_warn_token1] = ACTIONS(1563),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1563),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1563),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1563),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1563),
    [aux_sym_preproc_else_token1] = ACTIONS(1563),
    [anon_sym_LBRACK] = ACTIONS(1563),
    [anon_sym_LBRACE] = ACTIONS(1563),
    [anon_sym_COLON] = ACTIONS(1563),
    [anon_sym_LPAREN] = ACTIONS(1561),
    [anon_sym_SEMI] = ACTIONS(1563),
    [anon_sym_BANG] = ACTIONS(1561),
    [anon_sym_TILDE] = ACTIONS(1563),
    [anon_sym_DASH] = ACTIONS(1561),
    [anon_sym_PLUS] = ACTIONS(1561),
    [anon_sym_STAR] = ACTIONS(1563),
    [anon_sym_SLASH] = ACTIONS(1561),
    [anon_sym_PERCENT] = ACTIONS(1563),
    [anon_sym_PIPE_PIPE] = ACTIONS(1563),
    [anon_sym_AMP_AMP] = ACTIONS(1563),
    [anon_sym_PIPE] = ACTIONS(1561),
    [anon_sym_CARET] = ACTIONS(1563),
    [anon_sym_AMP] = ACTIONS(1561),
    [anon_sym_EQ_EQ] = ACTIONS(1563),
    [anon_sym_BANG_EQ] = ACTIONS(1563),
    [anon_sym_LT_GT] = ACTIONS(1563),
    [anon_sym_GT] = ACTIONS(1561),
    [anon_sym_GT_EQ] = ACTIONS(1563),
    [anon_sym_LT_EQ] = ACTIONS(1563),
    [anon_sym_LT] = ACTIONS(1561),
    [anon_sym_LT_LT] = ACTIONS(1563),
    [anon_sym_GT_GT] = ACTIONS(1563),
    [anon_sym_DASH_DASH] = ACTIONS(1563),
    [anon_sym_PLUS_PLUS] = ACTIONS(1563),
    [anon_sym_DOT] = ACTIONS(1561),
    [anon_sym_DQUOTE] = ACTIONS(1563),
    [anon_sym_SQUOTE] = ACTIONS(1563),
    [sym_number_literal] = ACTIONS(1563),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1563),
  },
  [STATE(119)] = {
    [ts_builtin_sym_end] = ACTIONS(1545),
    [sym_identifier] = ACTIONS(1543),
    [sym_program_name_keyword] = ACTIONS(1543),
    [sym_module_name_keyword] = ACTIONS(1543),
    [sym_define_device_keyword] = ACTIONS(1543),
    [sym_define_combine_keyword] = ACTIONS(1543),
    [sym_define_constant_keyword] = ACTIONS(1543),
    [sym_define_type_keyword] = ACTIONS(1543),
    [sym_define_variable_keyword] = ACTIONS(1543),
    [sym_define_system_variable_keyword] = ACTIONS(1543),
    [sym_define_connect_level_keyword] = ACTIONS(1543),
    [sym_define_latching_keyword] = ACTIONS(1543),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1543),
    [sym_define_toggling_keyword] = ACTIONS(1543),
    [sym_define_call_keyword] = ACTIONS(1543),
    [sym_define_function_keyword] = ACTIONS(1543),
    [sym_define_library_function_keyword] = ACTIONS(1543),
    [sym_define_start_keyword] = ACTIONS(1543),
    [sym_define_event_keyword] = ACTIONS(1543),
    [sym_define_program_keyword] = ACTIONS(1543),
    [sym_define_module_keyword] = ACTIONS(1543),
    [sym_char_keyword] = ACTIONS(1543),
    [sym_widechar_keyword] = ACTIONS(1543),
    [sym_integer_keyword] = ACTIONS(1543),
    [sym_sinteger_keyword] = ACTIONS(1543),
    [sym_long_keyword] = ACTIONS(1543),
    [sym_slong_keyword] = ACTIONS(1543),
    [sym_float_keyword] = ACTIONS(1543),
    [sym_double_keyword] = ACTIONS(1543),
    [sym_variant_keyword] = ACTIONS(1543),
    [sym_variantarray_keyword] = ACTIONS(1543),
    [sym_local_var_keyword] = ACTIONS(1543),
    [sym_stack_var_keyword] = ACTIONS(1543),
    [sym_constant_keyword] = ACTIONS(1543),
    [sym_volatile_keyword] = ACTIONS(1543),
    [sym_non_volatile_keyword] = ACTIONS(1543),
    [sym_persistent_keyword] = ACTIONS(1543),
    [sym_dev_keyword] = ACTIONS(1543),
    [sym_devlev_keyword] = ACTIONS(1543),
    [sym_devchan_keyword] = ACTIONS(1543),
    [sym_if_keyword] = ACTIONS(1543),
    [sym_else_keyword] = ACTIONS(1543),
    [sym_switch_keyword] = ACTIONS(1543),
    [sym_case_keyword] = ACTIONS(1543),
    [sym_default_keyword] = ACTIONS(1543),
    [sym_while_keyword] = ACTIONS(1543),
    [sym_for_keyword] = ACTIONS(1543),
    [sym_break_keyword] = ACTIONS(1543),
    [sym_continue_keyword] = ACTIONS(1543),
    [sym_return_keyword] = ACTIONS(1543),
    [sym_select_keyword] = ACTIONS(1543),
    [sym_active_keyword] = ACTIONS(1543),
    [sym_struct_keyword] = ACTIONS(1543),
    [sym_structure_keyword] = ACTIONS(1543),
    [sym_button_event_keyword] = ACTIONS(1543),
    [sym_channel_event_keyword] = ACTIONS(1543),
    [sym_level_event_keyword] = ACTIONS(1543),
    [sym_data_event_keyword] = ACTIONS(1543),
    [sym_timeline_event_keyword] = ACTIONS(1543),
    [sym_custom_event_keyword] = ACTIONS(1543),
    [sym_push_keyword] = ACTIONS(1543),
    [sym_release_keyword] = ACTIONS(1543),
    [aux_sym_on_keyword_token1] = ACTIONS(1543),
    [aux_sym_off_keyword_token1] = ACTIONS(1543),
    [sym_send_level_keyword] = ACTIONS(1543),
    [sym_send_string_keyword] = ACTIONS(1543),
    [sym_send_command_keyword] = ACTIONS(1543),
    [sym_clear_buffer_keyword] = ACTIONS(1543),
    [sym_create_buffer_keyword] = ACTIONS(1543),
    [sym_create_multi_buffer_keyword] = ACTIONS(1543),
    [sym_call_keyword] = ACTIONS(1543),
    [sym_system_call_keyword] = ACTIONS(1543),
    [sym_devchan_to_keyword] = ACTIONS(1543),
    [sym_devchan_min_to_keyword] = ACTIONS(1543),
    [sym_devchan_total_off_keyword] = ACTIONS(1543),
    [sym_devchan_pulse_keyword] = ACTIONS(1543),
    [sym_wait_keyword] = ACTIONS(1543),
    [sym_cancel_wait_keyword] = ACTIONS(1543),
    [sym_wait_until_keyword] = ACTIONS(1543),
    [sym_cancel_wait_until_keyword] = ACTIONS(1543),
    [sym_cancel_all_wait_keyword] = ACTIONS(1543),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1543),
    [sym_and] = ACTIONS(1543),
    [sym_or] = ACTIONS(1543),
    [sym_not] = ACTIONS(1543),
    [sym_xor] = ACTIONS(1543),
    [sym_band] = ACTIONS(1543),
    [sym_bor] = ACTIONS(1543),
    [sym_bxor] = ACTIONS(1543),
    [sym_bnot] = ACTIONS(1543),
    [sym_lshift] = ACTIONS(1543),
    [sym_rshift] = ACTIONS(1543),
    [sym___file__] = ACTIONS(1543),
    [sym___date__] = ACTIONS(1543),
    [sym___ldate__] = ACTIONS(1543),
    [sym___time__] = ACTIONS(1543),
    [sym___line__] = ACTIONS(1543),
    [sym___name__] = ACTIONS(1543),
    [sym___netlinx__] = ACTIONS(1543),
    [sym_netlinx_axi_version] = ACTIONS(1543),
    [sym_url_flg_tcp] = ACTIONS(1543),
    [sym_url_flg_accinfopresent] = ACTIONS(1543),
    [sym_url_flg_temp] = ACTIONS(1543),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1543),
    [sym_url_flg_stat_mask] = ACTIONS(1543),
    [sym_url_flg_stat_lookup] = ACTIONS(1543),
    [sym_url_flg_stat_connecting] = ACTIONS(1543),
    [sym_url_flg_stat_waiting] = ACTIONS(1543),
    [sym_url_flg_stat_connected] = ACTIONS(1543),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1543),
    [sym_first_virtual_device] = ACTIONS(1543),
    [sym_first_local_port] = ACTIONS(1543),
    [sym_dynamic_virtual_device] = ACTIONS(1543),
    [sym_timeline_once] = ACTIONS(1543),
    [sym_timeline_repeat] = ACTIONS(1543),
    [sym_timeline_absolute] = ACTIONS(1543),
    [sym_timeline_relative] = ACTIONS(1543),
    [sym_do_push_timed_infinite] = ACTIONS(1543),
    [sym_source_type_no_address] = ACTIONS(1543),
    [sym_source_type_neuron_id] = ACTIONS(1543),
    [sym_source_type_ip_address] = ACTIONS(1543),
    [sym_source_type_axlink] = ACTIONS(1543),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1543),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1543),
    [sym_source_type_ip_socket_address] = ACTIONS(1543),
    [sym_source_type_rs232] = ACTIONS(1543),
    [sym_source_type_internal] = ACTIONS(1543),
    [sym_source_type_mac_address] = ACTIONS(1543),
    [sym_source_type_ipv4_port] = ACTIONS(1543),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1543),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1543),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1543),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1543),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1543),
    [sym_true] = ACTIONS(1543),
    [sym_false] = ACTIONS(1543),
    [sym_file_read_only] = ACTIONS(1543),
    [sym_file_rw_new] = ACTIONS(1543),
    [sym_file_rw_append] = ACTIONS(1543),
    [sym_ip_tcp] = ACTIONS(1543),
    [sym_ip_udp] = ACTIONS(1543),
    [sym_ip_udp_2way] = ACTIONS(1543),
    [sym_xml_encode_types] = ACTIONS(1543),
    [sym_xml_encode_char_as_list] = ACTIONS(1543),
    [sym_xml_encode_le] = ACTIONS(1543),
    [sym_xml_decode_types] = ACTIONS(1543),
    [sym_xml_decode_no_preserve] = ACTIONS(1543),
    [sym_duet_dev_type_display_device] = ACTIONS(1543),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1543),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1543),
    [sym_duet_dev_type_io_device] = ACTIONS(1543),
    [sym_duet_dev_type_relay_device] = ACTIONS(1543),
    [sym_duet_dev_type_ups] = ACTIONS(1543),
    [sym_duet_dev_type_amplifier] = ACTIONS(1543),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1543),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1543),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1543),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1543),
    [sym_duet_dev_type_camera] = ACTIONS(1543),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1543),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1543),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1543),
    [sym_duet_dev_type_dss] = ACTIONS(1543),
    [sym_duet_dev_type_dvr] = ACTIONS(1543),
    [sym_duet_dev_type_disc_device] = ACTIONS(1543),
    [sym_duet_dev_type_document_camera] = ACTIONS(1543),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1543),
    [sym_duet_dev_type_hvac] = ACTIONS(1543),
    [sym_duet_dev_type_keypad] = ACTIONS(1543),
    [sym_duet_dev_type_light] = ACTIONS(1543),
    [sym_duet_dev_type_monitor] = ACTIONS(1543),
    [sym_duet_dev_type_motor] = ACTIONS(1543),
    [sym_duet_dev_type_multi_window] = ACTIONS(1543),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1543),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1543),
    [sym_duet_dev_type_receiver] = ACTIONS(1543),
    [sym_duet_dev_type_security_system] = ACTIONS(1543),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1543),
    [sym_duet_dev_type_settop_box] = ACTIONS(1543),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1543),
    [sym_duet_dev_type_switcher] = ACTIONS(1543),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1543),
    [sym_duet_dev_type_tv] = ACTIONS(1543),
    [sym_duet_dev_type_utility] = ACTIONS(1543),
    [sym_duet_dev_type_vcr] = ACTIONS(1543),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1543),
    [sym_duet_dev_type_video_processor] = ACTIONS(1543),
    [sym_duet_dev_type_video_projector] = ACTIONS(1543),
    [sym_duet_dev_type_video_wall] = ACTIONS(1543),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1543),
    [sym_duet_dev_type_weather] = ACTIONS(1543),
    [sym_duet_dev_not_polled] = ACTIONS(1543),
    [sym_duet_dev_polled] = ACTIONS(1543),
    [sym_clkmgr_mode_network] = ACTIONS(1543),
    [sym_clkmgr_mode_standalone] = ACTIONS(1543),
    [sym_normal_standby] = ACTIONS(1543),
    [sym_normal_wake] = ACTIONS(1543),
    [sym_ip_multicast_ttl_option] = ACTIONS(1543),
    [sym_ip_ttl_subnet] = ACTIONS(1543),
    [sym_ip_ttl_site] = ACTIONS(1543),
    [sym_ip_ttl_region] = ACTIONS(1543),
    [sym_ip_ttl_continent] = ACTIONS(1543),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1543),
    [sym_ip_nodelay_on] = ACTIONS(1543),
    [sym_ip_nodelay_off] = ACTIONS(1543),
    [sym_amx_error] = ACTIONS(1543),
    [sym_amx_warning] = ACTIONS(1543),
    [sym_amx_info] = ACTIONS(1543),
    [sym_amx_debug] = ACTIONS(1543),
    [sym_netlinx_login_success] = ACTIONS(1543),
    [sym_netlinx_login_fail] = ACTIONS(1543),
    [sym_netlinx_logout] = ACTIONS(1543),
    [sym_null_str] = ACTIONS(1543),
    [sym_smtp_address] = ACTIONS(1543),
    [sym_smtp_port_number] = ACTIONS(1543),
    [sym_smtp_username] = ACTIONS(1543),
    [sym_smtp_password] = ACTIONS(1543),
    [sym_smtp_require_tls] = ACTIONS(1543),
    [sym_smtp_from] = ACTIONS(1543),
    [sym_smtp_tls_true] = ACTIONS(1543),
    [sym_smtp_tls_false] = ACTIONS(1543),
    [sym_listview_on_row_select_event] = ACTIONS(1543),
    [sym_data_structure_unknown] = ACTIONS(1543),
    [sym_data_structure_datafeed] = ACTIONS(1543),
    [sym_data_structure_datarecordset] = ACTIONS(1543),
    [sym_data_structure_datarecord] = ACTIONS(1543),
    [sym_data_structure_datafield] = ACTIONS(1543),
    [sym_data_max_content_field_count] = ACTIONS(1543),
    [sym_data_max_metadata_field_count] = ACTIONS(1543),
    [sym_data_max_id_length] = ACTIONS(1543),
    [sym_data_max_name_length] = ACTIONS(1543),
    [sym_data_max_description_length] = ACTIONS(1543),
    [sym_data_max_type_length] = ACTIONS(1543),
    [sym_data_max_format_length] = ACTIONS(1543),
    [sym_data_max_source_length] = ACTIONS(1543),
    [sym_data_max_value_length] = ACTIONS(1543),
    [sym_data_max_label_length] = ACTIONS(1543),
    [sym_data_max_url_length] = ACTIONS(1543),
    [sym_max_last_login_info_length] = ACTIONS(1543),
    [sym_data_type_unknown] = ACTIONS(1543),
    [sym_data_type_string] = ACTIONS(1543),
    [sym_data_type_datetime] = ACTIONS(1543),
    [sym_data_type_date] = ACTIONS(1543),
    [sym_data_type_time] = ACTIONS(1543),
    [sym_data_type_image] = ACTIONS(1543),
    [sym_data_format_url] = ACTIONS(1543),
    [sym_data_format_phone] = ACTIONS(1543),
    [sym_data_format_email] = ACTIONS(1543),
    [sym_data_format_iso8601] = ACTIONS(1543),
    [sym_auditlog_priv] = ACTIONS(1543),
    [sym_configuration_priv] = ACTIONS(1543),
    [sym_deviceconfig_priv] = ACTIONS(1543),
    [sym_ftp_priv] = ACTIONS(1543),
    [sym_http_priv] = ACTIONS(1543),
    [sym_networkconfig_priv] = ACTIONS(1543),
    [sym_programport_priv] = ACTIONS(1543),
    [sym_remoteui_priv] = ACTIONS(1543),
    [sym_securitycontrol_priv] = ACTIONS(1543),
    [sym_softwaremanagement_priv] = ACTIONS(1543),
    [sym_terminal_priv] = ACTIONS(1543),
    [sym_tpadmin_priv] = ACTIONS(1543),
    [sym_user1_priv] = ACTIONS(1543),
    [sym_user2_priv] = ACTIONS(1543),
    [sym_user3_priv] = ACTIONS(1543),
    [sym_user4_priv] = ACTIONS(1543),
    [sym_usermanagement_priv] = ACTIONS(1543),
    [sym_valid_account] = ACTIONS(1543),
    [sym_err_name_invalid] = ACTIONS(1543),
    [sym_err_password_invalid] = ACTIONS(1543),
    [sym_err_invalid_account] = ACTIONS(1543),
    [sym_err_no_authorization] = ACTIONS(1543),
    [sym_err_invalid_parameter] = ACTIONS(1543),
    [sym_err_user_locked_out] = ACTIONS(1543),
    [sym_err_user_account_expired] = ACTIONS(1543),
    [sym_err_general_error] = ACTIONS(1543),
    [sym_err_authentication_error] = ACTIONS(1543),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1543),
    [sym_tls_validate_certificate] = ACTIONS(1543),
    [sym_tdata] = ACTIONS(1543),
    [sym_tchannel] = ACTIONS(1543),
    [sym_tlevel] = ACTIONS(1543),
    [sym_tbutton] = ACTIONS(1543),
    [sym_ttimeline] = ACTIONS(1543),
    [sym_tcustom] = ACTIONS(1543),
    [sym_url_struct] = ACTIONS(1543),
    [sym_dns_struct] = ACTIONS(1543),
    [sym_ip_address_struct] = ACTIONS(1543),
    [sym_dev_info_struct] = ACTIONS(1543),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1543),
    [sym_data_feed] = ACTIONS(1543),
    [sym_data_field] = ACTIONS(1543),
    [sym_data_record] = ACTIONS(1543),
    [sym_wc_data_feed] = ACTIONS(1543),
    [sym_wc_data_field] = ACTIONS(1543),
    [sym_wc_data_record] = ACTIONS(1543),
    [sym_last_login_info] = ACTIONS(1543),
    [sym____reserved___] = ACTIONS(1543),
    [sym_date] = ACTIONS(1543),
    [sym_day] = ACTIONS(1543),
    [sym_time] = ACTIONS(1543),
    [sym_ldate] = ACTIONS(1543),
    [sym_push_device] = ACTIONS(1543),
    [sym_push_channel] = ACTIONS(1543),
    [sym_push_devchan] = ACTIONS(1543),
    [sym_release_device] = ACTIONS(1543),
    [sym_release_channel] = ACTIONS(1543),
    [sym_release_devchan] = ACTIONS(1543),
    [sym_master_slot] = ACTIONS(1543),
    [sym_get_pulse_time] = ACTIONS(1543),
    [sym_get_timer] = ACTIONS(1543),
    [sym_master_sn] = ACTIONS(1543),
    [sym_system_number] = ACTIONS(1543),
    [sym_button] = ACTIONS(1543),
    [sym_dv_channel] = ACTIONS(1543),
    [sym_data] = ACTIONS(1543),
    [sym_level] = ACTIONS(1543),
    [sym_timeline] = ACTIONS(1543),
    [sym_channel] = ACTIONS(1543),
    [sym_custom] = ACTIONS(1543),
    [sym_atoi] = ACTIONS(1543),
    [sym_atol] = ACTIONS(1543),
    [sym_atof] = ACTIONS(1543),
    [sym_itoa] = ACTIONS(1543),
    [sym_itohex] = ACTIONS(1543),
    [sym_hextoi] = ACTIONS(1543),
    [sym_ftoa] = ACTIONS(1543),
    [sym_find_string] = ACTIONS(1543),
    [sym_left_string] = ACTIONS(1543),
    [sym_right_string] = ACTIONS(1543),
    [sym_mid_string] = ACTIONS(1543),
    [sym_remove_string] = ACTIONS(1543),
    [sym_get_buffer_string] = ACTIONS(1543),
    [sym_get_buffer_char] = ACTIONS(1543),
    [sym_get_multi_buffer_string] = ACTIONS(1543),
    [sym_length_string] = ACTIONS(1543),
    [sym_max_length_string] = ACTIONS(1543),
    [sym_set_length_string] = ACTIONS(1543),
    [sym_compare_string] = ACTIONS(1543),
    [sym_upper_string] = ACTIONS(1543),
    [sym_lower_string] = ACTIONS(1543),
    [sym_get_last] = ACTIONS(1543),
    [sym_date_to_day] = ACTIONS(1543),
    [sym_date_to_month] = ACTIONS(1543),
    [sym_date_to_year] = ACTIONS(1543),
    [sym_time_to_hour] = ACTIONS(1543),
    [sym_time_to_minute] = ACTIONS(1543),
    [sym_time_to_second] = ACTIONS(1543),
    [sym_day_of_week] = ACTIONS(1543),
    [sym_set_pulse_time] = ACTIONS(1543),
    [sym_set_timer] = ACTIONS(1543),
    [sym_file_open] = ACTIONS(1543),
    [sym_file_close] = ACTIONS(1543),
    [sym_file_read] = ACTIONS(1543),
    [sym_file_read_line] = ACTIONS(1543),
    [sym_file_write] = ACTIONS(1543),
    [sym_file_write_line] = ACTIONS(1543),
    [sym_file_copy] = ACTIONS(1543),
    [sym_file_delete] = ACTIONS(1543),
    [sym_file_rename] = ACTIONS(1543),
    [sym_file_dir] = ACTIONS(1543),
    [sym_file_getdir] = ACTIONS(1543),
    [sym_file_setdir] = ACTIONS(1543),
    [sym_file_create_dir] = ACTIONS(1543),
    [sym_file_remove_dir] = ACTIONS(1543),
    [sym_file_seek] = ACTIONS(1543),
    [sym_length_array] = ACTIONS(1543),
    [sym_max_length_array] = ACTIONS(1543),
    [sym_set_length_array] = ACTIONS(1543),
    [sym_ip_client_open] = ACTIONS(1543),
    [sym_ip_client_close] = ACTIONS(1543),
    [sym_ip_server_open] = ACTIONS(1543),
    [sym_ip_server_close] = ACTIONS(1543),
    [sym_ip_bound_client_open] = ACTIONS(1543),
    [sym_ip_mc_server_open] = ACTIONS(1543),
    [sym_ip_set_option] = ACTIONS(1543),
    [sym_ssh_client_open] = ACTIONS(1543),
    [sym_ssh_client_close] = ACTIONS(1543),
    [sym_tls_client_open] = ACTIONS(1543),
    [sym_tls_client_close] = ACTIONS(1543),
    [sym_do_push] = ACTIONS(1543),
    [sym_do_push_timed] = ACTIONS(1543),
    [sym_do_release] = ACTIONS(1543),
    [sym_device_id] = ACTIONS(1543),
    [sym_device_info] = ACTIONS(1543),
    [sym_device_id_string] = ACTIONS(1543),
    [sym_device_standby] = ACTIONS(1543),
    [sym_device_wake] = ACTIONS(1543),
    [sym_timeline_create] = ACTIONS(1543),
    [sym_timeline_kill] = ACTIONS(1543),
    [sym_timeline_reload] = ACTIONS(1543),
    [sym_timeline_pause] = ACTIONS(1543),
    [sym_timeline_restart] = ACTIONS(1543),
    [sym_timeline_set] = ACTIONS(1543),
    [sym_timeline_get] = ACTIONS(1543),
    [sym_timeline_active] = ACTIONS(1543),
    [sym_timeline_dynamic_id] = ACTIONS(1543),
    [sym_variable_to_string] = ACTIONS(1543),
    [sym_string_to_variable] = ACTIONS(1543),
    [sym_length_variable_to_string] = ACTIONS(1543),
    [sym_type_cast] = ACTIONS(1543),
    [sym_variable_to_xml] = ACTIONS(1543),
    [sym_xml_to_variable] = ACTIONS(1543),
    [sym_length_variable_to_xml] = ACTIONS(1543),
    [sym_redirect_string] = ACTIONS(1543),
    [sym_random_number] = ACTIONS(1543),
    [sym_get_unique_id] = ACTIONS(1543),
    [sym_get_system_number] = ACTIONS(1543),
    [sym_set_system_number] = ACTIONS(1543),
    [sym_get_serial_number] = ACTIONS(1543),
    [sym_get_master_build] = ACTIONS(1543),
    [sym_set_validation_code] = ACTIONS(1543),
    [sym_reboot] = ACTIONS(1543),
    [sym_load_duet_module] = ACTIONS(1543),
    [sym_unload_duet_module] = ACTIONS(1543),
    [sym_duet_mem_size_set] = ACTIONS(1543),
    [sym_duet_mem_size_get] = ACTIONS(1543),
    [sym_dynamic_application_device] = ACTIONS(1543),
    [sym_dynamic_polled_port] = ACTIONS(1543),
    [sym_static_port_binding] = ACTIONS(1543),
    [sym_static_ip_binding] = ACTIONS(1543),
    [sym_get_dns_list] = ACTIONS(1543),
    [sym_set_dns_list] = ACTIONS(1543),
    [sym_get_ip_address] = ACTIONS(1543),
    [sym_set_ip_address] = ACTIONS(1543),
    [sym_get_url_list] = ACTIONS(1543),
    [sym_add_url_entry] = ACTIONS(1543),
    [sym_delete_url_entry] = ACTIONS(1543),
    [sym_abs_value] = ACTIONS(1543),
    [sym_max_value] = ACTIONS(1543),
    [sym_min_value] = ACTIONS(1543),
    [sym_exp_value] = ACTIONS(1543),
    [sym_log_value] = ACTIONS(1543),
    [sym_log10_value] = ACTIONS(1543),
    [sym_power_value] = ACTIONS(1543),
    [sym_sqrt_value] = ACTIONS(1543),
    [sym_raw_be] = ACTIONS(1543),
    [sym_raw_le] = ACTIONS(1543),
    [sym_internal_queue_size_set] = ACTIONS(1543),
    [sym_internal_queue_size_get] = ACTIONS(1543),
    [sym_internal_threshold_set] = ACTIONS(1543),
    [sym_internal_threshold_get] = ACTIONS(1543),
    [sym_rebuilt_event] = ACTIONS(1543),
    [sym_do_custom_event] = ACTIONS(1543),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1543),
    [sym_clkmgr_set_clk_source] = ACTIONS(1543),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1543),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1543),
    [sym_clkmgr_get_timezone] = ACTIONS(1543),
    [sym_clkmgr_set_timezone] = ACTIONS(1543),
    [sym_clkmgr_get_resync_period] = ACTIONS(1543),
    [sym_clkmgr_set_resync_period] = ACTIONS(1543),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1543),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1543),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1543),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1543),
    [sym_clkmgr_get_timeservers] = ACTIONS(1543),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1543),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1543),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1543),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1543),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1543),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1543),
    [sym_set_log_level] = ACTIONS(1543),
    [sym_get_log_level] = ACTIONS(1543),
    [sym_amx_log] = ACTIONS(1543),
    [sym_smtp_server_config_set] = ACTIONS(1543),
    [sym_smtp_server_config_get] = ACTIONS(1543),
    [sym_smtp_send] = ACTIONS(1543),
    [sym_validate_netlinx_account] = ACTIONS(1543),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1543),
    [sym_authenticate_certificate] = ACTIONS(1543),
    [sym_audit_netlinx_session_event] = ACTIONS(1543),
    [sym_audit_netlinx_generic_event] = ACTIONS(1543),
    [sym_get_available_flash_disk_space] = ACTIONS(1543),
    [sym_get_max_flash_disk_space] = ACTIONS(1543),
    [sym_data_create_feed] = ACTIONS(1543),
    [sym_data_delete_feed] = ACTIONS(1543),
    [sym_data_publish_feed] = ACTIONS(1543),
    [sym_data_get_published_feed] = ACTIONS(1543),
    [sym_data_add_record] = ACTIONS(1543),
    [sym_data_get_event_record] = ACTIONS(1543),
    [sym__wc_data_create_feed] = ACTIONS(1543),
    [sym__wc_data_add_record] = ACTIONS(1543),
    [sym__wc_data_get_event_record] = ACTIONS(1543),
    [sym_format] = ACTIONS(1543),
    [sym_astro_clock] = ACTIONS(1543),
    [sym_set_outdoor_temperature] = ACTIONS(1543),
    [sym_set_virtual_level_count] = ACTIONS(1543),
    [sym_set_virtual_channel_count] = ACTIONS(1543),
    [sym_set_virtual_port_count] = ACTIONS(1543),
    [sym__wc_encode_int] = ACTIONS(1543),
    [sym__wc_decode_int] = ACTIONS(1543),
    [sym__wc_int] = ACTIONS(1543),
    [sym__wc_to_ch_int] = ACTIONS(1543),
    [sym__ch_to_wc_int] = ACTIONS(1543),
    [sym__wc_find_string_int] = ACTIONS(1543),
    [sym__wc_left_string_int] = ACTIONS(1543),
    [sym__wc_lower_string_int] = ACTIONS(1543),
    [sym__wc_mid_string_int] = ACTIONS(1543),
    [sym__wc_remove_string_int] = ACTIONS(1543),
    [sym__wc_right_string_int] = ACTIONS(1543),
    [sym__wc_upper_string_int] = ACTIONS(1543),
    [sym__wc_compare_string_int] = ACTIONS(1543),
    [sym__wc_get_buffer_char_int] = ACTIONS(1543),
    [sym__wc_get_buffer_string_int] = ACTIONS(1543),
    [sym__wc_concat_string_int] = ACTIONS(1543),
    [sym__wc_file_open_int] = ACTIONS(1543),
    [sym__wc_file_close_int] = ACTIONS(1543),
    [sym__wc_file_read_int] = ACTIONS(1543),
    [sym__wc_file_read_line_int] = ACTIONS(1543),
    [sym__wc_file_write_int] = ACTIONS(1543),
    [sym__wc_file_write_line_int] = ACTIONS(1543),
    [anon_sym_EQ] = ACTIONS(1543),
    [aux_sym_preproc_include_token1] = ACTIONS(1543),
    [aux_sym_preproc_define_token1] = ACTIONS(1545),
    [aux_sym_preproc_warn_token1] = ACTIONS(1545),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1545),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1545),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1545),
    [anon_sym_LBRACK] = ACTIONS(1545),
    [anon_sym_LBRACE] = ACTIONS(1545),
    [anon_sym_COLON] = ACTIONS(1545),
    [anon_sym_LPAREN] = ACTIONS(1543),
    [anon_sym_SEMI] = ACTIONS(1545),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1545),
    [anon_sym_DASH] = ACTIONS(1543),
    [anon_sym_PLUS] = ACTIONS(1543),
    [anon_sym_STAR] = ACTIONS(1545),
    [anon_sym_SLASH] = ACTIONS(1543),
    [anon_sym_PERCENT] = ACTIONS(1545),
    [anon_sym_PIPE_PIPE] = ACTIONS(1545),
    [anon_sym_AMP_AMP] = ACTIONS(1545),
    [anon_sym_PIPE] = ACTIONS(1543),
    [anon_sym_CARET] = ACTIONS(1545),
    [anon_sym_AMP] = ACTIONS(1543),
    [anon_sym_EQ_EQ] = ACTIONS(1545),
    [anon_sym_BANG_EQ] = ACTIONS(1545),
    [anon_sym_LT_GT] = ACTIONS(1545),
    [anon_sym_GT] = ACTIONS(1543),
    [anon_sym_GT_EQ] = ACTIONS(1545),
    [anon_sym_LT_EQ] = ACTIONS(1545),
    [anon_sym_LT] = ACTIONS(1543),
    [anon_sym_LT_LT] = ACTIONS(1545),
    [anon_sym_GT_GT] = ACTIONS(1545),
    [anon_sym_DASH_DASH] = ACTIONS(1545),
    [anon_sym_PLUS_PLUS] = ACTIONS(1545),
    [anon_sym_DOT] = ACTIONS(1543),
    [anon_sym_DQUOTE] = ACTIONS(1545),
    [anon_sym_SQUOTE] = ACTIONS(1545),
    [sym_number_literal] = ACTIONS(1545),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1545),
  },
  [STATE(120)] = {
    [sym_identifier] = ACTIONS(1565),
    [sym_program_name_keyword] = ACTIONS(1565),
    [sym_module_name_keyword] = ACTIONS(1565),
    [sym_define_device_keyword] = ACTIONS(1565),
    [sym_define_combine_keyword] = ACTIONS(1565),
    [sym_define_constant_keyword] = ACTIONS(1565),
    [sym_define_type_keyword] = ACTIONS(1565),
    [sym_define_variable_keyword] = ACTIONS(1565),
    [sym_define_system_variable_keyword] = ACTIONS(1565),
    [sym_define_connect_level_keyword] = ACTIONS(1565),
    [sym_define_latching_keyword] = ACTIONS(1565),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1565),
    [sym_define_toggling_keyword] = ACTIONS(1565),
    [sym_define_call_keyword] = ACTIONS(1565),
    [sym_define_function_keyword] = ACTIONS(1565),
    [sym_define_library_function_keyword] = ACTIONS(1565),
    [sym_define_start_keyword] = ACTIONS(1565),
    [sym_define_event_keyword] = ACTIONS(1565),
    [sym_define_program_keyword] = ACTIONS(1565),
    [sym_define_module_keyword] = ACTIONS(1565),
    [sym_char_keyword] = ACTIONS(1565),
    [sym_widechar_keyword] = ACTIONS(1565),
    [sym_integer_keyword] = ACTIONS(1565),
    [sym_sinteger_keyword] = ACTIONS(1565),
    [sym_long_keyword] = ACTIONS(1565),
    [sym_slong_keyword] = ACTIONS(1565),
    [sym_float_keyword] = ACTIONS(1565),
    [sym_double_keyword] = ACTIONS(1565),
    [sym_variant_keyword] = ACTIONS(1565),
    [sym_variantarray_keyword] = ACTIONS(1565),
    [sym_local_var_keyword] = ACTIONS(1565),
    [sym_stack_var_keyword] = ACTIONS(1565),
    [sym_constant_keyword] = ACTIONS(1565),
    [sym_volatile_keyword] = ACTIONS(1565),
    [sym_non_volatile_keyword] = ACTIONS(1565),
    [sym_persistent_keyword] = ACTIONS(1565),
    [sym_dev_keyword] = ACTIONS(1565),
    [sym_devlev_keyword] = ACTIONS(1565),
    [sym_devchan_keyword] = ACTIONS(1565),
    [sym_if_keyword] = ACTIONS(1565),
    [sym_else_keyword] = ACTIONS(1565),
    [sym_switch_keyword] = ACTIONS(1565),
    [sym_case_keyword] = ACTIONS(1565),
    [sym_default_keyword] = ACTIONS(1565),
    [sym_while_keyword] = ACTIONS(1565),
    [sym_for_keyword] = ACTIONS(1565),
    [sym_break_keyword] = ACTIONS(1565),
    [sym_continue_keyword] = ACTIONS(1565),
    [sym_return_keyword] = ACTIONS(1565),
    [sym_select_keyword] = ACTIONS(1565),
    [sym_active_keyword] = ACTIONS(1565),
    [sym_struct_keyword] = ACTIONS(1565),
    [sym_structure_keyword] = ACTIONS(1565),
    [sym_button_event_keyword] = ACTIONS(1565),
    [sym_channel_event_keyword] = ACTIONS(1565),
    [sym_level_event_keyword] = ACTIONS(1565),
    [sym_data_event_keyword] = ACTIONS(1565),
    [sym_timeline_event_keyword] = ACTIONS(1565),
    [sym_custom_event_keyword] = ACTIONS(1565),
    [sym_push_keyword] = ACTIONS(1565),
    [sym_release_keyword] = ACTIONS(1565),
    [aux_sym_on_keyword_token1] = ACTIONS(1565),
    [aux_sym_off_keyword_token1] = ACTIONS(1565),
    [sym_send_level_keyword] = ACTIONS(1565),
    [sym_send_string_keyword] = ACTIONS(1565),
    [sym_send_command_keyword] = ACTIONS(1565),
    [sym_clear_buffer_keyword] = ACTIONS(1565),
    [sym_create_buffer_keyword] = ACTIONS(1565),
    [sym_create_multi_buffer_keyword] = ACTIONS(1565),
    [sym_call_keyword] = ACTIONS(1565),
    [sym_system_call_keyword] = ACTIONS(1565),
    [sym_devchan_to_keyword] = ACTIONS(1565),
    [sym_devchan_min_to_keyword] = ACTIONS(1565),
    [sym_devchan_total_off_keyword] = ACTIONS(1565),
    [sym_devchan_pulse_keyword] = ACTIONS(1565),
    [sym_wait_keyword] = ACTIONS(1565),
    [sym_cancel_wait_keyword] = ACTIONS(1565),
    [sym_wait_until_keyword] = ACTIONS(1565),
    [sym_cancel_wait_until_keyword] = ACTIONS(1565),
    [sym_cancel_all_wait_keyword] = ACTIONS(1565),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1565),
    [sym_and] = ACTIONS(1565),
    [sym_or] = ACTIONS(1565),
    [sym_not] = ACTIONS(1565),
    [sym_xor] = ACTIONS(1565),
    [sym_band] = ACTIONS(1565),
    [sym_bor] = ACTIONS(1565),
    [sym_bxor] = ACTIONS(1565),
    [sym_bnot] = ACTIONS(1565),
    [sym_lshift] = ACTIONS(1565),
    [sym_rshift] = ACTIONS(1565),
    [sym___file__] = ACTIONS(1565),
    [sym___date__] = ACTIONS(1565),
    [sym___ldate__] = ACTIONS(1565),
    [sym___time__] = ACTIONS(1565),
    [sym___line__] = ACTIONS(1565),
    [sym___name__] = ACTIONS(1565),
    [sym___netlinx__] = ACTIONS(1565),
    [sym_netlinx_axi_version] = ACTIONS(1565),
    [sym_url_flg_tcp] = ACTIONS(1565),
    [sym_url_flg_accinfopresent] = ACTIONS(1565),
    [sym_url_flg_temp] = ACTIONS(1565),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1565),
    [sym_url_flg_stat_mask] = ACTIONS(1565),
    [sym_url_flg_stat_lookup] = ACTIONS(1565),
    [sym_url_flg_stat_connecting] = ACTIONS(1565),
    [sym_url_flg_stat_waiting] = ACTIONS(1565),
    [sym_url_flg_stat_connected] = ACTIONS(1565),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1565),
    [sym_first_virtual_device] = ACTIONS(1565),
    [sym_first_local_port] = ACTIONS(1565),
    [sym_dynamic_virtual_device] = ACTIONS(1565),
    [sym_timeline_once] = ACTIONS(1565),
    [sym_timeline_repeat] = ACTIONS(1565),
    [sym_timeline_absolute] = ACTIONS(1565),
    [sym_timeline_relative] = ACTIONS(1565),
    [sym_do_push_timed_infinite] = ACTIONS(1565),
    [sym_source_type_no_address] = ACTIONS(1565),
    [sym_source_type_neuron_id] = ACTIONS(1565),
    [sym_source_type_ip_address] = ACTIONS(1565),
    [sym_source_type_axlink] = ACTIONS(1565),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1565),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1565),
    [sym_source_type_ip_socket_address] = ACTIONS(1565),
    [sym_source_type_rs232] = ACTIONS(1565),
    [sym_source_type_internal] = ACTIONS(1565),
    [sym_source_type_mac_address] = ACTIONS(1565),
    [sym_source_type_ipv4_port] = ACTIONS(1565),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1565),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1565),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1565),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1565),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1565),
    [sym_true] = ACTIONS(1565),
    [sym_false] = ACTIONS(1565),
    [sym_file_read_only] = ACTIONS(1565),
    [sym_file_rw_new] = ACTIONS(1565),
    [sym_file_rw_append] = ACTIONS(1565),
    [sym_ip_tcp] = ACTIONS(1565),
    [sym_ip_udp] = ACTIONS(1565),
    [sym_ip_udp_2way] = ACTIONS(1565),
    [sym_xml_encode_types] = ACTIONS(1565),
    [sym_xml_encode_char_as_list] = ACTIONS(1565),
    [sym_xml_encode_le] = ACTIONS(1565),
    [sym_xml_decode_types] = ACTIONS(1565),
    [sym_xml_decode_no_preserve] = ACTIONS(1565),
    [sym_duet_dev_type_display_device] = ACTIONS(1565),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1565),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1565),
    [sym_duet_dev_type_io_device] = ACTIONS(1565),
    [sym_duet_dev_type_relay_device] = ACTIONS(1565),
    [sym_duet_dev_type_ups] = ACTIONS(1565),
    [sym_duet_dev_type_amplifier] = ACTIONS(1565),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1565),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1565),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1565),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1565),
    [sym_duet_dev_type_camera] = ACTIONS(1565),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1565),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1565),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1565),
    [sym_duet_dev_type_dss] = ACTIONS(1565),
    [sym_duet_dev_type_dvr] = ACTIONS(1565),
    [sym_duet_dev_type_disc_device] = ACTIONS(1565),
    [sym_duet_dev_type_document_camera] = ACTIONS(1565),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1565),
    [sym_duet_dev_type_hvac] = ACTIONS(1565),
    [sym_duet_dev_type_keypad] = ACTIONS(1565),
    [sym_duet_dev_type_light] = ACTIONS(1565),
    [sym_duet_dev_type_monitor] = ACTIONS(1565),
    [sym_duet_dev_type_motor] = ACTIONS(1565),
    [sym_duet_dev_type_multi_window] = ACTIONS(1565),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1565),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1565),
    [sym_duet_dev_type_receiver] = ACTIONS(1565),
    [sym_duet_dev_type_security_system] = ACTIONS(1565),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1565),
    [sym_duet_dev_type_settop_box] = ACTIONS(1565),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1565),
    [sym_duet_dev_type_switcher] = ACTIONS(1565),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1565),
    [sym_duet_dev_type_tv] = ACTIONS(1565),
    [sym_duet_dev_type_utility] = ACTIONS(1565),
    [sym_duet_dev_type_vcr] = ACTIONS(1565),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1565),
    [sym_duet_dev_type_video_processor] = ACTIONS(1565),
    [sym_duet_dev_type_video_projector] = ACTIONS(1565),
    [sym_duet_dev_type_video_wall] = ACTIONS(1565),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1565),
    [sym_duet_dev_type_weather] = ACTIONS(1565),
    [sym_duet_dev_not_polled] = ACTIONS(1565),
    [sym_duet_dev_polled] = ACTIONS(1565),
    [sym_clkmgr_mode_network] = ACTIONS(1565),
    [sym_clkmgr_mode_standalone] = ACTIONS(1565),
    [sym_normal_standby] = ACTIONS(1565),
    [sym_normal_wake] = ACTIONS(1565),
    [sym_ip_multicast_ttl_option] = ACTIONS(1565),
    [sym_ip_ttl_subnet] = ACTIONS(1565),
    [sym_ip_ttl_site] = ACTIONS(1565),
    [sym_ip_ttl_region] = ACTIONS(1565),
    [sym_ip_ttl_continent] = ACTIONS(1565),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1565),
    [sym_ip_nodelay_on] = ACTIONS(1565),
    [sym_ip_nodelay_off] = ACTIONS(1565),
    [sym_amx_error] = ACTIONS(1565),
    [sym_amx_warning] = ACTIONS(1565),
    [sym_amx_info] = ACTIONS(1565),
    [sym_amx_debug] = ACTIONS(1565),
    [sym_netlinx_login_success] = ACTIONS(1565),
    [sym_netlinx_login_fail] = ACTIONS(1565),
    [sym_netlinx_logout] = ACTIONS(1565),
    [sym_null_str] = ACTIONS(1565),
    [sym_smtp_address] = ACTIONS(1565),
    [sym_smtp_port_number] = ACTIONS(1565),
    [sym_smtp_username] = ACTIONS(1565),
    [sym_smtp_password] = ACTIONS(1565),
    [sym_smtp_require_tls] = ACTIONS(1565),
    [sym_smtp_from] = ACTIONS(1565),
    [sym_smtp_tls_true] = ACTIONS(1565),
    [sym_smtp_tls_false] = ACTIONS(1565),
    [sym_listview_on_row_select_event] = ACTIONS(1565),
    [sym_data_structure_unknown] = ACTIONS(1565),
    [sym_data_structure_datafeed] = ACTIONS(1565),
    [sym_data_structure_datarecordset] = ACTIONS(1565),
    [sym_data_structure_datarecord] = ACTIONS(1565),
    [sym_data_structure_datafield] = ACTIONS(1565),
    [sym_data_max_content_field_count] = ACTIONS(1565),
    [sym_data_max_metadata_field_count] = ACTIONS(1565),
    [sym_data_max_id_length] = ACTIONS(1565),
    [sym_data_max_name_length] = ACTIONS(1565),
    [sym_data_max_description_length] = ACTIONS(1565),
    [sym_data_max_type_length] = ACTIONS(1565),
    [sym_data_max_format_length] = ACTIONS(1565),
    [sym_data_max_source_length] = ACTIONS(1565),
    [sym_data_max_value_length] = ACTIONS(1565),
    [sym_data_max_label_length] = ACTIONS(1565),
    [sym_data_max_url_length] = ACTIONS(1565),
    [sym_max_last_login_info_length] = ACTIONS(1565),
    [sym_data_type_unknown] = ACTIONS(1565),
    [sym_data_type_string] = ACTIONS(1565),
    [sym_data_type_datetime] = ACTIONS(1565),
    [sym_data_type_date] = ACTIONS(1565),
    [sym_data_type_time] = ACTIONS(1565),
    [sym_data_type_image] = ACTIONS(1565),
    [sym_data_format_url] = ACTIONS(1565),
    [sym_data_format_phone] = ACTIONS(1565),
    [sym_data_format_email] = ACTIONS(1565),
    [sym_data_format_iso8601] = ACTIONS(1565),
    [sym_auditlog_priv] = ACTIONS(1565),
    [sym_configuration_priv] = ACTIONS(1565),
    [sym_deviceconfig_priv] = ACTIONS(1565),
    [sym_ftp_priv] = ACTIONS(1565),
    [sym_http_priv] = ACTIONS(1565),
    [sym_networkconfig_priv] = ACTIONS(1565),
    [sym_programport_priv] = ACTIONS(1565),
    [sym_remoteui_priv] = ACTIONS(1565),
    [sym_securitycontrol_priv] = ACTIONS(1565),
    [sym_softwaremanagement_priv] = ACTIONS(1565),
    [sym_terminal_priv] = ACTIONS(1565),
    [sym_tpadmin_priv] = ACTIONS(1565),
    [sym_user1_priv] = ACTIONS(1565),
    [sym_user2_priv] = ACTIONS(1565),
    [sym_user3_priv] = ACTIONS(1565),
    [sym_user4_priv] = ACTIONS(1565),
    [sym_usermanagement_priv] = ACTIONS(1565),
    [sym_valid_account] = ACTIONS(1565),
    [sym_err_name_invalid] = ACTIONS(1565),
    [sym_err_password_invalid] = ACTIONS(1565),
    [sym_err_invalid_account] = ACTIONS(1565),
    [sym_err_no_authorization] = ACTIONS(1565),
    [sym_err_invalid_parameter] = ACTIONS(1565),
    [sym_err_user_locked_out] = ACTIONS(1565),
    [sym_err_user_account_expired] = ACTIONS(1565),
    [sym_err_general_error] = ACTIONS(1565),
    [sym_err_authentication_error] = ACTIONS(1565),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1565),
    [sym_tls_validate_certificate] = ACTIONS(1565),
    [sym_tdata] = ACTIONS(1565),
    [sym_tchannel] = ACTIONS(1565),
    [sym_tlevel] = ACTIONS(1565),
    [sym_tbutton] = ACTIONS(1565),
    [sym_ttimeline] = ACTIONS(1565),
    [sym_tcustom] = ACTIONS(1565),
    [sym_url_struct] = ACTIONS(1565),
    [sym_dns_struct] = ACTIONS(1565),
    [sym_ip_address_struct] = ACTIONS(1565),
    [sym_dev_info_struct] = ACTIONS(1565),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1565),
    [sym_data_feed] = ACTIONS(1565),
    [sym_data_field] = ACTIONS(1565),
    [sym_data_record] = ACTIONS(1565),
    [sym_wc_data_feed] = ACTIONS(1565),
    [sym_wc_data_field] = ACTIONS(1565),
    [sym_wc_data_record] = ACTIONS(1565),
    [sym_last_login_info] = ACTIONS(1565),
    [sym____reserved___] = ACTIONS(1565),
    [sym_date] = ACTIONS(1565),
    [sym_day] = ACTIONS(1565),
    [sym_time] = ACTIONS(1565),
    [sym_ldate] = ACTIONS(1565),
    [sym_push_device] = ACTIONS(1565),
    [sym_push_channel] = ACTIONS(1565),
    [sym_push_devchan] = ACTIONS(1565),
    [sym_release_device] = ACTIONS(1565),
    [sym_release_channel] = ACTIONS(1565),
    [sym_release_devchan] = ACTIONS(1565),
    [sym_master_slot] = ACTIONS(1565),
    [sym_get_pulse_time] = ACTIONS(1565),
    [sym_get_timer] = ACTIONS(1565),
    [sym_master_sn] = ACTIONS(1565),
    [sym_system_number] = ACTIONS(1565),
    [sym_button] = ACTIONS(1565),
    [sym_dv_channel] = ACTIONS(1565),
    [sym_data] = ACTIONS(1565),
    [sym_level] = ACTIONS(1565),
    [sym_timeline] = ACTIONS(1565),
    [sym_channel] = ACTIONS(1565),
    [sym_custom] = ACTIONS(1565),
    [sym_atoi] = ACTIONS(1565),
    [sym_atol] = ACTIONS(1565),
    [sym_atof] = ACTIONS(1565),
    [sym_itoa] = ACTIONS(1565),
    [sym_itohex] = ACTIONS(1565),
    [sym_hextoi] = ACTIONS(1565),
    [sym_ftoa] = ACTIONS(1565),
    [sym_find_string] = ACTIONS(1565),
    [sym_left_string] = ACTIONS(1565),
    [sym_right_string] = ACTIONS(1565),
    [sym_mid_string] = ACTIONS(1565),
    [sym_remove_string] = ACTIONS(1565),
    [sym_get_buffer_string] = ACTIONS(1565),
    [sym_get_buffer_char] = ACTIONS(1565),
    [sym_get_multi_buffer_string] = ACTIONS(1565),
    [sym_length_string] = ACTIONS(1565),
    [sym_max_length_string] = ACTIONS(1565),
    [sym_set_length_string] = ACTIONS(1565),
    [sym_compare_string] = ACTIONS(1565),
    [sym_upper_string] = ACTIONS(1565),
    [sym_lower_string] = ACTIONS(1565),
    [sym_get_last] = ACTIONS(1565),
    [sym_date_to_day] = ACTIONS(1565),
    [sym_date_to_month] = ACTIONS(1565),
    [sym_date_to_year] = ACTIONS(1565),
    [sym_time_to_hour] = ACTIONS(1565),
    [sym_time_to_minute] = ACTIONS(1565),
    [sym_time_to_second] = ACTIONS(1565),
    [sym_day_of_week] = ACTIONS(1565),
    [sym_set_pulse_time] = ACTIONS(1565),
    [sym_set_timer] = ACTIONS(1565),
    [sym_file_open] = ACTIONS(1565),
    [sym_file_close] = ACTIONS(1565),
    [sym_file_read] = ACTIONS(1565),
    [sym_file_read_line] = ACTIONS(1565),
    [sym_file_write] = ACTIONS(1565),
    [sym_file_write_line] = ACTIONS(1565),
    [sym_file_copy] = ACTIONS(1565),
    [sym_file_delete] = ACTIONS(1565),
    [sym_file_rename] = ACTIONS(1565),
    [sym_file_dir] = ACTIONS(1565),
    [sym_file_getdir] = ACTIONS(1565),
    [sym_file_setdir] = ACTIONS(1565),
    [sym_file_create_dir] = ACTIONS(1565),
    [sym_file_remove_dir] = ACTIONS(1565),
    [sym_file_seek] = ACTIONS(1565),
    [sym_length_array] = ACTIONS(1565),
    [sym_max_length_array] = ACTIONS(1565),
    [sym_set_length_array] = ACTIONS(1565),
    [sym_ip_client_open] = ACTIONS(1565),
    [sym_ip_client_close] = ACTIONS(1565),
    [sym_ip_server_open] = ACTIONS(1565),
    [sym_ip_server_close] = ACTIONS(1565),
    [sym_ip_bound_client_open] = ACTIONS(1565),
    [sym_ip_mc_server_open] = ACTIONS(1565),
    [sym_ip_set_option] = ACTIONS(1565),
    [sym_ssh_client_open] = ACTIONS(1565),
    [sym_ssh_client_close] = ACTIONS(1565),
    [sym_tls_client_open] = ACTIONS(1565),
    [sym_tls_client_close] = ACTIONS(1565),
    [sym_do_push] = ACTIONS(1565),
    [sym_do_push_timed] = ACTIONS(1565),
    [sym_do_release] = ACTIONS(1565),
    [sym_device_id] = ACTIONS(1565),
    [sym_device_info] = ACTIONS(1565),
    [sym_device_id_string] = ACTIONS(1565),
    [sym_device_standby] = ACTIONS(1565),
    [sym_device_wake] = ACTIONS(1565),
    [sym_timeline_create] = ACTIONS(1565),
    [sym_timeline_kill] = ACTIONS(1565),
    [sym_timeline_reload] = ACTIONS(1565),
    [sym_timeline_pause] = ACTIONS(1565),
    [sym_timeline_restart] = ACTIONS(1565),
    [sym_timeline_set] = ACTIONS(1565),
    [sym_timeline_get] = ACTIONS(1565),
    [sym_timeline_active] = ACTIONS(1565),
    [sym_timeline_dynamic_id] = ACTIONS(1565),
    [sym_variable_to_string] = ACTIONS(1565),
    [sym_string_to_variable] = ACTIONS(1565),
    [sym_length_variable_to_string] = ACTIONS(1565),
    [sym_type_cast] = ACTIONS(1565),
    [sym_variable_to_xml] = ACTIONS(1565),
    [sym_xml_to_variable] = ACTIONS(1565),
    [sym_length_variable_to_xml] = ACTIONS(1565),
    [sym_redirect_string] = ACTIONS(1565),
    [sym_random_number] = ACTIONS(1565),
    [sym_get_unique_id] = ACTIONS(1565),
    [sym_get_system_number] = ACTIONS(1565),
    [sym_set_system_number] = ACTIONS(1565),
    [sym_get_serial_number] = ACTIONS(1565),
    [sym_get_master_build] = ACTIONS(1565),
    [sym_set_validation_code] = ACTIONS(1565),
    [sym_reboot] = ACTIONS(1565),
    [sym_load_duet_module] = ACTIONS(1565),
    [sym_unload_duet_module] = ACTIONS(1565),
    [sym_duet_mem_size_set] = ACTIONS(1565),
    [sym_duet_mem_size_get] = ACTIONS(1565),
    [sym_dynamic_application_device] = ACTIONS(1565),
    [sym_dynamic_polled_port] = ACTIONS(1565),
    [sym_static_port_binding] = ACTIONS(1565),
    [sym_static_ip_binding] = ACTIONS(1565),
    [sym_get_dns_list] = ACTIONS(1565),
    [sym_set_dns_list] = ACTIONS(1565),
    [sym_get_ip_address] = ACTIONS(1565),
    [sym_set_ip_address] = ACTIONS(1565),
    [sym_get_url_list] = ACTIONS(1565),
    [sym_add_url_entry] = ACTIONS(1565),
    [sym_delete_url_entry] = ACTIONS(1565),
    [sym_abs_value] = ACTIONS(1565),
    [sym_max_value] = ACTIONS(1565),
    [sym_min_value] = ACTIONS(1565),
    [sym_exp_value] = ACTIONS(1565),
    [sym_log_value] = ACTIONS(1565),
    [sym_log10_value] = ACTIONS(1565),
    [sym_power_value] = ACTIONS(1565),
    [sym_sqrt_value] = ACTIONS(1565),
    [sym_raw_be] = ACTIONS(1565),
    [sym_raw_le] = ACTIONS(1565),
    [sym_internal_queue_size_set] = ACTIONS(1565),
    [sym_internal_queue_size_get] = ACTIONS(1565),
    [sym_internal_threshold_set] = ACTIONS(1565),
    [sym_internal_threshold_get] = ACTIONS(1565),
    [sym_rebuilt_event] = ACTIONS(1565),
    [sym_do_custom_event] = ACTIONS(1565),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1565),
    [sym_clkmgr_set_clk_source] = ACTIONS(1565),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1565),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1565),
    [sym_clkmgr_get_timezone] = ACTIONS(1565),
    [sym_clkmgr_set_timezone] = ACTIONS(1565),
    [sym_clkmgr_get_resync_period] = ACTIONS(1565),
    [sym_clkmgr_set_resync_period] = ACTIONS(1565),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1565),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1565),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1565),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1565),
    [sym_clkmgr_get_timeservers] = ACTIONS(1565),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1565),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1565),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1565),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1565),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1565),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1565),
    [sym_set_log_level] = ACTIONS(1565),
    [sym_get_log_level] = ACTIONS(1565),
    [sym_amx_log] = ACTIONS(1565),
    [sym_smtp_server_config_set] = ACTIONS(1565),
    [sym_smtp_server_config_get] = ACTIONS(1565),
    [sym_smtp_send] = ACTIONS(1565),
    [sym_validate_netlinx_account] = ACTIONS(1565),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1565),
    [sym_authenticate_certificate] = ACTIONS(1565),
    [sym_audit_netlinx_session_event] = ACTIONS(1565),
    [sym_audit_netlinx_generic_event] = ACTIONS(1565),
    [sym_get_available_flash_disk_space] = ACTIONS(1565),
    [sym_get_max_flash_disk_space] = ACTIONS(1565),
    [sym_data_create_feed] = ACTIONS(1565),
    [sym_data_delete_feed] = ACTIONS(1565),
    [sym_data_publish_feed] = ACTIONS(1565),
    [sym_data_get_published_feed] = ACTIONS(1565),
    [sym_data_add_record] = ACTIONS(1565),
    [sym_data_get_event_record] = ACTIONS(1565),
    [sym__wc_data_create_feed] = ACTIONS(1565),
    [sym__wc_data_add_record] = ACTIONS(1565),
    [sym__wc_data_get_event_record] = ACTIONS(1565),
    [sym_format] = ACTIONS(1565),
    [sym_astro_clock] = ACTIONS(1565),
    [sym_set_outdoor_temperature] = ACTIONS(1565),
    [sym_set_virtual_level_count] = ACTIONS(1565),
    [sym_set_virtual_channel_count] = ACTIONS(1565),
    [sym_set_virtual_port_count] = ACTIONS(1565),
    [sym__wc_encode_int] = ACTIONS(1565),
    [sym__wc_decode_int] = ACTIONS(1565),
    [sym__wc_int] = ACTIONS(1565),
    [sym__wc_to_ch_int] = ACTIONS(1565),
    [sym__ch_to_wc_int] = ACTIONS(1565),
    [sym__wc_find_string_int] = ACTIONS(1565),
    [sym__wc_left_string_int] = ACTIONS(1565),
    [sym__wc_lower_string_int] = ACTIONS(1565),
    [sym__wc_mid_string_int] = ACTIONS(1565),
    [sym__wc_remove_string_int] = ACTIONS(1565),
    [sym__wc_right_string_int] = ACTIONS(1565),
    [sym__wc_upper_string_int] = ACTIONS(1565),
    [sym__wc_compare_string_int] = ACTIONS(1565),
    [sym__wc_get_buffer_char_int] = ACTIONS(1565),
    [sym__wc_get_buffer_string_int] = ACTIONS(1565),
    [sym__wc_concat_string_int] = ACTIONS(1565),
    [sym__wc_file_open_int] = ACTIONS(1565),
    [sym__wc_file_close_int] = ACTIONS(1565),
    [sym__wc_file_read_int] = ACTIONS(1565),
    [sym__wc_file_read_line_int] = ACTIONS(1565),
    [sym__wc_file_write_int] = ACTIONS(1565),
    [sym__wc_file_write_line_int] = ACTIONS(1565),
    [aux_sym_preproc_include_token1] = ACTIONS(1565),
    [aux_sym_preproc_define_token1] = ACTIONS(1567),
    [aux_sym_preproc_warn_token1] = ACTIONS(1567),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1567),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1567),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1567),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1567),
    [aux_sym_preproc_else_token1] = ACTIONS(1567),
    [anon_sym_LBRACK] = ACTIONS(1567),
    [anon_sym_LBRACE] = ACTIONS(1567),
    [anon_sym_COLON] = ACTIONS(1567),
    [anon_sym_LPAREN] = ACTIONS(1565),
    [anon_sym_SEMI] = ACTIONS(1567),
    [anon_sym_BANG] = ACTIONS(1565),
    [anon_sym_TILDE] = ACTIONS(1567),
    [anon_sym_DASH] = ACTIONS(1565),
    [anon_sym_PLUS] = ACTIONS(1565),
    [anon_sym_STAR] = ACTIONS(1567),
    [anon_sym_SLASH] = ACTIONS(1565),
    [anon_sym_PERCENT] = ACTIONS(1567),
    [anon_sym_PIPE_PIPE] = ACTIONS(1567),
    [anon_sym_AMP_AMP] = ACTIONS(1567),
    [anon_sym_PIPE] = ACTIONS(1565),
    [anon_sym_CARET] = ACTIONS(1567),
    [anon_sym_AMP] = ACTIONS(1565),
    [anon_sym_EQ_EQ] = ACTIONS(1567),
    [anon_sym_BANG_EQ] = ACTIONS(1567),
    [anon_sym_LT_GT] = ACTIONS(1567),
    [anon_sym_GT] = ACTIONS(1565),
    [anon_sym_GT_EQ] = ACTIONS(1567),
    [anon_sym_LT_EQ] = ACTIONS(1567),
    [anon_sym_LT] = ACTIONS(1565),
    [anon_sym_LT_LT] = ACTIONS(1567),
    [anon_sym_GT_GT] = ACTIONS(1567),
    [anon_sym_DASH_DASH] = ACTIONS(1567),
    [anon_sym_PLUS_PLUS] = ACTIONS(1567),
    [anon_sym_DOT] = ACTIONS(1565),
    [anon_sym_DQUOTE] = ACTIONS(1567),
    [anon_sym_SQUOTE] = ACTIONS(1567),
    [sym_number_literal] = ACTIONS(1567),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1567),
  },
  [STATE(121)] = {
    [ts_builtin_sym_end] = ACTIONS(1533),
    [sym_identifier] = ACTIONS(1531),
    [sym_program_name_keyword] = ACTIONS(1531),
    [sym_module_name_keyword] = ACTIONS(1531),
    [sym_define_device_keyword] = ACTIONS(1531),
    [sym_define_combine_keyword] = ACTIONS(1531),
    [sym_define_constant_keyword] = ACTIONS(1531),
    [sym_define_type_keyword] = ACTIONS(1531),
    [sym_define_variable_keyword] = ACTIONS(1531),
    [sym_define_system_variable_keyword] = ACTIONS(1531),
    [sym_define_connect_level_keyword] = ACTIONS(1531),
    [sym_define_latching_keyword] = ACTIONS(1531),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1531),
    [sym_define_toggling_keyword] = ACTIONS(1531),
    [sym_define_call_keyword] = ACTIONS(1531),
    [sym_define_function_keyword] = ACTIONS(1531),
    [sym_define_library_function_keyword] = ACTIONS(1531),
    [sym_define_start_keyword] = ACTIONS(1531),
    [sym_define_event_keyword] = ACTIONS(1531),
    [sym_define_program_keyword] = ACTIONS(1531),
    [sym_define_module_keyword] = ACTIONS(1531),
    [sym_char_keyword] = ACTIONS(1531),
    [sym_widechar_keyword] = ACTIONS(1531),
    [sym_integer_keyword] = ACTIONS(1531),
    [sym_sinteger_keyword] = ACTIONS(1531),
    [sym_long_keyword] = ACTIONS(1531),
    [sym_slong_keyword] = ACTIONS(1531),
    [sym_float_keyword] = ACTIONS(1531),
    [sym_double_keyword] = ACTIONS(1531),
    [sym_variant_keyword] = ACTIONS(1531),
    [sym_variantarray_keyword] = ACTIONS(1531),
    [sym_local_var_keyword] = ACTIONS(1531),
    [sym_stack_var_keyword] = ACTIONS(1531),
    [sym_constant_keyword] = ACTIONS(1531),
    [sym_volatile_keyword] = ACTIONS(1531),
    [sym_non_volatile_keyword] = ACTIONS(1531),
    [sym_persistent_keyword] = ACTIONS(1531),
    [sym_dev_keyword] = ACTIONS(1531),
    [sym_devlev_keyword] = ACTIONS(1531),
    [sym_devchan_keyword] = ACTIONS(1531),
    [sym_if_keyword] = ACTIONS(1531),
    [sym_else_keyword] = ACTIONS(1531),
    [sym_switch_keyword] = ACTIONS(1531),
    [sym_case_keyword] = ACTIONS(1531),
    [sym_default_keyword] = ACTIONS(1531),
    [sym_while_keyword] = ACTIONS(1531),
    [sym_for_keyword] = ACTIONS(1531),
    [sym_break_keyword] = ACTIONS(1531),
    [sym_continue_keyword] = ACTIONS(1531),
    [sym_return_keyword] = ACTIONS(1531),
    [sym_select_keyword] = ACTIONS(1531),
    [sym_active_keyword] = ACTIONS(1531),
    [sym_struct_keyword] = ACTIONS(1531),
    [sym_structure_keyword] = ACTIONS(1531),
    [sym_button_event_keyword] = ACTIONS(1531),
    [sym_channel_event_keyword] = ACTIONS(1531),
    [sym_level_event_keyword] = ACTIONS(1531),
    [sym_data_event_keyword] = ACTIONS(1531),
    [sym_timeline_event_keyword] = ACTIONS(1531),
    [sym_custom_event_keyword] = ACTIONS(1531),
    [sym_push_keyword] = ACTIONS(1531),
    [sym_release_keyword] = ACTIONS(1531),
    [aux_sym_on_keyword_token1] = ACTIONS(1531),
    [aux_sym_off_keyword_token1] = ACTIONS(1531),
    [sym_send_level_keyword] = ACTIONS(1531),
    [sym_send_string_keyword] = ACTIONS(1531),
    [sym_send_command_keyword] = ACTIONS(1531),
    [sym_clear_buffer_keyword] = ACTIONS(1531),
    [sym_create_buffer_keyword] = ACTIONS(1531),
    [sym_create_multi_buffer_keyword] = ACTIONS(1531),
    [sym_call_keyword] = ACTIONS(1531),
    [sym_system_call_keyword] = ACTIONS(1531),
    [sym_devchan_to_keyword] = ACTIONS(1531),
    [sym_devchan_min_to_keyword] = ACTIONS(1531),
    [sym_devchan_total_off_keyword] = ACTIONS(1531),
    [sym_devchan_pulse_keyword] = ACTIONS(1531),
    [sym_wait_keyword] = ACTIONS(1531),
    [sym_cancel_wait_keyword] = ACTIONS(1531),
    [sym_wait_until_keyword] = ACTIONS(1531),
    [sym_cancel_wait_until_keyword] = ACTIONS(1531),
    [sym_cancel_all_wait_keyword] = ACTIONS(1531),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1531),
    [sym_and] = ACTIONS(1531),
    [sym_or] = ACTIONS(1531),
    [sym_not] = ACTIONS(1531),
    [sym_xor] = ACTIONS(1531),
    [sym_band] = ACTIONS(1531),
    [sym_bor] = ACTIONS(1531),
    [sym_bxor] = ACTIONS(1531),
    [sym_bnot] = ACTIONS(1531),
    [sym_lshift] = ACTIONS(1531),
    [sym_rshift] = ACTIONS(1531),
    [sym___file__] = ACTIONS(1531),
    [sym___date__] = ACTIONS(1531),
    [sym___ldate__] = ACTIONS(1531),
    [sym___time__] = ACTIONS(1531),
    [sym___line__] = ACTIONS(1531),
    [sym___name__] = ACTIONS(1531),
    [sym___netlinx__] = ACTIONS(1531),
    [sym_netlinx_axi_version] = ACTIONS(1531),
    [sym_url_flg_tcp] = ACTIONS(1531),
    [sym_url_flg_accinfopresent] = ACTIONS(1531),
    [sym_url_flg_temp] = ACTIONS(1531),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1531),
    [sym_url_flg_stat_mask] = ACTIONS(1531),
    [sym_url_flg_stat_lookup] = ACTIONS(1531),
    [sym_url_flg_stat_connecting] = ACTIONS(1531),
    [sym_url_flg_stat_waiting] = ACTIONS(1531),
    [sym_url_flg_stat_connected] = ACTIONS(1531),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1531),
    [sym_first_virtual_device] = ACTIONS(1531),
    [sym_first_local_port] = ACTIONS(1531),
    [sym_dynamic_virtual_device] = ACTIONS(1531),
    [sym_timeline_once] = ACTIONS(1531),
    [sym_timeline_repeat] = ACTIONS(1531),
    [sym_timeline_absolute] = ACTIONS(1531),
    [sym_timeline_relative] = ACTIONS(1531),
    [sym_do_push_timed_infinite] = ACTIONS(1531),
    [sym_source_type_no_address] = ACTIONS(1531),
    [sym_source_type_neuron_id] = ACTIONS(1531),
    [sym_source_type_ip_address] = ACTIONS(1531),
    [sym_source_type_axlink] = ACTIONS(1531),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1531),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1531),
    [sym_source_type_ip_socket_address] = ACTIONS(1531),
    [sym_source_type_rs232] = ACTIONS(1531),
    [sym_source_type_internal] = ACTIONS(1531),
    [sym_source_type_mac_address] = ACTIONS(1531),
    [sym_source_type_ipv4_port] = ACTIONS(1531),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1531),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1531),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1531),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1531),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1531),
    [sym_true] = ACTIONS(1531),
    [sym_false] = ACTIONS(1531),
    [sym_file_read_only] = ACTIONS(1531),
    [sym_file_rw_new] = ACTIONS(1531),
    [sym_file_rw_append] = ACTIONS(1531),
    [sym_ip_tcp] = ACTIONS(1531),
    [sym_ip_udp] = ACTIONS(1531),
    [sym_ip_udp_2way] = ACTIONS(1531),
    [sym_xml_encode_types] = ACTIONS(1531),
    [sym_xml_encode_char_as_list] = ACTIONS(1531),
    [sym_xml_encode_le] = ACTIONS(1531),
    [sym_xml_decode_types] = ACTIONS(1531),
    [sym_xml_decode_no_preserve] = ACTIONS(1531),
    [sym_duet_dev_type_display_device] = ACTIONS(1531),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1531),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1531),
    [sym_duet_dev_type_io_device] = ACTIONS(1531),
    [sym_duet_dev_type_relay_device] = ACTIONS(1531),
    [sym_duet_dev_type_ups] = ACTIONS(1531),
    [sym_duet_dev_type_amplifier] = ACTIONS(1531),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1531),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1531),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1531),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1531),
    [sym_duet_dev_type_camera] = ACTIONS(1531),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1531),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1531),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1531),
    [sym_duet_dev_type_dss] = ACTIONS(1531),
    [sym_duet_dev_type_dvr] = ACTIONS(1531),
    [sym_duet_dev_type_disc_device] = ACTIONS(1531),
    [sym_duet_dev_type_document_camera] = ACTIONS(1531),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1531),
    [sym_duet_dev_type_hvac] = ACTIONS(1531),
    [sym_duet_dev_type_keypad] = ACTIONS(1531),
    [sym_duet_dev_type_light] = ACTIONS(1531),
    [sym_duet_dev_type_monitor] = ACTIONS(1531),
    [sym_duet_dev_type_motor] = ACTIONS(1531),
    [sym_duet_dev_type_multi_window] = ACTIONS(1531),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1531),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1531),
    [sym_duet_dev_type_receiver] = ACTIONS(1531),
    [sym_duet_dev_type_security_system] = ACTIONS(1531),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1531),
    [sym_duet_dev_type_settop_box] = ACTIONS(1531),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1531),
    [sym_duet_dev_type_switcher] = ACTIONS(1531),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1531),
    [sym_duet_dev_type_tv] = ACTIONS(1531),
    [sym_duet_dev_type_utility] = ACTIONS(1531),
    [sym_duet_dev_type_vcr] = ACTIONS(1531),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1531),
    [sym_duet_dev_type_video_processor] = ACTIONS(1531),
    [sym_duet_dev_type_video_projector] = ACTIONS(1531),
    [sym_duet_dev_type_video_wall] = ACTIONS(1531),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1531),
    [sym_duet_dev_type_weather] = ACTIONS(1531),
    [sym_duet_dev_not_polled] = ACTIONS(1531),
    [sym_duet_dev_polled] = ACTIONS(1531),
    [sym_clkmgr_mode_network] = ACTIONS(1531),
    [sym_clkmgr_mode_standalone] = ACTIONS(1531),
    [sym_normal_standby] = ACTIONS(1531),
    [sym_normal_wake] = ACTIONS(1531),
    [sym_ip_multicast_ttl_option] = ACTIONS(1531),
    [sym_ip_ttl_subnet] = ACTIONS(1531),
    [sym_ip_ttl_site] = ACTIONS(1531),
    [sym_ip_ttl_region] = ACTIONS(1531),
    [sym_ip_ttl_continent] = ACTIONS(1531),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1531),
    [sym_ip_nodelay_on] = ACTIONS(1531),
    [sym_ip_nodelay_off] = ACTIONS(1531),
    [sym_amx_error] = ACTIONS(1531),
    [sym_amx_warning] = ACTIONS(1531),
    [sym_amx_info] = ACTIONS(1531),
    [sym_amx_debug] = ACTIONS(1531),
    [sym_netlinx_login_success] = ACTIONS(1531),
    [sym_netlinx_login_fail] = ACTIONS(1531),
    [sym_netlinx_logout] = ACTIONS(1531),
    [sym_null_str] = ACTIONS(1531),
    [sym_smtp_address] = ACTIONS(1531),
    [sym_smtp_port_number] = ACTIONS(1531),
    [sym_smtp_username] = ACTIONS(1531),
    [sym_smtp_password] = ACTIONS(1531),
    [sym_smtp_require_tls] = ACTIONS(1531),
    [sym_smtp_from] = ACTIONS(1531),
    [sym_smtp_tls_true] = ACTIONS(1531),
    [sym_smtp_tls_false] = ACTIONS(1531),
    [sym_listview_on_row_select_event] = ACTIONS(1531),
    [sym_data_structure_unknown] = ACTIONS(1531),
    [sym_data_structure_datafeed] = ACTIONS(1531),
    [sym_data_structure_datarecordset] = ACTIONS(1531),
    [sym_data_structure_datarecord] = ACTIONS(1531),
    [sym_data_structure_datafield] = ACTIONS(1531),
    [sym_data_max_content_field_count] = ACTIONS(1531),
    [sym_data_max_metadata_field_count] = ACTIONS(1531),
    [sym_data_max_id_length] = ACTIONS(1531),
    [sym_data_max_name_length] = ACTIONS(1531),
    [sym_data_max_description_length] = ACTIONS(1531),
    [sym_data_max_type_length] = ACTIONS(1531),
    [sym_data_max_format_length] = ACTIONS(1531),
    [sym_data_max_source_length] = ACTIONS(1531),
    [sym_data_max_value_length] = ACTIONS(1531),
    [sym_data_max_label_length] = ACTIONS(1531),
    [sym_data_max_url_length] = ACTIONS(1531),
    [sym_max_last_login_info_length] = ACTIONS(1531),
    [sym_data_type_unknown] = ACTIONS(1531),
    [sym_data_type_string] = ACTIONS(1531),
    [sym_data_type_datetime] = ACTIONS(1531),
    [sym_data_type_date] = ACTIONS(1531),
    [sym_data_type_time] = ACTIONS(1531),
    [sym_data_type_image] = ACTIONS(1531),
    [sym_data_format_url] = ACTIONS(1531),
    [sym_data_format_phone] = ACTIONS(1531),
    [sym_data_format_email] = ACTIONS(1531),
    [sym_data_format_iso8601] = ACTIONS(1531),
    [sym_auditlog_priv] = ACTIONS(1531),
    [sym_configuration_priv] = ACTIONS(1531),
    [sym_deviceconfig_priv] = ACTIONS(1531),
    [sym_ftp_priv] = ACTIONS(1531),
    [sym_http_priv] = ACTIONS(1531),
    [sym_networkconfig_priv] = ACTIONS(1531),
    [sym_programport_priv] = ACTIONS(1531),
    [sym_remoteui_priv] = ACTIONS(1531),
    [sym_securitycontrol_priv] = ACTIONS(1531),
    [sym_softwaremanagement_priv] = ACTIONS(1531),
    [sym_terminal_priv] = ACTIONS(1531),
    [sym_tpadmin_priv] = ACTIONS(1531),
    [sym_user1_priv] = ACTIONS(1531),
    [sym_user2_priv] = ACTIONS(1531),
    [sym_user3_priv] = ACTIONS(1531),
    [sym_user4_priv] = ACTIONS(1531),
    [sym_usermanagement_priv] = ACTIONS(1531),
    [sym_valid_account] = ACTIONS(1531),
    [sym_err_name_invalid] = ACTIONS(1531),
    [sym_err_password_invalid] = ACTIONS(1531),
    [sym_err_invalid_account] = ACTIONS(1531),
    [sym_err_no_authorization] = ACTIONS(1531),
    [sym_err_invalid_parameter] = ACTIONS(1531),
    [sym_err_user_locked_out] = ACTIONS(1531),
    [sym_err_user_account_expired] = ACTIONS(1531),
    [sym_err_general_error] = ACTIONS(1531),
    [sym_err_authentication_error] = ACTIONS(1531),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1531),
    [sym_tls_validate_certificate] = ACTIONS(1531),
    [sym_tdata] = ACTIONS(1531),
    [sym_tchannel] = ACTIONS(1531),
    [sym_tlevel] = ACTIONS(1531),
    [sym_tbutton] = ACTIONS(1531),
    [sym_ttimeline] = ACTIONS(1531),
    [sym_tcustom] = ACTIONS(1531),
    [sym_url_struct] = ACTIONS(1531),
    [sym_dns_struct] = ACTIONS(1531),
    [sym_ip_address_struct] = ACTIONS(1531),
    [sym_dev_info_struct] = ACTIONS(1531),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1531),
    [sym_data_feed] = ACTIONS(1531),
    [sym_data_field] = ACTIONS(1531),
    [sym_data_record] = ACTIONS(1531),
    [sym_wc_data_feed] = ACTIONS(1531),
    [sym_wc_data_field] = ACTIONS(1531),
    [sym_wc_data_record] = ACTIONS(1531),
    [sym_last_login_info] = ACTIONS(1531),
    [sym____reserved___] = ACTIONS(1531),
    [sym_date] = ACTIONS(1531),
    [sym_day] = ACTIONS(1531),
    [sym_time] = ACTIONS(1531),
    [sym_ldate] = ACTIONS(1531),
    [sym_push_device] = ACTIONS(1531),
    [sym_push_channel] = ACTIONS(1531),
    [sym_push_devchan] = ACTIONS(1531),
    [sym_release_device] = ACTIONS(1531),
    [sym_release_channel] = ACTIONS(1531),
    [sym_release_devchan] = ACTIONS(1531),
    [sym_master_slot] = ACTIONS(1531),
    [sym_get_pulse_time] = ACTIONS(1531),
    [sym_get_timer] = ACTIONS(1531),
    [sym_master_sn] = ACTIONS(1531),
    [sym_system_number] = ACTIONS(1531),
    [sym_button] = ACTIONS(1531),
    [sym_dv_channel] = ACTIONS(1531),
    [sym_data] = ACTIONS(1531),
    [sym_level] = ACTIONS(1531),
    [sym_timeline] = ACTIONS(1531),
    [sym_channel] = ACTIONS(1531),
    [sym_custom] = ACTIONS(1531),
    [sym_atoi] = ACTIONS(1531),
    [sym_atol] = ACTIONS(1531),
    [sym_atof] = ACTIONS(1531),
    [sym_itoa] = ACTIONS(1531),
    [sym_itohex] = ACTIONS(1531),
    [sym_hextoi] = ACTIONS(1531),
    [sym_ftoa] = ACTIONS(1531),
    [sym_find_string] = ACTIONS(1531),
    [sym_left_string] = ACTIONS(1531),
    [sym_right_string] = ACTIONS(1531),
    [sym_mid_string] = ACTIONS(1531),
    [sym_remove_string] = ACTIONS(1531),
    [sym_get_buffer_string] = ACTIONS(1531),
    [sym_get_buffer_char] = ACTIONS(1531),
    [sym_get_multi_buffer_string] = ACTIONS(1531),
    [sym_length_string] = ACTIONS(1531),
    [sym_max_length_string] = ACTIONS(1531),
    [sym_set_length_string] = ACTIONS(1531),
    [sym_compare_string] = ACTIONS(1531),
    [sym_upper_string] = ACTIONS(1531),
    [sym_lower_string] = ACTIONS(1531),
    [sym_get_last] = ACTIONS(1531),
    [sym_date_to_day] = ACTIONS(1531),
    [sym_date_to_month] = ACTIONS(1531),
    [sym_date_to_year] = ACTIONS(1531),
    [sym_time_to_hour] = ACTIONS(1531),
    [sym_time_to_minute] = ACTIONS(1531),
    [sym_time_to_second] = ACTIONS(1531),
    [sym_day_of_week] = ACTIONS(1531),
    [sym_set_pulse_time] = ACTIONS(1531),
    [sym_set_timer] = ACTIONS(1531),
    [sym_file_open] = ACTIONS(1531),
    [sym_file_close] = ACTIONS(1531),
    [sym_file_read] = ACTIONS(1531),
    [sym_file_read_line] = ACTIONS(1531),
    [sym_file_write] = ACTIONS(1531),
    [sym_file_write_line] = ACTIONS(1531),
    [sym_file_copy] = ACTIONS(1531),
    [sym_file_delete] = ACTIONS(1531),
    [sym_file_rename] = ACTIONS(1531),
    [sym_file_dir] = ACTIONS(1531),
    [sym_file_getdir] = ACTIONS(1531),
    [sym_file_setdir] = ACTIONS(1531),
    [sym_file_create_dir] = ACTIONS(1531),
    [sym_file_remove_dir] = ACTIONS(1531),
    [sym_file_seek] = ACTIONS(1531),
    [sym_length_array] = ACTIONS(1531),
    [sym_max_length_array] = ACTIONS(1531),
    [sym_set_length_array] = ACTIONS(1531),
    [sym_ip_client_open] = ACTIONS(1531),
    [sym_ip_client_close] = ACTIONS(1531),
    [sym_ip_server_open] = ACTIONS(1531),
    [sym_ip_server_close] = ACTIONS(1531),
    [sym_ip_bound_client_open] = ACTIONS(1531),
    [sym_ip_mc_server_open] = ACTIONS(1531),
    [sym_ip_set_option] = ACTIONS(1531),
    [sym_ssh_client_open] = ACTIONS(1531),
    [sym_ssh_client_close] = ACTIONS(1531),
    [sym_tls_client_open] = ACTIONS(1531),
    [sym_tls_client_close] = ACTIONS(1531),
    [sym_do_push] = ACTIONS(1531),
    [sym_do_push_timed] = ACTIONS(1531),
    [sym_do_release] = ACTIONS(1531),
    [sym_device_id] = ACTIONS(1531),
    [sym_device_info] = ACTIONS(1531),
    [sym_device_id_string] = ACTIONS(1531),
    [sym_device_standby] = ACTIONS(1531),
    [sym_device_wake] = ACTIONS(1531),
    [sym_timeline_create] = ACTIONS(1531),
    [sym_timeline_kill] = ACTIONS(1531),
    [sym_timeline_reload] = ACTIONS(1531),
    [sym_timeline_pause] = ACTIONS(1531),
    [sym_timeline_restart] = ACTIONS(1531),
    [sym_timeline_set] = ACTIONS(1531),
    [sym_timeline_get] = ACTIONS(1531),
    [sym_timeline_active] = ACTIONS(1531),
    [sym_timeline_dynamic_id] = ACTIONS(1531),
    [sym_variable_to_string] = ACTIONS(1531),
    [sym_string_to_variable] = ACTIONS(1531),
    [sym_length_variable_to_string] = ACTIONS(1531),
    [sym_type_cast] = ACTIONS(1531),
    [sym_variable_to_xml] = ACTIONS(1531),
    [sym_xml_to_variable] = ACTIONS(1531),
    [sym_length_variable_to_xml] = ACTIONS(1531),
    [sym_redirect_string] = ACTIONS(1531),
    [sym_random_number] = ACTIONS(1531),
    [sym_get_unique_id] = ACTIONS(1531),
    [sym_get_system_number] = ACTIONS(1531),
    [sym_set_system_number] = ACTIONS(1531),
    [sym_get_serial_number] = ACTIONS(1531),
    [sym_get_master_build] = ACTIONS(1531),
    [sym_set_validation_code] = ACTIONS(1531),
    [sym_reboot] = ACTIONS(1531),
    [sym_load_duet_module] = ACTIONS(1531),
    [sym_unload_duet_module] = ACTIONS(1531),
    [sym_duet_mem_size_set] = ACTIONS(1531),
    [sym_duet_mem_size_get] = ACTIONS(1531),
    [sym_dynamic_application_device] = ACTIONS(1531),
    [sym_dynamic_polled_port] = ACTIONS(1531),
    [sym_static_port_binding] = ACTIONS(1531),
    [sym_static_ip_binding] = ACTIONS(1531),
    [sym_get_dns_list] = ACTIONS(1531),
    [sym_set_dns_list] = ACTIONS(1531),
    [sym_get_ip_address] = ACTIONS(1531),
    [sym_set_ip_address] = ACTIONS(1531),
    [sym_get_url_list] = ACTIONS(1531),
    [sym_add_url_entry] = ACTIONS(1531),
    [sym_delete_url_entry] = ACTIONS(1531),
    [sym_abs_value] = ACTIONS(1531),
    [sym_max_value] = ACTIONS(1531),
    [sym_min_value] = ACTIONS(1531),
    [sym_exp_value] = ACTIONS(1531),
    [sym_log_value] = ACTIONS(1531),
    [sym_log10_value] = ACTIONS(1531),
    [sym_power_value] = ACTIONS(1531),
    [sym_sqrt_value] = ACTIONS(1531),
    [sym_raw_be] = ACTIONS(1531),
    [sym_raw_le] = ACTIONS(1531),
    [sym_internal_queue_size_set] = ACTIONS(1531),
    [sym_internal_queue_size_get] = ACTIONS(1531),
    [sym_internal_threshold_set] = ACTIONS(1531),
    [sym_internal_threshold_get] = ACTIONS(1531),
    [sym_rebuilt_event] = ACTIONS(1531),
    [sym_do_custom_event] = ACTIONS(1531),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1531),
    [sym_clkmgr_set_clk_source] = ACTIONS(1531),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1531),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1531),
    [sym_clkmgr_get_timezone] = ACTIONS(1531),
    [sym_clkmgr_set_timezone] = ACTIONS(1531),
    [sym_clkmgr_get_resync_period] = ACTIONS(1531),
    [sym_clkmgr_set_resync_period] = ACTIONS(1531),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1531),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1531),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1531),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1531),
    [sym_clkmgr_get_timeservers] = ACTIONS(1531),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1531),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1531),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1531),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1531),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1531),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1531),
    [sym_set_log_level] = ACTIONS(1531),
    [sym_get_log_level] = ACTIONS(1531),
    [sym_amx_log] = ACTIONS(1531),
    [sym_smtp_server_config_set] = ACTIONS(1531),
    [sym_smtp_server_config_get] = ACTIONS(1531),
    [sym_smtp_send] = ACTIONS(1531),
    [sym_validate_netlinx_account] = ACTIONS(1531),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1531),
    [sym_authenticate_certificate] = ACTIONS(1531),
    [sym_audit_netlinx_session_event] = ACTIONS(1531),
    [sym_audit_netlinx_generic_event] = ACTIONS(1531),
    [sym_get_available_flash_disk_space] = ACTIONS(1531),
    [sym_get_max_flash_disk_space] = ACTIONS(1531),
    [sym_data_create_feed] = ACTIONS(1531),
    [sym_data_delete_feed] = ACTIONS(1531),
    [sym_data_publish_feed] = ACTIONS(1531),
    [sym_data_get_published_feed] = ACTIONS(1531),
    [sym_data_add_record] = ACTIONS(1531),
    [sym_data_get_event_record] = ACTIONS(1531),
    [sym__wc_data_create_feed] = ACTIONS(1531),
    [sym__wc_data_add_record] = ACTIONS(1531),
    [sym__wc_data_get_event_record] = ACTIONS(1531),
    [sym_format] = ACTIONS(1531),
    [sym_astro_clock] = ACTIONS(1531),
    [sym_set_outdoor_temperature] = ACTIONS(1531),
    [sym_set_virtual_level_count] = ACTIONS(1531),
    [sym_set_virtual_channel_count] = ACTIONS(1531),
    [sym_set_virtual_port_count] = ACTIONS(1531),
    [sym__wc_encode_int] = ACTIONS(1531),
    [sym__wc_decode_int] = ACTIONS(1531),
    [sym__wc_int] = ACTIONS(1531),
    [sym__wc_to_ch_int] = ACTIONS(1531),
    [sym__ch_to_wc_int] = ACTIONS(1531),
    [sym__wc_find_string_int] = ACTIONS(1531),
    [sym__wc_left_string_int] = ACTIONS(1531),
    [sym__wc_lower_string_int] = ACTIONS(1531),
    [sym__wc_mid_string_int] = ACTIONS(1531),
    [sym__wc_remove_string_int] = ACTIONS(1531),
    [sym__wc_right_string_int] = ACTIONS(1531),
    [sym__wc_upper_string_int] = ACTIONS(1531),
    [sym__wc_compare_string_int] = ACTIONS(1531),
    [sym__wc_get_buffer_char_int] = ACTIONS(1531),
    [sym__wc_get_buffer_string_int] = ACTIONS(1531),
    [sym__wc_concat_string_int] = ACTIONS(1531),
    [sym__wc_file_open_int] = ACTIONS(1531),
    [sym__wc_file_close_int] = ACTIONS(1531),
    [sym__wc_file_read_int] = ACTIONS(1531),
    [sym__wc_file_read_line_int] = ACTIONS(1531),
    [sym__wc_file_write_int] = ACTIONS(1531),
    [sym__wc_file_write_line_int] = ACTIONS(1531),
    [anon_sym_EQ] = ACTIONS(1531),
    [aux_sym_preproc_include_token1] = ACTIONS(1531),
    [aux_sym_preproc_define_token1] = ACTIONS(1533),
    [aux_sym_preproc_warn_token1] = ACTIONS(1533),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1533),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1533),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1533),
    [anon_sym_LBRACK] = ACTIONS(1533),
    [anon_sym_LBRACE] = ACTIONS(1533),
    [anon_sym_COLON] = ACTIONS(1533),
    [anon_sym_LPAREN] = ACTIONS(1531),
    [anon_sym_SEMI] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1531),
    [anon_sym_TILDE] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(1533),
    [anon_sym_SLASH] = ACTIONS(1531),
    [anon_sym_PERCENT] = ACTIONS(1533),
    [anon_sym_PIPE_PIPE] = ACTIONS(1533),
    [anon_sym_AMP_AMP] = ACTIONS(1533),
    [anon_sym_PIPE] = ACTIONS(1531),
    [anon_sym_CARET] = ACTIONS(1533),
    [anon_sym_AMP] = ACTIONS(1531),
    [anon_sym_EQ_EQ] = ACTIONS(1533),
    [anon_sym_BANG_EQ] = ACTIONS(1533),
    [anon_sym_LT_GT] = ACTIONS(1533),
    [anon_sym_GT] = ACTIONS(1531),
    [anon_sym_GT_EQ] = ACTIONS(1533),
    [anon_sym_LT_EQ] = ACTIONS(1533),
    [anon_sym_LT] = ACTIONS(1531),
    [anon_sym_LT_LT] = ACTIONS(1533),
    [anon_sym_GT_GT] = ACTIONS(1533),
    [anon_sym_DASH_DASH] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1533),
    [anon_sym_DOT] = ACTIONS(1531),
    [anon_sym_DQUOTE] = ACTIONS(1533),
    [anon_sym_SQUOTE] = ACTIONS(1533),
    [sym_number_literal] = ACTIONS(1533),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1533),
  },
  [STATE(122)] = {
    [ts_builtin_sym_end] = ACTIONS(1537),
    [sym_identifier] = ACTIONS(1535),
    [sym_program_name_keyword] = ACTIONS(1535),
    [sym_module_name_keyword] = ACTIONS(1535),
    [sym_define_device_keyword] = ACTIONS(1535),
    [sym_define_combine_keyword] = ACTIONS(1535),
    [sym_define_constant_keyword] = ACTIONS(1535),
    [sym_define_type_keyword] = ACTIONS(1535),
    [sym_define_variable_keyword] = ACTIONS(1535),
    [sym_define_system_variable_keyword] = ACTIONS(1535),
    [sym_define_connect_level_keyword] = ACTIONS(1535),
    [sym_define_latching_keyword] = ACTIONS(1535),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1535),
    [sym_define_toggling_keyword] = ACTIONS(1535),
    [sym_define_call_keyword] = ACTIONS(1535),
    [sym_define_function_keyword] = ACTIONS(1535),
    [sym_define_library_function_keyword] = ACTIONS(1535),
    [sym_define_start_keyword] = ACTIONS(1535),
    [sym_define_event_keyword] = ACTIONS(1535),
    [sym_define_program_keyword] = ACTIONS(1535),
    [sym_define_module_keyword] = ACTIONS(1535),
    [sym_char_keyword] = ACTIONS(1535),
    [sym_widechar_keyword] = ACTIONS(1535),
    [sym_integer_keyword] = ACTIONS(1535),
    [sym_sinteger_keyword] = ACTIONS(1535),
    [sym_long_keyword] = ACTIONS(1535),
    [sym_slong_keyword] = ACTIONS(1535),
    [sym_float_keyword] = ACTIONS(1535),
    [sym_double_keyword] = ACTIONS(1535),
    [sym_variant_keyword] = ACTIONS(1535),
    [sym_variantarray_keyword] = ACTIONS(1535),
    [sym_local_var_keyword] = ACTIONS(1535),
    [sym_stack_var_keyword] = ACTIONS(1535),
    [sym_constant_keyword] = ACTIONS(1535),
    [sym_volatile_keyword] = ACTIONS(1535),
    [sym_non_volatile_keyword] = ACTIONS(1535),
    [sym_persistent_keyword] = ACTIONS(1535),
    [sym_dev_keyword] = ACTIONS(1535),
    [sym_devlev_keyword] = ACTIONS(1535),
    [sym_devchan_keyword] = ACTIONS(1535),
    [sym_if_keyword] = ACTIONS(1535),
    [sym_else_keyword] = ACTIONS(1535),
    [sym_switch_keyword] = ACTIONS(1535),
    [sym_case_keyword] = ACTIONS(1535),
    [sym_default_keyword] = ACTIONS(1535),
    [sym_while_keyword] = ACTIONS(1535),
    [sym_for_keyword] = ACTIONS(1535),
    [sym_break_keyword] = ACTIONS(1535),
    [sym_continue_keyword] = ACTIONS(1535),
    [sym_return_keyword] = ACTIONS(1535),
    [sym_select_keyword] = ACTIONS(1535),
    [sym_active_keyword] = ACTIONS(1535),
    [sym_struct_keyword] = ACTIONS(1535),
    [sym_structure_keyword] = ACTIONS(1535),
    [sym_button_event_keyword] = ACTIONS(1535),
    [sym_channel_event_keyword] = ACTIONS(1535),
    [sym_level_event_keyword] = ACTIONS(1535),
    [sym_data_event_keyword] = ACTIONS(1535),
    [sym_timeline_event_keyword] = ACTIONS(1535),
    [sym_custom_event_keyword] = ACTIONS(1535),
    [sym_push_keyword] = ACTIONS(1535),
    [sym_release_keyword] = ACTIONS(1535),
    [aux_sym_on_keyword_token1] = ACTIONS(1535),
    [aux_sym_off_keyword_token1] = ACTIONS(1535),
    [sym_send_level_keyword] = ACTIONS(1535),
    [sym_send_string_keyword] = ACTIONS(1535),
    [sym_send_command_keyword] = ACTIONS(1535),
    [sym_clear_buffer_keyword] = ACTIONS(1535),
    [sym_create_buffer_keyword] = ACTIONS(1535),
    [sym_create_multi_buffer_keyword] = ACTIONS(1535),
    [sym_call_keyword] = ACTIONS(1535),
    [sym_system_call_keyword] = ACTIONS(1535),
    [sym_devchan_to_keyword] = ACTIONS(1535),
    [sym_devchan_min_to_keyword] = ACTIONS(1535),
    [sym_devchan_total_off_keyword] = ACTIONS(1535),
    [sym_devchan_pulse_keyword] = ACTIONS(1535),
    [sym_wait_keyword] = ACTIONS(1535),
    [sym_cancel_wait_keyword] = ACTIONS(1535),
    [sym_wait_until_keyword] = ACTIONS(1535),
    [sym_cancel_wait_until_keyword] = ACTIONS(1535),
    [sym_cancel_all_wait_keyword] = ACTIONS(1535),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1535),
    [sym_and] = ACTIONS(1535),
    [sym_or] = ACTIONS(1535),
    [sym_not] = ACTIONS(1535),
    [sym_xor] = ACTIONS(1535),
    [sym_band] = ACTIONS(1535),
    [sym_bor] = ACTIONS(1535),
    [sym_bxor] = ACTIONS(1535),
    [sym_bnot] = ACTIONS(1535),
    [sym_lshift] = ACTIONS(1535),
    [sym_rshift] = ACTIONS(1535),
    [sym___file__] = ACTIONS(1535),
    [sym___date__] = ACTIONS(1535),
    [sym___ldate__] = ACTIONS(1535),
    [sym___time__] = ACTIONS(1535),
    [sym___line__] = ACTIONS(1535),
    [sym___name__] = ACTIONS(1535),
    [sym___netlinx__] = ACTIONS(1535),
    [sym_netlinx_axi_version] = ACTIONS(1535),
    [sym_url_flg_tcp] = ACTIONS(1535),
    [sym_url_flg_accinfopresent] = ACTIONS(1535),
    [sym_url_flg_temp] = ACTIONS(1535),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1535),
    [sym_url_flg_stat_mask] = ACTIONS(1535),
    [sym_url_flg_stat_lookup] = ACTIONS(1535),
    [sym_url_flg_stat_connecting] = ACTIONS(1535),
    [sym_url_flg_stat_waiting] = ACTIONS(1535),
    [sym_url_flg_stat_connected] = ACTIONS(1535),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1535),
    [sym_first_virtual_device] = ACTIONS(1535),
    [sym_first_local_port] = ACTIONS(1535),
    [sym_dynamic_virtual_device] = ACTIONS(1535),
    [sym_timeline_once] = ACTIONS(1535),
    [sym_timeline_repeat] = ACTIONS(1535),
    [sym_timeline_absolute] = ACTIONS(1535),
    [sym_timeline_relative] = ACTIONS(1535),
    [sym_do_push_timed_infinite] = ACTIONS(1535),
    [sym_source_type_no_address] = ACTIONS(1535),
    [sym_source_type_neuron_id] = ACTIONS(1535),
    [sym_source_type_ip_address] = ACTIONS(1535),
    [sym_source_type_axlink] = ACTIONS(1535),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1535),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1535),
    [sym_source_type_ip_socket_address] = ACTIONS(1535),
    [sym_source_type_rs232] = ACTIONS(1535),
    [sym_source_type_internal] = ACTIONS(1535),
    [sym_source_type_mac_address] = ACTIONS(1535),
    [sym_source_type_ipv4_port] = ACTIONS(1535),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1535),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1535),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1535),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1535),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1535),
    [sym_true] = ACTIONS(1535),
    [sym_false] = ACTIONS(1535),
    [sym_file_read_only] = ACTIONS(1535),
    [sym_file_rw_new] = ACTIONS(1535),
    [sym_file_rw_append] = ACTIONS(1535),
    [sym_ip_tcp] = ACTIONS(1535),
    [sym_ip_udp] = ACTIONS(1535),
    [sym_ip_udp_2way] = ACTIONS(1535),
    [sym_xml_encode_types] = ACTIONS(1535),
    [sym_xml_encode_char_as_list] = ACTIONS(1535),
    [sym_xml_encode_le] = ACTIONS(1535),
    [sym_xml_decode_types] = ACTIONS(1535),
    [sym_xml_decode_no_preserve] = ACTIONS(1535),
    [sym_duet_dev_type_display_device] = ACTIONS(1535),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1535),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1535),
    [sym_duet_dev_type_io_device] = ACTIONS(1535),
    [sym_duet_dev_type_relay_device] = ACTIONS(1535),
    [sym_duet_dev_type_ups] = ACTIONS(1535),
    [sym_duet_dev_type_amplifier] = ACTIONS(1535),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1535),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1535),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1535),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1535),
    [sym_duet_dev_type_camera] = ACTIONS(1535),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1535),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1535),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1535),
    [sym_duet_dev_type_dss] = ACTIONS(1535),
    [sym_duet_dev_type_dvr] = ACTIONS(1535),
    [sym_duet_dev_type_disc_device] = ACTIONS(1535),
    [sym_duet_dev_type_document_camera] = ACTIONS(1535),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1535),
    [sym_duet_dev_type_hvac] = ACTIONS(1535),
    [sym_duet_dev_type_keypad] = ACTIONS(1535),
    [sym_duet_dev_type_light] = ACTIONS(1535),
    [sym_duet_dev_type_monitor] = ACTIONS(1535),
    [sym_duet_dev_type_motor] = ACTIONS(1535),
    [sym_duet_dev_type_multi_window] = ACTIONS(1535),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1535),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1535),
    [sym_duet_dev_type_receiver] = ACTIONS(1535),
    [sym_duet_dev_type_security_system] = ACTIONS(1535),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1535),
    [sym_duet_dev_type_settop_box] = ACTIONS(1535),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1535),
    [sym_duet_dev_type_switcher] = ACTIONS(1535),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1535),
    [sym_duet_dev_type_tv] = ACTIONS(1535),
    [sym_duet_dev_type_utility] = ACTIONS(1535),
    [sym_duet_dev_type_vcr] = ACTIONS(1535),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1535),
    [sym_duet_dev_type_video_processor] = ACTIONS(1535),
    [sym_duet_dev_type_video_projector] = ACTIONS(1535),
    [sym_duet_dev_type_video_wall] = ACTIONS(1535),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1535),
    [sym_duet_dev_type_weather] = ACTIONS(1535),
    [sym_duet_dev_not_polled] = ACTIONS(1535),
    [sym_duet_dev_polled] = ACTIONS(1535),
    [sym_clkmgr_mode_network] = ACTIONS(1535),
    [sym_clkmgr_mode_standalone] = ACTIONS(1535),
    [sym_normal_standby] = ACTIONS(1535),
    [sym_normal_wake] = ACTIONS(1535),
    [sym_ip_multicast_ttl_option] = ACTIONS(1535),
    [sym_ip_ttl_subnet] = ACTIONS(1535),
    [sym_ip_ttl_site] = ACTIONS(1535),
    [sym_ip_ttl_region] = ACTIONS(1535),
    [sym_ip_ttl_continent] = ACTIONS(1535),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1535),
    [sym_ip_nodelay_on] = ACTIONS(1535),
    [sym_ip_nodelay_off] = ACTIONS(1535),
    [sym_amx_error] = ACTIONS(1535),
    [sym_amx_warning] = ACTIONS(1535),
    [sym_amx_info] = ACTIONS(1535),
    [sym_amx_debug] = ACTIONS(1535),
    [sym_netlinx_login_success] = ACTIONS(1535),
    [sym_netlinx_login_fail] = ACTIONS(1535),
    [sym_netlinx_logout] = ACTIONS(1535),
    [sym_null_str] = ACTIONS(1535),
    [sym_smtp_address] = ACTIONS(1535),
    [sym_smtp_port_number] = ACTIONS(1535),
    [sym_smtp_username] = ACTIONS(1535),
    [sym_smtp_password] = ACTIONS(1535),
    [sym_smtp_require_tls] = ACTIONS(1535),
    [sym_smtp_from] = ACTIONS(1535),
    [sym_smtp_tls_true] = ACTIONS(1535),
    [sym_smtp_tls_false] = ACTIONS(1535),
    [sym_listview_on_row_select_event] = ACTIONS(1535),
    [sym_data_structure_unknown] = ACTIONS(1535),
    [sym_data_structure_datafeed] = ACTIONS(1535),
    [sym_data_structure_datarecordset] = ACTIONS(1535),
    [sym_data_structure_datarecord] = ACTIONS(1535),
    [sym_data_structure_datafield] = ACTIONS(1535),
    [sym_data_max_content_field_count] = ACTIONS(1535),
    [sym_data_max_metadata_field_count] = ACTIONS(1535),
    [sym_data_max_id_length] = ACTIONS(1535),
    [sym_data_max_name_length] = ACTIONS(1535),
    [sym_data_max_description_length] = ACTIONS(1535),
    [sym_data_max_type_length] = ACTIONS(1535),
    [sym_data_max_format_length] = ACTIONS(1535),
    [sym_data_max_source_length] = ACTIONS(1535),
    [sym_data_max_value_length] = ACTIONS(1535),
    [sym_data_max_label_length] = ACTIONS(1535),
    [sym_data_max_url_length] = ACTIONS(1535),
    [sym_max_last_login_info_length] = ACTIONS(1535),
    [sym_data_type_unknown] = ACTIONS(1535),
    [sym_data_type_string] = ACTIONS(1535),
    [sym_data_type_datetime] = ACTIONS(1535),
    [sym_data_type_date] = ACTIONS(1535),
    [sym_data_type_time] = ACTIONS(1535),
    [sym_data_type_image] = ACTIONS(1535),
    [sym_data_format_url] = ACTIONS(1535),
    [sym_data_format_phone] = ACTIONS(1535),
    [sym_data_format_email] = ACTIONS(1535),
    [sym_data_format_iso8601] = ACTIONS(1535),
    [sym_auditlog_priv] = ACTIONS(1535),
    [sym_configuration_priv] = ACTIONS(1535),
    [sym_deviceconfig_priv] = ACTIONS(1535),
    [sym_ftp_priv] = ACTIONS(1535),
    [sym_http_priv] = ACTIONS(1535),
    [sym_networkconfig_priv] = ACTIONS(1535),
    [sym_programport_priv] = ACTIONS(1535),
    [sym_remoteui_priv] = ACTIONS(1535),
    [sym_securitycontrol_priv] = ACTIONS(1535),
    [sym_softwaremanagement_priv] = ACTIONS(1535),
    [sym_terminal_priv] = ACTIONS(1535),
    [sym_tpadmin_priv] = ACTIONS(1535),
    [sym_user1_priv] = ACTIONS(1535),
    [sym_user2_priv] = ACTIONS(1535),
    [sym_user3_priv] = ACTIONS(1535),
    [sym_user4_priv] = ACTIONS(1535),
    [sym_usermanagement_priv] = ACTIONS(1535),
    [sym_valid_account] = ACTIONS(1535),
    [sym_err_name_invalid] = ACTIONS(1535),
    [sym_err_password_invalid] = ACTIONS(1535),
    [sym_err_invalid_account] = ACTIONS(1535),
    [sym_err_no_authorization] = ACTIONS(1535),
    [sym_err_invalid_parameter] = ACTIONS(1535),
    [sym_err_user_locked_out] = ACTIONS(1535),
    [sym_err_user_account_expired] = ACTIONS(1535),
    [sym_err_general_error] = ACTIONS(1535),
    [sym_err_authentication_error] = ACTIONS(1535),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1535),
    [sym_tls_validate_certificate] = ACTIONS(1535),
    [sym_tdata] = ACTIONS(1535),
    [sym_tchannel] = ACTIONS(1535),
    [sym_tlevel] = ACTIONS(1535),
    [sym_tbutton] = ACTIONS(1535),
    [sym_ttimeline] = ACTIONS(1535),
    [sym_tcustom] = ACTIONS(1535),
    [sym_url_struct] = ACTIONS(1535),
    [sym_dns_struct] = ACTIONS(1535),
    [sym_ip_address_struct] = ACTIONS(1535),
    [sym_dev_info_struct] = ACTIONS(1535),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1535),
    [sym_data_feed] = ACTIONS(1535),
    [sym_data_field] = ACTIONS(1535),
    [sym_data_record] = ACTIONS(1535),
    [sym_wc_data_feed] = ACTIONS(1535),
    [sym_wc_data_field] = ACTIONS(1535),
    [sym_wc_data_record] = ACTIONS(1535),
    [sym_last_login_info] = ACTIONS(1535),
    [sym____reserved___] = ACTIONS(1535),
    [sym_date] = ACTIONS(1535),
    [sym_day] = ACTIONS(1535),
    [sym_time] = ACTIONS(1535),
    [sym_ldate] = ACTIONS(1535),
    [sym_push_device] = ACTIONS(1535),
    [sym_push_channel] = ACTIONS(1535),
    [sym_push_devchan] = ACTIONS(1535),
    [sym_release_device] = ACTIONS(1535),
    [sym_release_channel] = ACTIONS(1535),
    [sym_release_devchan] = ACTIONS(1535),
    [sym_master_slot] = ACTIONS(1535),
    [sym_get_pulse_time] = ACTIONS(1535),
    [sym_get_timer] = ACTIONS(1535),
    [sym_master_sn] = ACTIONS(1535),
    [sym_system_number] = ACTIONS(1535),
    [sym_button] = ACTIONS(1535),
    [sym_dv_channel] = ACTIONS(1535),
    [sym_data] = ACTIONS(1535),
    [sym_level] = ACTIONS(1535),
    [sym_timeline] = ACTIONS(1535),
    [sym_channel] = ACTIONS(1535),
    [sym_custom] = ACTIONS(1535),
    [sym_atoi] = ACTIONS(1535),
    [sym_atol] = ACTIONS(1535),
    [sym_atof] = ACTIONS(1535),
    [sym_itoa] = ACTIONS(1535),
    [sym_itohex] = ACTIONS(1535),
    [sym_hextoi] = ACTIONS(1535),
    [sym_ftoa] = ACTIONS(1535),
    [sym_find_string] = ACTIONS(1535),
    [sym_left_string] = ACTIONS(1535),
    [sym_right_string] = ACTIONS(1535),
    [sym_mid_string] = ACTIONS(1535),
    [sym_remove_string] = ACTIONS(1535),
    [sym_get_buffer_string] = ACTIONS(1535),
    [sym_get_buffer_char] = ACTIONS(1535),
    [sym_get_multi_buffer_string] = ACTIONS(1535),
    [sym_length_string] = ACTIONS(1535),
    [sym_max_length_string] = ACTIONS(1535),
    [sym_set_length_string] = ACTIONS(1535),
    [sym_compare_string] = ACTIONS(1535),
    [sym_upper_string] = ACTIONS(1535),
    [sym_lower_string] = ACTIONS(1535),
    [sym_get_last] = ACTIONS(1535),
    [sym_date_to_day] = ACTIONS(1535),
    [sym_date_to_month] = ACTIONS(1535),
    [sym_date_to_year] = ACTIONS(1535),
    [sym_time_to_hour] = ACTIONS(1535),
    [sym_time_to_minute] = ACTIONS(1535),
    [sym_time_to_second] = ACTIONS(1535),
    [sym_day_of_week] = ACTIONS(1535),
    [sym_set_pulse_time] = ACTIONS(1535),
    [sym_set_timer] = ACTIONS(1535),
    [sym_file_open] = ACTIONS(1535),
    [sym_file_close] = ACTIONS(1535),
    [sym_file_read] = ACTIONS(1535),
    [sym_file_read_line] = ACTIONS(1535),
    [sym_file_write] = ACTIONS(1535),
    [sym_file_write_line] = ACTIONS(1535),
    [sym_file_copy] = ACTIONS(1535),
    [sym_file_delete] = ACTIONS(1535),
    [sym_file_rename] = ACTIONS(1535),
    [sym_file_dir] = ACTIONS(1535),
    [sym_file_getdir] = ACTIONS(1535),
    [sym_file_setdir] = ACTIONS(1535),
    [sym_file_create_dir] = ACTIONS(1535),
    [sym_file_remove_dir] = ACTIONS(1535),
    [sym_file_seek] = ACTIONS(1535),
    [sym_length_array] = ACTIONS(1535),
    [sym_max_length_array] = ACTIONS(1535),
    [sym_set_length_array] = ACTIONS(1535),
    [sym_ip_client_open] = ACTIONS(1535),
    [sym_ip_client_close] = ACTIONS(1535),
    [sym_ip_server_open] = ACTIONS(1535),
    [sym_ip_server_close] = ACTIONS(1535),
    [sym_ip_bound_client_open] = ACTIONS(1535),
    [sym_ip_mc_server_open] = ACTIONS(1535),
    [sym_ip_set_option] = ACTIONS(1535),
    [sym_ssh_client_open] = ACTIONS(1535),
    [sym_ssh_client_close] = ACTIONS(1535),
    [sym_tls_client_open] = ACTIONS(1535),
    [sym_tls_client_close] = ACTIONS(1535),
    [sym_do_push] = ACTIONS(1535),
    [sym_do_push_timed] = ACTIONS(1535),
    [sym_do_release] = ACTIONS(1535),
    [sym_device_id] = ACTIONS(1535),
    [sym_device_info] = ACTIONS(1535),
    [sym_device_id_string] = ACTIONS(1535),
    [sym_device_standby] = ACTIONS(1535),
    [sym_device_wake] = ACTIONS(1535),
    [sym_timeline_create] = ACTIONS(1535),
    [sym_timeline_kill] = ACTIONS(1535),
    [sym_timeline_reload] = ACTIONS(1535),
    [sym_timeline_pause] = ACTIONS(1535),
    [sym_timeline_restart] = ACTIONS(1535),
    [sym_timeline_set] = ACTIONS(1535),
    [sym_timeline_get] = ACTIONS(1535),
    [sym_timeline_active] = ACTIONS(1535),
    [sym_timeline_dynamic_id] = ACTIONS(1535),
    [sym_variable_to_string] = ACTIONS(1535),
    [sym_string_to_variable] = ACTIONS(1535),
    [sym_length_variable_to_string] = ACTIONS(1535),
    [sym_type_cast] = ACTIONS(1535),
    [sym_variable_to_xml] = ACTIONS(1535),
    [sym_xml_to_variable] = ACTIONS(1535),
    [sym_length_variable_to_xml] = ACTIONS(1535),
    [sym_redirect_string] = ACTIONS(1535),
    [sym_random_number] = ACTIONS(1535),
    [sym_get_unique_id] = ACTIONS(1535),
    [sym_get_system_number] = ACTIONS(1535),
    [sym_set_system_number] = ACTIONS(1535),
    [sym_get_serial_number] = ACTIONS(1535),
    [sym_get_master_build] = ACTIONS(1535),
    [sym_set_validation_code] = ACTIONS(1535),
    [sym_reboot] = ACTIONS(1535),
    [sym_load_duet_module] = ACTIONS(1535),
    [sym_unload_duet_module] = ACTIONS(1535),
    [sym_duet_mem_size_set] = ACTIONS(1535),
    [sym_duet_mem_size_get] = ACTIONS(1535),
    [sym_dynamic_application_device] = ACTIONS(1535),
    [sym_dynamic_polled_port] = ACTIONS(1535),
    [sym_static_port_binding] = ACTIONS(1535),
    [sym_static_ip_binding] = ACTIONS(1535),
    [sym_get_dns_list] = ACTIONS(1535),
    [sym_set_dns_list] = ACTIONS(1535),
    [sym_get_ip_address] = ACTIONS(1535),
    [sym_set_ip_address] = ACTIONS(1535),
    [sym_get_url_list] = ACTIONS(1535),
    [sym_add_url_entry] = ACTIONS(1535),
    [sym_delete_url_entry] = ACTIONS(1535),
    [sym_abs_value] = ACTIONS(1535),
    [sym_max_value] = ACTIONS(1535),
    [sym_min_value] = ACTIONS(1535),
    [sym_exp_value] = ACTIONS(1535),
    [sym_log_value] = ACTIONS(1535),
    [sym_log10_value] = ACTIONS(1535),
    [sym_power_value] = ACTIONS(1535),
    [sym_sqrt_value] = ACTIONS(1535),
    [sym_raw_be] = ACTIONS(1535),
    [sym_raw_le] = ACTIONS(1535),
    [sym_internal_queue_size_set] = ACTIONS(1535),
    [sym_internal_queue_size_get] = ACTIONS(1535),
    [sym_internal_threshold_set] = ACTIONS(1535),
    [sym_internal_threshold_get] = ACTIONS(1535),
    [sym_rebuilt_event] = ACTIONS(1535),
    [sym_do_custom_event] = ACTIONS(1535),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1535),
    [sym_clkmgr_set_clk_source] = ACTIONS(1535),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1535),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1535),
    [sym_clkmgr_get_timezone] = ACTIONS(1535),
    [sym_clkmgr_set_timezone] = ACTIONS(1535),
    [sym_clkmgr_get_resync_period] = ACTIONS(1535),
    [sym_clkmgr_set_resync_period] = ACTIONS(1535),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1535),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1535),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1535),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1535),
    [sym_clkmgr_get_timeservers] = ACTIONS(1535),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1535),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1535),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1535),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1535),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1535),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1535),
    [sym_set_log_level] = ACTIONS(1535),
    [sym_get_log_level] = ACTIONS(1535),
    [sym_amx_log] = ACTIONS(1535),
    [sym_smtp_server_config_set] = ACTIONS(1535),
    [sym_smtp_server_config_get] = ACTIONS(1535),
    [sym_smtp_send] = ACTIONS(1535),
    [sym_validate_netlinx_account] = ACTIONS(1535),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1535),
    [sym_authenticate_certificate] = ACTIONS(1535),
    [sym_audit_netlinx_session_event] = ACTIONS(1535),
    [sym_audit_netlinx_generic_event] = ACTIONS(1535),
    [sym_get_available_flash_disk_space] = ACTIONS(1535),
    [sym_get_max_flash_disk_space] = ACTIONS(1535),
    [sym_data_create_feed] = ACTIONS(1535),
    [sym_data_delete_feed] = ACTIONS(1535),
    [sym_data_publish_feed] = ACTIONS(1535),
    [sym_data_get_published_feed] = ACTIONS(1535),
    [sym_data_add_record] = ACTIONS(1535),
    [sym_data_get_event_record] = ACTIONS(1535),
    [sym__wc_data_create_feed] = ACTIONS(1535),
    [sym__wc_data_add_record] = ACTIONS(1535),
    [sym__wc_data_get_event_record] = ACTIONS(1535),
    [sym_format] = ACTIONS(1535),
    [sym_astro_clock] = ACTIONS(1535),
    [sym_set_outdoor_temperature] = ACTIONS(1535),
    [sym_set_virtual_level_count] = ACTIONS(1535),
    [sym_set_virtual_channel_count] = ACTIONS(1535),
    [sym_set_virtual_port_count] = ACTIONS(1535),
    [sym__wc_encode_int] = ACTIONS(1535),
    [sym__wc_decode_int] = ACTIONS(1535),
    [sym__wc_int] = ACTIONS(1535),
    [sym__wc_to_ch_int] = ACTIONS(1535),
    [sym__ch_to_wc_int] = ACTIONS(1535),
    [sym__wc_find_string_int] = ACTIONS(1535),
    [sym__wc_left_string_int] = ACTIONS(1535),
    [sym__wc_lower_string_int] = ACTIONS(1535),
    [sym__wc_mid_string_int] = ACTIONS(1535),
    [sym__wc_remove_string_int] = ACTIONS(1535),
    [sym__wc_right_string_int] = ACTIONS(1535),
    [sym__wc_upper_string_int] = ACTIONS(1535),
    [sym__wc_compare_string_int] = ACTIONS(1535),
    [sym__wc_get_buffer_char_int] = ACTIONS(1535),
    [sym__wc_get_buffer_string_int] = ACTIONS(1535),
    [sym__wc_concat_string_int] = ACTIONS(1535),
    [sym__wc_file_open_int] = ACTIONS(1535),
    [sym__wc_file_close_int] = ACTIONS(1535),
    [sym__wc_file_read_int] = ACTIONS(1535),
    [sym__wc_file_read_line_int] = ACTIONS(1535),
    [sym__wc_file_write_int] = ACTIONS(1535),
    [sym__wc_file_write_line_int] = ACTIONS(1535),
    [anon_sym_EQ] = ACTIONS(1535),
    [aux_sym_preproc_include_token1] = ACTIONS(1535),
    [aux_sym_preproc_define_token1] = ACTIONS(1537),
    [aux_sym_preproc_warn_token1] = ACTIONS(1537),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1537),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1537),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1537),
    [anon_sym_LBRACK] = ACTIONS(1537),
    [anon_sym_LBRACE] = ACTIONS(1537),
    [anon_sym_COLON] = ACTIONS(1537),
    [anon_sym_LPAREN] = ACTIONS(1535),
    [anon_sym_SEMI] = ACTIONS(1537),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1537),
    [anon_sym_DASH] = ACTIONS(1535),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_STAR] = ACTIONS(1537),
    [anon_sym_SLASH] = ACTIONS(1535),
    [anon_sym_PERCENT] = ACTIONS(1537),
    [anon_sym_PIPE_PIPE] = ACTIONS(1537),
    [anon_sym_AMP_AMP] = ACTIONS(1537),
    [anon_sym_PIPE] = ACTIONS(1535),
    [anon_sym_CARET] = ACTIONS(1537),
    [anon_sym_AMP] = ACTIONS(1535),
    [anon_sym_EQ_EQ] = ACTIONS(1537),
    [anon_sym_BANG_EQ] = ACTIONS(1537),
    [anon_sym_LT_GT] = ACTIONS(1537),
    [anon_sym_GT] = ACTIONS(1535),
    [anon_sym_GT_EQ] = ACTIONS(1537),
    [anon_sym_LT_EQ] = ACTIONS(1537),
    [anon_sym_LT] = ACTIONS(1535),
    [anon_sym_LT_LT] = ACTIONS(1537),
    [anon_sym_GT_GT] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DOT] = ACTIONS(1535),
    [anon_sym_DQUOTE] = ACTIONS(1537),
    [anon_sym_SQUOTE] = ACTIONS(1537),
    [sym_number_literal] = ACTIONS(1537),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1537),
  },
  [STATE(123)] = {
    [sym_identifier] = ACTIONS(1569),
    [sym_program_name_keyword] = ACTIONS(1569),
    [sym_module_name_keyword] = ACTIONS(1569),
    [sym_define_device_keyword] = ACTIONS(1569),
    [sym_define_combine_keyword] = ACTIONS(1569),
    [sym_define_constant_keyword] = ACTIONS(1569),
    [sym_define_type_keyword] = ACTIONS(1569),
    [sym_define_variable_keyword] = ACTIONS(1569),
    [sym_define_system_variable_keyword] = ACTIONS(1569),
    [sym_define_connect_level_keyword] = ACTIONS(1569),
    [sym_define_latching_keyword] = ACTIONS(1569),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1569),
    [sym_define_toggling_keyword] = ACTIONS(1569),
    [sym_define_call_keyword] = ACTIONS(1569),
    [sym_define_function_keyword] = ACTIONS(1569),
    [sym_define_library_function_keyword] = ACTIONS(1569),
    [sym_define_start_keyword] = ACTIONS(1569),
    [sym_define_event_keyword] = ACTIONS(1569),
    [sym_define_program_keyword] = ACTIONS(1569),
    [sym_define_module_keyword] = ACTIONS(1569),
    [sym_char_keyword] = ACTIONS(1569),
    [sym_widechar_keyword] = ACTIONS(1569),
    [sym_integer_keyword] = ACTIONS(1569),
    [sym_sinteger_keyword] = ACTIONS(1569),
    [sym_long_keyword] = ACTIONS(1569),
    [sym_slong_keyword] = ACTIONS(1569),
    [sym_float_keyword] = ACTIONS(1569),
    [sym_double_keyword] = ACTIONS(1569),
    [sym_variant_keyword] = ACTIONS(1569),
    [sym_variantarray_keyword] = ACTIONS(1569),
    [sym_local_var_keyword] = ACTIONS(1569),
    [sym_stack_var_keyword] = ACTIONS(1569),
    [sym_constant_keyword] = ACTIONS(1569),
    [sym_volatile_keyword] = ACTIONS(1569),
    [sym_non_volatile_keyword] = ACTIONS(1569),
    [sym_persistent_keyword] = ACTIONS(1569),
    [sym_dev_keyword] = ACTIONS(1569),
    [sym_devlev_keyword] = ACTIONS(1569),
    [sym_devchan_keyword] = ACTIONS(1569),
    [sym_if_keyword] = ACTIONS(1569),
    [sym_else_keyword] = ACTIONS(1569),
    [sym_switch_keyword] = ACTIONS(1569),
    [sym_case_keyword] = ACTIONS(1569),
    [sym_default_keyword] = ACTIONS(1569),
    [sym_while_keyword] = ACTIONS(1569),
    [sym_for_keyword] = ACTIONS(1569),
    [sym_break_keyword] = ACTIONS(1569),
    [sym_continue_keyword] = ACTIONS(1569),
    [sym_return_keyword] = ACTIONS(1569),
    [sym_select_keyword] = ACTIONS(1569),
    [sym_active_keyword] = ACTIONS(1569),
    [sym_struct_keyword] = ACTIONS(1569),
    [sym_structure_keyword] = ACTIONS(1569),
    [sym_button_event_keyword] = ACTIONS(1569),
    [sym_channel_event_keyword] = ACTIONS(1569),
    [sym_level_event_keyword] = ACTIONS(1569),
    [sym_data_event_keyword] = ACTIONS(1569),
    [sym_timeline_event_keyword] = ACTIONS(1569),
    [sym_custom_event_keyword] = ACTIONS(1569),
    [sym_push_keyword] = ACTIONS(1569),
    [sym_release_keyword] = ACTIONS(1569),
    [aux_sym_on_keyword_token1] = ACTIONS(1569),
    [aux_sym_off_keyword_token1] = ACTIONS(1569),
    [sym_send_level_keyword] = ACTIONS(1569),
    [sym_send_string_keyword] = ACTIONS(1569),
    [sym_send_command_keyword] = ACTIONS(1569),
    [sym_clear_buffer_keyword] = ACTIONS(1569),
    [sym_create_buffer_keyword] = ACTIONS(1569),
    [sym_create_multi_buffer_keyword] = ACTIONS(1569),
    [sym_call_keyword] = ACTIONS(1569),
    [sym_system_call_keyword] = ACTIONS(1569),
    [sym_devchan_to_keyword] = ACTIONS(1569),
    [sym_devchan_min_to_keyword] = ACTIONS(1569),
    [sym_devchan_total_off_keyword] = ACTIONS(1569),
    [sym_devchan_pulse_keyword] = ACTIONS(1569),
    [sym_wait_keyword] = ACTIONS(1569),
    [sym_cancel_wait_keyword] = ACTIONS(1569),
    [sym_wait_until_keyword] = ACTIONS(1569),
    [sym_cancel_wait_until_keyword] = ACTIONS(1569),
    [sym_cancel_all_wait_keyword] = ACTIONS(1569),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1569),
    [sym_and] = ACTIONS(1569),
    [sym_or] = ACTIONS(1569),
    [sym_not] = ACTIONS(1569),
    [sym_xor] = ACTIONS(1569),
    [sym_band] = ACTIONS(1569),
    [sym_bor] = ACTIONS(1569),
    [sym_bxor] = ACTIONS(1569),
    [sym_bnot] = ACTIONS(1569),
    [sym_lshift] = ACTIONS(1569),
    [sym_rshift] = ACTIONS(1569),
    [sym___file__] = ACTIONS(1569),
    [sym___date__] = ACTIONS(1569),
    [sym___ldate__] = ACTIONS(1569),
    [sym___time__] = ACTIONS(1569),
    [sym___line__] = ACTIONS(1569),
    [sym___name__] = ACTIONS(1569),
    [sym___netlinx__] = ACTIONS(1569),
    [sym_netlinx_axi_version] = ACTIONS(1569),
    [sym_url_flg_tcp] = ACTIONS(1569),
    [sym_url_flg_accinfopresent] = ACTIONS(1569),
    [sym_url_flg_temp] = ACTIONS(1569),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1569),
    [sym_url_flg_stat_mask] = ACTIONS(1569),
    [sym_url_flg_stat_lookup] = ACTIONS(1569),
    [sym_url_flg_stat_connecting] = ACTIONS(1569),
    [sym_url_flg_stat_waiting] = ACTIONS(1569),
    [sym_url_flg_stat_connected] = ACTIONS(1569),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1569),
    [sym_first_virtual_device] = ACTIONS(1569),
    [sym_first_local_port] = ACTIONS(1569),
    [sym_dynamic_virtual_device] = ACTIONS(1569),
    [sym_timeline_once] = ACTIONS(1569),
    [sym_timeline_repeat] = ACTIONS(1569),
    [sym_timeline_absolute] = ACTIONS(1569),
    [sym_timeline_relative] = ACTIONS(1569),
    [sym_do_push_timed_infinite] = ACTIONS(1569),
    [sym_source_type_no_address] = ACTIONS(1569),
    [sym_source_type_neuron_id] = ACTIONS(1569),
    [sym_source_type_ip_address] = ACTIONS(1569),
    [sym_source_type_axlink] = ACTIONS(1569),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1569),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1569),
    [sym_source_type_ip_socket_address] = ACTIONS(1569),
    [sym_source_type_rs232] = ACTIONS(1569),
    [sym_source_type_internal] = ACTIONS(1569),
    [sym_source_type_mac_address] = ACTIONS(1569),
    [sym_source_type_ipv4_port] = ACTIONS(1569),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1569),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1569),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1569),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1569),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1569),
    [sym_true] = ACTIONS(1569),
    [sym_false] = ACTIONS(1569),
    [sym_file_read_only] = ACTIONS(1569),
    [sym_file_rw_new] = ACTIONS(1569),
    [sym_file_rw_append] = ACTIONS(1569),
    [sym_ip_tcp] = ACTIONS(1569),
    [sym_ip_udp] = ACTIONS(1569),
    [sym_ip_udp_2way] = ACTIONS(1569),
    [sym_xml_encode_types] = ACTIONS(1569),
    [sym_xml_encode_char_as_list] = ACTIONS(1569),
    [sym_xml_encode_le] = ACTIONS(1569),
    [sym_xml_decode_types] = ACTIONS(1569),
    [sym_xml_decode_no_preserve] = ACTIONS(1569),
    [sym_duet_dev_type_display_device] = ACTIONS(1569),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1569),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1569),
    [sym_duet_dev_type_io_device] = ACTIONS(1569),
    [sym_duet_dev_type_relay_device] = ACTIONS(1569),
    [sym_duet_dev_type_ups] = ACTIONS(1569),
    [sym_duet_dev_type_amplifier] = ACTIONS(1569),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1569),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1569),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1569),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1569),
    [sym_duet_dev_type_camera] = ACTIONS(1569),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1569),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1569),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1569),
    [sym_duet_dev_type_dss] = ACTIONS(1569),
    [sym_duet_dev_type_dvr] = ACTIONS(1569),
    [sym_duet_dev_type_disc_device] = ACTIONS(1569),
    [sym_duet_dev_type_document_camera] = ACTIONS(1569),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1569),
    [sym_duet_dev_type_hvac] = ACTIONS(1569),
    [sym_duet_dev_type_keypad] = ACTIONS(1569),
    [sym_duet_dev_type_light] = ACTIONS(1569),
    [sym_duet_dev_type_monitor] = ACTIONS(1569),
    [sym_duet_dev_type_motor] = ACTIONS(1569),
    [sym_duet_dev_type_multi_window] = ACTIONS(1569),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1569),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1569),
    [sym_duet_dev_type_receiver] = ACTIONS(1569),
    [sym_duet_dev_type_security_system] = ACTIONS(1569),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1569),
    [sym_duet_dev_type_settop_box] = ACTIONS(1569),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1569),
    [sym_duet_dev_type_switcher] = ACTIONS(1569),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1569),
    [sym_duet_dev_type_tv] = ACTIONS(1569),
    [sym_duet_dev_type_utility] = ACTIONS(1569),
    [sym_duet_dev_type_vcr] = ACTIONS(1569),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1569),
    [sym_duet_dev_type_video_processor] = ACTIONS(1569),
    [sym_duet_dev_type_video_projector] = ACTIONS(1569),
    [sym_duet_dev_type_video_wall] = ACTIONS(1569),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1569),
    [sym_duet_dev_type_weather] = ACTIONS(1569),
    [sym_duet_dev_not_polled] = ACTIONS(1569),
    [sym_duet_dev_polled] = ACTIONS(1569),
    [sym_clkmgr_mode_network] = ACTIONS(1569),
    [sym_clkmgr_mode_standalone] = ACTIONS(1569),
    [sym_normal_standby] = ACTIONS(1569),
    [sym_normal_wake] = ACTIONS(1569),
    [sym_ip_multicast_ttl_option] = ACTIONS(1569),
    [sym_ip_ttl_subnet] = ACTIONS(1569),
    [sym_ip_ttl_site] = ACTIONS(1569),
    [sym_ip_ttl_region] = ACTIONS(1569),
    [sym_ip_ttl_continent] = ACTIONS(1569),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1569),
    [sym_ip_nodelay_on] = ACTIONS(1569),
    [sym_ip_nodelay_off] = ACTIONS(1569),
    [sym_amx_error] = ACTIONS(1569),
    [sym_amx_warning] = ACTIONS(1569),
    [sym_amx_info] = ACTIONS(1569),
    [sym_amx_debug] = ACTIONS(1569),
    [sym_netlinx_login_success] = ACTIONS(1569),
    [sym_netlinx_login_fail] = ACTIONS(1569),
    [sym_netlinx_logout] = ACTIONS(1569),
    [sym_null_str] = ACTIONS(1569),
    [sym_smtp_address] = ACTIONS(1569),
    [sym_smtp_port_number] = ACTIONS(1569),
    [sym_smtp_username] = ACTIONS(1569),
    [sym_smtp_password] = ACTIONS(1569),
    [sym_smtp_require_tls] = ACTIONS(1569),
    [sym_smtp_from] = ACTIONS(1569),
    [sym_smtp_tls_true] = ACTIONS(1569),
    [sym_smtp_tls_false] = ACTIONS(1569),
    [sym_listview_on_row_select_event] = ACTIONS(1569),
    [sym_data_structure_unknown] = ACTIONS(1569),
    [sym_data_structure_datafeed] = ACTIONS(1569),
    [sym_data_structure_datarecordset] = ACTIONS(1569),
    [sym_data_structure_datarecord] = ACTIONS(1569),
    [sym_data_structure_datafield] = ACTIONS(1569),
    [sym_data_max_content_field_count] = ACTIONS(1569),
    [sym_data_max_metadata_field_count] = ACTIONS(1569),
    [sym_data_max_id_length] = ACTIONS(1569),
    [sym_data_max_name_length] = ACTIONS(1569),
    [sym_data_max_description_length] = ACTIONS(1569),
    [sym_data_max_type_length] = ACTIONS(1569),
    [sym_data_max_format_length] = ACTIONS(1569),
    [sym_data_max_source_length] = ACTIONS(1569),
    [sym_data_max_value_length] = ACTIONS(1569),
    [sym_data_max_label_length] = ACTIONS(1569),
    [sym_data_max_url_length] = ACTIONS(1569),
    [sym_max_last_login_info_length] = ACTIONS(1569),
    [sym_data_type_unknown] = ACTIONS(1569),
    [sym_data_type_string] = ACTIONS(1569),
    [sym_data_type_datetime] = ACTIONS(1569),
    [sym_data_type_date] = ACTIONS(1569),
    [sym_data_type_time] = ACTIONS(1569),
    [sym_data_type_image] = ACTIONS(1569),
    [sym_data_format_url] = ACTIONS(1569),
    [sym_data_format_phone] = ACTIONS(1569),
    [sym_data_format_email] = ACTIONS(1569),
    [sym_data_format_iso8601] = ACTIONS(1569),
    [sym_auditlog_priv] = ACTIONS(1569),
    [sym_configuration_priv] = ACTIONS(1569),
    [sym_deviceconfig_priv] = ACTIONS(1569),
    [sym_ftp_priv] = ACTIONS(1569),
    [sym_http_priv] = ACTIONS(1569),
    [sym_networkconfig_priv] = ACTIONS(1569),
    [sym_programport_priv] = ACTIONS(1569),
    [sym_remoteui_priv] = ACTIONS(1569),
    [sym_securitycontrol_priv] = ACTIONS(1569),
    [sym_softwaremanagement_priv] = ACTIONS(1569),
    [sym_terminal_priv] = ACTIONS(1569),
    [sym_tpadmin_priv] = ACTIONS(1569),
    [sym_user1_priv] = ACTIONS(1569),
    [sym_user2_priv] = ACTIONS(1569),
    [sym_user3_priv] = ACTIONS(1569),
    [sym_user4_priv] = ACTIONS(1569),
    [sym_usermanagement_priv] = ACTIONS(1569),
    [sym_valid_account] = ACTIONS(1569),
    [sym_err_name_invalid] = ACTIONS(1569),
    [sym_err_password_invalid] = ACTIONS(1569),
    [sym_err_invalid_account] = ACTIONS(1569),
    [sym_err_no_authorization] = ACTIONS(1569),
    [sym_err_invalid_parameter] = ACTIONS(1569),
    [sym_err_user_locked_out] = ACTIONS(1569),
    [sym_err_user_account_expired] = ACTIONS(1569),
    [sym_err_general_error] = ACTIONS(1569),
    [sym_err_authentication_error] = ACTIONS(1569),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1569),
    [sym_tls_validate_certificate] = ACTIONS(1569),
    [sym_tdata] = ACTIONS(1569),
    [sym_tchannel] = ACTIONS(1569),
    [sym_tlevel] = ACTIONS(1569),
    [sym_tbutton] = ACTIONS(1569),
    [sym_ttimeline] = ACTIONS(1569),
    [sym_tcustom] = ACTIONS(1569),
    [sym_url_struct] = ACTIONS(1569),
    [sym_dns_struct] = ACTIONS(1569),
    [sym_ip_address_struct] = ACTIONS(1569),
    [sym_dev_info_struct] = ACTIONS(1569),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1569),
    [sym_data_feed] = ACTIONS(1569),
    [sym_data_field] = ACTIONS(1569),
    [sym_data_record] = ACTIONS(1569),
    [sym_wc_data_feed] = ACTIONS(1569),
    [sym_wc_data_field] = ACTIONS(1569),
    [sym_wc_data_record] = ACTIONS(1569),
    [sym_last_login_info] = ACTIONS(1569),
    [sym____reserved___] = ACTIONS(1569),
    [sym_date] = ACTIONS(1569),
    [sym_day] = ACTIONS(1569),
    [sym_time] = ACTIONS(1569),
    [sym_ldate] = ACTIONS(1569),
    [sym_push_device] = ACTIONS(1569),
    [sym_push_channel] = ACTIONS(1569),
    [sym_push_devchan] = ACTIONS(1569),
    [sym_release_device] = ACTIONS(1569),
    [sym_release_channel] = ACTIONS(1569),
    [sym_release_devchan] = ACTIONS(1569),
    [sym_master_slot] = ACTIONS(1569),
    [sym_get_pulse_time] = ACTIONS(1569),
    [sym_get_timer] = ACTIONS(1569),
    [sym_master_sn] = ACTIONS(1569),
    [sym_system_number] = ACTIONS(1569),
    [sym_button] = ACTIONS(1569),
    [sym_dv_channel] = ACTIONS(1569),
    [sym_data] = ACTIONS(1569),
    [sym_level] = ACTIONS(1569),
    [sym_timeline] = ACTIONS(1569),
    [sym_channel] = ACTIONS(1569),
    [sym_custom] = ACTIONS(1569),
    [sym_atoi] = ACTIONS(1569),
    [sym_atol] = ACTIONS(1569),
    [sym_atof] = ACTIONS(1569),
    [sym_itoa] = ACTIONS(1569),
    [sym_itohex] = ACTIONS(1569),
    [sym_hextoi] = ACTIONS(1569),
    [sym_ftoa] = ACTIONS(1569),
    [sym_find_string] = ACTIONS(1569),
    [sym_left_string] = ACTIONS(1569),
    [sym_right_string] = ACTIONS(1569),
    [sym_mid_string] = ACTIONS(1569),
    [sym_remove_string] = ACTIONS(1569),
    [sym_get_buffer_string] = ACTIONS(1569),
    [sym_get_buffer_char] = ACTIONS(1569),
    [sym_get_multi_buffer_string] = ACTIONS(1569),
    [sym_length_string] = ACTIONS(1569),
    [sym_max_length_string] = ACTIONS(1569),
    [sym_set_length_string] = ACTIONS(1569),
    [sym_compare_string] = ACTIONS(1569),
    [sym_upper_string] = ACTIONS(1569),
    [sym_lower_string] = ACTIONS(1569),
    [sym_get_last] = ACTIONS(1569),
    [sym_date_to_day] = ACTIONS(1569),
    [sym_date_to_month] = ACTIONS(1569),
    [sym_date_to_year] = ACTIONS(1569),
    [sym_time_to_hour] = ACTIONS(1569),
    [sym_time_to_minute] = ACTIONS(1569),
    [sym_time_to_second] = ACTIONS(1569),
    [sym_day_of_week] = ACTIONS(1569),
    [sym_set_pulse_time] = ACTIONS(1569),
    [sym_set_timer] = ACTIONS(1569),
    [sym_file_open] = ACTIONS(1569),
    [sym_file_close] = ACTIONS(1569),
    [sym_file_read] = ACTIONS(1569),
    [sym_file_read_line] = ACTIONS(1569),
    [sym_file_write] = ACTIONS(1569),
    [sym_file_write_line] = ACTIONS(1569),
    [sym_file_copy] = ACTIONS(1569),
    [sym_file_delete] = ACTIONS(1569),
    [sym_file_rename] = ACTIONS(1569),
    [sym_file_dir] = ACTIONS(1569),
    [sym_file_getdir] = ACTIONS(1569),
    [sym_file_setdir] = ACTIONS(1569),
    [sym_file_create_dir] = ACTIONS(1569),
    [sym_file_remove_dir] = ACTIONS(1569),
    [sym_file_seek] = ACTIONS(1569),
    [sym_length_array] = ACTIONS(1569),
    [sym_max_length_array] = ACTIONS(1569),
    [sym_set_length_array] = ACTIONS(1569),
    [sym_ip_client_open] = ACTIONS(1569),
    [sym_ip_client_close] = ACTIONS(1569),
    [sym_ip_server_open] = ACTIONS(1569),
    [sym_ip_server_close] = ACTIONS(1569),
    [sym_ip_bound_client_open] = ACTIONS(1569),
    [sym_ip_mc_server_open] = ACTIONS(1569),
    [sym_ip_set_option] = ACTIONS(1569),
    [sym_ssh_client_open] = ACTIONS(1569),
    [sym_ssh_client_close] = ACTIONS(1569),
    [sym_tls_client_open] = ACTIONS(1569),
    [sym_tls_client_close] = ACTIONS(1569),
    [sym_do_push] = ACTIONS(1569),
    [sym_do_push_timed] = ACTIONS(1569),
    [sym_do_release] = ACTIONS(1569),
    [sym_device_id] = ACTIONS(1569),
    [sym_device_info] = ACTIONS(1569),
    [sym_device_id_string] = ACTIONS(1569),
    [sym_device_standby] = ACTIONS(1569),
    [sym_device_wake] = ACTIONS(1569),
    [sym_timeline_create] = ACTIONS(1569),
    [sym_timeline_kill] = ACTIONS(1569),
    [sym_timeline_reload] = ACTIONS(1569),
    [sym_timeline_pause] = ACTIONS(1569),
    [sym_timeline_restart] = ACTIONS(1569),
    [sym_timeline_set] = ACTIONS(1569),
    [sym_timeline_get] = ACTIONS(1569),
    [sym_timeline_active] = ACTIONS(1569),
    [sym_timeline_dynamic_id] = ACTIONS(1569),
    [sym_variable_to_string] = ACTIONS(1569),
    [sym_string_to_variable] = ACTIONS(1569),
    [sym_length_variable_to_string] = ACTIONS(1569),
    [sym_type_cast] = ACTIONS(1569),
    [sym_variable_to_xml] = ACTIONS(1569),
    [sym_xml_to_variable] = ACTIONS(1569),
    [sym_length_variable_to_xml] = ACTIONS(1569),
    [sym_redirect_string] = ACTIONS(1569),
    [sym_random_number] = ACTIONS(1569),
    [sym_get_unique_id] = ACTIONS(1569),
    [sym_get_system_number] = ACTIONS(1569),
    [sym_set_system_number] = ACTIONS(1569),
    [sym_get_serial_number] = ACTIONS(1569),
    [sym_get_master_build] = ACTIONS(1569),
    [sym_set_validation_code] = ACTIONS(1569),
    [sym_reboot] = ACTIONS(1569),
    [sym_load_duet_module] = ACTIONS(1569),
    [sym_unload_duet_module] = ACTIONS(1569),
    [sym_duet_mem_size_set] = ACTIONS(1569),
    [sym_duet_mem_size_get] = ACTIONS(1569),
    [sym_dynamic_application_device] = ACTIONS(1569),
    [sym_dynamic_polled_port] = ACTIONS(1569),
    [sym_static_port_binding] = ACTIONS(1569),
    [sym_static_ip_binding] = ACTIONS(1569),
    [sym_get_dns_list] = ACTIONS(1569),
    [sym_set_dns_list] = ACTIONS(1569),
    [sym_get_ip_address] = ACTIONS(1569),
    [sym_set_ip_address] = ACTIONS(1569),
    [sym_get_url_list] = ACTIONS(1569),
    [sym_add_url_entry] = ACTIONS(1569),
    [sym_delete_url_entry] = ACTIONS(1569),
    [sym_abs_value] = ACTIONS(1569),
    [sym_max_value] = ACTIONS(1569),
    [sym_min_value] = ACTIONS(1569),
    [sym_exp_value] = ACTIONS(1569),
    [sym_log_value] = ACTIONS(1569),
    [sym_log10_value] = ACTIONS(1569),
    [sym_power_value] = ACTIONS(1569),
    [sym_sqrt_value] = ACTIONS(1569),
    [sym_raw_be] = ACTIONS(1569),
    [sym_raw_le] = ACTIONS(1569),
    [sym_internal_queue_size_set] = ACTIONS(1569),
    [sym_internal_queue_size_get] = ACTIONS(1569),
    [sym_internal_threshold_set] = ACTIONS(1569),
    [sym_internal_threshold_get] = ACTIONS(1569),
    [sym_rebuilt_event] = ACTIONS(1569),
    [sym_do_custom_event] = ACTIONS(1569),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1569),
    [sym_clkmgr_set_clk_source] = ACTIONS(1569),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1569),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1569),
    [sym_clkmgr_get_timezone] = ACTIONS(1569),
    [sym_clkmgr_set_timezone] = ACTIONS(1569),
    [sym_clkmgr_get_resync_period] = ACTIONS(1569),
    [sym_clkmgr_set_resync_period] = ACTIONS(1569),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1569),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1569),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1569),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1569),
    [sym_clkmgr_get_timeservers] = ACTIONS(1569),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1569),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1569),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1569),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1569),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1569),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1569),
    [sym_set_log_level] = ACTIONS(1569),
    [sym_get_log_level] = ACTIONS(1569),
    [sym_amx_log] = ACTIONS(1569),
    [sym_smtp_server_config_set] = ACTIONS(1569),
    [sym_smtp_server_config_get] = ACTIONS(1569),
    [sym_smtp_send] = ACTIONS(1569),
    [sym_validate_netlinx_account] = ACTIONS(1569),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1569),
    [sym_authenticate_certificate] = ACTIONS(1569),
    [sym_audit_netlinx_session_event] = ACTIONS(1569),
    [sym_audit_netlinx_generic_event] = ACTIONS(1569),
    [sym_get_available_flash_disk_space] = ACTIONS(1569),
    [sym_get_max_flash_disk_space] = ACTIONS(1569),
    [sym_data_create_feed] = ACTIONS(1569),
    [sym_data_delete_feed] = ACTIONS(1569),
    [sym_data_publish_feed] = ACTIONS(1569),
    [sym_data_get_published_feed] = ACTIONS(1569),
    [sym_data_add_record] = ACTIONS(1569),
    [sym_data_get_event_record] = ACTIONS(1569),
    [sym__wc_data_create_feed] = ACTIONS(1569),
    [sym__wc_data_add_record] = ACTIONS(1569),
    [sym__wc_data_get_event_record] = ACTIONS(1569),
    [sym_format] = ACTIONS(1569),
    [sym_astro_clock] = ACTIONS(1569),
    [sym_set_outdoor_temperature] = ACTIONS(1569),
    [sym_set_virtual_level_count] = ACTIONS(1569),
    [sym_set_virtual_channel_count] = ACTIONS(1569),
    [sym_set_virtual_port_count] = ACTIONS(1569),
    [sym__wc_encode_int] = ACTIONS(1569),
    [sym__wc_decode_int] = ACTIONS(1569),
    [sym__wc_int] = ACTIONS(1569),
    [sym__wc_to_ch_int] = ACTIONS(1569),
    [sym__ch_to_wc_int] = ACTIONS(1569),
    [sym__wc_find_string_int] = ACTIONS(1569),
    [sym__wc_left_string_int] = ACTIONS(1569),
    [sym__wc_lower_string_int] = ACTIONS(1569),
    [sym__wc_mid_string_int] = ACTIONS(1569),
    [sym__wc_remove_string_int] = ACTIONS(1569),
    [sym__wc_right_string_int] = ACTIONS(1569),
    [sym__wc_upper_string_int] = ACTIONS(1569),
    [sym__wc_compare_string_int] = ACTIONS(1569),
    [sym__wc_get_buffer_char_int] = ACTIONS(1569),
    [sym__wc_get_buffer_string_int] = ACTIONS(1569),
    [sym__wc_concat_string_int] = ACTIONS(1569),
    [sym__wc_file_open_int] = ACTIONS(1569),
    [sym__wc_file_close_int] = ACTIONS(1569),
    [sym__wc_file_read_int] = ACTIONS(1569),
    [sym__wc_file_read_line_int] = ACTIONS(1569),
    [sym__wc_file_write_int] = ACTIONS(1569),
    [sym__wc_file_write_line_int] = ACTIONS(1569),
    [aux_sym_preproc_include_token1] = ACTIONS(1569),
    [aux_sym_preproc_define_token1] = ACTIONS(1571),
    [aux_sym_preproc_warn_token1] = ACTIONS(1571),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1571),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1571),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1571),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1571),
    [aux_sym_preproc_else_token1] = ACTIONS(1571),
    [anon_sym_LBRACK] = ACTIONS(1571),
    [anon_sym_LBRACE] = ACTIONS(1571),
    [anon_sym_COLON] = ACTIONS(1571),
    [anon_sym_LPAREN] = ACTIONS(1569),
    [anon_sym_SEMI] = ACTIONS(1571),
    [anon_sym_BANG] = ACTIONS(1569),
    [anon_sym_TILDE] = ACTIONS(1571),
    [anon_sym_DASH] = ACTIONS(1569),
    [anon_sym_PLUS] = ACTIONS(1569),
    [anon_sym_STAR] = ACTIONS(1571),
    [anon_sym_SLASH] = ACTIONS(1569),
    [anon_sym_PERCENT] = ACTIONS(1571),
    [anon_sym_PIPE_PIPE] = ACTIONS(1571),
    [anon_sym_AMP_AMP] = ACTIONS(1571),
    [anon_sym_PIPE] = ACTIONS(1569),
    [anon_sym_CARET] = ACTIONS(1571),
    [anon_sym_AMP] = ACTIONS(1569),
    [anon_sym_EQ_EQ] = ACTIONS(1571),
    [anon_sym_BANG_EQ] = ACTIONS(1571),
    [anon_sym_LT_GT] = ACTIONS(1571),
    [anon_sym_GT] = ACTIONS(1569),
    [anon_sym_GT_EQ] = ACTIONS(1571),
    [anon_sym_LT_EQ] = ACTIONS(1571),
    [anon_sym_LT] = ACTIONS(1569),
    [anon_sym_LT_LT] = ACTIONS(1571),
    [anon_sym_GT_GT] = ACTIONS(1571),
    [anon_sym_DASH_DASH] = ACTIONS(1571),
    [anon_sym_PLUS_PLUS] = ACTIONS(1571),
    [anon_sym_DOT] = ACTIONS(1569),
    [anon_sym_DQUOTE] = ACTIONS(1571),
    [anon_sym_SQUOTE] = ACTIONS(1571),
    [sym_number_literal] = ACTIONS(1571),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1571),
  },
  [STATE(124)] = {
    [sym_identifier] = ACTIONS(1573),
    [sym_program_name_keyword] = ACTIONS(1573),
    [sym_module_name_keyword] = ACTIONS(1573),
    [sym_define_device_keyword] = ACTIONS(1573),
    [sym_define_combine_keyword] = ACTIONS(1573),
    [sym_define_constant_keyword] = ACTIONS(1573),
    [sym_define_type_keyword] = ACTIONS(1573),
    [sym_define_variable_keyword] = ACTIONS(1573),
    [sym_define_system_variable_keyword] = ACTIONS(1573),
    [sym_define_connect_level_keyword] = ACTIONS(1573),
    [sym_define_latching_keyword] = ACTIONS(1573),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1573),
    [sym_define_toggling_keyword] = ACTIONS(1573),
    [sym_define_call_keyword] = ACTIONS(1573),
    [sym_define_function_keyword] = ACTIONS(1573),
    [sym_define_library_function_keyword] = ACTIONS(1573),
    [sym_define_start_keyword] = ACTIONS(1573),
    [sym_define_event_keyword] = ACTIONS(1573),
    [sym_define_program_keyword] = ACTIONS(1573),
    [sym_define_module_keyword] = ACTIONS(1573),
    [sym_char_keyword] = ACTIONS(1573),
    [sym_widechar_keyword] = ACTIONS(1573),
    [sym_integer_keyword] = ACTIONS(1573),
    [sym_sinteger_keyword] = ACTIONS(1573),
    [sym_long_keyword] = ACTIONS(1573),
    [sym_slong_keyword] = ACTIONS(1573),
    [sym_float_keyword] = ACTIONS(1573),
    [sym_double_keyword] = ACTIONS(1573),
    [sym_variant_keyword] = ACTIONS(1573),
    [sym_variantarray_keyword] = ACTIONS(1573),
    [sym_local_var_keyword] = ACTIONS(1573),
    [sym_stack_var_keyword] = ACTIONS(1573),
    [sym_constant_keyword] = ACTIONS(1573),
    [sym_volatile_keyword] = ACTIONS(1573),
    [sym_non_volatile_keyword] = ACTIONS(1573),
    [sym_persistent_keyword] = ACTIONS(1573),
    [sym_dev_keyword] = ACTIONS(1573),
    [sym_devlev_keyword] = ACTIONS(1573),
    [sym_devchan_keyword] = ACTIONS(1573),
    [sym_if_keyword] = ACTIONS(1573),
    [sym_else_keyword] = ACTIONS(1573),
    [sym_switch_keyword] = ACTIONS(1573),
    [sym_case_keyword] = ACTIONS(1573),
    [sym_default_keyword] = ACTIONS(1573),
    [sym_while_keyword] = ACTIONS(1573),
    [sym_for_keyword] = ACTIONS(1573),
    [sym_break_keyword] = ACTIONS(1573),
    [sym_continue_keyword] = ACTIONS(1573),
    [sym_return_keyword] = ACTIONS(1573),
    [sym_select_keyword] = ACTIONS(1573),
    [sym_active_keyword] = ACTIONS(1573),
    [sym_struct_keyword] = ACTIONS(1573),
    [sym_structure_keyword] = ACTIONS(1573),
    [sym_button_event_keyword] = ACTIONS(1573),
    [sym_channel_event_keyword] = ACTIONS(1573),
    [sym_level_event_keyword] = ACTIONS(1573),
    [sym_data_event_keyword] = ACTIONS(1573),
    [sym_timeline_event_keyword] = ACTIONS(1573),
    [sym_custom_event_keyword] = ACTIONS(1573),
    [sym_push_keyword] = ACTIONS(1573),
    [sym_release_keyword] = ACTIONS(1573),
    [aux_sym_on_keyword_token1] = ACTIONS(1573),
    [aux_sym_off_keyword_token1] = ACTIONS(1573),
    [sym_send_level_keyword] = ACTIONS(1573),
    [sym_send_string_keyword] = ACTIONS(1573),
    [sym_send_command_keyword] = ACTIONS(1573),
    [sym_clear_buffer_keyword] = ACTIONS(1573),
    [sym_create_buffer_keyword] = ACTIONS(1573),
    [sym_create_multi_buffer_keyword] = ACTIONS(1573),
    [sym_call_keyword] = ACTIONS(1573),
    [sym_system_call_keyword] = ACTIONS(1573),
    [sym_devchan_to_keyword] = ACTIONS(1573),
    [sym_devchan_min_to_keyword] = ACTIONS(1573),
    [sym_devchan_total_off_keyword] = ACTIONS(1573),
    [sym_devchan_pulse_keyword] = ACTIONS(1573),
    [sym_wait_keyword] = ACTIONS(1573),
    [sym_cancel_wait_keyword] = ACTIONS(1573),
    [sym_wait_until_keyword] = ACTIONS(1573),
    [sym_cancel_wait_until_keyword] = ACTIONS(1573),
    [sym_cancel_all_wait_keyword] = ACTIONS(1573),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1573),
    [sym_and] = ACTIONS(1573),
    [sym_or] = ACTIONS(1573),
    [sym_not] = ACTIONS(1573),
    [sym_xor] = ACTIONS(1573),
    [sym_band] = ACTIONS(1573),
    [sym_bor] = ACTIONS(1573),
    [sym_bxor] = ACTIONS(1573),
    [sym_bnot] = ACTIONS(1573),
    [sym_lshift] = ACTIONS(1573),
    [sym_rshift] = ACTIONS(1573),
    [sym___file__] = ACTIONS(1573),
    [sym___date__] = ACTIONS(1573),
    [sym___ldate__] = ACTIONS(1573),
    [sym___time__] = ACTIONS(1573),
    [sym___line__] = ACTIONS(1573),
    [sym___name__] = ACTIONS(1573),
    [sym___netlinx__] = ACTIONS(1573),
    [sym_netlinx_axi_version] = ACTIONS(1573),
    [sym_url_flg_tcp] = ACTIONS(1573),
    [sym_url_flg_accinfopresent] = ACTIONS(1573),
    [sym_url_flg_temp] = ACTIONS(1573),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1573),
    [sym_url_flg_stat_mask] = ACTIONS(1573),
    [sym_url_flg_stat_lookup] = ACTIONS(1573),
    [sym_url_flg_stat_connecting] = ACTIONS(1573),
    [sym_url_flg_stat_waiting] = ACTIONS(1573),
    [sym_url_flg_stat_connected] = ACTIONS(1573),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1573),
    [sym_first_virtual_device] = ACTIONS(1573),
    [sym_first_local_port] = ACTIONS(1573),
    [sym_dynamic_virtual_device] = ACTIONS(1573),
    [sym_timeline_once] = ACTIONS(1573),
    [sym_timeline_repeat] = ACTIONS(1573),
    [sym_timeline_absolute] = ACTIONS(1573),
    [sym_timeline_relative] = ACTIONS(1573),
    [sym_do_push_timed_infinite] = ACTIONS(1573),
    [sym_source_type_no_address] = ACTIONS(1573),
    [sym_source_type_neuron_id] = ACTIONS(1573),
    [sym_source_type_ip_address] = ACTIONS(1573),
    [sym_source_type_axlink] = ACTIONS(1573),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1573),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1573),
    [sym_source_type_ip_socket_address] = ACTIONS(1573),
    [sym_source_type_rs232] = ACTIONS(1573),
    [sym_source_type_internal] = ACTIONS(1573),
    [sym_source_type_mac_address] = ACTIONS(1573),
    [sym_source_type_ipv4_port] = ACTIONS(1573),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1573),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1573),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1573),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1573),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1573),
    [sym_true] = ACTIONS(1573),
    [sym_false] = ACTIONS(1573),
    [sym_file_read_only] = ACTIONS(1573),
    [sym_file_rw_new] = ACTIONS(1573),
    [sym_file_rw_append] = ACTIONS(1573),
    [sym_ip_tcp] = ACTIONS(1573),
    [sym_ip_udp] = ACTIONS(1573),
    [sym_ip_udp_2way] = ACTIONS(1573),
    [sym_xml_encode_types] = ACTIONS(1573),
    [sym_xml_encode_char_as_list] = ACTIONS(1573),
    [sym_xml_encode_le] = ACTIONS(1573),
    [sym_xml_decode_types] = ACTIONS(1573),
    [sym_xml_decode_no_preserve] = ACTIONS(1573),
    [sym_duet_dev_type_display_device] = ACTIONS(1573),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1573),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1573),
    [sym_duet_dev_type_io_device] = ACTIONS(1573),
    [sym_duet_dev_type_relay_device] = ACTIONS(1573),
    [sym_duet_dev_type_ups] = ACTIONS(1573),
    [sym_duet_dev_type_amplifier] = ACTIONS(1573),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1573),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1573),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1573),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1573),
    [sym_duet_dev_type_camera] = ACTIONS(1573),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1573),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1573),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1573),
    [sym_duet_dev_type_dss] = ACTIONS(1573),
    [sym_duet_dev_type_dvr] = ACTIONS(1573),
    [sym_duet_dev_type_disc_device] = ACTIONS(1573),
    [sym_duet_dev_type_document_camera] = ACTIONS(1573),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1573),
    [sym_duet_dev_type_hvac] = ACTIONS(1573),
    [sym_duet_dev_type_keypad] = ACTIONS(1573),
    [sym_duet_dev_type_light] = ACTIONS(1573),
    [sym_duet_dev_type_monitor] = ACTIONS(1573),
    [sym_duet_dev_type_motor] = ACTIONS(1573),
    [sym_duet_dev_type_multi_window] = ACTIONS(1573),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1573),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1573),
    [sym_duet_dev_type_receiver] = ACTIONS(1573),
    [sym_duet_dev_type_security_system] = ACTIONS(1573),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1573),
    [sym_duet_dev_type_settop_box] = ACTIONS(1573),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1573),
    [sym_duet_dev_type_switcher] = ACTIONS(1573),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1573),
    [sym_duet_dev_type_tv] = ACTIONS(1573),
    [sym_duet_dev_type_utility] = ACTIONS(1573),
    [sym_duet_dev_type_vcr] = ACTIONS(1573),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1573),
    [sym_duet_dev_type_video_processor] = ACTIONS(1573),
    [sym_duet_dev_type_video_projector] = ACTIONS(1573),
    [sym_duet_dev_type_video_wall] = ACTIONS(1573),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1573),
    [sym_duet_dev_type_weather] = ACTIONS(1573),
    [sym_duet_dev_not_polled] = ACTIONS(1573),
    [sym_duet_dev_polled] = ACTIONS(1573),
    [sym_clkmgr_mode_network] = ACTIONS(1573),
    [sym_clkmgr_mode_standalone] = ACTIONS(1573),
    [sym_normal_standby] = ACTIONS(1573),
    [sym_normal_wake] = ACTIONS(1573),
    [sym_ip_multicast_ttl_option] = ACTIONS(1573),
    [sym_ip_ttl_subnet] = ACTIONS(1573),
    [sym_ip_ttl_site] = ACTIONS(1573),
    [sym_ip_ttl_region] = ACTIONS(1573),
    [sym_ip_ttl_continent] = ACTIONS(1573),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1573),
    [sym_ip_nodelay_on] = ACTIONS(1573),
    [sym_ip_nodelay_off] = ACTIONS(1573),
    [sym_amx_error] = ACTIONS(1573),
    [sym_amx_warning] = ACTIONS(1573),
    [sym_amx_info] = ACTIONS(1573),
    [sym_amx_debug] = ACTIONS(1573),
    [sym_netlinx_login_success] = ACTIONS(1573),
    [sym_netlinx_login_fail] = ACTIONS(1573),
    [sym_netlinx_logout] = ACTIONS(1573),
    [sym_null_str] = ACTIONS(1573),
    [sym_smtp_address] = ACTIONS(1573),
    [sym_smtp_port_number] = ACTIONS(1573),
    [sym_smtp_username] = ACTIONS(1573),
    [sym_smtp_password] = ACTIONS(1573),
    [sym_smtp_require_tls] = ACTIONS(1573),
    [sym_smtp_from] = ACTIONS(1573),
    [sym_smtp_tls_true] = ACTIONS(1573),
    [sym_smtp_tls_false] = ACTIONS(1573),
    [sym_listview_on_row_select_event] = ACTIONS(1573),
    [sym_data_structure_unknown] = ACTIONS(1573),
    [sym_data_structure_datafeed] = ACTIONS(1573),
    [sym_data_structure_datarecordset] = ACTIONS(1573),
    [sym_data_structure_datarecord] = ACTIONS(1573),
    [sym_data_structure_datafield] = ACTIONS(1573),
    [sym_data_max_content_field_count] = ACTIONS(1573),
    [sym_data_max_metadata_field_count] = ACTIONS(1573),
    [sym_data_max_id_length] = ACTIONS(1573),
    [sym_data_max_name_length] = ACTIONS(1573),
    [sym_data_max_description_length] = ACTIONS(1573),
    [sym_data_max_type_length] = ACTIONS(1573),
    [sym_data_max_format_length] = ACTIONS(1573),
    [sym_data_max_source_length] = ACTIONS(1573),
    [sym_data_max_value_length] = ACTIONS(1573),
    [sym_data_max_label_length] = ACTIONS(1573),
    [sym_data_max_url_length] = ACTIONS(1573),
    [sym_max_last_login_info_length] = ACTIONS(1573),
    [sym_data_type_unknown] = ACTIONS(1573),
    [sym_data_type_string] = ACTIONS(1573),
    [sym_data_type_datetime] = ACTIONS(1573),
    [sym_data_type_date] = ACTIONS(1573),
    [sym_data_type_time] = ACTIONS(1573),
    [sym_data_type_image] = ACTIONS(1573),
    [sym_data_format_url] = ACTIONS(1573),
    [sym_data_format_phone] = ACTIONS(1573),
    [sym_data_format_email] = ACTIONS(1573),
    [sym_data_format_iso8601] = ACTIONS(1573),
    [sym_auditlog_priv] = ACTIONS(1573),
    [sym_configuration_priv] = ACTIONS(1573),
    [sym_deviceconfig_priv] = ACTIONS(1573),
    [sym_ftp_priv] = ACTIONS(1573),
    [sym_http_priv] = ACTIONS(1573),
    [sym_networkconfig_priv] = ACTIONS(1573),
    [sym_programport_priv] = ACTIONS(1573),
    [sym_remoteui_priv] = ACTIONS(1573),
    [sym_securitycontrol_priv] = ACTIONS(1573),
    [sym_softwaremanagement_priv] = ACTIONS(1573),
    [sym_terminal_priv] = ACTIONS(1573),
    [sym_tpadmin_priv] = ACTIONS(1573),
    [sym_user1_priv] = ACTIONS(1573),
    [sym_user2_priv] = ACTIONS(1573),
    [sym_user3_priv] = ACTIONS(1573),
    [sym_user4_priv] = ACTIONS(1573),
    [sym_usermanagement_priv] = ACTIONS(1573),
    [sym_valid_account] = ACTIONS(1573),
    [sym_err_name_invalid] = ACTIONS(1573),
    [sym_err_password_invalid] = ACTIONS(1573),
    [sym_err_invalid_account] = ACTIONS(1573),
    [sym_err_no_authorization] = ACTIONS(1573),
    [sym_err_invalid_parameter] = ACTIONS(1573),
    [sym_err_user_locked_out] = ACTIONS(1573),
    [sym_err_user_account_expired] = ACTIONS(1573),
    [sym_err_general_error] = ACTIONS(1573),
    [sym_err_authentication_error] = ACTIONS(1573),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1573),
    [sym_tls_validate_certificate] = ACTIONS(1573),
    [sym_tdata] = ACTIONS(1573),
    [sym_tchannel] = ACTIONS(1573),
    [sym_tlevel] = ACTIONS(1573),
    [sym_tbutton] = ACTIONS(1573),
    [sym_ttimeline] = ACTIONS(1573),
    [sym_tcustom] = ACTIONS(1573),
    [sym_url_struct] = ACTIONS(1573),
    [sym_dns_struct] = ACTIONS(1573),
    [sym_ip_address_struct] = ACTIONS(1573),
    [sym_dev_info_struct] = ACTIONS(1573),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1573),
    [sym_data_feed] = ACTIONS(1573),
    [sym_data_field] = ACTIONS(1573),
    [sym_data_record] = ACTIONS(1573),
    [sym_wc_data_feed] = ACTIONS(1573),
    [sym_wc_data_field] = ACTIONS(1573),
    [sym_wc_data_record] = ACTIONS(1573),
    [sym_last_login_info] = ACTIONS(1573),
    [sym____reserved___] = ACTIONS(1573),
    [sym_date] = ACTIONS(1573),
    [sym_day] = ACTIONS(1573),
    [sym_time] = ACTIONS(1573),
    [sym_ldate] = ACTIONS(1573),
    [sym_push_device] = ACTIONS(1573),
    [sym_push_channel] = ACTIONS(1573),
    [sym_push_devchan] = ACTIONS(1573),
    [sym_release_device] = ACTIONS(1573),
    [sym_release_channel] = ACTIONS(1573),
    [sym_release_devchan] = ACTIONS(1573),
    [sym_master_slot] = ACTIONS(1573),
    [sym_get_pulse_time] = ACTIONS(1573),
    [sym_get_timer] = ACTIONS(1573),
    [sym_master_sn] = ACTIONS(1573),
    [sym_system_number] = ACTIONS(1573),
    [sym_button] = ACTIONS(1573),
    [sym_dv_channel] = ACTIONS(1573),
    [sym_data] = ACTIONS(1573),
    [sym_level] = ACTIONS(1573),
    [sym_timeline] = ACTIONS(1573),
    [sym_channel] = ACTIONS(1573),
    [sym_custom] = ACTIONS(1573),
    [sym_atoi] = ACTIONS(1573),
    [sym_atol] = ACTIONS(1573),
    [sym_atof] = ACTIONS(1573),
    [sym_itoa] = ACTIONS(1573),
    [sym_itohex] = ACTIONS(1573),
    [sym_hextoi] = ACTIONS(1573),
    [sym_ftoa] = ACTIONS(1573),
    [sym_find_string] = ACTIONS(1573),
    [sym_left_string] = ACTIONS(1573),
    [sym_right_string] = ACTIONS(1573),
    [sym_mid_string] = ACTIONS(1573),
    [sym_remove_string] = ACTIONS(1573),
    [sym_get_buffer_string] = ACTIONS(1573),
    [sym_get_buffer_char] = ACTIONS(1573),
    [sym_get_multi_buffer_string] = ACTIONS(1573),
    [sym_length_string] = ACTIONS(1573),
    [sym_max_length_string] = ACTIONS(1573),
    [sym_set_length_string] = ACTIONS(1573),
    [sym_compare_string] = ACTIONS(1573),
    [sym_upper_string] = ACTIONS(1573),
    [sym_lower_string] = ACTIONS(1573),
    [sym_get_last] = ACTIONS(1573),
    [sym_date_to_day] = ACTIONS(1573),
    [sym_date_to_month] = ACTIONS(1573),
    [sym_date_to_year] = ACTIONS(1573),
    [sym_time_to_hour] = ACTIONS(1573),
    [sym_time_to_minute] = ACTIONS(1573),
    [sym_time_to_second] = ACTIONS(1573),
    [sym_day_of_week] = ACTIONS(1573),
    [sym_set_pulse_time] = ACTIONS(1573),
    [sym_set_timer] = ACTIONS(1573),
    [sym_file_open] = ACTIONS(1573),
    [sym_file_close] = ACTIONS(1573),
    [sym_file_read] = ACTIONS(1573),
    [sym_file_read_line] = ACTIONS(1573),
    [sym_file_write] = ACTIONS(1573),
    [sym_file_write_line] = ACTIONS(1573),
    [sym_file_copy] = ACTIONS(1573),
    [sym_file_delete] = ACTIONS(1573),
    [sym_file_rename] = ACTIONS(1573),
    [sym_file_dir] = ACTIONS(1573),
    [sym_file_getdir] = ACTIONS(1573),
    [sym_file_setdir] = ACTIONS(1573),
    [sym_file_create_dir] = ACTIONS(1573),
    [sym_file_remove_dir] = ACTIONS(1573),
    [sym_file_seek] = ACTIONS(1573),
    [sym_length_array] = ACTIONS(1573),
    [sym_max_length_array] = ACTIONS(1573),
    [sym_set_length_array] = ACTIONS(1573),
    [sym_ip_client_open] = ACTIONS(1573),
    [sym_ip_client_close] = ACTIONS(1573),
    [sym_ip_server_open] = ACTIONS(1573),
    [sym_ip_server_close] = ACTIONS(1573),
    [sym_ip_bound_client_open] = ACTIONS(1573),
    [sym_ip_mc_server_open] = ACTIONS(1573),
    [sym_ip_set_option] = ACTIONS(1573),
    [sym_ssh_client_open] = ACTIONS(1573),
    [sym_ssh_client_close] = ACTIONS(1573),
    [sym_tls_client_open] = ACTIONS(1573),
    [sym_tls_client_close] = ACTIONS(1573),
    [sym_do_push] = ACTIONS(1573),
    [sym_do_push_timed] = ACTIONS(1573),
    [sym_do_release] = ACTIONS(1573),
    [sym_device_id] = ACTIONS(1573),
    [sym_device_info] = ACTIONS(1573),
    [sym_device_id_string] = ACTIONS(1573),
    [sym_device_standby] = ACTIONS(1573),
    [sym_device_wake] = ACTIONS(1573),
    [sym_timeline_create] = ACTIONS(1573),
    [sym_timeline_kill] = ACTIONS(1573),
    [sym_timeline_reload] = ACTIONS(1573),
    [sym_timeline_pause] = ACTIONS(1573),
    [sym_timeline_restart] = ACTIONS(1573),
    [sym_timeline_set] = ACTIONS(1573),
    [sym_timeline_get] = ACTIONS(1573),
    [sym_timeline_active] = ACTIONS(1573),
    [sym_timeline_dynamic_id] = ACTIONS(1573),
    [sym_variable_to_string] = ACTIONS(1573),
    [sym_string_to_variable] = ACTIONS(1573),
    [sym_length_variable_to_string] = ACTIONS(1573),
    [sym_type_cast] = ACTIONS(1573),
    [sym_variable_to_xml] = ACTIONS(1573),
    [sym_xml_to_variable] = ACTIONS(1573),
    [sym_length_variable_to_xml] = ACTIONS(1573),
    [sym_redirect_string] = ACTIONS(1573),
    [sym_random_number] = ACTIONS(1573),
    [sym_get_unique_id] = ACTIONS(1573),
    [sym_get_system_number] = ACTIONS(1573),
    [sym_set_system_number] = ACTIONS(1573),
    [sym_get_serial_number] = ACTIONS(1573),
    [sym_get_master_build] = ACTIONS(1573),
    [sym_set_validation_code] = ACTIONS(1573),
    [sym_reboot] = ACTIONS(1573),
    [sym_load_duet_module] = ACTIONS(1573),
    [sym_unload_duet_module] = ACTIONS(1573),
    [sym_duet_mem_size_set] = ACTIONS(1573),
    [sym_duet_mem_size_get] = ACTIONS(1573),
    [sym_dynamic_application_device] = ACTIONS(1573),
    [sym_dynamic_polled_port] = ACTIONS(1573),
    [sym_static_port_binding] = ACTIONS(1573),
    [sym_static_ip_binding] = ACTIONS(1573),
    [sym_get_dns_list] = ACTIONS(1573),
    [sym_set_dns_list] = ACTIONS(1573),
    [sym_get_ip_address] = ACTIONS(1573),
    [sym_set_ip_address] = ACTIONS(1573),
    [sym_get_url_list] = ACTIONS(1573),
    [sym_add_url_entry] = ACTIONS(1573),
    [sym_delete_url_entry] = ACTIONS(1573),
    [sym_abs_value] = ACTIONS(1573),
    [sym_max_value] = ACTIONS(1573),
    [sym_min_value] = ACTIONS(1573),
    [sym_exp_value] = ACTIONS(1573),
    [sym_log_value] = ACTIONS(1573),
    [sym_log10_value] = ACTIONS(1573),
    [sym_power_value] = ACTIONS(1573),
    [sym_sqrt_value] = ACTIONS(1573),
    [sym_raw_be] = ACTIONS(1573),
    [sym_raw_le] = ACTIONS(1573),
    [sym_internal_queue_size_set] = ACTIONS(1573),
    [sym_internal_queue_size_get] = ACTIONS(1573),
    [sym_internal_threshold_set] = ACTIONS(1573),
    [sym_internal_threshold_get] = ACTIONS(1573),
    [sym_rebuilt_event] = ACTIONS(1573),
    [sym_do_custom_event] = ACTIONS(1573),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1573),
    [sym_clkmgr_set_clk_source] = ACTIONS(1573),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1573),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1573),
    [sym_clkmgr_get_timezone] = ACTIONS(1573),
    [sym_clkmgr_set_timezone] = ACTIONS(1573),
    [sym_clkmgr_get_resync_period] = ACTIONS(1573),
    [sym_clkmgr_set_resync_period] = ACTIONS(1573),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1573),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1573),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1573),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1573),
    [sym_clkmgr_get_timeservers] = ACTIONS(1573),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1573),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1573),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1573),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1573),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1573),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1573),
    [sym_set_log_level] = ACTIONS(1573),
    [sym_get_log_level] = ACTIONS(1573),
    [sym_amx_log] = ACTIONS(1573),
    [sym_smtp_server_config_set] = ACTIONS(1573),
    [sym_smtp_server_config_get] = ACTIONS(1573),
    [sym_smtp_send] = ACTIONS(1573),
    [sym_validate_netlinx_account] = ACTIONS(1573),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1573),
    [sym_authenticate_certificate] = ACTIONS(1573),
    [sym_audit_netlinx_session_event] = ACTIONS(1573),
    [sym_audit_netlinx_generic_event] = ACTIONS(1573),
    [sym_get_available_flash_disk_space] = ACTIONS(1573),
    [sym_get_max_flash_disk_space] = ACTIONS(1573),
    [sym_data_create_feed] = ACTIONS(1573),
    [sym_data_delete_feed] = ACTIONS(1573),
    [sym_data_publish_feed] = ACTIONS(1573),
    [sym_data_get_published_feed] = ACTIONS(1573),
    [sym_data_add_record] = ACTIONS(1573),
    [sym_data_get_event_record] = ACTIONS(1573),
    [sym__wc_data_create_feed] = ACTIONS(1573),
    [sym__wc_data_add_record] = ACTIONS(1573),
    [sym__wc_data_get_event_record] = ACTIONS(1573),
    [sym_format] = ACTIONS(1573),
    [sym_astro_clock] = ACTIONS(1573),
    [sym_set_outdoor_temperature] = ACTIONS(1573),
    [sym_set_virtual_level_count] = ACTIONS(1573),
    [sym_set_virtual_channel_count] = ACTIONS(1573),
    [sym_set_virtual_port_count] = ACTIONS(1573),
    [sym__wc_encode_int] = ACTIONS(1573),
    [sym__wc_decode_int] = ACTIONS(1573),
    [sym__wc_int] = ACTIONS(1573),
    [sym__wc_to_ch_int] = ACTIONS(1573),
    [sym__ch_to_wc_int] = ACTIONS(1573),
    [sym__wc_find_string_int] = ACTIONS(1573),
    [sym__wc_left_string_int] = ACTIONS(1573),
    [sym__wc_lower_string_int] = ACTIONS(1573),
    [sym__wc_mid_string_int] = ACTIONS(1573),
    [sym__wc_remove_string_int] = ACTIONS(1573),
    [sym__wc_right_string_int] = ACTIONS(1573),
    [sym__wc_upper_string_int] = ACTIONS(1573),
    [sym__wc_compare_string_int] = ACTIONS(1573),
    [sym__wc_get_buffer_char_int] = ACTIONS(1573),
    [sym__wc_get_buffer_string_int] = ACTIONS(1573),
    [sym__wc_concat_string_int] = ACTIONS(1573),
    [sym__wc_file_open_int] = ACTIONS(1573),
    [sym__wc_file_close_int] = ACTIONS(1573),
    [sym__wc_file_read_int] = ACTIONS(1573),
    [sym__wc_file_read_line_int] = ACTIONS(1573),
    [sym__wc_file_write_int] = ACTIONS(1573),
    [sym__wc_file_write_line_int] = ACTIONS(1573),
    [aux_sym_preproc_include_token1] = ACTIONS(1573),
    [aux_sym_preproc_define_token1] = ACTIONS(1575),
    [aux_sym_preproc_warn_token1] = ACTIONS(1575),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1575),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1575),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1575),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1575),
    [aux_sym_preproc_else_token1] = ACTIONS(1575),
    [anon_sym_LBRACK] = ACTIONS(1575),
    [anon_sym_LBRACE] = ACTIONS(1575),
    [anon_sym_COLON] = ACTIONS(1575),
    [anon_sym_LPAREN] = ACTIONS(1573),
    [anon_sym_SEMI] = ACTIONS(1575),
    [anon_sym_BANG] = ACTIONS(1573),
    [anon_sym_TILDE] = ACTIONS(1575),
    [anon_sym_DASH] = ACTIONS(1573),
    [anon_sym_PLUS] = ACTIONS(1573),
    [anon_sym_STAR] = ACTIONS(1575),
    [anon_sym_SLASH] = ACTIONS(1573),
    [anon_sym_PERCENT] = ACTIONS(1575),
    [anon_sym_PIPE_PIPE] = ACTIONS(1575),
    [anon_sym_AMP_AMP] = ACTIONS(1575),
    [anon_sym_PIPE] = ACTIONS(1573),
    [anon_sym_CARET] = ACTIONS(1575),
    [anon_sym_AMP] = ACTIONS(1573),
    [anon_sym_EQ_EQ] = ACTIONS(1575),
    [anon_sym_BANG_EQ] = ACTIONS(1575),
    [anon_sym_LT_GT] = ACTIONS(1575),
    [anon_sym_GT] = ACTIONS(1573),
    [anon_sym_GT_EQ] = ACTIONS(1575),
    [anon_sym_LT_EQ] = ACTIONS(1575),
    [anon_sym_LT] = ACTIONS(1573),
    [anon_sym_LT_LT] = ACTIONS(1575),
    [anon_sym_GT_GT] = ACTIONS(1575),
    [anon_sym_DASH_DASH] = ACTIONS(1575),
    [anon_sym_PLUS_PLUS] = ACTIONS(1575),
    [anon_sym_DOT] = ACTIONS(1573),
    [anon_sym_DQUOTE] = ACTIONS(1575),
    [anon_sym_SQUOTE] = ACTIONS(1575),
    [sym_number_literal] = ACTIONS(1575),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1575),
  },
  [STATE(125)] = {
    [sym_identifier] = ACTIONS(1577),
    [sym_program_name_keyword] = ACTIONS(1577),
    [sym_module_name_keyword] = ACTIONS(1577),
    [sym_define_device_keyword] = ACTIONS(1577),
    [sym_define_combine_keyword] = ACTIONS(1577),
    [sym_define_constant_keyword] = ACTIONS(1577),
    [sym_define_type_keyword] = ACTIONS(1577),
    [sym_define_variable_keyword] = ACTIONS(1577),
    [sym_define_system_variable_keyword] = ACTIONS(1577),
    [sym_define_connect_level_keyword] = ACTIONS(1577),
    [sym_define_latching_keyword] = ACTIONS(1577),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1577),
    [sym_define_toggling_keyword] = ACTIONS(1577),
    [sym_define_call_keyword] = ACTIONS(1577),
    [sym_define_function_keyword] = ACTIONS(1577),
    [sym_define_library_function_keyword] = ACTIONS(1577),
    [sym_define_start_keyword] = ACTIONS(1577),
    [sym_define_event_keyword] = ACTIONS(1577),
    [sym_define_program_keyword] = ACTIONS(1577),
    [sym_define_module_keyword] = ACTIONS(1577),
    [sym_char_keyword] = ACTIONS(1577),
    [sym_widechar_keyword] = ACTIONS(1577),
    [sym_integer_keyword] = ACTIONS(1577),
    [sym_sinteger_keyword] = ACTIONS(1577),
    [sym_long_keyword] = ACTIONS(1577),
    [sym_slong_keyword] = ACTIONS(1577),
    [sym_float_keyword] = ACTIONS(1577),
    [sym_double_keyword] = ACTIONS(1577),
    [sym_variant_keyword] = ACTIONS(1577),
    [sym_variantarray_keyword] = ACTIONS(1577),
    [sym_local_var_keyword] = ACTIONS(1577),
    [sym_stack_var_keyword] = ACTIONS(1577),
    [sym_constant_keyword] = ACTIONS(1577),
    [sym_volatile_keyword] = ACTIONS(1577),
    [sym_non_volatile_keyword] = ACTIONS(1577),
    [sym_persistent_keyword] = ACTIONS(1577),
    [sym_dev_keyword] = ACTIONS(1577),
    [sym_devlev_keyword] = ACTIONS(1577),
    [sym_devchan_keyword] = ACTIONS(1577),
    [sym_if_keyword] = ACTIONS(1577),
    [sym_else_keyword] = ACTIONS(1577),
    [sym_switch_keyword] = ACTIONS(1577),
    [sym_case_keyword] = ACTIONS(1577),
    [sym_default_keyword] = ACTIONS(1577),
    [sym_while_keyword] = ACTIONS(1577),
    [sym_for_keyword] = ACTIONS(1577),
    [sym_break_keyword] = ACTIONS(1577),
    [sym_continue_keyword] = ACTIONS(1577),
    [sym_return_keyword] = ACTIONS(1577),
    [sym_select_keyword] = ACTIONS(1577),
    [sym_active_keyword] = ACTIONS(1577),
    [sym_struct_keyword] = ACTIONS(1577),
    [sym_structure_keyword] = ACTIONS(1577),
    [sym_button_event_keyword] = ACTIONS(1577),
    [sym_channel_event_keyword] = ACTIONS(1577),
    [sym_level_event_keyword] = ACTIONS(1577),
    [sym_data_event_keyword] = ACTIONS(1577),
    [sym_timeline_event_keyword] = ACTIONS(1577),
    [sym_custom_event_keyword] = ACTIONS(1577),
    [sym_push_keyword] = ACTIONS(1577),
    [sym_release_keyword] = ACTIONS(1577),
    [aux_sym_on_keyword_token1] = ACTIONS(1577),
    [aux_sym_off_keyword_token1] = ACTIONS(1577),
    [sym_send_level_keyword] = ACTIONS(1577),
    [sym_send_string_keyword] = ACTIONS(1577),
    [sym_send_command_keyword] = ACTIONS(1577),
    [sym_clear_buffer_keyword] = ACTIONS(1577),
    [sym_create_buffer_keyword] = ACTIONS(1577),
    [sym_create_multi_buffer_keyword] = ACTIONS(1577),
    [sym_call_keyword] = ACTIONS(1577),
    [sym_system_call_keyword] = ACTIONS(1577),
    [sym_devchan_to_keyword] = ACTIONS(1577),
    [sym_devchan_min_to_keyword] = ACTIONS(1577),
    [sym_devchan_total_off_keyword] = ACTIONS(1577),
    [sym_devchan_pulse_keyword] = ACTIONS(1577),
    [sym_wait_keyword] = ACTIONS(1577),
    [sym_cancel_wait_keyword] = ACTIONS(1577),
    [sym_wait_until_keyword] = ACTIONS(1577),
    [sym_cancel_wait_until_keyword] = ACTIONS(1577),
    [sym_cancel_all_wait_keyword] = ACTIONS(1577),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1577),
    [sym_and] = ACTIONS(1577),
    [sym_or] = ACTIONS(1577),
    [sym_not] = ACTIONS(1577),
    [sym_xor] = ACTIONS(1577),
    [sym_band] = ACTIONS(1577),
    [sym_bor] = ACTIONS(1577),
    [sym_bxor] = ACTIONS(1577),
    [sym_bnot] = ACTIONS(1577),
    [sym_lshift] = ACTIONS(1577),
    [sym_rshift] = ACTIONS(1577),
    [sym___file__] = ACTIONS(1577),
    [sym___date__] = ACTIONS(1577),
    [sym___ldate__] = ACTIONS(1577),
    [sym___time__] = ACTIONS(1577),
    [sym___line__] = ACTIONS(1577),
    [sym___name__] = ACTIONS(1577),
    [sym___netlinx__] = ACTIONS(1577),
    [sym_netlinx_axi_version] = ACTIONS(1577),
    [sym_url_flg_tcp] = ACTIONS(1577),
    [sym_url_flg_accinfopresent] = ACTIONS(1577),
    [sym_url_flg_temp] = ACTIONS(1577),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1577),
    [sym_url_flg_stat_mask] = ACTIONS(1577),
    [sym_url_flg_stat_lookup] = ACTIONS(1577),
    [sym_url_flg_stat_connecting] = ACTIONS(1577),
    [sym_url_flg_stat_waiting] = ACTIONS(1577),
    [sym_url_flg_stat_connected] = ACTIONS(1577),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1577),
    [sym_first_virtual_device] = ACTIONS(1577),
    [sym_first_local_port] = ACTIONS(1577),
    [sym_dynamic_virtual_device] = ACTIONS(1577),
    [sym_timeline_once] = ACTIONS(1577),
    [sym_timeline_repeat] = ACTIONS(1577),
    [sym_timeline_absolute] = ACTIONS(1577),
    [sym_timeline_relative] = ACTIONS(1577),
    [sym_do_push_timed_infinite] = ACTIONS(1577),
    [sym_source_type_no_address] = ACTIONS(1577),
    [sym_source_type_neuron_id] = ACTIONS(1577),
    [sym_source_type_ip_address] = ACTIONS(1577),
    [sym_source_type_axlink] = ACTIONS(1577),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1577),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1577),
    [sym_source_type_ip_socket_address] = ACTIONS(1577),
    [sym_source_type_rs232] = ACTIONS(1577),
    [sym_source_type_internal] = ACTIONS(1577),
    [sym_source_type_mac_address] = ACTIONS(1577),
    [sym_source_type_ipv4_port] = ACTIONS(1577),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1577),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1577),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1577),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1577),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1577),
    [sym_true] = ACTIONS(1577),
    [sym_false] = ACTIONS(1577),
    [sym_file_read_only] = ACTIONS(1577),
    [sym_file_rw_new] = ACTIONS(1577),
    [sym_file_rw_append] = ACTIONS(1577),
    [sym_ip_tcp] = ACTIONS(1577),
    [sym_ip_udp] = ACTIONS(1577),
    [sym_ip_udp_2way] = ACTIONS(1577),
    [sym_xml_encode_types] = ACTIONS(1577),
    [sym_xml_encode_char_as_list] = ACTIONS(1577),
    [sym_xml_encode_le] = ACTIONS(1577),
    [sym_xml_decode_types] = ACTIONS(1577),
    [sym_xml_decode_no_preserve] = ACTIONS(1577),
    [sym_duet_dev_type_display_device] = ACTIONS(1577),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1577),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1577),
    [sym_duet_dev_type_io_device] = ACTIONS(1577),
    [sym_duet_dev_type_relay_device] = ACTIONS(1577),
    [sym_duet_dev_type_ups] = ACTIONS(1577),
    [sym_duet_dev_type_amplifier] = ACTIONS(1577),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1577),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1577),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1577),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1577),
    [sym_duet_dev_type_camera] = ACTIONS(1577),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1577),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1577),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1577),
    [sym_duet_dev_type_dss] = ACTIONS(1577),
    [sym_duet_dev_type_dvr] = ACTIONS(1577),
    [sym_duet_dev_type_disc_device] = ACTIONS(1577),
    [sym_duet_dev_type_document_camera] = ACTIONS(1577),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1577),
    [sym_duet_dev_type_hvac] = ACTIONS(1577),
    [sym_duet_dev_type_keypad] = ACTIONS(1577),
    [sym_duet_dev_type_light] = ACTIONS(1577),
    [sym_duet_dev_type_monitor] = ACTIONS(1577),
    [sym_duet_dev_type_motor] = ACTIONS(1577),
    [sym_duet_dev_type_multi_window] = ACTIONS(1577),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1577),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1577),
    [sym_duet_dev_type_receiver] = ACTIONS(1577),
    [sym_duet_dev_type_security_system] = ACTIONS(1577),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1577),
    [sym_duet_dev_type_settop_box] = ACTIONS(1577),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1577),
    [sym_duet_dev_type_switcher] = ACTIONS(1577),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1577),
    [sym_duet_dev_type_tv] = ACTIONS(1577),
    [sym_duet_dev_type_utility] = ACTIONS(1577),
    [sym_duet_dev_type_vcr] = ACTIONS(1577),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1577),
    [sym_duet_dev_type_video_processor] = ACTIONS(1577),
    [sym_duet_dev_type_video_projector] = ACTIONS(1577),
    [sym_duet_dev_type_video_wall] = ACTIONS(1577),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1577),
    [sym_duet_dev_type_weather] = ACTIONS(1577),
    [sym_duet_dev_not_polled] = ACTIONS(1577),
    [sym_duet_dev_polled] = ACTIONS(1577),
    [sym_clkmgr_mode_network] = ACTIONS(1577),
    [sym_clkmgr_mode_standalone] = ACTIONS(1577),
    [sym_normal_standby] = ACTIONS(1577),
    [sym_normal_wake] = ACTIONS(1577),
    [sym_ip_multicast_ttl_option] = ACTIONS(1577),
    [sym_ip_ttl_subnet] = ACTIONS(1577),
    [sym_ip_ttl_site] = ACTIONS(1577),
    [sym_ip_ttl_region] = ACTIONS(1577),
    [sym_ip_ttl_continent] = ACTIONS(1577),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1577),
    [sym_ip_nodelay_on] = ACTIONS(1577),
    [sym_ip_nodelay_off] = ACTIONS(1577),
    [sym_amx_error] = ACTIONS(1577),
    [sym_amx_warning] = ACTIONS(1577),
    [sym_amx_info] = ACTIONS(1577),
    [sym_amx_debug] = ACTIONS(1577),
    [sym_netlinx_login_success] = ACTIONS(1577),
    [sym_netlinx_login_fail] = ACTIONS(1577),
    [sym_netlinx_logout] = ACTIONS(1577),
    [sym_null_str] = ACTIONS(1577),
    [sym_smtp_address] = ACTIONS(1577),
    [sym_smtp_port_number] = ACTIONS(1577),
    [sym_smtp_username] = ACTIONS(1577),
    [sym_smtp_password] = ACTIONS(1577),
    [sym_smtp_require_tls] = ACTIONS(1577),
    [sym_smtp_from] = ACTIONS(1577),
    [sym_smtp_tls_true] = ACTIONS(1577),
    [sym_smtp_tls_false] = ACTIONS(1577),
    [sym_listview_on_row_select_event] = ACTIONS(1577),
    [sym_data_structure_unknown] = ACTIONS(1577),
    [sym_data_structure_datafeed] = ACTIONS(1577),
    [sym_data_structure_datarecordset] = ACTIONS(1577),
    [sym_data_structure_datarecord] = ACTIONS(1577),
    [sym_data_structure_datafield] = ACTIONS(1577),
    [sym_data_max_content_field_count] = ACTIONS(1577),
    [sym_data_max_metadata_field_count] = ACTIONS(1577),
    [sym_data_max_id_length] = ACTIONS(1577),
    [sym_data_max_name_length] = ACTIONS(1577),
    [sym_data_max_description_length] = ACTIONS(1577),
    [sym_data_max_type_length] = ACTIONS(1577),
    [sym_data_max_format_length] = ACTIONS(1577),
    [sym_data_max_source_length] = ACTIONS(1577),
    [sym_data_max_value_length] = ACTIONS(1577),
    [sym_data_max_label_length] = ACTIONS(1577),
    [sym_data_max_url_length] = ACTIONS(1577),
    [sym_max_last_login_info_length] = ACTIONS(1577),
    [sym_data_type_unknown] = ACTIONS(1577),
    [sym_data_type_string] = ACTIONS(1577),
    [sym_data_type_datetime] = ACTIONS(1577),
    [sym_data_type_date] = ACTIONS(1577),
    [sym_data_type_time] = ACTIONS(1577),
    [sym_data_type_image] = ACTIONS(1577),
    [sym_data_format_url] = ACTIONS(1577),
    [sym_data_format_phone] = ACTIONS(1577),
    [sym_data_format_email] = ACTIONS(1577),
    [sym_data_format_iso8601] = ACTIONS(1577),
    [sym_auditlog_priv] = ACTIONS(1577),
    [sym_configuration_priv] = ACTIONS(1577),
    [sym_deviceconfig_priv] = ACTIONS(1577),
    [sym_ftp_priv] = ACTIONS(1577),
    [sym_http_priv] = ACTIONS(1577),
    [sym_networkconfig_priv] = ACTIONS(1577),
    [sym_programport_priv] = ACTIONS(1577),
    [sym_remoteui_priv] = ACTIONS(1577),
    [sym_securitycontrol_priv] = ACTIONS(1577),
    [sym_softwaremanagement_priv] = ACTIONS(1577),
    [sym_terminal_priv] = ACTIONS(1577),
    [sym_tpadmin_priv] = ACTIONS(1577),
    [sym_user1_priv] = ACTIONS(1577),
    [sym_user2_priv] = ACTIONS(1577),
    [sym_user3_priv] = ACTIONS(1577),
    [sym_user4_priv] = ACTIONS(1577),
    [sym_usermanagement_priv] = ACTIONS(1577),
    [sym_valid_account] = ACTIONS(1577),
    [sym_err_name_invalid] = ACTIONS(1577),
    [sym_err_password_invalid] = ACTIONS(1577),
    [sym_err_invalid_account] = ACTIONS(1577),
    [sym_err_no_authorization] = ACTIONS(1577),
    [sym_err_invalid_parameter] = ACTIONS(1577),
    [sym_err_user_locked_out] = ACTIONS(1577),
    [sym_err_user_account_expired] = ACTIONS(1577),
    [sym_err_general_error] = ACTIONS(1577),
    [sym_err_authentication_error] = ACTIONS(1577),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1577),
    [sym_tls_validate_certificate] = ACTIONS(1577),
    [sym_tdata] = ACTIONS(1577),
    [sym_tchannel] = ACTIONS(1577),
    [sym_tlevel] = ACTIONS(1577),
    [sym_tbutton] = ACTIONS(1577),
    [sym_ttimeline] = ACTIONS(1577),
    [sym_tcustom] = ACTIONS(1577),
    [sym_url_struct] = ACTIONS(1577),
    [sym_dns_struct] = ACTIONS(1577),
    [sym_ip_address_struct] = ACTIONS(1577),
    [sym_dev_info_struct] = ACTIONS(1577),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1577),
    [sym_data_feed] = ACTIONS(1577),
    [sym_data_field] = ACTIONS(1577),
    [sym_data_record] = ACTIONS(1577),
    [sym_wc_data_feed] = ACTIONS(1577),
    [sym_wc_data_field] = ACTIONS(1577),
    [sym_wc_data_record] = ACTIONS(1577),
    [sym_last_login_info] = ACTIONS(1577),
    [sym____reserved___] = ACTIONS(1577),
    [sym_date] = ACTIONS(1577),
    [sym_day] = ACTIONS(1577),
    [sym_time] = ACTIONS(1577),
    [sym_ldate] = ACTIONS(1577),
    [sym_push_device] = ACTIONS(1577),
    [sym_push_channel] = ACTIONS(1577),
    [sym_push_devchan] = ACTIONS(1577),
    [sym_release_device] = ACTIONS(1577),
    [sym_release_channel] = ACTIONS(1577),
    [sym_release_devchan] = ACTIONS(1577),
    [sym_master_slot] = ACTIONS(1577),
    [sym_get_pulse_time] = ACTIONS(1577),
    [sym_get_timer] = ACTIONS(1577),
    [sym_master_sn] = ACTIONS(1577),
    [sym_system_number] = ACTIONS(1577),
    [sym_button] = ACTIONS(1577),
    [sym_dv_channel] = ACTIONS(1577),
    [sym_data] = ACTIONS(1577),
    [sym_level] = ACTIONS(1577),
    [sym_timeline] = ACTIONS(1577),
    [sym_channel] = ACTIONS(1577),
    [sym_custom] = ACTIONS(1577),
    [sym_atoi] = ACTIONS(1577),
    [sym_atol] = ACTIONS(1577),
    [sym_atof] = ACTIONS(1577),
    [sym_itoa] = ACTIONS(1577),
    [sym_itohex] = ACTIONS(1577),
    [sym_hextoi] = ACTIONS(1577),
    [sym_ftoa] = ACTIONS(1577),
    [sym_find_string] = ACTIONS(1577),
    [sym_left_string] = ACTIONS(1577),
    [sym_right_string] = ACTIONS(1577),
    [sym_mid_string] = ACTIONS(1577),
    [sym_remove_string] = ACTIONS(1577),
    [sym_get_buffer_string] = ACTIONS(1577),
    [sym_get_buffer_char] = ACTIONS(1577),
    [sym_get_multi_buffer_string] = ACTIONS(1577),
    [sym_length_string] = ACTIONS(1577),
    [sym_max_length_string] = ACTIONS(1577),
    [sym_set_length_string] = ACTIONS(1577),
    [sym_compare_string] = ACTIONS(1577),
    [sym_upper_string] = ACTIONS(1577),
    [sym_lower_string] = ACTIONS(1577),
    [sym_get_last] = ACTIONS(1577),
    [sym_date_to_day] = ACTIONS(1577),
    [sym_date_to_month] = ACTIONS(1577),
    [sym_date_to_year] = ACTIONS(1577),
    [sym_time_to_hour] = ACTIONS(1577),
    [sym_time_to_minute] = ACTIONS(1577),
    [sym_time_to_second] = ACTIONS(1577),
    [sym_day_of_week] = ACTIONS(1577),
    [sym_set_pulse_time] = ACTIONS(1577),
    [sym_set_timer] = ACTIONS(1577),
    [sym_file_open] = ACTIONS(1577),
    [sym_file_close] = ACTIONS(1577),
    [sym_file_read] = ACTIONS(1577),
    [sym_file_read_line] = ACTIONS(1577),
    [sym_file_write] = ACTIONS(1577),
    [sym_file_write_line] = ACTIONS(1577),
    [sym_file_copy] = ACTIONS(1577),
    [sym_file_delete] = ACTIONS(1577),
    [sym_file_rename] = ACTIONS(1577),
    [sym_file_dir] = ACTIONS(1577),
    [sym_file_getdir] = ACTIONS(1577),
    [sym_file_setdir] = ACTIONS(1577),
    [sym_file_create_dir] = ACTIONS(1577),
    [sym_file_remove_dir] = ACTIONS(1577),
    [sym_file_seek] = ACTIONS(1577),
    [sym_length_array] = ACTIONS(1577),
    [sym_max_length_array] = ACTIONS(1577),
    [sym_set_length_array] = ACTIONS(1577),
    [sym_ip_client_open] = ACTIONS(1577),
    [sym_ip_client_close] = ACTIONS(1577),
    [sym_ip_server_open] = ACTIONS(1577),
    [sym_ip_server_close] = ACTIONS(1577),
    [sym_ip_bound_client_open] = ACTIONS(1577),
    [sym_ip_mc_server_open] = ACTIONS(1577),
    [sym_ip_set_option] = ACTIONS(1577),
    [sym_ssh_client_open] = ACTIONS(1577),
    [sym_ssh_client_close] = ACTIONS(1577),
    [sym_tls_client_open] = ACTIONS(1577),
    [sym_tls_client_close] = ACTIONS(1577),
    [sym_do_push] = ACTIONS(1577),
    [sym_do_push_timed] = ACTIONS(1577),
    [sym_do_release] = ACTIONS(1577),
    [sym_device_id] = ACTIONS(1577),
    [sym_device_info] = ACTIONS(1577),
    [sym_device_id_string] = ACTIONS(1577),
    [sym_device_standby] = ACTIONS(1577),
    [sym_device_wake] = ACTIONS(1577),
    [sym_timeline_create] = ACTIONS(1577),
    [sym_timeline_kill] = ACTIONS(1577),
    [sym_timeline_reload] = ACTIONS(1577),
    [sym_timeline_pause] = ACTIONS(1577),
    [sym_timeline_restart] = ACTIONS(1577),
    [sym_timeline_set] = ACTIONS(1577),
    [sym_timeline_get] = ACTIONS(1577),
    [sym_timeline_active] = ACTIONS(1577),
    [sym_timeline_dynamic_id] = ACTIONS(1577),
    [sym_variable_to_string] = ACTIONS(1577),
    [sym_string_to_variable] = ACTIONS(1577),
    [sym_length_variable_to_string] = ACTIONS(1577),
    [sym_type_cast] = ACTIONS(1577),
    [sym_variable_to_xml] = ACTIONS(1577),
    [sym_xml_to_variable] = ACTIONS(1577),
    [sym_length_variable_to_xml] = ACTIONS(1577),
    [sym_redirect_string] = ACTIONS(1577),
    [sym_random_number] = ACTIONS(1577),
    [sym_get_unique_id] = ACTIONS(1577),
    [sym_get_system_number] = ACTIONS(1577),
    [sym_set_system_number] = ACTIONS(1577),
    [sym_get_serial_number] = ACTIONS(1577),
    [sym_get_master_build] = ACTIONS(1577),
    [sym_set_validation_code] = ACTIONS(1577),
    [sym_reboot] = ACTIONS(1577),
    [sym_load_duet_module] = ACTIONS(1577),
    [sym_unload_duet_module] = ACTIONS(1577),
    [sym_duet_mem_size_set] = ACTIONS(1577),
    [sym_duet_mem_size_get] = ACTIONS(1577),
    [sym_dynamic_application_device] = ACTIONS(1577),
    [sym_dynamic_polled_port] = ACTIONS(1577),
    [sym_static_port_binding] = ACTIONS(1577),
    [sym_static_ip_binding] = ACTIONS(1577),
    [sym_get_dns_list] = ACTIONS(1577),
    [sym_set_dns_list] = ACTIONS(1577),
    [sym_get_ip_address] = ACTIONS(1577),
    [sym_set_ip_address] = ACTIONS(1577),
    [sym_get_url_list] = ACTIONS(1577),
    [sym_add_url_entry] = ACTIONS(1577),
    [sym_delete_url_entry] = ACTIONS(1577),
    [sym_abs_value] = ACTIONS(1577),
    [sym_max_value] = ACTIONS(1577),
    [sym_min_value] = ACTIONS(1577),
    [sym_exp_value] = ACTIONS(1577),
    [sym_log_value] = ACTIONS(1577),
    [sym_log10_value] = ACTIONS(1577),
    [sym_power_value] = ACTIONS(1577),
    [sym_sqrt_value] = ACTIONS(1577),
    [sym_raw_be] = ACTIONS(1577),
    [sym_raw_le] = ACTIONS(1577),
    [sym_internal_queue_size_set] = ACTIONS(1577),
    [sym_internal_queue_size_get] = ACTIONS(1577),
    [sym_internal_threshold_set] = ACTIONS(1577),
    [sym_internal_threshold_get] = ACTIONS(1577),
    [sym_rebuilt_event] = ACTIONS(1577),
    [sym_do_custom_event] = ACTIONS(1577),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1577),
    [sym_clkmgr_set_clk_source] = ACTIONS(1577),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1577),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1577),
    [sym_clkmgr_get_timezone] = ACTIONS(1577),
    [sym_clkmgr_set_timezone] = ACTIONS(1577),
    [sym_clkmgr_get_resync_period] = ACTIONS(1577),
    [sym_clkmgr_set_resync_period] = ACTIONS(1577),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1577),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1577),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1577),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1577),
    [sym_clkmgr_get_timeservers] = ACTIONS(1577),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1577),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1577),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1577),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1577),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1577),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1577),
    [sym_set_log_level] = ACTIONS(1577),
    [sym_get_log_level] = ACTIONS(1577),
    [sym_amx_log] = ACTIONS(1577),
    [sym_smtp_server_config_set] = ACTIONS(1577),
    [sym_smtp_server_config_get] = ACTIONS(1577),
    [sym_smtp_send] = ACTIONS(1577),
    [sym_validate_netlinx_account] = ACTIONS(1577),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1577),
    [sym_authenticate_certificate] = ACTIONS(1577),
    [sym_audit_netlinx_session_event] = ACTIONS(1577),
    [sym_audit_netlinx_generic_event] = ACTIONS(1577),
    [sym_get_available_flash_disk_space] = ACTIONS(1577),
    [sym_get_max_flash_disk_space] = ACTIONS(1577),
    [sym_data_create_feed] = ACTIONS(1577),
    [sym_data_delete_feed] = ACTIONS(1577),
    [sym_data_publish_feed] = ACTIONS(1577),
    [sym_data_get_published_feed] = ACTIONS(1577),
    [sym_data_add_record] = ACTIONS(1577),
    [sym_data_get_event_record] = ACTIONS(1577),
    [sym__wc_data_create_feed] = ACTIONS(1577),
    [sym__wc_data_add_record] = ACTIONS(1577),
    [sym__wc_data_get_event_record] = ACTIONS(1577),
    [sym_format] = ACTIONS(1577),
    [sym_astro_clock] = ACTIONS(1577),
    [sym_set_outdoor_temperature] = ACTIONS(1577),
    [sym_set_virtual_level_count] = ACTIONS(1577),
    [sym_set_virtual_channel_count] = ACTIONS(1577),
    [sym_set_virtual_port_count] = ACTIONS(1577),
    [sym__wc_encode_int] = ACTIONS(1577),
    [sym__wc_decode_int] = ACTIONS(1577),
    [sym__wc_int] = ACTIONS(1577),
    [sym__wc_to_ch_int] = ACTIONS(1577),
    [sym__ch_to_wc_int] = ACTIONS(1577),
    [sym__wc_find_string_int] = ACTIONS(1577),
    [sym__wc_left_string_int] = ACTIONS(1577),
    [sym__wc_lower_string_int] = ACTIONS(1577),
    [sym__wc_mid_string_int] = ACTIONS(1577),
    [sym__wc_remove_string_int] = ACTIONS(1577),
    [sym__wc_right_string_int] = ACTIONS(1577),
    [sym__wc_upper_string_int] = ACTIONS(1577),
    [sym__wc_compare_string_int] = ACTIONS(1577),
    [sym__wc_get_buffer_char_int] = ACTIONS(1577),
    [sym__wc_get_buffer_string_int] = ACTIONS(1577),
    [sym__wc_concat_string_int] = ACTIONS(1577),
    [sym__wc_file_open_int] = ACTIONS(1577),
    [sym__wc_file_close_int] = ACTIONS(1577),
    [sym__wc_file_read_int] = ACTIONS(1577),
    [sym__wc_file_read_line_int] = ACTIONS(1577),
    [sym__wc_file_write_int] = ACTIONS(1577),
    [sym__wc_file_write_line_int] = ACTIONS(1577),
    [aux_sym_preproc_include_token1] = ACTIONS(1577),
    [aux_sym_preproc_define_token1] = ACTIONS(1579),
    [aux_sym_preproc_warn_token1] = ACTIONS(1579),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1579),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1579),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1579),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1579),
    [aux_sym_preproc_else_token1] = ACTIONS(1579),
    [anon_sym_LBRACK] = ACTIONS(1579),
    [anon_sym_LBRACE] = ACTIONS(1579),
    [anon_sym_COLON] = ACTIONS(1579),
    [anon_sym_LPAREN] = ACTIONS(1577),
    [anon_sym_SEMI] = ACTIONS(1579),
    [anon_sym_BANG] = ACTIONS(1577),
    [anon_sym_TILDE] = ACTIONS(1579),
    [anon_sym_DASH] = ACTIONS(1577),
    [anon_sym_PLUS] = ACTIONS(1577),
    [anon_sym_STAR] = ACTIONS(1579),
    [anon_sym_SLASH] = ACTIONS(1577),
    [anon_sym_PERCENT] = ACTIONS(1579),
    [anon_sym_PIPE_PIPE] = ACTIONS(1579),
    [anon_sym_AMP_AMP] = ACTIONS(1579),
    [anon_sym_PIPE] = ACTIONS(1577),
    [anon_sym_CARET] = ACTIONS(1579),
    [anon_sym_AMP] = ACTIONS(1577),
    [anon_sym_EQ_EQ] = ACTIONS(1579),
    [anon_sym_BANG_EQ] = ACTIONS(1579),
    [anon_sym_LT_GT] = ACTIONS(1579),
    [anon_sym_GT] = ACTIONS(1577),
    [anon_sym_GT_EQ] = ACTIONS(1579),
    [anon_sym_LT_EQ] = ACTIONS(1579),
    [anon_sym_LT] = ACTIONS(1577),
    [anon_sym_LT_LT] = ACTIONS(1579),
    [anon_sym_GT_GT] = ACTIONS(1579),
    [anon_sym_DASH_DASH] = ACTIONS(1579),
    [anon_sym_PLUS_PLUS] = ACTIONS(1579),
    [anon_sym_DOT] = ACTIONS(1577),
    [anon_sym_DQUOTE] = ACTIONS(1579),
    [anon_sym_SQUOTE] = ACTIONS(1579),
    [sym_number_literal] = ACTIONS(1579),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1579),
  },
  [STATE(126)] = {
    [sym_identifier] = ACTIONS(1581),
    [sym_program_name_keyword] = ACTIONS(1581),
    [sym_module_name_keyword] = ACTIONS(1581),
    [sym_define_device_keyword] = ACTIONS(1581),
    [sym_define_combine_keyword] = ACTIONS(1581),
    [sym_define_constant_keyword] = ACTIONS(1581),
    [sym_define_type_keyword] = ACTIONS(1581),
    [sym_define_variable_keyword] = ACTIONS(1581),
    [sym_define_system_variable_keyword] = ACTIONS(1581),
    [sym_define_connect_level_keyword] = ACTIONS(1581),
    [sym_define_latching_keyword] = ACTIONS(1581),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1581),
    [sym_define_toggling_keyword] = ACTIONS(1581),
    [sym_define_call_keyword] = ACTIONS(1581),
    [sym_define_function_keyword] = ACTIONS(1581),
    [sym_define_library_function_keyword] = ACTIONS(1581),
    [sym_define_start_keyword] = ACTIONS(1581),
    [sym_define_event_keyword] = ACTIONS(1581),
    [sym_define_program_keyword] = ACTIONS(1581),
    [sym_define_module_keyword] = ACTIONS(1581),
    [sym_char_keyword] = ACTIONS(1581),
    [sym_widechar_keyword] = ACTIONS(1581),
    [sym_integer_keyword] = ACTIONS(1581),
    [sym_sinteger_keyword] = ACTIONS(1581),
    [sym_long_keyword] = ACTIONS(1581),
    [sym_slong_keyword] = ACTIONS(1581),
    [sym_float_keyword] = ACTIONS(1581),
    [sym_double_keyword] = ACTIONS(1581),
    [sym_variant_keyword] = ACTIONS(1581),
    [sym_variantarray_keyword] = ACTIONS(1581),
    [sym_local_var_keyword] = ACTIONS(1581),
    [sym_stack_var_keyword] = ACTIONS(1581),
    [sym_constant_keyword] = ACTIONS(1581),
    [sym_volatile_keyword] = ACTIONS(1581),
    [sym_non_volatile_keyword] = ACTIONS(1581),
    [sym_persistent_keyword] = ACTIONS(1581),
    [sym_dev_keyword] = ACTIONS(1581),
    [sym_devlev_keyword] = ACTIONS(1581),
    [sym_devchan_keyword] = ACTIONS(1581),
    [sym_if_keyword] = ACTIONS(1581),
    [sym_else_keyword] = ACTIONS(1581),
    [sym_switch_keyword] = ACTIONS(1581),
    [sym_case_keyword] = ACTIONS(1581),
    [sym_default_keyword] = ACTIONS(1581),
    [sym_while_keyword] = ACTIONS(1581),
    [sym_for_keyword] = ACTIONS(1581),
    [sym_break_keyword] = ACTIONS(1581),
    [sym_continue_keyword] = ACTIONS(1581),
    [sym_return_keyword] = ACTIONS(1581),
    [sym_select_keyword] = ACTIONS(1581),
    [sym_active_keyword] = ACTIONS(1581),
    [sym_struct_keyword] = ACTIONS(1581),
    [sym_structure_keyword] = ACTIONS(1581),
    [sym_button_event_keyword] = ACTIONS(1581),
    [sym_channel_event_keyword] = ACTIONS(1581),
    [sym_level_event_keyword] = ACTIONS(1581),
    [sym_data_event_keyword] = ACTIONS(1581),
    [sym_timeline_event_keyword] = ACTIONS(1581),
    [sym_custom_event_keyword] = ACTIONS(1581),
    [sym_push_keyword] = ACTIONS(1581),
    [sym_release_keyword] = ACTIONS(1581),
    [aux_sym_on_keyword_token1] = ACTIONS(1581),
    [aux_sym_off_keyword_token1] = ACTIONS(1581),
    [sym_send_level_keyword] = ACTIONS(1581),
    [sym_send_string_keyword] = ACTIONS(1581),
    [sym_send_command_keyword] = ACTIONS(1581),
    [sym_clear_buffer_keyword] = ACTIONS(1581),
    [sym_create_buffer_keyword] = ACTIONS(1581),
    [sym_create_multi_buffer_keyword] = ACTIONS(1581),
    [sym_call_keyword] = ACTIONS(1581),
    [sym_system_call_keyword] = ACTIONS(1581),
    [sym_devchan_to_keyword] = ACTIONS(1581),
    [sym_devchan_min_to_keyword] = ACTIONS(1581),
    [sym_devchan_total_off_keyword] = ACTIONS(1581),
    [sym_devchan_pulse_keyword] = ACTIONS(1581),
    [sym_wait_keyword] = ACTIONS(1581),
    [sym_cancel_wait_keyword] = ACTIONS(1581),
    [sym_wait_until_keyword] = ACTIONS(1581),
    [sym_cancel_wait_until_keyword] = ACTIONS(1581),
    [sym_cancel_all_wait_keyword] = ACTIONS(1581),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1581),
    [sym_and] = ACTIONS(1581),
    [sym_or] = ACTIONS(1581),
    [sym_not] = ACTIONS(1581),
    [sym_xor] = ACTIONS(1581),
    [sym_band] = ACTIONS(1581),
    [sym_bor] = ACTIONS(1581),
    [sym_bxor] = ACTIONS(1581),
    [sym_bnot] = ACTIONS(1581),
    [sym_lshift] = ACTIONS(1581),
    [sym_rshift] = ACTIONS(1581),
    [sym___file__] = ACTIONS(1581),
    [sym___date__] = ACTIONS(1581),
    [sym___ldate__] = ACTIONS(1581),
    [sym___time__] = ACTIONS(1581),
    [sym___line__] = ACTIONS(1581),
    [sym___name__] = ACTIONS(1581),
    [sym___netlinx__] = ACTIONS(1581),
    [sym_netlinx_axi_version] = ACTIONS(1581),
    [sym_url_flg_tcp] = ACTIONS(1581),
    [sym_url_flg_accinfopresent] = ACTIONS(1581),
    [sym_url_flg_temp] = ACTIONS(1581),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1581),
    [sym_url_flg_stat_mask] = ACTIONS(1581),
    [sym_url_flg_stat_lookup] = ACTIONS(1581),
    [sym_url_flg_stat_connecting] = ACTIONS(1581),
    [sym_url_flg_stat_waiting] = ACTIONS(1581),
    [sym_url_flg_stat_connected] = ACTIONS(1581),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1581),
    [sym_first_virtual_device] = ACTIONS(1581),
    [sym_first_local_port] = ACTIONS(1581),
    [sym_dynamic_virtual_device] = ACTIONS(1581),
    [sym_timeline_once] = ACTIONS(1581),
    [sym_timeline_repeat] = ACTIONS(1581),
    [sym_timeline_absolute] = ACTIONS(1581),
    [sym_timeline_relative] = ACTIONS(1581),
    [sym_do_push_timed_infinite] = ACTIONS(1581),
    [sym_source_type_no_address] = ACTIONS(1581),
    [sym_source_type_neuron_id] = ACTIONS(1581),
    [sym_source_type_ip_address] = ACTIONS(1581),
    [sym_source_type_axlink] = ACTIONS(1581),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1581),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1581),
    [sym_source_type_ip_socket_address] = ACTIONS(1581),
    [sym_source_type_rs232] = ACTIONS(1581),
    [sym_source_type_internal] = ACTIONS(1581),
    [sym_source_type_mac_address] = ACTIONS(1581),
    [sym_source_type_ipv4_port] = ACTIONS(1581),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1581),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1581),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1581),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1581),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1581),
    [sym_true] = ACTIONS(1581),
    [sym_false] = ACTIONS(1581),
    [sym_file_read_only] = ACTIONS(1581),
    [sym_file_rw_new] = ACTIONS(1581),
    [sym_file_rw_append] = ACTIONS(1581),
    [sym_ip_tcp] = ACTIONS(1581),
    [sym_ip_udp] = ACTIONS(1581),
    [sym_ip_udp_2way] = ACTIONS(1581),
    [sym_xml_encode_types] = ACTIONS(1581),
    [sym_xml_encode_char_as_list] = ACTIONS(1581),
    [sym_xml_encode_le] = ACTIONS(1581),
    [sym_xml_decode_types] = ACTIONS(1581),
    [sym_xml_decode_no_preserve] = ACTIONS(1581),
    [sym_duet_dev_type_display_device] = ACTIONS(1581),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1581),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1581),
    [sym_duet_dev_type_io_device] = ACTIONS(1581),
    [sym_duet_dev_type_relay_device] = ACTIONS(1581),
    [sym_duet_dev_type_ups] = ACTIONS(1581),
    [sym_duet_dev_type_amplifier] = ACTIONS(1581),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1581),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1581),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1581),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1581),
    [sym_duet_dev_type_camera] = ACTIONS(1581),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1581),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1581),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1581),
    [sym_duet_dev_type_dss] = ACTIONS(1581),
    [sym_duet_dev_type_dvr] = ACTIONS(1581),
    [sym_duet_dev_type_disc_device] = ACTIONS(1581),
    [sym_duet_dev_type_document_camera] = ACTIONS(1581),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1581),
    [sym_duet_dev_type_hvac] = ACTIONS(1581),
    [sym_duet_dev_type_keypad] = ACTIONS(1581),
    [sym_duet_dev_type_light] = ACTIONS(1581),
    [sym_duet_dev_type_monitor] = ACTIONS(1581),
    [sym_duet_dev_type_motor] = ACTIONS(1581),
    [sym_duet_dev_type_multi_window] = ACTIONS(1581),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1581),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1581),
    [sym_duet_dev_type_receiver] = ACTIONS(1581),
    [sym_duet_dev_type_security_system] = ACTIONS(1581),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1581),
    [sym_duet_dev_type_settop_box] = ACTIONS(1581),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1581),
    [sym_duet_dev_type_switcher] = ACTIONS(1581),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1581),
    [sym_duet_dev_type_tv] = ACTIONS(1581),
    [sym_duet_dev_type_utility] = ACTIONS(1581),
    [sym_duet_dev_type_vcr] = ACTIONS(1581),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1581),
    [sym_duet_dev_type_video_processor] = ACTIONS(1581),
    [sym_duet_dev_type_video_projector] = ACTIONS(1581),
    [sym_duet_dev_type_video_wall] = ACTIONS(1581),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1581),
    [sym_duet_dev_type_weather] = ACTIONS(1581),
    [sym_duet_dev_not_polled] = ACTIONS(1581),
    [sym_duet_dev_polled] = ACTIONS(1581),
    [sym_clkmgr_mode_network] = ACTIONS(1581),
    [sym_clkmgr_mode_standalone] = ACTIONS(1581),
    [sym_normal_standby] = ACTIONS(1581),
    [sym_normal_wake] = ACTIONS(1581),
    [sym_ip_multicast_ttl_option] = ACTIONS(1581),
    [sym_ip_ttl_subnet] = ACTIONS(1581),
    [sym_ip_ttl_site] = ACTIONS(1581),
    [sym_ip_ttl_region] = ACTIONS(1581),
    [sym_ip_ttl_continent] = ACTIONS(1581),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1581),
    [sym_ip_nodelay_on] = ACTIONS(1581),
    [sym_ip_nodelay_off] = ACTIONS(1581),
    [sym_amx_error] = ACTIONS(1581),
    [sym_amx_warning] = ACTIONS(1581),
    [sym_amx_info] = ACTIONS(1581),
    [sym_amx_debug] = ACTIONS(1581),
    [sym_netlinx_login_success] = ACTIONS(1581),
    [sym_netlinx_login_fail] = ACTIONS(1581),
    [sym_netlinx_logout] = ACTIONS(1581),
    [sym_null_str] = ACTIONS(1581),
    [sym_smtp_address] = ACTIONS(1581),
    [sym_smtp_port_number] = ACTIONS(1581),
    [sym_smtp_username] = ACTIONS(1581),
    [sym_smtp_password] = ACTIONS(1581),
    [sym_smtp_require_tls] = ACTIONS(1581),
    [sym_smtp_from] = ACTIONS(1581),
    [sym_smtp_tls_true] = ACTIONS(1581),
    [sym_smtp_tls_false] = ACTIONS(1581),
    [sym_listview_on_row_select_event] = ACTIONS(1581),
    [sym_data_structure_unknown] = ACTIONS(1581),
    [sym_data_structure_datafeed] = ACTIONS(1581),
    [sym_data_structure_datarecordset] = ACTIONS(1581),
    [sym_data_structure_datarecord] = ACTIONS(1581),
    [sym_data_structure_datafield] = ACTIONS(1581),
    [sym_data_max_content_field_count] = ACTIONS(1581),
    [sym_data_max_metadata_field_count] = ACTIONS(1581),
    [sym_data_max_id_length] = ACTIONS(1581),
    [sym_data_max_name_length] = ACTIONS(1581),
    [sym_data_max_description_length] = ACTIONS(1581),
    [sym_data_max_type_length] = ACTIONS(1581),
    [sym_data_max_format_length] = ACTIONS(1581),
    [sym_data_max_source_length] = ACTIONS(1581),
    [sym_data_max_value_length] = ACTIONS(1581),
    [sym_data_max_label_length] = ACTIONS(1581),
    [sym_data_max_url_length] = ACTIONS(1581),
    [sym_max_last_login_info_length] = ACTIONS(1581),
    [sym_data_type_unknown] = ACTIONS(1581),
    [sym_data_type_string] = ACTIONS(1581),
    [sym_data_type_datetime] = ACTIONS(1581),
    [sym_data_type_date] = ACTIONS(1581),
    [sym_data_type_time] = ACTIONS(1581),
    [sym_data_type_image] = ACTIONS(1581),
    [sym_data_format_url] = ACTIONS(1581),
    [sym_data_format_phone] = ACTIONS(1581),
    [sym_data_format_email] = ACTIONS(1581),
    [sym_data_format_iso8601] = ACTIONS(1581),
    [sym_auditlog_priv] = ACTIONS(1581),
    [sym_configuration_priv] = ACTIONS(1581),
    [sym_deviceconfig_priv] = ACTIONS(1581),
    [sym_ftp_priv] = ACTIONS(1581),
    [sym_http_priv] = ACTIONS(1581),
    [sym_networkconfig_priv] = ACTIONS(1581),
    [sym_programport_priv] = ACTIONS(1581),
    [sym_remoteui_priv] = ACTIONS(1581),
    [sym_securitycontrol_priv] = ACTIONS(1581),
    [sym_softwaremanagement_priv] = ACTIONS(1581),
    [sym_terminal_priv] = ACTIONS(1581),
    [sym_tpadmin_priv] = ACTIONS(1581),
    [sym_user1_priv] = ACTIONS(1581),
    [sym_user2_priv] = ACTIONS(1581),
    [sym_user3_priv] = ACTIONS(1581),
    [sym_user4_priv] = ACTIONS(1581),
    [sym_usermanagement_priv] = ACTIONS(1581),
    [sym_valid_account] = ACTIONS(1581),
    [sym_err_name_invalid] = ACTIONS(1581),
    [sym_err_password_invalid] = ACTIONS(1581),
    [sym_err_invalid_account] = ACTIONS(1581),
    [sym_err_no_authorization] = ACTIONS(1581),
    [sym_err_invalid_parameter] = ACTIONS(1581),
    [sym_err_user_locked_out] = ACTIONS(1581),
    [sym_err_user_account_expired] = ACTIONS(1581),
    [sym_err_general_error] = ACTIONS(1581),
    [sym_err_authentication_error] = ACTIONS(1581),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1581),
    [sym_tls_validate_certificate] = ACTIONS(1581),
    [sym_tdata] = ACTIONS(1581),
    [sym_tchannel] = ACTIONS(1581),
    [sym_tlevel] = ACTIONS(1581),
    [sym_tbutton] = ACTIONS(1581),
    [sym_ttimeline] = ACTIONS(1581),
    [sym_tcustom] = ACTIONS(1581),
    [sym_url_struct] = ACTIONS(1581),
    [sym_dns_struct] = ACTIONS(1581),
    [sym_ip_address_struct] = ACTIONS(1581),
    [sym_dev_info_struct] = ACTIONS(1581),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1581),
    [sym_data_feed] = ACTIONS(1581),
    [sym_data_field] = ACTIONS(1581),
    [sym_data_record] = ACTIONS(1581),
    [sym_wc_data_feed] = ACTIONS(1581),
    [sym_wc_data_field] = ACTIONS(1581),
    [sym_wc_data_record] = ACTIONS(1581),
    [sym_last_login_info] = ACTIONS(1581),
    [sym____reserved___] = ACTIONS(1581),
    [sym_date] = ACTIONS(1581),
    [sym_day] = ACTIONS(1581),
    [sym_time] = ACTIONS(1581),
    [sym_ldate] = ACTIONS(1581),
    [sym_push_device] = ACTIONS(1581),
    [sym_push_channel] = ACTIONS(1581),
    [sym_push_devchan] = ACTIONS(1581),
    [sym_release_device] = ACTIONS(1581),
    [sym_release_channel] = ACTIONS(1581),
    [sym_release_devchan] = ACTIONS(1581),
    [sym_master_slot] = ACTIONS(1581),
    [sym_get_pulse_time] = ACTIONS(1581),
    [sym_get_timer] = ACTIONS(1581),
    [sym_master_sn] = ACTIONS(1581),
    [sym_system_number] = ACTIONS(1581),
    [sym_button] = ACTIONS(1581),
    [sym_dv_channel] = ACTIONS(1581),
    [sym_data] = ACTIONS(1581),
    [sym_level] = ACTIONS(1581),
    [sym_timeline] = ACTIONS(1581),
    [sym_channel] = ACTIONS(1581),
    [sym_custom] = ACTIONS(1581),
    [sym_atoi] = ACTIONS(1581),
    [sym_atol] = ACTIONS(1581),
    [sym_atof] = ACTIONS(1581),
    [sym_itoa] = ACTIONS(1581),
    [sym_itohex] = ACTIONS(1581),
    [sym_hextoi] = ACTIONS(1581),
    [sym_ftoa] = ACTIONS(1581),
    [sym_find_string] = ACTIONS(1581),
    [sym_left_string] = ACTIONS(1581),
    [sym_right_string] = ACTIONS(1581),
    [sym_mid_string] = ACTIONS(1581),
    [sym_remove_string] = ACTIONS(1581),
    [sym_get_buffer_string] = ACTIONS(1581),
    [sym_get_buffer_char] = ACTIONS(1581),
    [sym_get_multi_buffer_string] = ACTIONS(1581),
    [sym_length_string] = ACTIONS(1581),
    [sym_max_length_string] = ACTIONS(1581),
    [sym_set_length_string] = ACTIONS(1581),
    [sym_compare_string] = ACTIONS(1581),
    [sym_upper_string] = ACTIONS(1581),
    [sym_lower_string] = ACTIONS(1581),
    [sym_get_last] = ACTIONS(1581),
    [sym_date_to_day] = ACTIONS(1581),
    [sym_date_to_month] = ACTIONS(1581),
    [sym_date_to_year] = ACTIONS(1581),
    [sym_time_to_hour] = ACTIONS(1581),
    [sym_time_to_minute] = ACTIONS(1581),
    [sym_time_to_second] = ACTIONS(1581),
    [sym_day_of_week] = ACTIONS(1581),
    [sym_set_pulse_time] = ACTIONS(1581),
    [sym_set_timer] = ACTIONS(1581),
    [sym_file_open] = ACTIONS(1581),
    [sym_file_close] = ACTIONS(1581),
    [sym_file_read] = ACTIONS(1581),
    [sym_file_read_line] = ACTIONS(1581),
    [sym_file_write] = ACTIONS(1581),
    [sym_file_write_line] = ACTIONS(1581),
    [sym_file_copy] = ACTIONS(1581),
    [sym_file_delete] = ACTIONS(1581),
    [sym_file_rename] = ACTIONS(1581),
    [sym_file_dir] = ACTIONS(1581),
    [sym_file_getdir] = ACTIONS(1581),
    [sym_file_setdir] = ACTIONS(1581),
    [sym_file_create_dir] = ACTIONS(1581),
    [sym_file_remove_dir] = ACTIONS(1581),
    [sym_file_seek] = ACTIONS(1581),
    [sym_length_array] = ACTIONS(1581),
    [sym_max_length_array] = ACTIONS(1581),
    [sym_set_length_array] = ACTIONS(1581),
    [sym_ip_client_open] = ACTIONS(1581),
    [sym_ip_client_close] = ACTIONS(1581),
    [sym_ip_server_open] = ACTIONS(1581),
    [sym_ip_server_close] = ACTIONS(1581),
    [sym_ip_bound_client_open] = ACTIONS(1581),
    [sym_ip_mc_server_open] = ACTIONS(1581),
    [sym_ip_set_option] = ACTIONS(1581),
    [sym_ssh_client_open] = ACTIONS(1581),
    [sym_ssh_client_close] = ACTIONS(1581),
    [sym_tls_client_open] = ACTIONS(1581),
    [sym_tls_client_close] = ACTIONS(1581),
    [sym_do_push] = ACTIONS(1581),
    [sym_do_push_timed] = ACTIONS(1581),
    [sym_do_release] = ACTIONS(1581),
    [sym_device_id] = ACTIONS(1581),
    [sym_device_info] = ACTIONS(1581),
    [sym_device_id_string] = ACTIONS(1581),
    [sym_device_standby] = ACTIONS(1581),
    [sym_device_wake] = ACTIONS(1581),
    [sym_timeline_create] = ACTIONS(1581),
    [sym_timeline_kill] = ACTIONS(1581),
    [sym_timeline_reload] = ACTIONS(1581),
    [sym_timeline_pause] = ACTIONS(1581),
    [sym_timeline_restart] = ACTIONS(1581),
    [sym_timeline_set] = ACTIONS(1581),
    [sym_timeline_get] = ACTIONS(1581),
    [sym_timeline_active] = ACTIONS(1581),
    [sym_timeline_dynamic_id] = ACTIONS(1581),
    [sym_variable_to_string] = ACTIONS(1581),
    [sym_string_to_variable] = ACTIONS(1581),
    [sym_length_variable_to_string] = ACTIONS(1581),
    [sym_type_cast] = ACTIONS(1581),
    [sym_variable_to_xml] = ACTIONS(1581),
    [sym_xml_to_variable] = ACTIONS(1581),
    [sym_length_variable_to_xml] = ACTIONS(1581),
    [sym_redirect_string] = ACTIONS(1581),
    [sym_random_number] = ACTIONS(1581),
    [sym_get_unique_id] = ACTIONS(1581),
    [sym_get_system_number] = ACTIONS(1581),
    [sym_set_system_number] = ACTIONS(1581),
    [sym_get_serial_number] = ACTIONS(1581),
    [sym_get_master_build] = ACTIONS(1581),
    [sym_set_validation_code] = ACTIONS(1581),
    [sym_reboot] = ACTIONS(1581),
    [sym_load_duet_module] = ACTIONS(1581),
    [sym_unload_duet_module] = ACTIONS(1581),
    [sym_duet_mem_size_set] = ACTIONS(1581),
    [sym_duet_mem_size_get] = ACTIONS(1581),
    [sym_dynamic_application_device] = ACTIONS(1581),
    [sym_dynamic_polled_port] = ACTIONS(1581),
    [sym_static_port_binding] = ACTIONS(1581),
    [sym_static_ip_binding] = ACTIONS(1581),
    [sym_get_dns_list] = ACTIONS(1581),
    [sym_set_dns_list] = ACTIONS(1581),
    [sym_get_ip_address] = ACTIONS(1581),
    [sym_set_ip_address] = ACTIONS(1581),
    [sym_get_url_list] = ACTIONS(1581),
    [sym_add_url_entry] = ACTIONS(1581),
    [sym_delete_url_entry] = ACTIONS(1581),
    [sym_abs_value] = ACTIONS(1581),
    [sym_max_value] = ACTIONS(1581),
    [sym_min_value] = ACTIONS(1581),
    [sym_exp_value] = ACTIONS(1581),
    [sym_log_value] = ACTIONS(1581),
    [sym_log10_value] = ACTIONS(1581),
    [sym_power_value] = ACTIONS(1581),
    [sym_sqrt_value] = ACTIONS(1581),
    [sym_raw_be] = ACTIONS(1581),
    [sym_raw_le] = ACTIONS(1581),
    [sym_internal_queue_size_set] = ACTIONS(1581),
    [sym_internal_queue_size_get] = ACTIONS(1581),
    [sym_internal_threshold_set] = ACTIONS(1581),
    [sym_internal_threshold_get] = ACTIONS(1581),
    [sym_rebuilt_event] = ACTIONS(1581),
    [sym_do_custom_event] = ACTIONS(1581),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1581),
    [sym_clkmgr_set_clk_source] = ACTIONS(1581),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1581),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1581),
    [sym_clkmgr_get_timezone] = ACTIONS(1581),
    [sym_clkmgr_set_timezone] = ACTIONS(1581),
    [sym_clkmgr_get_resync_period] = ACTIONS(1581),
    [sym_clkmgr_set_resync_period] = ACTIONS(1581),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1581),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1581),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1581),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1581),
    [sym_clkmgr_get_timeservers] = ACTIONS(1581),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1581),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1581),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1581),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1581),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1581),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1581),
    [sym_set_log_level] = ACTIONS(1581),
    [sym_get_log_level] = ACTIONS(1581),
    [sym_amx_log] = ACTIONS(1581),
    [sym_smtp_server_config_set] = ACTIONS(1581),
    [sym_smtp_server_config_get] = ACTIONS(1581),
    [sym_smtp_send] = ACTIONS(1581),
    [sym_validate_netlinx_account] = ACTIONS(1581),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1581),
    [sym_authenticate_certificate] = ACTIONS(1581),
    [sym_audit_netlinx_session_event] = ACTIONS(1581),
    [sym_audit_netlinx_generic_event] = ACTIONS(1581),
    [sym_get_available_flash_disk_space] = ACTIONS(1581),
    [sym_get_max_flash_disk_space] = ACTIONS(1581),
    [sym_data_create_feed] = ACTIONS(1581),
    [sym_data_delete_feed] = ACTIONS(1581),
    [sym_data_publish_feed] = ACTIONS(1581),
    [sym_data_get_published_feed] = ACTIONS(1581),
    [sym_data_add_record] = ACTIONS(1581),
    [sym_data_get_event_record] = ACTIONS(1581),
    [sym__wc_data_create_feed] = ACTIONS(1581),
    [sym__wc_data_add_record] = ACTIONS(1581),
    [sym__wc_data_get_event_record] = ACTIONS(1581),
    [sym_format] = ACTIONS(1581),
    [sym_astro_clock] = ACTIONS(1581),
    [sym_set_outdoor_temperature] = ACTIONS(1581),
    [sym_set_virtual_level_count] = ACTIONS(1581),
    [sym_set_virtual_channel_count] = ACTIONS(1581),
    [sym_set_virtual_port_count] = ACTIONS(1581),
    [sym__wc_encode_int] = ACTIONS(1581),
    [sym__wc_decode_int] = ACTIONS(1581),
    [sym__wc_int] = ACTIONS(1581),
    [sym__wc_to_ch_int] = ACTIONS(1581),
    [sym__ch_to_wc_int] = ACTIONS(1581),
    [sym__wc_find_string_int] = ACTIONS(1581),
    [sym__wc_left_string_int] = ACTIONS(1581),
    [sym__wc_lower_string_int] = ACTIONS(1581),
    [sym__wc_mid_string_int] = ACTIONS(1581),
    [sym__wc_remove_string_int] = ACTIONS(1581),
    [sym__wc_right_string_int] = ACTIONS(1581),
    [sym__wc_upper_string_int] = ACTIONS(1581),
    [sym__wc_compare_string_int] = ACTIONS(1581),
    [sym__wc_get_buffer_char_int] = ACTIONS(1581),
    [sym__wc_get_buffer_string_int] = ACTIONS(1581),
    [sym__wc_concat_string_int] = ACTIONS(1581),
    [sym__wc_file_open_int] = ACTIONS(1581),
    [sym__wc_file_close_int] = ACTIONS(1581),
    [sym__wc_file_read_int] = ACTIONS(1581),
    [sym__wc_file_read_line_int] = ACTIONS(1581),
    [sym__wc_file_write_int] = ACTIONS(1581),
    [sym__wc_file_write_line_int] = ACTIONS(1581),
    [aux_sym_preproc_include_token1] = ACTIONS(1581),
    [aux_sym_preproc_define_token1] = ACTIONS(1583),
    [aux_sym_preproc_warn_token1] = ACTIONS(1583),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1583),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1583),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1583),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1583),
    [aux_sym_preproc_else_token1] = ACTIONS(1583),
    [anon_sym_LBRACK] = ACTIONS(1583),
    [anon_sym_LBRACE] = ACTIONS(1583),
    [anon_sym_COLON] = ACTIONS(1583),
    [anon_sym_LPAREN] = ACTIONS(1581),
    [anon_sym_SEMI] = ACTIONS(1583),
    [anon_sym_BANG] = ACTIONS(1581),
    [anon_sym_TILDE] = ACTIONS(1583),
    [anon_sym_DASH] = ACTIONS(1581),
    [anon_sym_PLUS] = ACTIONS(1581),
    [anon_sym_STAR] = ACTIONS(1583),
    [anon_sym_SLASH] = ACTIONS(1581),
    [anon_sym_PERCENT] = ACTIONS(1583),
    [anon_sym_PIPE_PIPE] = ACTIONS(1583),
    [anon_sym_AMP_AMP] = ACTIONS(1583),
    [anon_sym_PIPE] = ACTIONS(1581),
    [anon_sym_CARET] = ACTIONS(1583),
    [anon_sym_AMP] = ACTIONS(1581),
    [anon_sym_EQ_EQ] = ACTIONS(1583),
    [anon_sym_BANG_EQ] = ACTIONS(1583),
    [anon_sym_LT_GT] = ACTIONS(1583),
    [anon_sym_GT] = ACTIONS(1581),
    [anon_sym_GT_EQ] = ACTIONS(1583),
    [anon_sym_LT_EQ] = ACTIONS(1583),
    [anon_sym_LT] = ACTIONS(1581),
    [anon_sym_LT_LT] = ACTIONS(1583),
    [anon_sym_GT_GT] = ACTIONS(1583),
    [anon_sym_DASH_DASH] = ACTIONS(1583),
    [anon_sym_PLUS_PLUS] = ACTIONS(1583),
    [anon_sym_DOT] = ACTIONS(1581),
    [anon_sym_DQUOTE] = ACTIONS(1583),
    [anon_sym_SQUOTE] = ACTIONS(1583),
    [sym_number_literal] = ACTIONS(1583),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1583),
  },
  [STATE(127)] = {
    [sym_identifier] = ACTIONS(1585),
    [sym_program_name_keyword] = ACTIONS(1585),
    [sym_module_name_keyword] = ACTIONS(1585),
    [sym_define_device_keyword] = ACTIONS(1585),
    [sym_define_combine_keyword] = ACTIONS(1585),
    [sym_define_constant_keyword] = ACTIONS(1585),
    [sym_define_type_keyword] = ACTIONS(1585),
    [sym_define_variable_keyword] = ACTIONS(1585),
    [sym_define_system_variable_keyword] = ACTIONS(1585),
    [sym_define_connect_level_keyword] = ACTIONS(1585),
    [sym_define_latching_keyword] = ACTIONS(1585),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1585),
    [sym_define_toggling_keyword] = ACTIONS(1585),
    [sym_define_call_keyword] = ACTIONS(1585),
    [sym_define_function_keyword] = ACTIONS(1585),
    [sym_define_library_function_keyword] = ACTIONS(1585),
    [sym_define_start_keyword] = ACTIONS(1585),
    [sym_define_event_keyword] = ACTIONS(1585),
    [sym_define_program_keyword] = ACTIONS(1585),
    [sym_define_module_keyword] = ACTIONS(1585),
    [sym_char_keyword] = ACTIONS(1585),
    [sym_widechar_keyword] = ACTIONS(1585),
    [sym_integer_keyword] = ACTIONS(1585),
    [sym_sinteger_keyword] = ACTIONS(1585),
    [sym_long_keyword] = ACTIONS(1585),
    [sym_slong_keyword] = ACTIONS(1585),
    [sym_float_keyword] = ACTIONS(1585),
    [sym_double_keyword] = ACTIONS(1585),
    [sym_variant_keyword] = ACTIONS(1585),
    [sym_variantarray_keyword] = ACTIONS(1585),
    [sym_local_var_keyword] = ACTIONS(1585),
    [sym_stack_var_keyword] = ACTIONS(1585),
    [sym_constant_keyword] = ACTIONS(1585),
    [sym_volatile_keyword] = ACTIONS(1585),
    [sym_non_volatile_keyword] = ACTIONS(1585),
    [sym_persistent_keyword] = ACTIONS(1585),
    [sym_dev_keyword] = ACTIONS(1585),
    [sym_devlev_keyword] = ACTIONS(1585),
    [sym_devchan_keyword] = ACTIONS(1585),
    [sym_if_keyword] = ACTIONS(1585),
    [sym_else_keyword] = ACTIONS(1585),
    [sym_switch_keyword] = ACTIONS(1585),
    [sym_case_keyword] = ACTIONS(1585),
    [sym_default_keyword] = ACTIONS(1585),
    [sym_while_keyword] = ACTIONS(1585),
    [sym_for_keyword] = ACTIONS(1585),
    [sym_break_keyword] = ACTIONS(1585),
    [sym_continue_keyword] = ACTIONS(1585),
    [sym_return_keyword] = ACTIONS(1585),
    [sym_select_keyword] = ACTIONS(1585),
    [sym_active_keyword] = ACTIONS(1585),
    [sym_struct_keyword] = ACTIONS(1585),
    [sym_structure_keyword] = ACTIONS(1585),
    [sym_button_event_keyword] = ACTIONS(1585),
    [sym_channel_event_keyword] = ACTIONS(1585),
    [sym_level_event_keyword] = ACTIONS(1585),
    [sym_data_event_keyword] = ACTIONS(1585),
    [sym_timeline_event_keyword] = ACTIONS(1585),
    [sym_custom_event_keyword] = ACTIONS(1585),
    [sym_push_keyword] = ACTIONS(1585),
    [sym_release_keyword] = ACTIONS(1585),
    [aux_sym_on_keyword_token1] = ACTIONS(1585),
    [aux_sym_off_keyword_token1] = ACTIONS(1585),
    [sym_send_level_keyword] = ACTIONS(1585),
    [sym_send_string_keyword] = ACTIONS(1585),
    [sym_send_command_keyword] = ACTIONS(1585),
    [sym_clear_buffer_keyword] = ACTIONS(1585),
    [sym_create_buffer_keyword] = ACTIONS(1585),
    [sym_create_multi_buffer_keyword] = ACTIONS(1585),
    [sym_call_keyword] = ACTIONS(1585),
    [sym_system_call_keyword] = ACTIONS(1585),
    [sym_devchan_to_keyword] = ACTIONS(1585),
    [sym_devchan_min_to_keyword] = ACTIONS(1585),
    [sym_devchan_total_off_keyword] = ACTIONS(1585),
    [sym_devchan_pulse_keyword] = ACTIONS(1585),
    [sym_wait_keyword] = ACTIONS(1585),
    [sym_cancel_wait_keyword] = ACTIONS(1585),
    [sym_wait_until_keyword] = ACTIONS(1585),
    [sym_cancel_wait_until_keyword] = ACTIONS(1585),
    [sym_cancel_all_wait_keyword] = ACTIONS(1585),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1585),
    [sym_and] = ACTIONS(1585),
    [sym_or] = ACTIONS(1585),
    [sym_not] = ACTIONS(1585),
    [sym_xor] = ACTIONS(1585),
    [sym_band] = ACTIONS(1585),
    [sym_bor] = ACTIONS(1585),
    [sym_bxor] = ACTIONS(1585),
    [sym_bnot] = ACTIONS(1585),
    [sym_lshift] = ACTIONS(1585),
    [sym_rshift] = ACTIONS(1585),
    [sym___file__] = ACTIONS(1585),
    [sym___date__] = ACTIONS(1585),
    [sym___ldate__] = ACTIONS(1585),
    [sym___time__] = ACTIONS(1585),
    [sym___line__] = ACTIONS(1585),
    [sym___name__] = ACTIONS(1585),
    [sym___netlinx__] = ACTIONS(1585),
    [sym_netlinx_axi_version] = ACTIONS(1585),
    [sym_url_flg_tcp] = ACTIONS(1585),
    [sym_url_flg_accinfopresent] = ACTIONS(1585),
    [sym_url_flg_temp] = ACTIONS(1585),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1585),
    [sym_url_flg_stat_mask] = ACTIONS(1585),
    [sym_url_flg_stat_lookup] = ACTIONS(1585),
    [sym_url_flg_stat_connecting] = ACTIONS(1585),
    [sym_url_flg_stat_waiting] = ACTIONS(1585),
    [sym_url_flg_stat_connected] = ACTIONS(1585),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1585),
    [sym_first_virtual_device] = ACTIONS(1585),
    [sym_first_local_port] = ACTIONS(1585),
    [sym_dynamic_virtual_device] = ACTIONS(1585),
    [sym_timeline_once] = ACTIONS(1585),
    [sym_timeline_repeat] = ACTIONS(1585),
    [sym_timeline_absolute] = ACTIONS(1585),
    [sym_timeline_relative] = ACTIONS(1585),
    [sym_do_push_timed_infinite] = ACTIONS(1585),
    [sym_source_type_no_address] = ACTIONS(1585),
    [sym_source_type_neuron_id] = ACTIONS(1585),
    [sym_source_type_ip_address] = ACTIONS(1585),
    [sym_source_type_axlink] = ACTIONS(1585),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1585),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1585),
    [sym_source_type_ip_socket_address] = ACTIONS(1585),
    [sym_source_type_rs232] = ACTIONS(1585),
    [sym_source_type_internal] = ACTIONS(1585),
    [sym_source_type_mac_address] = ACTIONS(1585),
    [sym_source_type_ipv4_port] = ACTIONS(1585),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1585),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1585),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1585),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1585),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1585),
    [sym_true] = ACTIONS(1585),
    [sym_false] = ACTIONS(1585),
    [sym_file_read_only] = ACTIONS(1585),
    [sym_file_rw_new] = ACTIONS(1585),
    [sym_file_rw_append] = ACTIONS(1585),
    [sym_ip_tcp] = ACTIONS(1585),
    [sym_ip_udp] = ACTIONS(1585),
    [sym_ip_udp_2way] = ACTIONS(1585),
    [sym_xml_encode_types] = ACTIONS(1585),
    [sym_xml_encode_char_as_list] = ACTIONS(1585),
    [sym_xml_encode_le] = ACTIONS(1585),
    [sym_xml_decode_types] = ACTIONS(1585),
    [sym_xml_decode_no_preserve] = ACTIONS(1585),
    [sym_duet_dev_type_display_device] = ACTIONS(1585),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1585),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1585),
    [sym_duet_dev_type_io_device] = ACTIONS(1585),
    [sym_duet_dev_type_relay_device] = ACTIONS(1585),
    [sym_duet_dev_type_ups] = ACTIONS(1585),
    [sym_duet_dev_type_amplifier] = ACTIONS(1585),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1585),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1585),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1585),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1585),
    [sym_duet_dev_type_camera] = ACTIONS(1585),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1585),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1585),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1585),
    [sym_duet_dev_type_dss] = ACTIONS(1585),
    [sym_duet_dev_type_dvr] = ACTIONS(1585),
    [sym_duet_dev_type_disc_device] = ACTIONS(1585),
    [sym_duet_dev_type_document_camera] = ACTIONS(1585),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1585),
    [sym_duet_dev_type_hvac] = ACTIONS(1585),
    [sym_duet_dev_type_keypad] = ACTIONS(1585),
    [sym_duet_dev_type_light] = ACTIONS(1585),
    [sym_duet_dev_type_monitor] = ACTIONS(1585),
    [sym_duet_dev_type_motor] = ACTIONS(1585),
    [sym_duet_dev_type_multi_window] = ACTIONS(1585),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1585),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1585),
    [sym_duet_dev_type_receiver] = ACTIONS(1585),
    [sym_duet_dev_type_security_system] = ACTIONS(1585),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1585),
    [sym_duet_dev_type_settop_box] = ACTIONS(1585),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1585),
    [sym_duet_dev_type_switcher] = ACTIONS(1585),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1585),
    [sym_duet_dev_type_tv] = ACTIONS(1585),
    [sym_duet_dev_type_utility] = ACTIONS(1585),
    [sym_duet_dev_type_vcr] = ACTIONS(1585),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1585),
    [sym_duet_dev_type_video_processor] = ACTIONS(1585),
    [sym_duet_dev_type_video_projector] = ACTIONS(1585),
    [sym_duet_dev_type_video_wall] = ACTIONS(1585),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1585),
    [sym_duet_dev_type_weather] = ACTIONS(1585),
    [sym_duet_dev_not_polled] = ACTIONS(1585),
    [sym_duet_dev_polled] = ACTIONS(1585),
    [sym_clkmgr_mode_network] = ACTIONS(1585),
    [sym_clkmgr_mode_standalone] = ACTIONS(1585),
    [sym_normal_standby] = ACTIONS(1585),
    [sym_normal_wake] = ACTIONS(1585),
    [sym_ip_multicast_ttl_option] = ACTIONS(1585),
    [sym_ip_ttl_subnet] = ACTIONS(1585),
    [sym_ip_ttl_site] = ACTIONS(1585),
    [sym_ip_ttl_region] = ACTIONS(1585),
    [sym_ip_ttl_continent] = ACTIONS(1585),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1585),
    [sym_ip_nodelay_on] = ACTIONS(1585),
    [sym_ip_nodelay_off] = ACTIONS(1585),
    [sym_amx_error] = ACTIONS(1585),
    [sym_amx_warning] = ACTIONS(1585),
    [sym_amx_info] = ACTIONS(1585),
    [sym_amx_debug] = ACTIONS(1585),
    [sym_netlinx_login_success] = ACTIONS(1585),
    [sym_netlinx_login_fail] = ACTIONS(1585),
    [sym_netlinx_logout] = ACTIONS(1585),
    [sym_null_str] = ACTIONS(1585),
    [sym_smtp_address] = ACTIONS(1585),
    [sym_smtp_port_number] = ACTIONS(1585),
    [sym_smtp_username] = ACTIONS(1585),
    [sym_smtp_password] = ACTIONS(1585),
    [sym_smtp_require_tls] = ACTIONS(1585),
    [sym_smtp_from] = ACTIONS(1585),
    [sym_smtp_tls_true] = ACTIONS(1585),
    [sym_smtp_tls_false] = ACTIONS(1585),
    [sym_listview_on_row_select_event] = ACTIONS(1585),
    [sym_data_structure_unknown] = ACTIONS(1585),
    [sym_data_structure_datafeed] = ACTIONS(1585),
    [sym_data_structure_datarecordset] = ACTIONS(1585),
    [sym_data_structure_datarecord] = ACTIONS(1585),
    [sym_data_structure_datafield] = ACTIONS(1585),
    [sym_data_max_content_field_count] = ACTIONS(1585),
    [sym_data_max_metadata_field_count] = ACTIONS(1585),
    [sym_data_max_id_length] = ACTIONS(1585),
    [sym_data_max_name_length] = ACTIONS(1585),
    [sym_data_max_description_length] = ACTIONS(1585),
    [sym_data_max_type_length] = ACTIONS(1585),
    [sym_data_max_format_length] = ACTIONS(1585),
    [sym_data_max_source_length] = ACTIONS(1585),
    [sym_data_max_value_length] = ACTIONS(1585),
    [sym_data_max_label_length] = ACTIONS(1585),
    [sym_data_max_url_length] = ACTIONS(1585),
    [sym_max_last_login_info_length] = ACTIONS(1585),
    [sym_data_type_unknown] = ACTIONS(1585),
    [sym_data_type_string] = ACTIONS(1585),
    [sym_data_type_datetime] = ACTIONS(1585),
    [sym_data_type_date] = ACTIONS(1585),
    [sym_data_type_time] = ACTIONS(1585),
    [sym_data_type_image] = ACTIONS(1585),
    [sym_data_format_url] = ACTIONS(1585),
    [sym_data_format_phone] = ACTIONS(1585),
    [sym_data_format_email] = ACTIONS(1585),
    [sym_data_format_iso8601] = ACTIONS(1585),
    [sym_auditlog_priv] = ACTIONS(1585),
    [sym_configuration_priv] = ACTIONS(1585),
    [sym_deviceconfig_priv] = ACTIONS(1585),
    [sym_ftp_priv] = ACTIONS(1585),
    [sym_http_priv] = ACTIONS(1585),
    [sym_networkconfig_priv] = ACTIONS(1585),
    [sym_programport_priv] = ACTIONS(1585),
    [sym_remoteui_priv] = ACTIONS(1585),
    [sym_securitycontrol_priv] = ACTIONS(1585),
    [sym_softwaremanagement_priv] = ACTIONS(1585),
    [sym_terminal_priv] = ACTIONS(1585),
    [sym_tpadmin_priv] = ACTIONS(1585),
    [sym_user1_priv] = ACTIONS(1585),
    [sym_user2_priv] = ACTIONS(1585),
    [sym_user3_priv] = ACTIONS(1585),
    [sym_user4_priv] = ACTIONS(1585),
    [sym_usermanagement_priv] = ACTIONS(1585),
    [sym_valid_account] = ACTIONS(1585),
    [sym_err_name_invalid] = ACTIONS(1585),
    [sym_err_password_invalid] = ACTIONS(1585),
    [sym_err_invalid_account] = ACTIONS(1585),
    [sym_err_no_authorization] = ACTIONS(1585),
    [sym_err_invalid_parameter] = ACTIONS(1585),
    [sym_err_user_locked_out] = ACTIONS(1585),
    [sym_err_user_account_expired] = ACTIONS(1585),
    [sym_err_general_error] = ACTIONS(1585),
    [sym_err_authentication_error] = ACTIONS(1585),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1585),
    [sym_tls_validate_certificate] = ACTIONS(1585),
    [sym_tdata] = ACTIONS(1585),
    [sym_tchannel] = ACTIONS(1585),
    [sym_tlevel] = ACTIONS(1585),
    [sym_tbutton] = ACTIONS(1585),
    [sym_ttimeline] = ACTIONS(1585),
    [sym_tcustom] = ACTIONS(1585),
    [sym_url_struct] = ACTIONS(1585),
    [sym_dns_struct] = ACTIONS(1585),
    [sym_ip_address_struct] = ACTIONS(1585),
    [sym_dev_info_struct] = ACTIONS(1585),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1585),
    [sym_data_feed] = ACTIONS(1585),
    [sym_data_field] = ACTIONS(1585),
    [sym_data_record] = ACTIONS(1585),
    [sym_wc_data_feed] = ACTIONS(1585),
    [sym_wc_data_field] = ACTIONS(1585),
    [sym_wc_data_record] = ACTIONS(1585),
    [sym_last_login_info] = ACTIONS(1585),
    [sym____reserved___] = ACTIONS(1585),
    [sym_date] = ACTIONS(1585),
    [sym_day] = ACTIONS(1585),
    [sym_time] = ACTIONS(1585),
    [sym_ldate] = ACTIONS(1585),
    [sym_push_device] = ACTIONS(1585),
    [sym_push_channel] = ACTIONS(1585),
    [sym_push_devchan] = ACTIONS(1585),
    [sym_release_device] = ACTIONS(1585),
    [sym_release_channel] = ACTIONS(1585),
    [sym_release_devchan] = ACTIONS(1585),
    [sym_master_slot] = ACTIONS(1585),
    [sym_get_pulse_time] = ACTIONS(1585),
    [sym_get_timer] = ACTIONS(1585),
    [sym_master_sn] = ACTIONS(1585),
    [sym_system_number] = ACTIONS(1585),
    [sym_button] = ACTIONS(1585),
    [sym_dv_channel] = ACTIONS(1585),
    [sym_data] = ACTIONS(1585),
    [sym_level] = ACTIONS(1585),
    [sym_timeline] = ACTIONS(1585),
    [sym_channel] = ACTIONS(1585),
    [sym_custom] = ACTIONS(1585),
    [sym_atoi] = ACTIONS(1585),
    [sym_atol] = ACTIONS(1585),
    [sym_atof] = ACTIONS(1585),
    [sym_itoa] = ACTIONS(1585),
    [sym_itohex] = ACTIONS(1585),
    [sym_hextoi] = ACTIONS(1585),
    [sym_ftoa] = ACTIONS(1585),
    [sym_find_string] = ACTIONS(1585),
    [sym_left_string] = ACTIONS(1585),
    [sym_right_string] = ACTIONS(1585),
    [sym_mid_string] = ACTIONS(1585),
    [sym_remove_string] = ACTIONS(1585),
    [sym_get_buffer_string] = ACTIONS(1585),
    [sym_get_buffer_char] = ACTIONS(1585),
    [sym_get_multi_buffer_string] = ACTIONS(1585),
    [sym_length_string] = ACTIONS(1585),
    [sym_max_length_string] = ACTIONS(1585),
    [sym_set_length_string] = ACTIONS(1585),
    [sym_compare_string] = ACTIONS(1585),
    [sym_upper_string] = ACTIONS(1585),
    [sym_lower_string] = ACTIONS(1585),
    [sym_get_last] = ACTIONS(1585),
    [sym_date_to_day] = ACTIONS(1585),
    [sym_date_to_month] = ACTIONS(1585),
    [sym_date_to_year] = ACTIONS(1585),
    [sym_time_to_hour] = ACTIONS(1585),
    [sym_time_to_minute] = ACTIONS(1585),
    [sym_time_to_second] = ACTIONS(1585),
    [sym_day_of_week] = ACTIONS(1585),
    [sym_set_pulse_time] = ACTIONS(1585),
    [sym_set_timer] = ACTIONS(1585),
    [sym_file_open] = ACTIONS(1585),
    [sym_file_close] = ACTIONS(1585),
    [sym_file_read] = ACTIONS(1585),
    [sym_file_read_line] = ACTIONS(1585),
    [sym_file_write] = ACTIONS(1585),
    [sym_file_write_line] = ACTIONS(1585),
    [sym_file_copy] = ACTIONS(1585),
    [sym_file_delete] = ACTIONS(1585),
    [sym_file_rename] = ACTIONS(1585),
    [sym_file_dir] = ACTIONS(1585),
    [sym_file_getdir] = ACTIONS(1585),
    [sym_file_setdir] = ACTIONS(1585),
    [sym_file_create_dir] = ACTIONS(1585),
    [sym_file_remove_dir] = ACTIONS(1585),
    [sym_file_seek] = ACTIONS(1585),
    [sym_length_array] = ACTIONS(1585),
    [sym_max_length_array] = ACTIONS(1585),
    [sym_set_length_array] = ACTIONS(1585),
    [sym_ip_client_open] = ACTIONS(1585),
    [sym_ip_client_close] = ACTIONS(1585),
    [sym_ip_server_open] = ACTIONS(1585),
    [sym_ip_server_close] = ACTIONS(1585),
    [sym_ip_bound_client_open] = ACTIONS(1585),
    [sym_ip_mc_server_open] = ACTIONS(1585),
    [sym_ip_set_option] = ACTIONS(1585),
    [sym_ssh_client_open] = ACTIONS(1585),
    [sym_ssh_client_close] = ACTIONS(1585),
    [sym_tls_client_open] = ACTIONS(1585),
    [sym_tls_client_close] = ACTIONS(1585),
    [sym_do_push] = ACTIONS(1585),
    [sym_do_push_timed] = ACTIONS(1585),
    [sym_do_release] = ACTIONS(1585),
    [sym_device_id] = ACTIONS(1585),
    [sym_device_info] = ACTIONS(1585),
    [sym_device_id_string] = ACTIONS(1585),
    [sym_device_standby] = ACTIONS(1585),
    [sym_device_wake] = ACTIONS(1585),
    [sym_timeline_create] = ACTIONS(1585),
    [sym_timeline_kill] = ACTIONS(1585),
    [sym_timeline_reload] = ACTIONS(1585),
    [sym_timeline_pause] = ACTIONS(1585),
    [sym_timeline_restart] = ACTIONS(1585),
    [sym_timeline_set] = ACTIONS(1585),
    [sym_timeline_get] = ACTIONS(1585),
    [sym_timeline_active] = ACTIONS(1585),
    [sym_timeline_dynamic_id] = ACTIONS(1585),
    [sym_variable_to_string] = ACTIONS(1585),
    [sym_string_to_variable] = ACTIONS(1585),
    [sym_length_variable_to_string] = ACTIONS(1585),
    [sym_type_cast] = ACTIONS(1585),
    [sym_variable_to_xml] = ACTIONS(1585),
    [sym_xml_to_variable] = ACTIONS(1585),
    [sym_length_variable_to_xml] = ACTIONS(1585),
    [sym_redirect_string] = ACTIONS(1585),
    [sym_random_number] = ACTIONS(1585),
    [sym_get_unique_id] = ACTIONS(1585),
    [sym_get_system_number] = ACTIONS(1585),
    [sym_set_system_number] = ACTIONS(1585),
    [sym_get_serial_number] = ACTIONS(1585),
    [sym_get_master_build] = ACTIONS(1585),
    [sym_set_validation_code] = ACTIONS(1585),
    [sym_reboot] = ACTIONS(1585),
    [sym_load_duet_module] = ACTIONS(1585),
    [sym_unload_duet_module] = ACTIONS(1585),
    [sym_duet_mem_size_set] = ACTIONS(1585),
    [sym_duet_mem_size_get] = ACTIONS(1585),
    [sym_dynamic_application_device] = ACTIONS(1585),
    [sym_dynamic_polled_port] = ACTIONS(1585),
    [sym_static_port_binding] = ACTIONS(1585),
    [sym_static_ip_binding] = ACTIONS(1585),
    [sym_get_dns_list] = ACTIONS(1585),
    [sym_set_dns_list] = ACTIONS(1585),
    [sym_get_ip_address] = ACTIONS(1585),
    [sym_set_ip_address] = ACTIONS(1585),
    [sym_get_url_list] = ACTIONS(1585),
    [sym_add_url_entry] = ACTIONS(1585),
    [sym_delete_url_entry] = ACTIONS(1585),
    [sym_abs_value] = ACTIONS(1585),
    [sym_max_value] = ACTIONS(1585),
    [sym_min_value] = ACTIONS(1585),
    [sym_exp_value] = ACTIONS(1585),
    [sym_log_value] = ACTIONS(1585),
    [sym_log10_value] = ACTIONS(1585),
    [sym_power_value] = ACTIONS(1585),
    [sym_sqrt_value] = ACTIONS(1585),
    [sym_raw_be] = ACTIONS(1585),
    [sym_raw_le] = ACTIONS(1585),
    [sym_internal_queue_size_set] = ACTIONS(1585),
    [sym_internal_queue_size_get] = ACTIONS(1585),
    [sym_internal_threshold_set] = ACTIONS(1585),
    [sym_internal_threshold_get] = ACTIONS(1585),
    [sym_rebuilt_event] = ACTIONS(1585),
    [sym_do_custom_event] = ACTIONS(1585),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1585),
    [sym_clkmgr_set_clk_source] = ACTIONS(1585),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1585),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1585),
    [sym_clkmgr_get_timezone] = ACTIONS(1585),
    [sym_clkmgr_set_timezone] = ACTIONS(1585),
    [sym_clkmgr_get_resync_period] = ACTIONS(1585),
    [sym_clkmgr_set_resync_period] = ACTIONS(1585),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1585),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1585),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1585),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1585),
    [sym_clkmgr_get_timeservers] = ACTIONS(1585),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1585),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1585),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1585),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1585),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1585),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1585),
    [sym_set_log_level] = ACTIONS(1585),
    [sym_get_log_level] = ACTIONS(1585),
    [sym_amx_log] = ACTIONS(1585),
    [sym_smtp_server_config_set] = ACTIONS(1585),
    [sym_smtp_server_config_get] = ACTIONS(1585),
    [sym_smtp_send] = ACTIONS(1585),
    [sym_validate_netlinx_account] = ACTIONS(1585),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1585),
    [sym_authenticate_certificate] = ACTIONS(1585),
    [sym_audit_netlinx_session_event] = ACTIONS(1585),
    [sym_audit_netlinx_generic_event] = ACTIONS(1585),
    [sym_get_available_flash_disk_space] = ACTIONS(1585),
    [sym_get_max_flash_disk_space] = ACTIONS(1585),
    [sym_data_create_feed] = ACTIONS(1585),
    [sym_data_delete_feed] = ACTIONS(1585),
    [sym_data_publish_feed] = ACTIONS(1585),
    [sym_data_get_published_feed] = ACTIONS(1585),
    [sym_data_add_record] = ACTIONS(1585),
    [sym_data_get_event_record] = ACTIONS(1585),
    [sym__wc_data_create_feed] = ACTIONS(1585),
    [sym__wc_data_add_record] = ACTIONS(1585),
    [sym__wc_data_get_event_record] = ACTIONS(1585),
    [sym_format] = ACTIONS(1585),
    [sym_astro_clock] = ACTIONS(1585),
    [sym_set_outdoor_temperature] = ACTIONS(1585),
    [sym_set_virtual_level_count] = ACTIONS(1585),
    [sym_set_virtual_channel_count] = ACTIONS(1585),
    [sym_set_virtual_port_count] = ACTIONS(1585),
    [sym__wc_encode_int] = ACTIONS(1585),
    [sym__wc_decode_int] = ACTIONS(1585),
    [sym__wc_int] = ACTIONS(1585),
    [sym__wc_to_ch_int] = ACTIONS(1585),
    [sym__ch_to_wc_int] = ACTIONS(1585),
    [sym__wc_find_string_int] = ACTIONS(1585),
    [sym__wc_left_string_int] = ACTIONS(1585),
    [sym__wc_lower_string_int] = ACTIONS(1585),
    [sym__wc_mid_string_int] = ACTIONS(1585),
    [sym__wc_remove_string_int] = ACTIONS(1585),
    [sym__wc_right_string_int] = ACTIONS(1585),
    [sym__wc_upper_string_int] = ACTIONS(1585),
    [sym__wc_compare_string_int] = ACTIONS(1585),
    [sym__wc_get_buffer_char_int] = ACTIONS(1585),
    [sym__wc_get_buffer_string_int] = ACTIONS(1585),
    [sym__wc_concat_string_int] = ACTIONS(1585),
    [sym__wc_file_open_int] = ACTIONS(1585),
    [sym__wc_file_close_int] = ACTIONS(1585),
    [sym__wc_file_read_int] = ACTIONS(1585),
    [sym__wc_file_read_line_int] = ACTIONS(1585),
    [sym__wc_file_write_int] = ACTIONS(1585),
    [sym__wc_file_write_line_int] = ACTIONS(1585),
    [aux_sym_preproc_include_token1] = ACTIONS(1585),
    [aux_sym_preproc_define_token1] = ACTIONS(1587),
    [aux_sym_preproc_warn_token1] = ACTIONS(1587),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1587),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1587),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1587),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1587),
    [aux_sym_preproc_else_token1] = ACTIONS(1587),
    [anon_sym_LBRACK] = ACTIONS(1587),
    [anon_sym_LBRACE] = ACTIONS(1587),
    [anon_sym_COLON] = ACTIONS(1587),
    [anon_sym_LPAREN] = ACTIONS(1585),
    [anon_sym_SEMI] = ACTIONS(1587),
    [anon_sym_BANG] = ACTIONS(1585),
    [anon_sym_TILDE] = ACTIONS(1587),
    [anon_sym_DASH] = ACTIONS(1585),
    [anon_sym_PLUS] = ACTIONS(1585),
    [anon_sym_STAR] = ACTIONS(1587),
    [anon_sym_SLASH] = ACTIONS(1585),
    [anon_sym_PERCENT] = ACTIONS(1587),
    [anon_sym_PIPE_PIPE] = ACTIONS(1587),
    [anon_sym_AMP_AMP] = ACTIONS(1587),
    [anon_sym_PIPE] = ACTIONS(1585),
    [anon_sym_CARET] = ACTIONS(1587),
    [anon_sym_AMP] = ACTIONS(1585),
    [anon_sym_EQ_EQ] = ACTIONS(1587),
    [anon_sym_BANG_EQ] = ACTIONS(1587),
    [anon_sym_LT_GT] = ACTIONS(1587),
    [anon_sym_GT] = ACTIONS(1585),
    [anon_sym_GT_EQ] = ACTIONS(1587),
    [anon_sym_LT_EQ] = ACTIONS(1587),
    [anon_sym_LT] = ACTIONS(1585),
    [anon_sym_LT_LT] = ACTIONS(1587),
    [anon_sym_GT_GT] = ACTIONS(1587),
    [anon_sym_DASH_DASH] = ACTIONS(1587),
    [anon_sym_PLUS_PLUS] = ACTIONS(1587),
    [anon_sym_DOT] = ACTIONS(1585),
    [anon_sym_DQUOTE] = ACTIONS(1587),
    [anon_sym_SQUOTE] = ACTIONS(1587),
    [sym_number_literal] = ACTIONS(1587),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1587),
  },
  [STATE(128)] = {
    [ts_builtin_sym_end] = ACTIONS(1523),
    [sym_identifier] = ACTIONS(1521),
    [sym_program_name_keyword] = ACTIONS(1521),
    [sym_module_name_keyword] = ACTIONS(1521),
    [sym_define_device_keyword] = ACTIONS(1521),
    [sym_define_combine_keyword] = ACTIONS(1521),
    [sym_define_constant_keyword] = ACTIONS(1521),
    [sym_define_type_keyword] = ACTIONS(1521),
    [sym_define_variable_keyword] = ACTIONS(1521),
    [sym_define_system_variable_keyword] = ACTIONS(1521),
    [sym_define_connect_level_keyword] = ACTIONS(1521),
    [sym_define_latching_keyword] = ACTIONS(1521),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1521),
    [sym_define_toggling_keyword] = ACTIONS(1521),
    [sym_define_call_keyword] = ACTIONS(1521),
    [sym_define_function_keyword] = ACTIONS(1521),
    [sym_define_library_function_keyword] = ACTIONS(1521),
    [sym_define_start_keyword] = ACTIONS(1521),
    [sym_define_event_keyword] = ACTIONS(1521),
    [sym_define_program_keyword] = ACTIONS(1521),
    [sym_define_module_keyword] = ACTIONS(1521),
    [sym_char_keyword] = ACTIONS(1521),
    [sym_widechar_keyword] = ACTIONS(1521),
    [sym_integer_keyword] = ACTIONS(1521),
    [sym_sinteger_keyword] = ACTIONS(1521),
    [sym_long_keyword] = ACTIONS(1521),
    [sym_slong_keyword] = ACTIONS(1521),
    [sym_float_keyword] = ACTIONS(1521),
    [sym_double_keyword] = ACTIONS(1521),
    [sym_variant_keyword] = ACTIONS(1521),
    [sym_variantarray_keyword] = ACTIONS(1521),
    [sym_local_var_keyword] = ACTIONS(1521),
    [sym_stack_var_keyword] = ACTIONS(1521),
    [sym_constant_keyword] = ACTIONS(1521),
    [sym_volatile_keyword] = ACTIONS(1521),
    [sym_non_volatile_keyword] = ACTIONS(1521),
    [sym_persistent_keyword] = ACTIONS(1521),
    [sym_dev_keyword] = ACTIONS(1521),
    [sym_devlev_keyword] = ACTIONS(1521),
    [sym_devchan_keyword] = ACTIONS(1521),
    [sym_if_keyword] = ACTIONS(1521),
    [sym_else_keyword] = ACTIONS(1521),
    [sym_switch_keyword] = ACTIONS(1521),
    [sym_case_keyword] = ACTIONS(1521),
    [sym_default_keyword] = ACTIONS(1521),
    [sym_while_keyword] = ACTIONS(1521),
    [sym_for_keyword] = ACTIONS(1521),
    [sym_break_keyword] = ACTIONS(1521),
    [sym_continue_keyword] = ACTIONS(1521),
    [sym_return_keyword] = ACTIONS(1521),
    [sym_select_keyword] = ACTIONS(1521),
    [sym_active_keyword] = ACTIONS(1521),
    [sym_struct_keyword] = ACTIONS(1521),
    [sym_structure_keyword] = ACTIONS(1521),
    [sym_button_event_keyword] = ACTIONS(1521),
    [sym_channel_event_keyword] = ACTIONS(1521),
    [sym_level_event_keyword] = ACTIONS(1521),
    [sym_data_event_keyword] = ACTIONS(1521),
    [sym_timeline_event_keyword] = ACTIONS(1521),
    [sym_custom_event_keyword] = ACTIONS(1521),
    [sym_push_keyword] = ACTIONS(1521),
    [sym_release_keyword] = ACTIONS(1521),
    [aux_sym_on_keyword_token1] = ACTIONS(1521),
    [aux_sym_off_keyword_token1] = ACTIONS(1521),
    [sym_send_level_keyword] = ACTIONS(1521),
    [sym_send_string_keyword] = ACTIONS(1521),
    [sym_send_command_keyword] = ACTIONS(1521),
    [sym_clear_buffer_keyword] = ACTIONS(1521),
    [sym_create_buffer_keyword] = ACTIONS(1521),
    [sym_create_multi_buffer_keyword] = ACTIONS(1521),
    [sym_call_keyword] = ACTIONS(1521),
    [sym_system_call_keyword] = ACTIONS(1521),
    [sym_devchan_to_keyword] = ACTIONS(1521),
    [sym_devchan_min_to_keyword] = ACTIONS(1521),
    [sym_devchan_total_off_keyword] = ACTIONS(1521),
    [sym_devchan_pulse_keyword] = ACTIONS(1521),
    [sym_wait_keyword] = ACTIONS(1521),
    [sym_cancel_wait_keyword] = ACTIONS(1521),
    [sym_wait_until_keyword] = ACTIONS(1521),
    [sym_cancel_wait_until_keyword] = ACTIONS(1521),
    [sym_cancel_all_wait_keyword] = ACTIONS(1521),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1521),
    [sym_and] = ACTIONS(1521),
    [sym_or] = ACTIONS(1521),
    [sym_not] = ACTIONS(1521),
    [sym_xor] = ACTIONS(1521),
    [sym_band] = ACTIONS(1521),
    [sym_bor] = ACTIONS(1521),
    [sym_bxor] = ACTIONS(1521),
    [sym_bnot] = ACTIONS(1521),
    [sym_lshift] = ACTIONS(1521),
    [sym_rshift] = ACTIONS(1521),
    [sym___file__] = ACTIONS(1521),
    [sym___date__] = ACTIONS(1521),
    [sym___ldate__] = ACTIONS(1521),
    [sym___time__] = ACTIONS(1521),
    [sym___line__] = ACTIONS(1521),
    [sym___name__] = ACTIONS(1521),
    [sym___netlinx__] = ACTIONS(1521),
    [sym_netlinx_axi_version] = ACTIONS(1521),
    [sym_url_flg_tcp] = ACTIONS(1521),
    [sym_url_flg_accinfopresent] = ACTIONS(1521),
    [sym_url_flg_temp] = ACTIONS(1521),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1521),
    [sym_url_flg_stat_mask] = ACTIONS(1521),
    [sym_url_flg_stat_lookup] = ACTIONS(1521),
    [sym_url_flg_stat_connecting] = ACTIONS(1521),
    [sym_url_flg_stat_waiting] = ACTIONS(1521),
    [sym_url_flg_stat_connected] = ACTIONS(1521),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1521),
    [sym_first_virtual_device] = ACTIONS(1521),
    [sym_first_local_port] = ACTIONS(1521),
    [sym_dynamic_virtual_device] = ACTIONS(1521),
    [sym_timeline_once] = ACTIONS(1521),
    [sym_timeline_repeat] = ACTIONS(1521),
    [sym_timeline_absolute] = ACTIONS(1521),
    [sym_timeline_relative] = ACTIONS(1521),
    [sym_do_push_timed_infinite] = ACTIONS(1521),
    [sym_source_type_no_address] = ACTIONS(1521),
    [sym_source_type_neuron_id] = ACTIONS(1521),
    [sym_source_type_ip_address] = ACTIONS(1521),
    [sym_source_type_axlink] = ACTIONS(1521),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1521),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1521),
    [sym_source_type_ip_socket_address] = ACTIONS(1521),
    [sym_source_type_rs232] = ACTIONS(1521),
    [sym_source_type_internal] = ACTIONS(1521),
    [sym_source_type_mac_address] = ACTIONS(1521),
    [sym_source_type_ipv4_port] = ACTIONS(1521),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1521),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1521),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1521),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1521),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1521),
    [sym_true] = ACTIONS(1521),
    [sym_false] = ACTIONS(1521),
    [sym_file_read_only] = ACTIONS(1521),
    [sym_file_rw_new] = ACTIONS(1521),
    [sym_file_rw_append] = ACTIONS(1521),
    [sym_ip_tcp] = ACTIONS(1521),
    [sym_ip_udp] = ACTIONS(1521),
    [sym_ip_udp_2way] = ACTIONS(1521),
    [sym_xml_encode_types] = ACTIONS(1521),
    [sym_xml_encode_char_as_list] = ACTIONS(1521),
    [sym_xml_encode_le] = ACTIONS(1521),
    [sym_xml_decode_types] = ACTIONS(1521),
    [sym_xml_decode_no_preserve] = ACTIONS(1521),
    [sym_duet_dev_type_display_device] = ACTIONS(1521),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1521),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1521),
    [sym_duet_dev_type_io_device] = ACTIONS(1521),
    [sym_duet_dev_type_relay_device] = ACTIONS(1521),
    [sym_duet_dev_type_ups] = ACTIONS(1521),
    [sym_duet_dev_type_amplifier] = ACTIONS(1521),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1521),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1521),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1521),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1521),
    [sym_duet_dev_type_camera] = ACTIONS(1521),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1521),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1521),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1521),
    [sym_duet_dev_type_dss] = ACTIONS(1521),
    [sym_duet_dev_type_dvr] = ACTIONS(1521),
    [sym_duet_dev_type_disc_device] = ACTIONS(1521),
    [sym_duet_dev_type_document_camera] = ACTIONS(1521),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1521),
    [sym_duet_dev_type_hvac] = ACTIONS(1521),
    [sym_duet_dev_type_keypad] = ACTIONS(1521),
    [sym_duet_dev_type_light] = ACTIONS(1521),
    [sym_duet_dev_type_monitor] = ACTIONS(1521),
    [sym_duet_dev_type_motor] = ACTIONS(1521),
    [sym_duet_dev_type_multi_window] = ACTIONS(1521),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1521),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1521),
    [sym_duet_dev_type_receiver] = ACTIONS(1521),
    [sym_duet_dev_type_security_system] = ACTIONS(1521),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1521),
    [sym_duet_dev_type_settop_box] = ACTIONS(1521),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1521),
    [sym_duet_dev_type_switcher] = ACTIONS(1521),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1521),
    [sym_duet_dev_type_tv] = ACTIONS(1521),
    [sym_duet_dev_type_utility] = ACTIONS(1521),
    [sym_duet_dev_type_vcr] = ACTIONS(1521),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1521),
    [sym_duet_dev_type_video_processor] = ACTIONS(1521),
    [sym_duet_dev_type_video_projector] = ACTIONS(1521),
    [sym_duet_dev_type_video_wall] = ACTIONS(1521),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1521),
    [sym_duet_dev_type_weather] = ACTIONS(1521),
    [sym_duet_dev_not_polled] = ACTIONS(1521),
    [sym_duet_dev_polled] = ACTIONS(1521),
    [sym_clkmgr_mode_network] = ACTIONS(1521),
    [sym_clkmgr_mode_standalone] = ACTIONS(1521),
    [sym_normal_standby] = ACTIONS(1521),
    [sym_normal_wake] = ACTIONS(1521),
    [sym_ip_multicast_ttl_option] = ACTIONS(1521),
    [sym_ip_ttl_subnet] = ACTIONS(1521),
    [sym_ip_ttl_site] = ACTIONS(1521),
    [sym_ip_ttl_region] = ACTIONS(1521),
    [sym_ip_ttl_continent] = ACTIONS(1521),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1521),
    [sym_ip_nodelay_on] = ACTIONS(1521),
    [sym_ip_nodelay_off] = ACTIONS(1521),
    [sym_amx_error] = ACTIONS(1521),
    [sym_amx_warning] = ACTIONS(1521),
    [sym_amx_info] = ACTIONS(1521),
    [sym_amx_debug] = ACTIONS(1521),
    [sym_netlinx_login_success] = ACTIONS(1521),
    [sym_netlinx_login_fail] = ACTIONS(1521),
    [sym_netlinx_logout] = ACTIONS(1521),
    [sym_null_str] = ACTIONS(1521),
    [sym_smtp_address] = ACTIONS(1521),
    [sym_smtp_port_number] = ACTIONS(1521),
    [sym_smtp_username] = ACTIONS(1521),
    [sym_smtp_password] = ACTIONS(1521),
    [sym_smtp_require_tls] = ACTIONS(1521),
    [sym_smtp_from] = ACTIONS(1521),
    [sym_smtp_tls_true] = ACTIONS(1521),
    [sym_smtp_tls_false] = ACTIONS(1521),
    [sym_listview_on_row_select_event] = ACTIONS(1521),
    [sym_data_structure_unknown] = ACTIONS(1521),
    [sym_data_structure_datafeed] = ACTIONS(1521),
    [sym_data_structure_datarecordset] = ACTIONS(1521),
    [sym_data_structure_datarecord] = ACTIONS(1521),
    [sym_data_structure_datafield] = ACTIONS(1521),
    [sym_data_max_content_field_count] = ACTIONS(1521),
    [sym_data_max_metadata_field_count] = ACTIONS(1521),
    [sym_data_max_id_length] = ACTIONS(1521),
    [sym_data_max_name_length] = ACTIONS(1521),
    [sym_data_max_description_length] = ACTIONS(1521),
    [sym_data_max_type_length] = ACTIONS(1521),
    [sym_data_max_format_length] = ACTIONS(1521),
    [sym_data_max_source_length] = ACTIONS(1521),
    [sym_data_max_value_length] = ACTIONS(1521),
    [sym_data_max_label_length] = ACTIONS(1521),
    [sym_data_max_url_length] = ACTIONS(1521),
    [sym_max_last_login_info_length] = ACTIONS(1521),
    [sym_data_type_unknown] = ACTIONS(1521),
    [sym_data_type_string] = ACTIONS(1521),
    [sym_data_type_datetime] = ACTIONS(1521),
    [sym_data_type_date] = ACTIONS(1521),
    [sym_data_type_time] = ACTIONS(1521),
    [sym_data_type_image] = ACTIONS(1521),
    [sym_data_format_url] = ACTIONS(1521),
    [sym_data_format_phone] = ACTIONS(1521),
    [sym_data_format_email] = ACTIONS(1521),
    [sym_data_format_iso8601] = ACTIONS(1521),
    [sym_auditlog_priv] = ACTIONS(1521),
    [sym_configuration_priv] = ACTIONS(1521),
    [sym_deviceconfig_priv] = ACTIONS(1521),
    [sym_ftp_priv] = ACTIONS(1521),
    [sym_http_priv] = ACTIONS(1521),
    [sym_networkconfig_priv] = ACTIONS(1521),
    [sym_programport_priv] = ACTIONS(1521),
    [sym_remoteui_priv] = ACTIONS(1521),
    [sym_securitycontrol_priv] = ACTIONS(1521),
    [sym_softwaremanagement_priv] = ACTIONS(1521),
    [sym_terminal_priv] = ACTIONS(1521),
    [sym_tpadmin_priv] = ACTIONS(1521),
    [sym_user1_priv] = ACTIONS(1521),
    [sym_user2_priv] = ACTIONS(1521),
    [sym_user3_priv] = ACTIONS(1521),
    [sym_user4_priv] = ACTIONS(1521),
    [sym_usermanagement_priv] = ACTIONS(1521),
    [sym_valid_account] = ACTIONS(1521),
    [sym_err_name_invalid] = ACTIONS(1521),
    [sym_err_password_invalid] = ACTIONS(1521),
    [sym_err_invalid_account] = ACTIONS(1521),
    [sym_err_no_authorization] = ACTIONS(1521),
    [sym_err_invalid_parameter] = ACTIONS(1521),
    [sym_err_user_locked_out] = ACTIONS(1521),
    [sym_err_user_account_expired] = ACTIONS(1521),
    [sym_err_general_error] = ACTIONS(1521),
    [sym_err_authentication_error] = ACTIONS(1521),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1521),
    [sym_tls_validate_certificate] = ACTIONS(1521),
    [sym_tdata] = ACTIONS(1521),
    [sym_tchannel] = ACTIONS(1521),
    [sym_tlevel] = ACTIONS(1521),
    [sym_tbutton] = ACTIONS(1521),
    [sym_ttimeline] = ACTIONS(1521),
    [sym_tcustom] = ACTIONS(1521),
    [sym_url_struct] = ACTIONS(1521),
    [sym_dns_struct] = ACTIONS(1521),
    [sym_ip_address_struct] = ACTIONS(1521),
    [sym_dev_info_struct] = ACTIONS(1521),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1521),
    [sym_data_feed] = ACTIONS(1521),
    [sym_data_field] = ACTIONS(1521),
    [sym_data_record] = ACTIONS(1521),
    [sym_wc_data_feed] = ACTIONS(1521),
    [sym_wc_data_field] = ACTIONS(1521),
    [sym_wc_data_record] = ACTIONS(1521),
    [sym_last_login_info] = ACTIONS(1521),
    [sym____reserved___] = ACTIONS(1521),
    [sym_date] = ACTIONS(1521),
    [sym_day] = ACTIONS(1521),
    [sym_time] = ACTIONS(1521),
    [sym_ldate] = ACTIONS(1521),
    [sym_push_device] = ACTIONS(1521),
    [sym_push_channel] = ACTIONS(1521),
    [sym_push_devchan] = ACTIONS(1521),
    [sym_release_device] = ACTIONS(1521),
    [sym_release_channel] = ACTIONS(1521),
    [sym_release_devchan] = ACTIONS(1521),
    [sym_master_slot] = ACTIONS(1521),
    [sym_get_pulse_time] = ACTIONS(1521),
    [sym_get_timer] = ACTIONS(1521),
    [sym_master_sn] = ACTIONS(1521),
    [sym_system_number] = ACTIONS(1521),
    [sym_button] = ACTIONS(1521),
    [sym_dv_channel] = ACTIONS(1521),
    [sym_data] = ACTIONS(1521),
    [sym_level] = ACTIONS(1521),
    [sym_timeline] = ACTIONS(1521),
    [sym_channel] = ACTIONS(1521),
    [sym_custom] = ACTIONS(1521),
    [sym_atoi] = ACTIONS(1521),
    [sym_atol] = ACTIONS(1521),
    [sym_atof] = ACTIONS(1521),
    [sym_itoa] = ACTIONS(1521),
    [sym_itohex] = ACTIONS(1521),
    [sym_hextoi] = ACTIONS(1521),
    [sym_ftoa] = ACTIONS(1521),
    [sym_find_string] = ACTIONS(1521),
    [sym_left_string] = ACTIONS(1521),
    [sym_right_string] = ACTIONS(1521),
    [sym_mid_string] = ACTIONS(1521),
    [sym_remove_string] = ACTIONS(1521),
    [sym_get_buffer_string] = ACTIONS(1521),
    [sym_get_buffer_char] = ACTIONS(1521),
    [sym_get_multi_buffer_string] = ACTIONS(1521),
    [sym_length_string] = ACTIONS(1521),
    [sym_max_length_string] = ACTIONS(1521),
    [sym_set_length_string] = ACTIONS(1521),
    [sym_compare_string] = ACTIONS(1521),
    [sym_upper_string] = ACTIONS(1521),
    [sym_lower_string] = ACTIONS(1521),
    [sym_get_last] = ACTIONS(1521),
    [sym_date_to_day] = ACTIONS(1521),
    [sym_date_to_month] = ACTIONS(1521),
    [sym_date_to_year] = ACTIONS(1521),
    [sym_time_to_hour] = ACTIONS(1521),
    [sym_time_to_minute] = ACTIONS(1521),
    [sym_time_to_second] = ACTIONS(1521),
    [sym_day_of_week] = ACTIONS(1521),
    [sym_set_pulse_time] = ACTIONS(1521),
    [sym_set_timer] = ACTIONS(1521),
    [sym_file_open] = ACTIONS(1521),
    [sym_file_close] = ACTIONS(1521),
    [sym_file_read] = ACTIONS(1521),
    [sym_file_read_line] = ACTIONS(1521),
    [sym_file_write] = ACTIONS(1521),
    [sym_file_write_line] = ACTIONS(1521),
    [sym_file_copy] = ACTIONS(1521),
    [sym_file_delete] = ACTIONS(1521),
    [sym_file_rename] = ACTIONS(1521),
    [sym_file_dir] = ACTIONS(1521),
    [sym_file_getdir] = ACTIONS(1521),
    [sym_file_setdir] = ACTIONS(1521),
    [sym_file_create_dir] = ACTIONS(1521),
    [sym_file_remove_dir] = ACTIONS(1521),
    [sym_file_seek] = ACTIONS(1521),
    [sym_length_array] = ACTIONS(1521),
    [sym_max_length_array] = ACTIONS(1521),
    [sym_set_length_array] = ACTIONS(1521),
    [sym_ip_client_open] = ACTIONS(1521),
    [sym_ip_client_close] = ACTIONS(1521),
    [sym_ip_server_open] = ACTIONS(1521),
    [sym_ip_server_close] = ACTIONS(1521),
    [sym_ip_bound_client_open] = ACTIONS(1521),
    [sym_ip_mc_server_open] = ACTIONS(1521),
    [sym_ip_set_option] = ACTIONS(1521),
    [sym_ssh_client_open] = ACTIONS(1521),
    [sym_ssh_client_close] = ACTIONS(1521),
    [sym_tls_client_open] = ACTIONS(1521),
    [sym_tls_client_close] = ACTIONS(1521),
    [sym_do_push] = ACTIONS(1521),
    [sym_do_push_timed] = ACTIONS(1521),
    [sym_do_release] = ACTIONS(1521),
    [sym_device_id] = ACTIONS(1521),
    [sym_device_info] = ACTIONS(1521),
    [sym_device_id_string] = ACTIONS(1521),
    [sym_device_standby] = ACTIONS(1521),
    [sym_device_wake] = ACTIONS(1521),
    [sym_timeline_create] = ACTIONS(1521),
    [sym_timeline_kill] = ACTIONS(1521),
    [sym_timeline_reload] = ACTIONS(1521),
    [sym_timeline_pause] = ACTIONS(1521),
    [sym_timeline_restart] = ACTIONS(1521),
    [sym_timeline_set] = ACTIONS(1521),
    [sym_timeline_get] = ACTIONS(1521),
    [sym_timeline_active] = ACTIONS(1521),
    [sym_timeline_dynamic_id] = ACTIONS(1521),
    [sym_variable_to_string] = ACTIONS(1521),
    [sym_string_to_variable] = ACTIONS(1521),
    [sym_length_variable_to_string] = ACTIONS(1521),
    [sym_type_cast] = ACTIONS(1521),
    [sym_variable_to_xml] = ACTIONS(1521),
    [sym_xml_to_variable] = ACTIONS(1521),
    [sym_length_variable_to_xml] = ACTIONS(1521),
    [sym_redirect_string] = ACTIONS(1521),
    [sym_random_number] = ACTIONS(1521),
    [sym_get_unique_id] = ACTIONS(1521),
    [sym_get_system_number] = ACTIONS(1521),
    [sym_set_system_number] = ACTIONS(1521),
    [sym_get_serial_number] = ACTIONS(1521),
    [sym_get_master_build] = ACTIONS(1521),
    [sym_set_validation_code] = ACTIONS(1521),
    [sym_reboot] = ACTIONS(1521),
    [sym_load_duet_module] = ACTIONS(1521),
    [sym_unload_duet_module] = ACTIONS(1521),
    [sym_duet_mem_size_set] = ACTIONS(1521),
    [sym_duet_mem_size_get] = ACTIONS(1521),
    [sym_dynamic_application_device] = ACTIONS(1521),
    [sym_dynamic_polled_port] = ACTIONS(1521),
    [sym_static_port_binding] = ACTIONS(1521),
    [sym_static_ip_binding] = ACTIONS(1521),
    [sym_get_dns_list] = ACTIONS(1521),
    [sym_set_dns_list] = ACTIONS(1521),
    [sym_get_ip_address] = ACTIONS(1521),
    [sym_set_ip_address] = ACTIONS(1521),
    [sym_get_url_list] = ACTIONS(1521),
    [sym_add_url_entry] = ACTIONS(1521),
    [sym_delete_url_entry] = ACTIONS(1521),
    [sym_abs_value] = ACTIONS(1521),
    [sym_max_value] = ACTIONS(1521),
    [sym_min_value] = ACTIONS(1521),
    [sym_exp_value] = ACTIONS(1521),
    [sym_log_value] = ACTIONS(1521),
    [sym_log10_value] = ACTIONS(1521),
    [sym_power_value] = ACTIONS(1521),
    [sym_sqrt_value] = ACTIONS(1521),
    [sym_raw_be] = ACTIONS(1521),
    [sym_raw_le] = ACTIONS(1521),
    [sym_internal_queue_size_set] = ACTIONS(1521),
    [sym_internal_queue_size_get] = ACTIONS(1521),
    [sym_internal_threshold_set] = ACTIONS(1521),
    [sym_internal_threshold_get] = ACTIONS(1521),
    [sym_rebuilt_event] = ACTIONS(1521),
    [sym_do_custom_event] = ACTIONS(1521),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1521),
    [sym_clkmgr_set_clk_source] = ACTIONS(1521),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1521),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1521),
    [sym_clkmgr_get_timezone] = ACTIONS(1521),
    [sym_clkmgr_set_timezone] = ACTIONS(1521),
    [sym_clkmgr_get_resync_period] = ACTIONS(1521),
    [sym_clkmgr_set_resync_period] = ACTIONS(1521),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1521),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1521),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1521),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1521),
    [sym_clkmgr_get_timeservers] = ACTIONS(1521),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1521),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1521),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1521),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1521),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1521),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1521),
    [sym_set_log_level] = ACTIONS(1521),
    [sym_get_log_level] = ACTIONS(1521),
    [sym_amx_log] = ACTIONS(1521),
    [sym_smtp_server_config_set] = ACTIONS(1521),
    [sym_smtp_server_config_get] = ACTIONS(1521),
    [sym_smtp_send] = ACTIONS(1521),
    [sym_validate_netlinx_account] = ACTIONS(1521),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1521),
    [sym_authenticate_certificate] = ACTIONS(1521),
    [sym_audit_netlinx_session_event] = ACTIONS(1521),
    [sym_audit_netlinx_generic_event] = ACTIONS(1521),
    [sym_get_available_flash_disk_space] = ACTIONS(1521),
    [sym_get_max_flash_disk_space] = ACTIONS(1521),
    [sym_data_create_feed] = ACTIONS(1521),
    [sym_data_delete_feed] = ACTIONS(1521),
    [sym_data_publish_feed] = ACTIONS(1521),
    [sym_data_get_published_feed] = ACTIONS(1521),
    [sym_data_add_record] = ACTIONS(1521),
    [sym_data_get_event_record] = ACTIONS(1521),
    [sym__wc_data_create_feed] = ACTIONS(1521),
    [sym__wc_data_add_record] = ACTIONS(1521),
    [sym__wc_data_get_event_record] = ACTIONS(1521),
    [sym_format] = ACTIONS(1521),
    [sym_astro_clock] = ACTIONS(1521),
    [sym_set_outdoor_temperature] = ACTIONS(1521),
    [sym_set_virtual_level_count] = ACTIONS(1521),
    [sym_set_virtual_channel_count] = ACTIONS(1521),
    [sym_set_virtual_port_count] = ACTIONS(1521),
    [sym__wc_encode_int] = ACTIONS(1521),
    [sym__wc_decode_int] = ACTIONS(1521),
    [sym__wc_int] = ACTIONS(1521),
    [sym__wc_to_ch_int] = ACTIONS(1521),
    [sym__ch_to_wc_int] = ACTIONS(1521),
    [sym__wc_find_string_int] = ACTIONS(1521),
    [sym__wc_left_string_int] = ACTIONS(1521),
    [sym__wc_lower_string_int] = ACTIONS(1521),
    [sym__wc_mid_string_int] = ACTIONS(1521),
    [sym__wc_remove_string_int] = ACTIONS(1521),
    [sym__wc_right_string_int] = ACTIONS(1521),
    [sym__wc_upper_string_int] = ACTIONS(1521),
    [sym__wc_compare_string_int] = ACTIONS(1521),
    [sym__wc_get_buffer_char_int] = ACTIONS(1521),
    [sym__wc_get_buffer_string_int] = ACTIONS(1521),
    [sym__wc_concat_string_int] = ACTIONS(1521),
    [sym__wc_file_open_int] = ACTIONS(1521),
    [sym__wc_file_close_int] = ACTIONS(1521),
    [sym__wc_file_read_int] = ACTIONS(1521),
    [sym__wc_file_read_line_int] = ACTIONS(1521),
    [sym__wc_file_write_int] = ACTIONS(1521),
    [sym__wc_file_write_line_int] = ACTIONS(1521),
    [anon_sym_EQ] = ACTIONS(1521),
    [aux_sym_preproc_include_token1] = ACTIONS(1521),
    [aux_sym_preproc_define_token1] = ACTIONS(1523),
    [aux_sym_preproc_warn_token1] = ACTIONS(1523),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1523),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1523),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(1523),
    [anon_sym_LBRACE] = ACTIONS(1523),
    [anon_sym_COLON] = ACTIONS(1523),
    [anon_sym_LPAREN] = ACTIONS(1521),
    [anon_sym_SEMI] = ACTIONS(1523),
    [anon_sym_BANG] = ACTIONS(1521),
    [anon_sym_TILDE] = ACTIONS(1523),
    [anon_sym_DASH] = ACTIONS(1521),
    [anon_sym_PLUS] = ACTIONS(1521),
    [anon_sym_STAR] = ACTIONS(1523),
    [anon_sym_SLASH] = ACTIONS(1521),
    [anon_sym_PERCENT] = ACTIONS(1523),
    [anon_sym_PIPE_PIPE] = ACTIONS(1523),
    [anon_sym_AMP_AMP] = ACTIONS(1523),
    [anon_sym_PIPE] = ACTIONS(1521),
    [anon_sym_CARET] = ACTIONS(1523),
    [anon_sym_AMP] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1523),
    [anon_sym_BANG_EQ] = ACTIONS(1523),
    [anon_sym_LT_GT] = ACTIONS(1523),
    [anon_sym_GT] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1523),
    [anon_sym_LT_EQ] = ACTIONS(1523),
    [anon_sym_LT] = ACTIONS(1521),
    [anon_sym_LT_LT] = ACTIONS(1523),
    [anon_sym_GT_GT] = ACTIONS(1523),
    [anon_sym_DASH_DASH] = ACTIONS(1523),
    [anon_sym_PLUS_PLUS] = ACTIONS(1523),
    [anon_sym_DOT] = ACTIONS(1521),
    [anon_sym_DQUOTE] = ACTIONS(1523),
    [anon_sym_SQUOTE] = ACTIONS(1523),
    [sym_number_literal] = ACTIONS(1523),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1523),
  },
  [STATE(129)] = {
    [sym_identifier] = ACTIONS(1589),
    [sym_program_name_keyword] = ACTIONS(1589),
    [sym_module_name_keyword] = ACTIONS(1589),
    [sym_define_device_keyword] = ACTIONS(1589),
    [sym_define_combine_keyword] = ACTIONS(1589),
    [sym_define_constant_keyword] = ACTIONS(1589),
    [sym_define_type_keyword] = ACTIONS(1589),
    [sym_define_variable_keyword] = ACTIONS(1589),
    [sym_define_system_variable_keyword] = ACTIONS(1589),
    [sym_define_connect_level_keyword] = ACTIONS(1589),
    [sym_define_latching_keyword] = ACTIONS(1589),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1589),
    [sym_define_toggling_keyword] = ACTIONS(1589),
    [sym_define_call_keyword] = ACTIONS(1589),
    [sym_define_function_keyword] = ACTIONS(1589),
    [sym_define_library_function_keyword] = ACTIONS(1589),
    [sym_define_start_keyword] = ACTIONS(1589),
    [sym_define_event_keyword] = ACTIONS(1589),
    [sym_define_program_keyword] = ACTIONS(1589),
    [sym_define_module_keyword] = ACTIONS(1589),
    [sym_char_keyword] = ACTIONS(1589),
    [sym_widechar_keyword] = ACTIONS(1589),
    [sym_integer_keyword] = ACTIONS(1589),
    [sym_sinteger_keyword] = ACTIONS(1589),
    [sym_long_keyword] = ACTIONS(1589),
    [sym_slong_keyword] = ACTIONS(1589),
    [sym_float_keyword] = ACTIONS(1589),
    [sym_double_keyword] = ACTIONS(1589),
    [sym_variant_keyword] = ACTIONS(1589),
    [sym_variantarray_keyword] = ACTIONS(1589),
    [sym_local_var_keyword] = ACTIONS(1589),
    [sym_stack_var_keyword] = ACTIONS(1589),
    [sym_constant_keyword] = ACTIONS(1589),
    [sym_volatile_keyword] = ACTIONS(1589),
    [sym_non_volatile_keyword] = ACTIONS(1589),
    [sym_persistent_keyword] = ACTIONS(1589),
    [sym_dev_keyword] = ACTIONS(1589),
    [sym_devlev_keyword] = ACTIONS(1589),
    [sym_devchan_keyword] = ACTIONS(1589),
    [sym_if_keyword] = ACTIONS(1589),
    [sym_else_keyword] = ACTIONS(1589),
    [sym_switch_keyword] = ACTIONS(1589),
    [sym_case_keyword] = ACTIONS(1589),
    [sym_default_keyword] = ACTIONS(1589),
    [sym_while_keyword] = ACTIONS(1589),
    [sym_for_keyword] = ACTIONS(1589),
    [sym_break_keyword] = ACTIONS(1589),
    [sym_continue_keyword] = ACTIONS(1589),
    [sym_return_keyword] = ACTIONS(1589),
    [sym_select_keyword] = ACTIONS(1589),
    [sym_active_keyword] = ACTIONS(1589),
    [sym_struct_keyword] = ACTIONS(1589),
    [sym_structure_keyword] = ACTIONS(1589),
    [sym_button_event_keyword] = ACTIONS(1589),
    [sym_channel_event_keyword] = ACTIONS(1589),
    [sym_level_event_keyword] = ACTIONS(1589),
    [sym_data_event_keyword] = ACTIONS(1589),
    [sym_timeline_event_keyword] = ACTIONS(1589),
    [sym_custom_event_keyword] = ACTIONS(1589),
    [sym_push_keyword] = ACTIONS(1589),
    [sym_release_keyword] = ACTIONS(1589),
    [aux_sym_on_keyword_token1] = ACTIONS(1589),
    [aux_sym_off_keyword_token1] = ACTIONS(1589),
    [sym_send_level_keyword] = ACTIONS(1589),
    [sym_send_string_keyword] = ACTIONS(1589),
    [sym_send_command_keyword] = ACTIONS(1589),
    [sym_clear_buffer_keyword] = ACTIONS(1589),
    [sym_create_buffer_keyword] = ACTIONS(1589),
    [sym_create_multi_buffer_keyword] = ACTIONS(1589),
    [sym_call_keyword] = ACTIONS(1589),
    [sym_system_call_keyword] = ACTIONS(1589),
    [sym_devchan_to_keyword] = ACTIONS(1589),
    [sym_devchan_min_to_keyword] = ACTIONS(1589),
    [sym_devchan_total_off_keyword] = ACTIONS(1589),
    [sym_devchan_pulse_keyword] = ACTIONS(1589),
    [sym_wait_keyword] = ACTIONS(1589),
    [sym_cancel_wait_keyword] = ACTIONS(1589),
    [sym_wait_until_keyword] = ACTIONS(1589),
    [sym_cancel_wait_until_keyword] = ACTIONS(1589),
    [sym_cancel_all_wait_keyword] = ACTIONS(1589),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1589),
    [sym_and] = ACTIONS(1589),
    [sym_or] = ACTIONS(1589),
    [sym_not] = ACTIONS(1589),
    [sym_xor] = ACTIONS(1589),
    [sym_band] = ACTIONS(1589),
    [sym_bor] = ACTIONS(1589),
    [sym_bxor] = ACTIONS(1589),
    [sym_bnot] = ACTIONS(1589),
    [sym_lshift] = ACTIONS(1589),
    [sym_rshift] = ACTIONS(1589),
    [sym___file__] = ACTIONS(1589),
    [sym___date__] = ACTIONS(1589),
    [sym___ldate__] = ACTIONS(1589),
    [sym___time__] = ACTIONS(1589),
    [sym___line__] = ACTIONS(1589),
    [sym___name__] = ACTIONS(1589),
    [sym___netlinx__] = ACTIONS(1589),
    [sym_netlinx_axi_version] = ACTIONS(1589),
    [sym_url_flg_tcp] = ACTIONS(1589),
    [sym_url_flg_accinfopresent] = ACTIONS(1589),
    [sym_url_flg_temp] = ACTIONS(1589),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1589),
    [sym_url_flg_stat_mask] = ACTIONS(1589),
    [sym_url_flg_stat_lookup] = ACTIONS(1589),
    [sym_url_flg_stat_connecting] = ACTIONS(1589),
    [sym_url_flg_stat_waiting] = ACTIONS(1589),
    [sym_url_flg_stat_connected] = ACTIONS(1589),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1589),
    [sym_first_virtual_device] = ACTIONS(1589),
    [sym_first_local_port] = ACTIONS(1589),
    [sym_dynamic_virtual_device] = ACTIONS(1589),
    [sym_timeline_once] = ACTIONS(1589),
    [sym_timeline_repeat] = ACTIONS(1589),
    [sym_timeline_absolute] = ACTIONS(1589),
    [sym_timeline_relative] = ACTIONS(1589),
    [sym_do_push_timed_infinite] = ACTIONS(1589),
    [sym_source_type_no_address] = ACTIONS(1589),
    [sym_source_type_neuron_id] = ACTIONS(1589),
    [sym_source_type_ip_address] = ACTIONS(1589),
    [sym_source_type_axlink] = ACTIONS(1589),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1589),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1589),
    [sym_source_type_ip_socket_address] = ACTIONS(1589),
    [sym_source_type_rs232] = ACTIONS(1589),
    [sym_source_type_internal] = ACTIONS(1589),
    [sym_source_type_mac_address] = ACTIONS(1589),
    [sym_source_type_ipv4_port] = ACTIONS(1589),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1589),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1589),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1589),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1589),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1589),
    [sym_true] = ACTIONS(1589),
    [sym_false] = ACTIONS(1589),
    [sym_file_read_only] = ACTIONS(1589),
    [sym_file_rw_new] = ACTIONS(1589),
    [sym_file_rw_append] = ACTIONS(1589),
    [sym_ip_tcp] = ACTIONS(1589),
    [sym_ip_udp] = ACTIONS(1589),
    [sym_ip_udp_2way] = ACTIONS(1589),
    [sym_xml_encode_types] = ACTIONS(1589),
    [sym_xml_encode_char_as_list] = ACTIONS(1589),
    [sym_xml_encode_le] = ACTIONS(1589),
    [sym_xml_decode_types] = ACTIONS(1589),
    [sym_xml_decode_no_preserve] = ACTIONS(1589),
    [sym_duet_dev_type_display_device] = ACTIONS(1589),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1589),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1589),
    [sym_duet_dev_type_io_device] = ACTIONS(1589),
    [sym_duet_dev_type_relay_device] = ACTIONS(1589),
    [sym_duet_dev_type_ups] = ACTIONS(1589),
    [sym_duet_dev_type_amplifier] = ACTIONS(1589),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1589),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1589),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1589),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1589),
    [sym_duet_dev_type_camera] = ACTIONS(1589),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1589),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1589),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1589),
    [sym_duet_dev_type_dss] = ACTIONS(1589),
    [sym_duet_dev_type_dvr] = ACTIONS(1589),
    [sym_duet_dev_type_disc_device] = ACTIONS(1589),
    [sym_duet_dev_type_document_camera] = ACTIONS(1589),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1589),
    [sym_duet_dev_type_hvac] = ACTIONS(1589),
    [sym_duet_dev_type_keypad] = ACTIONS(1589),
    [sym_duet_dev_type_light] = ACTIONS(1589),
    [sym_duet_dev_type_monitor] = ACTIONS(1589),
    [sym_duet_dev_type_motor] = ACTIONS(1589),
    [sym_duet_dev_type_multi_window] = ACTIONS(1589),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1589),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1589),
    [sym_duet_dev_type_receiver] = ACTIONS(1589),
    [sym_duet_dev_type_security_system] = ACTIONS(1589),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1589),
    [sym_duet_dev_type_settop_box] = ACTIONS(1589),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1589),
    [sym_duet_dev_type_switcher] = ACTIONS(1589),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1589),
    [sym_duet_dev_type_tv] = ACTIONS(1589),
    [sym_duet_dev_type_utility] = ACTIONS(1589),
    [sym_duet_dev_type_vcr] = ACTIONS(1589),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1589),
    [sym_duet_dev_type_video_processor] = ACTIONS(1589),
    [sym_duet_dev_type_video_projector] = ACTIONS(1589),
    [sym_duet_dev_type_video_wall] = ACTIONS(1589),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1589),
    [sym_duet_dev_type_weather] = ACTIONS(1589),
    [sym_duet_dev_not_polled] = ACTIONS(1589),
    [sym_duet_dev_polled] = ACTIONS(1589),
    [sym_clkmgr_mode_network] = ACTIONS(1589),
    [sym_clkmgr_mode_standalone] = ACTIONS(1589),
    [sym_normal_standby] = ACTIONS(1589),
    [sym_normal_wake] = ACTIONS(1589),
    [sym_ip_multicast_ttl_option] = ACTIONS(1589),
    [sym_ip_ttl_subnet] = ACTIONS(1589),
    [sym_ip_ttl_site] = ACTIONS(1589),
    [sym_ip_ttl_region] = ACTIONS(1589),
    [sym_ip_ttl_continent] = ACTIONS(1589),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1589),
    [sym_ip_nodelay_on] = ACTIONS(1589),
    [sym_ip_nodelay_off] = ACTIONS(1589),
    [sym_amx_error] = ACTIONS(1589),
    [sym_amx_warning] = ACTIONS(1589),
    [sym_amx_info] = ACTIONS(1589),
    [sym_amx_debug] = ACTIONS(1589),
    [sym_netlinx_login_success] = ACTIONS(1589),
    [sym_netlinx_login_fail] = ACTIONS(1589),
    [sym_netlinx_logout] = ACTIONS(1589),
    [sym_null_str] = ACTIONS(1589),
    [sym_smtp_address] = ACTIONS(1589),
    [sym_smtp_port_number] = ACTIONS(1589),
    [sym_smtp_username] = ACTIONS(1589),
    [sym_smtp_password] = ACTIONS(1589),
    [sym_smtp_require_tls] = ACTIONS(1589),
    [sym_smtp_from] = ACTIONS(1589),
    [sym_smtp_tls_true] = ACTIONS(1589),
    [sym_smtp_tls_false] = ACTIONS(1589),
    [sym_listview_on_row_select_event] = ACTIONS(1589),
    [sym_data_structure_unknown] = ACTIONS(1589),
    [sym_data_structure_datafeed] = ACTIONS(1589),
    [sym_data_structure_datarecordset] = ACTIONS(1589),
    [sym_data_structure_datarecord] = ACTIONS(1589),
    [sym_data_structure_datafield] = ACTIONS(1589),
    [sym_data_max_content_field_count] = ACTIONS(1589),
    [sym_data_max_metadata_field_count] = ACTIONS(1589),
    [sym_data_max_id_length] = ACTIONS(1589),
    [sym_data_max_name_length] = ACTIONS(1589),
    [sym_data_max_description_length] = ACTIONS(1589),
    [sym_data_max_type_length] = ACTIONS(1589),
    [sym_data_max_format_length] = ACTIONS(1589),
    [sym_data_max_source_length] = ACTIONS(1589),
    [sym_data_max_value_length] = ACTIONS(1589),
    [sym_data_max_label_length] = ACTIONS(1589),
    [sym_data_max_url_length] = ACTIONS(1589),
    [sym_max_last_login_info_length] = ACTIONS(1589),
    [sym_data_type_unknown] = ACTIONS(1589),
    [sym_data_type_string] = ACTIONS(1589),
    [sym_data_type_datetime] = ACTIONS(1589),
    [sym_data_type_date] = ACTIONS(1589),
    [sym_data_type_time] = ACTIONS(1589),
    [sym_data_type_image] = ACTIONS(1589),
    [sym_data_format_url] = ACTIONS(1589),
    [sym_data_format_phone] = ACTIONS(1589),
    [sym_data_format_email] = ACTIONS(1589),
    [sym_data_format_iso8601] = ACTIONS(1589),
    [sym_auditlog_priv] = ACTIONS(1589),
    [sym_configuration_priv] = ACTIONS(1589),
    [sym_deviceconfig_priv] = ACTIONS(1589),
    [sym_ftp_priv] = ACTIONS(1589),
    [sym_http_priv] = ACTIONS(1589),
    [sym_networkconfig_priv] = ACTIONS(1589),
    [sym_programport_priv] = ACTIONS(1589),
    [sym_remoteui_priv] = ACTIONS(1589),
    [sym_securitycontrol_priv] = ACTIONS(1589),
    [sym_softwaremanagement_priv] = ACTIONS(1589),
    [sym_terminal_priv] = ACTIONS(1589),
    [sym_tpadmin_priv] = ACTIONS(1589),
    [sym_user1_priv] = ACTIONS(1589),
    [sym_user2_priv] = ACTIONS(1589),
    [sym_user3_priv] = ACTIONS(1589),
    [sym_user4_priv] = ACTIONS(1589),
    [sym_usermanagement_priv] = ACTIONS(1589),
    [sym_valid_account] = ACTIONS(1589),
    [sym_err_name_invalid] = ACTIONS(1589),
    [sym_err_password_invalid] = ACTIONS(1589),
    [sym_err_invalid_account] = ACTIONS(1589),
    [sym_err_no_authorization] = ACTIONS(1589),
    [sym_err_invalid_parameter] = ACTIONS(1589),
    [sym_err_user_locked_out] = ACTIONS(1589),
    [sym_err_user_account_expired] = ACTIONS(1589),
    [sym_err_general_error] = ACTIONS(1589),
    [sym_err_authentication_error] = ACTIONS(1589),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1589),
    [sym_tls_validate_certificate] = ACTIONS(1589),
    [sym_tdata] = ACTIONS(1589),
    [sym_tchannel] = ACTIONS(1589),
    [sym_tlevel] = ACTIONS(1589),
    [sym_tbutton] = ACTIONS(1589),
    [sym_ttimeline] = ACTIONS(1589),
    [sym_tcustom] = ACTIONS(1589),
    [sym_url_struct] = ACTIONS(1589),
    [sym_dns_struct] = ACTIONS(1589),
    [sym_ip_address_struct] = ACTIONS(1589),
    [sym_dev_info_struct] = ACTIONS(1589),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1589),
    [sym_data_feed] = ACTIONS(1589),
    [sym_data_field] = ACTIONS(1589),
    [sym_data_record] = ACTIONS(1589),
    [sym_wc_data_feed] = ACTIONS(1589),
    [sym_wc_data_field] = ACTIONS(1589),
    [sym_wc_data_record] = ACTIONS(1589),
    [sym_last_login_info] = ACTIONS(1589),
    [sym____reserved___] = ACTIONS(1589),
    [sym_date] = ACTIONS(1589),
    [sym_day] = ACTIONS(1589),
    [sym_time] = ACTIONS(1589),
    [sym_ldate] = ACTIONS(1589),
    [sym_push_device] = ACTIONS(1589),
    [sym_push_channel] = ACTIONS(1589),
    [sym_push_devchan] = ACTIONS(1589),
    [sym_release_device] = ACTIONS(1589),
    [sym_release_channel] = ACTIONS(1589),
    [sym_release_devchan] = ACTIONS(1589),
    [sym_master_slot] = ACTIONS(1589),
    [sym_get_pulse_time] = ACTIONS(1589),
    [sym_get_timer] = ACTIONS(1589),
    [sym_master_sn] = ACTIONS(1589),
    [sym_system_number] = ACTIONS(1589),
    [sym_button] = ACTIONS(1589),
    [sym_dv_channel] = ACTIONS(1589),
    [sym_data] = ACTIONS(1589),
    [sym_level] = ACTIONS(1589),
    [sym_timeline] = ACTIONS(1589),
    [sym_channel] = ACTIONS(1589),
    [sym_custom] = ACTIONS(1589),
    [sym_atoi] = ACTIONS(1589),
    [sym_atol] = ACTIONS(1589),
    [sym_atof] = ACTIONS(1589),
    [sym_itoa] = ACTIONS(1589),
    [sym_itohex] = ACTIONS(1589),
    [sym_hextoi] = ACTIONS(1589),
    [sym_ftoa] = ACTIONS(1589),
    [sym_find_string] = ACTIONS(1589),
    [sym_left_string] = ACTIONS(1589),
    [sym_right_string] = ACTIONS(1589),
    [sym_mid_string] = ACTIONS(1589),
    [sym_remove_string] = ACTIONS(1589),
    [sym_get_buffer_string] = ACTIONS(1589),
    [sym_get_buffer_char] = ACTIONS(1589),
    [sym_get_multi_buffer_string] = ACTIONS(1589),
    [sym_length_string] = ACTIONS(1589),
    [sym_max_length_string] = ACTIONS(1589),
    [sym_set_length_string] = ACTIONS(1589),
    [sym_compare_string] = ACTIONS(1589),
    [sym_upper_string] = ACTIONS(1589),
    [sym_lower_string] = ACTIONS(1589),
    [sym_get_last] = ACTIONS(1589),
    [sym_date_to_day] = ACTIONS(1589),
    [sym_date_to_month] = ACTIONS(1589),
    [sym_date_to_year] = ACTIONS(1589),
    [sym_time_to_hour] = ACTIONS(1589),
    [sym_time_to_minute] = ACTIONS(1589),
    [sym_time_to_second] = ACTIONS(1589),
    [sym_day_of_week] = ACTIONS(1589),
    [sym_set_pulse_time] = ACTIONS(1589),
    [sym_set_timer] = ACTIONS(1589),
    [sym_file_open] = ACTIONS(1589),
    [sym_file_close] = ACTIONS(1589),
    [sym_file_read] = ACTIONS(1589),
    [sym_file_read_line] = ACTIONS(1589),
    [sym_file_write] = ACTIONS(1589),
    [sym_file_write_line] = ACTIONS(1589),
    [sym_file_copy] = ACTIONS(1589),
    [sym_file_delete] = ACTIONS(1589),
    [sym_file_rename] = ACTIONS(1589),
    [sym_file_dir] = ACTIONS(1589),
    [sym_file_getdir] = ACTIONS(1589),
    [sym_file_setdir] = ACTIONS(1589),
    [sym_file_create_dir] = ACTIONS(1589),
    [sym_file_remove_dir] = ACTIONS(1589),
    [sym_file_seek] = ACTIONS(1589),
    [sym_length_array] = ACTIONS(1589),
    [sym_max_length_array] = ACTIONS(1589),
    [sym_set_length_array] = ACTIONS(1589),
    [sym_ip_client_open] = ACTIONS(1589),
    [sym_ip_client_close] = ACTIONS(1589),
    [sym_ip_server_open] = ACTIONS(1589),
    [sym_ip_server_close] = ACTIONS(1589),
    [sym_ip_bound_client_open] = ACTIONS(1589),
    [sym_ip_mc_server_open] = ACTIONS(1589),
    [sym_ip_set_option] = ACTIONS(1589),
    [sym_ssh_client_open] = ACTIONS(1589),
    [sym_ssh_client_close] = ACTIONS(1589),
    [sym_tls_client_open] = ACTIONS(1589),
    [sym_tls_client_close] = ACTIONS(1589),
    [sym_do_push] = ACTIONS(1589),
    [sym_do_push_timed] = ACTIONS(1589),
    [sym_do_release] = ACTIONS(1589),
    [sym_device_id] = ACTIONS(1589),
    [sym_device_info] = ACTIONS(1589),
    [sym_device_id_string] = ACTIONS(1589),
    [sym_device_standby] = ACTIONS(1589),
    [sym_device_wake] = ACTIONS(1589),
    [sym_timeline_create] = ACTIONS(1589),
    [sym_timeline_kill] = ACTIONS(1589),
    [sym_timeline_reload] = ACTIONS(1589),
    [sym_timeline_pause] = ACTIONS(1589),
    [sym_timeline_restart] = ACTIONS(1589),
    [sym_timeline_set] = ACTIONS(1589),
    [sym_timeline_get] = ACTIONS(1589),
    [sym_timeline_active] = ACTIONS(1589),
    [sym_timeline_dynamic_id] = ACTIONS(1589),
    [sym_variable_to_string] = ACTIONS(1589),
    [sym_string_to_variable] = ACTIONS(1589),
    [sym_length_variable_to_string] = ACTIONS(1589),
    [sym_type_cast] = ACTIONS(1589),
    [sym_variable_to_xml] = ACTIONS(1589),
    [sym_xml_to_variable] = ACTIONS(1589),
    [sym_length_variable_to_xml] = ACTIONS(1589),
    [sym_redirect_string] = ACTIONS(1589),
    [sym_random_number] = ACTIONS(1589),
    [sym_get_unique_id] = ACTIONS(1589),
    [sym_get_system_number] = ACTIONS(1589),
    [sym_set_system_number] = ACTIONS(1589),
    [sym_get_serial_number] = ACTIONS(1589),
    [sym_get_master_build] = ACTIONS(1589),
    [sym_set_validation_code] = ACTIONS(1589),
    [sym_reboot] = ACTIONS(1589),
    [sym_load_duet_module] = ACTIONS(1589),
    [sym_unload_duet_module] = ACTIONS(1589),
    [sym_duet_mem_size_set] = ACTIONS(1589),
    [sym_duet_mem_size_get] = ACTIONS(1589),
    [sym_dynamic_application_device] = ACTIONS(1589),
    [sym_dynamic_polled_port] = ACTIONS(1589),
    [sym_static_port_binding] = ACTIONS(1589),
    [sym_static_ip_binding] = ACTIONS(1589),
    [sym_get_dns_list] = ACTIONS(1589),
    [sym_set_dns_list] = ACTIONS(1589),
    [sym_get_ip_address] = ACTIONS(1589),
    [sym_set_ip_address] = ACTIONS(1589),
    [sym_get_url_list] = ACTIONS(1589),
    [sym_add_url_entry] = ACTIONS(1589),
    [sym_delete_url_entry] = ACTIONS(1589),
    [sym_abs_value] = ACTIONS(1589),
    [sym_max_value] = ACTIONS(1589),
    [sym_min_value] = ACTIONS(1589),
    [sym_exp_value] = ACTIONS(1589),
    [sym_log_value] = ACTIONS(1589),
    [sym_log10_value] = ACTIONS(1589),
    [sym_power_value] = ACTIONS(1589),
    [sym_sqrt_value] = ACTIONS(1589),
    [sym_raw_be] = ACTIONS(1589),
    [sym_raw_le] = ACTIONS(1589),
    [sym_internal_queue_size_set] = ACTIONS(1589),
    [sym_internal_queue_size_get] = ACTIONS(1589),
    [sym_internal_threshold_set] = ACTIONS(1589),
    [sym_internal_threshold_get] = ACTIONS(1589),
    [sym_rebuilt_event] = ACTIONS(1589),
    [sym_do_custom_event] = ACTIONS(1589),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1589),
    [sym_clkmgr_set_clk_source] = ACTIONS(1589),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1589),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1589),
    [sym_clkmgr_get_timezone] = ACTIONS(1589),
    [sym_clkmgr_set_timezone] = ACTIONS(1589),
    [sym_clkmgr_get_resync_period] = ACTIONS(1589),
    [sym_clkmgr_set_resync_period] = ACTIONS(1589),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1589),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1589),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1589),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1589),
    [sym_clkmgr_get_timeservers] = ACTIONS(1589),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1589),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1589),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1589),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1589),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1589),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1589),
    [sym_set_log_level] = ACTIONS(1589),
    [sym_get_log_level] = ACTIONS(1589),
    [sym_amx_log] = ACTIONS(1589),
    [sym_smtp_server_config_set] = ACTIONS(1589),
    [sym_smtp_server_config_get] = ACTIONS(1589),
    [sym_smtp_send] = ACTIONS(1589),
    [sym_validate_netlinx_account] = ACTIONS(1589),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1589),
    [sym_authenticate_certificate] = ACTIONS(1589),
    [sym_audit_netlinx_session_event] = ACTIONS(1589),
    [sym_audit_netlinx_generic_event] = ACTIONS(1589),
    [sym_get_available_flash_disk_space] = ACTIONS(1589),
    [sym_get_max_flash_disk_space] = ACTIONS(1589),
    [sym_data_create_feed] = ACTIONS(1589),
    [sym_data_delete_feed] = ACTIONS(1589),
    [sym_data_publish_feed] = ACTIONS(1589),
    [sym_data_get_published_feed] = ACTIONS(1589),
    [sym_data_add_record] = ACTIONS(1589),
    [sym_data_get_event_record] = ACTIONS(1589),
    [sym__wc_data_create_feed] = ACTIONS(1589),
    [sym__wc_data_add_record] = ACTIONS(1589),
    [sym__wc_data_get_event_record] = ACTIONS(1589),
    [sym_format] = ACTIONS(1589),
    [sym_astro_clock] = ACTIONS(1589),
    [sym_set_outdoor_temperature] = ACTIONS(1589),
    [sym_set_virtual_level_count] = ACTIONS(1589),
    [sym_set_virtual_channel_count] = ACTIONS(1589),
    [sym_set_virtual_port_count] = ACTIONS(1589),
    [sym__wc_encode_int] = ACTIONS(1589),
    [sym__wc_decode_int] = ACTIONS(1589),
    [sym__wc_int] = ACTIONS(1589),
    [sym__wc_to_ch_int] = ACTIONS(1589),
    [sym__ch_to_wc_int] = ACTIONS(1589),
    [sym__wc_find_string_int] = ACTIONS(1589),
    [sym__wc_left_string_int] = ACTIONS(1589),
    [sym__wc_lower_string_int] = ACTIONS(1589),
    [sym__wc_mid_string_int] = ACTIONS(1589),
    [sym__wc_remove_string_int] = ACTIONS(1589),
    [sym__wc_right_string_int] = ACTIONS(1589),
    [sym__wc_upper_string_int] = ACTIONS(1589),
    [sym__wc_compare_string_int] = ACTIONS(1589),
    [sym__wc_get_buffer_char_int] = ACTIONS(1589),
    [sym__wc_get_buffer_string_int] = ACTIONS(1589),
    [sym__wc_concat_string_int] = ACTIONS(1589),
    [sym__wc_file_open_int] = ACTIONS(1589),
    [sym__wc_file_close_int] = ACTIONS(1589),
    [sym__wc_file_read_int] = ACTIONS(1589),
    [sym__wc_file_read_line_int] = ACTIONS(1589),
    [sym__wc_file_write_int] = ACTIONS(1589),
    [sym__wc_file_write_line_int] = ACTIONS(1589),
    [aux_sym_preproc_include_token1] = ACTIONS(1589),
    [aux_sym_preproc_define_token1] = ACTIONS(1591),
    [aux_sym_preproc_warn_token1] = ACTIONS(1591),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1591),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1591),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1591),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1591),
    [aux_sym_preproc_else_token1] = ACTIONS(1591),
    [anon_sym_LBRACK] = ACTIONS(1591),
    [anon_sym_LBRACE] = ACTIONS(1591),
    [anon_sym_COLON] = ACTIONS(1591),
    [anon_sym_LPAREN] = ACTIONS(1589),
    [anon_sym_SEMI] = ACTIONS(1591),
    [anon_sym_BANG] = ACTIONS(1589),
    [anon_sym_TILDE] = ACTIONS(1591),
    [anon_sym_DASH] = ACTIONS(1589),
    [anon_sym_PLUS] = ACTIONS(1589),
    [anon_sym_STAR] = ACTIONS(1591),
    [anon_sym_SLASH] = ACTIONS(1589),
    [anon_sym_PERCENT] = ACTIONS(1591),
    [anon_sym_PIPE_PIPE] = ACTIONS(1591),
    [anon_sym_AMP_AMP] = ACTIONS(1591),
    [anon_sym_PIPE] = ACTIONS(1589),
    [anon_sym_CARET] = ACTIONS(1591),
    [anon_sym_AMP] = ACTIONS(1589),
    [anon_sym_EQ_EQ] = ACTIONS(1591),
    [anon_sym_BANG_EQ] = ACTIONS(1591),
    [anon_sym_LT_GT] = ACTIONS(1591),
    [anon_sym_GT] = ACTIONS(1589),
    [anon_sym_GT_EQ] = ACTIONS(1591),
    [anon_sym_LT_EQ] = ACTIONS(1591),
    [anon_sym_LT] = ACTIONS(1589),
    [anon_sym_LT_LT] = ACTIONS(1591),
    [anon_sym_GT_GT] = ACTIONS(1591),
    [anon_sym_DASH_DASH] = ACTIONS(1591),
    [anon_sym_PLUS_PLUS] = ACTIONS(1591),
    [anon_sym_DOT] = ACTIONS(1589),
    [anon_sym_DQUOTE] = ACTIONS(1591),
    [anon_sym_SQUOTE] = ACTIONS(1591),
    [sym_number_literal] = ACTIONS(1591),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1591),
  },
  [STATE(130)] = {
    [sym_identifier] = ACTIONS(1492),
    [sym_program_name_keyword] = ACTIONS(1492),
    [sym_module_name_keyword] = ACTIONS(1492),
    [sym_define_device_keyword] = ACTIONS(1492),
    [sym_define_combine_keyword] = ACTIONS(1492),
    [sym_define_constant_keyword] = ACTIONS(1492),
    [sym_define_type_keyword] = ACTIONS(1492),
    [sym_define_variable_keyword] = ACTIONS(1492),
    [sym_define_system_variable_keyword] = ACTIONS(1492),
    [sym_define_connect_level_keyword] = ACTIONS(1492),
    [sym_define_latching_keyword] = ACTIONS(1492),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1492),
    [sym_define_toggling_keyword] = ACTIONS(1492),
    [sym_define_call_keyword] = ACTIONS(1492),
    [sym_define_function_keyword] = ACTIONS(1492),
    [sym_define_library_function_keyword] = ACTIONS(1492),
    [sym_define_start_keyword] = ACTIONS(1492),
    [sym_define_event_keyword] = ACTIONS(1492),
    [sym_define_program_keyword] = ACTIONS(1492),
    [sym_define_module_keyword] = ACTIONS(1492),
    [sym_char_keyword] = ACTIONS(1492),
    [sym_widechar_keyword] = ACTIONS(1492),
    [sym_integer_keyword] = ACTIONS(1492),
    [sym_sinteger_keyword] = ACTIONS(1492),
    [sym_long_keyword] = ACTIONS(1492),
    [sym_slong_keyword] = ACTIONS(1492),
    [sym_float_keyword] = ACTIONS(1492),
    [sym_double_keyword] = ACTIONS(1492),
    [sym_variant_keyword] = ACTIONS(1492),
    [sym_variantarray_keyword] = ACTIONS(1492),
    [sym_local_var_keyword] = ACTIONS(1492),
    [sym_stack_var_keyword] = ACTIONS(1492),
    [sym_constant_keyword] = ACTIONS(1492),
    [sym_volatile_keyword] = ACTIONS(1492),
    [sym_non_volatile_keyword] = ACTIONS(1492),
    [sym_persistent_keyword] = ACTIONS(1492),
    [sym_dev_keyword] = ACTIONS(1492),
    [sym_devlev_keyword] = ACTIONS(1492),
    [sym_devchan_keyword] = ACTIONS(1492),
    [sym_if_keyword] = ACTIONS(1492),
    [sym_else_keyword] = ACTIONS(1492),
    [sym_switch_keyword] = ACTIONS(1492),
    [sym_case_keyword] = ACTIONS(1492),
    [sym_default_keyword] = ACTIONS(1492),
    [sym_while_keyword] = ACTIONS(1492),
    [sym_for_keyword] = ACTIONS(1492),
    [sym_break_keyword] = ACTIONS(1492),
    [sym_continue_keyword] = ACTIONS(1492),
    [sym_return_keyword] = ACTIONS(1492),
    [sym_select_keyword] = ACTIONS(1492),
    [sym_active_keyword] = ACTIONS(1492),
    [sym_struct_keyword] = ACTIONS(1492),
    [sym_structure_keyword] = ACTIONS(1492),
    [sym_button_event_keyword] = ACTIONS(1492),
    [sym_channel_event_keyword] = ACTIONS(1492),
    [sym_level_event_keyword] = ACTIONS(1492),
    [sym_data_event_keyword] = ACTIONS(1492),
    [sym_timeline_event_keyword] = ACTIONS(1492),
    [sym_custom_event_keyword] = ACTIONS(1492),
    [sym_push_keyword] = ACTIONS(1492),
    [sym_release_keyword] = ACTIONS(1492),
    [aux_sym_on_keyword_token1] = ACTIONS(1492),
    [aux_sym_off_keyword_token1] = ACTIONS(1492),
    [sym_send_level_keyword] = ACTIONS(1492),
    [sym_send_string_keyword] = ACTIONS(1492),
    [sym_send_command_keyword] = ACTIONS(1492),
    [sym_clear_buffer_keyword] = ACTIONS(1492),
    [sym_create_buffer_keyword] = ACTIONS(1492),
    [sym_create_multi_buffer_keyword] = ACTIONS(1492),
    [sym_call_keyword] = ACTIONS(1492),
    [sym_system_call_keyword] = ACTIONS(1492),
    [sym_devchan_to_keyword] = ACTIONS(1492),
    [sym_devchan_min_to_keyword] = ACTIONS(1492),
    [sym_devchan_total_off_keyword] = ACTIONS(1492),
    [sym_devchan_pulse_keyword] = ACTIONS(1492),
    [sym_wait_keyword] = ACTIONS(1492),
    [sym_cancel_wait_keyword] = ACTIONS(1492),
    [sym_wait_until_keyword] = ACTIONS(1492),
    [sym_cancel_wait_until_keyword] = ACTIONS(1492),
    [sym_cancel_all_wait_keyword] = ACTIONS(1492),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1492),
    [sym_and] = ACTIONS(1492),
    [sym_or] = ACTIONS(1492),
    [sym_not] = ACTIONS(1492),
    [sym_xor] = ACTIONS(1492),
    [sym_band] = ACTIONS(1492),
    [sym_bor] = ACTIONS(1492),
    [sym_bxor] = ACTIONS(1492),
    [sym_bnot] = ACTIONS(1492),
    [sym_lshift] = ACTIONS(1492),
    [sym_rshift] = ACTIONS(1492),
    [sym___file__] = ACTIONS(1492),
    [sym___date__] = ACTIONS(1492),
    [sym___ldate__] = ACTIONS(1492),
    [sym___time__] = ACTIONS(1492),
    [sym___line__] = ACTIONS(1492),
    [sym___name__] = ACTIONS(1492),
    [sym___netlinx__] = ACTIONS(1492),
    [sym_netlinx_axi_version] = ACTIONS(1492),
    [sym_url_flg_tcp] = ACTIONS(1492),
    [sym_url_flg_accinfopresent] = ACTIONS(1492),
    [sym_url_flg_temp] = ACTIONS(1492),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1492),
    [sym_url_flg_stat_mask] = ACTIONS(1492),
    [sym_url_flg_stat_lookup] = ACTIONS(1492),
    [sym_url_flg_stat_connecting] = ACTIONS(1492),
    [sym_url_flg_stat_waiting] = ACTIONS(1492),
    [sym_url_flg_stat_connected] = ACTIONS(1492),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1492),
    [sym_first_virtual_device] = ACTIONS(1492),
    [sym_first_local_port] = ACTIONS(1492),
    [sym_dynamic_virtual_device] = ACTIONS(1492),
    [sym_timeline_once] = ACTIONS(1492),
    [sym_timeline_repeat] = ACTIONS(1492),
    [sym_timeline_absolute] = ACTIONS(1492),
    [sym_timeline_relative] = ACTIONS(1492),
    [sym_do_push_timed_infinite] = ACTIONS(1492),
    [sym_source_type_no_address] = ACTIONS(1492),
    [sym_source_type_neuron_id] = ACTIONS(1492),
    [sym_source_type_ip_address] = ACTIONS(1492),
    [sym_source_type_axlink] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1492),
    [sym_source_type_ip_socket_address] = ACTIONS(1492),
    [sym_source_type_rs232] = ACTIONS(1492),
    [sym_source_type_internal] = ACTIONS(1492),
    [sym_source_type_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1492),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1492),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1492),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1492),
    [sym_true] = ACTIONS(1492),
    [sym_false] = ACTIONS(1492),
    [sym_file_read_only] = ACTIONS(1492),
    [sym_file_rw_new] = ACTIONS(1492),
    [sym_file_rw_append] = ACTIONS(1492),
    [sym_ip_tcp] = ACTIONS(1492),
    [sym_ip_udp] = ACTIONS(1492),
    [sym_ip_udp_2way] = ACTIONS(1492),
    [sym_xml_encode_types] = ACTIONS(1492),
    [sym_xml_encode_char_as_list] = ACTIONS(1492),
    [sym_xml_encode_le] = ACTIONS(1492),
    [sym_xml_decode_types] = ACTIONS(1492),
    [sym_xml_decode_no_preserve] = ACTIONS(1492),
    [sym_duet_dev_type_display_device] = ACTIONS(1492),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_io_device] = ACTIONS(1492),
    [sym_duet_dev_type_relay_device] = ACTIONS(1492),
    [sym_duet_dev_type_ups] = ACTIONS(1492),
    [sym_duet_dev_type_amplifier] = ACTIONS(1492),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1492),
    [sym_duet_dev_type_camera] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1492),
    [sym_duet_dev_type_dss] = ACTIONS(1492),
    [sym_duet_dev_type_dvr] = ACTIONS(1492),
    [sym_duet_dev_type_disc_device] = ACTIONS(1492),
    [sym_duet_dev_type_document_camera] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1492),
    [sym_duet_dev_type_hvac] = ACTIONS(1492),
    [sym_duet_dev_type_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_light] = ACTIONS(1492),
    [sym_duet_dev_type_monitor] = ACTIONS(1492),
    [sym_duet_dev_type_motor] = ACTIONS(1492),
    [sym_duet_dev_type_multi_window] = ACTIONS(1492),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1492),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1492),
    [sym_duet_dev_type_receiver] = ACTIONS(1492),
    [sym_duet_dev_type_security_system] = ACTIONS(1492),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1492),
    [sym_duet_dev_type_settop_box] = ACTIONS(1492),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1492),
    [sym_duet_dev_type_switcher] = ACTIONS(1492),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_tv] = ACTIONS(1492),
    [sym_duet_dev_type_utility] = ACTIONS(1492),
    [sym_duet_dev_type_vcr] = ACTIONS(1492),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_video_processor] = ACTIONS(1492),
    [sym_duet_dev_type_video_projector] = ACTIONS(1492),
    [sym_duet_dev_type_video_wall] = ACTIONS(1492),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1492),
    [sym_duet_dev_type_weather] = ACTIONS(1492),
    [sym_duet_dev_not_polled] = ACTIONS(1492),
    [sym_duet_dev_polled] = ACTIONS(1492),
    [sym_clkmgr_mode_network] = ACTIONS(1492),
    [sym_clkmgr_mode_standalone] = ACTIONS(1492),
    [sym_normal_standby] = ACTIONS(1492),
    [sym_normal_wake] = ACTIONS(1492),
    [sym_ip_multicast_ttl_option] = ACTIONS(1492),
    [sym_ip_ttl_subnet] = ACTIONS(1492),
    [sym_ip_ttl_site] = ACTIONS(1492),
    [sym_ip_ttl_region] = ACTIONS(1492),
    [sym_ip_ttl_continent] = ACTIONS(1492),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1492),
    [sym_ip_nodelay_on] = ACTIONS(1492),
    [sym_ip_nodelay_off] = ACTIONS(1492),
    [sym_amx_error] = ACTIONS(1492),
    [sym_amx_warning] = ACTIONS(1492),
    [sym_amx_info] = ACTIONS(1492),
    [sym_amx_debug] = ACTIONS(1492),
    [sym_netlinx_login_success] = ACTIONS(1492),
    [sym_netlinx_login_fail] = ACTIONS(1492),
    [sym_netlinx_logout] = ACTIONS(1492),
    [sym_null_str] = ACTIONS(1492),
    [sym_smtp_address] = ACTIONS(1492),
    [sym_smtp_port_number] = ACTIONS(1492),
    [sym_smtp_username] = ACTIONS(1492),
    [sym_smtp_password] = ACTIONS(1492),
    [sym_smtp_require_tls] = ACTIONS(1492),
    [sym_smtp_from] = ACTIONS(1492),
    [sym_smtp_tls_true] = ACTIONS(1492),
    [sym_smtp_tls_false] = ACTIONS(1492),
    [sym_listview_on_row_select_event] = ACTIONS(1492),
    [sym_data_structure_unknown] = ACTIONS(1492),
    [sym_data_structure_datafeed] = ACTIONS(1492),
    [sym_data_structure_datarecordset] = ACTIONS(1492),
    [sym_data_structure_datarecord] = ACTIONS(1492),
    [sym_data_structure_datafield] = ACTIONS(1492),
    [sym_data_max_content_field_count] = ACTIONS(1492),
    [sym_data_max_metadata_field_count] = ACTIONS(1492),
    [sym_data_max_id_length] = ACTIONS(1492),
    [sym_data_max_name_length] = ACTIONS(1492),
    [sym_data_max_description_length] = ACTIONS(1492),
    [sym_data_max_type_length] = ACTIONS(1492),
    [sym_data_max_format_length] = ACTIONS(1492),
    [sym_data_max_source_length] = ACTIONS(1492),
    [sym_data_max_value_length] = ACTIONS(1492),
    [sym_data_max_label_length] = ACTIONS(1492),
    [sym_data_max_url_length] = ACTIONS(1492),
    [sym_max_last_login_info_length] = ACTIONS(1492),
    [sym_data_type_unknown] = ACTIONS(1492),
    [sym_data_type_string] = ACTIONS(1492),
    [sym_data_type_datetime] = ACTIONS(1492),
    [sym_data_type_date] = ACTIONS(1492),
    [sym_data_type_time] = ACTIONS(1492),
    [sym_data_type_image] = ACTIONS(1492),
    [sym_data_format_url] = ACTIONS(1492),
    [sym_data_format_phone] = ACTIONS(1492),
    [sym_data_format_email] = ACTIONS(1492),
    [sym_data_format_iso8601] = ACTIONS(1492),
    [sym_auditlog_priv] = ACTIONS(1492),
    [sym_configuration_priv] = ACTIONS(1492),
    [sym_deviceconfig_priv] = ACTIONS(1492),
    [sym_ftp_priv] = ACTIONS(1492),
    [sym_http_priv] = ACTIONS(1492),
    [sym_networkconfig_priv] = ACTIONS(1492),
    [sym_programport_priv] = ACTIONS(1492),
    [sym_remoteui_priv] = ACTIONS(1492),
    [sym_securitycontrol_priv] = ACTIONS(1492),
    [sym_softwaremanagement_priv] = ACTIONS(1492),
    [sym_terminal_priv] = ACTIONS(1492),
    [sym_tpadmin_priv] = ACTIONS(1492),
    [sym_user1_priv] = ACTIONS(1492),
    [sym_user2_priv] = ACTIONS(1492),
    [sym_user3_priv] = ACTIONS(1492),
    [sym_user4_priv] = ACTIONS(1492),
    [sym_usermanagement_priv] = ACTIONS(1492),
    [sym_valid_account] = ACTIONS(1492),
    [sym_err_name_invalid] = ACTIONS(1492),
    [sym_err_password_invalid] = ACTIONS(1492),
    [sym_err_invalid_account] = ACTIONS(1492),
    [sym_err_no_authorization] = ACTIONS(1492),
    [sym_err_invalid_parameter] = ACTIONS(1492),
    [sym_err_user_locked_out] = ACTIONS(1492),
    [sym_err_user_account_expired] = ACTIONS(1492),
    [sym_err_general_error] = ACTIONS(1492),
    [sym_err_authentication_error] = ACTIONS(1492),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1492),
    [sym_tls_validate_certificate] = ACTIONS(1492),
    [sym_tdata] = ACTIONS(1492),
    [sym_tchannel] = ACTIONS(1492),
    [sym_tlevel] = ACTIONS(1492),
    [sym_tbutton] = ACTIONS(1492),
    [sym_ttimeline] = ACTIONS(1492),
    [sym_tcustom] = ACTIONS(1492),
    [sym_url_struct] = ACTIONS(1492),
    [sym_dns_struct] = ACTIONS(1492),
    [sym_ip_address_struct] = ACTIONS(1492),
    [sym_dev_info_struct] = ACTIONS(1492),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1492),
    [sym_data_feed] = ACTIONS(1492),
    [sym_data_field] = ACTIONS(1492),
    [sym_data_record] = ACTIONS(1492),
    [sym_wc_data_feed] = ACTIONS(1492),
    [sym_wc_data_field] = ACTIONS(1492),
    [sym_wc_data_record] = ACTIONS(1492),
    [sym_last_login_info] = ACTIONS(1492),
    [sym____reserved___] = ACTIONS(1492),
    [sym_date] = ACTIONS(1492),
    [sym_day] = ACTIONS(1492),
    [sym_time] = ACTIONS(1492),
    [sym_ldate] = ACTIONS(1492),
    [sym_push_device] = ACTIONS(1492),
    [sym_push_channel] = ACTIONS(1492),
    [sym_push_devchan] = ACTIONS(1492),
    [sym_release_device] = ACTIONS(1492),
    [sym_release_channel] = ACTIONS(1492),
    [sym_release_devchan] = ACTIONS(1492),
    [sym_master_slot] = ACTIONS(1492),
    [sym_get_pulse_time] = ACTIONS(1492),
    [sym_get_timer] = ACTIONS(1492),
    [sym_master_sn] = ACTIONS(1492),
    [sym_system_number] = ACTIONS(1492),
    [sym_button] = ACTIONS(1492),
    [sym_dv_channel] = ACTIONS(1492),
    [sym_data] = ACTIONS(1492),
    [sym_level] = ACTIONS(1492),
    [sym_timeline] = ACTIONS(1492),
    [sym_channel] = ACTIONS(1492),
    [sym_custom] = ACTIONS(1492),
    [sym_atoi] = ACTIONS(1492),
    [sym_atol] = ACTIONS(1492),
    [sym_atof] = ACTIONS(1492),
    [sym_itoa] = ACTIONS(1492),
    [sym_itohex] = ACTIONS(1492),
    [sym_hextoi] = ACTIONS(1492),
    [sym_ftoa] = ACTIONS(1492),
    [sym_find_string] = ACTIONS(1492),
    [sym_left_string] = ACTIONS(1492),
    [sym_right_string] = ACTIONS(1492),
    [sym_mid_string] = ACTIONS(1492),
    [sym_remove_string] = ACTIONS(1492),
    [sym_get_buffer_string] = ACTIONS(1492),
    [sym_get_buffer_char] = ACTIONS(1492),
    [sym_get_multi_buffer_string] = ACTIONS(1492),
    [sym_length_string] = ACTIONS(1492),
    [sym_max_length_string] = ACTIONS(1492),
    [sym_set_length_string] = ACTIONS(1492),
    [sym_compare_string] = ACTIONS(1492),
    [sym_upper_string] = ACTIONS(1492),
    [sym_lower_string] = ACTIONS(1492),
    [sym_get_last] = ACTIONS(1492),
    [sym_date_to_day] = ACTIONS(1492),
    [sym_date_to_month] = ACTIONS(1492),
    [sym_date_to_year] = ACTIONS(1492),
    [sym_time_to_hour] = ACTIONS(1492),
    [sym_time_to_minute] = ACTIONS(1492),
    [sym_time_to_second] = ACTIONS(1492),
    [sym_day_of_week] = ACTIONS(1492),
    [sym_set_pulse_time] = ACTIONS(1492),
    [sym_set_timer] = ACTIONS(1492),
    [sym_file_open] = ACTIONS(1492),
    [sym_file_close] = ACTIONS(1492),
    [sym_file_read] = ACTIONS(1492),
    [sym_file_read_line] = ACTIONS(1492),
    [sym_file_write] = ACTIONS(1492),
    [sym_file_write_line] = ACTIONS(1492),
    [sym_file_copy] = ACTIONS(1492),
    [sym_file_delete] = ACTIONS(1492),
    [sym_file_rename] = ACTIONS(1492),
    [sym_file_dir] = ACTIONS(1492),
    [sym_file_getdir] = ACTIONS(1492),
    [sym_file_setdir] = ACTIONS(1492),
    [sym_file_create_dir] = ACTIONS(1492),
    [sym_file_remove_dir] = ACTIONS(1492),
    [sym_file_seek] = ACTIONS(1492),
    [sym_length_array] = ACTIONS(1492),
    [sym_max_length_array] = ACTIONS(1492),
    [sym_set_length_array] = ACTIONS(1492),
    [sym_ip_client_open] = ACTIONS(1492),
    [sym_ip_client_close] = ACTIONS(1492),
    [sym_ip_server_open] = ACTIONS(1492),
    [sym_ip_server_close] = ACTIONS(1492),
    [sym_ip_bound_client_open] = ACTIONS(1492),
    [sym_ip_mc_server_open] = ACTIONS(1492),
    [sym_ip_set_option] = ACTIONS(1492),
    [sym_ssh_client_open] = ACTIONS(1492),
    [sym_ssh_client_close] = ACTIONS(1492),
    [sym_tls_client_open] = ACTIONS(1492),
    [sym_tls_client_close] = ACTIONS(1492),
    [sym_do_push] = ACTIONS(1492),
    [sym_do_push_timed] = ACTIONS(1492),
    [sym_do_release] = ACTIONS(1492),
    [sym_device_id] = ACTIONS(1492),
    [sym_device_info] = ACTIONS(1492),
    [sym_device_id_string] = ACTIONS(1492),
    [sym_device_standby] = ACTIONS(1492),
    [sym_device_wake] = ACTIONS(1492),
    [sym_timeline_create] = ACTIONS(1492),
    [sym_timeline_kill] = ACTIONS(1492),
    [sym_timeline_reload] = ACTIONS(1492),
    [sym_timeline_pause] = ACTIONS(1492),
    [sym_timeline_restart] = ACTIONS(1492),
    [sym_timeline_set] = ACTIONS(1492),
    [sym_timeline_get] = ACTIONS(1492),
    [sym_timeline_active] = ACTIONS(1492),
    [sym_timeline_dynamic_id] = ACTIONS(1492),
    [sym_variable_to_string] = ACTIONS(1492),
    [sym_string_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_string] = ACTIONS(1492),
    [sym_type_cast] = ACTIONS(1492),
    [sym_variable_to_xml] = ACTIONS(1492),
    [sym_xml_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_xml] = ACTIONS(1492),
    [sym_redirect_string] = ACTIONS(1492),
    [sym_random_number] = ACTIONS(1492),
    [sym_get_unique_id] = ACTIONS(1492),
    [sym_get_system_number] = ACTIONS(1492),
    [sym_set_system_number] = ACTIONS(1492),
    [sym_get_serial_number] = ACTIONS(1492),
    [sym_get_master_build] = ACTIONS(1492),
    [sym_set_validation_code] = ACTIONS(1492),
    [sym_reboot] = ACTIONS(1492),
    [sym_load_duet_module] = ACTIONS(1492),
    [sym_unload_duet_module] = ACTIONS(1492),
    [sym_duet_mem_size_set] = ACTIONS(1492),
    [sym_duet_mem_size_get] = ACTIONS(1492),
    [sym_dynamic_application_device] = ACTIONS(1492),
    [sym_dynamic_polled_port] = ACTIONS(1492),
    [sym_static_port_binding] = ACTIONS(1492),
    [sym_static_ip_binding] = ACTIONS(1492),
    [sym_get_dns_list] = ACTIONS(1492),
    [sym_set_dns_list] = ACTIONS(1492),
    [sym_get_ip_address] = ACTIONS(1492),
    [sym_set_ip_address] = ACTIONS(1492),
    [sym_get_url_list] = ACTIONS(1492),
    [sym_add_url_entry] = ACTIONS(1492),
    [sym_delete_url_entry] = ACTIONS(1492),
    [sym_abs_value] = ACTIONS(1492),
    [sym_max_value] = ACTIONS(1492),
    [sym_min_value] = ACTIONS(1492),
    [sym_exp_value] = ACTIONS(1492),
    [sym_log_value] = ACTIONS(1492),
    [sym_log10_value] = ACTIONS(1492),
    [sym_power_value] = ACTIONS(1492),
    [sym_sqrt_value] = ACTIONS(1492),
    [sym_raw_be] = ACTIONS(1492),
    [sym_raw_le] = ACTIONS(1492),
    [sym_internal_queue_size_set] = ACTIONS(1492),
    [sym_internal_queue_size_get] = ACTIONS(1492),
    [sym_internal_threshold_set] = ACTIONS(1492),
    [sym_internal_threshold_get] = ACTIONS(1492),
    [sym_rebuilt_event] = ACTIONS(1492),
    [sym_do_custom_event] = ACTIONS(1492),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1492),
    [sym_clkmgr_set_clk_source] = ACTIONS(1492),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1492),
    [sym_clkmgr_get_timezone] = ACTIONS(1492),
    [sym_clkmgr_set_timezone] = ACTIONS(1492),
    [sym_clkmgr_get_resync_period] = ACTIONS(1492),
    [sym_clkmgr_set_resync_period] = ACTIONS(1492),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_timeservers] = ACTIONS(1492),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_set_log_level] = ACTIONS(1492),
    [sym_get_log_level] = ACTIONS(1492),
    [sym_amx_log] = ACTIONS(1492),
    [sym_smtp_server_config_set] = ACTIONS(1492),
    [sym_smtp_server_config_get] = ACTIONS(1492),
    [sym_smtp_send] = ACTIONS(1492),
    [sym_validate_netlinx_account] = ACTIONS(1492),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1492),
    [sym_authenticate_certificate] = ACTIONS(1492),
    [sym_audit_netlinx_session_event] = ACTIONS(1492),
    [sym_audit_netlinx_generic_event] = ACTIONS(1492),
    [sym_get_available_flash_disk_space] = ACTIONS(1492),
    [sym_get_max_flash_disk_space] = ACTIONS(1492),
    [sym_data_create_feed] = ACTIONS(1492),
    [sym_data_delete_feed] = ACTIONS(1492),
    [sym_data_publish_feed] = ACTIONS(1492),
    [sym_data_get_published_feed] = ACTIONS(1492),
    [sym_data_add_record] = ACTIONS(1492),
    [sym_data_get_event_record] = ACTIONS(1492),
    [sym__wc_data_create_feed] = ACTIONS(1492),
    [sym__wc_data_add_record] = ACTIONS(1492),
    [sym__wc_data_get_event_record] = ACTIONS(1492),
    [sym_format] = ACTIONS(1492),
    [sym_astro_clock] = ACTIONS(1492),
    [sym_set_outdoor_temperature] = ACTIONS(1492),
    [sym_set_virtual_level_count] = ACTIONS(1492),
    [sym_set_virtual_channel_count] = ACTIONS(1492),
    [sym_set_virtual_port_count] = ACTIONS(1492),
    [sym__wc_encode_int] = ACTIONS(1492),
    [sym__wc_decode_int] = ACTIONS(1492),
    [sym__wc_int] = ACTIONS(1492),
    [sym__wc_to_ch_int] = ACTIONS(1492),
    [sym__ch_to_wc_int] = ACTIONS(1492),
    [sym__wc_find_string_int] = ACTIONS(1492),
    [sym__wc_left_string_int] = ACTIONS(1492),
    [sym__wc_lower_string_int] = ACTIONS(1492),
    [sym__wc_mid_string_int] = ACTIONS(1492),
    [sym__wc_remove_string_int] = ACTIONS(1492),
    [sym__wc_right_string_int] = ACTIONS(1492),
    [sym__wc_upper_string_int] = ACTIONS(1492),
    [sym__wc_compare_string_int] = ACTIONS(1492),
    [sym__wc_get_buffer_char_int] = ACTIONS(1492),
    [sym__wc_get_buffer_string_int] = ACTIONS(1492),
    [sym__wc_concat_string_int] = ACTIONS(1492),
    [sym__wc_file_open_int] = ACTIONS(1492),
    [sym__wc_file_close_int] = ACTIONS(1492),
    [sym__wc_file_read_int] = ACTIONS(1492),
    [sym__wc_file_read_line_int] = ACTIONS(1492),
    [sym__wc_file_write_int] = ACTIONS(1492),
    [sym__wc_file_write_line_int] = ACTIONS(1492),
    [aux_sym_preproc_include_token1] = ACTIONS(1492),
    [aux_sym_preproc_define_token1] = ACTIONS(1494),
    [aux_sym_preproc_warn_token1] = ACTIONS(1494),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1494),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1494),
    [aux_sym_preproc_else_token1] = ACTIONS(1494),
    [anon_sym_LBRACK] = ACTIONS(1494),
    [anon_sym_LBRACE] = ACTIONS(1494),
    [anon_sym_COLON] = ACTIONS(1494),
    [anon_sym_LPAREN] = ACTIONS(1492),
    [anon_sym_SEMI] = ACTIONS(1494),
    [anon_sym_BANG] = ACTIONS(1492),
    [anon_sym_TILDE] = ACTIONS(1494),
    [anon_sym_DASH] = ACTIONS(1492),
    [anon_sym_PLUS] = ACTIONS(1492),
    [anon_sym_STAR] = ACTIONS(1494),
    [anon_sym_SLASH] = ACTIONS(1492),
    [anon_sym_PERCENT] = ACTIONS(1494),
    [anon_sym_PIPE_PIPE] = ACTIONS(1494),
    [anon_sym_AMP_AMP] = ACTIONS(1494),
    [anon_sym_PIPE] = ACTIONS(1492),
    [anon_sym_CARET] = ACTIONS(1494),
    [anon_sym_AMP] = ACTIONS(1492),
    [anon_sym_EQ_EQ] = ACTIONS(1494),
    [anon_sym_BANG_EQ] = ACTIONS(1494),
    [anon_sym_LT_GT] = ACTIONS(1494),
    [anon_sym_GT] = ACTIONS(1492),
    [anon_sym_GT_EQ] = ACTIONS(1494),
    [anon_sym_LT_EQ] = ACTIONS(1494),
    [anon_sym_LT] = ACTIONS(1492),
    [anon_sym_LT_LT] = ACTIONS(1494),
    [anon_sym_GT_GT] = ACTIONS(1494),
    [anon_sym_DASH_DASH] = ACTIONS(1494),
    [anon_sym_PLUS_PLUS] = ACTIONS(1494),
    [anon_sym_DOT] = ACTIONS(1492),
    [anon_sym_DQUOTE] = ACTIONS(1494),
    [anon_sym_SQUOTE] = ACTIONS(1494),
    [sym_number_literal] = ACTIONS(1494),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1494),
  },
  [STATE(131)] = {
    [ts_builtin_sym_end] = ACTIONS(1513),
    [sym_identifier] = ACTIONS(1511),
    [sym_program_name_keyword] = ACTIONS(1511),
    [sym_module_name_keyword] = ACTIONS(1511),
    [sym_define_device_keyword] = ACTIONS(1511),
    [sym_define_combine_keyword] = ACTIONS(1511),
    [sym_define_constant_keyword] = ACTIONS(1511),
    [sym_define_type_keyword] = ACTIONS(1511),
    [sym_define_variable_keyword] = ACTIONS(1511),
    [sym_define_system_variable_keyword] = ACTIONS(1511),
    [sym_define_connect_level_keyword] = ACTIONS(1511),
    [sym_define_latching_keyword] = ACTIONS(1511),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1511),
    [sym_define_toggling_keyword] = ACTIONS(1511),
    [sym_define_call_keyword] = ACTIONS(1511),
    [sym_define_function_keyword] = ACTIONS(1511),
    [sym_define_library_function_keyword] = ACTIONS(1511),
    [sym_define_start_keyword] = ACTIONS(1511),
    [sym_define_event_keyword] = ACTIONS(1511),
    [sym_define_program_keyword] = ACTIONS(1511),
    [sym_define_module_keyword] = ACTIONS(1511),
    [sym_char_keyword] = ACTIONS(1511),
    [sym_widechar_keyword] = ACTIONS(1511),
    [sym_integer_keyword] = ACTIONS(1511),
    [sym_sinteger_keyword] = ACTIONS(1511),
    [sym_long_keyword] = ACTIONS(1511),
    [sym_slong_keyword] = ACTIONS(1511),
    [sym_float_keyword] = ACTIONS(1511),
    [sym_double_keyword] = ACTIONS(1511),
    [sym_variant_keyword] = ACTIONS(1511),
    [sym_variantarray_keyword] = ACTIONS(1511),
    [sym_local_var_keyword] = ACTIONS(1511),
    [sym_stack_var_keyword] = ACTIONS(1511),
    [sym_constant_keyword] = ACTIONS(1511),
    [sym_volatile_keyword] = ACTIONS(1511),
    [sym_non_volatile_keyword] = ACTIONS(1511),
    [sym_persistent_keyword] = ACTIONS(1511),
    [sym_dev_keyword] = ACTIONS(1511),
    [sym_devlev_keyword] = ACTIONS(1511),
    [sym_devchan_keyword] = ACTIONS(1511),
    [sym_if_keyword] = ACTIONS(1511),
    [sym_else_keyword] = ACTIONS(1511),
    [sym_switch_keyword] = ACTIONS(1511),
    [sym_case_keyword] = ACTIONS(1511),
    [sym_default_keyword] = ACTIONS(1511),
    [sym_while_keyword] = ACTIONS(1511),
    [sym_for_keyword] = ACTIONS(1511),
    [sym_break_keyword] = ACTIONS(1511),
    [sym_continue_keyword] = ACTIONS(1511),
    [sym_return_keyword] = ACTIONS(1511),
    [sym_select_keyword] = ACTIONS(1511),
    [sym_active_keyword] = ACTIONS(1511),
    [sym_struct_keyword] = ACTIONS(1511),
    [sym_structure_keyword] = ACTIONS(1511),
    [sym_button_event_keyword] = ACTIONS(1511),
    [sym_channel_event_keyword] = ACTIONS(1511),
    [sym_level_event_keyword] = ACTIONS(1511),
    [sym_data_event_keyword] = ACTIONS(1511),
    [sym_timeline_event_keyword] = ACTIONS(1511),
    [sym_custom_event_keyword] = ACTIONS(1511),
    [sym_push_keyword] = ACTIONS(1511),
    [sym_release_keyword] = ACTIONS(1511),
    [aux_sym_on_keyword_token1] = ACTIONS(1511),
    [aux_sym_off_keyword_token1] = ACTIONS(1511),
    [sym_send_level_keyword] = ACTIONS(1511),
    [sym_send_string_keyword] = ACTIONS(1511),
    [sym_send_command_keyword] = ACTIONS(1511),
    [sym_clear_buffer_keyword] = ACTIONS(1511),
    [sym_create_buffer_keyword] = ACTIONS(1511),
    [sym_create_multi_buffer_keyword] = ACTIONS(1511),
    [sym_call_keyword] = ACTIONS(1511),
    [sym_system_call_keyword] = ACTIONS(1511),
    [sym_devchan_to_keyword] = ACTIONS(1511),
    [sym_devchan_min_to_keyword] = ACTIONS(1511),
    [sym_devchan_total_off_keyword] = ACTIONS(1511),
    [sym_devchan_pulse_keyword] = ACTIONS(1511),
    [sym_wait_keyword] = ACTIONS(1511),
    [sym_cancel_wait_keyword] = ACTIONS(1511),
    [sym_wait_until_keyword] = ACTIONS(1511),
    [sym_cancel_wait_until_keyword] = ACTIONS(1511),
    [sym_cancel_all_wait_keyword] = ACTIONS(1511),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1511),
    [sym_and] = ACTIONS(1511),
    [sym_or] = ACTIONS(1511),
    [sym_not] = ACTIONS(1511),
    [sym_xor] = ACTIONS(1511),
    [sym_band] = ACTIONS(1511),
    [sym_bor] = ACTIONS(1511),
    [sym_bxor] = ACTIONS(1511),
    [sym_bnot] = ACTIONS(1511),
    [sym_lshift] = ACTIONS(1511),
    [sym_rshift] = ACTIONS(1511),
    [sym___file__] = ACTIONS(1511),
    [sym___date__] = ACTIONS(1511),
    [sym___ldate__] = ACTIONS(1511),
    [sym___time__] = ACTIONS(1511),
    [sym___line__] = ACTIONS(1511),
    [sym___name__] = ACTIONS(1511),
    [sym___netlinx__] = ACTIONS(1511),
    [sym_netlinx_axi_version] = ACTIONS(1511),
    [sym_url_flg_tcp] = ACTIONS(1511),
    [sym_url_flg_accinfopresent] = ACTIONS(1511),
    [sym_url_flg_temp] = ACTIONS(1511),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1511),
    [sym_url_flg_stat_mask] = ACTIONS(1511),
    [sym_url_flg_stat_lookup] = ACTIONS(1511),
    [sym_url_flg_stat_connecting] = ACTIONS(1511),
    [sym_url_flg_stat_waiting] = ACTIONS(1511),
    [sym_url_flg_stat_connected] = ACTIONS(1511),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1511),
    [sym_first_virtual_device] = ACTIONS(1511),
    [sym_first_local_port] = ACTIONS(1511),
    [sym_dynamic_virtual_device] = ACTIONS(1511),
    [sym_timeline_once] = ACTIONS(1511),
    [sym_timeline_repeat] = ACTIONS(1511),
    [sym_timeline_absolute] = ACTIONS(1511),
    [sym_timeline_relative] = ACTIONS(1511),
    [sym_do_push_timed_infinite] = ACTIONS(1511),
    [sym_source_type_no_address] = ACTIONS(1511),
    [sym_source_type_neuron_id] = ACTIONS(1511),
    [sym_source_type_ip_address] = ACTIONS(1511),
    [sym_source_type_axlink] = ACTIONS(1511),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1511),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1511),
    [sym_source_type_ip_socket_address] = ACTIONS(1511),
    [sym_source_type_rs232] = ACTIONS(1511),
    [sym_source_type_internal] = ACTIONS(1511),
    [sym_source_type_mac_address] = ACTIONS(1511),
    [sym_source_type_ipv4_port] = ACTIONS(1511),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1511),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1511),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1511),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1511),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1511),
    [sym_true] = ACTIONS(1511),
    [sym_false] = ACTIONS(1511),
    [sym_file_read_only] = ACTIONS(1511),
    [sym_file_rw_new] = ACTIONS(1511),
    [sym_file_rw_append] = ACTIONS(1511),
    [sym_ip_tcp] = ACTIONS(1511),
    [sym_ip_udp] = ACTIONS(1511),
    [sym_ip_udp_2way] = ACTIONS(1511),
    [sym_xml_encode_types] = ACTIONS(1511),
    [sym_xml_encode_char_as_list] = ACTIONS(1511),
    [sym_xml_encode_le] = ACTIONS(1511),
    [sym_xml_decode_types] = ACTIONS(1511),
    [sym_xml_decode_no_preserve] = ACTIONS(1511),
    [sym_duet_dev_type_display_device] = ACTIONS(1511),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1511),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1511),
    [sym_duet_dev_type_io_device] = ACTIONS(1511),
    [sym_duet_dev_type_relay_device] = ACTIONS(1511),
    [sym_duet_dev_type_ups] = ACTIONS(1511),
    [sym_duet_dev_type_amplifier] = ACTIONS(1511),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1511),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1511),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1511),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1511),
    [sym_duet_dev_type_camera] = ACTIONS(1511),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1511),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1511),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1511),
    [sym_duet_dev_type_dss] = ACTIONS(1511),
    [sym_duet_dev_type_dvr] = ACTIONS(1511),
    [sym_duet_dev_type_disc_device] = ACTIONS(1511),
    [sym_duet_dev_type_document_camera] = ACTIONS(1511),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1511),
    [sym_duet_dev_type_hvac] = ACTIONS(1511),
    [sym_duet_dev_type_keypad] = ACTIONS(1511),
    [sym_duet_dev_type_light] = ACTIONS(1511),
    [sym_duet_dev_type_monitor] = ACTIONS(1511),
    [sym_duet_dev_type_motor] = ACTIONS(1511),
    [sym_duet_dev_type_multi_window] = ACTIONS(1511),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1511),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1511),
    [sym_duet_dev_type_receiver] = ACTIONS(1511),
    [sym_duet_dev_type_security_system] = ACTIONS(1511),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1511),
    [sym_duet_dev_type_settop_box] = ACTIONS(1511),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1511),
    [sym_duet_dev_type_switcher] = ACTIONS(1511),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1511),
    [sym_duet_dev_type_tv] = ACTIONS(1511),
    [sym_duet_dev_type_utility] = ACTIONS(1511),
    [sym_duet_dev_type_vcr] = ACTIONS(1511),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1511),
    [sym_duet_dev_type_video_processor] = ACTIONS(1511),
    [sym_duet_dev_type_video_projector] = ACTIONS(1511),
    [sym_duet_dev_type_video_wall] = ACTIONS(1511),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1511),
    [sym_duet_dev_type_weather] = ACTIONS(1511),
    [sym_duet_dev_not_polled] = ACTIONS(1511),
    [sym_duet_dev_polled] = ACTIONS(1511),
    [sym_clkmgr_mode_network] = ACTIONS(1511),
    [sym_clkmgr_mode_standalone] = ACTIONS(1511),
    [sym_normal_standby] = ACTIONS(1511),
    [sym_normal_wake] = ACTIONS(1511),
    [sym_ip_multicast_ttl_option] = ACTIONS(1511),
    [sym_ip_ttl_subnet] = ACTIONS(1511),
    [sym_ip_ttl_site] = ACTIONS(1511),
    [sym_ip_ttl_region] = ACTIONS(1511),
    [sym_ip_ttl_continent] = ACTIONS(1511),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1511),
    [sym_ip_nodelay_on] = ACTIONS(1511),
    [sym_ip_nodelay_off] = ACTIONS(1511),
    [sym_amx_error] = ACTIONS(1511),
    [sym_amx_warning] = ACTIONS(1511),
    [sym_amx_info] = ACTIONS(1511),
    [sym_amx_debug] = ACTIONS(1511),
    [sym_netlinx_login_success] = ACTIONS(1511),
    [sym_netlinx_login_fail] = ACTIONS(1511),
    [sym_netlinx_logout] = ACTIONS(1511),
    [sym_null_str] = ACTIONS(1511),
    [sym_smtp_address] = ACTIONS(1511),
    [sym_smtp_port_number] = ACTIONS(1511),
    [sym_smtp_username] = ACTIONS(1511),
    [sym_smtp_password] = ACTIONS(1511),
    [sym_smtp_require_tls] = ACTIONS(1511),
    [sym_smtp_from] = ACTIONS(1511),
    [sym_smtp_tls_true] = ACTIONS(1511),
    [sym_smtp_tls_false] = ACTIONS(1511),
    [sym_listview_on_row_select_event] = ACTIONS(1511),
    [sym_data_structure_unknown] = ACTIONS(1511),
    [sym_data_structure_datafeed] = ACTIONS(1511),
    [sym_data_structure_datarecordset] = ACTIONS(1511),
    [sym_data_structure_datarecord] = ACTIONS(1511),
    [sym_data_structure_datafield] = ACTIONS(1511),
    [sym_data_max_content_field_count] = ACTIONS(1511),
    [sym_data_max_metadata_field_count] = ACTIONS(1511),
    [sym_data_max_id_length] = ACTIONS(1511),
    [sym_data_max_name_length] = ACTIONS(1511),
    [sym_data_max_description_length] = ACTIONS(1511),
    [sym_data_max_type_length] = ACTIONS(1511),
    [sym_data_max_format_length] = ACTIONS(1511),
    [sym_data_max_source_length] = ACTIONS(1511),
    [sym_data_max_value_length] = ACTIONS(1511),
    [sym_data_max_label_length] = ACTIONS(1511),
    [sym_data_max_url_length] = ACTIONS(1511),
    [sym_max_last_login_info_length] = ACTIONS(1511),
    [sym_data_type_unknown] = ACTIONS(1511),
    [sym_data_type_string] = ACTIONS(1511),
    [sym_data_type_datetime] = ACTIONS(1511),
    [sym_data_type_date] = ACTIONS(1511),
    [sym_data_type_time] = ACTIONS(1511),
    [sym_data_type_image] = ACTIONS(1511),
    [sym_data_format_url] = ACTIONS(1511),
    [sym_data_format_phone] = ACTIONS(1511),
    [sym_data_format_email] = ACTIONS(1511),
    [sym_data_format_iso8601] = ACTIONS(1511),
    [sym_auditlog_priv] = ACTIONS(1511),
    [sym_configuration_priv] = ACTIONS(1511),
    [sym_deviceconfig_priv] = ACTIONS(1511),
    [sym_ftp_priv] = ACTIONS(1511),
    [sym_http_priv] = ACTIONS(1511),
    [sym_networkconfig_priv] = ACTIONS(1511),
    [sym_programport_priv] = ACTIONS(1511),
    [sym_remoteui_priv] = ACTIONS(1511),
    [sym_securitycontrol_priv] = ACTIONS(1511),
    [sym_softwaremanagement_priv] = ACTIONS(1511),
    [sym_terminal_priv] = ACTIONS(1511),
    [sym_tpadmin_priv] = ACTIONS(1511),
    [sym_user1_priv] = ACTIONS(1511),
    [sym_user2_priv] = ACTIONS(1511),
    [sym_user3_priv] = ACTIONS(1511),
    [sym_user4_priv] = ACTIONS(1511),
    [sym_usermanagement_priv] = ACTIONS(1511),
    [sym_valid_account] = ACTIONS(1511),
    [sym_err_name_invalid] = ACTIONS(1511),
    [sym_err_password_invalid] = ACTIONS(1511),
    [sym_err_invalid_account] = ACTIONS(1511),
    [sym_err_no_authorization] = ACTIONS(1511),
    [sym_err_invalid_parameter] = ACTIONS(1511),
    [sym_err_user_locked_out] = ACTIONS(1511),
    [sym_err_user_account_expired] = ACTIONS(1511),
    [sym_err_general_error] = ACTIONS(1511),
    [sym_err_authentication_error] = ACTIONS(1511),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1511),
    [sym_tls_validate_certificate] = ACTIONS(1511),
    [sym_tdata] = ACTIONS(1511),
    [sym_tchannel] = ACTIONS(1511),
    [sym_tlevel] = ACTIONS(1511),
    [sym_tbutton] = ACTIONS(1511),
    [sym_ttimeline] = ACTIONS(1511),
    [sym_tcustom] = ACTIONS(1511),
    [sym_url_struct] = ACTIONS(1511),
    [sym_dns_struct] = ACTIONS(1511),
    [sym_ip_address_struct] = ACTIONS(1511),
    [sym_dev_info_struct] = ACTIONS(1511),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1511),
    [sym_data_feed] = ACTIONS(1511),
    [sym_data_field] = ACTIONS(1511),
    [sym_data_record] = ACTIONS(1511),
    [sym_wc_data_feed] = ACTIONS(1511),
    [sym_wc_data_field] = ACTIONS(1511),
    [sym_wc_data_record] = ACTIONS(1511),
    [sym_last_login_info] = ACTIONS(1511),
    [sym____reserved___] = ACTIONS(1511),
    [sym_date] = ACTIONS(1511),
    [sym_day] = ACTIONS(1511),
    [sym_time] = ACTIONS(1511),
    [sym_ldate] = ACTIONS(1511),
    [sym_push_device] = ACTIONS(1511),
    [sym_push_channel] = ACTIONS(1511),
    [sym_push_devchan] = ACTIONS(1511),
    [sym_release_device] = ACTIONS(1511),
    [sym_release_channel] = ACTIONS(1511),
    [sym_release_devchan] = ACTIONS(1511),
    [sym_master_slot] = ACTIONS(1511),
    [sym_get_pulse_time] = ACTIONS(1511),
    [sym_get_timer] = ACTIONS(1511),
    [sym_master_sn] = ACTIONS(1511),
    [sym_system_number] = ACTIONS(1511),
    [sym_button] = ACTIONS(1511),
    [sym_dv_channel] = ACTIONS(1511),
    [sym_data] = ACTIONS(1511),
    [sym_level] = ACTIONS(1511),
    [sym_timeline] = ACTIONS(1511),
    [sym_channel] = ACTIONS(1511),
    [sym_custom] = ACTIONS(1511),
    [sym_atoi] = ACTIONS(1511),
    [sym_atol] = ACTIONS(1511),
    [sym_atof] = ACTIONS(1511),
    [sym_itoa] = ACTIONS(1511),
    [sym_itohex] = ACTIONS(1511),
    [sym_hextoi] = ACTIONS(1511),
    [sym_ftoa] = ACTIONS(1511),
    [sym_find_string] = ACTIONS(1511),
    [sym_left_string] = ACTIONS(1511),
    [sym_right_string] = ACTIONS(1511),
    [sym_mid_string] = ACTIONS(1511),
    [sym_remove_string] = ACTIONS(1511),
    [sym_get_buffer_string] = ACTIONS(1511),
    [sym_get_buffer_char] = ACTIONS(1511),
    [sym_get_multi_buffer_string] = ACTIONS(1511),
    [sym_length_string] = ACTIONS(1511),
    [sym_max_length_string] = ACTIONS(1511),
    [sym_set_length_string] = ACTIONS(1511),
    [sym_compare_string] = ACTIONS(1511),
    [sym_upper_string] = ACTIONS(1511),
    [sym_lower_string] = ACTIONS(1511),
    [sym_get_last] = ACTIONS(1511),
    [sym_date_to_day] = ACTIONS(1511),
    [sym_date_to_month] = ACTIONS(1511),
    [sym_date_to_year] = ACTIONS(1511),
    [sym_time_to_hour] = ACTIONS(1511),
    [sym_time_to_minute] = ACTIONS(1511),
    [sym_time_to_second] = ACTIONS(1511),
    [sym_day_of_week] = ACTIONS(1511),
    [sym_set_pulse_time] = ACTIONS(1511),
    [sym_set_timer] = ACTIONS(1511),
    [sym_file_open] = ACTIONS(1511),
    [sym_file_close] = ACTIONS(1511),
    [sym_file_read] = ACTIONS(1511),
    [sym_file_read_line] = ACTIONS(1511),
    [sym_file_write] = ACTIONS(1511),
    [sym_file_write_line] = ACTIONS(1511),
    [sym_file_copy] = ACTIONS(1511),
    [sym_file_delete] = ACTIONS(1511),
    [sym_file_rename] = ACTIONS(1511),
    [sym_file_dir] = ACTIONS(1511),
    [sym_file_getdir] = ACTIONS(1511),
    [sym_file_setdir] = ACTIONS(1511),
    [sym_file_create_dir] = ACTIONS(1511),
    [sym_file_remove_dir] = ACTIONS(1511),
    [sym_file_seek] = ACTIONS(1511),
    [sym_length_array] = ACTIONS(1511),
    [sym_max_length_array] = ACTIONS(1511),
    [sym_set_length_array] = ACTIONS(1511),
    [sym_ip_client_open] = ACTIONS(1511),
    [sym_ip_client_close] = ACTIONS(1511),
    [sym_ip_server_open] = ACTIONS(1511),
    [sym_ip_server_close] = ACTIONS(1511),
    [sym_ip_bound_client_open] = ACTIONS(1511),
    [sym_ip_mc_server_open] = ACTIONS(1511),
    [sym_ip_set_option] = ACTIONS(1511),
    [sym_ssh_client_open] = ACTIONS(1511),
    [sym_ssh_client_close] = ACTIONS(1511),
    [sym_tls_client_open] = ACTIONS(1511),
    [sym_tls_client_close] = ACTIONS(1511),
    [sym_do_push] = ACTIONS(1511),
    [sym_do_push_timed] = ACTIONS(1511),
    [sym_do_release] = ACTIONS(1511),
    [sym_device_id] = ACTIONS(1511),
    [sym_device_info] = ACTIONS(1511),
    [sym_device_id_string] = ACTIONS(1511),
    [sym_device_standby] = ACTIONS(1511),
    [sym_device_wake] = ACTIONS(1511),
    [sym_timeline_create] = ACTIONS(1511),
    [sym_timeline_kill] = ACTIONS(1511),
    [sym_timeline_reload] = ACTIONS(1511),
    [sym_timeline_pause] = ACTIONS(1511),
    [sym_timeline_restart] = ACTIONS(1511),
    [sym_timeline_set] = ACTIONS(1511),
    [sym_timeline_get] = ACTIONS(1511),
    [sym_timeline_active] = ACTIONS(1511),
    [sym_timeline_dynamic_id] = ACTIONS(1511),
    [sym_variable_to_string] = ACTIONS(1511),
    [sym_string_to_variable] = ACTIONS(1511),
    [sym_length_variable_to_string] = ACTIONS(1511),
    [sym_type_cast] = ACTIONS(1511),
    [sym_variable_to_xml] = ACTIONS(1511),
    [sym_xml_to_variable] = ACTIONS(1511),
    [sym_length_variable_to_xml] = ACTIONS(1511),
    [sym_redirect_string] = ACTIONS(1511),
    [sym_random_number] = ACTIONS(1511),
    [sym_get_unique_id] = ACTIONS(1511),
    [sym_get_system_number] = ACTIONS(1511),
    [sym_set_system_number] = ACTIONS(1511),
    [sym_get_serial_number] = ACTIONS(1511),
    [sym_get_master_build] = ACTIONS(1511),
    [sym_set_validation_code] = ACTIONS(1511),
    [sym_reboot] = ACTIONS(1511),
    [sym_load_duet_module] = ACTIONS(1511),
    [sym_unload_duet_module] = ACTIONS(1511),
    [sym_duet_mem_size_set] = ACTIONS(1511),
    [sym_duet_mem_size_get] = ACTIONS(1511),
    [sym_dynamic_application_device] = ACTIONS(1511),
    [sym_dynamic_polled_port] = ACTIONS(1511),
    [sym_static_port_binding] = ACTIONS(1511),
    [sym_static_ip_binding] = ACTIONS(1511),
    [sym_get_dns_list] = ACTIONS(1511),
    [sym_set_dns_list] = ACTIONS(1511),
    [sym_get_ip_address] = ACTIONS(1511),
    [sym_set_ip_address] = ACTIONS(1511),
    [sym_get_url_list] = ACTIONS(1511),
    [sym_add_url_entry] = ACTIONS(1511),
    [sym_delete_url_entry] = ACTIONS(1511),
    [sym_abs_value] = ACTIONS(1511),
    [sym_max_value] = ACTIONS(1511),
    [sym_min_value] = ACTIONS(1511),
    [sym_exp_value] = ACTIONS(1511),
    [sym_log_value] = ACTIONS(1511),
    [sym_log10_value] = ACTIONS(1511),
    [sym_power_value] = ACTIONS(1511),
    [sym_sqrt_value] = ACTIONS(1511),
    [sym_raw_be] = ACTIONS(1511),
    [sym_raw_le] = ACTIONS(1511),
    [sym_internal_queue_size_set] = ACTIONS(1511),
    [sym_internal_queue_size_get] = ACTIONS(1511),
    [sym_internal_threshold_set] = ACTIONS(1511),
    [sym_internal_threshold_get] = ACTIONS(1511),
    [sym_rebuilt_event] = ACTIONS(1511),
    [sym_do_custom_event] = ACTIONS(1511),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1511),
    [sym_clkmgr_set_clk_source] = ACTIONS(1511),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1511),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1511),
    [sym_clkmgr_get_timezone] = ACTIONS(1511),
    [sym_clkmgr_set_timezone] = ACTIONS(1511),
    [sym_clkmgr_get_resync_period] = ACTIONS(1511),
    [sym_clkmgr_set_resync_period] = ACTIONS(1511),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1511),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1511),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1511),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1511),
    [sym_clkmgr_get_timeservers] = ACTIONS(1511),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1511),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1511),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1511),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1511),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1511),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1511),
    [sym_set_log_level] = ACTIONS(1511),
    [sym_get_log_level] = ACTIONS(1511),
    [sym_amx_log] = ACTIONS(1511),
    [sym_smtp_server_config_set] = ACTIONS(1511),
    [sym_smtp_server_config_get] = ACTIONS(1511),
    [sym_smtp_send] = ACTIONS(1511),
    [sym_validate_netlinx_account] = ACTIONS(1511),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1511),
    [sym_authenticate_certificate] = ACTIONS(1511),
    [sym_audit_netlinx_session_event] = ACTIONS(1511),
    [sym_audit_netlinx_generic_event] = ACTIONS(1511),
    [sym_get_available_flash_disk_space] = ACTIONS(1511),
    [sym_get_max_flash_disk_space] = ACTIONS(1511),
    [sym_data_create_feed] = ACTIONS(1511),
    [sym_data_delete_feed] = ACTIONS(1511),
    [sym_data_publish_feed] = ACTIONS(1511),
    [sym_data_get_published_feed] = ACTIONS(1511),
    [sym_data_add_record] = ACTIONS(1511),
    [sym_data_get_event_record] = ACTIONS(1511),
    [sym__wc_data_create_feed] = ACTIONS(1511),
    [sym__wc_data_add_record] = ACTIONS(1511),
    [sym__wc_data_get_event_record] = ACTIONS(1511),
    [sym_format] = ACTIONS(1511),
    [sym_astro_clock] = ACTIONS(1511),
    [sym_set_outdoor_temperature] = ACTIONS(1511),
    [sym_set_virtual_level_count] = ACTIONS(1511),
    [sym_set_virtual_channel_count] = ACTIONS(1511),
    [sym_set_virtual_port_count] = ACTIONS(1511),
    [sym__wc_encode_int] = ACTIONS(1511),
    [sym__wc_decode_int] = ACTIONS(1511),
    [sym__wc_int] = ACTIONS(1511),
    [sym__wc_to_ch_int] = ACTIONS(1511),
    [sym__ch_to_wc_int] = ACTIONS(1511),
    [sym__wc_find_string_int] = ACTIONS(1511),
    [sym__wc_left_string_int] = ACTIONS(1511),
    [sym__wc_lower_string_int] = ACTIONS(1511),
    [sym__wc_mid_string_int] = ACTIONS(1511),
    [sym__wc_remove_string_int] = ACTIONS(1511),
    [sym__wc_right_string_int] = ACTIONS(1511),
    [sym__wc_upper_string_int] = ACTIONS(1511),
    [sym__wc_compare_string_int] = ACTIONS(1511),
    [sym__wc_get_buffer_char_int] = ACTIONS(1511),
    [sym__wc_get_buffer_string_int] = ACTIONS(1511),
    [sym__wc_concat_string_int] = ACTIONS(1511),
    [sym__wc_file_open_int] = ACTIONS(1511),
    [sym__wc_file_close_int] = ACTIONS(1511),
    [sym__wc_file_read_int] = ACTIONS(1511),
    [sym__wc_file_read_line_int] = ACTIONS(1511),
    [sym__wc_file_write_int] = ACTIONS(1511),
    [sym__wc_file_write_line_int] = ACTIONS(1511),
    [anon_sym_EQ] = ACTIONS(1511),
    [aux_sym_preproc_include_token1] = ACTIONS(1511),
    [aux_sym_preproc_define_token1] = ACTIONS(1513),
    [aux_sym_preproc_warn_token1] = ACTIONS(1513),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1513),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1513),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1513),
    [anon_sym_LBRACK] = ACTIONS(1513),
    [anon_sym_LBRACE] = ACTIONS(1513),
    [anon_sym_COLON] = ACTIONS(1513),
    [anon_sym_LPAREN] = ACTIONS(1511),
    [anon_sym_SEMI] = ACTIONS(1513),
    [anon_sym_BANG] = ACTIONS(1511),
    [anon_sym_TILDE] = ACTIONS(1513),
    [anon_sym_DASH] = ACTIONS(1511),
    [anon_sym_PLUS] = ACTIONS(1511),
    [anon_sym_STAR] = ACTIONS(1513),
    [anon_sym_SLASH] = ACTIONS(1511),
    [anon_sym_PERCENT] = ACTIONS(1513),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_AMP_AMP] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1511),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_AMP] = ACTIONS(1511),
    [anon_sym_EQ_EQ] = ACTIONS(1513),
    [anon_sym_BANG_EQ] = ACTIONS(1513),
    [anon_sym_LT_GT] = ACTIONS(1513),
    [anon_sym_GT] = ACTIONS(1511),
    [anon_sym_GT_EQ] = ACTIONS(1513),
    [anon_sym_LT_EQ] = ACTIONS(1513),
    [anon_sym_LT] = ACTIONS(1511),
    [anon_sym_LT_LT] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1513),
    [anon_sym_DASH_DASH] = ACTIONS(1513),
    [anon_sym_PLUS_PLUS] = ACTIONS(1513),
    [anon_sym_DOT] = ACTIONS(1511),
    [anon_sym_DQUOTE] = ACTIONS(1513),
    [anon_sym_SQUOTE] = ACTIONS(1513),
    [sym_number_literal] = ACTIONS(1513),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1513),
  },
  [STATE(132)] = {
    [sym_identifier] = ACTIONS(1593),
    [sym_program_name_keyword] = ACTIONS(1593),
    [sym_module_name_keyword] = ACTIONS(1593),
    [sym_define_device_keyword] = ACTIONS(1593),
    [sym_define_combine_keyword] = ACTIONS(1593),
    [sym_define_constant_keyword] = ACTIONS(1593),
    [sym_define_type_keyword] = ACTIONS(1593),
    [sym_define_variable_keyword] = ACTIONS(1593),
    [sym_define_system_variable_keyword] = ACTIONS(1593),
    [sym_define_connect_level_keyword] = ACTIONS(1593),
    [sym_define_latching_keyword] = ACTIONS(1593),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1593),
    [sym_define_toggling_keyword] = ACTIONS(1593),
    [sym_define_call_keyword] = ACTIONS(1593),
    [sym_define_function_keyword] = ACTIONS(1593),
    [sym_define_library_function_keyword] = ACTIONS(1593),
    [sym_define_start_keyword] = ACTIONS(1593),
    [sym_define_event_keyword] = ACTIONS(1593),
    [sym_define_program_keyword] = ACTIONS(1593),
    [sym_define_module_keyword] = ACTIONS(1593),
    [sym_char_keyword] = ACTIONS(1593),
    [sym_widechar_keyword] = ACTIONS(1593),
    [sym_integer_keyword] = ACTIONS(1593),
    [sym_sinteger_keyword] = ACTIONS(1593),
    [sym_long_keyword] = ACTIONS(1593),
    [sym_slong_keyword] = ACTIONS(1593),
    [sym_float_keyword] = ACTIONS(1593),
    [sym_double_keyword] = ACTIONS(1593),
    [sym_variant_keyword] = ACTIONS(1593),
    [sym_variantarray_keyword] = ACTIONS(1593),
    [sym_local_var_keyword] = ACTIONS(1593),
    [sym_stack_var_keyword] = ACTIONS(1593),
    [sym_constant_keyword] = ACTIONS(1593),
    [sym_volatile_keyword] = ACTIONS(1593),
    [sym_non_volatile_keyword] = ACTIONS(1593),
    [sym_persistent_keyword] = ACTIONS(1593),
    [sym_dev_keyword] = ACTIONS(1593),
    [sym_devlev_keyword] = ACTIONS(1593),
    [sym_devchan_keyword] = ACTIONS(1593),
    [sym_if_keyword] = ACTIONS(1593),
    [sym_else_keyword] = ACTIONS(1593),
    [sym_switch_keyword] = ACTIONS(1593),
    [sym_case_keyword] = ACTIONS(1593),
    [sym_default_keyword] = ACTIONS(1593),
    [sym_while_keyword] = ACTIONS(1593),
    [sym_for_keyword] = ACTIONS(1593),
    [sym_break_keyword] = ACTIONS(1593),
    [sym_continue_keyword] = ACTIONS(1593),
    [sym_return_keyword] = ACTIONS(1593),
    [sym_select_keyword] = ACTIONS(1593),
    [sym_active_keyword] = ACTIONS(1593),
    [sym_struct_keyword] = ACTIONS(1593),
    [sym_structure_keyword] = ACTIONS(1593),
    [sym_button_event_keyword] = ACTIONS(1593),
    [sym_channel_event_keyword] = ACTIONS(1593),
    [sym_level_event_keyword] = ACTIONS(1593),
    [sym_data_event_keyword] = ACTIONS(1593),
    [sym_timeline_event_keyword] = ACTIONS(1593),
    [sym_custom_event_keyword] = ACTIONS(1593),
    [sym_push_keyword] = ACTIONS(1593),
    [sym_release_keyword] = ACTIONS(1593),
    [aux_sym_on_keyword_token1] = ACTIONS(1593),
    [aux_sym_off_keyword_token1] = ACTIONS(1593),
    [sym_send_level_keyword] = ACTIONS(1593),
    [sym_send_string_keyword] = ACTIONS(1593),
    [sym_send_command_keyword] = ACTIONS(1593),
    [sym_clear_buffer_keyword] = ACTIONS(1593),
    [sym_create_buffer_keyword] = ACTIONS(1593),
    [sym_create_multi_buffer_keyword] = ACTIONS(1593),
    [sym_call_keyword] = ACTIONS(1593),
    [sym_system_call_keyword] = ACTIONS(1593),
    [sym_devchan_to_keyword] = ACTIONS(1593),
    [sym_devchan_min_to_keyword] = ACTIONS(1593),
    [sym_devchan_total_off_keyword] = ACTIONS(1593),
    [sym_devchan_pulse_keyword] = ACTIONS(1593),
    [sym_wait_keyword] = ACTIONS(1593),
    [sym_cancel_wait_keyword] = ACTIONS(1593),
    [sym_wait_until_keyword] = ACTIONS(1593),
    [sym_cancel_wait_until_keyword] = ACTIONS(1593),
    [sym_cancel_all_wait_keyword] = ACTIONS(1593),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1593),
    [sym_and] = ACTIONS(1593),
    [sym_or] = ACTIONS(1593),
    [sym_not] = ACTIONS(1593),
    [sym_xor] = ACTIONS(1593),
    [sym_band] = ACTIONS(1593),
    [sym_bor] = ACTIONS(1593),
    [sym_bxor] = ACTIONS(1593),
    [sym_bnot] = ACTIONS(1593),
    [sym_lshift] = ACTIONS(1593),
    [sym_rshift] = ACTIONS(1593),
    [sym___file__] = ACTIONS(1593),
    [sym___date__] = ACTIONS(1593),
    [sym___ldate__] = ACTIONS(1593),
    [sym___time__] = ACTIONS(1593),
    [sym___line__] = ACTIONS(1593),
    [sym___name__] = ACTIONS(1593),
    [sym___netlinx__] = ACTIONS(1593),
    [sym_netlinx_axi_version] = ACTIONS(1593),
    [sym_url_flg_tcp] = ACTIONS(1593),
    [sym_url_flg_accinfopresent] = ACTIONS(1593),
    [sym_url_flg_temp] = ACTIONS(1593),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1593),
    [sym_url_flg_stat_mask] = ACTIONS(1593),
    [sym_url_flg_stat_lookup] = ACTIONS(1593),
    [sym_url_flg_stat_connecting] = ACTIONS(1593),
    [sym_url_flg_stat_waiting] = ACTIONS(1593),
    [sym_url_flg_stat_connected] = ACTIONS(1593),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1593),
    [sym_first_virtual_device] = ACTIONS(1593),
    [sym_first_local_port] = ACTIONS(1593),
    [sym_dynamic_virtual_device] = ACTIONS(1593),
    [sym_timeline_once] = ACTIONS(1593),
    [sym_timeline_repeat] = ACTIONS(1593),
    [sym_timeline_absolute] = ACTIONS(1593),
    [sym_timeline_relative] = ACTIONS(1593),
    [sym_do_push_timed_infinite] = ACTIONS(1593),
    [sym_source_type_no_address] = ACTIONS(1593),
    [sym_source_type_neuron_id] = ACTIONS(1593),
    [sym_source_type_ip_address] = ACTIONS(1593),
    [sym_source_type_axlink] = ACTIONS(1593),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1593),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1593),
    [sym_source_type_ip_socket_address] = ACTIONS(1593),
    [sym_source_type_rs232] = ACTIONS(1593),
    [sym_source_type_internal] = ACTIONS(1593),
    [sym_source_type_mac_address] = ACTIONS(1593),
    [sym_source_type_ipv4_port] = ACTIONS(1593),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1593),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1593),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1593),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1593),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1593),
    [sym_true] = ACTIONS(1593),
    [sym_false] = ACTIONS(1593),
    [sym_file_read_only] = ACTIONS(1593),
    [sym_file_rw_new] = ACTIONS(1593),
    [sym_file_rw_append] = ACTIONS(1593),
    [sym_ip_tcp] = ACTIONS(1593),
    [sym_ip_udp] = ACTIONS(1593),
    [sym_ip_udp_2way] = ACTIONS(1593),
    [sym_xml_encode_types] = ACTIONS(1593),
    [sym_xml_encode_char_as_list] = ACTIONS(1593),
    [sym_xml_encode_le] = ACTIONS(1593),
    [sym_xml_decode_types] = ACTIONS(1593),
    [sym_xml_decode_no_preserve] = ACTIONS(1593),
    [sym_duet_dev_type_display_device] = ACTIONS(1593),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1593),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1593),
    [sym_duet_dev_type_io_device] = ACTIONS(1593),
    [sym_duet_dev_type_relay_device] = ACTIONS(1593),
    [sym_duet_dev_type_ups] = ACTIONS(1593),
    [sym_duet_dev_type_amplifier] = ACTIONS(1593),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1593),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1593),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1593),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1593),
    [sym_duet_dev_type_camera] = ACTIONS(1593),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1593),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1593),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1593),
    [sym_duet_dev_type_dss] = ACTIONS(1593),
    [sym_duet_dev_type_dvr] = ACTIONS(1593),
    [sym_duet_dev_type_disc_device] = ACTIONS(1593),
    [sym_duet_dev_type_document_camera] = ACTIONS(1593),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1593),
    [sym_duet_dev_type_hvac] = ACTIONS(1593),
    [sym_duet_dev_type_keypad] = ACTIONS(1593),
    [sym_duet_dev_type_light] = ACTIONS(1593),
    [sym_duet_dev_type_monitor] = ACTIONS(1593),
    [sym_duet_dev_type_motor] = ACTIONS(1593),
    [sym_duet_dev_type_multi_window] = ACTIONS(1593),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1593),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1593),
    [sym_duet_dev_type_receiver] = ACTIONS(1593),
    [sym_duet_dev_type_security_system] = ACTIONS(1593),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1593),
    [sym_duet_dev_type_settop_box] = ACTIONS(1593),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1593),
    [sym_duet_dev_type_switcher] = ACTIONS(1593),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1593),
    [sym_duet_dev_type_tv] = ACTIONS(1593),
    [sym_duet_dev_type_utility] = ACTIONS(1593),
    [sym_duet_dev_type_vcr] = ACTIONS(1593),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1593),
    [sym_duet_dev_type_video_processor] = ACTIONS(1593),
    [sym_duet_dev_type_video_projector] = ACTIONS(1593),
    [sym_duet_dev_type_video_wall] = ACTIONS(1593),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1593),
    [sym_duet_dev_type_weather] = ACTIONS(1593),
    [sym_duet_dev_not_polled] = ACTIONS(1593),
    [sym_duet_dev_polled] = ACTIONS(1593),
    [sym_clkmgr_mode_network] = ACTIONS(1593),
    [sym_clkmgr_mode_standalone] = ACTIONS(1593),
    [sym_normal_standby] = ACTIONS(1593),
    [sym_normal_wake] = ACTIONS(1593),
    [sym_ip_multicast_ttl_option] = ACTIONS(1593),
    [sym_ip_ttl_subnet] = ACTIONS(1593),
    [sym_ip_ttl_site] = ACTIONS(1593),
    [sym_ip_ttl_region] = ACTIONS(1593),
    [sym_ip_ttl_continent] = ACTIONS(1593),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1593),
    [sym_ip_nodelay_on] = ACTIONS(1593),
    [sym_ip_nodelay_off] = ACTIONS(1593),
    [sym_amx_error] = ACTIONS(1593),
    [sym_amx_warning] = ACTIONS(1593),
    [sym_amx_info] = ACTIONS(1593),
    [sym_amx_debug] = ACTIONS(1593),
    [sym_netlinx_login_success] = ACTIONS(1593),
    [sym_netlinx_login_fail] = ACTIONS(1593),
    [sym_netlinx_logout] = ACTIONS(1593),
    [sym_null_str] = ACTIONS(1593),
    [sym_smtp_address] = ACTIONS(1593),
    [sym_smtp_port_number] = ACTIONS(1593),
    [sym_smtp_username] = ACTIONS(1593),
    [sym_smtp_password] = ACTIONS(1593),
    [sym_smtp_require_tls] = ACTIONS(1593),
    [sym_smtp_from] = ACTIONS(1593),
    [sym_smtp_tls_true] = ACTIONS(1593),
    [sym_smtp_tls_false] = ACTIONS(1593),
    [sym_listview_on_row_select_event] = ACTIONS(1593),
    [sym_data_structure_unknown] = ACTIONS(1593),
    [sym_data_structure_datafeed] = ACTIONS(1593),
    [sym_data_structure_datarecordset] = ACTIONS(1593),
    [sym_data_structure_datarecord] = ACTIONS(1593),
    [sym_data_structure_datafield] = ACTIONS(1593),
    [sym_data_max_content_field_count] = ACTIONS(1593),
    [sym_data_max_metadata_field_count] = ACTIONS(1593),
    [sym_data_max_id_length] = ACTIONS(1593),
    [sym_data_max_name_length] = ACTIONS(1593),
    [sym_data_max_description_length] = ACTIONS(1593),
    [sym_data_max_type_length] = ACTIONS(1593),
    [sym_data_max_format_length] = ACTIONS(1593),
    [sym_data_max_source_length] = ACTIONS(1593),
    [sym_data_max_value_length] = ACTIONS(1593),
    [sym_data_max_label_length] = ACTIONS(1593),
    [sym_data_max_url_length] = ACTIONS(1593),
    [sym_max_last_login_info_length] = ACTIONS(1593),
    [sym_data_type_unknown] = ACTIONS(1593),
    [sym_data_type_string] = ACTIONS(1593),
    [sym_data_type_datetime] = ACTIONS(1593),
    [sym_data_type_date] = ACTIONS(1593),
    [sym_data_type_time] = ACTIONS(1593),
    [sym_data_type_image] = ACTIONS(1593),
    [sym_data_format_url] = ACTIONS(1593),
    [sym_data_format_phone] = ACTIONS(1593),
    [sym_data_format_email] = ACTIONS(1593),
    [sym_data_format_iso8601] = ACTIONS(1593),
    [sym_auditlog_priv] = ACTIONS(1593),
    [sym_configuration_priv] = ACTIONS(1593),
    [sym_deviceconfig_priv] = ACTIONS(1593),
    [sym_ftp_priv] = ACTIONS(1593),
    [sym_http_priv] = ACTIONS(1593),
    [sym_networkconfig_priv] = ACTIONS(1593),
    [sym_programport_priv] = ACTIONS(1593),
    [sym_remoteui_priv] = ACTIONS(1593),
    [sym_securitycontrol_priv] = ACTIONS(1593),
    [sym_softwaremanagement_priv] = ACTIONS(1593),
    [sym_terminal_priv] = ACTIONS(1593),
    [sym_tpadmin_priv] = ACTIONS(1593),
    [sym_user1_priv] = ACTIONS(1593),
    [sym_user2_priv] = ACTIONS(1593),
    [sym_user3_priv] = ACTIONS(1593),
    [sym_user4_priv] = ACTIONS(1593),
    [sym_usermanagement_priv] = ACTIONS(1593),
    [sym_valid_account] = ACTIONS(1593),
    [sym_err_name_invalid] = ACTIONS(1593),
    [sym_err_password_invalid] = ACTIONS(1593),
    [sym_err_invalid_account] = ACTIONS(1593),
    [sym_err_no_authorization] = ACTIONS(1593),
    [sym_err_invalid_parameter] = ACTIONS(1593),
    [sym_err_user_locked_out] = ACTIONS(1593),
    [sym_err_user_account_expired] = ACTIONS(1593),
    [sym_err_general_error] = ACTIONS(1593),
    [sym_err_authentication_error] = ACTIONS(1593),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1593),
    [sym_tls_validate_certificate] = ACTIONS(1593),
    [sym_tdata] = ACTIONS(1593),
    [sym_tchannel] = ACTIONS(1593),
    [sym_tlevel] = ACTIONS(1593),
    [sym_tbutton] = ACTIONS(1593),
    [sym_ttimeline] = ACTIONS(1593),
    [sym_tcustom] = ACTIONS(1593),
    [sym_url_struct] = ACTIONS(1593),
    [sym_dns_struct] = ACTIONS(1593),
    [sym_ip_address_struct] = ACTIONS(1593),
    [sym_dev_info_struct] = ACTIONS(1593),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1593),
    [sym_data_feed] = ACTIONS(1593),
    [sym_data_field] = ACTIONS(1593),
    [sym_data_record] = ACTIONS(1593),
    [sym_wc_data_feed] = ACTIONS(1593),
    [sym_wc_data_field] = ACTIONS(1593),
    [sym_wc_data_record] = ACTIONS(1593),
    [sym_last_login_info] = ACTIONS(1593),
    [sym____reserved___] = ACTIONS(1593),
    [sym_date] = ACTIONS(1593),
    [sym_day] = ACTIONS(1593),
    [sym_time] = ACTIONS(1593),
    [sym_ldate] = ACTIONS(1593),
    [sym_push_device] = ACTIONS(1593),
    [sym_push_channel] = ACTIONS(1593),
    [sym_push_devchan] = ACTIONS(1593),
    [sym_release_device] = ACTIONS(1593),
    [sym_release_channel] = ACTIONS(1593),
    [sym_release_devchan] = ACTIONS(1593),
    [sym_master_slot] = ACTIONS(1593),
    [sym_get_pulse_time] = ACTIONS(1593),
    [sym_get_timer] = ACTIONS(1593),
    [sym_master_sn] = ACTIONS(1593),
    [sym_system_number] = ACTIONS(1593),
    [sym_button] = ACTIONS(1593),
    [sym_dv_channel] = ACTIONS(1593),
    [sym_data] = ACTIONS(1593),
    [sym_level] = ACTIONS(1593),
    [sym_timeline] = ACTIONS(1593),
    [sym_channel] = ACTIONS(1593),
    [sym_custom] = ACTIONS(1593),
    [sym_atoi] = ACTIONS(1593),
    [sym_atol] = ACTIONS(1593),
    [sym_atof] = ACTIONS(1593),
    [sym_itoa] = ACTIONS(1593),
    [sym_itohex] = ACTIONS(1593),
    [sym_hextoi] = ACTIONS(1593),
    [sym_ftoa] = ACTIONS(1593),
    [sym_find_string] = ACTIONS(1593),
    [sym_left_string] = ACTIONS(1593),
    [sym_right_string] = ACTIONS(1593),
    [sym_mid_string] = ACTIONS(1593),
    [sym_remove_string] = ACTIONS(1593),
    [sym_get_buffer_string] = ACTIONS(1593),
    [sym_get_buffer_char] = ACTIONS(1593),
    [sym_get_multi_buffer_string] = ACTIONS(1593),
    [sym_length_string] = ACTIONS(1593),
    [sym_max_length_string] = ACTIONS(1593),
    [sym_set_length_string] = ACTIONS(1593),
    [sym_compare_string] = ACTIONS(1593),
    [sym_upper_string] = ACTIONS(1593),
    [sym_lower_string] = ACTIONS(1593),
    [sym_get_last] = ACTIONS(1593),
    [sym_date_to_day] = ACTIONS(1593),
    [sym_date_to_month] = ACTIONS(1593),
    [sym_date_to_year] = ACTIONS(1593),
    [sym_time_to_hour] = ACTIONS(1593),
    [sym_time_to_minute] = ACTIONS(1593),
    [sym_time_to_second] = ACTIONS(1593),
    [sym_day_of_week] = ACTIONS(1593),
    [sym_set_pulse_time] = ACTIONS(1593),
    [sym_set_timer] = ACTIONS(1593),
    [sym_file_open] = ACTIONS(1593),
    [sym_file_close] = ACTIONS(1593),
    [sym_file_read] = ACTIONS(1593),
    [sym_file_read_line] = ACTIONS(1593),
    [sym_file_write] = ACTIONS(1593),
    [sym_file_write_line] = ACTIONS(1593),
    [sym_file_copy] = ACTIONS(1593),
    [sym_file_delete] = ACTIONS(1593),
    [sym_file_rename] = ACTIONS(1593),
    [sym_file_dir] = ACTIONS(1593),
    [sym_file_getdir] = ACTIONS(1593),
    [sym_file_setdir] = ACTIONS(1593),
    [sym_file_create_dir] = ACTIONS(1593),
    [sym_file_remove_dir] = ACTIONS(1593),
    [sym_file_seek] = ACTIONS(1593),
    [sym_length_array] = ACTIONS(1593),
    [sym_max_length_array] = ACTIONS(1593),
    [sym_set_length_array] = ACTIONS(1593),
    [sym_ip_client_open] = ACTIONS(1593),
    [sym_ip_client_close] = ACTIONS(1593),
    [sym_ip_server_open] = ACTIONS(1593),
    [sym_ip_server_close] = ACTIONS(1593),
    [sym_ip_bound_client_open] = ACTIONS(1593),
    [sym_ip_mc_server_open] = ACTIONS(1593),
    [sym_ip_set_option] = ACTIONS(1593),
    [sym_ssh_client_open] = ACTIONS(1593),
    [sym_ssh_client_close] = ACTIONS(1593),
    [sym_tls_client_open] = ACTIONS(1593),
    [sym_tls_client_close] = ACTIONS(1593),
    [sym_do_push] = ACTIONS(1593),
    [sym_do_push_timed] = ACTIONS(1593),
    [sym_do_release] = ACTIONS(1593),
    [sym_device_id] = ACTIONS(1593),
    [sym_device_info] = ACTIONS(1593),
    [sym_device_id_string] = ACTIONS(1593),
    [sym_device_standby] = ACTIONS(1593),
    [sym_device_wake] = ACTIONS(1593),
    [sym_timeline_create] = ACTIONS(1593),
    [sym_timeline_kill] = ACTIONS(1593),
    [sym_timeline_reload] = ACTIONS(1593),
    [sym_timeline_pause] = ACTIONS(1593),
    [sym_timeline_restart] = ACTIONS(1593),
    [sym_timeline_set] = ACTIONS(1593),
    [sym_timeline_get] = ACTIONS(1593),
    [sym_timeline_active] = ACTIONS(1593),
    [sym_timeline_dynamic_id] = ACTIONS(1593),
    [sym_variable_to_string] = ACTIONS(1593),
    [sym_string_to_variable] = ACTIONS(1593),
    [sym_length_variable_to_string] = ACTIONS(1593),
    [sym_type_cast] = ACTIONS(1593),
    [sym_variable_to_xml] = ACTIONS(1593),
    [sym_xml_to_variable] = ACTIONS(1593),
    [sym_length_variable_to_xml] = ACTIONS(1593),
    [sym_redirect_string] = ACTIONS(1593),
    [sym_random_number] = ACTIONS(1593),
    [sym_get_unique_id] = ACTIONS(1593),
    [sym_get_system_number] = ACTIONS(1593),
    [sym_set_system_number] = ACTIONS(1593),
    [sym_get_serial_number] = ACTIONS(1593),
    [sym_get_master_build] = ACTIONS(1593),
    [sym_set_validation_code] = ACTIONS(1593),
    [sym_reboot] = ACTIONS(1593),
    [sym_load_duet_module] = ACTIONS(1593),
    [sym_unload_duet_module] = ACTIONS(1593),
    [sym_duet_mem_size_set] = ACTIONS(1593),
    [sym_duet_mem_size_get] = ACTIONS(1593),
    [sym_dynamic_application_device] = ACTIONS(1593),
    [sym_dynamic_polled_port] = ACTIONS(1593),
    [sym_static_port_binding] = ACTIONS(1593),
    [sym_static_ip_binding] = ACTIONS(1593),
    [sym_get_dns_list] = ACTIONS(1593),
    [sym_set_dns_list] = ACTIONS(1593),
    [sym_get_ip_address] = ACTIONS(1593),
    [sym_set_ip_address] = ACTIONS(1593),
    [sym_get_url_list] = ACTIONS(1593),
    [sym_add_url_entry] = ACTIONS(1593),
    [sym_delete_url_entry] = ACTIONS(1593),
    [sym_abs_value] = ACTIONS(1593),
    [sym_max_value] = ACTIONS(1593),
    [sym_min_value] = ACTIONS(1593),
    [sym_exp_value] = ACTIONS(1593),
    [sym_log_value] = ACTIONS(1593),
    [sym_log10_value] = ACTIONS(1593),
    [sym_power_value] = ACTIONS(1593),
    [sym_sqrt_value] = ACTIONS(1593),
    [sym_raw_be] = ACTIONS(1593),
    [sym_raw_le] = ACTIONS(1593),
    [sym_internal_queue_size_set] = ACTIONS(1593),
    [sym_internal_queue_size_get] = ACTIONS(1593),
    [sym_internal_threshold_set] = ACTIONS(1593),
    [sym_internal_threshold_get] = ACTIONS(1593),
    [sym_rebuilt_event] = ACTIONS(1593),
    [sym_do_custom_event] = ACTIONS(1593),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1593),
    [sym_clkmgr_set_clk_source] = ACTIONS(1593),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1593),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1593),
    [sym_clkmgr_get_timezone] = ACTIONS(1593),
    [sym_clkmgr_set_timezone] = ACTIONS(1593),
    [sym_clkmgr_get_resync_period] = ACTIONS(1593),
    [sym_clkmgr_set_resync_period] = ACTIONS(1593),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1593),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1593),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1593),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1593),
    [sym_clkmgr_get_timeservers] = ACTIONS(1593),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1593),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1593),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1593),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1593),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1593),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1593),
    [sym_set_log_level] = ACTIONS(1593),
    [sym_get_log_level] = ACTIONS(1593),
    [sym_amx_log] = ACTIONS(1593),
    [sym_smtp_server_config_set] = ACTIONS(1593),
    [sym_smtp_server_config_get] = ACTIONS(1593),
    [sym_smtp_send] = ACTIONS(1593),
    [sym_validate_netlinx_account] = ACTIONS(1593),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1593),
    [sym_authenticate_certificate] = ACTIONS(1593),
    [sym_audit_netlinx_session_event] = ACTIONS(1593),
    [sym_audit_netlinx_generic_event] = ACTIONS(1593),
    [sym_get_available_flash_disk_space] = ACTIONS(1593),
    [sym_get_max_flash_disk_space] = ACTIONS(1593),
    [sym_data_create_feed] = ACTIONS(1593),
    [sym_data_delete_feed] = ACTIONS(1593),
    [sym_data_publish_feed] = ACTIONS(1593),
    [sym_data_get_published_feed] = ACTIONS(1593),
    [sym_data_add_record] = ACTIONS(1593),
    [sym_data_get_event_record] = ACTIONS(1593),
    [sym__wc_data_create_feed] = ACTIONS(1593),
    [sym__wc_data_add_record] = ACTIONS(1593),
    [sym__wc_data_get_event_record] = ACTIONS(1593),
    [sym_format] = ACTIONS(1593),
    [sym_astro_clock] = ACTIONS(1593),
    [sym_set_outdoor_temperature] = ACTIONS(1593),
    [sym_set_virtual_level_count] = ACTIONS(1593),
    [sym_set_virtual_channel_count] = ACTIONS(1593),
    [sym_set_virtual_port_count] = ACTIONS(1593),
    [sym__wc_encode_int] = ACTIONS(1593),
    [sym__wc_decode_int] = ACTIONS(1593),
    [sym__wc_int] = ACTIONS(1593),
    [sym__wc_to_ch_int] = ACTIONS(1593),
    [sym__ch_to_wc_int] = ACTIONS(1593),
    [sym__wc_find_string_int] = ACTIONS(1593),
    [sym__wc_left_string_int] = ACTIONS(1593),
    [sym__wc_lower_string_int] = ACTIONS(1593),
    [sym__wc_mid_string_int] = ACTIONS(1593),
    [sym__wc_remove_string_int] = ACTIONS(1593),
    [sym__wc_right_string_int] = ACTIONS(1593),
    [sym__wc_upper_string_int] = ACTIONS(1593),
    [sym__wc_compare_string_int] = ACTIONS(1593),
    [sym__wc_get_buffer_char_int] = ACTIONS(1593),
    [sym__wc_get_buffer_string_int] = ACTIONS(1593),
    [sym__wc_concat_string_int] = ACTIONS(1593),
    [sym__wc_file_open_int] = ACTIONS(1593),
    [sym__wc_file_close_int] = ACTIONS(1593),
    [sym__wc_file_read_int] = ACTIONS(1593),
    [sym__wc_file_read_line_int] = ACTIONS(1593),
    [sym__wc_file_write_int] = ACTIONS(1593),
    [sym__wc_file_write_line_int] = ACTIONS(1593),
    [aux_sym_preproc_include_token1] = ACTIONS(1593),
    [aux_sym_preproc_define_token1] = ACTIONS(1595),
    [aux_sym_preproc_warn_token1] = ACTIONS(1595),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1595),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1595),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1595),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1595),
    [aux_sym_preproc_else_token1] = ACTIONS(1595),
    [anon_sym_LBRACK] = ACTIONS(1595),
    [anon_sym_LBRACE] = ACTIONS(1595),
    [anon_sym_COLON] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1593),
    [anon_sym_SEMI] = ACTIONS(1595),
    [anon_sym_BANG] = ACTIONS(1593),
    [anon_sym_TILDE] = ACTIONS(1595),
    [anon_sym_DASH] = ACTIONS(1593),
    [anon_sym_PLUS] = ACTIONS(1593),
    [anon_sym_STAR] = ACTIONS(1595),
    [anon_sym_SLASH] = ACTIONS(1593),
    [anon_sym_PERCENT] = ACTIONS(1595),
    [anon_sym_PIPE_PIPE] = ACTIONS(1595),
    [anon_sym_AMP_AMP] = ACTIONS(1595),
    [anon_sym_PIPE] = ACTIONS(1593),
    [anon_sym_CARET] = ACTIONS(1595),
    [anon_sym_AMP] = ACTIONS(1593),
    [anon_sym_EQ_EQ] = ACTIONS(1595),
    [anon_sym_BANG_EQ] = ACTIONS(1595),
    [anon_sym_LT_GT] = ACTIONS(1595),
    [anon_sym_GT] = ACTIONS(1593),
    [anon_sym_GT_EQ] = ACTIONS(1595),
    [anon_sym_LT_EQ] = ACTIONS(1595),
    [anon_sym_LT] = ACTIONS(1593),
    [anon_sym_LT_LT] = ACTIONS(1595),
    [anon_sym_GT_GT] = ACTIONS(1595),
    [anon_sym_DASH_DASH] = ACTIONS(1595),
    [anon_sym_PLUS_PLUS] = ACTIONS(1595),
    [anon_sym_DOT] = ACTIONS(1593),
    [anon_sym_DQUOTE] = ACTIONS(1595),
    [anon_sym_SQUOTE] = ACTIONS(1595),
    [sym_number_literal] = ACTIONS(1595),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1595),
  },
  [STATE(133)] = {
    [sym_identifier] = ACTIONS(1597),
    [sym_program_name_keyword] = ACTIONS(1597),
    [sym_module_name_keyword] = ACTIONS(1597),
    [sym_define_device_keyword] = ACTIONS(1597),
    [sym_define_combine_keyword] = ACTIONS(1597),
    [sym_define_constant_keyword] = ACTIONS(1597),
    [sym_define_type_keyword] = ACTIONS(1597),
    [sym_define_variable_keyword] = ACTIONS(1597),
    [sym_define_system_variable_keyword] = ACTIONS(1597),
    [sym_define_connect_level_keyword] = ACTIONS(1597),
    [sym_define_latching_keyword] = ACTIONS(1597),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1597),
    [sym_define_toggling_keyword] = ACTIONS(1597),
    [sym_define_call_keyword] = ACTIONS(1597),
    [sym_define_function_keyword] = ACTIONS(1597),
    [sym_define_library_function_keyword] = ACTIONS(1597),
    [sym_define_start_keyword] = ACTIONS(1597),
    [sym_define_event_keyword] = ACTIONS(1597),
    [sym_define_program_keyword] = ACTIONS(1597),
    [sym_define_module_keyword] = ACTIONS(1597),
    [sym_char_keyword] = ACTIONS(1597),
    [sym_widechar_keyword] = ACTIONS(1597),
    [sym_integer_keyword] = ACTIONS(1597),
    [sym_sinteger_keyword] = ACTIONS(1597),
    [sym_long_keyword] = ACTIONS(1597),
    [sym_slong_keyword] = ACTIONS(1597),
    [sym_float_keyword] = ACTIONS(1597),
    [sym_double_keyword] = ACTIONS(1597),
    [sym_variant_keyword] = ACTIONS(1597),
    [sym_variantarray_keyword] = ACTIONS(1597),
    [sym_local_var_keyword] = ACTIONS(1597),
    [sym_stack_var_keyword] = ACTIONS(1597),
    [sym_constant_keyword] = ACTIONS(1597),
    [sym_volatile_keyword] = ACTIONS(1597),
    [sym_non_volatile_keyword] = ACTIONS(1597),
    [sym_persistent_keyword] = ACTIONS(1597),
    [sym_dev_keyword] = ACTIONS(1597),
    [sym_devlev_keyword] = ACTIONS(1597),
    [sym_devchan_keyword] = ACTIONS(1597),
    [sym_if_keyword] = ACTIONS(1597),
    [sym_else_keyword] = ACTIONS(1597),
    [sym_switch_keyword] = ACTIONS(1597),
    [sym_case_keyword] = ACTIONS(1597),
    [sym_default_keyword] = ACTIONS(1597),
    [sym_while_keyword] = ACTIONS(1597),
    [sym_for_keyword] = ACTIONS(1597),
    [sym_break_keyword] = ACTIONS(1597),
    [sym_continue_keyword] = ACTIONS(1597),
    [sym_return_keyword] = ACTIONS(1597),
    [sym_select_keyword] = ACTIONS(1597),
    [sym_active_keyword] = ACTIONS(1597),
    [sym_struct_keyword] = ACTIONS(1597),
    [sym_structure_keyword] = ACTIONS(1597),
    [sym_button_event_keyword] = ACTIONS(1597),
    [sym_channel_event_keyword] = ACTIONS(1597),
    [sym_level_event_keyword] = ACTIONS(1597),
    [sym_data_event_keyword] = ACTIONS(1597),
    [sym_timeline_event_keyword] = ACTIONS(1597),
    [sym_custom_event_keyword] = ACTIONS(1597),
    [sym_push_keyword] = ACTIONS(1597),
    [sym_release_keyword] = ACTIONS(1597),
    [aux_sym_on_keyword_token1] = ACTIONS(1597),
    [aux_sym_off_keyword_token1] = ACTIONS(1597),
    [sym_send_level_keyword] = ACTIONS(1597),
    [sym_send_string_keyword] = ACTIONS(1597),
    [sym_send_command_keyword] = ACTIONS(1597),
    [sym_clear_buffer_keyword] = ACTIONS(1597),
    [sym_create_buffer_keyword] = ACTIONS(1597),
    [sym_create_multi_buffer_keyword] = ACTIONS(1597),
    [sym_call_keyword] = ACTIONS(1597),
    [sym_system_call_keyword] = ACTIONS(1597),
    [sym_devchan_to_keyword] = ACTIONS(1597),
    [sym_devchan_min_to_keyword] = ACTIONS(1597),
    [sym_devchan_total_off_keyword] = ACTIONS(1597),
    [sym_devchan_pulse_keyword] = ACTIONS(1597),
    [sym_wait_keyword] = ACTIONS(1597),
    [sym_cancel_wait_keyword] = ACTIONS(1597),
    [sym_wait_until_keyword] = ACTIONS(1597),
    [sym_cancel_wait_until_keyword] = ACTIONS(1597),
    [sym_cancel_all_wait_keyword] = ACTIONS(1597),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1597),
    [sym_and] = ACTIONS(1597),
    [sym_or] = ACTIONS(1597),
    [sym_not] = ACTIONS(1597),
    [sym_xor] = ACTIONS(1597),
    [sym_band] = ACTIONS(1597),
    [sym_bor] = ACTIONS(1597),
    [sym_bxor] = ACTIONS(1597),
    [sym_bnot] = ACTIONS(1597),
    [sym_lshift] = ACTIONS(1597),
    [sym_rshift] = ACTIONS(1597),
    [sym___file__] = ACTIONS(1597),
    [sym___date__] = ACTIONS(1597),
    [sym___ldate__] = ACTIONS(1597),
    [sym___time__] = ACTIONS(1597),
    [sym___line__] = ACTIONS(1597),
    [sym___name__] = ACTIONS(1597),
    [sym___netlinx__] = ACTIONS(1597),
    [sym_netlinx_axi_version] = ACTIONS(1597),
    [sym_url_flg_tcp] = ACTIONS(1597),
    [sym_url_flg_accinfopresent] = ACTIONS(1597),
    [sym_url_flg_temp] = ACTIONS(1597),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1597),
    [sym_url_flg_stat_mask] = ACTIONS(1597),
    [sym_url_flg_stat_lookup] = ACTIONS(1597),
    [sym_url_flg_stat_connecting] = ACTIONS(1597),
    [sym_url_flg_stat_waiting] = ACTIONS(1597),
    [sym_url_flg_stat_connected] = ACTIONS(1597),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1597),
    [sym_first_virtual_device] = ACTIONS(1597),
    [sym_first_local_port] = ACTIONS(1597),
    [sym_dynamic_virtual_device] = ACTIONS(1597),
    [sym_timeline_once] = ACTIONS(1597),
    [sym_timeline_repeat] = ACTIONS(1597),
    [sym_timeline_absolute] = ACTIONS(1597),
    [sym_timeline_relative] = ACTIONS(1597),
    [sym_do_push_timed_infinite] = ACTIONS(1597),
    [sym_source_type_no_address] = ACTIONS(1597),
    [sym_source_type_neuron_id] = ACTIONS(1597),
    [sym_source_type_ip_address] = ACTIONS(1597),
    [sym_source_type_axlink] = ACTIONS(1597),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1597),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1597),
    [sym_source_type_ip_socket_address] = ACTIONS(1597),
    [sym_source_type_rs232] = ACTIONS(1597),
    [sym_source_type_internal] = ACTIONS(1597),
    [sym_source_type_mac_address] = ACTIONS(1597),
    [sym_source_type_ipv4_port] = ACTIONS(1597),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1597),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1597),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1597),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1597),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1597),
    [sym_true] = ACTIONS(1597),
    [sym_false] = ACTIONS(1597),
    [sym_file_read_only] = ACTIONS(1597),
    [sym_file_rw_new] = ACTIONS(1597),
    [sym_file_rw_append] = ACTIONS(1597),
    [sym_ip_tcp] = ACTIONS(1597),
    [sym_ip_udp] = ACTIONS(1597),
    [sym_ip_udp_2way] = ACTIONS(1597),
    [sym_xml_encode_types] = ACTIONS(1597),
    [sym_xml_encode_char_as_list] = ACTIONS(1597),
    [sym_xml_encode_le] = ACTIONS(1597),
    [sym_xml_decode_types] = ACTIONS(1597),
    [sym_xml_decode_no_preserve] = ACTIONS(1597),
    [sym_duet_dev_type_display_device] = ACTIONS(1597),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1597),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1597),
    [sym_duet_dev_type_io_device] = ACTIONS(1597),
    [sym_duet_dev_type_relay_device] = ACTIONS(1597),
    [sym_duet_dev_type_ups] = ACTIONS(1597),
    [sym_duet_dev_type_amplifier] = ACTIONS(1597),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1597),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1597),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1597),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1597),
    [sym_duet_dev_type_camera] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1597),
    [sym_duet_dev_type_dss] = ACTIONS(1597),
    [sym_duet_dev_type_dvr] = ACTIONS(1597),
    [sym_duet_dev_type_disc_device] = ACTIONS(1597),
    [sym_duet_dev_type_document_camera] = ACTIONS(1597),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1597),
    [sym_duet_dev_type_hvac] = ACTIONS(1597),
    [sym_duet_dev_type_keypad] = ACTIONS(1597),
    [sym_duet_dev_type_light] = ACTIONS(1597),
    [sym_duet_dev_type_monitor] = ACTIONS(1597),
    [sym_duet_dev_type_motor] = ACTIONS(1597),
    [sym_duet_dev_type_multi_window] = ACTIONS(1597),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1597),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1597),
    [sym_duet_dev_type_receiver] = ACTIONS(1597),
    [sym_duet_dev_type_security_system] = ACTIONS(1597),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1597),
    [sym_duet_dev_type_settop_box] = ACTIONS(1597),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1597),
    [sym_duet_dev_type_switcher] = ACTIONS(1597),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1597),
    [sym_duet_dev_type_tv] = ACTIONS(1597),
    [sym_duet_dev_type_utility] = ACTIONS(1597),
    [sym_duet_dev_type_vcr] = ACTIONS(1597),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1597),
    [sym_duet_dev_type_video_processor] = ACTIONS(1597),
    [sym_duet_dev_type_video_projector] = ACTIONS(1597),
    [sym_duet_dev_type_video_wall] = ACTIONS(1597),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1597),
    [sym_duet_dev_type_weather] = ACTIONS(1597),
    [sym_duet_dev_not_polled] = ACTIONS(1597),
    [sym_duet_dev_polled] = ACTIONS(1597),
    [sym_clkmgr_mode_network] = ACTIONS(1597),
    [sym_clkmgr_mode_standalone] = ACTIONS(1597),
    [sym_normal_standby] = ACTIONS(1597),
    [sym_normal_wake] = ACTIONS(1597),
    [sym_ip_multicast_ttl_option] = ACTIONS(1597),
    [sym_ip_ttl_subnet] = ACTIONS(1597),
    [sym_ip_ttl_site] = ACTIONS(1597),
    [sym_ip_ttl_region] = ACTIONS(1597),
    [sym_ip_ttl_continent] = ACTIONS(1597),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1597),
    [sym_ip_nodelay_on] = ACTIONS(1597),
    [sym_ip_nodelay_off] = ACTIONS(1597),
    [sym_amx_error] = ACTIONS(1597),
    [sym_amx_warning] = ACTIONS(1597),
    [sym_amx_info] = ACTIONS(1597),
    [sym_amx_debug] = ACTIONS(1597),
    [sym_netlinx_login_success] = ACTIONS(1597),
    [sym_netlinx_login_fail] = ACTIONS(1597),
    [sym_netlinx_logout] = ACTIONS(1597),
    [sym_null_str] = ACTIONS(1597),
    [sym_smtp_address] = ACTIONS(1597),
    [sym_smtp_port_number] = ACTIONS(1597),
    [sym_smtp_username] = ACTIONS(1597),
    [sym_smtp_password] = ACTIONS(1597),
    [sym_smtp_require_tls] = ACTIONS(1597),
    [sym_smtp_from] = ACTIONS(1597),
    [sym_smtp_tls_true] = ACTIONS(1597),
    [sym_smtp_tls_false] = ACTIONS(1597),
    [sym_listview_on_row_select_event] = ACTIONS(1597),
    [sym_data_structure_unknown] = ACTIONS(1597),
    [sym_data_structure_datafeed] = ACTIONS(1597),
    [sym_data_structure_datarecordset] = ACTIONS(1597),
    [sym_data_structure_datarecord] = ACTIONS(1597),
    [sym_data_structure_datafield] = ACTIONS(1597),
    [sym_data_max_content_field_count] = ACTIONS(1597),
    [sym_data_max_metadata_field_count] = ACTIONS(1597),
    [sym_data_max_id_length] = ACTIONS(1597),
    [sym_data_max_name_length] = ACTIONS(1597),
    [sym_data_max_description_length] = ACTIONS(1597),
    [sym_data_max_type_length] = ACTIONS(1597),
    [sym_data_max_format_length] = ACTIONS(1597),
    [sym_data_max_source_length] = ACTIONS(1597),
    [sym_data_max_value_length] = ACTIONS(1597),
    [sym_data_max_label_length] = ACTIONS(1597),
    [sym_data_max_url_length] = ACTIONS(1597),
    [sym_max_last_login_info_length] = ACTIONS(1597),
    [sym_data_type_unknown] = ACTIONS(1597),
    [sym_data_type_string] = ACTIONS(1597),
    [sym_data_type_datetime] = ACTIONS(1597),
    [sym_data_type_date] = ACTIONS(1597),
    [sym_data_type_time] = ACTIONS(1597),
    [sym_data_type_image] = ACTIONS(1597),
    [sym_data_format_url] = ACTIONS(1597),
    [sym_data_format_phone] = ACTIONS(1597),
    [sym_data_format_email] = ACTIONS(1597),
    [sym_data_format_iso8601] = ACTIONS(1597),
    [sym_auditlog_priv] = ACTIONS(1597),
    [sym_configuration_priv] = ACTIONS(1597),
    [sym_deviceconfig_priv] = ACTIONS(1597),
    [sym_ftp_priv] = ACTIONS(1597),
    [sym_http_priv] = ACTIONS(1597),
    [sym_networkconfig_priv] = ACTIONS(1597),
    [sym_programport_priv] = ACTIONS(1597),
    [sym_remoteui_priv] = ACTIONS(1597),
    [sym_securitycontrol_priv] = ACTIONS(1597),
    [sym_softwaremanagement_priv] = ACTIONS(1597),
    [sym_terminal_priv] = ACTIONS(1597),
    [sym_tpadmin_priv] = ACTIONS(1597),
    [sym_user1_priv] = ACTIONS(1597),
    [sym_user2_priv] = ACTIONS(1597),
    [sym_user3_priv] = ACTIONS(1597),
    [sym_user4_priv] = ACTIONS(1597),
    [sym_usermanagement_priv] = ACTIONS(1597),
    [sym_valid_account] = ACTIONS(1597),
    [sym_err_name_invalid] = ACTIONS(1597),
    [sym_err_password_invalid] = ACTIONS(1597),
    [sym_err_invalid_account] = ACTIONS(1597),
    [sym_err_no_authorization] = ACTIONS(1597),
    [sym_err_invalid_parameter] = ACTIONS(1597),
    [sym_err_user_locked_out] = ACTIONS(1597),
    [sym_err_user_account_expired] = ACTIONS(1597),
    [sym_err_general_error] = ACTIONS(1597),
    [sym_err_authentication_error] = ACTIONS(1597),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1597),
    [sym_tls_validate_certificate] = ACTIONS(1597),
    [sym_tdata] = ACTIONS(1597),
    [sym_tchannel] = ACTIONS(1597),
    [sym_tlevel] = ACTIONS(1597),
    [sym_tbutton] = ACTIONS(1597),
    [sym_ttimeline] = ACTIONS(1597),
    [sym_tcustom] = ACTIONS(1597),
    [sym_url_struct] = ACTIONS(1597),
    [sym_dns_struct] = ACTIONS(1597),
    [sym_ip_address_struct] = ACTIONS(1597),
    [sym_dev_info_struct] = ACTIONS(1597),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1597),
    [sym_data_feed] = ACTIONS(1597),
    [sym_data_field] = ACTIONS(1597),
    [sym_data_record] = ACTIONS(1597),
    [sym_wc_data_feed] = ACTIONS(1597),
    [sym_wc_data_field] = ACTIONS(1597),
    [sym_wc_data_record] = ACTIONS(1597),
    [sym_last_login_info] = ACTIONS(1597),
    [sym____reserved___] = ACTIONS(1597),
    [sym_date] = ACTIONS(1597),
    [sym_day] = ACTIONS(1597),
    [sym_time] = ACTIONS(1597),
    [sym_ldate] = ACTIONS(1597),
    [sym_push_device] = ACTIONS(1597),
    [sym_push_channel] = ACTIONS(1597),
    [sym_push_devchan] = ACTIONS(1597),
    [sym_release_device] = ACTIONS(1597),
    [sym_release_channel] = ACTIONS(1597),
    [sym_release_devchan] = ACTIONS(1597),
    [sym_master_slot] = ACTIONS(1597),
    [sym_get_pulse_time] = ACTIONS(1597),
    [sym_get_timer] = ACTIONS(1597),
    [sym_master_sn] = ACTIONS(1597),
    [sym_system_number] = ACTIONS(1597),
    [sym_button] = ACTIONS(1597),
    [sym_dv_channel] = ACTIONS(1597),
    [sym_data] = ACTIONS(1597),
    [sym_level] = ACTIONS(1597),
    [sym_timeline] = ACTIONS(1597),
    [sym_channel] = ACTIONS(1597),
    [sym_custom] = ACTIONS(1597),
    [sym_atoi] = ACTIONS(1597),
    [sym_atol] = ACTIONS(1597),
    [sym_atof] = ACTIONS(1597),
    [sym_itoa] = ACTIONS(1597),
    [sym_itohex] = ACTIONS(1597),
    [sym_hextoi] = ACTIONS(1597),
    [sym_ftoa] = ACTIONS(1597),
    [sym_find_string] = ACTIONS(1597),
    [sym_left_string] = ACTIONS(1597),
    [sym_right_string] = ACTIONS(1597),
    [sym_mid_string] = ACTIONS(1597),
    [sym_remove_string] = ACTIONS(1597),
    [sym_get_buffer_string] = ACTIONS(1597),
    [sym_get_buffer_char] = ACTIONS(1597),
    [sym_get_multi_buffer_string] = ACTIONS(1597),
    [sym_length_string] = ACTIONS(1597),
    [sym_max_length_string] = ACTIONS(1597),
    [sym_set_length_string] = ACTIONS(1597),
    [sym_compare_string] = ACTIONS(1597),
    [sym_upper_string] = ACTIONS(1597),
    [sym_lower_string] = ACTIONS(1597),
    [sym_get_last] = ACTIONS(1597),
    [sym_date_to_day] = ACTIONS(1597),
    [sym_date_to_month] = ACTIONS(1597),
    [sym_date_to_year] = ACTIONS(1597),
    [sym_time_to_hour] = ACTIONS(1597),
    [sym_time_to_minute] = ACTIONS(1597),
    [sym_time_to_second] = ACTIONS(1597),
    [sym_day_of_week] = ACTIONS(1597),
    [sym_set_pulse_time] = ACTIONS(1597),
    [sym_set_timer] = ACTIONS(1597),
    [sym_file_open] = ACTIONS(1597),
    [sym_file_close] = ACTIONS(1597),
    [sym_file_read] = ACTIONS(1597),
    [sym_file_read_line] = ACTIONS(1597),
    [sym_file_write] = ACTIONS(1597),
    [sym_file_write_line] = ACTIONS(1597),
    [sym_file_copy] = ACTIONS(1597),
    [sym_file_delete] = ACTIONS(1597),
    [sym_file_rename] = ACTIONS(1597),
    [sym_file_dir] = ACTIONS(1597),
    [sym_file_getdir] = ACTIONS(1597),
    [sym_file_setdir] = ACTIONS(1597),
    [sym_file_create_dir] = ACTIONS(1597),
    [sym_file_remove_dir] = ACTIONS(1597),
    [sym_file_seek] = ACTIONS(1597),
    [sym_length_array] = ACTIONS(1597),
    [sym_max_length_array] = ACTIONS(1597),
    [sym_set_length_array] = ACTIONS(1597),
    [sym_ip_client_open] = ACTIONS(1597),
    [sym_ip_client_close] = ACTIONS(1597),
    [sym_ip_server_open] = ACTIONS(1597),
    [sym_ip_server_close] = ACTIONS(1597),
    [sym_ip_bound_client_open] = ACTIONS(1597),
    [sym_ip_mc_server_open] = ACTIONS(1597),
    [sym_ip_set_option] = ACTIONS(1597),
    [sym_ssh_client_open] = ACTIONS(1597),
    [sym_ssh_client_close] = ACTIONS(1597),
    [sym_tls_client_open] = ACTIONS(1597),
    [sym_tls_client_close] = ACTIONS(1597),
    [sym_do_push] = ACTIONS(1597),
    [sym_do_push_timed] = ACTIONS(1597),
    [sym_do_release] = ACTIONS(1597),
    [sym_device_id] = ACTIONS(1597),
    [sym_device_info] = ACTIONS(1597),
    [sym_device_id_string] = ACTIONS(1597),
    [sym_device_standby] = ACTIONS(1597),
    [sym_device_wake] = ACTIONS(1597),
    [sym_timeline_create] = ACTIONS(1597),
    [sym_timeline_kill] = ACTIONS(1597),
    [sym_timeline_reload] = ACTIONS(1597),
    [sym_timeline_pause] = ACTIONS(1597),
    [sym_timeline_restart] = ACTIONS(1597),
    [sym_timeline_set] = ACTIONS(1597),
    [sym_timeline_get] = ACTIONS(1597),
    [sym_timeline_active] = ACTIONS(1597),
    [sym_timeline_dynamic_id] = ACTIONS(1597),
    [sym_variable_to_string] = ACTIONS(1597),
    [sym_string_to_variable] = ACTIONS(1597),
    [sym_length_variable_to_string] = ACTIONS(1597),
    [sym_type_cast] = ACTIONS(1597),
    [sym_variable_to_xml] = ACTIONS(1597),
    [sym_xml_to_variable] = ACTIONS(1597),
    [sym_length_variable_to_xml] = ACTIONS(1597),
    [sym_redirect_string] = ACTIONS(1597),
    [sym_random_number] = ACTIONS(1597),
    [sym_get_unique_id] = ACTIONS(1597),
    [sym_get_system_number] = ACTIONS(1597),
    [sym_set_system_number] = ACTIONS(1597),
    [sym_get_serial_number] = ACTIONS(1597),
    [sym_get_master_build] = ACTIONS(1597),
    [sym_set_validation_code] = ACTIONS(1597),
    [sym_reboot] = ACTIONS(1597),
    [sym_load_duet_module] = ACTIONS(1597),
    [sym_unload_duet_module] = ACTIONS(1597),
    [sym_duet_mem_size_set] = ACTIONS(1597),
    [sym_duet_mem_size_get] = ACTIONS(1597),
    [sym_dynamic_application_device] = ACTIONS(1597),
    [sym_dynamic_polled_port] = ACTIONS(1597),
    [sym_static_port_binding] = ACTIONS(1597),
    [sym_static_ip_binding] = ACTIONS(1597),
    [sym_get_dns_list] = ACTIONS(1597),
    [sym_set_dns_list] = ACTIONS(1597),
    [sym_get_ip_address] = ACTIONS(1597),
    [sym_set_ip_address] = ACTIONS(1597),
    [sym_get_url_list] = ACTIONS(1597),
    [sym_add_url_entry] = ACTIONS(1597),
    [sym_delete_url_entry] = ACTIONS(1597),
    [sym_abs_value] = ACTIONS(1597),
    [sym_max_value] = ACTIONS(1597),
    [sym_min_value] = ACTIONS(1597),
    [sym_exp_value] = ACTIONS(1597),
    [sym_log_value] = ACTIONS(1597),
    [sym_log10_value] = ACTIONS(1597),
    [sym_power_value] = ACTIONS(1597),
    [sym_sqrt_value] = ACTIONS(1597),
    [sym_raw_be] = ACTIONS(1597),
    [sym_raw_le] = ACTIONS(1597),
    [sym_internal_queue_size_set] = ACTIONS(1597),
    [sym_internal_queue_size_get] = ACTIONS(1597),
    [sym_internal_threshold_set] = ACTIONS(1597),
    [sym_internal_threshold_get] = ACTIONS(1597),
    [sym_rebuilt_event] = ACTIONS(1597),
    [sym_do_custom_event] = ACTIONS(1597),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1597),
    [sym_clkmgr_set_clk_source] = ACTIONS(1597),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1597),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1597),
    [sym_clkmgr_get_timezone] = ACTIONS(1597),
    [sym_clkmgr_set_timezone] = ACTIONS(1597),
    [sym_clkmgr_get_resync_period] = ACTIONS(1597),
    [sym_clkmgr_set_resync_period] = ACTIONS(1597),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1597),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1597),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1597),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1597),
    [sym_clkmgr_get_timeservers] = ACTIONS(1597),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1597),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1597),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1597),
    [sym_set_log_level] = ACTIONS(1597),
    [sym_get_log_level] = ACTIONS(1597),
    [sym_amx_log] = ACTIONS(1597),
    [sym_smtp_server_config_set] = ACTIONS(1597),
    [sym_smtp_server_config_get] = ACTIONS(1597),
    [sym_smtp_send] = ACTIONS(1597),
    [sym_validate_netlinx_account] = ACTIONS(1597),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1597),
    [sym_authenticate_certificate] = ACTIONS(1597),
    [sym_audit_netlinx_session_event] = ACTIONS(1597),
    [sym_audit_netlinx_generic_event] = ACTIONS(1597),
    [sym_get_available_flash_disk_space] = ACTIONS(1597),
    [sym_get_max_flash_disk_space] = ACTIONS(1597),
    [sym_data_create_feed] = ACTIONS(1597),
    [sym_data_delete_feed] = ACTIONS(1597),
    [sym_data_publish_feed] = ACTIONS(1597),
    [sym_data_get_published_feed] = ACTIONS(1597),
    [sym_data_add_record] = ACTIONS(1597),
    [sym_data_get_event_record] = ACTIONS(1597),
    [sym__wc_data_create_feed] = ACTIONS(1597),
    [sym__wc_data_add_record] = ACTIONS(1597),
    [sym__wc_data_get_event_record] = ACTIONS(1597),
    [sym_format] = ACTIONS(1597),
    [sym_astro_clock] = ACTIONS(1597),
    [sym_set_outdoor_temperature] = ACTIONS(1597),
    [sym_set_virtual_level_count] = ACTIONS(1597),
    [sym_set_virtual_channel_count] = ACTIONS(1597),
    [sym_set_virtual_port_count] = ACTIONS(1597),
    [sym__wc_encode_int] = ACTIONS(1597),
    [sym__wc_decode_int] = ACTIONS(1597),
    [sym__wc_int] = ACTIONS(1597),
    [sym__wc_to_ch_int] = ACTIONS(1597),
    [sym__ch_to_wc_int] = ACTIONS(1597),
    [sym__wc_find_string_int] = ACTIONS(1597),
    [sym__wc_left_string_int] = ACTIONS(1597),
    [sym__wc_lower_string_int] = ACTIONS(1597),
    [sym__wc_mid_string_int] = ACTIONS(1597),
    [sym__wc_remove_string_int] = ACTIONS(1597),
    [sym__wc_right_string_int] = ACTIONS(1597),
    [sym__wc_upper_string_int] = ACTIONS(1597),
    [sym__wc_compare_string_int] = ACTIONS(1597),
    [sym__wc_get_buffer_char_int] = ACTIONS(1597),
    [sym__wc_get_buffer_string_int] = ACTIONS(1597),
    [sym__wc_concat_string_int] = ACTIONS(1597),
    [sym__wc_file_open_int] = ACTIONS(1597),
    [sym__wc_file_close_int] = ACTIONS(1597),
    [sym__wc_file_read_int] = ACTIONS(1597),
    [sym__wc_file_read_line_int] = ACTIONS(1597),
    [sym__wc_file_write_int] = ACTIONS(1597),
    [sym__wc_file_write_line_int] = ACTIONS(1597),
    [aux_sym_preproc_include_token1] = ACTIONS(1597),
    [aux_sym_preproc_define_token1] = ACTIONS(1599),
    [aux_sym_preproc_warn_token1] = ACTIONS(1599),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1599),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1599),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1599),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1599),
    [aux_sym_preproc_else_token1] = ACTIONS(1599),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1599),
    [anon_sym_COLON] = ACTIONS(1599),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_SEMI] = ACTIONS(1599),
    [anon_sym_BANG] = ACTIONS(1597),
    [anon_sym_TILDE] = ACTIONS(1599),
    [anon_sym_DASH] = ACTIONS(1597),
    [anon_sym_PLUS] = ACTIONS(1597),
    [anon_sym_STAR] = ACTIONS(1599),
    [anon_sym_SLASH] = ACTIONS(1597),
    [anon_sym_PERCENT] = ACTIONS(1599),
    [anon_sym_PIPE_PIPE] = ACTIONS(1599),
    [anon_sym_AMP_AMP] = ACTIONS(1599),
    [anon_sym_PIPE] = ACTIONS(1597),
    [anon_sym_CARET] = ACTIONS(1599),
    [anon_sym_AMP] = ACTIONS(1597),
    [anon_sym_EQ_EQ] = ACTIONS(1599),
    [anon_sym_BANG_EQ] = ACTIONS(1599),
    [anon_sym_LT_GT] = ACTIONS(1599),
    [anon_sym_GT] = ACTIONS(1597),
    [anon_sym_GT_EQ] = ACTIONS(1599),
    [anon_sym_LT_EQ] = ACTIONS(1599),
    [anon_sym_LT] = ACTIONS(1597),
    [anon_sym_LT_LT] = ACTIONS(1599),
    [anon_sym_GT_GT] = ACTIONS(1599),
    [anon_sym_DASH_DASH] = ACTIONS(1599),
    [anon_sym_PLUS_PLUS] = ACTIONS(1599),
    [anon_sym_DOT] = ACTIONS(1597),
    [anon_sym_DQUOTE] = ACTIONS(1599),
    [anon_sym_SQUOTE] = ACTIONS(1599),
    [sym_number_literal] = ACTIONS(1599),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1599),
  },
  [STATE(134)] = {
    [sym_identifier] = ACTIONS(1601),
    [sym_program_name_keyword] = ACTIONS(1601),
    [sym_module_name_keyword] = ACTIONS(1601),
    [sym_define_device_keyword] = ACTIONS(1601),
    [sym_define_combine_keyword] = ACTIONS(1601),
    [sym_define_constant_keyword] = ACTIONS(1601),
    [sym_define_type_keyword] = ACTIONS(1601),
    [sym_define_variable_keyword] = ACTIONS(1601),
    [sym_define_system_variable_keyword] = ACTIONS(1601),
    [sym_define_connect_level_keyword] = ACTIONS(1601),
    [sym_define_latching_keyword] = ACTIONS(1601),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1601),
    [sym_define_toggling_keyword] = ACTIONS(1601),
    [sym_define_call_keyword] = ACTIONS(1601),
    [sym_define_function_keyword] = ACTIONS(1601),
    [sym_define_library_function_keyword] = ACTIONS(1601),
    [sym_define_start_keyword] = ACTIONS(1601),
    [sym_define_event_keyword] = ACTIONS(1601),
    [sym_define_program_keyword] = ACTIONS(1601),
    [sym_define_module_keyword] = ACTIONS(1601),
    [sym_char_keyword] = ACTIONS(1601),
    [sym_widechar_keyword] = ACTIONS(1601),
    [sym_integer_keyword] = ACTIONS(1601),
    [sym_sinteger_keyword] = ACTIONS(1601),
    [sym_long_keyword] = ACTIONS(1601),
    [sym_slong_keyword] = ACTIONS(1601),
    [sym_float_keyword] = ACTIONS(1601),
    [sym_double_keyword] = ACTIONS(1601),
    [sym_variant_keyword] = ACTIONS(1601),
    [sym_variantarray_keyword] = ACTIONS(1601),
    [sym_local_var_keyword] = ACTIONS(1601),
    [sym_stack_var_keyword] = ACTIONS(1601),
    [sym_constant_keyword] = ACTIONS(1601),
    [sym_volatile_keyword] = ACTIONS(1601),
    [sym_non_volatile_keyword] = ACTIONS(1601),
    [sym_persistent_keyword] = ACTIONS(1601),
    [sym_dev_keyword] = ACTIONS(1601),
    [sym_devlev_keyword] = ACTIONS(1601),
    [sym_devchan_keyword] = ACTIONS(1601),
    [sym_if_keyword] = ACTIONS(1601),
    [sym_else_keyword] = ACTIONS(1601),
    [sym_switch_keyword] = ACTIONS(1601),
    [sym_case_keyword] = ACTIONS(1601),
    [sym_default_keyword] = ACTIONS(1601),
    [sym_while_keyword] = ACTIONS(1601),
    [sym_for_keyword] = ACTIONS(1601),
    [sym_break_keyword] = ACTIONS(1601),
    [sym_continue_keyword] = ACTIONS(1601),
    [sym_return_keyword] = ACTIONS(1601),
    [sym_select_keyword] = ACTIONS(1601),
    [sym_active_keyword] = ACTIONS(1601),
    [sym_struct_keyword] = ACTIONS(1601),
    [sym_structure_keyword] = ACTIONS(1601),
    [sym_button_event_keyword] = ACTIONS(1601),
    [sym_channel_event_keyword] = ACTIONS(1601),
    [sym_level_event_keyword] = ACTIONS(1601),
    [sym_data_event_keyword] = ACTIONS(1601),
    [sym_timeline_event_keyword] = ACTIONS(1601),
    [sym_custom_event_keyword] = ACTIONS(1601),
    [sym_push_keyword] = ACTIONS(1601),
    [sym_release_keyword] = ACTIONS(1601),
    [aux_sym_on_keyword_token1] = ACTIONS(1601),
    [aux_sym_off_keyword_token1] = ACTIONS(1601),
    [sym_send_level_keyword] = ACTIONS(1601),
    [sym_send_string_keyword] = ACTIONS(1601),
    [sym_send_command_keyword] = ACTIONS(1601),
    [sym_clear_buffer_keyword] = ACTIONS(1601),
    [sym_create_buffer_keyword] = ACTIONS(1601),
    [sym_create_multi_buffer_keyword] = ACTIONS(1601),
    [sym_call_keyword] = ACTIONS(1601),
    [sym_system_call_keyword] = ACTIONS(1601),
    [sym_devchan_to_keyword] = ACTIONS(1601),
    [sym_devchan_min_to_keyword] = ACTIONS(1601),
    [sym_devchan_total_off_keyword] = ACTIONS(1601),
    [sym_devchan_pulse_keyword] = ACTIONS(1601),
    [sym_wait_keyword] = ACTIONS(1601),
    [sym_cancel_wait_keyword] = ACTIONS(1601),
    [sym_wait_until_keyword] = ACTIONS(1601),
    [sym_cancel_wait_until_keyword] = ACTIONS(1601),
    [sym_cancel_all_wait_keyword] = ACTIONS(1601),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1601),
    [sym_and] = ACTIONS(1601),
    [sym_or] = ACTIONS(1601),
    [sym_not] = ACTIONS(1601),
    [sym_xor] = ACTIONS(1601),
    [sym_band] = ACTIONS(1601),
    [sym_bor] = ACTIONS(1601),
    [sym_bxor] = ACTIONS(1601),
    [sym_bnot] = ACTIONS(1601),
    [sym_lshift] = ACTIONS(1601),
    [sym_rshift] = ACTIONS(1601),
    [sym___file__] = ACTIONS(1601),
    [sym___date__] = ACTIONS(1601),
    [sym___ldate__] = ACTIONS(1601),
    [sym___time__] = ACTIONS(1601),
    [sym___line__] = ACTIONS(1601),
    [sym___name__] = ACTIONS(1601),
    [sym___netlinx__] = ACTIONS(1601),
    [sym_netlinx_axi_version] = ACTIONS(1601),
    [sym_url_flg_tcp] = ACTIONS(1601),
    [sym_url_flg_accinfopresent] = ACTIONS(1601),
    [sym_url_flg_temp] = ACTIONS(1601),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1601),
    [sym_url_flg_stat_mask] = ACTIONS(1601),
    [sym_url_flg_stat_lookup] = ACTIONS(1601),
    [sym_url_flg_stat_connecting] = ACTIONS(1601),
    [sym_url_flg_stat_waiting] = ACTIONS(1601),
    [sym_url_flg_stat_connected] = ACTIONS(1601),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1601),
    [sym_first_virtual_device] = ACTIONS(1601),
    [sym_first_local_port] = ACTIONS(1601),
    [sym_dynamic_virtual_device] = ACTIONS(1601),
    [sym_timeline_once] = ACTIONS(1601),
    [sym_timeline_repeat] = ACTIONS(1601),
    [sym_timeline_absolute] = ACTIONS(1601),
    [sym_timeline_relative] = ACTIONS(1601),
    [sym_do_push_timed_infinite] = ACTIONS(1601),
    [sym_source_type_no_address] = ACTIONS(1601),
    [sym_source_type_neuron_id] = ACTIONS(1601),
    [sym_source_type_ip_address] = ACTIONS(1601),
    [sym_source_type_axlink] = ACTIONS(1601),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1601),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1601),
    [sym_source_type_ip_socket_address] = ACTIONS(1601),
    [sym_source_type_rs232] = ACTIONS(1601),
    [sym_source_type_internal] = ACTIONS(1601),
    [sym_source_type_mac_address] = ACTIONS(1601),
    [sym_source_type_ipv4_port] = ACTIONS(1601),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1601),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1601),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1601),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1601),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1601),
    [sym_true] = ACTIONS(1601),
    [sym_false] = ACTIONS(1601),
    [sym_file_read_only] = ACTIONS(1601),
    [sym_file_rw_new] = ACTIONS(1601),
    [sym_file_rw_append] = ACTIONS(1601),
    [sym_ip_tcp] = ACTIONS(1601),
    [sym_ip_udp] = ACTIONS(1601),
    [sym_ip_udp_2way] = ACTIONS(1601),
    [sym_xml_encode_types] = ACTIONS(1601),
    [sym_xml_encode_char_as_list] = ACTIONS(1601),
    [sym_xml_encode_le] = ACTIONS(1601),
    [sym_xml_decode_types] = ACTIONS(1601),
    [sym_xml_decode_no_preserve] = ACTIONS(1601),
    [sym_duet_dev_type_display_device] = ACTIONS(1601),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1601),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1601),
    [sym_duet_dev_type_io_device] = ACTIONS(1601),
    [sym_duet_dev_type_relay_device] = ACTIONS(1601),
    [sym_duet_dev_type_ups] = ACTIONS(1601),
    [sym_duet_dev_type_amplifier] = ACTIONS(1601),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1601),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1601),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1601),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1601),
    [sym_duet_dev_type_camera] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1601),
    [sym_duet_dev_type_dss] = ACTIONS(1601),
    [sym_duet_dev_type_dvr] = ACTIONS(1601),
    [sym_duet_dev_type_disc_device] = ACTIONS(1601),
    [sym_duet_dev_type_document_camera] = ACTIONS(1601),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1601),
    [sym_duet_dev_type_hvac] = ACTIONS(1601),
    [sym_duet_dev_type_keypad] = ACTIONS(1601),
    [sym_duet_dev_type_light] = ACTIONS(1601),
    [sym_duet_dev_type_monitor] = ACTIONS(1601),
    [sym_duet_dev_type_motor] = ACTIONS(1601),
    [sym_duet_dev_type_multi_window] = ACTIONS(1601),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1601),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1601),
    [sym_duet_dev_type_receiver] = ACTIONS(1601),
    [sym_duet_dev_type_security_system] = ACTIONS(1601),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1601),
    [sym_duet_dev_type_settop_box] = ACTIONS(1601),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1601),
    [sym_duet_dev_type_switcher] = ACTIONS(1601),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1601),
    [sym_duet_dev_type_tv] = ACTIONS(1601),
    [sym_duet_dev_type_utility] = ACTIONS(1601),
    [sym_duet_dev_type_vcr] = ACTIONS(1601),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1601),
    [sym_duet_dev_type_video_processor] = ACTIONS(1601),
    [sym_duet_dev_type_video_projector] = ACTIONS(1601),
    [sym_duet_dev_type_video_wall] = ACTIONS(1601),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1601),
    [sym_duet_dev_type_weather] = ACTIONS(1601),
    [sym_duet_dev_not_polled] = ACTIONS(1601),
    [sym_duet_dev_polled] = ACTIONS(1601),
    [sym_clkmgr_mode_network] = ACTIONS(1601),
    [sym_clkmgr_mode_standalone] = ACTIONS(1601),
    [sym_normal_standby] = ACTIONS(1601),
    [sym_normal_wake] = ACTIONS(1601),
    [sym_ip_multicast_ttl_option] = ACTIONS(1601),
    [sym_ip_ttl_subnet] = ACTIONS(1601),
    [sym_ip_ttl_site] = ACTIONS(1601),
    [sym_ip_ttl_region] = ACTIONS(1601),
    [sym_ip_ttl_continent] = ACTIONS(1601),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1601),
    [sym_ip_nodelay_on] = ACTIONS(1601),
    [sym_ip_nodelay_off] = ACTIONS(1601),
    [sym_amx_error] = ACTIONS(1601),
    [sym_amx_warning] = ACTIONS(1601),
    [sym_amx_info] = ACTIONS(1601),
    [sym_amx_debug] = ACTIONS(1601),
    [sym_netlinx_login_success] = ACTIONS(1601),
    [sym_netlinx_login_fail] = ACTIONS(1601),
    [sym_netlinx_logout] = ACTIONS(1601),
    [sym_null_str] = ACTIONS(1601),
    [sym_smtp_address] = ACTIONS(1601),
    [sym_smtp_port_number] = ACTIONS(1601),
    [sym_smtp_username] = ACTIONS(1601),
    [sym_smtp_password] = ACTIONS(1601),
    [sym_smtp_require_tls] = ACTIONS(1601),
    [sym_smtp_from] = ACTIONS(1601),
    [sym_smtp_tls_true] = ACTIONS(1601),
    [sym_smtp_tls_false] = ACTIONS(1601),
    [sym_listview_on_row_select_event] = ACTIONS(1601),
    [sym_data_structure_unknown] = ACTIONS(1601),
    [sym_data_structure_datafeed] = ACTIONS(1601),
    [sym_data_structure_datarecordset] = ACTIONS(1601),
    [sym_data_structure_datarecord] = ACTIONS(1601),
    [sym_data_structure_datafield] = ACTIONS(1601),
    [sym_data_max_content_field_count] = ACTIONS(1601),
    [sym_data_max_metadata_field_count] = ACTIONS(1601),
    [sym_data_max_id_length] = ACTIONS(1601),
    [sym_data_max_name_length] = ACTIONS(1601),
    [sym_data_max_description_length] = ACTIONS(1601),
    [sym_data_max_type_length] = ACTIONS(1601),
    [sym_data_max_format_length] = ACTIONS(1601),
    [sym_data_max_source_length] = ACTIONS(1601),
    [sym_data_max_value_length] = ACTIONS(1601),
    [sym_data_max_label_length] = ACTIONS(1601),
    [sym_data_max_url_length] = ACTIONS(1601),
    [sym_max_last_login_info_length] = ACTIONS(1601),
    [sym_data_type_unknown] = ACTIONS(1601),
    [sym_data_type_string] = ACTIONS(1601),
    [sym_data_type_datetime] = ACTIONS(1601),
    [sym_data_type_date] = ACTIONS(1601),
    [sym_data_type_time] = ACTIONS(1601),
    [sym_data_type_image] = ACTIONS(1601),
    [sym_data_format_url] = ACTIONS(1601),
    [sym_data_format_phone] = ACTIONS(1601),
    [sym_data_format_email] = ACTIONS(1601),
    [sym_data_format_iso8601] = ACTIONS(1601),
    [sym_auditlog_priv] = ACTIONS(1601),
    [sym_configuration_priv] = ACTIONS(1601),
    [sym_deviceconfig_priv] = ACTIONS(1601),
    [sym_ftp_priv] = ACTIONS(1601),
    [sym_http_priv] = ACTIONS(1601),
    [sym_networkconfig_priv] = ACTIONS(1601),
    [sym_programport_priv] = ACTIONS(1601),
    [sym_remoteui_priv] = ACTIONS(1601),
    [sym_securitycontrol_priv] = ACTIONS(1601),
    [sym_softwaremanagement_priv] = ACTIONS(1601),
    [sym_terminal_priv] = ACTIONS(1601),
    [sym_tpadmin_priv] = ACTIONS(1601),
    [sym_user1_priv] = ACTIONS(1601),
    [sym_user2_priv] = ACTIONS(1601),
    [sym_user3_priv] = ACTIONS(1601),
    [sym_user4_priv] = ACTIONS(1601),
    [sym_usermanagement_priv] = ACTIONS(1601),
    [sym_valid_account] = ACTIONS(1601),
    [sym_err_name_invalid] = ACTIONS(1601),
    [sym_err_password_invalid] = ACTIONS(1601),
    [sym_err_invalid_account] = ACTIONS(1601),
    [sym_err_no_authorization] = ACTIONS(1601),
    [sym_err_invalid_parameter] = ACTIONS(1601),
    [sym_err_user_locked_out] = ACTIONS(1601),
    [sym_err_user_account_expired] = ACTIONS(1601),
    [sym_err_general_error] = ACTIONS(1601),
    [sym_err_authentication_error] = ACTIONS(1601),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1601),
    [sym_tls_validate_certificate] = ACTIONS(1601),
    [sym_tdata] = ACTIONS(1601),
    [sym_tchannel] = ACTIONS(1601),
    [sym_tlevel] = ACTIONS(1601),
    [sym_tbutton] = ACTIONS(1601),
    [sym_ttimeline] = ACTIONS(1601),
    [sym_tcustom] = ACTIONS(1601),
    [sym_url_struct] = ACTIONS(1601),
    [sym_dns_struct] = ACTIONS(1601),
    [sym_ip_address_struct] = ACTIONS(1601),
    [sym_dev_info_struct] = ACTIONS(1601),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1601),
    [sym_data_feed] = ACTIONS(1601),
    [sym_data_field] = ACTIONS(1601),
    [sym_data_record] = ACTIONS(1601),
    [sym_wc_data_feed] = ACTIONS(1601),
    [sym_wc_data_field] = ACTIONS(1601),
    [sym_wc_data_record] = ACTIONS(1601),
    [sym_last_login_info] = ACTIONS(1601),
    [sym____reserved___] = ACTIONS(1601),
    [sym_date] = ACTIONS(1601),
    [sym_day] = ACTIONS(1601),
    [sym_time] = ACTIONS(1601),
    [sym_ldate] = ACTIONS(1601),
    [sym_push_device] = ACTIONS(1601),
    [sym_push_channel] = ACTIONS(1601),
    [sym_push_devchan] = ACTIONS(1601),
    [sym_release_device] = ACTIONS(1601),
    [sym_release_channel] = ACTIONS(1601),
    [sym_release_devchan] = ACTIONS(1601),
    [sym_master_slot] = ACTIONS(1601),
    [sym_get_pulse_time] = ACTIONS(1601),
    [sym_get_timer] = ACTIONS(1601),
    [sym_master_sn] = ACTIONS(1601),
    [sym_system_number] = ACTIONS(1601),
    [sym_button] = ACTIONS(1601),
    [sym_dv_channel] = ACTIONS(1601),
    [sym_data] = ACTIONS(1601),
    [sym_level] = ACTIONS(1601),
    [sym_timeline] = ACTIONS(1601),
    [sym_channel] = ACTIONS(1601),
    [sym_custom] = ACTIONS(1601),
    [sym_atoi] = ACTIONS(1601),
    [sym_atol] = ACTIONS(1601),
    [sym_atof] = ACTIONS(1601),
    [sym_itoa] = ACTIONS(1601),
    [sym_itohex] = ACTIONS(1601),
    [sym_hextoi] = ACTIONS(1601),
    [sym_ftoa] = ACTIONS(1601),
    [sym_find_string] = ACTIONS(1601),
    [sym_left_string] = ACTIONS(1601),
    [sym_right_string] = ACTIONS(1601),
    [sym_mid_string] = ACTIONS(1601),
    [sym_remove_string] = ACTIONS(1601),
    [sym_get_buffer_string] = ACTIONS(1601),
    [sym_get_buffer_char] = ACTIONS(1601),
    [sym_get_multi_buffer_string] = ACTIONS(1601),
    [sym_length_string] = ACTIONS(1601),
    [sym_max_length_string] = ACTIONS(1601),
    [sym_set_length_string] = ACTIONS(1601),
    [sym_compare_string] = ACTIONS(1601),
    [sym_upper_string] = ACTIONS(1601),
    [sym_lower_string] = ACTIONS(1601),
    [sym_get_last] = ACTIONS(1601),
    [sym_date_to_day] = ACTIONS(1601),
    [sym_date_to_month] = ACTIONS(1601),
    [sym_date_to_year] = ACTIONS(1601),
    [sym_time_to_hour] = ACTIONS(1601),
    [sym_time_to_minute] = ACTIONS(1601),
    [sym_time_to_second] = ACTIONS(1601),
    [sym_day_of_week] = ACTIONS(1601),
    [sym_set_pulse_time] = ACTIONS(1601),
    [sym_set_timer] = ACTIONS(1601),
    [sym_file_open] = ACTIONS(1601),
    [sym_file_close] = ACTIONS(1601),
    [sym_file_read] = ACTIONS(1601),
    [sym_file_read_line] = ACTIONS(1601),
    [sym_file_write] = ACTIONS(1601),
    [sym_file_write_line] = ACTIONS(1601),
    [sym_file_copy] = ACTIONS(1601),
    [sym_file_delete] = ACTIONS(1601),
    [sym_file_rename] = ACTIONS(1601),
    [sym_file_dir] = ACTIONS(1601),
    [sym_file_getdir] = ACTIONS(1601),
    [sym_file_setdir] = ACTIONS(1601),
    [sym_file_create_dir] = ACTIONS(1601),
    [sym_file_remove_dir] = ACTIONS(1601),
    [sym_file_seek] = ACTIONS(1601),
    [sym_length_array] = ACTIONS(1601),
    [sym_max_length_array] = ACTIONS(1601),
    [sym_set_length_array] = ACTIONS(1601),
    [sym_ip_client_open] = ACTIONS(1601),
    [sym_ip_client_close] = ACTIONS(1601),
    [sym_ip_server_open] = ACTIONS(1601),
    [sym_ip_server_close] = ACTIONS(1601),
    [sym_ip_bound_client_open] = ACTIONS(1601),
    [sym_ip_mc_server_open] = ACTIONS(1601),
    [sym_ip_set_option] = ACTIONS(1601),
    [sym_ssh_client_open] = ACTIONS(1601),
    [sym_ssh_client_close] = ACTIONS(1601),
    [sym_tls_client_open] = ACTIONS(1601),
    [sym_tls_client_close] = ACTIONS(1601),
    [sym_do_push] = ACTIONS(1601),
    [sym_do_push_timed] = ACTIONS(1601),
    [sym_do_release] = ACTIONS(1601),
    [sym_device_id] = ACTIONS(1601),
    [sym_device_info] = ACTIONS(1601),
    [sym_device_id_string] = ACTIONS(1601),
    [sym_device_standby] = ACTIONS(1601),
    [sym_device_wake] = ACTIONS(1601),
    [sym_timeline_create] = ACTIONS(1601),
    [sym_timeline_kill] = ACTIONS(1601),
    [sym_timeline_reload] = ACTIONS(1601),
    [sym_timeline_pause] = ACTIONS(1601),
    [sym_timeline_restart] = ACTIONS(1601),
    [sym_timeline_set] = ACTIONS(1601),
    [sym_timeline_get] = ACTIONS(1601),
    [sym_timeline_active] = ACTIONS(1601),
    [sym_timeline_dynamic_id] = ACTIONS(1601),
    [sym_variable_to_string] = ACTIONS(1601),
    [sym_string_to_variable] = ACTIONS(1601),
    [sym_length_variable_to_string] = ACTIONS(1601),
    [sym_type_cast] = ACTIONS(1601),
    [sym_variable_to_xml] = ACTIONS(1601),
    [sym_xml_to_variable] = ACTIONS(1601),
    [sym_length_variable_to_xml] = ACTIONS(1601),
    [sym_redirect_string] = ACTIONS(1601),
    [sym_random_number] = ACTIONS(1601),
    [sym_get_unique_id] = ACTIONS(1601),
    [sym_get_system_number] = ACTIONS(1601),
    [sym_set_system_number] = ACTIONS(1601),
    [sym_get_serial_number] = ACTIONS(1601),
    [sym_get_master_build] = ACTIONS(1601),
    [sym_set_validation_code] = ACTIONS(1601),
    [sym_reboot] = ACTIONS(1601),
    [sym_load_duet_module] = ACTIONS(1601),
    [sym_unload_duet_module] = ACTIONS(1601),
    [sym_duet_mem_size_set] = ACTIONS(1601),
    [sym_duet_mem_size_get] = ACTIONS(1601),
    [sym_dynamic_application_device] = ACTIONS(1601),
    [sym_dynamic_polled_port] = ACTIONS(1601),
    [sym_static_port_binding] = ACTIONS(1601),
    [sym_static_ip_binding] = ACTIONS(1601),
    [sym_get_dns_list] = ACTIONS(1601),
    [sym_set_dns_list] = ACTIONS(1601),
    [sym_get_ip_address] = ACTIONS(1601),
    [sym_set_ip_address] = ACTIONS(1601),
    [sym_get_url_list] = ACTIONS(1601),
    [sym_add_url_entry] = ACTIONS(1601),
    [sym_delete_url_entry] = ACTIONS(1601),
    [sym_abs_value] = ACTIONS(1601),
    [sym_max_value] = ACTIONS(1601),
    [sym_min_value] = ACTIONS(1601),
    [sym_exp_value] = ACTIONS(1601),
    [sym_log_value] = ACTIONS(1601),
    [sym_log10_value] = ACTIONS(1601),
    [sym_power_value] = ACTIONS(1601),
    [sym_sqrt_value] = ACTIONS(1601),
    [sym_raw_be] = ACTIONS(1601),
    [sym_raw_le] = ACTIONS(1601),
    [sym_internal_queue_size_set] = ACTIONS(1601),
    [sym_internal_queue_size_get] = ACTIONS(1601),
    [sym_internal_threshold_set] = ACTIONS(1601),
    [sym_internal_threshold_get] = ACTIONS(1601),
    [sym_rebuilt_event] = ACTIONS(1601),
    [sym_do_custom_event] = ACTIONS(1601),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1601),
    [sym_clkmgr_set_clk_source] = ACTIONS(1601),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1601),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1601),
    [sym_clkmgr_get_timezone] = ACTIONS(1601),
    [sym_clkmgr_set_timezone] = ACTIONS(1601),
    [sym_clkmgr_get_resync_period] = ACTIONS(1601),
    [sym_clkmgr_set_resync_period] = ACTIONS(1601),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1601),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1601),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1601),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1601),
    [sym_clkmgr_get_timeservers] = ACTIONS(1601),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1601),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1601),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1601),
    [sym_set_log_level] = ACTIONS(1601),
    [sym_get_log_level] = ACTIONS(1601),
    [sym_amx_log] = ACTIONS(1601),
    [sym_smtp_server_config_set] = ACTIONS(1601),
    [sym_smtp_server_config_get] = ACTIONS(1601),
    [sym_smtp_send] = ACTIONS(1601),
    [sym_validate_netlinx_account] = ACTIONS(1601),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1601),
    [sym_authenticate_certificate] = ACTIONS(1601),
    [sym_audit_netlinx_session_event] = ACTIONS(1601),
    [sym_audit_netlinx_generic_event] = ACTIONS(1601),
    [sym_get_available_flash_disk_space] = ACTIONS(1601),
    [sym_get_max_flash_disk_space] = ACTIONS(1601),
    [sym_data_create_feed] = ACTIONS(1601),
    [sym_data_delete_feed] = ACTIONS(1601),
    [sym_data_publish_feed] = ACTIONS(1601),
    [sym_data_get_published_feed] = ACTIONS(1601),
    [sym_data_add_record] = ACTIONS(1601),
    [sym_data_get_event_record] = ACTIONS(1601),
    [sym__wc_data_create_feed] = ACTIONS(1601),
    [sym__wc_data_add_record] = ACTIONS(1601),
    [sym__wc_data_get_event_record] = ACTIONS(1601),
    [sym_format] = ACTIONS(1601),
    [sym_astro_clock] = ACTIONS(1601),
    [sym_set_outdoor_temperature] = ACTIONS(1601),
    [sym_set_virtual_level_count] = ACTIONS(1601),
    [sym_set_virtual_channel_count] = ACTIONS(1601),
    [sym_set_virtual_port_count] = ACTIONS(1601),
    [sym__wc_encode_int] = ACTIONS(1601),
    [sym__wc_decode_int] = ACTIONS(1601),
    [sym__wc_int] = ACTIONS(1601),
    [sym__wc_to_ch_int] = ACTIONS(1601),
    [sym__ch_to_wc_int] = ACTIONS(1601),
    [sym__wc_find_string_int] = ACTIONS(1601),
    [sym__wc_left_string_int] = ACTIONS(1601),
    [sym__wc_lower_string_int] = ACTIONS(1601),
    [sym__wc_mid_string_int] = ACTIONS(1601),
    [sym__wc_remove_string_int] = ACTIONS(1601),
    [sym__wc_right_string_int] = ACTIONS(1601),
    [sym__wc_upper_string_int] = ACTIONS(1601),
    [sym__wc_compare_string_int] = ACTIONS(1601),
    [sym__wc_get_buffer_char_int] = ACTIONS(1601),
    [sym__wc_get_buffer_string_int] = ACTIONS(1601),
    [sym__wc_concat_string_int] = ACTIONS(1601),
    [sym__wc_file_open_int] = ACTIONS(1601),
    [sym__wc_file_close_int] = ACTIONS(1601),
    [sym__wc_file_read_int] = ACTIONS(1601),
    [sym__wc_file_read_line_int] = ACTIONS(1601),
    [sym__wc_file_write_int] = ACTIONS(1601),
    [sym__wc_file_write_line_int] = ACTIONS(1601),
    [aux_sym_preproc_include_token1] = ACTIONS(1601),
    [aux_sym_preproc_define_token1] = ACTIONS(1603),
    [aux_sym_preproc_warn_token1] = ACTIONS(1603),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1603),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1603),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1603),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1603),
    [aux_sym_preproc_else_token1] = ACTIONS(1603),
    [anon_sym_LBRACK] = ACTIONS(1603),
    [anon_sym_LBRACE] = ACTIONS(1603),
    [anon_sym_COLON] = ACTIONS(1603),
    [anon_sym_LPAREN] = ACTIONS(1601),
    [anon_sym_SEMI] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1601),
    [anon_sym_TILDE] = ACTIONS(1603),
    [anon_sym_DASH] = ACTIONS(1601),
    [anon_sym_PLUS] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1603),
    [anon_sym_SLASH] = ACTIONS(1601),
    [anon_sym_PERCENT] = ACTIONS(1603),
    [anon_sym_PIPE_PIPE] = ACTIONS(1603),
    [anon_sym_AMP_AMP] = ACTIONS(1603),
    [anon_sym_PIPE] = ACTIONS(1601),
    [anon_sym_CARET] = ACTIONS(1603),
    [anon_sym_AMP] = ACTIONS(1601),
    [anon_sym_EQ_EQ] = ACTIONS(1603),
    [anon_sym_BANG_EQ] = ACTIONS(1603),
    [anon_sym_LT_GT] = ACTIONS(1603),
    [anon_sym_GT] = ACTIONS(1601),
    [anon_sym_GT_EQ] = ACTIONS(1603),
    [anon_sym_LT_EQ] = ACTIONS(1603),
    [anon_sym_LT] = ACTIONS(1601),
    [anon_sym_LT_LT] = ACTIONS(1603),
    [anon_sym_GT_GT] = ACTIONS(1603),
    [anon_sym_DASH_DASH] = ACTIONS(1603),
    [anon_sym_PLUS_PLUS] = ACTIONS(1603),
    [anon_sym_DOT] = ACTIONS(1601),
    [anon_sym_DQUOTE] = ACTIONS(1603),
    [anon_sym_SQUOTE] = ACTIONS(1603),
    [sym_number_literal] = ACTIONS(1603),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1603),
  },
  [STATE(135)] = {
    [ts_builtin_sym_end] = ACTIONS(1563),
    [sym_identifier] = ACTIONS(1561),
    [sym_program_name_keyword] = ACTIONS(1561),
    [sym_module_name_keyword] = ACTIONS(1561),
    [sym_define_device_keyword] = ACTIONS(1561),
    [sym_define_combine_keyword] = ACTIONS(1561),
    [sym_define_constant_keyword] = ACTIONS(1561),
    [sym_define_type_keyword] = ACTIONS(1561),
    [sym_define_variable_keyword] = ACTIONS(1561),
    [sym_define_system_variable_keyword] = ACTIONS(1561),
    [sym_define_connect_level_keyword] = ACTIONS(1561),
    [sym_define_latching_keyword] = ACTIONS(1561),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1561),
    [sym_define_toggling_keyword] = ACTIONS(1561),
    [sym_define_call_keyword] = ACTIONS(1561),
    [sym_define_function_keyword] = ACTIONS(1561),
    [sym_define_library_function_keyword] = ACTIONS(1561),
    [sym_define_start_keyword] = ACTIONS(1561),
    [sym_define_event_keyword] = ACTIONS(1561),
    [sym_define_program_keyword] = ACTIONS(1561),
    [sym_define_module_keyword] = ACTIONS(1561),
    [sym_char_keyword] = ACTIONS(1561),
    [sym_widechar_keyword] = ACTIONS(1561),
    [sym_integer_keyword] = ACTIONS(1561),
    [sym_sinteger_keyword] = ACTIONS(1561),
    [sym_long_keyword] = ACTIONS(1561),
    [sym_slong_keyword] = ACTIONS(1561),
    [sym_float_keyword] = ACTIONS(1561),
    [sym_double_keyword] = ACTIONS(1561),
    [sym_variant_keyword] = ACTIONS(1561),
    [sym_variantarray_keyword] = ACTIONS(1561),
    [sym_local_var_keyword] = ACTIONS(1561),
    [sym_stack_var_keyword] = ACTIONS(1561),
    [sym_constant_keyword] = ACTIONS(1561),
    [sym_volatile_keyword] = ACTIONS(1561),
    [sym_non_volatile_keyword] = ACTIONS(1561),
    [sym_persistent_keyword] = ACTIONS(1561),
    [sym_dev_keyword] = ACTIONS(1561),
    [sym_devlev_keyword] = ACTIONS(1561),
    [sym_devchan_keyword] = ACTIONS(1561),
    [sym_if_keyword] = ACTIONS(1561),
    [sym_else_keyword] = ACTIONS(1561),
    [sym_switch_keyword] = ACTIONS(1561),
    [sym_case_keyword] = ACTIONS(1561),
    [sym_default_keyword] = ACTIONS(1561),
    [sym_while_keyword] = ACTIONS(1561),
    [sym_for_keyword] = ACTIONS(1561),
    [sym_break_keyword] = ACTIONS(1561),
    [sym_continue_keyword] = ACTIONS(1561),
    [sym_return_keyword] = ACTIONS(1561),
    [sym_select_keyword] = ACTIONS(1561),
    [sym_active_keyword] = ACTIONS(1561),
    [sym_struct_keyword] = ACTIONS(1561),
    [sym_structure_keyword] = ACTIONS(1561),
    [sym_button_event_keyword] = ACTIONS(1561),
    [sym_channel_event_keyword] = ACTIONS(1561),
    [sym_level_event_keyword] = ACTIONS(1561),
    [sym_data_event_keyword] = ACTIONS(1561),
    [sym_timeline_event_keyword] = ACTIONS(1561),
    [sym_custom_event_keyword] = ACTIONS(1561),
    [sym_push_keyword] = ACTIONS(1561),
    [sym_release_keyword] = ACTIONS(1561),
    [aux_sym_on_keyword_token1] = ACTIONS(1561),
    [aux_sym_off_keyword_token1] = ACTIONS(1561),
    [sym_send_level_keyword] = ACTIONS(1561),
    [sym_send_string_keyword] = ACTIONS(1561),
    [sym_send_command_keyword] = ACTIONS(1561),
    [sym_clear_buffer_keyword] = ACTIONS(1561),
    [sym_create_buffer_keyword] = ACTIONS(1561),
    [sym_create_multi_buffer_keyword] = ACTIONS(1561),
    [sym_call_keyword] = ACTIONS(1561),
    [sym_system_call_keyword] = ACTIONS(1561),
    [sym_devchan_to_keyword] = ACTIONS(1561),
    [sym_devchan_min_to_keyword] = ACTIONS(1561),
    [sym_devchan_total_off_keyword] = ACTIONS(1561),
    [sym_devchan_pulse_keyword] = ACTIONS(1561),
    [sym_wait_keyword] = ACTIONS(1561),
    [sym_cancel_wait_keyword] = ACTIONS(1561),
    [sym_wait_until_keyword] = ACTIONS(1561),
    [sym_cancel_wait_until_keyword] = ACTIONS(1561),
    [sym_cancel_all_wait_keyword] = ACTIONS(1561),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1561),
    [sym_and] = ACTIONS(1561),
    [sym_or] = ACTIONS(1561),
    [sym_not] = ACTIONS(1561),
    [sym_xor] = ACTIONS(1561),
    [sym_band] = ACTIONS(1561),
    [sym_bor] = ACTIONS(1561),
    [sym_bxor] = ACTIONS(1561),
    [sym_bnot] = ACTIONS(1561),
    [sym_lshift] = ACTIONS(1561),
    [sym_rshift] = ACTIONS(1561),
    [sym___file__] = ACTIONS(1561),
    [sym___date__] = ACTIONS(1561),
    [sym___ldate__] = ACTIONS(1561),
    [sym___time__] = ACTIONS(1561),
    [sym___line__] = ACTIONS(1561),
    [sym___name__] = ACTIONS(1561),
    [sym___netlinx__] = ACTIONS(1561),
    [sym_netlinx_axi_version] = ACTIONS(1561),
    [sym_url_flg_tcp] = ACTIONS(1561),
    [sym_url_flg_accinfopresent] = ACTIONS(1561),
    [sym_url_flg_temp] = ACTIONS(1561),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1561),
    [sym_url_flg_stat_mask] = ACTIONS(1561),
    [sym_url_flg_stat_lookup] = ACTIONS(1561),
    [sym_url_flg_stat_connecting] = ACTIONS(1561),
    [sym_url_flg_stat_waiting] = ACTIONS(1561),
    [sym_url_flg_stat_connected] = ACTIONS(1561),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1561),
    [sym_first_virtual_device] = ACTIONS(1561),
    [sym_first_local_port] = ACTIONS(1561),
    [sym_dynamic_virtual_device] = ACTIONS(1561),
    [sym_timeline_once] = ACTIONS(1561),
    [sym_timeline_repeat] = ACTIONS(1561),
    [sym_timeline_absolute] = ACTIONS(1561),
    [sym_timeline_relative] = ACTIONS(1561),
    [sym_do_push_timed_infinite] = ACTIONS(1561),
    [sym_source_type_no_address] = ACTIONS(1561),
    [sym_source_type_neuron_id] = ACTIONS(1561),
    [sym_source_type_ip_address] = ACTIONS(1561),
    [sym_source_type_axlink] = ACTIONS(1561),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1561),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1561),
    [sym_source_type_ip_socket_address] = ACTIONS(1561),
    [sym_source_type_rs232] = ACTIONS(1561),
    [sym_source_type_internal] = ACTIONS(1561),
    [sym_source_type_mac_address] = ACTIONS(1561),
    [sym_source_type_ipv4_port] = ACTIONS(1561),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1561),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1561),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1561),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1561),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1561),
    [sym_true] = ACTIONS(1561),
    [sym_false] = ACTIONS(1561),
    [sym_file_read_only] = ACTIONS(1561),
    [sym_file_rw_new] = ACTIONS(1561),
    [sym_file_rw_append] = ACTIONS(1561),
    [sym_ip_tcp] = ACTIONS(1561),
    [sym_ip_udp] = ACTIONS(1561),
    [sym_ip_udp_2way] = ACTIONS(1561),
    [sym_xml_encode_types] = ACTIONS(1561),
    [sym_xml_encode_char_as_list] = ACTIONS(1561),
    [sym_xml_encode_le] = ACTIONS(1561),
    [sym_xml_decode_types] = ACTIONS(1561),
    [sym_xml_decode_no_preserve] = ACTIONS(1561),
    [sym_duet_dev_type_display_device] = ACTIONS(1561),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1561),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1561),
    [sym_duet_dev_type_io_device] = ACTIONS(1561),
    [sym_duet_dev_type_relay_device] = ACTIONS(1561),
    [sym_duet_dev_type_ups] = ACTIONS(1561),
    [sym_duet_dev_type_amplifier] = ACTIONS(1561),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1561),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1561),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1561),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1561),
    [sym_duet_dev_type_camera] = ACTIONS(1561),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1561),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1561),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1561),
    [sym_duet_dev_type_dss] = ACTIONS(1561),
    [sym_duet_dev_type_dvr] = ACTIONS(1561),
    [sym_duet_dev_type_disc_device] = ACTIONS(1561),
    [sym_duet_dev_type_document_camera] = ACTIONS(1561),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1561),
    [sym_duet_dev_type_hvac] = ACTIONS(1561),
    [sym_duet_dev_type_keypad] = ACTIONS(1561),
    [sym_duet_dev_type_light] = ACTIONS(1561),
    [sym_duet_dev_type_monitor] = ACTIONS(1561),
    [sym_duet_dev_type_motor] = ACTIONS(1561),
    [sym_duet_dev_type_multi_window] = ACTIONS(1561),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1561),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1561),
    [sym_duet_dev_type_receiver] = ACTIONS(1561),
    [sym_duet_dev_type_security_system] = ACTIONS(1561),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1561),
    [sym_duet_dev_type_settop_box] = ACTIONS(1561),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1561),
    [sym_duet_dev_type_switcher] = ACTIONS(1561),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1561),
    [sym_duet_dev_type_tv] = ACTIONS(1561),
    [sym_duet_dev_type_utility] = ACTIONS(1561),
    [sym_duet_dev_type_vcr] = ACTIONS(1561),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1561),
    [sym_duet_dev_type_video_processor] = ACTIONS(1561),
    [sym_duet_dev_type_video_projector] = ACTIONS(1561),
    [sym_duet_dev_type_video_wall] = ACTIONS(1561),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1561),
    [sym_duet_dev_type_weather] = ACTIONS(1561),
    [sym_duet_dev_not_polled] = ACTIONS(1561),
    [sym_duet_dev_polled] = ACTIONS(1561),
    [sym_clkmgr_mode_network] = ACTIONS(1561),
    [sym_clkmgr_mode_standalone] = ACTIONS(1561),
    [sym_normal_standby] = ACTIONS(1561),
    [sym_normal_wake] = ACTIONS(1561),
    [sym_ip_multicast_ttl_option] = ACTIONS(1561),
    [sym_ip_ttl_subnet] = ACTIONS(1561),
    [sym_ip_ttl_site] = ACTIONS(1561),
    [sym_ip_ttl_region] = ACTIONS(1561),
    [sym_ip_ttl_continent] = ACTIONS(1561),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1561),
    [sym_ip_nodelay_on] = ACTIONS(1561),
    [sym_ip_nodelay_off] = ACTIONS(1561),
    [sym_amx_error] = ACTIONS(1561),
    [sym_amx_warning] = ACTIONS(1561),
    [sym_amx_info] = ACTIONS(1561),
    [sym_amx_debug] = ACTIONS(1561),
    [sym_netlinx_login_success] = ACTIONS(1561),
    [sym_netlinx_login_fail] = ACTIONS(1561),
    [sym_netlinx_logout] = ACTIONS(1561),
    [sym_null_str] = ACTIONS(1561),
    [sym_smtp_address] = ACTIONS(1561),
    [sym_smtp_port_number] = ACTIONS(1561),
    [sym_smtp_username] = ACTIONS(1561),
    [sym_smtp_password] = ACTIONS(1561),
    [sym_smtp_require_tls] = ACTIONS(1561),
    [sym_smtp_from] = ACTIONS(1561),
    [sym_smtp_tls_true] = ACTIONS(1561),
    [sym_smtp_tls_false] = ACTIONS(1561),
    [sym_listview_on_row_select_event] = ACTIONS(1561),
    [sym_data_structure_unknown] = ACTIONS(1561),
    [sym_data_structure_datafeed] = ACTIONS(1561),
    [sym_data_structure_datarecordset] = ACTIONS(1561),
    [sym_data_structure_datarecord] = ACTIONS(1561),
    [sym_data_structure_datafield] = ACTIONS(1561),
    [sym_data_max_content_field_count] = ACTIONS(1561),
    [sym_data_max_metadata_field_count] = ACTIONS(1561),
    [sym_data_max_id_length] = ACTIONS(1561),
    [sym_data_max_name_length] = ACTIONS(1561),
    [sym_data_max_description_length] = ACTIONS(1561),
    [sym_data_max_type_length] = ACTIONS(1561),
    [sym_data_max_format_length] = ACTIONS(1561),
    [sym_data_max_source_length] = ACTIONS(1561),
    [sym_data_max_value_length] = ACTIONS(1561),
    [sym_data_max_label_length] = ACTIONS(1561),
    [sym_data_max_url_length] = ACTIONS(1561),
    [sym_max_last_login_info_length] = ACTIONS(1561),
    [sym_data_type_unknown] = ACTIONS(1561),
    [sym_data_type_string] = ACTIONS(1561),
    [sym_data_type_datetime] = ACTIONS(1561),
    [sym_data_type_date] = ACTIONS(1561),
    [sym_data_type_time] = ACTIONS(1561),
    [sym_data_type_image] = ACTIONS(1561),
    [sym_data_format_url] = ACTIONS(1561),
    [sym_data_format_phone] = ACTIONS(1561),
    [sym_data_format_email] = ACTIONS(1561),
    [sym_data_format_iso8601] = ACTIONS(1561),
    [sym_auditlog_priv] = ACTIONS(1561),
    [sym_configuration_priv] = ACTIONS(1561),
    [sym_deviceconfig_priv] = ACTIONS(1561),
    [sym_ftp_priv] = ACTIONS(1561),
    [sym_http_priv] = ACTIONS(1561),
    [sym_networkconfig_priv] = ACTIONS(1561),
    [sym_programport_priv] = ACTIONS(1561),
    [sym_remoteui_priv] = ACTIONS(1561),
    [sym_securitycontrol_priv] = ACTIONS(1561),
    [sym_softwaremanagement_priv] = ACTIONS(1561),
    [sym_terminal_priv] = ACTIONS(1561),
    [sym_tpadmin_priv] = ACTIONS(1561),
    [sym_user1_priv] = ACTIONS(1561),
    [sym_user2_priv] = ACTIONS(1561),
    [sym_user3_priv] = ACTIONS(1561),
    [sym_user4_priv] = ACTIONS(1561),
    [sym_usermanagement_priv] = ACTIONS(1561),
    [sym_valid_account] = ACTIONS(1561),
    [sym_err_name_invalid] = ACTIONS(1561),
    [sym_err_password_invalid] = ACTIONS(1561),
    [sym_err_invalid_account] = ACTIONS(1561),
    [sym_err_no_authorization] = ACTIONS(1561),
    [sym_err_invalid_parameter] = ACTIONS(1561),
    [sym_err_user_locked_out] = ACTIONS(1561),
    [sym_err_user_account_expired] = ACTIONS(1561),
    [sym_err_general_error] = ACTIONS(1561),
    [sym_err_authentication_error] = ACTIONS(1561),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1561),
    [sym_tls_validate_certificate] = ACTIONS(1561),
    [sym_tdata] = ACTIONS(1561),
    [sym_tchannel] = ACTIONS(1561),
    [sym_tlevel] = ACTIONS(1561),
    [sym_tbutton] = ACTIONS(1561),
    [sym_ttimeline] = ACTIONS(1561),
    [sym_tcustom] = ACTIONS(1561),
    [sym_url_struct] = ACTIONS(1561),
    [sym_dns_struct] = ACTIONS(1561),
    [sym_ip_address_struct] = ACTIONS(1561),
    [sym_dev_info_struct] = ACTIONS(1561),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1561),
    [sym_data_feed] = ACTIONS(1561),
    [sym_data_field] = ACTIONS(1561),
    [sym_data_record] = ACTIONS(1561),
    [sym_wc_data_feed] = ACTIONS(1561),
    [sym_wc_data_field] = ACTIONS(1561),
    [sym_wc_data_record] = ACTIONS(1561),
    [sym_last_login_info] = ACTIONS(1561),
    [sym____reserved___] = ACTIONS(1561),
    [sym_date] = ACTIONS(1561),
    [sym_day] = ACTIONS(1561),
    [sym_time] = ACTIONS(1561),
    [sym_ldate] = ACTIONS(1561),
    [sym_push_device] = ACTIONS(1561),
    [sym_push_channel] = ACTIONS(1561),
    [sym_push_devchan] = ACTIONS(1561),
    [sym_release_device] = ACTIONS(1561),
    [sym_release_channel] = ACTIONS(1561),
    [sym_release_devchan] = ACTIONS(1561),
    [sym_master_slot] = ACTIONS(1561),
    [sym_get_pulse_time] = ACTIONS(1561),
    [sym_get_timer] = ACTIONS(1561),
    [sym_master_sn] = ACTIONS(1561),
    [sym_system_number] = ACTIONS(1561),
    [sym_button] = ACTIONS(1561),
    [sym_dv_channel] = ACTIONS(1561),
    [sym_data] = ACTIONS(1561),
    [sym_level] = ACTIONS(1561),
    [sym_timeline] = ACTIONS(1561),
    [sym_channel] = ACTIONS(1561),
    [sym_custom] = ACTIONS(1561),
    [sym_atoi] = ACTIONS(1561),
    [sym_atol] = ACTIONS(1561),
    [sym_atof] = ACTIONS(1561),
    [sym_itoa] = ACTIONS(1561),
    [sym_itohex] = ACTIONS(1561),
    [sym_hextoi] = ACTIONS(1561),
    [sym_ftoa] = ACTIONS(1561),
    [sym_find_string] = ACTIONS(1561),
    [sym_left_string] = ACTIONS(1561),
    [sym_right_string] = ACTIONS(1561),
    [sym_mid_string] = ACTIONS(1561),
    [sym_remove_string] = ACTIONS(1561),
    [sym_get_buffer_string] = ACTIONS(1561),
    [sym_get_buffer_char] = ACTIONS(1561),
    [sym_get_multi_buffer_string] = ACTIONS(1561),
    [sym_length_string] = ACTIONS(1561),
    [sym_max_length_string] = ACTIONS(1561),
    [sym_set_length_string] = ACTIONS(1561),
    [sym_compare_string] = ACTIONS(1561),
    [sym_upper_string] = ACTIONS(1561),
    [sym_lower_string] = ACTIONS(1561),
    [sym_get_last] = ACTIONS(1561),
    [sym_date_to_day] = ACTIONS(1561),
    [sym_date_to_month] = ACTIONS(1561),
    [sym_date_to_year] = ACTIONS(1561),
    [sym_time_to_hour] = ACTIONS(1561),
    [sym_time_to_minute] = ACTIONS(1561),
    [sym_time_to_second] = ACTIONS(1561),
    [sym_day_of_week] = ACTIONS(1561),
    [sym_set_pulse_time] = ACTIONS(1561),
    [sym_set_timer] = ACTIONS(1561),
    [sym_file_open] = ACTIONS(1561),
    [sym_file_close] = ACTIONS(1561),
    [sym_file_read] = ACTIONS(1561),
    [sym_file_read_line] = ACTIONS(1561),
    [sym_file_write] = ACTIONS(1561),
    [sym_file_write_line] = ACTIONS(1561),
    [sym_file_copy] = ACTIONS(1561),
    [sym_file_delete] = ACTIONS(1561),
    [sym_file_rename] = ACTIONS(1561),
    [sym_file_dir] = ACTIONS(1561),
    [sym_file_getdir] = ACTIONS(1561),
    [sym_file_setdir] = ACTIONS(1561),
    [sym_file_create_dir] = ACTIONS(1561),
    [sym_file_remove_dir] = ACTIONS(1561),
    [sym_file_seek] = ACTIONS(1561),
    [sym_length_array] = ACTIONS(1561),
    [sym_max_length_array] = ACTIONS(1561),
    [sym_set_length_array] = ACTIONS(1561),
    [sym_ip_client_open] = ACTIONS(1561),
    [sym_ip_client_close] = ACTIONS(1561),
    [sym_ip_server_open] = ACTIONS(1561),
    [sym_ip_server_close] = ACTIONS(1561),
    [sym_ip_bound_client_open] = ACTIONS(1561),
    [sym_ip_mc_server_open] = ACTIONS(1561),
    [sym_ip_set_option] = ACTIONS(1561),
    [sym_ssh_client_open] = ACTIONS(1561),
    [sym_ssh_client_close] = ACTIONS(1561),
    [sym_tls_client_open] = ACTIONS(1561),
    [sym_tls_client_close] = ACTIONS(1561),
    [sym_do_push] = ACTIONS(1561),
    [sym_do_push_timed] = ACTIONS(1561),
    [sym_do_release] = ACTIONS(1561),
    [sym_device_id] = ACTIONS(1561),
    [sym_device_info] = ACTIONS(1561),
    [sym_device_id_string] = ACTIONS(1561),
    [sym_device_standby] = ACTIONS(1561),
    [sym_device_wake] = ACTIONS(1561),
    [sym_timeline_create] = ACTIONS(1561),
    [sym_timeline_kill] = ACTIONS(1561),
    [sym_timeline_reload] = ACTIONS(1561),
    [sym_timeline_pause] = ACTIONS(1561),
    [sym_timeline_restart] = ACTIONS(1561),
    [sym_timeline_set] = ACTIONS(1561),
    [sym_timeline_get] = ACTIONS(1561),
    [sym_timeline_active] = ACTIONS(1561),
    [sym_timeline_dynamic_id] = ACTIONS(1561),
    [sym_variable_to_string] = ACTIONS(1561),
    [sym_string_to_variable] = ACTIONS(1561),
    [sym_length_variable_to_string] = ACTIONS(1561),
    [sym_type_cast] = ACTIONS(1561),
    [sym_variable_to_xml] = ACTIONS(1561),
    [sym_xml_to_variable] = ACTIONS(1561),
    [sym_length_variable_to_xml] = ACTIONS(1561),
    [sym_redirect_string] = ACTIONS(1561),
    [sym_random_number] = ACTIONS(1561),
    [sym_get_unique_id] = ACTIONS(1561),
    [sym_get_system_number] = ACTIONS(1561),
    [sym_set_system_number] = ACTIONS(1561),
    [sym_get_serial_number] = ACTIONS(1561),
    [sym_get_master_build] = ACTIONS(1561),
    [sym_set_validation_code] = ACTIONS(1561),
    [sym_reboot] = ACTIONS(1561),
    [sym_load_duet_module] = ACTIONS(1561),
    [sym_unload_duet_module] = ACTIONS(1561),
    [sym_duet_mem_size_set] = ACTIONS(1561),
    [sym_duet_mem_size_get] = ACTIONS(1561),
    [sym_dynamic_application_device] = ACTIONS(1561),
    [sym_dynamic_polled_port] = ACTIONS(1561),
    [sym_static_port_binding] = ACTIONS(1561),
    [sym_static_ip_binding] = ACTIONS(1561),
    [sym_get_dns_list] = ACTIONS(1561),
    [sym_set_dns_list] = ACTIONS(1561),
    [sym_get_ip_address] = ACTIONS(1561),
    [sym_set_ip_address] = ACTIONS(1561),
    [sym_get_url_list] = ACTIONS(1561),
    [sym_add_url_entry] = ACTIONS(1561),
    [sym_delete_url_entry] = ACTIONS(1561),
    [sym_abs_value] = ACTIONS(1561),
    [sym_max_value] = ACTIONS(1561),
    [sym_min_value] = ACTIONS(1561),
    [sym_exp_value] = ACTIONS(1561),
    [sym_log_value] = ACTIONS(1561),
    [sym_log10_value] = ACTIONS(1561),
    [sym_power_value] = ACTIONS(1561),
    [sym_sqrt_value] = ACTIONS(1561),
    [sym_raw_be] = ACTIONS(1561),
    [sym_raw_le] = ACTIONS(1561),
    [sym_internal_queue_size_set] = ACTIONS(1561),
    [sym_internal_queue_size_get] = ACTIONS(1561),
    [sym_internal_threshold_set] = ACTIONS(1561),
    [sym_internal_threshold_get] = ACTIONS(1561),
    [sym_rebuilt_event] = ACTIONS(1561),
    [sym_do_custom_event] = ACTIONS(1561),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1561),
    [sym_clkmgr_set_clk_source] = ACTIONS(1561),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1561),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1561),
    [sym_clkmgr_get_timezone] = ACTIONS(1561),
    [sym_clkmgr_set_timezone] = ACTIONS(1561),
    [sym_clkmgr_get_resync_period] = ACTIONS(1561),
    [sym_clkmgr_set_resync_period] = ACTIONS(1561),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1561),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1561),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1561),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1561),
    [sym_clkmgr_get_timeservers] = ACTIONS(1561),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1561),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1561),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1561),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1561),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1561),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1561),
    [sym_set_log_level] = ACTIONS(1561),
    [sym_get_log_level] = ACTIONS(1561),
    [sym_amx_log] = ACTIONS(1561),
    [sym_smtp_server_config_set] = ACTIONS(1561),
    [sym_smtp_server_config_get] = ACTIONS(1561),
    [sym_smtp_send] = ACTIONS(1561),
    [sym_validate_netlinx_account] = ACTIONS(1561),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1561),
    [sym_authenticate_certificate] = ACTIONS(1561),
    [sym_audit_netlinx_session_event] = ACTIONS(1561),
    [sym_audit_netlinx_generic_event] = ACTIONS(1561),
    [sym_get_available_flash_disk_space] = ACTIONS(1561),
    [sym_get_max_flash_disk_space] = ACTIONS(1561),
    [sym_data_create_feed] = ACTIONS(1561),
    [sym_data_delete_feed] = ACTIONS(1561),
    [sym_data_publish_feed] = ACTIONS(1561),
    [sym_data_get_published_feed] = ACTIONS(1561),
    [sym_data_add_record] = ACTIONS(1561),
    [sym_data_get_event_record] = ACTIONS(1561),
    [sym__wc_data_create_feed] = ACTIONS(1561),
    [sym__wc_data_add_record] = ACTIONS(1561),
    [sym__wc_data_get_event_record] = ACTIONS(1561),
    [sym_format] = ACTIONS(1561),
    [sym_astro_clock] = ACTIONS(1561),
    [sym_set_outdoor_temperature] = ACTIONS(1561),
    [sym_set_virtual_level_count] = ACTIONS(1561),
    [sym_set_virtual_channel_count] = ACTIONS(1561),
    [sym_set_virtual_port_count] = ACTIONS(1561),
    [sym__wc_encode_int] = ACTIONS(1561),
    [sym__wc_decode_int] = ACTIONS(1561),
    [sym__wc_int] = ACTIONS(1561),
    [sym__wc_to_ch_int] = ACTIONS(1561),
    [sym__ch_to_wc_int] = ACTIONS(1561),
    [sym__wc_find_string_int] = ACTIONS(1561),
    [sym__wc_left_string_int] = ACTIONS(1561),
    [sym__wc_lower_string_int] = ACTIONS(1561),
    [sym__wc_mid_string_int] = ACTIONS(1561),
    [sym__wc_remove_string_int] = ACTIONS(1561),
    [sym__wc_right_string_int] = ACTIONS(1561),
    [sym__wc_upper_string_int] = ACTIONS(1561),
    [sym__wc_compare_string_int] = ACTIONS(1561),
    [sym__wc_get_buffer_char_int] = ACTIONS(1561),
    [sym__wc_get_buffer_string_int] = ACTIONS(1561),
    [sym__wc_concat_string_int] = ACTIONS(1561),
    [sym__wc_file_open_int] = ACTIONS(1561),
    [sym__wc_file_close_int] = ACTIONS(1561),
    [sym__wc_file_read_int] = ACTIONS(1561),
    [sym__wc_file_read_line_int] = ACTIONS(1561),
    [sym__wc_file_write_int] = ACTIONS(1561),
    [sym__wc_file_write_line_int] = ACTIONS(1561),
    [aux_sym_preproc_include_token1] = ACTIONS(1561),
    [aux_sym_preproc_define_token1] = ACTIONS(1563),
    [aux_sym_preproc_warn_token1] = ACTIONS(1563),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1563),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1563),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1563),
    [anon_sym_LBRACK] = ACTIONS(1563),
    [anon_sym_LBRACE] = ACTIONS(1563),
    [anon_sym_COLON] = ACTIONS(1563),
    [anon_sym_LPAREN] = ACTIONS(1561),
    [anon_sym_SEMI] = ACTIONS(1563),
    [anon_sym_BANG] = ACTIONS(1561),
    [anon_sym_TILDE] = ACTIONS(1563),
    [anon_sym_DASH] = ACTIONS(1561),
    [anon_sym_PLUS] = ACTIONS(1561),
    [anon_sym_STAR] = ACTIONS(1563),
    [anon_sym_SLASH] = ACTIONS(1561),
    [anon_sym_PERCENT] = ACTIONS(1563),
    [anon_sym_PIPE_PIPE] = ACTIONS(1563),
    [anon_sym_AMP_AMP] = ACTIONS(1563),
    [anon_sym_PIPE] = ACTIONS(1561),
    [anon_sym_CARET] = ACTIONS(1563),
    [anon_sym_AMP] = ACTIONS(1561),
    [anon_sym_EQ_EQ] = ACTIONS(1563),
    [anon_sym_BANG_EQ] = ACTIONS(1563),
    [anon_sym_LT_GT] = ACTIONS(1563),
    [anon_sym_GT] = ACTIONS(1561),
    [anon_sym_GT_EQ] = ACTIONS(1563),
    [anon_sym_LT_EQ] = ACTIONS(1563),
    [anon_sym_LT] = ACTIONS(1561),
    [anon_sym_LT_LT] = ACTIONS(1563),
    [anon_sym_GT_GT] = ACTIONS(1563),
    [anon_sym_DASH_DASH] = ACTIONS(1563),
    [anon_sym_PLUS_PLUS] = ACTIONS(1563),
    [anon_sym_DOT] = ACTIONS(1561),
    [anon_sym_DQUOTE] = ACTIONS(1563),
    [anon_sym_SQUOTE] = ACTIONS(1563),
    [sym_number_literal] = ACTIONS(1563),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1563),
  },
  [STATE(136)] = {
    [ts_builtin_sym_end] = ACTIONS(1587),
    [sym_identifier] = ACTIONS(1585),
    [sym_program_name_keyword] = ACTIONS(1585),
    [sym_module_name_keyword] = ACTIONS(1585),
    [sym_define_device_keyword] = ACTIONS(1585),
    [sym_define_combine_keyword] = ACTIONS(1585),
    [sym_define_constant_keyword] = ACTIONS(1585),
    [sym_define_type_keyword] = ACTIONS(1585),
    [sym_define_variable_keyword] = ACTIONS(1585),
    [sym_define_system_variable_keyword] = ACTIONS(1585),
    [sym_define_connect_level_keyword] = ACTIONS(1585),
    [sym_define_latching_keyword] = ACTIONS(1585),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1585),
    [sym_define_toggling_keyword] = ACTIONS(1585),
    [sym_define_call_keyword] = ACTIONS(1585),
    [sym_define_function_keyword] = ACTIONS(1585),
    [sym_define_library_function_keyword] = ACTIONS(1585),
    [sym_define_start_keyword] = ACTIONS(1585),
    [sym_define_event_keyword] = ACTIONS(1585),
    [sym_define_program_keyword] = ACTIONS(1585),
    [sym_define_module_keyword] = ACTIONS(1585),
    [sym_char_keyword] = ACTIONS(1585),
    [sym_widechar_keyword] = ACTIONS(1585),
    [sym_integer_keyword] = ACTIONS(1585),
    [sym_sinteger_keyword] = ACTIONS(1585),
    [sym_long_keyword] = ACTIONS(1585),
    [sym_slong_keyword] = ACTIONS(1585),
    [sym_float_keyword] = ACTIONS(1585),
    [sym_double_keyword] = ACTIONS(1585),
    [sym_variant_keyword] = ACTIONS(1585),
    [sym_variantarray_keyword] = ACTIONS(1585),
    [sym_local_var_keyword] = ACTIONS(1585),
    [sym_stack_var_keyword] = ACTIONS(1585),
    [sym_constant_keyword] = ACTIONS(1585),
    [sym_volatile_keyword] = ACTIONS(1585),
    [sym_non_volatile_keyword] = ACTIONS(1585),
    [sym_persistent_keyword] = ACTIONS(1585),
    [sym_dev_keyword] = ACTIONS(1585),
    [sym_devlev_keyword] = ACTIONS(1585),
    [sym_devchan_keyword] = ACTIONS(1585),
    [sym_if_keyword] = ACTIONS(1585),
    [sym_else_keyword] = ACTIONS(1585),
    [sym_switch_keyword] = ACTIONS(1585),
    [sym_case_keyword] = ACTIONS(1585),
    [sym_default_keyword] = ACTIONS(1585),
    [sym_while_keyword] = ACTIONS(1585),
    [sym_for_keyword] = ACTIONS(1585),
    [sym_break_keyword] = ACTIONS(1585),
    [sym_continue_keyword] = ACTIONS(1585),
    [sym_return_keyword] = ACTIONS(1585),
    [sym_select_keyword] = ACTIONS(1585),
    [sym_active_keyword] = ACTIONS(1585),
    [sym_struct_keyword] = ACTIONS(1585),
    [sym_structure_keyword] = ACTIONS(1585),
    [sym_button_event_keyword] = ACTIONS(1585),
    [sym_channel_event_keyword] = ACTIONS(1585),
    [sym_level_event_keyword] = ACTIONS(1585),
    [sym_data_event_keyword] = ACTIONS(1585),
    [sym_timeline_event_keyword] = ACTIONS(1585),
    [sym_custom_event_keyword] = ACTIONS(1585),
    [sym_push_keyword] = ACTIONS(1585),
    [sym_release_keyword] = ACTIONS(1585),
    [aux_sym_on_keyword_token1] = ACTIONS(1585),
    [aux_sym_off_keyword_token1] = ACTIONS(1585),
    [sym_send_level_keyword] = ACTIONS(1585),
    [sym_send_string_keyword] = ACTIONS(1585),
    [sym_send_command_keyword] = ACTIONS(1585),
    [sym_clear_buffer_keyword] = ACTIONS(1585),
    [sym_create_buffer_keyword] = ACTIONS(1585),
    [sym_create_multi_buffer_keyword] = ACTIONS(1585),
    [sym_call_keyword] = ACTIONS(1585),
    [sym_system_call_keyword] = ACTIONS(1585),
    [sym_devchan_to_keyword] = ACTIONS(1585),
    [sym_devchan_min_to_keyword] = ACTIONS(1585),
    [sym_devchan_total_off_keyword] = ACTIONS(1585),
    [sym_devchan_pulse_keyword] = ACTIONS(1585),
    [sym_wait_keyword] = ACTIONS(1585),
    [sym_cancel_wait_keyword] = ACTIONS(1585),
    [sym_wait_until_keyword] = ACTIONS(1585),
    [sym_cancel_wait_until_keyword] = ACTIONS(1585),
    [sym_cancel_all_wait_keyword] = ACTIONS(1585),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1585),
    [sym_and] = ACTIONS(1585),
    [sym_or] = ACTIONS(1585),
    [sym_not] = ACTIONS(1585),
    [sym_xor] = ACTIONS(1585),
    [sym_band] = ACTIONS(1585),
    [sym_bor] = ACTIONS(1585),
    [sym_bxor] = ACTIONS(1585),
    [sym_bnot] = ACTIONS(1585),
    [sym_lshift] = ACTIONS(1585),
    [sym_rshift] = ACTIONS(1585),
    [sym___file__] = ACTIONS(1585),
    [sym___date__] = ACTIONS(1585),
    [sym___ldate__] = ACTIONS(1585),
    [sym___time__] = ACTIONS(1585),
    [sym___line__] = ACTIONS(1585),
    [sym___name__] = ACTIONS(1585),
    [sym___netlinx__] = ACTIONS(1585),
    [sym_netlinx_axi_version] = ACTIONS(1585),
    [sym_url_flg_tcp] = ACTIONS(1585),
    [sym_url_flg_accinfopresent] = ACTIONS(1585),
    [sym_url_flg_temp] = ACTIONS(1585),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1585),
    [sym_url_flg_stat_mask] = ACTIONS(1585),
    [sym_url_flg_stat_lookup] = ACTIONS(1585),
    [sym_url_flg_stat_connecting] = ACTIONS(1585),
    [sym_url_flg_stat_waiting] = ACTIONS(1585),
    [sym_url_flg_stat_connected] = ACTIONS(1585),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1585),
    [sym_first_virtual_device] = ACTIONS(1585),
    [sym_first_local_port] = ACTIONS(1585),
    [sym_dynamic_virtual_device] = ACTIONS(1585),
    [sym_timeline_once] = ACTIONS(1585),
    [sym_timeline_repeat] = ACTIONS(1585),
    [sym_timeline_absolute] = ACTIONS(1585),
    [sym_timeline_relative] = ACTIONS(1585),
    [sym_do_push_timed_infinite] = ACTIONS(1585),
    [sym_source_type_no_address] = ACTIONS(1585),
    [sym_source_type_neuron_id] = ACTIONS(1585),
    [sym_source_type_ip_address] = ACTIONS(1585),
    [sym_source_type_axlink] = ACTIONS(1585),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1585),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1585),
    [sym_source_type_ip_socket_address] = ACTIONS(1585),
    [sym_source_type_rs232] = ACTIONS(1585),
    [sym_source_type_internal] = ACTIONS(1585),
    [sym_source_type_mac_address] = ACTIONS(1585),
    [sym_source_type_ipv4_port] = ACTIONS(1585),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1585),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1585),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1585),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1585),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1585),
    [sym_true] = ACTIONS(1585),
    [sym_false] = ACTIONS(1585),
    [sym_file_read_only] = ACTIONS(1585),
    [sym_file_rw_new] = ACTIONS(1585),
    [sym_file_rw_append] = ACTIONS(1585),
    [sym_ip_tcp] = ACTIONS(1585),
    [sym_ip_udp] = ACTIONS(1585),
    [sym_ip_udp_2way] = ACTIONS(1585),
    [sym_xml_encode_types] = ACTIONS(1585),
    [sym_xml_encode_char_as_list] = ACTIONS(1585),
    [sym_xml_encode_le] = ACTIONS(1585),
    [sym_xml_decode_types] = ACTIONS(1585),
    [sym_xml_decode_no_preserve] = ACTIONS(1585),
    [sym_duet_dev_type_display_device] = ACTIONS(1585),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1585),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1585),
    [sym_duet_dev_type_io_device] = ACTIONS(1585),
    [sym_duet_dev_type_relay_device] = ACTIONS(1585),
    [sym_duet_dev_type_ups] = ACTIONS(1585),
    [sym_duet_dev_type_amplifier] = ACTIONS(1585),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1585),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1585),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1585),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1585),
    [sym_duet_dev_type_camera] = ACTIONS(1585),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1585),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1585),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1585),
    [sym_duet_dev_type_dss] = ACTIONS(1585),
    [sym_duet_dev_type_dvr] = ACTIONS(1585),
    [sym_duet_dev_type_disc_device] = ACTIONS(1585),
    [sym_duet_dev_type_document_camera] = ACTIONS(1585),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1585),
    [sym_duet_dev_type_hvac] = ACTIONS(1585),
    [sym_duet_dev_type_keypad] = ACTIONS(1585),
    [sym_duet_dev_type_light] = ACTIONS(1585),
    [sym_duet_dev_type_monitor] = ACTIONS(1585),
    [sym_duet_dev_type_motor] = ACTIONS(1585),
    [sym_duet_dev_type_multi_window] = ACTIONS(1585),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1585),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1585),
    [sym_duet_dev_type_receiver] = ACTIONS(1585),
    [sym_duet_dev_type_security_system] = ACTIONS(1585),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1585),
    [sym_duet_dev_type_settop_box] = ACTIONS(1585),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1585),
    [sym_duet_dev_type_switcher] = ACTIONS(1585),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1585),
    [sym_duet_dev_type_tv] = ACTIONS(1585),
    [sym_duet_dev_type_utility] = ACTIONS(1585),
    [sym_duet_dev_type_vcr] = ACTIONS(1585),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1585),
    [sym_duet_dev_type_video_processor] = ACTIONS(1585),
    [sym_duet_dev_type_video_projector] = ACTIONS(1585),
    [sym_duet_dev_type_video_wall] = ACTIONS(1585),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1585),
    [sym_duet_dev_type_weather] = ACTIONS(1585),
    [sym_duet_dev_not_polled] = ACTIONS(1585),
    [sym_duet_dev_polled] = ACTIONS(1585),
    [sym_clkmgr_mode_network] = ACTIONS(1585),
    [sym_clkmgr_mode_standalone] = ACTIONS(1585),
    [sym_normal_standby] = ACTIONS(1585),
    [sym_normal_wake] = ACTIONS(1585),
    [sym_ip_multicast_ttl_option] = ACTIONS(1585),
    [sym_ip_ttl_subnet] = ACTIONS(1585),
    [sym_ip_ttl_site] = ACTIONS(1585),
    [sym_ip_ttl_region] = ACTIONS(1585),
    [sym_ip_ttl_continent] = ACTIONS(1585),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1585),
    [sym_ip_nodelay_on] = ACTIONS(1585),
    [sym_ip_nodelay_off] = ACTIONS(1585),
    [sym_amx_error] = ACTIONS(1585),
    [sym_amx_warning] = ACTIONS(1585),
    [sym_amx_info] = ACTIONS(1585),
    [sym_amx_debug] = ACTIONS(1585),
    [sym_netlinx_login_success] = ACTIONS(1585),
    [sym_netlinx_login_fail] = ACTIONS(1585),
    [sym_netlinx_logout] = ACTIONS(1585),
    [sym_null_str] = ACTIONS(1585),
    [sym_smtp_address] = ACTIONS(1585),
    [sym_smtp_port_number] = ACTIONS(1585),
    [sym_smtp_username] = ACTIONS(1585),
    [sym_smtp_password] = ACTIONS(1585),
    [sym_smtp_require_tls] = ACTIONS(1585),
    [sym_smtp_from] = ACTIONS(1585),
    [sym_smtp_tls_true] = ACTIONS(1585),
    [sym_smtp_tls_false] = ACTIONS(1585),
    [sym_listview_on_row_select_event] = ACTIONS(1585),
    [sym_data_structure_unknown] = ACTIONS(1585),
    [sym_data_structure_datafeed] = ACTIONS(1585),
    [sym_data_structure_datarecordset] = ACTIONS(1585),
    [sym_data_structure_datarecord] = ACTIONS(1585),
    [sym_data_structure_datafield] = ACTIONS(1585),
    [sym_data_max_content_field_count] = ACTIONS(1585),
    [sym_data_max_metadata_field_count] = ACTIONS(1585),
    [sym_data_max_id_length] = ACTIONS(1585),
    [sym_data_max_name_length] = ACTIONS(1585),
    [sym_data_max_description_length] = ACTIONS(1585),
    [sym_data_max_type_length] = ACTIONS(1585),
    [sym_data_max_format_length] = ACTIONS(1585),
    [sym_data_max_source_length] = ACTIONS(1585),
    [sym_data_max_value_length] = ACTIONS(1585),
    [sym_data_max_label_length] = ACTIONS(1585),
    [sym_data_max_url_length] = ACTIONS(1585),
    [sym_max_last_login_info_length] = ACTIONS(1585),
    [sym_data_type_unknown] = ACTIONS(1585),
    [sym_data_type_string] = ACTIONS(1585),
    [sym_data_type_datetime] = ACTIONS(1585),
    [sym_data_type_date] = ACTIONS(1585),
    [sym_data_type_time] = ACTIONS(1585),
    [sym_data_type_image] = ACTIONS(1585),
    [sym_data_format_url] = ACTIONS(1585),
    [sym_data_format_phone] = ACTIONS(1585),
    [sym_data_format_email] = ACTIONS(1585),
    [sym_data_format_iso8601] = ACTIONS(1585),
    [sym_auditlog_priv] = ACTIONS(1585),
    [sym_configuration_priv] = ACTIONS(1585),
    [sym_deviceconfig_priv] = ACTIONS(1585),
    [sym_ftp_priv] = ACTIONS(1585),
    [sym_http_priv] = ACTIONS(1585),
    [sym_networkconfig_priv] = ACTIONS(1585),
    [sym_programport_priv] = ACTIONS(1585),
    [sym_remoteui_priv] = ACTIONS(1585),
    [sym_securitycontrol_priv] = ACTIONS(1585),
    [sym_softwaremanagement_priv] = ACTIONS(1585),
    [sym_terminal_priv] = ACTIONS(1585),
    [sym_tpadmin_priv] = ACTIONS(1585),
    [sym_user1_priv] = ACTIONS(1585),
    [sym_user2_priv] = ACTIONS(1585),
    [sym_user3_priv] = ACTIONS(1585),
    [sym_user4_priv] = ACTIONS(1585),
    [sym_usermanagement_priv] = ACTIONS(1585),
    [sym_valid_account] = ACTIONS(1585),
    [sym_err_name_invalid] = ACTIONS(1585),
    [sym_err_password_invalid] = ACTIONS(1585),
    [sym_err_invalid_account] = ACTIONS(1585),
    [sym_err_no_authorization] = ACTIONS(1585),
    [sym_err_invalid_parameter] = ACTIONS(1585),
    [sym_err_user_locked_out] = ACTIONS(1585),
    [sym_err_user_account_expired] = ACTIONS(1585),
    [sym_err_general_error] = ACTIONS(1585),
    [sym_err_authentication_error] = ACTIONS(1585),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1585),
    [sym_tls_validate_certificate] = ACTIONS(1585),
    [sym_tdata] = ACTIONS(1585),
    [sym_tchannel] = ACTIONS(1585),
    [sym_tlevel] = ACTIONS(1585),
    [sym_tbutton] = ACTIONS(1585),
    [sym_ttimeline] = ACTIONS(1585),
    [sym_tcustom] = ACTIONS(1585),
    [sym_url_struct] = ACTIONS(1585),
    [sym_dns_struct] = ACTIONS(1585),
    [sym_ip_address_struct] = ACTIONS(1585),
    [sym_dev_info_struct] = ACTIONS(1585),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1585),
    [sym_data_feed] = ACTIONS(1585),
    [sym_data_field] = ACTIONS(1585),
    [sym_data_record] = ACTIONS(1585),
    [sym_wc_data_feed] = ACTIONS(1585),
    [sym_wc_data_field] = ACTIONS(1585),
    [sym_wc_data_record] = ACTIONS(1585),
    [sym_last_login_info] = ACTIONS(1585),
    [sym____reserved___] = ACTIONS(1585),
    [sym_date] = ACTIONS(1585),
    [sym_day] = ACTIONS(1585),
    [sym_time] = ACTIONS(1585),
    [sym_ldate] = ACTIONS(1585),
    [sym_push_device] = ACTIONS(1585),
    [sym_push_channel] = ACTIONS(1585),
    [sym_push_devchan] = ACTIONS(1585),
    [sym_release_device] = ACTIONS(1585),
    [sym_release_channel] = ACTIONS(1585),
    [sym_release_devchan] = ACTIONS(1585),
    [sym_master_slot] = ACTIONS(1585),
    [sym_get_pulse_time] = ACTIONS(1585),
    [sym_get_timer] = ACTIONS(1585),
    [sym_master_sn] = ACTIONS(1585),
    [sym_system_number] = ACTIONS(1585),
    [sym_button] = ACTIONS(1585),
    [sym_dv_channel] = ACTIONS(1585),
    [sym_data] = ACTIONS(1585),
    [sym_level] = ACTIONS(1585),
    [sym_timeline] = ACTIONS(1585),
    [sym_channel] = ACTIONS(1585),
    [sym_custom] = ACTIONS(1585),
    [sym_atoi] = ACTIONS(1585),
    [sym_atol] = ACTIONS(1585),
    [sym_atof] = ACTIONS(1585),
    [sym_itoa] = ACTIONS(1585),
    [sym_itohex] = ACTIONS(1585),
    [sym_hextoi] = ACTIONS(1585),
    [sym_ftoa] = ACTIONS(1585),
    [sym_find_string] = ACTIONS(1585),
    [sym_left_string] = ACTIONS(1585),
    [sym_right_string] = ACTIONS(1585),
    [sym_mid_string] = ACTIONS(1585),
    [sym_remove_string] = ACTIONS(1585),
    [sym_get_buffer_string] = ACTIONS(1585),
    [sym_get_buffer_char] = ACTIONS(1585),
    [sym_get_multi_buffer_string] = ACTIONS(1585),
    [sym_length_string] = ACTIONS(1585),
    [sym_max_length_string] = ACTIONS(1585),
    [sym_set_length_string] = ACTIONS(1585),
    [sym_compare_string] = ACTIONS(1585),
    [sym_upper_string] = ACTIONS(1585),
    [sym_lower_string] = ACTIONS(1585),
    [sym_get_last] = ACTIONS(1585),
    [sym_date_to_day] = ACTIONS(1585),
    [sym_date_to_month] = ACTIONS(1585),
    [sym_date_to_year] = ACTIONS(1585),
    [sym_time_to_hour] = ACTIONS(1585),
    [sym_time_to_minute] = ACTIONS(1585),
    [sym_time_to_second] = ACTIONS(1585),
    [sym_day_of_week] = ACTIONS(1585),
    [sym_set_pulse_time] = ACTIONS(1585),
    [sym_set_timer] = ACTIONS(1585),
    [sym_file_open] = ACTIONS(1585),
    [sym_file_close] = ACTIONS(1585),
    [sym_file_read] = ACTIONS(1585),
    [sym_file_read_line] = ACTIONS(1585),
    [sym_file_write] = ACTIONS(1585),
    [sym_file_write_line] = ACTIONS(1585),
    [sym_file_copy] = ACTIONS(1585),
    [sym_file_delete] = ACTIONS(1585),
    [sym_file_rename] = ACTIONS(1585),
    [sym_file_dir] = ACTIONS(1585),
    [sym_file_getdir] = ACTIONS(1585),
    [sym_file_setdir] = ACTIONS(1585),
    [sym_file_create_dir] = ACTIONS(1585),
    [sym_file_remove_dir] = ACTIONS(1585),
    [sym_file_seek] = ACTIONS(1585),
    [sym_length_array] = ACTIONS(1585),
    [sym_max_length_array] = ACTIONS(1585),
    [sym_set_length_array] = ACTIONS(1585),
    [sym_ip_client_open] = ACTIONS(1585),
    [sym_ip_client_close] = ACTIONS(1585),
    [sym_ip_server_open] = ACTIONS(1585),
    [sym_ip_server_close] = ACTIONS(1585),
    [sym_ip_bound_client_open] = ACTIONS(1585),
    [sym_ip_mc_server_open] = ACTIONS(1585),
    [sym_ip_set_option] = ACTIONS(1585),
    [sym_ssh_client_open] = ACTIONS(1585),
    [sym_ssh_client_close] = ACTIONS(1585),
    [sym_tls_client_open] = ACTIONS(1585),
    [sym_tls_client_close] = ACTIONS(1585),
    [sym_do_push] = ACTIONS(1585),
    [sym_do_push_timed] = ACTIONS(1585),
    [sym_do_release] = ACTIONS(1585),
    [sym_device_id] = ACTIONS(1585),
    [sym_device_info] = ACTIONS(1585),
    [sym_device_id_string] = ACTIONS(1585),
    [sym_device_standby] = ACTIONS(1585),
    [sym_device_wake] = ACTIONS(1585),
    [sym_timeline_create] = ACTIONS(1585),
    [sym_timeline_kill] = ACTIONS(1585),
    [sym_timeline_reload] = ACTIONS(1585),
    [sym_timeline_pause] = ACTIONS(1585),
    [sym_timeline_restart] = ACTIONS(1585),
    [sym_timeline_set] = ACTIONS(1585),
    [sym_timeline_get] = ACTIONS(1585),
    [sym_timeline_active] = ACTIONS(1585),
    [sym_timeline_dynamic_id] = ACTIONS(1585),
    [sym_variable_to_string] = ACTIONS(1585),
    [sym_string_to_variable] = ACTIONS(1585),
    [sym_length_variable_to_string] = ACTIONS(1585),
    [sym_type_cast] = ACTIONS(1585),
    [sym_variable_to_xml] = ACTIONS(1585),
    [sym_xml_to_variable] = ACTIONS(1585),
    [sym_length_variable_to_xml] = ACTIONS(1585),
    [sym_redirect_string] = ACTIONS(1585),
    [sym_random_number] = ACTIONS(1585),
    [sym_get_unique_id] = ACTIONS(1585),
    [sym_get_system_number] = ACTIONS(1585),
    [sym_set_system_number] = ACTIONS(1585),
    [sym_get_serial_number] = ACTIONS(1585),
    [sym_get_master_build] = ACTIONS(1585),
    [sym_set_validation_code] = ACTIONS(1585),
    [sym_reboot] = ACTIONS(1585),
    [sym_load_duet_module] = ACTIONS(1585),
    [sym_unload_duet_module] = ACTIONS(1585),
    [sym_duet_mem_size_set] = ACTIONS(1585),
    [sym_duet_mem_size_get] = ACTIONS(1585),
    [sym_dynamic_application_device] = ACTIONS(1585),
    [sym_dynamic_polled_port] = ACTIONS(1585),
    [sym_static_port_binding] = ACTIONS(1585),
    [sym_static_ip_binding] = ACTIONS(1585),
    [sym_get_dns_list] = ACTIONS(1585),
    [sym_set_dns_list] = ACTIONS(1585),
    [sym_get_ip_address] = ACTIONS(1585),
    [sym_set_ip_address] = ACTIONS(1585),
    [sym_get_url_list] = ACTIONS(1585),
    [sym_add_url_entry] = ACTIONS(1585),
    [sym_delete_url_entry] = ACTIONS(1585),
    [sym_abs_value] = ACTIONS(1585),
    [sym_max_value] = ACTIONS(1585),
    [sym_min_value] = ACTIONS(1585),
    [sym_exp_value] = ACTIONS(1585),
    [sym_log_value] = ACTIONS(1585),
    [sym_log10_value] = ACTIONS(1585),
    [sym_power_value] = ACTIONS(1585),
    [sym_sqrt_value] = ACTIONS(1585),
    [sym_raw_be] = ACTIONS(1585),
    [sym_raw_le] = ACTIONS(1585),
    [sym_internal_queue_size_set] = ACTIONS(1585),
    [sym_internal_queue_size_get] = ACTIONS(1585),
    [sym_internal_threshold_set] = ACTIONS(1585),
    [sym_internal_threshold_get] = ACTIONS(1585),
    [sym_rebuilt_event] = ACTIONS(1585),
    [sym_do_custom_event] = ACTIONS(1585),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1585),
    [sym_clkmgr_set_clk_source] = ACTIONS(1585),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1585),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1585),
    [sym_clkmgr_get_timezone] = ACTIONS(1585),
    [sym_clkmgr_set_timezone] = ACTIONS(1585),
    [sym_clkmgr_get_resync_period] = ACTIONS(1585),
    [sym_clkmgr_set_resync_period] = ACTIONS(1585),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1585),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1585),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1585),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1585),
    [sym_clkmgr_get_timeservers] = ACTIONS(1585),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1585),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1585),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1585),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1585),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1585),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1585),
    [sym_set_log_level] = ACTIONS(1585),
    [sym_get_log_level] = ACTIONS(1585),
    [sym_amx_log] = ACTIONS(1585),
    [sym_smtp_server_config_set] = ACTIONS(1585),
    [sym_smtp_server_config_get] = ACTIONS(1585),
    [sym_smtp_send] = ACTIONS(1585),
    [sym_validate_netlinx_account] = ACTIONS(1585),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1585),
    [sym_authenticate_certificate] = ACTIONS(1585),
    [sym_audit_netlinx_session_event] = ACTIONS(1585),
    [sym_audit_netlinx_generic_event] = ACTIONS(1585),
    [sym_get_available_flash_disk_space] = ACTIONS(1585),
    [sym_get_max_flash_disk_space] = ACTIONS(1585),
    [sym_data_create_feed] = ACTIONS(1585),
    [sym_data_delete_feed] = ACTIONS(1585),
    [sym_data_publish_feed] = ACTIONS(1585),
    [sym_data_get_published_feed] = ACTIONS(1585),
    [sym_data_add_record] = ACTIONS(1585),
    [sym_data_get_event_record] = ACTIONS(1585),
    [sym__wc_data_create_feed] = ACTIONS(1585),
    [sym__wc_data_add_record] = ACTIONS(1585),
    [sym__wc_data_get_event_record] = ACTIONS(1585),
    [sym_format] = ACTIONS(1585),
    [sym_astro_clock] = ACTIONS(1585),
    [sym_set_outdoor_temperature] = ACTIONS(1585),
    [sym_set_virtual_level_count] = ACTIONS(1585),
    [sym_set_virtual_channel_count] = ACTIONS(1585),
    [sym_set_virtual_port_count] = ACTIONS(1585),
    [sym__wc_encode_int] = ACTIONS(1585),
    [sym__wc_decode_int] = ACTIONS(1585),
    [sym__wc_int] = ACTIONS(1585),
    [sym__wc_to_ch_int] = ACTIONS(1585),
    [sym__ch_to_wc_int] = ACTIONS(1585),
    [sym__wc_find_string_int] = ACTIONS(1585),
    [sym__wc_left_string_int] = ACTIONS(1585),
    [sym__wc_lower_string_int] = ACTIONS(1585),
    [sym__wc_mid_string_int] = ACTIONS(1585),
    [sym__wc_remove_string_int] = ACTIONS(1585),
    [sym__wc_right_string_int] = ACTIONS(1585),
    [sym__wc_upper_string_int] = ACTIONS(1585),
    [sym__wc_compare_string_int] = ACTIONS(1585),
    [sym__wc_get_buffer_char_int] = ACTIONS(1585),
    [sym__wc_get_buffer_string_int] = ACTIONS(1585),
    [sym__wc_concat_string_int] = ACTIONS(1585),
    [sym__wc_file_open_int] = ACTIONS(1585),
    [sym__wc_file_close_int] = ACTIONS(1585),
    [sym__wc_file_read_int] = ACTIONS(1585),
    [sym__wc_file_read_line_int] = ACTIONS(1585),
    [sym__wc_file_write_int] = ACTIONS(1585),
    [sym__wc_file_write_line_int] = ACTIONS(1585),
    [aux_sym_preproc_include_token1] = ACTIONS(1585),
    [aux_sym_preproc_define_token1] = ACTIONS(1587),
    [aux_sym_preproc_warn_token1] = ACTIONS(1587),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1587),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1587),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1587),
    [anon_sym_LBRACK] = ACTIONS(1587),
    [anon_sym_LBRACE] = ACTIONS(1587),
    [anon_sym_COLON] = ACTIONS(1587),
    [anon_sym_LPAREN] = ACTIONS(1585),
    [anon_sym_SEMI] = ACTIONS(1587),
    [anon_sym_BANG] = ACTIONS(1585),
    [anon_sym_TILDE] = ACTIONS(1587),
    [anon_sym_DASH] = ACTIONS(1585),
    [anon_sym_PLUS] = ACTIONS(1585),
    [anon_sym_STAR] = ACTIONS(1587),
    [anon_sym_SLASH] = ACTIONS(1585),
    [anon_sym_PERCENT] = ACTIONS(1587),
    [anon_sym_PIPE_PIPE] = ACTIONS(1587),
    [anon_sym_AMP_AMP] = ACTIONS(1587),
    [anon_sym_PIPE] = ACTIONS(1585),
    [anon_sym_CARET] = ACTIONS(1587),
    [anon_sym_AMP] = ACTIONS(1585),
    [anon_sym_EQ_EQ] = ACTIONS(1587),
    [anon_sym_BANG_EQ] = ACTIONS(1587),
    [anon_sym_LT_GT] = ACTIONS(1587),
    [anon_sym_GT] = ACTIONS(1585),
    [anon_sym_GT_EQ] = ACTIONS(1587),
    [anon_sym_LT_EQ] = ACTIONS(1587),
    [anon_sym_LT] = ACTIONS(1585),
    [anon_sym_LT_LT] = ACTIONS(1587),
    [anon_sym_GT_GT] = ACTIONS(1587),
    [anon_sym_DASH_DASH] = ACTIONS(1587),
    [anon_sym_PLUS_PLUS] = ACTIONS(1587),
    [anon_sym_DOT] = ACTIONS(1585),
    [anon_sym_DQUOTE] = ACTIONS(1587),
    [anon_sym_SQUOTE] = ACTIONS(1587),
    [sym_number_literal] = ACTIONS(1587),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1587),
  },
  [STATE(137)] = {
    [ts_builtin_sym_end] = ACTIONS(1494),
    [sym_identifier] = ACTIONS(1492),
    [sym_program_name_keyword] = ACTIONS(1492),
    [sym_module_name_keyword] = ACTIONS(1492),
    [sym_define_device_keyword] = ACTIONS(1492),
    [sym_define_combine_keyword] = ACTIONS(1492),
    [sym_define_constant_keyword] = ACTIONS(1492),
    [sym_define_type_keyword] = ACTIONS(1492),
    [sym_define_variable_keyword] = ACTIONS(1492),
    [sym_define_system_variable_keyword] = ACTIONS(1492),
    [sym_define_connect_level_keyword] = ACTIONS(1492),
    [sym_define_latching_keyword] = ACTIONS(1492),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1492),
    [sym_define_toggling_keyword] = ACTIONS(1492),
    [sym_define_call_keyword] = ACTIONS(1492),
    [sym_define_function_keyword] = ACTIONS(1492),
    [sym_define_library_function_keyword] = ACTIONS(1492),
    [sym_define_start_keyword] = ACTIONS(1492),
    [sym_define_event_keyword] = ACTIONS(1492),
    [sym_define_program_keyword] = ACTIONS(1492),
    [sym_define_module_keyword] = ACTIONS(1492),
    [sym_char_keyword] = ACTIONS(1492),
    [sym_widechar_keyword] = ACTIONS(1492),
    [sym_integer_keyword] = ACTIONS(1492),
    [sym_sinteger_keyword] = ACTIONS(1492),
    [sym_long_keyword] = ACTIONS(1492),
    [sym_slong_keyword] = ACTIONS(1492),
    [sym_float_keyword] = ACTIONS(1492),
    [sym_double_keyword] = ACTIONS(1492),
    [sym_variant_keyword] = ACTIONS(1492),
    [sym_variantarray_keyword] = ACTIONS(1492),
    [sym_local_var_keyword] = ACTIONS(1492),
    [sym_stack_var_keyword] = ACTIONS(1492),
    [sym_constant_keyword] = ACTIONS(1492),
    [sym_volatile_keyword] = ACTIONS(1492),
    [sym_non_volatile_keyword] = ACTIONS(1492),
    [sym_persistent_keyword] = ACTIONS(1492),
    [sym_dev_keyword] = ACTIONS(1492),
    [sym_devlev_keyword] = ACTIONS(1492),
    [sym_devchan_keyword] = ACTIONS(1492),
    [sym_if_keyword] = ACTIONS(1492),
    [sym_else_keyword] = ACTIONS(1492),
    [sym_switch_keyword] = ACTIONS(1492),
    [sym_case_keyword] = ACTIONS(1492),
    [sym_default_keyword] = ACTIONS(1492),
    [sym_while_keyword] = ACTIONS(1492),
    [sym_for_keyword] = ACTIONS(1492),
    [sym_break_keyword] = ACTIONS(1492),
    [sym_continue_keyword] = ACTIONS(1492),
    [sym_return_keyword] = ACTIONS(1492),
    [sym_select_keyword] = ACTIONS(1492),
    [sym_active_keyword] = ACTIONS(1492),
    [sym_struct_keyword] = ACTIONS(1492),
    [sym_structure_keyword] = ACTIONS(1492),
    [sym_button_event_keyword] = ACTIONS(1492),
    [sym_channel_event_keyword] = ACTIONS(1492),
    [sym_level_event_keyword] = ACTIONS(1492),
    [sym_data_event_keyword] = ACTIONS(1492),
    [sym_timeline_event_keyword] = ACTIONS(1492),
    [sym_custom_event_keyword] = ACTIONS(1492),
    [sym_push_keyword] = ACTIONS(1492),
    [sym_release_keyword] = ACTIONS(1492),
    [aux_sym_on_keyword_token1] = ACTIONS(1492),
    [aux_sym_off_keyword_token1] = ACTIONS(1492),
    [sym_send_level_keyword] = ACTIONS(1492),
    [sym_send_string_keyword] = ACTIONS(1492),
    [sym_send_command_keyword] = ACTIONS(1492),
    [sym_clear_buffer_keyword] = ACTIONS(1492),
    [sym_create_buffer_keyword] = ACTIONS(1492),
    [sym_create_multi_buffer_keyword] = ACTIONS(1492),
    [sym_call_keyword] = ACTIONS(1492),
    [sym_system_call_keyword] = ACTIONS(1492),
    [sym_devchan_to_keyword] = ACTIONS(1492),
    [sym_devchan_min_to_keyword] = ACTIONS(1492),
    [sym_devchan_total_off_keyword] = ACTIONS(1492),
    [sym_devchan_pulse_keyword] = ACTIONS(1492),
    [sym_wait_keyword] = ACTIONS(1492),
    [sym_cancel_wait_keyword] = ACTIONS(1492),
    [sym_wait_until_keyword] = ACTIONS(1492),
    [sym_cancel_wait_until_keyword] = ACTIONS(1492),
    [sym_cancel_all_wait_keyword] = ACTIONS(1492),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1492),
    [sym_and] = ACTIONS(1492),
    [sym_or] = ACTIONS(1492),
    [sym_not] = ACTIONS(1492),
    [sym_xor] = ACTIONS(1492),
    [sym_band] = ACTIONS(1492),
    [sym_bor] = ACTIONS(1492),
    [sym_bxor] = ACTIONS(1492),
    [sym_bnot] = ACTIONS(1492),
    [sym_lshift] = ACTIONS(1492),
    [sym_rshift] = ACTIONS(1492),
    [sym___file__] = ACTIONS(1492),
    [sym___date__] = ACTIONS(1492),
    [sym___ldate__] = ACTIONS(1492),
    [sym___time__] = ACTIONS(1492),
    [sym___line__] = ACTIONS(1492),
    [sym___name__] = ACTIONS(1492),
    [sym___netlinx__] = ACTIONS(1492),
    [sym_netlinx_axi_version] = ACTIONS(1492),
    [sym_url_flg_tcp] = ACTIONS(1492),
    [sym_url_flg_accinfopresent] = ACTIONS(1492),
    [sym_url_flg_temp] = ACTIONS(1492),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1492),
    [sym_url_flg_stat_mask] = ACTIONS(1492),
    [sym_url_flg_stat_lookup] = ACTIONS(1492),
    [sym_url_flg_stat_connecting] = ACTIONS(1492),
    [sym_url_flg_stat_waiting] = ACTIONS(1492),
    [sym_url_flg_stat_connected] = ACTIONS(1492),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1492),
    [sym_first_virtual_device] = ACTIONS(1492),
    [sym_first_local_port] = ACTIONS(1492),
    [sym_dynamic_virtual_device] = ACTIONS(1492),
    [sym_timeline_once] = ACTIONS(1492),
    [sym_timeline_repeat] = ACTIONS(1492),
    [sym_timeline_absolute] = ACTIONS(1492),
    [sym_timeline_relative] = ACTIONS(1492),
    [sym_do_push_timed_infinite] = ACTIONS(1492),
    [sym_source_type_no_address] = ACTIONS(1492),
    [sym_source_type_neuron_id] = ACTIONS(1492),
    [sym_source_type_ip_address] = ACTIONS(1492),
    [sym_source_type_axlink] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1492),
    [sym_source_type_ip_socket_address] = ACTIONS(1492),
    [sym_source_type_rs232] = ACTIONS(1492),
    [sym_source_type_internal] = ACTIONS(1492),
    [sym_source_type_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1492),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1492),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1492),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1492),
    [sym_true] = ACTIONS(1492),
    [sym_false] = ACTIONS(1492),
    [sym_file_read_only] = ACTIONS(1492),
    [sym_file_rw_new] = ACTIONS(1492),
    [sym_file_rw_append] = ACTIONS(1492),
    [sym_ip_tcp] = ACTIONS(1492),
    [sym_ip_udp] = ACTIONS(1492),
    [sym_ip_udp_2way] = ACTIONS(1492),
    [sym_xml_encode_types] = ACTIONS(1492),
    [sym_xml_encode_char_as_list] = ACTIONS(1492),
    [sym_xml_encode_le] = ACTIONS(1492),
    [sym_xml_decode_types] = ACTIONS(1492),
    [sym_xml_decode_no_preserve] = ACTIONS(1492),
    [sym_duet_dev_type_display_device] = ACTIONS(1492),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_io_device] = ACTIONS(1492),
    [sym_duet_dev_type_relay_device] = ACTIONS(1492),
    [sym_duet_dev_type_ups] = ACTIONS(1492),
    [sym_duet_dev_type_amplifier] = ACTIONS(1492),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1492),
    [sym_duet_dev_type_camera] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1492),
    [sym_duet_dev_type_dss] = ACTIONS(1492),
    [sym_duet_dev_type_dvr] = ACTIONS(1492),
    [sym_duet_dev_type_disc_device] = ACTIONS(1492),
    [sym_duet_dev_type_document_camera] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1492),
    [sym_duet_dev_type_hvac] = ACTIONS(1492),
    [sym_duet_dev_type_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_light] = ACTIONS(1492),
    [sym_duet_dev_type_monitor] = ACTIONS(1492),
    [sym_duet_dev_type_motor] = ACTIONS(1492),
    [sym_duet_dev_type_multi_window] = ACTIONS(1492),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1492),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1492),
    [sym_duet_dev_type_receiver] = ACTIONS(1492),
    [sym_duet_dev_type_security_system] = ACTIONS(1492),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1492),
    [sym_duet_dev_type_settop_box] = ACTIONS(1492),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1492),
    [sym_duet_dev_type_switcher] = ACTIONS(1492),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_tv] = ACTIONS(1492),
    [sym_duet_dev_type_utility] = ACTIONS(1492),
    [sym_duet_dev_type_vcr] = ACTIONS(1492),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_video_processor] = ACTIONS(1492),
    [sym_duet_dev_type_video_projector] = ACTIONS(1492),
    [sym_duet_dev_type_video_wall] = ACTIONS(1492),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1492),
    [sym_duet_dev_type_weather] = ACTIONS(1492),
    [sym_duet_dev_not_polled] = ACTIONS(1492),
    [sym_duet_dev_polled] = ACTIONS(1492),
    [sym_clkmgr_mode_network] = ACTIONS(1492),
    [sym_clkmgr_mode_standalone] = ACTIONS(1492),
    [sym_normal_standby] = ACTIONS(1492),
    [sym_normal_wake] = ACTIONS(1492),
    [sym_ip_multicast_ttl_option] = ACTIONS(1492),
    [sym_ip_ttl_subnet] = ACTIONS(1492),
    [sym_ip_ttl_site] = ACTIONS(1492),
    [sym_ip_ttl_region] = ACTIONS(1492),
    [sym_ip_ttl_continent] = ACTIONS(1492),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1492),
    [sym_ip_nodelay_on] = ACTIONS(1492),
    [sym_ip_nodelay_off] = ACTIONS(1492),
    [sym_amx_error] = ACTIONS(1492),
    [sym_amx_warning] = ACTIONS(1492),
    [sym_amx_info] = ACTIONS(1492),
    [sym_amx_debug] = ACTIONS(1492),
    [sym_netlinx_login_success] = ACTIONS(1492),
    [sym_netlinx_login_fail] = ACTIONS(1492),
    [sym_netlinx_logout] = ACTIONS(1492),
    [sym_null_str] = ACTIONS(1492),
    [sym_smtp_address] = ACTIONS(1492),
    [sym_smtp_port_number] = ACTIONS(1492),
    [sym_smtp_username] = ACTIONS(1492),
    [sym_smtp_password] = ACTIONS(1492),
    [sym_smtp_require_tls] = ACTIONS(1492),
    [sym_smtp_from] = ACTIONS(1492),
    [sym_smtp_tls_true] = ACTIONS(1492),
    [sym_smtp_tls_false] = ACTIONS(1492),
    [sym_listview_on_row_select_event] = ACTIONS(1492),
    [sym_data_structure_unknown] = ACTIONS(1492),
    [sym_data_structure_datafeed] = ACTIONS(1492),
    [sym_data_structure_datarecordset] = ACTIONS(1492),
    [sym_data_structure_datarecord] = ACTIONS(1492),
    [sym_data_structure_datafield] = ACTIONS(1492),
    [sym_data_max_content_field_count] = ACTIONS(1492),
    [sym_data_max_metadata_field_count] = ACTIONS(1492),
    [sym_data_max_id_length] = ACTIONS(1492),
    [sym_data_max_name_length] = ACTIONS(1492),
    [sym_data_max_description_length] = ACTIONS(1492),
    [sym_data_max_type_length] = ACTIONS(1492),
    [sym_data_max_format_length] = ACTIONS(1492),
    [sym_data_max_source_length] = ACTIONS(1492),
    [sym_data_max_value_length] = ACTIONS(1492),
    [sym_data_max_label_length] = ACTIONS(1492),
    [sym_data_max_url_length] = ACTIONS(1492),
    [sym_max_last_login_info_length] = ACTIONS(1492),
    [sym_data_type_unknown] = ACTIONS(1492),
    [sym_data_type_string] = ACTIONS(1492),
    [sym_data_type_datetime] = ACTIONS(1492),
    [sym_data_type_date] = ACTIONS(1492),
    [sym_data_type_time] = ACTIONS(1492),
    [sym_data_type_image] = ACTIONS(1492),
    [sym_data_format_url] = ACTIONS(1492),
    [sym_data_format_phone] = ACTIONS(1492),
    [sym_data_format_email] = ACTIONS(1492),
    [sym_data_format_iso8601] = ACTIONS(1492),
    [sym_auditlog_priv] = ACTIONS(1492),
    [sym_configuration_priv] = ACTIONS(1492),
    [sym_deviceconfig_priv] = ACTIONS(1492),
    [sym_ftp_priv] = ACTIONS(1492),
    [sym_http_priv] = ACTIONS(1492),
    [sym_networkconfig_priv] = ACTIONS(1492),
    [sym_programport_priv] = ACTIONS(1492),
    [sym_remoteui_priv] = ACTIONS(1492),
    [sym_securitycontrol_priv] = ACTIONS(1492),
    [sym_softwaremanagement_priv] = ACTIONS(1492),
    [sym_terminal_priv] = ACTIONS(1492),
    [sym_tpadmin_priv] = ACTIONS(1492),
    [sym_user1_priv] = ACTIONS(1492),
    [sym_user2_priv] = ACTIONS(1492),
    [sym_user3_priv] = ACTIONS(1492),
    [sym_user4_priv] = ACTIONS(1492),
    [sym_usermanagement_priv] = ACTIONS(1492),
    [sym_valid_account] = ACTIONS(1492),
    [sym_err_name_invalid] = ACTIONS(1492),
    [sym_err_password_invalid] = ACTIONS(1492),
    [sym_err_invalid_account] = ACTIONS(1492),
    [sym_err_no_authorization] = ACTIONS(1492),
    [sym_err_invalid_parameter] = ACTIONS(1492),
    [sym_err_user_locked_out] = ACTIONS(1492),
    [sym_err_user_account_expired] = ACTIONS(1492),
    [sym_err_general_error] = ACTIONS(1492),
    [sym_err_authentication_error] = ACTIONS(1492),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1492),
    [sym_tls_validate_certificate] = ACTIONS(1492),
    [sym_tdata] = ACTIONS(1492),
    [sym_tchannel] = ACTIONS(1492),
    [sym_tlevel] = ACTIONS(1492),
    [sym_tbutton] = ACTIONS(1492),
    [sym_ttimeline] = ACTIONS(1492),
    [sym_tcustom] = ACTIONS(1492),
    [sym_url_struct] = ACTIONS(1492),
    [sym_dns_struct] = ACTIONS(1492),
    [sym_ip_address_struct] = ACTIONS(1492),
    [sym_dev_info_struct] = ACTIONS(1492),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1492),
    [sym_data_feed] = ACTIONS(1492),
    [sym_data_field] = ACTIONS(1492),
    [sym_data_record] = ACTIONS(1492),
    [sym_wc_data_feed] = ACTIONS(1492),
    [sym_wc_data_field] = ACTIONS(1492),
    [sym_wc_data_record] = ACTIONS(1492),
    [sym_last_login_info] = ACTIONS(1492),
    [sym____reserved___] = ACTIONS(1492),
    [sym_date] = ACTIONS(1492),
    [sym_day] = ACTIONS(1492),
    [sym_time] = ACTIONS(1492),
    [sym_ldate] = ACTIONS(1492),
    [sym_push_device] = ACTIONS(1492),
    [sym_push_channel] = ACTIONS(1492),
    [sym_push_devchan] = ACTIONS(1492),
    [sym_release_device] = ACTIONS(1492),
    [sym_release_channel] = ACTIONS(1492),
    [sym_release_devchan] = ACTIONS(1492),
    [sym_master_slot] = ACTIONS(1492),
    [sym_get_pulse_time] = ACTIONS(1492),
    [sym_get_timer] = ACTIONS(1492),
    [sym_master_sn] = ACTIONS(1492),
    [sym_system_number] = ACTIONS(1492),
    [sym_button] = ACTIONS(1492),
    [sym_dv_channel] = ACTIONS(1492),
    [sym_data] = ACTIONS(1492),
    [sym_level] = ACTIONS(1492),
    [sym_timeline] = ACTIONS(1492),
    [sym_channel] = ACTIONS(1492),
    [sym_custom] = ACTIONS(1492),
    [sym_atoi] = ACTIONS(1492),
    [sym_atol] = ACTIONS(1492),
    [sym_atof] = ACTIONS(1492),
    [sym_itoa] = ACTIONS(1492),
    [sym_itohex] = ACTIONS(1492),
    [sym_hextoi] = ACTIONS(1492),
    [sym_ftoa] = ACTIONS(1492),
    [sym_find_string] = ACTIONS(1492),
    [sym_left_string] = ACTIONS(1492),
    [sym_right_string] = ACTIONS(1492),
    [sym_mid_string] = ACTIONS(1492),
    [sym_remove_string] = ACTIONS(1492),
    [sym_get_buffer_string] = ACTIONS(1492),
    [sym_get_buffer_char] = ACTIONS(1492),
    [sym_get_multi_buffer_string] = ACTIONS(1492),
    [sym_length_string] = ACTIONS(1492),
    [sym_max_length_string] = ACTIONS(1492),
    [sym_set_length_string] = ACTIONS(1492),
    [sym_compare_string] = ACTIONS(1492),
    [sym_upper_string] = ACTIONS(1492),
    [sym_lower_string] = ACTIONS(1492),
    [sym_get_last] = ACTIONS(1492),
    [sym_date_to_day] = ACTIONS(1492),
    [sym_date_to_month] = ACTIONS(1492),
    [sym_date_to_year] = ACTIONS(1492),
    [sym_time_to_hour] = ACTIONS(1492),
    [sym_time_to_minute] = ACTIONS(1492),
    [sym_time_to_second] = ACTIONS(1492),
    [sym_day_of_week] = ACTIONS(1492),
    [sym_set_pulse_time] = ACTIONS(1492),
    [sym_set_timer] = ACTIONS(1492),
    [sym_file_open] = ACTIONS(1492),
    [sym_file_close] = ACTIONS(1492),
    [sym_file_read] = ACTIONS(1492),
    [sym_file_read_line] = ACTIONS(1492),
    [sym_file_write] = ACTIONS(1492),
    [sym_file_write_line] = ACTIONS(1492),
    [sym_file_copy] = ACTIONS(1492),
    [sym_file_delete] = ACTIONS(1492),
    [sym_file_rename] = ACTIONS(1492),
    [sym_file_dir] = ACTIONS(1492),
    [sym_file_getdir] = ACTIONS(1492),
    [sym_file_setdir] = ACTIONS(1492),
    [sym_file_create_dir] = ACTIONS(1492),
    [sym_file_remove_dir] = ACTIONS(1492),
    [sym_file_seek] = ACTIONS(1492),
    [sym_length_array] = ACTIONS(1492),
    [sym_max_length_array] = ACTIONS(1492),
    [sym_set_length_array] = ACTIONS(1492),
    [sym_ip_client_open] = ACTIONS(1492),
    [sym_ip_client_close] = ACTIONS(1492),
    [sym_ip_server_open] = ACTIONS(1492),
    [sym_ip_server_close] = ACTIONS(1492),
    [sym_ip_bound_client_open] = ACTIONS(1492),
    [sym_ip_mc_server_open] = ACTIONS(1492),
    [sym_ip_set_option] = ACTIONS(1492),
    [sym_ssh_client_open] = ACTIONS(1492),
    [sym_ssh_client_close] = ACTIONS(1492),
    [sym_tls_client_open] = ACTIONS(1492),
    [sym_tls_client_close] = ACTIONS(1492),
    [sym_do_push] = ACTIONS(1492),
    [sym_do_push_timed] = ACTIONS(1492),
    [sym_do_release] = ACTIONS(1492),
    [sym_device_id] = ACTIONS(1492),
    [sym_device_info] = ACTIONS(1492),
    [sym_device_id_string] = ACTIONS(1492),
    [sym_device_standby] = ACTIONS(1492),
    [sym_device_wake] = ACTIONS(1492),
    [sym_timeline_create] = ACTIONS(1492),
    [sym_timeline_kill] = ACTIONS(1492),
    [sym_timeline_reload] = ACTIONS(1492),
    [sym_timeline_pause] = ACTIONS(1492),
    [sym_timeline_restart] = ACTIONS(1492),
    [sym_timeline_set] = ACTIONS(1492),
    [sym_timeline_get] = ACTIONS(1492),
    [sym_timeline_active] = ACTIONS(1492),
    [sym_timeline_dynamic_id] = ACTIONS(1492),
    [sym_variable_to_string] = ACTIONS(1492),
    [sym_string_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_string] = ACTIONS(1492),
    [sym_type_cast] = ACTIONS(1492),
    [sym_variable_to_xml] = ACTIONS(1492),
    [sym_xml_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_xml] = ACTIONS(1492),
    [sym_redirect_string] = ACTIONS(1492),
    [sym_random_number] = ACTIONS(1492),
    [sym_get_unique_id] = ACTIONS(1492),
    [sym_get_system_number] = ACTIONS(1492),
    [sym_set_system_number] = ACTIONS(1492),
    [sym_get_serial_number] = ACTIONS(1492),
    [sym_get_master_build] = ACTIONS(1492),
    [sym_set_validation_code] = ACTIONS(1492),
    [sym_reboot] = ACTIONS(1492),
    [sym_load_duet_module] = ACTIONS(1492),
    [sym_unload_duet_module] = ACTIONS(1492),
    [sym_duet_mem_size_set] = ACTIONS(1492),
    [sym_duet_mem_size_get] = ACTIONS(1492),
    [sym_dynamic_application_device] = ACTIONS(1492),
    [sym_dynamic_polled_port] = ACTIONS(1492),
    [sym_static_port_binding] = ACTIONS(1492),
    [sym_static_ip_binding] = ACTIONS(1492),
    [sym_get_dns_list] = ACTIONS(1492),
    [sym_set_dns_list] = ACTIONS(1492),
    [sym_get_ip_address] = ACTIONS(1492),
    [sym_set_ip_address] = ACTIONS(1492),
    [sym_get_url_list] = ACTIONS(1492),
    [sym_add_url_entry] = ACTIONS(1492),
    [sym_delete_url_entry] = ACTIONS(1492),
    [sym_abs_value] = ACTIONS(1492),
    [sym_max_value] = ACTIONS(1492),
    [sym_min_value] = ACTIONS(1492),
    [sym_exp_value] = ACTIONS(1492),
    [sym_log_value] = ACTIONS(1492),
    [sym_log10_value] = ACTIONS(1492),
    [sym_power_value] = ACTIONS(1492),
    [sym_sqrt_value] = ACTIONS(1492),
    [sym_raw_be] = ACTIONS(1492),
    [sym_raw_le] = ACTIONS(1492),
    [sym_internal_queue_size_set] = ACTIONS(1492),
    [sym_internal_queue_size_get] = ACTIONS(1492),
    [sym_internal_threshold_set] = ACTIONS(1492),
    [sym_internal_threshold_get] = ACTIONS(1492),
    [sym_rebuilt_event] = ACTIONS(1492),
    [sym_do_custom_event] = ACTIONS(1492),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1492),
    [sym_clkmgr_set_clk_source] = ACTIONS(1492),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1492),
    [sym_clkmgr_get_timezone] = ACTIONS(1492),
    [sym_clkmgr_set_timezone] = ACTIONS(1492),
    [sym_clkmgr_get_resync_period] = ACTIONS(1492),
    [sym_clkmgr_set_resync_period] = ACTIONS(1492),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_timeservers] = ACTIONS(1492),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_set_log_level] = ACTIONS(1492),
    [sym_get_log_level] = ACTIONS(1492),
    [sym_amx_log] = ACTIONS(1492),
    [sym_smtp_server_config_set] = ACTIONS(1492),
    [sym_smtp_server_config_get] = ACTIONS(1492),
    [sym_smtp_send] = ACTIONS(1492),
    [sym_validate_netlinx_account] = ACTIONS(1492),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1492),
    [sym_authenticate_certificate] = ACTIONS(1492),
    [sym_audit_netlinx_session_event] = ACTIONS(1492),
    [sym_audit_netlinx_generic_event] = ACTIONS(1492),
    [sym_get_available_flash_disk_space] = ACTIONS(1492),
    [sym_get_max_flash_disk_space] = ACTIONS(1492),
    [sym_data_create_feed] = ACTIONS(1492),
    [sym_data_delete_feed] = ACTIONS(1492),
    [sym_data_publish_feed] = ACTIONS(1492),
    [sym_data_get_published_feed] = ACTIONS(1492),
    [sym_data_add_record] = ACTIONS(1492),
    [sym_data_get_event_record] = ACTIONS(1492),
    [sym__wc_data_create_feed] = ACTIONS(1492),
    [sym__wc_data_add_record] = ACTIONS(1492),
    [sym__wc_data_get_event_record] = ACTIONS(1492),
    [sym_format] = ACTIONS(1492),
    [sym_astro_clock] = ACTIONS(1492),
    [sym_set_outdoor_temperature] = ACTIONS(1492),
    [sym_set_virtual_level_count] = ACTIONS(1492),
    [sym_set_virtual_channel_count] = ACTIONS(1492),
    [sym_set_virtual_port_count] = ACTIONS(1492),
    [sym__wc_encode_int] = ACTIONS(1492),
    [sym__wc_decode_int] = ACTIONS(1492),
    [sym__wc_int] = ACTIONS(1492),
    [sym__wc_to_ch_int] = ACTIONS(1492),
    [sym__ch_to_wc_int] = ACTIONS(1492),
    [sym__wc_find_string_int] = ACTIONS(1492),
    [sym__wc_left_string_int] = ACTIONS(1492),
    [sym__wc_lower_string_int] = ACTIONS(1492),
    [sym__wc_mid_string_int] = ACTIONS(1492),
    [sym__wc_remove_string_int] = ACTIONS(1492),
    [sym__wc_right_string_int] = ACTIONS(1492),
    [sym__wc_upper_string_int] = ACTIONS(1492),
    [sym__wc_compare_string_int] = ACTIONS(1492),
    [sym__wc_get_buffer_char_int] = ACTIONS(1492),
    [sym__wc_get_buffer_string_int] = ACTIONS(1492),
    [sym__wc_concat_string_int] = ACTIONS(1492),
    [sym__wc_file_open_int] = ACTIONS(1492),
    [sym__wc_file_close_int] = ACTIONS(1492),
    [sym__wc_file_read_int] = ACTIONS(1492),
    [sym__wc_file_read_line_int] = ACTIONS(1492),
    [sym__wc_file_write_int] = ACTIONS(1492),
    [sym__wc_file_write_line_int] = ACTIONS(1492),
    [aux_sym_preproc_include_token1] = ACTIONS(1492),
    [aux_sym_preproc_define_token1] = ACTIONS(1494),
    [aux_sym_preproc_warn_token1] = ACTIONS(1494),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1494),
    [anon_sym_LBRACK] = ACTIONS(1494),
    [anon_sym_LBRACE] = ACTIONS(1494),
    [anon_sym_COLON] = ACTIONS(1494),
    [anon_sym_LPAREN] = ACTIONS(1492),
    [anon_sym_SEMI] = ACTIONS(1494),
    [anon_sym_BANG] = ACTIONS(1492),
    [anon_sym_TILDE] = ACTIONS(1494),
    [anon_sym_DASH] = ACTIONS(1492),
    [anon_sym_PLUS] = ACTIONS(1492),
    [anon_sym_STAR] = ACTIONS(1494),
    [anon_sym_SLASH] = ACTIONS(1492),
    [anon_sym_PERCENT] = ACTIONS(1494),
    [anon_sym_PIPE_PIPE] = ACTIONS(1494),
    [anon_sym_AMP_AMP] = ACTIONS(1494),
    [anon_sym_PIPE] = ACTIONS(1492),
    [anon_sym_CARET] = ACTIONS(1494),
    [anon_sym_AMP] = ACTIONS(1492),
    [anon_sym_EQ_EQ] = ACTIONS(1494),
    [anon_sym_BANG_EQ] = ACTIONS(1494),
    [anon_sym_LT_GT] = ACTIONS(1494),
    [anon_sym_GT] = ACTIONS(1492),
    [anon_sym_GT_EQ] = ACTIONS(1494),
    [anon_sym_LT_EQ] = ACTIONS(1494),
    [anon_sym_LT] = ACTIONS(1492),
    [anon_sym_LT_LT] = ACTIONS(1494),
    [anon_sym_GT_GT] = ACTIONS(1494),
    [anon_sym_DASH_DASH] = ACTIONS(1494),
    [anon_sym_PLUS_PLUS] = ACTIONS(1494),
    [anon_sym_DOT] = ACTIONS(1492),
    [anon_sym_DQUOTE] = ACTIONS(1494),
    [anon_sym_SQUOTE] = ACTIONS(1494),
    [sym_number_literal] = ACTIONS(1494),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1494),
  },
  [STATE(138)] = {
    [ts_builtin_sym_end] = ACTIONS(1469),
    [sym_identifier] = ACTIONS(1465),
    [sym_program_name_keyword] = ACTIONS(1465),
    [sym_module_name_keyword] = ACTIONS(1465),
    [sym_define_device_keyword] = ACTIONS(1465),
    [sym_define_combine_keyword] = ACTIONS(1465),
    [sym_define_constant_keyword] = ACTIONS(1465),
    [sym_define_type_keyword] = ACTIONS(1465),
    [sym_define_variable_keyword] = ACTIONS(1465),
    [sym_define_system_variable_keyword] = ACTIONS(1465),
    [sym_define_connect_level_keyword] = ACTIONS(1465),
    [sym_define_latching_keyword] = ACTIONS(1465),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1465),
    [sym_define_toggling_keyword] = ACTIONS(1465),
    [sym_define_call_keyword] = ACTIONS(1465),
    [sym_define_function_keyword] = ACTIONS(1465),
    [sym_define_library_function_keyword] = ACTIONS(1465),
    [sym_define_start_keyword] = ACTIONS(1465),
    [sym_define_event_keyword] = ACTIONS(1465),
    [sym_define_program_keyword] = ACTIONS(1465),
    [sym_define_module_keyword] = ACTIONS(1465),
    [sym_char_keyword] = ACTIONS(1465),
    [sym_widechar_keyword] = ACTIONS(1465),
    [sym_integer_keyword] = ACTIONS(1465),
    [sym_sinteger_keyword] = ACTIONS(1465),
    [sym_long_keyword] = ACTIONS(1465),
    [sym_slong_keyword] = ACTIONS(1465),
    [sym_float_keyword] = ACTIONS(1465),
    [sym_double_keyword] = ACTIONS(1465),
    [sym_variant_keyword] = ACTIONS(1465),
    [sym_variantarray_keyword] = ACTIONS(1465),
    [sym_local_var_keyword] = ACTIONS(1465),
    [sym_stack_var_keyword] = ACTIONS(1465),
    [sym_constant_keyword] = ACTIONS(1465),
    [sym_volatile_keyword] = ACTIONS(1465),
    [sym_non_volatile_keyword] = ACTIONS(1465),
    [sym_persistent_keyword] = ACTIONS(1465),
    [sym_dev_keyword] = ACTIONS(1465),
    [sym_devlev_keyword] = ACTIONS(1465),
    [sym_devchan_keyword] = ACTIONS(1465),
    [sym_if_keyword] = ACTIONS(1465),
    [sym_else_keyword] = ACTIONS(1465),
    [sym_switch_keyword] = ACTIONS(1465),
    [sym_case_keyword] = ACTIONS(1465),
    [sym_default_keyword] = ACTIONS(1465),
    [sym_while_keyword] = ACTIONS(1465),
    [sym_for_keyword] = ACTIONS(1465),
    [sym_break_keyword] = ACTIONS(1465),
    [sym_continue_keyword] = ACTIONS(1465),
    [sym_return_keyword] = ACTIONS(1465),
    [sym_select_keyword] = ACTIONS(1465),
    [sym_active_keyword] = ACTIONS(1465),
    [sym_struct_keyword] = ACTIONS(1465),
    [sym_structure_keyword] = ACTIONS(1465),
    [sym_button_event_keyword] = ACTIONS(1465),
    [sym_channel_event_keyword] = ACTIONS(1465),
    [sym_level_event_keyword] = ACTIONS(1465),
    [sym_data_event_keyword] = ACTIONS(1465),
    [sym_timeline_event_keyword] = ACTIONS(1465),
    [sym_custom_event_keyword] = ACTIONS(1465),
    [sym_push_keyword] = ACTIONS(1465),
    [sym_release_keyword] = ACTIONS(1465),
    [aux_sym_on_keyword_token1] = ACTIONS(1465),
    [aux_sym_off_keyword_token1] = ACTIONS(1465),
    [sym_send_level_keyword] = ACTIONS(1465),
    [sym_send_string_keyword] = ACTIONS(1465),
    [sym_send_command_keyword] = ACTIONS(1465),
    [sym_clear_buffer_keyword] = ACTIONS(1465),
    [sym_create_buffer_keyword] = ACTIONS(1465),
    [sym_create_multi_buffer_keyword] = ACTIONS(1465),
    [sym_call_keyword] = ACTIONS(1465),
    [sym_system_call_keyword] = ACTIONS(1465),
    [sym_devchan_to_keyword] = ACTIONS(1465),
    [sym_devchan_min_to_keyword] = ACTIONS(1465),
    [sym_devchan_total_off_keyword] = ACTIONS(1465),
    [sym_devchan_pulse_keyword] = ACTIONS(1465),
    [sym_wait_keyword] = ACTIONS(1465),
    [sym_cancel_wait_keyword] = ACTIONS(1465),
    [sym_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym_tdata] = ACTIONS(1465),
    [sym_tchannel] = ACTIONS(1465),
    [sym_tlevel] = ACTIONS(1465),
    [sym_tbutton] = ACTIONS(1465),
    [sym_ttimeline] = ACTIONS(1465),
    [sym_tcustom] = ACTIONS(1465),
    [sym_url_struct] = ACTIONS(1465),
    [sym_dns_struct] = ACTIONS(1465),
    [sym_ip_address_struct] = ACTIONS(1465),
    [sym_dev_info_struct] = ACTIONS(1465),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1465),
    [sym_data_feed] = ACTIONS(1465),
    [sym_data_field] = ACTIONS(1465),
    [sym_data_record] = ACTIONS(1465),
    [sym_wc_data_feed] = ACTIONS(1465),
    [sym_wc_data_field] = ACTIONS(1465),
    [sym_wc_data_record] = ACTIONS(1465),
    [sym_last_login_info] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [aux_sym_preproc_include_token1] = ACTIONS(1465),
    [aux_sym_preproc_define_token1] = ACTIONS(1469),
    [aux_sym_preproc_warn_token1] = ACTIONS(1469),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1469),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1469),
  },
  [STATE(139)] = {
    [ts_builtin_sym_end] = ACTIONS(1575),
    [sym_identifier] = ACTIONS(1573),
    [sym_program_name_keyword] = ACTIONS(1573),
    [sym_module_name_keyword] = ACTIONS(1573),
    [sym_define_device_keyword] = ACTIONS(1573),
    [sym_define_combine_keyword] = ACTIONS(1573),
    [sym_define_constant_keyword] = ACTIONS(1573),
    [sym_define_type_keyword] = ACTIONS(1573),
    [sym_define_variable_keyword] = ACTIONS(1573),
    [sym_define_system_variable_keyword] = ACTIONS(1573),
    [sym_define_connect_level_keyword] = ACTIONS(1573),
    [sym_define_latching_keyword] = ACTIONS(1573),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1573),
    [sym_define_toggling_keyword] = ACTIONS(1573),
    [sym_define_call_keyword] = ACTIONS(1573),
    [sym_define_function_keyword] = ACTIONS(1573),
    [sym_define_library_function_keyword] = ACTIONS(1573),
    [sym_define_start_keyword] = ACTIONS(1573),
    [sym_define_event_keyword] = ACTIONS(1573),
    [sym_define_program_keyword] = ACTIONS(1573),
    [sym_define_module_keyword] = ACTIONS(1573),
    [sym_char_keyword] = ACTIONS(1573),
    [sym_widechar_keyword] = ACTIONS(1573),
    [sym_integer_keyword] = ACTIONS(1573),
    [sym_sinteger_keyword] = ACTIONS(1573),
    [sym_long_keyword] = ACTIONS(1573),
    [sym_slong_keyword] = ACTIONS(1573),
    [sym_float_keyword] = ACTIONS(1573),
    [sym_double_keyword] = ACTIONS(1573),
    [sym_variant_keyword] = ACTIONS(1573),
    [sym_variantarray_keyword] = ACTIONS(1573),
    [sym_local_var_keyword] = ACTIONS(1573),
    [sym_stack_var_keyword] = ACTIONS(1573),
    [sym_constant_keyword] = ACTIONS(1573),
    [sym_volatile_keyword] = ACTIONS(1573),
    [sym_non_volatile_keyword] = ACTIONS(1573),
    [sym_persistent_keyword] = ACTIONS(1573),
    [sym_dev_keyword] = ACTIONS(1573),
    [sym_devlev_keyword] = ACTIONS(1573),
    [sym_devchan_keyword] = ACTIONS(1573),
    [sym_if_keyword] = ACTIONS(1573),
    [sym_else_keyword] = ACTIONS(1573),
    [sym_switch_keyword] = ACTIONS(1573),
    [sym_case_keyword] = ACTIONS(1573),
    [sym_default_keyword] = ACTIONS(1573),
    [sym_while_keyword] = ACTIONS(1573),
    [sym_for_keyword] = ACTIONS(1573),
    [sym_break_keyword] = ACTIONS(1573),
    [sym_continue_keyword] = ACTIONS(1573),
    [sym_return_keyword] = ACTIONS(1573),
    [sym_select_keyword] = ACTIONS(1573),
    [sym_active_keyword] = ACTIONS(1573),
    [sym_struct_keyword] = ACTIONS(1573),
    [sym_structure_keyword] = ACTIONS(1573),
    [sym_button_event_keyword] = ACTIONS(1573),
    [sym_channel_event_keyword] = ACTIONS(1573),
    [sym_level_event_keyword] = ACTIONS(1573),
    [sym_data_event_keyword] = ACTIONS(1573),
    [sym_timeline_event_keyword] = ACTIONS(1573),
    [sym_custom_event_keyword] = ACTIONS(1573),
    [sym_push_keyword] = ACTIONS(1573),
    [sym_release_keyword] = ACTIONS(1573),
    [aux_sym_on_keyword_token1] = ACTIONS(1573),
    [aux_sym_off_keyword_token1] = ACTIONS(1573),
    [sym_send_level_keyword] = ACTIONS(1573),
    [sym_send_string_keyword] = ACTIONS(1573),
    [sym_send_command_keyword] = ACTIONS(1573),
    [sym_clear_buffer_keyword] = ACTIONS(1573),
    [sym_create_buffer_keyword] = ACTIONS(1573),
    [sym_create_multi_buffer_keyword] = ACTIONS(1573),
    [sym_call_keyword] = ACTIONS(1573),
    [sym_system_call_keyword] = ACTIONS(1573),
    [sym_devchan_to_keyword] = ACTIONS(1573),
    [sym_devchan_min_to_keyword] = ACTIONS(1573),
    [sym_devchan_total_off_keyword] = ACTIONS(1573),
    [sym_devchan_pulse_keyword] = ACTIONS(1573),
    [sym_wait_keyword] = ACTIONS(1573),
    [sym_cancel_wait_keyword] = ACTIONS(1573),
    [sym_wait_until_keyword] = ACTIONS(1573),
    [sym_cancel_wait_until_keyword] = ACTIONS(1573),
    [sym_cancel_all_wait_keyword] = ACTIONS(1573),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1573),
    [sym_and] = ACTIONS(1573),
    [sym_or] = ACTIONS(1573),
    [sym_not] = ACTIONS(1573),
    [sym_xor] = ACTIONS(1573),
    [sym_band] = ACTIONS(1573),
    [sym_bor] = ACTIONS(1573),
    [sym_bxor] = ACTIONS(1573),
    [sym_bnot] = ACTIONS(1573),
    [sym_lshift] = ACTIONS(1573),
    [sym_rshift] = ACTIONS(1573),
    [sym___file__] = ACTIONS(1573),
    [sym___date__] = ACTIONS(1573),
    [sym___ldate__] = ACTIONS(1573),
    [sym___time__] = ACTIONS(1573),
    [sym___line__] = ACTIONS(1573),
    [sym___name__] = ACTIONS(1573),
    [sym___netlinx__] = ACTIONS(1573),
    [sym_netlinx_axi_version] = ACTIONS(1573),
    [sym_url_flg_tcp] = ACTIONS(1573),
    [sym_url_flg_accinfopresent] = ACTIONS(1573),
    [sym_url_flg_temp] = ACTIONS(1573),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1573),
    [sym_url_flg_stat_mask] = ACTIONS(1573),
    [sym_url_flg_stat_lookup] = ACTIONS(1573),
    [sym_url_flg_stat_connecting] = ACTIONS(1573),
    [sym_url_flg_stat_waiting] = ACTIONS(1573),
    [sym_url_flg_stat_connected] = ACTIONS(1573),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1573),
    [sym_first_virtual_device] = ACTIONS(1573),
    [sym_first_local_port] = ACTIONS(1573),
    [sym_dynamic_virtual_device] = ACTIONS(1573),
    [sym_timeline_once] = ACTIONS(1573),
    [sym_timeline_repeat] = ACTIONS(1573),
    [sym_timeline_absolute] = ACTIONS(1573),
    [sym_timeline_relative] = ACTIONS(1573),
    [sym_do_push_timed_infinite] = ACTIONS(1573),
    [sym_source_type_no_address] = ACTIONS(1573),
    [sym_source_type_neuron_id] = ACTIONS(1573),
    [sym_source_type_ip_address] = ACTIONS(1573),
    [sym_source_type_axlink] = ACTIONS(1573),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1573),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1573),
    [sym_source_type_ip_socket_address] = ACTIONS(1573),
    [sym_source_type_rs232] = ACTIONS(1573),
    [sym_source_type_internal] = ACTIONS(1573),
    [sym_source_type_mac_address] = ACTIONS(1573),
    [sym_source_type_ipv4_port] = ACTIONS(1573),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1573),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1573),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1573),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1573),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1573),
    [sym_true] = ACTIONS(1573),
    [sym_false] = ACTIONS(1573),
    [sym_file_read_only] = ACTIONS(1573),
    [sym_file_rw_new] = ACTIONS(1573),
    [sym_file_rw_append] = ACTIONS(1573),
    [sym_ip_tcp] = ACTIONS(1573),
    [sym_ip_udp] = ACTIONS(1573),
    [sym_ip_udp_2way] = ACTIONS(1573),
    [sym_xml_encode_types] = ACTIONS(1573),
    [sym_xml_encode_char_as_list] = ACTIONS(1573),
    [sym_xml_encode_le] = ACTIONS(1573),
    [sym_xml_decode_types] = ACTIONS(1573),
    [sym_xml_decode_no_preserve] = ACTIONS(1573),
    [sym_duet_dev_type_display_device] = ACTIONS(1573),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1573),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1573),
    [sym_duet_dev_type_io_device] = ACTIONS(1573),
    [sym_duet_dev_type_relay_device] = ACTIONS(1573),
    [sym_duet_dev_type_ups] = ACTIONS(1573),
    [sym_duet_dev_type_amplifier] = ACTIONS(1573),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1573),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1573),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1573),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1573),
    [sym_duet_dev_type_camera] = ACTIONS(1573),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1573),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1573),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1573),
    [sym_duet_dev_type_dss] = ACTIONS(1573),
    [sym_duet_dev_type_dvr] = ACTIONS(1573),
    [sym_duet_dev_type_disc_device] = ACTIONS(1573),
    [sym_duet_dev_type_document_camera] = ACTIONS(1573),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1573),
    [sym_duet_dev_type_hvac] = ACTIONS(1573),
    [sym_duet_dev_type_keypad] = ACTIONS(1573),
    [sym_duet_dev_type_light] = ACTIONS(1573),
    [sym_duet_dev_type_monitor] = ACTIONS(1573),
    [sym_duet_dev_type_motor] = ACTIONS(1573),
    [sym_duet_dev_type_multi_window] = ACTIONS(1573),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1573),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1573),
    [sym_duet_dev_type_receiver] = ACTIONS(1573),
    [sym_duet_dev_type_security_system] = ACTIONS(1573),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1573),
    [sym_duet_dev_type_settop_box] = ACTIONS(1573),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1573),
    [sym_duet_dev_type_switcher] = ACTIONS(1573),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1573),
    [sym_duet_dev_type_tv] = ACTIONS(1573),
    [sym_duet_dev_type_utility] = ACTIONS(1573),
    [sym_duet_dev_type_vcr] = ACTIONS(1573),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1573),
    [sym_duet_dev_type_video_processor] = ACTIONS(1573),
    [sym_duet_dev_type_video_projector] = ACTIONS(1573),
    [sym_duet_dev_type_video_wall] = ACTIONS(1573),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1573),
    [sym_duet_dev_type_weather] = ACTIONS(1573),
    [sym_duet_dev_not_polled] = ACTIONS(1573),
    [sym_duet_dev_polled] = ACTIONS(1573),
    [sym_clkmgr_mode_network] = ACTIONS(1573),
    [sym_clkmgr_mode_standalone] = ACTIONS(1573),
    [sym_normal_standby] = ACTIONS(1573),
    [sym_normal_wake] = ACTIONS(1573),
    [sym_ip_multicast_ttl_option] = ACTIONS(1573),
    [sym_ip_ttl_subnet] = ACTIONS(1573),
    [sym_ip_ttl_site] = ACTIONS(1573),
    [sym_ip_ttl_region] = ACTIONS(1573),
    [sym_ip_ttl_continent] = ACTIONS(1573),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1573),
    [sym_ip_nodelay_on] = ACTIONS(1573),
    [sym_ip_nodelay_off] = ACTIONS(1573),
    [sym_amx_error] = ACTIONS(1573),
    [sym_amx_warning] = ACTIONS(1573),
    [sym_amx_info] = ACTIONS(1573),
    [sym_amx_debug] = ACTIONS(1573),
    [sym_netlinx_login_success] = ACTIONS(1573),
    [sym_netlinx_login_fail] = ACTIONS(1573),
    [sym_netlinx_logout] = ACTIONS(1573),
    [sym_null_str] = ACTIONS(1573),
    [sym_smtp_address] = ACTIONS(1573),
    [sym_smtp_port_number] = ACTIONS(1573),
    [sym_smtp_username] = ACTIONS(1573),
    [sym_smtp_password] = ACTIONS(1573),
    [sym_smtp_require_tls] = ACTIONS(1573),
    [sym_smtp_from] = ACTIONS(1573),
    [sym_smtp_tls_true] = ACTIONS(1573),
    [sym_smtp_tls_false] = ACTIONS(1573),
    [sym_listview_on_row_select_event] = ACTIONS(1573),
    [sym_data_structure_unknown] = ACTIONS(1573),
    [sym_data_structure_datafeed] = ACTIONS(1573),
    [sym_data_structure_datarecordset] = ACTIONS(1573),
    [sym_data_structure_datarecord] = ACTIONS(1573),
    [sym_data_structure_datafield] = ACTIONS(1573),
    [sym_data_max_content_field_count] = ACTIONS(1573),
    [sym_data_max_metadata_field_count] = ACTIONS(1573),
    [sym_data_max_id_length] = ACTIONS(1573),
    [sym_data_max_name_length] = ACTIONS(1573),
    [sym_data_max_description_length] = ACTIONS(1573),
    [sym_data_max_type_length] = ACTIONS(1573),
    [sym_data_max_format_length] = ACTIONS(1573),
    [sym_data_max_source_length] = ACTIONS(1573),
    [sym_data_max_value_length] = ACTIONS(1573),
    [sym_data_max_label_length] = ACTIONS(1573),
    [sym_data_max_url_length] = ACTIONS(1573),
    [sym_max_last_login_info_length] = ACTIONS(1573),
    [sym_data_type_unknown] = ACTIONS(1573),
    [sym_data_type_string] = ACTIONS(1573),
    [sym_data_type_datetime] = ACTIONS(1573),
    [sym_data_type_date] = ACTIONS(1573),
    [sym_data_type_time] = ACTIONS(1573),
    [sym_data_type_image] = ACTIONS(1573),
    [sym_data_format_url] = ACTIONS(1573),
    [sym_data_format_phone] = ACTIONS(1573),
    [sym_data_format_email] = ACTIONS(1573),
    [sym_data_format_iso8601] = ACTIONS(1573),
    [sym_auditlog_priv] = ACTIONS(1573),
    [sym_configuration_priv] = ACTIONS(1573),
    [sym_deviceconfig_priv] = ACTIONS(1573),
    [sym_ftp_priv] = ACTIONS(1573),
    [sym_http_priv] = ACTIONS(1573),
    [sym_networkconfig_priv] = ACTIONS(1573),
    [sym_programport_priv] = ACTIONS(1573),
    [sym_remoteui_priv] = ACTIONS(1573),
    [sym_securitycontrol_priv] = ACTIONS(1573),
    [sym_softwaremanagement_priv] = ACTIONS(1573),
    [sym_terminal_priv] = ACTIONS(1573),
    [sym_tpadmin_priv] = ACTIONS(1573),
    [sym_user1_priv] = ACTIONS(1573),
    [sym_user2_priv] = ACTIONS(1573),
    [sym_user3_priv] = ACTIONS(1573),
    [sym_user4_priv] = ACTIONS(1573),
    [sym_usermanagement_priv] = ACTIONS(1573),
    [sym_valid_account] = ACTIONS(1573),
    [sym_err_name_invalid] = ACTIONS(1573),
    [sym_err_password_invalid] = ACTIONS(1573),
    [sym_err_invalid_account] = ACTIONS(1573),
    [sym_err_no_authorization] = ACTIONS(1573),
    [sym_err_invalid_parameter] = ACTIONS(1573),
    [sym_err_user_locked_out] = ACTIONS(1573),
    [sym_err_user_account_expired] = ACTIONS(1573),
    [sym_err_general_error] = ACTIONS(1573),
    [sym_err_authentication_error] = ACTIONS(1573),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1573),
    [sym_tls_validate_certificate] = ACTIONS(1573),
    [sym_tdata] = ACTIONS(1573),
    [sym_tchannel] = ACTIONS(1573),
    [sym_tlevel] = ACTIONS(1573),
    [sym_tbutton] = ACTIONS(1573),
    [sym_ttimeline] = ACTIONS(1573),
    [sym_tcustom] = ACTIONS(1573),
    [sym_url_struct] = ACTIONS(1573),
    [sym_dns_struct] = ACTIONS(1573),
    [sym_ip_address_struct] = ACTIONS(1573),
    [sym_dev_info_struct] = ACTIONS(1573),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1573),
    [sym_data_feed] = ACTIONS(1573),
    [sym_data_field] = ACTIONS(1573),
    [sym_data_record] = ACTIONS(1573),
    [sym_wc_data_feed] = ACTIONS(1573),
    [sym_wc_data_field] = ACTIONS(1573),
    [sym_wc_data_record] = ACTIONS(1573),
    [sym_last_login_info] = ACTIONS(1573),
    [sym____reserved___] = ACTIONS(1573),
    [sym_date] = ACTIONS(1573),
    [sym_day] = ACTIONS(1573),
    [sym_time] = ACTIONS(1573),
    [sym_ldate] = ACTIONS(1573),
    [sym_push_device] = ACTIONS(1573),
    [sym_push_channel] = ACTIONS(1573),
    [sym_push_devchan] = ACTIONS(1573),
    [sym_release_device] = ACTIONS(1573),
    [sym_release_channel] = ACTIONS(1573),
    [sym_release_devchan] = ACTIONS(1573),
    [sym_master_slot] = ACTIONS(1573),
    [sym_get_pulse_time] = ACTIONS(1573),
    [sym_get_timer] = ACTIONS(1573),
    [sym_master_sn] = ACTIONS(1573),
    [sym_system_number] = ACTIONS(1573),
    [sym_button] = ACTIONS(1573),
    [sym_dv_channel] = ACTIONS(1573),
    [sym_data] = ACTIONS(1573),
    [sym_level] = ACTIONS(1573),
    [sym_timeline] = ACTIONS(1573),
    [sym_channel] = ACTIONS(1573),
    [sym_custom] = ACTIONS(1573),
    [sym_atoi] = ACTIONS(1573),
    [sym_atol] = ACTIONS(1573),
    [sym_atof] = ACTIONS(1573),
    [sym_itoa] = ACTIONS(1573),
    [sym_itohex] = ACTIONS(1573),
    [sym_hextoi] = ACTIONS(1573),
    [sym_ftoa] = ACTIONS(1573),
    [sym_find_string] = ACTIONS(1573),
    [sym_left_string] = ACTIONS(1573),
    [sym_right_string] = ACTIONS(1573),
    [sym_mid_string] = ACTIONS(1573),
    [sym_remove_string] = ACTIONS(1573),
    [sym_get_buffer_string] = ACTIONS(1573),
    [sym_get_buffer_char] = ACTIONS(1573),
    [sym_get_multi_buffer_string] = ACTIONS(1573),
    [sym_length_string] = ACTIONS(1573),
    [sym_max_length_string] = ACTIONS(1573),
    [sym_set_length_string] = ACTIONS(1573),
    [sym_compare_string] = ACTIONS(1573),
    [sym_upper_string] = ACTIONS(1573),
    [sym_lower_string] = ACTIONS(1573),
    [sym_get_last] = ACTIONS(1573),
    [sym_date_to_day] = ACTIONS(1573),
    [sym_date_to_month] = ACTIONS(1573),
    [sym_date_to_year] = ACTIONS(1573),
    [sym_time_to_hour] = ACTIONS(1573),
    [sym_time_to_minute] = ACTIONS(1573),
    [sym_time_to_second] = ACTIONS(1573),
    [sym_day_of_week] = ACTIONS(1573),
    [sym_set_pulse_time] = ACTIONS(1573),
    [sym_set_timer] = ACTIONS(1573),
    [sym_file_open] = ACTIONS(1573),
    [sym_file_close] = ACTIONS(1573),
    [sym_file_read] = ACTIONS(1573),
    [sym_file_read_line] = ACTIONS(1573),
    [sym_file_write] = ACTIONS(1573),
    [sym_file_write_line] = ACTIONS(1573),
    [sym_file_copy] = ACTIONS(1573),
    [sym_file_delete] = ACTIONS(1573),
    [sym_file_rename] = ACTIONS(1573),
    [sym_file_dir] = ACTIONS(1573),
    [sym_file_getdir] = ACTIONS(1573),
    [sym_file_setdir] = ACTIONS(1573),
    [sym_file_create_dir] = ACTIONS(1573),
    [sym_file_remove_dir] = ACTIONS(1573),
    [sym_file_seek] = ACTIONS(1573),
    [sym_length_array] = ACTIONS(1573),
    [sym_max_length_array] = ACTIONS(1573),
    [sym_set_length_array] = ACTIONS(1573),
    [sym_ip_client_open] = ACTIONS(1573),
    [sym_ip_client_close] = ACTIONS(1573),
    [sym_ip_server_open] = ACTIONS(1573),
    [sym_ip_server_close] = ACTIONS(1573),
    [sym_ip_bound_client_open] = ACTIONS(1573),
    [sym_ip_mc_server_open] = ACTIONS(1573),
    [sym_ip_set_option] = ACTIONS(1573),
    [sym_ssh_client_open] = ACTIONS(1573),
    [sym_ssh_client_close] = ACTIONS(1573),
    [sym_tls_client_open] = ACTIONS(1573),
    [sym_tls_client_close] = ACTIONS(1573),
    [sym_do_push] = ACTIONS(1573),
    [sym_do_push_timed] = ACTIONS(1573),
    [sym_do_release] = ACTIONS(1573),
    [sym_device_id] = ACTIONS(1573),
    [sym_device_info] = ACTIONS(1573),
    [sym_device_id_string] = ACTIONS(1573),
    [sym_device_standby] = ACTIONS(1573),
    [sym_device_wake] = ACTIONS(1573),
    [sym_timeline_create] = ACTIONS(1573),
    [sym_timeline_kill] = ACTIONS(1573),
    [sym_timeline_reload] = ACTIONS(1573),
    [sym_timeline_pause] = ACTIONS(1573),
    [sym_timeline_restart] = ACTIONS(1573),
    [sym_timeline_set] = ACTIONS(1573),
    [sym_timeline_get] = ACTIONS(1573),
    [sym_timeline_active] = ACTIONS(1573),
    [sym_timeline_dynamic_id] = ACTIONS(1573),
    [sym_variable_to_string] = ACTIONS(1573),
    [sym_string_to_variable] = ACTIONS(1573),
    [sym_length_variable_to_string] = ACTIONS(1573),
    [sym_type_cast] = ACTIONS(1573),
    [sym_variable_to_xml] = ACTIONS(1573),
    [sym_xml_to_variable] = ACTIONS(1573),
    [sym_length_variable_to_xml] = ACTIONS(1573),
    [sym_redirect_string] = ACTIONS(1573),
    [sym_random_number] = ACTIONS(1573),
    [sym_get_unique_id] = ACTIONS(1573),
    [sym_get_system_number] = ACTIONS(1573),
    [sym_set_system_number] = ACTIONS(1573),
    [sym_get_serial_number] = ACTIONS(1573),
    [sym_get_master_build] = ACTIONS(1573),
    [sym_set_validation_code] = ACTIONS(1573),
    [sym_reboot] = ACTIONS(1573),
    [sym_load_duet_module] = ACTIONS(1573),
    [sym_unload_duet_module] = ACTIONS(1573),
    [sym_duet_mem_size_set] = ACTIONS(1573),
    [sym_duet_mem_size_get] = ACTIONS(1573),
    [sym_dynamic_application_device] = ACTIONS(1573),
    [sym_dynamic_polled_port] = ACTIONS(1573),
    [sym_static_port_binding] = ACTIONS(1573),
    [sym_static_ip_binding] = ACTIONS(1573),
    [sym_get_dns_list] = ACTIONS(1573),
    [sym_set_dns_list] = ACTIONS(1573),
    [sym_get_ip_address] = ACTIONS(1573),
    [sym_set_ip_address] = ACTIONS(1573),
    [sym_get_url_list] = ACTIONS(1573),
    [sym_add_url_entry] = ACTIONS(1573),
    [sym_delete_url_entry] = ACTIONS(1573),
    [sym_abs_value] = ACTIONS(1573),
    [sym_max_value] = ACTIONS(1573),
    [sym_min_value] = ACTIONS(1573),
    [sym_exp_value] = ACTIONS(1573),
    [sym_log_value] = ACTIONS(1573),
    [sym_log10_value] = ACTIONS(1573),
    [sym_power_value] = ACTIONS(1573),
    [sym_sqrt_value] = ACTIONS(1573),
    [sym_raw_be] = ACTIONS(1573),
    [sym_raw_le] = ACTIONS(1573),
    [sym_internal_queue_size_set] = ACTIONS(1573),
    [sym_internal_queue_size_get] = ACTIONS(1573),
    [sym_internal_threshold_set] = ACTIONS(1573),
    [sym_internal_threshold_get] = ACTIONS(1573),
    [sym_rebuilt_event] = ACTIONS(1573),
    [sym_do_custom_event] = ACTIONS(1573),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1573),
    [sym_clkmgr_set_clk_source] = ACTIONS(1573),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1573),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1573),
    [sym_clkmgr_get_timezone] = ACTIONS(1573),
    [sym_clkmgr_set_timezone] = ACTIONS(1573),
    [sym_clkmgr_get_resync_period] = ACTIONS(1573),
    [sym_clkmgr_set_resync_period] = ACTIONS(1573),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1573),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1573),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1573),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1573),
    [sym_clkmgr_get_timeservers] = ACTIONS(1573),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1573),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1573),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1573),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1573),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1573),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1573),
    [sym_set_log_level] = ACTIONS(1573),
    [sym_get_log_level] = ACTIONS(1573),
    [sym_amx_log] = ACTIONS(1573),
    [sym_smtp_server_config_set] = ACTIONS(1573),
    [sym_smtp_server_config_get] = ACTIONS(1573),
    [sym_smtp_send] = ACTIONS(1573),
    [sym_validate_netlinx_account] = ACTIONS(1573),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1573),
    [sym_authenticate_certificate] = ACTIONS(1573),
    [sym_audit_netlinx_session_event] = ACTIONS(1573),
    [sym_audit_netlinx_generic_event] = ACTIONS(1573),
    [sym_get_available_flash_disk_space] = ACTIONS(1573),
    [sym_get_max_flash_disk_space] = ACTIONS(1573),
    [sym_data_create_feed] = ACTIONS(1573),
    [sym_data_delete_feed] = ACTIONS(1573),
    [sym_data_publish_feed] = ACTIONS(1573),
    [sym_data_get_published_feed] = ACTIONS(1573),
    [sym_data_add_record] = ACTIONS(1573),
    [sym_data_get_event_record] = ACTIONS(1573),
    [sym__wc_data_create_feed] = ACTIONS(1573),
    [sym__wc_data_add_record] = ACTIONS(1573),
    [sym__wc_data_get_event_record] = ACTIONS(1573),
    [sym_format] = ACTIONS(1573),
    [sym_astro_clock] = ACTIONS(1573),
    [sym_set_outdoor_temperature] = ACTIONS(1573),
    [sym_set_virtual_level_count] = ACTIONS(1573),
    [sym_set_virtual_channel_count] = ACTIONS(1573),
    [sym_set_virtual_port_count] = ACTIONS(1573),
    [sym__wc_encode_int] = ACTIONS(1573),
    [sym__wc_decode_int] = ACTIONS(1573),
    [sym__wc_int] = ACTIONS(1573),
    [sym__wc_to_ch_int] = ACTIONS(1573),
    [sym__ch_to_wc_int] = ACTIONS(1573),
    [sym__wc_find_string_int] = ACTIONS(1573),
    [sym__wc_left_string_int] = ACTIONS(1573),
    [sym__wc_lower_string_int] = ACTIONS(1573),
    [sym__wc_mid_string_int] = ACTIONS(1573),
    [sym__wc_remove_string_int] = ACTIONS(1573),
    [sym__wc_right_string_int] = ACTIONS(1573),
    [sym__wc_upper_string_int] = ACTIONS(1573),
    [sym__wc_compare_string_int] = ACTIONS(1573),
    [sym__wc_get_buffer_char_int] = ACTIONS(1573),
    [sym__wc_get_buffer_string_int] = ACTIONS(1573),
    [sym__wc_concat_string_int] = ACTIONS(1573),
    [sym__wc_file_open_int] = ACTIONS(1573),
    [sym__wc_file_close_int] = ACTIONS(1573),
    [sym__wc_file_read_int] = ACTIONS(1573),
    [sym__wc_file_read_line_int] = ACTIONS(1573),
    [sym__wc_file_write_int] = ACTIONS(1573),
    [sym__wc_file_write_line_int] = ACTIONS(1573),
    [aux_sym_preproc_include_token1] = ACTIONS(1573),
    [aux_sym_preproc_define_token1] = ACTIONS(1575),
    [aux_sym_preproc_warn_token1] = ACTIONS(1575),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1575),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1575),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1575),
    [anon_sym_LBRACK] = ACTIONS(1575),
    [anon_sym_LBRACE] = ACTIONS(1575),
    [anon_sym_COLON] = ACTIONS(1575),
    [anon_sym_LPAREN] = ACTIONS(1573),
    [anon_sym_SEMI] = ACTIONS(1575),
    [anon_sym_BANG] = ACTIONS(1573),
    [anon_sym_TILDE] = ACTIONS(1575),
    [anon_sym_DASH] = ACTIONS(1573),
    [anon_sym_PLUS] = ACTIONS(1573),
    [anon_sym_STAR] = ACTIONS(1575),
    [anon_sym_SLASH] = ACTIONS(1573),
    [anon_sym_PERCENT] = ACTIONS(1575),
    [anon_sym_PIPE_PIPE] = ACTIONS(1575),
    [anon_sym_AMP_AMP] = ACTIONS(1575),
    [anon_sym_PIPE] = ACTIONS(1573),
    [anon_sym_CARET] = ACTIONS(1575),
    [anon_sym_AMP] = ACTIONS(1573),
    [anon_sym_EQ_EQ] = ACTIONS(1575),
    [anon_sym_BANG_EQ] = ACTIONS(1575),
    [anon_sym_LT_GT] = ACTIONS(1575),
    [anon_sym_GT] = ACTIONS(1573),
    [anon_sym_GT_EQ] = ACTIONS(1575),
    [anon_sym_LT_EQ] = ACTIONS(1575),
    [anon_sym_LT] = ACTIONS(1573),
    [anon_sym_LT_LT] = ACTIONS(1575),
    [anon_sym_GT_GT] = ACTIONS(1575),
    [anon_sym_DASH_DASH] = ACTIONS(1575),
    [anon_sym_PLUS_PLUS] = ACTIONS(1575),
    [anon_sym_DOT] = ACTIONS(1573),
    [anon_sym_DQUOTE] = ACTIONS(1575),
    [anon_sym_SQUOTE] = ACTIONS(1575),
    [sym_number_literal] = ACTIONS(1575),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1575),
  },
  [STATE(140)] = {
    [ts_builtin_sym_end] = ACTIONS(1567),
    [sym_identifier] = ACTIONS(1565),
    [sym_program_name_keyword] = ACTIONS(1565),
    [sym_module_name_keyword] = ACTIONS(1565),
    [sym_define_device_keyword] = ACTIONS(1565),
    [sym_define_combine_keyword] = ACTIONS(1565),
    [sym_define_constant_keyword] = ACTIONS(1565),
    [sym_define_type_keyword] = ACTIONS(1565),
    [sym_define_variable_keyword] = ACTIONS(1565),
    [sym_define_system_variable_keyword] = ACTIONS(1565),
    [sym_define_connect_level_keyword] = ACTIONS(1565),
    [sym_define_latching_keyword] = ACTIONS(1565),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1565),
    [sym_define_toggling_keyword] = ACTIONS(1565),
    [sym_define_call_keyword] = ACTIONS(1565),
    [sym_define_function_keyword] = ACTIONS(1565),
    [sym_define_library_function_keyword] = ACTIONS(1565),
    [sym_define_start_keyword] = ACTIONS(1565),
    [sym_define_event_keyword] = ACTIONS(1565),
    [sym_define_program_keyword] = ACTIONS(1565),
    [sym_define_module_keyword] = ACTIONS(1565),
    [sym_char_keyword] = ACTIONS(1565),
    [sym_widechar_keyword] = ACTIONS(1565),
    [sym_integer_keyword] = ACTIONS(1565),
    [sym_sinteger_keyword] = ACTIONS(1565),
    [sym_long_keyword] = ACTIONS(1565),
    [sym_slong_keyword] = ACTIONS(1565),
    [sym_float_keyword] = ACTIONS(1565),
    [sym_double_keyword] = ACTIONS(1565),
    [sym_variant_keyword] = ACTIONS(1565),
    [sym_variantarray_keyword] = ACTIONS(1565),
    [sym_local_var_keyword] = ACTIONS(1565),
    [sym_stack_var_keyword] = ACTIONS(1565),
    [sym_constant_keyword] = ACTIONS(1565),
    [sym_volatile_keyword] = ACTIONS(1565),
    [sym_non_volatile_keyword] = ACTIONS(1565),
    [sym_persistent_keyword] = ACTIONS(1565),
    [sym_dev_keyword] = ACTIONS(1565),
    [sym_devlev_keyword] = ACTIONS(1565),
    [sym_devchan_keyword] = ACTIONS(1565),
    [sym_if_keyword] = ACTIONS(1565),
    [sym_else_keyword] = ACTIONS(1565),
    [sym_switch_keyword] = ACTIONS(1565),
    [sym_case_keyword] = ACTIONS(1565),
    [sym_default_keyword] = ACTIONS(1565),
    [sym_while_keyword] = ACTIONS(1565),
    [sym_for_keyword] = ACTIONS(1565),
    [sym_break_keyword] = ACTIONS(1565),
    [sym_continue_keyword] = ACTIONS(1565),
    [sym_return_keyword] = ACTIONS(1565),
    [sym_select_keyword] = ACTIONS(1565),
    [sym_active_keyword] = ACTIONS(1565),
    [sym_struct_keyword] = ACTIONS(1565),
    [sym_structure_keyword] = ACTIONS(1565),
    [sym_button_event_keyword] = ACTIONS(1565),
    [sym_channel_event_keyword] = ACTIONS(1565),
    [sym_level_event_keyword] = ACTIONS(1565),
    [sym_data_event_keyword] = ACTIONS(1565),
    [sym_timeline_event_keyword] = ACTIONS(1565),
    [sym_custom_event_keyword] = ACTIONS(1565),
    [sym_push_keyword] = ACTIONS(1565),
    [sym_release_keyword] = ACTIONS(1565),
    [aux_sym_on_keyword_token1] = ACTIONS(1565),
    [aux_sym_off_keyword_token1] = ACTIONS(1565),
    [sym_send_level_keyword] = ACTIONS(1565),
    [sym_send_string_keyword] = ACTIONS(1565),
    [sym_send_command_keyword] = ACTIONS(1565),
    [sym_clear_buffer_keyword] = ACTIONS(1565),
    [sym_create_buffer_keyword] = ACTIONS(1565),
    [sym_create_multi_buffer_keyword] = ACTIONS(1565),
    [sym_call_keyword] = ACTIONS(1565),
    [sym_system_call_keyword] = ACTIONS(1565),
    [sym_devchan_to_keyword] = ACTIONS(1565),
    [sym_devchan_min_to_keyword] = ACTIONS(1565),
    [sym_devchan_total_off_keyword] = ACTIONS(1565),
    [sym_devchan_pulse_keyword] = ACTIONS(1565),
    [sym_wait_keyword] = ACTIONS(1565),
    [sym_cancel_wait_keyword] = ACTIONS(1565),
    [sym_wait_until_keyword] = ACTIONS(1565),
    [sym_cancel_wait_until_keyword] = ACTIONS(1565),
    [sym_cancel_all_wait_keyword] = ACTIONS(1565),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1565),
    [sym_and] = ACTIONS(1565),
    [sym_or] = ACTIONS(1565),
    [sym_not] = ACTIONS(1565),
    [sym_xor] = ACTIONS(1565),
    [sym_band] = ACTIONS(1565),
    [sym_bor] = ACTIONS(1565),
    [sym_bxor] = ACTIONS(1565),
    [sym_bnot] = ACTIONS(1565),
    [sym_lshift] = ACTIONS(1565),
    [sym_rshift] = ACTIONS(1565),
    [sym___file__] = ACTIONS(1565),
    [sym___date__] = ACTIONS(1565),
    [sym___ldate__] = ACTIONS(1565),
    [sym___time__] = ACTIONS(1565),
    [sym___line__] = ACTIONS(1565),
    [sym___name__] = ACTIONS(1565),
    [sym___netlinx__] = ACTIONS(1565),
    [sym_netlinx_axi_version] = ACTIONS(1565),
    [sym_url_flg_tcp] = ACTIONS(1565),
    [sym_url_flg_accinfopresent] = ACTIONS(1565),
    [sym_url_flg_temp] = ACTIONS(1565),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1565),
    [sym_url_flg_stat_mask] = ACTIONS(1565),
    [sym_url_flg_stat_lookup] = ACTIONS(1565),
    [sym_url_flg_stat_connecting] = ACTIONS(1565),
    [sym_url_flg_stat_waiting] = ACTIONS(1565),
    [sym_url_flg_stat_connected] = ACTIONS(1565),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1565),
    [sym_first_virtual_device] = ACTIONS(1565),
    [sym_first_local_port] = ACTIONS(1565),
    [sym_dynamic_virtual_device] = ACTIONS(1565),
    [sym_timeline_once] = ACTIONS(1565),
    [sym_timeline_repeat] = ACTIONS(1565),
    [sym_timeline_absolute] = ACTIONS(1565),
    [sym_timeline_relative] = ACTIONS(1565),
    [sym_do_push_timed_infinite] = ACTIONS(1565),
    [sym_source_type_no_address] = ACTIONS(1565),
    [sym_source_type_neuron_id] = ACTIONS(1565),
    [sym_source_type_ip_address] = ACTIONS(1565),
    [sym_source_type_axlink] = ACTIONS(1565),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1565),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1565),
    [sym_source_type_ip_socket_address] = ACTIONS(1565),
    [sym_source_type_rs232] = ACTIONS(1565),
    [sym_source_type_internal] = ACTIONS(1565),
    [sym_source_type_mac_address] = ACTIONS(1565),
    [sym_source_type_ipv4_port] = ACTIONS(1565),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1565),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1565),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1565),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1565),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1565),
    [sym_true] = ACTIONS(1565),
    [sym_false] = ACTIONS(1565),
    [sym_file_read_only] = ACTIONS(1565),
    [sym_file_rw_new] = ACTIONS(1565),
    [sym_file_rw_append] = ACTIONS(1565),
    [sym_ip_tcp] = ACTIONS(1565),
    [sym_ip_udp] = ACTIONS(1565),
    [sym_ip_udp_2way] = ACTIONS(1565),
    [sym_xml_encode_types] = ACTIONS(1565),
    [sym_xml_encode_char_as_list] = ACTIONS(1565),
    [sym_xml_encode_le] = ACTIONS(1565),
    [sym_xml_decode_types] = ACTIONS(1565),
    [sym_xml_decode_no_preserve] = ACTIONS(1565),
    [sym_duet_dev_type_display_device] = ACTIONS(1565),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1565),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1565),
    [sym_duet_dev_type_io_device] = ACTIONS(1565),
    [sym_duet_dev_type_relay_device] = ACTIONS(1565),
    [sym_duet_dev_type_ups] = ACTIONS(1565),
    [sym_duet_dev_type_amplifier] = ACTIONS(1565),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1565),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1565),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1565),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1565),
    [sym_duet_dev_type_camera] = ACTIONS(1565),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1565),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1565),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1565),
    [sym_duet_dev_type_dss] = ACTIONS(1565),
    [sym_duet_dev_type_dvr] = ACTIONS(1565),
    [sym_duet_dev_type_disc_device] = ACTIONS(1565),
    [sym_duet_dev_type_document_camera] = ACTIONS(1565),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1565),
    [sym_duet_dev_type_hvac] = ACTIONS(1565),
    [sym_duet_dev_type_keypad] = ACTIONS(1565),
    [sym_duet_dev_type_light] = ACTIONS(1565),
    [sym_duet_dev_type_monitor] = ACTIONS(1565),
    [sym_duet_dev_type_motor] = ACTIONS(1565),
    [sym_duet_dev_type_multi_window] = ACTIONS(1565),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1565),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1565),
    [sym_duet_dev_type_receiver] = ACTIONS(1565),
    [sym_duet_dev_type_security_system] = ACTIONS(1565),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1565),
    [sym_duet_dev_type_settop_box] = ACTIONS(1565),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1565),
    [sym_duet_dev_type_switcher] = ACTIONS(1565),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1565),
    [sym_duet_dev_type_tv] = ACTIONS(1565),
    [sym_duet_dev_type_utility] = ACTIONS(1565),
    [sym_duet_dev_type_vcr] = ACTIONS(1565),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1565),
    [sym_duet_dev_type_video_processor] = ACTIONS(1565),
    [sym_duet_dev_type_video_projector] = ACTIONS(1565),
    [sym_duet_dev_type_video_wall] = ACTIONS(1565),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1565),
    [sym_duet_dev_type_weather] = ACTIONS(1565),
    [sym_duet_dev_not_polled] = ACTIONS(1565),
    [sym_duet_dev_polled] = ACTIONS(1565),
    [sym_clkmgr_mode_network] = ACTIONS(1565),
    [sym_clkmgr_mode_standalone] = ACTIONS(1565),
    [sym_normal_standby] = ACTIONS(1565),
    [sym_normal_wake] = ACTIONS(1565),
    [sym_ip_multicast_ttl_option] = ACTIONS(1565),
    [sym_ip_ttl_subnet] = ACTIONS(1565),
    [sym_ip_ttl_site] = ACTIONS(1565),
    [sym_ip_ttl_region] = ACTIONS(1565),
    [sym_ip_ttl_continent] = ACTIONS(1565),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1565),
    [sym_ip_nodelay_on] = ACTIONS(1565),
    [sym_ip_nodelay_off] = ACTIONS(1565),
    [sym_amx_error] = ACTIONS(1565),
    [sym_amx_warning] = ACTIONS(1565),
    [sym_amx_info] = ACTIONS(1565),
    [sym_amx_debug] = ACTIONS(1565),
    [sym_netlinx_login_success] = ACTIONS(1565),
    [sym_netlinx_login_fail] = ACTIONS(1565),
    [sym_netlinx_logout] = ACTIONS(1565),
    [sym_null_str] = ACTIONS(1565),
    [sym_smtp_address] = ACTIONS(1565),
    [sym_smtp_port_number] = ACTIONS(1565),
    [sym_smtp_username] = ACTIONS(1565),
    [sym_smtp_password] = ACTIONS(1565),
    [sym_smtp_require_tls] = ACTIONS(1565),
    [sym_smtp_from] = ACTIONS(1565),
    [sym_smtp_tls_true] = ACTIONS(1565),
    [sym_smtp_tls_false] = ACTIONS(1565),
    [sym_listview_on_row_select_event] = ACTIONS(1565),
    [sym_data_structure_unknown] = ACTIONS(1565),
    [sym_data_structure_datafeed] = ACTIONS(1565),
    [sym_data_structure_datarecordset] = ACTIONS(1565),
    [sym_data_structure_datarecord] = ACTIONS(1565),
    [sym_data_structure_datafield] = ACTIONS(1565),
    [sym_data_max_content_field_count] = ACTIONS(1565),
    [sym_data_max_metadata_field_count] = ACTIONS(1565),
    [sym_data_max_id_length] = ACTIONS(1565),
    [sym_data_max_name_length] = ACTIONS(1565),
    [sym_data_max_description_length] = ACTIONS(1565),
    [sym_data_max_type_length] = ACTIONS(1565),
    [sym_data_max_format_length] = ACTIONS(1565),
    [sym_data_max_source_length] = ACTIONS(1565),
    [sym_data_max_value_length] = ACTIONS(1565),
    [sym_data_max_label_length] = ACTIONS(1565),
    [sym_data_max_url_length] = ACTIONS(1565),
    [sym_max_last_login_info_length] = ACTIONS(1565),
    [sym_data_type_unknown] = ACTIONS(1565),
    [sym_data_type_string] = ACTIONS(1565),
    [sym_data_type_datetime] = ACTIONS(1565),
    [sym_data_type_date] = ACTIONS(1565),
    [sym_data_type_time] = ACTIONS(1565),
    [sym_data_type_image] = ACTIONS(1565),
    [sym_data_format_url] = ACTIONS(1565),
    [sym_data_format_phone] = ACTIONS(1565),
    [sym_data_format_email] = ACTIONS(1565),
    [sym_data_format_iso8601] = ACTIONS(1565),
    [sym_auditlog_priv] = ACTIONS(1565),
    [sym_configuration_priv] = ACTIONS(1565),
    [sym_deviceconfig_priv] = ACTIONS(1565),
    [sym_ftp_priv] = ACTIONS(1565),
    [sym_http_priv] = ACTIONS(1565),
    [sym_networkconfig_priv] = ACTIONS(1565),
    [sym_programport_priv] = ACTIONS(1565),
    [sym_remoteui_priv] = ACTIONS(1565),
    [sym_securitycontrol_priv] = ACTIONS(1565),
    [sym_softwaremanagement_priv] = ACTIONS(1565),
    [sym_terminal_priv] = ACTIONS(1565),
    [sym_tpadmin_priv] = ACTIONS(1565),
    [sym_user1_priv] = ACTIONS(1565),
    [sym_user2_priv] = ACTIONS(1565),
    [sym_user3_priv] = ACTIONS(1565),
    [sym_user4_priv] = ACTIONS(1565),
    [sym_usermanagement_priv] = ACTIONS(1565),
    [sym_valid_account] = ACTIONS(1565),
    [sym_err_name_invalid] = ACTIONS(1565),
    [sym_err_password_invalid] = ACTIONS(1565),
    [sym_err_invalid_account] = ACTIONS(1565),
    [sym_err_no_authorization] = ACTIONS(1565),
    [sym_err_invalid_parameter] = ACTIONS(1565),
    [sym_err_user_locked_out] = ACTIONS(1565),
    [sym_err_user_account_expired] = ACTIONS(1565),
    [sym_err_general_error] = ACTIONS(1565),
    [sym_err_authentication_error] = ACTIONS(1565),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1565),
    [sym_tls_validate_certificate] = ACTIONS(1565),
    [sym_tdata] = ACTIONS(1565),
    [sym_tchannel] = ACTIONS(1565),
    [sym_tlevel] = ACTIONS(1565),
    [sym_tbutton] = ACTIONS(1565),
    [sym_ttimeline] = ACTIONS(1565),
    [sym_tcustom] = ACTIONS(1565),
    [sym_url_struct] = ACTIONS(1565),
    [sym_dns_struct] = ACTIONS(1565),
    [sym_ip_address_struct] = ACTIONS(1565),
    [sym_dev_info_struct] = ACTIONS(1565),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1565),
    [sym_data_feed] = ACTIONS(1565),
    [sym_data_field] = ACTIONS(1565),
    [sym_data_record] = ACTIONS(1565),
    [sym_wc_data_feed] = ACTIONS(1565),
    [sym_wc_data_field] = ACTIONS(1565),
    [sym_wc_data_record] = ACTIONS(1565),
    [sym_last_login_info] = ACTIONS(1565),
    [sym____reserved___] = ACTIONS(1565),
    [sym_date] = ACTIONS(1565),
    [sym_day] = ACTIONS(1565),
    [sym_time] = ACTIONS(1565),
    [sym_ldate] = ACTIONS(1565),
    [sym_push_device] = ACTIONS(1565),
    [sym_push_channel] = ACTIONS(1565),
    [sym_push_devchan] = ACTIONS(1565),
    [sym_release_device] = ACTIONS(1565),
    [sym_release_channel] = ACTIONS(1565),
    [sym_release_devchan] = ACTIONS(1565),
    [sym_master_slot] = ACTIONS(1565),
    [sym_get_pulse_time] = ACTIONS(1565),
    [sym_get_timer] = ACTIONS(1565),
    [sym_master_sn] = ACTIONS(1565),
    [sym_system_number] = ACTIONS(1565),
    [sym_button] = ACTIONS(1565),
    [sym_dv_channel] = ACTIONS(1565),
    [sym_data] = ACTIONS(1565),
    [sym_level] = ACTIONS(1565),
    [sym_timeline] = ACTIONS(1565),
    [sym_channel] = ACTIONS(1565),
    [sym_custom] = ACTIONS(1565),
    [sym_atoi] = ACTIONS(1565),
    [sym_atol] = ACTIONS(1565),
    [sym_atof] = ACTIONS(1565),
    [sym_itoa] = ACTIONS(1565),
    [sym_itohex] = ACTIONS(1565),
    [sym_hextoi] = ACTIONS(1565),
    [sym_ftoa] = ACTIONS(1565),
    [sym_find_string] = ACTIONS(1565),
    [sym_left_string] = ACTIONS(1565),
    [sym_right_string] = ACTIONS(1565),
    [sym_mid_string] = ACTIONS(1565),
    [sym_remove_string] = ACTIONS(1565),
    [sym_get_buffer_string] = ACTIONS(1565),
    [sym_get_buffer_char] = ACTIONS(1565),
    [sym_get_multi_buffer_string] = ACTIONS(1565),
    [sym_length_string] = ACTIONS(1565),
    [sym_max_length_string] = ACTIONS(1565),
    [sym_set_length_string] = ACTIONS(1565),
    [sym_compare_string] = ACTIONS(1565),
    [sym_upper_string] = ACTIONS(1565),
    [sym_lower_string] = ACTIONS(1565),
    [sym_get_last] = ACTIONS(1565),
    [sym_date_to_day] = ACTIONS(1565),
    [sym_date_to_month] = ACTIONS(1565),
    [sym_date_to_year] = ACTIONS(1565),
    [sym_time_to_hour] = ACTIONS(1565),
    [sym_time_to_minute] = ACTIONS(1565),
    [sym_time_to_second] = ACTIONS(1565),
    [sym_day_of_week] = ACTIONS(1565),
    [sym_set_pulse_time] = ACTIONS(1565),
    [sym_set_timer] = ACTIONS(1565),
    [sym_file_open] = ACTIONS(1565),
    [sym_file_close] = ACTIONS(1565),
    [sym_file_read] = ACTIONS(1565),
    [sym_file_read_line] = ACTIONS(1565),
    [sym_file_write] = ACTIONS(1565),
    [sym_file_write_line] = ACTIONS(1565),
    [sym_file_copy] = ACTIONS(1565),
    [sym_file_delete] = ACTIONS(1565),
    [sym_file_rename] = ACTIONS(1565),
    [sym_file_dir] = ACTIONS(1565),
    [sym_file_getdir] = ACTIONS(1565),
    [sym_file_setdir] = ACTIONS(1565),
    [sym_file_create_dir] = ACTIONS(1565),
    [sym_file_remove_dir] = ACTIONS(1565),
    [sym_file_seek] = ACTIONS(1565),
    [sym_length_array] = ACTIONS(1565),
    [sym_max_length_array] = ACTIONS(1565),
    [sym_set_length_array] = ACTIONS(1565),
    [sym_ip_client_open] = ACTIONS(1565),
    [sym_ip_client_close] = ACTIONS(1565),
    [sym_ip_server_open] = ACTIONS(1565),
    [sym_ip_server_close] = ACTIONS(1565),
    [sym_ip_bound_client_open] = ACTIONS(1565),
    [sym_ip_mc_server_open] = ACTIONS(1565),
    [sym_ip_set_option] = ACTIONS(1565),
    [sym_ssh_client_open] = ACTIONS(1565),
    [sym_ssh_client_close] = ACTIONS(1565),
    [sym_tls_client_open] = ACTIONS(1565),
    [sym_tls_client_close] = ACTIONS(1565),
    [sym_do_push] = ACTIONS(1565),
    [sym_do_push_timed] = ACTIONS(1565),
    [sym_do_release] = ACTIONS(1565),
    [sym_device_id] = ACTIONS(1565),
    [sym_device_info] = ACTIONS(1565),
    [sym_device_id_string] = ACTIONS(1565),
    [sym_device_standby] = ACTIONS(1565),
    [sym_device_wake] = ACTIONS(1565),
    [sym_timeline_create] = ACTIONS(1565),
    [sym_timeline_kill] = ACTIONS(1565),
    [sym_timeline_reload] = ACTIONS(1565),
    [sym_timeline_pause] = ACTIONS(1565),
    [sym_timeline_restart] = ACTIONS(1565),
    [sym_timeline_set] = ACTIONS(1565),
    [sym_timeline_get] = ACTIONS(1565),
    [sym_timeline_active] = ACTIONS(1565),
    [sym_timeline_dynamic_id] = ACTIONS(1565),
    [sym_variable_to_string] = ACTIONS(1565),
    [sym_string_to_variable] = ACTIONS(1565),
    [sym_length_variable_to_string] = ACTIONS(1565),
    [sym_type_cast] = ACTIONS(1565),
    [sym_variable_to_xml] = ACTIONS(1565),
    [sym_xml_to_variable] = ACTIONS(1565),
    [sym_length_variable_to_xml] = ACTIONS(1565),
    [sym_redirect_string] = ACTIONS(1565),
    [sym_random_number] = ACTIONS(1565),
    [sym_get_unique_id] = ACTIONS(1565),
    [sym_get_system_number] = ACTIONS(1565),
    [sym_set_system_number] = ACTIONS(1565),
    [sym_get_serial_number] = ACTIONS(1565),
    [sym_get_master_build] = ACTIONS(1565),
    [sym_set_validation_code] = ACTIONS(1565),
    [sym_reboot] = ACTIONS(1565),
    [sym_load_duet_module] = ACTIONS(1565),
    [sym_unload_duet_module] = ACTIONS(1565),
    [sym_duet_mem_size_set] = ACTIONS(1565),
    [sym_duet_mem_size_get] = ACTIONS(1565),
    [sym_dynamic_application_device] = ACTIONS(1565),
    [sym_dynamic_polled_port] = ACTIONS(1565),
    [sym_static_port_binding] = ACTIONS(1565),
    [sym_static_ip_binding] = ACTIONS(1565),
    [sym_get_dns_list] = ACTIONS(1565),
    [sym_set_dns_list] = ACTIONS(1565),
    [sym_get_ip_address] = ACTIONS(1565),
    [sym_set_ip_address] = ACTIONS(1565),
    [sym_get_url_list] = ACTIONS(1565),
    [sym_add_url_entry] = ACTIONS(1565),
    [sym_delete_url_entry] = ACTIONS(1565),
    [sym_abs_value] = ACTIONS(1565),
    [sym_max_value] = ACTIONS(1565),
    [sym_min_value] = ACTIONS(1565),
    [sym_exp_value] = ACTIONS(1565),
    [sym_log_value] = ACTIONS(1565),
    [sym_log10_value] = ACTIONS(1565),
    [sym_power_value] = ACTIONS(1565),
    [sym_sqrt_value] = ACTIONS(1565),
    [sym_raw_be] = ACTIONS(1565),
    [sym_raw_le] = ACTIONS(1565),
    [sym_internal_queue_size_set] = ACTIONS(1565),
    [sym_internal_queue_size_get] = ACTIONS(1565),
    [sym_internal_threshold_set] = ACTIONS(1565),
    [sym_internal_threshold_get] = ACTIONS(1565),
    [sym_rebuilt_event] = ACTIONS(1565),
    [sym_do_custom_event] = ACTIONS(1565),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1565),
    [sym_clkmgr_set_clk_source] = ACTIONS(1565),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1565),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1565),
    [sym_clkmgr_get_timezone] = ACTIONS(1565),
    [sym_clkmgr_set_timezone] = ACTIONS(1565),
    [sym_clkmgr_get_resync_period] = ACTIONS(1565),
    [sym_clkmgr_set_resync_period] = ACTIONS(1565),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1565),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1565),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1565),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1565),
    [sym_clkmgr_get_timeservers] = ACTIONS(1565),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1565),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1565),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1565),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1565),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1565),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1565),
    [sym_set_log_level] = ACTIONS(1565),
    [sym_get_log_level] = ACTIONS(1565),
    [sym_amx_log] = ACTIONS(1565),
    [sym_smtp_server_config_set] = ACTIONS(1565),
    [sym_smtp_server_config_get] = ACTIONS(1565),
    [sym_smtp_send] = ACTIONS(1565),
    [sym_validate_netlinx_account] = ACTIONS(1565),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1565),
    [sym_authenticate_certificate] = ACTIONS(1565),
    [sym_audit_netlinx_session_event] = ACTIONS(1565),
    [sym_audit_netlinx_generic_event] = ACTIONS(1565),
    [sym_get_available_flash_disk_space] = ACTIONS(1565),
    [sym_get_max_flash_disk_space] = ACTIONS(1565),
    [sym_data_create_feed] = ACTIONS(1565),
    [sym_data_delete_feed] = ACTIONS(1565),
    [sym_data_publish_feed] = ACTIONS(1565),
    [sym_data_get_published_feed] = ACTIONS(1565),
    [sym_data_add_record] = ACTIONS(1565),
    [sym_data_get_event_record] = ACTIONS(1565),
    [sym__wc_data_create_feed] = ACTIONS(1565),
    [sym__wc_data_add_record] = ACTIONS(1565),
    [sym__wc_data_get_event_record] = ACTIONS(1565),
    [sym_format] = ACTIONS(1565),
    [sym_astro_clock] = ACTIONS(1565),
    [sym_set_outdoor_temperature] = ACTIONS(1565),
    [sym_set_virtual_level_count] = ACTIONS(1565),
    [sym_set_virtual_channel_count] = ACTIONS(1565),
    [sym_set_virtual_port_count] = ACTIONS(1565),
    [sym__wc_encode_int] = ACTIONS(1565),
    [sym__wc_decode_int] = ACTIONS(1565),
    [sym__wc_int] = ACTIONS(1565),
    [sym__wc_to_ch_int] = ACTIONS(1565),
    [sym__ch_to_wc_int] = ACTIONS(1565),
    [sym__wc_find_string_int] = ACTIONS(1565),
    [sym__wc_left_string_int] = ACTIONS(1565),
    [sym__wc_lower_string_int] = ACTIONS(1565),
    [sym__wc_mid_string_int] = ACTIONS(1565),
    [sym__wc_remove_string_int] = ACTIONS(1565),
    [sym__wc_right_string_int] = ACTIONS(1565),
    [sym__wc_upper_string_int] = ACTIONS(1565),
    [sym__wc_compare_string_int] = ACTIONS(1565),
    [sym__wc_get_buffer_char_int] = ACTIONS(1565),
    [sym__wc_get_buffer_string_int] = ACTIONS(1565),
    [sym__wc_concat_string_int] = ACTIONS(1565),
    [sym__wc_file_open_int] = ACTIONS(1565),
    [sym__wc_file_close_int] = ACTIONS(1565),
    [sym__wc_file_read_int] = ACTIONS(1565),
    [sym__wc_file_read_line_int] = ACTIONS(1565),
    [sym__wc_file_write_int] = ACTIONS(1565),
    [sym__wc_file_write_line_int] = ACTIONS(1565),
    [aux_sym_preproc_include_token1] = ACTIONS(1565),
    [aux_sym_preproc_define_token1] = ACTIONS(1567),
    [aux_sym_preproc_warn_token1] = ACTIONS(1567),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1567),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1567),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1567),
    [anon_sym_LBRACK] = ACTIONS(1567),
    [anon_sym_LBRACE] = ACTIONS(1567),
    [anon_sym_COLON] = ACTIONS(1567),
    [anon_sym_LPAREN] = ACTIONS(1565),
    [anon_sym_SEMI] = ACTIONS(1567),
    [anon_sym_BANG] = ACTIONS(1565),
    [anon_sym_TILDE] = ACTIONS(1567),
    [anon_sym_DASH] = ACTIONS(1565),
    [anon_sym_PLUS] = ACTIONS(1565),
    [anon_sym_STAR] = ACTIONS(1567),
    [anon_sym_SLASH] = ACTIONS(1565),
    [anon_sym_PERCENT] = ACTIONS(1567),
    [anon_sym_PIPE_PIPE] = ACTIONS(1567),
    [anon_sym_AMP_AMP] = ACTIONS(1567),
    [anon_sym_PIPE] = ACTIONS(1565),
    [anon_sym_CARET] = ACTIONS(1567),
    [anon_sym_AMP] = ACTIONS(1565),
    [anon_sym_EQ_EQ] = ACTIONS(1567),
    [anon_sym_BANG_EQ] = ACTIONS(1567),
    [anon_sym_LT_GT] = ACTIONS(1567),
    [anon_sym_GT] = ACTIONS(1565),
    [anon_sym_GT_EQ] = ACTIONS(1567),
    [anon_sym_LT_EQ] = ACTIONS(1567),
    [anon_sym_LT] = ACTIONS(1565),
    [anon_sym_LT_LT] = ACTIONS(1567),
    [anon_sym_GT_GT] = ACTIONS(1567),
    [anon_sym_DASH_DASH] = ACTIONS(1567),
    [anon_sym_PLUS_PLUS] = ACTIONS(1567),
    [anon_sym_DOT] = ACTIONS(1565),
    [anon_sym_DQUOTE] = ACTIONS(1567),
    [anon_sym_SQUOTE] = ACTIONS(1567),
    [sym_number_literal] = ACTIONS(1567),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1567),
  },
  [STATE(141)] = {
    [ts_builtin_sym_end] = ACTIONS(1595),
    [sym_identifier] = ACTIONS(1593),
    [sym_program_name_keyword] = ACTIONS(1593),
    [sym_module_name_keyword] = ACTIONS(1593),
    [sym_define_device_keyword] = ACTIONS(1593),
    [sym_define_combine_keyword] = ACTIONS(1593),
    [sym_define_constant_keyword] = ACTIONS(1593),
    [sym_define_type_keyword] = ACTIONS(1593),
    [sym_define_variable_keyword] = ACTIONS(1593),
    [sym_define_system_variable_keyword] = ACTIONS(1593),
    [sym_define_connect_level_keyword] = ACTIONS(1593),
    [sym_define_latching_keyword] = ACTIONS(1593),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1593),
    [sym_define_toggling_keyword] = ACTIONS(1593),
    [sym_define_call_keyword] = ACTIONS(1593),
    [sym_define_function_keyword] = ACTIONS(1593),
    [sym_define_library_function_keyword] = ACTIONS(1593),
    [sym_define_start_keyword] = ACTIONS(1593),
    [sym_define_event_keyword] = ACTIONS(1593),
    [sym_define_program_keyword] = ACTIONS(1593),
    [sym_define_module_keyword] = ACTIONS(1593),
    [sym_char_keyword] = ACTIONS(1593),
    [sym_widechar_keyword] = ACTIONS(1593),
    [sym_integer_keyword] = ACTIONS(1593),
    [sym_sinteger_keyword] = ACTIONS(1593),
    [sym_long_keyword] = ACTIONS(1593),
    [sym_slong_keyword] = ACTIONS(1593),
    [sym_float_keyword] = ACTIONS(1593),
    [sym_double_keyword] = ACTIONS(1593),
    [sym_variant_keyword] = ACTIONS(1593),
    [sym_variantarray_keyword] = ACTIONS(1593),
    [sym_local_var_keyword] = ACTIONS(1593),
    [sym_stack_var_keyword] = ACTIONS(1593),
    [sym_constant_keyword] = ACTIONS(1593),
    [sym_volatile_keyword] = ACTIONS(1593),
    [sym_non_volatile_keyword] = ACTIONS(1593),
    [sym_persistent_keyword] = ACTIONS(1593),
    [sym_dev_keyword] = ACTIONS(1593),
    [sym_devlev_keyword] = ACTIONS(1593),
    [sym_devchan_keyword] = ACTIONS(1593),
    [sym_if_keyword] = ACTIONS(1593),
    [sym_else_keyword] = ACTIONS(1593),
    [sym_switch_keyword] = ACTIONS(1593),
    [sym_case_keyword] = ACTIONS(1593),
    [sym_default_keyword] = ACTIONS(1593),
    [sym_while_keyword] = ACTIONS(1593),
    [sym_for_keyword] = ACTIONS(1593),
    [sym_break_keyword] = ACTIONS(1593),
    [sym_continue_keyword] = ACTIONS(1593),
    [sym_return_keyword] = ACTIONS(1593),
    [sym_select_keyword] = ACTIONS(1593),
    [sym_active_keyword] = ACTIONS(1593),
    [sym_struct_keyword] = ACTIONS(1593),
    [sym_structure_keyword] = ACTIONS(1593),
    [sym_button_event_keyword] = ACTIONS(1593),
    [sym_channel_event_keyword] = ACTIONS(1593),
    [sym_level_event_keyword] = ACTIONS(1593),
    [sym_data_event_keyword] = ACTIONS(1593),
    [sym_timeline_event_keyword] = ACTIONS(1593),
    [sym_custom_event_keyword] = ACTIONS(1593),
    [sym_push_keyword] = ACTIONS(1593),
    [sym_release_keyword] = ACTIONS(1593),
    [aux_sym_on_keyword_token1] = ACTIONS(1593),
    [aux_sym_off_keyword_token1] = ACTIONS(1593),
    [sym_send_level_keyword] = ACTIONS(1593),
    [sym_send_string_keyword] = ACTIONS(1593),
    [sym_send_command_keyword] = ACTIONS(1593),
    [sym_clear_buffer_keyword] = ACTIONS(1593),
    [sym_create_buffer_keyword] = ACTIONS(1593),
    [sym_create_multi_buffer_keyword] = ACTIONS(1593),
    [sym_call_keyword] = ACTIONS(1593),
    [sym_system_call_keyword] = ACTIONS(1593),
    [sym_devchan_to_keyword] = ACTIONS(1593),
    [sym_devchan_min_to_keyword] = ACTIONS(1593),
    [sym_devchan_total_off_keyword] = ACTIONS(1593),
    [sym_devchan_pulse_keyword] = ACTIONS(1593),
    [sym_wait_keyword] = ACTIONS(1593),
    [sym_cancel_wait_keyword] = ACTIONS(1593),
    [sym_wait_until_keyword] = ACTIONS(1593),
    [sym_cancel_wait_until_keyword] = ACTIONS(1593),
    [sym_cancel_all_wait_keyword] = ACTIONS(1593),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1593),
    [sym_and] = ACTIONS(1593),
    [sym_or] = ACTIONS(1593),
    [sym_not] = ACTIONS(1593),
    [sym_xor] = ACTIONS(1593),
    [sym_band] = ACTIONS(1593),
    [sym_bor] = ACTIONS(1593),
    [sym_bxor] = ACTIONS(1593),
    [sym_bnot] = ACTIONS(1593),
    [sym_lshift] = ACTIONS(1593),
    [sym_rshift] = ACTIONS(1593),
    [sym___file__] = ACTIONS(1593),
    [sym___date__] = ACTIONS(1593),
    [sym___ldate__] = ACTIONS(1593),
    [sym___time__] = ACTIONS(1593),
    [sym___line__] = ACTIONS(1593),
    [sym___name__] = ACTIONS(1593),
    [sym___netlinx__] = ACTIONS(1593),
    [sym_netlinx_axi_version] = ACTIONS(1593),
    [sym_url_flg_tcp] = ACTIONS(1593),
    [sym_url_flg_accinfopresent] = ACTIONS(1593),
    [sym_url_flg_temp] = ACTIONS(1593),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1593),
    [sym_url_flg_stat_mask] = ACTIONS(1593),
    [sym_url_flg_stat_lookup] = ACTIONS(1593),
    [sym_url_flg_stat_connecting] = ACTIONS(1593),
    [sym_url_flg_stat_waiting] = ACTIONS(1593),
    [sym_url_flg_stat_connected] = ACTIONS(1593),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1593),
    [sym_first_virtual_device] = ACTIONS(1593),
    [sym_first_local_port] = ACTIONS(1593),
    [sym_dynamic_virtual_device] = ACTIONS(1593),
    [sym_timeline_once] = ACTIONS(1593),
    [sym_timeline_repeat] = ACTIONS(1593),
    [sym_timeline_absolute] = ACTIONS(1593),
    [sym_timeline_relative] = ACTIONS(1593),
    [sym_do_push_timed_infinite] = ACTIONS(1593),
    [sym_source_type_no_address] = ACTIONS(1593),
    [sym_source_type_neuron_id] = ACTIONS(1593),
    [sym_source_type_ip_address] = ACTIONS(1593),
    [sym_source_type_axlink] = ACTIONS(1593),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1593),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1593),
    [sym_source_type_ip_socket_address] = ACTIONS(1593),
    [sym_source_type_rs232] = ACTIONS(1593),
    [sym_source_type_internal] = ACTIONS(1593),
    [sym_source_type_mac_address] = ACTIONS(1593),
    [sym_source_type_ipv4_port] = ACTIONS(1593),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1593),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1593),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1593),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1593),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1593),
    [sym_true] = ACTIONS(1593),
    [sym_false] = ACTIONS(1593),
    [sym_file_read_only] = ACTIONS(1593),
    [sym_file_rw_new] = ACTIONS(1593),
    [sym_file_rw_append] = ACTIONS(1593),
    [sym_ip_tcp] = ACTIONS(1593),
    [sym_ip_udp] = ACTIONS(1593),
    [sym_ip_udp_2way] = ACTIONS(1593),
    [sym_xml_encode_types] = ACTIONS(1593),
    [sym_xml_encode_char_as_list] = ACTIONS(1593),
    [sym_xml_encode_le] = ACTIONS(1593),
    [sym_xml_decode_types] = ACTIONS(1593),
    [sym_xml_decode_no_preserve] = ACTIONS(1593),
    [sym_duet_dev_type_display_device] = ACTIONS(1593),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1593),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1593),
    [sym_duet_dev_type_io_device] = ACTIONS(1593),
    [sym_duet_dev_type_relay_device] = ACTIONS(1593),
    [sym_duet_dev_type_ups] = ACTIONS(1593),
    [sym_duet_dev_type_amplifier] = ACTIONS(1593),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1593),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1593),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1593),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1593),
    [sym_duet_dev_type_camera] = ACTIONS(1593),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1593),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1593),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1593),
    [sym_duet_dev_type_dss] = ACTIONS(1593),
    [sym_duet_dev_type_dvr] = ACTIONS(1593),
    [sym_duet_dev_type_disc_device] = ACTIONS(1593),
    [sym_duet_dev_type_document_camera] = ACTIONS(1593),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1593),
    [sym_duet_dev_type_hvac] = ACTIONS(1593),
    [sym_duet_dev_type_keypad] = ACTIONS(1593),
    [sym_duet_dev_type_light] = ACTIONS(1593),
    [sym_duet_dev_type_monitor] = ACTIONS(1593),
    [sym_duet_dev_type_motor] = ACTIONS(1593),
    [sym_duet_dev_type_multi_window] = ACTIONS(1593),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1593),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1593),
    [sym_duet_dev_type_receiver] = ACTIONS(1593),
    [sym_duet_dev_type_security_system] = ACTIONS(1593),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1593),
    [sym_duet_dev_type_settop_box] = ACTIONS(1593),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1593),
    [sym_duet_dev_type_switcher] = ACTIONS(1593),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1593),
    [sym_duet_dev_type_tv] = ACTIONS(1593),
    [sym_duet_dev_type_utility] = ACTIONS(1593),
    [sym_duet_dev_type_vcr] = ACTIONS(1593),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1593),
    [sym_duet_dev_type_video_processor] = ACTIONS(1593),
    [sym_duet_dev_type_video_projector] = ACTIONS(1593),
    [sym_duet_dev_type_video_wall] = ACTIONS(1593),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1593),
    [sym_duet_dev_type_weather] = ACTIONS(1593),
    [sym_duet_dev_not_polled] = ACTIONS(1593),
    [sym_duet_dev_polled] = ACTIONS(1593),
    [sym_clkmgr_mode_network] = ACTIONS(1593),
    [sym_clkmgr_mode_standalone] = ACTIONS(1593),
    [sym_normal_standby] = ACTIONS(1593),
    [sym_normal_wake] = ACTIONS(1593),
    [sym_ip_multicast_ttl_option] = ACTIONS(1593),
    [sym_ip_ttl_subnet] = ACTIONS(1593),
    [sym_ip_ttl_site] = ACTIONS(1593),
    [sym_ip_ttl_region] = ACTIONS(1593),
    [sym_ip_ttl_continent] = ACTIONS(1593),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1593),
    [sym_ip_nodelay_on] = ACTIONS(1593),
    [sym_ip_nodelay_off] = ACTIONS(1593),
    [sym_amx_error] = ACTIONS(1593),
    [sym_amx_warning] = ACTIONS(1593),
    [sym_amx_info] = ACTIONS(1593),
    [sym_amx_debug] = ACTIONS(1593),
    [sym_netlinx_login_success] = ACTIONS(1593),
    [sym_netlinx_login_fail] = ACTIONS(1593),
    [sym_netlinx_logout] = ACTIONS(1593),
    [sym_null_str] = ACTIONS(1593),
    [sym_smtp_address] = ACTIONS(1593),
    [sym_smtp_port_number] = ACTIONS(1593),
    [sym_smtp_username] = ACTIONS(1593),
    [sym_smtp_password] = ACTIONS(1593),
    [sym_smtp_require_tls] = ACTIONS(1593),
    [sym_smtp_from] = ACTIONS(1593),
    [sym_smtp_tls_true] = ACTIONS(1593),
    [sym_smtp_tls_false] = ACTIONS(1593),
    [sym_listview_on_row_select_event] = ACTIONS(1593),
    [sym_data_structure_unknown] = ACTIONS(1593),
    [sym_data_structure_datafeed] = ACTIONS(1593),
    [sym_data_structure_datarecordset] = ACTIONS(1593),
    [sym_data_structure_datarecord] = ACTIONS(1593),
    [sym_data_structure_datafield] = ACTIONS(1593),
    [sym_data_max_content_field_count] = ACTIONS(1593),
    [sym_data_max_metadata_field_count] = ACTIONS(1593),
    [sym_data_max_id_length] = ACTIONS(1593),
    [sym_data_max_name_length] = ACTIONS(1593),
    [sym_data_max_description_length] = ACTIONS(1593),
    [sym_data_max_type_length] = ACTIONS(1593),
    [sym_data_max_format_length] = ACTIONS(1593),
    [sym_data_max_source_length] = ACTIONS(1593),
    [sym_data_max_value_length] = ACTIONS(1593),
    [sym_data_max_label_length] = ACTIONS(1593),
    [sym_data_max_url_length] = ACTIONS(1593),
    [sym_max_last_login_info_length] = ACTIONS(1593),
    [sym_data_type_unknown] = ACTIONS(1593),
    [sym_data_type_string] = ACTIONS(1593),
    [sym_data_type_datetime] = ACTIONS(1593),
    [sym_data_type_date] = ACTIONS(1593),
    [sym_data_type_time] = ACTIONS(1593),
    [sym_data_type_image] = ACTIONS(1593),
    [sym_data_format_url] = ACTIONS(1593),
    [sym_data_format_phone] = ACTIONS(1593),
    [sym_data_format_email] = ACTIONS(1593),
    [sym_data_format_iso8601] = ACTIONS(1593),
    [sym_auditlog_priv] = ACTIONS(1593),
    [sym_configuration_priv] = ACTIONS(1593),
    [sym_deviceconfig_priv] = ACTIONS(1593),
    [sym_ftp_priv] = ACTIONS(1593),
    [sym_http_priv] = ACTIONS(1593),
    [sym_networkconfig_priv] = ACTIONS(1593),
    [sym_programport_priv] = ACTIONS(1593),
    [sym_remoteui_priv] = ACTIONS(1593),
    [sym_securitycontrol_priv] = ACTIONS(1593),
    [sym_softwaremanagement_priv] = ACTIONS(1593),
    [sym_terminal_priv] = ACTIONS(1593),
    [sym_tpadmin_priv] = ACTIONS(1593),
    [sym_user1_priv] = ACTIONS(1593),
    [sym_user2_priv] = ACTIONS(1593),
    [sym_user3_priv] = ACTIONS(1593),
    [sym_user4_priv] = ACTIONS(1593),
    [sym_usermanagement_priv] = ACTIONS(1593),
    [sym_valid_account] = ACTIONS(1593),
    [sym_err_name_invalid] = ACTIONS(1593),
    [sym_err_password_invalid] = ACTIONS(1593),
    [sym_err_invalid_account] = ACTIONS(1593),
    [sym_err_no_authorization] = ACTIONS(1593),
    [sym_err_invalid_parameter] = ACTIONS(1593),
    [sym_err_user_locked_out] = ACTIONS(1593),
    [sym_err_user_account_expired] = ACTIONS(1593),
    [sym_err_general_error] = ACTIONS(1593),
    [sym_err_authentication_error] = ACTIONS(1593),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1593),
    [sym_tls_validate_certificate] = ACTIONS(1593),
    [sym_tdata] = ACTIONS(1593),
    [sym_tchannel] = ACTIONS(1593),
    [sym_tlevel] = ACTIONS(1593),
    [sym_tbutton] = ACTIONS(1593),
    [sym_ttimeline] = ACTIONS(1593),
    [sym_tcustom] = ACTIONS(1593),
    [sym_url_struct] = ACTIONS(1593),
    [sym_dns_struct] = ACTIONS(1593),
    [sym_ip_address_struct] = ACTIONS(1593),
    [sym_dev_info_struct] = ACTIONS(1593),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1593),
    [sym_data_feed] = ACTIONS(1593),
    [sym_data_field] = ACTIONS(1593),
    [sym_data_record] = ACTIONS(1593),
    [sym_wc_data_feed] = ACTIONS(1593),
    [sym_wc_data_field] = ACTIONS(1593),
    [sym_wc_data_record] = ACTIONS(1593),
    [sym_last_login_info] = ACTIONS(1593),
    [sym____reserved___] = ACTIONS(1593),
    [sym_date] = ACTIONS(1593),
    [sym_day] = ACTIONS(1593),
    [sym_time] = ACTIONS(1593),
    [sym_ldate] = ACTIONS(1593),
    [sym_push_device] = ACTIONS(1593),
    [sym_push_channel] = ACTIONS(1593),
    [sym_push_devchan] = ACTIONS(1593),
    [sym_release_device] = ACTIONS(1593),
    [sym_release_channel] = ACTIONS(1593),
    [sym_release_devchan] = ACTIONS(1593),
    [sym_master_slot] = ACTIONS(1593),
    [sym_get_pulse_time] = ACTIONS(1593),
    [sym_get_timer] = ACTIONS(1593),
    [sym_master_sn] = ACTIONS(1593),
    [sym_system_number] = ACTIONS(1593),
    [sym_button] = ACTIONS(1593),
    [sym_dv_channel] = ACTIONS(1593),
    [sym_data] = ACTIONS(1593),
    [sym_level] = ACTIONS(1593),
    [sym_timeline] = ACTIONS(1593),
    [sym_channel] = ACTIONS(1593),
    [sym_custom] = ACTIONS(1593),
    [sym_atoi] = ACTIONS(1593),
    [sym_atol] = ACTIONS(1593),
    [sym_atof] = ACTIONS(1593),
    [sym_itoa] = ACTIONS(1593),
    [sym_itohex] = ACTIONS(1593),
    [sym_hextoi] = ACTIONS(1593),
    [sym_ftoa] = ACTIONS(1593),
    [sym_find_string] = ACTIONS(1593),
    [sym_left_string] = ACTIONS(1593),
    [sym_right_string] = ACTIONS(1593),
    [sym_mid_string] = ACTIONS(1593),
    [sym_remove_string] = ACTIONS(1593),
    [sym_get_buffer_string] = ACTIONS(1593),
    [sym_get_buffer_char] = ACTIONS(1593),
    [sym_get_multi_buffer_string] = ACTIONS(1593),
    [sym_length_string] = ACTIONS(1593),
    [sym_max_length_string] = ACTIONS(1593),
    [sym_set_length_string] = ACTIONS(1593),
    [sym_compare_string] = ACTIONS(1593),
    [sym_upper_string] = ACTIONS(1593),
    [sym_lower_string] = ACTIONS(1593),
    [sym_get_last] = ACTIONS(1593),
    [sym_date_to_day] = ACTIONS(1593),
    [sym_date_to_month] = ACTIONS(1593),
    [sym_date_to_year] = ACTIONS(1593),
    [sym_time_to_hour] = ACTIONS(1593),
    [sym_time_to_minute] = ACTIONS(1593),
    [sym_time_to_second] = ACTIONS(1593),
    [sym_day_of_week] = ACTIONS(1593),
    [sym_set_pulse_time] = ACTIONS(1593),
    [sym_set_timer] = ACTIONS(1593),
    [sym_file_open] = ACTIONS(1593),
    [sym_file_close] = ACTIONS(1593),
    [sym_file_read] = ACTIONS(1593),
    [sym_file_read_line] = ACTIONS(1593),
    [sym_file_write] = ACTIONS(1593),
    [sym_file_write_line] = ACTIONS(1593),
    [sym_file_copy] = ACTIONS(1593),
    [sym_file_delete] = ACTIONS(1593),
    [sym_file_rename] = ACTIONS(1593),
    [sym_file_dir] = ACTIONS(1593),
    [sym_file_getdir] = ACTIONS(1593),
    [sym_file_setdir] = ACTIONS(1593),
    [sym_file_create_dir] = ACTIONS(1593),
    [sym_file_remove_dir] = ACTIONS(1593),
    [sym_file_seek] = ACTIONS(1593),
    [sym_length_array] = ACTIONS(1593),
    [sym_max_length_array] = ACTIONS(1593),
    [sym_set_length_array] = ACTIONS(1593),
    [sym_ip_client_open] = ACTIONS(1593),
    [sym_ip_client_close] = ACTIONS(1593),
    [sym_ip_server_open] = ACTIONS(1593),
    [sym_ip_server_close] = ACTIONS(1593),
    [sym_ip_bound_client_open] = ACTIONS(1593),
    [sym_ip_mc_server_open] = ACTIONS(1593),
    [sym_ip_set_option] = ACTIONS(1593),
    [sym_ssh_client_open] = ACTIONS(1593),
    [sym_ssh_client_close] = ACTIONS(1593),
    [sym_tls_client_open] = ACTIONS(1593),
    [sym_tls_client_close] = ACTIONS(1593),
    [sym_do_push] = ACTIONS(1593),
    [sym_do_push_timed] = ACTIONS(1593),
    [sym_do_release] = ACTIONS(1593),
    [sym_device_id] = ACTIONS(1593),
    [sym_device_info] = ACTIONS(1593),
    [sym_device_id_string] = ACTIONS(1593),
    [sym_device_standby] = ACTIONS(1593),
    [sym_device_wake] = ACTIONS(1593),
    [sym_timeline_create] = ACTIONS(1593),
    [sym_timeline_kill] = ACTIONS(1593),
    [sym_timeline_reload] = ACTIONS(1593),
    [sym_timeline_pause] = ACTIONS(1593),
    [sym_timeline_restart] = ACTIONS(1593),
    [sym_timeline_set] = ACTIONS(1593),
    [sym_timeline_get] = ACTIONS(1593),
    [sym_timeline_active] = ACTIONS(1593),
    [sym_timeline_dynamic_id] = ACTIONS(1593),
    [sym_variable_to_string] = ACTIONS(1593),
    [sym_string_to_variable] = ACTIONS(1593),
    [sym_length_variable_to_string] = ACTIONS(1593),
    [sym_type_cast] = ACTIONS(1593),
    [sym_variable_to_xml] = ACTIONS(1593),
    [sym_xml_to_variable] = ACTIONS(1593),
    [sym_length_variable_to_xml] = ACTIONS(1593),
    [sym_redirect_string] = ACTIONS(1593),
    [sym_random_number] = ACTIONS(1593),
    [sym_get_unique_id] = ACTIONS(1593),
    [sym_get_system_number] = ACTIONS(1593),
    [sym_set_system_number] = ACTIONS(1593),
    [sym_get_serial_number] = ACTIONS(1593),
    [sym_get_master_build] = ACTIONS(1593),
    [sym_set_validation_code] = ACTIONS(1593),
    [sym_reboot] = ACTIONS(1593),
    [sym_load_duet_module] = ACTIONS(1593),
    [sym_unload_duet_module] = ACTIONS(1593),
    [sym_duet_mem_size_set] = ACTIONS(1593),
    [sym_duet_mem_size_get] = ACTIONS(1593),
    [sym_dynamic_application_device] = ACTIONS(1593),
    [sym_dynamic_polled_port] = ACTIONS(1593),
    [sym_static_port_binding] = ACTIONS(1593),
    [sym_static_ip_binding] = ACTIONS(1593),
    [sym_get_dns_list] = ACTIONS(1593),
    [sym_set_dns_list] = ACTIONS(1593),
    [sym_get_ip_address] = ACTIONS(1593),
    [sym_set_ip_address] = ACTIONS(1593),
    [sym_get_url_list] = ACTIONS(1593),
    [sym_add_url_entry] = ACTIONS(1593),
    [sym_delete_url_entry] = ACTIONS(1593),
    [sym_abs_value] = ACTIONS(1593),
    [sym_max_value] = ACTIONS(1593),
    [sym_min_value] = ACTIONS(1593),
    [sym_exp_value] = ACTIONS(1593),
    [sym_log_value] = ACTIONS(1593),
    [sym_log10_value] = ACTIONS(1593),
    [sym_power_value] = ACTIONS(1593),
    [sym_sqrt_value] = ACTIONS(1593),
    [sym_raw_be] = ACTIONS(1593),
    [sym_raw_le] = ACTIONS(1593),
    [sym_internal_queue_size_set] = ACTIONS(1593),
    [sym_internal_queue_size_get] = ACTIONS(1593),
    [sym_internal_threshold_set] = ACTIONS(1593),
    [sym_internal_threshold_get] = ACTIONS(1593),
    [sym_rebuilt_event] = ACTIONS(1593),
    [sym_do_custom_event] = ACTIONS(1593),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1593),
    [sym_clkmgr_set_clk_source] = ACTIONS(1593),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1593),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1593),
    [sym_clkmgr_get_timezone] = ACTIONS(1593),
    [sym_clkmgr_set_timezone] = ACTIONS(1593),
    [sym_clkmgr_get_resync_period] = ACTIONS(1593),
    [sym_clkmgr_set_resync_period] = ACTIONS(1593),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1593),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1593),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1593),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1593),
    [sym_clkmgr_get_timeservers] = ACTIONS(1593),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1593),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1593),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1593),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1593),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1593),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1593),
    [sym_set_log_level] = ACTIONS(1593),
    [sym_get_log_level] = ACTIONS(1593),
    [sym_amx_log] = ACTIONS(1593),
    [sym_smtp_server_config_set] = ACTIONS(1593),
    [sym_smtp_server_config_get] = ACTIONS(1593),
    [sym_smtp_send] = ACTIONS(1593),
    [sym_validate_netlinx_account] = ACTIONS(1593),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1593),
    [sym_authenticate_certificate] = ACTIONS(1593),
    [sym_audit_netlinx_session_event] = ACTIONS(1593),
    [sym_audit_netlinx_generic_event] = ACTIONS(1593),
    [sym_get_available_flash_disk_space] = ACTIONS(1593),
    [sym_get_max_flash_disk_space] = ACTIONS(1593),
    [sym_data_create_feed] = ACTIONS(1593),
    [sym_data_delete_feed] = ACTIONS(1593),
    [sym_data_publish_feed] = ACTIONS(1593),
    [sym_data_get_published_feed] = ACTIONS(1593),
    [sym_data_add_record] = ACTIONS(1593),
    [sym_data_get_event_record] = ACTIONS(1593),
    [sym__wc_data_create_feed] = ACTIONS(1593),
    [sym__wc_data_add_record] = ACTIONS(1593),
    [sym__wc_data_get_event_record] = ACTIONS(1593),
    [sym_format] = ACTIONS(1593),
    [sym_astro_clock] = ACTIONS(1593),
    [sym_set_outdoor_temperature] = ACTIONS(1593),
    [sym_set_virtual_level_count] = ACTIONS(1593),
    [sym_set_virtual_channel_count] = ACTIONS(1593),
    [sym_set_virtual_port_count] = ACTIONS(1593),
    [sym__wc_encode_int] = ACTIONS(1593),
    [sym__wc_decode_int] = ACTIONS(1593),
    [sym__wc_int] = ACTIONS(1593),
    [sym__wc_to_ch_int] = ACTIONS(1593),
    [sym__ch_to_wc_int] = ACTIONS(1593),
    [sym__wc_find_string_int] = ACTIONS(1593),
    [sym__wc_left_string_int] = ACTIONS(1593),
    [sym__wc_lower_string_int] = ACTIONS(1593),
    [sym__wc_mid_string_int] = ACTIONS(1593),
    [sym__wc_remove_string_int] = ACTIONS(1593),
    [sym__wc_right_string_int] = ACTIONS(1593),
    [sym__wc_upper_string_int] = ACTIONS(1593),
    [sym__wc_compare_string_int] = ACTIONS(1593),
    [sym__wc_get_buffer_char_int] = ACTIONS(1593),
    [sym__wc_get_buffer_string_int] = ACTIONS(1593),
    [sym__wc_concat_string_int] = ACTIONS(1593),
    [sym__wc_file_open_int] = ACTIONS(1593),
    [sym__wc_file_close_int] = ACTIONS(1593),
    [sym__wc_file_read_int] = ACTIONS(1593),
    [sym__wc_file_read_line_int] = ACTIONS(1593),
    [sym__wc_file_write_int] = ACTIONS(1593),
    [sym__wc_file_write_line_int] = ACTIONS(1593),
    [aux_sym_preproc_include_token1] = ACTIONS(1593),
    [aux_sym_preproc_define_token1] = ACTIONS(1595),
    [aux_sym_preproc_warn_token1] = ACTIONS(1595),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1595),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1595),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1595),
    [anon_sym_LBRACK] = ACTIONS(1595),
    [anon_sym_LBRACE] = ACTIONS(1595),
    [anon_sym_COLON] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1593),
    [anon_sym_SEMI] = ACTIONS(1595),
    [anon_sym_BANG] = ACTIONS(1593),
    [anon_sym_TILDE] = ACTIONS(1595),
    [anon_sym_DASH] = ACTIONS(1593),
    [anon_sym_PLUS] = ACTIONS(1593),
    [anon_sym_STAR] = ACTIONS(1595),
    [anon_sym_SLASH] = ACTIONS(1593),
    [anon_sym_PERCENT] = ACTIONS(1595),
    [anon_sym_PIPE_PIPE] = ACTIONS(1595),
    [anon_sym_AMP_AMP] = ACTIONS(1595),
    [anon_sym_PIPE] = ACTIONS(1593),
    [anon_sym_CARET] = ACTIONS(1595),
    [anon_sym_AMP] = ACTIONS(1593),
    [anon_sym_EQ_EQ] = ACTIONS(1595),
    [anon_sym_BANG_EQ] = ACTIONS(1595),
    [anon_sym_LT_GT] = ACTIONS(1595),
    [anon_sym_GT] = ACTIONS(1593),
    [anon_sym_GT_EQ] = ACTIONS(1595),
    [anon_sym_LT_EQ] = ACTIONS(1595),
    [anon_sym_LT] = ACTIONS(1593),
    [anon_sym_LT_LT] = ACTIONS(1595),
    [anon_sym_GT_GT] = ACTIONS(1595),
    [anon_sym_DASH_DASH] = ACTIONS(1595),
    [anon_sym_PLUS_PLUS] = ACTIONS(1595),
    [anon_sym_DOT] = ACTIONS(1593),
    [anon_sym_DQUOTE] = ACTIONS(1595),
    [anon_sym_SQUOTE] = ACTIONS(1595),
    [sym_number_literal] = ACTIONS(1595),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1595),
  },
  [STATE(142)] = {
    [ts_builtin_sym_end] = ACTIONS(1505),
    [sym_identifier] = ACTIONS(1503),
    [sym_program_name_keyword] = ACTIONS(1503),
    [sym_module_name_keyword] = ACTIONS(1503),
    [sym_define_device_keyword] = ACTIONS(1503),
    [sym_define_combine_keyword] = ACTIONS(1503),
    [sym_define_constant_keyword] = ACTIONS(1503),
    [sym_define_type_keyword] = ACTIONS(1503),
    [sym_define_variable_keyword] = ACTIONS(1503),
    [sym_define_system_variable_keyword] = ACTIONS(1503),
    [sym_define_connect_level_keyword] = ACTIONS(1503),
    [sym_define_latching_keyword] = ACTIONS(1503),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1503),
    [sym_define_toggling_keyword] = ACTIONS(1503),
    [sym_define_call_keyword] = ACTIONS(1503),
    [sym_define_function_keyword] = ACTIONS(1503),
    [sym_define_library_function_keyword] = ACTIONS(1503),
    [sym_define_start_keyword] = ACTIONS(1503),
    [sym_define_event_keyword] = ACTIONS(1503),
    [sym_define_program_keyword] = ACTIONS(1503),
    [sym_define_module_keyword] = ACTIONS(1503),
    [sym_char_keyword] = ACTIONS(1503),
    [sym_widechar_keyword] = ACTIONS(1503),
    [sym_integer_keyword] = ACTIONS(1503),
    [sym_sinteger_keyword] = ACTIONS(1503),
    [sym_long_keyword] = ACTIONS(1503),
    [sym_slong_keyword] = ACTIONS(1503),
    [sym_float_keyword] = ACTIONS(1503),
    [sym_double_keyword] = ACTIONS(1503),
    [sym_variant_keyword] = ACTIONS(1503),
    [sym_variantarray_keyword] = ACTIONS(1503),
    [sym_local_var_keyword] = ACTIONS(1503),
    [sym_stack_var_keyword] = ACTIONS(1503),
    [sym_constant_keyword] = ACTIONS(1503),
    [sym_volatile_keyword] = ACTIONS(1503),
    [sym_non_volatile_keyword] = ACTIONS(1503),
    [sym_persistent_keyword] = ACTIONS(1503),
    [sym_dev_keyword] = ACTIONS(1503),
    [sym_devlev_keyword] = ACTIONS(1503),
    [sym_devchan_keyword] = ACTIONS(1503),
    [sym_if_keyword] = ACTIONS(1503),
    [sym_else_keyword] = ACTIONS(1503),
    [sym_switch_keyword] = ACTIONS(1503),
    [sym_case_keyword] = ACTIONS(1503),
    [sym_default_keyword] = ACTIONS(1503),
    [sym_while_keyword] = ACTIONS(1503),
    [sym_for_keyword] = ACTIONS(1503),
    [sym_break_keyword] = ACTIONS(1503),
    [sym_continue_keyword] = ACTIONS(1503),
    [sym_return_keyword] = ACTIONS(1503),
    [sym_select_keyword] = ACTIONS(1503),
    [sym_active_keyword] = ACTIONS(1503),
    [sym_struct_keyword] = ACTIONS(1503),
    [sym_structure_keyword] = ACTIONS(1503),
    [sym_button_event_keyword] = ACTIONS(1503),
    [sym_channel_event_keyword] = ACTIONS(1503),
    [sym_level_event_keyword] = ACTIONS(1503),
    [sym_data_event_keyword] = ACTIONS(1503),
    [sym_timeline_event_keyword] = ACTIONS(1503),
    [sym_custom_event_keyword] = ACTIONS(1503),
    [sym_push_keyword] = ACTIONS(1503),
    [sym_release_keyword] = ACTIONS(1503),
    [aux_sym_on_keyword_token1] = ACTIONS(1503),
    [aux_sym_off_keyword_token1] = ACTIONS(1503),
    [sym_send_level_keyword] = ACTIONS(1503),
    [sym_send_string_keyword] = ACTIONS(1503),
    [sym_send_command_keyword] = ACTIONS(1503),
    [sym_clear_buffer_keyword] = ACTIONS(1503),
    [sym_create_buffer_keyword] = ACTIONS(1503),
    [sym_create_multi_buffer_keyword] = ACTIONS(1503),
    [sym_call_keyword] = ACTIONS(1503),
    [sym_system_call_keyword] = ACTIONS(1503),
    [sym_devchan_to_keyword] = ACTIONS(1503),
    [sym_devchan_min_to_keyword] = ACTIONS(1503),
    [sym_devchan_total_off_keyword] = ACTIONS(1503),
    [sym_devchan_pulse_keyword] = ACTIONS(1503),
    [sym_wait_keyword] = ACTIONS(1503),
    [sym_cancel_wait_keyword] = ACTIONS(1503),
    [sym_wait_until_keyword] = ACTIONS(1503),
    [sym_cancel_wait_until_keyword] = ACTIONS(1503),
    [sym_cancel_all_wait_keyword] = ACTIONS(1503),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1503),
    [sym_and] = ACTIONS(1503),
    [sym_or] = ACTIONS(1503),
    [sym_not] = ACTIONS(1503),
    [sym_xor] = ACTIONS(1503),
    [sym_band] = ACTIONS(1503),
    [sym_bor] = ACTIONS(1503),
    [sym_bxor] = ACTIONS(1503),
    [sym_bnot] = ACTIONS(1503),
    [sym_lshift] = ACTIONS(1503),
    [sym_rshift] = ACTIONS(1503),
    [sym___file__] = ACTIONS(1503),
    [sym___date__] = ACTIONS(1503),
    [sym___ldate__] = ACTIONS(1503),
    [sym___time__] = ACTIONS(1503),
    [sym___line__] = ACTIONS(1503),
    [sym___name__] = ACTIONS(1503),
    [sym___netlinx__] = ACTIONS(1503),
    [sym_netlinx_axi_version] = ACTIONS(1503),
    [sym_url_flg_tcp] = ACTIONS(1503),
    [sym_url_flg_accinfopresent] = ACTIONS(1503),
    [sym_url_flg_temp] = ACTIONS(1503),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1503),
    [sym_url_flg_stat_mask] = ACTIONS(1503),
    [sym_url_flg_stat_lookup] = ACTIONS(1503),
    [sym_url_flg_stat_connecting] = ACTIONS(1503),
    [sym_url_flg_stat_waiting] = ACTIONS(1503),
    [sym_url_flg_stat_connected] = ACTIONS(1503),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1503),
    [sym_first_virtual_device] = ACTIONS(1503),
    [sym_first_local_port] = ACTIONS(1503),
    [sym_dynamic_virtual_device] = ACTIONS(1503),
    [sym_timeline_once] = ACTIONS(1503),
    [sym_timeline_repeat] = ACTIONS(1503),
    [sym_timeline_absolute] = ACTIONS(1503),
    [sym_timeline_relative] = ACTIONS(1503),
    [sym_do_push_timed_infinite] = ACTIONS(1503),
    [sym_source_type_no_address] = ACTIONS(1503),
    [sym_source_type_neuron_id] = ACTIONS(1503),
    [sym_source_type_ip_address] = ACTIONS(1503),
    [sym_source_type_axlink] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1503),
    [sym_source_type_ip_socket_address] = ACTIONS(1503),
    [sym_source_type_rs232] = ACTIONS(1503),
    [sym_source_type_internal] = ACTIONS(1503),
    [sym_source_type_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1503),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1503),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1503),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1503),
    [sym_true] = ACTIONS(1503),
    [sym_false] = ACTIONS(1503),
    [sym_file_read_only] = ACTIONS(1503),
    [sym_file_rw_new] = ACTIONS(1503),
    [sym_file_rw_append] = ACTIONS(1503),
    [sym_ip_tcp] = ACTIONS(1503),
    [sym_ip_udp] = ACTIONS(1503),
    [sym_ip_udp_2way] = ACTIONS(1503),
    [sym_xml_encode_types] = ACTIONS(1503),
    [sym_xml_encode_char_as_list] = ACTIONS(1503),
    [sym_xml_encode_le] = ACTIONS(1503),
    [sym_xml_decode_types] = ACTIONS(1503),
    [sym_xml_decode_no_preserve] = ACTIONS(1503),
    [sym_duet_dev_type_display_device] = ACTIONS(1503),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_io_device] = ACTIONS(1503),
    [sym_duet_dev_type_relay_device] = ACTIONS(1503),
    [sym_duet_dev_type_ups] = ACTIONS(1503),
    [sym_duet_dev_type_amplifier] = ACTIONS(1503),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1503),
    [sym_duet_dev_type_camera] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1503),
    [sym_duet_dev_type_dss] = ACTIONS(1503),
    [sym_duet_dev_type_dvr] = ACTIONS(1503),
    [sym_duet_dev_type_disc_device] = ACTIONS(1503),
    [sym_duet_dev_type_document_camera] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1503),
    [sym_duet_dev_type_hvac] = ACTIONS(1503),
    [sym_duet_dev_type_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_light] = ACTIONS(1503),
    [sym_duet_dev_type_monitor] = ACTIONS(1503),
    [sym_duet_dev_type_motor] = ACTIONS(1503),
    [sym_duet_dev_type_multi_window] = ACTIONS(1503),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1503),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1503),
    [sym_duet_dev_type_receiver] = ACTIONS(1503),
    [sym_duet_dev_type_security_system] = ACTIONS(1503),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1503),
    [sym_duet_dev_type_settop_box] = ACTIONS(1503),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1503),
    [sym_duet_dev_type_switcher] = ACTIONS(1503),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_tv] = ACTIONS(1503),
    [sym_duet_dev_type_utility] = ACTIONS(1503),
    [sym_duet_dev_type_vcr] = ACTIONS(1503),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_video_processor] = ACTIONS(1503),
    [sym_duet_dev_type_video_projector] = ACTIONS(1503),
    [sym_duet_dev_type_video_wall] = ACTIONS(1503),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1503),
    [sym_duet_dev_type_weather] = ACTIONS(1503),
    [sym_duet_dev_not_polled] = ACTIONS(1503),
    [sym_duet_dev_polled] = ACTIONS(1503),
    [sym_clkmgr_mode_network] = ACTIONS(1503),
    [sym_clkmgr_mode_standalone] = ACTIONS(1503),
    [sym_normal_standby] = ACTIONS(1503),
    [sym_normal_wake] = ACTIONS(1503),
    [sym_ip_multicast_ttl_option] = ACTIONS(1503),
    [sym_ip_ttl_subnet] = ACTIONS(1503),
    [sym_ip_ttl_site] = ACTIONS(1503),
    [sym_ip_ttl_region] = ACTIONS(1503),
    [sym_ip_ttl_continent] = ACTIONS(1503),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1503),
    [sym_ip_nodelay_on] = ACTIONS(1503),
    [sym_ip_nodelay_off] = ACTIONS(1503),
    [sym_amx_error] = ACTIONS(1503),
    [sym_amx_warning] = ACTIONS(1503),
    [sym_amx_info] = ACTIONS(1503),
    [sym_amx_debug] = ACTIONS(1503),
    [sym_netlinx_login_success] = ACTIONS(1503),
    [sym_netlinx_login_fail] = ACTIONS(1503),
    [sym_netlinx_logout] = ACTIONS(1503),
    [sym_null_str] = ACTIONS(1503),
    [sym_smtp_address] = ACTIONS(1503),
    [sym_smtp_port_number] = ACTIONS(1503),
    [sym_smtp_username] = ACTIONS(1503),
    [sym_smtp_password] = ACTIONS(1503),
    [sym_smtp_require_tls] = ACTIONS(1503),
    [sym_smtp_from] = ACTIONS(1503),
    [sym_smtp_tls_true] = ACTIONS(1503),
    [sym_smtp_tls_false] = ACTIONS(1503),
    [sym_listview_on_row_select_event] = ACTIONS(1503),
    [sym_data_structure_unknown] = ACTIONS(1503),
    [sym_data_structure_datafeed] = ACTIONS(1503),
    [sym_data_structure_datarecordset] = ACTIONS(1503),
    [sym_data_structure_datarecord] = ACTIONS(1503),
    [sym_data_structure_datafield] = ACTIONS(1503),
    [sym_data_max_content_field_count] = ACTIONS(1503),
    [sym_data_max_metadata_field_count] = ACTIONS(1503),
    [sym_data_max_id_length] = ACTIONS(1503),
    [sym_data_max_name_length] = ACTIONS(1503),
    [sym_data_max_description_length] = ACTIONS(1503),
    [sym_data_max_type_length] = ACTIONS(1503),
    [sym_data_max_format_length] = ACTIONS(1503),
    [sym_data_max_source_length] = ACTIONS(1503),
    [sym_data_max_value_length] = ACTIONS(1503),
    [sym_data_max_label_length] = ACTIONS(1503),
    [sym_data_max_url_length] = ACTIONS(1503),
    [sym_max_last_login_info_length] = ACTIONS(1503),
    [sym_data_type_unknown] = ACTIONS(1503),
    [sym_data_type_string] = ACTIONS(1503),
    [sym_data_type_datetime] = ACTIONS(1503),
    [sym_data_type_date] = ACTIONS(1503),
    [sym_data_type_time] = ACTIONS(1503),
    [sym_data_type_image] = ACTIONS(1503),
    [sym_data_format_url] = ACTIONS(1503),
    [sym_data_format_phone] = ACTIONS(1503),
    [sym_data_format_email] = ACTIONS(1503),
    [sym_data_format_iso8601] = ACTIONS(1503),
    [sym_auditlog_priv] = ACTIONS(1503),
    [sym_configuration_priv] = ACTIONS(1503),
    [sym_deviceconfig_priv] = ACTIONS(1503),
    [sym_ftp_priv] = ACTIONS(1503),
    [sym_http_priv] = ACTIONS(1503),
    [sym_networkconfig_priv] = ACTIONS(1503),
    [sym_programport_priv] = ACTIONS(1503),
    [sym_remoteui_priv] = ACTIONS(1503),
    [sym_securitycontrol_priv] = ACTIONS(1503),
    [sym_softwaremanagement_priv] = ACTIONS(1503),
    [sym_terminal_priv] = ACTIONS(1503),
    [sym_tpadmin_priv] = ACTIONS(1503),
    [sym_user1_priv] = ACTIONS(1503),
    [sym_user2_priv] = ACTIONS(1503),
    [sym_user3_priv] = ACTIONS(1503),
    [sym_user4_priv] = ACTIONS(1503),
    [sym_usermanagement_priv] = ACTIONS(1503),
    [sym_valid_account] = ACTIONS(1503),
    [sym_err_name_invalid] = ACTIONS(1503),
    [sym_err_password_invalid] = ACTIONS(1503),
    [sym_err_invalid_account] = ACTIONS(1503),
    [sym_err_no_authorization] = ACTIONS(1503),
    [sym_err_invalid_parameter] = ACTIONS(1503),
    [sym_err_user_locked_out] = ACTIONS(1503),
    [sym_err_user_account_expired] = ACTIONS(1503),
    [sym_err_general_error] = ACTIONS(1503),
    [sym_err_authentication_error] = ACTIONS(1503),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1503),
    [sym_tls_validate_certificate] = ACTIONS(1503),
    [sym_tdata] = ACTIONS(1503),
    [sym_tchannel] = ACTIONS(1503),
    [sym_tlevel] = ACTIONS(1503),
    [sym_tbutton] = ACTIONS(1503),
    [sym_ttimeline] = ACTIONS(1503),
    [sym_tcustom] = ACTIONS(1503),
    [sym_url_struct] = ACTIONS(1503),
    [sym_dns_struct] = ACTIONS(1503),
    [sym_ip_address_struct] = ACTIONS(1503),
    [sym_dev_info_struct] = ACTIONS(1503),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1503),
    [sym_data_feed] = ACTIONS(1503),
    [sym_data_field] = ACTIONS(1503),
    [sym_data_record] = ACTIONS(1503),
    [sym_wc_data_feed] = ACTIONS(1503),
    [sym_wc_data_field] = ACTIONS(1503),
    [sym_wc_data_record] = ACTIONS(1503),
    [sym_last_login_info] = ACTIONS(1503),
    [sym____reserved___] = ACTIONS(1503),
    [sym_date] = ACTIONS(1503),
    [sym_day] = ACTIONS(1503),
    [sym_time] = ACTIONS(1503),
    [sym_ldate] = ACTIONS(1503),
    [sym_push_device] = ACTIONS(1503),
    [sym_push_channel] = ACTIONS(1503),
    [sym_push_devchan] = ACTIONS(1503),
    [sym_release_device] = ACTIONS(1503),
    [sym_release_channel] = ACTIONS(1503),
    [sym_release_devchan] = ACTIONS(1503),
    [sym_master_slot] = ACTIONS(1503),
    [sym_get_pulse_time] = ACTIONS(1503),
    [sym_get_timer] = ACTIONS(1503),
    [sym_master_sn] = ACTIONS(1503),
    [sym_system_number] = ACTIONS(1503),
    [sym_button] = ACTIONS(1503),
    [sym_dv_channel] = ACTIONS(1503),
    [sym_data] = ACTIONS(1503),
    [sym_level] = ACTIONS(1503),
    [sym_timeline] = ACTIONS(1503),
    [sym_channel] = ACTIONS(1503),
    [sym_custom] = ACTIONS(1503),
    [sym_atoi] = ACTIONS(1503),
    [sym_atol] = ACTIONS(1503),
    [sym_atof] = ACTIONS(1503),
    [sym_itoa] = ACTIONS(1503),
    [sym_itohex] = ACTIONS(1503),
    [sym_hextoi] = ACTIONS(1503),
    [sym_ftoa] = ACTIONS(1503),
    [sym_find_string] = ACTIONS(1503),
    [sym_left_string] = ACTIONS(1503),
    [sym_right_string] = ACTIONS(1503),
    [sym_mid_string] = ACTIONS(1503),
    [sym_remove_string] = ACTIONS(1503),
    [sym_get_buffer_string] = ACTIONS(1503),
    [sym_get_buffer_char] = ACTIONS(1503),
    [sym_get_multi_buffer_string] = ACTIONS(1503),
    [sym_length_string] = ACTIONS(1503),
    [sym_max_length_string] = ACTIONS(1503),
    [sym_set_length_string] = ACTIONS(1503),
    [sym_compare_string] = ACTIONS(1503),
    [sym_upper_string] = ACTIONS(1503),
    [sym_lower_string] = ACTIONS(1503),
    [sym_get_last] = ACTIONS(1503),
    [sym_date_to_day] = ACTIONS(1503),
    [sym_date_to_month] = ACTIONS(1503),
    [sym_date_to_year] = ACTIONS(1503),
    [sym_time_to_hour] = ACTIONS(1503),
    [sym_time_to_minute] = ACTIONS(1503),
    [sym_time_to_second] = ACTIONS(1503),
    [sym_day_of_week] = ACTIONS(1503),
    [sym_set_pulse_time] = ACTIONS(1503),
    [sym_set_timer] = ACTIONS(1503),
    [sym_file_open] = ACTIONS(1503),
    [sym_file_close] = ACTIONS(1503),
    [sym_file_read] = ACTIONS(1503),
    [sym_file_read_line] = ACTIONS(1503),
    [sym_file_write] = ACTIONS(1503),
    [sym_file_write_line] = ACTIONS(1503),
    [sym_file_copy] = ACTIONS(1503),
    [sym_file_delete] = ACTIONS(1503),
    [sym_file_rename] = ACTIONS(1503),
    [sym_file_dir] = ACTIONS(1503),
    [sym_file_getdir] = ACTIONS(1503),
    [sym_file_setdir] = ACTIONS(1503),
    [sym_file_create_dir] = ACTIONS(1503),
    [sym_file_remove_dir] = ACTIONS(1503),
    [sym_file_seek] = ACTIONS(1503),
    [sym_length_array] = ACTIONS(1503),
    [sym_max_length_array] = ACTIONS(1503),
    [sym_set_length_array] = ACTIONS(1503),
    [sym_ip_client_open] = ACTIONS(1503),
    [sym_ip_client_close] = ACTIONS(1503),
    [sym_ip_server_open] = ACTIONS(1503),
    [sym_ip_server_close] = ACTIONS(1503),
    [sym_ip_bound_client_open] = ACTIONS(1503),
    [sym_ip_mc_server_open] = ACTIONS(1503),
    [sym_ip_set_option] = ACTIONS(1503),
    [sym_ssh_client_open] = ACTIONS(1503),
    [sym_ssh_client_close] = ACTIONS(1503),
    [sym_tls_client_open] = ACTIONS(1503),
    [sym_tls_client_close] = ACTIONS(1503),
    [sym_do_push] = ACTIONS(1503),
    [sym_do_push_timed] = ACTIONS(1503),
    [sym_do_release] = ACTIONS(1503),
    [sym_device_id] = ACTIONS(1503),
    [sym_device_info] = ACTIONS(1503),
    [sym_device_id_string] = ACTIONS(1503),
    [sym_device_standby] = ACTIONS(1503),
    [sym_device_wake] = ACTIONS(1503),
    [sym_timeline_create] = ACTIONS(1503),
    [sym_timeline_kill] = ACTIONS(1503),
    [sym_timeline_reload] = ACTIONS(1503),
    [sym_timeline_pause] = ACTIONS(1503),
    [sym_timeline_restart] = ACTIONS(1503),
    [sym_timeline_set] = ACTIONS(1503),
    [sym_timeline_get] = ACTIONS(1503),
    [sym_timeline_active] = ACTIONS(1503),
    [sym_timeline_dynamic_id] = ACTIONS(1503),
    [sym_variable_to_string] = ACTIONS(1503),
    [sym_string_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_string] = ACTIONS(1503),
    [sym_type_cast] = ACTIONS(1503),
    [sym_variable_to_xml] = ACTIONS(1503),
    [sym_xml_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_xml] = ACTIONS(1503),
    [sym_redirect_string] = ACTIONS(1503),
    [sym_random_number] = ACTIONS(1503),
    [sym_get_unique_id] = ACTIONS(1503),
    [sym_get_system_number] = ACTIONS(1503),
    [sym_set_system_number] = ACTIONS(1503),
    [sym_get_serial_number] = ACTIONS(1503),
    [sym_get_master_build] = ACTIONS(1503),
    [sym_set_validation_code] = ACTIONS(1503),
    [sym_reboot] = ACTIONS(1503),
    [sym_load_duet_module] = ACTIONS(1503),
    [sym_unload_duet_module] = ACTIONS(1503),
    [sym_duet_mem_size_set] = ACTIONS(1503),
    [sym_duet_mem_size_get] = ACTIONS(1503),
    [sym_dynamic_application_device] = ACTIONS(1503),
    [sym_dynamic_polled_port] = ACTIONS(1503),
    [sym_static_port_binding] = ACTIONS(1503),
    [sym_static_ip_binding] = ACTIONS(1503),
    [sym_get_dns_list] = ACTIONS(1503),
    [sym_set_dns_list] = ACTIONS(1503),
    [sym_get_ip_address] = ACTIONS(1503),
    [sym_set_ip_address] = ACTIONS(1503),
    [sym_get_url_list] = ACTIONS(1503),
    [sym_add_url_entry] = ACTIONS(1503),
    [sym_delete_url_entry] = ACTIONS(1503),
    [sym_abs_value] = ACTIONS(1503),
    [sym_max_value] = ACTIONS(1503),
    [sym_min_value] = ACTIONS(1503),
    [sym_exp_value] = ACTIONS(1503),
    [sym_log_value] = ACTIONS(1503),
    [sym_log10_value] = ACTIONS(1503),
    [sym_power_value] = ACTIONS(1503),
    [sym_sqrt_value] = ACTIONS(1503),
    [sym_raw_be] = ACTIONS(1503),
    [sym_raw_le] = ACTIONS(1503),
    [sym_internal_queue_size_set] = ACTIONS(1503),
    [sym_internal_queue_size_get] = ACTIONS(1503),
    [sym_internal_threshold_set] = ACTIONS(1503),
    [sym_internal_threshold_get] = ACTIONS(1503),
    [sym_rebuilt_event] = ACTIONS(1503),
    [sym_do_custom_event] = ACTIONS(1503),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1503),
    [sym_clkmgr_set_clk_source] = ACTIONS(1503),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1503),
    [sym_clkmgr_get_timezone] = ACTIONS(1503),
    [sym_clkmgr_set_timezone] = ACTIONS(1503),
    [sym_clkmgr_get_resync_period] = ACTIONS(1503),
    [sym_clkmgr_set_resync_period] = ACTIONS(1503),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_timeservers] = ACTIONS(1503),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_set_log_level] = ACTIONS(1503),
    [sym_get_log_level] = ACTIONS(1503),
    [sym_amx_log] = ACTIONS(1503),
    [sym_smtp_server_config_set] = ACTIONS(1503),
    [sym_smtp_server_config_get] = ACTIONS(1503),
    [sym_smtp_send] = ACTIONS(1503),
    [sym_validate_netlinx_account] = ACTIONS(1503),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1503),
    [sym_authenticate_certificate] = ACTIONS(1503),
    [sym_audit_netlinx_session_event] = ACTIONS(1503),
    [sym_audit_netlinx_generic_event] = ACTIONS(1503),
    [sym_get_available_flash_disk_space] = ACTIONS(1503),
    [sym_get_max_flash_disk_space] = ACTIONS(1503),
    [sym_data_create_feed] = ACTIONS(1503),
    [sym_data_delete_feed] = ACTIONS(1503),
    [sym_data_publish_feed] = ACTIONS(1503),
    [sym_data_get_published_feed] = ACTIONS(1503),
    [sym_data_add_record] = ACTIONS(1503),
    [sym_data_get_event_record] = ACTIONS(1503),
    [sym__wc_data_create_feed] = ACTIONS(1503),
    [sym__wc_data_add_record] = ACTIONS(1503),
    [sym__wc_data_get_event_record] = ACTIONS(1503),
    [sym_format] = ACTIONS(1503),
    [sym_astro_clock] = ACTIONS(1503),
    [sym_set_outdoor_temperature] = ACTIONS(1503),
    [sym_set_virtual_level_count] = ACTIONS(1503),
    [sym_set_virtual_channel_count] = ACTIONS(1503),
    [sym_set_virtual_port_count] = ACTIONS(1503),
    [sym__wc_encode_int] = ACTIONS(1503),
    [sym__wc_decode_int] = ACTIONS(1503),
    [sym__wc_int] = ACTIONS(1503),
    [sym__wc_to_ch_int] = ACTIONS(1503),
    [sym__ch_to_wc_int] = ACTIONS(1503),
    [sym__wc_find_string_int] = ACTIONS(1503),
    [sym__wc_left_string_int] = ACTIONS(1503),
    [sym__wc_lower_string_int] = ACTIONS(1503),
    [sym__wc_mid_string_int] = ACTIONS(1503),
    [sym__wc_remove_string_int] = ACTIONS(1503),
    [sym__wc_right_string_int] = ACTIONS(1503),
    [sym__wc_upper_string_int] = ACTIONS(1503),
    [sym__wc_compare_string_int] = ACTIONS(1503),
    [sym__wc_get_buffer_char_int] = ACTIONS(1503),
    [sym__wc_get_buffer_string_int] = ACTIONS(1503),
    [sym__wc_concat_string_int] = ACTIONS(1503),
    [sym__wc_file_open_int] = ACTIONS(1503),
    [sym__wc_file_close_int] = ACTIONS(1503),
    [sym__wc_file_read_int] = ACTIONS(1503),
    [sym__wc_file_read_line_int] = ACTIONS(1503),
    [sym__wc_file_write_int] = ACTIONS(1503),
    [sym__wc_file_write_line_int] = ACTIONS(1503),
    [aux_sym_preproc_include_token1] = ACTIONS(1503),
    [aux_sym_preproc_define_token1] = ACTIONS(1505),
    [aux_sym_preproc_warn_token1] = ACTIONS(1505),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1505),
    [anon_sym_LBRACK] = ACTIONS(1505),
    [anon_sym_LBRACE] = ACTIONS(1505),
    [anon_sym_COLON] = ACTIONS(1505),
    [anon_sym_LPAREN] = ACTIONS(1503),
    [anon_sym_SEMI] = ACTIONS(1505),
    [anon_sym_BANG] = ACTIONS(1503),
    [anon_sym_TILDE] = ACTIONS(1505),
    [anon_sym_DASH] = ACTIONS(1503),
    [anon_sym_PLUS] = ACTIONS(1503),
    [anon_sym_STAR] = ACTIONS(1505),
    [anon_sym_SLASH] = ACTIONS(1503),
    [anon_sym_PERCENT] = ACTIONS(1505),
    [anon_sym_PIPE_PIPE] = ACTIONS(1505),
    [anon_sym_AMP_AMP] = ACTIONS(1505),
    [anon_sym_PIPE] = ACTIONS(1503),
    [anon_sym_CARET] = ACTIONS(1505),
    [anon_sym_AMP] = ACTIONS(1503),
    [anon_sym_EQ_EQ] = ACTIONS(1505),
    [anon_sym_BANG_EQ] = ACTIONS(1505),
    [anon_sym_LT_GT] = ACTIONS(1505),
    [anon_sym_GT] = ACTIONS(1503),
    [anon_sym_GT_EQ] = ACTIONS(1505),
    [anon_sym_LT_EQ] = ACTIONS(1505),
    [anon_sym_LT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1505),
    [anon_sym_GT_GT] = ACTIONS(1505),
    [anon_sym_DASH_DASH] = ACTIONS(1505),
    [anon_sym_PLUS_PLUS] = ACTIONS(1505),
    [anon_sym_DOT] = ACTIONS(1503),
    [anon_sym_DQUOTE] = ACTIONS(1505),
    [anon_sym_SQUOTE] = ACTIONS(1505),
    [sym_number_literal] = ACTIONS(1505),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1505),
  },
  [STATE(143)] = {
    [ts_builtin_sym_end] = ACTIONS(1583),
    [sym_identifier] = ACTIONS(1581),
    [sym_program_name_keyword] = ACTIONS(1581),
    [sym_module_name_keyword] = ACTIONS(1581),
    [sym_define_device_keyword] = ACTIONS(1581),
    [sym_define_combine_keyword] = ACTIONS(1581),
    [sym_define_constant_keyword] = ACTIONS(1581),
    [sym_define_type_keyword] = ACTIONS(1581),
    [sym_define_variable_keyword] = ACTIONS(1581),
    [sym_define_system_variable_keyword] = ACTIONS(1581),
    [sym_define_connect_level_keyword] = ACTIONS(1581),
    [sym_define_latching_keyword] = ACTIONS(1581),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1581),
    [sym_define_toggling_keyword] = ACTIONS(1581),
    [sym_define_call_keyword] = ACTIONS(1581),
    [sym_define_function_keyword] = ACTIONS(1581),
    [sym_define_library_function_keyword] = ACTIONS(1581),
    [sym_define_start_keyword] = ACTIONS(1581),
    [sym_define_event_keyword] = ACTIONS(1581),
    [sym_define_program_keyword] = ACTIONS(1581),
    [sym_define_module_keyword] = ACTIONS(1581),
    [sym_char_keyword] = ACTIONS(1581),
    [sym_widechar_keyword] = ACTIONS(1581),
    [sym_integer_keyword] = ACTIONS(1581),
    [sym_sinteger_keyword] = ACTIONS(1581),
    [sym_long_keyword] = ACTIONS(1581),
    [sym_slong_keyword] = ACTIONS(1581),
    [sym_float_keyword] = ACTIONS(1581),
    [sym_double_keyword] = ACTIONS(1581),
    [sym_variant_keyword] = ACTIONS(1581),
    [sym_variantarray_keyword] = ACTIONS(1581),
    [sym_local_var_keyword] = ACTIONS(1581),
    [sym_stack_var_keyword] = ACTIONS(1581),
    [sym_constant_keyword] = ACTIONS(1581),
    [sym_volatile_keyword] = ACTIONS(1581),
    [sym_non_volatile_keyword] = ACTIONS(1581),
    [sym_persistent_keyword] = ACTIONS(1581),
    [sym_dev_keyword] = ACTIONS(1581),
    [sym_devlev_keyword] = ACTIONS(1581),
    [sym_devchan_keyword] = ACTIONS(1581),
    [sym_if_keyword] = ACTIONS(1581),
    [sym_else_keyword] = ACTIONS(1581),
    [sym_switch_keyword] = ACTIONS(1581),
    [sym_case_keyword] = ACTIONS(1581),
    [sym_default_keyword] = ACTIONS(1581),
    [sym_while_keyword] = ACTIONS(1581),
    [sym_for_keyword] = ACTIONS(1581),
    [sym_break_keyword] = ACTIONS(1581),
    [sym_continue_keyword] = ACTIONS(1581),
    [sym_return_keyword] = ACTIONS(1581),
    [sym_select_keyword] = ACTIONS(1581),
    [sym_active_keyword] = ACTIONS(1581),
    [sym_struct_keyword] = ACTIONS(1581),
    [sym_structure_keyword] = ACTIONS(1581),
    [sym_button_event_keyword] = ACTIONS(1581),
    [sym_channel_event_keyword] = ACTIONS(1581),
    [sym_level_event_keyword] = ACTIONS(1581),
    [sym_data_event_keyword] = ACTIONS(1581),
    [sym_timeline_event_keyword] = ACTIONS(1581),
    [sym_custom_event_keyword] = ACTIONS(1581),
    [sym_push_keyword] = ACTIONS(1581),
    [sym_release_keyword] = ACTIONS(1581),
    [aux_sym_on_keyword_token1] = ACTIONS(1581),
    [aux_sym_off_keyword_token1] = ACTIONS(1581),
    [sym_send_level_keyword] = ACTIONS(1581),
    [sym_send_string_keyword] = ACTIONS(1581),
    [sym_send_command_keyword] = ACTIONS(1581),
    [sym_clear_buffer_keyword] = ACTIONS(1581),
    [sym_create_buffer_keyword] = ACTIONS(1581),
    [sym_create_multi_buffer_keyword] = ACTIONS(1581),
    [sym_call_keyword] = ACTIONS(1581),
    [sym_system_call_keyword] = ACTIONS(1581),
    [sym_devchan_to_keyword] = ACTIONS(1581),
    [sym_devchan_min_to_keyword] = ACTIONS(1581),
    [sym_devchan_total_off_keyword] = ACTIONS(1581),
    [sym_devchan_pulse_keyword] = ACTIONS(1581),
    [sym_wait_keyword] = ACTIONS(1581),
    [sym_cancel_wait_keyword] = ACTIONS(1581),
    [sym_wait_until_keyword] = ACTIONS(1581),
    [sym_cancel_wait_until_keyword] = ACTIONS(1581),
    [sym_cancel_all_wait_keyword] = ACTIONS(1581),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1581),
    [sym_and] = ACTIONS(1581),
    [sym_or] = ACTIONS(1581),
    [sym_not] = ACTIONS(1581),
    [sym_xor] = ACTIONS(1581),
    [sym_band] = ACTIONS(1581),
    [sym_bor] = ACTIONS(1581),
    [sym_bxor] = ACTIONS(1581),
    [sym_bnot] = ACTIONS(1581),
    [sym_lshift] = ACTIONS(1581),
    [sym_rshift] = ACTIONS(1581),
    [sym___file__] = ACTIONS(1581),
    [sym___date__] = ACTIONS(1581),
    [sym___ldate__] = ACTIONS(1581),
    [sym___time__] = ACTIONS(1581),
    [sym___line__] = ACTIONS(1581),
    [sym___name__] = ACTIONS(1581),
    [sym___netlinx__] = ACTIONS(1581),
    [sym_netlinx_axi_version] = ACTIONS(1581),
    [sym_url_flg_tcp] = ACTIONS(1581),
    [sym_url_flg_accinfopresent] = ACTIONS(1581),
    [sym_url_flg_temp] = ACTIONS(1581),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1581),
    [sym_url_flg_stat_mask] = ACTIONS(1581),
    [sym_url_flg_stat_lookup] = ACTIONS(1581),
    [sym_url_flg_stat_connecting] = ACTIONS(1581),
    [sym_url_flg_stat_waiting] = ACTIONS(1581),
    [sym_url_flg_stat_connected] = ACTIONS(1581),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1581),
    [sym_first_virtual_device] = ACTIONS(1581),
    [sym_first_local_port] = ACTIONS(1581),
    [sym_dynamic_virtual_device] = ACTIONS(1581),
    [sym_timeline_once] = ACTIONS(1581),
    [sym_timeline_repeat] = ACTIONS(1581),
    [sym_timeline_absolute] = ACTIONS(1581),
    [sym_timeline_relative] = ACTIONS(1581),
    [sym_do_push_timed_infinite] = ACTIONS(1581),
    [sym_source_type_no_address] = ACTIONS(1581),
    [sym_source_type_neuron_id] = ACTIONS(1581),
    [sym_source_type_ip_address] = ACTIONS(1581),
    [sym_source_type_axlink] = ACTIONS(1581),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1581),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1581),
    [sym_source_type_ip_socket_address] = ACTIONS(1581),
    [sym_source_type_rs232] = ACTIONS(1581),
    [sym_source_type_internal] = ACTIONS(1581),
    [sym_source_type_mac_address] = ACTIONS(1581),
    [sym_source_type_ipv4_port] = ACTIONS(1581),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1581),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1581),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1581),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1581),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1581),
    [sym_true] = ACTIONS(1581),
    [sym_false] = ACTIONS(1581),
    [sym_file_read_only] = ACTIONS(1581),
    [sym_file_rw_new] = ACTIONS(1581),
    [sym_file_rw_append] = ACTIONS(1581),
    [sym_ip_tcp] = ACTIONS(1581),
    [sym_ip_udp] = ACTIONS(1581),
    [sym_ip_udp_2way] = ACTIONS(1581),
    [sym_xml_encode_types] = ACTIONS(1581),
    [sym_xml_encode_char_as_list] = ACTIONS(1581),
    [sym_xml_encode_le] = ACTIONS(1581),
    [sym_xml_decode_types] = ACTIONS(1581),
    [sym_xml_decode_no_preserve] = ACTIONS(1581),
    [sym_duet_dev_type_display_device] = ACTIONS(1581),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1581),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1581),
    [sym_duet_dev_type_io_device] = ACTIONS(1581),
    [sym_duet_dev_type_relay_device] = ACTIONS(1581),
    [sym_duet_dev_type_ups] = ACTIONS(1581),
    [sym_duet_dev_type_amplifier] = ACTIONS(1581),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1581),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1581),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1581),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1581),
    [sym_duet_dev_type_camera] = ACTIONS(1581),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1581),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1581),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1581),
    [sym_duet_dev_type_dss] = ACTIONS(1581),
    [sym_duet_dev_type_dvr] = ACTIONS(1581),
    [sym_duet_dev_type_disc_device] = ACTIONS(1581),
    [sym_duet_dev_type_document_camera] = ACTIONS(1581),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1581),
    [sym_duet_dev_type_hvac] = ACTIONS(1581),
    [sym_duet_dev_type_keypad] = ACTIONS(1581),
    [sym_duet_dev_type_light] = ACTIONS(1581),
    [sym_duet_dev_type_monitor] = ACTIONS(1581),
    [sym_duet_dev_type_motor] = ACTIONS(1581),
    [sym_duet_dev_type_multi_window] = ACTIONS(1581),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1581),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1581),
    [sym_duet_dev_type_receiver] = ACTIONS(1581),
    [sym_duet_dev_type_security_system] = ACTIONS(1581),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1581),
    [sym_duet_dev_type_settop_box] = ACTIONS(1581),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1581),
    [sym_duet_dev_type_switcher] = ACTIONS(1581),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1581),
    [sym_duet_dev_type_tv] = ACTIONS(1581),
    [sym_duet_dev_type_utility] = ACTIONS(1581),
    [sym_duet_dev_type_vcr] = ACTIONS(1581),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1581),
    [sym_duet_dev_type_video_processor] = ACTIONS(1581),
    [sym_duet_dev_type_video_projector] = ACTIONS(1581),
    [sym_duet_dev_type_video_wall] = ACTIONS(1581),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1581),
    [sym_duet_dev_type_weather] = ACTIONS(1581),
    [sym_duet_dev_not_polled] = ACTIONS(1581),
    [sym_duet_dev_polled] = ACTIONS(1581),
    [sym_clkmgr_mode_network] = ACTIONS(1581),
    [sym_clkmgr_mode_standalone] = ACTIONS(1581),
    [sym_normal_standby] = ACTIONS(1581),
    [sym_normal_wake] = ACTIONS(1581),
    [sym_ip_multicast_ttl_option] = ACTIONS(1581),
    [sym_ip_ttl_subnet] = ACTIONS(1581),
    [sym_ip_ttl_site] = ACTIONS(1581),
    [sym_ip_ttl_region] = ACTIONS(1581),
    [sym_ip_ttl_continent] = ACTIONS(1581),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1581),
    [sym_ip_nodelay_on] = ACTIONS(1581),
    [sym_ip_nodelay_off] = ACTIONS(1581),
    [sym_amx_error] = ACTIONS(1581),
    [sym_amx_warning] = ACTIONS(1581),
    [sym_amx_info] = ACTIONS(1581),
    [sym_amx_debug] = ACTIONS(1581),
    [sym_netlinx_login_success] = ACTIONS(1581),
    [sym_netlinx_login_fail] = ACTIONS(1581),
    [sym_netlinx_logout] = ACTIONS(1581),
    [sym_null_str] = ACTIONS(1581),
    [sym_smtp_address] = ACTIONS(1581),
    [sym_smtp_port_number] = ACTIONS(1581),
    [sym_smtp_username] = ACTIONS(1581),
    [sym_smtp_password] = ACTIONS(1581),
    [sym_smtp_require_tls] = ACTIONS(1581),
    [sym_smtp_from] = ACTIONS(1581),
    [sym_smtp_tls_true] = ACTIONS(1581),
    [sym_smtp_tls_false] = ACTIONS(1581),
    [sym_listview_on_row_select_event] = ACTIONS(1581),
    [sym_data_structure_unknown] = ACTIONS(1581),
    [sym_data_structure_datafeed] = ACTIONS(1581),
    [sym_data_structure_datarecordset] = ACTIONS(1581),
    [sym_data_structure_datarecord] = ACTIONS(1581),
    [sym_data_structure_datafield] = ACTIONS(1581),
    [sym_data_max_content_field_count] = ACTIONS(1581),
    [sym_data_max_metadata_field_count] = ACTIONS(1581),
    [sym_data_max_id_length] = ACTIONS(1581),
    [sym_data_max_name_length] = ACTIONS(1581),
    [sym_data_max_description_length] = ACTIONS(1581),
    [sym_data_max_type_length] = ACTIONS(1581),
    [sym_data_max_format_length] = ACTIONS(1581),
    [sym_data_max_source_length] = ACTIONS(1581),
    [sym_data_max_value_length] = ACTIONS(1581),
    [sym_data_max_label_length] = ACTIONS(1581),
    [sym_data_max_url_length] = ACTIONS(1581),
    [sym_max_last_login_info_length] = ACTIONS(1581),
    [sym_data_type_unknown] = ACTIONS(1581),
    [sym_data_type_string] = ACTIONS(1581),
    [sym_data_type_datetime] = ACTIONS(1581),
    [sym_data_type_date] = ACTIONS(1581),
    [sym_data_type_time] = ACTIONS(1581),
    [sym_data_type_image] = ACTIONS(1581),
    [sym_data_format_url] = ACTIONS(1581),
    [sym_data_format_phone] = ACTIONS(1581),
    [sym_data_format_email] = ACTIONS(1581),
    [sym_data_format_iso8601] = ACTIONS(1581),
    [sym_auditlog_priv] = ACTIONS(1581),
    [sym_configuration_priv] = ACTIONS(1581),
    [sym_deviceconfig_priv] = ACTIONS(1581),
    [sym_ftp_priv] = ACTIONS(1581),
    [sym_http_priv] = ACTIONS(1581),
    [sym_networkconfig_priv] = ACTIONS(1581),
    [sym_programport_priv] = ACTIONS(1581),
    [sym_remoteui_priv] = ACTIONS(1581),
    [sym_securitycontrol_priv] = ACTIONS(1581),
    [sym_softwaremanagement_priv] = ACTIONS(1581),
    [sym_terminal_priv] = ACTIONS(1581),
    [sym_tpadmin_priv] = ACTIONS(1581),
    [sym_user1_priv] = ACTIONS(1581),
    [sym_user2_priv] = ACTIONS(1581),
    [sym_user3_priv] = ACTIONS(1581),
    [sym_user4_priv] = ACTIONS(1581),
    [sym_usermanagement_priv] = ACTIONS(1581),
    [sym_valid_account] = ACTIONS(1581),
    [sym_err_name_invalid] = ACTIONS(1581),
    [sym_err_password_invalid] = ACTIONS(1581),
    [sym_err_invalid_account] = ACTIONS(1581),
    [sym_err_no_authorization] = ACTIONS(1581),
    [sym_err_invalid_parameter] = ACTIONS(1581),
    [sym_err_user_locked_out] = ACTIONS(1581),
    [sym_err_user_account_expired] = ACTIONS(1581),
    [sym_err_general_error] = ACTIONS(1581),
    [sym_err_authentication_error] = ACTIONS(1581),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1581),
    [sym_tls_validate_certificate] = ACTIONS(1581),
    [sym_tdata] = ACTIONS(1581),
    [sym_tchannel] = ACTIONS(1581),
    [sym_tlevel] = ACTIONS(1581),
    [sym_tbutton] = ACTIONS(1581),
    [sym_ttimeline] = ACTIONS(1581),
    [sym_tcustom] = ACTIONS(1581),
    [sym_url_struct] = ACTIONS(1581),
    [sym_dns_struct] = ACTIONS(1581),
    [sym_ip_address_struct] = ACTIONS(1581),
    [sym_dev_info_struct] = ACTIONS(1581),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1581),
    [sym_data_feed] = ACTIONS(1581),
    [sym_data_field] = ACTIONS(1581),
    [sym_data_record] = ACTIONS(1581),
    [sym_wc_data_feed] = ACTIONS(1581),
    [sym_wc_data_field] = ACTIONS(1581),
    [sym_wc_data_record] = ACTIONS(1581),
    [sym_last_login_info] = ACTIONS(1581),
    [sym____reserved___] = ACTIONS(1581),
    [sym_date] = ACTIONS(1581),
    [sym_day] = ACTIONS(1581),
    [sym_time] = ACTIONS(1581),
    [sym_ldate] = ACTIONS(1581),
    [sym_push_device] = ACTIONS(1581),
    [sym_push_channel] = ACTIONS(1581),
    [sym_push_devchan] = ACTIONS(1581),
    [sym_release_device] = ACTIONS(1581),
    [sym_release_channel] = ACTIONS(1581),
    [sym_release_devchan] = ACTIONS(1581),
    [sym_master_slot] = ACTIONS(1581),
    [sym_get_pulse_time] = ACTIONS(1581),
    [sym_get_timer] = ACTIONS(1581),
    [sym_master_sn] = ACTIONS(1581),
    [sym_system_number] = ACTIONS(1581),
    [sym_button] = ACTIONS(1581),
    [sym_dv_channel] = ACTIONS(1581),
    [sym_data] = ACTIONS(1581),
    [sym_level] = ACTIONS(1581),
    [sym_timeline] = ACTIONS(1581),
    [sym_channel] = ACTIONS(1581),
    [sym_custom] = ACTIONS(1581),
    [sym_atoi] = ACTIONS(1581),
    [sym_atol] = ACTIONS(1581),
    [sym_atof] = ACTIONS(1581),
    [sym_itoa] = ACTIONS(1581),
    [sym_itohex] = ACTIONS(1581),
    [sym_hextoi] = ACTIONS(1581),
    [sym_ftoa] = ACTIONS(1581),
    [sym_find_string] = ACTIONS(1581),
    [sym_left_string] = ACTIONS(1581),
    [sym_right_string] = ACTIONS(1581),
    [sym_mid_string] = ACTIONS(1581),
    [sym_remove_string] = ACTIONS(1581),
    [sym_get_buffer_string] = ACTIONS(1581),
    [sym_get_buffer_char] = ACTIONS(1581),
    [sym_get_multi_buffer_string] = ACTIONS(1581),
    [sym_length_string] = ACTIONS(1581),
    [sym_max_length_string] = ACTIONS(1581),
    [sym_set_length_string] = ACTIONS(1581),
    [sym_compare_string] = ACTIONS(1581),
    [sym_upper_string] = ACTIONS(1581),
    [sym_lower_string] = ACTIONS(1581),
    [sym_get_last] = ACTIONS(1581),
    [sym_date_to_day] = ACTIONS(1581),
    [sym_date_to_month] = ACTIONS(1581),
    [sym_date_to_year] = ACTIONS(1581),
    [sym_time_to_hour] = ACTIONS(1581),
    [sym_time_to_minute] = ACTIONS(1581),
    [sym_time_to_second] = ACTIONS(1581),
    [sym_day_of_week] = ACTIONS(1581),
    [sym_set_pulse_time] = ACTIONS(1581),
    [sym_set_timer] = ACTIONS(1581),
    [sym_file_open] = ACTIONS(1581),
    [sym_file_close] = ACTIONS(1581),
    [sym_file_read] = ACTIONS(1581),
    [sym_file_read_line] = ACTIONS(1581),
    [sym_file_write] = ACTIONS(1581),
    [sym_file_write_line] = ACTIONS(1581),
    [sym_file_copy] = ACTIONS(1581),
    [sym_file_delete] = ACTIONS(1581),
    [sym_file_rename] = ACTIONS(1581),
    [sym_file_dir] = ACTIONS(1581),
    [sym_file_getdir] = ACTIONS(1581),
    [sym_file_setdir] = ACTIONS(1581),
    [sym_file_create_dir] = ACTIONS(1581),
    [sym_file_remove_dir] = ACTIONS(1581),
    [sym_file_seek] = ACTIONS(1581),
    [sym_length_array] = ACTIONS(1581),
    [sym_max_length_array] = ACTIONS(1581),
    [sym_set_length_array] = ACTIONS(1581),
    [sym_ip_client_open] = ACTIONS(1581),
    [sym_ip_client_close] = ACTIONS(1581),
    [sym_ip_server_open] = ACTIONS(1581),
    [sym_ip_server_close] = ACTIONS(1581),
    [sym_ip_bound_client_open] = ACTIONS(1581),
    [sym_ip_mc_server_open] = ACTIONS(1581),
    [sym_ip_set_option] = ACTIONS(1581),
    [sym_ssh_client_open] = ACTIONS(1581),
    [sym_ssh_client_close] = ACTIONS(1581),
    [sym_tls_client_open] = ACTIONS(1581),
    [sym_tls_client_close] = ACTIONS(1581),
    [sym_do_push] = ACTIONS(1581),
    [sym_do_push_timed] = ACTIONS(1581),
    [sym_do_release] = ACTIONS(1581),
    [sym_device_id] = ACTIONS(1581),
    [sym_device_info] = ACTIONS(1581),
    [sym_device_id_string] = ACTIONS(1581),
    [sym_device_standby] = ACTIONS(1581),
    [sym_device_wake] = ACTIONS(1581),
    [sym_timeline_create] = ACTIONS(1581),
    [sym_timeline_kill] = ACTIONS(1581),
    [sym_timeline_reload] = ACTIONS(1581),
    [sym_timeline_pause] = ACTIONS(1581),
    [sym_timeline_restart] = ACTIONS(1581),
    [sym_timeline_set] = ACTIONS(1581),
    [sym_timeline_get] = ACTIONS(1581),
    [sym_timeline_active] = ACTIONS(1581),
    [sym_timeline_dynamic_id] = ACTIONS(1581),
    [sym_variable_to_string] = ACTIONS(1581),
    [sym_string_to_variable] = ACTIONS(1581),
    [sym_length_variable_to_string] = ACTIONS(1581),
    [sym_type_cast] = ACTIONS(1581),
    [sym_variable_to_xml] = ACTIONS(1581),
    [sym_xml_to_variable] = ACTIONS(1581),
    [sym_length_variable_to_xml] = ACTIONS(1581),
    [sym_redirect_string] = ACTIONS(1581),
    [sym_random_number] = ACTIONS(1581),
    [sym_get_unique_id] = ACTIONS(1581),
    [sym_get_system_number] = ACTIONS(1581),
    [sym_set_system_number] = ACTIONS(1581),
    [sym_get_serial_number] = ACTIONS(1581),
    [sym_get_master_build] = ACTIONS(1581),
    [sym_set_validation_code] = ACTIONS(1581),
    [sym_reboot] = ACTIONS(1581),
    [sym_load_duet_module] = ACTIONS(1581),
    [sym_unload_duet_module] = ACTIONS(1581),
    [sym_duet_mem_size_set] = ACTIONS(1581),
    [sym_duet_mem_size_get] = ACTIONS(1581),
    [sym_dynamic_application_device] = ACTIONS(1581),
    [sym_dynamic_polled_port] = ACTIONS(1581),
    [sym_static_port_binding] = ACTIONS(1581),
    [sym_static_ip_binding] = ACTIONS(1581),
    [sym_get_dns_list] = ACTIONS(1581),
    [sym_set_dns_list] = ACTIONS(1581),
    [sym_get_ip_address] = ACTIONS(1581),
    [sym_set_ip_address] = ACTIONS(1581),
    [sym_get_url_list] = ACTIONS(1581),
    [sym_add_url_entry] = ACTIONS(1581),
    [sym_delete_url_entry] = ACTIONS(1581),
    [sym_abs_value] = ACTIONS(1581),
    [sym_max_value] = ACTIONS(1581),
    [sym_min_value] = ACTIONS(1581),
    [sym_exp_value] = ACTIONS(1581),
    [sym_log_value] = ACTIONS(1581),
    [sym_log10_value] = ACTIONS(1581),
    [sym_power_value] = ACTIONS(1581),
    [sym_sqrt_value] = ACTIONS(1581),
    [sym_raw_be] = ACTIONS(1581),
    [sym_raw_le] = ACTIONS(1581),
    [sym_internal_queue_size_set] = ACTIONS(1581),
    [sym_internal_queue_size_get] = ACTIONS(1581),
    [sym_internal_threshold_set] = ACTIONS(1581),
    [sym_internal_threshold_get] = ACTIONS(1581),
    [sym_rebuilt_event] = ACTIONS(1581),
    [sym_do_custom_event] = ACTIONS(1581),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1581),
    [sym_clkmgr_set_clk_source] = ACTIONS(1581),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1581),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1581),
    [sym_clkmgr_get_timezone] = ACTIONS(1581),
    [sym_clkmgr_set_timezone] = ACTIONS(1581),
    [sym_clkmgr_get_resync_period] = ACTIONS(1581),
    [sym_clkmgr_set_resync_period] = ACTIONS(1581),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1581),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1581),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1581),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1581),
    [sym_clkmgr_get_timeservers] = ACTIONS(1581),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1581),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1581),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1581),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1581),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1581),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1581),
    [sym_set_log_level] = ACTIONS(1581),
    [sym_get_log_level] = ACTIONS(1581),
    [sym_amx_log] = ACTIONS(1581),
    [sym_smtp_server_config_set] = ACTIONS(1581),
    [sym_smtp_server_config_get] = ACTIONS(1581),
    [sym_smtp_send] = ACTIONS(1581),
    [sym_validate_netlinx_account] = ACTIONS(1581),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1581),
    [sym_authenticate_certificate] = ACTIONS(1581),
    [sym_audit_netlinx_session_event] = ACTIONS(1581),
    [sym_audit_netlinx_generic_event] = ACTIONS(1581),
    [sym_get_available_flash_disk_space] = ACTIONS(1581),
    [sym_get_max_flash_disk_space] = ACTIONS(1581),
    [sym_data_create_feed] = ACTIONS(1581),
    [sym_data_delete_feed] = ACTIONS(1581),
    [sym_data_publish_feed] = ACTIONS(1581),
    [sym_data_get_published_feed] = ACTIONS(1581),
    [sym_data_add_record] = ACTIONS(1581),
    [sym_data_get_event_record] = ACTIONS(1581),
    [sym__wc_data_create_feed] = ACTIONS(1581),
    [sym__wc_data_add_record] = ACTIONS(1581),
    [sym__wc_data_get_event_record] = ACTIONS(1581),
    [sym_format] = ACTIONS(1581),
    [sym_astro_clock] = ACTIONS(1581),
    [sym_set_outdoor_temperature] = ACTIONS(1581),
    [sym_set_virtual_level_count] = ACTIONS(1581),
    [sym_set_virtual_channel_count] = ACTIONS(1581),
    [sym_set_virtual_port_count] = ACTIONS(1581),
    [sym__wc_encode_int] = ACTIONS(1581),
    [sym__wc_decode_int] = ACTIONS(1581),
    [sym__wc_int] = ACTIONS(1581),
    [sym__wc_to_ch_int] = ACTIONS(1581),
    [sym__ch_to_wc_int] = ACTIONS(1581),
    [sym__wc_find_string_int] = ACTIONS(1581),
    [sym__wc_left_string_int] = ACTIONS(1581),
    [sym__wc_lower_string_int] = ACTIONS(1581),
    [sym__wc_mid_string_int] = ACTIONS(1581),
    [sym__wc_remove_string_int] = ACTIONS(1581),
    [sym__wc_right_string_int] = ACTIONS(1581),
    [sym__wc_upper_string_int] = ACTIONS(1581),
    [sym__wc_compare_string_int] = ACTIONS(1581),
    [sym__wc_get_buffer_char_int] = ACTIONS(1581),
    [sym__wc_get_buffer_string_int] = ACTIONS(1581),
    [sym__wc_concat_string_int] = ACTIONS(1581),
    [sym__wc_file_open_int] = ACTIONS(1581),
    [sym__wc_file_close_int] = ACTIONS(1581),
    [sym__wc_file_read_int] = ACTIONS(1581),
    [sym__wc_file_read_line_int] = ACTIONS(1581),
    [sym__wc_file_write_int] = ACTIONS(1581),
    [sym__wc_file_write_line_int] = ACTIONS(1581),
    [aux_sym_preproc_include_token1] = ACTIONS(1581),
    [aux_sym_preproc_define_token1] = ACTIONS(1583),
    [aux_sym_preproc_warn_token1] = ACTIONS(1583),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1583),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1583),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1583),
    [anon_sym_LBRACK] = ACTIONS(1583),
    [anon_sym_LBRACE] = ACTIONS(1583),
    [anon_sym_COLON] = ACTIONS(1583),
    [anon_sym_LPAREN] = ACTIONS(1581),
    [anon_sym_SEMI] = ACTIONS(1583),
    [anon_sym_BANG] = ACTIONS(1581),
    [anon_sym_TILDE] = ACTIONS(1583),
    [anon_sym_DASH] = ACTIONS(1581),
    [anon_sym_PLUS] = ACTIONS(1581),
    [anon_sym_STAR] = ACTIONS(1583),
    [anon_sym_SLASH] = ACTIONS(1581),
    [anon_sym_PERCENT] = ACTIONS(1583),
    [anon_sym_PIPE_PIPE] = ACTIONS(1583),
    [anon_sym_AMP_AMP] = ACTIONS(1583),
    [anon_sym_PIPE] = ACTIONS(1581),
    [anon_sym_CARET] = ACTIONS(1583),
    [anon_sym_AMP] = ACTIONS(1581),
    [anon_sym_EQ_EQ] = ACTIONS(1583),
    [anon_sym_BANG_EQ] = ACTIONS(1583),
    [anon_sym_LT_GT] = ACTIONS(1583),
    [anon_sym_GT] = ACTIONS(1581),
    [anon_sym_GT_EQ] = ACTIONS(1583),
    [anon_sym_LT_EQ] = ACTIONS(1583),
    [anon_sym_LT] = ACTIONS(1581),
    [anon_sym_LT_LT] = ACTIONS(1583),
    [anon_sym_GT_GT] = ACTIONS(1583),
    [anon_sym_DASH_DASH] = ACTIONS(1583),
    [anon_sym_PLUS_PLUS] = ACTIONS(1583),
    [anon_sym_DOT] = ACTIONS(1581),
    [anon_sym_DQUOTE] = ACTIONS(1583),
    [anon_sym_SQUOTE] = ACTIONS(1583),
    [sym_number_literal] = ACTIONS(1583),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1583),
  },
  [STATE(144)] = {
    [ts_builtin_sym_end] = ACTIONS(1571),
    [sym_identifier] = ACTIONS(1569),
    [sym_program_name_keyword] = ACTIONS(1569),
    [sym_module_name_keyword] = ACTIONS(1569),
    [sym_define_device_keyword] = ACTIONS(1569),
    [sym_define_combine_keyword] = ACTIONS(1569),
    [sym_define_constant_keyword] = ACTIONS(1569),
    [sym_define_type_keyword] = ACTIONS(1569),
    [sym_define_variable_keyword] = ACTIONS(1569),
    [sym_define_system_variable_keyword] = ACTIONS(1569),
    [sym_define_connect_level_keyword] = ACTIONS(1569),
    [sym_define_latching_keyword] = ACTIONS(1569),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1569),
    [sym_define_toggling_keyword] = ACTIONS(1569),
    [sym_define_call_keyword] = ACTIONS(1569),
    [sym_define_function_keyword] = ACTIONS(1569),
    [sym_define_library_function_keyword] = ACTIONS(1569),
    [sym_define_start_keyword] = ACTIONS(1569),
    [sym_define_event_keyword] = ACTIONS(1569),
    [sym_define_program_keyword] = ACTIONS(1569),
    [sym_define_module_keyword] = ACTIONS(1569),
    [sym_char_keyword] = ACTIONS(1569),
    [sym_widechar_keyword] = ACTIONS(1569),
    [sym_integer_keyword] = ACTIONS(1569),
    [sym_sinteger_keyword] = ACTIONS(1569),
    [sym_long_keyword] = ACTIONS(1569),
    [sym_slong_keyword] = ACTIONS(1569),
    [sym_float_keyword] = ACTIONS(1569),
    [sym_double_keyword] = ACTIONS(1569),
    [sym_variant_keyword] = ACTIONS(1569),
    [sym_variantarray_keyword] = ACTIONS(1569),
    [sym_local_var_keyword] = ACTIONS(1569),
    [sym_stack_var_keyword] = ACTIONS(1569),
    [sym_constant_keyword] = ACTIONS(1569),
    [sym_volatile_keyword] = ACTIONS(1569),
    [sym_non_volatile_keyword] = ACTIONS(1569),
    [sym_persistent_keyword] = ACTIONS(1569),
    [sym_dev_keyword] = ACTIONS(1569),
    [sym_devlev_keyword] = ACTIONS(1569),
    [sym_devchan_keyword] = ACTIONS(1569),
    [sym_if_keyword] = ACTIONS(1569),
    [sym_else_keyword] = ACTIONS(1569),
    [sym_switch_keyword] = ACTIONS(1569),
    [sym_case_keyword] = ACTIONS(1569),
    [sym_default_keyword] = ACTIONS(1569),
    [sym_while_keyword] = ACTIONS(1569),
    [sym_for_keyword] = ACTIONS(1569),
    [sym_break_keyword] = ACTIONS(1569),
    [sym_continue_keyword] = ACTIONS(1569),
    [sym_return_keyword] = ACTIONS(1569),
    [sym_select_keyword] = ACTIONS(1569),
    [sym_active_keyword] = ACTIONS(1569),
    [sym_struct_keyword] = ACTIONS(1569),
    [sym_structure_keyword] = ACTIONS(1569),
    [sym_button_event_keyword] = ACTIONS(1569),
    [sym_channel_event_keyword] = ACTIONS(1569),
    [sym_level_event_keyword] = ACTIONS(1569),
    [sym_data_event_keyword] = ACTIONS(1569),
    [sym_timeline_event_keyword] = ACTIONS(1569),
    [sym_custom_event_keyword] = ACTIONS(1569),
    [sym_push_keyword] = ACTIONS(1569),
    [sym_release_keyword] = ACTIONS(1569),
    [aux_sym_on_keyword_token1] = ACTIONS(1569),
    [aux_sym_off_keyword_token1] = ACTIONS(1569),
    [sym_send_level_keyword] = ACTIONS(1569),
    [sym_send_string_keyword] = ACTIONS(1569),
    [sym_send_command_keyword] = ACTIONS(1569),
    [sym_clear_buffer_keyword] = ACTIONS(1569),
    [sym_create_buffer_keyword] = ACTIONS(1569),
    [sym_create_multi_buffer_keyword] = ACTIONS(1569),
    [sym_call_keyword] = ACTIONS(1569),
    [sym_system_call_keyword] = ACTIONS(1569),
    [sym_devchan_to_keyword] = ACTIONS(1569),
    [sym_devchan_min_to_keyword] = ACTIONS(1569),
    [sym_devchan_total_off_keyword] = ACTIONS(1569),
    [sym_devchan_pulse_keyword] = ACTIONS(1569),
    [sym_wait_keyword] = ACTIONS(1569),
    [sym_cancel_wait_keyword] = ACTIONS(1569),
    [sym_wait_until_keyword] = ACTIONS(1569),
    [sym_cancel_wait_until_keyword] = ACTIONS(1569),
    [sym_cancel_all_wait_keyword] = ACTIONS(1569),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1569),
    [sym_and] = ACTIONS(1569),
    [sym_or] = ACTIONS(1569),
    [sym_not] = ACTIONS(1569),
    [sym_xor] = ACTIONS(1569),
    [sym_band] = ACTIONS(1569),
    [sym_bor] = ACTIONS(1569),
    [sym_bxor] = ACTIONS(1569),
    [sym_bnot] = ACTIONS(1569),
    [sym_lshift] = ACTIONS(1569),
    [sym_rshift] = ACTIONS(1569),
    [sym___file__] = ACTIONS(1569),
    [sym___date__] = ACTIONS(1569),
    [sym___ldate__] = ACTIONS(1569),
    [sym___time__] = ACTIONS(1569),
    [sym___line__] = ACTIONS(1569),
    [sym___name__] = ACTIONS(1569),
    [sym___netlinx__] = ACTIONS(1569),
    [sym_netlinx_axi_version] = ACTIONS(1569),
    [sym_url_flg_tcp] = ACTIONS(1569),
    [sym_url_flg_accinfopresent] = ACTIONS(1569),
    [sym_url_flg_temp] = ACTIONS(1569),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1569),
    [sym_url_flg_stat_mask] = ACTIONS(1569),
    [sym_url_flg_stat_lookup] = ACTIONS(1569),
    [sym_url_flg_stat_connecting] = ACTIONS(1569),
    [sym_url_flg_stat_waiting] = ACTIONS(1569),
    [sym_url_flg_stat_connected] = ACTIONS(1569),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1569),
    [sym_first_virtual_device] = ACTIONS(1569),
    [sym_first_local_port] = ACTIONS(1569),
    [sym_dynamic_virtual_device] = ACTIONS(1569),
    [sym_timeline_once] = ACTIONS(1569),
    [sym_timeline_repeat] = ACTIONS(1569),
    [sym_timeline_absolute] = ACTIONS(1569),
    [sym_timeline_relative] = ACTIONS(1569),
    [sym_do_push_timed_infinite] = ACTIONS(1569),
    [sym_source_type_no_address] = ACTIONS(1569),
    [sym_source_type_neuron_id] = ACTIONS(1569),
    [sym_source_type_ip_address] = ACTIONS(1569),
    [sym_source_type_axlink] = ACTIONS(1569),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1569),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1569),
    [sym_source_type_ip_socket_address] = ACTIONS(1569),
    [sym_source_type_rs232] = ACTIONS(1569),
    [sym_source_type_internal] = ACTIONS(1569),
    [sym_source_type_mac_address] = ACTIONS(1569),
    [sym_source_type_ipv4_port] = ACTIONS(1569),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1569),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1569),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1569),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1569),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1569),
    [sym_true] = ACTIONS(1569),
    [sym_false] = ACTIONS(1569),
    [sym_file_read_only] = ACTIONS(1569),
    [sym_file_rw_new] = ACTIONS(1569),
    [sym_file_rw_append] = ACTIONS(1569),
    [sym_ip_tcp] = ACTIONS(1569),
    [sym_ip_udp] = ACTIONS(1569),
    [sym_ip_udp_2way] = ACTIONS(1569),
    [sym_xml_encode_types] = ACTIONS(1569),
    [sym_xml_encode_char_as_list] = ACTIONS(1569),
    [sym_xml_encode_le] = ACTIONS(1569),
    [sym_xml_decode_types] = ACTIONS(1569),
    [sym_xml_decode_no_preserve] = ACTIONS(1569),
    [sym_duet_dev_type_display_device] = ACTIONS(1569),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1569),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1569),
    [sym_duet_dev_type_io_device] = ACTIONS(1569),
    [sym_duet_dev_type_relay_device] = ACTIONS(1569),
    [sym_duet_dev_type_ups] = ACTIONS(1569),
    [sym_duet_dev_type_amplifier] = ACTIONS(1569),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1569),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1569),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1569),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1569),
    [sym_duet_dev_type_camera] = ACTIONS(1569),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1569),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1569),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1569),
    [sym_duet_dev_type_dss] = ACTIONS(1569),
    [sym_duet_dev_type_dvr] = ACTIONS(1569),
    [sym_duet_dev_type_disc_device] = ACTIONS(1569),
    [sym_duet_dev_type_document_camera] = ACTIONS(1569),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1569),
    [sym_duet_dev_type_hvac] = ACTIONS(1569),
    [sym_duet_dev_type_keypad] = ACTIONS(1569),
    [sym_duet_dev_type_light] = ACTIONS(1569),
    [sym_duet_dev_type_monitor] = ACTIONS(1569),
    [sym_duet_dev_type_motor] = ACTIONS(1569),
    [sym_duet_dev_type_multi_window] = ACTIONS(1569),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1569),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1569),
    [sym_duet_dev_type_receiver] = ACTIONS(1569),
    [sym_duet_dev_type_security_system] = ACTIONS(1569),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1569),
    [sym_duet_dev_type_settop_box] = ACTIONS(1569),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1569),
    [sym_duet_dev_type_switcher] = ACTIONS(1569),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1569),
    [sym_duet_dev_type_tv] = ACTIONS(1569),
    [sym_duet_dev_type_utility] = ACTIONS(1569),
    [sym_duet_dev_type_vcr] = ACTIONS(1569),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1569),
    [sym_duet_dev_type_video_processor] = ACTIONS(1569),
    [sym_duet_dev_type_video_projector] = ACTIONS(1569),
    [sym_duet_dev_type_video_wall] = ACTIONS(1569),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1569),
    [sym_duet_dev_type_weather] = ACTIONS(1569),
    [sym_duet_dev_not_polled] = ACTIONS(1569),
    [sym_duet_dev_polled] = ACTIONS(1569),
    [sym_clkmgr_mode_network] = ACTIONS(1569),
    [sym_clkmgr_mode_standalone] = ACTIONS(1569),
    [sym_normal_standby] = ACTIONS(1569),
    [sym_normal_wake] = ACTIONS(1569),
    [sym_ip_multicast_ttl_option] = ACTIONS(1569),
    [sym_ip_ttl_subnet] = ACTIONS(1569),
    [sym_ip_ttl_site] = ACTIONS(1569),
    [sym_ip_ttl_region] = ACTIONS(1569),
    [sym_ip_ttl_continent] = ACTIONS(1569),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1569),
    [sym_ip_nodelay_on] = ACTIONS(1569),
    [sym_ip_nodelay_off] = ACTIONS(1569),
    [sym_amx_error] = ACTIONS(1569),
    [sym_amx_warning] = ACTIONS(1569),
    [sym_amx_info] = ACTIONS(1569),
    [sym_amx_debug] = ACTIONS(1569),
    [sym_netlinx_login_success] = ACTIONS(1569),
    [sym_netlinx_login_fail] = ACTIONS(1569),
    [sym_netlinx_logout] = ACTIONS(1569),
    [sym_null_str] = ACTIONS(1569),
    [sym_smtp_address] = ACTIONS(1569),
    [sym_smtp_port_number] = ACTIONS(1569),
    [sym_smtp_username] = ACTIONS(1569),
    [sym_smtp_password] = ACTIONS(1569),
    [sym_smtp_require_tls] = ACTIONS(1569),
    [sym_smtp_from] = ACTIONS(1569),
    [sym_smtp_tls_true] = ACTIONS(1569),
    [sym_smtp_tls_false] = ACTIONS(1569),
    [sym_listview_on_row_select_event] = ACTIONS(1569),
    [sym_data_structure_unknown] = ACTIONS(1569),
    [sym_data_structure_datafeed] = ACTIONS(1569),
    [sym_data_structure_datarecordset] = ACTIONS(1569),
    [sym_data_structure_datarecord] = ACTIONS(1569),
    [sym_data_structure_datafield] = ACTIONS(1569),
    [sym_data_max_content_field_count] = ACTIONS(1569),
    [sym_data_max_metadata_field_count] = ACTIONS(1569),
    [sym_data_max_id_length] = ACTIONS(1569),
    [sym_data_max_name_length] = ACTIONS(1569),
    [sym_data_max_description_length] = ACTIONS(1569),
    [sym_data_max_type_length] = ACTIONS(1569),
    [sym_data_max_format_length] = ACTIONS(1569),
    [sym_data_max_source_length] = ACTIONS(1569),
    [sym_data_max_value_length] = ACTIONS(1569),
    [sym_data_max_label_length] = ACTIONS(1569),
    [sym_data_max_url_length] = ACTIONS(1569),
    [sym_max_last_login_info_length] = ACTIONS(1569),
    [sym_data_type_unknown] = ACTIONS(1569),
    [sym_data_type_string] = ACTIONS(1569),
    [sym_data_type_datetime] = ACTIONS(1569),
    [sym_data_type_date] = ACTIONS(1569),
    [sym_data_type_time] = ACTIONS(1569),
    [sym_data_type_image] = ACTIONS(1569),
    [sym_data_format_url] = ACTIONS(1569),
    [sym_data_format_phone] = ACTIONS(1569),
    [sym_data_format_email] = ACTIONS(1569),
    [sym_data_format_iso8601] = ACTIONS(1569),
    [sym_auditlog_priv] = ACTIONS(1569),
    [sym_configuration_priv] = ACTIONS(1569),
    [sym_deviceconfig_priv] = ACTIONS(1569),
    [sym_ftp_priv] = ACTIONS(1569),
    [sym_http_priv] = ACTIONS(1569),
    [sym_networkconfig_priv] = ACTIONS(1569),
    [sym_programport_priv] = ACTIONS(1569),
    [sym_remoteui_priv] = ACTIONS(1569),
    [sym_securitycontrol_priv] = ACTIONS(1569),
    [sym_softwaremanagement_priv] = ACTIONS(1569),
    [sym_terminal_priv] = ACTIONS(1569),
    [sym_tpadmin_priv] = ACTIONS(1569),
    [sym_user1_priv] = ACTIONS(1569),
    [sym_user2_priv] = ACTIONS(1569),
    [sym_user3_priv] = ACTIONS(1569),
    [sym_user4_priv] = ACTIONS(1569),
    [sym_usermanagement_priv] = ACTIONS(1569),
    [sym_valid_account] = ACTIONS(1569),
    [sym_err_name_invalid] = ACTIONS(1569),
    [sym_err_password_invalid] = ACTIONS(1569),
    [sym_err_invalid_account] = ACTIONS(1569),
    [sym_err_no_authorization] = ACTIONS(1569),
    [sym_err_invalid_parameter] = ACTIONS(1569),
    [sym_err_user_locked_out] = ACTIONS(1569),
    [sym_err_user_account_expired] = ACTIONS(1569),
    [sym_err_general_error] = ACTIONS(1569),
    [sym_err_authentication_error] = ACTIONS(1569),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1569),
    [sym_tls_validate_certificate] = ACTIONS(1569),
    [sym_tdata] = ACTIONS(1569),
    [sym_tchannel] = ACTIONS(1569),
    [sym_tlevel] = ACTIONS(1569),
    [sym_tbutton] = ACTIONS(1569),
    [sym_ttimeline] = ACTIONS(1569),
    [sym_tcustom] = ACTIONS(1569),
    [sym_url_struct] = ACTIONS(1569),
    [sym_dns_struct] = ACTIONS(1569),
    [sym_ip_address_struct] = ACTIONS(1569),
    [sym_dev_info_struct] = ACTIONS(1569),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1569),
    [sym_data_feed] = ACTIONS(1569),
    [sym_data_field] = ACTIONS(1569),
    [sym_data_record] = ACTIONS(1569),
    [sym_wc_data_feed] = ACTIONS(1569),
    [sym_wc_data_field] = ACTIONS(1569),
    [sym_wc_data_record] = ACTIONS(1569),
    [sym_last_login_info] = ACTIONS(1569),
    [sym____reserved___] = ACTIONS(1569),
    [sym_date] = ACTIONS(1569),
    [sym_day] = ACTIONS(1569),
    [sym_time] = ACTIONS(1569),
    [sym_ldate] = ACTIONS(1569),
    [sym_push_device] = ACTIONS(1569),
    [sym_push_channel] = ACTIONS(1569),
    [sym_push_devchan] = ACTIONS(1569),
    [sym_release_device] = ACTIONS(1569),
    [sym_release_channel] = ACTIONS(1569),
    [sym_release_devchan] = ACTIONS(1569),
    [sym_master_slot] = ACTIONS(1569),
    [sym_get_pulse_time] = ACTIONS(1569),
    [sym_get_timer] = ACTIONS(1569),
    [sym_master_sn] = ACTIONS(1569),
    [sym_system_number] = ACTIONS(1569),
    [sym_button] = ACTIONS(1569),
    [sym_dv_channel] = ACTIONS(1569),
    [sym_data] = ACTIONS(1569),
    [sym_level] = ACTIONS(1569),
    [sym_timeline] = ACTIONS(1569),
    [sym_channel] = ACTIONS(1569),
    [sym_custom] = ACTIONS(1569),
    [sym_atoi] = ACTIONS(1569),
    [sym_atol] = ACTIONS(1569),
    [sym_atof] = ACTIONS(1569),
    [sym_itoa] = ACTIONS(1569),
    [sym_itohex] = ACTIONS(1569),
    [sym_hextoi] = ACTIONS(1569),
    [sym_ftoa] = ACTIONS(1569),
    [sym_find_string] = ACTIONS(1569),
    [sym_left_string] = ACTIONS(1569),
    [sym_right_string] = ACTIONS(1569),
    [sym_mid_string] = ACTIONS(1569),
    [sym_remove_string] = ACTIONS(1569),
    [sym_get_buffer_string] = ACTIONS(1569),
    [sym_get_buffer_char] = ACTIONS(1569),
    [sym_get_multi_buffer_string] = ACTIONS(1569),
    [sym_length_string] = ACTIONS(1569),
    [sym_max_length_string] = ACTIONS(1569),
    [sym_set_length_string] = ACTIONS(1569),
    [sym_compare_string] = ACTIONS(1569),
    [sym_upper_string] = ACTIONS(1569),
    [sym_lower_string] = ACTIONS(1569),
    [sym_get_last] = ACTIONS(1569),
    [sym_date_to_day] = ACTIONS(1569),
    [sym_date_to_month] = ACTIONS(1569),
    [sym_date_to_year] = ACTIONS(1569),
    [sym_time_to_hour] = ACTIONS(1569),
    [sym_time_to_minute] = ACTIONS(1569),
    [sym_time_to_second] = ACTIONS(1569),
    [sym_day_of_week] = ACTIONS(1569),
    [sym_set_pulse_time] = ACTIONS(1569),
    [sym_set_timer] = ACTIONS(1569),
    [sym_file_open] = ACTIONS(1569),
    [sym_file_close] = ACTIONS(1569),
    [sym_file_read] = ACTIONS(1569),
    [sym_file_read_line] = ACTIONS(1569),
    [sym_file_write] = ACTIONS(1569),
    [sym_file_write_line] = ACTIONS(1569),
    [sym_file_copy] = ACTIONS(1569),
    [sym_file_delete] = ACTIONS(1569),
    [sym_file_rename] = ACTIONS(1569),
    [sym_file_dir] = ACTIONS(1569),
    [sym_file_getdir] = ACTIONS(1569),
    [sym_file_setdir] = ACTIONS(1569),
    [sym_file_create_dir] = ACTIONS(1569),
    [sym_file_remove_dir] = ACTIONS(1569),
    [sym_file_seek] = ACTIONS(1569),
    [sym_length_array] = ACTIONS(1569),
    [sym_max_length_array] = ACTIONS(1569),
    [sym_set_length_array] = ACTIONS(1569),
    [sym_ip_client_open] = ACTIONS(1569),
    [sym_ip_client_close] = ACTIONS(1569),
    [sym_ip_server_open] = ACTIONS(1569),
    [sym_ip_server_close] = ACTIONS(1569),
    [sym_ip_bound_client_open] = ACTIONS(1569),
    [sym_ip_mc_server_open] = ACTIONS(1569),
    [sym_ip_set_option] = ACTIONS(1569),
    [sym_ssh_client_open] = ACTIONS(1569),
    [sym_ssh_client_close] = ACTIONS(1569),
    [sym_tls_client_open] = ACTIONS(1569),
    [sym_tls_client_close] = ACTIONS(1569),
    [sym_do_push] = ACTIONS(1569),
    [sym_do_push_timed] = ACTIONS(1569),
    [sym_do_release] = ACTIONS(1569),
    [sym_device_id] = ACTIONS(1569),
    [sym_device_info] = ACTIONS(1569),
    [sym_device_id_string] = ACTIONS(1569),
    [sym_device_standby] = ACTIONS(1569),
    [sym_device_wake] = ACTIONS(1569),
    [sym_timeline_create] = ACTIONS(1569),
    [sym_timeline_kill] = ACTIONS(1569),
    [sym_timeline_reload] = ACTIONS(1569),
    [sym_timeline_pause] = ACTIONS(1569),
    [sym_timeline_restart] = ACTIONS(1569),
    [sym_timeline_set] = ACTIONS(1569),
    [sym_timeline_get] = ACTIONS(1569),
    [sym_timeline_active] = ACTIONS(1569),
    [sym_timeline_dynamic_id] = ACTIONS(1569),
    [sym_variable_to_string] = ACTIONS(1569),
    [sym_string_to_variable] = ACTIONS(1569),
    [sym_length_variable_to_string] = ACTIONS(1569),
    [sym_type_cast] = ACTIONS(1569),
    [sym_variable_to_xml] = ACTIONS(1569),
    [sym_xml_to_variable] = ACTIONS(1569),
    [sym_length_variable_to_xml] = ACTIONS(1569),
    [sym_redirect_string] = ACTIONS(1569),
    [sym_random_number] = ACTIONS(1569),
    [sym_get_unique_id] = ACTIONS(1569),
    [sym_get_system_number] = ACTIONS(1569),
    [sym_set_system_number] = ACTIONS(1569),
    [sym_get_serial_number] = ACTIONS(1569),
    [sym_get_master_build] = ACTIONS(1569),
    [sym_set_validation_code] = ACTIONS(1569),
    [sym_reboot] = ACTIONS(1569),
    [sym_load_duet_module] = ACTIONS(1569),
    [sym_unload_duet_module] = ACTIONS(1569),
    [sym_duet_mem_size_set] = ACTIONS(1569),
    [sym_duet_mem_size_get] = ACTIONS(1569),
    [sym_dynamic_application_device] = ACTIONS(1569),
    [sym_dynamic_polled_port] = ACTIONS(1569),
    [sym_static_port_binding] = ACTIONS(1569),
    [sym_static_ip_binding] = ACTIONS(1569),
    [sym_get_dns_list] = ACTIONS(1569),
    [sym_set_dns_list] = ACTIONS(1569),
    [sym_get_ip_address] = ACTIONS(1569),
    [sym_set_ip_address] = ACTIONS(1569),
    [sym_get_url_list] = ACTIONS(1569),
    [sym_add_url_entry] = ACTIONS(1569),
    [sym_delete_url_entry] = ACTIONS(1569),
    [sym_abs_value] = ACTIONS(1569),
    [sym_max_value] = ACTIONS(1569),
    [sym_min_value] = ACTIONS(1569),
    [sym_exp_value] = ACTIONS(1569),
    [sym_log_value] = ACTIONS(1569),
    [sym_log10_value] = ACTIONS(1569),
    [sym_power_value] = ACTIONS(1569),
    [sym_sqrt_value] = ACTIONS(1569),
    [sym_raw_be] = ACTIONS(1569),
    [sym_raw_le] = ACTIONS(1569),
    [sym_internal_queue_size_set] = ACTIONS(1569),
    [sym_internal_queue_size_get] = ACTIONS(1569),
    [sym_internal_threshold_set] = ACTIONS(1569),
    [sym_internal_threshold_get] = ACTIONS(1569),
    [sym_rebuilt_event] = ACTIONS(1569),
    [sym_do_custom_event] = ACTIONS(1569),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1569),
    [sym_clkmgr_set_clk_source] = ACTIONS(1569),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1569),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1569),
    [sym_clkmgr_get_timezone] = ACTIONS(1569),
    [sym_clkmgr_set_timezone] = ACTIONS(1569),
    [sym_clkmgr_get_resync_period] = ACTIONS(1569),
    [sym_clkmgr_set_resync_period] = ACTIONS(1569),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1569),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1569),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1569),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1569),
    [sym_clkmgr_get_timeservers] = ACTIONS(1569),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1569),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1569),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1569),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1569),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1569),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1569),
    [sym_set_log_level] = ACTIONS(1569),
    [sym_get_log_level] = ACTIONS(1569),
    [sym_amx_log] = ACTIONS(1569),
    [sym_smtp_server_config_set] = ACTIONS(1569),
    [sym_smtp_server_config_get] = ACTIONS(1569),
    [sym_smtp_send] = ACTIONS(1569),
    [sym_validate_netlinx_account] = ACTIONS(1569),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1569),
    [sym_authenticate_certificate] = ACTIONS(1569),
    [sym_audit_netlinx_session_event] = ACTIONS(1569),
    [sym_audit_netlinx_generic_event] = ACTIONS(1569),
    [sym_get_available_flash_disk_space] = ACTIONS(1569),
    [sym_get_max_flash_disk_space] = ACTIONS(1569),
    [sym_data_create_feed] = ACTIONS(1569),
    [sym_data_delete_feed] = ACTIONS(1569),
    [sym_data_publish_feed] = ACTIONS(1569),
    [sym_data_get_published_feed] = ACTIONS(1569),
    [sym_data_add_record] = ACTIONS(1569),
    [sym_data_get_event_record] = ACTIONS(1569),
    [sym__wc_data_create_feed] = ACTIONS(1569),
    [sym__wc_data_add_record] = ACTIONS(1569),
    [sym__wc_data_get_event_record] = ACTIONS(1569),
    [sym_format] = ACTIONS(1569),
    [sym_astro_clock] = ACTIONS(1569),
    [sym_set_outdoor_temperature] = ACTIONS(1569),
    [sym_set_virtual_level_count] = ACTIONS(1569),
    [sym_set_virtual_channel_count] = ACTIONS(1569),
    [sym_set_virtual_port_count] = ACTIONS(1569),
    [sym__wc_encode_int] = ACTIONS(1569),
    [sym__wc_decode_int] = ACTIONS(1569),
    [sym__wc_int] = ACTIONS(1569),
    [sym__wc_to_ch_int] = ACTIONS(1569),
    [sym__ch_to_wc_int] = ACTIONS(1569),
    [sym__wc_find_string_int] = ACTIONS(1569),
    [sym__wc_left_string_int] = ACTIONS(1569),
    [sym__wc_lower_string_int] = ACTIONS(1569),
    [sym__wc_mid_string_int] = ACTIONS(1569),
    [sym__wc_remove_string_int] = ACTIONS(1569),
    [sym__wc_right_string_int] = ACTIONS(1569),
    [sym__wc_upper_string_int] = ACTIONS(1569),
    [sym__wc_compare_string_int] = ACTIONS(1569),
    [sym__wc_get_buffer_char_int] = ACTIONS(1569),
    [sym__wc_get_buffer_string_int] = ACTIONS(1569),
    [sym__wc_concat_string_int] = ACTIONS(1569),
    [sym__wc_file_open_int] = ACTIONS(1569),
    [sym__wc_file_close_int] = ACTIONS(1569),
    [sym__wc_file_read_int] = ACTIONS(1569),
    [sym__wc_file_read_line_int] = ACTIONS(1569),
    [sym__wc_file_write_int] = ACTIONS(1569),
    [sym__wc_file_write_line_int] = ACTIONS(1569),
    [aux_sym_preproc_include_token1] = ACTIONS(1569),
    [aux_sym_preproc_define_token1] = ACTIONS(1571),
    [aux_sym_preproc_warn_token1] = ACTIONS(1571),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1571),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1571),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1571),
    [anon_sym_LBRACK] = ACTIONS(1571),
    [anon_sym_LBRACE] = ACTIONS(1571),
    [anon_sym_COLON] = ACTIONS(1571),
    [anon_sym_LPAREN] = ACTIONS(1569),
    [anon_sym_SEMI] = ACTIONS(1571),
    [anon_sym_BANG] = ACTIONS(1569),
    [anon_sym_TILDE] = ACTIONS(1571),
    [anon_sym_DASH] = ACTIONS(1569),
    [anon_sym_PLUS] = ACTIONS(1569),
    [anon_sym_STAR] = ACTIONS(1571),
    [anon_sym_SLASH] = ACTIONS(1569),
    [anon_sym_PERCENT] = ACTIONS(1571),
    [anon_sym_PIPE_PIPE] = ACTIONS(1571),
    [anon_sym_AMP_AMP] = ACTIONS(1571),
    [anon_sym_PIPE] = ACTIONS(1569),
    [anon_sym_CARET] = ACTIONS(1571),
    [anon_sym_AMP] = ACTIONS(1569),
    [anon_sym_EQ_EQ] = ACTIONS(1571),
    [anon_sym_BANG_EQ] = ACTIONS(1571),
    [anon_sym_LT_GT] = ACTIONS(1571),
    [anon_sym_GT] = ACTIONS(1569),
    [anon_sym_GT_EQ] = ACTIONS(1571),
    [anon_sym_LT_EQ] = ACTIONS(1571),
    [anon_sym_LT] = ACTIONS(1569),
    [anon_sym_LT_LT] = ACTIONS(1571),
    [anon_sym_GT_GT] = ACTIONS(1571),
    [anon_sym_DASH_DASH] = ACTIONS(1571),
    [anon_sym_PLUS_PLUS] = ACTIONS(1571),
    [anon_sym_DOT] = ACTIONS(1569),
    [anon_sym_DQUOTE] = ACTIONS(1571),
    [anon_sym_SQUOTE] = ACTIONS(1571),
    [sym_number_literal] = ACTIONS(1571),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1571),
  },
  [STATE(145)] = {
    [ts_builtin_sym_end] = ACTIONS(1599),
    [sym_identifier] = ACTIONS(1597),
    [sym_program_name_keyword] = ACTIONS(1597),
    [sym_module_name_keyword] = ACTIONS(1597),
    [sym_define_device_keyword] = ACTIONS(1597),
    [sym_define_combine_keyword] = ACTIONS(1597),
    [sym_define_constant_keyword] = ACTIONS(1597),
    [sym_define_type_keyword] = ACTIONS(1597),
    [sym_define_variable_keyword] = ACTIONS(1597),
    [sym_define_system_variable_keyword] = ACTIONS(1597),
    [sym_define_connect_level_keyword] = ACTIONS(1597),
    [sym_define_latching_keyword] = ACTIONS(1597),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1597),
    [sym_define_toggling_keyword] = ACTIONS(1597),
    [sym_define_call_keyword] = ACTIONS(1597),
    [sym_define_function_keyword] = ACTIONS(1597),
    [sym_define_library_function_keyword] = ACTIONS(1597),
    [sym_define_start_keyword] = ACTIONS(1597),
    [sym_define_event_keyword] = ACTIONS(1597),
    [sym_define_program_keyword] = ACTIONS(1597),
    [sym_define_module_keyword] = ACTIONS(1597),
    [sym_char_keyword] = ACTIONS(1597),
    [sym_widechar_keyword] = ACTIONS(1597),
    [sym_integer_keyword] = ACTIONS(1597),
    [sym_sinteger_keyword] = ACTIONS(1597),
    [sym_long_keyword] = ACTIONS(1597),
    [sym_slong_keyword] = ACTIONS(1597),
    [sym_float_keyword] = ACTIONS(1597),
    [sym_double_keyword] = ACTIONS(1597),
    [sym_variant_keyword] = ACTIONS(1597),
    [sym_variantarray_keyword] = ACTIONS(1597),
    [sym_local_var_keyword] = ACTIONS(1597),
    [sym_stack_var_keyword] = ACTIONS(1597),
    [sym_constant_keyword] = ACTIONS(1597),
    [sym_volatile_keyword] = ACTIONS(1597),
    [sym_non_volatile_keyword] = ACTIONS(1597),
    [sym_persistent_keyword] = ACTIONS(1597),
    [sym_dev_keyword] = ACTIONS(1597),
    [sym_devlev_keyword] = ACTIONS(1597),
    [sym_devchan_keyword] = ACTIONS(1597),
    [sym_if_keyword] = ACTIONS(1597),
    [sym_else_keyword] = ACTIONS(1597),
    [sym_switch_keyword] = ACTIONS(1597),
    [sym_case_keyword] = ACTIONS(1597),
    [sym_default_keyword] = ACTIONS(1597),
    [sym_while_keyword] = ACTIONS(1597),
    [sym_for_keyword] = ACTIONS(1597),
    [sym_break_keyword] = ACTIONS(1597),
    [sym_continue_keyword] = ACTIONS(1597),
    [sym_return_keyword] = ACTIONS(1597),
    [sym_select_keyword] = ACTIONS(1597),
    [sym_active_keyword] = ACTIONS(1597),
    [sym_struct_keyword] = ACTIONS(1597),
    [sym_structure_keyword] = ACTIONS(1597),
    [sym_button_event_keyword] = ACTIONS(1597),
    [sym_channel_event_keyword] = ACTIONS(1597),
    [sym_level_event_keyword] = ACTIONS(1597),
    [sym_data_event_keyword] = ACTIONS(1597),
    [sym_timeline_event_keyword] = ACTIONS(1597),
    [sym_custom_event_keyword] = ACTIONS(1597),
    [sym_push_keyword] = ACTIONS(1597),
    [sym_release_keyword] = ACTIONS(1597),
    [aux_sym_on_keyword_token1] = ACTIONS(1597),
    [aux_sym_off_keyword_token1] = ACTIONS(1597),
    [sym_send_level_keyword] = ACTIONS(1597),
    [sym_send_string_keyword] = ACTIONS(1597),
    [sym_send_command_keyword] = ACTIONS(1597),
    [sym_clear_buffer_keyword] = ACTIONS(1597),
    [sym_create_buffer_keyword] = ACTIONS(1597),
    [sym_create_multi_buffer_keyword] = ACTIONS(1597),
    [sym_call_keyword] = ACTIONS(1597),
    [sym_system_call_keyword] = ACTIONS(1597),
    [sym_devchan_to_keyword] = ACTIONS(1597),
    [sym_devchan_min_to_keyword] = ACTIONS(1597),
    [sym_devchan_total_off_keyword] = ACTIONS(1597),
    [sym_devchan_pulse_keyword] = ACTIONS(1597),
    [sym_wait_keyword] = ACTIONS(1597),
    [sym_cancel_wait_keyword] = ACTIONS(1597),
    [sym_wait_until_keyword] = ACTIONS(1597),
    [sym_cancel_wait_until_keyword] = ACTIONS(1597),
    [sym_cancel_all_wait_keyword] = ACTIONS(1597),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1597),
    [sym_and] = ACTIONS(1597),
    [sym_or] = ACTIONS(1597),
    [sym_not] = ACTIONS(1597),
    [sym_xor] = ACTIONS(1597),
    [sym_band] = ACTIONS(1597),
    [sym_bor] = ACTIONS(1597),
    [sym_bxor] = ACTIONS(1597),
    [sym_bnot] = ACTIONS(1597),
    [sym_lshift] = ACTIONS(1597),
    [sym_rshift] = ACTIONS(1597),
    [sym___file__] = ACTIONS(1597),
    [sym___date__] = ACTIONS(1597),
    [sym___ldate__] = ACTIONS(1597),
    [sym___time__] = ACTIONS(1597),
    [sym___line__] = ACTIONS(1597),
    [sym___name__] = ACTIONS(1597),
    [sym___netlinx__] = ACTIONS(1597),
    [sym_netlinx_axi_version] = ACTIONS(1597),
    [sym_url_flg_tcp] = ACTIONS(1597),
    [sym_url_flg_accinfopresent] = ACTIONS(1597),
    [sym_url_flg_temp] = ACTIONS(1597),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1597),
    [sym_url_flg_stat_mask] = ACTIONS(1597),
    [sym_url_flg_stat_lookup] = ACTIONS(1597),
    [sym_url_flg_stat_connecting] = ACTIONS(1597),
    [sym_url_flg_stat_waiting] = ACTIONS(1597),
    [sym_url_flg_stat_connected] = ACTIONS(1597),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1597),
    [sym_first_virtual_device] = ACTIONS(1597),
    [sym_first_local_port] = ACTIONS(1597),
    [sym_dynamic_virtual_device] = ACTIONS(1597),
    [sym_timeline_once] = ACTIONS(1597),
    [sym_timeline_repeat] = ACTIONS(1597),
    [sym_timeline_absolute] = ACTIONS(1597),
    [sym_timeline_relative] = ACTIONS(1597),
    [sym_do_push_timed_infinite] = ACTIONS(1597),
    [sym_source_type_no_address] = ACTIONS(1597),
    [sym_source_type_neuron_id] = ACTIONS(1597),
    [sym_source_type_ip_address] = ACTIONS(1597),
    [sym_source_type_axlink] = ACTIONS(1597),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1597),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1597),
    [sym_source_type_ip_socket_address] = ACTIONS(1597),
    [sym_source_type_rs232] = ACTIONS(1597),
    [sym_source_type_internal] = ACTIONS(1597),
    [sym_source_type_mac_address] = ACTIONS(1597),
    [sym_source_type_ipv4_port] = ACTIONS(1597),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1597),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1597),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1597),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1597),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1597),
    [sym_true] = ACTIONS(1597),
    [sym_false] = ACTIONS(1597),
    [sym_file_read_only] = ACTIONS(1597),
    [sym_file_rw_new] = ACTIONS(1597),
    [sym_file_rw_append] = ACTIONS(1597),
    [sym_ip_tcp] = ACTIONS(1597),
    [sym_ip_udp] = ACTIONS(1597),
    [sym_ip_udp_2way] = ACTIONS(1597),
    [sym_xml_encode_types] = ACTIONS(1597),
    [sym_xml_encode_char_as_list] = ACTIONS(1597),
    [sym_xml_encode_le] = ACTIONS(1597),
    [sym_xml_decode_types] = ACTIONS(1597),
    [sym_xml_decode_no_preserve] = ACTIONS(1597),
    [sym_duet_dev_type_display_device] = ACTIONS(1597),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1597),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1597),
    [sym_duet_dev_type_io_device] = ACTIONS(1597),
    [sym_duet_dev_type_relay_device] = ACTIONS(1597),
    [sym_duet_dev_type_ups] = ACTIONS(1597),
    [sym_duet_dev_type_amplifier] = ACTIONS(1597),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1597),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1597),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1597),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1597),
    [sym_duet_dev_type_camera] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1597),
    [sym_duet_dev_type_dss] = ACTIONS(1597),
    [sym_duet_dev_type_dvr] = ACTIONS(1597),
    [sym_duet_dev_type_disc_device] = ACTIONS(1597),
    [sym_duet_dev_type_document_camera] = ACTIONS(1597),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1597),
    [sym_duet_dev_type_hvac] = ACTIONS(1597),
    [sym_duet_dev_type_keypad] = ACTIONS(1597),
    [sym_duet_dev_type_light] = ACTIONS(1597),
    [sym_duet_dev_type_monitor] = ACTIONS(1597),
    [sym_duet_dev_type_motor] = ACTIONS(1597),
    [sym_duet_dev_type_multi_window] = ACTIONS(1597),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1597),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1597),
    [sym_duet_dev_type_receiver] = ACTIONS(1597),
    [sym_duet_dev_type_security_system] = ACTIONS(1597),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1597),
    [sym_duet_dev_type_settop_box] = ACTIONS(1597),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1597),
    [sym_duet_dev_type_switcher] = ACTIONS(1597),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1597),
    [sym_duet_dev_type_tv] = ACTIONS(1597),
    [sym_duet_dev_type_utility] = ACTIONS(1597),
    [sym_duet_dev_type_vcr] = ACTIONS(1597),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1597),
    [sym_duet_dev_type_video_processor] = ACTIONS(1597),
    [sym_duet_dev_type_video_projector] = ACTIONS(1597),
    [sym_duet_dev_type_video_wall] = ACTIONS(1597),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1597),
    [sym_duet_dev_type_weather] = ACTIONS(1597),
    [sym_duet_dev_not_polled] = ACTIONS(1597),
    [sym_duet_dev_polled] = ACTIONS(1597),
    [sym_clkmgr_mode_network] = ACTIONS(1597),
    [sym_clkmgr_mode_standalone] = ACTIONS(1597),
    [sym_normal_standby] = ACTIONS(1597),
    [sym_normal_wake] = ACTIONS(1597),
    [sym_ip_multicast_ttl_option] = ACTIONS(1597),
    [sym_ip_ttl_subnet] = ACTIONS(1597),
    [sym_ip_ttl_site] = ACTIONS(1597),
    [sym_ip_ttl_region] = ACTIONS(1597),
    [sym_ip_ttl_continent] = ACTIONS(1597),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1597),
    [sym_ip_nodelay_on] = ACTIONS(1597),
    [sym_ip_nodelay_off] = ACTIONS(1597),
    [sym_amx_error] = ACTIONS(1597),
    [sym_amx_warning] = ACTIONS(1597),
    [sym_amx_info] = ACTIONS(1597),
    [sym_amx_debug] = ACTIONS(1597),
    [sym_netlinx_login_success] = ACTIONS(1597),
    [sym_netlinx_login_fail] = ACTIONS(1597),
    [sym_netlinx_logout] = ACTIONS(1597),
    [sym_null_str] = ACTIONS(1597),
    [sym_smtp_address] = ACTIONS(1597),
    [sym_smtp_port_number] = ACTIONS(1597),
    [sym_smtp_username] = ACTIONS(1597),
    [sym_smtp_password] = ACTIONS(1597),
    [sym_smtp_require_tls] = ACTIONS(1597),
    [sym_smtp_from] = ACTIONS(1597),
    [sym_smtp_tls_true] = ACTIONS(1597),
    [sym_smtp_tls_false] = ACTIONS(1597),
    [sym_listview_on_row_select_event] = ACTIONS(1597),
    [sym_data_structure_unknown] = ACTIONS(1597),
    [sym_data_structure_datafeed] = ACTIONS(1597),
    [sym_data_structure_datarecordset] = ACTIONS(1597),
    [sym_data_structure_datarecord] = ACTIONS(1597),
    [sym_data_structure_datafield] = ACTIONS(1597),
    [sym_data_max_content_field_count] = ACTIONS(1597),
    [sym_data_max_metadata_field_count] = ACTIONS(1597),
    [sym_data_max_id_length] = ACTIONS(1597),
    [sym_data_max_name_length] = ACTIONS(1597),
    [sym_data_max_description_length] = ACTIONS(1597),
    [sym_data_max_type_length] = ACTIONS(1597),
    [sym_data_max_format_length] = ACTIONS(1597),
    [sym_data_max_source_length] = ACTIONS(1597),
    [sym_data_max_value_length] = ACTIONS(1597),
    [sym_data_max_label_length] = ACTIONS(1597),
    [sym_data_max_url_length] = ACTIONS(1597),
    [sym_max_last_login_info_length] = ACTIONS(1597),
    [sym_data_type_unknown] = ACTIONS(1597),
    [sym_data_type_string] = ACTIONS(1597),
    [sym_data_type_datetime] = ACTIONS(1597),
    [sym_data_type_date] = ACTIONS(1597),
    [sym_data_type_time] = ACTIONS(1597),
    [sym_data_type_image] = ACTIONS(1597),
    [sym_data_format_url] = ACTIONS(1597),
    [sym_data_format_phone] = ACTIONS(1597),
    [sym_data_format_email] = ACTIONS(1597),
    [sym_data_format_iso8601] = ACTIONS(1597),
    [sym_auditlog_priv] = ACTIONS(1597),
    [sym_configuration_priv] = ACTIONS(1597),
    [sym_deviceconfig_priv] = ACTIONS(1597),
    [sym_ftp_priv] = ACTIONS(1597),
    [sym_http_priv] = ACTIONS(1597),
    [sym_networkconfig_priv] = ACTIONS(1597),
    [sym_programport_priv] = ACTIONS(1597),
    [sym_remoteui_priv] = ACTIONS(1597),
    [sym_securitycontrol_priv] = ACTIONS(1597),
    [sym_softwaremanagement_priv] = ACTIONS(1597),
    [sym_terminal_priv] = ACTIONS(1597),
    [sym_tpadmin_priv] = ACTIONS(1597),
    [sym_user1_priv] = ACTIONS(1597),
    [sym_user2_priv] = ACTIONS(1597),
    [sym_user3_priv] = ACTIONS(1597),
    [sym_user4_priv] = ACTIONS(1597),
    [sym_usermanagement_priv] = ACTIONS(1597),
    [sym_valid_account] = ACTIONS(1597),
    [sym_err_name_invalid] = ACTIONS(1597),
    [sym_err_password_invalid] = ACTIONS(1597),
    [sym_err_invalid_account] = ACTIONS(1597),
    [sym_err_no_authorization] = ACTIONS(1597),
    [sym_err_invalid_parameter] = ACTIONS(1597),
    [sym_err_user_locked_out] = ACTIONS(1597),
    [sym_err_user_account_expired] = ACTIONS(1597),
    [sym_err_general_error] = ACTIONS(1597),
    [sym_err_authentication_error] = ACTIONS(1597),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1597),
    [sym_tls_validate_certificate] = ACTIONS(1597),
    [sym_tdata] = ACTIONS(1597),
    [sym_tchannel] = ACTIONS(1597),
    [sym_tlevel] = ACTIONS(1597),
    [sym_tbutton] = ACTIONS(1597),
    [sym_ttimeline] = ACTIONS(1597),
    [sym_tcustom] = ACTIONS(1597),
    [sym_url_struct] = ACTIONS(1597),
    [sym_dns_struct] = ACTIONS(1597),
    [sym_ip_address_struct] = ACTIONS(1597),
    [sym_dev_info_struct] = ACTIONS(1597),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1597),
    [sym_data_feed] = ACTIONS(1597),
    [sym_data_field] = ACTIONS(1597),
    [sym_data_record] = ACTIONS(1597),
    [sym_wc_data_feed] = ACTIONS(1597),
    [sym_wc_data_field] = ACTIONS(1597),
    [sym_wc_data_record] = ACTIONS(1597),
    [sym_last_login_info] = ACTIONS(1597),
    [sym____reserved___] = ACTIONS(1597),
    [sym_date] = ACTIONS(1597),
    [sym_day] = ACTIONS(1597),
    [sym_time] = ACTIONS(1597),
    [sym_ldate] = ACTIONS(1597),
    [sym_push_device] = ACTIONS(1597),
    [sym_push_channel] = ACTIONS(1597),
    [sym_push_devchan] = ACTIONS(1597),
    [sym_release_device] = ACTIONS(1597),
    [sym_release_channel] = ACTIONS(1597),
    [sym_release_devchan] = ACTIONS(1597),
    [sym_master_slot] = ACTIONS(1597),
    [sym_get_pulse_time] = ACTIONS(1597),
    [sym_get_timer] = ACTIONS(1597),
    [sym_master_sn] = ACTIONS(1597),
    [sym_system_number] = ACTIONS(1597),
    [sym_button] = ACTIONS(1597),
    [sym_dv_channel] = ACTIONS(1597),
    [sym_data] = ACTIONS(1597),
    [sym_level] = ACTIONS(1597),
    [sym_timeline] = ACTIONS(1597),
    [sym_channel] = ACTIONS(1597),
    [sym_custom] = ACTIONS(1597),
    [sym_atoi] = ACTIONS(1597),
    [sym_atol] = ACTIONS(1597),
    [sym_atof] = ACTIONS(1597),
    [sym_itoa] = ACTIONS(1597),
    [sym_itohex] = ACTIONS(1597),
    [sym_hextoi] = ACTIONS(1597),
    [sym_ftoa] = ACTIONS(1597),
    [sym_find_string] = ACTIONS(1597),
    [sym_left_string] = ACTIONS(1597),
    [sym_right_string] = ACTIONS(1597),
    [sym_mid_string] = ACTIONS(1597),
    [sym_remove_string] = ACTIONS(1597),
    [sym_get_buffer_string] = ACTIONS(1597),
    [sym_get_buffer_char] = ACTIONS(1597),
    [sym_get_multi_buffer_string] = ACTIONS(1597),
    [sym_length_string] = ACTIONS(1597),
    [sym_max_length_string] = ACTIONS(1597),
    [sym_set_length_string] = ACTIONS(1597),
    [sym_compare_string] = ACTIONS(1597),
    [sym_upper_string] = ACTIONS(1597),
    [sym_lower_string] = ACTIONS(1597),
    [sym_get_last] = ACTIONS(1597),
    [sym_date_to_day] = ACTIONS(1597),
    [sym_date_to_month] = ACTIONS(1597),
    [sym_date_to_year] = ACTIONS(1597),
    [sym_time_to_hour] = ACTIONS(1597),
    [sym_time_to_minute] = ACTIONS(1597),
    [sym_time_to_second] = ACTIONS(1597),
    [sym_day_of_week] = ACTIONS(1597),
    [sym_set_pulse_time] = ACTIONS(1597),
    [sym_set_timer] = ACTIONS(1597),
    [sym_file_open] = ACTIONS(1597),
    [sym_file_close] = ACTIONS(1597),
    [sym_file_read] = ACTIONS(1597),
    [sym_file_read_line] = ACTIONS(1597),
    [sym_file_write] = ACTIONS(1597),
    [sym_file_write_line] = ACTIONS(1597),
    [sym_file_copy] = ACTIONS(1597),
    [sym_file_delete] = ACTIONS(1597),
    [sym_file_rename] = ACTIONS(1597),
    [sym_file_dir] = ACTIONS(1597),
    [sym_file_getdir] = ACTIONS(1597),
    [sym_file_setdir] = ACTIONS(1597),
    [sym_file_create_dir] = ACTIONS(1597),
    [sym_file_remove_dir] = ACTIONS(1597),
    [sym_file_seek] = ACTIONS(1597),
    [sym_length_array] = ACTIONS(1597),
    [sym_max_length_array] = ACTIONS(1597),
    [sym_set_length_array] = ACTIONS(1597),
    [sym_ip_client_open] = ACTIONS(1597),
    [sym_ip_client_close] = ACTIONS(1597),
    [sym_ip_server_open] = ACTIONS(1597),
    [sym_ip_server_close] = ACTIONS(1597),
    [sym_ip_bound_client_open] = ACTIONS(1597),
    [sym_ip_mc_server_open] = ACTIONS(1597),
    [sym_ip_set_option] = ACTIONS(1597),
    [sym_ssh_client_open] = ACTIONS(1597),
    [sym_ssh_client_close] = ACTIONS(1597),
    [sym_tls_client_open] = ACTIONS(1597),
    [sym_tls_client_close] = ACTIONS(1597),
    [sym_do_push] = ACTIONS(1597),
    [sym_do_push_timed] = ACTIONS(1597),
    [sym_do_release] = ACTIONS(1597),
    [sym_device_id] = ACTIONS(1597),
    [sym_device_info] = ACTIONS(1597),
    [sym_device_id_string] = ACTIONS(1597),
    [sym_device_standby] = ACTIONS(1597),
    [sym_device_wake] = ACTIONS(1597),
    [sym_timeline_create] = ACTIONS(1597),
    [sym_timeline_kill] = ACTIONS(1597),
    [sym_timeline_reload] = ACTIONS(1597),
    [sym_timeline_pause] = ACTIONS(1597),
    [sym_timeline_restart] = ACTIONS(1597),
    [sym_timeline_set] = ACTIONS(1597),
    [sym_timeline_get] = ACTIONS(1597),
    [sym_timeline_active] = ACTIONS(1597),
    [sym_timeline_dynamic_id] = ACTIONS(1597),
    [sym_variable_to_string] = ACTIONS(1597),
    [sym_string_to_variable] = ACTIONS(1597),
    [sym_length_variable_to_string] = ACTIONS(1597),
    [sym_type_cast] = ACTIONS(1597),
    [sym_variable_to_xml] = ACTIONS(1597),
    [sym_xml_to_variable] = ACTIONS(1597),
    [sym_length_variable_to_xml] = ACTIONS(1597),
    [sym_redirect_string] = ACTIONS(1597),
    [sym_random_number] = ACTIONS(1597),
    [sym_get_unique_id] = ACTIONS(1597),
    [sym_get_system_number] = ACTIONS(1597),
    [sym_set_system_number] = ACTIONS(1597),
    [sym_get_serial_number] = ACTIONS(1597),
    [sym_get_master_build] = ACTIONS(1597),
    [sym_set_validation_code] = ACTIONS(1597),
    [sym_reboot] = ACTIONS(1597),
    [sym_load_duet_module] = ACTIONS(1597),
    [sym_unload_duet_module] = ACTIONS(1597),
    [sym_duet_mem_size_set] = ACTIONS(1597),
    [sym_duet_mem_size_get] = ACTIONS(1597),
    [sym_dynamic_application_device] = ACTIONS(1597),
    [sym_dynamic_polled_port] = ACTIONS(1597),
    [sym_static_port_binding] = ACTIONS(1597),
    [sym_static_ip_binding] = ACTIONS(1597),
    [sym_get_dns_list] = ACTIONS(1597),
    [sym_set_dns_list] = ACTIONS(1597),
    [sym_get_ip_address] = ACTIONS(1597),
    [sym_set_ip_address] = ACTIONS(1597),
    [sym_get_url_list] = ACTIONS(1597),
    [sym_add_url_entry] = ACTIONS(1597),
    [sym_delete_url_entry] = ACTIONS(1597),
    [sym_abs_value] = ACTIONS(1597),
    [sym_max_value] = ACTIONS(1597),
    [sym_min_value] = ACTIONS(1597),
    [sym_exp_value] = ACTIONS(1597),
    [sym_log_value] = ACTIONS(1597),
    [sym_log10_value] = ACTIONS(1597),
    [sym_power_value] = ACTIONS(1597),
    [sym_sqrt_value] = ACTIONS(1597),
    [sym_raw_be] = ACTIONS(1597),
    [sym_raw_le] = ACTIONS(1597),
    [sym_internal_queue_size_set] = ACTIONS(1597),
    [sym_internal_queue_size_get] = ACTIONS(1597),
    [sym_internal_threshold_set] = ACTIONS(1597),
    [sym_internal_threshold_get] = ACTIONS(1597),
    [sym_rebuilt_event] = ACTIONS(1597),
    [sym_do_custom_event] = ACTIONS(1597),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1597),
    [sym_clkmgr_set_clk_source] = ACTIONS(1597),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1597),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1597),
    [sym_clkmgr_get_timezone] = ACTIONS(1597),
    [sym_clkmgr_set_timezone] = ACTIONS(1597),
    [sym_clkmgr_get_resync_period] = ACTIONS(1597),
    [sym_clkmgr_set_resync_period] = ACTIONS(1597),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1597),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1597),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1597),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1597),
    [sym_clkmgr_get_timeservers] = ACTIONS(1597),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1597),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1597),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1597),
    [sym_set_log_level] = ACTIONS(1597),
    [sym_get_log_level] = ACTIONS(1597),
    [sym_amx_log] = ACTIONS(1597),
    [sym_smtp_server_config_set] = ACTIONS(1597),
    [sym_smtp_server_config_get] = ACTIONS(1597),
    [sym_smtp_send] = ACTIONS(1597),
    [sym_validate_netlinx_account] = ACTIONS(1597),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1597),
    [sym_authenticate_certificate] = ACTIONS(1597),
    [sym_audit_netlinx_session_event] = ACTIONS(1597),
    [sym_audit_netlinx_generic_event] = ACTIONS(1597),
    [sym_get_available_flash_disk_space] = ACTIONS(1597),
    [sym_get_max_flash_disk_space] = ACTIONS(1597),
    [sym_data_create_feed] = ACTIONS(1597),
    [sym_data_delete_feed] = ACTIONS(1597),
    [sym_data_publish_feed] = ACTIONS(1597),
    [sym_data_get_published_feed] = ACTIONS(1597),
    [sym_data_add_record] = ACTIONS(1597),
    [sym_data_get_event_record] = ACTIONS(1597),
    [sym__wc_data_create_feed] = ACTIONS(1597),
    [sym__wc_data_add_record] = ACTIONS(1597),
    [sym__wc_data_get_event_record] = ACTIONS(1597),
    [sym_format] = ACTIONS(1597),
    [sym_astro_clock] = ACTIONS(1597),
    [sym_set_outdoor_temperature] = ACTIONS(1597),
    [sym_set_virtual_level_count] = ACTIONS(1597),
    [sym_set_virtual_channel_count] = ACTIONS(1597),
    [sym_set_virtual_port_count] = ACTIONS(1597),
    [sym__wc_encode_int] = ACTIONS(1597),
    [sym__wc_decode_int] = ACTIONS(1597),
    [sym__wc_int] = ACTIONS(1597),
    [sym__wc_to_ch_int] = ACTIONS(1597),
    [sym__ch_to_wc_int] = ACTIONS(1597),
    [sym__wc_find_string_int] = ACTIONS(1597),
    [sym__wc_left_string_int] = ACTIONS(1597),
    [sym__wc_lower_string_int] = ACTIONS(1597),
    [sym__wc_mid_string_int] = ACTIONS(1597),
    [sym__wc_remove_string_int] = ACTIONS(1597),
    [sym__wc_right_string_int] = ACTIONS(1597),
    [sym__wc_upper_string_int] = ACTIONS(1597),
    [sym__wc_compare_string_int] = ACTIONS(1597),
    [sym__wc_get_buffer_char_int] = ACTIONS(1597),
    [sym__wc_get_buffer_string_int] = ACTIONS(1597),
    [sym__wc_concat_string_int] = ACTIONS(1597),
    [sym__wc_file_open_int] = ACTIONS(1597),
    [sym__wc_file_close_int] = ACTIONS(1597),
    [sym__wc_file_read_int] = ACTIONS(1597),
    [sym__wc_file_read_line_int] = ACTIONS(1597),
    [sym__wc_file_write_int] = ACTIONS(1597),
    [sym__wc_file_write_line_int] = ACTIONS(1597),
    [aux_sym_preproc_include_token1] = ACTIONS(1597),
    [aux_sym_preproc_define_token1] = ACTIONS(1599),
    [aux_sym_preproc_warn_token1] = ACTIONS(1599),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1599),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1599),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1599),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1599),
    [anon_sym_COLON] = ACTIONS(1599),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_SEMI] = ACTIONS(1599),
    [anon_sym_BANG] = ACTIONS(1597),
    [anon_sym_TILDE] = ACTIONS(1599),
    [anon_sym_DASH] = ACTIONS(1597),
    [anon_sym_PLUS] = ACTIONS(1597),
    [anon_sym_STAR] = ACTIONS(1599),
    [anon_sym_SLASH] = ACTIONS(1597),
    [anon_sym_PERCENT] = ACTIONS(1599),
    [anon_sym_PIPE_PIPE] = ACTIONS(1599),
    [anon_sym_AMP_AMP] = ACTIONS(1599),
    [anon_sym_PIPE] = ACTIONS(1597),
    [anon_sym_CARET] = ACTIONS(1599),
    [anon_sym_AMP] = ACTIONS(1597),
    [anon_sym_EQ_EQ] = ACTIONS(1599),
    [anon_sym_BANG_EQ] = ACTIONS(1599),
    [anon_sym_LT_GT] = ACTIONS(1599),
    [anon_sym_GT] = ACTIONS(1597),
    [anon_sym_GT_EQ] = ACTIONS(1599),
    [anon_sym_LT_EQ] = ACTIONS(1599),
    [anon_sym_LT] = ACTIONS(1597),
    [anon_sym_LT_LT] = ACTIONS(1599),
    [anon_sym_GT_GT] = ACTIONS(1599),
    [anon_sym_DASH_DASH] = ACTIONS(1599),
    [anon_sym_PLUS_PLUS] = ACTIONS(1599),
    [anon_sym_DOT] = ACTIONS(1597),
    [anon_sym_DQUOTE] = ACTIONS(1599),
    [anon_sym_SQUOTE] = ACTIONS(1599),
    [sym_number_literal] = ACTIONS(1599),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1599),
  },
  [STATE(146)] = {
    [ts_builtin_sym_end] = ACTIONS(1603),
    [sym_identifier] = ACTIONS(1601),
    [sym_program_name_keyword] = ACTIONS(1601),
    [sym_module_name_keyword] = ACTIONS(1601),
    [sym_define_device_keyword] = ACTIONS(1601),
    [sym_define_combine_keyword] = ACTIONS(1601),
    [sym_define_constant_keyword] = ACTIONS(1601),
    [sym_define_type_keyword] = ACTIONS(1601),
    [sym_define_variable_keyword] = ACTIONS(1601),
    [sym_define_system_variable_keyword] = ACTIONS(1601),
    [sym_define_connect_level_keyword] = ACTIONS(1601),
    [sym_define_latching_keyword] = ACTIONS(1601),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1601),
    [sym_define_toggling_keyword] = ACTIONS(1601),
    [sym_define_call_keyword] = ACTIONS(1601),
    [sym_define_function_keyword] = ACTIONS(1601),
    [sym_define_library_function_keyword] = ACTIONS(1601),
    [sym_define_start_keyword] = ACTIONS(1601),
    [sym_define_event_keyword] = ACTIONS(1601),
    [sym_define_program_keyword] = ACTIONS(1601),
    [sym_define_module_keyword] = ACTIONS(1601),
    [sym_char_keyword] = ACTIONS(1601),
    [sym_widechar_keyword] = ACTIONS(1601),
    [sym_integer_keyword] = ACTIONS(1601),
    [sym_sinteger_keyword] = ACTIONS(1601),
    [sym_long_keyword] = ACTIONS(1601),
    [sym_slong_keyword] = ACTIONS(1601),
    [sym_float_keyword] = ACTIONS(1601),
    [sym_double_keyword] = ACTIONS(1601),
    [sym_variant_keyword] = ACTIONS(1601),
    [sym_variantarray_keyword] = ACTIONS(1601),
    [sym_local_var_keyword] = ACTIONS(1601),
    [sym_stack_var_keyword] = ACTIONS(1601),
    [sym_constant_keyword] = ACTIONS(1601),
    [sym_volatile_keyword] = ACTIONS(1601),
    [sym_non_volatile_keyword] = ACTIONS(1601),
    [sym_persistent_keyword] = ACTIONS(1601),
    [sym_dev_keyword] = ACTIONS(1601),
    [sym_devlev_keyword] = ACTIONS(1601),
    [sym_devchan_keyword] = ACTIONS(1601),
    [sym_if_keyword] = ACTIONS(1601),
    [sym_else_keyword] = ACTIONS(1601),
    [sym_switch_keyword] = ACTIONS(1601),
    [sym_case_keyword] = ACTIONS(1601),
    [sym_default_keyword] = ACTIONS(1601),
    [sym_while_keyword] = ACTIONS(1601),
    [sym_for_keyword] = ACTIONS(1601),
    [sym_break_keyword] = ACTIONS(1601),
    [sym_continue_keyword] = ACTIONS(1601),
    [sym_return_keyword] = ACTIONS(1601),
    [sym_select_keyword] = ACTIONS(1601),
    [sym_active_keyword] = ACTIONS(1601),
    [sym_struct_keyword] = ACTIONS(1601),
    [sym_structure_keyword] = ACTIONS(1601),
    [sym_button_event_keyword] = ACTIONS(1601),
    [sym_channel_event_keyword] = ACTIONS(1601),
    [sym_level_event_keyword] = ACTIONS(1601),
    [sym_data_event_keyword] = ACTIONS(1601),
    [sym_timeline_event_keyword] = ACTIONS(1601),
    [sym_custom_event_keyword] = ACTIONS(1601),
    [sym_push_keyword] = ACTIONS(1601),
    [sym_release_keyword] = ACTIONS(1601),
    [aux_sym_on_keyword_token1] = ACTIONS(1601),
    [aux_sym_off_keyword_token1] = ACTIONS(1601),
    [sym_send_level_keyword] = ACTIONS(1601),
    [sym_send_string_keyword] = ACTIONS(1601),
    [sym_send_command_keyword] = ACTIONS(1601),
    [sym_clear_buffer_keyword] = ACTIONS(1601),
    [sym_create_buffer_keyword] = ACTIONS(1601),
    [sym_create_multi_buffer_keyword] = ACTIONS(1601),
    [sym_call_keyword] = ACTIONS(1601),
    [sym_system_call_keyword] = ACTIONS(1601),
    [sym_devchan_to_keyword] = ACTIONS(1601),
    [sym_devchan_min_to_keyword] = ACTIONS(1601),
    [sym_devchan_total_off_keyword] = ACTIONS(1601),
    [sym_devchan_pulse_keyword] = ACTIONS(1601),
    [sym_wait_keyword] = ACTIONS(1601),
    [sym_cancel_wait_keyword] = ACTIONS(1601),
    [sym_wait_until_keyword] = ACTIONS(1601),
    [sym_cancel_wait_until_keyword] = ACTIONS(1601),
    [sym_cancel_all_wait_keyword] = ACTIONS(1601),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1601),
    [sym_and] = ACTIONS(1601),
    [sym_or] = ACTIONS(1601),
    [sym_not] = ACTIONS(1601),
    [sym_xor] = ACTIONS(1601),
    [sym_band] = ACTIONS(1601),
    [sym_bor] = ACTIONS(1601),
    [sym_bxor] = ACTIONS(1601),
    [sym_bnot] = ACTIONS(1601),
    [sym_lshift] = ACTIONS(1601),
    [sym_rshift] = ACTIONS(1601),
    [sym___file__] = ACTIONS(1601),
    [sym___date__] = ACTIONS(1601),
    [sym___ldate__] = ACTIONS(1601),
    [sym___time__] = ACTIONS(1601),
    [sym___line__] = ACTIONS(1601),
    [sym___name__] = ACTIONS(1601),
    [sym___netlinx__] = ACTIONS(1601),
    [sym_netlinx_axi_version] = ACTIONS(1601),
    [sym_url_flg_tcp] = ACTIONS(1601),
    [sym_url_flg_accinfopresent] = ACTIONS(1601),
    [sym_url_flg_temp] = ACTIONS(1601),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1601),
    [sym_url_flg_stat_mask] = ACTIONS(1601),
    [sym_url_flg_stat_lookup] = ACTIONS(1601),
    [sym_url_flg_stat_connecting] = ACTIONS(1601),
    [sym_url_flg_stat_waiting] = ACTIONS(1601),
    [sym_url_flg_stat_connected] = ACTIONS(1601),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1601),
    [sym_first_virtual_device] = ACTIONS(1601),
    [sym_first_local_port] = ACTIONS(1601),
    [sym_dynamic_virtual_device] = ACTIONS(1601),
    [sym_timeline_once] = ACTIONS(1601),
    [sym_timeline_repeat] = ACTIONS(1601),
    [sym_timeline_absolute] = ACTIONS(1601),
    [sym_timeline_relative] = ACTIONS(1601),
    [sym_do_push_timed_infinite] = ACTIONS(1601),
    [sym_source_type_no_address] = ACTIONS(1601),
    [sym_source_type_neuron_id] = ACTIONS(1601),
    [sym_source_type_ip_address] = ACTIONS(1601),
    [sym_source_type_axlink] = ACTIONS(1601),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1601),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1601),
    [sym_source_type_ip_socket_address] = ACTIONS(1601),
    [sym_source_type_rs232] = ACTIONS(1601),
    [sym_source_type_internal] = ACTIONS(1601),
    [sym_source_type_mac_address] = ACTIONS(1601),
    [sym_source_type_ipv4_port] = ACTIONS(1601),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1601),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1601),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1601),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1601),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1601),
    [sym_true] = ACTIONS(1601),
    [sym_false] = ACTIONS(1601),
    [sym_file_read_only] = ACTIONS(1601),
    [sym_file_rw_new] = ACTIONS(1601),
    [sym_file_rw_append] = ACTIONS(1601),
    [sym_ip_tcp] = ACTIONS(1601),
    [sym_ip_udp] = ACTIONS(1601),
    [sym_ip_udp_2way] = ACTIONS(1601),
    [sym_xml_encode_types] = ACTIONS(1601),
    [sym_xml_encode_char_as_list] = ACTIONS(1601),
    [sym_xml_encode_le] = ACTIONS(1601),
    [sym_xml_decode_types] = ACTIONS(1601),
    [sym_xml_decode_no_preserve] = ACTIONS(1601),
    [sym_duet_dev_type_display_device] = ACTIONS(1601),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1601),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1601),
    [sym_duet_dev_type_io_device] = ACTIONS(1601),
    [sym_duet_dev_type_relay_device] = ACTIONS(1601),
    [sym_duet_dev_type_ups] = ACTIONS(1601),
    [sym_duet_dev_type_amplifier] = ACTIONS(1601),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1601),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1601),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1601),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1601),
    [sym_duet_dev_type_camera] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1601),
    [sym_duet_dev_type_dss] = ACTIONS(1601),
    [sym_duet_dev_type_dvr] = ACTIONS(1601),
    [sym_duet_dev_type_disc_device] = ACTIONS(1601),
    [sym_duet_dev_type_document_camera] = ACTIONS(1601),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1601),
    [sym_duet_dev_type_hvac] = ACTIONS(1601),
    [sym_duet_dev_type_keypad] = ACTIONS(1601),
    [sym_duet_dev_type_light] = ACTIONS(1601),
    [sym_duet_dev_type_monitor] = ACTIONS(1601),
    [sym_duet_dev_type_motor] = ACTIONS(1601),
    [sym_duet_dev_type_multi_window] = ACTIONS(1601),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1601),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1601),
    [sym_duet_dev_type_receiver] = ACTIONS(1601),
    [sym_duet_dev_type_security_system] = ACTIONS(1601),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1601),
    [sym_duet_dev_type_settop_box] = ACTIONS(1601),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1601),
    [sym_duet_dev_type_switcher] = ACTIONS(1601),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1601),
    [sym_duet_dev_type_tv] = ACTIONS(1601),
    [sym_duet_dev_type_utility] = ACTIONS(1601),
    [sym_duet_dev_type_vcr] = ACTIONS(1601),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1601),
    [sym_duet_dev_type_video_processor] = ACTIONS(1601),
    [sym_duet_dev_type_video_projector] = ACTIONS(1601),
    [sym_duet_dev_type_video_wall] = ACTIONS(1601),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1601),
    [sym_duet_dev_type_weather] = ACTIONS(1601),
    [sym_duet_dev_not_polled] = ACTIONS(1601),
    [sym_duet_dev_polled] = ACTIONS(1601),
    [sym_clkmgr_mode_network] = ACTIONS(1601),
    [sym_clkmgr_mode_standalone] = ACTIONS(1601),
    [sym_normal_standby] = ACTIONS(1601),
    [sym_normal_wake] = ACTIONS(1601),
    [sym_ip_multicast_ttl_option] = ACTIONS(1601),
    [sym_ip_ttl_subnet] = ACTIONS(1601),
    [sym_ip_ttl_site] = ACTIONS(1601),
    [sym_ip_ttl_region] = ACTIONS(1601),
    [sym_ip_ttl_continent] = ACTIONS(1601),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1601),
    [sym_ip_nodelay_on] = ACTIONS(1601),
    [sym_ip_nodelay_off] = ACTIONS(1601),
    [sym_amx_error] = ACTIONS(1601),
    [sym_amx_warning] = ACTIONS(1601),
    [sym_amx_info] = ACTIONS(1601),
    [sym_amx_debug] = ACTIONS(1601),
    [sym_netlinx_login_success] = ACTIONS(1601),
    [sym_netlinx_login_fail] = ACTIONS(1601),
    [sym_netlinx_logout] = ACTIONS(1601),
    [sym_null_str] = ACTIONS(1601),
    [sym_smtp_address] = ACTIONS(1601),
    [sym_smtp_port_number] = ACTIONS(1601),
    [sym_smtp_username] = ACTIONS(1601),
    [sym_smtp_password] = ACTIONS(1601),
    [sym_smtp_require_tls] = ACTIONS(1601),
    [sym_smtp_from] = ACTIONS(1601),
    [sym_smtp_tls_true] = ACTIONS(1601),
    [sym_smtp_tls_false] = ACTIONS(1601),
    [sym_listview_on_row_select_event] = ACTIONS(1601),
    [sym_data_structure_unknown] = ACTIONS(1601),
    [sym_data_structure_datafeed] = ACTIONS(1601),
    [sym_data_structure_datarecordset] = ACTIONS(1601),
    [sym_data_structure_datarecord] = ACTIONS(1601),
    [sym_data_structure_datafield] = ACTIONS(1601),
    [sym_data_max_content_field_count] = ACTIONS(1601),
    [sym_data_max_metadata_field_count] = ACTIONS(1601),
    [sym_data_max_id_length] = ACTIONS(1601),
    [sym_data_max_name_length] = ACTIONS(1601),
    [sym_data_max_description_length] = ACTIONS(1601),
    [sym_data_max_type_length] = ACTIONS(1601),
    [sym_data_max_format_length] = ACTIONS(1601),
    [sym_data_max_source_length] = ACTIONS(1601),
    [sym_data_max_value_length] = ACTIONS(1601),
    [sym_data_max_label_length] = ACTIONS(1601),
    [sym_data_max_url_length] = ACTIONS(1601),
    [sym_max_last_login_info_length] = ACTIONS(1601),
    [sym_data_type_unknown] = ACTIONS(1601),
    [sym_data_type_string] = ACTIONS(1601),
    [sym_data_type_datetime] = ACTIONS(1601),
    [sym_data_type_date] = ACTIONS(1601),
    [sym_data_type_time] = ACTIONS(1601),
    [sym_data_type_image] = ACTIONS(1601),
    [sym_data_format_url] = ACTIONS(1601),
    [sym_data_format_phone] = ACTIONS(1601),
    [sym_data_format_email] = ACTIONS(1601),
    [sym_data_format_iso8601] = ACTIONS(1601),
    [sym_auditlog_priv] = ACTIONS(1601),
    [sym_configuration_priv] = ACTIONS(1601),
    [sym_deviceconfig_priv] = ACTIONS(1601),
    [sym_ftp_priv] = ACTIONS(1601),
    [sym_http_priv] = ACTIONS(1601),
    [sym_networkconfig_priv] = ACTIONS(1601),
    [sym_programport_priv] = ACTIONS(1601),
    [sym_remoteui_priv] = ACTIONS(1601),
    [sym_securitycontrol_priv] = ACTIONS(1601),
    [sym_softwaremanagement_priv] = ACTIONS(1601),
    [sym_terminal_priv] = ACTIONS(1601),
    [sym_tpadmin_priv] = ACTIONS(1601),
    [sym_user1_priv] = ACTIONS(1601),
    [sym_user2_priv] = ACTIONS(1601),
    [sym_user3_priv] = ACTIONS(1601),
    [sym_user4_priv] = ACTIONS(1601),
    [sym_usermanagement_priv] = ACTIONS(1601),
    [sym_valid_account] = ACTIONS(1601),
    [sym_err_name_invalid] = ACTIONS(1601),
    [sym_err_password_invalid] = ACTIONS(1601),
    [sym_err_invalid_account] = ACTIONS(1601),
    [sym_err_no_authorization] = ACTIONS(1601),
    [sym_err_invalid_parameter] = ACTIONS(1601),
    [sym_err_user_locked_out] = ACTIONS(1601),
    [sym_err_user_account_expired] = ACTIONS(1601),
    [sym_err_general_error] = ACTIONS(1601),
    [sym_err_authentication_error] = ACTIONS(1601),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1601),
    [sym_tls_validate_certificate] = ACTIONS(1601),
    [sym_tdata] = ACTIONS(1601),
    [sym_tchannel] = ACTIONS(1601),
    [sym_tlevel] = ACTIONS(1601),
    [sym_tbutton] = ACTIONS(1601),
    [sym_ttimeline] = ACTIONS(1601),
    [sym_tcustom] = ACTIONS(1601),
    [sym_url_struct] = ACTIONS(1601),
    [sym_dns_struct] = ACTIONS(1601),
    [sym_ip_address_struct] = ACTIONS(1601),
    [sym_dev_info_struct] = ACTIONS(1601),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1601),
    [sym_data_feed] = ACTIONS(1601),
    [sym_data_field] = ACTIONS(1601),
    [sym_data_record] = ACTIONS(1601),
    [sym_wc_data_feed] = ACTIONS(1601),
    [sym_wc_data_field] = ACTIONS(1601),
    [sym_wc_data_record] = ACTIONS(1601),
    [sym_last_login_info] = ACTIONS(1601),
    [sym____reserved___] = ACTIONS(1601),
    [sym_date] = ACTIONS(1601),
    [sym_day] = ACTIONS(1601),
    [sym_time] = ACTIONS(1601),
    [sym_ldate] = ACTIONS(1601),
    [sym_push_device] = ACTIONS(1601),
    [sym_push_channel] = ACTIONS(1601),
    [sym_push_devchan] = ACTIONS(1601),
    [sym_release_device] = ACTIONS(1601),
    [sym_release_channel] = ACTIONS(1601),
    [sym_release_devchan] = ACTIONS(1601),
    [sym_master_slot] = ACTIONS(1601),
    [sym_get_pulse_time] = ACTIONS(1601),
    [sym_get_timer] = ACTIONS(1601),
    [sym_master_sn] = ACTIONS(1601),
    [sym_system_number] = ACTIONS(1601),
    [sym_button] = ACTIONS(1601),
    [sym_dv_channel] = ACTIONS(1601),
    [sym_data] = ACTIONS(1601),
    [sym_level] = ACTIONS(1601),
    [sym_timeline] = ACTIONS(1601),
    [sym_channel] = ACTIONS(1601),
    [sym_custom] = ACTIONS(1601),
    [sym_atoi] = ACTIONS(1601),
    [sym_atol] = ACTIONS(1601),
    [sym_atof] = ACTIONS(1601),
    [sym_itoa] = ACTIONS(1601),
    [sym_itohex] = ACTIONS(1601),
    [sym_hextoi] = ACTIONS(1601),
    [sym_ftoa] = ACTIONS(1601),
    [sym_find_string] = ACTIONS(1601),
    [sym_left_string] = ACTIONS(1601),
    [sym_right_string] = ACTIONS(1601),
    [sym_mid_string] = ACTIONS(1601),
    [sym_remove_string] = ACTIONS(1601),
    [sym_get_buffer_string] = ACTIONS(1601),
    [sym_get_buffer_char] = ACTIONS(1601),
    [sym_get_multi_buffer_string] = ACTIONS(1601),
    [sym_length_string] = ACTIONS(1601),
    [sym_max_length_string] = ACTIONS(1601),
    [sym_set_length_string] = ACTIONS(1601),
    [sym_compare_string] = ACTIONS(1601),
    [sym_upper_string] = ACTIONS(1601),
    [sym_lower_string] = ACTIONS(1601),
    [sym_get_last] = ACTIONS(1601),
    [sym_date_to_day] = ACTIONS(1601),
    [sym_date_to_month] = ACTIONS(1601),
    [sym_date_to_year] = ACTIONS(1601),
    [sym_time_to_hour] = ACTIONS(1601),
    [sym_time_to_minute] = ACTIONS(1601),
    [sym_time_to_second] = ACTIONS(1601),
    [sym_day_of_week] = ACTIONS(1601),
    [sym_set_pulse_time] = ACTIONS(1601),
    [sym_set_timer] = ACTIONS(1601),
    [sym_file_open] = ACTIONS(1601),
    [sym_file_close] = ACTIONS(1601),
    [sym_file_read] = ACTIONS(1601),
    [sym_file_read_line] = ACTIONS(1601),
    [sym_file_write] = ACTIONS(1601),
    [sym_file_write_line] = ACTIONS(1601),
    [sym_file_copy] = ACTIONS(1601),
    [sym_file_delete] = ACTIONS(1601),
    [sym_file_rename] = ACTIONS(1601),
    [sym_file_dir] = ACTIONS(1601),
    [sym_file_getdir] = ACTIONS(1601),
    [sym_file_setdir] = ACTIONS(1601),
    [sym_file_create_dir] = ACTIONS(1601),
    [sym_file_remove_dir] = ACTIONS(1601),
    [sym_file_seek] = ACTIONS(1601),
    [sym_length_array] = ACTIONS(1601),
    [sym_max_length_array] = ACTIONS(1601),
    [sym_set_length_array] = ACTIONS(1601),
    [sym_ip_client_open] = ACTIONS(1601),
    [sym_ip_client_close] = ACTIONS(1601),
    [sym_ip_server_open] = ACTIONS(1601),
    [sym_ip_server_close] = ACTIONS(1601),
    [sym_ip_bound_client_open] = ACTIONS(1601),
    [sym_ip_mc_server_open] = ACTIONS(1601),
    [sym_ip_set_option] = ACTIONS(1601),
    [sym_ssh_client_open] = ACTIONS(1601),
    [sym_ssh_client_close] = ACTIONS(1601),
    [sym_tls_client_open] = ACTIONS(1601),
    [sym_tls_client_close] = ACTIONS(1601),
    [sym_do_push] = ACTIONS(1601),
    [sym_do_push_timed] = ACTIONS(1601),
    [sym_do_release] = ACTIONS(1601),
    [sym_device_id] = ACTIONS(1601),
    [sym_device_info] = ACTIONS(1601),
    [sym_device_id_string] = ACTIONS(1601),
    [sym_device_standby] = ACTIONS(1601),
    [sym_device_wake] = ACTIONS(1601),
    [sym_timeline_create] = ACTIONS(1601),
    [sym_timeline_kill] = ACTIONS(1601),
    [sym_timeline_reload] = ACTIONS(1601),
    [sym_timeline_pause] = ACTIONS(1601),
    [sym_timeline_restart] = ACTIONS(1601),
    [sym_timeline_set] = ACTIONS(1601),
    [sym_timeline_get] = ACTIONS(1601),
    [sym_timeline_active] = ACTIONS(1601),
    [sym_timeline_dynamic_id] = ACTIONS(1601),
    [sym_variable_to_string] = ACTIONS(1601),
    [sym_string_to_variable] = ACTIONS(1601),
    [sym_length_variable_to_string] = ACTIONS(1601),
    [sym_type_cast] = ACTIONS(1601),
    [sym_variable_to_xml] = ACTIONS(1601),
    [sym_xml_to_variable] = ACTIONS(1601),
    [sym_length_variable_to_xml] = ACTIONS(1601),
    [sym_redirect_string] = ACTIONS(1601),
    [sym_random_number] = ACTIONS(1601),
    [sym_get_unique_id] = ACTIONS(1601),
    [sym_get_system_number] = ACTIONS(1601),
    [sym_set_system_number] = ACTIONS(1601),
    [sym_get_serial_number] = ACTIONS(1601),
    [sym_get_master_build] = ACTIONS(1601),
    [sym_set_validation_code] = ACTIONS(1601),
    [sym_reboot] = ACTIONS(1601),
    [sym_load_duet_module] = ACTIONS(1601),
    [sym_unload_duet_module] = ACTIONS(1601),
    [sym_duet_mem_size_set] = ACTIONS(1601),
    [sym_duet_mem_size_get] = ACTIONS(1601),
    [sym_dynamic_application_device] = ACTIONS(1601),
    [sym_dynamic_polled_port] = ACTIONS(1601),
    [sym_static_port_binding] = ACTIONS(1601),
    [sym_static_ip_binding] = ACTIONS(1601),
    [sym_get_dns_list] = ACTIONS(1601),
    [sym_set_dns_list] = ACTIONS(1601),
    [sym_get_ip_address] = ACTIONS(1601),
    [sym_set_ip_address] = ACTIONS(1601),
    [sym_get_url_list] = ACTIONS(1601),
    [sym_add_url_entry] = ACTIONS(1601),
    [sym_delete_url_entry] = ACTIONS(1601),
    [sym_abs_value] = ACTIONS(1601),
    [sym_max_value] = ACTIONS(1601),
    [sym_min_value] = ACTIONS(1601),
    [sym_exp_value] = ACTIONS(1601),
    [sym_log_value] = ACTIONS(1601),
    [sym_log10_value] = ACTIONS(1601),
    [sym_power_value] = ACTIONS(1601),
    [sym_sqrt_value] = ACTIONS(1601),
    [sym_raw_be] = ACTIONS(1601),
    [sym_raw_le] = ACTIONS(1601),
    [sym_internal_queue_size_set] = ACTIONS(1601),
    [sym_internal_queue_size_get] = ACTIONS(1601),
    [sym_internal_threshold_set] = ACTIONS(1601),
    [sym_internal_threshold_get] = ACTIONS(1601),
    [sym_rebuilt_event] = ACTIONS(1601),
    [sym_do_custom_event] = ACTIONS(1601),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1601),
    [sym_clkmgr_set_clk_source] = ACTIONS(1601),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1601),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1601),
    [sym_clkmgr_get_timezone] = ACTIONS(1601),
    [sym_clkmgr_set_timezone] = ACTIONS(1601),
    [sym_clkmgr_get_resync_period] = ACTIONS(1601),
    [sym_clkmgr_set_resync_period] = ACTIONS(1601),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1601),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1601),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1601),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1601),
    [sym_clkmgr_get_timeservers] = ACTIONS(1601),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1601),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1601),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1601),
    [sym_set_log_level] = ACTIONS(1601),
    [sym_get_log_level] = ACTIONS(1601),
    [sym_amx_log] = ACTIONS(1601),
    [sym_smtp_server_config_set] = ACTIONS(1601),
    [sym_smtp_server_config_get] = ACTIONS(1601),
    [sym_smtp_send] = ACTIONS(1601),
    [sym_validate_netlinx_account] = ACTIONS(1601),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1601),
    [sym_authenticate_certificate] = ACTIONS(1601),
    [sym_audit_netlinx_session_event] = ACTIONS(1601),
    [sym_audit_netlinx_generic_event] = ACTIONS(1601),
    [sym_get_available_flash_disk_space] = ACTIONS(1601),
    [sym_get_max_flash_disk_space] = ACTIONS(1601),
    [sym_data_create_feed] = ACTIONS(1601),
    [sym_data_delete_feed] = ACTIONS(1601),
    [sym_data_publish_feed] = ACTIONS(1601),
    [sym_data_get_published_feed] = ACTIONS(1601),
    [sym_data_add_record] = ACTIONS(1601),
    [sym_data_get_event_record] = ACTIONS(1601),
    [sym__wc_data_create_feed] = ACTIONS(1601),
    [sym__wc_data_add_record] = ACTIONS(1601),
    [sym__wc_data_get_event_record] = ACTIONS(1601),
    [sym_format] = ACTIONS(1601),
    [sym_astro_clock] = ACTIONS(1601),
    [sym_set_outdoor_temperature] = ACTIONS(1601),
    [sym_set_virtual_level_count] = ACTIONS(1601),
    [sym_set_virtual_channel_count] = ACTIONS(1601),
    [sym_set_virtual_port_count] = ACTIONS(1601),
    [sym__wc_encode_int] = ACTIONS(1601),
    [sym__wc_decode_int] = ACTIONS(1601),
    [sym__wc_int] = ACTIONS(1601),
    [sym__wc_to_ch_int] = ACTIONS(1601),
    [sym__ch_to_wc_int] = ACTIONS(1601),
    [sym__wc_find_string_int] = ACTIONS(1601),
    [sym__wc_left_string_int] = ACTIONS(1601),
    [sym__wc_lower_string_int] = ACTIONS(1601),
    [sym__wc_mid_string_int] = ACTIONS(1601),
    [sym__wc_remove_string_int] = ACTIONS(1601),
    [sym__wc_right_string_int] = ACTIONS(1601),
    [sym__wc_upper_string_int] = ACTIONS(1601),
    [sym__wc_compare_string_int] = ACTIONS(1601),
    [sym__wc_get_buffer_char_int] = ACTIONS(1601),
    [sym__wc_get_buffer_string_int] = ACTIONS(1601),
    [sym__wc_concat_string_int] = ACTIONS(1601),
    [sym__wc_file_open_int] = ACTIONS(1601),
    [sym__wc_file_close_int] = ACTIONS(1601),
    [sym__wc_file_read_int] = ACTIONS(1601),
    [sym__wc_file_read_line_int] = ACTIONS(1601),
    [sym__wc_file_write_int] = ACTIONS(1601),
    [sym__wc_file_write_line_int] = ACTIONS(1601),
    [aux_sym_preproc_include_token1] = ACTIONS(1601),
    [aux_sym_preproc_define_token1] = ACTIONS(1603),
    [aux_sym_preproc_warn_token1] = ACTIONS(1603),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1603),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1603),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1603),
    [anon_sym_LBRACK] = ACTIONS(1603),
    [anon_sym_LBRACE] = ACTIONS(1603),
    [anon_sym_COLON] = ACTIONS(1603),
    [anon_sym_LPAREN] = ACTIONS(1601),
    [anon_sym_SEMI] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1601),
    [anon_sym_TILDE] = ACTIONS(1603),
    [anon_sym_DASH] = ACTIONS(1601),
    [anon_sym_PLUS] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1603),
    [anon_sym_SLASH] = ACTIONS(1601),
    [anon_sym_PERCENT] = ACTIONS(1603),
    [anon_sym_PIPE_PIPE] = ACTIONS(1603),
    [anon_sym_AMP_AMP] = ACTIONS(1603),
    [anon_sym_PIPE] = ACTIONS(1601),
    [anon_sym_CARET] = ACTIONS(1603),
    [anon_sym_AMP] = ACTIONS(1601),
    [anon_sym_EQ_EQ] = ACTIONS(1603),
    [anon_sym_BANG_EQ] = ACTIONS(1603),
    [anon_sym_LT_GT] = ACTIONS(1603),
    [anon_sym_GT] = ACTIONS(1601),
    [anon_sym_GT_EQ] = ACTIONS(1603),
    [anon_sym_LT_EQ] = ACTIONS(1603),
    [anon_sym_LT] = ACTIONS(1601),
    [anon_sym_LT_LT] = ACTIONS(1603),
    [anon_sym_GT_GT] = ACTIONS(1603),
    [anon_sym_DASH_DASH] = ACTIONS(1603),
    [anon_sym_PLUS_PLUS] = ACTIONS(1603),
    [anon_sym_DOT] = ACTIONS(1601),
    [anon_sym_DQUOTE] = ACTIONS(1603),
    [anon_sym_SQUOTE] = ACTIONS(1603),
    [sym_number_literal] = ACTIONS(1603),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1603),
  },
  [STATE(147)] = {
    [ts_builtin_sym_end] = ACTIONS(1579),
    [sym_identifier] = ACTIONS(1577),
    [sym_program_name_keyword] = ACTIONS(1577),
    [sym_module_name_keyword] = ACTIONS(1577),
    [sym_define_device_keyword] = ACTIONS(1577),
    [sym_define_combine_keyword] = ACTIONS(1577),
    [sym_define_constant_keyword] = ACTIONS(1577),
    [sym_define_type_keyword] = ACTIONS(1577),
    [sym_define_variable_keyword] = ACTIONS(1577),
    [sym_define_system_variable_keyword] = ACTIONS(1577),
    [sym_define_connect_level_keyword] = ACTIONS(1577),
    [sym_define_latching_keyword] = ACTIONS(1577),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1577),
    [sym_define_toggling_keyword] = ACTIONS(1577),
    [sym_define_call_keyword] = ACTIONS(1577),
    [sym_define_function_keyword] = ACTIONS(1577),
    [sym_define_library_function_keyword] = ACTIONS(1577),
    [sym_define_start_keyword] = ACTIONS(1577),
    [sym_define_event_keyword] = ACTIONS(1577),
    [sym_define_program_keyword] = ACTIONS(1577),
    [sym_define_module_keyword] = ACTIONS(1577),
    [sym_char_keyword] = ACTIONS(1577),
    [sym_widechar_keyword] = ACTIONS(1577),
    [sym_integer_keyword] = ACTIONS(1577),
    [sym_sinteger_keyword] = ACTIONS(1577),
    [sym_long_keyword] = ACTIONS(1577),
    [sym_slong_keyword] = ACTIONS(1577),
    [sym_float_keyword] = ACTIONS(1577),
    [sym_double_keyword] = ACTIONS(1577),
    [sym_variant_keyword] = ACTIONS(1577),
    [sym_variantarray_keyword] = ACTIONS(1577),
    [sym_local_var_keyword] = ACTIONS(1577),
    [sym_stack_var_keyword] = ACTIONS(1577),
    [sym_constant_keyword] = ACTIONS(1577),
    [sym_volatile_keyword] = ACTIONS(1577),
    [sym_non_volatile_keyword] = ACTIONS(1577),
    [sym_persistent_keyword] = ACTIONS(1577),
    [sym_dev_keyword] = ACTIONS(1577),
    [sym_devlev_keyword] = ACTIONS(1577),
    [sym_devchan_keyword] = ACTIONS(1577),
    [sym_if_keyword] = ACTIONS(1577),
    [sym_else_keyword] = ACTIONS(1577),
    [sym_switch_keyword] = ACTIONS(1577),
    [sym_case_keyword] = ACTIONS(1577),
    [sym_default_keyword] = ACTIONS(1577),
    [sym_while_keyword] = ACTIONS(1577),
    [sym_for_keyword] = ACTIONS(1577),
    [sym_break_keyword] = ACTIONS(1577),
    [sym_continue_keyword] = ACTIONS(1577),
    [sym_return_keyword] = ACTIONS(1577),
    [sym_select_keyword] = ACTIONS(1577),
    [sym_active_keyword] = ACTIONS(1577),
    [sym_struct_keyword] = ACTIONS(1577),
    [sym_structure_keyword] = ACTIONS(1577),
    [sym_button_event_keyword] = ACTIONS(1577),
    [sym_channel_event_keyword] = ACTIONS(1577),
    [sym_level_event_keyword] = ACTIONS(1577),
    [sym_data_event_keyword] = ACTIONS(1577),
    [sym_timeline_event_keyword] = ACTIONS(1577),
    [sym_custom_event_keyword] = ACTIONS(1577),
    [sym_push_keyword] = ACTIONS(1577),
    [sym_release_keyword] = ACTIONS(1577),
    [aux_sym_on_keyword_token1] = ACTIONS(1577),
    [aux_sym_off_keyword_token1] = ACTIONS(1577),
    [sym_send_level_keyword] = ACTIONS(1577),
    [sym_send_string_keyword] = ACTIONS(1577),
    [sym_send_command_keyword] = ACTIONS(1577),
    [sym_clear_buffer_keyword] = ACTIONS(1577),
    [sym_create_buffer_keyword] = ACTIONS(1577),
    [sym_create_multi_buffer_keyword] = ACTIONS(1577),
    [sym_call_keyword] = ACTIONS(1577),
    [sym_system_call_keyword] = ACTIONS(1577),
    [sym_devchan_to_keyword] = ACTIONS(1577),
    [sym_devchan_min_to_keyword] = ACTIONS(1577),
    [sym_devchan_total_off_keyword] = ACTIONS(1577),
    [sym_devchan_pulse_keyword] = ACTIONS(1577),
    [sym_wait_keyword] = ACTIONS(1577),
    [sym_cancel_wait_keyword] = ACTIONS(1577),
    [sym_wait_until_keyword] = ACTIONS(1577),
    [sym_cancel_wait_until_keyword] = ACTIONS(1577),
    [sym_cancel_all_wait_keyword] = ACTIONS(1577),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1577),
    [sym_and] = ACTIONS(1577),
    [sym_or] = ACTIONS(1577),
    [sym_not] = ACTIONS(1577),
    [sym_xor] = ACTIONS(1577),
    [sym_band] = ACTIONS(1577),
    [sym_bor] = ACTIONS(1577),
    [sym_bxor] = ACTIONS(1577),
    [sym_bnot] = ACTIONS(1577),
    [sym_lshift] = ACTIONS(1577),
    [sym_rshift] = ACTIONS(1577),
    [sym___file__] = ACTIONS(1577),
    [sym___date__] = ACTIONS(1577),
    [sym___ldate__] = ACTIONS(1577),
    [sym___time__] = ACTIONS(1577),
    [sym___line__] = ACTIONS(1577),
    [sym___name__] = ACTIONS(1577),
    [sym___netlinx__] = ACTIONS(1577),
    [sym_netlinx_axi_version] = ACTIONS(1577),
    [sym_url_flg_tcp] = ACTIONS(1577),
    [sym_url_flg_accinfopresent] = ACTIONS(1577),
    [sym_url_flg_temp] = ACTIONS(1577),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1577),
    [sym_url_flg_stat_mask] = ACTIONS(1577),
    [sym_url_flg_stat_lookup] = ACTIONS(1577),
    [sym_url_flg_stat_connecting] = ACTIONS(1577),
    [sym_url_flg_stat_waiting] = ACTIONS(1577),
    [sym_url_flg_stat_connected] = ACTIONS(1577),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1577),
    [sym_first_virtual_device] = ACTIONS(1577),
    [sym_first_local_port] = ACTIONS(1577),
    [sym_dynamic_virtual_device] = ACTIONS(1577),
    [sym_timeline_once] = ACTIONS(1577),
    [sym_timeline_repeat] = ACTIONS(1577),
    [sym_timeline_absolute] = ACTIONS(1577),
    [sym_timeline_relative] = ACTIONS(1577),
    [sym_do_push_timed_infinite] = ACTIONS(1577),
    [sym_source_type_no_address] = ACTIONS(1577),
    [sym_source_type_neuron_id] = ACTIONS(1577),
    [sym_source_type_ip_address] = ACTIONS(1577),
    [sym_source_type_axlink] = ACTIONS(1577),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1577),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1577),
    [sym_source_type_ip_socket_address] = ACTIONS(1577),
    [sym_source_type_rs232] = ACTIONS(1577),
    [sym_source_type_internal] = ACTIONS(1577),
    [sym_source_type_mac_address] = ACTIONS(1577),
    [sym_source_type_ipv4_port] = ACTIONS(1577),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1577),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1577),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1577),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1577),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1577),
    [sym_true] = ACTIONS(1577),
    [sym_false] = ACTIONS(1577),
    [sym_file_read_only] = ACTIONS(1577),
    [sym_file_rw_new] = ACTIONS(1577),
    [sym_file_rw_append] = ACTIONS(1577),
    [sym_ip_tcp] = ACTIONS(1577),
    [sym_ip_udp] = ACTIONS(1577),
    [sym_ip_udp_2way] = ACTIONS(1577),
    [sym_xml_encode_types] = ACTIONS(1577),
    [sym_xml_encode_char_as_list] = ACTIONS(1577),
    [sym_xml_encode_le] = ACTIONS(1577),
    [sym_xml_decode_types] = ACTIONS(1577),
    [sym_xml_decode_no_preserve] = ACTIONS(1577),
    [sym_duet_dev_type_display_device] = ACTIONS(1577),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1577),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1577),
    [sym_duet_dev_type_io_device] = ACTIONS(1577),
    [sym_duet_dev_type_relay_device] = ACTIONS(1577),
    [sym_duet_dev_type_ups] = ACTIONS(1577),
    [sym_duet_dev_type_amplifier] = ACTIONS(1577),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1577),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1577),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1577),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1577),
    [sym_duet_dev_type_camera] = ACTIONS(1577),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1577),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1577),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1577),
    [sym_duet_dev_type_dss] = ACTIONS(1577),
    [sym_duet_dev_type_dvr] = ACTIONS(1577),
    [sym_duet_dev_type_disc_device] = ACTIONS(1577),
    [sym_duet_dev_type_document_camera] = ACTIONS(1577),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1577),
    [sym_duet_dev_type_hvac] = ACTIONS(1577),
    [sym_duet_dev_type_keypad] = ACTIONS(1577),
    [sym_duet_dev_type_light] = ACTIONS(1577),
    [sym_duet_dev_type_monitor] = ACTIONS(1577),
    [sym_duet_dev_type_motor] = ACTIONS(1577),
    [sym_duet_dev_type_multi_window] = ACTIONS(1577),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1577),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1577),
    [sym_duet_dev_type_receiver] = ACTIONS(1577),
    [sym_duet_dev_type_security_system] = ACTIONS(1577),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1577),
    [sym_duet_dev_type_settop_box] = ACTIONS(1577),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1577),
    [sym_duet_dev_type_switcher] = ACTIONS(1577),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1577),
    [sym_duet_dev_type_tv] = ACTIONS(1577),
    [sym_duet_dev_type_utility] = ACTIONS(1577),
    [sym_duet_dev_type_vcr] = ACTIONS(1577),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1577),
    [sym_duet_dev_type_video_processor] = ACTIONS(1577),
    [sym_duet_dev_type_video_projector] = ACTIONS(1577),
    [sym_duet_dev_type_video_wall] = ACTIONS(1577),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1577),
    [sym_duet_dev_type_weather] = ACTIONS(1577),
    [sym_duet_dev_not_polled] = ACTIONS(1577),
    [sym_duet_dev_polled] = ACTIONS(1577),
    [sym_clkmgr_mode_network] = ACTIONS(1577),
    [sym_clkmgr_mode_standalone] = ACTIONS(1577),
    [sym_normal_standby] = ACTIONS(1577),
    [sym_normal_wake] = ACTIONS(1577),
    [sym_ip_multicast_ttl_option] = ACTIONS(1577),
    [sym_ip_ttl_subnet] = ACTIONS(1577),
    [sym_ip_ttl_site] = ACTIONS(1577),
    [sym_ip_ttl_region] = ACTIONS(1577),
    [sym_ip_ttl_continent] = ACTIONS(1577),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1577),
    [sym_ip_nodelay_on] = ACTIONS(1577),
    [sym_ip_nodelay_off] = ACTIONS(1577),
    [sym_amx_error] = ACTIONS(1577),
    [sym_amx_warning] = ACTIONS(1577),
    [sym_amx_info] = ACTIONS(1577),
    [sym_amx_debug] = ACTIONS(1577),
    [sym_netlinx_login_success] = ACTIONS(1577),
    [sym_netlinx_login_fail] = ACTIONS(1577),
    [sym_netlinx_logout] = ACTIONS(1577),
    [sym_null_str] = ACTIONS(1577),
    [sym_smtp_address] = ACTIONS(1577),
    [sym_smtp_port_number] = ACTIONS(1577),
    [sym_smtp_username] = ACTIONS(1577),
    [sym_smtp_password] = ACTIONS(1577),
    [sym_smtp_require_tls] = ACTIONS(1577),
    [sym_smtp_from] = ACTIONS(1577),
    [sym_smtp_tls_true] = ACTIONS(1577),
    [sym_smtp_tls_false] = ACTIONS(1577),
    [sym_listview_on_row_select_event] = ACTIONS(1577),
    [sym_data_structure_unknown] = ACTIONS(1577),
    [sym_data_structure_datafeed] = ACTIONS(1577),
    [sym_data_structure_datarecordset] = ACTIONS(1577),
    [sym_data_structure_datarecord] = ACTIONS(1577),
    [sym_data_structure_datafield] = ACTIONS(1577),
    [sym_data_max_content_field_count] = ACTIONS(1577),
    [sym_data_max_metadata_field_count] = ACTIONS(1577),
    [sym_data_max_id_length] = ACTIONS(1577),
    [sym_data_max_name_length] = ACTIONS(1577),
    [sym_data_max_description_length] = ACTIONS(1577),
    [sym_data_max_type_length] = ACTIONS(1577),
    [sym_data_max_format_length] = ACTIONS(1577),
    [sym_data_max_source_length] = ACTIONS(1577),
    [sym_data_max_value_length] = ACTIONS(1577),
    [sym_data_max_label_length] = ACTIONS(1577),
    [sym_data_max_url_length] = ACTIONS(1577),
    [sym_max_last_login_info_length] = ACTIONS(1577),
    [sym_data_type_unknown] = ACTIONS(1577),
    [sym_data_type_string] = ACTIONS(1577),
    [sym_data_type_datetime] = ACTIONS(1577),
    [sym_data_type_date] = ACTIONS(1577),
    [sym_data_type_time] = ACTIONS(1577),
    [sym_data_type_image] = ACTIONS(1577),
    [sym_data_format_url] = ACTIONS(1577),
    [sym_data_format_phone] = ACTIONS(1577),
    [sym_data_format_email] = ACTIONS(1577),
    [sym_data_format_iso8601] = ACTIONS(1577),
    [sym_auditlog_priv] = ACTIONS(1577),
    [sym_configuration_priv] = ACTIONS(1577),
    [sym_deviceconfig_priv] = ACTIONS(1577),
    [sym_ftp_priv] = ACTIONS(1577),
    [sym_http_priv] = ACTIONS(1577),
    [sym_networkconfig_priv] = ACTIONS(1577),
    [sym_programport_priv] = ACTIONS(1577),
    [sym_remoteui_priv] = ACTIONS(1577),
    [sym_securitycontrol_priv] = ACTIONS(1577),
    [sym_softwaremanagement_priv] = ACTIONS(1577),
    [sym_terminal_priv] = ACTIONS(1577),
    [sym_tpadmin_priv] = ACTIONS(1577),
    [sym_user1_priv] = ACTIONS(1577),
    [sym_user2_priv] = ACTIONS(1577),
    [sym_user3_priv] = ACTIONS(1577),
    [sym_user4_priv] = ACTIONS(1577),
    [sym_usermanagement_priv] = ACTIONS(1577),
    [sym_valid_account] = ACTIONS(1577),
    [sym_err_name_invalid] = ACTIONS(1577),
    [sym_err_password_invalid] = ACTIONS(1577),
    [sym_err_invalid_account] = ACTIONS(1577),
    [sym_err_no_authorization] = ACTIONS(1577),
    [sym_err_invalid_parameter] = ACTIONS(1577),
    [sym_err_user_locked_out] = ACTIONS(1577),
    [sym_err_user_account_expired] = ACTIONS(1577),
    [sym_err_general_error] = ACTIONS(1577),
    [sym_err_authentication_error] = ACTIONS(1577),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1577),
    [sym_tls_validate_certificate] = ACTIONS(1577),
    [sym_tdata] = ACTIONS(1577),
    [sym_tchannel] = ACTIONS(1577),
    [sym_tlevel] = ACTIONS(1577),
    [sym_tbutton] = ACTIONS(1577),
    [sym_ttimeline] = ACTIONS(1577),
    [sym_tcustom] = ACTIONS(1577),
    [sym_url_struct] = ACTIONS(1577),
    [sym_dns_struct] = ACTIONS(1577),
    [sym_ip_address_struct] = ACTIONS(1577),
    [sym_dev_info_struct] = ACTIONS(1577),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1577),
    [sym_data_feed] = ACTIONS(1577),
    [sym_data_field] = ACTIONS(1577),
    [sym_data_record] = ACTIONS(1577),
    [sym_wc_data_feed] = ACTIONS(1577),
    [sym_wc_data_field] = ACTIONS(1577),
    [sym_wc_data_record] = ACTIONS(1577),
    [sym_last_login_info] = ACTIONS(1577),
    [sym____reserved___] = ACTIONS(1577),
    [sym_date] = ACTIONS(1577),
    [sym_day] = ACTIONS(1577),
    [sym_time] = ACTIONS(1577),
    [sym_ldate] = ACTIONS(1577),
    [sym_push_device] = ACTIONS(1577),
    [sym_push_channel] = ACTIONS(1577),
    [sym_push_devchan] = ACTIONS(1577),
    [sym_release_device] = ACTIONS(1577),
    [sym_release_channel] = ACTIONS(1577),
    [sym_release_devchan] = ACTIONS(1577),
    [sym_master_slot] = ACTIONS(1577),
    [sym_get_pulse_time] = ACTIONS(1577),
    [sym_get_timer] = ACTIONS(1577),
    [sym_master_sn] = ACTIONS(1577),
    [sym_system_number] = ACTIONS(1577),
    [sym_button] = ACTIONS(1577),
    [sym_dv_channel] = ACTIONS(1577),
    [sym_data] = ACTIONS(1577),
    [sym_level] = ACTIONS(1577),
    [sym_timeline] = ACTIONS(1577),
    [sym_channel] = ACTIONS(1577),
    [sym_custom] = ACTIONS(1577),
    [sym_atoi] = ACTIONS(1577),
    [sym_atol] = ACTIONS(1577),
    [sym_atof] = ACTIONS(1577),
    [sym_itoa] = ACTIONS(1577),
    [sym_itohex] = ACTIONS(1577),
    [sym_hextoi] = ACTIONS(1577),
    [sym_ftoa] = ACTIONS(1577),
    [sym_find_string] = ACTIONS(1577),
    [sym_left_string] = ACTIONS(1577),
    [sym_right_string] = ACTIONS(1577),
    [sym_mid_string] = ACTIONS(1577),
    [sym_remove_string] = ACTIONS(1577),
    [sym_get_buffer_string] = ACTIONS(1577),
    [sym_get_buffer_char] = ACTIONS(1577),
    [sym_get_multi_buffer_string] = ACTIONS(1577),
    [sym_length_string] = ACTIONS(1577),
    [sym_max_length_string] = ACTIONS(1577),
    [sym_set_length_string] = ACTIONS(1577),
    [sym_compare_string] = ACTIONS(1577),
    [sym_upper_string] = ACTIONS(1577),
    [sym_lower_string] = ACTIONS(1577),
    [sym_get_last] = ACTIONS(1577),
    [sym_date_to_day] = ACTIONS(1577),
    [sym_date_to_month] = ACTIONS(1577),
    [sym_date_to_year] = ACTIONS(1577),
    [sym_time_to_hour] = ACTIONS(1577),
    [sym_time_to_minute] = ACTIONS(1577),
    [sym_time_to_second] = ACTIONS(1577),
    [sym_day_of_week] = ACTIONS(1577),
    [sym_set_pulse_time] = ACTIONS(1577),
    [sym_set_timer] = ACTIONS(1577),
    [sym_file_open] = ACTIONS(1577),
    [sym_file_close] = ACTIONS(1577),
    [sym_file_read] = ACTIONS(1577),
    [sym_file_read_line] = ACTIONS(1577),
    [sym_file_write] = ACTIONS(1577),
    [sym_file_write_line] = ACTIONS(1577),
    [sym_file_copy] = ACTIONS(1577),
    [sym_file_delete] = ACTIONS(1577),
    [sym_file_rename] = ACTIONS(1577),
    [sym_file_dir] = ACTIONS(1577),
    [sym_file_getdir] = ACTIONS(1577),
    [sym_file_setdir] = ACTIONS(1577),
    [sym_file_create_dir] = ACTIONS(1577),
    [sym_file_remove_dir] = ACTIONS(1577),
    [sym_file_seek] = ACTIONS(1577),
    [sym_length_array] = ACTIONS(1577),
    [sym_max_length_array] = ACTIONS(1577),
    [sym_set_length_array] = ACTIONS(1577),
    [sym_ip_client_open] = ACTIONS(1577),
    [sym_ip_client_close] = ACTIONS(1577),
    [sym_ip_server_open] = ACTIONS(1577),
    [sym_ip_server_close] = ACTIONS(1577),
    [sym_ip_bound_client_open] = ACTIONS(1577),
    [sym_ip_mc_server_open] = ACTIONS(1577),
    [sym_ip_set_option] = ACTIONS(1577),
    [sym_ssh_client_open] = ACTIONS(1577),
    [sym_ssh_client_close] = ACTIONS(1577),
    [sym_tls_client_open] = ACTIONS(1577),
    [sym_tls_client_close] = ACTIONS(1577),
    [sym_do_push] = ACTIONS(1577),
    [sym_do_push_timed] = ACTIONS(1577),
    [sym_do_release] = ACTIONS(1577),
    [sym_device_id] = ACTIONS(1577),
    [sym_device_info] = ACTIONS(1577),
    [sym_device_id_string] = ACTIONS(1577),
    [sym_device_standby] = ACTIONS(1577),
    [sym_device_wake] = ACTIONS(1577),
    [sym_timeline_create] = ACTIONS(1577),
    [sym_timeline_kill] = ACTIONS(1577),
    [sym_timeline_reload] = ACTIONS(1577),
    [sym_timeline_pause] = ACTIONS(1577),
    [sym_timeline_restart] = ACTIONS(1577),
    [sym_timeline_set] = ACTIONS(1577),
    [sym_timeline_get] = ACTIONS(1577),
    [sym_timeline_active] = ACTIONS(1577),
    [sym_timeline_dynamic_id] = ACTIONS(1577),
    [sym_variable_to_string] = ACTIONS(1577),
    [sym_string_to_variable] = ACTIONS(1577),
    [sym_length_variable_to_string] = ACTIONS(1577),
    [sym_type_cast] = ACTIONS(1577),
    [sym_variable_to_xml] = ACTIONS(1577),
    [sym_xml_to_variable] = ACTIONS(1577),
    [sym_length_variable_to_xml] = ACTIONS(1577),
    [sym_redirect_string] = ACTIONS(1577),
    [sym_random_number] = ACTIONS(1577),
    [sym_get_unique_id] = ACTIONS(1577),
    [sym_get_system_number] = ACTIONS(1577),
    [sym_set_system_number] = ACTIONS(1577),
    [sym_get_serial_number] = ACTIONS(1577),
    [sym_get_master_build] = ACTIONS(1577),
    [sym_set_validation_code] = ACTIONS(1577),
    [sym_reboot] = ACTIONS(1577),
    [sym_load_duet_module] = ACTIONS(1577),
    [sym_unload_duet_module] = ACTIONS(1577),
    [sym_duet_mem_size_set] = ACTIONS(1577),
    [sym_duet_mem_size_get] = ACTIONS(1577),
    [sym_dynamic_application_device] = ACTIONS(1577),
    [sym_dynamic_polled_port] = ACTIONS(1577),
    [sym_static_port_binding] = ACTIONS(1577),
    [sym_static_ip_binding] = ACTIONS(1577),
    [sym_get_dns_list] = ACTIONS(1577),
    [sym_set_dns_list] = ACTIONS(1577),
    [sym_get_ip_address] = ACTIONS(1577),
    [sym_set_ip_address] = ACTIONS(1577),
    [sym_get_url_list] = ACTIONS(1577),
    [sym_add_url_entry] = ACTIONS(1577),
    [sym_delete_url_entry] = ACTIONS(1577),
    [sym_abs_value] = ACTIONS(1577),
    [sym_max_value] = ACTIONS(1577),
    [sym_min_value] = ACTIONS(1577),
    [sym_exp_value] = ACTIONS(1577),
    [sym_log_value] = ACTIONS(1577),
    [sym_log10_value] = ACTIONS(1577),
    [sym_power_value] = ACTIONS(1577),
    [sym_sqrt_value] = ACTIONS(1577),
    [sym_raw_be] = ACTIONS(1577),
    [sym_raw_le] = ACTIONS(1577),
    [sym_internal_queue_size_set] = ACTIONS(1577),
    [sym_internal_queue_size_get] = ACTIONS(1577),
    [sym_internal_threshold_set] = ACTIONS(1577),
    [sym_internal_threshold_get] = ACTIONS(1577),
    [sym_rebuilt_event] = ACTIONS(1577),
    [sym_do_custom_event] = ACTIONS(1577),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1577),
    [sym_clkmgr_set_clk_source] = ACTIONS(1577),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1577),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1577),
    [sym_clkmgr_get_timezone] = ACTIONS(1577),
    [sym_clkmgr_set_timezone] = ACTIONS(1577),
    [sym_clkmgr_get_resync_period] = ACTIONS(1577),
    [sym_clkmgr_set_resync_period] = ACTIONS(1577),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1577),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1577),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1577),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1577),
    [sym_clkmgr_get_timeservers] = ACTIONS(1577),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1577),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1577),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1577),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1577),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1577),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1577),
    [sym_set_log_level] = ACTIONS(1577),
    [sym_get_log_level] = ACTIONS(1577),
    [sym_amx_log] = ACTIONS(1577),
    [sym_smtp_server_config_set] = ACTIONS(1577),
    [sym_smtp_server_config_get] = ACTIONS(1577),
    [sym_smtp_send] = ACTIONS(1577),
    [sym_validate_netlinx_account] = ACTIONS(1577),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1577),
    [sym_authenticate_certificate] = ACTIONS(1577),
    [sym_audit_netlinx_session_event] = ACTIONS(1577),
    [sym_audit_netlinx_generic_event] = ACTIONS(1577),
    [sym_get_available_flash_disk_space] = ACTIONS(1577),
    [sym_get_max_flash_disk_space] = ACTIONS(1577),
    [sym_data_create_feed] = ACTIONS(1577),
    [sym_data_delete_feed] = ACTIONS(1577),
    [sym_data_publish_feed] = ACTIONS(1577),
    [sym_data_get_published_feed] = ACTIONS(1577),
    [sym_data_add_record] = ACTIONS(1577),
    [sym_data_get_event_record] = ACTIONS(1577),
    [sym__wc_data_create_feed] = ACTIONS(1577),
    [sym__wc_data_add_record] = ACTIONS(1577),
    [sym__wc_data_get_event_record] = ACTIONS(1577),
    [sym_format] = ACTIONS(1577),
    [sym_astro_clock] = ACTIONS(1577),
    [sym_set_outdoor_temperature] = ACTIONS(1577),
    [sym_set_virtual_level_count] = ACTIONS(1577),
    [sym_set_virtual_channel_count] = ACTIONS(1577),
    [sym_set_virtual_port_count] = ACTIONS(1577),
    [sym__wc_encode_int] = ACTIONS(1577),
    [sym__wc_decode_int] = ACTIONS(1577),
    [sym__wc_int] = ACTIONS(1577),
    [sym__wc_to_ch_int] = ACTIONS(1577),
    [sym__ch_to_wc_int] = ACTIONS(1577),
    [sym__wc_find_string_int] = ACTIONS(1577),
    [sym__wc_left_string_int] = ACTIONS(1577),
    [sym__wc_lower_string_int] = ACTIONS(1577),
    [sym__wc_mid_string_int] = ACTIONS(1577),
    [sym__wc_remove_string_int] = ACTIONS(1577),
    [sym__wc_right_string_int] = ACTIONS(1577),
    [sym__wc_upper_string_int] = ACTIONS(1577),
    [sym__wc_compare_string_int] = ACTIONS(1577),
    [sym__wc_get_buffer_char_int] = ACTIONS(1577),
    [sym__wc_get_buffer_string_int] = ACTIONS(1577),
    [sym__wc_concat_string_int] = ACTIONS(1577),
    [sym__wc_file_open_int] = ACTIONS(1577),
    [sym__wc_file_close_int] = ACTIONS(1577),
    [sym__wc_file_read_int] = ACTIONS(1577),
    [sym__wc_file_read_line_int] = ACTIONS(1577),
    [sym__wc_file_write_int] = ACTIONS(1577),
    [sym__wc_file_write_line_int] = ACTIONS(1577),
    [aux_sym_preproc_include_token1] = ACTIONS(1577),
    [aux_sym_preproc_define_token1] = ACTIONS(1579),
    [aux_sym_preproc_warn_token1] = ACTIONS(1579),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1579),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1579),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1579),
    [anon_sym_LBRACK] = ACTIONS(1579),
    [anon_sym_LBRACE] = ACTIONS(1579),
    [anon_sym_COLON] = ACTIONS(1579),
    [anon_sym_LPAREN] = ACTIONS(1577),
    [anon_sym_SEMI] = ACTIONS(1579),
    [anon_sym_BANG] = ACTIONS(1577),
    [anon_sym_TILDE] = ACTIONS(1579),
    [anon_sym_DASH] = ACTIONS(1577),
    [anon_sym_PLUS] = ACTIONS(1577),
    [anon_sym_STAR] = ACTIONS(1579),
    [anon_sym_SLASH] = ACTIONS(1577),
    [anon_sym_PERCENT] = ACTIONS(1579),
    [anon_sym_PIPE_PIPE] = ACTIONS(1579),
    [anon_sym_AMP_AMP] = ACTIONS(1579),
    [anon_sym_PIPE] = ACTIONS(1577),
    [anon_sym_CARET] = ACTIONS(1579),
    [anon_sym_AMP] = ACTIONS(1577),
    [anon_sym_EQ_EQ] = ACTIONS(1579),
    [anon_sym_BANG_EQ] = ACTIONS(1579),
    [anon_sym_LT_GT] = ACTIONS(1579),
    [anon_sym_GT] = ACTIONS(1577),
    [anon_sym_GT_EQ] = ACTIONS(1579),
    [anon_sym_LT_EQ] = ACTIONS(1579),
    [anon_sym_LT] = ACTIONS(1577),
    [anon_sym_LT_LT] = ACTIONS(1579),
    [anon_sym_GT_GT] = ACTIONS(1579),
    [anon_sym_DASH_DASH] = ACTIONS(1579),
    [anon_sym_PLUS_PLUS] = ACTIONS(1579),
    [anon_sym_DOT] = ACTIONS(1577),
    [anon_sym_DQUOTE] = ACTIONS(1579),
    [anon_sym_SQUOTE] = ACTIONS(1579),
    [sym_number_literal] = ACTIONS(1579),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1579),
  },
  [STATE(148)] = {
    [ts_builtin_sym_end] = ACTIONS(1591),
    [sym_identifier] = ACTIONS(1589),
    [sym_program_name_keyword] = ACTIONS(1589),
    [sym_module_name_keyword] = ACTIONS(1589),
    [sym_define_device_keyword] = ACTIONS(1589),
    [sym_define_combine_keyword] = ACTIONS(1589),
    [sym_define_constant_keyword] = ACTIONS(1589),
    [sym_define_type_keyword] = ACTIONS(1589),
    [sym_define_variable_keyword] = ACTIONS(1589),
    [sym_define_system_variable_keyword] = ACTIONS(1589),
    [sym_define_connect_level_keyword] = ACTIONS(1589),
    [sym_define_latching_keyword] = ACTIONS(1589),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1589),
    [sym_define_toggling_keyword] = ACTIONS(1589),
    [sym_define_call_keyword] = ACTIONS(1589),
    [sym_define_function_keyword] = ACTIONS(1589),
    [sym_define_library_function_keyword] = ACTIONS(1589),
    [sym_define_start_keyword] = ACTIONS(1589),
    [sym_define_event_keyword] = ACTIONS(1589),
    [sym_define_program_keyword] = ACTIONS(1589),
    [sym_define_module_keyword] = ACTIONS(1589),
    [sym_char_keyword] = ACTIONS(1589),
    [sym_widechar_keyword] = ACTIONS(1589),
    [sym_integer_keyword] = ACTIONS(1589),
    [sym_sinteger_keyword] = ACTIONS(1589),
    [sym_long_keyword] = ACTIONS(1589),
    [sym_slong_keyword] = ACTIONS(1589),
    [sym_float_keyword] = ACTIONS(1589),
    [sym_double_keyword] = ACTIONS(1589),
    [sym_variant_keyword] = ACTIONS(1589),
    [sym_variantarray_keyword] = ACTIONS(1589),
    [sym_local_var_keyword] = ACTIONS(1589),
    [sym_stack_var_keyword] = ACTIONS(1589),
    [sym_constant_keyword] = ACTIONS(1589),
    [sym_volatile_keyword] = ACTIONS(1589),
    [sym_non_volatile_keyword] = ACTIONS(1589),
    [sym_persistent_keyword] = ACTIONS(1589),
    [sym_dev_keyword] = ACTIONS(1589),
    [sym_devlev_keyword] = ACTIONS(1589),
    [sym_devchan_keyword] = ACTIONS(1589),
    [sym_if_keyword] = ACTIONS(1589),
    [sym_else_keyword] = ACTIONS(1589),
    [sym_switch_keyword] = ACTIONS(1589),
    [sym_case_keyword] = ACTIONS(1589),
    [sym_default_keyword] = ACTIONS(1589),
    [sym_while_keyword] = ACTIONS(1589),
    [sym_for_keyword] = ACTIONS(1589),
    [sym_break_keyword] = ACTIONS(1589),
    [sym_continue_keyword] = ACTIONS(1589),
    [sym_return_keyword] = ACTIONS(1589),
    [sym_select_keyword] = ACTIONS(1589),
    [sym_active_keyword] = ACTIONS(1589),
    [sym_struct_keyword] = ACTIONS(1589),
    [sym_structure_keyword] = ACTIONS(1589),
    [sym_button_event_keyword] = ACTIONS(1589),
    [sym_channel_event_keyword] = ACTIONS(1589),
    [sym_level_event_keyword] = ACTIONS(1589),
    [sym_data_event_keyword] = ACTIONS(1589),
    [sym_timeline_event_keyword] = ACTIONS(1589),
    [sym_custom_event_keyword] = ACTIONS(1589),
    [sym_push_keyword] = ACTIONS(1589),
    [sym_release_keyword] = ACTIONS(1589),
    [aux_sym_on_keyword_token1] = ACTIONS(1589),
    [aux_sym_off_keyword_token1] = ACTIONS(1589),
    [sym_send_level_keyword] = ACTIONS(1589),
    [sym_send_string_keyword] = ACTIONS(1589),
    [sym_send_command_keyword] = ACTIONS(1589),
    [sym_clear_buffer_keyword] = ACTIONS(1589),
    [sym_create_buffer_keyword] = ACTIONS(1589),
    [sym_create_multi_buffer_keyword] = ACTIONS(1589),
    [sym_call_keyword] = ACTIONS(1589),
    [sym_system_call_keyword] = ACTIONS(1589),
    [sym_devchan_to_keyword] = ACTIONS(1589),
    [sym_devchan_min_to_keyword] = ACTIONS(1589),
    [sym_devchan_total_off_keyword] = ACTIONS(1589),
    [sym_devchan_pulse_keyword] = ACTIONS(1589),
    [sym_wait_keyword] = ACTIONS(1589),
    [sym_cancel_wait_keyword] = ACTIONS(1589),
    [sym_wait_until_keyword] = ACTIONS(1589),
    [sym_cancel_wait_until_keyword] = ACTIONS(1589),
    [sym_cancel_all_wait_keyword] = ACTIONS(1589),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1589),
    [sym_and] = ACTIONS(1589),
    [sym_or] = ACTIONS(1589),
    [sym_not] = ACTIONS(1589),
    [sym_xor] = ACTIONS(1589),
    [sym_band] = ACTIONS(1589),
    [sym_bor] = ACTIONS(1589),
    [sym_bxor] = ACTIONS(1589),
    [sym_bnot] = ACTIONS(1589),
    [sym_lshift] = ACTIONS(1589),
    [sym_rshift] = ACTIONS(1589),
    [sym___file__] = ACTIONS(1589),
    [sym___date__] = ACTIONS(1589),
    [sym___ldate__] = ACTIONS(1589),
    [sym___time__] = ACTIONS(1589),
    [sym___line__] = ACTIONS(1589),
    [sym___name__] = ACTIONS(1589),
    [sym___netlinx__] = ACTIONS(1589),
    [sym_netlinx_axi_version] = ACTIONS(1589),
    [sym_url_flg_tcp] = ACTIONS(1589),
    [sym_url_flg_accinfopresent] = ACTIONS(1589),
    [sym_url_flg_temp] = ACTIONS(1589),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1589),
    [sym_url_flg_stat_mask] = ACTIONS(1589),
    [sym_url_flg_stat_lookup] = ACTIONS(1589),
    [sym_url_flg_stat_connecting] = ACTIONS(1589),
    [sym_url_flg_stat_waiting] = ACTIONS(1589),
    [sym_url_flg_stat_connected] = ACTIONS(1589),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1589),
    [sym_first_virtual_device] = ACTIONS(1589),
    [sym_first_local_port] = ACTIONS(1589),
    [sym_dynamic_virtual_device] = ACTIONS(1589),
    [sym_timeline_once] = ACTIONS(1589),
    [sym_timeline_repeat] = ACTIONS(1589),
    [sym_timeline_absolute] = ACTIONS(1589),
    [sym_timeline_relative] = ACTIONS(1589),
    [sym_do_push_timed_infinite] = ACTIONS(1589),
    [sym_source_type_no_address] = ACTIONS(1589),
    [sym_source_type_neuron_id] = ACTIONS(1589),
    [sym_source_type_ip_address] = ACTIONS(1589),
    [sym_source_type_axlink] = ACTIONS(1589),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1589),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1589),
    [sym_source_type_ip_socket_address] = ACTIONS(1589),
    [sym_source_type_rs232] = ACTIONS(1589),
    [sym_source_type_internal] = ACTIONS(1589),
    [sym_source_type_mac_address] = ACTIONS(1589),
    [sym_source_type_ipv4_port] = ACTIONS(1589),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1589),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1589),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1589),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1589),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1589),
    [sym_true] = ACTIONS(1589),
    [sym_false] = ACTIONS(1589),
    [sym_file_read_only] = ACTIONS(1589),
    [sym_file_rw_new] = ACTIONS(1589),
    [sym_file_rw_append] = ACTIONS(1589),
    [sym_ip_tcp] = ACTIONS(1589),
    [sym_ip_udp] = ACTIONS(1589),
    [sym_ip_udp_2way] = ACTIONS(1589),
    [sym_xml_encode_types] = ACTIONS(1589),
    [sym_xml_encode_char_as_list] = ACTIONS(1589),
    [sym_xml_encode_le] = ACTIONS(1589),
    [sym_xml_decode_types] = ACTIONS(1589),
    [sym_xml_decode_no_preserve] = ACTIONS(1589),
    [sym_duet_dev_type_display_device] = ACTIONS(1589),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1589),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1589),
    [sym_duet_dev_type_io_device] = ACTIONS(1589),
    [sym_duet_dev_type_relay_device] = ACTIONS(1589),
    [sym_duet_dev_type_ups] = ACTIONS(1589),
    [sym_duet_dev_type_amplifier] = ACTIONS(1589),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1589),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1589),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1589),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1589),
    [sym_duet_dev_type_camera] = ACTIONS(1589),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1589),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1589),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1589),
    [sym_duet_dev_type_dss] = ACTIONS(1589),
    [sym_duet_dev_type_dvr] = ACTIONS(1589),
    [sym_duet_dev_type_disc_device] = ACTIONS(1589),
    [sym_duet_dev_type_document_camera] = ACTIONS(1589),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1589),
    [sym_duet_dev_type_hvac] = ACTIONS(1589),
    [sym_duet_dev_type_keypad] = ACTIONS(1589),
    [sym_duet_dev_type_light] = ACTIONS(1589),
    [sym_duet_dev_type_monitor] = ACTIONS(1589),
    [sym_duet_dev_type_motor] = ACTIONS(1589),
    [sym_duet_dev_type_multi_window] = ACTIONS(1589),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1589),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1589),
    [sym_duet_dev_type_receiver] = ACTIONS(1589),
    [sym_duet_dev_type_security_system] = ACTIONS(1589),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1589),
    [sym_duet_dev_type_settop_box] = ACTIONS(1589),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1589),
    [sym_duet_dev_type_switcher] = ACTIONS(1589),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1589),
    [sym_duet_dev_type_tv] = ACTIONS(1589),
    [sym_duet_dev_type_utility] = ACTIONS(1589),
    [sym_duet_dev_type_vcr] = ACTIONS(1589),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1589),
    [sym_duet_dev_type_video_processor] = ACTIONS(1589),
    [sym_duet_dev_type_video_projector] = ACTIONS(1589),
    [sym_duet_dev_type_video_wall] = ACTIONS(1589),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1589),
    [sym_duet_dev_type_weather] = ACTIONS(1589),
    [sym_duet_dev_not_polled] = ACTIONS(1589),
    [sym_duet_dev_polled] = ACTIONS(1589),
    [sym_clkmgr_mode_network] = ACTIONS(1589),
    [sym_clkmgr_mode_standalone] = ACTIONS(1589),
    [sym_normal_standby] = ACTIONS(1589),
    [sym_normal_wake] = ACTIONS(1589),
    [sym_ip_multicast_ttl_option] = ACTIONS(1589),
    [sym_ip_ttl_subnet] = ACTIONS(1589),
    [sym_ip_ttl_site] = ACTIONS(1589),
    [sym_ip_ttl_region] = ACTIONS(1589),
    [sym_ip_ttl_continent] = ACTIONS(1589),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1589),
    [sym_ip_nodelay_on] = ACTIONS(1589),
    [sym_ip_nodelay_off] = ACTIONS(1589),
    [sym_amx_error] = ACTIONS(1589),
    [sym_amx_warning] = ACTIONS(1589),
    [sym_amx_info] = ACTIONS(1589),
    [sym_amx_debug] = ACTIONS(1589),
    [sym_netlinx_login_success] = ACTIONS(1589),
    [sym_netlinx_login_fail] = ACTIONS(1589),
    [sym_netlinx_logout] = ACTIONS(1589),
    [sym_null_str] = ACTIONS(1589),
    [sym_smtp_address] = ACTIONS(1589),
    [sym_smtp_port_number] = ACTIONS(1589),
    [sym_smtp_username] = ACTIONS(1589),
    [sym_smtp_password] = ACTIONS(1589),
    [sym_smtp_require_tls] = ACTIONS(1589),
    [sym_smtp_from] = ACTIONS(1589),
    [sym_smtp_tls_true] = ACTIONS(1589),
    [sym_smtp_tls_false] = ACTIONS(1589),
    [sym_listview_on_row_select_event] = ACTIONS(1589),
    [sym_data_structure_unknown] = ACTIONS(1589),
    [sym_data_structure_datafeed] = ACTIONS(1589),
    [sym_data_structure_datarecordset] = ACTIONS(1589),
    [sym_data_structure_datarecord] = ACTIONS(1589),
    [sym_data_structure_datafield] = ACTIONS(1589),
    [sym_data_max_content_field_count] = ACTIONS(1589),
    [sym_data_max_metadata_field_count] = ACTIONS(1589),
    [sym_data_max_id_length] = ACTIONS(1589),
    [sym_data_max_name_length] = ACTIONS(1589),
    [sym_data_max_description_length] = ACTIONS(1589),
    [sym_data_max_type_length] = ACTIONS(1589),
    [sym_data_max_format_length] = ACTIONS(1589),
    [sym_data_max_source_length] = ACTIONS(1589),
    [sym_data_max_value_length] = ACTIONS(1589),
    [sym_data_max_label_length] = ACTIONS(1589),
    [sym_data_max_url_length] = ACTIONS(1589),
    [sym_max_last_login_info_length] = ACTIONS(1589),
    [sym_data_type_unknown] = ACTIONS(1589),
    [sym_data_type_string] = ACTIONS(1589),
    [sym_data_type_datetime] = ACTIONS(1589),
    [sym_data_type_date] = ACTIONS(1589),
    [sym_data_type_time] = ACTIONS(1589),
    [sym_data_type_image] = ACTIONS(1589),
    [sym_data_format_url] = ACTIONS(1589),
    [sym_data_format_phone] = ACTIONS(1589),
    [sym_data_format_email] = ACTIONS(1589),
    [sym_data_format_iso8601] = ACTIONS(1589),
    [sym_auditlog_priv] = ACTIONS(1589),
    [sym_configuration_priv] = ACTIONS(1589),
    [sym_deviceconfig_priv] = ACTIONS(1589),
    [sym_ftp_priv] = ACTIONS(1589),
    [sym_http_priv] = ACTIONS(1589),
    [sym_networkconfig_priv] = ACTIONS(1589),
    [sym_programport_priv] = ACTIONS(1589),
    [sym_remoteui_priv] = ACTIONS(1589),
    [sym_securitycontrol_priv] = ACTIONS(1589),
    [sym_softwaremanagement_priv] = ACTIONS(1589),
    [sym_terminal_priv] = ACTIONS(1589),
    [sym_tpadmin_priv] = ACTIONS(1589),
    [sym_user1_priv] = ACTIONS(1589),
    [sym_user2_priv] = ACTIONS(1589),
    [sym_user3_priv] = ACTIONS(1589),
    [sym_user4_priv] = ACTIONS(1589),
    [sym_usermanagement_priv] = ACTIONS(1589),
    [sym_valid_account] = ACTIONS(1589),
    [sym_err_name_invalid] = ACTIONS(1589),
    [sym_err_password_invalid] = ACTIONS(1589),
    [sym_err_invalid_account] = ACTIONS(1589),
    [sym_err_no_authorization] = ACTIONS(1589),
    [sym_err_invalid_parameter] = ACTIONS(1589),
    [sym_err_user_locked_out] = ACTIONS(1589),
    [sym_err_user_account_expired] = ACTIONS(1589),
    [sym_err_general_error] = ACTIONS(1589),
    [sym_err_authentication_error] = ACTIONS(1589),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1589),
    [sym_tls_validate_certificate] = ACTIONS(1589),
    [sym_tdata] = ACTIONS(1589),
    [sym_tchannel] = ACTIONS(1589),
    [sym_tlevel] = ACTIONS(1589),
    [sym_tbutton] = ACTIONS(1589),
    [sym_ttimeline] = ACTIONS(1589),
    [sym_tcustom] = ACTIONS(1589),
    [sym_url_struct] = ACTIONS(1589),
    [sym_dns_struct] = ACTIONS(1589),
    [sym_ip_address_struct] = ACTIONS(1589),
    [sym_dev_info_struct] = ACTIONS(1589),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1589),
    [sym_data_feed] = ACTIONS(1589),
    [sym_data_field] = ACTIONS(1589),
    [sym_data_record] = ACTIONS(1589),
    [sym_wc_data_feed] = ACTIONS(1589),
    [sym_wc_data_field] = ACTIONS(1589),
    [sym_wc_data_record] = ACTIONS(1589),
    [sym_last_login_info] = ACTIONS(1589),
    [sym____reserved___] = ACTIONS(1589),
    [sym_date] = ACTIONS(1589),
    [sym_day] = ACTIONS(1589),
    [sym_time] = ACTIONS(1589),
    [sym_ldate] = ACTIONS(1589),
    [sym_push_device] = ACTIONS(1589),
    [sym_push_channel] = ACTIONS(1589),
    [sym_push_devchan] = ACTIONS(1589),
    [sym_release_device] = ACTIONS(1589),
    [sym_release_channel] = ACTIONS(1589),
    [sym_release_devchan] = ACTIONS(1589),
    [sym_master_slot] = ACTIONS(1589),
    [sym_get_pulse_time] = ACTIONS(1589),
    [sym_get_timer] = ACTIONS(1589),
    [sym_master_sn] = ACTIONS(1589),
    [sym_system_number] = ACTIONS(1589),
    [sym_button] = ACTIONS(1589),
    [sym_dv_channel] = ACTIONS(1589),
    [sym_data] = ACTIONS(1589),
    [sym_level] = ACTIONS(1589),
    [sym_timeline] = ACTIONS(1589),
    [sym_channel] = ACTIONS(1589),
    [sym_custom] = ACTIONS(1589),
    [sym_atoi] = ACTIONS(1589),
    [sym_atol] = ACTIONS(1589),
    [sym_atof] = ACTIONS(1589),
    [sym_itoa] = ACTIONS(1589),
    [sym_itohex] = ACTIONS(1589),
    [sym_hextoi] = ACTIONS(1589),
    [sym_ftoa] = ACTIONS(1589),
    [sym_find_string] = ACTIONS(1589),
    [sym_left_string] = ACTIONS(1589),
    [sym_right_string] = ACTIONS(1589),
    [sym_mid_string] = ACTIONS(1589),
    [sym_remove_string] = ACTIONS(1589),
    [sym_get_buffer_string] = ACTIONS(1589),
    [sym_get_buffer_char] = ACTIONS(1589),
    [sym_get_multi_buffer_string] = ACTIONS(1589),
    [sym_length_string] = ACTIONS(1589),
    [sym_max_length_string] = ACTIONS(1589),
    [sym_set_length_string] = ACTIONS(1589),
    [sym_compare_string] = ACTIONS(1589),
    [sym_upper_string] = ACTIONS(1589),
    [sym_lower_string] = ACTIONS(1589),
    [sym_get_last] = ACTIONS(1589),
    [sym_date_to_day] = ACTIONS(1589),
    [sym_date_to_month] = ACTIONS(1589),
    [sym_date_to_year] = ACTIONS(1589),
    [sym_time_to_hour] = ACTIONS(1589),
    [sym_time_to_minute] = ACTIONS(1589),
    [sym_time_to_second] = ACTIONS(1589),
    [sym_day_of_week] = ACTIONS(1589),
    [sym_set_pulse_time] = ACTIONS(1589),
    [sym_set_timer] = ACTIONS(1589),
    [sym_file_open] = ACTIONS(1589),
    [sym_file_close] = ACTIONS(1589),
    [sym_file_read] = ACTIONS(1589),
    [sym_file_read_line] = ACTIONS(1589),
    [sym_file_write] = ACTIONS(1589),
    [sym_file_write_line] = ACTIONS(1589),
    [sym_file_copy] = ACTIONS(1589),
    [sym_file_delete] = ACTIONS(1589),
    [sym_file_rename] = ACTIONS(1589),
    [sym_file_dir] = ACTIONS(1589),
    [sym_file_getdir] = ACTIONS(1589),
    [sym_file_setdir] = ACTIONS(1589),
    [sym_file_create_dir] = ACTIONS(1589),
    [sym_file_remove_dir] = ACTIONS(1589),
    [sym_file_seek] = ACTIONS(1589),
    [sym_length_array] = ACTIONS(1589),
    [sym_max_length_array] = ACTIONS(1589),
    [sym_set_length_array] = ACTIONS(1589),
    [sym_ip_client_open] = ACTIONS(1589),
    [sym_ip_client_close] = ACTIONS(1589),
    [sym_ip_server_open] = ACTIONS(1589),
    [sym_ip_server_close] = ACTIONS(1589),
    [sym_ip_bound_client_open] = ACTIONS(1589),
    [sym_ip_mc_server_open] = ACTIONS(1589),
    [sym_ip_set_option] = ACTIONS(1589),
    [sym_ssh_client_open] = ACTIONS(1589),
    [sym_ssh_client_close] = ACTIONS(1589),
    [sym_tls_client_open] = ACTIONS(1589),
    [sym_tls_client_close] = ACTIONS(1589),
    [sym_do_push] = ACTIONS(1589),
    [sym_do_push_timed] = ACTIONS(1589),
    [sym_do_release] = ACTIONS(1589),
    [sym_device_id] = ACTIONS(1589),
    [sym_device_info] = ACTIONS(1589),
    [sym_device_id_string] = ACTIONS(1589),
    [sym_device_standby] = ACTIONS(1589),
    [sym_device_wake] = ACTIONS(1589),
    [sym_timeline_create] = ACTIONS(1589),
    [sym_timeline_kill] = ACTIONS(1589),
    [sym_timeline_reload] = ACTIONS(1589),
    [sym_timeline_pause] = ACTIONS(1589),
    [sym_timeline_restart] = ACTIONS(1589),
    [sym_timeline_set] = ACTIONS(1589),
    [sym_timeline_get] = ACTIONS(1589),
    [sym_timeline_active] = ACTIONS(1589),
    [sym_timeline_dynamic_id] = ACTIONS(1589),
    [sym_variable_to_string] = ACTIONS(1589),
    [sym_string_to_variable] = ACTIONS(1589),
    [sym_length_variable_to_string] = ACTIONS(1589),
    [sym_type_cast] = ACTIONS(1589),
    [sym_variable_to_xml] = ACTIONS(1589),
    [sym_xml_to_variable] = ACTIONS(1589),
    [sym_length_variable_to_xml] = ACTIONS(1589),
    [sym_redirect_string] = ACTIONS(1589),
    [sym_random_number] = ACTIONS(1589),
    [sym_get_unique_id] = ACTIONS(1589),
    [sym_get_system_number] = ACTIONS(1589),
    [sym_set_system_number] = ACTIONS(1589),
    [sym_get_serial_number] = ACTIONS(1589),
    [sym_get_master_build] = ACTIONS(1589),
    [sym_set_validation_code] = ACTIONS(1589),
    [sym_reboot] = ACTIONS(1589),
    [sym_load_duet_module] = ACTIONS(1589),
    [sym_unload_duet_module] = ACTIONS(1589),
    [sym_duet_mem_size_set] = ACTIONS(1589),
    [sym_duet_mem_size_get] = ACTIONS(1589),
    [sym_dynamic_application_device] = ACTIONS(1589),
    [sym_dynamic_polled_port] = ACTIONS(1589),
    [sym_static_port_binding] = ACTIONS(1589),
    [sym_static_ip_binding] = ACTIONS(1589),
    [sym_get_dns_list] = ACTIONS(1589),
    [sym_set_dns_list] = ACTIONS(1589),
    [sym_get_ip_address] = ACTIONS(1589),
    [sym_set_ip_address] = ACTIONS(1589),
    [sym_get_url_list] = ACTIONS(1589),
    [sym_add_url_entry] = ACTIONS(1589),
    [sym_delete_url_entry] = ACTIONS(1589),
    [sym_abs_value] = ACTIONS(1589),
    [sym_max_value] = ACTIONS(1589),
    [sym_min_value] = ACTIONS(1589),
    [sym_exp_value] = ACTIONS(1589),
    [sym_log_value] = ACTIONS(1589),
    [sym_log10_value] = ACTIONS(1589),
    [sym_power_value] = ACTIONS(1589),
    [sym_sqrt_value] = ACTIONS(1589),
    [sym_raw_be] = ACTIONS(1589),
    [sym_raw_le] = ACTIONS(1589),
    [sym_internal_queue_size_set] = ACTIONS(1589),
    [sym_internal_queue_size_get] = ACTIONS(1589),
    [sym_internal_threshold_set] = ACTIONS(1589),
    [sym_internal_threshold_get] = ACTIONS(1589),
    [sym_rebuilt_event] = ACTIONS(1589),
    [sym_do_custom_event] = ACTIONS(1589),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1589),
    [sym_clkmgr_set_clk_source] = ACTIONS(1589),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1589),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1589),
    [sym_clkmgr_get_timezone] = ACTIONS(1589),
    [sym_clkmgr_set_timezone] = ACTIONS(1589),
    [sym_clkmgr_get_resync_period] = ACTIONS(1589),
    [sym_clkmgr_set_resync_period] = ACTIONS(1589),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1589),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1589),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1589),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1589),
    [sym_clkmgr_get_timeservers] = ACTIONS(1589),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1589),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1589),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1589),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1589),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1589),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1589),
    [sym_set_log_level] = ACTIONS(1589),
    [sym_get_log_level] = ACTIONS(1589),
    [sym_amx_log] = ACTIONS(1589),
    [sym_smtp_server_config_set] = ACTIONS(1589),
    [sym_smtp_server_config_get] = ACTIONS(1589),
    [sym_smtp_send] = ACTIONS(1589),
    [sym_validate_netlinx_account] = ACTIONS(1589),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1589),
    [sym_authenticate_certificate] = ACTIONS(1589),
    [sym_audit_netlinx_session_event] = ACTIONS(1589),
    [sym_audit_netlinx_generic_event] = ACTIONS(1589),
    [sym_get_available_flash_disk_space] = ACTIONS(1589),
    [sym_get_max_flash_disk_space] = ACTIONS(1589),
    [sym_data_create_feed] = ACTIONS(1589),
    [sym_data_delete_feed] = ACTIONS(1589),
    [sym_data_publish_feed] = ACTIONS(1589),
    [sym_data_get_published_feed] = ACTIONS(1589),
    [sym_data_add_record] = ACTIONS(1589),
    [sym_data_get_event_record] = ACTIONS(1589),
    [sym__wc_data_create_feed] = ACTIONS(1589),
    [sym__wc_data_add_record] = ACTIONS(1589),
    [sym__wc_data_get_event_record] = ACTIONS(1589),
    [sym_format] = ACTIONS(1589),
    [sym_astro_clock] = ACTIONS(1589),
    [sym_set_outdoor_temperature] = ACTIONS(1589),
    [sym_set_virtual_level_count] = ACTIONS(1589),
    [sym_set_virtual_channel_count] = ACTIONS(1589),
    [sym_set_virtual_port_count] = ACTIONS(1589),
    [sym__wc_encode_int] = ACTIONS(1589),
    [sym__wc_decode_int] = ACTIONS(1589),
    [sym__wc_int] = ACTIONS(1589),
    [sym__wc_to_ch_int] = ACTIONS(1589),
    [sym__ch_to_wc_int] = ACTIONS(1589),
    [sym__wc_find_string_int] = ACTIONS(1589),
    [sym__wc_left_string_int] = ACTIONS(1589),
    [sym__wc_lower_string_int] = ACTIONS(1589),
    [sym__wc_mid_string_int] = ACTIONS(1589),
    [sym__wc_remove_string_int] = ACTIONS(1589),
    [sym__wc_right_string_int] = ACTIONS(1589),
    [sym__wc_upper_string_int] = ACTIONS(1589),
    [sym__wc_compare_string_int] = ACTIONS(1589),
    [sym__wc_get_buffer_char_int] = ACTIONS(1589),
    [sym__wc_get_buffer_string_int] = ACTIONS(1589),
    [sym__wc_concat_string_int] = ACTIONS(1589),
    [sym__wc_file_open_int] = ACTIONS(1589),
    [sym__wc_file_close_int] = ACTIONS(1589),
    [sym__wc_file_read_int] = ACTIONS(1589),
    [sym__wc_file_read_line_int] = ACTIONS(1589),
    [sym__wc_file_write_int] = ACTIONS(1589),
    [sym__wc_file_write_line_int] = ACTIONS(1589),
    [aux_sym_preproc_include_token1] = ACTIONS(1589),
    [aux_sym_preproc_define_token1] = ACTIONS(1591),
    [aux_sym_preproc_warn_token1] = ACTIONS(1591),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1591),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1591),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1591),
    [anon_sym_LBRACK] = ACTIONS(1591),
    [anon_sym_LBRACE] = ACTIONS(1591),
    [anon_sym_COLON] = ACTIONS(1591),
    [anon_sym_LPAREN] = ACTIONS(1589),
    [anon_sym_SEMI] = ACTIONS(1591),
    [anon_sym_BANG] = ACTIONS(1589),
    [anon_sym_TILDE] = ACTIONS(1591),
    [anon_sym_DASH] = ACTIONS(1589),
    [anon_sym_PLUS] = ACTIONS(1589),
    [anon_sym_STAR] = ACTIONS(1591),
    [anon_sym_SLASH] = ACTIONS(1589),
    [anon_sym_PERCENT] = ACTIONS(1591),
    [anon_sym_PIPE_PIPE] = ACTIONS(1591),
    [anon_sym_AMP_AMP] = ACTIONS(1591),
    [anon_sym_PIPE] = ACTIONS(1589),
    [anon_sym_CARET] = ACTIONS(1591),
    [anon_sym_AMP] = ACTIONS(1589),
    [anon_sym_EQ_EQ] = ACTIONS(1591),
    [anon_sym_BANG_EQ] = ACTIONS(1591),
    [anon_sym_LT_GT] = ACTIONS(1591),
    [anon_sym_GT] = ACTIONS(1589),
    [anon_sym_GT_EQ] = ACTIONS(1591),
    [anon_sym_LT_EQ] = ACTIONS(1591),
    [anon_sym_LT] = ACTIONS(1589),
    [anon_sym_LT_LT] = ACTIONS(1591),
    [anon_sym_GT_GT] = ACTIONS(1591),
    [anon_sym_DASH_DASH] = ACTIONS(1591),
    [anon_sym_PLUS_PLUS] = ACTIONS(1591),
    [anon_sym_DOT] = ACTIONS(1589),
    [anon_sym_DQUOTE] = ACTIONS(1591),
    [anon_sym_SQUOTE] = ACTIONS(1591),
    [sym_number_literal] = ACTIONS(1591),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1591),
  },
  [STATE(149)] = {
    [sym_expression] = STATE(1141),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(1242),
    [sym_define_call_keyword] = ACTIONS(1245),
    [sym_define_function_keyword] = ACTIONS(1245),
    [sym_define_library_function_keyword] = ACTIONS(1245),
    [sym_define_module_keyword] = ACTIONS(1245),
    [sym_char_keyword] = ACTIONS(1245),
    [sym_widechar_keyword] = ACTIONS(1245),
    [sym_integer_keyword] = ACTIONS(1245),
    [sym_sinteger_keyword] = ACTIONS(1245),
    [sym_long_keyword] = ACTIONS(1245),
    [sym_slong_keyword] = ACTIONS(1245),
    [sym_float_keyword] = ACTIONS(1245),
    [sym_double_keyword] = ACTIONS(1245),
    [sym_variant_keyword] = ACTIONS(1245),
    [sym_variantarray_keyword] = ACTIONS(1245),
    [sym_local_var_keyword] = ACTIONS(1245),
    [sym_stack_var_keyword] = ACTIONS(1245),
    [sym_constant_keyword] = ACTIONS(1245),
    [sym_volatile_keyword] = ACTIONS(1245),
    [sym_non_volatile_keyword] = ACTIONS(1245),
    [sym_persistent_keyword] = ACTIONS(1245),
    [sym_dev_keyword] = ACTIONS(1245),
    [sym_devlev_keyword] = ACTIONS(1245),
    [sym_devchan_keyword] = ACTIONS(1245),
    [sym_if_keyword] = ACTIONS(1245),
    [sym_else_keyword] = ACTIONS(1245),
    [sym_switch_keyword] = ACTIONS(1245),
    [sym_case_keyword] = ACTIONS(1245),
    [sym_default_keyword] = ACTIONS(1245),
    [sym_while_keyword] = ACTIONS(1245),
    [sym_for_keyword] = ACTIONS(1245),
    [sym_break_keyword] = ACTIONS(1245),
    [sym_continue_keyword] = ACTIONS(1245),
    [sym_return_keyword] = ACTIONS(1245),
    [sym_select_keyword] = ACTIONS(1245),
    [sym_active_keyword] = ACTIONS(1245),
    [sym_struct_keyword] = ACTIONS(1245),
    [sym_structure_keyword] = ACTIONS(1245),
    [aux_sym_on_keyword_token1] = ACTIONS(1245),
    [aux_sym_off_keyword_token1] = ACTIONS(1245),
    [sym_send_level_keyword] = ACTIONS(1245),
    [sym_send_string_keyword] = ACTIONS(1245),
    [sym_send_command_keyword] = ACTIONS(1245),
    [sym_clear_buffer_keyword] = ACTIONS(1245),
    [sym_create_buffer_keyword] = ACTIONS(1245),
    [sym_create_multi_buffer_keyword] = ACTIONS(1245),
    [sym_call_keyword] = ACTIONS(1245),
    [sym_system_call_keyword] = ACTIONS(1245),
    [sym_devchan_to_keyword] = ACTIONS(1245),
    [sym_devchan_min_to_keyword] = ACTIONS(1245),
    [sym_devchan_total_off_keyword] = ACTIONS(1245),
    [sym_devchan_pulse_keyword] = ACTIONS(1245),
    [sym_wait_keyword] = ACTIONS(1245),
    [sym_cancel_wait_keyword] = ACTIONS(1245),
    [sym_wait_until_keyword] = ACTIONS(1245),
    [sym_cancel_wait_until_keyword] = ACTIONS(1245),
    [sym_cancel_all_wait_keyword] = ACTIONS(1245),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1245),
    [sym_and] = ACTIONS(1245),
    [sym_or] = ACTIONS(1245),
    [sym_not] = ACTIONS(1247),
    [sym_xor] = ACTIONS(1245),
    [sym_band] = ACTIONS(1245),
    [sym_bor] = ACTIONS(1245),
    [sym_bxor] = ACTIONS(1245),
    [sym_bnot] = ACTIONS(1247),
    [sym_lshift] = ACTIONS(1245),
    [sym_rshift] = ACTIONS(1245),
    [sym___file__] = ACTIONS(1250),
    [sym___date__] = ACTIONS(1250),
    [sym___ldate__] = ACTIONS(1250),
    [sym___time__] = ACTIONS(1250),
    [sym___line__] = ACTIONS(1250),
    [sym___name__] = ACTIONS(1250),
    [sym___netlinx__] = ACTIONS(1253),
    [sym_netlinx_axi_version] = ACTIONS(1253),
    [sym_url_flg_tcp] = ACTIONS(1253),
    [sym_url_flg_accinfopresent] = ACTIONS(1253),
    [sym_url_flg_temp] = ACTIONS(1253),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1253),
    [sym_url_flg_stat_mask] = ACTIONS(1253),
    [sym_url_flg_stat_lookup] = ACTIONS(1253),
    [sym_url_flg_stat_connecting] = ACTIONS(1253),
    [sym_url_flg_stat_waiting] = ACTIONS(1253),
    [sym_url_flg_stat_connected] = ACTIONS(1253),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1253),
    [sym_first_virtual_device] = ACTIONS(1253),
    [sym_first_local_port] = ACTIONS(1253),
    [sym_dynamic_virtual_device] = ACTIONS(1253),
    [sym_timeline_once] = ACTIONS(1253),
    [sym_timeline_repeat] = ACTIONS(1253),
    [sym_timeline_absolute] = ACTIONS(1253),
    [sym_timeline_relative] = ACTIONS(1253),
    [sym_do_push_timed_infinite] = ACTIONS(1253),
    [sym_source_type_no_address] = ACTIONS(1253),
    [sym_source_type_neuron_id] = ACTIONS(1253),
    [sym_source_type_ip_address] = ACTIONS(1253),
    [sym_source_type_axlink] = ACTIONS(1253),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1253),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1253),
    [sym_source_type_ip_socket_address] = ACTIONS(1253),
    [sym_source_type_rs232] = ACTIONS(1253),
    [sym_source_type_internal] = ACTIONS(1253),
    [sym_source_type_mac_address] = ACTIONS(1253),
    [sym_source_type_ipv4_port] = ACTIONS(1253),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1253),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1253),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1253),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1253),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1253),
    [sym_true] = ACTIONS(1253),
    [sym_false] = ACTIONS(1253),
    [sym_file_read_only] = ACTIONS(1253),
    [sym_file_rw_new] = ACTIONS(1253),
    [sym_file_rw_append] = ACTIONS(1253),
    [sym_ip_tcp] = ACTIONS(1253),
    [sym_ip_udp] = ACTIONS(1253),
    [sym_ip_udp_2way] = ACTIONS(1253),
    [sym_xml_encode_types] = ACTIONS(1253),
    [sym_xml_encode_char_as_list] = ACTIONS(1253),
    [sym_xml_encode_le] = ACTIONS(1253),
    [sym_xml_decode_types] = ACTIONS(1253),
    [sym_xml_decode_no_preserve] = ACTIONS(1253),
    [sym_duet_dev_type_display_device] = ACTIONS(1253),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1253),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1253),
    [sym_duet_dev_type_io_device] = ACTIONS(1253),
    [sym_duet_dev_type_relay_device] = ACTIONS(1253),
    [sym_duet_dev_type_ups] = ACTIONS(1253),
    [sym_duet_dev_type_amplifier] = ACTIONS(1253),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1253),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1253),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1253),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1253),
    [sym_duet_dev_type_camera] = ACTIONS(1253),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1253),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1253),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1253),
    [sym_duet_dev_type_dss] = ACTIONS(1253),
    [sym_duet_dev_type_dvr] = ACTIONS(1253),
    [sym_duet_dev_type_disc_device] = ACTIONS(1253),
    [sym_duet_dev_type_document_camera] = ACTIONS(1253),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1253),
    [sym_duet_dev_type_hvac] = ACTIONS(1253),
    [sym_duet_dev_type_keypad] = ACTIONS(1253),
    [sym_duet_dev_type_light] = ACTIONS(1253),
    [sym_duet_dev_type_monitor] = ACTIONS(1253),
    [sym_duet_dev_type_motor] = ACTIONS(1253),
    [sym_duet_dev_type_multi_window] = ACTIONS(1253),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1253),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1253),
    [sym_duet_dev_type_receiver] = ACTIONS(1253),
    [sym_duet_dev_type_security_system] = ACTIONS(1253),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1253),
    [sym_duet_dev_type_settop_box] = ACTIONS(1253),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1253),
    [sym_duet_dev_type_switcher] = ACTIONS(1253),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1253),
    [sym_duet_dev_type_tv] = ACTIONS(1253),
    [sym_duet_dev_type_utility] = ACTIONS(1253),
    [sym_duet_dev_type_vcr] = ACTIONS(1253),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1253),
    [sym_duet_dev_type_video_processor] = ACTIONS(1253),
    [sym_duet_dev_type_video_projector] = ACTIONS(1253),
    [sym_duet_dev_type_video_wall] = ACTIONS(1253),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1253),
    [sym_duet_dev_type_weather] = ACTIONS(1253),
    [sym_duet_dev_not_polled] = ACTIONS(1253),
    [sym_duet_dev_polled] = ACTIONS(1253),
    [sym_clkmgr_mode_network] = ACTIONS(1253),
    [sym_clkmgr_mode_standalone] = ACTIONS(1253),
    [sym_normal_standby] = ACTIONS(1253),
    [sym_normal_wake] = ACTIONS(1253),
    [sym_ip_multicast_ttl_option] = ACTIONS(1253),
    [sym_ip_ttl_subnet] = ACTIONS(1253),
    [sym_ip_ttl_site] = ACTIONS(1253),
    [sym_ip_ttl_region] = ACTIONS(1253),
    [sym_ip_ttl_continent] = ACTIONS(1253),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1253),
    [sym_ip_nodelay_on] = ACTIONS(1253),
    [sym_ip_nodelay_off] = ACTIONS(1253),
    [sym_amx_error] = ACTIONS(1253),
    [sym_amx_warning] = ACTIONS(1253),
    [sym_amx_info] = ACTIONS(1253),
    [sym_amx_debug] = ACTIONS(1253),
    [sym_netlinx_login_success] = ACTIONS(1253),
    [sym_netlinx_login_fail] = ACTIONS(1253),
    [sym_netlinx_logout] = ACTIONS(1253),
    [sym_null_str] = ACTIONS(1253),
    [sym_smtp_address] = ACTIONS(1253),
    [sym_smtp_port_number] = ACTIONS(1253),
    [sym_smtp_username] = ACTIONS(1253),
    [sym_smtp_password] = ACTIONS(1253),
    [sym_smtp_require_tls] = ACTIONS(1253),
    [sym_smtp_from] = ACTIONS(1253),
    [sym_smtp_tls_true] = ACTIONS(1253),
    [sym_smtp_tls_false] = ACTIONS(1253),
    [sym_listview_on_row_select_event] = ACTIONS(1253),
    [sym_data_structure_unknown] = ACTIONS(1253),
    [sym_data_structure_datafeed] = ACTIONS(1253),
    [sym_data_structure_datarecordset] = ACTIONS(1253),
    [sym_data_structure_datarecord] = ACTIONS(1253),
    [sym_data_structure_datafield] = ACTIONS(1253),
    [sym_data_max_content_field_count] = ACTIONS(1253),
    [sym_data_max_metadata_field_count] = ACTIONS(1253),
    [sym_data_max_id_length] = ACTIONS(1253),
    [sym_data_max_name_length] = ACTIONS(1253),
    [sym_data_max_description_length] = ACTIONS(1253),
    [sym_data_max_type_length] = ACTIONS(1253),
    [sym_data_max_format_length] = ACTIONS(1253),
    [sym_data_max_source_length] = ACTIONS(1253),
    [sym_data_max_value_length] = ACTIONS(1253),
    [sym_data_max_label_length] = ACTIONS(1253),
    [sym_data_max_url_length] = ACTIONS(1253),
    [sym_max_last_login_info_length] = ACTIONS(1253),
    [sym_data_type_unknown] = ACTIONS(1253),
    [sym_data_type_string] = ACTIONS(1253),
    [sym_data_type_datetime] = ACTIONS(1253),
    [sym_data_type_date] = ACTIONS(1253),
    [sym_data_type_time] = ACTIONS(1253),
    [sym_data_type_image] = ACTIONS(1253),
    [sym_data_format_url] = ACTIONS(1253),
    [sym_data_format_phone] = ACTIONS(1253),
    [sym_data_format_email] = ACTIONS(1253),
    [sym_data_format_iso8601] = ACTIONS(1253),
    [sym_auditlog_priv] = ACTIONS(1253),
    [sym_configuration_priv] = ACTIONS(1253),
    [sym_deviceconfig_priv] = ACTIONS(1253),
    [sym_ftp_priv] = ACTIONS(1253),
    [sym_http_priv] = ACTIONS(1253),
    [sym_networkconfig_priv] = ACTIONS(1253),
    [sym_programport_priv] = ACTIONS(1253),
    [sym_remoteui_priv] = ACTIONS(1253),
    [sym_securitycontrol_priv] = ACTIONS(1253),
    [sym_softwaremanagement_priv] = ACTIONS(1253),
    [sym_terminal_priv] = ACTIONS(1253),
    [sym_tpadmin_priv] = ACTIONS(1253),
    [sym_user1_priv] = ACTIONS(1253),
    [sym_user2_priv] = ACTIONS(1253),
    [sym_user3_priv] = ACTIONS(1253),
    [sym_user4_priv] = ACTIONS(1253),
    [sym_usermanagement_priv] = ACTIONS(1253),
    [sym_valid_account] = ACTIONS(1253),
    [sym_err_name_invalid] = ACTIONS(1253),
    [sym_err_password_invalid] = ACTIONS(1253),
    [sym_err_invalid_account] = ACTIONS(1253),
    [sym_err_no_authorization] = ACTIONS(1253),
    [sym_err_invalid_parameter] = ACTIONS(1253),
    [sym_err_user_locked_out] = ACTIONS(1253),
    [sym_err_user_account_expired] = ACTIONS(1253),
    [sym_err_general_error] = ACTIONS(1253),
    [sym_err_authentication_error] = ACTIONS(1253),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1253),
    [sym_tls_validate_certificate] = ACTIONS(1253),
    [sym_tdata] = ACTIONS(1245),
    [sym_tchannel] = ACTIONS(1245),
    [sym_tlevel] = ACTIONS(1245),
    [sym_tbutton] = ACTIONS(1245),
    [sym_ttimeline] = ACTIONS(1245),
    [sym_tcustom] = ACTIONS(1245),
    [sym_url_struct] = ACTIONS(1245),
    [sym_dns_struct] = ACTIONS(1245),
    [sym_ip_address_struct] = ACTIONS(1245),
    [sym_dev_info_struct] = ACTIONS(1245),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1245),
    [sym_data_feed] = ACTIONS(1245),
    [sym_data_field] = ACTIONS(1245),
    [sym_data_record] = ACTIONS(1245),
    [sym_wc_data_feed] = ACTIONS(1245),
    [sym_wc_data_field] = ACTIONS(1245),
    [sym_wc_data_record] = ACTIONS(1245),
    [sym_last_login_info] = ACTIONS(1245),
    [sym____reserved___] = ACTIONS(1256),
    [sym_date] = ACTIONS(1256),
    [sym_day] = ACTIONS(1256),
    [sym_time] = ACTIONS(1256),
    [sym_ldate] = ACTIONS(1256),
    [sym_push_device] = ACTIONS(1256),
    [sym_push_channel] = ACTIONS(1256),
    [sym_push_devchan] = ACTIONS(1256),
    [sym_release_device] = ACTIONS(1256),
    [sym_release_channel] = ACTIONS(1256),
    [sym_release_devchan] = ACTIONS(1256),
    [sym_master_slot] = ACTIONS(1256),
    [sym_get_pulse_time] = ACTIONS(1256),
    [sym_get_timer] = ACTIONS(1256),
    [sym_master_sn] = ACTIONS(1256),
    [sym_system_number] = ACTIONS(1256),
    [sym_button] = ACTIONS(1256),
    [sym_dv_channel] = ACTIONS(1256),
    [sym_data] = ACTIONS(1256),
    [sym_level] = ACTIONS(1256),
    [sym_timeline] = ACTIONS(1256),
    [sym_channel] = ACTIONS(1256),
    [sym_custom] = ACTIONS(1256),
    [sym_atoi] = ACTIONS(1259),
    [sym_atol] = ACTIONS(1259),
    [sym_atof] = ACTIONS(1259),
    [sym_itoa] = ACTIONS(1259),
    [sym_itohex] = ACTIONS(1259),
    [sym_hextoi] = ACTIONS(1259),
    [sym_ftoa] = ACTIONS(1259),
    [sym_find_string] = ACTIONS(1259),
    [sym_left_string] = ACTIONS(1259),
    [sym_right_string] = ACTIONS(1259),
    [sym_mid_string] = ACTIONS(1259),
    [sym_remove_string] = ACTIONS(1259),
    [sym_get_buffer_string] = ACTIONS(1259),
    [sym_get_buffer_char] = ACTIONS(1259),
    [sym_get_multi_buffer_string] = ACTIONS(1259),
    [sym_length_string] = ACTIONS(1259),
    [sym_max_length_string] = ACTIONS(1259),
    [sym_set_length_string] = ACTIONS(1259),
    [sym_compare_string] = ACTIONS(1259),
    [sym_upper_string] = ACTIONS(1259),
    [sym_lower_string] = ACTIONS(1259),
    [sym_get_last] = ACTIONS(1259),
    [sym_date_to_day] = ACTIONS(1259),
    [sym_date_to_month] = ACTIONS(1259),
    [sym_date_to_year] = ACTIONS(1259),
    [sym_time_to_hour] = ACTIONS(1259),
    [sym_time_to_minute] = ACTIONS(1259),
    [sym_time_to_second] = ACTIONS(1259),
    [sym_day_of_week] = ACTIONS(1259),
    [sym_set_pulse_time] = ACTIONS(1259),
    [sym_set_timer] = ACTIONS(1259),
    [sym_file_open] = ACTIONS(1259),
    [sym_file_close] = ACTIONS(1259),
    [sym_file_read] = ACTIONS(1259),
    [sym_file_read_line] = ACTIONS(1259),
    [sym_file_write] = ACTIONS(1259),
    [sym_file_write_line] = ACTIONS(1259),
    [sym_file_copy] = ACTIONS(1259),
    [sym_file_delete] = ACTIONS(1259),
    [sym_file_rename] = ACTIONS(1259),
    [sym_file_dir] = ACTIONS(1259),
    [sym_file_getdir] = ACTIONS(1259),
    [sym_file_setdir] = ACTIONS(1259),
    [sym_file_create_dir] = ACTIONS(1259),
    [sym_file_remove_dir] = ACTIONS(1259),
    [sym_file_seek] = ACTIONS(1259),
    [sym_length_array] = ACTIONS(1259),
    [sym_max_length_array] = ACTIONS(1259),
    [sym_set_length_array] = ACTIONS(1259),
    [sym_ip_client_open] = ACTIONS(1259),
    [sym_ip_client_close] = ACTIONS(1259),
    [sym_ip_server_open] = ACTIONS(1259),
    [sym_ip_server_close] = ACTIONS(1259),
    [sym_ip_bound_client_open] = ACTIONS(1259),
    [sym_ip_mc_server_open] = ACTIONS(1259),
    [sym_ip_set_option] = ACTIONS(1259),
    [sym_ssh_client_open] = ACTIONS(1259),
    [sym_ssh_client_close] = ACTIONS(1259),
    [sym_tls_client_open] = ACTIONS(1259),
    [sym_tls_client_close] = ACTIONS(1259),
    [sym_do_push] = ACTIONS(1259),
    [sym_do_push_timed] = ACTIONS(1259),
    [sym_do_release] = ACTIONS(1259),
    [sym_device_id] = ACTIONS(1259),
    [sym_device_info] = ACTIONS(1259),
    [sym_device_id_string] = ACTIONS(1259),
    [sym_device_standby] = ACTIONS(1259),
    [sym_device_wake] = ACTIONS(1259),
    [sym_timeline_create] = ACTIONS(1259),
    [sym_timeline_kill] = ACTIONS(1259),
    [sym_timeline_reload] = ACTIONS(1259),
    [sym_timeline_pause] = ACTIONS(1259),
    [sym_timeline_restart] = ACTIONS(1259),
    [sym_timeline_set] = ACTIONS(1259),
    [sym_timeline_get] = ACTIONS(1259),
    [sym_timeline_active] = ACTIONS(1259),
    [sym_timeline_dynamic_id] = ACTIONS(1259),
    [sym_variable_to_string] = ACTIONS(1259),
    [sym_string_to_variable] = ACTIONS(1259),
    [sym_length_variable_to_string] = ACTIONS(1259),
    [sym_type_cast] = ACTIONS(1259),
    [sym_variable_to_xml] = ACTIONS(1259),
    [sym_xml_to_variable] = ACTIONS(1259),
    [sym_length_variable_to_xml] = ACTIONS(1259),
    [sym_redirect_string] = ACTIONS(1259),
    [sym_random_number] = ACTIONS(1259),
    [sym_get_unique_id] = ACTIONS(1259),
    [sym_get_system_number] = ACTIONS(1259),
    [sym_set_system_number] = ACTIONS(1259),
    [sym_get_serial_number] = ACTIONS(1259),
    [sym_get_master_build] = ACTIONS(1259),
    [sym_set_validation_code] = ACTIONS(1259),
    [sym_reboot] = ACTIONS(1259),
    [sym_load_duet_module] = ACTIONS(1259),
    [sym_unload_duet_module] = ACTIONS(1259),
    [sym_duet_mem_size_set] = ACTIONS(1259),
    [sym_duet_mem_size_get] = ACTIONS(1259),
    [sym_dynamic_application_device] = ACTIONS(1259),
    [sym_dynamic_polled_port] = ACTIONS(1259),
    [sym_static_port_binding] = ACTIONS(1259),
    [sym_static_ip_binding] = ACTIONS(1259),
    [sym_get_dns_list] = ACTIONS(1259),
    [sym_set_dns_list] = ACTIONS(1259),
    [sym_get_ip_address] = ACTIONS(1259),
    [sym_set_ip_address] = ACTIONS(1259),
    [sym_get_url_list] = ACTIONS(1259),
    [sym_add_url_entry] = ACTIONS(1259),
    [sym_delete_url_entry] = ACTIONS(1259),
    [sym_abs_value] = ACTIONS(1259),
    [sym_max_value] = ACTIONS(1259),
    [sym_min_value] = ACTIONS(1259),
    [sym_exp_value] = ACTIONS(1259),
    [sym_log_value] = ACTIONS(1259),
    [sym_log10_value] = ACTIONS(1259),
    [sym_power_value] = ACTIONS(1259),
    [sym_sqrt_value] = ACTIONS(1259),
    [sym_raw_be] = ACTIONS(1259),
    [sym_raw_le] = ACTIONS(1259),
    [sym_internal_queue_size_set] = ACTIONS(1259),
    [sym_internal_queue_size_get] = ACTIONS(1259),
    [sym_internal_threshold_set] = ACTIONS(1259),
    [sym_internal_threshold_get] = ACTIONS(1259),
    [sym_rebuilt_event] = ACTIONS(1259),
    [sym_do_custom_event] = ACTIONS(1259),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1259),
    [sym_clkmgr_set_clk_source] = ACTIONS(1259),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1259),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1259),
    [sym_clkmgr_get_timezone] = ACTIONS(1259),
    [sym_clkmgr_set_timezone] = ACTIONS(1259),
    [sym_clkmgr_get_resync_period] = ACTIONS(1259),
    [sym_clkmgr_set_resync_period] = ACTIONS(1259),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1259),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1259),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1259),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1259),
    [sym_clkmgr_get_timeservers] = ACTIONS(1259),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1259),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1259),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1259),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1259),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1259),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1259),
    [sym_set_log_level] = ACTIONS(1259),
    [sym_get_log_level] = ACTIONS(1259),
    [sym_amx_log] = ACTIONS(1259),
    [sym_smtp_server_config_set] = ACTIONS(1259),
    [sym_smtp_server_config_get] = ACTIONS(1259),
    [sym_smtp_send] = ACTIONS(1259),
    [sym_validate_netlinx_account] = ACTIONS(1259),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1259),
    [sym_authenticate_certificate] = ACTIONS(1259),
    [sym_audit_netlinx_session_event] = ACTIONS(1259),
    [sym_audit_netlinx_generic_event] = ACTIONS(1259),
    [sym_get_available_flash_disk_space] = ACTIONS(1259),
    [sym_get_max_flash_disk_space] = ACTIONS(1259),
    [sym_data_create_feed] = ACTIONS(1259),
    [sym_data_delete_feed] = ACTIONS(1259),
    [sym_data_publish_feed] = ACTIONS(1259),
    [sym_data_get_published_feed] = ACTIONS(1259),
    [sym_data_add_record] = ACTIONS(1259),
    [sym_data_get_event_record] = ACTIONS(1259),
    [sym__wc_data_create_feed] = ACTIONS(1259),
    [sym__wc_data_add_record] = ACTIONS(1259),
    [sym__wc_data_get_event_record] = ACTIONS(1259),
    [sym_format] = ACTIONS(1259),
    [sym_astro_clock] = ACTIONS(1259),
    [sym_set_outdoor_temperature] = ACTIONS(1259),
    [sym_set_virtual_level_count] = ACTIONS(1259),
    [sym_set_virtual_channel_count] = ACTIONS(1259),
    [sym_set_virtual_port_count] = ACTIONS(1259),
    [sym__wc_encode_int] = ACTIONS(1259),
    [sym__wc_decode_int] = ACTIONS(1259),
    [sym__wc_int] = ACTIONS(1259),
    [sym__wc_to_ch_int] = ACTIONS(1259),
    [sym__ch_to_wc_int] = ACTIONS(1259),
    [sym__wc_find_string_int] = ACTIONS(1259),
    [sym__wc_left_string_int] = ACTIONS(1259),
    [sym__wc_lower_string_int] = ACTIONS(1259),
    [sym__wc_mid_string_int] = ACTIONS(1259),
    [sym__wc_remove_string_int] = ACTIONS(1259),
    [sym__wc_right_string_int] = ACTIONS(1259),
    [sym__wc_upper_string_int] = ACTIONS(1259),
    [sym__wc_compare_string_int] = ACTIONS(1259),
    [sym__wc_get_buffer_char_int] = ACTIONS(1259),
    [sym__wc_get_buffer_string_int] = ACTIONS(1259),
    [sym__wc_concat_string_int] = ACTIONS(1259),
    [sym__wc_file_open_int] = ACTIONS(1259),
    [sym__wc_file_close_int] = ACTIONS(1259),
    [sym__wc_file_read_int] = ACTIONS(1259),
    [sym__wc_file_read_line_int] = ACTIONS(1259),
    [sym__wc_file_write_int] = ACTIONS(1259),
    [sym__wc_file_write_line_int] = ACTIONS(1259),
    [aux_sym_preproc_include_token1] = ACTIONS(1245),
    [aux_sym_preproc_define_token1] = ACTIONS(1262),
    [aux_sym_preproc_warn_token1] = ACTIONS(1262),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1262),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1262),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1262),
    [anon_sym_LBRACK] = ACTIONS(1264),
    [anon_sym_LBRACE] = ACTIONS(1262),
    [anon_sym_RBRACE] = ACTIONS(1262),
    [anon_sym_COLON] = ACTIONS(1262),
    [anon_sym_LPAREN] = ACTIONS(1267),
    [anon_sym_SEMI] = ACTIONS(1262),
    [anon_sym_BANG] = ACTIONS(1247),
    [anon_sym_TILDE] = ACTIONS(1270),
    [anon_sym_DASH] = ACTIONS(1247),
    [anon_sym_PLUS] = ACTIONS(1247),
    [anon_sym_STAR] = ACTIONS(1262),
    [anon_sym_SLASH] = ACTIONS(1245),
    [anon_sym_PERCENT] = ACTIONS(1262),
    [anon_sym_PIPE_PIPE] = ACTIONS(1262),
    [anon_sym_AMP_AMP] = ACTIONS(1262),
    [anon_sym_PIPE] = ACTIONS(1245),
    [anon_sym_CARET] = ACTIONS(1262),
    [anon_sym_AMP] = ACTIONS(1245),
    [anon_sym_EQ_EQ] = ACTIONS(1262),
    [anon_sym_BANG_EQ] = ACTIONS(1262),
    [anon_sym_LT_GT] = ACTIONS(1262),
    [anon_sym_GT] = ACTIONS(1245),
    [anon_sym_GT_EQ] = ACTIONS(1262),
    [anon_sym_LT_EQ] = ACTIONS(1262),
    [anon_sym_LT] = ACTIONS(1245),
    [anon_sym_LT_LT] = ACTIONS(1262),
    [anon_sym_GT_GT] = ACTIONS(1262),
    [anon_sym_DASH_DASH] = ACTIONS(1273),
    [anon_sym_PLUS_PLUS] = ACTIONS(1273),
    [anon_sym_DOT] = ACTIONS(1245),
    [anon_sym_DQUOTE] = ACTIONS(1276),
    [anon_sym_SQUOTE] = ACTIONS(1279),
    [sym_number_literal] = ACTIONS(1282),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1262),
  },
  [STATE(150)] = {
    [sym_argument_list] = STATE(171),
    [sym_string_literal] = STATE(460),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1407),
    [sym_define_call_keyword] = ACTIONS(1407),
    [sym_define_function_keyword] = ACTIONS(1407),
    [sym_define_library_function_keyword] = ACTIONS(1407),
    [sym_define_module_keyword] = ACTIONS(1407),
    [sym_char_keyword] = ACTIONS(1407),
    [sym_widechar_keyword] = ACTIONS(1407),
    [sym_integer_keyword] = ACTIONS(1407),
    [sym_sinteger_keyword] = ACTIONS(1407),
    [sym_long_keyword] = ACTIONS(1407),
    [sym_slong_keyword] = ACTIONS(1407),
    [sym_float_keyword] = ACTIONS(1407),
    [sym_double_keyword] = ACTIONS(1407),
    [sym_variant_keyword] = ACTIONS(1407),
    [sym_variantarray_keyword] = ACTIONS(1407),
    [sym_local_var_keyword] = ACTIONS(1407),
    [sym_stack_var_keyword] = ACTIONS(1407),
    [sym_constant_keyword] = ACTIONS(1407),
    [sym_volatile_keyword] = ACTIONS(1407),
    [sym_non_volatile_keyword] = ACTIONS(1407),
    [sym_persistent_keyword] = ACTIONS(1407),
    [sym_dev_keyword] = ACTIONS(1407),
    [sym_devlev_keyword] = ACTIONS(1407),
    [sym_devchan_keyword] = ACTIONS(1407),
    [sym_if_keyword] = ACTIONS(1407),
    [sym_else_keyword] = ACTIONS(1407),
    [sym_switch_keyword] = ACTIONS(1407),
    [sym_case_keyword] = ACTIONS(1407),
    [sym_default_keyword] = ACTIONS(1407),
    [sym_while_keyword] = ACTIONS(1407),
    [sym_for_keyword] = ACTIONS(1407),
    [sym_break_keyword] = ACTIONS(1407),
    [sym_continue_keyword] = ACTIONS(1407),
    [sym_return_keyword] = ACTIONS(1407),
    [sym_select_keyword] = ACTIONS(1407),
    [sym_active_keyword] = ACTIONS(1407),
    [sym_struct_keyword] = ACTIONS(1407),
    [sym_structure_keyword] = ACTIONS(1407),
    [aux_sym_on_keyword_token1] = ACTIONS(1407),
    [aux_sym_off_keyword_token1] = ACTIONS(1407),
    [sym_send_level_keyword] = ACTIONS(1407),
    [sym_send_string_keyword] = ACTIONS(1407),
    [sym_send_command_keyword] = ACTIONS(1407),
    [sym_clear_buffer_keyword] = ACTIONS(1407),
    [sym_create_buffer_keyword] = ACTIONS(1407),
    [sym_create_multi_buffer_keyword] = ACTIONS(1407),
    [sym_call_keyword] = ACTIONS(1407),
    [sym_system_call_keyword] = ACTIONS(1407),
    [sym_devchan_to_keyword] = ACTIONS(1407),
    [sym_devchan_min_to_keyword] = ACTIONS(1407),
    [sym_devchan_total_off_keyword] = ACTIONS(1407),
    [sym_devchan_pulse_keyword] = ACTIONS(1407),
    [sym_wait_keyword] = ACTIONS(1407),
    [sym_cancel_wait_keyword] = ACTIONS(1407),
    [sym_wait_until_keyword] = ACTIONS(1407),
    [sym_cancel_wait_until_keyword] = ACTIONS(1407),
    [sym_cancel_all_wait_keyword] = ACTIONS(1407),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1407),
    [sym_and] = ACTIONS(1605),
    [sym_or] = ACTIONS(1607),
    [sym_not] = ACTIONS(1407),
    [sym_xor] = ACTIONS(1609),
    [sym_band] = ACTIONS(1611),
    [sym_bor] = ACTIONS(1613),
    [sym_bxor] = ACTIONS(1609),
    [sym_bnot] = ACTIONS(1407),
    [sym_lshift] = ACTIONS(1615),
    [sym_rshift] = ACTIONS(1615),
    [sym___file__] = ACTIONS(1407),
    [sym___date__] = ACTIONS(1407),
    [sym___ldate__] = ACTIONS(1407),
    [sym___time__] = ACTIONS(1407),
    [sym___line__] = ACTIONS(1407),
    [sym___name__] = ACTIONS(1407),
    [sym___netlinx__] = ACTIONS(1407),
    [sym_netlinx_axi_version] = ACTIONS(1407),
    [sym_url_flg_tcp] = ACTIONS(1407),
    [sym_url_flg_accinfopresent] = ACTIONS(1407),
    [sym_url_flg_temp] = ACTIONS(1407),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1407),
    [sym_url_flg_stat_mask] = ACTIONS(1407),
    [sym_url_flg_stat_lookup] = ACTIONS(1407),
    [sym_url_flg_stat_connecting] = ACTIONS(1407),
    [sym_url_flg_stat_waiting] = ACTIONS(1407),
    [sym_url_flg_stat_connected] = ACTIONS(1407),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1407),
    [sym_first_virtual_device] = ACTIONS(1407),
    [sym_first_local_port] = ACTIONS(1407),
    [sym_dynamic_virtual_device] = ACTIONS(1407),
    [sym_timeline_once] = ACTIONS(1407),
    [sym_timeline_repeat] = ACTIONS(1407),
    [sym_timeline_absolute] = ACTIONS(1407),
    [sym_timeline_relative] = ACTIONS(1407),
    [sym_do_push_timed_infinite] = ACTIONS(1407),
    [sym_source_type_no_address] = ACTIONS(1407),
    [sym_source_type_neuron_id] = ACTIONS(1407),
    [sym_source_type_ip_address] = ACTIONS(1407),
    [sym_source_type_axlink] = ACTIONS(1407),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1407),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1407),
    [sym_source_type_ip_socket_address] = ACTIONS(1407),
    [sym_source_type_rs232] = ACTIONS(1407),
    [sym_source_type_internal] = ACTIONS(1407),
    [sym_source_type_mac_address] = ACTIONS(1407),
    [sym_source_type_ipv4_port] = ACTIONS(1407),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1407),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1407),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1407),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1407),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1407),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1407),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1407),
    [sym_true] = ACTIONS(1407),
    [sym_false] = ACTIONS(1407),
    [sym_file_read_only] = ACTIONS(1407),
    [sym_file_rw_new] = ACTIONS(1407),
    [sym_file_rw_append] = ACTIONS(1407),
    [sym_ip_tcp] = ACTIONS(1407),
    [sym_ip_udp] = ACTIONS(1407),
    [sym_ip_udp_2way] = ACTIONS(1407),
    [sym_xml_encode_types] = ACTIONS(1407),
    [sym_xml_encode_char_as_list] = ACTIONS(1407),
    [sym_xml_encode_le] = ACTIONS(1407),
    [sym_xml_decode_types] = ACTIONS(1407),
    [sym_xml_decode_no_preserve] = ACTIONS(1407),
    [sym_duet_dev_type_display_device] = ACTIONS(1407),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1407),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1407),
    [sym_duet_dev_type_io_device] = ACTIONS(1407),
    [sym_duet_dev_type_relay_device] = ACTIONS(1407),
    [sym_duet_dev_type_ups] = ACTIONS(1407),
    [sym_duet_dev_type_amplifier] = ACTIONS(1407),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1407),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1407),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1407),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1407),
    [sym_duet_dev_type_camera] = ACTIONS(1407),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1407),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1407),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1407),
    [sym_duet_dev_type_dss] = ACTIONS(1407),
    [sym_duet_dev_type_dvr] = ACTIONS(1407),
    [sym_duet_dev_type_disc_device] = ACTIONS(1407),
    [sym_duet_dev_type_document_camera] = ACTIONS(1407),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1407),
    [sym_duet_dev_type_hvac] = ACTIONS(1407),
    [sym_duet_dev_type_keypad] = ACTIONS(1407),
    [sym_duet_dev_type_light] = ACTIONS(1407),
    [sym_duet_dev_type_monitor] = ACTIONS(1407),
    [sym_duet_dev_type_motor] = ACTIONS(1407),
    [sym_duet_dev_type_multi_window] = ACTIONS(1407),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1407),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1407),
    [sym_duet_dev_type_receiver] = ACTIONS(1407),
    [sym_duet_dev_type_security_system] = ACTIONS(1407),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1407),
    [sym_duet_dev_type_settop_box] = ACTIONS(1407),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1407),
    [sym_duet_dev_type_switcher] = ACTIONS(1407),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1407),
    [sym_duet_dev_type_tv] = ACTIONS(1407),
    [sym_duet_dev_type_utility] = ACTIONS(1407),
    [sym_duet_dev_type_vcr] = ACTIONS(1407),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1407),
    [sym_duet_dev_type_video_processor] = ACTIONS(1407),
    [sym_duet_dev_type_video_projector] = ACTIONS(1407),
    [sym_duet_dev_type_video_wall] = ACTIONS(1407),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1407),
    [sym_duet_dev_type_weather] = ACTIONS(1407),
    [sym_duet_dev_not_polled] = ACTIONS(1407),
    [sym_duet_dev_polled] = ACTIONS(1407),
    [sym_clkmgr_mode_network] = ACTIONS(1407),
    [sym_clkmgr_mode_standalone] = ACTIONS(1407),
    [sym_normal_standby] = ACTIONS(1407),
    [sym_normal_wake] = ACTIONS(1407),
    [sym_ip_multicast_ttl_option] = ACTIONS(1407),
    [sym_ip_ttl_subnet] = ACTIONS(1407),
    [sym_ip_ttl_site] = ACTIONS(1407),
    [sym_ip_ttl_region] = ACTIONS(1407),
    [sym_ip_ttl_continent] = ACTIONS(1407),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1407),
    [sym_ip_nodelay_on] = ACTIONS(1407),
    [sym_ip_nodelay_off] = ACTIONS(1407),
    [sym_amx_error] = ACTIONS(1407),
    [sym_amx_warning] = ACTIONS(1407),
    [sym_amx_info] = ACTIONS(1407),
    [sym_amx_debug] = ACTIONS(1407),
    [sym_netlinx_login_success] = ACTIONS(1407),
    [sym_netlinx_login_fail] = ACTIONS(1407),
    [sym_netlinx_logout] = ACTIONS(1407),
    [sym_null_str] = ACTIONS(1407),
    [sym_smtp_address] = ACTIONS(1407),
    [sym_smtp_port_number] = ACTIONS(1407),
    [sym_smtp_username] = ACTIONS(1407),
    [sym_smtp_password] = ACTIONS(1407),
    [sym_smtp_require_tls] = ACTIONS(1407),
    [sym_smtp_from] = ACTIONS(1407),
    [sym_smtp_tls_true] = ACTIONS(1407),
    [sym_smtp_tls_false] = ACTIONS(1407),
    [sym_listview_on_row_select_event] = ACTIONS(1407),
    [sym_data_structure_unknown] = ACTIONS(1407),
    [sym_data_structure_datafeed] = ACTIONS(1407),
    [sym_data_structure_datarecordset] = ACTIONS(1407),
    [sym_data_structure_datarecord] = ACTIONS(1407),
    [sym_data_structure_datafield] = ACTIONS(1407),
    [sym_data_max_content_field_count] = ACTIONS(1407),
    [sym_data_max_metadata_field_count] = ACTIONS(1407),
    [sym_data_max_id_length] = ACTIONS(1407),
    [sym_data_max_name_length] = ACTIONS(1407),
    [sym_data_max_description_length] = ACTIONS(1407),
    [sym_data_max_type_length] = ACTIONS(1407),
    [sym_data_max_format_length] = ACTIONS(1407),
    [sym_data_max_source_length] = ACTIONS(1407),
    [sym_data_max_value_length] = ACTIONS(1407),
    [sym_data_max_label_length] = ACTIONS(1407),
    [sym_data_max_url_length] = ACTIONS(1407),
    [sym_max_last_login_info_length] = ACTIONS(1407),
    [sym_data_type_unknown] = ACTIONS(1407),
    [sym_data_type_string] = ACTIONS(1407),
    [sym_data_type_datetime] = ACTIONS(1407),
    [sym_data_type_date] = ACTIONS(1407),
    [sym_data_type_time] = ACTIONS(1407),
    [sym_data_type_image] = ACTIONS(1407),
    [sym_data_format_url] = ACTIONS(1407),
    [sym_data_format_phone] = ACTIONS(1407),
    [sym_data_format_email] = ACTIONS(1407),
    [sym_data_format_iso8601] = ACTIONS(1407),
    [sym_auditlog_priv] = ACTIONS(1407),
    [sym_configuration_priv] = ACTIONS(1407),
    [sym_deviceconfig_priv] = ACTIONS(1407),
    [sym_ftp_priv] = ACTIONS(1407),
    [sym_http_priv] = ACTIONS(1407),
    [sym_networkconfig_priv] = ACTIONS(1407),
    [sym_programport_priv] = ACTIONS(1407),
    [sym_remoteui_priv] = ACTIONS(1407),
    [sym_securitycontrol_priv] = ACTIONS(1407),
    [sym_softwaremanagement_priv] = ACTIONS(1407),
    [sym_terminal_priv] = ACTIONS(1407),
    [sym_tpadmin_priv] = ACTIONS(1407),
    [sym_user1_priv] = ACTIONS(1407),
    [sym_user2_priv] = ACTIONS(1407),
    [sym_user3_priv] = ACTIONS(1407),
    [sym_user4_priv] = ACTIONS(1407),
    [sym_usermanagement_priv] = ACTIONS(1407),
    [sym_valid_account] = ACTIONS(1407),
    [sym_err_name_invalid] = ACTIONS(1407),
    [sym_err_password_invalid] = ACTIONS(1407),
    [sym_err_invalid_account] = ACTIONS(1407),
    [sym_err_no_authorization] = ACTIONS(1407),
    [sym_err_invalid_parameter] = ACTIONS(1407),
    [sym_err_user_locked_out] = ACTIONS(1407),
    [sym_err_user_account_expired] = ACTIONS(1407),
    [sym_err_general_error] = ACTIONS(1407),
    [sym_err_authentication_error] = ACTIONS(1407),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1407),
    [sym_tls_validate_certificate] = ACTIONS(1407),
    [sym_tdata] = ACTIONS(1407),
    [sym_tchannel] = ACTIONS(1407),
    [sym_tlevel] = ACTIONS(1407),
    [sym_tbutton] = ACTIONS(1407),
    [sym_ttimeline] = ACTIONS(1407),
    [sym_tcustom] = ACTIONS(1407),
    [sym_url_struct] = ACTIONS(1407),
    [sym_dns_struct] = ACTIONS(1407),
    [sym_ip_address_struct] = ACTIONS(1407),
    [sym_dev_info_struct] = ACTIONS(1407),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1407),
    [sym_data_feed] = ACTIONS(1407),
    [sym_data_field] = ACTIONS(1407),
    [sym_data_record] = ACTIONS(1407),
    [sym_wc_data_feed] = ACTIONS(1407),
    [sym_wc_data_field] = ACTIONS(1407),
    [sym_wc_data_record] = ACTIONS(1407),
    [sym_last_login_info] = ACTIONS(1407),
    [sym____reserved___] = ACTIONS(1407),
    [sym_date] = ACTIONS(1407),
    [sym_day] = ACTIONS(1407),
    [sym_time] = ACTIONS(1407),
    [sym_ldate] = ACTIONS(1407),
    [sym_push_device] = ACTIONS(1407),
    [sym_push_channel] = ACTIONS(1407),
    [sym_push_devchan] = ACTIONS(1407),
    [sym_release_device] = ACTIONS(1407),
    [sym_release_channel] = ACTIONS(1407),
    [sym_release_devchan] = ACTIONS(1407),
    [sym_master_slot] = ACTIONS(1407),
    [sym_get_pulse_time] = ACTIONS(1407),
    [sym_get_timer] = ACTIONS(1407),
    [sym_master_sn] = ACTIONS(1407),
    [sym_system_number] = ACTIONS(1407),
    [sym_button] = ACTIONS(1407),
    [sym_dv_channel] = ACTIONS(1407),
    [sym_data] = ACTIONS(1407),
    [sym_level] = ACTIONS(1407),
    [sym_timeline] = ACTIONS(1407),
    [sym_channel] = ACTIONS(1407),
    [sym_custom] = ACTIONS(1407),
    [sym_atoi] = ACTIONS(1407),
    [sym_atol] = ACTIONS(1407),
    [sym_atof] = ACTIONS(1407),
    [sym_itoa] = ACTIONS(1407),
    [sym_itohex] = ACTIONS(1407),
    [sym_hextoi] = ACTIONS(1407),
    [sym_ftoa] = ACTIONS(1407),
    [sym_find_string] = ACTIONS(1407),
    [sym_left_string] = ACTIONS(1407),
    [sym_right_string] = ACTIONS(1407),
    [sym_mid_string] = ACTIONS(1407),
    [sym_remove_string] = ACTIONS(1407),
    [sym_get_buffer_string] = ACTIONS(1407),
    [sym_get_buffer_char] = ACTIONS(1407),
    [sym_get_multi_buffer_string] = ACTIONS(1407),
    [sym_length_string] = ACTIONS(1407),
    [sym_max_length_string] = ACTIONS(1407),
    [sym_set_length_string] = ACTIONS(1407),
    [sym_compare_string] = ACTIONS(1407),
    [sym_upper_string] = ACTIONS(1407),
    [sym_lower_string] = ACTIONS(1407),
    [sym_get_last] = ACTIONS(1407),
    [sym_date_to_day] = ACTIONS(1407),
    [sym_date_to_month] = ACTIONS(1407),
    [sym_date_to_year] = ACTIONS(1407),
    [sym_time_to_hour] = ACTIONS(1407),
    [sym_time_to_minute] = ACTIONS(1407),
    [sym_time_to_second] = ACTIONS(1407),
    [sym_day_of_week] = ACTIONS(1407),
    [sym_set_pulse_time] = ACTIONS(1407),
    [sym_set_timer] = ACTIONS(1407),
    [sym_file_open] = ACTIONS(1407),
    [sym_file_close] = ACTIONS(1407),
    [sym_file_read] = ACTIONS(1407),
    [sym_file_read_line] = ACTIONS(1407),
    [sym_file_write] = ACTIONS(1407),
    [sym_file_write_line] = ACTIONS(1407),
    [sym_file_copy] = ACTIONS(1407),
    [sym_file_delete] = ACTIONS(1407),
    [sym_file_rename] = ACTIONS(1407),
    [sym_file_dir] = ACTIONS(1407),
    [sym_file_getdir] = ACTIONS(1407),
    [sym_file_setdir] = ACTIONS(1407),
    [sym_file_create_dir] = ACTIONS(1407),
    [sym_file_remove_dir] = ACTIONS(1407),
    [sym_file_seek] = ACTIONS(1407),
    [sym_length_array] = ACTIONS(1407),
    [sym_max_length_array] = ACTIONS(1407),
    [sym_set_length_array] = ACTIONS(1407),
    [sym_ip_client_open] = ACTIONS(1407),
    [sym_ip_client_close] = ACTIONS(1407),
    [sym_ip_server_open] = ACTIONS(1407),
    [sym_ip_server_close] = ACTIONS(1407),
    [sym_ip_bound_client_open] = ACTIONS(1407),
    [sym_ip_mc_server_open] = ACTIONS(1407),
    [sym_ip_set_option] = ACTIONS(1407),
    [sym_ssh_client_open] = ACTIONS(1407),
    [sym_ssh_client_close] = ACTIONS(1407),
    [sym_tls_client_open] = ACTIONS(1407),
    [sym_tls_client_close] = ACTIONS(1407),
    [sym_do_push] = ACTIONS(1407),
    [sym_do_push_timed] = ACTIONS(1407),
    [sym_do_release] = ACTIONS(1407),
    [sym_device_id] = ACTIONS(1407),
    [sym_device_info] = ACTIONS(1407),
    [sym_device_id_string] = ACTIONS(1407),
    [sym_device_standby] = ACTIONS(1407),
    [sym_device_wake] = ACTIONS(1407),
    [sym_timeline_create] = ACTIONS(1407),
    [sym_timeline_kill] = ACTIONS(1407),
    [sym_timeline_reload] = ACTIONS(1407),
    [sym_timeline_pause] = ACTIONS(1407),
    [sym_timeline_restart] = ACTIONS(1407),
    [sym_timeline_set] = ACTIONS(1407),
    [sym_timeline_get] = ACTIONS(1407),
    [sym_timeline_active] = ACTIONS(1407),
    [sym_timeline_dynamic_id] = ACTIONS(1407),
    [sym_variable_to_string] = ACTIONS(1407),
    [sym_string_to_variable] = ACTIONS(1407),
    [sym_length_variable_to_string] = ACTIONS(1407),
    [sym_type_cast] = ACTIONS(1407),
    [sym_variable_to_xml] = ACTIONS(1407),
    [sym_xml_to_variable] = ACTIONS(1407),
    [sym_length_variable_to_xml] = ACTIONS(1407),
    [sym_redirect_string] = ACTIONS(1407),
    [sym_random_number] = ACTIONS(1407),
    [sym_get_unique_id] = ACTIONS(1407),
    [sym_get_system_number] = ACTIONS(1407),
    [sym_set_system_number] = ACTIONS(1407),
    [sym_get_serial_number] = ACTIONS(1407),
    [sym_get_master_build] = ACTIONS(1407),
    [sym_set_validation_code] = ACTIONS(1407),
    [sym_reboot] = ACTIONS(1407),
    [sym_load_duet_module] = ACTIONS(1407),
    [sym_unload_duet_module] = ACTIONS(1407),
    [sym_duet_mem_size_set] = ACTIONS(1407),
    [sym_duet_mem_size_get] = ACTIONS(1407),
    [sym_dynamic_application_device] = ACTIONS(1407),
    [sym_dynamic_polled_port] = ACTIONS(1407),
    [sym_static_port_binding] = ACTIONS(1407),
    [sym_static_ip_binding] = ACTIONS(1407),
    [sym_get_dns_list] = ACTIONS(1407),
    [sym_set_dns_list] = ACTIONS(1407),
    [sym_get_ip_address] = ACTIONS(1407),
    [sym_set_ip_address] = ACTIONS(1407),
    [sym_get_url_list] = ACTIONS(1407),
    [sym_add_url_entry] = ACTIONS(1407),
    [sym_delete_url_entry] = ACTIONS(1407),
    [sym_abs_value] = ACTIONS(1407),
    [sym_max_value] = ACTIONS(1407),
    [sym_min_value] = ACTIONS(1407),
    [sym_exp_value] = ACTIONS(1407),
    [sym_log_value] = ACTIONS(1407),
    [sym_log10_value] = ACTIONS(1407),
    [sym_power_value] = ACTIONS(1407),
    [sym_sqrt_value] = ACTIONS(1407),
    [sym_raw_be] = ACTIONS(1407),
    [sym_raw_le] = ACTIONS(1407),
    [sym_internal_queue_size_set] = ACTIONS(1407),
    [sym_internal_queue_size_get] = ACTIONS(1407),
    [sym_internal_threshold_set] = ACTIONS(1407),
    [sym_internal_threshold_get] = ACTIONS(1407),
    [sym_rebuilt_event] = ACTIONS(1407),
    [sym_do_custom_event] = ACTIONS(1407),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1407),
    [sym_clkmgr_set_clk_source] = ACTIONS(1407),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1407),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1407),
    [sym_clkmgr_get_timezone] = ACTIONS(1407),
    [sym_clkmgr_set_timezone] = ACTIONS(1407),
    [sym_clkmgr_get_resync_period] = ACTIONS(1407),
    [sym_clkmgr_set_resync_period] = ACTIONS(1407),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1407),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1407),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1407),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1407),
    [sym_clkmgr_get_timeservers] = ACTIONS(1407),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1407),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1407),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1407),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1407),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1407),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1407),
    [sym_set_log_level] = ACTIONS(1407),
    [sym_get_log_level] = ACTIONS(1407),
    [sym_amx_log] = ACTIONS(1407),
    [sym_smtp_server_config_set] = ACTIONS(1407),
    [sym_smtp_server_config_get] = ACTIONS(1407),
    [sym_smtp_send] = ACTIONS(1407),
    [sym_validate_netlinx_account] = ACTIONS(1407),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1407),
    [sym_authenticate_certificate] = ACTIONS(1407),
    [sym_audit_netlinx_session_event] = ACTIONS(1407),
    [sym_audit_netlinx_generic_event] = ACTIONS(1407),
    [sym_get_available_flash_disk_space] = ACTIONS(1407),
    [sym_get_max_flash_disk_space] = ACTIONS(1407),
    [sym_data_create_feed] = ACTIONS(1407),
    [sym_data_delete_feed] = ACTIONS(1407),
    [sym_data_publish_feed] = ACTIONS(1407),
    [sym_data_get_published_feed] = ACTIONS(1407),
    [sym_data_add_record] = ACTIONS(1407),
    [sym_data_get_event_record] = ACTIONS(1407),
    [sym__wc_data_create_feed] = ACTIONS(1407),
    [sym__wc_data_add_record] = ACTIONS(1407),
    [sym__wc_data_get_event_record] = ACTIONS(1407),
    [sym_format] = ACTIONS(1407),
    [sym_astro_clock] = ACTIONS(1407),
    [sym_set_outdoor_temperature] = ACTIONS(1407),
    [sym_set_virtual_level_count] = ACTIONS(1407),
    [sym_set_virtual_channel_count] = ACTIONS(1407),
    [sym_set_virtual_port_count] = ACTIONS(1407),
    [sym__wc_encode_int] = ACTIONS(1407),
    [sym__wc_decode_int] = ACTIONS(1407),
    [sym__wc_int] = ACTIONS(1407),
    [sym__wc_to_ch_int] = ACTIONS(1407),
    [sym__ch_to_wc_int] = ACTIONS(1407),
    [sym__wc_find_string_int] = ACTIONS(1407),
    [sym__wc_left_string_int] = ACTIONS(1407),
    [sym__wc_lower_string_int] = ACTIONS(1407),
    [sym__wc_mid_string_int] = ACTIONS(1407),
    [sym__wc_remove_string_int] = ACTIONS(1407),
    [sym__wc_right_string_int] = ACTIONS(1407),
    [sym__wc_upper_string_int] = ACTIONS(1407),
    [sym__wc_compare_string_int] = ACTIONS(1407),
    [sym__wc_get_buffer_char_int] = ACTIONS(1407),
    [sym__wc_get_buffer_string_int] = ACTIONS(1407),
    [sym__wc_concat_string_int] = ACTIONS(1407),
    [sym__wc_file_open_int] = ACTIONS(1407),
    [sym__wc_file_close_int] = ACTIONS(1407),
    [sym__wc_file_read_int] = ACTIONS(1407),
    [sym__wc_file_read_line_int] = ACTIONS(1407),
    [sym__wc_file_write_int] = ACTIONS(1407),
    [sym__wc_file_write_line_int] = ACTIONS(1407),
    [aux_sym_preproc_include_token1] = ACTIONS(1407),
    [aux_sym_preproc_define_token1] = ACTIONS(1409),
    [aux_sym_preproc_warn_token1] = ACTIONS(1409),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1409),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1409),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1409),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1409),
    [anon_sym_RBRACE] = ACTIONS(1409),
    [anon_sym_COLON] = ACTIONS(1619),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1623),
    [anon_sym_BANG] = ACTIONS(1407),
    [anon_sym_TILDE] = ACTIONS(1409),
    [anon_sym_DASH] = ACTIONS(1625),
    [anon_sym_PLUS] = ACTIONS(1625),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1631),
    [anon_sym_AMP_AMP] = ACTIONS(1633),
    [anon_sym_PIPE] = ACTIONS(1613),
    [anon_sym_CARET] = ACTIONS(1635),
    [anon_sym_AMP] = ACTIONS(1611),
    [anon_sym_EQ_EQ] = ACTIONS(1637),
    [anon_sym_BANG_EQ] = ACTIONS(1637),
    [anon_sym_LT_GT] = ACTIONS(1637),
    [anon_sym_GT] = ACTIONS(1639),
    [anon_sym_GT_EQ] = ACTIONS(1641),
    [anon_sym_LT_EQ] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1639),
    [anon_sym_LT_LT] = ACTIONS(1643),
    [anon_sym_GT_GT] = ACTIONS(1643),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1409),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(1409),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1623),
  },
  [STATE(151)] = {
    [sym_argument_list] = STATE(171),
    [sym_string_literal] = STATE(461),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1357),
    [sym_define_call_keyword] = ACTIONS(1357),
    [sym_define_function_keyword] = ACTIONS(1357),
    [sym_define_library_function_keyword] = ACTIONS(1357),
    [sym_define_module_keyword] = ACTIONS(1357),
    [sym_char_keyword] = ACTIONS(1357),
    [sym_widechar_keyword] = ACTIONS(1357),
    [sym_integer_keyword] = ACTIONS(1357),
    [sym_sinteger_keyword] = ACTIONS(1357),
    [sym_long_keyword] = ACTIONS(1357),
    [sym_slong_keyword] = ACTIONS(1357),
    [sym_float_keyword] = ACTIONS(1357),
    [sym_double_keyword] = ACTIONS(1357),
    [sym_variant_keyword] = ACTIONS(1357),
    [sym_variantarray_keyword] = ACTIONS(1357),
    [sym_local_var_keyword] = ACTIONS(1357),
    [sym_stack_var_keyword] = ACTIONS(1357),
    [sym_constant_keyword] = ACTIONS(1357),
    [sym_volatile_keyword] = ACTIONS(1357),
    [sym_non_volatile_keyword] = ACTIONS(1357),
    [sym_persistent_keyword] = ACTIONS(1357),
    [sym_dev_keyword] = ACTIONS(1357),
    [sym_devlev_keyword] = ACTIONS(1357),
    [sym_devchan_keyword] = ACTIONS(1357),
    [sym_if_keyword] = ACTIONS(1357),
    [sym_else_keyword] = ACTIONS(1357),
    [sym_switch_keyword] = ACTIONS(1357),
    [sym_case_keyword] = ACTIONS(1357),
    [sym_default_keyword] = ACTIONS(1357),
    [sym_while_keyword] = ACTIONS(1357),
    [sym_for_keyword] = ACTIONS(1357),
    [sym_break_keyword] = ACTIONS(1357),
    [sym_continue_keyword] = ACTIONS(1357),
    [sym_return_keyword] = ACTIONS(1357),
    [sym_select_keyword] = ACTIONS(1357),
    [sym_active_keyword] = ACTIONS(1357),
    [sym_struct_keyword] = ACTIONS(1357),
    [sym_structure_keyword] = ACTIONS(1357),
    [aux_sym_on_keyword_token1] = ACTIONS(1357),
    [aux_sym_off_keyword_token1] = ACTIONS(1357),
    [sym_send_level_keyword] = ACTIONS(1357),
    [sym_send_string_keyword] = ACTIONS(1357),
    [sym_send_command_keyword] = ACTIONS(1357),
    [sym_clear_buffer_keyword] = ACTIONS(1357),
    [sym_create_buffer_keyword] = ACTIONS(1357),
    [sym_create_multi_buffer_keyword] = ACTIONS(1357),
    [sym_call_keyword] = ACTIONS(1357),
    [sym_system_call_keyword] = ACTIONS(1357),
    [sym_devchan_to_keyword] = ACTIONS(1357),
    [sym_devchan_min_to_keyword] = ACTIONS(1357),
    [sym_devchan_total_off_keyword] = ACTIONS(1357),
    [sym_devchan_pulse_keyword] = ACTIONS(1357),
    [sym_wait_keyword] = ACTIONS(1357),
    [sym_cancel_wait_keyword] = ACTIONS(1357),
    [sym_wait_until_keyword] = ACTIONS(1357),
    [sym_cancel_wait_until_keyword] = ACTIONS(1357),
    [sym_cancel_all_wait_keyword] = ACTIONS(1357),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1357),
    [sym_and] = ACTIONS(1605),
    [sym_or] = ACTIONS(1607),
    [sym_not] = ACTIONS(1357),
    [sym_xor] = ACTIONS(1609),
    [sym_band] = ACTIONS(1611),
    [sym_bor] = ACTIONS(1613),
    [sym_bxor] = ACTIONS(1609),
    [sym_bnot] = ACTIONS(1357),
    [sym_lshift] = ACTIONS(1615),
    [sym_rshift] = ACTIONS(1615),
    [sym___file__] = ACTIONS(1357),
    [sym___date__] = ACTIONS(1357),
    [sym___ldate__] = ACTIONS(1357),
    [sym___time__] = ACTIONS(1357),
    [sym___line__] = ACTIONS(1357),
    [sym___name__] = ACTIONS(1357),
    [sym___netlinx__] = ACTIONS(1357),
    [sym_netlinx_axi_version] = ACTIONS(1357),
    [sym_url_flg_tcp] = ACTIONS(1357),
    [sym_url_flg_accinfopresent] = ACTIONS(1357),
    [sym_url_flg_temp] = ACTIONS(1357),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1357),
    [sym_url_flg_stat_mask] = ACTIONS(1357),
    [sym_url_flg_stat_lookup] = ACTIONS(1357),
    [sym_url_flg_stat_connecting] = ACTIONS(1357),
    [sym_url_flg_stat_waiting] = ACTIONS(1357),
    [sym_url_flg_stat_connected] = ACTIONS(1357),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1357),
    [sym_first_virtual_device] = ACTIONS(1357),
    [sym_first_local_port] = ACTIONS(1357),
    [sym_dynamic_virtual_device] = ACTIONS(1357),
    [sym_timeline_once] = ACTIONS(1357),
    [sym_timeline_repeat] = ACTIONS(1357),
    [sym_timeline_absolute] = ACTIONS(1357),
    [sym_timeline_relative] = ACTIONS(1357),
    [sym_do_push_timed_infinite] = ACTIONS(1357),
    [sym_source_type_no_address] = ACTIONS(1357),
    [sym_source_type_neuron_id] = ACTIONS(1357),
    [sym_source_type_ip_address] = ACTIONS(1357),
    [sym_source_type_axlink] = ACTIONS(1357),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1357),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1357),
    [sym_source_type_ip_socket_address] = ACTIONS(1357),
    [sym_source_type_rs232] = ACTIONS(1357),
    [sym_source_type_internal] = ACTIONS(1357),
    [sym_source_type_mac_address] = ACTIONS(1357),
    [sym_source_type_ipv4_port] = ACTIONS(1357),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1357),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1357),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1357),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1357),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1357),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1357),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1357),
    [sym_true] = ACTIONS(1357),
    [sym_false] = ACTIONS(1357),
    [sym_file_read_only] = ACTIONS(1357),
    [sym_file_rw_new] = ACTIONS(1357),
    [sym_file_rw_append] = ACTIONS(1357),
    [sym_ip_tcp] = ACTIONS(1357),
    [sym_ip_udp] = ACTIONS(1357),
    [sym_ip_udp_2way] = ACTIONS(1357),
    [sym_xml_encode_types] = ACTIONS(1357),
    [sym_xml_encode_char_as_list] = ACTIONS(1357),
    [sym_xml_encode_le] = ACTIONS(1357),
    [sym_xml_decode_types] = ACTIONS(1357),
    [sym_xml_decode_no_preserve] = ACTIONS(1357),
    [sym_duet_dev_type_display_device] = ACTIONS(1357),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1357),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1357),
    [sym_duet_dev_type_io_device] = ACTIONS(1357),
    [sym_duet_dev_type_relay_device] = ACTIONS(1357),
    [sym_duet_dev_type_ups] = ACTIONS(1357),
    [sym_duet_dev_type_amplifier] = ACTIONS(1357),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1357),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1357),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1357),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1357),
    [sym_duet_dev_type_camera] = ACTIONS(1357),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1357),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1357),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1357),
    [sym_duet_dev_type_dss] = ACTIONS(1357),
    [sym_duet_dev_type_dvr] = ACTIONS(1357),
    [sym_duet_dev_type_disc_device] = ACTIONS(1357),
    [sym_duet_dev_type_document_camera] = ACTIONS(1357),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1357),
    [sym_duet_dev_type_hvac] = ACTIONS(1357),
    [sym_duet_dev_type_keypad] = ACTIONS(1357),
    [sym_duet_dev_type_light] = ACTIONS(1357),
    [sym_duet_dev_type_monitor] = ACTIONS(1357),
    [sym_duet_dev_type_motor] = ACTIONS(1357),
    [sym_duet_dev_type_multi_window] = ACTIONS(1357),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1357),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1357),
    [sym_duet_dev_type_receiver] = ACTIONS(1357),
    [sym_duet_dev_type_security_system] = ACTIONS(1357),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1357),
    [sym_duet_dev_type_settop_box] = ACTIONS(1357),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1357),
    [sym_duet_dev_type_switcher] = ACTIONS(1357),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1357),
    [sym_duet_dev_type_tv] = ACTIONS(1357),
    [sym_duet_dev_type_utility] = ACTIONS(1357),
    [sym_duet_dev_type_vcr] = ACTIONS(1357),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1357),
    [sym_duet_dev_type_video_processor] = ACTIONS(1357),
    [sym_duet_dev_type_video_projector] = ACTIONS(1357),
    [sym_duet_dev_type_video_wall] = ACTIONS(1357),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1357),
    [sym_duet_dev_type_weather] = ACTIONS(1357),
    [sym_duet_dev_not_polled] = ACTIONS(1357),
    [sym_duet_dev_polled] = ACTIONS(1357),
    [sym_clkmgr_mode_network] = ACTIONS(1357),
    [sym_clkmgr_mode_standalone] = ACTIONS(1357),
    [sym_normal_standby] = ACTIONS(1357),
    [sym_normal_wake] = ACTIONS(1357),
    [sym_ip_multicast_ttl_option] = ACTIONS(1357),
    [sym_ip_ttl_subnet] = ACTIONS(1357),
    [sym_ip_ttl_site] = ACTIONS(1357),
    [sym_ip_ttl_region] = ACTIONS(1357),
    [sym_ip_ttl_continent] = ACTIONS(1357),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1357),
    [sym_ip_nodelay_on] = ACTIONS(1357),
    [sym_ip_nodelay_off] = ACTIONS(1357),
    [sym_amx_error] = ACTIONS(1357),
    [sym_amx_warning] = ACTIONS(1357),
    [sym_amx_info] = ACTIONS(1357),
    [sym_amx_debug] = ACTIONS(1357),
    [sym_netlinx_login_success] = ACTIONS(1357),
    [sym_netlinx_login_fail] = ACTIONS(1357),
    [sym_netlinx_logout] = ACTIONS(1357),
    [sym_null_str] = ACTIONS(1357),
    [sym_smtp_address] = ACTIONS(1357),
    [sym_smtp_port_number] = ACTIONS(1357),
    [sym_smtp_username] = ACTIONS(1357),
    [sym_smtp_password] = ACTIONS(1357),
    [sym_smtp_require_tls] = ACTIONS(1357),
    [sym_smtp_from] = ACTIONS(1357),
    [sym_smtp_tls_true] = ACTIONS(1357),
    [sym_smtp_tls_false] = ACTIONS(1357),
    [sym_listview_on_row_select_event] = ACTIONS(1357),
    [sym_data_structure_unknown] = ACTIONS(1357),
    [sym_data_structure_datafeed] = ACTIONS(1357),
    [sym_data_structure_datarecordset] = ACTIONS(1357),
    [sym_data_structure_datarecord] = ACTIONS(1357),
    [sym_data_structure_datafield] = ACTIONS(1357),
    [sym_data_max_content_field_count] = ACTIONS(1357),
    [sym_data_max_metadata_field_count] = ACTIONS(1357),
    [sym_data_max_id_length] = ACTIONS(1357),
    [sym_data_max_name_length] = ACTIONS(1357),
    [sym_data_max_description_length] = ACTIONS(1357),
    [sym_data_max_type_length] = ACTIONS(1357),
    [sym_data_max_format_length] = ACTIONS(1357),
    [sym_data_max_source_length] = ACTIONS(1357),
    [sym_data_max_value_length] = ACTIONS(1357),
    [sym_data_max_label_length] = ACTIONS(1357),
    [sym_data_max_url_length] = ACTIONS(1357),
    [sym_max_last_login_info_length] = ACTIONS(1357),
    [sym_data_type_unknown] = ACTIONS(1357),
    [sym_data_type_string] = ACTIONS(1357),
    [sym_data_type_datetime] = ACTIONS(1357),
    [sym_data_type_date] = ACTIONS(1357),
    [sym_data_type_time] = ACTIONS(1357),
    [sym_data_type_image] = ACTIONS(1357),
    [sym_data_format_url] = ACTIONS(1357),
    [sym_data_format_phone] = ACTIONS(1357),
    [sym_data_format_email] = ACTIONS(1357),
    [sym_data_format_iso8601] = ACTIONS(1357),
    [sym_auditlog_priv] = ACTIONS(1357),
    [sym_configuration_priv] = ACTIONS(1357),
    [sym_deviceconfig_priv] = ACTIONS(1357),
    [sym_ftp_priv] = ACTIONS(1357),
    [sym_http_priv] = ACTIONS(1357),
    [sym_networkconfig_priv] = ACTIONS(1357),
    [sym_programport_priv] = ACTIONS(1357),
    [sym_remoteui_priv] = ACTIONS(1357),
    [sym_securitycontrol_priv] = ACTIONS(1357),
    [sym_softwaremanagement_priv] = ACTIONS(1357),
    [sym_terminal_priv] = ACTIONS(1357),
    [sym_tpadmin_priv] = ACTIONS(1357),
    [sym_user1_priv] = ACTIONS(1357),
    [sym_user2_priv] = ACTIONS(1357),
    [sym_user3_priv] = ACTIONS(1357),
    [sym_user4_priv] = ACTIONS(1357),
    [sym_usermanagement_priv] = ACTIONS(1357),
    [sym_valid_account] = ACTIONS(1357),
    [sym_err_name_invalid] = ACTIONS(1357),
    [sym_err_password_invalid] = ACTIONS(1357),
    [sym_err_invalid_account] = ACTIONS(1357),
    [sym_err_no_authorization] = ACTIONS(1357),
    [sym_err_invalid_parameter] = ACTIONS(1357),
    [sym_err_user_locked_out] = ACTIONS(1357),
    [sym_err_user_account_expired] = ACTIONS(1357),
    [sym_err_general_error] = ACTIONS(1357),
    [sym_err_authentication_error] = ACTIONS(1357),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1357),
    [sym_tls_validate_certificate] = ACTIONS(1357),
    [sym_tdata] = ACTIONS(1357),
    [sym_tchannel] = ACTIONS(1357),
    [sym_tlevel] = ACTIONS(1357),
    [sym_tbutton] = ACTIONS(1357),
    [sym_ttimeline] = ACTIONS(1357),
    [sym_tcustom] = ACTIONS(1357),
    [sym_url_struct] = ACTIONS(1357),
    [sym_dns_struct] = ACTIONS(1357),
    [sym_ip_address_struct] = ACTIONS(1357),
    [sym_dev_info_struct] = ACTIONS(1357),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1357),
    [sym_data_feed] = ACTIONS(1357),
    [sym_data_field] = ACTIONS(1357),
    [sym_data_record] = ACTIONS(1357),
    [sym_wc_data_feed] = ACTIONS(1357),
    [sym_wc_data_field] = ACTIONS(1357),
    [sym_wc_data_record] = ACTIONS(1357),
    [sym_last_login_info] = ACTIONS(1357),
    [sym____reserved___] = ACTIONS(1357),
    [sym_date] = ACTIONS(1357),
    [sym_day] = ACTIONS(1357),
    [sym_time] = ACTIONS(1357),
    [sym_ldate] = ACTIONS(1357),
    [sym_push_device] = ACTIONS(1357),
    [sym_push_channel] = ACTIONS(1357),
    [sym_push_devchan] = ACTIONS(1357),
    [sym_release_device] = ACTIONS(1357),
    [sym_release_channel] = ACTIONS(1357),
    [sym_release_devchan] = ACTIONS(1357),
    [sym_master_slot] = ACTIONS(1357),
    [sym_get_pulse_time] = ACTIONS(1357),
    [sym_get_timer] = ACTIONS(1357),
    [sym_master_sn] = ACTIONS(1357),
    [sym_system_number] = ACTIONS(1357),
    [sym_button] = ACTIONS(1357),
    [sym_dv_channel] = ACTIONS(1357),
    [sym_data] = ACTIONS(1357),
    [sym_level] = ACTIONS(1357),
    [sym_timeline] = ACTIONS(1357),
    [sym_channel] = ACTIONS(1357),
    [sym_custom] = ACTIONS(1357),
    [sym_atoi] = ACTIONS(1357),
    [sym_atol] = ACTIONS(1357),
    [sym_atof] = ACTIONS(1357),
    [sym_itoa] = ACTIONS(1357),
    [sym_itohex] = ACTIONS(1357),
    [sym_hextoi] = ACTIONS(1357),
    [sym_ftoa] = ACTIONS(1357),
    [sym_find_string] = ACTIONS(1357),
    [sym_left_string] = ACTIONS(1357),
    [sym_right_string] = ACTIONS(1357),
    [sym_mid_string] = ACTIONS(1357),
    [sym_remove_string] = ACTIONS(1357),
    [sym_get_buffer_string] = ACTIONS(1357),
    [sym_get_buffer_char] = ACTIONS(1357),
    [sym_get_multi_buffer_string] = ACTIONS(1357),
    [sym_length_string] = ACTIONS(1357),
    [sym_max_length_string] = ACTIONS(1357),
    [sym_set_length_string] = ACTIONS(1357),
    [sym_compare_string] = ACTIONS(1357),
    [sym_upper_string] = ACTIONS(1357),
    [sym_lower_string] = ACTIONS(1357),
    [sym_get_last] = ACTIONS(1357),
    [sym_date_to_day] = ACTIONS(1357),
    [sym_date_to_month] = ACTIONS(1357),
    [sym_date_to_year] = ACTIONS(1357),
    [sym_time_to_hour] = ACTIONS(1357),
    [sym_time_to_minute] = ACTIONS(1357),
    [sym_time_to_second] = ACTIONS(1357),
    [sym_day_of_week] = ACTIONS(1357),
    [sym_set_pulse_time] = ACTIONS(1357),
    [sym_set_timer] = ACTIONS(1357),
    [sym_file_open] = ACTIONS(1357),
    [sym_file_close] = ACTIONS(1357),
    [sym_file_read] = ACTIONS(1357),
    [sym_file_read_line] = ACTIONS(1357),
    [sym_file_write] = ACTIONS(1357),
    [sym_file_write_line] = ACTIONS(1357),
    [sym_file_copy] = ACTIONS(1357),
    [sym_file_delete] = ACTIONS(1357),
    [sym_file_rename] = ACTIONS(1357),
    [sym_file_dir] = ACTIONS(1357),
    [sym_file_getdir] = ACTIONS(1357),
    [sym_file_setdir] = ACTIONS(1357),
    [sym_file_create_dir] = ACTIONS(1357),
    [sym_file_remove_dir] = ACTIONS(1357),
    [sym_file_seek] = ACTIONS(1357),
    [sym_length_array] = ACTIONS(1357),
    [sym_max_length_array] = ACTIONS(1357),
    [sym_set_length_array] = ACTIONS(1357),
    [sym_ip_client_open] = ACTIONS(1357),
    [sym_ip_client_close] = ACTIONS(1357),
    [sym_ip_server_open] = ACTIONS(1357),
    [sym_ip_server_close] = ACTIONS(1357),
    [sym_ip_bound_client_open] = ACTIONS(1357),
    [sym_ip_mc_server_open] = ACTIONS(1357),
    [sym_ip_set_option] = ACTIONS(1357),
    [sym_ssh_client_open] = ACTIONS(1357),
    [sym_ssh_client_close] = ACTIONS(1357),
    [sym_tls_client_open] = ACTIONS(1357),
    [sym_tls_client_close] = ACTIONS(1357),
    [sym_do_push] = ACTIONS(1357),
    [sym_do_push_timed] = ACTIONS(1357),
    [sym_do_release] = ACTIONS(1357),
    [sym_device_id] = ACTIONS(1357),
    [sym_device_info] = ACTIONS(1357),
    [sym_device_id_string] = ACTIONS(1357),
    [sym_device_standby] = ACTIONS(1357),
    [sym_device_wake] = ACTIONS(1357),
    [sym_timeline_create] = ACTIONS(1357),
    [sym_timeline_kill] = ACTIONS(1357),
    [sym_timeline_reload] = ACTIONS(1357),
    [sym_timeline_pause] = ACTIONS(1357),
    [sym_timeline_restart] = ACTIONS(1357),
    [sym_timeline_set] = ACTIONS(1357),
    [sym_timeline_get] = ACTIONS(1357),
    [sym_timeline_active] = ACTIONS(1357),
    [sym_timeline_dynamic_id] = ACTIONS(1357),
    [sym_variable_to_string] = ACTIONS(1357),
    [sym_string_to_variable] = ACTIONS(1357),
    [sym_length_variable_to_string] = ACTIONS(1357),
    [sym_type_cast] = ACTIONS(1357),
    [sym_variable_to_xml] = ACTIONS(1357),
    [sym_xml_to_variable] = ACTIONS(1357),
    [sym_length_variable_to_xml] = ACTIONS(1357),
    [sym_redirect_string] = ACTIONS(1357),
    [sym_random_number] = ACTIONS(1357),
    [sym_get_unique_id] = ACTIONS(1357),
    [sym_get_system_number] = ACTIONS(1357),
    [sym_set_system_number] = ACTIONS(1357),
    [sym_get_serial_number] = ACTIONS(1357),
    [sym_get_master_build] = ACTIONS(1357),
    [sym_set_validation_code] = ACTIONS(1357),
    [sym_reboot] = ACTIONS(1357),
    [sym_load_duet_module] = ACTIONS(1357),
    [sym_unload_duet_module] = ACTIONS(1357),
    [sym_duet_mem_size_set] = ACTIONS(1357),
    [sym_duet_mem_size_get] = ACTIONS(1357),
    [sym_dynamic_application_device] = ACTIONS(1357),
    [sym_dynamic_polled_port] = ACTIONS(1357),
    [sym_static_port_binding] = ACTIONS(1357),
    [sym_static_ip_binding] = ACTIONS(1357),
    [sym_get_dns_list] = ACTIONS(1357),
    [sym_set_dns_list] = ACTIONS(1357),
    [sym_get_ip_address] = ACTIONS(1357),
    [sym_set_ip_address] = ACTIONS(1357),
    [sym_get_url_list] = ACTIONS(1357),
    [sym_add_url_entry] = ACTIONS(1357),
    [sym_delete_url_entry] = ACTIONS(1357),
    [sym_abs_value] = ACTIONS(1357),
    [sym_max_value] = ACTIONS(1357),
    [sym_min_value] = ACTIONS(1357),
    [sym_exp_value] = ACTIONS(1357),
    [sym_log_value] = ACTIONS(1357),
    [sym_log10_value] = ACTIONS(1357),
    [sym_power_value] = ACTIONS(1357),
    [sym_sqrt_value] = ACTIONS(1357),
    [sym_raw_be] = ACTIONS(1357),
    [sym_raw_le] = ACTIONS(1357),
    [sym_internal_queue_size_set] = ACTIONS(1357),
    [sym_internal_queue_size_get] = ACTIONS(1357),
    [sym_internal_threshold_set] = ACTIONS(1357),
    [sym_internal_threshold_get] = ACTIONS(1357),
    [sym_rebuilt_event] = ACTIONS(1357),
    [sym_do_custom_event] = ACTIONS(1357),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1357),
    [sym_clkmgr_set_clk_source] = ACTIONS(1357),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1357),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1357),
    [sym_clkmgr_get_timezone] = ACTIONS(1357),
    [sym_clkmgr_set_timezone] = ACTIONS(1357),
    [sym_clkmgr_get_resync_period] = ACTIONS(1357),
    [sym_clkmgr_set_resync_period] = ACTIONS(1357),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1357),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1357),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1357),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1357),
    [sym_clkmgr_get_timeservers] = ACTIONS(1357),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1357),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1357),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1357),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1357),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1357),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1357),
    [sym_set_log_level] = ACTIONS(1357),
    [sym_get_log_level] = ACTIONS(1357),
    [sym_amx_log] = ACTIONS(1357),
    [sym_smtp_server_config_set] = ACTIONS(1357),
    [sym_smtp_server_config_get] = ACTIONS(1357),
    [sym_smtp_send] = ACTIONS(1357),
    [sym_validate_netlinx_account] = ACTIONS(1357),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1357),
    [sym_authenticate_certificate] = ACTIONS(1357),
    [sym_audit_netlinx_session_event] = ACTIONS(1357),
    [sym_audit_netlinx_generic_event] = ACTIONS(1357),
    [sym_get_available_flash_disk_space] = ACTIONS(1357),
    [sym_get_max_flash_disk_space] = ACTIONS(1357),
    [sym_data_create_feed] = ACTIONS(1357),
    [sym_data_delete_feed] = ACTIONS(1357),
    [sym_data_publish_feed] = ACTIONS(1357),
    [sym_data_get_published_feed] = ACTIONS(1357),
    [sym_data_add_record] = ACTIONS(1357),
    [sym_data_get_event_record] = ACTIONS(1357),
    [sym__wc_data_create_feed] = ACTIONS(1357),
    [sym__wc_data_add_record] = ACTIONS(1357),
    [sym__wc_data_get_event_record] = ACTIONS(1357),
    [sym_format] = ACTIONS(1357),
    [sym_astro_clock] = ACTIONS(1357),
    [sym_set_outdoor_temperature] = ACTIONS(1357),
    [sym_set_virtual_level_count] = ACTIONS(1357),
    [sym_set_virtual_channel_count] = ACTIONS(1357),
    [sym_set_virtual_port_count] = ACTIONS(1357),
    [sym__wc_encode_int] = ACTIONS(1357),
    [sym__wc_decode_int] = ACTIONS(1357),
    [sym__wc_int] = ACTIONS(1357),
    [sym__wc_to_ch_int] = ACTIONS(1357),
    [sym__ch_to_wc_int] = ACTIONS(1357),
    [sym__wc_find_string_int] = ACTIONS(1357),
    [sym__wc_left_string_int] = ACTIONS(1357),
    [sym__wc_lower_string_int] = ACTIONS(1357),
    [sym__wc_mid_string_int] = ACTIONS(1357),
    [sym__wc_remove_string_int] = ACTIONS(1357),
    [sym__wc_right_string_int] = ACTIONS(1357),
    [sym__wc_upper_string_int] = ACTIONS(1357),
    [sym__wc_compare_string_int] = ACTIONS(1357),
    [sym__wc_get_buffer_char_int] = ACTIONS(1357),
    [sym__wc_get_buffer_string_int] = ACTIONS(1357),
    [sym__wc_concat_string_int] = ACTIONS(1357),
    [sym__wc_file_open_int] = ACTIONS(1357),
    [sym__wc_file_close_int] = ACTIONS(1357),
    [sym__wc_file_read_int] = ACTIONS(1357),
    [sym__wc_file_read_line_int] = ACTIONS(1357),
    [sym__wc_file_write_int] = ACTIONS(1357),
    [sym__wc_file_write_line_int] = ACTIONS(1357),
    [aux_sym_preproc_include_token1] = ACTIONS(1357),
    [aux_sym_preproc_define_token1] = ACTIONS(1371),
    [aux_sym_preproc_warn_token1] = ACTIONS(1371),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1371),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1371),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1371),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1371),
    [anon_sym_RBRACE] = ACTIONS(1371),
    [anon_sym_COLON] = ACTIONS(1619),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1651),
    [anon_sym_BANG] = ACTIONS(1357),
    [anon_sym_TILDE] = ACTIONS(1371),
    [anon_sym_DASH] = ACTIONS(1625),
    [anon_sym_PLUS] = ACTIONS(1625),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1631),
    [anon_sym_AMP_AMP] = ACTIONS(1633),
    [anon_sym_PIPE] = ACTIONS(1613),
    [anon_sym_CARET] = ACTIONS(1635),
    [anon_sym_AMP] = ACTIONS(1611),
    [anon_sym_EQ_EQ] = ACTIONS(1637),
    [anon_sym_BANG_EQ] = ACTIONS(1637),
    [anon_sym_LT_GT] = ACTIONS(1637),
    [anon_sym_GT] = ACTIONS(1639),
    [anon_sym_GT_EQ] = ACTIONS(1641),
    [anon_sym_LT_EQ] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1639),
    [anon_sym_LT_LT] = ACTIONS(1643),
    [anon_sym_GT_GT] = ACTIONS(1643),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1371),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(1371),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1651),
  },
  [STATE(152)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1609),
    [sym_band] = ACTIONS(1611),
    [sym_bor] = ACTIONS(1613),
    [sym_bxor] = ACTIONS(1609),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1615),
    [sym_rshift] = ACTIONS(1615),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1625),
    [anon_sym_PLUS] = ACTIONS(1625),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1613),
    [anon_sym_CARET] = ACTIONS(1635),
    [anon_sym_AMP] = ACTIONS(1611),
    [anon_sym_EQ_EQ] = ACTIONS(1637),
    [anon_sym_BANG_EQ] = ACTIONS(1637),
    [anon_sym_LT_GT] = ACTIONS(1637),
    [anon_sym_GT] = ACTIONS(1639),
    [anon_sym_GT_EQ] = ACTIONS(1641),
    [anon_sym_LT_EQ] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1639),
    [anon_sym_LT_LT] = ACTIONS(1643),
    [anon_sym_GT_GT] = ACTIONS(1643),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(153)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1413),
    [sym_rshift] = ACTIONS(1413),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_STAR] = ACTIONS(1415),
    [anon_sym_SLASH] = ACTIONS(1413),
    [anon_sym_PERCENT] = ACTIONS(1415),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1415),
    [anon_sym_GT_GT] = ACTIONS(1415),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(154)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1615),
    [sym_rshift] = ACTIONS(1615),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1625),
    [anon_sym_PLUS] = ACTIONS(1625),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1643),
    [anon_sym_GT_GT] = ACTIONS(1643),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(155)] = {
    [aux_sym_subscript_expression_repeat1] = STATE(155),
    [sym_identifier] = ACTIONS(1496),
    [sym_define_call_keyword] = ACTIONS(1496),
    [sym_define_function_keyword] = ACTIONS(1496),
    [sym_define_library_function_keyword] = ACTIONS(1496),
    [sym_define_module_keyword] = ACTIONS(1496),
    [sym_char_keyword] = ACTIONS(1496),
    [sym_widechar_keyword] = ACTIONS(1496),
    [sym_integer_keyword] = ACTIONS(1496),
    [sym_sinteger_keyword] = ACTIONS(1496),
    [sym_long_keyword] = ACTIONS(1496),
    [sym_slong_keyword] = ACTIONS(1496),
    [sym_float_keyword] = ACTIONS(1496),
    [sym_double_keyword] = ACTIONS(1496),
    [sym_variant_keyword] = ACTIONS(1496),
    [sym_variantarray_keyword] = ACTIONS(1496),
    [sym_local_var_keyword] = ACTIONS(1496),
    [sym_stack_var_keyword] = ACTIONS(1496),
    [sym_constant_keyword] = ACTIONS(1496),
    [sym_volatile_keyword] = ACTIONS(1496),
    [sym_non_volatile_keyword] = ACTIONS(1496),
    [sym_persistent_keyword] = ACTIONS(1496),
    [sym_dev_keyword] = ACTIONS(1496),
    [sym_devlev_keyword] = ACTIONS(1496),
    [sym_devchan_keyword] = ACTIONS(1496),
    [sym_if_keyword] = ACTIONS(1496),
    [sym_else_keyword] = ACTIONS(1496),
    [sym_switch_keyword] = ACTIONS(1496),
    [sym_case_keyword] = ACTIONS(1496),
    [sym_default_keyword] = ACTIONS(1496),
    [sym_while_keyword] = ACTIONS(1496),
    [sym_for_keyword] = ACTIONS(1496),
    [sym_break_keyword] = ACTIONS(1496),
    [sym_continue_keyword] = ACTIONS(1496),
    [sym_return_keyword] = ACTIONS(1496),
    [sym_select_keyword] = ACTIONS(1496),
    [sym_active_keyword] = ACTIONS(1496),
    [sym_struct_keyword] = ACTIONS(1496),
    [sym_structure_keyword] = ACTIONS(1496),
    [aux_sym_on_keyword_token1] = ACTIONS(1496),
    [aux_sym_off_keyword_token1] = ACTIONS(1496),
    [sym_send_level_keyword] = ACTIONS(1496),
    [sym_send_string_keyword] = ACTIONS(1496),
    [sym_send_command_keyword] = ACTIONS(1496),
    [sym_clear_buffer_keyword] = ACTIONS(1496),
    [sym_create_buffer_keyword] = ACTIONS(1496),
    [sym_create_multi_buffer_keyword] = ACTIONS(1496),
    [sym_call_keyword] = ACTIONS(1496),
    [sym_system_call_keyword] = ACTIONS(1496),
    [sym_devchan_to_keyword] = ACTIONS(1496),
    [sym_devchan_min_to_keyword] = ACTIONS(1496),
    [sym_devchan_total_off_keyword] = ACTIONS(1496),
    [sym_devchan_pulse_keyword] = ACTIONS(1496),
    [sym_wait_keyword] = ACTIONS(1496),
    [sym_cancel_wait_keyword] = ACTIONS(1496),
    [sym_wait_until_keyword] = ACTIONS(1496),
    [sym_cancel_wait_until_keyword] = ACTIONS(1496),
    [sym_cancel_all_wait_keyword] = ACTIONS(1496),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1496),
    [sym_and] = ACTIONS(1496),
    [sym_or] = ACTIONS(1496),
    [sym_not] = ACTIONS(1496),
    [sym_xor] = ACTIONS(1496),
    [sym_band] = ACTIONS(1496),
    [sym_bor] = ACTIONS(1496),
    [sym_bxor] = ACTIONS(1496),
    [sym_bnot] = ACTIONS(1496),
    [sym_lshift] = ACTIONS(1496),
    [sym_rshift] = ACTIONS(1496),
    [sym___file__] = ACTIONS(1496),
    [sym___date__] = ACTIONS(1496),
    [sym___ldate__] = ACTIONS(1496),
    [sym___time__] = ACTIONS(1496),
    [sym___line__] = ACTIONS(1496),
    [sym___name__] = ACTIONS(1496),
    [sym___netlinx__] = ACTIONS(1496),
    [sym_netlinx_axi_version] = ACTIONS(1496),
    [sym_url_flg_tcp] = ACTIONS(1496),
    [sym_url_flg_accinfopresent] = ACTIONS(1496),
    [sym_url_flg_temp] = ACTIONS(1496),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1496),
    [sym_url_flg_stat_mask] = ACTIONS(1496),
    [sym_url_flg_stat_lookup] = ACTIONS(1496),
    [sym_url_flg_stat_connecting] = ACTIONS(1496),
    [sym_url_flg_stat_waiting] = ACTIONS(1496),
    [sym_url_flg_stat_connected] = ACTIONS(1496),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1496),
    [sym_first_virtual_device] = ACTIONS(1496),
    [sym_first_local_port] = ACTIONS(1496),
    [sym_dynamic_virtual_device] = ACTIONS(1496),
    [sym_timeline_once] = ACTIONS(1496),
    [sym_timeline_repeat] = ACTIONS(1496),
    [sym_timeline_absolute] = ACTIONS(1496),
    [sym_timeline_relative] = ACTIONS(1496),
    [sym_do_push_timed_infinite] = ACTIONS(1496),
    [sym_source_type_no_address] = ACTIONS(1496),
    [sym_source_type_neuron_id] = ACTIONS(1496),
    [sym_source_type_ip_address] = ACTIONS(1496),
    [sym_source_type_axlink] = ACTIONS(1496),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1496),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1496),
    [sym_source_type_ip_socket_address] = ACTIONS(1496),
    [sym_source_type_rs232] = ACTIONS(1496),
    [sym_source_type_internal] = ACTIONS(1496),
    [sym_source_type_mac_address] = ACTIONS(1496),
    [sym_source_type_ipv4_port] = ACTIONS(1496),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1496),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1496),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1496),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1496),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1496),
    [sym_true] = ACTIONS(1496),
    [sym_false] = ACTIONS(1496),
    [sym_file_read_only] = ACTIONS(1496),
    [sym_file_rw_new] = ACTIONS(1496),
    [sym_file_rw_append] = ACTIONS(1496),
    [sym_ip_tcp] = ACTIONS(1496),
    [sym_ip_udp] = ACTIONS(1496),
    [sym_ip_udp_2way] = ACTIONS(1496),
    [sym_xml_encode_types] = ACTIONS(1496),
    [sym_xml_encode_char_as_list] = ACTIONS(1496),
    [sym_xml_encode_le] = ACTIONS(1496),
    [sym_xml_decode_types] = ACTIONS(1496),
    [sym_xml_decode_no_preserve] = ACTIONS(1496),
    [sym_duet_dev_type_display_device] = ACTIONS(1496),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1496),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1496),
    [sym_duet_dev_type_io_device] = ACTIONS(1496),
    [sym_duet_dev_type_relay_device] = ACTIONS(1496),
    [sym_duet_dev_type_ups] = ACTIONS(1496),
    [sym_duet_dev_type_amplifier] = ACTIONS(1496),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1496),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1496),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1496),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1496),
    [sym_duet_dev_type_camera] = ACTIONS(1496),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1496),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1496),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1496),
    [sym_duet_dev_type_dss] = ACTIONS(1496),
    [sym_duet_dev_type_dvr] = ACTIONS(1496),
    [sym_duet_dev_type_disc_device] = ACTIONS(1496),
    [sym_duet_dev_type_document_camera] = ACTIONS(1496),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1496),
    [sym_duet_dev_type_hvac] = ACTIONS(1496),
    [sym_duet_dev_type_keypad] = ACTIONS(1496),
    [sym_duet_dev_type_light] = ACTIONS(1496),
    [sym_duet_dev_type_monitor] = ACTIONS(1496),
    [sym_duet_dev_type_motor] = ACTIONS(1496),
    [sym_duet_dev_type_multi_window] = ACTIONS(1496),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1496),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1496),
    [sym_duet_dev_type_receiver] = ACTIONS(1496),
    [sym_duet_dev_type_security_system] = ACTIONS(1496),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1496),
    [sym_duet_dev_type_settop_box] = ACTIONS(1496),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1496),
    [sym_duet_dev_type_switcher] = ACTIONS(1496),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1496),
    [sym_duet_dev_type_tv] = ACTIONS(1496),
    [sym_duet_dev_type_utility] = ACTIONS(1496),
    [sym_duet_dev_type_vcr] = ACTIONS(1496),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1496),
    [sym_duet_dev_type_video_processor] = ACTIONS(1496),
    [sym_duet_dev_type_video_projector] = ACTIONS(1496),
    [sym_duet_dev_type_video_wall] = ACTIONS(1496),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1496),
    [sym_duet_dev_type_weather] = ACTIONS(1496),
    [sym_duet_dev_not_polled] = ACTIONS(1496),
    [sym_duet_dev_polled] = ACTIONS(1496),
    [sym_clkmgr_mode_network] = ACTIONS(1496),
    [sym_clkmgr_mode_standalone] = ACTIONS(1496),
    [sym_normal_standby] = ACTIONS(1496),
    [sym_normal_wake] = ACTIONS(1496),
    [sym_ip_multicast_ttl_option] = ACTIONS(1496),
    [sym_ip_ttl_subnet] = ACTIONS(1496),
    [sym_ip_ttl_site] = ACTIONS(1496),
    [sym_ip_ttl_region] = ACTIONS(1496),
    [sym_ip_ttl_continent] = ACTIONS(1496),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1496),
    [sym_ip_nodelay_on] = ACTIONS(1496),
    [sym_ip_nodelay_off] = ACTIONS(1496),
    [sym_amx_error] = ACTIONS(1496),
    [sym_amx_warning] = ACTIONS(1496),
    [sym_amx_info] = ACTIONS(1496),
    [sym_amx_debug] = ACTIONS(1496),
    [sym_netlinx_login_success] = ACTIONS(1496),
    [sym_netlinx_login_fail] = ACTIONS(1496),
    [sym_netlinx_logout] = ACTIONS(1496),
    [sym_null_str] = ACTIONS(1496),
    [sym_smtp_address] = ACTIONS(1496),
    [sym_smtp_port_number] = ACTIONS(1496),
    [sym_smtp_username] = ACTIONS(1496),
    [sym_smtp_password] = ACTIONS(1496),
    [sym_smtp_require_tls] = ACTIONS(1496),
    [sym_smtp_from] = ACTIONS(1496),
    [sym_smtp_tls_true] = ACTIONS(1496),
    [sym_smtp_tls_false] = ACTIONS(1496),
    [sym_listview_on_row_select_event] = ACTIONS(1496),
    [sym_data_structure_unknown] = ACTIONS(1496),
    [sym_data_structure_datafeed] = ACTIONS(1496),
    [sym_data_structure_datarecordset] = ACTIONS(1496),
    [sym_data_structure_datarecord] = ACTIONS(1496),
    [sym_data_structure_datafield] = ACTIONS(1496),
    [sym_data_max_content_field_count] = ACTIONS(1496),
    [sym_data_max_metadata_field_count] = ACTIONS(1496),
    [sym_data_max_id_length] = ACTIONS(1496),
    [sym_data_max_name_length] = ACTIONS(1496),
    [sym_data_max_description_length] = ACTIONS(1496),
    [sym_data_max_type_length] = ACTIONS(1496),
    [sym_data_max_format_length] = ACTIONS(1496),
    [sym_data_max_source_length] = ACTIONS(1496),
    [sym_data_max_value_length] = ACTIONS(1496),
    [sym_data_max_label_length] = ACTIONS(1496),
    [sym_data_max_url_length] = ACTIONS(1496),
    [sym_max_last_login_info_length] = ACTIONS(1496),
    [sym_data_type_unknown] = ACTIONS(1496),
    [sym_data_type_string] = ACTIONS(1496),
    [sym_data_type_datetime] = ACTIONS(1496),
    [sym_data_type_date] = ACTIONS(1496),
    [sym_data_type_time] = ACTIONS(1496),
    [sym_data_type_image] = ACTIONS(1496),
    [sym_data_format_url] = ACTIONS(1496),
    [sym_data_format_phone] = ACTIONS(1496),
    [sym_data_format_email] = ACTIONS(1496),
    [sym_data_format_iso8601] = ACTIONS(1496),
    [sym_auditlog_priv] = ACTIONS(1496),
    [sym_configuration_priv] = ACTIONS(1496),
    [sym_deviceconfig_priv] = ACTIONS(1496),
    [sym_ftp_priv] = ACTIONS(1496),
    [sym_http_priv] = ACTIONS(1496),
    [sym_networkconfig_priv] = ACTIONS(1496),
    [sym_programport_priv] = ACTIONS(1496),
    [sym_remoteui_priv] = ACTIONS(1496),
    [sym_securitycontrol_priv] = ACTIONS(1496),
    [sym_softwaremanagement_priv] = ACTIONS(1496),
    [sym_terminal_priv] = ACTIONS(1496),
    [sym_tpadmin_priv] = ACTIONS(1496),
    [sym_user1_priv] = ACTIONS(1496),
    [sym_user2_priv] = ACTIONS(1496),
    [sym_user3_priv] = ACTIONS(1496),
    [sym_user4_priv] = ACTIONS(1496),
    [sym_usermanagement_priv] = ACTIONS(1496),
    [sym_valid_account] = ACTIONS(1496),
    [sym_err_name_invalid] = ACTIONS(1496),
    [sym_err_password_invalid] = ACTIONS(1496),
    [sym_err_invalid_account] = ACTIONS(1496),
    [sym_err_no_authorization] = ACTIONS(1496),
    [sym_err_invalid_parameter] = ACTIONS(1496),
    [sym_err_user_locked_out] = ACTIONS(1496),
    [sym_err_user_account_expired] = ACTIONS(1496),
    [sym_err_general_error] = ACTIONS(1496),
    [sym_err_authentication_error] = ACTIONS(1496),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1496),
    [sym_tls_validate_certificate] = ACTIONS(1496),
    [sym_tdata] = ACTIONS(1496),
    [sym_tchannel] = ACTIONS(1496),
    [sym_tlevel] = ACTIONS(1496),
    [sym_tbutton] = ACTIONS(1496),
    [sym_ttimeline] = ACTIONS(1496),
    [sym_tcustom] = ACTIONS(1496),
    [sym_url_struct] = ACTIONS(1496),
    [sym_dns_struct] = ACTIONS(1496),
    [sym_ip_address_struct] = ACTIONS(1496),
    [sym_dev_info_struct] = ACTIONS(1496),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1496),
    [sym_data_feed] = ACTIONS(1496),
    [sym_data_field] = ACTIONS(1496),
    [sym_data_record] = ACTIONS(1496),
    [sym_wc_data_feed] = ACTIONS(1496),
    [sym_wc_data_field] = ACTIONS(1496),
    [sym_wc_data_record] = ACTIONS(1496),
    [sym_last_login_info] = ACTIONS(1496),
    [sym____reserved___] = ACTIONS(1496),
    [sym_date] = ACTIONS(1496),
    [sym_day] = ACTIONS(1496),
    [sym_time] = ACTIONS(1496),
    [sym_ldate] = ACTIONS(1496),
    [sym_push_device] = ACTIONS(1496),
    [sym_push_channel] = ACTIONS(1496),
    [sym_push_devchan] = ACTIONS(1496),
    [sym_release_device] = ACTIONS(1496),
    [sym_release_channel] = ACTIONS(1496),
    [sym_release_devchan] = ACTIONS(1496),
    [sym_master_slot] = ACTIONS(1496),
    [sym_get_pulse_time] = ACTIONS(1496),
    [sym_get_timer] = ACTIONS(1496),
    [sym_master_sn] = ACTIONS(1496),
    [sym_system_number] = ACTIONS(1496),
    [sym_button] = ACTIONS(1496),
    [sym_dv_channel] = ACTIONS(1496),
    [sym_data] = ACTIONS(1496),
    [sym_level] = ACTIONS(1496),
    [sym_timeline] = ACTIONS(1496),
    [sym_channel] = ACTIONS(1496),
    [sym_custom] = ACTIONS(1496),
    [sym_atoi] = ACTIONS(1496),
    [sym_atol] = ACTIONS(1496),
    [sym_atof] = ACTIONS(1496),
    [sym_itoa] = ACTIONS(1496),
    [sym_itohex] = ACTIONS(1496),
    [sym_hextoi] = ACTIONS(1496),
    [sym_ftoa] = ACTIONS(1496),
    [sym_find_string] = ACTIONS(1496),
    [sym_left_string] = ACTIONS(1496),
    [sym_right_string] = ACTIONS(1496),
    [sym_mid_string] = ACTIONS(1496),
    [sym_remove_string] = ACTIONS(1496),
    [sym_get_buffer_string] = ACTIONS(1496),
    [sym_get_buffer_char] = ACTIONS(1496),
    [sym_get_multi_buffer_string] = ACTIONS(1496),
    [sym_length_string] = ACTIONS(1496),
    [sym_max_length_string] = ACTIONS(1496),
    [sym_set_length_string] = ACTIONS(1496),
    [sym_compare_string] = ACTIONS(1496),
    [sym_upper_string] = ACTIONS(1496),
    [sym_lower_string] = ACTIONS(1496),
    [sym_get_last] = ACTIONS(1496),
    [sym_date_to_day] = ACTIONS(1496),
    [sym_date_to_month] = ACTIONS(1496),
    [sym_date_to_year] = ACTIONS(1496),
    [sym_time_to_hour] = ACTIONS(1496),
    [sym_time_to_minute] = ACTIONS(1496),
    [sym_time_to_second] = ACTIONS(1496),
    [sym_day_of_week] = ACTIONS(1496),
    [sym_set_pulse_time] = ACTIONS(1496),
    [sym_set_timer] = ACTIONS(1496),
    [sym_file_open] = ACTIONS(1496),
    [sym_file_close] = ACTIONS(1496),
    [sym_file_read] = ACTIONS(1496),
    [sym_file_read_line] = ACTIONS(1496),
    [sym_file_write] = ACTIONS(1496),
    [sym_file_write_line] = ACTIONS(1496),
    [sym_file_copy] = ACTIONS(1496),
    [sym_file_delete] = ACTIONS(1496),
    [sym_file_rename] = ACTIONS(1496),
    [sym_file_dir] = ACTIONS(1496),
    [sym_file_getdir] = ACTIONS(1496),
    [sym_file_setdir] = ACTIONS(1496),
    [sym_file_create_dir] = ACTIONS(1496),
    [sym_file_remove_dir] = ACTIONS(1496),
    [sym_file_seek] = ACTIONS(1496),
    [sym_length_array] = ACTIONS(1496),
    [sym_max_length_array] = ACTIONS(1496),
    [sym_set_length_array] = ACTIONS(1496),
    [sym_ip_client_open] = ACTIONS(1496),
    [sym_ip_client_close] = ACTIONS(1496),
    [sym_ip_server_open] = ACTIONS(1496),
    [sym_ip_server_close] = ACTIONS(1496),
    [sym_ip_bound_client_open] = ACTIONS(1496),
    [sym_ip_mc_server_open] = ACTIONS(1496),
    [sym_ip_set_option] = ACTIONS(1496),
    [sym_ssh_client_open] = ACTIONS(1496),
    [sym_ssh_client_close] = ACTIONS(1496),
    [sym_tls_client_open] = ACTIONS(1496),
    [sym_tls_client_close] = ACTIONS(1496),
    [sym_do_push] = ACTIONS(1496),
    [sym_do_push_timed] = ACTIONS(1496),
    [sym_do_release] = ACTIONS(1496),
    [sym_device_id] = ACTIONS(1496),
    [sym_device_info] = ACTIONS(1496),
    [sym_device_id_string] = ACTIONS(1496),
    [sym_device_standby] = ACTIONS(1496),
    [sym_device_wake] = ACTIONS(1496),
    [sym_timeline_create] = ACTIONS(1496),
    [sym_timeline_kill] = ACTIONS(1496),
    [sym_timeline_reload] = ACTIONS(1496),
    [sym_timeline_pause] = ACTIONS(1496),
    [sym_timeline_restart] = ACTIONS(1496),
    [sym_timeline_set] = ACTIONS(1496),
    [sym_timeline_get] = ACTIONS(1496),
    [sym_timeline_active] = ACTIONS(1496),
    [sym_timeline_dynamic_id] = ACTIONS(1496),
    [sym_variable_to_string] = ACTIONS(1496),
    [sym_string_to_variable] = ACTIONS(1496),
    [sym_length_variable_to_string] = ACTIONS(1496),
    [sym_type_cast] = ACTIONS(1496),
    [sym_variable_to_xml] = ACTIONS(1496),
    [sym_xml_to_variable] = ACTIONS(1496),
    [sym_length_variable_to_xml] = ACTIONS(1496),
    [sym_redirect_string] = ACTIONS(1496),
    [sym_random_number] = ACTIONS(1496),
    [sym_get_unique_id] = ACTIONS(1496),
    [sym_get_system_number] = ACTIONS(1496),
    [sym_set_system_number] = ACTIONS(1496),
    [sym_get_serial_number] = ACTIONS(1496),
    [sym_get_master_build] = ACTIONS(1496),
    [sym_set_validation_code] = ACTIONS(1496),
    [sym_reboot] = ACTIONS(1496),
    [sym_load_duet_module] = ACTIONS(1496),
    [sym_unload_duet_module] = ACTIONS(1496),
    [sym_duet_mem_size_set] = ACTIONS(1496),
    [sym_duet_mem_size_get] = ACTIONS(1496),
    [sym_dynamic_application_device] = ACTIONS(1496),
    [sym_dynamic_polled_port] = ACTIONS(1496),
    [sym_static_port_binding] = ACTIONS(1496),
    [sym_static_ip_binding] = ACTIONS(1496),
    [sym_get_dns_list] = ACTIONS(1496),
    [sym_set_dns_list] = ACTIONS(1496),
    [sym_get_ip_address] = ACTIONS(1496),
    [sym_set_ip_address] = ACTIONS(1496),
    [sym_get_url_list] = ACTIONS(1496),
    [sym_add_url_entry] = ACTIONS(1496),
    [sym_delete_url_entry] = ACTIONS(1496),
    [sym_abs_value] = ACTIONS(1496),
    [sym_max_value] = ACTIONS(1496),
    [sym_min_value] = ACTIONS(1496),
    [sym_exp_value] = ACTIONS(1496),
    [sym_log_value] = ACTIONS(1496),
    [sym_log10_value] = ACTIONS(1496),
    [sym_power_value] = ACTIONS(1496),
    [sym_sqrt_value] = ACTIONS(1496),
    [sym_raw_be] = ACTIONS(1496),
    [sym_raw_le] = ACTIONS(1496),
    [sym_internal_queue_size_set] = ACTIONS(1496),
    [sym_internal_queue_size_get] = ACTIONS(1496),
    [sym_internal_threshold_set] = ACTIONS(1496),
    [sym_internal_threshold_get] = ACTIONS(1496),
    [sym_rebuilt_event] = ACTIONS(1496),
    [sym_do_custom_event] = ACTIONS(1496),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1496),
    [sym_clkmgr_set_clk_source] = ACTIONS(1496),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1496),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1496),
    [sym_clkmgr_get_timezone] = ACTIONS(1496),
    [sym_clkmgr_set_timezone] = ACTIONS(1496),
    [sym_clkmgr_get_resync_period] = ACTIONS(1496),
    [sym_clkmgr_set_resync_period] = ACTIONS(1496),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1496),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1496),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1496),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1496),
    [sym_clkmgr_get_timeservers] = ACTIONS(1496),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1496),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1496),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1496),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1496),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1496),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1496),
    [sym_set_log_level] = ACTIONS(1496),
    [sym_get_log_level] = ACTIONS(1496),
    [sym_amx_log] = ACTIONS(1496),
    [sym_smtp_server_config_set] = ACTIONS(1496),
    [sym_smtp_server_config_get] = ACTIONS(1496),
    [sym_smtp_send] = ACTIONS(1496),
    [sym_validate_netlinx_account] = ACTIONS(1496),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1496),
    [sym_authenticate_certificate] = ACTIONS(1496),
    [sym_audit_netlinx_session_event] = ACTIONS(1496),
    [sym_audit_netlinx_generic_event] = ACTIONS(1496),
    [sym_get_available_flash_disk_space] = ACTIONS(1496),
    [sym_get_max_flash_disk_space] = ACTIONS(1496),
    [sym_data_create_feed] = ACTIONS(1496),
    [sym_data_delete_feed] = ACTIONS(1496),
    [sym_data_publish_feed] = ACTIONS(1496),
    [sym_data_get_published_feed] = ACTIONS(1496),
    [sym_data_add_record] = ACTIONS(1496),
    [sym_data_get_event_record] = ACTIONS(1496),
    [sym__wc_data_create_feed] = ACTIONS(1496),
    [sym__wc_data_add_record] = ACTIONS(1496),
    [sym__wc_data_get_event_record] = ACTIONS(1496),
    [sym_format] = ACTIONS(1496),
    [sym_astro_clock] = ACTIONS(1496),
    [sym_set_outdoor_temperature] = ACTIONS(1496),
    [sym_set_virtual_level_count] = ACTIONS(1496),
    [sym_set_virtual_channel_count] = ACTIONS(1496),
    [sym_set_virtual_port_count] = ACTIONS(1496),
    [sym__wc_encode_int] = ACTIONS(1496),
    [sym__wc_decode_int] = ACTIONS(1496),
    [sym__wc_int] = ACTIONS(1496),
    [sym__wc_to_ch_int] = ACTIONS(1496),
    [sym__ch_to_wc_int] = ACTIONS(1496),
    [sym__wc_find_string_int] = ACTIONS(1496),
    [sym__wc_left_string_int] = ACTIONS(1496),
    [sym__wc_lower_string_int] = ACTIONS(1496),
    [sym__wc_mid_string_int] = ACTIONS(1496),
    [sym__wc_remove_string_int] = ACTIONS(1496),
    [sym__wc_right_string_int] = ACTIONS(1496),
    [sym__wc_upper_string_int] = ACTIONS(1496),
    [sym__wc_compare_string_int] = ACTIONS(1496),
    [sym__wc_get_buffer_char_int] = ACTIONS(1496),
    [sym__wc_get_buffer_string_int] = ACTIONS(1496),
    [sym__wc_concat_string_int] = ACTIONS(1496),
    [sym__wc_file_open_int] = ACTIONS(1496),
    [sym__wc_file_close_int] = ACTIONS(1496),
    [sym__wc_file_read_int] = ACTIONS(1496),
    [sym__wc_file_read_line_int] = ACTIONS(1496),
    [sym__wc_file_write_int] = ACTIONS(1496),
    [sym__wc_file_write_line_int] = ACTIONS(1496),
    [anon_sym_EQ] = ACTIONS(1496),
    [aux_sym_preproc_include_token1] = ACTIONS(1496),
    [aux_sym_preproc_define_token1] = ACTIONS(1498),
    [aux_sym_preproc_warn_token1] = ACTIONS(1498),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1498),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1498),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1498),
    [anon_sym_LBRACK] = ACTIONS(1653),
    [anon_sym_LBRACE] = ACTIONS(1498),
    [anon_sym_RBRACE] = ACTIONS(1498),
    [anon_sym_COLON] = ACTIONS(1498),
    [anon_sym_LPAREN] = ACTIONS(1496),
    [anon_sym_SEMI] = ACTIONS(1498),
    [anon_sym_BANG] = ACTIONS(1496),
    [anon_sym_TILDE] = ACTIONS(1498),
    [anon_sym_DASH] = ACTIONS(1496),
    [anon_sym_PLUS] = ACTIONS(1496),
    [anon_sym_STAR] = ACTIONS(1498),
    [anon_sym_SLASH] = ACTIONS(1496),
    [anon_sym_PERCENT] = ACTIONS(1498),
    [anon_sym_PIPE_PIPE] = ACTIONS(1498),
    [anon_sym_AMP_AMP] = ACTIONS(1498),
    [anon_sym_PIPE] = ACTIONS(1496),
    [anon_sym_CARET] = ACTIONS(1498),
    [anon_sym_AMP] = ACTIONS(1496),
    [anon_sym_EQ_EQ] = ACTIONS(1498),
    [anon_sym_BANG_EQ] = ACTIONS(1498),
    [anon_sym_LT_GT] = ACTIONS(1498),
    [anon_sym_GT] = ACTIONS(1496),
    [anon_sym_GT_EQ] = ACTIONS(1498),
    [anon_sym_LT_EQ] = ACTIONS(1498),
    [anon_sym_LT] = ACTIONS(1496),
    [anon_sym_LT_LT] = ACTIONS(1498),
    [anon_sym_GT_GT] = ACTIONS(1498),
    [anon_sym_DASH_DASH] = ACTIONS(1498),
    [anon_sym_PLUS_PLUS] = ACTIONS(1498),
    [anon_sym_DOT] = ACTIONS(1496),
    [anon_sym_DQUOTE] = ACTIONS(1498),
    [anon_sym_SQUOTE] = ACTIONS(1498),
    [sym_number_literal] = ACTIONS(1498),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1498),
  },
  [STATE(156)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1503),
    [sym_define_call_keyword] = ACTIONS(1503),
    [sym_define_function_keyword] = ACTIONS(1503),
    [sym_define_library_function_keyword] = ACTIONS(1503),
    [sym_define_module_keyword] = ACTIONS(1503),
    [sym_char_keyword] = ACTIONS(1503),
    [sym_widechar_keyword] = ACTIONS(1503),
    [sym_integer_keyword] = ACTIONS(1503),
    [sym_sinteger_keyword] = ACTIONS(1503),
    [sym_long_keyword] = ACTIONS(1503),
    [sym_slong_keyword] = ACTIONS(1503),
    [sym_float_keyword] = ACTIONS(1503),
    [sym_double_keyword] = ACTIONS(1503),
    [sym_variant_keyword] = ACTIONS(1503),
    [sym_variantarray_keyword] = ACTIONS(1503),
    [sym_local_var_keyword] = ACTIONS(1503),
    [sym_stack_var_keyword] = ACTIONS(1503),
    [sym_constant_keyword] = ACTIONS(1503),
    [sym_volatile_keyword] = ACTIONS(1503),
    [sym_non_volatile_keyword] = ACTIONS(1503),
    [sym_persistent_keyword] = ACTIONS(1503),
    [sym_dev_keyword] = ACTIONS(1503),
    [sym_devlev_keyword] = ACTIONS(1503),
    [sym_devchan_keyword] = ACTIONS(1503),
    [sym_if_keyword] = ACTIONS(1503),
    [sym_else_keyword] = ACTIONS(1503),
    [sym_switch_keyword] = ACTIONS(1503),
    [sym_case_keyword] = ACTIONS(1503),
    [sym_default_keyword] = ACTIONS(1503),
    [sym_while_keyword] = ACTIONS(1503),
    [sym_for_keyword] = ACTIONS(1503),
    [sym_break_keyword] = ACTIONS(1503),
    [sym_continue_keyword] = ACTIONS(1503),
    [sym_return_keyword] = ACTIONS(1503),
    [sym_select_keyword] = ACTIONS(1503),
    [sym_active_keyword] = ACTIONS(1503),
    [sym_struct_keyword] = ACTIONS(1503),
    [sym_structure_keyword] = ACTIONS(1503),
    [aux_sym_on_keyword_token1] = ACTIONS(1503),
    [aux_sym_off_keyword_token1] = ACTIONS(1503),
    [sym_send_level_keyword] = ACTIONS(1503),
    [sym_send_string_keyword] = ACTIONS(1503),
    [sym_send_command_keyword] = ACTIONS(1503),
    [sym_clear_buffer_keyword] = ACTIONS(1503),
    [sym_create_buffer_keyword] = ACTIONS(1503),
    [sym_create_multi_buffer_keyword] = ACTIONS(1503),
    [sym_call_keyword] = ACTIONS(1503),
    [sym_system_call_keyword] = ACTIONS(1503),
    [sym_devchan_to_keyword] = ACTIONS(1503),
    [sym_devchan_min_to_keyword] = ACTIONS(1503),
    [sym_devchan_total_off_keyword] = ACTIONS(1503),
    [sym_devchan_pulse_keyword] = ACTIONS(1503),
    [sym_wait_keyword] = ACTIONS(1503),
    [sym_cancel_wait_keyword] = ACTIONS(1503),
    [sym_wait_until_keyword] = ACTIONS(1503),
    [sym_cancel_wait_until_keyword] = ACTIONS(1503),
    [sym_cancel_all_wait_keyword] = ACTIONS(1503),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1503),
    [sym_and] = ACTIONS(1605),
    [sym_or] = ACTIONS(1607),
    [sym_not] = ACTIONS(1503),
    [sym_xor] = ACTIONS(1609),
    [sym_band] = ACTIONS(1611),
    [sym_bor] = ACTIONS(1613),
    [sym_bxor] = ACTIONS(1609),
    [sym_bnot] = ACTIONS(1503),
    [sym_lshift] = ACTIONS(1615),
    [sym_rshift] = ACTIONS(1615),
    [sym___file__] = ACTIONS(1503),
    [sym___date__] = ACTIONS(1503),
    [sym___ldate__] = ACTIONS(1503),
    [sym___time__] = ACTIONS(1503),
    [sym___line__] = ACTIONS(1503),
    [sym___name__] = ACTIONS(1503),
    [sym___netlinx__] = ACTIONS(1503),
    [sym_netlinx_axi_version] = ACTIONS(1503),
    [sym_url_flg_tcp] = ACTIONS(1503),
    [sym_url_flg_accinfopresent] = ACTIONS(1503),
    [sym_url_flg_temp] = ACTIONS(1503),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1503),
    [sym_url_flg_stat_mask] = ACTIONS(1503),
    [sym_url_flg_stat_lookup] = ACTIONS(1503),
    [sym_url_flg_stat_connecting] = ACTIONS(1503),
    [sym_url_flg_stat_waiting] = ACTIONS(1503),
    [sym_url_flg_stat_connected] = ACTIONS(1503),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1503),
    [sym_first_virtual_device] = ACTIONS(1503),
    [sym_first_local_port] = ACTIONS(1503),
    [sym_dynamic_virtual_device] = ACTIONS(1503),
    [sym_timeline_once] = ACTIONS(1503),
    [sym_timeline_repeat] = ACTIONS(1503),
    [sym_timeline_absolute] = ACTIONS(1503),
    [sym_timeline_relative] = ACTIONS(1503),
    [sym_do_push_timed_infinite] = ACTIONS(1503),
    [sym_source_type_no_address] = ACTIONS(1503),
    [sym_source_type_neuron_id] = ACTIONS(1503),
    [sym_source_type_ip_address] = ACTIONS(1503),
    [sym_source_type_axlink] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1503),
    [sym_source_type_ip_socket_address] = ACTIONS(1503),
    [sym_source_type_rs232] = ACTIONS(1503),
    [sym_source_type_internal] = ACTIONS(1503),
    [sym_source_type_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1503),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1503),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1503),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1503),
    [sym_true] = ACTIONS(1503),
    [sym_false] = ACTIONS(1503),
    [sym_file_read_only] = ACTIONS(1503),
    [sym_file_rw_new] = ACTIONS(1503),
    [sym_file_rw_append] = ACTIONS(1503),
    [sym_ip_tcp] = ACTIONS(1503),
    [sym_ip_udp] = ACTIONS(1503),
    [sym_ip_udp_2way] = ACTIONS(1503),
    [sym_xml_encode_types] = ACTIONS(1503),
    [sym_xml_encode_char_as_list] = ACTIONS(1503),
    [sym_xml_encode_le] = ACTIONS(1503),
    [sym_xml_decode_types] = ACTIONS(1503),
    [sym_xml_decode_no_preserve] = ACTIONS(1503),
    [sym_duet_dev_type_display_device] = ACTIONS(1503),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_io_device] = ACTIONS(1503),
    [sym_duet_dev_type_relay_device] = ACTIONS(1503),
    [sym_duet_dev_type_ups] = ACTIONS(1503),
    [sym_duet_dev_type_amplifier] = ACTIONS(1503),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1503),
    [sym_duet_dev_type_camera] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1503),
    [sym_duet_dev_type_dss] = ACTIONS(1503),
    [sym_duet_dev_type_dvr] = ACTIONS(1503),
    [sym_duet_dev_type_disc_device] = ACTIONS(1503),
    [sym_duet_dev_type_document_camera] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1503),
    [sym_duet_dev_type_hvac] = ACTIONS(1503),
    [sym_duet_dev_type_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_light] = ACTIONS(1503),
    [sym_duet_dev_type_monitor] = ACTIONS(1503),
    [sym_duet_dev_type_motor] = ACTIONS(1503),
    [sym_duet_dev_type_multi_window] = ACTIONS(1503),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1503),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1503),
    [sym_duet_dev_type_receiver] = ACTIONS(1503),
    [sym_duet_dev_type_security_system] = ACTIONS(1503),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1503),
    [sym_duet_dev_type_settop_box] = ACTIONS(1503),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1503),
    [sym_duet_dev_type_switcher] = ACTIONS(1503),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_tv] = ACTIONS(1503),
    [sym_duet_dev_type_utility] = ACTIONS(1503),
    [sym_duet_dev_type_vcr] = ACTIONS(1503),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_video_processor] = ACTIONS(1503),
    [sym_duet_dev_type_video_projector] = ACTIONS(1503),
    [sym_duet_dev_type_video_wall] = ACTIONS(1503),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1503),
    [sym_duet_dev_type_weather] = ACTIONS(1503),
    [sym_duet_dev_not_polled] = ACTIONS(1503),
    [sym_duet_dev_polled] = ACTIONS(1503),
    [sym_clkmgr_mode_network] = ACTIONS(1503),
    [sym_clkmgr_mode_standalone] = ACTIONS(1503),
    [sym_normal_standby] = ACTIONS(1503),
    [sym_normal_wake] = ACTIONS(1503),
    [sym_ip_multicast_ttl_option] = ACTIONS(1503),
    [sym_ip_ttl_subnet] = ACTIONS(1503),
    [sym_ip_ttl_site] = ACTIONS(1503),
    [sym_ip_ttl_region] = ACTIONS(1503),
    [sym_ip_ttl_continent] = ACTIONS(1503),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1503),
    [sym_ip_nodelay_on] = ACTIONS(1503),
    [sym_ip_nodelay_off] = ACTIONS(1503),
    [sym_amx_error] = ACTIONS(1503),
    [sym_amx_warning] = ACTIONS(1503),
    [sym_amx_info] = ACTIONS(1503),
    [sym_amx_debug] = ACTIONS(1503),
    [sym_netlinx_login_success] = ACTIONS(1503),
    [sym_netlinx_login_fail] = ACTIONS(1503),
    [sym_netlinx_logout] = ACTIONS(1503),
    [sym_null_str] = ACTIONS(1503),
    [sym_smtp_address] = ACTIONS(1503),
    [sym_smtp_port_number] = ACTIONS(1503),
    [sym_smtp_username] = ACTIONS(1503),
    [sym_smtp_password] = ACTIONS(1503),
    [sym_smtp_require_tls] = ACTIONS(1503),
    [sym_smtp_from] = ACTIONS(1503),
    [sym_smtp_tls_true] = ACTIONS(1503),
    [sym_smtp_tls_false] = ACTIONS(1503),
    [sym_listview_on_row_select_event] = ACTIONS(1503),
    [sym_data_structure_unknown] = ACTIONS(1503),
    [sym_data_structure_datafeed] = ACTIONS(1503),
    [sym_data_structure_datarecordset] = ACTIONS(1503),
    [sym_data_structure_datarecord] = ACTIONS(1503),
    [sym_data_structure_datafield] = ACTIONS(1503),
    [sym_data_max_content_field_count] = ACTIONS(1503),
    [sym_data_max_metadata_field_count] = ACTIONS(1503),
    [sym_data_max_id_length] = ACTIONS(1503),
    [sym_data_max_name_length] = ACTIONS(1503),
    [sym_data_max_description_length] = ACTIONS(1503),
    [sym_data_max_type_length] = ACTIONS(1503),
    [sym_data_max_format_length] = ACTIONS(1503),
    [sym_data_max_source_length] = ACTIONS(1503),
    [sym_data_max_value_length] = ACTIONS(1503),
    [sym_data_max_label_length] = ACTIONS(1503),
    [sym_data_max_url_length] = ACTIONS(1503),
    [sym_max_last_login_info_length] = ACTIONS(1503),
    [sym_data_type_unknown] = ACTIONS(1503),
    [sym_data_type_string] = ACTIONS(1503),
    [sym_data_type_datetime] = ACTIONS(1503),
    [sym_data_type_date] = ACTIONS(1503),
    [sym_data_type_time] = ACTIONS(1503),
    [sym_data_type_image] = ACTIONS(1503),
    [sym_data_format_url] = ACTIONS(1503),
    [sym_data_format_phone] = ACTIONS(1503),
    [sym_data_format_email] = ACTIONS(1503),
    [sym_data_format_iso8601] = ACTIONS(1503),
    [sym_auditlog_priv] = ACTIONS(1503),
    [sym_configuration_priv] = ACTIONS(1503),
    [sym_deviceconfig_priv] = ACTIONS(1503),
    [sym_ftp_priv] = ACTIONS(1503),
    [sym_http_priv] = ACTIONS(1503),
    [sym_networkconfig_priv] = ACTIONS(1503),
    [sym_programport_priv] = ACTIONS(1503),
    [sym_remoteui_priv] = ACTIONS(1503),
    [sym_securitycontrol_priv] = ACTIONS(1503),
    [sym_softwaremanagement_priv] = ACTIONS(1503),
    [sym_terminal_priv] = ACTIONS(1503),
    [sym_tpadmin_priv] = ACTIONS(1503),
    [sym_user1_priv] = ACTIONS(1503),
    [sym_user2_priv] = ACTIONS(1503),
    [sym_user3_priv] = ACTIONS(1503),
    [sym_user4_priv] = ACTIONS(1503),
    [sym_usermanagement_priv] = ACTIONS(1503),
    [sym_valid_account] = ACTIONS(1503),
    [sym_err_name_invalid] = ACTIONS(1503),
    [sym_err_password_invalid] = ACTIONS(1503),
    [sym_err_invalid_account] = ACTIONS(1503),
    [sym_err_no_authorization] = ACTIONS(1503),
    [sym_err_invalid_parameter] = ACTIONS(1503),
    [sym_err_user_locked_out] = ACTIONS(1503),
    [sym_err_user_account_expired] = ACTIONS(1503),
    [sym_err_general_error] = ACTIONS(1503),
    [sym_err_authentication_error] = ACTIONS(1503),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1503),
    [sym_tls_validate_certificate] = ACTIONS(1503),
    [sym_tdata] = ACTIONS(1503),
    [sym_tchannel] = ACTIONS(1503),
    [sym_tlevel] = ACTIONS(1503),
    [sym_tbutton] = ACTIONS(1503),
    [sym_ttimeline] = ACTIONS(1503),
    [sym_tcustom] = ACTIONS(1503),
    [sym_url_struct] = ACTIONS(1503),
    [sym_dns_struct] = ACTIONS(1503),
    [sym_ip_address_struct] = ACTIONS(1503),
    [sym_dev_info_struct] = ACTIONS(1503),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1503),
    [sym_data_feed] = ACTIONS(1503),
    [sym_data_field] = ACTIONS(1503),
    [sym_data_record] = ACTIONS(1503),
    [sym_wc_data_feed] = ACTIONS(1503),
    [sym_wc_data_field] = ACTIONS(1503),
    [sym_wc_data_record] = ACTIONS(1503),
    [sym_last_login_info] = ACTIONS(1503),
    [sym____reserved___] = ACTIONS(1503),
    [sym_date] = ACTIONS(1503),
    [sym_day] = ACTIONS(1503),
    [sym_time] = ACTIONS(1503),
    [sym_ldate] = ACTIONS(1503),
    [sym_push_device] = ACTIONS(1503),
    [sym_push_channel] = ACTIONS(1503),
    [sym_push_devchan] = ACTIONS(1503),
    [sym_release_device] = ACTIONS(1503),
    [sym_release_channel] = ACTIONS(1503),
    [sym_release_devchan] = ACTIONS(1503),
    [sym_master_slot] = ACTIONS(1503),
    [sym_get_pulse_time] = ACTIONS(1503),
    [sym_get_timer] = ACTIONS(1503),
    [sym_master_sn] = ACTIONS(1503),
    [sym_system_number] = ACTIONS(1503),
    [sym_button] = ACTIONS(1503),
    [sym_dv_channel] = ACTIONS(1503),
    [sym_data] = ACTIONS(1503),
    [sym_level] = ACTIONS(1503),
    [sym_timeline] = ACTIONS(1503),
    [sym_channel] = ACTIONS(1503),
    [sym_custom] = ACTIONS(1503),
    [sym_atoi] = ACTIONS(1503),
    [sym_atol] = ACTIONS(1503),
    [sym_atof] = ACTIONS(1503),
    [sym_itoa] = ACTIONS(1503),
    [sym_itohex] = ACTIONS(1503),
    [sym_hextoi] = ACTIONS(1503),
    [sym_ftoa] = ACTIONS(1503),
    [sym_find_string] = ACTIONS(1503),
    [sym_left_string] = ACTIONS(1503),
    [sym_right_string] = ACTIONS(1503),
    [sym_mid_string] = ACTIONS(1503),
    [sym_remove_string] = ACTIONS(1503),
    [sym_get_buffer_string] = ACTIONS(1503),
    [sym_get_buffer_char] = ACTIONS(1503),
    [sym_get_multi_buffer_string] = ACTIONS(1503),
    [sym_length_string] = ACTIONS(1503),
    [sym_max_length_string] = ACTIONS(1503),
    [sym_set_length_string] = ACTIONS(1503),
    [sym_compare_string] = ACTIONS(1503),
    [sym_upper_string] = ACTIONS(1503),
    [sym_lower_string] = ACTIONS(1503),
    [sym_get_last] = ACTIONS(1503),
    [sym_date_to_day] = ACTIONS(1503),
    [sym_date_to_month] = ACTIONS(1503),
    [sym_date_to_year] = ACTIONS(1503),
    [sym_time_to_hour] = ACTIONS(1503),
    [sym_time_to_minute] = ACTIONS(1503),
    [sym_time_to_second] = ACTIONS(1503),
    [sym_day_of_week] = ACTIONS(1503),
    [sym_set_pulse_time] = ACTIONS(1503),
    [sym_set_timer] = ACTIONS(1503),
    [sym_file_open] = ACTIONS(1503),
    [sym_file_close] = ACTIONS(1503),
    [sym_file_read] = ACTIONS(1503),
    [sym_file_read_line] = ACTIONS(1503),
    [sym_file_write] = ACTIONS(1503),
    [sym_file_write_line] = ACTIONS(1503),
    [sym_file_copy] = ACTIONS(1503),
    [sym_file_delete] = ACTIONS(1503),
    [sym_file_rename] = ACTIONS(1503),
    [sym_file_dir] = ACTIONS(1503),
    [sym_file_getdir] = ACTIONS(1503),
    [sym_file_setdir] = ACTIONS(1503),
    [sym_file_create_dir] = ACTIONS(1503),
    [sym_file_remove_dir] = ACTIONS(1503),
    [sym_file_seek] = ACTIONS(1503),
    [sym_length_array] = ACTIONS(1503),
    [sym_max_length_array] = ACTIONS(1503),
    [sym_set_length_array] = ACTIONS(1503),
    [sym_ip_client_open] = ACTIONS(1503),
    [sym_ip_client_close] = ACTIONS(1503),
    [sym_ip_server_open] = ACTIONS(1503),
    [sym_ip_server_close] = ACTIONS(1503),
    [sym_ip_bound_client_open] = ACTIONS(1503),
    [sym_ip_mc_server_open] = ACTIONS(1503),
    [sym_ip_set_option] = ACTIONS(1503),
    [sym_ssh_client_open] = ACTIONS(1503),
    [sym_ssh_client_close] = ACTIONS(1503),
    [sym_tls_client_open] = ACTIONS(1503),
    [sym_tls_client_close] = ACTIONS(1503),
    [sym_do_push] = ACTIONS(1503),
    [sym_do_push_timed] = ACTIONS(1503),
    [sym_do_release] = ACTIONS(1503),
    [sym_device_id] = ACTIONS(1503),
    [sym_device_info] = ACTIONS(1503),
    [sym_device_id_string] = ACTIONS(1503),
    [sym_device_standby] = ACTIONS(1503),
    [sym_device_wake] = ACTIONS(1503),
    [sym_timeline_create] = ACTIONS(1503),
    [sym_timeline_kill] = ACTIONS(1503),
    [sym_timeline_reload] = ACTIONS(1503),
    [sym_timeline_pause] = ACTIONS(1503),
    [sym_timeline_restart] = ACTIONS(1503),
    [sym_timeline_set] = ACTIONS(1503),
    [sym_timeline_get] = ACTIONS(1503),
    [sym_timeline_active] = ACTIONS(1503),
    [sym_timeline_dynamic_id] = ACTIONS(1503),
    [sym_variable_to_string] = ACTIONS(1503),
    [sym_string_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_string] = ACTIONS(1503),
    [sym_type_cast] = ACTIONS(1503),
    [sym_variable_to_xml] = ACTIONS(1503),
    [sym_xml_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_xml] = ACTIONS(1503),
    [sym_redirect_string] = ACTIONS(1503),
    [sym_random_number] = ACTIONS(1503),
    [sym_get_unique_id] = ACTIONS(1503),
    [sym_get_system_number] = ACTIONS(1503),
    [sym_set_system_number] = ACTIONS(1503),
    [sym_get_serial_number] = ACTIONS(1503),
    [sym_get_master_build] = ACTIONS(1503),
    [sym_set_validation_code] = ACTIONS(1503),
    [sym_reboot] = ACTIONS(1503),
    [sym_load_duet_module] = ACTIONS(1503),
    [sym_unload_duet_module] = ACTIONS(1503),
    [sym_duet_mem_size_set] = ACTIONS(1503),
    [sym_duet_mem_size_get] = ACTIONS(1503),
    [sym_dynamic_application_device] = ACTIONS(1503),
    [sym_dynamic_polled_port] = ACTIONS(1503),
    [sym_static_port_binding] = ACTIONS(1503),
    [sym_static_ip_binding] = ACTIONS(1503),
    [sym_get_dns_list] = ACTIONS(1503),
    [sym_set_dns_list] = ACTIONS(1503),
    [sym_get_ip_address] = ACTIONS(1503),
    [sym_set_ip_address] = ACTIONS(1503),
    [sym_get_url_list] = ACTIONS(1503),
    [sym_add_url_entry] = ACTIONS(1503),
    [sym_delete_url_entry] = ACTIONS(1503),
    [sym_abs_value] = ACTIONS(1503),
    [sym_max_value] = ACTIONS(1503),
    [sym_min_value] = ACTIONS(1503),
    [sym_exp_value] = ACTIONS(1503),
    [sym_log_value] = ACTIONS(1503),
    [sym_log10_value] = ACTIONS(1503),
    [sym_power_value] = ACTIONS(1503),
    [sym_sqrt_value] = ACTIONS(1503),
    [sym_raw_be] = ACTIONS(1503),
    [sym_raw_le] = ACTIONS(1503),
    [sym_internal_queue_size_set] = ACTIONS(1503),
    [sym_internal_queue_size_get] = ACTIONS(1503),
    [sym_internal_threshold_set] = ACTIONS(1503),
    [sym_internal_threshold_get] = ACTIONS(1503),
    [sym_rebuilt_event] = ACTIONS(1503),
    [sym_do_custom_event] = ACTIONS(1503),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1503),
    [sym_clkmgr_set_clk_source] = ACTIONS(1503),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1503),
    [sym_clkmgr_get_timezone] = ACTIONS(1503),
    [sym_clkmgr_set_timezone] = ACTIONS(1503),
    [sym_clkmgr_get_resync_period] = ACTIONS(1503),
    [sym_clkmgr_set_resync_period] = ACTIONS(1503),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_timeservers] = ACTIONS(1503),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_set_log_level] = ACTIONS(1503),
    [sym_get_log_level] = ACTIONS(1503),
    [sym_amx_log] = ACTIONS(1503),
    [sym_smtp_server_config_set] = ACTIONS(1503),
    [sym_smtp_server_config_get] = ACTIONS(1503),
    [sym_smtp_send] = ACTIONS(1503),
    [sym_validate_netlinx_account] = ACTIONS(1503),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1503),
    [sym_authenticate_certificate] = ACTIONS(1503),
    [sym_audit_netlinx_session_event] = ACTIONS(1503),
    [sym_audit_netlinx_generic_event] = ACTIONS(1503),
    [sym_get_available_flash_disk_space] = ACTIONS(1503),
    [sym_get_max_flash_disk_space] = ACTIONS(1503),
    [sym_data_create_feed] = ACTIONS(1503),
    [sym_data_delete_feed] = ACTIONS(1503),
    [sym_data_publish_feed] = ACTIONS(1503),
    [sym_data_get_published_feed] = ACTIONS(1503),
    [sym_data_add_record] = ACTIONS(1503),
    [sym_data_get_event_record] = ACTIONS(1503),
    [sym__wc_data_create_feed] = ACTIONS(1503),
    [sym__wc_data_add_record] = ACTIONS(1503),
    [sym__wc_data_get_event_record] = ACTIONS(1503),
    [sym_format] = ACTIONS(1503),
    [sym_astro_clock] = ACTIONS(1503),
    [sym_set_outdoor_temperature] = ACTIONS(1503),
    [sym_set_virtual_level_count] = ACTIONS(1503),
    [sym_set_virtual_channel_count] = ACTIONS(1503),
    [sym_set_virtual_port_count] = ACTIONS(1503),
    [sym__wc_encode_int] = ACTIONS(1503),
    [sym__wc_decode_int] = ACTIONS(1503),
    [sym__wc_int] = ACTIONS(1503),
    [sym__wc_to_ch_int] = ACTIONS(1503),
    [sym__ch_to_wc_int] = ACTIONS(1503),
    [sym__wc_find_string_int] = ACTIONS(1503),
    [sym__wc_left_string_int] = ACTIONS(1503),
    [sym__wc_lower_string_int] = ACTIONS(1503),
    [sym__wc_mid_string_int] = ACTIONS(1503),
    [sym__wc_remove_string_int] = ACTIONS(1503),
    [sym__wc_right_string_int] = ACTIONS(1503),
    [sym__wc_upper_string_int] = ACTIONS(1503),
    [sym__wc_compare_string_int] = ACTIONS(1503),
    [sym__wc_get_buffer_char_int] = ACTIONS(1503),
    [sym__wc_get_buffer_string_int] = ACTIONS(1503),
    [sym__wc_concat_string_int] = ACTIONS(1503),
    [sym__wc_file_open_int] = ACTIONS(1503),
    [sym__wc_file_close_int] = ACTIONS(1503),
    [sym__wc_file_read_int] = ACTIONS(1503),
    [sym__wc_file_read_line_int] = ACTIONS(1503),
    [sym__wc_file_write_int] = ACTIONS(1503),
    [sym__wc_file_write_line_int] = ACTIONS(1503),
    [aux_sym_preproc_include_token1] = ACTIONS(1503),
    [aux_sym_preproc_define_token1] = ACTIONS(1505),
    [aux_sym_preproc_warn_token1] = ACTIONS(1505),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1505),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1505),
    [anon_sym_RBRACE] = ACTIONS(1505),
    [anon_sym_COLON] = ACTIONS(1619),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1505),
    [anon_sym_BANG] = ACTIONS(1503),
    [anon_sym_TILDE] = ACTIONS(1505),
    [anon_sym_DASH] = ACTIONS(1625),
    [anon_sym_PLUS] = ACTIONS(1625),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1631),
    [anon_sym_AMP_AMP] = ACTIONS(1633),
    [anon_sym_PIPE] = ACTIONS(1613),
    [anon_sym_CARET] = ACTIONS(1635),
    [anon_sym_AMP] = ACTIONS(1611),
    [anon_sym_EQ_EQ] = ACTIONS(1637),
    [anon_sym_BANG_EQ] = ACTIONS(1637),
    [anon_sym_LT_GT] = ACTIONS(1637),
    [anon_sym_GT] = ACTIONS(1639),
    [anon_sym_GT_EQ] = ACTIONS(1641),
    [anon_sym_LT_EQ] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1639),
    [anon_sym_LT_LT] = ACTIONS(1643),
    [anon_sym_GT_GT] = ACTIONS(1643),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1505),
    [anon_sym_SQUOTE] = ACTIONS(1505),
    [sym_number_literal] = ACTIONS(1505),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1505),
  },
  [STATE(157)] = {
    [sym_identifier] = ACTIONS(1507),
    [sym_define_call_keyword] = ACTIONS(1507),
    [sym_define_function_keyword] = ACTIONS(1507),
    [sym_define_library_function_keyword] = ACTIONS(1507),
    [sym_define_module_keyword] = ACTIONS(1507),
    [sym_char_keyword] = ACTIONS(1507),
    [sym_widechar_keyword] = ACTIONS(1507),
    [sym_integer_keyword] = ACTIONS(1507),
    [sym_sinteger_keyword] = ACTIONS(1507),
    [sym_long_keyword] = ACTIONS(1507),
    [sym_slong_keyword] = ACTIONS(1507),
    [sym_float_keyword] = ACTIONS(1507),
    [sym_double_keyword] = ACTIONS(1507),
    [sym_variant_keyword] = ACTIONS(1507),
    [sym_variantarray_keyword] = ACTIONS(1507),
    [sym_local_var_keyword] = ACTIONS(1507),
    [sym_stack_var_keyword] = ACTIONS(1507),
    [sym_constant_keyword] = ACTIONS(1507),
    [sym_volatile_keyword] = ACTIONS(1507),
    [sym_non_volatile_keyword] = ACTIONS(1507),
    [sym_persistent_keyword] = ACTIONS(1507),
    [sym_dev_keyword] = ACTIONS(1507),
    [sym_devlev_keyword] = ACTIONS(1507),
    [sym_devchan_keyword] = ACTIONS(1507),
    [sym_if_keyword] = ACTIONS(1507),
    [sym_else_keyword] = ACTIONS(1507),
    [sym_switch_keyword] = ACTIONS(1507),
    [sym_case_keyword] = ACTIONS(1507),
    [sym_default_keyword] = ACTIONS(1507),
    [sym_while_keyword] = ACTIONS(1507),
    [sym_for_keyword] = ACTIONS(1507),
    [sym_break_keyword] = ACTIONS(1507),
    [sym_continue_keyword] = ACTIONS(1507),
    [sym_return_keyword] = ACTIONS(1507),
    [sym_select_keyword] = ACTIONS(1507),
    [sym_active_keyword] = ACTIONS(1507),
    [sym_struct_keyword] = ACTIONS(1507),
    [sym_structure_keyword] = ACTIONS(1507),
    [aux_sym_on_keyword_token1] = ACTIONS(1507),
    [aux_sym_off_keyword_token1] = ACTIONS(1507),
    [sym_send_level_keyword] = ACTIONS(1507),
    [sym_send_string_keyword] = ACTIONS(1507),
    [sym_send_command_keyword] = ACTIONS(1507),
    [sym_clear_buffer_keyword] = ACTIONS(1507),
    [sym_create_buffer_keyword] = ACTIONS(1507),
    [sym_create_multi_buffer_keyword] = ACTIONS(1507),
    [sym_call_keyword] = ACTIONS(1507),
    [sym_system_call_keyword] = ACTIONS(1507),
    [sym_devchan_to_keyword] = ACTIONS(1507),
    [sym_devchan_min_to_keyword] = ACTIONS(1507),
    [sym_devchan_total_off_keyword] = ACTIONS(1507),
    [sym_devchan_pulse_keyword] = ACTIONS(1507),
    [sym_wait_keyword] = ACTIONS(1507),
    [sym_cancel_wait_keyword] = ACTIONS(1507),
    [sym_wait_until_keyword] = ACTIONS(1507),
    [sym_cancel_wait_until_keyword] = ACTIONS(1507),
    [sym_cancel_all_wait_keyword] = ACTIONS(1507),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1507),
    [sym_and] = ACTIONS(1507),
    [sym_or] = ACTIONS(1507),
    [sym_not] = ACTIONS(1507),
    [sym_xor] = ACTIONS(1507),
    [sym_band] = ACTIONS(1507),
    [sym_bor] = ACTIONS(1507),
    [sym_bxor] = ACTIONS(1507),
    [sym_bnot] = ACTIONS(1507),
    [sym_lshift] = ACTIONS(1507),
    [sym_rshift] = ACTIONS(1507),
    [sym___file__] = ACTIONS(1507),
    [sym___date__] = ACTIONS(1507),
    [sym___ldate__] = ACTIONS(1507),
    [sym___time__] = ACTIONS(1507),
    [sym___line__] = ACTIONS(1507),
    [sym___name__] = ACTIONS(1507),
    [sym___netlinx__] = ACTIONS(1507),
    [sym_netlinx_axi_version] = ACTIONS(1507),
    [sym_url_flg_tcp] = ACTIONS(1507),
    [sym_url_flg_accinfopresent] = ACTIONS(1507),
    [sym_url_flg_temp] = ACTIONS(1507),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1507),
    [sym_url_flg_stat_mask] = ACTIONS(1507),
    [sym_url_flg_stat_lookup] = ACTIONS(1507),
    [sym_url_flg_stat_connecting] = ACTIONS(1507),
    [sym_url_flg_stat_waiting] = ACTIONS(1507),
    [sym_url_flg_stat_connected] = ACTIONS(1507),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1507),
    [sym_first_virtual_device] = ACTIONS(1507),
    [sym_first_local_port] = ACTIONS(1507),
    [sym_dynamic_virtual_device] = ACTIONS(1507),
    [sym_timeline_once] = ACTIONS(1507),
    [sym_timeline_repeat] = ACTIONS(1507),
    [sym_timeline_absolute] = ACTIONS(1507),
    [sym_timeline_relative] = ACTIONS(1507),
    [sym_do_push_timed_infinite] = ACTIONS(1507),
    [sym_source_type_no_address] = ACTIONS(1507),
    [sym_source_type_neuron_id] = ACTIONS(1507),
    [sym_source_type_ip_address] = ACTIONS(1507),
    [sym_source_type_axlink] = ACTIONS(1507),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1507),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1507),
    [sym_source_type_ip_socket_address] = ACTIONS(1507),
    [sym_source_type_rs232] = ACTIONS(1507),
    [sym_source_type_internal] = ACTIONS(1507),
    [sym_source_type_mac_address] = ACTIONS(1507),
    [sym_source_type_ipv4_port] = ACTIONS(1507),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1507),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1507),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1507),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1507),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1507),
    [sym_true] = ACTIONS(1507),
    [sym_false] = ACTIONS(1507),
    [sym_file_read_only] = ACTIONS(1507),
    [sym_file_rw_new] = ACTIONS(1507),
    [sym_file_rw_append] = ACTIONS(1507),
    [sym_ip_tcp] = ACTIONS(1507),
    [sym_ip_udp] = ACTIONS(1507),
    [sym_ip_udp_2way] = ACTIONS(1507),
    [sym_xml_encode_types] = ACTIONS(1507),
    [sym_xml_encode_char_as_list] = ACTIONS(1507),
    [sym_xml_encode_le] = ACTIONS(1507),
    [sym_xml_decode_types] = ACTIONS(1507),
    [sym_xml_decode_no_preserve] = ACTIONS(1507),
    [sym_duet_dev_type_display_device] = ACTIONS(1507),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1507),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1507),
    [sym_duet_dev_type_io_device] = ACTIONS(1507),
    [sym_duet_dev_type_relay_device] = ACTIONS(1507),
    [sym_duet_dev_type_ups] = ACTIONS(1507),
    [sym_duet_dev_type_amplifier] = ACTIONS(1507),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1507),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1507),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1507),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1507),
    [sym_duet_dev_type_camera] = ACTIONS(1507),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1507),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1507),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1507),
    [sym_duet_dev_type_dss] = ACTIONS(1507),
    [sym_duet_dev_type_dvr] = ACTIONS(1507),
    [sym_duet_dev_type_disc_device] = ACTIONS(1507),
    [sym_duet_dev_type_document_camera] = ACTIONS(1507),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1507),
    [sym_duet_dev_type_hvac] = ACTIONS(1507),
    [sym_duet_dev_type_keypad] = ACTIONS(1507),
    [sym_duet_dev_type_light] = ACTIONS(1507),
    [sym_duet_dev_type_monitor] = ACTIONS(1507),
    [sym_duet_dev_type_motor] = ACTIONS(1507),
    [sym_duet_dev_type_multi_window] = ACTIONS(1507),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1507),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1507),
    [sym_duet_dev_type_receiver] = ACTIONS(1507),
    [sym_duet_dev_type_security_system] = ACTIONS(1507),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1507),
    [sym_duet_dev_type_settop_box] = ACTIONS(1507),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1507),
    [sym_duet_dev_type_switcher] = ACTIONS(1507),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1507),
    [sym_duet_dev_type_tv] = ACTIONS(1507),
    [sym_duet_dev_type_utility] = ACTIONS(1507),
    [sym_duet_dev_type_vcr] = ACTIONS(1507),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1507),
    [sym_duet_dev_type_video_processor] = ACTIONS(1507),
    [sym_duet_dev_type_video_projector] = ACTIONS(1507),
    [sym_duet_dev_type_video_wall] = ACTIONS(1507),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1507),
    [sym_duet_dev_type_weather] = ACTIONS(1507),
    [sym_duet_dev_not_polled] = ACTIONS(1507),
    [sym_duet_dev_polled] = ACTIONS(1507),
    [sym_clkmgr_mode_network] = ACTIONS(1507),
    [sym_clkmgr_mode_standalone] = ACTIONS(1507),
    [sym_normal_standby] = ACTIONS(1507),
    [sym_normal_wake] = ACTIONS(1507),
    [sym_ip_multicast_ttl_option] = ACTIONS(1507),
    [sym_ip_ttl_subnet] = ACTIONS(1507),
    [sym_ip_ttl_site] = ACTIONS(1507),
    [sym_ip_ttl_region] = ACTIONS(1507),
    [sym_ip_ttl_continent] = ACTIONS(1507),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1507),
    [sym_ip_nodelay_on] = ACTIONS(1507),
    [sym_ip_nodelay_off] = ACTIONS(1507),
    [sym_amx_error] = ACTIONS(1507),
    [sym_amx_warning] = ACTIONS(1507),
    [sym_amx_info] = ACTIONS(1507),
    [sym_amx_debug] = ACTIONS(1507),
    [sym_netlinx_login_success] = ACTIONS(1507),
    [sym_netlinx_login_fail] = ACTIONS(1507),
    [sym_netlinx_logout] = ACTIONS(1507),
    [sym_null_str] = ACTIONS(1507),
    [sym_smtp_address] = ACTIONS(1507),
    [sym_smtp_port_number] = ACTIONS(1507),
    [sym_smtp_username] = ACTIONS(1507),
    [sym_smtp_password] = ACTIONS(1507),
    [sym_smtp_require_tls] = ACTIONS(1507),
    [sym_smtp_from] = ACTIONS(1507),
    [sym_smtp_tls_true] = ACTIONS(1507),
    [sym_smtp_tls_false] = ACTIONS(1507),
    [sym_listview_on_row_select_event] = ACTIONS(1507),
    [sym_data_structure_unknown] = ACTIONS(1507),
    [sym_data_structure_datafeed] = ACTIONS(1507),
    [sym_data_structure_datarecordset] = ACTIONS(1507),
    [sym_data_structure_datarecord] = ACTIONS(1507),
    [sym_data_structure_datafield] = ACTIONS(1507),
    [sym_data_max_content_field_count] = ACTIONS(1507),
    [sym_data_max_metadata_field_count] = ACTIONS(1507),
    [sym_data_max_id_length] = ACTIONS(1507),
    [sym_data_max_name_length] = ACTIONS(1507),
    [sym_data_max_description_length] = ACTIONS(1507),
    [sym_data_max_type_length] = ACTIONS(1507),
    [sym_data_max_format_length] = ACTIONS(1507),
    [sym_data_max_source_length] = ACTIONS(1507),
    [sym_data_max_value_length] = ACTIONS(1507),
    [sym_data_max_label_length] = ACTIONS(1507),
    [sym_data_max_url_length] = ACTIONS(1507),
    [sym_max_last_login_info_length] = ACTIONS(1507),
    [sym_data_type_unknown] = ACTIONS(1507),
    [sym_data_type_string] = ACTIONS(1507),
    [sym_data_type_datetime] = ACTIONS(1507),
    [sym_data_type_date] = ACTIONS(1507),
    [sym_data_type_time] = ACTIONS(1507),
    [sym_data_type_image] = ACTIONS(1507),
    [sym_data_format_url] = ACTIONS(1507),
    [sym_data_format_phone] = ACTIONS(1507),
    [sym_data_format_email] = ACTIONS(1507),
    [sym_data_format_iso8601] = ACTIONS(1507),
    [sym_auditlog_priv] = ACTIONS(1507),
    [sym_configuration_priv] = ACTIONS(1507),
    [sym_deviceconfig_priv] = ACTIONS(1507),
    [sym_ftp_priv] = ACTIONS(1507),
    [sym_http_priv] = ACTIONS(1507),
    [sym_networkconfig_priv] = ACTIONS(1507),
    [sym_programport_priv] = ACTIONS(1507),
    [sym_remoteui_priv] = ACTIONS(1507),
    [sym_securitycontrol_priv] = ACTIONS(1507),
    [sym_softwaremanagement_priv] = ACTIONS(1507),
    [sym_terminal_priv] = ACTIONS(1507),
    [sym_tpadmin_priv] = ACTIONS(1507),
    [sym_user1_priv] = ACTIONS(1507),
    [sym_user2_priv] = ACTIONS(1507),
    [sym_user3_priv] = ACTIONS(1507),
    [sym_user4_priv] = ACTIONS(1507),
    [sym_usermanagement_priv] = ACTIONS(1507),
    [sym_valid_account] = ACTIONS(1507),
    [sym_err_name_invalid] = ACTIONS(1507),
    [sym_err_password_invalid] = ACTIONS(1507),
    [sym_err_invalid_account] = ACTIONS(1507),
    [sym_err_no_authorization] = ACTIONS(1507),
    [sym_err_invalid_parameter] = ACTIONS(1507),
    [sym_err_user_locked_out] = ACTIONS(1507),
    [sym_err_user_account_expired] = ACTIONS(1507),
    [sym_err_general_error] = ACTIONS(1507),
    [sym_err_authentication_error] = ACTIONS(1507),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1507),
    [sym_tls_validate_certificate] = ACTIONS(1507),
    [sym_tdata] = ACTIONS(1507),
    [sym_tchannel] = ACTIONS(1507),
    [sym_tlevel] = ACTIONS(1507),
    [sym_tbutton] = ACTIONS(1507),
    [sym_ttimeline] = ACTIONS(1507),
    [sym_tcustom] = ACTIONS(1507),
    [sym_url_struct] = ACTIONS(1507),
    [sym_dns_struct] = ACTIONS(1507),
    [sym_ip_address_struct] = ACTIONS(1507),
    [sym_dev_info_struct] = ACTIONS(1507),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1507),
    [sym_data_feed] = ACTIONS(1507),
    [sym_data_field] = ACTIONS(1507),
    [sym_data_record] = ACTIONS(1507),
    [sym_wc_data_feed] = ACTIONS(1507),
    [sym_wc_data_field] = ACTIONS(1507),
    [sym_wc_data_record] = ACTIONS(1507),
    [sym_last_login_info] = ACTIONS(1507),
    [sym____reserved___] = ACTIONS(1507),
    [sym_date] = ACTIONS(1507),
    [sym_day] = ACTIONS(1507),
    [sym_time] = ACTIONS(1507),
    [sym_ldate] = ACTIONS(1507),
    [sym_push_device] = ACTIONS(1507),
    [sym_push_channel] = ACTIONS(1507),
    [sym_push_devchan] = ACTIONS(1507),
    [sym_release_device] = ACTIONS(1507),
    [sym_release_channel] = ACTIONS(1507),
    [sym_release_devchan] = ACTIONS(1507),
    [sym_master_slot] = ACTIONS(1507),
    [sym_get_pulse_time] = ACTIONS(1507),
    [sym_get_timer] = ACTIONS(1507),
    [sym_master_sn] = ACTIONS(1507),
    [sym_system_number] = ACTIONS(1507),
    [sym_button] = ACTIONS(1507),
    [sym_dv_channel] = ACTIONS(1507),
    [sym_data] = ACTIONS(1507),
    [sym_level] = ACTIONS(1507),
    [sym_timeline] = ACTIONS(1507),
    [sym_channel] = ACTIONS(1507),
    [sym_custom] = ACTIONS(1507),
    [sym_atoi] = ACTIONS(1507),
    [sym_atol] = ACTIONS(1507),
    [sym_atof] = ACTIONS(1507),
    [sym_itoa] = ACTIONS(1507),
    [sym_itohex] = ACTIONS(1507),
    [sym_hextoi] = ACTIONS(1507),
    [sym_ftoa] = ACTIONS(1507),
    [sym_find_string] = ACTIONS(1507),
    [sym_left_string] = ACTIONS(1507),
    [sym_right_string] = ACTIONS(1507),
    [sym_mid_string] = ACTIONS(1507),
    [sym_remove_string] = ACTIONS(1507),
    [sym_get_buffer_string] = ACTIONS(1507),
    [sym_get_buffer_char] = ACTIONS(1507),
    [sym_get_multi_buffer_string] = ACTIONS(1507),
    [sym_length_string] = ACTIONS(1507),
    [sym_max_length_string] = ACTIONS(1507),
    [sym_set_length_string] = ACTIONS(1507),
    [sym_compare_string] = ACTIONS(1507),
    [sym_upper_string] = ACTIONS(1507),
    [sym_lower_string] = ACTIONS(1507),
    [sym_get_last] = ACTIONS(1507),
    [sym_date_to_day] = ACTIONS(1507),
    [sym_date_to_month] = ACTIONS(1507),
    [sym_date_to_year] = ACTIONS(1507),
    [sym_time_to_hour] = ACTIONS(1507),
    [sym_time_to_minute] = ACTIONS(1507),
    [sym_time_to_second] = ACTIONS(1507),
    [sym_day_of_week] = ACTIONS(1507),
    [sym_set_pulse_time] = ACTIONS(1507),
    [sym_set_timer] = ACTIONS(1507),
    [sym_file_open] = ACTIONS(1507),
    [sym_file_close] = ACTIONS(1507),
    [sym_file_read] = ACTIONS(1507),
    [sym_file_read_line] = ACTIONS(1507),
    [sym_file_write] = ACTIONS(1507),
    [sym_file_write_line] = ACTIONS(1507),
    [sym_file_copy] = ACTIONS(1507),
    [sym_file_delete] = ACTIONS(1507),
    [sym_file_rename] = ACTIONS(1507),
    [sym_file_dir] = ACTIONS(1507),
    [sym_file_getdir] = ACTIONS(1507),
    [sym_file_setdir] = ACTIONS(1507),
    [sym_file_create_dir] = ACTIONS(1507),
    [sym_file_remove_dir] = ACTIONS(1507),
    [sym_file_seek] = ACTIONS(1507),
    [sym_length_array] = ACTIONS(1507),
    [sym_max_length_array] = ACTIONS(1507),
    [sym_set_length_array] = ACTIONS(1507),
    [sym_ip_client_open] = ACTIONS(1507),
    [sym_ip_client_close] = ACTIONS(1507),
    [sym_ip_server_open] = ACTIONS(1507),
    [sym_ip_server_close] = ACTIONS(1507),
    [sym_ip_bound_client_open] = ACTIONS(1507),
    [sym_ip_mc_server_open] = ACTIONS(1507),
    [sym_ip_set_option] = ACTIONS(1507),
    [sym_ssh_client_open] = ACTIONS(1507),
    [sym_ssh_client_close] = ACTIONS(1507),
    [sym_tls_client_open] = ACTIONS(1507),
    [sym_tls_client_close] = ACTIONS(1507),
    [sym_do_push] = ACTIONS(1507),
    [sym_do_push_timed] = ACTIONS(1507),
    [sym_do_release] = ACTIONS(1507),
    [sym_device_id] = ACTIONS(1507),
    [sym_device_info] = ACTIONS(1507),
    [sym_device_id_string] = ACTIONS(1507),
    [sym_device_standby] = ACTIONS(1507),
    [sym_device_wake] = ACTIONS(1507),
    [sym_timeline_create] = ACTIONS(1507),
    [sym_timeline_kill] = ACTIONS(1507),
    [sym_timeline_reload] = ACTIONS(1507),
    [sym_timeline_pause] = ACTIONS(1507),
    [sym_timeline_restart] = ACTIONS(1507),
    [sym_timeline_set] = ACTIONS(1507),
    [sym_timeline_get] = ACTIONS(1507),
    [sym_timeline_active] = ACTIONS(1507),
    [sym_timeline_dynamic_id] = ACTIONS(1507),
    [sym_variable_to_string] = ACTIONS(1507),
    [sym_string_to_variable] = ACTIONS(1507),
    [sym_length_variable_to_string] = ACTIONS(1507),
    [sym_type_cast] = ACTIONS(1507),
    [sym_variable_to_xml] = ACTIONS(1507),
    [sym_xml_to_variable] = ACTIONS(1507),
    [sym_length_variable_to_xml] = ACTIONS(1507),
    [sym_redirect_string] = ACTIONS(1507),
    [sym_random_number] = ACTIONS(1507),
    [sym_get_unique_id] = ACTIONS(1507),
    [sym_get_system_number] = ACTIONS(1507),
    [sym_set_system_number] = ACTIONS(1507),
    [sym_get_serial_number] = ACTIONS(1507),
    [sym_get_master_build] = ACTIONS(1507),
    [sym_set_validation_code] = ACTIONS(1507),
    [sym_reboot] = ACTIONS(1507),
    [sym_load_duet_module] = ACTIONS(1507),
    [sym_unload_duet_module] = ACTIONS(1507),
    [sym_duet_mem_size_set] = ACTIONS(1507),
    [sym_duet_mem_size_get] = ACTIONS(1507),
    [sym_dynamic_application_device] = ACTIONS(1507),
    [sym_dynamic_polled_port] = ACTIONS(1507),
    [sym_static_port_binding] = ACTIONS(1507),
    [sym_static_ip_binding] = ACTIONS(1507),
    [sym_get_dns_list] = ACTIONS(1507),
    [sym_set_dns_list] = ACTIONS(1507),
    [sym_get_ip_address] = ACTIONS(1507),
    [sym_set_ip_address] = ACTIONS(1507),
    [sym_get_url_list] = ACTIONS(1507),
    [sym_add_url_entry] = ACTIONS(1507),
    [sym_delete_url_entry] = ACTIONS(1507),
    [sym_abs_value] = ACTIONS(1507),
    [sym_max_value] = ACTIONS(1507),
    [sym_min_value] = ACTIONS(1507),
    [sym_exp_value] = ACTIONS(1507),
    [sym_log_value] = ACTIONS(1507),
    [sym_log10_value] = ACTIONS(1507),
    [sym_power_value] = ACTIONS(1507),
    [sym_sqrt_value] = ACTIONS(1507),
    [sym_raw_be] = ACTIONS(1507),
    [sym_raw_le] = ACTIONS(1507),
    [sym_internal_queue_size_set] = ACTIONS(1507),
    [sym_internal_queue_size_get] = ACTIONS(1507),
    [sym_internal_threshold_set] = ACTIONS(1507),
    [sym_internal_threshold_get] = ACTIONS(1507),
    [sym_rebuilt_event] = ACTIONS(1507),
    [sym_do_custom_event] = ACTIONS(1507),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1507),
    [sym_clkmgr_set_clk_source] = ACTIONS(1507),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1507),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1507),
    [sym_clkmgr_get_timezone] = ACTIONS(1507),
    [sym_clkmgr_set_timezone] = ACTIONS(1507),
    [sym_clkmgr_get_resync_period] = ACTIONS(1507),
    [sym_clkmgr_set_resync_period] = ACTIONS(1507),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1507),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1507),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1507),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1507),
    [sym_clkmgr_get_timeservers] = ACTIONS(1507),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1507),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1507),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1507),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1507),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1507),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1507),
    [sym_set_log_level] = ACTIONS(1507),
    [sym_get_log_level] = ACTIONS(1507),
    [sym_amx_log] = ACTIONS(1507),
    [sym_smtp_server_config_set] = ACTIONS(1507),
    [sym_smtp_server_config_get] = ACTIONS(1507),
    [sym_smtp_send] = ACTIONS(1507),
    [sym_validate_netlinx_account] = ACTIONS(1507),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1507),
    [sym_authenticate_certificate] = ACTIONS(1507),
    [sym_audit_netlinx_session_event] = ACTIONS(1507),
    [sym_audit_netlinx_generic_event] = ACTIONS(1507),
    [sym_get_available_flash_disk_space] = ACTIONS(1507),
    [sym_get_max_flash_disk_space] = ACTIONS(1507),
    [sym_data_create_feed] = ACTIONS(1507),
    [sym_data_delete_feed] = ACTIONS(1507),
    [sym_data_publish_feed] = ACTIONS(1507),
    [sym_data_get_published_feed] = ACTIONS(1507),
    [sym_data_add_record] = ACTIONS(1507),
    [sym_data_get_event_record] = ACTIONS(1507),
    [sym__wc_data_create_feed] = ACTIONS(1507),
    [sym__wc_data_add_record] = ACTIONS(1507),
    [sym__wc_data_get_event_record] = ACTIONS(1507),
    [sym_format] = ACTIONS(1507),
    [sym_astro_clock] = ACTIONS(1507),
    [sym_set_outdoor_temperature] = ACTIONS(1507),
    [sym_set_virtual_level_count] = ACTIONS(1507),
    [sym_set_virtual_channel_count] = ACTIONS(1507),
    [sym_set_virtual_port_count] = ACTIONS(1507),
    [sym__wc_encode_int] = ACTIONS(1507),
    [sym__wc_decode_int] = ACTIONS(1507),
    [sym__wc_int] = ACTIONS(1507),
    [sym__wc_to_ch_int] = ACTIONS(1507),
    [sym__ch_to_wc_int] = ACTIONS(1507),
    [sym__wc_find_string_int] = ACTIONS(1507),
    [sym__wc_left_string_int] = ACTIONS(1507),
    [sym__wc_lower_string_int] = ACTIONS(1507),
    [sym__wc_mid_string_int] = ACTIONS(1507),
    [sym__wc_remove_string_int] = ACTIONS(1507),
    [sym__wc_right_string_int] = ACTIONS(1507),
    [sym__wc_upper_string_int] = ACTIONS(1507),
    [sym__wc_compare_string_int] = ACTIONS(1507),
    [sym__wc_get_buffer_char_int] = ACTIONS(1507),
    [sym__wc_get_buffer_string_int] = ACTIONS(1507),
    [sym__wc_concat_string_int] = ACTIONS(1507),
    [sym__wc_file_open_int] = ACTIONS(1507),
    [sym__wc_file_close_int] = ACTIONS(1507),
    [sym__wc_file_read_int] = ACTIONS(1507),
    [sym__wc_file_read_line_int] = ACTIONS(1507),
    [sym__wc_file_write_int] = ACTIONS(1507),
    [sym__wc_file_write_line_int] = ACTIONS(1507),
    [anon_sym_EQ] = ACTIONS(1507),
    [aux_sym_preproc_include_token1] = ACTIONS(1507),
    [aux_sym_preproc_define_token1] = ACTIONS(1509),
    [aux_sym_preproc_warn_token1] = ACTIONS(1509),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1509),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1509),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1509),
    [anon_sym_LBRACK] = ACTIONS(1509),
    [anon_sym_LBRACE] = ACTIONS(1509),
    [anon_sym_RBRACE] = ACTIONS(1509),
    [anon_sym_COLON] = ACTIONS(1509),
    [anon_sym_LPAREN] = ACTIONS(1507),
    [anon_sym_SEMI] = ACTIONS(1509),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1509),
    [anon_sym_DASH] = ACTIONS(1507),
    [anon_sym_PLUS] = ACTIONS(1507),
    [anon_sym_STAR] = ACTIONS(1509),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_PERCENT] = ACTIONS(1509),
    [anon_sym_PIPE_PIPE] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1509),
    [anon_sym_PIPE] = ACTIONS(1507),
    [anon_sym_CARET] = ACTIONS(1509),
    [anon_sym_AMP] = ACTIONS(1507),
    [anon_sym_EQ_EQ] = ACTIONS(1509),
    [anon_sym_BANG_EQ] = ACTIONS(1509),
    [anon_sym_LT_GT] = ACTIONS(1509),
    [anon_sym_GT] = ACTIONS(1507),
    [anon_sym_GT_EQ] = ACTIONS(1509),
    [anon_sym_LT_EQ] = ACTIONS(1509),
    [anon_sym_LT] = ACTIONS(1507),
    [anon_sym_LT_LT] = ACTIONS(1509),
    [anon_sym_GT_GT] = ACTIONS(1509),
    [anon_sym_DASH_DASH] = ACTIONS(1509),
    [anon_sym_PLUS_PLUS] = ACTIONS(1509),
    [anon_sym_DOT] = ACTIONS(1507),
    [sym_range_operator] = ACTIONS(1509),
    [anon_sym_DQUOTE] = ACTIONS(1509),
    [anon_sym_SQUOTE] = ACTIONS(1509),
    [sym_number_literal] = ACTIONS(1509),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1509),
  },
  [STATE(158)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1615),
    [sym_rshift] = ACTIONS(1615),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1625),
    [anon_sym_PLUS] = ACTIONS(1625),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1639),
    [anon_sym_GT_EQ] = ACTIONS(1641),
    [anon_sym_LT_EQ] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1639),
    [anon_sym_LT_LT] = ACTIONS(1643),
    [anon_sym_GT_GT] = ACTIONS(1643),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(159)] = {
    [sym_identifier] = ACTIONS(1465),
    [sym_define_call_keyword] = ACTIONS(1465),
    [sym_define_function_keyword] = ACTIONS(1465),
    [sym_define_library_function_keyword] = ACTIONS(1465),
    [sym_define_module_keyword] = ACTIONS(1465),
    [sym_char_keyword] = ACTIONS(1465),
    [sym_widechar_keyword] = ACTIONS(1465),
    [sym_integer_keyword] = ACTIONS(1465),
    [sym_sinteger_keyword] = ACTIONS(1465),
    [sym_long_keyword] = ACTIONS(1465),
    [sym_slong_keyword] = ACTIONS(1465),
    [sym_float_keyword] = ACTIONS(1465),
    [sym_double_keyword] = ACTIONS(1465),
    [sym_variant_keyword] = ACTIONS(1465),
    [sym_variantarray_keyword] = ACTIONS(1465),
    [sym_local_var_keyword] = ACTIONS(1465),
    [sym_stack_var_keyword] = ACTIONS(1465),
    [sym_constant_keyword] = ACTIONS(1465),
    [sym_volatile_keyword] = ACTIONS(1465),
    [sym_non_volatile_keyword] = ACTIONS(1465),
    [sym_persistent_keyword] = ACTIONS(1465),
    [sym_dev_keyword] = ACTIONS(1465),
    [sym_devlev_keyword] = ACTIONS(1465),
    [sym_devchan_keyword] = ACTIONS(1465),
    [sym_if_keyword] = ACTIONS(1465),
    [sym_else_keyword] = ACTIONS(1465),
    [sym_switch_keyword] = ACTIONS(1465),
    [sym_case_keyword] = ACTIONS(1465),
    [sym_default_keyword] = ACTIONS(1465),
    [sym_while_keyword] = ACTIONS(1465),
    [sym_for_keyword] = ACTIONS(1465),
    [sym_break_keyword] = ACTIONS(1465),
    [sym_continue_keyword] = ACTIONS(1465),
    [sym_return_keyword] = ACTIONS(1465),
    [sym_select_keyword] = ACTIONS(1465),
    [sym_active_keyword] = ACTIONS(1465),
    [sym_struct_keyword] = ACTIONS(1465),
    [sym_structure_keyword] = ACTIONS(1465),
    [aux_sym_on_keyword_token1] = ACTIONS(1465),
    [aux_sym_off_keyword_token1] = ACTIONS(1465),
    [sym_send_level_keyword] = ACTIONS(1465),
    [sym_send_string_keyword] = ACTIONS(1465),
    [sym_send_command_keyword] = ACTIONS(1465),
    [sym_clear_buffer_keyword] = ACTIONS(1465),
    [sym_create_buffer_keyword] = ACTIONS(1465),
    [sym_create_multi_buffer_keyword] = ACTIONS(1465),
    [sym_call_keyword] = ACTIONS(1465),
    [sym_system_call_keyword] = ACTIONS(1465),
    [sym_devchan_to_keyword] = ACTIONS(1465),
    [sym_devchan_min_to_keyword] = ACTIONS(1465),
    [sym_devchan_total_off_keyword] = ACTIONS(1465),
    [sym_devchan_pulse_keyword] = ACTIONS(1465),
    [sym_wait_keyword] = ACTIONS(1465),
    [sym_cancel_wait_keyword] = ACTIONS(1465),
    [sym_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym_tdata] = ACTIONS(1465),
    [sym_tchannel] = ACTIONS(1465),
    [sym_tlevel] = ACTIONS(1465),
    [sym_tbutton] = ACTIONS(1465),
    [sym_ttimeline] = ACTIONS(1465),
    [sym_tcustom] = ACTIONS(1465),
    [sym_url_struct] = ACTIONS(1465),
    [sym_dns_struct] = ACTIONS(1465),
    [sym_ip_address_struct] = ACTIONS(1465),
    [sym_dev_info_struct] = ACTIONS(1465),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1465),
    [sym_data_feed] = ACTIONS(1465),
    [sym_data_field] = ACTIONS(1465),
    [sym_data_record] = ACTIONS(1465),
    [sym_wc_data_feed] = ACTIONS(1465),
    [sym_wc_data_field] = ACTIONS(1465),
    [sym_wc_data_record] = ACTIONS(1465),
    [sym_last_login_info] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [anon_sym_EQ] = ACTIONS(1656),
    [aux_sym_preproc_include_token1] = ACTIONS(1465),
    [aux_sym_preproc_define_token1] = ACTIONS(1469),
    [aux_sym_preproc_warn_token1] = ACTIONS(1469),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_RBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1469),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [sym_range_operator] = ACTIONS(1658),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1469),
  },
  [STATE(160)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1473),
    [sym_define_call_keyword] = ACTIONS(1473),
    [sym_define_function_keyword] = ACTIONS(1473),
    [sym_define_library_function_keyword] = ACTIONS(1473),
    [sym_define_module_keyword] = ACTIONS(1473),
    [sym_char_keyword] = ACTIONS(1473),
    [sym_widechar_keyword] = ACTIONS(1473),
    [sym_integer_keyword] = ACTIONS(1473),
    [sym_sinteger_keyword] = ACTIONS(1473),
    [sym_long_keyword] = ACTIONS(1473),
    [sym_slong_keyword] = ACTIONS(1473),
    [sym_float_keyword] = ACTIONS(1473),
    [sym_double_keyword] = ACTIONS(1473),
    [sym_variant_keyword] = ACTIONS(1473),
    [sym_variantarray_keyword] = ACTIONS(1473),
    [sym_local_var_keyword] = ACTIONS(1473),
    [sym_stack_var_keyword] = ACTIONS(1473),
    [sym_constant_keyword] = ACTIONS(1473),
    [sym_volatile_keyword] = ACTIONS(1473),
    [sym_non_volatile_keyword] = ACTIONS(1473),
    [sym_persistent_keyword] = ACTIONS(1473),
    [sym_dev_keyword] = ACTIONS(1473),
    [sym_devlev_keyword] = ACTIONS(1473),
    [sym_devchan_keyword] = ACTIONS(1473),
    [sym_if_keyword] = ACTIONS(1473),
    [sym_else_keyword] = ACTIONS(1473),
    [sym_switch_keyword] = ACTIONS(1473),
    [sym_case_keyword] = ACTIONS(1473),
    [sym_default_keyword] = ACTIONS(1473),
    [sym_while_keyword] = ACTIONS(1473),
    [sym_for_keyword] = ACTIONS(1473),
    [sym_break_keyword] = ACTIONS(1473),
    [sym_continue_keyword] = ACTIONS(1473),
    [sym_return_keyword] = ACTIONS(1473),
    [sym_select_keyword] = ACTIONS(1473),
    [sym_active_keyword] = ACTIONS(1473),
    [sym_struct_keyword] = ACTIONS(1473),
    [sym_structure_keyword] = ACTIONS(1473),
    [aux_sym_on_keyword_token1] = ACTIONS(1473),
    [aux_sym_off_keyword_token1] = ACTIONS(1473),
    [sym_send_level_keyword] = ACTIONS(1473),
    [sym_send_string_keyword] = ACTIONS(1473),
    [sym_send_command_keyword] = ACTIONS(1473),
    [sym_clear_buffer_keyword] = ACTIONS(1473),
    [sym_create_buffer_keyword] = ACTIONS(1473),
    [sym_create_multi_buffer_keyword] = ACTIONS(1473),
    [sym_call_keyword] = ACTIONS(1473),
    [sym_system_call_keyword] = ACTIONS(1473),
    [sym_devchan_to_keyword] = ACTIONS(1473),
    [sym_devchan_min_to_keyword] = ACTIONS(1473),
    [sym_devchan_total_off_keyword] = ACTIONS(1473),
    [sym_devchan_pulse_keyword] = ACTIONS(1473),
    [sym_wait_keyword] = ACTIONS(1473),
    [sym_cancel_wait_keyword] = ACTIONS(1473),
    [sym_wait_until_keyword] = ACTIONS(1473),
    [sym_cancel_wait_until_keyword] = ACTIONS(1473),
    [sym_cancel_all_wait_keyword] = ACTIONS(1473),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1473),
    [sym_and] = ACTIONS(1473),
    [sym_or] = ACTIONS(1473),
    [sym_not] = ACTIONS(1473),
    [sym_xor] = ACTIONS(1473),
    [sym_band] = ACTIONS(1473),
    [sym_bor] = ACTIONS(1473),
    [sym_bxor] = ACTIONS(1473),
    [sym_bnot] = ACTIONS(1473),
    [sym_lshift] = ACTIONS(1473),
    [sym_rshift] = ACTIONS(1473),
    [sym___file__] = ACTIONS(1473),
    [sym___date__] = ACTIONS(1473),
    [sym___ldate__] = ACTIONS(1473),
    [sym___time__] = ACTIONS(1473),
    [sym___line__] = ACTIONS(1473),
    [sym___name__] = ACTIONS(1473),
    [sym___netlinx__] = ACTIONS(1473),
    [sym_netlinx_axi_version] = ACTIONS(1473),
    [sym_url_flg_tcp] = ACTIONS(1473),
    [sym_url_flg_accinfopresent] = ACTIONS(1473),
    [sym_url_flg_temp] = ACTIONS(1473),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1473),
    [sym_url_flg_stat_mask] = ACTIONS(1473),
    [sym_url_flg_stat_lookup] = ACTIONS(1473),
    [sym_url_flg_stat_connecting] = ACTIONS(1473),
    [sym_url_flg_stat_waiting] = ACTIONS(1473),
    [sym_url_flg_stat_connected] = ACTIONS(1473),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1473),
    [sym_first_virtual_device] = ACTIONS(1473),
    [sym_first_local_port] = ACTIONS(1473),
    [sym_dynamic_virtual_device] = ACTIONS(1473),
    [sym_timeline_once] = ACTIONS(1473),
    [sym_timeline_repeat] = ACTIONS(1473),
    [sym_timeline_absolute] = ACTIONS(1473),
    [sym_timeline_relative] = ACTIONS(1473),
    [sym_do_push_timed_infinite] = ACTIONS(1473),
    [sym_source_type_no_address] = ACTIONS(1473),
    [sym_source_type_neuron_id] = ACTIONS(1473),
    [sym_source_type_ip_address] = ACTIONS(1473),
    [sym_source_type_axlink] = ACTIONS(1473),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1473),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1473),
    [sym_source_type_ip_socket_address] = ACTIONS(1473),
    [sym_source_type_rs232] = ACTIONS(1473),
    [sym_source_type_internal] = ACTIONS(1473),
    [sym_source_type_mac_address] = ACTIONS(1473),
    [sym_source_type_ipv4_port] = ACTIONS(1473),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1473),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1473),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1473),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1473),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1473),
    [sym_true] = ACTIONS(1473),
    [sym_false] = ACTIONS(1473),
    [sym_file_read_only] = ACTIONS(1473),
    [sym_file_rw_new] = ACTIONS(1473),
    [sym_file_rw_append] = ACTIONS(1473),
    [sym_ip_tcp] = ACTIONS(1473),
    [sym_ip_udp] = ACTIONS(1473),
    [sym_ip_udp_2way] = ACTIONS(1473),
    [sym_xml_encode_types] = ACTIONS(1473),
    [sym_xml_encode_char_as_list] = ACTIONS(1473),
    [sym_xml_encode_le] = ACTIONS(1473),
    [sym_xml_decode_types] = ACTIONS(1473),
    [sym_xml_decode_no_preserve] = ACTIONS(1473),
    [sym_duet_dev_type_display_device] = ACTIONS(1473),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1473),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1473),
    [sym_duet_dev_type_io_device] = ACTIONS(1473),
    [sym_duet_dev_type_relay_device] = ACTIONS(1473),
    [sym_duet_dev_type_ups] = ACTIONS(1473),
    [sym_duet_dev_type_amplifier] = ACTIONS(1473),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1473),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1473),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1473),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1473),
    [sym_duet_dev_type_camera] = ACTIONS(1473),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1473),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1473),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1473),
    [sym_duet_dev_type_dss] = ACTIONS(1473),
    [sym_duet_dev_type_dvr] = ACTIONS(1473),
    [sym_duet_dev_type_disc_device] = ACTIONS(1473),
    [sym_duet_dev_type_document_camera] = ACTIONS(1473),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1473),
    [sym_duet_dev_type_hvac] = ACTIONS(1473),
    [sym_duet_dev_type_keypad] = ACTIONS(1473),
    [sym_duet_dev_type_light] = ACTIONS(1473),
    [sym_duet_dev_type_monitor] = ACTIONS(1473),
    [sym_duet_dev_type_motor] = ACTIONS(1473),
    [sym_duet_dev_type_multi_window] = ACTIONS(1473),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1473),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1473),
    [sym_duet_dev_type_receiver] = ACTIONS(1473),
    [sym_duet_dev_type_security_system] = ACTIONS(1473),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1473),
    [sym_duet_dev_type_settop_box] = ACTIONS(1473),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1473),
    [sym_duet_dev_type_switcher] = ACTIONS(1473),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1473),
    [sym_duet_dev_type_tv] = ACTIONS(1473),
    [sym_duet_dev_type_utility] = ACTIONS(1473),
    [sym_duet_dev_type_vcr] = ACTIONS(1473),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1473),
    [sym_duet_dev_type_video_processor] = ACTIONS(1473),
    [sym_duet_dev_type_video_projector] = ACTIONS(1473),
    [sym_duet_dev_type_video_wall] = ACTIONS(1473),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1473),
    [sym_duet_dev_type_weather] = ACTIONS(1473),
    [sym_duet_dev_not_polled] = ACTIONS(1473),
    [sym_duet_dev_polled] = ACTIONS(1473),
    [sym_clkmgr_mode_network] = ACTIONS(1473),
    [sym_clkmgr_mode_standalone] = ACTIONS(1473),
    [sym_normal_standby] = ACTIONS(1473),
    [sym_normal_wake] = ACTIONS(1473),
    [sym_ip_multicast_ttl_option] = ACTIONS(1473),
    [sym_ip_ttl_subnet] = ACTIONS(1473),
    [sym_ip_ttl_site] = ACTIONS(1473),
    [sym_ip_ttl_region] = ACTIONS(1473),
    [sym_ip_ttl_continent] = ACTIONS(1473),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1473),
    [sym_ip_nodelay_on] = ACTIONS(1473),
    [sym_ip_nodelay_off] = ACTIONS(1473),
    [sym_amx_error] = ACTIONS(1473),
    [sym_amx_warning] = ACTIONS(1473),
    [sym_amx_info] = ACTIONS(1473),
    [sym_amx_debug] = ACTIONS(1473),
    [sym_netlinx_login_success] = ACTIONS(1473),
    [sym_netlinx_login_fail] = ACTIONS(1473),
    [sym_netlinx_logout] = ACTIONS(1473),
    [sym_null_str] = ACTIONS(1473),
    [sym_smtp_address] = ACTIONS(1473),
    [sym_smtp_port_number] = ACTIONS(1473),
    [sym_smtp_username] = ACTIONS(1473),
    [sym_smtp_password] = ACTIONS(1473),
    [sym_smtp_require_tls] = ACTIONS(1473),
    [sym_smtp_from] = ACTIONS(1473),
    [sym_smtp_tls_true] = ACTIONS(1473),
    [sym_smtp_tls_false] = ACTIONS(1473),
    [sym_listview_on_row_select_event] = ACTIONS(1473),
    [sym_data_structure_unknown] = ACTIONS(1473),
    [sym_data_structure_datafeed] = ACTIONS(1473),
    [sym_data_structure_datarecordset] = ACTIONS(1473),
    [sym_data_structure_datarecord] = ACTIONS(1473),
    [sym_data_structure_datafield] = ACTIONS(1473),
    [sym_data_max_content_field_count] = ACTIONS(1473),
    [sym_data_max_metadata_field_count] = ACTIONS(1473),
    [sym_data_max_id_length] = ACTIONS(1473),
    [sym_data_max_name_length] = ACTIONS(1473),
    [sym_data_max_description_length] = ACTIONS(1473),
    [sym_data_max_type_length] = ACTIONS(1473),
    [sym_data_max_format_length] = ACTIONS(1473),
    [sym_data_max_source_length] = ACTIONS(1473),
    [sym_data_max_value_length] = ACTIONS(1473),
    [sym_data_max_label_length] = ACTIONS(1473),
    [sym_data_max_url_length] = ACTIONS(1473),
    [sym_max_last_login_info_length] = ACTIONS(1473),
    [sym_data_type_unknown] = ACTIONS(1473),
    [sym_data_type_string] = ACTIONS(1473),
    [sym_data_type_datetime] = ACTIONS(1473),
    [sym_data_type_date] = ACTIONS(1473),
    [sym_data_type_time] = ACTIONS(1473),
    [sym_data_type_image] = ACTIONS(1473),
    [sym_data_format_url] = ACTIONS(1473),
    [sym_data_format_phone] = ACTIONS(1473),
    [sym_data_format_email] = ACTIONS(1473),
    [sym_data_format_iso8601] = ACTIONS(1473),
    [sym_auditlog_priv] = ACTIONS(1473),
    [sym_configuration_priv] = ACTIONS(1473),
    [sym_deviceconfig_priv] = ACTIONS(1473),
    [sym_ftp_priv] = ACTIONS(1473),
    [sym_http_priv] = ACTIONS(1473),
    [sym_networkconfig_priv] = ACTIONS(1473),
    [sym_programport_priv] = ACTIONS(1473),
    [sym_remoteui_priv] = ACTIONS(1473),
    [sym_securitycontrol_priv] = ACTIONS(1473),
    [sym_softwaremanagement_priv] = ACTIONS(1473),
    [sym_terminal_priv] = ACTIONS(1473),
    [sym_tpadmin_priv] = ACTIONS(1473),
    [sym_user1_priv] = ACTIONS(1473),
    [sym_user2_priv] = ACTIONS(1473),
    [sym_user3_priv] = ACTIONS(1473),
    [sym_user4_priv] = ACTIONS(1473),
    [sym_usermanagement_priv] = ACTIONS(1473),
    [sym_valid_account] = ACTIONS(1473),
    [sym_err_name_invalid] = ACTIONS(1473),
    [sym_err_password_invalid] = ACTIONS(1473),
    [sym_err_invalid_account] = ACTIONS(1473),
    [sym_err_no_authorization] = ACTIONS(1473),
    [sym_err_invalid_parameter] = ACTIONS(1473),
    [sym_err_user_locked_out] = ACTIONS(1473),
    [sym_err_user_account_expired] = ACTIONS(1473),
    [sym_err_general_error] = ACTIONS(1473),
    [sym_err_authentication_error] = ACTIONS(1473),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1473),
    [sym_tls_validate_certificate] = ACTIONS(1473),
    [sym_tdata] = ACTIONS(1473),
    [sym_tchannel] = ACTIONS(1473),
    [sym_tlevel] = ACTIONS(1473),
    [sym_tbutton] = ACTIONS(1473),
    [sym_ttimeline] = ACTIONS(1473),
    [sym_tcustom] = ACTIONS(1473),
    [sym_url_struct] = ACTIONS(1473),
    [sym_dns_struct] = ACTIONS(1473),
    [sym_ip_address_struct] = ACTIONS(1473),
    [sym_dev_info_struct] = ACTIONS(1473),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1473),
    [sym_data_feed] = ACTIONS(1473),
    [sym_data_field] = ACTIONS(1473),
    [sym_data_record] = ACTIONS(1473),
    [sym_wc_data_feed] = ACTIONS(1473),
    [sym_wc_data_field] = ACTIONS(1473),
    [sym_wc_data_record] = ACTIONS(1473),
    [sym_last_login_info] = ACTIONS(1473),
    [sym____reserved___] = ACTIONS(1473),
    [sym_date] = ACTIONS(1473),
    [sym_day] = ACTIONS(1473),
    [sym_time] = ACTIONS(1473),
    [sym_ldate] = ACTIONS(1473),
    [sym_push_device] = ACTIONS(1473),
    [sym_push_channel] = ACTIONS(1473),
    [sym_push_devchan] = ACTIONS(1473),
    [sym_release_device] = ACTIONS(1473),
    [sym_release_channel] = ACTIONS(1473),
    [sym_release_devchan] = ACTIONS(1473),
    [sym_master_slot] = ACTIONS(1473),
    [sym_get_pulse_time] = ACTIONS(1473),
    [sym_get_timer] = ACTIONS(1473),
    [sym_master_sn] = ACTIONS(1473),
    [sym_system_number] = ACTIONS(1473),
    [sym_button] = ACTIONS(1473),
    [sym_dv_channel] = ACTIONS(1473),
    [sym_data] = ACTIONS(1473),
    [sym_level] = ACTIONS(1473),
    [sym_timeline] = ACTIONS(1473),
    [sym_channel] = ACTIONS(1473),
    [sym_custom] = ACTIONS(1473),
    [sym_atoi] = ACTIONS(1473),
    [sym_atol] = ACTIONS(1473),
    [sym_atof] = ACTIONS(1473),
    [sym_itoa] = ACTIONS(1473),
    [sym_itohex] = ACTIONS(1473),
    [sym_hextoi] = ACTIONS(1473),
    [sym_ftoa] = ACTIONS(1473),
    [sym_find_string] = ACTIONS(1473),
    [sym_left_string] = ACTIONS(1473),
    [sym_right_string] = ACTIONS(1473),
    [sym_mid_string] = ACTIONS(1473),
    [sym_remove_string] = ACTIONS(1473),
    [sym_get_buffer_string] = ACTIONS(1473),
    [sym_get_buffer_char] = ACTIONS(1473),
    [sym_get_multi_buffer_string] = ACTIONS(1473),
    [sym_length_string] = ACTIONS(1473),
    [sym_max_length_string] = ACTIONS(1473),
    [sym_set_length_string] = ACTIONS(1473),
    [sym_compare_string] = ACTIONS(1473),
    [sym_upper_string] = ACTIONS(1473),
    [sym_lower_string] = ACTIONS(1473),
    [sym_get_last] = ACTIONS(1473),
    [sym_date_to_day] = ACTIONS(1473),
    [sym_date_to_month] = ACTIONS(1473),
    [sym_date_to_year] = ACTIONS(1473),
    [sym_time_to_hour] = ACTIONS(1473),
    [sym_time_to_minute] = ACTIONS(1473),
    [sym_time_to_second] = ACTIONS(1473),
    [sym_day_of_week] = ACTIONS(1473),
    [sym_set_pulse_time] = ACTIONS(1473),
    [sym_set_timer] = ACTIONS(1473),
    [sym_file_open] = ACTIONS(1473),
    [sym_file_close] = ACTIONS(1473),
    [sym_file_read] = ACTIONS(1473),
    [sym_file_read_line] = ACTIONS(1473),
    [sym_file_write] = ACTIONS(1473),
    [sym_file_write_line] = ACTIONS(1473),
    [sym_file_copy] = ACTIONS(1473),
    [sym_file_delete] = ACTIONS(1473),
    [sym_file_rename] = ACTIONS(1473),
    [sym_file_dir] = ACTIONS(1473),
    [sym_file_getdir] = ACTIONS(1473),
    [sym_file_setdir] = ACTIONS(1473),
    [sym_file_create_dir] = ACTIONS(1473),
    [sym_file_remove_dir] = ACTIONS(1473),
    [sym_file_seek] = ACTIONS(1473),
    [sym_length_array] = ACTIONS(1473),
    [sym_max_length_array] = ACTIONS(1473),
    [sym_set_length_array] = ACTIONS(1473),
    [sym_ip_client_open] = ACTIONS(1473),
    [sym_ip_client_close] = ACTIONS(1473),
    [sym_ip_server_open] = ACTIONS(1473),
    [sym_ip_server_close] = ACTIONS(1473),
    [sym_ip_bound_client_open] = ACTIONS(1473),
    [sym_ip_mc_server_open] = ACTIONS(1473),
    [sym_ip_set_option] = ACTIONS(1473),
    [sym_ssh_client_open] = ACTIONS(1473),
    [sym_ssh_client_close] = ACTIONS(1473),
    [sym_tls_client_open] = ACTIONS(1473),
    [sym_tls_client_close] = ACTIONS(1473),
    [sym_do_push] = ACTIONS(1473),
    [sym_do_push_timed] = ACTIONS(1473),
    [sym_do_release] = ACTIONS(1473),
    [sym_device_id] = ACTIONS(1473),
    [sym_device_info] = ACTIONS(1473),
    [sym_device_id_string] = ACTIONS(1473),
    [sym_device_standby] = ACTIONS(1473),
    [sym_device_wake] = ACTIONS(1473),
    [sym_timeline_create] = ACTIONS(1473),
    [sym_timeline_kill] = ACTIONS(1473),
    [sym_timeline_reload] = ACTIONS(1473),
    [sym_timeline_pause] = ACTIONS(1473),
    [sym_timeline_restart] = ACTIONS(1473),
    [sym_timeline_set] = ACTIONS(1473),
    [sym_timeline_get] = ACTIONS(1473),
    [sym_timeline_active] = ACTIONS(1473),
    [sym_timeline_dynamic_id] = ACTIONS(1473),
    [sym_variable_to_string] = ACTIONS(1473),
    [sym_string_to_variable] = ACTIONS(1473),
    [sym_length_variable_to_string] = ACTIONS(1473),
    [sym_type_cast] = ACTIONS(1473),
    [sym_variable_to_xml] = ACTIONS(1473),
    [sym_xml_to_variable] = ACTIONS(1473),
    [sym_length_variable_to_xml] = ACTIONS(1473),
    [sym_redirect_string] = ACTIONS(1473),
    [sym_random_number] = ACTIONS(1473),
    [sym_get_unique_id] = ACTIONS(1473),
    [sym_get_system_number] = ACTIONS(1473),
    [sym_set_system_number] = ACTIONS(1473),
    [sym_get_serial_number] = ACTIONS(1473),
    [sym_get_master_build] = ACTIONS(1473),
    [sym_set_validation_code] = ACTIONS(1473),
    [sym_reboot] = ACTIONS(1473),
    [sym_load_duet_module] = ACTIONS(1473),
    [sym_unload_duet_module] = ACTIONS(1473),
    [sym_duet_mem_size_set] = ACTIONS(1473),
    [sym_duet_mem_size_get] = ACTIONS(1473),
    [sym_dynamic_application_device] = ACTIONS(1473),
    [sym_dynamic_polled_port] = ACTIONS(1473),
    [sym_static_port_binding] = ACTIONS(1473),
    [sym_static_ip_binding] = ACTIONS(1473),
    [sym_get_dns_list] = ACTIONS(1473),
    [sym_set_dns_list] = ACTIONS(1473),
    [sym_get_ip_address] = ACTIONS(1473),
    [sym_set_ip_address] = ACTIONS(1473),
    [sym_get_url_list] = ACTIONS(1473),
    [sym_add_url_entry] = ACTIONS(1473),
    [sym_delete_url_entry] = ACTIONS(1473),
    [sym_abs_value] = ACTIONS(1473),
    [sym_max_value] = ACTIONS(1473),
    [sym_min_value] = ACTIONS(1473),
    [sym_exp_value] = ACTIONS(1473),
    [sym_log_value] = ACTIONS(1473),
    [sym_log10_value] = ACTIONS(1473),
    [sym_power_value] = ACTIONS(1473),
    [sym_sqrt_value] = ACTIONS(1473),
    [sym_raw_be] = ACTIONS(1473),
    [sym_raw_le] = ACTIONS(1473),
    [sym_internal_queue_size_set] = ACTIONS(1473),
    [sym_internal_queue_size_get] = ACTIONS(1473),
    [sym_internal_threshold_set] = ACTIONS(1473),
    [sym_internal_threshold_get] = ACTIONS(1473),
    [sym_rebuilt_event] = ACTIONS(1473),
    [sym_do_custom_event] = ACTIONS(1473),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1473),
    [sym_clkmgr_set_clk_source] = ACTIONS(1473),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1473),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1473),
    [sym_clkmgr_get_timezone] = ACTIONS(1473),
    [sym_clkmgr_set_timezone] = ACTIONS(1473),
    [sym_clkmgr_get_resync_period] = ACTIONS(1473),
    [sym_clkmgr_set_resync_period] = ACTIONS(1473),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1473),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1473),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1473),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1473),
    [sym_clkmgr_get_timeservers] = ACTIONS(1473),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1473),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1473),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1473),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1473),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1473),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1473),
    [sym_set_log_level] = ACTIONS(1473),
    [sym_get_log_level] = ACTIONS(1473),
    [sym_amx_log] = ACTIONS(1473),
    [sym_smtp_server_config_set] = ACTIONS(1473),
    [sym_smtp_server_config_get] = ACTIONS(1473),
    [sym_smtp_send] = ACTIONS(1473),
    [sym_validate_netlinx_account] = ACTIONS(1473),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1473),
    [sym_authenticate_certificate] = ACTIONS(1473),
    [sym_audit_netlinx_session_event] = ACTIONS(1473),
    [sym_audit_netlinx_generic_event] = ACTIONS(1473),
    [sym_get_available_flash_disk_space] = ACTIONS(1473),
    [sym_get_max_flash_disk_space] = ACTIONS(1473),
    [sym_data_create_feed] = ACTIONS(1473),
    [sym_data_delete_feed] = ACTIONS(1473),
    [sym_data_publish_feed] = ACTIONS(1473),
    [sym_data_get_published_feed] = ACTIONS(1473),
    [sym_data_add_record] = ACTIONS(1473),
    [sym_data_get_event_record] = ACTIONS(1473),
    [sym__wc_data_create_feed] = ACTIONS(1473),
    [sym__wc_data_add_record] = ACTIONS(1473),
    [sym__wc_data_get_event_record] = ACTIONS(1473),
    [sym_format] = ACTIONS(1473),
    [sym_astro_clock] = ACTIONS(1473),
    [sym_set_outdoor_temperature] = ACTIONS(1473),
    [sym_set_virtual_level_count] = ACTIONS(1473),
    [sym_set_virtual_channel_count] = ACTIONS(1473),
    [sym_set_virtual_port_count] = ACTIONS(1473),
    [sym__wc_encode_int] = ACTIONS(1473),
    [sym__wc_decode_int] = ACTIONS(1473),
    [sym__wc_int] = ACTIONS(1473),
    [sym__wc_to_ch_int] = ACTIONS(1473),
    [sym__ch_to_wc_int] = ACTIONS(1473),
    [sym__wc_find_string_int] = ACTIONS(1473),
    [sym__wc_left_string_int] = ACTIONS(1473),
    [sym__wc_lower_string_int] = ACTIONS(1473),
    [sym__wc_mid_string_int] = ACTIONS(1473),
    [sym__wc_remove_string_int] = ACTIONS(1473),
    [sym__wc_right_string_int] = ACTIONS(1473),
    [sym__wc_upper_string_int] = ACTIONS(1473),
    [sym__wc_compare_string_int] = ACTIONS(1473),
    [sym__wc_get_buffer_char_int] = ACTIONS(1473),
    [sym__wc_get_buffer_string_int] = ACTIONS(1473),
    [sym__wc_concat_string_int] = ACTIONS(1473),
    [sym__wc_file_open_int] = ACTIONS(1473),
    [sym__wc_file_close_int] = ACTIONS(1473),
    [sym__wc_file_read_int] = ACTIONS(1473),
    [sym__wc_file_read_line_int] = ACTIONS(1473),
    [sym__wc_file_write_int] = ACTIONS(1473),
    [sym__wc_file_write_line_int] = ACTIONS(1473),
    [aux_sym_preproc_include_token1] = ACTIONS(1473),
    [aux_sym_preproc_define_token1] = ACTIONS(1475),
    [aux_sym_preproc_warn_token1] = ACTIONS(1475),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1475),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1475),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1475),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1475),
    [anon_sym_RBRACE] = ACTIONS(1475),
    [anon_sym_COLON] = ACTIONS(1475),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1475),
    [anon_sym_BANG] = ACTIONS(1473),
    [anon_sym_TILDE] = ACTIONS(1475),
    [anon_sym_DASH] = ACTIONS(1473),
    [anon_sym_PLUS] = ACTIONS(1473),
    [anon_sym_STAR] = ACTIONS(1475),
    [anon_sym_SLASH] = ACTIONS(1473),
    [anon_sym_PERCENT] = ACTIONS(1475),
    [anon_sym_PIPE_PIPE] = ACTIONS(1475),
    [anon_sym_AMP_AMP] = ACTIONS(1475),
    [anon_sym_PIPE] = ACTIONS(1473),
    [anon_sym_CARET] = ACTIONS(1475),
    [anon_sym_AMP] = ACTIONS(1473),
    [anon_sym_EQ_EQ] = ACTIONS(1475),
    [anon_sym_BANG_EQ] = ACTIONS(1475),
    [anon_sym_LT_GT] = ACTIONS(1475),
    [anon_sym_GT] = ACTIONS(1473),
    [anon_sym_GT_EQ] = ACTIONS(1475),
    [anon_sym_LT_EQ] = ACTIONS(1475),
    [anon_sym_LT] = ACTIONS(1473),
    [anon_sym_LT_LT] = ACTIONS(1475),
    [anon_sym_GT_GT] = ACTIONS(1475),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1475),
    [anon_sym_SQUOTE] = ACTIONS(1475),
    [sym_number_literal] = ACTIONS(1475),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1475),
  },
  [STATE(161)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1477),
    [sym_define_call_keyword] = ACTIONS(1477),
    [sym_define_function_keyword] = ACTIONS(1477),
    [sym_define_library_function_keyword] = ACTIONS(1477),
    [sym_define_module_keyword] = ACTIONS(1477),
    [sym_char_keyword] = ACTIONS(1477),
    [sym_widechar_keyword] = ACTIONS(1477),
    [sym_integer_keyword] = ACTIONS(1477),
    [sym_sinteger_keyword] = ACTIONS(1477),
    [sym_long_keyword] = ACTIONS(1477),
    [sym_slong_keyword] = ACTIONS(1477),
    [sym_float_keyword] = ACTIONS(1477),
    [sym_double_keyword] = ACTIONS(1477),
    [sym_variant_keyword] = ACTIONS(1477),
    [sym_variantarray_keyword] = ACTIONS(1477),
    [sym_local_var_keyword] = ACTIONS(1477),
    [sym_stack_var_keyword] = ACTIONS(1477),
    [sym_constant_keyword] = ACTIONS(1477),
    [sym_volatile_keyword] = ACTIONS(1477),
    [sym_non_volatile_keyword] = ACTIONS(1477),
    [sym_persistent_keyword] = ACTIONS(1477),
    [sym_dev_keyword] = ACTIONS(1477),
    [sym_devlev_keyword] = ACTIONS(1477),
    [sym_devchan_keyword] = ACTIONS(1477),
    [sym_if_keyword] = ACTIONS(1477),
    [sym_else_keyword] = ACTIONS(1477),
    [sym_switch_keyword] = ACTIONS(1477),
    [sym_case_keyword] = ACTIONS(1477),
    [sym_default_keyword] = ACTIONS(1477),
    [sym_while_keyword] = ACTIONS(1477),
    [sym_for_keyword] = ACTIONS(1477),
    [sym_break_keyword] = ACTIONS(1477),
    [sym_continue_keyword] = ACTIONS(1477),
    [sym_return_keyword] = ACTIONS(1477),
    [sym_select_keyword] = ACTIONS(1477),
    [sym_active_keyword] = ACTIONS(1477),
    [sym_struct_keyword] = ACTIONS(1477),
    [sym_structure_keyword] = ACTIONS(1477),
    [aux_sym_on_keyword_token1] = ACTIONS(1477),
    [aux_sym_off_keyword_token1] = ACTIONS(1477),
    [sym_send_level_keyword] = ACTIONS(1477),
    [sym_send_string_keyword] = ACTIONS(1477),
    [sym_send_command_keyword] = ACTIONS(1477),
    [sym_clear_buffer_keyword] = ACTIONS(1477),
    [sym_create_buffer_keyword] = ACTIONS(1477),
    [sym_create_multi_buffer_keyword] = ACTIONS(1477),
    [sym_call_keyword] = ACTIONS(1477),
    [sym_system_call_keyword] = ACTIONS(1477),
    [sym_devchan_to_keyword] = ACTIONS(1477),
    [sym_devchan_min_to_keyword] = ACTIONS(1477),
    [sym_devchan_total_off_keyword] = ACTIONS(1477),
    [sym_devchan_pulse_keyword] = ACTIONS(1477),
    [sym_wait_keyword] = ACTIONS(1477),
    [sym_cancel_wait_keyword] = ACTIONS(1477),
    [sym_wait_until_keyword] = ACTIONS(1477),
    [sym_cancel_wait_until_keyword] = ACTIONS(1477),
    [sym_cancel_all_wait_keyword] = ACTIONS(1477),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1477),
    [sym_and] = ACTIONS(1477),
    [sym_or] = ACTIONS(1477),
    [sym_not] = ACTIONS(1477),
    [sym_xor] = ACTIONS(1477),
    [sym_band] = ACTIONS(1477),
    [sym_bor] = ACTIONS(1477),
    [sym_bxor] = ACTIONS(1477),
    [sym_bnot] = ACTIONS(1477),
    [sym_lshift] = ACTIONS(1477),
    [sym_rshift] = ACTIONS(1477),
    [sym___file__] = ACTIONS(1477),
    [sym___date__] = ACTIONS(1477),
    [sym___ldate__] = ACTIONS(1477),
    [sym___time__] = ACTIONS(1477),
    [sym___line__] = ACTIONS(1477),
    [sym___name__] = ACTIONS(1477),
    [sym___netlinx__] = ACTIONS(1477),
    [sym_netlinx_axi_version] = ACTIONS(1477),
    [sym_url_flg_tcp] = ACTIONS(1477),
    [sym_url_flg_accinfopresent] = ACTIONS(1477),
    [sym_url_flg_temp] = ACTIONS(1477),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1477),
    [sym_url_flg_stat_mask] = ACTIONS(1477),
    [sym_url_flg_stat_lookup] = ACTIONS(1477),
    [sym_url_flg_stat_connecting] = ACTIONS(1477),
    [sym_url_flg_stat_waiting] = ACTIONS(1477),
    [sym_url_flg_stat_connected] = ACTIONS(1477),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1477),
    [sym_first_virtual_device] = ACTIONS(1477),
    [sym_first_local_port] = ACTIONS(1477),
    [sym_dynamic_virtual_device] = ACTIONS(1477),
    [sym_timeline_once] = ACTIONS(1477),
    [sym_timeline_repeat] = ACTIONS(1477),
    [sym_timeline_absolute] = ACTIONS(1477),
    [sym_timeline_relative] = ACTIONS(1477),
    [sym_do_push_timed_infinite] = ACTIONS(1477),
    [sym_source_type_no_address] = ACTIONS(1477),
    [sym_source_type_neuron_id] = ACTIONS(1477),
    [sym_source_type_ip_address] = ACTIONS(1477),
    [sym_source_type_axlink] = ACTIONS(1477),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1477),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1477),
    [sym_source_type_ip_socket_address] = ACTIONS(1477),
    [sym_source_type_rs232] = ACTIONS(1477),
    [sym_source_type_internal] = ACTIONS(1477),
    [sym_source_type_mac_address] = ACTIONS(1477),
    [sym_source_type_ipv4_port] = ACTIONS(1477),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1477),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1477),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1477),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1477),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1477),
    [sym_true] = ACTIONS(1477),
    [sym_false] = ACTIONS(1477),
    [sym_file_read_only] = ACTIONS(1477),
    [sym_file_rw_new] = ACTIONS(1477),
    [sym_file_rw_append] = ACTIONS(1477),
    [sym_ip_tcp] = ACTIONS(1477),
    [sym_ip_udp] = ACTIONS(1477),
    [sym_ip_udp_2way] = ACTIONS(1477),
    [sym_xml_encode_types] = ACTIONS(1477),
    [sym_xml_encode_char_as_list] = ACTIONS(1477),
    [sym_xml_encode_le] = ACTIONS(1477),
    [sym_xml_decode_types] = ACTIONS(1477),
    [sym_xml_decode_no_preserve] = ACTIONS(1477),
    [sym_duet_dev_type_display_device] = ACTIONS(1477),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1477),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1477),
    [sym_duet_dev_type_io_device] = ACTIONS(1477),
    [sym_duet_dev_type_relay_device] = ACTIONS(1477),
    [sym_duet_dev_type_ups] = ACTIONS(1477),
    [sym_duet_dev_type_amplifier] = ACTIONS(1477),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1477),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1477),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1477),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1477),
    [sym_duet_dev_type_camera] = ACTIONS(1477),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1477),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1477),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1477),
    [sym_duet_dev_type_dss] = ACTIONS(1477),
    [sym_duet_dev_type_dvr] = ACTIONS(1477),
    [sym_duet_dev_type_disc_device] = ACTIONS(1477),
    [sym_duet_dev_type_document_camera] = ACTIONS(1477),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1477),
    [sym_duet_dev_type_hvac] = ACTIONS(1477),
    [sym_duet_dev_type_keypad] = ACTIONS(1477),
    [sym_duet_dev_type_light] = ACTIONS(1477),
    [sym_duet_dev_type_monitor] = ACTIONS(1477),
    [sym_duet_dev_type_motor] = ACTIONS(1477),
    [sym_duet_dev_type_multi_window] = ACTIONS(1477),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1477),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1477),
    [sym_duet_dev_type_receiver] = ACTIONS(1477),
    [sym_duet_dev_type_security_system] = ACTIONS(1477),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1477),
    [sym_duet_dev_type_settop_box] = ACTIONS(1477),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1477),
    [sym_duet_dev_type_switcher] = ACTIONS(1477),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1477),
    [sym_duet_dev_type_tv] = ACTIONS(1477),
    [sym_duet_dev_type_utility] = ACTIONS(1477),
    [sym_duet_dev_type_vcr] = ACTIONS(1477),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1477),
    [sym_duet_dev_type_video_processor] = ACTIONS(1477),
    [sym_duet_dev_type_video_projector] = ACTIONS(1477),
    [sym_duet_dev_type_video_wall] = ACTIONS(1477),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1477),
    [sym_duet_dev_type_weather] = ACTIONS(1477),
    [sym_duet_dev_not_polled] = ACTIONS(1477),
    [sym_duet_dev_polled] = ACTIONS(1477),
    [sym_clkmgr_mode_network] = ACTIONS(1477),
    [sym_clkmgr_mode_standalone] = ACTIONS(1477),
    [sym_normal_standby] = ACTIONS(1477),
    [sym_normal_wake] = ACTIONS(1477),
    [sym_ip_multicast_ttl_option] = ACTIONS(1477),
    [sym_ip_ttl_subnet] = ACTIONS(1477),
    [sym_ip_ttl_site] = ACTIONS(1477),
    [sym_ip_ttl_region] = ACTIONS(1477),
    [sym_ip_ttl_continent] = ACTIONS(1477),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1477),
    [sym_ip_nodelay_on] = ACTIONS(1477),
    [sym_ip_nodelay_off] = ACTIONS(1477),
    [sym_amx_error] = ACTIONS(1477),
    [sym_amx_warning] = ACTIONS(1477),
    [sym_amx_info] = ACTIONS(1477),
    [sym_amx_debug] = ACTIONS(1477),
    [sym_netlinx_login_success] = ACTIONS(1477),
    [sym_netlinx_login_fail] = ACTIONS(1477),
    [sym_netlinx_logout] = ACTIONS(1477),
    [sym_null_str] = ACTIONS(1477),
    [sym_smtp_address] = ACTIONS(1477),
    [sym_smtp_port_number] = ACTIONS(1477),
    [sym_smtp_username] = ACTIONS(1477),
    [sym_smtp_password] = ACTIONS(1477),
    [sym_smtp_require_tls] = ACTIONS(1477),
    [sym_smtp_from] = ACTIONS(1477),
    [sym_smtp_tls_true] = ACTIONS(1477),
    [sym_smtp_tls_false] = ACTIONS(1477),
    [sym_listview_on_row_select_event] = ACTIONS(1477),
    [sym_data_structure_unknown] = ACTIONS(1477),
    [sym_data_structure_datafeed] = ACTIONS(1477),
    [sym_data_structure_datarecordset] = ACTIONS(1477),
    [sym_data_structure_datarecord] = ACTIONS(1477),
    [sym_data_structure_datafield] = ACTIONS(1477),
    [sym_data_max_content_field_count] = ACTIONS(1477),
    [sym_data_max_metadata_field_count] = ACTIONS(1477),
    [sym_data_max_id_length] = ACTIONS(1477),
    [sym_data_max_name_length] = ACTIONS(1477),
    [sym_data_max_description_length] = ACTIONS(1477),
    [sym_data_max_type_length] = ACTIONS(1477),
    [sym_data_max_format_length] = ACTIONS(1477),
    [sym_data_max_source_length] = ACTIONS(1477),
    [sym_data_max_value_length] = ACTIONS(1477),
    [sym_data_max_label_length] = ACTIONS(1477),
    [sym_data_max_url_length] = ACTIONS(1477),
    [sym_max_last_login_info_length] = ACTIONS(1477),
    [sym_data_type_unknown] = ACTIONS(1477),
    [sym_data_type_string] = ACTIONS(1477),
    [sym_data_type_datetime] = ACTIONS(1477),
    [sym_data_type_date] = ACTIONS(1477),
    [sym_data_type_time] = ACTIONS(1477),
    [sym_data_type_image] = ACTIONS(1477),
    [sym_data_format_url] = ACTIONS(1477),
    [sym_data_format_phone] = ACTIONS(1477),
    [sym_data_format_email] = ACTIONS(1477),
    [sym_data_format_iso8601] = ACTIONS(1477),
    [sym_auditlog_priv] = ACTIONS(1477),
    [sym_configuration_priv] = ACTIONS(1477),
    [sym_deviceconfig_priv] = ACTIONS(1477),
    [sym_ftp_priv] = ACTIONS(1477),
    [sym_http_priv] = ACTIONS(1477),
    [sym_networkconfig_priv] = ACTIONS(1477),
    [sym_programport_priv] = ACTIONS(1477),
    [sym_remoteui_priv] = ACTIONS(1477),
    [sym_securitycontrol_priv] = ACTIONS(1477),
    [sym_softwaremanagement_priv] = ACTIONS(1477),
    [sym_terminal_priv] = ACTIONS(1477),
    [sym_tpadmin_priv] = ACTIONS(1477),
    [sym_user1_priv] = ACTIONS(1477),
    [sym_user2_priv] = ACTIONS(1477),
    [sym_user3_priv] = ACTIONS(1477),
    [sym_user4_priv] = ACTIONS(1477),
    [sym_usermanagement_priv] = ACTIONS(1477),
    [sym_valid_account] = ACTIONS(1477),
    [sym_err_name_invalid] = ACTIONS(1477),
    [sym_err_password_invalid] = ACTIONS(1477),
    [sym_err_invalid_account] = ACTIONS(1477),
    [sym_err_no_authorization] = ACTIONS(1477),
    [sym_err_invalid_parameter] = ACTIONS(1477),
    [sym_err_user_locked_out] = ACTIONS(1477),
    [sym_err_user_account_expired] = ACTIONS(1477),
    [sym_err_general_error] = ACTIONS(1477),
    [sym_err_authentication_error] = ACTIONS(1477),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1477),
    [sym_tls_validate_certificate] = ACTIONS(1477),
    [sym_tdata] = ACTIONS(1477),
    [sym_tchannel] = ACTIONS(1477),
    [sym_tlevel] = ACTIONS(1477),
    [sym_tbutton] = ACTIONS(1477),
    [sym_ttimeline] = ACTIONS(1477),
    [sym_tcustom] = ACTIONS(1477),
    [sym_url_struct] = ACTIONS(1477),
    [sym_dns_struct] = ACTIONS(1477),
    [sym_ip_address_struct] = ACTIONS(1477),
    [sym_dev_info_struct] = ACTIONS(1477),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1477),
    [sym_data_feed] = ACTIONS(1477),
    [sym_data_field] = ACTIONS(1477),
    [sym_data_record] = ACTIONS(1477),
    [sym_wc_data_feed] = ACTIONS(1477),
    [sym_wc_data_field] = ACTIONS(1477),
    [sym_wc_data_record] = ACTIONS(1477),
    [sym_last_login_info] = ACTIONS(1477),
    [sym____reserved___] = ACTIONS(1477),
    [sym_date] = ACTIONS(1477),
    [sym_day] = ACTIONS(1477),
    [sym_time] = ACTIONS(1477),
    [sym_ldate] = ACTIONS(1477),
    [sym_push_device] = ACTIONS(1477),
    [sym_push_channel] = ACTIONS(1477),
    [sym_push_devchan] = ACTIONS(1477),
    [sym_release_device] = ACTIONS(1477),
    [sym_release_channel] = ACTIONS(1477),
    [sym_release_devchan] = ACTIONS(1477),
    [sym_master_slot] = ACTIONS(1477),
    [sym_get_pulse_time] = ACTIONS(1477),
    [sym_get_timer] = ACTIONS(1477),
    [sym_master_sn] = ACTIONS(1477),
    [sym_system_number] = ACTIONS(1477),
    [sym_button] = ACTIONS(1477),
    [sym_dv_channel] = ACTIONS(1477),
    [sym_data] = ACTIONS(1477),
    [sym_level] = ACTIONS(1477),
    [sym_timeline] = ACTIONS(1477),
    [sym_channel] = ACTIONS(1477),
    [sym_custom] = ACTIONS(1477),
    [sym_atoi] = ACTIONS(1477),
    [sym_atol] = ACTIONS(1477),
    [sym_atof] = ACTIONS(1477),
    [sym_itoa] = ACTIONS(1477),
    [sym_itohex] = ACTIONS(1477),
    [sym_hextoi] = ACTIONS(1477),
    [sym_ftoa] = ACTIONS(1477),
    [sym_find_string] = ACTIONS(1477),
    [sym_left_string] = ACTIONS(1477),
    [sym_right_string] = ACTIONS(1477),
    [sym_mid_string] = ACTIONS(1477),
    [sym_remove_string] = ACTIONS(1477),
    [sym_get_buffer_string] = ACTIONS(1477),
    [sym_get_buffer_char] = ACTIONS(1477),
    [sym_get_multi_buffer_string] = ACTIONS(1477),
    [sym_length_string] = ACTIONS(1477),
    [sym_max_length_string] = ACTIONS(1477),
    [sym_set_length_string] = ACTIONS(1477),
    [sym_compare_string] = ACTIONS(1477),
    [sym_upper_string] = ACTIONS(1477),
    [sym_lower_string] = ACTIONS(1477),
    [sym_get_last] = ACTIONS(1477),
    [sym_date_to_day] = ACTIONS(1477),
    [sym_date_to_month] = ACTIONS(1477),
    [sym_date_to_year] = ACTIONS(1477),
    [sym_time_to_hour] = ACTIONS(1477),
    [sym_time_to_minute] = ACTIONS(1477),
    [sym_time_to_second] = ACTIONS(1477),
    [sym_day_of_week] = ACTIONS(1477),
    [sym_set_pulse_time] = ACTIONS(1477),
    [sym_set_timer] = ACTIONS(1477),
    [sym_file_open] = ACTIONS(1477),
    [sym_file_close] = ACTIONS(1477),
    [sym_file_read] = ACTIONS(1477),
    [sym_file_read_line] = ACTIONS(1477),
    [sym_file_write] = ACTIONS(1477),
    [sym_file_write_line] = ACTIONS(1477),
    [sym_file_copy] = ACTIONS(1477),
    [sym_file_delete] = ACTIONS(1477),
    [sym_file_rename] = ACTIONS(1477),
    [sym_file_dir] = ACTIONS(1477),
    [sym_file_getdir] = ACTIONS(1477),
    [sym_file_setdir] = ACTIONS(1477),
    [sym_file_create_dir] = ACTIONS(1477),
    [sym_file_remove_dir] = ACTIONS(1477),
    [sym_file_seek] = ACTIONS(1477),
    [sym_length_array] = ACTIONS(1477),
    [sym_max_length_array] = ACTIONS(1477),
    [sym_set_length_array] = ACTIONS(1477),
    [sym_ip_client_open] = ACTIONS(1477),
    [sym_ip_client_close] = ACTIONS(1477),
    [sym_ip_server_open] = ACTIONS(1477),
    [sym_ip_server_close] = ACTIONS(1477),
    [sym_ip_bound_client_open] = ACTIONS(1477),
    [sym_ip_mc_server_open] = ACTIONS(1477),
    [sym_ip_set_option] = ACTIONS(1477),
    [sym_ssh_client_open] = ACTIONS(1477),
    [sym_ssh_client_close] = ACTIONS(1477),
    [sym_tls_client_open] = ACTIONS(1477),
    [sym_tls_client_close] = ACTIONS(1477),
    [sym_do_push] = ACTIONS(1477),
    [sym_do_push_timed] = ACTIONS(1477),
    [sym_do_release] = ACTIONS(1477),
    [sym_device_id] = ACTIONS(1477),
    [sym_device_info] = ACTIONS(1477),
    [sym_device_id_string] = ACTIONS(1477),
    [sym_device_standby] = ACTIONS(1477),
    [sym_device_wake] = ACTIONS(1477),
    [sym_timeline_create] = ACTIONS(1477),
    [sym_timeline_kill] = ACTIONS(1477),
    [sym_timeline_reload] = ACTIONS(1477),
    [sym_timeline_pause] = ACTIONS(1477),
    [sym_timeline_restart] = ACTIONS(1477),
    [sym_timeline_set] = ACTIONS(1477),
    [sym_timeline_get] = ACTIONS(1477),
    [sym_timeline_active] = ACTIONS(1477),
    [sym_timeline_dynamic_id] = ACTIONS(1477),
    [sym_variable_to_string] = ACTIONS(1477),
    [sym_string_to_variable] = ACTIONS(1477),
    [sym_length_variable_to_string] = ACTIONS(1477),
    [sym_type_cast] = ACTIONS(1477),
    [sym_variable_to_xml] = ACTIONS(1477),
    [sym_xml_to_variable] = ACTIONS(1477),
    [sym_length_variable_to_xml] = ACTIONS(1477),
    [sym_redirect_string] = ACTIONS(1477),
    [sym_random_number] = ACTIONS(1477),
    [sym_get_unique_id] = ACTIONS(1477),
    [sym_get_system_number] = ACTIONS(1477),
    [sym_set_system_number] = ACTIONS(1477),
    [sym_get_serial_number] = ACTIONS(1477),
    [sym_get_master_build] = ACTIONS(1477),
    [sym_set_validation_code] = ACTIONS(1477),
    [sym_reboot] = ACTIONS(1477),
    [sym_load_duet_module] = ACTIONS(1477),
    [sym_unload_duet_module] = ACTIONS(1477),
    [sym_duet_mem_size_set] = ACTIONS(1477),
    [sym_duet_mem_size_get] = ACTIONS(1477),
    [sym_dynamic_application_device] = ACTIONS(1477),
    [sym_dynamic_polled_port] = ACTIONS(1477),
    [sym_static_port_binding] = ACTIONS(1477),
    [sym_static_ip_binding] = ACTIONS(1477),
    [sym_get_dns_list] = ACTIONS(1477),
    [sym_set_dns_list] = ACTIONS(1477),
    [sym_get_ip_address] = ACTIONS(1477),
    [sym_set_ip_address] = ACTIONS(1477),
    [sym_get_url_list] = ACTIONS(1477),
    [sym_add_url_entry] = ACTIONS(1477),
    [sym_delete_url_entry] = ACTIONS(1477),
    [sym_abs_value] = ACTIONS(1477),
    [sym_max_value] = ACTIONS(1477),
    [sym_min_value] = ACTIONS(1477),
    [sym_exp_value] = ACTIONS(1477),
    [sym_log_value] = ACTIONS(1477),
    [sym_log10_value] = ACTIONS(1477),
    [sym_power_value] = ACTIONS(1477),
    [sym_sqrt_value] = ACTIONS(1477),
    [sym_raw_be] = ACTIONS(1477),
    [sym_raw_le] = ACTIONS(1477),
    [sym_internal_queue_size_set] = ACTIONS(1477),
    [sym_internal_queue_size_get] = ACTIONS(1477),
    [sym_internal_threshold_set] = ACTIONS(1477),
    [sym_internal_threshold_get] = ACTIONS(1477),
    [sym_rebuilt_event] = ACTIONS(1477),
    [sym_do_custom_event] = ACTIONS(1477),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1477),
    [sym_clkmgr_set_clk_source] = ACTIONS(1477),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1477),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1477),
    [sym_clkmgr_get_timezone] = ACTIONS(1477),
    [sym_clkmgr_set_timezone] = ACTIONS(1477),
    [sym_clkmgr_get_resync_period] = ACTIONS(1477),
    [sym_clkmgr_set_resync_period] = ACTIONS(1477),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1477),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1477),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1477),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1477),
    [sym_clkmgr_get_timeservers] = ACTIONS(1477),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1477),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1477),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1477),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1477),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1477),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1477),
    [sym_set_log_level] = ACTIONS(1477),
    [sym_get_log_level] = ACTIONS(1477),
    [sym_amx_log] = ACTIONS(1477),
    [sym_smtp_server_config_set] = ACTIONS(1477),
    [sym_smtp_server_config_get] = ACTIONS(1477),
    [sym_smtp_send] = ACTIONS(1477),
    [sym_validate_netlinx_account] = ACTIONS(1477),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1477),
    [sym_authenticate_certificate] = ACTIONS(1477),
    [sym_audit_netlinx_session_event] = ACTIONS(1477),
    [sym_audit_netlinx_generic_event] = ACTIONS(1477),
    [sym_get_available_flash_disk_space] = ACTIONS(1477),
    [sym_get_max_flash_disk_space] = ACTIONS(1477),
    [sym_data_create_feed] = ACTIONS(1477),
    [sym_data_delete_feed] = ACTIONS(1477),
    [sym_data_publish_feed] = ACTIONS(1477),
    [sym_data_get_published_feed] = ACTIONS(1477),
    [sym_data_add_record] = ACTIONS(1477),
    [sym_data_get_event_record] = ACTIONS(1477),
    [sym__wc_data_create_feed] = ACTIONS(1477),
    [sym__wc_data_add_record] = ACTIONS(1477),
    [sym__wc_data_get_event_record] = ACTIONS(1477),
    [sym_format] = ACTIONS(1477),
    [sym_astro_clock] = ACTIONS(1477),
    [sym_set_outdoor_temperature] = ACTIONS(1477),
    [sym_set_virtual_level_count] = ACTIONS(1477),
    [sym_set_virtual_channel_count] = ACTIONS(1477),
    [sym_set_virtual_port_count] = ACTIONS(1477),
    [sym__wc_encode_int] = ACTIONS(1477),
    [sym__wc_decode_int] = ACTIONS(1477),
    [sym__wc_int] = ACTIONS(1477),
    [sym__wc_to_ch_int] = ACTIONS(1477),
    [sym__ch_to_wc_int] = ACTIONS(1477),
    [sym__wc_find_string_int] = ACTIONS(1477),
    [sym__wc_left_string_int] = ACTIONS(1477),
    [sym__wc_lower_string_int] = ACTIONS(1477),
    [sym__wc_mid_string_int] = ACTIONS(1477),
    [sym__wc_remove_string_int] = ACTIONS(1477),
    [sym__wc_right_string_int] = ACTIONS(1477),
    [sym__wc_upper_string_int] = ACTIONS(1477),
    [sym__wc_compare_string_int] = ACTIONS(1477),
    [sym__wc_get_buffer_char_int] = ACTIONS(1477),
    [sym__wc_get_buffer_string_int] = ACTIONS(1477),
    [sym__wc_concat_string_int] = ACTIONS(1477),
    [sym__wc_file_open_int] = ACTIONS(1477),
    [sym__wc_file_close_int] = ACTIONS(1477),
    [sym__wc_file_read_int] = ACTIONS(1477),
    [sym__wc_file_read_line_int] = ACTIONS(1477),
    [sym__wc_file_write_int] = ACTIONS(1477),
    [sym__wc_file_write_line_int] = ACTIONS(1477),
    [aux_sym_preproc_include_token1] = ACTIONS(1477),
    [aux_sym_preproc_define_token1] = ACTIONS(1479),
    [aux_sym_preproc_warn_token1] = ACTIONS(1479),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1479),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1479),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1479),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1479),
    [anon_sym_RBRACE] = ACTIONS(1479),
    [anon_sym_COLON] = ACTIONS(1479),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1479),
    [anon_sym_BANG] = ACTIONS(1477),
    [anon_sym_TILDE] = ACTIONS(1479),
    [anon_sym_DASH] = ACTIONS(1477),
    [anon_sym_PLUS] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(1479),
    [anon_sym_SLASH] = ACTIONS(1477),
    [anon_sym_PERCENT] = ACTIONS(1479),
    [anon_sym_PIPE_PIPE] = ACTIONS(1479),
    [anon_sym_AMP_AMP] = ACTIONS(1479),
    [anon_sym_PIPE] = ACTIONS(1477),
    [anon_sym_CARET] = ACTIONS(1479),
    [anon_sym_AMP] = ACTIONS(1477),
    [anon_sym_EQ_EQ] = ACTIONS(1479),
    [anon_sym_BANG_EQ] = ACTIONS(1479),
    [anon_sym_LT_GT] = ACTIONS(1479),
    [anon_sym_GT] = ACTIONS(1477),
    [anon_sym_GT_EQ] = ACTIONS(1479),
    [anon_sym_LT_EQ] = ACTIONS(1479),
    [anon_sym_LT] = ACTIONS(1477),
    [anon_sym_LT_LT] = ACTIONS(1479),
    [anon_sym_GT_GT] = ACTIONS(1479),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1479),
    [anon_sym_SQUOTE] = ACTIONS(1479),
    [sym_number_literal] = ACTIONS(1479),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1479),
  },
  [STATE(162)] = {
    [aux_sym_subscript_expression_repeat1] = STATE(155),
    [sym_identifier] = ACTIONS(1488),
    [sym_define_call_keyword] = ACTIONS(1488),
    [sym_define_function_keyword] = ACTIONS(1488),
    [sym_define_library_function_keyword] = ACTIONS(1488),
    [sym_define_module_keyword] = ACTIONS(1488),
    [sym_char_keyword] = ACTIONS(1488),
    [sym_widechar_keyword] = ACTIONS(1488),
    [sym_integer_keyword] = ACTIONS(1488),
    [sym_sinteger_keyword] = ACTIONS(1488),
    [sym_long_keyword] = ACTIONS(1488),
    [sym_slong_keyword] = ACTIONS(1488),
    [sym_float_keyword] = ACTIONS(1488),
    [sym_double_keyword] = ACTIONS(1488),
    [sym_variant_keyword] = ACTIONS(1488),
    [sym_variantarray_keyword] = ACTIONS(1488),
    [sym_local_var_keyword] = ACTIONS(1488),
    [sym_stack_var_keyword] = ACTIONS(1488),
    [sym_constant_keyword] = ACTIONS(1488),
    [sym_volatile_keyword] = ACTIONS(1488),
    [sym_non_volatile_keyword] = ACTIONS(1488),
    [sym_persistent_keyword] = ACTIONS(1488),
    [sym_dev_keyword] = ACTIONS(1488),
    [sym_devlev_keyword] = ACTIONS(1488),
    [sym_devchan_keyword] = ACTIONS(1488),
    [sym_if_keyword] = ACTIONS(1488),
    [sym_else_keyword] = ACTIONS(1488),
    [sym_switch_keyword] = ACTIONS(1488),
    [sym_case_keyword] = ACTIONS(1488),
    [sym_default_keyword] = ACTIONS(1488),
    [sym_while_keyword] = ACTIONS(1488),
    [sym_for_keyword] = ACTIONS(1488),
    [sym_break_keyword] = ACTIONS(1488),
    [sym_continue_keyword] = ACTIONS(1488),
    [sym_return_keyword] = ACTIONS(1488),
    [sym_select_keyword] = ACTIONS(1488),
    [sym_active_keyword] = ACTIONS(1488),
    [sym_struct_keyword] = ACTIONS(1488),
    [sym_structure_keyword] = ACTIONS(1488),
    [aux_sym_on_keyword_token1] = ACTIONS(1488),
    [aux_sym_off_keyword_token1] = ACTIONS(1488),
    [sym_send_level_keyword] = ACTIONS(1488),
    [sym_send_string_keyword] = ACTIONS(1488),
    [sym_send_command_keyword] = ACTIONS(1488),
    [sym_clear_buffer_keyword] = ACTIONS(1488),
    [sym_create_buffer_keyword] = ACTIONS(1488),
    [sym_create_multi_buffer_keyword] = ACTIONS(1488),
    [sym_call_keyword] = ACTIONS(1488),
    [sym_system_call_keyword] = ACTIONS(1488),
    [sym_devchan_to_keyword] = ACTIONS(1488),
    [sym_devchan_min_to_keyword] = ACTIONS(1488),
    [sym_devchan_total_off_keyword] = ACTIONS(1488),
    [sym_devchan_pulse_keyword] = ACTIONS(1488),
    [sym_wait_keyword] = ACTIONS(1488),
    [sym_cancel_wait_keyword] = ACTIONS(1488),
    [sym_wait_until_keyword] = ACTIONS(1488),
    [sym_cancel_wait_until_keyword] = ACTIONS(1488),
    [sym_cancel_all_wait_keyword] = ACTIONS(1488),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1488),
    [sym_and] = ACTIONS(1488),
    [sym_or] = ACTIONS(1488),
    [sym_not] = ACTIONS(1488),
    [sym_xor] = ACTIONS(1488),
    [sym_band] = ACTIONS(1488),
    [sym_bor] = ACTIONS(1488),
    [sym_bxor] = ACTIONS(1488),
    [sym_bnot] = ACTIONS(1488),
    [sym_lshift] = ACTIONS(1488),
    [sym_rshift] = ACTIONS(1488),
    [sym___file__] = ACTIONS(1488),
    [sym___date__] = ACTIONS(1488),
    [sym___ldate__] = ACTIONS(1488),
    [sym___time__] = ACTIONS(1488),
    [sym___line__] = ACTIONS(1488),
    [sym___name__] = ACTIONS(1488),
    [sym___netlinx__] = ACTIONS(1488),
    [sym_netlinx_axi_version] = ACTIONS(1488),
    [sym_url_flg_tcp] = ACTIONS(1488),
    [sym_url_flg_accinfopresent] = ACTIONS(1488),
    [sym_url_flg_temp] = ACTIONS(1488),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1488),
    [sym_url_flg_stat_mask] = ACTIONS(1488),
    [sym_url_flg_stat_lookup] = ACTIONS(1488),
    [sym_url_flg_stat_connecting] = ACTIONS(1488),
    [sym_url_flg_stat_waiting] = ACTIONS(1488),
    [sym_url_flg_stat_connected] = ACTIONS(1488),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1488),
    [sym_first_virtual_device] = ACTIONS(1488),
    [sym_first_local_port] = ACTIONS(1488),
    [sym_dynamic_virtual_device] = ACTIONS(1488),
    [sym_timeline_once] = ACTIONS(1488),
    [sym_timeline_repeat] = ACTIONS(1488),
    [sym_timeline_absolute] = ACTIONS(1488),
    [sym_timeline_relative] = ACTIONS(1488),
    [sym_do_push_timed_infinite] = ACTIONS(1488),
    [sym_source_type_no_address] = ACTIONS(1488),
    [sym_source_type_neuron_id] = ACTIONS(1488),
    [sym_source_type_ip_address] = ACTIONS(1488),
    [sym_source_type_axlink] = ACTIONS(1488),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1488),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1488),
    [sym_source_type_ip_socket_address] = ACTIONS(1488),
    [sym_source_type_rs232] = ACTIONS(1488),
    [sym_source_type_internal] = ACTIONS(1488),
    [sym_source_type_mac_address] = ACTIONS(1488),
    [sym_source_type_ipv4_port] = ACTIONS(1488),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1488),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1488),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1488),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1488),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1488),
    [sym_true] = ACTIONS(1488),
    [sym_false] = ACTIONS(1488),
    [sym_file_read_only] = ACTIONS(1488),
    [sym_file_rw_new] = ACTIONS(1488),
    [sym_file_rw_append] = ACTIONS(1488),
    [sym_ip_tcp] = ACTIONS(1488),
    [sym_ip_udp] = ACTIONS(1488),
    [sym_ip_udp_2way] = ACTIONS(1488),
    [sym_xml_encode_types] = ACTIONS(1488),
    [sym_xml_encode_char_as_list] = ACTIONS(1488),
    [sym_xml_encode_le] = ACTIONS(1488),
    [sym_xml_decode_types] = ACTIONS(1488),
    [sym_xml_decode_no_preserve] = ACTIONS(1488),
    [sym_duet_dev_type_display_device] = ACTIONS(1488),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1488),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1488),
    [sym_duet_dev_type_io_device] = ACTIONS(1488),
    [sym_duet_dev_type_relay_device] = ACTIONS(1488),
    [sym_duet_dev_type_ups] = ACTIONS(1488),
    [sym_duet_dev_type_amplifier] = ACTIONS(1488),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1488),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1488),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1488),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1488),
    [sym_duet_dev_type_camera] = ACTIONS(1488),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1488),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1488),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1488),
    [sym_duet_dev_type_dss] = ACTIONS(1488),
    [sym_duet_dev_type_dvr] = ACTIONS(1488),
    [sym_duet_dev_type_disc_device] = ACTIONS(1488),
    [sym_duet_dev_type_document_camera] = ACTIONS(1488),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1488),
    [sym_duet_dev_type_hvac] = ACTIONS(1488),
    [sym_duet_dev_type_keypad] = ACTIONS(1488),
    [sym_duet_dev_type_light] = ACTIONS(1488),
    [sym_duet_dev_type_monitor] = ACTIONS(1488),
    [sym_duet_dev_type_motor] = ACTIONS(1488),
    [sym_duet_dev_type_multi_window] = ACTIONS(1488),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1488),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1488),
    [sym_duet_dev_type_receiver] = ACTIONS(1488),
    [sym_duet_dev_type_security_system] = ACTIONS(1488),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1488),
    [sym_duet_dev_type_settop_box] = ACTIONS(1488),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1488),
    [sym_duet_dev_type_switcher] = ACTIONS(1488),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1488),
    [sym_duet_dev_type_tv] = ACTIONS(1488),
    [sym_duet_dev_type_utility] = ACTIONS(1488),
    [sym_duet_dev_type_vcr] = ACTIONS(1488),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1488),
    [sym_duet_dev_type_video_processor] = ACTIONS(1488),
    [sym_duet_dev_type_video_projector] = ACTIONS(1488),
    [sym_duet_dev_type_video_wall] = ACTIONS(1488),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1488),
    [sym_duet_dev_type_weather] = ACTIONS(1488),
    [sym_duet_dev_not_polled] = ACTIONS(1488),
    [sym_duet_dev_polled] = ACTIONS(1488),
    [sym_clkmgr_mode_network] = ACTIONS(1488),
    [sym_clkmgr_mode_standalone] = ACTIONS(1488),
    [sym_normal_standby] = ACTIONS(1488),
    [sym_normal_wake] = ACTIONS(1488),
    [sym_ip_multicast_ttl_option] = ACTIONS(1488),
    [sym_ip_ttl_subnet] = ACTIONS(1488),
    [sym_ip_ttl_site] = ACTIONS(1488),
    [sym_ip_ttl_region] = ACTIONS(1488),
    [sym_ip_ttl_continent] = ACTIONS(1488),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1488),
    [sym_ip_nodelay_on] = ACTIONS(1488),
    [sym_ip_nodelay_off] = ACTIONS(1488),
    [sym_amx_error] = ACTIONS(1488),
    [sym_amx_warning] = ACTIONS(1488),
    [sym_amx_info] = ACTIONS(1488),
    [sym_amx_debug] = ACTIONS(1488),
    [sym_netlinx_login_success] = ACTIONS(1488),
    [sym_netlinx_login_fail] = ACTIONS(1488),
    [sym_netlinx_logout] = ACTIONS(1488),
    [sym_null_str] = ACTIONS(1488),
    [sym_smtp_address] = ACTIONS(1488),
    [sym_smtp_port_number] = ACTIONS(1488),
    [sym_smtp_username] = ACTIONS(1488),
    [sym_smtp_password] = ACTIONS(1488),
    [sym_smtp_require_tls] = ACTIONS(1488),
    [sym_smtp_from] = ACTIONS(1488),
    [sym_smtp_tls_true] = ACTIONS(1488),
    [sym_smtp_tls_false] = ACTIONS(1488),
    [sym_listview_on_row_select_event] = ACTIONS(1488),
    [sym_data_structure_unknown] = ACTIONS(1488),
    [sym_data_structure_datafeed] = ACTIONS(1488),
    [sym_data_structure_datarecordset] = ACTIONS(1488),
    [sym_data_structure_datarecord] = ACTIONS(1488),
    [sym_data_structure_datafield] = ACTIONS(1488),
    [sym_data_max_content_field_count] = ACTIONS(1488),
    [sym_data_max_metadata_field_count] = ACTIONS(1488),
    [sym_data_max_id_length] = ACTIONS(1488),
    [sym_data_max_name_length] = ACTIONS(1488),
    [sym_data_max_description_length] = ACTIONS(1488),
    [sym_data_max_type_length] = ACTIONS(1488),
    [sym_data_max_format_length] = ACTIONS(1488),
    [sym_data_max_source_length] = ACTIONS(1488),
    [sym_data_max_value_length] = ACTIONS(1488),
    [sym_data_max_label_length] = ACTIONS(1488),
    [sym_data_max_url_length] = ACTIONS(1488),
    [sym_max_last_login_info_length] = ACTIONS(1488),
    [sym_data_type_unknown] = ACTIONS(1488),
    [sym_data_type_string] = ACTIONS(1488),
    [sym_data_type_datetime] = ACTIONS(1488),
    [sym_data_type_date] = ACTIONS(1488),
    [sym_data_type_time] = ACTIONS(1488),
    [sym_data_type_image] = ACTIONS(1488),
    [sym_data_format_url] = ACTIONS(1488),
    [sym_data_format_phone] = ACTIONS(1488),
    [sym_data_format_email] = ACTIONS(1488),
    [sym_data_format_iso8601] = ACTIONS(1488),
    [sym_auditlog_priv] = ACTIONS(1488),
    [sym_configuration_priv] = ACTIONS(1488),
    [sym_deviceconfig_priv] = ACTIONS(1488),
    [sym_ftp_priv] = ACTIONS(1488),
    [sym_http_priv] = ACTIONS(1488),
    [sym_networkconfig_priv] = ACTIONS(1488),
    [sym_programport_priv] = ACTIONS(1488),
    [sym_remoteui_priv] = ACTIONS(1488),
    [sym_securitycontrol_priv] = ACTIONS(1488),
    [sym_softwaremanagement_priv] = ACTIONS(1488),
    [sym_terminal_priv] = ACTIONS(1488),
    [sym_tpadmin_priv] = ACTIONS(1488),
    [sym_user1_priv] = ACTIONS(1488),
    [sym_user2_priv] = ACTIONS(1488),
    [sym_user3_priv] = ACTIONS(1488),
    [sym_user4_priv] = ACTIONS(1488),
    [sym_usermanagement_priv] = ACTIONS(1488),
    [sym_valid_account] = ACTIONS(1488),
    [sym_err_name_invalid] = ACTIONS(1488),
    [sym_err_password_invalid] = ACTIONS(1488),
    [sym_err_invalid_account] = ACTIONS(1488),
    [sym_err_no_authorization] = ACTIONS(1488),
    [sym_err_invalid_parameter] = ACTIONS(1488),
    [sym_err_user_locked_out] = ACTIONS(1488),
    [sym_err_user_account_expired] = ACTIONS(1488),
    [sym_err_general_error] = ACTIONS(1488),
    [sym_err_authentication_error] = ACTIONS(1488),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1488),
    [sym_tls_validate_certificate] = ACTIONS(1488),
    [sym_tdata] = ACTIONS(1488),
    [sym_tchannel] = ACTIONS(1488),
    [sym_tlevel] = ACTIONS(1488),
    [sym_tbutton] = ACTIONS(1488),
    [sym_ttimeline] = ACTIONS(1488),
    [sym_tcustom] = ACTIONS(1488),
    [sym_url_struct] = ACTIONS(1488),
    [sym_dns_struct] = ACTIONS(1488),
    [sym_ip_address_struct] = ACTIONS(1488),
    [sym_dev_info_struct] = ACTIONS(1488),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1488),
    [sym_data_feed] = ACTIONS(1488),
    [sym_data_field] = ACTIONS(1488),
    [sym_data_record] = ACTIONS(1488),
    [sym_wc_data_feed] = ACTIONS(1488),
    [sym_wc_data_field] = ACTIONS(1488),
    [sym_wc_data_record] = ACTIONS(1488),
    [sym_last_login_info] = ACTIONS(1488),
    [sym____reserved___] = ACTIONS(1488),
    [sym_date] = ACTIONS(1488),
    [sym_day] = ACTIONS(1488),
    [sym_time] = ACTIONS(1488),
    [sym_ldate] = ACTIONS(1488),
    [sym_push_device] = ACTIONS(1488),
    [sym_push_channel] = ACTIONS(1488),
    [sym_push_devchan] = ACTIONS(1488),
    [sym_release_device] = ACTIONS(1488),
    [sym_release_channel] = ACTIONS(1488),
    [sym_release_devchan] = ACTIONS(1488),
    [sym_master_slot] = ACTIONS(1488),
    [sym_get_pulse_time] = ACTIONS(1488),
    [sym_get_timer] = ACTIONS(1488),
    [sym_master_sn] = ACTIONS(1488),
    [sym_system_number] = ACTIONS(1488),
    [sym_button] = ACTIONS(1488),
    [sym_dv_channel] = ACTIONS(1488),
    [sym_data] = ACTIONS(1488),
    [sym_level] = ACTIONS(1488),
    [sym_timeline] = ACTIONS(1488),
    [sym_channel] = ACTIONS(1488),
    [sym_custom] = ACTIONS(1488),
    [sym_atoi] = ACTIONS(1488),
    [sym_atol] = ACTIONS(1488),
    [sym_atof] = ACTIONS(1488),
    [sym_itoa] = ACTIONS(1488),
    [sym_itohex] = ACTIONS(1488),
    [sym_hextoi] = ACTIONS(1488),
    [sym_ftoa] = ACTIONS(1488),
    [sym_find_string] = ACTIONS(1488),
    [sym_left_string] = ACTIONS(1488),
    [sym_right_string] = ACTIONS(1488),
    [sym_mid_string] = ACTIONS(1488),
    [sym_remove_string] = ACTIONS(1488),
    [sym_get_buffer_string] = ACTIONS(1488),
    [sym_get_buffer_char] = ACTIONS(1488),
    [sym_get_multi_buffer_string] = ACTIONS(1488),
    [sym_length_string] = ACTIONS(1488),
    [sym_max_length_string] = ACTIONS(1488),
    [sym_set_length_string] = ACTIONS(1488),
    [sym_compare_string] = ACTIONS(1488),
    [sym_upper_string] = ACTIONS(1488),
    [sym_lower_string] = ACTIONS(1488),
    [sym_get_last] = ACTIONS(1488),
    [sym_date_to_day] = ACTIONS(1488),
    [sym_date_to_month] = ACTIONS(1488),
    [sym_date_to_year] = ACTIONS(1488),
    [sym_time_to_hour] = ACTIONS(1488),
    [sym_time_to_minute] = ACTIONS(1488),
    [sym_time_to_second] = ACTIONS(1488),
    [sym_day_of_week] = ACTIONS(1488),
    [sym_set_pulse_time] = ACTIONS(1488),
    [sym_set_timer] = ACTIONS(1488),
    [sym_file_open] = ACTIONS(1488),
    [sym_file_close] = ACTIONS(1488),
    [sym_file_read] = ACTIONS(1488),
    [sym_file_read_line] = ACTIONS(1488),
    [sym_file_write] = ACTIONS(1488),
    [sym_file_write_line] = ACTIONS(1488),
    [sym_file_copy] = ACTIONS(1488),
    [sym_file_delete] = ACTIONS(1488),
    [sym_file_rename] = ACTIONS(1488),
    [sym_file_dir] = ACTIONS(1488),
    [sym_file_getdir] = ACTIONS(1488),
    [sym_file_setdir] = ACTIONS(1488),
    [sym_file_create_dir] = ACTIONS(1488),
    [sym_file_remove_dir] = ACTIONS(1488),
    [sym_file_seek] = ACTIONS(1488),
    [sym_length_array] = ACTIONS(1488),
    [sym_max_length_array] = ACTIONS(1488),
    [sym_set_length_array] = ACTIONS(1488),
    [sym_ip_client_open] = ACTIONS(1488),
    [sym_ip_client_close] = ACTIONS(1488),
    [sym_ip_server_open] = ACTIONS(1488),
    [sym_ip_server_close] = ACTIONS(1488),
    [sym_ip_bound_client_open] = ACTIONS(1488),
    [sym_ip_mc_server_open] = ACTIONS(1488),
    [sym_ip_set_option] = ACTIONS(1488),
    [sym_ssh_client_open] = ACTIONS(1488),
    [sym_ssh_client_close] = ACTIONS(1488),
    [sym_tls_client_open] = ACTIONS(1488),
    [sym_tls_client_close] = ACTIONS(1488),
    [sym_do_push] = ACTIONS(1488),
    [sym_do_push_timed] = ACTIONS(1488),
    [sym_do_release] = ACTIONS(1488),
    [sym_device_id] = ACTIONS(1488),
    [sym_device_info] = ACTIONS(1488),
    [sym_device_id_string] = ACTIONS(1488),
    [sym_device_standby] = ACTIONS(1488),
    [sym_device_wake] = ACTIONS(1488),
    [sym_timeline_create] = ACTIONS(1488),
    [sym_timeline_kill] = ACTIONS(1488),
    [sym_timeline_reload] = ACTIONS(1488),
    [sym_timeline_pause] = ACTIONS(1488),
    [sym_timeline_restart] = ACTIONS(1488),
    [sym_timeline_set] = ACTIONS(1488),
    [sym_timeline_get] = ACTIONS(1488),
    [sym_timeline_active] = ACTIONS(1488),
    [sym_timeline_dynamic_id] = ACTIONS(1488),
    [sym_variable_to_string] = ACTIONS(1488),
    [sym_string_to_variable] = ACTIONS(1488),
    [sym_length_variable_to_string] = ACTIONS(1488),
    [sym_type_cast] = ACTIONS(1488),
    [sym_variable_to_xml] = ACTIONS(1488),
    [sym_xml_to_variable] = ACTIONS(1488),
    [sym_length_variable_to_xml] = ACTIONS(1488),
    [sym_redirect_string] = ACTIONS(1488),
    [sym_random_number] = ACTIONS(1488),
    [sym_get_unique_id] = ACTIONS(1488),
    [sym_get_system_number] = ACTIONS(1488),
    [sym_set_system_number] = ACTIONS(1488),
    [sym_get_serial_number] = ACTIONS(1488),
    [sym_get_master_build] = ACTIONS(1488),
    [sym_set_validation_code] = ACTIONS(1488),
    [sym_reboot] = ACTIONS(1488),
    [sym_load_duet_module] = ACTIONS(1488),
    [sym_unload_duet_module] = ACTIONS(1488),
    [sym_duet_mem_size_set] = ACTIONS(1488),
    [sym_duet_mem_size_get] = ACTIONS(1488),
    [sym_dynamic_application_device] = ACTIONS(1488),
    [sym_dynamic_polled_port] = ACTIONS(1488),
    [sym_static_port_binding] = ACTIONS(1488),
    [sym_static_ip_binding] = ACTIONS(1488),
    [sym_get_dns_list] = ACTIONS(1488),
    [sym_set_dns_list] = ACTIONS(1488),
    [sym_get_ip_address] = ACTIONS(1488),
    [sym_set_ip_address] = ACTIONS(1488),
    [sym_get_url_list] = ACTIONS(1488),
    [sym_add_url_entry] = ACTIONS(1488),
    [sym_delete_url_entry] = ACTIONS(1488),
    [sym_abs_value] = ACTIONS(1488),
    [sym_max_value] = ACTIONS(1488),
    [sym_min_value] = ACTIONS(1488),
    [sym_exp_value] = ACTIONS(1488),
    [sym_log_value] = ACTIONS(1488),
    [sym_log10_value] = ACTIONS(1488),
    [sym_power_value] = ACTIONS(1488),
    [sym_sqrt_value] = ACTIONS(1488),
    [sym_raw_be] = ACTIONS(1488),
    [sym_raw_le] = ACTIONS(1488),
    [sym_internal_queue_size_set] = ACTIONS(1488),
    [sym_internal_queue_size_get] = ACTIONS(1488),
    [sym_internal_threshold_set] = ACTIONS(1488),
    [sym_internal_threshold_get] = ACTIONS(1488),
    [sym_rebuilt_event] = ACTIONS(1488),
    [sym_do_custom_event] = ACTIONS(1488),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1488),
    [sym_clkmgr_set_clk_source] = ACTIONS(1488),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1488),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1488),
    [sym_clkmgr_get_timezone] = ACTIONS(1488),
    [sym_clkmgr_set_timezone] = ACTIONS(1488),
    [sym_clkmgr_get_resync_period] = ACTIONS(1488),
    [sym_clkmgr_set_resync_period] = ACTIONS(1488),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1488),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1488),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1488),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1488),
    [sym_clkmgr_get_timeservers] = ACTIONS(1488),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1488),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1488),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1488),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1488),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1488),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1488),
    [sym_set_log_level] = ACTIONS(1488),
    [sym_get_log_level] = ACTIONS(1488),
    [sym_amx_log] = ACTIONS(1488),
    [sym_smtp_server_config_set] = ACTIONS(1488),
    [sym_smtp_server_config_get] = ACTIONS(1488),
    [sym_smtp_send] = ACTIONS(1488),
    [sym_validate_netlinx_account] = ACTIONS(1488),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1488),
    [sym_authenticate_certificate] = ACTIONS(1488),
    [sym_audit_netlinx_session_event] = ACTIONS(1488),
    [sym_audit_netlinx_generic_event] = ACTIONS(1488),
    [sym_get_available_flash_disk_space] = ACTIONS(1488),
    [sym_get_max_flash_disk_space] = ACTIONS(1488),
    [sym_data_create_feed] = ACTIONS(1488),
    [sym_data_delete_feed] = ACTIONS(1488),
    [sym_data_publish_feed] = ACTIONS(1488),
    [sym_data_get_published_feed] = ACTIONS(1488),
    [sym_data_add_record] = ACTIONS(1488),
    [sym_data_get_event_record] = ACTIONS(1488),
    [sym__wc_data_create_feed] = ACTIONS(1488),
    [sym__wc_data_add_record] = ACTIONS(1488),
    [sym__wc_data_get_event_record] = ACTIONS(1488),
    [sym_format] = ACTIONS(1488),
    [sym_astro_clock] = ACTIONS(1488),
    [sym_set_outdoor_temperature] = ACTIONS(1488),
    [sym_set_virtual_level_count] = ACTIONS(1488),
    [sym_set_virtual_channel_count] = ACTIONS(1488),
    [sym_set_virtual_port_count] = ACTIONS(1488),
    [sym__wc_encode_int] = ACTIONS(1488),
    [sym__wc_decode_int] = ACTIONS(1488),
    [sym__wc_int] = ACTIONS(1488),
    [sym__wc_to_ch_int] = ACTIONS(1488),
    [sym__ch_to_wc_int] = ACTIONS(1488),
    [sym__wc_find_string_int] = ACTIONS(1488),
    [sym__wc_left_string_int] = ACTIONS(1488),
    [sym__wc_lower_string_int] = ACTIONS(1488),
    [sym__wc_mid_string_int] = ACTIONS(1488),
    [sym__wc_remove_string_int] = ACTIONS(1488),
    [sym__wc_right_string_int] = ACTIONS(1488),
    [sym__wc_upper_string_int] = ACTIONS(1488),
    [sym__wc_compare_string_int] = ACTIONS(1488),
    [sym__wc_get_buffer_char_int] = ACTIONS(1488),
    [sym__wc_get_buffer_string_int] = ACTIONS(1488),
    [sym__wc_concat_string_int] = ACTIONS(1488),
    [sym__wc_file_open_int] = ACTIONS(1488),
    [sym__wc_file_close_int] = ACTIONS(1488),
    [sym__wc_file_read_int] = ACTIONS(1488),
    [sym__wc_file_read_line_int] = ACTIONS(1488),
    [sym__wc_file_write_int] = ACTIONS(1488),
    [sym__wc_file_write_line_int] = ACTIONS(1488),
    [anon_sym_EQ] = ACTIONS(1488),
    [aux_sym_preproc_include_token1] = ACTIONS(1488),
    [aux_sym_preproc_define_token1] = ACTIONS(1490),
    [aux_sym_preproc_warn_token1] = ACTIONS(1490),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1490),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1490),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1490),
    [anon_sym_LBRACK] = ACTIONS(1490),
    [anon_sym_LBRACE] = ACTIONS(1490),
    [anon_sym_RBRACE] = ACTIONS(1490),
    [anon_sym_COLON] = ACTIONS(1490),
    [anon_sym_LPAREN] = ACTIONS(1488),
    [anon_sym_SEMI] = ACTIONS(1490),
    [anon_sym_BANG] = ACTIONS(1488),
    [anon_sym_TILDE] = ACTIONS(1490),
    [anon_sym_DASH] = ACTIONS(1488),
    [anon_sym_PLUS] = ACTIONS(1488),
    [anon_sym_STAR] = ACTIONS(1490),
    [anon_sym_SLASH] = ACTIONS(1488),
    [anon_sym_PERCENT] = ACTIONS(1490),
    [anon_sym_PIPE_PIPE] = ACTIONS(1490),
    [anon_sym_AMP_AMP] = ACTIONS(1490),
    [anon_sym_PIPE] = ACTIONS(1488),
    [anon_sym_CARET] = ACTIONS(1490),
    [anon_sym_AMP] = ACTIONS(1488),
    [anon_sym_EQ_EQ] = ACTIONS(1490),
    [anon_sym_BANG_EQ] = ACTIONS(1490),
    [anon_sym_LT_GT] = ACTIONS(1490),
    [anon_sym_GT] = ACTIONS(1488),
    [anon_sym_GT_EQ] = ACTIONS(1490),
    [anon_sym_LT_EQ] = ACTIONS(1490),
    [anon_sym_LT] = ACTIONS(1488),
    [anon_sym_LT_LT] = ACTIONS(1490),
    [anon_sym_GT_GT] = ACTIONS(1490),
    [anon_sym_DASH_DASH] = ACTIONS(1490),
    [anon_sym_PLUS_PLUS] = ACTIONS(1490),
    [anon_sym_DOT] = ACTIONS(1488),
    [anon_sym_DQUOTE] = ACTIONS(1490),
    [anon_sym_SQUOTE] = ACTIONS(1490),
    [sym_number_literal] = ACTIONS(1490),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1490),
  },
  [STATE(163)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1492),
    [sym_define_call_keyword] = ACTIONS(1492),
    [sym_define_function_keyword] = ACTIONS(1492),
    [sym_define_library_function_keyword] = ACTIONS(1492),
    [sym_define_module_keyword] = ACTIONS(1492),
    [sym_char_keyword] = ACTIONS(1492),
    [sym_widechar_keyword] = ACTIONS(1492),
    [sym_integer_keyword] = ACTIONS(1492),
    [sym_sinteger_keyword] = ACTIONS(1492),
    [sym_long_keyword] = ACTIONS(1492),
    [sym_slong_keyword] = ACTIONS(1492),
    [sym_float_keyword] = ACTIONS(1492),
    [sym_double_keyword] = ACTIONS(1492),
    [sym_variant_keyword] = ACTIONS(1492),
    [sym_variantarray_keyword] = ACTIONS(1492),
    [sym_local_var_keyword] = ACTIONS(1492),
    [sym_stack_var_keyword] = ACTIONS(1492),
    [sym_constant_keyword] = ACTIONS(1492),
    [sym_volatile_keyword] = ACTIONS(1492),
    [sym_non_volatile_keyword] = ACTIONS(1492),
    [sym_persistent_keyword] = ACTIONS(1492),
    [sym_dev_keyword] = ACTIONS(1492),
    [sym_devlev_keyword] = ACTIONS(1492),
    [sym_devchan_keyword] = ACTIONS(1492),
    [sym_if_keyword] = ACTIONS(1492),
    [sym_else_keyword] = ACTIONS(1492),
    [sym_switch_keyword] = ACTIONS(1492),
    [sym_case_keyword] = ACTIONS(1492),
    [sym_default_keyword] = ACTIONS(1492),
    [sym_while_keyword] = ACTIONS(1492),
    [sym_for_keyword] = ACTIONS(1492),
    [sym_break_keyword] = ACTIONS(1492),
    [sym_continue_keyword] = ACTIONS(1492),
    [sym_return_keyword] = ACTIONS(1492),
    [sym_select_keyword] = ACTIONS(1492),
    [sym_active_keyword] = ACTIONS(1492),
    [sym_struct_keyword] = ACTIONS(1492),
    [sym_structure_keyword] = ACTIONS(1492),
    [aux_sym_on_keyword_token1] = ACTIONS(1492),
    [aux_sym_off_keyword_token1] = ACTIONS(1492),
    [sym_send_level_keyword] = ACTIONS(1492),
    [sym_send_string_keyword] = ACTIONS(1492),
    [sym_send_command_keyword] = ACTIONS(1492),
    [sym_clear_buffer_keyword] = ACTIONS(1492),
    [sym_create_buffer_keyword] = ACTIONS(1492),
    [sym_create_multi_buffer_keyword] = ACTIONS(1492),
    [sym_call_keyword] = ACTIONS(1492),
    [sym_system_call_keyword] = ACTIONS(1492),
    [sym_devchan_to_keyword] = ACTIONS(1492),
    [sym_devchan_min_to_keyword] = ACTIONS(1492),
    [sym_devchan_total_off_keyword] = ACTIONS(1492),
    [sym_devchan_pulse_keyword] = ACTIONS(1492),
    [sym_wait_keyword] = ACTIONS(1492),
    [sym_cancel_wait_keyword] = ACTIONS(1492),
    [sym_wait_until_keyword] = ACTIONS(1492),
    [sym_cancel_wait_until_keyword] = ACTIONS(1492),
    [sym_cancel_all_wait_keyword] = ACTIONS(1492),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1492),
    [sym_and] = ACTIONS(1605),
    [sym_or] = ACTIONS(1607),
    [sym_not] = ACTIONS(1492),
    [sym_xor] = ACTIONS(1609),
    [sym_band] = ACTIONS(1611),
    [sym_bor] = ACTIONS(1613),
    [sym_bxor] = ACTIONS(1609),
    [sym_bnot] = ACTIONS(1492),
    [sym_lshift] = ACTIONS(1615),
    [sym_rshift] = ACTIONS(1615),
    [sym___file__] = ACTIONS(1492),
    [sym___date__] = ACTIONS(1492),
    [sym___ldate__] = ACTIONS(1492),
    [sym___time__] = ACTIONS(1492),
    [sym___line__] = ACTIONS(1492),
    [sym___name__] = ACTIONS(1492),
    [sym___netlinx__] = ACTIONS(1492),
    [sym_netlinx_axi_version] = ACTIONS(1492),
    [sym_url_flg_tcp] = ACTIONS(1492),
    [sym_url_flg_accinfopresent] = ACTIONS(1492),
    [sym_url_flg_temp] = ACTIONS(1492),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1492),
    [sym_url_flg_stat_mask] = ACTIONS(1492),
    [sym_url_flg_stat_lookup] = ACTIONS(1492),
    [sym_url_flg_stat_connecting] = ACTIONS(1492),
    [sym_url_flg_stat_waiting] = ACTIONS(1492),
    [sym_url_flg_stat_connected] = ACTIONS(1492),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1492),
    [sym_first_virtual_device] = ACTIONS(1492),
    [sym_first_local_port] = ACTIONS(1492),
    [sym_dynamic_virtual_device] = ACTIONS(1492),
    [sym_timeline_once] = ACTIONS(1492),
    [sym_timeline_repeat] = ACTIONS(1492),
    [sym_timeline_absolute] = ACTIONS(1492),
    [sym_timeline_relative] = ACTIONS(1492),
    [sym_do_push_timed_infinite] = ACTIONS(1492),
    [sym_source_type_no_address] = ACTIONS(1492),
    [sym_source_type_neuron_id] = ACTIONS(1492),
    [sym_source_type_ip_address] = ACTIONS(1492),
    [sym_source_type_axlink] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1492),
    [sym_source_type_ip_socket_address] = ACTIONS(1492),
    [sym_source_type_rs232] = ACTIONS(1492),
    [sym_source_type_internal] = ACTIONS(1492),
    [sym_source_type_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1492),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1492),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1492),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1492),
    [sym_true] = ACTIONS(1492),
    [sym_false] = ACTIONS(1492),
    [sym_file_read_only] = ACTIONS(1492),
    [sym_file_rw_new] = ACTIONS(1492),
    [sym_file_rw_append] = ACTIONS(1492),
    [sym_ip_tcp] = ACTIONS(1492),
    [sym_ip_udp] = ACTIONS(1492),
    [sym_ip_udp_2way] = ACTIONS(1492),
    [sym_xml_encode_types] = ACTIONS(1492),
    [sym_xml_encode_char_as_list] = ACTIONS(1492),
    [sym_xml_encode_le] = ACTIONS(1492),
    [sym_xml_decode_types] = ACTIONS(1492),
    [sym_xml_decode_no_preserve] = ACTIONS(1492),
    [sym_duet_dev_type_display_device] = ACTIONS(1492),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_io_device] = ACTIONS(1492),
    [sym_duet_dev_type_relay_device] = ACTIONS(1492),
    [sym_duet_dev_type_ups] = ACTIONS(1492),
    [sym_duet_dev_type_amplifier] = ACTIONS(1492),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1492),
    [sym_duet_dev_type_camera] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1492),
    [sym_duet_dev_type_dss] = ACTIONS(1492),
    [sym_duet_dev_type_dvr] = ACTIONS(1492),
    [sym_duet_dev_type_disc_device] = ACTIONS(1492),
    [sym_duet_dev_type_document_camera] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1492),
    [sym_duet_dev_type_hvac] = ACTIONS(1492),
    [sym_duet_dev_type_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_light] = ACTIONS(1492),
    [sym_duet_dev_type_monitor] = ACTIONS(1492),
    [sym_duet_dev_type_motor] = ACTIONS(1492),
    [sym_duet_dev_type_multi_window] = ACTIONS(1492),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1492),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1492),
    [sym_duet_dev_type_receiver] = ACTIONS(1492),
    [sym_duet_dev_type_security_system] = ACTIONS(1492),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1492),
    [sym_duet_dev_type_settop_box] = ACTIONS(1492),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1492),
    [sym_duet_dev_type_switcher] = ACTIONS(1492),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_tv] = ACTIONS(1492),
    [sym_duet_dev_type_utility] = ACTIONS(1492),
    [sym_duet_dev_type_vcr] = ACTIONS(1492),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_video_processor] = ACTIONS(1492),
    [sym_duet_dev_type_video_projector] = ACTIONS(1492),
    [sym_duet_dev_type_video_wall] = ACTIONS(1492),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1492),
    [sym_duet_dev_type_weather] = ACTIONS(1492),
    [sym_duet_dev_not_polled] = ACTIONS(1492),
    [sym_duet_dev_polled] = ACTIONS(1492),
    [sym_clkmgr_mode_network] = ACTIONS(1492),
    [sym_clkmgr_mode_standalone] = ACTIONS(1492),
    [sym_normal_standby] = ACTIONS(1492),
    [sym_normal_wake] = ACTIONS(1492),
    [sym_ip_multicast_ttl_option] = ACTIONS(1492),
    [sym_ip_ttl_subnet] = ACTIONS(1492),
    [sym_ip_ttl_site] = ACTIONS(1492),
    [sym_ip_ttl_region] = ACTIONS(1492),
    [sym_ip_ttl_continent] = ACTIONS(1492),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1492),
    [sym_ip_nodelay_on] = ACTIONS(1492),
    [sym_ip_nodelay_off] = ACTIONS(1492),
    [sym_amx_error] = ACTIONS(1492),
    [sym_amx_warning] = ACTIONS(1492),
    [sym_amx_info] = ACTIONS(1492),
    [sym_amx_debug] = ACTIONS(1492),
    [sym_netlinx_login_success] = ACTIONS(1492),
    [sym_netlinx_login_fail] = ACTIONS(1492),
    [sym_netlinx_logout] = ACTIONS(1492),
    [sym_null_str] = ACTIONS(1492),
    [sym_smtp_address] = ACTIONS(1492),
    [sym_smtp_port_number] = ACTIONS(1492),
    [sym_smtp_username] = ACTIONS(1492),
    [sym_smtp_password] = ACTIONS(1492),
    [sym_smtp_require_tls] = ACTIONS(1492),
    [sym_smtp_from] = ACTIONS(1492),
    [sym_smtp_tls_true] = ACTIONS(1492),
    [sym_smtp_tls_false] = ACTIONS(1492),
    [sym_listview_on_row_select_event] = ACTIONS(1492),
    [sym_data_structure_unknown] = ACTIONS(1492),
    [sym_data_structure_datafeed] = ACTIONS(1492),
    [sym_data_structure_datarecordset] = ACTIONS(1492),
    [sym_data_structure_datarecord] = ACTIONS(1492),
    [sym_data_structure_datafield] = ACTIONS(1492),
    [sym_data_max_content_field_count] = ACTIONS(1492),
    [sym_data_max_metadata_field_count] = ACTIONS(1492),
    [sym_data_max_id_length] = ACTIONS(1492),
    [sym_data_max_name_length] = ACTIONS(1492),
    [sym_data_max_description_length] = ACTIONS(1492),
    [sym_data_max_type_length] = ACTIONS(1492),
    [sym_data_max_format_length] = ACTIONS(1492),
    [sym_data_max_source_length] = ACTIONS(1492),
    [sym_data_max_value_length] = ACTIONS(1492),
    [sym_data_max_label_length] = ACTIONS(1492),
    [sym_data_max_url_length] = ACTIONS(1492),
    [sym_max_last_login_info_length] = ACTIONS(1492),
    [sym_data_type_unknown] = ACTIONS(1492),
    [sym_data_type_string] = ACTIONS(1492),
    [sym_data_type_datetime] = ACTIONS(1492),
    [sym_data_type_date] = ACTIONS(1492),
    [sym_data_type_time] = ACTIONS(1492),
    [sym_data_type_image] = ACTIONS(1492),
    [sym_data_format_url] = ACTIONS(1492),
    [sym_data_format_phone] = ACTIONS(1492),
    [sym_data_format_email] = ACTIONS(1492),
    [sym_data_format_iso8601] = ACTIONS(1492),
    [sym_auditlog_priv] = ACTIONS(1492),
    [sym_configuration_priv] = ACTIONS(1492),
    [sym_deviceconfig_priv] = ACTIONS(1492),
    [sym_ftp_priv] = ACTIONS(1492),
    [sym_http_priv] = ACTIONS(1492),
    [sym_networkconfig_priv] = ACTIONS(1492),
    [sym_programport_priv] = ACTIONS(1492),
    [sym_remoteui_priv] = ACTIONS(1492),
    [sym_securitycontrol_priv] = ACTIONS(1492),
    [sym_softwaremanagement_priv] = ACTIONS(1492),
    [sym_terminal_priv] = ACTIONS(1492),
    [sym_tpadmin_priv] = ACTIONS(1492),
    [sym_user1_priv] = ACTIONS(1492),
    [sym_user2_priv] = ACTIONS(1492),
    [sym_user3_priv] = ACTIONS(1492),
    [sym_user4_priv] = ACTIONS(1492),
    [sym_usermanagement_priv] = ACTIONS(1492),
    [sym_valid_account] = ACTIONS(1492),
    [sym_err_name_invalid] = ACTIONS(1492),
    [sym_err_password_invalid] = ACTIONS(1492),
    [sym_err_invalid_account] = ACTIONS(1492),
    [sym_err_no_authorization] = ACTIONS(1492),
    [sym_err_invalid_parameter] = ACTIONS(1492),
    [sym_err_user_locked_out] = ACTIONS(1492),
    [sym_err_user_account_expired] = ACTIONS(1492),
    [sym_err_general_error] = ACTIONS(1492),
    [sym_err_authentication_error] = ACTIONS(1492),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1492),
    [sym_tls_validate_certificate] = ACTIONS(1492),
    [sym_tdata] = ACTIONS(1492),
    [sym_tchannel] = ACTIONS(1492),
    [sym_tlevel] = ACTIONS(1492),
    [sym_tbutton] = ACTIONS(1492),
    [sym_ttimeline] = ACTIONS(1492),
    [sym_tcustom] = ACTIONS(1492),
    [sym_url_struct] = ACTIONS(1492),
    [sym_dns_struct] = ACTIONS(1492),
    [sym_ip_address_struct] = ACTIONS(1492),
    [sym_dev_info_struct] = ACTIONS(1492),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1492),
    [sym_data_feed] = ACTIONS(1492),
    [sym_data_field] = ACTIONS(1492),
    [sym_data_record] = ACTIONS(1492),
    [sym_wc_data_feed] = ACTIONS(1492),
    [sym_wc_data_field] = ACTIONS(1492),
    [sym_wc_data_record] = ACTIONS(1492),
    [sym_last_login_info] = ACTIONS(1492),
    [sym____reserved___] = ACTIONS(1492),
    [sym_date] = ACTIONS(1492),
    [sym_day] = ACTIONS(1492),
    [sym_time] = ACTIONS(1492),
    [sym_ldate] = ACTIONS(1492),
    [sym_push_device] = ACTIONS(1492),
    [sym_push_channel] = ACTIONS(1492),
    [sym_push_devchan] = ACTIONS(1492),
    [sym_release_device] = ACTIONS(1492),
    [sym_release_channel] = ACTIONS(1492),
    [sym_release_devchan] = ACTIONS(1492),
    [sym_master_slot] = ACTIONS(1492),
    [sym_get_pulse_time] = ACTIONS(1492),
    [sym_get_timer] = ACTIONS(1492),
    [sym_master_sn] = ACTIONS(1492),
    [sym_system_number] = ACTIONS(1492),
    [sym_button] = ACTIONS(1492),
    [sym_dv_channel] = ACTIONS(1492),
    [sym_data] = ACTIONS(1492),
    [sym_level] = ACTIONS(1492),
    [sym_timeline] = ACTIONS(1492),
    [sym_channel] = ACTIONS(1492),
    [sym_custom] = ACTIONS(1492),
    [sym_atoi] = ACTIONS(1492),
    [sym_atol] = ACTIONS(1492),
    [sym_atof] = ACTIONS(1492),
    [sym_itoa] = ACTIONS(1492),
    [sym_itohex] = ACTIONS(1492),
    [sym_hextoi] = ACTIONS(1492),
    [sym_ftoa] = ACTIONS(1492),
    [sym_find_string] = ACTIONS(1492),
    [sym_left_string] = ACTIONS(1492),
    [sym_right_string] = ACTIONS(1492),
    [sym_mid_string] = ACTIONS(1492),
    [sym_remove_string] = ACTIONS(1492),
    [sym_get_buffer_string] = ACTIONS(1492),
    [sym_get_buffer_char] = ACTIONS(1492),
    [sym_get_multi_buffer_string] = ACTIONS(1492),
    [sym_length_string] = ACTIONS(1492),
    [sym_max_length_string] = ACTIONS(1492),
    [sym_set_length_string] = ACTIONS(1492),
    [sym_compare_string] = ACTIONS(1492),
    [sym_upper_string] = ACTIONS(1492),
    [sym_lower_string] = ACTIONS(1492),
    [sym_get_last] = ACTIONS(1492),
    [sym_date_to_day] = ACTIONS(1492),
    [sym_date_to_month] = ACTIONS(1492),
    [sym_date_to_year] = ACTIONS(1492),
    [sym_time_to_hour] = ACTIONS(1492),
    [sym_time_to_minute] = ACTIONS(1492),
    [sym_time_to_second] = ACTIONS(1492),
    [sym_day_of_week] = ACTIONS(1492),
    [sym_set_pulse_time] = ACTIONS(1492),
    [sym_set_timer] = ACTIONS(1492),
    [sym_file_open] = ACTIONS(1492),
    [sym_file_close] = ACTIONS(1492),
    [sym_file_read] = ACTIONS(1492),
    [sym_file_read_line] = ACTIONS(1492),
    [sym_file_write] = ACTIONS(1492),
    [sym_file_write_line] = ACTIONS(1492),
    [sym_file_copy] = ACTIONS(1492),
    [sym_file_delete] = ACTIONS(1492),
    [sym_file_rename] = ACTIONS(1492),
    [sym_file_dir] = ACTIONS(1492),
    [sym_file_getdir] = ACTIONS(1492),
    [sym_file_setdir] = ACTIONS(1492),
    [sym_file_create_dir] = ACTIONS(1492),
    [sym_file_remove_dir] = ACTIONS(1492),
    [sym_file_seek] = ACTIONS(1492),
    [sym_length_array] = ACTIONS(1492),
    [sym_max_length_array] = ACTIONS(1492),
    [sym_set_length_array] = ACTIONS(1492),
    [sym_ip_client_open] = ACTIONS(1492),
    [sym_ip_client_close] = ACTIONS(1492),
    [sym_ip_server_open] = ACTIONS(1492),
    [sym_ip_server_close] = ACTIONS(1492),
    [sym_ip_bound_client_open] = ACTIONS(1492),
    [sym_ip_mc_server_open] = ACTIONS(1492),
    [sym_ip_set_option] = ACTIONS(1492),
    [sym_ssh_client_open] = ACTIONS(1492),
    [sym_ssh_client_close] = ACTIONS(1492),
    [sym_tls_client_open] = ACTIONS(1492),
    [sym_tls_client_close] = ACTIONS(1492),
    [sym_do_push] = ACTIONS(1492),
    [sym_do_push_timed] = ACTIONS(1492),
    [sym_do_release] = ACTIONS(1492),
    [sym_device_id] = ACTIONS(1492),
    [sym_device_info] = ACTIONS(1492),
    [sym_device_id_string] = ACTIONS(1492),
    [sym_device_standby] = ACTIONS(1492),
    [sym_device_wake] = ACTIONS(1492),
    [sym_timeline_create] = ACTIONS(1492),
    [sym_timeline_kill] = ACTIONS(1492),
    [sym_timeline_reload] = ACTIONS(1492),
    [sym_timeline_pause] = ACTIONS(1492),
    [sym_timeline_restart] = ACTIONS(1492),
    [sym_timeline_set] = ACTIONS(1492),
    [sym_timeline_get] = ACTIONS(1492),
    [sym_timeline_active] = ACTIONS(1492),
    [sym_timeline_dynamic_id] = ACTIONS(1492),
    [sym_variable_to_string] = ACTIONS(1492),
    [sym_string_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_string] = ACTIONS(1492),
    [sym_type_cast] = ACTIONS(1492),
    [sym_variable_to_xml] = ACTIONS(1492),
    [sym_xml_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_xml] = ACTIONS(1492),
    [sym_redirect_string] = ACTIONS(1492),
    [sym_random_number] = ACTIONS(1492),
    [sym_get_unique_id] = ACTIONS(1492),
    [sym_get_system_number] = ACTIONS(1492),
    [sym_set_system_number] = ACTIONS(1492),
    [sym_get_serial_number] = ACTIONS(1492),
    [sym_get_master_build] = ACTIONS(1492),
    [sym_set_validation_code] = ACTIONS(1492),
    [sym_reboot] = ACTIONS(1492),
    [sym_load_duet_module] = ACTIONS(1492),
    [sym_unload_duet_module] = ACTIONS(1492),
    [sym_duet_mem_size_set] = ACTIONS(1492),
    [sym_duet_mem_size_get] = ACTIONS(1492),
    [sym_dynamic_application_device] = ACTIONS(1492),
    [sym_dynamic_polled_port] = ACTIONS(1492),
    [sym_static_port_binding] = ACTIONS(1492),
    [sym_static_ip_binding] = ACTIONS(1492),
    [sym_get_dns_list] = ACTIONS(1492),
    [sym_set_dns_list] = ACTIONS(1492),
    [sym_get_ip_address] = ACTIONS(1492),
    [sym_set_ip_address] = ACTIONS(1492),
    [sym_get_url_list] = ACTIONS(1492),
    [sym_add_url_entry] = ACTIONS(1492),
    [sym_delete_url_entry] = ACTIONS(1492),
    [sym_abs_value] = ACTIONS(1492),
    [sym_max_value] = ACTIONS(1492),
    [sym_min_value] = ACTIONS(1492),
    [sym_exp_value] = ACTIONS(1492),
    [sym_log_value] = ACTIONS(1492),
    [sym_log10_value] = ACTIONS(1492),
    [sym_power_value] = ACTIONS(1492),
    [sym_sqrt_value] = ACTIONS(1492),
    [sym_raw_be] = ACTIONS(1492),
    [sym_raw_le] = ACTIONS(1492),
    [sym_internal_queue_size_set] = ACTIONS(1492),
    [sym_internal_queue_size_get] = ACTIONS(1492),
    [sym_internal_threshold_set] = ACTIONS(1492),
    [sym_internal_threshold_get] = ACTIONS(1492),
    [sym_rebuilt_event] = ACTIONS(1492),
    [sym_do_custom_event] = ACTIONS(1492),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1492),
    [sym_clkmgr_set_clk_source] = ACTIONS(1492),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1492),
    [sym_clkmgr_get_timezone] = ACTIONS(1492),
    [sym_clkmgr_set_timezone] = ACTIONS(1492),
    [sym_clkmgr_get_resync_period] = ACTIONS(1492),
    [sym_clkmgr_set_resync_period] = ACTIONS(1492),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_timeservers] = ACTIONS(1492),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_set_log_level] = ACTIONS(1492),
    [sym_get_log_level] = ACTIONS(1492),
    [sym_amx_log] = ACTIONS(1492),
    [sym_smtp_server_config_set] = ACTIONS(1492),
    [sym_smtp_server_config_get] = ACTIONS(1492),
    [sym_smtp_send] = ACTIONS(1492),
    [sym_validate_netlinx_account] = ACTIONS(1492),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1492),
    [sym_authenticate_certificate] = ACTIONS(1492),
    [sym_audit_netlinx_session_event] = ACTIONS(1492),
    [sym_audit_netlinx_generic_event] = ACTIONS(1492),
    [sym_get_available_flash_disk_space] = ACTIONS(1492),
    [sym_get_max_flash_disk_space] = ACTIONS(1492),
    [sym_data_create_feed] = ACTIONS(1492),
    [sym_data_delete_feed] = ACTIONS(1492),
    [sym_data_publish_feed] = ACTIONS(1492),
    [sym_data_get_published_feed] = ACTIONS(1492),
    [sym_data_add_record] = ACTIONS(1492),
    [sym_data_get_event_record] = ACTIONS(1492),
    [sym__wc_data_create_feed] = ACTIONS(1492),
    [sym__wc_data_add_record] = ACTIONS(1492),
    [sym__wc_data_get_event_record] = ACTIONS(1492),
    [sym_format] = ACTIONS(1492),
    [sym_astro_clock] = ACTIONS(1492),
    [sym_set_outdoor_temperature] = ACTIONS(1492),
    [sym_set_virtual_level_count] = ACTIONS(1492),
    [sym_set_virtual_channel_count] = ACTIONS(1492),
    [sym_set_virtual_port_count] = ACTIONS(1492),
    [sym__wc_encode_int] = ACTIONS(1492),
    [sym__wc_decode_int] = ACTIONS(1492),
    [sym__wc_int] = ACTIONS(1492),
    [sym__wc_to_ch_int] = ACTIONS(1492),
    [sym__ch_to_wc_int] = ACTIONS(1492),
    [sym__wc_find_string_int] = ACTIONS(1492),
    [sym__wc_left_string_int] = ACTIONS(1492),
    [sym__wc_lower_string_int] = ACTIONS(1492),
    [sym__wc_mid_string_int] = ACTIONS(1492),
    [sym__wc_remove_string_int] = ACTIONS(1492),
    [sym__wc_right_string_int] = ACTIONS(1492),
    [sym__wc_upper_string_int] = ACTIONS(1492),
    [sym__wc_compare_string_int] = ACTIONS(1492),
    [sym__wc_get_buffer_char_int] = ACTIONS(1492),
    [sym__wc_get_buffer_string_int] = ACTIONS(1492),
    [sym__wc_concat_string_int] = ACTIONS(1492),
    [sym__wc_file_open_int] = ACTIONS(1492),
    [sym__wc_file_close_int] = ACTIONS(1492),
    [sym__wc_file_read_int] = ACTIONS(1492),
    [sym__wc_file_read_line_int] = ACTIONS(1492),
    [sym__wc_file_write_int] = ACTIONS(1492),
    [sym__wc_file_write_line_int] = ACTIONS(1492),
    [aux_sym_preproc_include_token1] = ACTIONS(1492),
    [aux_sym_preproc_define_token1] = ACTIONS(1494),
    [aux_sym_preproc_warn_token1] = ACTIONS(1494),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1494),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1494),
    [anon_sym_RBRACE] = ACTIONS(1494),
    [anon_sym_COLON] = ACTIONS(1619),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1494),
    [anon_sym_BANG] = ACTIONS(1492),
    [anon_sym_TILDE] = ACTIONS(1494),
    [anon_sym_DASH] = ACTIONS(1625),
    [anon_sym_PLUS] = ACTIONS(1625),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1631),
    [anon_sym_AMP_AMP] = ACTIONS(1633),
    [anon_sym_PIPE] = ACTIONS(1613),
    [anon_sym_CARET] = ACTIONS(1635),
    [anon_sym_AMP] = ACTIONS(1611),
    [anon_sym_EQ_EQ] = ACTIONS(1637),
    [anon_sym_BANG_EQ] = ACTIONS(1637),
    [anon_sym_LT_GT] = ACTIONS(1637),
    [anon_sym_GT] = ACTIONS(1639),
    [anon_sym_GT_EQ] = ACTIONS(1641),
    [anon_sym_LT_EQ] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1639),
    [anon_sym_LT_LT] = ACTIONS(1643),
    [anon_sym_GT_GT] = ACTIONS(1643),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1494),
    [anon_sym_SQUOTE] = ACTIONS(1494),
    [sym_number_literal] = ACTIONS(1494),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1494),
  },
  [STATE(164)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1413),
    [sym_rshift] = ACTIONS(1413),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1415),
    [anon_sym_GT_GT] = ACTIONS(1415),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(165)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1605),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1609),
    [sym_band] = ACTIONS(1611),
    [sym_bor] = ACTIONS(1613),
    [sym_bxor] = ACTIONS(1609),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1615),
    [sym_rshift] = ACTIONS(1615),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1625),
    [anon_sym_PLUS] = ACTIONS(1625),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1633),
    [anon_sym_PIPE] = ACTIONS(1613),
    [anon_sym_CARET] = ACTIONS(1635),
    [anon_sym_AMP] = ACTIONS(1611),
    [anon_sym_EQ_EQ] = ACTIONS(1637),
    [anon_sym_BANG_EQ] = ACTIONS(1637),
    [anon_sym_LT_GT] = ACTIONS(1637),
    [anon_sym_GT] = ACTIONS(1639),
    [anon_sym_GT_EQ] = ACTIONS(1641),
    [anon_sym_LT_EQ] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1639),
    [anon_sym_LT_LT] = ACTIONS(1643),
    [anon_sym_GT_GT] = ACTIONS(1643),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(166)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1481),
    [sym_define_call_keyword] = ACTIONS(1481),
    [sym_define_function_keyword] = ACTIONS(1481),
    [sym_define_library_function_keyword] = ACTIONS(1481),
    [sym_define_module_keyword] = ACTIONS(1481),
    [sym_char_keyword] = ACTIONS(1481),
    [sym_widechar_keyword] = ACTIONS(1481),
    [sym_integer_keyword] = ACTIONS(1481),
    [sym_sinteger_keyword] = ACTIONS(1481),
    [sym_long_keyword] = ACTIONS(1481),
    [sym_slong_keyword] = ACTIONS(1481),
    [sym_float_keyword] = ACTIONS(1481),
    [sym_double_keyword] = ACTIONS(1481),
    [sym_variant_keyword] = ACTIONS(1481),
    [sym_variantarray_keyword] = ACTIONS(1481),
    [sym_local_var_keyword] = ACTIONS(1481),
    [sym_stack_var_keyword] = ACTIONS(1481),
    [sym_constant_keyword] = ACTIONS(1481),
    [sym_volatile_keyword] = ACTIONS(1481),
    [sym_non_volatile_keyword] = ACTIONS(1481),
    [sym_persistent_keyword] = ACTIONS(1481),
    [sym_dev_keyword] = ACTIONS(1481),
    [sym_devlev_keyword] = ACTIONS(1481),
    [sym_devchan_keyword] = ACTIONS(1481),
    [sym_if_keyword] = ACTIONS(1481),
    [sym_else_keyword] = ACTIONS(1481),
    [sym_switch_keyword] = ACTIONS(1481),
    [sym_case_keyword] = ACTIONS(1481),
    [sym_default_keyword] = ACTIONS(1481),
    [sym_while_keyword] = ACTIONS(1481),
    [sym_for_keyword] = ACTIONS(1481),
    [sym_break_keyword] = ACTIONS(1481),
    [sym_continue_keyword] = ACTIONS(1481),
    [sym_return_keyword] = ACTIONS(1481),
    [sym_select_keyword] = ACTIONS(1481),
    [sym_active_keyword] = ACTIONS(1481),
    [sym_struct_keyword] = ACTIONS(1481),
    [sym_structure_keyword] = ACTIONS(1481),
    [aux_sym_on_keyword_token1] = ACTIONS(1481),
    [aux_sym_off_keyword_token1] = ACTIONS(1481),
    [sym_send_level_keyword] = ACTIONS(1481),
    [sym_send_string_keyword] = ACTIONS(1481),
    [sym_send_command_keyword] = ACTIONS(1481),
    [sym_clear_buffer_keyword] = ACTIONS(1481),
    [sym_create_buffer_keyword] = ACTIONS(1481),
    [sym_create_multi_buffer_keyword] = ACTIONS(1481),
    [sym_call_keyword] = ACTIONS(1481),
    [sym_system_call_keyword] = ACTIONS(1481),
    [sym_devchan_to_keyword] = ACTIONS(1481),
    [sym_devchan_min_to_keyword] = ACTIONS(1481),
    [sym_devchan_total_off_keyword] = ACTIONS(1481),
    [sym_devchan_pulse_keyword] = ACTIONS(1481),
    [sym_wait_keyword] = ACTIONS(1481),
    [sym_cancel_wait_keyword] = ACTIONS(1481),
    [sym_wait_until_keyword] = ACTIONS(1481),
    [sym_cancel_wait_until_keyword] = ACTIONS(1481),
    [sym_cancel_all_wait_keyword] = ACTIONS(1481),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1481),
    [sym_and] = ACTIONS(1605),
    [sym_or] = ACTIONS(1607),
    [sym_not] = ACTIONS(1481),
    [sym_xor] = ACTIONS(1609),
    [sym_band] = ACTIONS(1611),
    [sym_bor] = ACTIONS(1613),
    [sym_bxor] = ACTIONS(1609),
    [sym_bnot] = ACTIONS(1481),
    [sym_lshift] = ACTIONS(1615),
    [sym_rshift] = ACTIONS(1615),
    [sym___file__] = ACTIONS(1481),
    [sym___date__] = ACTIONS(1481),
    [sym___ldate__] = ACTIONS(1481),
    [sym___time__] = ACTIONS(1481),
    [sym___line__] = ACTIONS(1481),
    [sym___name__] = ACTIONS(1481),
    [sym___netlinx__] = ACTIONS(1481),
    [sym_netlinx_axi_version] = ACTIONS(1481),
    [sym_url_flg_tcp] = ACTIONS(1481),
    [sym_url_flg_accinfopresent] = ACTIONS(1481),
    [sym_url_flg_temp] = ACTIONS(1481),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1481),
    [sym_url_flg_stat_mask] = ACTIONS(1481),
    [sym_url_flg_stat_lookup] = ACTIONS(1481),
    [sym_url_flg_stat_connecting] = ACTIONS(1481),
    [sym_url_flg_stat_waiting] = ACTIONS(1481),
    [sym_url_flg_stat_connected] = ACTIONS(1481),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1481),
    [sym_first_virtual_device] = ACTIONS(1481),
    [sym_first_local_port] = ACTIONS(1481),
    [sym_dynamic_virtual_device] = ACTIONS(1481),
    [sym_timeline_once] = ACTIONS(1481),
    [sym_timeline_repeat] = ACTIONS(1481),
    [sym_timeline_absolute] = ACTIONS(1481),
    [sym_timeline_relative] = ACTIONS(1481),
    [sym_do_push_timed_infinite] = ACTIONS(1481),
    [sym_source_type_no_address] = ACTIONS(1481),
    [sym_source_type_neuron_id] = ACTIONS(1481),
    [sym_source_type_ip_address] = ACTIONS(1481),
    [sym_source_type_axlink] = ACTIONS(1481),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1481),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1481),
    [sym_source_type_ip_socket_address] = ACTIONS(1481),
    [sym_source_type_rs232] = ACTIONS(1481),
    [sym_source_type_internal] = ACTIONS(1481),
    [sym_source_type_mac_address] = ACTIONS(1481),
    [sym_source_type_ipv4_port] = ACTIONS(1481),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1481),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1481),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1481),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1481),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1481),
    [sym_true] = ACTIONS(1481),
    [sym_false] = ACTIONS(1481),
    [sym_file_read_only] = ACTIONS(1481),
    [sym_file_rw_new] = ACTIONS(1481),
    [sym_file_rw_append] = ACTIONS(1481),
    [sym_ip_tcp] = ACTIONS(1481),
    [sym_ip_udp] = ACTIONS(1481),
    [sym_ip_udp_2way] = ACTIONS(1481),
    [sym_xml_encode_types] = ACTIONS(1481),
    [sym_xml_encode_char_as_list] = ACTIONS(1481),
    [sym_xml_encode_le] = ACTIONS(1481),
    [sym_xml_decode_types] = ACTIONS(1481),
    [sym_xml_decode_no_preserve] = ACTIONS(1481),
    [sym_duet_dev_type_display_device] = ACTIONS(1481),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1481),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1481),
    [sym_duet_dev_type_io_device] = ACTIONS(1481),
    [sym_duet_dev_type_relay_device] = ACTIONS(1481),
    [sym_duet_dev_type_ups] = ACTIONS(1481),
    [sym_duet_dev_type_amplifier] = ACTIONS(1481),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1481),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1481),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1481),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1481),
    [sym_duet_dev_type_camera] = ACTIONS(1481),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1481),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1481),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1481),
    [sym_duet_dev_type_dss] = ACTIONS(1481),
    [sym_duet_dev_type_dvr] = ACTIONS(1481),
    [sym_duet_dev_type_disc_device] = ACTIONS(1481),
    [sym_duet_dev_type_document_camera] = ACTIONS(1481),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1481),
    [sym_duet_dev_type_hvac] = ACTIONS(1481),
    [sym_duet_dev_type_keypad] = ACTIONS(1481),
    [sym_duet_dev_type_light] = ACTIONS(1481),
    [sym_duet_dev_type_monitor] = ACTIONS(1481),
    [sym_duet_dev_type_motor] = ACTIONS(1481),
    [sym_duet_dev_type_multi_window] = ACTIONS(1481),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1481),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1481),
    [sym_duet_dev_type_receiver] = ACTIONS(1481),
    [sym_duet_dev_type_security_system] = ACTIONS(1481),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1481),
    [sym_duet_dev_type_settop_box] = ACTIONS(1481),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1481),
    [sym_duet_dev_type_switcher] = ACTIONS(1481),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1481),
    [sym_duet_dev_type_tv] = ACTIONS(1481),
    [sym_duet_dev_type_utility] = ACTIONS(1481),
    [sym_duet_dev_type_vcr] = ACTIONS(1481),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1481),
    [sym_duet_dev_type_video_processor] = ACTIONS(1481),
    [sym_duet_dev_type_video_projector] = ACTIONS(1481),
    [sym_duet_dev_type_video_wall] = ACTIONS(1481),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1481),
    [sym_duet_dev_type_weather] = ACTIONS(1481),
    [sym_duet_dev_not_polled] = ACTIONS(1481),
    [sym_duet_dev_polled] = ACTIONS(1481),
    [sym_clkmgr_mode_network] = ACTIONS(1481),
    [sym_clkmgr_mode_standalone] = ACTIONS(1481),
    [sym_normal_standby] = ACTIONS(1481),
    [sym_normal_wake] = ACTIONS(1481),
    [sym_ip_multicast_ttl_option] = ACTIONS(1481),
    [sym_ip_ttl_subnet] = ACTIONS(1481),
    [sym_ip_ttl_site] = ACTIONS(1481),
    [sym_ip_ttl_region] = ACTIONS(1481),
    [sym_ip_ttl_continent] = ACTIONS(1481),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1481),
    [sym_ip_nodelay_on] = ACTIONS(1481),
    [sym_ip_nodelay_off] = ACTIONS(1481),
    [sym_amx_error] = ACTIONS(1481),
    [sym_amx_warning] = ACTIONS(1481),
    [sym_amx_info] = ACTIONS(1481),
    [sym_amx_debug] = ACTIONS(1481),
    [sym_netlinx_login_success] = ACTIONS(1481),
    [sym_netlinx_login_fail] = ACTIONS(1481),
    [sym_netlinx_logout] = ACTIONS(1481),
    [sym_null_str] = ACTIONS(1481),
    [sym_smtp_address] = ACTIONS(1481),
    [sym_smtp_port_number] = ACTIONS(1481),
    [sym_smtp_username] = ACTIONS(1481),
    [sym_smtp_password] = ACTIONS(1481),
    [sym_smtp_require_tls] = ACTIONS(1481),
    [sym_smtp_from] = ACTIONS(1481),
    [sym_smtp_tls_true] = ACTIONS(1481),
    [sym_smtp_tls_false] = ACTIONS(1481),
    [sym_listview_on_row_select_event] = ACTIONS(1481),
    [sym_data_structure_unknown] = ACTIONS(1481),
    [sym_data_structure_datafeed] = ACTIONS(1481),
    [sym_data_structure_datarecordset] = ACTIONS(1481),
    [sym_data_structure_datarecord] = ACTIONS(1481),
    [sym_data_structure_datafield] = ACTIONS(1481),
    [sym_data_max_content_field_count] = ACTIONS(1481),
    [sym_data_max_metadata_field_count] = ACTIONS(1481),
    [sym_data_max_id_length] = ACTIONS(1481),
    [sym_data_max_name_length] = ACTIONS(1481),
    [sym_data_max_description_length] = ACTIONS(1481),
    [sym_data_max_type_length] = ACTIONS(1481),
    [sym_data_max_format_length] = ACTIONS(1481),
    [sym_data_max_source_length] = ACTIONS(1481),
    [sym_data_max_value_length] = ACTIONS(1481),
    [sym_data_max_label_length] = ACTIONS(1481),
    [sym_data_max_url_length] = ACTIONS(1481),
    [sym_max_last_login_info_length] = ACTIONS(1481),
    [sym_data_type_unknown] = ACTIONS(1481),
    [sym_data_type_string] = ACTIONS(1481),
    [sym_data_type_datetime] = ACTIONS(1481),
    [sym_data_type_date] = ACTIONS(1481),
    [sym_data_type_time] = ACTIONS(1481),
    [sym_data_type_image] = ACTIONS(1481),
    [sym_data_format_url] = ACTIONS(1481),
    [sym_data_format_phone] = ACTIONS(1481),
    [sym_data_format_email] = ACTIONS(1481),
    [sym_data_format_iso8601] = ACTIONS(1481),
    [sym_auditlog_priv] = ACTIONS(1481),
    [sym_configuration_priv] = ACTIONS(1481),
    [sym_deviceconfig_priv] = ACTIONS(1481),
    [sym_ftp_priv] = ACTIONS(1481),
    [sym_http_priv] = ACTIONS(1481),
    [sym_networkconfig_priv] = ACTIONS(1481),
    [sym_programport_priv] = ACTIONS(1481),
    [sym_remoteui_priv] = ACTIONS(1481),
    [sym_securitycontrol_priv] = ACTIONS(1481),
    [sym_softwaremanagement_priv] = ACTIONS(1481),
    [sym_terminal_priv] = ACTIONS(1481),
    [sym_tpadmin_priv] = ACTIONS(1481),
    [sym_user1_priv] = ACTIONS(1481),
    [sym_user2_priv] = ACTIONS(1481),
    [sym_user3_priv] = ACTIONS(1481),
    [sym_user4_priv] = ACTIONS(1481),
    [sym_usermanagement_priv] = ACTIONS(1481),
    [sym_valid_account] = ACTIONS(1481),
    [sym_err_name_invalid] = ACTIONS(1481),
    [sym_err_password_invalid] = ACTIONS(1481),
    [sym_err_invalid_account] = ACTIONS(1481),
    [sym_err_no_authorization] = ACTIONS(1481),
    [sym_err_invalid_parameter] = ACTIONS(1481),
    [sym_err_user_locked_out] = ACTIONS(1481),
    [sym_err_user_account_expired] = ACTIONS(1481),
    [sym_err_general_error] = ACTIONS(1481),
    [sym_err_authentication_error] = ACTIONS(1481),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1481),
    [sym_tls_validate_certificate] = ACTIONS(1481),
    [sym_tdata] = ACTIONS(1481),
    [sym_tchannel] = ACTIONS(1481),
    [sym_tlevel] = ACTIONS(1481),
    [sym_tbutton] = ACTIONS(1481),
    [sym_ttimeline] = ACTIONS(1481),
    [sym_tcustom] = ACTIONS(1481),
    [sym_url_struct] = ACTIONS(1481),
    [sym_dns_struct] = ACTIONS(1481),
    [sym_ip_address_struct] = ACTIONS(1481),
    [sym_dev_info_struct] = ACTIONS(1481),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1481),
    [sym_data_feed] = ACTIONS(1481),
    [sym_data_field] = ACTIONS(1481),
    [sym_data_record] = ACTIONS(1481),
    [sym_wc_data_feed] = ACTIONS(1481),
    [sym_wc_data_field] = ACTIONS(1481),
    [sym_wc_data_record] = ACTIONS(1481),
    [sym_last_login_info] = ACTIONS(1481),
    [sym____reserved___] = ACTIONS(1481),
    [sym_date] = ACTIONS(1481),
    [sym_day] = ACTIONS(1481),
    [sym_time] = ACTIONS(1481),
    [sym_ldate] = ACTIONS(1481),
    [sym_push_device] = ACTIONS(1481),
    [sym_push_channel] = ACTIONS(1481),
    [sym_push_devchan] = ACTIONS(1481),
    [sym_release_device] = ACTIONS(1481),
    [sym_release_channel] = ACTIONS(1481),
    [sym_release_devchan] = ACTIONS(1481),
    [sym_master_slot] = ACTIONS(1481),
    [sym_get_pulse_time] = ACTIONS(1481),
    [sym_get_timer] = ACTIONS(1481),
    [sym_master_sn] = ACTIONS(1481),
    [sym_system_number] = ACTIONS(1481),
    [sym_button] = ACTIONS(1481),
    [sym_dv_channel] = ACTIONS(1481),
    [sym_data] = ACTIONS(1481),
    [sym_level] = ACTIONS(1481),
    [sym_timeline] = ACTIONS(1481),
    [sym_channel] = ACTIONS(1481),
    [sym_custom] = ACTIONS(1481),
    [sym_atoi] = ACTIONS(1481),
    [sym_atol] = ACTIONS(1481),
    [sym_atof] = ACTIONS(1481),
    [sym_itoa] = ACTIONS(1481),
    [sym_itohex] = ACTIONS(1481),
    [sym_hextoi] = ACTIONS(1481),
    [sym_ftoa] = ACTIONS(1481),
    [sym_find_string] = ACTIONS(1481),
    [sym_left_string] = ACTIONS(1481),
    [sym_right_string] = ACTIONS(1481),
    [sym_mid_string] = ACTIONS(1481),
    [sym_remove_string] = ACTIONS(1481),
    [sym_get_buffer_string] = ACTIONS(1481),
    [sym_get_buffer_char] = ACTIONS(1481),
    [sym_get_multi_buffer_string] = ACTIONS(1481),
    [sym_length_string] = ACTIONS(1481),
    [sym_max_length_string] = ACTIONS(1481),
    [sym_set_length_string] = ACTIONS(1481),
    [sym_compare_string] = ACTIONS(1481),
    [sym_upper_string] = ACTIONS(1481),
    [sym_lower_string] = ACTIONS(1481),
    [sym_get_last] = ACTIONS(1481),
    [sym_date_to_day] = ACTIONS(1481),
    [sym_date_to_month] = ACTIONS(1481),
    [sym_date_to_year] = ACTIONS(1481),
    [sym_time_to_hour] = ACTIONS(1481),
    [sym_time_to_minute] = ACTIONS(1481),
    [sym_time_to_second] = ACTIONS(1481),
    [sym_day_of_week] = ACTIONS(1481),
    [sym_set_pulse_time] = ACTIONS(1481),
    [sym_set_timer] = ACTIONS(1481),
    [sym_file_open] = ACTIONS(1481),
    [sym_file_close] = ACTIONS(1481),
    [sym_file_read] = ACTIONS(1481),
    [sym_file_read_line] = ACTIONS(1481),
    [sym_file_write] = ACTIONS(1481),
    [sym_file_write_line] = ACTIONS(1481),
    [sym_file_copy] = ACTIONS(1481),
    [sym_file_delete] = ACTIONS(1481),
    [sym_file_rename] = ACTIONS(1481),
    [sym_file_dir] = ACTIONS(1481),
    [sym_file_getdir] = ACTIONS(1481),
    [sym_file_setdir] = ACTIONS(1481),
    [sym_file_create_dir] = ACTIONS(1481),
    [sym_file_remove_dir] = ACTIONS(1481),
    [sym_file_seek] = ACTIONS(1481),
    [sym_length_array] = ACTIONS(1481),
    [sym_max_length_array] = ACTIONS(1481),
    [sym_set_length_array] = ACTIONS(1481),
    [sym_ip_client_open] = ACTIONS(1481),
    [sym_ip_client_close] = ACTIONS(1481),
    [sym_ip_server_open] = ACTIONS(1481),
    [sym_ip_server_close] = ACTIONS(1481),
    [sym_ip_bound_client_open] = ACTIONS(1481),
    [sym_ip_mc_server_open] = ACTIONS(1481),
    [sym_ip_set_option] = ACTIONS(1481),
    [sym_ssh_client_open] = ACTIONS(1481),
    [sym_ssh_client_close] = ACTIONS(1481),
    [sym_tls_client_open] = ACTIONS(1481),
    [sym_tls_client_close] = ACTIONS(1481),
    [sym_do_push] = ACTIONS(1481),
    [sym_do_push_timed] = ACTIONS(1481),
    [sym_do_release] = ACTIONS(1481),
    [sym_device_id] = ACTIONS(1481),
    [sym_device_info] = ACTIONS(1481),
    [sym_device_id_string] = ACTIONS(1481),
    [sym_device_standby] = ACTIONS(1481),
    [sym_device_wake] = ACTIONS(1481),
    [sym_timeline_create] = ACTIONS(1481),
    [sym_timeline_kill] = ACTIONS(1481),
    [sym_timeline_reload] = ACTIONS(1481),
    [sym_timeline_pause] = ACTIONS(1481),
    [sym_timeline_restart] = ACTIONS(1481),
    [sym_timeline_set] = ACTIONS(1481),
    [sym_timeline_get] = ACTIONS(1481),
    [sym_timeline_active] = ACTIONS(1481),
    [sym_timeline_dynamic_id] = ACTIONS(1481),
    [sym_variable_to_string] = ACTIONS(1481),
    [sym_string_to_variable] = ACTIONS(1481),
    [sym_length_variable_to_string] = ACTIONS(1481),
    [sym_type_cast] = ACTIONS(1481),
    [sym_variable_to_xml] = ACTIONS(1481),
    [sym_xml_to_variable] = ACTIONS(1481),
    [sym_length_variable_to_xml] = ACTIONS(1481),
    [sym_redirect_string] = ACTIONS(1481),
    [sym_random_number] = ACTIONS(1481),
    [sym_get_unique_id] = ACTIONS(1481),
    [sym_get_system_number] = ACTIONS(1481),
    [sym_set_system_number] = ACTIONS(1481),
    [sym_get_serial_number] = ACTIONS(1481),
    [sym_get_master_build] = ACTIONS(1481),
    [sym_set_validation_code] = ACTIONS(1481),
    [sym_reboot] = ACTIONS(1481),
    [sym_load_duet_module] = ACTIONS(1481),
    [sym_unload_duet_module] = ACTIONS(1481),
    [sym_duet_mem_size_set] = ACTIONS(1481),
    [sym_duet_mem_size_get] = ACTIONS(1481),
    [sym_dynamic_application_device] = ACTIONS(1481),
    [sym_dynamic_polled_port] = ACTIONS(1481),
    [sym_static_port_binding] = ACTIONS(1481),
    [sym_static_ip_binding] = ACTIONS(1481),
    [sym_get_dns_list] = ACTIONS(1481),
    [sym_set_dns_list] = ACTIONS(1481),
    [sym_get_ip_address] = ACTIONS(1481),
    [sym_set_ip_address] = ACTIONS(1481),
    [sym_get_url_list] = ACTIONS(1481),
    [sym_add_url_entry] = ACTIONS(1481),
    [sym_delete_url_entry] = ACTIONS(1481),
    [sym_abs_value] = ACTIONS(1481),
    [sym_max_value] = ACTIONS(1481),
    [sym_min_value] = ACTIONS(1481),
    [sym_exp_value] = ACTIONS(1481),
    [sym_log_value] = ACTIONS(1481),
    [sym_log10_value] = ACTIONS(1481),
    [sym_power_value] = ACTIONS(1481),
    [sym_sqrt_value] = ACTIONS(1481),
    [sym_raw_be] = ACTIONS(1481),
    [sym_raw_le] = ACTIONS(1481),
    [sym_internal_queue_size_set] = ACTIONS(1481),
    [sym_internal_queue_size_get] = ACTIONS(1481),
    [sym_internal_threshold_set] = ACTIONS(1481),
    [sym_internal_threshold_get] = ACTIONS(1481),
    [sym_rebuilt_event] = ACTIONS(1481),
    [sym_do_custom_event] = ACTIONS(1481),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1481),
    [sym_clkmgr_set_clk_source] = ACTIONS(1481),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1481),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1481),
    [sym_clkmgr_get_timezone] = ACTIONS(1481),
    [sym_clkmgr_set_timezone] = ACTIONS(1481),
    [sym_clkmgr_get_resync_period] = ACTIONS(1481),
    [sym_clkmgr_set_resync_period] = ACTIONS(1481),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1481),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1481),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1481),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1481),
    [sym_clkmgr_get_timeservers] = ACTIONS(1481),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1481),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1481),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1481),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1481),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1481),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1481),
    [sym_set_log_level] = ACTIONS(1481),
    [sym_get_log_level] = ACTIONS(1481),
    [sym_amx_log] = ACTIONS(1481),
    [sym_smtp_server_config_set] = ACTIONS(1481),
    [sym_smtp_server_config_get] = ACTIONS(1481),
    [sym_smtp_send] = ACTIONS(1481),
    [sym_validate_netlinx_account] = ACTIONS(1481),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1481),
    [sym_authenticate_certificate] = ACTIONS(1481),
    [sym_audit_netlinx_session_event] = ACTIONS(1481),
    [sym_audit_netlinx_generic_event] = ACTIONS(1481),
    [sym_get_available_flash_disk_space] = ACTIONS(1481),
    [sym_get_max_flash_disk_space] = ACTIONS(1481),
    [sym_data_create_feed] = ACTIONS(1481),
    [sym_data_delete_feed] = ACTIONS(1481),
    [sym_data_publish_feed] = ACTIONS(1481),
    [sym_data_get_published_feed] = ACTIONS(1481),
    [sym_data_add_record] = ACTIONS(1481),
    [sym_data_get_event_record] = ACTIONS(1481),
    [sym__wc_data_create_feed] = ACTIONS(1481),
    [sym__wc_data_add_record] = ACTIONS(1481),
    [sym__wc_data_get_event_record] = ACTIONS(1481),
    [sym_format] = ACTIONS(1481),
    [sym_astro_clock] = ACTIONS(1481),
    [sym_set_outdoor_temperature] = ACTIONS(1481),
    [sym_set_virtual_level_count] = ACTIONS(1481),
    [sym_set_virtual_channel_count] = ACTIONS(1481),
    [sym_set_virtual_port_count] = ACTIONS(1481),
    [sym__wc_encode_int] = ACTIONS(1481),
    [sym__wc_decode_int] = ACTIONS(1481),
    [sym__wc_int] = ACTIONS(1481),
    [sym__wc_to_ch_int] = ACTIONS(1481),
    [sym__ch_to_wc_int] = ACTIONS(1481),
    [sym__wc_find_string_int] = ACTIONS(1481),
    [sym__wc_left_string_int] = ACTIONS(1481),
    [sym__wc_lower_string_int] = ACTIONS(1481),
    [sym__wc_mid_string_int] = ACTIONS(1481),
    [sym__wc_remove_string_int] = ACTIONS(1481),
    [sym__wc_right_string_int] = ACTIONS(1481),
    [sym__wc_upper_string_int] = ACTIONS(1481),
    [sym__wc_compare_string_int] = ACTIONS(1481),
    [sym__wc_get_buffer_char_int] = ACTIONS(1481),
    [sym__wc_get_buffer_string_int] = ACTIONS(1481),
    [sym__wc_concat_string_int] = ACTIONS(1481),
    [sym__wc_file_open_int] = ACTIONS(1481),
    [sym__wc_file_close_int] = ACTIONS(1481),
    [sym__wc_file_read_int] = ACTIONS(1481),
    [sym__wc_file_read_line_int] = ACTIONS(1481),
    [sym__wc_file_write_int] = ACTIONS(1481),
    [sym__wc_file_write_line_int] = ACTIONS(1481),
    [aux_sym_preproc_include_token1] = ACTIONS(1481),
    [aux_sym_preproc_define_token1] = ACTIONS(1483),
    [aux_sym_preproc_warn_token1] = ACTIONS(1483),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1483),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1483),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1483),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1483),
    [anon_sym_RBRACE] = ACTIONS(1483),
    [anon_sym_COLON] = ACTIONS(1660),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1483),
    [anon_sym_BANG] = ACTIONS(1481),
    [anon_sym_TILDE] = ACTIONS(1483),
    [anon_sym_DASH] = ACTIONS(1625),
    [anon_sym_PLUS] = ACTIONS(1625),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1631),
    [anon_sym_AMP_AMP] = ACTIONS(1633),
    [anon_sym_PIPE] = ACTIONS(1613),
    [anon_sym_CARET] = ACTIONS(1635),
    [anon_sym_AMP] = ACTIONS(1611),
    [anon_sym_EQ_EQ] = ACTIONS(1637),
    [anon_sym_BANG_EQ] = ACTIONS(1637),
    [anon_sym_LT_GT] = ACTIONS(1637),
    [anon_sym_GT] = ACTIONS(1639),
    [anon_sym_GT_EQ] = ACTIONS(1641),
    [anon_sym_LT_EQ] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1639),
    [anon_sym_LT_LT] = ACTIONS(1643),
    [anon_sym_GT_GT] = ACTIONS(1643),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1483),
    [anon_sym_SQUOTE] = ACTIONS(1483),
    [sym_number_literal] = ACTIONS(1483),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1483),
  },
  [STATE(167)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1611),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1615),
    [sym_rshift] = ACTIONS(1615),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1625),
    [anon_sym_PLUS] = ACTIONS(1625),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1611),
    [anon_sym_EQ_EQ] = ACTIONS(1637),
    [anon_sym_BANG_EQ] = ACTIONS(1637),
    [anon_sym_LT_GT] = ACTIONS(1637),
    [anon_sym_GT] = ACTIONS(1639),
    [anon_sym_GT_EQ] = ACTIONS(1641),
    [anon_sym_LT_EQ] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1639),
    [anon_sym_LT_LT] = ACTIONS(1643),
    [anon_sym_GT_GT] = ACTIONS(1643),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(168)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1615),
    [sym_rshift] = ACTIONS(1615),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1625),
    [anon_sym_PLUS] = ACTIONS(1625),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1637),
    [anon_sym_BANG_EQ] = ACTIONS(1637),
    [anon_sym_LT_GT] = ACTIONS(1637),
    [anon_sym_GT] = ACTIONS(1639),
    [anon_sym_GT_EQ] = ACTIONS(1641),
    [anon_sym_LT_EQ] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1639),
    [anon_sym_LT_LT] = ACTIONS(1643),
    [anon_sym_GT_GT] = ACTIONS(1643),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(169)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1609),
    [sym_band] = ACTIONS(1611),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1609),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1615),
    [sym_rshift] = ACTIONS(1615),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1625),
    [anon_sym_PLUS] = ACTIONS(1625),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1635),
    [anon_sym_AMP] = ACTIONS(1611),
    [anon_sym_EQ_EQ] = ACTIONS(1637),
    [anon_sym_BANG_EQ] = ACTIONS(1637),
    [anon_sym_LT_GT] = ACTIONS(1637),
    [anon_sym_GT] = ACTIONS(1639),
    [anon_sym_GT_EQ] = ACTIONS(1641),
    [anon_sym_LT_EQ] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1639),
    [anon_sym_LT_LT] = ACTIONS(1643),
    [anon_sym_GT_GT] = ACTIONS(1643),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(170)] = {
    [sym_argument_list] = STATE(171),
    [aux_sym_subscript_expression_repeat1] = STATE(162),
    [sym_identifier] = ACTIONS(1413),
    [sym_define_call_keyword] = ACTIONS(1413),
    [sym_define_function_keyword] = ACTIONS(1413),
    [sym_define_library_function_keyword] = ACTIONS(1413),
    [sym_define_module_keyword] = ACTIONS(1413),
    [sym_char_keyword] = ACTIONS(1413),
    [sym_widechar_keyword] = ACTIONS(1413),
    [sym_integer_keyword] = ACTIONS(1413),
    [sym_sinteger_keyword] = ACTIONS(1413),
    [sym_long_keyword] = ACTIONS(1413),
    [sym_slong_keyword] = ACTIONS(1413),
    [sym_float_keyword] = ACTIONS(1413),
    [sym_double_keyword] = ACTIONS(1413),
    [sym_variant_keyword] = ACTIONS(1413),
    [sym_variantarray_keyword] = ACTIONS(1413),
    [sym_local_var_keyword] = ACTIONS(1413),
    [sym_stack_var_keyword] = ACTIONS(1413),
    [sym_constant_keyword] = ACTIONS(1413),
    [sym_volatile_keyword] = ACTIONS(1413),
    [sym_non_volatile_keyword] = ACTIONS(1413),
    [sym_persistent_keyword] = ACTIONS(1413),
    [sym_dev_keyword] = ACTIONS(1413),
    [sym_devlev_keyword] = ACTIONS(1413),
    [sym_devchan_keyword] = ACTIONS(1413),
    [sym_if_keyword] = ACTIONS(1413),
    [sym_else_keyword] = ACTIONS(1413),
    [sym_switch_keyword] = ACTIONS(1413),
    [sym_case_keyword] = ACTIONS(1413),
    [sym_default_keyword] = ACTIONS(1413),
    [sym_while_keyword] = ACTIONS(1413),
    [sym_for_keyword] = ACTIONS(1413),
    [sym_break_keyword] = ACTIONS(1413),
    [sym_continue_keyword] = ACTIONS(1413),
    [sym_return_keyword] = ACTIONS(1413),
    [sym_select_keyword] = ACTIONS(1413),
    [sym_active_keyword] = ACTIONS(1413),
    [sym_struct_keyword] = ACTIONS(1413),
    [sym_structure_keyword] = ACTIONS(1413),
    [aux_sym_on_keyword_token1] = ACTIONS(1413),
    [aux_sym_off_keyword_token1] = ACTIONS(1413),
    [sym_send_level_keyword] = ACTIONS(1413),
    [sym_send_string_keyword] = ACTIONS(1413),
    [sym_send_command_keyword] = ACTIONS(1413),
    [sym_clear_buffer_keyword] = ACTIONS(1413),
    [sym_create_buffer_keyword] = ACTIONS(1413),
    [sym_create_multi_buffer_keyword] = ACTIONS(1413),
    [sym_call_keyword] = ACTIONS(1413),
    [sym_system_call_keyword] = ACTIONS(1413),
    [sym_devchan_to_keyword] = ACTIONS(1413),
    [sym_devchan_min_to_keyword] = ACTIONS(1413),
    [sym_devchan_total_off_keyword] = ACTIONS(1413),
    [sym_devchan_pulse_keyword] = ACTIONS(1413),
    [sym_wait_keyword] = ACTIONS(1413),
    [sym_cancel_wait_keyword] = ACTIONS(1413),
    [sym_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_wait_until_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_keyword] = ACTIONS(1413),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1413),
    [sym_rshift] = ACTIONS(1413),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym_tdata] = ACTIONS(1413),
    [sym_tchannel] = ACTIONS(1413),
    [sym_tlevel] = ACTIONS(1413),
    [sym_tbutton] = ACTIONS(1413),
    [sym_ttimeline] = ACTIONS(1413),
    [sym_tcustom] = ACTIONS(1413),
    [sym_url_struct] = ACTIONS(1413),
    [sym_dns_struct] = ACTIONS(1413),
    [sym_ip_address_struct] = ACTIONS(1413),
    [sym_dev_info_struct] = ACTIONS(1413),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1413),
    [sym_data_feed] = ACTIONS(1413),
    [sym_data_field] = ACTIONS(1413),
    [sym_data_record] = ACTIONS(1413),
    [sym_wc_data_feed] = ACTIONS(1413),
    [sym_wc_data_field] = ACTIONS(1413),
    [sym_wc_data_record] = ACTIONS(1413),
    [sym_last_login_info] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_include_token1] = ACTIONS(1413),
    [aux_sym_preproc_define_token1] = ACTIONS(1415),
    [aux_sym_preproc_warn_token1] = ACTIONS(1415),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(1617),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(1621),
    [anon_sym_SEMI] = ACTIONS(1415),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1625),
    [anon_sym_PLUS] = ACTIONS(1625),
    [anon_sym_STAR] = ACTIONS(1627),
    [anon_sym_SLASH] = ACTIONS(1629),
    [anon_sym_PERCENT] = ACTIONS(1627),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1415),
    [anon_sym_GT_GT] = ACTIONS(1415),
    [anon_sym_DASH_DASH] = ACTIONS(1645),
    [anon_sym_PLUS_PLUS] = ACTIONS(1645),
    [anon_sym_DOT] = ACTIONS(1647),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1415),
  },
  [STATE(171)] = {
    [sym_identifier] = ACTIONS(1511),
    [sym_define_call_keyword] = ACTIONS(1511),
    [sym_define_function_keyword] = ACTIONS(1511),
    [sym_define_library_function_keyword] = ACTIONS(1511),
    [sym_define_module_keyword] = ACTIONS(1511),
    [sym_char_keyword] = ACTIONS(1511),
    [sym_widechar_keyword] = ACTIONS(1511),
    [sym_integer_keyword] = ACTIONS(1511),
    [sym_sinteger_keyword] = ACTIONS(1511),
    [sym_long_keyword] = ACTIONS(1511),
    [sym_slong_keyword] = ACTIONS(1511),
    [sym_float_keyword] = ACTIONS(1511),
    [sym_double_keyword] = ACTIONS(1511),
    [sym_variant_keyword] = ACTIONS(1511),
    [sym_variantarray_keyword] = ACTIONS(1511),
    [sym_local_var_keyword] = ACTIONS(1511),
    [sym_stack_var_keyword] = ACTIONS(1511),
    [sym_constant_keyword] = ACTIONS(1511),
    [sym_volatile_keyword] = ACTIONS(1511),
    [sym_non_volatile_keyword] = ACTIONS(1511),
    [sym_persistent_keyword] = ACTIONS(1511),
    [sym_dev_keyword] = ACTIONS(1511),
    [sym_devlev_keyword] = ACTIONS(1511),
    [sym_devchan_keyword] = ACTIONS(1511),
    [sym_if_keyword] = ACTIONS(1511),
    [sym_else_keyword] = ACTIONS(1511),
    [sym_switch_keyword] = ACTIONS(1511),
    [sym_case_keyword] = ACTIONS(1511),
    [sym_default_keyword] = ACTIONS(1511),
    [sym_while_keyword] = ACTIONS(1511),
    [sym_for_keyword] = ACTIONS(1511),
    [sym_break_keyword] = ACTIONS(1511),
    [sym_continue_keyword] = ACTIONS(1511),
    [sym_return_keyword] = ACTIONS(1511),
    [sym_select_keyword] = ACTIONS(1511),
    [sym_active_keyword] = ACTIONS(1511),
    [sym_struct_keyword] = ACTIONS(1511),
    [sym_structure_keyword] = ACTIONS(1511),
    [aux_sym_on_keyword_token1] = ACTIONS(1511),
    [aux_sym_off_keyword_token1] = ACTIONS(1511),
    [sym_send_level_keyword] = ACTIONS(1511),
    [sym_send_string_keyword] = ACTIONS(1511),
    [sym_send_command_keyword] = ACTIONS(1511),
    [sym_clear_buffer_keyword] = ACTIONS(1511),
    [sym_create_buffer_keyword] = ACTIONS(1511),
    [sym_create_multi_buffer_keyword] = ACTIONS(1511),
    [sym_call_keyword] = ACTIONS(1511),
    [sym_system_call_keyword] = ACTIONS(1511),
    [sym_devchan_to_keyword] = ACTIONS(1511),
    [sym_devchan_min_to_keyword] = ACTIONS(1511),
    [sym_devchan_total_off_keyword] = ACTIONS(1511),
    [sym_devchan_pulse_keyword] = ACTIONS(1511),
    [sym_wait_keyword] = ACTIONS(1511),
    [sym_cancel_wait_keyword] = ACTIONS(1511),
    [sym_wait_until_keyword] = ACTIONS(1511),
    [sym_cancel_wait_until_keyword] = ACTIONS(1511),
    [sym_cancel_all_wait_keyword] = ACTIONS(1511),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1511),
    [sym_and] = ACTIONS(1511),
    [sym_or] = ACTIONS(1511),
    [sym_not] = ACTIONS(1511),
    [sym_xor] = ACTIONS(1511),
    [sym_band] = ACTIONS(1511),
    [sym_bor] = ACTIONS(1511),
    [sym_bxor] = ACTIONS(1511),
    [sym_bnot] = ACTIONS(1511),
    [sym_lshift] = ACTIONS(1511),
    [sym_rshift] = ACTIONS(1511),
    [sym___file__] = ACTIONS(1511),
    [sym___date__] = ACTIONS(1511),
    [sym___ldate__] = ACTIONS(1511),
    [sym___time__] = ACTIONS(1511),
    [sym___line__] = ACTIONS(1511),
    [sym___name__] = ACTIONS(1511),
    [sym___netlinx__] = ACTIONS(1511),
    [sym_netlinx_axi_version] = ACTIONS(1511),
    [sym_url_flg_tcp] = ACTIONS(1511),
    [sym_url_flg_accinfopresent] = ACTIONS(1511),
    [sym_url_flg_temp] = ACTIONS(1511),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1511),
    [sym_url_flg_stat_mask] = ACTIONS(1511),
    [sym_url_flg_stat_lookup] = ACTIONS(1511),
    [sym_url_flg_stat_connecting] = ACTIONS(1511),
    [sym_url_flg_stat_waiting] = ACTIONS(1511),
    [sym_url_flg_stat_connected] = ACTIONS(1511),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1511),
    [sym_first_virtual_device] = ACTIONS(1511),
    [sym_first_local_port] = ACTIONS(1511),
    [sym_dynamic_virtual_device] = ACTIONS(1511),
    [sym_timeline_once] = ACTIONS(1511),
    [sym_timeline_repeat] = ACTIONS(1511),
    [sym_timeline_absolute] = ACTIONS(1511),
    [sym_timeline_relative] = ACTIONS(1511),
    [sym_do_push_timed_infinite] = ACTIONS(1511),
    [sym_source_type_no_address] = ACTIONS(1511),
    [sym_source_type_neuron_id] = ACTIONS(1511),
    [sym_source_type_ip_address] = ACTIONS(1511),
    [sym_source_type_axlink] = ACTIONS(1511),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1511),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1511),
    [sym_source_type_ip_socket_address] = ACTIONS(1511),
    [sym_source_type_rs232] = ACTIONS(1511),
    [sym_source_type_internal] = ACTIONS(1511),
    [sym_source_type_mac_address] = ACTIONS(1511),
    [sym_source_type_ipv4_port] = ACTIONS(1511),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1511),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1511),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1511),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1511),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1511),
    [sym_true] = ACTIONS(1511),
    [sym_false] = ACTIONS(1511),
    [sym_file_read_only] = ACTIONS(1511),
    [sym_file_rw_new] = ACTIONS(1511),
    [sym_file_rw_append] = ACTIONS(1511),
    [sym_ip_tcp] = ACTIONS(1511),
    [sym_ip_udp] = ACTIONS(1511),
    [sym_ip_udp_2way] = ACTIONS(1511),
    [sym_xml_encode_types] = ACTIONS(1511),
    [sym_xml_encode_char_as_list] = ACTIONS(1511),
    [sym_xml_encode_le] = ACTIONS(1511),
    [sym_xml_decode_types] = ACTIONS(1511),
    [sym_xml_decode_no_preserve] = ACTIONS(1511),
    [sym_duet_dev_type_display_device] = ACTIONS(1511),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1511),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1511),
    [sym_duet_dev_type_io_device] = ACTIONS(1511),
    [sym_duet_dev_type_relay_device] = ACTIONS(1511),
    [sym_duet_dev_type_ups] = ACTIONS(1511),
    [sym_duet_dev_type_amplifier] = ACTIONS(1511),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1511),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1511),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1511),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1511),
    [sym_duet_dev_type_camera] = ACTIONS(1511),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1511),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1511),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1511),
    [sym_duet_dev_type_dss] = ACTIONS(1511),
    [sym_duet_dev_type_dvr] = ACTIONS(1511),
    [sym_duet_dev_type_disc_device] = ACTIONS(1511),
    [sym_duet_dev_type_document_camera] = ACTIONS(1511),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1511),
    [sym_duet_dev_type_hvac] = ACTIONS(1511),
    [sym_duet_dev_type_keypad] = ACTIONS(1511),
    [sym_duet_dev_type_light] = ACTIONS(1511),
    [sym_duet_dev_type_monitor] = ACTIONS(1511),
    [sym_duet_dev_type_motor] = ACTIONS(1511),
    [sym_duet_dev_type_multi_window] = ACTIONS(1511),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1511),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1511),
    [sym_duet_dev_type_receiver] = ACTIONS(1511),
    [sym_duet_dev_type_security_system] = ACTIONS(1511),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1511),
    [sym_duet_dev_type_settop_box] = ACTIONS(1511),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1511),
    [sym_duet_dev_type_switcher] = ACTIONS(1511),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1511),
    [sym_duet_dev_type_tv] = ACTIONS(1511),
    [sym_duet_dev_type_utility] = ACTIONS(1511),
    [sym_duet_dev_type_vcr] = ACTIONS(1511),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1511),
    [sym_duet_dev_type_video_processor] = ACTIONS(1511),
    [sym_duet_dev_type_video_projector] = ACTIONS(1511),
    [sym_duet_dev_type_video_wall] = ACTIONS(1511),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1511),
    [sym_duet_dev_type_weather] = ACTIONS(1511),
    [sym_duet_dev_not_polled] = ACTIONS(1511),
    [sym_duet_dev_polled] = ACTIONS(1511),
    [sym_clkmgr_mode_network] = ACTIONS(1511),
    [sym_clkmgr_mode_standalone] = ACTIONS(1511),
    [sym_normal_standby] = ACTIONS(1511),
    [sym_normal_wake] = ACTIONS(1511),
    [sym_ip_multicast_ttl_option] = ACTIONS(1511),
    [sym_ip_ttl_subnet] = ACTIONS(1511),
    [sym_ip_ttl_site] = ACTIONS(1511),
    [sym_ip_ttl_region] = ACTIONS(1511),
    [sym_ip_ttl_continent] = ACTIONS(1511),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1511),
    [sym_ip_nodelay_on] = ACTIONS(1511),
    [sym_ip_nodelay_off] = ACTIONS(1511),
    [sym_amx_error] = ACTIONS(1511),
    [sym_amx_warning] = ACTIONS(1511),
    [sym_amx_info] = ACTIONS(1511),
    [sym_amx_debug] = ACTIONS(1511),
    [sym_netlinx_login_success] = ACTIONS(1511),
    [sym_netlinx_login_fail] = ACTIONS(1511),
    [sym_netlinx_logout] = ACTIONS(1511),
    [sym_null_str] = ACTIONS(1511),
    [sym_smtp_address] = ACTIONS(1511),
    [sym_smtp_port_number] = ACTIONS(1511),
    [sym_smtp_username] = ACTIONS(1511),
    [sym_smtp_password] = ACTIONS(1511),
    [sym_smtp_require_tls] = ACTIONS(1511),
    [sym_smtp_from] = ACTIONS(1511),
    [sym_smtp_tls_true] = ACTIONS(1511),
    [sym_smtp_tls_false] = ACTIONS(1511),
    [sym_listview_on_row_select_event] = ACTIONS(1511),
    [sym_data_structure_unknown] = ACTIONS(1511),
    [sym_data_structure_datafeed] = ACTIONS(1511),
    [sym_data_structure_datarecordset] = ACTIONS(1511),
    [sym_data_structure_datarecord] = ACTIONS(1511),
    [sym_data_structure_datafield] = ACTIONS(1511),
    [sym_data_max_content_field_count] = ACTIONS(1511),
    [sym_data_max_metadata_field_count] = ACTIONS(1511),
    [sym_data_max_id_length] = ACTIONS(1511),
    [sym_data_max_name_length] = ACTIONS(1511),
    [sym_data_max_description_length] = ACTIONS(1511),
    [sym_data_max_type_length] = ACTIONS(1511),
    [sym_data_max_format_length] = ACTIONS(1511),
    [sym_data_max_source_length] = ACTIONS(1511),
    [sym_data_max_value_length] = ACTIONS(1511),
    [sym_data_max_label_length] = ACTIONS(1511),
    [sym_data_max_url_length] = ACTIONS(1511),
    [sym_max_last_login_info_length] = ACTIONS(1511),
    [sym_data_type_unknown] = ACTIONS(1511),
    [sym_data_type_string] = ACTIONS(1511),
    [sym_data_type_datetime] = ACTIONS(1511),
    [sym_data_type_date] = ACTIONS(1511),
    [sym_data_type_time] = ACTIONS(1511),
    [sym_data_type_image] = ACTIONS(1511),
    [sym_data_format_url] = ACTIONS(1511),
    [sym_data_format_phone] = ACTIONS(1511),
    [sym_data_format_email] = ACTIONS(1511),
    [sym_data_format_iso8601] = ACTIONS(1511),
    [sym_auditlog_priv] = ACTIONS(1511),
    [sym_configuration_priv] = ACTIONS(1511),
    [sym_deviceconfig_priv] = ACTIONS(1511),
    [sym_ftp_priv] = ACTIONS(1511),
    [sym_http_priv] = ACTIONS(1511),
    [sym_networkconfig_priv] = ACTIONS(1511),
    [sym_programport_priv] = ACTIONS(1511),
    [sym_remoteui_priv] = ACTIONS(1511),
    [sym_securitycontrol_priv] = ACTIONS(1511),
    [sym_softwaremanagement_priv] = ACTIONS(1511),
    [sym_terminal_priv] = ACTIONS(1511),
    [sym_tpadmin_priv] = ACTIONS(1511),
    [sym_user1_priv] = ACTIONS(1511),
    [sym_user2_priv] = ACTIONS(1511),
    [sym_user3_priv] = ACTIONS(1511),
    [sym_user4_priv] = ACTIONS(1511),
    [sym_usermanagement_priv] = ACTIONS(1511),
    [sym_valid_account] = ACTIONS(1511),
    [sym_err_name_invalid] = ACTIONS(1511),
    [sym_err_password_invalid] = ACTIONS(1511),
    [sym_err_invalid_account] = ACTIONS(1511),
    [sym_err_no_authorization] = ACTIONS(1511),
    [sym_err_invalid_parameter] = ACTIONS(1511),
    [sym_err_user_locked_out] = ACTIONS(1511),
    [sym_err_user_account_expired] = ACTIONS(1511),
    [sym_err_general_error] = ACTIONS(1511),
    [sym_err_authentication_error] = ACTIONS(1511),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1511),
    [sym_tls_validate_certificate] = ACTIONS(1511),
    [sym_tdata] = ACTIONS(1511),
    [sym_tchannel] = ACTIONS(1511),
    [sym_tlevel] = ACTIONS(1511),
    [sym_tbutton] = ACTIONS(1511),
    [sym_ttimeline] = ACTIONS(1511),
    [sym_tcustom] = ACTIONS(1511),
    [sym_url_struct] = ACTIONS(1511),
    [sym_dns_struct] = ACTIONS(1511),
    [sym_ip_address_struct] = ACTIONS(1511),
    [sym_dev_info_struct] = ACTIONS(1511),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1511),
    [sym_data_feed] = ACTIONS(1511),
    [sym_data_field] = ACTIONS(1511),
    [sym_data_record] = ACTIONS(1511),
    [sym_wc_data_feed] = ACTIONS(1511),
    [sym_wc_data_field] = ACTIONS(1511),
    [sym_wc_data_record] = ACTIONS(1511),
    [sym_last_login_info] = ACTIONS(1511),
    [sym____reserved___] = ACTIONS(1511),
    [sym_date] = ACTIONS(1511),
    [sym_day] = ACTIONS(1511),
    [sym_time] = ACTIONS(1511),
    [sym_ldate] = ACTIONS(1511),
    [sym_push_device] = ACTIONS(1511),
    [sym_push_channel] = ACTIONS(1511),
    [sym_push_devchan] = ACTIONS(1511),
    [sym_release_device] = ACTIONS(1511),
    [sym_release_channel] = ACTIONS(1511),
    [sym_release_devchan] = ACTIONS(1511),
    [sym_master_slot] = ACTIONS(1511),
    [sym_get_pulse_time] = ACTIONS(1511),
    [sym_get_timer] = ACTIONS(1511),
    [sym_master_sn] = ACTIONS(1511),
    [sym_system_number] = ACTIONS(1511),
    [sym_button] = ACTIONS(1511),
    [sym_dv_channel] = ACTIONS(1511),
    [sym_data] = ACTIONS(1511),
    [sym_level] = ACTIONS(1511),
    [sym_timeline] = ACTIONS(1511),
    [sym_channel] = ACTIONS(1511),
    [sym_custom] = ACTIONS(1511),
    [sym_atoi] = ACTIONS(1511),
    [sym_atol] = ACTIONS(1511),
    [sym_atof] = ACTIONS(1511),
    [sym_itoa] = ACTIONS(1511),
    [sym_itohex] = ACTIONS(1511),
    [sym_hextoi] = ACTIONS(1511),
    [sym_ftoa] = ACTIONS(1511),
    [sym_find_string] = ACTIONS(1511),
    [sym_left_string] = ACTIONS(1511),
    [sym_right_string] = ACTIONS(1511),
    [sym_mid_string] = ACTIONS(1511),
    [sym_remove_string] = ACTIONS(1511),
    [sym_get_buffer_string] = ACTIONS(1511),
    [sym_get_buffer_char] = ACTIONS(1511),
    [sym_get_multi_buffer_string] = ACTIONS(1511),
    [sym_length_string] = ACTIONS(1511),
    [sym_max_length_string] = ACTIONS(1511),
    [sym_set_length_string] = ACTIONS(1511),
    [sym_compare_string] = ACTIONS(1511),
    [sym_upper_string] = ACTIONS(1511),
    [sym_lower_string] = ACTIONS(1511),
    [sym_get_last] = ACTIONS(1511),
    [sym_date_to_day] = ACTIONS(1511),
    [sym_date_to_month] = ACTIONS(1511),
    [sym_date_to_year] = ACTIONS(1511),
    [sym_time_to_hour] = ACTIONS(1511),
    [sym_time_to_minute] = ACTIONS(1511),
    [sym_time_to_second] = ACTIONS(1511),
    [sym_day_of_week] = ACTIONS(1511),
    [sym_set_pulse_time] = ACTIONS(1511),
    [sym_set_timer] = ACTIONS(1511),
    [sym_file_open] = ACTIONS(1511),
    [sym_file_close] = ACTIONS(1511),
    [sym_file_read] = ACTIONS(1511),
    [sym_file_read_line] = ACTIONS(1511),
    [sym_file_write] = ACTIONS(1511),
    [sym_file_write_line] = ACTIONS(1511),
    [sym_file_copy] = ACTIONS(1511),
    [sym_file_delete] = ACTIONS(1511),
    [sym_file_rename] = ACTIONS(1511),
    [sym_file_dir] = ACTIONS(1511),
    [sym_file_getdir] = ACTIONS(1511),
    [sym_file_setdir] = ACTIONS(1511),
    [sym_file_create_dir] = ACTIONS(1511),
    [sym_file_remove_dir] = ACTIONS(1511),
    [sym_file_seek] = ACTIONS(1511),
    [sym_length_array] = ACTIONS(1511),
    [sym_max_length_array] = ACTIONS(1511),
    [sym_set_length_array] = ACTIONS(1511),
    [sym_ip_client_open] = ACTIONS(1511),
    [sym_ip_client_close] = ACTIONS(1511),
    [sym_ip_server_open] = ACTIONS(1511),
    [sym_ip_server_close] = ACTIONS(1511),
    [sym_ip_bound_client_open] = ACTIONS(1511),
    [sym_ip_mc_server_open] = ACTIONS(1511),
    [sym_ip_set_option] = ACTIONS(1511),
    [sym_ssh_client_open] = ACTIONS(1511),
    [sym_ssh_client_close] = ACTIONS(1511),
    [sym_tls_client_open] = ACTIONS(1511),
    [sym_tls_client_close] = ACTIONS(1511),
    [sym_do_push] = ACTIONS(1511),
    [sym_do_push_timed] = ACTIONS(1511),
    [sym_do_release] = ACTIONS(1511),
    [sym_device_id] = ACTIONS(1511),
    [sym_device_info] = ACTIONS(1511),
    [sym_device_id_string] = ACTIONS(1511),
    [sym_device_standby] = ACTIONS(1511),
    [sym_device_wake] = ACTIONS(1511),
    [sym_timeline_create] = ACTIONS(1511),
    [sym_timeline_kill] = ACTIONS(1511),
    [sym_timeline_reload] = ACTIONS(1511),
    [sym_timeline_pause] = ACTIONS(1511),
    [sym_timeline_restart] = ACTIONS(1511),
    [sym_timeline_set] = ACTIONS(1511),
    [sym_timeline_get] = ACTIONS(1511),
    [sym_timeline_active] = ACTIONS(1511),
    [sym_timeline_dynamic_id] = ACTIONS(1511),
    [sym_variable_to_string] = ACTIONS(1511),
    [sym_string_to_variable] = ACTIONS(1511),
    [sym_length_variable_to_string] = ACTIONS(1511),
    [sym_type_cast] = ACTIONS(1511),
    [sym_variable_to_xml] = ACTIONS(1511),
    [sym_xml_to_variable] = ACTIONS(1511),
    [sym_length_variable_to_xml] = ACTIONS(1511),
    [sym_redirect_string] = ACTIONS(1511),
    [sym_random_number] = ACTIONS(1511),
    [sym_get_unique_id] = ACTIONS(1511),
    [sym_get_system_number] = ACTIONS(1511),
    [sym_set_system_number] = ACTIONS(1511),
    [sym_get_serial_number] = ACTIONS(1511),
    [sym_get_master_build] = ACTIONS(1511),
    [sym_set_validation_code] = ACTIONS(1511),
    [sym_reboot] = ACTIONS(1511),
    [sym_load_duet_module] = ACTIONS(1511),
    [sym_unload_duet_module] = ACTIONS(1511),
    [sym_duet_mem_size_set] = ACTIONS(1511),
    [sym_duet_mem_size_get] = ACTIONS(1511),
    [sym_dynamic_application_device] = ACTIONS(1511),
    [sym_dynamic_polled_port] = ACTIONS(1511),
    [sym_static_port_binding] = ACTIONS(1511),
    [sym_static_ip_binding] = ACTIONS(1511),
    [sym_get_dns_list] = ACTIONS(1511),
    [sym_set_dns_list] = ACTIONS(1511),
    [sym_get_ip_address] = ACTIONS(1511),
    [sym_set_ip_address] = ACTIONS(1511),
    [sym_get_url_list] = ACTIONS(1511),
    [sym_add_url_entry] = ACTIONS(1511),
    [sym_delete_url_entry] = ACTIONS(1511),
    [sym_abs_value] = ACTIONS(1511),
    [sym_max_value] = ACTIONS(1511),
    [sym_min_value] = ACTIONS(1511),
    [sym_exp_value] = ACTIONS(1511),
    [sym_log_value] = ACTIONS(1511),
    [sym_log10_value] = ACTIONS(1511),
    [sym_power_value] = ACTIONS(1511),
    [sym_sqrt_value] = ACTIONS(1511),
    [sym_raw_be] = ACTIONS(1511),
    [sym_raw_le] = ACTIONS(1511),
    [sym_internal_queue_size_set] = ACTIONS(1511),
    [sym_internal_queue_size_get] = ACTIONS(1511),
    [sym_internal_threshold_set] = ACTIONS(1511),
    [sym_internal_threshold_get] = ACTIONS(1511),
    [sym_rebuilt_event] = ACTIONS(1511),
    [sym_do_custom_event] = ACTIONS(1511),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1511),
    [sym_clkmgr_set_clk_source] = ACTIONS(1511),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1511),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1511),
    [sym_clkmgr_get_timezone] = ACTIONS(1511),
    [sym_clkmgr_set_timezone] = ACTIONS(1511),
    [sym_clkmgr_get_resync_period] = ACTIONS(1511),
    [sym_clkmgr_set_resync_period] = ACTIONS(1511),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1511),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1511),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1511),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1511),
    [sym_clkmgr_get_timeservers] = ACTIONS(1511),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1511),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1511),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1511),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1511),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1511),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1511),
    [sym_set_log_level] = ACTIONS(1511),
    [sym_get_log_level] = ACTIONS(1511),
    [sym_amx_log] = ACTIONS(1511),
    [sym_smtp_server_config_set] = ACTIONS(1511),
    [sym_smtp_server_config_get] = ACTIONS(1511),
    [sym_smtp_send] = ACTIONS(1511),
    [sym_validate_netlinx_account] = ACTIONS(1511),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1511),
    [sym_authenticate_certificate] = ACTIONS(1511),
    [sym_audit_netlinx_session_event] = ACTIONS(1511),
    [sym_audit_netlinx_generic_event] = ACTIONS(1511),
    [sym_get_available_flash_disk_space] = ACTIONS(1511),
    [sym_get_max_flash_disk_space] = ACTIONS(1511),
    [sym_data_create_feed] = ACTIONS(1511),
    [sym_data_delete_feed] = ACTIONS(1511),
    [sym_data_publish_feed] = ACTIONS(1511),
    [sym_data_get_published_feed] = ACTIONS(1511),
    [sym_data_add_record] = ACTIONS(1511),
    [sym_data_get_event_record] = ACTIONS(1511),
    [sym__wc_data_create_feed] = ACTIONS(1511),
    [sym__wc_data_add_record] = ACTIONS(1511),
    [sym__wc_data_get_event_record] = ACTIONS(1511),
    [sym_format] = ACTIONS(1511),
    [sym_astro_clock] = ACTIONS(1511),
    [sym_set_outdoor_temperature] = ACTIONS(1511),
    [sym_set_virtual_level_count] = ACTIONS(1511),
    [sym_set_virtual_channel_count] = ACTIONS(1511),
    [sym_set_virtual_port_count] = ACTIONS(1511),
    [sym__wc_encode_int] = ACTIONS(1511),
    [sym__wc_decode_int] = ACTIONS(1511),
    [sym__wc_int] = ACTIONS(1511),
    [sym__wc_to_ch_int] = ACTIONS(1511),
    [sym__ch_to_wc_int] = ACTIONS(1511),
    [sym__wc_find_string_int] = ACTIONS(1511),
    [sym__wc_left_string_int] = ACTIONS(1511),
    [sym__wc_lower_string_int] = ACTIONS(1511),
    [sym__wc_mid_string_int] = ACTIONS(1511),
    [sym__wc_remove_string_int] = ACTIONS(1511),
    [sym__wc_right_string_int] = ACTIONS(1511),
    [sym__wc_upper_string_int] = ACTIONS(1511),
    [sym__wc_compare_string_int] = ACTIONS(1511),
    [sym__wc_get_buffer_char_int] = ACTIONS(1511),
    [sym__wc_get_buffer_string_int] = ACTIONS(1511),
    [sym__wc_concat_string_int] = ACTIONS(1511),
    [sym__wc_file_open_int] = ACTIONS(1511),
    [sym__wc_file_close_int] = ACTIONS(1511),
    [sym__wc_file_read_int] = ACTIONS(1511),
    [sym__wc_file_read_line_int] = ACTIONS(1511),
    [sym__wc_file_write_int] = ACTIONS(1511),
    [sym__wc_file_write_line_int] = ACTIONS(1511),
    [anon_sym_EQ] = ACTIONS(1511),
    [aux_sym_preproc_include_token1] = ACTIONS(1511),
    [aux_sym_preproc_define_token1] = ACTIONS(1513),
    [aux_sym_preproc_warn_token1] = ACTIONS(1513),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1513),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1513),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1513),
    [anon_sym_LBRACK] = ACTIONS(1513),
    [anon_sym_LBRACE] = ACTIONS(1513),
    [anon_sym_RBRACE] = ACTIONS(1513),
    [anon_sym_COLON] = ACTIONS(1513),
    [anon_sym_LPAREN] = ACTIONS(1511),
    [anon_sym_SEMI] = ACTIONS(1513),
    [anon_sym_BANG] = ACTIONS(1511),
    [anon_sym_TILDE] = ACTIONS(1513),
    [anon_sym_DASH] = ACTIONS(1511),
    [anon_sym_PLUS] = ACTIONS(1511),
    [anon_sym_STAR] = ACTIONS(1513),
    [anon_sym_SLASH] = ACTIONS(1511),
    [anon_sym_PERCENT] = ACTIONS(1513),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_AMP_AMP] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1511),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_AMP] = ACTIONS(1511),
    [anon_sym_EQ_EQ] = ACTIONS(1513),
    [anon_sym_BANG_EQ] = ACTIONS(1513),
    [anon_sym_LT_GT] = ACTIONS(1513),
    [anon_sym_GT] = ACTIONS(1511),
    [anon_sym_GT_EQ] = ACTIONS(1513),
    [anon_sym_LT_EQ] = ACTIONS(1513),
    [anon_sym_LT] = ACTIONS(1511),
    [anon_sym_LT_LT] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1513),
    [anon_sym_DASH_DASH] = ACTIONS(1513),
    [anon_sym_PLUS_PLUS] = ACTIONS(1513),
    [anon_sym_DOT] = ACTIONS(1511),
    [anon_sym_DQUOTE] = ACTIONS(1513),
    [anon_sym_SQUOTE] = ACTIONS(1513),
    [sym_number_literal] = ACTIONS(1513),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1513),
  },
  [STATE(172)] = {
    [sym_identifier] = ACTIONS(1547),
    [sym_define_call_keyword] = ACTIONS(1547),
    [sym_define_function_keyword] = ACTIONS(1547),
    [sym_define_library_function_keyword] = ACTIONS(1547),
    [sym_define_module_keyword] = ACTIONS(1547),
    [sym_char_keyword] = ACTIONS(1547),
    [sym_widechar_keyword] = ACTIONS(1547),
    [sym_integer_keyword] = ACTIONS(1547),
    [sym_sinteger_keyword] = ACTIONS(1547),
    [sym_long_keyword] = ACTIONS(1547),
    [sym_slong_keyword] = ACTIONS(1547),
    [sym_float_keyword] = ACTIONS(1547),
    [sym_double_keyword] = ACTIONS(1547),
    [sym_variant_keyword] = ACTIONS(1547),
    [sym_variantarray_keyword] = ACTIONS(1547),
    [sym_local_var_keyword] = ACTIONS(1547),
    [sym_stack_var_keyword] = ACTIONS(1547),
    [sym_constant_keyword] = ACTIONS(1547),
    [sym_volatile_keyword] = ACTIONS(1547),
    [sym_non_volatile_keyword] = ACTIONS(1547),
    [sym_persistent_keyword] = ACTIONS(1547),
    [sym_dev_keyword] = ACTIONS(1547),
    [sym_devlev_keyword] = ACTIONS(1547),
    [sym_devchan_keyword] = ACTIONS(1547),
    [sym_if_keyword] = ACTIONS(1547),
    [sym_else_keyword] = ACTIONS(1547),
    [sym_switch_keyword] = ACTIONS(1547),
    [sym_case_keyword] = ACTIONS(1547),
    [sym_default_keyword] = ACTIONS(1547),
    [sym_while_keyword] = ACTIONS(1547),
    [sym_for_keyword] = ACTIONS(1547),
    [sym_break_keyword] = ACTIONS(1547),
    [sym_continue_keyword] = ACTIONS(1547),
    [sym_return_keyword] = ACTIONS(1547),
    [sym_select_keyword] = ACTIONS(1547),
    [sym_active_keyword] = ACTIONS(1547),
    [sym_struct_keyword] = ACTIONS(1547),
    [sym_structure_keyword] = ACTIONS(1547),
    [aux_sym_on_keyword_token1] = ACTIONS(1547),
    [aux_sym_off_keyword_token1] = ACTIONS(1547),
    [sym_send_level_keyword] = ACTIONS(1547),
    [sym_send_string_keyword] = ACTIONS(1547),
    [sym_send_command_keyword] = ACTIONS(1547),
    [sym_clear_buffer_keyword] = ACTIONS(1547),
    [sym_create_buffer_keyword] = ACTIONS(1547),
    [sym_create_multi_buffer_keyword] = ACTIONS(1547),
    [sym_call_keyword] = ACTIONS(1547),
    [sym_system_call_keyword] = ACTIONS(1547),
    [sym_devchan_to_keyword] = ACTIONS(1547),
    [sym_devchan_min_to_keyword] = ACTIONS(1547),
    [sym_devchan_total_off_keyword] = ACTIONS(1547),
    [sym_devchan_pulse_keyword] = ACTIONS(1547),
    [sym_wait_keyword] = ACTIONS(1547),
    [sym_cancel_wait_keyword] = ACTIONS(1547),
    [sym_wait_until_keyword] = ACTIONS(1547),
    [sym_cancel_wait_until_keyword] = ACTIONS(1547),
    [sym_cancel_all_wait_keyword] = ACTIONS(1547),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1547),
    [sym_and] = ACTIONS(1547),
    [sym_or] = ACTIONS(1547),
    [sym_not] = ACTIONS(1547),
    [sym_xor] = ACTIONS(1547),
    [sym_band] = ACTIONS(1547),
    [sym_bor] = ACTIONS(1547),
    [sym_bxor] = ACTIONS(1547),
    [sym_bnot] = ACTIONS(1547),
    [sym_lshift] = ACTIONS(1547),
    [sym_rshift] = ACTIONS(1547),
    [sym___file__] = ACTIONS(1547),
    [sym___date__] = ACTIONS(1547),
    [sym___ldate__] = ACTIONS(1547),
    [sym___time__] = ACTIONS(1547),
    [sym___line__] = ACTIONS(1547),
    [sym___name__] = ACTIONS(1547),
    [sym___netlinx__] = ACTIONS(1547),
    [sym_netlinx_axi_version] = ACTIONS(1547),
    [sym_url_flg_tcp] = ACTIONS(1547),
    [sym_url_flg_accinfopresent] = ACTIONS(1547),
    [sym_url_flg_temp] = ACTIONS(1547),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1547),
    [sym_url_flg_stat_mask] = ACTIONS(1547),
    [sym_url_flg_stat_lookup] = ACTIONS(1547),
    [sym_url_flg_stat_connecting] = ACTIONS(1547),
    [sym_url_flg_stat_waiting] = ACTIONS(1547),
    [sym_url_flg_stat_connected] = ACTIONS(1547),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1547),
    [sym_first_virtual_device] = ACTIONS(1547),
    [sym_first_local_port] = ACTIONS(1547),
    [sym_dynamic_virtual_device] = ACTIONS(1547),
    [sym_timeline_once] = ACTIONS(1547),
    [sym_timeline_repeat] = ACTIONS(1547),
    [sym_timeline_absolute] = ACTIONS(1547),
    [sym_timeline_relative] = ACTIONS(1547),
    [sym_do_push_timed_infinite] = ACTIONS(1547),
    [sym_source_type_no_address] = ACTIONS(1547),
    [sym_source_type_neuron_id] = ACTIONS(1547),
    [sym_source_type_ip_address] = ACTIONS(1547),
    [sym_source_type_axlink] = ACTIONS(1547),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1547),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1547),
    [sym_source_type_ip_socket_address] = ACTIONS(1547),
    [sym_source_type_rs232] = ACTIONS(1547),
    [sym_source_type_internal] = ACTIONS(1547),
    [sym_source_type_mac_address] = ACTIONS(1547),
    [sym_source_type_ipv4_port] = ACTIONS(1547),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1547),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1547),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1547),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1547),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1547),
    [sym_true] = ACTIONS(1547),
    [sym_false] = ACTIONS(1547),
    [sym_file_read_only] = ACTIONS(1547),
    [sym_file_rw_new] = ACTIONS(1547),
    [sym_file_rw_append] = ACTIONS(1547),
    [sym_ip_tcp] = ACTIONS(1547),
    [sym_ip_udp] = ACTIONS(1547),
    [sym_ip_udp_2way] = ACTIONS(1547),
    [sym_xml_encode_types] = ACTIONS(1547),
    [sym_xml_encode_char_as_list] = ACTIONS(1547),
    [sym_xml_encode_le] = ACTIONS(1547),
    [sym_xml_decode_types] = ACTIONS(1547),
    [sym_xml_decode_no_preserve] = ACTIONS(1547),
    [sym_duet_dev_type_display_device] = ACTIONS(1547),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1547),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1547),
    [sym_duet_dev_type_io_device] = ACTIONS(1547),
    [sym_duet_dev_type_relay_device] = ACTIONS(1547),
    [sym_duet_dev_type_ups] = ACTIONS(1547),
    [sym_duet_dev_type_amplifier] = ACTIONS(1547),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1547),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1547),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1547),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1547),
    [sym_duet_dev_type_camera] = ACTIONS(1547),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1547),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1547),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1547),
    [sym_duet_dev_type_dss] = ACTIONS(1547),
    [sym_duet_dev_type_dvr] = ACTIONS(1547),
    [sym_duet_dev_type_disc_device] = ACTIONS(1547),
    [sym_duet_dev_type_document_camera] = ACTIONS(1547),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1547),
    [sym_duet_dev_type_hvac] = ACTIONS(1547),
    [sym_duet_dev_type_keypad] = ACTIONS(1547),
    [sym_duet_dev_type_light] = ACTIONS(1547),
    [sym_duet_dev_type_monitor] = ACTIONS(1547),
    [sym_duet_dev_type_motor] = ACTIONS(1547),
    [sym_duet_dev_type_multi_window] = ACTIONS(1547),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1547),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1547),
    [sym_duet_dev_type_receiver] = ACTIONS(1547),
    [sym_duet_dev_type_security_system] = ACTIONS(1547),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1547),
    [sym_duet_dev_type_settop_box] = ACTIONS(1547),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1547),
    [sym_duet_dev_type_switcher] = ACTIONS(1547),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1547),
    [sym_duet_dev_type_tv] = ACTIONS(1547),
    [sym_duet_dev_type_utility] = ACTIONS(1547),
    [sym_duet_dev_type_vcr] = ACTIONS(1547),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1547),
    [sym_duet_dev_type_video_processor] = ACTIONS(1547),
    [sym_duet_dev_type_video_projector] = ACTIONS(1547),
    [sym_duet_dev_type_video_wall] = ACTIONS(1547),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1547),
    [sym_duet_dev_type_weather] = ACTIONS(1547),
    [sym_duet_dev_not_polled] = ACTIONS(1547),
    [sym_duet_dev_polled] = ACTIONS(1547),
    [sym_clkmgr_mode_network] = ACTIONS(1547),
    [sym_clkmgr_mode_standalone] = ACTIONS(1547),
    [sym_normal_standby] = ACTIONS(1547),
    [sym_normal_wake] = ACTIONS(1547),
    [sym_ip_multicast_ttl_option] = ACTIONS(1547),
    [sym_ip_ttl_subnet] = ACTIONS(1547),
    [sym_ip_ttl_site] = ACTIONS(1547),
    [sym_ip_ttl_region] = ACTIONS(1547),
    [sym_ip_ttl_continent] = ACTIONS(1547),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1547),
    [sym_ip_nodelay_on] = ACTIONS(1547),
    [sym_ip_nodelay_off] = ACTIONS(1547),
    [sym_amx_error] = ACTIONS(1547),
    [sym_amx_warning] = ACTIONS(1547),
    [sym_amx_info] = ACTIONS(1547),
    [sym_amx_debug] = ACTIONS(1547),
    [sym_netlinx_login_success] = ACTIONS(1547),
    [sym_netlinx_login_fail] = ACTIONS(1547),
    [sym_netlinx_logout] = ACTIONS(1547),
    [sym_null_str] = ACTIONS(1547),
    [sym_smtp_address] = ACTIONS(1547),
    [sym_smtp_port_number] = ACTIONS(1547),
    [sym_smtp_username] = ACTIONS(1547),
    [sym_smtp_password] = ACTIONS(1547),
    [sym_smtp_require_tls] = ACTIONS(1547),
    [sym_smtp_from] = ACTIONS(1547),
    [sym_smtp_tls_true] = ACTIONS(1547),
    [sym_smtp_tls_false] = ACTIONS(1547),
    [sym_listview_on_row_select_event] = ACTIONS(1547),
    [sym_data_structure_unknown] = ACTIONS(1547),
    [sym_data_structure_datafeed] = ACTIONS(1547),
    [sym_data_structure_datarecordset] = ACTIONS(1547),
    [sym_data_structure_datarecord] = ACTIONS(1547),
    [sym_data_structure_datafield] = ACTIONS(1547),
    [sym_data_max_content_field_count] = ACTIONS(1547),
    [sym_data_max_metadata_field_count] = ACTIONS(1547),
    [sym_data_max_id_length] = ACTIONS(1547),
    [sym_data_max_name_length] = ACTIONS(1547),
    [sym_data_max_description_length] = ACTIONS(1547),
    [sym_data_max_type_length] = ACTIONS(1547),
    [sym_data_max_format_length] = ACTIONS(1547),
    [sym_data_max_source_length] = ACTIONS(1547),
    [sym_data_max_value_length] = ACTIONS(1547),
    [sym_data_max_label_length] = ACTIONS(1547),
    [sym_data_max_url_length] = ACTIONS(1547),
    [sym_max_last_login_info_length] = ACTIONS(1547),
    [sym_data_type_unknown] = ACTIONS(1547),
    [sym_data_type_string] = ACTIONS(1547),
    [sym_data_type_datetime] = ACTIONS(1547),
    [sym_data_type_date] = ACTIONS(1547),
    [sym_data_type_time] = ACTIONS(1547),
    [sym_data_type_image] = ACTIONS(1547),
    [sym_data_format_url] = ACTIONS(1547),
    [sym_data_format_phone] = ACTIONS(1547),
    [sym_data_format_email] = ACTIONS(1547),
    [sym_data_format_iso8601] = ACTIONS(1547),
    [sym_auditlog_priv] = ACTIONS(1547),
    [sym_configuration_priv] = ACTIONS(1547),
    [sym_deviceconfig_priv] = ACTIONS(1547),
    [sym_ftp_priv] = ACTIONS(1547),
    [sym_http_priv] = ACTIONS(1547),
    [sym_networkconfig_priv] = ACTIONS(1547),
    [sym_programport_priv] = ACTIONS(1547),
    [sym_remoteui_priv] = ACTIONS(1547),
    [sym_securitycontrol_priv] = ACTIONS(1547),
    [sym_softwaremanagement_priv] = ACTIONS(1547),
    [sym_terminal_priv] = ACTIONS(1547),
    [sym_tpadmin_priv] = ACTIONS(1547),
    [sym_user1_priv] = ACTIONS(1547),
    [sym_user2_priv] = ACTIONS(1547),
    [sym_user3_priv] = ACTIONS(1547),
    [sym_user4_priv] = ACTIONS(1547),
    [sym_usermanagement_priv] = ACTIONS(1547),
    [sym_valid_account] = ACTIONS(1547),
    [sym_err_name_invalid] = ACTIONS(1547),
    [sym_err_password_invalid] = ACTIONS(1547),
    [sym_err_invalid_account] = ACTIONS(1547),
    [sym_err_no_authorization] = ACTIONS(1547),
    [sym_err_invalid_parameter] = ACTIONS(1547),
    [sym_err_user_locked_out] = ACTIONS(1547),
    [sym_err_user_account_expired] = ACTIONS(1547),
    [sym_err_general_error] = ACTIONS(1547),
    [sym_err_authentication_error] = ACTIONS(1547),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1547),
    [sym_tls_validate_certificate] = ACTIONS(1547),
    [sym_tdata] = ACTIONS(1547),
    [sym_tchannel] = ACTIONS(1547),
    [sym_tlevel] = ACTIONS(1547),
    [sym_tbutton] = ACTIONS(1547),
    [sym_ttimeline] = ACTIONS(1547),
    [sym_tcustom] = ACTIONS(1547),
    [sym_url_struct] = ACTIONS(1547),
    [sym_dns_struct] = ACTIONS(1547),
    [sym_ip_address_struct] = ACTIONS(1547),
    [sym_dev_info_struct] = ACTIONS(1547),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1547),
    [sym_data_feed] = ACTIONS(1547),
    [sym_data_field] = ACTIONS(1547),
    [sym_data_record] = ACTIONS(1547),
    [sym_wc_data_feed] = ACTIONS(1547),
    [sym_wc_data_field] = ACTIONS(1547),
    [sym_wc_data_record] = ACTIONS(1547),
    [sym_last_login_info] = ACTIONS(1547),
    [sym____reserved___] = ACTIONS(1547),
    [sym_date] = ACTIONS(1547),
    [sym_day] = ACTIONS(1547),
    [sym_time] = ACTIONS(1547),
    [sym_ldate] = ACTIONS(1547),
    [sym_push_device] = ACTIONS(1547),
    [sym_push_channel] = ACTIONS(1547),
    [sym_push_devchan] = ACTIONS(1547),
    [sym_release_device] = ACTIONS(1547),
    [sym_release_channel] = ACTIONS(1547),
    [sym_release_devchan] = ACTIONS(1547),
    [sym_master_slot] = ACTIONS(1547),
    [sym_get_pulse_time] = ACTIONS(1547),
    [sym_get_timer] = ACTIONS(1547),
    [sym_master_sn] = ACTIONS(1547),
    [sym_system_number] = ACTIONS(1547),
    [sym_button] = ACTIONS(1547),
    [sym_dv_channel] = ACTIONS(1547),
    [sym_data] = ACTIONS(1547),
    [sym_level] = ACTIONS(1547),
    [sym_timeline] = ACTIONS(1547),
    [sym_channel] = ACTIONS(1547),
    [sym_custom] = ACTIONS(1547),
    [sym_atoi] = ACTIONS(1547),
    [sym_atol] = ACTIONS(1547),
    [sym_atof] = ACTIONS(1547),
    [sym_itoa] = ACTIONS(1547),
    [sym_itohex] = ACTIONS(1547),
    [sym_hextoi] = ACTIONS(1547),
    [sym_ftoa] = ACTIONS(1547),
    [sym_find_string] = ACTIONS(1547),
    [sym_left_string] = ACTIONS(1547),
    [sym_right_string] = ACTIONS(1547),
    [sym_mid_string] = ACTIONS(1547),
    [sym_remove_string] = ACTIONS(1547),
    [sym_get_buffer_string] = ACTIONS(1547),
    [sym_get_buffer_char] = ACTIONS(1547),
    [sym_get_multi_buffer_string] = ACTIONS(1547),
    [sym_length_string] = ACTIONS(1547),
    [sym_max_length_string] = ACTIONS(1547),
    [sym_set_length_string] = ACTIONS(1547),
    [sym_compare_string] = ACTIONS(1547),
    [sym_upper_string] = ACTIONS(1547),
    [sym_lower_string] = ACTIONS(1547),
    [sym_get_last] = ACTIONS(1547),
    [sym_date_to_day] = ACTIONS(1547),
    [sym_date_to_month] = ACTIONS(1547),
    [sym_date_to_year] = ACTIONS(1547),
    [sym_time_to_hour] = ACTIONS(1547),
    [sym_time_to_minute] = ACTIONS(1547),
    [sym_time_to_second] = ACTIONS(1547),
    [sym_day_of_week] = ACTIONS(1547),
    [sym_set_pulse_time] = ACTIONS(1547),
    [sym_set_timer] = ACTIONS(1547),
    [sym_file_open] = ACTIONS(1547),
    [sym_file_close] = ACTIONS(1547),
    [sym_file_read] = ACTIONS(1547),
    [sym_file_read_line] = ACTIONS(1547),
    [sym_file_write] = ACTIONS(1547),
    [sym_file_write_line] = ACTIONS(1547),
    [sym_file_copy] = ACTIONS(1547),
    [sym_file_delete] = ACTIONS(1547),
    [sym_file_rename] = ACTIONS(1547),
    [sym_file_dir] = ACTIONS(1547),
    [sym_file_getdir] = ACTIONS(1547),
    [sym_file_setdir] = ACTIONS(1547),
    [sym_file_create_dir] = ACTIONS(1547),
    [sym_file_remove_dir] = ACTIONS(1547),
    [sym_file_seek] = ACTIONS(1547),
    [sym_length_array] = ACTIONS(1547),
    [sym_max_length_array] = ACTIONS(1547),
    [sym_set_length_array] = ACTIONS(1547),
    [sym_ip_client_open] = ACTIONS(1547),
    [sym_ip_client_close] = ACTIONS(1547),
    [sym_ip_server_open] = ACTIONS(1547),
    [sym_ip_server_close] = ACTIONS(1547),
    [sym_ip_bound_client_open] = ACTIONS(1547),
    [sym_ip_mc_server_open] = ACTIONS(1547),
    [sym_ip_set_option] = ACTIONS(1547),
    [sym_ssh_client_open] = ACTIONS(1547),
    [sym_ssh_client_close] = ACTIONS(1547),
    [sym_tls_client_open] = ACTIONS(1547),
    [sym_tls_client_close] = ACTIONS(1547),
    [sym_do_push] = ACTIONS(1547),
    [sym_do_push_timed] = ACTIONS(1547),
    [sym_do_release] = ACTIONS(1547),
    [sym_device_id] = ACTIONS(1547),
    [sym_device_info] = ACTIONS(1547),
    [sym_device_id_string] = ACTIONS(1547),
    [sym_device_standby] = ACTIONS(1547),
    [sym_device_wake] = ACTIONS(1547),
    [sym_timeline_create] = ACTIONS(1547),
    [sym_timeline_kill] = ACTIONS(1547),
    [sym_timeline_reload] = ACTIONS(1547),
    [sym_timeline_pause] = ACTIONS(1547),
    [sym_timeline_restart] = ACTIONS(1547),
    [sym_timeline_set] = ACTIONS(1547),
    [sym_timeline_get] = ACTIONS(1547),
    [sym_timeline_active] = ACTIONS(1547),
    [sym_timeline_dynamic_id] = ACTIONS(1547),
    [sym_variable_to_string] = ACTIONS(1547),
    [sym_string_to_variable] = ACTIONS(1547),
    [sym_length_variable_to_string] = ACTIONS(1547),
    [sym_type_cast] = ACTIONS(1547),
    [sym_variable_to_xml] = ACTIONS(1547),
    [sym_xml_to_variable] = ACTIONS(1547),
    [sym_length_variable_to_xml] = ACTIONS(1547),
    [sym_redirect_string] = ACTIONS(1547),
    [sym_random_number] = ACTIONS(1547),
    [sym_get_unique_id] = ACTIONS(1547),
    [sym_get_system_number] = ACTIONS(1547),
    [sym_set_system_number] = ACTIONS(1547),
    [sym_get_serial_number] = ACTIONS(1547),
    [sym_get_master_build] = ACTIONS(1547),
    [sym_set_validation_code] = ACTIONS(1547),
    [sym_reboot] = ACTIONS(1547),
    [sym_load_duet_module] = ACTIONS(1547),
    [sym_unload_duet_module] = ACTIONS(1547),
    [sym_duet_mem_size_set] = ACTIONS(1547),
    [sym_duet_mem_size_get] = ACTIONS(1547),
    [sym_dynamic_application_device] = ACTIONS(1547),
    [sym_dynamic_polled_port] = ACTIONS(1547),
    [sym_static_port_binding] = ACTIONS(1547),
    [sym_static_ip_binding] = ACTIONS(1547),
    [sym_get_dns_list] = ACTIONS(1547),
    [sym_set_dns_list] = ACTIONS(1547),
    [sym_get_ip_address] = ACTIONS(1547),
    [sym_set_ip_address] = ACTIONS(1547),
    [sym_get_url_list] = ACTIONS(1547),
    [sym_add_url_entry] = ACTIONS(1547),
    [sym_delete_url_entry] = ACTIONS(1547),
    [sym_abs_value] = ACTIONS(1547),
    [sym_max_value] = ACTIONS(1547),
    [sym_min_value] = ACTIONS(1547),
    [sym_exp_value] = ACTIONS(1547),
    [sym_log_value] = ACTIONS(1547),
    [sym_log10_value] = ACTIONS(1547),
    [sym_power_value] = ACTIONS(1547),
    [sym_sqrt_value] = ACTIONS(1547),
    [sym_raw_be] = ACTIONS(1547),
    [sym_raw_le] = ACTIONS(1547),
    [sym_internal_queue_size_set] = ACTIONS(1547),
    [sym_internal_queue_size_get] = ACTIONS(1547),
    [sym_internal_threshold_set] = ACTIONS(1547),
    [sym_internal_threshold_get] = ACTIONS(1547),
    [sym_rebuilt_event] = ACTIONS(1547),
    [sym_do_custom_event] = ACTIONS(1547),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1547),
    [sym_clkmgr_set_clk_source] = ACTIONS(1547),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1547),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1547),
    [sym_clkmgr_get_timezone] = ACTIONS(1547),
    [sym_clkmgr_set_timezone] = ACTIONS(1547),
    [sym_clkmgr_get_resync_period] = ACTIONS(1547),
    [sym_clkmgr_set_resync_period] = ACTIONS(1547),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1547),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1547),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1547),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1547),
    [sym_clkmgr_get_timeservers] = ACTIONS(1547),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1547),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1547),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1547),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1547),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1547),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1547),
    [sym_set_log_level] = ACTIONS(1547),
    [sym_get_log_level] = ACTIONS(1547),
    [sym_amx_log] = ACTIONS(1547),
    [sym_smtp_server_config_set] = ACTIONS(1547),
    [sym_smtp_server_config_get] = ACTIONS(1547),
    [sym_smtp_send] = ACTIONS(1547),
    [sym_validate_netlinx_account] = ACTIONS(1547),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1547),
    [sym_authenticate_certificate] = ACTIONS(1547),
    [sym_audit_netlinx_session_event] = ACTIONS(1547),
    [sym_audit_netlinx_generic_event] = ACTIONS(1547),
    [sym_get_available_flash_disk_space] = ACTIONS(1547),
    [sym_get_max_flash_disk_space] = ACTIONS(1547),
    [sym_data_create_feed] = ACTIONS(1547),
    [sym_data_delete_feed] = ACTIONS(1547),
    [sym_data_publish_feed] = ACTIONS(1547),
    [sym_data_get_published_feed] = ACTIONS(1547),
    [sym_data_add_record] = ACTIONS(1547),
    [sym_data_get_event_record] = ACTIONS(1547),
    [sym__wc_data_create_feed] = ACTIONS(1547),
    [sym__wc_data_add_record] = ACTIONS(1547),
    [sym__wc_data_get_event_record] = ACTIONS(1547),
    [sym_format] = ACTIONS(1547),
    [sym_astro_clock] = ACTIONS(1547),
    [sym_set_outdoor_temperature] = ACTIONS(1547),
    [sym_set_virtual_level_count] = ACTIONS(1547),
    [sym_set_virtual_channel_count] = ACTIONS(1547),
    [sym_set_virtual_port_count] = ACTIONS(1547),
    [sym__wc_encode_int] = ACTIONS(1547),
    [sym__wc_decode_int] = ACTIONS(1547),
    [sym__wc_int] = ACTIONS(1547),
    [sym__wc_to_ch_int] = ACTIONS(1547),
    [sym__ch_to_wc_int] = ACTIONS(1547),
    [sym__wc_find_string_int] = ACTIONS(1547),
    [sym__wc_left_string_int] = ACTIONS(1547),
    [sym__wc_lower_string_int] = ACTIONS(1547),
    [sym__wc_mid_string_int] = ACTIONS(1547),
    [sym__wc_remove_string_int] = ACTIONS(1547),
    [sym__wc_right_string_int] = ACTIONS(1547),
    [sym__wc_upper_string_int] = ACTIONS(1547),
    [sym__wc_compare_string_int] = ACTIONS(1547),
    [sym__wc_get_buffer_char_int] = ACTIONS(1547),
    [sym__wc_get_buffer_string_int] = ACTIONS(1547),
    [sym__wc_concat_string_int] = ACTIONS(1547),
    [sym__wc_file_open_int] = ACTIONS(1547),
    [sym__wc_file_close_int] = ACTIONS(1547),
    [sym__wc_file_read_int] = ACTIONS(1547),
    [sym__wc_file_read_line_int] = ACTIONS(1547),
    [sym__wc_file_write_int] = ACTIONS(1547),
    [sym__wc_file_write_line_int] = ACTIONS(1547),
    [anon_sym_EQ] = ACTIONS(1547),
    [aux_sym_preproc_include_token1] = ACTIONS(1547),
    [aux_sym_preproc_define_token1] = ACTIONS(1549),
    [aux_sym_preproc_warn_token1] = ACTIONS(1549),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1549),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1549),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1549),
    [anon_sym_LBRACK] = ACTIONS(1549),
    [anon_sym_LBRACE] = ACTIONS(1549),
    [anon_sym_RBRACE] = ACTIONS(1549),
    [anon_sym_COLON] = ACTIONS(1549),
    [anon_sym_LPAREN] = ACTIONS(1547),
    [anon_sym_SEMI] = ACTIONS(1549),
    [anon_sym_BANG] = ACTIONS(1547),
    [anon_sym_TILDE] = ACTIONS(1549),
    [anon_sym_DASH] = ACTIONS(1547),
    [anon_sym_PLUS] = ACTIONS(1547),
    [anon_sym_STAR] = ACTIONS(1549),
    [anon_sym_SLASH] = ACTIONS(1547),
    [anon_sym_PERCENT] = ACTIONS(1549),
    [anon_sym_PIPE_PIPE] = ACTIONS(1549),
    [anon_sym_AMP_AMP] = ACTIONS(1549),
    [anon_sym_PIPE] = ACTIONS(1547),
    [anon_sym_CARET] = ACTIONS(1549),
    [anon_sym_AMP] = ACTIONS(1547),
    [anon_sym_EQ_EQ] = ACTIONS(1549),
    [anon_sym_BANG_EQ] = ACTIONS(1549),
    [anon_sym_LT_GT] = ACTIONS(1549),
    [anon_sym_GT] = ACTIONS(1547),
    [anon_sym_GT_EQ] = ACTIONS(1549),
    [anon_sym_LT_EQ] = ACTIONS(1549),
    [anon_sym_LT] = ACTIONS(1547),
    [anon_sym_LT_LT] = ACTIONS(1549),
    [anon_sym_GT_GT] = ACTIONS(1549),
    [anon_sym_DASH_DASH] = ACTIONS(1549),
    [anon_sym_PLUS_PLUS] = ACTIONS(1549),
    [anon_sym_DOT] = ACTIONS(1547),
    [anon_sym_DQUOTE] = ACTIONS(1549),
    [anon_sym_SQUOTE] = ACTIONS(1549),
    [sym_number_literal] = ACTIONS(1549),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1549),
  },
  [STATE(173)] = {
    [sym_identifier] = ACTIONS(1539),
    [sym_define_call_keyword] = ACTIONS(1539),
    [sym_define_function_keyword] = ACTIONS(1539),
    [sym_define_library_function_keyword] = ACTIONS(1539),
    [sym_define_module_keyword] = ACTIONS(1539),
    [sym_char_keyword] = ACTIONS(1539),
    [sym_widechar_keyword] = ACTIONS(1539),
    [sym_integer_keyword] = ACTIONS(1539),
    [sym_sinteger_keyword] = ACTIONS(1539),
    [sym_long_keyword] = ACTIONS(1539),
    [sym_slong_keyword] = ACTIONS(1539),
    [sym_float_keyword] = ACTIONS(1539),
    [sym_double_keyword] = ACTIONS(1539),
    [sym_variant_keyword] = ACTIONS(1539),
    [sym_variantarray_keyword] = ACTIONS(1539),
    [sym_local_var_keyword] = ACTIONS(1539),
    [sym_stack_var_keyword] = ACTIONS(1539),
    [sym_constant_keyword] = ACTIONS(1539),
    [sym_volatile_keyword] = ACTIONS(1539),
    [sym_non_volatile_keyword] = ACTIONS(1539),
    [sym_persistent_keyword] = ACTIONS(1539),
    [sym_dev_keyword] = ACTIONS(1539),
    [sym_devlev_keyword] = ACTIONS(1539),
    [sym_devchan_keyword] = ACTIONS(1539),
    [sym_if_keyword] = ACTIONS(1539),
    [sym_else_keyword] = ACTIONS(1539),
    [sym_switch_keyword] = ACTIONS(1539),
    [sym_case_keyword] = ACTIONS(1539),
    [sym_default_keyword] = ACTIONS(1539),
    [sym_while_keyword] = ACTIONS(1539),
    [sym_for_keyword] = ACTIONS(1539),
    [sym_break_keyword] = ACTIONS(1539),
    [sym_continue_keyword] = ACTIONS(1539),
    [sym_return_keyword] = ACTIONS(1539),
    [sym_select_keyword] = ACTIONS(1539),
    [sym_active_keyword] = ACTIONS(1539),
    [sym_struct_keyword] = ACTIONS(1539),
    [sym_structure_keyword] = ACTIONS(1539),
    [aux_sym_on_keyword_token1] = ACTIONS(1539),
    [aux_sym_off_keyword_token1] = ACTIONS(1539),
    [sym_send_level_keyword] = ACTIONS(1539),
    [sym_send_string_keyword] = ACTIONS(1539),
    [sym_send_command_keyword] = ACTIONS(1539),
    [sym_clear_buffer_keyword] = ACTIONS(1539),
    [sym_create_buffer_keyword] = ACTIONS(1539),
    [sym_create_multi_buffer_keyword] = ACTIONS(1539),
    [sym_call_keyword] = ACTIONS(1539),
    [sym_system_call_keyword] = ACTIONS(1539),
    [sym_devchan_to_keyword] = ACTIONS(1539),
    [sym_devchan_min_to_keyword] = ACTIONS(1539),
    [sym_devchan_total_off_keyword] = ACTIONS(1539),
    [sym_devchan_pulse_keyword] = ACTIONS(1539),
    [sym_wait_keyword] = ACTIONS(1539),
    [sym_cancel_wait_keyword] = ACTIONS(1539),
    [sym_wait_until_keyword] = ACTIONS(1539),
    [sym_cancel_wait_until_keyword] = ACTIONS(1539),
    [sym_cancel_all_wait_keyword] = ACTIONS(1539),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1539),
    [sym_and] = ACTIONS(1539),
    [sym_or] = ACTIONS(1539),
    [sym_not] = ACTIONS(1539),
    [sym_xor] = ACTIONS(1539),
    [sym_band] = ACTIONS(1539),
    [sym_bor] = ACTIONS(1539),
    [sym_bxor] = ACTIONS(1539),
    [sym_bnot] = ACTIONS(1539),
    [sym_lshift] = ACTIONS(1539),
    [sym_rshift] = ACTIONS(1539),
    [sym___file__] = ACTIONS(1539),
    [sym___date__] = ACTIONS(1539),
    [sym___ldate__] = ACTIONS(1539),
    [sym___time__] = ACTIONS(1539),
    [sym___line__] = ACTIONS(1539),
    [sym___name__] = ACTIONS(1539),
    [sym___netlinx__] = ACTIONS(1539),
    [sym_netlinx_axi_version] = ACTIONS(1539),
    [sym_url_flg_tcp] = ACTIONS(1539),
    [sym_url_flg_accinfopresent] = ACTIONS(1539),
    [sym_url_flg_temp] = ACTIONS(1539),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1539),
    [sym_url_flg_stat_mask] = ACTIONS(1539),
    [sym_url_flg_stat_lookup] = ACTIONS(1539),
    [sym_url_flg_stat_connecting] = ACTIONS(1539),
    [sym_url_flg_stat_waiting] = ACTIONS(1539),
    [sym_url_flg_stat_connected] = ACTIONS(1539),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1539),
    [sym_first_virtual_device] = ACTIONS(1539),
    [sym_first_local_port] = ACTIONS(1539),
    [sym_dynamic_virtual_device] = ACTIONS(1539),
    [sym_timeline_once] = ACTIONS(1539),
    [sym_timeline_repeat] = ACTIONS(1539),
    [sym_timeline_absolute] = ACTIONS(1539),
    [sym_timeline_relative] = ACTIONS(1539),
    [sym_do_push_timed_infinite] = ACTIONS(1539),
    [sym_source_type_no_address] = ACTIONS(1539),
    [sym_source_type_neuron_id] = ACTIONS(1539),
    [sym_source_type_ip_address] = ACTIONS(1539),
    [sym_source_type_axlink] = ACTIONS(1539),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1539),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1539),
    [sym_source_type_ip_socket_address] = ACTIONS(1539),
    [sym_source_type_rs232] = ACTIONS(1539),
    [sym_source_type_internal] = ACTIONS(1539),
    [sym_source_type_mac_address] = ACTIONS(1539),
    [sym_source_type_ipv4_port] = ACTIONS(1539),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1539),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1539),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1539),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1539),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1539),
    [sym_true] = ACTIONS(1539),
    [sym_false] = ACTIONS(1539),
    [sym_file_read_only] = ACTIONS(1539),
    [sym_file_rw_new] = ACTIONS(1539),
    [sym_file_rw_append] = ACTIONS(1539),
    [sym_ip_tcp] = ACTIONS(1539),
    [sym_ip_udp] = ACTIONS(1539),
    [sym_ip_udp_2way] = ACTIONS(1539),
    [sym_xml_encode_types] = ACTIONS(1539),
    [sym_xml_encode_char_as_list] = ACTIONS(1539),
    [sym_xml_encode_le] = ACTIONS(1539),
    [sym_xml_decode_types] = ACTIONS(1539),
    [sym_xml_decode_no_preserve] = ACTIONS(1539),
    [sym_duet_dev_type_display_device] = ACTIONS(1539),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1539),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1539),
    [sym_duet_dev_type_io_device] = ACTIONS(1539),
    [sym_duet_dev_type_relay_device] = ACTIONS(1539),
    [sym_duet_dev_type_ups] = ACTIONS(1539),
    [sym_duet_dev_type_amplifier] = ACTIONS(1539),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1539),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1539),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1539),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1539),
    [sym_duet_dev_type_camera] = ACTIONS(1539),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1539),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1539),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1539),
    [sym_duet_dev_type_dss] = ACTIONS(1539),
    [sym_duet_dev_type_dvr] = ACTIONS(1539),
    [sym_duet_dev_type_disc_device] = ACTIONS(1539),
    [sym_duet_dev_type_document_camera] = ACTIONS(1539),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1539),
    [sym_duet_dev_type_hvac] = ACTIONS(1539),
    [sym_duet_dev_type_keypad] = ACTIONS(1539),
    [sym_duet_dev_type_light] = ACTIONS(1539),
    [sym_duet_dev_type_monitor] = ACTIONS(1539),
    [sym_duet_dev_type_motor] = ACTIONS(1539),
    [sym_duet_dev_type_multi_window] = ACTIONS(1539),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1539),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1539),
    [sym_duet_dev_type_receiver] = ACTIONS(1539),
    [sym_duet_dev_type_security_system] = ACTIONS(1539),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1539),
    [sym_duet_dev_type_settop_box] = ACTIONS(1539),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1539),
    [sym_duet_dev_type_switcher] = ACTIONS(1539),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1539),
    [sym_duet_dev_type_tv] = ACTIONS(1539),
    [sym_duet_dev_type_utility] = ACTIONS(1539),
    [sym_duet_dev_type_vcr] = ACTIONS(1539),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1539),
    [sym_duet_dev_type_video_processor] = ACTIONS(1539),
    [sym_duet_dev_type_video_projector] = ACTIONS(1539),
    [sym_duet_dev_type_video_wall] = ACTIONS(1539),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1539),
    [sym_duet_dev_type_weather] = ACTIONS(1539),
    [sym_duet_dev_not_polled] = ACTIONS(1539),
    [sym_duet_dev_polled] = ACTIONS(1539),
    [sym_clkmgr_mode_network] = ACTIONS(1539),
    [sym_clkmgr_mode_standalone] = ACTIONS(1539),
    [sym_normal_standby] = ACTIONS(1539),
    [sym_normal_wake] = ACTIONS(1539),
    [sym_ip_multicast_ttl_option] = ACTIONS(1539),
    [sym_ip_ttl_subnet] = ACTIONS(1539),
    [sym_ip_ttl_site] = ACTIONS(1539),
    [sym_ip_ttl_region] = ACTIONS(1539),
    [sym_ip_ttl_continent] = ACTIONS(1539),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1539),
    [sym_ip_nodelay_on] = ACTIONS(1539),
    [sym_ip_nodelay_off] = ACTIONS(1539),
    [sym_amx_error] = ACTIONS(1539),
    [sym_amx_warning] = ACTIONS(1539),
    [sym_amx_info] = ACTIONS(1539),
    [sym_amx_debug] = ACTIONS(1539),
    [sym_netlinx_login_success] = ACTIONS(1539),
    [sym_netlinx_login_fail] = ACTIONS(1539),
    [sym_netlinx_logout] = ACTIONS(1539),
    [sym_null_str] = ACTIONS(1539),
    [sym_smtp_address] = ACTIONS(1539),
    [sym_smtp_port_number] = ACTIONS(1539),
    [sym_smtp_username] = ACTIONS(1539),
    [sym_smtp_password] = ACTIONS(1539),
    [sym_smtp_require_tls] = ACTIONS(1539),
    [sym_smtp_from] = ACTIONS(1539),
    [sym_smtp_tls_true] = ACTIONS(1539),
    [sym_smtp_tls_false] = ACTIONS(1539),
    [sym_listview_on_row_select_event] = ACTIONS(1539),
    [sym_data_structure_unknown] = ACTIONS(1539),
    [sym_data_structure_datafeed] = ACTIONS(1539),
    [sym_data_structure_datarecordset] = ACTIONS(1539),
    [sym_data_structure_datarecord] = ACTIONS(1539),
    [sym_data_structure_datafield] = ACTIONS(1539),
    [sym_data_max_content_field_count] = ACTIONS(1539),
    [sym_data_max_metadata_field_count] = ACTIONS(1539),
    [sym_data_max_id_length] = ACTIONS(1539),
    [sym_data_max_name_length] = ACTIONS(1539),
    [sym_data_max_description_length] = ACTIONS(1539),
    [sym_data_max_type_length] = ACTIONS(1539),
    [sym_data_max_format_length] = ACTIONS(1539),
    [sym_data_max_source_length] = ACTIONS(1539),
    [sym_data_max_value_length] = ACTIONS(1539),
    [sym_data_max_label_length] = ACTIONS(1539),
    [sym_data_max_url_length] = ACTIONS(1539),
    [sym_max_last_login_info_length] = ACTIONS(1539),
    [sym_data_type_unknown] = ACTIONS(1539),
    [sym_data_type_string] = ACTIONS(1539),
    [sym_data_type_datetime] = ACTIONS(1539),
    [sym_data_type_date] = ACTIONS(1539),
    [sym_data_type_time] = ACTIONS(1539),
    [sym_data_type_image] = ACTIONS(1539),
    [sym_data_format_url] = ACTIONS(1539),
    [sym_data_format_phone] = ACTIONS(1539),
    [sym_data_format_email] = ACTIONS(1539),
    [sym_data_format_iso8601] = ACTIONS(1539),
    [sym_auditlog_priv] = ACTIONS(1539),
    [sym_configuration_priv] = ACTIONS(1539),
    [sym_deviceconfig_priv] = ACTIONS(1539),
    [sym_ftp_priv] = ACTIONS(1539),
    [sym_http_priv] = ACTIONS(1539),
    [sym_networkconfig_priv] = ACTIONS(1539),
    [sym_programport_priv] = ACTIONS(1539),
    [sym_remoteui_priv] = ACTIONS(1539),
    [sym_securitycontrol_priv] = ACTIONS(1539),
    [sym_softwaremanagement_priv] = ACTIONS(1539),
    [sym_terminal_priv] = ACTIONS(1539),
    [sym_tpadmin_priv] = ACTIONS(1539),
    [sym_user1_priv] = ACTIONS(1539),
    [sym_user2_priv] = ACTIONS(1539),
    [sym_user3_priv] = ACTIONS(1539),
    [sym_user4_priv] = ACTIONS(1539),
    [sym_usermanagement_priv] = ACTIONS(1539),
    [sym_valid_account] = ACTIONS(1539),
    [sym_err_name_invalid] = ACTIONS(1539),
    [sym_err_password_invalid] = ACTIONS(1539),
    [sym_err_invalid_account] = ACTIONS(1539),
    [sym_err_no_authorization] = ACTIONS(1539),
    [sym_err_invalid_parameter] = ACTIONS(1539),
    [sym_err_user_locked_out] = ACTIONS(1539),
    [sym_err_user_account_expired] = ACTIONS(1539),
    [sym_err_general_error] = ACTIONS(1539),
    [sym_err_authentication_error] = ACTIONS(1539),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1539),
    [sym_tls_validate_certificate] = ACTIONS(1539),
    [sym_tdata] = ACTIONS(1539),
    [sym_tchannel] = ACTIONS(1539),
    [sym_tlevel] = ACTIONS(1539),
    [sym_tbutton] = ACTIONS(1539),
    [sym_ttimeline] = ACTIONS(1539),
    [sym_tcustom] = ACTIONS(1539),
    [sym_url_struct] = ACTIONS(1539),
    [sym_dns_struct] = ACTIONS(1539),
    [sym_ip_address_struct] = ACTIONS(1539),
    [sym_dev_info_struct] = ACTIONS(1539),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1539),
    [sym_data_feed] = ACTIONS(1539),
    [sym_data_field] = ACTIONS(1539),
    [sym_data_record] = ACTIONS(1539),
    [sym_wc_data_feed] = ACTIONS(1539),
    [sym_wc_data_field] = ACTIONS(1539),
    [sym_wc_data_record] = ACTIONS(1539),
    [sym_last_login_info] = ACTIONS(1539),
    [sym____reserved___] = ACTIONS(1539),
    [sym_date] = ACTIONS(1539),
    [sym_day] = ACTIONS(1539),
    [sym_time] = ACTIONS(1539),
    [sym_ldate] = ACTIONS(1539),
    [sym_push_device] = ACTIONS(1539),
    [sym_push_channel] = ACTIONS(1539),
    [sym_push_devchan] = ACTIONS(1539),
    [sym_release_device] = ACTIONS(1539),
    [sym_release_channel] = ACTIONS(1539),
    [sym_release_devchan] = ACTIONS(1539),
    [sym_master_slot] = ACTIONS(1539),
    [sym_get_pulse_time] = ACTIONS(1539),
    [sym_get_timer] = ACTIONS(1539),
    [sym_master_sn] = ACTIONS(1539),
    [sym_system_number] = ACTIONS(1539),
    [sym_button] = ACTIONS(1539),
    [sym_dv_channel] = ACTIONS(1539),
    [sym_data] = ACTIONS(1539),
    [sym_level] = ACTIONS(1539),
    [sym_timeline] = ACTIONS(1539),
    [sym_channel] = ACTIONS(1539),
    [sym_custom] = ACTIONS(1539),
    [sym_atoi] = ACTIONS(1539),
    [sym_atol] = ACTIONS(1539),
    [sym_atof] = ACTIONS(1539),
    [sym_itoa] = ACTIONS(1539),
    [sym_itohex] = ACTIONS(1539),
    [sym_hextoi] = ACTIONS(1539),
    [sym_ftoa] = ACTIONS(1539),
    [sym_find_string] = ACTIONS(1539),
    [sym_left_string] = ACTIONS(1539),
    [sym_right_string] = ACTIONS(1539),
    [sym_mid_string] = ACTIONS(1539),
    [sym_remove_string] = ACTIONS(1539),
    [sym_get_buffer_string] = ACTIONS(1539),
    [sym_get_buffer_char] = ACTIONS(1539),
    [sym_get_multi_buffer_string] = ACTIONS(1539),
    [sym_length_string] = ACTIONS(1539),
    [sym_max_length_string] = ACTIONS(1539),
    [sym_set_length_string] = ACTIONS(1539),
    [sym_compare_string] = ACTIONS(1539),
    [sym_upper_string] = ACTIONS(1539),
    [sym_lower_string] = ACTIONS(1539),
    [sym_get_last] = ACTIONS(1539),
    [sym_date_to_day] = ACTIONS(1539),
    [sym_date_to_month] = ACTIONS(1539),
    [sym_date_to_year] = ACTIONS(1539),
    [sym_time_to_hour] = ACTIONS(1539),
    [sym_time_to_minute] = ACTIONS(1539),
    [sym_time_to_second] = ACTIONS(1539),
    [sym_day_of_week] = ACTIONS(1539),
    [sym_set_pulse_time] = ACTIONS(1539),
    [sym_set_timer] = ACTIONS(1539),
    [sym_file_open] = ACTIONS(1539),
    [sym_file_close] = ACTIONS(1539),
    [sym_file_read] = ACTIONS(1539),
    [sym_file_read_line] = ACTIONS(1539),
    [sym_file_write] = ACTIONS(1539),
    [sym_file_write_line] = ACTIONS(1539),
    [sym_file_copy] = ACTIONS(1539),
    [sym_file_delete] = ACTIONS(1539),
    [sym_file_rename] = ACTIONS(1539),
    [sym_file_dir] = ACTIONS(1539),
    [sym_file_getdir] = ACTIONS(1539),
    [sym_file_setdir] = ACTIONS(1539),
    [sym_file_create_dir] = ACTIONS(1539),
    [sym_file_remove_dir] = ACTIONS(1539),
    [sym_file_seek] = ACTIONS(1539),
    [sym_length_array] = ACTIONS(1539),
    [sym_max_length_array] = ACTIONS(1539),
    [sym_set_length_array] = ACTIONS(1539),
    [sym_ip_client_open] = ACTIONS(1539),
    [sym_ip_client_close] = ACTIONS(1539),
    [sym_ip_server_open] = ACTIONS(1539),
    [sym_ip_server_close] = ACTIONS(1539),
    [sym_ip_bound_client_open] = ACTIONS(1539),
    [sym_ip_mc_server_open] = ACTIONS(1539),
    [sym_ip_set_option] = ACTIONS(1539),
    [sym_ssh_client_open] = ACTIONS(1539),
    [sym_ssh_client_close] = ACTIONS(1539),
    [sym_tls_client_open] = ACTIONS(1539),
    [sym_tls_client_close] = ACTIONS(1539),
    [sym_do_push] = ACTIONS(1539),
    [sym_do_push_timed] = ACTIONS(1539),
    [sym_do_release] = ACTIONS(1539),
    [sym_device_id] = ACTIONS(1539),
    [sym_device_info] = ACTIONS(1539),
    [sym_device_id_string] = ACTIONS(1539),
    [sym_device_standby] = ACTIONS(1539),
    [sym_device_wake] = ACTIONS(1539),
    [sym_timeline_create] = ACTIONS(1539),
    [sym_timeline_kill] = ACTIONS(1539),
    [sym_timeline_reload] = ACTIONS(1539),
    [sym_timeline_pause] = ACTIONS(1539),
    [sym_timeline_restart] = ACTIONS(1539),
    [sym_timeline_set] = ACTIONS(1539),
    [sym_timeline_get] = ACTIONS(1539),
    [sym_timeline_active] = ACTIONS(1539),
    [sym_timeline_dynamic_id] = ACTIONS(1539),
    [sym_variable_to_string] = ACTIONS(1539),
    [sym_string_to_variable] = ACTIONS(1539),
    [sym_length_variable_to_string] = ACTIONS(1539),
    [sym_type_cast] = ACTIONS(1539),
    [sym_variable_to_xml] = ACTIONS(1539),
    [sym_xml_to_variable] = ACTIONS(1539),
    [sym_length_variable_to_xml] = ACTIONS(1539),
    [sym_redirect_string] = ACTIONS(1539),
    [sym_random_number] = ACTIONS(1539),
    [sym_get_unique_id] = ACTIONS(1539),
    [sym_get_system_number] = ACTIONS(1539),
    [sym_set_system_number] = ACTIONS(1539),
    [sym_get_serial_number] = ACTIONS(1539),
    [sym_get_master_build] = ACTIONS(1539),
    [sym_set_validation_code] = ACTIONS(1539),
    [sym_reboot] = ACTIONS(1539),
    [sym_load_duet_module] = ACTIONS(1539),
    [sym_unload_duet_module] = ACTIONS(1539),
    [sym_duet_mem_size_set] = ACTIONS(1539),
    [sym_duet_mem_size_get] = ACTIONS(1539),
    [sym_dynamic_application_device] = ACTIONS(1539),
    [sym_dynamic_polled_port] = ACTIONS(1539),
    [sym_static_port_binding] = ACTIONS(1539),
    [sym_static_ip_binding] = ACTIONS(1539),
    [sym_get_dns_list] = ACTIONS(1539),
    [sym_set_dns_list] = ACTIONS(1539),
    [sym_get_ip_address] = ACTIONS(1539),
    [sym_set_ip_address] = ACTIONS(1539),
    [sym_get_url_list] = ACTIONS(1539),
    [sym_add_url_entry] = ACTIONS(1539),
    [sym_delete_url_entry] = ACTIONS(1539),
    [sym_abs_value] = ACTIONS(1539),
    [sym_max_value] = ACTIONS(1539),
    [sym_min_value] = ACTIONS(1539),
    [sym_exp_value] = ACTIONS(1539),
    [sym_log_value] = ACTIONS(1539),
    [sym_log10_value] = ACTIONS(1539),
    [sym_power_value] = ACTIONS(1539),
    [sym_sqrt_value] = ACTIONS(1539),
    [sym_raw_be] = ACTIONS(1539),
    [sym_raw_le] = ACTIONS(1539),
    [sym_internal_queue_size_set] = ACTIONS(1539),
    [sym_internal_queue_size_get] = ACTIONS(1539),
    [sym_internal_threshold_set] = ACTIONS(1539),
    [sym_internal_threshold_get] = ACTIONS(1539),
    [sym_rebuilt_event] = ACTIONS(1539),
    [sym_do_custom_event] = ACTIONS(1539),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1539),
    [sym_clkmgr_set_clk_source] = ACTIONS(1539),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1539),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1539),
    [sym_clkmgr_get_timezone] = ACTIONS(1539),
    [sym_clkmgr_set_timezone] = ACTIONS(1539),
    [sym_clkmgr_get_resync_period] = ACTIONS(1539),
    [sym_clkmgr_set_resync_period] = ACTIONS(1539),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1539),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1539),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1539),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1539),
    [sym_clkmgr_get_timeservers] = ACTIONS(1539),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1539),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1539),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1539),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1539),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1539),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1539),
    [sym_set_log_level] = ACTIONS(1539),
    [sym_get_log_level] = ACTIONS(1539),
    [sym_amx_log] = ACTIONS(1539),
    [sym_smtp_server_config_set] = ACTIONS(1539),
    [sym_smtp_server_config_get] = ACTIONS(1539),
    [sym_smtp_send] = ACTIONS(1539),
    [sym_validate_netlinx_account] = ACTIONS(1539),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1539),
    [sym_authenticate_certificate] = ACTIONS(1539),
    [sym_audit_netlinx_session_event] = ACTIONS(1539),
    [sym_audit_netlinx_generic_event] = ACTIONS(1539),
    [sym_get_available_flash_disk_space] = ACTIONS(1539),
    [sym_get_max_flash_disk_space] = ACTIONS(1539),
    [sym_data_create_feed] = ACTIONS(1539),
    [sym_data_delete_feed] = ACTIONS(1539),
    [sym_data_publish_feed] = ACTIONS(1539),
    [sym_data_get_published_feed] = ACTIONS(1539),
    [sym_data_add_record] = ACTIONS(1539),
    [sym_data_get_event_record] = ACTIONS(1539),
    [sym__wc_data_create_feed] = ACTIONS(1539),
    [sym__wc_data_add_record] = ACTIONS(1539),
    [sym__wc_data_get_event_record] = ACTIONS(1539),
    [sym_format] = ACTIONS(1539),
    [sym_astro_clock] = ACTIONS(1539),
    [sym_set_outdoor_temperature] = ACTIONS(1539),
    [sym_set_virtual_level_count] = ACTIONS(1539),
    [sym_set_virtual_channel_count] = ACTIONS(1539),
    [sym_set_virtual_port_count] = ACTIONS(1539),
    [sym__wc_encode_int] = ACTIONS(1539),
    [sym__wc_decode_int] = ACTIONS(1539),
    [sym__wc_int] = ACTIONS(1539),
    [sym__wc_to_ch_int] = ACTIONS(1539),
    [sym__ch_to_wc_int] = ACTIONS(1539),
    [sym__wc_find_string_int] = ACTIONS(1539),
    [sym__wc_left_string_int] = ACTIONS(1539),
    [sym__wc_lower_string_int] = ACTIONS(1539),
    [sym__wc_mid_string_int] = ACTIONS(1539),
    [sym__wc_remove_string_int] = ACTIONS(1539),
    [sym__wc_right_string_int] = ACTIONS(1539),
    [sym__wc_upper_string_int] = ACTIONS(1539),
    [sym__wc_compare_string_int] = ACTIONS(1539),
    [sym__wc_get_buffer_char_int] = ACTIONS(1539),
    [sym__wc_get_buffer_string_int] = ACTIONS(1539),
    [sym__wc_concat_string_int] = ACTIONS(1539),
    [sym__wc_file_open_int] = ACTIONS(1539),
    [sym__wc_file_close_int] = ACTIONS(1539),
    [sym__wc_file_read_int] = ACTIONS(1539),
    [sym__wc_file_read_line_int] = ACTIONS(1539),
    [sym__wc_file_write_int] = ACTIONS(1539),
    [sym__wc_file_write_line_int] = ACTIONS(1539),
    [anon_sym_EQ] = ACTIONS(1539),
    [aux_sym_preproc_include_token1] = ACTIONS(1539),
    [aux_sym_preproc_define_token1] = ACTIONS(1541),
    [aux_sym_preproc_warn_token1] = ACTIONS(1541),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1541),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1541),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1541),
    [anon_sym_LBRACK] = ACTIONS(1541),
    [anon_sym_LBRACE] = ACTIONS(1541),
    [anon_sym_RBRACE] = ACTIONS(1541),
    [anon_sym_COLON] = ACTIONS(1541),
    [anon_sym_LPAREN] = ACTIONS(1539),
    [anon_sym_SEMI] = ACTIONS(1541),
    [anon_sym_BANG] = ACTIONS(1539),
    [anon_sym_TILDE] = ACTIONS(1541),
    [anon_sym_DASH] = ACTIONS(1539),
    [anon_sym_PLUS] = ACTIONS(1539),
    [anon_sym_STAR] = ACTIONS(1541),
    [anon_sym_SLASH] = ACTIONS(1539),
    [anon_sym_PERCENT] = ACTIONS(1541),
    [anon_sym_PIPE_PIPE] = ACTIONS(1541),
    [anon_sym_AMP_AMP] = ACTIONS(1541),
    [anon_sym_PIPE] = ACTIONS(1539),
    [anon_sym_CARET] = ACTIONS(1541),
    [anon_sym_AMP] = ACTIONS(1539),
    [anon_sym_EQ_EQ] = ACTIONS(1541),
    [anon_sym_BANG_EQ] = ACTIONS(1541),
    [anon_sym_LT_GT] = ACTIONS(1541),
    [anon_sym_GT] = ACTIONS(1539),
    [anon_sym_GT_EQ] = ACTIONS(1541),
    [anon_sym_LT_EQ] = ACTIONS(1541),
    [anon_sym_LT] = ACTIONS(1539),
    [anon_sym_LT_LT] = ACTIONS(1541),
    [anon_sym_GT_GT] = ACTIONS(1541),
    [anon_sym_DASH_DASH] = ACTIONS(1541),
    [anon_sym_PLUS_PLUS] = ACTIONS(1541),
    [anon_sym_DOT] = ACTIONS(1539),
    [anon_sym_DQUOTE] = ACTIONS(1541),
    [anon_sym_SQUOTE] = ACTIONS(1541),
    [sym_number_literal] = ACTIONS(1541),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1541),
  },
  [STATE(174)] = {
    [sym_identifier] = ACTIONS(1521),
    [sym_define_call_keyword] = ACTIONS(1521),
    [sym_define_function_keyword] = ACTIONS(1521),
    [sym_define_library_function_keyword] = ACTIONS(1521),
    [sym_define_module_keyword] = ACTIONS(1521),
    [sym_char_keyword] = ACTIONS(1521),
    [sym_widechar_keyword] = ACTIONS(1521),
    [sym_integer_keyword] = ACTIONS(1521),
    [sym_sinteger_keyword] = ACTIONS(1521),
    [sym_long_keyword] = ACTIONS(1521),
    [sym_slong_keyword] = ACTIONS(1521),
    [sym_float_keyword] = ACTIONS(1521),
    [sym_double_keyword] = ACTIONS(1521),
    [sym_variant_keyword] = ACTIONS(1521),
    [sym_variantarray_keyword] = ACTIONS(1521),
    [sym_local_var_keyword] = ACTIONS(1521),
    [sym_stack_var_keyword] = ACTIONS(1521),
    [sym_constant_keyword] = ACTIONS(1521),
    [sym_volatile_keyword] = ACTIONS(1521),
    [sym_non_volatile_keyword] = ACTIONS(1521),
    [sym_persistent_keyword] = ACTIONS(1521),
    [sym_dev_keyword] = ACTIONS(1521),
    [sym_devlev_keyword] = ACTIONS(1521),
    [sym_devchan_keyword] = ACTIONS(1521),
    [sym_if_keyword] = ACTIONS(1521),
    [sym_else_keyword] = ACTIONS(1521),
    [sym_switch_keyword] = ACTIONS(1521),
    [sym_case_keyword] = ACTIONS(1521),
    [sym_default_keyword] = ACTIONS(1521),
    [sym_while_keyword] = ACTIONS(1521),
    [sym_for_keyword] = ACTIONS(1521),
    [sym_break_keyword] = ACTIONS(1521),
    [sym_continue_keyword] = ACTIONS(1521),
    [sym_return_keyword] = ACTIONS(1521),
    [sym_select_keyword] = ACTIONS(1521),
    [sym_active_keyword] = ACTIONS(1521),
    [sym_struct_keyword] = ACTIONS(1521),
    [sym_structure_keyword] = ACTIONS(1521),
    [aux_sym_on_keyword_token1] = ACTIONS(1521),
    [aux_sym_off_keyword_token1] = ACTIONS(1521),
    [sym_send_level_keyword] = ACTIONS(1521),
    [sym_send_string_keyword] = ACTIONS(1521),
    [sym_send_command_keyword] = ACTIONS(1521),
    [sym_clear_buffer_keyword] = ACTIONS(1521),
    [sym_create_buffer_keyword] = ACTIONS(1521),
    [sym_create_multi_buffer_keyword] = ACTIONS(1521),
    [sym_call_keyword] = ACTIONS(1521),
    [sym_system_call_keyword] = ACTIONS(1521),
    [sym_devchan_to_keyword] = ACTIONS(1521),
    [sym_devchan_min_to_keyword] = ACTIONS(1521),
    [sym_devchan_total_off_keyword] = ACTIONS(1521),
    [sym_devchan_pulse_keyword] = ACTIONS(1521),
    [sym_wait_keyword] = ACTIONS(1521),
    [sym_cancel_wait_keyword] = ACTIONS(1521),
    [sym_wait_until_keyword] = ACTIONS(1521),
    [sym_cancel_wait_until_keyword] = ACTIONS(1521),
    [sym_cancel_all_wait_keyword] = ACTIONS(1521),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1521),
    [sym_and] = ACTIONS(1521),
    [sym_or] = ACTIONS(1521),
    [sym_not] = ACTIONS(1521),
    [sym_xor] = ACTIONS(1521),
    [sym_band] = ACTIONS(1521),
    [sym_bor] = ACTIONS(1521),
    [sym_bxor] = ACTIONS(1521),
    [sym_bnot] = ACTIONS(1521),
    [sym_lshift] = ACTIONS(1521),
    [sym_rshift] = ACTIONS(1521),
    [sym___file__] = ACTIONS(1521),
    [sym___date__] = ACTIONS(1521),
    [sym___ldate__] = ACTIONS(1521),
    [sym___time__] = ACTIONS(1521),
    [sym___line__] = ACTIONS(1521),
    [sym___name__] = ACTIONS(1521),
    [sym___netlinx__] = ACTIONS(1521),
    [sym_netlinx_axi_version] = ACTIONS(1521),
    [sym_url_flg_tcp] = ACTIONS(1521),
    [sym_url_flg_accinfopresent] = ACTIONS(1521),
    [sym_url_flg_temp] = ACTIONS(1521),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1521),
    [sym_url_flg_stat_mask] = ACTIONS(1521),
    [sym_url_flg_stat_lookup] = ACTIONS(1521),
    [sym_url_flg_stat_connecting] = ACTIONS(1521),
    [sym_url_flg_stat_waiting] = ACTIONS(1521),
    [sym_url_flg_stat_connected] = ACTIONS(1521),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1521),
    [sym_first_virtual_device] = ACTIONS(1521),
    [sym_first_local_port] = ACTIONS(1521),
    [sym_dynamic_virtual_device] = ACTIONS(1521),
    [sym_timeline_once] = ACTIONS(1521),
    [sym_timeline_repeat] = ACTIONS(1521),
    [sym_timeline_absolute] = ACTIONS(1521),
    [sym_timeline_relative] = ACTIONS(1521),
    [sym_do_push_timed_infinite] = ACTIONS(1521),
    [sym_source_type_no_address] = ACTIONS(1521),
    [sym_source_type_neuron_id] = ACTIONS(1521),
    [sym_source_type_ip_address] = ACTIONS(1521),
    [sym_source_type_axlink] = ACTIONS(1521),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1521),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1521),
    [sym_source_type_ip_socket_address] = ACTIONS(1521),
    [sym_source_type_rs232] = ACTIONS(1521),
    [sym_source_type_internal] = ACTIONS(1521),
    [sym_source_type_mac_address] = ACTIONS(1521),
    [sym_source_type_ipv4_port] = ACTIONS(1521),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1521),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1521),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1521),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1521),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1521),
    [sym_true] = ACTIONS(1521),
    [sym_false] = ACTIONS(1521),
    [sym_file_read_only] = ACTIONS(1521),
    [sym_file_rw_new] = ACTIONS(1521),
    [sym_file_rw_append] = ACTIONS(1521),
    [sym_ip_tcp] = ACTIONS(1521),
    [sym_ip_udp] = ACTIONS(1521),
    [sym_ip_udp_2way] = ACTIONS(1521),
    [sym_xml_encode_types] = ACTIONS(1521),
    [sym_xml_encode_char_as_list] = ACTIONS(1521),
    [sym_xml_encode_le] = ACTIONS(1521),
    [sym_xml_decode_types] = ACTIONS(1521),
    [sym_xml_decode_no_preserve] = ACTIONS(1521),
    [sym_duet_dev_type_display_device] = ACTIONS(1521),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1521),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1521),
    [sym_duet_dev_type_io_device] = ACTIONS(1521),
    [sym_duet_dev_type_relay_device] = ACTIONS(1521),
    [sym_duet_dev_type_ups] = ACTIONS(1521),
    [sym_duet_dev_type_amplifier] = ACTIONS(1521),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1521),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1521),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1521),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1521),
    [sym_duet_dev_type_camera] = ACTIONS(1521),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1521),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1521),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1521),
    [sym_duet_dev_type_dss] = ACTIONS(1521),
    [sym_duet_dev_type_dvr] = ACTIONS(1521),
    [sym_duet_dev_type_disc_device] = ACTIONS(1521),
    [sym_duet_dev_type_document_camera] = ACTIONS(1521),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1521),
    [sym_duet_dev_type_hvac] = ACTIONS(1521),
    [sym_duet_dev_type_keypad] = ACTIONS(1521),
    [sym_duet_dev_type_light] = ACTIONS(1521),
    [sym_duet_dev_type_monitor] = ACTIONS(1521),
    [sym_duet_dev_type_motor] = ACTIONS(1521),
    [sym_duet_dev_type_multi_window] = ACTIONS(1521),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1521),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1521),
    [sym_duet_dev_type_receiver] = ACTIONS(1521),
    [sym_duet_dev_type_security_system] = ACTIONS(1521),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1521),
    [sym_duet_dev_type_settop_box] = ACTIONS(1521),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1521),
    [sym_duet_dev_type_switcher] = ACTIONS(1521),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1521),
    [sym_duet_dev_type_tv] = ACTIONS(1521),
    [sym_duet_dev_type_utility] = ACTIONS(1521),
    [sym_duet_dev_type_vcr] = ACTIONS(1521),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1521),
    [sym_duet_dev_type_video_processor] = ACTIONS(1521),
    [sym_duet_dev_type_video_projector] = ACTIONS(1521),
    [sym_duet_dev_type_video_wall] = ACTIONS(1521),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1521),
    [sym_duet_dev_type_weather] = ACTIONS(1521),
    [sym_duet_dev_not_polled] = ACTIONS(1521),
    [sym_duet_dev_polled] = ACTIONS(1521),
    [sym_clkmgr_mode_network] = ACTIONS(1521),
    [sym_clkmgr_mode_standalone] = ACTIONS(1521),
    [sym_normal_standby] = ACTIONS(1521),
    [sym_normal_wake] = ACTIONS(1521),
    [sym_ip_multicast_ttl_option] = ACTIONS(1521),
    [sym_ip_ttl_subnet] = ACTIONS(1521),
    [sym_ip_ttl_site] = ACTIONS(1521),
    [sym_ip_ttl_region] = ACTIONS(1521),
    [sym_ip_ttl_continent] = ACTIONS(1521),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1521),
    [sym_ip_nodelay_on] = ACTIONS(1521),
    [sym_ip_nodelay_off] = ACTIONS(1521),
    [sym_amx_error] = ACTIONS(1521),
    [sym_amx_warning] = ACTIONS(1521),
    [sym_amx_info] = ACTIONS(1521),
    [sym_amx_debug] = ACTIONS(1521),
    [sym_netlinx_login_success] = ACTIONS(1521),
    [sym_netlinx_login_fail] = ACTIONS(1521),
    [sym_netlinx_logout] = ACTIONS(1521),
    [sym_null_str] = ACTIONS(1521),
    [sym_smtp_address] = ACTIONS(1521),
    [sym_smtp_port_number] = ACTIONS(1521),
    [sym_smtp_username] = ACTIONS(1521),
    [sym_smtp_password] = ACTIONS(1521),
    [sym_smtp_require_tls] = ACTIONS(1521),
    [sym_smtp_from] = ACTIONS(1521),
    [sym_smtp_tls_true] = ACTIONS(1521),
    [sym_smtp_tls_false] = ACTIONS(1521),
    [sym_listview_on_row_select_event] = ACTIONS(1521),
    [sym_data_structure_unknown] = ACTIONS(1521),
    [sym_data_structure_datafeed] = ACTIONS(1521),
    [sym_data_structure_datarecordset] = ACTIONS(1521),
    [sym_data_structure_datarecord] = ACTIONS(1521),
    [sym_data_structure_datafield] = ACTIONS(1521),
    [sym_data_max_content_field_count] = ACTIONS(1521),
    [sym_data_max_metadata_field_count] = ACTIONS(1521),
    [sym_data_max_id_length] = ACTIONS(1521),
    [sym_data_max_name_length] = ACTIONS(1521),
    [sym_data_max_description_length] = ACTIONS(1521),
    [sym_data_max_type_length] = ACTIONS(1521),
    [sym_data_max_format_length] = ACTIONS(1521),
    [sym_data_max_source_length] = ACTIONS(1521),
    [sym_data_max_value_length] = ACTIONS(1521),
    [sym_data_max_label_length] = ACTIONS(1521),
    [sym_data_max_url_length] = ACTIONS(1521),
    [sym_max_last_login_info_length] = ACTIONS(1521),
    [sym_data_type_unknown] = ACTIONS(1521),
    [sym_data_type_string] = ACTIONS(1521),
    [sym_data_type_datetime] = ACTIONS(1521),
    [sym_data_type_date] = ACTIONS(1521),
    [sym_data_type_time] = ACTIONS(1521),
    [sym_data_type_image] = ACTIONS(1521),
    [sym_data_format_url] = ACTIONS(1521),
    [sym_data_format_phone] = ACTIONS(1521),
    [sym_data_format_email] = ACTIONS(1521),
    [sym_data_format_iso8601] = ACTIONS(1521),
    [sym_auditlog_priv] = ACTIONS(1521),
    [sym_configuration_priv] = ACTIONS(1521),
    [sym_deviceconfig_priv] = ACTIONS(1521),
    [sym_ftp_priv] = ACTIONS(1521),
    [sym_http_priv] = ACTIONS(1521),
    [sym_networkconfig_priv] = ACTIONS(1521),
    [sym_programport_priv] = ACTIONS(1521),
    [sym_remoteui_priv] = ACTIONS(1521),
    [sym_securitycontrol_priv] = ACTIONS(1521),
    [sym_softwaremanagement_priv] = ACTIONS(1521),
    [sym_terminal_priv] = ACTIONS(1521),
    [sym_tpadmin_priv] = ACTIONS(1521),
    [sym_user1_priv] = ACTIONS(1521),
    [sym_user2_priv] = ACTIONS(1521),
    [sym_user3_priv] = ACTIONS(1521),
    [sym_user4_priv] = ACTIONS(1521),
    [sym_usermanagement_priv] = ACTIONS(1521),
    [sym_valid_account] = ACTIONS(1521),
    [sym_err_name_invalid] = ACTIONS(1521),
    [sym_err_password_invalid] = ACTIONS(1521),
    [sym_err_invalid_account] = ACTIONS(1521),
    [sym_err_no_authorization] = ACTIONS(1521),
    [sym_err_invalid_parameter] = ACTIONS(1521),
    [sym_err_user_locked_out] = ACTIONS(1521),
    [sym_err_user_account_expired] = ACTIONS(1521),
    [sym_err_general_error] = ACTIONS(1521),
    [sym_err_authentication_error] = ACTIONS(1521),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1521),
    [sym_tls_validate_certificate] = ACTIONS(1521),
    [sym_tdata] = ACTIONS(1521),
    [sym_tchannel] = ACTIONS(1521),
    [sym_tlevel] = ACTIONS(1521),
    [sym_tbutton] = ACTIONS(1521),
    [sym_ttimeline] = ACTIONS(1521),
    [sym_tcustom] = ACTIONS(1521),
    [sym_url_struct] = ACTIONS(1521),
    [sym_dns_struct] = ACTIONS(1521),
    [sym_ip_address_struct] = ACTIONS(1521),
    [sym_dev_info_struct] = ACTIONS(1521),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1521),
    [sym_data_feed] = ACTIONS(1521),
    [sym_data_field] = ACTIONS(1521),
    [sym_data_record] = ACTIONS(1521),
    [sym_wc_data_feed] = ACTIONS(1521),
    [sym_wc_data_field] = ACTIONS(1521),
    [sym_wc_data_record] = ACTIONS(1521),
    [sym_last_login_info] = ACTIONS(1521),
    [sym____reserved___] = ACTIONS(1521),
    [sym_date] = ACTIONS(1521),
    [sym_day] = ACTIONS(1521),
    [sym_time] = ACTIONS(1521),
    [sym_ldate] = ACTIONS(1521),
    [sym_push_device] = ACTIONS(1521),
    [sym_push_channel] = ACTIONS(1521),
    [sym_push_devchan] = ACTIONS(1521),
    [sym_release_device] = ACTIONS(1521),
    [sym_release_channel] = ACTIONS(1521),
    [sym_release_devchan] = ACTIONS(1521),
    [sym_master_slot] = ACTIONS(1521),
    [sym_get_pulse_time] = ACTIONS(1521),
    [sym_get_timer] = ACTIONS(1521),
    [sym_master_sn] = ACTIONS(1521),
    [sym_system_number] = ACTIONS(1521),
    [sym_button] = ACTIONS(1521),
    [sym_dv_channel] = ACTIONS(1521),
    [sym_data] = ACTIONS(1521),
    [sym_level] = ACTIONS(1521),
    [sym_timeline] = ACTIONS(1521),
    [sym_channel] = ACTIONS(1521),
    [sym_custom] = ACTIONS(1521),
    [sym_atoi] = ACTIONS(1521),
    [sym_atol] = ACTIONS(1521),
    [sym_atof] = ACTIONS(1521),
    [sym_itoa] = ACTIONS(1521),
    [sym_itohex] = ACTIONS(1521),
    [sym_hextoi] = ACTIONS(1521),
    [sym_ftoa] = ACTIONS(1521),
    [sym_find_string] = ACTIONS(1521),
    [sym_left_string] = ACTIONS(1521),
    [sym_right_string] = ACTIONS(1521),
    [sym_mid_string] = ACTIONS(1521),
    [sym_remove_string] = ACTIONS(1521),
    [sym_get_buffer_string] = ACTIONS(1521),
    [sym_get_buffer_char] = ACTIONS(1521),
    [sym_get_multi_buffer_string] = ACTIONS(1521),
    [sym_length_string] = ACTIONS(1521),
    [sym_max_length_string] = ACTIONS(1521),
    [sym_set_length_string] = ACTIONS(1521),
    [sym_compare_string] = ACTIONS(1521),
    [sym_upper_string] = ACTIONS(1521),
    [sym_lower_string] = ACTIONS(1521),
    [sym_get_last] = ACTIONS(1521),
    [sym_date_to_day] = ACTIONS(1521),
    [sym_date_to_month] = ACTIONS(1521),
    [sym_date_to_year] = ACTIONS(1521),
    [sym_time_to_hour] = ACTIONS(1521),
    [sym_time_to_minute] = ACTIONS(1521),
    [sym_time_to_second] = ACTIONS(1521),
    [sym_day_of_week] = ACTIONS(1521),
    [sym_set_pulse_time] = ACTIONS(1521),
    [sym_set_timer] = ACTIONS(1521),
    [sym_file_open] = ACTIONS(1521),
    [sym_file_close] = ACTIONS(1521),
    [sym_file_read] = ACTIONS(1521),
    [sym_file_read_line] = ACTIONS(1521),
    [sym_file_write] = ACTIONS(1521),
    [sym_file_write_line] = ACTIONS(1521),
    [sym_file_copy] = ACTIONS(1521),
    [sym_file_delete] = ACTIONS(1521),
    [sym_file_rename] = ACTIONS(1521),
    [sym_file_dir] = ACTIONS(1521),
    [sym_file_getdir] = ACTIONS(1521),
    [sym_file_setdir] = ACTIONS(1521),
    [sym_file_create_dir] = ACTIONS(1521),
    [sym_file_remove_dir] = ACTIONS(1521),
    [sym_file_seek] = ACTIONS(1521),
    [sym_length_array] = ACTIONS(1521),
    [sym_max_length_array] = ACTIONS(1521),
    [sym_set_length_array] = ACTIONS(1521),
    [sym_ip_client_open] = ACTIONS(1521),
    [sym_ip_client_close] = ACTIONS(1521),
    [sym_ip_server_open] = ACTIONS(1521),
    [sym_ip_server_close] = ACTIONS(1521),
    [sym_ip_bound_client_open] = ACTIONS(1521),
    [sym_ip_mc_server_open] = ACTIONS(1521),
    [sym_ip_set_option] = ACTIONS(1521),
    [sym_ssh_client_open] = ACTIONS(1521),
    [sym_ssh_client_close] = ACTIONS(1521),
    [sym_tls_client_open] = ACTIONS(1521),
    [sym_tls_client_close] = ACTIONS(1521),
    [sym_do_push] = ACTIONS(1521),
    [sym_do_push_timed] = ACTIONS(1521),
    [sym_do_release] = ACTIONS(1521),
    [sym_device_id] = ACTIONS(1521),
    [sym_device_info] = ACTIONS(1521),
    [sym_device_id_string] = ACTIONS(1521),
    [sym_device_standby] = ACTIONS(1521),
    [sym_device_wake] = ACTIONS(1521),
    [sym_timeline_create] = ACTIONS(1521),
    [sym_timeline_kill] = ACTIONS(1521),
    [sym_timeline_reload] = ACTIONS(1521),
    [sym_timeline_pause] = ACTIONS(1521),
    [sym_timeline_restart] = ACTIONS(1521),
    [sym_timeline_set] = ACTIONS(1521),
    [sym_timeline_get] = ACTIONS(1521),
    [sym_timeline_active] = ACTIONS(1521),
    [sym_timeline_dynamic_id] = ACTIONS(1521),
    [sym_variable_to_string] = ACTIONS(1521),
    [sym_string_to_variable] = ACTIONS(1521),
    [sym_length_variable_to_string] = ACTIONS(1521),
    [sym_type_cast] = ACTIONS(1521),
    [sym_variable_to_xml] = ACTIONS(1521),
    [sym_xml_to_variable] = ACTIONS(1521),
    [sym_length_variable_to_xml] = ACTIONS(1521),
    [sym_redirect_string] = ACTIONS(1521),
    [sym_random_number] = ACTIONS(1521),
    [sym_get_unique_id] = ACTIONS(1521),
    [sym_get_system_number] = ACTIONS(1521),
    [sym_set_system_number] = ACTIONS(1521),
    [sym_get_serial_number] = ACTIONS(1521),
    [sym_get_master_build] = ACTIONS(1521),
    [sym_set_validation_code] = ACTIONS(1521),
    [sym_reboot] = ACTIONS(1521),
    [sym_load_duet_module] = ACTIONS(1521),
    [sym_unload_duet_module] = ACTIONS(1521),
    [sym_duet_mem_size_set] = ACTIONS(1521),
    [sym_duet_mem_size_get] = ACTIONS(1521),
    [sym_dynamic_application_device] = ACTIONS(1521),
    [sym_dynamic_polled_port] = ACTIONS(1521),
    [sym_static_port_binding] = ACTIONS(1521),
    [sym_static_ip_binding] = ACTIONS(1521),
    [sym_get_dns_list] = ACTIONS(1521),
    [sym_set_dns_list] = ACTIONS(1521),
    [sym_get_ip_address] = ACTIONS(1521),
    [sym_set_ip_address] = ACTIONS(1521),
    [sym_get_url_list] = ACTIONS(1521),
    [sym_add_url_entry] = ACTIONS(1521),
    [sym_delete_url_entry] = ACTIONS(1521),
    [sym_abs_value] = ACTIONS(1521),
    [sym_max_value] = ACTIONS(1521),
    [sym_min_value] = ACTIONS(1521),
    [sym_exp_value] = ACTIONS(1521),
    [sym_log_value] = ACTIONS(1521),
    [sym_log10_value] = ACTIONS(1521),
    [sym_power_value] = ACTIONS(1521),
    [sym_sqrt_value] = ACTIONS(1521),
    [sym_raw_be] = ACTIONS(1521),
    [sym_raw_le] = ACTIONS(1521),
    [sym_internal_queue_size_set] = ACTIONS(1521),
    [sym_internal_queue_size_get] = ACTIONS(1521),
    [sym_internal_threshold_set] = ACTIONS(1521),
    [sym_internal_threshold_get] = ACTIONS(1521),
    [sym_rebuilt_event] = ACTIONS(1521),
    [sym_do_custom_event] = ACTIONS(1521),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1521),
    [sym_clkmgr_set_clk_source] = ACTIONS(1521),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1521),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1521),
    [sym_clkmgr_get_timezone] = ACTIONS(1521),
    [sym_clkmgr_set_timezone] = ACTIONS(1521),
    [sym_clkmgr_get_resync_period] = ACTIONS(1521),
    [sym_clkmgr_set_resync_period] = ACTIONS(1521),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1521),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1521),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1521),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1521),
    [sym_clkmgr_get_timeservers] = ACTIONS(1521),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1521),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1521),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1521),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1521),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1521),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1521),
    [sym_set_log_level] = ACTIONS(1521),
    [sym_get_log_level] = ACTIONS(1521),
    [sym_amx_log] = ACTIONS(1521),
    [sym_smtp_server_config_set] = ACTIONS(1521),
    [sym_smtp_server_config_get] = ACTIONS(1521),
    [sym_smtp_send] = ACTIONS(1521),
    [sym_validate_netlinx_account] = ACTIONS(1521),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1521),
    [sym_authenticate_certificate] = ACTIONS(1521),
    [sym_audit_netlinx_session_event] = ACTIONS(1521),
    [sym_audit_netlinx_generic_event] = ACTIONS(1521),
    [sym_get_available_flash_disk_space] = ACTIONS(1521),
    [sym_get_max_flash_disk_space] = ACTIONS(1521),
    [sym_data_create_feed] = ACTIONS(1521),
    [sym_data_delete_feed] = ACTIONS(1521),
    [sym_data_publish_feed] = ACTIONS(1521),
    [sym_data_get_published_feed] = ACTIONS(1521),
    [sym_data_add_record] = ACTIONS(1521),
    [sym_data_get_event_record] = ACTIONS(1521),
    [sym__wc_data_create_feed] = ACTIONS(1521),
    [sym__wc_data_add_record] = ACTIONS(1521),
    [sym__wc_data_get_event_record] = ACTIONS(1521),
    [sym_format] = ACTIONS(1521),
    [sym_astro_clock] = ACTIONS(1521),
    [sym_set_outdoor_temperature] = ACTIONS(1521),
    [sym_set_virtual_level_count] = ACTIONS(1521),
    [sym_set_virtual_channel_count] = ACTIONS(1521),
    [sym_set_virtual_port_count] = ACTIONS(1521),
    [sym__wc_encode_int] = ACTIONS(1521),
    [sym__wc_decode_int] = ACTIONS(1521),
    [sym__wc_int] = ACTIONS(1521),
    [sym__wc_to_ch_int] = ACTIONS(1521),
    [sym__ch_to_wc_int] = ACTIONS(1521),
    [sym__wc_find_string_int] = ACTIONS(1521),
    [sym__wc_left_string_int] = ACTIONS(1521),
    [sym__wc_lower_string_int] = ACTIONS(1521),
    [sym__wc_mid_string_int] = ACTIONS(1521),
    [sym__wc_remove_string_int] = ACTIONS(1521),
    [sym__wc_right_string_int] = ACTIONS(1521),
    [sym__wc_upper_string_int] = ACTIONS(1521),
    [sym__wc_compare_string_int] = ACTIONS(1521),
    [sym__wc_get_buffer_char_int] = ACTIONS(1521),
    [sym__wc_get_buffer_string_int] = ACTIONS(1521),
    [sym__wc_concat_string_int] = ACTIONS(1521),
    [sym__wc_file_open_int] = ACTIONS(1521),
    [sym__wc_file_close_int] = ACTIONS(1521),
    [sym__wc_file_read_int] = ACTIONS(1521),
    [sym__wc_file_read_line_int] = ACTIONS(1521),
    [sym__wc_file_write_int] = ACTIONS(1521),
    [sym__wc_file_write_line_int] = ACTIONS(1521),
    [anon_sym_EQ] = ACTIONS(1521),
    [aux_sym_preproc_include_token1] = ACTIONS(1521),
    [aux_sym_preproc_define_token1] = ACTIONS(1523),
    [aux_sym_preproc_warn_token1] = ACTIONS(1523),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1523),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1523),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(1523),
    [anon_sym_LBRACE] = ACTIONS(1523),
    [anon_sym_RBRACE] = ACTIONS(1523),
    [anon_sym_COLON] = ACTIONS(1523),
    [anon_sym_LPAREN] = ACTIONS(1521),
    [anon_sym_SEMI] = ACTIONS(1523),
    [anon_sym_BANG] = ACTIONS(1521),
    [anon_sym_TILDE] = ACTIONS(1523),
    [anon_sym_DASH] = ACTIONS(1521),
    [anon_sym_PLUS] = ACTIONS(1521),
    [anon_sym_STAR] = ACTIONS(1523),
    [anon_sym_SLASH] = ACTIONS(1521),
    [anon_sym_PERCENT] = ACTIONS(1523),
    [anon_sym_PIPE_PIPE] = ACTIONS(1523),
    [anon_sym_AMP_AMP] = ACTIONS(1523),
    [anon_sym_PIPE] = ACTIONS(1521),
    [anon_sym_CARET] = ACTIONS(1523),
    [anon_sym_AMP] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1523),
    [anon_sym_BANG_EQ] = ACTIONS(1523),
    [anon_sym_LT_GT] = ACTIONS(1523),
    [anon_sym_GT] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1523),
    [anon_sym_LT_EQ] = ACTIONS(1523),
    [anon_sym_LT] = ACTIONS(1521),
    [anon_sym_LT_LT] = ACTIONS(1523),
    [anon_sym_GT_GT] = ACTIONS(1523),
    [anon_sym_DASH_DASH] = ACTIONS(1523),
    [anon_sym_PLUS_PLUS] = ACTIONS(1523),
    [anon_sym_DOT] = ACTIONS(1521),
    [anon_sym_DQUOTE] = ACTIONS(1523),
    [anon_sym_SQUOTE] = ACTIONS(1523),
    [sym_number_literal] = ACTIONS(1523),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1523),
  },
  [STATE(175)] = {
    [sym_identifier] = ACTIONS(1465),
    [sym_define_call_keyword] = ACTIONS(1465),
    [sym_define_function_keyword] = ACTIONS(1465),
    [sym_define_library_function_keyword] = ACTIONS(1465),
    [sym_define_module_keyword] = ACTIONS(1465),
    [sym_char_keyword] = ACTIONS(1465),
    [sym_widechar_keyword] = ACTIONS(1465),
    [sym_integer_keyword] = ACTIONS(1465),
    [sym_sinteger_keyword] = ACTIONS(1465),
    [sym_long_keyword] = ACTIONS(1465),
    [sym_slong_keyword] = ACTIONS(1465),
    [sym_float_keyword] = ACTIONS(1465),
    [sym_double_keyword] = ACTIONS(1465),
    [sym_variant_keyword] = ACTIONS(1465),
    [sym_variantarray_keyword] = ACTIONS(1465),
    [sym_local_var_keyword] = ACTIONS(1465),
    [sym_stack_var_keyword] = ACTIONS(1465),
    [sym_constant_keyword] = ACTIONS(1465),
    [sym_volatile_keyword] = ACTIONS(1465),
    [sym_non_volatile_keyword] = ACTIONS(1465),
    [sym_persistent_keyword] = ACTIONS(1465),
    [sym_dev_keyword] = ACTIONS(1465),
    [sym_devlev_keyword] = ACTIONS(1465),
    [sym_devchan_keyword] = ACTIONS(1465),
    [sym_if_keyword] = ACTIONS(1465),
    [sym_else_keyword] = ACTIONS(1465),
    [sym_switch_keyword] = ACTIONS(1465),
    [sym_case_keyword] = ACTIONS(1465),
    [sym_default_keyword] = ACTIONS(1465),
    [sym_while_keyword] = ACTIONS(1465),
    [sym_for_keyword] = ACTIONS(1465),
    [sym_break_keyword] = ACTIONS(1465),
    [sym_continue_keyword] = ACTIONS(1465),
    [sym_return_keyword] = ACTIONS(1465),
    [sym_select_keyword] = ACTIONS(1465),
    [sym_active_keyword] = ACTIONS(1465),
    [sym_struct_keyword] = ACTIONS(1465),
    [sym_structure_keyword] = ACTIONS(1465),
    [aux_sym_on_keyword_token1] = ACTIONS(1465),
    [aux_sym_off_keyword_token1] = ACTIONS(1465),
    [sym_send_level_keyword] = ACTIONS(1465),
    [sym_send_string_keyword] = ACTIONS(1465),
    [sym_send_command_keyword] = ACTIONS(1465),
    [sym_clear_buffer_keyword] = ACTIONS(1465),
    [sym_create_buffer_keyword] = ACTIONS(1465),
    [sym_create_multi_buffer_keyword] = ACTIONS(1465),
    [sym_call_keyword] = ACTIONS(1465),
    [sym_system_call_keyword] = ACTIONS(1465),
    [sym_devchan_to_keyword] = ACTIONS(1465),
    [sym_devchan_min_to_keyword] = ACTIONS(1465),
    [sym_devchan_total_off_keyword] = ACTIONS(1465),
    [sym_devchan_pulse_keyword] = ACTIONS(1465),
    [sym_wait_keyword] = ACTIONS(1465),
    [sym_cancel_wait_keyword] = ACTIONS(1465),
    [sym_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym_tdata] = ACTIONS(1465),
    [sym_tchannel] = ACTIONS(1465),
    [sym_tlevel] = ACTIONS(1465),
    [sym_tbutton] = ACTIONS(1465),
    [sym_ttimeline] = ACTIONS(1465),
    [sym_tcustom] = ACTIONS(1465),
    [sym_url_struct] = ACTIONS(1465),
    [sym_dns_struct] = ACTIONS(1465),
    [sym_ip_address_struct] = ACTIONS(1465),
    [sym_dev_info_struct] = ACTIONS(1465),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1465),
    [sym_data_feed] = ACTIONS(1465),
    [sym_data_field] = ACTIONS(1465),
    [sym_data_record] = ACTIONS(1465),
    [sym_wc_data_feed] = ACTIONS(1465),
    [sym_wc_data_field] = ACTIONS(1465),
    [sym_wc_data_record] = ACTIONS(1465),
    [sym_last_login_info] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [anon_sym_EQ] = ACTIONS(1663),
    [aux_sym_preproc_include_token1] = ACTIONS(1465),
    [aux_sym_preproc_define_token1] = ACTIONS(1469),
    [aux_sym_preproc_warn_token1] = ACTIONS(1469),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_RBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1469),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1469),
  },
  [STATE(176)] = {
    [sym_identifier] = ACTIONS(1465),
    [sym_define_call_keyword] = ACTIONS(1465),
    [sym_define_function_keyword] = ACTIONS(1465),
    [sym_define_library_function_keyword] = ACTIONS(1465),
    [sym_define_module_keyword] = ACTIONS(1465),
    [sym_char_keyword] = ACTIONS(1465),
    [sym_widechar_keyword] = ACTIONS(1465),
    [sym_integer_keyword] = ACTIONS(1465),
    [sym_sinteger_keyword] = ACTIONS(1465),
    [sym_long_keyword] = ACTIONS(1465),
    [sym_slong_keyword] = ACTIONS(1465),
    [sym_float_keyword] = ACTIONS(1465),
    [sym_double_keyword] = ACTIONS(1465),
    [sym_variant_keyword] = ACTIONS(1465),
    [sym_variantarray_keyword] = ACTIONS(1465),
    [sym_local_var_keyword] = ACTIONS(1465),
    [sym_stack_var_keyword] = ACTIONS(1465),
    [sym_constant_keyword] = ACTIONS(1465),
    [sym_volatile_keyword] = ACTIONS(1465),
    [sym_non_volatile_keyword] = ACTIONS(1465),
    [sym_persistent_keyword] = ACTIONS(1465),
    [sym_dev_keyword] = ACTIONS(1465),
    [sym_devlev_keyword] = ACTIONS(1465),
    [sym_devchan_keyword] = ACTIONS(1465),
    [sym_if_keyword] = ACTIONS(1465),
    [sym_else_keyword] = ACTIONS(1465),
    [sym_switch_keyword] = ACTIONS(1465),
    [sym_case_keyword] = ACTIONS(1465),
    [sym_default_keyword] = ACTIONS(1465),
    [sym_while_keyword] = ACTIONS(1465),
    [sym_for_keyword] = ACTIONS(1465),
    [sym_break_keyword] = ACTIONS(1465),
    [sym_continue_keyword] = ACTIONS(1465),
    [sym_return_keyword] = ACTIONS(1465),
    [sym_select_keyword] = ACTIONS(1465),
    [sym_active_keyword] = ACTIONS(1465),
    [sym_struct_keyword] = ACTIONS(1465),
    [sym_structure_keyword] = ACTIONS(1465),
    [aux_sym_on_keyword_token1] = ACTIONS(1465),
    [aux_sym_off_keyword_token1] = ACTIONS(1465),
    [sym_send_level_keyword] = ACTIONS(1465),
    [sym_send_string_keyword] = ACTIONS(1465),
    [sym_send_command_keyword] = ACTIONS(1465),
    [sym_clear_buffer_keyword] = ACTIONS(1465),
    [sym_create_buffer_keyword] = ACTIONS(1465),
    [sym_create_multi_buffer_keyword] = ACTIONS(1465),
    [sym_call_keyword] = ACTIONS(1465),
    [sym_system_call_keyword] = ACTIONS(1465),
    [sym_devchan_to_keyword] = ACTIONS(1465),
    [sym_devchan_min_to_keyword] = ACTIONS(1465),
    [sym_devchan_total_off_keyword] = ACTIONS(1465),
    [sym_devchan_pulse_keyword] = ACTIONS(1465),
    [sym_wait_keyword] = ACTIONS(1465),
    [sym_cancel_wait_keyword] = ACTIONS(1465),
    [sym_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym_tdata] = ACTIONS(1465),
    [sym_tchannel] = ACTIONS(1465),
    [sym_tlevel] = ACTIONS(1465),
    [sym_tbutton] = ACTIONS(1465),
    [sym_ttimeline] = ACTIONS(1465),
    [sym_tcustom] = ACTIONS(1465),
    [sym_url_struct] = ACTIONS(1465),
    [sym_dns_struct] = ACTIONS(1465),
    [sym_ip_address_struct] = ACTIONS(1465),
    [sym_dev_info_struct] = ACTIONS(1465),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1465),
    [sym_data_feed] = ACTIONS(1465),
    [sym_data_field] = ACTIONS(1465),
    [sym_data_record] = ACTIONS(1465),
    [sym_wc_data_feed] = ACTIONS(1465),
    [sym_wc_data_field] = ACTIONS(1465),
    [sym_wc_data_record] = ACTIONS(1465),
    [sym_last_login_info] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [anon_sym_EQ] = ACTIONS(1656),
    [aux_sym_preproc_include_token1] = ACTIONS(1465),
    [aux_sym_preproc_define_token1] = ACTIONS(1469),
    [aux_sym_preproc_warn_token1] = ACTIONS(1469),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_RBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1469),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1469),
  },
  [STATE(177)] = {
    [sym_identifier] = ACTIONS(1551),
    [sym_define_call_keyword] = ACTIONS(1551),
    [sym_define_function_keyword] = ACTIONS(1551),
    [sym_define_library_function_keyword] = ACTIONS(1551),
    [sym_define_module_keyword] = ACTIONS(1551),
    [sym_char_keyword] = ACTIONS(1551),
    [sym_widechar_keyword] = ACTIONS(1551),
    [sym_integer_keyword] = ACTIONS(1551),
    [sym_sinteger_keyword] = ACTIONS(1551),
    [sym_long_keyword] = ACTIONS(1551),
    [sym_slong_keyword] = ACTIONS(1551),
    [sym_float_keyword] = ACTIONS(1551),
    [sym_double_keyword] = ACTIONS(1551),
    [sym_variant_keyword] = ACTIONS(1551),
    [sym_variantarray_keyword] = ACTIONS(1551),
    [sym_local_var_keyword] = ACTIONS(1551),
    [sym_stack_var_keyword] = ACTIONS(1551),
    [sym_constant_keyword] = ACTIONS(1551),
    [sym_volatile_keyword] = ACTIONS(1551),
    [sym_non_volatile_keyword] = ACTIONS(1551),
    [sym_persistent_keyword] = ACTIONS(1551),
    [sym_dev_keyword] = ACTIONS(1551),
    [sym_devlev_keyword] = ACTIONS(1551),
    [sym_devchan_keyword] = ACTIONS(1551),
    [sym_if_keyword] = ACTIONS(1551),
    [sym_else_keyword] = ACTIONS(1551),
    [sym_switch_keyword] = ACTIONS(1551),
    [sym_case_keyword] = ACTIONS(1551),
    [sym_default_keyword] = ACTIONS(1551),
    [sym_while_keyword] = ACTIONS(1551),
    [sym_for_keyword] = ACTIONS(1551),
    [sym_break_keyword] = ACTIONS(1551),
    [sym_continue_keyword] = ACTIONS(1551),
    [sym_return_keyword] = ACTIONS(1551),
    [sym_select_keyword] = ACTIONS(1551),
    [sym_active_keyword] = ACTIONS(1551),
    [sym_struct_keyword] = ACTIONS(1551),
    [sym_structure_keyword] = ACTIONS(1551),
    [aux_sym_on_keyword_token1] = ACTIONS(1551),
    [aux_sym_off_keyword_token1] = ACTIONS(1551),
    [sym_send_level_keyword] = ACTIONS(1551),
    [sym_send_string_keyword] = ACTIONS(1551),
    [sym_send_command_keyword] = ACTIONS(1551),
    [sym_clear_buffer_keyword] = ACTIONS(1551),
    [sym_create_buffer_keyword] = ACTIONS(1551),
    [sym_create_multi_buffer_keyword] = ACTIONS(1551),
    [sym_call_keyword] = ACTIONS(1551),
    [sym_system_call_keyword] = ACTIONS(1551),
    [sym_devchan_to_keyword] = ACTIONS(1551),
    [sym_devchan_min_to_keyword] = ACTIONS(1551),
    [sym_devchan_total_off_keyword] = ACTIONS(1551),
    [sym_devchan_pulse_keyword] = ACTIONS(1551),
    [sym_wait_keyword] = ACTIONS(1551),
    [sym_cancel_wait_keyword] = ACTIONS(1551),
    [sym_wait_until_keyword] = ACTIONS(1551),
    [sym_cancel_wait_until_keyword] = ACTIONS(1551),
    [sym_cancel_all_wait_keyword] = ACTIONS(1551),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1551),
    [sym_and] = ACTIONS(1551),
    [sym_or] = ACTIONS(1551),
    [sym_not] = ACTIONS(1551),
    [sym_xor] = ACTIONS(1551),
    [sym_band] = ACTIONS(1551),
    [sym_bor] = ACTIONS(1551),
    [sym_bxor] = ACTIONS(1551),
    [sym_bnot] = ACTIONS(1551),
    [sym_lshift] = ACTIONS(1551),
    [sym_rshift] = ACTIONS(1551),
    [sym___file__] = ACTIONS(1551),
    [sym___date__] = ACTIONS(1551),
    [sym___ldate__] = ACTIONS(1551),
    [sym___time__] = ACTIONS(1551),
    [sym___line__] = ACTIONS(1551),
    [sym___name__] = ACTIONS(1551),
    [sym___netlinx__] = ACTIONS(1551),
    [sym_netlinx_axi_version] = ACTIONS(1551),
    [sym_url_flg_tcp] = ACTIONS(1551),
    [sym_url_flg_accinfopresent] = ACTIONS(1551),
    [sym_url_flg_temp] = ACTIONS(1551),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1551),
    [sym_url_flg_stat_mask] = ACTIONS(1551),
    [sym_url_flg_stat_lookup] = ACTIONS(1551),
    [sym_url_flg_stat_connecting] = ACTIONS(1551),
    [sym_url_flg_stat_waiting] = ACTIONS(1551),
    [sym_url_flg_stat_connected] = ACTIONS(1551),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1551),
    [sym_first_virtual_device] = ACTIONS(1551),
    [sym_first_local_port] = ACTIONS(1551),
    [sym_dynamic_virtual_device] = ACTIONS(1551),
    [sym_timeline_once] = ACTIONS(1551),
    [sym_timeline_repeat] = ACTIONS(1551),
    [sym_timeline_absolute] = ACTIONS(1551),
    [sym_timeline_relative] = ACTIONS(1551),
    [sym_do_push_timed_infinite] = ACTIONS(1551),
    [sym_source_type_no_address] = ACTIONS(1551),
    [sym_source_type_neuron_id] = ACTIONS(1551),
    [sym_source_type_ip_address] = ACTIONS(1551),
    [sym_source_type_axlink] = ACTIONS(1551),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1551),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1551),
    [sym_source_type_ip_socket_address] = ACTIONS(1551),
    [sym_source_type_rs232] = ACTIONS(1551),
    [sym_source_type_internal] = ACTIONS(1551),
    [sym_source_type_mac_address] = ACTIONS(1551),
    [sym_source_type_ipv4_port] = ACTIONS(1551),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1551),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1551),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1551),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1551),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1551),
    [sym_true] = ACTIONS(1551),
    [sym_false] = ACTIONS(1551),
    [sym_file_read_only] = ACTIONS(1551),
    [sym_file_rw_new] = ACTIONS(1551),
    [sym_file_rw_append] = ACTIONS(1551),
    [sym_ip_tcp] = ACTIONS(1551),
    [sym_ip_udp] = ACTIONS(1551),
    [sym_ip_udp_2way] = ACTIONS(1551),
    [sym_xml_encode_types] = ACTIONS(1551),
    [sym_xml_encode_char_as_list] = ACTIONS(1551),
    [sym_xml_encode_le] = ACTIONS(1551),
    [sym_xml_decode_types] = ACTIONS(1551),
    [sym_xml_decode_no_preserve] = ACTIONS(1551),
    [sym_duet_dev_type_display_device] = ACTIONS(1551),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1551),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1551),
    [sym_duet_dev_type_io_device] = ACTIONS(1551),
    [sym_duet_dev_type_relay_device] = ACTIONS(1551),
    [sym_duet_dev_type_ups] = ACTIONS(1551),
    [sym_duet_dev_type_amplifier] = ACTIONS(1551),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1551),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1551),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1551),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1551),
    [sym_duet_dev_type_camera] = ACTIONS(1551),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1551),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1551),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1551),
    [sym_duet_dev_type_dss] = ACTIONS(1551),
    [sym_duet_dev_type_dvr] = ACTIONS(1551),
    [sym_duet_dev_type_disc_device] = ACTIONS(1551),
    [sym_duet_dev_type_document_camera] = ACTIONS(1551),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1551),
    [sym_duet_dev_type_hvac] = ACTIONS(1551),
    [sym_duet_dev_type_keypad] = ACTIONS(1551),
    [sym_duet_dev_type_light] = ACTIONS(1551),
    [sym_duet_dev_type_monitor] = ACTIONS(1551),
    [sym_duet_dev_type_motor] = ACTIONS(1551),
    [sym_duet_dev_type_multi_window] = ACTIONS(1551),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1551),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1551),
    [sym_duet_dev_type_receiver] = ACTIONS(1551),
    [sym_duet_dev_type_security_system] = ACTIONS(1551),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1551),
    [sym_duet_dev_type_settop_box] = ACTIONS(1551),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1551),
    [sym_duet_dev_type_switcher] = ACTIONS(1551),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1551),
    [sym_duet_dev_type_tv] = ACTIONS(1551),
    [sym_duet_dev_type_utility] = ACTIONS(1551),
    [sym_duet_dev_type_vcr] = ACTIONS(1551),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1551),
    [sym_duet_dev_type_video_processor] = ACTIONS(1551),
    [sym_duet_dev_type_video_projector] = ACTIONS(1551),
    [sym_duet_dev_type_video_wall] = ACTIONS(1551),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1551),
    [sym_duet_dev_type_weather] = ACTIONS(1551),
    [sym_duet_dev_not_polled] = ACTIONS(1551),
    [sym_duet_dev_polled] = ACTIONS(1551),
    [sym_clkmgr_mode_network] = ACTIONS(1551),
    [sym_clkmgr_mode_standalone] = ACTIONS(1551),
    [sym_normal_standby] = ACTIONS(1551),
    [sym_normal_wake] = ACTIONS(1551),
    [sym_ip_multicast_ttl_option] = ACTIONS(1551),
    [sym_ip_ttl_subnet] = ACTIONS(1551),
    [sym_ip_ttl_site] = ACTIONS(1551),
    [sym_ip_ttl_region] = ACTIONS(1551),
    [sym_ip_ttl_continent] = ACTIONS(1551),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1551),
    [sym_ip_nodelay_on] = ACTIONS(1551),
    [sym_ip_nodelay_off] = ACTIONS(1551),
    [sym_amx_error] = ACTIONS(1551),
    [sym_amx_warning] = ACTIONS(1551),
    [sym_amx_info] = ACTIONS(1551),
    [sym_amx_debug] = ACTIONS(1551),
    [sym_netlinx_login_success] = ACTIONS(1551),
    [sym_netlinx_login_fail] = ACTIONS(1551),
    [sym_netlinx_logout] = ACTIONS(1551),
    [sym_null_str] = ACTIONS(1551),
    [sym_smtp_address] = ACTIONS(1551),
    [sym_smtp_port_number] = ACTIONS(1551),
    [sym_smtp_username] = ACTIONS(1551),
    [sym_smtp_password] = ACTIONS(1551),
    [sym_smtp_require_tls] = ACTIONS(1551),
    [sym_smtp_from] = ACTIONS(1551),
    [sym_smtp_tls_true] = ACTIONS(1551),
    [sym_smtp_tls_false] = ACTIONS(1551),
    [sym_listview_on_row_select_event] = ACTIONS(1551),
    [sym_data_structure_unknown] = ACTIONS(1551),
    [sym_data_structure_datafeed] = ACTIONS(1551),
    [sym_data_structure_datarecordset] = ACTIONS(1551),
    [sym_data_structure_datarecord] = ACTIONS(1551),
    [sym_data_structure_datafield] = ACTIONS(1551),
    [sym_data_max_content_field_count] = ACTIONS(1551),
    [sym_data_max_metadata_field_count] = ACTIONS(1551),
    [sym_data_max_id_length] = ACTIONS(1551),
    [sym_data_max_name_length] = ACTIONS(1551),
    [sym_data_max_description_length] = ACTIONS(1551),
    [sym_data_max_type_length] = ACTIONS(1551),
    [sym_data_max_format_length] = ACTIONS(1551),
    [sym_data_max_source_length] = ACTIONS(1551),
    [sym_data_max_value_length] = ACTIONS(1551),
    [sym_data_max_label_length] = ACTIONS(1551),
    [sym_data_max_url_length] = ACTIONS(1551),
    [sym_max_last_login_info_length] = ACTIONS(1551),
    [sym_data_type_unknown] = ACTIONS(1551),
    [sym_data_type_string] = ACTIONS(1551),
    [sym_data_type_datetime] = ACTIONS(1551),
    [sym_data_type_date] = ACTIONS(1551),
    [sym_data_type_time] = ACTIONS(1551),
    [sym_data_type_image] = ACTIONS(1551),
    [sym_data_format_url] = ACTIONS(1551),
    [sym_data_format_phone] = ACTIONS(1551),
    [sym_data_format_email] = ACTIONS(1551),
    [sym_data_format_iso8601] = ACTIONS(1551),
    [sym_auditlog_priv] = ACTIONS(1551),
    [sym_configuration_priv] = ACTIONS(1551),
    [sym_deviceconfig_priv] = ACTIONS(1551),
    [sym_ftp_priv] = ACTIONS(1551),
    [sym_http_priv] = ACTIONS(1551),
    [sym_networkconfig_priv] = ACTIONS(1551),
    [sym_programport_priv] = ACTIONS(1551),
    [sym_remoteui_priv] = ACTIONS(1551),
    [sym_securitycontrol_priv] = ACTIONS(1551),
    [sym_softwaremanagement_priv] = ACTIONS(1551),
    [sym_terminal_priv] = ACTIONS(1551),
    [sym_tpadmin_priv] = ACTIONS(1551),
    [sym_user1_priv] = ACTIONS(1551),
    [sym_user2_priv] = ACTIONS(1551),
    [sym_user3_priv] = ACTIONS(1551),
    [sym_user4_priv] = ACTIONS(1551),
    [sym_usermanagement_priv] = ACTIONS(1551),
    [sym_valid_account] = ACTIONS(1551),
    [sym_err_name_invalid] = ACTIONS(1551),
    [sym_err_password_invalid] = ACTIONS(1551),
    [sym_err_invalid_account] = ACTIONS(1551),
    [sym_err_no_authorization] = ACTIONS(1551),
    [sym_err_invalid_parameter] = ACTIONS(1551),
    [sym_err_user_locked_out] = ACTIONS(1551),
    [sym_err_user_account_expired] = ACTIONS(1551),
    [sym_err_general_error] = ACTIONS(1551),
    [sym_err_authentication_error] = ACTIONS(1551),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1551),
    [sym_tls_validate_certificate] = ACTIONS(1551),
    [sym_tdata] = ACTIONS(1551),
    [sym_tchannel] = ACTIONS(1551),
    [sym_tlevel] = ACTIONS(1551),
    [sym_tbutton] = ACTIONS(1551),
    [sym_ttimeline] = ACTIONS(1551),
    [sym_tcustom] = ACTIONS(1551),
    [sym_url_struct] = ACTIONS(1551),
    [sym_dns_struct] = ACTIONS(1551),
    [sym_ip_address_struct] = ACTIONS(1551),
    [sym_dev_info_struct] = ACTIONS(1551),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1551),
    [sym_data_feed] = ACTIONS(1551),
    [sym_data_field] = ACTIONS(1551),
    [sym_data_record] = ACTIONS(1551),
    [sym_wc_data_feed] = ACTIONS(1551),
    [sym_wc_data_field] = ACTIONS(1551),
    [sym_wc_data_record] = ACTIONS(1551),
    [sym_last_login_info] = ACTIONS(1551),
    [sym____reserved___] = ACTIONS(1551),
    [sym_date] = ACTIONS(1551),
    [sym_day] = ACTIONS(1551),
    [sym_time] = ACTIONS(1551),
    [sym_ldate] = ACTIONS(1551),
    [sym_push_device] = ACTIONS(1551),
    [sym_push_channel] = ACTIONS(1551),
    [sym_push_devchan] = ACTIONS(1551),
    [sym_release_device] = ACTIONS(1551),
    [sym_release_channel] = ACTIONS(1551),
    [sym_release_devchan] = ACTIONS(1551),
    [sym_master_slot] = ACTIONS(1551),
    [sym_get_pulse_time] = ACTIONS(1551),
    [sym_get_timer] = ACTIONS(1551),
    [sym_master_sn] = ACTIONS(1551),
    [sym_system_number] = ACTIONS(1551),
    [sym_button] = ACTIONS(1551),
    [sym_dv_channel] = ACTIONS(1551),
    [sym_data] = ACTIONS(1551),
    [sym_level] = ACTIONS(1551),
    [sym_timeline] = ACTIONS(1551),
    [sym_channel] = ACTIONS(1551),
    [sym_custom] = ACTIONS(1551),
    [sym_atoi] = ACTIONS(1551),
    [sym_atol] = ACTIONS(1551),
    [sym_atof] = ACTIONS(1551),
    [sym_itoa] = ACTIONS(1551),
    [sym_itohex] = ACTIONS(1551),
    [sym_hextoi] = ACTIONS(1551),
    [sym_ftoa] = ACTIONS(1551),
    [sym_find_string] = ACTIONS(1551),
    [sym_left_string] = ACTIONS(1551),
    [sym_right_string] = ACTIONS(1551),
    [sym_mid_string] = ACTIONS(1551),
    [sym_remove_string] = ACTIONS(1551),
    [sym_get_buffer_string] = ACTIONS(1551),
    [sym_get_buffer_char] = ACTIONS(1551),
    [sym_get_multi_buffer_string] = ACTIONS(1551),
    [sym_length_string] = ACTIONS(1551),
    [sym_max_length_string] = ACTIONS(1551),
    [sym_set_length_string] = ACTIONS(1551),
    [sym_compare_string] = ACTIONS(1551),
    [sym_upper_string] = ACTIONS(1551),
    [sym_lower_string] = ACTIONS(1551),
    [sym_get_last] = ACTIONS(1551),
    [sym_date_to_day] = ACTIONS(1551),
    [sym_date_to_month] = ACTIONS(1551),
    [sym_date_to_year] = ACTIONS(1551),
    [sym_time_to_hour] = ACTIONS(1551),
    [sym_time_to_minute] = ACTIONS(1551),
    [sym_time_to_second] = ACTIONS(1551),
    [sym_day_of_week] = ACTIONS(1551),
    [sym_set_pulse_time] = ACTIONS(1551),
    [sym_set_timer] = ACTIONS(1551),
    [sym_file_open] = ACTIONS(1551),
    [sym_file_close] = ACTIONS(1551),
    [sym_file_read] = ACTIONS(1551),
    [sym_file_read_line] = ACTIONS(1551),
    [sym_file_write] = ACTIONS(1551),
    [sym_file_write_line] = ACTIONS(1551),
    [sym_file_copy] = ACTIONS(1551),
    [sym_file_delete] = ACTIONS(1551),
    [sym_file_rename] = ACTIONS(1551),
    [sym_file_dir] = ACTIONS(1551),
    [sym_file_getdir] = ACTIONS(1551),
    [sym_file_setdir] = ACTIONS(1551),
    [sym_file_create_dir] = ACTIONS(1551),
    [sym_file_remove_dir] = ACTIONS(1551),
    [sym_file_seek] = ACTIONS(1551),
    [sym_length_array] = ACTIONS(1551),
    [sym_max_length_array] = ACTIONS(1551),
    [sym_set_length_array] = ACTIONS(1551),
    [sym_ip_client_open] = ACTIONS(1551),
    [sym_ip_client_close] = ACTIONS(1551),
    [sym_ip_server_open] = ACTIONS(1551),
    [sym_ip_server_close] = ACTIONS(1551),
    [sym_ip_bound_client_open] = ACTIONS(1551),
    [sym_ip_mc_server_open] = ACTIONS(1551),
    [sym_ip_set_option] = ACTIONS(1551),
    [sym_ssh_client_open] = ACTIONS(1551),
    [sym_ssh_client_close] = ACTIONS(1551),
    [sym_tls_client_open] = ACTIONS(1551),
    [sym_tls_client_close] = ACTIONS(1551),
    [sym_do_push] = ACTIONS(1551),
    [sym_do_push_timed] = ACTIONS(1551),
    [sym_do_release] = ACTIONS(1551),
    [sym_device_id] = ACTIONS(1551),
    [sym_device_info] = ACTIONS(1551),
    [sym_device_id_string] = ACTIONS(1551),
    [sym_device_standby] = ACTIONS(1551),
    [sym_device_wake] = ACTIONS(1551),
    [sym_timeline_create] = ACTIONS(1551),
    [sym_timeline_kill] = ACTIONS(1551),
    [sym_timeline_reload] = ACTIONS(1551),
    [sym_timeline_pause] = ACTIONS(1551),
    [sym_timeline_restart] = ACTIONS(1551),
    [sym_timeline_set] = ACTIONS(1551),
    [sym_timeline_get] = ACTIONS(1551),
    [sym_timeline_active] = ACTIONS(1551),
    [sym_timeline_dynamic_id] = ACTIONS(1551),
    [sym_variable_to_string] = ACTIONS(1551),
    [sym_string_to_variable] = ACTIONS(1551),
    [sym_length_variable_to_string] = ACTIONS(1551),
    [sym_type_cast] = ACTIONS(1551),
    [sym_variable_to_xml] = ACTIONS(1551),
    [sym_xml_to_variable] = ACTIONS(1551),
    [sym_length_variable_to_xml] = ACTIONS(1551),
    [sym_redirect_string] = ACTIONS(1551),
    [sym_random_number] = ACTIONS(1551),
    [sym_get_unique_id] = ACTIONS(1551),
    [sym_get_system_number] = ACTIONS(1551),
    [sym_set_system_number] = ACTIONS(1551),
    [sym_get_serial_number] = ACTIONS(1551),
    [sym_get_master_build] = ACTIONS(1551),
    [sym_set_validation_code] = ACTIONS(1551),
    [sym_reboot] = ACTIONS(1551),
    [sym_load_duet_module] = ACTIONS(1551),
    [sym_unload_duet_module] = ACTIONS(1551),
    [sym_duet_mem_size_set] = ACTIONS(1551),
    [sym_duet_mem_size_get] = ACTIONS(1551),
    [sym_dynamic_application_device] = ACTIONS(1551),
    [sym_dynamic_polled_port] = ACTIONS(1551),
    [sym_static_port_binding] = ACTIONS(1551),
    [sym_static_ip_binding] = ACTIONS(1551),
    [sym_get_dns_list] = ACTIONS(1551),
    [sym_set_dns_list] = ACTIONS(1551),
    [sym_get_ip_address] = ACTIONS(1551),
    [sym_set_ip_address] = ACTIONS(1551),
    [sym_get_url_list] = ACTIONS(1551),
    [sym_add_url_entry] = ACTIONS(1551),
    [sym_delete_url_entry] = ACTIONS(1551),
    [sym_abs_value] = ACTIONS(1551),
    [sym_max_value] = ACTIONS(1551),
    [sym_min_value] = ACTIONS(1551),
    [sym_exp_value] = ACTIONS(1551),
    [sym_log_value] = ACTIONS(1551),
    [sym_log10_value] = ACTIONS(1551),
    [sym_power_value] = ACTIONS(1551),
    [sym_sqrt_value] = ACTIONS(1551),
    [sym_raw_be] = ACTIONS(1551),
    [sym_raw_le] = ACTIONS(1551),
    [sym_internal_queue_size_set] = ACTIONS(1551),
    [sym_internal_queue_size_get] = ACTIONS(1551),
    [sym_internal_threshold_set] = ACTIONS(1551),
    [sym_internal_threshold_get] = ACTIONS(1551),
    [sym_rebuilt_event] = ACTIONS(1551),
    [sym_do_custom_event] = ACTIONS(1551),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1551),
    [sym_clkmgr_set_clk_source] = ACTIONS(1551),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1551),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1551),
    [sym_clkmgr_get_timezone] = ACTIONS(1551),
    [sym_clkmgr_set_timezone] = ACTIONS(1551),
    [sym_clkmgr_get_resync_period] = ACTIONS(1551),
    [sym_clkmgr_set_resync_period] = ACTIONS(1551),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1551),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1551),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1551),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1551),
    [sym_clkmgr_get_timeservers] = ACTIONS(1551),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1551),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1551),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1551),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1551),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1551),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1551),
    [sym_set_log_level] = ACTIONS(1551),
    [sym_get_log_level] = ACTIONS(1551),
    [sym_amx_log] = ACTIONS(1551),
    [sym_smtp_server_config_set] = ACTIONS(1551),
    [sym_smtp_server_config_get] = ACTIONS(1551),
    [sym_smtp_send] = ACTIONS(1551),
    [sym_validate_netlinx_account] = ACTIONS(1551),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1551),
    [sym_authenticate_certificate] = ACTIONS(1551),
    [sym_audit_netlinx_session_event] = ACTIONS(1551),
    [sym_audit_netlinx_generic_event] = ACTIONS(1551),
    [sym_get_available_flash_disk_space] = ACTIONS(1551),
    [sym_get_max_flash_disk_space] = ACTIONS(1551),
    [sym_data_create_feed] = ACTIONS(1551),
    [sym_data_delete_feed] = ACTIONS(1551),
    [sym_data_publish_feed] = ACTIONS(1551),
    [sym_data_get_published_feed] = ACTIONS(1551),
    [sym_data_add_record] = ACTIONS(1551),
    [sym_data_get_event_record] = ACTIONS(1551),
    [sym__wc_data_create_feed] = ACTIONS(1551),
    [sym__wc_data_add_record] = ACTIONS(1551),
    [sym__wc_data_get_event_record] = ACTIONS(1551),
    [sym_format] = ACTIONS(1551),
    [sym_astro_clock] = ACTIONS(1551),
    [sym_set_outdoor_temperature] = ACTIONS(1551),
    [sym_set_virtual_level_count] = ACTIONS(1551),
    [sym_set_virtual_channel_count] = ACTIONS(1551),
    [sym_set_virtual_port_count] = ACTIONS(1551),
    [sym__wc_encode_int] = ACTIONS(1551),
    [sym__wc_decode_int] = ACTIONS(1551),
    [sym__wc_int] = ACTIONS(1551),
    [sym__wc_to_ch_int] = ACTIONS(1551),
    [sym__ch_to_wc_int] = ACTIONS(1551),
    [sym__wc_find_string_int] = ACTIONS(1551),
    [sym__wc_left_string_int] = ACTIONS(1551),
    [sym__wc_lower_string_int] = ACTIONS(1551),
    [sym__wc_mid_string_int] = ACTIONS(1551),
    [sym__wc_remove_string_int] = ACTIONS(1551),
    [sym__wc_right_string_int] = ACTIONS(1551),
    [sym__wc_upper_string_int] = ACTIONS(1551),
    [sym__wc_compare_string_int] = ACTIONS(1551),
    [sym__wc_get_buffer_char_int] = ACTIONS(1551),
    [sym__wc_get_buffer_string_int] = ACTIONS(1551),
    [sym__wc_concat_string_int] = ACTIONS(1551),
    [sym__wc_file_open_int] = ACTIONS(1551),
    [sym__wc_file_close_int] = ACTIONS(1551),
    [sym__wc_file_read_int] = ACTIONS(1551),
    [sym__wc_file_read_line_int] = ACTIONS(1551),
    [sym__wc_file_write_int] = ACTIONS(1551),
    [sym__wc_file_write_line_int] = ACTIONS(1551),
    [anon_sym_EQ] = ACTIONS(1551),
    [aux_sym_preproc_include_token1] = ACTIONS(1551),
    [aux_sym_preproc_define_token1] = ACTIONS(1553),
    [aux_sym_preproc_warn_token1] = ACTIONS(1553),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1553),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1553),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1553),
    [anon_sym_LBRACK] = ACTIONS(1553),
    [anon_sym_LBRACE] = ACTIONS(1553),
    [anon_sym_RBRACE] = ACTIONS(1553),
    [anon_sym_COLON] = ACTIONS(1553),
    [anon_sym_LPAREN] = ACTIONS(1551),
    [anon_sym_SEMI] = ACTIONS(1553),
    [anon_sym_BANG] = ACTIONS(1551),
    [anon_sym_TILDE] = ACTIONS(1553),
    [anon_sym_DASH] = ACTIONS(1551),
    [anon_sym_PLUS] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(1553),
    [anon_sym_SLASH] = ACTIONS(1551),
    [anon_sym_PERCENT] = ACTIONS(1553),
    [anon_sym_PIPE_PIPE] = ACTIONS(1553),
    [anon_sym_AMP_AMP] = ACTIONS(1553),
    [anon_sym_PIPE] = ACTIONS(1551),
    [anon_sym_CARET] = ACTIONS(1553),
    [anon_sym_AMP] = ACTIONS(1551),
    [anon_sym_EQ_EQ] = ACTIONS(1553),
    [anon_sym_BANG_EQ] = ACTIONS(1553),
    [anon_sym_LT_GT] = ACTIONS(1553),
    [anon_sym_GT] = ACTIONS(1551),
    [anon_sym_GT_EQ] = ACTIONS(1553),
    [anon_sym_LT_EQ] = ACTIONS(1553),
    [anon_sym_LT] = ACTIONS(1551),
    [anon_sym_LT_LT] = ACTIONS(1553),
    [anon_sym_GT_GT] = ACTIONS(1553),
    [anon_sym_DASH_DASH] = ACTIONS(1553),
    [anon_sym_PLUS_PLUS] = ACTIONS(1553),
    [anon_sym_DOT] = ACTIONS(1551),
    [anon_sym_DQUOTE] = ACTIONS(1553),
    [anon_sym_SQUOTE] = ACTIONS(1553),
    [sym_number_literal] = ACTIONS(1553),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1553),
  },
  [STATE(178)] = {
    [sym_identifier] = ACTIONS(1543),
    [sym_define_call_keyword] = ACTIONS(1543),
    [sym_define_function_keyword] = ACTIONS(1543),
    [sym_define_library_function_keyword] = ACTIONS(1543),
    [sym_define_module_keyword] = ACTIONS(1543),
    [sym_char_keyword] = ACTIONS(1543),
    [sym_widechar_keyword] = ACTIONS(1543),
    [sym_integer_keyword] = ACTIONS(1543),
    [sym_sinteger_keyword] = ACTIONS(1543),
    [sym_long_keyword] = ACTIONS(1543),
    [sym_slong_keyword] = ACTIONS(1543),
    [sym_float_keyword] = ACTIONS(1543),
    [sym_double_keyword] = ACTIONS(1543),
    [sym_variant_keyword] = ACTIONS(1543),
    [sym_variantarray_keyword] = ACTIONS(1543),
    [sym_local_var_keyword] = ACTIONS(1543),
    [sym_stack_var_keyword] = ACTIONS(1543),
    [sym_constant_keyword] = ACTIONS(1543),
    [sym_volatile_keyword] = ACTIONS(1543),
    [sym_non_volatile_keyword] = ACTIONS(1543),
    [sym_persistent_keyword] = ACTIONS(1543),
    [sym_dev_keyword] = ACTIONS(1543),
    [sym_devlev_keyword] = ACTIONS(1543),
    [sym_devchan_keyword] = ACTIONS(1543),
    [sym_if_keyword] = ACTIONS(1543),
    [sym_else_keyword] = ACTIONS(1543),
    [sym_switch_keyword] = ACTIONS(1543),
    [sym_case_keyword] = ACTIONS(1543),
    [sym_default_keyword] = ACTIONS(1543),
    [sym_while_keyword] = ACTIONS(1543),
    [sym_for_keyword] = ACTIONS(1543),
    [sym_break_keyword] = ACTIONS(1543),
    [sym_continue_keyword] = ACTIONS(1543),
    [sym_return_keyword] = ACTIONS(1543),
    [sym_select_keyword] = ACTIONS(1543),
    [sym_active_keyword] = ACTIONS(1543),
    [sym_struct_keyword] = ACTIONS(1543),
    [sym_structure_keyword] = ACTIONS(1543),
    [aux_sym_on_keyword_token1] = ACTIONS(1543),
    [aux_sym_off_keyword_token1] = ACTIONS(1543),
    [sym_send_level_keyword] = ACTIONS(1543),
    [sym_send_string_keyword] = ACTIONS(1543),
    [sym_send_command_keyword] = ACTIONS(1543),
    [sym_clear_buffer_keyword] = ACTIONS(1543),
    [sym_create_buffer_keyword] = ACTIONS(1543),
    [sym_create_multi_buffer_keyword] = ACTIONS(1543),
    [sym_call_keyword] = ACTIONS(1543),
    [sym_system_call_keyword] = ACTIONS(1543),
    [sym_devchan_to_keyword] = ACTIONS(1543),
    [sym_devchan_min_to_keyword] = ACTIONS(1543),
    [sym_devchan_total_off_keyword] = ACTIONS(1543),
    [sym_devchan_pulse_keyword] = ACTIONS(1543),
    [sym_wait_keyword] = ACTIONS(1543),
    [sym_cancel_wait_keyword] = ACTIONS(1543),
    [sym_wait_until_keyword] = ACTIONS(1543),
    [sym_cancel_wait_until_keyword] = ACTIONS(1543),
    [sym_cancel_all_wait_keyword] = ACTIONS(1543),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1543),
    [sym_and] = ACTIONS(1543),
    [sym_or] = ACTIONS(1543),
    [sym_not] = ACTIONS(1543),
    [sym_xor] = ACTIONS(1543),
    [sym_band] = ACTIONS(1543),
    [sym_bor] = ACTIONS(1543),
    [sym_bxor] = ACTIONS(1543),
    [sym_bnot] = ACTIONS(1543),
    [sym_lshift] = ACTIONS(1543),
    [sym_rshift] = ACTIONS(1543),
    [sym___file__] = ACTIONS(1543),
    [sym___date__] = ACTIONS(1543),
    [sym___ldate__] = ACTIONS(1543),
    [sym___time__] = ACTIONS(1543),
    [sym___line__] = ACTIONS(1543),
    [sym___name__] = ACTIONS(1543),
    [sym___netlinx__] = ACTIONS(1543),
    [sym_netlinx_axi_version] = ACTIONS(1543),
    [sym_url_flg_tcp] = ACTIONS(1543),
    [sym_url_flg_accinfopresent] = ACTIONS(1543),
    [sym_url_flg_temp] = ACTIONS(1543),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1543),
    [sym_url_flg_stat_mask] = ACTIONS(1543),
    [sym_url_flg_stat_lookup] = ACTIONS(1543),
    [sym_url_flg_stat_connecting] = ACTIONS(1543),
    [sym_url_flg_stat_waiting] = ACTIONS(1543),
    [sym_url_flg_stat_connected] = ACTIONS(1543),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1543),
    [sym_first_virtual_device] = ACTIONS(1543),
    [sym_first_local_port] = ACTIONS(1543),
    [sym_dynamic_virtual_device] = ACTIONS(1543),
    [sym_timeline_once] = ACTIONS(1543),
    [sym_timeline_repeat] = ACTIONS(1543),
    [sym_timeline_absolute] = ACTIONS(1543),
    [sym_timeline_relative] = ACTIONS(1543),
    [sym_do_push_timed_infinite] = ACTIONS(1543),
    [sym_source_type_no_address] = ACTIONS(1543),
    [sym_source_type_neuron_id] = ACTIONS(1543),
    [sym_source_type_ip_address] = ACTIONS(1543),
    [sym_source_type_axlink] = ACTIONS(1543),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1543),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1543),
    [sym_source_type_ip_socket_address] = ACTIONS(1543),
    [sym_source_type_rs232] = ACTIONS(1543),
    [sym_source_type_internal] = ACTIONS(1543),
    [sym_source_type_mac_address] = ACTIONS(1543),
    [sym_source_type_ipv4_port] = ACTIONS(1543),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1543),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1543),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1543),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1543),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1543),
    [sym_true] = ACTIONS(1543),
    [sym_false] = ACTIONS(1543),
    [sym_file_read_only] = ACTIONS(1543),
    [sym_file_rw_new] = ACTIONS(1543),
    [sym_file_rw_append] = ACTIONS(1543),
    [sym_ip_tcp] = ACTIONS(1543),
    [sym_ip_udp] = ACTIONS(1543),
    [sym_ip_udp_2way] = ACTIONS(1543),
    [sym_xml_encode_types] = ACTIONS(1543),
    [sym_xml_encode_char_as_list] = ACTIONS(1543),
    [sym_xml_encode_le] = ACTIONS(1543),
    [sym_xml_decode_types] = ACTIONS(1543),
    [sym_xml_decode_no_preserve] = ACTIONS(1543),
    [sym_duet_dev_type_display_device] = ACTIONS(1543),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1543),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1543),
    [sym_duet_dev_type_io_device] = ACTIONS(1543),
    [sym_duet_dev_type_relay_device] = ACTIONS(1543),
    [sym_duet_dev_type_ups] = ACTIONS(1543),
    [sym_duet_dev_type_amplifier] = ACTIONS(1543),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1543),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1543),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1543),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1543),
    [sym_duet_dev_type_camera] = ACTIONS(1543),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1543),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1543),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1543),
    [sym_duet_dev_type_dss] = ACTIONS(1543),
    [sym_duet_dev_type_dvr] = ACTIONS(1543),
    [sym_duet_dev_type_disc_device] = ACTIONS(1543),
    [sym_duet_dev_type_document_camera] = ACTIONS(1543),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1543),
    [sym_duet_dev_type_hvac] = ACTIONS(1543),
    [sym_duet_dev_type_keypad] = ACTIONS(1543),
    [sym_duet_dev_type_light] = ACTIONS(1543),
    [sym_duet_dev_type_monitor] = ACTIONS(1543),
    [sym_duet_dev_type_motor] = ACTIONS(1543),
    [sym_duet_dev_type_multi_window] = ACTIONS(1543),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1543),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1543),
    [sym_duet_dev_type_receiver] = ACTIONS(1543),
    [sym_duet_dev_type_security_system] = ACTIONS(1543),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1543),
    [sym_duet_dev_type_settop_box] = ACTIONS(1543),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1543),
    [sym_duet_dev_type_switcher] = ACTIONS(1543),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1543),
    [sym_duet_dev_type_tv] = ACTIONS(1543),
    [sym_duet_dev_type_utility] = ACTIONS(1543),
    [sym_duet_dev_type_vcr] = ACTIONS(1543),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1543),
    [sym_duet_dev_type_video_processor] = ACTIONS(1543),
    [sym_duet_dev_type_video_projector] = ACTIONS(1543),
    [sym_duet_dev_type_video_wall] = ACTIONS(1543),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1543),
    [sym_duet_dev_type_weather] = ACTIONS(1543),
    [sym_duet_dev_not_polled] = ACTIONS(1543),
    [sym_duet_dev_polled] = ACTIONS(1543),
    [sym_clkmgr_mode_network] = ACTIONS(1543),
    [sym_clkmgr_mode_standalone] = ACTIONS(1543),
    [sym_normal_standby] = ACTIONS(1543),
    [sym_normal_wake] = ACTIONS(1543),
    [sym_ip_multicast_ttl_option] = ACTIONS(1543),
    [sym_ip_ttl_subnet] = ACTIONS(1543),
    [sym_ip_ttl_site] = ACTIONS(1543),
    [sym_ip_ttl_region] = ACTIONS(1543),
    [sym_ip_ttl_continent] = ACTIONS(1543),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1543),
    [sym_ip_nodelay_on] = ACTIONS(1543),
    [sym_ip_nodelay_off] = ACTIONS(1543),
    [sym_amx_error] = ACTIONS(1543),
    [sym_amx_warning] = ACTIONS(1543),
    [sym_amx_info] = ACTIONS(1543),
    [sym_amx_debug] = ACTIONS(1543),
    [sym_netlinx_login_success] = ACTIONS(1543),
    [sym_netlinx_login_fail] = ACTIONS(1543),
    [sym_netlinx_logout] = ACTIONS(1543),
    [sym_null_str] = ACTIONS(1543),
    [sym_smtp_address] = ACTIONS(1543),
    [sym_smtp_port_number] = ACTIONS(1543),
    [sym_smtp_username] = ACTIONS(1543),
    [sym_smtp_password] = ACTIONS(1543),
    [sym_smtp_require_tls] = ACTIONS(1543),
    [sym_smtp_from] = ACTIONS(1543),
    [sym_smtp_tls_true] = ACTIONS(1543),
    [sym_smtp_tls_false] = ACTIONS(1543),
    [sym_listview_on_row_select_event] = ACTIONS(1543),
    [sym_data_structure_unknown] = ACTIONS(1543),
    [sym_data_structure_datafeed] = ACTIONS(1543),
    [sym_data_structure_datarecordset] = ACTIONS(1543),
    [sym_data_structure_datarecord] = ACTIONS(1543),
    [sym_data_structure_datafield] = ACTIONS(1543),
    [sym_data_max_content_field_count] = ACTIONS(1543),
    [sym_data_max_metadata_field_count] = ACTIONS(1543),
    [sym_data_max_id_length] = ACTIONS(1543),
    [sym_data_max_name_length] = ACTIONS(1543),
    [sym_data_max_description_length] = ACTIONS(1543),
    [sym_data_max_type_length] = ACTIONS(1543),
    [sym_data_max_format_length] = ACTIONS(1543),
    [sym_data_max_source_length] = ACTIONS(1543),
    [sym_data_max_value_length] = ACTIONS(1543),
    [sym_data_max_label_length] = ACTIONS(1543),
    [sym_data_max_url_length] = ACTIONS(1543),
    [sym_max_last_login_info_length] = ACTIONS(1543),
    [sym_data_type_unknown] = ACTIONS(1543),
    [sym_data_type_string] = ACTIONS(1543),
    [sym_data_type_datetime] = ACTIONS(1543),
    [sym_data_type_date] = ACTIONS(1543),
    [sym_data_type_time] = ACTIONS(1543),
    [sym_data_type_image] = ACTIONS(1543),
    [sym_data_format_url] = ACTIONS(1543),
    [sym_data_format_phone] = ACTIONS(1543),
    [sym_data_format_email] = ACTIONS(1543),
    [sym_data_format_iso8601] = ACTIONS(1543),
    [sym_auditlog_priv] = ACTIONS(1543),
    [sym_configuration_priv] = ACTIONS(1543),
    [sym_deviceconfig_priv] = ACTIONS(1543),
    [sym_ftp_priv] = ACTIONS(1543),
    [sym_http_priv] = ACTIONS(1543),
    [sym_networkconfig_priv] = ACTIONS(1543),
    [sym_programport_priv] = ACTIONS(1543),
    [sym_remoteui_priv] = ACTIONS(1543),
    [sym_securitycontrol_priv] = ACTIONS(1543),
    [sym_softwaremanagement_priv] = ACTIONS(1543),
    [sym_terminal_priv] = ACTIONS(1543),
    [sym_tpadmin_priv] = ACTIONS(1543),
    [sym_user1_priv] = ACTIONS(1543),
    [sym_user2_priv] = ACTIONS(1543),
    [sym_user3_priv] = ACTIONS(1543),
    [sym_user4_priv] = ACTIONS(1543),
    [sym_usermanagement_priv] = ACTIONS(1543),
    [sym_valid_account] = ACTIONS(1543),
    [sym_err_name_invalid] = ACTIONS(1543),
    [sym_err_password_invalid] = ACTIONS(1543),
    [sym_err_invalid_account] = ACTIONS(1543),
    [sym_err_no_authorization] = ACTIONS(1543),
    [sym_err_invalid_parameter] = ACTIONS(1543),
    [sym_err_user_locked_out] = ACTIONS(1543),
    [sym_err_user_account_expired] = ACTIONS(1543),
    [sym_err_general_error] = ACTIONS(1543),
    [sym_err_authentication_error] = ACTIONS(1543),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1543),
    [sym_tls_validate_certificate] = ACTIONS(1543),
    [sym_tdata] = ACTIONS(1543),
    [sym_tchannel] = ACTIONS(1543),
    [sym_tlevel] = ACTIONS(1543),
    [sym_tbutton] = ACTIONS(1543),
    [sym_ttimeline] = ACTIONS(1543),
    [sym_tcustom] = ACTIONS(1543),
    [sym_url_struct] = ACTIONS(1543),
    [sym_dns_struct] = ACTIONS(1543),
    [sym_ip_address_struct] = ACTIONS(1543),
    [sym_dev_info_struct] = ACTIONS(1543),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1543),
    [sym_data_feed] = ACTIONS(1543),
    [sym_data_field] = ACTIONS(1543),
    [sym_data_record] = ACTIONS(1543),
    [sym_wc_data_feed] = ACTIONS(1543),
    [sym_wc_data_field] = ACTIONS(1543),
    [sym_wc_data_record] = ACTIONS(1543),
    [sym_last_login_info] = ACTIONS(1543),
    [sym____reserved___] = ACTIONS(1543),
    [sym_date] = ACTIONS(1543),
    [sym_day] = ACTIONS(1543),
    [sym_time] = ACTIONS(1543),
    [sym_ldate] = ACTIONS(1543),
    [sym_push_device] = ACTIONS(1543),
    [sym_push_channel] = ACTIONS(1543),
    [sym_push_devchan] = ACTIONS(1543),
    [sym_release_device] = ACTIONS(1543),
    [sym_release_channel] = ACTIONS(1543),
    [sym_release_devchan] = ACTIONS(1543),
    [sym_master_slot] = ACTIONS(1543),
    [sym_get_pulse_time] = ACTIONS(1543),
    [sym_get_timer] = ACTIONS(1543),
    [sym_master_sn] = ACTIONS(1543),
    [sym_system_number] = ACTIONS(1543),
    [sym_button] = ACTIONS(1543),
    [sym_dv_channel] = ACTIONS(1543),
    [sym_data] = ACTIONS(1543),
    [sym_level] = ACTIONS(1543),
    [sym_timeline] = ACTIONS(1543),
    [sym_channel] = ACTIONS(1543),
    [sym_custom] = ACTIONS(1543),
    [sym_atoi] = ACTIONS(1543),
    [sym_atol] = ACTIONS(1543),
    [sym_atof] = ACTIONS(1543),
    [sym_itoa] = ACTIONS(1543),
    [sym_itohex] = ACTIONS(1543),
    [sym_hextoi] = ACTIONS(1543),
    [sym_ftoa] = ACTIONS(1543),
    [sym_find_string] = ACTIONS(1543),
    [sym_left_string] = ACTIONS(1543),
    [sym_right_string] = ACTIONS(1543),
    [sym_mid_string] = ACTIONS(1543),
    [sym_remove_string] = ACTIONS(1543),
    [sym_get_buffer_string] = ACTIONS(1543),
    [sym_get_buffer_char] = ACTIONS(1543),
    [sym_get_multi_buffer_string] = ACTIONS(1543),
    [sym_length_string] = ACTIONS(1543),
    [sym_max_length_string] = ACTIONS(1543),
    [sym_set_length_string] = ACTIONS(1543),
    [sym_compare_string] = ACTIONS(1543),
    [sym_upper_string] = ACTIONS(1543),
    [sym_lower_string] = ACTIONS(1543),
    [sym_get_last] = ACTIONS(1543),
    [sym_date_to_day] = ACTIONS(1543),
    [sym_date_to_month] = ACTIONS(1543),
    [sym_date_to_year] = ACTIONS(1543),
    [sym_time_to_hour] = ACTIONS(1543),
    [sym_time_to_minute] = ACTIONS(1543),
    [sym_time_to_second] = ACTIONS(1543),
    [sym_day_of_week] = ACTIONS(1543),
    [sym_set_pulse_time] = ACTIONS(1543),
    [sym_set_timer] = ACTIONS(1543),
    [sym_file_open] = ACTIONS(1543),
    [sym_file_close] = ACTIONS(1543),
    [sym_file_read] = ACTIONS(1543),
    [sym_file_read_line] = ACTIONS(1543),
    [sym_file_write] = ACTIONS(1543),
    [sym_file_write_line] = ACTIONS(1543),
    [sym_file_copy] = ACTIONS(1543),
    [sym_file_delete] = ACTIONS(1543),
    [sym_file_rename] = ACTIONS(1543),
    [sym_file_dir] = ACTIONS(1543),
    [sym_file_getdir] = ACTIONS(1543),
    [sym_file_setdir] = ACTIONS(1543),
    [sym_file_create_dir] = ACTIONS(1543),
    [sym_file_remove_dir] = ACTIONS(1543),
    [sym_file_seek] = ACTIONS(1543),
    [sym_length_array] = ACTIONS(1543),
    [sym_max_length_array] = ACTIONS(1543),
    [sym_set_length_array] = ACTIONS(1543),
    [sym_ip_client_open] = ACTIONS(1543),
    [sym_ip_client_close] = ACTIONS(1543),
    [sym_ip_server_open] = ACTIONS(1543),
    [sym_ip_server_close] = ACTIONS(1543),
    [sym_ip_bound_client_open] = ACTIONS(1543),
    [sym_ip_mc_server_open] = ACTIONS(1543),
    [sym_ip_set_option] = ACTIONS(1543),
    [sym_ssh_client_open] = ACTIONS(1543),
    [sym_ssh_client_close] = ACTIONS(1543),
    [sym_tls_client_open] = ACTIONS(1543),
    [sym_tls_client_close] = ACTIONS(1543),
    [sym_do_push] = ACTIONS(1543),
    [sym_do_push_timed] = ACTIONS(1543),
    [sym_do_release] = ACTIONS(1543),
    [sym_device_id] = ACTIONS(1543),
    [sym_device_info] = ACTIONS(1543),
    [sym_device_id_string] = ACTIONS(1543),
    [sym_device_standby] = ACTIONS(1543),
    [sym_device_wake] = ACTIONS(1543),
    [sym_timeline_create] = ACTIONS(1543),
    [sym_timeline_kill] = ACTIONS(1543),
    [sym_timeline_reload] = ACTIONS(1543),
    [sym_timeline_pause] = ACTIONS(1543),
    [sym_timeline_restart] = ACTIONS(1543),
    [sym_timeline_set] = ACTIONS(1543),
    [sym_timeline_get] = ACTIONS(1543),
    [sym_timeline_active] = ACTIONS(1543),
    [sym_timeline_dynamic_id] = ACTIONS(1543),
    [sym_variable_to_string] = ACTIONS(1543),
    [sym_string_to_variable] = ACTIONS(1543),
    [sym_length_variable_to_string] = ACTIONS(1543),
    [sym_type_cast] = ACTIONS(1543),
    [sym_variable_to_xml] = ACTIONS(1543),
    [sym_xml_to_variable] = ACTIONS(1543),
    [sym_length_variable_to_xml] = ACTIONS(1543),
    [sym_redirect_string] = ACTIONS(1543),
    [sym_random_number] = ACTIONS(1543),
    [sym_get_unique_id] = ACTIONS(1543),
    [sym_get_system_number] = ACTIONS(1543),
    [sym_set_system_number] = ACTIONS(1543),
    [sym_get_serial_number] = ACTIONS(1543),
    [sym_get_master_build] = ACTIONS(1543),
    [sym_set_validation_code] = ACTIONS(1543),
    [sym_reboot] = ACTIONS(1543),
    [sym_load_duet_module] = ACTIONS(1543),
    [sym_unload_duet_module] = ACTIONS(1543),
    [sym_duet_mem_size_set] = ACTIONS(1543),
    [sym_duet_mem_size_get] = ACTIONS(1543),
    [sym_dynamic_application_device] = ACTIONS(1543),
    [sym_dynamic_polled_port] = ACTIONS(1543),
    [sym_static_port_binding] = ACTIONS(1543),
    [sym_static_ip_binding] = ACTIONS(1543),
    [sym_get_dns_list] = ACTIONS(1543),
    [sym_set_dns_list] = ACTIONS(1543),
    [sym_get_ip_address] = ACTIONS(1543),
    [sym_set_ip_address] = ACTIONS(1543),
    [sym_get_url_list] = ACTIONS(1543),
    [sym_add_url_entry] = ACTIONS(1543),
    [sym_delete_url_entry] = ACTIONS(1543),
    [sym_abs_value] = ACTIONS(1543),
    [sym_max_value] = ACTIONS(1543),
    [sym_min_value] = ACTIONS(1543),
    [sym_exp_value] = ACTIONS(1543),
    [sym_log_value] = ACTIONS(1543),
    [sym_log10_value] = ACTIONS(1543),
    [sym_power_value] = ACTIONS(1543),
    [sym_sqrt_value] = ACTIONS(1543),
    [sym_raw_be] = ACTIONS(1543),
    [sym_raw_le] = ACTIONS(1543),
    [sym_internal_queue_size_set] = ACTIONS(1543),
    [sym_internal_queue_size_get] = ACTIONS(1543),
    [sym_internal_threshold_set] = ACTIONS(1543),
    [sym_internal_threshold_get] = ACTIONS(1543),
    [sym_rebuilt_event] = ACTIONS(1543),
    [sym_do_custom_event] = ACTIONS(1543),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1543),
    [sym_clkmgr_set_clk_source] = ACTIONS(1543),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1543),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1543),
    [sym_clkmgr_get_timezone] = ACTIONS(1543),
    [sym_clkmgr_set_timezone] = ACTIONS(1543),
    [sym_clkmgr_get_resync_period] = ACTIONS(1543),
    [sym_clkmgr_set_resync_period] = ACTIONS(1543),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1543),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1543),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1543),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1543),
    [sym_clkmgr_get_timeservers] = ACTIONS(1543),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1543),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1543),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1543),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1543),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1543),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1543),
    [sym_set_log_level] = ACTIONS(1543),
    [sym_get_log_level] = ACTIONS(1543),
    [sym_amx_log] = ACTIONS(1543),
    [sym_smtp_server_config_set] = ACTIONS(1543),
    [sym_smtp_server_config_get] = ACTIONS(1543),
    [sym_smtp_send] = ACTIONS(1543),
    [sym_validate_netlinx_account] = ACTIONS(1543),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1543),
    [sym_authenticate_certificate] = ACTIONS(1543),
    [sym_audit_netlinx_session_event] = ACTIONS(1543),
    [sym_audit_netlinx_generic_event] = ACTIONS(1543),
    [sym_get_available_flash_disk_space] = ACTIONS(1543),
    [sym_get_max_flash_disk_space] = ACTIONS(1543),
    [sym_data_create_feed] = ACTIONS(1543),
    [sym_data_delete_feed] = ACTIONS(1543),
    [sym_data_publish_feed] = ACTIONS(1543),
    [sym_data_get_published_feed] = ACTIONS(1543),
    [sym_data_add_record] = ACTIONS(1543),
    [sym_data_get_event_record] = ACTIONS(1543),
    [sym__wc_data_create_feed] = ACTIONS(1543),
    [sym__wc_data_add_record] = ACTIONS(1543),
    [sym__wc_data_get_event_record] = ACTIONS(1543),
    [sym_format] = ACTIONS(1543),
    [sym_astro_clock] = ACTIONS(1543),
    [sym_set_outdoor_temperature] = ACTIONS(1543),
    [sym_set_virtual_level_count] = ACTIONS(1543),
    [sym_set_virtual_channel_count] = ACTIONS(1543),
    [sym_set_virtual_port_count] = ACTIONS(1543),
    [sym__wc_encode_int] = ACTIONS(1543),
    [sym__wc_decode_int] = ACTIONS(1543),
    [sym__wc_int] = ACTIONS(1543),
    [sym__wc_to_ch_int] = ACTIONS(1543),
    [sym__ch_to_wc_int] = ACTIONS(1543),
    [sym__wc_find_string_int] = ACTIONS(1543),
    [sym__wc_left_string_int] = ACTIONS(1543),
    [sym__wc_lower_string_int] = ACTIONS(1543),
    [sym__wc_mid_string_int] = ACTIONS(1543),
    [sym__wc_remove_string_int] = ACTIONS(1543),
    [sym__wc_right_string_int] = ACTIONS(1543),
    [sym__wc_upper_string_int] = ACTIONS(1543),
    [sym__wc_compare_string_int] = ACTIONS(1543),
    [sym__wc_get_buffer_char_int] = ACTIONS(1543),
    [sym__wc_get_buffer_string_int] = ACTIONS(1543),
    [sym__wc_concat_string_int] = ACTIONS(1543),
    [sym__wc_file_open_int] = ACTIONS(1543),
    [sym__wc_file_close_int] = ACTIONS(1543),
    [sym__wc_file_read_int] = ACTIONS(1543),
    [sym__wc_file_read_line_int] = ACTIONS(1543),
    [sym__wc_file_write_int] = ACTIONS(1543),
    [sym__wc_file_write_line_int] = ACTIONS(1543),
    [anon_sym_EQ] = ACTIONS(1543),
    [aux_sym_preproc_include_token1] = ACTIONS(1543),
    [aux_sym_preproc_define_token1] = ACTIONS(1545),
    [aux_sym_preproc_warn_token1] = ACTIONS(1545),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1545),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1545),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1545),
    [anon_sym_LBRACK] = ACTIONS(1545),
    [anon_sym_LBRACE] = ACTIONS(1545),
    [anon_sym_RBRACE] = ACTIONS(1545),
    [anon_sym_COLON] = ACTIONS(1545),
    [anon_sym_LPAREN] = ACTIONS(1543),
    [anon_sym_SEMI] = ACTIONS(1545),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1545),
    [anon_sym_DASH] = ACTIONS(1543),
    [anon_sym_PLUS] = ACTIONS(1543),
    [anon_sym_STAR] = ACTIONS(1545),
    [anon_sym_SLASH] = ACTIONS(1543),
    [anon_sym_PERCENT] = ACTIONS(1545),
    [anon_sym_PIPE_PIPE] = ACTIONS(1545),
    [anon_sym_AMP_AMP] = ACTIONS(1545),
    [anon_sym_PIPE] = ACTIONS(1543),
    [anon_sym_CARET] = ACTIONS(1545),
    [anon_sym_AMP] = ACTIONS(1543),
    [anon_sym_EQ_EQ] = ACTIONS(1545),
    [anon_sym_BANG_EQ] = ACTIONS(1545),
    [anon_sym_LT_GT] = ACTIONS(1545),
    [anon_sym_GT] = ACTIONS(1543),
    [anon_sym_GT_EQ] = ACTIONS(1545),
    [anon_sym_LT_EQ] = ACTIONS(1545),
    [anon_sym_LT] = ACTIONS(1543),
    [anon_sym_LT_LT] = ACTIONS(1545),
    [anon_sym_GT_GT] = ACTIONS(1545),
    [anon_sym_DASH_DASH] = ACTIONS(1545),
    [anon_sym_PLUS_PLUS] = ACTIONS(1545),
    [anon_sym_DOT] = ACTIONS(1543),
    [anon_sym_DQUOTE] = ACTIONS(1545),
    [anon_sym_SQUOTE] = ACTIONS(1545),
    [sym_number_literal] = ACTIONS(1545),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1545),
  },
  [STATE(179)] = {
    [sym_identifier] = ACTIONS(1555),
    [sym_define_call_keyword] = ACTIONS(1555),
    [sym_define_function_keyword] = ACTIONS(1555),
    [sym_define_library_function_keyword] = ACTIONS(1555),
    [sym_define_module_keyword] = ACTIONS(1555),
    [sym_char_keyword] = ACTIONS(1555),
    [sym_widechar_keyword] = ACTIONS(1555),
    [sym_integer_keyword] = ACTIONS(1555),
    [sym_sinteger_keyword] = ACTIONS(1555),
    [sym_long_keyword] = ACTIONS(1555),
    [sym_slong_keyword] = ACTIONS(1555),
    [sym_float_keyword] = ACTIONS(1555),
    [sym_double_keyword] = ACTIONS(1555),
    [sym_variant_keyword] = ACTIONS(1555),
    [sym_variantarray_keyword] = ACTIONS(1555),
    [sym_local_var_keyword] = ACTIONS(1555),
    [sym_stack_var_keyword] = ACTIONS(1555),
    [sym_constant_keyword] = ACTIONS(1555),
    [sym_volatile_keyword] = ACTIONS(1555),
    [sym_non_volatile_keyword] = ACTIONS(1555),
    [sym_persistent_keyword] = ACTIONS(1555),
    [sym_dev_keyword] = ACTIONS(1555),
    [sym_devlev_keyword] = ACTIONS(1555),
    [sym_devchan_keyword] = ACTIONS(1555),
    [sym_if_keyword] = ACTIONS(1555),
    [sym_else_keyword] = ACTIONS(1555),
    [sym_switch_keyword] = ACTIONS(1555),
    [sym_case_keyword] = ACTIONS(1555),
    [sym_default_keyword] = ACTIONS(1555),
    [sym_while_keyword] = ACTIONS(1555),
    [sym_for_keyword] = ACTIONS(1555),
    [sym_break_keyword] = ACTIONS(1555),
    [sym_continue_keyword] = ACTIONS(1555),
    [sym_return_keyword] = ACTIONS(1555),
    [sym_select_keyword] = ACTIONS(1555),
    [sym_active_keyword] = ACTIONS(1555),
    [sym_struct_keyword] = ACTIONS(1555),
    [sym_structure_keyword] = ACTIONS(1555),
    [aux_sym_on_keyword_token1] = ACTIONS(1555),
    [aux_sym_off_keyword_token1] = ACTIONS(1555),
    [sym_send_level_keyword] = ACTIONS(1555),
    [sym_send_string_keyword] = ACTIONS(1555),
    [sym_send_command_keyword] = ACTIONS(1555),
    [sym_clear_buffer_keyword] = ACTIONS(1555),
    [sym_create_buffer_keyword] = ACTIONS(1555),
    [sym_create_multi_buffer_keyword] = ACTIONS(1555),
    [sym_call_keyword] = ACTIONS(1555),
    [sym_system_call_keyword] = ACTIONS(1555),
    [sym_devchan_to_keyword] = ACTIONS(1555),
    [sym_devchan_min_to_keyword] = ACTIONS(1555),
    [sym_devchan_total_off_keyword] = ACTIONS(1555),
    [sym_devchan_pulse_keyword] = ACTIONS(1555),
    [sym_wait_keyword] = ACTIONS(1555),
    [sym_cancel_wait_keyword] = ACTIONS(1555),
    [sym_wait_until_keyword] = ACTIONS(1555),
    [sym_cancel_wait_until_keyword] = ACTIONS(1555),
    [sym_cancel_all_wait_keyword] = ACTIONS(1555),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1555),
    [sym_and] = ACTIONS(1555),
    [sym_or] = ACTIONS(1555),
    [sym_not] = ACTIONS(1555),
    [sym_xor] = ACTIONS(1555),
    [sym_band] = ACTIONS(1555),
    [sym_bor] = ACTIONS(1555),
    [sym_bxor] = ACTIONS(1555),
    [sym_bnot] = ACTIONS(1555),
    [sym_lshift] = ACTIONS(1555),
    [sym_rshift] = ACTIONS(1555),
    [sym___file__] = ACTIONS(1555),
    [sym___date__] = ACTIONS(1555),
    [sym___ldate__] = ACTIONS(1555),
    [sym___time__] = ACTIONS(1555),
    [sym___line__] = ACTIONS(1555),
    [sym___name__] = ACTIONS(1555),
    [sym___netlinx__] = ACTIONS(1555),
    [sym_netlinx_axi_version] = ACTIONS(1555),
    [sym_url_flg_tcp] = ACTIONS(1555),
    [sym_url_flg_accinfopresent] = ACTIONS(1555),
    [sym_url_flg_temp] = ACTIONS(1555),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1555),
    [sym_url_flg_stat_mask] = ACTIONS(1555),
    [sym_url_flg_stat_lookup] = ACTIONS(1555),
    [sym_url_flg_stat_connecting] = ACTIONS(1555),
    [sym_url_flg_stat_waiting] = ACTIONS(1555),
    [sym_url_flg_stat_connected] = ACTIONS(1555),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1555),
    [sym_first_virtual_device] = ACTIONS(1555),
    [sym_first_local_port] = ACTIONS(1555),
    [sym_dynamic_virtual_device] = ACTIONS(1555),
    [sym_timeline_once] = ACTIONS(1555),
    [sym_timeline_repeat] = ACTIONS(1555),
    [sym_timeline_absolute] = ACTIONS(1555),
    [sym_timeline_relative] = ACTIONS(1555),
    [sym_do_push_timed_infinite] = ACTIONS(1555),
    [sym_source_type_no_address] = ACTIONS(1555),
    [sym_source_type_neuron_id] = ACTIONS(1555),
    [sym_source_type_ip_address] = ACTIONS(1555),
    [sym_source_type_axlink] = ACTIONS(1555),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1555),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1555),
    [sym_source_type_ip_socket_address] = ACTIONS(1555),
    [sym_source_type_rs232] = ACTIONS(1555),
    [sym_source_type_internal] = ACTIONS(1555),
    [sym_source_type_mac_address] = ACTIONS(1555),
    [sym_source_type_ipv4_port] = ACTIONS(1555),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1555),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1555),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1555),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1555),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1555),
    [sym_true] = ACTIONS(1555),
    [sym_false] = ACTIONS(1555),
    [sym_file_read_only] = ACTIONS(1555),
    [sym_file_rw_new] = ACTIONS(1555),
    [sym_file_rw_append] = ACTIONS(1555),
    [sym_ip_tcp] = ACTIONS(1555),
    [sym_ip_udp] = ACTIONS(1555),
    [sym_ip_udp_2way] = ACTIONS(1555),
    [sym_xml_encode_types] = ACTIONS(1555),
    [sym_xml_encode_char_as_list] = ACTIONS(1555),
    [sym_xml_encode_le] = ACTIONS(1555),
    [sym_xml_decode_types] = ACTIONS(1555),
    [sym_xml_decode_no_preserve] = ACTIONS(1555),
    [sym_duet_dev_type_display_device] = ACTIONS(1555),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1555),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1555),
    [sym_duet_dev_type_io_device] = ACTIONS(1555),
    [sym_duet_dev_type_relay_device] = ACTIONS(1555),
    [sym_duet_dev_type_ups] = ACTIONS(1555),
    [sym_duet_dev_type_amplifier] = ACTIONS(1555),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1555),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1555),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1555),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1555),
    [sym_duet_dev_type_camera] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1555),
    [sym_duet_dev_type_dss] = ACTIONS(1555),
    [sym_duet_dev_type_dvr] = ACTIONS(1555),
    [sym_duet_dev_type_disc_device] = ACTIONS(1555),
    [sym_duet_dev_type_document_camera] = ACTIONS(1555),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1555),
    [sym_duet_dev_type_hvac] = ACTIONS(1555),
    [sym_duet_dev_type_keypad] = ACTIONS(1555),
    [sym_duet_dev_type_light] = ACTIONS(1555),
    [sym_duet_dev_type_monitor] = ACTIONS(1555),
    [sym_duet_dev_type_motor] = ACTIONS(1555),
    [sym_duet_dev_type_multi_window] = ACTIONS(1555),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1555),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1555),
    [sym_duet_dev_type_receiver] = ACTIONS(1555),
    [sym_duet_dev_type_security_system] = ACTIONS(1555),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1555),
    [sym_duet_dev_type_settop_box] = ACTIONS(1555),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1555),
    [sym_duet_dev_type_switcher] = ACTIONS(1555),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1555),
    [sym_duet_dev_type_tv] = ACTIONS(1555),
    [sym_duet_dev_type_utility] = ACTIONS(1555),
    [sym_duet_dev_type_vcr] = ACTIONS(1555),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1555),
    [sym_duet_dev_type_video_processor] = ACTIONS(1555),
    [sym_duet_dev_type_video_projector] = ACTIONS(1555),
    [sym_duet_dev_type_video_wall] = ACTIONS(1555),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1555),
    [sym_duet_dev_type_weather] = ACTIONS(1555),
    [sym_duet_dev_not_polled] = ACTIONS(1555),
    [sym_duet_dev_polled] = ACTIONS(1555),
    [sym_clkmgr_mode_network] = ACTIONS(1555),
    [sym_clkmgr_mode_standalone] = ACTIONS(1555),
    [sym_normal_standby] = ACTIONS(1555),
    [sym_normal_wake] = ACTIONS(1555),
    [sym_ip_multicast_ttl_option] = ACTIONS(1555),
    [sym_ip_ttl_subnet] = ACTIONS(1555),
    [sym_ip_ttl_site] = ACTIONS(1555),
    [sym_ip_ttl_region] = ACTIONS(1555),
    [sym_ip_ttl_continent] = ACTIONS(1555),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1555),
    [sym_ip_nodelay_on] = ACTIONS(1555),
    [sym_ip_nodelay_off] = ACTIONS(1555),
    [sym_amx_error] = ACTIONS(1555),
    [sym_amx_warning] = ACTIONS(1555),
    [sym_amx_info] = ACTIONS(1555),
    [sym_amx_debug] = ACTIONS(1555),
    [sym_netlinx_login_success] = ACTIONS(1555),
    [sym_netlinx_login_fail] = ACTIONS(1555),
    [sym_netlinx_logout] = ACTIONS(1555),
    [sym_null_str] = ACTIONS(1555),
    [sym_smtp_address] = ACTIONS(1555),
    [sym_smtp_port_number] = ACTIONS(1555),
    [sym_smtp_username] = ACTIONS(1555),
    [sym_smtp_password] = ACTIONS(1555),
    [sym_smtp_require_tls] = ACTIONS(1555),
    [sym_smtp_from] = ACTIONS(1555),
    [sym_smtp_tls_true] = ACTIONS(1555),
    [sym_smtp_tls_false] = ACTIONS(1555),
    [sym_listview_on_row_select_event] = ACTIONS(1555),
    [sym_data_structure_unknown] = ACTIONS(1555),
    [sym_data_structure_datafeed] = ACTIONS(1555),
    [sym_data_structure_datarecordset] = ACTIONS(1555),
    [sym_data_structure_datarecord] = ACTIONS(1555),
    [sym_data_structure_datafield] = ACTIONS(1555),
    [sym_data_max_content_field_count] = ACTIONS(1555),
    [sym_data_max_metadata_field_count] = ACTIONS(1555),
    [sym_data_max_id_length] = ACTIONS(1555),
    [sym_data_max_name_length] = ACTIONS(1555),
    [sym_data_max_description_length] = ACTIONS(1555),
    [sym_data_max_type_length] = ACTIONS(1555),
    [sym_data_max_format_length] = ACTIONS(1555),
    [sym_data_max_source_length] = ACTIONS(1555),
    [sym_data_max_value_length] = ACTIONS(1555),
    [sym_data_max_label_length] = ACTIONS(1555),
    [sym_data_max_url_length] = ACTIONS(1555),
    [sym_max_last_login_info_length] = ACTIONS(1555),
    [sym_data_type_unknown] = ACTIONS(1555),
    [sym_data_type_string] = ACTIONS(1555),
    [sym_data_type_datetime] = ACTIONS(1555),
    [sym_data_type_date] = ACTIONS(1555),
    [sym_data_type_time] = ACTIONS(1555),
    [sym_data_type_image] = ACTIONS(1555),
    [sym_data_format_url] = ACTIONS(1555),
    [sym_data_format_phone] = ACTIONS(1555),
    [sym_data_format_email] = ACTIONS(1555),
    [sym_data_format_iso8601] = ACTIONS(1555),
    [sym_auditlog_priv] = ACTIONS(1555),
    [sym_configuration_priv] = ACTIONS(1555),
    [sym_deviceconfig_priv] = ACTIONS(1555),
    [sym_ftp_priv] = ACTIONS(1555),
    [sym_http_priv] = ACTIONS(1555),
    [sym_networkconfig_priv] = ACTIONS(1555),
    [sym_programport_priv] = ACTIONS(1555),
    [sym_remoteui_priv] = ACTIONS(1555),
    [sym_securitycontrol_priv] = ACTIONS(1555),
    [sym_softwaremanagement_priv] = ACTIONS(1555),
    [sym_terminal_priv] = ACTIONS(1555),
    [sym_tpadmin_priv] = ACTIONS(1555),
    [sym_user1_priv] = ACTIONS(1555),
    [sym_user2_priv] = ACTIONS(1555),
    [sym_user3_priv] = ACTIONS(1555),
    [sym_user4_priv] = ACTIONS(1555),
    [sym_usermanagement_priv] = ACTIONS(1555),
    [sym_valid_account] = ACTIONS(1555),
    [sym_err_name_invalid] = ACTIONS(1555),
    [sym_err_password_invalid] = ACTIONS(1555),
    [sym_err_invalid_account] = ACTIONS(1555),
    [sym_err_no_authorization] = ACTIONS(1555),
    [sym_err_invalid_parameter] = ACTIONS(1555),
    [sym_err_user_locked_out] = ACTIONS(1555),
    [sym_err_user_account_expired] = ACTIONS(1555),
    [sym_err_general_error] = ACTIONS(1555),
    [sym_err_authentication_error] = ACTIONS(1555),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1555),
    [sym_tls_validate_certificate] = ACTIONS(1555),
    [sym_tdata] = ACTIONS(1555),
    [sym_tchannel] = ACTIONS(1555),
    [sym_tlevel] = ACTIONS(1555),
    [sym_tbutton] = ACTIONS(1555),
    [sym_ttimeline] = ACTIONS(1555),
    [sym_tcustom] = ACTIONS(1555),
    [sym_url_struct] = ACTIONS(1555),
    [sym_dns_struct] = ACTIONS(1555),
    [sym_ip_address_struct] = ACTIONS(1555),
    [sym_dev_info_struct] = ACTIONS(1555),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1555),
    [sym_data_feed] = ACTIONS(1555),
    [sym_data_field] = ACTIONS(1555),
    [sym_data_record] = ACTIONS(1555),
    [sym_wc_data_feed] = ACTIONS(1555),
    [sym_wc_data_field] = ACTIONS(1555),
    [sym_wc_data_record] = ACTIONS(1555),
    [sym_last_login_info] = ACTIONS(1555),
    [sym____reserved___] = ACTIONS(1555),
    [sym_date] = ACTIONS(1555),
    [sym_day] = ACTIONS(1555),
    [sym_time] = ACTIONS(1555),
    [sym_ldate] = ACTIONS(1555),
    [sym_push_device] = ACTIONS(1555),
    [sym_push_channel] = ACTIONS(1555),
    [sym_push_devchan] = ACTIONS(1555),
    [sym_release_device] = ACTIONS(1555),
    [sym_release_channel] = ACTIONS(1555),
    [sym_release_devchan] = ACTIONS(1555),
    [sym_master_slot] = ACTIONS(1555),
    [sym_get_pulse_time] = ACTIONS(1555),
    [sym_get_timer] = ACTIONS(1555),
    [sym_master_sn] = ACTIONS(1555),
    [sym_system_number] = ACTIONS(1555),
    [sym_button] = ACTIONS(1555),
    [sym_dv_channel] = ACTIONS(1555),
    [sym_data] = ACTIONS(1555),
    [sym_level] = ACTIONS(1555),
    [sym_timeline] = ACTIONS(1555),
    [sym_channel] = ACTIONS(1555),
    [sym_custom] = ACTIONS(1555),
    [sym_atoi] = ACTIONS(1555),
    [sym_atol] = ACTIONS(1555),
    [sym_atof] = ACTIONS(1555),
    [sym_itoa] = ACTIONS(1555),
    [sym_itohex] = ACTIONS(1555),
    [sym_hextoi] = ACTIONS(1555),
    [sym_ftoa] = ACTIONS(1555),
    [sym_find_string] = ACTIONS(1555),
    [sym_left_string] = ACTIONS(1555),
    [sym_right_string] = ACTIONS(1555),
    [sym_mid_string] = ACTIONS(1555),
    [sym_remove_string] = ACTIONS(1555),
    [sym_get_buffer_string] = ACTIONS(1555),
    [sym_get_buffer_char] = ACTIONS(1555),
    [sym_get_multi_buffer_string] = ACTIONS(1555),
    [sym_length_string] = ACTIONS(1555),
    [sym_max_length_string] = ACTIONS(1555),
    [sym_set_length_string] = ACTIONS(1555),
    [sym_compare_string] = ACTIONS(1555),
    [sym_upper_string] = ACTIONS(1555),
    [sym_lower_string] = ACTIONS(1555),
    [sym_get_last] = ACTIONS(1555),
    [sym_date_to_day] = ACTIONS(1555),
    [sym_date_to_month] = ACTIONS(1555),
    [sym_date_to_year] = ACTIONS(1555),
    [sym_time_to_hour] = ACTIONS(1555),
    [sym_time_to_minute] = ACTIONS(1555),
    [sym_time_to_second] = ACTIONS(1555),
    [sym_day_of_week] = ACTIONS(1555),
    [sym_set_pulse_time] = ACTIONS(1555),
    [sym_set_timer] = ACTIONS(1555),
    [sym_file_open] = ACTIONS(1555),
    [sym_file_close] = ACTIONS(1555),
    [sym_file_read] = ACTIONS(1555),
    [sym_file_read_line] = ACTIONS(1555),
    [sym_file_write] = ACTIONS(1555),
    [sym_file_write_line] = ACTIONS(1555),
    [sym_file_copy] = ACTIONS(1555),
    [sym_file_delete] = ACTIONS(1555),
    [sym_file_rename] = ACTIONS(1555),
    [sym_file_dir] = ACTIONS(1555),
    [sym_file_getdir] = ACTIONS(1555),
    [sym_file_setdir] = ACTIONS(1555),
    [sym_file_create_dir] = ACTIONS(1555),
    [sym_file_remove_dir] = ACTIONS(1555),
    [sym_file_seek] = ACTIONS(1555),
    [sym_length_array] = ACTIONS(1555),
    [sym_max_length_array] = ACTIONS(1555),
    [sym_set_length_array] = ACTIONS(1555),
    [sym_ip_client_open] = ACTIONS(1555),
    [sym_ip_client_close] = ACTIONS(1555),
    [sym_ip_server_open] = ACTIONS(1555),
    [sym_ip_server_close] = ACTIONS(1555),
    [sym_ip_bound_client_open] = ACTIONS(1555),
    [sym_ip_mc_server_open] = ACTIONS(1555),
    [sym_ip_set_option] = ACTIONS(1555),
    [sym_ssh_client_open] = ACTIONS(1555),
    [sym_ssh_client_close] = ACTIONS(1555),
    [sym_tls_client_open] = ACTIONS(1555),
    [sym_tls_client_close] = ACTIONS(1555),
    [sym_do_push] = ACTIONS(1555),
    [sym_do_push_timed] = ACTIONS(1555),
    [sym_do_release] = ACTIONS(1555),
    [sym_device_id] = ACTIONS(1555),
    [sym_device_info] = ACTIONS(1555),
    [sym_device_id_string] = ACTIONS(1555),
    [sym_device_standby] = ACTIONS(1555),
    [sym_device_wake] = ACTIONS(1555),
    [sym_timeline_create] = ACTIONS(1555),
    [sym_timeline_kill] = ACTIONS(1555),
    [sym_timeline_reload] = ACTIONS(1555),
    [sym_timeline_pause] = ACTIONS(1555),
    [sym_timeline_restart] = ACTIONS(1555),
    [sym_timeline_set] = ACTIONS(1555),
    [sym_timeline_get] = ACTIONS(1555),
    [sym_timeline_active] = ACTIONS(1555),
    [sym_timeline_dynamic_id] = ACTIONS(1555),
    [sym_variable_to_string] = ACTIONS(1555),
    [sym_string_to_variable] = ACTIONS(1555),
    [sym_length_variable_to_string] = ACTIONS(1555),
    [sym_type_cast] = ACTIONS(1555),
    [sym_variable_to_xml] = ACTIONS(1555),
    [sym_xml_to_variable] = ACTIONS(1555),
    [sym_length_variable_to_xml] = ACTIONS(1555),
    [sym_redirect_string] = ACTIONS(1555),
    [sym_random_number] = ACTIONS(1555),
    [sym_get_unique_id] = ACTIONS(1555),
    [sym_get_system_number] = ACTIONS(1555),
    [sym_set_system_number] = ACTIONS(1555),
    [sym_get_serial_number] = ACTIONS(1555),
    [sym_get_master_build] = ACTIONS(1555),
    [sym_set_validation_code] = ACTIONS(1555),
    [sym_reboot] = ACTIONS(1555),
    [sym_load_duet_module] = ACTIONS(1555),
    [sym_unload_duet_module] = ACTIONS(1555),
    [sym_duet_mem_size_set] = ACTIONS(1555),
    [sym_duet_mem_size_get] = ACTIONS(1555),
    [sym_dynamic_application_device] = ACTIONS(1555),
    [sym_dynamic_polled_port] = ACTIONS(1555),
    [sym_static_port_binding] = ACTIONS(1555),
    [sym_static_ip_binding] = ACTIONS(1555),
    [sym_get_dns_list] = ACTIONS(1555),
    [sym_set_dns_list] = ACTIONS(1555),
    [sym_get_ip_address] = ACTIONS(1555),
    [sym_set_ip_address] = ACTIONS(1555),
    [sym_get_url_list] = ACTIONS(1555),
    [sym_add_url_entry] = ACTIONS(1555),
    [sym_delete_url_entry] = ACTIONS(1555),
    [sym_abs_value] = ACTIONS(1555),
    [sym_max_value] = ACTIONS(1555),
    [sym_min_value] = ACTIONS(1555),
    [sym_exp_value] = ACTIONS(1555),
    [sym_log_value] = ACTIONS(1555),
    [sym_log10_value] = ACTIONS(1555),
    [sym_power_value] = ACTIONS(1555),
    [sym_sqrt_value] = ACTIONS(1555),
    [sym_raw_be] = ACTIONS(1555),
    [sym_raw_le] = ACTIONS(1555),
    [sym_internal_queue_size_set] = ACTIONS(1555),
    [sym_internal_queue_size_get] = ACTIONS(1555),
    [sym_internal_threshold_set] = ACTIONS(1555),
    [sym_internal_threshold_get] = ACTIONS(1555),
    [sym_rebuilt_event] = ACTIONS(1555),
    [sym_do_custom_event] = ACTIONS(1555),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1555),
    [sym_clkmgr_set_clk_source] = ACTIONS(1555),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1555),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1555),
    [sym_clkmgr_get_timezone] = ACTIONS(1555),
    [sym_clkmgr_set_timezone] = ACTIONS(1555),
    [sym_clkmgr_get_resync_period] = ACTIONS(1555),
    [sym_clkmgr_set_resync_period] = ACTIONS(1555),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1555),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1555),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1555),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1555),
    [sym_clkmgr_get_timeservers] = ACTIONS(1555),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1555),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1555),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1555),
    [sym_set_log_level] = ACTIONS(1555),
    [sym_get_log_level] = ACTIONS(1555),
    [sym_amx_log] = ACTIONS(1555),
    [sym_smtp_server_config_set] = ACTIONS(1555),
    [sym_smtp_server_config_get] = ACTIONS(1555),
    [sym_smtp_send] = ACTIONS(1555),
    [sym_validate_netlinx_account] = ACTIONS(1555),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1555),
    [sym_authenticate_certificate] = ACTIONS(1555),
    [sym_audit_netlinx_session_event] = ACTIONS(1555),
    [sym_audit_netlinx_generic_event] = ACTIONS(1555),
    [sym_get_available_flash_disk_space] = ACTIONS(1555),
    [sym_get_max_flash_disk_space] = ACTIONS(1555),
    [sym_data_create_feed] = ACTIONS(1555),
    [sym_data_delete_feed] = ACTIONS(1555),
    [sym_data_publish_feed] = ACTIONS(1555),
    [sym_data_get_published_feed] = ACTIONS(1555),
    [sym_data_add_record] = ACTIONS(1555),
    [sym_data_get_event_record] = ACTIONS(1555),
    [sym__wc_data_create_feed] = ACTIONS(1555),
    [sym__wc_data_add_record] = ACTIONS(1555),
    [sym__wc_data_get_event_record] = ACTIONS(1555),
    [sym_format] = ACTIONS(1555),
    [sym_astro_clock] = ACTIONS(1555),
    [sym_set_outdoor_temperature] = ACTIONS(1555),
    [sym_set_virtual_level_count] = ACTIONS(1555),
    [sym_set_virtual_channel_count] = ACTIONS(1555),
    [sym_set_virtual_port_count] = ACTIONS(1555),
    [sym__wc_encode_int] = ACTIONS(1555),
    [sym__wc_decode_int] = ACTIONS(1555),
    [sym__wc_int] = ACTIONS(1555),
    [sym__wc_to_ch_int] = ACTIONS(1555),
    [sym__ch_to_wc_int] = ACTIONS(1555),
    [sym__wc_find_string_int] = ACTIONS(1555),
    [sym__wc_left_string_int] = ACTIONS(1555),
    [sym__wc_lower_string_int] = ACTIONS(1555),
    [sym__wc_mid_string_int] = ACTIONS(1555),
    [sym__wc_remove_string_int] = ACTIONS(1555),
    [sym__wc_right_string_int] = ACTIONS(1555),
    [sym__wc_upper_string_int] = ACTIONS(1555),
    [sym__wc_compare_string_int] = ACTIONS(1555),
    [sym__wc_get_buffer_char_int] = ACTIONS(1555),
    [sym__wc_get_buffer_string_int] = ACTIONS(1555),
    [sym__wc_concat_string_int] = ACTIONS(1555),
    [sym__wc_file_open_int] = ACTIONS(1555),
    [sym__wc_file_close_int] = ACTIONS(1555),
    [sym__wc_file_read_int] = ACTIONS(1555),
    [sym__wc_file_read_line_int] = ACTIONS(1555),
    [sym__wc_file_write_int] = ACTIONS(1555),
    [sym__wc_file_write_line_int] = ACTIONS(1555),
    [anon_sym_EQ] = ACTIONS(1555),
    [aux_sym_preproc_include_token1] = ACTIONS(1555),
    [aux_sym_preproc_define_token1] = ACTIONS(1557),
    [aux_sym_preproc_warn_token1] = ACTIONS(1557),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1557),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1557),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1557),
    [anon_sym_LBRACK] = ACTIONS(1557),
    [anon_sym_LBRACE] = ACTIONS(1557),
    [anon_sym_RBRACE] = ACTIONS(1557),
    [anon_sym_COLON] = ACTIONS(1557),
    [anon_sym_LPAREN] = ACTIONS(1555),
    [anon_sym_SEMI] = ACTIONS(1557),
    [anon_sym_BANG] = ACTIONS(1555),
    [anon_sym_TILDE] = ACTIONS(1557),
    [anon_sym_DASH] = ACTIONS(1555),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_STAR] = ACTIONS(1557),
    [anon_sym_SLASH] = ACTIONS(1555),
    [anon_sym_PERCENT] = ACTIONS(1557),
    [anon_sym_PIPE_PIPE] = ACTIONS(1557),
    [anon_sym_AMP_AMP] = ACTIONS(1557),
    [anon_sym_PIPE] = ACTIONS(1555),
    [anon_sym_CARET] = ACTIONS(1557),
    [anon_sym_AMP] = ACTIONS(1555),
    [anon_sym_EQ_EQ] = ACTIONS(1557),
    [anon_sym_BANG_EQ] = ACTIONS(1557),
    [anon_sym_LT_GT] = ACTIONS(1557),
    [anon_sym_GT] = ACTIONS(1555),
    [anon_sym_GT_EQ] = ACTIONS(1557),
    [anon_sym_LT_EQ] = ACTIONS(1557),
    [anon_sym_LT] = ACTIONS(1555),
    [anon_sym_LT_LT] = ACTIONS(1557),
    [anon_sym_GT_GT] = ACTIONS(1557),
    [anon_sym_DASH_DASH] = ACTIONS(1557),
    [anon_sym_PLUS_PLUS] = ACTIONS(1557),
    [anon_sym_DOT] = ACTIONS(1555),
    [anon_sym_DQUOTE] = ACTIONS(1557),
    [anon_sym_SQUOTE] = ACTIONS(1557),
    [sym_number_literal] = ACTIONS(1557),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1557),
  },
  [STATE(180)] = {
    [sym_identifier] = ACTIONS(1535),
    [sym_define_call_keyword] = ACTIONS(1535),
    [sym_define_function_keyword] = ACTIONS(1535),
    [sym_define_library_function_keyword] = ACTIONS(1535),
    [sym_define_module_keyword] = ACTIONS(1535),
    [sym_char_keyword] = ACTIONS(1535),
    [sym_widechar_keyword] = ACTIONS(1535),
    [sym_integer_keyword] = ACTIONS(1535),
    [sym_sinteger_keyword] = ACTIONS(1535),
    [sym_long_keyword] = ACTIONS(1535),
    [sym_slong_keyword] = ACTIONS(1535),
    [sym_float_keyword] = ACTIONS(1535),
    [sym_double_keyword] = ACTIONS(1535),
    [sym_variant_keyword] = ACTIONS(1535),
    [sym_variantarray_keyword] = ACTIONS(1535),
    [sym_local_var_keyword] = ACTIONS(1535),
    [sym_stack_var_keyword] = ACTIONS(1535),
    [sym_constant_keyword] = ACTIONS(1535),
    [sym_volatile_keyword] = ACTIONS(1535),
    [sym_non_volatile_keyword] = ACTIONS(1535),
    [sym_persistent_keyword] = ACTIONS(1535),
    [sym_dev_keyword] = ACTIONS(1535),
    [sym_devlev_keyword] = ACTIONS(1535),
    [sym_devchan_keyword] = ACTIONS(1535),
    [sym_if_keyword] = ACTIONS(1535),
    [sym_else_keyword] = ACTIONS(1535),
    [sym_switch_keyword] = ACTIONS(1535),
    [sym_case_keyword] = ACTIONS(1535),
    [sym_default_keyword] = ACTIONS(1535),
    [sym_while_keyword] = ACTIONS(1535),
    [sym_for_keyword] = ACTIONS(1535),
    [sym_break_keyword] = ACTIONS(1535),
    [sym_continue_keyword] = ACTIONS(1535),
    [sym_return_keyword] = ACTIONS(1535),
    [sym_select_keyword] = ACTIONS(1535),
    [sym_active_keyword] = ACTIONS(1535),
    [sym_struct_keyword] = ACTIONS(1535),
    [sym_structure_keyword] = ACTIONS(1535),
    [aux_sym_on_keyword_token1] = ACTIONS(1535),
    [aux_sym_off_keyword_token1] = ACTIONS(1535),
    [sym_send_level_keyword] = ACTIONS(1535),
    [sym_send_string_keyword] = ACTIONS(1535),
    [sym_send_command_keyword] = ACTIONS(1535),
    [sym_clear_buffer_keyword] = ACTIONS(1535),
    [sym_create_buffer_keyword] = ACTIONS(1535),
    [sym_create_multi_buffer_keyword] = ACTIONS(1535),
    [sym_call_keyword] = ACTIONS(1535),
    [sym_system_call_keyword] = ACTIONS(1535),
    [sym_devchan_to_keyword] = ACTIONS(1535),
    [sym_devchan_min_to_keyword] = ACTIONS(1535),
    [sym_devchan_total_off_keyword] = ACTIONS(1535),
    [sym_devchan_pulse_keyword] = ACTIONS(1535),
    [sym_wait_keyword] = ACTIONS(1535),
    [sym_cancel_wait_keyword] = ACTIONS(1535),
    [sym_wait_until_keyword] = ACTIONS(1535),
    [sym_cancel_wait_until_keyword] = ACTIONS(1535),
    [sym_cancel_all_wait_keyword] = ACTIONS(1535),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1535),
    [sym_and] = ACTIONS(1535),
    [sym_or] = ACTIONS(1535),
    [sym_not] = ACTIONS(1535),
    [sym_xor] = ACTIONS(1535),
    [sym_band] = ACTIONS(1535),
    [sym_bor] = ACTIONS(1535),
    [sym_bxor] = ACTIONS(1535),
    [sym_bnot] = ACTIONS(1535),
    [sym_lshift] = ACTIONS(1535),
    [sym_rshift] = ACTIONS(1535),
    [sym___file__] = ACTIONS(1535),
    [sym___date__] = ACTIONS(1535),
    [sym___ldate__] = ACTIONS(1535),
    [sym___time__] = ACTIONS(1535),
    [sym___line__] = ACTIONS(1535),
    [sym___name__] = ACTIONS(1535),
    [sym___netlinx__] = ACTIONS(1535),
    [sym_netlinx_axi_version] = ACTIONS(1535),
    [sym_url_flg_tcp] = ACTIONS(1535),
    [sym_url_flg_accinfopresent] = ACTIONS(1535),
    [sym_url_flg_temp] = ACTIONS(1535),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1535),
    [sym_url_flg_stat_mask] = ACTIONS(1535),
    [sym_url_flg_stat_lookup] = ACTIONS(1535),
    [sym_url_flg_stat_connecting] = ACTIONS(1535),
    [sym_url_flg_stat_waiting] = ACTIONS(1535),
    [sym_url_flg_stat_connected] = ACTIONS(1535),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1535),
    [sym_first_virtual_device] = ACTIONS(1535),
    [sym_first_local_port] = ACTIONS(1535),
    [sym_dynamic_virtual_device] = ACTIONS(1535),
    [sym_timeline_once] = ACTIONS(1535),
    [sym_timeline_repeat] = ACTIONS(1535),
    [sym_timeline_absolute] = ACTIONS(1535),
    [sym_timeline_relative] = ACTIONS(1535),
    [sym_do_push_timed_infinite] = ACTIONS(1535),
    [sym_source_type_no_address] = ACTIONS(1535),
    [sym_source_type_neuron_id] = ACTIONS(1535),
    [sym_source_type_ip_address] = ACTIONS(1535),
    [sym_source_type_axlink] = ACTIONS(1535),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1535),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1535),
    [sym_source_type_ip_socket_address] = ACTIONS(1535),
    [sym_source_type_rs232] = ACTIONS(1535),
    [sym_source_type_internal] = ACTIONS(1535),
    [sym_source_type_mac_address] = ACTIONS(1535),
    [sym_source_type_ipv4_port] = ACTIONS(1535),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1535),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1535),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1535),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1535),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1535),
    [sym_true] = ACTIONS(1535),
    [sym_false] = ACTIONS(1535),
    [sym_file_read_only] = ACTIONS(1535),
    [sym_file_rw_new] = ACTIONS(1535),
    [sym_file_rw_append] = ACTIONS(1535),
    [sym_ip_tcp] = ACTIONS(1535),
    [sym_ip_udp] = ACTIONS(1535),
    [sym_ip_udp_2way] = ACTIONS(1535),
    [sym_xml_encode_types] = ACTIONS(1535),
    [sym_xml_encode_char_as_list] = ACTIONS(1535),
    [sym_xml_encode_le] = ACTIONS(1535),
    [sym_xml_decode_types] = ACTIONS(1535),
    [sym_xml_decode_no_preserve] = ACTIONS(1535),
    [sym_duet_dev_type_display_device] = ACTIONS(1535),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1535),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1535),
    [sym_duet_dev_type_io_device] = ACTIONS(1535),
    [sym_duet_dev_type_relay_device] = ACTIONS(1535),
    [sym_duet_dev_type_ups] = ACTIONS(1535),
    [sym_duet_dev_type_amplifier] = ACTIONS(1535),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1535),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1535),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1535),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1535),
    [sym_duet_dev_type_camera] = ACTIONS(1535),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1535),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1535),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1535),
    [sym_duet_dev_type_dss] = ACTIONS(1535),
    [sym_duet_dev_type_dvr] = ACTIONS(1535),
    [sym_duet_dev_type_disc_device] = ACTIONS(1535),
    [sym_duet_dev_type_document_camera] = ACTIONS(1535),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1535),
    [sym_duet_dev_type_hvac] = ACTIONS(1535),
    [sym_duet_dev_type_keypad] = ACTIONS(1535),
    [sym_duet_dev_type_light] = ACTIONS(1535),
    [sym_duet_dev_type_monitor] = ACTIONS(1535),
    [sym_duet_dev_type_motor] = ACTIONS(1535),
    [sym_duet_dev_type_multi_window] = ACTIONS(1535),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1535),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1535),
    [sym_duet_dev_type_receiver] = ACTIONS(1535),
    [sym_duet_dev_type_security_system] = ACTIONS(1535),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1535),
    [sym_duet_dev_type_settop_box] = ACTIONS(1535),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1535),
    [sym_duet_dev_type_switcher] = ACTIONS(1535),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1535),
    [sym_duet_dev_type_tv] = ACTIONS(1535),
    [sym_duet_dev_type_utility] = ACTIONS(1535),
    [sym_duet_dev_type_vcr] = ACTIONS(1535),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1535),
    [sym_duet_dev_type_video_processor] = ACTIONS(1535),
    [sym_duet_dev_type_video_projector] = ACTIONS(1535),
    [sym_duet_dev_type_video_wall] = ACTIONS(1535),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1535),
    [sym_duet_dev_type_weather] = ACTIONS(1535),
    [sym_duet_dev_not_polled] = ACTIONS(1535),
    [sym_duet_dev_polled] = ACTIONS(1535),
    [sym_clkmgr_mode_network] = ACTIONS(1535),
    [sym_clkmgr_mode_standalone] = ACTIONS(1535),
    [sym_normal_standby] = ACTIONS(1535),
    [sym_normal_wake] = ACTIONS(1535),
    [sym_ip_multicast_ttl_option] = ACTIONS(1535),
    [sym_ip_ttl_subnet] = ACTIONS(1535),
    [sym_ip_ttl_site] = ACTIONS(1535),
    [sym_ip_ttl_region] = ACTIONS(1535),
    [sym_ip_ttl_continent] = ACTIONS(1535),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1535),
    [sym_ip_nodelay_on] = ACTIONS(1535),
    [sym_ip_nodelay_off] = ACTIONS(1535),
    [sym_amx_error] = ACTIONS(1535),
    [sym_amx_warning] = ACTIONS(1535),
    [sym_amx_info] = ACTIONS(1535),
    [sym_amx_debug] = ACTIONS(1535),
    [sym_netlinx_login_success] = ACTIONS(1535),
    [sym_netlinx_login_fail] = ACTIONS(1535),
    [sym_netlinx_logout] = ACTIONS(1535),
    [sym_null_str] = ACTIONS(1535),
    [sym_smtp_address] = ACTIONS(1535),
    [sym_smtp_port_number] = ACTIONS(1535),
    [sym_smtp_username] = ACTIONS(1535),
    [sym_smtp_password] = ACTIONS(1535),
    [sym_smtp_require_tls] = ACTIONS(1535),
    [sym_smtp_from] = ACTIONS(1535),
    [sym_smtp_tls_true] = ACTIONS(1535),
    [sym_smtp_tls_false] = ACTIONS(1535),
    [sym_listview_on_row_select_event] = ACTIONS(1535),
    [sym_data_structure_unknown] = ACTIONS(1535),
    [sym_data_structure_datafeed] = ACTIONS(1535),
    [sym_data_structure_datarecordset] = ACTIONS(1535),
    [sym_data_structure_datarecord] = ACTIONS(1535),
    [sym_data_structure_datafield] = ACTIONS(1535),
    [sym_data_max_content_field_count] = ACTIONS(1535),
    [sym_data_max_metadata_field_count] = ACTIONS(1535),
    [sym_data_max_id_length] = ACTIONS(1535),
    [sym_data_max_name_length] = ACTIONS(1535),
    [sym_data_max_description_length] = ACTIONS(1535),
    [sym_data_max_type_length] = ACTIONS(1535),
    [sym_data_max_format_length] = ACTIONS(1535),
    [sym_data_max_source_length] = ACTIONS(1535),
    [sym_data_max_value_length] = ACTIONS(1535),
    [sym_data_max_label_length] = ACTIONS(1535),
    [sym_data_max_url_length] = ACTIONS(1535),
    [sym_max_last_login_info_length] = ACTIONS(1535),
    [sym_data_type_unknown] = ACTIONS(1535),
    [sym_data_type_string] = ACTIONS(1535),
    [sym_data_type_datetime] = ACTIONS(1535),
    [sym_data_type_date] = ACTIONS(1535),
    [sym_data_type_time] = ACTIONS(1535),
    [sym_data_type_image] = ACTIONS(1535),
    [sym_data_format_url] = ACTIONS(1535),
    [sym_data_format_phone] = ACTIONS(1535),
    [sym_data_format_email] = ACTIONS(1535),
    [sym_data_format_iso8601] = ACTIONS(1535),
    [sym_auditlog_priv] = ACTIONS(1535),
    [sym_configuration_priv] = ACTIONS(1535),
    [sym_deviceconfig_priv] = ACTIONS(1535),
    [sym_ftp_priv] = ACTIONS(1535),
    [sym_http_priv] = ACTIONS(1535),
    [sym_networkconfig_priv] = ACTIONS(1535),
    [sym_programport_priv] = ACTIONS(1535),
    [sym_remoteui_priv] = ACTIONS(1535),
    [sym_securitycontrol_priv] = ACTIONS(1535),
    [sym_softwaremanagement_priv] = ACTIONS(1535),
    [sym_terminal_priv] = ACTIONS(1535),
    [sym_tpadmin_priv] = ACTIONS(1535),
    [sym_user1_priv] = ACTIONS(1535),
    [sym_user2_priv] = ACTIONS(1535),
    [sym_user3_priv] = ACTIONS(1535),
    [sym_user4_priv] = ACTIONS(1535),
    [sym_usermanagement_priv] = ACTIONS(1535),
    [sym_valid_account] = ACTIONS(1535),
    [sym_err_name_invalid] = ACTIONS(1535),
    [sym_err_password_invalid] = ACTIONS(1535),
    [sym_err_invalid_account] = ACTIONS(1535),
    [sym_err_no_authorization] = ACTIONS(1535),
    [sym_err_invalid_parameter] = ACTIONS(1535),
    [sym_err_user_locked_out] = ACTIONS(1535),
    [sym_err_user_account_expired] = ACTIONS(1535),
    [sym_err_general_error] = ACTIONS(1535),
    [sym_err_authentication_error] = ACTIONS(1535),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1535),
    [sym_tls_validate_certificate] = ACTIONS(1535),
    [sym_tdata] = ACTIONS(1535),
    [sym_tchannel] = ACTIONS(1535),
    [sym_tlevel] = ACTIONS(1535),
    [sym_tbutton] = ACTIONS(1535),
    [sym_ttimeline] = ACTIONS(1535),
    [sym_tcustom] = ACTIONS(1535),
    [sym_url_struct] = ACTIONS(1535),
    [sym_dns_struct] = ACTIONS(1535),
    [sym_ip_address_struct] = ACTIONS(1535),
    [sym_dev_info_struct] = ACTIONS(1535),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1535),
    [sym_data_feed] = ACTIONS(1535),
    [sym_data_field] = ACTIONS(1535),
    [sym_data_record] = ACTIONS(1535),
    [sym_wc_data_feed] = ACTIONS(1535),
    [sym_wc_data_field] = ACTIONS(1535),
    [sym_wc_data_record] = ACTIONS(1535),
    [sym_last_login_info] = ACTIONS(1535),
    [sym____reserved___] = ACTIONS(1535),
    [sym_date] = ACTIONS(1535),
    [sym_day] = ACTIONS(1535),
    [sym_time] = ACTIONS(1535),
    [sym_ldate] = ACTIONS(1535),
    [sym_push_device] = ACTIONS(1535),
    [sym_push_channel] = ACTIONS(1535),
    [sym_push_devchan] = ACTIONS(1535),
    [sym_release_device] = ACTIONS(1535),
    [sym_release_channel] = ACTIONS(1535),
    [sym_release_devchan] = ACTIONS(1535),
    [sym_master_slot] = ACTIONS(1535),
    [sym_get_pulse_time] = ACTIONS(1535),
    [sym_get_timer] = ACTIONS(1535),
    [sym_master_sn] = ACTIONS(1535),
    [sym_system_number] = ACTIONS(1535),
    [sym_button] = ACTIONS(1535),
    [sym_dv_channel] = ACTIONS(1535),
    [sym_data] = ACTIONS(1535),
    [sym_level] = ACTIONS(1535),
    [sym_timeline] = ACTIONS(1535),
    [sym_channel] = ACTIONS(1535),
    [sym_custom] = ACTIONS(1535),
    [sym_atoi] = ACTIONS(1535),
    [sym_atol] = ACTIONS(1535),
    [sym_atof] = ACTIONS(1535),
    [sym_itoa] = ACTIONS(1535),
    [sym_itohex] = ACTIONS(1535),
    [sym_hextoi] = ACTIONS(1535),
    [sym_ftoa] = ACTIONS(1535),
    [sym_find_string] = ACTIONS(1535),
    [sym_left_string] = ACTIONS(1535),
    [sym_right_string] = ACTIONS(1535),
    [sym_mid_string] = ACTIONS(1535),
    [sym_remove_string] = ACTIONS(1535),
    [sym_get_buffer_string] = ACTIONS(1535),
    [sym_get_buffer_char] = ACTIONS(1535),
    [sym_get_multi_buffer_string] = ACTIONS(1535),
    [sym_length_string] = ACTIONS(1535),
    [sym_max_length_string] = ACTIONS(1535),
    [sym_set_length_string] = ACTIONS(1535),
    [sym_compare_string] = ACTIONS(1535),
    [sym_upper_string] = ACTIONS(1535),
    [sym_lower_string] = ACTIONS(1535),
    [sym_get_last] = ACTIONS(1535),
    [sym_date_to_day] = ACTIONS(1535),
    [sym_date_to_month] = ACTIONS(1535),
    [sym_date_to_year] = ACTIONS(1535),
    [sym_time_to_hour] = ACTIONS(1535),
    [sym_time_to_minute] = ACTIONS(1535),
    [sym_time_to_second] = ACTIONS(1535),
    [sym_day_of_week] = ACTIONS(1535),
    [sym_set_pulse_time] = ACTIONS(1535),
    [sym_set_timer] = ACTIONS(1535),
    [sym_file_open] = ACTIONS(1535),
    [sym_file_close] = ACTIONS(1535),
    [sym_file_read] = ACTIONS(1535),
    [sym_file_read_line] = ACTIONS(1535),
    [sym_file_write] = ACTIONS(1535),
    [sym_file_write_line] = ACTIONS(1535),
    [sym_file_copy] = ACTIONS(1535),
    [sym_file_delete] = ACTIONS(1535),
    [sym_file_rename] = ACTIONS(1535),
    [sym_file_dir] = ACTIONS(1535),
    [sym_file_getdir] = ACTIONS(1535),
    [sym_file_setdir] = ACTIONS(1535),
    [sym_file_create_dir] = ACTIONS(1535),
    [sym_file_remove_dir] = ACTIONS(1535),
    [sym_file_seek] = ACTIONS(1535),
    [sym_length_array] = ACTIONS(1535),
    [sym_max_length_array] = ACTIONS(1535),
    [sym_set_length_array] = ACTIONS(1535),
    [sym_ip_client_open] = ACTIONS(1535),
    [sym_ip_client_close] = ACTIONS(1535),
    [sym_ip_server_open] = ACTIONS(1535),
    [sym_ip_server_close] = ACTIONS(1535),
    [sym_ip_bound_client_open] = ACTIONS(1535),
    [sym_ip_mc_server_open] = ACTIONS(1535),
    [sym_ip_set_option] = ACTIONS(1535),
    [sym_ssh_client_open] = ACTIONS(1535),
    [sym_ssh_client_close] = ACTIONS(1535),
    [sym_tls_client_open] = ACTIONS(1535),
    [sym_tls_client_close] = ACTIONS(1535),
    [sym_do_push] = ACTIONS(1535),
    [sym_do_push_timed] = ACTIONS(1535),
    [sym_do_release] = ACTIONS(1535),
    [sym_device_id] = ACTIONS(1535),
    [sym_device_info] = ACTIONS(1535),
    [sym_device_id_string] = ACTIONS(1535),
    [sym_device_standby] = ACTIONS(1535),
    [sym_device_wake] = ACTIONS(1535),
    [sym_timeline_create] = ACTIONS(1535),
    [sym_timeline_kill] = ACTIONS(1535),
    [sym_timeline_reload] = ACTIONS(1535),
    [sym_timeline_pause] = ACTIONS(1535),
    [sym_timeline_restart] = ACTIONS(1535),
    [sym_timeline_set] = ACTIONS(1535),
    [sym_timeline_get] = ACTIONS(1535),
    [sym_timeline_active] = ACTIONS(1535),
    [sym_timeline_dynamic_id] = ACTIONS(1535),
    [sym_variable_to_string] = ACTIONS(1535),
    [sym_string_to_variable] = ACTIONS(1535),
    [sym_length_variable_to_string] = ACTIONS(1535),
    [sym_type_cast] = ACTIONS(1535),
    [sym_variable_to_xml] = ACTIONS(1535),
    [sym_xml_to_variable] = ACTIONS(1535),
    [sym_length_variable_to_xml] = ACTIONS(1535),
    [sym_redirect_string] = ACTIONS(1535),
    [sym_random_number] = ACTIONS(1535),
    [sym_get_unique_id] = ACTIONS(1535),
    [sym_get_system_number] = ACTIONS(1535),
    [sym_set_system_number] = ACTIONS(1535),
    [sym_get_serial_number] = ACTIONS(1535),
    [sym_get_master_build] = ACTIONS(1535),
    [sym_set_validation_code] = ACTIONS(1535),
    [sym_reboot] = ACTIONS(1535),
    [sym_load_duet_module] = ACTIONS(1535),
    [sym_unload_duet_module] = ACTIONS(1535),
    [sym_duet_mem_size_set] = ACTIONS(1535),
    [sym_duet_mem_size_get] = ACTIONS(1535),
    [sym_dynamic_application_device] = ACTIONS(1535),
    [sym_dynamic_polled_port] = ACTIONS(1535),
    [sym_static_port_binding] = ACTIONS(1535),
    [sym_static_ip_binding] = ACTIONS(1535),
    [sym_get_dns_list] = ACTIONS(1535),
    [sym_set_dns_list] = ACTIONS(1535),
    [sym_get_ip_address] = ACTIONS(1535),
    [sym_set_ip_address] = ACTIONS(1535),
    [sym_get_url_list] = ACTIONS(1535),
    [sym_add_url_entry] = ACTIONS(1535),
    [sym_delete_url_entry] = ACTIONS(1535),
    [sym_abs_value] = ACTIONS(1535),
    [sym_max_value] = ACTIONS(1535),
    [sym_min_value] = ACTIONS(1535),
    [sym_exp_value] = ACTIONS(1535),
    [sym_log_value] = ACTIONS(1535),
    [sym_log10_value] = ACTIONS(1535),
    [sym_power_value] = ACTIONS(1535),
    [sym_sqrt_value] = ACTIONS(1535),
    [sym_raw_be] = ACTIONS(1535),
    [sym_raw_le] = ACTIONS(1535),
    [sym_internal_queue_size_set] = ACTIONS(1535),
    [sym_internal_queue_size_get] = ACTIONS(1535),
    [sym_internal_threshold_set] = ACTIONS(1535),
    [sym_internal_threshold_get] = ACTIONS(1535),
    [sym_rebuilt_event] = ACTIONS(1535),
    [sym_do_custom_event] = ACTIONS(1535),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1535),
    [sym_clkmgr_set_clk_source] = ACTIONS(1535),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1535),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1535),
    [sym_clkmgr_get_timezone] = ACTIONS(1535),
    [sym_clkmgr_set_timezone] = ACTIONS(1535),
    [sym_clkmgr_get_resync_period] = ACTIONS(1535),
    [sym_clkmgr_set_resync_period] = ACTIONS(1535),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1535),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1535),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1535),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1535),
    [sym_clkmgr_get_timeservers] = ACTIONS(1535),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1535),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1535),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1535),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1535),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1535),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1535),
    [sym_set_log_level] = ACTIONS(1535),
    [sym_get_log_level] = ACTIONS(1535),
    [sym_amx_log] = ACTIONS(1535),
    [sym_smtp_server_config_set] = ACTIONS(1535),
    [sym_smtp_server_config_get] = ACTIONS(1535),
    [sym_smtp_send] = ACTIONS(1535),
    [sym_validate_netlinx_account] = ACTIONS(1535),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1535),
    [sym_authenticate_certificate] = ACTIONS(1535),
    [sym_audit_netlinx_session_event] = ACTIONS(1535),
    [sym_audit_netlinx_generic_event] = ACTIONS(1535),
    [sym_get_available_flash_disk_space] = ACTIONS(1535),
    [sym_get_max_flash_disk_space] = ACTIONS(1535),
    [sym_data_create_feed] = ACTIONS(1535),
    [sym_data_delete_feed] = ACTIONS(1535),
    [sym_data_publish_feed] = ACTIONS(1535),
    [sym_data_get_published_feed] = ACTIONS(1535),
    [sym_data_add_record] = ACTIONS(1535),
    [sym_data_get_event_record] = ACTIONS(1535),
    [sym__wc_data_create_feed] = ACTIONS(1535),
    [sym__wc_data_add_record] = ACTIONS(1535),
    [sym__wc_data_get_event_record] = ACTIONS(1535),
    [sym_format] = ACTIONS(1535),
    [sym_astro_clock] = ACTIONS(1535),
    [sym_set_outdoor_temperature] = ACTIONS(1535),
    [sym_set_virtual_level_count] = ACTIONS(1535),
    [sym_set_virtual_channel_count] = ACTIONS(1535),
    [sym_set_virtual_port_count] = ACTIONS(1535),
    [sym__wc_encode_int] = ACTIONS(1535),
    [sym__wc_decode_int] = ACTIONS(1535),
    [sym__wc_int] = ACTIONS(1535),
    [sym__wc_to_ch_int] = ACTIONS(1535),
    [sym__ch_to_wc_int] = ACTIONS(1535),
    [sym__wc_find_string_int] = ACTIONS(1535),
    [sym__wc_left_string_int] = ACTIONS(1535),
    [sym__wc_lower_string_int] = ACTIONS(1535),
    [sym__wc_mid_string_int] = ACTIONS(1535),
    [sym__wc_remove_string_int] = ACTIONS(1535),
    [sym__wc_right_string_int] = ACTIONS(1535),
    [sym__wc_upper_string_int] = ACTIONS(1535),
    [sym__wc_compare_string_int] = ACTIONS(1535),
    [sym__wc_get_buffer_char_int] = ACTIONS(1535),
    [sym__wc_get_buffer_string_int] = ACTIONS(1535),
    [sym__wc_concat_string_int] = ACTIONS(1535),
    [sym__wc_file_open_int] = ACTIONS(1535),
    [sym__wc_file_close_int] = ACTIONS(1535),
    [sym__wc_file_read_int] = ACTIONS(1535),
    [sym__wc_file_read_line_int] = ACTIONS(1535),
    [sym__wc_file_write_int] = ACTIONS(1535),
    [sym__wc_file_write_line_int] = ACTIONS(1535),
    [anon_sym_EQ] = ACTIONS(1535),
    [aux_sym_preproc_include_token1] = ACTIONS(1535),
    [aux_sym_preproc_define_token1] = ACTIONS(1537),
    [aux_sym_preproc_warn_token1] = ACTIONS(1537),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1537),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1537),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1537),
    [anon_sym_LBRACK] = ACTIONS(1537),
    [anon_sym_LBRACE] = ACTIONS(1537),
    [anon_sym_RBRACE] = ACTIONS(1537),
    [anon_sym_COLON] = ACTIONS(1537),
    [anon_sym_LPAREN] = ACTIONS(1535),
    [anon_sym_SEMI] = ACTIONS(1537),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1537),
    [anon_sym_DASH] = ACTIONS(1535),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_STAR] = ACTIONS(1537),
    [anon_sym_SLASH] = ACTIONS(1535),
    [anon_sym_PERCENT] = ACTIONS(1537),
    [anon_sym_PIPE_PIPE] = ACTIONS(1537),
    [anon_sym_AMP_AMP] = ACTIONS(1537),
    [anon_sym_PIPE] = ACTIONS(1535),
    [anon_sym_CARET] = ACTIONS(1537),
    [anon_sym_AMP] = ACTIONS(1535),
    [anon_sym_EQ_EQ] = ACTIONS(1537),
    [anon_sym_BANG_EQ] = ACTIONS(1537),
    [anon_sym_LT_GT] = ACTIONS(1537),
    [anon_sym_GT] = ACTIONS(1535),
    [anon_sym_GT_EQ] = ACTIONS(1537),
    [anon_sym_LT_EQ] = ACTIONS(1537),
    [anon_sym_LT] = ACTIONS(1535),
    [anon_sym_LT_LT] = ACTIONS(1537),
    [anon_sym_GT_GT] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DOT] = ACTIONS(1535),
    [anon_sym_DQUOTE] = ACTIONS(1537),
    [anon_sym_SQUOTE] = ACTIONS(1537),
    [sym_number_literal] = ACTIONS(1537),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1537),
  },
  [STATE(181)] = {
    [sym_identifier] = ACTIONS(1531),
    [sym_define_call_keyword] = ACTIONS(1531),
    [sym_define_function_keyword] = ACTIONS(1531),
    [sym_define_library_function_keyword] = ACTIONS(1531),
    [sym_define_module_keyword] = ACTIONS(1531),
    [sym_char_keyword] = ACTIONS(1531),
    [sym_widechar_keyword] = ACTIONS(1531),
    [sym_integer_keyword] = ACTIONS(1531),
    [sym_sinteger_keyword] = ACTIONS(1531),
    [sym_long_keyword] = ACTIONS(1531),
    [sym_slong_keyword] = ACTIONS(1531),
    [sym_float_keyword] = ACTIONS(1531),
    [sym_double_keyword] = ACTIONS(1531),
    [sym_variant_keyword] = ACTIONS(1531),
    [sym_variantarray_keyword] = ACTIONS(1531),
    [sym_local_var_keyword] = ACTIONS(1531),
    [sym_stack_var_keyword] = ACTIONS(1531),
    [sym_constant_keyword] = ACTIONS(1531),
    [sym_volatile_keyword] = ACTIONS(1531),
    [sym_non_volatile_keyword] = ACTIONS(1531),
    [sym_persistent_keyword] = ACTIONS(1531),
    [sym_dev_keyword] = ACTIONS(1531),
    [sym_devlev_keyword] = ACTIONS(1531),
    [sym_devchan_keyword] = ACTIONS(1531),
    [sym_if_keyword] = ACTIONS(1531),
    [sym_else_keyword] = ACTIONS(1531),
    [sym_switch_keyword] = ACTIONS(1531),
    [sym_case_keyword] = ACTIONS(1531),
    [sym_default_keyword] = ACTIONS(1531),
    [sym_while_keyword] = ACTIONS(1531),
    [sym_for_keyword] = ACTIONS(1531),
    [sym_break_keyword] = ACTIONS(1531),
    [sym_continue_keyword] = ACTIONS(1531),
    [sym_return_keyword] = ACTIONS(1531),
    [sym_select_keyword] = ACTIONS(1531),
    [sym_active_keyword] = ACTIONS(1531),
    [sym_struct_keyword] = ACTIONS(1531),
    [sym_structure_keyword] = ACTIONS(1531),
    [aux_sym_on_keyword_token1] = ACTIONS(1531),
    [aux_sym_off_keyword_token1] = ACTIONS(1531),
    [sym_send_level_keyword] = ACTIONS(1531),
    [sym_send_string_keyword] = ACTIONS(1531),
    [sym_send_command_keyword] = ACTIONS(1531),
    [sym_clear_buffer_keyword] = ACTIONS(1531),
    [sym_create_buffer_keyword] = ACTIONS(1531),
    [sym_create_multi_buffer_keyword] = ACTIONS(1531),
    [sym_call_keyword] = ACTIONS(1531),
    [sym_system_call_keyword] = ACTIONS(1531),
    [sym_devchan_to_keyword] = ACTIONS(1531),
    [sym_devchan_min_to_keyword] = ACTIONS(1531),
    [sym_devchan_total_off_keyword] = ACTIONS(1531),
    [sym_devchan_pulse_keyword] = ACTIONS(1531),
    [sym_wait_keyword] = ACTIONS(1531),
    [sym_cancel_wait_keyword] = ACTIONS(1531),
    [sym_wait_until_keyword] = ACTIONS(1531),
    [sym_cancel_wait_until_keyword] = ACTIONS(1531),
    [sym_cancel_all_wait_keyword] = ACTIONS(1531),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1531),
    [sym_and] = ACTIONS(1531),
    [sym_or] = ACTIONS(1531),
    [sym_not] = ACTIONS(1531),
    [sym_xor] = ACTIONS(1531),
    [sym_band] = ACTIONS(1531),
    [sym_bor] = ACTIONS(1531),
    [sym_bxor] = ACTIONS(1531),
    [sym_bnot] = ACTIONS(1531),
    [sym_lshift] = ACTIONS(1531),
    [sym_rshift] = ACTIONS(1531),
    [sym___file__] = ACTIONS(1531),
    [sym___date__] = ACTIONS(1531),
    [sym___ldate__] = ACTIONS(1531),
    [sym___time__] = ACTIONS(1531),
    [sym___line__] = ACTIONS(1531),
    [sym___name__] = ACTIONS(1531),
    [sym___netlinx__] = ACTIONS(1531),
    [sym_netlinx_axi_version] = ACTIONS(1531),
    [sym_url_flg_tcp] = ACTIONS(1531),
    [sym_url_flg_accinfopresent] = ACTIONS(1531),
    [sym_url_flg_temp] = ACTIONS(1531),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1531),
    [sym_url_flg_stat_mask] = ACTIONS(1531),
    [sym_url_flg_stat_lookup] = ACTIONS(1531),
    [sym_url_flg_stat_connecting] = ACTIONS(1531),
    [sym_url_flg_stat_waiting] = ACTIONS(1531),
    [sym_url_flg_stat_connected] = ACTIONS(1531),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1531),
    [sym_first_virtual_device] = ACTIONS(1531),
    [sym_first_local_port] = ACTIONS(1531),
    [sym_dynamic_virtual_device] = ACTIONS(1531),
    [sym_timeline_once] = ACTIONS(1531),
    [sym_timeline_repeat] = ACTIONS(1531),
    [sym_timeline_absolute] = ACTIONS(1531),
    [sym_timeline_relative] = ACTIONS(1531),
    [sym_do_push_timed_infinite] = ACTIONS(1531),
    [sym_source_type_no_address] = ACTIONS(1531),
    [sym_source_type_neuron_id] = ACTIONS(1531),
    [sym_source_type_ip_address] = ACTIONS(1531),
    [sym_source_type_axlink] = ACTIONS(1531),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1531),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1531),
    [sym_source_type_ip_socket_address] = ACTIONS(1531),
    [sym_source_type_rs232] = ACTIONS(1531),
    [sym_source_type_internal] = ACTIONS(1531),
    [sym_source_type_mac_address] = ACTIONS(1531),
    [sym_source_type_ipv4_port] = ACTIONS(1531),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1531),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1531),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1531),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1531),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1531),
    [sym_true] = ACTIONS(1531),
    [sym_false] = ACTIONS(1531),
    [sym_file_read_only] = ACTIONS(1531),
    [sym_file_rw_new] = ACTIONS(1531),
    [sym_file_rw_append] = ACTIONS(1531),
    [sym_ip_tcp] = ACTIONS(1531),
    [sym_ip_udp] = ACTIONS(1531),
    [sym_ip_udp_2way] = ACTIONS(1531),
    [sym_xml_encode_types] = ACTIONS(1531),
    [sym_xml_encode_char_as_list] = ACTIONS(1531),
    [sym_xml_encode_le] = ACTIONS(1531),
    [sym_xml_decode_types] = ACTIONS(1531),
    [sym_xml_decode_no_preserve] = ACTIONS(1531),
    [sym_duet_dev_type_display_device] = ACTIONS(1531),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1531),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1531),
    [sym_duet_dev_type_io_device] = ACTIONS(1531),
    [sym_duet_dev_type_relay_device] = ACTIONS(1531),
    [sym_duet_dev_type_ups] = ACTIONS(1531),
    [sym_duet_dev_type_amplifier] = ACTIONS(1531),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1531),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1531),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1531),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1531),
    [sym_duet_dev_type_camera] = ACTIONS(1531),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1531),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1531),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1531),
    [sym_duet_dev_type_dss] = ACTIONS(1531),
    [sym_duet_dev_type_dvr] = ACTIONS(1531),
    [sym_duet_dev_type_disc_device] = ACTIONS(1531),
    [sym_duet_dev_type_document_camera] = ACTIONS(1531),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1531),
    [sym_duet_dev_type_hvac] = ACTIONS(1531),
    [sym_duet_dev_type_keypad] = ACTIONS(1531),
    [sym_duet_dev_type_light] = ACTIONS(1531),
    [sym_duet_dev_type_monitor] = ACTIONS(1531),
    [sym_duet_dev_type_motor] = ACTIONS(1531),
    [sym_duet_dev_type_multi_window] = ACTIONS(1531),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1531),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1531),
    [sym_duet_dev_type_receiver] = ACTIONS(1531),
    [sym_duet_dev_type_security_system] = ACTIONS(1531),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1531),
    [sym_duet_dev_type_settop_box] = ACTIONS(1531),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1531),
    [sym_duet_dev_type_switcher] = ACTIONS(1531),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1531),
    [sym_duet_dev_type_tv] = ACTIONS(1531),
    [sym_duet_dev_type_utility] = ACTIONS(1531),
    [sym_duet_dev_type_vcr] = ACTIONS(1531),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1531),
    [sym_duet_dev_type_video_processor] = ACTIONS(1531),
    [sym_duet_dev_type_video_projector] = ACTIONS(1531),
    [sym_duet_dev_type_video_wall] = ACTIONS(1531),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1531),
    [sym_duet_dev_type_weather] = ACTIONS(1531),
    [sym_duet_dev_not_polled] = ACTIONS(1531),
    [sym_duet_dev_polled] = ACTIONS(1531),
    [sym_clkmgr_mode_network] = ACTIONS(1531),
    [sym_clkmgr_mode_standalone] = ACTIONS(1531),
    [sym_normal_standby] = ACTIONS(1531),
    [sym_normal_wake] = ACTIONS(1531),
    [sym_ip_multicast_ttl_option] = ACTIONS(1531),
    [sym_ip_ttl_subnet] = ACTIONS(1531),
    [sym_ip_ttl_site] = ACTIONS(1531),
    [sym_ip_ttl_region] = ACTIONS(1531),
    [sym_ip_ttl_continent] = ACTIONS(1531),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1531),
    [sym_ip_nodelay_on] = ACTIONS(1531),
    [sym_ip_nodelay_off] = ACTIONS(1531),
    [sym_amx_error] = ACTIONS(1531),
    [sym_amx_warning] = ACTIONS(1531),
    [sym_amx_info] = ACTIONS(1531),
    [sym_amx_debug] = ACTIONS(1531),
    [sym_netlinx_login_success] = ACTIONS(1531),
    [sym_netlinx_login_fail] = ACTIONS(1531),
    [sym_netlinx_logout] = ACTIONS(1531),
    [sym_null_str] = ACTIONS(1531),
    [sym_smtp_address] = ACTIONS(1531),
    [sym_smtp_port_number] = ACTIONS(1531),
    [sym_smtp_username] = ACTIONS(1531),
    [sym_smtp_password] = ACTIONS(1531),
    [sym_smtp_require_tls] = ACTIONS(1531),
    [sym_smtp_from] = ACTIONS(1531),
    [sym_smtp_tls_true] = ACTIONS(1531),
    [sym_smtp_tls_false] = ACTIONS(1531),
    [sym_listview_on_row_select_event] = ACTIONS(1531),
    [sym_data_structure_unknown] = ACTIONS(1531),
    [sym_data_structure_datafeed] = ACTIONS(1531),
    [sym_data_structure_datarecordset] = ACTIONS(1531),
    [sym_data_structure_datarecord] = ACTIONS(1531),
    [sym_data_structure_datafield] = ACTIONS(1531),
    [sym_data_max_content_field_count] = ACTIONS(1531),
    [sym_data_max_metadata_field_count] = ACTIONS(1531),
    [sym_data_max_id_length] = ACTIONS(1531),
    [sym_data_max_name_length] = ACTIONS(1531),
    [sym_data_max_description_length] = ACTIONS(1531),
    [sym_data_max_type_length] = ACTIONS(1531),
    [sym_data_max_format_length] = ACTIONS(1531),
    [sym_data_max_source_length] = ACTIONS(1531),
    [sym_data_max_value_length] = ACTIONS(1531),
    [sym_data_max_label_length] = ACTIONS(1531),
    [sym_data_max_url_length] = ACTIONS(1531),
    [sym_max_last_login_info_length] = ACTIONS(1531),
    [sym_data_type_unknown] = ACTIONS(1531),
    [sym_data_type_string] = ACTIONS(1531),
    [sym_data_type_datetime] = ACTIONS(1531),
    [sym_data_type_date] = ACTIONS(1531),
    [sym_data_type_time] = ACTIONS(1531),
    [sym_data_type_image] = ACTIONS(1531),
    [sym_data_format_url] = ACTIONS(1531),
    [sym_data_format_phone] = ACTIONS(1531),
    [sym_data_format_email] = ACTIONS(1531),
    [sym_data_format_iso8601] = ACTIONS(1531),
    [sym_auditlog_priv] = ACTIONS(1531),
    [sym_configuration_priv] = ACTIONS(1531),
    [sym_deviceconfig_priv] = ACTIONS(1531),
    [sym_ftp_priv] = ACTIONS(1531),
    [sym_http_priv] = ACTIONS(1531),
    [sym_networkconfig_priv] = ACTIONS(1531),
    [sym_programport_priv] = ACTIONS(1531),
    [sym_remoteui_priv] = ACTIONS(1531),
    [sym_securitycontrol_priv] = ACTIONS(1531),
    [sym_softwaremanagement_priv] = ACTIONS(1531),
    [sym_terminal_priv] = ACTIONS(1531),
    [sym_tpadmin_priv] = ACTIONS(1531),
    [sym_user1_priv] = ACTIONS(1531),
    [sym_user2_priv] = ACTIONS(1531),
    [sym_user3_priv] = ACTIONS(1531),
    [sym_user4_priv] = ACTIONS(1531),
    [sym_usermanagement_priv] = ACTIONS(1531),
    [sym_valid_account] = ACTIONS(1531),
    [sym_err_name_invalid] = ACTIONS(1531),
    [sym_err_password_invalid] = ACTIONS(1531),
    [sym_err_invalid_account] = ACTIONS(1531),
    [sym_err_no_authorization] = ACTIONS(1531),
    [sym_err_invalid_parameter] = ACTIONS(1531),
    [sym_err_user_locked_out] = ACTIONS(1531),
    [sym_err_user_account_expired] = ACTIONS(1531),
    [sym_err_general_error] = ACTIONS(1531),
    [sym_err_authentication_error] = ACTIONS(1531),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1531),
    [sym_tls_validate_certificate] = ACTIONS(1531),
    [sym_tdata] = ACTIONS(1531),
    [sym_tchannel] = ACTIONS(1531),
    [sym_tlevel] = ACTIONS(1531),
    [sym_tbutton] = ACTIONS(1531),
    [sym_ttimeline] = ACTIONS(1531),
    [sym_tcustom] = ACTIONS(1531),
    [sym_url_struct] = ACTIONS(1531),
    [sym_dns_struct] = ACTIONS(1531),
    [sym_ip_address_struct] = ACTIONS(1531),
    [sym_dev_info_struct] = ACTIONS(1531),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1531),
    [sym_data_feed] = ACTIONS(1531),
    [sym_data_field] = ACTIONS(1531),
    [sym_data_record] = ACTIONS(1531),
    [sym_wc_data_feed] = ACTIONS(1531),
    [sym_wc_data_field] = ACTIONS(1531),
    [sym_wc_data_record] = ACTIONS(1531),
    [sym_last_login_info] = ACTIONS(1531),
    [sym____reserved___] = ACTIONS(1531),
    [sym_date] = ACTIONS(1531),
    [sym_day] = ACTIONS(1531),
    [sym_time] = ACTIONS(1531),
    [sym_ldate] = ACTIONS(1531),
    [sym_push_device] = ACTIONS(1531),
    [sym_push_channel] = ACTIONS(1531),
    [sym_push_devchan] = ACTIONS(1531),
    [sym_release_device] = ACTIONS(1531),
    [sym_release_channel] = ACTIONS(1531),
    [sym_release_devchan] = ACTIONS(1531),
    [sym_master_slot] = ACTIONS(1531),
    [sym_get_pulse_time] = ACTIONS(1531),
    [sym_get_timer] = ACTIONS(1531),
    [sym_master_sn] = ACTIONS(1531),
    [sym_system_number] = ACTIONS(1531),
    [sym_button] = ACTIONS(1531),
    [sym_dv_channel] = ACTIONS(1531),
    [sym_data] = ACTIONS(1531),
    [sym_level] = ACTIONS(1531),
    [sym_timeline] = ACTIONS(1531),
    [sym_channel] = ACTIONS(1531),
    [sym_custom] = ACTIONS(1531),
    [sym_atoi] = ACTIONS(1531),
    [sym_atol] = ACTIONS(1531),
    [sym_atof] = ACTIONS(1531),
    [sym_itoa] = ACTIONS(1531),
    [sym_itohex] = ACTIONS(1531),
    [sym_hextoi] = ACTIONS(1531),
    [sym_ftoa] = ACTIONS(1531),
    [sym_find_string] = ACTIONS(1531),
    [sym_left_string] = ACTIONS(1531),
    [sym_right_string] = ACTIONS(1531),
    [sym_mid_string] = ACTIONS(1531),
    [sym_remove_string] = ACTIONS(1531),
    [sym_get_buffer_string] = ACTIONS(1531),
    [sym_get_buffer_char] = ACTIONS(1531),
    [sym_get_multi_buffer_string] = ACTIONS(1531),
    [sym_length_string] = ACTIONS(1531),
    [sym_max_length_string] = ACTIONS(1531),
    [sym_set_length_string] = ACTIONS(1531),
    [sym_compare_string] = ACTIONS(1531),
    [sym_upper_string] = ACTIONS(1531),
    [sym_lower_string] = ACTIONS(1531),
    [sym_get_last] = ACTIONS(1531),
    [sym_date_to_day] = ACTIONS(1531),
    [sym_date_to_month] = ACTIONS(1531),
    [sym_date_to_year] = ACTIONS(1531),
    [sym_time_to_hour] = ACTIONS(1531),
    [sym_time_to_minute] = ACTIONS(1531),
    [sym_time_to_second] = ACTIONS(1531),
    [sym_day_of_week] = ACTIONS(1531),
    [sym_set_pulse_time] = ACTIONS(1531),
    [sym_set_timer] = ACTIONS(1531),
    [sym_file_open] = ACTIONS(1531),
    [sym_file_close] = ACTIONS(1531),
    [sym_file_read] = ACTIONS(1531),
    [sym_file_read_line] = ACTIONS(1531),
    [sym_file_write] = ACTIONS(1531),
    [sym_file_write_line] = ACTIONS(1531),
    [sym_file_copy] = ACTIONS(1531),
    [sym_file_delete] = ACTIONS(1531),
    [sym_file_rename] = ACTIONS(1531),
    [sym_file_dir] = ACTIONS(1531),
    [sym_file_getdir] = ACTIONS(1531),
    [sym_file_setdir] = ACTIONS(1531),
    [sym_file_create_dir] = ACTIONS(1531),
    [sym_file_remove_dir] = ACTIONS(1531),
    [sym_file_seek] = ACTIONS(1531),
    [sym_length_array] = ACTIONS(1531),
    [sym_max_length_array] = ACTIONS(1531),
    [sym_set_length_array] = ACTIONS(1531),
    [sym_ip_client_open] = ACTIONS(1531),
    [sym_ip_client_close] = ACTIONS(1531),
    [sym_ip_server_open] = ACTIONS(1531),
    [sym_ip_server_close] = ACTIONS(1531),
    [sym_ip_bound_client_open] = ACTIONS(1531),
    [sym_ip_mc_server_open] = ACTIONS(1531),
    [sym_ip_set_option] = ACTIONS(1531),
    [sym_ssh_client_open] = ACTIONS(1531),
    [sym_ssh_client_close] = ACTIONS(1531),
    [sym_tls_client_open] = ACTIONS(1531),
    [sym_tls_client_close] = ACTIONS(1531),
    [sym_do_push] = ACTIONS(1531),
    [sym_do_push_timed] = ACTIONS(1531),
    [sym_do_release] = ACTIONS(1531),
    [sym_device_id] = ACTIONS(1531),
    [sym_device_info] = ACTIONS(1531),
    [sym_device_id_string] = ACTIONS(1531),
    [sym_device_standby] = ACTIONS(1531),
    [sym_device_wake] = ACTIONS(1531),
    [sym_timeline_create] = ACTIONS(1531),
    [sym_timeline_kill] = ACTIONS(1531),
    [sym_timeline_reload] = ACTIONS(1531),
    [sym_timeline_pause] = ACTIONS(1531),
    [sym_timeline_restart] = ACTIONS(1531),
    [sym_timeline_set] = ACTIONS(1531),
    [sym_timeline_get] = ACTIONS(1531),
    [sym_timeline_active] = ACTIONS(1531),
    [sym_timeline_dynamic_id] = ACTIONS(1531),
    [sym_variable_to_string] = ACTIONS(1531),
    [sym_string_to_variable] = ACTIONS(1531),
    [sym_length_variable_to_string] = ACTIONS(1531),
    [sym_type_cast] = ACTIONS(1531),
    [sym_variable_to_xml] = ACTIONS(1531),
    [sym_xml_to_variable] = ACTIONS(1531),
    [sym_length_variable_to_xml] = ACTIONS(1531),
    [sym_redirect_string] = ACTIONS(1531),
    [sym_random_number] = ACTIONS(1531),
    [sym_get_unique_id] = ACTIONS(1531),
    [sym_get_system_number] = ACTIONS(1531),
    [sym_set_system_number] = ACTIONS(1531),
    [sym_get_serial_number] = ACTIONS(1531),
    [sym_get_master_build] = ACTIONS(1531),
    [sym_set_validation_code] = ACTIONS(1531),
    [sym_reboot] = ACTIONS(1531),
    [sym_load_duet_module] = ACTIONS(1531),
    [sym_unload_duet_module] = ACTIONS(1531),
    [sym_duet_mem_size_set] = ACTIONS(1531),
    [sym_duet_mem_size_get] = ACTIONS(1531),
    [sym_dynamic_application_device] = ACTIONS(1531),
    [sym_dynamic_polled_port] = ACTIONS(1531),
    [sym_static_port_binding] = ACTIONS(1531),
    [sym_static_ip_binding] = ACTIONS(1531),
    [sym_get_dns_list] = ACTIONS(1531),
    [sym_set_dns_list] = ACTIONS(1531),
    [sym_get_ip_address] = ACTIONS(1531),
    [sym_set_ip_address] = ACTIONS(1531),
    [sym_get_url_list] = ACTIONS(1531),
    [sym_add_url_entry] = ACTIONS(1531),
    [sym_delete_url_entry] = ACTIONS(1531),
    [sym_abs_value] = ACTIONS(1531),
    [sym_max_value] = ACTIONS(1531),
    [sym_min_value] = ACTIONS(1531),
    [sym_exp_value] = ACTIONS(1531),
    [sym_log_value] = ACTIONS(1531),
    [sym_log10_value] = ACTIONS(1531),
    [sym_power_value] = ACTIONS(1531),
    [sym_sqrt_value] = ACTIONS(1531),
    [sym_raw_be] = ACTIONS(1531),
    [sym_raw_le] = ACTIONS(1531),
    [sym_internal_queue_size_set] = ACTIONS(1531),
    [sym_internal_queue_size_get] = ACTIONS(1531),
    [sym_internal_threshold_set] = ACTIONS(1531),
    [sym_internal_threshold_get] = ACTIONS(1531),
    [sym_rebuilt_event] = ACTIONS(1531),
    [sym_do_custom_event] = ACTIONS(1531),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1531),
    [sym_clkmgr_set_clk_source] = ACTIONS(1531),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1531),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1531),
    [sym_clkmgr_get_timezone] = ACTIONS(1531),
    [sym_clkmgr_set_timezone] = ACTIONS(1531),
    [sym_clkmgr_get_resync_period] = ACTIONS(1531),
    [sym_clkmgr_set_resync_period] = ACTIONS(1531),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1531),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1531),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1531),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1531),
    [sym_clkmgr_get_timeservers] = ACTIONS(1531),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1531),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1531),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1531),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1531),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1531),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1531),
    [sym_set_log_level] = ACTIONS(1531),
    [sym_get_log_level] = ACTIONS(1531),
    [sym_amx_log] = ACTIONS(1531),
    [sym_smtp_server_config_set] = ACTIONS(1531),
    [sym_smtp_server_config_get] = ACTIONS(1531),
    [sym_smtp_send] = ACTIONS(1531),
    [sym_validate_netlinx_account] = ACTIONS(1531),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1531),
    [sym_authenticate_certificate] = ACTIONS(1531),
    [sym_audit_netlinx_session_event] = ACTIONS(1531),
    [sym_audit_netlinx_generic_event] = ACTIONS(1531),
    [sym_get_available_flash_disk_space] = ACTIONS(1531),
    [sym_get_max_flash_disk_space] = ACTIONS(1531),
    [sym_data_create_feed] = ACTIONS(1531),
    [sym_data_delete_feed] = ACTIONS(1531),
    [sym_data_publish_feed] = ACTIONS(1531),
    [sym_data_get_published_feed] = ACTIONS(1531),
    [sym_data_add_record] = ACTIONS(1531),
    [sym_data_get_event_record] = ACTIONS(1531),
    [sym__wc_data_create_feed] = ACTIONS(1531),
    [sym__wc_data_add_record] = ACTIONS(1531),
    [sym__wc_data_get_event_record] = ACTIONS(1531),
    [sym_format] = ACTIONS(1531),
    [sym_astro_clock] = ACTIONS(1531),
    [sym_set_outdoor_temperature] = ACTIONS(1531),
    [sym_set_virtual_level_count] = ACTIONS(1531),
    [sym_set_virtual_channel_count] = ACTIONS(1531),
    [sym_set_virtual_port_count] = ACTIONS(1531),
    [sym__wc_encode_int] = ACTIONS(1531),
    [sym__wc_decode_int] = ACTIONS(1531),
    [sym__wc_int] = ACTIONS(1531),
    [sym__wc_to_ch_int] = ACTIONS(1531),
    [sym__ch_to_wc_int] = ACTIONS(1531),
    [sym__wc_find_string_int] = ACTIONS(1531),
    [sym__wc_left_string_int] = ACTIONS(1531),
    [sym__wc_lower_string_int] = ACTIONS(1531),
    [sym__wc_mid_string_int] = ACTIONS(1531),
    [sym__wc_remove_string_int] = ACTIONS(1531),
    [sym__wc_right_string_int] = ACTIONS(1531),
    [sym__wc_upper_string_int] = ACTIONS(1531),
    [sym__wc_compare_string_int] = ACTIONS(1531),
    [sym__wc_get_buffer_char_int] = ACTIONS(1531),
    [sym__wc_get_buffer_string_int] = ACTIONS(1531),
    [sym__wc_concat_string_int] = ACTIONS(1531),
    [sym__wc_file_open_int] = ACTIONS(1531),
    [sym__wc_file_close_int] = ACTIONS(1531),
    [sym__wc_file_read_int] = ACTIONS(1531),
    [sym__wc_file_read_line_int] = ACTIONS(1531),
    [sym__wc_file_write_int] = ACTIONS(1531),
    [sym__wc_file_write_line_int] = ACTIONS(1531),
    [anon_sym_EQ] = ACTIONS(1531),
    [aux_sym_preproc_include_token1] = ACTIONS(1531),
    [aux_sym_preproc_define_token1] = ACTIONS(1533),
    [aux_sym_preproc_warn_token1] = ACTIONS(1533),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1533),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1533),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1533),
    [anon_sym_LBRACK] = ACTIONS(1533),
    [anon_sym_LBRACE] = ACTIONS(1533),
    [anon_sym_RBRACE] = ACTIONS(1533),
    [anon_sym_COLON] = ACTIONS(1533),
    [anon_sym_LPAREN] = ACTIONS(1531),
    [anon_sym_SEMI] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1531),
    [anon_sym_TILDE] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(1533),
    [anon_sym_SLASH] = ACTIONS(1531),
    [anon_sym_PERCENT] = ACTIONS(1533),
    [anon_sym_PIPE_PIPE] = ACTIONS(1533),
    [anon_sym_AMP_AMP] = ACTIONS(1533),
    [anon_sym_PIPE] = ACTIONS(1531),
    [anon_sym_CARET] = ACTIONS(1533),
    [anon_sym_AMP] = ACTIONS(1531),
    [anon_sym_EQ_EQ] = ACTIONS(1533),
    [anon_sym_BANG_EQ] = ACTIONS(1533),
    [anon_sym_LT_GT] = ACTIONS(1533),
    [anon_sym_GT] = ACTIONS(1531),
    [anon_sym_GT_EQ] = ACTIONS(1533),
    [anon_sym_LT_EQ] = ACTIONS(1533),
    [anon_sym_LT] = ACTIONS(1531),
    [anon_sym_LT_LT] = ACTIONS(1533),
    [anon_sym_GT_GT] = ACTIONS(1533),
    [anon_sym_DASH_DASH] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1533),
    [anon_sym_DOT] = ACTIONS(1531),
    [anon_sym_DQUOTE] = ACTIONS(1533),
    [anon_sym_SQUOTE] = ACTIONS(1533),
    [sym_number_literal] = ACTIONS(1533),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1533),
  },
  [STATE(182)] = {
    [sym_identifier] = ACTIONS(1465),
    [sym_define_call_keyword] = ACTIONS(1465),
    [sym_define_function_keyword] = ACTIONS(1465),
    [sym_define_library_function_keyword] = ACTIONS(1465),
    [sym_define_module_keyword] = ACTIONS(1465),
    [sym_char_keyword] = ACTIONS(1465),
    [sym_widechar_keyword] = ACTIONS(1465),
    [sym_integer_keyword] = ACTIONS(1465),
    [sym_sinteger_keyword] = ACTIONS(1465),
    [sym_long_keyword] = ACTIONS(1465),
    [sym_slong_keyword] = ACTIONS(1465),
    [sym_float_keyword] = ACTIONS(1465),
    [sym_double_keyword] = ACTIONS(1465),
    [sym_variant_keyword] = ACTIONS(1465),
    [sym_variantarray_keyword] = ACTIONS(1465),
    [sym_local_var_keyword] = ACTIONS(1465),
    [sym_stack_var_keyword] = ACTIONS(1465),
    [sym_constant_keyword] = ACTIONS(1465),
    [sym_volatile_keyword] = ACTIONS(1465),
    [sym_non_volatile_keyword] = ACTIONS(1465),
    [sym_persistent_keyword] = ACTIONS(1465),
    [sym_dev_keyword] = ACTIONS(1465),
    [sym_devlev_keyword] = ACTIONS(1465),
    [sym_devchan_keyword] = ACTIONS(1465),
    [sym_if_keyword] = ACTIONS(1465),
    [sym_else_keyword] = ACTIONS(1465),
    [sym_switch_keyword] = ACTIONS(1465),
    [sym_case_keyword] = ACTIONS(1465),
    [sym_default_keyword] = ACTIONS(1465),
    [sym_while_keyword] = ACTIONS(1465),
    [sym_for_keyword] = ACTIONS(1465),
    [sym_break_keyword] = ACTIONS(1465),
    [sym_continue_keyword] = ACTIONS(1465),
    [sym_return_keyword] = ACTIONS(1465),
    [sym_select_keyword] = ACTIONS(1465),
    [sym_active_keyword] = ACTIONS(1465),
    [sym_struct_keyword] = ACTIONS(1465),
    [sym_structure_keyword] = ACTIONS(1465),
    [aux_sym_on_keyword_token1] = ACTIONS(1465),
    [aux_sym_off_keyword_token1] = ACTIONS(1465),
    [sym_send_level_keyword] = ACTIONS(1465),
    [sym_send_string_keyword] = ACTIONS(1465),
    [sym_send_command_keyword] = ACTIONS(1465),
    [sym_clear_buffer_keyword] = ACTIONS(1465),
    [sym_create_buffer_keyword] = ACTIONS(1465),
    [sym_create_multi_buffer_keyword] = ACTIONS(1465),
    [sym_call_keyword] = ACTIONS(1465),
    [sym_system_call_keyword] = ACTIONS(1465),
    [sym_devchan_to_keyword] = ACTIONS(1465),
    [sym_devchan_min_to_keyword] = ACTIONS(1465),
    [sym_devchan_total_off_keyword] = ACTIONS(1465),
    [sym_devchan_pulse_keyword] = ACTIONS(1465),
    [sym_wait_keyword] = ACTIONS(1465),
    [sym_cancel_wait_keyword] = ACTIONS(1465),
    [sym_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_wait_until_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_keyword] = ACTIONS(1465),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym_tdata] = ACTIONS(1465),
    [sym_tchannel] = ACTIONS(1465),
    [sym_tlevel] = ACTIONS(1465),
    [sym_tbutton] = ACTIONS(1465),
    [sym_ttimeline] = ACTIONS(1465),
    [sym_tcustom] = ACTIONS(1465),
    [sym_url_struct] = ACTIONS(1465),
    [sym_dns_struct] = ACTIONS(1465),
    [sym_ip_address_struct] = ACTIONS(1465),
    [sym_dev_info_struct] = ACTIONS(1465),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1465),
    [sym_data_feed] = ACTIONS(1465),
    [sym_data_field] = ACTIONS(1465),
    [sym_data_record] = ACTIONS(1465),
    [sym_wc_data_feed] = ACTIONS(1465),
    [sym_wc_data_field] = ACTIONS(1465),
    [sym_wc_data_record] = ACTIONS(1465),
    [sym_last_login_info] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [aux_sym_preproc_include_token1] = ACTIONS(1465),
    [aux_sym_preproc_define_token1] = ACTIONS(1469),
    [aux_sym_preproc_warn_token1] = ACTIONS(1469),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_RBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_SEMI] = ACTIONS(1469),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1469),
  },
  [STATE(183)] = {
    [sym_identifier] = ACTIONS(1573),
    [sym_define_call_keyword] = ACTIONS(1573),
    [sym_define_function_keyword] = ACTIONS(1573),
    [sym_define_library_function_keyword] = ACTIONS(1573),
    [sym_define_module_keyword] = ACTIONS(1573),
    [sym_char_keyword] = ACTIONS(1573),
    [sym_widechar_keyword] = ACTIONS(1573),
    [sym_integer_keyword] = ACTIONS(1573),
    [sym_sinteger_keyword] = ACTIONS(1573),
    [sym_long_keyword] = ACTIONS(1573),
    [sym_slong_keyword] = ACTIONS(1573),
    [sym_float_keyword] = ACTIONS(1573),
    [sym_double_keyword] = ACTIONS(1573),
    [sym_variant_keyword] = ACTIONS(1573),
    [sym_variantarray_keyword] = ACTIONS(1573),
    [sym_local_var_keyword] = ACTIONS(1573),
    [sym_stack_var_keyword] = ACTIONS(1573),
    [sym_constant_keyword] = ACTIONS(1573),
    [sym_volatile_keyword] = ACTIONS(1573),
    [sym_non_volatile_keyword] = ACTIONS(1573),
    [sym_persistent_keyword] = ACTIONS(1573),
    [sym_dev_keyword] = ACTIONS(1573),
    [sym_devlev_keyword] = ACTIONS(1573),
    [sym_devchan_keyword] = ACTIONS(1573),
    [sym_if_keyword] = ACTIONS(1573),
    [sym_else_keyword] = ACTIONS(1573),
    [sym_switch_keyword] = ACTIONS(1573),
    [sym_case_keyword] = ACTIONS(1573),
    [sym_default_keyword] = ACTIONS(1573),
    [sym_while_keyword] = ACTIONS(1573),
    [sym_for_keyword] = ACTIONS(1573),
    [sym_break_keyword] = ACTIONS(1573),
    [sym_continue_keyword] = ACTIONS(1573),
    [sym_return_keyword] = ACTIONS(1573),
    [sym_select_keyword] = ACTIONS(1573),
    [sym_active_keyword] = ACTIONS(1573),
    [sym_struct_keyword] = ACTIONS(1573),
    [sym_structure_keyword] = ACTIONS(1573),
    [aux_sym_on_keyword_token1] = ACTIONS(1573),
    [aux_sym_off_keyword_token1] = ACTIONS(1573),
    [sym_send_level_keyword] = ACTIONS(1573),
    [sym_send_string_keyword] = ACTIONS(1573),
    [sym_send_command_keyword] = ACTIONS(1573),
    [sym_clear_buffer_keyword] = ACTIONS(1573),
    [sym_create_buffer_keyword] = ACTIONS(1573),
    [sym_create_multi_buffer_keyword] = ACTIONS(1573),
    [sym_call_keyword] = ACTIONS(1573),
    [sym_system_call_keyword] = ACTIONS(1573),
    [sym_devchan_to_keyword] = ACTIONS(1573),
    [sym_devchan_min_to_keyword] = ACTIONS(1573),
    [sym_devchan_total_off_keyword] = ACTIONS(1573),
    [sym_devchan_pulse_keyword] = ACTIONS(1573),
    [sym_wait_keyword] = ACTIONS(1573),
    [sym_cancel_wait_keyword] = ACTIONS(1573),
    [sym_wait_until_keyword] = ACTIONS(1573),
    [sym_cancel_wait_until_keyword] = ACTIONS(1573),
    [sym_cancel_all_wait_keyword] = ACTIONS(1573),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1573),
    [sym_and] = ACTIONS(1573),
    [sym_or] = ACTIONS(1573),
    [sym_not] = ACTIONS(1573),
    [sym_xor] = ACTIONS(1573),
    [sym_band] = ACTIONS(1573),
    [sym_bor] = ACTIONS(1573),
    [sym_bxor] = ACTIONS(1573),
    [sym_bnot] = ACTIONS(1573),
    [sym_lshift] = ACTIONS(1573),
    [sym_rshift] = ACTIONS(1573),
    [sym___file__] = ACTIONS(1573),
    [sym___date__] = ACTIONS(1573),
    [sym___ldate__] = ACTIONS(1573),
    [sym___time__] = ACTIONS(1573),
    [sym___line__] = ACTIONS(1573),
    [sym___name__] = ACTIONS(1573),
    [sym___netlinx__] = ACTIONS(1573),
    [sym_netlinx_axi_version] = ACTIONS(1573),
    [sym_url_flg_tcp] = ACTIONS(1573),
    [sym_url_flg_accinfopresent] = ACTIONS(1573),
    [sym_url_flg_temp] = ACTIONS(1573),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1573),
    [sym_url_flg_stat_mask] = ACTIONS(1573),
    [sym_url_flg_stat_lookup] = ACTIONS(1573),
    [sym_url_flg_stat_connecting] = ACTIONS(1573),
    [sym_url_flg_stat_waiting] = ACTIONS(1573),
    [sym_url_flg_stat_connected] = ACTIONS(1573),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1573),
    [sym_first_virtual_device] = ACTIONS(1573),
    [sym_first_local_port] = ACTIONS(1573),
    [sym_dynamic_virtual_device] = ACTIONS(1573),
    [sym_timeline_once] = ACTIONS(1573),
    [sym_timeline_repeat] = ACTIONS(1573),
    [sym_timeline_absolute] = ACTIONS(1573),
    [sym_timeline_relative] = ACTIONS(1573),
    [sym_do_push_timed_infinite] = ACTIONS(1573),
    [sym_source_type_no_address] = ACTIONS(1573),
    [sym_source_type_neuron_id] = ACTIONS(1573),
    [sym_source_type_ip_address] = ACTIONS(1573),
    [sym_source_type_axlink] = ACTIONS(1573),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1573),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1573),
    [sym_source_type_ip_socket_address] = ACTIONS(1573),
    [sym_source_type_rs232] = ACTIONS(1573),
    [sym_source_type_internal] = ACTIONS(1573),
    [sym_source_type_mac_address] = ACTIONS(1573),
    [sym_source_type_ipv4_port] = ACTIONS(1573),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1573),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1573),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1573),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1573),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1573),
    [sym_true] = ACTIONS(1573),
    [sym_false] = ACTIONS(1573),
    [sym_file_read_only] = ACTIONS(1573),
    [sym_file_rw_new] = ACTIONS(1573),
    [sym_file_rw_append] = ACTIONS(1573),
    [sym_ip_tcp] = ACTIONS(1573),
    [sym_ip_udp] = ACTIONS(1573),
    [sym_ip_udp_2way] = ACTIONS(1573),
    [sym_xml_encode_types] = ACTIONS(1573),
    [sym_xml_encode_char_as_list] = ACTIONS(1573),
    [sym_xml_encode_le] = ACTIONS(1573),
    [sym_xml_decode_types] = ACTIONS(1573),
    [sym_xml_decode_no_preserve] = ACTIONS(1573),
    [sym_duet_dev_type_display_device] = ACTIONS(1573),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1573),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1573),
    [sym_duet_dev_type_io_device] = ACTIONS(1573),
    [sym_duet_dev_type_relay_device] = ACTIONS(1573),
    [sym_duet_dev_type_ups] = ACTIONS(1573),
    [sym_duet_dev_type_amplifier] = ACTIONS(1573),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1573),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1573),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1573),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1573),
    [sym_duet_dev_type_camera] = ACTIONS(1573),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1573),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1573),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1573),
    [sym_duet_dev_type_dss] = ACTIONS(1573),
    [sym_duet_dev_type_dvr] = ACTIONS(1573),
    [sym_duet_dev_type_disc_device] = ACTIONS(1573),
    [sym_duet_dev_type_document_camera] = ACTIONS(1573),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1573),
    [sym_duet_dev_type_hvac] = ACTIONS(1573),
    [sym_duet_dev_type_keypad] = ACTIONS(1573),
    [sym_duet_dev_type_light] = ACTIONS(1573),
    [sym_duet_dev_type_monitor] = ACTIONS(1573),
    [sym_duet_dev_type_motor] = ACTIONS(1573),
    [sym_duet_dev_type_multi_window] = ACTIONS(1573),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1573),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1573),
    [sym_duet_dev_type_receiver] = ACTIONS(1573),
    [sym_duet_dev_type_security_system] = ACTIONS(1573),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1573),
    [sym_duet_dev_type_settop_box] = ACTIONS(1573),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1573),
    [sym_duet_dev_type_switcher] = ACTIONS(1573),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1573),
    [sym_duet_dev_type_tv] = ACTIONS(1573),
    [sym_duet_dev_type_utility] = ACTIONS(1573),
    [sym_duet_dev_type_vcr] = ACTIONS(1573),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1573),
    [sym_duet_dev_type_video_processor] = ACTIONS(1573),
    [sym_duet_dev_type_video_projector] = ACTIONS(1573),
    [sym_duet_dev_type_video_wall] = ACTIONS(1573),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1573),
    [sym_duet_dev_type_weather] = ACTIONS(1573),
    [sym_duet_dev_not_polled] = ACTIONS(1573),
    [sym_duet_dev_polled] = ACTIONS(1573),
    [sym_clkmgr_mode_network] = ACTIONS(1573),
    [sym_clkmgr_mode_standalone] = ACTIONS(1573),
    [sym_normal_standby] = ACTIONS(1573),
    [sym_normal_wake] = ACTIONS(1573),
    [sym_ip_multicast_ttl_option] = ACTIONS(1573),
    [sym_ip_ttl_subnet] = ACTIONS(1573),
    [sym_ip_ttl_site] = ACTIONS(1573),
    [sym_ip_ttl_region] = ACTIONS(1573),
    [sym_ip_ttl_continent] = ACTIONS(1573),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1573),
    [sym_ip_nodelay_on] = ACTIONS(1573),
    [sym_ip_nodelay_off] = ACTIONS(1573),
    [sym_amx_error] = ACTIONS(1573),
    [sym_amx_warning] = ACTIONS(1573),
    [sym_amx_info] = ACTIONS(1573),
    [sym_amx_debug] = ACTIONS(1573),
    [sym_netlinx_login_success] = ACTIONS(1573),
    [sym_netlinx_login_fail] = ACTIONS(1573),
    [sym_netlinx_logout] = ACTIONS(1573),
    [sym_null_str] = ACTIONS(1573),
    [sym_smtp_address] = ACTIONS(1573),
    [sym_smtp_port_number] = ACTIONS(1573),
    [sym_smtp_username] = ACTIONS(1573),
    [sym_smtp_password] = ACTIONS(1573),
    [sym_smtp_require_tls] = ACTIONS(1573),
    [sym_smtp_from] = ACTIONS(1573),
    [sym_smtp_tls_true] = ACTIONS(1573),
    [sym_smtp_tls_false] = ACTIONS(1573),
    [sym_listview_on_row_select_event] = ACTIONS(1573),
    [sym_data_structure_unknown] = ACTIONS(1573),
    [sym_data_structure_datafeed] = ACTIONS(1573),
    [sym_data_structure_datarecordset] = ACTIONS(1573),
    [sym_data_structure_datarecord] = ACTIONS(1573),
    [sym_data_structure_datafield] = ACTIONS(1573),
    [sym_data_max_content_field_count] = ACTIONS(1573),
    [sym_data_max_metadata_field_count] = ACTIONS(1573),
    [sym_data_max_id_length] = ACTIONS(1573),
    [sym_data_max_name_length] = ACTIONS(1573),
    [sym_data_max_description_length] = ACTIONS(1573),
    [sym_data_max_type_length] = ACTIONS(1573),
    [sym_data_max_format_length] = ACTIONS(1573),
    [sym_data_max_source_length] = ACTIONS(1573),
    [sym_data_max_value_length] = ACTIONS(1573),
    [sym_data_max_label_length] = ACTIONS(1573),
    [sym_data_max_url_length] = ACTIONS(1573),
    [sym_max_last_login_info_length] = ACTIONS(1573),
    [sym_data_type_unknown] = ACTIONS(1573),
    [sym_data_type_string] = ACTIONS(1573),
    [sym_data_type_datetime] = ACTIONS(1573),
    [sym_data_type_date] = ACTIONS(1573),
    [sym_data_type_time] = ACTIONS(1573),
    [sym_data_type_image] = ACTIONS(1573),
    [sym_data_format_url] = ACTIONS(1573),
    [sym_data_format_phone] = ACTIONS(1573),
    [sym_data_format_email] = ACTIONS(1573),
    [sym_data_format_iso8601] = ACTIONS(1573),
    [sym_auditlog_priv] = ACTIONS(1573),
    [sym_configuration_priv] = ACTIONS(1573),
    [sym_deviceconfig_priv] = ACTIONS(1573),
    [sym_ftp_priv] = ACTIONS(1573),
    [sym_http_priv] = ACTIONS(1573),
    [sym_networkconfig_priv] = ACTIONS(1573),
    [sym_programport_priv] = ACTIONS(1573),
    [sym_remoteui_priv] = ACTIONS(1573),
    [sym_securitycontrol_priv] = ACTIONS(1573),
    [sym_softwaremanagement_priv] = ACTIONS(1573),
    [sym_terminal_priv] = ACTIONS(1573),
    [sym_tpadmin_priv] = ACTIONS(1573),
    [sym_user1_priv] = ACTIONS(1573),
    [sym_user2_priv] = ACTIONS(1573),
    [sym_user3_priv] = ACTIONS(1573),
    [sym_user4_priv] = ACTIONS(1573),
    [sym_usermanagement_priv] = ACTIONS(1573),
    [sym_valid_account] = ACTIONS(1573),
    [sym_err_name_invalid] = ACTIONS(1573),
    [sym_err_password_invalid] = ACTIONS(1573),
    [sym_err_invalid_account] = ACTIONS(1573),
    [sym_err_no_authorization] = ACTIONS(1573),
    [sym_err_invalid_parameter] = ACTIONS(1573),
    [sym_err_user_locked_out] = ACTIONS(1573),
    [sym_err_user_account_expired] = ACTIONS(1573),
    [sym_err_general_error] = ACTIONS(1573),
    [sym_err_authentication_error] = ACTIONS(1573),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1573),
    [sym_tls_validate_certificate] = ACTIONS(1573),
    [sym_tdata] = ACTIONS(1573),
    [sym_tchannel] = ACTIONS(1573),
    [sym_tlevel] = ACTIONS(1573),
    [sym_tbutton] = ACTIONS(1573),
    [sym_ttimeline] = ACTIONS(1573),
    [sym_tcustom] = ACTIONS(1573),
    [sym_url_struct] = ACTIONS(1573),
    [sym_dns_struct] = ACTIONS(1573),
    [sym_ip_address_struct] = ACTIONS(1573),
    [sym_dev_info_struct] = ACTIONS(1573),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1573),
    [sym_data_feed] = ACTIONS(1573),
    [sym_data_field] = ACTIONS(1573),
    [sym_data_record] = ACTIONS(1573),
    [sym_wc_data_feed] = ACTIONS(1573),
    [sym_wc_data_field] = ACTIONS(1573),
    [sym_wc_data_record] = ACTIONS(1573),
    [sym_last_login_info] = ACTIONS(1573),
    [sym____reserved___] = ACTIONS(1573),
    [sym_date] = ACTIONS(1573),
    [sym_day] = ACTIONS(1573),
    [sym_time] = ACTIONS(1573),
    [sym_ldate] = ACTIONS(1573),
    [sym_push_device] = ACTIONS(1573),
    [sym_push_channel] = ACTIONS(1573),
    [sym_push_devchan] = ACTIONS(1573),
    [sym_release_device] = ACTIONS(1573),
    [sym_release_channel] = ACTIONS(1573),
    [sym_release_devchan] = ACTIONS(1573),
    [sym_master_slot] = ACTIONS(1573),
    [sym_get_pulse_time] = ACTIONS(1573),
    [sym_get_timer] = ACTIONS(1573),
    [sym_master_sn] = ACTIONS(1573),
    [sym_system_number] = ACTIONS(1573),
    [sym_button] = ACTIONS(1573),
    [sym_dv_channel] = ACTIONS(1573),
    [sym_data] = ACTIONS(1573),
    [sym_level] = ACTIONS(1573),
    [sym_timeline] = ACTIONS(1573),
    [sym_channel] = ACTIONS(1573),
    [sym_custom] = ACTIONS(1573),
    [sym_atoi] = ACTIONS(1573),
    [sym_atol] = ACTIONS(1573),
    [sym_atof] = ACTIONS(1573),
    [sym_itoa] = ACTIONS(1573),
    [sym_itohex] = ACTIONS(1573),
    [sym_hextoi] = ACTIONS(1573),
    [sym_ftoa] = ACTIONS(1573),
    [sym_find_string] = ACTIONS(1573),
    [sym_left_string] = ACTIONS(1573),
    [sym_right_string] = ACTIONS(1573),
    [sym_mid_string] = ACTIONS(1573),
    [sym_remove_string] = ACTIONS(1573),
    [sym_get_buffer_string] = ACTIONS(1573),
    [sym_get_buffer_char] = ACTIONS(1573),
    [sym_get_multi_buffer_string] = ACTIONS(1573),
    [sym_length_string] = ACTIONS(1573),
    [sym_max_length_string] = ACTIONS(1573),
    [sym_set_length_string] = ACTIONS(1573),
    [sym_compare_string] = ACTIONS(1573),
    [sym_upper_string] = ACTIONS(1573),
    [sym_lower_string] = ACTIONS(1573),
    [sym_get_last] = ACTIONS(1573),
    [sym_date_to_day] = ACTIONS(1573),
    [sym_date_to_month] = ACTIONS(1573),
    [sym_date_to_year] = ACTIONS(1573),
    [sym_time_to_hour] = ACTIONS(1573),
    [sym_time_to_minute] = ACTIONS(1573),
    [sym_time_to_second] = ACTIONS(1573),
    [sym_day_of_week] = ACTIONS(1573),
    [sym_set_pulse_time] = ACTIONS(1573),
    [sym_set_timer] = ACTIONS(1573),
    [sym_file_open] = ACTIONS(1573),
    [sym_file_close] = ACTIONS(1573),
    [sym_file_read] = ACTIONS(1573),
    [sym_file_read_line] = ACTIONS(1573),
    [sym_file_write] = ACTIONS(1573),
    [sym_file_write_line] = ACTIONS(1573),
    [sym_file_copy] = ACTIONS(1573),
    [sym_file_delete] = ACTIONS(1573),
    [sym_file_rename] = ACTIONS(1573),
    [sym_file_dir] = ACTIONS(1573),
    [sym_file_getdir] = ACTIONS(1573),
    [sym_file_setdir] = ACTIONS(1573),
    [sym_file_create_dir] = ACTIONS(1573),
    [sym_file_remove_dir] = ACTIONS(1573),
    [sym_file_seek] = ACTIONS(1573),
    [sym_length_array] = ACTIONS(1573),
    [sym_max_length_array] = ACTIONS(1573),
    [sym_set_length_array] = ACTIONS(1573),
    [sym_ip_client_open] = ACTIONS(1573),
    [sym_ip_client_close] = ACTIONS(1573),
    [sym_ip_server_open] = ACTIONS(1573),
    [sym_ip_server_close] = ACTIONS(1573),
    [sym_ip_bound_client_open] = ACTIONS(1573),
    [sym_ip_mc_server_open] = ACTIONS(1573),
    [sym_ip_set_option] = ACTIONS(1573),
    [sym_ssh_client_open] = ACTIONS(1573),
    [sym_ssh_client_close] = ACTIONS(1573),
    [sym_tls_client_open] = ACTIONS(1573),
    [sym_tls_client_close] = ACTIONS(1573),
    [sym_do_push] = ACTIONS(1573),
    [sym_do_push_timed] = ACTIONS(1573),
    [sym_do_release] = ACTIONS(1573),
    [sym_device_id] = ACTIONS(1573),
    [sym_device_info] = ACTIONS(1573),
    [sym_device_id_string] = ACTIONS(1573),
    [sym_device_standby] = ACTIONS(1573),
    [sym_device_wake] = ACTIONS(1573),
    [sym_timeline_create] = ACTIONS(1573),
    [sym_timeline_kill] = ACTIONS(1573),
    [sym_timeline_reload] = ACTIONS(1573),
    [sym_timeline_pause] = ACTIONS(1573),
    [sym_timeline_restart] = ACTIONS(1573),
    [sym_timeline_set] = ACTIONS(1573),
    [sym_timeline_get] = ACTIONS(1573),
    [sym_timeline_active] = ACTIONS(1573),
    [sym_timeline_dynamic_id] = ACTIONS(1573),
    [sym_variable_to_string] = ACTIONS(1573),
    [sym_string_to_variable] = ACTIONS(1573),
    [sym_length_variable_to_string] = ACTIONS(1573),
    [sym_type_cast] = ACTIONS(1573),
    [sym_variable_to_xml] = ACTIONS(1573),
    [sym_xml_to_variable] = ACTIONS(1573),
    [sym_length_variable_to_xml] = ACTIONS(1573),
    [sym_redirect_string] = ACTIONS(1573),
    [sym_random_number] = ACTIONS(1573),
    [sym_get_unique_id] = ACTIONS(1573),
    [sym_get_system_number] = ACTIONS(1573),
    [sym_set_system_number] = ACTIONS(1573),
    [sym_get_serial_number] = ACTIONS(1573),
    [sym_get_master_build] = ACTIONS(1573),
    [sym_set_validation_code] = ACTIONS(1573),
    [sym_reboot] = ACTIONS(1573),
    [sym_load_duet_module] = ACTIONS(1573),
    [sym_unload_duet_module] = ACTIONS(1573),
    [sym_duet_mem_size_set] = ACTIONS(1573),
    [sym_duet_mem_size_get] = ACTIONS(1573),
    [sym_dynamic_application_device] = ACTIONS(1573),
    [sym_dynamic_polled_port] = ACTIONS(1573),
    [sym_static_port_binding] = ACTIONS(1573),
    [sym_static_ip_binding] = ACTIONS(1573),
    [sym_get_dns_list] = ACTIONS(1573),
    [sym_set_dns_list] = ACTIONS(1573),
    [sym_get_ip_address] = ACTIONS(1573),
    [sym_set_ip_address] = ACTIONS(1573),
    [sym_get_url_list] = ACTIONS(1573),
    [sym_add_url_entry] = ACTIONS(1573),
    [sym_delete_url_entry] = ACTIONS(1573),
    [sym_abs_value] = ACTIONS(1573),
    [sym_max_value] = ACTIONS(1573),
    [sym_min_value] = ACTIONS(1573),
    [sym_exp_value] = ACTIONS(1573),
    [sym_log_value] = ACTIONS(1573),
    [sym_log10_value] = ACTIONS(1573),
    [sym_power_value] = ACTIONS(1573),
    [sym_sqrt_value] = ACTIONS(1573),
    [sym_raw_be] = ACTIONS(1573),
    [sym_raw_le] = ACTIONS(1573),
    [sym_internal_queue_size_set] = ACTIONS(1573),
    [sym_internal_queue_size_get] = ACTIONS(1573),
    [sym_internal_threshold_set] = ACTIONS(1573),
    [sym_internal_threshold_get] = ACTIONS(1573),
    [sym_rebuilt_event] = ACTIONS(1573),
    [sym_do_custom_event] = ACTIONS(1573),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1573),
    [sym_clkmgr_set_clk_source] = ACTIONS(1573),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1573),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1573),
    [sym_clkmgr_get_timezone] = ACTIONS(1573),
    [sym_clkmgr_set_timezone] = ACTIONS(1573),
    [sym_clkmgr_get_resync_period] = ACTIONS(1573),
    [sym_clkmgr_set_resync_period] = ACTIONS(1573),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1573),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1573),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1573),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1573),
    [sym_clkmgr_get_timeservers] = ACTIONS(1573),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1573),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1573),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1573),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1573),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1573),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1573),
    [sym_set_log_level] = ACTIONS(1573),
    [sym_get_log_level] = ACTIONS(1573),
    [sym_amx_log] = ACTIONS(1573),
    [sym_smtp_server_config_set] = ACTIONS(1573),
    [sym_smtp_server_config_get] = ACTIONS(1573),
    [sym_smtp_send] = ACTIONS(1573),
    [sym_validate_netlinx_account] = ACTIONS(1573),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1573),
    [sym_authenticate_certificate] = ACTIONS(1573),
    [sym_audit_netlinx_session_event] = ACTIONS(1573),
    [sym_audit_netlinx_generic_event] = ACTIONS(1573),
    [sym_get_available_flash_disk_space] = ACTIONS(1573),
    [sym_get_max_flash_disk_space] = ACTIONS(1573),
    [sym_data_create_feed] = ACTIONS(1573),
    [sym_data_delete_feed] = ACTIONS(1573),
    [sym_data_publish_feed] = ACTIONS(1573),
    [sym_data_get_published_feed] = ACTIONS(1573),
    [sym_data_add_record] = ACTIONS(1573),
    [sym_data_get_event_record] = ACTIONS(1573),
    [sym__wc_data_create_feed] = ACTIONS(1573),
    [sym__wc_data_add_record] = ACTIONS(1573),
    [sym__wc_data_get_event_record] = ACTIONS(1573),
    [sym_format] = ACTIONS(1573),
    [sym_astro_clock] = ACTIONS(1573),
    [sym_set_outdoor_temperature] = ACTIONS(1573),
    [sym_set_virtual_level_count] = ACTIONS(1573),
    [sym_set_virtual_channel_count] = ACTIONS(1573),
    [sym_set_virtual_port_count] = ACTIONS(1573),
    [sym__wc_encode_int] = ACTIONS(1573),
    [sym__wc_decode_int] = ACTIONS(1573),
    [sym__wc_int] = ACTIONS(1573),
    [sym__wc_to_ch_int] = ACTIONS(1573),
    [sym__ch_to_wc_int] = ACTIONS(1573),
    [sym__wc_find_string_int] = ACTIONS(1573),
    [sym__wc_left_string_int] = ACTIONS(1573),
    [sym__wc_lower_string_int] = ACTIONS(1573),
    [sym__wc_mid_string_int] = ACTIONS(1573),
    [sym__wc_remove_string_int] = ACTIONS(1573),
    [sym__wc_right_string_int] = ACTIONS(1573),
    [sym__wc_upper_string_int] = ACTIONS(1573),
    [sym__wc_compare_string_int] = ACTIONS(1573),
    [sym__wc_get_buffer_char_int] = ACTIONS(1573),
    [sym__wc_get_buffer_string_int] = ACTIONS(1573),
    [sym__wc_concat_string_int] = ACTIONS(1573),
    [sym__wc_file_open_int] = ACTIONS(1573),
    [sym__wc_file_close_int] = ACTIONS(1573),
    [sym__wc_file_read_int] = ACTIONS(1573),
    [sym__wc_file_read_line_int] = ACTIONS(1573),
    [sym__wc_file_write_int] = ACTIONS(1573),
    [sym__wc_file_write_line_int] = ACTIONS(1573),
    [aux_sym_preproc_include_token1] = ACTIONS(1573),
    [aux_sym_preproc_define_token1] = ACTIONS(1575),
    [aux_sym_preproc_warn_token1] = ACTIONS(1575),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1575),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1575),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1575),
    [anon_sym_LBRACK] = ACTIONS(1575),
    [anon_sym_LBRACE] = ACTIONS(1575),
    [anon_sym_RBRACE] = ACTIONS(1575),
    [anon_sym_COLON] = ACTIONS(1575),
    [anon_sym_LPAREN] = ACTIONS(1573),
    [anon_sym_SEMI] = ACTIONS(1575),
    [anon_sym_BANG] = ACTIONS(1573),
    [anon_sym_TILDE] = ACTIONS(1575),
    [anon_sym_DASH] = ACTIONS(1573),
    [anon_sym_PLUS] = ACTIONS(1573),
    [anon_sym_STAR] = ACTIONS(1575),
    [anon_sym_SLASH] = ACTIONS(1573),
    [anon_sym_PERCENT] = ACTIONS(1575),
    [anon_sym_PIPE_PIPE] = ACTIONS(1575),
    [anon_sym_AMP_AMP] = ACTIONS(1575),
    [anon_sym_PIPE] = ACTIONS(1573),
    [anon_sym_CARET] = ACTIONS(1575),
    [anon_sym_AMP] = ACTIONS(1573),
    [anon_sym_EQ_EQ] = ACTIONS(1575),
    [anon_sym_BANG_EQ] = ACTIONS(1575),
    [anon_sym_LT_GT] = ACTIONS(1575),
    [anon_sym_GT] = ACTIONS(1573),
    [anon_sym_GT_EQ] = ACTIONS(1575),
    [anon_sym_LT_EQ] = ACTIONS(1575),
    [anon_sym_LT] = ACTIONS(1573),
    [anon_sym_LT_LT] = ACTIONS(1575),
    [anon_sym_GT_GT] = ACTIONS(1575),
    [anon_sym_DASH_DASH] = ACTIONS(1575),
    [anon_sym_PLUS_PLUS] = ACTIONS(1575),
    [anon_sym_DOT] = ACTIONS(1573),
    [anon_sym_DQUOTE] = ACTIONS(1575),
    [anon_sym_SQUOTE] = ACTIONS(1575),
    [sym_number_literal] = ACTIONS(1575),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1575),
  },
  [STATE(184)] = {
    [sym_identifier] = ACTIONS(1589),
    [sym_define_call_keyword] = ACTIONS(1589),
    [sym_define_function_keyword] = ACTIONS(1589),
    [sym_define_library_function_keyword] = ACTIONS(1589),
    [sym_define_module_keyword] = ACTIONS(1589),
    [sym_char_keyword] = ACTIONS(1589),
    [sym_widechar_keyword] = ACTIONS(1589),
    [sym_integer_keyword] = ACTIONS(1589),
    [sym_sinteger_keyword] = ACTIONS(1589),
    [sym_long_keyword] = ACTIONS(1589),
    [sym_slong_keyword] = ACTIONS(1589),
    [sym_float_keyword] = ACTIONS(1589),
    [sym_double_keyword] = ACTIONS(1589),
    [sym_variant_keyword] = ACTIONS(1589),
    [sym_variantarray_keyword] = ACTIONS(1589),
    [sym_local_var_keyword] = ACTIONS(1589),
    [sym_stack_var_keyword] = ACTIONS(1589),
    [sym_constant_keyword] = ACTIONS(1589),
    [sym_volatile_keyword] = ACTIONS(1589),
    [sym_non_volatile_keyword] = ACTIONS(1589),
    [sym_persistent_keyword] = ACTIONS(1589),
    [sym_dev_keyword] = ACTIONS(1589),
    [sym_devlev_keyword] = ACTIONS(1589),
    [sym_devchan_keyword] = ACTIONS(1589),
    [sym_if_keyword] = ACTIONS(1589),
    [sym_else_keyword] = ACTIONS(1589),
    [sym_switch_keyword] = ACTIONS(1589),
    [sym_case_keyword] = ACTIONS(1589),
    [sym_default_keyword] = ACTIONS(1589),
    [sym_while_keyword] = ACTIONS(1589),
    [sym_for_keyword] = ACTIONS(1589),
    [sym_break_keyword] = ACTIONS(1589),
    [sym_continue_keyword] = ACTIONS(1589),
    [sym_return_keyword] = ACTIONS(1589),
    [sym_select_keyword] = ACTIONS(1589),
    [sym_active_keyword] = ACTIONS(1589),
    [sym_struct_keyword] = ACTIONS(1589),
    [sym_structure_keyword] = ACTIONS(1589),
    [aux_sym_on_keyword_token1] = ACTIONS(1589),
    [aux_sym_off_keyword_token1] = ACTIONS(1589),
    [sym_send_level_keyword] = ACTIONS(1589),
    [sym_send_string_keyword] = ACTIONS(1589),
    [sym_send_command_keyword] = ACTIONS(1589),
    [sym_clear_buffer_keyword] = ACTIONS(1589),
    [sym_create_buffer_keyword] = ACTIONS(1589),
    [sym_create_multi_buffer_keyword] = ACTIONS(1589),
    [sym_call_keyword] = ACTIONS(1589),
    [sym_system_call_keyword] = ACTIONS(1589),
    [sym_devchan_to_keyword] = ACTIONS(1589),
    [sym_devchan_min_to_keyword] = ACTIONS(1589),
    [sym_devchan_total_off_keyword] = ACTIONS(1589),
    [sym_devchan_pulse_keyword] = ACTIONS(1589),
    [sym_wait_keyword] = ACTIONS(1589),
    [sym_cancel_wait_keyword] = ACTIONS(1589),
    [sym_wait_until_keyword] = ACTIONS(1589),
    [sym_cancel_wait_until_keyword] = ACTIONS(1589),
    [sym_cancel_all_wait_keyword] = ACTIONS(1589),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1589),
    [sym_and] = ACTIONS(1589),
    [sym_or] = ACTIONS(1589),
    [sym_not] = ACTIONS(1589),
    [sym_xor] = ACTIONS(1589),
    [sym_band] = ACTIONS(1589),
    [sym_bor] = ACTIONS(1589),
    [sym_bxor] = ACTIONS(1589),
    [sym_bnot] = ACTIONS(1589),
    [sym_lshift] = ACTIONS(1589),
    [sym_rshift] = ACTIONS(1589),
    [sym___file__] = ACTIONS(1589),
    [sym___date__] = ACTIONS(1589),
    [sym___ldate__] = ACTIONS(1589),
    [sym___time__] = ACTIONS(1589),
    [sym___line__] = ACTIONS(1589),
    [sym___name__] = ACTIONS(1589),
    [sym___netlinx__] = ACTIONS(1589),
    [sym_netlinx_axi_version] = ACTIONS(1589),
    [sym_url_flg_tcp] = ACTIONS(1589),
    [sym_url_flg_accinfopresent] = ACTIONS(1589),
    [sym_url_flg_temp] = ACTIONS(1589),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1589),
    [sym_url_flg_stat_mask] = ACTIONS(1589),
    [sym_url_flg_stat_lookup] = ACTIONS(1589),
    [sym_url_flg_stat_connecting] = ACTIONS(1589),
    [sym_url_flg_stat_waiting] = ACTIONS(1589),
    [sym_url_flg_stat_connected] = ACTIONS(1589),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1589),
    [sym_first_virtual_device] = ACTIONS(1589),
    [sym_first_local_port] = ACTIONS(1589),
    [sym_dynamic_virtual_device] = ACTIONS(1589),
    [sym_timeline_once] = ACTIONS(1589),
    [sym_timeline_repeat] = ACTIONS(1589),
    [sym_timeline_absolute] = ACTIONS(1589),
    [sym_timeline_relative] = ACTIONS(1589),
    [sym_do_push_timed_infinite] = ACTIONS(1589),
    [sym_source_type_no_address] = ACTIONS(1589),
    [sym_source_type_neuron_id] = ACTIONS(1589),
    [sym_source_type_ip_address] = ACTIONS(1589),
    [sym_source_type_axlink] = ACTIONS(1589),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1589),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1589),
    [sym_source_type_ip_socket_address] = ACTIONS(1589),
    [sym_source_type_rs232] = ACTIONS(1589),
    [sym_source_type_internal] = ACTIONS(1589),
    [sym_source_type_mac_address] = ACTIONS(1589),
    [sym_source_type_ipv4_port] = ACTIONS(1589),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1589),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1589),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1589),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1589),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1589),
    [sym_true] = ACTIONS(1589),
    [sym_false] = ACTIONS(1589),
    [sym_file_read_only] = ACTIONS(1589),
    [sym_file_rw_new] = ACTIONS(1589),
    [sym_file_rw_append] = ACTIONS(1589),
    [sym_ip_tcp] = ACTIONS(1589),
    [sym_ip_udp] = ACTIONS(1589),
    [sym_ip_udp_2way] = ACTIONS(1589),
    [sym_xml_encode_types] = ACTIONS(1589),
    [sym_xml_encode_char_as_list] = ACTIONS(1589),
    [sym_xml_encode_le] = ACTIONS(1589),
    [sym_xml_decode_types] = ACTIONS(1589),
    [sym_xml_decode_no_preserve] = ACTIONS(1589),
    [sym_duet_dev_type_display_device] = ACTIONS(1589),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1589),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1589),
    [sym_duet_dev_type_io_device] = ACTIONS(1589),
    [sym_duet_dev_type_relay_device] = ACTIONS(1589),
    [sym_duet_dev_type_ups] = ACTIONS(1589),
    [sym_duet_dev_type_amplifier] = ACTIONS(1589),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1589),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1589),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1589),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1589),
    [sym_duet_dev_type_camera] = ACTIONS(1589),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1589),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1589),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1589),
    [sym_duet_dev_type_dss] = ACTIONS(1589),
    [sym_duet_dev_type_dvr] = ACTIONS(1589),
    [sym_duet_dev_type_disc_device] = ACTIONS(1589),
    [sym_duet_dev_type_document_camera] = ACTIONS(1589),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1589),
    [sym_duet_dev_type_hvac] = ACTIONS(1589),
    [sym_duet_dev_type_keypad] = ACTIONS(1589),
    [sym_duet_dev_type_light] = ACTIONS(1589),
    [sym_duet_dev_type_monitor] = ACTIONS(1589),
    [sym_duet_dev_type_motor] = ACTIONS(1589),
    [sym_duet_dev_type_multi_window] = ACTIONS(1589),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1589),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1589),
    [sym_duet_dev_type_receiver] = ACTIONS(1589),
    [sym_duet_dev_type_security_system] = ACTIONS(1589),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1589),
    [sym_duet_dev_type_settop_box] = ACTIONS(1589),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1589),
    [sym_duet_dev_type_switcher] = ACTIONS(1589),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1589),
    [sym_duet_dev_type_tv] = ACTIONS(1589),
    [sym_duet_dev_type_utility] = ACTIONS(1589),
    [sym_duet_dev_type_vcr] = ACTIONS(1589),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1589),
    [sym_duet_dev_type_video_processor] = ACTIONS(1589),
    [sym_duet_dev_type_video_projector] = ACTIONS(1589),
    [sym_duet_dev_type_video_wall] = ACTIONS(1589),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1589),
    [sym_duet_dev_type_weather] = ACTIONS(1589),
    [sym_duet_dev_not_polled] = ACTIONS(1589),
    [sym_duet_dev_polled] = ACTIONS(1589),
    [sym_clkmgr_mode_network] = ACTIONS(1589),
    [sym_clkmgr_mode_standalone] = ACTIONS(1589),
    [sym_normal_standby] = ACTIONS(1589),
    [sym_normal_wake] = ACTIONS(1589),
    [sym_ip_multicast_ttl_option] = ACTIONS(1589),
    [sym_ip_ttl_subnet] = ACTIONS(1589),
    [sym_ip_ttl_site] = ACTIONS(1589),
    [sym_ip_ttl_region] = ACTIONS(1589),
    [sym_ip_ttl_continent] = ACTIONS(1589),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1589),
    [sym_ip_nodelay_on] = ACTIONS(1589),
    [sym_ip_nodelay_off] = ACTIONS(1589),
    [sym_amx_error] = ACTIONS(1589),
    [sym_amx_warning] = ACTIONS(1589),
    [sym_amx_info] = ACTIONS(1589),
    [sym_amx_debug] = ACTIONS(1589),
    [sym_netlinx_login_success] = ACTIONS(1589),
    [sym_netlinx_login_fail] = ACTIONS(1589),
    [sym_netlinx_logout] = ACTIONS(1589),
    [sym_null_str] = ACTIONS(1589),
    [sym_smtp_address] = ACTIONS(1589),
    [sym_smtp_port_number] = ACTIONS(1589),
    [sym_smtp_username] = ACTIONS(1589),
    [sym_smtp_password] = ACTIONS(1589),
    [sym_smtp_require_tls] = ACTIONS(1589),
    [sym_smtp_from] = ACTIONS(1589),
    [sym_smtp_tls_true] = ACTIONS(1589),
    [sym_smtp_tls_false] = ACTIONS(1589),
    [sym_listview_on_row_select_event] = ACTIONS(1589),
    [sym_data_structure_unknown] = ACTIONS(1589),
    [sym_data_structure_datafeed] = ACTIONS(1589),
    [sym_data_structure_datarecordset] = ACTIONS(1589),
    [sym_data_structure_datarecord] = ACTIONS(1589),
    [sym_data_structure_datafield] = ACTIONS(1589),
    [sym_data_max_content_field_count] = ACTIONS(1589),
    [sym_data_max_metadata_field_count] = ACTIONS(1589),
    [sym_data_max_id_length] = ACTIONS(1589),
    [sym_data_max_name_length] = ACTIONS(1589),
    [sym_data_max_description_length] = ACTIONS(1589),
    [sym_data_max_type_length] = ACTIONS(1589),
    [sym_data_max_format_length] = ACTIONS(1589),
    [sym_data_max_source_length] = ACTIONS(1589),
    [sym_data_max_value_length] = ACTIONS(1589),
    [sym_data_max_label_length] = ACTIONS(1589),
    [sym_data_max_url_length] = ACTIONS(1589),
    [sym_max_last_login_info_length] = ACTIONS(1589),
    [sym_data_type_unknown] = ACTIONS(1589),
    [sym_data_type_string] = ACTIONS(1589),
    [sym_data_type_datetime] = ACTIONS(1589),
    [sym_data_type_date] = ACTIONS(1589),
    [sym_data_type_time] = ACTIONS(1589),
    [sym_data_type_image] = ACTIONS(1589),
    [sym_data_format_url] = ACTIONS(1589),
    [sym_data_format_phone] = ACTIONS(1589),
    [sym_data_format_email] = ACTIONS(1589),
    [sym_data_format_iso8601] = ACTIONS(1589),
    [sym_auditlog_priv] = ACTIONS(1589),
    [sym_configuration_priv] = ACTIONS(1589),
    [sym_deviceconfig_priv] = ACTIONS(1589),
    [sym_ftp_priv] = ACTIONS(1589),
    [sym_http_priv] = ACTIONS(1589),
    [sym_networkconfig_priv] = ACTIONS(1589),
    [sym_programport_priv] = ACTIONS(1589),
    [sym_remoteui_priv] = ACTIONS(1589),
    [sym_securitycontrol_priv] = ACTIONS(1589),
    [sym_softwaremanagement_priv] = ACTIONS(1589),
    [sym_terminal_priv] = ACTIONS(1589),
    [sym_tpadmin_priv] = ACTIONS(1589),
    [sym_user1_priv] = ACTIONS(1589),
    [sym_user2_priv] = ACTIONS(1589),
    [sym_user3_priv] = ACTIONS(1589),
    [sym_user4_priv] = ACTIONS(1589),
    [sym_usermanagement_priv] = ACTIONS(1589),
    [sym_valid_account] = ACTIONS(1589),
    [sym_err_name_invalid] = ACTIONS(1589),
    [sym_err_password_invalid] = ACTIONS(1589),
    [sym_err_invalid_account] = ACTIONS(1589),
    [sym_err_no_authorization] = ACTIONS(1589),
    [sym_err_invalid_parameter] = ACTIONS(1589),
    [sym_err_user_locked_out] = ACTIONS(1589),
    [sym_err_user_account_expired] = ACTIONS(1589),
    [sym_err_general_error] = ACTIONS(1589),
    [sym_err_authentication_error] = ACTIONS(1589),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1589),
    [sym_tls_validate_certificate] = ACTIONS(1589),
    [sym_tdata] = ACTIONS(1589),
    [sym_tchannel] = ACTIONS(1589),
    [sym_tlevel] = ACTIONS(1589),
    [sym_tbutton] = ACTIONS(1589),
    [sym_ttimeline] = ACTIONS(1589),
    [sym_tcustom] = ACTIONS(1589),
    [sym_url_struct] = ACTIONS(1589),
    [sym_dns_struct] = ACTIONS(1589),
    [sym_ip_address_struct] = ACTIONS(1589),
    [sym_dev_info_struct] = ACTIONS(1589),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1589),
    [sym_data_feed] = ACTIONS(1589),
    [sym_data_field] = ACTIONS(1589),
    [sym_data_record] = ACTIONS(1589),
    [sym_wc_data_feed] = ACTIONS(1589),
    [sym_wc_data_field] = ACTIONS(1589),
    [sym_wc_data_record] = ACTIONS(1589),
    [sym_last_login_info] = ACTIONS(1589),
    [sym____reserved___] = ACTIONS(1589),
    [sym_date] = ACTIONS(1589),
    [sym_day] = ACTIONS(1589),
    [sym_time] = ACTIONS(1589),
    [sym_ldate] = ACTIONS(1589),
    [sym_push_device] = ACTIONS(1589),
    [sym_push_channel] = ACTIONS(1589),
    [sym_push_devchan] = ACTIONS(1589),
    [sym_release_device] = ACTIONS(1589),
    [sym_release_channel] = ACTIONS(1589),
    [sym_release_devchan] = ACTIONS(1589),
    [sym_master_slot] = ACTIONS(1589),
    [sym_get_pulse_time] = ACTIONS(1589),
    [sym_get_timer] = ACTIONS(1589),
    [sym_master_sn] = ACTIONS(1589),
    [sym_system_number] = ACTIONS(1589),
    [sym_button] = ACTIONS(1589),
    [sym_dv_channel] = ACTIONS(1589),
    [sym_data] = ACTIONS(1589),
    [sym_level] = ACTIONS(1589),
    [sym_timeline] = ACTIONS(1589),
    [sym_channel] = ACTIONS(1589),
    [sym_custom] = ACTIONS(1589),
    [sym_atoi] = ACTIONS(1589),
    [sym_atol] = ACTIONS(1589),
    [sym_atof] = ACTIONS(1589),
    [sym_itoa] = ACTIONS(1589),
    [sym_itohex] = ACTIONS(1589),
    [sym_hextoi] = ACTIONS(1589),
    [sym_ftoa] = ACTIONS(1589),
    [sym_find_string] = ACTIONS(1589),
    [sym_left_string] = ACTIONS(1589),
    [sym_right_string] = ACTIONS(1589),
    [sym_mid_string] = ACTIONS(1589),
    [sym_remove_string] = ACTIONS(1589),
    [sym_get_buffer_string] = ACTIONS(1589),
    [sym_get_buffer_char] = ACTIONS(1589),
    [sym_get_multi_buffer_string] = ACTIONS(1589),
    [sym_length_string] = ACTIONS(1589),
    [sym_max_length_string] = ACTIONS(1589),
    [sym_set_length_string] = ACTIONS(1589),
    [sym_compare_string] = ACTIONS(1589),
    [sym_upper_string] = ACTIONS(1589),
    [sym_lower_string] = ACTIONS(1589),
    [sym_get_last] = ACTIONS(1589),
    [sym_date_to_day] = ACTIONS(1589),
    [sym_date_to_month] = ACTIONS(1589),
    [sym_date_to_year] = ACTIONS(1589),
    [sym_time_to_hour] = ACTIONS(1589),
    [sym_time_to_minute] = ACTIONS(1589),
    [sym_time_to_second] = ACTIONS(1589),
    [sym_day_of_week] = ACTIONS(1589),
    [sym_set_pulse_time] = ACTIONS(1589),
    [sym_set_timer] = ACTIONS(1589),
    [sym_file_open] = ACTIONS(1589),
    [sym_file_close] = ACTIONS(1589),
    [sym_file_read] = ACTIONS(1589),
    [sym_file_read_line] = ACTIONS(1589),
    [sym_file_write] = ACTIONS(1589),
    [sym_file_write_line] = ACTIONS(1589),
    [sym_file_copy] = ACTIONS(1589),
    [sym_file_delete] = ACTIONS(1589),
    [sym_file_rename] = ACTIONS(1589),
    [sym_file_dir] = ACTIONS(1589),
    [sym_file_getdir] = ACTIONS(1589),
    [sym_file_setdir] = ACTIONS(1589),
    [sym_file_create_dir] = ACTIONS(1589),
    [sym_file_remove_dir] = ACTIONS(1589),
    [sym_file_seek] = ACTIONS(1589),
    [sym_length_array] = ACTIONS(1589),
    [sym_max_length_array] = ACTIONS(1589),
    [sym_set_length_array] = ACTIONS(1589),
    [sym_ip_client_open] = ACTIONS(1589),
    [sym_ip_client_close] = ACTIONS(1589),
    [sym_ip_server_open] = ACTIONS(1589),
    [sym_ip_server_close] = ACTIONS(1589),
    [sym_ip_bound_client_open] = ACTIONS(1589),
    [sym_ip_mc_server_open] = ACTIONS(1589),
    [sym_ip_set_option] = ACTIONS(1589),
    [sym_ssh_client_open] = ACTIONS(1589),
    [sym_ssh_client_close] = ACTIONS(1589),
    [sym_tls_client_open] = ACTIONS(1589),
    [sym_tls_client_close] = ACTIONS(1589),
    [sym_do_push] = ACTIONS(1589),
    [sym_do_push_timed] = ACTIONS(1589),
    [sym_do_release] = ACTIONS(1589),
    [sym_device_id] = ACTIONS(1589),
    [sym_device_info] = ACTIONS(1589),
    [sym_device_id_string] = ACTIONS(1589),
    [sym_device_standby] = ACTIONS(1589),
    [sym_device_wake] = ACTIONS(1589),
    [sym_timeline_create] = ACTIONS(1589),
    [sym_timeline_kill] = ACTIONS(1589),
    [sym_timeline_reload] = ACTIONS(1589),
    [sym_timeline_pause] = ACTIONS(1589),
    [sym_timeline_restart] = ACTIONS(1589),
    [sym_timeline_set] = ACTIONS(1589),
    [sym_timeline_get] = ACTIONS(1589),
    [sym_timeline_active] = ACTIONS(1589),
    [sym_timeline_dynamic_id] = ACTIONS(1589),
    [sym_variable_to_string] = ACTIONS(1589),
    [sym_string_to_variable] = ACTIONS(1589),
    [sym_length_variable_to_string] = ACTIONS(1589),
    [sym_type_cast] = ACTIONS(1589),
    [sym_variable_to_xml] = ACTIONS(1589),
    [sym_xml_to_variable] = ACTIONS(1589),
    [sym_length_variable_to_xml] = ACTIONS(1589),
    [sym_redirect_string] = ACTIONS(1589),
    [sym_random_number] = ACTIONS(1589),
    [sym_get_unique_id] = ACTIONS(1589),
    [sym_get_system_number] = ACTIONS(1589),
    [sym_set_system_number] = ACTIONS(1589),
    [sym_get_serial_number] = ACTIONS(1589),
    [sym_get_master_build] = ACTIONS(1589),
    [sym_set_validation_code] = ACTIONS(1589),
    [sym_reboot] = ACTIONS(1589),
    [sym_load_duet_module] = ACTIONS(1589),
    [sym_unload_duet_module] = ACTIONS(1589),
    [sym_duet_mem_size_set] = ACTIONS(1589),
    [sym_duet_mem_size_get] = ACTIONS(1589),
    [sym_dynamic_application_device] = ACTIONS(1589),
    [sym_dynamic_polled_port] = ACTIONS(1589),
    [sym_static_port_binding] = ACTIONS(1589),
    [sym_static_ip_binding] = ACTIONS(1589),
    [sym_get_dns_list] = ACTIONS(1589),
    [sym_set_dns_list] = ACTIONS(1589),
    [sym_get_ip_address] = ACTIONS(1589),
    [sym_set_ip_address] = ACTIONS(1589),
    [sym_get_url_list] = ACTIONS(1589),
    [sym_add_url_entry] = ACTIONS(1589),
    [sym_delete_url_entry] = ACTIONS(1589),
    [sym_abs_value] = ACTIONS(1589),
    [sym_max_value] = ACTIONS(1589),
    [sym_min_value] = ACTIONS(1589),
    [sym_exp_value] = ACTIONS(1589),
    [sym_log_value] = ACTIONS(1589),
    [sym_log10_value] = ACTIONS(1589),
    [sym_power_value] = ACTIONS(1589),
    [sym_sqrt_value] = ACTIONS(1589),
    [sym_raw_be] = ACTIONS(1589),
    [sym_raw_le] = ACTIONS(1589),
    [sym_internal_queue_size_set] = ACTIONS(1589),
    [sym_internal_queue_size_get] = ACTIONS(1589),
    [sym_internal_threshold_set] = ACTIONS(1589),
    [sym_internal_threshold_get] = ACTIONS(1589),
    [sym_rebuilt_event] = ACTIONS(1589),
    [sym_do_custom_event] = ACTIONS(1589),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1589),
    [sym_clkmgr_set_clk_source] = ACTIONS(1589),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1589),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1589),
    [sym_clkmgr_get_timezone] = ACTIONS(1589),
    [sym_clkmgr_set_timezone] = ACTIONS(1589),
    [sym_clkmgr_get_resync_period] = ACTIONS(1589),
    [sym_clkmgr_set_resync_period] = ACTIONS(1589),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1589),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1589),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1589),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1589),
    [sym_clkmgr_get_timeservers] = ACTIONS(1589),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1589),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1589),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1589),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1589),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1589),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1589),
    [sym_set_log_level] = ACTIONS(1589),
    [sym_get_log_level] = ACTIONS(1589),
    [sym_amx_log] = ACTIONS(1589),
    [sym_smtp_server_config_set] = ACTIONS(1589),
    [sym_smtp_server_config_get] = ACTIONS(1589),
    [sym_smtp_send] = ACTIONS(1589),
    [sym_validate_netlinx_account] = ACTIONS(1589),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1589),
    [sym_authenticate_certificate] = ACTIONS(1589),
    [sym_audit_netlinx_session_event] = ACTIONS(1589),
    [sym_audit_netlinx_generic_event] = ACTIONS(1589),
    [sym_get_available_flash_disk_space] = ACTIONS(1589),
    [sym_get_max_flash_disk_space] = ACTIONS(1589),
    [sym_data_create_feed] = ACTIONS(1589),
    [sym_data_delete_feed] = ACTIONS(1589),
    [sym_data_publish_feed] = ACTIONS(1589),
    [sym_data_get_published_feed] = ACTIONS(1589),
    [sym_data_add_record] = ACTIONS(1589),
    [sym_data_get_event_record] = ACTIONS(1589),
    [sym__wc_data_create_feed] = ACTIONS(1589),
    [sym__wc_data_add_record] = ACTIONS(1589),
    [sym__wc_data_get_event_record] = ACTIONS(1589),
    [sym_format] = ACTIONS(1589),
    [sym_astro_clock] = ACTIONS(1589),
    [sym_set_outdoor_temperature] = ACTIONS(1589),
    [sym_set_virtual_level_count] = ACTIONS(1589),
    [sym_set_virtual_channel_count] = ACTIONS(1589),
    [sym_set_virtual_port_count] = ACTIONS(1589),
    [sym__wc_encode_int] = ACTIONS(1589),
    [sym__wc_decode_int] = ACTIONS(1589),
    [sym__wc_int] = ACTIONS(1589),
    [sym__wc_to_ch_int] = ACTIONS(1589),
    [sym__ch_to_wc_int] = ACTIONS(1589),
    [sym__wc_find_string_int] = ACTIONS(1589),
    [sym__wc_left_string_int] = ACTIONS(1589),
    [sym__wc_lower_string_int] = ACTIONS(1589),
    [sym__wc_mid_string_int] = ACTIONS(1589),
    [sym__wc_remove_string_int] = ACTIONS(1589),
    [sym__wc_right_string_int] = ACTIONS(1589),
    [sym__wc_upper_string_int] = ACTIONS(1589),
    [sym__wc_compare_string_int] = ACTIONS(1589),
    [sym__wc_get_buffer_char_int] = ACTIONS(1589),
    [sym__wc_get_buffer_string_int] = ACTIONS(1589),
    [sym__wc_concat_string_int] = ACTIONS(1589),
    [sym__wc_file_open_int] = ACTIONS(1589),
    [sym__wc_file_close_int] = ACTIONS(1589),
    [sym__wc_file_read_int] = ACTIONS(1589),
    [sym__wc_file_read_line_int] = ACTIONS(1589),
    [sym__wc_file_write_int] = ACTIONS(1589),
    [sym__wc_file_write_line_int] = ACTIONS(1589),
    [aux_sym_preproc_include_token1] = ACTIONS(1589),
    [aux_sym_preproc_define_token1] = ACTIONS(1591),
    [aux_sym_preproc_warn_token1] = ACTIONS(1591),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1591),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1591),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1591),
    [anon_sym_LBRACK] = ACTIONS(1591),
    [anon_sym_LBRACE] = ACTIONS(1591),
    [anon_sym_RBRACE] = ACTIONS(1591),
    [anon_sym_COLON] = ACTIONS(1591),
    [anon_sym_LPAREN] = ACTIONS(1589),
    [anon_sym_SEMI] = ACTIONS(1591),
    [anon_sym_BANG] = ACTIONS(1589),
    [anon_sym_TILDE] = ACTIONS(1591),
    [anon_sym_DASH] = ACTIONS(1589),
    [anon_sym_PLUS] = ACTIONS(1589),
    [anon_sym_STAR] = ACTIONS(1591),
    [anon_sym_SLASH] = ACTIONS(1589),
    [anon_sym_PERCENT] = ACTIONS(1591),
    [anon_sym_PIPE_PIPE] = ACTIONS(1591),
    [anon_sym_AMP_AMP] = ACTIONS(1591),
    [anon_sym_PIPE] = ACTIONS(1589),
    [anon_sym_CARET] = ACTIONS(1591),
    [anon_sym_AMP] = ACTIONS(1589),
    [anon_sym_EQ_EQ] = ACTIONS(1591),
    [anon_sym_BANG_EQ] = ACTIONS(1591),
    [anon_sym_LT_GT] = ACTIONS(1591),
    [anon_sym_GT] = ACTIONS(1589),
    [anon_sym_GT_EQ] = ACTIONS(1591),
    [anon_sym_LT_EQ] = ACTIONS(1591),
    [anon_sym_LT] = ACTIONS(1589),
    [anon_sym_LT_LT] = ACTIONS(1591),
    [anon_sym_GT_GT] = ACTIONS(1591),
    [anon_sym_DASH_DASH] = ACTIONS(1591),
    [anon_sym_PLUS_PLUS] = ACTIONS(1591),
    [anon_sym_DOT] = ACTIONS(1589),
    [anon_sym_DQUOTE] = ACTIONS(1591),
    [anon_sym_SQUOTE] = ACTIONS(1591),
    [sym_number_literal] = ACTIONS(1591),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1591),
  },
  [STATE(185)] = {
    [sym_identifier] = ACTIONS(1561),
    [sym_define_call_keyword] = ACTIONS(1561),
    [sym_define_function_keyword] = ACTIONS(1561),
    [sym_define_library_function_keyword] = ACTIONS(1561),
    [sym_define_module_keyword] = ACTIONS(1561),
    [sym_char_keyword] = ACTIONS(1561),
    [sym_widechar_keyword] = ACTIONS(1561),
    [sym_integer_keyword] = ACTIONS(1561),
    [sym_sinteger_keyword] = ACTIONS(1561),
    [sym_long_keyword] = ACTIONS(1561),
    [sym_slong_keyword] = ACTIONS(1561),
    [sym_float_keyword] = ACTIONS(1561),
    [sym_double_keyword] = ACTIONS(1561),
    [sym_variant_keyword] = ACTIONS(1561),
    [sym_variantarray_keyword] = ACTIONS(1561),
    [sym_local_var_keyword] = ACTIONS(1561),
    [sym_stack_var_keyword] = ACTIONS(1561),
    [sym_constant_keyword] = ACTIONS(1561),
    [sym_volatile_keyword] = ACTIONS(1561),
    [sym_non_volatile_keyword] = ACTIONS(1561),
    [sym_persistent_keyword] = ACTIONS(1561),
    [sym_dev_keyword] = ACTIONS(1561),
    [sym_devlev_keyword] = ACTIONS(1561),
    [sym_devchan_keyword] = ACTIONS(1561),
    [sym_if_keyword] = ACTIONS(1561),
    [sym_else_keyword] = ACTIONS(1561),
    [sym_switch_keyword] = ACTIONS(1561),
    [sym_case_keyword] = ACTIONS(1561),
    [sym_default_keyword] = ACTIONS(1561),
    [sym_while_keyword] = ACTIONS(1561),
    [sym_for_keyword] = ACTIONS(1561),
    [sym_break_keyword] = ACTIONS(1561),
    [sym_continue_keyword] = ACTIONS(1561),
    [sym_return_keyword] = ACTIONS(1561),
    [sym_select_keyword] = ACTIONS(1561),
    [sym_active_keyword] = ACTIONS(1561),
    [sym_struct_keyword] = ACTIONS(1561),
    [sym_structure_keyword] = ACTIONS(1561),
    [aux_sym_on_keyword_token1] = ACTIONS(1561),
    [aux_sym_off_keyword_token1] = ACTIONS(1561),
    [sym_send_level_keyword] = ACTIONS(1561),
    [sym_send_string_keyword] = ACTIONS(1561),
    [sym_send_command_keyword] = ACTIONS(1561),
    [sym_clear_buffer_keyword] = ACTIONS(1561),
    [sym_create_buffer_keyword] = ACTIONS(1561),
    [sym_create_multi_buffer_keyword] = ACTIONS(1561),
    [sym_call_keyword] = ACTIONS(1561),
    [sym_system_call_keyword] = ACTIONS(1561),
    [sym_devchan_to_keyword] = ACTIONS(1561),
    [sym_devchan_min_to_keyword] = ACTIONS(1561),
    [sym_devchan_total_off_keyword] = ACTIONS(1561),
    [sym_devchan_pulse_keyword] = ACTIONS(1561),
    [sym_wait_keyword] = ACTIONS(1561),
    [sym_cancel_wait_keyword] = ACTIONS(1561),
    [sym_wait_until_keyword] = ACTIONS(1561),
    [sym_cancel_wait_until_keyword] = ACTIONS(1561),
    [sym_cancel_all_wait_keyword] = ACTIONS(1561),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1561),
    [sym_and] = ACTIONS(1561),
    [sym_or] = ACTIONS(1561),
    [sym_not] = ACTIONS(1561),
    [sym_xor] = ACTIONS(1561),
    [sym_band] = ACTIONS(1561),
    [sym_bor] = ACTIONS(1561),
    [sym_bxor] = ACTIONS(1561),
    [sym_bnot] = ACTIONS(1561),
    [sym_lshift] = ACTIONS(1561),
    [sym_rshift] = ACTIONS(1561),
    [sym___file__] = ACTIONS(1561),
    [sym___date__] = ACTIONS(1561),
    [sym___ldate__] = ACTIONS(1561),
    [sym___time__] = ACTIONS(1561),
    [sym___line__] = ACTIONS(1561),
    [sym___name__] = ACTIONS(1561),
    [sym___netlinx__] = ACTIONS(1561),
    [sym_netlinx_axi_version] = ACTIONS(1561),
    [sym_url_flg_tcp] = ACTIONS(1561),
    [sym_url_flg_accinfopresent] = ACTIONS(1561),
    [sym_url_flg_temp] = ACTIONS(1561),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1561),
    [sym_url_flg_stat_mask] = ACTIONS(1561),
    [sym_url_flg_stat_lookup] = ACTIONS(1561),
    [sym_url_flg_stat_connecting] = ACTIONS(1561),
    [sym_url_flg_stat_waiting] = ACTIONS(1561),
    [sym_url_flg_stat_connected] = ACTIONS(1561),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1561),
    [sym_first_virtual_device] = ACTIONS(1561),
    [sym_first_local_port] = ACTIONS(1561),
    [sym_dynamic_virtual_device] = ACTIONS(1561),
    [sym_timeline_once] = ACTIONS(1561),
    [sym_timeline_repeat] = ACTIONS(1561),
    [sym_timeline_absolute] = ACTIONS(1561),
    [sym_timeline_relative] = ACTIONS(1561),
    [sym_do_push_timed_infinite] = ACTIONS(1561),
    [sym_source_type_no_address] = ACTIONS(1561),
    [sym_source_type_neuron_id] = ACTIONS(1561),
    [sym_source_type_ip_address] = ACTIONS(1561),
    [sym_source_type_axlink] = ACTIONS(1561),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1561),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1561),
    [sym_source_type_ip_socket_address] = ACTIONS(1561),
    [sym_source_type_rs232] = ACTIONS(1561),
    [sym_source_type_internal] = ACTIONS(1561),
    [sym_source_type_mac_address] = ACTIONS(1561),
    [sym_source_type_ipv4_port] = ACTIONS(1561),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1561),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1561),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1561),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1561),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1561),
    [sym_true] = ACTIONS(1561),
    [sym_false] = ACTIONS(1561),
    [sym_file_read_only] = ACTIONS(1561),
    [sym_file_rw_new] = ACTIONS(1561),
    [sym_file_rw_append] = ACTIONS(1561),
    [sym_ip_tcp] = ACTIONS(1561),
    [sym_ip_udp] = ACTIONS(1561),
    [sym_ip_udp_2way] = ACTIONS(1561),
    [sym_xml_encode_types] = ACTIONS(1561),
    [sym_xml_encode_char_as_list] = ACTIONS(1561),
    [sym_xml_encode_le] = ACTIONS(1561),
    [sym_xml_decode_types] = ACTIONS(1561),
    [sym_xml_decode_no_preserve] = ACTIONS(1561),
    [sym_duet_dev_type_display_device] = ACTIONS(1561),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1561),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1561),
    [sym_duet_dev_type_io_device] = ACTIONS(1561),
    [sym_duet_dev_type_relay_device] = ACTIONS(1561),
    [sym_duet_dev_type_ups] = ACTIONS(1561),
    [sym_duet_dev_type_amplifier] = ACTIONS(1561),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1561),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1561),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1561),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1561),
    [sym_duet_dev_type_camera] = ACTIONS(1561),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1561),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1561),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1561),
    [sym_duet_dev_type_dss] = ACTIONS(1561),
    [sym_duet_dev_type_dvr] = ACTIONS(1561),
    [sym_duet_dev_type_disc_device] = ACTIONS(1561),
    [sym_duet_dev_type_document_camera] = ACTIONS(1561),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1561),
    [sym_duet_dev_type_hvac] = ACTIONS(1561),
    [sym_duet_dev_type_keypad] = ACTIONS(1561),
    [sym_duet_dev_type_light] = ACTIONS(1561),
    [sym_duet_dev_type_monitor] = ACTIONS(1561),
    [sym_duet_dev_type_motor] = ACTIONS(1561),
    [sym_duet_dev_type_multi_window] = ACTIONS(1561),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1561),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1561),
    [sym_duet_dev_type_receiver] = ACTIONS(1561),
    [sym_duet_dev_type_security_system] = ACTIONS(1561),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1561),
    [sym_duet_dev_type_settop_box] = ACTIONS(1561),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1561),
    [sym_duet_dev_type_switcher] = ACTIONS(1561),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1561),
    [sym_duet_dev_type_tv] = ACTIONS(1561),
    [sym_duet_dev_type_utility] = ACTIONS(1561),
    [sym_duet_dev_type_vcr] = ACTIONS(1561),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1561),
    [sym_duet_dev_type_video_processor] = ACTIONS(1561),
    [sym_duet_dev_type_video_projector] = ACTIONS(1561),
    [sym_duet_dev_type_video_wall] = ACTIONS(1561),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1561),
    [sym_duet_dev_type_weather] = ACTIONS(1561),
    [sym_duet_dev_not_polled] = ACTIONS(1561),
    [sym_duet_dev_polled] = ACTIONS(1561),
    [sym_clkmgr_mode_network] = ACTIONS(1561),
    [sym_clkmgr_mode_standalone] = ACTIONS(1561),
    [sym_normal_standby] = ACTIONS(1561),
    [sym_normal_wake] = ACTIONS(1561),
    [sym_ip_multicast_ttl_option] = ACTIONS(1561),
    [sym_ip_ttl_subnet] = ACTIONS(1561),
    [sym_ip_ttl_site] = ACTIONS(1561),
    [sym_ip_ttl_region] = ACTIONS(1561),
    [sym_ip_ttl_continent] = ACTIONS(1561),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1561),
    [sym_ip_nodelay_on] = ACTIONS(1561),
    [sym_ip_nodelay_off] = ACTIONS(1561),
    [sym_amx_error] = ACTIONS(1561),
    [sym_amx_warning] = ACTIONS(1561),
    [sym_amx_info] = ACTIONS(1561),
    [sym_amx_debug] = ACTIONS(1561),
    [sym_netlinx_login_success] = ACTIONS(1561),
    [sym_netlinx_login_fail] = ACTIONS(1561),
    [sym_netlinx_logout] = ACTIONS(1561),
    [sym_null_str] = ACTIONS(1561),
    [sym_smtp_address] = ACTIONS(1561),
    [sym_smtp_port_number] = ACTIONS(1561),
    [sym_smtp_username] = ACTIONS(1561),
    [sym_smtp_password] = ACTIONS(1561),
    [sym_smtp_require_tls] = ACTIONS(1561),
    [sym_smtp_from] = ACTIONS(1561),
    [sym_smtp_tls_true] = ACTIONS(1561),
    [sym_smtp_tls_false] = ACTIONS(1561),
    [sym_listview_on_row_select_event] = ACTIONS(1561),
    [sym_data_structure_unknown] = ACTIONS(1561),
    [sym_data_structure_datafeed] = ACTIONS(1561),
    [sym_data_structure_datarecordset] = ACTIONS(1561),
    [sym_data_structure_datarecord] = ACTIONS(1561),
    [sym_data_structure_datafield] = ACTIONS(1561),
    [sym_data_max_content_field_count] = ACTIONS(1561),
    [sym_data_max_metadata_field_count] = ACTIONS(1561),
    [sym_data_max_id_length] = ACTIONS(1561),
    [sym_data_max_name_length] = ACTIONS(1561),
    [sym_data_max_description_length] = ACTIONS(1561),
    [sym_data_max_type_length] = ACTIONS(1561),
    [sym_data_max_format_length] = ACTIONS(1561),
    [sym_data_max_source_length] = ACTIONS(1561),
    [sym_data_max_value_length] = ACTIONS(1561),
    [sym_data_max_label_length] = ACTIONS(1561),
    [sym_data_max_url_length] = ACTIONS(1561),
    [sym_max_last_login_info_length] = ACTIONS(1561),
    [sym_data_type_unknown] = ACTIONS(1561),
    [sym_data_type_string] = ACTIONS(1561),
    [sym_data_type_datetime] = ACTIONS(1561),
    [sym_data_type_date] = ACTIONS(1561),
    [sym_data_type_time] = ACTIONS(1561),
    [sym_data_type_image] = ACTIONS(1561),
    [sym_data_format_url] = ACTIONS(1561),
    [sym_data_format_phone] = ACTIONS(1561),
    [sym_data_format_email] = ACTIONS(1561),
    [sym_data_format_iso8601] = ACTIONS(1561),
    [sym_auditlog_priv] = ACTIONS(1561),
    [sym_configuration_priv] = ACTIONS(1561),
    [sym_deviceconfig_priv] = ACTIONS(1561),
    [sym_ftp_priv] = ACTIONS(1561),
    [sym_http_priv] = ACTIONS(1561),
    [sym_networkconfig_priv] = ACTIONS(1561),
    [sym_programport_priv] = ACTIONS(1561),
    [sym_remoteui_priv] = ACTIONS(1561),
    [sym_securitycontrol_priv] = ACTIONS(1561),
    [sym_softwaremanagement_priv] = ACTIONS(1561),
    [sym_terminal_priv] = ACTIONS(1561),
    [sym_tpadmin_priv] = ACTIONS(1561),
    [sym_user1_priv] = ACTIONS(1561),
    [sym_user2_priv] = ACTIONS(1561),
    [sym_user3_priv] = ACTIONS(1561),
    [sym_user4_priv] = ACTIONS(1561),
    [sym_usermanagement_priv] = ACTIONS(1561),
    [sym_valid_account] = ACTIONS(1561),
    [sym_err_name_invalid] = ACTIONS(1561),
    [sym_err_password_invalid] = ACTIONS(1561),
    [sym_err_invalid_account] = ACTIONS(1561),
    [sym_err_no_authorization] = ACTIONS(1561),
    [sym_err_invalid_parameter] = ACTIONS(1561),
    [sym_err_user_locked_out] = ACTIONS(1561),
    [sym_err_user_account_expired] = ACTIONS(1561),
    [sym_err_general_error] = ACTIONS(1561),
    [sym_err_authentication_error] = ACTIONS(1561),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1561),
    [sym_tls_validate_certificate] = ACTIONS(1561),
    [sym_tdata] = ACTIONS(1561),
    [sym_tchannel] = ACTIONS(1561),
    [sym_tlevel] = ACTIONS(1561),
    [sym_tbutton] = ACTIONS(1561),
    [sym_ttimeline] = ACTIONS(1561),
    [sym_tcustom] = ACTIONS(1561),
    [sym_url_struct] = ACTIONS(1561),
    [sym_dns_struct] = ACTIONS(1561),
    [sym_ip_address_struct] = ACTIONS(1561),
    [sym_dev_info_struct] = ACTIONS(1561),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1561),
    [sym_data_feed] = ACTIONS(1561),
    [sym_data_field] = ACTIONS(1561),
    [sym_data_record] = ACTIONS(1561),
    [sym_wc_data_feed] = ACTIONS(1561),
    [sym_wc_data_field] = ACTIONS(1561),
    [sym_wc_data_record] = ACTIONS(1561),
    [sym_last_login_info] = ACTIONS(1561),
    [sym____reserved___] = ACTIONS(1561),
    [sym_date] = ACTIONS(1561),
    [sym_day] = ACTIONS(1561),
    [sym_time] = ACTIONS(1561),
    [sym_ldate] = ACTIONS(1561),
    [sym_push_device] = ACTIONS(1561),
    [sym_push_channel] = ACTIONS(1561),
    [sym_push_devchan] = ACTIONS(1561),
    [sym_release_device] = ACTIONS(1561),
    [sym_release_channel] = ACTIONS(1561),
    [sym_release_devchan] = ACTIONS(1561),
    [sym_master_slot] = ACTIONS(1561),
    [sym_get_pulse_time] = ACTIONS(1561),
    [sym_get_timer] = ACTIONS(1561),
    [sym_master_sn] = ACTIONS(1561),
    [sym_system_number] = ACTIONS(1561),
    [sym_button] = ACTIONS(1561),
    [sym_dv_channel] = ACTIONS(1561),
    [sym_data] = ACTIONS(1561),
    [sym_level] = ACTIONS(1561),
    [sym_timeline] = ACTIONS(1561),
    [sym_channel] = ACTIONS(1561),
    [sym_custom] = ACTIONS(1561),
    [sym_atoi] = ACTIONS(1561),
    [sym_atol] = ACTIONS(1561),
    [sym_atof] = ACTIONS(1561),
    [sym_itoa] = ACTIONS(1561),
    [sym_itohex] = ACTIONS(1561),
    [sym_hextoi] = ACTIONS(1561),
    [sym_ftoa] = ACTIONS(1561),
    [sym_find_string] = ACTIONS(1561),
    [sym_left_string] = ACTIONS(1561),
    [sym_right_string] = ACTIONS(1561),
    [sym_mid_string] = ACTIONS(1561),
    [sym_remove_string] = ACTIONS(1561),
    [sym_get_buffer_string] = ACTIONS(1561),
    [sym_get_buffer_char] = ACTIONS(1561),
    [sym_get_multi_buffer_string] = ACTIONS(1561),
    [sym_length_string] = ACTIONS(1561),
    [sym_max_length_string] = ACTIONS(1561),
    [sym_set_length_string] = ACTIONS(1561),
    [sym_compare_string] = ACTIONS(1561),
    [sym_upper_string] = ACTIONS(1561),
    [sym_lower_string] = ACTIONS(1561),
    [sym_get_last] = ACTIONS(1561),
    [sym_date_to_day] = ACTIONS(1561),
    [sym_date_to_month] = ACTIONS(1561),
    [sym_date_to_year] = ACTIONS(1561),
    [sym_time_to_hour] = ACTIONS(1561),
    [sym_time_to_minute] = ACTIONS(1561),
    [sym_time_to_second] = ACTIONS(1561),
    [sym_day_of_week] = ACTIONS(1561),
    [sym_set_pulse_time] = ACTIONS(1561),
    [sym_set_timer] = ACTIONS(1561),
    [sym_file_open] = ACTIONS(1561),
    [sym_file_close] = ACTIONS(1561),
    [sym_file_read] = ACTIONS(1561),
    [sym_file_read_line] = ACTIONS(1561),
    [sym_file_write] = ACTIONS(1561),
    [sym_file_write_line] = ACTIONS(1561),
    [sym_file_copy] = ACTIONS(1561),
    [sym_file_delete] = ACTIONS(1561),
    [sym_file_rename] = ACTIONS(1561),
    [sym_file_dir] = ACTIONS(1561),
    [sym_file_getdir] = ACTIONS(1561),
    [sym_file_setdir] = ACTIONS(1561),
    [sym_file_create_dir] = ACTIONS(1561),
    [sym_file_remove_dir] = ACTIONS(1561),
    [sym_file_seek] = ACTIONS(1561),
    [sym_length_array] = ACTIONS(1561),
    [sym_max_length_array] = ACTIONS(1561),
    [sym_set_length_array] = ACTIONS(1561),
    [sym_ip_client_open] = ACTIONS(1561),
    [sym_ip_client_close] = ACTIONS(1561),
    [sym_ip_server_open] = ACTIONS(1561),
    [sym_ip_server_close] = ACTIONS(1561),
    [sym_ip_bound_client_open] = ACTIONS(1561),
    [sym_ip_mc_server_open] = ACTIONS(1561),
    [sym_ip_set_option] = ACTIONS(1561),
    [sym_ssh_client_open] = ACTIONS(1561),
    [sym_ssh_client_close] = ACTIONS(1561),
    [sym_tls_client_open] = ACTIONS(1561),
    [sym_tls_client_close] = ACTIONS(1561),
    [sym_do_push] = ACTIONS(1561),
    [sym_do_push_timed] = ACTIONS(1561),
    [sym_do_release] = ACTIONS(1561),
    [sym_device_id] = ACTIONS(1561),
    [sym_device_info] = ACTIONS(1561),
    [sym_device_id_string] = ACTIONS(1561),
    [sym_device_standby] = ACTIONS(1561),
    [sym_device_wake] = ACTIONS(1561),
    [sym_timeline_create] = ACTIONS(1561),
    [sym_timeline_kill] = ACTIONS(1561),
    [sym_timeline_reload] = ACTIONS(1561),
    [sym_timeline_pause] = ACTIONS(1561),
    [sym_timeline_restart] = ACTIONS(1561),
    [sym_timeline_set] = ACTIONS(1561),
    [sym_timeline_get] = ACTIONS(1561),
    [sym_timeline_active] = ACTIONS(1561),
    [sym_timeline_dynamic_id] = ACTIONS(1561),
    [sym_variable_to_string] = ACTIONS(1561),
    [sym_string_to_variable] = ACTIONS(1561),
    [sym_length_variable_to_string] = ACTIONS(1561),
    [sym_type_cast] = ACTIONS(1561),
    [sym_variable_to_xml] = ACTIONS(1561),
    [sym_xml_to_variable] = ACTIONS(1561),
    [sym_length_variable_to_xml] = ACTIONS(1561),
    [sym_redirect_string] = ACTIONS(1561),
    [sym_random_number] = ACTIONS(1561),
    [sym_get_unique_id] = ACTIONS(1561),
    [sym_get_system_number] = ACTIONS(1561),
    [sym_set_system_number] = ACTIONS(1561),
    [sym_get_serial_number] = ACTIONS(1561),
    [sym_get_master_build] = ACTIONS(1561),
    [sym_set_validation_code] = ACTIONS(1561),
    [sym_reboot] = ACTIONS(1561),
    [sym_load_duet_module] = ACTIONS(1561),
    [sym_unload_duet_module] = ACTIONS(1561),
    [sym_duet_mem_size_set] = ACTIONS(1561),
    [sym_duet_mem_size_get] = ACTIONS(1561),
    [sym_dynamic_application_device] = ACTIONS(1561),
    [sym_dynamic_polled_port] = ACTIONS(1561),
    [sym_static_port_binding] = ACTIONS(1561),
    [sym_static_ip_binding] = ACTIONS(1561),
    [sym_get_dns_list] = ACTIONS(1561),
    [sym_set_dns_list] = ACTIONS(1561),
    [sym_get_ip_address] = ACTIONS(1561),
    [sym_set_ip_address] = ACTIONS(1561),
    [sym_get_url_list] = ACTIONS(1561),
    [sym_add_url_entry] = ACTIONS(1561),
    [sym_delete_url_entry] = ACTIONS(1561),
    [sym_abs_value] = ACTIONS(1561),
    [sym_max_value] = ACTIONS(1561),
    [sym_min_value] = ACTIONS(1561),
    [sym_exp_value] = ACTIONS(1561),
    [sym_log_value] = ACTIONS(1561),
    [sym_log10_value] = ACTIONS(1561),
    [sym_power_value] = ACTIONS(1561),
    [sym_sqrt_value] = ACTIONS(1561),
    [sym_raw_be] = ACTIONS(1561),
    [sym_raw_le] = ACTIONS(1561),
    [sym_internal_queue_size_set] = ACTIONS(1561),
    [sym_internal_queue_size_get] = ACTIONS(1561),
    [sym_internal_threshold_set] = ACTIONS(1561),
    [sym_internal_threshold_get] = ACTIONS(1561),
    [sym_rebuilt_event] = ACTIONS(1561),
    [sym_do_custom_event] = ACTIONS(1561),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1561),
    [sym_clkmgr_set_clk_source] = ACTIONS(1561),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1561),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1561),
    [sym_clkmgr_get_timezone] = ACTIONS(1561),
    [sym_clkmgr_set_timezone] = ACTIONS(1561),
    [sym_clkmgr_get_resync_period] = ACTIONS(1561),
    [sym_clkmgr_set_resync_period] = ACTIONS(1561),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1561),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1561),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1561),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1561),
    [sym_clkmgr_get_timeservers] = ACTIONS(1561),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1561),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1561),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1561),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1561),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1561),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1561),
    [sym_set_log_level] = ACTIONS(1561),
    [sym_get_log_level] = ACTIONS(1561),
    [sym_amx_log] = ACTIONS(1561),
    [sym_smtp_server_config_set] = ACTIONS(1561),
    [sym_smtp_server_config_get] = ACTIONS(1561),
    [sym_smtp_send] = ACTIONS(1561),
    [sym_validate_netlinx_account] = ACTIONS(1561),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1561),
    [sym_authenticate_certificate] = ACTIONS(1561),
    [sym_audit_netlinx_session_event] = ACTIONS(1561),
    [sym_audit_netlinx_generic_event] = ACTIONS(1561),
    [sym_get_available_flash_disk_space] = ACTIONS(1561),
    [sym_get_max_flash_disk_space] = ACTIONS(1561),
    [sym_data_create_feed] = ACTIONS(1561),
    [sym_data_delete_feed] = ACTIONS(1561),
    [sym_data_publish_feed] = ACTIONS(1561),
    [sym_data_get_published_feed] = ACTIONS(1561),
    [sym_data_add_record] = ACTIONS(1561),
    [sym_data_get_event_record] = ACTIONS(1561),
    [sym__wc_data_create_feed] = ACTIONS(1561),
    [sym__wc_data_add_record] = ACTIONS(1561),
    [sym__wc_data_get_event_record] = ACTIONS(1561),
    [sym_format] = ACTIONS(1561),
    [sym_astro_clock] = ACTIONS(1561),
    [sym_set_outdoor_temperature] = ACTIONS(1561),
    [sym_set_virtual_level_count] = ACTIONS(1561),
    [sym_set_virtual_channel_count] = ACTIONS(1561),
    [sym_set_virtual_port_count] = ACTIONS(1561),
    [sym__wc_encode_int] = ACTIONS(1561),
    [sym__wc_decode_int] = ACTIONS(1561),
    [sym__wc_int] = ACTIONS(1561),
    [sym__wc_to_ch_int] = ACTIONS(1561),
    [sym__ch_to_wc_int] = ACTIONS(1561),
    [sym__wc_find_string_int] = ACTIONS(1561),
    [sym__wc_left_string_int] = ACTIONS(1561),
    [sym__wc_lower_string_int] = ACTIONS(1561),
    [sym__wc_mid_string_int] = ACTIONS(1561),
    [sym__wc_remove_string_int] = ACTIONS(1561),
    [sym__wc_right_string_int] = ACTIONS(1561),
    [sym__wc_upper_string_int] = ACTIONS(1561),
    [sym__wc_compare_string_int] = ACTIONS(1561),
    [sym__wc_get_buffer_char_int] = ACTIONS(1561),
    [sym__wc_get_buffer_string_int] = ACTIONS(1561),
    [sym__wc_concat_string_int] = ACTIONS(1561),
    [sym__wc_file_open_int] = ACTIONS(1561),
    [sym__wc_file_close_int] = ACTIONS(1561),
    [sym__wc_file_read_int] = ACTIONS(1561),
    [sym__wc_file_read_line_int] = ACTIONS(1561),
    [sym__wc_file_write_int] = ACTIONS(1561),
    [sym__wc_file_write_line_int] = ACTIONS(1561),
    [aux_sym_preproc_include_token1] = ACTIONS(1561),
    [aux_sym_preproc_define_token1] = ACTIONS(1563),
    [aux_sym_preproc_warn_token1] = ACTIONS(1563),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1563),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1563),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1563),
    [anon_sym_LBRACK] = ACTIONS(1563),
    [anon_sym_LBRACE] = ACTIONS(1563),
    [anon_sym_RBRACE] = ACTIONS(1563),
    [anon_sym_COLON] = ACTIONS(1563),
    [anon_sym_LPAREN] = ACTIONS(1561),
    [anon_sym_SEMI] = ACTIONS(1563),
    [anon_sym_BANG] = ACTIONS(1561),
    [anon_sym_TILDE] = ACTIONS(1563),
    [anon_sym_DASH] = ACTIONS(1561),
    [anon_sym_PLUS] = ACTIONS(1561),
    [anon_sym_STAR] = ACTIONS(1563),
    [anon_sym_SLASH] = ACTIONS(1561),
    [anon_sym_PERCENT] = ACTIONS(1563),
    [anon_sym_PIPE_PIPE] = ACTIONS(1563),
    [anon_sym_AMP_AMP] = ACTIONS(1563),
    [anon_sym_PIPE] = ACTIONS(1561),
    [anon_sym_CARET] = ACTIONS(1563),
    [anon_sym_AMP] = ACTIONS(1561),
    [anon_sym_EQ_EQ] = ACTIONS(1563),
    [anon_sym_BANG_EQ] = ACTIONS(1563),
    [anon_sym_LT_GT] = ACTIONS(1563),
    [anon_sym_GT] = ACTIONS(1561),
    [anon_sym_GT_EQ] = ACTIONS(1563),
    [anon_sym_LT_EQ] = ACTIONS(1563),
    [anon_sym_LT] = ACTIONS(1561),
    [anon_sym_LT_LT] = ACTIONS(1563),
    [anon_sym_GT_GT] = ACTIONS(1563),
    [anon_sym_DASH_DASH] = ACTIONS(1563),
    [anon_sym_PLUS_PLUS] = ACTIONS(1563),
    [anon_sym_DOT] = ACTIONS(1561),
    [anon_sym_DQUOTE] = ACTIONS(1563),
    [anon_sym_SQUOTE] = ACTIONS(1563),
    [sym_number_literal] = ACTIONS(1563),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1563),
  },
  [STATE(186)] = {
    [sym_identifier] = ACTIONS(1585),
    [sym_define_call_keyword] = ACTIONS(1585),
    [sym_define_function_keyword] = ACTIONS(1585),
    [sym_define_library_function_keyword] = ACTIONS(1585),
    [sym_define_module_keyword] = ACTIONS(1585),
    [sym_char_keyword] = ACTIONS(1585),
    [sym_widechar_keyword] = ACTIONS(1585),
    [sym_integer_keyword] = ACTIONS(1585),
    [sym_sinteger_keyword] = ACTIONS(1585),
    [sym_long_keyword] = ACTIONS(1585),
    [sym_slong_keyword] = ACTIONS(1585),
    [sym_float_keyword] = ACTIONS(1585),
    [sym_double_keyword] = ACTIONS(1585),
    [sym_variant_keyword] = ACTIONS(1585),
    [sym_variantarray_keyword] = ACTIONS(1585),
    [sym_local_var_keyword] = ACTIONS(1585),
    [sym_stack_var_keyword] = ACTIONS(1585),
    [sym_constant_keyword] = ACTIONS(1585),
    [sym_volatile_keyword] = ACTIONS(1585),
    [sym_non_volatile_keyword] = ACTIONS(1585),
    [sym_persistent_keyword] = ACTIONS(1585),
    [sym_dev_keyword] = ACTIONS(1585),
    [sym_devlev_keyword] = ACTIONS(1585),
    [sym_devchan_keyword] = ACTIONS(1585),
    [sym_if_keyword] = ACTIONS(1585),
    [sym_else_keyword] = ACTIONS(1585),
    [sym_switch_keyword] = ACTIONS(1585),
    [sym_case_keyword] = ACTIONS(1585),
    [sym_default_keyword] = ACTIONS(1585),
    [sym_while_keyword] = ACTIONS(1585),
    [sym_for_keyword] = ACTIONS(1585),
    [sym_break_keyword] = ACTIONS(1585),
    [sym_continue_keyword] = ACTIONS(1585),
    [sym_return_keyword] = ACTIONS(1585),
    [sym_select_keyword] = ACTIONS(1585),
    [sym_active_keyword] = ACTIONS(1585),
    [sym_struct_keyword] = ACTIONS(1585),
    [sym_structure_keyword] = ACTIONS(1585),
    [aux_sym_on_keyword_token1] = ACTIONS(1585),
    [aux_sym_off_keyword_token1] = ACTIONS(1585),
    [sym_send_level_keyword] = ACTIONS(1585),
    [sym_send_string_keyword] = ACTIONS(1585),
    [sym_send_command_keyword] = ACTIONS(1585),
    [sym_clear_buffer_keyword] = ACTIONS(1585),
    [sym_create_buffer_keyword] = ACTIONS(1585),
    [sym_create_multi_buffer_keyword] = ACTIONS(1585),
    [sym_call_keyword] = ACTIONS(1585),
    [sym_system_call_keyword] = ACTIONS(1585),
    [sym_devchan_to_keyword] = ACTIONS(1585),
    [sym_devchan_min_to_keyword] = ACTIONS(1585),
    [sym_devchan_total_off_keyword] = ACTIONS(1585),
    [sym_devchan_pulse_keyword] = ACTIONS(1585),
    [sym_wait_keyword] = ACTIONS(1585),
    [sym_cancel_wait_keyword] = ACTIONS(1585),
    [sym_wait_until_keyword] = ACTIONS(1585),
    [sym_cancel_wait_until_keyword] = ACTIONS(1585),
    [sym_cancel_all_wait_keyword] = ACTIONS(1585),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1585),
    [sym_and] = ACTIONS(1585),
    [sym_or] = ACTIONS(1585),
    [sym_not] = ACTIONS(1585),
    [sym_xor] = ACTIONS(1585),
    [sym_band] = ACTIONS(1585),
    [sym_bor] = ACTIONS(1585),
    [sym_bxor] = ACTIONS(1585),
    [sym_bnot] = ACTIONS(1585),
    [sym_lshift] = ACTIONS(1585),
    [sym_rshift] = ACTIONS(1585),
    [sym___file__] = ACTIONS(1585),
    [sym___date__] = ACTIONS(1585),
    [sym___ldate__] = ACTIONS(1585),
    [sym___time__] = ACTIONS(1585),
    [sym___line__] = ACTIONS(1585),
    [sym___name__] = ACTIONS(1585),
    [sym___netlinx__] = ACTIONS(1585),
    [sym_netlinx_axi_version] = ACTIONS(1585),
    [sym_url_flg_tcp] = ACTIONS(1585),
    [sym_url_flg_accinfopresent] = ACTIONS(1585),
    [sym_url_flg_temp] = ACTIONS(1585),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1585),
    [sym_url_flg_stat_mask] = ACTIONS(1585),
    [sym_url_flg_stat_lookup] = ACTIONS(1585),
    [sym_url_flg_stat_connecting] = ACTIONS(1585),
    [sym_url_flg_stat_waiting] = ACTIONS(1585),
    [sym_url_flg_stat_connected] = ACTIONS(1585),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1585),
    [sym_first_virtual_device] = ACTIONS(1585),
    [sym_first_local_port] = ACTIONS(1585),
    [sym_dynamic_virtual_device] = ACTIONS(1585),
    [sym_timeline_once] = ACTIONS(1585),
    [sym_timeline_repeat] = ACTIONS(1585),
    [sym_timeline_absolute] = ACTIONS(1585),
    [sym_timeline_relative] = ACTIONS(1585),
    [sym_do_push_timed_infinite] = ACTIONS(1585),
    [sym_source_type_no_address] = ACTIONS(1585),
    [sym_source_type_neuron_id] = ACTIONS(1585),
    [sym_source_type_ip_address] = ACTIONS(1585),
    [sym_source_type_axlink] = ACTIONS(1585),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1585),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1585),
    [sym_source_type_ip_socket_address] = ACTIONS(1585),
    [sym_source_type_rs232] = ACTIONS(1585),
    [sym_source_type_internal] = ACTIONS(1585),
    [sym_source_type_mac_address] = ACTIONS(1585),
    [sym_source_type_ipv4_port] = ACTIONS(1585),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1585),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1585),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1585),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1585),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1585),
    [sym_true] = ACTIONS(1585),
    [sym_false] = ACTIONS(1585),
    [sym_file_read_only] = ACTIONS(1585),
    [sym_file_rw_new] = ACTIONS(1585),
    [sym_file_rw_append] = ACTIONS(1585),
    [sym_ip_tcp] = ACTIONS(1585),
    [sym_ip_udp] = ACTIONS(1585),
    [sym_ip_udp_2way] = ACTIONS(1585),
    [sym_xml_encode_types] = ACTIONS(1585),
    [sym_xml_encode_char_as_list] = ACTIONS(1585),
    [sym_xml_encode_le] = ACTIONS(1585),
    [sym_xml_decode_types] = ACTIONS(1585),
    [sym_xml_decode_no_preserve] = ACTIONS(1585),
    [sym_duet_dev_type_display_device] = ACTIONS(1585),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1585),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1585),
    [sym_duet_dev_type_io_device] = ACTIONS(1585),
    [sym_duet_dev_type_relay_device] = ACTIONS(1585),
    [sym_duet_dev_type_ups] = ACTIONS(1585),
    [sym_duet_dev_type_amplifier] = ACTIONS(1585),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1585),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1585),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1585),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1585),
    [sym_duet_dev_type_camera] = ACTIONS(1585),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1585),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1585),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1585),
    [sym_duet_dev_type_dss] = ACTIONS(1585),
    [sym_duet_dev_type_dvr] = ACTIONS(1585),
    [sym_duet_dev_type_disc_device] = ACTIONS(1585),
    [sym_duet_dev_type_document_camera] = ACTIONS(1585),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1585),
    [sym_duet_dev_type_hvac] = ACTIONS(1585),
    [sym_duet_dev_type_keypad] = ACTIONS(1585),
    [sym_duet_dev_type_light] = ACTIONS(1585),
    [sym_duet_dev_type_monitor] = ACTIONS(1585),
    [sym_duet_dev_type_motor] = ACTIONS(1585),
    [sym_duet_dev_type_multi_window] = ACTIONS(1585),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1585),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1585),
    [sym_duet_dev_type_receiver] = ACTIONS(1585),
    [sym_duet_dev_type_security_system] = ACTIONS(1585),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1585),
    [sym_duet_dev_type_settop_box] = ACTIONS(1585),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1585),
    [sym_duet_dev_type_switcher] = ACTIONS(1585),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1585),
    [sym_duet_dev_type_tv] = ACTIONS(1585),
    [sym_duet_dev_type_utility] = ACTIONS(1585),
    [sym_duet_dev_type_vcr] = ACTIONS(1585),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1585),
    [sym_duet_dev_type_video_processor] = ACTIONS(1585),
    [sym_duet_dev_type_video_projector] = ACTIONS(1585),
    [sym_duet_dev_type_video_wall] = ACTIONS(1585),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1585),
    [sym_duet_dev_type_weather] = ACTIONS(1585),
    [sym_duet_dev_not_polled] = ACTIONS(1585),
    [sym_duet_dev_polled] = ACTIONS(1585),
    [sym_clkmgr_mode_network] = ACTIONS(1585),
    [sym_clkmgr_mode_standalone] = ACTIONS(1585),
    [sym_normal_standby] = ACTIONS(1585),
    [sym_normal_wake] = ACTIONS(1585),
    [sym_ip_multicast_ttl_option] = ACTIONS(1585),
    [sym_ip_ttl_subnet] = ACTIONS(1585),
    [sym_ip_ttl_site] = ACTIONS(1585),
    [sym_ip_ttl_region] = ACTIONS(1585),
    [sym_ip_ttl_continent] = ACTIONS(1585),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1585),
    [sym_ip_nodelay_on] = ACTIONS(1585),
    [sym_ip_nodelay_off] = ACTIONS(1585),
    [sym_amx_error] = ACTIONS(1585),
    [sym_amx_warning] = ACTIONS(1585),
    [sym_amx_info] = ACTIONS(1585),
    [sym_amx_debug] = ACTIONS(1585),
    [sym_netlinx_login_success] = ACTIONS(1585),
    [sym_netlinx_login_fail] = ACTIONS(1585),
    [sym_netlinx_logout] = ACTIONS(1585),
    [sym_null_str] = ACTIONS(1585),
    [sym_smtp_address] = ACTIONS(1585),
    [sym_smtp_port_number] = ACTIONS(1585),
    [sym_smtp_username] = ACTIONS(1585),
    [sym_smtp_password] = ACTIONS(1585),
    [sym_smtp_require_tls] = ACTIONS(1585),
    [sym_smtp_from] = ACTIONS(1585),
    [sym_smtp_tls_true] = ACTIONS(1585),
    [sym_smtp_tls_false] = ACTIONS(1585),
    [sym_listview_on_row_select_event] = ACTIONS(1585),
    [sym_data_structure_unknown] = ACTIONS(1585),
    [sym_data_structure_datafeed] = ACTIONS(1585),
    [sym_data_structure_datarecordset] = ACTIONS(1585),
    [sym_data_structure_datarecord] = ACTIONS(1585),
    [sym_data_structure_datafield] = ACTIONS(1585),
    [sym_data_max_content_field_count] = ACTIONS(1585),
    [sym_data_max_metadata_field_count] = ACTIONS(1585),
    [sym_data_max_id_length] = ACTIONS(1585),
    [sym_data_max_name_length] = ACTIONS(1585),
    [sym_data_max_description_length] = ACTIONS(1585),
    [sym_data_max_type_length] = ACTIONS(1585),
    [sym_data_max_format_length] = ACTIONS(1585),
    [sym_data_max_source_length] = ACTIONS(1585),
    [sym_data_max_value_length] = ACTIONS(1585),
    [sym_data_max_label_length] = ACTIONS(1585),
    [sym_data_max_url_length] = ACTIONS(1585),
    [sym_max_last_login_info_length] = ACTIONS(1585),
    [sym_data_type_unknown] = ACTIONS(1585),
    [sym_data_type_string] = ACTIONS(1585),
    [sym_data_type_datetime] = ACTIONS(1585),
    [sym_data_type_date] = ACTIONS(1585),
    [sym_data_type_time] = ACTIONS(1585),
    [sym_data_type_image] = ACTIONS(1585),
    [sym_data_format_url] = ACTIONS(1585),
    [sym_data_format_phone] = ACTIONS(1585),
    [sym_data_format_email] = ACTIONS(1585),
    [sym_data_format_iso8601] = ACTIONS(1585),
    [sym_auditlog_priv] = ACTIONS(1585),
    [sym_configuration_priv] = ACTIONS(1585),
    [sym_deviceconfig_priv] = ACTIONS(1585),
    [sym_ftp_priv] = ACTIONS(1585),
    [sym_http_priv] = ACTIONS(1585),
    [sym_networkconfig_priv] = ACTIONS(1585),
    [sym_programport_priv] = ACTIONS(1585),
    [sym_remoteui_priv] = ACTIONS(1585),
    [sym_securitycontrol_priv] = ACTIONS(1585),
    [sym_softwaremanagement_priv] = ACTIONS(1585),
    [sym_terminal_priv] = ACTIONS(1585),
    [sym_tpadmin_priv] = ACTIONS(1585),
    [sym_user1_priv] = ACTIONS(1585),
    [sym_user2_priv] = ACTIONS(1585),
    [sym_user3_priv] = ACTIONS(1585),
    [sym_user4_priv] = ACTIONS(1585),
    [sym_usermanagement_priv] = ACTIONS(1585),
    [sym_valid_account] = ACTIONS(1585),
    [sym_err_name_invalid] = ACTIONS(1585),
    [sym_err_password_invalid] = ACTIONS(1585),
    [sym_err_invalid_account] = ACTIONS(1585),
    [sym_err_no_authorization] = ACTIONS(1585),
    [sym_err_invalid_parameter] = ACTIONS(1585),
    [sym_err_user_locked_out] = ACTIONS(1585),
    [sym_err_user_account_expired] = ACTIONS(1585),
    [sym_err_general_error] = ACTIONS(1585),
    [sym_err_authentication_error] = ACTIONS(1585),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1585),
    [sym_tls_validate_certificate] = ACTIONS(1585),
    [sym_tdata] = ACTIONS(1585),
    [sym_tchannel] = ACTIONS(1585),
    [sym_tlevel] = ACTIONS(1585),
    [sym_tbutton] = ACTIONS(1585),
    [sym_ttimeline] = ACTIONS(1585),
    [sym_tcustom] = ACTIONS(1585),
    [sym_url_struct] = ACTIONS(1585),
    [sym_dns_struct] = ACTIONS(1585),
    [sym_ip_address_struct] = ACTIONS(1585),
    [sym_dev_info_struct] = ACTIONS(1585),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1585),
    [sym_data_feed] = ACTIONS(1585),
    [sym_data_field] = ACTIONS(1585),
    [sym_data_record] = ACTIONS(1585),
    [sym_wc_data_feed] = ACTIONS(1585),
    [sym_wc_data_field] = ACTIONS(1585),
    [sym_wc_data_record] = ACTIONS(1585),
    [sym_last_login_info] = ACTIONS(1585),
    [sym____reserved___] = ACTIONS(1585),
    [sym_date] = ACTIONS(1585),
    [sym_day] = ACTIONS(1585),
    [sym_time] = ACTIONS(1585),
    [sym_ldate] = ACTIONS(1585),
    [sym_push_device] = ACTIONS(1585),
    [sym_push_channel] = ACTIONS(1585),
    [sym_push_devchan] = ACTIONS(1585),
    [sym_release_device] = ACTIONS(1585),
    [sym_release_channel] = ACTIONS(1585),
    [sym_release_devchan] = ACTIONS(1585),
    [sym_master_slot] = ACTIONS(1585),
    [sym_get_pulse_time] = ACTIONS(1585),
    [sym_get_timer] = ACTIONS(1585),
    [sym_master_sn] = ACTIONS(1585),
    [sym_system_number] = ACTIONS(1585),
    [sym_button] = ACTIONS(1585),
    [sym_dv_channel] = ACTIONS(1585),
    [sym_data] = ACTIONS(1585),
    [sym_level] = ACTIONS(1585),
    [sym_timeline] = ACTIONS(1585),
    [sym_channel] = ACTIONS(1585),
    [sym_custom] = ACTIONS(1585),
    [sym_atoi] = ACTIONS(1585),
    [sym_atol] = ACTIONS(1585),
    [sym_atof] = ACTIONS(1585),
    [sym_itoa] = ACTIONS(1585),
    [sym_itohex] = ACTIONS(1585),
    [sym_hextoi] = ACTIONS(1585),
    [sym_ftoa] = ACTIONS(1585),
    [sym_find_string] = ACTIONS(1585),
    [sym_left_string] = ACTIONS(1585),
    [sym_right_string] = ACTIONS(1585),
    [sym_mid_string] = ACTIONS(1585),
    [sym_remove_string] = ACTIONS(1585),
    [sym_get_buffer_string] = ACTIONS(1585),
    [sym_get_buffer_char] = ACTIONS(1585),
    [sym_get_multi_buffer_string] = ACTIONS(1585),
    [sym_length_string] = ACTIONS(1585),
    [sym_max_length_string] = ACTIONS(1585),
    [sym_set_length_string] = ACTIONS(1585),
    [sym_compare_string] = ACTIONS(1585),
    [sym_upper_string] = ACTIONS(1585),
    [sym_lower_string] = ACTIONS(1585),
    [sym_get_last] = ACTIONS(1585),
    [sym_date_to_day] = ACTIONS(1585),
    [sym_date_to_month] = ACTIONS(1585),
    [sym_date_to_year] = ACTIONS(1585),
    [sym_time_to_hour] = ACTIONS(1585),
    [sym_time_to_minute] = ACTIONS(1585),
    [sym_time_to_second] = ACTIONS(1585),
    [sym_day_of_week] = ACTIONS(1585),
    [sym_set_pulse_time] = ACTIONS(1585),
    [sym_set_timer] = ACTIONS(1585),
    [sym_file_open] = ACTIONS(1585),
    [sym_file_close] = ACTIONS(1585),
    [sym_file_read] = ACTIONS(1585),
    [sym_file_read_line] = ACTIONS(1585),
    [sym_file_write] = ACTIONS(1585),
    [sym_file_write_line] = ACTIONS(1585),
    [sym_file_copy] = ACTIONS(1585),
    [sym_file_delete] = ACTIONS(1585),
    [sym_file_rename] = ACTIONS(1585),
    [sym_file_dir] = ACTIONS(1585),
    [sym_file_getdir] = ACTIONS(1585),
    [sym_file_setdir] = ACTIONS(1585),
    [sym_file_create_dir] = ACTIONS(1585),
    [sym_file_remove_dir] = ACTIONS(1585),
    [sym_file_seek] = ACTIONS(1585),
    [sym_length_array] = ACTIONS(1585),
    [sym_max_length_array] = ACTIONS(1585),
    [sym_set_length_array] = ACTIONS(1585),
    [sym_ip_client_open] = ACTIONS(1585),
    [sym_ip_client_close] = ACTIONS(1585),
    [sym_ip_server_open] = ACTIONS(1585),
    [sym_ip_server_close] = ACTIONS(1585),
    [sym_ip_bound_client_open] = ACTIONS(1585),
    [sym_ip_mc_server_open] = ACTIONS(1585),
    [sym_ip_set_option] = ACTIONS(1585),
    [sym_ssh_client_open] = ACTIONS(1585),
    [sym_ssh_client_close] = ACTIONS(1585),
    [sym_tls_client_open] = ACTIONS(1585),
    [sym_tls_client_close] = ACTIONS(1585),
    [sym_do_push] = ACTIONS(1585),
    [sym_do_push_timed] = ACTIONS(1585),
    [sym_do_release] = ACTIONS(1585),
    [sym_device_id] = ACTIONS(1585),
    [sym_device_info] = ACTIONS(1585),
    [sym_device_id_string] = ACTIONS(1585),
    [sym_device_standby] = ACTIONS(1585),
    [sym_device_wake] = ACTIONS(1585),
    [sym_timeline_create] = ACTIONS(1585),
    [sym_timeline_kill] = ACTIONS(1585),
    [sym_timeline_reload] = ACTIONS(1585),
    [sym_timeline_pause] = ACTIONS(1585),
    [sym_timeline_restart] = ACTIONS(1585),
    [sym_timeline_set] = ACTIONS(1585),
    [sym_timeline_get] = ACTIONS(1585),
    [sym_timeline_active] = ACTIONS(1585),
    [sym_timeline_dynamic_id] = ACTIONS(1585),
    [sym_variable_to_string] = ACTIONS(1585),
    [sym_string_to_variable] = ACTIONS(1585),
    [sym_length_variable_to_string] = ACTIONS(1585),
    [sym_type_cast] = ACTIONS(1585),
    [sym_variable_to_xml] = ACTIONS(1585),
    [sym_xml_to_variable] = ACTIONS(1585),
    [sym_length_variable_to_xml] = ACTIONS(1585),
    [sym_redirect_string] = ACTIONS(1585),
    [sym_random_number] = ACTIONS(1585),
    [sym_get_unique_id] = ACTIONS(1585),
    [sym_get_system_number] = ACTIONS(1585),
    [sym_set_system_number] = ACTIONS(1585),
    [sym_get_serial_number] = ACTIONS(1585),
    [sym_get_master_build] = ACTIONS(1585),
    [sym_set_validation_code] = ACTIONS(1585),
    [sym_reboot] = ACTIONS(1585),
    [sym_load_duet_module] = ACTIONS(1585),
    [sym_unload_duet_module] = ACTIONS(1585),
    [sym_duet_mem_size_set] = ACTIONS(1585),
    [sym_duet_mem_size_get] = ACTIONS(1585),
    [sym_dynamic_application_device] = ACTIONS(1585),
    [sym_dynamic_polled_port] = ACTIONS(1585),
    [sym_static_port_binding] = ACTIONS(1585),
    [sym_static_ip_binding] = ACTIONS(1585),
    [sym_get_dns_list] = ACTIONS(1585),
    [sym_set_dns_list] = ACTIONS(1585),
    [sym_get_ip_address] = ACTIONS(1585),
    [sym_set_ip_address] = ACTIONS(1585),
    [sym_get_url_list] = ACTIONS(1585),
    [sym_add_url_entry] = ACTIONS(1585),
    [sym_delete_url_entry] = ACTIONS(1585),
    [sym_abs_value] = ACTIONS(1585),
    [sym_max_value] = ACTIONS(1585),
    [sym_min_value] = ACTIONS(1585),
    [sym_exp_value] = ACTIONS(1585),
    [sym_log_value] = ACTIONS(1585),
    [sym_log10_value] = ACTIONS(1585),
    [sym_power_value] = ACTIONS(1585),
    [sym_sqrt_value] = ACTIONS(1585),
    [sym_raw_be] = ACTIONS(1585),
    [sym_raw_le] = ACTIONS(1585),
    [sym_internal_queue_size_set] = ACTIONS(1585),
    [sym_internal_queue_size_get] = ACTIONS(1585),
    [sym_internal_threshold_set] = ACTIONS(1585),
    [sym_internal_threshold_get] = ACTIONS(1585),
    [sym_rebuilt_event] = ACTIONS(1585),
    [sym_do_custom_event] = ACTIONS(1585),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1585),
    [sym_clkmgr_set_clk_source] = ACTIONS(1585),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1585),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1585),
    [sym_clkmgr_get_timezone] = ACTIONS(1585),
    [sym_clkmgr_set_timezone] = ACTIONS(1585),
    [sym_clkmgr_get_resync_period] = ACTIONS(1585),
    [sym_clkmgr_set_resync_period] = ACTIONS(1585),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1585),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1585),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1585),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1585),
    [sym_clkmgr_get_timeservers] = ACTIONS(1585),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1585),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1585),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1585),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1585),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1585),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1585),
    [sym_set_log_level] = ACTIONS(1585),
    [sym_get_log_level] = ACTIONS(1585),
    [sym_amx_log] = ACTIONS(1585),
    [sym_smtp_server_config_set] = ACTIONS(1585),
    [sym_smtp_server_config_get] = ACTIONS(1585),
    [sym_smtp_send] = ACTIONS(1585),
    [sym_validate_netlinx_account] = ACTIONS(1585),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1585),
    [sym_authenticate_certificate] = ACTIONS(1585),
    [sym_audit_netlinx_session_event] = ACTIONS(1585),
    [sym_audit_netlinx_generic_event] = ACTIONS(1585),
    [sym_get_available_flash_disk_space] = ACTIONS(1585),
    [sym_get_max_flash_disk_space] = ACTIONS(1585),
    [sym_data_create_feed] = ACTIONS(1585),
    [sym_data_delete_feed] = ACTIONS(1585),
    [sym_data_publish_feed] = ACTIONS(1585),
    [sym_data_get_published_feed] = ACTIONS(1585),
    [sym_data_add_record] = ACTIONS(1585),
    [sym_data_get_event_record] = ACTIONS(1585),
    [sym__wc_data_create_feed] = ACTIONS(1585),
    [sym__wc_data_add_record] = ACTIONS(1585),
    [sym__wc_data_get_event_record] = ACTIONS(1585),
    [sym_format] = ACTIONS(1585),
    [sym_astro_clock] = ACTIONS(1585),
    [sym_set_outdoor_temperature] = ACTIONS(1585),
    [sym_set_virtual_level_count] = ACTIONS(1585),
    [sym_set_virtual_channel_count] = ACTIONS(1585),
    [sym_set_virtual_port_count] = ACTIONS(1585),
    [sym__wc_encode_int] = ACTIONS(1585),
    [sym__wc_decode_int] = ACTIONS(1585),
    [sym__wc_int] = ACTIONS(1585),
    [sym__wc_to_ch_int] = ACTIONS(1585),
    [sym__ch_to_wc_int] = ACTIONS(1585),
    [sym__wc_find_string_int] = ACTIONS(1585),
    [sym__wc_left_string_int] = ACTIONS(1585),
    [sym__wc_lower_string_int] = ACTIONS(1585),
    [sym__wc_mid_string_int] = ACTIONS(1585),
    [sym__wc_remove_string_int] = ACTIONS(1585),
    [sym__wc_right_string_int] = ACTIONS(1585),
    [sym__wc_upper_string_int] = ACTIONS(1585),
    [sym__wc_compare_string_int] = ACTIONS(1585),
    [sym__wc_get_buffer_char_int] = ACTIONS(1585),
    [sym__wc_get_buffer_string_int] = ACTIONS(1585),
    [sym__wc_concat_string_int] = ACTIONS(1585),
    [sym__wc_file_open_int] = ACTIONS(1585),
    [sym__wc_file_close_int] = ACTIONS(1585),
    [sym__wc_file_read_int] = ACTIONS(1585),
    [sym__wc_file_read_line_int] = ACTIONS(1585),
    [sym__wc_file_write_int] = ACTIONS(1585),
    [sym__wc_file_write_line_int] = ACTIONS(1585),
    [aux_sym_preproc_include_token1] = ACTIONS(1585),
    [aux_sym_preproc_define_token1] = ACTIONS(1587),
    [aux_sym_preproc_warn_token1] = ACTIONS(1587),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1587),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1587),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1587),
    [anon_sym_LBRACK] = ACTIONS(1587),
    [anon_sym_LBRACE] = ACTIONS(1587),
    [anon_sym_RBRACE] = ACTIONS(1587),
    [anon_sym_COLON] = ACTIONS(1587),
    [anon_sym_LPAREN] = ACTIONS(1585),
    [anon_sym_SEMI] = ACTIONS(1587),
    [anon_sym_BANG] = ACTIONS(1585),
    [anon_sym_TILDE] = ACTIONS(1587),
    [anon_sym_DASH] = ACTIONS(1585),
    [anon_sym_PLUS] = ACTIONS(1585),
    [anon_sym_STAR] = ACTIONS(1587),
    [anon_sym_SLASH] = ACTIONS(1585),
    [anon_sym_PERCENT] = ACTIONS(1587),
    [anon_sym_PIPE_PIPE] = ACTIONS(1587),
    [anon_sym_AMP_AMP] = ACTIONS(1587),
    [anon_sym_PIPE] = ACTIONS(1585),
    [anon_sym_CARET] = ACTIONS(1587),
    [anon_sym_AMP] = ACTIONS(1585),
    [anon_sym_EQ_EQ] = ACTIONS(1587),
    [anon_sym_BANG_EQ] = ACTIONS(1587),
    [anon_sym_LT_GT] = ACTIONS(1587),
    [anon_sym_GT] = ACTIONS(1585),
    [anon_sym_GT_EQ] = ACTIONS(1587),
    [anon_sym_LT_EQ] = ACTIONS(1587),
    [anon_sym_LT] = ACTIONS(1585),
    [anon_sym_LT_LT] = ACTIONS(1587),
    [anon_sym_GT_GT] = ACTIONS(1587),
    [anon_sym_DASH_DASH] = ACTIONS(1587),
    [anon_sym_PLUS_PLUS] = ACTIONS(1587),
    [anon_sym_DOT] = ACTIONS(1585),
    [anon_sym_DQUOTE] = ACTIONS(1587),
    [anon_sym_SQUOTE] = ACTIONS(1587),
    [sym_number_literal] = ACTIONS(1587),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1587),
  },
  [STATE(187)] = {
    [sym_identifier] = ACTIONS(1577),
    [sym_define_call_keyword] = ACTIONS(1577),
    [sym_define_function_keyword] = ACTIONS(1577),
    [sym_define_library_function_keyword] = ACTIONS(1577),
    [sym_define_module_keyword] = ACTIONS(1577),
    [sym_char_keyword] = ACTIONS(1577),
    [sym_widechar_keyword] = ACTIONS(1577),
    [sym_integer_keyword] = ACTIONS(1577),
    [sym_sinteger_keyword] = ACTIONS(1577),
    [sym_long_keyword] = ACTIONS(1577),
    [sym_slong_keyword] = ACTIONS(1577),
    [sym_float_keyword] = ACTIONS(1577),
    [sym_double_keyword] = ACTIONS(1577),
    [sym_variant_keyword] = ACTIONS(1577),
    [sym_variantarray_keyword] = ACTIONS(1577),
    [sym_local_var_keyword] = ACTIONS(1577),
    [sym_stack_var_keyword] = ACTIONS(1577),
    [sym_constant_keyword] = ACTIONS(1577),
    [sym_volatile_keyword] = ACTIONS(1577),
    [sym_non_volatile_keyword] = ACTIONS(1577),
    [sym_persistent_keyword] = ACTIONS(1577),
    [sym_dev_keyword] = ACTIONS(1577),
    [sym_devlev_keyword] = ACTIONS(1577),
    [sym_devchan_keyword] = ACTIONS(1577),
    [sym_if_keyword] = ACTIONS(1577),
    [sym_else_keyword] = ACTIONS(1577),
    [sym_switch_keyword] = ACTIONS(1577),
    [sym_case_keyword] = ACTIONS(1577),
    [sym_default_keyword] = ACTIONS(1577),
    [sym_while_keyword] = ACTIONS(1577),
    [sym_for_keyword] = ACTIONS(1577),
    [sym_break_keyword] = ACTIONS(1577),
    [sym_continue_keyword] = ACTIONS(1577),
    [sym_return_keyword] = ACTIONS(1577),
    [sym_select_keyword] = ACTIONS(1577),
    [sym_active_keyword] = ACTIONS(1577),
    [sym_struct_keyword] = ACTIONS(1577),
    [sym_structure_keyword] = ACTIONS(1577),
    [aux_sym_on_keyword_token1] = ACTIONS(1577),
    [aux_sym_off_keyword_token1] = ACTIONS(1577),
    [sym_send_level_keyword] = ACTIONS(1577),
    [sym_send_string_keyword] = ACTIONS(1577),
    [sym_send_command_keyword] = ACTIONS(1577),
    [sym_clear_buffer_keyword] = ACTIONS(1577),
    [sym_create_buffer_keyword] = ACTIONS(1577),
    [sym_create_multi_buffer_keyword] = ACTIONS(1577),
    [sym_call_keyword] = ACTIONS(1577),
    [sym_system_call_keyword] = ACTIONS(1577),
    [sym_devchan_to_keyword] = ACTIONS(1577),
    [sym_devchan_min_to_keyword] = ACTIONS(1577),
    [sym_devchan_total_off_keyword] = ACTIONS(1577),
    [sym_devchan_pulse_keyword] = ACTIONS(1577),
    [sym_wait_keyword] = ACTIONS(1577),
    [sym_cancel_wait_keyword] = ACTIONS(1577),
    [sym_wait_until_keyword] = ACTIONS(1577),
    [sym_cancel_wait_until_keyword] = ACTIONS(1577),
    [sym_cancel_all_wait_keyword] = ACTIONS(1577),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1577),
    [sym_and] = ACTIONS(1577),
    [sym_or] = ACTIONS(1577),
    [sym_not] = ACTIONS(1577),
    [sym_xor] = ACTIONS(1577),
    [sym_band] = ACTIONS(1577),
    [sym_bor] = ACTIONS(1577),
    [sym_bxor] = ACTIONS(1577),
    [sym_bnot] = ACTIONS(1577),
    [sym_lshift] = ACTIONS(1577),
    [sym_rshift] = ACTIONS(1577),
    [sym___file__] = ACTIONS(1577),
    [sym___date__] = ACTIONS(1577),
    [sym___ldate__] = ACTIONS(1577),
    [sym___time__] = ACTIONS(1577),
    [sym___line__] = ACTIONS(1577),
    [sym___name__] = ACTIONS(1577),
    [sym___netlinx__] = ACTIONS(1577),
    [sym_netlinx_axi_version] = ACTIONS(1577),
    [sym_url_flg_tcp] = ACTIONS(1577),
    [sym_url_flg_accinfopresent] = ACTIONS(1577),
    [sym_url_flg_temp] = ACTIONS(1577),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1577),
    [sym_url_flg_stat_mask] = ACTIONS(1577),
    [sym_url_flg_stat_lookup] = ACTIONS(1577),
    [sym_url_flg_stat_connecting] = ACTIONS(1577),
    [sym_url_flg_stat_waiting] = ACTIONS(1577),
    [sym_url_flg_stat_connected] = ACTIONS(1577),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1577),
    [sym_first_virtual_device] = ACTIONS(1577),
    [sym_first_local_port] = ACTIONS(1577),
    [sym_dynamic_virtual_device] = ACTIONS(1577),
    [sym_timeline_once] = ACTIONS(1577),
    [sym_timeline_repeat] = ACTIONS(1577),
    [sym_timeline_absolute] = ACTIONS(1577),
    [sym_timeline_relative] = ACTIONS(1577),
    [sym_do_push_timed_infinite] = ACTIONS(1577),
    [sym_source_type_no_address] = ACTIONS(1577),
    [sym_source_type_neuron_id] = ACTIONS(1577),
    [sym_source_type_ip_address] = ACTIONS(1577),
    [sym_source_type_axlink] = ACTIONS(1577),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1577),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1577),
    [sym_source_type_ip_socket_address] = ACTIONS(1577),
    [sym_source_type_rs232] = ACTIONS(1577),
    [sym_source_type_internal] = ACTIONS(1577),
    [sym_source_type_mac_address] = ACTIONS(1577),
    [sym_source_type_ipv4_port] = ACTIONS(1577),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1577),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1577),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1577),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1577),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1577),
    [sym_true] = ACTIONS(1577),
    [sym_false] = ACTIONS(1577),
    [sym_file_read_only] = ACTIONS(1577),
    [sym_file_rw_new] = ACTIONS(1577),
    [sym_file_rw_append] = ACTIONS(1577),
    [sym_ip_tcp] = ACTIONS(1577),
    [sym_ip_udp] = ACTIONS(1577),
    [sym_ip_udp_2way] = ACTIONS(1577),
    [sym_xml_encode_types] = ACTIONS(1577),
    [sym_xml_encode_char_as_list] = ACTIONS(1577),
    [sym_xml_encode_le] = ACTIONS(1577),
    [sym_xml_decode_types] = ACTIONS(1577),
    [sym_xml_decode_no_preserve] = ACTIONS(1577),
    [sym_duet_dev_type_display_device] = ACTIONS(1577),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1577),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1577),
    [sym_duet_dev_type_io_device] = ACTIONS(1577),
    [sym_duet_dev_type_relay_device] = ACTIONS(1577),
    [sym_duet_dev_type_ups] = ACTIONS(1577),
    [sym_duet_dev_type_amplifier] = ACTIONS(1577),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1577),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1577),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1577),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1577),
    [sym_duet_dev_type_camera] = ACTIONS(1577),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1577),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1577),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1577),
    [sym_duet_dev_type_dss] = ACTIONS(1577),
    [sym_duet_dev_type_dvr] = ACTIONS(1577),
    [sym_duet_dev_type_disc_device] = ACTIONS(1577),
    [sym_duet_dev_type_document_camera] = ACTIONS(1577),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1577),
    [sym_duet_dev_type_hvac] = ACTIONS(1577),
    [sym_duet_dev_type_keypad] = ACTIONS(1577),
    [sym_duet_dev_type_light] = ACTIONS(1577),
    [sym_duet_dev_type_monitor] = ACTIONS(1577),
    [sym_duet_dev_type_motor] = ACTIONS(1577),
    [sym_duet_dev_type_multi_window] = ACTIONS(1577),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1577),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1577),
    [sym_duet_dev_type_receiver] = ACTIONS(1577),
    [sym_duet_dev_type_security_system] = ACTIONS(1577),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1577),
    [sym_duet_dev_type_settop_box] = ACTIONS(1577),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1577),
    [sym_duet_dev_type_switcher] = ACTIONS(1577),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1577),
    [sym_duet_dev_type_tv] = ACTIONS(1577),
    [sym_duet_dev_type_utility] = ACTIONS(1577),
    [sym_duet_dev_type_vcr] = ACTIONS(1577),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1577),
    [sym_duet_dev_type_video_processor] = ACTIONS(1577),
    [sym_duet_dev_type_video_projector] = ACTIONS(1577),
    [sym_duet_dev_type_video_wall] = ACTIONS(1577),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1577),
    [sym_duet_dev_type_weather] = ACTIONS(1577),
    [sym_duet_dev_not_polled] = ACTIONS(1577),
    [sym_duet_dev_polled] = ACTIONS(1577),
    [sym_clkmgr_mode_network] = ACTIONS(1577),
    [sym_clkmgr_mode_standalone] = ACTIONS(1577),
    [sym_normal_standby] = ACTIONS(1577),
    [sym_normal_wake] = ACTIONS(1577),
    [sym_ip_multicast_ttl_option] = ACTIONS(1577),
    [sym_ip_ttl_subnet] = ACTIONS(1577),
    [sym_ip_ttl_site] = ACTIONS(1577),
    [sym_ip_ttl_region] = ACTIONS(1577),
    [sym_ip_ttl_continent] = ACTIONS(1577),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1577),
    [sym_ip_nodelay_on] = ACTIONS(1577),
    [sym_ip_nodelay_off] = ACTIONS(1577),
    [sym_amx_error] = ACTIONS(1577),
    [sym_amx_warning] = ACTIONS(1577),
    [sym_amx_info] = ACTIONS(1577),
    [sym_amx_debug] = ACTIONS(1577),
    [sym_netlinx_login_success] = ACTIONS(1577),
    [sym_netlinx_login_fail] = ACTIONS(1577),
    [sym_netlinx_logout] = ACTIONS(1577),
    [sym_null_str] = ACTIONS(1577),
    [sym_smtp_address] = ACTIONS(1577),
    [sym_smtp_port_number] = ACTIONS(1577),
    [sym_smtp_username] = ACTIONS(1577),
    [sym_smtp_password] = ACTIONS(1577),
    [sym_smtp_require_tls] = ACTIONS(1577),
    [sym_smtp_from] = ACTIONS(1577),
    [sym_smtp_tls_true] = ACTIONS(1577),
    [sym_smtp_tls_false] = ACTIONS(1577),
    [sym_listview_on_row_select_event] = ACTIONS(1577),
    [sym_data_structure_unknown] = ACTIONS(1577),
    [sym_data_structure_datafeed] = ACTIONS(1577),
    [sym_data_structure_datarecordset] = ACTIONS(1577),
    [sym_data_structure_datarecord] = ACTIONS(1577),
    [sym_data_structure_datafield] = ACTIONS(1577),
    [sym_data_max_content_field_count] = ACTIONS(1577),
    [sym_data_max_metadata_field_count] = ACTIONS(1577),
    [sym_data_max_id_length] = ACTIONS(1577),
    [sym_data_max_name_length] = ACTIONS(1577),
    [sym_data_max_description_length] = ACTIONS(1577),
    [sym_data_max_type_length] = ACTIONS(1577),
    [sym_data_max_format_length] = ACTIONS(1577),
    [sym_data_max_source_length] = ACTIONS(1577),
    [sym_data_max_value_length] = ACTIONS(1577),
    [sym_data_max_label_length] = ACTIONS(1577),
    [sym_data_max_url_length] = ACTIONS(1577),
    [sym_max_last_login_info_length] = ACTIONS(1577),
    [sym_data_type_unknown] = ACTIONS(1577),
    [sym_data_type_string] = ACTIONS(1577),
    [sym_data_type_datetime] = ACTIONS(1577),
    [sym_data_type_date] = ACTIONS(1577),
    [sym_data_type_time] = ACTIONS(1577),
    [sym_data_type_image] = ACTIONS(1577),
    [sym_data_format_url] = ACTIONS(1577),
    [sym_data_format_phone] = ACTIONS(1577),
    [sym_data_format_email] = ACTIONS(1577),
    [sym_data_format_iso8601] = ACTIONS(1577),
    [sym_auditlog_priv] = ACTIONS(1577),
    [sym_configuration_priv] = ACTIONS(1577),
    [sym_deviceconfig_priv] = ACTIONS(1577),
    [sym_ftp_priv] = ACTIONS(1577),
    [sym_http_priv] = ACTIONS(1577),
    [sym_networkconfig_priv] = ACTIONS(1577),
    [sym_programport_priv] = ACTIONS(1577),
    [sym_remoteui_priv] = ACTIONS(1577),
    [sym_securitycontrol_priv] = ACTIONS(1577),
    [sym_softwaremanagement_priv] = ACTIONS(1577),
    [sym_terminal_priv] = ACTIONS(1577),
    [sym_tpadmin_priv] = ACTIONS(1577),
    [sym_user1_priv] = ACTIONS(1577),
    [sym_user2_priv] = ACTIONS(1577),
    [sym_user3_priv] = ACTIONS(1577),
    [sym_user4_priv] = ACTIONS(1577),
    [sym_usermanagement_priv] = ACTIONS(1577),
    [sym_valid_account] = ACTIONS(1577),
    [sym_err_name_invalid] = ACTIONS(1577),
    [sym_err_password_invalid] = ACTIONS(1577),
    [sym_err_invalid_account] = ACTIONS(1577),
    [sym_err_no_authorization] = ACTIONS(1577),
    [sym_err_invalid_parameter] = ACTIONS(1577),
    [sym_err_user_locked_out] = ACTIONS(1577),
    [sym_err_user_account_expired] = ACTIONS(1577),
    [sym_err_general_error] = ACTIONS(1577),
    [sym_err_authentication_error] = ACTIONS(1577),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1577),
    [sym_tls_validate_certificate] = ACTIONS(1577),
    [sym_tdata] = ACTIONS(1577),
    [sym_tchannel] = ACTIONS(1577),
    [sym_tlevel] = ACTIONS(1577),
    [sym_tbutton] = ACTIONS(1577),
    [sym_ttimeline] = ACTIONS(1577),
    [sym_tcustom] = ACTIONS(1577),
    [sym_url_struct] = ACTIONS(1577),
    [sym_dns_struct] = ACTIONS(1577),
    [sym_ip_address_struct] = ACTIONS(1577),
    [sym_dev_info_struct] = ACTIONS(1577),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1577),
    [sym_data_feed] = ACTIONS(1577),
    [sym_data_field] = ACTIONS(1577),
    [sym_data_record] = ACTIONS(1577),
    [sym_wc_data_feed] = ACTIONS(1577),
    [sym_wc_data_field] = ACTIONS(1577),
    [sym_wc_data_record] = ACTIONS(1577),
    [sym_last_login_info] = ACTIONS(1577),
    [sym____reserved___] = ACTIONS(1577),
    [sym_date] = ACTIONS(1577),
    [sym_day] = ACTIONS(1577),
    [sym_time] = ACTIONS(1577),
    [sym_ldate] = ACTIONS(1577),
    [sym_push_device] = ACTIONS(1577),
    [sym_push_channel] = ACTIONS(1577),
    [sym_push_devchan] = ACTIONS(1577),
    [sym_release_device] = ACTIONS(1577),
    [sym_release_channel] = ACTIONS(1577),
    [sym_release_devchan] = ACTIONS(1577),
    [sym_master_slot] = ACTIONS(1577),
    [sym_get_pulse_time] = ACTIONS(1577),
    [sym_get_timer] = ACTIONS(1577),
    [sym_master_sn] = ACTIONS(1577),
    [sym_system_number] = ACTIONS(1577),
    [sym_button] = ACTIONS(1577),
    [sym_dv_channel] = ACTIONS(1577),
    [sym_data] = ACTIONS(1577),
    [sym_level] = ACTIONS(1577),
    [sym_timeline] = ACTIONS(1577),
    [sym_channel] = ACTIONS(1577),
    [sym_custom] = ACTIONS(1577),
    [sym_atoi] = ACTIONS(1577),
    [sym_atol] = ACTIONS(1577),
    [sym_atof] = ACTIONS(1577),
    [sym_itoa] = ACTIONS(1577),
    [sym_itohex] = ACTIONS(1577),
    [sym_hextoi] = ACTIONS(1577),
    [sym_ftoa] = ACTIONS(1577),
    [sym_find_string] = ACTIONS(1577),
    [sym_left_string] = ACTIONS(1577),
    [sym_right_string] = ACTIONS(1577),
    [sym_mid_string] = ACTIONS(1577),
    [sym_remove_string] = ACTIONS(1577),
    [sym_get_buffer_string] = ACTIONS(1577),
    [sym_get_buffer_char] = ACTIONS(1577),
    [sym_get_multi_buffer_string] = ACTIONS(1577),
    [sym_length_string] = ACTIONS(1577),
    [sym_max_length_string] = ACTIONS(1577),
    [sym_set_length_string] = ACTIONS(1577),
    [sym_compare_string] = ACTIONS(1577),
    [sym_upper_string] = ACTIONS(1577),
    [sym_lower_string] = ACTIONS(1577),
    [sym_get_last] = ACTIONS(1577),
    [sym_date_to_day] = ACTIONS(1577),
    [sym_date_to_month] = ACTIONS(1577),
    [sym_date_to_year] = ACTIONS(1577),
    [sym_time_to_hour] = ACTIONS(1577),
    [sym_time_to_minute] = ACTIONS(1577),
    [sym_time_to_second] = ACTIONS(1577),
    [sym_day_of_week] = ACTIONS(1577),
    [sym_set_pulse_time] = ACTIONS(1577),
    [sym_set_timer] = ACTIONS(1577),
    [sym_file_open] = ACTIONS(1577),
    [sym_file_close] = ACTIONS(1577),
    [sym_file_read] = ACTIONS(1577),
    [sym_file_read_line] = ACTIONS(1577),
    [sym_file_write] = ACTIONS(1577),
    [sym_file_write_line] = ACTIONS(1577),
    [sym_file_copy] = ACTIONS(1577),
    [sym_file_delete] = ACTIONS(1577),
    [sym_file_rename] = ACTIONS(1577),
    [sym_file_dir] = ACTIONS(1577),
    [sym_file_getdir] = ACTIONS(1577),
    [sym_file_setdir] = ACTIONS(1577),
    [sym_file_create_dir] = ACTIONS(1577),
    [sym_file_remove_dir] = ACTIONS(1577),
    [sym_file_seek] = ACTIONS(1577),
    [sym_length_array] = ACTIONS(1577),
    [sym_max_length_array] = ACTIONS(1577),
    [sym_set_length_array] = ACTIONS(1577),
    [sym_ip_client_open] = ACTIONS(1577),
    [sym_ip_client_close] = ACTIONS(1577),
    [sym_ip_server_open] = ACTIONS(1577),
    [sym_ip_server_close] = ACTIONS(1577),
    [sym_ip_bound_client_open] = ACTIONS(1577),
    [sym_ip_mc_server_open] = ACTIONS(1577),
    [sym_ip_set_option] = ACTIONS(1577),
    [sym_ssh_client_open] = ACTIONS(1577),
    [sym_ssh_client_close] = ACTIONS(1577),
    [sym_tls_client_open] = ACTIONS(1577),
    [sym_tls_client_close] = ACTIONS(1577),
    [sym_do_push] = ACTIONS(1577),
    [sym_do_push_timed] = ACTIONS(1577),
    [sym_do_release] = ACTIONS(1577),
    [sym_device_id] = ACTIONS(1577),
    [sym_device_info] = ACTIONS(1577),
    [sym_device_id_string] = ACTIONS(1577),
    [sym_device_standby] = ACTIONS(1577),
    [sym_device_wake] = ACTIONS(1577),
    [sym_timeline_create] = ACTIONS(1577),
    [sym_timeline_kill] = ACTIONS(1577),
    [sym_timeline_reload] = ACTIONS(1577),
    [sym_timeline_pause] = ACTIONS(1577),
    [sym_timeline_restart] = ACTIONS(1577),
    [sym_timeline_set] = ACTIONS(1577),
    [sym_timeline_get] = ACTIONS(1577),
    [sym_timeline_active] = ACTIONS(1577),
    [sym_timeline_dynamic_id] = ACTIONS(1577),
    [sym_variable_to_string] = ACTIONS(1577),
    [sym_string_to_variable] = ACTIONS(1577),
    [sym_length_variable_to_string] = ACTIONS(1577),
    [sym_type_cast] = ACTIONS(1577),
    [sym_variable_to_xml] = ACTIONS(1577),
    [sym_xml_to_variable] = ACTIONS(1577),
    [sym_length_variable_to_xml] = ACTIONS(1577),
    [sym_redirect_string] = ACTIONS(1577),
    [sym_random_number] = ACTIONS(1577),
    [sym_get_unique_id] = ACTIONS(1577),
    [sym_get_system_number] = ACTIONS(1577),
    [sym_set_system_number] = ACTIONS(1577),
    [sym_get_serial_number] = ACTIONS(1577),
    [sym_get_master_build] = ACTIONS(1577),
    [sym_set_validation_code] = ACTIONS(1577),
    [sym_reboot] = ACTIONS(1577),
    [sym_load_duet_module] = ACTIONS(1577),
    [sym_unload_duet_module] = ACTIONS(1577),
    [sym_duet_mem_size_set] = ACTIONS(1577),
    [sym_duet_mem_size_get] = ACTIONS(1577),
    [sym_dynamic_application_device] = ACTIONS(1577),
    [sym_dynamic_polled_port] = ACTIONS(1577),
    [sym_static_port_binding] = ACTIONS(1577),
    [sym_static_ip_binding] = ACTIONS(1577),
    [sym_get_dns_list] = ACTIONS(1577),
    [sym_set_dns_list] = ACTIONS(1577),
    [sym_get_ip_address] = ACTIONS(1577),
    [sym_set_ip_address] = ACTIONS(1577),
    [sym_get_url_list] = ACTIONS(1577),
    [sym_add_url_entry] = ACTIONS(1577),
    [sym_delete_url_entry] = ACTIONS(1577),
    [sym_abs_value] = ACTIONS(1577),
    [sym_max_value] = ACTIONS(1577),
    [sym_min_value] = ACTIONS(1577),
    [sym_exp_value] = ACTIONS(1577),
    [sym_log_value] = ACTIONS(1577),
    [sym_log10_value] = ACTIONS(1577),
    [sym_power_value] = ACTIONS(1577),
    [sym_sqrt_value] = ACTIONS(1577),
    [sym_raw_be] = ACTIONS(1577),
    [sym_raw_le] = ACTIONS(1577),
    [sym_internal_queue_size_set] = ACTIONS(1577),
    [sym_internal_queue_size_get] = ACTIONS(1577),
    [sym_internal_threshold_set] = ACTIONS(1577),
    [sym_internal_threshold_get] = ACTIONS(1577),
    [sym_rebuilt_event] = ACTIONS(1577),
    [sym_do_custom_event] = ACTIONS(1577),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1577),
    [sym_clkmgr_set_clk_source] = ACTIONS(1577),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1577),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1577),
    [sym_clkmgr_get_timezone] = ACTIONS(1577),
    [sym_clkmgr_set_timezone] = ACTIONS(1577),
    [sym_clkmgr_get_resync_period] = ACTIONS(1577),
    [sym_clkmgr_set_resync_period] = ACTIONS(1577),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1577),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1577),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1577),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1577),
    [sym_clkmgr_get_timeservers] = ACTIONS(1577),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1577),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1577),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1577),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1577),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1577),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1577),
    [sym_set_log_level] = ACTIONS(1577),
    [sym_get_log_level] = ACTIONS(1577),
    [sym_amx_log] = ACTIONS(1577),
    [sym_smtp_server_config_set] = ACTIONS(1577),
    [sym_smtp_server_config_get] = ACTIONS(1577),
    [sym_smtp_send] = ACTIONS(1577),
    [sym_validate_netlinx_account] = ACTIONS(1577),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1577),
    [sym_authenticate_certificate] = ACTIONS(1577),
    [sym_audit_netlinx_session_event] = ACTIONS(1577),
    [sym_audit_netlinx_generic_event] = ACTIONS(1577),
    [sym_get_available_flash_disk_space] = ACTIONS(1577),
    [sym_get_max_flash_disk_space] = ACTIONS(1577),
    [sym_data_create_feed] = ACTIONS(1577),
    [sym_data_delete_feed] = ACTIONS(1577),
    [sym_data_publish_feed] = ACTIONS(1577),
    [sym_data_get_published_feed] = ACTIONS(1577),
    [sym_data_add_record] = ACTIONS(1577),
    [sym_data_get_event_record] = ACTIONS(1577),
    [sym__wc_data_create_feed] = ACTIONS(1577),
    [sym__wc_data_add_record] = ACTIONS(1577),
    [sym__wc_data_get_event_record] = ACTIONS(1577),
    [sym_format] = ACTIONS(1577),
    [sym_astro_clock] = ACTIONS(1577),
    [sym_set_outdoor_temperature] = ACTIONS(1577),
    [sym_set_virtual_level_count] = ACTIONS(1577),
    [sym_set_virtual_channel_count] = ACTIONS(1577),
    [sym_set_virtual_port_count] = ACTIONS(1577),
    [sym__wc_encode_int] = ACTIONS(1577),
    [sym__wc_decode_int] = ACTIONS(1577),
    [sym__wc_int] = ACTIONS(1577),
    [sym__wc_to_ch_int] = ACTIONS(1577),
    [sym__ch_to_wc_int] = ACTIONS(1577),
    [sym__wc_find_string_int] = ACTIONS(1577),
    [sym__wc_left_string_int] = ACTIONS(1577),
    [sym__wc_lower_string_int] = ACTIONS(1577),
    [sym__wc_mid_string_int] = ACTIONS(1577),
    [sym__wc_remove_string_int] = ACTIONS(1577),
    [sym__wc_right_string_int] = ACTIONS(1577),
    [sym__wc_upper_string_int] = ACTIONS(1577),
    [sym__wc_compare_string_int] = ACTIONS(1577),
    [sym__wc_get_buffer_char_int] = ACTIONS(1577),
    [sym__wc_get_buffer_string_int] = ACTIONS(1577),
    [sym__wc_concat_string_int] = ACTIONS(1577),
    [sym__wc_file_open_int] = ACTIONS(1577),
    [sym__wc_file_close_int] = ACTIONS(1577),
    [sym__wc_file_read_int] = ACTIONS(1577),
    [sym__wc_file_read_line_int] = ACTIONS(1577),
    [sym__wc_file_write_int] = ACTIONS(1577),
    [sym__wc_file_write_line_int] = ACTIONS(1577),
    [aux_sym_preproc_include_token1] = ACTIONS(1577),
    [aux_sym_preproc_define_token1] = ACTIONS(1579),
    [aux_sym_preproc_warn_token1] = ACTIONS(1579),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1579),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1579),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1579),
    [anon_sym_LBRACK] = ACTIONS(1579),
    [anon_sym_LBRACE] = ACTIONS(1579),
    [anon_sym_RBRACE] = ACTIONS(1579),
    [anon_sym_COLON] = ACTIONS(1579),
    [anon_sym_LPAREN] = ACTIONS(1577),
    [anon_sym_SEMI] = ACTIONS(1579),
    [anon_sym_BANG] = ACTIONS(1577),
    [anon_sym_TILDE] = ACTIONS(1579),
    [anon_sym_DASH] = ACTIONS(1577),
    [anon_sym_PLUS] = ACTIONS(1577),
    [anon_sym_STAR] = ACTIONS(1579),
    [anon_sym_SLASH] = ACTIONS(1577),
    [anon_sym_PERCENT] = ACTIONS(1579),
    [anon_sym_PIPE_PIPE] = ACTIONS(1579),
    [anon_sym_AMP_AMP] = ACTIONS(1579),
    [anon_sym_PIPE] = ACTIONS(1577),
    [anon_sym_CARET] = ACTIONS(1579),
    [anon_sym_AMP] = ACTIONS(1577),
    [anon_sym_EQ_EQ] = ACTIONS(1579),
    [anon_sym_BANG_EQ] = ACTIONS(1579),
    [anon_sym_LT_GT] = ACTIONS(1579),
    [anon_sym_GT] = ACTIONS(1577),
    [anon_sym_GT_EQ] = ACTIONS(1579),
    [anon_sym_LT_EQ] = ACTIONS(1579),
    [anon_sym_LT] = ACTIONS(1577),
    [anon_sym_LT_LT] = ACTIONS(1579),
    [anon_sym_GT_GT] = ACTIONS(1579),
    [anon_sym_DASH_DASH] = ACTIONS(1579),
    [anon_sym_PLUS_PLUS] = ACTIONS(1579),
    [anon_sym_DOT] = ACTIONS(1577),
    [anon_sym_DQUOTE] = ACTIONS(1579),
    [anon_sym_SQUOTE] = ACTIONS(1579),
    [sym_number_literal] = ACTIONS(1579),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1579),
  },
  [STATE(188)] = {
    [sym_identifier] = ACTIONS(1569),
    [sym_define_call_keyword] = ACTIONS(1569),
    [sym_define_function_keyword] = ACTIONS(1569),
    [sym_define_library_function_keyword] = ACTIONS(1569),
    [sym_define_module_keyword] = ACTIONS(1569),
    [sym_char_keyword] = ACTIONS(1569),
    [sym_widechar_keyword] = ACTIONS(1569),
    [sym_integer_keyword] = ACTIONS(1569),
    [sym_sinteger_keyword] = ACTIONS(1569),
    [sym_long_keyword] = ACTIONS(1569),
    [sym_slong_keyword] = ACTIONS(1569),
    [sym_float_keyword] = ACTIONS(1569),
    [sym_double_keyword] = ACTIONS(1569),
    [sym_variant_keyword] = ACTIONS(1569),
    [sym_variantarray_keyword] = ACTIONS(1569),
    [sym_local_var_keyword] = ACTIONS(1569),
    [sym_stack_var_keyword] = ACTIONS(1569),
    [sym_constant_keyword] = ACTIONS(1569),
    [sym_volatile_keyword] = ACTIONS(1569),
    [sym_non_volatile_keyword] = ACTIONS(1569),
    [sym_persistent_keyword] = ACTIONS(1569),
    [sym_dev_keyword] = ACTIONS(1569),
    [sym_devlev_keyword] = ACTIONS(1569),
    [sym_devchan_keyword] = ACTIONS(1569),
    [sym_if_keyword] = ACTIONS(1569),
    [sym_else_keyword] = ACTIONS(1569),
    [sym_switch_keyword] = ACTIONS(1569),
    [sym_case_keyword] = ACTIONS(1569),
    [sym_default_keyword] = ACTIONS(1569),
    [sym_while_keyword] = ACTIONS(1569),
    [sym_for_keyword] = ACTIONS(1569),
    [sym_break_keyword] = ACTIONS(1569),
    [sym_continue_keyword] = ACTIONS(1569),
    [sym_return_keyword] = ACTIONS(1569),
    [sym_select_keyword] = ACTIONS(1569),
    [sym_active_keyword] = ACTIONS(1569),
    [sym_struct_keyword] = ACTIONS(1569),
    [sym_structure_keyword] = ACTIONS(1569),
    [aux_sym_on_keyword_token1] = ACTIONS(1569),
    [aux_sym_off_keyword_token1] = ACTIONS(1569),
    [sym_send_level_keyword] = ACTIONS(1569),
    [sym_send_string_keyword] = ACTIONS(1569),
    [sym_send_command_keyword] = ACTIONS(1569),
    [sym_clear_buffer_keyword] = ACTIONS(1569),
    [sym_create_buffer_keyword] = ACTIONS(1569),
    [sym_create_multi_buffer_keyword] = ACTIONS(1569),
    [sym_call_keyword] = ACTIONS(1569),
    [sym_system_call_keyword] = ACTIONS(1569),
    [sym_devchan_to_keyword] = ACTIONS(1569),
    [sym_devchan_min_to_keyword] = ACTIONS(1569),
    [sym_devchan_total_off_keyword] = ACTIONS(1569),
    [sym_devchan_pulse_keyword] = ACTIONS(1569),
    [sym_wait_keyword] = ACTIONS(1569),
    [sym_cancel_wait_keyword] = ACTIONS(1569),
    [sym_wait_until_keyword] = ACTIONS(1569),
    [sym_cancel_wait_until_keyword] = ACTIONS(1569),
    [sym_cancel_all_wait_keyword] = ACTIONS(1569),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1569),
    [sym_and] = ACTIONS(1569),
    [sym_or] = ACTIONS(1569),
    [sym_not] = ACTIONS(1569),
    [sym_xor] = ACTIONS(1569),
    [sym_band] = ACTIONS(1569),
    [sym_bor] = ACTIONS(1569),
    [sym_bxor] = ACTIONS(1569),
    [sym_bnot] = ACTIONS(1569),
    [sym_lshift] = ACTIONS(1569),
    [sym_rshift] = ACTIONS(1569),
    [sym___file__] = ACTIONS(1569),
    [sym___date__] = ACTIONS(1569),
    [sym___ldate__] = ACTIONS(1569),
    [sym___time__] = ACTIONS(1569),
    [sym___line__] = ACTIONS(1569),
    [sym___name__] = ACTIONS(1569),
    [sym___netlinx__] = ACTIONS(1569),
    [sym_netlinx_axi_version] = ACTIONS(1569),
    [sym_url_flg_tcp] = ACTIONS(1569),
    [sym_url_flg_accinfopresent] = ACTIONS(1569),
    [sym_url_flg_temp] = ACTIONS(1569),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1569),
    [sym_url_flg_stat_mask] = ACTIONS(1569),
    [sym_url_flg_stat_lookup] = ACTIONS(1569),
    [sym_url_flg_stat_connecting] = ACTIONS(1569),
    [sym_url_flg_stat_waiting] = ACTIONS(1569),
    [sym_url_flg_stat_connected] = ACTIONS(1569),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1569),
    [sym_first_virtual_device] = ACTIONS(1569),
    [sym_first_local_port] = ACTIONS(1569),
    [sym_dynamic_virtual_device] = ACTIONS(1569),
    [sym_timeline_once] = ACTIONS(1569),
    [sym_timeline_repeat] = ACTIONS(1569),
    [sym_timeline_absolute] = ACTIONS(1569),
    [sym_timeline_relative] = ACTIONS(1569),
    [sym_do_push_timed_infinite] = ACTIONS(1569),
    [sym_source_type_no_address] = ACTIONS(1569),
    [sym_source_type_neuron_id] = ACTIONS(1569),
    [sym_source_type_ip_address] = ACTIONS(1569),
    [sym_source_type_axlink] = ACTIONS(1569),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1569),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1569),
    [sym_source_type_ip_socket_address] = ACTIONS(1569),
    [sym_source_type_rs232] = ACTIONS(1569),
    [sym_source_type_internal] = ACTIONS(1569),
    [sym_source_type_mac_address] = ACTIONS(1569),
    [sym_source_type_ipv4_port] = ACTIONS(1569),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1569),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1569),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1569),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1569),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1569),
    [sym_true] = ACTIONS(1569),
    [sym_false] = ACTIONS(1569),
    [sym_file_read_only] = ACTIONS(1569),
    [sym_file_rw_new] = ACTIONS(1569),
    [sym_file_rw_append] = ACTIONS(1569),
    [sym_ip_tcp] = ACTIONS(1569),
    [sym_ip_udp] = ACTIONS(1569),
    [sym_ip_udp_2way] = ACTIONS(1569),
    [sym_xml_encode_types] = ACTIONS(1569),
    [sym_xml_encode_char_as_list] = ACTIONS(1569),
    [sym_xml_encode_le] = ACTIONS(1569),
    [sym_xml_decode_types] = ACTIONS(1569),
    [sym_xml_decode_no_preserve] = ACTIONS(1569),
    [sym_duet_dev_type_display_device] = ACTIONS(1569),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1569),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1569),
    [sym_duet_dev_type_io_device] = ACTIONS(1569),
    [sym_duet_dev_type_relay_device] = ACTIONS(1569),
    [sym_duet_dev_type_ups] = ACTIONS(1569),
    [sym_duet_dev_type_amplifier] = ACTIONS(1569),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1569),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1569),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1569),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1569),
    [sym_duet_dev_type_camera] = ACTIONS(1569),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1569),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1569),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1569),
    [sym_duet_dev_type_dss] = ACTIONS(1569),
    [sym_duet_dev_type_dvr] = ACTIONS(1569),
    [sym_duet_dev_type_disc_device] = ACTIONS(1569),
    [sym_duet_dev_type_document_camera] = ACTIONS(1569),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1569),
    [sym_duet_dev_type_hvac] = ACTIONS(1569),
    [sym_duet_dev_type_keypad] = ACTIONS(1569),
    [sym_duet_dev_type_light] = ACTIONS(1569),
    [sym_duet_dev_type_monitor] = ACTIONS(1569),
    [sym_duet_dev_type_motor] = ACTIONS(1569),
    [sym_duet_dev_type_multi_window] = ACTIONS(1569),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1569),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1569),
    [sym_duet_dev_type_receiver] = ACTIONS(1569),
    [sym_duet_dev_type_security_system] = ACTIONS(1569),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1569),
    [sym_duet_dev_type_settop_box] = ACTIONS(1569),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1569),
    [sym_duet_dev_type_switcher] = ACTIONS(1569),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1569),
    [sym_duet_dev_type_tv] = ACTIONS(1569),
    [sym_duet_dev_type_utility] = ACTIONS(1569),
    [sym_duet_dev_type_vcr] = ACTIONS(1569),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1569),
    [sym_duet_dev_type_video_processor] = ACTIONS(1569),
    [sym_duet_dev_type_video_projector] = ACTIONS(1569),
    [sym_duet_dev_type_video_wall] = ACTIONS(1569),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1569),
    [sym_duet_dev_type_weather] = ACTIONS(1569),
    [sym_duet_dev_not_polled] = ACTIONS(1569),
    [sym_duet_dev_polled] = ACTIONS(1569),
    [sym_clkmgr_mode_network] = ACTIONS(1569),
    [sym_clkmgr_mode_standalone] = ACTIONS(1569),
    [sym_normal_standby] = ACTIONS(1569),
    [sym_normal_wake] = ACTIONS(1569),
    [sym_ip_multicast_ttl_option] = ACTIONS(1569),
    [sym_ip_ttl_subnet] = ACTIONS(1569),
    [sym_ip_ttl_site] = ACTIONS(1569),
    [sym_ip_ttl_region] = ACTIONS(1569),
    [sym_ip_ttl_continent] = ACTIONS(1569),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1569),
    [sym_ip_nodelay_on] = ACTIONS(1569),
    [sym_ip_nodelay_off] = ACTIONS(1569),
    [sym_amx_error] = ACTIONS(1569),
    [sym_amx_warning] = ACTIONS(1569),
    [sym_amx_info] = ACTIONS(1569),
    [sym_amx_debug] = ACTIONS(1569),
    [sym_netlinx_login_success] = ACTIONS(1569),
    [sym_netlinx_login_fail] = ACTIONS(1569),
    [sym_netlinx_logout] = ACTIONS(1569),
    [sym_null_str] = ACTIONS(1569),
    [sym_smtp_address] = ACTIONS(1569),
    [sym_smtp_port_number] = ACTIONS(1569),
    [sym_smtp_username] = ACTIONS(1569),
    [sym_smtp_password] = ACTIONS(1569),
    [sym_smtp_require_tls] = ACTIONS(1569),
    [sym_smtp_from] = ACTIONS(1569),
    [sym_smtp_tls_true] = ACTIONS(1569),
    [sym_smtp_tls_false] = ACTIONS(1569),
    [sym_listview_on_row_select_event] = ACTIONS(1569),
    [sym_data_structure_unknown] = ACTIONS(1569),
    [sym_data_structure_datafeed] = ACTIONS(1569),
    [sym_data_structure_datarecordset] = ACTIONS(1569),
    [sym_data_structure_datarecord] = ACTIONS(1569),
    [sym_data_structure_datafield] = ACTIONS(1569),
    [sym_data_max_content_field_count] = ACTIONS(1569),
    [sym_data_max_metadata_field_count] = ACTIONS(1569),
    [sym_data_max_id_length] = ACTIONS(1569),
    [sym_data_max_name_length] = ACTIONS(1569),
    [sym_data_max_description_length] = ACTIONS(1569),
    [sym_data_max_type_length] = ACTIONS(1569),
    [sym_data_max_format_length] = ACTIONS(1569),
    [sym_data_max_source_length] = ACTIONS(1569),
    [sym_data_max_value_length] = ACTIONS(1569),
    [sym_data_max_label_length] = ACTIONS(1569),
    [sym_data_max_url_length] = ACTIONS(1569),
    [sym_max_last_login_info_length] = ACTIONS(1569),
    [sym_data_type_unknown] = ACTIONS(1569),
    [sym_data_type_string] = ACTIONS(1569),
    [sym_data_type_datetime] = ACTIONS(1569),
    [sym_data_type_date] = ACTIONS(1569),
    [sym_data_type_time] = ACTIONS(1569),
    [sym_data_type_image] = ACTIONS(1569),
    [sym_data_format_url] = ACTIONS(1569),
    [sym_data_format_phone] = ACTIONS(1569),
    [sym_data_format_email] = ACTIONS(1569),
    [sym_data_format_iso8601] = ACTIONS(1569),
    [sym_auditlog_priv] = ACTIONS(1569),
    [sym_configuration_priv] = ACTIONS(1569),
    [sym_deviceconfig_priv] = ACTIONS(1569),
    [sym_ftp_priv] = ACTIONS(1569),
    [sym_http_priv] = ACTIONS(1569),
    [sym_networkconfig_priv] = ACTIONS(1569),
    [sym_programport_priv] = ACTIONS(1569),
    [sym_remoteui_priv] = ACTIONS(1569),
    [sym_securitycontrol_priv] = ACTIONS(1569),
    [sym_softwaremanagement_priv] = ACTIONS(1569),
    [sym_terminal_priv] = ACTIONS(1569),
    [sym_tpadmin_priv] = ACTIONS(1569),
    [sym_user1_priv] = ACTIONS(1569),
    [sym_user2_priv] = ACTIONS(1569),
    [sym_user3_priv] = ACTIONS(1569),
    [sym_user4_priv] = ACTIONS(1569),
    [sym_usermanagement_priv] = ACTIONS(1569),
    [sym_valid_account] = ACTIONS(1569),
    [sym_err_name_invalid] = ACTIONS(1569),
    [sym_err_password_invalid] = ACTIONS(1569),
    [sym_err_invalid_account] = ACTIONS(1569),
    [sym_err_no_authorization] = ACTIONS(1569),
    [sym_err_invalid_parameter] = ACTIONS(1569),
    [sym_err_user_locked_out] = ACTIONS(1569),
    [sym_err_user_account_expired] = ACTIONS(1569),
    [sym_err_general_error] = ACTIONS(1569),
    [sym_err_authentication_error] = ACTIONS(1569),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1569),
    [sym_tls_validate_certificate] = ACTIONS(1569),
    [sym_tdata] = ACTIONS(1569),
    [sym_tchannel] = ACTIONS(1569),
    [sym_tlevel] = ACTIONS(1569),
    [sym_tbutton] = ACTIONS(1569),
    [sym_ttimeline] = ACTIONS(1569),
    [sym_tcustom] = ACTIONS(1569),
    [sym_url_struct] = ACTIONS(1569),
    [sym_dns_struct] = ACTIONS(1569),
    [sym_ip_address_struct] = ACTIONS(1569),
    [sym_dev_info_struct] = ACTIONS(1569),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1569),
    [sym_data_feed] = ACTIONS(1569),
    [sym_data_field] = ACTIONS(1569),
    [sym_data_record] = ACTIONS(1569),
    [sym_wc_data_feed] = ACTIONS(1569),
    [sym_wc_data_field] = ACTIONS(1569),
    [sym_wc_data_record] = ACTIONS(1569),
    [sym_last_login_info] = ACTIONS(1569),
    [sym____reserved___] = ACTIONS(1569),
    [sym_date] = ACTIONS(1569),
    [sym_day] = ACTIONS(1569),
    [sym_time] = ACTIONS(1569),
    [sym_ldate] = ACTIONS(1569),
    [sym_push_device] = ACTIONS(1569),
    [sym_push_channel] = ACTIONS(1569),
    [sym_push_devchan] = ACTIONS(1569),
    [sym_release_device] = ACTIONS(1569),
    [sym_release_channel] = ACTIONS(1569),
    [sym_release_devchan] = ACTIONS(1569),
    [sym_master_slot] = ACTIONS(1569),
    [sym_get_pulse_time] = ACTIONS(1569),
    [sym_get_timer] = ACTIONS(1569),
    [sym_master_sn] = ACTIONS(1569),
    [sym_system_number] = ACTIONS(1569),
    [sym_button] = ACTIONS(1569),
    [sym_dv_channel] = ACTIONS(1569),
    [sym_data] = ACTIONS(1569),
    [sym_level] = ACTIONS(1569),
    [sym_timeline] = ACTIONS(1569),
    [sym_channel] = ACTIONS(1569),
    [sym_custom] = ACTIONS(1569),
    [sym_atoi] = ACTIONS(1569),
    [sym_atol] = ACTIONS(1569),
    [sym_atof] = ACTIONS(1569),
    [sym_itoa] = ACTIONS(1569),
    [sym_itohex] = ACTIONS(1569),
    [sym_hextoi] = ACTIONS(1569),
    [sym_ftoa] = ACTIONS(1569),
    [sym_find_string] = ACTIONS(1569),
    [sym_left_string] = ACTIONS(1569),
    [sym_right_string] = ACTIONS(1569),
    [sym_mid_string] = ACTIONS(1569),
    [sym_remove_string] = ACTIONS(1569),
    [sym_get_buffer_string] = ACTIONS(1569),
    [sym_get_buffer_char] = ACTIONS(1569),
    [sym_get_multi_buffer_string] = ACTIONS(1569),
    [sym_length_string] = ACTIONS(1569),
    [sym_max_length_string] = ACTIONS(1569),
    [sym_set_length_string] = ACTIONS(1569),
    [sym_compare_string] = ACTIONS(1569),
    [sym_upper_string] = ACTIONS(1569),
    [sym_lower_string] = ACTIONS(1569),
    [sym_get_last] = ACTIONS(1569),
    [sym_date_to_day] = ACTIONS(1569),
    [sym_date_to_month] = ACTIONS(1569),
    [sym_date_to_year] = ACTIONS(1569),
    [sym_time_to_hour] = ACTIONS(1569),
    [sym_time_to_minute] = ACTIONS(1569),
    [sym_time_to_second] = ACTIONS(1569),
    [sym_day_of_week] = ACTIONS(1569),
    [sym_set_pulse_time] = ACTIONS(1569),
    [sym_set_timer] = ACTIONS(1569),
    [sym_file_open] = ACTIONS(1569),
    [sym_file_close] = ACTIONS(1569),
    [sym_file_read] = ACTIONS(1569),
    [sym_file_read_line] = ACTIONS(1569),
    [sym_file_write] = ACTIONS(1569),
    [sym_file_write_line] = ACTIONS(1569),
    [sym_file_copy] = ACTIONS(1569),
    [sym_file_delete] = ACTIONS(1569),
    [sym_file_rename] = ACTIONS(1569),
    [sym_file_dir] = ACTIONS(1569),
    [sym_file_getdir] = ACTIONS(1569),
    [sym_file_setdir] = ACTIONS(1569),
    [sym_file_create_dir] = ACTIONS(1569),
    [sym_file_remove_dir] = ACTIONS(1569),
    [sym_file_seek] = ACTIONS(1569),
    [sym_length_array] = ACTIONS(1569),
    [sym_max_length_array] = ACTIONS(1569),
    [sym_set_length_array] = ACTIONS(1569),
    [sym_ip_client_open] = ACTIONS(1569),
    [sym_ip_client_close] = ACTIONS(1569),
    [sym_ip_server_open] = ACTIONS(1569),
    [sym_ip_server_close] = ACTIONS(1569),
    [sym_ip_bound_client_open] = ACTIONS(1569),
    [sym_ip_mc_server_open] = ACTIONS(1569),
    [sym_ip_set_option] = ACTIONS(1569),
    [sym_ssh_client_open] = ACTIONS(1569),
    [sym_ssh_client_close] = ACTIONS(1569),
    [sym_tls_client_open] = ACTIONS(1569),
    [sym_tls_client_close] = ACTIONS(1569),
    [sym_do_push] = ACTIONS(1569),
    [sym_do_push_timed] = ACTIONS(1569),
    [sym_do_release] = ACTIONS(1569),
    [sym_device_id] = ACTIONS(1569),
    [sym_device_info] = ACTIONS(1569),
    [sym_device_id_string] = ACTIONS(1569),
    [sym_device_standby] = ACTIONS(1569),
    [sym_device_wake] = ACTIONS(1569),
    [sym_timeline_create] = ACTIONS(1569),
    [sym_timeline_kill] = ACTIONS(1569),
    [sym_timeline_reload] = ACTIONS(1569),
    [sym_timeline_pause] = ACTIONS(1569),
    [sym_timeline_restart] = ACTIONS(1569),
    [sym_timeline_set] = ACTIONS(1569),
    [sym_timeline_get] = ACTIONS(1569),
    [sym_timeline_active] = ACTIONS(1569),
    [sym_timeline_dynamic_id] = ACTIONS(1569),
    [sym_variable_to_string] = ACTIONS(1569),
    [sym_string_to_variable] = ACTIONS(1569),
    [sym_length_variable_to_string] = ACTIONS(1569),
    [sym_type_cast] = ACTIONS(1569),
    [sym_variable_to_xml] = ACTIONS(1569),
    [sym_xml_to_variable] = ACTIONS(1569),
    [sym_length_variable_to_xml] = ACTIONS(1569),
    [sym_redirect_string] = ACTIONS(1569),
    [sym_random_number] = ACTIONS(1569),
    [sym_get_unique_id] = ACTIONS(1569),
    [sym_get_system_number] = ACTIONS(1569),
    [sym_set_system_number] = ACTIONS(1569),
    [sym_get_serial_number] = ACTIONS(1569),
    [sym_get_master_build] = ACTIONS(1569),
    [sym_set_validation_code] = ACTIONS(1569),
    [sym_reboot] = ACTIONS(1569),
    [sym_load_duet_module] = ACTIONS(1569),
    [sym_unload_duet_module] = ACTIONS(1569),
    [sym_duet_mem_size_set] = ACTIONS(1569),
    [sym_duet_mem_size_get] = ACTIONS(1569),
    [sym_dynamic_application_device] = ACTIONS(1569),
    [sym_dynamic_polled_port] = ACTIONS(1569),
    [sym_static_port_binding] = ACTIONS(1569),
    [sym_static_ip_binding] = ACTIONS(1569),
    [sym_get_dns_list] = ACTIONS(1569),
    [sym_set_dns_list] = ACTIONS(1569),
    [sym_get_ip_address] = ACTIONS(1569),
    [sym_set_ip_address] = ACTIONS(1569),
    [sym_get_url_list] = ACTIONS(1569),
    [sym_add_url_entry] = ACTIONS(1569),
    [sym_delete_url_entry] = ACTIONS(1569),
    [sym_abs_value] = ACTIONS(1569),
    [sym_max_value] = ACTIONS(1569),
    [sym_min_value] = ACTIONS(1569),
    [sym_exp_value] = ACTIONS(1569),
    [sym_log_value] = ACTIONS(1569),
    [sym_log10_value] = ACTIONS(1569),
    [sym_power_value] = ACTIONS(1569),
    [sym_sqrt_value] = ACTIONS(1569),
    [sym_raw_be] = ACTIONS(1569),
    [sym_raw_le] = ACTIONS(1569),
    [sym_internal_queue_size_set] = ACTIONS(1569),
    [sym_internal_queue_size_get] = ACTIONS(1569),
    [sym_internal_threshold_set] = ACTIONS(1569),
    [sym_internal_threshold_get] = ACTIONS(1569),
    [sym_rebuilt_event] = ACTIONS(1569),
    [sym_do_custom_event] = ACTIONS(1569),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1569),
    [sym_clkmgr_set_clk_source] = ACTIONS(1569),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1569),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1569),
    [sym_clkmgr_get_timezone] = ACTIONS(1569),
    [sym_clkmgr_set_timezone] = ACTIONS(1569),
    [sym_clkmgr_get_resync_period] = ACTIONS(1569),
    [sym_clkmgr_set_resync_period] = ACTIONS(1569),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1569),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1569),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1569),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1569),
    [sym_clkmgr_get_timeservers] = ACTIONS(1569),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1569),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1569),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1569),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1569),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1569),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1569),
    [sym_set_log_level] = ACTIONS(1569),
    [sym_get_log_level] = ACTIONS(1569),
    [sym_amx_log] = ACTIONS(1569),
    [sym_smtp_server_config_set] = ACTIONS(1569),
    [sym_smtp_server_config_get] = ACTIONS(1569),
    [sym_smtp_send] = ACTIONS(1569),
    [sym_validate_netlinx_account] = ACTIONS(1569),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1569),
    [sym_authenticate_certificate] = ACTIONS(1569),
    [sym_audit_netlinx_session_event] = ACTIONS(1569),
    [sym_audit_netlinx_generic_event] = ACTIONS(1569),
    [sym_get_available_flash_disk_space] = ACTIONS(1569),
    [sym_get_max_flash_disk_space] = ACTIONS(1569),
    [sym_data_create_feed] = ACTIONS(1569),
    [sym_data_delete_feed] = ACTIONS(1569),
    [sym_data_publish_feed] = ACTIONS(1569),
    [sym_data_get_published_feed] = ACTIONS(1569),
    [sym_data_add_record] = ACTIONS(1569),
    [sym_data_get_event_record] = ACTIONS(1569),
    [sym__wc_data_create_feed] = ACTIONS(1569),
    [sym__wc_data_add_record] = ACTIONS(1569),
    [sym__wc_data_get_event_record] = ACTIONS(1569),
    [sym_format] = ACTIONS(1569),
    [sym_astro_clock] = ACTIONS(1569),
    [sym_set_outdoor_temperature] = ACTIONS(1569),
    [sym_set_virtual_level_count] = ACTIONS(1569),
    [sym_set_virtual_channel_count] = ACTIONS(1569),
    [sym_set_virtual_port_count] = ACTIONS(1569),
    [sym__wc_encode_int] = ACTIONS(1569),
    [sym__wc_decode_int] = ACTIONS(1569),
    [sym__wc_int] = ACTIONS(1569),
    [sym__wc_to_ch_int] = ACTIONS(1569),
    [sym__ch_to_wc_int] = ACTIONS(1569),
    [sym__wc_find_string_int] = ACTIONS(1569),
    [sym__wc_left_string_int] = ACTIONS(1569),
    [sym__wc_lower_string_int] = ACTIONS(1569),
    [sym__wc_mid_string_int] = ACTIONS(1569),
    [sym__wc_remove_string_int] = ACTIONS(1569),
    [sym__wc_right_string_int] = ACTIONS(1569),
    [sym__wc_upper_string_int] = ACTIONS(1569),
    [sym__wc_compare_string_int] = ACTIONS(1569),
    [sym__wc_get_buffer_char_int] = ACTIONS(1569),
    [sym__wc_get_buffer_string_int] = ACTIONS(1569),
    [sym__wc_concat_string_int] = ACTIONS(1569),
    [sym__wc_file_open_int] = ACTIONS(1569),
    [sym__wc_file_close_int] = ACTIONS(1569),
    [sym__wc_file_read_int] = ACTIONS(1569),
    [sym__wc_file_read_line_int] = ACTIONS(1569),
    [sym__wc_file_write_int] = ACTIONS(1569),
    [sym__wc_file_write_line_int] = ACTIONS(1569),
    [aux_sym_preproc_include_token1] = ACTIONS(1569),
    [aux_sym_preproc_define_token1] = ACTIONS(1571),
    [aux_sym_preproc_warn_token1] = ACTIONS(1571),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1571),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1571),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1571),
    [anon_sym_LBRACK] = ACTIONS(1571),
    [anon_sym_LBRACE] = ACTIONS(1571),
    [anon_sym_RBRACE] = ACTIONS(1571),
    [anon_sym_COLON] = ACTIONS(1571),
    [anon_sym_LPAREN] = ACTIONS(1569),
    [anon_sym_SEMI] = ACTIONS(1571),
    [anon_sym_BANG] = ACTIONS(1569),
    [anon_sym_TILDE] = ACTIONS(1571),
    [anon_sym_DASH] = ACTIONS(1569),
    [anon_sym_PLUS] = ACTIONS(1569),
    [anon_sym_STAR] = ACTIONS(1571),
    [anon_sym_SLASH] = ACTIONS(1569),
    [anon_sym_PERCENT] = ACTIONS(1571),
    [anon_sym_PIPE_PIPE] = ACTIONS(1571),
    [anon_sym_AMP_AMP] = ACTIONS(1571),
    [anon_sym_PIPE] = ACTIONS(1569),
    [anon_sym_CARET] = ACTIONS(1571),
    [anon_sym_AMP] = ACTIONS(1569),
    [anon_sym_EQ_EQ] = ACTIONS(1571),
    [anon_sym_BANG_EQ] = ACTIONS(1571),
    [anon_sym_LT_GT] = ACTIONS(1571),
    [anon_sym_GT] = ACTIONS(1569),
    [anon_sym_GT_EQ] = ACTIONS(1571),
    [anon_sym_LT_EQ] = ACTIONS(1571),
    [anon_sym_LT] = ACTIONS(1569),
    [anon_sym_LT_LT] = ACTIONS(1571),
    [anon_sym_GT_GT] = ACTIONS(1571),
    [anon_sym_DASH_DASH] = ACTIONS(1571),
    [anon_sym_PLUS_PLUS] = ACTIONS(1571),
    [anon_sym_DOT] = ACTIONS(1569),
    [anon_sym_DQUOTE] = ACTIONS(1571),
    [anon_sym_SQUOTE] = ACTIONS(1571),
    [sym_number_literal] = ACTIONS(1571),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1571),
  },
  [STATE(189)] = {
    [sym_identifier] = ACTIONS(1492),
    [sym_define_call_keyword] = ACTIONS(1492),
    [sym_define_function_keyword] = ACTIONS(1492),
    [sym_define_library_function_keyword] = ACTIONS(1492),
    [sym_define_module_keyword] = ACTIONS(1492),
    [sym_char_keyword] = ACTIONS(1492),
    [sym_widechar_keyword] = ACTIONS(1492),
    [sym_integer_keyword] = ACTIONS(1492),
    [sym_sinteger_keyword] = ACTIONS(1492),
    [sym_long_keyword] = ACTIONS(1492),
    [sym_slong_keyword] = ACTIONS(1492),
    [sym_float_keyword] = ACTIONS(1492),
    [sym_double_keyword] = ACTIONS(1492),
    [sym_variant_keyword] = ACTIONS(1492),
    [sym_variantarray_keyword] = ACTIONS(1492),
    [sym_local_var_keyword] = ACTIONS(1492),
    [sym_stack_var_keyword] = ACTIONS(1492),
    [sym_constant_keyword] = ACTIONS(1492),
    [sym_volatile_keyword] = ACTIONS(1492),
    [sym_non_volatile_keyword] = ACTIONS(1492),
    [sym_persistent_keyword] = ACTIONS(1492),
    [sym_dev_keyword] = ACTIONS(1492),
    [sym_devlev_keyword] = ACTIONS(1492),
    [sym_devchan_keyword] = ACTIONS(1492),
    [sym_if_keyword] = ACTIONS(1492),
    [sym_else_keyword] = ACTIONS(1492),
    [sym_switch_keyword] = ACTIONS(1492),
    [sym_case_keyword] = ACTIONS(1492),
    [sym_default_keyword] = ACTIONS(1492),
    [sym_while_keyword] = ACTIONS(1492),
    [sym_for_keyword] = ACTIONS(1492),
    [sym_break_keyword] = ACTIONS(1492),
    [sym_continue_keyword] = ACTIONS(1492),
    [sym_return_keyword] = ACTIONS(1492),
    [sym_select_keyword] = ACTIONS(1492),
    [sym_active_keyword] = ACTIONS(1492),
    [sym_struct_keyword] = ACTIONS(1492),
    [sym_structure_keyword] = ACTIONS(1492),
    [aux_sym_on_keyword_token1] = ACTIONS(1492),
    [aux_sym_off_keyword_token1] = ACTIONS(1492),
    [sym_send_level_keyword] = ACTIONS(1492),
    [sym_send_string_keyword] = ACTIONS(1492),
    [sym_send_command_keyword] = ACTIONS(1492),
    [sym_clear_buffer_keyword] = ACTIONS(1492),
    [sym_create_buffer_keyword] = ACTIONS(1492),
    [sym_create_multi_buffer_keyword] = ACTIONS(1492),
    [sym_call_keyword] = ACTIONS(1492),
    [sym_system_call_keyword] = ACTIONS(1492),
    [sym_devchan_to_keyword] = ACTIONS(1492),
    [sym_devchan_min_to_keyword] = ACTIONS(1492),
    [sym_devchan_total_off_keyword] = ACTIONS(1492),
    [sym_devchan_pulse_keyword] = ACTIONS(1492),
    [sym_wait_keyword] = ACTIONS(1492),
    [sym_cancel_wait_keyword] = ACTIONS(1492),
    [sym_wait_until_keyword] = ACTIONS(1492),
    [sym_cancel_wait_until_keyword] = ACTIONS(1492),
    [sym_cancel_all_wait_keyword] = ACTIONS(1492),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1492),
    [sym_and] = ACTIONS(1492),
    [sym_or] = ACTIONS(1492),
    [sym_not] = ACTIONS(1492),
    [sym_xor] = ACTIONS(1492),
    [sym_band] = ACTIONS(1492),
    [sym_bor] = ACTIONS(1492),
    [sym_bxor] = ACTIONS(1492),
    [sym_bnot] = ACTIONS(1492),
    [sym_lshift] = ACTIONS(1492),
    [sym_rshift] = ACTIONS(1492),
    [sym___file__] = ACTIONS(1492),
    [sym___date__] = ACTIONS(1492),
    [sym___ldate__] = ACTIONS(1492),
    [sym___time__] = ACTIONS(1492),
    [sym___line__] = ACTIONS(1492),
    [sym___name__] = ACTIONS(1492),
    [sym___netlinx__] = ACTIONS(1492),
    [sym_netlinx_axi_version] = ACTIONS(1492),
    [sym_url_flg_tcp] = ACTIONS(1492),
    [sym_url_flg_accinfopresent] = ACTIONS(1492),
    [sym_url_flg_temp] = ACTIONS(1492),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1492),
    [sym_url_flg_stat_mask] = ACTIONS(1492),
    [sym_url_flg_stat_lookup] = ACTIONS(1492),
    [sym_url_flg_stat_connecting] = ACTIONS(1492),
    [sym_url_flg_stat_waiting] = ACTIONS(1492),
    [sym_url_flg_stat_connected] = ACTIONS(1492),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1492),
    [sym_first_virtual_device] = ACTIONS(1492),
    [sym_first_local_port] = ACTIONS(1492),
    [sym_dynamic_virtual_device] = ACTIONS(1492),
    [sym_timeline_once] = ACTIONS(1492),
    [sym_timeline_repeat] = ACTIONS(1492),
    [sym_timeline_absolute] = ACTIONS(1492),
    [sym_timeline_relative] = ACTIONS(1492),
    [sym_do_push_timed_infinite] = ACTIONS(1492),
    [sym_source_type_no_address] = ACTIONS(1492),
    [sym_source_type_neuron_id] = ACTIONS(1492),
    [sym_source_type_ip_address] = ACTIONS(1492),
    [sym_source_type_axlink] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1492),
    [sym_source_type_ip_socket_address] = ACTIONS(1492),
    [sym_source_type_rs232] = ACTIONS(1492),
    [sym_source_type_internal] = ACTIONS(1492),
    [sym_source_type_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1492),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1492),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1492),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1492),
    [sym_true] = ACTIONS(1492),
    [sym_false] = ACTIONS(1492),
    [sym_file_read_only] = ACTIONS(1492),
    [sym_file_rw_new] = ACTIONS(1492),
    [sym_file_rw_append] = ACTIONS(1492),
    [sym_ip_tcp] = ACTIONS(1492),
    [sym_ip_udp] = ACTIONS(1492),
    [sym_ip_udp_2way] = ACTIONS(1492),
    [sym_xml_encode_types] = ACTIONS(1492),
    [sym_xml_encode_char_as_list] = ACTIONS(1492),
    [sym_xml_encode_le] = ACTIONS(1492),
    [sym_xml_decode_types] = ACTIONS(1492),
    [sym_xml_decode_no_preserve] = ACTIONS(1492),
    [sym_duet_dev_type_display_device] = ACTIONS(1492),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_io_device] = ACTIONS(1492),
    [sym_duet_dev_type_relay_device] = ACTIONS(1492),
    [sym_duet_dev_type_ups] = ACTIONS(1492),
    [sym_duet_dev_type_amplifier] = ACTIONS(1492),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1492),
    [sym_duet_dev_type_camera] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1492),
    [sym_duet_dev_type_dss] = ACTIONS(1492),
    [sym_duet_dev_type_dvr] = ACTIONS(1492),
    [sym_duet_dev_type_disc_device] = ACTIONS(1492),
    [sym_duet_dev_type_document_camera] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1492),
    [sym_duet_dev_type_hvac] = ACTIONS(1492),
    [sym_duet_dev_type_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_light] = ACTIONS(1492),
    [sym_duet_dev_type_monitor] = ACTIONS(1492),
    [sym_duet_dev_type_motor] = ACTIONS(1492),
    [sym_duet_dev_type_multi_window] = ACTIONS(1492),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1492),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1492),
    [sym_duet_dev_type_receiver] = ACTIONS(1492),
    [sym_duet_dev_type_security_system] = ACTIONS(1492),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1492),
    [sym_duet_dev_type_settop_box] = ACTIONS(1492),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1492),
    [sym_duet_dev_type_switcher] = ACTIONS(1492),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_tv] = ACTIONS(1492),
    [sym_duet_dev_type_utility] = ACTIONS(1492),
    [sym_duet_dev_type_vcr] = ACTIONS(1492),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_video_processor] = ACTIONS(1492),
    [sym_duet_dev_type_video_projector] = ACTIONS(1492),
    [sym_duet_dev_type_video_wall] = ACTIONS(1492),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1492),
    [sym_duet_dev_type_weather] = ACTIONS(1492),
    [sym_duet_dev_not_polled] = ACTIONS(1492),
    [sym_duet_dev_polled] = ACTIONS(1492),
    [sym_clkmgr_mode_network] = ACTIONS(1492),
    [sym_clkmgr_mode_standalone] = ACTIONS(1492),
    [sym_normal_standby] = ACTIONS(1492),
    [sym_normal_wake] = ACTIONS(1492),
    [sym_ip_multicast_ttl_option] = ACTIONS(1492),
    [sym_ip_ttl_subnet] = ACTIONS(1492),
    [sym_ip_ttl_site] = ACTIONS(1492),
    [sym_ip_ttl_region] = ACTIONS(1492),
    [sym_ip_ttl_continent] = ACTIONS(1492),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1492),
    [sym_ip_nodelay_on] = ACTIONS(1492),
    [sym_ip_nodelay_off] = ACTIONS(1492),
    [sym_amx_error] = ACTIONS(1492),
    [sym_amx_warning] = ACTIONS(1492),
    [sym_amx_info] = ACTIONS(1492),
    [sym_amx_debug] = ACTIONS(1492),
    [sym_netlinx_login_success] = ACTIONS(1492),
    [sym_netlinx_login_fail] = ACTIONS(1492),
    [sym_netlinx_logout] = ACTIONS(1492),
    [sym_null_str] = ACTIONS(1492),
    [sym_smtp_address] = ACTIONS(1492),
    [sym_smtp_port_number] = ACTIONS(1492),
    [sym_smtp_username] = ACTIONS(1492),
    [sym_smtp_password] = ACTIONS(1492),
    [sym_smtp_require_tls] = ACTIONS(1492),
    [sym_smtp_from] = ACTIONS(1492),
    [sym_smtp_tls_true] = ACTIONS(1492),
    [sym_smtp_tls_false] = ACTIONS(1492),
    [sym_listview_on_row_select_event] = ACTIONS(1492),
    [sym_data_structure_unknown] = ACTIONS(1492),
    [sym_data_structure_datafeed] = ACTIONS(1492),
    [sym_data_structure_datarecordset] = ACTIONS(1492),
    [sym_data_structure_datarecord] = ACTIONS(1492),
    [sym_data_structure_datafield] = ACTIONS(1492),
    [sym_data_max_content_field_count] = ACTIONS(1492),
    [sym_data_max_metadata_field_count] = ACTIONS(1492),
    [sym_data_max_id_length] = ACTIONS(1492),
    [sym_data_max_name_length] = ACTIONS(1492),
    [sym_data_max_description_length] = ACTIONS(1492),
    [sym_data_max_type_length] = ACTIONS(1492),
    [sym_data_max_format_length] = ACTIONS(1492),
    [sym_data_max_source_length] = ACTIONS(1492),
    [sym_data_max_value_length] = ACTIONS(1492),
    [sym_data_max_label_length] = ACTIONS(1492),
    [sym_data_max_url_length] = ACTIONS(1492),
    [sym_max_last_login_info_length] = ACTIONS(1492),
    [sym_data_type_unknown] = ACTIONS(1492),
    [sym_data_type_string] = ACTIONS(1492),
    [sym_data_type_datetime] = ACTIONS(1492),
    [sym_data_type_date] = ACTIONS(1492),
    [sym_data_type_time] = ACTIONS(1492),
    [sym_data_type_image] = ACTIONS(1492),
    [sym_data_format_url] = ACTIONS(1492),
    [sym_data_format_phone] = ACTIONS(1492),
    [sym_data_format_email] = ACTIONS(1492),
    [sym_data_format_iso8601] = ACTIONS(1492),
    [sym_auditlog_priv] = ACTIONS(1492),
    [sym_configuration_priv] = ACTIONS(1492),
    [sym_deviceconfig_priv] = ACTIONS(1492),
    [sym_ftp_priv] = ACTIONS(1492),
    [sym_http_priv] = ACTIONS(1492),
    [sym_networkconfig_priv] = ACTIONS(1492),
    [sym_programport_priv] = ACTIONS(1492),
    [sym_remoteui_priv] = ACTIONS(1492),
    [sym_securitycontrol_priv] = ACTIONS(1492),
    [sym_softwaremanagement_priv] = ACTIONS(1492),
    [sym_terminal_priv] = ACTIONS(1492),
    [sym_tpadmin_priv] = ACTIONS(1492),
    [sym_user1_priv] = ACTIONS(1492),
    [sym_user2_priv] = ACTIONS(1492),
    [sym_user3_priv] = ACTIONS(1492),
    [sym_user4_priv] = ACTIONS(1492),
    [sym_usermanagement_priv] = ACTIONS(1492),
    [sym_valid_account] = ACTIONS(1492),
    [sym_err_name_invalid] = ACTIONS(1492),
    [sym_err_password_invalid] = ACTIONS(1492),
    [sym_err_invalid_account] = ACTIONS(1492),
    [sym_err_no_authorization] = ACTIONS(1492),
    [sym_err_invalid_parameter] = ACTIONS(1492),
    [sym_err_user_locked_out] = ACTIONS(1492),
    [sym_err_user_account_expired] = ACTIONS(1492),
    [sym_err_general_error] = ACTIONS(1492),
    [sym_err_authentication_error] = ACTIONS(1492),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1492),
    [sym_tls_validate_certificate] = ACTIONS(1492),
    [sym_tdata] = ACTIONS(1492),
    [sym_tchannel] = ACTIONS(1492),
    [sym_tlevel] = ACTIONS(1492),
    [sym_tbutton] = ACTIONS(1492),
    [sym_ttimeline] = ACTIONS(1492),
    [sym_tcustom] = ACTIONS(1492),
    [sym_url_struct] = ACTIONS(1492),
    [sym_dns_struct] = ACTIONS(1492),
    [sym_ip_address_struct] = ACTIONS(1492),
    [sym_dev_info_struct] = ACTIONS(1492),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1492),
    [sym_data_feed] = ACTIONS(1492),
    [sym_data_field] = ACTIONS(1492),
    [sym_data_record] = ACTIONS(1492),
    [sym_wc_data_feed] = ACTIONS(1492),
    [sym_wc_data_field] = ACTIONS(1492),
    [sym_wc_data_record] = ACTIONS(1492),
    [sym_last_login_info] = ACTIONS(1492),
    [sym____reserved___] = ACTIONS(1492),
    [sym_date] = ACTIONS(1492),
    [sym_day] = ACTIONS(1492),
    [sym_time] = ACTIONS(1492),
    [sym_ldate] = ACTIONS(1492),
    [sym_push_device] = ACTIONS(1492),
    [sym_push_channel] = ACTIONS(1492),
    [sym_push_devchan] = ACTIONS(1492),
    [sym_release_device] = ACTIONS(1492),
    [sym_release_channel] = ACTIONS(1492),
    [sym_release_devchan] = ACTIONS(1492),
    [sym_master_slot] = ACTIONS(1492),
    [sym_get_pulse_time] = ACTIONS(1492),
    [sym_get_timer] = ACTIONS(1492),
    [sym_master_sn] = ACTIONS(1492),
    [sym_system_number] = ACTIONS(1492),
    [sym_button] = ACTIONS(1492),
    [sym_dv_channel] = ACTIONS(1492),
    [sym_data] = ACTIONS(1492),
    [sym_level] = ACTIONS(1492),
    [sym_timeline] = ACTIONS(1492),
    [sym_channel] = ACTIONS(1492),
    [sym_custom] = ACTIONS(1492),
    [sym_atoi] = ACTIONS(1492),
    [sym_atol] = ACTIONS(1492),
    [sym_atof] = ACTIONS(1492),
    [sym_itoa] = ACTIONS(1492),
    [sym_itohex] = ACTIONS(1492),
    [sym_hextoi] = ACTIONS(1492),
    [sym_ftoa] = ACTIONS(1492),
    [sym_find_string] = ACTIONS(1492),
    [sym_left_string] = ACTIONS(1492),
    [sym_right_string] = ACTIONS(1492),
    [sym_mid_string] = ACTIONS(1492),
    [sym_remove_string] = ACTIONS(1492),
    [sym_get_buffer_string] = ACTIONS(1492),
    [sym_get_buffer_char] = ACTIONS(1492),
    [sym_get_multi_buffer_string] = ACTIONS(1492),
    [sym_length_string] = ACTIONS(1492),
    [sym_max_length_string] = ACTIONS(1492),
    [sym_set_length_string] = ACTIONS(1492),
    [sym_compare_string] = ACTIONS(1492),
    [sym_upper_string] = ACTIONS(1492),
    [sym_lower_string] = ACTIONS(1492),
    [sym_get_last] = ACTIONS(1492),
    [sym_date_to_day] = ACTIONS(1492),
    [sym_date_to_month] = ACTIONS(1492),
    [sym_date_to_year] = ACTIONS(1492),
    [sym_time_to_hour] = ACTIONS(1492),
    [sym_time_to_minute] = ACTIONS(1492),
    [sym_time_to_second] = ACTIONS(1492),
    [sym_day_of_week] = ACTIONS(1492),
    [sym_set_pulse_time] = ACTIONS(1492),
    [sym_set_timer] = ACTIONS(1492),
    [sym_file_open] = ACTIONS(1492),
    [sym_file_close] = ACTIONS(1492),
    [sym_file_read] = ACTIONS(1492),
    [sym_file_read_line] = ACTIONS(1492),
    [sym_file_write] = ACTIONS(1492),
    [sym_file_write_line] = ACTIONS(1492),
    [sym_file_copy] = ACTIONS(1492),
    [sym_file_delete] = ACTIONS(1492),
    [sym_file_rename] = ACTIONS(1492),
    [sym_file_dir] = ACTIONS(1492),
    [sym_file_getdir] = ACTIONS(1492),
    [sym_file_setdir] = ACTIONS(1492),
    [sym_file_create_dir] = ACTIONS(1492),
    [sym_file_remove_dir] = ACTIONS(1492),
    [sym_file_seek] = ACTIONS(1492),
    [sym_length_array] = ACTIONS(1492),
    [sym_max_length_array] = ACTIONS(1492),
    [sym_set_length_array] = ACTIONS(1492),
    [sym_ip_client_open] = ACTIONS(1492),
    [sym_ip_client_close] = ACTIONS(1492),
    [sym_ip_server_open] = ACTIONS(1492),
    [sym_ip_server_close] = ACTIONS(1492),
    [sym_ip_bound_client_open] = ACTIONS(1492),
    [sym_ip_mc_server_open] = ACTIONS(1492),
    [sym_ip_set_option] = ACTIONS(1492),
    [sym_ssh_client_open] = ACTIONS(1492),
    [sym_ssh_client_close] = ACTIONS(1492),
    [sym_tls_client_open] = ACTIONS(1492),
    [sym_tls_client_close] = ACTIONS(1492),
    [sym_do_push] = ACTIONS(1492),
    [sym_do_push_timed] = ACTIONS(1492),
    [sym_do_release] = ACTIONS(1492),
    [sym_device_id] = ACTIONS(1492),
    [sym_device_info] = ACTIONS(1492),
    [sym_device_id_string] = ACTIONS(1492),
    [sym_device_standby] = ACTIONS(1492),
    [sym_device_wake] = ACTIONS(1492),
    [sym_timeline_create] = ACTIONS(1492),
    [sym_timeline_kill] = ACTIONS(1492),
    [sym_timeline_reload] = ACTIONS(1492),
    [sym_timeline_pause] = ACTIONS(1492),
    [sym_timeline_restart] = ACTIONS(1492),
    [sym_timeline_set] = ACTIONS(1492),
    [sym_timeline_get] = ACTIONS(1492),
    [sym_timeline_active] = ACTIONS(1492),
    [sym_timeline_dynamic_id] = ACTIONS(1492),
    [sym_variable_to_string] = ACTIONS(1492),
    [sym_string_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_string] = ACTIONS(1492),
    [sym_type_cast] = ACTIONS(1492),
    [sym_variable_to_xml] = ACTIONS(1492),
    [sym_xml_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_xml] = ACTIONS(1492),
    [sym_redirect_string] = ACTIONS(1492),
    [sym_random_number] = ACTIONS(1492),
    [sym_get_unique_id] = ACTIONS(1492),
    [sym_get_system_number] = ACTIONS(1492),
    [sym_set_system_number] = ACTIONS(1492),
    [sym_get_serial_number] = ACTIONS(1492),
    [sym_get_master_build] = ACTIONS(1492),
    [sym_set_validation_code] = ACTIONS(1492),
    [sym_reboot] = ACTIONS(1492),
    [sym_load_duet_module] = ACTIONS(1492),
    [sym_unload_duet_module] = ACTIONS(1492),
    [sym_duet_mem_size_set] = ACTIONS(1492),
    [sym_duet_mem_size_get] = ACTIONS(1492),
    [sym_dynamic_application_device] = ACTIONS(1492),
    [sym_dynamic_polled_port] = ACTIONS(1492),
    [sym_static_port_binding] = ACTIONS(1492),
    [sym_static_ip_binding] = ACTIONS(1492),
    [sym_get_dns_list] = ACTIONS(1492),
    [sym_set_dns_list] = ACTIONS(1492),
    [sym_get_ip_address] = ACTIONS(1492),
    [sym_set_ip_address] = ACTIONS(1492),
    [sym_get_url_list] = ACTIONS(1492),
    [sym_add_url_entry] = ACTIONS(1492),
    [sym_delete_url_entry] = ACTIONS(1492),
    [sym_abs_value] = ACTIONS(1492),
    [sym_max_value] = ACTIONS(1492),
    [sym_min_value] = ACTIONS(1492),
    [sym_exp_value] = ACTIONS(1492),
    [sym_log_value] = ACTIONS(1492),
    [sym_log10_value] = ACTIONS(1492),
    [sym_power_value] = ACTIONS(1492),
    [sym_sqrt_value] = ACTIONS(1492),
    [sym_raw_be] = ACTIONS(1492),
    [sym_raw_le] = ACTIONS(1492),
    [sym_internal_queue_size_set] = ACTIONS(1492),
    [sym_internal_queue_size_get] = ACTIONS(1492),
    [sym_internal_threshold_set] = ACTIONS(1492),
    [sym_internal_threshold_get] = ACTIONS(1492),
    [sym_rebuilt_event] = ACTIONS(1492),
    [sym_do_custom_event] = ACTIONS(1492),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1492),
    [sym_clkmgr_set_clk_source] = ACTIONS(1492),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1492),
    [sym_clkmgr_get_timezone] = ACTIONS(1492),
    [sym_clkmgr_set_timezone] = ACTIONS(1492),
    [sym_clkmgr_get_resync_period] = ACTIONS(1492),
    [sym_clkmgr_set_resync_period] = ACTIONS(1492),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_timeservers] = ACTIONS(1492),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_set_log_level] = ACTIONS(1492),
    [sym_get_log_level] = ACTIONS(1492),
    [sym_amx_log] = ACTIONS(1492),
    [sym_smtp_server_config_set] = ACTIONS(1492),
    [sym_smtp_server_config_get] = ACTIONS(1492),
    [sym_smtp_send] = ACTIONS(1492),
    [sym_validate_netlinx_account] = ACTIONS(1492),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1492),
    [sym_authenticate_certificate] = ACTIONS(1492),
    [sym_audit_netlinx_session_event] = ACTIONS(1492),
    [sym_audit_netlinx_generic_event] = ACTIONS(1492),
    [sym_get_available_flash_disk_space] = ACTIONS(1492),
    [sym_get_max_flash_disk_space] = ACTIONS(1492),
    [sym_data_create_feed] = ACTIONS(1492),
    [sym_data_delete_feed] = ACTIONS(1492),
    [sym_data_publish_feed] = ACTIONS(1492),
    [sym_data_get_published_feed] = ACTIONS(1492),
    [sym_data_add_record] = ACTIONS(1492),
    [sym_data_get_event_record] = ACTIONS(1492),
    [sym__wc_data_create_feed] = ACTIONS(1492),
    [sym__wc_data_add_record] = ACTIONS(1492),
    [sym__wc_data_get_event_record] = ACTIONS(1492),
    [sym_format] = ACTIONS(1492),
    [sym_astro_clock] = ACTIONS(1492),
    [sym_set_outdoor_temperature] = ACTIONS(1492),
    [sym_set_virtual_level_count] = ACTIONS(1492),
    [sym_set_virtual_channel_count] = ACTIONS(1492),
    [sym_set_virtual_port_count] = ACTIONS(1492),
    [sym__wc_encode_int] = ACTIONS(1492),
    [sym__wc_decode_int] = ACTIONS(1492),
    [sym__wc_int] = ACTIONS(1492),
    [sym__wc_to_ch_int] = ACTIONS(1492),
    [sym__ch_to_wc_int] = ACTIONS(1492),
    [sym__wc_find_string_int] = ACTIONS(1492),
    [sym__wc_left_string_int] = ACTIONS(1492),
    [sym__wc_lower_string_int] = ACTIONS(1492),
    [sym__wc_mid_string_int] = ACTIONS(1492),
    [sym__wc_remove_string_int] = ACTIONS(1492),
    [sym__wc_right_string_int] = ACTIONS(1492),
    [sym__wc_upper_string_int] = ACTIONS(1492),
    [sym__wc_compare_string_int] = ACTIONS(1492),
    [sym__wc_get_buffer_char_int] = ACTIONS(1492),
    [sym__wc_get_buffer_string_int] = ACTIONS(1492),
    [sym__wc_concat_string_int] = ACTIONS(1492),
    [sym__wc_file_open_int] = ACTIONS(1492),
    [sym__wc_file_close_int] = ACTIONS(1492),
    [sym__wc_file_read_int] = ACTIONS(1492),
    [sym__wc_file_read_line_int] = ACTIONS(1492),
    [sym__wc_file_write_int] = ACTIONS(1492),
    [sym__wc_file_write_line_int] = ACTIONS(1492),
    [aux_sym_preproc_include_token1] = ACTIONS(1492),
    [aux_sym_preproc_define_token1] = ACTIONS(1494),
    [aux_sym_preproc_warn_token1] = ACTIONS(1494),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1494),
    [anon_sym_LBRACK] = ACTIONS(1494),
    [anon_sym_LBRACE] = ACTIONS(1494),
    [anon_sym_RBRACE] = ACTIONS(1494),
    [anon_sym_COLON] = ACTIONS(1494),
    [anon_sym_LPAREN] = ACTIONS(1492),
    [anon_sym_SEMI] = ACTIONS(1494),
    [anon_sym_BANG] = ACTIONS(1492),
    [anon_sym_TILDE] = ACTIONS(1494),
    [anon_sym_DASH] = ACTIONS(1492),
    [anon_sym_PLUS] = ACTIONS(1492),
    [anon_sym_STAR] = ACTIONS(1494),
    [anon_sym_SLASH] = ACTIONS(1492),
    [anon_sym_PERCENT] = ACTIONS(1494),
    [anon_sym_PIPE_PIPE] = ACTIONS(1494),
    [anon_sym_AMP_AMP] = ACTIONS(1494),
    [anon_sym_PIPE] = ACTIONS(1492),
    [anon_sym_CARET] = ACTIONS(1494),
    [anon_sym_AMP] = ACTIONS(1492),
    [anon_sym_EQ_EQ] = ACTIONS(1494),
    [anon_sym_BANG_EQ] = ACTIONS(1494),
    [anon_sym_LT_GT] = ACTIONS(1494),
    [anon_sym_GT] = ACTIONS(1492),
    [anon_sym_GT_EQ] = ACTIONS(1494),
    [anon_sym_LT_EQ] = ACTIONS(1494),
    [anon_sym_LT] = ACTIONS(1492),
    [anon_sym_LT_LT] = ACTIONS(1494),
    [anon_sym_GT_GT] = ACTIONS(1494),
    [anon_sym_DASH_DASH] = ACTIONS(1494),
    [anon_sym_PLUS_PLUS] = ACTIONS(1494),
    [anon_sym_DOT] = ACTIONS(1492),
    [anon_sym_DQUOTE] = ACTIONS(1494),
    [anon_sym_SQUOTE] = ACTIONS(1494),
    [sym_number_literal] = ACTIONS(1494),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1494),
  },
  [STATE(190)] = {
    [sym_identifier] = ACTIONS(1565),
    [sym_define_call_keyword] = ACTIONS(1565),
    [sym_define_function_keyword] = ACTIONS(1565),
    [sym_define_library_function_keyword] = ACTIONS(1565),
    [sym_define_module_keyword] = ACTIONS(1565),
    [sym_char_keyword] = ACTIONS(1565),
    [sym_widechar_keyword] = ACTIONS(1565),
    [sym_integer_keyword] = ACTIONS(1565),
    [sym_sinteger_keyword] = ACTIONS(1565),
    [sym_long_keyword] = ACTIONS(1565),
    [sym_slong_keyword] = ACTIONS(1565),
    [sym_float_keyword] = ACTIONS(1565),
    [sym_double_keyword] = ACTIONS(1565),
    [sym_variant_keyword] = ACTIONS(1565),
    [sym_variantarray_keyword] = ACTIONS(1565),
    [sym_local_var_keyword] = ACTIONS(1565),
    [sym_stack_var_keyword] = ACTIONS(1565),
    [sym_constant_keyword] = ACTIONS(1565),
    [sym_volatile_keyword] = ACTIONS(1565),
    [sym_non_volatile_keyword] = ACTIONS(1565),
    [sym_persistent_keyword] = ACTIONS(1565),
    [sym_dev_keyword] = ACTIONS(1565),
    [sym_devlev_keyword] = ACTIONS(1565),
    [sym_devchan_keyword] = ACTIONS(1565),
    [sym_if_keyword] = ACTIONS(1565),
    [sym_else_keyword] = ACTIONS(1565),
    [sym_switch_keyword] = ACTIONS(1565),
    [sym_case_keyword] = ACTIONS(1565),
    [sym_default_keyword] = ACTIONS(1565),
    [sym_while_keyword] = ACTIONS(1565),
    [sym_for_keyword] = ACTIONS(1565),
    [sym_break_keyword] = ACTIONS(1565),
    [sym_continue_keyword] = ACTIONS(1565),
    [sym_return_keyword] = ACTIONS(1565),
    [sym_select_keyword] = ACTIONS(1565),
    [sym_active_keyword] = ACTIONS(1565),
    [sym_struct_keyword] = ACTIONS(1565),
    [sym_structure_keyword] = ACTIONS(1565),
    [aux_sym_on_keyword_token1] = ACTIONS(1565),
    [aux_sym_off_keyword_token1] = ACTIONS(1565),
    [sym_send_level_keyword] = ACTIONS(1565),
    [sym_send_string_keyword] = ACTIONS(1565),
    [sym_send_command_keyword] = ACTIONS(1565),
    [sym_clear_buffer_keyword] = ACTIONS(1565),
    [sym_create_buffer_keyword] = ACTIONS(1565),
    [sym_create_multi_buffer_keyword] = ACTIONS(1565),
    [sym_call_keyword] = ACTIONS(1565),
    [sym_system_call_keyword] = ACTIONS(1565),
    [sym_devchan_to_keyword] = ACTIONS(1565),
    [sym_devchan_min_to_keyword] = ACTIONS(1565),
    [sym_devchan_total_off_keyword] = ACTIONS(1565),
    [sym_devchan_pulse_keyword] = ACTIONS(1565),
    [sym_wait_keyword] = ACTIONS(1565),
    [sym_cancel_wait_keyword] = ACTIONS(1565),
    [sym_wait_until_keyword] = ACTIONS(1565),
    [sym_cancel_wait_until_keyword] = ACTIONS(1565),
    [sym_cancel_all_wait_keyword] = ACTIONS(1565),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1565),
    [sym_and] = ACTIONS(1565),
    [sym_or] = ACTIONS(1565),
    [sym_not] = ACTIONS(1565),
    [sym_xor] = ACTIONS(1565),
    [sym_band] = ACTIONS(1565),
    [sym_bor] = ACTIONS(1565),
    [sym_bxor] = ACTIONS(1565),
    [sym_bnot] = ACTIONS(1565),
    [sym_lshift] = ACTIONS(1565),
    [sym_rshift] = ACTIONS(1565),
    [sym___file__] = ACTIONS(1565),
    [sym___date__] = ACTIONS(1565),
    [sym___ldate__] = ACTIONS(1565),
    [sym___time__] = ACTIONS(1565),
    [sym___line__] = ACTIONS(1565),
    [sym___name__] = ACTIONS(1565),
    [sym___netlinx__] = ACTIONS(1565),
    [sym_netlinx_axi_version] = ACTIONS(1565),
    [sym_url_flg_tcp] = ACTIONS(1565),
    [sym_url_flg_accinfopresent] = ACTIONS(1565),
    [sym_url_flg_temp] = ACTIONS(1565),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1565),
    [sym_url_flg_stat_mask] = ACTIONS(1565),
    [sym_url_flg_stat_lookup] = ACTIONS(1565),
    [sym_url_flg_stat_connecting] = ACTIONS(1565),
    [sym_url_flg_stat_waiting] = ACTIONS(1565),
    [sym_url_flg_stat_connected] = ACTIONS(1565),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1565),
    [sym_first_virtual_device] = ACTIONS(1565),
    [sym_first_local_port] = ACTIONS(1565),
    [sym_dynamic_virtual_device] = ACTIONS(1565),
    [sym_timeline_once] = ACTIONS(1565),
    [sym_timeline_repeat] = ACTIONS(1565),
    [sym_timeline_absolute] = ACTIONS(1565),
    [sym_timeline_relative] = ACTIONS(1565),
    [sym_do_push_timed_infinite] = ACTIONS(1565),
    [sym_source_type_no_address] = ACTIONS(1565),
    [sym_source_type_neuron_id] = ACTIONS(1565),
    [sym_source_type_ip_address] = ACTIONS(1565),
    [sym_source_type_axlink] = ACTIONS(1565),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1565),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1565),
    [sym_source_type_ip_socket_address] = ACTIONS(1565),
    [sym_source_type_rs232] = ACTIONS(1565),
    [sym_source_type_internal] = ACTIONS(1565),
    [sym_source_type_mac_address] = ACTIONS(1565),
    [sym_source_type_ipv4_port] = ACTIONS(1565),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1565),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1565),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1565),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1565),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1565),
    [sym_true] = ACTIONS(1565),
    [sym_false] = ACTIONS(1565),
    [sym_file_read_only] = ACTIONS(1565),
    [sym_file_rw_new] = ACTIONS(1565),
    [sym_file_rw_append] = ACTIONS(1565),
    [sym_ip_tcp] = ACTIONS(1565),
    [sym_ip_udp] = ACTIONS(1565),
    [sym_ip_udp_2way] = ACTIONS(1565),
    [sym_xml_encode_types] = ACTIONS(1565),
    [sym_xml_encode_char_as_list] = ACTIONS(1565),
    [sym_xml_encode_le] = ACTIONS(1565),
    [sym_xml_decode_types] = ACTIONS(1565),
    [sym_xml_decode_no_preserve] = ACTIONS(1565),
    [sym_duet_dev_type_display_device] = ACTIONS(1565),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1565),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1565),
    [sym_duet_dev_type_io_device] = ACTIONS(1565),
    [sym_duet_dev_type_relay_device] = ACTIONS(1565),
    [sym_duet_dev_type_ups] = ACTIONS(1565),
    [sym_duet_dev_type_amplifier] = ACTIONS(1565),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1565),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1565),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1565),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1565),
    [sym_duet_dev_type_camera] = ACTIONS(1565),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1565),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1565),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1565),
    [sym_duet_dev_type_dss] = ACTIONS(1565),
    [sym_duet_dev_type_dvr] = ACTIONS(1565),
    [sym_duet_dev_type_disc_device] = ACTIONS(1565),
    [sym_duet_dev_type_document_camera] = ACTIONS(1565),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1565),
    [sym_duet_dev_type_hvac] = ACTIONS(1565),
    [sym_duet_dev_type_keypad] = ACTIONS(1565),
    [sym_duet_dev_type_light] = ACTIONS(1565),
    [sym_duet_dev_type_monitor] = ACTIONS(1565),
    [sym_duet_dev_type_motor] = ACTIONS(1565),
    [sym_duet_dev_type_multi_window] = ACTIONS(1565),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1565),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1565),
    [sym_duet_dev_type_receiver] = ACTIONS(1565),
    [sym_duet_dev_type_security_system] = ACTIONS(1565),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1565),
    [sym_duet_dev_type_settop_box] = ACTIONS(1565),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1565),
    [sym_duet_dev_type_switcher] = ACTIONS(1565),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1565),
    [sym_duet_dev_type_tv] = ACTIONS(1565),
    [sym_duet_dev_type_utility] = ACTIONS(1565),
    [sym_duet_dev_type_vcr] = ACTIONS(1565),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1565),
    [sym_duet_dev_type_video_processor] = ACTIONS(1565),
    [sym_duet_dev_type_video_projector] = ACTIONS(1565),
    [sym_duet_dev_type_video_wall] = ACTIONS(1565),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1565),
    [sym_duet_dev_type_weather] = ACTIONS(1565),
    [sym_duet_dev_not_polled] = ACTIONS(1565),
    [sym_duet_dev_polled] = ACTIONS(1565),
    [sym_clkmgr_mode_network] = ACTIONS(1565),
    [sym_clkmgr_mode_standalone] = ACTIONS(1565),
    [sym_normal_standby] = ACTIONS(1565),
    [sym_normal_wake] = ACTIONS(1565),
    [sym_ip_multicast_ttl_option] = ACTIONS(1565),
    [sym_ip_ttl_subnet] = ACTIONS(1565),
    [sym_ip_ttl_site] = ACTIONS(1565),
    [sym_ip_ttl_region] = ACTIONS(1565),
    [sym_ip_ttl_continent] = ACTIONS(1565),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1565),
    [sym_ip_nodelay_on] = ACTIONS(1565),
    [sym_ip_nodelay_off] = ACTIONS(1565),
    [sym_amx_error] = ACTIONS(1565),
    [sym_amx_warning] = ACTIONS(1565),
    [sym_amx_info] = ACTIONS(1565),
    [sym_amx_debug] = ACTIONS(1565),
    [sym_netlinx_login_success] = ACTIONS(1565),
    [sym_netlinx_login_fail] = ACTIONS(1565),
    [sym_netlinx_logout] = ACTIONS(1565),
    [sym_null_str] = ACTIONS(1565),
    [sym_smtp_address] = ACTIONS(1565),
    [sym_smtp_port_number] = ACTIONS(1565),
    [sym_smtp_username] = ACTIONS(1565),
    [sym_smtp_password] = ACTIONS(1565),
    [sym_smtp_require_tls] = ACTIONS(1565),
    [sym_smtp_from] = ACTIONS(1565),
    [sym_smtp_tls_true] = ACTIONS(1565),
    [sym_smtp_tls_false] = ACTIONS(1565),
    [sym_listview_on_row_select_event] = ACTIONS(1565),
    [sym_data_structure_unknown] = ACTIONS(1565),
    [sym_data_structure_datafeed] = ACTIONS(1565),
    [sym_data_structure_datarecordset] = ACTIONS(1565),
    [sym_data_structure_datarecord] = ACTIONS(1565),
    [sym_data_structure_datafield] = ACTIONS(1565),
    [sym_data_max_content_field_count] = ACTIONS(1565),
    [sym_data_max_metadata_field_count] = ACTIONS(1565),
    [sym_data_max_id_length] = ACTIONS(1565),
    [sym_data_max_name_length] = ACTIONS(1565),
    [sym_data_max_description_length] = ACTIONS(1565),
    [sym_data_max_type_length] = ACTIONS(1565),
    [sym_data_max_format_length] = ACTIONS(1565),
    [sym_data_max_source_length] = ACTIONS(1565),
    [sym_data_max_value_length] = ACTIONS(1565),
    [sym_data_max_label_length] = ACTIONS(1565),
    [sym_data_max_url_length] = ACTIONS(1565),
    [sym_max_last_login_info_length] = ACTIONS(1565),
    [sym_data_type_unknown] = ACTIONS(1565),
    [sym_data_type_string] = ACTIONS(1565),
    [sym_data_type_datetime] = ACTIONS(1565),
    [sym_data_type_date] = ACTIONS(1565),
    [sym_data_type_time] = ACTIONS(1565),
    [sym_data_type_image] = ACTIONS(1565),
    [sym_data_format_url] = ACTIONS(1565),
    [sym_data_format_phone] = ACTIONS(1565),
    [sym_data_format_email] = ACTIONS(1565),
    [sym_data_format_iso8601] = ACTIONS(1565),
    [sym_auditlog_priv] = ACTIONS(1565),
    [sym_configuration_priv] = ACTIONS(1565),
    [sym_deviceconfig_priv] = ACTIONS(1565),
    [sym_ftp_priv] = ACTIONS(1565),
    [sym_http_priv] = ACTIONS(1565),
    [sym_networkconfig_priv] = ACTIONS(1565),
    [sym_programport_priv] = ACTIONS(1565),
    [sym_remoteui_priv] = ACTIONS(1565),
    [sym_securitycontrol_priv] = ACTIONS(1565),
    [sym_softwaremanagement_priv] = ACTIONS(1565),
    [sym_terminal_priv] = ACTIONS(1565),
    [sym_tpadmin_priv] = ACTIONS(1565),
    [sym_user1_priv] = ACTIONS(1565),
    [sym_user2_priv] = ACTIONS(1565),
    [sym_user3_priv] = ACTIONS(1565),
    [sym_user4_priv] = ACTIONS(1565),
    [sym_usermanagement_priv] = ACTIONS(1565),
    [sym_valid_account] = ACTIONS(1565),
    [sym_err_name_invalid] = ACTIONS(1565),
    [sym_err_password_invalid] = ACTIONS(1565),
    [sym_err_invalid_account] = ACTIONS(1565),
    [sym_err_no_authorization] = ACTIONS(1565),
    [sym_err_invalid_parameter] = ACTIONS(1565),
    [sym_err_user_locked_out] = ACTIONS(1565),
    [sym_err_user_account_expired] = ACTIONS(1565),
    [sym_err_general_error] = ACTIONS(1565),
    [sym_err_authentication_error] = ACTIONS(1565),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1565),
    [sym_tls_validate_certificate] = ACTIONS(1565),
    [sym_tdata] = ACTIONS(1565),
    [sym_tchannel] = ACTIONS(1565),
    [sym_tlevel] = ACTIONS(1565),
    [sym_tbutton] = ACTIONS(1565),
    [sym_ttimeline] = ACTIONS(1565),
    [sym_tcustom] = ACTIONS(1565),
    [sym_url_struct] = ACTIONS(1565),
    [sym_dns_struct] = ACTIONS(1565),
    [sym_ip_address_struct] = ACTIONS(1565),
    [sym_dev_info_struct] = ACTIONS(1565),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1565),
    [sym_data_feed] = ACTIONS(1565),
    [sym_data_field] = ACTIONS(1565),
    [sym_data_record] = ACTIONS(1565),
    [sym_wc_data_feed] = ACTIONS(1565),
    [sym_wc_data_field] = ACTIONS(1565),
    [sym_wc_data_record] = ACTIONS(1565),
    [sym_last_login_info] = ACTIONS(1565),
    [sym____reserved___] = ACTIONS(1565),
    [sym_date] = ACTIONS(1565),
    [sym_day] = ACTIONS(1565),
    [sym_time] = ACTIONS(1565),
    [sym_ldate] = ACTIONS(1565),
    [sym_push_device] = ACTIONS(1565),
    [sym_push_channel] = ACTIONS(1565),
    [sym_push_devchan] = ACTIONS(1565),
    [sym_release_device] = ACTIONS(1565),
    [sym_release_channel] = ACTIONS(1565),
    [sym_release_devchan] = ACTIONS(1565),
    [sym_master_slot] = ACTIONS(1565),
    [sym_get_pulse_time] = ACTIONS(1565),
    [sym_get_timer] = ACTIONS(1565),
    [sym_master_sn] = ACTIONS(1565),
    [sym_system_number] = ACTIONS(1565),
    [sym_button] = ACTIONS(1565),
    [sym_dv_channel] = ACTIONS(1565),
    [sym_data] = ACTIONS(1565),
    [sym_level] = ACTIONS(1565),
    [sym_timeline] = ACTIONS(1565),
    [sym_channel] = ACTIONS(1565),
    [sym_custom] = ACTIONS(1565),
    [sym_atoi] = ACTIONS(1565),
    [sym_atol] = ACTIONS(1565),
    [sym_atof] = ACTIONS(1565),
    [sym_itoa] = ACTIONS(1565),
    [sym_itohex] = ACTIONS(1565),
    [sym_hextoi] = ACTIONS(1565),
    [sym_ftoa] = ACTIONS(1565),
    [sym_find_string] = ACTIONS(1565),
    [sym_left_string] = ACTIONS(1565),
    [sym_right_string] = ACTIONS(1565),
    [sym_mid_string] = ACTIONS(1565),
    [sym_remove_string] = ACTIONS(1565),
    [sym_get_buffer_string] = ACTIONS(1565),
    [sym_get_buffer_char] = ACTIONS(1565),
    [sym_get_multi_buffer_string] = ACTIONS(1565),
    [sym_length_string] = ACTIONS(1565),
    [sym_max_length_string] = ACTIONS(1565),
    [sym_set_length_string] = ACTIONS(1565),
    [sym_compare_string] = ACTIONS(1565),
    [sym_upper_string] = ACTIONS(1565),
    [sym_lower_string] = ACTIONS(1565),
    [sym_get_last] = ACTIONS(1565),
    [sym_date_to_day] = ACTIONS(1565),
    [sym_date_to_month] = ACTIONS(1565),
    [sym_date_to_year] = ACTIONS(1565),
    [sym_time_to_hour] = ACTIONS(1565),
    [sym_time_to_minute] = ACTIONS(1565),
    [sym_time_to_second] = ACTIONS(1565),
    [sym_day_of_week] = ACTIONS(1565),
    [sym_set_pulse_time] = ACTIONS(1565),
    [sym_set_timer] = ACTIONS(1565),
    [sym_file_open] = ACTIONS(1565),
    [sym_file_close] = ACTIONS(1565),
    [sym_file_read] = ACTIONS(1565),
    [sym_file_read_line] = ACTIONS(1565),
    [sym_file_write] = ACTIONS(1565),
    [sym_file_write_line] = ACTIONS(1565),
    [sym_file_copy] = ACTIONS(1565),
    [sym_file_delete] = ACTIONS(1565),
    [sym_file_rename] = ACTIONS(1565),
    [sym_file_dir] = ACTIONS(1565),
    [sym_file_getdir] = ACTIONS(1565),
    [sym_file_setdir] = ACTIONS(1565),
    [sym_file_create_dir] = ACTIONS(1565),
    [sym_file_remove_dir] = ACTIONS(1565),
    [sym_file_seek] = ACTIONS(1565),
    [sym_length_array] = ACTIONS(1565),
    [sym_max_length_array] = ACTIONS(1565),
    [sym_set_length_array] = ACTIONS(1565),
    [sym_ip_client_open] = ACTIONS(1565),
    [sym_ip_client_close] = ACTIONS(1565),
    [sym_ip_server_open] = ACTIONS(1565),
    [sym_ip_server_close] = ACTIONS(1565),
    [sym_ip_bound_client_open] = ACTIONS(1565),
    [sym_ip_mc_server_open] = ACTIONS(1565),
    [sym_ip_set_option] = ACTIONS(1565),
    [sym_ssh_client_open] = ACTIONS(1565),
    [sym_ssh_client_close] = ACTIONS(1565),
    [sym_tls_client_open] = ACTIONS(1565),
    [sym_tls_client_close] = ACTIONS(1565),
    [sym_do_push] = ACTIONS(1565),
    [sym_do_push_timed] = ACTIONS(1565),
    [sym_do_release] = ACTIONS(1565),
    [sym_device_id] = ACTIONS(1565),
    [sym_device_info] = ACTIONS(1565),
    [sym_device_id_string] = ACTIONS(1565),
    [sym_device_standby] = ACTIONS(1565),
    [sym_device_wake] = ACTIONS(1565),
    [sym_timeline_create] = ACTIONS(1565),
    [sym_timeline_kill] = ACTIONS(1565),
    [sym_timeline_reload] = ACTIONS(1565),
    [sym_timeline_pause] = ACTIONS(1565),
    [sym_timeline_restart] = ACTIONS(1565),
    [sym_timeline_set] = ACTIONS(1565),
    [sym_timeline_get] = ACTIONS(1565),
    [sym_timeline_active] = ACTIONS(1565),
    [sym_timeline_dynamic_id] = ACTIONS(1565),
    [sym_variable_to_string] = ACTIONS(1565),
    [sym_string_to_variable] = ACTIONS(1565),
    [sym_length_variable_to_string] = ACTIONS(1565),
    [sym_type_cast] = ACTIONS(1565),
    [sym_variable_to_xml] = ACTIONS(1565),
    [sym_xml_to_variable] = ACTIONS(1565),
    [sym_length_variable_to_xml] = ACTIONS(1565),
    [sym_redirect_string] = ACTIONS(1565),
    [sym_random_number] = ACTIONS(1565),
    [sym_get_unique_id] = ACTIONS(1565),
    [sym_get_system_number] = ACTIONS(1565),
    [sym_set_system_number] = ACTIONS(1565),
    [sym_get_serial_number] = ACTIONS(1565),
    [sym_get_master_build] = ACTIONS(1565),
    [sym_set_validation_code] = ACTIONS(1565),
    [sym_reboot] = ACTIONS(1565),
    [sym_load_duet_module] = ACTIONS(1565),
    [sym_unload_duet_module] = ACTIONS(1565),
    [sym_duet_mem_size_set] = ACTIONS(1565),
    [sym_duet_mem_size_get] = ACTIONS(1565),
    [sym_dynamic_application_device] = ACTIONS(1565),
    [sym_dynamic_polled_port] = ACTIONS(1565),
    [sym_static_port_binding] = ACTIONS(1565),
    [sym_static_ip_binding] = ACTIONS(1565),
    [sym_get_dns_list] = ACTIONS(1565),
    [sym_set_dns_list] = ACTIONS(1565),
    [sym_get_ip_address] = ACTIONS(1565),
    [sym_set_ip_address] = ACTIONS(1565),
    [sym_get_url_list] = ACTIONS(1565),
    [sym_add_url_entry] = ACTIONS(1565),
    [sym_delete_url_entry] = ACTIONS(1565),
    [sym_abs_value] = ACTIONS(1565),
    [sym_max_value] = ACTIONS(1565),
    [sym_min_value] = ACTIONS(1565),
    [sym_exp_value] = ACTIONS(1565),
    [sym_log_value] = ACTIONS(1565),
    [sym_log10_value] = ACTIONS(1565),
    [sym_power_value] = ACTIONS(1565),
    [sym_sqrt_value] = ACTIONS(1565),
    [sym_raw_be] = ACTIONS(1565),
    [sym_raw_le] = ACTIONS(1565),
    [sym_internal_queue_size_set] = ACTIONS(1565),
    [sym_internal_queue_size_get] = ACTIONS(1565),
    [sym_internal_threshold_set] = ACTIONS(1565),
    [sym_internal_threshold_get] = ACTIONS(1565),
    [sym_rebuilt_event] = ACTIONS(1565),
    [sym_do_custom_event] = ACTIONS(1565),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1565),
    [sym_clkmgr_set_clk_source] = ACTIONS(1565),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1565),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1565),
    [sym_clkmgr_get_timezone] = ACTIONS(1565),
    [sym_clkmgr_set_timezone] = ACTIONS(1565),
    [sym_clkmgr_get_resync_period] = ACTIONS(1565),
    [sym_clkmgr_set_resync_period] = ACTIONS(1565),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1565),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1565),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1565),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1565),
    [sym_clkmgr_get_timeservers] = ACTIONS(1565),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1565),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1565),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1565),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1565),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1565),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1565),
    [sym_set_log_level] = ACTIONS(1565),
    [sym_get_log_level] = ACTIONS(1565),
    [sym_amx_log] = ACTIONS(1565),
    [sym_smtp_server_config_set] = ACTIONS(1565),
    [sym_smtp_server_config_get] = ACTIONS(1565),
    [sym_smtp_send] = ACTIONS(1565),
    [sym_validate_netlinx_account] = ACTIONS(1565),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1565),
    [sym_authenticate_certificate] = ACTIONS(1565),
    [sym_audit_netlinx_session_event] = ACTIONS(1565),
    [sym_audit_netlinx_generic_event] = ACTIONS(1565),
    [sym_get_available_flash_disk_space] = ACTIONS(1565),
    [sym_get_max_flash_disk_space] = ACTIONS(1565),
    [sym_data_create_feed] = ACTIONS(1565),
    [sym_data_delete_feed] = ACTIONS(1565),
    [sym_data_publish_feed] = ACTIONS(1565),
    [sym_data_get_published_feed] = ACTIONS(1565),
    [sym_data_add_record] = ACTIONS(1565),
    [sym_data_get_event_record] = ACTIONS(1565),
    [sym__wc_data_create_feed] = ACTIONS(1565),
    [sym__wc_data_add_record] = ACTIONS(1565),
    [sym__wc_data_get_event_record] = ACTIONS(1565),
    [sym_format] = ACTIONS(1565),
    [sym_astro_clock] = ACTIONS(1565),
    [sym_set_outdoor_temperature] = ACTIONS(1565),
    [sym_set_virtual_level_count] = ACTIONS(1565),
    [sym_set_virtual_channel_count] = ACTIONS(1565),
    [sym_set_virtual_port_count] = ACTIONS(1565),
    [sym__wc_encode_int] = ACTIONS(1565),
    [sym__wc_decode_int] = ACTIONS(1565),
    [sym__wc_int] = ACTIONS(1565),
    [sym__wc_to_ch_int] = ACTIONS(1565),
    [sym__ch_to_wc_int] = ACTIONS(1565),
    [sym__wc_find_string_int] = ACTIONS(1565),
    [sym__wc_left_string_int] = ACTIONS(1565),
    [sym__wc_lower_string_int] = ACTIONS(1565),
    [sym__wc_mid_string_int] = ACTIONS(1565),
    [sym__wc_remove_string_int] = ACTIONS(1565),
    [sym__wc_right_string_int] = ACTIONS(1565),
    [sym__wc_upper_string_int] = ACTIONS(1565),
    [sym__wc_compare_string_int] = ACTIONS(1565),
    [sym__wc_get_buffer_char_int] = ACTIONS(1565),
    [sym__wc_get_buffer_string_int] = ACTIONS(1565),
    [sym__wc_concat_string_int] = ACTIONS(1565),
    [sym__wc_file_open_int] = ACTIONS(1565),
    [sym__wc_file_close_int] = ACTIONS(1565),
    [sym__wc_file_read_int] = ACTIONS(1565),
    [sym__wc_file_read_line_int] = ACTIONS(1565),
    [sym__wc_file_write_int] = ACTIONS(1565),
    [sym__wc_file_write_line_int] = ACTIONS(1565),
    [aux_sym_preproc_include_token1] = ACTIONS(1565),
    [aux_sym_preproc_define_token1] = ACTIONS(1567),
    [aux_sym_preproc_warn_token1] = ACTIONS(1567),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1567),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1567),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1567),
    [anon_sym_LBRACK] = ACTIONS(1567),
    [anon_sym_LBRACE] = ACTIONS(1567),
    [anon_sym_RBRACE] = ACTIONS(1567),
    [anon_sym_COLON] = ACTIONS(1567),
    [anon_sym_LPAREN] = ACTIONS(1565),
    [anon_sym_SEMI] = ACTIONS(1567),
    [anon_sym_BANG] = ACTIONS(1565),
    [anon_sym_TILDE] = ACTIONS(1567),
    [anon_sym_DASH] = ACTIONS(1565),
    [anon_sym_PLUS] = ACTIONS(1565),
    [anon_sym_STAR] = ACTIONS(1567),
    [anon_sym_SLASH] = ACTIONS(1565),
    [anon_sym_PERCENT] = ACTIONS(1567),
    [anon_sym_PIPE_PIPE] = ACTIONS(1567),
    [anon_sym_AMP_AMP] = ACTIONS(1567),
    [anon_sym_PIPE] = ACTIONS(1565),
    [anon_sym_CARET] = ACTIONS(1567),
    [anon_sym_AMP] = ACTIONS(1565),
    [anon_sym_EQ_EQ] = ACTIONS(1567),
    [anon_sym_BANG_EQ] = ACTIONS(1567),
    [anon_sym_LT_GT] = ACTIONS(1567),
    [anon_sym_GT] = ACTIONS(1565),
    [anon_sym_GT_EQ] = ACTIONS(1567),
    [anon_sym_LT_EQ] = ACTIONS(1567),
    [anon_sym_LT] = ACTIONS(1565),
    [anon_sym_LT_LT] = ACTIONS(1567),
    [anon_sym_GT_GT] = ACTIONS(1567),
    [anon_sym_DASH_DASH] = ACTIONS(1567),
    [anon_sym_PLUS_PLUS] = ACTIONS(1567),
    [anon_sym_DOT] = ACTIONS(1565),
    [anon_sym_DQUOTE] = ACTIONS(1567),
    [anon_sym_SQUOTE] = ACTIONS(1567),
    [sym_number_literal] = ACTIONS(1567),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1567),
  },
  [STATE(191)] = {
    [sym_identifier] = ACTIONS(1593),
    [sym_define_call_keyword] = ACTIONS(1593),
    [sym_define_function_keyword] = ACTIONS(1593),
    [sym_define_library_function_keyword] = ACTIONS(1593),
    [sym_define_module_keyword] = ACTIONS(1593),
    [sym_char_keyword] = ACTIONS(1593),
    [sym_widechar_keyword] = ACTIONS(1593),
    [sym_integer_keyword] = ACTIONS(1593),
    [sym_sinteger_keyword] = ACTIONS(1593),
    [sym_long_keyword] = ACTIONS(1593),
    [sym_slong_keyword] = ACTIONS(1593),
    [sym_float_keyword] = ACTIONS(1593),
    [sym_double_keyword] = ACTIONS(1593),
    [sym_variant_keyword] = ACTIONS(1593),
    [sym_variantarray_keyword] = ACTIONS(1593),
    [sym_local_var_keyword] = ACTIONS(1593),
    [sym_stack_var_keyword] = ACTIONS(1593),
    [sym_constant_keyword] = ACTIONS(1593),
    [sym_volatile_keyword] = ACTIONS(1593),
    [sym_non_volatile_keyword] = ACTIONS(1593),
    [sym_persistent_keyword] = ACTIONS(1593),
    [sym_dev_keyword] = ACTIONS(1593),
    [sym_devlev_keyword] = ACTIONS(1593),
    [sym_devchan_keyword] = ACTIONS(1593),
    [sym_if_keyword] = ACTIONS(1593),
    [sym_else_keyword] = ACTIONS(1593),
    [sym_switch_keyword] = ACTIONS(1593),
    [sym_case_keyword] = ACTIONS(1593),
    [sym_default_keyword] = ACTIONS(1593),
    [sym_while_keyword] = ACTIONS(1593),
    [sym_for_keyword] = ACTIONS(1593),
    [sym_break_keyword] = ACTIONS(1593),
    [sym_continue_keyword] = ACTIONS(1593),
    [sym_return_keyword] = ACTIONS(1593),
    [sym_select_keyword] = ACTIONS(1593),
    [sym_active_keyword] = ACTIONS(1593),
    [sym_struct_keyword] = ACTIONS(1593),
    [sym_structure_keyword] = ACTIONS(1593),
    [aux_sym_on_keyword_token1] = ACTIONS(1593),
    [aux_sym_off_keyword_token1] = ACTIONS(1593),
    [sym_send_level_keyword] = ACTIONS(1593),
    [sym_send_string_keyword] = ACTIONS(1593),
    [sym_send_command_keyword] = ACTIONS(1593),
    [sym_clear_buffer_keyword] = ACTIONS(1593),
    [sym_create_buffer_keyword] = ACTIONS(1593),
    [sym_create_multi_buffer_keyword] = ACTIONS(1593),
    [sym_call_keyword] = ACTIONS(1593),
    [sym_system_call_keyword] = ACTIONS(1593),
    [sym_devchan_to_keyword] = ACTIONS(1593),
    [sym_devchan_min_to_keyword] = ACTIONS(1593),
    [sym_devchan_total_off_keyword] = ACTIONS(1593),
    [sym_devchan_pulse_keyword] = ACTIONS(1593),
    [sym_wait_keyword] = ACTIONS(1593),
    [sym_cancel_wait_keyword] = ACTIONS(1593),
    [sym_wait_until_keyword] = ACTIONS(1593),
    [sym_cancel_wait_until_keyword] = ACTIONS(1593),
    [sym_cancel_all_wait_keyword] = ACTIONS(1593),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1593),
    [sym_and] = ACTIONS(1593),
    [sym_or] = ACTIONS(1593),
    [sym_not] = ACTIONS(1593),
    [sym_xor] = ACTIONS(1593),
    [sym_band] = ACTIONS(1593),
    [sym_bor] = ACTIONS(1593),
    [sym_bxor] = ACTIONS(1593),
    [sym_bnot] = ACTIONS(1593),
    [sym_lshift] = ACTIONS(1593),
    [sym_rshift] = ACTIONS(1593),
    [sym___file__] = ACTIONS(1593),
    [sym___date__] = ACTIONS(1593),
    [sym___ldate__] = ACTIONS(1593),
    [sym___time__] = ACTIONS(1593),
    [sym___line__] = ACTIONS(1593),
    [sym___name__] = ACTIONS(1593),
    [sym___netlinx__] = ACTIONS(1593),
    [sym_netlinx_axi_version] = ACTIONS(1593),
    [sym_url_flg_tcp] = ACTIONS(1593),
    [sym_url_flg_accinfopresent] = ACTIONS(1593),
    [sym_url_flg_temp] = ACTIONS(1593),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1593),
    [sym_url_flg_stat_mask] = ACTIONS(1593),
    [sym_url_flg_stat_lookup] = ACTIONS(1593),
    [sym_url_flg_stat_connecting] = ACTIONS(1593),
    [sym_url_flg_stat_waiting] = ACTIONS(1593),
    [sym_url_flg_stat_connected] = ACTIONS(1593),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1593),
    [sym_first_virtual_device] = ACTIONS(1593),
    [sym_first_local_port] = ACTIONS(1593),
    [sym_dynamic_virtual_device] = ACTIONS(1593),
    [sym_timeline_once] = ACTIONS(1593),
    [sym_timeline_repeat] = ACTIONS(1593),
    [sym_timeline_absolute] = ACTIONS(1593),
    [sym_timeline_relative] = ACTIONS(1593),
    [sym_do_push_timed_infinite] = ACTIONS(1593),
    [sym_source_type_no_address] = ACTIONS(1593),
    [sym_source_type_neuron_id] = ACTIONS(1593),
    [sym_source_type_ip_address] = ACTIONS(1593),
    [sym_source_type_axlink] = ACTIONS(1593),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1593),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1593),
    [sym_source_type_ip_socket_address] = ACTIONS(1593),
    [sym_source_type_rs232] = ACTIONS(1593),
    [sym_source_type_internal] = ACTIONS(1593),
    [sym_source_type_mac_address] = ACTIONS(1593),
    [sym_source_type_ipv4_port] = ACTIONS(1593),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1593),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1593),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1593),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1593),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1593),
    [sym_true] = ACTIONS(1593),
    [sym_false] = ACTIONS(1593),
    [sym_file_read_only] = ACTIONS(1593),
    [sym_file_rw_new] = ACTIONS(1593),
    [sym_file_rw_append] = ACTIONS(1593),
    [sym_ip_tcp] = ACTIONS(1593),
    [sym_ip_udp] = ACTIONS(1593),
    [sym_ip_udp_2way] = ACTIONS(1593),
    [sym_xml_encode_types] = ACTIONS(1593),
    [sym_xml_encode_char_as_list] = ACTIONS(1593),
    [sym_xml_encode_le] = ACTIONS(1593),
    [sym_xml_decode_types] = ACTIONS(1593),
    [sym_xml_decode_no_preserve] = ACTIONS(1593),
    [sym_duet_dev_type_display_device] = ACTIONS(1593),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1593),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1593),
    [sym_duet_dev_type_io_device] = ACTIONS(1593),
    [sym_duet_dev_type_relay_device] = ACTIONS(1593),
    [sym_duet_dev_type_ups] = ACTIONS(1593),
    [sym_duet_dev_type_amplifier] = ACTIONS(1593),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1593),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1593),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1593),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1593),
    [sym_duet_dev_type_camera] = ACTIONS(1593),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1593),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1593),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1593),
    [sym_duet_dev_type_dss] = ACTIONS(1593),
    [sym_duet_dev_type_dvr] = ACTIONS(1593),
    [sym_duet_dev_type_disc_device] = ACTIONS(1593),
    [sym_duet_dev_type_document_camera] = ACTIONS(1593),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1593),
    [sym_duet_dev_type_hvac] = ACTIONS(1593),
    [sym_duet_dev_type_keypad] = ACTIONS(1593),
    [sym_duet_dev_type_light] = ACTIONS(1593),
    [sym_duet_dev_type_monitor] = ACTIONS(1593),
    [sym_duet_dev_type_motor] = ACTIONS(1593),
    [sym_duet_dev_type_multi_window] = ACTIONS(1593),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1593),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1593),
    [sym_duet_dev_type_receiver] = ACTIONS(1593),
    [sym_duet_dev_type_security_system] = ACTIONS(1593),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1593),
    [sym_duet_dev_type_settop_box] = ACTIONS(1593),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1593),
    [sym_duet_dev_type_switcher] = ACTIONS(1593),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1593),
    [sym_duet_dev_type_tv] = ACTIONS(1593),
    [sym_duet_dev_type_utility] = ACTIONS(1593),
    [sym_duet_dev_type_vcr] = ACTIONS(1593),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1593),
    [sym_duet_dev_type_video_processor] = ACTIONS(1593),
    [sym_duet_dev_type_video_projector] = ACTIONS(1593),
    [sym_duet_dev_type_video_wall] = ACTIONS(1593),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1593),
    [sym_duet_dev_type_weather] = ACTIONS(1593),
    [sym_duet_dev_not_polled] = ACTIONS(1593),
    [sym_duet_dev_polled] = ACTIONS(1593),
    [sym_clkmgr_mode_network] = ACTIONS(1593),
    [sym_clkmgr_mode_standalone] = ACTIONS(1593),
    [sym_normal_standby] = ACTIONS(1593),
    [sym_normal_wake] = ACTIONS(1593),
    [sym_ip_multicast_ttl_option] = ACTIONS(1593),
    [sym_ip_ttl_subnet] = ACTIONS(1593),
    [sym_ip_ttl_site] = ACTIONS(1593),
    [sym_ip_ttl_region] = ACTIONS(1593),
    [sym_ip_ttl_continent] = ACTIONS(1593),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1593),
    [sym_ip_nodelay_on] = ACTIONS(1593),
    [sym_ip_nodelay_off] = ACTIONS(1593),
    [sym_amx_error] = ACTIONS(1593),
    [sym_amx_warning] = ACTIONS(1593),
    [sym_amx_info] = ACTIONS(1593),
    [sym_amx_debug] = ACTIONS(1593),
    [sym_netlinx_login_success] = ACTIONS(1593),
    [sym_netlinx_login_fail] = ACTIONS(1593),
    [sym_netlinx_logout] = ACTIONS(1593),
    [sym_null_str] = ACTIONS(1593),
    [sym_smtp_address] = ACTIONS(1593),
    [sym_smtp_port_number] = ACTIONS(1593),
    [sym_smtp_username] = ACTIONS(1593),
    [sym_smtp_password] = ACTIONS(1593),
    [sym_smtp_require_tls] = ACTIONS(1593),
    [sym_smtp_from] = ACTIONS(1593),
    [sym_smtp_tls_true] = ACTIONS(1593),
    [sym_smtp_tls_false] = ACTIONS(1593),
    [sym_listview_on_row_select_event] = ACTIONS(1593),
    [sym_data_structure_unknown] = ACTIONS(1593),
    [sym_data_structure_datafeed] = ACTIONS(1593),
    [sym_data_structure_datarecordset] = ACTIONS(1593),
    [sym_data_structure_datarecord] = ACTIONS(1593),
    [sym_data_structure_datafield] = ACTIONS(1593),
    [sym_data_max_content_field_count] = ACTIONS(1593),
    [sym_data_max_metadata_field_count] = ACTIONS(1593),
    [sym_data_max_id_length] = ACTIONS(1593),
    [sym_data_max_name_length] = ACTIONS(1593),
    [sym_data_max_description_length] = ACTIONS(1593),
    [sym_data_max_type_length] = ACTIONS(1593),
    [sym_data_max_format_length] = ACTIONS(1593),
    [sym_data_max_source_length] = ACTIONS(1593),
    [sym_data_max_value_length] = ACTIONS(1593),
    [sym_data_max_label_length] = ACTIONS(1593),
    [sym_data_max_url_length] = ACTIONS(1593),
    [sym_max_last_login_info_length] = ACTIONS(1593),
    [sym_data_type_unknown] = ACTIONS(1593),
    [sym_data_type_string] = ACTIONS(1593),
    [sym_data_type_datetime] = ACTIONS(1593),
    [sym_data_type_date] = ACTIONS(1593),
    [sym_data_type_time] = ACTIONS(1593),
    [sym_data_type_image] = ACTIONS(1593),
    [sym_data_format_url] = ACTIONS(1593),
    [sym_data_format_phone] = ACTIONS(1593),
    [sym_data_format_email] = ACTIONS(1593),
    [sym_data_format_iso8601] = ACTIONS(1593),
    [sym_auditlog_priv] = ACTIONS(1593),
    [sym_configuration_priv] = ACTIONS(1593),
    [sym_deviceconfig_priv] = ACTIONS(1593),
    [sym_ftp_priv] = ACTIONS(1593),
    [sym_http_priv] = ACTIONS(1593),
    [sym_networkconfig_priv] = ACTIONS(1593),
    [sym_programport_priv] = ACTIONS(1593),
    [sym_remoteui_priv] = ACTIONS(1593),
    [sym_securitycontrol_priv] = ACTIONS(1593),
    [sym_softwaremanagement_priv] = ACTIONS(1593),
    [sym_terminal_priv] = ACTIONS(1593),
    [sym_tpadmin_priv] = ACTIONS(1593),
    [sym_user1_priv] = ACTIONS(1593),
    [sym_user2_priv] = ACTIONS(1593),
    [sym_user3_priv] = ACTIONS(1593),
    [sym_user4_priv] = ACTIONS(1593),
    [sym_usermanagement_priv] = ACTIONS(1593),
    [sym_valid_account] = ACTIONS(1593),
    [sym_err_name_invalid] = ACTIONS(1593),
    [sym_err_password_invalid] = ACTIONS(1593),
    [sym_err_invalid_account] = ACTIONS(1593),
    [sym_err_no_authorization] = ACTIONS(1593),
    [sym_err_invalid_parameter] = ACTIONS(1593),
    [sym_err_user_locked_out] = ACTIONS(1593),
    [sym_err_user_account_expired] = ACTIONS(1593),
    [sym_err_general_error] = ACTIONS(1593),
    [sym_err_authentication_error] = ACTIONS(1593),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1593),
    [sym_tls_validate_certificate] = ACTIONS(1593),
    [sym_tdata] = ACTIONS(1593),
    [sym_tchannel] = ACTIONS(1593),
    [sym_tlevel] = ACTIONS(1593),
    [sym_tbutton] = ACTIONS(1593),
    [sym_ttimeline] = ACTIONS(1593),
    [sym_tcustom] = ACTIONS(1593),
    [sym_url_struct] = ACTIONS(1593),
    [sym_dns_struct] = ACTIONS(1593),
    [sym_ip_address_struct] = ACTIONS(1593),
    [sym_dev_info_struct] = ACTIONS(1593),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1593),
    [sym_data_feed] = ACTIONS(1593),
    [sym_data_field] = ACTIONS(1593),
    [sym_data_record] = ACTIONS(1593),
    [sym_wc_data_feed] = ACTIONS(1593),
    [sym_wc_data_field] = ACTIONS(1593),
    [sym_wc_data_record] = ACTIONS(1593),
    [sym_last_login_info] = ACTIONS(1593),
    [sym____reserved___] = ACTIONS(1593),
    [sym_date] = ACTIONS(1593),
    [sym_day] = ACTIONS(1593),
    [sym_time] = ACTIONS(1593),
    [sym_ldate] = ACTIONS(1593),
    [sym_push_device] = ACTIONS(1593),
    [sym_push_channel] = ACTIONS(1593),
    [sym_push_devchan] = ACTIONS(1593),
    [sym_release_device] = ACTIONS(1593),
    [sym_release_channel] = ACTIONS(1593),
    [sym_release_devchan] = ACTIONS(1593),
    [sym_master_slot] = ACTIONS(1593),
    [sym_get_pulse_time] = ACTIONS(1593),
    [sym_get_timer] = ACTIONS(1593),
    [sym_master_sn] = ACTIONS(1593),
    [sym_system_number] = ACTIONS(1593),
    [sym_button] = ACTIONS(1593),
    [sym_dv_channel] = ACTIONS(1593),
    [sym_data] = ACTIONS(1593),
    [sym_level] = ACTIONS(1593),
    [sym_timeline] = ACTIONS(1593),
    [sym_channel] = ACTIONS(1593),
    [sym_custom] = ACTIONS(1593),
    [sym_atoi] = ACTIONS(1593),
    [sym_atol] = ACTIONS(1593),
    [sym_atof] = ACTIONS(1593),
    [sym_itoa] = ACTIONS(1593),
    [sym_itohex] = ACTIONS(1593),
    [sym_hextoi] = ACTIONS(1593),
    [sym_ftoa] = ACTIONS(1593),
    [sym_find_string] = ACTIONS(1593),
    [sym_left_string] = ACTIONS(1593),
    [sym_right_string] = ACTIONS(1593),
    [sym_mid_string] = ACTIONS(1593),
    [sym_remove_string] = ACTIONS(1593),
    [sym_get_buffer_string] = ACTIONS(1593),
    [sym_get_buffer_char] = ACTIONS(1593),
    [sym_get_multi_buffer_string] = ACTIONS(1593),
    [sym_length_string] = ACTIONS(1593),
    [sym_max_length_string] = ACTIONS(1593),
    [sym_set_length_string] = ACTIONS(1593),
    [sym_compare_string] = ACTIONS(1593),
    [sym_upper_string] = ACTIONS(1593),
    [sym_lower_string] = ACTIONS(1593),
    [sym_get_last] = ACTIONS(1593),
    [sym_date_to_day] = ACTIONS(1593),
    [sym_date_to_month] = ACTIONS(1593),
    [sym_date_to_year] = ACTIONS(1593),
    [sym_time_to_hour] = ACTIONS(1593),
    [sym_time_to_minute] = ACTIONS(1593),
    [sym_time_to_second] = ACTIONS(1593),
    [sym_day_of_week] = ACTIONS(1593),
    [sym_set_pulse_time] = ACTIONS(1593),
    [sym_set_timer] = ACTIONS(1593),
    [sym_file_open] = ACTIONS(1593),
    [sym_file_close] = ACTIONS(1593),
    [sym_file_read] = ACTIONS(1593),
    [sym_file_read_line] = ACTIONS(1593),
    [sym_file_write] = ACTIONS(1593),
    [sym_file_write_line] = ACTIONS(1593),
    [sym_file_copy] = ACTIONS(1593),
    [sym_file_delete] = ACTIONS(1593),
    [sym_file_rename] = ACTIONS(1593),
    [sym_file_dir] = ACTIONS(1593),
    [sym_file_getdir] = ACTIONS(1593),
    [sym_file_setdir] = ACTIONS(1593),
    [sym_file_create_dir] = ACTIONS(1593),
    [sym_file_remove_dir] = ACTIONS(1593),
    [sym_file_seek] = ACTIONS(1593),
    [sym_length_array] = ACTIONS(1593),
    [sym_max_length_array] = ACTIONS(1593),
    [sym_set_length_array] = ACTIONS(1593),
    [sym_ip_client_open] = ACTIONS(1593),
    [sym_ip_client_close] = ACTIONS(1593),
    [sym_ip_server_open] = ACTIONS(1593),
    [sym_ip_server_close] = ACTIONS(1593),
    [sym_ip_bound_client_open] = ACTIONS(1593),
    [sym_ip_mc_server_open] = ACTIONS(1593),
    [sym_ip_set_option] = ACTIONS(1593),
    [sym_ssh_client_open] = ACTIONS(1593),
    [sym_ssh_client_close] = ACTIONS(1593),
    [sym_tls_client_open] = ACTIONS(1593),
    [sym_tls_client_close] = ACTIONS(1593),
    [sym_do_push] = ACTIONS(1593),
    [sym_do_push_timed] = ACTIONS(1593),
    [sym_do_release] = ACTIONS(1593),
    [sym_device_id] = ACTIONS(1593),
    [sym_device_info] = ACTIONS(1593),
    [sym_device_id_string] = ACTIONS(1593),
    [sym_device_standby] = ACTIONS(1593),
    [sym_device_wake] = ACTIONS(1593),
    [sym_timeline_create] = ACTIONS(1593),
    [sym_timeline_kill] = ACTIONS(1593),
    [sym_timeline_reload] = ACTIONS(1593),
    [sym_timeline_pause] = ACTIONS(1593),
    [sym_timeline_restart] = ACTIONS(1593),
    [sym_timeline_set] = ACTIONS(1593),
    [sym_timeline_get] = ACTIONS(1593),
    [sym_timeline_active] = ACTIONS(1593),
    [sym_timeline_dynamic_id] = ACTIONS(1593),
    [sym_variable_to_string] = ACTIONS(1593),
    [sym_string_to_variable] = ACTIONS(1593),
    [sym_length_variable_to_string] = ACTIONS(1593),
    [sym_type_cast] = ACTIONS(1593),
    [sym_variable_to_xml] = ACTIONS(1593),
    [sym_xml_to_variable] = ACTIONS(1593),
    [sym_length_variable_to_xml] = ACTIONS(1593),
    [sym_redirect_string] = ACTIONS(1593),
    [sym_random_number] = ACTIONS(1593),
    [sym_get_unique_id] = ACTIONS(1593),
    [sym_get_system_number] = ACTIONS(1593),
    [sym_set_system_number] = ACTIONS(1593),
    [sym_get_serial_number] = ACTIONS(1593),
    [sym_get_master_build] = ACTIONS(1593),
    [sym_set_validation_code] = ACTIONS(1593),
    [sym_reboot] = ACTIONS(1593),
    [sym_load_duet_module] = ACTIONS(1593),
    [sym_unload_duet_module] = ACTIONS(1593),
    [sym_duet_mem_size_set] = ACTIONS(1593),
    [sym_duet_mem_size_get] = ACTIONS(1593),
    [sym_dynamic_application_device] = ACTIONS(1593),
    [sym_dynamic_polled_port] = ACTIONS(1593),
    [sym_static_port_binding] = ACTIONS(1593),
    [sym_static_ip_binding] = ACTIONS(1593),
    [sym_get_dns_list] = ACTIONS(1593),
    [sym_set_dns_list] = ACTIONS(1593),
    [sym_get_ip_address] = ACTIONS(1593),
    [sym_set_ip_address] = ACTIONS(1593),
    [sym_get_url_list] = ACTIONS(1593),
    [sym_add_url_entry] = ACTIONS(1593),
    [sym_delete_url_entry] = ACTIONS(1593),
    [sym_abs_value] = ACTIONS(1593),
    [sym_max_value] = ACTIONS(1593),
    [sym_min_value] = ACTIONS(1593),
    [sym_exp_value] = ACTIONS(1593),
    [sym_log_value] = ACTIONS(1593),
    [sym_log10_value] = ACTIONS(1593),
    [sym_power_value] = ACTIONS(1593),
    [sym_sqrt_value] = ACTIONS(1593),
    [sym_raw_be] = ACTIONS(1593),
    [sym_raw_le] = ACTIONS(1593),
    [sym_internal_queue_size_set] = ACTIONS(1593),
    [sym_internal_queue_size_get] = ACTIONS(1593),
    [sym_internal_threshold_set] = ACTIONS(1593),
    [sym_internal_threshold_get] = ACTIONS(1593),
    [sym_rebuilt_event] = ACTIONS(1593),
    [sym_do_custom_event] = ACTIONS(1593),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1593),
    [sym_clkmgr_set_clk_source] = ACTIONS(1593),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1593),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1593),
    [sym_clkmgr_get_timezone] = ACTIONS(1593),
    [sym_clkmgr_set_timezone] = ACTIONS(1593),
    [sym_clkmgr_get_resync_period] = ACTIONS(1593),
    [sym_clkmgr_set_resync_period] = ACTIONS(1593),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1593),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1593),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1593),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1593),
    [sym_clkmgr_get_timeservers] = ACTIONS(1593),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1593),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1593),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1593),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1593),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1593),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1593),
    [sym_set_log_level] = ACTIONS(1593),
    [sym_get_log_level] = ACTIONS(1593),
    [sym_amx_log] = ACTIONS(1593),
    [sym_smtp_server_config_set] = ACTIONS(1593),
    [sym_smtp_server_config_get] = ACTIONS(1593),
    [sym_smtp_send] = ACTIONS(1593),
    [sym_validate_netlinx_account] = ACTIONS(1593),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1593),
    [sym_authenticate_certificate] = ACTIONS(1593),
    [sym_audit_netlinx_session_event] = ACTIONS(1593),
    [sym_audit_netlinx_generic_event] = ACTIONS(1593),
    [sym_get_available_flash_disk_space] = ACTIONS(1593),
    [sym_get_max_flash_disk_space] = ACTIONS(1593),
    [sym_data_create_feed] = ACTIONS(1593),
    [sym_data_delete_feed] = ACTIONS(1593),
    [sym_data_publish_feed] = ACTIONS(1593),
    [sym_data_get_published_feed] = ACTIONS(1593),
    [sym_data_add_record] = ACTIONS(1593),
    [sym_data_get_event_record] = ACTIONS(1593),
    [sym__wc_data_create_feed] = ACTIONS(1593),
    [sym__wc_data_add_record] = ACTIONS(1593),
    [sym__wc_data_get_event_record] = ACTIONS(1593),
    [sym_format] = ACTIONS(1593),
    [sym_astro_clock] = ACTIONS(1593),
    [sym_set_outdoor_temperature] = ACTIONS(1593),
    [sym_set_virtual_level_count] = ACTIONS(1593),
    [sym_set_virtual_channel_count] = ACTIONS(1593),
    [sym_set_virtual_port_count] = ACTIONS(1593),
    [sym__wc_encode_int] = ACTIONS(1593),
    [sym__wc_decode_int] = ACTIONS(1593),
    [sym__wc_int] = ACTIONS(1593),
    [sym__wc_to_ch_int] = ACTIONS(1593),
    [sym__ch_to_wc_int] = ACTIONS(1593),
    [sym__wc_find_string_int] = ACTIONS(1593),
    [sym__wc_left_string_int] = ACTIONS(1593),
    [sym__wc_lower_string_int] = ACTIONS(1593),
    [sym__wc_mid_string_int] = ACTIONS(1593),
    [sym__wc_remove_string_int] = ACTIONS(1593),
    [sym__wc_right_string_int] = ACTIONS(1593),
    [sym__wc_upper_string_int] = ACTIONS(1593),
    [sym__wc_compare_string_int] = ACTIONS(1593),
    [sym__wc_get_buffer_char_int] = ACTIONS(1593),
    [sym__wc_get_buffer_string_int] = ACTIONS(1593),
    [sym__wc_concat_string_int] = ACTIONS(1593),
    [sym__wc_file_open_int] = ACTIONS(1593),
    [sym__wc_file_close_int] = ACTIONS(1593),
    [sym__wc_file_read_int] = ACTIONS(1593),
    [sym__wc_file_read_line_int] = ACTIONS(1593),
    [sym__wc_file_write_int] = ACTIONS(1593),
    [sym__wc_file_write_line_int] = ACTIONS(1593),
    [aux_sym_preproc_include_token1] = ACTIONS(1593),
    [aux_sym_preproc_define_token1] = ACTIONS(1595),
    [aux_sym_preproc_warn_token1] = ACTIONS(1595),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1595),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1595),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1595),
    [anon_sym_LBRACK] = ACTIONS(1595),
    [anon_sym_LBRACE] = ACTIONS(1595),
    [anon_sym_RBRACE] = ACTIONS(1595),
    [anon_sym_COLON] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1593),
    [anon_sym_SEMI] = ACTIONS(1595),
    [anon_sym_BANG] = ACTIONS(1593),
    [anon_sym_TILDE] = ACTIONS(1595),
    [anon_sym_DASH] = ACTIONS(1593),
    [anon_sym_PLUS] = ACTIONS(1593),
    [anon_sym_STAR] = ACTIONS(1595),
    [anon_sym_SLASH] = ACTIONS(1593),
    [anon_sym_PERCENT] = ACTIONS(1595),
    [anon_sym_PIPE_PIPE] = ACTIONS(1595),
    [anon_sym_AMP_AMP] = ACTIONS(1595),
    [anon_sym_PIPE] = ACTIONS(1593),
    [anon_sym_CARET] = ACTIONS(1595),
    [anon_sym_AMP] = ACTIONS(1593),
    [anon_sym_EQ_EQ] = ACTIONS(1595),
    [anon_sym_BANG_EQ] = ACTIONS(1595),
    [anon_sym_LT_GT] = ACTIONS(1595),
    [anon_sym_GT] = ACTIONS(1593),
    [anon_sym_GT_EQ] = ACTIONS(1595),
    [anon_sym_LT_EQ] = ACTIONS(1595),
    [anon_sym_LT] = ACTIONS(1593),
    [anon_sym_LT_LT] = ACTIONS(1595),
    [anon_sym_GT_GT] = ACTIONS(1595),
    [anon_sym_DASH_DASH] = ACTIONS(1595),
    [anon_sym_PLUS_PLUS] = ACTIONS(1595),
    [anon_sym_DOT] = ACTIONS(1593),
    [anon_sym_DQUOTE] = ACTIONS(1595),
    [anon_sym_SQUOTE] = ACTIONS(1595),
    [sym_number_literal] = ACTIONS(1595),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1595),
  },
  [STATE(192)] = {
    [sym_identifier] = ACTIONS(1601),
    [sym_define_call_keyword] = ACTIONS(1601),
    [sym_define_function_keyword] = ACTIONS(1601),
    [sym_define_library_function_keyword] = ACTIONS(1601),
    [sym_define_module_keyword] = ACTIONS(1601),
    [sym_char_keyword] = ACTIONS(1601),
    [sym_widechar_keyword] = ACTIONS(1601),
    [sym_integer_keyword] = ACTIONS(1601),
    [sym_sinteger_keyword] = ACTIONS(1601),
    [sym_long_keyword] = ACTIONS(1601),
    [sym_slong_keyword] = ACTIONS(1601),
    [sym_float_keyword] = ACTIONS(1601),
    [sym_double_keyword] = ACTIONS(1601),
    [sym_variant_keyword] = ACTIONS(1601),
    [sym_variantarray_keyword] = ACTIONS(1601),
    [sym_local_var_keyword] = ACTIONS(1601),
    [sym_stack_var_keyword] = ACTIONS(1601),
    [sym_constant_keyword] = ACTIONS(1601),
    [sym_volatile_keyword] = ACTIONS(1601),
    [sym_non_volatile_keyword] = ACTIONS(1601),
    [sym_persistent_keyword] = ACTIONS(1601),
    [sym_dev_keyword] = ACTIONS(1601),
    [sym_devlev_keyword] = ACTIONS(1601),
    [sym_devchan_keyword] = ACTIONS(1601),
    [sym_if_keyword] = ACTIONS(1601),
    [sym_else_keyword] = ACTIONS(1601),
    [sym_switch_keyword] = ACTIONS(1601),
    [sym_case_keyword] = ACTIONS(1601),
    [sym_default_keyword] = ACTIONS(1601),
    [sym_while_keyword] = ACTIONS(1601),
    [sym_for_keyword] = ACTIONS(1601),
    [sym_break_keyword] = ACTIONS(1601),
    [sym_continue_keyword] = ACTIONS(1601),
    [sym_return_keyword] = ACTIONS(1601),
    [sym_select_keyword] = ACTIONS(1601),
    [sym_active_keyword] = ACTIONS(1601),
    [sym_struct_keyword] = ACTIONS(1601),
    [sym_structure_keyword] = ACTIONS(1601),
    [aux_sym_on_keyword_token1] = ACTIONS(1601),
    [aux_sym_off_keyword_token1] = ACTIONS(1601),
    [sym_send_level_keyword] = ACTIONS(1601),
    [sym_send_string_keyword] = ACTIONS(1601),
    [sym_send_command_keyword] = ACTIONS(1601),
    [sym_clear_buffer_keyword] = ACTIONS(1601),
    [sym_create_buffer_keyword] = ACTIONS(1601),
    [sym_create_multi_buffer_keyword] = ACTIONS(1601),
    [sym_call_keyword] = ACTIONS(1601),
    [sym_system_call_keyword] = ACTIONS(1601),
    [sym_devchan_to_keyword] = ACTIONS(1601),
    [sym_devchan_min_to_keyword] = ACTIONS(1601),
    [sym_devchan_total_off_keyword] = ACTIONS(1601),
    [sym_devchan_pulse_keyword] = ACTIONS(1601),
    [sym_wait_keyword] = ACTIONS(1601),
    [sym_cancel_wait_keyword] = ACTIONS(1601),
    [sym_wait_until_keyword] = ACTIONS(1601),
    [sym_cancel_wait_until_keyword] = ACTIONS(1601),
    [sym_cancel_all_wait_keyword] = ACTIONS(1601),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1601),
    [sym_and] = ACTIONS(1601),
    [sym_or] = ACTIONS(1601),
    [sym_not] = ACTIONS(1601),
    [sym_xor] = ACTIONS(1601),
    [sym_band] = ACTIONS(1601),
    [sym_bor] = ACTIONS(1601),
    [sym_bxor] = ACTIONS(1601),
    [sym_bnot] = ACTIONS(1601),
    [sym_lshift] = ACTIONS(1601),
    [sym_rshift] = ACTIONS(1601),
    [sym___file__] = ACTIONS(1601),
    [sym___date__] = ACTIONS(1601),
    [sym___ldate__] = ACTIONS(1601),
    [sym___time__] = ACTIONS(1601),
    [sym___line__] = ACTIONS(1601),
    [sym___name__] = ACTIONS(1601),
    [sym___netlinx__] = ACTIONS(1601),
    [sym_netlinx_axi_version] = ACTIONS(1601),
    [sym_url_flg_tcp] = ACTIONS(1601),
    [sym_url_flg_accinfopresent] = ACTIONS(1601),
    [sym_url_flg_temp] = ACTIONS(1601),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1601),
    [sym_url_flg_stat_mask] = ACTIONS(1601),
    [sym_url_flg_stat_lookup] = ACTIONS(1601),
    [sym_url_flg_stat_connecting] = ACTIONS(1601),
    [sym_url_flg_stat_waiting] = ACTIONS(1601),
    [sym_url_flg_stat_connected] = ACTIONS(1601),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1601),
    [sym_first_virtual_device] = ACTIONS(1601),
    [sym_first_local_port] = ACTIONS(1601),
    [sym_dynamic_virtual_device] = ACTIONS(1601),
    [sym_timeline_once] = ACTIONS(1601),
    [sym_timeline_repeat] = ACTIONS(1601),
    [sym_timeline_absolute] = ACTIONS(1601),
    [sym_timeline_relative] = ACTIONS(1601),
    [sym_do_push_timed_infinite] = ACTIONS(1601),
    [sym_source_type_no_address] = ACTIONS(1601),
    [sym_source_type_neuron_id] = ACTIONS(1601),
    [sym_source_type_ip_address] = ACTIONS(1601),
    [sym_source_type_axlink] = ACTIONS(1601),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1601),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1601),
    [sym_source_type_ip_socket_address] = ACTIONS(1601),
    [sym_source_type_rs232] = ACTIONS(1601),
    [sym_source_type_internal] = ACTIONS(1601),
    [sym_source_type_mac_address] = ACTIONS(1601),
    [sym_source_type_ipv4_port] = ACTIONS(1601),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1601),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1601),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1601),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1601),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1601),
    [sym_true] = ACTIONS(1601),
    [sym_false] = ACTIONS(1601),
    [sym_file_read_only] = ACTIONS(1601),
    [sym_file_rw_new] = ACTIONS(1601),
    [sym_file_rw_append] = ACTIONS(1601),
    [sym_ip_tcp] = ACTIONS(1601),
    [sym_ip_udp] = ACTIONS(1601),
    [sym_ip_udp_2way] = ACTIONS(1601),
    [sym_xml_encode_types] = ACTIONS(1601),
    [sym_xml_encode_char_as_list] = ACTIONS(1601),
    [sym_xml_encode_le] = ACTIONS(1601),
    [sym_xml_decode_types] = ACTIONS(1601),
    [sym_xml_decode_no_preserve] = ACTIONS(1601),
    [sym_duet_dev_type_display_device] = ACTIONS(1601),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1601),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1601),
    [sym_duet_dev_type_io_device] = ACTIONS(1601),
    [sym_duet_dev_type_relay_device] = ACTIONS(1601),
    [sym_duet_dev_type_ups] = ACTIONS(1601),
    [sym_duet_dev_type_amplifier] = ACTIONS(1601),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1601),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1601),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1601),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1601),
    [sym_duet_dev_type_camera] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1601),
    [sym_duet_dev_type_dss] = ACTIONS(1601),
    [sym_duet_dev_type_dvr] = ACTIONS(1601),
    [sym_duet_dev_type_disc_device] = ACTIONS(1601),
    [sym_duet_dev_type_document_camera] = ACTIONS(1601),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1601),
    [sym_duet_dev_type_hvac] = ACTIONS(1601),
    [sym_duet_dev_type_keypad] = ACTIONS(1601),
    [sym_duet_dev_type_light] = ACTIONS(1601),
    [sym_duet_dev_type_monitor] = ACTIONS(1601),
    [sym_duet_dev_type_motor] = ACTIONS(1601),
    [sym_duet_dev_type_multi_window] = ACTIONS(1601),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1601),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1601),
    [sym_duet_dev_type_receiver] = ACTIONS(1601),
    [sym_duet_dev_type_security_system] = ACTIONS(1601),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1601),
    [sym_duet_dev_type_settop_box] = ACTIONS(1601),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1601),
    [sym_duet_dev_type_switcher] = ACTIONS(1601),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1601),
    [sym_duet_dev_type_tv] = ACTIONS(1601),
    [sym_duet_dev_type_utility] = ACTIONS(1601),
    [sym_duet_dev_type_vcr] = ACTIONS(1601),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1601),
    [sym_duet_dev_type_video_processor] = ACTIONS(1601),
    [sym_duet_dev_type_video_projector] = ACTIONS(1601),
    [sym_duet_dev_type_video_wall] = ACTIONS(1601),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1601),
    [sym_duet_dev_type_weather] = ACTIONS(1601),
    [sym_duet_dev_not_polled] = ACTIONS(1601),
    [sym_duet_dev_polled] = ACTIONS(1601),
    [sym_clkmgr_mode_network] = ACTIONS(1601),
    [sym_clkmgr_mode_standalone] = ACTIONS(1601),
    [sym_normal_standby] = ACTIONS(1601),
    [sym_normal_wake] = ACTIONS(1601),
    [sym_ip_multicast_ttl_option] = ACTIONS(1601),
    [sym_ip_ttl_subnet] = ACTIONS(1601),
    [sym_ip_ttl_site] = ACTIONS(1601),
    [sym_ip_ttl_region] = ACTIONS(1601),
    [sym_ip_ttl_continent] = ACTIONS(1601),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1601),
    [sym_ip_nodelay_on] = ACTIONS(1601),
    [sym_ip_nodelay_off] = ACTIONS(1601),
    [sym_amx_error] = ACTIONS(1601),
    [sym_amx_warning] = ACTIONS(1601),
    [sym_amx_info] = ACTIONS(1601),
    [sym_amx_debug] = ACTIONS(1601),
    [sym_netlinx_login_success] = ACTIONS(1601),
    [sym_netlinx_login_fail] = ACTIONS(1601),
    [sym_netlinx_logout] = ACTIONS(1601),
    [sym_null_str] = ACTIONS(1601),
    [sym_smtp_address] = ACTIONS(1601),
    [sym_smtp_port_number] = ACTIONS(1601),
    [sym_smtp_username] = ACTIONS(1601),
    [sym_smtp_password] = ACTIONS(1601),
    [sym_smtp_require_tls] = ACTIONS(1601),
    [sym_smtp_from] = ACTIONS(1601),
    [sym_smtp_tls_true] = ACTIONS(1601),
    [sym_smtp_tls_false] = ACTIONS(1601),
    [sym_listview_on_row_select_event] = ACTIONS(1601),
    [sym_data_structure_unknown] = ACTIONS(1601),
    [sym_data_structure_datafeed] = ACTIONS(1601),
    [sym_data_structure_datarecordset] = ACTIONS(1601),
    [sym_data_structure_datarecord] = ACTIONS(1601),
    [sym_data_structure_datafield] = ACTIONS(1601),
    [sym_data_max_content_field_count] = ACTIONS(1601),
    [sym_data_max_metadata_field_count] = ACTIONS(1601),
    [sym_data_max_id_length] = ACTIONS(1601),
    [sym_data_max_name_length] = ACTIONS(1601),
    [sym_data_max_description_length] = ACTIONS(1601),
    [sym_data_max_type_length] = ACTIONS(1601),
    [sym_data_max_format_length] = ACTIONS(1601),
    [sym_data_max_source_length] = ACTIONS(1601),
    [sym_data_max_value_length] = ACTIONS(1601),
    [sym_data_max_label_length] = ACTIONS(1601),
    [sym_data_max_url_length] = ACTIONS(1601),
    [sym_max_last_login_info_length] = ACTIONS(1601),
    [sym_data_type_unknown] = ACTIONS(1601),
    [sym_data_type_string] = ACTIONS(1601),
    [sym_data_type_datetime] = ACTIONS(1601),
    [sym_data_type_date] = ACTIONS(1601),
    [sym_data_type_time] = ACTIONS(1601),
    [sym_data_type_image] = ACTIONS(1601),
    [sym_data_format_url] = ACTIONS(1601),
    [sym_data_format_phone] = ACTIONS(1601),
    [sym_data_format_email] = ACTIONS(1601),
    [sym_data_format_iso8601] = ACTIONS(1601),
    [sym_auditlog_priv] = ACTIONS(1601),
    [sym_configuration_priv] = ACTIONS(1601),
    [sym_deviceconfig_priv] = ACTIONS(1601),
    [sym_ftp_priv] = ACTIONS(1601),
    [sym_http_priv] = ACTIONS(1601),
    [sym_networkconfig_priv] = ACTIONS(1601),
    [sym_programport_priv] = ACTIONS(1601),
    [sym_remoteui_priv] = ACTIONS(1601),
    [sym_securitycontrol_priv] = ACTIONS(1601),
    [sym_softwaremanagement_priv] = ACTIONS(1601),
    [sym_terminal_priv] = ACTIONS(1601),
    [sym_tpadmin_priv] = ACTIONS(1601),
    [sym_user1_priv] = ACTIONS(1601),
    [sym_user2_priv] = ACTIONS(1601),
    [sym_user3_priv] = ACTIONS(1601),
    [sym_user4_priv] = ACTIONS(1601),
    [sym_usermanagement_priv] = ACTIONS(1601),
    [sym_valid_account] = ACTIONS(1601),
    [sym_err_name_invalid] = ACTIONS(1601),
    [sym_err_password_invalid] = ACTIONS(1601),
    [sym_err_invalid_account] = ACTIONS(1601),
    [sym_err_no_authorization] = ACTIONS(1601),
    [sym_err_invalid_parameter] = ACTIONS(1601),
    [sym_err_user_locked_out] = ACTIONS(1601),
    [sym_err_user_account_expired] = ACTIONS(1601),
    [sym_err_general_error] = ACTIONS(1601),
    [sym_err_authentication_error] = ACTIONS(1601),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1601),
    [sym_tls_validate_certificate] = ACTIONS(1601),
    [sym_tdata] = ACTIONS(1601),
    [sym_tchannel] = ACTIONS(1601),
    [sym_tlevel] = ACTIONS(1601),
    [sym_tbutton] = ACTIONS(1601),
    [sym_ttimeline] = ACTIONS(1601),
    [sym_tcustom] = ACTIONS(1601),
    [sym_url_struct] = ACTIONS(1601),
    [sym_dns_struct] = ACTIONS(1601),
    [sym_ip_address_struct] = ACTIONS(1601),
    [sym_dev_info_struct] = ACTIONS(1601),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1601),
    [sym_data_feed] = ACTIONS(1601),
    [sym_data_field] = ACTIONS(1601),
    [sym_data_record] = ACTIONS(1601),
    [sym_wc_data_feed] = ACTIONS(1601),
    [sym_wc_data_field] = ACTIONS(1601),
    [sym_wc_data_record] = ACTIONS(1601),
    [sym_last_login_info] = ACTIONS(1601),
    [sym____reserved___] = ACTIONS(1601),
    [sym_date] = ACTIONS(1601),
    [sym_day] = ACTIONS(1601),
    [sym_time] = ACTIONS(1601),
    [sym_ldate] = ACTIONS(1601),
    [sym_push_device] = ACTIONS(1601),
    [sym_push_channel] = ACTIONS(1601),
    [sym_push_devchan] = ACTIONS(1601),
    [sym_release_device] = ACTIONS(1601),
    [sym_release_channel] = ACTIONS(1601),
    [sym_release_devchan] = ACTIONS(1601),
    [sym_master_slot] = ACTIONS(1601),
    [sym_get_pulse_time] = ACTIONS(1601),
    [sym_get_timer] = ACTIONS(1601),
    [sym_master_sn] = ACTIONS(1601),
    [sym_system_number] = ACTIONS(1601),
    [sym_button] = ACTIONS(1601),
    [sym_dv_channel] = ACTIONS(1601),
    [sym_data] = ACTIONS(1601),
    [sym_level] = ACTIONS(1601),
    [sym_timeline] = ACTIONS(1601),
    [sym_channel] = ACTIONS(1601),
    [sym_custom] = ACTIONS(1601),
    [sym_atoi] = ACTIONS(1601),
    [sym_atol] = ACTIONS(1601),
    [sym_atof] = ACTIONS(1601),
    [sym_itoa] = ACTIONS(1601),
    [sym_itohex] = ACTIONS(1601),
    [sym_hextoi] = ACTIONS(1601),
    [sym_ftoa] = ACTIONS(1601),
    [sym_find_string] = ACTIONS(1601),
    [sym_left_string] = ACTIONS(1601),
    [sym_right_string] = ACTIONS(1601),
    [sym_mid_string] = ACTIONS(1601),
    [sym_remove_string] = ACTIONS(1601),
    [sym_get_buffer_string] = ACTIONS(1601),
    [sym_get_buffer_char] = ACTIONS(1601),
    [sym_get_multi_buffer_string] = ACTIONS(1601),
    [sym_length_string] = ACTIONS(1601),
    [sym_max_length_string] = ACTIONS(1601),
    [sym_set_length_string] = ACTIONS(1601),
    [sym_compare_string] = ACTIONS(1601),
    [sym_upper_string] = ACTIONS(1601),
    [sym_lower_string] = ACTIONS(1601),
    [sym_get_last] = ACTIONS(1601),
    [sym_date_to_day] = ACTIONS(1601),
    [sym_date_to_month] = ACTIONS(1601),
    [sym_date_to_year] = ACTIONS(1601),
    [sym_time_to_hour] = ACTIONS(1601),
    [sym_time_to_minute] = ACTIONS(1601),
    [sym_time_to_second] = ACTIONS(1601),
    [sym_day_of_week] = ACTIONS(1601),
    [sym_set_pulse_time] = ACTIONS(1601),
    [sym_set_timer] = ACTIONS(1601),
    [sym_file_open] = ACTIONS(1601),
    [sym_file_close] = ACTIONS(1601),
    [sym_file_read] = ACTIONS(1601),
    [sym_file_read_line] = ACTIONS(1601),
    [sym_file_write] = ACTIONS(1601),
    [sym_file_write_line] = ACTIONS(1601),
    [sym_file_copy] = ACTIONS(1601),
    [sym_file_delete] = ACTIONS(1601),
    [sym_file_rename] = ACTIONS(1601),
    [sym_file_dir] = ACTIONS(1601),
    [sym_file_getdir] = ACTIONS(1601),
    [sym_file_setdir] = ACTIONS(1601),
    [sym_file_create_dir] = ACTIONS(1601),
    [sym_file_remove_dir] = ACTIONS(1601),
    [sym_file_seek] = ACTIONS(1601),
    [sym_length_array] = ACTIONS(1601),
    [sym_max_length_array] = ACTIONS(1601),
    [sym_set_length_array] = ACTIONS(1601),
    [sym_ip_client_open] = ACTIONS(1601),
    [sym_ip_client_close] = ACTIONS(1601),
    [sym_ip_server_open] = ACTIONS(1601),
    [sym_ip_server_close] = ACTIONS(1601),
    [sym_ip_bound_client_open] = ACTIONS(1601),
    [sym_ip_mc_server_open] = ACTIONS(1601),
    [sym_ip_set_option] = ACTIONS(1601),
    [sym_ssh_client_open] = ACTIONS(1601),
    [sym_ssh_client_close] = ACTIONS(1601),
    [sym_tls_client_open] = ACTIONS(1601),
    [sym_tls_client_close] = ACTIONS(1601),
    [sym_do_push] = ACTIONS(1601),
    [sym_do_push_timed] = ACTIONS(1601),
    [sym_do_release] = ACTIONS(1601),
    [sym_device_id] = ACTIONS(1601),
    [sym_device_info] = ACTIONS(1601),
    [sym_device_id_string] = ACTIONS(1601),
    [sym_device_standby] = ACTIONS(1601),
    [sym_device_wake] = ACTIONS(1601),
    [sym_timeline_create] = ACTIONS(1601),
    [sym_timeline_kill] = ACTIONS(1601),
    [sym_timeline_reload] = ACTIONS(1601),
    [sym_timeline_pause] = ACTIONS(1601),
    [sym_timeline_restart] = ACTIONS(1601),
    [sym_timeline_set] = ACTIONS(1601),
    [sym_timeline_get] = ACTIONS(1601),
    [sym_timeline_active] = ACTIONS(1601),
    [sym_timeline_dynamic_id] = ACTIONS(1601),
    [sym_variable_to_string] = ACTIONS(1601),
    [sym_string_to_variable] = ACTIONS(1601),
    [sym_length_variable_to_string] = ACTIONS(1601),
    [sym_type_cast] = ACTIONS(1601),
    [sym_variable_to_xml] = ACTIONS(1601),
    [sym_xml_to_variable] = ACTIONS(1601),
    [sym_length_variable_to_xml] = ACTIONS(1601),
    [sym_redirect_string] = ACTIONS(1601),
    [sym_random_number] = ACTIONS(1601),
    [sym_get_unique_id] = ACTIONS(1601),
    [sym_get_system_number] = ACTIONS(1601),
    [sym_set_system_number] = ACTIONS(1601),
    [sym_get_serial_number] = ACTIONS(1601),
    [sym_get_master_build] = ACTIONS(1601),
    [sym_set_validation_code] = ACTIONS(1601),
    [sym_reboot] = ACTIONS(1601),
    [sym_load_duet_module] = ACTIONS(1601),
    [sym_unload_duet_module] = ACTIONS(1601),
    [sym_duet_mem_size_set] = ACTIONS(1601),
    [sym_duet_mem_size_get] = ACTIONS(1601),
    [sym_dynamic_application_device] = ACTIONS(1601),
    [sym_dynamic_polled_port] = ACTIONS(1601),
    [sym_static_port_binding] = ACTIONS(1601),
    [sym_static_ip_binding] = ACTIONS(1601),
    [sym_get_dns_list] = ACTIONS(1601),
    [sym_set_dns_list] = ACTIONS(1601),
    [sym_get_ip_address] = ACTIONS(1601),
    [sym_set_ip_address] = ACTIONS(1601),
    [sym_get_url_list] = ACTIONS(1601),
    [sym_add_url_entry] = ACTIONS(1601),
    [sym_delete_url_entry] = ACTIONS(1601),
    [sym_abs_value] = ACTIONS(1601),
    [sym_max_value] = ACTIONS(1601),
    [sym_min_value] = ACTIONS(1601),
    [sym_exp_value] = ACTIONS(1601),
    [sym_log_value] = ACTIONS(1601),
    [sym_log10_value] = ACTIONS(1601),
    [sym_power_value] = ACTIONS(1601),
    [sym_sqrt_value] = ACTIONS(1601),
    [sym_raw_be] = ACTIONS(1601),
    [sym_raw_le] = ACTIONS(1601),
    [sym_internal_queue_size_set] = ACTIONS(1601),
    [sym_internal_queue_size_get] = ACTIONS(1601),
    [sym_internal_threshold_set] = ACTIONS(1601),
    [sym_internal_threshold_get] = ACTIONS(1601),
    [sym_rebuilt_event] = ACTIONS(1601),
    [sym_do_custom_event] = ACTIONS(1601),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1601),
    [sym_clkmgr_set_clk_source] = ACTIONS(1601),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1601),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1601),
    [sym_clkmgr_get_timezone] = ACTIONS(1601),
    [sym_clkmgr_set_timezone] = ACTIONS(1601),
    [sym_clkmgr_get_resync_period] = ACTIONS(1601),
    [sym_clkmgr_set_resync_period] = ACTIONS(1601),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1601),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1601),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1601),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1601),
    [sym_clkmgr_get_timeservers] = ACTIONS(1601),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1601),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1601),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1601),
    [sym_set_log_level] = ACTIONS(1601),
    [sym_get_log_level] = ACTIONS(1601),
    [sym_amx_log] = ACTIONS(1601),
    [sym_smtp_server_config_set] = ACTIONS(1601),
    [sym_smtp_server_config_get] = ACTIONS(1601),
    [sym_smtp_send] = ACTIONS(1601),
    [sym_validate_netlinx_account] = ACTIONS(1601),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1601),
    [sym_authenticate_certificate] = ACTIONS(1601),
    [sym_audit_netlinx_session_event] = ACTIONS(1601),
    [sym_audit_netlinx_generic_event] = ACTIONS(1601),
    [sym_get_available_flash_disk_space] = ACTIONS(1601),
    [sym_get_max_flash_disk_space] = ACTIONS(1601),
    [sym_data_create_feed] = ACTIONS(1601),
    [sym_data_delete_feed] = ACTIONS(1601),
    [sym_data_publish_feed] = ACTIONS(1601),
    [sym_data_get_published_feed] = ACTIONS(1601),
    [sym_data_add_record] = ACTIONS(1601),
    [sym_data_get_event_record] = ACTIONS(1601),
    [sym__wc_data_create_feed] = ACTIONS(1601),
    [sym__wc_data_add_record] = ACTIONS(1601),
    [sym__wc_data_get_event_record] = ACTIONS(1601),
    [sym_format] = ACTIONS(1601),
    [sym_astro_clock] = ACTIONS(1601),
    [sym_set_outdoor_temperature] = ACTIONS(1601),
    [sym_set_virtual_level_count] = ACTIONS(1601),
    [sym_set_virtual_channel_count] = ACTIONS(1601),
    [sym_set_virtual_port_count] = ACTIONS(1601),
    [sym__wc_encode_int] = ACTIONS(1601),
    [sym__wc_decode_int] = ACTIONS(1601),
    [sym__wc_int] = ACTIONS(1601),
    [sym__wc_to_ch_int] = ACTIONS(1601),
    [sym__ch_to_wc_int] = ACTIONS(1601),
    [sym__wc_find_string_int] = ACTIONS(1601),
    [sym__wc_left_string_int] = ACTIONS(1601),
    [sym__wc_lower_string_int] = ACTIONS(1601),
    [sym__wc_mid_string_int] = ACTIONS(1601),
    [sym__wc_remove_string_int] = ACTIONS(1601),
    [sym__wc_right_string_int] = ACTIONS(1601),
    [sym__wc_upper_string_int] = ACTIONS(1601),
    [sym__wc_compare_string_int] = ACTIONS(1601),
    [sym__wc_get_buffer_char_int] = ACTIONS(1601),
    [sym__wc_get_buffer_string_int] = ACTIONS(1601),
    [sym__wc_concat_string_int] = ACTIONS(1601),
    [sym__wc_file_open_int] = ACTIONS(1601),
    [sym__wc_file_close_int] = ACTIONS(1601),
    [sym__wc_file_read_int] = ACTIONS(1601),
    [sym__wc_file_read_line_int] = ACTIONS(1601),
    [sym__wc_file_write_int] = ACTIONS(1601),
    [sym__wc_file_write_line_int] = ACTIONS(1601),
    [aux_sym_preproc_include_token1] = ACTIONS(1601),
    [aux_sym_preproc_define_token1] = ACTIONS(1603),
    [aux_sym_preproc_warn_token1] = ACTIONS(1603),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1603),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1603),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1603),
    [anon_sym_LBRACK] = ACTIONS(1603),
    [anon_sym_LBRACE] = ACTIONS(1603),
    [anon_sym_RBRACE] = ACTIONS(1603),
    [anon_sym_COLON] = ACTIONS(1603),
    [anon_sym_LPAREN] = ACTIONS(1601),
    [anon_sym_SEMI] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1601),
    [anon_sym_TILDE] = ACTIONS(1603),
    [anon_sym_DASH] = ACTIONS(1601),
    [anon_sym_PLUS] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1603),
    [anon_sym_SLASH] = ACTIONS(1601),
    [anon_sym_PERCENT] = ACTIONS(1603),
    [anon_sym_PIPE_PIPE] = ACTIONS(1603),
    [anon_sym_AMP_AMP] = ACTIONS(1603),
    [anon_sym_PIPE] = ACTIONS(1601),
    [anon_sym_CARET] = ACTIONS(1603),
    [anon_sym_AMP] = ACTIONS(1601),
    [anon_sym_EQ_EQ] = ACTIONS(1603),
    [anon_sym_BANG_EQ] = ACTIONS(1603),
    [anon_sym_LT_GT] = ACTIONS(1603),
    [anon_sym_GT] = ACTIONS(1601),
    [anon_sym_GT_EQ] = ACTIONS(1603),
    [anon_sym_LT_EQ] = ACTIONS(1603),
    [anon_sym_LT] = ACTIONS(1601),
    [anon_sym_LT_LT] = ACTIONS(1603),
    [anon_sym_GT_GT] = ACTIONS(1603),
    [anon_sym_DASH_DASH] = ACTIONS(1603),
    [anon_sym_PLUS_PLUS] = ACTIONS(1603),
    [anon_sym_DOT] = ACTIONS(1601),
    [anon_sym_DQUOTE] = ACTIONS(1603),
    [anon_sym_SQUOTE] = ACTIONS(1603),
    [sym_number_literal] = ACTIONS(1603),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1603),
  },
  [STATE(193)] = {
    [sym_identifier] = ACTIONS(1581),
    [sym_define_call_keyword] = ACTIONS(1581),
    [sym_define_function_keyword] = ACTIONS(1581),
    [sym_define_library_function_keyword] = ACTIONS(1581),
    [sym_define_module_keyword] = ACTIONS(1581),
    [sym_char_keyword] = ACTIONS(1581),
    [sym_widechar_keyword] = ACTIONS(1581),
    [sym_integer_keyword] = ACTIONS(1581),
    [sym_sinteger_keyword] = ACTIONS(1581),
    [sym_long_keyword] = ACTIONS(1581),
    [sym_slong_keyword] = ACTIONS(1581),
    [sym_float_keyword] = ACTIONS(1581),
    [sym_double_keyword] = ACTIONS(1581),
    [sym_variant_keyword] = ACTIONS(1581),
    [sym_variantarray_keyword] = ACTIONS(1581),
    [sym_local_var_keyword] = ACTIONS(1581),
    [sym_stack_var_keyword] = ACTIONS(1581),
    [sym_constant_keyword] = ACTIONS(1581),
    [sym_volatile_keyword] = ACTIONS(1581),
    [sym_non_volatile_keyword] = ACTIONS(1581),
    [sym_persistent_keyword] = ACTIONS(1581),
    [sym_dev_keyword] = ACTIONS(1581),
    [sym_devlev_keyword] = ACTIONS(1581),
    [sym_devchan_keyword] = ACTIONS(1581),
    [sym_if_keyword] = ACTIONS(1581),
    [sym_else_keyword] = ACTIONS(1581),
    [sym_switch_keyword] = ACTIONS(1581),
    [sym_case_keyword] = ACTIONS(1581),
    [sym_default_keyword] = ACTIONS(1581),
    [sym_while_keyword] = ACTIONS(1581),
    [sym_for_keyword] = ACTIONS(1581),
    [sym_break_keyword] = ACTIONS(1581),
    [sym_continue_keyword] = ACTIONS(1581),
    [sym_return_keyword] = ACTIONS(1581),
    [sym_select_keyword] = ACTIONS(1581),
    [sym_active_keyword] = ACTIONS(1581),
    [sym_struct_keyword] = ACTIONS(1581),
    [sym_structure_keyword] = ACTIONS(1581),
    [aux_sym_on_keyword_token1] = ACTIONS(1581),
    [aux_sym_off_keyword_token1] = ACTIONS(1581),
    [sym_send_level_keyword] = ACTIONS(1581),
    [sym_send_string_keyword] = ACTIONS(1581),
    [sym_send_command_keyword] = ACTIONS(1581),
    [sym_clear_buffer_keyword] = ACTIONS(1581),
    [sym_create_buffer_keyword] = ACTIONS(1581),
    [sym_create_multi_buffer_keyword] = ACTIONS(1581),
    [sym_call_keyword] = ACTIONS(1581),
    [sym_system_call_keyword] = ACTIONS(1581),
    [sym_devchan_to_keyword] = ACTIONS(1581),
    [sym_devchan_min_to_keyword] = ACTIONS(1581),
    [sym_devchan_total_off_keyword] = ACTIONS(1581),
    [sym_devchan_pulse_keyword] = ACTIONS(1581),
    [sym_wait_keyword] = ACTIONS(1581),
    [sym_cancel_wait_keyword] = ACTIONS(1581),
    [sym_wait_until_keyword] = ACTIONS(1581),
    [sym_cancel_wait_until_keyword] = ACTIONS(1581),
    [sym_cancel_all_wait_keyword] = ACTIONS(1581),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1581),
    [sym_and] = ACTIONS(1581),
    [sym_or] = ACTIONS(1581),
    [sym_not] = ACTIONS(1581),
    [sym_xor] = ACTIONS(1581),
    [sym_band] = ACTIONS(1581),
    [sym_bor] = ACTIONS(1581),
    [sym_bxor] = ACTIONS(1581),
    [sym_bnot] = ACTIONS(1581),
    [sym_lshift] = ACTIONS(1581),
    [sym_rshift] = ACTIONS(1581),
    [sym___file__] = ACTIONS(1581),
    [sym___date__] = ACTIONS(1581),
    [sym___ldate__] = ACTIONS(1581),
    [sym___time__] = ACTIONS(1581),
    [sym___line__] = ACTIONS(1581),
    [sym___name__] = ACTIONS(1581),
    [sym___netlinx__] = ACTIONS(1581),
    [sym_netlinx_axi_version] = ACTIONS(1581),
    [sym_url_flg_tcp] = ACTIONS(1581),
    [sym_url_flg_accinfopresent] = ACTIONS(1581),
    [sym_url_flg_temp] = ACTIONS(1581),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1581),
    [sym_url_flg_stat_mask] = ACTIONS(1581),
    [sym_url_flg_stat_lookup] = ACTIONS(1581),
    [sym_url_flg_stat_connecting] = ACTIONS(1581),
    [sym_url_flg_stat_waiting] = ACTIONS(1581),
    [sym_url_flg_stat_connected] = ACTIONS(1581),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1581),
    [sym_first_virtual_device] = ACTIONS(1581),
    [sym_first_local_port] = ACTIONS(1581),
    [sym_dynamic_virtual_device] = ACTIONS(1581),
    [sym_timeline_once] = ACTIONS(1581),
    [sym_timeline_repeat] = ACTIONS(1581),
    [sym_timeline_absolute] = ACTIONS(1581),
    [sym_timeline_relative] = ACTIONS(1581),
    [sym_do_push_timed_infinite] = ACTIONS(1581),
    [sym_source_type_no_address] = ACTIONS(1581),
    [sym_source_type_neuron_id] = ACTIONS(1581),
    [sym_source_type_ip_address] = ACTIONS(1581),
    [sym_source_type_axlink] = ACTIONS(1581),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1581),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1581),
    [sym_source_type_ip_socket_address] = ACTIONS(1581),
    [sym_source_type_rs232] = ACTIONS(1581),
    [sym_source_type_internal] = ACTIONS(1581),
    [sym_source_type_mac_address] = ACTIONS(1581),
    [sym_source_type_ipv4_port] = ACTIONS(1581),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1581),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1581),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1581),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1581),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1581),
    [sym_true] = ACTIONS(1581),
    [sym_false] = ACTIONS(1581),
    [sym_file_read_only] = ACTIONS(1581),
    [sym_file_rw_new] = ACTIONS(1581),
    [sym_file_rw_append] = ACTIONS(1581),
    [sym_ip_tcp] = ACTIONS(1581),
    [sym_ip_udp] = ACTIONS(1581),
    [sym_ip_udp_2way] = ACTIONS(1581),
    [sym_xml_encode_types] = ACTIONS(1581),
    [sym_xml_encode_char_as_list] = ACTIONS(1581),
    [sym_xml_encode_le] = ACTIONS(1581),
    [sym_xml_decode_types] = ACTIONS(1581),
    [sym_xml_decode_no_preserve] = ACTIONS(1581),
    [sym_duet_dev_type_display_device] = ACTIONS(1581),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1581),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1581),
    [sym_duet_dev_type_io_device] = ACTIONS(1581),
    [sym_duet_dev_type_relay_device] = ACTIONS(1581),
    [sym_duet_dev_type_ups] = ACTIONS(1581),
    [sym_duet_dev_type_amplifier] = ACTIONS(1581),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1581),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1581),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1581),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1581),
    [sym_duet_dev_type_camera] = ACTIONS(1581),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1581),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1581),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1581),
    [sym_duet_dev_type_dss] = ACTIONS(1581),
    [sym_duet_dev_type_dvr] = ACTIONS(1581),
    [sym_duet_dev_type_disc_device] = ACTIONS(1581),
    [sym_duet_dev_type_document_camera] = ACTIONS(1581),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1581),
    [sym_duet_dev_type_hvac] = ACTIONS(1581),
    [sym_duet_dev_type_keypad] = ACTIONS(1581),
    [sym_duet_dev_type_light] = ACTIONS(1581),
    [sym_duet_dev_type_monitor] = ACTIONS(1581),
    [sym_duet_dev_type_motor] = ACTIONS(1581),
    [sym_duet_dev_type_multi_window] = ACTIONS(1581),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1581),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1581),
    [sym_duet_dev_type_receiver] = ACTIONS(1581),
    [sym_duet_dev_type_security_system] = ACTIONS(1581),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1581),
    [sym_duet_dev_type_settop_box] = ACTIONS(1581),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1581),
    [sym_duet_dev_type_switcher] = ACTIONS(1581),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1581),
    [sym_duet_dev_type_tv] = ACTIONS(1581),
    [sym_duet_dev_type_utility] = ACTIONS(1581),
    [sym_duet_dev_type_vcr] = ACTIONS(1581),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1581),
    [sym_duet_dev_type_video_processor] = ACTIONS(1581),
    [sym_duet_dev_type_video_projector] = ACTIONS(1581),
    [sym_duet_dev_type_video_wall] = ACTIONS(1581),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1581),
    [sym_duet_dev_type_weather] = ACTIONS(1581),
    [sym_duet_dev_not_polled] = ACTIONS(1581),
    [sym_duet_dev_polled] = ACTIONS(1581),
    [sym_clkmgr_mode_network] = ACTIONS(1581),
    [sym_clkmgr_mode_standalone] = ACTIONS(1581),
    [sym_normal_standby] = ACTIONS(1581),
    [sym_normal_wake] = ACTIONS(1581),
    [sym_ip_multicast_ttl_option] = ACTIONS(1581),
    [sym_ip_ttl_subnet] = ACTIONS(1581),
    [sym_ip_ttl_site] = ACTIONS(1581),
    [sym_ip_ttl_region] = ACTIONS(1581),
    [sym_ip_ttl_continent] = ACTIONS(1581),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1581),
    [sym_ip_nodelay_on] = ACTIONS(1581),
    [sym_ip_nodelay_off] = ACTIONS(1581),
    [sym_amx_error] = ACTIONS(1581),
    [sym_amx_warning] = ACTIONS(1581),
    [sym_amx_info] = ACTIONS(1581),
    [sym_amx_debug] = ACTIONS(1581),
    [sym_netlinx_login_success] = ACTIONS(1581),
    [sym_netlinx_login_fail] = ACTIONS(1581),
    [sym_netlinx_logout] = ACTIONS(1581),
    [sym_null_str] = ACTIONS(1581),
    [sym_smtp_address] = ACTIONS(1581),
    [sym_smtp_port_number] = ACTIONS(1581),
    [sym_smtp_username] = ACTIONS(1581),
    [sym_smtp_password] = ACTIONS(1581),
    [sym_smtp_require_tls] = ACTIONS(1581),
    [sym_smtp_from] = ACTIONS(1581),
    [sym_smtp_tls_true] = ACTIONS(1581),
    [sym_smtp_tls_false] = ACTIONS(1581),
    [sym_listview_on_row_select_event] = ACTIONS(1581),
    [sym_data_structure_unknown] = ACTIONS(1581),
    [sym_data_structure_datafeed] = ACTIONS(1581),
    [sym_data_structure_datarecordset] = ACTIONS(1581),
    [sym_data_structure_datarecord] = ACTIONS(1581),
    [sym_data_structure_datafield] = ACTIONS(1581),
    [sym_data_max_content_field_count] = ACTIONS(1581),
    [sym_data_max_metadata_field_count] = ACTIONS(1581),
    [sym_data_max_id_length] = ACTIONS(1581),
    [sym_data_max_name_length] = ACTIONS(1581),
    [sym_data_max_description_length] = ACTIONS(1581),
    [sym_data_max_type_length] = ACTIONS(1581),
    [sym_data_max_format_length] = ACTIONS(1581),
    [sym_data_max_source_length] = ACTIONS(1581),
    [sym_data_max_value_length] = ACTIONS(1581),
    [sym_data_max_label_length] = ACTIONS(1581),
    [sym_data_max_url_length] = ACTIONS(1581),
    [sym_max_last_login_info_length] = ACTIONS(1581),
    [sym_data_type_unknown] = ACTIONS(1581),
    [sym_data_type_string] = ACTIONS(1581),
    [sym_data_type_datetime] = ACTIONS(1581),
    [sym_data_type_date] = ACTIONS(1581),
    [sym_data_type_time] = ACTIONS(1581),
    [sym_data_type_image] = ACTIONS(1581),
    [sym_data_format_url] = ACTIONS(1581),
    [sym_data_format_phone] = ACTIONS(1581),
    [sym_data_format_email] = ACTIONS(1581),
    [sym_data_format_iso8601] = ACTIONS(1581),
    [sym_auditlog_priv] = ACTIONS(1581),
    [sym_configuration_priv] = ACTIONS(1581),
    [sym_deviceconfig_priv] = ACTIONS(1581),
    [sym_ftp_priv] = ACTIONS(1581),
    [sym_http_priv] = ACTIONS(1581),
    [sym_networkconfig_priv] = ACTIONS(1581),
    [sym_programport_priv] = ACTIONS(1581),
    [sym_remoteui_priv] = ACTIONS(1581),
    [sym_securitycontrol_priv] = ACTIONS(1581),
    [sym_softwaremanagement_priv] = ACTIONS(1581),
    [sym_terminal_priv] = ACTIONS(1581),
    [sym_tpadmin_priv] = ACTIONS(1581),
    [sym_user1_priv] = ACTIONS(1581),
    [sym_user2_priv] = ACTIONS(1581),
    [sym_user3_priv] = ACTIONS(1581),
    [sym_user4_priv] = ACTIONS(1581),
    [sym_usermanagement_priv] = ACTIONS(1581),
    [sym_valid_account] = ACTIONS(1581),
    [sym_err_name_invalid] = ACTIONS(1581),
    [sym_err_password_invalid] = ACTIONS(1581),
    [sym_err_invalid_account] = ACTIONS(1581),
    [sym_err_no_authorization] = ACTIONS(1581),
    [sym_err_invalid_parameter] = ACTIONS(1581),
    [sym_err_user_locked_out] = ACTIONS(1581),
    [sym_err_user_account_expired] = ACTIONS(1581),
    [sym_err_general_error] = ACTIONS(1581),
    [sym_err_authentication_error] = ACTIONS(1581),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1581),
    [sym_tls_validate_certificate] = ACTIONS(1581),
    [sym_tdata] = ACTIONS(1581),
    [sym_tchannel] = ACTIONS(1581),
    [sym_tlevel] = ACTIONS(1581),
    [sym_tbutton] = ACTIONS(1581),
    [sym_ttimeline] = ACTIONS(1581),
    [sym_tcustom] = ACTIONS(1581),
    [sym_url_struct] = ACTIONS(1581),
    [sym_dns_struct] = ACTIONS(1581),
    [sym_ip_address_struct] = ACTIONS(1581),
    [sym_dev_info_struct] = ACTIONS(1581),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1581),
    [sym_data_feed] = ACTIONS(1581),
    [sym_data_field] = ACTIONS(1581),
    [sym_data_record] = ACTIONS(1581),
    [sym_wc_data_feed] = ACTIONS(1581),
    [sym_wc_data_field] = ACTIONS(1581),
    [sym_wc_data_record] = ACTIONS(1581),
    [sym_last_login_info] = ACTIONS(1581),
    [sym____reserved___] = ACTIONS(1581),
    [sym_date] = ACTIONS(1581),
    [sym_day] = ACTIONS(1581),
    [sym_time] = ACTIONS(1581),
    [sym_ldate] = ACTIONS(1581),
    [sym_push_device] = ACTIONS(1581),
    [sym_push_channel] = ACTIONS(1581),
    [sym_push_devchan] = ACTIONS(1581),
    [sym_release_device] = ACTIONS(1581),
    [sym_release_channel] = ACTIONS(1581),
    [sym_release_devchan] = ACTIONS(1581),
    [sym_master_slot] = ACTIONS(1581),
    [sym_get_pulse_time] = ACTIONS(1581),
    [sym_get_timer] = ACTIONS(1581),
    [sym_master_sn] = ACTIONS(1581),
    [sym_system_number] = ACTIONS(1581),
    [sym_button] = ACTIONS(1581),
    [sym_dv_channel] = ACTIONS(1581),
    [sym_data] = ACTIONS(1581),
    [sym_level] = ACTIONS(1581),
    [sym_timeline] = ACTIONS(1581),
    [sym_channel] = ACTIONS(1581),
    [sym_custom] = ACTIONS(1581),
    [sym_atoi] = ACTIONS(1581),
    [sym_atol] = ACTIONS(1581),
    [sym_atof] = ACTIONS(1581),
    [sym_itoa] = ACTIONS(1581),
    [sym_itohex] = ACTIONS(1581),
    [sym_hextoi] = ACTIONS(1581),
    [sym_ftoa] = ACTIONS(1581),
    [sym_find_string] = ACTIONS(1581),
    [sym_left_string] = ACTIONS(1581),
    [sym_right_string] = ACTIONS(1581),
    [sym_mid_string] = ACTIONS(1581),
    [sym_remove_string] = ACTIONS(1581),
    [sym_get_buffer_string] = ACTIONS(1581),
    [sym_get_buffer_char] = ACTIONS(1581),
    [sym_get_multi_buffer_string] = ACTIONS(1581),
    [sym_length_string] = ACTIONS(1581),
    [sym_max_length_string] = ACTIONS(1581),
    [sym_set_length_string] = ACTIONS(1581),
    [sym_compare_string] = ACTIONS(1581),
    [sym_upper_string] = ACTIONS(1581),
    [sym_lower_string] = ACTIONS(1581),
    [sym_get_last] = ACTIONS(1581),
    [sym_date_to_day] = ACTIONS(1581),
    [sym_date_to_month] = ACTIONS(1581),
    [sym_date_to_year] = ACTIONS(1581),
    [sym_time_to_hour] = ACTIONS(1581),
    [sym_time_to_minute] = ACTIONS(1581),
    [sym_time_to_second] = ACTIONS(1581),
    [sym_day_of_week] = ACTIONS(1581),
    [sym_set_pulse_time] = ACTIONS(1581),
    [sym_set_timer] = ACTIONS(1581),
    [sym_file_open] = ACTIONS(1581),
    [sym_file_close] = ACTIONS(1581),
    [sym_file_read] = ACTIONS(1581),
    [sym_file_read_line] = ACTIONS(1581),
    [sym_file_write] = ACTIONS(1581),
    [sym_file_write_line] = ACTIONS(1581),
    [sym_file_copy] = ACTIONS(1581),
    [sym_file_delete] = ACTIONS(1581),
    [sym_file_rename] = ACTIONS(1581),
    [sym_file_dir] = ACTIONS(1581),
    [sym_file_getdir] = ACTIONS(1581),
    [sym_file_setdir] = ACTIONS(1581),
    [sym_file_create_dir] = ACTIONS(1581),
    [sym_file_remove_dir] = ACTIONS(1581),
    [sym_file_seek] = ACTIONS(1581),
    [sym_length_array] = ACTIONS(1581),
    [sym_max_length_array] = ACTIONS(1581),
    [sym_set_length_array] = ACTIONS(1581),
    [sym_ip_client_open] = ACTIONS(1581),
    [sym_ip_client_close] = ACTIONS(1581),
    [sym_ip_server_open] = ACTIONS(1581),
    [sym_ip_server_close] = ACTIONS(1581),
    [sym_ip_bound_client_open] = ACTIONS(1581),
    [sym_ip_mc_server_open] = ACTIONS(1581),
    [sym_ip_set_option] = ACTIONS(1581),
    [sym_ssh_client_open] = ACTIONS(1581),
    [sym_ssh_client_close] = ACTIONS(1581),
    [sym_tls_client_open] = ACTIONS(1581),
    [sym_tls_client_close] = ACTIONS(1581),
    [sym_do_push] = ACTIONS(1581),
    [sym_do_push_timed] = ACTIONS(1581),
    [sym_do_release] = ACTIONS(1581),
    [sym_device_id] = ACTIONS(1581),
    [sym_device_info] = ACTIONS(1581),
    [sym_device_id_string] = ACTIONS(1581),
    [sym_device_standby] = ACTIONS(1581),
    [sym_device_wake] = ACTIONS(1581),
    [sym_timeline_create] = ACTIONS(1581),
    [sym_timeline_kill] = ACTIONS(1581),
    [sym_timeline_reload] = ACTIONS(1581),
    [sym_timeline_pause] = ACTIONS(1581),
    [sym_timeline_restart] = ACTIONS(1581),
    [sym_timeline_set] = ACTIONS(1581),
    [sym_timeline_get] = ACTIONS(1581),
    [sym_timeline_active] = ACTIONS(1581),
    [sym_timeline_dynamic_id] = ACTIONS(1581),
    [sym_variable_to_string] = ACTIONS(1581),
    [sym_string_to_variable] = ACTIONS(1581),
    [sym_length_variable_to_string] = ACTIONS(1581),
    [sym_type_cast] = ACTIONS(1581),
    [sym_variable_to_xml] = ACTIONS(1581),
    [sym_xml_to_variable] = ACTIONS(1581),
    [sym_length_variable_to_xml] = ACTIONS(1581),
    [sym_redirect_string] = ACTIONS(1581),
    [sym_random_number] = ACTIONS(1581),
    [sym_get_unique_id] = ACTIONS(1581),
    [sym_get_system_number] = ACTIONS(1581),
    [sym_set_system_number] = ACTIONS(1581),
    [sym_get_serial_number] = ACTIONS(1581),
    [sym_get_master_build] = ACTIONS(1581),
    [sym_set_validation_code] = ACTIONS(1581),
    [sym_reboot] = ACTIONS(1581),
    [sym_load_duet_module] = ACTIONS(1581),
    [sym_unload_duet_module] = ACTIONS(1581),
    [sym_duet_mem_size_set] = ACTIONS(1581),
    [sym_duet_mem_size_get] = ACTIONS(1581),
    [sym_dynamic_application_device] = ACTIONS(1581),
    [sym_dynamic_polled_port] = ACTIONS(1581),
    [sym_static_port_binding] = ACTIONS(1581),
    [sym_static_ip_binding] = ACTIONS(1581),
    [sym_get_dns_list] = ACTIONS(1581),
    [sym_set_dns_list] = ACTIONS(1581),
    [sym_get_ip_address] = ACTIONS(1581),
    [sym_set_ip_address] = ACTIONS(1581),
    [sym_get_url_list] = ACTIONS(1581),
    [sym_add_url_entry] = ACTIONS(1581),
    [sym_delete_url_entry] = ACTIONS(1581),
    [sym_abs_value] = ACTIONS(1581),
    [sym_max_value] = ACTIONS(1581),
    [sym_min_value] = ACTIONS(1581),
    [sym_exp_value] = ACTIONS(1581),
    [sym_log_value] = ACTIONS(1581),
    [sym_log10_value] = ACTIONS(1581),
    [sym_power_value] = ACTIONS(1581),
    [sym_sqrt_value] = ACTIONS(1581),
    [sym_raw_be] = ACTIONS(1581),
    [sym_raw_le] = ACTIONS(1581),
    [sym_internal_queue_size_set] = ACTIONS(1581),
    [sym_internal_queue_size_get] = ACTIONS(1581),
    [sym_internal_threshold_set] = ACTIONS(1581),
    [sym_internal_threshold_get] = ACTIONS(1581),
    [sym_rebuilt_event] = ACTIONS(1581),
    [sym_do_custom_event] = ACTIONS(1581),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1581),
    [sym_clkmgr_set_clk_source] = ACTIONS(1581),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1581),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1581),
    [sym_clkmgr_get_timezone] = ACTIONS(1581),
    [sym_clkmgr_set_timezone] = ACTIONS(1581),
    [sym_clkmgr_get_resync_period] = ACTIONS(1581),
    [sym_clkmgr_set_resync_period] = ACTIONS(1581),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1581),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1581),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1581),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1581),
    [sym_clkmgr_get_timeservers] = ACTIONS(1581),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1581),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1581),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1581),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1581),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1581),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1581),
    [sym_set_log_level] = ACTIONS(1581),
    [sym_get_log_level] = ACTIONS(1581),
    [sym_amx_log] = ACTIONS(1581),
    [sym_smtp_server_config_set] = ACTIONS(1581),
    [sym_smtp_server_config_get] = ACTIONS(1581),
    [sym_smtp_send] = ACTIONS(1581),
    [sym_validate_netlinx_account] = ACTIONS(1581),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1581),
    [sym_authenticate_certificate] = ACTIONS(1581),
    [sym_audit_netlinx_session_event] = ACTIONS(1581),
    [sym_audit_netlinx_generic_event] = ACTIONS(1581),
    [sym_get_available_flash_disk_space] = ACTIONS(1581),
    [sym_get_max_flash_disk_space] = ACTIONS(1581),
    [sym_data_create_feed] = ACTIONS(1581),
    [sym_data_delete_feed] = ACTIONS(1581),
    [sym_data_publish_feed] = ACTIONS(1581),
    [sym_data_get_published_feed] = ACTIONS(1581),
    [sym_data_add_record] = ACTIONS(1581),
    [sym_data_get_event_record] = ACTIONS(1581),
    [sym__wc_data_create_feed] = ACTIONS(1581),
    [sym__wc_data_add_record] = ACTIONS(1581),
    [sym__wc_data_get_event_record] = ACTIONS(1581),
    [sym_format] = ACTIONS(1581),
    [sym_astro_clock] = ACTIONS(1581),
    [sym_set_outdoor_temperature] = ACTIONS(1581),
    [sym_set_virtual_level_count] = ACTIONS(1581),
    [sym_set_virtual_channel_count] = ACTIONS(1581),
    [sym_set_virtual_port_count] = ACTIONS(1581),
    [sym__wc_encode_int] = ACTIONS(1581),
    [sym__wc_decode_int] = ACTIONS(1581),
    [sym__wc_int] = ACTIONS(1581),
    [sym__wc_to_ch_int] = ACTIONS(1581),
    [sym__ch_to_wc_int] = ACTIONS(1581),
    [sym__wc_find_string_int] = ACTIONS(1581),
    [sym__wc_left_string_int] = ACTIONS(1581),
    [sym__wc_lower_string_int] = ACTIONS(1581),
    [sym__wc_mid_string_int] = ACTIONS(1581),
    [sym__wc_remove_string_int] = ACTIONS(1581),
    [sym__wc_right_string_int] = ACTIONS(1581),
    [sym__wc_upper_string_int] = ACTIONS(1581),
    [sym__wc_compare_string_int] = ACTIONS(1581),
    [sym__wc_get_buffer_char_int] = ACTIONS(1581),
    [sym__wc_get_buffer_string_int] = ACTIONS(1581),
    [sym__wc_concat_string_int] = ACTIONS(1581),
    [sym__wc_file_open_int] = ACTIONS(1581),
    [sym__wc_file_close_int] = ACTIONS(1581),
    [sym__wc_file_read_int] = ACTIONS(1581),
    [sym__wc_file_read_line_int] = ACTIONS(1581),
    [sym__wc_file_write_int] = ACTIONS(1581),
    [sym__wc_file_write_line_int] = ACTIONS(1581),
    [aux_sym_preproc_include_token1] = ACTIONS(1581),
    [aux_sym_preproc_define_token1] = ACTIONS(1583),
    [aux_sym_preproc_warn_token1] = ACTIONS(1583),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1583),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1583),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1583),
    [anon_sym_LBRACK] = ACTIONS(1583),
    [anon_sym_LBRACE] = ACTIONS(1583),
    [anon_sym_RBRACE] = ACTIONS(1583),
    [anon_sym_COLON] = ACTIONS(1583),
    [anon_sym_LPAREN] = ACTIONS(1581),
    [anon_sym_SEMI] = ACTIONS(1583),
    [anon_sym_BANG] = ACTIONS(1581),
    [anon_sym_TILDE] = ACTIONS(1583),
    [anon_sym_DASH] = ACTIONS(1581),
    [anon_sym_PLUS] = ACTIONS(1581),
    [anon_sym_STAR] = ACTIONS(1583),
    [anon_sym_SLASH] = ACTIONS(1581),
    [anon_sym_PERCENT] = ACTIONS(1583),
    [anon_sym_PIPE_PIPE] = ACTIONS(1583),
    [anon_sym_AMP_AMP] = ACTIONS(1583),
    [anon_sym_PIPE] = ACTIONS(1581),
    [anon_sym_CARET] = ACTIONS(1583),
    [anon_sym_AMP] = ACTIONS(1581),
    [anon_sym_EQ_EQ] = ACTIONS(1583),
    [anon_sym_BANG_EQ] = ACTIONS(1583),
    [anon_sym_LT_GT] = ACTIONS(1583),
    [anon_sym_GT] = ACTIONS(1581),
    [anon_sym_GT_EQ] = ACTIONS(1583),
    [anon_sym_LT_EQ] = ACTIONS(1583),
    [anon_sym_LT] = ACTIONS(1581),
    [anon_sym_LT_LT] = ACTIONS(1583),
    [anon_sym_GT_GT] = ACTIONS(1583),
    [anon_sym_DASH_DASH] = ACTIONS(1583),
    [anon_sym_PLUS_PLUS] = ACTIONS(1583),
    [anon_sym_DOT] = ACTIONS(1581),
    [anon_sym_DQUOTE] = ACTIONS(1583),
    [anon_sym_SQUOTE] = ACTIONS(1583),
    [sym_number_literal] = ACTIONS(1583),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1583),
  },
  [STATE(194)] = {
    [sym_identifier] = ACTIONS(1597),
    [sym_define_call_keyword] = ACTIONS(1597),
    [sym_define_function_keyword] = ACTIONS(1597),
    [sym_define_library_function_keyword] = ACTIONS(1597),
    [sym_define_module_keyword] = ACTIONS(1597),
    [sym_char_keyword] = ACTIONS(1597),
    [sym_widechar_keyword] = ACTIONS(1597),
    [sym_integer_keyword] = ACTIONS(1597),
    [sym_sinteger_keyword] = ACTIONS(1597),
    [sym_long_keyword] = ACTIONS(1597),
    [sym_slong_keyword] = ACTIONS(1597),
    [sym_float_keyword] = ACTIONS(1597),
    [sym_double_keyword] = ACTIONS(1597),
    [sym_variant_keyword] = ACTIONS(1597),
    [sym_variantarray_keyword] = ACTIONS(1597),
    [sym_local_var_keyword] = ACTIONS(1597),
    [sym_stack_var_keyword] = ACTIONS(1597),
    [sym_constant_keyword] = ACTIONS(1597),
    [sym_volatile_keyword] = ACTIONS(1597),
    [sym_non_volatile_keyword] = ACTIONS(1597),
    [sym_persistent_keyword] = ACTIONS(1597),
    [sym_dev_keyword] = ACTIONS(1597),
    [sym_devlev_keyword] = ACTIONS(1597),
    [sym_devchan_keyword] = ACTIONS(1597),
    [sym_if_keyword] = ACTIONS(1597),
    [sym_else_keyword] = ACTIONS(1597),
    [sym_switch_keyword] = ACTIONS(1597),
    [sym_case_keyword] = ACTIONS(1597),
    [sym_default_keyword] = ACTIONS(1597),
    [sym_while_keyword] = ACTIONS(1597),
    [sym_for_keyword] = ACTIONS(1597),
    [sym_break_keyword] = ACTIONS(1597),
    [sym_continue_keyword] = ACTIONS(1597),
    [sym_return_keyword] = ACTIONS(1597),
    [sym_select_keyword] = ACTIONS(1597),
    [sym_active_keyword] = ACTIONS(1597),
    [sym_struct_keyword] = ACTIONS(1597),
    [sym_structure_keyword] = ACTIONS(1597),
    [aux_sym_on_keyword_token1] = ACTIONS(1597),
    [aux_sym_off_keyword_token1] = ACTIONS(1597),
    [sym_send_level_keyword] = ACTIONS(1597),
    [sym_send_string_keyword] = ACTIONS(1597),
    [sym_send_command_keyword] = ACTIONS(1597),
    [sym_clear_buffer_keyword] = ACTIONS(1597),
    [sym_create_buffer_keyword] = ACTIONS(1597),
    [sym_create_multi_buffer_keyword] = ACTIONS(1597),
    [sym_call_keyword] = ACTIONS(1597),
    [sym_system_call_keyword] = ACTIONS(1597),
    [sym_devchan_to_keyword] = ACTIONS(1597),
    [sym_devchan_min_to_keyword] = ACTIONS(1597),
    [sym_devchan_total_off_keyword] = ACTIONS(1597),
    [sym_devchan_pulse_keyword] = ACTIONS(1597),
    [sym_wait_keyword] = ACTIONS(1597),
    [sym_cancel_wait_keyword] = ACTIONS(1597),
    [sym_wait_until_keyword] = ACTIONS(1597),
    [sym_cancel_wait_until_keyword] = ACTIONS(1597),
    [sym_cancel_all_wait_keyword] = ACTIONS(1597),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1597),
    [sym_and] = ACTIONS(1597),
    [sym_or] = ACTIONS(1597),
    [sym_not] = ACTIONS(1597),
    [sym_xor] = ACTIONS(1597),
    [sym_band] = ACTIONS(1597),
    [sym_bor] = ACTIONS(1597),
    [sym_bxor] = ACTIONS(1597),
    [sym_bnot] = ACTIONS(1597),
    [sym_lshift] = ACTIONS(1597),
    [sym_rshift] = ACTIONS(1597),
    [sym___file__] = ACTIONS(1597),
    [sym___date__] = ACTIONS(1597),
    [sym___ldate__] = ACTIONS(1597),
    [sym___time__] = ACTIONS(1597),
    [sym___line__] = ACTIONS(1597),
    [sym___name__] = ACTIONS(1597),
    [sym___netlinx__] = ACTIONS(1597),
    [sym_netlinx_axi_version] = ACTIONS(1597),
    [sym_url_flg_tcp] = ACTIONS(1597),
    [sym_url_flg_accinfopresent] = ACTIONS(1597),
    [sym_url_flg_temp] = ACTIONS(1597),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1597),
    [sym_url_flg_stat_mask] = ACTIONS(1597),
    [sym_url_flg_stat_lookup] = ACTIONS(1597),
    [sym_url_flg_stat_connecting] = ACTIONS(1597),
    [sym_url_flg_stat_waiting] = ACTIONS(1597),
    [sym_url_flg_stat_connected] = ACTIONS(1597),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1597),
    [sym_first_virtual_device] = ACTIONS(1597),
    [sym_first_local_port] = ACTIONS(1597),
    [sym_dynamic_virtual_device] = ACTIONS(1597),
    [sym_timeline_once] = ACTIONS(1597),
    [sym_timeline_repeat] = ACTIONS(1597),
    [sym_timeline_absolute] = ACTIONS(1597),
    [sym_timeline_relative] = ACTIONS(1597),
    [sym_do_push_timed_infinite] = ACTIONS(1597),
    [sym_source_type_no_address] = ACTIONS(1597),
    [sym_source_type_neuron_id] = ACTIONS(1597),
    [sym_source_type_ip_address] = ACTIONS(1597),
    [sym_source_type_axlink] = ACTIONS(1597),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1597),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1597),
    [sym_source_type_ip_socket_address] = ACTIONS(1597),
    [sym_source_type_rs232] = ACTIONS(1597),
    [sym_source_type_internal] = ACTIONS(1597),
    [sym_source_type_mac_address] = ACTIONS(1597),
    [sym_source_type_ipv4_port] = ACTIONS(1597),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1597),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1597),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1597),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1597),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1597),
    [sym_true] = ACTIONS(1597),
    [sym_false] = ACTIONS(1597),
    [sym_file_read_only] = ACTIONS(1597),
    [sym_file_rw_new] = ACTIONS(1597),
    [sym_file_rw_append] = ACTIONS(1597),
    [sym_ip_tcp] = ACTIONS(1597),
    [sym_ip_udp] = ACTIONS(1597),
    [sym_ip_udp_2way] = ACTIONS(1597),
    [sym_xml_encode_types] = ACTIONS(1597),
    [sym_xml_encode_char_as_list] = ACTIONS(1597),
    [sym_xml_encode_le] = ACTIONS(1597),
    [sym_xml_decode_types] = ACTIONS(1597),
    [sym_xml_decode_no_preserve] = ACTIONS(1597),
    [sym_duet_dev_type_display_device] = ACTIONS(1597),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1597),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1597),
    [sym_duet_dev_type_io_device] = ACTIONS(1597),
    [sym_duet_dev_type_relay_device] = ACTIONS(1597),
    [sym_duet_dev_type_ups] = ACTIONS(1597),
    [sym_duet_dev_type_amplifier] = ACTIONS(1597),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1597),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1597),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1597),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1597),
    [sym_duet_dev_type_camera] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1597),
    [sym_duet_dev_type_dss] = ACTIONS(1597),
    [sym_duet_dev_type_dvr] = ACTIONS(1597),
    [sym_duet_dev_type_disc_device] = ACTIONS(1597),
    [sym_duet_dev_type_document_camera] = ACTIONS(1597),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1597),
    [sym_duet_dev_type_hvac] = ACTIONS(1597),
    [sym_duet_dev_type_keypad] = ACTIONS(1597),
    [sym_duet_dev_type_light] = ACTIONS(1597),
    [sym_duet_dev_type_monitor] = ACTIONS(1597),
    [sym_duet_dev_type_motor] = ACTIONS(1597),
    [sym_duet_dev_type_multi_window] = ACTIONS(1597),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1597),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1597),
    [sym_duet_dev_type_receiver] = ACTIONS(1597),
    [sym_duet_dev_type_security_system] = ACTIONS(1597),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1597),
    [sym_duet_dev_type_settop_box] = ACTIONS(1597),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1597),
    [sym_duet_dev_type_switcher] = ACTIONS(1597),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1597),
    [sym_duet_dev_type_tv] = ACTIONS(1597),
    [sym_duet_dev_type_utility] = ACTIONS(1597),
    [sym_duet_dev_type_vcr] = ACTIONS(1597),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1597),
    [sym_duet_dev_type_video_processor] = ACTIONS(1597),
    [sym_duet_dev_type_video_projector] = ACTIONS(1597),
    [sym_duet_dev_type_video_wall] = ACTIONS(1597),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1597),
    [sym_duet_dev_type_weather] = ACTIONS(1597),
    [sym_duet_dev_not_polled] = ACTIONS(1597),
    [sym_duet_dev_polled] = ACTIONS(1597),
    [sym_clkmgr_mode_network] = ACTIONS(1597),
    [sym_clkmgr_mode_standalone] = ACTIONS(1597),
    [sym_normal_standby] = ACTIONS(1597),
    [sym_normal_wake] = ACTIONS(1597),
    [sym_ip_multicast_ttl_option] = ACTIONS(1597),
    [sym_ip_ttl_subnet] = ACTIONS(1597),
    [sym_ip_ttl_site] = ACTIONS(1597),
    [sym_ip_ttl_region] = ACTIONS(1597),
    [sym_ip_ttl_continent] = ACTIONS(1597),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1597),
    [sym_ip_nodelay_on] = ACTIONS(1597),
    [sym_ip_nodelay_off] = ACTIONS(1597),
    [sym_amx_error] = ACTIONS(1597),
    [sym_amx_warning] = ACTIONS(1597),
    [sym_amx_info] = ACTIONS(1597),
    [sym_amx_debug] = ACTIONS(1597),
    [sym_netlinx_login_success] = ACTIONS(1597),
    [sym_netlinx_login_fail] = ACTIONS(1597),
    [sym_netlinx_logout] = ACTIONS(1597),
    [sym_null_str] = ACTIONS(1597),
    [sym_smtp_address] = ACTIONS(1597),
    [sym_smtp_port_number] = ACTIONS(1597),
    [sym_smtp_username] = ACTIONS(1597),
    [sym_smtp_password] = ACTIONS(1597),
    [sym_smtp_require_tls] = ACTIONS(1597),
    [sym_smtp_from] = ACTIONS(1597),
    [sym_smtp_tls_true] = ACTIONS(1597),
    [sym_smtp_tls_false] = ACTIONS(1597),
    [sym_listview_on_row_select_event] = ACTIONS(1597),
    [sym_data_structure_unknown] = ACTIONS(1597),
    [sym_data_structure_datafeed] = ACTIONS(1597),
    [sym_data_structure_datarecordset] = ACTIONS(1597),
    [sym_data_structure_datarecord] = ACTIONS(1597),
    [sym_data_structure_datafield] = ACTIONS(1597),
    [sym_data_max_content_field_count] = ACTIONS(1597),
    [sym_data_max_metadata_field_count] = ACTIONS(1597),
    [sym_data_max_id_length] = ACTIONS(1597),
    [sym_data_max_name_length] = ACTIONS(1597),
    [sym_data_max_description_length] = ACTIONS(1597),
    [sym_data_max_type_length] = ACTIONS(1597),
    [sym_data_max_format_length] = ACTIONS(1597),
    [sym_data_max_source_length] = ACTIONS(1597),
    [sym_data_max_value_length] = ACTIONS(1597),
    [sym_data_max_label_length] = ACTIONS(1597),
    [sym_data_max_url_length] = ACTIONS(1597),
    [sym_max_last_login_info_length] = ACTIONS(1597),
    [sym_data_type_unknown] = ACTIONS(1597),
    [sym_data_type_string] = ACTIONS(1597),
    [sym_data_type_datetime] = ACTIONS(1597),
    [sym_data_type_date] = ACTIONS(1597),
    [sym_data_type_time] = ACTIONS(1597),
    [sym_data_type_image] = ACTIONS(1597),
    [sym_data_format_url] = ACTIONS(1597),
    [sym_data_format_phone] = ACTIONS(1597),
    [sym_data_format_email] = ACTIONS(1597),
    [sym_data_format_iso8601] = ACTIONS(1597),
    [sym_auditlog_priv] = ACTIONS(1597),
    [sym_configuration_priv] = ACTIONS(1597),
    [sym_deviceconfig_priv] = ACTIONS(1597),
    [sym_ftp_priv] = ACTIONS(1597),
    [sym_http_priv] = ACTIONS(1597),
    [sym_networkconfig_priv] = ACTIONS(1597),
    [sym_programport_priv] = ACTIONS(1597),
    [sym_remoteui_priv] = ACTIONS(1597),
    [sym_securitycontrol_priv] = ACTIONS(1597),
    [sym_softwaremanagement_priv] = ACTIONS(1597),
    [sym_terminal_priv] = ACTIONS(1597),
    [sym_tpadmin_priv] = ACTIONS(1597),
    [sym_user1_priv] = ACTIONS(1597),
    [sym_user2_priv] = ACTIONS(1597),
    [sym_user3_priv] = ACTIONS(1597),
    [sym_user4_priv] = ACTIONS(1597),
    [sym_usermanagement_priv] = ACTIONS(1597),
    [sym_valid_account] = ACTIONS(1597),
    [sym_err_name_invalid] = ACTIONS(1597),
    [sym_err_password_invalid] = ACTIONS(1597),
    [sym_err_invalid_account] = ACTIONS(1597),
    [sym_err_no_authorization] = ACTIONS(1597),
    [sym_err_invalid_parameter] = ACTIONS(1597),
    [sym_err_user_locked_out] = ACTIONS(1597),
    [sym_err_user_account_expired] = ACTIONS(1597),
    [sym_err_general_error] = ACTIONS(1597),
    [sym_err_authentication_error] = ACTIONS(1597),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1597),
    [sym_tls_validate_certificate] = ACTIONS(1597),
    [sym_tdata] = ACTIONS(1597),
    [sym_tchannel] = ACTIONS(1597),
    [sym_tlevel] = ACTIONS(1597),
    [sym_tbutton] = ACTIONS(1597),
    [sym_ttimeline] = ACTIONS(1597),
    [sym_tcustom] = ACTIONS(1597),
    [sym_url_struct] = ACTIONS(1597),
    [sym_dns_struct] = ACTIONS(1597),
    [sym_ip_address_struct] = ACTIONS(1597),
    [sym_dev_info_struct] = ACTIONS(1597),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1597),
    [sym_data_feed] = ACTIONS(1597),
    [sym_data_field] = ACTIONS(1597),
    [sym_data_record] = ACTIONS(1597),
    [sym_wc_data_feed] = ACTIONS(1597),
    [sym_wc_data_field] = ACTIONS(1597),
    [sym_wc_data_record] = ACTIONS(1597),
    [sym_last_login_info] = ACTIONS(1597),
    [sym____reserved___] = ACTIONS(1597),
    [sym_date] = ACTIONS(1597),
    [sym_day] = ACTIONS(1597),
    [sym_time] = ACTIONS(1597),
    [sym_ldate] = ACTIONS(1597),
    [sym_push_device] = ACTIONS(1597),
    [sym_push_channel] = ACTIONS(1597),
    [sym_push_devchan] = ACTIONS(1597),
    [sym_release_device] = ACTIONS(1597),
    [sym_release_channel] = ACTIONS(1597),
    [sym_release_devchan] = ACTIONS(1597),
    [sym_master_slot] = ACTIONS(1597),
    [sym_get_pulse_time] = ACTIONS(1597),
    [sym_get_timer] = ACTIONS(1597),
    [sym_master_sn] = ACTIONS(1597),
    [sym_system_number] = ACTIONS(1597),
    [sym_button] = ACTIONS(1597),
    [sym_dv_channel] = ACTIONS(1597),
    [sym_data] = ACTIONS(1597),
    [sym_level] = ACTIONS(1597),
    [sym_timeline] = ACTIONS(1597),
    [sym_channel] = ACTIONS(1597),
    [sym_custom] = ACTIONS(1597),
    [sym_atoi] = ACTIONS(1597),
    [sym_atol] = ACTIONS(1597),
    [sym_atof] = ACTIONS(1597),
    [sym_itoa] = ACTIONS(1597),
    [sym_itohex] = ACTIONS(1597),
    [sym_hextoi] = ACTIONS(1597),
    [sym_ftoa] = ACTIONS(1597),
    [sym_find_string] = ACTIONS(1597),
    [sym_left_string] = ACTIONS(1597),
    [sym_right_string] = ACTIONS(1597),
    [sym_mid_string] = ACTIONS(1597),
    [sym_remove_string] = ACTIONS(1597),
    [sym_get_buffer_string] = ACTIONS(1597),
    [sym_get_buffer_char] = ACTIONS(1597),
    [sym_get_multi_buffer_string] = ACTIONS(1597),
    [sym_length_string] = ACTIONS(1597),
    [sym_max_length_string] = ACTIONS(1597),
    [sym_set_length_string] = ACTIONS(1597),
    [sym_compare_string] = ACTIONS(1597),
    [sym_upper_string] = ACTIONS(1597),
    [sym_lower_string] = ACTIONS(1597),
    [sym_get_last] = ACTIONS(1597),
    [sym_date_to_day] = ACTIONS(1597),
    [sym_date_to_month] = ACTIONS(1597),
    [sym_date_to_year] = ACTIONS(1597),
    [sym_time_to_hour] = ACTIONS(1597),
    [sym_time_to_minute] = ACTIONS(1597),
    [sym_time_to_second] = ACTIONS(1597),
    [sym_day_of_week] = ACTIONS(1597),
    [sym_set_pulse_time] = ACTIONS(1597),
    [sym_set_timer] = ACTIONS(1597),
    [sym_file_open] = ACTIONS(1597),
    [sym_file_close] = ACTIONS(1597),
    [sym_file_read] = ACTIONS(1597),
    [sym_file_read_line] = ACTIONS(1597),
    [sym_file_write] = ACTIONS(1597),
    [sym_file_write_line] = ACTIONS(1597),
    [sym_file_copy] = ACTIONS(1597),
    [sym_file_delete] = ACTIONS(1597),
    [sym_file_rename] = ACTIONS(1597),
    [sym_file_dir] = ACTIONS(1597),
    [sym_file_getdir] = ACTIONS(1597),
    [sym_file_setdir] = ACTIONS(1597),
    [sym_file_create_dir] = ACTIONS(1597),
    [sym_file_remove_dir] = ACTIONS(1597),
    [sym_file_seek] = ACTIONS(1597),
    [sym_length_array] = ACTIONS(1597),
    [sym_max_length_array] = ACTIONS(1597),
    [sym_set_length_array] = ACTIONS(1597),
    [sym_ip_client_open] = ACTIONS(1597),
    [sym_ip_client_close] = ACTIONS(1597),
    [sym_ip_server_open] = ACTIONS(1597),
    [sym_ip_server_close] = ACTIONS(1597),
    [sym_ip_bound_client_open] = ACTIONS(1597),
    [sym_ip_mc_server_open] = ACTIONS(1597),
    [sym_ip_set_option] = ACTIONS(1597),
    [sym_ssh_client_open] = ACTIONS(1597),
    [sym_ssh_client_close] = ACTIONS(1597),
    [sym_tls_client_open] = ACTIONS(1597),
    [sym_tls_client_close] = ACTIONS(1597),
    [sym_do_push] = ACTIONS(1597),
    [sym_do_push_timed] = ACTIONS(1597),
    [sym_do_release] = ACTIONS(1597),
    [sym_device_id] = ACTIONS(1597),
    [sym_device_info] = ACTIONS(1597),
    [sym_device_id_string] = ACTIONS(1597),
    [sym_device_standby] = ACTIONS(1597),
    [sym_device_wake] = ACTIONS(1597),
    [sym_timeline_create] = ACTIONS(1597),
    [sym_timeline_kill] = ACTIONS(1597),
    [sym_timeline_reload] = ACTIONS(1597),
    [sym_timeline_pause] = ACTIONS(1597),
    [sym_timeline_restart] = ACTIONS(1597),
    [sym_timeline_set] = ACTIONS(1597),
    [sym_timeline_get] = ACTIONS(1597),
    [sym_timeline_active] = ACTIONS(1597),
    [sym_timeline_dynamic_id] = ACTIONS(1597),
    [sym_variable_to_string] = ACTIONS(1597),
    [sym_string_to_variable] = ACTIONS(1597),
    [sym_length_variable_to_string] = ACTIONS(1597),
    [sym_type_cast] = ACTIONS(1597),
    [sym_variable_to_xml] = ACTIONS(1597),
    [sym_xml_to_variable] = ACTIONS(1597),
    [sym_length_variable_to_xml] = ACTIONS(1597),
    [sym_redirect_string] = ACTIONS(1597),
    [sym_random_number] = ACTIONS(1597),
    [sym_get_unique_id] = ACTIONS(1597),
    [sym_get_system_number] = ACTIONS(1597),
    [sym_set_system_number] = ACTIONS(1597),
    [sym_get_serial_number] = ACTIONS(1597),
    [sym_get_master_build] = ACTIONS(1597),
    [sym_set_validation_code] = ACTIONS(1597),
    [sym_reboot] = ACTIONS(1597),
    [sym_load_duet_module] = ACTIONS(1597),
    [sym_unload_duet_module] = ACTIONS(1597),
    [sym_duet_mem_size_set] = ACTIONS(1597),
    [sym_duet_mem_size_get] = ACTIONS(1597),
    [sym_dynamic_application_device] = ACTIONS(1597),
    [sym_dynamic_polled_port] = ACTIONS(1597),
    [sym_static_port_binding] = ACTIONS(1597),
    [sym_static_ip_binding] = ACTIONS(1597),
    [sym_get_dns_list] = ACTIONS(1597),
    [sym_set_dns_list] = ACTIONS(1597),
    [sym_get_ip_address] = ACTIONS(1597),
    [sym_set_ip_address] = ACTIONS(1597),
    [sym_get_url_list] = ACTIONS(1597),
    [sym_add_url_entry] = ACTIONS(1597),
    [sym_delete_url_entry] = ACTIONS(1597),
    [sym_abs_value] = ACTIONS(1597),
    [sym_max_value] = ACTIONS(1597),
    [sym_min_value] = ACTIONS(1597),
    [sym_exp_value] = ACTIONS(1597),
    [sym_log_value] = ACTIONS(1597),
    [sym_log10_value] = ACTIONS(1597),
    [sym_power_value] = ACTIONS(1597),
    [sym_sqrt_value] = ACTIONS(1597),
    [sym_raw_be] = ACTIONS(1597),
    [sym_raw_le] = ACTIONS(1597),
    [sym_internal_queue_size_set] = ACTIONS(1597),
    [sym_internal_queue_size_get] = ACTIONS(1597),
    [sym_internal_threshold_set] = ACTIONS(1597),
    [sym_internal_threshold_get] = ACTIONS(1597),
    [sym_rebuilt_event] = ACTIONS(1597),
    [sym_do_custom_event] = ACTIONS(1597),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1597),
    [sym_clkmgr_set_clk_source] = ACTIONS(1597),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1597),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1597),
    [sym_clkmgr_get_timezone] = ACTIONS(1597),
    [sym_clkmgr_set_timezone] = ACTIONS(1597),
    [sym_clkmgr_get_resync_period] = ACTIONS(1597),
    [sym_clkmgr_set_resync_period] = ACTIONS(1597),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1597),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1597),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1597),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1597),
    [sym_clkmgr_get_timeservers] = ACTIONS(1597),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1597),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1597),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1597),
    [sym_set_log_level] = ACTIONS(1597),
    [sym_get_log_level] = ACTIONS(1597),
    [sym_amx_log] = ACTIONS(1597),
    [sym_smtp_server_config_set] = ACTIONS(1597),
    [sym_smtp_server_config_get] = ACTIONS(1597),
    [sym_smtp_send] = ACTIONS(1597),
    [sym_validate_netlinx_account] = ACTIONS(1597),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1597),
    [sym_authenticate_certificate] = ACTIONS(1597),
    [sym_audit_netlinx_session_event] = ACTIONS(1597),
    [sym_audit_netlinx_generic_event] = ACTIONS(1597),
    [sym_get_available_flash_disk_space] = ACTIONS(1597),
    [sym_get_max_flash_disk_space] = ACTIONS(1597),
    [sym_data_create_feed] = ACTIONS(1597),
    [sym_data_delete_feed] = ACTIONS(1597),
    [sym_data_publish_feed] = ACTIONS(1597),
    [sym_data_get_published_feed] = ACTIONS(1597),
    [sym_data_add_record] = ACTIONS(1597),
    [sym_data_get_event_record] = ACTIONS(1597),
    [sym__wc_data_create_feed] = ACTIONS(1597),
    [sym__wc_data_add_record] = ACTIONS(1597),
    [sym__wc_data_get_event_record] = ACTIONS(1597),
    [sym_format] = ACTIONS(1597),
    [sym_astro_clock] = ACTIONS(1597),
    [sym_set_outdoor_temperature] = ACTIONS(1597),
    [sym_set_virtual_level_count] = ACTIONS(1597),
    [sym_set_virtual_channel_count] = ACTIONS(1597),
    [sym_set_virtual_port_count] = ACTIONS(1597),
    [sym__wc_encode_int] = ACTIONS(1597),
    [sym__wc_decode_int] = ACTIONS(1597),
    [sym__wc_int] = ACTIONS(1597),
    [sym__wc_to_ch_int] = ACTIONS(1597),
    [sym__ch_to_wc_int] = ACTIONS(1597),
    [sym__wc_find_string_int] = ACTIONS(1597),
    [sym__wc_left_string_int] = ACTIONS(1597),
    [sym__wc_lower_string_int] = ACTIONS(1597),
    [sym__wc_mid_string_int] = ACTIONS(1597),
    [sym__wc_remove_string_int] = ACTIONS(1597),
    [sym__wc_right_string_int] = ACTIONS(1597),
    [sym__wc_upper_string_int] = ACTIONS(1597),
    [sym__wc_compare_string_int] = ACTIONS(1597),
    [sym__wc_get_buffer_char_int] = ACTIONS(1597),
    [sym__wc_get_buffer_string_int] = ACTIONS(1597),
    [sym__wc_concat_string_int] = ACTIONS(1597),
    [sym__wc_file_open_int] = ACTIONS(1597),
    [sym__wc_file_close_int] = ACTIONS(1597),
    [sym__wc_file_read_int] = ACTIONS(1597),
    [sym__wc_file_read_line_int] = ACTIONS(1597),
    [sym__wc_file_write_int] = ACTIONS(1597),
    [sym__wc_file_write_line_int] = ACTIONS(1597),
    [aux_sym_preproc_include_token1] = ACTIONS(1597),
    [aux_sym_preproc_define_token1] = ACTIONS(1599),
    [aux_sym_preproc_warn_token1] = ACTIONS(1599),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1599),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1599),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1599),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1599),
    [anon_sym_RBRACE] = ACTIONS(1599),
    [anon_sym_COLON] = ACTIONS(1599),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_SEMI] = ACTIONS(1599),
    [anon_sym_BANG] = ACTIONS(1597),
    [anon_sym_TILDE] = ACTIONS(1599),
    [anon_sym_DASH] = ACTIONS(1597),
    [anon_sym_PLUS] = ACTIONS(1597),
    [anon_sym_STAR] = ACTIONS(1599),
    [anon_sym_SLASH] = ACTIONS(1597),
    [anon_sym_PERCENT] = ACTIONS(1599),
    [anon_sym_PIPE_PIPE] = ACTIONS(1599),
    [anon_sym_AMP_AMP] = ACTIONS(1599),
    [anon_sym_PIPE] = ACTIONS(1597),
    [anon_sym_CARET] = ACTIONS(1599),
    [anon_sym_AMP] = ACTIONS(1597),
    [anon_sym_EQ_EQ] = ACTIONS(1599),
    [anon_sym_BANG_EQ] = ACTIONS(1599),
    [anon_sym_LT_GT] = ACTIONS(1599),
    [anon_sym_GT] = ACTIONS(1597),
    [anon_sym_GT_EQ] = ACTIONS(1599),
    [anon_sym_LT_EQ] = ACTIONS(1599),
    [anon_sym_LT] = ACTIONS(1597),
    [anon_sym_LT_LT] = ACTIONS(1599),
    [anon_sym_GT_GT] = ACTIONS(1599),
    [anon_sym_DASH_DASH] = ACTIONS(1599),
    [anon_sym_PLUS_PLUS] = ACTIONS(1599),
    [anon_sym_DOT] = ACTIONS(1597),
    [anon_sym_DQUOTE] = ACTIONS(1599),
    [anon_sym_SQUOTE] = ACTIONS(1599),
    [sym_number_literal] = ACTIONS(1599),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1599),
  },
  [STATE(195)] = {
    [sym_identifier] = ACTIONS(1503),
    [sym_define_call_keyword] = ACTIONS(1503),
    [sym_define_function_keyword] = ACTIONS(1503),
    [sym_define_library_function_keyword] = ACTIONS(1503),
    [sym_define_module_keyword] = ACTIONS(1503),
    [sym_char_keyword] = ACTIONS(1503),
    [sym_widechar_keyword] = ACTIONS(1503),
    [sym_integer_keyword] = ACTIONS(1503),
    [sym_sinteger_keyword] = ACTIONS(1503),
    [sym_long_keyword] = ACTIONS(1503),
    [sym_slong_keyword] = ACTIONS(1503),
    [sym_float_keyword] = ACTIONS(1503),
    [sym_double_keyword] = ACTIONS(1503),
    [sym_variant_keyword] = ACTIONS(1503),
    [sym_variantarray_keyword] = ACTIONS(1503),
    [sym_local_var_keyword] = ACTIONS(1503),
    [sym_stack_var_keyword] = ACTIONS(1503),
    [sym_constant_keyword] = ACTIONS(1503),
    [sym_volatile_keyword] = ACTIONS(1503),
    [sym_non_volatile_keyword] = ACTIONS(1503),
    [sym_persistent_keyword] = ACTIONS(1503),
    [sym_dev_keyword] = ACTIONS(1503),
    [sym_devlev_keyword] = ACTIONS(1503),
    [sym_devchan_keyword] = ACTIONS(1503),
    [sym_if_keyword] = ACTIONS(1503),
    [sym_else_keyword] = ACTIONS(1503),
    [sym_switch_keyword] = ACTIONS(1503),
    [sym_case_keyword] = ACTIONS(1503),
    [sym_default_keyword] = ACTIONS(1503),
    [sym_while_keyword] = ACTIONS(1503),
    [sym_for_keyword] = ACTIONS(1503),
    [sym_break_keyword] = ACTIONS(1503),
    [sym_continue_keyword] = ACTIONS(1503),
    [sym_return_keyword] = ACTIONS(1503),
    [sym_select_keyword] = ACTIONS(1503),
    [sym_active_keyword] = ACTIONS(1503),
    [sym_struct_keyword] = ACTIONS(1503),
    [sym_structure_keyword] = ACTIONS(1503),
    [aux_sym_on_keyword_token1] = ACTIONS(1503),
    [aux_sym_off_keyword_token1] = ACTIONS(1503),
    [sym_send_level_keyword] = ACTIONS(1503),
    [sym_send_string_keyword] = ACTIONS(1503),
    [sym_send_command_keyword] = ACTIONS(1503),
    [sym_clear_buffer_keyword] = ACTIONS(1503),
    [sym_create_buffer_keyword] = ACTIONS(1503),
    [sym_create_multi_buffer_keyword] = ACTIONS(1503),
    [sym_call_keyword] = ACTIONS(1503),
    [sym_system_call_keyword] = ACTIONS(1503),
    [sym_devchan_to_keyword] = ACTIONS(1503),
    [sym_devchan_min_to_keyword] = ACTIONS(1503),
    [sym_devchan_total_off_keyword] = ACTIONS(1503),
    [sym_devchan_pulse_keyword] = ACTIONS(1503),
    [sym_wait_keyword] = ACTIONS(1503),
    [sym_cancel_wait_keyword] = ACTIONS(1503),
    [sym_wait_until_keyword] = ACTIONS(1503),
    [sym_cancel_wait_until_keyword] = ACTIONS(1503),
    [sym_cancel_all_wait_keyword] = ACTIONS(1503),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1503),
    [sym_and] = ACTIONS(1503),
    [sym_or] = ACTIONS(1503),
    [sym_not] = ACTIONS(1503),
    [sym_xor] = ACTIONS(1503),
    [sym_band] = ACTIONS(1503),
    [sym_bor] = ACTIONS(1503),
    [sym_bxor] = ACTIONS(1503),
    [sym_bnot] = ACTIONS(1503),
    [sym_lshift] = ACTIONS(1503),
    [sym_rshift] = ACTIONS(1503),
    [sym___file__] = ACTIONS(1503),
    [sym___date__] = ACTIONS(1503),
    [sym___ldate__] = ACTIONS(1503),
    [sym___time__] = ACTIONS(1503),
    [sym___line__] = ACTIONS(1503),
    [sym___name__] = ACTIONS(1503),
    [sym___netlinx__] = ACTIONS(1503),
    [sym_netlinx_axi_version] = ACTIONS(1503),
    [sym_url_flg_tcp] = ACTIONS(1503),
    [sym_url_flg_accinfopresent] = ACTIONS(1503),
    [sym_url_flg_temp] = ACTIONS(1503),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1503),
    [sym_url_flg_stat_mask] = ACTIONS(1503),
    [sym_url_flg_stat_lookup] = ACTIONS(1503),
    [sym_url_flg_stat_connecting] = ACTIONS(1503),
    [sym_url_flg_stat_waiting] = ACTIONS(1503),
    [sym_url_flg_stat_connected] = ACTIONS(1503),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1503),
    [sym_first_virtual_device] = ACTIONS(1503),
    [sym_first_local_port] = ACTIONS(1503),
    [sym_dynamic_virtual_device] = ACTIONS(1503),
    [sym_timeline_once] = ACTIONS(1503),
    [sym_timeline_repeat] = ACTIONS(1503),
    [sym_timeline_absolute] = ACTIONS(1503),
    [sym_timeline_relative] = ACTIONS(1503),
    [sym_do_push_timed_infinite] = ACTIONS(1503),
    [sym_source_type_no_address] = ACTIONS(1503),
    [sym_source_type_neuron_id] = ACTIONS(1503),
    [sym_source_type_ip_address] = ACTIONS(1503),
    [sym_source_type_axlink] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1503),
    [sym_source_type_ip_socket_address] = ACTIONS(1503),
    [sym_source_type_rs232] = ACTIONS(1503),
    [sym_source_type_internal] = ACTIONS(1503),
    [sym_source_type_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1503),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1503),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1503),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1503),
    [sym_true] = ACTIONS(1503),
    [sym_false] = ACTIONS(1503),
    [sym_file_read_only] = ACTIONS(1503),
    [sym_file_rw_new] = ACTIONS(1503),
    [sym_file_rw_append] = ACTIONS(1503),
    [sym_ip_tcp] = ACTIONS(1503),
    [sym_ip_udp] = ACTIONS(1503),
    [sym_ip_udp_2way] = ACTIONS(1503),
    [sym_xml_encode_types] = ACTIONS(1503),
    [sym_xml_encode_char_as_list] = ACTIONS(1503),
    [sym_xml_encode_le] = ACTIONS(1503),
    [sym_xml_decode_types] = ACTIONS(1503),
    [sym_xml_decode_no_preserve] = ACTIONS(1503),
    [sym_duet_dev_type_display_device] = ACTIONS(1503),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_io_device] = ACTIONS(1503),
    [sym_duet_dev_type_relay_device] = ACTIONS(1503),
    [sym_duet_dev_type_ups] = ACTIONS(1503),
    [sym_duet_dev_type_amplifier] = ACTIONS(1503),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1503),
    [sym_duet_dev_type_camera] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1503),
    [sym_duet_dev_type_dss] = ACTIONS(1503),
    [sym_duet_dev_type_dvr] = ACTIONS(1503),
    [sym_duet_dev_type_disc_device] = ACTIONS(1503),
    [sym_duet_dev_type_document_camera] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1503),
    [sym_duet_dev_type_hvac] = ACTIONS(1503),
    [sym_duet_dev_type_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_light] = ACTIONS(1503),
    [sym_duet_dev_type_monitor] = ACTIONS(1503),
    [sym_duet_dev_type_motor] = ACTIONS(1503),
    [sym_duet_dev_type_multi_window] = ACTIONS(1503),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1503),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1503),
    [sym_duet_dev_type_receiver] = ACTIONS(1503),
    [sym_duet_dev_type_security_system] = ACTIONS(1503),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1503),
    [sym_duet_dev_type_settop_box] = ACTIONS(1503),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1503),
    [sym_duet_dev_type_switcher] = ACTIONS(1503),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_tv] = ACTIONS(1503),
    [sym_duet_dev_type_utility] = ACTIONS(1503),
    [sym_duet_dev_type_vcr] = ACTIONS(1503),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_video_processor] = ACTIONS(1503),
    [sym_duet_dev_type_video_projector] = ACTIONS(1503),
    [sym_duet_dev_type_video_wall] = ACTIONS(1503),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1503),
    [sym_duet_dev_type_weather] = ACTIONS(1503),
    [sym_duet_dev_not_polled] = ACTIONS(1503),
    [sym_duet_dev_polled] = ACTIONS(1503),
    [sym_clkmgr_mode_network] = ACTIONS(1503),
    [sym_clkmgr_mode_standalone] = ACTIONS(1503),
    [sym_normal_standby] = ACTIONS(1503),
    [sym_normal_wake] = ACTIONS(1503),
    [sym_ip_multicast_ttl_option] = ACTIONS(1503),
    [sym_ip_ttl_subnet] = ACTIONS(1503),
    [sym_ip_ttl_site] = ACTIONS(1503),
    [sym_ip_ttl_region] = ACTIONS(1503),
    [sym_ip_ttl_continent] = ACTIONS(1503),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1503),
    [sym_ip_nodelay_on] = ACTIONS(1503),
    [sym_ip_nodelay_off] = ACTIONS(1503),
    [sym_amx_error] = ACTIONS(1503),
    [sym_amx_warning] = ACTIONS(1503),
    [sym_amx_info] = ACTIONS(1503),
    [sym_amx_debug] = ACTIONS(1503),
    [sym_netlinx_login_success] = ACTIONS(1503),
    [sym_netlinx_login_fail] = ACTIONS(1503),
    [sym_netlinx_logout] = ACTIONS(1503),
    [sym_null_str] = ACTIONS(1503),
    [sym_smtp_address] = ACTIONS(1503),
    [sym_smtp_port_number] = ACTIONS(1503),
    [sym_smtp_username] = ACTIONS(1503),
    [sym_smtp_password] = ACTIONS(1503),
    [sym_smtp_require_tls] = ACTIONS(1503),
    [sym_smtp_from] = ACTIONS(1503),
    [sym_smtp_tls_true] = ACTIONS(1503),
    [sym_smtp_tls_false] = ACTIONS(1503),
    [sym_listview_on_row_select_event] = ACTIONS(1503),
    [sym_data_structure_unknown] = ACTIONS(1503),
    [sym_data_structure_datafeed] = ACTIONS(1503),
    [sym_data_structure_datarecordset] = ACTIONS(1503),
    [sym_data_structure_datarecord] = ACTIONS(1503),
    [sym_data_structure_datafield] = ACTIONS(1503),
    [sym_data_max_content_field_count] = ACTIONS(1503),
    [sym_data_max_metadata_field_count] = ACTIONS(1503),
    [sym_data_max_id_length] = ACTIONS(1503),
    [sym_data_max_name_length] = ACTIONS(1503),
    [sym_data_max_description_length] = ACTIONS(1503),
    [sym_data_max_type_length] = ACTIONS(1503),
    [sym_data_max_format_length] = ACTIONS(1503),
    [sym_data_max_source_length] = ACTIONS(1503),
    [sym_data_max_value_length] = ACTIONS(1503),
    [sym_data_max_label_length] = ACTIONS(1503),
    [sym_data_max_url_length] = ACTIONS(1503),
    [sym_max_last_login_info_length] = ACTIONS(1503),
    [sym_data_type_unknown] = ACTIONS(1503),
    [sym_data_type_string] = ACTIONS(1503),
    [sym_data_type_datetime] = ACTIONS(1503),
    [sym_data_type_date] = ACTIONS(1503),
    [sym_data_type_time] = ACTIONS(1503),
    [sym_data_type_image] = ACTIONS(1503),
    [sym_data_format_url] = ACTIONS(1503),
    [sym_data_format_phone] = ACTIONS(1503),
    [sym_data_format_email] = ACTIONS(1503),
    [sym_data_format_iso8601] = ACTIONS(1503),
    [sym_auditlog_priv] = ACTIONS(1503),
    [sym_configuration_priv] = ACTIONS(1503),
    [sym_deviceconfig_priv] = ACTIONS(1503),
    [sym_ftp_priv] = ACTIONS(1503),
    [sym_http_priv] = ACTIONS(1503),
    [sym_networkconfig_priv] = ACTIONS(1503),
    [sym_programport_priv] = ACTIONS(1503),
    [sym_remoteui_priv] = ACTIONS(1503),
    [sym_securitycontrol_priv] = ACTIONS(1503),
    [sym_softwaremanagement_priv] = ACTIONS(1503),
    [sym_terminal_priv] = ACTIONS(1503),
    [sym_tpadmin_priv] = ACTIONS(1503),
    [sym_user1_priv] = ACTIONS(1503),
    [sym_user2_priv] = ACTIONS(1503),
    [sym_user3_priv] = ACTIONS(1503),
    [sym_user4_priv] = ACTIONS(1503),
    [sym_usermanagement_priv] = ACTIONS(1503),
    [sym_valid_account] = ACTIONS(1503),
    [sym_err_name_invalid] = ACTIONS(1503),
    [sym_err_password_invalid] = ACTIONS(1503),
    [sym_err_invalid_account] = ACTIONS(1503),
    [sym_err_no_authorization] = ACTIONS(1503),
    [sym_err_invalid_parameter] = ACTIONS(1503),
    [sym_err_user_locked_out] = ACTIONS(1503),
    [sym_err_user_account_expired] = ACTIONS(1503),
    [sym_err_general_error] = ACTIONS(1503),
    [sym_err_authentication_error] = ACTIONS(1503),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1503),
    [sym_tls_validate_certificate] = ACTIONS(1503),
    [sym_tdata] = ACTIONS(1503),
    [sym_tchannel] = ACTIONS(1503),
    [sym_tlevel] = ACTIONS(1503),
    [sym_tbutton] = ACTIONS(1503),
    [sym_ttimeline] = ACTIONS(1503),
    [sym_tcustom] = ACTIONS(1503),
    [sym_url_struct] = ACTIONS(1503),
    [sym_dns_struct] = ACTIONS(1503),
    [sym_ip_address_struct] = ACTIONS(1503),
    [sym_dev_info_struct] = ACTIONS(1503),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1503),
    [sym_data_feed] = ACTIONS(1503),
    [sym_data_field] = ACTIONS(1503),
    [sym_data_record] = ACTIONS(1503),
    [sym_wc_data_feed] = ACTIONS(1503),
    [sym_wc_data_field] = ACTIONS(1503),
    [sym_wc_data_record] = ACTIONS(1503),
    [sym_last_login_info] = ACTIONS(1503),
    [sym____reserved___] = ACTIONS(1503),
    [sym_date] = ACTIONS(1503),
    [sym_day] = ACTIONS(1503),
    [sym_time] = ACTIONS(1503),
    [sym_ldate] = ACTIONS(1503),
    [sym_push_device] = ACTIONS(1503),
    [sym_push_channel] = ACTIONS(1503),
    [sym_push_devchan] = ACTIONS(1503),
    [sym_release_device] = ACTIONS(1503),
    [sym_release_channel] = ACTIONS(1503),
    [sym_release_devchan] = ACTIONS(1503),
    [sym_master_slot] = ACTIONS(1503),
    [sym_get_pulse_time] = ACTIONS(1503),
    [sym_get_timer] = ACTIONS(1503),
    [sym_master_sn] = ACTIONS(1503),
    [sym_system_number] = ACTIONS(1503),
    [sym_button] = ACTIONS(1503),
    [sym_dv_channel] = ACTIONS(1503),
    [sym_data] = ACTIONS(1503),
    [sym_level] = ACTIONS(1503),
    [sym_timeline] = ACTIONS(1503),
    [sym_channel] = ACTIONS(1503),
    [sym_custom] = ACTIONS(1503),
    [sym_atoi] = ACTIONS(1503),
    [sym_atol] = ACTIONS(1503),
    [sym_atof] = ACTIONS(1503),
    [sym_itoa] = ACTIONS(1503),
    [sym_itohex] = ACTIONS(1503),
    [sym_hextoi] = ACTIONS(1503),
    [sym_ftoa] = ACTIONS(1503),
    [sym_find_string] = ACTIONS(1503),
    [sym_left_string] = ACTIONS(1503),
    [sym_right_string] = ACTIONS(1503),
    [sym_mid_string] = ACTIONS(1503),
    [sym_remove_string] = ACTIONS(1503),
    [sym_get_buffer_string] = ACTIONS(1503),
    [sym_get_buffer_char] = ACTIONS(1503),
    [sym_get_multi_buffer_string] = ACTIONS(1503),
    [sym_length_string] = ACTIONS(1503),
    [sym_max_length_string] = ACTIONS(1503),
    [sym_set_length_string] = ACTIONS(1503),
    [sym_compare_string] = ACTIONS(1503),
    [sym_upper_string] = ACTIONS(1503),
    [sym_lower_string] = ACTIONS(1503),
    [sym_get_last] = ACTIONS(1503),
    [sym_date_to_day] = ACTIONS(1503),
    [sym_date_to_month] = ACTIONS(1503),
    [sym_date_to_year] = ACTIONS(1503),
    [sym_time_to_hour] = ACTIONS(1503),
    [sym_time_to_minute] = ACTIONS(1503),
    [sym_time_to_second] = ACTIONS(1503),
    [sym_day_of_week] = ACTIONS(1503),
    [sym_set_pulse_time] = ACTIONS(1503),
    [sym_set_timer] = ACTIONS(1503),
    [sym_file_open] = ACTIONS(1503),
    [sym_file_close] = ACTIONS(1503),
    [sym_file_read] = ACTIONS(1503),
    [sym_file_read_line] = ACTIONS(1503),
    [sym_file_write] = ACTIONS(1503),
    [sym_file_write_line] = ACTIONS(1503),
    [sym_file_copy] = ACTIONS(1503),
    [sym_file_delete] = ACTIONS(1503),
    [sym_file_rename] = ACTIONS(1503),
    [sym_file_dir] = ACTIONS(1503),
    [sym_file_getdir] = ACTIONS(1503),
    [sym_file_setdir] = ACTIONS(1503),
    [sym_file_create_dir] = ACTIONS(1503),
    [sym_file_remove_dir] = ACTIONS(1503),
    [sym_file_seek] = ACTIONS(1503),
    [sym_length_array] = ACTIONS(1503),
    [sym_max_length_array] = ACTIONS(1503),
    [sym_set_length_array] = ACTIONS(1503),
    [sym_ip_client_open] = ACTIONS(1503),
    [sym_ip_client_close] = ACTIONS(1503),
    [sym_ip_server_open] = ACTIONS(1503),
    [sym_ip_server_close] = ACTIONS(1503),
    [sym_ip_bound_client_open] = ACTIONS(1503),
    [sym_ip_mc_server_open] = ACTIONS(1503),
    [sym_ip_set_option] = ACTIONS(1503),
    [sym_ssh_client_open] = ACTIONS(1503),
    [sym_ssh_client_close] = ACTIONS(1503),
    [sym_tls_client_open] = ACTIONS(1503),
    [sym_tls_client_close] = ACTIONS(1503),
    [sym_do_push] = ACTIONS(1503),
    [sym_do_push_timed] = ACTIONS(1503),
    [sym_do_release] = ACTIONS(1503),
    [sym_device_id] = ACTIONS(1503),
    [sym_device_info] = ACTIONS(1503),
    [sym_device_id_string] = ACTIONS(1503),
    [sym_device_standby] = ACTIONS(1503),
    [sym_device_wake] = ACTIONS(1503),
    [sym_timeline_create] = ACTIONS(1503),
    [sym_timeline_kill] = ACTIONS(1503),
    [sym_timeline_reload] = ACTIONS(1503),
    [sym_timeline_pause] = ACTIONS(1503),
    [sym_timeline_restart] = ACTIONS(1503),
    [sym_timeline_set] = ACTIONS(1503),
    [sym_timeline_get] = ACTIONS(1503),
    [sym_timeline_active] = ACTIONS(1503),
    [sym_timeline_dynamic_id] = ACTIONS(1503),
    [sym_variable_to_string] = ACTIONS(1503),
    [sym_string_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_string] = ACTIONS(1503),
    [sym_type_cast] = ACTIONS(1503),
    [sym_variable_to_xml] = ACTIONS(1503),
    [sym_xml_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_xml] = ACTIONS(1503),
    [sym_redirect_string] = ACTIONS(1503),
    [sym_random_number] = ACTIONS(1503),
    [sym_get_unique_id] = ACTIONS(1503),
    [sym_get_system_number] = ACTIONS(1503),
    [sym_set_system_number] = ACTIONS(1503),
    [sym_get_serial_number] = ACTIONS(1503),
    [sym_get_master_build] = ACTIONS(1503),
    [sym_set_validation_code] = ACTIONS(1503),
    [sym_reboot] = ACTIONS(1503),
    [sym_load_duet_module] = ACTIONS(1503),
    [sym_unload_duet_module] = ACTIONS(1503),
    [sym_duet_mem_size_set] = ACTIONS(1503),
    [sym_duet_mem_size_get] = ACTIONS(1503),
    [sym_dynamic_application_device] = ACTIONS(1503),
    [sym_dynamic_polled_port] = ACTIONS(1503),
    [sym_static_port_binding] = ACTIONS(1503),
    [sym_static_ip_binding] = ACTIONS(1503),
    [sym_get_dns_list] = ACTIONS(1503),
    [sym_set_dns_list] = ACTIONS(1503),
    [sym_get_ip_address] = ACTIONS(1503),
    [sym_set_ip_address] = ACTIONS(1503),
    [sym_get_url_list] = ACTIONS(1503),
    [sym_add_url_entry] = ACTIONS(1503),
    [sym_delete_url_entry] = ACTIONS(1503),
    [sym_abs_value] = ACTIONS(1503),
    [sym_max_value] = ACTIONS(1503),
    [sym_min_value] = ACTIONS(1503),
    [sym_exp_value] = ACTIONS(1503),
    [sym_log_value] = ACTIONS(1503),
    [sym_log10_value] = ACTIONS(1503),
    [sym_power_value] = ACTIONS(1503),
    [sym_sqrt_value] = ACTIONS(1503),
    [sym_raw_be] = ACTIONS(1503),
    [sym_raw_le] = ACTIONS(1503),
    [sym_internal_queue_size_set] = ACTIONS(1503),
    [sym_internal_queue_size_get] = ACTIONS(1503),
    [sym_internal_threshold_set] = ACTIONS(1503),
    [sym_internal_threshold_get] = ACTIONS(1503),
    [sym_rebuilt_event] = ACTIONS(1503),
    [sym_do_custom_event] = ACTIONS(1503),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1503),
    [sym_clkmgr_set_clk_source] = ACTIONS(1503),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1503),
    [sym_clkmgr_get_timezone] = ACTIONS(1503),
    [sym_clkmgr_set_timezone] = ACTIONS(1503),
    [sym_clkmgr_get_resync_period] = ACTIONS(1503),
    [sym_clkmgr_set_resync_period] = ACTIONS(1503),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_timeservers] = ACTIONS(1503),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_set_log_level] = ACTIONS(1503),
    [sym_get_log_level] = ACTIONS(1503),
    [sym_amx_log] = ACTIONS(1503),
    [sym_smtp_server_config_set] = ACTIONS(1503),
    [sym_smtp_server_config_get] = ACTIONS(1503),
    [sym_smtp_send] = ACTIONS(1503),
    [sym_validate_netlinx_account] = ACTIONS(1503),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1503),
    [sym_authenticate_certificate] = ACTIONS(1503),
    [sym_audit_netlinx_session_event] = ACTIONS(1503),
    [sym_audit_netlinx_generic_event] = ACTIONS(1503),
    [sym_get_available_flash_disk_space] = ACTIONS(1503),
    [sym_get_max_flash_disk_space] = ACTIONS(1503),
    [sym_data_create_feed] = ACTIONS(1503),
    [sym_data_delete_feed] = ACTIONS(1503),
    [sym_data_publish_feed] = ACTIONS(1503),
    [sym_data_get_published_feed] = ACTIONS(1503),
    [sym_data_add_record] = ACTIONS(1503),
    [sym_data_get_event_record] = ACTIONS(1503),
    [sym__wc_data_create_feed] = ACTIONS(1503),
    [sym__wc_data_add_record] = ACTIONS(1503),
    [sym__wc_data_get_event_record] = ACTIONS(1503),
    [sym_format] = ACTIONS(1503),
    [sym_astro_clock] = ACTIONS(1503),
    [sym_set_outdoor_temperature] = ACTIONS(1503),
    [sym_set_virtual_level_count] = ACTIONS(1503),
    [sym_set_virtual_channel_count] = ACTIONS(1503),
    [sym_set_virtual_port_count] = ACTIONS(1503),
    [sym__wc_encode_int] = ACTIONS(1503),
    [sym__wc_decode_int] = ACTIONS(1503),
    [sym__wc_int] = ACTIONS(1503),
    [sym__wc_to_ch_int] = ACTIONS(1503),
    [sym__ch_to_wc_int] = ACTIONS(1503),
    [sym__wc_find_string_int] = ACTIONS(1503),
    [sym__wc_left_string_int] = ACTIONS(1503),
    [sym__wc_lower_string_int] = ACTIONS(1503),
    [sym__wc_mid_string_int] = ACTIONS(1503),
    [sym__wc_remove_string_int] = ACTIONS(1503),
    [sym__wc_right_string_int] = ACTIONS(1503),
    [sym__wc_upper_string_int] = ACTIONS(1503),
    [sym__wc_compare_string_int] = ACTIONS(1503),
    [sym__wc_get_buffer_char_int] = ACTIONS(1503),
    [sym__wc_get_buffer_string_int] = ACTIONS(1503),
    [sym__wc_concat_string_int] = ACTIONS(1503),
    [sym__wc_file_open_int] = ACTIONS(1503),
    [sym__wc_file_close_int] = ACTIONS(1503),
    [sym__wc_file_read_int] = ACTIONS(1503),
    [sym__wc_file_read_line_int] = ACTIONS(1503),
    [sym__wc_file_write_int] = ACTIONS(1503),
    [sym__wc_file_write_line_int] = ACTIONS(1503),
    [aux_sym_preproc_include_token1] = ACTIONS(1503),
    [aux_sym_preproc_define_token1] = ACTIONS(1505),
    [aux_sym_preproc_warn_token1] = ACTIONS(1505),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1505),
    [anon_sym_LBRACK] = ACTIONS(1505),
    [anon_sym_LBRACE] = ACTIONS(1505),
    [anon_sym_RBRACE] = ACTIONS(1505),
    [anon_sym_COLON] = ACTIONS(1505),
    [anon_sym_LPAREN] = ACTIONS(1503),
    [anon_sym_SEMI] = ACTIONS(1505),
    [anon_sym_BANG] = ACTIONS(1503),
    [anon_sym_TILDE] = ACTIONS(1505),
    [anon_sym_DASH] = ACTIONS(1503),
    [anon_sym_PLUS] = ACTIONS(1503),
    [anon_sym_STAR] = ACTIONS(1505),
    [anon_sym_SLASH] = ACTIONS(1503),
    [anon_sym_PERCENT] = ACTIONS(1505),
    [anon_sym_PIPE_PIPE] = ACTIONS(1505),
    [anon_sym_AMP_AMP] = ACTIONS(1505),
    [anon_sym_PIPE] = ACTIONS(1503),
    [anon_sym_CARET] = ACTIONS(1505),
    [anon_sym_AMP] = ACTIONS(1503),
    [anon_sym_EQ_EQ] = ACTIONS(1505),
    [anon_sym_BANG_EQ] = ACTIONS(1505),
    [anon_sym_LT_GT] = ACTIONS(1505),
    [anon_sym_GT] = ACTIONS(1503),
    [anon_sym_GT_EQ] = ACTIONS(1505),
    [anon_sym_LT_EQ] = ACTIONS(1505),
    [anon_sym_LT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1505),
    [anon_sym_GT_GT] = ACTIONS(1505),
    [anon_sym_DASH_DASH] = ACTIONS(1505),
    [anon_sym_PLUS_PLUS] = ACTIONS(1505),
    [anon_sym_DOT] = ACTIONS(1503),
    [anon_sym_DQUOTE] = ACTIONS(1505),
    [anon_sym_SQUOTE] = ACTIONS(1505),
    [sym_number_literal] = ACTIONS(1505),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1505),
  },
  [STATE(196)] = {
    [ts_builtin_sym_end] = ACTIONS(1665),
    [sym_identifier] = ACTIONS(1667),
    [sym_program_name_keyword] = ACTIONS(1667),
    [sym_module_name_keyword] = ACTIONS(1667),
    [sym_define_device_keyword] = ACTIONS(1667),
    [sym_define_combine_keyword] = ACTIONS(1667),
    [sym_define_constant_keyword] = ACTIONS(1667),
    [sym_define_type_keyword] = ACTIONS(1667),
    [sym_define_variable_keyword] = ACTIONS(1667),
    [sym_define_system_variable_keyword] = ACTIONS(1667),
    [sym_define_connect_level_keyword] = ACTIONS(1667),
    [sym_define_latching_keyword] = ACTIONS(1667),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1667),
    [sym_define_toggling_keyword] = ACTIONS(1667),
    [sym_define_call_keyword] = ACTIONS(1667),
    [sym_define_function_keyword] = ACTIONS(1667),
    [sym_define_library_function_keyword] = ACTIONS(1667),
    [sym_define_start_keyword] = ACTIONS(1667),
    [sym_define_event_keyword] = ACTIONS(1667),
    [sym_define_program_keyword] = ACTIONS(1667),
    [sym_define_module_keyword] = ACTIONS(1667),
    [sym_char_keyword] = ACTIONS(1667),
    [sym_widechar_keyword] = ACTIONS(1667),
    [sym_integer_keyword] = ACTIONS(1667),
    [sym_sinteger_keyword] = ACTIONS(1667),
    [sym_long_keyword] = ACTIONS(1667),
    [sym_slong_keyword] = ACTIONS(1667),
    [sym_float_keyword] = ACTIONS(1667),
    [sym_double_keyword] = ACTIONS(1667),
    [sym_variant_keyword] = ACTIONS(1667),
    [sym_variantarray_keyword] = ACTIONS(1667),
    [sym_local_var_keyword] = ACTIONS(1667),
    [sym_stack_var_keyword] = ACTIONS(1667),
    [sym_constant_keyword] = ACTIONS(1667),
    [sym_volatile_keyword] = ACTIONS(1667),
    [sym_non_volatile_keyword] = ACTIONS(1667),
    [sym_persistent_keyword] = ACTIONS(1667),
    [sym_dev_keyword] = ACTIONS(1667),
    [sym_devlev_keyword] = ACTIONS(1667),
    [sym_devchan_keyword] = ACTIONS(1667),
    [sym_if_keyword] = ACTIONS(1667),
    [sym_else_keyword] = ACTIONS(1667),
    [sym_switch_keyword] = ACTIONS(1667),
    [sym_case_keyword] = ACTIONS(1667),
    [sym_default_keyword] = ACTIONS(1667),
    [sym_while_keyword] = ACTIONS(1667),
    [sym_for_keyword] = ACTIONS(1667),
    [sym_break_keyword] = ACTIONS(1667),
    [sym_continue_keyword] = ACTIONS(1667),
    [sym_return_keyword] = ACTIONS(1667),
    [sym_select_keyword] = ACTIONS(1667),
    [sym_active_keyword] = ACTIONS(1667),
    [sym_struct_keyword] = ACTIONS(1667),
    [sym_structure_keyword] = ACTIONS(1667),
    [sym_button_event_keyword] = ACTIONS(1667),
    [sym_channel_event_keyword] = ACTIONS(1667),
    [sym_level_event_keyword] = ACTIONS(1667),
    [sym_data_event_keyword] = ACTIONS(1667),
    [sym_timeline_event_keyword] = ACTIONS(1667),
    [sym_custom_event_keyword] = ACTIONS(1667),
    [sym_push_keyword] = ACTIONS(1667),
    [sym_release_keyword] = ACTIONS(1667),
    [aux_sym_on_keyword_token1] = ACTIONS(1667),
    [aux_sym_off_keyword_token1] = ACTIONS(1667),
    [sym_send_level_keyword] = ACTIONS(1667),
    [sym_send_string_keyword] = ACTIONS(1667),
    [sym_send_command_keyword] = ACTIONS(1667),
    [sym_clear_buffer_keyword] = ACTIONS(1667),
    [sym_create_buffer_keyword] = ACTIONS(1667),
    [sym_create_multi_buffer_keyword] = ACTIONS(1667),
    [sym_call_keyword] = ACTIONS(1667),
    [sym_system_call_keyword] = ACTIONS(1667),
    [sym_devchan_to_keyword] = ACTIONS(1667),
    [sym_devchan_min_to_keyword] = ACTIONS(1667),
    [sym_devchan_total_off_keyword] = ACTIONS(1667),
    [sym_devchan_pulse_keyword] = ACTIONS(1667),
    [sym_wait_keyword] = ACTIONS(1667),
    [sym_cancel_wait_keyword] = ACTIONS(1667),
    [sym_wait_until_keyword] = ACTIONS(1667),
    [sym_cancel_wait_until_keyword] = ACTIONS(1667),
    [sym_cancel_all_wait_keyword] = ACTIONS(1667),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1667),
    [sym_not] = ACTIONS(1667),
    [sym_bnot] = ACTIONS(1667),
    [sym___file__] = ACTIONS(1667),
    [sym___date__] = ACTIONS(1667),
    [sym___ldate__] = ACTIONS(1667),
    [sym___time__] = ACTIONS(1667),
    [sym___line__] = ACTIONS(1667),
    [sym___name__] = ACTIONS(1667),
    [sym___netlinx__] = ACTIONS(1667),
    [sym_netlinx_axi_version] = ACTIONS(1667),
    [sym_url_flg_tcp] = ACTIONS(1667),
    [sym_url_flg_accinfopresent] = ACTIONS(1667),
    [sym_url_flg_temp] = ACTIONS(1667),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1667),
    [sym_url_flg_stat_mask] = ACTIONS(1667),
    [sym_url_flg_stat_lookup] = ACTIONS(1667),
    [sym_url_flg_stat_connecting] = ACTIONS(1667),
    [sym_url_flg_stat_waiting] = ACTIONS(1667),
    [sym_url_flg_stat_connected] = ACTIONS(1667),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1667),
    [sym_first_virtual_device] = ACTIONS(1667),
    [sym_first_local_port] = ACTIONS(1667),
    [sym_dynamic_virtual_device] = ACTIONS(1667),
    [sym_timeline_once] = ACTIONS(1667),
    [sym_timeline_repeat] = ACTIONS(1667),
    [sym_timeline_absolute] = ACTIONS(1667),
    [sym_timeline_relative] = ACTIONS(1667),
    [sym_do_push_timed_infinite] = ACTIONS(1667),
    [sym_source_type_no_address] = ACTIONS(1667),
    [sym_source_type_neuron_id] = ACTIONS(1667),
    [sym_source_type_ip_address] = ACTIONS(1667),
    [sym_source_type_axlink] = ACTIONS(1667),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1667),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1667),
    [sym_source_type_ip_socket_address] = ACTIONS(1667),
    [sym_source_type_rs232] = ACTIONS(1667),
    [sym_source_type_internal] = ACTIONS(1667),
    [sym_source_type_mac_address] = ACTIONS(1667),
    [sym_source_type_ipv4_port] = ACTIONS(1667),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1667),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1667),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1667),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1667),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1667),
    [sym_true] = ACTIONS(1667),
    [sym_false] = ACTIONS(1667),
    [sym_file_read_only] = ACTIONS(1667),
    [sym_file_rw_new] = ACTIONS(1667),
    [sym_file_rw_append] = ACTIONS(1667),
    [sym_ip_tcp] = ACTIONS(1667),
    [sym_ip_udp] = ACTIONS(1667),
    [sym_ip_udp_2way] = ACTIONS(1667),
    [sym_xml_encode_types] = ACTIONS(1667),
    [sym_xml_encode_char_as_list] = ACTIONS(1667),
    [sym_xml_encode_le] = ACTIONS(1667),
    [sym_xml_decode_types] = ACTIONS(1667),
    [sym_xml_decode_no_preserve] = ACTIONS(1667),
    [sym_duet_dev_type_display_device] = ACTIONS(1667),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1667),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1667),
    [sym_duet_dev_type_io_device] = ACTIONS(1667),
    [sym_duet_dev_type_relay_device] = ACTIONS(1667),
    [sym_duet_dev_type_ups] = ACTIONS(1667),
    [sym_duet_dev_type_amplifier] = ACTIONS(1667),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1667),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1667),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1667),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1667),
    [sym_duet_dev_type_camera] = ACTIONS(1667),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1667),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1667),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1667),
    [sym_duet_dev_type_dss] = ACTIONS(1667),
    [sym_duet_dev_type_dvr] = ACTIONS(1667),
    [sym_duet_dev_type_disc_device] = ACTIONS(1667),
    [sym_duet_dev_type_document_camera] = ACTIONS(1667),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1667),
    [sym_duet_dev_type_hvac] = ACTIONS(1667),
    [sym_duet_dev_type_keypad] = ACTIONS(1667),
    [sym_duet_dev_type_light] = ACTIONS(1667),
    [sym_duet_dev_type_monitor] = ACTIONS(1667),
    [sym_duet_dev_type_motor] = ACTIONS(1667),
    [sym_duet_dev_type_multi_window] = ACTIONS(1667),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1667),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1667),
    [sym_duet_dev_type_receiver] = ACTIONS(1667),
    [sym_duet_dev_type_security_system] = ACTIONS(1667),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1667),
    [sym_duet_dev_type_settop_box] = ACTIONS(1667),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1667),
    [sym_duet_dev_type_switcher] = ACTIONS(1667),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1667),
    [sym_duet_dev_type_tv] = ACTIONS(1667),
    [sym_duet_dev_type_utility] = ACTIONS(1667),
    [sym_duet_dev_type_vcr] = ACTIONS(1667),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1667),
    [sym_duet_dev_type_video_processor] = ACTIONS(1667),
    [sym_duet_dev_type_video_projector] = ACTIONS(1667),
    [sym_duet_dev_type_video_wall] = ACTIONS(1667),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1667),
    [sym_duet_dev_type_weather] = ACTIONS(1667),
    [sym_duet_dev_not_polled] = ACTIONS(1667),
    [sym_duet_dev_polled] = ACTIONS(1667),
    [sym_clkmgr_mode_network] = ACTIONS(1667),
    [sym_clkmgr_mode_standalone] = ACTIONS(1667),
    [sym_normal_standby] = ACTIONS(1667),
    [sym_normal_wake] = ACTIONS(1667),
    [sym_ip_multicast_ttl_option] = ACTIONS(1667),
    [sym_ip_ttl_subnet] = ACTIONS(1667),
    [sym_ip_ttl_site] = ACTIONS(1667),
    [sym_ip_ttl_region] = ACTIONS(1667),
    [sym_ip_ttl_continent] = ACTIONS(1667),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1667),
    [sym_ip_nodelay_on] = ACTIONS(1667),
    [sym_ip_nodelay_off] = ACTIONS(1667),
    [sym_amx_error] = ACTIONS(1667),
    [sym_amx_warning] = ACTIONS(1667),
    [sym_amx_info] = ACTIONS(1667),
    [sym_amx_debug] = ACTIONS(1667),
    [sym_netlinx_login_success] = ACTIONS(1667),
    [sym_netlinx_login_fail] = ACTIONS(1667),
    [sym_netlinx_logout] = ACTIONS(1667),
    [sym_null_str] = ACTIONS(1667),
    [sym_smtp_address] = ACTIONS(1667),
    [sym_smtp_port_number] = ACTIONS(1667),
    [sym_smtp_username] = ACTIONS(1667),
    [sym_smtp_password] = ACTIONS(1667),
    [sym_smtp_require_tls] = ACTIONS(1667),
    [sym_smtp_from] = ACTIONS(1667),
    [sym_smtp_tls_true] = ACTIONS(1667),
    [sym_smtp_tls_false] = ACTIONS(1667),
    [sym_listview_on_row_select_event] = ACTIONS(1667),
    [sym_data_structure_unknown] = ACTIONS(1667),
    [sym_data_structure_datafeed] = ACTIONS(1667),
    [sym_data_structure_datarecordset] = ACTIONS(1667),
    [sym_data_structure_datarecord] = ACTIONS(1667),
    [sym_data_structure_datafield] = ACTIONS(1667),
    [sym_data_max_content_field_count] = ACTIONS(1667),
    [sym_data_max_metadata_field_count] = ACTIONS(1667),
    [sym_data_max_id_length] = ACTIONS(1667),
    [sym_data_max_name_length] = ACTIONS(1667),
    [sym_data_max_description_length] = ACTIONS(1667),
    [sym_data_max_type_length] = ACTIONS(1667),
    [sym_data_max_format_length] = ACTIONS(1667),
    [sym_data_max_source_length] = ACTIONS(1667),
    [sym_data_max_value_length] = ACTIONS(1667),
    [sym_data_max_label_length] = ACTIONS(1667),
    [sym_data_max_url_length] = ACTIONS(1667),
    [sym_max_last_login_info_length] = ACTIONS(1667),
    [sym_data_type_unknown] = ACTIONS(1667),
    [sym_data_type_string] = ACTIONS(1667),
    [sym_data_type_datetime] = ACTIONS(1667),
    [sym_data_type_date] = ACTIONS(1667),
    [sym_data_type_time] = ACTIONS(1667),
    [sym_data_type_image] = ACTIONS(1667),
    [sym_data_format_url] = ACTIONS(1667),
    [sym_data_format_phone] = ACTIONS(1667),
    [sym_data_format_email] = ACTIONS(1667),
    [sym_data_format_iso8601] = ACTIONS(1667),
    [sym_auditlog_priv] = ACTIONS(1667),
    [sym_configuration_priv] = ACTIONS(1667),
    [sym_deviceconfig_priv] = ACTIONS(1667),
    [sym_ftp_priv] = ACTIONS(1667),
    [sym_http_priv] = ACTIONS(1667),
    [sym_networkconfig_priv] = ACTIONS(1667),
    [sym_programport_priv] = ACTIONS(1667),
    [sym_remoteui_priv] = ACTIONS(1667),
    [sym_securitycontrol_priv] = ACTIONS(1667),
    [sym_softwaremanagement_priv] = ACTIONS(1667),
    [sym_terminal_priv] = ACTIONS(1667),
    [sym_tpadmin_priv] = ACTIONS(1667),
    [sym_user1_priv] = ACTIONS(1667),
    [sym_user2_priv] = ACTIONS(1667),
    [sym_user3_priv] = ACTIONS(1667),
    [sym_user4_priv] = ACTIONS(1667),
    [sym_usermanagement_priv] = ACTIONS(1667),
    [sym_valid_account] = ACTIONS(1667),
    [sym_err_name_invalid] = ACTIONS(1667),
    [sym_err_password_invalid] = ACTIONS(1667),
    [sym_err_invalid_account] = ACTIONS(1667),
    [sym_err_no_authorization] = ACTIONS(1667),
    [sym_err_invalid_parameter] = ACTIONS(1667),
    [sym_err_user_locked_out] = ACTIONS(1667),
    [sym_err_user_account_expired] = ACTIONS(1667),
    [sym_err_general_error] = ACTIONS(1667),
    [sym_err_authentication_error] = ACTIONS(1667),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1667),
    [sym_tls_validate_certificate] = ACTIONS(1667),
    [sym_tdata] = ACTIONS(1667),
    [sym_tchannel] = ACTIONS(1667),
    [sym_tlevel] = ACTIONS(1667),
    [sym_tbutton] = ACTIONS(1667),
    [sym_ttimeline] = ACTIONS(1667),
    [sym_tcustom] = ACTIONS(1667),
    [sym_url_struct] = ACTIONS(1667),
    [sym_dns_struct] = ACTIONS(1667),
    [sym_ip_address_struct] = ACTIONS(1667),
    [sym_dev_info_struct] = ACTIONS(1667),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1667),
    [sym_data_feed] = ACTIONS(1667),
    [sym_data_field] = ACTIONS(1667),
    [sym_data_record] = ACTIONS(1667),
    [sym_wc_data_feed] = ACTIONS(1667),
    [sym_wc_data_field] = ACTIONS(1667),
    [sym_wc_data_record] = ACTIONS(1667),
    [sym_last_login_info] = ACTIONS(1667),
    [sym____reserved___] = ACTIONS(1667),
    [sym_date] = ACTIONS(1667),
    [sym_day] = ACTIONS(1667),
    [sym_time] = ACTIONS(1667),
    [sym_ldate] = ACTIONS(1667),
    [sym_push_device] = ACTIONS(1667),
    [sym_push_channel] = ACTIONS(1667),
    [sym_push_devchan] = ACTIONS(1667),
    [sym_release_device] = ACTIONS(1667),
    [sym_release_channel] = ACTIONS(1667),
    [sym_release_devchan] = ACTIONS(1667),
    [sym_master_slot] = ACTIONS(1667),
    [sym_get_pulse_time] = ACTIONS(1667),
    [sym_get_timer] = ACTIONS(1667),
    [sym_master_sn] = ACTIONS(1667),
    [sym_system_number] = ACTIONS(1667),
    [sym_button] = ACTIONS(1667),
    [sym_dv_channel] = ACTIONS(1667),
    [sym_data] = ACTIONS(1667),
    [sym_level] = ACTIONS(1667),
    [sym_timeline] = ACTIONS(1667),
    [sym_channel] = ACTIONS(1667),
    [sym_custom] = ACTIONS(1667),
    [sym_atoi] = ACTIONS(1667),
    [sym_atol] = ACTIONS(1667),
    [sym_atof] = ACTIONS(1667),
    [sym_itoa] = ACTIONS(1667),
    [sym_itohex] = ACTIONS(1667),
    [sym_hextoi] = ACTIONS(1667),
    [sym_ftoa] = ACTIONS(1667),
    [sym_find_string] = ACTIONS(1667),
    [sym_left_string] = ACTIONS(1667),
    [sym_right_string] = ACTIONS(1667),
    [sym_mid_string] = ACTIONS(1667),
    [sym_remove_string] = ACTIONS(1667),
    [sym_get_buffer_string] = ACTIONS(1667),
    [sym_get_buffer_char] = ACTIONS(1667),
    [sym_get_multi_buffer_string] = ACTIONS(1667),
    [sym_length_string] = ACTIONS(1667),
    [sym_max_length_string] = ACTIONS(1667),
    [sym_set_length_string] = ACTIONS(1667),
    [sym_compare_string] = ACTIONS(1667),
    [sym_upper_string] = ACTIONS(1667),
    [sym_lower_string] = ACTIONS(1667),
    [sym_get_last] = ACTIONS(1667),
    [sym_date_to_day] = ACTIONS(1667),
    [sym_date_to_month] = ACTIONS(1667),
    [sym_date_to_year] = ACTIONS(1667),
    [sym_time_to_hour] = ACTIONS(1667),
    [sym_time_to_minute] = ACTIONS(1667),
    [sym_time_to_second] = ACTIONS(1667),
    [sym_day_of_week] = ACTIONS(1667),
    [sym_set_pulse_time] = ACTIONS(1667),
    [sym_set_timer] = ACTIONS(1667),
    [sym_file_open] = ACTIONS(1667),
    [sym_file_close] = ACTIONS(1667),
    [sym_file_read] = ACTIONS(1667),
    [sym_file_read_line] = ACTIONS(1667),
    [sym_file_write] = ACTIONS(1667),
    [sym_file_write_line] = ACTIONS(1667),
    [sym_file_copy] = ACTIONS(1667),
    [sym_file_delete] = ACTIONS(1667),
    [sym_file_rename] = ACTIONS(1667),
    [sym_file_dir] = ACTIONS(1667),
    [sym_file_getdir] = ACTIONS(1667),
    [sym_file_setdir] = ACTIONS(1667),
    [sym_file_create_dir] = ACTIONS(1667),
    [sym_file_remove_dir] = ACTIONS(1667),
    [sym_file_seek] = ACTIONS(1667),
    [sym_length_array] = ACTIONS(1667),
    [sym_max_length_array] = ACTIONS(1667),
    [sym_set_length_array] = ACTIONS(1667),
    [sym_ip_client_open] = ACTIONS(1667),
    [sym_ip_client_close] = ACTIONS(1667),
    [sym_ip_server_open] = ACTIONS(1667),
    [sym_ip_server_close] = ACTIONS(1667),
    [sym_ip_bound_client_open] = ACTIONS(1667),
    [sym_ip_mc_server_open] = ACTIONS(1667),
    [sym_ip_set_option] = ACTIONS(1667),
    [sym_ssh_client_open] = ACTIONS(1667),
    [sym_ssh_client_close] = ACTIONS(1667),
    [sym_tls_client_open] = ACTIONS(1667),
    [sym_tls_client_close] = ACTIONS(1667),
    [sym_do_push] = ACTIONS(1667),
    [sym_do_push_timed] = ACTIONS(1667),
    [sym_do_release] = ACTIONS(1667),
    [sym_device_id] = ACTIONS(1667),
    [sym_device_info] = ACTIONS(1667),
    [sym_device_id_string] = ACTIONS(1667),
    [sym_device_standby] = ACTIONS(1667),
    [sym_device_wake] = ACTIONS(1667),
    [sym_timeline_create] = ACTIONS(1667),
    [sym_timeline_kill] = ACTIONS(1667),
    [sym_timeline_reload] = ACTIONS(1667),
    [sym_timeline_pause] = ACTIONS(1667),
    [sym_timeline_restart] = ACTIONS(1667),
    [sym_timeline_set] = ACTIONS(1667),
    [sym_timeline_get] = ACTIONS(1667),
    [sym_timeline_active] = ACTIONS(1667),
    [sym_timeline_dynamic_id] = ACTIONS(1667),
    [sym_variable_to_string] = ACTIONS(1667),
    [sym_string_to_variable] = ACTIONS(1667),
    [sym_length_variable_to_string] = ACTIONS(1667),
    [sym_type_cast] = ACTIONS(1667),
    [sym_variable_to_xml] = ACTIONS(1667),
    [sym_xml_to_variable] = ACTIONS(1667),
    [sym_length_variable_to_xml] = ACTIONS(1667),
    [sym_redirect_string] = ACTIONS(1667),
    [sym_random_number] = ACTIONS(1667),
    [sym_get_unique_id] = ACTIONS(1667),
    [sym_get_system_number] = ACTIONS(1667),
    [sym_set_system_number] = ACTIONS(1667),
    [sym_get_serial_number] = ACTIONS(1667),
    [sym_get_master_build] = ACTIONS(1667),
    [sym_set_validation_code] = ACTIONS(1667),
    [sym_reboot] = ACTIONS(1667),
    [sym_load_duet_module] = ACTIONS(1667),
    [sym_unload_duet_module] = ACTIONS(1667),
    [sym_duet_mem_size_set] = ACTIONS(1667),
    [sym_duet_mem_size_get] = ACTIONS(1667),
    [sym_dynamic_application_device] = ACTIONS(1667),
    [sym_dynamic_polled_port] = ACTIONS(1667),
    [sym_static_port_binding] = ACTIONS(1667),
    [sym_static_ip_binding] = ACTIONS(1667),
    [sym_get_dns_list] = ACTIONS(1667),
    [sym_set_dns_list] = ACTIONS(1667),
    [sym_get_ip_address] = ACTIONS(1667),
    [sym_set_ip_address] = ACTIONS(1667),
    [sym_get_url_list] = ACTIONS(1667),
    [sym_add_url_entry] = ACTIONS(1667),
    [sym_delete_url_entry] = ACTIONS(1667),
    [sym_abs_value] = ACTIONS(1667),
    [sym_max_value] = ACTIONS(1667),
    [sym_min_value] = ACTIONS(1667),
    [sym_exp_value] = ACTIONS(1667),
    [sym_log_value] = ACTIONS(1667),
    [sym_log10_value] = ACTIONS(1667),
    [sym_power_value] = ACTIONS(1667),
    [sym_sqrt_value] = ACTIONS(1667),
    [sym_raw_be] = ACTIONS(1667),
    [sym_raw_le] = ACTIONS(1667),
    [sym_internal_queue_size_set] = ACTIONS(1667),
    [sym_internal_queue_size_get] = ACTIONS(1667),
    [sym_internal_threshold_set] = ACTIONS(1667),
    [sym_internal_threshold_get] = ACTIONS(1667),
    [sym_rebuilt_event] = ACTIONS(1667),
    [sym_do_custom_event] = ACTIONS(1667),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1667),
    [sym_clkmgr_set_clk_source] = ACTIONS(1667),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1667),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1667),
    [sym_clkmgr_get_timezone] = ACTIONS(1667),
    [sym_clkmgr_set_timezone] = ACTIONS(1667),
    [sym_clkmgr_get_resync_period] = ACTIONS(1667),
    [sym_clkmgr_set_resync_period] = ACTIONS(1667),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1667),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1667),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1667),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1667),
    [sym_clkmgr_get_timeservers] = ACTIONS(1667),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1667),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1667),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1667),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1667),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1667),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1667),
    [sym_set_log_level] = ACTIONS(1667),
    [sym_get_log_level] = ACTIONS(1667),
    [sym_amx_log] = ACTIONS(1667),
    [sym_smtp_server_config_set] = ACTIONS(1667),
    [sym_smtp_server_config_get] = ACTIONS(1667),
    [sym_smtp_send] = ACTIONS(1667),
    [sym_validate_netlinx_account] = ACTIONS(1667),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1667),
    [sym_authenticate_certificate] = ACTIONS(1667),
    [sym_audit_netlinx_session_event] = ACTIONS(1667),
    [sym_audit_netlinx_generic_event] = ACTIONS(1667),
    [sym_get_available_flash_disk_space] = ACTIONS(1667),
    [sym_get_max_flash_disk_space] = ACTIONS(1667),
    [sym_data_create_feed] = ACTIONS(1667),
    [sym_data_delete_feed] = ACTIONS(1667),
    [sym_data_publish_feed] = ACTIONS(1667),
    [sym_data_get_published_feed] = ACTIONS(1667),
    [sym_data_add_record] = ACTIONS(1667),
    [sym_data_get_event_record] = ACTIONS(1667),
    [sym__wc_data_create_feed] = ACTIONS(1667),
    [sym__wc_data_add_record] = ACTIONS(1667),
    [sym__wc_data_get_event_record] = ACTIONS(1667),
    [sym_format] = ACTIONS(1667),
    [sym_astro_clock] = ACTIONS(1667),
    [sym_set_outdoor_temperature] = ACTIONS(1667),
    [sym_set_virtual_level_count] = ACTIONS(1667),
    [sym_set_virtual_channel_count] = ACTIONS(1667),
    [sym_set_virtual_port_count] = ACTIONS(1667),
    [sym__wc_encode_int] = ACTIONS(1667),
    [sym__wc_decode_int] = ACTIONS(1667),
    [sym__wc_int] = ACTIONS(1667),
    [sym__wc_to_ch_int] = ACTIONS(1667),
    [sym__ch_to_wc_int] = ACTIONS(1667),
    [sym__wc_find_string_int] = ACTIONS(1667),
    [sym__wc_left_string_int] = ACTIONS(1667),
    [sym__wc_lower_string_int] = ACTIONS(1667),
    [sym__wc_mid_string_int] = ACTIONS(1667),
    [sym__wc_remove_string_int] = ACTIONS(1667),
    [sym__wc_right_string_int] = ACTIONS(1667),
    [sym__wc_upper_string_int] = ACTIONS(1667),
    [sym__wc_compare_string_int] = ACTIONS(1667),
    [sym__wc_get_buffer_char_int] = ACTIONS(1667),
    [sym__wc_get_buffer_string_int] = ACTIONS(1667),
    [sym__wc_concat_string_int] = ACTIONS(1667),
    [sym__wc_file_open_int] = ACTIONS(1667),
    [sym__wc_file_close_int] = ACTIONS(1667),
    [sym__wc_file_read_int] = ACTIONS(1667),
    [sym__wc_file_read_line_int] = ACTIONS(1667),
    [sym__wc_file_write_int] = ACTIONS(1667),
    [sym__wc_file_write_line_int] = ACTIONS(1667),
    [aux_sym_preproc_include_token1] = ACTIONS(1667),
    [aux_sym_preproc_define_token1] = ACTIONS(1665),
    [aux_sym_preproc_warn_token1] = ACTIONS(1665),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1665),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1665),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1665),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1665),
    [aux_sym_preproc_else_token1] = ACTIONS(1665),
    [anon_sym_COMMA] = ACTIONS(1665),
    [anon_sym_LBRACK] = ACTIONS(1665),
    [anon_sym_LBRACE] = ACTIONS(1665),
    [anon_sym_RBRACE] = ACTIONS(1665),
    [anon_sym_LPAREN] = ACTIONS(1667),
    [anon_sym_RPAREN] = ACTIONS(1665),
    [anon_sym_BANG] = ACTIONS(1665),
    [anon_sym_TILDE] = ACTIONS(1665),
    [anon_sym_DASH] = ACTIONS(1667),
    [anon_sym_PLUS] = ACTIONS(1667),
    [anon_sym_DASH_DASH] = ACTIONS(1665),
    [anon_sym_PLUS_PLUS] = ACTIONS(1665),
    [anon_sym_DQUOTE] = ACTIONS(1665),
    [anon_sym_SQUOTE] = ACTIONS(1665),
    [sym_number_literal] = ACTIONS(1665),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(197)] = {
    [ts_builtin_sym_end] = ACTIONS(1669),
    [sym_identifier] = ACTIONS(1671),
    [sym_program_name_keyword] = ACTIONS(1671),
    [sym_module_name_keyword] = ACTIONS(1671),
    [sym_define_device_keyword] = ACTIONS(1671),
    [sym_define_combine_keyword] = ACTIONS(1671),
    [sym_define_constant_keyword] = ACTIONS(1671),
    [sym_define_type_keyword] = ACTIONS(1671),
    [sym_define_variable_keyword] = ACTIONS(1671),
    [sym_define_system_variable_keyword] = ACTIONS(1671),
    [sym_define_connect_level_keyword] = ACTIONS(1671),
    [sym_define_latching_keyword] = ACTIONS(1671),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1671),
    [sym_define_toggling_keyword] = ACTIONS(1671),
    [sym_define_call_keyword] = ACTIONS(1671),
    [sym_define_function_keyword] = ACTIONS(1671),
    [sym_define_library_function_keyword] = ACTIONS(1671),
    [sym_define_start_keyword] = ACTIONS(1671),
    [sym_define_event_keyword] = ACTIONS(1671),
    [sym_define_program_keyword] = ACTIONS(1671),
    [sym_define_module_keyword] = ACTIONS(1671),
    [sym_char_keyword] = ACTIONS(1671),
    [sym_widechar_keyword] = ACTIONS(1671),
    [sym_integer_keyword] = ACTIONS(1671),
    [sym_sinteger_keyword] = ACTIONS(1671),
    [sym_long_keyword] = ACTIONS(1671),
    [sym_slong_keyword] = ACTIONS(1671),
    [sym_float_keyword] = ACTIONS(1671),
    [sym_double_keyword] = ACTIONS(1671),
    [sym_variant_keyword] = ACTIONS(1671),
    [sym_variantarray_keyword] = ACTIONS(1671),
    [sym_local_var_keyword] = ACTIONS(1671),
    [sym_stack_var_keyword] = ACTIONS(1671),
    [sym_constant_keyword] = ACTIONS(1671),
    [sym_volatile_keyword] = ACTIONS(1671),
    [sym_non_volatile_keyword] = ACTIONS(1671),
    [sym_persistent_keyword] = ACTIONS(1671),
    [sym_dev_keyword] = ACTIONS(1671),
    [sym_devlev_keyword] = ACTIONS(1671),
    [sym_devchan_keyword] = ACTIONS(1671),
    [sym_if_keyword] = ACTIONS(1671),
    [sym_else_keyword] = ACTIONS(1671),
    [sym_switch_keyword] = ACTIONS(1671),
    [sym_case_keyword] = ACTIONS(1671),
    [sym_default_keyword] = ACTIONS(1671),
    [sym_while_keyword] = ACTIONS(1671),
    [sym_for_keyword] = ACTIONS(1671),
    [sym_break_keyword] = ACTIONS(1671),
    [sym_continue_keyword] = ACTIONS(1671),
    [sym_return_keyword] = ACTIONS(1671),
    [sym_select_keyword] = ACTIONS(1671),
    [sym_active_keyword] = ACTIONS(1671),
    [sym_struct_keyword] = ACTIONS(1671),
    [sym_structure_keyword] = ACTIONS(1671),
    [sym_button_event_keyword] = ACTIONS(1671),
    [sym_channel_event_keyword] = ACTIONS(1671),
    [sym_level_event_keyword] = ACTIONS(1671),
    [sym_data_event_keyword] = ACTIONS(1671),
    [sym_timeline_event_keyword] = ACTIONS(1671),
    [sym_custom_event_keyword] = ACTIONS(1671),
    [sym_push_keyword] = ACTIONS(1671),
    [sym_release_keyword] = ACTIONS(1671),
    [aux_sym_on_keyword_token1] = ACTIONS(1671),
    [aux_sym_off_keyword_token1] = ACTIONS(1671),
    [sym_send_level_keyword] = ACTIONS(1671),
    [sym_send_string_keyword] = ACTIONS(1671),
    [sym_send_command_keyword] = ACTIONS(1671),
    [sym_clear_buffer_keyword] = ACTIONS(1671),
    [sym_create_buffer_keyword] = ACTIONS(1671),
    [sym_create_multi_buffer_keyword] = ACTIONS(1671),
    [sym_call_keyword] = ACTIONS(1671),
    [sym_system_call_keyword] = ACTIONS(1671),
    [sym_devchan_to_keyword] = ACTIONS(1671),
    [sym_devchan_min_to_keyword] = ACTIONS(1671),
    [sym_devchan_total_off_keyword] = ACTIONS(1671),
    [sym_devchan_pulse_keyword] = ACTIONS(1671),
    [sym_wait_keyword] = ACTIONS(1671),
    [sym_cancel_wait_keyword] = ACTIONS(1671),
    [sym_wait_until_keyword] = ACTIONS(1671),
    [sym_cancel_wait_until_keyword] = ACTIONS(1671),
    [sym_cancel_all_wait_keyword] = ACTIONS(1671),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1671),
    [sym_not] = ACTIONS(1671),
    [sym_bnot] = ACTIONS(1671),
    [sym___file__] = ACTIONS(1671),
    [sym___date__] = ACTIONS(1671),
    [sym___ldate__] = ACTIONS(1671),
    [sym___time__] = ACTIONS(1671),
    [sym___line__] = ACTIONS(1671),
    [sym___name__] = ACTIONS(1671),
    [sym___netlinx__] = ACTIONS(1671),
    [sym_netlinx_axi_version] = ACTIONS(1671),
    [sym_url_flg_tcp] = ACTIONS(1671),
    [sym_url_flg_accinfopresent] = ACTIONS(1671),
    [sym_url_flg_temp] = ACTIONS(1671),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1671),
    [sym_url_flg_stat_mask] = ACTIONS(1671),
    [sym_url_flg_stat_lookup] = ACTIONS(1671),
    [sym_url_flg_stat_connecting] = ACTIONS(1671),
    [sym_url_flg_stat_waiting] = ACTIONS(1671),
    [sym_url_flg_stat_connected] = ACTIONS(1671),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1671),
    [sym_first_virtual_device] = ACTIONS(1671),
    [sym_first_local_port] = ACTIONS(1671),
    [sym_dynamic_virtual_device] = ACTIONS(1671),
    [sym_timeline_once] = ACTIONS(1671),
    [sym_timeline_repeat] = ACTIONS(1671),
    [sym_timeline_absolute] = ACTIONS(1671),
    [sym_timeline_relative] = ACTIONS(1671),
    [sym_do_push_timed_infinite] = ACTIONS(1671),
    [sym_source_type_no_address] = ACTIONS(1671),
    [sym_source_type_neuron_id] = ACTIONS(1671),
    [sym_source_type_ip_address] = ACTIONS(1671),
    [sym_source_type_axlink] = ACTIONS(1671),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1671),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1671),
    [sym_source_type_ip_socket_address] = ACTIONS(1671),
    [sym_source_type_rs232] = ACTIONS(1671),
    [sym_source_type_internal] = ACTIONS(1671),
    [sym_source_type_mac_address] = ACTIONS(1671),
    [sym_source_type_ipv4_port] = ACTIONS(1671),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1671),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1671),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1671),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1671),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1671),
    [sym_true] = ACTIONS(1671),
    [sym_false] = ACTIONS(1671),
    [sym_file_read_only] = ACTIONS(1671),
    [sym_file_rw_new] = ACTIONS(1671),
    [sym_file_rw_append] = ACTIONS(1671),
    [sym_ip_tcp] = ACTIONS(1671),
    [sym_ip_udp] = ACTIONS(1671),
    [sym_ip_udp_2way] = ACTIONS(1671),
    [sym_xml_encode_types] = ACTIONS(1671),
    [sym_xml_encode_char_as_list] = ACTIONS(1671),
    [sym_xml_encode_le] = ACTIONS(1671),
    [sym_xml_decode_types] = ACTIONS(1671),
    [sym_xml_decode_no_preserve] = ACTIONS(1671),
    [sym_duet_dev_type_display_device] = ACTIONS(1671),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1671),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1671),
    [sym_duet_dev_type_io_device] = ACTIONS(1671),
    [sym_duet_dev_type_relay_device] = ACTIONS(1671),
    [sym_duet_dev_type_ups] = ACTIONS(1671),
    [sym_duet_dev_type_amplifier] = ACTIONS(1671),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1671),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1671),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1671),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1671),
    [sym_duet_dev_type_camera] = ACTIONS(1671),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1671),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1671),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1671),
    [sym_duet_dev_type_dss] = ACTIONS(1671),
    [sym_duet_dev_type_dvr] = ACTIONS(1671),
    [sym_duet_dev_type_disc_device] = ACTIONS(1671),
    [sym_duet_dev_type_document_camera] = ACTIONS(1671),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1671),
    [sym_duet_dev_type_hvac] = ACTIONS(1671),
    [sym_duet_dev_type_keypad] = ACTIONS(1671),
    [sym_duet_dev_type_light] = ACTIONS(1671),
    [sym_duet_dev_type_monitor] = ACTIONS(1671),
    [sym_duet_dev_type_motor] = ACTIONS(1671),
    [sym_duet_dev_type_multi_window] = ACTIONS(1671),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1671),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1671),
    [sym_duet_dev_type_receiver] = ACTIONS(1671),
    [sym_duet_dev_type_security_system] = ACTIONS(1671),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1671),
    [sym_duet_dev_type_settop_box] = ACTIONS(1671),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1671),
    [sym_duet_dev_type_switcher] = ACTIONS(1671),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1671),
    [sym_duet_dev_type_tv] = ACTIONS(1671),
    [sym_duet_dev_type_utility] = ACTIONS(1671),
    [sym_duet_dev_type_vcr] = ACTIONS(1671),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1671),
    [sym_duet_dev_type_video_processor] = ACTIONS(1671),
    [sym_duet_dev_type_video_projector] = ACTIONS(1671),
    [sym_duet_dev_type_video_wall] = ACTIONS(1671),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1671),
    [sym_duet_dev_type_weather] = ACTIONS(1671),
    [sym_duet_dev_not_polled] = ACTIONS(1671),
    [sym_duet_dev_polled] = ACTIONS(1671),
    [sym_clkmgr_mode_network] = ACTIONS(1671),
    [sym_clkmgr_mode_standalone] = ACTIONS(1671),
    [sym_normal_standby] = ACTIONS(1671),
    [sym_normal_wake] = ACTIONS(1671),
    [sym_ip_multicast_ttl_option] = ACTIONS(1671),
    [sym_ip_ttl_subnet] = ACTIONS(1671),
    [sym_ip_ttl_site] = ACTIONS(1671),
    [sym_ip_ttl_region] = ACTIONS(1671),
    [sym_ip_ttl_continent] = ACTIONS(1671),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1671),
    [sym_ip_nodelay_on] = ACTIONS(1671),
    [sym_ip_nodelay_off] = ACTIONS(1671),
    [sym_amx_error] = ACTIONS(1671),
    [sym_amx_warning] = ACTIONS(1671),
    [sym_amx_info] = ACTIONS(1671),
    [sym_amx_debug] = ACTIONS(1671),
    [sym_netlinx_login_success] = ACTIONS(1671),
    [sym_netlinx_login_fail] = ACTIONS(1671),
    [sym_netlinx_logout] = ACTIONS(1671),
    [sym_null_str] = ACTIONS(1671),
    [sym_smtp_address] = ACTIONS(1671),
    [sym_smtp_port_number] = ACTIONS(1671),
    [sym_smtp_username] = ACTIONS(1671),
    [sym_smtp_password] = ACTIONS(1671),
    [sym_smtp_require_tls] = ACTIONS(1671),
    [sym_smtp_from] = ACTIONS(1671),
    [sym_smtp_tls_true] = ACTIONS(1671),
    [sym_smtp_tls_false] = ACTIONS(1671),
    [sym_listview_on_row_select_event] = ACTIONS(1671),
    [sym_data_structure_unknown] = ACTIONS(1671),
    [sym_data_structure_datafeed] = ACTIONS(1671),
    [sym_data_structure_datarecordset] = ACTIONS(1671),
    [sym_data_structure_datarecord] = ACTIONS(1671),
    [sym_data_structure_datafield] = ACTIONS(1671),
    [sym_data_max_content_field_count] = ACTIONS(1671),
    [sym_data_max_metadata_field_count] = ACTIONS(1671),
    [sym_data_max_id_length] = ACTIONS(1671),
    [sym_data_max_name_length] = ACTIONS(1671),
    [sym_data_max_description_length] = ACTIONS(1671),
    [sym_data_max_type_length] = ACTIONS(1671),
    [sym_data_max_format_length] = ACTIONS(1671),
    [sym_data_max_source_length] = ACTIONS(1671),
    [sym_data_max_value_length] = ACTIONS(1671),
    [sym_data_max_label_length] = ACTIONS(1671),
    [sym_data_max_url_length] = ACTIONS(1671),
    [sym_max_last_login_info_length] = ACTIONS(1671),
    [sym_data_type_unknown] = ACTIONS(1671),
    [sym_data_type_string] = ACTIONS(1671),
    [sym_data_type_datetime] = ACTIONS(1671),
    [sym_data_type_date] = ACTIONS(1671),
    [sym_data_type_time] = ACTIONS(1671),
    [sym_data_type_image] = ACTIONS(1671),
    [sym_data_format_url] = ACTIONS(1671),
    [sym_data_format_phone] = ACTIONS(1671),
    [sym_data_format_email] = ACTIONS(1671),
    [sym_data_format_iso8601] = ACTIONS(1671),
    [sym_auditlog_priv] = ACTIONS(1671),
    [sym_configuration_priv] = ACTIONS(1671),
    [sym_deviceconfig_priv] = ACTIONS(1671),
    [sym_ftp_priv] = ACTIONS(1671),
    [sym_http_priv] = ACTIONS(1671),
    [sym_networkconfig_priv] = ACTIONS(1671),
    [sym_programport_priv] = ACTIONS(1671),
    [sym_remoteui_priv] = ACTIONS(1671),
    [sym_securitycontrol_priv] = ACTIONS(1671),
    [sym_softwaremanagement_priv] = ACTIONS(1671),
    [sym_terminal_priv] = ACTIONS(1671),
    [sym_tpadmin_priv] = ACTIONS(1671),
    [sym_user1_priv] = ACTIONS(1671),
    [sym_user2_priv] = ACTIONS(1671),
    [sym_user3_priv] = ACTIONS(1671),
    [sym_user4_priv] = ACTIONS(1671),
    [sym_usermanagement_priv] = ACTIONS(1671),
    [sym_valid_account] = ACTIONS(1671),
    [sym_err_name_invalid] = ACTIONS(1671),
    [sym_err_password_invalid] = ACTIONS(1671),
    [sym_err_invalid_account] = ACTIONS(1671),
    [sym_err_no_authorization] = ACTIONS(1671),
    [sym_err_invalid_parameter] = ACTIONS(1671),
    [sym_err_user_locked_out] = ACTIONS(1671),
    [sym_err_user_account_expired] = ACTIONS(1671),
    [sym_err_general_error] = ACTIONS(1671),
    [sym_err_authentication_error] = ACTIONS(1671),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1671),
    [sym_tls_validate_certificate] = ACTIONS(1671),
    [sym_tdata] = ACTIONS(1671),
    [sym_tchannel] = ACTIONS(1671),
    [sym_tlevel] = ACTIONS(1671),
    [sym_tbutton] = ACTIONS(1671),
    [sym_ttimeline] = ACTIONS(1671),
    [sym_tcustom] = ACTIONS(1671),
    [sym_url_struct] = ACTIONS(1671),
    [sym_dns_struct] = ACTIONS(1671),
    [sym_ip_address_struct] = ACTIONS(1671),
    [sym_dev_info_struct] = ACTIONS(1671),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1671),
    [sym_data_feed] = ACTIONS(1671),
    [sym_data_field] = ACTIONS(1671),
    [sym_data_record] = ACTIONS(1671),
    [sym_wc_data_feed] = ACTIONS(1671),
    [sym_wc_data_field] = ACTIONS(1671),
    [sym_wc_data_record] = ACTIONS(1671),
    [sym_last_login_info] = ACTIONS(1671),
    [sym____reserved___] = ACTIONS(1671),
    [sym_date] = ACTIONS(1671),
    [sym_day] = ACTIONS(1671),
    [sym_time] = ACTIONS(1671),
    [sym_ldate] = ACTIONS(1671),
    [sym_push_device] = ACTIONS(1671),
    [sym_push_channel] = ACTIONS(1671),
    [sym_push_devchan] = ACTIONS(1671),
    [sym_release_device] = ACTIONS(1671),
    [sym_release_channel] = ACTIONS(1671),
    [sym_release_devchan] = ACTIONS(1671),
    [sym_master_slot] = ACTIONS(1671),
    [sym_get_pulse_time] = ACTIONS(1671),
    [sym_get_timer] = ACTIONS(1671),
    [sym_master_sn] = ACTIONS(1671),
    [sym_system_number] = ACTIONS(1671),
    [sym_button] = ACTIONS(1671),
    [sym_dv_channel] = ACTIONS(1671),
    [sym_data] = ACTIONS(1671),
    [sym_level] = ACTIONS(1671),
    [sym_timeline] = ACTIONS(1671),
    [sym_channel] = ACTIONS(1671),
    [sym_custom] = ACTIONS(1671),
    [sym_atoi] = ACTIONS(1671),
    [sym_atol] = ACTIONS(1671),
    [sym_atof] = ACTIONS(1671),
    [sym_itoa] = ACTIONS(1671),
    [sym_itohex] = ACTIONS(1671),
    [sym_hextoi] = ACTIONS(1671),
    [sym_ftoa] = ACTIONS(1671),
    [sym_find_string] = ACTIONS(1671),
    [sym_left_string] = ACTIONS(1671),
    [sym_right_string] = ACTIONS(1671),
    [sym_mid_string] = ACTIONS(1671),
    [sym_remove_string] = ACTIONS(1671),
    [sym_get_buffer_string] = ACTIONS(1671),
    [sym_get_buffer_char] = ACTIONS(1671),
    [sym_get_multi_buffer_string] = ACTIONS(1671),
    [sym_length_string] = ACTIONS(1671),
    [sym_max_length_string] = ACTIONS(1671),
    [sym_set_length_string] = ACTIONS(1671),
    [sym_compare_string] = ACTIONS(1671),
    [sym_upper_string] = ACTIONS(1671),
    [sym_lower_string] = ACTIONS(1671),
    [sym_get_last] = ACTIONS(1671),
    [sym_date_to_day] = ACTIONS(1671),
    [sym_date_to_month] = ACTIONS(1671),
    [sym_date_to_year] = ACTIONS(1671),
    [sym_time_to_hour] = ACTIONS(1671),
    [sym_time_to_minute] = ACTIONS(1671),
    [sym_time_to_second] = ACTIONS(1671),
    [sym_day_of_week] = ACTIONS(1671),
    [sym_set_pulse_time] = ACTIONS(1671),
    [sym_set_timer] = ACTIONS(1671),
    [sym_file_open] = ACTIONS(1671),
    [sym_file_close] = ACTIONS(1671),
    [sym_file_read] = ACTIONS(1671),
    [sym_file_read_line] = ACTIONS(1671),
    [sym_file_write] = ACTIONS(1671),
    [sym_file_write_line] = ACTIONS(1671),
    [sym_file_copy] = ACTIONS(1671),
    [sym_file_delete] = ACTIONS(1671),
    [sym_file_rename] = ACTIONS(1671),
    [sym_file_dir] = ACTIONS(1671),
    [sym_file_getdir] = ACTIONS(1671),
    [sym_file_setdir] = ACTIONS(1671),
    [sym_file_create_dir] = ACTIONS(1671),
    [sym_file_remove_dir] = ACTIONS(1671),
    [sym_file_seek] = ACTIONS(1671),
    [sym_length_array] = ACTIONS(1671),
    [sym_max_length_array] = ACTIONS(1671),
    [sym_set_length_array] = ACTIONS(1671),
    [sym_ip_client_open] = ACTIONS(1671),
    [sym_ip_client_close] = ACTIONS(1671),
    [sym_ip_server_open] = ACTIONS(1671),
    [sym_ip_server_close] = ACTIONS(1671),
    [sym_ip_bound_client_open] = ACTIONS(1671),
    [sym_ip_mc_server_open] = ACTIONS(1671),
    [sym_ip_set_option] = ACTIONS(1671),
    [sym_ssh_client_open] = ACTIONS(1671),
    [sym_ssh_client_close] = ACTIONS(1671),
    [sym_tls_client_open] = ACTIONS(1671),
    [sym_tls_client_close] = ACTIONS(1671),
    [sym_do_push] = ACTIONS(1671),
    [sym_do_push_timed] = ACTIONS(1671),
    [sym_do_release] = ACTIONS(1671),
    [sym_device_id] = ACTIONS(1671),
    [sym_device_info] = ACTIONS(1671),
    [sym_device_id_string] = ACTIONS(1671),
    [sym_device_standby] = ACTIONS(1671),
    [sym_device_wake] = ACTIONS(1671),
    [sym_timeline_create] = ACTIONS(1671),
    [sym_timeline_kill] = ACTIONS(1671),
    [sym_timeline_reload] = ACTIONS(1671),
    [sym_timeline_pause] = ACTIONS(1671),
    [sym_timeline_restart] = ACTIONS(1671),
    [sym_timeline_set] = ACTIONS(1671),
    [sym_timeline_get] = ACTIONS(1671),
    [sym_timeline_active] = ACTIONS(1671),
    [sym_timeline_dynamic_id] = ACTIONS(1671),
    [sym_variable_to_string] = ACTIONS(1671),
    [sym_string_to_variable] = ACTIONS(1671),
    [sym_length_variable_to_string] = ACTIONS(1671),
    [sym_type_cast] = ACTIONS(1671),
    [sym_variable_to_xml] = ACTIONS(1671),
    [sym_xml_to_variable] = ACTIONS(1671),
    [sym_length_variable_to_xml] = ACTIONS(1671),
    [sym_redirect_string] = ACTIONS(1671),
    [sym_random_number] = ACTIONS(1671),
    [sym_get_unique_id] = ACTIONS(1671),
    [sym_get_system_number] = ACTIONS(1671),
    [sym_set_system_number] = ACTIONS(1671),
    [sym_get_serial_number] = ACTIONS(1671),
    [sym_get_master_build] = ACTIONS(1671),
    [sym_set_validation_code] = ACTIONS(1671),
    [sym_reboot] = ACTIONS(1671),
    [sym_load_duet_module] = ACTIONS(1671),
    [sym_unload_duet_module] = ACTIONS(1671),
    [sym_duet_mem_size_set] = ACTIONS(1671),
    [sym_duet_mem_size_get] = ACTIONS(1671),
    [sym_dynamic_application_device] = ACTIONS(1671),
    [sym_dynamic_polled_port] = ACTIONS(1671),
    [sym_static_port_binding] = ACTIONS(1671),
    [sym_static_ip_binding] = ACTIONS(1671),
    [sym_get_dns_list] = ACTIONS(1671),
    [sym_set_dns_list] = ACTIONS(1671),
    [sym_get_ip_address] = ACTIONS(1671),
    [sym_set_ip_address] = ACTIONS(1671),
    [sym_get_url_list] = ACTIONS(1671),
    [sym_add_url_entry] = ACTIONS(1671),
    [sym_delete_url_entry] = ACTIONS(1671),
    [sym_abs_value] = ACTIONS(1671),
    [sym_max_value] = ACTIONS(1671),
    [sym_min_value] = ACTIONS(1671),
    [sym_exp_value] = ACTIONS(1671),
    [sym_log_value] = ACTIONS(1671),
    [sym_log10_value] = ACTIONS(1671),
    [sym_power_value] = ACTIONS(1671),
    [sym_sqrt_value] = ACTIONS(1671),
    [sym_raw_be] = ACTIONS(1671),
    [sym_raw_le] = ACTIONS(1671),
    [sym_internal_queue_size_set] = ACTIONS(1671),
    [sym_internal_queue_size_get] = ACTIONS(1671),
    [sym_internal_threshold_set] = ACTIONS(1671),
    [sym_internal_threshold_get] = ACTIONS(1671),
    [sym_rebuilt_event] = ACTIONS(1671),
    [sym_do_custom_event] = ACTIONS(1671),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1671),
    [sym_clkmgr_set_clk_source] = ACTIONS(1671),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1671),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1671),
    [sym_clkmgr_get_timezone] = ACTIONS(1671),
    [sym_clkmgr_set_timezone] = ACTIONS(1671),
    [sym_clkmgr_get_resync_period] = ACTIONS(1671),
    [sym_clkmgr_set_resync_period] = ACTIONS(1671),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1671),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1671),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1671),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1671),
    [sym_clkmgr_get_timeservers] = ACTIONS(1671),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1671),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1671),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1671),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1671),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1671),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1671),
    [sym_set_log_level] = ACTIONS(1671),
    [sym_get_log_level] = ACTIONS(1671),
    [sym_amx_log] = ACTIONS(1671),
    [sym_smtp_server_config_set] = ACTIONS(1671),
    [sym_smtp_server_config_get] = ACTIONS(1671),
    [sym_smtp_send] = ACTIONS(1671),
    [sym_validate_netlinx_account] = ACTIONS(1671),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1671),
    [sym_authenticate_certificate] = ACTIONS(1671),
    [sym_audit_netlinx_session_event] = ACTIONS(1671),
    [sym_audit_netlinx_generic_event] = ACTIONS(1671),
    [sym_get_available_flash_disk_space] = ACTIONS(1671),
    [sym_get_max_flash_disk_space] = ACTIONS(1671),
    [sym_data_create_feed] = ACTIONS(1671),
    [sym_data_delete_feed] = ACTIONS(1671),
    [sym_data_publish_feed] = ACTIONS(1671),
    [sym_data_get_published_feed] = ACTIONS(1671),
    [sym_data_add_record] = ACTIONS(1671),
    [sym_data_get_event_record] = ACTIONS(1671),
    [sym__wc_data_create_feed] = ACTIONS(1671),
    [sym__wc_data_add_record] = ACTIONS(1671),
    [sym__wc_data_get_event_record] = ACTIONS(1671),
    [sym_format] = ACTIONS(1671),
    [sym_astro_clock] = ACTIONS(1671),
    [sym_set_outdoor_temperature] = ACTIONS(1671),
    [sym_set_virtual_level_count] = ACTIONS(1671),
    [sym_set_virtual_channel_count] = ACTIONS(1671),
    [sym_set_virtual_port_count] = ACTIONS(1671),
    [sym__wc_encode_int] = ACTIONS(1671),
    [sym__wc_decode_int] = ACTIONS(1671),
    [sym__wc_int] = ACTIONS(1671),
    [sym__wc_to_ch_int] = ACTIONS(1671),
    [sym__ch_to_wc_int] = ACTIONS(1671),
    [sym__wc_find_string_int] = ACTIONS(1671),
    [sym__wc_left_string_int] = ACTIONS(1671),
    [sym__wc_lower_string_int] = ACTIONS(1671),
    [sym__wc_mid_string_int] = ACTIONS(1671),
    [sym__wc_remove_string_int] = ACTIONS(1671),
    [sym__wc_right_string_int] = ACTIONS(1671),
    [sym__wc_upper_string_int] = ACTIONS(1671),
    [sym__wc_compare_string_int] = ACTIONS(1671),
    [sym__wc_get_buffer_char_int] = ACTIONS(1671),
    [sym__wc_get_buffer_string_int] = ACTIONS(1671),
    [sym__wc_concat_string_int] = ACTIONS(1671),
    [sym__wc_file_open_int] = ACTIONS(1671),
    [sym__wc_file_close_int] = ACTIONS(1671),
    [sym__wc_file_read_int] = ACTIONS(1671),
    [sym__wc_file_read_line_int] = ACTIONS(1671),
    [sym__wc_file_write_int] = ACTIONS(1671),
    [sym__wc_file_write_line_int] = ACTIONS(1671),
    [aux_sym_preproc_include_token1] = ACTIONS(1671),
    [aux_sym_preproc_define_token1] = ACTIONS(1669),
    [aux_sym_preproc_warn_token1] = ACTIONS(1669),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1669),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1669),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1669),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1669),
    [aux_sym_preproc_else_token1] = ACTIONS(1669),
    [anon_sym_COMMA] = ACTIONS(1669),
    [anon_sym_LBRACK] = ACTIONS(1669),
    [anon_sym_LBRACE] = ACTIONS(1669),
    [anon_sym_RBRACE] = ACTIONS(1669),
    [anon_sym_LPAREN] = ACTIONS(1671),
    [anon_sym_RPAREN] = ACTIONS(1669),
    [anon_sym_BANG] = ACTIONS(1669),
    [anon_sym_TILDE] = ACTIONS(1669),
    [anon_sym_DASH] = ACTIONS(1671),
    [anon_sym_PLUS] = ACTIONS(1671),
    [anon_sym_DASH_DASH] = ACTIONS(1669),
    [anon_sym_PLUS_PLUS] = ACTIONS(1669),
    [anon_sym_DQUOTE] = ACTIONS(1669),
    [anon_sym_SQUOTE] = ACTIONS(1669),
    [sym_number_literal] = ACTIONS(1669),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(198)] = {
    [sym_identifier] = ACTIONS(1673),
    [sym_program_name_keyword] = ACTIONS(1673),
    [sym_module_name_keyword] = ACTIONS(1673),
    [sym_define_device_keyword] = ACTIONS(1673),
    [sym_define_combine_keyword] = ACTIONS(1673),
    [sym_define_constant_keyword] = ACTIONS(1673),
    [sym_define_type_keyword] = ACTIONS(1673),
    [sym_define_variable_keyword] = ACTIONS(1673),
    [sym_define_system_variable_keyword] = ACTIONS(1673),
    [sym_define_connect_level_keyword] = ACTIONS(1673),
    [sym_define_latching_keyword] = ACTIONS(1673),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1673),
    [sym_define_toggling_keyword] = ACTIONS(1673),
    [sym_define_call_keyword] = ACTIONS(1673),
    [sym_define_function_keyword] = ACTIONS(1673),
    [sym_define_library_function_keyword] = ACTIONS(1673),
    [sym_define_start_keyword] = ACTIONS(1673),
    [sym_define_event_keyword] = ACTIONS(1673),
    [sym_define_program_keyword] = ACTIONS(1673),
    [sym_define_module_keyword] = ACTIONS(1673),
    [sym_char_keyword] = ACTIONS(1673),
    [sym_widechar_keyword] = ACTIONS(1673),
    [sym_integer_keyword] = ACTIONS(1673),
    [sym_sinteger_keyword] = ACTIONS(1673),
    [sym_long_keyword] = ACTIONS(1673),
    [sym_slong_keyword] = ACTIONS(1673),
    [sym_float_keyword] = ACTIONS(1673),
    [sym_double_keyword] = ACTIONS(1673),
    [sym_variant_keyword] = ACTIONS(1673),
    [sym_variantarray_keyword] = ACTIONS(1673),
    [sym_local_var_keyword] = ACTIONS(1673),
    [sym_stack_var_keyword] = ACTIONS(1673),
    [sym_constant_keyword] = ACTIONS(1673),
    [sym_volatile_keyword] = ACTIONS(1673),
    [sym_non_volatile_keyword] = ACTIONS(1673),
    [sym_persistent_keyword] = ACTIONS(1673),
    [sym_dev_keyword] = ACTIONS(1673),
    [sym_devlev_keyword] = ACTIONS(1673),
    [sym_devchan_keyword] = ACTIONS(1673),
    [sym_if_keyword] = ACTIONS(1673),
    [sym_else_keyword] = ACTIONS(1673),
    [sym_switch_keyword] = ACTIONS(1673),
    [sym_case_keyword] = ACTIONS(1673),
    [sym_default_keyword] = ACTIONS(1673),
    [sym_while_keyword] = ACTIONS(1673),
    [sym_for_keyword] = ACTIONS(1673),
    [sym_break_keyword] = ACTIONS(1673),
    [sym_continue_keyword] = ACTIONS(1673),
    [sym_return_keyword] = ACTIONS(1673),
    [sym_select_keyword] = ACTIONS(1673),
    [sym_active_keyword] = ACTIONS(1673),
    [sym_struct_keyword] = ACTIONS(1673),
    [sym_structure_keyword] = ACTIONS(1673),
    [sym_button_event_keyword] = ACTIONS(1673),
    [sym_channel_event_keyword] = ACTIONS(1673),
    [sym_level_event_keyword] = ACTIONS(1673),
    [sym_data_event_keyword] = ACTIONS(1673),
    [sym_timeline_event_keyword] = ACTIONS(1673),
    [sym_custom_event_keyword] = ACTIONS(1673),
    [sym_push_keyword] = ACTIONS(1673),
    [sym_release_keyword] = ACTIONS(1673),
    [aux_sym_on_keyword_token1] = ACTIONS(1673),
    [aux_sym_off_keyword_token1] = ACTIONS(1673),
    [sym_send_level_keyword] = ACTIONS(1673),
    [sym_send_string_keyword] = ACTIONS(1673),
    [sym_send_command_keyword] = ACTIONS(1673),
    [sym_clear_buffer_keyword] = ACTIONS(1673),
    [sym_create_buffer_keyword] = ACTIONS(1673),
    [sym_create_multi_buffer_keyword] = ACTIONS(1673),
    [sym_call_keyword] = ACTIONS(1673),
    [sym_system_call_keyword] = ACTIONS(1673),
    [sym_devchan_to_keyword] = ACTIONS(1673),
    [sym_devchan_min_to_keyword] = ACTIONS(1673),
    [sym_devchan_total_off_keyword] = ACTIONS(1673),
    [sym_devchan_pulse_keyword] = ACTIONS(1673),
    [sym_wait_keyword] = ACTIONS(1673),
    [sym_cancel_wait_keyword] = ACTIONS(1673),
    [sym_wait_until_keyword] = ACTIONS(1673),
    [sym_cancel_wait_until_keyword] = ACTIONS(1673),
    [sym_cancel_all_wait_keyword] = ACTIONS(1673),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1673),
    [sym_not] = ACTIONS(1673),
    [sym_bnot] = ACTIONS(1673),
    [sym___file__] = ACTIONS(1673),
    [sym___date__] = ACTIONS(1673),
    [sym___ldate__] = ACTIONS(1673),
    [sym___time__] = ACTIONS(1673),
    [sym___line__] = ACTIONS(1673),
    [sym___name__] = ACTIONS(1673),
    [sym___netlinx__] = ACTIONS(1673),
    [sym_netlinx_axi_version] = ACTIONS(1673),
    [sym_url_flg_tcp] = ACTIONS(1673),
    [sym_url_flg_accinfopresent] = ACTIONS(1673),
    [sym_url_flg_temp] = ACTIONS(1673),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1673),
    [sym_url_flg_stat_mask] = ACTIONS(1673),
    [sym_url_flg_stat_lookup] = ACTIONS(1673),
    [sym_url_flg_stat_connecting] = ACTIONS(1673),
    [sym_url_flg_stat_waiting] = ACTIONS(1673),
    [sym_url_flg_stat_connected] = ACTIONS(1673),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1673),
    [sym_first_virtual_device] = ACTIONS(1673),
    [sym_first_local_port] = ACTIONS(1673),
    [sym_dynamic_virtual_device] = ACTIONS(1673),
    [sym_timeline_once] = ACTIONS(1673),
    [sym_timeline_repeat] = ACTIONS(1673),
    [sym_timeline_absolute] = ACTIONS(1673),
    [sym_timeline_relative] = ACTIONS(1673),
    [sym_do_push_timed_infinite] = ACTIONS(1673),
    [sym_source_type_no_address] = ACTIONS(1673),
    [sym_source_type_neuron_id] = ACTIONS(1673),
    [sym_source_type_ip_address] = ACTIONS(1673),
    [sym_source_type_axlink] = ACTIONS(1673),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1673),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1673),
    [sym_source_type_ip_socket_address] = ACTIONS(1673),
    [sym_source_type_rs232] = ACTIONS(1673),
    [sym_source_type_internal] = ACTIONS(1673),
    [sym_source_type_mac_address] = ACTIONS(1673),
    [sym_source_type_ipv4_port] = ACTIONS(1673),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1673),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1673),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1673),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1673),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1673),
    [sym_true] = ACTIONS(1673),
    [sym_false] = ACTIONS(1673),
    [sym_file_read_only] = ACTIONS(1673),
    [sym_file_rw_new] = ACTIONS(1673),
    [sym_file_rw_append] = ACTIONS(1673),
    [sym_ip_tcp] = ACTIONS(1673),
    [sym_ip_udp] = ACTIONS(1673),
    [sym_ip_udp_2way] = ACTIONS(1673),
    [sym_xml_encode_types] = ACTIONS(1673),
    [sym_xml_encode_char_as_list] = ACTIONS(1673),
    [sym_xml_encode_le] = ACTIONS(1673),
    [sym_xml_decode_types] = ACTIONS(1673),
    [sym_xml_decode_no_preserve] = ACTIONS(1673),
    [sym_duet_dev_type_display_device] = ACTIONS(1673),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1673),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1673),
    [sym_duet_dev_type_io_device] = ACTIONS(1673),
    [sym_duet_dev_type_relay_device] = ACTIONS(1673),
    [sym_duet_dev_type_ups] = ACTIONS(1673),
    [sym_duet_dev_type_amplifier] = ACTIONS(1673),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1673),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1673),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1673),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1673),
    [sym_duet_dev_type_camera] = ACTIONS(1673),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1673),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1673),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1673),
    [sym_duet_dev_type_dss] = ACTIONS(1673),
    [sym_duet_dev_type_dvr] = ACTIONS(1673),
    [sym_duet_dev_type_disc_device] = ACTIONS(1673),
    [sym_duet_dev_type_document_camera] = ACTIONS(1673),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1673),
    [sym_duet_dev_type_hvac] = ACTIONS(1673),
    [sym_duet_dev_type_keypad] = ACTIONS(1673),
    [sym_duet_dev_type_light] = ACTIONS(1673),
    [sym_duet_dev_type_monitor] = ACTIONS(1673),
    [sym_duet_dev_type_motor] = ACTIONS(1673),
    [sym_duet_dev_type_multi_window] = ACTIONS(1673),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1673),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1673),
    [sym_duet_dev_type_receiver] = ACTIONS(1673),
    [sym_duet_dev_type_security_system] = ACTIONS(1673),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1673),
    [sym_duet_dev_type_settop_box] = ACTIONS(1673),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1673),
    [sym_duet_dev_type_switcher] = ACTIONS(1673),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1673),
    [sym_duet_dev_type_tv] = ACTIONS(1673),
    [sym_duet_dev_type_utility] = ACTIONS(1673),
    [sym_duet_dev_type_vcr] = ACTIONS(1673),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1673),
    [sym_duet_dev_type_video_processor] = ACTIONS(1673),
    [sym_duet_dev_type_video_projector] = ACTIONS(1673),
    [sym_duet_dev_type_video_wall] = ACTIONS(1673),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1673),
    [sym_duet_dev_type_weather] = ACTIONS(1673),
    [sym_duet_dev_not_polled] = ACTIONS(1673),
    [sym_duet_dev_polled] = ACTIONS(1673),
    [sym_clkmgr_mode_network] = ACTIONS(1673),
    [sym_clkmgr_mode_standalone] = ACTIONS(1673),
    [sym_normal_standby] = ACTIONS(1673),
    [sym_normal_wake] = ACTIONS(1673),
    [sym_ip_multicast_ttl_option] = ACTIONS(1673),
    [sym_ip_ttl_subnet] = ACTIONS(1673),
    [sym_ip_ttl_site] = ACTIONS(1673),
    [sym_ip_ttl_region] = ACTIONS(1673),
    [sym_ip_ttl_continent] = ACTIONS(1673),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1673),
    [sym_ip_nodelay_on] = ACTIONS(1673),
    [sym_ip_nodelay_off] = ACTIONS(1673),
    [sym_amx_error] = ACTIONS(1673),
    [sym_amx_warning] = ACTIONS(1673),
    [sym_amx_info] = ACTIONS(1673),
    [sym_amx_debug] = ACTIONS(1673),
    [sym_netlinx_login_success] = ACTIONS(1673),
    [sym_netlinx_login_fail] = ACTIONS(1673),
    [sym_netlinx_logout] = ACTIONS(1673),
    [sym_null_str] = ACTIONS(1673),
    [sym_smtp_address] = ACTIONS(1673),
    [sym_smtp_port_number] = ACTIONS(1673),
    [sym_smtp_username] = ACTIONS(1673),
    [sym_smtp_password] = ACTIONS(1673),
    [sym_smtp_require_tls] = ACTIONS(1673),
    [sym_smtp_from] = ACTIONS(1673),
    [sym_smtp_tls_true] = ACTIONS(1673),
    [sym_smtp_tls_false] = ACTIONS(1673),
    [sym_listview_on_row_select_event] = ACTIONS(1673),
    [sym_data_structure_unknown] = ACTIONS(1673),
    [sym_data_structure_datafeed] = ACTIONS(1673),
    [sym_data_structure_datarecordset] = ACTIONS(1673),
    [sym_data_structure_datarecord] = ACTIONS(1673),
    [sym_data_structure_datafield] = ACTIONS(1673),
    [sym_data_max_content_field_count] = ACTIONS(1673),
    [sym_data_max_metadata_field_count] = ACTIONS(1673),
    [sym_data_max_id_length] = ACTIONS(1673),
    [sym_data_max_name_length] = ACTIONS(1673),
    [sym_data_max_description_length] = ACTIONS(1673),
    [sym_data_max_type_length] = ACTIONS(1673),
    [sym_data_max_format_length] = ACTIONS(1673),
    [sym_data_max_source_length] = ACTIONS(1673),
    [sym_data_max_value_length] = ACTIONS(1673),
    [sym_data_max_label_length] = ACTIONS(1673),
    [sym_data_max_url_length] = ACTIONS(1673),
    [sym_max_last_login_info_length] = ACTIONS(1673),
    [sym_data_type_unknown] = ACTIONS(1673),
    [sym_data_type_string] = ACTIONS(1673),
    [sym_data_type_datetime] = ACTIONS(1673),
    [sym_data_type_date] = ACTIONS(1673),
    [sym_data_type_time] = ACTIONS(1673),
    [sym_data_type_image] = ACTIONS(1673),
    [sym_data_format_url] = ACTIONS(1673),
    [sym_data_format_phone] = ACTIONS(1673),
    [sym_data_format_email] = ACTIONS(1673),
    [sym_data_format_iso8601] = ACTIONS(1673),
    [sym_auditlog_priv] = ACTIONS(1673),
    [sym_configuration_priv] = ACTIONS(1673),
    [sym_deviceconfig_priv] = ACTIONS(1673),
    [sym_ftp_priv] = ACTIONS(1673),
    [sym_http_priv] = ACTIONS(1673),
    [sym_networkconfig_priv] = ACTIONS(1673),
    [sym_programport_priv] = ACTIONS(1673),
    [sym_remoteui_priv] = ACTIONS(1673),
    [sym_securitycontrol_priv] = ACTIONS(1673),
    [sym_softwaremanagement_priv] = ACTIONS(1673),
    [sym_terminal_priv] = ACTIONS(1673),
    [sym_tpadmin_priv] = ACTIONS(1673),
    [sym_user1_priv] = ACTIONS(1673),
    [sym_user2_priv] = ACTIONS(1673),
    [sym_user3_priv] = ACTIONS(1673),
    [sym_user4_priv] = ACTIONS(1673),
    [sym_usermanagement_priv] = ACTIONS(1673),
    [sym_valid_account] = ACTIONS(1673),
    [sym_err_name_invalid] = ACTIONS(1673),
    [sym_err_password_invalid] = ACTIONS(1673),
    [sym_err_invalid_account] = ACTIONS(1673),
    [sym_err_no_authorization] = ACTIONS(1673),
    [sym_err_invalid_parameter] = ACTIONS(1673),
    [sym_err_user_locked_out] = ACTIONS(1673),
    [sym_err_user_account_expired] = ACTIONS(1673),
    [sym_err_general_error] = ACTIONS(1673),
    [sym_err_authentication_error] = ACTIONS(1673),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1673),
    [sym_tls_validate_certificate] = ACTIONS(1673),
    [sym_tdata] = ACTIONS(1673),
    [sym_tchannel] = ACTIONS(1673),
    [sym_tlevel] = ACTIONS(1673),
    [sym_tbutton] = ACTIONS(1673),
    [sym_ttimeline] = ACTIONS(1673),
    [sym_tcustom] = ACTIONS(1673),
    [sym_url_struct] = ACTIONS(1673),
    [sym_dns_struct] = ACTIONS(1673),
    [sym_ip_address_struct] = ACTIONS(1673),
    [sym_dev_info_struct] = ACTIONS(1673),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1673),
    [sym_data_feed] = ACTIONS(1673),
    [sym_data_field] = ACTIONS(1673),
    [sym_data_record] = ACTIONS(1673),
    [sym_wc_data_feed] = ACTIONS(1673),
    [sym_wc_data_field] = ACTIONS(1673),
    [sym_wc_data_record] = ACTIONS(1673),
    [sym_last_login_info] = ACTIONS(1673),
    [sym____reserved___] = ACTIONS(1673),
    [sym_date] = ACTIONS(1673),
    [sym_day] = ACTIONS(1673),
    [sym_time] = ACTIONS(1673),
    [sym_ldate] = ACTIONS(1673),
    [sym_push_device] = ACTIONS(1673),
    [sym_push_channel] = ACTIONS(1673),
    [sym_push_devchan] = ACTIONS(1673),
    [sym_release_device] = ACTIONS(1673),
    [sym_release_channel] = ACTIONS(1673),
    [sym_release_devchan] = ACTIONS(1673),
    [sym_master_slot] = ACTIONS(1673),
    [sym_get_pulse_time] = ACTIONS(1673),
    [sym_get_timer] = ACTIONS(1673),
    [sym_master_sn] = ACTIONS(1673),
    [sym_system_number] = ACTIONS(1673),
    [sym_button] = ACTIONS(1673),
    [sym_dv_channel] = ACTIONS(1673),
    [sym_data] = ACTIONS(1673),
    [sym_level] = ACTIONS(1673),
    [sym_timeline] = ACTIONS(1673),
    [sym_channel] = ACTIONS(1673),
    [sym_custom] = ACTIONS(1673),
    [sym_atoi] = ACTIONS(1673),
    [sym_atol] = ACTIONS(1673),
    [sym_atof] = ACTIONS(1673),
    [sym_itoa] = ACTIONS(1673),
    [sym_itohex] = ACTIONS(1673),
    [sym_hextoi] = ACTIONS(1673),
    [sym_ftoa] = ACTIONS(1673),
    [sym_find_string] = ACTIONS(1673),
    [sym_left_string] = ACTIONS(1673),
    [sym_right_string] = ACTIONS(1673),
    [sym_mid_string] = ACTIONS(1673),
    [sym_remove_string] = ACTIONS(1673),
    [sym_get_buffer_string] = ACTIONS(1673),
    [sym_get_buffer_char] = ACTIONS(1673),
    [sym_get_multi_buffer_string] = ACTIONS(1673),
    [sym_length_string] = ACTIONS(1673),
    [sym_max_length_string] = ACTIONS(1673),
    [sym_set_length_string] = ACTIONS(1673),
    [sym_compare_string] = ACTIONS(1673),
    [sym_upper_string] = ACTIONS(1673),
    [sym_lower_string] = ACTIONS(1673),
    [sym_get_last] = ACTIONS(1673),
    [sym_date_to_day] = ACTIONS(1673),
    [sym_date_to_month] = ACTIONS(1673),
    [sym_date_to_year] = ACTIONS(1673),
    [sym_time_to_hour] = ACTIONS(1673),
    [sym_time_to_minute] = ACTIONS(1673),
    [sym_time_to_second] = ACTIONS(1673),
    [sym_day_of_week] = ACTIONS(1673),
    [sym_set_pulse_time] = ACTIONS(1673),
    [sym_set_timer] = ACTIONS(1673),
    [sym_file_open] = ACTIONS(1673),
    [sym_file_close] = ACTIONS(1673),
    [sym_file_read] = ACTIONS(1673),
    [sym_file_read_line] = ACTIONS(1673),
    [sym_file_write] = ACTIONS(1673),
    [sym_file_write_line] = ACTIONS(1673),
    [sym_file_copy] = ACTIONS(1673),
    [sym_file_delete] = ACTIONS(1673),
    [sym_file_rename] = ACTIONS(1673),
    [sym_file_dir] = ACTIONS(1673),
    [sym_file_getdir] = ACTIONS(1673),
    [sym_file_setdir] = ACTIONS(1673),
    [sym_file_create_dir] = ACTIONS(1673),
    [sym_file_remove_dir] = ACTIONS(1673),
    [sym_file_seek] = ACTIONS(1673),
    [sym_length_array] = ACTIONS(1673),
    [sym_max_length_array] = ACTIONS(1673),
    [sym_set_length_array] = ACTIONS(1673),
    [sym_ip_client_open] = ACTIONS(1673),
    [sym_ip_client_close] = ACTIONS(1673),
    [sym_ip_server_open] = ACTIONS(1673),
    [sym_ip_server_close] = ACTIONS(1673),
    [sym_ip_bound_client_open] = ACTIONS(1673),
    [sym_ip_mc_server_open] = ACTIONS(1673),
    [sym_ip_set_option] = ACTIONS(1673),
    [sym_ssh_client_open] = ACTIONS(1673),
    [sym_ssh_client_close] = ACTIONS(1673),
    [sym_tls_client_open] = ACTIONS(1673),
    [sym_tls_client_close] = ACTIONS(1673),
    [sym_do_push] = ACTIONS(1673),
    [sym_do_push_timed] = ACTIONS(1673),
    [sym_do_release] = ACTIONS(1673),
    [sym_device_id] = ACTIONS(1673),
    [sym_device_info] = ACTIONS(1673),
    [sym_device_id_string] = ACTIONS(1673),
    [sym_device_standby] = ACTIONS(1673),
    [sym_device_wake] = ACTIONS(1673),
    [sym_timeline_create] = ACTIONS(1673),
    [sym_timeline_kill] = ACTIONS(1673),
    [sym_timeline_reload] = ACTIONS(1673),
    [sym_timeline_pause] = ACTIONS(1673),
    [sym_timeline_restart] = ACTIONS(1673),
    [sym_timeline_set] = ACTIONS(1673),
    [sym_timeline_get] = ACTIONS(1673),
    [sym_timeline_active] = ACTIONS(1673),
    [sym_timeline_dynamic_id] = ACTIONS(1673),
    [sym_variable_to_string] = ACTIONS(1673),
    [sym_string_to_variable] = ACTIONS(1673),
    [sym_length_variable_to_string] = ACTIONS(1673),
    [sym_type_cast] = ACTIONS(1673),
    [sym_variable_to_xml] = ACTIONS(1673),
    [sym_xml_to_variable] = ACTIONS(1673),
    [sym_length_variable_to_xml] = ACTIONS(1673),
    [sym_redirect_string] = ACTIONS(1673),
    [sym_random_number] = ACTIONS(1673),
    [sym_get_unique_id] = ACTIONS(1673),
    [sym_get_system_number] = ACTIONS(1673),
    [sym_set_system_number] = ACTIONS(1673),
    [sym_get_serial_number] = ACTIONS(1673),
    [sym_get_master_build] = ACTIONS(1673),
    [sym_set_validation_code] = ACTIONS(1673),
    [sym_reboot] = ACTIONS(1673),
    [sym_load_duet_module] = ACTIONS(1673),
    [sym_unload_duet_module] = ACTIONS(1673),
    [sym_duet_mem_size_set] = ACTIONS(1673),
    [sym_duet_mem_size_get] = ACTIONS(1673),
    [sym_dynamic_application_device] = ACTIONS(1673),
    [sym_dynamic_polled_port] = ACTIONS(1673),
    [sym_static_port_binding] = ACTIONS(1673),
    [sym_static_ip_binding] = ACTIONS(1673),
    [sym_get_dns_list] = ACTIONS(1673),
    [sym_set_dns_list] = ACTIONS(1673),
    [sym_get_ip_address] = ACTIONS(1673),
    [sym_set_ip_address] = ACTIONS(1673),
    [sym_get_url_list] = ACTIONS(1673),
    [sym_add_url_entry] = ACTIONS(1673),
    [sym_delete_url_entry] = ACTIONS(1673),
    [sym_abs_value] = ACTIONS(1673),
    [sym_max_value] = ACTIONS(1673),
    [sym_min_value] = ACTIONS(1673),
    [sym_exp_value] = ACTIONS(1673),
    [sym_log_value] = ACTIONS(1673),
    [sym_log10_value] = ACTIONS(1673),
    [sym_power_value] = ACTIONS(1673),
    [sym_sqrt_value] = ACTIONS(1673),
    [sym_raw_be] = ACTIONS(1673),
    [sym_raw_le] = ACTIONS(1673),
    [sym_internal_queue_size_set] = ACTIONS(1673),
    [sym_internal_queue_size_get] = ACTIONS(1673),
    [sym_internal_threshold_set] = ACTIONS(1673),
    [sym_internal_threshold_get] = ACTIONS(1673),
    [sym_rebuilt_event] = ACTIONS(1673),
    [sym_do_custom_event] = ACTIONS(1673),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1673),
    [sym_clkmgr_set_clk_source] = ACTIONS(1673),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1673),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1673),
    [sym_clkmgr_get_timezone] = ACTIONS(1673),
    [sym_clkmgr_set_timezone] = ACTIONS(1673),
    [sym_clkmgr_get_resync_period] = ACTIONS(1673),
    [sym_clkmgr_set_resync_period] = ACTIONS(1673),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1673),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1673),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1673),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1673),
    [sym_clkmgr_get_timeservers] = ACTIONS(1673),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1673),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1673),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1673),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1673),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1673),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1673),
    [sym_set_log_level] = ACTIONS(1673),
    [sym_get_log_level] = ACTIONS(1673),
    [sym_amx_log] = ACTIONS(1673),
    [sym_smtp_server_config_set] = ACTIONS(1673),
    [sym_smtp_server_config_get] = ACTIONS(1673),
    [sym_smtp_send] = ACTIONS(1673),
    [sym_validate_netlinx_account] = ACTIONS(1673),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1673),
    [sym_authenticate_certificate] = ACTIONS(1673),
    [sym_audit_netlinx_session_event] = ACTIONS(1673),
    [sym_audit_netlinx_generic_event] = ACTIONS(1673),
    [sym_get_available_flash_disk_space] = ACTIONS(1673),
    [sym_get_max_flash_disk_space] = ACTIONS(1673),
    [sym_data_create_feed] = ACTIONS(1673),
    [sym_data_delete_feed] = ACTIONS(1673),
    [sym_data_publish_feed] = ACTIONS(1673),
    [sym_data_get_published_feed] = ACTIONS(1673),
    [sym_data_add_record] = ACTIONS(1673),
    [sym_data_get_event_record] = ACTIONS(1673),
    [sym__wc_data_create_feed] = ACTIONS(1673),
    [sym__wc_data_add_record] = ACTIONS(1673),
    [sym__wc_data_get_event_record] = ACTIONS(1673),
    [sym_format] = ACTIONS(1673),
    [sym_astro_clock] = ACTIONS(1673),
    [sym_set_outdoor_temperature] = ACTIONS(1673),
    [sym_set_virtual_level_count] = ACTIONS(1673),
    [sym_set_virtual_channel_count] = ACTIONS(1673),
    [sym_set_virtual_port_count] = ACTIONS(1673),
    [sym__wc_encode_int] = ACTIONS(1673),
    [sym__wc_decode_int] = ACTIONS(1673),
    [sym__wc_int] = ACTIONS(1673),
    [sym__wc_to_ch_int] = ACTIONS(1673),
    [sym__ch_to_wc_int] = ACTIONS(1673),
    [sym__wc_find_string_int] = ACTIONS(1673),
    [sym__wc_left_string_int] = ACTIONS(1673),
    [sym__wc_lower_string_int] = ACTIONS(1673),
    [sym__wc_mid_string_int] = ACTIONS(1673),
    [sym__wc_remove_string_int] = ACTIONS(1673),
    [sym__wc_right_string_int] = ACTIONS(1673),
    [sym__wc_upper_string_int] = ACTIONS(1673),
    [sym__wc_compare_string_int] = ACTIONS(1673),
    [sym__wc_get_buffer_char_int] = ACTIONS(1673),
    [sym__wc_get_buffer_string_int] = ACTIONS(1673),
    [sym__wc_concat_string_int] = ACTIONS(1673),
    [sym__wc_file_open_int] = ACTIONS(1673),
    [sym__wc_file_close_int] = ACTIONS(1673),
    [sym__wc_file_read_int] = ACTIONS(1673),
    [sym__wc_file_read_line_int] = ACTIONS(1673),
    [sym__wc_file_write_int] = ACTIONS(1673),
    [sym__wc_file_write_line_int] = ACTIONS(1673),
    [aux_sym_preproc_include_token1] = ACTIONS(1673),
    [aux_sym_preproc_define_token1] = ACTIONS(1675),
    [aux_sym_preproc_warn_token1] = ACTIONS(1675),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1675),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1675),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1675),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1675),
    [aux_sym_preproc_else_token1] = ACTIONS(1675),
    [anon_sym_LBRACK] = ACTIONS(1675),
    [anon_sym_LBRACE] = ACTIONS(1675),
    [anon_sym_LPAREN] = ACTIONS(1673),
    [anon_sym_SEMI] = ACTIONS(1677),
    [anon_sym_BANG] = ACTIONS(1675),
    [anon_sym_TILDE] = ACTIONS(1675),
    [anon_sym_DASH] = ACTIONS(1673),
    [anon_sym_PLUS] = ACTIONS(1673),
    [anon_sym_DASH_DASH] = ACTIONS(1675),
    [anon_sym_PLUS_PLUS] = ACTIONS(1675),
    [anon_sym_DQUOTE] = ACTIONS(1675),
    [anon_sym_SQUOTE] = ACTIONS(1675),
    [sym_number_literal] = ACTIONS(1675),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1677),
  },
  [STATE(199)] = {
    [sym_identifier] = ACTIONS(1679),
    [sym_program_name_keyword] = ACTIONS(1679),
    [sym_module_name_keyword] = ACTIONS(1679),
    [sym_define_device_keyword] = ACTIONS(1679),
    [sym_define_combine_keyword] = ACTIONS(1679),
    [sym_define_constant_keyword] = ACTIONS(1679),
    [sym_define_type_keyword] = ACTIONS(1679),
    [sym_define_variable_keyword] = ACTIONS(1679),
    [sym_define_system_variable_keyword] = ACTIONS(1679),
    [sym_define_connect_level_keyword] = ACTIONS(1679),
    [sym_define_latching_keyword] = ACTIONS(1679),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1679),
    [sym_define_toggling_keyword] = ACTIONS(1679),
    [sym_define_call_keyword] = ACTIONS(1679),
    [sym_define_function_keyword] = ACTIONS(1679),
    [sym_define_library_function_keyword] = ACTIONS(1679),
    [sym_define_start_keyword] = ACTIONS(1679),
    [sym_define_event_keyword] = ACTIONS(1679),
    [sym_define_program_keyword] = ACTIONS(1679),
    [sym_define_module_keyword] = ACTIONS(1679),
    [sym_char_keyword] = ACTIONS(1679),
    [sym_widechar_keyword] = ACTIONS(1679),
    [sym_integer_keyword] = ACTIONS(1679),
    [sym_sinteger_keyword] = ACTIONS(1679),
    [sym_long_keyword] = ACTIONS(1679),
    [sym_slong_keyword] = ACTIONS(1679),
    [sym_float_keyword] = ACTIONS(1679),
    [sym_double_keyword] = ACTIONS(1679),
    [sym_variant_keyword] = ACTIONS(1679),
    [sym_variantarray_keyword] = ACTIONS(1679),
    [sym_local_var_keyword] = ACTIONS(1679),
    [sym_stack_var_keyword] = ACTIONS(1679),
    [sym_constant_keyword] = ACTIONS(1679),
    [sym_volatile_keyword] = ACTIONS(1679),
    [sym_non_volatile_keyword] = ACTIONS(1679),
    [sym_persistent_keyword] = ACTIONS(1679),
    [sym_dev_keyword] = ACTIONS(1679),
    [sym_devlev_keyword] = ACTIONS(1679),
    [sym_devchan_keyword] = ACTIONS(1679),
    [sym_if_keyword] = ACTIONS(1679),
    [sym_else_keyword] = ACTIONS(1679),
    [sym_switch_keyword] = ACTIONS(1679),
    [sym_case_keyword] = ACTIONS(1679),
    [sym_default_keyword] = ACTIONS(1679),
    [sym_while_keyword] = ACTIONS(1679),
    [sym_for_keyword] = ACTIONS(1679),
    [sym_break_keyword] = ACTIONS(1679),
    [sym_continue_keyword] = ACTIONS(1679),
    [sym_return_keyword] = ACTIONS(1679),
    [sym_select_keyword] = ACTIONS(1679),
    [sym_active_keyword] = ACTIONS(1679),
    [sym_struct_keyword] = ACTIONS(1679),
    [sym_structure_keyword] = ACTIONS(1679),
    [sym_button_event_keyword] = ACTIONS(1679),
    [sym_channel_event_keyword] = ACTIONS(1679),
    [sym_level_event_keyword] = ACTIONS(1679),
    [sym_data_event_keyword] = ACTIONS(1679),
    [sym_timeline_event_keyword] = ACTIONS(1679),
    [sym_custom_event_keyword] = ACTIONS(1679),
    [sym_push_keyword] = ACTIONS(1679),
    [sym_release_keyword] = ACTIONS(1679),
    [aux_sym_on_keyword_token1] = ACTIONS(1679),
    [aux_sym_off_keyword_token1] = ACTIONS(1679),
    [sym_send_level_keyword] = ACTIONS(1679),
    [sym_send_string_keyword] = ACTIONS(1679),
    [sym_send_command_keyword] = ACTIONS(1679),
    [sym_clear_buffer_keyword] = ACTIONS(1679),
    [sym_create_buffer_keyword] = ACTIONS(1679),
    [sym_create_multi_buffer_keyword] = ACTIONS(1679),
    [sym_call_keyword] = ACTIONS(1679),
    [sym_system_call_keyword] = ACTIONS(1679),
    [sym_devchan_to_keyword] = ACTIONS(1679),
    [sym_devchan_min_to_keyword] = ACTIONS(1679),
    [sym_devchan_total_off_keyword] = ACTIONS(1679),
    [sym_devchan_pulse_keyword] = ACTIONS(1679),
    [sym_wait_keyword] = ACTIONS(1679),
    [sym_cancel_wait_keyword] = ACTIONS(1679),
    [sym_wait_until_keyword] = ACTIONS(1679),
    [sym_cancel_wait_until_keyword] = ACTIONS(1679),
    [sym_cancel_all_wait_keyword] = ACTIONS(1679),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1679),
    [sym_not] = ACTIONS(1679),
    [sym_bnot] = ACTIONS(1679),
    [sym___file__] = ACTIONS(1679),
    [sym___date__] = ACTIONS(1679),
    [sym___ldate__] = ACTIONS(1679),
    [sym___time__] = ACTIONS(1679),
    [sym___line__] = ACTIONS(1679),
    [sym___name__] = ACTIONS(1679),
    [sym___netlinx__] = ACTIONS(1679),
    [sym_netlinx_axi_version] = ACTIONS(1679),
    [sym_url_flg_tcp] = ACTIONS(1679),
    [sym_url_flg_accinfopresent] = ACTIONS(1679),
    [sym_url_flg_temp] = ACTIONS(1679),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1679),
    [sym_url_flg_stat_mask] = ACTIONS(1679),
    [sym_url_flg_stat_lookup] = ACTIONS(1679),
    [sym_url_flg_stat_connecting] = ACTIONS(1679),
    [sym_url_flg_stat_waiting] = ACTIONS(1679),
    [sym_url_flg_stat_connected] = ACTIONS(1679),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1679),
    [sym_first_virtual_device] = ACTIONS(1679),
    [sym_first_local_port] = ACTIONS(1679),
    [sym_dynamic_virtual_device] = ACTIONS(1679),
    [sym_timeline_once] = ACTIONS(1679),
    [sym_timeline_repeat] = ACTIONS(1679),
    [sym_timeline_absolute] = ACTIONS(1679),
    [sym_timeline_relative] = ACTIONS(1679),
    [sym_do_push_timed_infinite] = ACTIONS(1679),
    [sym_source_type_no_address] = ACTIONS(1679),
    [sym_source_type_neuron_id] = ACTIONS(1679),
    [sym_source_type_ip_address] = ACTIONS(1679),
    [sym_source_type_axlink] = ACTIONS(1679),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1679),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1679),
    [sym_source_type_ip_socket_address] = ACTIONS(1679),
    [sym_source_type_rs232] = ACTIONS(1679),
    [sym_source_type_internal] = ACTIONS(1679),
    [sym_source_type_mac_address] = ACTIONS(1679),
    [sym_source_type_ipv4_port] = ACTIONS(1679),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1679),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1679),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1679),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1679),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1679),
    [sym_true] = ACTIONS(1679),
    [sym_false] = ACTIONS(1679),
    [sym_file_read_only] = ACTIONS(1679),
    [sym_file_rw_new] = ACTIONS(1679),
    [sym_file_rw_append] = ACTIONS(1679),
    [sym_ip_tcp] = ACTIONS(1679),
    [sym_ip_udp] = ACTIONS(1679),
    [sym_ip_udp_2way] = ACTIONS(1679),
    [sym_xml_encode_types] = ACTIONS(1679),
    [sym_xml_encode_char_as_list] = ACTIONS(1679),
    [sym_xml_encode_le] = ACTIONS(1679),
    [sym_xml_decode_types] = ACTIONS(1679),
    [sym_xml_decode_no_preserve] = ACTIONS(1679),
    [sym_duet_dev_type_display_device] = ACTIONS(1679),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1679),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1679),
    [sym_duet_dev_type_io_device] = ACTIONS(1679),
    [sym_duet_dev_type_relay_device] = ACTIONS(1679),
    [sym_duet_dev_type_ups] = ACTIONS(1679),
    [sym_duet_dev_type_amplifier] = ACTIONS(1679),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1679),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1679),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1679),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1679),
    [sym_duet_dev_type_camera] = ACTIONS(1679),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1679),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1679),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1679),
    [sym_duet_dev_type_dss] = ACTIONS(1679),
    [sym_duet_dev_type_dvr] = ACTIONS(1679),
    [sym_duet_dev_type_disc_device] = ACTIONS(1679),
    [sym_duet_dev_type_document_camera] = ACTIONS(1679),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1679),
    [sym_duet_dev_type_hvac] = ACTIONS(1679),
    [sym_duet_dev_type_keypad] = ACTIONS(1679),
    [sym_duet_dev_type_light] = ACTIONS(1679),
    [sym_duet_dev_type_monitor] = ACTIONS(1679),
    [sym_duet_dev_type_motor] = ACTIONS(1679),
    [sym_duet_dev_type_multi_window] = ACTIONS(1679),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1679),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1679),
    [sym_duet_dev_type_receiver] = ACTIONS(1679),
    [sym_duet_dev_type_security_system] = ACTIONS(1679),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1679),
    [sym_duet_dev_type_settop_box] = ACTIONS(1679),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1679),
    [sym_duet_dev_type_switcher] = ACTIONS(1679),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1679),
    [sym_duet_dev_type_tv] = ACTIONS(1679),
    [sym_duet_dev_type_utility] = ACTIONS(1679),
    [sym_duet_dev_type_vcr] = ACTIONS(1679),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1679),
    [sym_duet_dev_type_video_processor] = ACTIONS(1679),
    [sym_duet_dev_type_video_projector] = ACTIONS(1679),
    [sym_duet_dev_type_video_wall] = ACTIONS(1679),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1679),
    [sym_duet_dev_type_weather] = ACTIONS(1679),
    [sym_duet_dev_not_polled] = ACTIONS(1679),
    [sym_duet_dev_polled] = ACTIONS(1679),
    [sym_clkmgr_mode_network] = ACTIONS(1679),
    [sym_clkmgr_mode_standalone] = ACTIONS(1679),
    [sym_normal_standby] = ACTIONS(1679),
    [sym_normal_wake] = ACTIONS(1679),
    [sym_ip_multicast_ttl_option] = ACTIONS(1679),
    [sym_ip_ttl_subnet] = ACTIONS(1679),
    [sym_ip_ttl_site] = ACTIONS(1679),
    [sym_ip_ttl_region] = ACTIONS(1679),
    [sym_ip_ttl_continent] = ACTIONS(1679),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1679),
    [sym_ip_nodelay_on] = ACTIONS(1679),
    [sym_ip_nodelay_off] = ACTIONS(1679),
    [sym_amx_error] = ACTIONS(1679),
    [sym_amx_warning] = ACTIONS(1679),
    [sym_amx_info] = ACTIONS(1679),
    [sym_amx_debug] = ACTIONS(1679),
    [sym_netlinx_login_success] = ACTIONS(1679),
    [sym_netlinx_login_fail] = ACTIONS(1679),
    [sym_netlinx_logout] = ACTIONS(1679),
    [sym_null_str] = ACTIONS(1679),
    [sym_smtp_address] = ACTIONS(1679),
    [sym_smtp_port_number] = ACTIONS(1679),
    [sym_smtp_username] = ACTIONS(1679),
    [sym_smtp_password] = ACTIONS(1679),
    [sym_smtp_require_tls] = ACTIONS(1679),
    [sym_smtp_from] = ACTIONS(1679),
    [sym_smtp_tls_true] = ACTIONS(1679),
    [sym_smtp_tls_false] = ACTIONS(1679),
    [sym_listview_on_row_select_event] = ACTIONS(1679),
    [sym_data_structure_unknown] = ACTIONS(1679),
    [sym_data_structure_datafeed] = ACTIONS(1679),
    [sym_data_structure_datarecordset] = ACTIONS(1679),
    [sym_data_structure_datarecord] = ACTIONS(1679),
    [sym_data_structure_datafield] = ACTIONS(1679),
    [sym_data_max_content_field_count] = ACTIONS(1679),
    [sym_data_max_metadata_field_count] = ACTIONS(1679),
    [sym_data_max_id_length] = ACTIONS(1679),
    [sym_data_max_name_length] = ACTIONS(1679),
    [sym_data_max_description_length] = ACTIONS(1679),
    [sym_data_max_type_length] = ACTIONS(1679),
    [sym_data_max_format_length] = ACTIONS(1679),
    [sym_data_max_source_length] = ACTIONS(1679),
    [sym_data_max_value_length] = ACTIONS(1679),
    [sym_data_max_label_length] = ACTIONS(1679),
    [sym_data_max_url_length] = ACTIONS(1679),
    [sym_max_last_login_info_length] = ACTIONS(1679),
    [sym_data_type_unknown] = ACTIONS(1679),
    [sym_data_type_string] = ACTIONS(1679),
    [sym_data_type_datetime] = ACTIONS(1679),
    [sym_data_type_date] = ACTIONS(1679),
    [sym_data_type_time] = ACTIONS(1679),
    [sym_data_type_image] = ACTIONS(1679),
    [sym_data_format_url] = ACTIONS(1679),
    [sym_data_format_phone] = ACTIONS(1679),
    [sym_data_format_email] = ACTIONS(1679),
    [sym_data_format_iso8601] = ACTIONS(1679),
    [sym_auditlog_priv] = ACTIONS(1679),
    [sym_configuration_priv] = ACTIONS(1679),
    [sym_deviceconfig_priv] = ACTIONS(1679),
    [sym_ftp_priv] = ACTIONS(1679),
    [sym_http_priv] = ACTIONS(1679),
    [sym_networkconfig_priv] = ACTIONS(1679),
    [sym_programport_priv] = ACTIONS(1679),
    [sym_remoteui_priv] = ACTIONS(1679),
    [sym_securitycontrol_priv] = ACTIONS(1679),
    [sym_softwaremanagement_priv] = ACTIONS(1679),
    [sym_terminal_priv] = ACTIONS(1679),
    [sym_tpadmin_priv] = ACTIONS(1679),
    [sym_user1_priv] = ACTIONS(1679),
    [sym_user2_priv] = ACTIONS(1679),
    [sym_user3_priv] = ACTIONS(1679),
    [sym_user4_priv] = ACTIONS(1679),
    [sym_usermanagement_priv] = ACTIONS(1679),
    [sym_valid_account] = ACTIONS(1679),
    [sym_err_name_invalid] = ACTIONS(1679),
    [sym_err_password_invalid] = ACTIONS(1679),
    [sym_err_invalid_account] = ACTIONS(1679),
    [sym_err_no_authorization] = ACTIONS(1679),
    [sym_err_invalid_parameter] = ACTIONS(1679),
    [sym_err_user_locked_out] = ACTIONS(1679),
    [sym_err_user_account_expired] = ACTIONS(1679),
    [sym_err_general_error] = ACTIONS(1679),
    [sym_err_authentication_error] = ACTIONS(1679),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1679),
    [sym_tls_validate_certificate] = ACTIONS(1679),
    [sym_tdata] = ACTIONS(1679),
    [sym_tchannel] = ACTIONS(1679),
    [sym_tlevel] = ACTIONS(1679),
    [sym_tbutton] = ACTIONS(1679),
    [sym_ttimeline] = ACTIONS(1679),
    [sym_tcustom] = ACTIONS(1679),
    [sym_url_struct] = ACTIONS(1679),
    [sym_dns_struct] = ACTIONS(1679),
    [sym_ip_address_struct] = ACTIONS(1679),
    [sym_dev_info_struct] = ACTIONS(1679),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1679),
    [sym_data_feed] = ACTIONS(1679),
    [sym_data_field] = ACTIONS(1679),
    [sym_data_record] = ACTIONS(1679),
    [sym_wc_data_feed] = ACTIONS(1679),
    [sym_wc_data_field] = ACTIONS(1679),
    [sym_wc_data_record] = ACTIONS(1679),
    [sym_last_login_info] = ACTIONS(1679),
    [sym____reserved___] = ACTIONS(1679),
    [sym_date] = ACTIONS(1679),
    [sym_day] = ACTIONS(1679),
    [sym_time] = ACTIONS(1679),
    [sym_ldate] = ACTIONS(1679),
    [sym_push_device] = ACTIONS(1679),
    [sym_push_channel] = ACTIONS(1679),
    [sym_push_devchan] = ACTIONS(1679),
    [sym_release_device] = ACTIONS(1679),
    [sym_release_channel] = ACTIONS(1679),
    [sym_release_devchan] = ACTIONS(1679),
    [sym_master_slot] = ACTIONS(1679),
    [sym_get_pulse_time] = ACTIONS(1679),
    [sym_get_timer] = ACTIONS(1679),
    [sym_master_sn] = ACTIONS(1679),
    [sym_system_number] = ACTIONS(1679),
    [sym_button] = ACTIONS(1679),
    [sym_dv_channel] = ACTIONS(1679),
    [sym_data] = ACTIONS(1679),
    [sym_level] = ACTIONS(1679),
    [sym_timeline] = ACTIONS(1679),
    [sym_channel] = ACTIONS(1679),
    [sym_custom] = ACTIONS(1679),
    [sym_atoi] = ACTIONS(1679),
    [sym_atol] = ACTIONS(1679),
    [sym_atof] = ACTIONS(1679),
    [sym_itoa] = ACTIONS(1679),
    [sym_itohex] = ACTIONS(1679),
    [sym_hextoi] = ACTIONS(1679),
    [sym_ftoa] = ACTIONS(1679),
    [sym_find_string] = ACTIONS(1679),
    [sym_left_string] = ACTIONS(1679),
    [sym_right_string] = ACTIONS(1679),
    [sym_mid_string] = ACTIONS(1679),
    [sym_remove_string] = ACTIONS(1679),
    [sym_get_buffer_string] = ACTIONS(1679),
    [sym_get_buffer_char] = ACTIONS(1679),
    [sym_get_multi_buffer_string] = ACTIONS(1679),
    [sym_length_string] = ACTIONS(1679),
    [sym_max_length_string] = ACTIONS(1679),
    [sym_set_length_string] = ACTIONS(1679),
    [sym_compare_string] = ACTIONS(1679),
    [sym_upper_string] = ACTIONS(1679),
    [sym_lower_string] = ACTIONS(1679),
    [sym_get_last] = ACTIONS(1679),
    [sym_date_to_day] = ACTIONS(1679),
    [sym_date_to_month] = ACTIONS(1679),
    [sym_date_to_year] = ACTIONS(1679),
    [sym_time_to_hour] = ACTIONS(1679),
    [sym_time_to_minute] = ACTIONS(1679),
    [sym_time_to_second] = ACTIONS(1679),
    [sym_day_of_week] = ACTIONS(1679),
    [sym_set_pulse_time] = ACTIONS(1679),
    [sym_set_timer] = ACTIONS(1679),
    [sym_file_open] = ACTIONS(1679),
    [sym_file_close] = ACTIONS(1679),
    [sym_file_read] = ACTIONS(1679),
    [sym_file_read_line] = ACTIONS(1679),
    [sym_file_write] = ACTIONS(1679),
    [sym_file_write_line] = ACTIONS(1679),
    [sym_file_copy] = ACTIONS(1679),
    [sym_file_delete] = ACTIONS(1679),
    [sym_file_rename] = ACTIONS(1679),
    [sym_file_dir] = ACTIONS(1679),
    [sym_file_getdir] = ACTIONS(1679),
    [sym_file_setdir] = ACTIONS(1679),
    [sym_file_create_dir] = ACTIONS(1679),
    [sym_file_remove_dir] = ACTIONS(1679),
    [sym_file_seek] = ACTIONS(1679),
    [sym_length_array] = ACTIONS(1679),
    [sym_max_length_array] = ACTIONS(1679),
    [sym_set_length_array] = ACTIONS(1679),
    [sym_ip_client_open] = ACTIONS(1679),
    [sym_ip_client_close] = ACTIONS(1679),
    [sym_ip_server_open] = ACTIONS(1679),
    [sym_ip_server_close] = ACTIONS(1679),
    [sym_ip_bound_client_open] = ACTIONS(1679),
    [sym_ip_mc_server_open] = ACTIONS(1679),
    [sym_ip_set_option] = ACTIONS(1679),
    [sym_ssh_client_open] = ACTIONS(1679),
    [sym_ssh_client_close] = ACTIONS(1679),
    [sym_tls_client_open] = ACTIONS(1679),
    [sym_tls_client_close] = ACTIONS(1679),
    [sym_do_push] = ACTIONS(1679),
    [sym_do_push_timed] = ACTIONS(1679),
    [sym_do_release] = ACTIONS(1679),
    [sym_device_id] = ACTIONS(1679),
    [sym_device_info] = ACTIONS(1679),
    [sym_device_id_string] = ACTIONS(1679),
    [sym_device_standby] = ACTIONS(1679),
    [sym_device_wake] = ACTIONS(1679),
    [sym_timeline_create] = ACTIONS(1679),
    [sym_timeline_kill] = ACTIONS(1679),
    [sym_timeline_reload] = ACTIONS(1679),
    [sym_timeline_pause] = ACTIONS(1679),
    [sym_timeline_restart] = ACTIONS(1679),
    [sym_timeline_set] = ACTIONS(1679),
    [sym_timeline_get] = ACTIONS(1679),
    [sym_timeline_active] = ACTIONS(1679),
    [sym_timeline_dynamic_id] = ACTIONS(1679),
    [sym_variable_to_string] = ACTIONS(1679),
    [sym_string_to_variable] = ACTIONS(1679),
    [sym_length_variable_to_string] = ACTIONS(1679),
    [sym_type_cast] = ACTIONS(1679),
    [sym_variable_to_xml] = ACTIONS(1679),
    [sym_xml_to_variable] = ACTIONS(1679),
    [sym_length_variable_to_xml] = ACTIONS(1679),
    [sym_redirect_string] = ACTIONS(1679),
    [sym_random_number] = ACTIONS(1679),
    [sym_get_unique_id] = ACTIONS(1679),
    [sym_get_system_number] = ACTIONS(1679),
    [sym_set_system_number] = ACTIONS(1679),
    [sym_get_serial_number] = ACTIONS(1679),
    [sym_get_master_build] = ACTIONS(1679),
    [sym_set_validation_code] = ACTIONS(1679),
    [sym_reboot] = ACTIONS(1679),
    [sym_load_duet_module] = ACTIONS(1679),
    [sym_unload_duet_module] = ACTIONS(1679),
    [sym_duet_mem_size_set] = ACTIONS(1679),
    [sym_duet_mem_size_get] = ACTIONS(1679),
    [sym_dynamic_application_device] = ACTIONS(1679),
    [sym_dynamic_polled_port] = ACTIONS(1679),
    [sym_static_port_binding] = ACTIONS(1679),
    [sym_static_ip_binding] = ACTIONS(1679),
    [sym_get_dns_list] = ACTIONS(1679),
    [sym_set_dns_list] = ACTIONS(1679),
    [sym_get_ip_address] = ACTIONS(1679),
    [sym_set_ip_address] = ACTIONS(1679),
    [sym_get_url_list] = ACTIONS(1679),
    [sym_add_url_entry] = ACTIONS(1679),
    [sym_delete_url_entry] = ACTIONS(1679),
    [sym_abs_value] = ACTIONS(1679),
    [sym_max_value] = ACTIONS(1679),
    [sym_min_value] = ACTIONS(1679),
    [sym_exp_value] = ACTIONS(1679),
    [sym_log_value] = ACTIONS(1679),
    [sym_log10_value] = ACTIONS(1679),
    [sym_power_value] = ACTIONS(1679),
    [sym_sqrt_value] = ACTIONS(1679),
    [sym_raw_be] = ACTIONS(1679),
    [sym_raw_le] = ACTIONS(1679),
    [sym_internal_queue_size_set] = ACTIONS(1679),
    [sym_internal_queue_size_get] = ACTIONS(1679),
    [sym_internal_threshold_set] = ACTIONS(1679),
    [sym_internal_threshold_get] = ACTIONS(1679),
    [sym_rebuilt_event] = ACTIONS(1679),
    [sym_do_custom_event] = ACTIONS(1679),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1679),
    [sym_clkmgr_set_clk_source] = ACTIONS(1679),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1679),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1679),
    [sym_clkmgr_get_timezone] = ACTIONS(1679),
    [sym_clkmgr_set_timezone] = ACTIONS(1679),
    [sym_clkmgr_get_resync_period] = ACTIONS(1679),
    [sym_clkmgr_set_resync_period] = ACTIONS(1679),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1679),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1679),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1679),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1679),
    [sym_clkmgr_get_timeservers] = ACTIONS(1679),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1679),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1679),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1679),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1679),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1679),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1679),
    [sym_set_log_level] = ACTIONS(1679),
    [sym_get_log_level] = ACTIONS(1679),
    [sym_amx_log] = ACTIONS(1679),
    [sym_smtp_server_config_set] = ACTIONS(1679),
    [sym_smtp_server_config_get] = ACTIONS(1679),
    [sym_smtp_send] = ACTIONS(1679),
    [sym_validate_netlinx_account] = ACTIONS(1679),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1679),
    [sym_authenticate_certificate] = ACTIONS(1679),
    [sym_audit_netlinx_session_event] = ACTIONS(1679),
    [sym_audit_netlinx_generic_event] = ACTIONS(1679),
    [sym_get_available_flash_disk_space] = ACTIONS(1679),
    [sym_get_max_flash_disk_space] = ACTIONS(1679),
    [sym_data_create_feed] = ACTIONS(1679),
    [sym_data_delete_feed] = ACTIONS(1679),
    [sym_data_publish_feed] = ACTIONS(1679),
    [sym_data_get_published_feed] = ACTIONS(1679),
    [sym_data_add_record] = ACTIONS(1679),
    [sym_data_get_event_record] = ACTIONS(1679),
    [sym__wc_data_create_feed] = ACTIONS(1679),
    [sym__wc_data_add_record] = ACTIONS(1679),
    [sym__wc_data_get_event_record] = ACTIONS(1679),
    [sym_format] = ACTIONS(1679),
    [sym_astro_clock] = ACTIONS(1679),
    [sym_set_outdoor_temperature] = ACTIONS(1679),
    [sym_set_virtual_level_count] = ACTIONS(1679),
    [sym_set_virtual_channel_count] = ACTIONS(1679),
    [sym_set_virtual_port_count] = ACTIONS(1679),
    [sym__wc_encode_int] = ACTIONS(1679),
    [sym__wc_decode_int] = ACTIONS(1679),
    [sym__wc_int] = ACTIONS(1679),
    [sym__wc_to_ch_int] = ACTIONS(1679),
    [sym__ch_to_wc_int] = ACTIONS(1679),
    [sym__wc_find_string_int] = ACTIONS(1679),
    [sym__wc_left_string_int] = ACTIONS(1679),
    [sym__wc_lower_string_int] = ACTIONS(1679),
    [sym__wc_mid_string_int] = ACTIONS(1679),
    [sym__wc_remove_string_int] = ACTIONS(1679),
    [sym__wc_right_string_int] = ACTIONS(1679),
    [sym__wc_upper_string_int] = ACTIONS(1679),
    [sym__wc_compare_string_int] = ACTIONS(1679),
    [sym__wc_get_buffer_char_int] = ACTIONS(1679),
    [sym__wc_get_buffer_string_int] = ACTIONS(1679),
    [sym__wc_concat_string_int] = ACTIONS(1679),
    [sym__wc_file_open_int] = ACTIONS(1679),
    [sym__wc_file_close_int] = ACTIONS(1679),
    [sym__wc_file_read_int] = ACTIONS(1679),
    [sym__wc_file_read_line_int] = ACTIONS(1679),
    [sym__wc_file_write_int] = ACTIONS(1679),
    [sym__wc_file_write_line_int] = ACTIONS(1679),
    [aux_sym_preproc_include_token1] = ACTIONS(1679),
    [aux_sym_preproc_define_token1] = ACTIONS(1681),
    [aux_sym_preproc_warn_token1] = ACTIONS(1681),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1681),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1681),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1681),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1681),
    [aux_sym_preproc_else_token1] = ACTIONS(1681),
    [anon_sym_LBRACK] = ACTIONS(1681),
    [anon_sym_LBRACE] = ACTIONS(1681),
    [anon_sym_LPAREN] = ACTIONS(1679),
    [anon_sym_SEMI] = ACTIONS(1683),
    [anon_sym_BANG] = ACTIONS(1681),
    [anon_sym_TILDE] = ACTIONS(1681),
    [anon_sym_DASH] = ACTIONS(1679),
    [anon_sym_PLUS] = ACTIONS(1679),
    [anon_sym_DASH_DASH] = ACTIONS(1681),
    [anon_sym_PLUS_PLUS] = ACTIONS(1681),
    [anon_sym_DQUOTE] = ACTIONS(1681),
    [anon_sym_SQUOTE] = ACTIONS(1681),
    [sym_number_literal] = ACTIONS(1681),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1683),
  },
  [STATE(200)] = {
    [ts_builtin_sym_end] = ACTIONS(1681),
    [sym_identifier] = ACTIONS(1679),
    [sym_program_name_keyword] = ACTIONS(1679),
    [sym_module_name_keyword] = ACTIONS(1679),
    [sym_define_device_keyword] = ACTIONS(1679),
    [sym_define_combine_keyword] = ACTIONS(1679),
    [sym_define_constant_keyword] = ACTIONS(1679),
    [sym_define_type_keyword] = ACTIONS(1679),
    [sym_define_variable_keyword] = ACTIONS(1679),
    [sym_define_system_variable_keyword] = ACTIONS(1679),
    [sym_define_connect_level_keyword] = ACTIONS(1679),
    [sym_define_latching_keyword] = ACTIONS(1679),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1679),
    [sym_define_toggling_keyword] = ACTIONS(1679),
    [sym_define_call_keyword] = ACTIONS(1679),
    [sym_define_function_keyword] = ACTIONS(1679),
    [sym_define_library_function_keyword] = ACTIONS(1679),
    [sym_define_start_keyword] = ACTIONS(1679),
    [sym_define_event_keyword] = ACTIONS(1679),
    [sym_define_program_keyword] = ACTIONS(1679),
    [sym_define_module_keyword] = ACTIONS(1679),
    [sym_char_keyword] = ACTIONS(1679),
    [sym_widechar_keyword] = ACTIONS(1679),
    [sym_integer_keyword] = ACTIONS(1679),
    [sym_sinteger_keyword] = ACTIONS(1679),
    [sym_long_keyword] = ACTIONS(1679),
    [sym_slong_keyword] = ACTIONS(1679),
    [sym_float_keyword] = ACTIONS(1679),
    [sym_double_keyword] = ACTIONS(1679),
    [sym_variant_keyword] = ACTIONS(1679),
    [sym_variantarray_keyword] = ACTIONS(1679),
    [sym_local_var_keyword] = ACTIONS(1679),
    [sym_stack_var_keyword] = ACTIONS(1679),
    [sym_constant_keyword] = ACTIONS(1679),
    [sym_volatile_keyword] = ACTIONS(1679),
    [sym_non_volatile_keyword] = ACTIONS(1679),
    [sym_persistent_keyword] = ACTIONS(1679),
    [sym_dev_keyword] = ACTIONS(1679),
    [sym_devlev_keyword] = ACTIONS(1679),
    [sym_devchan_keyword] = ACTIONS(1679),
    [sym_if_keyword] = ACTIONS(1679),
    [sym_else_keyword] = ACTIONS(1679),
    [sym_switch_keyword] = ACTIONS(1679),
    [sym_case_keyword] = ACTIONS(1679),
    [sym_default_keyword] = ACTIONS(1679),
    [sym_while_keyword] = ACTIONS(1679),
    [sym_for_keyword] = ACTIONS(1679),
    [sym_break_keyword] = ACTIONS(1679),
    [sym_continue_keyword] = ACTIONS(1679),
    [sym_return_keyword] = ACTIONS(1679),
    [sym_select_keyword] = ACTIONS(1679),
    [sym_active_keyword] = ACTIONS(1679),
    [sym_struct_keyword] = ACTIONS(1679),
    [sym_structure_keyword] = ACTIONS(1679),
    [sym_button_event_keyword] = ACTIONS(1679),
    [sym_channel_event_keyword] = ACTIONS(1679),
    [sym_level_event_keyword] = ACTIONS(1679),
    [sym_data_event_keyword] = ACTIONS(1679),
    [sym_timeline_event_keyword] = ACTIONS(1679),
    [sym_custom_event_keyword] = ACTIONS(1679),
    [sym_push_keyword] = ACTIONS(1679),
    [sym_release_keyword] = ACTIONS(1679),
    [aux_sym_on_keyword_token1] = ACTIONS(1679),
    [aux_sym_off_keyword_token1] = ACTIONS(1679),
    [sym_send_level_keyword] = ACTIONS(1679),
    [sym_send_string_keyword] = ACTIONS(1679),
    [sym_send_command_keyword] = ACTIONS(1679),
    [sym_clear_buffer_keyword] = ACTIONS(1679),
    [sym_create_buffer_keyword] = ACTIONS(1679),
    [sym_create_multi_buffer_keyword] = ACTIONS(1679),
    [sym_call_keyword] = ACTIONS(1679),
    [sym_system_call_keyword] = ACTIONS(1679),
    [sym_devchan_to_keyword] = ACTIONS(1679),
    [sym_devchan_min_to_keyword] = ACTIONS(1679),
    [sym_devchan_total_off_keyword] = ACTIONS(1679),
    [sym_devchan_pulse_keyword] = ACTIONS(1679),
    [sym_wait_keyword] = ACTIONS(1679),
    [sym_cancel_wait_keyword] = ACTIONS(1679),
    [sym_wait_until_keyword] = ACTIONS(1679),
    [sym_cancel_wait_until_keyword] = ACTIONS(1679),
    [sym_cancel_all_wait_keyword] = ACTIONS(1679),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1679),
    [sym_not] = ACTIONS(1679),
    [sym_bnot] = ACTIONS(1679),
    [sym___file__] = ACTIONS(1679),
    [sym___date__] = ACTIONS(1679),
    [sym___ldate__] = ACTIONS(1679),
    [sym___time__] = ACTIONS(1679),
    [sym___line__] = ACTIONS(1679),
    [sym___name__] = ACTIONS(1679),
    [sym___netlinx__] = ACTIONS(1679),
    [sym_netlinx_axi_version] = ACTIONS(1679),
    [sym_url_flg_tcp] = ACTIONS(1679),
    [sym_url_flg_accinfopresent] = ACTIONS(1679),
    [sym_url_flg_temp] = ACTIONS(1679),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1679),
    [sym_url_flg_stat_mask] = ACTIONS(1679),
    [sym_url_flg_stat_lookup] = ACTIONS(1679),
    [sym_url_flg_stat_connecting] = ACTIONS(1679),
    [sym_url_flg_stat_waiting] = ACTIONS(1679),
    [sym_url_flg_stat_connected] = ACTIONS(1679),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1679),
    [sym_first_virtual_device] = ACTIONS(1679),
    [sym_first_local_port] = ACTIONS(1679),
    [sym_dynamic_virtual_device] = ACTIONS(1679),
    [sym_timeline_once] = ACTIONS(1679),
    [sym_timeline_repeat] = ACTIONS(1679),
    [sym_timeline_absolute] = ACTIONS(1679),
    [sym_timeline_relative] = ACTIONS(1679),
    [sym_do_push_timed_infinite] = ACTIONS(1679),
    [sym_source_type_no_address] = ACTIONS(1679),
    [sym_source_type_neuron_id] = ACTIONS(1679),
    [sym_source_type_ip_address] = ACTIONS(1679),
    [sym_source_type_axlink] = ACTIONS(1679),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1679),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1679),
    [sym_source_type_ip_socket_address] = ACTIONS(1679),
    [sym_source_type_rs232] = ACTIONS(1679),
    [sym_source_type_internal] = ACTIONS(1679),
    [sym_source_type_mac_address] = ACTIONS(1679),
    [sym_source_type_ipv4_port] = ACTIONS(1679),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1679),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1679),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1679),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1679),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1679),
    [sym_true] = ACTIONS(1679),
    [sym_false] = ACTIONS(1679),
    [sym_file_read_only] = ACTIONS(1679),
    [sym_file_rw_new] = ACTIONS(1679),
    [sym_file_rw_append] = ACTIONS(1679),
    [sym_ip_tcp] = ACTIONS(1679),
    [sym_ip_udp] = ACTIONS(1679),
    [sym_ip_udp_2way] = ACTIONS(1679),
    [sym_xml_encode_types] = ACTIONS(1679),
    [sym_xml_encode_char_as_list] = ACTIONS(1679),
    [sym_xml_encode_le] = ACTIONS(1679),
    [sym_xml_decode_types] = ACTIONS(1679),
    [sym_xml_decode_no_preserve] = ACTIONS(1679),
    [sym_duet_dev_type_display_device] = ACTIONS(1679),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1679),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1679),
    [sym_duet_dev_type_io_device] = ACTIONS(1679),
    [sym_duet_dev_type_relay_device] = ACTIONS(1679),
    [sym_duet_dev_type_ups] = ACTIONS(1679),
    [sym_duet_dev_type_amplifier] = ACTIONS(1679),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1679),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1679),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1679),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1679),
    [sym_duet_dev_type_camera] = ACTIONS(1679),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1679),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1679),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1679),
    [sym_duet_dev_type_dss] = ACTIONS(1679),
    [sym_duet_dev_type_dvr] = ACTIONS(1679),
    [sym_duet_dev_type_disc_device] = ACTIONS(1679),
    [sym_duet_dev_type_document_camera] = ACTIONS(1679),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1679),
    [sym_duet_dev_type_hvac] = ACTIONS(1679),
    [sym_duet_dev_type_keypad] = ACTIONS(1679),
    [sym_duet_dev_type_light] = ACTIONS(1679),
    [sym_duet_dev_type_monitor] = ACTIONS(1679),
    [sym_duet_dev_type_motor] = ACTIONS(1679),
    [sym_duet_dev_type_multi_window] = ACTIONS(1679),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1679),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1679),
    [sym_duet_dev_type_receiver] = ACTIONS(1679),
    [sym_duet_dev_type_security_system] = ACTIONS(1679),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1679),
    [sym_duet_dev_type_settop_box] = ACTIONS(1679),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1679),
    [sym_duet_dev_type_switcher] = ACTIONS(1679),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1679),
    [sym_duet_dev_type_tv] = ACTIONS(1679),
    [sym_duet_dev_type_utility] = ACTIONS(1679),
    [sym_duet_dev_type_vcr] = ACTIONS(1679),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1679),
    [sym_duet_dev_type_video_processor] = ACTIONS(1679),
    [sym_duet_dev_type_video_projector] = ACTIONS(1679),
    [sym_duet_dev_type_video_wall] = ACTIONS(1679),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1679),
    [sym_duet_dev_type_weather] = ACTIONS(1679),
    [sym_duet_dev_not_polled] = ACTIONS(1679),
    [sym_duet_dev_polled] = ACTIONS(1679),
    [sym_clkmgr_mode_network] = ACTIONS(1679),
    [sym_clkmgr_mode_standalone] = ACTIONS(1679),
    [sym_normal_standby] = ACTIONS(1679),
    [sym_normal_wake] = ACTIONS(1679),
    [sym_ip_multicast_ttl_option] = ACTIONS(1679),
    [sym_ip_ttl_subnet] = ACTIONS(1679),
    [sym_ip_ttl_site] = ACTIONS(1679),
    [sym_ip_ttl_region] = ACTIONS(1679),
    [sym_ip_ttl_continent] = ACTIONS(1679),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1679),
    [sym_ip_nodelay_on] = ACTIONS(1679),
    [sym_ip_nodelay_off] = ACTIONS(1679),
    [sym_amx_error] = ACTIONS(1679),
    [sym_amx_warning] = ACTIONS(1679),
    [sym_amx_info] = ACTIONS(1679),
    [sym_amx_debug] = ACTIONS(1679),
    [sym_netlinx_login_success] = ACTIONS(1679),
    [sym_netlinx_login_fail] = ACTIONS(1679),
    [sym_netlinx_logout] = ACTIONS(1679),
    [sym_null_str] = ACTIONS(1679),
    [sym_smtp_address] = ACTIONS(1679),
    [sym_smtp_port_number] = ACTIONS(1679),
    [sym_smtp_username] = ACTIONS(1679),
    [sym_smtp_password] = ACTIONS(1679),
    [sym_smtp_require_tls] = ACTIONS(1679),
    [sym_smtp_from] = ACTIONS(1679),
    [sym_smtp_tls_true] = ACTIONS(1679),
    [sym_smtp_tls_false] = ACTIONS(1679),
    [sym_listview_on_row_select_event] = ACTIONS(1679),
    [sym_data_structure_unknown] = ACTIONS(1679),
    [sym_data_structure_datafeed] = ACTIONS(1679),
    [sym_data_structure_datarecordset] = ACTIONS(1679),
    [sym_data_structure_datarecord] = ACTIONS(1679),
    [sym_data_structure_datafield] = ACTIONS(1679),
    [sym_data_max_content_field_count] = ACTIONS(1679),
    [sym_data_max_metadata_field_count] = ACTIONS(1679),
    [sym_data_max_id_length] = ACTIONS(1679),
    [sym_data_max_name_length] = ACTIONS(1679),
    [sym_data_max_description_length] = ACTIONS(1679),
    [sym_data_max_type_length] = ACTIONS(1679),
    [sym_data_max_format_length] = ACTIONS(1679),
    [sym_data_max_source_length] = ACTIONS(1679),
    [sym_data_max_value_length] = ACTIONS(1679),
    [sym_data_max_label_length] = ACTIONS(1679),
    [sym_data_max_url_length] = ACTIONS(1679),
    [sym_max_last_login_info_length] = ACTIONS(1679),
    [sym_data_type_unknown] = ACTIONS(1679),
    [sym_data_type_string] = ACTIONS(1679),
    [sym_data_type_datetime] = ACTIONS(1679),
    [sym_data_type_date] = ACTIONS(1679),
    [sym_data_type_time] = ACTIONS(1679),
    [sym_data_type_image] = ACTIONS(1679),
    [sym_data_format_url] = ACTIONS(1679),
    [sym_data_format_phone] = ACTIONS(1679),
    [sym_data_format_email] = ACTIONS(1679),
    [sym_data_format_iso8601] = ACTIONS(1679),
    [sym_auditlog_priv] = ACTIONS(1679),
    [sym_configuration_priv] = ACTIONS(1679),
    [sym_deviceconfig_priv] = ACTIONS(1679),
    [sym_ftp_priv] = ACTIONS(1679),
    [sym_http_priv] = ACTIONS(1679),
    [sym_networkconfig_priv] = ACTIONS(1679),
    [sym_programport_priv] = ACTIONS(1679),
    [sym_remoteui_priv] = ACTIONS(1679),
    [sym_securitycontrol_priv] = ACTIONS(1679),
    [sym_softwaremanagement_priv] = ACTIONS(1679),
    [sym_terminal_priv] = ACTIONS(1679),
    [sym_tpadmin_priv] = ACTIONS(1679),
    [sym_user1_priv] = ACTIONS(1679),
    [sym_user2_priv] = ACTIONS(1679),
    [sym_user3_priv] = ACTIONS(1679),
    [sym_user4_priv] = ACTIONS(1679),
    [sym_usermanagement_priv] = ACTIONS(1679),
    [sym_valid_account] = ACTIONS(1679),
    [sym_err_name_invalid] = ACTIONS(1679),
    [sym_err_password_invalid] = ACTIONS(1679),
    [sym_err_invalid_account] = ACTIONS(1679),
    [sym_err_no_authorization] = ACTIONS(1679),
    [sym_err_invalid_parameter] = ACTIONS(1679),
    [sym_err_user_locked_out] = ACTIONS(1679),
    [sym_err_user_account_expired] = ACTIONS(1679),
    [sym_err_general_error] = ACTIONS(1679),
    [sym_err_authentication_error] = ACTIONS(1679),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1679),
    [sym_tls_validate_certificate] = ACTIONS(1679),
    [sym_tdata] = ACTIONS(1679),
    [sym_tchannel] = ACTIONS(1679),
    [sym_tlevel] = ACTIONS(1679),
    [sym_tbutton] = ACTIONS(1679),
    [sym_ttimeline] = ACTIONS(1679),
    [sym_tcustom] = ACTIONS(1679),
    [sym_url_struct] = ACTIONS(1679),
    [sym_dns_struct] = ACTIONS(1679),
    [sym_ip_address_struct] = ACTIONS(1679),
    [sym_dev_info_struct] = ACTIONS(1679),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1679),
    [sym_data_feed] = ACTIONS(1679),
    [sym_data_field] = ACTIONS(1679),
    [sym_data_record] = ACTIONS(1679),
    [sym_wc_data_feed] = ACTIONS(1679),
    [sym_wc_data_field] = ACTIONS(1679),
    [sym_wc_data_record] = ACTIONS(1679),
    [sym_last_login_info] = ACTIONS(1679),
    [sym____reserved___] = ACTIONS(1679),
    [sym_date] = ACTIONS(1679),
    [sym_day] = ACTIONS(1679),
    [sym_time] = ACTIONS(1679),
    [sym_ldate] = ACTIONS(1679),
    [sym_push_device] = ACTIONS(1679),
    [sym_push_channel] = ACTIONS(1679),
    [sym_push_devchan] = ACTIONS(1679),
    [sym_release_device] = ACTIONS(1679),
    [sym_release_channel] = ACTIONS(1679),
    [sym_release_devchan] = ACTIONS(1679),
    [sym_master_slot] = ACTIONS(1679),
    [sym_get_pulse_time] = ACTIONS(1679),
    [sym_get_timer] = ACTIONS(1679),
    [sym_master_sn] = ACTIONS(1679),
    [sym_system_number] = ACTIONS(1679),
    [sym_button] = ACTIONS(1679),
    [sym_dv_channel] = ACTIONS(1679),
    [sym_data] = ACTIONS(1679),
    [sym_level] = ACTIONS(1679),
    [sym_timeline] = ACTIONS(1679),
    [sym_channel] = ACTIONS(1679),
    [sym_custom] = ACTIONS(1679),
    [sym_atoi] = ACTIONS(1679),
    [sym_atol] = ACTIONS(1679),
    [sym_atof] = ACTIONS(1679),
    [sym_itoa] = ACTIONS(1679),
    [sym_itohex] = ACTIONS(1679),
    [sym_hextoi] = ACTIONS(1679),
    [sym_ftoa] = ACTIONS(1679),
    [sym_find_string] = ACTIONS(1679),
    [sym_left_string] = ACTIONS(1679),
    [sym_right_string] = ACTIONS(1679),
    [sym_mid_string] = ACTIONS(1679),
    [sym_remove_string] = ACTIONS(1679),
    [sym_get_buffer_string] = ACTIONS(1679),
    [sym_get_buffer_char] = ACTIONS(1679),
    [sym_get_multi_buffer_string] = ACTIONS(1679),
    [sym_length_string] = ACTIONS(1679),
    [sym_max_length_string] = ACTIONS(1679),
    [sym_set_length_string] = ACTIONS(1679),
    [sym_compare_string] = ACTIONS(1679),
    [sym_upper_string] = ACTIONS(1679),
    [sym_lower_string] = ACTIONS(1679),
    [sym_get_last] = ACTIONS(1679),
    [sym_date_to_day] = ACTIONS(1679),
    [sym_date_to_month] = ACTIONS(1679),
    [sym_date_to_year] = ACTIONS(1679),
    [sym_time_to_hour] = ACTIONS(1679),
    [sym_time_to_minute] = ACTIONS(1679),
    [sym_time_to_second] = ACTIONS(1679),
    [sym_day_of_week] = ACTIONS(1679),
    [sym_set_pulse_time] = ACTIONS(1679),
    [sym_set_timer] = ACTIONS(1679),
    [sym_file_open] = ACTIONS(1679),
    [sym_file_close] = ACTIONS(1679),
    [sym_file_read] = ACTIONS(1679),
    [sym_file_read_line] = ACTIONS(1679),
    [sym_file_write] = ACTIONS(1679),
    [sym_file_write_line] = ACTIONS(1679),
    [sym_file_copy] = ACTIONS(1679),
    [sym_file_delete] = ACTIONS(1679),
    [sym_file_rename] = ACTIONS(1679),
    [sym_file_dir] = ACTIONS(1679),
    [sym_file_getdir] = ACTIONS(1679),
    [sym_file_setdir] = ACTIONS(1679),
    [sym_file_create_dir] = ACTIONS(1679),
    [sym_file_remove_dir] = ACTIONS(1679),
    [sym_file_seek] = ACTIONS(1679),
    [sym_length_array] = ACTIONS(1679),
    [sym_max_length_array] = ACTIONS(1679),
    [sym_set_length_array] = ACTIONS(1679),
    [sym_ip_client_open] = ACTIONS(1679),
    [sym_ip_client_close] = ACTIONS(1679),
    [sym_ip_server_open] = ACTIONS(1679),
    [sym_ip_server_close] = ACTIONS(1679),
    [sym_ip_bound_client_open] = ACTIONS(1679),
    [sym_ip_mc_server_open] = ACTIONS(1679),
    [sym_ip_set_option] = ACTIONS(1679),
    [sym_ssh_client_open] = ACTIONS(1679),
    [sym_ssh_client_close] = ACTIONS(1679),
    [sym_tls_client_open] = ACTIONS(1679),
    [sym_tls_client_close] = ACTIONS(1679),
    [sym_do_push] = ACTIONS(1679),
    [sym_do_push_timed] = ACTIONS(1679),
    [sym_do_release] = ACTIONS(1679),
    [sym_device_id] = ACTIONS(1679),
    [sym_device_info] = ACTIONS(1679),
    [sym_device_id_string] = ACTIONS(1679),
    [sym_device_standby] = ACTIONS(1679),
    [sym_device_wake] = ACTIONS(1679),
    [sym_timeline_create] = ACTIONS(1679),
    [sym_timeline_kill] = ACTIONS(1679),
    [sym_timeline_reload] = ACTIONS(1679),
    [sym_timeline_pause] = ACTIONS(1679),
    [sym_timeline_restart] = ACTIONS(1679),
    [sym_timeline_set] = ACTIONS(1679),
    [sym_timeline_get] = ACTIONS(1679),
    [sym_timeline_active] = ACTIONS(1679),
    [sym_timeline_dynamic_id] = ACTIONS(1679),
    [sym_variable_to_string] = ACTIONS(1679),
    [sym_string_to_variable] = ACTIONS(1679),
    [sym_length_variable_to_string] = ACTIONS(1679),
    [sym_type_cast] = ACTIONS(1679),
    [sym_variable_to_xml] = ACTIONS(1679),
    [sym_xml_to_variable] = ACTIONS(1679),
    [sym_length_variable_to_xml] = ACTIONS(1679),
    [sym_redirect_string] = ACTIONS(1679),
    [sym_random_number] = ACTIONS(1679),
    [sym_get_unique_id] = ACTIONS(1679),
    [sym_get_system_number] = ACTIONS(1679),
    [sym_set_system_number] = ACTIONS(1679),
    [sym_get_serial_number] = ACTIONS(1679),
    [sym_get_master_build] = ACTIONS(1679),
    [sym_set_validation_code] = ACTIONS(1679),
    [sym_reboot] = ACTIONS(1679),
    [sym_load_duet_module] = ACTIONS(1679),
    [sym_unload_duet_module] = ACTIONS(1679),
    [sym_duet_mem_size_set] = ACTIONS(1679),
    [sym_duet_mem_size_get] = ACTIONS(1679),
    [sym_dynamic_application_device] = ACTIONS(1679),
    [sym_dynamic_polled_port] = ACTIONS(1679),
    [sym_static_port_binding] = ACTIONS(1679),
    [sym_static_ip_binding] = ACTIONS(1679),
    [sym_get_dns_list] = ACTIONS(1679),
    [sym_set_dns_list] = ACTIONS(1679),
    [sym_get_ip_address] = ACTIONS(1679),
    [sym_set_ip_address] = ACTIONS(1679),
    [sym_get_url_list] = ACTIONS(1679),
    [sym_add_url_entry] = ACTIONS(1679),
    [sym_delete_url_entry] = ACTIONS(1679),
    [sym_abs_value] = ACTIONS(1679),
    [sym_max_value] = ACTIONS(1679),
    [sym_min_value] = ACTIONS(1679),
    [sym_exp_value] = ACTIONS(1679),
    [sym_log_value] = ACTIONS(1679),
    [sym_log10_value] = ACTIONS(1679),
    [sym_power_value] = ACTIONS(1679),
    [sym_sqrt_value] = ACTIONS(1679),
    [sym_raw_be] = ACTIONS(1679),
    [sym_raw_le] = ACTIONS(1679),
    [sym_internal_queue_size_set] = ACTIONS(1679),
    [sym_internal_queue_size_get] = ACTIONS(1679),
    [sym_internal_threshold_set] = ACTIONS(1679),
    [sym_internal_threshold_get] = ACTIONS(1679),
    [sym_rebuilt_event] = ACTIONS(1679),
    [sym_do_custom_event] = ACTIONS(1679),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1679),
    [sym_clkmgr_set_clk_source] = ACTIONS(1679),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1679),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1679),
    [sym_clkmgr_get_timezone] = ACTIONS(1679),
    [sym_clkmgr_set_timezone] = ACTIONS(1679),
    [sym_clkmgr_get_resync_period] = ACTIONS(1679),
    [sym_clkmgr_set_resync_period] = ACTIONS(1679),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1679),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1679),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1679),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1679),
    [sym_clkmgr_get_timeservers] = ACTIONS(1679),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1679),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1679),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1679),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1679),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1679),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1679),
    [sym_set_log_level] = ACTIONS(1679),
    [sym_get_log_level] = ACTIONS(1679),
    [sym_amx_log] = ACTIONS(1679),
    [sym_smtp_server_config_set] = ACTIONS(1679),
    [sym_smtp_server_config_get] = ACTIONS(1679),
    [sym_smtp_send] = ACTIONS(1679),
    [sym_validate_netlinx_account] = ACTIONS(1679),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1679),
    [sym_authenticate_certificate] = ACTIONS(1679),
    [sym_audit_netlinx_session_event] = ACTIONS(1679),
    [sym_audit_netlinx_generic_event] = ACTIONS(1679),
    [sym_get_available_flash_disk_space] = ACTIONS(1679),
    [sym_get_max_flash_disk_space] = ACTIONS(1679),
    [sym_data_create_feed] = ACTIONS(1679),
    [sym_data_delete_feed] = ACTIONS(1679),
    [sym_data_publish_feed] = ACTIONS(1679),
    [sym_data_get_published_feed] = ACTIONS(1679),
    [sym_data_add_record] = ACTIONS(1679),
    [sym_data_get_event_record] = ACTIONS(1679),
    [sym__wc_data_create_feed] = ACTIONS(1679),
    [sym__wc_data_add_record] = ACTIONS(1679),
    [sym__wc_data_get_event_record] = ACTIONS(1679),
    [sym_format] = ACTIONS(1679),
    [sym_astro_clock] = ACTIONS(1679),
    [sym_set_outdoor_temperature] = ACTIONS(1679),
    [sym_set_virtual_level_count] = ACTIONS(1679),
    [sym_set_virtual_channel_count] = ACTIONS(1679),
    [sym_set_virtual_port_count] = ACTIONS(1679),
    [sym__wc_encode_int] = ACTIONS(1679),
    [sym__wc_decode_int] = ACTIONS(1679),
    [sym__wc_int] = ACTIONS(1679),
    [sym__wc_to_ch_int] = ACTIONS(1679),
    [sym__ch_to_wc_int] = ACTIONS(1679),
    [sym__wc_find_string_int] = ACTIONS(1679),
    [sym__wc_left_string_int] = ACTIONS(1679),
    [sym__wc_lower_string_int] = ACTIONS(1679),
    [sym__wc_mid_string_int] = ACTIONS(1679),
    [sym__wc_remove_string_int] = ACTIONS(1679),
    [sym__wc_right_string_int] = ACTIONS(1679),
    [sym__wc_upper_string_int] = ACTIONS(1679),
    [sym__wc_compare_string_int] = ACTIONS(1679),
    [sym__wc_get_buffer_char_int] = ACTIONS(1679),
    [sym__wc_get_buffer_string_int] = ACTIONS(1679),
    [sym__wc_concat_string_int] = ACTIONS(1679),
    [sym__wc_file_open_int] = ACTIONS(1679),
    [sym__wc_file_close_int] = ACTIONS(1679),
    [sym__wc_file_read_int] = ACTIONS(1679),
    [sym__wc_file_read_line_int] = ACTIONS(1679),
    [sym__wc_file_write_int] = ACTIONS(1679),
    [sym__wc_file_write_line_int] = ACTIONS(1679),
    [aux_sym_preproc_include_token1] = ACTIONS(1679),
    [aux_sym_preproc_define_token1] = ACTIONS(1681),
    [aux_sym_preproc_warn_token1] = ACTIONS(1681),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1681),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1681),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1681),
    [anon_sym_LBRACK] = ACTIONS(1681),
    [anon_sym_LBRACE] = ACTIONS(1681),
    [anon_sym_LPAREN] = ACTIONS(1679),
    [anon_sym_SEMI] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1681),
    [anon_sym_TILDE] = ACTIONS(1681),
    [anon_sym_DASH] = ACTIONS(1679),
    [anon_sym_PLUS] = ACTIONS(1679),
    [anon_sym_DASH_DASH] = ACTIONS(1681),
    [anon_sym_PLUS_PLUS] = ACTIONS(1681),
    [anon_sym_DQUOTE] = ACTIONS(1681),
    [anon_sym_SQUOTE] = ACTIONS(1681),
    [sym_number_literal] = ACTIONS(1681),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1685),
  },
  [STATE(201)] = {
    [sym_else_clause] = STATE(209),
    [sym_identifier] = ACTIONS(1687),
    [sym_program_name_keyword] = ACTIONS(1687),
    [sym_module_name_keyword] = ACTIONS(1687),
    [sym_define_device_keyword] = ACTIONS(1687),
    [sym_define_combine_keyword] = ACTIONS(1687),
    [sym_define_constant_keyword] = ACTIONS(1687),
    [sym_define_type_keyword] = ACTIONS(1687),
    [sym_define_variable_keyword] = ACTIONS(1687),
    [sym_define_system_variable_keyword] = ACTIONS(1687),
    [sym_define_connect_level_keyword] = ACTIONS(1687),
    [sym_define_latching_keyword] = ACTIONS(1687),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1687),
    [sym_define_toggling_keyword] = ACTIONS(1687),
    [sym_define_call_keyword] = ACTIONS(1687),
    [sym_define_function_keyword] = ACTIONS(1687),
    [sym_define_library_function_keyword] = ACTIONS(1687),
    [sym_define_start_keyword] = ACTIONS(1687),
    [sym_define_event_keyword] = ACTIONS(1687),
    [sym_define_program_keyword] = ACTIONS(1687),
    [sym_define_module_keyword] = ACTIONS(1687),
    [sym_char_keyword] = ACTIONS(1687),
    [sym_widechar_keyword] = ACTIONS(1687),
    [sym_integer_keyword] = ACTIONS(1687),
    [sym_sinteger_keyword] = ACTIONS(1687),
    [sym_long_keyword] = ACTIONS(1687),
    [sym_slong_keyword] = ACTIONS(1687),
    [sym_float_keyword] = ACTIONS(1687),
    [sym_double_keyword] = ACTIONS(1687),
    [sym_variant_keyword] = ACTIONS(1687),
    [sym_variantarray_keyword] = ACTIONS(1687),
    [sym_local_var_keyword] = ACTIONS(1687),
    [sym_stack_var_keyword] = ACTIONS(1687),
    [sym_constant_keyword] = ACTIONS(1687),
    [sym_volatile_keyword] = ACTIONS(1687),
    [sym_non_volatile_keyword] = ACTIONS(1687),
    [sym_persistent_keyword] = ACTIONS(1687),
    [sym_dev_keyword] = ACTIONS(1687),
    [sym_devlev_keyword] = ACTIONS(1687),
    [sym_devchan_keyword] = ACTIONS(1687),
    [sym_if_keyword] = ACTIONS(1687),
    [sym_else_keyword] = ACTIONS(1689),
    [sym_switch_keyword] = ACTIONS(1687),
    [sym_case_keyword] = ACTIONS(1687),
    [sym_default_keyword] = ACTIONS(1687),
    [sym_while_keyword] = ACTIONS(1687),
    [sym_for_keyword] = ACTIONS(1687),
    [sym_break_keyword] = ACTIONS(1687),
    [sym_continue_keyword] = ACTIONS(1687),
    [sym_return_keyword] = ACTIONS(1687),
    [sym_select_keyword] = ACTIONS(1687),
    [sym_active_keyword] = ACTIONS(1687),
    [sym_struct_keyword] = ACTIONS(1687),
    [sym_structure_keyword] = ACTIONS(1687),
    [sym_button_event_keyword] = ACTIONS(1687),
    [sym_channel_event_keyword] = ACTIONS(1687),
    [sym_level_event_keyword] = ACTIONS(1687),
    [sym_data_event_keyword] = ACTIONS(1687),
    [sym_timeline_event_keyword] = ACTIONS(1687),
    [sym_custom_event_keyword] = ACTIONS(1687),
    [sym_push_keyword] = ACTIONS(1687),
    [sym_release_keyword] = ACTIONS(1687),
    [aux_sym_on_keyword_token1] = ACTIONS(1687),
    [aux_sym_off_keyword_token1] = ACTIONS(1687),
    [sym_send_level_keyword] = ACTIONS(1687),
    [sym_send_string_keyword] = ACTIONS(1687),
    [sym_send_command_keyword] = ACTIONS(1687),
    [sym_clear_buffer_keyword] = ACTIONS(1687),
    [sym_create_buffer_keyword] = ACTIONS(1687),
    [sym_create_multi_buffer_keyword] = ACTIONS(1687),
    [sym_call_keyword] = ACTIONS(1687),
    [sym_system_call_keyword] = ACTIONS(1687),
    [sym_devchan_to_keyword] = ACTIONS(1687),
    [sym_devchan_min_to_keyword] = ACTIONS(1687),
    [sym_devchan_total_off_keyword] = ACTIONS(1687),
    [sym_devchan_pulse_keyword] = ACTIONS(1687),
    [sym_wait_keyword] = ACTIONS(1687),
    [sym_cancel_wait_keyword] = ACTIONS(1687),
    [sym_wait_until_keyword] = ACTIONS(1687),
    [sym_cancel_wait_until_keyword] = ACTIONS(1687),
    [sym_cancel_all_wait_keyword] = ACTIONS(1687),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1687),
    [sym_not] = ACTIONS(1687),
    [sym_bnot] = ACTIONS(1687),
    [sym___file__] = ACTIONS(1687),
    [sym___date__] = ACTIONS(1687),
    [sym___ldate__] = ACTIONS(1687),
    [sym___time__] = ACTIONS(1687),
    [sym___line__] = ACTIONS(1687),
    [sym___name__] = ACTIONS(1687),
    [sym___netlinx__] = ACTIONS(1687),
    [sym_netlinx_axi_version] = ACTIONS(1687),
    [sym_url_flg_tcp] = ACTIONS(1687),
    [sym_url_flg_accinfopresent] = ACTIONS(1687),
    [sym_url_flg_temp] = ACTIONS(1687),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1687),
    [sym_url_flg_stat_mask] = ACTIONS(1687),
    [sym_url_flg_stat_lookup] = ACTIONS(1687),
    [sym_url_flg_stat_connecting] = ACTIONS(1687),
    [sym_url_flg_stat_waiting] = ACTIONS(1687),
    [sym_url_flg_stat_connected] = ACTIONS(1687),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1687),
    [sym_first_virtual_device] = ACTIONS(1687),
    [sym_first_local_port] = ACTIONS(1687),
    [sym_dynamic_virtual_device] = ACTIONS(1687),
    [sym_timeline_once] = ACTIONS(1687),
    [sym_timeline_repeat] = ACTIONS(1687),
    [sym_timeline_absolute] = ACTIONS(1687),
    [sym_timeline_relative] = ACTIONS(1687),
    [sym_do_push_timed_infinite] = ACTIONS(1687),
    [sym_source_type_no_address] = ACTIONS(1687),
    [sym_source_type_neuron_id] = ACTIONS(1687),
    [sym_source_type_ip_address] = ACTIONS(1687),
    [sym_source_type_axlink] = ACTIONS(1687),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1687),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1687),
    [sym_source_type_ip_socket_address] = ACTIONS(1687),
    [sym_source_type_rs232] = ACTIONS(1687),
    [sym_source_type_internal] = ACTIONS(1687),
    [sym_source_type_mac_address] = ACTIONS(1687),
    [sym_source_type_ipv4_port] = ACTIONS(1687),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1687),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1687),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1687),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1687),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1687),
    [sym_true] = ACTIONS(1687),
    [sym_false] = ACTIONS(1687),
    [sym_file_read_only] = ACTIONS(1687),
    [sym_file_rw_new] = ACTIONS(1687),
    [sym_file_rw_append] = ACTIONS(1687),
    [sym_ip_tcp] = ACTIONS(1687),
    [sym_ip_udp] = ACTIONS(1687),
    [sym_ip_udp_2way] = ACTIONS(1687),
    [sym_xml_encode_types] = ACTIONS(1687),
    [sym_xml_encode_char_as_list] = ACTIONS(1687),
    [sym_xml_encode_le] = ACTIONS(1687),
    [sym_xml_decode_types] = ACTIONS(1687),
    [sym_xml_decode_no_preserve] = ACTIONS(1687),
    [sym_duet_dev_type_display_device] = ACTIONS(1687),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1687),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1687),
    [sym_duet_dev_type_io_device] = ACTIONS(1687),
    [sym_duet_dev_type_relay_device] = ACTIONS(1687),
    [sym_duet_dev_type_ups] = ACTIONS(1687),
    [sym_duet_dev_type_amplifier] = ACTIONS(1687),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1687),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1687),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1687),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1687),
    [sym_duet_dev_type_camera] = ACTIONS(1687),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1687),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1687),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1687),
    [sym_duet_dev_type_dss] = ACTIONS(1687),
    [sym_duet_dev_type_dvr] = ACTIONS(1687),
    [sym_duet_dev_type_disc_device] = ACTIONS(1687),
    [sym_duet_dev_type_document_camera] = ACTIONS(1687),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1687),
    [sym_duet_dev_type_hvac] = ACTIONS(1687),
    [sym_duet_dev_type_keypad] = ACTIONS(1687),
    [sym_duet_dev_type_light] = ACTIONS(1687),
    [sym_duet_dev_type_monitor] = ACTIONS(1687),
    [sym_duet_dev_type_motor] = ACTIONS(1687),
    [sym_duet_dev_type_multi_window] = ACTIONS(1687),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1687),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1687),
    [sym_duet_dev_type_receiver] = ACTIONS(1687),
    [sym_duet_dev_type_security_system] = ACTIONS(1687),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1687),
    [sym_duet_dev_type_settop_box] = ACTIONS(1687),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1687),
    [sym_duet_dev_type_switcher] = ACTIONS(1687),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1687),
    [sym_duet_dev_type_tv] = ACTIONS(1687),
    [sym_duet_dev_type_utility] = ACTIONS(1687),
    [sym_duet_dev_type_vcr] = ACTIONS(1687),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1687),
    [sym_duet_dev_type_video_processor] = ACTIONS(1687),
    [sym_duet_dev_type_video_projector] = ACTIONS(1687),
    [sym_duet_dev_type_video_wall] = ACTIONS(1687),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1687),
    [sym_duet_dev_type_weather] = ACTIONS(1687),
    [sym_duet_dev_not_polled] = ACTIONS(1687),
    [sym_duet_dev_polled] = ACTIONS(1687),
    [sym_clkmgr_mode_network] = ACTIONS(1687),
    [sym_clkmgr_mode_standalone] = ACTIONS(1687),
    [sym_normal_standby] = ACTIONS(1687),
    [sym_normal_wake] = ACTIONS(1687),
    [sym_ip_multicast_ttl_option] = ACTIONS(1687),
    [sym_ip_ttl_subnet] = ACTIONS(1687),
    [sym_ip_ttl_site] = ACTIONS(1687),
    [sym_ip_ttl_region] = ACTIONS(1687),
    [sym_ip_ttl_continent] = ACTIONS(1687),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1687),
    [sym_ip_nodelay_on] = ACTIONS(1687),
    [sym_ip_nodelay_off] = ACTIONS(1687),
    [sym_amx_error] = ACTIONS(1687),
    [sym_amx_warning] = ACTIONS(1687),
    [sym_amx_info] = ACTIONS(1687),
    [sym_amx_debug] = ACTIONS(1687),
    [sym_netlinx_login_success] = ACTIONS(1687),
    [sym_netlinx_login_fail] = ACTIONS(1687),
    [sym_netlinx_logout] = ACTIONS(1687),
    [sym_null_str] = ACTIONS(1687),
    [sym_smtp_address] = ACTIONS(1687),
    [sym_smtp_port_number] = ACTIONS(1687),
    [sym_smtp_username] = ACTIONS(1687),
    [sym_smtp_password] = ACTIONS(1687),
    [sym_smtp_require_tls] = ACTIONS(1687),
    [sym_smtp_from] = ACTIONS(1687),
    [sym_smtp_tls_true] = ACTIONS(1687),
    [sym_smtp_tls_false] = ACTIONS(1687),
    [sym_listview_on_row_select_event] = ACTIONS(1687),
    [sym_data_structure_unknown] = ACTIONS(1687),
    [sym_data_structure_datafeed] = ACTIONS(1687),
    [sym_data_structure_datarecordset] = ACTIONS(1687),
    [sym_data_structure_datarecord] = ACTIONS(1687),
    [sym_data_structure_datafield] = ACTIONS(1687),
    [sym_data_max_content_field_count] = ACTIONS(1687),
    [sym_data_max_metadata_field_count] = ACTIONS(1687),
    [sym_data_max_id_length] = ACTIONS(1687),
    [sym_data_max_name_length] = ACTIONS(1687),
    [sym_data_max_description_length] = ACTIONS(1687),
    [sym_data_max_type_length] = ACTIONS(1687),
    [sym_data_max_format_length] = ACTIONS(1687),
    [sym_data_max_source_length] = ACTIONS(1687),
    [sym_data_max_value_length] = ACTIONS(1687),
    [sym_data_max_label_length] = ACTIONS(1687),
    [sym_data_max_url_length] = ACTIONS(1687),
    [sym_max_last_login_info_length] = ACTIONS(1687),
    [sym_data_type_unknown] = ACTIONS(1687),
    [sym_data_type_string] = ACTIONS(1687),
    [sym_data_type_datetime] = ACTIONS(1687),
    [sym_data_type_date] = ACTIONS(1687),
    [sym_data_type_time] = ACTIONS(1687),
    [sym_data_type_image] = ACTIONS(1687),
    [sym_data_format_url] = ACTIONS(1687),
    [sym_data_format_phone] = ACTIONS(1687),
    [sym_data_format_email] = ACTIONS(1687),
    [sym_data_format_iso8601] = ACTIONS(1687),
    [sym_auditlog_priv] = ACTIONS(1687),
    [sym_configuration_priv] = ACTIONS(1687),
    [sym_deviceconfig_priv] = ACTIONS(1687),
    [sym_ftp_priv] = ACTIONS(1687),
    [sym_http_priv] = ACTIONS(1687),
    [sym_networkconfig_priv] = ACTIONS(1687),
    [sym_programport_priv] = ACTIONS(1687),
    [sym_remoteui_priv] = ACTIONS(1687),
    [sym_securitycontrol_priv] = ACTIONS(1687),
    [sym_softwaremanagement_priv] = ACTIONS(1687),
    [sym_terminal_priv] = ACTIONS(1687),
    [sym_tpadmin_priv] = ACTIONS(1687),
    [sym_user1_priv] = ACTIONS(1687),
    [sym_user2_priv] = ACTIONS(1687),
    [sym_user3_priv] = ACTIONS(1687),
    [sym_user4_priv] = ACTIONS(1687),
    [sym_usermanagement_priv] = ACTIONS(1687),
    [sym_valid_account] = ACTIONS(1687),
    [sym_err_name_invalid] = ACTIONS(1687),
    [sym_err_password_invalid] = ACTIONS(1687),
    [sym_err_invalid_account] = ACTIONS(1687),
    [sym_err_no_authorization] = ACTIONS(1687),
    [sym_err_invalid_parameter] = ACTIONS(1687),
    [sym_err_user_locked_out] = ACTIONS(1687),
    [sym_err_user_account_expired] = ACTIONS(1687),
    [sym_err_general_error] = ACTIONS(1687),
    [sym_err_authentication_error] = ACTIONS(1687),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1687),
    [sym_tls_validate_certificate] = ACTIONS(1687),
    [sym_tdata] = ACTIONS(1687),
    [sym_tchannel] = ACTIONS(1687),
    [sym_tlevel] = ACTIONS(1687),
    [sym_tbutton] = ACTIONS(1687),
    [sym_ttimeline] = ACTIONS(1687),
    [sym_tcustom] = ACTIONS(1687),
    [sym_url_struct] = ACTIONS(1687),
    [sym_dns_struct] = ACTIONS(1687),
    [sym_ip_address_struct] = ACTIONS(1687),
    [sym_dev_info_struct] = ACTIONS(1687),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1687),
    [sym_data_feed] = ACTIONS(1687),
    [sym_data_field] = ACTIONS(1687),
    [sym_data_record] = ACTIONS(1687),
    [sym_wc_data_feed] = ACTIONS(1687),
    [sym_wc_data_field] = ACTIONS(1687),
    [sym_wc_data_record] = ACTIONS(1687),
    [sym_last_login_info] = ACTIONS(1687),
    [sym____reserved___] = ACTIONS(1687),
    [sym_date] = ACTIONS(1687),
    [sym_day] = ACTIONS(1687),
    [sym_time] = ACTIONS(1687),
    [sym_ldate] = ACTIONS(1687),
    [sym_push_device] = ACTIONS(1687),
    [sym_push_channel] = ACTIONS(1687),
    [sym_push_devchan] = ACTIONS(1687),
    [sym_release_device] = ACTIONS(1687),
    [sym_release_channel] = ACTIONS(1687),
    [sym_release_devchan] = ACTIONS(1687),
    [sym_master_slot] = ACTIONS(1687),
    [sym_get_pulse_time] = ACTIONS(1687),
    [sym_get_timer] = ACTIONS(1687),
    [sym_master_sn] = ACTIONS(1687),
    [sym_system_number] = ACTIONS(1687),
    [sym_button] = ACTIONS(1687),
    [sym_dv_channel] = ACTIONS(1687),
    [sym_data] = ACTIONS(1687),
    [sym_level] = ACTIONS(1687),
    [sym_timeline] = ACTIONS(1687),
    [sym_channel] = ACTIONS(1687),
    [sym_custom] = ACTIONS(1687),
    [sym_atoi] = ACTIONS(1687),
    [sym_atol] = ACTIONS(1687),
    [sym_atof] = ACTIONS(1687),
    [sym_itoa] = ACTIONS(1687),
    [sym_itohex] = ACTIONS(1687),
    [sym_hextoi] = ACTIONS(1687),
    [sym_ftoa] = ACTIONS(1687),
    [sym_find_string] = ACTIONS(1687),
    [sym_left_string] = ACTIONS(1687),
    [sym_right_string] = ACTIONS(1687),
    [sym_mid_string] = ACTIONS(1687),
    [sym_remove_string] = ACTIONS(1687),
    [sym_get_buffer_string] = ACTIONS(1687),
    [sym_get_buffer_char] = ACTIONS(1687),
    [sym_get_multi_buffer_string] = ACTIONS(1687),
    [sym_length_string] = ACTIONS(1687),
    [sym_max_length_string] = ACTIONS(1687),
    [sym_set_length_string] = ACTIONS(1687),
    [sym_compare_string] = ACTIONS(1687),
    [sym_upper_string] = ACTIONS(1687),
    [sym_lower_string] = ACTIONS(1687),
    [sym_get_last] = ACTIONS(1687),
    [sym_date_to_day] = ACTIONS(1687),
    [sym_date_to_month] = ACTIONS(1687),
    [sym_date_to_year] = ACTIONS(1687),
    [sym_time_to_hour] = ACTIONS(1687),
    [sym_time_to_minute] = ACTIONS(1687),
    [sym_time_to_second] = ACTIONS(1687),
    [sym_day_of_week] = ACTIONS(1687),
    [sym_set_pulse_time] = ACTIONS(1687),
    [sym_set_timer] = ACTIONS(1687),
    [sym_file_open] = ACTIONS(1687),
    [sym_file_close] = ACTIONS(1687),
    [sym_file_read] = ACTIONS(1687),
    [sym_file_read_line] = ACTIONS(1687),
    [sym_file_write] = ACTIONS(1687),
    [sym_file_write_line] = ACTIONS(1687),
    [sym_file_copy] = ACTIONS(1687),
    [sym_file_delete] = ACTIONS(1687),
    [sym_file_rename] = ACTIONS(1687),
    [sym_file_dir] = ACTIONS(1687),
    [sym_file_getdir] = ACTIONS(1687),
    [sym_file_setdir] = ACTIONS(1687),
    [sym_file_create_dir] = ACTIONS(1687),
    [sym_file_remove_dir] = ACTIONS(1687),
    [sym_file_seek] = ACTIONS(1687),
    [sym_length_array] = ACTIONS(1687),
    [sym_max_length_array] = ACTIONS(1687),
    [sym_set_length_array] = ACTIONS(1687),
    [sym_ip_client_open] = ACTIONS(1687),
    [sym_ip_client_close] = ACTIONS(1687),
    [sym_ip_server_open] = ACTIONS(1687),
    [sym_ip_server_close] = ACTIONS(1687),
    [sym_ip_bound_client_open] = ACTIONS(1687),
    [sym_ip_mc_server_open] = ACTIONS(1687),
    [sym_ip_set_option] = ACTIONS(1687),
    [sym_ssh_client_open] = ACTIONS(1687),
    [sym_ssh_client_close] = ACTIONS(1687),
    [sym_tls_client_open] = ACTIONS(1687),
    [sym_tls_client_close] = ACTIONS(1687),
    [sym_do_push] = ACTIONS(1687),
    [sym_do_push_timed] = ACTIONS(1687),
    [sym_do_release] = ACTIONS(1687),
    [sym_device_id] = ACTIONS(1687),
    [sym_device_info] = ACTIONS(1687),
    [sym_device_id_string] = ACTIONS(1687),
    [sym_device_standby] = ACTIONS(1687),
    [sym_device_wake] = ACTIONS(1687),
    [sym_timeline_create] = ACTIONS(1687),
    [sym_timeline_kill] = ACTIONS(1687),
    [sym_timeline_reload] = ACTIONS(1687),
    [sym_timeline_pause] = ACTIONS(1687),
    [sym_timeline_restart] = ACTIONS(1687),
    [sym_timeline_set] = ACTIONS(1687),
    [sym_timeline_get] = ACTIONS(1687),
    [sym_timeline_active] = ACTIONS(1687),
    [sym_timeline_dynamic_id] = ACTIONS(1687),
    [sym_variable_to_string] = ACTIONS(1687),
    [sym_string_to_variable] = ACTIONS(1687),
    [sym_length_variable_to_string] = ACTIONS(1687),
    [sym_type_cast] = ACTIONS(1687),
    [sym_variable_to_xml] = ACTIONS(1687),
    [sym_xml_to_variable] = ACTIONS(1687),
    [sym_length_variable_to_xml] = ACTIONS(1687),
    [sym_redirect_string] = ACTIONS(1687),
    [sym_random_number] = ACTIONS(1687),
    [sym_get_unique_id] = ACTIONS(1687),
    [sym_get_system_number] = ACTIONS(1687),
    [sym_set_system_number] = ACTIONS(1687),
    [sym_get_serial_number] = ACTIONS(1687),
    [sym_get_master_build] = ACTIONS(1687),
    [sym_set_validation_code] = ACTIONS(1687),
    [sym_reboot] = ACTIONS(1687),
    [sym_load_duet_module] = ACTIONS(1687),
    [sym_unload_duet_module] = ACTIONS(1687),
    [sym_duet_mem_size_set] = ACTIONS(1687),
    [sym_duet_mem_size_get] = ACTIONS(1687),
    [sym_dynamic_application_device] = ACTIONS(1687),
    [sym_dynamic_polled_port] = ACTIONS(1687),
    [sym_static_port_binding] = ACTIONS(1687),
    [sym_static_ip_binding] = ACTIONS(1687),
    [sym_get_dns_list] = ACTIONS(1687),
    [sym_set_dns_list] = ACTIONS(1687),
    [sym_get_ip_address] = ACTIONS(1687),
    [sym_set_ip_address] = ACTIONS(1687),
    [sym_get_url_list] = ACTIONS(1687),
    [sym_add_url_entry] = ACTIONS(1687),
    [sym_delete_url_entry] = ACTIONS(1687),
    [sym_abs_value] = ACTIONS(1687),
    [sym_max_value] = ACTIONS(1687),
    [sym_min_value] = ACTIONS(1687),
    [sym_exp_value] = ACTIONS(1687),
    [sym_log_value] = ACTIONS(1687),
    [sym_log10_value] = ACTIONS(1687),
    [sym_power_value] = ACTIONS(1687),
    [sym_sqrt_value] = ACTIONS(1687),
    [sym_raw_be] = ACTIONS(1687),
    [sym_raw_le] = ACTIONS(1687),
    [sym_internal_queue_size_set] = ACTIONS(1687),
    [sym_internal_queue_size_get] = ACTIONS(1687),
    [sym_internal_threshold_set] = ACTIONS(1687),
    [sym_internal_threshold_get] = ACTIONS(1687),
    [sym_rebuilt_event] = ACTIONS(1687),
    [sym_do_custom_event] = ACTIONS(1687),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1687),
    [sym_clkmgr_set_clk_source] = ACTIONS(1687),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1687),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1687),
    [sym_clkmgr_get_timezone] = ACTIONS(1687),
    [sym_clkmgr_set_timezone] = ACTIONS(1687),
    [sym_clkmgr_get_resync_period] = ACTIONS(1687),
    [sym_clkmgr_set_resync_period] = ACTIONS(1687),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1687),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1687),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1687),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1687),
    [sym_clkmgr_get_timeservers] = ACTIONS(1687),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1687),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1687),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1687),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1687),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1687),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1687),
    [sym_set_log_level] = ACTIONS(1687),
    [sym_get_log_level] = ACTIONS(1687),
    [sym_amx_log] = ACTIONS(1687),
    [sym_smtp_server_config_set] = ACTIONS(1687),
    [sym_smtp_server_config_get] = ACTIONS(1687),
    [sym_smtp_send] = ACTIONS(1687),
    [sym_validate_netlinx_account] = ACTIONS(1687),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1687),
    [sym_authenticate_certificate] = ACTIONS(1687),
    [sym_audit_netlinx_session_event] = ACTIONS(1687),
    [sym_audit_netlinx_generic_event] = ACTIONS(1687),
    [sym_get_available_flash_disk_space] = ACTIONS(1687),
    [sym_get_max_flash_disk_space] = ACTIONS(1687),
    [sym_data_create_feed] = ACTIONS(1687),
    [sym_data_delete_feed] = ACTIONS(1687),
    [sym_data_publish_feed] = ACTIONS(1687),
    [sym_data_get_published_feed] = ACTIONS(1687),
    [sym_data_add_record] = ACTIONS(1687),
    [sym_data_get_event_record] = ACTIONS(1687),
    [sym__wc_data_create_feed] = ACTIONS(1687),
    [sym__wc_data_add_record] = ACTIONS(1687),
    [sym__wc_data_get_event_record] = ACTIONS(1687),
    [sym_format] = ACTIONS(1687),
    [sym_astro_clock] = ACTIONS(1687),
    [sym_set_outdoor_temperature] = ACTIONS(1687),
    [sym_set_virtual_level_count] = ACTIONS(1687),
    [sym_set_virtual_channel_count] = ACTIONS(1687),
    [sym_set_virtual_port_count] = ACTIONS(1687),
    [sym__wc_encode_int] = ACTIONS(1687),
    [sym__wc_decode_int] = ACTIONS(1687),
    [sym__wc_int] = ACTIONS(1687),
    [sym__wc_to_ch_int] = ACTIONS(1687),
    [sym__ch_to_wc_int] = ACTIONS(1687),
    [sym__wc_find_string_int] = ACTIONS(1687),
    [sym__wc_left_string_int] = ACTIONS(1687),
    [sym__wc_lower_string_int] = ACTIONS(1687),
    [sym__wc_mid_string_int] = ACTIONS(1687),
    [sym__wc_remove_string_int] = ACTIONS(1687),
    [sym__wc_right_string_int] = ACTIONS(1687),
    [sym__wc_upper_string_int] = ACTIONS(1687),
    [sym__wc_compare_string_int] = ACTIONS(1687),
    [sym__wc_get_buffer_char_int] = ACTIONS(1687),
    [sym__wc_get_buffer_string_int] = ACTIONS(1687),
    [sym__wc_concat_string_int] = ACTIONS(1687),
    [sym__wc_file_open_int] = ACTIONS(1687),
    [sym__wc_file_close_int] = ACTIONS(1687),
    [sym__wc_file_read_int] = ACTIONS(1687),
    [sym__wc_file_read_line_int] = ACTIONS(1687),
    [sym__wc_file_write_int] = ACTIONS(1687),
    [sym__wc_file_write_line_int] = ACTIONS(1687),
    [aux_sym_preproc_include_token1] = ACTIONS(1687),
    [aux_sym_preproc_define_token1] = ACTIONS(1691),
    [aux_sym_preproc_warn_token1] = ACTIONS(1691),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1691),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1691),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1691),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1691),
    [aux_sym_preproc_else_token1] = ACTIONS(1691),
    [anon_sym_LBRACK] = ACTIONS(1691),
    [anon_sym_LBRACE] = ACTIONS(1691),
    [anon_sym_LPAREN] = ACTIONS(1687),
    [anon_sym_BANG] = ACTIONS(1691),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1687),
    [anon_sym_PLUS] = ACTIONS(1687),
    [anon_sym_DASH_DASH] = ACTIONS(1691),
    [anon_sym_PLUS_PLUS] = ACTIONS(1691),
    [anon_sym_DQUOTE] = ACTIONS(1691),
    [anon_sym_SQUOTE] = ACTIONS(1691),
    [sym_number_literal] = ACTIONS(1691),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(202)] = {
    [ts_builtin_sym_end] = ACTIONS(1675),
    [sym_identifier] = ACTIONS(1673),
    [sym_program_name_keyword] = ACTIONS(1673),
    [sym_module_name_keyword] = ACTIONS(1673),
    [sym_define_device_keyword] = ACTIONS(1673),
    [sym_define_combine_keyword] = ACTIONS(1673),
    [sym_define_constant_keyword] = ACTIONS(1673),
    [sym_define_type_keyword] = ACTIONS(1673),
    [sym_define_variable_keyword] = ACTIONS(1673),
    [sym_define_system_variable_keyword] = ACTIONS(1673),
    [sym_define_connect_level_keyword] = ACTIONS(1673),
    [sym_define_latching_keyword] = ACTIONS(1673),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1673),
    [sym_define_toggling_keyword] = ACTIONS(1673),
    [sym_define_call_keyword] = ACTIONS(1673),
    [sym_define_function_keyword] = ACTIONS(1673),
    [sym_define_library_function_keyword] = ACTIONS(1673),
    [sym_define_start_keyword] = ACTIONS(1673),
    [sym_define_event_keyword] = ACTIONS(1673),
    [sym_define_program_keyword] = ACTIONS(1673),
    [sym_define_module_keyword] = ACTIONS(1673),
    [sym_char_keyword] = ACTIONS(1673),
    [sym_widechar_keyword] = ACTIONS(1673),
    [sym_integer_keyword] = ACTIONS(1673),
    [sym_sinteger_keyword] = ACTIONS(1673),
    [sym_long_keyword] = ACTIONS(1673),
    [sym_slong_keyword] = ACTIONS(1673),
    [sym_float_keyword] = ACTIONS(1673),
    [sym_double_keyword] = ACTIONS(1673),
    [sym_variant_keyword] = ACTIONS(1673),
    [sym_variantarray_keyword] = ACTIONS(1673),
    [sym_local_var_keyword] = ACTIONS(1673),
    [sym_stack_var_keyword] = ACTIONS(1673),
    [sym_constant_keyword] = ACTIONS(1673),
    [sym_volatile_keyword] = ACTIONS(1673),
    [sym_non_volatile_keyword] = ACTIONS(1673),
    [sym_persistent_keyword] = ACTIONS(1673),
    [sym_dev_keyword] = ACTIONS(1673),
    [sym_devlev_keyword] = ACTIONS(1673),
    [sym_devchan_keyword] = ACTIONS(1673),
    [sym_if_keyword] = ACTIONS(1673),
    [sym_else_keyword] = ACTIONS(1673),
    [sym_switch_keyword] = ACTIONS(1673),
    [sym_case_keyword] = ACTIONS(1673),
    [sym_default_keyword] = ACTIONS(1673),
    [sym_while_keyword] = ACTIONS(1673),
    [sym_for_keyword] = ACTIONS(1673),
    [sym_break_keyword] = ACTIONS(1673),
    [sym_continue_keyword] = ACTIONS(1673),
    [sym_return_keyword] = ACTIONS(1673),
    [sym_select_keyword] = ACTIONS(1673),
    [sym_active_keyword] = ACTIONS(1673),
    [sym_struct_keyword] = ACTIONS(1673),
    [sym_structure_keyword] = ACTIONS(1673),
    [sym_button_event_keyword] = ACTIONS(1673),
    [sym_channel_event_keyword] = ACTIONS(1673),
    [sym_level_event_keyword] = ACTIONS(1673),
    [sym_data_event_keyword] = ACTIONS(1673),
    [sym_timeline_event_keyword] = ACTIONS(1673),
    [sym_custom_event_keyword] = ACTIONS(1673),
    [sym_push_keyword] = ACTIONS(1673),
    [sym_release_keyword] = ACTIONS(1673),
    [aux_sym_on_keyword_token1] = ACTIONS(1673),
    [aux_sym_off_keyword_token1] = ACTIONS(1673),
    [sym_send_level_keyword] = ACTIONS(1673),
    [sym_send_string_keyword] = ACTIONS(1673),
    [sym_send_command_keyword] = ACTIONS(1673),
    [sym_clear_buffer_keyword] = ACTIONS(1673),
    [sym_create_buffer_keyword] = ACTIONS(1673),
    [sym_create_multi_buffer_keyword] = ACTIONS(1673),
    [sym_call_keyword] = ACTIONS(1673),
    [sym_system_call_keyword] = ACTIONS(1673),
    [sym_devchan_to_keyword] = ACTIONS(1673),
    [sym_devchan_min_to_keyword] = ACTIONS(1673),
    [sym_devchan_total_off_keyword] = ACTIONS(1673),
    [sym_devchan_pulse_keyword] = ACTIONS(1673),
    [sym_wait_keyword] = ACTIONS(1673),
    [sym_cancel_wait_keyword] = ACTIONS(1673),
    [sym_wait_until_keyword] = ACTIONS(1673),
    [sym_cancel_wait_until_keyword] = ACTIONS(1673),
    [sym_cancel_all_wait_keyword] = ACTIONS(1673),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1673),
    [sym_not] = ACTIONS(1673),
    [sym_bnot] = ACTIONS(1673),
    [sym___file__] = ACTIONS(1673),
    [sym___date__] = ACTIONS(1673),
    [sym___ldate__] = ACTIONS(1673),
    [sym___time__] = ACTIONS(1673),
    [sym___line__] = ACTIONS(1673),
    [sym___name__] = ACTIONS(1673),
    [sym___netlinx__] = ACTIONS(1673),
    [sym_netlinx_axi_version] = ACTIONS(1673),
    [sym_url_flg_tcp] = ACTIONS(1673),
    [sym_url_flg_accinfopresent] = ACTIONS(1673),
    [sym_url_flg_temp] = ACTIONS(1673),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1673),
    [sym_url_flg_stat_mask] = ACTIONS(1673),
    [sym_url_flg_stat_lookup] = ACTIONS(1673),
    [sym_url_flg_stat_connecting] = ACTIONS(1673),
    [sym_url_flg_stat_waiting] = ACTIONS(1673),
    [sym_url_flg_stat_connected] = ACTIONS(1673),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1673),
    [sym_first_virtual_device] = ACTIONS(1673),
    [sym_first_local_port] = ACTIONS(1673),
    [sym_dynamic_virtual_device] = ACTIONS(1673),
    [sym_timeline_once] = ACTIONS(1673),
    [sym_timeline_repeat] = ACTIONS(1673),
    [sym_timeline_absolute] = ACTIONS(1673),
    [sym_timeline_relative] = ACTIONS(1673),
    [sym_do_push_timed_infinite] = ACTIONS(1673),
    [sym_source_type_no_address] = ACTIONS(1673),
    [sym_source_type_neuron_id] = ACTIONS(1673),
    [sym_source_type_ip_address] = ACTIONS(1673),
    [sym_source_type_axlink] = ACTIONS(1673),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1673),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1673),
    [sym_source_type_ip_socket_address] = ACTIONS(1673),
    [sym_source_type_rs232] = ACTIONS(1673),
    [sym_source_type_internal] = ACTIONS(1673),
    [sym_source_type_mac_address] = ACTIONS(1673),
    [sym_source_type_ipv4_port] = ACTIONS(1673),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1673),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1673),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1673),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1673),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1673),
    [sym_true] = ACTIONS(1673),
    [sym_false] = ACTIONS(1673),
    [sym_file_read_only] = ACTIONS(1673),
    [sym_file_rw_new] = ACTIONS(1673),
    [sym_file_rw_append] = ACTIONS(1673),
    [sym_ip_tcp] = ACTIONS(1673),
    [sym_ip_udp] = ACTIONS(1673),
    [sym_ip_udp_2way] = ACTIONS(1673),
    [sym_xml_encode_types] = ACTIONS(1673),
    [sym_xml_encode_char_as_list] = ACTIONS(1673),
    [sym_xml_encode_le] = ACTIONS(1673),
    [sym_xml_decode_types] = ACTIONS(1673),
    [sym_xml_decode_no_preserve] = ACTIONS(1673),
    [sym_duet_dev_type_display_device] = ACTIONS(1673),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1673),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1673),
    [sym_duet_dev_type_io_device] = ACTIONS(1673),
    [sym_duet_dev_type_relay_device] = ACTIONS(1673),
    [sym_duet_dev_type_ups] = ACTIONS(1673),
    [sym_duet_dev_type_amplifier] = ACTIONS(1673),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1673),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1673),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1673),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1673),
    [sym_duet_dev_type_camera] = ACTIONS(1673),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1673),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1673),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1673),
    [sym_duet_dev_type_dss] = ACTIONS(1673),
    [sym_duet_dev_type_dvr] = ACTIONS(1673),
    [sym_duet_dev_type_disc_device] = ACTIONS(1673),
    [sym_duet_dev_type_document_camera] = ACTIONS(1673),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1673),
    [sym_duet_dev_type_hvac] = ACTIONS(1673),
    [sym_duet_dev_type_keypad] = ACTIONS(1673),
    [sym_duet_dev_type_light] = ACTIONS(1673),
    [sym_duet_dev_type_monitor] = ACTIONS(1673),
    [sym_duet_dev_type_motor] = ACTIONS(1673),
    [sym_duet_dev_type_multi_window] = ACTIONS(1673),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1673),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1673),
    [sym_duet_dev_type_receiver] = ACTIONS(1673),
    [sym_duet_dev_type_security_system] = ACTIONS(1673),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1673),
    [sym_duet_dev_type_settop_box] = ACTIONS(1673),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1673),
    [sym_duet_dev_type_switcher] = ACTIONS(1673),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1673),
    [sym_duet_dev_type_tv] = ACTIONS(1673),
    [sym_duet_dev_type_utility] = ACTIONS(1673),
    [sym_duet_dev_type_vcr] = ACTIONS(1673),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1673),
    [sym_duet_dev_type_video_processor] = ACTIONS(1673),
    [sym_duet_dev_type_video_projector] = ACTIONS(1673),
    [sym_duet_dev_type_video_wall] = ACTIONS(1673),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1673),
    [sym_duet_dev_type_weather] = ACTIONS(1673),
    [sym_duet_dev_not_polled] = ACTIONS(1673),
    [sym_duet_dev_polled] = ACTIONS(1673),
    [sym_clkmgr_mode_network] = ACTIONS(1673),
    [sym_clkmgr_mode_standalone] = ACTIONS(1673),
    [sym_normal_standby] = ACTIONS(1673),
    [sym_normal_wake] = ACTIONS(1673),
    [sym_ip_multicast_ttl_option] = ACTIONS(1673),
    [sym_ip_ttl_subnet] = ACTIONS(1673),
    [sym_ip_ttl_site] = ACTIONS(1673),
    [sym_ip_ttl_region] = ACTIONS(1673),
    [sym_ip_ttl_continent] = ACTIONS(1673),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1673),
    [sym_ip_nodelay_on] = ACTIONS(1673),
    [sym_ip_nodelay_off] = ACTIONS(1673),
    [sym_amx_error] = ACTIONS(1673),
    [sym_amx_warning] = ACTIONS(1673),
    [sym_amx_info] = ACTIONS(1673),
    [sym_amx_debug] = ACTIONS(1673),
    [sym_netlinx_login_success] = ACTIONS(1673),
    [sym_netlinx_login_fail] = ACTIONS(1673),
    [sym_netlinx_logout] = ACTIONS(1673),
    [sym_null_str] = ACTIONS(1673),
    [sym_smtp_address] = ACTIONS(1673),
    [sym_smtp_port_number] = ACTIONS(1673),
    [sym_smtp_username] = ACTIONS(1673),
    [sym_smtp_password] = ACTIONS(1673),
    [sym_smtp_require_tls] = ACTIONS(1673),
    [sym_smtp_from] = ACTIONS(1673),
    [sym_smtp_tls_true] = ACTIONS(1673),
    [sym_smtp_tls_false] = ACTIONS(1673),
    [sym_listview_on_row_select_event] = ACTIONS(1673),
    [sym_data_structure_unknown] = ACTIONS(1673),
    [sym_data_structure_datafeed] = ACTIONS(1673),
    [sym_data_structure_datarecordset] = ACTIONS(1673),
    [sym_data_structure_datarecord] = ACTIONS(1673),
    [sym_data_structure_datafield] = ACTIONS(1673),
    [sym_data_max_content_field_count] = ACTIONS(1673),
    [sym_data_max_metadata_field_count] = ACTIONS(1673),
    [sym_data_max_id_length] = ACTIONS(1673),
    [sym_data_max_name_length] = ACTIONS(1673),
    [sym_data_max_description_length] = ACTIONS(1673),
    [sym_data_max_type_length] = ACTIONS(1673),
    [sym_data_max_format_length] = ACTIONS(1673),
    [sym_data_max_source_length] = ACTIONS(1673),
    [sym_data_max_value_length] = ACTIONS(1673),
    [sym_data_max_label_length] = ACTIONS(1673),
    [sym_data_max_url_length] = ACTIONS(1673),
    [sym_max_last_login_info_length] = ACTIONS(1673),
    [sym_data_type_unknown] = ACTIONS(1673),
    [sym_data_type_string] = ACTIONS(1673),
    [sym_data_type_datetime] = ACTIONS(1673),
    [sym_data_type_date] = ACTIONS(1673),
    [sym_data_type_time] = ACTIONS(1673),
    [sym_data_type_image] = ACTIONS(1673),
    [sym_data_format_url] = ACTIONS(1673),
    [sym_data_format_phone] = ACTIONS(1673),
    [sym_data_format_email] = ACTIONS(1673),
    [sym_data_format_iso8601] = ACTIONS(1673),
    [sym_auditlog_priv] = ACTIONS(1673),
    [sym_configuration_priv] = ACTIONS(1673),
    [sym_deviceconfig_priv] = ACTIONS(1673),
    [sym_ftp_priv] = ACTIONS(1673),
    [sym_http_priv] = ACTIONS(1673),
    [sym_networkconfig_priv] = ACTIONS(1673),
    [sym_programport_priv] = ACTIONS(1673),
    [sym_remoteui_priv] = ACTIONS(1673),
    [sym_securitycontrol_priv] = ACTIONS(1673),
    [sym_softwaremanagement_priv] = ACTIONS(1673),
    [sym_terminal_priv] = ACTIONS(1673),
    [sym_tpadmin_priv] = ACTIONS(1673),
    [sym_user1_priv] = ACTIONS(1673),
    [sym_user2_priv] = ACTIONS(1673),
    [sym_user3_priv] = ACTIONS(1673),
    [sym_user4_priv] = ACTIONS(1673),
    [sym_usermanagement_priv] = ACTIONS(1673),
    [sym_valid_account] = ACTIONS(1673),
    [sym_err_name_invalid] = ACTIONS(1673),
    [sym_err_password_invalid] = ACTIONS(1673),
    [sym_err_invalid_account] = ACTIONS(1673),
    [sym_err_no_authorization] = ACTIONS(1673),
    [sym_err_invalid_parameter] = ACTIONS(1673),
    [sym_err_user_locked_out] = ACTIONS(1673),
    [sym_err_user_account_expired] = ACTIONS(1673),
    [sym_err_general_error] = ACTIONS(1673),
    [sym_err_authentication_error] = ACTIONS(1673),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1673),
    [sym_tls_validate_certificate] = ACTIONS(1673),
    [sym_tdata] = ACTIONS(1673),
    [sym_tchannel] = ACTIONS(1673),
    [sym_tlevel] = ACTIONS(1673),
    [sym_tbutton] = ACTIONS(1673),
    [sym_ttimeline] = ACTIONS(1673),
    [sym_tcustom] = ACTIONS(1673),
    [sym_url_struct] = ACTIONS(1673),
    [sym_dns_struct] = ACTIONS(1673),
    [sym_ip_address_struct] = ACTIONS(1673),
    [sym_dev_info_struct] = ACTIONS(1673),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1673),
    [sym_data_feed] = ACTIONS(1673),
    [sym_data_field] = ACTIONS(1673),
    [sym_data_record] = ACTIONS(1673),
    [sym_wc_data_feed] = ACTIONS(1673),
    [sym_wc_data_field] = ACTIONS(1673),
    [sym_wc_data_record] = ACTIONS(1673),
    [sym_last_login_info] = ACTIONS(1673),
    [sym____reserved___] = ACTIONS(1673),
    [sym_date] = ACTIONS(1673),
    [sym_day] = ACTIONS(1673),
    [sym_time] = ACTIONS(1673),
    [sym_ldate] = ACTIONS(1673),
    [sym_push_device] = ACTIONS(1673),
    [sym_push_channel] = ACTIONS(1673),
    [sym_push_devchan] = ACTIONS(1673),
    [sym_release_device] = ACTIONS(1673),
    [sym_release_channel] = ACTIONS(1673),
    [sym_release_devchan] = ACTIONS(1673),
    [sym_master_slot] = ACTIONS(1673),
    [sym_get_pulse_time] = ACTIONS(1673),
    [sym_get_timer] = ACTIONS(1673),
    [sym_master_sn] = ACTIONS(1673),
    [sym_system_number] = ACTIONS(1673),
    [sym_button] = ACTIONS(1673),
    [sym_dv_channel] = ACTIONS(1673),
    [sym_data] = ACTIONS(1673),
    [sym_level] = ACTIONS(1673),
    [sym_timeline] = ACTIONS(1673),
    [sym_channel] = ACTIONS(1673),
    [sym_custom] = ACTIONS(1673),
    [sym_atoi] = ACTIONS(1673),
    [sym_atol] = ACTIONS(1673),
    [sym_atof] = ACTIONS(1673),
    [sym_itoa] = ACTIONS(1673),
    [sym_itohex] = ACTIONS(1673),
    [sym_hextoi] = ACTIONS(1673),
    [sym_ftoa] = ACTIONS(1673),
    [sym_find_string] = ACTIONS(1673),
    [sym_left_string] = ACTIONS(1673),
    [sym_right_string] = ACTIONS(1673),
    [sym_mid_string] = ACTIONS(1673),
    [sym_remove_string] = ACTIONS(1673),
    [sym_get_buffer_string] = ACTIONS(1673),
    [sym_get_buffer_char] = ACTIONS(1673),
    [sym_get_multi_buffer_string] = ACTIONS(1673),
    [sym_length_string] = ACTIONS(1673),
    [sym_max_length_string] = ACTIONS(1673),
    [sym_set_length_string] = ACTIONS(1673),
    [sym_compare_string] = ACTIONS(1673),
    [sym_upper_string] = ACTIONS(1673),
    [sym_lower_string] = ACTIONS(1673),
    [sym_get_last] = ACTIONS(1673),
    [sym_date_to_day] = ACTIONS(1673),
    [sym_date_to_month] = ACTIONS(1673),
    [sym_date_to_year] = ACTIONS(1673),
    [sym_time_to_hour] = ACTIONS(1673),
    [sym_time_to_minute] = ACTIONS(1673),
    [sym_time_to_second] = ACTIONS(1673),
    [sym_day_of_week] = ACTIONS(1673),
    [sym_set_pulse_time] = ACTIONS(1673),
    [sym_set_timer] = ACTIONS(1673),
    [sym_file_open] = ACTIONS(1673),
    [sym_file_close] = ACTIONS(1673),
    [sym_file_read] = ACTIONS(1673),
    [sym_file_read_line] = ACTIONS(1673),
    [sym_file_write] = ACTIONS(1673),
    [sym_file_write_line] = ACTIONS(1673),
    [sym_file_copy] = ACTIONS(1673),
    [sym_file_delete] = ACTIONS(1673),
    [sym_file_rename] = ACTIONS(1673),
    [sym_file_dir] = ACTIONS(1673),
    [sym_file_getdir] = ACTIONS(1673),
    [sym_file_setdir] = ACTIONS(1673),
    [sym_file_create_dir] = ACTIONS(1673),
    [sym_file_remove_dir] = ACTIONS(1673),
    [sym_file_seek] = ACTIONS(1673),
    [sym_length_array] = ACTIONS(1673),
    [sym_max_length_array] = ACTIONS(1673),
    [sym_set_length_array] = ACTIONS(1673),
    [sym_ip_client_open] = ACTIONS(1673),
    [sym_ip_client_close] = ACTIONS(1673),
    [sym_ip_server_open] = ACTIONS(1673),
    [sym_ip_server_close] = ACTIONS(1673),
    [sym_ip_bound_client_open] = ACTIONS(1673),
    [sym_ip_mc_server_open] = ACTIONS(1673),
    [sym_ip_set_option] = ACTIONS(1673),
    [sym_ssh_client_open] = ACTIONS(1673),
    [sym_ssh_client_close] = ACTIONS(1673),
    [sym_tls_client_open] = ACTIONS(1673),
    [sym_tls_client_close] = ACTIONS(1673),
    [sym_do_push] = ACTIONS(1673),
    [sym_do_push_timed] = ACTIONS(1673),
    [sym_do_release] = ACTIONS(1673),
    [sym_device_id] = ACTIONS(1673),
    [sym_device_info] = ACTIONS(1673),
    [sym_device_id_string] = ACTIONS(1673),
    [sym_device_standby] = ACTIONS(1673),
    [sym_device_wake] = ACTIONS(1673),
    [sym_timeline_create] = ACTIONS(1673),
    [sym_timeline_kill] = ACTIONS(1673),
    [sym_timeline_reload] = ACTIONS(1673),
    [sym_timeline_pause] = ACTIONS(1673),
    [sym_timeline_restart] = ACTIONS(1673),
    [sym_timeline_set] = ACTIONS(1673),
    [sym_timeline_get] = ACTIONS(1673),
    [sym_timeline_active] = ACTIONS(1673),
    [sym_timeline_dynamic_id] = ACTIONS(1673),
    [sym_variable_to_string] = ACTIONS(1673),
    [sym_string_to_variable] = ACTIONS(1673),
    [sym_length_variable_to_string] = ACTIONS(1673),
    [sym_type_cast] = ACTIONS(1673),
    [sym_variable_to_xml] = ACTIONS(1673),
    [sym_xml_to_variable] = ACTIONS(1673),
    [sym_length_variable_to_xml] = ACTIONS(1673),
    [sym_redirect_string] = ACTIONS(1673),
    [sym_random_number] = ACTIONS(1673),
    [sym_get_unique_id] = ACTIONS(1673),
    [sym_get_system_number] = ACTIONS(1673),
    [sym_set_system_number] = ACTIONS(1673),
    [sym_get_serial_number] = ACTIONS(1673),
    [sym_get_master_build] = ACTIONS(1673),
    [sym_set_validation_code] = ACTIONS(1673),
    [sym_reboot] = ACTIONS(1673),
    [sym_load_duet_module] = ACTIONS(1673),
    [sym_unload_duet_module] = ACTIONS(1673),
    [sym_duet_mem_size_set] = ACTIONS(1673),
    [sym_duet_mem_size_get] = ACTIONS(1673),
    [sym_dynamic_application_device] = ACTIONS(1673),
    [sym_dynamic_polled_port] = ACTIONS(1673),
    [sym_static_port_binding] = ACTIONS(1673),
    [sym_static_ip_binding] = ACTIONS(1673),
    [sym_get_dns_list] = ACTIONS(1673),
    [sym_set_dns_list] = ACTIONS(1673),
    [sym_get_ip_address] = ACTIONS(1673),
    [sym_set_ip_address] = ACTIONS(1673),
    [sym_get_url_list] = ACTIONS(1673),
    [sym_add_url_entry] = ACTIONS(1673),
    [sym_delete_url_entry] = ACTIONS(1673),
    [sym_abs_value] = ACTIONS(1673),
    [sym_max_value] = ACTIONS(1673),
    [sym_min_value] = ACTIONS(1673),
    [sym_exp_value] = ACTIONS(1673),
    [sym_log_value] = ACTIONS(1673),
    [sym_log10_value] = ACTIONS(1673),
    [sym_power_value] = ACTIONS(1673),
    [sym_sqrt_value] = ACTIONS(1673),
    [sym_raw_be] = ACTIONS(1673),
    [sym_raw_le] = ACTIONS(1673),
    [sym_internal_queue_size_set] = ACTIONS(1673),
    [sym_internal_queue_size_get] = ACTIONS(1673),
    [sym_internal_threshold_set] = ACTIONS(1673),
    [sym_internal_threshold_get] = ACTIONS(1673),
    [sym_rebuilt_event] = ACTIONS(1673),
    [sym_do_custom_event] = ACTIONS(1673),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1673),
    [sym_clkmgr_set_clk_source] = ACTIONS(1673),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1673),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1673),
    [sym_clkmgr_get_timezone] = ACTIONS(1673),
    [sym_clkmgr_set_timezone] = ACTIONS(1673),
    [sym_clkmgr_get_resync_period] = ACTIONS(1673),
    [sym_clkmgr_set_resync_period] = ACTIONS(1673),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1673),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1673),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1673),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1673),
    [sym_clkmgr_get_timeservers] = ACTIONS(1673),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1673),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1673),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1673),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1673),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1673),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1673),
    [sym_set_log_level] = ACTIONS(1673),
    [sym_get_log_level] = ACTIONS(1673),
    [sym_amx_log] = ACTIONS(1673),
    [sym_smtp_server_config_set] = ACTIONS(1673),
    [sym_smtp_server_config_get] = ACTIONS(1673),
    [sym_smtp_send] = ACTIONS(1673),
    [sym_validate_netlinx_account] = ACTIONS(1673),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1673),
    [sym_authenticate_certificate] = ACTIONS(1673),
    [sym_audit_netlinx_session_event] = ACTIONS(1673),
    [sym_audit_netlinx_generic_event] = ACTIONS(1673),
    [sym_get_available_flash_disk_space] = ACTIONS(1673),
    [sym_get_max_flash_disk_space] = ACTIONS(1673),
    [sym_data_create_feed] = ACTIONS(1673),
    [sym_data_delete_feed] = ACTIONS(1673),
    [sym_data_publish_feed] = ACTIONS(1673),
    [sym_data_get_published_feed] = ACTIONS(1673),
    [sym_data_add_record] = ACTIONS(1673),
    [sym_data_get_event_record] = ACTIONS(1673),
    [sym__wc_data_create_feed] = ACTIONS(1673),
    [sym__wc_data_add_record] = ACTIONS(1673),
    [sym__wc_data_get_event_record] = ACTIONS(1673),
    [sym_format] = ACTIONS(1673),
    [sym_astro_clock] = ACTIONS(1673),
    [sym_set_outdoor_temperature] = ACTIONS(1673),
    [sym_set_virtual_level_count] = ACTIONS(1673),
    [sym_set_virtual_channel_count] = ACTIONS(1673),
    [sym_set_virtual_port_count] = ACTIONS(1673),
    [sym__wc_encode_int] = ACTIONS(1673),
    [sym__wc_decode_int] = ACTIONS(1673),
    [sym__wc_int] = ACTIONS(1673),
    [sym__wc_to_ch_int] = ACTIONS(1673),
    [sym__ch_to_wc_int] = ACTIONS(1673),
    [sym__wc_find_string_int] = ACTIONS(1673),
    [sym__wc_left_string_int] = ACTIONS(1673),
    [sym__wc_lower_string_int] = ACTIONS(1673),
    [sym__wc_mid_string_int] = ACTIONS(1673),
    [sym__wc_remove_string_int] = ACTIONS(1673),
    [sym__wc_right_string_int] = ACTIONS(1673),
    [sym__wc_upper_string_int] = ACTIONS(1673),
    [sym__wc_compare_string_int] = ACTIONS(1673),
    [sym__wc_get_buffer_char_int] = ACTIONS(1673),
    [sym__wc_get_buffer_string_int] = ACTIONS(1673),
    [sym__wc_concat_string_int] = ACTIONS(1673),
    [sym__wc_file_open_int] = ACTIONS(1673),
    [sym__wc_file_close_int] = ACTIONS(1673),
    [sym__wc_file_read_int] = ACTIONS(1673),
    [sym__wc_file_read_line_int] = ACTIONS(1673),
    [sym__wc_file_write_int] = ACTIONS(1673),
    [sym__wc_file_write_line_int] = ACTIONS(1673),
    [aux_sym_preproc_include_token1] = ACTIONS(1673),
    [aux_sym_preproc_define_token1] = ACTIONS(1675),
    [aux_sym_preproc_warn_token1] = ACTIONS(1675),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1675),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1675),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1675),
    [anon_sym_LBRACK] = ACTIONS(1675),
    [anon_sym_LBRACE] = ACTIONS(1675),
    [anon_sym_LPAREN] = ACTIONS(1673),
    [anon_sym_SEMI] = ACTIONS(1693),
    [anon_sym_BANG] = ACTIONS(1675),
    [anon_sym_TILDE] = ACTIONS(1675),
    [anon_sym_DASH] = ACTIONS(1673),
    [anon_sym_PLUS] = ACTIONS(1673),
    [anon_sym_DASH_DASH] = ACTIONS(1675),
    [anon_sym_PLUS_PLUS] = ACTIONS(1675),
    [anon_sym_DQUOTE] = ACTIONS(1675),
    [anon_sym_SQUOTE] = ACTIONS(1675),
    [sym_number_literal] = ACTIONS(1675),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1693),
  },
  [STATE(203)] = {
    [ts_builtin_sym_end] = ACTIONS(1695),
    [sym_identifier] = ACTIONS(1697),
    [sym_program_name_keyword] = ACTIONS(1697),
    [sym_module_name_keyword] = ACTIONS(1697),
    [sym_define_device_keyword] = ACTIONS(1697),
    [sym_define_combine_keyword] = ACTIONS(1697),
    [sym_define_constant_keyword] = ACTIONS(1697),
    [sym_define_type_keyword] = ACTIONS(1697),
    [sym_define_variable_keyword] = ACTIONS(1697),
    [sym_define_system_variable_keyword] = ACTIONS(1697),
    [sym_define_connect_level_keyword] = ACTIONS(1697),
    [sym_define_latching_keyword] = ACTIONS(1697),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1697),
    [sym_define_toggling_keyword] = ACTIONS(1697),
    [sym_define_call_keyword] = ACTIONS(1697),
    [sym_define_function_keyword] = ACTIONS(1697),
    [sym_define_library_function_keyword] = ACTIONS(1697),
    [sym_define_start_keyword] = ACTIONS(1697),
    [sym_define_event_keyword] = ACTIONS(1697),
    [sym_define_program_keyword] = ACTIONS(1697),
    [sym_define_module_keyword] = ACTIONS(1697),
    [sym_char_keyword] = ACTIONS(1697),
    [sym_widechar_keyword] = ACTIONS(1697),
    [sym_integer_keyword] = ACTIONS(1697),
    [sym_sinteger_keyword] = ACTIONS(1697),
    [sym_long_keyword] = ACTIONS(1697),
    [sym_slong_keyword] = ACTIONS(1697),
    [sym_float_keyword] = ACTIONS(1697),
    [sym_double_keyword] = ACTIONS(1697),
    [sym_variant_keyword] = ACTIONS(1697),
    [sym_variantarray_keyword] = ACTIONS(1697),
    [sym_local_var_keyword] = ACTIONS(1697),
    [sym_stack_var_keyword] = ACTIONS(1697),
    [sym_constant_keyword] = ACTIONS(1697),
    [sym_volatile_keyword] = ACTIONS(1697),
    [sym_non_volatile_keyword] = ACTIONS(1697),
    [sym_persistent_keyword] = ACTIONS(1697),
    [sym_dev_keyword] = ACTIONS(1697),
    [sym_devlev_keyword] = ACTIONS(1697),
    [sym_devchan_keyword] = ACTIONS(1697),
    [sym_if_keyword] = ACTIONS(1697),
    [sym_switch_keyword] = ACTIONS(1697),
    [sym_case_keyword] = ACTIONS(1697),
    [sym_default_keyword] = ACTIONS(1697),
    [sym_while_keyword] = ACTIONS(1697),
    [sym_for_keyword] = ACTIONS(1697),
    [sym_break_keyword] = ACTIONS(1697),
    [sym_continue_keyword] = ACTIONS(1697),
    [sym_return_keyword] = ACTIONS(1697),
    [sym_select_keyword] = ACTIONS(1697),
    [sym_active_keyword] = ACTIONS(1697),
    [sym_struct_keyword] = ACTIONS(1697),
    [sym_structure_keyword] = ACTIONS(1697),
    [sym_button_event_keyword] = ACTIONS(1697),
    [sym_channel_event_keyword] = ACTIONS(1697),
    [sym_level_event_keyword] = ACTIONS(1697),
    [sym_data_event_keyword] = ACTIONS(1697),
    [sym_timeline_event_keyword] = ACTIONS(1697),
    [sym_custom_event_keyword] = ACTIONS(1697),
    [sym_push_keyword] = ACTIONS(1697),
    [sym_release_keyword] = ACTIONS(1697),
    [aux_sym_on_keyword_token1] = ACTIONS(1697),
    [aux_sym_off_keyword_token1] = ACTIONS(1697),
    [sym_send_level_keyword] = ACTIONS(1697),
    [sym_send_string_keyword] = ACTIONS(1697),
    [sym_send_command_keyword] = ACTIONS(1697),
    [sym_clear_buffer_keyword] = ACTIONS(1697),
    [sym_create_buffer_keyword] = ACTIONS(1697),
    [sym_create_multi_buffer_keyword] = ACTIONS(1697),
    [sym_call_keyword] = ACTIONS(1697),
    [sym_system_call_keyword] = ACTIONS(1697),
    [sym_devchan_to_keyword] = ACTIONS(1697),
    [sym_devchan_min_to_keyword] = ACTIONS(1697),
    [sym_devchan_total_off_keyword] = ACTIONS(1697),
    [sym_devchan_pulse_keyword] = ACTIONS(1697),
    [sym_wait_keyword] = ACTIONS(1697),
    [sym_cancel_wait_keyword] = ACTIONS(1697),
    [sym_wait_until_keyword] = ACTIONS(1697),
    [sym_cancel_wait_until_keyword] = ACTIONS(1697),
    [sym_cancel_all_wait_keyword] = ACTIONS(1697),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1697),
    [sym_not] = ACTIONS(1697),
    [sym_bnot] = ACTIONS(1697),
    [sym___file__] = ACTIONS(1697),
    [sym___date__] = ACTIONS(1697),
    [sym___ldate__] = ACTIONS(1697),
    [sym___time__] = ACTIONS(1697),
    [sym___line__] = ACTIONS(1697),
    [sym___name__] = ACTIONS(1697),
    [sym___netlinx__] = ACTIONS(1697),
    [sym_netlinx_axi_version] = ACTIONS(1697),
    [sym_url_flg_tcp] = ACTIONS(1697),
    [sym_url_flg_accinfopresent] = ACTIONS(1697),
    [sym_url_flg_temp] = ACTIONS(1697),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1697),
    [sym_url_flg_stat_mask] = ACTIONS(1697),
    [sym_url_flg_stat_lookup] = ACTIONS(1697),
    [sym_url_flg_stat_connecting] = ACTIONS(1697),
    [sym_url_flg_stat_waiting] = ACTIONS(1697),
    [sym_url_flg_stat_connected] = ACTIONS(1697),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1697),
    [sym_first_virtual_device] = ACTIONS(1697),
    [sym_first_local_port] = ACTIONS(1697),
    [sym_dynamic_virtual_device] = ACTIONS(1697),
    [sym_timeline_once] = ACTIONS(1697),
    [sym_timeline_repeat] = ACTIONS(1697),
    [sym_timeline_absolute] = ACTIONS(1697),
    [sym_timeline_relative] = ACTIONS(1697),
    [sym_do_push_timed_infinite] = ACTIONS(1697),
    [sym_source_type_no_address] = ACTIONS(1697),
    [sym_source_type_neuron_id] = ACTIONS(1697),
    [sym_source_type_ip_address] = ACTIONS(1697),
    [sym_source_type_axlink] = ACTIONS(1697),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1697),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1697),
    [sym_source_type_ip_socket_address] = ACTIONS(1697),
    [sym_source_type_rs232] = ACTIONS(1697),
    [sym_source_type_internal] = ACTIONS(1697),
    [sym_source_type_mac_address] = ACTIONS(1697),
    [sym_source_type_ipv4_port] = ACTIONS(1697),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1697),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1697),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1697),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1697),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1697),
    [sym_true] = ACTIONS(1697),
    [sym_false] = ACTIONS(1697),
    [sym_file_read_only] = ACTIONS(1697),
    [sym_file_rw_new] = ACTIONS(1697),
    [sym_file_rw_append] = ACTIONS(1697),
    [sym_ip_tcp] = ACTIONS(1697),
    [sym_ip_udp] = ACTIONS(1697),
    [sym_ip_udp_2way] = ACTIONS(1697),
    [sym_xml_encode_types] = ACTIONS(1697),
    [sym_xml_encode_char_as_list] = ACTIONS(1697),
    [sym_xml_encode_le] = ACTIONS(1697),
    [sym_xml_decode_types] = ACTIONS(1697),
    [sym_xml_decode_no_preserve] = ACTIONS(1697),
    [sym_duet_dev_type_display_device] = ACTIONS(1697),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1697),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1697),
    [sym_duet_dev_type_io_device] = ACTIONS(1697),
    [sym_duet_dev_type_relay_device] = ACTIONS(1697),
    [sym_duet_dev_type_ups] = ACTIONS(1697),
    [sym_duet_dev_type_amplifier] = ACTIONS(1697),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1697),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1697),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1697),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1697),
    [sym_duet_dev_type_camera] = ACTIONS(1697),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1697),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1697),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1697),
    [sym_duet_dev_type_dss] = ACTIONS(1697),
    [sym_duet_dev_type_dvr] = ACTIONS(1697),
    [sym_duet_dev_type_disc_device] = ACTIONS(1697),
    [sym_duet_dev_type_document_camera] = ACTIONS(1697),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1697),
    [sym_duet_dev_type_hvac] = ACTIONS(1697),
    [sym_duet_dev_type_keypad] = ACTIONS(1697),
    [sym_duet_dev_type_light] = ACTIONS(1697),
    [sym_duet_dev_type_monitor] = ACTIONS(1697),
    [sym_duet_dev_type_motor] = ACTIONS(1697),
    [sym_duet_dev_type_multi_window] = ACTIONS(1697),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1697),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1697),
    [sym_duet_dev_type_receiver] = ACTIONS(1697),
    [sym_duet_dev_type_security_system] = ACTIONS(1697),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1697),
    [sym_duet_dev_type_settop_box] = ACTIONS(1697),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1697),
    [sym_duet_dev_type_switcher] = ACTIONS(1697),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1697),
    [sym_duet_dev_type_tv] = ACTIONS(1697),
    [sym_duet_dev_type_utility] = ACTIONS(1697),
    [sym_duet_dev_type_vcr] = ACTIONS(1697),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1697),
    [sym_duet_dev_type_video_processor] = ACTIONS(1697),
    [sym_duet_dev_type_video_projector] = ACTIONS(1697),
    [sym_duet_dev_type_video_wall] = ACTIONS(1697),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1697),
    [sym_duet_dev_type_weather] = ACTIONS(1697),
    [sym_duet_dev_not_polled] = ACTIONS(1697),
    [sym_duet_dev_polled] = ACTIONS(1697),
    [sym_clkmgr_mode_network] = ACTIONS(1697),
    [sym_clkmgr_mode_standalone] = ACTIONS(1697),
    [sym_normal_standby] = ACTIONS(1697),
    [sym_normal_wake] = ACTIONS(1697),
    [sym_ip_multicast_ttl_option] = ACTIONS(1697),
    [sym_ip_ttl_subnet] = ACTIONS(1697),
    [sym_ip_ttl_site] = ACTIONS(1697),
    [sym_ip_ttl_region] = ACTIONS(1697),
    [sym_ip_ttl_continent] = ACTIONS(1697),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1697),
    [sym_ip_nodelay_on] = ACTIONS(1697),
    [sym_ip_nodelay_off] = ACTIONS(1697),
    [sym_amx_error] = ACTIONS(1697),
    [sym_amx_warning] = ACTIONS(1697),
    [sym_amx_info] = ACTIONS(1697),
    [sym_amx_debug] = ACTIONS(1697),
    [sym_netlinx_login_success] = ACTIONS(1697),
    [sym_netlinx_login_fail] = ACTIONS(1697),
    [sym_netlinx_logout] = ACTIONS(1697),
    [sym_null_str] = ACTIONS(1697),
    [sym_smtp_address] = ACTIONS(1697),
    [sym_smtp_port_number] = ACTIONS(1697),
    [sym_smtp_username] = ACTIONS(1697),
    [sym_smtp_password] = ACTIONS(1697),
    [sym_smtp_require_tls] = ACTIONS(1697),
    [sym_smtp_from] = ACTIONS(1697),
    [sym_smtp_tls_true] = ACTIONS(1697),
    [sym_smtp_tls_false] = ACTIONS(1697),
    [sym_listview_on_row_select_event] = ACTIONS(1697),
    [sym_data_structure_unknown] = ACTIONS(1697),
    [sym_data_structure_datafeed] = ACTIONS(1697),
    [sym_data_structure_datarecordset] = ACTIONS(1697),
    [sym_data_structure_datarecord] = ACTIONS(1697),
    [sym_data_structure_datafield] = ACTIONS(1697),
    [sym_data_max_content_field_count] = ACTIONS(1697),
    [sym_data_max_metadata_field_count] = ACTIONS(1697),
    [sym_data_max_id_length] = ACTIONS(1697),
    [sym_data_max_name_length] = ACTIONS(1697),
    [sym_data_max_description_length] = ACTIONS(1697),
    [sym_data_max_type_length] = ACTIONS(1697),
    [sym_data_max_format_length] = ACTIONS(1697),
    [sym_data_max_source_length] = ACTIONS(1697),
    [sym_data_max_value_length] = ACTIONS(1697),
    [sym_data_max_label_length] = ACTIONS(1697),
    [sym_data_max_url_length] = ACTIONS(1697),
    [sym_max_last_login_info_length] = ACTIONS(1697),
    [sym_data_type_unknown] = ACTIONS(1697),
    [sym_data_type_string] = ACTIONS(1697),
    [sym_data_type_datetime] = ACTIONS(1697),
    [sym_data_type_date] = ACTIONS(1697),
    [sym_data_type_time] = ACTIONS(1697),
    [sym_data_type_image] = ACTIONS(1697),
    [sym_data_format_url] = ACTIONS(1697),
    [sym_data_format_phone] = ACTIONS(1697),
    [sym_data_format_email] = ACTIONS(1697),
    [sym_data_format_iso8601] = ACTIONS(1697),
    [sym_auditlog_priv] = ACTIONS(1697),
    [sym_configuration_priv] = ACTIONS(1697),
    [sym_deviceconfig_priv] = ACTIONS(1697),
    [sym_ftp_priv] = ACTIONS(1697),
    [sym_http_priv] = ACTIONS(1697),
    [sym_networkconfig_priv] = ACTIONS(1697),
    [sym_programport_priv] = ACTIONS(1697),
    [sym_remoteui_priv] = ACTIONS(1697),
    [sym_securitycontrol_priv] = ACTIONS(1697),
    [sym_softwaremanagement_priv] = ACTIONS(1697),
    [sym_terminal_priv] = ACTIONS(1697),
    [sym_tpadmin_priv] = ACTIONS(1697),
    [sym_user1_priv] = ACTIONS(1697),
    [sym_user2_priv] = ACTIONS(1697),
    [sym_user3_priv] = ACTIONS(1697),
    [sym_user4_priv] = ACTIONS(1697),
    [sym_usermanagement_priv] = ACTIONS(1697),
    [sym_valid_account] = ACTIONS(1697),
    [sym_err_name_invalid] = ACTIONS(1697),
    [sym_err_password_invalid] = ACTIONS(1697),
    [sym_err_invalid_account] = ACTIONS(1697),
    [sym_err_no_authorization] = ACTIONS(1697),
    [sym_err_invalid_parameter] = ACTIONS(1697),
    [sym_err_user_locked_out] = ACTIONS(1697),
    [sym_err_user_account_expired] = ACTIONS(1697),
    [sym_err_general_error] = ACTIONS(1697),
    [sym_err_authentication_error] = ACTIONS(1697),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1697),
    [sym_tls_validate_certificate] = ACTIONS(1697),
    [sym_tdata] = ACTIONS(1697),
    [sym_tchannel] = ACTIONS(1697),
    [sym_tlevel] = ACTIONS(1697),
    [sym_tbutton] = ACTIONS(1697),
    [sym_ttimeline] = ACTIONS(1697),
    [sym_tcustom] = ACTIONS(1697),
    [sym_url_struct] = ACTIONS(1697),
    [sym_dns_struct] = ACTIONS(1697),
    [sym_ip_address_struct] = ACTIONS(1697),
    [sym_dev_info_struct] = ACTIONS(1697),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1697),
    [sym_data_feed] = ACTIONS(1697),
    [sym_data_field] = ACTIONS(1697),
    [sym_data_record] = ACTIONS(1697),
    [sym_wc_data_feed] = ACTIONS(1697),
    [sym_wc_data_field] = ACTIONS(1697),
    [sym_wc_data_record] = ACTIONS(1697),
    [sym_last_login_info] = ACTIONS(1697),
    [sym____reserved___] = ACTIONS(1697),
    [sym_date] = ACTIONS(1697),
    [sym_day] = ACTIONS(1697),
    [sym_time] = ACTIONS(1697),
    [sym_ldate] = ACTIONS(1697),
    [sym_push_device] = ACTIONS(1697),
    [sym_push_channel] = ACTIONS(1697),
    [sym_push_devchan] = ACTIONS(1697),
    [sym_release_device] = ACTIONS(1697),
    [sym_release_channel] = ACTIONS(1697),
    [sym_release_devchan] = ACTIONS(1697),
    [sym_master_slot] = ACTIONS(1697),
    [sym_get_pulse_time] = ACTIONS(1697),
    [sym_get_timer] = ACTIONS(1697),
    [sym_master_sn] = ACTIONS(1697),
    [sym_system_number] = ACTIONS(1697),
    [sym_button] = ACTIONS(1697),
    [sym_dv_channel] = ACTIONS(1697),
    [sym_data] = ACTIONS(1697),
    [sym_level] = ACTIONS(1697),
    [sym_timeline] = ACTIONS(1697),
    [sym_channel] = ACTIONS(1697),
    [sym_custom] = ACTIONS(1697),
    [sym_atoi] = ACTIONS(1697),
    [sym_atol] = ACTIONS(1697),
    [sym_atof] = ACTIONS(1697),
    [sym_itoa] = ACTIONS(1697),
    [sym_itohex] = ACTIONS(1697),
    [sym_hextoi] = ACTIONS(1697),
    [sym_ftoa] = ACTIONS(1697),
    [sym_find_string] = ACTIONS(1697),
    [sym_left_string] = ACTIONS(1697),
    [sym_right_string] = ACTIONS(1697),
    [sym_mid_string] = ACTIONS(1697),
    [sym_remove_string] = ACTIONS(1697),
    [sym_get_buffer_string] = ACTIONS(1697),
    [sym_get_buffer_char] = ACTIONS(1697),
    [sym_get_multi_buffer_string] = ACTIONS(1697),
    [sym_length_string] = ACTIONS(1697),
    [sym_max_length_string] = ACTIONS(1697),
    [sym_set_length_string] = ACTIONS(1697),
    [sym_compare_string] = ACTIONS(1697),
    [sym_upper_string] = ACTIONS(1697),
    [sym_lower_string] = ACTIONS(1697),
    [sym_get_last] = ACTIONS(1697),
    [sym_date_to_day] = ACTIONS(1697),
    [sym_date_to_month] = ACTIONS(1697),
    [sym_date_to_year] = ACTIONS(1697),
    [sym_time_to_hour] = ACTIONS(1697),
    [sym_time_to_minute] = ACTIONS(1697),
    [sym_time_to_second] = ACTIONS(1697),
    [sym_day_of_week] = ACTIONS(1697),
    [sym_set_pulse_time] = ACTIONS(1697),
    [sym_set_timer] = ACTIONS(1697),
    [sym_file_open] = ACTIONS(1697),
    [sym_file_close] = ACTIONS(1697),
    [sym_file_read] = ACTIONS(1697),
    [sym_file_read_line] = ACTIONS(1697),
    [sym_file_write] = ACTIONS(1697),
    [sym_file_write_line] = ACTIONS(1697),
    [sym_file_copy] = ACTIONS(1697),
    [sym_file_delete] = ACTIONS(1697),
    [sym_file_rename] = ACTIONS(1697),
    [sym_file_dir] = ACTIONS(1697),
    [sym_file_getdir] = ACTIONS(1697),
    [sym_file_setdir] = ACTIONS(1697),
    [sym_file_create_dir] = ACTIONS(1697),
    [sym_file_remove_dir] = ACTIONS(1697),
    [sym_file_seek] = ACTIONS(1697),
    [sym_length_array] = ACTIONS(1697),
    [sym_max_length_array] = ACTIONS(1697),
    [sym_set_length_array] = ACTIONS(1697),
    [sym_ip_client_open] = ACTIONS(1697),
    [sym_ip_client_close] = ACTIONS(1697),
    [sym_ip_server_open] = ACTIONS(1697),
    [sym_ip_server_close] = ACTIONS(1697),
    [sym_ip_bound_client_open] = ACTIONS(1697),
    [sym_ip_mc_server_open] = ACTIONS(1697),
    [sym_ip_set_option] = ACTIONS(1697),
    [sym_ssh_client_open] = ACTIONS(1697),
    [sym_ssh_client_close] = ACTIONS(1697),
    [sym_tls_client_open] = ACTIONS(1697),
    [sym_tls_client_close] = ACTIONS(1697),
    [sym_do_push] = ACTIONS(1697),
    [sym_do_push_timed] = ACTIONS(1697),
    [sym_do_release] = ACTIONS(1697),
    [sym_device_id] = ACTIONS(1697),
    [sym_device_info] = ACTIONS(1697),
    [sym_device_id_string] = ACTIONS(1697),
    [sym_device_standby] = ACTIONS(1697),
    [sym_device_wake] = ACTIONS(1697),
    [sym_timeline_create] = ACTIONS(1697),
    [sym_timeline_kill] = ACTIONS(1697),
    [sym_timeline_reload] = ACTIONS(1697),
    [sym_timeline_pause] = ACTIONS(1697),
    [sym_timeline_restart] = ACTIONS(1697),
    [sym_timeline_set] = ACTIONS(1697),
    [sym_timeline_get] = ACTIONS(1697),
    [sym_timeline_active] = ACTIONS(1697),
    [sym_timeline_dynamic_id] = ACTIONS(1697),
    [sym_variable_to_string] = ACTIONS(1697),
    [sym_string_to_variable] = ACTIONS(1697),
    [sym_length_variable_to_string] = ACTIONS(1697),
    [sym_type_cast] = ACTIONS(1697),
    [sym_variable_to_xml] = ACTIONS(1697),
    [sym_xml_to_variable] = ACTIONS(1697),
    [sym_length_variable_to_xml] = ACTIONS(1697),
    [sym_redirect_string] = ACTIONS(1697),
    [sym_random_number] = ACTIONS(1697),
    [sym_get_unique_id] = ACTIONS(1697),
    [sym_get_system_number] = ACTIONS(1697),
    [sym_set_system_number] = ACTIONS(1697),
    [sym_get_serial_number] = ACTIONS(1697),
    [sym_get_master_build] = ACTIONS(1697),
    [sym_set_validation_code] = ACTIONS(1697),
    [sym_reboot] = ACTIONS(1697),
    [sym_load_duet_module] = ACTIONS(1697),
    [sym_unload_duet_module] = ACTIONS(1697),
    [sym_duet_mem_size_set] = ACTIONS(1697),
    [sym_duet_mem_size_get] = ACTIONS(1697),
    [sym_dynamic_application_device] = ACTIONS(1697),
    [sym_dynamic_polled_port] = ACTIONS(1697),
    [sym_static_port_binding] = ACTIONS(1697),
    [sym_static_ip_binding] = ACTIONS(1697),
    [sym_get_dns_list] = ACTIONS(1697),
    [sym_set_dns_list] = ACTIONS(1697),
    [sym_get_ip_address] = ACTIONS(1697),
    [sym_set_ip_address] = ACTIONS(1697),
    [sym_get_url_list] = ACTIONS(1697),
    [sym_add_url_entry] = ACTIONS(1697),
    [sym_delete_url_entry] = ACTIONS(1697),
    [sym_abs_value] = ACTIONS(1697),
    [sym_max_value] = ACTIONS(1697),
    [sym_min_value] = ACTIONS(1697),
    [sym_exp_value] = ACTIONS(1697),
    [sym_log_value] = ACTIONS(1697),
    [sym_log10_value] = ACTIONS(1697),
    [sym_power_value] = ACTIONS(1697),
    [sym_sqrt_value] = ACTIONS(1697),
    [sym_raw_be] = ACTIONS(1697),
    [sym_raw_le] = ACTIONS(1697),
    [sym_internal_queue_size_set] = ACTIONS(1697),
    [sym_internal_queue_size_get] = ACTIONS(1697),
    [sym_internal_threshold_set] = ACTIONS(1697),
    [sym_internal_threshold_get] = ACTIONS(1697),
    [sym_rebuilt_event] = ACTIONS(1697),
    [sym_do_custom_event] = ACTIONS(1697),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1697),
    [sym_clkmgr_set_clk_source] = ACTIONS(1697),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1697),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1697),
    [sym_clkmgr_get_timezone] = ACTIONS(1697),
    [sym_clkmgr_set_timezone] = ACTIONS(1697),
    [sym_clkmgr_get_resync_period] = ACTIONS(1697),
    [sym_clkmgr_set_resync_period] = ACTIONS(1697),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1697),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1697),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1697),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1697),
    [sym_clkmgr_get_timeservers] = ACTIONS(1697),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1697),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1697),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1697),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1697),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1697),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1697),
    [sym_set_log_level] = ACTIONS(1697),
    [sym_get_log_level] = ACTIONS(1697),
    [sym_amx_log] = ACTIONS(1697),
    [sym_smtp_server_config_set] = ACTIONS(1697),
    [sym_smtp_server_config_get] = ACTIONS(1697),
    [sym_smtp_send] = ACTIONS(1697),
    [sym_validate_netlinx_account] = ACTIONS(1697),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1697),
    [sym_authenticate_certificate] = ACTIONS(1697),
    [sym_audit_netlinx_session_event] = ACTIONS(1697),
    [sym_audit_netlinx_generic_event] = ACTIONS(1697),
    [sym_get_available_flash_disk_space] = ACTIONS(1697),
    [sym_get_max_flash_disk_space] = ACTIONS(1697),
    [sym_data_create_feed] = ACTIONS(1697),
    [sym_data_delete_feed] = ACTIONS(1697),
    [sym_data_publish_feed] = ACTIONS(1697),
    [sym_data_get_published_feed] = ACTIONS(1697),
    [sym_data_add_record] = ACTIONS(1697),
    [sym_data_get_event_record] = ACTIONS(1697),
    [sym__wc_data_create_feed] = ACTIONS(1697),
    [sym__wc_data_add_record] = ACTIONS(1697),
    [sym__wc_data_get_event_record] = ACTIONS(1697),
    [sym_format] = ACTIONS(1697),
    [sym_astro_clock] = ACTIONS(1697),
    [sym_set_outdoor_temperature] = ACTIONS(1697),
    [sym_set_virtual_level_count] = ACTIONS(1697),
    [sym_set_virtual_channel_count] = ACTIONS(1697),
    [sym_set_virtual_port_count] = ACTIONS(1697),
    [sym__wc_encode_int] = ACTIONS(1697),
    [sym__wc_decode_int] = ACTIONS(1697),
    [sym__wc_int] = ACTIONS(1697),
    [sym__wc_to_ch_int] = ACTIONS(1697),
    [sym__ch_to_wc_int] = ACTIONS(1697),
    [sym__wc_find_string_int] = ACTIONS(1697),
    [sym__wc_left_string_int] = ACTIONS(1697),
    [sym__wc_lower_string_int] = ACTIONS(1697),
    [sym__wc_mid_string_int] = ACTIONS(1697),
    [sym__wc_remove_string_int] = ACTIONS(1697),
    [sym__wc_right_string_int] = ACTIONS(1697),
    [sym__wc_upper_string_int] = ACTIONS(1697),
    [sym__wc_compare_string_int] = ACTIONS(1697),
    [sym__wc_get_buffer_char_int] = ACTIONS(1697),
    [sym__wc_get_buffer_string_int] = ACTIONS(1697),
    [sym__wc_concat_string_int] = ACTIONS(1697),
    [sym__wc_file_open_int] = ACTIONS(1697),
    [sym__wc_file_close_int] = ACTIONS(1697),
    [sym__wc_file_read_int] = ACTIONS(1697),
    [sym__wc_file_read_line_int] = ACTIONS(1697),
    [sym__wc_file_write_int] = ACTIONS(1697),
    [sym__wc_file_write_line_int] = ACTIONS(1697),
    [aux_sym_preproc_include_token1] = ACTIONS(1697),
    [aux_sym_preproc_define_token1] = ACTIONS(1695),
    [aux_sym_preproc_warn_token1] = ACTIONS(1695),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1695),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1695),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1695),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1695),
    [aux_sym_preproc_else_token1] = ACTIONS(1695),
    [anon_sym_LBRACK] = ACTIONS(1695),
    [anon_sym_LBRACE] = ACTIONS(1695),
    [anon_sym_RBRACE] = ACTIONS(1695),
    [anon_sym_LPAREN] = ACTIONS(1697),
    [anon_sym_BANG] = ACTIONS(1695),
    [anon_sym_TILDE] = ACTIONS(1695),
    [anon_sym_DASH] = ACTIONS(1697),
    [anon_sym_PLUS] = ACTIONS(1697),
    [anon_sym_DASH_DASH] = ACTIONS(1695),
    [anon_sym_PLUS_PLUS] = ACTIONS(1695),
    [anon_sym_DQUOTE] = ACTIONS(1695),
    [anon_sym_SQUOTE] = ACTIONS(1695),
    [sym_number_literal] = ACTIONS(1695),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(204)] = {
    [ts_builtin_sym_end] = ACTIONS(1699),
    [sym_identifier] = ACTIONS(1701),
    [sym_program_name_keyword] = ACTIONS(1701),
    [sym_module_name_keyword] = ACTIONS(1701),
    [sym_define_device_keyword] = ACTIONS(1701),
    [sym_define_combine_keyword] = ACTIONS(1701),
    [sym_define_constant_keyword] = ACTIONS(1701),
    [sym_define_type_keyword] = ACTIONS(1701),
    [sym_define_variable_keyword] = ACTIONS(1701),
    [sym_define_system_variable_keyword] = ACTIONS(1701),
    [sym_define_connect_level_keyword] = ACTIONS(1701),
    [sym_define_latching_keyword] = ACTIONS(1701),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1701),
    [sym_define_toggling_keyword] = ACTIONS(1701),
    [sym_define_call_keyword] = ACTIONS(1701),
    [sym_define_function_keyword] = ACTIONS(1701),
    [sym_define_library_function_keyword] = ACTIONS(1701),
    [sym_define_start_keyword] = ACTIONS(1701),
    [sym_define_event_keyword] = ACTIONS(1701),
    [sym_define_program_keyword] = ACTIONS(1701),
    [sym_define_module_keyword] = ACTIONS(1701),
    [sym_char_keyword] = ACTIONS(1701),
    [sym_widechar_keyword] = ACTIONS(1701),
    [sym_integer_keyword] = ACTIONS(1701),
    [sym_sinteger_keyword] = ACTIONS(1701),
    [sym_long_keyword] = ACTIONS(1701),
    [sym_slong_keyword] = ACTIONS(1701),
    [sym_float_keyword] = ACTIONS(1701),
    [sym_double_keyword] = ACTIONS(1701),
    [sym_variant_keyword] = ACTIONS(1701),
    [sym_variantarray_keyword] = ACTIONS(1701),
    [sym_local_var_keyword] = ACTIONS(1701),
    [sym_stack_var_keyword] = ACTIONS(1701),
    [sym_constant_keyword] = ACTIONS(1701),
    [sym_volatile_keyword] = ACTIONS(1701),
    [sym_non_volatile_keyword] = ACTIONS(1701),
    [sym_persistent_keyword] = ACTIONS(1701),
    [sym_dev_keyword] = ACTIONS(1701),
    [sym_devlev_keyword] = ACTIONS(1701),
    [sym_devchan_keyword] = ACTIONS(1701),
    [sym_if_keyword] = ACTIONS(1701),
    [sym_switch_keyword] = ACTIONS(1701),
    [sym_case_keyword] = ACTIONS(1701),
    [sym_default_keyword] = ACTIONS(1701),
    [sym_while_keyword] = ACTIONS(1701),
    [sym_for_keyword] = ACTIONS(1701),
    [sym_break_keyword] = ACTIONS(1701),
    [sym_continue_keyword] = ACTIONS(1701),
    [sym_return_keyword] = ACTIONS(1701),
    [sym_select_keyword] = ACTIONS(1701),
    [sym_active_keyword] = ACTIONS(1701),
    [sym_struct_keyword] = ACTIONS(1701),
    [sym_structure_keyword] = ACTIONS(1701),
    [sym_button_event_keyword] = ACTIONS(1701),
    [sym_channel_event_keyword] = ACTIONS(1701),
    [sym_level_event_keyword] = ACTIONS(1701),
    [sym_data_event_keyword] = ACTIONS(1701),
    [sym_timeline_event_keyword] = ACTIONS(1701),
    [sym_custom_event_keyword] = ACTIONS(1701),
    [sym_push_keyword] = ACTIONS(1701),
    [sym_release_keyword] = ACTIONS(1701),
    [aux_sym_on_keyword_token1] = ACTIONS(1701),
    [aux_sym_off_keyword_token1] = ACTIONS(1701),
    [sym_send_level_keyword] = ACTIONS(1701),
    [sym_send_string_keyword] = ACTIONS(1701),
    [sym_send_command_keyword] = ACTIONS(1701),
    [sym_clear_buffer_keyword] = ACTIONS(1701),
    [sym_create_buffer_keyword] = ACTIONS(1701),
    [sym_create_multi_buffer_keyword] = ACTIONS(1701),
    [sym_call_keyword] = ACTIONS(1701),
    [sym_system_call_keyword] = ACTIONS(1701),
    [sym_devchan_to_keyword] = ACTIONS(1701),
    [sym_devchan_min_to_keyword] = ACTIONS(1701),
    [sym_devchan_total_off_keyword] = ACTIONS(1701),
    [sym_devchan_pulse_keyword] = ACTIONS(1701),
    [sym_wait_keyword] = ACTIONS(1701),
    [sym_cancel_wait_keyword] = ACTIONS(1701),
    [sym_wait_until_keyword] = ACTIONS(1701),
    [sym_cancel_wait_until_keyword] = ACTIONS(1701),
    [sym_cancel_all_wait_keyword] = ACTIONS(1701),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1701),
    [sym_not] = ACTIONS(1701),
    [sym_bnot] = ACTIONS(1701),
    [sym___file__] = ACTIONS(1701),
    [sym___date__] = ACTIONS(1701),
    [sym___ldate__] = ACTIONS(1701),
    [sym___time__] = ACTIONS(1701),
    [sym___line__] = ACTIONS(1701),
    [sym___name__] = ACTIONS(1701),
    [sym___netlinx__] = ACTIONS(1701),
    [sym_netlinx_axi_version] = ACTIONS(1701),
    [sym_url_flg_tcp] = ACTIONS(1701),
    [sym_url_flg_accinfopresent] = ACTIONS(1701),
    [sym_url_flg_temp] = ACTIONS(1701),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1701),
    [sym_url_flg_stat_mask] = ACTIONS(1701),
    [sym_url_flg_stat_lookup] = ACTIONS(1701),
    [sym_url_flg_stat_connecting] = ACTIONS(1701),
    [sym_url_flg_stat_waiting] = ACTIONS(1701),
    [sym_url_flg_stat_connected] = ACTIONS(1701),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1701),
    [sym_first_virtual_device] = ACTIONS(1701),
    [sym_first_local_port] = ACTIONS(1701),
    [sym_dynamic_virtual_device] = ACTIONS(1701),
    [sym_timeline_once] = ACTIONS(1701),
    [sym_timeline_repeat] = ACTIONS(1701),
    [sym_timeline_absolute] = ACTIONS(1701),
    [sym_timeline_relative] = ACTIONS(1701),
    [sym_do_push_timed_infinite] = ACTIONS(1701),
    [sym_source_type_no_address] = ACTIONS(1701),
    [sym_source_type_neuron_id] = ACTIONS(1701),
    [sym_source_type_ip_address] = ACTIONS(1701),
    [sym_source_type_axlink] = ACTIONS(1701),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1701),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1701),
    [sym_source_type_ip_socket_address] = ACTIONS(1701),
    [sym_source_type_rs232] = ACTIONS(1701),
    [sym_source_type_internal] = ACTIONS(1701),
    [sym_source_type_mac_address] = ACTIONS(1701),
    [sym_source_type_ipv4_port] = ACTIONS(1701),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1701),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1701),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1701),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1701),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1701),
    [sym_true] = ACTIONS(1701),
    [sym_false] = ACTIONS(1701),
    [sym_file_read_only] = ACTIONS(1701),
    [sym_file_rw_new] = ACTIONS(1701),
    [sym_file_rw_append] = ACTIONS(1701),
    [sym_ip_tcp] = ACTIONS(1701),
    [sym_ip_udp] = ACTIONS(1701),
    [sym_ip_udp_2way] = ACTIONS(1701),
    [sym_xml_encode_types] = ACTIONS(1701),
    [sym_xml_encode_char_as_list] = ACTIONS(1701),
    [sym_xml_encode_le] = ACTIONS(1701),
    [sym_xml_decode_types] = ACTIONS(1701),
    [sym_xml_decode_no_preserve] = ACTIONS(1701),
    [sym_duet_dev_type_display_device] = ACTIONS(1701),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1701),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1701),
    [sym_duet_dev_type_io_device] = ACTIONS(1701),
    [sym_duet_dev_type_relay_device] = ACTIONS(1701),
    [sym_duet_dev_type_ups] = ACTIONS(1701),
    [sym_duet_dev_type_amplifier] = ACTIONS(1701),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1701),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1701),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1701),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1701),
    [sym_duet_dev_type_camera] = ACTIONS(1701),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1701),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1701),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1701),
    [sym_duet_dev_type_dss] = ACTIONS(1701),
    [sym_duet_dev_type_dvr] = ACTIONS(1701),
    [sym_duet_dev_type_disc_device] = ACTIONS(1701),
    [sym_duet_dev_type_document_camera] = ACTIONS(1701),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1701),
    [sym_duet_dev_type_hvac] = ACTIONS(1701),
    [sym_duet_dev_type_keypad] = ACTIONS(1701),
    [sym_duet_dev_type_light] = ACTIONS(1701),
    [sym_duet_dev_type_monitor] = ACTIONS(1701),
    [sym_duet_dev_type_motor] = ACTIONS(1701),
    [sym_duet_dev_type_multi_window] = ACTIONS(1701),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1701),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1701),
    [sym_duet_dev_type_receiver] = ACTIONS(1701),
    [sym_duet_dev_type_security_system] = ACTIONS(1701),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1701),
    [sym_duet_dev_type_settop_box] = ACTIONS(1701),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1701),
    [sym_duet_dev_type_switcher] = ACTIONS(1701),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1701),
    [sym_duet_dev_type_tv] = ACTIONS(1701),
    [sym_duet_dev_type_utility] = ACTIONS(1701),
    [sym_duet_dev_type_vcr] = ACTIONS(1701),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1701),
    [sym_duet_dev_type_video_processor] = ACTIONS(1701),
    [sym_duet_dev_type_video_projector] = ACTIONS(1701),
    [sym_duet_dev_type_video_wall] = ACTIONS(1701),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1701),
    [sym_duet_dev_type_weather] = ACTIONS(1701),
    [sym_duet_dev_not_polled] = ACTIONS(1701),
    [sym_duet_dev_polled] = ACTIONS(1701),
    [sym_clkmgr_mode_network] = ACTIONS(1701),
    [sym_clkmgr_mode_standalone] = ACTIONS(1701),
    [sym_normal_standby] = ACTIONS(1701),
    [sym_normal_wake] = ACTIONS(1701),
    [sym_ip_multicast_ttl_option] = ACTIONS(1701),
    [sym_ip_ttl_subnet] = ACTIONS(1701),
    [sym_ip_ttl_site] = ACTIONS(1701),
    [sym_ip_ttl_region] = ACTIONS(1701),
    [sym_ip_ttl_continent] = ACTIONS(1701),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1701),
    [sym_ip_nodelay_on] = ACTIONS(1701),
    [sym_ip_nodelay_off] = ACTIONS(1701),
    [sym_amx_error] = ACTIONS(1701),
    [sym_amx_warning] = ACTIONS(1701),
    [sym_amx_info] = ACTIONS(1701),
    [sym_amx_debug] = ACTIONS(1701),
    [sym_netlinx_login_success] = ACTIONS(1701),
    [sym_netlinx_login_fail] = ACTIONS(1701),
    [sym_netlinx_logout] = ACTIONS(1701),
    [sym_null_str] = ACTIONS(1701),
    [sym_smtp_address] = ACTIONS(1701),
    [sym_smtp_port_number] = ACTIONS(1701),
    [sym_smtp_username] = ACTIONS(1701),
    [sym_smtp_password] = ACTIONS(1701),
    [sym_smtp_require_tls] = ACTIONS(1701),
    [sym_smtp_from] = ACTIONS(1701),
    [sym_smtp_tls_true] = ACTIONS(1701),
    [sym_smtp_tls_false] = ACTIONS(1701),
    [sym_listview_on_row_select_event] = ACTIONS(1701),
    [sym_data_structure_unknown] = ACTIONS(1701),
    [sym_data_structure_datafeed] = ACTIONS(1701),
    [sym_data_structure_datarecordset] = ACTIONS(1701),
    [sym_data_structure_datarecord] = ACTIONS(1701),
    [sym_data_structure_datafield] = ACTIONS(1701),
    [sym_data_max_content_field_count] = ACTIONS(1701),
    [sym_data_max_metadata_field_count] = ACTIONS(1701),
    [sym_data_max_id_length] = ACTIONS(1701),
    [sym_data_max_name_length] = ACTIONS(1701),
    [sym_data_max_description_length] = ACTIONS(1701),
    [sym_data_max_type_length] = ACTIONS(1701),
    [sym_data_max_format_length] = ACTIONS(1701),
    [sym_data_max_source_length] = ACTIONS(1701),
    [sym_data_max_value_length] = ACTIONS(1701),
    [sym_data_max_label_length] = ACTIONS(1701),
    [sym_data_max_url_length] = ACTIONS(1701),
    [sym_max_last_login_info_length] = ACTIONS(1701),
    [sym_data_type_unknown] = ACTIONS(1701),
    [sym_data_type_string] = ACTIONS(1701),
    [sym_data_type_datetime] = ACTIONS(1701),
    [sym_data_type_date] = ACTIONS(1701),
    [sym_data_type_time] = ACTIONS(1701),
    [sym_data_type_image] = ACTIONS(1701),
    [sym_data_format_url] = ACTIONS(1701),
    [sym_data_format_phone] = ACTIONS(1701),
    [sym_data_format_email] = ACTIONS(1701),
    [sym_data_format_iso8601] = ACTIONS(1701),
    [sym_auditlog_priv] = ACTIONS(1701),
    [sym_configuration_priv] = ACTIONS(1701),
    [sym_deviceconfig_priv] = ACTIONS(1701),
    [sym_ftp_priv] = ACTIONS(1701),
    [sym_http_priv] = ACTIONS(1701),
    [sym_networkconfig_priv] = ACTIONS(1701),
    [sym_programport_priv] = ACTIONS(1701),
    [sym_remoteui_priv] = ACTIONS(1701),
    [sym_securitycontrol_priv] = ACTIONS(1701),
    [sym_softwaremanagement_priv] = ACTIONS(1701),
    [sym_terminal_priv] = ACTIONS(1701),
    [sym_tpadmin_priv] = ACTIONS(1701),
    [sym_user1_priv] = ACTIONS(1701),
    [sym_user2_priv] = ACTIONS(1701),
    [sym_user3_priv] = ACTIONS(1701),
    [sym_user4_priv] = ACTIONS(1701),
    [sym_usermanagement_priv] = ACTIONS(1701),
    [sym_valid_account] = ACTIONS(1701),
    [sym_err_name_invalid] = ACTIONS(1701),
    [sym_err_password_invalid] = ACTIONS(1701),
    [sym_err_invalid_account] = ACTIONS(1701),
    [sym_err_no_authorization] = ACTIONS(1701),
    [sym_err_invalid_parameter] = ACTIONS(1701),
    [sym_err_user_locked_out] = ACTIONS(1701),
    [sym_err_user_account_expired] = ACTIONS(1701),
    [sym_err_general_error] = ACTIONS(1701),
    [sym_err_authentication_error] = ACTIONS(1701),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1701),
    [sym_tls_validate_certificate] = ACTIONS(1701),
    [sym_tdata] = ACTIONS(1701),
    [sym_tchannel] = ACTIONS(1701),
    [sym_tlevel] = ACTIONS(1701),
    [sym_tbutton] = ACTIONS(1701),
    [sym_ttimeline] = ACTIONS(1701),
    [sym_tcustom] = ACTIONS(1701),
    [sym_url_struct] = ACTIONS(1701),
    [sym_dns_struct] = ACTIONS(1701),
    [sym_ip_address_struct] = ACTIONS(1701),
    [sym_dev_info_struct] = ACTIONS(1701),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1701),
    [sym_data_feed] = ACTIONS(1701),
    [sym_data_field] = ACTIONS(1701),
    [sym_data_record] = ACTIONS(1701),
    [sym_wc_data_feed] = ACTIONS(1701),
    [sym_wc_data_field] = ACTIONS(1701),
    [sym_wc_data_record] = ACTIONS(1701),
    [sym_last_login_info] = ACTIONS(1701),
    [sym____reserved___] = ACTIONS(1701),
    [sym_date] = ACTIONS(1701),
    [sym_day] = ACTIONS(1701),
    [sym_time] = ACTIONS(1701),
    [sym_ldate] = ACTIONS(1701),
    [sym_push_device] = ACTIONS(1701),
    [sym_push_channel] = ACTIONS(1701),
    [sym_push_devchan] = ACTIONS(1701),
    [sym_release_device] = ACTIONS(1701),
    [sym_release_channel] = ACTIONS(1701),
    [sym_release_devchan] = ACTIONS(1701),
    [sym_master_slot] = ACTIONS(1701),
    [sym_get_pulse_time] = ACTIONS(1701),
    [sym_get_timer] = ACTIONS(1701),
    [sym_master_sn] = ACTIONS(1701),
    [sym_system_number] = ACTIONS(1701),
    [sym_button] = ACTIONS(1701),
    [sym_dv_channel] = ACTIONS(1701),
    [sym_data] = ACTIONS(1701),
    [sym_level] = ACTIONS(1701),
    [sym_timeline] = ACTIONS(1701),
    [sym_channel] = ACTIONS(1701),
    [sym_custom] = ACTIONS(1701),
    [sym_atoi] = ACTIONS(1701),
    [sym_atol] = ACTIONS(1701),
    [sym_atof] = ACTIONS(1701),
    [sym_itoa] = ACTIONS(1701),
    [sym_itohex] = ACTIONS(1701),
    [sym_hextoi] = ACTIONS(1701),
    [sym_ftoa] = ACTIONS(1701),
    [sym_find_string] = ACTIONS(1701),
    [sym_left_string] = ACTIONS(1701),
    [sym_right_string] = ACTIONS(1701),
    [sym_mid_string] = ACTIONS(1701),
    [sym_remove_string] = ACTIONS(1701),
    [sym_get_buffer_string] = ACTIONS(1701),
    [sym_get_buffer_char] = ACTIONS(1701),
    [sym_get_multi_buffer_string] = ACTIONS(1701),
    [sym_length_string] = ACTIONS(1701),
    [sym_max_length_string] = ACTIONS(1701),
    [sym_set_length_string] = ACTIONS(1701),
    [sym_compare_string] = ACTIONS(1701),
    [sym_upper_string] = ACTIONS(1701),
    [sym_lower_string] = ACTIONS(1701),
    [sym_get_last] = ACTIONS(1701),
    [sym_date_to_day] = ACTIONS(1701),
    [sym_date_to_month] = ACTIONS(1701),
    [sym_date_to_year] = ACTIONS(1701),
    [sym_time_to_hour] = ACTIONS(1701),
    [sym_time_to_minute] = ACTIONS(1701),
    [sym_time_to_second] = ACTIONS(1701),
    [sym_day_of_week] = ACTIONS(1701),
    [sym_set_pulse_time] = ACTIONS(1701),
    [sym_set_timer] = ACTIONS(1701),
    [sym_file_open] = ACTIONS(1701),
    [sym_file_close] = ACTIONS(1701),
    [sym_file_read] = ACTIONS(1701),
    [sym_file_read_line] = ACTIONS(1701),
    [sym_file_write] = ACTIONS(1701),
    [sym_file_write_line] = ACTIONS(1701),
    [sym_file_copy] = ACTIONS(1701),
    [sym_file_delete] = ACTIONS(1701),
    [sym_file_rename] = ACTIONS(1701),
    [sym_file_dir] = ACTIONS(1701),
    [sym_file_getdir] = ACTIONS(1701),
    [sym_file_setdir] = ACTIONS(1701),
    [sym_file_create_dir] = ACTIONS(1701),
    [sym_file_remove_dir] = ACTIONS(1701),
    [sym_file_seek] = ACTIONS(1701),
    [sym_length_array] = ACTIONS(1701),
    [sym_max_length_array] = ACTIONS(1701),
    [sym_set_length_array] = ACTIONS(1701),
    [sym_ip_client_open] = ACTIONS(1701),
    [sym_ip_client_close] = ACTIONS(1701),
    [sym_ip_server_open] = ACTIONS(1701),
    [sym_ip_server_close] = ACTIONS(1701),
    [sym_ip_bound_client_open] = ACTIONS(1701),
    [sym_ip_mc_server_open] = ACTIONS(1701),
    [sym_ip_set_option] = ACTIONS(1701),
    [sym_ssh_client_open] = ACTIONS(1701),
    [sym_ssh_client_close] = ACTIONS(1701),
    [sym_tls_client_open] = ACTIONS(1701),
    [sym_tls_client_close] = ACTIONS(1701),
    [sym_do_push] = ACTIONS(1701),
    [sym_do_push_timed] = ACTIONS(1701),
    [sym_do_release] = ACTIONS(1701),
    [sym_device_id] = ACTIONS(1701),
    [sym_device_info] = ACTIONS(1701),
    [sym_device_id_string] = ACTIONS(1701),
    [sym_device_standby] = ACTIONS(1701),
    [sym_device_wake] = ACTIONS(1701),
    [sym_timeline_create] = ACTIONS(1701),
    [sym_timeline_kill] = ACTIONS(1701),
    [sym_timeline_reload] = ACTIONS(1701),
    [sym_timeline_pause] = ACTIONS(1701),
    [sym_timeline_restart] = ACTIONS(1701),
    [sym_timeline_set] = ACTIONS(1701),
    [sym_timeline_get] = ACTIONS(1701),
    [sym_timeline_active] = ACTIONS(1701),
    [sym_timeline_dynamic_id] = ACTIONS(1701),
    [sym_variable_to_string] = ACTIONS(1701),
    [sym_string_to_variable] = ACTIONS(1701),
    [sym_length_variable_to_string] = ACTIONS(1701),
    [sym_type_cast] = ACTIONS(1701),
    [sym_variable_to_xml] = ACTIONS(1701),
    [sym_xml_to_variable] = ACTIONS(1701),
    [sym_length_variable_to_xml] = ACTIONS(1701),
    [sym_redirect_string] = ACTIONS(1701),
    [sym_random_number] = ACTIONS(1701),
    [sym_get_unique_id] = ACTIONS(1701),
    [sym_get_system_number] = ACTIONS(1701),
    [sym_set_system_number] = ACTIONS(1701),
    [sym_get_serial_number] = ACTIONS(1701),
    [sym_get_master_build] = ACTIONS(1701),
    [sym_set_validation_code] = ACTIONS(1701),
    [sym_reboot] = ACTIONS(1701),
    [sym_load_duet_module] = ACTIONS(1701),
    [sym_unload_duet_module] = ACTIONS(1701),
    [sym_duet_mem_size_set] = ACTIONS(1701),
    [sym_duet_mem_size_get] = ACTIONS(1701),
    [sym_dynamic_application_device] = ACTIONS(1701),
    [sym_dynamic_polled_port] = ACTIONS(1701),
    [sym_static_port_binding] = ACTIONS(1701),
    [sym_static_ip_binding] = ACTIONS(1701),
    [sym_get_dns_list] = ACTIONS(1701),
    [sym_set_dns_list] = ACTIONS(1701),
    [sym_get_ip_address] = ACTIONS(1701),
    [sym_set_ip_address] = ACTIONS(1701),
    [sym_get_url_list] = ACTIONS(1701),
    [sym_add_url_entry] = ACTIONS(1701),
    [sym_delete_url_entry] = ACTIONS(1701),
    [sym_abs_value] = ACTIONS(1701),
    [sym_max_value] = ACTIONS(1701),
    [sym_min_value] = ACTIONS(1701),
    [sym_exp_value] = ACTIONS(1701),
    [sym_log_value] = ACTIONS(1701),
    [sym_log10_value] = ACTIONS(1701),
    [sym_power_value] = ACTIONS(1701),
    [sym_sqrt_value] = ACTIONS(1701),
    [sym_raw_be] = ACTIONS(1701),
    [sym_raw_le] = ACTIONS(1701),
    [sym_internal_queue_size_set] = ACTIONS(1701),
    [sym_internal_queue_size_get] = ACTIONS(1701),
    [sym_internal_threshold_set] = ACTIONS(1701),
    [sym_internal_threshold_get] = ACTIONS(1701),
    [sym_rebuilt_event] = ACTIONS(1701),
    [sym_do_custom_event] = ACTIONS(1701),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1701),
    [sym_clkmgr_set_clk_source] = ACTIONS(1701),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1701),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1701),
    [sym_clkmgr_get_timezone] = ACTIONS(1701),
    [sym_clkmgr_set_timezone] = ACTIONS(1701),
    [sym_clkmgr_get_resync_period] = ACTIONS(1701),
    [sym_clkmgr_set_resync_period] = ACTIONS(1701),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1701),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1701),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1701),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1701),
    [sym_clkmgr_get_timeservers] = ACTIONS(1701),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1701),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1701),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1701),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1701),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1701),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1701),
    [sym_set_log_level] = ACTIONS(1701),
    [sym_get_log_level] = ACTIONS(1701),
    [sym_amx_log] = ACTIONS(1701),
    [sym_smtp_server_config_set] = ACTIONS(1701),
    [sym_smtp_server_config_get] = ACTIONS(1701),
    [sym_smtp_send] = ACTIONS(1701),
    [sym_validate_netlinx_account] = ACTIONS(1701),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1701),
    [sym_authenticate_certificate] = ACTIONS(1701),
    [sym_audit_netlinx_session_event] = ACTIONS(1701),
    [sym_audit_netlinx_generic_event] = ACTIONS(1701),
    [sym_get_available_flash_disk_space] = ACTIONS(1701),
    [sym_get_max_flash_disk_space] = ACTIONS(1701),
    [sym_data_create_feed] = ACTIONS(1701),
    [sym_data_delete_feed] = ACTIONS(1701),
    [sym_data_publish_feed] = ACTIONS(1701),
    [sym_data_get_published_feed] = ACTIONS(1701),
    [sym_data_add_record] = ACTIONS(1701),
    [sym_data_get_event_record] = ACTIONS(1701),
    [sym__wc_data_create_feed] = ACTIONS(1701),
    [sym__wc_data_add_record] = ACTIONS(1701),
    [sym__wc_data_get_event_record] = ACTIONS(1701),
    [sym_format] = ACTIONS(1701),
    [sym_astro_clock] = ACTIONS(1701),
    [sym_set_outdoor_temperature] = ACTIONS(1701),
    [sym_set_virtual_level_count] = ACTIONS(1701),
    [sym_set_virtual_channel_count] = ACTIONS(1701),
    [sym_set_virtual_port_count] = ACTIONS(1701),
    [sym__wc_encode_int] = ACTIONS(1701),
    [sym__wc_decode_int] = ACTIONS(1701),
    [sym__wc_int] = ACTIONS(1701),
    [sym__wc_to_ch_int] = ACTIONS(1701),
    [sym__ch_to_wc_int] = ACTIONS(1701),
    [sym__wc_find_string_int] = ACTIONS(1701),
    [sym__wc_left_string_int] = ACTIONS(1701),
    [sym__wc_lower_string_int] = ACTIONS(1701),
    [sym__wc_mid_string_int] = ACTIONS(1701),
    [sym__wc_remove_string_int] = ACTIONS(1701),
    [sym__wc_right_string_int] = ACTIONS(1701),
    [sym__wc_upper_string_int] = ACTIONS(1701),
    [sym__wc_compare_string_int] = ACTIONS(1701),
    [sym__wc_get_buffer_char_int] = ACTIONS(1701),
    [sym__wc_get_buffer_string_int] = ACTIONS(1701),
    [sym__wc_concat_string_int] = ACTIONS(1701),
    [sym__wc_file_open_int] = ACTIONS(1701),
    [sym__wc_file_close_int] = ACTIONS(1701),
    [sym__wc_file_read_int] = ACTIONS(1701),
    [sym__wc_file_read_line_int] = ACTIONS(1701),
    [sym__wc_file_write_int] = ACTIONS(1701),
    [sym__wc_file_write_line_int] = ACTIONS(1701),
    [aux_sym_preproc_include_token1] = ACTIONS(1701),
    [aux_sym_preproc_define_token1] = ACTIONS(1699),
    [aux_sym_preproc_warn_token1] = ACTIONS(1699),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1699),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1699),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1699),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1699),
    [aux_sym_preproc_else_token1] = ACTIONS(1699),
    [anon_sym_LBRACK] = ACTIONS(1699),
    [anon_sym_LBRACE] = ACTIONS(1699),
    [anon_sym_RBRACE] = ACTIONS(1699),
    [anon_sym_LPAREN] = ACTIONS(1701),
    [anon_sym_BANG] = ACTIONS(1699),
    [anon_sym_TILDE] = ACTIONS(1699),
    [anon_sym_DASH] = ACTIONS(1701),
    [anon_sym_PLUS] = ACTIONS(1701),
    [anon_sym_DASH_DASH] = ACTIONS(1699),
    [anon_sym_PLUS_PLUS] = ACTIONS(1699),
    [anon_sym_DQUOTE] = ACTIONS(1699),
    [anon_sym_SQUOTE] = ACTIONS(1699),
    [sym_number_literal] = ACTIONS(1699),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(205)] = {
    [ts_builtin_sym_end] = ACTIONS(1703),
    [sym_identifier] = ACTIONS(1705),
    [sym_program_name_keyword] = ACTIONS(1705),
    [sym_module_name_keyword] = ACTIONS(1705),
    [sym_define_device_keyword] = ACTIONS(1705),
    [sym_define_combine_keyword] = ACTIONS(1705),
    [sym_define_constant_keyword] = ACTIONS(1705),
    [sym_define_type_keyword] = ACTIONS(1705),
    [sym_define_variable_keyword] = ACTIONS(1705),
    [sym_define_system_variable_keyword] = ACTIONS(1705),
    [sym_define_connect_level_keyword] = ACTIONS(1705),
    [sym_define_latching_keyword] = ACTIONS(1705),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1705),
    [sym_define_toggling_keyword] = ACTIONS(1705),
    [sym_define_call_keyword] = ACTIONS(1705),
    [sym_define_function_keyword] = ACTIONS(1705),
    [sym_define_library_function_keyword] = ACTIONS(1705),
    [sym_define_start_keyword] = ACTIONS(1705),
    [sym_define_event_keyword] = ACTIONS(1705),
    [sym_define_program_keyword] = ACTIONS(1705),
    [sym_define_module_keyword] = ACTIONS(1705),
    [sym_char_keyword] = ACTIONS(1705),
    [sym_widechar_keyword] = ACTIONS(1705),
    [sym_integer_keyword] = ACTIONS(1705),
    [sym_sinteger_keyword] = ACTIONS(1705),
    [sym_long_keyword] = ACTIONS(1705),
    [sym_slong_keyword] = ACTIONS(1705),
    [sym_float_keyword] = ACTIONS(1705),
    [sym_double_keyword] = ACTIONS(1705),
    [sym_variant_keyword] = ACTIONS(1705),
    [sym_variantarray_keyword] = ACTIONS(1705),
    [sym_local_var_keyword] = ACTIONS(1705),
    [sym_stack_var_keyword] = ACTIONS(1705),
    [sym_constant_keyword] = ACTIONS(1705),
    [sym_volatile_keyword] = ACTIONS(1705),
    [sym_non_volatile_keyword] = ACTIONS(1705),
    [sym_persistent_keyword] = ACTIONS(1705),
    [sym_dev_keyword] = ACTIONS(1705),
    [sym_devlev_keyword] = ACTIONS(1705),
    [sym_devchan_keyword] = ACTIONS(1705),
    [sym_if_keyword] = ACTIONS(1705),
    [sym_switch_keyword] = ACTIONS(1705),
    [sym_case_keyword] = ACTIONS(1705),
    [sym_default_keyword] = ACTIONS(1705),
    [sym_while_keyword] = ACTIONS(1705),
    [sym_for_keyword] = ACTIONS(1705),
    [sym_break_keyword] = ACTIONS(1705),
    [sym_continue_keyword] = ACTIONS(1705),
    [sym_return_keyword] = ACTIONS(1705),
    [sym_select_keyword] = ACTIONS(1705),
    [sym_active_keyword] = ACTIONS(1705),
    [sym_struct_keyword] = ACTIONS(1705),
    [sym_structure_keyword] = ACTIONS(1705),
    [sym_button_event_keyword] = ACTIONS(1705),
    [sym_channel_event_keyword] = ACTIONS(1705),
    [sym_level_event_keyword] = ACTIONS(1705),
    [sym_data_event_keyword] = ACTIONS(1705),
    [sym_timeline_event_keyword] = ACTIONS(1705),
    [sym_custom_event_keyword] = ACTIONS(1705),
    [sym_push_keyword] = ACTIONS(1705),
    [sym_release_keyword] = ACTIONS(1705),
    [aux_sym_on_keyword_token1] = ACTIONS(1705),
    [aux_sym_off_keyword_token1] = ACTIONS(1705),
    [sym_send_level_keyword] = ACTIONS(1705),
    [sym_send_string_keyword] = ACTIONS(1705),
    [sym_send_command_keyword] = ACTIONS(1705),
    [sym_clear_buffer_keyword] = ACTIONS(1705),
    [sym_create_buffer_keyword] = ACTIONS(1705),
    [sym_create_multi_buffer_keyword] = ACTIONS(1705),
    [sym_call_keyword] = ACTIONS(1705),
    [sym_system_call_keyword] = ACTIONS(1705),
    [sym_devchan_to_keyword] = ACTIONS(1705),
    [sym_devchan_min_to_keyword] = ACTIONS(1705),
    [sym_devchan_total_off_keyword] = ACTIONS(1705),
    [sym_devchan_pulse_keyword] = ACTIONS(1705),
    [sym_wait_keyword] = ACTIONS(1705),
    [sym_cancel_wait_keyword] = ACTIONS(1705),
    [sym_wait_until_keyword] = ACTIONS(1705),
    [sym_cancel_wait_until_keyword] = ACTIONS(1705),
    [sym_cancel_all_wait_keyword] = ACTIONS(1705),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1705),
    [sym_not] = ACTIONS(1705),
    [sym_bnot] = ACTIONS(1705),
    [sym___file__] = ACTIONS(1705),
    [sym___date__] = ACTIONS(1705),
    [sym___ldate__] = ACTIONS(1705),
    [sym___time__] = ACTIONS(1705),
    [sym___line__] = ACTIONS(1705),
    [sym___name__] = ACTIONS(1705),
    [sym___netlinx__] = ACTIONS(1705),
    [sym_netlinx_axi_version] = ACTIONS(1705),
    [sym_url_flg_tcp] = ACTIONS(1705),
    [sym_url_flg_accinfopresent] = ACTIONS(1705),
    [sym_url_flg_temp] = ACTIONS(1705),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1705),
    [sym_url_flg_stat_mask] = ACTIONS(1705),
    [sym_url_flg_stat_lookup] = ACTIONS(1705),
    [sym_url_flg_stat_connecting] = ACTIONS(1705),
    [sym_url_flg_stat_waiting] = ACTIONS(1705),
    [sym_url_flg_stat_connected] = ACTIONS(1705),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1705),
    [sym_first_virtual_device] = ACTIONS(1705),
    [sym_first_local_port] = ACTIONS(1705),
    [sym_dynamic_virtual_device] = ACTIONS(1705),
    [sym_timeline_once] = ACTIONS(1705),
    [sym_timeline_repeat] = ACTIONS(1705),
    [sym_timeline_absolute] = ACTIONS(1705),
    [sym_timeline_relative] = ACTIONS(1705),
    [sym_do_push_timed_infinite] = ACTIONS(1705),
    [sym_source_type_no_address] = ACTIONS(1705),
    [sym_source_type_neuron_id] = ACTIONS(1705),
    [sym_source_type_ip_address] = ACTIONS(1705),
    [sym_source_type_axlink] = ACTIONS(1705),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1705),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1705),
    [sym_source_type_ip_socket_address] = ACTIONS(1705),
    [sym_source_type_rs232] = ACTIONS(1705),
    [sym_source_type_internal] = ACTIONS(1705),
    [sym_source_type_mac_address] = ACTIONS(1705),
    [sym_source_type_ipv4_port] = ACTIONS(1705),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1705),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1705),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1705),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1705),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1705),
    [sym_true] = ACTIONS(1705),
    [sym_false] = ACTIONS(1705),
    [sym_file_read_only] = ACTIONS(1705),
    [sym_file_rw_new] = ACTIONS(1705),
    [sym_file_rw_append] = ACTIONS(1705),
    [sym_ip_tcp] = ACTIONS(1705),
    [sym_ip_udp] = ACTIONS(1705),
    [sym_ip_udp_2way] = ACTIONS(1705),
    [sym_xml_encode_types] = ACTIONS(1705),
    [sym_xml_encode_char_as_list] = ACTIONS(1705),
    [sym_xml_encode_le] = ACTIONS(1705),
    [sym_xml_decode_types] = ACTIONS(1705),
    [sym_xml_decode_no_preserve] = ACTIONS(1705),
    [sym_duet_dev_type_display_device] = ACTIONS(1705),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1705),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1705),
    [sym_duet_dev_type_io_device] = ACTIONS(1705),
    [sym_duet_dev_type_relay_device] = ACTIONS(1705),
    [sym_duet_dev_type_ups] = ACTIONS(1705),
    [sym_duet_dev_type_amplifier] = ACTIONS(1705),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1705),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1705),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1705),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1705),
    [sym_duet_dev_type_camera] = ACTIONS(1705),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1705),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1705),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1705),
    [sym_duet_dev_type_dss] = ACTIONS(1705),
    [sym_duet_dev_type_dvr] = ACTIONS(1705),
    [sym_duet_dev_type_disc_device] = ACTIONS(1705),
    [sym_duet_dev_type_document_camera] = ACTIONS(1705),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1705),
    [sym_duet_dev_type_hvac] = ACTIONS(1705),
    [sym_duet_dev_type_keypad] = ACTIONS(1705),
    [sym_duet_dev_type_light] = ACTIONS(1705),
    [sym_duet_dev_type_monitor] = ACTIONS(1705),
    [sym_duet_dev_type_motor] = ACTIONS(1705),
    [sym_duet_dev_type_multi_window] = ACTIONS(1705),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1705),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1705),
    [sym_duet_dev_type_receiver] = ACTIONS(1705),
    [sym_duet_dev_type_security_system] = ACTIONS(1705),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1705),
    [sym_duet_dev_type_settop_box] = ACTIONS(1705),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1705),
    [sym_duet_dev_type_switcher] = ACTIONS(1705),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1705),
    [sym_duet_dev_type_tv] = ACTIONS(1705),
    [sym_duet_dev_type_utility] = ACTIONS(1705),
    [sym_duet_dev_type_vcr] = ACTIONS(1705),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1705),
    [sym_duet_dev_type_video_processor] = ACTIONS(1705),
    [sym_duet_dev_type_video_projector] = ACTIONS(1705),
    [sym_duet_dev_type_video_wall] = ACTIONS(1705),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1705),
    [sym_duet_dev_type_weather] = ACTIONS(1705),
    [sym_duet_dev_not_polled] = ACTIONS(1705),
    [sym_duet_dev_polled] = ACTIONS(1705),
    [sym_clkmgr_mode_network] = ACTIONS(1705),
    [sym_clkmgr_mode_standalone] = ACTIONS(1705),
    [sym_normal_standby] = ACTIONS(1705),
    [sym_normal_wake] = ACTIONS(1705),
    [sym_ip_multicast_ttl_option] = ACTIONS(1705),
    [sym_ip_ttl_subnet] = ACTIONS(1705),
    [sym_ip_ttl_site] = ACTIONS(1705),
    [sym_ip_ttl_region] = ACTIONS(1705),
    [sym_ip_ttl_continent] = ACTIONS(1705),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1705),
    [sym_ip_nodelay_on] = ACTIONS(1705),
    [sym_ip_nodelay_off] = ACTIONS(1705),
    [sym_amx_error] = ACTIONS(1705),
    [sym_amx_warning] = ACTIONS(1705),
    [sym_amx_info] = ACTIONS(1705),
    [sym_amx_debug] = ACTIONS(1705),
    [sym_netlinx_login_success] = ACTIONS(1705),
    [sym_netlinx_login_fail] = ACTIONS(1705),
    [sym_netlinx_logout] = ACTIONS(1705),
    [sym_null_str] = ACTIONS(1705),
    [sym_smtp_address] = ACTIONS(1705),
    [sym_smtp_port_number] = ACTIONS(1705),
    [sym_smtp_username] = ACTIONS(1705),
    [sym_smtp_password] = ACTIONS(1705),
    [sym_smtp_require_tls] = ACTIONS(1705),
    [sym_smtp_from] = ACTIONS(1705),
    [sym_smtp_tls_true] = ACTIONS(1705),
    [sym_smtp_tls_false] = ACTIONS(1705),
    [sym_listview_on_row_select_event] = ACTIONS(1705),
    [sym_data_structure_unknown] = ACTIONS(1705),
    [sym_data_structure_datafeed] = ACTIONS(1705),
    [sym_data_structure_datarecordset] = ACTIONS(1705),
    [sym_data_structure_datarecord] = ACTIONS(1705),
    [sym_data_structure_datafield] = ACTIONS(1705),
    [sym_data_max_content_field_count] = ACTIONS(1705),
    [sym_data_max_metadata_field_count] = ACTIONS(1705),
    [sym_data_max_id_length] = ACTIONS(1705),
    [sym_data_max_name_length] = ACTIONS(1705),
    [sym_data_max_description_length] = ACTIONS(1705),
    [sym_data_max_type_length] = ACTIONS(1705),
    [sym_data_max_format_length] = ACTIONS(1705),
    [sym_data_max_source_length] = ACTIONS(1705),
    [sym_data_max_value_length] = ACTIONS(1705),
    [sym_data_max_label_length] = ACTIONS(1705),
    [sym_data_max_url_length] = ACTIONS(1705),
    [sym_max_last_login_info_length] = ACTIONS(1705),
    [sym_data_type_unknown] = ACTIONS(1705),
    [sym_data_type_string] = ACTIONS(1705),
    [sym_data_type_datetime] = ACTIONS(1705),
    [sym_data_type_date] = ACTIONS(1705),
    [sym_data_type_time] = ACTIONS(1705),
    [sym_data_type_image] = ACTIONS(1705),
    [sym_data_format_url] = ACTIONS(1705),
    [sym_data_format_phone] = ACTIONS(1705),
    [sym_data_format_email] = ACTIONS(1705),
    [sym_data_format_iso8601] = ACTIONS(1705),
    [sym_auditlog_priv] = ACTIONS(1705),
    [sym_configuration_priv] = ACTIONS(1705),
    [sym_deviceconfig_priv] = ACTIONS(1705),
    [sym_ftp_priv] = ACTIONS(1705),
    [sym_http_priv] = ACTIONS(1705),
    [sym_networkconfig_priv] = ACTIONS(1705),
    [sym_programport_priv] = ACTIONS(1705),
    [sym_remoteui_priv] = ACTIONS(1705),
    [sym_securitycontrol_priv] = ACTIONS(1705),
    [sym_softwaremanagement_priv] = ACTIONS(1705),
    [sym_terminal_priv] = ACTIONS(1705),
    [sym_tpadmin_priv] = ACTIONS(1705),
    [sym_user1_priv] = ACTIONS(1705),
    [sym_user2_priv] = ACTIONS(1705),
    [sym_user3_priv] = ACTIONS(1705),
    [sym_user4_priv] = ACTIONS(1705),
    [sym_usermanagement_priv] = ACTIONS(1705),
    [sym_valid_account] = ACTIONS(1705),
    [sym_err_name_invalid] = ACTIONS(1705),
    [sym_err_password_invalid] = ACTIONS(1705),
    [sym_err_invalid_account] = ACTIONS(1705),
    [sym_err_no_authorization] = ACTIONS(1705),
    [sym_err_invalid_parameter] = ACTIONS(1705),
    [sym_err_user_locked_out] = ACTIONS(1705),
    [sym_err_user_account_expired] = ACTIONS(1705),
    [sym_err_general_error] = ACTIONS(1705),
    [sym_err_authentication_error] = ACTIONS(1705),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1705),
    [sym_tls_validate_certificate] = ACTIONS(1705),
    [sym_tdata] = ACTIONS(1705),
    [sym_tchannel] = ACTIONS(1705),
    [sym_tlevel] = ACTIONS(1705),
    [sym_tbutton] = ACTIONS(1705),
    [sym_ttimeline] = ACTIONS(1705),
    [sym_tcustom] = ACTIONS(1705),
    [sym_url_struct] = ACTIONS(1705),
    [sym_dns_struct] = ACTIONS(1705),
    [sym_ip_address_struct] = ACTIONS(1705),
    [sym_dev_info_struct] = ACTIONS(1705),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1705),
    [sym_data_feed] = ACTIONS(1705),
    [sym_data_field] = ACTIONS(1705),
    [sym_data_record] = ACTIONS(1705),
    [sym_wc_data_feed] = ACTIONS(1705),
    [sym_wc_data_field] = ACTIONS(1705),
    [sym_wc_data_record] = ACTIONS(1705),
    [sym_last_login_info] = ACTIONS(1705),
    [sym____reserved___] = ACTIONS(1705),
    [sym_date] = ACTIONS(1705),
    [sym_day] = ACTIONS(1705),
    [sym_time] = ACTIONS(1705),
    [sym_ldate] = ACTIONS(1705),
    [sym_push_device] = ACTIONS(1705),
    [sym_push_channel] = ACTIONS(1705),
    [sym_push_devchan] = ACTIONS(1705),
    [sym_release_device] = ACTIONS(1705),
    [sym_release_channel] = ACTIONS(1705),
    [sym_release_devchan] = ACTIONS(1705),
    [sym_master_slot] = ACTIONS(1705),
    [sym_get_pulse_time] = ACTIONS(1705),
    [sym_get_timer] = ACTIONS(1705),
    [sym_master_sn] = ACTIONS(1705),
    [sym_system_number] = ACTIONS(1705),
    [sym_button] = ACTIONS(1705),
    [sym_dv_channel] = ACTIONS(1705),
    [sym_data] = ACTIONS(1705),
    [sym_level] = ACTIONS(1705),
    [sym_timeline] = ACTIONS(1705),
    [sym_channel] = ACTIONS(1705),
    [sym_custom] = ACTIONS(1705),
    [sym_atoi] = ACTIONS(1705),
    [sym_atol] = ACTIONS(1705),
    [sym_atof] = ACTIONS(1705),
    [sym_itoa] = ACTIONS(1705),
    [sym_itohex] = ACTIONS(1705),
    [sym_hextoi] = ACTIONS(1705),
    [sym_ftoa] = ACTIONS(1705),
    [sym_find_string] = ACTIONS(1705),
    [sym_left_string] = ACTIONS(1705),
    [sym_right_string] = ACTIONS(1705),
    [sym_mid_string] = ACTIONS(1705),
    [sym_remove_string] = ACTIONS(1705),
    [sym_get_buffer_string] = ACTIONS(1705),
    [sym_get_buffer_char] = ACTIONS(1705),
    [sym_get_multi_buffer_string] = ACTIONS(1705),
    [sym_length_string] = ACTIONS(1705),
    [sym_max_length_string] = ACTIONS(1705),
    [sym_set_length_string] = ACTIONS(1705),
    [sym_compare_string] = ACTIONS(1705),
    [sym_upper_string] = ACTIONS(1705),
    [sym_lower_string] = ACTIONS(1705),
    [sym_get_last] = ACTIONS(1705),
    [sym_date_to_day] = ACTIONS(1705),
    [sym_date_to_month] = ACTIONS(1705),
    [sym_date_to_year] = ACTIONS(1705),
    [sym_time_to_hour] = ACTIONS(1705),
    [sym_time_to_minute] = ACTIONS(1705),
    [sym_time_to_second] = ACTIONS(1705),
    [sym_day_of_week] = ACTIONS(1705),
    [sym_set_pulse_time] = ACTIONS(1705),
    [sym_set_timer] = ACTIONS(1705),
    [sym_file_open] = ACTIONS(1705),
    [sym_file_close] = ACTIONS(1705),
    [sym_file_read] = ACTIONS(1705),
    [sym_file_read_line] = ACTIONS(1705),
    [sym_file_write] = ACTIONS(1705),
    [sym_file_write_line] = ACTIONS(1705),
    [sym_file_copy] = ACTIONS(1705),
    [sym_file_delete] = ACTIONS(1705),
    [sym_file_rename] = ACTIONS(1705),
    [sym_file_dir] = ACTIONS(1705),
    [sym_file_getdir] = ACTIONS(1705),
    [sym_file_setdir] = ACTIONS(1705),
    [sym_file_create_dir] = ACTIONS(1705),
    [sym_file_remove_dir] = ACTIONS(1705),
    [sym_file_seek] = ACTIONS(1705),
    [sym_length_array] = ACTIONS(1705),
    [sym_max_length_array] = ACTIONS(1705),
    [sym_set_length_array] = ACTIONS(1705),
    [sym_ip_client_open] = ACTIONS(1705),
    [sym_ip_client_close] = ACTIONS(1705),
    [sym_ip_server_open] = ACTIONS(1705),
    [sym_ip_server_close] = ACTIONS(1705),
    [sym_ip_bound_client_open] = ACTIONS(1705),
    [sym_ip_mc_server_open] = ACTIONS(1705),
    [sym_ip_set_option] = ACTIONS(1705),
    [sym_ssh_client_open] = ACTIONS(1705),
    [sym_ssh_client_close] = ACTIONS(1705),
    [sym_tls_client_open] = ACTIONS(1705),
    [sym_tls_client_close] = ACTIONS(1705),
    [sym_do_push] = ACTIONS(1705),
    [sym_do_push_timed] = ACTIONS(1705),
    [sym_do_release] = ACTIONS(1705),
    [sym_device_id] = ACTIONS(1705),
    [sym_device_info] = ACTIONS(1705),
    [sym_device_id_string] = ACTIONS(1705),
    [sym_device_standby] = ACTIONS(1705),
    [sym_device_wake] = ACTIONS(1705),
    [sym_timeline_create] = ACTIONS(1705),
    [sym_timeline_kill] = ACTIONS(1705),
    [sym_timeline_reload] = ACTIONS(1705),
    [sym_timeline_pause] = ACTIONS(1705),
    [sym_timeline_restart] = ACTIONS(1705),
    [sym_timeline_set] = ACTIONS(1705),
    [sym_timeline_get] = ACTIONS(1705),
    [sym_timeline_active] = ACTIONS(1705),
    [sym_timeline_dynamic_id] = ACTIONS(1705),
    [sym_variable_to_string] = ACTIONS(1705),
    [sym_string_to_variable] = ACTIONS(1705),
    [sym_length_variable_to_string] = ACTIONS(1705),
    [sym_type_cast] = ACTIONS(1705),
    [sym_variable_to_xml] = ACTIONS(1705),
    [sym_xml_to_variable] = ACTIONS(1705),
    [sym_length_variable_to_xml] = ACTIONS(1705),
    [sym_redirect_string] = ACTIONS(1705),
    [sym_random_number] = ACTIONS(1705),
    [sym_get_unique_id] = ACTIONS(1705),
    [sym_get_system_number] = ACTIONS(1705),
    [sym_set_system_number] = ACTIONS(1705),
    [sym_get_serial_number] = ACTIONS(1705),
    [sym_get_master_build] = ACTIONS(1705),
    [sym_set_validation_code] = ACTIONS(1705),
    [sym_reboot] = ACTIONS(1705),
    [sym_load_duet_module] = ACTIONS(1705),
    [sym_unload_duet_module] = ACTIONS(1705),
    [sym_duet_mem_size_set] = ACTIONS(1705),
    [sym_duet_mem_size_get] = ACTIONS(1705),
    [sym_dynamic_application_device] = ACTIONS(1705),
    [sym_dynamic_polled_port] = ACTIONS(1705),
    [sym_static_port_binding] = ACTIONS(1705),
    [sym_static_ip_binding] = ACTIONS(1705),
    [sym_get_dns_list] = ACTIONS(1705),
    [sym_set_dns_list] = ACTIONS(1705),
    [sym_get_ip_address] = ACTIONS(1705),
    [sym_set_ip_address] = ACTIONS(1705),
    [sym_get_url_list] = ACTIONS(1705),
    [sym_add_url_entry] = ACTIONS(1705),
    [sym_delete_url_entry] = ACTIONS(1705),
    [sym_abs_value] = ACTIONS(1705),
    [sym_max_value] = ACTIONS(1705),
    [sym_min_value] = ACTIONS(1705),
    [sym_exp_value] = ACTIONS(1705),
    [sym_log_value] = ACTIONS(1705),
    [sym_log10_value] = ACTIONS(1705),
    [sym_power_value] = ACTIONS(1705),
    [sym_sqrt_value] = ACTIONS(1705),
    [sym_raw_be] = ACTIONS(1705),
    [sym_raw_le] = ACTIONS(1705),
    [sym_internal_queue_size_set] = ACTIONS(1705),
    [sym_internal_queue_size_get] = ACTIONS(1705),
    [sym_internal_threshold_set] = ACTIONS(1705),
    [sym_internal_threshold_get] = ACTIONS(1705),
    [sym_rebuilt_event] = ACTIONS(1705),
    [sym_do_custom_event] = ACTIONS(1705),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1705),
    [sym_clkmgr_set_clk_source] = ACTIONS(1705),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1705),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1705),
    [sym_clkmgr_get_timezone] = ACTIONS(1705),
    [sym_clkmgr_set_timezone] = ACTIONS(1705),
    [sym_clkmgr_get_resync_period] = ACTIONS(1705),
    [sym_clkmgr_set_resync_period] = ACTIONS(1705),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1705),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1705),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1705),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1705),
    [sym_clkmgr_get_timeservers] = ACTIONS(1705),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1705),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1705),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1705),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1705),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1705),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1705),
    [sym_set_log_level] = ACTIONS(1705),
    [sym_get_log_level] = ACTIONS(1705),
    [sym_amx_log] = ACTIONS(1705),
    [sym_smtp_server_config_set] = ACTIONS(1705),
    [sym_smtp_server_config_get] = ACTIONS(1705),
    [sym_smtp_send] = ACTIONS(1705),
    [sym_validate_netlinx_account] = ACTIONS(1705),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1705),
    [sym_authenticate_certificate] = ACTIONS(1705),
    [sym_audit_netlinx_session_event] = ACTIONS(1705),
    [sym_audit_netlinx_generic_event] = ACTIONS(1705),
    [sym_get_available_flash_disk_space] = ACTIONS(1705),
    [sym_get_max_flash_disk_space] = ACTIONS(1705),
    [sym_data_create_feed] = ACTIONS(1705),
    [sym_data_delete_feed] = ACTIONS(1705),
    [sym_data_publish_feed] = ACTIONS(1705),
    [sym_data_get_published_feed] = ACTIONS(1705),
    [sym_data_add_record] = ACTIONS(1705),
    [sym_data_get_event_record] = ACTIONS(1705),
    [sym__wc_data_create_feed] = ACTIONS(1705),
    [sym__wc_data_add_record] = ACTIONS(1705),
    [sym__wc_data_get_event_record] = ACTIONS(1705),
    [sym_format] = ACTIONS(1705),
    [sym_astro_clock] = ACTIONS(1705),
    [sym_set_outdoor_temperature] = ACTIONS(1705),
    [sym_set_virtual_level_count] = ACTIONS(1705),
    [sym_set_virtual_channel_count] = ACTIONS(1705),
    [sym_set_virtual_port_count] = ACTIONS(1705),
    [sym__wc_encode_int] = ACTIONS(1705),
    [sym__wc_decode_int] = ACTIONS(1705),
    [sym__wc_int] = ACTIONS(1705),
    [sym__wc_to_ch_int] = ACTIONS(1705),
    [sym__ch_to_wc_int] = ACTIONS(1705),
    [sym__wc_find_string_int] = ACTIONS(1705),
    [sym__wc_left_string_int] = ACTIONS(1705),
    [sym__wc_lower_string_int] = ACTIONS(1705),
    [sym__wc_mid_string_int] = ACTIONS(1705),
    [sym__wc_remove_string_int] = ACTIONS(1705),
    [sym__wc_right_string_int] = ACTIONS(1705),
    [sym__wc_upper_string_int] = ACTIONS(1705),
    [sym__wc_compare_string_int] = ACTIONS(1705),
    [sym__wc_get_buffer_char_int] = ACTIONS(1705),
    [sym__wc_get_buffer_string_int] = ACTIONS(1705),
    [sym__wc_concat_string_int] = ACTIONS(1705),
    [sym__wc_file_open_int] = ACTIONS(1705),
    [sym__wc_file_close_int] = ACTIONS(1705),
    [sym__wc_file_read_int] = ACTIONS(1705),
    [sym__wc_file_read_line_int] = ACTIONS(1705),
    [sym__wc_file_write_int] = ACTIONS(1705),
    [sym__wc_file_write_line_int] = ACTIONS(1705),
    [aux_sym_preproc_include_token1] = ACTIONS(1705),
    [aux_sym_preproc_define_token1] = ACTIONS(1703),
    [aux_sym_preproc_warn_token1] = ACTIONS(1703),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1703),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1703),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1703),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1703),
    [aux_sym_preproc_else_token1] = ACTIONS(1703),
    [anon_sym_LBRACK] = ACTIONS(1703),
    [anon_sym_LBRACE] = ACTIONS(1703),
    [anon_sym_RBRACE] = ACTIONS(1703),
    [anon_sym_LPAREN] = ACTIONS(1705),
    [anon_sym_BANG] = ACTIONS(1703),
    [anon_sym_TILDE] = ACTIONS(1703),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_DASH_DASH] = ACTIONS(1703),
    [anon_sym_PLUS_PLUS] = ACTIONS(1703),
    [anon_sym_DQUOTE] = ACTIONS(1703),
    [anon_sym_SQUOTE] = ACTIONS(1703),
    [sym_number_literal] = ACTIONS(1703),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(206)] = {
    [ts_builtin_sym_end] = ACTIONS(1707),
    [sym_identifier] = ACTIONS(1709),
    [sym_program_name_keyword] = ACTIONS(1709),
    [sym_module_name_keyword] = ACTIONS(1709),
    [sym_define_device_keyword] = ACTIONS(1709),
    [sym_define_combine_keyword] = ACTIONS(1709),
    [sym_define_constant_keyword] = ACTIONS(1709),
    [sym_define_type_keyword] = ACTIONS(1709),
    [sym_define_variable_keyword] = ACTIONS(1709),
    [sym_define_system_variable_keyword] = ACTIONS(1709),
    [sym_define_connect_level_keyword] = ACTIONS(1709),
    [sym_define_latching_keyword] = ACTIONS(1709),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1709),
    [sym_define_toggling_keyword] = ACTIONS(1709),
    [sym_define_call_keyword] = ACTIONS(1709),
    [sym_define_function_keyword] = ACTIONS(1709),
    [sym_define_library_function_keyword] = ACTIONS(1709),
    [sym_define_start_keyword] = ACTIONS(1709),
    [sym_define_event_keyword] = ACTIONS(1709),
    [sym_define_program_keyword] = ACTIONS(1709),
    [sym_define_module_keyword] = ACTIONS(1709),
    [sym_char_keyword] = ACTIONS(1709),
    [sym_widechar_keyword] = ACTIONS(1709),
    [sym_integer_keyword] = ACTIONS(1709),
    [sym_sinteger_keyword] = ACTIONS(1709),
    [sym_long_keyword] = ACTIONS(1709),
    [sym_slong_keyword] = ACTIONS(1709),
    [sym_float_keyword] = ACTIONS(1709),
    [sym_double_keyword] = ACTIONS(1709),
    [sym_variant_keyword] = ACTIONS(1709),
    [sym_variantarray_keyword] = ACTIONS(1709),
    [sym_local_var_keyword] = ACTIONS(1709),
    [sym_stack_var_keyword] = ACTIONS(1709),
    [sym_constant_keyword] = ACTIONS(1709),
    [sym_volatile_keyword] = ACTIONS(1709),
    [sym_non_volatile_keyword] = ACTIONS(1709),
    [sym_persistent_keyword] = ACTIONS(1709),
    [sym_dev_keyword] = ACTIONS(1709),
    [sym_devlev_keyword] = ACTIONS(1709),
    [sym_devchan_keyword] = ACTIONS(1709),
    [sym_if_keyword] = ACTIONS(1709),
    [sym_switch_keyword] = ACTIONS(1709),
    [sym_case_keyword] = ACTIONS(1709),
    [sym_default_keyword] = ACTIONS(1709),
    [sym_while_keyword] = ACTIONS(1709),
    [sym_for_keyword] = ACTIONS(1709),
    [sym_break_keyword] = ACTIONS(1709),
    [sym_continue_keyword] = ACTIONS(1709),
    [sym_return_keyword] = ACTIONS(1709),
    [sym_select_keyword] = ACTIONS(1709),
    [sym_active_keyword] = ACTIONS(1709),
    [sym_struct_keyword] = ACTIONS(1709),
    [sym_structure_keyword] = ACTIONS(1709),
    [sym_button_event_keyword] = ACTIONS(1709),
    [sym_channel_event_keyword] = ACTIONS(1709),
    [sym_level_event_keyword] = ACTIONS(1709),
    [sym_data_event_keyword] = ACTIONS(1709),
    [sym_timeline_event_keyword] = ACTIONS(1709),
    [sym_custom_event_keyword] = ACTIONS(1709),
    [sym_push_keyword] = ACTIONS(1709),
    [sym_release_keyword] = ACTIONS(1709),
    [aux_sym_on_keyword_token1] = ACTIONS(1709),
    [aux_sym_off_keyword_token1] = ACTIONS(1709),
    [sym_send_level_keyword] = ACTIONS(1709),
    [sym_send_string_keyword] = ACTIONS(1709),
    [sym_send_command_keyword] = ACTIONS(1709),
    [sym_clear_buffer_keyword] = ACTIONS(1709),
    [sym_create_buffer_keyword] = ACTIONS(1709),
    [sym_create_multi_buffer_keyword] = ACTIONS(1709),
    [sym_call_keyword] = ACTIONS(1709),
    [sym_system_call_keyword] = ACTIONS(1709),
    [sym_devchan_to_keyword] = ACTIONS(1709),
    [sym_devchan_min_to_keyword] = ACTIONS(1709),
    [sym_devchan_total_off_keyword] = ACTIONS(1709),
    [sym_devchan_pulse_keyword] = ACTIONS(1709),
    [sym_wait_keyword] = ACTIONS(1709),
    [sym_cancel_wait_keyword] = ACTIONS(1709),
    [sym_wait_until_keyword] = ACTIONS(1709),
    [sym_cancel_wait_until_keyword] = ACTIONS(1709),
    [sym_cancel_all_wait_keyword] = ACTIONS(1709),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1709),
    [sym_not] = ACTIONS(1709),
    [sym_bnot] = ACTIONS(1709),
    [sym___file__] = ACTIONS(1709),
    [sym___date__] = ACTIONS(1709),
    [sym___ldate__] = ACTIONS(1709),
    [sym___time__] = ACTIONS(1709),
    [sym___line__] = ACTIONS(1709),
    [sym___name__] = ACTIONS(1709),
    [sym___netlinx__] = ACTIONS(1709),
    [sym_netlinx_axi_version] = ACTIONS(1709),
    [sym_url_flg_tcp] = ACTIONS(1709),
    [sym_url_flg_accinfopresent] = ACTIONS(1709),
    [sym_url_flg_temp] = ACTIONS(1709),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1709),
    [sym_url_flg_stat_mask] = ACTIONS(1709),
    [sym_url_flg_stat_lookup] = ACTIONS(1709),
    [sym_url_flg_stat_connecting] = ACTIONS(1709),
    [sym_url_flg_stat_waiting] = ACTIONS(1709),
    [sym_url_flg_stat_connected] = ACTIONS(1709),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1709),
    [sym_first_virtual_device] = ACTIONS(1709),
    [sym_first_local_port] = ACTIONS(1709),
    [sym_dynamic_virtual_device] = ACTIONS(1709),
    [sym_timeline_once] = ACTIONS(1709),
    [sym_timeline_repeat] = ACTIONS(1709),
    [sym_timeline_absolute] = ACTIONS(1709),
    [sym_timeline_relative] = ACTIONS(1709),
    [sym_do_push_timed_infinite] = ACTIONS(1709),
    [sym_source_type_no_address] = ACTIONS(1709),
    [sym_source_type_neuron_id] = ACTIONS(1709),
    [sym_source_type_ip_address] = ACTIONS(1709),
    [sym_source_type_axlink] = ACTIONS(1709),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1709),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1709),
    [sym_source_type_ip_socket_address] = ACTIONS(1709),
    [sym_source_type_rs232] = ACTIONS(1709),
    [sym_source_type_internal] = ACTIONS(1709),
    [sym_source_type_mac_address] = ACTIONS(1709),
    [sym_source_type_ipv4_port] = ACTIONS(1709),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1709),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1709),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1709),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1709),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1709),
    [sym_true] = ACTIONS(1709),
    [sym_false] = ACTIONS(1709),
    [sym_file_read_only] = ACTIONS(1709),
    [sym_file_rw_new] = ACTIONS(1709),
    [sym_file_rw_append] = ACTIONS(1709),
    [sym_ip_tcp] = ACTIONS(1709),
    [sym_ip_udp] = ACTIONS(1709),
    [sym_ip_udp_2way] = ACTIONS(1709),
    [sym_xml_encode_types] = ACTIONS(1709),
    [sym_xml_encode_char_as_list] = ACTIONS(1709),
    [sym_xml_encode_le] = ACTIONS(1709),
    [sym_xml_decode_types] = ACTIONS(1709),
    [sym_xml_decode_no_preserve] = ACTIONS(1709),
    [sym_duet_dev_type_display_device] = ACTIONS(1709),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1709),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1709),
    [sym_duet_dev_type_io_device] = ACTIONS(1709),
    [sym_duet_dev_type_relay_device] = ACTIONS(1709),
    [sym_duet_dev_type_ups] = ACTIONS(1709),
    [sym_duet_dev_type_amplifier] = ACTIONS(1709),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1709),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1709),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1709),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1709),
    [sym_duet_dev_type_camera] = ACTIONS(1709),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1709),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1709),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1709),
    [sym_duet_dev_type_dss] = ACTIONS(1709),
    [sym_duet_dev_type_dvr] = ACTIONS(1709),
    [sym_duet_dev_type_disc_device] = ACTIONS(1709),
    [sym_duet_dev_type_document_camera] = ACTIONS(1709),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1709),
    [sym_duet_dev_type_hvac] = ACTIONS(1709),
    [sym_duet_dev_type_keypad] = ACTIONS(1709),
    [sym_duet_dev_type_light] = ACTIONS(1709),
    [sym_duet_dev_type_monitor] = ACTIONS(1709),
    [sym_duet_dev_type_motor] = ACTIONS(1709),
    [sym_duet_dev_type_multi_window] = ACTIONS(1709),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1709),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1709),
    [sym_duet_dev_type_receiver] = ACTIONS(1709),
    [sym_duet_dev_type_security_system] = ACTIONS(1709),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1709),
    [sym_duet_dev_type_settop_box] = ACTIONS(1709),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1709),
    [sym_duet_dev_type_switcher] = ACTIONS(1709),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1709),
    [sym_duet_dev_type_tv] = ACTIONS(1709),
    [sym_duet_dev_type_utility] = ACTIONS(1709),
    [sym_duet_dev_type_vcr] = ACTIONS(1709),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1709),
    [sym_duet_dev_type_video_processor] = ACTIONS(1709),
    [sym_duet_dev_type_video_projector] = ACTIONS(1709),
    [sym_duet_dev_type_video_wall] = ACTIONS(1709),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1709),
    [sym_duet_dev_type_weather] = ACTIONS(1709),
    [sym_duet_dev_not_polled] = ACTIONS(1709),
    [sym_duet_dev_polled] = ACTIONS(1709),
    [sym_clkmgr_mode_network] = ACTIONS(1709),
    [sym_clkmgr_mode_standalone] = ACTIONS(1709),
    [sym_normal_standby] = ACTIONS(1709),
    [sym_normal_wake] = ACTIONS(1709),
    [sym_ip_multicast_ttl_option] = ACTIONS(1709),
    [sym_ip_ttl_subnet] = ACTIONS(1709),
    [sym_ip_ttl_site] = ACTIONS(1709),
    [sym_ip_ttl_region] = ACTIONS(1709),
    [sym_ip_ttl_continent] = ACTIONS(1709),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1709),
    [sym_ip_nodelay_on] = ACTIONS(1709),
    [sym_ip_nodelay_off] = ACTIONS(1709),
    [sym_amx_error] = ACTIONS(1709),
    [sym_amx_warning] = ACTIONS(1709),
    [sym_amx_info] = ACTIONS(1709),
    [sym_amx_debug] = ACTIONS(1709),
    [sym_netlinx_login_success] = ACTIONS(1709),
    [sym_netlinx_login_fail] = ACTIONS(1709),
    [sym_netlinx_logout] = ACTIONS(1709),
    [sym_null_str] = ACTIONS(1709),
    [sym_smtp_address] = ACTIONS(1709),
    [sym_smtp_port_number] = ACTIONS(1709),
    [sym_smtp_username] = ACTIONS(1709),
    [sym_smtp_password] = ACTIONS(1709),
    [sym_smtp_require_tls] = ACTIONS(1709),
    [sym_smtp_from] = ACTIONS(1709),
    [sym_smtp_tls_true] = ACTIONS(1709),
    [sym_smtp_tls_false] = ACTIONS(1709),
    [sym_listview_on_row_select_event] = ACTIONS(1709),
    [sym_data_structure_unknown] = ACTIONS(1709),
    [sym_data_structure_datafeed] = ACTIONS(1709),
    [sym_data_structure_datarecordset] = ACTIONS(1709),
    [sym_data_structure_datarecord] = ACTIONS(1709),
    [sym_data_structure_datafield] = ACTIONS(1709),
    [sym_data_max_content_field_count] = ACTIONS(1709),
    [sym_data_max_metadata_field_count] = ACTIONS(1709),
    [sym_data_max_id_length] = ACTIONS(1709),
    [sym_data_max_name_length] = ACTIONS(1709),
    [sym_data_max_description_length] = ACTIONS(1709),
    [sym_data_max_type_length] = ACTIONS(1709),
    [sym_data_max_format_length] = ACTIONS(1709),
    [sym_data_max_source_length] = ACTIONS(1709),
    [sym_data_max_value_length] = ACTIONS(1709),
    [sym_data_max_label_length] = ACTIONS(1709),
    [sym_data_max_url_length] = ACTIONS(1709),
    [sym_max_last_login_info_length] = ACTIONS(1709),
    [sym_data_type_unknown] = ACTIONS(1709),
    [sym_data_type_string] = ACTIONS(1709),
    [sym_data_type_datetime] = ACTIONS(1709),
    [sym_data_type_date] = ACTIONS(1709),
    [sym_data_type_time] = ACTIONS(1709),
    [sym_data_type_image] = ACTIONS(1709),
    [sym_data_format_url] = ACTIONS(1709),
    [sym_data_format_phone] = ACTIONS(1709),
    [sym_data_format_email] = ACTIONS(1709),
    [sym_data_format_iso8601] = ACTIONS(1709),
    [sym_auditlog_priv] = ACTIONS(1709),
    [sym_configuration_priv] = ACTIONS(1709),
    [sym_deviceconfig_priv] = ACTIONS(1709),
    [sym_ftp_priv] = ACTIONS(1709),
    [sym_http_priv] = ACTIONS(1709),
    [sym_networkconfig_priv] = ACTIONS(1709),
    [sym_programport_priv] = ACTIONS(1709),
    [sym_remoteui_priv] = ACTIONS(1709),
    [sym_securitycontrol_priv] = ACTIONS(1709),
    [sym_softwaremanagement_priv] = ACTIONS(1709),
    [sym_terminal_priv] = ACTIONS(1709),
    [sym_tpadmin_priv] = ACTIONS(1709),
    [sym_user1_priv] = ACTIONS(1709),
    [sym_user2_priv] = ACTIONS(1709),
    [sym_user3_priv] = ACTIONS(1709),
    [sym_user4_priv] = ACTIONS(1709),
    [sym_usermanagement_priv] = ACTIONS(1709),
    [sym_valid_account] = ACTIONS(1709),
    [sym_err_name_invalid] = ACTIONS(1709),
    [sym_err_password_invalid] = ACTIONS(1709),
    [sym_err_invalid_account] = ACTIONS(1709),
    [sym_err_no_authorization] = ACTIONS(1709),
    [sym_err_invalid_parameter] = ACTIONS(1709),
    [sym_err_user_locked_out] = ACTIONS(1709),
    [sym_err_user_account_expired] = ACTIONS(1709),
    [sym_err_general_error] = ACTIONS(1709),
    [sym_err_authentication_error] = ACTIONS(1709),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1709),
    [sym_tls_validate_certificate] = ACTIONS(1709),
    [sym_tdata] = ACTIONS(1709),
    [sym_tchannel] = ACTIONS(1709),
    [sym_tlevel] = ACTIONS(1709),
    [sym_tbutton] = ACTIONS(1709),
    [sym_ttimeline] = ACTIONS(1709),
    [sym_tcustom] = ACTIONS(1709),
    [sym_url_struct] = ACTIONS(1709),
    [sym_dns_struct] = ACTIONS(1709),
    [sym_ip_address_struct] = ACTIONS(1709),
    [sym_dev_info_struct] = ACTIONS(1709),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1709),
    [sym_data_feed] = ACTIONS(1709),
    [sym_data_field] = ACTIONS(1709),
    [sym_data_record] = ACTIONS(1709),
    [sym_wc_data_feed] = ACTIONS(1709),
    [sym_wc_data_field] = ACTIONS(1709),
    [sym_wc_data_record] = ACTIONS(1709),
    [sym_last_login_info] = ACTIONS(1709),
    [sym____reserved___] = ACTIONS(1709),
    [sym_date] = ACTIONS(1709),
    [sym_day] = ACTIONS(1709),
    [sym_time] = ACTIONS(1709),
    [sym_ldate] = ACTIONS(1709),
    [sym_push_device] = ACTIONS(1709),
    [sym_push_channel] = ACTIONS(1709),
    [sym_push_devchan] = ACTIONS(1709),
    [sym_release_device] = ACTIONS(1709),
    [sym_release_channel] = ACTIONS(1709),
    [sym_release_devchan] = ACTIONS(1709),
    [sym_master_slot] = ACTIONS(1709),
    [sym_get_pulse_time] = ACTIONS(1709),
    [sym_get_timer] = ACTIONS(1709),
    [sym_master_sn] = ACTIONS(1709),
    [sym_system_number] = ACTIONS(1709),
    [sym_button] = ACTIONS(1709),
    [sym_dv_channel] = ACTIONS(1709),
    [sym_data] = ACTIONS(1709),
    [sym_level] = ACTIONS(1709),
    [sym_timeline] = ACTIONS(1709),
    [sym_channel] = ACTIONS(1709),
    [sym_custom] = ACTIONS(1709),
    [sym_atoi] = ACTIONS(1709),
    [sym_atol] = ACTIONS(1709),
    [sym_atof] = ACTIONS(1709),
    [sym_itoa] = ACTIONS(1709),
    [sym_itohex] = ACTIONS(1709),
    [sym_hextoi] = ACTIONS(1709),
    [sym_ftoa] = ACTIONS(1709),
    [sym_find_string] = ACTIONS(1709),
    [sym_left_string] = ACTIONS(1709),
    [sym_right_string] = ACTIONS(1709),
    [sym_mid_string] = ACTIONS(1709),
    [sym_remove_string] = ACTIONS(1709),
    [sym_get_buffer_string] = ACTIONS(1709),
    [sym_get_buffer_char] = ACTIONS(1709),
    [sym_get_multi_buffer_string] = ACTIONS(1709),
    [sym_length_string] = ACTIONS(1709),
    [sym_max_length_string] = ACTIONS(1709),
    [sym_set_length_string] = ACTIONS(1709),
    [sym_compare_string] = ACTIONS(1709),
    [sym_upper_string] = ACTIONS(1709),
    [sym_lower_string] = ACTIONS(1709),
    [sym_get_last] = ACTIONS(1709),
    [sym_date_to_day] = ACTIONS(1709),
    [sym_date_to_month] = ACTIONS(1709),
    [sym_date_to_year] = ACTIONS(1709),
    [sym_time_to_hour] = ACTIONS(1709),
    [sym_time_to_minute] = ACTIONS(1709),
    [sym_time_to_second] = ACTIONS(1709),
    [sym_day_of_week] = ACTIONS(1709),
    [sym_set_pulse_time] = ACTIONS(1709),
    [sym_set_timer] = ACTIONS(1709),
    [sym_file_open] = ACTIONS(1709),
    [sym_file_close] = ACTIONS(1709),
    [sym_file_read] = ACTIONS(1709),
    [sym_file_read_line] = ACTIONS(1709),
    [sym_file_write] = ACTIONS(1709),
    [sym_file_write_line] = ACTIONS(1709),
    [sym_file_copy] = ACTIONS(1709),
    [sym_file_delete] = ACTIONS(1709),
    [sym_file_rename] = ACTIONS(1709),
    [sym_file_dir] = ACTIONS(1709),
    [sym_file_getdir] = ACTIONS(1709),
    [sym_file_setdir] = ACTIONS(1709),
    [sym_file_create_dir] = ACTIONS(1709),
    [sym_file_remove_dir] = ACTIONS(1709),
    [sym_file_seek] = ACTIONS(1709),
    [sym_length_array] = ACTIONS(1709),
    [sym_max_length_array] = ACTIONS(1709),
    [sym_set_length_array] = ACTIONS(1709),
    [sym_ip_client_open] = ACTIONS(1709),
    [sym_ip_client_close] = ACTIONS(1709),
    [sym_ip_server_open] = ACTIONS(1709),
    [sym_ip_server_close] = ACTIONS(1709),
    [sym_ip_bound_client_open] = ACTIONS(1709),
    [sym_ip_mc_server_open] = ACTIONS(1709),
    [sym_ip_set_option] = ACTIONS(1709),
    [sym_ssh_client_open] = ACTIONS(1709),
    [sym_ssh_client_close] = ACTIONS(1709),
    [sym_tls_client_open] = ACTIONS(1709),
    [sym_tls_client_close] = ACTIONS(1709),
    [sym_do_push] = ACTIONS(1709),
    [sym_do_push_timed] = ACTIONS(1709),
    [sym_do_release] = ACTIONS(1709),
    [sym_device_id] = ACTIONS(1709),
    [sym_device_info] = ACTIONS(1709),
    [sym_device_id_string] = ACTIONS(1709),
    [sym_device_standby] = ACTIONS(1709),
    [sym_device_wake] = ACTIONS(1709),
    [sym_timeline_create] = ACTIONS(1709),
    [sym_timeline_kill] = ACTIONS(1709),
    [sym_timeline_reload] = ACTIONS(1709),
    [sym_timeline_pause] = ACTIONS(1709),
    [sym_timeline_restart] = ACTIONS(1709),
    [sym_timeline_set] = ACTIONS(1709),
    [sym_timeline_get] = ACTIONS(1709),
    [sym_timeline_active] = ACTIONS(1709),
    [sym_timeline_dynamic_id] = ACTIONS(1709),
    [sym_variable_to_string] = ACTIONS(1709),
    [sym_string_to_variable] = ACTIONS(1709),
    [sym_length_variable_to_string] = ACTIONS(1709),
    [sym_type_cast] = ACTIONS(1709),
    [sym_variable_to_xml] = ACTIONS(1709),
    [sym_xml_to_variable] = ACTIONS(1709),
    [sym_length_variable_to_xml] = ACTIONS(1709),
    [sym_redirect_string] = ACTIONS(1709),
    [sym_random_number] = ACTIONS(1709),
    [sym_get_unique_id] = ACTIONS(1709),
    [sym_get_system_number] = ACTIONS(1709),
    [sym_set_system_number] = ACTIONS(1709),
    [sym_get_serial_number] = ACTIONS(1709),
    [sym_get_master_build] = ACTIONS(1709),
    [sym_set_validation_code] = ACTIONS(1709),
    [sym_reboot] = ACTIONS(1709),
    [sym_load_duet_module] = ACTIONS(1709),
    [sym_unload_duet_module] = ACTIONS(1709),
    [sym_duet_mem_size_set] = ACTIONS(1709),
    [sym_duet_mem_size_get] = ACTIONS(1709),
    [sym_dynamic_application_device] = ACTIONS(1709),
    [sym_dynamic_polled_port] = ACTIONS(1709),
    [sym_static_port_binding] = ACTIONS(1709),
    [sym_static_ip_binding] = ACTIONS(1709),
    [sym_get_dns_list] = ACTIONS(1709),
    [sym_set_dns_list] = ACTIONS(1709),
    [sym_get_ip_address] = ACTIONS(1709),
    [sym_set_ip_address] = ACTIONS(1709),
    [sym_get_url_list] = ACTIONS(1709),
    [sym_add_url_entry] = ACTIONS(1709),
    [sym_delete_url_entry] = ACTIONS(1709),
    [sym_abs_value] = ACTIONS(1709),
    [sym_max_value] = ACTIONS(1709),
    [sym_min_value] = ACTIONS(1709),
    [sym_exp_value] = ACTIONS(1709),
    [sym_log_value] = ACTIONS(1709),
    [sym_log10_value] = ACTIONS(1709),
    [sym_power_value] = ACTIONS(1709),
    [sym_sqrt_value] = ACTIONS(1709),
    [sym_raw_be] = ACTIONS(1709),
    [sym_raw_le] = ACTIONS(1709),
    [sym_internal_queue_size_set] = ACTIONS(1709),
    [sym_internal_queue_size_get] = ACTIONS(1709),
    [sym_internal_threshold_set] = ACTIONS(1709),
    [sym_internal_threshold_get] = ACTIONS(1709),
    [sym_rebuilt_event] = ACTIONS(1709),
    [sym_do_custom_event] = ACTIONS(1709),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1709),
    [sym_clkmgr_set_clk_source] = ACTIONS(1709),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1709),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1709),
    [sym_clkmgr_get_timezone] = ACTIONS(1709),
    [sym_clkmgr_set_timezone] = ACTIONS(1709),
    [sym_clkmgr_get_resync_period] = ACTIONS(1709),
    [sym_clkmgr_set_resync_period] = ACTIONS(1709),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1709),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1709),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1709),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1709),
    [sym_clkmgr_get_timeservers] = ACTIONS(1709),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1709),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1709),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1709),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1709),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1709),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1709),
    [sym_set_log_level] = ACTIONS(1709),
    [sym_get_log_level] = ACTIONS(1709),
    [sym_amx_log] = ACTIONS(1709),
    [sym_smtp_server_config_set] = ACTIONS(1709),
    [sym_smtp_server_config_get] = ACTIONS(1709),
    [sym_smtp_send] = ACTIONS(1709),
    [sym_validate_netlinx_account] = ACTIONS(1709),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1709),
    [sym_authenticate_certificate] = ACTIONS(1709),
    [sym_audit_netlinx_session_event] = ACTIONS(1709),
    [sym_audit_netlinx_generic_event] = ACTIONS(1709),
    [sym_get_available_flash_disk_space] = ACTIONS(1709),
    [sym_get_max_flash_disk_space] = ACTIONS(1709),
    [sym_data_create_feed] = ACTIONS(1709),
    [sym_data_delete_feed] = ACTIONS(1709),
    [sym_data_publish_feed] = ACTIONS(1709),
    [sym_data_get_published_feed] = ACTIONS(1709),
    [sym_data_add_record] = ACTIONS(1709),
    [sym_data_get_event_record] = ACTIONS(1709),
    [sym__wc_data_create_feed] = ACTIONS(1709),
    [sym__wc_data_add_record] = ACTIONS(1709),
    [sym__wc_data_get_event_record] = ACTIONS(1709),
    [sym_format] = ACTIONS(1709),
    [sym_astro_clock] = ACTIONS(1709),
    [sym_set_outdoor_temperature] = ACTIONS(1709),
    [sym_set_virtual_level_count] = ACTIONS(1709),
    [sym_set_virtual_channel_count] = ACTIONS(1709),
    [sym_set_virtual_port_count] = ACTIONS(1709),
    [sym__wc_encode_int] = ACTIONS(1709),
    [sym__wc_decode_int] = ACTIONS(1709),
    [sym__wc_int] = ACTIONS(1709),
    [sym__wc_to_ch_int] = ACTIONS(1709),
    [sym__ch_to_wc_int] = ACTIONS(1709),
    [sym__wc_find_string_int] = ACTIONS(1709),
    [sym__wc_left_string_int] = ACTIONS(1709),
    [sym__wc_lower_string_int] = ACTIONS(1709),
    [sym__wc_mid_string_int] = ACTIONS(1709),
    [sym__wc_remove_string_int] = ACTIONS(1709),
    [sym__wc_right_string_int] = ACTIONS(1709),
    [sym__wc_upper_string_int] = ACTIONS(1709),
    [sym__wc_compare_string_int] = ACTIONS(1709),
    [sym__wc_get_buffer_char_int] = ACTIONS(1709),
    [sym__wc_get_buffer_string_int] = ACTIONS(1709),
    [sym__wc_concat_string_int] = ACTIONS(1709),
    [sym__wc_file_open_int] = ACTIONS(1709),
    [sym__wc_file_close_int] = ACTIONS(1709),
    [sym__wc_file_read_int] = ACTIONS(1709),
    [sym__wc_file_read_line_int] = ACTIONS(1709),
    [sym__wc_file_write_int] = ACTIONS(1709),
    [sym__wc_file_write_line_int] = ACTIONS(1709),
    [aux_sym_preproc_include_token1] = ACTIONS(1709),
    [aux_sym_preproc_define_token1] = ACTIONS(1707),
    [aux_sym_preproc_warn_token1] = ACTIONS(1707),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1707),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1707),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1707),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1707),
    [aux_sym_preproc_else_token1] = ACTIONS(1707),
    [anon_sym_LBRACK] = ACTIONS(1707),
    [anon_sym_LBRACE] = ACTIONS(1707),
    [anon_sym_RBRACE] = ACTIONS(1707),
    [anon_sym_LPAREN] = ACTIONS(1709),
    [anon_sym_BANG] = ACTIONS(1707),
    [anon_sym_TILDE] = ACTIONS(1707),
    [anon_sym_DASH] = ACTIONS(1709),
    [anon_sym_PLUS] = ACTIONS(1709),
    [anon_sym_DASH_DASH] = ACTIONS(1707),
    [anon_sym_PLUS_PLUS] = ACTIONS(1707),
    [anon_sym_DQUOTE] = ACTIONS(1707),
    [anon_sym_SQUOTE] = ACTIONS(1707),
    [sym_number_literal] = ACTIONS(1707),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(207)] = {
    [ts_builtin_sym_end] = ACTIONS(1711),
    [sym_identifier] = ACTIONS(1713),
    [sym_program_name_keyword] = ACTIONS(1713),
    [sym_module_name_keyword] = ACTIONS(1713),
    [sym_define_device_keyword] = ACTIONS(1713),
    [sym_define_combine_keyword] = ACTIONS(1713),
    [sym_define_constant_keyword] = ACTIONS(1713),
    [sym_define_type_keyword] = ACTIONS(1713),
    [sym_define_variable_keyword] = ACTIONS(1713),
    [sym_define_system_variable_keyword] = ACTIONS(1713),
    [sym_define_connect_level_keyword] = ACTIONS(1713),
    [sym_define_latching_keyword] = ACTIONS(1713),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1713),
    [sym_define_toggling_keyword] = ACTIONS(1713),
    [sym_define_call_keyword] = ACTIONS(1713),
    [sym_define_function_keyword] = ACTIONS(1713),
    [sym_define_library_function_keyword] = ACTIONS(1713),
    [sym_define_start_keyword] = ACTIONS(1713),
    [sym_define_event_keyword] = ACTIONS(1713),
    [sym_define_program_keyword] = ACTIONS(1713),
    [sym_define_module_keyword] = ACTIONS(1713),
    [sym_char_keyword] = ACTIONS(1713),
    [sym_widechar_keyword] = ACTIONS(1713),
    [sym_integer_keyword] = ACTIONS(1713),
    [sym_sinteger_keyword] = ACTIONS(1713),
    [sym_long_keyword] = ACTIONS(1713),
    [sym_slong_keyword] = ACTIONS(1713),
    [sym_float_keyword] = ACTIONS(1713),
    [sym_double_keyword] = ACTIONS(1713),
    [sym_variant_keyword] = ACTIONS(1713),
    [sym_variantarray_keyword] = ACTIONS(1713),
    [sym_local_var_keyword] = ACTIONS(1713),
    [sym_stack_var_keyword] = ACTIONS(1713),
    [sym_constant_keyword] = ACTIONS(1713),
    [sym_volatile_keyword] = ACTIONS(1713),
    [sym_non_volatile_keyword] = ACTIONS(1713),
    [sym_persistent_keyword] = ACTIONS(1713),
    [sym_dev_keyword] = ACTIONS(1713),
    [sym_devlev_keyword] = ACTIONS(1713),
    [sym_devchan_keyword] = ACTIONS(1713),
    [sym_if_keyword] = ACTIONS(1713),
    [sym_switch_keyword] = ACTIONS(1713),
    [sym_case_keyword] = ACTIONS(1713),
    [sym_default_keyword] = ACTIONS(1713),
    [sym_while_keyword] = ACTIONS(1713),
    [sym_for_keyword] = ACTIONS(1713),
    [sym_break_keyword] = ACTIONS(1713),
    [sym_continue_keyword] = ACTIONS(1713),
    [sym_return_keyword] = ACTIONS(1713),
    [sym_select_keyword] = ACTIONS(1713),
    [sym_active_keyword] = ACTIONS(1713),
    [sym_struct_keyword] = ACTIONS(1713),
    [sym_structure_keyword] = ACTIONS(1713),
    [sym_button_event_keyword] = ACTIONS(1713),
    [sym_channel_event_keyword] = ACTIONS(1713),
    [sym_level_event_keyword] = ACTIONS(1713),
    [sym_data_event_keyword] = ACTIONS(1713),
    [sym_timeline_event_keyword] = ACTIONS(1713),
    [sym_custom_event_keyword] = ACTIONS(1713),
    [sym_push_keyword] = ACTIONS(1713),
    [sym_release_keyword] = ACTIONS(1713),
    [aux_sym_on_keyword_token1] = ACTIONS(1713),
    [aux_sym_off_keyword_token1] = ACTIONS(1713),
    [sym_send_level_keyword] = ACTIONS(1713),
    [sym_send_string_keyword] = ACTIONS(1713),
    [sym_send_command_keyword] = ACTIONS(1713),
    [sym_clear_buffer_keyword] = ACTIONS(1713),
    [sym_create_buffer_keyword] = ACTIONS(1713),
    [sym_create_multi_buffer_keyword] = ACTIONS(1713),
    [sym_call_keyword] = ACTIONS(1713),
    [sym_system_call_keyword] = ACTIONS(1713),
    [sym_devchan_to_keyword] = ACTIONS(1713),
    [sym_devchan_min_to_keyword] = ACTIONS(1713),
    [sym_devchan_total_off_keyword] = ACTIONS(1713),
    [sym_devchan_pulse_keyword] = ACTIONS(1713),
    [sym_wait_keyword] = ACTIONS(1713),
    [sym_cancel_wait_keyword] = ACTIONS(1713),
    [sym_wait_until_keyword] = ACTIONS(1713),
    [sym_cancel_wait_until_keyword] = ACTIONS(1713),
    [sym_cancel_all_wait_keyword] = ACTIONS(1713),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1713),
    [sym_not] = ACTIONS(1713),
    [sym_bnot] = ACTIONS(1713),
    [sym___file__] = ACTIONS(1713),
    [sym___date__] = ACTIONS(1713),
    [sym___ldate__] = ACTIONS(1713),
    [sym___time__] = ACTIONS(1713),
    [sym___line__] = ACTIONS(1713),
    [sym___name__] = ACTIONS(1713),
    [sym___netlinx__] = ACTIONS(1713),
    [sym_netlinx_axi_version] = ACTIONS(1713),
    [sym_url_flg_tcp] = ACTIONS(1713),
    [sym_url_flg_accinfopresent] = ACTIONS(1713),
    [sym_url_flg_temp] = ACTIONS(1713),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1713),
    [sym_url_flg_stat_mask] = ACTIONS(1713),
    [sym_url_flg_stat_lookup] = ACTIONS(1713),
    [sym_url_flg_stat_connecting] = ACTIONS(1713),
    [sym_url_flg_stat_waiting] = ACTIONS(1713),
    [sym_url_flg_stat_connected] = ACTIONS(1713),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1713),
    [sym_first_virtual_device] = ACTIONS(1713),
    [sym_first_local_port] = ACTIONS(1713),
    [sym_dynamic_virtual_device] = ACTIONS(1713),
    [sym_timeline_once] = ACTIONS(1713),
    [sym_timeline_repeat] = ACTIONS(1713),
    [sym_timeline_absolute] = ACTIONS(1713),
    [sym_timeline_relative] = ACTIONS(1713),
    [sym_do_push_timed_infinite] = ACTIONS(1713),
    [sym_source_type_no_address] = ACTIONS(1713),
    [sym_source_type_neuron_id] = ACTIONS(1713),
    [sym_source_type_ip_address] = ACTIONS(1713),
    [sym_source_type_axlink] = ACTIONS(1713),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1713),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1713),
    [sym_source_type_ip_socket_address] = ACTIONS(1713),
    [sym_source_type_rs232] = ACTIONS(1713),
    [sym_source_type_internal] = ACTIONS(1713),
    [sym_source_type_mac_address] = ACTIONS(1713),
    [sym_source_type_ipv4_port] = ACTIONS(1713),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1713),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1713),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1713),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1713),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1713),
    [sym_true] = ACTIONS(1713),
    [sym_false] = ACTIONS(1713),
    [sym_file_read_only] = ACTIONS(1713),
    [sym_file_rw_new] = ACTIONS(1713),
    [sym_file_rw_append] = ACTIONS(1713),
    [sym_ip_tcp] = ACTIONS(1713),
    [sym_ip_udp] = ACTIONS(1713),
    [sym_ip_udp_2way] = ACTIONS(1713),
    [sym_xml_encode_types] = ACTIONS(1713),
    [sym_xml_encode_char_as_list] = ACTIONS(1713),
    [sym_xml_encode_le] = ACTIONS(1713),
    [sym_xml_decode_types] = ACTIONS(1713),
    [sym_xml_decode_no_preserve] = ACTIONS(1713),
    [sym_duet_dev_type_display_device] = ACTIONS(1713),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1713),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1713),
    [sym_duet_dev_type_io_device] = ACTIONS(1713),
    [sym_duet_dev_type_relay_device] = ACTIONS(1713),
    [sym_duet_dev_type_ups] = ACTIONS(1713),
    [sym_duet_dev_type_amplifier] = ACTIONS(1713),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1713),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1713),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1713),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1713),
    [sym_duet_dev_type_camera] = ACTIONS(1713),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1713),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1713),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1713),
    [sym_duet_dev_type_dss] = ACTIONS(1713),
    [sym_duet_dev_type_dvr] = ACTIONS(1713),
    [sym_duet_dev_type_disc_device] = ACTIONS(1713),
    [sym_duet_dev_type_document_camera] = ACTIONS(1713),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1713),
    [sym_duet_dev_type_hvac] = ACTIONS(1713),
    [sym_duet_dev_type_keypad] = ACTIONS(1713),
    [sym_duet_dev_type_light] = ACTIONS(1713),
    [sym_duet_dev_type_monitor] = ACTIONS(1713),
    [sym_duet_dev_type_motor] = ACTIONS(1713),
    [sym_duet_dev_type_multi_window] = ACTIONS(1713),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1713),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1713),
    [sym_duet_dev_type_receiver] = ACTIONS(1713),
    [sym_duet_dev_type_security_system] = ACTIONS(1713),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1713),
    [sym_duet_dev_type_settop_box] = ACTIONS(1713),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1713),
    [sym_duet_dev_type_switcher] = ACTIONS(1713),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1713),
    [sym_duet_dev_type_tv] = ACTIONS(1713),
    [sym_duet_dev_type_utility] = ACTIONS(1713),
    [sym_duet_dev_type_vcr] = ACTIONS(1713),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1713),
    [sym_duet_dev_type_video_processor] = ACTIONS(1713),
    [sym_duet_dev_type_video_projector] = ACTIONS(1713),
    [sym_duet_dev_type_video_wall] = ACTIONS(1713),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1713),
    [sym_duet_dev_type_weather] = ACTIONS(1713),
    [sym_duet_dev_not_polled] = ACTIONS(1713),
    [sym_duet_dev_polled] = ACTIONS(1713),
    [sym_clkmgr_mode_network] = ACTIONS(1713),
    [sym_clkmgr_mode_standalone] = ACTIONS(1713),
    [sym_normal_standby] = ACTIONS(1713),
    [sym_normal_wake] = ACTIONS(1713),
    [sym_ip_multicast_ttl_option] = ACTIONS(1713),
    [sym_ip_ttl_subnet] = ACTIONS(1713),
    [sym_ip_ttl_site] = ACTIONS(1713),
    [sym_ip_ttl_region] = ACTIONS(1713),
    [sym_ip_ttl_continent] = ACTIONS(1713),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1713),
    [sym_ip_nodelay_on] = ACTIONS(1713),
    [sym_ip_nodelay_off] = ACTIONS(1713),
    [sym_amx_error] = ACTIONS(1713),
    [sym_amx_warning] = ACTIONS(1713),
    [sym_amx_info] = ACTIONS(1713),
    [sym_amx_debug] = ACTIONS(1713),
    [sym_netlinx_login_success] = ACTIONS(1713),
    [sym_netlinx_login_fail] = ACTIONS(1713),
    [sym_netlinx_logout] = ACTIONS(1713),
    [sym_null_str] = ACTIONS(1713),
    [sym_smtp_address] = ACTIONS(1713),
    [sym_smtp_port_number] = ACTIONS(1713),
    [sym_smtp_username] = ACTIONS(1713),
    [sym_smtp_password] = ACTIONS(1713),
    [sym_smtp_require_tls] = ACTIONS(1713),
    [sym_smtp_from] = ACTIONS(1713),
    [sym_smtp_tls_true] = ACTIONS(1713),
    [sym_smtp_tls_false] = ACTIONS(1713),
    [sym_listview_on_row_select_event] = ACTIONS(1713),
    [sym_data_structure_unknown] = ACTIONS(1713),
    [sym_data_structure_datafeed] = ACTIONS(1713),
    [sym_data_structure_datarecordset] = ACTIONS(1713),
    [sym_data_structure_datarecord] = ACTIONS(1713),
    [sym_data_structure_datafield] = ACTIONS(1713),
    [sym_data_max_content_field_count] = ACTIONS(1713),
    [sym_data_max_metadata_field_count] = ACTIONS(1713),
    [sym_data_max_id_length] = ACTIONS(1713),
    [sym_data_max_name_length] = ACTIONS(1713),
    [sym_data_max_description_length] = ACTIONS(1713),
    [sym_data_max_type_length] = ACTIONS(1713),
    [sym_data_max_format_length] = ACTIONS(1713),
    [sym_data_max_source_length] = ACTIONS(1713),
    [sym_data_max_value_length] = ACTIONS(1713),
    [sym_data_max_label_length] = ACTIONS(1713),
    [sym_data_max_url_length] = ACTIONS(1713),
    [sym_max_last_login_info_length] = ACTIONS(1713),
    [sym_data_type_unknown] = ACTIONS(1713),
    [sym_data_type_string] = ACTIONS(1713),
    [sym_data_type_datetime] = ACTIONS(1713),
    [sym_data_type_date] = ACTIONS(1713),
    [sym_data_type_time] = ACTIONS(1713),
    [sym_data_type_image] = ACTIONS(1713),
    [sym_data_format_url] = ACTIONS(1713),
    [sym_data_format_phone] = ACTIONS(1713),
    [sym_data_format_email] = ACTIONS(1713),
    [sym_data_format_iso8601] = ACTIONS(1713),
    [sym_auditlog_priv] = ACTIONS(1713),
    [sym_configuration_priv] = ACTIONS(1713),
    [sym_deviceconfig_priv] = ACTIONS(1713),
    [sym_ftp_priv] = ACTIONS(1713),
    [sym_http_priv] = ACTIONS(1713),
    [sym_networkconfig_priv] = ACTIONS(1713),
    [sym_programport_priv] = ACTIONS(1713),
    [sym_remoteui_priv] = ACTIONS(1713),
    [sym_securitycontrol_priv] = ACTIONS(1713),
    [sym_softwaremanagement_priv] = ACTIONS(1713),
    [sym_terminal_priv] = ACTIONS(1713),
    [sym_tpadmin_priv] = ACTIONS(1713),
    [sym_user1_priv] = ACTIONS(1713),
    [sym_user2_priv] = ACTIONS(1713),
    [sym_user3_priv] = ACTIONS(1713),
    [sym_user4_priv] = ACTIONS(1713),
    [sym_usermanagement_priv] = ACTIONS(1713),
    [sym_valid_account] = ACTIONS(1713),
    [sym_err_name_invalid] = ACTIONS(1713),
    [sym_err_password_invalid] = ACTIONS(1713),
    [sym_err_invalid_account] = ACTIONS(1713),
    [sym_err_no_authorization] = ACTIONS(1713),
    [sym_err_invalid_parameter] = ACTIONS(1713),
    [sym_err_user_locked_out] = ACTIONS(1713),
    [sym_err_user_account_expired] = ACTIONS(1713),
    [sym_err_general_error] = ACTIONS(1713),
    [sym_err_authentication_error] = ACTIONS(1713),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1713),
    [sym_tls_validate_certificate] = ACTIONS(1713),
    [sym_tdata] = ACTIONS(1713),
    [sym_tchannel] = ACTIONS(1713),
    [sym_tlevel] = ACTIONS(1713),
    [sym_tbutton] = ACTIONS(1713),
    [sym_ttimeline] = ACTIONS(1713),
    [sym_tcustom] = ACTIONS(1713),
    [sym_url_struct] = ACTIONS(1713),
    [sym_dns_struct] = ACTIONS(1713),
    [sym_ip_address_struct] = ACTIONS(1713),
    [sym_dev_info_struct] = ACTIONS(1713),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1713),
    [sym_data_feed] = ACTIONS(1713),
    [sym_data_field] = ACTIONS(1713),
    [sym_data_record] = ACTIONS(1713),
    [sym_wc_data_feed] = ACTIONS(1713),
    [sym_wc_data_field] = ACTIONS(1713),
    [sym_wc_data_record] = ACTIONS(1713),
    [sym_last_login_info] = ACTIONS(1713),
    [sym____reserved___] = ACTIONS(1713),
    [sym_date] = ACTIONS(1713),
    [sym_day] = ACTIONS(1713),
    [sym_time] = ACTIONS(1713),
    [sym_ldate] = ACTIONS(1713),
    [sym_push_device] = ACTIONS(1713),
    [sym_push_channel] = ACTIONS(1713),
    [sym_push_devchan] = ACTIONS(1713),
    [sym_release_device] = ACTIONS(1713),
    [sym_release_channel] = ACTIONS(1713),
    [sym_release_devchan] = ACTIONS(1713),
    [sym_master_slot] = ACTIONS(1713),
    [sym_get_pulse_time] = ACTIONS(1713),
    [sym_get_timer] = ACTIONS(1713),
    [sym_master_sn] = ACTIONS(1713),
    [sym_system_number] = ACTIONS(1713),
    [sym_button] = ACTIONS(1713),
    [sym_dv_channel] = ACTIONS(1713),
    [sym_data] = ACTIONS(1713),
    [sym_level] = ACTIONS(1713),
    [sym_timeline] = ACTIONS(1713),
    [sym_channel] = ACTIONS(1713),
    [sym_custom] = ACTIONS(1713),
    [sym_atoi] = ACTIONS(1713),
    [sym_atol] = ACTIONS(1713),
    [sym_atof] = ACTIONS(1713),
    [sym_itoa] = ACTIONS(1713),
    [sym_itohex] = ACTIONS(1713),
    [sym_hextoi] = ACTIONS(1713),
    [sym_ftoa] = ACTIONS(1713),
    [sym_find_string] = ACTIONS(1713),
    [sym_left_string] = ACTIONS(1713),
    [sym_right_string] = ACTIONS(1713),
    [sym_mid_string] = ACTIONS(1713),
    [sym_remove_string] = ACTIONS(1713),
    [sym_get_buffer_string] = ACTIONS(1713),
    [sym_get_buffer_char] = ACTIONS(1713),
    [sym_get_multi_buffer_string] = ACTIONS(1713),
    [sym_length_string] = ACTIONS(1713),
    [sym_max_length_string] = ACTIONS(1713),
    [sym_set_length_string] = ACTIONS(1713),
    [sym_compare_string] = ACTIONS(1713),
    [sym_upper_string] = ACTIONS(1713),
    [sym_lower_string] = ACTIONS(1713),
    [sym_get_last] = ACTIONS(1713),
    [sym_date_to_day] = ACTIONS(1713),
    [sym_date_to_month] = ACTIONS(1713),
    [sym_date_to_year] = ACTIONS(1713),
    [sym_time_to_hour] = ACTIONS(1713),
    [sym_time_to_minute] = ACTIONS(1713),
    [sym_time_to_second] = ACTIONS(1713),
    [sym_day_of_week] = ACTIONS(1713),
    [sym_set_pulse_time] = ACTIONS(1713),
    [sym_set_timer] = ACTIONS(1713),
    [sym_file_open] = ACTIONS(1713),
    [sym_file_close] = ACTIONS(1713),
    [sym_file_read] = ACTIONS(1713),
    [sym_file_read_line] = ACTIONS(1713),
    [sym_file_write] = ACTIONS(1713),
    [sym_file_write_line] = ACTIONS(1713),
    [sym_file_copy] = ACTIONS(1713),
    [sym_file_delete] = ACTIONS(1713),
    [sym_file_rename] = ACTIONS(1713),
    [sym_file_dir] = ACTIONS(1713),
    [sym_file_getdir] = ACTIONS(1713),
    [sym_file_setdir] = ACTIONS(1713),
    [sym_file_create_dir] = ACTIONS(1713),
    [sym_file_remove_dir] = ACTIONS(1713),
    [sym_file_seek] = ACTIONS(1713),
    [sym_length_array] = ACTIONS(1713),
    [sym_max_length_array] = ACTIONS(1713),
    [sym_set_length_array] = ACTIONS(1713),
    [sym_ip_client_open] = ACTIONS(1713),
    [sym_ip_client_close] = ACTIONS(1713),
    [sym_ip_server_open] = ACTIONS(1713),
    [sym_ip_server_close] = ACTIONS(1713),
    [sym_ip_bound_client_open] = ACTIONS(1713),
    [sym_ip_mc_server_open] = ACTIONS(1713),
    [sym_ip_set_option] = ACTIONS(1713),
    [sym_ssh_client_open] = ACTIONS(1713),
    [sym_ssh_client_close] = ACTIONS(1713),
    [sym_tls_client_open] = ACTIONS(1713),
    [sym_tls_client_close] = ACTIONS(1713),
    [sym_do_push] = ACTIONS(1713),
    [sym_do_push_timed] = ACTIONS(1713),
    [sym_do_release] = ACTIONS(1713),
    [sym_device_id] = ACTIONS(1713),
    [sym_device_info] = ACTIONS(1713),
    [sym_device_id_string] = ACTIONS(1713),
    [sym_device_standby] = ACTIONS(1713),
    [sym_device_wake] = ACTIONS(1713),
    [sym_timeline_create] = ACTIONS(1713),
    [sym_timeline_kill] = ACTIONS(1713),
    [sym_timeline_reload] = ACTIONS(1713),
    [sym_timeline_pause] = ACTIONS(1713),
    [sym_timeline_restart] = ACTIONS(1713),
    [sym_timeline_set] = ACTIONS(1713),
    [sym_timeline_get] = ACTIONS(1713),
    [sym_timeline_active] = ACTIONS(1713),
    [sym_timeline_dynamic_id] = ACTIONS(1713),
    [sym_variable_to_string] = ACTIONS(1713),
    [sym_string_to_variable] = ACTIONS(1713),
    [sym_length_variable_to_string] = ACTIONS(1713),
    [sym_type_cast] = ACTIONS(1713),
    [sym_variable_to_xml] = ACTIONS(1713),
    [sym_xml_to_variable] = ACTIONS(1713),
    [sym_length_variable_to_xml] = ACTIONS(1713),
    [sym_redirect_string] = ACTIONS(1713),
    [sym_random_number] = ACTIONS(1713),
    [sym_get_unique_id] = ACTIONS(1713),
    [sym_get_system_number] = ACTIONS(1713),
    [sym_set_system_number] = ACTIONS(1713),
    [sym_get_serial_number] = ACTIONS(1713),
    [sym_get_master_build] = ACTIONS(1713),
    [sym_set_validation_code] = ACTIONS(1713),
    [sym_reboot] = ACTIONS(1713),
    [sym_load_duet_module] = ACTIONS(1713),
    [sym_unload_duet_module] = ACTIONS(1713),
    [sym_duet_mem_size_set] = ACTIONS(1713),
    [sym_duet_mem_size_get] = ACTIONS(1713),
    [sym_dynamic_application_device] = ACTIONS(1713),
    [sym_dynamic_polled_port] = ACTIONS(1713),
    [sym_static_port_binding] = ACTIONS(1713),
    [sym_static_ip_binding] = ACTIONS(1713),
    [sym_get_dns_list] = ACTIONS(1713),
    [sym_set_dns_list] = ACTIONS(1713),
    [sym_get_ip_address] = ACTIONS(1713),
    [sym_set_ip_address] = ACTIONS(1713),
    [sym_get_url_list] = ACTIONS(1713),
    [sym_add_url_entry] = ACTIONS(1713),
    [sym_delete_url_entry] = ACTIONS(1713),
    [sym_abs_value] = ACTIONS(1713),
    [sym_max_value] = ACTIONS(1713),
    [sym_min_value] = ACTIONS(1713),
    [sym_exp_value] = ACTIONS(1713),
    [sym_log_value] = ACTIONS(1713),
    [sym_log10_value] = ACTIONS(1713),
    [sym_power_value] = ACTIONS(1713),
    [sym_sqrt_value] = ACTIONS(1713),
    [sym_raw_be] = ACTIONS(1713),
    [sym_raw_le] = ACTIONS(1713),
    [sym_internal_queue_size_set] = ACTIONS(1713),
    [sym_internal_queue_size_get] = ACTIONS(1713),
    [sym_internal_threshold_set] = ACTIONS(1713),
    [sym_internal_threshold_get] = ACTIONS(1713),
    [sym_rebuilt_event] = ACTIONS(1713),
    [sym_do_custom_event] = ACTIONS(1713),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1713),
    [sym_clkmgr_set_clk_source] = ACTIONS(1713),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1713),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1713),
    [sym_clkmgr_get_timezone] = ACTIONS(1713),
    [sym_clkmgr_set_timezone] = ACTIONS(1713),
    [sym_clkmgr_get_resync_period] = ACTIONS(1713),
    [sym_clkmgr_set_resync_period] = ACTIONS(1713),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1713),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1713),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1713),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1713),
    [sym_clkmgr_get_timeservers] = ACTIONS(1713),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1713),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1713),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1713),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1713),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1713),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1713),
    [sym_set_log_level] = ACTIONS(1713),
    [sym_get_log_level] = ACTIONS(1713),
    [sym_amx_log] = ACTIONS(1713),
    [sym_smtp_server_config_set] = ACTIONS(1713),
    [sym_smtp_server_config_get] = ACTIONS(1713),
    [sym_smtp_send] = ACTIONS(1713),
    [sym_validate_netlinx_account] = ACTIONS(1713),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1713),
    [sym_authenticate_certificate] = ACTIONS(1713),
    [sym_audit_netlinx_session_event] = ACTIONS(1713),
    [sym_audit_netlinx_generic_event] = ACTIONS(1713),
    [sym_get_available_flash_disk_space] = ACTIONS(1713),
    [sym_get_max_flash_disk_space] = ACTIONS(1713),
    [sym_data_create_feed] = ACTIONS(1713),
    [sym_data_delete_feed] = ACTIONS(1713),
    [sym_data_publish_feed] = ACTIONS(1713),
    [sym_data_get_published_feed] = ACTIONS(1713),
    [sym_data_add_record] = ACTIONS(1713),
    [sym_data_get_event_record] = ACTIONS(1713),
    [sym__wc_data_create_feed] = ACTIONS(1713),
    [sym__wc_data_add_record] = ACTIONS(1713),
    [sym__wc_data_get_event_record] = ACTIONS(1713),
    [sym_format] = ACTIONS(1713),
    [sym_astro_clock] = ACTIONS(1713),
    [sym_set_outdoor_temperature] = ACTIONS(1713),
    [sym_set_virtual_level_count] = ACTIONS(1713),
    [sym_set_virtual_channel_count] = ACTIONS(1713),
    [sym_set_virtual_port_count] = ACTIONS(1713),
    [sym__wc_encode_int] = ACTIONS(1713),
    [sym__wc_decode_int] = ACTIONS(1713),
    [sym__wc_int] = ACTIONS(1713),
    [sym__wc_to_ch_int] = ACTIONS(1713),
    [sym__ch_to_wc_int] = ACTIONS(1713),
    [sym__wc_find_string_int] = ACTIONS(1713),
    [sym__wc_left_string_int] = ACTIONS(1713),
    [sym__wc_lower_string_int] = ACTIONS(1713),
    [sym__wc_mid_string_int] = ACTIONS(1713),
    [sym__wc_remove_string_int] = ACTIONS(1713),
    [sym__wc_right_string_int] = ACTIONS(1713),
    [sym__wc_upper_string_int] = ACTIONS(1713),
    [sym__wc_compare_string_int] = ACTIONS(1713),
    [sym__wc_get_buffer_char_int] = ACTIONS(1713),
    [sym__wc_get_buffer_string_int] = ACTIONS(1713),
    [sym__wc_concat_string_int] = ACTIONS(1713),
    [sym__wc_file_open_int] = ACTIONS(1713),
    [sym__wc_file_close_int] = ACTIONS(1713),
    [sym__wc_file_read_int] = ACTIONS(1713),
    [sym__wc_file_read_line_int] = ACTIONS(1713),
    [sym__wc_file_write_int] = ACTIONS(1713),
    [sym__wc_file_write_line_int] = ACTIONS(1713),
    [aux_sym_preproc_include_token1] = ACTIONS(1713),
    [aux_sym_preproc_define_token1] = ACTIONS(1711),
    [aux_sym_preproc_warn_token1] = ACTIONS(1711),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1711),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1711),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1711),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1711),
    [aux_sym_preproc_else_token1] = ACTIONS(1711),
    [anon_sym_LBRACK] = ACTIONS(1711),
    [anon_sym_LBRACE] = ACTIONS(1711),
    [anon_sym_RBRACE] = ACTIONS(1711),
    [anon_sym_LPAREN] = ACTIONS(1713),
    [anon_sym_BANG] = ACTIONS(1711),
    [anon_sym_TILDE] = ACTIONS(1711),
    [anon_sym_DASH] = ACTIONS(1713),
    [anon_sym_PLUS] = ACTIONS(1713),
    [anon_sym_DASH_DASH] = ACTIONS(1711),
    [anon_sym_PLUS_PLUS] = ACTIONS(1711),
    [anon_sym_DQUOTE] = ACTIONS(1711),
    [anon_sym_SQUOTE] = ACTIONS(1711),
    [sym_number_literal] = ACTIONS(1711),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(208)] = {
    [sym_identifier] = ACTIONS(1715),
    [sym_program_name_keyword] = ACTIONS(1715),
    [sym_module_name_keyword] = ACTIONS(1715),
    [sym_define_device_keyword] = ACTIONS(1715),
    [sym_define_combine_keyword] = ACTIONS(1715),
    [sym_define_constant_keyword] = ACTIONS(1715),
    [sym_define_type_keyword] = ACTIONS(1715),
    [sym_define_variable_keyword] = ACTIONS(1715),
    [sym_define_system_variable_keyword] = ACTIONS(1715),
    [sym_define_connect_level_keyword] = ACTIONS(1715),
    [sym_define_latching_keyword] = ACTIONS(1715),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1715),
    [sym_define_toggling_keyword] = ACTIONS(1715),
    [sym_define_call_keyword] = ACTIONS(1715),
    [sym_define_function_keyword] = ACTIONS(1715),
    [sym_define_library_function_keyword] = ACTIONS(1715),
    [sym_define_start_keyword] = ACTIONS(1715),
    [sym_define_event_keyword] = ACTIONS(1715),
    [sym_define_program_keyword] = ACTIONS(1715),
    [sym_define_module_keyword] = ACTIONS(1715),
    [sym_char_keyword] = ACTIONS(1715),
    [sym_widechar_keyword] = ACTIONS(1715),
    [sym_integer_keyword] = ACTIONS(1715),
    [sym_sinteger_keyword] = ACTIONS(1715),
    [sym_long_keyword] = ACTIONS(1715),
    [sym_slong_keyword] = ACTIONS(1715),
    [sym_float_keyword] = ACTIONS(1715),
    [sym_double_keyword] = ACTIONS(1715),
    [sym_variant_keyword] = ACTIONS(1715),
    [sym_variantarray_keyword] = ACTIONS(1715),
    [sym_local_var_keyword] = ACTIONS(1715),
    [sym_stack_var_keyword] = ACTIONS(1715),
    [sym_constant_keyword] = ACTIONS(1715),
    [sym_volatile_keyword] = ACTIONS(1715),
    [sym_non_volatile_keyword] = ACTIONS(1715),
    [sym_persistent_keyword] = ACTIONS(1715),
    [sym_dev_keyword] = ACTIONS(1715),
    [sym_devlev_keyword] = ACTIONS(1715),
    [sym_devchan_keyword] = ACTIONS(1715),
    [sym_if_keyword] = ACTIONS(1715),
    [sym_else_keyword] = ACTIONS(1715),
    [sym_switch_keyword] = ACTIONS(1715),
    [sym_case_keyword] = ACTIONS(1715),
    [sym_default_keyword] = ACTIONS(1715),
    [sym_while_keyword] = ACTIONS(1715),
    [sym_for_keyword] = ACTIONS(1715),
    [sym_break_keyword] = ACTIONS(1715),
    [sym_continue_keyword] = ACTIONS(1715),
    [sym_return_keyword] = ACTIONS(1715),
    [sym_select_keyword] = ACTIONS(1715),
    [sym_active_keyword] = ACTIONS(1715),
    [sym_struct_keyword] = ACTIONS(1715),
    [sym_structure_keyword] = ACTIONS(1715),
    [sym_button_event_keyword] = ACTIONS(1715),
    [sym_channel_event_keyword] = ACTIONS(1715),
    [sym_level_event_keyword] = ACTIONS(1715),
    [sym_data_event_keyword] = ACTIONS(1715),
    [sym_timeline_event_keyword] = ACTIONS(1715),
    [sym_custom_event_keyword] = ACTIONS(1715),
    [sym_push_keyword] = ACTIONS(1715),
    [sym_release_keyword] = ACTIONS(1715),
    [aux_sym_on_keyword_token1] = ACTIONS(1715),
    [aux_sym_off_keyword_token1] = ACTIONS(1715),
    [sym_send_level_keyword] = ACTIONS(1715),
    [sym_send_string_keyword] = ACTIONS(1715),
    [sym_send_command_keyword] = ACTIONS(1715),
    [sym_clear_buffer_keyword] = ACTIONS(1715),
    [sym_create_buffer_keyword] = ACTIONS(1715),
    [sym_create_multi_buffer_keyword] = ACTIONS(1715),
    [sym_call_keyword] = ACTIONS(1715),
    [sym_system_call_keyword] = ACTIONS(1715),
    [sym_devchan_to_keyword] = ACTIONS(1715),
    [sym_devchan_min_to_keyword] = ACTIONS(1715),
    [sym_devchan_total_off_keyword] = ACTIONS(1715),
    [sym_devchan_pulse_keyword] = ACTIONS(1715),
    [sym_wait_keyword] = ACTIONS(1715),
    [sym_cancel_wait_keyword] = ACTIONS(1715),
    [sym_wait_until_keyword] = ACTIONS(1715),
    [sym_cancel_wait_until_keyword] = ACTIONS(1715),
    [sym_cancel_all_wait_keyword] = ACTIONS(1715),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1715),
    [sym_not] = ACTIONS(1715),
    [sym_bnot] = ACTIONS(1715),
    [sym___file__] = ACTIONS(1715),
    [sym___date__] = ACTIONS(1715),
    [sym___ldate__] = ACTIONS(1715),
    [sym___time__] = ACTIONS(1715),
    [sym___line__] = ACTIONS(1715),
    [sym___name__] = ACTIONS(1715),
    [sym___netlinx__] = ACTIONS(1715),
    [sym_netlinx_axi_version] = ACTIONS(1715),
    [sym_url_flg_tcp] = ACTIONS(1715),
    [sym_url_flg_accinfopresent] = ACTIONS(1715),
    [sym_url_flg_temp] = ACTIONS(1715),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1715),
    [sym_url_flg_stat_mask] = ACTIONS(1715),
    [sym_url_flg_stat_lookup] = ACTIONS(1715),
    [sym_url_flg_stat_connecting] = ACTIONS(1715),
    [sym_url_flg_stat_waiting] = ACTIONS(1715),
    [sym_url_flg_stat_connected] = ACTIONS(1715),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1715),
    [sym_first_virtual_device] = ACTIONS(1715),
    [sym_first_local_port] = ACTIONS(1715),
    [sym_dynamic_virtual_device] = ACTIONS(1715),
    [sym_timeline_once] = ACTIONS(1715),
    [sym_timeline_repeat] = ACTIONS(1715),
    [sym_timeline_absolute] = ACTIONS(1715),
    [sym_timeline_relative] = ACTIONS(1715),
    [sym_do_push_timed_infinite] = ACTIONS(1715),
    [sym_source_type_no_address] = ACTIONS(1715),
    [sym_source_type_neuron_id] = ACTIONS(1715),
    [sym_source_type_ip_address] = ACTIONS(1715),
    [sym_source_type_axlink] = ACTIONS(1715),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1715),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1715),
    [sym_source_type_ip_socket_address] = ACTIONS(1715),
    [sym_source_type_rs232] = ACTIONS(1715),
    [sym_source_type_internal] = ACTIONS(1715),
    [sym_source_type_mac_address] = ACTIONS(1715),
    [sym_source_type_ipv4_port] = ACTIONS(1715),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1715),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1715),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1715),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1715),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1715),
    [sym_true] = ACTIONS(1715),
    [sym_false] = ACTIONS(1715),
    [sym_file_read_only] = ACTIONS(1715),
    [sym_file_rw_new] = ACTIONS(1715),
    [sym_file_rw_append] = ACTIONS(1715),
    [sym_ip_tcp] = ACTIONS(1715),
    [sym_ip_udp] = ACTIONS(1715),
    [sym_ip_udp_2way] = ACTIONS(1715),
    [sym_xml_encode_types] = ACTIONS(1715),
    [sym_xml_encode_char_as_list] = ACTIONS(1715),
    [sym_xml_encode_le] = ACTIONS(1715),
    [sym_xml_decode_types] = ACTIONS(1715),
    [sym_xml_decode_no_preserve] = ACTIONS(1715),
    [sym_duet_dev_type_display_device] = ACTIONS(1715),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1715),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1715),
    [sym_duet_dev_type_io_device] = ACTIONS(1715),
    [sym_duet_dev_type_relay_device] = ACTIONS(1715),
    [sym_duet_dev_type_ups] = ACTIONS(1715),
    [sym_duet_dev_type_amplifier] = ACTIONS(1715),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1715),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1715),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1715),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1715),
    [sym_duet_dev_type_camera] = ACTIONS(1715),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1715),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1715),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1715),
    [sym_duet_dev_type_dss] = ACTIONS(1715),
    [sym_duet_dev_type_dvr] = ACTIONS(1715),
    [sym_duet_dev_type_disc_device] = ACTIONS(1715),
    [sym_duet_dev_type_document_camera] = ACTIONS(1715),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1715),
    [sym_duet_dev_type_hvac] = ACTIONS(1715),
    [sym_duet_dev_type_keypad] = ACTIONS(1715),
    [sym_duet_dev_type_light] = ACTIONS(1715),
    [sym_duet_dev_type_monitor] = ACTIONS(1715),
    [sym_duet_dev_type_motor] = ACTIONS(1715),
    [sym_duet_dev_type_multi_window] = ACTIONS(1715),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1715),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1715),
    [sym_duet_dev_type_receiver] = ACTIONS(1715),
    [sym_duet_dev_type_security_system] = ACTIONS(1715),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1715),
    [sym_duet_dev_type_settop_box] = ACTIONS(1715),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1715),
    [sym_duet_dev_type_switcher] = ACTIONS(1715),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1715),
    [sym_duet_dev_type_tv] = ACTIONS(1715),
    [sym_duet_dev_type_utility] = ACTIONS(1715),
    [sym_duet_dev_type_vcr] = ACTIONS(1715),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1715),
    [sym_duet_dev_type_video_processor] = ACTIONS(1715),
    [sym_duet_dev_type_video_projector] = ACTIONS(1715),
    [sym_duet_dev_type_video_wall] = ACTIONS(1715),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1715),
    [sym_duet_dev_type_weather] = ACTIONS(1715),
    [sym_duet_dev_not_polled] = ACTIONS(1715),
    [sym_duet_dev_polled] = ACTIONS(1715),
    [sym_clkmgr_mode_network] = ACTIONS(1715),
    [sym_clkmgr_mode_standalone] = ACTIONS(1715),
    [sym_normal_standby] = ACTIONS(1715),
    [sym_normal_wake] = ACTIONS(1715),
    [sym_ip_multicast_ttl_option] = ACTIONS(1715),
    [sym_ip_ttl_subnet] = ACTIONS(1715),
    [sym_ip_ttl_site] = ACTIONS(1715),
    [sym_ip_ttl_region] = ACTIONS(1715),
    [sym_ip_ttl_continent] = ACTIONS(1715),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1715),
    [sym_ip_nodelay_on] = ACTIONS(1715),
    [sym_ip_nodelay_off] = ACTIONS(1715),
    [sym_amx_error] = ACTIONS(1715),
    [sym_amx_warning] = ACTIONS(1715),
    [sym_amx_info] = ACTIONS(1715),
    [sym_amx_debug] = ACTIONS(1715),
    [sym_netlinx_login_success] = ACTIONS(1715),
    [sym_netlinx_login_fail] = ACTIONS(1715),
    [sym_netlinx_logout] = ACTIONS(1715),
    [sym_null_str] = ACTIONS(1715),
    [sym_smtp_address] = ACTIONS(1715),
    [sym_smtp_port_number] = ACTIONS(1715),
    [sym_smtp_username] = ACTIONS(1715),
    [sym_smtp_password] = ACTIONS(1715),
    [sym_smtp_require_tls] = ACTIONS(1715),
    [sym_smtp_from] = ACTIONS(1715),
    [sym_smtp_tls_true] = ACTIONS(1715),
    [sym_smtp_tls_false] = ACTIONS(1715),
    [sym_listview_on_row_select_event] = ACTIONS(1715),
    [sym_data_structure_unknown] = ACTIONS(1715),
    [sym_data_structure_datafeed] = ACTIONS(1715),
    [sym_data_structure_datarecordset] = ACTIONS(1715),
    [sym_data_structure_datarecord] = ACTIONS(1715),
    [sym_data_structure_datafield] = ACTIONS(1715),
    [sym_data_max_content_field_count] = ACTIONS(1715),
    [sym_data_max_metadata_field_count] = ACTIONS(1715),
    [sym_data_max_id_length] = ACTIONS(1715),
    [sym_data_max_name_length] = ACTIONS(1715),
    [sym_data_max_description_length] = ACTIONS(1715),
    [sym_data_max_type_length] = ACTIONS(1715),
    [sym_data_max_format_length] = ACTIONS(1715),
    [sym_data_max_source_length] = ACTIONS(1715),
    [sym_data_max_value_length] = ACTIONS(1715),
    [sym_data_max_label_length] = ACTIONS(1715),
    [sym_data_max_url_length] = ACTIONS(1715),
    [sym_max_last_login_info_length] = ACTIONS(1715),
    [sym_data_type_unknown] = ACTIONS(1715),
    [sym_data_type_string] = ACTIONS(1715),
    [sym_data_type_datetime] = ACTIONS(1715),
    [sym_data_type_date] = ACTIONS(1715),
    [sym_data_type_time] = ACTIONS(1715),
    [sym_data_type_image] = ACTIONS(1715),
    [sym_data_format_url] = ACTIONS(1715),
    [sym_data_format_phone] = ACTIONS(1715),
    [sym_data_format_email] = ACTIONS(1715),
    [sym_data_format_iso8601] = ACTIONS(1715),
    [sym_auditlog_priv] = ACTIONS(1715),
    [sym_configuration_priv] = ACTIONS(1715),
    [sym_deviceconfig_priv] = ACTIONS(1715),
    [sym_ftp_priv] = ACTIONS(1715),
    [sym_http_priv] = ACTIONS(1715),
    [sym_networkconfig_priv] = ACTIONS(1715),
    [sym_programport_priv] = ACTIONS(1715),
    [sym_remoteui_priv] = ACTIONS(1715),
    [sym_securitycontrol_priv] = ACTIONS(1715),
    [sym_softwaremanagement_priv] = ACTIONS(1715),
    [sym_terminal_priv] = ACTIONS(1715),
    [sym_tpadmin_priv] = ACTIONS(1715),
    [sym_user1_priv] = ACTIONS(1715),
    [sym_user2_priv] = ACTIONS(1715),
    [sym_user3_priv] = ACTIONS(1715),
    [sym_user4_priv] = ACTIONS(1715),
    [sym_usermanagement_priv] = ACTIONS(1715),
    [sym_valid_account] = ACTIONS(1715),
    [sym_err_name_invalid] = ACTIONS(1715),
    [sym_err_password_invalid] = ACTIONS(1715),
    [sym_err_invalid_account] = ACTIONS(1715),
    [sym_err_no_authorization] = ACTIONS(1715),
    [sym_err_invalid_parameter] = ACTIONS(1715),
    [sym_err_user_locked_out] = ACTIONS(1715),
    [sym_err_user_account_expired] = ACTIONS(1715),
    [sym_err_general_error] = ACTIONS(1715),
    [sym_err_authentication_error] = ACTIONS(1715),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1715),
    [sym_tls_validate_certificate] = ACTIONS(1715),
    [sym_tdata] = ACTIONS(1715),
    [sym_tchannel] = ACTIONS(1715),
    [sym_tlevel] = ACTIONS(1715),
    [sym_tbutton] = ACTIONS(1715),
    [sym_ttimeline] = ACTIONS(1715),
    [sym_tcustom] = ACTIONS(1715),
    [sym_url_struct] = ACTIONS(1715),
    [sym_dns_struct] = ACTIONS(1715),
    [sym_ip_address_struct] = ACTIONS(1715),
    [sym_dev_info_struct] = ACTIONS(1715),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1715),
    [sym_data_feed] = ACTIONS(1715),
    [sym_data_field] = ACTIONS(1715),
    [sym_data_record] = ACTIONS(1715),
    [sym_wc_data_feed] = ACTIONS(1715),
    [sym_wc_data_field] = ACTIONS(1715),
    [sym_wc_data_record] = ACTIONS(1715),
    [sym_last_login_info] = ACTIONS(1715),
    [sym____reserved___] = ACTIONS(1715),
    [sym_date] = ACTIONS(1715),
    [sym_day] = ACTIONS(1715),
    [sym_time] = ACTIONS(1715),
    [sym_ldate] = ACTIONS(1715),
    [sym_push_device] = ACTIONS(1715),
    [sym_push_channel] = ACTIONS(1715),
    [sym_push_devchan] = ACTIONS(1715),
    [sym_release_device] = ACTIONS(1715),
    [sym_release_channel] = ACTIONS(1715),
    [sym_release_devchan] = ACTIONS(1715),
    [sym_master_slot] = ACTIONS(1715),
    [sym_get_pulse_time] = ACTIONS(1715),
    [sym_get_timer] = ACTIONS(1715),
    [sym_master_sn] = ACTIONS(1715),
    [sym_system_number] = ACTIONS(1715),
    [sym_button] = ACTIONS(1715),
    [sym_dv_channel] = ACTIONS(1715),
    [sym_data] = ACTIONS(1715),
    [sym_level] = ACTIONS(1715),
    [sym_timeline] = ACTIONS(1715),
    [sym_channel] = ACTIONS(1715),
    [sym_custom] = ACTIONS(1715),
    [sym_atoi] = ACTIONS(1715),
    [sym_atol] = ACTIONS(1715),
    [sym_atof] = ACTIONS(1715),
    [sym_itoa] = ACTIONS(1715),
    [sym_itohex] = ACTIONS(1715),
    [sym_hextoi] = ACTIONS(1715),
    [sym_ftoa] = ACTIONS(1715),
    [sym_find_string] = ACTIONS(1715),
    [sym_left_string] = ACTIONS(1715),
    [sym_right_string] = ACTIONS(1715),
    [sym_mid_string] = ACTIONS(1715),
    [sym_remove_string] = ACTIONS(1715),
    [sym_get_buffer_string] = ACTIONS(1715),
    [sym_get_buffer_char] = ACTIONS(1715),
    [sym_get_multi_buffer_string] = ACTIONS(1715),
    [sym_length_string] = ACTIONS(1715),
    [sym_max_length_string] = ACTIONS(1715),
    [sym_set_length_string] = ACTIONS(1715),
    [sym_compare_string] = ACTIONS(1715),
    [sym_upper_string] = ACTIONS(1715),
    [sym_lower_string] = ACTIONS(1715),
    [sym_get_last] = ACTIONS(1715),
    [sym_date_to_day] = ACTIONS(1715),
    [sym_date_to_month] = ACTIONS(1715),
    [sym_date_to_year] = ACTIONS(1715),
    [sym_time_to_hour] = ACTIONS(1715),
    [sym_time_to_minute] = ACTIONS(1715),
    [sym_time_to_second] = ACTIONS(1715),
    [sym_day_of_week] = ACTIONS(1715),
    [sym_set_pulse_time] = ACTIONS(1715),
    [sym_set_timer] = ACTIONS(1715),
    [sym_file_open] = ACTIONS(1715),
    [sym_file_close] = ACTIONS(1715),
    [sym_file_read] = ACTIONS(1715),
    [sym_file_read_line] = ACTIONS(1715),
    [sym_file_write] = ACTIONS(1715),
    [sym_file_write_line] = ACTIONS(1715),
    [sym_file_copy] = ACTIONS(1715),
    [sym_file_delete] = ACTIONS(1715),
    [sym_file_rename] = ACTIONS(1715),
    [sym_file_dir] = ACTIONS(1715),
    [sym_file_getdir] = ACTIONS(1715),
    [sym_file_setdir] = ACTIONS(1715),
    [sym_file_create_dir] = ACTIONS(1715),
    [sym_file_remove_dir] = ACTIONS(1715),
    [sym_file_seek] = ACTIONS(1715),
    [sym_length_array] = ACTIONS(1715),
    [sym_max_length_array] = ACTIONS(1715),
    [sym_set_length_array] = ACTIONS(1715),
    [sym_ip_client_open] = ACTIONS(1715),
    [sym_ip_client_close] = ACTIONS(1715),
    [sym_ip_server_open] = ACTIONS(1715),
    [sym_ip_server_close] = ACTIONS(1715),
    [sym_ip_bound_client_open] = ACTIONS(1715),
    [sym_ip_mc_server_open] = ACTIONS(1715),
    [sym_ip_set_option] = ACTIONS(1715),
    [sym_ssh_client_open] = ACTIONS(1715),
    [sym_ssh_client_close] = ACTIONS(1715),
    [sym_tls_client_open] = ACTIONS(1715),
    [sym_tls_client_close] = ACTIONS(1715),
    [sym_do_push] = ACTIONS(1715),
    [sym_do_push_timed] = ACTIONS(1715),
    [sym_do_release] = ACTIONS(1715),
    [sym_device_id] = ACTIONS(1715),
    [sym_device_info] = ACTIONS(1715),
    [sym_device_id_string] = ACTIONS(1715),
    [sym_device_standby] = ACTIONS(1715),
    [sym_device_wake] = ACTIONS(1715),
    [sym_timeline_create] = ACTIONS(1715),
    [sym_timeline_kill] = ACTIONS(1715),
    [sym_timeline_reload] = ACTIONS(1715),
    [sym_timeline_pause] = ACTIONS(1715),
    [sym_timeline_restart] = ACTIONS(1715),
    [sym_timeline_set] = ACTIONS(1715),
    [sym_timeline_get] = ACTIONS(1715),
    [sym_timeline_active] = ACTIONS(1715),
    [sym_timeline_dynamic_id] = ACTIONS(1715),
    [sym_variable_to_string] = ACTIONS(1715),
    [sym_string_to_variable] = ACTIONS(1715),
    [sym_length_variable_to_string] = ACTIONS(1715),
    [sym_type_cast] = ACTIONS(1715),
    [sym_variable_to_xml] = ACTIONS(1715),
    [sym_xml_to_variable] = ACTIONS(1715),
    [sym_length_variable_to_xml] = ACTIONS(1715),
    [sym_redirect_string] = ACTIONS(1715),
    [sym_random_number] = ACTIONS(1715),
    [sym_get_unique_id] = ACTIONS(1715),
    [sym_get_system_number] = ACTIONS(1715),
    [sym_set_system_number] = ACTIONS(1715),
    [sym_get_serial_number] = ACTIONS(1715),
    [sym_get_master_build] = ACTIONS(1715),
    [sym_set_validation_code] = ACTIONS(1715),
    [sym_reboot] = ACTIONS(1715),
    [sym_load_duet_module] = ACTIONS(1715),
    [sym_unload_duet_module] = ACTIONS(1715),
    [sym_duet_mem_size_set] = ACTIONS(1715),
    [sym_duet_mem_size_get] = ACTIONS(1715),
    [sym_dynamic_application_device] = ACTIONS(1715),
    [sym_dynamic_polled_port] = ACTIONS(1715),
    [sym_static_port_binding] = ACTIONS(1715),
    [sym_static_ip_binding] = ACTIONS(1715),
    [sym_get_dns_list] = ACTIONS(1715),
    [sym_set_dns_list] = ACTIONS(1715),
    [sym_get_ip_address] = ACTIONS(1715),
    [sym_set_ip_address] = ACTIONS(1715),
    [sym_get_url_list] = ACTIONS(1715),
    [sym_add_url_entry] = ACTIONS(1715),
    [sym_delete_url_entry] = ACTIONS(1715),
    [sym_abs_value] = ACTIONS(1715),
    [sym_max_value] = ACTIONS(1715),
    [sym_min_value] = ACTIONS(1715),
    [sym_exp_value] = ACTIONS(1715),
    [sym_log_value] = ACTIONS(1715),
    [sym_log10_value] = ACTIONS(1715),
    [sym_power_value] = ACTIONS(1715),
    [sym_sqrt_value] = ACTIONS(1715),
    [sym_raw_be] = ACTIONS(1715),
    [sym_raw_le] = ACTIONS(1715),
    [sym_internal_queue_size_set] = ACTIONS(1715),
    [sym_internal_queue_size_get] = ACTIONS(1715),
    [sym_internal_threshold_set] = ACTIONS(1715),
    [sym_internal_threshold_get] = ACTIONS(1715),
    [sym_rebuilt_event] = ACTIONS(1715),
    [sym_do_custom_event] = ACTIONS(1715),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1715),
    [sym_clkmgr_set_clk_source] = ACTIONS(1715),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1715),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1715),
    [sym_clkmgr_get_timezone] = ACTIONS(1715),
    [sym_clkmgr_set_timezone] = ACTIONS(1715),
    [sym_clkmgr_get_resync_period] = ACTIONS(1715),
    [sym_clkmgr_set_resync_period] = ACTIONS(1715),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1715),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1715),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1715),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1715),
    [sym_clkmgr_get_timeservers] = ACTIONS(1715),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1715),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1715),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1715),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1715),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1715),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1715),
    [sym_set_log_level] = ACTIONS(1715),
    [sym_get_log_level] = ACTIONS(1715),
    [sym_amx_log] = ACTIONS(1715),
    [sym_smtp_server_config_set] = ACTIONS(1715),
    [sym_smtp_server_config_get] = ACTIONS(1715),
    [sym_smtp_send] = ACTIONS(1715),
    [sym_validate_netlinx_account] = ACTIONS(1715),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1715),
    [sym_authenticate_certificate] = ACTIONS(1715),
    [sym_audit_netlinx_session_event] = ACTIONS(1715),
    [sym_audit_netlinx_generic_event] = ACTIONS(1715),
    [sym_get_available_flash_disk_space] = ACTIONS(1715),
    [sym_get_max_flash_disk_space] = ACTIONS(1715),
    [sym_data_create_feed] = ACTIONS(1715),
    [sym_data_delete_feed] = ACTIONS(1715),
    [sym_data_publish_feed] = ACTIONS(1715),
    [sym_data_get_published_feed] = ACTIONS(1715),
    [sym_data_add_record] = ACTIONS(1715),
    [sym_data_get_event_record] = ACTIONS(1715),
    [sym__wc_data_create_feed] = ACTIONS(1715),
    [sym__wc_data_add_record] = ACTIONS(1715),
    [sym__wc_data_get_event_record] = ACTIONS(1715),
    [sym_format] = ACTIONS(1715),
    [sym_astro_clock] = ACTIONS(1715),
    [sym_set_outdoor_temperature] = ACTIONS(1715),
    [sym_set_virtual_level_count] = ACTIONS(1715),
    [sym_set_virtual_channel_count] = ACTIONS(1715),
    [sym_set_virtual_port_count] = ACTIONS(1715),
    [sym__wc_encode_int] = ACTIONS(1715),
    [sym__wc_decode_int] = ACTIONS(1715),
    [sym__wc_int] = ACTIONS(1715),
    [sym__wc_to_ch_int] = ACTIONS(1715),
    [sym__ch_to_wc_int] = ACTIONS(1715),
    [sym__wc_find_string_int] = ACTIONS(1715),
    [sym__wc_left_string_int] = ACTIONS(1715),
    [sym__wc_lower_string_int] = ACTIONS(1715),
    [sym__wc_mid_string_int] = ACTIONS(1715),
    [sym__wc_remove_string_int] = ACTIONS(1715),
    [sym__wc_right_string_int] = ACTIONS(1715),
    [sym__wc_upper_string_int] = ACTIONS(1715),
    [sym__wc_compare_string_int] = ACTIONS(1715),
    [sym__wc_get_buffer_char_int] = ACTIONS(1715),
    [sym__wc_get_buffer_string_int] = ACTIONS(1715),
    [sym__wc_concat_string_int] = ACTIONS(1715),
    [sym__wc_file_open_int] = ACTIONS(1715),
    [sym__wc_file_close_int] = ACTIONS(1715),
    [sym__wc_file_read_int] = ACTIONS(1715),
    [sym__wc_file_read_line_int] = ACTIONS(1715),
    [sym__wc_file_write_int] = ACTIONS(1715),
    [sym__wc_file_write_line_int] = ACTIONS(1715),
    [aux_sym_preproc_include_token1] = ACTIONS(1715),
    [aux_sym_preproc_define_token1] = ACTIONS(1717),
    [aux_sym_preproc_warn_token1] = ACTIONS(1717),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1717),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1717),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1717),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1717),
    [aux_sym_preproc_else_token1] = ACTIONS(1717),
    [anon_sym_LBRACK] = ACTIONS(1717),
    [anon_sym_LBRACE] = ACTIONS(1717),
    [anon_sym_LPAREN] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [anon_sym_DASH] = ACTIONS(1715),
    [anon_sym_PLUS] = ACTIONS(1715),
    [anon_sym_DASH_DASH] = ACTIONS(1717),
    [anon_sym_PLUS_PLUS] = ACTIONS(1717),
    [anon_sym_DQUOTE] = ACTIONS(1717),
    [anon_sym_SQUOTE] = ACTIONS(1717),
    [sym_number_literal] = ACTIONS(1717),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(209)] = {
    [sym_identifier] = ACTIONS(1719),
    [sym_program_name_keyword] = ACTIONS(1719),
    [sym_module_name_keyword] = ACTIONS(1719),
    [sym_define_device_keyword] = ACTIONS(1719),
    [sym_define_combine_keyword] = ACTIONS(1719),
    [sym_define_constant_keyword] = ACTIONS(1719),
    [sym_define_type_keyword] = ACTIONS(1719),
    [sym_define_variable_keyword] = ACTIONS(1719),
    [sym_define_system_variable_keyword] = ACTIONS(1719),
    [sym_define_connect_level_keyword] = ACTIONS(1719),
    [sym_define_latching_keyword] = ACTIONS(1719),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1719),
    [sym_define_toggling_keyword] = ACTIONS(1719),
    [sym_define_call_keyword] = ACTIONS(1719),
    [sym_define_function_keyword] = ACTIONS(1719),
    [sym_define_library_function_keyword] = ACTIONS(1719),
    [sym_define_start_keyword] = ACTIONS(1719),
    [sym_define_event_keyword] = ACTIONS(1719),
    [sym_define_program_keyword] = ACTIONS(1719),
    [sym_define_module_keyword] = ACTIONS(1719),
    [sym_char_keyword] = ACTIONS(1719),
    [sym_widechar_keyword] = ACTIONS(1719),
    [sym_integer_keyword] = ACTIONS(1719),
    [sym_sinteger_keyword] = ACTIONS(1719),
    [sym_long_keyword] = ACTIONS(1719),
    [sym_slong_keyword] = ACTIONS(1719),
    [sym_float_keyword] = ACTIONS(1719),
    [sym_double_keyword] = ACTIONS(1719),
    [sym_variant_keyword] = ACTIONS(1719),
    [sym_variantarray_keyword] = ACTIONS(1719),
    [sym_local_var_keyword] = ACTIONS(1719),
    [sym_stack_var_keyword] = ACTIONS(1719),
    [sym_constant_keyword] = ACTIONS(1719),
    [sym_volatile_keyword] = ACTIONS(1719),
    [sym_non_volatile_keyword] = ACTIONS(1719),
    [sym_persistent_keyword] = ACTIONS(1719),
    [sym_dev_keyword] = ACTIONS(1719),
    [sym_devlev_keyword] = ACTIONS(1719),
    [sym_devchan_keyword] = ACTIONS(1719),
    [sym_if_keyword] = ACTIONS(1719),
    [sym_else_keyword] = ACTIONS(1719),
    [sym_switch_keyword] = ACTIONS(1719),
    [sym_case_keyword] = ACTIONS(1719),
    [sym_default_keyword] = ACTIONS(1719),
    [sym_while_keyword] = ACTIONS(1719),
    [sym_for_keyword] = ACTIONS(1719),
    [sym_break_keyword] = ACTIONS(1719),
    [sym_continue_keyword] = ACTIONS(1719),
    [sym_return_keyword] = ACTIONS(1719),
    [sym_select_keyword] = ACTIONS(1719),
    [sym_active_keyword] = ACTIONS(1719),
    [sym_struct_keyword] = ACTIONS(1719),
    [sym_structure_keyword] = ACTIONS(1719),
    [sym_button_event_keyword] = ACTIONS(1719),
    [sym_channel_event_keyword] = ACTIONS(1719),
    [sym_level_event_keyword] = ACTIONS(1719),
    [sym_data_event_keyword] = ACTIONS(1719),
    [sym_timeline_event_keyword] = ACTIONS(1719),
    [sym_custom_event_keyword] = ACTIONS(1719),
    [sym_push_keyword] = ACTIONS(1719),
    [sym_release_keyword] = ACTIONS(1719),
    [aux_sym_on_keyword_token1] = ACTIONS(1719),
    [aux_sym_off_keyword_token1] = ACTIONS(1719),
    [sym_send_level_keyword] = ACTIONS(1719),
    [sym_send_string_keyword] = ACTIONS(1719),
    [sym_send_command_keyword] = ACTIONS(1719),
    [sym_clear_buffer_keyword] = ACTIONS(1719),
    [sym_create_buffer_keyword] = ACTIONS(1719),
    [sym_create_multi_buffer_keyword] = ACTIONS(1719),
    [sym_call_keyword] = ACTIONS(1719),
    [sym_system_call_keyword] = ACTIONS(1719),
    [sym_devchan_to_keyword] = ACTIONS(1719),
    [sym_devchan_min_to_keyword] = ACTIONS(1719),
    [sym_devchan_total_off_keyword] = ACTIONS(1719),
    [sym_devchan_pulse_keyword] = ACTIONS(1719),
    [sym_wait_keyword] = ACTIONS(1719),
    [sym_cancel_wait_keyword] = ACTIONS(1719),
    [sym_wait_until_keyword] = ACTIONS(1719),
    [sym_cancel_wait_until_keyword] = ACTIONS(1719),
    [sym_cancel_all_wait_keyword] = ACTIONS(1719),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1719),
    [sym_not] = ACTIONS(1719),
    [sym_bnot] = ACTIONS(1719),
    [sym___file__] = ACTIONS(1719),
    [sym___date__] = ACTIONS(1719),
    [sym___ldate__] = ACTIONS(1719),
    [sym___time__] = ACTIONS(1719),
    [sym___line__] = ACTIONS(1719),
    [sym___name__] = ACTIONS(1719),
    [sym___netlinx__] = ACTIONS(1719),
    [sym_netlinx_axi_version] = ACTIONS(1719),
    [sym_url_flg_tcp] = ACTIONS(1719),
    [sym_url_flg_accinfopresent] = ACTIONS(1719),
    [sym_url_flg_temp] = ACTIONS(1719),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1719),
    [sym_url_flg_stat_mask] = ACTIONS(1719),
    [sym_url_flg_stat_lookup] = ACTIONS(1719),
    [sym_url_flg_stat_connecting] = ACTIONS(1719),
    [sym_url_flg_stat_waiting] = ACTIONS(1719),
    [sym_url_flg_stat_connected] = ACTIONS(1719),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1719),
    [sym_first_virtual_device] = ACTIONS(1719),
    [sym_first_local_port] = ACTIONS(1719),
    [sym_dynamic_virtual_device] = ACTIONS(1719),
    [sym_timeline_once] = ACTIONS(1719),
    [sym_timeline_repeat] = ACTIONS(1719),
    [sym_timeline_absolute] = ACTIONS(1719),
    [sym_timeline_relative] = ACTIONS(1719),
    [sym_do_push_timed_infinite] = ACTIONS(1719),
    [sym_source_type_no_address] = ACTIONS(1719),
    [sym_source_type_neuron_id] = ACTIONS(1719),
    [sym_source_type_ip_address] = ACTIONS(1719),
    [sym_source_type_axlink] = ACTIONS(1719),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1719),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1719),
    [sym_source_type_ip_socket_address] = ACTIONS(1719),
    [sym_source_type_rs232] = ACTIONS(1719),
    [sym_source_type_internal] = ACTIONS(1719),
    [sym_source_type_mac_address] = ACTIONS(1719),
    [sym_source_type_ipv4_port] = ACTIONS(1719),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1719),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1719),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1719),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1719),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1719),
    [sym_true] = ACTIONS(1719),
    [sym_false] = ACTIONS(1719),
    [sym_file_read_only] = ACTIONS(1719),
    [sym_file_rw_new] = ACTIONS(1719),
    [sym_file_rw_append] = ACTIONS(1719),
    [sym_ip_tcp] = ACTIONS(1719),
    [sym_ip_udp] = ACTIONS(1719),
    [sym_ip_udp_2way] = ACTIONS(1719),
    [sym_xml_encode_types] = ACTIONS(1719),
    [sym_xml_encode_char_as_list] = ACTIONS(1719),
    [sym_xml_encode_le] = ACTIONS(1719),
    [sym_xml_decode_types] = ACTIONS(1719),
    [sym_xml_decode_no_preserve] = ACTIONS(1719),
    [sym_duet_dev_type_display_device] = ACTIONS(1719),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1719),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1719),
    [sym_duet_dev_type_io_device] = ACTIONS(1719),
    [sym_duet_dev_type_relay_device] = ACTIONS(1719),
    [sym_duet_dev_type_ups] = ACTIONS(1719),
    [sym_duet_dev_type_amplifier] = ACTIONS(1719),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1719),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1719),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1719),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1719),
    [sym_duet_dev_type_camera] = ACTIONS(1719),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1719),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1719),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1719),
    [sym_duet_dev_type_dss] = ACTIONS(1719),
    [sym_duet_dev_type_dvr] = ACTIONS(1719),
    [sym_duet_dev_type_disc_device] = ACTIONS(1719),
    [sym_duet_dev_type_document_camera] = ACTIONS(1719),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1719),
    [sym_duet_dev_type_hvac] = ACTIONS(1719),
    [sym_duet_dev_type_keypad] = ACTIONS(1719),
    [sym_duet_dev_type_light] = ACTIONS(1719),
    [sym_duet_dev_type_monitor] = ACTIONS(1719),
    [sym_duet_dev_type_motor] = ACTIONS(1719),
    [sym_duet_dev_type_multi_window] = ACTIONS(1719),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1719),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1719),
    [sym_duet_dev_type_receiver] = ACTIONS(1719),
    [sym_duet_dev_type_security_system] = ACTIONS(1719),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1719),
    [sym_duet_dev_type_settop_box] = ACTIONS(1719),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1719),
    [sym_duet_dev_type_switcher] = ACTIONS(1719),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1719),
    [sym_duet_dev_type_tv] = ACTIONS(1719),
    [sym_duet_dev_type_utility] = ACTIONS(1719),
    [sym_duet_dev_type_vcr] = ACTIONS(1719),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1719),
    [sym_duet_dev_type_video_processor] = ACTIONS(1719),
    [sym_duet_dev_type_video_projector] = ACTIONS(1719),
    [sym_duet_dev_type_video_wall] = ACTIONS(1719),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1719),
    [sym_duet_dev_type_weather] = ACTIONS(1719),
    [sym_duet_dev_not_polled] = ACTIONS(1719),
    [sym_duet_dev_polled] = ACTIONS(1719),
    [sym_clkmgr_mode_network] = ACTIONS(1719),
    [sym_clkmgr_mode_standalone] = ACTIONS(1719),
    [sym_normal_standby] = ACTIONS(1719),
    [sym_normal_wake] = ACTIONS(1719),
    [sym_ip_multicast_ttl_option] = ACTIONS(1719),
    [sym_ip_ttl_subnet] = ACTIONS(1719),
    [sym_ip_ttl_site] = ACTIONS(1719),
    [sym_ip_ttl_region] = ACTIONS(1719),
    [sym_ip_ttl_continent] = ACTIONS(1719),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1719),
    [sym_ip_nodelay_on] = ACTIONS(1719),
    [sym_ip_nodelay_off] = ACTIONS(1719),
    [sym_amx_error] = ACTIONS(1719),
    [sym_amx_warning] = ACTIONS(1719),
    [sym_amx_info] = ACTIONS(1719),
    [sym_amx_debug] = ACTIONS(1719),
    [sym_netlinx_login_success] = ACTIONS(1719),
    [sym_netlinx_login_fail] = ACTIONS(1719),
    [sym_netlinx_logout] = ACTIONS(1719),
    [sym_null_str] = ACTIONS(1719),
    [sym_smtp_address] = ACTIONS(1719),
    [sym_smtp_port_number] = ACTIONS(1719),
    [sym_smtp_username] = ACTIONS(1719),
    [sym_smtp_password] = ACTIONS(1719),
    [sym_smtp_require_tls] = ACTIONS(1719),
    [sym_smtp_from] = ACTIONS(1719),
    [sym_smtp_tls_true] = ACTIONS(1719),
    [sym_smtp_tls_false] = ACTIONS(1719),
    [sym_listview_on_row_select_event] = ACTIONS(1719),
    [sym_data_structure_unknown] = ACTIONS(1719),
    [sym_data_structure_datafeed] = ACTIONS(1719),
    [sym_data_structure_datarecordset] = ACTIONS(1719),
    [sym_data_structure_datarecord] = ACTIONS(1719),
    [sym_data_structure_datafield] = ACTIONS(1719),
    [sym_data_max_content_field_count] = ACTIONS(1719),
    [sym_data_max_metadata_field_count] = ACTIONS(1719),
    [sym_data_max_id_length] = ACTIONS(1719),
    [sym_data_max_name_length] = ACTIONS(1719),
    [sym_data_max_description_length] = ACTIONS(1719),
    [sym_data_max_type_length] = ACTIONS(1719),
    [sym_data_max_format_length] = ACTIONS(1719),
    [sym_data_max_source_length] = ACTIONS(1719),
    [sym_data_max_value_length] = ACTIONS(1719),
    [sym_data_max_label_length] = ACTIONS(1719),
    [sym_data_max_url_length] = ACTIONS(1719),
    [sym_max_last_login_info_length] = ACTIONS(1719),
    [sym_data_type_unknown] = ACTIONS(1719),
    [sym_data_type_string] = ACTIONS(1719),
    [sym_data_type_datetime] = ACTIONS(1719),
    [sym_data_type_date] = ACTIONS(1719),
    [sym_data_type_time] = ACTIONS(1719),
    [sym_data_type_image] = ACTIONS(1719),
    [sym_data_format_url] = ACTIONS(1719),
    [sym_data_format_phone] = ACTIONS(1719),
    [sym_data_format_email] = ACTIONS(1719),
    [sym_data_format_iso8601] = ACTIONS(1719),
    [sym_auditlog_priv] = ACTIONS(1719),
    [sym_configuration_priv] = ACTIONS(1719),
    [sym_deviceconfig_priv] = ACTIONS(1719),
    [sym_ftp_priv] = ACTIONS(1719),
    [sym_http_priv] = ACTIONS(1719),
    [sym_networkconfig_priv] = ACTIONS(1719),
    [sym_programport_priv] = ACTIONS(1719),
    [sym_remoteui_priv] = ACTIONS(1719),
    [sym_securitycontrol_priv] = ACTIONS(1719),
    [sym_softwaremanagement_priv] = ACTIONS(1719),
    [sym_terminal_priv] = ACTIONS(1719),
    [sym_tpadmin_priv] = ACTIONS(1719),
    [sym_user1_priv] = ACTIONS(1719),
    [sym_user2_priv] = ACTIONS(1719),
    [sym_user3_priv] = ACTIONS(1719),
    [sym_user4_priv] = ACTIONS(1719),
    [sym_usermanagement_priv] = ACTIONS(1719),
    [sym_valid_account] = ACTIONS(1719),
    [sym_err_name_invalid] = ACTIONS(1719),
    [sym_err_password_invalid] = ACTIONS(1719),
    [sym_err_invalid_account] = ACTIONS(1719),
    [sym_err_no_authorization] = ACTIONS(1719),
    [sym_err_invalid_parameter] = ACTIONS(1719),
    [sym_err_user_locked_out] = ACTIONS(1719),
    [sym_err_user_account_expired] = ACTIONS(1719),
    [sym_err_general_error] = ACTIONS(1719),
    [sym_err_authentication_error] = ACTIONS(1719),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1719),
    [sym_tls_validate_certificate] = ACTIONS(1719),
    [sym_tdata] = ACTIONS(1719),
    [sym_tchannel] = ACTIONS(1719),
    [sym_tlevel] = ACTIONS(1719),
    [sym_tbutton] = ACTIONS(1719),
    [sym_ttimeline] = ACTIONS(1719),
    [sym_tcustom] = ACTIONS(1719),
    [sym_url_struct] = ACTIONS(1719),
    [sym_dns_struct] = ACTIONS(1719),
    [sym_ip_address_struct] = ACTIONS(1719),
    [sym_dev_info_struct] = ACTIONS(1719),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1719),
    [sym_data_feed] = ACTIONS(1719),
    [sym_data_field] = ACTIONS(1719),
    [sym_data_record] = ACTIONS(1719),
    [sym_wc_data_feed] = ACTIONS(1719),
    [sym_wc_data_field] = ACTIONS(1719),
    [sym_wc_data_record] = ACTIONS(1719),
    [sym_last_login_info] = ACTIONS(1719),
    [sym____reserved___] = ACTIONS(1719),
    [sym_date] = ACTIONS(1719),
    [sym_day] = ACTIONS(1719),
    [sym_time] = ACTIONS(1719),
    [sym_ldate] = ACTIONS(1719),
    [sym_push_device] = ACTIONS(1719),
    [sym_push_channel] = ACTIONS(1719),
    [sym_push_devchan] = ACTIONS(1719),
    [sym_release_device] = ACTIONS(1719),
    [sym_release_channel] = ACTIONS(1719),
    [sym_release_devchan] = ACTIONS(1719),
    [sym_master_slot] = ACTIONS(1719),
    [sym_get_pulse_time] = ACTIONS(1719),
    [sym_get_timer] = ACTIONS(1719),
    [sym_master_sn] = ACTIONS(1719),
    [sym_system_number] = ACTIONS(1719),
    [sym_button] = ACTIONS(1719),
    [sym_dv_channel] = ACTIONS(1719),
    [sym_data] = ACTIONS(1719),
    [sym_level] = ACTIONS(1719),
    [sym_timeline] = ACTIONS(1719),
    [sym_channel] = ACTIONS(1719),
    [sym_custom] = ACTIONS(1719),
    [sym_atoi] = ACTIONS(1719),
    [sym_atol] = ACTIONS(1719),
    [sym_atof] = ACTIONS(1719),
    [sym_itoa] = ACTIONS(1719),
    [sym_itohex] = ACTIONS(1719),
    [sym_hextoi] = ACTIONS(1719),
    [sym_ftoa] = ACTIONS(1719),
    [sym_find_string] = ACTIONS(1719),
    [sym_left_string] = ACTIONS(1719),
    [sym_right_string] = ACTIONS(1719),
    [sym_mid_string] = ACTIONS(1719),
    [sym_remove_string] = ACTIONS(1719),
    [sym_get_buffer_string] = ACTIONS(1719),
    [sym_get_buffer_char] = ACTIONS(1719),
    [sym_get_multi_buffer_string] = ACTIONS(1719),
    [sym_length_string] = ACTIONS(1719),
    [sym_max_length_string] = ACTIONS(1719),
    [sym_set_length_string] = ACTIONS(1719),
    [sym_compare_string] = ACTIONS(1719),
    [sym_upper_string] = ACTIONS(1719),
    [sym_lower_string] = ACTIONS(1719),
    [sym_get_last] = ACTIONS(1719),
    [sym_date_to_day] = ACTIONS(1719),
    [sym_date_to_month] = ACTIONS(1719),
    [sym_date_to_year] = ACTIONS(1719),
    [sym_time_to_hour] = ACTIONS(1719),
    [sym_time_to_minute] = ACTIONS(1719),
    [sym_time_to_second] = ACTIONS(1719),
    [sym_day_of_week] = ACTIONS(1719),
    [sym_set_pulse_time] = ACTIONS(1719),
    [sym_set_timer] = ACTIONS(1719),
    [sym_file_open] = ACTIONS(1719),
    [sym_file_close] = ACTIONS(1719),
    [sym_file_read] = ACTIONS(1719),
    [sym_file_read_line] = ACTIONS(1719),
    [sym_file_write] = ACTIONS(1719),
    [sym_file_write_line] = ACTIONS(1719),
    [sym_file_copy] = ACTIONS(1719),
    [sym_file_delete] = ACTIONS(1719),
    [sym_file_rename] = ACTIONS(1719),
    [sym_file_dir] = ACTIONS(1719),
    [sym_file_getdir] = ACTIONS(1719),
    [sym_file_setdir] = ACTIONS(1719),
    [sym_file_create_dir] = ACTIONS(1719),
    [sym_file_remove_dir] = ACTIONS(1719),
    [sym_file_seek] = ACTIONS(1719),
    [sym_length_array] = ACTIONS(1719),
    [sym_max_length_array] = ACTIONS(1719),
    [sym_set_length_array] = ACTIONS(1719),
    [sym_ip_client_open] = ACTIONS(1719),
    [sym_ip_client_close] = ACTIONS(1719),
    [sym_ip_server_open] = ACTIONS(1719),
    [sym_ip_server_close] = ACTIONS(1719),
    [sym_ip_bound_client_open] = ACTIONS(1719),
    [sym_ip_mc_server_open] = ACTIONS(1719),
    [sym_ip_set_option] = ACTIONS(1719),
    [sym_ssh_client_open] = ACTIONS(1719),
    [sym_ssh_client_close] = ACTIONS(1719),
    [sym_tls_client_open] = ACTIONS(1719),
    [sym_tls_client_close] = ACTIONS(1719),
    [sym_do_push] = ACTIONS(1719),
    [sym_do_push_timed] = ACTIONS(1719),
    [sym_do_release] = ACTIONS(1719),
    [sym_device_id] = ACTIONS(1719),
    [sym_device_info] = ACTIONS(1719),
    [sym_device_id_string] = ACTIONS(1719),
    [sym_device_standby] = ACTIONS(1719),
    [sym_device_wake] = ACTIONS(1719),
    [sym_timeline_create] = ACTIONS(1719),
    [sym_timeline_kill] = ACTIONS(1719),
    [sym_timeline_reload] = ACTIONS(1719),
    [sym_timeline_pause] = ACTIONS(1719),
    [sym_timeline_restart] = ACTIONS(1719),
    [sym_timeline_set] = ACTIONS(1719),
    [sym_timeline_get] = ACTIONS(1719),
    [sym_timeline_active] = ACTIONS(1719),
    [sym_timeline_dynamic_id] = ACTIONS(1719),
    [sym_variable_to_string] = ACTIONS(1719),
    [sym_string_to_variable] = ACTIONS(1719),
    [sym_length_variable_to_string] = ACTIONS(1719),
    [sym_type_cast] = ACTIONS(1719),
    [sym_variable_to_xml] = ACTIONS(1719),
    [sym_xml_to_variable] = ACTIONS(1719),
    [sym_length_variable_to_xml] = ACTIONS(1719),
    [sym_redirect_string] = ACTIONS(1719),
    [sym_random_number] = ACTIONS(1719),
    [sym_get_unique_id] = ACTIONS(1719),
    [sym_get_system_number] = ACTIONS(1719),
    [sym_set_system_number] = ACTIONS(1719),
    [sym_get_serial_number] = ACTIONS(1719),
    [sym_get_master_build] = ACTIONS(1719),
    [sym_set_validation_code] = ACTIONS(1719),
    [sym_reboot] = ACTIONS(1719),
    [sym_load_duet_module] = ACTIONS(1719),
    [sym_unload_duet_module] = ACTIONS(1719),
    [sym_duet_mem_size_set] = ACTIONS(1719),
    [sym_duet_mem_size_get] = ACTIONS(1719),
    [sym_dynamic_application_device] = ACTIONS(1719),
    [sym_dynamic_polled_port] = ACTIONS(1719),
    [sym_static_port_binding] = ACTIONS(1719),
    [sym_static_ip_binding] = ACTIONS(1719),
    [sym_get_dns_list] = ACTIONS(1719),
    [sym_set_dns_list] = ACTIONS(1719),
    [sym_get_ip_address] = ACTIONS(1719),
    [sym_set_ip_address] = ACTIONS(1719),
    [sym_get_url_list] = ACTIONS(1719),
    [sym_add_url_entry] = ACTIONS(1719),
    [sym_delete_url_entry] = ACTIONS(1719),
    [sym_abs_value] = ACTIONS(1719),
    [sym_max_value] = ACTIONS(1719),
    [sym_min_value] = ACTIONS(1719),
    [sym_exp_value] = ACTIONS(1719),
    [sym_log_value] = ACTIONS(1719),
    [sym_log10_value] = ACTIONS(1719),
    [sym_power_value] = ACTIONS(1719),
    [sym_sqrt_value] = ACTIONS(1719),
    [sym_raw_be] = ACTIONS(1719),
    [sym_raw_le] = ACTIONS(1719),
    [sym_internal_queue_size_set] = ACTIONS(1719),
    [sym_internal_queue_size_get] = ACTIONS(1719),
    [sym_internal_threshold_set] = ACTIONS(1719),
    [sym_internal_threshold_get] = ACTIONS(1719),
    [sym_rebuilt_event] = ACTIONS(1719),
    [sym_do_custom_event] = ACTIONS(1719),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1719),
    [sym_clkmgr_set_clk_source] = ACTIONS(1719),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1719),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1719),
    [sym_clkmgr_get_timezone] = ACTIONS(1719),
    [sym_clkmgr_set_timezone] = ACTIONS(1719),
    [sym_clkmgr_get_resync_period] = ACTIONS(1719),
    [sym_clkmgr_set_resync_period] = ACTIONS(1719),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1719),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1719),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1719),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1719),
    [sym_clkmgr_get_timeservers] = ACTIONS(1719),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1719),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1719),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1719),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1719),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1719),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1719),
    [sym_set_log_level] = ACTIONS(1719),
    [sym_get_log_level] = ACTIONS(1719),
    [sym_amx_log] = ACTIONS(1719),
    [sym_smtp_server_config_set] = ACTIONS(1719),
    [sym_smtp_server_config_get] = ACTIONS(1719),
    [sym_smtp_send] = ACTIONS(1719),
    [sym_validate_netlinx_account] = ACTIONS(1719),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1719),
    [sym_authenticate_certificate] = ACTIONS(1719),
    [sym_audit_netlinx_session_event] = ACTIONS(1719),
    [sym_audit_netlinx_generic_event] = ACTIONS(1719),
    [sym_get_available_flash_disk_space] = ACTIONS(1719),
    [sym_get_max_flash_disk_space] = ACTIONS(1719),
    [sym_data_create_feed] = ACTIONS(1719),
    [sym_data_delete_feed] = ACTIONS(1719),
    [sym_data_publish_feed] = ACTIONS(1719),
    [sym_data_get_published_feed] = ACTIONS(1719),
    [sym_data_add_record] = ACTIONS(1719),
    [sym_data_get_event_record] = ACTIONS(1719),
    [sym__wc_data_create_feed] = ACTIONS(1719),
    [sym__wc_data_add_record] = ACTIONS(1719),
    [sym__wc_data_get_event_record] = ACTIONS(1719),
    [sym_format] = ACTIONS(1719),
    [sym_astro_clock] = ACTIONS(1719),
    [sym_set_outdoor_temperature] = ACTIONS(1719),
    [sym_set_virtual_level_count] = ACTIONS(1719),
    [sym_set_virtual_channel_count] = ACTIONS(1719),
    [sym_set_virtual_port_count] = ACTIONS(1719),
    [sym__wc_encode_int] = ACTIONS(1719),
    [sym__wc_decode_int] = ACTIONS(1719),
    [sym__wc_int] = ACTIONS(1719),
    [sym__wc_to_ch_int] = ACTIONS(1719),
    [sym__ch_to_wc_int] = ACTIONS(1719),
    [sym__wc_find_string_int] = ACTIONS(1719),
    [sym__wc_left_string_int] = ACTIONS(1719),
    [sym__wc_lower_string_int] = ACTIONS(1719),
    [sym__wc_mid_string_int] = ACTIONS(1719),
    [sym__wc_remove_string_int] = ACTIONS(1719),
    [sym__wc_right_string_int] = ACTIONS(1719),
    [sym__wc_upper_string_int] = ACTIONS(1719),
    [sym__wc_compare_string_int] = ACTIONS(1719),
    [sym__wc_get_buffer_char_int] = ACTIONS(1719),
    [sym__wc_get_buffer_string_int] = ACTIONS(1719),
    [sym__wc_concat_string_int] = ACTIONS(1719),
    [sym__wc_file_open_int] = ACTIONS(1719),
    [sym__wc_file_close_int] = ACTIONS(1719),
    [sym__wc_file_read_int] = ACTIONS(1719),
    [sym__wc_file_read_line_int] = ACTIONS(1719),
    [sym__wc_file_write_int] = ACTIONS(1719),
    [sym__wc_file_write_line_int] = ACTIONS(1719),
    [aux_sym_preproc_include_token1] = ACTIONS(1719),
    [aux_sym_preproc_define_token1] = ACTIONS(1721),
    [aux_sym_preproc_warn_token1] = ACTIONS(1721),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1721),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1721),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1721),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1721),
    [aux_sym_preproc_else_token1] = ACTIONS(1721),
    [anon_sym_LBRACK] = ACTIONS(1721),
    [anon_sym_LBRACE] = ACTIONS(1721),
    [anon_sym_LPAREN] = ACTIONS(1719),
    [anon_sym_BANG] = ACTIONS(1721),
    [anon_sym_TILDE] = ACTIONS(1721),
    [anon_sym_DASH] = ACTIONS(1719),
    [anon_sym_PLUS] = ACTIONS(1719),
    [anon_sym_DASH_DASH] = ACTIONS(1721),
    [anon_sym_PLUS_PLUS] = ACTIONS(1721),
    [anon_sym_DQUOTE] = ACTIONS(1721),
    [anon_sym_SQUOTE] = ACTIONS(1721),
    [sym_number_literal] = ACTIONS(1721),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(210)] = {
    [sym_identifier] = ACTIONS(1723),
    [sym_program_name_keyword] = ACTIONS(1723),
    [sym_module_name_keyword] = ACTIONS(1723),
    [sym_define_device_keyword] = ACTIONS(1723),
    [sym_define_combine_keyword] = ACTIONS(1723),
    [sym_define_constant_keyword] = ACTIONS(1723),
    [sym_define_type_keyword] = ACTIONS(1723),
    [sym_define_variable_keyword] = ACTIONS(1723),
    [sym_define_system_variable_keyword] = ACTIONS(1723),
    [sym_define_connect_level_keyword] = ACTIONS(1723),
    [sym_define_latching_keyword] = ACTIONS(1723),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1723),
    [sym_define_toggling_keyword] = ACTIONS(1723),
    [sym_define_call_keyword] = ACTIONS(1723),
    [sym_define_function_keyword] = ACTIONS(1723),
    [sym_define_library_function_keyword] = ACTIONS(1723),
    [sym_define_start_keyword] = ACTIONS(1723),
    [sym_define_event_keyword] = ACTIONS(1723),
    [sym_define_program_keyword] = ACTIONS(1723),
    [sym_define_module_keyword] = ACTIONS(1723),
    [sym_char_keyword] = ACTIONS(1723),
    [sym_widechar_keyword] = ACTIONS(1723),
    [sym_integer_keyword] = ACTIONS(1723),
    [sym_sinteger_keyword] = ACTIONS(1723),
    [sym_long_keyword] = ACTIONS(1723),
    [sym_slong_keyword] = ACTIONS(1723),
    [sym_float_keyword] = ACTIONS(1723),
    [sym_double_keyword] = ACTIONS(1723),
    [sym_variant_keyword] = ACTIONS(1723),
    [sym_variantarray_keyword] = ACTIONS(1723),
    [sym_local_var_keyword] = ACTIONS(1723),
    [sym_stack_var_keyword] = ACTIONS(1723),
    [sym_constant_keyword] = ACTIONS(1723),
    [sym_volatile_keyword] = ACTIONS(1723),
    [sym_non_volatile_keyword] = ACTIONS(1723),
    [sym_persistent_keyword] = ACTIONS(1723),
    [sym_dev_keyword] = ACTIONS(1723),
    [sym_devlev_keyword] = ACTIONS(1723),
    [sym_devchan_keyword] = ACTIONS(1723),
    [sym_if_keyword] = ACTIONS(1723),
    [sym_else_keyword] = ACTIONS(1723),
    [sym_switch_keyword] = ACTIONS(1723),
    [sym_case_keyword] = ACTIONS(1723),
    [sym_default_keyword] = ACTIONS(1723),
    [sym_while_keyword] = ACTIONS(1723),
    [sym_for_keyword] = ACTIONS(1723),
    [sym_break_keyword] = ACTIONS(1723),
    [sym_continue_keyword] = ACTIONS(1723),
    [sym_return_keyword] = ACTIONS(1723),
    [sym_select_keyword] = ACTIONS(1723),
    [sym_active_keyword] = ACTIONS(1723),
    [sym_struct_keyword] = ACTIONS(1723),
    [sym_structure_keyword] = ACTIONS(1723),
    [sym_button_event_keyword] = ACTIONS(1723),
    [sym_channel_event_keyword] = ACTIONS(1723),
    [sym_level_event_keyword] = ACTIONS(1723),
    [sym_data_event_keyword] = ACTIONS(1723),
    [sym_timeline_event_keyword] = ACTIONS(1723),
    [sym_custom_event_keyword] = ACTIONS(1723),
    [sym_push_keyword] = ACTIONS(1723),
    [sym_release_keyword] = ACTIONS(1723),
    [aux_sym_on_keyword_token1] = ACTIONS(1723),
    [aux_sym_off_keyword_token1] = ACTIONS(1723),
    [sym_send_level_keyword] = ACTIONS(1723),
    [sym_send_string_keyword] = ACTIONS(1723),
    [sym_send_command_keyword] = ACTIONS(1723),
    [sym_clear_buffer_keyword] = ACTIONS(1723),
    [sym_create_buffer_keyword] = ACTIONS(1723),
    [sym_create_multi_buffer_keyword] = ACTIONS(1723),
    [sym_call_keyword] = ACTIONS(1723),
    [sym_system_call_keyword] = ACTIONS(1723),
    [sym_devchan_to_keyword] = ACTIONS(1723),
    [sym_devchan_min_to_keyword] = ACTIONS(1723),
    [sym_devchan_total_off_keyword] = ACTIONS(1723),
    [sym_devchan_pulse_keyword] = ACTIONS(1723),
    [sym_wait_keyword] = ACTIONS(1723),
    [sym_cancel_wait_keyword] = ACTIONS(1723),
    [sym_wait_until_keyword] = ACTIONS(1723),
    [sym_cancel_wait_until_keyword] = ACTIONS(1723),
    [sym_cancel_all_wait_keyword] = ACTIONS(1723),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1723),
    [sym_not] = ACTIONS(1723),
    [sym_bnot] = ACTIONS(1723),
    [sym___file__] = ACTIONS(1723),
    [sym___date__] = ACTIONS(1723),
    [sym___ldate__] = ACTIONS(1723),
    [sym___time__] = ACTIONS(1723),
    [sym___line__] = ACTIONS(1723),
    [sym___name__] = ACTIONS(1723),
    [sym___netlinx__] = ACTIONS(1723),
    [sym_netlinx_axi_version] = ACTIONS(1723),
    [sym_url_flg_tcp] = ACTIONS(1723),
    [sym_url_flg_accinfopresent] = ACTIONS(1723),
    [sym_url_flg_temp] = ACTIONS(1723),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1723),
    [sym_url_flg_stat_mask] = ACTIONS(1723),
    [sym_url_flg_stat_lookup] = ACTIONS(1723),
    [sym_url_flg_stat_connecting] = ACTIONS(1723),
    [sym_url_flg_stat_waiting] = ACTIONS(1723),
    [sym_url_flg_stat_connected] = ACTIONS(1723),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1723),
    [sym_first_virtual_device] = ACTIONS(1723),
    [sym_first_local_port] = ACTIONS(1723),
    [sym_dynamic_virtual_device] = ACTIONS(1723),
    [sym_timeline_once] = ACTIONS(1723),
    [sym_timeline_repeat] = ACTIONS(1723),
    [sym_timeline_absolute] = ACTIONS(1723),
    [sym_timeline_relative] = ACTIONS(1723),
    [sym_do_push_timed_infinite] = ACTIONS(1723),
    [sym_source_type_no_address] = ACTIONS(1723),
    [sym_source_type_neuron_id] = ACTIONS(1723),
    [sym_source_type_ip_address] = ACTIONS(1723),
    [sym_source_type_axlink] = ACTIONS(1723),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1723),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1723),
    [sym_source_type_ip_socket_address] = ACTIONS(1723),
    [sym_source_type_rs232] = ACTIONS(1723),
    [sym_source_type_internal] = ACTIONS(1723),
    [sym_source_type_mac_address] = ACTIONS(1723),
    [sym_source_type_ipv4_port] = ACTIONS(1723),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1723),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1723),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1723),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1723),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1723),
    [sym_true] = ACTIONS(1723),
    [sym_false] = ACTIONS(1723),
    [sym_file_read_only] = ACTIONS(1723),
    [sym_file_rw_new] = ACTIONS(1723),
    [sym_file_rw_append] = ACTIONS(1723),
    [sym_ip_tcp] = ACTIONS(1723),
    [sym_ip_udp] = ACTIONS(1723),
    [sym_ip_udp_2way] = ACTIONS(1723),
    [sym_xml_encode_types] = ACTIONS(1723),
    [sym_xml_encode_char_as_list] = ACTIONS(1723),
    [sym_xml_encode_le] = ACTIONS(1723),
    [sym_xml_decode_types] = ACTIONS(1723),
    [sym_xml_decode_no_preserve] = ACTIONS(1723),
    [sym_duet_dev_type_display_device] = ACTIONS(1723),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1723),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1723),
    [sym_duet_dev_type_io_device] = ACTIONS(1723),
    [sym_duet_dev_type_relay_device] = ACTIONS(1723),
    [sym_duet_dev_type_ups] = ACTIONS(1723),
    [sym_duet_dev_type_amplifier] = ACTIONS(1723),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1723),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1723),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1723),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1723),
    [sym_duet_dev_type_camera] = ACTIONS(1723),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1723),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1723),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1723),
    [sym_duet_dev_type_dss] = ACTIONS(1723),
    [sym_duet_dev_type_dvr] = ACTIONS(1723),
    [sym_duet_dev_type_disc_device] = ACTIONS(1723),
    [sym_duet_dev_type_document_camera] = ACTIONS(1723),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1723),
    [sym_duet_dev_type_hvac] = ACTIONS(1723),
    [sym_duet_dev_type_keypad] = ACTIONS(1723),
    [sym_duet_dev_type_light] = ACTIONS(1723),
    [sym_duet_dev_type_monitor] = ACTIONS(1723),
    [sym_duet_dev_type_motor] = ACTIONS(1723),
    [sym_duet_dev_type_multi_window] = ACTIONS(1723),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1723),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1723),
    [sym_duet_dev_type_receiver] = ACTIONS(1723),
    [sym_duet_dev_type_security_system] = ACTIONS(1723),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1723),
    [sym_duet_dev_type_settop_box] = ACTIONS(1723),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1723),
    [sym_duet_dev_type_switcher] = ACTIONS(1723),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1723),
    [sym_duet_dev_type_tv] = ACTIONS(1723),
    [sym_duet_dev_type_utility] = ACTIONS(1723),
    [sym_duet_dev_type_vcr] = ACTIONS(1723),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1723),
    [sym_duet_dev_type_video_processor] = ACTIONS(1723),
    [sym_duet_dev_type_video_projector] = ACTIONS(1723),
    [sym_duet_dev_type_video_wall] = ACTIONS(1723),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1723),
    [sym_duet_dev_type_weather] = ACTIONS(1723),
    [sym_duet_dev_not_polled] = ACTIONS(1723),
    [sym_duet_dev_polled] = ACTIONS(1723),
    [sym_clkmgr_mode_network] = ACTIONS(1723),
    [sym_clkmgr_mode_standalone] = ACTIONS(1723),
    [sym_normal_standby] = ACTIONS(1723),
    [sym_normal_wake] = ACTIONS(1723),
    [sym_ip_multicast_ttl_option] = ACTIONS(1723),
    [sym_ip_ttl_subnet] = ACTIONS(1723),
    [sym_ip_ttl_site] = ACTIONS(1723),
    [sym_ip_ttl_region] = ACTIONS(1723),
    [sym_ip_ttl_continent] = ACTIONS(1723),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1723),
    [sym_ip_nodelay_on] = ACTIONS(1723),
    [sym_ip_nodelay_off] = ACTIONS(1723),
    [sym_amx_error] = ACTIONS(1723),
    [sym_amx_warning] = ACTIONS(1723),
    [sym_amx_info] = ACTIONS(1723),
    [sym_amx_debug] = ACTIONS(1723),
    [sym_netlinx_login_success] = ACTIONS(1723),
    [sym_netlinx_login_fail] = ACTIONS(1723),
    [sym_netlinx_logout] = ACTIONS(1723),
    [sym_null_str] = ACTIONS(1723),
    [sym_smtp_address] = ACTIONS(1723),
    [sym_smtp_port_number] = ACTIONS(1723),
    [sym_smtp_username] = ACTIONS(1723),
    [sym_smtp_password] = ACTIONS(1723),
    [sym_smtp_require_tls] = ACTIONS(1723),
    [sym_smtp_from] = ACTIONS(1723),
    [sym_smtp_tls_true] = ACTIONS(1723),
    [sym_smtp_tls_false] = ACTIONS(1723),
    [sym_listview_on_row_select_event] = ACTIONS(1723),
    [sym_data_structure_unknown] = ACTIONS(1723),
    [sym_data_structure_datafeed] = ACTIONS(1723),
    [sym_data_structure_datarecordset] = ACTIONS(1723),
    [sym_data_structure_datarecord] = ACTIONS(1723),
    [sym_data_structure_datafield] = ACTIONS(1723),
    [sym_data_max_content_field_count] = ACTIONS(1723),
    [sym_data_max_metadata_field_count] = ACTIONS(1723),
    [sym_data_max_id_length] = ACTIONS(1723),
    [sym_data_max_name_length] = ACTIONS(1723),
    [sym_data_max_description_length] = ACTIONS(1723),
    [sym_data_max_type_length] = ACTIONS(1723),
    [sym_data_max_format_length] = ACTIONS(1723),
    [sym_data_max_source_length] = ACTIONS(1723),
    [sym_data_max_value_length] = ACTIONS(1723),
    [sym_data_max_label_length] = ACTIONS(1723),
    [sym_data_max_url_length] = ACTIONS(1723),
    [sym_max_last_login_info_length] = ACTIONS(1723),
    [sym_data_type_unknown] = ACTIONS(1723),
    [sym_data_type_string] = ACTIONS(1723),
    [sym_data_type_datetime] = ACTIONS(1723),
    [sym_data_type_date] = ACTIONS(1723),
    [sym_data_type_time] = ACTIONS(1723),
    [sym_data_type_image] = ACTIONS(1723),
    [sym_data_format_url] = ACTIONS(1723),
    [sym_data_format_phone] = ACTIONS(1723),
    [sym_data_format_email] = ACTIONS(1723),
    [sym_data_format_iso8601] = ACTIONS(1723),
    [sym_auditlog_priv] = ACTIONS(1723),
    [sym_configuration_priv] = ACTIONS(1723),
    [sym_deviceconfig_priv] = ACTIONS(1723),
    [sym_ftp_priv] = ACTIONS(1723),
    [sym_http_priv] = ACTIONS(1723),
    [sym_networkconfig_priv] = ACTIONS(1723),
    [sym_programport_priv] = ACTIONS(1723),
    [sym_remoteui_priv] = ACTIONS(1723),
    [sym_securitycontrol_priv] = ACTIONS(1723),
    [sym_softwaremanagement_priv] = ACTIONS(1723),
    [sym_terminal_priv] = ACTIONS(1723),
    [sym_tpadmin_priv] = ACTIONS(1723),
    [sym_user1_priv] = ACTIONS(1723),
    [sym_user2_priv] = ACTIONS(1723),
    [sym_user3_priv] = ACTIONS(1723),
    [sym_user4_priv] = ACTIONS(1723),
    [sym_usermanagement_priv] = ACTIONS(1723),
    [sym_valid_account] = ACTIONS(1723),
    [sym_err_name_invalid] = ACTIONS(1723),
    [sym_err_password_invalid] = ACTIONS(1723),
    [sym_err_invalid_account] = ACTIONS(1723),
    [sym_err_no_authorization] = ACTIONS(1723),
    [sym_err_invalid_parameter] = ACTIONS(1723),
    [sym_err_user_locked_out] = ACTIONS(1723),
    [sym_err_user_account_expired] = ACTIONS(1723),
    [sym_err_general_error] = ACTIONS(1723),
    [sym_err_authentication_error] = ACTIONS(1723),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1723),
    [sym_tls_validate_certificate] = ACTIONS(1723),
    [sym_tdata] = ACTIONS(1723),
    [sym_tchannel] = ACTIONS(1723),
    [sym_tlevel] = ACTIONS(1723),
    [sym_tbutton] = ACTIONS(1723),
    [sym_ttimeline] = ACTIONS(1723),
    [sym_tcustom] = ACTIONS(1723),
    [sym_url_struct] = ACTIONS(1723),
    [sym_dns_struct] = ACTIONS(1723),
    [sym_ip_address_struct] = ACTIONS(1723),
    [sym_dev_info_struct] = ACTIONS(1723),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1723),
    [sym_data_feed] = ACTIONS(1723),
    [sym_data_field] = ACTIONS(1723),
    [sym_data_record] = ACTIONS(1723),
    [sym_wc_data_feed] = ACTIONS(1723),
    [sym_wc_data_field] = ACTIONS(1723),
    [sym_wc_data_record] = ACTIONS(1723),
    [sym_last_login_info] = ACTIONS(1723),
    [sym____reserved___] = ACTIONS(1723),
    [sym_date] = ACTIONS(1723),
    [sym_day] = ACTIONS(1723),
    [sym_time] = ACTIONS(1723),
    [sym_ldate] = ACTIONS(1723),
    [sym_push_device] = ACTIONS(1723),
    [sym_push_channel] = ACTIONS(1723),
    [sym_push_devchan] = ACTIONS(1723),
    [sym_release_device] = ACTIONS(1723),
    [sym_release_channel] = ACTIONS(1723),
    [sym_release_devchan] = ACTIONS(1723),
    [sym_master_slot] = ACTIONS(1723),
    [sym_get_pulse_time] = ACTIONS(1723),
    [sym_get_timer] = ACTIONS(1723),
    [sym_master_sn] = ACTIONS(1723),
    [sym_system_number] = ACTIONS(1723),
    [sym_button] = ACTIONS(1723),
    [sym_dv_channel] = ACTIONS(1723),
    [sym_data] = ACTIONS(1723),
    [sym_level] = ACTIONS(1723),
    [sym_timeline] = ACTIONS(1723),
    [sym_channel] = ACTIONS(1723),
    [sym_custom] = ACTIONS(1723),
    [sym_atoi] = ACTIONS(1723),
    [sym_atol] = ACTIONS(1723),
    [sym_atof] = ACTIONS(1723),
    [sym_itoa] = ACTIONS(1723),
    [sym_itohex] = ACTIONS(1723),
    [sym_hextoi] = ACTIONS(1723),
    [sym_ftoa] = ACTIONS(1723),
    [sym_find_string] = ACTIONS(1723),
    [sym_left_string] = ACTIONS(1723),
    [sym_right_string] = ACTIONS(1723),
    [sym_mid_string] = ACTIONS(1723),
    [sym_remove_string] = ACTIONS(1723),
    [sym_get_buffer_string] = ACTIONS(1723),
    [sym_get_buffer_char] = ACTIONS(1723),
    [sym_get_multi_buffer_string] = ACTIONS(1723),
    [sym_length_string] = ACTIONS(1723),
    [sym_max_length_string] = ACTIONS(1723),
    [sym_set_length_string] = ACTIONS(1723),
    [sym_compare_string] = ACTIONS(1723),
    [sym_upper_string] = ACTIONS(1723),
    [sym_lower_string] = ACTIONS(1723),
    [sym_get_last] = ACTIONS(1723),
    [sym_date_to_day] = ACTIONS(1723),
    [sym_date_to_month] = ACTIONS(1723),
    [sym_date_to_year] = ACTIONS(1723),
    [sym_time_to_hour] = ACTIONS(1723),
    [sym_time_to_minute] = ACTIONS(1723),
    [sym_time_to_second] = ACTIONS(1723),
    [sym_day_of_week] = ACTIONS(1723),
    [sym_set_pulse_time] = ACTIONS(1723),
    [sym_set_timer] = ACTIONS(1723),
    [sym_file_open] = ACTIONS(1723),
    [sym_file_close] = ACTIONS(1723),
    [sym_file_read] = ACTIONS(1723),
    [sym_file_read_line] = ACTIONS(1723),
    [sym_file_write] = ACTIONS(1723),
    [sym_file_write_line] = ACTIONS(1723),
    [sym_file_copy] = ACTIONS(1723),
    [sym_file_delete] = ACTIONS(1723),
    [sym_file_rename] = ACTIONS(1723),
    [sym_file_dir] = ACTIONS(1723),
    [sym_file_getdir] = ACTIONS(1723),
    [sym_file_setdir] = ACTIONS(1723),
    [sym_file_create_dir] = ACTIONS(1723),
    [sym_file_remove_dir] = ACTIONS(1723),
    [sym_file_seek] = ACTIONS(1723),
    [sym_length_array] = ACTIONS(1723),
    [sym_max_length_array] = ACTIONS(1723),
    [sym_set_length_array] = ACTIONS(1723),
    [sym_ip_client_open] = ACTIONS(1723),
    [sym_ip_client_close] = ACTIONS(1723),
    [sym_ip_server_open] = ACTIONS(1723),
    [sym_ip_server_close] = ACTIONS(1723),
    [sym_ip_bound_client_open] = ACTIONS(1723),
    [sym_ip_mc_server_open] = ACTIONS(1723),
    [sym_ip_set_option] = ACTIONS(1723),
    [sym_ssh_client_open] = ACTIONS(1723),
    [sym_ssh_client_close] = ACTIONS(1723),
    [sym_tls_client_open] = ACTIONS(1723),
    [sym_tls_client_close] = ACTIONS(1723),
    [sym_do_push] = ACTIONS(1723),
    [sym_do_push_timed] = ACTIONS(1723),
    [sym_do_release] = ACTIONS(1723),
    [sym_device_id] = ACTIONS(1723),
    [sym_device_info] = ACTIONS(1723),
    [sym_device_id_string] = ACTIONS(1723),
    [sym_device_standby] = ACTIONS(1723),
    [sym_device_wake] = ACTIONS(1723),
    [sym_timeline_create] = ACTIONS(1723),
    [sym_timeline_kill] = ACTIONS(1723),
    [sym_timeline_reload] = ACTIONS(1723),
    [sym_timeline_pause] = ACTIONS(1723),
    [sym_timeline_restart] = ACTIONS(1723),
    [sym_timeline_set] = ACTIONS(1723),
    [sym_timeline_get] = ACTIONS(1723),
    [sym_timeline_active] = ACTIONS(1723),
    [sym_timeline_dynamic_id] = ACTIONS(1723),
    [sym_variable_to_string] = ACTIONS(1723),
    [sym_string_to_variable] = ACTIONS(1723),
    [sym_length_variable_to_string] = ACTIONS(1723),
    [sym_type_cast] = ACTIONS(1723),
    [sym_variable_to_xml] = ACTIONS(1723),
    [sym_xml_to_variable] = ACTIONS(1723),
    [sym_length_variable_to_xml] = ACTIONS(1723),
    [sym_redirect_string] = ACTIONS(1723),
    [sym_random_number] = ACTIONS(1723),
    [sym_get_unique_id] = ACTIONS(1723),
    [sym_get_system_number] = ACTIONS(1723),
    [sym_set_system_number] = ACTIONS(1723),
    [sym_get_serial_number] = ACTIONS(1723),
    [sym_get_master_build] = ACTIONS(1723),
    [sym_set_validation_code] = ACTIONS(1723),
    [sym_reboot] = ACTIONS(1723),
    [sym_load_duet_module] = ACTIONS(1723),
    [sym_unload_duet_module] = ACTIONS(1723),
    [sym_duet_mem_size_set] = ACTIONS(1723),
    [sym_duet_mem_size_get] = ACTIONS(1723),
    [sym_dynamic_application_device] = ACTIONS(1723),
    [sym_dynamic_polled_port] = ACTIONS(1723),
    [sym_static_port_binding] = ACTIONS(1723),
    [sym_static_ip_binding] = ACTIONS(1723),
    [sym_get_dns_list] = ACTIONS(1723),
    [sym_set_dns_list] = ACTIONS(1723),
    [sym_get_ip_address] = ACTIONS(1723),
    [sym_set_ip_address] = ACTIONS(1723),
    [sym_get_url_list] = ACTIONS(1723),
    [sym_add_url_entry] = ACTIONS(1723),
    [sym_delete_url_entry] = ACTIONS(1723),
    [sym_abs_value] = ACTIONS(1723),
    [sym_max_value] = ACTIONS(1723),
    [sym_min_value] = ACTIONS(1723),
    [sym_exp_value] = ACTIONS(1723),
    [sym_log_value] = ACTIONS(1723),
    [sym_log10_value] = ACTIONS(1723),
    [sym_power_value] = ACTIONS(1723),
    [sym_sqrt_value] = ACTIONS(1723),
    [sym_raw_be] = ACTIONS(1723),
    [sym_raw_le] = ACTIONS(1723),
    [sym_internal_queue_size_set] = ACTIONS(1723),
    [sym_internal_queue_size_get] = ACTIONS(1723),
    [sym_internal_threshold_set] = ACTIONS(1723),
    [sym_internal_threshold_get] = ACTIONS(1723),
    [sym_rebuilt_event] = ACTIONS(1723),
    [sym_do_custom_event] = ACTIONS(1723),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1723),
    [sym_clkmgr_set_clk_source] = ACTIONS(1723),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1723),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1723),
    [sym_clkmgr_get_timezone] = ACTIONS(1723),
    [sym_clkmgr_set_timezone] = ACTIONS(1723),
    [sym_clkmgr_get_resync_period] = ACTIONS(1723),
    [sym_clkmgr_set_resync_period] = ACTIONS(1723),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1723),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1723),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1723),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1723),
    [sym_clkmgr_get_timeservers] = ACTIONS(1723),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1723),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1723),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1723),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1723),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1723),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1723),
    [sym_set_log_level] = ACTIONS(1723),
    [sym_get_log_level] = ACTIONS(1723),
    [sym_amx_log] = ACTIONS(1723),
    [sym_smtp_server_config_set] = ACTIONS(1723),
    [sym_smtp_server_config_get] = ACTIONS(1723),
    [sym_smtp_send] = ACTIONS(1723),
    [sym_validate_netlinx_account] = ACTIONS(1723),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1723),
    [sym_authenticate_certificate] = ACTIONS(1723),
    [sym_audit_netlinx_session_event] = ACTIONS(1723),
    [sym_audit_netlinx_generic_event] = ACTIONS(1723),
    [sym_get_available_flash_disk_space] = ACTIONS(1723),
    [sym_get_max_flash_disk_space] = ACTIONS(1723),
    [sym_data_create_feed] = ACTIONS(1723),
    [sym_data_delete_feed] = ACTIONS(1723),
    [sym_data_publish_feed] = ACTIONS(1723),
    [sym_data_get_published_feed] = ACTIONS(1723),
    [sym_data_add_record] = ACTIONS(1723),
    [sym_data_get_event_record] = ACTIONS(1723),
    [sym__wc_data_create_feed] = ACTIONS(1723),
    [sym__wc_data_add_record] = ACTIONS(1723),
    [sym__wc_data_get_event_record] = ACTIONS(1723),
    [sym_format] = ACTIONS(1723),
    [sym_astro_clock] = ACTIONS(1723),
    [sym_set_outdoor_temperature] = ACTIONS(1723),
    [sym_set_virtual_level_count] = ACTIONS(1723),
    [sym_set_virtual_channel_count] = ACTIONS(1723),
    [sym_set_virtual_port_count] = ACTIONS(1723),
    [sym__wc_encode_int] = ACTIONS(1723),
    [sym__wc_decode_int] = ACTIONS(1723),
    [sym__wc_int] = ACTIONS(1723),
    [sym__wc_to_ch_int] = ACTIONS(1723),
    [sym__ch_to_wc_int] = ACTIONS(1723),
    [sym__wc_find_string_int] = ACTIONS(1723),
    [sym__wc_left_string_int] = ACTIONS(1723),
    [sym__wc_lower_string_int] = ACTIONS(1723),
    [sym__wc_mid_string_int] = ACTIONS(1723),
    [sym__wc_remove_string_int] = ACTIONS(1723),
    [sym__wc_right_string_int] = ACTIONS(1723),
    [sym__wc_upper_string_int] = ACTIONS(1723),
    [sym__wc_compare_string_int] = ACTIONS(1723),
    [sym__wc_get_buffer_char_int] = ACTIONS(1723),
    [sym__wc_get_buffer_string_int] = ACTIONS(1723),
    [sym__wc_concat_string_int] = ACTIONS(1723),
    [sym__wc_file_open_int] = ACTIONS(1723),
    [sym__wc_file_close_int] = ACTIONS(1723),
    [sym__wc_file_read_int] = ACTIONS(1723),
    [sym__wc_file_read_line_int] = ACTIONS(1723),
    [sym__wc_file_write_int] = ACTIONS(1723),
    [sym__wc_file_write_line_int] = ACTIONS(1723),
    [aux_sym_preproc_include_token1] = ACTIONS(1723),
    [aux_sym_preproc_define_token1] = ACTIONS(1725),
    [aux_sym_preproc_warn_token1] = ACTIONS(1725),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1725),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1725),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1725),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1725),
    [aux_sym_preproc_else_token1] = ACTIONS(1725),
    [anon_sym_LBRACK] = ACTIONS(1725),
    [anon_sym_LBRACE] = ACTIONS(1725),
    [anon_sym_LPAREN] = ACTIONS(1723),
    [anon_sym_BANG] = ACTIONS(1725),
    [anon_sym_TILDE] = ACTIONS(1725),
    [anon_sym_DASH] = ACTIONS(1723),
    [anon_sym_PLUS] = ACTIONS(1723),
    [anon_sym_DASH_DASH] = ACTIONS(1725),
    [anon_sym_PLUS_PLUS] = ACTIONS(1725),
    [anon_sym_DQUOTE] = ACTIONS(1725),
    [anon_sym_SQUOTE] = ACTIONS(1725),
    [sym_number_literal] = ACTIONS(1725),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(211)] = {
    [sym_identifier] = ACTIONS(1727),
    [sym_program_name_keyword] = ACTIONS(1727),
    [sym_module_name_keyword] = ACTIONS(1727),
    [sym_define_device_keyword] = ACTIONS(1727),
    [sym_define_combine_keyword] = ACTIONS(1727),
    [sym_define_constant_keyword] = ACTIONS(1727),
    [sym_define_type_keyword] = ACTIONS(1727),
    [sym_define_variable_keyword] = ACTIONS(1727),
    [sym_define_system_variable_keyword] = ACTIONS(1727),
    [sym_define_connect_level_keyword] = ACTIONS(1727),
    [sym_define_latching_keyword] = ACTIONS(1727),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1727),
    [sym_define_toggling_keyword] = ACTIONS(1727),
    [sym_define_call_keyword] = ACTIONS(1727),
    [sym_define_function_keyword] = ACTIONS(1727),
    [sym_define_library_function_keyword] = ACTIONS(1727),
    [sym_define_start_keyword] = ACTIONS(1727),
    [sym_define_event_keyword] = ACTIONS(1727),
    [sym_define_program_keyword] = ACTIONS(1727),
    [sym_define_module_keyword] = ACTIONS(1727),
    [sym_char_keyword] = ACTIONS(1727),
    [sym_widechar_keyword] = ACTIONS(1727),
    [sym_integer_keyword] = ACTIONS(1727),
    [sym_sinteger_keyword] = ACTIONS(1727),
    [sym_long_keyword] = ACTIONS(1727),
    [sym_slong_keyword] = ACTIONS(1727),
    [sym_float_keyword] = ACTIONS(1727),
    [sym_double_keyword] = ACTIONS(1727),
    [sym_variant_keyword] = ACTIONS(1727),
    [sym_variantarray_keyword] = ACTIONS(1727),
    [sym_local_var_keyword] = ACTIONS(1727),
    [sym_stack_var_keyword] = ACTIONS(1727),
    [sym_constant_keyword] = ACTIONS(1727),
    [sym_volatile_keyword] = ACTIONS(1727),
    [sym_non_volatile_keyword] = ACTIONS(1727),
    [sym_persistent_keyword] = ACTIONS(1727),
    [sym_dev_keyword] = ACTIONS(1727),
    [sym_devlev_keyword] = ACTIONS(1727),
    [sym_devchan_keyword] = ACTIONS(1727),
    [sym_if_keyword] = ACTIONS(1727),
    [sym_else_keyword] = ACTIONS(1727),
    [sym_switch_keyword] = ACTIONS(1727),
    [sym_case_keyword] = ACTIONS(1727),
    [sym_default_keyword] = ACTIONS(1727),
    [sym_while_keyword] = ACTIONS(1727),
    [sym_for_keyword] = ACTIONS(1727),
    [sym_break_keyword] = ACTIONS(1727),
    [sym_continue_keyword] = ACTIONS(1727),
    [sym_return_keyword] = ACTIONS(1727),
    [sym_select_keyword] = ACTIONS(1727),
    [sym_active_keyword] = ACTIONS(1727),
    [sym_struct_keyword] = ACTIONS(1727),
    [sym_structure_keyword] = ACTIONS(1727),
    [sym_button_event_keyword] = ACTIONS(1727),
    [sym_channel_event_keyword] = ACTIONS(1727),
    [sym_level_event_keyword] = ACTIONS(1727),
    [sym_data_event_keyword] = ACTIONS(1727),
    [sym_timeline_event_keyword] = ACTIONS(1727),
    [sym_custom_event_keyword] = ACTIONS(1727),
    [sym_push_keyword] = ACTIONS(1727),
    [sym_release_keyword] = ACTIONS(1727),
    [aux_sym_on_keyword_token1] = ACTIONS(1727),
    [aux_sym_off_keyword_token1] = ACTIONS(1727),
    [sym_send_level_keyword] = ACTIONS(1727),
    [sym_send_string_keyword] = ACTIONS(1727),
    [sym_send_command_keyword] = ACTIONS(1727),
    [sym_clear_buffer_keyword] = ACTIONS(1727),
    [sym_create_buffer_keyword] = ACTIONS(1727),
    [sym_create_multi_buffer_keyword] = ACTIONS(1727),
    [sym_call_keyword] = ACTIONS(1727),
    [sym_system_call_keyword] = ACTIONS(1727),
    [sym_devchan_to_keyword] = ACTIONS(1727),
    [sym_devchan_min_to_keyword] = ACTIONS(1727),
    [sym_devchan_total_off_keyword] = ACTIONS(1727),
    [sym_devchan_pulse_keyword] = ACTIONS(1727),
    [sym_wait_keyword] = ACTIONS(1727),
    [sym_cancel_wait_keyword] = ACTIONS(1727),
    [sym_wait_until_keyword] = ACTIONS(1727),
    [sym_cancel_wait_until_keyword] = ACTIONS(1727),
    [sym_cancel_all_wait_keyword] = ACTIONS(1727),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1727),
    [sym_not] = ACTIONS(1727),
    [sym_bnot] = ACTIONS(1727),
    [sym___file__] = ACTIONS(1727),
    [sym___date__] = ACTIONS(1727),
    [sym___ldate__] = ACTIONS(1727),
    [sym___time__] = ACTIONS(1727),
    [sym___line__] = ACTIONS(1727),
    [sym___name__] = ACTIONS(1727),
    [sym___netlinx__] = ACTIONS(1727),
    [sym_netlinx_axi_version] = ACTIONS(1727),
    [sym_url_flg_tcp] = ACTIONS(1727),
    [sym_url_flg_accinfopresent] = ACTIONS(1727),
    [sym_url_flg_temp] = ACTIONS(1727),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1727),
    [sym_url_flg_stat_mask] = ACTIONS(1727),
    [sym_url_flg_stat_lookup] = ACTIONS(1727),
    [sym_url_flg_stat_connecting] = ACTIONS(1727),
    [sym_url_flg_stat_waiting] = ACTIONS(1727),
    [sym_url_flg_stat_connected] = ACTIONS(1727),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1727),
    [sym_first_virtual_device] = ACTIONS(1727),
    [sym_first_local_port] = ACTIONS(1727),
    [sym_dynamic_virtual_device] = ACTIONS(1727),
    [sym_timeline_once] = ACTIONS(1727),
    [sym_timeline_repeat] = ACTIONS(1727),
    [sym_timeline_absolute] = ACTIONS(1727),
    [sym_timeline_relative] = ACTIONS(1727),
    [sym_do_push_timed_infinite] = ACTIONS(1727),
    [sym_source_type_no_address] = ACTIONS(1727),
    [sym_source_type_neuron_id] = ACTIONS(1727),
    [sym_source_type_ip_address] = ACTIONS(1727),
    [sym_source_type_axlink] = ACTIONS(1727),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1727),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1727),
    [sym_source_type_ip_socket_address] = ACTIONS(1727),
    [sym_source_type_rs232] = ACTIONS(1727),
    [sym_source_type_internal] = ACTIONS(1727),
    [sym_source_type_mac_address] = ACTIONS(1727),
    [sym_source_type_ipv4_port] = ACTIONS(1727),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1727),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1727),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1727),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1727),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1727),
    [sym_true] = ACTIONS(1727),
    [sym_false] = ACTIONS(1727),
    [sym_file_read_only] = ACTIONS(1727),
    [sym_file_rw_new] = ACTIONS(1727),
    [sym_file_rw_append] = ACTIONS(1727),
    [sym_ip_tcp] = ACTIONS(1727),
    [sym_ip_udp] = ACTIONS(1727),
    [sym_ip_udp_2way] = ACTIONS(1727),
    [sym_xml_encode_types] = ACTIONS(1727),
    [sym_xml_encode_char_as_list] = ACTIONS(1727),
    [sym_xml_encode_le] = ACTIONS(1727),
    [sym_xml_decode_types] = ACTIONS(1727),
    [sym_xml_decode_no_preserve] = ACTIONS(1727),
    [sym_duet_dev_type_display_device] = ACTIONS(1727),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1727),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1727),
    [sym_duet_dev_type_io_device] = ACTIONS(1727),
    [sym_duet_dev_type_relay_device] = ACTIONS(1727),
    [sym_duet_dev_type_ups] = ACTIONS(1727),
    [sym_duet_dev_type_amplifier] = ACTIONS(1727),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1727),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1727),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1727),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1727),
    [sym_duet_dev_type_camera] = ACTIONS(1727),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1727),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1727),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1727),
    [sym_duet_dev_type_dss] = ACTIONS(1727),
    [sym_duet_dev_type_dvr] = ACTIONS(1727),
    [sym_duet_dev_type_disc_device] = ACTIONS(1727),
    [sym_duet_dev_type_document_camera] = ACTIONS(1727),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1727),
    [sym_duet_dev_type_hvac] = ACTIONS(1727),
    [sym_duet_dev_type_keypad] = ACTIONS(1727),
    [sym_duet_dev_type_light] = ACTIONS(1727),
    [sym_duet_dev_type_monitor] = ACTIONS(1727),
    [sym_duet_dev_type_motor] = ACTIONS(1727),
    [sym_duet_dev_type_multi_window] = ACTIONS(1727),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1727),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1727),
    [sym_duet_dev_type_receiver] = ACTIONS(1727),
    [sym_duet_dev_type_security_system] = ACTIONS(1727),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1727),
    [sym_duet_dev_type_settop_box] = ACTIONS(1727),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1727),
    [sym_duet_dev_type_switcher] = ACTIONS(1727),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1727),
    [sym_duet_dev_type_tv] = ACTIONS(1727),
    [sym_duet_dev_type_utility] = ACTIONS(1727),
    [sym_duet_dev_type_vcr] = ACTIONS(1727),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1727),
    [sym_duet_dev_type_video_processor] = ACTIONS(1727),
    [sym_duet_dev_type_video_projector] = ACTIONS(1727),
    [sym_duet_dev_type_video_wall] = ACTIONS(1727),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1727),
    [sym_duet_dev_type_weather] = ACTIONS(1727),
    [sym_duet_dev_not_polled] = ACTIONS(1727),
    [sym_duet_dev_polled] = ACTIONS(1727),
    [sym_clkmgr_mode_network] = ACTIONS(1727),
    [sym_clkmgr_mode_standalone] = ACTIONS(1727),
    [sym_normal_standby] = ACTIONS(1727),
    [sym_normal_wake] = ACTIONS(1727),
    [sym_ip_multicast_ttl_option] = ACTIONS(1727),
    [sym_ip_ttl_subnet] = ACTIONS(1727),
    [sym_ip_ttl_site] = ACTIONS(1727),
    [sym_ip_ttl_region] = ACTIONS(1727),
    [sym_ip_ttl_continent] = ACTIONS(1727),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1727),
    [sym_ip_nodelay_on] = ACTIONS(1727),
    [sym_ip_nodelay_off] = ACTIONS(1727),
    [sym_amx_error] = ACTIONS(1727),
    [sym_amx_warning] = ACTIONS(1727),
    [sym_amx_info] = ACTIONS(1727),
    [sym_amx_debug] = ACTIONS(1727),
    [sym_netlinx_login_success] = ACTIONS(1727),
    [sym_netlinx_login_fail] = ACTIONS(1727),
    [sym_netlinx_logout] = ACTIONS(1727),
    [sym_null_str] = ACTIONS(1727),
    [sym_smtp_address] = ACTIONS(1727),
    [sym_smtp_port_number] = ACTIONS(1727),
    [sym_smtp_username] = ACTIONS(1727),
    [sym_smtp_password] = ACTIONS(1727),
    [sym_smtp_require_tls] = ACTIONS(1727),
    [sym_smtp_from] = ACTIONS(1727),
    [sym_smtp_tls_true] = ACTIONS(1727),
    [sym_smtp_tls_false] = ACTIONS(1727),
    [sym_listview_on_row_select_event] = ACTIONS(1727),
    [sym_data_structure_unknown] = ACTIONS(1727),
    [sym_data_structure_datafeed] = ACTIONS(1727),
    [sym_data_structure_datarecordset] = ACTIONS(1727),
    [sym_data_structure_datarecord] = ACTIONS(1727),
    [sym_data_structure_datafield] = ACTIONS(1727),
    [sym_data_max_content_field_count] = ACTIONS(1727),
    [sym_data_max_metadata_field_count] = ACTIONS(1727),
    [sym_data_max_id_length] = ACTIONS(1727),
    [sym_data_max_name_length] = ACTIONS(1727),
    [sym_data_max_description_length] = ACTIONS(1727),
    [sym_data_max_type_length] = ACTIONS(1727),
    [sym_data_max_format_length] = ACTIONS(1727),
    [sym_data_max_source_length] = ACTIONS(1727),
    [sym_data_max_value_length] = ACTIONS(1727),
    [sym_data_max_label_length] = ACTIONS(1727),
    [sym_data_max_url_length] = ACTIONS(1727),
    [sym_max_last_login_info_length] = ACTIONS(1727),
    [sym_data_type_unknown] = ACTIONS(1727),
    [sym_data_type_string] = ACTIONS(1727),
    [sym_data_type_datetime] = ACTIONS(1727),
    [sym_data_type_date] = ACTIONS(1727),
    [sym_data_type_time] = ACTIONS(1727),
    [sym_data_type_image] = ACTIONS(1727),
    [sym_data_format_url] = ACTIONS(1727),
    [sym_data_format_phone] = ACTIONS(1727),
    [sym_data_format_email] = ACTIONS(1727),
    [sym_data_format_iso8601] = ACTIONS(1727),
    [sym_auditlog_priv] = ACTIONS(1727),
    [sym_configuration_priv] = ACTIONS(1727),
    [sym_deviceconfig_priv] = ACTIONS(1727),
    [sym_ftp_priv] = ACTIONS(1727),
    [sym_http_priv] = ACTIONS(1727),
    [sym_networkconfig_priv] = ACTIONS(1727),
    [sym_programport_priv] = ACTIONS(1727),
    [sym_remoteui_priv] = ACTIONS(1727),
    [sym_securitycontrol_priv] = ACTIONS(1727),
    [sym_softwaremanagement_priv] = ACTIONS(1727),
    [sym_terminal_priv] = ACTIONS(1727),
    [sym_tpadmin_priv] = ACTIONS(1727),
    [sym_user1_priv] = ACTIONS(1727),
    [sym_user2_priv] = ACTIONS(1727),
    [sym_user3_priv] = ACTIONS(1727),
    [sym_user4_priv] = ACTIONS(1727),
    [sym_usermanagement_priv] = ACTIONS(1727),
    [sym_valid_account] = ACTIONS(1727),
    [sym_err_name_invalid] = ACTIONS(1727),
    [sym_err_password_invalid] = ACTIONS(1727),
    [sym_err_invalid_account] = ACTIONS(1727),
    [sym_err_no_authorization] = ACTIONS(1727),
    [sym_err_invalid_parameter] = ACTIONS(1727),
    [sym_err_user_locked_out] = ACTIONS(1727),
    [sym_err_user_account_expired] = ACTIONS(1727),
    [sym_err_general_error] = ACTIONS(1727),
    [sym_err_authentication_error] = ACTIONS(1727),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1727),
    [sym_tls_validate_certificate] = ACTIONS(1727),
    [sym_tdata] = ACTIONS(1727),
    [sym_tchannel] = ACTIONS(1727),
    [sym_tlevel] = ACTIONS(1727),
    [sym_tbutton] = ACTIONS(1727),
    [sym_ttimeline] = ACTIONS(1727),
    [sym_tcustom] = ACTIONS(1727),
    [sym_url_struct] = ACTIONS(1727),
    [sym_dns_struct] = ACTIONS(1727),
    [sym_ip_address_struct] = ACTIONS(1727),
    [sym_dev_info_struct] = ACTIONS(1727),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1727),
    [sym_data_feed] = ACTIONS(1727),
    [sym_data_field] = ACTIONS(1727),
    [sym_data_record] = ACTIONS(1727),
    [sym_wc_data_feed] = ACTIONS(1727),
    [sym_wc_data_field] = ACTIONS(1727),
    [sym_wc_data_record] = ACTIONS(1727),
    [sym_last_login_info] = ACTIONS(1727),
    [sym____reserved___] = ACTIONS(1727),
    [sym_date] = ACTIONS(1727),
    [sym_day] = ACTIONS(1727),
    [sym_time] = ACTIONS(1727),
    [sym_ldate] = ACTIONS(1727),
    [sym_push_device] = ACTIONS(1727),
    [sym_push_channel] = ACTIONS(1727),
    [sym_push_devchan] = ACTIONS(1727),
    [sym_release_device] = ACTIONS(1727),
    [sym_release_channel] = ACTIONS(1727),
    [sym_release_devchan] = ACTIONS(1727),
    [sym_master_slot] = ACTIONS(1727),
    [sym_get_pulse_time] = ACTIONS(1727),
    [sym_get_timer] = ACTIONS(1727),
    [sym_master_sn] = ACTIONS(1727),
    [sym_system_number] = ACTIONS(1727),
    [sym_button] = ACTIONS(1727),
    [sym_dv_channel] = ACTIONS(1727),
    [sym_data] = ACTIONS(1727),
    [sym_level] = ACTIONS(1727),
    [sym_timeline] = ACTIONS(1727),
    [sym_channel] = ACTIONS(1727),
    [sym_custom] = ACTIONS(1727),
    [sym_atoi] = ACTIONS(1727),
    [sym_atol] = ACTIONS(1727),
    [sym_atof] = ACTIONS(1727),
    [sym_itoa] = ACTIONS(1727),
    [sym_itohex] = ACTIONS(1727),
    [sym_hextoi] = ACTIONS(1727),
    [sym_ftoa] = ACTIONS(1727),
    [sym_find_string] = ACTIONS(1727),
    [sym_left_string] = ACTIONS(1727),
    [sym_right_string] = ACTIONS(1727),
    [sym_mid_string] = ACTIONS(1727),
    [sym_remove_string] = ACTIONS(1727),
    [sym_get_buffer_string] = ACTIONS(1727),
    [sym_get_buffer_char] = ACTIONS(1727),
    [sym_get_multi_buffer_string] = ACTIONS(1727),
    [sym_length_string] = ACTIONS(1727),
    [sym_max_length_string] = ACTIONS(1727),
    [sym_set_length_string] = ACTIONS(1727),
    [sym_compare_string] = ACTIONS(1727),
    [sym_upper_string] = ACTIONS(1727),
    [sym_lower_string] = ACTIONS(1727),
    [sym_get_last] = ACTIONS(1727),
    [sym_date_to_day] = ACTIONS(1727),
    [sym_date_to_month] = ACTIONS(1727),
    [sym_date_to_year] = ACTIONS(1727),
    [sym_time_to_hour] = ACTIONS(1727),
    [sym_time_to_minute] = ACTIONS(1727),
    [sym_time_to_second] = ACTIONS(1727),
    [sym_day_of_week] = ACTIONS(1727),
    [sym_set_pulse_time] = ACTIONS(1727),
    [sym_set_timer] = ACTIONS(1727),
    [sym_file_open] = ACTIONS(1727),
    [sym_file_close] = ACTIONS(1727),
    [sym_file_read] = ACTIONS(1727),
    [sym_file_read_line] = ACTIONS(1727),
    [sym_file_write] = ACTIONS(1727),
    [sym_file_write_line] = ACTIONS(1727),
    [sym_file_copy] = ACTIONS(1727),
    [sym_file_delete] = ACTIONS(1727),
    [sym_file_rename] = ACTIONS(1727),
    [sym_file_dir] = ACTIONS(1727),
    [sym_file_getdir] = ACTIONS(1727),
    [sym_file_setdir] = ACTIONS(1727),
    [sym_file_create_dir] = ACTIONS(1727),
    [sym_file_remove_dir] = ACTIONS(1727),
    [sym_file_seek] = ACTIONS(1727),
    [sym_length_array] = ACTIONS(1727),
    [sym_max_length_array] = ACTIONS(1727),
    [sym_set_length_array] = ACTIONS(1727),
    [sym_ip_client_open] = ACTIONS(1727),
    [sym_ip_client_close] = ACTIONS(1727),
    [sym_ip_server_open] = ACTIONS(1727),
    [sym_ip_server_close] = ACTIONS(1727),
    [sym_ip_bound_client_open] = ACTIONS(1727),
    [sym_ip_mc_server_open] = ACTIONS(1727),
    [sym_ip_set_option] = ACTIONS(1727),
    [sym_ssh_client_open] = ACTIONS(1727),
    [sym_ssh_client_close] = ACTIONS(1727),
    [sym_tls_client_open] = ACTIONS(1727),
    [sym_tls_client_close] = ACTIONS(1727),
    [sym_do_push] = ACTIONS(1727),
    [sym_do_push_timed] = ACTIONS(1727),
    [sym_do_release] = ACTIONS(1727),
    [sym_device_id] = ACTIONS(1727),
    [sym_device_info] = ACTIONS(1727),
    [sym_device_id_string] = ACTIONS(1727),
    [sym_device_standby] = ACTIONS(1727),
    [sym_device_wake] = ACTIONS(1727),
    [sym_timeline_create] = ACTIONS(1727),
    [sym_timeline_kill] = ACTIONS(1727),
    [sym_timeline_reload] = ACTIONS(1727),
    [sym_timeline_pause] = ACTIONS(1727),
    [sym_timeline_restart] = ACTIONS(1727),
    [sym_timeline_set] = ACTIONS(1727),
    [sym_timeline_get] = ACTIONS(1727),
    [sym_timeline_active] = ACTIONS(1727),
    [sym_timeline_dynamic_id] = ACTIONS(1727),
    [sym_variable_to_string] = ACTIONS(1727),
    [sym_string_to_variable] = ACTIONS(1727),
    [sym_length_variable_to_string] = ACTIONS(1727),
    [sym_type_cast] = ACTIONS(1727),
    [sym_variable_to_xml] = ACTIONS(1727),
    [sym_xml_to_variable] = ACTIONS(1727),
    [sym_length_variable_to_xml] = ACTIONS(1727),
    [sym_redirect_string] = ACTIONS(1727),
    [sym_random_number] = ACTIONS(1727),
    [sym_get_unique_id] = ACTIONS(1727),
    [sym_get_system_number] = ACTIONS(1727),
    [sym_set_system_number] = ACTIONS(1727),
    [sym_get_serial_number] = ACTIONS(1727),
    [sym_get_master_build] = ACTIONS(1727),
    [sym_set_validation_code] = ACTIONS(1727),
    [sym_reboot] = ACTIONS(1727),
    [sym_load_duet_module] = ACTIONS(1727),
    [sym_unload_duet_module] = ACTIONS(1727),
    [sym_duet_mem_size_set] = ACTIONS(1727),
    [sym_duet_mem_size_get] = ACTIONS(1727),
    [sym_dynamic_application_device] = ACTIONS(1727),
    [sym_dynamic_polled_port] = ACTIONS(1727),
    [sym_static_port_binding] = ACTIONS(1727),
    [sym_static_ip_binding] = ACTIONS(1727),
    [sym_get_dns_list] = ACTIONS(1727),
    [sym_set_dns_list] = ACTIONS(1727),
    [sym_get_ip_address] = ACTIONS(1727),
    [sym_set_ip_address] = ACTIONS(1727),
    [sym_get_url_list] = ACTIONS(1727),
    [sym_add_url_entry] = ACTIONS(1727),
    [sym_delete_url_entry] = ACTIONS(1727),
    [sym_abs_value] = ACTIONS(1727),
    [sym_max_value] = ACTIONS(1727),
    [sym_min_value] = ACTIONS(1727),
    [sym_exp_value] = ACTIONS(1727),
    [sym_log_value] = ACTIONS(1727),
    [sym_log10_value] = ACTIONS(1727),
    [sym_power_value] = ACTIONS(1727),
    [sym_sqrt_value] = ACTIONS(1727),
    [sym_raw_be] = ACTIONS(1727),
    [sym_raw_le] = ACTIONS(1727),
    [sym_internal_queue_size_set] = ACTIONS(1727),
    [sym_internal_queue_size_get] = ACTIONS(1727),
    [sym_internal_threshold_set] = ACTIONS(1727),
    [sym_internal_threshold_get] = ACTIONS(1727),
    [sym_rebuilt_event] = ACTIONS(1727),
    [sym_do_custom_event] = ACTIONS(1727),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1727),
    [sym_clkmgr_set_clk_source] = ACTIONS(1727),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1727),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1727),
    [sym_clkmgr_get_timezone] = ACTIONS(1727),
    [sym_clkmgr_set_timezone] = ACTIONS(1727),
    [sym_clkmgr_get_resync_period] = ACTIONS(1727),
    [sym_clkmgr_set_resync_period] = ACTIONS(1727),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1727),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1727),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1727),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1727),
    [sym_clkmgr_get_timeservers] = ACTIONS(1727),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1727),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1727),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1727),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1727),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1727),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1727),
    [sym_set_log_level] = ACTIONS(1727),
    [sym_get_log_level] = ACTIONS(1727),
    [sym_amx_log] = ACTIONS(1727),
    [sym_smtp_server_config_set] = ACTIONS(1727),
    [sym_smtp_server_config_get] = ACTIONS(1727),
    [sym_smtp_send] = ACTIONS(1727),
    [sym_validate_netlinx_account] = ACTIONS(1727),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1727),
    [sym_authenticate_certificate] = ACTIONS(1727),
    [sym_audit_netlinx_session_event] = ACTIONS(1727),
    [sym_audit_netlinx_generic_event] = ACTIONS(1727),
    [sym_get_available_flash_disk_space] = ACTIONS(1727),
    [sym_get_max_flash_disk_space] = ACTIONS(1727),
    [sym_data_create_feed] = ACTIONS(1727),
    [sym_data_delete_feed] = ACTIONS(1727),
    [sym_data_publish_feed] = ACTIONS(1727),
    [sym_data_get_published_feed] = ACTIONS(1727),
    [sym_data_add_record] = ACTIONS(1727),
    [sym_data_get_event_record] = ACTIONS(1727),
    [sym__wc_data_create_feed] = ACTIONS(1727),
    [sym__wc_data_add_record] = ACTIONS(1727),
    [sym__wc_data_get_event_record] = ACTIONS(1727),
    [sym_format] = ACTIONS(1727),
    [sym_astro_clock] = ACTIONS(1727),
    [sym_set_outdoor_temperature] = ACTIONS(1727),
    [sym_set_virtual_level_count] = ACTIONS(1727),
    [sym_set_virtual_channel_count] = ACTIONS(1727),
    [sym_set_virtual_port_count] = ACTIONS(1727),
    [sym__wc_encode_int] = ACTIONS(1727),
    [sym__wc_decode_int] = ACTIONS(1727),
    [sym__wc_int] = ACTIONS(1727),
    [sym__wc_to_ch_int] = ACTIONS(1727),
    [sym__ch_to_wc_int] = ACTIONS(1727),
    [sym__wc_find_string_int] = ACTIONS(1727),
    [sym__wc_left_string_int] = ACTIONS(1727),
    [sym__wc_lower_string_int] = ACTIONS(1727),
    [sym__wc_mid_string_int] = ACTIONS(1727),
    [sym__wc_remove_string_int] = ACTIONS(1727),
    [sym__wc_right_string_int] = ACTIONS(1727),
    [sym__wc_upper_string_int] = ACTIONS(1727),
    [sym__wc_compare_string_int] = ACTIONS(1727),
    [sym__wc_get_buffer_char_int] = ACTIONS(1727),
    [sym__wc_get_buffer_string_int] = ACTIONS(1727),
    [sym__wc_concat_string_int] = ACTIONS(1727),
    [sym__wc_file_open_int] = ACTIONS(1727),
    [sym__wc_file_close_int] = ACTIONS(1727),
    [sym__wc_file_read_int] = ACTIONS(1727),
    [sym__wc_file_read_line_int] = ACTIONS(1727),
    [sym__wc_file_write_int] = ACTIONS(1727),
    [sym__wc_file_write_line_int] = ACTIONS(1727),
    [aux_sym_preproc_include_token1] = ACTIONS(1727),
    [aux_sym_preproc_define_token1] = ACTIONS(1729),
    [aux_sym_preproc_warn_token1] = ACTIONS(1729),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1729),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1729),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1729),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1729),
    [aux_sym_preproc_else_token1] = ACTIONS(1729),
    [anon_sym_LBRACK] = ACTIONS(1729),
    [anon_sym_LBRACE] = ACTIONS(1729),
    [anon_sym_LPAREN] = ACTIONS(1727),
    [anon_sym_BANG] = ACTIONS(1729),
    [anon_sym_TILDE] = ACTIONS(1729),
    [anon_sym_DASH] = ACTIONS(1727),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_DASH_DASH] = ACTIONS(1729),
    [anon_sym_PLUS_PLUS] = ACTIONS(1729),
    [anon_sym_DQUOTE] = ACTIONS(1729),
    [anon_sym_SQUOTE] = ACTIONS(1729),
    [sym_number_literal] = ACTIONS(1729),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(212)] = {
    [sym_identifier] = ACTIONS(1731),
    [sym_program_name_keyword] = ACTIONS(1731),
    [sym_module_name_keyword] = ACTIONS(1731),
    [sym_define_device_keyword] = ACTIONS(1731),
    [sym_define_combine_keyword] = ACTIONS(1731),
    [sym_define_constant_keyword] = ACTIONS(1731),
    [sym_define_type_keyword] = ACTIONS(1731),
    [sym_define_variable_keyword] = ACTIONS(1731),
    [sym_define_system_variable_keyword] = ACTIONS(1731),
    [sym_define_connect_level_keyword] = ACTIONS(1731),
    [sym_define_latching_keyword] = ACTIONS(1731),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1731),
    [sym_define_toggling_keyword] = ACTIONS(1731),
    [sym_define_call_keyword] = ACTIONS(1731),
    [sym_define_function_keyword] = ACTIONS(1731),
    [sym_define_library_function_keyword] = ACTIONS(1731),
    [sym_define_start_keyword] = ACTIONS(1731),
    [sym_define_event_keyword] = ACTIONS(1731),
    [sym_define_program_keyword] = ACTIONS(1731),
    [sym_define_module_keyword] = ACTIONS(1731),
    [sym_char_keyword] = ACTIONS(1731),
    [sym_widechar_keyword] = ACTIONS(1731),
    [sym_integer_keyword] = ACTIONS(1731),
    [sym_sinteger_keyword] = ACTIONS(1731),
    [sym_long_keyword] = ACTIONS(1731),
    [sym_slong_keyword] = ACTIONS(1731),
    [sym_float_keyword] = ACTIONS(1731),
    [sym_double_keyword] = ACTIONS(1731),
    [sym_variant_keyword] = ACTIONS(1731),
    [sym_variantarray_keyword] = ACTIONS(1731),
    [sym_local_var_keyword] = ACTIONS(1731),
    [sym_stack_var_keyword] = ACTIONS(1731),
    [sym_constant_keyword] = ACTIONS(1731),
    [sym_volatile_keyword] = ACTIONS(1731),
    [sym_non_volatile_keyword] = ACTIONS(1731),
    [sym_persistent_keyword] = ACTIONS(1731),
    [sym_dev_keyword] = ACTIONS(1731),
    [sym_devlev_keyword] = ACTIONS(1731),
    [sym_devchan_keyword] = ACTIONS(1731),
    [sym_if_keyword] = ACTIONS(1731),
    [sym_else_keyword] = ACTIONS(1731),
    [sym_switch_keyword] = ACTIONS(1731),
    [sym_case_keyword] = ACTIONS(1731),
    [sym_default_keyword] = ACTIONS(1731),
    [sym_while_keyword] = ACTIONS(1731),
    [sym_for_keyword] = ACTIONS(1731),
    [sym_break_keyword] = ACTIONS(1731),
    [sym_continue_keyword] = ACTIONS(1731),
    [sym_return_keyword] = ACTIONS(1731),
    [sym_select_keyword] = ACTIONS(1731),
    [sym_active_keyword] = ACTIONS(1731),
    [sym_struct_keyword] = ACTIONS(1731),
    [sym_structure_keyword] = ACTIONS(1731),
    [sym_button_event_keyword] = ACTIONS(1731),
    [sym_channel_event_keyword] = ACTIONS(1731),
    [sym_level_event_keyword] = ACTIONS(1731),
    [sym_data_event_keyword] = ACTIONS(1731),
    [sym_timeline_event_keyword] = ACTIONS(1731),
    [sym_custom_event_keyword] = ACTIONS(1731),
    [sym_push_keyword] = ACTIONS(1731),
    [sym_release_keyword] = ACTIONS(1731),
    [aux_sym_on_keyword_token1] = ACTIONS(1731),
    [aux_sym_off_keyword_token1] = ACTIONS(1731),
    [sym_send_level_keyword] = ACTIONS(1731),
    [sym_send_string_keyword] = ACTIONS(1731),
    [sym_send_command_keyword] = ACTIONS(1731),
    [sym_clear_buffer_keyword] = ACTIONS(1731),
    [sym_create_buffer_keyword] = ACTIONS(1731),
    [sym_create_multi_buffer_keyword] = ACTIONS(1731),
    [sym_call_keyword] = ACTIONS(1731),
    [sym_system_call_keyword] = ACTIONS(1731),
    [sym_devchan_to_keyword] = ACTIONS(1731),
    [sym_devchan_min_to_keyword] = ACTIONS(1731),
    [sym_devchan_total_off_keyword] = ACTIONS(1731),
    [sym_devchan_pulse_keyword] = ACTIONS(1731),
    [sym_wait_keyword] = ACTIONS(1731),
    [sym_cancel_wait_keyword] = ACTIONS(1731),
    [sym_wait_until_keyword] = ACTIONS(1731),
    [sym_cancel_wait_until_keyword] = ACTIONS(1731),
    [sym_cancel_all_wait_keyword] = ACTIONS(1731),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1731),
    [sym_not] = ACTIONS(1731),
    [sym_bnot] = ACTIONS(1731),
    [sym___file__] = ACTIONS(1731),
    [sym___date__] = ACTIONS(1731),
    [sym___ldate__] = ACTIONS(1731),
    [sym___time__] = ACTIONS(1731),
    [sym___line__] = ACTIONS(1731),
    [sym___name__] = ACTIONS(1731),
    [sym___netlinx__] = ACTIONS(1731),
    [sym_netlinx_axi_version] = ACTIONS(1731),
    [sym_url_flg_tcp] = ACTIONS(1731),
    [sym_url_flg_accinfopresent] = ACTIONS(1731),
    [sym_url_flg_temp] = ACTIONS(1731),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1731),
    [sym_url_flg_stat_mask] = ACTIONS(1731),
    [sym_url_flg_stat_lookup] = ACTIONS(1731),
    [sym_url_flg_stat_connecting] = ACTIONS(1731),
    [sym_url_flg_stat_waiting] = ACTIONS(1731),
    [sym_url_flg_stat_connected] = ACTIONS(1731),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1731),
    [sym_first_virtual_device] = ACTIONS(1731),
    [sym_first_local_port] = ACTIONS(1731),
    [sym_dynamic_virtual_device] = ACTIONS(1731),
    [sym_timeline_once] = ACTIONS(1731),
    [sym_timeline_repeat] = ACTIONS(1731),
    [sym_timeline_absolute] = ACTIONS(1731),
    [sym_timeline_relative] = ACTIONS(1731),
    [sym_do_push_timed_infinite] = ACTIONS(1731),
    [sym_source_type_no_address] = ACTIONS(1731),
    [sym_source_type_neuron_id] = ACTIONS(1731),
    [sym_source_type_ip_address] = ACTIONS(1731),
    [sym_source_type_axlink] = ACTIONS(1731),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1731),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1731),
    [sym_source_type_ip_socket_address] = ACTIONS(1731),
    [sym_source_type_rs232] = ACTIONS(1731),
    [sym_source_type_internal] = ACTIONS(1731),
    [sym_source_type_mac_address] = ACTIONS(1731),
    [sym_source_type_ipv4_port] = ACTIONS(1731),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1731),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1731),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1731),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1731),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1731),
    [sym_true] = ACTIONS(1731),
    [sym_false] = ACTIONS(1731),
    [sym_file_read_only] = ACTIONS(1731),
    [sym_file_rw_new] = ACTIONS(1731),
    [sym_file_rw_append] = ACTIONS(1731),
    [sym_ip_tcp] = ACTIONS(1731),
    [sym_ip_udp] = ACTIONS(1731),
    [sym_ip_udp_2way] = ACTIONS(1731),
    [sym_xml_encode_types] = ACTIONS(1731),
    [sym_xml_encode_char_as_list] = ACTIONS(1731),
    [sym_xml_encode_le] = ACTIONS(1731),
    [sym_xml_decode_types] = ACTIONS(1731),
    [sym_xml_decode_no_preserve] = ACTIONS(1731),
    [sym_duet_dev_type_display_device] = ACTIONS(1731),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1731),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1731),
    [sym_duet_dev_type_io_device] = ACTIONS(1731),
    [sym_duet_dev_type_relay_device] = ACTIONS(1731),
    [sym_duet_dev_type_ups] = ACTIONS(1731),
    [sym_duet_dev_type_amplifier] = ACTIONS(1731),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1731),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1731),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1731),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1731),
    [sym_duet_dev_type_camera] = ACTIONS(1731),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1731),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1731),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1731),
    [sym_duet_dev_type_dss] = ACTIONS(1731),
    [sym_duet_dev_type_dvr] = ACTIONS(1731),
    [sym_duet_dev_type_disc_device] = ACTIONS(1731),
    [sym_duet_dev_type_document_camera] = ACTIONS(1731),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1731),
    [sym_duet_dev_type_hvac] = ACTIONS(1731),
    [sym_duet_dev_type_keypad] = ACTIONS(1731),
    [sym_duet_dev_type_light] = ACTIONS(1731),
    [sym_duet_dev_type_monitor] = ACTIONS(1731),
    [sym_duet_dev_type_motor] = ACTIONS(1731),
    [sym_duet_dev_type_multi_window] = ACTIONS(1731),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1731),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1731),
    [sym_duet_dev_type_receiver] = ACTIONS(1731),
    [sym_duet_dev_type_security_system] = ACTIONS(1731),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1731),
    [sym_duet_dev_type_settop_box] = ACTIONS(1731),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1731),
    [sym_duet_dev_type_switcher] = ACTIONS(1731),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1731),
    [sym_duet_dev_type_tv] = ACTIONS(1731),
    [sym_duet_dev_type_utility] = ACTIONS(1731),
    [sym_duet_dev_type_vcr] = ACTIONS(1731),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1731),
    [sym_duet_dev_type_video_processor] = ACTIONS(1731),
    [sym_duet_dev_type_video_projector] = ACTIONS(1731),
    [sym_duet_dev_type_video_wall] = ACTIONS(1731),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1731),
    [sym_duet_dev_type_weather] = ACTIONS(1731),
    [sym_duet_dev_not_polled] = ACTIONS(1731),
    [sym_duet_dev_polled] = ACTIONS(1731),
    [sym_clkmgr_mode_network] = ACTIONS(1731),
    [sym_clkmgr_mode_standalone] = ACTIONS(1731),
    [sym_normal_standby] = ACTIONS(1731),
    [sym_normal_wake] = ACTIONS(1731),
    [sym_ip_multicast_ttl_option] = ACTIONS(1731),
    [sym_ip_ttl_subnet] = ACTIONS(1731),
    [sym_ip_ttl_site] = ACTIONS(1731),
    [sym_ip_ttl_region] = ACTIONS(1731),
    [sym_ip_ttl_continent] = ACTIONS(1731),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1731),
    [sym_ip_nodelay_on] = ACTIONS(1731),
    [sym_ip_nodelay_off] = ACTIONS(1731),
    [sym_amx_error] = ACTIONS(1731),
    [sym_amx_warning] = ACTIONS(1731),
    [sym_amx_info] = ACTIONS(1731),
    [sym_amx_debug] = ACTIONS(1731),
    [sym_netlinx_login_success] = ACTIONS(1731),
    [sym_netlinx_login_fail] = ACTIONS(1731),
    [sym_netlinx_logout] = ACTIONS(1731),
    [sym_null_str] = ACTIONS(1731),
    [sym_smtp_address] = ACTIONS(1731),
    [sym_smtp_port_number] = ACTIONS(1731),
    [sym_smtp_username] = ACTIONS(1731),
    [sym_smtp_password] = ACTIONS(1731),
    [sym_smtp_require_tls] = ACTIONS(1731),
    [sym_smtp_from] = ACTIONS(1731),
    [sym_smtp_tls_true] = ACTIONS(1731),
    [sym_smtp_tls_false] = ACTIONS(1731),
    [sym_listview_on_row_select_event] = ACTIONS(1731),
    [sym_data_structure_unknown] = ACTIONS(1731),
    [sym_data_structure_datafeed] = ACTIONS(1731),
    [sym_data_structure_datarecordset] = ACTIONS(1731),
    [sym_data_structure_datarecord] = ACTIONS(1731),
    [sym_data_structure_datafield] = ACTIONS(1731),
    [sym_data_max_content_field_count] = ACTIONS(1731),
    [sym_data_max_metadata_field_count] = ACTIONS(1731),
    [sym_data_max_id_length] = ACTIONS(1731),
    [sym_data_max_name_length] = ACTIONS(1731),
    [sym_data_max_description_length] = ACTIONS(1731),
    [sym_data_max_type_length] = ACTIONS(1731),
    [sym_data_max_format_length] = ACTIONS(1731),
    [sym_data_max_source_length] = ACTIONS(1731),
    [sym_data_max_value_length] = ACTIONS(1731),
    [sym_data_max_label_length] = ACTIONS(1731),
    [sym_data_max_url_length] = ACTIONS(1731),
    [sym_max_last_login_info_length] = ACTIONS(1731),
    [sym_data_type_unknown] = ACTIONS(1731),
    [sym_data_type_string] = ACTIONS(1731),
    [sym_data_type_datetime] = ACTIONS(1731),
    [sym_data_type_date] = ACTIONS(1731),
    [sym_data_type_time] = ACTIONS(1731),
    [sym_data_type_image] = ACTIONS(1731),
    [sym_data_format_url] = ACTIONS(1731),
    [sym_data_format_phone] = ACTIONS(1731),
    [sym_data_format_email] = ACTIONS(1731),
    [sym_data_format_iso8601] = ACTIONS(1731),
    [sym_auditlog_priv] = ACTIONS(1731),
    [sym_configuration_priv] = ACTIONS(1731),
    [sym_deviceconfig_priv] = ACTIONS(1731),
    [sym_ftp_priv] = ACTIONS(1731),
    [sym_http_priv] = ACTIONS(1731),
    [sym_networkconfig_priv] = ACTIONS(1731),
    [sym_programport_priv] = ACTIONS(1731),
    [sym_remoteui_priv] = ACTIONS(1731),
    [sym_securitycontrol_priv] = ACTIONS(1731),
    [sym_softwaremanagement_priv] = ACTIONS(1731),
    [sym_terminal_priv] = ACTIONS(1731),
    [sym_tpadmin_priv] = ACTIONS(1731),
    [sym_user1_priv] = ACTIONS(1731),
    [sym_user2_priv] = ACTIONS(1731),
    [sym_user3_priv] = ACTIONS(1731),
    [sym_user4_priv] = ACTIONS(1731),
    [sym_usermanagement_priv] = ACTIONS(1731),
    [sym_valid_account] = ACTIONS(1731),
    [sym_err_name_invalid] = ACTIONS(1731),
    [sym_err_password_invalid] = ACTIONS(1731),
    [sym_err_invalid_account] = ACTIONS(1731),
    [sym_err_no_authorization] = ACTIONS(1731),
    [sym_err_invalid_parameter] = ACTIONS(1731),
    [sym_err_user_locked_out] = ACTIONS(1731),
    [sym_err_user_account_expired] = ACTIONS(1731),
    [sym_err_general_error] = ACTIONS(1731),
    [sym_err_authentication_error] = ACTIONS(1731),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1731),
    [sym_tls_validate_certificate] = ACTIONS(1731),
    [sym_tdata] = ACTIONS(1731),
    [sym_tchannel] = ACTIONS(1731),
    [sym_tlevel] = ACTIONS(1731),
    [sym_tbutton] = ACTIONS(1731),
    [sym_ttimeline] = ACTIONS(1731),
    [sym_tcustom] = ACTIONS(1731),
    [sym_url_struct] = ACTIONS(1731),
    [sym_dns_struct] = ACTIONS(1731),
    [sym_ip_address_struct] = ACTIONS(1731),
    [sym_dev_info_struct] = ACTIONS(1731),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1731),
    [sym_data_feed] = ACTIONS(1731),
    [sym_data_field] = ACTIONS(1731),
    [sym_data_record] = ACTIONS(1731),
    [sym_wc_data_feed] = ACTIONS(1731),
    [sym_wc_data_field] = ACTIONS(1731),
    [sym_wc_data_record] = ACTIONS(1731),
    [sym_last_login_info] = ACTIONS(1731),
    [sym____reserved___] = ACTIONS(1731),
    [sym_date] = ACTIONS(1731),
    [sym_day] = ACTIONS(1731),
    [sym_time] = ACTIONS(1731),
    [sym_ldate] = ACTIONS(1731),
    [sym_push_device] = ACTIONS(1731),
    [sym_push_channel] = ACTIONS(1731),
    [sym_push_devchan] = ACTIONS(1731),
    [sym_release_device] = ACTIONS(1731),
    [sym_release_channel] = ACTIONS(1731),
    [sym_release_devchan] = ACTIONS(1731),
    [sym_master_slot] = ACTIONS(1731),
    [sym_get_pulse_time] = ACTIONS(1731),
    [sym_get_timer] = ACTIONS(1731),
    [sym_master_sn] = ACTIONS(1731),
    [sym_system_number] = ACTIONS(1731),
    [sym_button] = ACTIONS(1731),
    [sym_dv_channel] = ACTIONS(1731),
    [sym_data] = ACTIONS(1731),
    [sym_level] = ACTIONS(1731),
    [sym_timeline] = ACTIONS(1731),
    [sym_channel] = ACTIONS(1731),
    [sym_custom] = ACTIONS(1731),
    [sym_atoi] = ACTIONS(1731),
    [sym_atol] = ACTIONS(1731),
    [sym_atof] = ACTIONS(1731),
    [sym_itoa] = ACTIONS(1731),
    [sym_itohex] = ACTIONS(1731),
    [sym_hextoi] = ACTIONS(1731),
    [sym_ftoa] = ACTIONS(1731),
    [sym_find_string] = ACTIONS(1731),
    [sym_left_string] = ACTIONS(1731),
    [sym_right_string] = ACTIONS(1731),
    [sym_mid_string] = ACTIONS(1731),
    [sym_remove_string] = ACTIONS(1731),
    [sym_get_buffer_string] = ACTIONS(1731),
    [sym_get_buffer_char] = ACTIONS(1731),
    [sym_get_multi_buffer_string] = ACTIONS(1731),
    [sym_length_string] = ACTIONS(1731),
    [sym_max_length_string] = ACTIONS(1731),
    [sym_set_length_string] = ACTIONS(1731),
    [sym_compare_string] = ACTIONS(1731),
    [sym_upper_string] = ACTIONS(1731),
    [sym_lower_string] = ACTIONS(1731),
    [sym_get_last] = ACTIONS(1731),
    [sym_date_to_day] = ACTIONS(1731),
    [sym_date_to_month] = ACTIONS(1731),
    [sym_date_to_year] = ACTIONS(1731),
    [sym_time_to_hour] = ACTIONS(1731),
    [sym_time_to_minute] = ACTIONS(1731),
    [sym_time_to_second] = ACTIONS(1731),
    [sym_day_of_week] = ACTIONS(1731),
    [sym_set_pulse_time] = ACTIONS(1731),
    [sym_set_timer] = ACTIONS(1731),
    [sym_file_open] = ACTIONS(1731),
    [sym_file_close] = ACTIONS(1731),
    [sym_file_read] = ACTIONS(1731),
    [sym_file_read_line] = ACTIONS(1731),
    [sym_file_write] = ACTIONS(1731),
    [sym_file_write_line] = ACTIONS(1731),
    [sym_file_copy] = ACTIONS(1731),
    [sym_file_delete] = ACTIONS(1731),
    [sym_file_rename] = ACTIONS(1731),
    [sym_file_dir] = ACTIONS(1731),
    [sym_file_getdir] = ACTIONS(1731),
    [sym_file_setdir] = ACTIONS(1731),
    [sym_file_create_dir] = ACTIONS(1731),
    [sym_file_remove_dir] = ACTIONS(1731),
    [sym_file_seek] = ACTIONS(1731),
    [sym_length_array] = ACTIONS(1731),
    [sym_max_length_array] = ACTIONS(1731),
    [sym_set_length_array] = ACTIONS(1731),
    [sym_ip_client_open] = ACTIONS(1731),
    [sym_ip_client_close] = ACTIONS(1731),
    [sym_ip_server_open] = ACTIONS(1731),
    [sym_ip_server_close] = ACTIONS(1731),
    [sym_ip_bound_client_open] = ACTIONS(1731),
    [sym_ip_mc_server_open] = ACTIONS(1731),
    [sym_ip_set_option] = ACTIONS(1731),
    [sym_ssh_client_open] = ACTIONS(1731),
    [sym_ssh_client_close] = ACTIONS(1731),
    [sym_tls_client_open] = ACTIONS(1731),
    [sym_tls_client_close] = ACTIONS(1731),
    [sym_do_push] = ACTIONS(1731),
    [sym_do_push_timed] = ACTIONS(1731),
    [sym_do_release] = ACTIONS(1731),
    [sym_device_id] = ACTIONS(1731),
    [sym_device_info] = ACTIONS(1731),
    [sym_device_id_string] = ACTIONS(1731),
    [sym_device_standby] = ACTIONS(1731),
    [sym_device_wake] = ACTIONS(1731),
    [sym_timeline_create] = ACTIONS(1731),
    [sym_timeline_kill] = ACTIONS(1731),
    [sym_timeline_reload] = ACTIONS(1731),
    [sym_timeline_pause] = ACTIONS(1731),
    [sym_timeline_restart] = ACTIONS(1731),
    [sym_timeline_set] = ACTIONS(1731),
    [sym_timeline_get] = ACTIONS(1731),
    [sym_timeline_active] = ACTIONS(1731),
    [sym_timeline_dynamic_id] = ACTIONS(1731),
    [sym_variable_to_string] = ACTIONS(1731),
    [sym_string_to_variable] = ACTIONS(1731),
    [sym_length_variable_to_string] = ACTIONS(1731),
    [sym_type_cast] = ACTIONS(1731),
    [sym_variable_to_xml] = ACTIONS(1731),
    [sym_xml_to_variable] = ACTIONS(1731),
    [sym_length_variable_to_xml] = ACTIONS(1731),
    [sym_redirect_string] = ACTIONS(1731),
    [sym_random_number] = ACTIONS(1731),
    [sym_get_unique_id] = ACTIONS(1731),
    [sym_get_system_number] = ACTIONS(1731),
    [sym_set_system_number] = ACTIONS(1731),
    [sym_get_serial_number] = ACTIONS(1731),
    [sym_get_master_build] = ACTIONS(1731),
    [sym_set_validation_code] = ACTIONS(1731),
    [sym_reboot] = ACTIONS(1731),
    [sym_load_duet_module] = ACTIONS(1731),
    [sym_unload_duet_module] = ACTIONS(1731),
    [sym_duet_mem_size_set] = ACTIONS(1731),
    [sym_duet_mem_size_get] = ACTIONS(1731),
    [sym_dynamic_application_device] = ACTIONS(1731),
    [sym_dynamic_polled_port] = ACTIONS(1731),
    [sym_static_port_binding] = ACTIONS(1731),
    [sym_static_ip_binding] = ACTIONS(1731),
    [sym_get_dns_list] = ACTIONS(1731),
    [sym_set_dns_list] = ACTIONS(1731),
    [sym_get_ip_address] = ACTIONS(1731),
    [sym_set_ip_address] = ACTIONS(1731),
    [sym_get_url_list] = ACTIONS(1731),
    [sym_add_url_entry] = ACTIONS(1731),
    [sym_delete_url_entry] = ACTIONS(1731),
    [sym_abs_value] = ACTIONS(1731),
    [sym_max_value] = ACTIONS(1731),
    [sym_min_value] = ACTIONS(1731),
    [sym_exp_value] = ACTIONS(1731),
    [sym_log_value] = ACTIONS(1731),
    [sym_log10_value] = ACTIONS(1731),
    [sym_power_value] = ACTIONS(1731),
    [sym_sqrt_value] = ACTIONS(1731),
    [sym_raw_be] = ACTIONS(1731),
    [sym_raw_le] = ACTIONS(1731),
    [sym_internal_queue_size_set] = ACTIONS(1731),
    [sym_internal_queue_size_get] = ACTIONS(1731),
    [sym_internal_threshold_set] = ACTIONS(1731),
    [sym_internal_threshold_get] = ACTIONS(1731),
    [sym_rebuilt_event] = ACTIONS(1731),
    [sym_do_custom_event] = ACTIONS(1731),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1731),
    [sym_clkmgr_set_clk_source] = ACTIONS(1731),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1731),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1731),
    [sym_clkmgr_get_timezone] = ACTIONS(1731),
    [sym_clkmgr_set_timezone] = ACTIONS(1731),
    [sym_clkmgr_get_resync_period] = ACTIONS(1731),
    [sym_clkmgr_set_resync_period] = ACTIONS(1731),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1731),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1731),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1731),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1731),
    [sym_clkmgr_get_timeservers] = ACTIONS(1731),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1731),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1731),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1731),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1731),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1731),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1731),
    [sym_set_log_level] = ACTIONS(1731),
    [sym_get_log_level] = ACTIONS(1731),
    [sym_amx_log] = ACTIONS(1731),
    [sym_smtp_server_config_set] = ACTIONS(1731),
    [sym_smtp_server_config_get] = ACTIONS(1731),
    [sym_smtp_send] = ACTIONS(1731),
    [sym_validate_netlinx_account] = ACTIONS(1731),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1731),
    [sym_authenticate_certificate] = ACTIONS(1731),
    [sym_audit_netlinx_session_event] = ACTIONS(1731),
    [sym_audit_netlinx_generic_event] = ACTIONS(1731),
    [sym_get_available_flash_disk_space] = ACTIONS(1731),
    [sym_get_max_flash_disk_space] = ACTIONS(1731),
    [sym_data_create_feed] = ACTIONS(1731),
    [sym_data_delete_feed] = ACTIONS(1731),
    [sym_data_publish_feed] = ACTIONS(1731),
    [sym_data_get_published_feed] = ACTIONS(1731),
    [sym_data_add_record] = ACTIONS(1731),
    [sym_data_get_event_record] = ACTIONS(1731),
    [sym__wc_data_create_feed] = ACTIONS(1731),
    [sym__wc_data_add_record] = ACTIONS(1731),
    [sym__wc_data_get_event_record] = ACTIONS(1731),
    [sym_format] = ACTIONS(1731),
    [sym_astro_clock] = ACTIONS(1731),
    [sym_set_outdoor_temperature] = ACTIONS(1731),
    [sym_set_virtual_level_count] = ACTIONS(1731),
    [sym_set_virtual_channel_count] = ACTIONS(1731),
    [sym_set_virtual_port_count] = ACTIONS(1731),
    [sym__wc_encode_int] = ACTIONS(1731),
    [sym__wc_decode_int] = ACTIONS(1731),
    [sym__wc_int] = ACTIONS(1731),
    [sym__wc_to_ch_int] = ACTIONS(1731),
    [sym__ch_to_wc_int] = ACTIONS(1731),
    [sym__wc_find_string_int] = ACTIONS(1731),
    [sym__wc_left_string_int] = ACTIONS(1731),
    [sym__wc_lower_string_int] = ACTIONS(1731),
    [sym__wc_mid_string_int] = ACTIONS(1731),
    [sym__wc_remove_string_int] = ACTIONS(1731),
    [sym__wc_right_string_int] = ACTIONS(1731),
    [sym__wc_upper_string_int] = ACTIONS(1731),
    [sym__wc_compare_string_int] = ACTIONS(1731),
    [sym__wc_get_buffer_char_int] = ACTIONS(1731),
    [sym__wc_get_buffer_string_int] = ACTIONS(1731),
    [sym__wc_concat_string_int] = ACTIONS(1731),
    [sym__wc_file_open_int] = ACTIONS(1731),
    [sym__wc_file_close_int] = ACTIONS(1731),
    [sym__wc_file_read_int] = ACTIONS(1731),
    [sym__wc_file_read_line_int] = ACTIONS(1731),
    [sym__wc_file_write_int] = ACTIONS(1731),
    [sym__wc_file_write_line_int] = ACTIONS(1731),
    [aux_sym_preproc_include_token1] = ACTIONS(1731),
    [aux_sym_preproc_define_token1] = ACTIONS(1733),
    [aux_sym_preproc_warn_token1] = ACTIONS(1733),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1733),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1733),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1733),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1733),
    [aux_sym_preproc_else_token1] = ACTIONS(1733),
    [anon_sym_LBRACK] = ACTIONS(1733),
    [anon_sym_LBRACE] = ACTIONS(1733),
    [anon_sym_LPAREN] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [anon_sym_DASH] = ACTIONS(1731),
    [anon_sym_PLUS] = ACTIONS(1731),
    [anon_sym_DASH_DASH] = ACTIONS(1733),
    [anon_sym_PLUS_PLUS] = ACTIONS(1733),
    [anon_sym_DQUOTE] = ACTIONS(1733),
    [anon_sym_SQUOTE] = ACTIONS(1733),
    [sym_number_literal] = ACTIONS(1733),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(213)] = {
    [sym_identifier] = ACTIONS(1735),
    [sym_program_name_keyword] = ACTIONS(1735),
    [sym_module_name_keyword] = ACTIONS(1735),
    [sym_define_device_keyword] = ACTIONS(1735),
    [sym_define_combine_keyword] = ACTIONS(1735),
    [sym_define_constant_keyword] = ACTIONS(1735),
    [sym_define_type_keyword] = ACTIONS(1735),
    [sym_define_variable_keyword] = ACTIONS(1735),
    [sym_define_system_variable_keyword] = ACTIONS(1735),
    [sym_define_connect_level_keyword] = ACTIONS(1735),
    [sym_define_latching_keyword] = ACTIONS(1735),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1735),
    [sym_define_toggling_keyword] = ACTIONS(1735),
    [sym_define_call_keyword] = ACTIONS(1735),
    [sym_define_function_keyword] = ACTIONS(1735),
    [sym_define_library_function_keyword] = ACTIONS(1735),
    [sym_define_start_keyword] = ACTIONS(1735),
    [sym_define_event_keyword] = ACTIONS(1735),
    [sym_define_program_keyword] = ACTIONS(1735),
    [sym_define_module_keyword] = ACTIONS(1735),
    [sym_char_keyword] = ACTIONS(1735),
    [sym_widechar_keyword] = ACTIONS(1735),
    [sym_integer_keyword] = ACTIONS(1735),
    [sym_sinteger_keyword] = ACTIONS(1735),
    [sym_long_keyword] = ACTIONS(1735),
    [sym_slong_keyword] = ACTIONS(1735),
    [sym_float_keyword] = ACTIONS(1735),
    [sym_double_keyword] = ACTIONS(1735),
    [sym_variant_keyword] = ACTIONS(1735),
    [sym_variantarray_keyword] = ACTIONS(1735),
    [sym_local_var_keyword] = ACTIONS(1735),
    [sym_stack_var_keyword] = ACTIONS(1735),
    [sym_constant_keyword] = ACTIONS(1735),
    [sym_volatile_keyword] = ACTIONS(1735),
    [sym_non_volatile_keyword] = ACTIONS(1735),
    [sym_persistent_keyword] = ACTIONS(1735),
    [sym_dev_keyword] = ACTIONS(1735),
    [sym_devlev_keyword] = ACTIONS(1735),
    [sym_devchan_keyword] = ACTIONS(1735),
    [sym_if_keyword] = ACTIONS(1735),
    [sym_else_keyword] = ACTIONS(1735),
    [sym_switch_keyword] = ACTIONS(1735),
    [sym_case_keyword] = ACTIONS(1735),
    [sym_default_keyword] = ACTIONS(1735),
    [sym_while_keyword] = ACTIONS(1735),
    [sym_for_keyword] = ACTIONS(1735),
    [sym_break_keyword] = ACTIONS(1735),
    [sym_continue_keyword] = ACTIONS(1735),
    [sym_return_keyword] = ACTIONS(1735),
    [sym_select_keyword] = ACTIONS(1735),
    [sym_active_keyword] = ACTIONS(1735),
    [sym_struct_keyword] = ACTIONS(1735),
    [sym_structure_keyword] = ACTIONS(1735),
    [sym_button_event_keyword] = ACTIONS(1735),
    [sym_channel_event_keyword] = ACTIONS(1735),
    [sym_level_event_keyword] = ACTIONS(1735),
    [sym_data_event_keyword] = ACTIONS(1735),
    [sym_timeline_event_keyword] = ACTIONS(1735),
    [sym_custom_event_keyword] = ACTIONS(1735),
    [sym_push_keyword] = ACTIONS(1735),
    [sym_release_keyword] = ACTIONS(1735),
    [aux_sym_on_keyword_token1] = ACTIONS(1735),
    [aux_sym_off_keyword_token1] = ACTIONS(1735),
    [sym_send_level_keyword] = ACTIONS(1735),
    [sym_send_string_keyword] = ACTIONS(1735),
    [sym_send_command_keyword] = ACTIONS(1735),
    [sym_clear_buffer_keyword] = ACTIONS(1735),
    [sym_create_buffer_keyword] = ACTIONS(1735),
    [sym_create_multi_buffer_keyword] = ACTIONS(1735),
    [sym_call_keyword] = ACTIONS(1735),
    [sym_system_call_keyword] = ACTIONS(1735),
    [sym_devchan_to_keyword] = ACTIONS(1735),
    [sym_devchan_min_to_keyword] = ACTIONS(1735),
    [sym_devchan_total_off_keyword] = ACTIONS(1735),
    [sym_devchan_pulse_keyword] = ACTIONS(1735),
    [sym_wait_keyword] = ACTIONS(1735),
    [sym_cancel_wait_keyword] = ACTIONS(1735),
    [sym_wait_until_keyword] = ACTIONS(1735),
    [sym_cancel_wait_until_keyword] = ACTIONS(1735),
    [sym_cancel_all_wait_keyword] = ACTIONS(1735),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1735),
    [sym_not] = ACTIONS(1735),
    [sym_bnot] = ACTIONS(1735),
    [sym___file__] = ACTIONS(1735),
    [sym___date__] = ACTIONS(1735),
    [sym___ldate__] = ACTIONS(1735),
    [sym___time__] = ACTIONS(1735),
    [sym___line__] = ACTIONS(1735),
    [sym___name__] = ACTIONS(1735),
    [sym___netlinx__] = ACTIONS(1735),
    [sym_netlinx_axi_version] = ACTIONS(1735),
    [sym_url_flg_tcp] = ACTIONS(1735),
    [sym_url_flg_accinfopresent] = ACTIONS(1735),
    [sym_url_flg_temp] = ACTIONS(1735),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1735),
    [sym_url_flg_stat_mask] = ACTIONS(1735),
    [sym_url_flg_stat_lookup] = ACTIONS(1735),
    [sym_url_flg_stat_connecting] = ACTIONS(1735),
    [sym_url_flg_stat_waiting] = ACTIONS(1735),
    [sym_url_flg_stat_connected] = ACTIONS(1735),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1735),
    [sym_first_virtual_device] = ACTIONS(1735),
    [sym_first_local_port] = ACTIONS(1735),
    [sym_dynamic_virtual_device] = ACTIONS(1735),
    [sym_timeline_once] = ACTIONS(1735),
    [sym_timeline_repeat] = ACTIONS(1735),
    [sym_timeline_absolute] = ACTIONS(1735),
    [sym_timeline_relative] = ACTIONS(1735),
    [sym_do_push_timed_infinite] = ACTIONS(1735),
    [sym_source_type_no_address] = ACTIONS(1735),
    [sym_source_type_neuron_id] = ACTIONS(1735),
    [sym_source_type_ip_address] = ACTIONS(1735),
    [sym_source_type_axlink] = ACTIONS(1735),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1735),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1735),
    [sym_source_type_ip_socket_address] = ACTIONS(1735),
    [sym_source_type_rs232] = ACTIONS(1735),
    [sym_source_type_internal] = ACTIONS(1735),
    [sym_source_type_mac_address] = ACTIONS(1735),
    [sym_source_type_ipv4_port] = ACTIONS(1735),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1735),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1735),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1735),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1735),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1735),
    [sym_true] = ACTIONS(1735),
    [sym_false] = ACTIONS(1735),
    [sym_file_read_only] = ACTIONS(1735),
    [sym_file_rw_new] = ACTIONS(1735),
    [sym_file_rw_append] = ACTIONS(1735),
    [sym_ip_tcp] = ACTIONS(1735),
    [sym_ip_udp] = ACTIONS(1735),
    [sym_ip_udp_2way] = ACTIONS(1735),
    [sym_xml_encode_types] = ACTIONS(1735),
    [sym_xml_encode_char_as_list] = ACTIONS(1735),
    [sym_xml_encode_le] = ACTIONS(1735),
    [sym_xml_decode_types] = ACTIONS(1735),
    [sym_xml_decode_no_preserve] = ACTIONS(1735),
    [sym_duet_dev_type_display_device] = ACTIONS(1735),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1735),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1735),
    [sym_duet_dev_type_io_device] = ACTIONS(1735),
    [sym_duet_dev_type_relay_device] = ACTIONS(1735),
    [sym_duet_dev_type_ups] = ACTIONS(1735),
    [sym_duet_dev_type_amplifier] = ACTIONS(1735),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1735),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1735),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1735),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1735),
    [sym_duet_dev_type_camera] = ACTIONS(1735),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1735),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1735),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1735),
    [sym_duet_dev_type_dss] = ACTIONS(1735),
    [sym_duet_dev_type_dvr] = ACTIONS(1735),
    [sym_duet_dev_type_disc_device] = ACTIONS(1735),
    [sym_duet_dev_type_document_camera] = ACTIONS(1735),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1735),
    [sym_duet_dev_type_hvac] = ACTIONS(1735),
    [sym_duet_dev_type_keypad] = ACTIONS(1735),
    [sym_duet_dev_type_light] = ACTIONS(1735),
    [sym_duet_dev_type_monitor] = ACTIONS(1735),
    [sym_duet_dev_type_motor] = ACTIONS(1735),
    [sym_duet_dev_type_multi_window] = ACTIONS(1735),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1735),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1735),
    [sym_duet_dev_type_receiver] = ACTIONS(1735),
    [sym_duet_dev_type_security_system] = ACTIONS(1735),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1735),
    [sym_duet_dev_type_settop_box] = ACTIONS(1735),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1735),
    [sym_duet_dev_type_switcher] = ACTIONS(1735),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1735),
    [sym_duet_dev_type_tv] = ACTIONS(1735),
    [sym_duet_dev_type_utility] = ACTIONS(1735),
    [sym_duet_dev_type_vcr] = ACTIONS(1735),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1735),
    [sym_duet_dev_type_video_processor] = ACTIONS(1735),
    [sym_duet_dev_type_video_projector] = ACTIONS(1735),
    [sym_duet_dev_type_video_wall] = ACTIONS(1735),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1735),
    [sym_duet_dev_type_weather] = ACTIONS(1735),
    [sym_duet_dev_not_polled] = ACTIONS(1735),
    [sym_duet_dev_polled] = ACTIONS(1735),
    [sym_clkmgr_mode_network] = ACTIONS(1735),
    [sym_clkmgr_mode_standalone] = ACTIONS(1735),
    [sym_normal_standby] = ACTIONS(1735),
    [sym_normal_wake] = ACTIONS(1735),
    [sym_ip_multicast_ttl_option] = ACTIONS(1735),
    [sym_ip_ttl_subnet] = ACTIONS(1735),
    [sym_ip_ttl_site] = ACTIONS(1735),
    [sym_ip_ttl_region] = ACTIONS(1735),
    [sym_ip_ttl_continent] = ACTIONS(1735),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1735),
    [sym_ip_nodelay_on] = ACTIONS(1735),
    [sym_ip_nodelay_off] = ACTIONS(1735),
    [sym_amx_error] = ACTIONS(1735),
    [sym_amx_warning] = ACTIONS(1735),
    [sym_amx_info] = ACTIONS(1735),
    [sym_amx_debug] = ACTIONS(1735),
    [sym_netlinx_login_success] = ACTIONS(1735),
    [sym_netlinx_login_fail] = ACTIONS(1735),
    [sym_netlinx_logout] = ACTIONS(1735),
    [sym_null_str] = ACTIONS(1735),
    [sym_smtp_address] = ACTIONS(1735),
    [sym_smtp_port_number] = ACTIONS(1735),
    [sym_smtp_username] = ACTIONS(1735),
    [sym_smtp_password] = ACTIONS(1735),
    [sym_smtp_require_tls] = ACTIONS(1735),
    [sym_smtp_from] = ACTIONS(1735),
    [sym_smtp_tls_true] = ACTIONS(1735),
    [sym_smtp_tls_false] = ACTIONS(1735),
    [sym_listview_on_row_select_event] = ACTIONS(1735),
    [sym_data_structure_unknown] = ACTIONS(1735),
    [sym_data_structure_datafeed] = ACTIONS(1735),
    [sym_data_structure_datarecordset] = ACTIONS(1735),
    [sym_data_structure_datarecord] = ACTIONS(1735),
    [sym_data_structure_datafield] = ACTIONS(1735),
    [sym_data_max_content_field_count] = ACTIONS(1735),
    [sym_data_max_metadata_field_count] = ACTIONS(1735),
    [sym_data_max_id_length] = ACTIONS(1735),
    [sym_data_max_name_length] = ACTIONS(1735),
    [sym_data_max_description_length] = ACTIONS(1735),
    [sym_data_max_type_length] = ACTIONS(1735),
    [sym_data_max_format_length] = ACTIONS(1735),
    [sym_data_max_source_length] = ACTIONS(1735),
    [sym_data_max_value_length] = ACTIONS(1735),
    [sym_data_max_label_length] = ACTIONS(1735),
    [sym_data_max_url_length] = ACTIONS(1735),
    [sym_max_last_login_info_length] = ACTIONS(1735),
    [sym_data_type_unknown] = ACTIONS(1735),
    [sym_data_type_string] = ACTIONS(1735),
    [sym_data_type_datetime] = ACTIONS(1735),
    [sym_data_type_date] = ACTIONS(1735),
    [sym_data_type_time] = ACTIONS(1735),
    [sym_data_type_image] = ACTIONS(1735),
    [sym_data_format_url] = ACTIONS(1735),
    [sym_data_format_phone] = ACTIONS(1735),
    [sym_data_format_email] = ACTIONS(1735),
    [sym_data_format_iso8601] = ACTIONS(1735),
    [sym_auditlog_priv] = ACTIONS(1735),
    [sym_configuration_priv] = ACTIONS(1735),
    [sym_deviceconfig_priv] = ACTIONS(1735),
    [sym_ftp_priv] = ACTIONS(1735),
    [sym_http_priv] = ACTIONS(1735),
    [sym_networkconfig_priv] = ACTIONS(1735),
    [sym_programport_priv] = ACTIONS(1735),
    [sym_remoteui_priv] = ACTIONS(1735),
    [sym_securitycontrol_priv] = ACTIONS(1735),
    [sym_softwaremanagement_priv] = ACTIONS(1735),
    [sym_terminal_priv] = ACTIONS(1735),
    [sym_tpadmin_priv] = ACTIONS(1735),
    [sym_user1_priv] = ACTIONS(1735),
    [sym_user2_priv] = ACTIONS(1735),
    [sym_user3_priv] = ACTIONS(1735),
    [sym_user4_priv] = ACTIONS(1735),
    [sym_usermanagement_priv] = ACTIONS(1735),
    [sym_valid_account] = ACTIONS(1735),
    [sym_err_name_invalid] = ACTIONS(1735),
    [sym_err_password_invalid] = ACTIONS(1735),
    [sym_err_invalid_account] = ACTIONS(1735),
    [sym_err_no_authorization] = ACTIONS(1735),
    [sym_err_invalid_parameter] = ACTIONS(1735),
    [sym_err_user_locked_out] = ACTIONS(1735),
    [sym_err_user_account_expired] = ACTIONS(1735),
    [sym_err_general_error] = ACTIONS(1735),
    [sym_err_authentication_error] = ACTIONS(1735),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1735),
    [sym_tls_validate_certificate] = ACTIONS(1735),
    [sym_tdata] = ACTIONS(1735),
    [sym_tchannel] = ACTIONS(1735),
    [sym_tlevel] = ACTIONS(1735),
    [sym_tbutton] = ACTIONS(1735),
    [sym_ttimeline] = ACTIONS(1735),
    [sym_tcustom] = ACTIONS(1735),
    [sym_url_struct] = ACTIONS(1735),
    [sym_dns_struct] = ACTIONS(1735),
    [sym_ip_address_struct] = ACTIONS(1735),
    [sym_dev_info_struct] = ACTIONS(1735),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1735),
    [sym_data_feed] = ACTIONS(1735),
    [sym_data_field] = ACTIONS(1735),
    [sym_data_record] = ACTIONS(1735),
    [sym_wc_data_feed] = ACTIONS(1735),
    [sym_wc_data_field] = ACTIONS(1735),
    [sym_wc_data_record] = ACTIONS(1735),
    [sym_last_login_info] = ACTIONS(1735),
    [sym____reserved___] = ACTIONS(1735),
    [sym_date] = ACTIONS(1735),
    [sym_day] = ACTIONS(1735),
    [sym_time] = ACTIONS(1735),
    [sym_ldate] = ACTIONS(1735),
    [sym_push_device] = ACTIONS(1735),
    [sym_push_channel] = ACTIONS(1735),
    [sym_push_devchan] = ACTIONS(1735),
    [sym_release_device] = ACTIONS(1735),
    [sym_release_channel] = ACTIONS(1735),
    [sym_release_devchan] = ACTIONS(1735),
    [sym_master_slot] = ACTIONS(1735),
    [sym_get_pulse_time] = ACTIONS(1735),
    [sym_get_timer] = ACTIONS(1735),
    [sym_master_sn] = ACTIONS(1735),
    [sym_system_number] = ACTIONS(1735),
    [sym_button] = ACTIONS(1735),
    [sym_dv_channel] = ACTIONS(1735),
    [sym_data] = ACTIONS(1735),
    [sym_level] = ACTIONS(1735),
    [sym_timeline] = ACTIONS(1735),
    [sym_channel] = ACTIONS(1735),
    [sym_custom] = ACTIONS(1735),
    [sym_atoi] = ACTIONS(1735),
    [sym_atol] = ACTIONS(1735),
    [sym_atof] = ACTIONS(1735),
    [sym_itoa] = ACTIONS(1735),
    [sym_itohex] = ACTIONS(1735),
    [sym_hextoi] = ACTIONS(1735),
    [sym_ftoa] = ACTIONS(1735),
    [sym_find_string] = ACTIONS(1735),
    [sym_left_string] = ACTIONS(1735),
    [sym_right_string] = ACTIONS(1735),
    [sym_mid_string] = ACTIONS(1735),
    [sym_remove_string] = ACTIONS(1735),
    [sym_get_buffer_string] = ACTIONS(1735),
    [sym_get_buffer_char] = ACTIONS(1735),
    [sym_get_multi_buffer_string] = ACTIONS(1735),
    [sym_length_string] = ACTIONS(1735),
    [sym_max_length_string] = ACTIONS(1735),
    [sym_set_length_string] = ACTIONS(1735),
    [sym_compare_string] = ACTIONS(1735),
    [sym_upper_string] = ACTIONS(1735),
    [sym_lower_string] = ACTIONS(1735),
    [sym_get_last] = ACTIONS(1735),
    [sym_date_to_day] = ACTIONS(1735),
    [sym_date_to_month] = ACTIONS(1735),
    [sym_date_to_year] = ACTIONS(1735),
    [sym_time_to_hour] = ACTIONS(1735),
    [sym_time_to_minute] = ACTIONS(1735),
    [sym_time_to_second] = ACTIONS(1735),
    [sym_day_of_week] = ACTIONS(1735),
    [sym_set_pulse_time] = ACTIONS(1735),
    [sym_set_timer] = ACTIONS(1735),
    [sym_file_open] = ACTIONS(1735),
    [sym_file_close] = ACTIONS(1735),
    [sym_file_read] = ACTIONS(1735),
    [sym_file_read_line] = ACTIONS(1735),
    [sym_file_write] = ACTIONS(1735),
    [sym_file_write_line] = ACTIONS(1735),
    [sym_file_copy] = ACTIONS(1735),
    [sym_file_delete] = ACTIONS(1735),
    [sym_file_rename] = ACTIONS(1735),
    [sym_file_dir] = ACTIONS(1735),
    [sym_file_getdir] = ACTIONS(1735),
    [sym_file_setdir] = ACTIONS(1735),
    [sym_file_create_dir] = ACTIONS(1735),
    [sym_file_remove_dir] = ACTIONS(1735),
    [sym_file_seek] = ACTIONS(1735),
    [sym_length_array] = ACTIONS(1735),
    [sym_max_length_array] = ACTIONS(1735),
    [sym_set_length_array] = ACTIONS(1735),
    [sym_ip_client_open] = ACTIONS(1735),
    [sym_ip_client_close] = ACTIONS(1735),
    [sym_ip_server_open] = ACTIONS(1735),
    [sym_ip_server_close] = ACTIONS(1735),
    [sym_ip_bound_client_open] = ACTIONS(1735),
    [sym_ip_mc_server_open] = ACTIONS(1735),
    [sym_ip_set_option] = ACTIONS(1735),
    [sym_ssh_client_open] = ACTIONS(1735),
    [sym_ssh_client_close] = ACTIONS(1735),
    [sym_tls_client_open] = ACTIONS(1735),
    [sym_tls_client_close] = ACTIONS(1735),
    [sym_do_push] = ACTIONS(1735),
    [sym_do_push_timed] = ACTIONS(1735),
    [sym_do_release] = ACTIONS(1735),
    [sym_device_id] = ACTIONS(1735),
    [sym_device_info] = ACTIONS(1735),
    [sym_device_id_string] = ACTIONS(1735),
    [sym_device_standby] = ACTIONS(1735),
    [sym_device_wake] = ACTIONS(1735),
    [sym_timeline_create] = ACTIONS(1735),
    [sym_timeline_kill] = ACTIONS(1735),
    [sym_timeline_reload] = ACTIONS(1735),
    [sym_timeline_pause] = ACTIONS(1735),
    [sym_timeline_restart] = ACTIONS(1735),
    [sym_timeline_set] = ACTIONS(1735),
    [sym_timeline_get] = ACTIONS(1735),
    [sym_timeline_active] = ACTIONS(1735),
    [sym_timeline_dynamic_id] = ACTIONS(1735),
    [sym_variable_to_string] = ACTIONS(1735),
    [sym_string_to_variable] = ACTIONS(1735),
    [sym_length_variable_to_string] = ACTIONS(1735),
    [sym_type_cast] = ACTIONS(1735),
    [sym_variable_to_xml] = ACTIONS(1735),
    [sym_xml_to_variable] = ACTIONS(1735),
    [sym_length_variable_to_xml] = ACTIONS(1735),
    [sym_redirect_string] = ACTIONS(1735),
    [sym_random_number] = ACTIONS(1735),
    [sym_get_unique_id] = ACTIONS(1735),
    [sym_get_system_number] = ACTIONS(1735),
    [sym_set_system_number] = ACTIONS(1735),
    [sym_get_serial_number] = ACTIONS(1735),
    [sym_get_master_build] = ACTIONS(1735),
    [sym_set_validation_code] = ACTIONS(1735),
    [sym_reboot] = ACTIONS(1735),
    [sym_load_duet_module] = ACTIONS(1735),
    [sym_unload_duet_module] = ACTIONS(1735),
    [sym_duet_mem_size_set] = ACTIONS(1735),
    [sym_duet_mem_size_get] = ACTIONS(1735),
    [sym_dynamic_application_device] = ACTIONS(1735),
    [sym_dynamic_polled_port] = ACTIONS(1735),
    [sym_static_port_binding] = ACTIONS(1735),
    [sym_static_ip_binding] = ACTIONS(1735),
    [sym_get_dns_list] = ACTIONS(1735),
    [sym_set_dns_list] = ACTIONS(1735),
    [sym_get_ip_address] = ACTIONS(1735),
    [sym_set_ip_address] = ACTIONS(1735),
    [sym_get_url_list] = ACTIONS(1735),
    [sym_add_url_entry] = ACTIONS(1735),
    [sym_delete_url_entry] = ACTIONS(1735),
    [sym_abs_value] = ACTIONS(1735),
    [sym_max_value] = ACTIONS(1735),
    [sym_min_value] = ACTIONS(1735),
    [sym_exp_value] = ACTIONS(1735),
    [sym_log_value] = ACTIONS(1735),
    [sym_log10_value] = ACTIONS(1735),
    [sym_power_value] = ACTIONS(1735),
    [sym_sqrt_value] = ACTIONS(1735),
    [sym_raw_be] = ACTIONS(1735),
    [sym_raw_le] = ACTIONS(1735),
    [sym_internal_queue_size_set] = ACTIONS(1735),
    [sym_internal_queue_size_get] = ACTIONS(1735),
    [sym_internal_threshold_set] = ACTIONS(1735),
    [sym_internal_threshold_get] = ACTIONS(1735),
    [sym_rebuilt_event] = ACTIONS(1735),
    [sym_do_custom_event] = ACTIONS(1735),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1735),
    [sym_clkmgr_set_clk_source] = ACTIONS(1735),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1735),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1735),
    [sym_clkmgr_get_timezone] = ACTIONS(1735),
    [sym_clkmgr_set_timezone] = ACTIONS(1735),
    [sym_clkmgr_get_resync_period] = ACTIONS(1735),
    [sym_clkmgr_set_resync_period] = ACTIONS(1735),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1735),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1735),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1735),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1735),
    [sym_clkmgr_get_timeservers] = ACTIONS(1735),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1735),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1735),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1735),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1735),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1735),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1735),
    [sym_set_log_level] = ACTIONS(1735),
    [sym_get_log_level] = ACTIONS(1735),
    [sym_amx_log] = ACTIONS(1735),
    [sym_smtp_server_config_set] = ACTIONS(1735),
    [sym_smtp_server_config_get] = ACTIONS(1735),
    [sym_smtp_send] = ACTIONS(1735),
    [sym_validate_netlinx_account] = ACTIONS(1735),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1735),
    [sym_authenticate_certificate] = ACTIONS(1735),
    [sym_audit_netlinx_session_event] = ACTIONS(1735),
    [sym_audit_netlinx_generic_event] = ACTIONS(1735),
    [sym_get_available_flash_disk_space] = ACTIONS(1735),
    [sym_get_max_flash_disk_space] = ACTIONS(1735),
    [sym_data_create_feed] = ACTIONS(1735),
    [sym_data_delete_feed] = ACTIONS(1735),
    [sym_data_publish_feed] = ACTIONS(1735),
    [sym_data_get_published_feed] = ACTIONS(1735),
    [sym_data_add_record] = ACTIONS(1735),
    [sym_data_get_event_record] = ACTIONS(1735),
    [sym__wc_data_create_feed] = ACTIONS(1735),
    [sym__wc_data_add_record] = ACTIONS(1735),
    [sym__wc_data_get_event_record] = ACTIONS(1735),
    [sym_format] = ACTIONS(1735),
    [sym_astro_clock] = ACTIONS(1735),
    [sym_set_outdoor_temperature] = ACTIONS(1735),
    [sym_set_virtual_level_count] = ACTIONS(1735),
    [sym_set_virtual_channel_count] = ACTIONS(1735),
    [sym_set_virtual_port_count] = ACTIONS(1735),
    [sym__wc_encode_int] = ACTIONS(1735),
    [sym__wc_decode_int] = ACTIONS(1735),
    [sym__wc_int] = ACTIONS(1735),
    [sym__wc_to_ch_int] = ACTIONS(1735),
    [sym__ch_to_wc_int] = ACTIONS(1735),
    [sym__wc_find_string_int] = ACTIONS(1735),
    [sym__wc_left_string_int] = ACTIONS(1735),
    [sym__wc_lower_string_int] = ACTIONS(1735),
    [sym__wc_mid_string_int] = ACTIONS(1735),
    [sym__wc_remove_string_int] = ACTIONS(1735),
    [sym__wc_right_string_int] = ACTIONS(1735),
    [sym__wc_upper_string_int] = ACTIONS(1735),
    [sym__wc_compare_string_int] = ACTIONS(1735),
    [sym__wc_get_buffer_char_int] = ACTIONS(1735),
    [sym__wc_get_buffer_string_int] = ACTIONS(1735),
    [sym__wc_concat_string_int] = ACTIONS(1735),
    [sym__wc_file_open_int] = ACTIONS(1735),
    [sym__wc_file_close_int] = ACTIONS(1735),
    [sym__wc_file_read_int] = ACTIONS(1735),
    [sym__wc_file_read_line_int] = ACTIONS(1735),
    [sym__wc_file_write_int] = ACTIONS(1735),
    [sym__wc_file_write_line_int] = ACTIONS(1735),
    [aux_sym_preproc_include_token1] = ACTIONS(1735),
    [aux_sym_preproc_define_token1] = ACTIONS(1737),
    [aux_sym_preproc_warn_token1] = ACTIONS(1737),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1737),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1737),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1737),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1737),
    [aux_sym_preproc_else_token1] = ACTIONS(1737),
    [anon_sym_LBRACK] = ACTIONS(1737),
    [anon_sym_LBRACE] = ACTIONS(1737),
    [anon_sym_LPAREN] = ACTIONS(1735),
    [anon_sym_BANG] = ACTIONS(1737),
    [anon_sym_TILDE] = ACTIONS(1737),
    [anon_sym_DASH] = ACTIONS(1735),
    [anon_sym_PLUS] = ACTIONS(1735),
    [anon_sym_DASH_DASH] = ACTIONS(1737),
    [anon_sym_PLUS_PLUS] = ACTIONS(1737),
    [anon_sym_DQUOTE] = ACTIONS(1737),
    [anon_sym_SQUOTE] = ACTIONS(1737),
    [sym_number_literal] = ACTIONS(1737),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(214)] = {
    [sym_identifier] = ACTIONS(1739),
    [sym_program_name_keyword] = ACTIONS(1739),
    [sym_module_name_keyword] = ACTIONS(1739),
    [sym_define_device_keyword] = ACTIONS(1739),
    [sym_define_combine_keyword] = ACTIONS(1739),
    [sym_define_constant_keyword] = ACTIONS(1739),
    [sym_define_type_keyword] = ACTIONS(1739),
    [sym_define_variable_keyword] = ACTIONS(1739),
    [sym_define_system_variable_keyword] = ACTIONS(1739),
    [sym_define_connect_level_keyword] = ACTIONS(1739),
    [sym_define_latching_keyword] = ACTIONS(1739),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1739),
    [sym_define_toggling_keyword] = ACTIONS(1739),
    [sym_define_call_keyword] = ACTIONS(1739),
    [sym_define_function_keyword] = ACTIONS(1739),
    [sym_define_library_function_keyword] = ACTIONS(1739),
    [sym_define_start_keyword] = ACTIONS(1739),
    [sym_define_event_keyword] = ACTIONS(1739),
    [sym_define_program_keyword] = ACTIONS(1739),
    [sym_define_module_keyword] = ACTIONS(1739),
    [sym_char_keyword] = ACTIONS(1739),
    [sym_widechar_keyword] = ACTIONS(1739),
    [sym_integer_keyword] = ACTIONS(1739),
    [sym_sinteger_keyword] = ACTIONS(1739),
    [sym_long_keyword] = ACTIONS(1739),
    [sym_slong_keyword] = ACTIONS(1739),
    [sym_float_keyword] = ACTIONS(1739),
    [sym_double_keyword] = ACTIONS(1739),
    [sym_variant_keyword] = ACTIONS(1739),
    [sym_variantarray_keyword] = ACTIONS(1739),
    [sym_local_var_keyword] = ACTIONS(1739),
    [sym_stack_var_keyword] = ACTIONS(1739),
    [sym_constant_keyword] = ACTIONS(1739),
    [sym_volatile_keyword] = ACTIONS(1739),
    [sym_non_volatile_keyword] = ACTIONS(1739),
    [sym_persistent_keyword] = ACTIONS(1739),
    [sym_dev_keyword] = ACTIONS(1739),
    [sym_devlev_keyword] = ACTIONS(1739),
    [sym_devchan_keyword] = ACTIONS(1739),
    [sym_if_keyword] = ACTIONS(1739),
    [sym_else_keyword] = ACTIONS(1739),
    [sym_switch_keyword] = ACTIONS(1739),
    [sym_case_keyword] = ACTIONS(1739),
    [sym_default_keyword] = ACTIONS(1739),
    [sym_while_keyword] = ACTIONS(1739),
    [sym_for_keyword] = ACTIONS(1739),
    [sym_break_keyword] = ACTIONS(1739),
    [sym_continue_keyword] = ACTIONS(1739),
    [sym_return_keyword] = ACTIONS(1739),
    [sym_select_keyword] = ACTIONS(1739),
    [sym_active_keyword] = ACTIONS(1739),
    [sym_struct_keyword] = ACTIONS(1739),
    [sym_structure_keyword] = ACTIONS(1739),
    [sym_button_event_keyword] = ACTIONS(1739),
    [sym_channel_event_keyword] = ACTIONS(1739),
    [sym_level_event_keyword] = ACTIONS(1739),
    [sym_data_event_keyword] = ACTIONS(1739),
    [sym_timeline_event_keyword] = ACTIONS(1739),
    [sym_custom_event_keyword] = ACTIONS(1739),
    [sym_push_keyword] = ACTIONS(1739),
    [sym_release_keyword] = ACTIONS(1739),
    [aux_sym_on_keyword_token1] = ACTIONS(1739),
    [aux_sym_off_keyword_token1] = ACTIONS(1739),
    [sym_send_level_keyword] = ACTIONS(1739),
    [sym_send_string_keyword] = ACTIONS(1739),
    [sym_send_command_keyword] = ACTIONS(1739),
    [sym_clear_buffer_keyword] = ACTIONS(1739),
    [sym_create_buffer_keyword] = ACTIONS(1739),
    [sym_create_multi_buffer_keyword] = ACTIONS(1739),
    [sym_call_keyword] = ACTIONS(1739),
    [sym_system_call_keyword] = ACTIONS(1739),
    [sym_devchan_to_keyword] = ACTIONS(1739),
    [sym_devchan_min_to_keyword] = ACTIONS(1739),
    [sym_devchan_total_off_keyword] = ACTIONS(1739),
    [sym_devchan_pulse_keyword] = ACTIONS(1739),
    [sym_wait_keyword] = ACTIONS(1739),
    [sym_cancel_wait_keyword] = ACTIONS(1739),
    [sym_wait_until_keyword] = ACTIONS(1739),
    [sym_cancel_wait_until_keyword] = ACTIONS(1739),
    [sym_cancel_all_wait_keyword] = ACTIONS(1739),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1739),
    [sym_not] = ACTIONS(1739),
    [sym_bnot] = ACTIONS(1739),
    [sym___file__] = ACTIONS(1739),
    [sym___date__] = ACTIONS(1739),
    [sym___ldate__] = ACTIONS(1739),
    [sym___time__] = ACTIONS(1739),
    [sym___line__] = ACTIONS(1739),
    [sym___name__] = ACTIONS(1739),
    [sym___netlinx__] = ACTIONS(1739),
    [sym_netlinx_axi_version] = ACTIONS(1739),
    [sym_url_flg_tcp] = ACTIONS(1739),
    [sym_url_flg_accinfopresent] = ACTIONS(1739),
    [sym_url_flg_temp] = ACTIONS(1739),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1739),
    [sym_url_flg_stat_mask] = ACTIONS(1739),
    [sym_url_flg_stat_lookup] = ACTIONS(1739),
    [sym_url_flg_stat_connecting] = ACTIONS(1739),
    [sym_url_flg_stat_waiting] = ACTIONS(1739),
    [sym_url_flg_stat_connected] = ACTIONS(1739),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1739),
    [sym_first_virtual_device] = ACTIONS(1739),
    [sym_first_local_port] = ACTIONS(1739),
    [sym_dynamic_virtual_device] = ACTIONS(1739),
    [sym_timeline_once] = ACTIONS(1739),
    [sym_timeline_repeat] = ACTIONS(1739),
    [sym_timeline_absolute] = ACTIONS(1739),
    [sym_timeline_relative] = ACTIONS(1739),
    [sym_do_push_timed_infinite] = ACTIONS(1739),
    [sym_source_type_no_address] = ACTIONS(1739),
    [sym_source_type_neuron_id] = ACTIONS(1739),
    [sym_source_type_ip_address] = ACTIONS(1739),
    [sym_source_type_axlink] = ACTIONS(1739),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1739),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1739),
    [sym_source_type_ip_socket_address] = ACTIONS(1739),
    [sym_source_type_rs232] = ACTIONS(1739),
    [sym_source_type_internal] = ACTIONS(1739),
    [sym_source_type_mac_address] = ACTIONS(1739),
    [sym_source_type_ipv4_port] = ACTIONS(1739),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1739),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1739),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1739),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1739),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1739),
    [sym_true] = ACTIONS(1739),
    [sym_false] = ACTIONS(1739),
    [sym_file_read_only] = ACTIONS(1739),
    [sym_file_rw_new] = ACTIONS(1739),
    [sym_file_rw_append] = ACTIONS(1739),
    [sym_ip_tcp] = ACTIONS(1739),
    [sym_ip_udp] = ACTIONS(1739),
    [sym_ip_udp_2way] = ACTIONS(1739),
    [sym_xml_encode_types] = ACTIONS(1739),
    [sym_xml_encode_char_as_list] = ACTIONS(1739),
    [sym_xml_encode_le] = ACTIONS(1739),
    [sym_xml_decode_types] = ACTIONS(1739),
    [sym_xml_decode_no_preserve] = ACTIONS(1739),
    [sym_duet_dev_type_display_device] = ACTIONS(1739),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1739),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1739),
    [sym_duet_dev_type_io_device] = ACTIONS(1739),
    [sym_duet_dev_type_relay_device] = ACTIONS(1739),
    [sym_duet_dev_type_ups] = ACTIONS(1739),
    [sym_duet_dev_type_amplifier] = ACTIONS(1739),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1739),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1739),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1739),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1739),
    [sym_duet_dev_type_camera] = ACTIONS(1739),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1739),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1739),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1739),
    [sym_duet_dev_type_dss] = ACTIONS(1739),
    [sym_duet_dev_type_dvr] = ACTIONS(1739),
    [sym_duet_dev_type_disc_device] = ACTIONS(1739),
    [sym_duet_dev_type_document_camera] = ACTIONS(1739),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1739),
    [sym_duet_dev_type_hvac] = ACTIONS(1739),
    [sym_duet_dev_type_keypad] = ACTIONS(1739),
    [sym_duet_dev_type_light] = ACTIONS(1739),
    [sym_duet_dev_type_monitor] = ACTIONS(1739),
    [sym_duet_dev_type_motor] = ACTIONS(1739),
    [sym_duet_dev_type_multi_window] = ACTIONS(1739),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1739),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1739),
    [sym_duet_dev_type_receiver] = ACTIONS(1739),
    [sym_duet_dev_type_security_system] = ACTIONS(1739),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1739),
    [sym_duet_dev_type_settop_box] = ACTIONS(1739),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1739),
    [sym_duet_dev_type_switcher] = ACTIONS(1739),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1739),
    [sym_duet_dev_type_tv] = ACTIONS(1739),
    [sym_duet_dev_type_utility] = ACTIONS(1739),
    [sym_duet_dev_type_vcr] = ACTIONS(1739),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1739),
    [sym_duet_dev_type_video_processor] = ACTIONS(1739),
    [sym_duet_dev_type_video_projector] = ACTIONS(1739),
    [sym_duet_dev_type_video_wall] = ACTIONS(1739),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1739),
    [sym_duet_dev_type_weather] = ACTIONS(1739),
    [sym_duet_dev_not_polled] = ACTIONS(1739),
    [sym_duet_dev_polled] = ACTIONS(1739),
    [sym_clkmgr_mode_network] = ACTIONS(1739),
    [sym_clkmgr_mode_standalone] = ACTIONS(1739),
    [sym_normal_standby] = ACTIONS(1739),
    [sym_normal_wake] = ACTIONS(1739),
    [sym_ip_multicast_ttl_option] = ACTIONS(1739),
    [sym_ip_ttl_subnet] = ACTIONS(1739),
    [sym_ip_ttl_site] = ACTIONS(1739),
    [sym_ip_ttl_region] = ACTIONS(1739),
    [sym_ip_ttl_continent] = ACTIONS(1739),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1739),
    [sym_ip_nodelay_on] = ACTIONS(1739),
    [sym_ip_nodelay_off] = ACTIONS(1739),
    [sym_amx_error] = ACTIONS(1739),
    [sym_amx_warning] = ACTIONS(1739),
    [sym_amx_info] = ACTIONS(1739),
    [sym_amx_debug] = ACTIONS(1739),
    [sym_netlinx_login_success] = ACTIONS(1739),
    [sym_netlinx_login_fail] = ACTIONS(1739),
    [sym_netlinx_logout] = ACTIONS(1739),
    [sym_null_str] = ACTIONS(1739),
    [sym_smtp_address] = ACTIONS(1739),
    [sym_smtp_port_number] = ACTIONS(1739),
    [sym_smtp_username] = ACTIONS(1739),
    [sym_smtp_password] = ACTIONS(1739),
    [sym_smtp_require_tls] = ACTIONS(1739),
    [sym_smtp_from] = ACTIONS(1739),
    [sym_smtp_tls_true] = ACTIONS(1739),
    [sym_smtp_tls_false] = ACTIONS(1739),
    [sym_listview_on_row_select_event] = ACTIONS(1739),
    [sym_data_structure_unknown] = ACTIONS(1739),
    [sym_data_structure_datafeed] = ACTIONS(1739),
    [sym_data_structure_datarecordset] = ACTIONS(1739),
    [sym_data_structure_datarecord] = ACTIONS(1739),
    [sym_data_structure_datafield] = ACTIONS(1739),
    [sym_data_max_content_field_count] = ACTIONS(1739),
    [sym_data_max_metadata_field_count] = ACTIONS(1739),
    [sym_data_max_id_length] = ACTIONS(1739),
    [sym_data_max_name_length] = ACTIONS(1739),
    [sym_data_max_description_length] = ACTIONS(1739),
    [sym_data_max_type_length] = ACTIONS(1739),
    [sym_data_max_format_length] = ACTIONS(1739),
    [sym_data_max_source_length] = ACTIONS(1739),
    [sym_data_max_value_length] = ACTIONS(1739),
    [sym_data_max_label_length] = ACTIONS(1739),
    [sym_data_max_url_length] = ACTIONS(1739),
    [sym_max_last_login_info_length] = ACTIONS(1739),
    [sym_data_type_unknown] = ACTIONS(1739),
    [sym_data_type_string] = ACTIONS(1739),
    [sym_data_type_datetime] = ACTIONS(1739),
    [sym_data_type_date] = ACTIONS(1739),
    [sym_data_type_time] = ACTIONS(1739),
    [sym_data_type_image] = ACTIONS(1739),
    [sym_data_format_url] = ACTIONS(1739),
    [sym_data_format_phone] = ACTIONS(1739),
    [sym_data_format_email] = ACTIONS(1739),
    [sym_data_format_iso8601] = ACTIONS(1739),
    [sym_auditlog_priv] = ACTIONS(1739),
    [sym_configuration_priv] = ACTIONS(1739),
    [sym_deviceconfig_priv] = ACTIONS(1739),
    [sym_ftp_priv] = ACTIONS(1739),
    [sym_http_priv] = ACTIONS(1739),
    [sym_networkconfig_priv] = ACTIONS(1739),
    [sym_programport_priv] = ACTIONS(1739),
    [sym_remoteui_priv] = ACTIONS(1739),
    [sym_securitycontrol_priv] = ACTIONS(1739),
    [sym_softwaremanagement_priv] = ACTIONS(1739),
    [sym_terminal_priv] = ACTIONS(1739),
    [sym_tpadmin_priv] = ACTIONS(1739),
    [sym_user1_priv] = ACTIONS(1739),
    [sym_user2_priv] = ACTIONS(1739),
    [sym_user3_priv] = ACTIONS(1739),
    [sym_user4_priv] = ACTIONS(1739),
    [sym_usermanagement_priv] = ACTIONS(1739),
    [sym_valid_account] = ACTIONS(1739),
    [sym_err_name_invalid] = ACTIONS(1739),
    [sym_err_password_invalid] = ACTIONS(1739),
    [sym_err_invalid_account] = ACTIONS(1739),
    [sym_err_no_authorization] = ACTIONS(1739),
    [sym_err_invalid_parameter] = ACTIONS(1739),
    [sym_err_user_locked_out] = ACTIONS(1739),
    [sym_err_user_account_expired] = ACTIONS(1739),
    [sym_err_general_error] = ACTIONS(1739),
    [sym_err_authentication_error] = ACTIONS(1739),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1739),
    [sym_tls_validate_certificate] = ACTIONS(1739),
    [sym_tdata] = ACTIONS(1739),
    [sym_tchannel] = ACTIONS(1739),
    [sym_tlevel] = ACTIONS(1739),
    [sym_tbutton] = ACTIONS(1739),
    [sym_ttimeline] = ACTIONS(1739),
    [sym_tcustom] = ACTIONS(1739),
    [sym_url_struct] = ACTIONS(1739),
    [sym_dns_struct] = ACTIONS(1739),
    [sym_ip_address_struct] = ACTIONS(1739),
    [sym_dev_info_struct] = ACTIONS(1739),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1739),
    [sym_data_feed] = ACTIONS(1739),
    [sym_data_field] = ACTIONS(1739),
    [sym_data_record] = ACTIONS(1739),
    [sym_wc_data_feed] = ACTIONS(1739),
    [sym_wc_data_field] = ACTIONS(1739),
    [sym_wc_data_record] = ACTIONS(1739),
    [sym_last_login_info] = ACTIONS(1739),
    [sym____reserved___] = ACTIONS(1739),
    [sym_date] = ACTIONS(1739),
    [sym_day] = ACTIONS(1739),
    [sym_time] = ACTIONS(1739),
    [sym_ldate] = ACTIONS(1739),
    [sym_push_device] = ACTIONS(1739),
    [sym_push_channel] = ACTIONS(1739),
    [sym_push_devchan] = ACTIONS(1739),
    [sym_release_device] = ACTIONS(1739),
    [sym_release_channel] = ACTIONS(1739),
    [sym_release_devchan] = ACTIONS(1739),
    [sym_master_slot] = ACTIONS(1739),
    [sym_get_pulse_time] = ACTIONS(1739),
    [sym_get_timer] = ACTIONS(1739),
    [sym_master_sn] = ACTIONS(1739),
    [sym_system_number] = ACTIONS(1739),
    [sym_button] = ACTIONS(1739),
    [sym_dv_channel] = ACTIONS(1739),
    [sym_data] = ACTIONS(1739),
    [sym_level] = ACTIONS(1739),
    [sym_timeline] = ACTIONS(1739),
    [sym_channel] = ACTIONS(1739),
    [sym_custom] = ACTIONS(1739),
    [sym_atoi] = ACTIONS(1739),
    [sym_atol] = ACTIONS(1739),
    [sym_atof] = ACTIONS(1739),
    [sym_itoa] = ACTIONS(1739),
    [sym_itohex] = ACTIONS(1739),
    [sym_hextoi] = ACTIONS(1739),
    [sym_ftoa] = ACTIONS(1739),
    [sym_find_string] = ACTIONS(1739),
    [sym_left_string] = ACTIONS(1739),
    [sym_right_string] = ACTIONS(1739),
    [sym_mid_string] = ACTIONS(1739),
    [sym_remove_string] = ACTIONS(1739),
    [sym_get_buffer_string] = ACTIONS(1739),
    [sym_get_buffer_char] = ACTIONS(1739),
    [sym_get_multi_buffer_string] = ACTIONS(1739),
    [sym_length_string] = ACTIONS(1739),
    [sym_max_length_string] = ACTIONS(1739),
    [sym_set_length_string] = ACTIONS(1739),
    [sym_compare_string] = ACTIONS(1739),
    [sym_upper_string] = ACTIONS(1739),
    [sym_lower_string] = ACTIONS(1739),
    [sym_get_last] = ACTIONS(1739),
    [sym_date_to_day] = ACTIONS(1739),
    [sym_date_to_month] = ACTIONS(1739),
    [sym_date_to_year] = ACTIONS(1739),
    [sym_time_to_hour] = ACTIONS(1739),
    [sym_time_to_minute] = ACTIONS(1739),
    [sym_time_to_second] = ACTIONS(1739),
    [sym_day_of_week] = ACTIONS(1739),
    [sym_set_pulse_time] = ACTIONS(1739),
    [sym_set_timer] = ACTIONS(1739),
    [sym_file_open] = ACTIONS(1739),
    [sym_file_close] = ACTIONS(1739),
    [sym_file_read] = ACTIONS(1739),
    [sym_file_read_line] = ACTIONS(1739),
    [sym_file_write] = ACTIONS(1739),
    [sym_file_write_line] = ACTIONS(1739),
    [sym_file_copy] = ACTIONS(1739),
    [sym_file_delete] = ACTIONS(1739),
    [sym_file_rename] = ACTIONS(1739),
    [sym_file_dir] = ACTIONS(1739),
    [sym_file_getdir] = ACTIONS(1739),
    [sym_file_setdir] = ACTIONS(1739),
    [sym_file_create_dir] = ACTIONS(1739),
    [sym_file_remove_dir] = ACTIONS(1739),
    [sym_file_seek] = ACTIONS(1739),
    [sym_length_array] = ACTIONS(1739),
    [sym_max_length_array] = ACTIONS(1739),
    [sym_set_length_array] = ACTIONS(1739),
    [sym_ip_client_open] = ACTIONS(1739),
    [sym_ip_client_close] = ACTIONS(1739),
    [sym_ip_server_open] = ACTIONS(1739),
    [sym_ip_server_close] = ACTIONS(1739),
    [sym_ip_bound_client_open] = ACTIONS(1739),
    [sym_ip_mc_server_open] = ACTIONS(1739),
    [sym_ip_set_option] = ACTIONS(1739),
    [sym_ssh_client_open] = ACTIONS(1739),
    [sym_ssh_client_close] = ACTIONS(1739),
    [sym_tls_client_open] = ACTIONS(1739),
    [sym_tls_client_close] = ACTIONS(1739),
    [sym_do_push] = ACTIONS(1739),
    [sym_do_push_timed] = ACTIONS(1739),
    [sym_do_release] = ACTIONS(1739),
    [sym_device_id] = ACTIONS(1739),
    [sym_device_info] = ACTIONS(1739),
    [sym_device_id_string] = ACTIONS(1739),
    [sym_device_standby] = ACTIONS(1739),
    [sym_device_wake] = ACTIONS(1739),
    [sym_timeline_create] = ACTIONS(1739),
    [sym_timeline_kill] = ACTIONS(1739),
    [sym_timeline_reload] = ACTIONS(1739),
    [sym_timeline_pause] = ACTIONS(1739),
    [sym_timeline_restart] = ACTIONS(1739),
    [sym_timeline_set] = ACTIONS(1739),
    [sym_timeline_get] = ACTIONS(1739),
    [sym_timeline_active] = ACTIONS(1739),
    [sym_timeline_dynamic_id] = ACTIONS(1739),
    [sym_variable_to_string] = ACTIONS(1739),
    [sym_string_to_variable] = ACTIONS(1739),
    [sym_length_variable_to_string] = ACTIONS(1739),
    [sym_type_cast] = ACTIONS(1739),
    [sym_variable_to_xml] = ACTIONS(1739),
    [sym_xml_to_variable] = ACTIONS(1739),
    [sym_length_variable_to_xml] = ACTIONS(1739),
    [sym_redirect_string] = ACTIONS(1739),
    [sym_random_number] = ACTIONS(1739),
    [sym_get_unique_id] = ACTIONS(1739),
    [sym_get_system_number] = ACTIONS(1739),
    [sym_set_system_number] = ACTIONS(1739),
    [sym_get_serial_number] = ACTIONS(1739),
    [sym_get_master_build] = ACTIONS(1739),
    [sym_set_validation_code] = ACTIONS(1739),
    [sym_reboot] = ACTIONS(1739),
    [sym_load_duet_module] = ACTIONS(1739),
    [sym_unload_duet_module] = ACTIONS(1739),
    [sym_duet_mem_size_set] = ACTIONS(1739),
    [sym_duet_mem_size_get] = ACTIONS(1739),
    [sym_dynamic_application_device] = ACTIONS(1739),
    [sym_dynamic_polled_port] = ACTIONS(1739),
    [sym_static_port_binding] = ACTIONS(1739),
    [sym_static_ip_binding] = ACTIONS(1739),
    [sym_get_dns_list] = ACTIONS(1739),
    [sym_set_dns_list] = ACTIONS(1739),
    [sym_get_ip_address] = ACTIONS(1739),
    [sym_set_ip_address] = ACTIONS(1739),
    [sym_get_url_list] = ACTIONS(1739),
    [sym_add_url_entry] = ACTIONS(1739),
    [sym_delete_url_entry] = ACTIONS(1739),
    [sym_abs_value] = ACTIONS(1739),
    [sym_max_value] = ACTIONS(1739),
    [sym_min_value] = ACTIONS(1739),
    [sym_exp_value] = ACTIONS(1739),
    [sym_log_value] = ACTIONS(1739),
    [sym_log10_value] = ACTIONS(1739),
    [sym_power_value] = ACTIONS(1739),
    [sym_sqrt_value] = ACTIONS(1739),
    [sym_raw_be] = ACTIONS(1739),
    [sym_raw_le] = ACTIONS(1739),
    [sym_internal_queue_size_set] = ACTIONS(1739),
    [sym_internal_queue_size_get] = ACTIONS(1739),
    [sym_internal_threshold_set] = ACTIONS(1739),
    [sym_internal_threshold_get] = ACTIONS(1739),
    [sym_rebuilt_event] = ACTIONS(1739),
    [sym_do_custom_event] = ACTIONS(1739),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1739),
    [sym_clkmgr_set_clk_source] = ACTIONS(1739),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1739),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1739),
    [sym_clkmgr_get_timezone] = ACTIONS(1739),
    [sym_clkmgr_set_timezone] = ACTIONS(1739),
    [sym_clkmgr_get_resync_period] = ACTIONS(1739),
    [sym_clkmgr_set_resync_period] = ACTIONS(1739),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1739),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1739),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1739),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1739),
    [sym_clkmgr_get_timeservers] = ACTIONS(1739),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1739),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1739),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1739),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1739),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1739),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1739),
    [sym_set_log_level] = ACTIONS(1739),
    [sym_get_log_level] = ACTIONS(1739),
    [sym_amx_log] = ACTIONS(1739),
    [sym_smtp_server_config_set] = ACTIONS(1739),
    [sym_smtp_server_config_get] = ACTIONS(1739),
    [sym_smtp_send] = ACTIONS(1739),
    [sym_validate_netlinx_account] = ACTIONS(1739),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1739),
    [sym_authenticate_certificate] = ACTIONS(1739),
    [sym_audit_netlinx_session_event] = ACTIONS(1739),
    [sym_audit_netlinx_generic_event] = ACTIONS(1739),
    [sym_get_available_flash_disk_space] = ACTIONS(1739),
    [sym_get_max_flash_disk_space] = ACTIONS(1739),
    [sym_data_create_feed] = ACTIONS(1739),
    [sym_data_delete_feed] = ACTIONS(1739),
    [sym_data_publish_feed] = ACTIONS(1739),
    [sym_data_get_published_feed] = ACTIONS(1739),
    [sym_data_add_record] = ACTIONS(1739),
    [sym_data_get_event_record] = ACTIONS(1739),
    [sym__wc_data_create_feed] = ACTIONS(1739),
    [sym__wc_data_add_record] = ACTIONS(1739),
    [sym__wc_data_get_event_record] = ACTIONS(1739),
    [sym_format] = ACTIONS(1739),
    [sym_astro_clock] = ACTIONS(1739),
    [sym_set_outdoor_temperature] = ACTIONS(1739),
    [sym_set_virtual_level_count] = ACTIONS(1739),
    [sym_set_virtual_channel_count] = ACTIONS(1739),
    [sym_set_virtual_port_count] = ACTIONS(1739),
    [sym__wc_encode_int] = ACTIONS(1739),
    [sym__wc_decode_int] = ACTIONS(1739),
    [sym__wc_int] = ACTIONS(1739),
    [sym__wc_to_ch_int] = ACTIONS(1739),
    [sym__ch_to_wc_int] = ACTIONS(1739),
    [sym__wc_find_string_int] = ACTIONS(1739),
    [sym__wc_left_string_int] = ACTIONS(1739),
    [sym__wc_lower_string_int] = ACTIONS(1739),
    [sym__wc_mid_string_int] = ACTIONS(1739),
    [sym__wc_remove_string_int] = ACTIONS(1739),
    [sym__wc_right_string_int] = ACTIONS(1739),
    [sym__wc_upper_string_int] = ACTIONS(1739),
    [sym__wc_compare_string_int] = ACTIONS(1739),
    [sym__wc_get_buffer_char_int] = ACTIONS(1739),
    [sym__wc_get_buffer_string_int] = ACTIONS(1739),
    [sym__wc_concat_string_int] = ACTIONS(1739),
    [sym__wc_file_open_int] = ACTIONS(1739),
    [sym__wc_file_close_int] = ACTIONS(1739),
    [sym__wc_file_read_int] = ACTIONS(1739),
    [sym__wc_file_read_line_int] = ACTIONS(1739),
    [sym__wc_file_write_int] = ACTIONS(1739),
    [sym__wc_file_write_line_int] = ACTIONS(1739),
    [aux_sym_preproc_include_token1] = ACTIONS(1739),
    [aux_sym_preproc_define_token1] = ACTIONS(1741),
    [aux_sym_preproc_warn_token1] = ACTIONS(1741),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1741),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1741),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1741),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1741),
    [aux_sym_preproc_else_token1] = ACTIONS(1741),
    [anon_sym_LBRACK] = ACTIONS(1741),
    [anon_sym_LBRACE] = ACTIONS(1741),
    [anon_sym_LPAREN] = ACTIONS(1739),
    [anon_sym_BANG] = ACTIONS(1741),
    [anon_sym_TILDE] = ACTIONS(1741),
    [anon_sym_DASH] = ACTIONS(1739),
    [anon_sym_PLUS] = ACTIONS(1739),
    [anon_sym_DASH_DASH] = ACTIONS(1741),
    [anon_sym_PLUS_PLUS] = ACTIONS(1741),
    [anon_sym_DQUOTE] = ACTIONS(1741),
    [anon_sym_SQUOTE] = ACTIONS(1741),
    [sym_number_literal] = ACTIONS(1741),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(215)] = {
    [sym_identifier] = ACTIONS(1743),
    [sym_program_name_keyword] = ACTIONS(1743),
    [sym_module_name_keyword] = ACTIONS(1743),
    [sym_define_device_keyword] = ACTIONS(1743),
    [sym_define_combine_keyword] = ACTIONS(1743),
    [sym_define_constant_keyword] = ACTIONS(1743),
    [sym_define_type_keyword] = ACTIONS(1743),
    [sym_define_variable_keyword] = ACTIONS(1743),
    [sym_define_system_variable_keyword] = ACTIONS(1743),
    [sym_define_connect_level_keyword] = ACTIONS(1743),
    [sym_define_latching_keyword] = ACTIONS(1743),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1743),
    [sym_define_toggling_keyword] = ACTIONS(1743),
    [sym_define_call_keyword] = ACTIONS(1743),
    [sym_define_function_keyword] = ACTIONS(1743),
    [sym_define_library_function_keyword] = ACTIONS(1743),
    [sym_define_start_keyword] = ACTIONS(1743),
    [sym_define_event_keyword] = ACTIONS(1743),
    [sym_define_program_keyword] = ACTIONS(1743),
    [sym_define_module_keyword] = ACTIONS(1743),
    [sym_char_keyword] = ACTIONS(1743),
    [sym_widechar_keyword] = ACTIONS(1743),
    [sym_integer_keyword] = ACTIONS(1743),
    [sym_sinteger_keyword] = ACTIONS(1743),
    [sym_long_keyword] = ACTIONS(1743),
    [sym_slong_keyword] = ACTIONS(1743),
    [sym_float_keyword] = ACTIONS(1743),
    [sym_double_keyword] = ACTIONS(1743),
    [sym_variant_keyword] = ACTIONS(1743),
    [sym_variantarray_keyword] = ACTIONS(1743),
    [sym_local_var_keyword] = ACTIONS(1743),
    [sym_stack_var_keyword] = ACTIONS(1743),
    [sym_constant_keyword] = ACTIONS(1743),
    [sym_volatile_keyword] = ACTIONS(1743),
    [sym_non_volatile_keyword] = ACTIONS(1743),
    [sym_persistent_keyword] = ACTIONS(1743),
    [sym_dev_keyword] = ACTIONS(1743),
    [sym_devlev_keyword] = ACTIONS(1743),
    [sym_devchan_keyword] = ACTIONS(1743),
    [sym_if_keyword] = ACTIONS(1743),
    [sym_else_keyword] = ACTIONS(1743),
    [sym_switch_keyword] = ACTIONS(1743),
    [sym_case_keyword] = ACTIONS(1743),
    [sym_default_keyword] = ACTIONS(1743),
    [sym_while_keyword] = ACTIONS(1743),
    [sym_for_keyword] = ACTIONS(1743),
    [sym_break_keyword] = ACTIONS(1743),
    [sym_continue_keyword] = ACTIONS(1743),
    [sym_return_keyword] = ACTIONS(1743),
    [sym_select_keyword] = ACTIONS(1743),
    [sym_active_keyword] = ACTIONS(1743),
    [sym_struct_keyword] = ACTIONS(1743),
    [sym_structure_keyword] = ACTIONS(1743),
    [sym_button_event_keyword] = ACTIONS(1743),
    [sym_channel_event_keyword] = ACTIONS(1743),
    [sym_level_event_keyword] = ACTIONS(1743),
    [sym_data_event_keyword] = ACTIONS(1743),
    [sym_timeline_event_keyword] = ACTIONS(1743),
    [sym_custom_event_keyword] = ACTIONS(1743),
    [sym_push_keyword] = ACTIONS(1743),
    [sym_release_keyword] = ACTIONS(1743),
    [aux_sym_on_keyword_token1] = ACTIONS(1743),
    [aux_sym_off_keyword_token1] = ACTIONS(1743),
    [sym_send_level_keyword] = ACTIONS(1743),
    [sym_send_string_keyword] = ACTIONS(1743),
    [sym_send_command_keyword] = ACTIONS(1743),
    [sym_clear_buffer_keyword] = ACTIONS(1743),
    [sym_create_buffer_keyword] = ACTIONS(1743),
    [sym_create_multi_buffer_keyword] = ACTIONS(1743),
    [sym_call_keyword] = ACTIONS(1743),
    [sym_system_call_keyword] = ACTIONS(1743),
    [sym_devchan_to_keyword] = ACTIONS(1743),
    [sym_devchan_min_to_keyword] = ACTIONS(1743),
    [sym_devchan_total_off_keyword] = ACTIONS(1743),
    [sym_devchan_pulse_keyword] = ACTIONS(1743),
    [sym_wait_keyword] = ACTIONS(1743),
    [sym_cancel_wait_keyword] = ACTIONS(1743),
    [sym_wait_until_keyword] = ACTIONS(1743),
    [sym_cancel_wait_until_keyword] = ACTIONS(1743),
    [sym_cancel_all_wait_keyword] = ACTIONS(1743),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1743),
    [sym_not] = ACTIONS(1743),
    [sym_bnot] = ACTIONS(1743),
    [sym___file__] = ACTIONS(1743),
    [sym___date__] = ACTIONS(1743),
    [sym___ldate__] = ACTIONS(1743),
    [sym___time__] = ACTIONS(1743),
    [sym___line__] = ACTIONS(1743),
    [sym___name__] = ACTIONS(1743),
    [sym___netlinx__] = ACTIONS(1743),
    [sym_netlinx_axi_version] = ACTIONS(1743),
    [sym_url_flg_tcp] = ACTIONS(1743),
    [sym_url_flg_accinfopresent] = ACTIONS(1743),
    [sym_url_flg_temp] = ACTIONS(1743),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1743),
    [sym_url_flg_stat_mask] = ACTIONS(1743),
    [sym_url_flg_stat_lookup] = ACTIONS(1743),
    [sym_url_flg_stat_connecting] = ACTIONS(1743),
    [sym_url_flg_stat_waiting] = ACTIONS(1743),
    [sym_url_flg_stat_connected] = ACTIONS(1743),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1743),
    [sym_first_virtual_device] = ACTIONS(1743),
    [sym_first_local_port] = ACTIONS(1743),
    [sym_dynamic_virtual_device] = ACTIONS(1743),
    [sym_timeline_once] = ACTIONS(1743),
    [sym_timeline_repeat] = ACTIONS(1743),
    [sym_timeline_absolute] = ACTIONS(1743),
    [sym_timeline_relative] = ACTIONS(1743),
    [sym_do_push_timed_infinite] = ACTIONS(1743),
    [sym_source_type_no_address] = ACTIONS(1743),
    [sym_source_type_neuron_id] = ACTIONS(1743),
    [sym_source_type_ip_address] = ACTIONS(1743),
    [sym_source_type_axlink] = ACTIONS(1743),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1743),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1743),
    [sym_source_type_ip_socket_address] = ACTIONS(1743),
    [sym_source_type_rs232] = ACTIONS(1743),
    [sym_source_type_internal] = ACTIONS(1743),
    [sym_source_type_mac_address] = ACTIONS(1743),
    [sym_source_type_ipv4_port] = ACTIONS(1743),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1743),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1743),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1743),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1743),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1743),
    [sym_true] = ACTIONS(1743),
    [sym_false] = ACTIONS(1743),
    [sym_file_read_only] = ACTIONS(1743),
    [sym_file_rw_new] = ACTIONS(1743),
    [sym_file_rw_append] = ACTIONS(1743),
    [sym_ip_tcp] = ACTIONS(1743),
    [sym_ip_udp] = ACTIONS(1743),
    [sym_ip_udp_2way] = ACTIONS(1743),
    [sym_xml_encode_types] = ACTIONS(1743),
    [sym_xml_encode_char_as_list] = ACTIONS(1743),
    [sym_xml_encode_le] = ACTIONS(1743),
    [sym_xml_decode_types] = ACTIONS(1743),
    [sym_xml_decode_no_preserve] = ACTIONS(1743),
    [sym_duet_dev_type_display_device] = ACTIONS(1743),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1743),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1743),
    [sym_duet_dev_type_io_device] = ACTIONS(1743),
    [sym_duet_dev_type_relay_device] = ACTIONS(1743),
    [sym_duet_dev_type_ups] = ACTIONS(1743),
    [sym_duet_dev_type_amplifier] = ACTIONS(1743),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1743),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1743),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1743),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1743),
    [sym_duet_dev_type_camera] = ACTIONS(1743),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1743),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1743),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1743),
    [sym_duet_dev_type_dss] = ACTIONS(1743),
    [sym_duet_dev_type_dvr] = ACTIONS(1743),
    [sym_duet_dev_type_disc_device] = ACTIONS(1743),
    [sym_duet_dev_type_document_camera] = ACTIONS(1743),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1743),
    [sym_duet_dev_type_hvac] = ACTIONS(1743),
    [sym_duet_dev_type_keypad] = ACTIONS(1743),
    [sym_duet_dev_type_light] = ACTIONS(1743),
    [sym_duet_dev_type_monitor] = ACTIONS(1743),
    [sym_duet_dev_type_motor] = ACTIONS(1743),
    [sym_duet_dev_type_multi_window] = ACTIONS(1743),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1743),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1743),
    [sym_duet_dev_type_receiver] = ACTIONS(1743),
    [sym_duet_dev_type_security_system] = ACTIONS(1743),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1743),
    [sym_duet_dev_type_settop_box] = ACTIONS(1743),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1743),
    [sym_duet_dev_type_switcher] = ACTIONS(1743),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1743),
    [sym_duet_dev_type_tv] = ACTIONS(1743),
    [sym_duet_dev_type_utility] = ACTIONS(1743),
    [sym_duet_dev_type_vcr] = ACTIONS(1743),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1743),
    [sym_duet_dev_type_video_processor] = ACTIONS(1743),
    [sym_duet_dev_type_video_projector] = ACTIONS(1743),
    [sym_duet_dev_type_video_wall] = ACTIONS(1743),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1743),
    [sym_duet_dev_type_weather] = ACTIONS(1743),
    [sym_duet_dev_not_polled] = ACTIONS(1743),
    [sym_duet_dev_polled] = ACTIONS(1743),
    [sym_clkmgr_mode_network] = ACTIONS(1743),
    [sym_clkmgr_mode_standalone] = ACTIONS(1743),
    [sym_normal_standby] = ACTIONS(1743),
    [sym_normal_wake] = ACTIONS(1743),
    [sym_ip_multicast_ttl_option] = ACTIONS(1743),
    [sym_ip_ttl_subnet] = ACTIONS(1743),
    [sym_ip_ttl_site] = ACTIONS(1743),
    [sym_ip_ttl_region] = ACTIONS(1743),
    [sym_ip_ttl_continent] = ACTIONS(1743),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1743),
    [sym_ip_nodelay_on] = ACTIONS(1743),
    [sym_ip_nodelay_off] = ACTIONS(1743),
    [sym_amx_error] = ACTIONS(1743),
    [sym_amx_warning] = ACTIONS(1743),
    [sym_amx_info] = ACTIONS(1743),
    [sym_amx_debug] = ACTIONS(1743),
    [sym_netlinx_login_success] = ACTIONS(1743),
    [sym_netlinx_login_fail] = ACTIONS(1743),
    [sym_netlinx_logout] = ACTIONS(1743),
    [sym_null_str] = ACTIONS(1743),
    [sym_smtp_address] = ACTIONS(1743),
    [sym_smtp_port_number] = ACTIONS(1743),
    [sym_smtp_username] = ACTIONS(1743),
    [sym_smtp_password] = ACTIONS(1743),
    [sym_smtp_require_tls] = ACTIONS(1743),
    [sym_smtp_from] = ACTIONS(1743),
    [sym_smtp_tls_true] = ACTIONS(1743),
    [sym_smtp_tls_false] = ACTIONS(1743),
    [sym_listview_on_row_select_event] = ACTIONS(1743),
    [sym_data_structure_unknown] = ACTIONS(1743),
    [sym_data_structure_datafeed] = ACTIONS(1743),
    [sym_data_structure_datarecordset] = ACTIONS(1743),
    [sym_data_structure_datarecord] = ACTIONS(1743),
    [sym_data_structure_datafield] = ACTIONS(1743),
    [sym_data_max_content_field_count] = ACTIONS(1743),
    [sym_data_max_metadata_field_count] = ACTIONS(1743),
    [sym_data_max_id_length] = ACTIONS(1743),
    [sym_data_max_name_length] = ACTIONS(1743),
    [sym_data_max_description_length] = ACTIONS(1743),
    [sym_data_max_type_length] = ACTIONS(1743),
    [sym_data_max_format_length] = ACTIONS(1743),
    [sym_data_max_source_length] = ACTIONS(1743),
    [sym_data_max_value_length] = ACTIONS(1743),
    [sym_data_max_label_length] = ACTIONS(1743),
    [sym_data_max_url_length] = ACTIONS(1743),
    [sym_max_last_login_info_length] = ACTIONS(1743),
    [sym_data_type_unknown] = ACTIONS(1743),
    [sym_data_type_string] = ACTIONS(1743),
    [sym_data_type_datetime] = ACTIONS(1743),
    [sym_data_type_date] = ACTIONS(1743),
    [sym_data_type_time] = ACTIONS(1743),
    [sym_data_type_image] = ACTIONS(1743),
    [sym_data_format_url] = ACTIONS(1743),
    [sym_data_format_phone] = ACTIONS(1743),
    [sym_data_format_email] = ACTIONS(1743),
    [sym_data_format_iso8601] = ACTIONS(1743),
    [sym_auditlog_priv] = ACTIONS(1743),
    [sym_configuration_priv] = ACTIONS(1743),
    [sym_deviceconfig_priv] = ACTIONS(1743),
    [sym_ftp_priv] = ACTIONS(1743),
    [sym_http_priv] = ACTIONS(1743),
    [sym_networkconfig_priv] = ACTIONS(1743),
    [sym_programport_priv] = ACTIONS(1743),
    [sym_remoteui_priv] = ACTIONS(1743),
    [sym_securitycontrol_priv] = ACTIONS(1743),
    [sym_softwaremanagement_priv] = ACTIONS(1743),
    [sym_terminal_priv] = ACTIONS(1743),
    [sym_tpadmin_priv] = ACTIONS(1743),
    [sym_user1_priv] = ACTIONS(1743),
    [sym_user2_priv] = ACTIONS(1743),
    [sym_user3_priv] = ACTIONS(1743),
    [sym_user4_priv] = ACTIONS(1743),
    [sym_usermanagement_priv] = ACTIONS(1743),
    [sym_valid_account] = ACTIONS(1743),
    [sym_err_name_invalid] = ACTIONS(1743),
    [sym_err_password_invalid] = ACTIONS(1743),
    [sym_err_invalid_account] = ACTIONS(1743),
    [sym_err_no_authorization] = ACTIONS(1743),
    [sym_err_invalid_parameter] = ACTIONS(1743),
    [sym_err_user_locked_out] = ACTIONS(1743),
    [sym_err_user_account_expired] = ACTIONS(1743),
    [sym_err_general_error] = ACTIONS(1743),
    [sym_err_authentication_error] = ACTIONS(1743),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1743),
    [sym_tls_validate_certificate] = ACTIONS(1743),
    [sym_tdata] = ACTIONS(1743),
    [sym_tchannel] = ACTIONS(1743),
    [sym_tlevel] = ACTIONS(1743),
    [sym_tbutton] = ACTIONS(1743),
    [sym_ttimeline] = ACTIONS(1743),
    [sym_tcustom] = ACTIONS(1743),
    [sym_url_struct] = ACTIONS(1743),
    [sym_dns_struct] = ACTIONS(1743),
    [sym_ip_address_struct] = ACTIONS(1743),
    [sym_dev_info_struct] = ACTIONS(1743),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1743),
    [sym_data_feed] = ACTIONS(1743),
    [sym_data_field] = ACTIONS(1743),
    [sym_data_record] = ACTIONS(1743),
    [sym_wc_data_feed] = ACTIONS(1743),
    [sym_wc_data_field] = ACTIONS(1743),
    [sym_wc_data_record] = ACTIONS(1743),
    [sym_last_login_info] = ACTIONS(1743),
    [sym____reserved___] = ACTIONS(1743),
    [sym_date] = ACTIONS(1743),
    [sym_day] = ACTIONS(1743),
    [sym_time] = ACTIONS(1743),
    [sym_ldate] = ACTIONS(1743),
    [sym_push_device] = ACTIONS(1743),
    [sym_push_channel] = ACTIONS(1743),
    [sym_push_devchan] = ACTIONS(1743),
    [sym_release_device] = ACTIONS(1743),
    [sym_release_channel] = ACTIONS(1743),
    [sym_release_devchan] = ACTIONS(1743),
    [sym_master_slot] = ACTIONS(1743),
    [sym_get_pulse_time] = ACTIONS(1743),
    [sym_get_timer] = ACTIONS(1743),
    [sym_master_sn] = ACTIONS(1743),
    [sym_system_number] = ACTIONS(1743),
    [sym_button] = ACTIONS(1743),
    [sym_dv_channel] = ACTIONS(1743),
    [sym_data] = ACTIONS(1743),
    [sym_level] = ACTIONS(1743),
    [sym_timeline] = ACTIONS(1743),
    [sym_channel] = ACTIONS(1743),
    [sym_custom] = ACTIONS(1743),
    [sym_atoi] = ACTIONS(1743),
    [sym_atol] = ACTIONS(1743),
    [sym_atof] = ACTIONS(1743),
    [sym_itoa] = ACTIONS(1743),
    [sym_itohex] = ACTIONS(1743),
    [sym_hextoi] = ACTIONS(1743),
    [sym_ftoa] = ACTIONS(1743),
    [sym_find_string] = ACTIONS(1743),
    [sym_left_string] = ACTIONS(1743),
    [sym_right_string] = ACTIONS(1743),
    [sym_mid_string] = ACTIONS(1743),
    [sym_remove_string] = ACTIONS(1743),
    [sym_get_buffer_string] = ACTIONS(1743),
    [sym_get_buffer_char] = ACTIONS(1743),
    [sym_get_multi_buffer_string] = ACTIONS(1743),
    [sym_length_string] = ACTIONS(1743),
    [sym_max_length_string] = ACTIONS(1743),
    [sym_set_length_string] = ACTIONS(1743),
    [sym_compare_string] = ACTIONS(1743),
    [sym_upper_string] = ACTIONS(1743),
    [sym_lower_string] = ACTIONS(1743),
    [sym_get_last] = ACTIONS(1743),
    [sym_date_to_day] = ACTIONS(1743),
    [sym_date_to_month] = ACTIONS(1743),
    [sym_date_to_year] = ACTIONS(1743),
    [sym_time_to_hour] = ACTIONS(1743),
    [sym_time_to_minute] = ACTIONS(1743),
    [sym_time_to_second] = ACTIONS(1743),
    [sym_day_of_week] = ACTIONS(1743),
    [sym_set_pulse_time] = ACTIONS(1743),
    [sym_set_timer] = ACTIONS(1743),
    [sym_file_open] = ACTIONS(1743),
    [sym_file_close] = ACTIONS(1743),
    [sym_file_read] = ACTIONS(1743),
    [sym_file_read_line] = ACTIONS(1743),
    [sym_file_write] = ACTIONS(1743),
    [sym_file_write_line] = ACTIONS(1743),
    [sym_file_copy] = ACTIONS(1743),
    [sym_file_delete] = ACTIONS(1743),
    [sym_file_rename] = ACTIONS(1743),
    [sym_file_dir] = ACTIONS(1743),
    [sym_file_getdir] = ACTIONS(1743),
    [sym_file_setdir] = ACTIONS(1743),
    [sym_file_create_dir] = ACTIONS(1743),
    [sym_file_remove_dir] = ACTIONS(1743),
    [sym_file_seek] = ACTIONS(1743),
    [sym_length_array] = ACTIONS(1743),
    [sym_max_length_array] = ACTIONS(1743),
    [sym_set_length_array] = ACTIONS(1743),
    [sym_ip_client_open] = ACTIONS(1743),
    [sym_ip_client_close] = ACTIONS(1743),
    [sym_ip_server_open] = ACTIONS(1743),
    [sym_ip_server_close] = ACTIONS(1743),
    [sym_ip_bound_client_open] = ACTIONS(1743),
    [sym_ip_mc_server_open] = ACTIONS(1743),
    [sym_ip_set_option] = ACTIONS(1743),
    [sym_ssh_client_open] = ACTIONS(1743),
    [sym_ssh_client_close] = ACTIONS(1743),
    [sym_tls_client_open] = ACTIONS(1743),
    [sym_tls_client_close] = ACTIONS(1743),
    [sym_do_push] = ACTIONS(1743),
    [sym_do_push_timed] = ACTIONS(1743),
    [sym_do_release] = ACTIONS(1743),
    [sym_device_id] = ACTIONS(1743),
    [sym_device_info] = ACTIONS(1743),
    [sym_device_id_string] = ACTIONS(1743),
    [sym_device_standby] = ACTIONS(1743),
    [sym_device_wake] = ACTIONS(1743),
    [sym_timeline_create] = ACTIONS(1743),
    [sym_timeline_kill] = ACTIONS(1743),
    [sym_timeline_reload] = ACTIONS(1743),
    [sym_timeline_pause] = ACTIONS(1743),
    [sym_timeline_restart] = ACTIONS(1743),
    [sym_timeline_set] = ACTIONS(1743),
    [sym_timeline_get] = ACTIONS(1743),
    [sym_timeline_active] = ACTIONS(1743),
    [sym_timeline_dynamic_id] = ACTIONS(1743),
    [sym_variable_to_string] = ACTIONS(1743),
    [sym_string_to_variable] = ACTIONS(1743),
    [sym_length_variable_to_string] = ACTIONS(1743),
    [sym_type_cast] = ACTIONS(1743),
    [sym_variable_to_xml] = ACTIONS(1743),
    [sym_xml_to_variable] = ACTIONS(1743),
    [sym_length_variable_to_xml] = ACTIONS(1743),
    [sym_redirect_string] = ACTIONS(1743),
    [sym_random_number] = ACTIONS(1743),
    [sym_get_unique_id] = ACTIONS(1743),
    [sym_get_system_number] = ACTIONS(1743),
    [sym_set_system_number] = ACTIONS(1743),
    [sym_get_serial_number] = ACTIONS(1743),
    [sym_get_master_build] = ACTIONS(1743),
    [sym_set_validation_code] = ACTIONS(1743),
    [sym_reboot] = ACTIONS(1743),
    [sym_load_duet_module] = ACTIONS(1743),
    [sym_unload_duet_module] = ACTIONS(1743),
    [sym_duet_mem_size_set] = ACTIONS(1743),
    [sym_duet_mem_size_get] = ACTIONS(1743),
    [sym_dynamic_application_device] = ACTIONS(1743),
    [sym_dynamic_polled_port] = ACTIONS(1743),
    [sym_static_port_binding] = ACTIONS(1743),
    [sym_static_ip_binding] = ACTIONS(1743),
    [sym_get_dns_list] = ACTIONS(1743),
    [sym_set_dns_list] = ACTIONS(1743),
    [sym_get_ip_address] = ACTIONS(1743),
    [sym_set_ip_address] = ACTIONS(1743),
    [sym_get_url_list] = ACTIONS(1743),
    [sym_add_url_entry] = ACTIONS(1743),
    [sym_delete_url_entry] = ACTIONS(1743),
    [sym_abs_value] = ACTIONS(1743),
    [sym_max_value] = ACTIONS(1743),
    [sym_min_value] = ACTIONS(1743),
    [sym_exp_value] = ACTIONS(1743),
    [sym_log_value] = ACTIONS(1743),
    [sym_log10_value] = ACTIONS(1743),
    [sym_power_value] = ACTIONS(1743),
    [sym_sqrt_value] = ACTIONS(1743),
    [sym_raw_be] = ACTIONS(1743),
    [sym_raw_le] = ACTIONS(1743),
    [sym_internal_queue_size_set] = ACTIONS(1743),
    [sym_internal_queue_size_get] = ACTIONS(1743),
    [sym_internal_threshold_set] = ACTIONS(1743),
    [sym_internal_threshold_get] = ACTIONS(1743),
    [sym_rebuilt_event] = ACTIONS(1743),
    [sym_do_custom_event] = ACTIONS(1743),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1743),
    [sym_clkmgr_set_clk_source] = ACTIONS(1743),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1743),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1743),
    [sym_clkmgr_get_timezone] = ACTIONS(1743),
    [sym_clkmgr_set_timezone] = ACTIONS(1743),
    [sym_clkmgr_get_resync_period] = ACTIONS(1743),
    [sym_clkmgr_set_resync_period] = ACTIONS(1743),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1743),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1743),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1743),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1743),
    [sym_clkmgr_get_timeservers] = ACTIONS(1743),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1743),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1743),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1743),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1743),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1743),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1743),
    [sym_set_log_level] = ACTIONS(1743),
    [sym_get_log_level] = ACTIONS(1743),
    [sym_amx_log] = ACTIONS(1743),
    [sym_smtp_server_config_set] = ACTIONS(1743),
    [sym_smtp_server_config_get] = ACTIONS(1743),
    [sym_smtp_send] = ACTIONS(1743),
    [sym_validate_netlinx_account] = ACTIONS(1743),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1743),
    [sym_authenticate_certificate] = ACTIONS(1743),
    [sym_audit_netlinx_session_event] = ACTIONS(1743),
    [sym_audit_netlinx_generic_event] = ACTIONS(1743),
    [sym_get_available_flash_disk_space] = ACTIONS(1743),
    [sym_get_max_flash_disk_space] = ACTIONS(1743),
    [sym_data_create_feed] = ACTIONS(1743),
    [sym_data_delete_feed] = ACTIONS(1743),
    [sym_data_publish_feed] = ACTIONS(1743),
    [sym_data_get_published_feed] = ACTIONS(1743),
    [sym_data_add_record] = ACTIONS(1743),
    [sym_data_get_event_record] = ACTIONS(1743),
    [sym__wc_data_create_feed] = ACTIONS(1743),
    [sym__wc_data_add_record] = ACTIONS(1743),
    [sym__wc_data_get_event_record] = ACTIONS(1743),
    [sym_format] = ACTIONS(1743),
    [sym_astro_clock] = ACTIONS(1743),
    [sym_set_outdoor_temperature] = ACTIONS(1743),
    [sym_set_virtual_level_count] = ACTIONS(1743),
    [sym_set_virtual_channel_count] = ACTIONS(1743),
    [sym_set_virtual_port_count] = ACTIONS(1743),
    [sym__wc_encode_int] = ACTIONS(1743),
    [sym__wc_decode_int] = ACTIONS(1743),
    [sym__wc_int] = ACTIONS(1743),
    [sym__wc_to_ch_int] = ACTIONS(1743),
    [sym__ch_to_wc_int] = ACTIONS(1743),
    [sym__wc_find_string_int] = ACTIONS(1743),
    [sym__wc_left_string_int] = ACTIONS(1743),
    [sym__wc_lower_string_int] = ACTIONS(1743),
    [sym__wc_mid_string_int] = ACTIONS(1743),
    [sym__wc_remove_string_int] = ACTIONS(1743),
    [sym__wc_right_string_int] = ACTIONS(1743),
    [sym__wc_upper_string_int] = ACTIONS(1743),
    [sym__wc_compare_string_int] = ACTIONS(1743),
    [sym__wc_get_buffer_char_int] = ACTIONS(1743),
    [sym__wc_get_buffer_string_int] = ACTIONS(1743),
    [sym__wc_concat_string_int] = ACTIONS(1743),
    [sym__wc_file_open_int] = ACTIONS(1743),
    [sym__wc_file_close_int] = ACTIONS(1743),
    [sym__wc_file_read_int] = ACTIONS(1743),
    [sym__wc_file_read_line_int] = ACTIONS(1743),
    [sym__wc_file_write_int] = ACTIONS(1743),
    [sym__wc_file_write_line_int] = ACTIONS(1743),
    [aux_sym_preproc_include_token1] = ACTIONS(1743),
    [aux_sym_preproc_define_token1] = ACTIONS(1745),
    [aux_sym_preproc_warn_token1] = ACTIONS(1745),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1745),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1745),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1745),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1745),
    [aux_sym_preproc_else_token1] = ACTIONS(1745),
    [anon_sym_LBRACK] = ACTIONS(1745),
    [anon_sym_LBRACE] = ACTIONS(1745),
    [anon_sym_LPAREN] = ACTIONS(1743),
    [anon_sym_BANG] = ACTIONS(1745),
    [anon_sym_TILDE] = ACTIONS(1745),
    [anon_sym_DASH] = ACTIONS(1743),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_DASH_DASH] = ACTIONS(1745),
    [anon_sym_PLUS_PLUS] = ACTIONS(1745),
    [anon_sym_DQUOTE] = ACTIONS(1745),
    [anon_sym_SQUOTE] = ACTIONS(1745),
    [sym_number_literal] = ACTIONS(1745),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(216)] = {
    [sym_identifier] = ACTIONS(1747),
    [sym_program_name_keyword] = ACTIONS(1747),
    [sym_module_name_keyword] = ACTIONS(1747),
    [sym_define_device_keyword] = ACTIONS(1747),
    [sym_define_combine_keyword] = ACTIONS(1747),
    [sym_define_constant_keyword] = ACTIONS(1747),
    [sym_define_type_keyword] = ACTIONS(1747),
    [sym_define_variable_keyword] = ACTIONS(1747),
    [sym_define_system_variable_keyword] = ACTIONS(1747),
    [sym_define_connect_level_keyword] = ACTIONS(1747),
    [sym_define_latching_keyword] = ACTIONS(1747),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1747),
    [sym_define_toggling_keyword] = ACTIONS(1747),
    [sym_define_call_keyword] = ACTIONS(1747),
    [sym_define_function_keyword] = ACTIONS(1747),
    [sym_define_library_function_keyword] = ACTIONS(1747),
    [sym_define_start_keyword] = ACTIONS(1747),
    [sym_define_event_keyword] = ACTIONS(1747),
    [sym_define_program_keyword] = ACTIONS(1747),
    [sym_define_module_keyword] = ACTIONS(1747),
    [sym_char_keyword] = ACTIONS(1747),
    [sym_widechar_keyword] = ACTIONS(1747),
    [sym_integer_keyword] = ACTIONS(1747),
    [sym_sinteger_keyword] = ACTIONS(1747),
    [sym_long_keyword] = ACTIONS(1747),
    [sym_slong_keyword] = ACTIONS(1747),
    [sym_float_keyword] = ACTIONS(1747),
    [sym_double_keyword] = ACTIONS(1747),
    [sym_variant_keyword] = ACTIONS(1747),
    [sym_variantarray_keyword] = ACTIONS(1747),
    [sym_local_var_keyword] = ACTIONS(1747),
    [sym_stack_var_keyword] = ACTIONS(1747),
    [sym_constant_keyword] = ACTIONS(1747),
    [sym_volatile_keyword] = ACTIONS(1747),
    [sym_non_volatile_keyword] = ACTIONS(1747),
    [sym_persistent_keyword] = ACTIONS(1747),
    [sym_dev_keyword] = ACTIONS(1747),
    [sym_devlev_keyword] = ACTIONS(1747),
    [sym_devchan_keyword] = ACTIONS(1747),
    [sym_if_keyword] = ACTIONS(1747),
    [sym_else_keyword] = ACTIONS(1747),
    [sym_switch_keyword] = ACTIONS(1747),
    [sym_case_keyword] = ACTIONS(1747),
    [sym_default_keyword] = ACTIONS(1747),
    [sym_while_keyword] = ACTIONS(1747),
    [sym_for_keyword] = ACTIONS(1747),
    [sym_break_keyword] = ACTIONS(1747),
    [sym_continue_keyword] = ACTIONS(1747),
    [sym_return_keyword] = ACTIONS(1747),
    [sym_select_keyword] = ACTIONS(1747),
    [sym_active_keyword] = ACTIONS(1747),
    [sym_struct_keyword] = ACTIONS(1747),
    [sym_structure_keyword] = ACTIONS(1747),
    [sym_button_event_keyword] = ACTIONS(1747),
    [sym_channel_event_keyword] = ACTIONS(1747),
    [sym_level_event_keyword] = ACTIONS(1747),
    [sym_data_event_keyword] = ACTIONS(1747),
    [sym_timeline_event_keyword] = ACTIONS(1747),
    [sym_custom_event_keyword] = ACTIONS(1747),
    [sym_push_keyword] = ACTIONS(1747),
    [sym_release_keyword] = ACTIONS(1747),
    [aux_sym_on_keyword_token1] = ACTIONS(1747),
    [aux_sym_off_keyword_token1] = ACTIONS(1747),
    [sym_send_level_keyword] = ACTIONS(1747),
    [sym_send_string_keyword] = ACTIONS(1747),
    [sym_send_command_keyword] = ACTIONS(1747),
    [sym_clear_buffer_keyword] = ACTIONS(1747),
    [sym_create_buffer_keyword] = ACTIONS(1747),
    [sym_create_multi_buffer_keyword] = ACTIONS(1747),
    [sym_call_keyword] = ACTIONS(1747),
    [sym_system_call_keyword] = ACTIONS(1747),
    [sym_devchan_to_keyword] = ACTIONS(1747),
    [sym_devchan_min_to_keyword] = ACTIONS(1747),
    [sym_devchan_total_off_keyword] = ACTIONS(1747),
    [sym_devchan_pulse_keyword] = ACTIONS(1747),
    [sym_wait_keyword] = ACTIONS(1747),
    [sym_cancel_wait_keyword] = ACTIONS(1747),
    [sym_wait_until_keyword] = ACTIONS(1747),
    [sym_cancel_wait_until_keyword] = ACTIONS(1747),
    [sym_cancel_all_wait_keyword] = ACTIONS(1747),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1747),
    [sym_not] = ACTIONS(1747),
    [sym_bnot] = ACTIONS(1747),
    [sym___file__] = ACTIONS(1747),
    [sym___date__] = ACTIONS(1747),
    [sym___ldate__] = ACTIONS(1747),
    [sym___time__] = ACTIONS(1747),
    [sym___line__] = ACTIONS(1747),
    [sym___name__] = ACTIONS(1747),
    [sym___netlinx__] = ACTIONS(1747),
    [sym_netlinx_axi_version] = ACTIONS(1747),
    [sym_url_flg_tcp] = ACTIONS(1747),
    [sym_url_flg_accinfopresent] = ACTIONS(1747),
    [sym_url_flg_temp] = ACTIONS(1747),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1747),
    [sym_url_flg_stat_mask] = ACTIONS(1747),
    [sym_url_flg_stat_lookup] = ACTIONS(1747),
    [sym_url_flg_stat_connecting] = ACTIONS(1747),
    [sym_url_flg_stat_waiting] = ACTIONS(1747),
    [sym_url_flg_stat_connected] = ACTIONS(1747),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1747),
    [sym_first_virtual_device] = ACTIONS(1747),
    [sym_first_local_port] = ACTIONS(1747),
    [sym_dynamic_virtual_device] = ACTIONS(1747),
    [sym_timeline_once] = ACTIONS(1747),
    [sym_timeline_repeat] = ACTIONS(1747),
    [sym_timeline_absolute] = ACTIONS(1747),
    [sym_timeline_relative] = ACTIONS(1747),
    [sym_do_push_timed_infinite] = ACTIONS(1747),
    [sym_source_type_no_address] = ACTIONS(1747),
    [sym_source_type_neuron_id] = ACTIONS(1747),
    [sym_source_type_ip_address] = ACTIONS(1747),
    [sym_source_type_axlink] = ACTIONS(1747),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1747),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1747),
    [sym_source_type_ip_socket_address] = ACTIONS(1747),
    [sym_source_type_rs232] = ACTIONS(1747),
    [sym_source_type_internal] = ACTIONS(1747),
    [sym_source_type_mac_address] = ACTIONS(1747),
    [sym_source_type_ipv4_port] = ACTIONS(1747),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1747),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1747),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1747),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1747),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1747),
    [sym_true] = ACTIONS(1747),
    [sym_false] = ACTIONS(1747),
    [sym_file_read_only] = ACTIONS(1747),
    [sym_file_rw_new] = ACTIONS(1747),
    [sym_file_rw_append] = ACTIONS(1747),
    [sym_ip_tcp] = ACTIONS(1747),
    [sym_ip_udp] = ACTIONS(1747),
    [sym_ip_udp_2way] = ACTIONS(1747),
    [sym_xml_encode_types] = ACTIONS(1747),
    [sym_xml_encode_char_as_list] = ACTIONS(1747),
    [sym_xml_encode_le] = ACTIONS(1747),
    [sym_xml_decode_types] = ACTIONS(1747),
    [sym_xml_decode_no_preserve] = ACTIONS(1747),
    [sym_duet_dev_type_display_device] = ACTIONS(1747),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1747),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1747),
    [sym_duet_dev_type_io_device] = ACTIONS(1747),
    [sym_duet_dev_type_relay_device] = ACTIONS(1747),
    [sym_duet_dev_type_ups] = ACTIONS(1747),
    [sym_duet_dev_type_amplifier] = ACTIONS(1747),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1747),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1747),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1747),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1747),
    [sym_duet_dev_type_camera] = ACTIONS(1747),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1747),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1747),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1747),
    [sym_duet_dev_type_dss] = ACTIONS(1747),
    [sym_duet_dev_type_dvr] = ACTIONS(1747),
    [sym_duet_dev_type_disc_device] = ACTIONS(1747),
    [sym_duet_dev_type_document_camera] = ACTIONS(1747),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1747),
    [sym_duet_dev_type_hvac] = ACTIONS(1747),
    [sym_duet_dev_type_keypad] = ACTIONS(1747),
    [sym_duet_dev_type_light] = ACTIONS(1747),
    [sym_duet_dev_type_monitor] = ACTIONS(1747),
    [sym_duet_dev_type_motor] = ACTIONS(1747),
    [sym_duet_dev_type_multi_window] = ACTIONS(1747),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1747),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1747),
    [sym_duet_dev_type_receiver] = ACTIONS(1747),
    [sym_duet_dev_type_security_system] = ACTIONS(1747),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1747),
    [sym_duet_dev_type_settop_box] = ACTIONS(1747),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1747),
    [sym_duet_dev_type_switcher] = ACTIONS(1747),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1747),
    [sym_duet_dev_type_tv] = ACTIONS(1747),
    [sym_duet_dev_type_utility] = ACTIONS(1747),
    [sym_duet_dev_type_vcr] = ACTIONS(1747),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1747),
    [sym_duet_dev_type_video_processor] = ACTIONS(1747),
    [sym_duet_dev_type_video_projector] = ACTIONS(1747),
    [sym_duet_dev_type_video_wall] = ACTIONS(1747),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1747),
    [sym_duet_dev_type_weather] = ACTIONS(1747),
    [sym_duet_dev_not_polled] = ACTIONS(1747),
    [sym_duet_dev_polled] = ACTIONS(1747),
    [sym_clkmgr_mode_network] = ACTIONS(1747),
    [sym_clkmgr_mode_standalone] = ACTIONS(1747),
    [sym_normal_standby] = ACTIONS(1747),
    [sym_normal_wake] = ACTIONS(1747),
    [sym_ip_multicast_ttl_option] = ACTIONS(1747),
    [sym_ip_ttl_subnet] = ACTIONS(1747),
    [sym_ip_ttl_site] = ACTIONS(1747),
    [sym_ip_ttl_region] = ACTIONS(1747),
    [sym_ip_ttl_continent] = ACTIONS(1747),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1747),
    [sym_ip_nodelay_on] = ACTIONS(1747),
    [sym_ip_nodelay_off] = ACTIONS(1747),
    [sym_amx_error] = ACTIONS(1747),
    [sym_amx_warning] = ACTIONS(1747),
    [sym_amx_info] = ACTIONS(1747),
    [sym_amx_debug] = ACTIONS(1747),
    [sym_netlinx_login_success] = ACTIONS(1747),
    [sym_netlinx_login_fail] = ACTIONS(1747),
    [sym_netlinx_logout] = ACTIONS(1747),
    [sym_null_str] = ACTIONS(1747),
    [sym_smtp_address] = ACTIONS(1747),
    [sym_smtp_port_number] = ACTIONS(1747),
    [sym_smtp_username] = ACTIONS(1747),
    [sym_smtp_password] = ACTIONS(1747),
    [sym_smtp_require_tls] = ACTIONS(1747),
    [sym_smtp_from] = ACTIONS(1747),
    [sym_smtp_tls_true] = ACTIONS(1747),
    [sym_smtp_tls_false] = ACTIONS(1747),
    [sym_listview_on_row_select_event] = ACTIONS(1747),
    [sym_data_structure_unknown] = ACTIONS(1747),
    [sym_data_structure_datafeed] = ACTIONS(1747),
    [sym_data_structure_datarecordset] = ACTIONS(1747),
    [sym_data_structure_datarecord] = ACTIONS(1747),
    [sym_data_structure_datafield] = ACTIONS(1747),
    [sym_data_max_content_field_count] = ACTIONS(1747),
    [sym_data_max_metadata_field_count] = ACTIONS(1747),
    [sym_data_max_id_length] = ACTIONS(1747),
    [sym_data_max_name_length] = ACTIONS(1747),
    [sym_data_max_description_length] = ACTIONS(1747),
    [sym_data_max_type_length] = ACTIONS(1747),
    [sym_data_max_format_length] = ACTIONS(1747),
    [sym_data_max_source_length] = ACTIONS(1747),
    [sym_data_max_value_length] = ACTIONS(1747),
    [sym_data_max_label_length] = ACTIONS(1747),
    [sym_data_max_url_length] = ACTIONS(1747),
    [sym_max_last_login_info_length] = ACTIONS(1747),
    [sym_data_type_unknown] = ACTIONS(1747),
    [sym_data_type_string] = ACTIONS(1747),
    [sym_data_type_datetime] = ACTIONS(1747),
    [sym_data_type_date] = ACTIONS(1747),
    [sym_data_type_time] = ACTIONS(1747),
    [sym_data_type_image] = ACTIONS(1747),
    [sym_data_format_url] = ACTIONS(1747),
    [sym_data_format_phone] = ACTIONS(1747),
    [sym_data_format_email] = ACTIONS(1747),
    [sym_data_format_iso8601] = ACTIONS(1747),
    [sym_auditlog_priv] = ACTIONS(1747),
    [sym_configuration_priv] = ACTIONS(1747),
    [sym_deviceconfig_priv] = ACTIONS(1747),
    [sym_ftp_priv] = ACTIONS(1747),
    [sym_http_priv] = ACTIONS(1747),
    [sym_networkconfig_priv] = ACTIONS(1747),
    [sym_programport_priv] = ACTIONS(1747),
    [sym_remoteui_priv] = ACTIONS(1747),
    [sym_securitycontrol_priv] = ACTIONS(1747),
    [sym_softwaremanagement_priv] = ACTIONS(1747),
    [sym_terminal_priv] = ACTIONS(1747),
    [sym_tpadmin_priv] = ACTIONS(1747),
    [sym_user1_priv] = ACTIONS(1747),
    [sym_user2_priv] = ACTIONS(1747),
    [sym_user3_priv] = ACTIONS(1747),
    [sym_user4_priv] = ACTIONS(1747),
    [sym_usermanagement_priv] = ACTIONS(1747),
    [sym_valid_account] = ACTIONS(1747),
    [sym_err_name_invalid] = ACTIONS(1747),
    [sym_err_password_invalid] = ACTIONS(1747),
    [sym_err_invalid_account] = ACTIONS(1747),
    [sym_err_no_authorization] = ACTIONS(1747),
    [sym_err_invalid_parameter] = ACTIONS(1747),
    [sym_err_user_locked_out] = ACTIONS(1747),
    [sym_err_user_account_expired] = ACTIONS(1747),
    [sym_err_general_error] = ACTIONS(1747),
    [sym_err_authentication_error] = ACTIONS(1747),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1747),
    [sym_tls_validate_certificate] = ACTIONS(1747),
    [sym_tdata] = ACTIONS(1747),
    [sym_tchannel] = ACTIONS(1747),
    [sym_tlevel] = ACTIONS(1747),
    [sym_tbutton] = ACTIONS(1747),
    [sym_ttimeline] = ACTIONS(1747),
    [sym_tcustom] = ACTIONS(1747),
    [sym_url_struct] = ACTIONS(1747),
    [sym_dns_struct] = ACTIONS(1747),
    [sym_ip_address_struct] = ACTIONS(1747),
    [sym_dev_info_struct] = ACTIONS(1747),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1747),
    [sym_data_feed] = ACTIONS(1747),
    [sym_data_field] = ACTIONS(1747),
    [sym_data_record] = ACTIONS(1747),
    [sym_wc_data_feed] = ACTIONS(1747),
    [sym_wc_data_field] = ACTIONS(1747),
    [sym_wc_data_record] = ACTIONS(1747),
    [sym_last_login_info] = ACTIONS(1747),
    [sym____reserved___] = ACTIONS(1747),
    [sym_date] = ACTIONS(1747),
    [sym_day] = ACTIONS(1747),
    [sym_time] = ACTIONS(1747),
    [sym_ldate] = ACTIONS(1747),
    [sym_push_device] = ACTIONS(1747),
    [sym_push_channel] = ACTIONS(1747),
    [sym_push_devchan] = ACTIONS(1747),
    [sym_release_device] = ACTIONS(1747),
    [sym_release_channel] = ACTIONS(1747),
    [sym_release_devchan] = ACTIONS(1747),
    [sym_master_slot] = ACTIONS(1747),
    [sym_get_pulse_time] = ACTIONS(1747),
    [sym_get_timer] = ACTIONS(1747),
    [sym_master_sn] = ACTIONS(1747),
    [sym_system_number] = ACTIONS(1747),
    [sym_button] = ACTIONS(1747),
    [sym_dv_channel] = ACTIONS(1747),
    [sym_data] = ACTIONS(1747),
    [sym_level] = ACTIONS(1747),
    [sym_timeline] = ACTIONS(1747),
    [sym_channel] = ACTIONS(1747),
    [sym_custom] = ACTIONS(1747),
    [sym_atoi] = ACTIONS(1747),
    [sym_atol] = ACTIONS(1747),
    [sym_atof] = ACTIONS(1747),
    [sym_itoa] = ACTIONS(1747),
    [sym_itohex] = ACTIONS(1747),
    [sym_hextoi] = ACTIONS(1747),
    [sym_ftoa] = ACTIONS(1747),
    [sym_find_string] = ACTIONS(1747),
    [sym_left_string] = ACTIONS(1747),
    [sym_right_string] = ACTIONS(1747),
    [sym_mid_string] = ACTIONS(1747),
    [sym_remove_string] = ACTIONS(1747),
    [sym_get_buffer_string] = ACTIONS(1747),
    [sym_get_buffer_char] = ACTIONS(1747),
    [sym_get_multi_buffer_string] = ACTIONS(1747),
    [sym_length_string] = ACTIONS(1747),
    [sym_max_length_string] = ACTIONS(1747),
    [sym_set_length_string] = ACTIONS(1747),
    [sym_compare_string] = ACTIONS(1747),
    [sym_upper_string] = ACTIONS(1747),
    [sym_lower_string] = ACTIONS(1747),
    [sym_get_last] = ACTIONS(1747),
    [sym_date_to_day] = ACTIONS(1747),
    [sym_date_to_month] = ACTIONS(1747),
    [sym_date_to_year] = ACTIONS(1747),
    [sym_time_to_hour] = ACTIONS(1747),
    [sym_time_to_minute] = ACTIONS(1747),
    [sym_time_to_second] = ACTIONS(1747),
    [sym_day_of_week] = ACTIONS(1747),
    [sym_set_pulse_time] = ACTIONS(1747),
    [sym_set_timer] = ACTIONS(1747),
    [sym_file_open] = ACTIONS(1747),
    [sym_file_close] = ACTIONS(1747),
    [sym_file_read] = ACTIONS(1747),
    [sym_file_read_line] = ACTIONS(1747),
    [sym_file_write] = ACTIONS(1747),
    [sym_file_write_line] = ACTIONS(1747),
    [sym_file_copy] = ACTIONS(1747),
    [sym_file_delete] = ACTIONS(1747),
    [sym_file_rename] = ACTIONS(1747),
    [sym_file_dir] = ACTIONS(1747),
    [sym_file_getdir] = ACTIONS(1747),
    [sym_file_setdir] = ACTIONS(1747),
    [sym_file_create_dir] = ACTIONS(1747),
    [sym_file_remove_dir] = ACTIONS(1747),
    [sym_file_seek] = ACTIONS(1747),
    [sym_length_array] = ACTIONS(1747),
    [sym_max_length_array] = ACTIONS(1747),
    [sym_set_length_array] = ACTIONS(1747),
    [sym_ip_client_open] = ACTIONS(1747),
    [sym_ip_client_close] = ACTIONS(1747),
    [sym_ip_server_open] = ACTIONS(1747),
    [sym_ip_server_close] = ACTIONS(1747),
    [sym_ip_bound_client_open] = ACTIONS(1747),
    [sym_ip_mc_server_open] = ACTIONS(1747),
    [sym_ip_set_option] = ACTIONS(1747),
    [sym_ssh_client_open] = ACTIONS(1747),
    [sym_ssh_client_close] = ACTIONS(1747),
    [sym_tls_client_open] = ACTIONS(1747),
    [sym_tls_client_close] = ACTIONS(1747),
    [sym_do_push] = ACTIONS(1747),
    [sym_do_push_timed] = ACTIONS(1747),
    [sym_do_release] = ACTIONS(1747),
    [sym_device_id] = ACTIONS(1747),
    [sym_device_info] = ACTIONS(1747),
    [sym_device_id_string] = ACTIONS(1747),
    [sym_device_standby] = ACTIONS(1747),
    [sym_device_wake] = ACTIONS(1747),
    [sym_timeline_create] = ACTIONS(1747),
    [sym_timeline_kill] = ACTIONS(1747),
    [sym_timeline_reload] = ACTIONS(1747),
    [sym_timeline_pause] = ACTIONS(1747),
    [sym_timeline_restart] = ACTIONS(1747),
    [sym_timeline_set] = ACTIONS(1747),
    [sym_timeline_get] = ACTIONS(1747),
    [sym_timeline_active] = ACTIONS(1747),
    [sym_timeline_dynamic_id] = ACTIONS(1747),
    [sym_variable_to_string] = ACTIONS(1747),
    [sym_string_to_variable] = ACTIONS(1747),
    [sym_length_variable_to_string] = ACTIONS(1747),
    [sym_type_cast] = ACTIONS(1747),
    [sym_variable_to_xml] = ACTIONS(1747),
    [sym_xml_to_variable] = ACTIONS(1747),
    [sym_length_variable_to_xml] = ACTIONS(1747),
    [sym_redirect_string] = ACTIONS(1747),
    [sym_random_number] = ACTIONS(1747),
    [sym_get_unique_id] = ACTIONS(1747),
    [sym_get_system_number] = ACTIONS(1747),
    [sym_set_system_number] = ACTIONS(1747),
    [sym_get_serial_number] = ACTIONS(1747),
    [sym_get_master_build] = ACTIONS(1747),
    [sym_set_validation_code] = ACTIONS(1747),
    [sym_reboot] = ACTIONS(1747),
    [sym_load_duet_module] = ACTIONS(1747),
    [sym_unload_duet_module] = ACTIONS(1747),
    [sym_duet_mem_size_set] = ACTIONS(1747),
    [sym_duet_mem_size_get] = ACTIONS(1747),
    [sym_dynamic_application_device] = ACTIONS(1747),
    [sym_dynamic_polled_port] = ACTIONS(1747),
    [sym_static_port_binding] = ACTIONS(1747),
    [sym_static_ip_binding] = ACTIONS(1747),
    [sym_get_dns_list] = ACTIONS(1747),
    [sym_set_dns_list] = ACTIONS(1747),
    [sym_get_ip_address] = ACTIONS(1747),
    [sym_set_ip_address] = ACTIONS(1747),
    [sym_get_url_list] = ACTIONS(1747),
    [sym_add_url_entry] = ACTIONS(1747),
    [sym_delete_url_entry] = ACTIONS(1747),
    [sym_abs_value] = ACTIONS(1747),
    [sym_max_value] = ACTIONS(1747),
    [sym_min_value] = ACTIONS(1747),
    [sym_exp_value] = ACTIONS(1747),
    [sym_log_value] = ACTIONS(1747),
    [sym_log10_value] = ACTIONS(1747),
    [sym_power_value] = ACTIONS(1747),
    [sym_sqrt_value] = ACTIONS(1747),
    [sym_raw_be] = ACTIONS(1747),
    [sym_raw_le] = ACTIONS(1747),
    [sym_internal_queue_size_set] = ACTIONS(1747),
    [sym_internal_queue_size_get] = ACTIONS(1747),
    [sym_internal_threshold_set] = ACTIONS(1747),
    [sym_internal_threshold_get] = ACTIONS(1747),
    [sym_rebuilt_event] = ACTIONS(1747),
    [sym_do_custom_event] = ACTIONS(1747),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1747),
    [sym_clkmgr_set_clk_source] = ACTIONS(1747),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1747),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1747),
    [sym_clkmgr_get_timezone] = ACTIONS(1747),
    [sym_clkmgr_set_timezone] = ACTIONS(1747),
    [sym_clkmgr_get_resync_period] = ACTIONS(1747),
    [sym_clkmgr_set_resync_period] = ACTIONS(1747),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1747),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1747),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1747),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1747),
    [sym_clkmgr_get_timeservers] = ACTIONS(1747),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1747),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1747),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1747),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1747),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1747),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1747),
    [sym_set_log_level] = ACTIONS(1747),
    [sym_get_log_level] = ACTIONS(1747),
    [sym_amx_log] = ACTIONS(1747),
    [sym_smtp_server_config_set] = ACTIONS(1747),
    [sym_smtp_server_config_get] = ACTIONS(1747),
    [sym_smtp_send] = ACTIONS(1747),
    [sym_validate_netlinx_account] = ACTIONS(1747),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1747),
    [sym_authenticate_certificate] = ACTIONS(1747),
    [sym_audit_netlinx_session_event] = ACTIONS(1747),
    [sym_audit_netlinx_generic_event] = ACTIONS(1747),
    [sym_get_available_flash_disk_space] = ACTIONS(1747),
    [sym_get_max_flash_disk_space] = ACTIONS(1747),
    [sym_data_create_feed] = ACTIONS(1747),
    [sym_data_delete_feed] = ACTIONS(1747),
    [sym_data_publish_feed] = ACTIONS(1747),
    [sym_data_get_published_feed] = ACTIONS(1747),
    [sym_data_add_record] = ACTIONS(1747),
    [sym_data_get_event_record] = ACTIONS(1747),
    [sym__wc_data_create_feed] = ACTIONS(1747),
    [sym__wc_data_add_record] = ACTIONS(1747),
    [sym__wc_data_get_event_record] = ACTIONS(1747),
    [sym_format] = ACTIONS(1747),
    [sym_astro_clock] = ACTIONS(1747),
    [sym_set_outdoor_temperature] = ACTIONS(1747),
    [sym_set_virtual_level_count] = ACTIONS(1747),
    [sym_set_virtual_channel_count] = ACTIONS(1747),
    [sym_set_virtual_port_count] = ACTIONS(1747),
    [sym__wc_encode_int] = ACTIONS(1747),
    [sym__wc_decode_int] = ACTIONS(1747),
    [sym__wc_int] = ACTIONS(1747),
    [sym__wc_to_ch_int] = ACTIONS(1747),
    [sym__ch_to_wc_int] = ACTIONS(1747),
    [sym__wc_find_string_int] = ACTIONS(1747),
    [sym__wc_left_string_int] = ACTIONS(1747),
    [sym__wc_lower_string_int] = ACTIONS(1747),
    [sym__wc_mid_string_int] = ACTIONS(1747),
    [sym__wc_remove_string_int] = ACTIONS(1747),
    [sym__wc_right_string_int] = ACTIONS(1747),
    [sym__wc_upper_string_int] = ACTIONS(1747),
    [sym__wc_compare_string_int] = ACTIONS(1747),
    [sym__wc_get_buffer_char_int] = ACTIONS(1747),
    [sym__wc_get_buffer_string_int] = ACTIONS(1747),
    [sym__wc_concat_string_int] = ACTIONS(1747),
    [sym__wc_file_open_int] = ACTIONS(1747),
    [sym__wc_file_close_int] = ACTIONS(1747),
    [sym__wc_file_read_int] = ACTIONS(1747),
    [sym__wc_file_read_line_int] = ACTIONS(1747),
    [sym__wc_file_write_int] = ACTIONS(1747),
    [sym__wc_file_write_line_int] = ACTIONS(1747),
    [aux_sym_preproc_include_token1] = ACTIONS(1747),
    [aux_sym_preproc_define_token1] = ACTIONS(1749),
    [aux_sym_preproc_warn_token1] = ACTIONS(1749),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1749),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1749),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1749),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1749),
    [aux_sym_preproc_else_token1] = ACTIONS(1749),
    [anon_sym_LBRACK] = ACTIONS(1749),
    [anon_sym_LBRACE] = ACTIONS(1749),
    [anon_sym_LPAREN] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [anon_sym_DASH] = ACTIONS(1747),
    [anon_sym_PLUS] = ACTIONS(1747),
    [anon_sym_DASH_DASH] = ACTIONS(1749),
    [anon_sym_PLUS_PLUS] = ACTIONS(1749),
    [anon_sym_DQUOTE] = ACTIONS(1749),
    [anon_sym_SQUOTE] = ACTIONS(1749),
    [sym_number_literal] = ACTIONS(1749),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(217)] = {
    [sym_identifier] = ACTIONS(1751),
    [sym_program_name_keyword] = ACTIONS(1751),
    [sym_module_name_keyword] = ACTIONS(1751),
    [sym_define_device_keyword] = ACTIONS(1751),
    [sym_define_combine_keyword] = ACTIONS(1751),
    [sym_define_constant_keyword] = ACTIONS(1751),
    [sym_define_type_keyword] = ACTIONS(1751),
    [sym_define_variable_keyword] = ACTIONS(1751),
    [sym_define_system_variable_keyword] = ACTIONS(1751),
    [sym_define_connect_level_keyword] = ACTIONS(1751),
    [sym_define_latching_keyword] = ACTIONS(1751),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1751),
    [sym_define_toggling_keyword] = ACTIONS(1751),
    [sym_define_call_keyword] = ACTIONS(1751),
    [sym_define_function_keyword] = ACTIONS(1751),
    [sym_define_library_function_keyword] = ACTIONS(1751),
    [sym_define_start_keyword] = ACTIONS(1751),
    [sym_define_event_keyword] = ACTIONS(1751),
    [sym_define_program_keyword] = ACTIONS(1751),
    [sym_define_module_keyword] = ACTIONS(1751),
    [sym_char_keyword] = ACTIONS(1751),
    [sym_widechar_keyword] = ACTIONS(1751),
    [sym_integer_keyword] = ACTIONS(1751),
    [sym_sinteger_keyword] = ACTIONS(1751),
    [sym_long_keyword] = ACTIONS(1751),
    [sym_slong_keyword] = ACTIONS(1751),
    [sym_float_keyword] = ACTIONS(1751),
    [sym_double_keyword] = ACTIONS(1751),
    [sym_variant_keyword] = ACTIONS(1751),
    [sym_variantarray_keyword] = ACTIONS(1751),
    [sym_local_var_keyword] = ACTIONS(1751),
    [sym_stack_var_keyword] = ACTIONS(1751),
    [sym_constant_keyword] = ACTIONS(1751),
    [sym_volatile_keyword] = ACTIONS(1751),
    [sym_non_volatile_keyword] = ACTIONS(1751),
    [sym_persistent_keyword] = ACTIONS(1751),
    [sym_dev_keyword] = ACTIONS(1751),
    [sym_devlev_keyword] = ACTIONS(1751),
    [sym_devchan_keyword] = ACTIONS(1751),
    [sym_if_keyword] = ACTIONS(1751),
    [sym_else_keyword] = ACTIONS(1751),
    [sym_switch_keyword] = ACTIONS(1751),
    [sym_case_keyword] = ACTIONS(1751),
    [sym_default_keyword] = ACTIONS(1751),
    [sym_while_keyword] = ACTIONS(1751),
    [sym_for_keyword] = ACTIONS(1751),
    [sym_break_keyword] = ACTIONS(1751),
    [sym_continue_keyword] = ACTIONS(1751),
    [sym_return_keyword] = ACTIONS(1751),
    [sym_select_keyword] = ACTIONS(1751),
    [sym_active_keyword] = ACTIONS(1751),
    [sym_struct_keyword] = ACTIONS(1751),
    [sym_structure_keyword] = ACTIONS(1751),
    [sym_button_event_keyword] = ACTIONS(1751),
    [sym_channel_event_keyword] = ACTIONS(1751),
    [sym_level_event_keyword] = ACTIONS(1751),
    [sym_data_event_keyword] = ACTIONS(1751),
    [sym_timeline_event_keyword] = ACTIONS(1751),
    [sym_custom_event_keyword] = ACTIONS(1751),
    [sym_push_keyword] = ACTIONS(1751),
    [sym_release_keyword] = ACTIONS(1751),
    [aux_sym_on_keyword_token1] = ACTIONS(1751),
    [aux_sym_off_keyword_token1] = ACTIONS(1751),
    [sym_send_level_keyword] = ACTIONS(1751),
    [sym_send_string_keyword] = ACTIONS(1751),
    [sym_send_command_keyword] = ACTIONS(1751),
    [sym_clear_buffer_keyword] = ACTIONS(1751),
    [sym_create_buffer_keyword] = ACTIONS(1751),
    [sym_create_multi_buffer_keyword] = ACTIONS(1751),
    [sym_call_keyword] = ACTIONS(1751),
    [sym_system_call_keyword] = ACTIONS(1751),
    [sym_devchan_to_keyword] = ACTIONS(1751),
    [sym_devchan_min_to_keyword] = ACTIONS(1751),
    [sym_devchan_total_off_keyword] = ACTIONS(1751),
    [sym_devchan_pulse_keyword] = ACTIONS(1751),
    [sym_wait_keyword] = ACTIONS(1751),
    [sym_cancel_wait_keyword] = ACTIONS(1751),
    [sym_wait_until_keyword] = ACTIONS(1751),
    [sym_cancel_wait_until_keyword] = ACTIONS(1751),
    [sym_cancel_all_wait_keyword] = ACTIONS(1751),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1751),
    [sym_not] = ACTIONS(1751),
    [sym_bnot] = ACTIONS(1751),
    [sym___file__] = ACTIONS(1751),
    [sym___date__] = ACTIONS(1751),
    [sym___ldate__] = ACTIONS(1751),
    [sym___time__] = ACTIONS(1751),
    [sym___line__] = ACTIONS(1751),
    [sym___name__] = ACTIONS(1751),
    [sym___netlinx__] = ACTIONS(1751),
    [sym_netlinx_axi_version] = ACTIONS(1751),
    [sym_url_flg_tcp] = ACTIONS(1751),
    [sym_url_flg_accinfopresent] = ACTIONS(1751),
    [sym_url_flg_temp] = ACTIONS(1751),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1751),
    [sym_url_flg_stat_mask] = ACTIONS(1751),
    [sym_url_flg_stat_lookup] = ACTIONS(1751),
    [sym_url_flg_stat_connecting] = ACTIONS(1751),
    [sym_url_flg_stat_waiting] = ACTIONS(1751),
    [sym_url_flg_stat_connected] = ACTIONS(1751),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1751),
    [sym_first_virtual_device] = ACTIONS(1751),
    [sym_first_local_port] = ACTIONS(1751),
    [sym_dynamic_virtual_device] = ACTIONS(1751),
    [sym_timeline_once] = ACTIONS(1751),
    [sym_timeline_repeat] = ACTIONS(1751),
    [sym_timeline_absolute] = ACTIONS(1751),
    [sym_timeline_relative] = ACTIONS(1751),
    [sym_do_push_timed_infinite] = ACTIONS(1751),
    [sym_source_type_no_address] = ACTIONS(1751),
    [sym_source_type_neuron_id] = ACTIONS(1751),
    [sym_source_type_ip_address] = ACTIONS(1751),
    [sym_source_type_axlink] = ACTIONS(1751),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1751),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1751),
    [sym_source_type_ip_socket_address] = ACTIONS(1751),
    [sym_source_type_rs232] = ACTIONS(1751),
    [sym_source_type_internal] = ACTIONS(1751),
    [sym_source_type_mac_address] = ACTIONS(1751),
    [sym_source_type_ipv4_port] = ACTIONS(1751),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1751),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1751),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1751),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1751),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1751),
    [sym_true] = ACTIONS(1751),
    [sym_false] = ACTIONS(1751),
    [sym_file_read_only] = ACTIONS(1751),
    [sym_file_rw_new] = ACTIONS(1751),
    [sym_file_rw_append] = ACTIONS(1751),
    [sym_ip_tcp] = ACTIONS(1751),
    [sym_ip_udp] = ACTIONS(1751),
    [sym_ip_udp_2way] = ACTIONS(1751),
    [sym_xml_encode_types] = ACTIONS(1751),
    [sym_xml_encode_char_as_list] = ACTIONS(1751),
    [sym_xml_encode_le] = ACTIONS(1751),
    [sym_xml_decode_types] = ACTIONS(1751),
    [sym_xml_decode_no_preserve] = ACTIONS(1751),
    [sym_duet_dev_type_display_device] = ACTIONS(1751),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1751),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1751),
    [sym_duet_dev_type_io_device] = ACTIONS(1751),
    [sym_duet_dev_type_relay_device] = ACTIONS(1751),
    [sym_duet_dev_type_ups] = ACTIONS(1751),
    [sym_duet_dev_type_amplifier] = ACTIONS(1751),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1751),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1751),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1751),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1751),
    [sym_duet_dev_type_camera] = ACTIONS(1751),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1751),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1751),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1751),
    [sym_duet_dev_type_dss] = ACTIONS(1751),
    [sym_duet_dev_type_dvr] = ACTIONS(1751),
    [sym_duet_dev_type_disc_device] = ACTIONS(1751),
    [sym_duet_dev_type_document_camera] = ACTIONS(1751),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1751),
    [sym_duet_dev_type_hvac] = ACTIONS(1751),
    [sym_duet_dev_type_keypad] = ACTIONS(1751),
    [sym_duet_dev_type_light] = ACTIONS(1751),
    [sym_duet_dev_type_monitor] = ACTIONS(1751),
    [sym_duet_dev_type_motor] = ACTIONS(1751),
    [sym_duet_dev_type_multi_window] = ACTIONS(1751),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1751),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1751),
    [sym_duet_dev_type_receiver] = ACTIONS(1751),
    [sym_duet_dev_type_security_system] = ACTIONS(1751),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1751),
    [sym_duet_dev_type_settop_box] = ACTIONS(1751),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1751),
    [sym_duet_dev_type_switcher] = ACTIONS(1751),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1751),
    [sym_duet_dev_type_tv] = ACTIONS(1751),
    [sym_duet_dev_type_utility] = ACTIONS(1751),
    [sym_duet_dev_type_vcr] = ACTIONS(1751),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1751),
    [sym_duet_dev_type_video_processor] = ACTIONS(1751),
    [sym_duet_dev_type_video_projector] = ACTIONS(1751),
    [sym_duet_dev_type_video_wall] = ACTIONS(1751),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1751),
    [sym_duet_dev_type_weather] = ACTIONS(1751),
    [sym_duet_dev_not_polled] = ACTIONS(1751),
    [sym_duet_dev_polled] = ACTIONS(1751),
    [sym_clkmgr_mode_network] = ACTIONS(1751),
    [sym_clkmgr_mode_standalone] = ACTIONS(1751),
    [sym_normal_standby] = ACTIONS(1751),
    [sym_normal_wake] = ACTIONS(1751),
    [sym_ip_multicast_ttl_option] = ACTIONS(1751),
    [sym_ip_ttl_subnet] = ACTIONS(1751),
    [sym_ip_ttl_site] = ACTIONS(1751),
    [sym_ip_ttl_region] = ACTIONS(1751),
    [sym_ip_ttl_continent] = ACTIONS(1751),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1751),
    [sym_ip_nodelay_on] = ACTIONS(1751),
    [sym_ip_nodelay_off] = ACTIONS(1751),
    [sym_amx_error] = ACTIONS(1751),
    [sym_amx_warning] = ACTIONS(1751),
    [sym_amx_info] = ACTIONS(1751),
    [sym_amx_debug] = ACTIONS(1751),
    [sym_netlinx_login_success] = ACTIONS(1751),
    [sym_netlinx_login_fail] = ACTIONS(1751),
    [sym_netlinx_logout] = ACTIONS(1751),
    [sym_null_str] = ACTIONS(1751),
    [sym_smtp_address] = ACTIONS(1751),
    [sym_smtp_port_number] = ACTIONS(1751),
    [sym_smtp_username] = ACTIONS(1751),
    [sym_smtp_password] = ACTIONS(1751),
    [sym_smtp_require_tls] = ACTIONS(1751),
    [sym_smtp_from] = ACTIONS(1751),
    [sym_smtp_tls_true] = ACTIONS(1751),
    [sym_smtp_tls_false] = ACTIONS(1751),
    [sym_listview_on_row_select_event] = ACTIONS(1751),
    [sym_data_structure_unknown] = ACTIONS(1751),
    [sym_data_structure_datafeed] = ACTIONS(1751),
    [sym_data_structure_datarecordset] = ACTIONS(1751),
    [sym_data_structure_datarecord] = ACTIONS(1751),
    [sym_data_structure_datafield] = ACTIONS(1751),
    [sym_data_max_content_field_count] = ACTIONS(1751),
    [sym_data_max_metadata_field_count] = ACTIONS(1751),
    [sym_data_max_id_length] = ACTIONS(1751),
    [sym_data_max_name_length] = ACTIONS(1751),
    [sym_data_max_description_length] = ACTIONS(1751),
    [sym_data_max_type_length] = ACTIONS(1751),
    [sym_data_max_format_length] = ACTIONS(1751),
    [sym_data_max_source_length] = ACTIONS(1751),
    [sym_data_max_value_length] = ACTIONS(1751),
    [sym_data_max_label_length] = ACTIONS(1751),
    [sym_data_max_url_length] = ACTIONS(1751),
    [sym_max_last_login_info_length] = ACTIONS(1751),
    [sym_data_type_unknown] = ACTIONS(1751),
    [sym_data_type_string] = ACTIONS(1751),
    [sym_data_type_datetime] = ACTIONS(1751),
    [sym_data_type_date] = ACTIONS(1751),
    [sym_data_type_time] = ACTIONS(1751),
    [sym_data_type_image] = ACTIONS(1751),
    [sym_data_format_url] = ACTIONS(1751),
    [sym_data_format_phone] = ACTIONS(1751),
    [sym_data_format_email] = ACTIONS(1751),
    [sym_data_format_iso8601] = ACTIONS(1751),
    [sym_auditlog_priv] = ACTIONS(1751),
    [sym_configuration_priv] = ACTIONS(1751),
    [sym_deviceconfig_priv] = ACTIONS(1751),
    [sym_ftp_priv] = ACTIONS(1751),
    [sym_http_priv] = ACTIONS(1751),
    [sym_networkconfig_priv] = ACTIONS(1751),
    [sym_programport_priv] = ACTIONS(1751),
    [sym_remoteui_priv] = ACTIONS(1751),
    [sym_securitycontrol_priv] = ACTIONS(1751),
    [sym_softwaremanagement_priv] = ACTIONS(1751),
    [sym_terminal_priv] = ACTIONS(1751),
    [sym_tpadmin_priv] = ACTIONS(1751),
    [sym_user1_priv] = ACTIONS(1751),
    [sym_user2_priv] = ACTIONS(1751),
    [sym_user3_priv] = ACTIONS(1751),
    [sym_user4_priv] = ACTIONS(1751),
    [sym_usermanagement_priv] = ACTIONS(1751),
    [sym_valid_account] = ACTIONS(1751),
    [sym_err_name_invalid] = ACTIONS(1751),
    [sym_err_password_invalid] = ACTIONS(1751),
    [sym_err_invalid_account] = ACTIONS(1751),
    [sym_err_no_authorization] = ACTIONS(1751),
    [sym_err_invalid_parameter] = ACTIONS(1751),
    [sym_err_user_locked_out] = ACTIONS(1751),
    [sym_err_user_account_expired] = ACTIONS(1751),
    [sym_err_general_error] = ACTIONS(1751),
    [sym_err_authentication_error] = ACTIONS(1751),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1751),
    [sym_tls_validate_certificate] = ACTIONS(1751),
    [sym_tdata] = ACTIONS(1751),
    [sym_tchannel] = ACTIONS(1751),
    [sym_tlevel] = ACTIONS(1751),
    [sym_tbutton] = ACTIONS(1751),
    [sym_ttimeline] = ACTIONS(1751),
    [sym_tcustom] = ACTIONS(1751),
    [sym_url_struct] = ACTIONS(1751),
    [sym_dns_struct] = ACTIONS(1751),
    [sym_ip_address_struct] = ACTIONS(1751),
    [sym_dev_info_struct] = ACTIONS(1751),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1751),
    [sym_data_feed] = ACTIONS(1751),
    [sym_data_field] = ACTIONS(1751),
    [sym_data_record] = ACTIONS(1751),
    [sym_wc_data_feed] = ACTIONS(1751),
    [sym_wc_data_field] = ACTIONS(1751),
    [sym_wc_data_record] = ACTIONS(1751),
    [sym_last_login_info] = ACTIONS(1751),
    [sym____reserved___] = ACTIONS(1751),
    [sym_date] = ACTIONS(1751),
    [sym_day] = ACTIONS(1751),
    [sym_time] = ACTIONS(1751),
    [sym_ldate] = ACTIONS(1751),
    [sym_push_device] = ACTIONS(1751),
    [sym_push_channel] = ACTIONS(1751),
    [sym_push_devchan] = ACTIONS(1751),
    [sym_release_device] = ACTIONS(1751),
    [sym_release_channel] = ACTIONS(1751),
    [sym_release_devchan] = ACTIONS(1751),
    [sym_master_slot] = ACTIONS(1751),
    [sym_get_pulse_time] = ACTIONS(1751),
    [sym_get_timer] = ACTIONS(1751),
    [sym_master_sn] = ACTIONS(1751),
    [sym_system_number] = ACTIONS(1751),
    [sym_button] = ACTIONS(1751),
    [sym_dv_channel] = ACTIONS(1751),
    [sym_data] = ACTIONS(1751),
    [sym_level] = ACTIONS(1751),
    [sym_timeline] = ACTIONS(1751),
    [sym_channel] = ACTIONS(1751),
    [sym_custom] = ACTIONS(1751),
    [sym_atoi] = ACTIONS(1751),
    [sym_atol] = ACTIONS(1751),
    [sym_atof] = ACTIONS(1751),
    [sym_itoa] = ACTIONS(1751),
    [sym_itohex] = ACTIONS(1751),
    [sym_hextoi] = ACTIONS(1751),
    [sym_ftoa] = ACTIONS(1751),
    [sym_find_string] = ACTIONS(1751),
    [sym_left_string] = ACTIONS(1751),
    [sym_right_string] = ACTIONS(1751),
    [sym_mid_string] = ACTIONS(1751),
    [sym_remove_string] = ACTIONS(1751),
    [sym_get_buffer_string] = ACTIONS(1751),
    [sym_get_buffer_char] = ACTIONS(1751),
    [sym_get_multi_buffer_string] = ACTIONS(1751),
    [sym_length_string] = ACTIONS(1751),
    [sym_max_length_string] = ACTIONS(1751),
    [sym_set_length_string] = ACTIONS(1751),
    [sym_compare_string] = ACTIONS(1751),
    [sym_upper_string] = ACTIONS(1751),
    [sym_lower_string] = ACTIONS(1751),
    [sym_get_last] = ACTIONS(1751),
    [sym_date_to_day] = ACTIONS(1751),
    [sym_date_to_month] = ACTIONS(1751),
    [sym_date_to_year] = ACTIONS(1751),
    [sym_time_to_hour] = ACTIONS(1751),
    [sym_time_to_minute] = ACTIONS(1751),
    [sym_time_to_second] = ACTIONS(1751),
    [sym_day_of_week] = ACTIONS(1751),
    [sym_set_pulse_time] = ACTIONS(1751),
    [sym_set_timer] = ACTIONS(1751),
    [sym_file_open] = ACTIONS(1751),
    [sym_file_close] = ACTIONS(1751),
    [sym_file_read] = ACTIONS(1751),
    [sym_file_read_line] = ACTIONS(1751),
    [sym_file_write] = ACTIONS(1751),
    [sym_file_write_line] = ACTIONS(1751),
    [sym_file_copy] = ACTIONS(1751),
    [sym_file_delete] = ACTIONS(1751),
    [sym_file_rename] = ACTIONS(1751),
    [sym_file_dir] = ACTIONS(1751),
    [sym_file_getdir] = ACTIONS(1751),
    [sym_file_setdir] = ACTIONS(1751),
    [sym_file_create_dir] = ACTIONS(1751),
    [sym_file_remove_dir] = ACTIONS(1751),
    [sym_file_seek] = ACTIONS(1751),
    [sym_length_array] = ACTIONS(1751),
    [sym_max_length_array] = ACTIONS(1751),
    [sym_set_length_array] = ACTIONS(1751),
    [sym_ip_client_open] = ACTIONS(1751),
    [sym_ip_client_close] = ACTIONS(1751),
    [sym_ip_server_open] = ACTIONS(1751),
    [sym_ip_server_close] = ACTIONS(1751),
    [sym_ip_bound_client_open] = ACTIONS(1751),
    [sym_ip_mc_server_open] = ACTIONS(1751),
    [sym_ip_set_option] = ACTIONS(1751),
    [sym_ssh_client_open] = ACTIONS(1751),
    [sym_ssh_client_close] = ACTIONS(1751),
    [sym_tls_client_open] = ACTIONS(1751),
    [sym_tls_client_close] = ACTIONS(1751),
    [sym_do_push] = ACTIONS(1751),
    [sym_do_push_timed] = ACTIONS(1751),
    [sym_do_release] = ACTIONS(1751),
    [sym_device_id] = ACTIONS(1751),
    [sym_device_info] = ACTIONS(1751),
    [sym_device_id_string] = ACTIONS(1751),
    [sym_device_standby] = ACTIONS(1751),
    [sym_device_wake] = ACTIONS(1751),
    [sym_timeline_create] = ACTIONS(1751),
    [sym_timeline_kill] = ACTIONS(1751),
    [sym_timeline_reload] = ACTIONS(1751),
    [sym_timeline_pause] = ACTIONS(1751),
    [sym_timeline_restart] = ACTIONS(1751),
    [sym_timeline_set] = ACTIONS(1751),
    [sym_timeline_get] = ACTIONS(1751),
    [sym_timeline_active] = ACTIONS(1751),
    [sym_timeline_dynamic_id] = ACTIONS(1751),
    [sym_variable_to_string] = ACTIONS(1751),
    [sym_string_to_variable] = ACTIONS(1751),
    [sym_length_variable_to_string] = ACTIONS(1751),
    [sym_type_cast] = ACTIONS(1751),
    [sym_variable_to_xml] = ACTIONS(1751),
    [sym_xml_to_variable] = ACTIONS(1751),
    [sym_length_variable_to_xml] = ACTIONS(1751),
    [sym_redirect_string] = ACTIONS(1751),
    [sym_random_number] = ACTIONS(1751),
    [sym_get_unique_id] = ACTIONS(1751),
    [sym_get_system_number] = ACTIONS(1751),
    [sym_set_system_number] = ACTIONS(1751),
    [sym_get_serial_number] = ACTIONS(1751),
    [sym_get_master_build] = ACTIONS(1751),
    [sym_set_validation_code] = ACTIONS(1751),
    [sym_reboot] = ACTIONS(1751),
    [sym_load_duet_module] = ACTIONS(1751),
    [sym_unload_duet_module] = ACTIONS(1751),
    [sym_duet_mem_size_set] = ACTIONS(1751),
    [sym_duet_mem_size_get] = ACTIONS(1751),
    [sym_dynamic_application_device] = ACTIONS(1751),
    [sym_dynamic_polled_port] = ACTIONS(1751),
    [sym_static_port_binding] = ACTIONS(1751),
    [sym_static_ip_binding] = ACTIONS(1751),
    [sym_get_dns_list] = ACTIONS(1751),
    [sym_set_dns_list] = ACTIONS(1751),
    [sym_get_ip_address] = ACTIONS(1751),
    [sym_set_ip_address] = ACTIONS(1751),
    [sym_get_url_list] = ACTIONS(1751),
    [sym_add_url_entry] = ACTIONS(1751),
    [sym_delete_url_entry] = ACTIONS(1751),
    [sym_abs_value] = ACTIONS(1751),
    [sym_max_value] = ACTIONS(1751),
    [sym_min_value] = ACTIONS(1751),
    [sym_exp_value] = ACTIONS(1751),
    [sym_log_value] = ACTIONS(1751),
    [sym_log10_value] = ACTIONS(1751),
    [sym_power_value] = ACTIONS(1751),
    [sym_sqrt_value] = ACTIONS(1751),
    [sym_raw_be] = ACTIONS(1751),
    [sym_raw_le] = ACTIONS(1751),
    [sym_internal_queue_size_set] = ACTIONS(1751),
    [sym_internal_queue_size_get] = ACTIONS(1751),
    [sym_internal_threshold_set] = ACTIONS(1751),
    [sym_internal_threshold_get] = ACTIONS(1751),
    [sym_rebuilt_event] = ACTIONS(1751),
    [sym_do_custom_event] = ACTIONS(1751),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1751),
    [sym_clkmgr_set_clk_source] = ACTIONS(1751),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1751),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1751),
    [sym_clkmgr_get_timezone] = ACTIONS(1751),
    [sym_clkmgr_set_timezone] = ACTIONS(1751),
    [sym_clkmgr_get_resync_period] = ACTIONS(1751),
    [sym_clkmgr_set_resync_period] = ACTIONS(1751),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1751),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1751),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1751),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1751),
    [sym_clkmgr_get_timeservers] = ACTIONS(1751),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1751),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1751),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1751),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1751),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1751),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1751),
    [sym_set_log_level] = ACTIONS(1751),
    [sym_get_log_level] = ACTIONS(1751),
    [sym_amx_log] = ACTIONS(1751),
    [sym_smtp_server_config_set] = ACTIONS(1751),
    [sym_smtp_server_config_get] = ACTIONS(1751),
    [sym_smtp_send] = ACTIONS(1751),
    [sym_validate_netlinx_account] = ACTIONS(1751),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1751),
    [sym_authenticate_certificate] = ACTIONS(1751),
    [sym_audit_netlinx_session_event] = ACTIONS(1751),
    [sym_audit_netlinx_generic_event] = ACTIONS(1751),
    [sym_get_available_flash_disk_space] = ACTIONS(1751),
    [sym_get_max_flash_disk_space] = ACTIONS(1751),
    [sym_data_create_feed] = ACTIONS(1751),
    [sym_data_delete_feed] = ACTIONS(1751),
    [sym_data_publish_feed] = ACTIONS(1751),
    [sym_data_get_published_feed] = ACTIONS(1751),
    [sym_data_add_record] = ACTIONS(1751),
    [sym_data_get_event_record] = ACTIONS(1751),
    [sym__wc_data_create_feed] = ACTIONS(1751),
    [sym__wc_data_add_record] = ACTIONS(1751),
    [sym__wc_data_get_event_record] = ACTIONS(1751),
    [sym_format] = ACTIONS(1751),
    [sym_astro_clock] = ACTIONS(1751),
    [sym_set_outdoor_temperature] = ACTIONS(1751),
    [sym_set_virtual_level_count] = ACTIONS(1751),
    [sym_set_virtual_channel_count] = ACTIONS(1751),
    [sym_set_virtual_port_count] = ACTIONS(1751),
    [sym__wc_encode_int] = ACTIONS(1751),
    [sym__wc_decode_int] = ACTIONS(1751),
    [sym__wc_int] = ACTIONS(1751),
    [sym__wc_to_ch_int] = ACTIONS(1751),
    [sym__ch_to_wc_int] = ACTIONS(1751),
    [sym__wc_find_string_int] = ACTIONS(1751),
    [sym__wc_left_string_int] = ACTIONS(1751),
    [sym__wc_lower_string_int] = ACTIONS(1751),
    [sym__wc_mid_string_int] = ACTIONS(1751),
    [sym__wc_remove_string_int] = ACTIONS(1751),
    [sym__wc_right_string_int] = ACTIONS(1751),
    [sym__wc_upper_string_int] = ACTIONS(1751),
    [sym__wc_compare_string_int] = ACTIONS(1751),
    [sym__wc_get_buffer_char_int] = ACTIONS(1751),
    [sym__wc_get_buffer_string_int] = ACTIONS(1751),
    [sym__wc_concat_string_int] = ACTIONS(1751),
    [sym__wc_file_open_int] = ACTIONS(1751),
    [sym__wc_file_close_int] = ACTIONS(1751),
    [sym__wc_file_read_int] = ACTIONS(1751),
    [sym__wc_file_read_line_int] = ACTIONS(1751),
    [sym__wc_file_write_int] = ACTIONS(1751),
    [sym__wc_file_write_line_int] = ACTIONS(1751),
    [aux_sym_preproc_include_token1] = ACTIONS(1751),
    [aux_sym_preproc_define_token1] = ACTIONS(1753),
    [aux_sym_preproc_warn_token1] = ACTIONS(1753),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1753),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1753),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1753),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1753),
    [aux_sym_preproc_else_token1] = ACTIONS(1753),
    [anon_sym_LBRACK] = ACTIONS(1753),
    [anon_sym_LBRACE] = ACTIONS(1753),
    [anon_sym_LPAREN] = ACTIONS(1751),
    [anon_sym_BANG] = ACTIONS(1753),
    [anon_sym_TILDE] = ACTIONS(1753),
    [anon_sym_DASH] = ACTIONS(1751),
    [anon_sym_PLUS] = ACTIONS(1751),
    [anon_sym_DASH_DASH] = ACTIONS(1753),
    [anon_sym_PLUS_PLUS] = ACTIONS(1753),
    [anon_sym_DQUOTE] = ACTIONS(1753),
    [anon_sym_SQUOTE] = ACTIONS(1753),
    [sym_number_literal] = ACTIONS(1753),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(218)] = {
    [sym_identifier] = ACTIONS(1755),
    [sym_program_name_keyword] = ACTIONS(1755),
    [sym_module_name_keyword] = ACTIONS(1755),
    [sym_define_device_keyword] = ACTIONS(1755),
    [sym_define_combine_keyword] = ACTIONS(1755),
    [sym_define_constant_keyword] = ACTIONS(1755),
    [sym_define_type_keyword] = ACTIONS(1755),
    [sym_define_variable_keyword] = ACTIONS(1755),
    [sym_define_system_variable_keyword] = ACTIONS(1755),
    [sym_define_connect_level_keyword] = ACTIONS(1755),
    [sym_define_latching_keyword] = ACTIONS(1755),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1755),
    [sym_define_toggling_keyword] = ACTIONS(1755),
    [sym_define_call_keyword] = ACTIONS(1755),
    [sym_define_function_keyword] = ACTIONS(1755),
    [sym_define_library_function_keyword] = ACTIONS(1755),
    [sym_define_start_keyword] = ACTIONS(1755),
    [sym_define_event_keyword] = ACTIONS(1755),
    [sym_define_program_keyword] = ACTIONS(1755),
    [sym_define_module_keyword] = ACTIONS(1755),
    [sym_char_keyword] = ACTIONS(1755),
    [sym_widechar_keyword] = ACTIONS(1755),
    [sym_integer_keyword] = ACTIONS(1755),
    [sym_sinteger_keyword] = ACTIONS(1755),
    [sym_long_keyword] = ACTIONS(1755),
    [sym_slong_keyword] = ACTIONS(1755),
    [sym_float_keyword] = ACTIONS(1755),
    [sym_double_keyword] = ACTIONS(1755),
    [sym_variant_keyword] = ACTIONS(1755),
    [sym_variantarray_keyword] = ACTIONS(1755),
    [sym_local_var_keyword] = ACTIONS(1755),
    [sym_stack_var_keyword] = ACTIONS(1755),
    [sym_constant_keyword] = ACTIONS(1755),
    [sym_volatile_keyword] = ACTIONS(1755),
    [sym_non_volatile_keyword] = ACTIONS(1755),
    [sym_persistent_keyword] = ACTIONS(1755),
    [sym_dev_keyword] = ACTIONS(1755),
    [sym_devlev_keyword] = ACTIONS(1755),
    [sym_devchan_keyword] = ACTIONS(1755),
    [sym_if_keyword] = ACTIONS(1755),
    [sym_else_keyword] = ACTIONS(1755),
    [sym_switch_keyword] = ACTIONS(1755),
    [sym_case_keyword] = ACTIONS(1755),
    [sym_default_keyword] = ACTIONS(1755),
    [sym_while_keyword] = ACTIONS(1755),
    [sym_for_keyword] = ACTIONS(1755),
    [sym_break_keyword] = ACTIONS(1755),
    [sym_continue_keyword] = ACTIONS(1755),
    [sym_return_keyword] = ACTIONS(1755),
    [sym_select_keyword] = ACTIONS(1755),
    [sym_active_keyword] = ACTIONS(1755),
    [sym_struct_keyword] = ACTIONS(1755),
    [sym_structure_keyword] = ACTIONS(1755),
    [sym_button_event_keyword] = ACTIONS(1755),
    [sym_channel_event_keyword] = ACTIONS(1755),
    [sym_level_event_keyword] = ACTIONS(1755),
    [sym_data_event_keyword] = ACTIONS(1755),
    [sym_timeline_event_keyword] = ACTIONS(1755),
    [sym_custom_event_keyword] = ACTIONS(1755),
    [sym_push_keyword] = ACTIONS(1755),
    [sym_release_keyword] = ACTIONS(1755),
    [aux_sym_on_keyword_token1] = ACTIONS(1755),
    [aux_sym_off_keyword_token1] = ACTIONS(1755),
    [sym_send_level_keyword] = ACTIONS(1755),
    [sym_send_string_keyword] = ACTIONS(1755),
    [sym_send_command_keyword] = ACTIONS(1755),
    [sym_clear_buffer_keyword] = ACTIONS(1755),
    [sym_create_buffer_keyword] = ACTIONS(1755),
    [sym_create_multi_buffer_keyword] = ACTIONS(1755),
    [sym_call_keyword] = ACTIONS(1755),
    [sym_system_call_keyword] = ACTIONS(1755),
    [sym_devchan_to_keyword] = ACTIONS(1755),
    [sym_devchan_min_to_keyword] = ACTIONS(1755),
    [sym_devchan_total_off_keyword] = ACTIONS(1755),
    [sym_devchan_pulse_keyword] = ACTIONS(1755),
    [sym_wait_keyword] = ACTIONS(1755),
    [sym_cancel_wait_keyword] = ACTIONS(1755),
    [sym_wait_until_keyword] = ACTIONS(1755),
    [sym_cancel_wait_until_keyword] = ACTIONS(1755),
    [sym_cancel_all_wait_keyword] = ACTIONS(1755),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1755),
    [sym_not] = ACTIONS(1755),
    [sym_bnot] = ACTIONS(1755),
    [sym___file__] = ACTIONS(1755),
    [sym___date__] = ACTIONS(1755),
    [sym___ldate__] = ACTIONS(1755),
    [sym___time__] = ACTIONS(1755),
    [sym___line__] = ACTIONS(1755),
    [sym___name__] = ACTIONS(1755),
    [sym___netlinx__] = ACTIONS(1755),
    [sym_netlinx_axi_version] = ACTIONS(1755),
    [sym_url_flg_tcp] = ACTIONS(1755),
    [sym_url_flg_accinfopresent] = ACTIONS(1755),
    [sym_url_flg_temp] = ACTIONS(1755),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1755),
    [sym_url_flg_stat_mask] = ACTIONS(1755),
    [sym_url_flg_stat_lookup] = ACTIONS(1755),
    [sym_url_flg_stat_connecting] = ACTIONS(1755),
    [sym_url_flg_stat_waiting] = ACTIONS(1755),
    [sym_url_flg_stat_connected] = ACTIONS(1755),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1755),
    [sym_first_virtual_device] = ACTIONS(1755),
    [sym_first_local_port] = ACTIONS(1755),
    [sym_dynamic_virtual_device] = ACTIONS(1755),
    [sym_timeline_once] = ACTIONS(1755),
    [sym_timeline_repeat] = ACTIONS(1755),
    [sym_timeline_absolute] = ACTIONS(1755),
    [sym_timeline_relative] = ACTIONS(1755),
    [sym_do_push_timed_infinite] = ACTIONS(1755),
    [sym_source_type_no_address] = ACTIONS(1755),
    [sym_source_type_neuron_id] = ACTIONS(1755),
    [sym_source_type_ip_address] = ACTIONS(1755),
    [sym_source_type_axlink] = ACTIONS(1755),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1755),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1755),
    [sym_source_type_ip_socket_address] = ACTIONS(1755),
    [sym_source_type_rs232] = ACTIONS(1755),
    [sym_source_type_internal] = ACTIONS(1755),
    [sym_source_type_mac_address] = ACTIONS(1755),
    [sym_source_type_ipv4_port] = ACTIONS(1755),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1755),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1755),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1755),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1755),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1755),
    [sym_true] = ACTIONS(1755),
    [sym_false] = ACTIONS(1755),
    [sym_file_read_only] = ACTIONS(1755),
    [sym_file_rw_new] = ACTIONS(1755),
    [sym_file_rw_append] = ACTIONS(1755),
    [sym_ip_tcp] = ACTIONS(1755),
    [sym_ip_udp] = ACTIONS(1755),
    [sym_ip_udp_2way] = ACTIONS(1755),
    [sym_xml_encode_types] = ACTIONS(1755),
    [sym_xml_encode_char_as_list] = ACTIONS(1755),
    [sym_xml_encode_le] = ACTIONS(1755),
    [sym_xml_decode_types] = ACTIONS(1755),
    [sym_xml_decode_no_preserve] = ACTIONS(1755),
    [sym_duet_dev_type_display_device] = ACTIONS(1755),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1755),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1755),
    [sym_duet_dev_type_io_device] = ACTIONS(1755),
    [sym_duet_dev_type_relay_device] = ACTIONS(1755),
    [sym_duet_dev_type_ups] = ACTIONS(1755),
    [sym_duet_dev_type_amplifier] = ACTIONS(1755),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1755),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1755),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1755),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1755),
    [sym_duet_dev_type_camera] = ACTIONS(1755),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1755),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1755),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1755),
    [sym_duet_dev_type_dss] = ACTIONS(1755),
    [sym_duet_dev_type_dvr] = ACTIONS(1755),
    [sym_duet_dev_type_disc_device] = ACTIONS(1755),
    [sym_duet_dev_type_document_camera] = ACTIONS(1755),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1755),
    [sym_duet_dev_type_hvac] = ACTIONS(1755),
    [sym_duet_dev_type_keypad] = ACTIONS(1755),
    [sym_duet_dev_type_light] = ACTIONS(1755),
    [sym_duet_dev_type_monitor] = ACTIONS(1755),
    [sym_duet_dev_type_motor] = ACTIONS(1755),
    [sym_duet_dev_type_multi_window] = ACTIONS(1755),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1755),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1755),
    [sym_duet_dev_type_receiver] = ACTIONS(1755),
    [sym_duet_dev_type_security_system] = ACTIONS(1755),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1755),
    [sym_duet_dev_type_settop_box] = ACTIONS(1755),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1755),
    [sym_duet_dev_type_switcher] = ACTIONS(1755),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1755),
    [sym_duet_dev_type_tv] = ACTIONS(1755),
    [sym_duet_dev_type_utility] = ACTIONS(1755),
    [sym_duet_dev_type_vcr] = ACTIONS(1755),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1755),
    [sym_duet_dev_type_video_processor] = ACTIONS(1755),
    [sym_duet_dev_type_video_projector] = ACTIONS(1755),
    [sym_duet_dev_type_video_wall] = ACTIONS(1755),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1755),
    [sym_duet_dev_type_weather] = ACTIONS(1755),
    [sym_duet_dev_not_polled] = ACTIONS(1755),
    [sym_duet_dev_polled] = ACTIONS(1755),
    [sym_clkmgr_mode_network] = ACTIONS(1755),
    [sym_clkmgr_mode_standalone] = ACTIONS(1755),
    [sym_normal_standby] = ACTIONS(1755),
    [sym_normal_wake] = ACTIONS(1755),
    [sym_ip_multicast_ttl_option] = ACTIONS(1755),
    [sym_ip_ttl_subnet] = ACTIONS(1755),
    [sym_ip_ttl_site] = ACTIONS(1755),
    [sym_ip_ttl_region] = ACTIONS(1755),
    [sym_ip_ttl_continent] = ACTIONS(1755),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1755),
    [sym_ip_nodelay_on] = ACTIONS(1755),
    [sym_ip_nodelay_off] = ACTIONS(1755),
    [sym_amx_error] = ACTIONS(1755),
    [sym_amx_warning] = ACTIONS(1755),
    [sym_amx_info] = ACTIONS(1755),
    [sym_amx_debug] = ACTIONS(1755),
    [sym_netlinx_login_success] = ACTIONS(1755),
    [sym_netlinx_login_fail] = ACTIONS(1755),
    [sym_netlinx_logout] = ACTIONS(1755),
    [sym_null_str] = ACTIONS(1755),
    [sym_smtp_address] = ACTIONS(1755),
    [sym_smtp_port_number] = ACTIONS(1755),
    [sym_smtp_username] = ACTIONS(1755),
    [sym_smtp_password] = ACTIONS(1755),
    [sym_smtp_require_tls] = ACTIONS(1755),
    [sym_smtp_from] = ACTIONS(1755),
    [sym_smtp_tls_true] = ACTIONS(1755),
    [sym_smtp_tls_false] = ACTIONS(1755),
    [sym_listview_on_row_select_event] = ACTIONS(1755),
    [sym_data_structure_unknown] = ACTIONS(1755),
    [sym_data_structure_datafeed] = ACTIONS(1755),
    [sym_data_structure_datarecordset] = ACTIONS(1755),
    [sym_data_structure_datarecord] = ACTIONS(1755),
    [sym_data_structure_datafield] = ACTIONS(1755),
    [sym_data_max_content_field_count] = ACTIONS(1755),
    [sym_data_max_metadata_field_count] = ACTIONS(1755),
    [sym_data_max_id_length] = ACTIONS(1755),
    [sym_data_max_name_length] = ACTIONS(1755),
    [sym_data_max_description_length] = ACTIONS(1755),
    [sym_data_max_type_length] = ACTIONS(1755),
    [sym_data_max_format_length] = ACTIONS(1755),
    [sym_data_max_source_length] = ACTIONS(1755),
    [sym_data_max_value_length] = ACTIONS(1755),
    [sym_data_max_label_length] = ACTIONS(1755),
    [sym_data_max_url_length] = ACTIONS(1755),
    [sym_max_last_login_info_length] = ACTIONS(1755),
    [sym_data_type_unknown] = ACTIONS(1755),
    [sym_data_type_string] = ACTIONS(1755),
    [sym_data_type_datetime] = ACTIONS(1755),
    [sym_data_type_date] = ACTIONS(1755),
    [sym_data_type_time] = ACTIONS(1755),
    [sym_data_type_image] = ACTIONS(1755),
    [sym_data_format_url] = ACTIONS(1755),
    [sym_data_format_phone] = ACTIONS(1755),
    [sym_data_format_email] = ACTIONS(1755),
    [sym_data_format_iso8601] = ACTIONS(1755),
    [sym_auditlog_priv] = ACTIONS(1755),
    [sym_configuration_priv] = ACTIONS(1755),
    [sym_deviceconfig_priv] = ACTIONS(1755),
    [sym_ftp_priv] = ACTIONS(1755),
    [sym_http_priv] = ACTIONS(1755),
    [sym_networkconfig_priv] = ACTIONS(1755),
    [sym_programport_priv] = ACTIONS(1755),
    [sym_remoteui_priv] = ACTIONS(1755),
    [sym_securitycontrol_priv] = ACTIONS(1755),
    [sym_softwaremanagement_priv] = ACTIONS(1755),
    [sym_terminal_priv] = ACTIONS(1755),
    [sym_tpadmin_priv] = ACTIONS(1755),
    [sym_user1_priv] = ACTIONS(1755),
    [sym_user2_priv] = ACTIONS(1755),
    [sym_user3_priv] = ACTIONS(1755),
    [sym_user4_priv] = ACTIONS(1755),
    [sym_usermanagement_priv] = ACTIONS(1755),
    [sym_valid_account] = ACTIONS(1755),
    [sym_err_name_invalid] = ACTIONS(1755),
    [sym_err_password_invalid] = ACTIONS(1755),
    [sym_err_invalid_account] = ACTIONS(1755),
    [sym_err_no_authorization] = ACTIONS(1755),
    [sym_err_invalid_parameter] = ACTIONS(1755),
    [sym_err_user_locked_out] = ACTIONS(1755),
    [sym_err_user_account_expired] = ACTIONS(1755),
    [sym_err_general_error] = ACTIONS(1755),
    [sym_err_authentication_error] = ACTIONS(1755),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1755),
    [sym_tls_validate_certificate] = ACTIONS(1755),
    [sym_tdata] = ACTIONS(1755),
    [sym_tchannel] = ACTIONS(1755),
    [sym_tlevel] = ACTIONS(1755),
    [sym_tbutton] = ACTIONS(1755),
    [sym_ttimeline] = ACTIONS(1755),
    [sym_tcustom] = ACTIONS(1755),
    [sym_url_struct] = ACTIONS(1755),
    [sym_dns_struct] = ACTIONS(1755),
    [sym_ip_address_struct] = ACTIONS(1755),
    [sym_dev_info_struct] = ACTIONS(1755),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1755),
    [sym_data_feed] = ACTIONS(1755),
    [sym_data_field] = ACTIONS(1755),
    [sym_data_record] = ACTIONS(1755),
    [sym_wc_data_feed] = ACTIONS(1755),
    [sym_wc_data_field] = ACTIONS(1755),
    [sym_wc_data_record] = ACTIONS(1755),
    [sym_last_login_info] = ACTIONS(1755),
    [sym____reserved___] = ACTIONS(1755),
    [sym_date] = ACTIONS(1755),
    [sym_day] = ACTIONS(1755),
    [sym_time] = ACTIONS(1755),
    [sym_ldate] = ACTIONS(1755),
    [sym_push_device] = ACTIONS(1755),
    [sym_push_channel] = ACTIONS(1755),
    [sym_push_devchan] = ACTIONS(1755),
    [sym_release_device] = ACTIONS(1755),
    [sym_release_channel] = ACTIONS(1755),
    [sym_release_devchan] = ACTIONS(1755),
    [sym_master_slot] = ACTIONS(1755),
    [sym_get_pulse_time] = ACTIONS(1755),
    [sym_get_timer] = ACTIONS(1755),
    [sym_master_sn] = ACTIONS(1755),
    [sym_system_number] = ACTIONS(1755),
    [sym_button] = ACTIONS(1755),
    [sym_dv_channel] = ACTIONS(1755),
    [sym_data] = ACTIONS(1755),
    [sym_level] = ACTIONS(1755),
    [sym_timeline] = ACTIONS(1755),
    [sym_channel] = ACTIONS(1755),
    [sym_custom] = ACTIONS(1755),
    [sym_atoi] = ACTIONS(1755),
    [sym_atol] = ACTIONS(1755),
    [sym_atof] = ACTIONS(1755),
    [sym_itoa] = ACTIONS(1755),
    [sym_itohex] = ACTIONS(1755),
    [sym_hextoi] = ACTIONS(1755),
    [sym_ftoa] = ACTIONS(1755),
    [sym_find_string] = ACTIONS(1755),
    [sym_left_string] = ACTIONS(1755),
    [sym_right_string] = ACTIONS(1755),
    [sym_mid_string] = ACTIONS(1755),
    [sym_remove_string] = ACTIONS(1755),
    [sym_get_buffer_string] = ACTIONS(1755),
    [sym_get_buffer_char] = ACTIONS(1755),
    [sym_get_multi_buffer_string] = ACTIONS(1755),
    [sym_length_string] = ACTIONS(1755),
    [sym_max_length_string] = ACTIONS(1755),
    [sym_set_length_string] = ACTIONS(1755),
    [sym_compare_string] = ACTIONS(1755),
    [sym_upper_string] = ACTIONS(1755),
    [sym_lower_string] = ACTIONS(1755),
    [sym_get_last] = ACTIONS(1755),
    [sym_date_to_day] = ACTIONS(1755),
    [sym_date_to_month] = ACTIONS(1755),
    [sym_date_to_year] = ACTIONS(1755),
    [sym_time_to_hour] = ACTIONS(1755),
    [sym_time_to_minute] = ACTIONS(1755),
    [sym_time_to_second] = ACTIONS(1755),
    [sym_day_of_week] = ACTIONS(1755),
    [sym_set_pulse_time] = ACTIONS(1755),
    [sym_set_timer] = ACTIONS(1755),
    [sym_file_open] = ACTIONS(1755),
    [sym_file_close] = ACTIONS(1755),
    [sym_file_read] = ACTIONS(1755),
    [sym_file_read_line] = ACTIONS(1755),
    [sym_file_write] = ACTIONS(1755),
    [sym_file_write_line] = ACTIONS(1755),
    [sym_file_copy] = ACTIONS(1755),
    [sym_file_delete] = ACTIONS(1755),
    [sym_file_rename] = ACTIONS(1755),
    [sym_file_dir] = ACTIONS(1755),
    [sym_file_getdir] = ACTIONS(1755),
    [sym_file_setdir] = ACTIONS(1755),
    [sym_file_create_dir] = ACTIONS(1755),
    [sym_file_remove_dir] = ACTIONS(1755),
    [sym_file_seek] = ACTIONS(1755),
    [sym_length_array] = ACTIONS(1755),
    [sym_max_length_array] = ACTIONS(1755),
    [sym_set_length_array] = ACTIONS(1755),
    [sym_ip_client_open] = ACTIONS(1755),
    [sym_ip_client_close] = ACTIONS(1755),
    [sym_ip_server_open] = ACTIONS(1755),
    [sym_ip_server_close] = ACTIONS(1755),
    [sym_ip_bound_client_open] = ACTIONS(1755),
    [sym_ip_mc_server_open] = ACTIONS(1755),
    [sym_ip_set_option] = ACTIONS(1755),
    [sym_ssh_client_open] = ACTIONS(1755),
    [sym_ssh_client_close] = ACTIONS(1755),
    [sym_tls_client_open] = ACTIONS(1755),
    [sym_tls_client_close] = ACTIONS(1755),
    [sym_do_push] = ACTIONS(1755),
    [sym_do_push_timed] = ACTIONS(1755),
    [sym_do_release] = ACTIONS(1755),
    [sym_device_id] = ACTIONS(1755),
    [sym_device_info] = ACTIONS(1755),
    [sym_device_id_string] = ACTIONS(1755),
    [sym_device_standby] = ACTIONS(1755),
    [sym_device_wake] = ACTIONS(1755),
    [sym_timeline_create] = ACTIONS(1755),
    [sym_timeline_kill] = ACTIONS(1755),
    [sym_timeline_reload] = ACTIONS(1755),
    [sym_timeline_pause] = ACTIONS(1755),
    [sym_timeline_restart] = ACTIONS(1755),
    [sym_timeline_set] = ACTIONS(1755),
    [sym_timeline_get] = ACTIONS(1755),
    [sym_timeline_active] = ACTIONS(1755),
    [sym_timeline_dynamic_id] = ACTIONS(1755),
    [sym_variable_to_string] = ACTIONS(1755),
    [sym_string_to_variable] = ACTIONS(1755),
    [sym_length_variable_to_string] = ACTIONS(1755),
    [sym_type_cast] = ACTIONS(1755),
    [sym_variable_to_xml] = ACTIONS(1755),
    [sym_xml_to_variable] = ACTIONS(1755),
    [sym_length_variable_to_xml] = ACTIONS(1755),
    [sym_redirect_string] = ACTIONS(1755),
    [sym_random_number] = ACTIONS(1755),
    [sym_get_unique_id] = ACTIONS(1755),
    [sym_get_system_number] = ACTIONS(1755),
    [sym_set_system_number] = ACTIONS(1755),
    [sym_get_serial_number] = ACTIONS(1755),
    [sym_get_master_build] = ACTIONS(1755),
    [sym_set_validation_code] = ACTIONS(1755),
    [sym_reboot] = ACTIONS(1755),
    [sym_load_duet_module] = ACTIONS(1755),
    [sym_unload_duet_module] = ACTIONS(1755),
    [sym_duet_mem_size_set] = ACTIONS(1755),
    [sym_duet_mem_size_get] = ACTIONS(1755),
    [sym_dynamic_application_device] = ACTIONS(1755),
    [sym_dynamic_polled_port] = ACTIONS(1755),
    [sym_static_port_binding] = ACTIONS(1755),
    [sym_static_ip_binding] = ACTIONS(1755),
    [sym_get_dns_list] = ACTIONS(1755),
    [sym_set_dns_list] = ACTIONS(1755),
    [sym_get_ip_address] = ACTIONS(1755),
    [sym_set_ip_address] = ACTIONS(1755),
    [sym_get_url_list] = ACTIONS(1755),
    [sym_add_url_entry] = ACTIONS(1755),
    [sym_delete_url_entry] = ACTIONS(1755),
    [sym_abs_value] = ACTIONS(1755),
    [sym_max_value] = ACTIONS(1755),
    [sym_min_value] = ACTIONS(1755),
    [sym_exp_value] = ACTIONS(1755),
    [sym_log_value] = ACTIONS(1755),
    [sym_log10_value] = ACTIONS(1755),
    [sym_power_value] = ACTIONS(1755),
    [sym_sqrt_value] = ACTIONS(1755),
    [sym_raw_be] = ACTIONS(1755),
    [sym_raw_le] = ACTIONS(1755),
    [sym_internal_queue_size_set] = ACTIONS(1755),
    [sym_internal_queue_size_get] = ACTIONS(1755),
    [sym_internal_threshold_set] = ACTIONS(1755),
    [sym_internal_threshold_get] = ACTIONS(1755),
    [sym_rebuilt_event] = ACTIONS(1755),
    [sym_do_custom_event] = ACTIONS(1755),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1755),
    [sym_clkmgr_set_clk_source] = ACTIONS(1755),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1755),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1755),
    [sym_clkmgr_get_timezone] = ACTIONS(1755),
    [sym_clkmgr_set_timezone] = ACTIONS(1755),
    [sym_clkmgr_get_resync_period] = ACTIONS(1755),
    [sym_clkmgr_set_resync_period] = ACTIONS(1755),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1755),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1755),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1755),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1755),
    [sym_clkmgr_get_timeservers] = ACTIONS(1755),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1755),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1755),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1755),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1755),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1755),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1755),
    [sym_set_log_level] = ACTIONS(1755),
    [sym_get_log_level] = ACTIONS(1755),
    [sym_amx_log] = ACTIONS(1755),
    [sym_smtp_server_config_set] = ACTIONS(1755),
    [sym_smtp_server_config_get] = ACTIONS(1755),
    [sym_smtp_send] = ACTIONS(1755),
    [sym_validate_netlinx_account] = ACTIONS(1755),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1755),
    [sym_authenticate_certificate] = ACTIONS(1755),
    [sym_audit_netlinx_session_event] = ACTIONS(1755),
    [sym_audit_netlinx_generic_event] = ACTIONS(1755),
    [sym_get_available_flash_disk_space] = ACTIONS(1755),
    [sym_get_max_flash_disk_space] = ACTIONS(1755),
    [sym_data_create_feed] = ACTIONS(1755),
    [sym_data_delete_feed] = ACTIONS(1755),
    [sym_data_publish_feed] = ACTIONS(1755),
    [sym_data_get_published_feed] = ACTIONS(1755),
    [sym_data_add_record] = ACTIONS(1755),
    [sym_data_get_event_record] = ACTIONS(1755),
    [sym__wc_data_create_feed] = ACTIONS(1755),
    [sym__wc_data_add_record] = ACTIONS(1755),
    [sym__wc_data_get_event_record] = ACTIONS(1755),
    [sym_format] = ACTIONS(1755),
    [sym_astro_clock] = ACTIONS(1755),
    [sym_set_outdoor_temperature] = ACTIONS(1755),
    [sym_set_virtual_level_count] = ACTIONS(1755),
    [sym_set_virtual_channel_count] = ACTIONS(1755),
    [sym_set_virtual_port_count] = ACTIONS(1755),
    [sym__wc_encode_int] = ACTIONS(1755),
    [sym__wc_decode_int] = ACTIONS(1755),
    [sym__wc_int] = ACTIONS(1755),
    [sym__wc_to_ch_int] = ACTIONS(1755),
    [sym__ch_to_wc_int] = ACTIONS(1755),
    [sym__wc_find_string_int] = ACTIONS(1755),
    [sym__wc_left_string_int] = ACTIONS(1755),
    [sym__wc_lower_string_int] = ACTIONS(1755),
    [sym__wc_mid_string_int] = ACTIONS(1755),
    [sym__wc_remove_string_int] = ACTIONS(1755),
    [sym__wc_right_string_int] = ACTIONS(1755),
    [sym__wc_upper_string_int] = ACTIONS(1755),
    [sym__wc_compare_string_int] = ACTIONS(1755),
    [sym__wc_get_buffer_char_int] = ACTIONS(1755),
    [sym__wc_get_buffer_string_int] = ACTIONS(1755),
    [sym__wc_concat_string_int] = ACTIONS(1755),
    [sym__wc_file_open_int] = ACTIONS(1755),
    [sym__wc_file_close_int] = ACTIONS(1755),
    [sym__wc_file_read_int] = ACTIONS(1755),
    [sym__wc_file_read_line_int] = ACTIONS(1755),
    [sym__wc_file_write_int] = ACTIONS(1755),
    [sym__wc_file_write_line_int] = ACTIONS(1755),
    [aux_sym_preproc_include_token1] = ACTIONS(1755),
    [aux_sym_preproc_define_token1] = ACTIONS(1757),
    [aux_sym_preproc_warn_token1] = ACTIONS(1757),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1757),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1757),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1757),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1757),
    [aux_sym_preproc_else_token1] = ACTIONS(1757),
    [anon_sym_LBRACK] = ACTIONS(1757),
    [anon_sym_LBRACE] = ACTIONS(1757),
    [anon_sym_LPAREN] = ACTIONS(1755),
    [anon_sym_BANG] = ACTIONS(1757),
    [anon_sym_TILDE] = ACTIONS(1757),
    [anon_sym_DASH] = ACTIONS(1755),
    [anon_sym_PLUS] = ACTIONS(1755),
    [anon_sym_DASH_DASH] = ACTIONS(1757),
    [anon_sym_PLUS_PLUS] = ACTIONS(1757),
    [anon_sym_DQUOTE] = ACTIONS(1757),
    [anon_sym_SQUOTE] = ACTIONS(1757),
    [sym_number_literal] = ACTIONS(1757),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(219)] = {
    [sym_identifier] = ACTIONS(1759),
    [sym_program_name_keyword] = ACTIONS(1759),
    [sym_module_name_keyword] = ACTIONS(1759),
    [sym_define_device_keyword] = ACTIONS(1759),
    [sym_define_combine_keyword] = ACTIONS(1759),
    [sym_define_constant_keyword] = ACTIONS(1759),
    [sym_define_type_keyword] = ACTIONS(1759),
    [sym_define_variable_keyword] = ACTIONS(1759),
    [sym_define_system_variable_keyword] = ACTIONS(1759),
    [sym_define_connect_level_keyword] = ACTIONS(1759),
    [sym_define_latching_keyword] = ACTIONS(1759),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1759),
    [sym_define_toggling_keyword] = ACTIONS(1759),
    [sym_define_call_keyword] = ACTIONS(1759),
    [sym_define_function_keyword] = ACTIONS(1759),
    [sym_define_library_function_keyword] = ACTIONS(1759),
    [sym_define_start_keyword] = ACTIONS(1759),
    [sym_define_event_keyword] = ACTIONS(1759),
    [sym_define_program_keyword] = ACTIONS(1759),
    [sym_define_module_keyword] = ACTIONS(1759),
    [sym_char_keyword] = ACTIONS(1759),
    [sym_widechar_keyword] = ACTIONS(1759),
    [sym_integer_keyword] = ACTIONS(1759),
    [sym_sinteger_keyword] = ACTIONS(1759),
    [sym_long_keyword] = ACTIONS(1759),
    [sym_slong_keyword] = ACTIONS(1759),
    [sym_float_keyword] = ACTIONS(1759),
    [sym_double_keyword] = ACTIONS(1759),
    [sym_variant_keyword] = ACTIONS(1759),
    [sym_variantarray_keyword] = ACTIONS(1759),
    [sym_local_var_keyword] = ACTIONS(1759),
    [sym_stack_var_keyword] = ACTIONS(1759),
    [sym_constant_keyword] = ACTIONS(1759),
    [sym_volatile_keyword] = ACTIONS(1759),
    [sym_non_volatile_keyword] = ACTIONS(1759),
    [sym_persistent_keyword] = ACTIONS(1759),
    [sym_dev_keyword] = ACTIONS(1759),
    [sym_devlev_keyword] = ACTIONS(1759),
    [sym_devchan_keyword] = ACTIONS(1759),
    [sym_if_keyword] = ACTIONS(1759),
    [sym_else_keyword] = ACTIONS(1759),
    [sym_switch_keyword] = ACTIONS(1759),
    [sym_case_keyword] = ACTIONS(1759),
    [sym_default_keyword] = ACTIONS(1759),
    [sym_while_keyword] = ACTIONS(1759),
    [sym_for_keyword] = ACTIONS(1759),
    [sym_break_keyword] = ACTIONS(1759),
    [sym_continue_keyword] = ACTIONS(1759),
    [sym_return_keyword] = ACTIONS(1759),
    [sym_select_keyword] = ACTIONS(1759),
    [sym_active_keyword] = ACTIONS(1759),
    [sym_struct_keyword] = ACTIONS(1759),
    [sym_structure_keyword] = ACTIONS(1759),
    [sym_button_event_keyword] = ACTIONS(1759),
    [sym_channel_event_keyword] = ACTIONS(1759),
    [sym_level_event_keyword] = ACTIONS(1759),
    [sym_data_event_keyword] = ACTIONS(1759),
    [sym_timeline_event_keyword] = ACTIONS(1759),
    [sym_custom_event_keyword] = ACTIONS(1759),
    [sym_push_keyword] = ACTIONS(1759),
    [sym_release_keyword] = ACTIONS(1759),
    [aux_sym_on_keyword_token1] = ACTIONS(1759),
    [aux_sym_off_keyword_token1] = ACTIONS(1759),
    [sym_send_level_keyword] = ACTIONS(1759),
    [sym_send_string_keyword] = ACTIONS(1759),
    [sym_send_command_keyword] = ACTIONS(1759),
    [sym_clear_buffer_keyword] = ACTIONS(1759),
    [sym_create_buffer_keyword] = ACTIONS(1759),
    [sym_create_multi_buffer_keyword] = ACTIONS(1759),
    [sym_call_keyword] = ACTIONS(1759),
    [sym_system_call_keyword] = ACTIONS(1759),
    [sym_devchan_to_keyword] = ACTIONS(1759),
    [sym_devchan_min_to_keyword] = ACTIONS(1759),
    [sym_devchan_total_off_keyword] = ACTIONS(1759),
    [sym_devchan_pulse_keyword] = ACTIONS(1759),
    [sym_wait_keyword] = ACTIONS(1759),
    [sym_cancel_wait_keyword] = ACTIONS(1759),
    [sym_wait_until_keyword] = ACTIONS(1759),
    [sym_cancel_wait_until_keyword] = ACTIONS(1759),
    [sym_cancel_all_wait_keyword] = ACTIONS(1759),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1759),
    [sym_not] = ACTIONS(1759),
    [sym_bnot] = ACTIONS(1759),
    [sym___file__] = ACTIONS(1759),
    [sym___date__] = ACTIONS(1759),
    [sym___ldate__] = ACTIONS(1759),
    [sym___time__] = ACTIONS(1759),
    [sym___line__] = ACTIONS(1759),
    [sym___name__] = ACTIONS(1759),
    [sym___netlinx__] = ACTIONS(1759),
    [sym_netlinx_axi_version] = ACTIONS(1759),
    [sym_url_flg_tcp] = ACTIONS(1759),
    [sym_url_flg_accinfopresent] = ACTIONS(1759),
    [sym_url_flg_temp] = ACTIONS(1759),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1759),
    [sym_url_flg_stat_mask] = ACTIONS(1759),
    [sym_url_flg_stat_lookup] = ACTIONS(1759),
    [sym_url_flg_stat_connecting] = ACTIONS(1759),
    [sym_url_flg_stat_waiting] = ACTIONS(1759),
    [sym_url_flg_stat_connected] = ACTIONS(1759),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1759),
    [sym_first_virtual_device] = ACTIONS(1759),
    [sym_first_local_port] = ACTIONS(1759),
    [sym_dynamic_virtual_device] = ACTIONS(1759),
    [sym_timeline_once] = ACTIONS(1759),
    [sym_timeline_repeat] = ACTIONS(1759),
    [sym_timeline_absolute] = ACTIONS(1759),
    [sym_timeline_relative] = ACTIONS(1759),
    [sym_do_push_timed_infinite] = ACTIONS(1759),
    [sym_source_type_no_address] = ACTIONS(1759),
    [sym_source_type_neuron_id] = ACTIONS(1759),
    [sym_source_type_ip_address] = ACTIONS(1759),
    [sym_source_type_axlink] = ACTIONS(1759),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1759),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1759),
    [sym_source_type_ip_socket_address] = ACTIONS(1759),
    [sym_source_type_rs232] = ACTIONS(1759),
    [sym_source_type_internal] = ACTIONS(1759),
    [sym_source_type_mac_address] = ACTIONS(1759),
    [sym_source_type_ipv4_port] = ACTIONS(1759),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1759),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1759),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1759),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1759),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1759),
    [sym_true] = ACTIONS(1759),
    [sym_false] = ACTIONS(1759),
    [sym_file_read_only] = ACTIONS(1759),
    [sym_file_rw_new] = ACTIONS(1759),
    [sym_file_rw_append] = ACTIONS(1759),
    [sym_ip_tcp] = ACTIONS(1759),
    [sym_ip_udp] = ACTIONS(1759),
    [sym_ip_udp_2way] = ACTIONS(1759),
    [sym_xml_encode_types] = ACTIONS(1759),
    [sym_xml_encode_char_as_list] = ACTIONS(1759),
    [sym_xml_encode_le] = ACTIONS(1759),
    [sym_xml_decode_types] = ACTIONS(1759),
    [sym_xml_decode_no_preserve] = ACTIONS(1759),
    [sym_duet_dev_type_display_device] = ACTIONS(1759),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1759),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1759),
    [sym_duet_dev_type_io_device] = ACTIONS(1759),
    [sym_duet_dev_type_relay_device] = ACTIONS(1759),
    [sym_duet_dev_type_ups] = ACTIONS(1759),
    [sym_duet_dev_type_amplifier] = ACTIONS(1759),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1759),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1759),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1759),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1759),
    [sym_duet_dev_type_camera] = ACTIONS(1759),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1759),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1759),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1759),
    [sym_duet_dev_type_dss] = ACTIONS(1759),
    [sym_duet_dev_type_dvr] = ACTIONS(1759),
    [sym_duet_dev_type_disc_device] = ACTIONS(1759),
    [sym_duet_dev_type_document_camera] = ACTIONS(1759),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1759),
    [sym_duet_dev_type_hvac] = ACTIONS(1759),
    [sym_duet_dev_type_keypad] = ACTIONS(1759),
    [sym_duet_dev_type_light] = ACTIONS(1759),
    [sym_duet_dev_type_monitor] = ACTIONS(1759),
    [sym_duet_dev_type_motor] = ACTIONS(1759),
    [sym_duet_dev_type_multi_window] = ACTIONS(1759),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1759),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1759),
    [sym_duet_dev_type_receiver] = ACTIONS(1759),
    [sym_duet_dev_type_security_system] = ACTIONS(1759),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1759),
    [sym_duet_dev_type_settop_box] = ACTIONS(1759),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1759),
    [sym_duet_dev_type_switcher] = ACTIONS(1759),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1759),
    [sym_duet_dev_type_tv] = ACTIONS(1759),
    [sym_duet_dev_type_utility] = ACTIONS(1759),
    [sym_duet_dev_type_vcr] = ACTIONS(1759),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1759),
    [sym_duet_dev_type_video_processor] = ACTIONS(1759),
    [sym_duet_dev_type_video_projector] = ACTIONS(1759),
    [sym_duet_dev_type_video_wall] = ACTIONS(1759),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1759),
    [sym_duet_dev_type_weather] = ACTIONS(1759),
    [sym_duet_dev_not_polled] = ACTIONS(1759),
    [sym_duet_dev_polled] = ACTIONS(1759),
    [sym_clkmgr_mode_network] = ACTIONS(1759),
    [sym_clkmgr_mode_standalone] = ACTIONS(1759),
    [sym_normal_standby] = ACTIONS(1759),
    [sym_normal_wake] = ACTIONS(1759),
    [sym_ip_multicast_ttl_option] = ACTIONS(1759),
    [sym_ip_ttl_subnet] = ACTIONS(1759),
    [sym_ip_ttl_site] = ACTIONS(1759),
    [sym_ip_ttl_region] = ACTIONS(1759),
    [sym_ip_ttl_continent] = ACTIONS(1759),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1759),
    [sym_ip_nodelay_on] = ACTIONS(1759),
    [sym_ip_nodelay_off] = ACTIONS(1759),
    [sym_amx_error] = ACTIONS(1759),
    [sym_amx_warning] = ACTIONS(1759),
    [sym_amx_info] = ACTIONS(1759),
    [sym_amx_debug] = ACTIONS(1759),
    [sym_netlinx_login_success] = ACTIONS(1759),
    [sym_netlinx_login_fail] = ACTIONS(1759),
    [sym_netlinx_logout] = ACTIONS(1759),
    [sym_null_str] = ACTIONS(1759),
    [sym_smtp_address] = ACTIONS(1759),
    [sym_smtp_port_number] = ACTIONS(1759),
    [sym_smtp_username] = ACTIONS(1759),
    [sym_smtp_password] = ACTIONS(1759),
    [sym_smtp_require_tls] = ACTIONS(1759),
    [sym_smtp_from] = ACTIONS(1759),
    [sym_smtp_tls_true] = ACTIONS(1759),
    [sym_smtp_tls_false] = ACTIONS(1759),
    [sym_listview_on_row_select_event] = ACTIONS(1759),
    [sym_data_structure_unknown] = ACTIONS(1759),
    [sym_data_structure_datafeed] = ACTIONS(1759),
    [sym_data_structure_datarecordset] = ACTIONS(1759),
    [sym_data_structure_datarecord] = ACTIONS(1759),
    [sym_data_structure_datafield] = ACTIONS(1759),
    [sym_data_max_content_field_count] = ACTIONS(1759),
    [sym_data_max_metadata_field_count] = ACTIONS(1759),
    [sym_data_max_id_length] = ACTIONS(1759),
    [sym_data_max_name_length] = ACTIONS(1759),
    [sym_data_max_description_length] = ACTIONS(1759),
    [sym_data_max_type_length] = ACTIONS(1759),
    [sym_data_max_format_length] = ACTIONS(1759),
    [sym_data_max_source_length] = ACTIONS(1759),
    [sym_data_max_value_length] = ACTIONS(1759),
    [sym_data_max_label_length] = ACTIONS(1759),
    [sym_data_max_url_length] = ACTIONS(1759),
    [sym_max_last_login_info_length] = ACTIONS(1759),
    [sym_data_type_unknown] = ACTIONS(1759),
    [sym_data_type_string] = ACTIONS(1759),
    [sym_data_type_datetime] = ACTIONS(1759),
    [sym_data_type_date] = ACTIONS(1759),
    [sym_data_type_time] = ACTIONS(1759),
    [sym_data_type_image] = ACTIONS(1759),
    [sym_data_format_url] = ACTIONS(1759),
    [sym_data_format_phone] = ACTIONS(1759),
    [sym_data_format_email] = ACTIONS(1759),
    [sym_data_format_iso8601] = ACTIONS(1759),
    [sym_auditlog_priv] = ACTIONS(1759),
    [sym_configuration_priv] = ACTIONS(1759),
    [sym_deviceconfig_priv] = ACTIONS(1759),
    [sym_ftp_priv] = ACTIONS(1759),
    [sym_http_priv] = ACTIONS(1759),
    [sym_networkconfig_priv] = ACTIONS(1759),
    [sym_programport_priv] = ACTIONS(1759),
    [sym_remoteui_priv] = ACTIONS(1759),
    [sym_securitycontrol_priv] = ACTIONS(1759),
    [sym_softwaremanagement_priv] = ACTIONS(1759),
    [sym_terminal_priv] = ACTIONS(1759),
    [sym_tpadmin_priv] = ACTIONS(1759),
    [sym_user1_priv] = ACTIONS(1759),
    [sym_user2_priv] = ACTIONS(1759),
    [sym_user3_priv] = ACTIONS(1759),
    [sym_user4_priv] = ACTIONS(1759),
    [sym_usermanagement_priv] = ACTIONS(1759),
    [sym_valid_account] = ACTIONS(1759),
    [sym_err_name_invalid] = ACTIONS(1759),
    [sym_err_password_invalid] = ACTIONS(1759),
    [sym_err_invalid_account] = ACTIONS(1759),
    [sym_err_no_authorization] = ACTIONS(1759),
    [sym_err_invalid_parameter] = ACTIONS(1759),
    [sym_err_user_locked_out] = ACTIONS(1759),
    [sym_err_user_account_expired] = ACTIONS(1759),
    [sym_err_general_error] = ACTIONS(1759),
    [sym_err_authentication_error] = ACTIONS(1759),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1759),
    [sym_tls_validate_certificate] = ACTIONS(1759),
    [sym_tdata] = ACTIONS(1759),
    [sym_tchannel] = ACTIONS(1759),
    [sym_tlevel] = ACTIONS(1759),
    [sym_tbutton] = ACTIONS(1759),
    [sym_ttimeline] = ACTIONS(1759),
    [sym_tcustom] = ACTIONS(1759),
    [sym_url_struct] = ACTIONS(1759),
    [sym_dns_struct] = ACTIONS(1759),
    [sym_ip_address_struct] = ACTIONS(1759),
    [sym_dev_info_struct] = ACTIONS(1759),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1759),
    [sym_data_feed] = ACTIONS(1759),
    [sym_data_field] = ACTIONS(1759),
    [sym_data_record] = ACTIONS(1759),
    [sym_wc_data_feed] = ACTIONS(1759),
    [sym_wc_data_field] = ACTIONS(1759),
    [sym_wc_data_record] = ACTIONS(1759),
    [sym_last_login_info] = ACTIONS(1759),
    [sym____reserved___] = ACTIONS(1759),
    [sym_date] = ACTIONS(1759),
    [sym_day] = ACTIONS(1759),
    [sym_time] = ACTIONS(1759),
    [sym_ldate] = ACTIONS(1759),
    [sym_push_device] = ACTIONS(1759),
    [sym_push_channel] = ACTIONS(1759),
    [sym_push_devchan] = ACTIONS(1759),
    [sym_release_device] = ACTIONS(1759),
    [sym_release_channel] = ACTIONS(1759),
    [sym_release_devchan] = ACTIONS(1759),
    [sym_master_slot] = ACTIONS(1759),
    [sym_get_pulse_time] = ACTIONS(1759),
    [sym_get_timer] = ACTIONS(1759),
    [sym_master_sn] = ACTIONS(1759),
    [sym_system_number] = ACTIONS(1759),
    [sym_button] = ACTIONS(1759),
    [sym_dv_channel] = ACTIONS(1759),
    [sym_data] = ACTIONS(1759),
    [sym_level] = ACTIONS(1759),
    [sym_timeline] = ACTIONS(1759),
    [sym_channel] = ACTIONS(1759),
    [sym_custom] = ACTIONS(1759),
    [sym_atoi] = ACTIONS(1759),
    [sym_atol] = ACTIONS(1759),
    [sym_atof] = ACTIONS(1759),
    [sym_itoa] = ACTIONS(1759),
    [sym_itohex] = ACTIONS(1759),
    [sym_hextoi] = ACTIONS(1759),
    [sym_ftoa] = ACTIONS(1759),
    [sym_find_string] = ACTIONS(1759),
    [sym_left_string] = ACTIONS(1759),
    [sym_right_string] = ACTIONS(1759),
    [sym_mid_string] = ACTIONS(1759),
    [sym_remove_string] = ACTIONS(1759),
    [sym_get_buffer_string] = ACTIONS(1759),
    [sym_get_buffer_char] = ACTIONS(1759),
    [sym_get_multi_buffer_string] = ACTIONS(1759),
    [sym_length_string] = ACTIONS(1759),
    [sym_max_length_string] = ACTIONS(1759),
    [sym_set_length_string] = ACTIONS(1759),
    [sym_compare_string] = ACTIONS(1759),
    [sym_upper_string] = ACTIONS(1759),
    [sym_lower_string] = ACTIONS(1759),
    [sym_get_last] = ACTIONS(1759),
    [sym_date_to_day] = ACTIONS(1759),
    [sym_date_to_month] = ACTIONS(1759),
    [sym_date_to_year] = ACTIONS(1759),
    [sym_time_to_hour] = ACTIONS(1759),
    [sym_time_to_minute] = ACTIONS(1759),
    [sym_time_to_second] = ACTIONS(1759),
    [sym_day_of_week] = ACTIONS(1759),
    [sym_set_pulse_time] = ACTIONS(1759),
    [sym_set_timer] = ACTIONS(1759),
    [sym_file_open] = ACTIONS(1759),
    [sym_file_close] = ACTIONS(1759),
    [sym_file_read] = ACTIONS(1759),
    [sym_file_read_line] = ACTIONS(1759),
    [sym_file_write] = ACTIONS(1759),
    [sym_file_write_line] = ACTIONS(1759),
    [sym_file_copy] = ACTIONS(1759),
    [sym_file_delete] = ACTIONS(1759),
    [sym_file_rename] = ACTIONS(1759),
    [sym_file_dir] = ACTIONS(1759),
    [sym_file_getdir] = ACTIONS(1759),
    [sym_file_setdir] = ACTIONS(1759),
    [sym_file_create_dir] = ACTIONS(1759),
    [sym_file_remove_dir] = ACTIONS(1759),
    [sym_file_seek] = ACTIONS(1759),
    [sym_length_array] = ACTIONS(1759),
    [sym_max_length_array] = ACTIONS(1759),
    [sym_set_length_array] = ACTIONS(1759),
    [sym_ip_client_open] = ACTIONS(1759),
    [sym_ip_client_close] = ACTIONS(1759),
    [sym_ip_server_open] = ACTIONS(1759),
    [sym_ip_server_close] = ACTIONS(1759),
    [sym_ip_bound_client_open] = ACTIONS(1759),
    [sym_ip_mc_server_open] = ACTIONS(1759),
    [sym_ip_set_option] = ACTIONS(1759),
    [sym_ssh_client_open] = ACTIONS(1759),
    [sym_ssh_client_close] = ACTIONS(1759),
    [sym_tls_client_open] = ACTIONS(1759),
    [sym_tls_client_close] = ACTIONS(1759),
    [sym_do_push] = ACTIONS(1759),
    [sym_do_push_timed] = ACTIONS(1759),
    [sym_do_release] = ACTIONS(1759),
    [sym_device_id] = ACTIONS(1759),
    [sym_device_info] = ACTIONS(1759),
    [sym_device_id_string] = ACTIONS(1759),
    [sym_device_standby] = ACTIONS(1759),
    [sym_device_wake] = ACTIONS(1759),
    [sym_timeline_create] = ACTIONS(1759),
    [sym_timeline_kill] = ACTIONS(1759),
    [sym_timeline_reload] = ACTIONS(1759),
    [sym_timeline_pause] = ACTIONS(1759),
    [sym_timeline_restart] = ACTIONS(1759),
    [sym_timeline_set] = ACTIONS(1759),
    [sym_timeline_get] = ACTIONS(1759),
    [sym_timeline_active] = ACTIONS(1759),
    [sym_timeline_dynamic_id] = ACTIONS(1759),
    [sym_variable_to_string] = ACTIONS(1759),
    [sym_string_to_variable] = ACTIONS(1759),
    [sym_length_variable_to_string] = ACTIONS(1759),
    [sym_type_cast] = ACTIONS(1759),
    [sym_variable_to_xml] = ACTIONS(1759),
    [sym_xml_to_variable] = ACTIONS(1759),
    [sym_length_variable_to_xml] = ACTIONS(1759),
    [sym_redirect_string] = ACTIONS(1759),
    [sym_random_number] = ACTIONS(1759),
    [sym_get_unique_id] = ACTIONS(1759),
    [sym_get_system_number] = ACTIONS(1759),
    [sym_set_system_number] = ACTIONS(1759),
    [sym_get_serial_number] = ACTIONS(1759),
    [sym_get_master_build] = ACTIONS(1759),
    [sym_set_validation_code] = ACTIONS(1759),
    [sym_reboot] = ACTIONS(1759),
    [sym_load_duet_module] = ACTIONS(1759),
    [sym_unload_duet_module] = ACTIONS(1759),
    [sym_duet_mem_size_set] = ACTIONS(1759),
    [sym_duet_mem_size_get] = ACTIONS(1759),
    [sym_dynamic_application_device] = ACTIONS(1759),
    [sym_dynamic_polled_port] = ACTIONS(1759),
    [sym_static_port_binding] = ACTIONS(1759),
    [sym_static_ip_binding] = ACTIONS(1759),
    [sym_get_dns_list] = ACTIONS(1759),
    [sym_set_dns_list] = ACTIONS(1759),
    [sym_get_ip_address] = ACTIONS(1759),
    [sym_set_ip_address] = ACTIONS(1759),
    [sym_get_url_list] = ACTIONS(1759),
    [sym_add_url_entry] = ACTIONS(1759),
    [sym_delete_url_entry] = ACTIONS(1759),
    [sym_abs_value] = ACTIONS(1759),
    [sym_max_value] = ACTIONS(1759),
    [sym_min_value] = ACTIONS(1759),
    [sym_exp_value] = ACTIONS(1759),
    [sym_log_value] = ACTIONS(1759),
    [sym_log10_value] = ACTIONS(1759),
    [sym_power_value] = ACTIONS(1759),
    [sym_sqrt_value] = ACTIONS(1759),
    [sym_raw_be] = ACTIONS(1759),
    [sym_raw_le] = ACTIONS(1759),
    [sym_internal_queue_size_set] = ACTIONS(1759),
    [sym_internal_queue_size_get] = ACTIONS(1759),
    [sym_internal_threshold_set] = ACTIONS(1759),
    [sym_internal_threshold_get] = ACTIONS(1759),
    [sym_rebuilt_event] = ACTIONS(1759),
    [sym_do_custom_event] = ACTIONS(1759),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1759),
    [sym_clkmgr_set_clk_source] = ACTIONS(1759),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1759),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1759),
    [sym_clkmgr_get_timezone] = ACTIONS(1759),
    [sym_clkmgr_set_timezone] = ACTIONS(1759),
    [sym_clkmgr_get_resync_period] = ACTIONS(1759),
    [sym_clkmgr_set_resync_period] = ACTIONS(1759),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1759),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1759),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1759),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1759),
    [sym_clkmgr_get_timeservers] = ACTIONS(1759),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1759),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1759),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1759),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1759),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1759),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1759),
    [sym_set_log_level] = ACTIONS(1759),
    [sym_get_log_level] = ACTIONS(1759),
    [sym_amx_log] = ACTIONS(1759),
    [sym_smtp_server_config_set] = ACTIONS(1759),
    [sym_smtp_server_config_get] = ACTIONS(1759),
    [sym_smtp_send] = ACTIONS(1759),
    [sym_validate_netlinx_account] = ACTIONS(1759),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1759),
    [sym_authenticate_certificate] = ACTIONS(1759),
    [sym_audit_netlinx_session_event] = ACTIONS(1759),
    [sym_audit_netlinx_generic_event] = ACTIONS(1759),
    [sym_get_available_flash_disk_space] = ACTIONS(1759),
    [sym_get_max_flash_disk_space] = ACTIONS(1759),
    [sym_data_create_feed] = ACTIONS(1759),
    [sym_data_delete_feed] = ACTIONS(1759),
    [sym_data_publish_feed] = ACTIONS(1759),
    [sym_data_get_published_feed] = ACTIONS(1759),
    [sym_data_add_record] = ACTIONS(1759),
    [sym_data_get_event_record] = ACTIONS(1759),
    [sym__wc_data_create_feed] = ACTIONS(1759),
    [sym__wc_data_add_record] = ACTIONS(1759),
    [sym__wc_data_get_event_record] = ACTIONS(1759),
    [sym_format] = ACTIONS(1759),
    [sym_astro_clock] = ACTIONS(1759),
    [sym_set_outdoor_temperature] = ACTIONS(1759),
    [sym_set_virtual_level_count] = ACTIONS(1759),
    [sym_set_virtual_channel_count] = ACTIONS(1759),
    [sym_set_virtual_port_count] = ACTIONS(1759),
    [sym__wc_encode_int] = ACTIONS(1759),
    [sym__wc_decode_int] = ACTIONS(1759),
    [sym__wc_int] = ACTIONS(1759),
    [sym__wc_to_ch_int] = ACTIONS(1759),
    [sym__ch_to_wc_int] = ACTIONS(1759),
    [sym__wc_find_string_int] = ACTIONS(1759),
    [sym__wc_left_string_int] = ACTIONS(1759),
    [sym__wc_lower_string_int] = ACTIONS(1759),
    [sym__wc_mid_string_int] = ACTIONS(1759),
    [sym__wc_remove_string_int] = ACTIONS(1759),
    [sym__wc_right_string_int] = ACTIONS(1759),
    [sym__wc_upper_string_int] = ACTIONS(1759),
    [sym__wc_compare_string_int] = ACTIONS(1759),
    [sym__wc_get_buffer_char_int] = ACTIONS(1759),
    [sym__wc_get_buffer_string_int] = ACTIONS(1759),
    [sym__wc_concat_string_int] = ACTIONS(1759),
    [sym__wc_file_open_int] = ACTIONS(1759),
    [sym__wc_file_close_int] = ACTIONS(1759),
    [sym__wc_file_read_int] = ACTIONS(1759),
    [sym__wc_file_read_line_int] = ACTIONS(1759),
    [sym__wc_file_write_int] = ACTIONS(1759),
    [sym__wc_file_write_line_int] = ACTIONS(1759),
    [aux_sym_preproc_include_token1] = ACTIONS(1759),
    [aux_sym_preproc_define_token1] = ACTIONS(1761),
    [aux_sym_preproc_warn_token1] = ACTIONS(1761),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1761),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1761),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1761),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1761),
    [aux_sym_preproc_else_token1] = ACTIONS(1761),
    [anon_sym_LBRACK] = ACTIONS(1761),
    [anon_sym_LBRACE] = ACTIONS(1761),
    [anon_sym_LPAREN] = ACTIONS(1759),
    [anon_sym_BANG] = ACTIONS(1761),
    [anon_sym_TILDE] = ACTIONS(1761),
    [anon_sym_DASH] = ACTIONS(1759),
    [anon_sym_PLUS] = ACTIONS(1759),
    [anon_sym_DASH_DASH] = ACTIONS(1761),
    [anon_sym_PLUS_PLUS] = ACTIONS(1761),
    [anon_sym_DQUOTE] = ACTIONS(1761),
    [anon_sym_SQUOTE] = ACTIONS(1761),
    [sym_number_literal] = ACTIONS(1761),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(220)] = {
    [sym_else_clause] = STATE(389),
    [ts_builtin_sym_end] = ACTIONS(1691),
    [sym_identifier] = ACTIONS(1687),
    [sym_program_name_keyword] = ACTIONS(1687),
    [sym_module_name_keyword] = ACTIONS(1687),
    [sym_define_device_keyword] = ACTIONS(1687),
    [sym_define_combine_keyword] = ACTIONS(1687),
    [sym_define_constant_keyword] = ACTIONS(1687),
    [sym_define_type_keyword] = ACTIONS(1687),
    [sym_define_variable_keyword] = ACTIONS(1687),
    [sym_define_system_variable_keyword] = ACTIONS(1687),
    [sym_define_connect_level_keyword] = ACTIONS(1687),
    [sym_define_latching_keyword] = ACTIONS(1687),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1687),
    [sym_define_toggling_keyword] = ACTIONS(1687),
    [sym_define_call_keyword] = ACTIONS(1687),
    [sym_define_function_keyword] = ACTIONS(1687),
    [sym_define_library_function_keyword] = ACTIONS(1687),
    [sym_define_start_keyword] = ACTIONS(1687),
    [sym_define_event_keyword] = ACTIONS(1687),
    [sym_define_program_keyword] = ACTIONS(1687),
    [sym_define_module_keyword] = ACTIONS(1687),
    [sym_char_keyword] = ACTIONS(1687),
    [sym_widechar_keyword] = ACTIONS(1687),
    [sym_integer_keyword] = ACTIONS(1687),
    [sym_sinteger_keyword] = ACTIONS(1687),
    [sym_long_keyword] = ACTIONS(1687),
    [sym_slong_keyword] = ACTIONS(1687),
    [sym_float_keyword] = ACTIONS(1687),
    [sym_double_keyword] = ACTIONS(1687),
    [sym_variant_keyword] = ACTIONS(1687),
    [sym_variantarray_keyword] = ACTIONS(1687),
    [sym_local_var_keyword] = ACTIONS(1687),
    [sym_stack_var_keyword] = ACTIONS(1687),
    [sym_constant_keyword] = ACTIONS(1687),
    [sym_volatile_keyword] = ACTIONS(1687),
    [sym_non_volatile_keyword] = ACTIONS(1687),
    [sym_persistent_keyword] = ACTIONS(1687),
    [sym_dev_keyword] = ACTIONS(1687),
    [sym_devlev_keyword] = ACTIONS(1687),
    [sym_devchan_keyword] = ACTIONS(1687),
    [sym_if_keyword] = ACTIONS(1687),
    [sym_else_keyword] = ACTIONS(1763),
    [sym_switch_keyword] = ACTIONS(1687),
    [sym_case_keyword] = ACTIONS(1687),
    [sym_default_keyword] = ACTIONS(1687),
    [sym_while_keyword] = ACTIONS(1687),
    [sym_for_keyword] = ACTIONS(1687),
    [sym_break_keyword] = ACTIONS(1687),
    [sym_continue_keyword] = ACTIONS(1687),
    [sym_return_keyword] = ACTIONS(1687),
    [sym_select_keyword] = ACTIONS(1687),
    [sym_active_keyword] = ACTIONS(1687),
    [sym_struct_keyword] = ACTIONS(1687),
    [sym_structure_keyword] = ACTIONS(1687),
    [sym_button_event_keyword] = ACTIONS(1687),
    [sym_channel_event_keyword] = ACTIONS(1687),
    [sym_level_event_keyword] = ACTIONS(1687),
    [sym_data_event_keyword] = ACTIONS(1687),
    [sym_timeline_event_keyword] = ACTIONS(1687),
    [sym_custom_event_keyword] = ACTIONS(1687),
    [sym_push_keyword] = ACTIONS(1687),
    [sym_release_keyword] = ACTIONS(1687),
    [aux_sym_on_keyword_token1] = ACTIONS(1687),
    [aux_sym_off_keyword_token1] = ACTIONS(1687),
    [sym_send_level_keyword] = ACTIONS(1687),
    [sym_send_string_keyword] = ACTIONS(1687),
    [sym_send_command_keyword] = ACTIONS(1687),
    [sym_clear_buffer_keyword] = ACTIONS(1687),
    [sym_create_buffer_keyword] = ACTIONS(1687),
    [sym_create_multi_buffer_keyword] = ACTIONS(1687),
    [sym_call_keyword] = ACTIONS(1687),
    [sym_system_call_keyword] = ACTIONS(1687),
    [sym_devchan_to_keyword] = ACTIONS(1687),
    [sym_devchan_min_to_keyword] = ACTIONS(1687),
    [sym_devchan_total_off_keyword] = ACTIONS(1687),
    [sym_devchan_pulse_keyword] = ACTIONS(1687),
    [sym_wait_keyword] = ACTIONS(1687),
    [sym_cancel_wait_keyword] = ACTIONS(1687),
    [sym_wait_until_keyword] = ACTIONS(1687),
    [sym_cancel_wait_until_keyword] = ACTIONS(1687),
    [sym_cancel_all_wait_keyword] = ACTIONS(1687),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1687),
    [sym_not] = ACTIONS(1687),
    [sym_bnot] = ACTIONS(1687),
    [sym___file__] = ACTIONS(1687),
    [sym___date__] = ACTIONS(1687),
    [sym___ldate__] = ACTIONS(1687),
    [sym___time__] = ACTIONS(1687),
    [sym___line__] = ACTIONS(1687),
    [sym___name__] = ACTIONS(1687),
    [sym___netlinx__] = ACTIONS(1687),
    [sym_netlinx_axi_version] = ACTIONS(1687),
    [sym_url_flg_tcp] = ACTIONS(1687),
    [sym_url_flg_accinfopresent] = ACTIONS(1687),
    [sym_url_flg_temp] = ACTIONS(1687),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1687),
    [sym_url_flg_stat_mask] = ACTIONS(1687),
    [sym_url_flg_stat_lookup] = ACTIONS(1687),
    [sym_url_flg_stat_connecting] = ACTIONS(1687),
    [sym_url_flg_stat_waiting] = ACTIONS(1687),
    [sym_url_flg_stat_connected] = ACTIONS(1687),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1687),
    [sym_first_virtual_device] = ACTIONS(1687),
    [sym_first_local_port] = ACTIONS(1687),
    [sym_dynamic_virtual_device] = ACTIONS(1687),
    [sym_timeline_once] = ACTIONS(1687),
    [sym_timeline_repeat] = ACTIONS(1687),
    [sym_timeline_absolute] = ACTIONS(1687),
    [sym_timeline_relative] = ACTIONS(1687),
    [sym_do_push_timed_infinite] = ACTIONS(1687),
    [sym_source_type_no_address] = ACTIONS(1687),
    [sym_source_type_neuron_id] = ACTIONS(1687),
    [sym_source_type_ip_address] = ACTIONS(1687),
    [sym_source_type_axlink] = ACTIONS(1687),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1687),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1687),
    [sym_source_type_ip_socket_address] = ACTIONS(1687),
    [sym_source_type_rs232] = ACTIONS(1687),
    [sym_source_type_internal] = ACTIONS(1687),
    [sym_source_type_mac_address] = ACTIONS(1687),
    [sym_source_type_ipv4_port] = ACTIONS(1687),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1687),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1687),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1687),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1687),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1687),
    [sym_true] = ACTIONS(1687),
    [sym_false] = ACTIONS(1687),
    [sym_file_read_only] = ACTIONS(1687),
    [sym_file_rw_new] = ACTIONS(1687),
    [sym_file_rw_append] = ACTIONS(1687),
    [sym_ip_tcp] = ACTIONS(1687),
    [sym_ip_udp] = ACTIONS(1687),
    [sym_ip_udp_2way] = ACTIONS(1687),
    [sym_xml_encode_types] = ACTIONS(1687),
    [sym_xml_encode_char_as_list] = ACTIONS(1687),
    [sym_xml_encode_le] = ACTIONS(1687),
    [sym_xml_decode_types] = ACTIONS(1687),
    [sym_xml_decode_no_preserve] = ACTIONS(1687),
    [sym_duet_dev_type_display_device] = ACTIONS(1687),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1687),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1687),
    [sym_duet_dev_type_io_device] = ACTIONS(1687),
    [sym_duet_dev_type_relay_device] = ACTIONS(1687),
    [sym_duet_dev_type_ups] = ACTIONS(1687),
    [sym_duet_dev_type_amplifier] = ACTIONS(1687),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1687),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1687),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1687),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1687),
    [sym_duet_dev_type_camera] = ACTIONS(1687),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1687),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1687),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1687),
    [sym_duet_dev_type_dss] = ACTIONS(1687),
    [sym_duet_dev_type_dvr] = ACTIONS(1687),
    [sym_duet_dev_type_disc_device] = ACTIONS(1687),
    [sym_duet_dev_type_document_camera] = ACTIONS(1687),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1687),
    [sym_duet_dev_type_hvac] = ACTIONS(1687),
    [sym_duet_dev_type_keypad] = ACTIONS(1687),
    [sym_duet_dev_type_light] = ACTIONS(1687),
    [sym_duet_dev_type_monitor] = ACTIONS(1687),
    [sym_duet_dev_type_motor] = ACTIONS(1687),
    [sym_duet_dev_type_multi_window] = ACTIONS(1687),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1687),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1687),
    [sym_duet_dev_type_receiver] = ACTIONS(1687),
    [sym_duet_dev_type_security_system] = ACTIONS(1687),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1687),
    [sym_duet_dev_type_settop_box] = ACTIONS(1687),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1687),
    [sym_duet_dev_type_switcher] = ACTIONS(1687),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1687),
    [sym_duet_dev_type_tv] = ACTIONS(1687),
    [sym_duet_dev_type_utility] = ACTIONS(1687),
    [sym_duet_dev_type_vcr] = ACTIONS(1687),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1687),
    [sym_duet_dev_type_video_processor] = ACTIONS(1687),
    [sym_duet_dev_type_video_projector] = ACTIONS(1687),
    [sym_duet_dev_type_video_wall] = ACTIONS(1687),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1687),
    [sym_duet_dev_type_weather] = ACTIONS(1687),
    [sym_duet_dev_not_polled] = ACTIONS(1687),
    [sym_duet_dev_polled] = ACTIONS(1687),
    [sym_clkmgr_mode_network] = ACTIONS(1687),
    [sym_clkmgr_mode_standalone] = ACTIONS(1687),
    [sym_normal_standby] = ACTIONS(1687),
    [sym_normal_wake] = ACTIONS(1687),
    [sym_ip_multicast_ttl_option] = ACTIONS(1687),
    [sym_ip_ttl_subnet] = ACTIONS(1687),
    [sym_ip_ttl_site] = ACTIONS(1687),
    [sym_ip_ttl_region] = ACTIONS(1687),
    [sym_ip_ttl_continent] = ACTIONS(1687),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1687),
    [sym_ip_nodelay_on] = ACTIONS(1687),
    [sym_ip_nodelay_off] = ACTIONS(1687),
    [sym_amx_error] = ACTIONS(1687),
    [sym_amx_warning] = ACTIONS(1687),
    [sym_amx_info] = ACTIONS(1687),
    [sym_amx_debug] = ACTIONS(1687),
    [sym_netlinx_login_success] = ACTIONS(1687),
    [sym_netlinx_login_fail] = ACTIONS(1687),
    [sym_netlinx_logout] = ACTIONS(1687),
    [sym_null_str] = ACTIONS(1687),
    [sym_smtp_address] = ACTIONS(1687),
    [sym_smtp_port_number] = ACTIONS(1687),
    [sym_smtp_username] = ACTIONS(1687),
    [sym_smtp_password] = ACTIONS(1687),
    [sym_smtp_require_tls] = ACTIONS(1687),
    [sym_smtp_from] = ACTIONS(1687),
    [sym_smtp_tls_true] = ACTIONS(1687),
    [sym_smtp_tls_false] = ACTIONS(1687),
    [sym_listview_on_row_select_event] = ACTIONS(1687),
    [sym_data_structure_unknown] = ACTIONS(1687),
    [sym_data_structure_datafeed] = ACTIONS(1687),
    [sym_data_structure_datarecordset] = ACTIONS(1687),
    [sym_data_structure_datarecord] = ACTIONS(1687),
    [sym_data_structure_datafield] = ACTIONS(1687),
    [sym_data_max_content_field_count] = ACTIONS(1687),
    [sym_data_max_metadata_field_count] = ACTIONS(1687),
    [sym_data_max_id_length] = ACTIONS(1687),
    [sym_data_max_name_length] = ACTIONS(1687),
    [sym_data_max_description_length] = ACTIONS(1687),
    [sym_data_max_type_length] = ACTIONS(1687),
    [sym_data_max_format_length] = ACTIONS(1687),
    [sym_data_max_source_length] = ACTIONS(1687),
    [sym_data_max_value_length] = ACTIONS(1687),
    [sym_data_max_label_length] = ACTIONS(1687),
    [sym_data_max_url_length] = ACTIONS(1687),
    [sym_max_last_login_info_length] = ACTIONS(1687),
    [sym_data_type_unknown] = ACTIONS(1687),
    [sym_data_type_string] = ACTIONS(1687),
    [sym_data_type_datetime] = ACTIONS(1687),
    [sym_data_type_date] = ACTIONS(1687),
    [sym_data_type_time] = ACTIONS(1687),
    [sym_data_type_image] = ACTIONS(1687),
    [sym_data_format_url] = ACTIONS(1687),
    [sym_data_format_phone] = ACTIONS(1687),
    [sym_data_format_email] = ACTIONS(1687),
    [sym_data_format_iso8601] = ACTIONS(1687),
    [sym_auditlog_priv] = ACTIONS(1687),
    [sym_configuration_priv] = ACTIONS(1687),
    [sym_deviceconfig_priv] = ACTIONS(1687),
    [sym_ftp_priv] = ACTIONS(1687),
    [sym_http_priv] = ACTIONS(1687),
    [sym_networkconfig_priv] = ACTIONS(1687),
    [sym_programport_priv] = ACTIONS(1687),
    [sym_remoteui_priv] = ACTIONS(1687),
    [sym_securitycontrol_priv] = ACTIONS(1687),
    [sym_softwaremanagement_priv] = ACTIONS(1687),
    [sym_terminal_priv] = ACTIONS(1687),
    [sym_tpadmin_priv] = ACTIONS(1687),
    [sym_user1_priv] = ACTIONS(1687),
    [sym_user2_priv] = ACTIONS(1687),
    [sym_user3_priv] = ACTIONS(1687),
    [sym_user4_priv] = ACTIONS(1687),
    [sym_usermanagement_priv] = ACTIONS(1687),
    [sym_valid_account] = ACTIONS(1687),
    [sym_err_name_invalid] = ACTIONS(1687),
    [sym_err_password_invalid] = ACTIONS(1687),
    [sym_err_invalid_account] = ACTIONS(1687),
    [sym_err_no_authorization] = ACTIONS(1687),
    [sym_err_invalid_parameter] = ACTIONS(1687),
    [sym_err_user_locked_out] = ACTIONS(1687),
    [sym_err_user_account_expired] = ACTIONS(1687),
    [sym_err_general_error] = ACTIONS(1687),
    [sym_err_authentication_error] = ACTIONS(1687),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1687),
    [sym_tls_validate_certificate] = ACTIONS(1687),
    [sym_tdata] = ACTIONS(1687),
    [sym_tchannel] = ACTIONS(1687),
    [sym_tlevel] = ACTIONS(1687),
    [sym_tbutton] = ACTIONS(1687),
    [sym_ttimeline] = ACTIONS(1687),
    [sym_tcustom] = ACTIONS(1687),
    [sym_url_struct] = ACTIONS(1687),
    [sym_dns_struct] = ACTIONS(1687),
    [sym_ip_address_struct] = ACTIONS(1687),
    [sym_dev_info_struct] = ACTIONS(1687),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1687),
    [sym_data_feed] = ACTIONS(1687),
    [sym_data_field] = ACTIONS(1687),
    [sym_data_record] = ACTIONS(1687),
    [sym_wc_data_feed] = ACTIONS(1687),
    [sym_wc_data_field] = ACTIONS(1687),
    [sym_wc_data_record] = ACTIONS(1687),
    [sym_last_login_info] = ACTIONS(1687),
    [sym____reserved___] = ACTIONS(1687),
    [sym_date] = ACTIONS(1687),
    [sym_day] = ACTIONS(1687),
    [sym_time] = ACTIONS(1687),
    [sym_ldate] = ACTIONS(1687),
    [sym_push_device] = ACTIONS(1687),
    [sym_push_channel] = ACTIONS(1687),
    [sym_push_devchan] = ACTIONS(1687),
    [sym_release_device] = ACTIONS(1687),
    [sym_release_channel] = ACTIONS(1687),
    [sym_release_devchan] = ACTIONS(1687),
    [sym_master_slot] = ACTIONS(1687),
    [sym_get_pulse_time] = ACTIONS(1687),
    [sym_get_timer] = ACTIONS(1687),
    [sym_master_sn] = ACTIONS(1687),
    [sym_system_number] = ACTIONS(1687),
    [sym_button] = ACTIONS(1687),
    [sym_dv_channel] = ACTIONS(1687),
    [sym_data] = ACTIONS(1687),
    [sym_level] = ACTIONS(1687),
    [sym_timeline] = ACTIONS(1687),
    [sym_channel] = ACTIONS(1687),
    [sym_custom] = ACTIONS(1687),
    [sym_atoi] = ACTIONS(1687),
    [sym_atol] = ACTIONS(1687),
    [sym_atof] = ACTIONS(1687),
    [sym_itoa] = ACTIONS(1687),
    [sym_itohex] = ACTIONS(1687),
    [sym_hextoi] = ACTIONS(1687),
    [sym_ftoa] = ACTIONS(1687),
    [sym_find_string] = ACTIONS(1687),
    [sym_left_string] = ACTIONS(1687),
    [sym_right_string] = ACTIONS(1687),
    [sym_mid_string] = ACTIONS(1687),
    [sym_remove_string] = ACTIONS(1687),
    [sym_get_buffer_string] = ACTIONS(1687),
    [sym_get_buffer_char] = ACTIONS(1687),
    [sym_get_multi_buffer_string] = ACTIONS(1687),
    [sym_length_string] = ACTIONS(1687),
    [sym_max_length_string] = ACTIONS(1687),
    [sym_set_length_string] = ACTIONS(1687),
    [sym_compare_string] = ACTIONS(1687),
    [sym_upper_string] = ACTIONS(1687),
    [sym_lower_string] = ACTIONS(1687),
    [sym_get_last] = ACTIONS(1687),
    [sym_date_to_day] = ACTIONS(1687),
    [sym_date_to_month] = ACTIONS(1687),
    [sym_date_to_year] = ACTIONS(1687),
    [sym_time_to_hour] = ACTIONS(1687),
    [sym_time_to_minute] = ACTIONS(1687),
    [sym_time_to_second] = ACTIONS(1687),
    [sym_day_of_week] = ACTIONS(1687),
    [sym_set_pulse_time] = ACTIONS(1687),
    [sym_set_timer] = ACTIONS(1687),
    [sym_file_open] = ACTIONS(1687),
    [sym_file_close] = ACTIONS(1687),
    [sym_file_read] = ACTIONS(1687),
    [sym_file_read_line] = ACTIONS(1687),
    [sym_file_write] = ACTIONS(1687),
    [sym_file_write_line] = ACTIONS(1687),
    [sym_file_copy] = ACTIONS(1687),
    [sym_file_delete] = ACTIONS(1687),
    [sym_file_rename] = ACTIONS(1687),
    [sym_file_dir] = ACTIONS(1687),
    [sym_file_getdir] = ACTIONS(1687),
    [sym_file_setdir] = ACTIONS(1687),
    [sym_file_create_dir] = ACTIONS(1687),
    [sym_file_remove_dir] = ACTIONS(1687),
    [sym_file_seek] = ACTIONS(1687),
    [sym_length_array] = ACTIONS(1687),
    [sym_max_length_array] = ACTIONS(1687),
    [sym_set_length_array] = ACTIONS(1687),
    [sym_ip_client_open] = ACTIONS(1687),
    [sym_ip_client_close] = ACTIONS(1687),
    [sym_ip_server_open] = ACTIONS(1687),
    [sym_ip_server_close] = ACTIONS(1687),
    [sym_ip_bound_client_open] = ACTIONS(1687),
    [sym_ip_mc_server_open] = ACTIONS(1687),
    [sym_ip_set_option] = ACTIONS(1687),
    [sym_ssh_client_open] = ACTIONS(1687),
    [sym_ssh_client_close] = ACTIONS(1687),
    [sym_tls_client_open] = ACTIONS(1687),
    [sym_tls_client_close] = ACTIONS(1687),
    [sym_do_push] = ACTIONS(1687),
    [sym_do_push_timed] = ACTIONS(1687),
    [sym_do_release] = ACTIONS(1687),
    [sym_device_id] = ACTIONS(1687),
    [sym_device_info] = ACTIONS(1687),
    [sym_device_id_string] = ACTIONS(1687),
    [sym_device_standby] = ACTIONS(1687),
    [sym_device_wake] = ACTIONS(1687),
    [sym_timeline_create] = ACTIONS(1687),
    [sym_timeline_kill] = ACTIONS(1687),
    [sym_timeline_reload] = ACTIONS(1687),
    [sym_timeline_pause] = ACTIONS(1687),
    [sym_timeline_restart] = ACTIONS(1687),
    [sym_timeline_set] = ACTIONS(1687),
    [sym_timeline_get] = ACTIONS(1687),
    [sym_timeline_active] = ACTIONS(1687),
    [sym_timeline_dynamic_id] = ACTIONS(1687),
    [sym_variable_to_string] = ACTIONS(1687),
    [sym_string_to_variable] = ACTIONS(1687),
    [sym_length_variable_to_string] = ACTIONS(1687),
    [sym_type_cast] = ACTIONS(1687),
    [sym_variable_to_xml] = ACTIONS(1687),
    [sym_xml_to_variable] = ACTIONS(1687),
    [sym_length_variable_to_xml] = ACTIONS(1687),
    [sym_redirect_string] = ACTIONS(1687),
    [sym_random_number] = ACTIONS(1687),
    [sym_get_unique_id] = ACTIONS(1687),
    [sym_get_system_number] = ACTIONS(1687),
    [sym_set_system_number] = ACTIONS(1687),
    [sym_get_serial_number] = ACTIONS(1687),
    [sym_get_master_build] = ACTIONS(1687),
    [sym_set_validation_code] = ACTIONS(1687),
    [sym_reboot] = ACTIONS(1687),
    [sym_load_duet_module] = ACTIONS(1687),
    [sym_unload_duet_module] = ACTIONS(1687),
    [sym_duet_mem_size_set] = ACTIONS(1687),
    [sym_duet_mem_size_get] = ACTIONS(1687),
    [sym_dynamic_application_device] = ACTIONS(1687),
    [sym_dynamic_polled_port] = ACTIONS(1687),
    [sym_static_port_binding] = ACTIONS(1687),
    [sym_static_ip_binding] = ACTIONS(1687),
    [sym_get_dns_list] = ACTIONS(1687),
    [sym_set_dns_list] = ACTIONS(1687),
    [sym_get_ip_address] = ACTIONS(1687),
    [sym_set_ip_address] = ACTIONS(1687),
    [sym_get_url_list] = ACTIONS(1687),
    [sym_add_url_entry] = ACTIONS(1687),
    [sym_delete_url_entry] = ACTIONS(1687),
    [sym_abs_value] = ACTIONS(1687),
    [sym_max_value] = ACTIONS(1687),
    [sym_min_value] = ACTIONS(1687),
    [sym_exp_value] = ACTIONS(1687),
    [sym_log_value] = ACTIONS(1687),
    [sym_log10_value] = ACTIONS(1687),
    [sym_power_value] = ACTIONS(1687),
    [sym_sqrt_value] = ACTIONS(1687),
    [sym_raw_be] = ACTIONS(1687),
    [sym_raw_le] = ACTIONS(1687),
    [sym_internal_queue_size_set] = ACTIONS(1687),
    [sym_internal_queue_size_get] = ACTIONS(1687),
    [sym_internal_threshold_set] = ACTIONS(1687),
    [sym_internal_threshold_get] = ACTIONS(1687),
    [sym_rebuilt_event] = ACTIONS(1687),
    [sym_do_custom_event] = ACTIONS(1687),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1687),
    [sym_clkmgr_set_clk_source] = ACTIONS(1687),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1687),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1687),
    [sym_clkmgr_get_timezone] = ACTIONS(1687),
    [sym_clkmgr_set_timezone] = ACTIONS(1687),
    [sym_clkmgr_get_resync_period] = ACTIONS(1687),
    [sym_clkmgr_set_resync_period] = ACTIONS(1687),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1687),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1687),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1687),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1687),
    [sym_clkmgr_get_timeservers] = ACTIONS(1687),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1687),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1687),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1687),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1687),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1687),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1687),
    [sym_set_log_level] = ACTIONS(1687),
    [sym_get_log_level] = ACTIONS(1687),
    [sym_amx_log] = ACTIONS(1687),
    [sym_smtp_server_config_set] = ACTIONS(1687),
    [sym_smtp_server_config_get] = ACTIONS(1687),
    [sym_smtp_send] = ACTIONS(1687),
    [sym_validate_netlinx_account] = ACTIONS(1687),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1687),
    [sym_authenticate_certificate] = ACTIONS(1687),
    [sym_audit_netlinx_session_event] = ACTIONS(1687),
    [sym_audit_netlinx_generic_event] = ACTIONS(1687),
    [sym_get_available_flash_disk_space] = ACTIONS(1687),
    [sym_get_max_flash_disk_space] = ACTIONS(1687),
    [sym_data_create_feed] = ACTIONS(1687),
    [sym_data_delete_feed] = ACTIONS(1687),
    [sym_data_publish_feed] = ACTIONS(1687),
    [sym_data_get_published_feed] = ACTIONS(1687),
    [sym_data_add_record] = ACTIONS(1687),
    [sym_data_get_event_record] = ACTIONS(1687),
    [sym__wc_data_create_feed] = ACTIONS(1687),
    [sym__wc_data_add_record] = ACTIONS(1687),
    [sym__wc_data_get_event_record] = ACTIONS(1687),
    [sym_format] = ACTIONS(1687),
    [sym_astro_clock] = ACTIONS(1687),
    [sym_set_outdoor_temperature] = ACTIONS(1687),
    [sym_set_virtual_level_count] = ACTIONS(1687),
    [sym_set_virtual_channel_count] = ACTIONS(1687),
    [sym_set_virtual_port_count] = ACTIONS(1687),
    [sym__wc_encode_int] = ACTIONS(1687),
    [sym__wc_decode_int] = ACTIONS(1687),
    [sym__wc_int] = ACTIONS(1687),
    [sym__wc_to_ch_int] = ACTIONS(1687),
    [sym__ch_to_wc_int] = ACTIONS(1687),
    [sym__wc_find_string_int] = ACTIONS(1687),
    [sym__wc_left_string_int] = ACTIONS(1687),
    [sym__wc_lower_string_int] = ACTIONS(1687),
    [sym__wc_mid_string_int] = ACTIONS(1687),
    [sym__wc_remove_string_int] = ACTIONS(1687),
    [sym__wc_right_string_int] = ACTIONS(1687),
    [sym__wc_upper_string_int] = ACTIONS(1687),
    [sym__wc_compare_string_int] = ACTIONS(1687),
    [sym__wc_get_buffer_char_int] = ACTIONS(1687),
    [sym__wc_get_buffer_string_int] = ACTIONS(1687),
    [sym__wc_concat_string_int] = ACTIONS(1687),
    [sym__wc_file_open_int] = ACTIONS(1687),
    [sym__wc_file_close_int] = ACTIONS(1687),
    [sym__wc_file_read_int] = ACTIONS(1687),
    [sym__wc_file_read_line_int] = ACTIONS(1687),
    [sym__wc_file_write_int] = ACTIONS(1687),
    [sym__wc_file_write_line_int] = ACTIONS(1687),
    [aux_sym_preproc_include_token1] = ACTIONS(1687),
    [aux_sym_preproc_define_token1] = ACTIONS(1691),
    [aux_sym_preproc_warn_token1] = ACTIONS(1691),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1691),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1691),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1691),
    [anon_sym_LBRACK] = ACTIONS(1691),
    [anon_sym_LBRACE] = ACTIONS(1691),
    [anon_sym_LPAREN] = ACTIONS(1687),
    [anon_sym_BANG] = ACTIONS(1691),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1687),
    [anon_sym_PLUS] = ACTIONS(1687),
    [anon_sym_DASH_DASH] = ACTIONS(1691),
    [anon_sym_PLUS_PLUS] = ACTIONS(1691),
    [anon_sym_DQUOTE] = ACTIONS(1691),
    [anon_sym_SQUOTE] = ACTIONS(1691),
    [sym_number_literal] = ACTIONS(1691),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(221)] = {
    [sym_identifier] = ACTIONS(1765),
    [sym_program_name_keyword] = ACTIONS(1765),
    [sym_module_name_keyword] = ACTIONS(1765),
    [sym_define_device_keyword] = ACTIONS(1765),
    [sym_define_combine_keyword] = ACTIONS(1765),
    [sym_define_constant_keyword] = ACTIONS(1765),
    [sym_define_type_keyword] = ACTIONS(1765),
    [sym_define_variable_keyword] = ACTIONS(1765),
    [sym_define_system_variable_keyword] = ACTIONS(1765),
    [sym_define_connect_level_keyword] = ACTIONS(1765),
    [sym_define_latching_keyword] = ACTIONS(1765),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1765),
    [sym_define_toggling_keyword] = ACTIONS(1765),
    [sym_define_call_keyword] = ACTIONS(1765),
    [sym_define_function_keyword] = ACTIONS(1765),
    [sym_define_library_function_keyword] = ACTIONS(1765),
    [sym_define_start_keyword] = ACTIONS(1765),
    [sym_define_event_keyword] = ACTIONS(1765),
    [sym_define_program_keyword] = ACTIONS(1765),
    [sym_define_module_keyword] = ACTIONS(1765),
    [sym_char_keyword] = ACTIONS(1765),
    [sym_widechar_keyword] = ACTIONS(1765),
    [sym_integer_keyword] = ACTIONS(1765),
    [sym_sinteger_keyword] = ACTIONS(1765),
    [sym_long_keyword] = ACTIONS(1765),
    [sym_slong_keyword] = ACTIONS(1765),
    [sym_float_keyword] = ACTIONS(1765),
    [sym_double_keyword] = ACTIONS(1765),
    [sym_variant_keyword] = ACTIONS(1765),
    [sym_variantarray_keyword] = ACTIONS(1765),
    [sym_local_var_keyword] = ACTIONS(1765),
    [sym_stack_var_keyword] = ACTIONS(1765),
    [sym_constant_keyword] = ACTIONS(1765),
    [sym_volatile_keyword] = ACTIONS(1765),
    [sym_non_volatile_keyword] = ACTIONS(1765),
    [sym_persistent_keyword] = ACTIONS(1765),
    [sym_dev_keyword] = ACTIONS(1765),
    [sym_devlev_keyword] = ACTIONS(1765),
    [sym_devchan_keyword] = ACTIONS(1765),
    [sym_if_keyword] = ACTIONS(1765),
    [sym_else_keyword] = ACTIONS(1765),
    [sym_switch_keyword] = ACTIONS(1765),
    [sym_case_keyword] = ACTIONS(1765),
    [sym_default_keyword] = ACTIONS(1765),
    [sym_while_keyword] = ACTIONS(1765),
    [sym_for_keyword] = ACTIONS(1765),
    [sym_break_keyword] = ACTIONS(1765),
    [sym_continue_keyword] = ACTIONS(1765),
    [sym_return_keyword] = ACTIONS(1765),
    [sym_select_keyword] = ACTIONS(1765),
    [sym_active_keyword] = ACTIONS(1765),
    [sym_struct_keyword] = ACTIONS(1765),
    [sym_structure_keyword] = ACTIONS(1765),
    [sym_button_event_keyword] = ACTIONS(1765),
    [sym_channel_event_keyword] = ACTIONS(1765),
    [sym_level_event_keyword] = ACTIONS(1765),
    [sym_data_event_keyword] = ACTIONS(1765),
    [sym_timeline_event_keyword] = ACTIONS(1765),
    [sym_custom_event_keyword] = ACTIONS(1765),
    [sym_push_keyword] = ACTIONS(1765),
    [sym_release_keyword] = ACTIONS(1765),
    [aux_sym_on_keyword_token1] = ACTIONS(1765),
    [aux_sym_off_keyword_token1] = ACTIONS(1765),
    [sym_send_level_keyword] = ACTIONS(1765),
    [sym_send_string_keyword] = ACTIONS(1765),
    [sym_send_command_keyword] = ACTIONS(1765),
    [sym_clear_buffer_keyword] = ACTIONS(1765),
    [sym_create_buffer_keyword] = ACTIONS(1765),
    [sym_create_multi_buffer_keyword] = ACTIONS(1765),
    [sym_call_keyword] = ACTIONS(1765),
    [sym_system_call_keyword] = ACTIONS(1765),
    [sym_devchan_to_keyword] = ACTIONS(1765),
    [sym_devchan_min_to_keyword] = ACTIONS(1765),
    [sym_devchan_total_off_keyword] = ACTIONS(1765),
    [sym_devchan_pulse_keyword] = ACTIONS(1765),
    [sym_wait_keyword] = ACTIONS(1765),
    [sym_cancel_wait_keyword] = ACTIONS(1765),
    [sym_wait_until_keyword] = ACTIONS(1765),
    [sym_cancel_wait_until_keyword] = ACTIONS(1765),
    [sym_cancel_all_wait_keyword] = ACTIONS(1765),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1765),
    [sym_not] = ACTIONS(1765),
    [sym_bnot] = ACTIONS(1765),
    [sym___file__] = ACTIONS(1765),
    [sym___date__] = ACTIONS(1765),
    [sym___ldate__] = ACTIONS(1765),
    [sym___time__] = ACTIONS(1765),
    [sym___line__] = ACTIONS(1765),
    [sym___name__] = ACTIONS(1765),
    [sym___netlinx__] = ACTIONS(1765),
    [sym_netlinx_axi_version] = ACTIONS(1765),
    [sym_url_flg_tcp] = ACTIONS(1765),
    [sym_url_flg_accinfopresent] = ACTIONS(1765),
    [sym_url_flg_temp] = ACTIONS(1765),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1765),
    [sym_url_flg_stat_mask] = ACTIONS(1765),
    [sym_url_flg_stat_lookup] = ACTIONS(1765),
    [sym_url_flg_stat_connecting] = ACTIONS(1765),
    [sym_url_flg_stat_waiting] = ACTIONS(1765),
    [sym_url_flg_stat_connected] = ACTIONS(1765),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1765),
    [sym_first_virtual_device] = ACTIONS(1765),
    [sym_first_local_port] = ACTIONS(1765),
    [sym_dynamic_virtual_device] = ACTIONS(1765),
    [sym_timeline_once] = ACTIONS(1765),
    [sym_timeline_repeat] = ACTIONS(1765),
    [sym_timeline_absolute] = ACTIONS(1765),
    [sym_timeline_relative] = ACTIONS(1765),
    [sym_do_push_timed_infinite] = ACTIONS(1765),
    [sym_source_type_no_address] = ACTIONS(1765),
    [sym_source_type_neuron_id] = ACTIONS(1765),
    [sym_source_type_ip_address] = ACTIONS(1765),
    [sym_source_type_axlink] = ACTIONS(1765),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1765),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1765),
    [sym_source_type_ip_socket_address] = ACTIONS(1765),
    [sym_source_type_rs232] = ACTIONS(1765),
    [sym_source_type_internal] = ACTIONS(1765),
    [sym_source_type_mac_address] = ACTIONS(1765),
    [sym_source_type_ipv4_port] = ACTIONS(1765),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1765),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1765),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1765),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1765),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1765),
    [sym_true] = ACTIONS(1765),
    [sym_false] = ACTIONS(1765),
    [sym_file_read_only] = ACTIONS(1765),
    [sym_file_rw_new] = ACTIONS(1765),
    [sym_file_rw_append] = ACTIONS(1765),
    [sym_ip_tcp] = ACTIONS(1765),
    [sym_ip_udp] = ACTIONS(1765),
    [sym_ip_udp_2way] = ACTIONS(1765),
    [sym_xml_encode_types] = ACTIONS(1765),
    [sym_xml_encode_char_as_list] = ACTIONS(1765),
    [sym_xml_encode_le] = ACTIONS(1765),
    [sym_xml_decode_types] = ACTIONS(1765),
    [sym_xml_decode_no_preserve] = ACTIONS(1765),
    [sym_duet_dev_type_display_device] = ACTIONS(1765),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1765),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1765),
    [sym_duet_dev_type_io_device] = ACTIONS(1765),
    [sym_duet_dev_type_relay_device] = ACTIONS(1765),
    [sym_duet_dev_type_ups] = ACTIONS(1765),
    [sym_duet_dev_type_amplifier] = ACTIONS(1765),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1765),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1765),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1765),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1765),
    [sym_duet_dev_type_camera] = ACTIONS(1765),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1765),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1765),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1765),
    [sym_duet_dev_type_dss] = ACTIONS(1765),
    [sym_duet_dev_type_dvr] = ACTIONS(1765),
    [sym_duet_dev_type_disc_device] = ACTIONS(1765),
    [sym_duet_dev_type_document_camera] = ACTIONS(1765),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1765),
    [sym_duet_dev_type_hvac] = ACTIONS(1765),
    [sym_duet_dev_type_keypad] = ACTIONS(1765),
    [sym_duet_dev_type_light] = ACTIONS(1765),
    [sym_duet_dev_type_monitor] = ACTIONS(1765),
    [sym_duet_dev_type_motor] = ACTIONS(1765),
    [sym_duet_dev_type_multi_window] = ACTIONS(1765),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1765),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1765),
    [sym_duet_dev_type_receiver] = ACTIONS(1765),
    [sym_duet_dev_type_security_system] = ACTIONS(1765),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1765),
    [sym_duet_dev_type_settop_box] = ACTIONS(1765),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1765),
    [sym_duet_dev_type_switcher] = ACTIONS(1765),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1765),
    [sym_duet_dev_type_tv] = ACTIONS(1765),
    [sym_duet_dev_type_utility] = ACTIONS(1765),
    [sym_duet_dev_type_vcr] = ACTIONS(1765),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1765),
    [sym_duet_dev_type_video_processor] = ACTIONS(1765),
    [sym_duet_dev_type_video_projector] = ACTIONS(1765),
    [sym_duet_dev_type_video_wall] = ACTIONS(1765),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1765),
    [sym_duet_dev_type_weather] = ACTIONS(1765),
    [sym_duet_dev_not_polled] = ACTIONS(1765),
    [sym_duet_dev_polled] = ACTIONS(1765),
    [sym_clkmgr_mode_network] = ACTIONS(1765),
    [sym_clkmgr_mode_standalone] = ACTIONS(1765),
    [sym_normal_standby] = ACTIONS(1765),
    [sym_normal_wake] = ACTIONS(1765),
    [sym_ip_multicast_ttl_option] = ACTIONS(1765),
    [sym_ip_ttl_subnet] = ACTIONS(1765),
    [sym_ip_ttl_site] = ACTIONS(1765),
    [sym_ip_ttl_region] = ACTIONS(1765),
    [sym_ip_ttl_continent] = ACTIONS(1765),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1765),
    [sym_ip_nodelay_on] = ACTIONS(1765),
    [sym_ip_nodelay_off] = ACTIONS(1765),
    [sym_amx_error] = ACTIONS(1765),
    [sym_amx_warning] = ACTIONS(1765),
    [sym_amx_info] = ACTIONS(1765),
    [sym_amx_debug] = ACTIONS(1765),
    [sym_netlinx_login_success] = ACTIONS(1765),
    [sym_netlinx_login_fail] = ACTIONS(1765),
    [sym_netlinx_logout] = ACTIONS(1765),
    [sym_null_str] = ACTIONS(1765),
    [sym_smtp_address] = ACTIONS(1765),
    [sym_smtp_port_number] = ACTIONS(1765),
    [sym_smtp_username] = ACTIONS(1765),
    [sym_smtp_password] = ACTIONS(1765),
    [sym_smtp_require_tls] = ACTIONS(1765),
    [sym_smtp_from] = ACTIONS(1765),
    [sym_smtp_tls_true] = ACTIONS(1765),
    [sym_smtp_tls_false] = ACTIONS(1765),
    [sym_listview_on_row_select_event] = ACTIONS(1765),
    [sym_data_structure_unknown] = ACTIONS(1765),
    [sym_data_structure_datafeed] = ACTIONS(1765),
    [sym_data_structure_datarecordset] = ACTIONS(1765),
    [sym_data_structure_datarecord] = ACTIONS(1765),
    [sym_data_structure_datafield] = ACTIONS(1765),
    [sym_data_max_content_field_count] = ACTIONS(1765),
    [sym_data_max_metadata_field_count] = ACTIONS(1765),
    [sym_data_max_id_length] = ACTIONS(1765),
    [sym_data_max_name_length] = ACTIONS(1765),
    [sym_data_max_description_length] = ACTIONS(1765),
    [sym_data_max_type_length] = ACTIONS(1765),
    [sym_data_max_format_length] = ACTIONS(1765),
    [sym_data_max_source_length] = ACTIONS(1765),
    [sym_data_max_value_length] = ACTIONS(1765),
    [sym_data_max_label_length] = ACTIONS(1765),
    [sym_data_max_url_length] = ACTIONS(1765),
    [sym_max_last_login_info_length] = ACTIONS(1765),
    [sym_data_type_unknown] = ACTIONS(1765),
    [sym_data_type_string] = ACTIONS(1765),
    [sym_data_type_datetime] = ACTIONS(1765),
    [sym_data_type_date] = ACTIONS(1765),
    [sym_data_type_time] = ACTIONS(1765),
    [sym_data_type_image] = ACTIONS(1765),
    [sym_data_format_url] = ACTIONS(1765),
    [sym_data_format_phone] = ACTIONS(1765),
    [sym_data_format_email] = ACTIONS(1765),
    [sym_data_format_iso8601] = ACTIONS(1765),
    [sym_auditlog_priv] = ACTIONS(1765),
    [sym_configuration_priv] = ACTIONS(1765),
    [sym_deviceconfig_priv] = ACTIONS(1765),
    [sym_ftp_priv] = ACTIONS(1765),
    [sym_http_priv] = ACTIONS(1765),
    [sym_networkconfig_priv] = ACTIONS(1765),
    [sym_programport_priv] = ACTIONS(1765),
    [sym_remoteui_priv] = ACTIONS(1765),
    [sym_securitycontrol_priv] = ACTIONS(1765),
    [sym_softwaremanagement_priv] = ACTIONS(1765),
    [sym_terminal_priv] = ACTIONS(1765),
    [sym_tpadmin_priv] = ACTIONS(1765),
    [sym_user1_priv] = ACTIONS(1765),
    [sym_user2_priv] = ACTIONS(1765),
    [sym_user3_priv] = ACTIONS(1765),
    [sym_user4_priv] = ACTIONS(1765),
    [sym_usermanagement_priv] = ACTIONS(1765),
    [sym_valid_account] = ACTIONS(1765),
    [sym_err_name_invalid] = ACTIONS(1765),
    [sym_err_password_invalid] = ACTIONS(1765),
    [sym_err_invalid_account] = ACTIONS(1765),
    [sym_err_no_authorization] = ACTIONS(1765),
    [sym_err_invalid_parameter] = ACTIONS(1765),
    [sym_err_user_locked_out] = ACTIONS(1765),
    [sym_err_user_account_expired] = ACTIONS(1765),
    [sym_err_general_error] = ACTIONS(1765),
    [sym_err_authentication_error] = ACTIONS(1765),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1765),
    [sym_tls_validate_certificate] = ACTIONS(1765),
    [sym_tdata] = ACTIONS(1765),
    [sym_tchannel] = ACTIONS(1765),
    [sym_tlevel] = ACTIONS(1765),
    [sym_tbutton] = ACTIONS(1765),
    [sym_ttimeline] = ACTIONS(1765),
    [sym_tcustom] = ACTIONS(1765),
    [sym_url_struct] = ACTIONS(1765),
    [sym_dns_struct] = ACTIONS(1765),
    [sym_ip_address_struct] = ACTIONS(1765),
    [sym_dev_info_struct] = ACTIONS(1765),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1765),
    [sym_data_feed] = ACTIONS(1765),
    [sym_data_field] = ACTIONS(1765),
    [sym_data_record] = ACTIONS(1765),
    [sym_wc_data_feed] = ACTIONS(1765),
    [sym_wc_data_field] = ACTIONS(1765),
    [sym_wc_data_record] = ACTIONS(1765),
    [sym_last_login_info] = ACTIONS(1765),
    [sym____reserved___] = ACTIONS(1765),
    [sym_date] = ACTIONS(1765),
    [sym_day] = ACTIONS(1765),
    [sym_time] = ACTIONS(1765),
    [sym_ldate] = ACTIONS(1765),
    [sym_push_device] = ACTIONS(1765),
    [sym_push_channel] = ACTIONS(1765),
    [sym_push_devchan] = ACTIONS(1765),
    [sym_release_device] = ACTIONS(1765),
    [sym_release_channel] = ACTIONS(1765),
    [sym_release_devchan] = ACTIONS(1765),
    [sym_master_slot] = ACTIONS(1765),
    [sym_get_pulse_time] = ACTIONS(1765),
    [sym_get_timer] = ACTIONS(1765),
    [sym_master_sn] = ACTIONS(1765),
    [sym_system_number] = ACTIONS(1765),
    [sym_button] = ACTIONS(1765),
    [sym_dv_channel] = ACTIONS(1765),
    [sym_data] = ACTIONS(1765),
    [sym_level] = ACTIONS(1765),
    [sym_timeline] = ACTIONS(1765),
    [sym_channel] = ACTIONS(1765),
    [sym_custom] = ACTIONS(1765),
    [sym_atoi] = ACTIONS(1765),
    [sym_atol] = ACTIONS(1765),
    [sym_atof] = ACTIONS(1765),
    [sym_itoa] = ACTIONS(1765),
    [sym_itohex] = ACTIONS(1765),
    [sym_hextoi] = ACTIONS(1765),
    [sym_ftoa] = ACTIONS(1765),
    [sym_find_string] = ACTIONS(1765),
    [sym_left_string] = ACTIONS(1765),
    [sym_right_string] = ACTIONS(1765),
    [sym_mid_string] = ACTIONS(1765),
    [sym_remove_string] = ACTIONS(1765),
    [sym_get_buffer_string] = ACTIONS(1765),
    [sym_get_buffer_char] = ACTIONS(1765),
    [sym_get_multi_buffer_string] = ACTIONS(1765),
    [sym_length_string] = ACTIONS(1765),
    [sym_max_length_string] = ACTIONS(1765),
    [sym_set_length_string] = ACTIONS(1765),
    [sym_compare_string] = ACTIONS(1765),
    [sym_upper_string] = ACTIONS(1765),
    [sym_lower_string] = ACTIONS(1765),
    [sym_get_last] = ACTIONS(1765),
    [sym_date_to_day] = ACTIONS(1765),
    [sym_date_to_month] = ACTIONS(1765),
    [sym_date_to_year] = ACTIONS(1765),
    [sym_time_to_hour] = ACTIONS(1765),
    [sym_time_to_minute] = ACTIONS(1765),
    [sym_time_to_second] = ACTIONS(1765),
    [sym_day_of_week] = ACTIONS(1765),
    [sym_set_pulse_time] = ACTIONS(1765),
    [sym_set_timer] = ACTIONS(1765),
    [sym_file_open] = ACTIONS(1765),
    [sym_file_close] = ACTIONS(1765),
    [sym_file_read] = ACTIONS(1765),
    [sym_file_read_line] = ACTIONS(1765),
    [sym_file_write] = ACTIONS(1765),
    [sym_file_write_line] = ACTIONS(1765),
    [sym_file_copy] = ACTIONS(1765),
    [sym_file_delete] = ACTIONS(1765),
    [sym_file_rename] = ACTIONS(1765),
    [sym_file_dir] = ACTIONS(1765),
    [sym_file_getdir] = ACTIONS(1765),
    [sym_file_setdir] = ACTIONS(1765),
    [sym_file_create_dir] = ACTIONS(1765),
    [sym_file_remove_dir] = ACTIONS(1765),
    [sym_file_seek] = ACTIONS(1765),
    [sym_length_array] = ACTIONS(1765),
    [sym_max_length_array] = ACTIONS(1765),
    [sym_set_length_array] = ACTIONS(1765),
    [sym_ip_client_open] = ACTIONS(1765),
    [sym_ip_client_close] = ACTIONS(1765),
    [sym_ip_server_open] = ACTIONS(1765),
    [sym_ip_server_close] = ACTIONS(1765),
    [sym_ip_bound_client_open] = ACTIONS(1765),
    [sym_ip_mc_server_open] = ACTIONS(1765),
    [sym_ip_set_option] = ACTIONS(1765),
    [sym_ssh_client_open] = ACTIONS(1765),
    [sym_ssh_client_close] = ACTIONS(1765),
    [sym_tls_client_open] = ACTIONS(1765),
    [sym_tls_client_close] = ACTIONS(1765),
    [sym_do_push] = ACTIONS(1765),
    [sym_do_push_timed] = ACTIONS(1765),
    [sym_do_release] = ACTIONS(1765),
    [sym_device_id] = ACTIONS(1765),
    [sym_device_info] = ACTIONS(1765),
    [sym_device_id_string] = ACTIONS(1765),
    [sym_device_standby] = ACTIONS(1765),
    [sym_device_wake] = ACTIONS(1765),
    [sym_timeline_create] = ACTIONS(1765),
    [sym_timeline_kill] = ACTIONS(1765),
    [sym_timeline_reload] = ACTIONS(1765),
    [sym_timeline_pause] = ACTIONS(1765),
    [sym_timeline_restart] = ACTIONS(1765),
    [sym_timeline_set] = ACTIONS(1765),
    [sym_timeline_get] = ACTIONS(1765),
    [sym_timeline_active] = ACTIONS(1765),
    [sym_timeline_dynamic_id] = ACTIONS(1765),
    [sym_variable_to_string] = ACTIONS(1765),
    [sym_string_to_variable] = ACTIONS(1765),
    [sym_length_variable_to_string] = ACTIONS(1765),
    [sym_type_cast] = ACTIONS(1765),
    [sym_variable_to_xml] = ACTIONS(1765),
    [sym_xml_to_variable] = ACTIONS(1765),
    [sym_length_variable_to_xml] = ACTIONS(1765),
    [sym_redirect_string] = ACTIONS(1765),
    [sym_random_number] = ACTIONS(1765),
    [sym_get_unique_id] = ACTIONS(1765),
    [sym_get_system_number] = ACTIONS(1765),
    [sym_set_system_number] = ACTIONS(1765),
    [sym_get_serial_number] = ACTIONS(1765),
    [sym_get_master_build] = ACTIONS(1765),
    [sym_set_validation_code] = ACTIONS(1765),
    [sym_reboot] = ACTIONS(1765),
    [sym_load_duet_module] = ACTIONS(1765),
    [sym_unload_duet_module] = ACTIONS(1765),
    [sym_duet_mem_size_set] = ACTIONS(1765),
    [sym_duet_mem_size_get] = ACTIONS(1765),
    [sym_dynamic_application_device] = ACTIONS(1765),
    [sym_dynamic_polled_port] = ACTIONS(1765),
    [sym_static_port_binding] = ACTIONS(1765),
    [sym_static_ip_binding] = ACTIONS(1765),
    [sym_get_dns_list] = ACTIONS(1765),
    [sym_set_dns_list] = ACTIONS(1765),
    [sym_get_ip_address] = ACTIONS(1765),
    [sym_set_ip_address] = ACTIONS(1765),
    [sym_get_url_list] = ACTIONS(1765),
    [sym_add_url_entry] = ACTIONS(1765),
    [sym_delete_url_entry] = ACTIONS(1765),
    [sym_abs_value] = ACTIONS(1765),
    [sym_max_value] = ACTIONS(1765),
    [sym_min_value] = ACTIONS(1765),
    [sym_exp_value] = ACTIONS(1765),
    [sym_log_value] = ACTIONS(1765),
    [sym_log10_value] = ACTIONS(1765),
    [sym_power_value] = ACTIONS(1765),
    [sym_sqrt_value] = ACTIONS(1765),
    [sym_raw_be] = ACTIONS(1765),
    [sym_raw_le] = ACTIONS(1765),
    [sym_internal_queue_size_set] = ACTIONS(1765),
    [sym_internal_queue_size_get] = ACTIONS(1765),
    [sym_internal_threshold_set] = ACTIONS(1765),
    [sym_internal_threshold_get] = ACTIONS(1765),
    [sym_rebuilt_event] = ACTIONS(1765),
    [sym_do_custom_event] = ACTIONS(1765),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1765),
    [sym_clkmgr_set_clk_source] = ACTIONS(1765),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1765),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1765),
    [sym_clkmgr_get_timezone] = ACTIONS(1765),
    [sym_clkmgr_set_timezone] = ACTIONS(1765),
    [sym_clkmgr_get_resync_period] = ACTIONS(1765),
    [sym_clkmgr_set_resync_period] = ACTIONS(1765),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1765),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1765),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1765),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1765),
    [sym_clkmgr_get_timeservers] = ACTIONS(1765),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1765),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1765),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1765),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1765),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1765),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1765),
    [sym_set_log_level] = ACTIONS(1765),
    [sym_get_log_level] = ACTIONS(1765),
    [sym_amx_log] = ACTIONS(1765),
    [sym_smtp_server_config_set] = ACTIONS(1765),
    [sym_smtp_server_config_get] = ACTIONS(1765),
    [sym_smtp_send] = ACTIONS(1765),
    [sym_validate_netlinx_account] = ACTIONS(1765),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1765),
    [sym_authenticate_certificate] = ACTIONS(1765),
    [sym_audit_netlinx_session_event] = ACTIONS(1765),
    [sym_audit_netlinx_generic_event] = ACTIONS(1765),
    [sym_get_available_flash_disk_space] = ACTIONS(1765),
    [sym_get_max_flash_disk_space] = ACTIONS(1765),
    [sym_data_create_feed] = ACTIONS(1765),
    [sym_data_delete_feed] = ACTIONS(1765),
    [sym_data_publish_feed] = ACTIONS(1765),
    [sym_data_get_published_feed] = ACTIONS(1765),
    [sym_data_add_record] = ACTIONS(1765),
    [sym_data_get_event_record] = ACTIONS(1765),
    [sym__wc_data_create_feed] = ACTIONS(1765),
    [sym__wc_data_add_record] = ACTIONS(1765),
    [sym__wc_data_get_event_record] = ACTIONS(1765),
    [sym_format] = ACTIONS(1765),
    [sym_astro_clock] = ACTIONS(1765),
    [sym_set_outdoor_temperature] = ACTIONS(1765),
    [sym_set_virtual_level_count] = ACTIONS(1765),
    [sym_set_virtual_channel_count] = ACTIONS(1765),
    [sym_set_virtual_port_count] = ACTIONS(1765),
    [sym__wc_encode_int] = ACTIONS(1765),
    [sym__wc_decode_int] = ACTIONS(1765),
    [sym__wc_int] = ACTIONS(1765),
    [sym__wc_to_ch_int] = ACTIONS(1765),
    [sym__ch_to_wc_int] = ACTIONS(1765),
    [sym__wc_find_string_int] = ACTIONS(1765),
    [sym__wc_left_string_int] = ACTIONS(1765),
    [sym__wc_lower_string_int] = ACTIONS(1765),
    [sym__wc_mid_string_int] = ACTIONS(1765),
    [sym__wc_remove_string_int] = ACTIONS(1765),
    [sym__wc_right_string_int] = ACTIONS(1765),
    [sym__wc_upper_string_int] = ACTIONS(1765),
    [sym__wc_compare_string_int] = ACTIONS(1765),
    [sym__wc_get_buffer_char_int] = ACTIONS(1765),
    [sym__wc_get_buffer_string_int] = ACTIONS(1765),
    [sym__wc_concat_string_int] = ACTIONS(1765),
    [sym__wc_file_open_int] = ACTIONS(1765),
    [sym__wc_file_close_int] = ACTIONS(1765),
    [sym__wc_file_read_int] = ACTIONS(1765),
    [sym__wc_file_read_line_int] = ACTIONS(1765),
    [sym__wc_file_write_int] = ACTIONS(1765),
    [sym__wc_file_write_line_int] = ACTIONS(1765),
    [aux_sym_preproc_include_token1] = ACTIONS(1765),
    [aux_sym_preproc_define_token1] = ACTIONS(1767),
    [aux_sym_preproc_warn_token1] = ACTIONS(1767),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1767),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1767),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1767),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1767),
    [aux_sym_preproc_else_token1] = ACTIONS(1767),
    [anon_sym_LBRACK] = ACTIONS(1767),
    [anon_sym_LBRACE] = ACTIONS(1767),
    [anon_sym_LPAREN] = ACTIONS(1765),
    [anon_sym_BANG] = ACTIONS(1767),
    [anon_sym_TILDE] = ACTIONS(1767),
    [anon_sym_DASH] = ACTIONS(1765),
    [anon_sym_PLUS] = ACTIONS(1765),
    [anon_sym_DASH_DASH] = ACTIONS(1767),
    [anon_sym_PLUS_PLUS] = ACTIONS(1767),
    [anon_sym_DQUOTE] = ACTIONS(1767),
    [anon_sym_SQUOTE] = ACTIONS(1767),
    [sym_number_literal] = ACTIONS(1767),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(222)] = {
    [sym_identifier] = ACTIONS(1769),
    [sym_program_name_keyword] = ACTIONS(1769),
    [sym_module_name_keyword] = ACTIONS(1769),
    [sym_define_device_keyword] = ACTIONS(1769),
    [sym_define_combine_keyword] = ACTIONS(1769),
    [sym_define_constant_keyword] = ACTIONS(1769),
    [sym_define_type_keyword] = ACTIONS(1769),
    [sym_define_variable_keyword] = ACTIONS(1769),
    [sym_define_system_variable_keyword] = ACTIONS(1769),
    [sym_define_connect_level_keyword] = ACTIONS(1769),
    [sym_define_latching_keyword] = ACTIONS(1769),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1769),
    [sym_define_toggling_keyword] = ACTIONS(1769),
    [sym_define_call_keyword] = ACTIONS(1769),
    [sym_define_function_keyword] = ACTIONS(1769),
    [sym_define_library_function_keyword] = ACTIONS(1769),
    [sym_define_start_keyword] = ACTIONS(1769),
    [sym_define_event_keyword] = ACTIONS(1769),
    [sym_define_program_keyword] = ACTIONS(1769),
    [sym_define_module_keyword] = ACTIONS(1769),
    [sym_char_keyword] = ACTIONS(1769),
    [sym_widechar_keyword] = ACTIONS(1769),
    [sym_integer_keyword] = ACTIONS(1769),
    [sym_sinteger_keyword] = ACTIONS(1769),
    [sym_long_keyword] = ACTIONS(1769),
    [sym_slong_keyword] = ACTIONS(1769),
    [sym_float_keyword] = ACTIONS(1769),
    [sym_double_keyword] = ACTIONS(1769),
    [sym_variant_keyword] = ACTIONS(1769),
    [sym_variantarray_keyword] = ACTIONS(1769),
    [sym_local_var_keyword] = ACTIONS(1769),
    [sym_stack_var_keyword] = ACTIONS(1769),
    [sym_constant_keyword] = ACTIONS(1769),
    [sym_volatile_keyword] = ACTIONS(1769),
    [sym_non_volatile_keyword] = ACTIONS(1769),
    [sym_persistent_keyword] = ACTIONS(1769),
    [sym_dev_keyword] = ACTIONS(1769),
    [sym_devlev_keyword] = ACTIONS(1769),
    [sym_devchan_keyword] = ACTIONS(1769),
    [sym_if_keyword] = ACTIONS(1769),
    [sym_else_keyword] = ACTIONS(1769),
    [sym_switch_keyword] = ACTIONS(1769),
    [sym_case_keyword] = ACTIONS(1769),
    [sym_default_keyword] = ACTIONS(1769),
    [sym_while_keyword] = ACTIONS(1769),
    [sym_for_keyword] = ACTIONS(1769),
    [sym_break_keyword] = ACTIONS(1769),
    [sym_continue_keyword] = ACTIONS(1769),
    [sym_return_keyword] = ACTIONS(1769),
    [sym_select_keyword] = ACTIONS(1769),
    [sym_active_keyword] = ACTIONS(1769),
    [sym_struct_keyword] = ACTIONS(1769),
    [sym_structure_keyword] = ACTIONS(1769),
    [sym_button_event_keyword] = ACTIONS(1769),
    [sym_channel_event_keyword] = ACTIONS(1769),
    [sym_level_event_keyword] = ACTIONS(1769),
    [sym_data_event_keyword] = ACTIONS(1769),
    [sym_timeline_event_keyword] = ACTIONS(1769),
    [sym_custom_event_keyword] = ACTIONS(1769),
    [sym_push_keyword] = ACTIONS(1769),
    [sym_release_keyword] = ACTIONS(1769),
    [aux_sym_on_keyword_token1] = ACTIONS(1769),
    [aux_sym_off_keyword_token1] = ACTIONS(1769),
    [sym_send_level_keyword] = ACTIONS(1769),
    [sym_send_string_keyword] = ACTIONS(1769),
    [sym_send_command_keyword] = ACTIONS(1769),
    [sym_clear_buffer_keyword] = ACTIONS(1769),
    [sym_create_buffer_keyword] = ACTIONS(1769),
    [sym_create_multi_buffer_keyword] = ACTIONS(1769),
    [sym_call_keyword] = ACTIONS(1769),
    [sym_system_call_keyword] = ACTIONS(1769),
    [sym_devchan_to_keyword] = ACTIONS(1769),
    [sym_devchan_min_to_keyword] = ACTIONS(1769),
    [sym_devchan_total_off_keyword] = ACTIONS(1769),
    [sym_devchan_pulse_keyword] = ACTIONS(1769),
    [sym_wait_keyword] = ACTIONS(1769),
    [sym_cancel_wait_keyword] = ACTIONS(1769),
    [sym_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1769),
    [sym_not] = ACTIONS(1769),
    [sym_bnot] = ACTIONS(1769),
    [sym___file__] = ACTIONS(1769),
    [sym___date__] = ACTIONS(1769),
    [sym___ldate__] = ACTIONS(1769),
    [sym___time__] = ACTIONS(1769),
    [sym___line__] = ACTIONS(1769),
    [sym___name__] = ACTIONS(1769),
    [sym___netlinx__] = ACTIONS(1769),
    [sym_netlinx_axi_version] = ACTIONS(1769),
    [sym_url_flg_tcp] = ACTIONS(1769),
    [sym_url_flg_accinfopresent] = ACTIONS(1769),
    [sym_url_flg_temp] = ACTIONS(1769),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1769),
    [sym_url_flg_stat_mask] = ACTIONS(1769),
    [sym_url_flg_stat_lookup] = ACTIONS(1769),
    [sym_url_flg_stat_connecting] = ACTIONS(1769),
    [sym_url_flg_stat_waiting] = ACTIONS(1769),
    [sym_url_flg_stat_connected] = ACTIONS(1769),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1769),
    [sym_first_virtual_device] = ACTIONS(1769),
    [sym_first_local_port] = ACTIONS(1769),
    [sym_dynamic_virtual_device] = ACTIONS(1769),
    [sym_timeline_once] = ACTIONS(1769),
    [sym_timeline_repeat] = ACTIONS(1769),
    [sym_timeline_absolute] = ACTIONS(1769),
    [sym_timeline_relative] = ACTIONS(1769),
    [sym_do_push_timed_infinite] = ACTIONS(1769),
    [sym_source_type_no_address] = ACTIONS(1769),
    [sym_source_type_neuron_id] = ACTIONS(1769),
    [sym_source_type_ip_address] = ACTIONS(1769),
    [sym_source_type_axlink] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1769),
    [sym_source_type_ip_socket_address] = ACTIONS(1769),
    [sym_source_type_rs232] = ACTIONS(1769),
    [sym_source_type_internal] = ACTIONS(1769),
    [sym_source_type_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1769),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1769),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1769),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1769),
    [sym_true] = ACTIONS(1769),
    [sym_false] = ACTIONS(1769),
    [sym_file_read_only] = ACTIONS(1769),
    [sym_file_rw_new] = ACTIONS(1769),
    [sym_file_rw_append] = ACTIONS(1769),
    [sym_ip_tcp] = ACTIONS(1769),
    [sym_ip_udp] = ACTIONS(1769),
    [sym_ip_udp_2way] = ACTIONS(1769),
    [sym_xml_encode_types] = ACTIONS(1769),
    [sym_xml_encode_char_as_list] = ACTIONS(1769),
    [sym_xml_encode_le] = ACTIONS(1769),
    [sym_xml_decode_types] = ACTIONS(1769),
    [sym_xml_decode_no_preserve] = ACTIONS(1769),
    [sym_duet_dev_type_display_device] = ACTIONS(1769),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_io_device] = ACTIONS(1769),
    [sym_duet_dev_type_relay_device] = ACTIONS(1769),
    [sym_duet_dev_type_ups] = ACTIONS(1769),
    [sym_duet_dev_type_amplifier] = ACTIONS(1769),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1769),
    [sym_duet_dev_type_camera] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1769),
    [sym_duet_dev_type_dss] = ACTIONS(1769),
    [sym_duet_dev_type_dvr] = ACTIONS(1769),
    [sym_duet_dev_type_disc_device] = ACTIONS(1769),
    [sym_duet_dev_type_document_camera] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1769),
    [sym_duet_dev_type_hvac] = ACTIONS(1769),
    [sym_duet_dev_type_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_light] = ACTIONS(1769),
    [sym_duet_dev_type_monitor] = ACTIONS(1769),
    [sym_duet_dev_type_motor] = ACTIONS(1769),
    [sym_duet_dev_type_multi_window] = ACTIONS(1769),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1769),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1769),
    [sym_duet_dev_type_receiver] = ACTIONS(1769),
    [sym_duet_dev_type_security_system] = ACTIONS(1769),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1769),
    [sym_duet_dev_type_settop_box] = ACTIONS(1769),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1769),
    [sym_duet_dev_type_switcher] = ACTIONS(1769),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_tv] = ACTIONS(1769),
    [sym_duet_dev_type_utility] = ACTIONS(1769),
    [sym_duet_dev_type_vcr] = ACTIONS(1769),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_video_processor] = ACTIONS(1769),
    [sym_duet_dev_type_video_projector] = ACTIONS(1769),
    [sym_duet_dev_type_video_wall] = ACTIONS(1769),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1769),
    [sym_duet_dev_type_weather] = ACTIONS(1769),
    [sym_duet_dev_not_polled] = ACTIONS(1769),
    [sym_duet_dev_polled] = ACTIONS(1769),
    [sym_clkmgr_mode_network] = ACTIONS(1769),
    [sym_clkmgr_mode_standalone] = ACTIONS(1769),
    [sym_normal_standby] = ACTIONS(1769),
    [sym_normal_wake] = ACTIONS(1769),
    [sym_ip_multicast_ttl_option] = ACTIONS(1769),
    [sym_ip_ttl_subnet] = ACTIONS(1769),
    [sym_ip_ttl_site] = ACTIONS(1769),
    [sym_ip_ttl_region] = ACTIONS(1769),
    [sym_ip_ttl_continent] = ACTIONS(1769),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1769),
    [sym_ip_nodelay_on] = ACTIONS(1769),
    [sym_ip_nodelay_off] = ACTIONS(1769),
    [sym_amx_error] = ACTIONS(1769),
    [sym_amx_warning] = ACTIONS(1769),
    [sym_amx_info] = ACTIONS(1769),
    [sym_amx_debug] = ACTIONS(1769),
    [sym_netlinx_login_success] = ACTIONS(1769),
    [sym_netlinx_login_fail] = ACTIONS(1769),
    [sym_netlinx_logout] = ACTIONS(1769),
    [sym_null_str] = ACTIONS(1769),
    [sym_smtp_address] = ACTIONS(1769),
    [sym_smtp_port_number] = ACTIONS(1769),
    [sym_smtp_username] = ACTIONS(1769),
    [sym_smtp_password] = ACTIONS(1769),
    [sym_smtp_require_tls] = ACTIONS(1769),
    [sym_smtp_from] = ACTIONS(1769),
    [sym_smtp_tls_true] = ACTIONS(1769),
    [sym_smtp_tls_false] = ACTIONS(1769),
    [sym_listview_on_row_select_event] = ACTIONS(1769),
    [sym_data_structure_unknown] = ACTIONS(1769),
    [sym_data_structure_datafeed] = ACTIONS(1769),
    [sym_data_structure_datarecordset] = ACTIONS(1769),
    [sym_data_structure_datarecord] = ACTIONS(1769),
    [sym_data_structure_datafield] = ACTIONS(1769),
    [sym_data_max_content_field_count] = ACTIONS(1769),
    [sym_data_max_metadata_field_count] = ACTIONS(1769),
    [sym_data_max_id_length] = ACTIONS(1769),
    [sym_data_max_name_length] = ACTIONS(1769),
    [sym_data_max_description_length] = ACTIONS(1769),
    [sym_data_max_type_length] = ACTIONS(1769),
    [sym_data_max_format_length] = ACTIONS(1769),
    [sym_data_max_source_length] = ACTIONS(1769),
    [sym_data_max_value_length] = ACTIONS(1769),
    [sym_data_max_label_length] = ACTIONS(1769),
    [sym_data_max_url_length] = ACTIONS(1769),
    [sym_max_last_login_info_length] = ACTIONS(1769),
    [sym_data_type_unknown] = ACTIONS(1769),
    [sym_data_type_string] = ACTIONS(1769),
    [sym_data_type_datetime] = ACTIONS(1769),
    [sym_data_type_date] = ACTIONS(1769),
    [sym_data_type_time] = ACTIONS(1769),
    [sym_data_type_image] = ACTIONS(1769),
    [sym_data_format_url] = ACTIONS(1769),
    [sym_data_format_phone] = ACTIONS(1769),
    [sym_data_format_email] = ACTIONS(1769),
    [sym_data_format_iso8601] = ACTIONS(1769),
    [sym_auditlog_priv] = ACTIONS(1769),
    [sym_configuration_priv] = ACTIONS(1769),
    [sym_deviceconfig_priv] = ACTIONS(1769),
    [sym_ftp_priv] = ACTIONS(1769),
    [sym_http_priv] = ACTIONS(1769),
    [sym_networkconfig_priv] = ACTIONS(1769),
    [sym_programport_priv] = ACTIONS(1769),
    [sym_remoteui_priv] = ACTIONS(1769),
    [sym_securitycontrol_priv] = ACTIONS(1769),
    [sym_softwaremanagement_priv] = ACTIONS(1769),
    [sym_terminal_priv] = ACTIONS(1769),
    [sym_tpadmin_priv] = ACTIONS(1769),
    [sym_user1_priv] = ACTIONS(1769),
    [sym_user2_priv] = ACTIONS(1769),
    [sym_user3_priv] = ACTIONS(1769),
    [sym_user4_priv] = ACTIONS(1769),
    [sym_usermanagement_priv] = ACTIONS(1769),
    [sym_valid_account] = ACTIONS(1769),
    [sym_err_name_invalid] = ACTIONS(1769),
    [sym_err_password_invalid] = ACTIONS(1769),
    [sym_err_invalid_account] = ACTIONS(1769),
    [sym_err_no_authorization] = ACTIONS(1769),
    [sym_err_invalid_parameter] = ACTIONS(1769),
    [sym_err_user_locked_out] = ACTIONS(1769),
    [sym_err_user_account_expired] = ACTIONS(1769),
    [sym_err_general_error] = ACTIONS(1769),
    [sym_err_authentication_error] = ACTIONS(1769),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1769),
    [sym_tls_validate_certificate] = ACTIONS(1769),
    [sym_tdata] = ACTIONS(1769),
    [sym_tchannel] = ACTIONS(1769),
    [sym_tlevel] = ACTIONS(1769),
    [sym_tbutton] = ACTIONS(1769),
    [sym_ttimeline] = ACTIONS(1769),
    [sym_tcustom] = ACTIONS(1769),
    [sym_url_struct] = ACTIONS(1769),
    [sym_dns_struct] = ACTIONS(1769),
    [sym_ip_address_struct] = ACTIONS(1769),
    [sym_dev_info_struct] = ACTIONS(1769),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1769),
    [sym_data_feed] = ACTIONS(1769),
    [sym_data_field] = ACTIONS(1769),
    [sym_data_record] = ACTIONS(1769),
    [sym_wc_data_feed] = ACTIONS(1769),
    [sym_wc_data_field] = ACTIONS(1769),
    [sym_wc_data_record] = ACTIONS(1769),
    [sym_last_login_info] = ACTIONS(1769),
    [sym____reserved___] = ACTIONS(1769),
    [sym_date] = ACTIONS(1769),
    [sym_day] = ACTIONS(1769),
    [sym_time] = ACTIONS(1769),
    [sym_ldate] = ACTIONS(1769),
    [sym_push_device] = ACTIONS(1769),
    [sym_push_channel] = ACTIONS(1769),
    [sym_push_devchan] = ACTIONS(1769),
    [sym_release_device] = ACTIONS(1769),
    [sym_release_channel] = ACTIONS(1769),
    [sym_release_devchan] = ACTIONS(1769),
    [sym_master_slot] = ACTIONS(1769),
    [sym_get_pulse_time] = ACTIONS(1769),
    [sym_get_timer] = ACTIONS(1769),
    [sym_master_sn] = ACTIONS(1769),
    [sym_system_number] = ACTIONS(1769),
    [sym_button] = ACTIONS(1769),
    [sym_dv_channel] = ACTIONS(1769),
    [sym_data] = ACTIONS(1769),
    [sym_level] = ACTIONS(1769),
    [sym_timeline] = ACTIONS(1769),
    [sym_channel] = ACTIONS(1769),
    [sym_custom] = ACTIONS(1769),
    [sym_atoi] = ACTIONS(1769),
    [sym_atol] = ACTIONS(1769),
    [sym_atof] = ACTIONS(1769),
    [sym_itoa] = ACTIONS(1769),
    [sym_itohex] = ACTIONS(1769),
    [sym_hextoi] = ACTIONS(1769),
    [sym_ftoa] = ACTIONS(1769),
    [sym_find_string] = ACTIONS(1769),
    [sym_left_string] = ACTIONS(1769),
    [sym_right_string] = ACTIONS(1769),
    [sym_mid_string] = ACTIONS(1769),
    [sym_remove_string] = ACTIONS(1769),
    [sym_get_buffer_string] = ACTIONS(1769),
    [sym_get_buffer_char] = ACTIONS(1769),
    [sym_get_multi_buffer_string] = ACTIONS(1769),
    [sym_length_string] = ACTIONS(1769),
    [sym_max_length_string] = ACTIONS(1769),
    [sym_set_length_string] = ACTIONS(1769),
    [sym_compare_string] = ACTIONS(1769),
    [sym_upper_string] = ACTIONS(1769),
    [sym_lower_string] = ACTIONS(1769),
    [sym_get_last] = ACTIONS(1769),
    [sym_date_to_day] = ACTIONS(1769),
    [sym_date_to_month] = ACTIONS(1769),
    [sym_date_to_year] = ACTIONS(1769),
    [sym_time_to_hour] = ACTIONS(1769),
    [sym_time_to_minute] = ACTIONS(1769),
    [sym_time_to_second] = ACTIONS(1769),
    [sym_day_of_week] = ACTIONS(1769),
    [sym_set_pulse_time] = ACTIONS(1769),
    [sym_set_timer] = ACTIONS(1769),
    [sym_file_open] = ACTIONS(1769),
    [sym_file_close] = ACTIONS(1769),
    [sym_file_read] = ACTIONS(1769),
    [sym_file_read_line] = ACTIONS(1769),
    [sym_file_write] = ACTIONS(1769),
    [sym_file_write_line] = ACTIONS(1769),
    [sym_file_copy] = ACTIONS(1769),
    [sym_file_delete] = ACTIONS(1769),
    [sym_file_rename] = ACTIONS(1769),
    [sym_file_dir] = ACTIONS(1769),
    [sym_file_getdir] = ACTIONS(1769),
    [sym_file_setdir] = ACTIONS(1769),
    [sym_file_create_dir] = ACTIONS(1769),
    [sym_file_remove_dir] = ACTIONS(1769),
    [sym_file_seek] = ACTIONS(1769),
    [sym_length_array] = ACTIONS(1769),
    [sym_max_length_array] = ACTIONS(1769),
    [sym_set_length_array] = ACTIONS(1769),
    [sym_ip_client_open] = ACTIONS(1769),
    [sym_ip_client_close] = ACTIONS(1769),
    [sym_ip_server_open] = ACTIONS(1769),
    [sym_ip_server_close] = ACTIONS(1769),
    [sym_ip_bound_client_open] = ACTIONS(1769),
    [sym_ip_mc_server_open] = ACTIONS(1769),
    [sym_ip_set_option] = ACTIONS(1769),
    [sym_ssh_client_open] = ACTIONS(1769),
    [sym_ssh_client_close] = ACTIONS(1769),
    [sym_tls_client_open] = ACTIONS(1769),
    [sym_tls_client_close] = ACTIONS(1769),
    [sym_do_push] = ACTIONS(1769),
    [sym_do_push_timed] = ACTIONS(1769),
    [sym_do_release] = ACTIONS(1769),
    [sym_device_id] = ACTIONS(1769),
    [sym_device_info] = ACTIONS(1769),
    [sym_device_id_string] = ACTIONS(1769),
    [sym_device_standby] = ACTIONS(1769),
    [sym_device_wake] = ACTIONS(1769),
    [sym_timeline_create] = ACTIONS(1769),
    [sym_timeline_kill] = ACTIONS(1769),
    [sym_timeline_reload] = ACTIONS(1769),
    [sym_timeline_pause] = ACTIONS(1769),
    [sym_timeline_restart] = ACTIONS(1769),
    [sym_timeline_set] = ACTIONS(1769),
    [sym_timeline_get] = ACTIONS(1769),
    [sym_timeline_active] = ACTIONS(1769),
    [sym_timeline_dynamic_id] = ACTIONS(1769),
    [sym_variable_to_string] = ACTIONS(1769),
    [sym_string_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_string] = ACTIONS(1769),
    [sym_type_cast] = ACTIONS(1769),
    [sym_variable_to_xml] = ACTIONS(1769),
    [sym_xml_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_xml] = ACTIONS(1769),
    [sym_redirect_string] = ACTIONS(1769),
    [sym_random_number] = ACTIONS(1769),
    [sym_get_unique_id] = ACTIONS(1769),
    [sym_get_system_number] = ACTIONS(1769),
    [sym_set_system_number] = ACTIONS(1769),
    [sym_get_serial_number] = ACTIONS(1769),
    [sym_get_master_build] = ACTIONS(1769),
    [sym_set_validation_code] = ACTIONS(1769),
    [sym_reboot] = ACTIONS(1769),
    [sym_load_duet_module] = ACTIONS(1769),
    [sym_unload_duet_module] = ACTIONS(1769),
    [sym_duet_mem_size_set] = ACTIONS(1769),
    [sym_duet_mem_size_get] = ACTIONS(1769),
    [sym_dynamic_application_device] = ACTIONS(1769),
    [sym_dynamic_polled_port] = ACTIONS(1769),
    [sym_static_port_binding] = ACTIONS(1769),
    [sym_static_ip_binding] = ACTIONS(1769),
    [sym_get_dns_list] = ACTIONS(1769),
    [sym_set_dns_list] = ACTIONS(1769),
    [sym_get_ip_address] = ACTIONS(1769),
    [sym_set_ip_address] = ACTIONS(1769),
    [sym_get_url_list] = ACTIONS(1769),
    [sym_add_url_entry] = ACTIONS(1769),
    [sym_delete_url_entry] = ACTIONS(1769),
    [sym_abs_value] = ACTIONS(1769),
    [sym_max_value] = ACTIONS(1769),
    [sym_min_value] = ACTIONS(1769),
    [sym_exp_value] = ACTIONS(1769),
    [sym_log_value] = ACTIONS(1769),
    [sym_log10_value] = ACTIONS(1769),
    [sym_power_value] = ACTIONS(1769),
    [sym_sqrt_value] = ACTIONS(1769),
    [sym_raw_be] = ACTIONS(1769),
    [sym_raw_le] = ACTIONS(1769),
    [sym_internal_queue_size_set] = ACTIONS(1769),
    [sym_internal_queue_size_get] = ACTIONS(1769),
    [sym_internal_threshold_set] = ACTIONS(1769),
    [sym_internal_threshold_get] = ACTIONS(1769),
    [sym_rebuilt_event] = ACTIONS(1769),
    [sym_do_custom_event] = ACTIONS(1769),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1769),
    [sym_clkmgr_set_clk_source] = ACTIONS(1769),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1769),
    [sym_clkmgr_get_timezone] = ACTIONS(1769),
    [sym_clkmgr_set_timezone] = ACTIONS(1769),
    [sym_clkmgr_get_resync_period] = ACTIONS(1769),
    [sym_clkmgr_set_resync_period] = ACTIONS(1769),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_timeservers] = ACTIONS(1769),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_set_log_level] = ACTIONS(1769),
    [sym_get_log_level] = ACTIONS(1769),
    [sym_amx_log] = ACTIONS(1769),
    [sym_smtp_server_config_set] = ACTIONS(1769),
    [sym_smtp_server_config_get] = ACTIONS(1769),
    [sym_smtp_send] = ACTIONS(1769),
    [sym_validate_netlinx_account] = ACTIONS(1769),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1769),
    [sym_authenticate_certificate] = ACTIONS(1769),
    [sym_audit_netlinx_session_event] = ACTIONS(1769),
    [sym_audit_netlinx_generic_event] = ACTIONS(1769),
    [sym_get_available_flash_disk_space] = ACTIONS(1769),
    [sym_get_max_flash_disk_space] = ACTIONS(1769),
    [sym_data_create_feed] = ACTIONS(1769),
    [sym_data_delete_feed] = ACTIONS(1769),
    [sym_data_publish_feed] = ACTIONS(1769),
    [sym_data_get_published_feed] = ACTIONS(1769),
    [sym_data_add_record] = ACTIONS(1769),
    [sym_data_get_event_record] = ACTIONS(1769),
    [sym__wc_data_create_feed] = ACTIONS(1769),
    [sym__wc_data_add_record] = ACTIONS(1769),
    [sym__wc_data_get_event_record] = ACTIONS(1769),
    [sym_format] = ACTIONS(1769),
    [sym_astro_clock] = ACTIONS(1769),
    [sym_set_outdoor_temperature] = ACTIONS(1769),
    [sym_set_virtual_level_count] = ACTIONS(1769),
    [sym_set_virtual_channel_count] = ACTIONS(1769),
    [sym_set_virtual_port_count] = ACTIONS(1769),
    [sym__wc_encode_int] = ACTIONS(1769),
    [sym__wc_decode_int] = ACTIONS(1769),
    [sym__wc_int] = ACTIONS(1769),
    [sym__wc_to_ch_int] = ACTIONS(1769),
    [sym__ch_to_wc_int] = ACTIONS(1769),
    [sym__wc_find_string_int] = ACTIONS(1769),
    [sym__wc_left_string_int] = ACTIONS(1769),
    [sym__wc_lower_string_int] = ACTIONS(1769),
    [sym__wc_mid_string_int] = ACTIONS(1769),
    [sym__wc_remove_string_int] = ACTIONS(1769),
    [sym__wc_right_string_int] = ACTIONS(1769),
    [sym__wc_upper_string_int] = ACTIONS(1769),
    [sym__wc_compare_string_int] = ACTIONS(1769),
    [sym__wc_get_buffer_char_int] = ACTIONS(1769),
    [sym__wc_get_buffer_string_int] = ACTIONS(1769),
    [sym__wc_concat_string_int] = ACTIONS(1769),
    [sym__wc_file_open_int] = ACTIONS(1769),
    [sym__wc_file_close_int] = ACTIONS(1769),
    [sym__wc_file_read_int] = ACTIONS(1769),
    [sym__wc_file_read_line_int] = ACTIONS(1769),
    [sym__wc_file_write_int] = ACTIONS(1769),
    [sym__wc_file_write_line_int] = ACTIONS(1769),
    [aux_sym_preproc_include_token1] = ACTIONS(1769),
    [aux_sym_preproc_define_token1] = ACTIONS(1771),
    [aux_sym_preproc_warn_token1] = ACTIONS(1771),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1771),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_else_token1] = ACTIONS(1771),
    [anon_sym_LBRACK] = ACTIONS(1771),
    [anon_sym_LBRACE] = ACTIONS(1771),
    [anon_sym_LPAREN] = ACTIONS(1769),
    [anon_sym_BANG] = ACTIONS(1771),
    [anon_sym_TILDE] = ACTIONS(1771),
    [anon_sym_DASH] = ACTIONS(1769),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_DASH_DASH] = ACTIONS(1771),
    [anon_sym_PLUS_PLUS] = ACTIONS(1771),
    [anon_sym_DQUOTE] = ACTIONS(1771),
    [anon_sym_SQUOTE] = ACTIONS(1771),
    [sym_number_literal] = ACTIONS(1771),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(223)] = {
    [sym_identifier] = ACTIONS(1769),
    [sym_program_name_keyword] = ACTIONS(1769),
    [sym_module_name_keyword] = ACTIONS(1769),
    [sym_define_device_keyword] = ACTIONS(1769),
    [sym_define_combine_keyword] = ACTIONS(1769),
    [sym_define_constant_keyword] = ACTIONS(1769),
    [sym_define_type_keyword] = ACTIONS(1769),
    [sym_define_variable_keyword] = ACTIONS(1769),
    [sym_define_system_variable_keyword] = ACTIONS(1769),
    [sym_define_connect_level_keyword] = ACTIONS(1769),
    [sym_define_latching_keyword] = ACTIONS(1769),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1769),
    [sym_define_toggling_keyword] = ACTIONS(1769),
    [sym_define_call_keyword] = ACTIONS(1769),
    [sym_define_function_keyword] = ACTIONS(1769),
    [sym_define_library_function_keyword] = ACTIONS(1769),
    [sym_define_start_keyword] = ACTIONS(1769),
    [sym_define_event_keyword] = ACTIONS(1769),
    [sym_define_program_keyword] = ACTIONS(1769),
    [sym_define_module_keyword] = ACTIONS(1769),
    [sym_char_keyword] = ACTIONS(1769),
    [sym_widechar_keyword] = ACTIONS(1769),
    [sym_integer_keyword] = ACTIONS(1769),
    [sym_sinteger_keyword] = ACTIONS(1769),
    [sym_long_keyword] = ACTIONS(1769),
    [sym_slong_keyword] = ACTIONS(1769),
    [sym_float_keyword] = ACTIONS(1769),
    [sym_double_keyword] = ACTIONS(1769),
    [sym_variant_keyword] = ACTIONS(1769),
    [sym_variantarray_keyword] = ACTIONS(1769),
    [sym_local_var_keyword] = ACTIONS(1769),
    [sym_stack_var_keyword] = ACTIONS(1769),
    [sym_constant_keyword] = ACTIONS(1769),
    [sym_volatile_keyword] = ACTIONS(1769),
    [sym_non_volatile_keyword] = ACTIONS(1769),
    [sym_persistent_keyword] = ACTIONS(1769),
    [sym_dev_keyword] = ACTIONS(1769),
    [sym_devlev_keyword] = ACTIONS(1769),
    [sym_devchan_keyword] = ACTIONS(1769),
    [sym_if_keyword] = ACTIONS(1769),
    [sym_else_keyword] = ACTIONS(1769),
    [sym_switch_keyword] = ACTIONS(1769),
    [sym_case_keyword] = ACTIONS(1769),
    [sym_default_keyword] = ACTIONS(1769),
    [sym_while_keyword] = ACTIONS(1769),
    [sym_for_keyword] = ACTIONS(1769),
    [sym_break_keyword] = ACTIONS(1769),
    [sym_continue_keyword] = ACTIONS(1769),
    [sym_return_keyword] = ACTIONS(1769),
    [sym_select_keyword] = ACTIONS(1769),
    [sym_active_keyword] = ACTIONS(1769),
    [sym_struct_keyword] = ACTIONS(1769),
    [sym_structure_keyword] = ACTIONS(1769),
    [sym_button_event_keyword] = ACTIONS(1769),
    [sym_channel_event_keyword] = ACTIONS(1769),
    [sym_level_event_keyword] = ACTIONS(1769),
    [sym_data_event_keyword] = ACTIONS(1769),
    [sym_timeline_event_keyword] = ACTIONS(1769),
    [sym_custom_event_keyword] = ACTIONS(1769),
    [sym_push_keyword] = ACTIONS(1769),
    [sym_release_keyword] = ACTIONS(1769),
    [aux_sym_on_keyword_token1] = ACTIONS(1769),
    [aux_sym_off_keyword_token1] = ACTIONS(1769),
    [sym_send_level_keyword] = ACTIONS(1769),
    [sym_send_string_keyword] = ACTIONS(1769),
    [sym_send_command_keyword] = ACTIONS(1769),
    [sym_clear_buffer_keyword] = ACTIONS(1769),
    [sym_create_buffer_keyword] = ACTIONS(1769),
    [sym_create_multi_buffer_keyword] = ACTIONS(1769),
    [sym_call_keyword] = ACTIONS(1769),
    [sym_system_call_keyword] = ACTIONS(1769),
    [sym_devchan_to_keyword] = ACTIONS(1769),
    [sym_devchan_min_to_keyword] = ACTIONS(1769),
    [sym_devchan_total_off_keyword] = ACTIONS(1769),
    [sym_devchan_pulse_keyword] = ACTIONS(1769),
    [sym_wait_keyword] = ACTIONS(1769),
    [sym_cancel_wait_keyword] = ACTIONS(1769),
    [sym_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1769),
    [sym_not] = ACTIONS(1769),
    [sym_bnot] = ACTIONS(1769),
    [sym___file__] = ACTIONS(1769),
    [sym___date__] = ACTIONS(1769),
    [sym___ldate__] = ACTIONS(1769),
    [sym___time__] = ACTIONS(1769),
    [sym___line__] = ACTIONS(1769),
    [sym___name__] = ACTIONS(1769),
    [sym___netlinx__] = ACTIONS(1769),
    [sym_netlinx_axi_version] = ACTIONS(1769),
    [sym_url_flg_tcp] = ACTIONS(1769),
    [sym_url_flg_accinfopresent] = ACTIONS(1769),
    [sym_url_flg_temp] = ACTIONS(1769),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1769),
    [sym_url_flg_stat_mask] = ACTIONS(1769),
    [sym_url_flg_stat_lookup] = ACTIONS(1769),
    [sym_url_flg_stat_connecting] = ACTIONS(1769),
    [sym_url_flg_stat_waiting] = ACTIONS(1769),
    [sym_url_flg_stat_connected] = ACTIONS(1769),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1769),
    [sym_first_virtual_device] = ACTIONS(1769),
    [sym_first_local_port] = ACTIONS(1769),
    [sym_dynamic_virtual_device] = ACTIONS(1769),
    [sym_timeline_once] = ACTIONS(1769),
    [sym_timeline_repeat] = ACTIONS(1769),
    [sym_timeline_absolute] = ACTIONS(1769),
    [sym_timeline_relative] = ACTIONS(1769),
    [sym_do_push_timed_infinite] = ACTIONS(1769),
    [sym_source_type_no_address] = ACTIONS(1769),
    [sym_source_type_neuron_id] = ACTIONS(1769),
    [sym_source_type_ip_address] = ACTIONS(1769),
    [sym_source_type_axlink] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1769),
    [sym_source_type_ip_socket_address] = ACTIONS(1769),
    [sym_source_type_rs232] = ACTIONS(1769),
    [sym_source_type_internal] = ACTIONS(1769),
    [sym_source_type_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1769),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1769),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1769),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1769),
    [sym_true] = ACTIONS(1769),
    [sym_false] = ACTIONS(1769),
    [sym_file_read_only] = ACTIONS(1769),
    [sym_file_rw_new] = ACTIONS(1769),
    [sym_file_rw_append] = ACTIONS(1769),
    [sym_ip_tcp] = ACTIONS(1769),
    [sym_ip_udp] = ACTIONS(1769),
    [sym_ip_udp_2way] = ACTIONS(1769),
    [sym_xml_encode_types] = ACTIONS(1769),
    [sym_xml_encode_char_as_list] = ACTIONS(1769),
    [sym_xml_encode_le] = ACTIONS(1769),
    [sym_xml_decode_types] = ACTIONS(1769),
    [sym_xml_decode_no_preserve] = ACTIONS(1769),
    [sym_duet_dev_type_display_device] = ACTIONS(1769),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_io_device] = ACTIONS(1769),
    [sym_duet_dev_type_relay_device] = ACTIONS(1769),
    [sym_duet_dev_type_ups] = ACTIONS(1769),
    [sym_duet_dev_type_amplifier] = ACTIONS(1769),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1769),
    [sym_duet_dev_type_camera] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1769),
    [sym_duet_dev_type_dss] = ACTIONS(1769),
    [sym_duet_dev_type_dvr] = ACTIONS(1769),
    [sym_duet_dev_type_disc_device] = ACTIONS(1769),
    [sym_duet_dev_type_document_camera] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1769),
    [sym_duet_dev_type_hvac] = ACTIONS(1769),
    [sym_duet_dev_type_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_light] = ACTIONS(1769),
    [sym_duet_dev_type_monitor] = ACTIONS(1769),
    [sym_duet_dev_type_motor] = ACTIONS(1769),
    [sym_duet_dev_type_multi_window] = ACTIONS(1769),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1769),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1769),
    [sym_duet_dev_type_receiver] = ACTIONS(1769),
    [sym_duet_dev_type_security_system] = ACTIONS(1769),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1769),
    [sym_duet_dev_type_settop_box] = ACTIONS(1769),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1769),
    [sym_duet_dev_type_switcher] = ACTIONS(1769),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_tv] = ACTIONS(1769),
    [sym_duet_dev_type_utility] = ACTIONS(1769),
    [sym_duet_dev_type_vcr] = ACTIONS(1769),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_video_processor] = ACTIONS(1769),
    [sym_duet_dev_type_video_projector] = ACTIONS(1769),
    [sym_duet_dev_type_video_wall] = ACTIONS(1769),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1769),
    [sym_duet_dev_type_weather] = ACTIONS(1769),
    [sym_duet_dev_not_polled] = ACTIONS(1769),
    [sym_duet_dev_polled] = ACTIONS(1769),
    [sym_clkmgr_mode_network] = ACTIONS(1769),
    [sym_clkmgr_mode_standalone] = ACTIONS(1769),
    [sym_normal_standby] = ACTIONS(1769),
    [sym_normal_wake] = ACTIONS(1769),
    [sym_ip_multicast_ttl_option] = ACTIONS(1769),
    [sym_ip_ttl_subnet] = ACTIONS(1769),
    [sym_ip_ttl_site] = ACTIONS(1769),
    [sym_ip_ttl_region] = ACTIONS(1769),
    [sym_ip_ttl_continent] = ACTIONS(1769),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1769),
    [sym_ip_nodelay_on] = ACTIONS(1769),
    [sym_ip_nodelay_off] = ACTIONS(1769),
    [sym_amx_error] = ACTIONS(1769),
    [sym_amx_warning] = ACTIONS(1769),
    [sym_amx_info] = ACTIONS(1769),
    [sym_amx_debug] = ACTIONS(1769),
    [sym_netlinx_login_success] = ACTIONS(1769),
    [sym_netlinx_login_fail] = ACTIONS(1769),
    [sym_netlinx_logout] = ACTIONS(1769),
    [sym_null_str] = ACTIONS(1769),
    [sym_smtp_address] = ACTIONS(1769),
    [sym_smtp_port_number] = ACTIONS(1769),
    [sym_smtp_username] = ACTIONS(1769),
    [sym_smtp_password] = ACTIONS(1769),
    [sym_smtp_require_tls] = ACTIONS(1769),
    [sym_smtp_from] = ACTIONS(1769),
    [sym_smtp_tls_true] = ACTIONS(1769),
    [sym_smtp_tls_false] = ACTIONS(1769),
    [sym_listview_on_row_select_event] = ACTIONS(1769),
    [sym_data_structure_unknown] = ACTIONS(1769),
    [sym_data_structure_datafeed] = ACTIONS(1769),
    [sym_data_structure_datarecordset] = ACTIONS(1769),
    [sym_data_structure_datarecord] = ACTIONS(1769),
    [sym_data_structure_datafield] = ACTIONS(1769),
    [sym_data_max_content_field_count] = ACTIONS(1769),
    [sym_data_max_metadata_field_count] = ACTIONS(1769),
    [sym_data_max_id_length] = ACTIONS(1769),
    [sym_data_max_name_length] = ACTIONS(1769),
    [sym_data_max_description_length] = ACTIONS(1769),
    [sym_data_max_type_length] = ACTIONS(1769),
    [sym_data_max_format_length] = ACTIONS(1769),
    [sym_data_max_source_length] = ACTIONS(1769),
    [sym_data_max_value_length] = ACTIONS(1769),
    [sym_data_max_label_length] = ACTIONS(1769),
    [sym_data_max_url_length] = ACTIONS(1769),
    [sym_max_last_login_info_length] = ACTIONS(1769),
    [sym_data_type_unknown] = ACTIONS(1769),
    [sym_data_type_string] = ACTIONS(1769),
    [sym_data_type_datetime] = ACTIONS(1769),
    [sym_data_type_date] = ACTIONS(1769),
    [sym_data_type_time] = ACTIONS(1769),
    [sym_data_type_image] = ACTIONS(1769),
    [sym_data_format_url] = ACTIONS(1769),
    [sym_data_format_phone] = ACTIONS(1769),
    [sym_data_format_email] = ACTIONS(1769),
    [sym_data_format_iso8601] = ACTIONS(1769),
    [sym_auditlog_priv] = ACTIONS(1769),
    [sym_configuration_priv] = ACTIONS(1769),
    [sym_deviceconfig_priv] = ACTIONS(1769),
    [sym_ftp_priv] = ACTIONS(1769),
    [sym_http_priv] = ACTIONS(1769),
    [sym_networkconfig_priv] = ACTIONS(1769),
    [sym_programport_priv] = ACTIONS(1769),
    [sym_remoteui_priv] = ACTIONS(1769),
    [sym_securitycontrol_priv] = ACTIONS(1769),
    [sym_softwaremanagement_priv] = ACTIONS(1769),
    [sym_terminal_priv] = ACTIONS(1769),
    [sym_tpadmin_priv] = ACTIONS(1769),
    [sym_user1_priv] = ACTIONS(1769),
    [sym_user2_priv] = ACTIONS(1769),
    [sym_user3_priv] = ACTIONS(1769),
    [sym_user4_priv] = ACTIONS(1769),
    [sym_usermanagement_priv] = ACTIONS(1769),
    [sym_valid_account] = ACTIONS(1769),
    [sym_err_name_invalid] = ACTIONS(1769),
    [sym_err_password_invalid] = ACTIONS(1769),
    [sym_err_invalid_account] = ACTIONS(1769),
    [sym_err_no_authorization] = ACTIONS(1769),
    [sym_err_invalid_parameter] = ACTIONS(1769),
    [sym_err_user_locked_out] = ACTIONS(1769),
    [sym_err_user_account_expired] = ACTIONS(1769),
    [sym_err_general_error] = ACTIONS(1769),
    [sym_err_authentication_error] = ACTIONS(1769),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1769),
    [sym_tls_validate_certificate] = ACTIONS(1769),
    [sym_tdata] = ACTIONS(1769),
    [sym_tchannel] = ACTIONS(1769),
    [sym_tlevel] = ACTIONS(1769),
    [sym_tbutton] = ACTIONS(1769),
    [sym_ttimeline] = ACTIONS(1769),
    [sym_tcustom] = ACTIONS(1769),
    [sym_url_struct] = ACTIONS(1769),
    [sym_dns_struct] = ACTIONS(1769),
    [sym_ip_address_struct] = ACTIONS(1769),
    [sym_dev_info_struct] = ACTIONS(1769),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1769),
    [sym_data_feed] = ACTIONS(1769),
    [sym_data_field] = ACTIONS(1769),
    [sym_data_record] = ACTIONS(1769),
    [sym_wc_data_feed] = ACTIONS(1769),
    [sym_wc_data_field] = ACTIONS(1769),
    [sym_wc_data_record] = ACTIONS(1769),
    [sym_last_login_info] = ACTIONS(1769),
    [sym____reserved___] = ACTIONS(1769),
    [sym_date] = ACTIONS(1769),
    [sym_day] = ACTIONS(1769),
    [sym_time] = ACTIONS(1769),
    [sym_ldate] = ACTIONS(1769),
    [sym_push_device] = ACTIONS(1769),
    [sym_push_channel] = ACTIONS(1769),
    [sym_push_devchan] = ACTIONS(1769),
    [sym_release_device] = ACTIONS(1769),
    [sym_release_channel] = ACTIONS(1769),
    [sym_release_devchan] = ACTIONS(1769),
    [sym_master_slot] = ACTIONS(1769),
    [sym_get_pulse_time] = ACTIONS(1769),
    [sym_get_timer] = ACTIONS(1769),
    [sym_master_sn] = ACTIONS(1769),
    [sym_system_number] = ACTIONS(1769),
    [sym_button] = ACTIONS(1769),
    [sym_dv_channel] = ACTIONS(1769),
    [sym_data] = ACTIONS(1769),
    [sym_level] = ACTIONS(1769),
    [sym_timeline] = ACTIONS(1769),
    [sym_channel] = ACTIONS(1769),
    [sym_custom] = ACTIONS(1769),
    [sym_atoi] = ACTIONS(1769),
    [sym_atol] = ACTIONS(1769),
    [sym_atof] = ACTIONS(1769),
    [sym_itoa] = ACTIONS(1769),
    [sym_itohex] = ACTIONS(1769),
    [sym_hextoi] = ACTIONS(1769),
    [sym_ftoa] = ACTIONS(1769),
    [sym_find_string] = ACTIONS(1769),
    [sym_left_string] = ACTIONS(1769),
    [sym_right_string] = ACTIONS(1769),
    [sym_mid_string] = ACTIONS(1769),
    [sym_remove_string] = ACTIONS(1769),
    [sym_get_buffer_string] = ACTIONS(1769),
    [sym_get_buffer_char] = ACTIONS(1769),
    [sym_get_multi_buffer_string] = ACTIONS(1769),
    [sym_length_string] = ACTIONS(1769),
    [sym_max_length_string] = ACTIONS(1769),
    [sym_set_length_string] = ACTIONS(1769),
    [sym_compare_string] = ACTIONS(1769),
    [sym_upper_string] = ACTIONS(1769),
    [sym_lower_string] = ACTIONS(1769),
    [sym_get_last] = ACTIONS(1769),
    [sym_date_to_day] = ACTIONS(1769),
    [sym_date_to_month] = ACTIONS(1769),
    [sym_date_to_year] = ACTIONS(1769),
    [sym_time_to_hour] = ACTIONS(1769),
    [sym_time_to_minute] = ACTIONS(1769),
    [sym_time_to_second] = ACTIONS(1769),
    [sym_day_of_week] = ACTIONS(1769),
    [sym_set_pulse_time] = ACTIONS(1769),
    [sym_set_timer] = ACTIONS(1769),
    [sym_file_open] = ACTIONS(1769),
    [sym_file_close] = ACTIONS(1769),
    [sym_file_read] = ACTIONS(1769),
    [sym_file_read_line] = ACTIONS(1769),
    [sym_file_write] = ACTIONS(1769),
    [sym_file_write_line] = ACTIONS(1769),
    [sym_file_copy] = ACTIONS(1769),
    [sym_file_delete] = ACTIONS(1769),
    [sym_file_rename] = ACTIONS(1769),
    [sym_file_dir] = ACTIONS(1769),
    [sym_file_getdir] = ACTIONS(1769),
    [sym_file_setdir] = ACTIONS(1769),
    [sym_file_create_dir] = ACTIONS(1769),
    [sym_file_remove_dir] = ACTIONS(1769),
    [sym_file_seek] = ACTIONS(1769),
    [sym_length_array] = ACTIONS(1769),
    [sym_max_length_array] = ACTIONS(1769),
    [sym_set_length_array] = ACTIONS(1769),
    [sym_ip_client_open] = ACTIONS(1769),
    [sym_ip_client_close] = ACTIONS(1769),
    [sym_ip_server_open] = ACTIONS(1769),
    [sym_ip_server_close] = ACTIONS(1769),
    [sym_ip_bound_client_open] = ACTIONS(1769),
    [sym_ip_mc_server_open] = ACTIONS(1769),
    [sym_ip_set_option] = ACTIONS(1769),
    [sym_ssh_client_open] = ACTIONS(1769),
    [sym_ssh_client_close] = ACTIONS(1769),
    [sym_tls_client_open] = ACTIONS(1769),
    [sym_tls_client_close] = ACTIONS(1769),
    [sym_do_push] = ACTIONS(1769),
    [sym_do_push_timed] = ACTIONS(1769),
    [sym_do_release] = ACTIONS(1769),
    [sym_device_id] = ACTIONS(1769),
    [sym_device_info] = ACTIONS(1769),
    [sym_device_id_string] = ACTIONS(1769),
    [sym_device_standby] = ACTIONS(1769),
    [sym_device_wake] = ACTIONS(1769),
    [sym_timeline_create] = ACTIONS(1769),
    [sym_timeline_kill] = ACTIONS(1769),
    [sym_timeline_reload] = ACTIONS(1769),
    [sym_timeline_pause] = ACTIONS(1769),
    [sym_timeline_restart] = ACTIONS(1769),
    [sym_timeline_set] = ACTIONS(1769),
    [sym_timeline_get] = ACTIONS(1769),
    [sym_timeline_active] = ACTIONS(1769),
    [sym_timeline_dynamic_id] = ACTIONS(1769),
    [sym_variable_to_string] = ACTIONS(1769),
    [sym_string_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_string] = ACTIONS(1769),
    [sym_type_cast] = ACTIONS(1769),
    [sym_variable_to_xml] = ACTIONS(1769),
    [sym_xml_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_xml] = ACTIONS(1769),
    [sym_redirect_string] = ACTIONS(1769),
    [sym_random_number] = ACTIONS(1769),
    [sym_get_unique_id] = ACTIONS(1769),
    [sym_get_system_number] = ACTIONS(1769),
    [sym_set_system_number] = ACTIONS(1769),
    [sym_get_serial_number] = ACTIONS(1769),
    [sym_get_master_build] = ACTIONS(1769),
    [sym_set_validation_code] = ACTIONS(1769),
    [sym_reboot] = ACTIONS(1769),
    [sym_load_duet_module] = ACTIONS(1769),
    [sym_unload_duet_module] = ACTIONS(1769),
    [sym_duet_mem_size_set] = ACTIONS(1769),
    [sym_duet_mem_size_get] = ACTIONS(1769),
    [sym_dynamic_application_device] = ACTIONS(1769),
    [sym_dynamic_polled_port] = ACTIONS(1769),
    [sym_static_port_binding] = ACTIONS(1769),
    [sym_static_ip_binding] = ACTIONS(1769),
    [sym_get_dns_list] = ACTIONS(1769),
    [sym_set_dns_list] = ACTIONS(1769),
    [sym_get_ip_address] = ACTIONS(1769),
    [sym_set_ip_address] = ACTIONS(1769),
    [sym_get_url_list] = ACTIONS(1769),
    [sym_add_url_entry] = ACTIONS(1769),
    [sym_delete_url_entry] = ACTIONS(1769),
    [sym_abs_value] = ACTIONS(1769),
    [sym_max_value] = ACTIONS(1769),
    [sym_min_value] = ACTIONS(1769),
    [sym_exp_value] = ACTIONS(1769),
    [sym_log_value] = ACTIONS(1769),
    [sym_log10_value] = ACTIONS(1769),
    [sym_power_value] = ACTIONS(1769),
    [sym_sqrt_value] = ACTIONS(1769),
    [sym_raw_be] = ACTIONS(1769),
    [sym_raw_le] = ACTIONS(1769),
    [sym_internal_queue_size_set] = ACTIONS(1769),
    [sym_internal_queue_size_get] = ACTIONS(1769),
    [sym_internal_threshold_set] = ACTIONS(1769),
    [sym_internal_threshold_get] = ACTIONS(1769),
    [sym_rebuilt_event] = ACTIONS(1769),
    [sym_do_custom_event] = ACTIONS(1769),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1769),
    [sym_clkmgr_set_clk_source] = ACTIONS(1769),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1769),
    [sym_clkmgr_get_timezone] = ACTIONS(1769),
    [sym_clkmgr_set_timezone] = ACTIONS(1769),
    [sym_clkmgr_get_resync_period] = ACTIONS(1769),
    [sym_clkmgr_set_resync_period] = ACTIONS(1769),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_timeservers] = ACTIONS(1769),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_set_log_level] = ACTIONS(1769),
    [sym_get_log_level] = ACTIONS(1769),
    [sym_amx_log] = ACTIONS(1769),
    [sym_smtp_server_config_set] = ACTIONS(1769),
    [sym_smtp_server_config_get] = ACTIONS(1769),
    [sym_smtp_send] = ACTIONS(1769),
    [sym_validate_netlinx_account] = ACTIONS(1769),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1769),
    [sym_authenticate_certificate] = ACTIONS(1769),
    [sym_audit_netlinx_session_event] = ACTIONS(1769),
    [sym_audit_netlinx_generic_event] = ACTIONS(1769),
    [sym_get_available_flash_disk_space] = ACTIONS(1769),
    [sym_get_max_flash_disk_space] = ACTIONS(1769),
    [sym_data_create_feed] = ACTIONS(1769),
    [sym_data_delete_feed] = ACTIONS(1769),
    [sym_data_publish_feed] = ACTIONS(1769),
    [sym_data_get_published_feed] = ACTIONS(1769),
    [sym_data_add_record] = ACTIONS(1769),
    [sym_data_get_event_record] = ACTIONS(1769),
    [sym__wc_data_create_feed] = ACTIONS(1769),
    [sym__wc_data_add_record] = ACTIONS(1769),
    [sym__wc_data_get_event_record] = ACTIONS(1769),
    [sym_format] = ACTIONS(1769),
    [sym_astro_clock] = ACTIONS(1769),
    [sym_set_outdoor_temperature] = ACTIONS(1769),
    [sym_set_virtual_level_count] = ACTIONS(1769),
    [sym_set_virtual_channel_count] = ACTIONS(1769),
    [sym_set_virtual_port_count] = ACTIONS(1769),
    [sym__wc_encode_int] = ACTIONS(1769),
    [sym__wc_decode_int] = ACTIONS(1769),
    [sym__wc_int] = ACTIONS(1769),
    [sym__wc_to_ch_int] = ACTIONS(1769),
    [sym__ch_to_wc_int] = ACTIONS(1769),
    [sym__wc_find_string_int] = ACTIONS(1769),
    [sym__wc_left_string_int] = ACTIONS(1769),
    [sym__wc_lower_string_int] = ACTIONS(1769),
    [sym__wc_mid_string_int] = ACTIONS(1769),
    [sym__wc_remove_string_int] = ACTIONS(1769),
    [sym__wc_right_string_int] = ACTIONS(1769),
    [sym__wc_upper_string_int] = ACTIONS(1769),
    [sym__wc_compare_string_int] = ACTIONS(1769),
    [sym__wc_get_buffer_char_int] = ACTIONS(1769),
    [sym__wc_get_buffer_string_int] = ACTIONS(1769),
    [sym__wc_concat_string_int] = ACTIONS(1769),
    [sym__wc_file_open_int] = ACTIONS(1769),
    [sym__wc_file_close_int] = ACTIONS(1769),
    [sym__wc_file_read_int] = ACTIONS(1769),
    [sym__wc_file_read_line_int] = ACTIONS(1769),
    [sym__wc_file_write_int] = ACTIONS(1769),
    [sym__wc_file_write_line_int] = ACTIONS(1769),
    [aux_sym_preproc_include_token1] = ACTIONS(1769),
    [aux_sym_preproc_define_token1] = ACTIONS(1771),
    [aux_sym_preproc_warn_token1] = ACTIONS(1771),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1771),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_else_token1] = ACTIONS(1771),
    [anon_sym_LBRACK] = ACTIONS(1771),
    [anon_sym_LBRACE] = ACTIONS(1771),
    [anon_sym_LPAREN] = ACTIONS(1769),
    [anon_sym_BANG] = ACTIONS(1771),
    [anon_sym_TILDE] = ACTIONS(1771),
    [anon_sym_DASH] = ACTIONS(1769),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_DASH_DASH] = ACTIONS(1771),
    [anon_sym_PLUS_PLUS] = ACTIONS(1771),
    [anon_sym_DQUOTE] = ACTIONS(1771),
    [anon_sym_SQUOTE] = ACTIONS(1771),
    [sym_number_literal] = ACTIONS(1771),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(224)] = {
    [sym_identifier] = ACTIONS(1769),
    [sym_program_name_keyword] = ACTIONS(1769),
    [sym_module_name_keyword] = ACTIONS(1769),
    [sym_define_device_keyword] = ACTIONS(1769),
    [sym_define_combine_keyword] = ACTIONS(1769),
    [sym_define_constant_keyword] = ACTIONS(1769),
    [sym_define_type_keyword] = ACTIONS(1769),
    [sym_define_variable_keyword] = ACTIONS(1769),
    [sym_define_system_variable_keyword] = ACTIONS(1769),
    [sym_define_connect_level_keyword] = ACTIONS(1769),
    [sym_define_latching_keyword] = ACTIONS(1769),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1769),
    [sym_define_toggling_keyword] = ACTIONS(1769),
    [sym_define_call_keyword] = ACTIONS(1769),
    [sym_define_function_keyword] = ACTIONS(1769),
    [sym_define_library_function_keyword] = ACTIONS(1769),
    [sym_define_start_keyword] = ACTIONS(1769),
    [sym_define_event_keyword] = ACTIONS(1769),
    [sym_define_program_keyword] = ACTIONS(1769),
    [sym_define_module_keyword] = ACTIONS(1769),
    [sym_char_keyword] = ACTIONS(1769),
    [sym_widechar_keyword] = ACTIONS(1769),
    [sym_integer_keyword] = ACTIONS(1769),
    [sym_sinteger_keyword] = ACTIONS(1769),
    [sym_long_keyword] = ACTIONS(1769),
    [sym_slong_keyword] = ACTIONS(1769),
    [sym_float_keyword] = ACTIONS(1769),
    [sym_double_keyword] = ACTIONS(1769),
    [sym_variant_keyword] = ACTIONS(1769),
    [sym_variantarray_keyword] = ACTIONS(1769),
    [sym_local_var_keyword] = ACTIONS(1769),
    [sym_stack_var_keyword] = ACTIONS(1769),
    [sym_constant_keyword] = ACTIONS(1769),
    [sym_volatile_keyword] = ACTIONS(1769),
    [sym_non_volatile_keyword] = ACTIONS(1769),
    [sym_persistent_keyword] = ACTIONS(1769),
    [sym_dev_keyword] = ACTIONS(1769),
    [sym_devlev_keyword] = ACTIONS(1769),
    [sym_devchan_keyword] = ACTIONS(1769),
    [sym_if_keyword] = ACTIONS(1769),
    [sym_else_keyword] = ACTIONS(1769),
    [sym_switch_keyword] = ACTIONS(1769),
    [sym_case_keyword] = ACTIONS(1769),
    [sym_default_keyword] = ACTIONS(1769),
    [sym_while_keyword] = ACTIONS(1769),
    [sym_for_keyword] = ACTIONS(1769),
    [sym_break_keyword] = ACTIONS(1769),
    [sym_continue_keyword] = ACTIONS(1769),
    [sym_return_keyword] = ACTIONS(1769),
    [sym_select_keyword] = ACTIONS(1769),
    [sym_active_keyword] = ACTIONS(1769),
    [sym_struct_keyword] = ACTIONS(1769),
    [sym_structure_keyword] = ACTIONS(1769),
    [sym_button_event_keyword] = ACTIONS(1769),
    [sym_channel_event_keyword] = ACTIONS(1769),
    [sym_level_event_keyword] = ACTIONS(1769),
    [sym_data_event_keyword] = ACTIONS(1769),
    [sym_timeline_event_keyword] = ACTIONS(1769),
    [sym_custom_event_keyword] = ACTIONS(1769),
    [sym_push_keyword] = ACTIONS(1769),
    [sym_release_keyword] = ACTIONS(1769),
    [aux_sym_on_keyword_token1] = ACTIONS(1769),
    [aux_sym_off_keyword_token1] = ACTIONS(1769),
    [sym_send_level_keyword] = ACTIONS(1769),
    [sym_send_string_keyword] = ACTIONS(1769),
    [sym_send_command_keyword] = ACTIONS(1769),
    [sym_clear_buffer_keyword] = ACTIONS(1769),
    [sym_create_buffer_keyword] = ACTIONS(1769),
    [sym_create_multi_buffer_keyword] = ACTIONS(1769),
    [sym_call_keyword] = ACTIONS(1769),
    [sym_system_call_keyword] = ACTIONS(1769),
    [sym_devchan_to_keyword] = ACTIONS(1769),
    [sym_devchan_min_to_keyword] = ACTIONS(1769),
    [sym_devchan_total_off_keyword] = ACTIONS(1769),
    [sym_devchan_pulse_keyword] = ACTIONS(1769),
    [sym_wait_keyword] = ACTIONS(1769),
    [sym_cancel_wait_keyword] = ACTIONS(1769),
    [sym_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1769),
    [sym_not] = ACTIONS(1769),
    [sym_bnot] = ACTIONS(1769),
    [sym___file__] = ACTIONS(1769),
    [sym___date__] = ACTIONS(1769),
    [sym___ldate__] = ACTIONS(1769),
    [sym___time__] = ACTIONS(1769),
    [sym___line__] = ACTIONS(1769),
    [sym___name__] = ACTIONS(1769),
    [sym___netlinx__] = ACTIONS(1769),
    [sym_netlinx_axi_version] = ACTIONS(1769),
    [sym_url_flg_tcp] = ACTIONS(1769),
    [sym_url_flg_accinfopresent] = ACTIONS(1769),
    [sym_url_flg_temp] = ACTIONS(1769),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1769),
    [sym_url_flg_stat_mask] = ACTIONS(1769),
    [sym_url_flg_stat_lookup] = ACTIONS(1769),
    [sym_url_flg_stat_connecting] = ACTIONS(1769),
    [sym_url_flg_stat_waiting] = ACTIONS(1769),
    [sym_url_flg_stat_connected] = ACTIONS(1769),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1769),
    [sym_first_virtual_device] = ACTIONS(1769),
    [sym_first_local_port] = ACTIONS(1769),
    [sym_dynamic_virtual_device] = ACTIONS(1769),
    [sym_timeline_once] = ACTIONS(1769),
    [sym_timeline_repeat] = ACTIONS(1769),
    [sym_timeline_absolute] = ACTIONS(1769),
    [sym_timeline_relative] = ACTIONS(1769),
    [sym_do_push_timed_infinite] = ACTIONS(1769),
    [sym_source_type_no_address] = ACTIONS(1769),
    [sym_source_type_neuron_id] = ACTIONS(1769),
    [sym_source_type_ip_address] = ACTIONS(1769),
    [sym_source_type_axlink] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1769),
    [sym_source_type_ip_socket_address] = ACTIONS(1769),
    [sym_source_type_rs232] = ACTIONS(1769),
    [sym_source_type_internal] = ACTIONS(1769),
    [sym_source_type_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1769),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1769),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1769),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1769),
    [sym_true] = ACTIONS(1769),
    [sym_false] = ACTIONS(1769),
    [sym_file_read_only] = ACTIONS(1769),
    [sym_file_rw_new] = ACTIONS(1769),
    [sym_file_rw_append] = ACTIONS(1769),
    [sym_ip_tcp] = ACTIONS(1769),
    [sym_ip_udp] = ACTIONS(1769),
    [sym_ip_udp_2way] = ACTIONS(1769),
    [sym_xml_encode_types] = ACTIONS(1769),
    [sym_xml_encode_char_as_list] = ACTIONS(1769),
    [sym_xml_encode_le] = ACTIONS(1769),
    [sym_xml_decode_types] = ACTIONS(1769),
    [sym_xml_decode_no_preserve] = ACTIONS(1769),
    [sym_duet_dev_type_display_device] = ACTIONS(1769),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_io_device] = ACTIONS(1769),
    [sym_duet_dev_type_relay_device] = ACTIONS(1769),
    [sym_duet_dev_type_ups] = ACTIONS(1769),
    [sym_duet_dev_type_amplifier] = ACTIONS(1769),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1769),
    [sym_duet_dev_type_camera] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1769),
    [sym_duet_dev_type_dss] = ACTIONS(1769),
    [sym_duet_dev_type_dvr] = ACTIONS(1769),
    [sym_duet_dev_type_disc_device] = ACTIONS(1769),
    [sym_duet_dev_type_document_camera] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1769),
    [sym_duet_dev_type_hvac] = ACTIONS(1769),
    [sym_duet_dev_type_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_light] = ACTIONS(1769),
    [sym_duet_dev_type_monitor] = ACTIONS(1769),
    [sym_duet_dev_type_motor] = ACTIONS(1769),
    [sym_duet_dev_type_multi_window] = ACTIONS(1769),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1769),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1769),
    [sym_duet_dev_type_receiver] = ACTIONS(1769),
    [sym_duet_dev_type_security_system] = ACTIONS(1769),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1769),
    [sym_duet_dev_type_settop_box] = ACTIONS(1769),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1769),
    [sym_duet_dev_type_switcher] = ACTIONS(1769),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_tv] = ACTIONS(1769),
    [sym_duet_dev_type_utility] = ACTIONS(1769),
    [sym_duet_dev_type_vcr] = ACTIONS(1769),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_video_processor] = ACTIONS(1769),
    [sym_duet_dev_type_video_projector] = ACTIONS(1769),
    [sym_duet_dev_type_video_wall] = ACTIONS(1769),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1769),
    [sym_duet_dev_type_weather] = ACTIONS(1769),
    [sym_duet_dev_not_polled] = ACTIONS(1769),
    [sym_duet_dev_polled] = ACTIONS(1769),
    [sym_clkmgr_mode_network] = ACTIONS(1769),
    [sym_clkmgr_mode_standalone] = ACTIONS(1769),
    [sym_normal_standby] = ACTIONS(1769),
    [sym_normal_wake] = ACTIONS(1769),
    [sym_ip_multicast_ttl_option] = ACTIONS(1769),
    [sym_ip_ttl_subnet] = ACTIONS(1769),
    [sym_ip_ttl_site] = ACTIONS(1769),
    [sym_ip_ttl_region] = ACTIONS(1769),
    [sym_ip_ttl_continent] = ACTIONS(1769),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1769),
    [sym_ip_nodelay_on] = ACTIONS(1769),
    [sym_ip_nodelay_off] = ACTIONS(1769),
    [sym_amx_error] = ACTIONS(1769),
    [sym_amx_warning] = ACTIONS(1769),
    [sym_amx_info] = ACTIONS(1769),
    [sym_amx_debug] = ACTIONS(1769),
    [sym_netlinx_login_success] = ACTIONS(1769),
    [sym_netlinx_login_fail] = ACTIONS(1769),
    [sym_netlinx_logout] = ACTIONS(1769),
    [sym_null_str] = ACTIONS(1769),
    [sym_smtp_address] = ACTIONS(1769),
    [sym_smtp_port_number] = ACTIONS(1769),
    [sym_smtp_username] = ACTIONS(1769),
    [sym_smtp_password] = ACTIONS(1769),
    [sym_smtp_require_tls] = ACTIONS(1769),
    [sym_smtp_from] = ACTIONS(1769),
    [sym_smtp_tls_true] = ACTIONS(1769),
    [sym_smtp_tls_false] = ACTIONS(1769),
    [sym_listview_on_row_select_event] = ACTIONS(1769),
    [sym_data_structure_unknown] = ACTIONS(1769),
    [sym_data_structure_datafeed] = ACTIONS(1769),
    [sym_data_structure_datarecordset] = ACTIONS(1769),
    [sym_data_structure_datarecord] = ACTIONS(1769),
    [sym_data_structure_datafield] = ACTIONS(1769),
    [sym_data_max_content_field_count] = ACTIONS(1769),
    [sym_data_max_metadata_field_count] = ACTIONS(1769),
    [sym_data_max_id_length] = ACTIONS(1769),
    [sym_data_max_name_length] = ACTIONS(1769),
    [sym_data_max_description_length] = ACTIONS(1769),
    [sym_data_max_type_length] = ACTIONS(1769),
    [sym_data_max_format_length] = ACTIONS(1769),
    [sym_data_max_source_length] = ACTIONS(1769),
    [sym_data_max_value_length] = ACTIONS(1769),
    [sym_data_max_label_length] = ACTIONS(1769),
    [sym_data_max_url_length] = ACTIONS(1769),
    [sym_max_last_login_info_length] = ACTIONS(1769),
    [sym_data_type_unknown] = ACTIONS(1769),
    [sym_data_type_string] = ACTIONS(1769),
    [sym_data_type_datetime] = ACTIONS(1769),
    [sym_data_type_date] = ACTIONS(1769),
    [sym_data_type_time] = ACTIONS(1769),
    [sym_data_type_image] = ACTIONS(1769),
    [sym_data_format_url] = ACTIONS(1769),
    [sym_data_format_phone] = ACTIONS(1769),
    [sym_data_format_email] = ACTIONS(1769),
    [sym_data_format_iso8601] = ACTIONS(1769),
    [sym_auditlog_priv] = ACTIONS(1769),
    [sym_configuration_priv] = ACTIONS(1769),
    [sym_deviceconfig_priv] = ACTIONS(1769),
    [sym_ftp_priv] = ACTIONS(1769),
    [sym_http_priv] = ACTIONS(1769),
    [sym_networkconfig_priv] = ACTIONS(1769),
    [sym_programport_priv] = ACTIONS(1769),
    [sym_remoteui_priv] = ACTIONS(1769),
    [sym_securitycontrol_priv] = ACTIONS(1769),
    [sym_softwaremanagement_priv] = ACTIONS(1769),
    [sym_terminal_priv] = ACTIONS(1769),
    [sym_tpadmin_priv] = ACTIONS(1769),
    [sym_user1_priv] = ACTIONS(1769),
    [sym_user2_priv] = ACTIONS(1769),
    [sym_user3_priv] = ACTIONS(1769),
    [sym_user4_priv] = ACTIONS(1769),
    [sym_usermanagement_priv] = ACTIONS(1769),
    [sym_valid_account] = ACTIONS(1769),
    [sym_err_name_invalid] = ACTIONS(1769),
    [sym_err_password_invalid] = ACTIONS(1769),
    [sym_err_invalid_account] = ACTIONS(1769),
    [sym_err_no_authorization] = ACTIONS(1769),
    [sym_err_invalid_parameter] = ACTIONS(1769),
    [sym_err_user_locked_out] = ACTIONS(1769),
    [sym_err_user_account_expired] = ACTIONS(1769),
    [sym_err_general_error] = ACTIONS(1769),
    [sym_err_authentication_error] = ACTIONS(1769),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1769),
    [sym_tls_validate_certificate] = ACTIONS(1769),
    [sym_tdata] = ACTIONS(1769),
    [sym_tchannel] = ACTIONS(1769),
    [sym_tlevel] = ACTIONS(1769),
    [sym_tbutton] = ACTIONS(1769),
    [sym_ttimeline] = ACTIONS(1769),
    [sym_tcustom] = ACTIONS(1769),
    [sym_url_struct] = ACTIONS(1769),
    [sym_dns_struct] = ACTIONS(1769),
    [sym_ip_address_struct] = ACTIONS(1769),
    [sym_dev_info_struct] = ACTIONS(1769),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1769),
    [sym_data_feed] = ACTIONS(1769),
    [sym_data_field] = ACTIONS(1769),
    [sym_data_record] = ACTIONS(1769),
    [sym_wc_data_feed] = ACTIONS(1769),
    [sym_wc_data_field] = ACTIONS(1769),
    [sym_wc_data_record] = ACTIONS(1769),
    [sym_last_login_info] = ACTIONS(1769),
    [sym____reserved___] = ACTIONS(1769),
    [sym_date] = ACTIONS(1769),
    [sym_day] = ACTIONS(1769),
    [sym_time] = ACTIONS(1769),
    [sym_ldate] = ACTIONS(1769),
    [sym_push_device] = ACTIONS(1769),
    [sym_push_channel] = ACTIONS(1769),
    [sym_push_devchan] = ACTIONS(1769),
    [sym_release_device] = ACTIONS(1769),
    [sym_release_channel] = ACTIONS(1769),
    [sym_release_devchan] = ACTIONS(1769),
    [sym_master_slot] = ACTIONS(1769),
    [sym_get_pulse_time] = ACTIONS(1769),
    [sym_get_timer] = ACTIONS(1769),
    [sym_master_sn] = ACTIONS(1769),
    [sym_system_number] = ACTIONS(1769),
    [sym_button] = ACTIONS(1769),
    [sym_dv_channel] = ACTIONS(1769),
    [sym_data] = ACTIONS(1769),
    [sym_level] = ACTIONS(1769),
    [sym_timeline] = ACTIONS(1769),
    [sym_channel] = ACTIONS(1769),
    [sym_custom] = ACTIONS(1769),
    [sym_atoi] = ACTIONS(1769),
    [sym_atol] = ACTIONS(1769),
    [sym_atof] = ACTIONS(1769),
    [sym_itoa] = ACTIONS(1769),
    [sym_itohex] = ACTIONS(1769),
    [sym_hextoi] = ACTIONS(1769),
    [sym_ftoa] = ACTIONS(1769),
    [sym_find_string] = ACTIONS(1769),
    [sym_left_string] = ACTIONS(1769),
    [sym_right_string] = ACTIONS(1769),
    [sym_mid_string] = ACTIONS(1769),
    [sym_remove_string] = ACTIONS(1769),
    [sym_get_buffer_string] = ACTIONS(1769),
    [sym_get_buffer_char] = ACTIONS(1769),
    [sym_get_multi_buffer_string] = ACTIONS(1769),
    [sym_length_string] = ACTIONS(1769),
    [sym_max_length_string] = ACTIONS(1769),
    [sym_set_length_string] = ACTIONS(1769),
    [sym_compare_string] = ACTIONS(1769),
    [sym_upper_string] = ACTIONS(1769),
    [sym_lower_string] = ACTIONS(1769),
    [sym_get_last] = ACTIONS(1769),
    [sym_date_to_day] = ACTIONS(1769),
    [sym_date_to_month] = ACTIONS(1769),
    [sym_date_to_year] = ACTIONS(1769),
    [sym_time_to_hour] = ACTIONS(1769),
    [sym_time_to_minute] = ACTIONS(1769),
    [sym_time_to_second] = ACTIONS(1769),
    [sym_day_of_week] = ACTIONS(1769),
    [sym_set_pulse_time] = ACTIONS(1769),
    [sym_set_timer] = ACTIONS(1769),
    [sym_file_open] = ACTIONS(1769),
    [sym_file_close] = ACTIONS(1769),
    [sym_file_read] = ACTIONS(1769),
    [sym_file_read_line] = ACTIONS(1769),
    [sym_file_write] = ACTIONS(1769),
    [sym_file_write_line] = ACTIONS(1769),
    [sym_file_copy] = ACTIONS(1769),
    [sym_file_delete] = ACTIONS(1769),
    [sym_file_rename] = ACTIONS(1769),
    [sym_file_dir] = ACTIONS(1769),
    [sym_file_getdir] = ACTIONS(1769),
    [sym_file_setdir] = ACTIONS(1769),
    [sym_file_create_dir] = ACTIONS(1769),
    [sym_file_remove_dir] = ACTIONS(1769),
    [sym_file_seek] = ACTIONS(1769),
    [sym_length_array] = ACTIONS(1769),
    [sym_max_length_array] = ACTIONS(1769),
    [sym_set_length_array] = ACTIONS(1769),
    [sym_ip_client_open] = ACTIONS(1769),
    [sym_ip_client_close] = ACTIONS(1769),
    [sym_ip_server_open] = ACTIONS(1769),
    [sym_ip_server_close] = ACTIONS(1769),
    [sym_ip_bound_client_open] = ACTIONS(1769),
    [sym_ip_mc_server_open] = ACTIONS(1769),
    [sym_ip_set_option] = ACTIONS(1769),
    [sym_ssh_client_open] = ACTIONS(1769),
    [sym_ssh_client_close] = ACTIONS(1769),
    [sym_tls_client_open] = ACTIONS(1769),
    [sym_tls_client_close] = ACTIONS(1769),
    [sym_do_push] = ACTIONS(1769),
    [sym_do_push_timed] = ACTIONS(1769),
    [sym_do_release] = ACTIONS(1769),
    [sym_device_id] = ACTIONS(1769),
    [sym_device_info] = ACTIONS(1769),
    [sym_device_id_string] = ACTIONS(1769),
    [sym_device_standby] = ACTIONS(1769),
    [sym_device_wake] = ACTIONS(1769),
    [sym_timeline_create] = ACTIONS(1769),
    [sym_timeline_kill] = ACTIONS(1769),
    [sym_timeline_reload] = ACTIONS(1769),
    [sym_timeline_pause] = ACTIONS(1769),
    [sym_timeline_restart] = ACTIONS(1769),
    [sym_timeline_set] = ACTIONS(1769),
    [sym_timeline_get] = ACTIONS(1769),
    [sym_timeline_active] = ACTIONS(1769),
    [sym_timeline_dynamic_id] = ACTIONS(1769),
    [sym_variable_to_string] = ACTIONS(1769),
    [sym_string_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_string] = ACTIONS(1769),
    [sym_type_cast] = ACTIONS(1769),
    [sym_variable_to_xml] = ACTIONS(1769),
    [sym_xml_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_xml] = ACTIONS(1769),
    [sym_redirect_string] = ACTIONS(1769),
    [sym_random_number] = ACTIONS(1769),
    [sym_get_unique_id] = ACTIONS(1769),
    [sym_get_system_number] = ACTIONS(1769),
    [sym_set_system_number] = ACTIONS(1769),
    [sym_get_serial_number] = ACTIONS(1769),
    [sym_get_master_build] = ACTIONS(1769),
    [sym_set_validation_code] = ACTIONS(1769),
    [sym_reboot] = ACTIONS(1769),
    [sym_load_duet_module] = ACTIONS(1769),
    [sym_unload_duet_module] = ACTIONS(1769),
    [sym_duet_mem_size_set] = ACTIONS(1769),
    [sym_duet_mem_size_get] = ACTIONS(1769),
    [sym_dynamic_application_device] = ACTIONS(1769),
    [sym_dynamic_polled_port] = ACTIONS(1769),
    [sym_static_port_binding] = ACTIONS(1769),
    [sym_static_ip_binding] = ACTIONS(1769),
    [sym_get_dns_list] = ACTIONS(1769),
    [sym_set_dns_list] = ACTIONS(1769),
    [sym_get_ip_address] = ACTIONS(1769),
    [sym_set_ip_address] = ACTIONS(1769),
    [sym_get_url_list] = ACTIONS(1769),
    [sym_add_url_entry] = ACTIONS(1769),
    [sym_delete_url_entry] = ACTIONS(1769),
    [sym_abs_value] = ACTIONS(1769),
    [sym_max_value] = ACTIONS(1769),
    [sym_min_value] = ACTIONS(1769),
    [sym_exp_value] = ACTIONS(1769),
    [sym_log_value] = ACTIONS(1769),
    [sym_log10_value] = ACTIONS(1769),
    [sym_power_value] = ACTIONS(1769),
    [sym_sqrt_value] = ACTIONS(1769),
    [sym_raw_be] = ACTIONS(1769),
    [sym_raw_le] = ACTIONS(1769),
    [sym_internal_queue_size_set] = ACTIONS(1769),
    [sym_internal_queue_size_get] = ACTIONS(1769),
    [sym_internal_threshold_set] = ACTIONS(1769),
    [sym_internal_threshold_get] = ACTIONS(1769),
    [sym_rebuilt_event] = ACTIONS(1769),
    [sym_do_custom_event] = ACTIONS(1769),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1769),
    [sym_clkmgr_set_clk_source] = ACTIONS(1769),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1769),
    [sym_clkmgr_get_timezone] = ACTIONS(1769),
    [sym_clkmgr_set_timezone] = ACTIONS(1769),
    [sym_clkmgr_get_resync_period] = ACTIONS(1769),
    [sym_clkmgr_set_resync_period] = ACTIONS(1769),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_timeservers] = ACTIONS(1769),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_set_log_level] = ACTIONS(1769),
    [sym_get_log_level] = ACTIONS(1769),
    [sym_amx_log] = ACTIONS(1769),
    [sym_smtp_server_config_set] = ACTIONS(1769),
    [sym_smtp_server_config_get] = ACTIONS(1769),
    [sym_smtp_send] = ACTIONS(1769),
    [sym_validate_netlinx_account] = ACTIONS(1769),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1769),
    [sym_authenticate_certificate] = ACTIONS(1769),
    [sym_audit_netlinx_session_event] = ACTIONS(1769),
    [sym_audit_netlinx_generic_event] = ACTIONS(1769),
    [sym_get_available_flash_disk_space] = ACTIONS(1769),
    [sym_get_max_flash_disk_space] = ACTIONS(1769),
    [sym_data_create_feed] = ACTIONS(1769),
    [sym_data_delete_feed] = ACTIONS(1769),
    [sym_data_publish_feed] = ACTIONS(1769),
    [sym_data_get_published_feed] = ACTIONS(1769),
    [sym_data_add_record] = ACTIONS(1769),
    [sym_data_get_event_record] = ACTIONS(1769),
    [sym__wc_data_create_feed] = ACTIONS(1769),
    [sym__wc_data_add_record] = ACTIONS(1769),
    [sym__wc_data_get_event_record] = ACTIONS(1769),
    [sym_format] = ACTIONS(1769),
    [sym_astro_clock] = ACTIONS(1769),
    [sym_set_outdoor_temperature] = ACTIONS(1769),
    [sym_set_virtual_level_count] = ACTIONS(1769),
    [sym_set_virtual_channel_count] = ACTIONS(1769),
    [sym_set_virtual_port_count] = ACTIONS(1769),
    [sym__wc_encode_int] = ACTIONS(1769),
    [sym__wc_decode_int] = ACTIONS(1769),
    [sym__wc_int] = ACTIONS(1769),
    [sym__wc_to_ch_int] = ACTIONS(1769),
    [sym__ch_to_wc_int] = ACTIONS(1769),
    [sym__wc_find_string_int] = ACTIONS(1769),
    [sym__wc_left_string_int] = ACTIONS(1769),
    [sym__wc_lower_string_int] = ACTIONS(1769),
    [sym__wc_mid_string_int] = ACTIONS(1769),
    [sym__wc_remove_string_int] = ACTIONS(1769),
    [sym__wc_right_string_int] = ACTIONS(1769),
    [sym__wc_upper_string_int] = ACTIONS(1769),
    [sym__wc_compare_string_int] = ACTIONS(1769),
    [sym__wc_get_buffer_char_int] = ACTIONS(1769),
    [sym__wc_get_buffer_string_int] = ACTIONS(1769),
    [sym__wc_concat_string_int] = ACTIONS(1769),
    [sym__wc_file_open_int] = ACTIONS(1769),
    [sym__wc_file_close_int] = ACTIONS(1769),
    [sym__wc_file_read_int] = ACTIONS(1769),
    [sym__wc_file_read_line_int] = ACTIONS(1769),
    [sym__wc_file_write_int] = ACTIONS(1769),
    [sym__wc_file_write_line_int] = ACTIONS(1769),
    [aux_sym_preproc_include_token1] = ACTIONS(1769),
    [aux_sym_preproc_define_token1] = ACTIONS(1771),
    [aux_sym_preproc_warn_token1] = ACTIONS(1771),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1771),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_else_token1] = ACTIONS(1771),
    [anon_sym_LBRACK] = ACTIONS(1771),
    [anon_sym_LBRACE] = ACTIONS(1771),
    [anon_sym_LPAREN] = ACTIONS(1769),
    [anon_sym_BANG] = ACTIONS(1771),
    [anon_sym_TILDE] = ACTIONS(1771),
    [anon_sym_DASH] = ACTIONS(1769),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_DASH_DASH] = ACTIONS(1771),
    [anon_sym_PLUS_PLUS] = ACTIONS(1771),
    [anon_sym_DQUOTE] = ACTIONS(1771),
    [anon_sym_SQUOTE] = ACTIONS(1771),
    [sym_number_literal] = ACTIONS(1771),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(225)] = {
    [sym_identifier] = ACTIONS(1769),
    [sym_program_name_keyword] = ACTIONS(1769),
    [sym_module_name_keyword] = ACTIONS(1769),
    [sym_define_device_keyword] = ACTIONS(1769),
    [sym_define_combine_keyword] = ACTIONS(1769),
    [sym_define_constant_keyword] = ACTIONS(1769),
    [sym_define_type_keyword] = ACTIONS(1769),
    [sym_define_variable_keyword] = ACTIONS(1769),
    [sym_define_system_variable_keyword] = ACTIONS(1769),
    [sym_define_connect_level_keyword] = ACTIONS(1769),
    [sym_define_latching_keyword] = ACTIONS(1769),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1769),
    [sym_define_toggling_keyword] = ACTIONS(1769),
    [sym_define_call_keyword] = ACTIONS(1769),
    [sym_define_function_keyword] = ACTIONS(1769),
    [sym_define_library_function_keyword] = ACTIONS(1769),
    [sym_define_start_keyword] = ACTIONS(1769),
    [sym_define_event_keyword] = ACTIONS(1769),
    [sym_define_program_keyword] = ACTIONS(1769),
    [sym_define_module_keyword] = ACTIONS(1769),
    [sym_char_keyword] = ACTIONS(1769),
    [sym_widechar_keyword] = ACTIONS(1769),
    [sym_integer_keyword] = ACTIONS(1769),
    [sym_sinteger_keyword] = ACTIONS(1769),
    [sym_long_keyword] = ACTIONS(1769),
    [sym_slong_keyword] = ACTIONS(1769),
    [sym_float_keyword] = ACTIONS(1769),
    [sym_double_keyword] = ACTIONS(1769),
    [sym_variant_keyword] = ACTIONS(1769),
    [sym_variantarray_keyword] = ACTIONS(1769),
    [sym_local_var_keyword] = ACTIONS(1769),
    [sym_stack_var_keyword] = ACTIONS(1769),
    [sym_constant_keyword] = ACTIONS(1769),
    [sym_volatile_keyword] = ACTIONS(1769),
    [sym_non_volatile_keyword] = ACTIONS(1769),
    [sym_persistent_keyword] = ACTIONS(1769),
    [sym_dev_keyword] = ACTIONS(1769),
    [sym_devlev_keyword] = ACTIONS(1769),
    [sym_devchan_keyword] = ACTIONS(1769),
    [sym_if_keyword] = ACTIONS(1769),
    [sym_else_keyword] = ACTIONS(1769),
    [sym_switch_keyword] = ACTIONS(1769),
    [sym_case_keyword] = ACTIONS(1769),
    [sym_default_keyword] = ACTIONS(1769),
    [sym_while_keyword] = ACTIONS(1769),
    [sym_for_keyword] = ACTIONS(1769),
    [sym_break_keyword] = ACTIONS(1769),
    [sym_continue_keyword] = ACTIONS(1769),
    [sym_return_keyword] = ACTIONS(1769),
    [sym_select_keyword] = ACTIONS(1769),
    [sym_active_keyword] = ACTIONS(1769),
    [sym_struct_keyword] = ACTIONS(1769),
    [sym_structure_keyword] = ACTIONS(1769),
    [sym_button_event_keyword] = ACTIONS(1769),
    [sym_channel_event_keyword] = ACTIONS(1769),
    [sym_level_event_keyword] = ACTIONS(1769),
    [sym_data_event_keyword] = ACTIONS(1769),
    [sym_timeline_event_keyword] = ACTIONS(1769),
    [sym_custom_event_keyword] = ACTIONS(1769),
    [sym_push_keyword] = ACTIONS(1769),
    [sym_release_keyword] = ACTIONS(1769),
    [aux_sym_on_keyword_token1] = ACTIONS(1769),
    [aux_sym_off_keyword_token1] = ACTIONS(1769),
    [sym_send_level_keyword] = ACTIONS(1769),
    [sym_send_string_keyword] = ACTIONS(1769),
    [sym_send_command_keyword] = ACTIONS(1769),
    [sym_clear_buffer_keyword] = ACTIONS(1769),
    [sym_create_buffer_keyword] = ACTIONS(1769),
    [sym_create_multi_buffer_keyword] = ACTIONS(1769),
    [sym_call_keyword] = ACTIONS(1769),
    [sym_system_call_keyword] = ACTIONS(1769),
    [sym_devchan_to_keyword] = ACTIONS(1769),
    [sym_devchan_min_to_keyword] = ACTIONS(1769),
    [sym_devchan_total_off_keyword] = ACTIONS(1769),
    [sym_devchan_pulse_keyword] = ACTIONS(1769),
    [sym_wait_keyword] = ACTIONS(1769),
    [sym_cancel_wait_keyword] = ACTIONS(1769),
    [sym_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1769),
    [sym_not] = ACTIONS(1769),
    [sym_bnot] = ACTIONS(1769),
    [sym___file__] = ACTIONS(1769),
    [sym___date__] = ACTIONS(1769),
    [sym___ldate__] = ACTIONS(1769),
    [sym___time__] = ACTIONS(1769),
    [sym___line__] = ACTIONS(1769),
    [sym___name__] = ACTIONS(1769),
    [sym___netlinx__] = ACTIONS(1769),
    [sym_netlinx_axi_version] = ACTIONS(1769),
    [sym_url_flg_tcp] = ACTIONS(1769),
    [sym_url_flg_accinfopresent] = ACTIONS(1769),
    [sym_url_flg_temp] = ACTIONS(1769),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1769),
    [sym_url_flg_stat_mask] = ACTIONS(1769),
    [sym_url_flg_stat_lookup] = ACTIONS(1769),
    [sym_url_flg_stat_connecting] = ACTIONS(1769),
    [sym_url_flg_stat_waiting] = ACTIONS(1769),
    [sym_url_flg_stat_connected] = ACTIONS(1769),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1769),
    [sym_first_virtual_device] = ACTIONS(1769),
    [sym_first_local_port] = ACTIONS(1769),
    [sym_dynamic_virtual_device] = ACTIONS(1769),
    [sym_timeline_once] = ACTIONS(1769),
    [sym_timeline_repeat] = ACTIONS(1769),
    [sym_timeline_absolute] = ACTIONS(1769),
    [sym_timeline_relative] = ACTIONS(1769),
    [sym_do_push_timed_infinite] = ACTIONS(1769),
    [sym_source_type_no_address] = ACTIONS(1769),
    [sym_source_type_neuron_id] = ACTIONS(1769),
    [sym_source_type_ip_address] = ACTIONS(1769),
    [sym_source_type_axlink] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1769),
    [sym_source_type_ip_socket_address] = ACTIONS(1769),
    [sym_source_type_rs232] = ACTIONS(1769),
    [sym_source_type_internal] = ACTIONS(1769),
    [sym_source_type_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1769),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1769),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1769),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1769),
    [sym_true] = ACTIONS(1769),
    [sym_false] = ACTIONS(1769),
    [sym_file_read_only] = ACTIONS(1769),
    [sym_file_rw_new] = ACTIONS(1769),
    [sym_file_rw_append] = ACTIONS(1769),
    [sym_ip_tcp] = ACTIONS(1769),
    [sym_ip_udp] = ACTIONS(1769),
    [sym_ip_udp_2way] = ACTIONS(1769),
    [sym_xml_encode_types] = ACTIONS(1769),
    [sym_xml_encode_char_as_list] = ACTIONS(1769),
    [sym_xml_encode_le] = ACTIONS(1769),
    [sym_xml_decode_types] = ACTIONS(1769),
    [sym_xml_decode_no_preserve] = ACTIONS(1769),
    [sym_duet_dev_type_display_device] = ACTIONS(1769),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_io_device] = ACTIONS(1769),
    [sym_duet_dev_type_relay_device] = ACTIONS(1769),
    [sym_duet_dev_type_ups] = ACTIONS(1769),
    [sym_duet_dev_type_amplifier] = ACTIONS(1769),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1769),
    [sym_duet_dev_type_camera] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1769),
    [sym_duet_dev_type_dss] = ACTIONS(1769),
    [sym_duet_dev_type_dvr] = ACTIONS(1769),
    [sym_duet_dev_type_disc_device] = ACTIONS(1769),
    [sym_duet_dev_type_document_camera] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1769),
    [sym_duet_dev_type_hvac] = ACTIONS(1769),
    [sym_duet_dev_type_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_light] = ACTIONS(1769),
    [sym_duet_dev_type_monitor] = ACTIONS(1769),
    [sym_duet_dev_type_motor] = ACTIONS(1769),
    [sym_duet_dev_type_multi_window] = ACTIONS(1769),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1769),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1769),
    [sym_duet_dev_type_receiver] = ACTIONS(1769),
    [sym_duet_dev_type_security_system] = ACTIONS(1769),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1769),
    [sym_duet_dev_type_settop_box] = ACTIONS(1769),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1769),
    [sym_duet_dev_type_switcher] = ACTIONS(1769),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_tv] = ACTIONS(1769),
    [sym_duet_dev_type_utility] = ACTIONS(1769),
    [sym_duet_dev_type_vcr] = ACTIONS(1769),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_video_processor] = ACTIONS(1769),
    [sym_duet_dev_type_video_projector] = ACTIONS(1769),
    [sym_duet_dev_type_video_wall] = ACTIONS(1769),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1769),
    [sym_duet_dev_type_weather] = ACTIONS(1769),
    [sym_duet_dev_not_polled] = ACTIONS(1769),
    [sym_duet_dev_polled] = ACTIONS(1769),
    [sym_clkmgr_mode_network] = ACTIONS(1769),
    [sym_clkmgr_mode_standalone] = ACTIONS(1769),
    [sym_normal_standby] = ACTIONS(1769),
    [sym_normal_wake] = ACTIONS(1769),
    [sym_ip_multicast_ttl_option] = ACTIONS(1769),
    [sym_ip_ttl_subnet] = ACTIONS(1769),
    [sym_ip_ttl_site] = ACTIONS(1769),
    [sym_ip_ttl_region] = ACTIONS(1769),
    [sym_ip_ttl_continent] = ACTIONS(1769),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1769),
    [sym_ip_nodelay_on] = ACTIONS(1769),
    [sym_ip_nodelay_off] = ACTIONS(1769),
    [sym_amx_error] = ACTIONS(1769),
    [sym_amx_warning] = ACTIONS(1769),
    [sym_amx_info] = ACTIONS(1769),
    [sym_amx_debug] = ACTIONS(1769),
    [sym_netlinx_login_success] = ACTIONS(1769),
    [sym_netlinx_login_fail] = ACTIONS(1769),
    [sym_netlinx_logout] = ACTIONS(1769),
    [sym_null_str] = ACTIONS(1769),
    [sym_smtp_address] = ACTIONS(1769),
    [sym_smtp_port_number] = ACTIONS(1769),
    [sym_smtp_username] = ACTIONS(1769),
    [sym_smtp_password] = ACTIONS(1769),
    [sym_smtp_require_tls] = ACTIONS(1769),
    [sym_smtp_from] = ACTIONS(1769),
    [sym_smtp_tls_true] = ACTIONS(1769),
    [sym_smtp_tls_false] = ACTIONS(1769),
    [sym_listview_on_row_select_event] = ACTIONS(1769),
    [sym_data_structure_unknown] = ACTIONS(1769),
    [sym_data_structure_datafeed] = ACTIONS(1769),
    [sym_data_structure_datarecordset] = ACTIONS(1769),
    [sym_data_structure_datarecord] = ACTIONS(1769),
    [sym_data_structure_datafield] = ACTIONS(1769),
    [sym_data_max_content_field_count] = ACTIONS(1769),
    [sym_data_max_metadata_field_count] = ACTIONS(1769),
    [sym_data_max_id_length] = ACTIONS(1769),
    [sym_data_max_name_length] = ACTIONS(1769),
    [sym_data_max_description_length] = ACTIONS(1769),
    [sym_data_max_type_length] = ACTIONS(1769),
    [sym_data_max_format_length] = ACTIONS(1769),
    [sym_data_max_source_length] = ACTIONS(1769),
    [sym_data_max_value_length] = ACTIONS(1769),
    [sym_data_max_label_length] = ACTIONS(1769),
    [sym_data_max_url_length] = ACTIONS(1769),
    [sym_max_last_login_info_length] = ACTIONS(1769),
    [sym_data_type_unknown] = ACTIONS(1769),
    [sym_data_type_string] = ACTIONS(1769),
    [sym_data_type_datetime] = ACTIONS(1769),
    [sym_data_type_date] = ACTIONS(1769),
    [sym_data_type_time] = ACTIONS(1769),
    [sym_data_type_image] = ACTIONS(1769),
    [sym_data_format_url] = ACTIONS(1769),
    [sym_data_format_phone] = ACTIONS(1769),
    [sym_data_format_email] = ACTIONS(1769),
    [sym_data_format_iso8601] = ACTIONS(1769),
    [sym_auditlog_priv] = ACTIONS(1769),
    [sym_configuration_priv] = ACTIONS(1769),
    [sym_deviceconfig_priv] = ACTIONS(1769),
    [sym_ftp_priv] = ACTIONS(1769),
    [sym_http_priv] = ACTIONS(1769),
    [sym_networkconfig_priv] = ACTIONS(1769),
    [sym_programport_priv] = ACTIONS(1769),
    [sym_remoteui_priv] = ACTIONS(1769),
    [sym_securitycontrol_priv] = ACTIONS(1769),
    [sym_softwaremanagement_priv] = ACTIONS(1769),
    [sym_terminal_priv] = ACTIONS(1769),
    [sym_tpadmin_priv] = ACTIONS(1769),
    [sym_user1_priv] = ACTIONS(1769),
    [sym_user2_priv] = ACTIONS(1769),
    [sym_user3_priv] = ACTIONS(1769),
    [sym_user4_priv] = ACTIONS(1769),
    [sym_usermanagement_priv] = ACTIONS(1769),
    [sym_valid_account] = ACTIONS(1769),
    [sym_err_name_invalid] = ACTIONS(1769),
    [sym_err_password_invalid] = ACTIONS(1769),
    [sym_err_invalid_account] = ACTIONS(1769),
    [sym_err_no_authorization] = ACTIONS(1769),
    [sym_err_invalid_parameter] = ACTIONS(1769),
    [sym_err_user_locked_out] = ACTIONS(1769),
    [sym_err_user_account_expired] = ACTIONS(1769),
    [sym_err_general_error] = ACTIONS(1769),
    [sym_err_authentication_error] = ACTIONS(1769),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1769),
    [sym_tls_validate_certificate] = ACTIONS(1769),
    [sym_tdata] = ACTIONS(1769),
    [sym_tchannel] = ACTIONS(1769),
    [sym_tlevel] = ACTIONS(1769),
    [sym_tbutton] = ACTIONS(1769),
    [sym_ttimeline] = ACTIONS(1769),
    [sym_tcustom] = ACTIONS(1769),
    [sym_url_struct] = ACTIONS(1769),
    [sym_dns_struct] = ACTIONS(1769),
    [sym_ip_address_struct] = ACTIONS(1769),
    [sym_dev_info_struct] = ACTIONS(1769),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1769),
    [sym_data_feed] = ACTIONS(1769),
    [sym_data_field] = ACTIONS(1769),
    [sym_data_record] = ACTIONS(1769),
    [sym_wc_data_feed] = ACTIONS(1769),
    [sym_wc_data_field] = ACTIONS(1769),
    [sym_wc_data_record] = ACTIONS(1769),
    [sym_last_login_info] = ACTIONS(1769),
    [sym____reserved___] = ACTIONS(1769),
    [sym_date] = ACTIONS(1769),
    [sym_day] = ACTIONS(1769),
    [sym_time] = ACTIONS(1769),
    [sym_ldate] = ACTIONS(1769),
    [sym_push_device] = ACTIONS(1769),
    [sym_push_channel] = ACTIONS(1769),
    [sym_push_devchan] = ACTIONS(1769),
    [sym_release_device] = ACTIONS(1769),
    [sym_release_channel] = ACTIONS(1769),
    [sym_release_devchan] = ACTIONS(1769),
    [sym_master_slot] = ACTIONS(1769),
    [sym_get_pulse_time] = ACTIONS(1769),
    [sym_get_timer] = ACTIONS(1769),
    [sym_master_sn] = ACTIONS(1769),
    [sym_system_number] = ACTIONS(1769),
    [sym_button] = ACTIONS(1769),
    [sym_dv_channel] = ACTIONS(1769),
    [sym_data] = ACTIONS(1769),
    [sym_level] = ACTIONS(1769),
    [sym_timeline] = ACTIONS(1769),
    [sym_channel] = ACTIONS(1769),
    [sym_custom] = ACTIONS(1769),
    [sym_atoi] = ACTIONS(1769),
    [sym_atol] = ACTIONS(1769),
    [sym_atof] = ACTIONS(1769),
    [sym_itoa] = ACTIONS(1769),
    [sym_itohex] = ACTIONS(1769),
    [sym_hextoi] = ACTIONS(1769),
    [sym_ftoa] = ACTIONS(1769),
    [sym_find_string] = ACTIONS(1769),
    [sym_left_string] = ACTIONS(1769),
    [sym_right_string] = ACTIONS(1769),
    [sym_mid_string] = ACTIONS(1769),
    [sym_remove_string] = ACTIONS(1769),
    [sym_get_buffer_string] = ACTIONS(1769),
    [sym_get_buffer_char] = ACTIONS(1769),
    [sym_get_multi_buffer_string] = ACTIONS(1769),
    [sym_length_string] = ACTIONS(1769),
    [sym_max_length_string] = ACTIONS(1769),
    [sym_set_length_string] = ACTIONS(1769),
    [sym_compare_string] = ACTIONS(1769),
    [sym_upper_string] = ACTIONS(1769),
    [sym_lower_string] = ACTIONS(1769),
    [sym_get_last] = ACTIONS(1769),
    [sym_date_to_day] = ACTIONS(1769),
    [sym_date_to_month] = ACTIONS(1769),
    [sym_date_to_year] = ACTIONS(1769),
    [sym_time_to_hour] = ACTIONS(1769),
    [sym_time_to_minute] = ACTIONS(1769),
    [sym_time_to_second] = ACTIONS(1769),
    [sym_day_of_week] = ACTIONS(1769),
    [sym_set_pulse_time] = ACTIONS(1769),
    [sym_set_timer] = ACTIONS(1769),
    [sym_file_open] = ACTIONS(1769),
    [sym_file_close] = ACTIONS(1769),
    [sym_file_read] = ACTIONS(1769),
    [sym_file_read_line] = ACTIONS(1769),
    [sym_file_write] = ACTIONS(1769),
    [sym_file_write_line] = ACTIONS(1769),
    [sym_file_copy] = ACTIONS(1769),
    [sym_file_delete] = ACTIONS(1769),
    [sym_file_rename] = ACTIONS(1769),
    [sym_file_dir] = ACTIONS(1769),
    [sym_file_getdir] = ACTIONS(1769),
    [sym_file_setdir] = ACTIONS(1769),
    [sym_file_create_dir] = ACTIONS(1769),
    [sym_file_remove_dir] = ACTIONS(1769),
    [sym_file_seek] = ACTIONS(1769),
    [sym_length_array] = ACTIONS(1769),
    [sym_max_length_array] = ACTIONS(1769),
    [sym_set_length_array] = ACTIONS(1769),
    [sym_ip_client_open] = ACTIONS(1769),
    [sym_ip_client_close] = ACTIONS(1769),
    [sym_ip_server_open] = ACTIONS(1769),
    [sym_ip_server_close] = ACTIONS(1769),
    [sym_ip_bound_client_open] = ACTIONS(1769),
    [sym_ip_mc_server_open] = ACTIONS(1769),
    [sym_ip_set_option] = ACTIONS(1769),
    [sym_ssh_client_open] = ACTIONS(1769),
    [sym_ssh_client_close] = ACTIONS(1769),
    [sym_tls_client_open] = ACTIONS(1769),
    [sym_tls_client_close] = ACTIONS(1769),
    [sym_do_push] = ACTIONS(1769),
    [sym_do_push_timed] = ACTIONS(1769),
    [sym_do_release] = ACTIONS(1769),
    [sym_device_id] = ACTIONS(1769),
    [sym_device_info] = ACTIONS(1769),
    [sym_device_id_string] = ACTIONS(1769),
    [sym_device_standby] = ACTIONS(1769),
    [sym_device_wake] = ACTIONS(1769),
    [sym_timeline_create] = ACTIONS(1769),
    [sym_timeline_kill] = ACTIONS(1769),
    [sym_timeline_reload] = ACTIONS(1769),
    [sym_timeline_pause] = ACTIONS(1769),
    [sym_timeline_restart] = ACTIONS(1769),
    [sym_timeline_set] = ACTIONS(1769),
    [sym_timeline_get] = ACTIONS(1769),
    [sym_timeline_active] = ACTIONS(1769),
    [sym_timeline_dynamic_id] = ACTIONS(1769),
    [sym_variable_to_string] = ACTIONS(1769),
    [sym_string_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_string] = ACTIONS(1769),
    [sym_type_cast] = ACTIONS(1769),
    [sym_variable_to_xml] = ACTIONS(1769),
    [sym_xml_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_xml] = ACTIONS(1769),
    [sym_redirect_string] = ACTIONS(1769),
    [sym_random_number] = ACTIONS(1769),
    [sym_get_unique_id] = ACTIONS(1769),
    [sym_get_system_number] = ACTIONS(1769),
    [sym_set_system_number] = ACTIONS(1769),
    [sym_get_serial_number] = ACTIONS(1769),
    [sym_get_master_build] = ACTIONS(1769),
    [sym_set_validation_code] = ACTIONS(1769),
    [sym_reboot] = ACTIONS(1769),
    [sym_load_duet_module] = ACTIONS(1769),
    [sym_unload_duet_module] = ACTIONS(1769),
    [sym_duet_mem_size_set] = ACTIONS(1769),
    [sym_duet_mem_size_get] = ACTIONS(1769),
    [sym_dynamic_application_device] = ACTIONS(1769),
    [sym_dynamic_polled_port] = ACTIONS(1769),
    [sym_static_port_binding] = ACTIONS(1769),
    [sym_static_ip_binding] = ACTIONS(1769),
    [sym_get_dns_list] = ACTIONS(1769),
    [sym_set_dns_list] = ACTIONS(1769),
    [sym_get_ip_address] = ACTIONS(1769),
    [sym_set_ip_address] = ACTIONS(1769),
    [sym_get_url_list] = ACTIONS(1769),
    [sym_add_url_entry] = ACTIONS(1769),
    [sym_delete_url_entry] = ACTIONS(1769),
    [sym_abs_value] = ACTIONS(1769),
    [sym_max_value] = ACTIONS(1769),
    [sym_min_value] = ACTIONS(1769),
    [sym_exp_value] = ACTIONS(1769),
    [sym_log_value] = ACTIONS(1769),
    [sym_log10_value] = ACTIONS(1769),
    [sym_power_value] = ACTIONS(1769),
    [sym_sqrt_value] = ACTIONS(1769),
    [sym_raw_be] = ACTIONS(1769),
    [sym_raw_le] = ACTIONS(1769),
    [sym_internal_queue_size_set] = ACTIONS(1769),
    [sym_internal_queue_size_get] = ACTIONS(1769),
    [sym_internal_threshold_set] = ACTIONS(1769),
    [sym_internal_threshold_get] = ACTIONS(1769),
    [sym_rebuilt_event] = ACTIONS(1769),
    [sym_do_custom_event] = ACTIONS(1769),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1769),
    [sym_clkmgr_set_clk_source] = ACTIONS(1769),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1769),
    [sym_clkmgr_get_timezone] = ACTIONS(1769),
    [sym_clkmgr_set_timezone] = ACTIONS(1769),
    [sym_clkmgr_get_resync_period] = ACTIONS(1769),
    [sym_clkmgr_set_resync_period] = ACTIONS(1769),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_timeservers] = ACTIONS(1769),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_set_log_level] = ACTIONS(1769),
    [sym_get_log_level] = ACTIONS(1769),
    [sym_amx_log] = ACTIONS(1769),
    [sym_smtp_server_config_set] = ACTIONS(1769),
    [sym_smtp_server_config_get] = ACTIONS(1769),
    [sym_smtp_send] = ACTIONS(1769),
    [sym_validate_netlinx_account] = ACTIONS(1769),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1769),
    [sym_authenticate_certificate] = ACTIONS(1769),
    [sym_audit_netlinx_session_event] = ACTIONS(1769),
    [sym_audit_netlinx_generic_event] = ACTIONS(1769),
    [sym_get_available_flash_disk_space] = ACTIONS(1769),
    [sym_get_max_flash_disk_space] = ACTIONS(1769),
    [sym_data_create_feed] = ACTIONS(1769),
    [sym_data_delete_feed] = ACTIONS(1769),
    [sym_data_publish_feed] = ACTIONS(1769),
    [sym_data_get_published_feed] = ACTIONS(1769),
    [sym_data_add_record] = ACTIONS(1769),
    [sym_data_get_event_record] = ACTIONS(1769),
    [sym__wc_data_create_feed] = ACTIONS(1769),
    [sym__wc_data_add_record] = ACTIONS(1769),
    [sym__wc_data_get_event_record] = ACTIONS(1769),
    [sym_format] = ACTIONS(1769),
    [sym_astro_clock] = ACTIONS(1769),
    [sym_set_outdoor_temperature] = ACTIONS(1769),
    [sym_set_virtual_level_count] = ACTIONS(1769),
    [sym_set_virtual_channel_count] = ACTIONS(1769),
    [sym_set_virtual_port_count] = ACTIONS(1769),
    [sym__wc_encode_int] = ACTIONS(1769),
    [sym__wc_decode_int] = ACTIONS(1769),
    [sym__wc_int] = ACTIONS(1769),
    [sym__wc_to_ch_int] = ACTIONS(1769),
    [sym__ch_to_wc_int] = ACTIONS(1769),
    [sym__wc_find_string_int] = ACTIONS(1769),
    [sym__wc_left_string_int] = ACTIONS(1769),
    [sym__wc_lower_string_int] = ACTIONS(1769),
    [sym__wc_mid_string_int] = ACTIONS(1769),
    [sym__wc_remove_string_int] = ACTIONS(1769),
    [sym__wc_right_string_int] = ACTIONS(1769),
    [sym__wc_upper_string_int] = ACTIONS(1769),
    [sym__wc_compare_string_int] = ACTIONS(1769),
    [sym__wc_get_buffer_char_int] = ACTIONS(1769),
    [sym__wc_get_buffer_string_int] = ACTIONS(1769),
    [sym__wc_concat_string_int] = ACTIONS(1769),
    [sym__wc_file_open_int] = ACTIONS(1769),
    [sym__wc_file_close_int] = ACTIONS(1769),
    [sym__wc_file_read_int] = ACTIONS(1769),
    [sym__wc_file_read_line_int] = ACTIONS(1769),
    [sym__wc_file_write_int] = ACTIONS(1769),
    [sym__wc_file_write_line_int] = ACTIONS(1769),
    [aux_sym_preproc_include_token1] = ACTIONS(1769),
    [aux_sym_preproc_define_token1] = ACTIONS(1771),
    [aux_sym_preproc_warn_token1] = ACTIONS(1771),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1771),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_else_token1] = ACTIONS(1771),
    [anon_sym_LBRACK] = ACTIONS(1771),
    [anon_sym_LBRACE] = ACTIONS(1771),
    [anon_sym_LPAREN] = ACTIONS(1769),
    [anon_sym_BANG] = ACTIONS(1771),
    [anon_sym_TILDE] = ACTIONS(1771),
    [anon_sym_DASH] = ACTIONS(1769),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_DASH_DASH] = ACTIONS(1771),
    [anon_sym_PLUS_PLUS] = ACTIONS(1771),
    [anon_sym_DQUOTE] = ACTIONS(1771),
    [anon_sym_SQUOTE] = ACTIONS(1771),
    [sym_number_literal] = ACTIONS(1771),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(226)] = {
    [sym_identifier] = ACTIONS(1773),
    [sym_program_name_keyword] = ACTIONS(1773),
    [sym_module_name_keyword] = ACTIONS(1773),
    [sym_define_device_keyword] = ACTIONS(1773),
    [sym_define_combine_keyword] = ACTIONS(1773),
    [sym_define_constant_keyword] = ACTIONS(1773),
    [sym_define_type_keyword] = ACTIONS(1773),
    [sym_define_variable_keyword] = ACTIONS(1773),
    [sym_define_system_variable_keyword] = ACTIONS(1773),
    [sym_define_connect_level_keyword] = ACTIONS(1773),
    [sym_define_latching_keyword] = ACTIONS(1773),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1773),
    [sym_define_toggling_keyword] = ACTIONS(1773),
    [sym_define_call_keyword] = ACTIONS(1773),
    [sym_define_function_keyword] = ACTIONS(1773),
    [sym_define_library_function_keyword] = ACTIONS(1773),
    [sym_define_start_keyword] = ACTIONS(1773),
    [sym_define_event_keyword] = ACTIONS(1773),
    [sym_define_program_keyword] = ACTIONS(1773),
    [sym_define_module_keyword] = ACTIONS(1773),
    [sym_char_keyword] = ACTIONS(1773),
    [sym_widechar_keyword] = ACTIONS(1773),
    [sym_integer_keyword] = ACTIONS(1773),
    [sym_sinteger_keyword] = ACTIONS(1773),
    [sym_long_keyword] = ACTIONS(1773),
    [sym_slong_keyword] = ACTIONS(1773),
    [sym_float_keyword] = ACTIONS(1773),
    [sym_double_keyword] = ACTIONS(1773),
    [sym_variant_keyword] = ACTIONS(1773),
    [sym_variantarray_keyword] = ACTIONS(1773),
    [sym_local_var_keyword] = ACTIONS(1773),
    [sym_stack_var_keyword] = ACTIONS(1773),
    [sym_constant_keyword] = ACTIONS(1773),
    [sym_volatile_keyword] = ACTIONS(1773),
    [sym_non_volatile_keyword] = ACTIONS(1773),
    [sym_persistent_keyword] = ACTIONS(1773),
    [sym_dev_keyword] = ACTIONS(1773),
    [sym_devlev_keyword] = ACTIONS(1773),
    [sym_devchan_keyword] = ACTIONS(1773),
    [sym_if_keyword] = ACTIONS(1773),
    [sym_else_keyword] = ACTIONS(1773),
    [sym_switch_keyword] = ACTIONS(1773),
    [sym_case_keyword] = ACTIONS(1773),
    [sym_default_keyword] = ACTIONS(1773),
    [sym_while_keyword] = ACTIONS(1773),
    [sym_for_keyword] = ACTIONS(1773),
    [sym_break_keyword] = ACTIONS(1773),
    [sym_continue_keyword] = ACTIONS(1773),
    [sym_return_keyword] = ACTIONS(1773),
    [sym_select_keyword] = ACTIONS(1773),
    [sym_active_keyword] = ACTIONS(1773),
    [sym_struct_keyword] = ACTIONS(1773),
    [sym_structure_keyword] = ACTIONS(1773),
    [sym_button_event_keyword] = ACTIONS(1773),
    [sym_channel_event_keyword] = ACTIONS(1773),
    [sym_level_event_keyword] = ACTIONS(1773),
    [sym_data_event_keyword] = ACTIONS(1773),
    [sym_timeline_event_keyword] = ACTIONS(1773),
    [sym_custom_event_keyword] = ACTIONS(1773),
    [sym_push_keyword] = ACTIONS(1773),
    [sym_release_keyword] = ACTIONS(1773),
    [aux_sym_on_keyword_token1] = ACTIONS(1773),
    [aux_sym_off_keyword_token1] = ACTIONS(1773),
    [sym_send_level_keyword] = ACTIONS(1773),
    [sym_send_string_keyword] = ACTIONS(1773),
    [sym_send_command_keyword] = ACTIONS(1773),
    [sym_clear_buffer_keyword] = ACTIONS(1773),
    [sym_create_buffer_keyword] = ACTIONS(1773),
    [sym_create_multi_buffer_keyword] = ACTIONS(1773),
    [sym_call_keyword] = ACTIONS(1773),
    [sym_system_call_keyword] = ACTIONS(1773),
    [sym_devchan_to_keyword] = ACTIONS(1773),
    [sym_devchan_min_to_keyword] = ACTIONS(1773),
    [sym_devchan_total_off_keyword] = ACTIONS(1773),
    [sym_devchan_pulse_keyword] = ACTIONS(1773),
    [sym_wait_keyword] = ACTIONS(1773),
    [sym_cancel_wait_keyword] = ACTIONS(1773),
    [sym_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1773),
    [sym_not] = ACTIONS(1773),
    [sym_bnot] = ACTIONS(1773),
    [sym___file__] = ACTIONS(1773),
    [sym___date__] = ACTIONS(1773),
    [sym___ldate__] = ACTIONS(1773),
    [sym___time__] = ACTIONS(1773),
    [sym___line__] = ACTIONS(1773),
    [sym___name__] = ACTIONS(1773),
    [sym___netlinx__] = ACTIONS(1773),
    [sym_netlinx_axi_version] = ACTIONS(1773),
    [sym_url_flg_tcp] = ACTIONS(1773),
    [sym_url_flg_accinfopresent] = ACTIONS(1773),
    [sym_url_flg_temp] = ACTIONS(1773),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1773),
    [sym_url_flg_stat_mask] = ACTIONS(1773),
    [sym_url_flg_stat_lookup] = ACTIONS(1773),
    [sym_url_flg_stat_connecting] = ACTIONS(1773),
    [sym_url_flg_stat_waiting] = ACTIONS(1773),
    [sym_url_flg_stat_connected] = ACTIONS(1773),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1773),
    [sym_first_virtual_device] = ACTIONS(1773),
    [sym_first_local_port] = ACTIONS(1773),
    [sym_dynamic_virtual_device] = ACTIONS(1773),
    [sym_timeline_once] = ACTIONS(1773),
    [sym_timeline_repeat] = ACTIONS(1773),
    [sym_timeline_absolute] = ACTIONS(1773),
    [sym_timeline_relative] = ACTIONS(1773),
    [sym_do_push_timed_infinite] = ACTIONS(1773),
    [sym_source_type_no_address] = ACTIONS(1773),
    [sym_source_type_neuron_id] = ACTIONS(1773),
    [sym_source_type_ip_address] = ACTIONS(1773),
    [sym_source_type_axlink] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1773),
    [sym_source_type_ip_socket_address] = ACTIONS(1773),
    [sym_source_type_rs232] = ACTIONS(1773),
    [sym_source_type_internal] = ACTIONS(1773),
    [sym_source_type_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1773),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1773),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1773),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1773),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_file_read_only] = ACTIONS(1773),
    [sym_file_rw_new] = ACTIONS(1773),
    [sym_file_rw_append] = ACTIONS(1773),
    [sym_ip_tcp] = ACTIONS(1773),
    [sym_ip_udp] = ACTIONS(1773),
    [sym_ip_udp_2way] = ACTIONS(1773),
    [sym_xml_encode_types] = ACTIONS(1773),
    [sym_xml_encode_char_as_list] = ACTIONS(1773),
    [sym_xml_encode_le] = ACTIONS(1773),
    [sym_xml_decode_types] = ACTIONS(1773),
    [sym_xml_decode_no_preserve] = ACTIONS(1773),
    [sym_duet_dev_type_display_device] = ACTIONS(1773),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_io_device] = ACTIONS(1773),
    [sym_duet_dev_type_relay_device] = ACTIONS(1773),
    [sym_duet_dev_type_ups] = ACTIONS(1773),
    [sym_duet_dev_type_amplifier] = ACTIONS(1773),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1773),
    [sym_duet_dev_type_camera] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1773),
    [sym_duet_dev_type_dss] = ACTIONS(1773),
    [sym_duet_dev_type_dvr] = ACTIONS(1773),
    [sym_duet_dev_type_disc_device] = ACTIONS(1773),
    [sym_duet_dev_type_document_camera] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1773),
    [sym_duet_dev_type_hvac] = ACTIONS(1773),
    [sym_duet_dev_type_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_light] = ACTIONS(1773),
    [sym_duet_dev_type_monitor] = ACTIONS(1773),
    [sym_duet_dev_type_motor] = ACTIONS(1773),
    [sym_duet_dev_type_multi_window] = ACTIONS(1773),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1773),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1773),
    [sym_duet_dev_type_receiver] = ACTIONS(1773),
    [sym_duet_dev_type_security_system] = ACTIONS(1773),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1773),
    [sym_duet_dev_type_settop_box] = ACTIONS(1773),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1773),
    [sym_duet_dev_type_switcher] = ACTIONS(1773),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_tv] = ACTIONS(1773),
    [sym_duet_dev_type_utility] = ACTIONS(1773),
    [sym_duet_dev_type_vcr] = ACTIONS(1773),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_video_processor] = ACTIONS(1773),
    [sym_duet_dev_type_video_projector] = ACTIONS(1773),
    [sym_duet_dev_type_video_wall] = ACTIONS(1773),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1773),
    [sym_duet_dev_type_weather] = ACTIONS(1773),
    [sym_duet_dev_not_polled] = ACTIONS(1773),
    [sym_duet_dev_polled] = ACTIONS(1773),
    [sym_clkmgr_mode_network] = ACTIONS(1773),
    [sym_clkmgr_mode_standalone] = ACTIONS(1773),
    [sym_normal_standby] = ACTIONS(1773),
    [sym_normal_wake] = ACTIONS(1773),
    [sym_ip_multicast_ttl_option] = ACTIONS(1773),
    [sym_ip_ttl_subnet] = ACTIONS(1773),
    [sym_ip_ttl_site] = ACTIONS(1773),
    [sym_ip_ttl_region] = ACTIONS(1773),
    [sym_ip_ttl_continent] = ACTIONS(1773),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1773),
    [sym_ip_nodelay_on] = ACTIONS(1773),
    [sym_ip_nodelay_off] = ACTIONS(1773),
    [sym_amx_error] = ACTIONS(1773),
    [sym_amx_warning] = ACTIONS(1773),
    [sym_amx_info] = ACTIONS(1773),
    [sym_amx_debug] = ACTIONS(1773),
    [sym_netlinx_login_success] = ACTIONS(1773),
    [sym_netlinx_login_fail] = ACTIONS(1773),
    [sym_netlinx_logout] = ACTIONS(1773),
    [sym_null_str] = ACTIONS(1773),
    [sym_smtp_address] = ACTIONS(1773),
    [sym_smtp_port_number] = ACTIONS(1773),
    [sym_smtp_username] = ACTIONS(1773),
    [sym_smtp_password] = ACTIONS(1773),
    [sym_smtp_require_tls] = ACTIONS(1773),
    [sym_smtp_from] = ACTIONS(1773),
    [sym_smtp_tls_true] = ACTIONS(1773),
    [sym_smtp_tls_false] = ACTIONS(1773),
    [sym_listview_on_row_select_event] = ACTIONS(1773),
    [sym_data_structure_unknown] = ACTIONS(1773),
    [sym_data_structure_datafeed] = ACTIONS(1773),
    [sym_data_structure_datarecordset] = ACTIONS(1773),
    [sym_data_structure_datarecord] = ACTIONS(1773),
    [sym_data_structure_datafield] = ACTIONS(1773),
    [sym_data_max_content_field_count] = ACTIONS(1773),
    [sym_data_max_metadata_field_count] = ACTIONS(1773),
    [sym_data_max_id_length] = ACTIONS(1773),
    [sym_data_max_name_length] = ACTIONS(1773),
    [sym_data_max_description_length] = ACTIONS(1773),
    [sym_data_max_type_length] = ACTIONS(1773),
    [sym_data_max_format_length] = ACTIONS(1773),
    [sym_data_max_source_length] = ACTIONS(1773),
    [sym_data_max_value_length] = ACTIONS(1773),
    [sym_data_max_label_length] = ACTIONS(1773),
    [sym_data_max_url_length] = ACTIONS(1773),
    [sym_max_last_login_info_length] = ACTIONS(1773),
    [sym_data_type_unknown] = ACTIONS(1773),
    [sym_data_type_string] = ACTIONS(1773),
    [sym_data_type_datetime] = ACTIONS(1773),
    [sym_data_type_date] = ACTIONS(1773),
    [sym_data_type_time] = ACTIONS(1773),
    [sym_data_type_image] = ACTIONS(1773),
    [sym_data_format_url] = ACTIONS(1773),
    [sym_data_format_phone] = ACTIONS(1773),
    [sym_data_format_email] = ACTIONS(1773),
    [sym_data_format_iso8601] = ACTIONS(1773),
    [sym_auditlog_priv] = ACTIONS(1773),
    [sym_configuration_priv] = ACTIONS(1773),
    [sym_deviceconfig_priv] = ACTIONS(1773),
    [sym_ftp_priv] = ACTIONS(1773),
    [sym_http_priv] = ACTIONS(1773),
    [sym_networkconfig_priv] = ACTIONS(1773),
    [sym_programport_priv] = ACTIONS(1773),
    [sym_remoteui_priv] = ACTIONS(1773),
    [sym_securitycontrol_priv] = ACTIONS(1773),
    [sym_softwaremanagement_priv] = ACTIONS(1773),
    [sym_terminal_priv] = ACTIONS(1773),
    [sym_tpadmin_priv] = ACTIONS(1773),
    [sym_user1_priv] = ACTIONS(1773),
    [sym_user2_priv] = ACTIONS(1773),
    [sym_user3_priv] = ACTIONS(1773),
    [sym_user4_priv] = ACTIONS(1773),
    [sym_usermanagement_priv] = ACTIONS(1773),
    [sym_valid_account] = ACTIONS(1773),
    [sym_err_name_invalid] = ACTIONS(1773),
    [sym_err_password_invalid] = ACTIONS(1773),
    [sym_err_invalid_account] = ACTIONS(1773),
    [sym_err_no_authorization] = ACTIONS(1773),
    [sym_err_invalid_parameter] = ACTIONS(1773),
    [sym_err_user_locked_out] = ACTIONS(1773),
    [sym_err_user_account_expired] = ACTIONS(1773),
    [sym_err_general_error] = ACTIONS(1773),
    [sym_err_authentication_error] = ACTIONS(1773),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1773),
    [sym_tls_validate_certificate] = ACTIONS(1773),
    [sym_tdata] = ACTIONS(1773),
    [sym_tchannel] = ACTIONS(1773),
    [sym_tlevel] = ACTIONS(1773),
    [sym_tbutton] = ACTIONS(1773),
    [sym_ttimeline] = ACTIONS(1773),
    [sym_tcustom] = ACTIONS(1773),
    [sym_url_struct] = ACTIONS(1773),
    [sym_dns_struct] = ACTIONS(1773),
    [sym_ip_address_struct] = ACTIONS(1773),
    [sym_dev_info_struct] = ACTIONS(1773),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1773),
    [sym_data_feed] = ACTIONS(1773),
    [sym_data_field] = ACTIONS(1773),
    [sym_data_record] = ACTIONS(1773),
    [sym_wc_data_feed] = ACTIONS(1773),
    [sym_wc_data_field] = ACTIONS(1773),
    [sym_wc_data_record] = ACTIONS(1773),
    [sym_last_login_info] = ACTIONS(1773),
    [sym____reserved___] = ACTIONS(1773),
    [sym_date] = ACTIONS(1773),
    [sym_day] = ACTIONS(1773),
    [sym_time] = ACTIONS(1773),
    [sym_ldate] = ACTIONS(1773),
    [sym_push_device] = ACTIONS(1773),
    [sym_push_channel] = ACTIONS(1773),
    [sym_push_devchan] = ACTIONS(1773),
    [sym_release_device] = ACTIONS(1773),
    [sym_release_channel] = ACTIONS(1773),
    [sym_release_devchan] = ACTIONS(1773),
    [sym_master_slot] = ACTIONS(1773),
    [sym_get_pulse_time] = ACTIONS(1773),
    [sym_get_timer] = ACTIONS(1773),
    [sym_master_sn] = ACTIONS(1773),
    [sym_system_number] = ACTIONS(1773),
    [sym_button] = ACTIONS(1773),
    [sym_dv_channel] = ACTIONS(1773),
    [sym_data] = ACTIONS(1773),
    [sym_level] = ACTIONS(1773),
    [sym_timeline] = ACTIONS(1773),
    [sym_channel] = ACTIONS(1773),
    [sym_custom] = ACTIONS(1773),
    [sym_atoi] = ACTIONS(1773),
    [sym_atol] = ACTIONS(1773),
    [sym_atof] = ACTIONS(1773),
    [sym_itoa] = ACTIONS(1773),
    [sym_itohex] = ACTIONS(1773),
    [sym_hextoi] = ACTIONS(1773),
    [sym_ftoa] = ACTIONS(1773),
    [sym_find_string] = ACTIONS(1773),
    [sym_left_string] = ACTIONS(1773),
    [sym_right_string] = ACTIONS(1773),
    [sym_mid_string] = ACTIONS(1773),
    [sym_remove_string] = ACTIONS(1773),
    [sym_get_buffer_string] = ACTIONS(1773),
    [sym_get_buffer_char] = ACTIONS(1773),
    [sym_get_multi_buffer_string] = ACTIONS(1773),
    [sym_length_string] = ACTIONS(1773),
    [sym_max_length_string] = ACTIONS(1773),
    [sym_set_length_string] = ACTIONS(1773),
    [sym_compare_string] = ACTIONS(1773),
    [sym_upper_string] = ACTIONS(1773),
    [sym_lower_string] = ACTIONS(1773),
    [sym_get_last] = ACTIONS(1773),
    [sym_date_to_day] = ACTIONS(1773),
    [sym_date_to_month] = ACTIONS(1773),
    [sym_date_to_year] = ACTIONS(1773),
    [sym_time_to_hour] = ACTIONS(1773),
    [sym_time_to_minute] = ACTIONS(1773),
    [sym_time_to_second] = ACTIONS(1773),
    [sym_day_of_week] = ACTIONS(1773),
    [sym_set_pulse_time] = ACTIONS(1773),
    [sym_set_timer] = ACTIONS(1773),
    [sym_file_open] = ACTIONS(1773),
    [sym_file_close] = ACTIONS(1773),
    [sym_file_read] = ACTIONS(1773),
    [sym_file_read_line] = ACTIONS(1773),
    [sym_file_write] = ACTIONS(1773),
    [sym_file_write_line] = ACTIONS(1773),
    [sym_file_copy] = ACTIONS(1773),
    [sym_file_delete] = ACTIONS(1773),
    [sym_file_rename] = ACTIONS(1773),
    [sym_file_dir] = ACTIONS(1773),
    [sym_file_getdir] = ACTIONS(1773),
    [sym_file_setdir] = ACTIONS(1773),
    [sym_file_create_dir] = ACTIONS(1773),
    [sym_file_remove_dir] = ACTIONS(1773),
    [sym_file_seek] = ACTIONS(1773),
    [sym_length_array] = ACTIONS(1773),
    [sym_max_length_array] = ACTIONS(1773),
    [sym_set_length_array] = ACTIONS(1773),
    [sym_ip_client_open] = ACTIONS(1773),
    [sym_ip_client_close] = ACTIONS(1773),
    [sym_ip_server_open] = ACTIONS(1773),
    [sym_ip_server_close] = ACTIONS(1773),
    [sym_ip_bound_client_open] = ACTIONS(1773),
    [sym_ip_mc_server_open] = ACTIONS(1773),
    [sym_ip_set_option] = ACTIONS(1773),
    [sym_ssh_client_open] = ACTIONS(1773),
    [sym_ssh_client_close] = ACTIONS(1773),
    [sym_tls_client_open] = ACTIONS(1773),
    [sym_tls_client_close] = ACTIONS(1773),
    [sym_do_push] = ACTIONS(1773),
    [sym_do_push_timed] = ACTIONS(1773),
    [sym_do_release] = ACTIONS(1773),
    [sym_device_id] = ACTIONS(1773),
    [sym_device_info] = ACTIONS(1773),
    [sym_device_id_string] = ACTIONS(1773),
    [sym_device_standby] = ACTIONS(1773),
    [sym_device_wake] = ACTIONS(1773),
    [sym_timeline_create] = ACTIONS(1773),
    [sym_timeline_kill] = ACTIONS(1773),
    [sym_timeline_reload] = ACTIONS(1773),
    [sym_timeline_pause] = ACTIONS(1773),
    [sym_timeline_restart] = ACTIONS(1773),
    [sym_timeline_set] = ACTIONS(1773),
    [sym_timeline_get] = ACTIONS(1773),
    [sym_timeline_active] = ACTIONS(1773),
    [sym_timeline_dynamic_id] = ACTIONS(1773),
    [sym_variable_to_string] = ACTIONS(1773),
    [sym_string_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_string] = ACTIONS(1773),
    [sym_type_cast] = ACTIONS(1773),
    [sym_variable_to_xml] = ACTIONS(1773),
    [sym_xml_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_xml] = ACTIONS(1773),
    [sym_redirect_string] = ACTIONS(1773),
    [sym_random_number] = ACTIONS(1773),
    [sym_get_unique_id] = ACTIONS(1773),
    [sym_get_system_number] = ACTIONS(1773),
    [sym_set_system_number] = ACTIONS(1773),
    [sym_get_serial_number] = ACTIONS(1773),
    [sym_get_master_build] = ACTIONS(1773),
    [sym_set_validation_code] = ACTIONS(1773),
    [sym_reboot] = ACTIONS(1773),
    [sym_load_duet_module] = ACTIONS(1773),
    [sym_unload_duet_module] = ACTIONS(1773),
    [sym_duet_mem_size_set] = ACTIONS(1773),
    [sym_duet_mem_size_get] = ACTIONS(1773),
    [sym_dynamic_application_device] = ACTIONS(1773),
    [sym_dynamic_polled_port] = ACTIONS(1773),
    [sym_static_port_binding] = ACTIONS(1773),
    [sym_static_ip_binding] = ACTIONS(1773),
    [sym_get_dns_list] = ACTIONS(1773),
    [sym_set_dns_list] = ACTIONS(1773),
    [sym_get_ip_address] = ACTIONS(1773),
    [sym_set_ip_address] = ACTIONS(1773),
    [sym_get_url_list] = ACTIONS(1773),
    [sym_add_url_entry] = ACTIONS(1773),
    [sym_delete_url_entry] = ACTIONS(1773),
    [sym_abs_value] = ACTIONS(1773),
    [sym_max_value] = ACTIONS(1773),
    [sym_min_value] = ACTIONS(1773),
    [sym_exp_value] = ACTIONS(1773),
    [sym_log_value] = ACTIONS(1773),
    [sym_log10_value] = ACTIONS(1773),
    [sym_power_value] = ACTIONS(1773),
    [sym_sqrt_value] = ACTIONS(1773),
    [sym_raw_be] = ACTIONS(1773),
    [sym_raw_le] = ACTIONS(1773),
    [sym_internal_queue_size_set] = ACTIONS(1773),
    [sym_internal_queue_size_get] = ACTIONS(1773),
    [sym_internal_threshold_set] = ACTIONS(1773),
    [sym_internal_threshold_get] = ACTIONS(1773),
    [sym_rebuilt_event] = ACTIONS(1773),
    [sym_do_custom_event] = ACTIONS(1773),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1773),
    [sym_clkmgr_set_clk_source] = ACTIONS(1773),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1773),
    [sym_clkmgr_get_timezone] = ACTIONS(1773),
    [sym_clkmgr_set_timezone] = ACTIONS(1773),
    [sym_clkmgr_get_resync_period] = ACTIONS(1773),
    [sym_clkmgr_set_resync_period] = ACTIONS(1773),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_timeservers] = ACTIONS(1773),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_set_log_level] = ACTIONS(1773),
    [sym_get_log_level] = ACTIONS(1773),
    [sym_amx_log] = ACTIONS(1773),
    [sym_smtp_server_config_set] = ACTIONS(1773),
    [sym_smtp_server_config_get] = ACTIONS(1773),
    [sym_smtp_send] = ACTIONS(1773),
    [sym_validate_netlinx_account] = ACTIONS(1773),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1773),
    [sym_authenticate_certificate] = ACTIONS(1773),
    [sym_audit_netlinx_session_event] = ACTIONS(1773),
    [sym_audit_netlinx_generic_event] = ACTIONS(1773),
    [sym_get_available_flash_disk_space] = ACTIONS(1773),
    [sym_get_max_flash_disk_space] = ACTIONS(1773),
    [sym_data_create_feed] = ACTIONS(1773),
    [sym_data_delete_feed] = ACTIONS(1773),
    [sym_data_publish_feed] = ACTIONS(1773),
    [sym_data_get_published_feed] = ACTIONS(1773),
    [sym_data_add_record] = ACTIONS(1773),
    [sym_data_get_event_record] = ACTIONS(1773),
    [sym__wc_data_create_feed] = ACTIONS(1773),
    [sym__wc_data_add_record] = ACTIONS(1773),
    [sym__wc_data_get_event_record] = ACTIONS(1773),
    [sym_format] = ACTIONS(1773),
    [sym_astro_clock] = ACTIONS(1773),
    [sym_set_outdoor_temperature] = ACTIONS(1773),
    [sym_set_virtual_level_count] = ACTIONS(1773),
    [sym_set_virtual_channel_count] = ACTIONS(1773),
    [sym_set_virtual_port_count] = ACTIONS(1773),
    [sym__wc_encode_int] = ACTIONS(1773),
    [sym__wc_decode_int] = ACTIONS(1773),
    [sym__wc_int] = ACTIONS(1773),
    [sym__wc_to_ch_int] = ACTIONS(1773),
    [sym__ch_to_wc_int] = ACTIONS(1773),
    [sym__wc_find_string_int] = ACTIONS(1773),
    [sym__wc_left_string_int] = ACTIONS(1773),
    [sym__wc_lower_string_int] = ACTIONS(1773),
    [sym__wc_mid_string_int] = ACTIONS(1773),
    [sym__wc_remove_string_int] = ACTIONS(1773),
    [sym__wc_right_string_int] = ACTIONS(1773),
    [sym__wc_upper_string_int] = ACTIONS(1773),
    [sym__wc_compare_string_int] = ACTIONS(1773),
    [sym__wc_get_buffer_char_int] = ACTIONS(1773),
    [sym__wc_get_buffer_string_int] = ACTIONS(1773),
    [sym__wc_concat_string_int] = ACTIONS(1773),
    [sym__wc_file_open_int] = ACTIONS(1773),
    [sym__wc_file_close_int] = ACTIONS(1773),
    [sym__wc_file_read_int] = ACTIONS(1773),
    [sym__wc_file_read_line_int] = ACTIONS(1773),
    [sym__wc_file_write_int] = ACTIONS(1773),
    [sym__wc_file_write_line_int] = ACTIONS(1773),
    [aux_sym_preproc_include_token1] = ACTIONS(1773),
    [aux_sym_preproc_define_token1] = ACTIONS(1775),
    [aux_sym_preproc_warn_token1] = ACTIONS(1775),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1775),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_else_token1] = ACTIONS(1775),
    [anon_sym_LBRACK] = ACTIONS(1775),
    [anon_sym_LBRACE] = ACTIONS(1775),
    [anon_sym_LPAREN] = ACTIONS(1773),
    [anon_sym_BANG] = ACTIONS(1775),
    [anon_sym_TILDE] = ACTIONS(1775),
    [anon_sym_DASH] = ACTIONS(1773),
    [anon_sym_PLUS] = ACTIONS(1773),
    [anon_sym_DASH_DASH] = ACTIONS(1775),
    [anon_sym_PLUS_PLUS] = ACTIONS(1775),
    [anon_sym_DQUOTE] = ACTIONS(1775),
    [anon_sym_SQUOTE] = ACTIONS(1775),
    [sym_number_literal] = ACTIONS(1775),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(227)] = {
    [sym_identifier] = ACTIONS(1773),
    [sym_program_name_keyword] = ACTIONS(1773),
    [sym_module_name_keyword] = ACTIONS(1773),
    [sym_define_device_keyword] = ACTIONS(1773),
    [sym_define_combine_keyword] = ACTIONS(1773),
    [sym_define_constant_keyword] = ACTIONS(1773),
    [sym_define_type_keyword] = ACTIONS(1773),
    [sym_define_variable_keyword] = ACTIONS(1773),
    [sym_define_system_variable_keyword] = ACTIONS(1773),
    [sym_define_connect_level_keyword] = ACTIONS(1773),
    [sym_define_latching_keyword] = ACTIONS(1773),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1773),
    [sym_define_toggling_keyword] = ACTIONS(1773),
    [sym_define_call_keyword] = ACTIONS(1773),
    [sym_define_function_keyword] = ACTIONS(1773),
    [sym_define_library_function_keyword] = ACTIONS(1773),
    [sym_define_start_keyword] = ACTIONS(1773),
    [sym_define_event_keyword] = ACTIONS(1773),
    [sym_define_program_keyword] = ACTIONS(1773),
    [sym_define_module_keyword] = ACTIONS(1773),
    [sym_char_keyword] = ACTIONS(1773),
    [sym_widechar_keyword] = ACTIONS(1773),
    [sym_integer_keyword] = ACTIONS(1773),
    [sym_sinteger_keyword] = ACTIONS(1773),
    [sym_long_keyword] = ACTIONS(1773),
    [sym_slong_keyword] = ACTIONS(1773),
    [sym_float_keyword] = ACTIONS(1773),
    [sym_double_keyword] = ACTIONS(1773),
    [sym_variant_keyword] = ACTIONS(1773),
    [sym_variantarray_keyword] = ACTIONS(1773),
    [sym_local_var_keyword] = ACTIONS(1773),
    [sym_stack_var_keyword] = ACTIONS(1773),
    [sym_constant_keyword] = ACTIONS(1773),
    [sym_volatile_keyword] = ACTIONS(1773),
    [sym_non_volatile_keyword] = ACTIONS(1773),
    [sym_persistent_keyword] = ACTIONS(1773),
    [sym_dev_keyword] = ACTIONS(1773),
    [sym_devlev_keyword] = ACTIONS(1773),
    [sym_devchan_keyword] = ACTIONS(1773),
    [sym_if_keyword] = ACTIONS(1773),
    [sym_else_keyword] = ACTIONS(1773),
    [sym_switch_keyword] = ACTIONS(1773),
    [sym_case_keyword] = ACTIONS(1773),
    [sym_default_keyword] = ACTIONS(1773),
    [sym_while_keyword] = ACTIONS(1773),
    [sym_for_keyword] = ACTIONS(1773),
    [sym_break_keyword] = ACTIONS(1773),
    [sym_continue_keyword] = ACTIONS(1773),
    [sym_return_keyword] = ACTIONS(1773),
    [sym_select_keyword] = ACTIONS(1773),
    [sym_active_keyword] = ACTIONS(1773),
    [sym_struct_keyword] = ACTIONS(1773),
    [sym_structure_keyword] = ACTIONS(1773),
    [sym_button_event_keyword] = ACTIONS(1773),
    [sym_channel_event_keyword] = ACTIONS(1773),
    [sym_level_event_keyword] = ACTIONS(1773),
    [sym_data_event_keyword] = ACTIONS(1773),
    [sym_timeline_event_keyword] = ACTIONS(1773),
    [sym_custom_event_keyword] = ACTIONS(1773),
    [sym_push_keyword] = ACTIONS(1773),
    [sym_release_keyword] = ACTIONS(1773),
    [aux_sym_on_keyword_token1] = ACTIONS(1773),
    [aux_sym_off_keyword_token1] = ACTIONS(1773),
    [sym_send_level_keyword] = ACTIONS(1773),
    [sym_send_string_keyword] = ACTIONS(1773),
    [sym_send_command_keyword] = ACTIONS(1773),
    [sym_clear_buffer_keyword] = ACTIONS(1773),
    [sym_create_buffer_keyword] = ACTIONS(1773),
    [sym_create_multi_buffer_keyword] = ACTIONS(1773),
    [sym_call_keyword] = ACTIONS(1773),
    [sym_system_call_keyword] = ACTIONS(1773),
    [sym_devchan_to_keyword] = ACTIONS(1773),
    [sym_devchan_min_to_keyword] = ACTIONS(1773),
    [sym_devchan_total_off_keyword] = ACTIONS(1773),
    [sym_devchan_pulse_keyword] = ACTIONS(1773),
    [sym_wait_keyword] = ACTIONS(1773),
    [sym_cancel_wait_keyword] = ACTIONS(1773),
    [sym_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1773),
    [sym_not] = ACTIONS(1773),
    [sym_bnot] = ACTIONS(1773),
    [sym___file__] = ACTIONS(1773),
    [sym___date__] = ACTIONS(1773),
    [sym___ldate__] = ACTIONS(1773),
    [sym___time__] = ACTIONS(1773),
    [sym___line__] = ACTIONS(1773),
    [sym___name__] = ACTIONS(1773),
    [sym___netlinx__] = ACTIONS(1773),
    [sym_netlinx_axi_version] = ACTIONS(1773),
    [sym_url_flg_tcp] = ACTIONS(1773),
    [sym_url_flg_accinfopresent] = ACTIONS(1773),
    [sym_url_flg_temp] = ACTIONS(1773),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1773),
    [sym_url_flg_stat_mask] = ACTIONS(1773),
    [sym_url_flg_stat_lookup] = ACTIONS(1773),
    [sym_url_flg_stat_connecting] = ACTIONS(1773),
    [sym_url_flg_stat_waiting] = ACTIONS(1773),
    [sym_url_flg_stat_connected] = ACTIONS(1773),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1773),
    [sym_first_virtual_device] = ACTIONS(1773),
    [sym_first_local_port] = ACTIONS(1773),
    [sym_dynamic_virtual_device] = ACTIONS(1773),
    [sym_timeline_once] = ACTIONS(1773),
    [sym_timeline_repeat] = ACTIONS(1773),
    [sym_timeline_absolute] = ACTIONS(1773),
    [sym_timeline_relative] = ACTIONS(1773),
    [sym_do_push_timed_infinite] = ACTIONS(1773),
    [sym_source_type_no_address] = ACTIONS(1773),
    [sym_source_type_neuron_id] = ACTIONS(1773),
    [sym_source_type_ip_address] = ACTIONS(1773),
    [sym_source_type_axlink] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1773),
    [sym_source_type_ip_socket_address] = ACTIONS(1773),
    [sym_source_type_rs232] = ACTIONS(1773),
    [sym_source_type_internal] = ACTIONS(1773),
    [sym_source_type_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1773),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1773),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1773),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1773),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_file_read_only] = ACTIONS(1773),
    [sym_file_rw_new] = ACTIONS(1773),
    [sym_file_rw_append] = ACTIONS(1773),
    [sym_ip_tcp] = ACTIONS(1773),
    [sym_ip_udp] = ACTIONS(1773),
    [sym_ip_udp_2way] = ACTIONS(1773),
    [sym_xml_encode_types] = ACTIONS(1773),
    [sym_xml_encode_char_as_list] = ACTIONS(1773),
    [sym_xml_encode_le] = ACTIONS(1773),
    [sym_xml_decode_types] = ACTIONS(1773),
    [sym_xml_decode_no_preserve] = ACTIONS(1773),
    [sym_duet_dev_type_display_device] = ACTIONS(1773),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_io_device] = ACTIONS(1773),
    [sym_duet_dev_type_relay_device] = ACTIONS(1773),
    [sym_duet_dev_type_ups] = ACTIONS(1773),
    [sym_duet_dev_type_amplifier] = ACTIONS(1773),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1773),
    [sym_duet_dev_type_camera] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1773),
    [sym_duet_dev_type_dss] = ACTIONS(1773),
    [sym_duet_dev_type_dvr] = ACTIONS(1773),
    [sym_duet_dev_type_disc_device] = ACTIONS(1773),
    [sym_duet_dev_type_document_camera] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1773),
    [sym_duet_dev_type_hvac] = ACTIONS(1773),
    [sym_duet_dev_type_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_light] = ACTIONS(1773),
    [sym_duet_dev_type_monitor] = ACTIONS(1773),
    [sym_duet_dev_type_motor] = ACTIONS(1773),
    [sym_duet_dev_type_multi_window] = ACTIONS(1773),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1773),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1773),
    [sym_duet_dev_type_receiver] = ACTIONS(1773),
    [sym_duet_dev_type_security_system] = ACTIONS(1773),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1773),
    [sym_duet_dev_type_settop_box] = ACTIONS(1773),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1773),
    [sym_duet_dev_type_switcher] = ACTIONS(1773),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_tv] = ACTIONS(1773),
    [sym_duet_dev_type_utility] = ACTIONS(1773),
    [sym_duet_dev_type_vcr] = ACTIONS(1773),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_video_processor] = ACTIONS(1773),
    [sym_duet_dev_type_video_projector] = ACTIONS(1773),
    [sym_duet_dev_type_video_wall] = ACTIONS(1773),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1773),
    [sym_duet_dev_type_weather] = ACTIONS(1773),
    [sym_duet_dev_not_polled] = ACTIONS(1773),
    [sym_duet_dev_polled] = ACTIONS(1773),
    [sym_clkmgr_mode_network] = ACTIONS(1773),
    [sym_clkmgr_mode_standalone] = ACTIONS(1773),
    [sym_normal_standby] = ACTIONS(1773),
    [sym_normal_wake] = ACTIONS(1773),
    [sym_ip_multicast_ttl_option] = ACTIONS(1773),
    [sym_ip_ttl_subnet] = ACTIONS(1773),
    [sym_ip_ttl_site] = ACTIONS(1773),
    [sym_ip_ttl_region] = ACTIONS(1773),
    [sym_ip_ttl_continent] = ACTIONS(1773),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1773),
    [sym_ip_nodelay_on] = ACTIONS(1773),
    [sym_ip_nodelay_off] = ACTIONS(1773),
    [sym_amx_error] = ACTIONS(1773),
    [sym_amx_warning] = ACTIONS(1773),
    [sym_amx_info] = ACTIONS(1773),
    [sym_amx_debug] = ACTIONS(1773),
    [sym_netlinx_login_success] = ACTIONS(1773),
    [sym_netlinx_login_fail] = ACTIONS(1773),
    [sym_netlinx_logout] = ACTIONS(1773),
    [sym_null_str] = ACTIONS(1773),
    [sym_smtp_address] = ACTIONS(1773),
    [sym_smtp_port_number] = ACTIONS(1773),
    [sym_smtp_username] = ACTIONS(1773),
    [sym_smtp_password] = ACTIONS(1773),
    [sym_smtp_require_tls] = ACTIONS(1773),
    [sym_smtp_from] = ACTIONS(1773),
    [sym_smtp_tls_true] = ACTIONS(1773),
    [sym_smtp_tls_false] = ACTIONS(1773),
    [sym_listview_on_row_select_event] = ACTIONS(1773),
    [sym_data_structure_unknown] = ACTIONS(1773),
    [sym_data_structure_datafeed] = ACTIONS(1773),
    [sym_data_structure_datarecordset] = ACTIONS(1773),
    [sym_data_structure_datarecord] = ACTIONS(1773),
    [sym_data_structure_datafield] = ACTIONS(1773),
    [sym_data_max_content_field_count] = ACTIONS(1773),
    [sym_data_max_metadata_field_count] = ACTIONS(1773),
    [sym_data_max_id_length] = ACTIONS(1773),
    [sym_data_max_name_length] = ACTIONS(1773),
    [sym_data_max_description_length] = ACTIONS(1773),
    [sym_data_max_type_length] = ACTIONS(1773),
    [sym_data_max_format_length] = ACTIONS(1773),
    [sym_data_max_source_length] = ACTIONS(1773),
    [sym_data_max_value_length] = ACTIONS(1773),
    [sym_data_max_label_length] = ACTIONS(1773),
    [sym_data_max_url_length] = ACTIONS(1773),
    [sym_max_last_login_info_length] = ACTIONS(1773),
    [sym_data_type_unknown] = ACTIONS(1773),
    [sym_data_type_string] = ACTIONS(1773),
    [sym_data_type_datetime] = ACTIONS(1773),
    [sym_data_type_date] = ACTIONS(1773),
    [sym_data_type_time] = ACTIONS(1773),
    [sym_data_type_image] = ACTIONS(1773),
    [sym_data_format_url] = ACTIONS(1773),
    [sym_data_format_phone] = ACTIONS(1773),
    [sym_data_format_email] = ACTIONS(1773),
    [sym_data_format_iso8601] = ACTIONS(1773),
    [sym_auditlog_priv] = ACTIONS(1773),
    [sym_configuration_priv] = ACTIONS(1773),
    [sym_deviceconfig_priv] = ACTIONS(1773),
    [sym_ftp_priv] = ACTIONS(1773),
    [sym_http_priv] = ACTIONS(1773),
    [sym_networkconfig_priv] = ACTIONS(1773),
    [sym_programport_priv] = ACTIONS(1773),
    [sym_remoteui_priv] = ACTIONS(1773),
    [sym_securitycontrol_priv] = ACTIONS(1773),
    [sym_softwaremanagement_priv] = ACTIONS(1773),
    [sym_terminal_priv] = ACTIONS(1773),
    [sym_tpadmin_priv] = ACTIONS(1773),
    [sym_user1_priv] = ACTIONS(1773),
    [sym_user2_priv] = ACTIONS(1773),
    [sym_user3_priv] = ACTIONS(1773),
    [sym_user4_priv] = ACTIONS(1773),
    [sym_usermanagement_priv] = ACTIONS(1773),
    [sym_valid_account] = ACTIONS(1773),
    [sym_err_name_invalid] = ACTIONS(1773),
    [sym_err_password_invalid] = ACTIONS(1773),
    [sym_err_invalid_account] = ACTIONS(1773),
    [sym_err_no_authorization] = ACTIONS(1773),
    [sym_err_invalid_parameter] = ACTIONS(1773),
    [sym_err_user_locked_out] = ACTIONS(1773),
    [sym_err_user_account_expired] = ACTIONS(1773),
    [sym_err_general_error] = ACTIONS(1773),
    [sym_err_authentication_error] = ACTIONS(1773),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1773),
    [sym_tls_validate_certificate] = ACTIONS(1773),
    [sym_tdata] = ACTIONS(1773),
    [sym_tchannel] = ACTIONS(1773),
    [sym_tlevel] = ACTIONS(1773),
    [sym_tbutton] = ACTIONS(1773),
    [sym_ttimeline] = ACTIONS(1773),
    [sym_tcustom] = ACTIONS(1773),
    [sym_url_struct] = ACTIONS(1773),
    [sym_dns_struct] = ACTIONS(1773),
    [sym_ip_address_struct] = ACTIONS(1773),
    [sym_dev_info_struct] = ACTIONS(1773),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1773),
    [sym_data_feed] = ACTIONS(1773),
    [sym_data_field] = ACTIONS(1773),
    [sym_data_record] = ACTIONS(1773),
    [sym_wc_data_feed] = ACTIONS(1773),
    [sym_wc_data_field] = ACTIONS(1773),
    [sym_wc_data_record] = ACTIONS(1773),
    [sym_last_login_info] = ACTIONS(1773),
    [sym____reserved___] = ACTIONS(1773),
    [sym_date] = ACTIONS(1773),
    [sym_day] = ACTIONS(1773),
    [sym_time] = ACTIONS(1773),
    [sym_ldate] = ACTIONS(1773),
    [sym_push_device] = ACTIONS(1773),
    [sym_push_channel] = ACTIONS(1773),
    [sym_push_devchan] = ACTIONS(1773),
    [sym_release_device] = ACTIONS(1773),
    [sym_release_channel] = ACTIONS(1773),
    [sym_release_devchan] = ACTIONS(1773),
    [sym_master_slot] = ACTIONS(1773),
    [sym_get_pulse_time] = ACTIONS(1773),
    [sym_get_timer] = ACTIONS(1773),
    [sym_master_sn] = ACTIONS(1773),
    [sym_system_number] = ACTIONS(1773),
    [sym_button] = ACTIONS(1773),
    [sym_dv_channel] = ACTIONS(1773),
    [sym_data] = ACTIONS(1773),
    [sym_level] = ACTIONS(1773),
    [sym_timeline] = ACTIONS(1773),
    [sym_channel] = ACTIONS(1773),
    [sym_custom] = ACTIONS(1773),
    [sym_atoi] = ACTIONS(1773),
    [sym_atol] = ACTIONS(1773),
    [sym_atof] = ACTIONS(1773),
    [sym_itoa] = ACTIONS(1773),
    [sym_itohex] = ACTIONS(1773),
    [sym_hextoi] = ACTIONS(1773),
    [sym_ftoa] = ACTIONS(1773),
    [sym_find_string] = ACTIONS(1773),
    [sym_left_string] = ACTIONS(1773),
    [sym_right_string] = ACTIONS(1773),
    [sym_mid_string] = ACTIONS(1773),
    [sym_remove_string] = ACTIONS(1773),
    [sym_get_buffer_string] = ACTIONS(1773),
    [sym_get_buffer_char] = ACTIONS(1773),
    [sym_get_multi_buffer_string] = ACTIONS(1773),
    [sym_length_string] = ACTIONS(1773),
    [sym_max_length_string] = ACTIONS(1773),
    [sym_set_length_string] = ACTIONS(1773),
    [sym_compare_string] = ACTIONS(1773),
    [sym_upper_string] = ACTIONS(1773),
    [sym_lower_string] = ACTIONS(1773),
    [sym_get_last] = ACTIONS(1773),
    [sym_date_to_day] = ACTIONS(1773),
    [sym_date_to_month] = ACTIONS(1773),
    [sym_date_to_year] = ACTIONS(1773),
    [sym_time_to_hour] = ACTIONS(1773),
    [sym_time_to_minute] = ACTIONS(1773),
    [sym_time_to_second] = ACTIONS(1773),
    [sym_day_of_week] = ACTIONS(1773),
    [sym_set_pulse_time] = ACTIONS(1773),
    [sym_set_timer] = ACTIONS(1773),
    [sym_file_open] = ACTIONS(1773),
    [sym_file_close] = ACTIONS(1773),
    [sym_file_read] = ACTIONS(1773),
    [sym_file_read_line] = ACTIONS(1773),
    [sym_file_write] = ACTIONS(1773),
    [sym_file_write_line] = ACTIONS(1773),
    [sym_file_copy] = ACTIONS(1773),
    [sym_file_delete] = ACTIONS(1773),
    [sym_file_rename] = ACTIONS(1773),
    [sym_file_dir] = ACTIONS(1773),
    [sym_file_getdir] = ACTIONS(1773),
    [sym_file_setdir] = ACTIONS(1773),
    [sym_file_create_dir] = ACTIONS(1773),
    [sym_file_remove_dir] = ACTIONS(1773),
    [sym_file_seek] = ACTIONS(1773),
    [sym_length_array] = ACTIONS(1773),
    [sym_max_length_array] = ACTIONS(1773),
    [sym_set_length_array] = ACTIONS(1773),
    [sym_ip_client_open] = ACTIONS(1773),
    [sym_ip_client_close] = ACTIONS(1773),
    [sym_ip_server_open] = ACTIONS(1773),
    [sym_ip_server_close] = ACTIONS(1773),
    [sym_ip_bound_client_open] = ACTIONS(1773),
    [sym_ip_mc_server_open] = ACTIONS(1773),
    [sym_ip_set_option] = ACTIONS(1773),
    [sym_ssh_client_open] = ACTIONS(1773),
    [sym_ssh_client_close] = ACTIONS(1773),
    [sym_tls_client_open] = ACTIONS(1773),
    [sym_tls_client_close] = ACTIONS(1773),
    [sym_do_push] = ACTIONS(1773),
    [sym_do_push_timed] = ACTIONS(1773),
    [sym_do_release] = ACTIONS(1773),
    [sym_device_id] = ACTIONS(1773),
    [sym_device_info] = ACTIONS(1773),
    [sym_device_id_string] = ACTIONS(1773),
    [sym_device_standby] = ACTIONS(1773),
    [sym_device_wake] = ACTIONS(1773),
    [sym_timeline_create] = ACTIONS(1773),
    [sym_timeline_kill] = ACTIONS(1773),
    [sym_timeline_reload] = ACTIONS(1773),
    [sym_timeline_pause] = ACTIONS(1773),
    [sym_timeline_restart] = ACTIONS(1773),
    [sym_timeline_set] = ACTIONS(1773),
    [sym_timeline_get] = ACTIONS(1773),
    [sym_timeline_active] = ACTIONS(1773),
    [sym_timeline_dynamic_id] = ACTIONS(1773),
    [sym_variable_to_string] = ACTIONS(1773),
    [sym_string_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_string] = ACTIONS(1773),
    [sym_type_cast] = ACTIONS(1773),
    [sym_variable_to_xml] = ACTIONS(1773),
    [sym_xml_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_xml] = ACTIONS(1773),
    [sym_redirect_string] = ACTIONS(1773),
    [sym_random_number] = ACTIONS(1773),
    [sym_get_unique_id] = ACTIONS(1773),
    [sym_get_system_number] = ACTIONS(1773),
    [sym_set_system_number] = ACTIONS(1773),
    [sym_get_serial_number] = ACTIONS(1773),
    [sym_get_master_build] = ACTIONS(1773),
    [sym_set_validation_code] = ACTIONS(1773),
    [sym_reboot] = ACTIONS(1773),
    [sym_load_duet_module] = ACTIONS(1773),
    [sym_unload_duet_module] = ACTIONS(1773),
    [sym_duet_mem_size_set] = ACTIONS(1773),
    [sym_duet_mem_size_get] = ACTIONS(1773),
    [sym_dynamic_application_device] = ACTIONS(1773),
    [sym_dynamic_polled_port] = ACTIONS(1773),
    [sym_static_port_binding] = ACTIONS(1773),
    [sym_static_ip_binding] = ACTIONS(1773),
    [sym_get_dns_list] = ACTIONS(1773),
    [sym_set_dns_list] = ACTIONS(1773),
    [sym_get_ip_address] = ACTIONS(1773),
    [sym_set_ip_address] = ACTIONS(1773),
    [sym_get_url_list] = ACTIONS(1773),
    [sym_add_url_entry] = ACTIONS(1773),
    [sym_delete_url_entry] = ACTIONS(1773),
    [sym_abs_value] = ACTIONS(1773),
    [sym_max_value] = ACTIONS(1773),
    [sym_min_value] = ACTIONS(1773),
    [sym_exp_value] = ACTIONS(1773),
    [sym_log_value] = ACTIONS(1773),
    [sym_log10_value] = ACTIONS(1773),
    [sym_power_value] = ACTIONS(1773),
    [sym_sqrt_value] = ACTIONS(1773),
    [sym_raw_be] = ACTIONS(1773),
    [sym_raw_le] = ACTIONS(1773),
    [sym_internal_queue_size_set] = ACTIONS(1773),
    [sym_internal_queue_size_get] = ACTIONS(1773),
    [sym_internal_threshold_set] = ACTIONS(1773),
    [sym_internal_threshold_get] = ACTIONS(1773),
    [sym_rebuilt_event] = ACTIONS(1773),
    [sym_do_custom_event] = ACTIONS(1773),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1773),
    [sym_clkmgr_set_clk_source] = ACTIONS(1773),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1773),
    [sym_clkmgr_get_timezone] = ACTIONS(1773),
    [sym_clkmgr_set_timezone] = ACTIONS(1773),
    [sym_clkmgr_get_resync_period] = ACTIONS(1773),
    [sym_clkmgr_set_resync_period] = ACTIONS(1773),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_timeservers] = ACTIONS(1773),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_set_log_level] = ACTIONS(1773),
    [sym_get_log_level] = ACTIONS(1773),
    [sym_amx_log] = ACTIONS(1773),
    [sym_smtp_server_config_set] = ACTIONS(1773),
    [sym_smtp_server_config_get] = ACTIONS(1773),
    [sym_smtp_send] = ACTIONS(1773),
    [sym_validate_netlinx_account] = ACTIONS(1773),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1773),
    [sym_authenticate_certificate] = ACTIONS(1773),
    [sym_audit_netlinx_session_event] = ACTIONS(1773),
    [sym_audit_netlinx_generic_event] = ACTIONS(1773),
    [sym_get_available_flash_disk_space] = ACTIONS(1773),
    [sym_get_max_flash_disk_space] = ACTIONS(1773),
    [sym_data_create_feed] = ACTIONS(1773),
    [sym_data_delete_feed] = ACTIONS(1773),
    [sym_data_publish_feed] = ACTIONS(1773),
    [sym_data_get_published_feed] = ACTIONS(1773),
    [sym_data_add_record] = ACTIONS(1773),
    [sym_data_get_event_record] = ACTIONS(1773),
    [sym__wc_data_create_feed] = ACTIONS(1773),
    [sym__wc_data_add_record] = ACTIONS(1773),
    [sym__wc_data_get_event_record] = ACTIONS(1773),
    [sym_format] = ACTIONS(1773),
    [sym_astro_clock] = ACTIONS(1773),
    [sym_set_outdoor_temperature] = ACTIONS(1773),
    [sym_set_virtual_level_count] = ACTIONS(1773),
    [sym_set_virtual_channel_count] = ACTIONS(1773),
    [sym_set_virtual_port_count] = ACTIONS(1773),
    [sym__wc_encode_int] = ACTIONS(1773),
    [sym__wc_decode_int] = ACTIONS(1773),
    [sym__wc_int] = ACTIONS(1773),
    [sym__wc_to_ch_int] = ACTIONS(1773),
    [sym__ch_to_wc_int] = ACTIONS(1773),
    [sym__wc_find_string_int] = ACTIONS(1773),
    [sym__wc_left_string_int] = ACTIONS(1773),
    [sym__wc_lower_string_int] = ACTIONS(1773),
    [sym__wc_mid_string_int] = ACTIONS(1773),
    [sym__wc_remove_string_int] = ACTIONS(1773),
    [sym__wc_right_string_int] = ACTIONS(1773),
    [sym__wc_upper_string_int] = ACTIONS(1773),
    [sym__wc_compare_string_int] = ACTIONS(1773),
    [sym__wc_get_buffer_char_int] = ACTIONS(1773),
    [sym__wc_get_buffer_string_int] = ACTIONS(1773),
    [sym__wc_concat_string_int] = ACTIONS(1773),
    [sym__wc_file_open_int] = ACTIONS(1773),
    [sym__wc_file_close_int] = ACTIONS(1773),
    [sym__wc_file_read_int] = ACTIONS(1773),
    [sym__wc_file_read_line_int] = ACTIONS(1773),
    [sym__wc_file_write_int] = ACTIONS(1773),
    [sym__wc_file_write_line_int] = ACTIONS(1773),
    [aux_sym_preproc_include_token1] = ACTIONS(1773),
    [aux_sym_preproc_define_token1] = ACTIONS(1775),
    [aux_sym_preproc_warn_token1] = ACTIONS(1775),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1775),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_else_token1] = ACTIONS(1775),
    [anon_sym_LBRACK] = ACTIONS(1775),
    [anon_sym_LBRACE] = ACTIONS(1775),
    [anon_sym_LPAREN] = ACTIONS(1773),
    [anon_sym_BANG] = ACTIONS(1775),
    [anon_sym_TILDE] = ACTIONS(1775),
    [anon_sym_DASH] = ACTIONS(1773),
    [anon_sym_PLUS] = ACTIONS(1773),
    [anon_sym_DASH_DASH] = ACTIONS(1775),
    [anon_sym_PLUS_PLUS] = ACTIONS(1775),
    [anon_sym_DQUOTE] = ACTIONS(1775),
    [anon_sym_SQUOTE] = ACTIONS(1775),
    [sym_number_literal] = ACTIONS(1775),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(228)] = {
    [sym_identifier] = ACTIONS(1773),
    [sym_program_name_keyword] = ACTIONS(1773),
    [sym_module_name_keyword] = ACTIONS(1773),
    [sym_define_device_keyword] = ACTIONS(1773),
    [sym_define_combine_keyword] = ACTIONS(1773),
    [sym_define_constant_keyword] = ACTIONS(1773),
    [sym_define_type_keyword] = ACTIONS(1773),
    [sym_define_variable_keyword] = ACTIONS(1773),
    [sym_define_system_variable_keyword] = ACTIONS(1773),
    [sym_define_connect_level_keyword] = ACTIONS(1773),
    [sym_define_latching_keyword] = ACTIONS(1773),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1773),
    [sym_define_toggling_keyword] = ACTIONS(1773),
    [sym_define_call_keyword] = ACTIONS(1773),
    [sym_define_function_keyword] = ACTIONS(1773),
    [sym_define_library_function_keyword] = ACTIONS(1773),
    [sym_define_start_keyword] = ACTIONS(1773),
    [sym_define_event_keyword] = ACTIONS(1773),
    [sym_define_program_keyword] = ACTIONS(1773),
    [sym_define_module_keyword] = ACTIONS(1773),
    [sym_char_keyword] = ACTIONS(1773),
    [sym_widechar_keyword] = ACTIONS(1773),
    [sym_integer_keyword] = ACTIONS(1773),
    [sym_sinteger_keyword] = ACTIONS(1773),
    [sym_long_keyword] = ACTIONS(1773),
    [sym_slong_keyword] = ACTIONS(1773),
    [sym_float_keyword] = ACTIONS(1773),
    [sym_double_keyword] = ACTIONS(1773),
    [sym_variant_keyword] = ACTIONS(1773),
    [sym_variantarray_keyword] = ACTIONS(1773),
    [sym_local_var_keyword] = ACTIONS(1773),
    [sym_stack_var_keyword] = ACTIONS(1773),
    [sym_constant_keyword] = ACTIONS(1773),
    [sym_volatile_keyword] = ACTIONS(1773),
    [sym_non_volatile_keyword] = ACTIONS(1773),
    [sym_persistent_keyword] = ACTIONS(1773),
    [sym_dev_keyword] = ACTIONS(1773),
    [sym_devlev_keyword] = ACTIONS(1773),
    [sym_devchan_keyword] = ACTIONS(1773),
    [sym_if_keyword] = ACTIONS(1773),
    [sym_else_keyword] = ACTIONS(1773),
    [sym_switch_keyword] = ACTIONS(1773),
    [sym_case_keyword] = ACTIONS(1773),
    [sym_default_keyword] = ACTIONS(1773),
    [sym_while_keyword] = ACTIONS(1773),
    [sym_for_keyword] = ACTIONS(1773),
    [sym_break_keyword] = ACTIONS(1773),
    [sym_continue_keyword] = ACTIONS(1773),
    [sym_return_keyword] = ACTIONS(1773),
    [sym_select_keyword] = ACTIONS(1773),
    [sym_active_keyword] = ACTIONS(1773),
    [sym_struct_keyword] = ACTIONS(1773),
    [sym_structure_keyword] = ACTIONS(1773),
    [sym_button_event_keyword] = ACTIONS(1773),
    [sym_channel_event_keyword] = ACTIONS(1773),
    [sym_level_event_keyword] = ACTIONS(1773),
    [sym_data_event_keyword] = ACTIONS(1773),
    [sym_timeline_event_keyword] = ACTIONS(1773),
    [sym_custom_event_keyword] = ACTIONS(1773),
    [sym_push_keyword] = ACTIONS(1773),
    [sym_release_keyword] = ACTIONS(1773),
    [aux_sym_on_keyword_token1] = ACTIONS(1773),
    [aux_sym_off_keyword_token1] = ACTIONS(1773),
    [sym_send_level_keyword] = ACTIONS(1773),
    [sym_send_string_keyword] = ACTIONS(1773),
    [sym_send_command_keyword] = ACTIONS(1773),
    [sym_clear_buffer_keyword] = ACTIONS(1773),
    [sym_create_buffer_keyword] = ACTIONS(1773),
    [sym_create_multi_buffer_keyword] = ACTIONS(1773),
    [sym_call_keyword] = ACTIONS(1773),
    [sym_system_call_keyword] = ACTIONS(1773),
    [sym_devchan_to_keyword] = ACTIONS(1773),
    [sym_devchan_min_to_keyword] = ACTIONS(1773),
    [sym_devchan_total_off_keyword] = ACTIONS(1773),
    [sym_devchan_pulse_keyword] = ACTIONS(1773),
    [sym_wait_keyword] = ACTIONS(1773),
    [sym_cancel_wait_keyword] = ACTIONS(1773),
    [sym_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1773),
    [sym_not] = ACTIONS(1773),
    [sym_bnot] = ACTIONS(1773),
    [sym___file__] = ACTIONS(1773),
    [sym___date__] = ACTIONS(1773),
    [sym___ldate__] = ACTIONS(1773),
    [sym___time__] = ACTIONS(1773),
    [sym___line__] = ACTIONS(1773),
    [sym___name__] = ACTIONS(1773),
    [sym___netlinx__] = ACTIONS(1773),
    [sym_netlinx_axi_version] = ACTIONS(1773),
    [sym_url_flg_tcp] = ACTIONS(1773),
    [sym_url_flg_accinfopresent] = ACTIONS(1773),
    [sym_url_flg_temp] = ACTIONS(1773),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1773),
    [sym_url_flg_stat_mask] = ACTIONS(1773),
    [sym_url_flg_stat_lookup] = ACTIONS(1773),
    [sym_url_flg_stat_connecting] = ACTIONS(1773),
    [sym_url_flg_stat_waiting] = ACTIONS(1773),
    [sym_url_flg_stat_connected] = ACTIONS(1773),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1773),
    [sym_first_virtual_device] = ACTIONS(1773),
    [sym_first_local_port] = ACTIONS(1773),
    [sym_dynamic_virtual_device] = ACTIONS(1773),
    [sym_timeline_once] = ACTIONS(1773),
    [sym_timeline_repeat] = ACTIONS(1773),
    [sym_timeline_absolute] = ACTIONS(1773),
    [sym_timeline_relative] = ACTIONS(1773),
    [sym_do_push_timed_infinite] = ACTIONS(1773),
    [sym_source_type_no_address] = ACTIONS(1773),
    [sym_source_type_neuron_id] = ACTIONS(1773),
    [sym_source_type_ip_address] = ACTIONS(1773),
    [sym_source_type_axlink] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1773),
    [sym_source_type_ip_socket_address] = ACTIONS(1773),
    [sym_source_type_rs232] = ACTIONS(1773),
    [sym_source_type_internal] = ACTIONS(1773),
    [sym_source_type_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1773),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1773),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1773),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1773),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_file_read_only] = ACTIONS(1773),
    [sym_file_rw_new] = ACTIONS(1773),
    [sym_file_rw_append] = ACTIONS(1773),
    [sym_ip_tcp] = ACTIONS(1773),
    [sym_ip_udp] = ACTIONS(1773),
    [sym_ip_udp_2way] = ACTIONS(1773),
    [sym_xml_encode_types] = ACTIONS(1773),
    [sym_xml_encode_char_as_list] = ACTIONS(1773),
    [sym_xml_encode_le] = ACTIONS(1773),
    [sym_xml_decode_types] = ACTIONS(1773),
    [sym_xml_decode_no_preserve] = ACTIONS(1773),
    [sym_duet_dev_type_display_device] = ACTIONS(1773),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_io_device] = ACTIONS(1773),
    [sym_duet_dev_type_relay_device] = ACTIONS(1773),
    [sym_duet_dev_type_ups] = ACTIONS(1773),
    [sym_duet_dev_type_amplifier] = ACTIONS(1773),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1773),
    [sym_duet_dev_type_camera] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1773),
    [sym_duet_dev_type_dss] = ACTIONS(1773),
    [sym_duet_dev_type_dvr] = ACTIONS(1773),
    [sym_duet_dev_type_disc_device] = ACTIONS(1773),
    [sym_duet_dev_type_document_camera] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1773),
    [sym_duet_dev_type_hvac] = ACTIONS(1773),
    [sym_duet_dev_type_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_light] = ACTIONS(1773),
    [sym_duet_dev_type_monitor] = ACTIONS(1773),
    [sym_duet_dev_type_motor] = ACTIONS(1773),
    [sym_duet_dev_type_multi_window] = ACTIONS(1773),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1773),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1773),
    [sym_duet_dev_type_receiver] = ACTIONS(1773),
    [sym_duet_dev_type_security_system] = ACTIONS(1773),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1773),
    [sym_duet_dev_type_settop_box] = ACTIONS(1773),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1773),
    [sym_duet_dev_type_switcher] = ACTIONS(1773),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_tv] = ACTIONS(1773),
    [sym_duet_dev_type_utility] = ACTIONS(1773),
    [sym_duet_dev_type_vcr] = ACTIONS(1773),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_video_processor] = ACTIONS(1773),
    [sym_duet_dev_type_video_projector] = ACTIONS(1773),
    [sym_duet_dev_type_video_wall] = ACTIONS(1773),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1773),
    [sym_duet_dev_type_weather] = ACTIONS(1773),
    [sym_duet_dev_not_polled] = ACTIONS(1773),
    [sym_duet_dev_polled] = ACTIONS(1773),
    [sym_clkmgr_mode_network] = ACTIONS(1773),
    [sym_clkmgr_mode_standalone] = ACTIONS(1773),
    [sym_normal_standby] = ACTIONS(1773),
    [sym_normal_wake] = ACTIONS(1773),
    [sym_ip_multicast_ttl_option] = ACTIONS(1773),
    [sym_ip_ttl_subnet] = ACTIONS(1773),
    [sym_ip_ttl_site] = ACTIONS(1773),
    [sym_ip_ttl_region] = ACTIONS(1773),
    [sym_ip_ttl_continent] = ACTIONS(1773),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1773),
    [sym_ip_nodelay_on] = ACTIONS(1773),
    [sym_ip_nodelay_off] = ACTIONS(1773),
    [sym_amx_error] = ACTIONS(1773),
    [sym_amx_warning] = ACTIONS(1773),
    [sym_amx_info] = ACTIONS(1773),
    [sym_amx_debug] = ACTIONS(1773),
    [sym_netlinx_login_success] = ACTIONS(1773),
    [sym_netlinx_login_fail] = ACTIONS(1773),
    [sym_netlinx_logout] = ACTIONS(1773),
    [sym_null_str] = ACTIONS(1773),
    [sym_smtp_address] = ACTIONS(1773),
    [sym_smtp_port_number] = ACTIONS(1773),
    [sym_smtp_username] = ACTIONS(1773),
    [sym_smtp_password] = ACTIONS(1773),
    [sym_smtp_require_tls] = ACTIONS(1773),
    [sym_smtp_from] = ACTIONS(1773),
    [sym_smtp_tls_true] = ACTIONS(1773),
    [sym_smtp_tls_false] = ACTIONS(1773),
    [sym_listview_on_row_select_event] = ACTIONS(1773),
    [sym_data_structure_unknown] = ACTIONS(1773),
    [sym_data_structure_datafeed] = ACTIONS(1773),
    [sym_data_structure_datarecordset] = ACTIONS(1773),
    [sym_data_structure_datarecord] = ACTIONS(1773),
    [sym_data_structure_datafield] = ACTIONS(1773),
    [sym_data_max_content_field_count] = ACTIONS(1773),
    [sym_data_max_metadata_field_count] = ACTIONS(1773),
    [sym_data_max_id_length] = ACTIONS(1773),
    [sym_data_max_name_length] = ACTIONS(1773),
    [sym_data_max_description_length] = ACTIONS(1773),
    [sym_data_max_type_length] = ACTIONS(1773),
    [sym_data_max_format_length] = ACTIONS(1773),
    [sym_data_max_source_length] = ACTIONS(1773),
    [sym_data_max_value_length] = ACTIONS(1773),
    [sym_data_max_label_length] = ACTIONS(1773),
    [sym_data_max_url_length] = ACTIONS(1773),
    [sym_max_last_login_info_length] = ACTIONS(1773),
    [sym_data_type_unknown] = ACTIONS(1773),
    [sym_data_type_string] = ACTIONS(1773),
    [sym_data_type_datetime] = ACTIONS(1773),
    [sym_data_type_date] = ACTIONS(1773),
    [sym_data_type_time] = ACTIONS(1773),
    [sym_data_type_image] = ACTIONS(1773),
    [sym_data_format_url] = ACTIONS(1773),
    [sym_data_format_phone] = ACTIONS(1773),
    [sym_data_format_email] = ACTIONS(1773),
    [sym_data_format_iso8601] = ACTIONS(1773),
    [sym_auditlog_priv] = ACTIONS(1773),
    [sym_configuration_priv] = ACTIONS(1773),
    [sym_deviceconfig_priv] = ACTIONS(1773),
    [sym_ftp_priv] = ACTIONS(1773),
    [sym_http_priv] = ACTIONS(1773),
    [sym_networkconfig_priv] = ACTIONS(1773),
    [sym_programport_priv] = ACTIONS(1773),
    [sym_remoteui_priv] = ACTIONS(1773),
    [sym_securitycontrol_priv] = ACTIONS(1773),
    [sym_softwaremanagement_priv] = ACTIONS(1773),
    [sym_terminal_priv] = ACTIONS(1773),
    [sym_tpadmin_priv] = ACTIONS(1773),
    [sym_user1_priv] = ACTIONS(1773),
    [sym_user2_priv] = ACTIONS(1773),
    [sym_user3_priv] = ACTIONS(1773),
    [sym_user4_priv] = ACTIONS(1773),
    [sym_usermanagement_priv] = ACTIONS(1773),
    [sym_valid_account] = ACTIONS(1773),
    [sym_err_name_invalid] = ACTIONS(1773),
    [sym_err_password_invalid] = ACTIONS(1773),
    [sym_err_invalid_account] = ACTIONS(1773),
    [sym_err_no_authorization] = ACTIONS(1773),
    [sym_err_invalid_parameter] = ACTIONS(1773),
    [sym_err_user_locked_out] = ACTIONS(1773),
    [sym_err_user_account_expired] = ACTIONS(1773),
    [sym_err_general_error] = ACTIONS(1773),
    [sym_err_authentication_error] = ACTIONS(1773),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1773),
    [sym_tls_validate_certificate] = ACTIONS(1773),
    [sym_tdata] = ACTIONS(1773),
    [sym_tchannel] = ACTIONS(1773),
    [sym_tlevel] = ACTIONS(1773),
    [sym_tbutton] = ACTIONS(1773),
    [sym_ttimeline] = ACTIONS(1773),
    [sym_tcustom] = ACTIONS(1773),
    [sym_url_struct] = ACTIONS(1773),
    [sym_dns_struct] = ACTIONS(1773),
    [sym_ip_address_struct] = ACTIONS(1773),
    [sym_dev_info_struct] = ACTIONS(1773),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1773),
    [sym_data_feed] = ACTIONS(1773),
    [sym_data_field] = ACTIONS(1773),
    [sym_data_record] = ACTIONS(1773),
    [sym_wc_data_feed] = ACTIONS(1773),
    [sym_wc_data_field] = ACTIONS(1773),
    [sym_wc_data_record] = ACTIONS(1773),
    [sym_last_login_info] = ACTIONS(1773),
    [sym____reserved___] = ACTIONS(1773),
    [sym_date] = ACTIONS(1773),
    [sym_day] = ACTIONS(1773),
    [sym_time] = ACTIONS(1773),
    [sym_ldate] = ACTIONS(1773),
    [sym_push_device] = ACTIONS(1773),
    [sym_push_channel] = ACTIONS(1773),
    [sym_push_devchan] = ACTIONS(1773),
    [sym_release_device] = ACTIONS(1773),
    [sym_release_channel] = ACTIONS(1773),
    [sym_release_devchan] = ACTIONS(1773),
    [sym_master_slot] = ACTIONS(1773),
    [sym_get_pulse_time] = ACTIONS(1773),
    [sym_get_timer] = ACTIONS(1773),
    [sym_master_sn] = ACTIONS(1773),
    [sym_system_number] = ACTIONS(1773),
    [sym_button] = ACTIONS(1773),
    [sym_dv_channel] = ACTIONS(1773),
    [sym_data] = ACTIONS(1773),
    [sym_level] = ACTIONS(1773),
    [sym_timeline] = ACTIONS(1773),
    [sym_channel] = ACTIONS(1773),
    [sym_custom] = ACTIONS(1773),
    [sym_atoi] = ACTIONS(1773),
    [sym_atol] = ACTIONS(1773),
    [sym_atof] = ACTIONS(1773),
    [sym_itoa] = ACTIONS(1773),
    [sym_itohex] = ACTIONS(1773),
    [sym_hextoi] = ACTIONS(1773),
    [sym_ftoa] = ACTIONS(1773),
    [sym_find_string] = ACTIONS(1773),
    [sym_left_string] = ACTIONS(1773),
    [sym_right_string] = ACTIONS(1773),
    [sym_mid_string] = ACTIONS(1773),
    [sym_remove_string] = ACTIONS(1773),
    [sym_get_buffer_string] = ACTIONS(1773),
    [sym_get_buffer_char] = ACTIONS(1773),
    [sym_get_multi_buffer_string] = ACTIONS(1773),
    [sym_length_string] = ACTIONS(1773),
    [sym_max_length_string] = ACTIONS(1773),
    [sym_set_length_string] = ACTIONS(1773),
    [sym_compare_string] = ACTIONS(1773),
    [sym_upper_string] = ACTIONS(1773),
    [sym_lower_string] = ACTIONS(1773),
    [sym_get_last] = ACTIONS(1773),
    [sym_date_to_day] = ACTIONS(1773),
    [sym_date_to_month] = ACTIONS(1773),
    [sym_date_to_year] = ACTIONS(1773),
    [sym_time_to_hour] = ACTIONS(1773),
    [sym_time_to_minute] = ACTIONS(1773),
    [sym_time_to_second] = ACTIONS(1773),
    [sym_day_of_week] = ACTIONS(1773),
    [sym_set_pulse_time] = ACTIONS(1773),
    [sym_set_timer] = ACTIONS(1773),
    [sym_file_open] = ACTIONS(1773),
    [sym_file_close] = ACTIONS(1773),
    [sym_file_read] = ACTIONS(1773),
    [sym_file_read_line] = ACTIONS(1773),
    [sym_file_write] = ACTIONS(1773),
    [sym_file_write_line] = ACTIONS(1773),
    [sym_file_copy] = ACTIONS(1773),
    [sym_file_delete] = ACTIONS(1773),
    [sym_file_rename] = ACTIONS(1773),
    [sym_file_dir] = ACTIONS(1773),
    [sym_file_getdir] = ACTIONS(1773),
    [sym_file_setdir] = ACTIONS(1773),
    [sym_file_create_dir] = ACTIONS(1773),
    [sym_file_remove_dir] = ACTIONS(1773),
    [sym_file_seek] = ACTIONS(1773),
    [sym_length_array] = ACTIONS(1773),
    [sym_max_length_array] = ACTIONS(1773),
    [sym_set_length_array] = ACTIONS(1773),
    [sym_ip_client_open] = ACTIONS(1773),
    [sym_ip_client_close] = ACTIONS(1773),
    [sym_ip_server_open] = ACTIONS(1773),
    [sym_ip_server_close] = ACTIONS(1773),
    [sym_ip_bound_client_open] = ACTIONS(1773),
    [sym_ip_mc_server_open] = ACTIONS(1773),
    [sym_ip_set_option] = ACTIONS(1773),
    [sym_ssh_client_open] = ACTIONS(1773),
    [sym_ssh_client_close] = ACTIONS(1773),
    [sym_tls_client_open] = ACTIONS(1773),
    [sym_tls_client_close] = ACTIONS(1773),
    [sym_do_push] = ACTIONS(1773),
    [sym_do_push_timed] = ACTIONS(1773),
    [sym_do_release] = ACTIONS(1773),
    [sym_device_id] = ACTIONS(1773),
    [sym_device_info] = ACTIONS(1773),
    [sym_device_id_string] = ACTIONS(1773),
    [sym_device_standby] = ACTIONS(1773),
    [sym_device_wake] = ACTIONS(1773),
    [sym_timeline_create] = ACTIONS(1773),
    [sym_timeline_kill] = ACTIONS(1773),
    [sym_timeline_reload] = ACTIONS(1773),
    [sym_timeline_pause] = ACTIONS(1773),
    [sym_timeline_restart] = ACTIONS(1773),
    [sym_timeline_set] = ACTIONS(1773),
    [sym_timeline_get] = ACTIONS(1773),
    [sym_timeline_active] = ACTIONS(1773),
    [sym_timeline_dynamic_id] = ACTIONS(1773),
    [sym_variable_to_string] = ACTIONS(1773),
    [sym_string_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_string] = ACTIONS(1773),
    [sym_type_cast] = ACTIONS(1773),
    [sym_variable_to_xml] = ACTIONS(1773),
    [sym_xml_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_xml] = ACTIONS(1773),
    [sym_redirect_string] = ACTIONS(1773),
    [sym_random_number] = ACTIONS(1773),
    [sym_get_unique_id] = ACTIONS(1773),
    [sym_get_system_number] = ACTIONS(1773),
    [sym_set_system_number] = ACTIONS(1773),
    [sym_get_serial_number] = ACTIONS(1773),
    [sym_get_master_build] = ACTIONS(1773),
    [sym_set_validation_code] = ACTIONS(1773),
    [sym_reboot] = ACTIONS(1773),
    [sym_load_duet_module] = ACTIONS(1773),
    [sym_unload_duet_module] = ACTIONS(1773),
    [sym_duet_mem_size_set] = ACTIONS(1773),
    [sym_duet_mem_size_get] = ACTIONS(1773),
    [sym_dynamic_application_device] = ACTIONS(1773),
    [sym_dynamic_polled_port] = ACTIONS(1773),
    [sym_static_port_binding] = ACTIONS(1773),
    [sym_static_ip_binding] = ACTIONS(1773),
    [sym_get_dns_list] = ACTIONS(1773),
    [sym_set_dns_list] = ACTIONS(1773),
    [sym_get_ip_address] = ACTIONS(1773),
    [sym_set_ip_address] = ACTIONS(1773),
    [sym_get_url_list] = ACTIONS(1773),
    [sym_add_url_entry] = ACTIONS(1773),
    [sym_delete_url_entry] = ACTIONS(1773),
    [sym_abs_value] = ACTIONS(1773),
    [sym_max_value] = ACTIONS(1773),
    [sym_min_value] = ACTIONS(1773),
    [sym_exp_value] = ACTIONS(1773),
    [sym_log_value] = ACTIONS(1773),
    [sym_log10_value] = ACTIONS(1773),
    [sym_power_value] = ACTIONS(1773),
    [sym_sqrt_value] = ACTIONS(1773),
    [sym_raw_be] = ACTIONS(1773),
    [sym_raw_le] = ACTIONS(1773),
    [sym_internal_queue_size_set] = ACTIONS(1773),
    [sym_internal_queue_size_get] = ACTIONS(1773),
    [sym_internal_threshold_set] = ACTIONS(1773),
    [sym_internal_threshold_get] = ACTIONS(1773),
    [sym_rebuilt_event] = ACTIONS(1773),
    [sym_do_custom_event] = ACTIONS(1773),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1773),
    [sym_clkmgr_set_clk_source] = ACTIONS(1773),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1773),
    [sym_clkmgr_get_timezone] = ACTIONS(1773),
    [sym_clkmgr_set_timezone] = ACTIONS(1773),
    [sym_clkmgr_get_resync_period] = ACTIONS(1773),
    [sym_clkmgr_set_resync_period] = ACTIONS(1773),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_timeservers] = ACTIONS(1773),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_set_log_level] = ACTIONS(1773),
    [sym_get_log_level] = ACTIONS(1773),
    [sym_amx_log] = ACTIONS(1773),
    [sym_smtp_server_config_set] = ACTIONS(1773),
    [sym_smtp_server_config_get] = ACTIONS(1773),
    [sym_smtp_send] = ACTIONS(1773),
    [sym_validate_netlinx_account] = ACTIONS(1773),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1773),
    [sym_authenticate_certificate] = ACTIONS(1773),
    [sym_audit_netlinx_session_event] = ACTIONS(1773),
    [sym_audit_netlinx_generic_event] = ACTIONS(1773),
    [sym_get_available_flash_disk_space] = ACTIONS(1773),
    [sym_get_max_flash_disk_space] = ACTIONS(1773),
    [sym_data_create_feed] = ACTIONS(1773),
    [sym_data_delete_feed] = ACTIONS(1773),
    [sym_data_publish_feed] = ACTIONS(1773),
    [sym_data_get_published_feed] = ACTIONS(1773),
    [sym_data_add_record] = ACTIONS(1773),
    [sym_data_get_event_record] = ACTIONS(1773),
    [sym__wc_data_create_feed] = ACTIONS(1773),
    [sym__wc_data_add_record] = ACTIONS(1773),
    [sym__wc_data_get_event_record] = ACTIONS(1773),
    [sym_format] = ACTIONS(1773),
    [sym_astro_clock] = ACTIONS(1773),
    [sym_set_outdoor_temperature] = ACTIONS(1773),
    [sym_set_virtual_level_count] = ACTIONS(1773),
    [sym_set_virtual_channel_count] = ACTIONS(1773),
    [sym_set_virtual_port_count] = ACTIONS(1773),
    [sym__wc_encode_int] = ACTIONS(1773),
    [sym__wc_decode_int] = ACTIONS(1773),
    [sym__wc_int] = ACTIONS(1773),
    [sym__wc_to_ch_int] = ACTIONS(1773),
    [sym__ch_to_wc_int] = ACTIONS(1773),
    [sym__wc_find_string_int] = ACTIONS(1773),
    [sym__wc_left_string_int] = ACTIONS(1773),
    [sym__wc_lower_string_int] = ACTIONS(1773),
    [sym__wc_mid_string_int] = ACTIONS(1773),
    [sym__wc_remove_string_int] = ACTIONS(1773),
    [sym__wc_right_string_int] = ACTIONS(1773),
    [sym__wc_upper_string_int] = ACTIONS(1773),
    [sym__wc_compare_string_int] = ACTIONS(1773),
    [sym__wc_get_buffer_char_int] = ACTIONS(1773),
    [sym__wc_get_buffer_string_int] = ACTIONS(1773),
    [sym__wc_concat_string_int] = ACTIONS(1773),
    [sym__wc_file_open_int] = ACTIONS(1773),
    [sym__wc_file_close_int] = ACTIONS(1773),
    [sym__wc_file_read_int] = ACTIONS(1773),
    [sym__wc_file_read_line_int] = ACTIONS(1773),
    [sym__wc_file_write_int] = ACTIONS(1773),
    [sym__wc_file_write_line_int] = ACTIONS(1773),
    [aux_sym_preproc_include_token1] = ACTIONS(1773),
    [aux_sym_preproc_define_token1] = ACTIONS(1775),
    [aux_sym_preproc_warn_token1] = ACTIONS(1775),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1775),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_else_token1] = ACTIONS(1775),
    [anon_sym_LBRACK] = ACTIONS(1775),
    [anon_sym_LBRACE] = ACTIONS(1775),
    [anon_sym_LPAREN] = ACTIONS(1773),
    [anon_sym_BANG] = ACTIONS(1775),
    [anon_sym_TILDE] = ACTIONS(1775),
    [anon_sym_DASH] = ACTIONS(1773),
    [anon_sym_PLUS] = ACTIONS(1773),
    [anon_sym_DASH_DASH] = ACTIONS(1775),
    [anon_sym_PLUS_PLUS] = ACTIONS(1775),
    [anon_sym_DQUOTE] = ACTIONS(1775),
    [anon_sym_SQUOTE] = ACTIONS(1775),
    [sym_number_literal] = ACTIONS(1775),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(229)] = {
    [sym_identifier] = ACTIONS(1773),
    [sym_program_name_keyword] = ACTIONS(1773),
    [sym_module_name_keyword] = ACTIONS(1773),
    [sym_define_device_keyword] = ACTIONS(1773),
    [sym_define_combine_keyword] = ACTIONS(1773),
    [sym_define_constant_keyword] = ACTIONS(1773),
    [sym_define_type_keyword] = ACTIONS(1773),
    [sym_define_variable_keyword] = ACTIONS(1773),
    [sym_define_system_variable_keyword] = ACTIONS(1773),
    [sym_define_connect_level_keyword] = ACTIONS(1773),
    [sym_define_latching_keyword] = ACTIONS(1773),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1773),
    [sym_define_toggling_keyword] = ACTIONS(1773),
    [sym_define_call_keyword] = ACTIONS(1773),
    [sym_define_function_keyword] = ACTIONS(1773),
    [sym_define_library_function_keyword] = ACTIONS(1773),
    [sym_define_start_keyword] = ACTIONS(1773),
    [sym_define_event_keyword] = ACTIONS(1773),
    [sym_define_program_keyword] = ACTIONS(1773),
    [sym_define_module_keyword] = ACTIONS(1773),
    [sym_char_keyword] = ACTIONS(1773),
    [sym_widechar_keyword] = ACTIONS(1773),
    [sym_integer_keyword] = ACTIONS(1773),
    [sym_sinteger_keyword] = ACTIONS(1773),
    [sym_long_keyword] = ACTIONS(1773),
    [sym_slong_keyword] = ACTIONS(1773),
    [sym_float_keyword] = ACTIONS(1773),
    [sym_double_keyword] = ACTIONS(1773),
    [sym_variant_keyword] = ACTIONS(1773),
    [sym_variantarray_keyword] = ACTIONS(1773),
    [sym_local_var_keyword] = ACTIONS(1773),
    [sym_stack_var_keyword] = ACTIONS(1773),
    [sym_constant_keyword] = ACTIONS(1773),
    [sym_volatile_keyword] = ACTIONS(1773),
    [sym_non_volatile_keyword] = ACTIONS(1773),
    [sym_persistent_keyword] = ACTIONS(1773),
    [sym_dev_keyword] = ACTIONS(1773),
    [sym_devlev_keyword] = ACTIONS(1773),
    [sym_devchan_keyword] = ACTIONS(1773),
    [sym_if_keyword] = ACTIONS(1773),
    [sym_else_keyword] = ACTIONS(1773),
    [sym_switch_keyword] = ACTIONS(1773),
    [sym_case_keyword] = ACTIONS(1773),
    [sym_default_keyword] = ACTIONS(1773),
    [sym_while_keyword] = ACTIONS(1773),
    [sym_for_keyword] = ACTIONS(1773),
    [sym_break_keyword] = ACTIONS(1773),
    [sym_continue_keyword] = ACTIONS(1773),
    [sym_return_keyword] = ACTIONS(1773),
    [sym_select_keyword] = ACTIONS(1773),
    [sym_active_keyword] = ACTIONS(1773),
    [sym_struct_keyword] = ACTIONS(1773),
    [sym_structure_keyword] = ACTIONS(1773),
    [sym_button_event_keyword] = ACTIONS(1773),
    [sym_channel_event_keyword] = ACTIONS(1773),
    [sym_level_event_keyword] = ACTIONS(1773),
    [sym_data_event_keyword] = ACTIONS(1773),
    [sym_timeline_event_keyword] = ACTIONS(1773),
    [sym_custom_event_keyword] = ACTIONS(1773),
    [sym_push_keyword] = ACTIONS(1773),
    [sym_release_keyword] = ACTIONS(1773),
    [aux_sym_on_keyword_token1] = ACTIONS(1773),
    [aux_sym_off_keyword_token1] = ACTIONS(1773),
    [sym_send_level_keyword] = ACTIONS(1773),
    [sym_send_string_keyword] = ACTIONS(1773),
    [sym_send_command_keyword] = ACTIONS(1773),
    [sym_clear_buffer_keyword] = ACTIONS(1773),
    [sym_create_buffer_keyword] = ACTIONS(1773),
    [sym_create_multi_buffer_keyword] = ACTIONS(1773),
    [sym_call_keyword] = ACTIONS(1773),
    [sym_system_call_keyword] = ACTIONS(1773),
    [sym_devchan_to_keyword] = ACTIONS(1773),
    [sym_devchan_min_to_keyword] = ACTIONS(1773),
    [sym_devchan_total_off_keyword] = ACTIONS(1773),
    [sym_devchan_pulse_keyword] = ACTIONS(1773),
    [sym_wait_keyword] = ACTIONS(1773),
    [sym_cancel_wait_keyword] = ACTIONS(1773),
    [sym_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1773),
    [sym_not] = ACTIONS(1773),
    [sym_bnot] = ACTIONS(1773),
    [sym___file__] = ACTIONS(1773),
    [sym___date__] = ACTIONS(1773),
    [sym___ldate__] = ACTIONS(1773),
    [sym___time__] = ACTIONS(1773),
    [sym___line__] = ACTIONS(1773),
    [sym___name__] = ACTIONS(1773),
    [sym___netlinx__] = ACTIONS(1773),
    [sym_netlinx_axi_version] = ACTIONS(1773),
    [sym_url_flg_tcp] = ACTIONS(1773),
    [sym_url_flg_accinfopresent] = ACTIONS(1773),
    [sym_url_flg_temp] = ACTIONS(1773),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1773),
    [sym_url_flg_stat_mask] = ACTIONS(1773),
    [sym_url_flg_stat_lookup] = ACTIONS(1773),
    [sym_url_flg_stat_connecting] = ACTIONS(1773),
    [sym_url_flg_stat_waiting] = ACTIONS(1773),
    [sym_url_flg_stat_connected] = ACTIONS(1773),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1773),
    [sym_first_virtual_device] = ACTIONS(1773),
    [sym_first_local_port] = ACTIONS(1773),
    [sym_dynamic_virtual_device] = ACTIONS(1773),
    [sym_timeline_once] = ACTIONS(1773),
    [sym_timeline_repeat] = ACTIONS(1773),
    [sym_timeline_absolute] = ACTIONS(1773),
    [sym_timeline_relative] = ACTIONS(1773),
    [sym_do_push_timed_infinite] = ACTIONS(1773),
    [sym_source_type_no_address] = ACTIONS(1773),
    [sym_source_type_neuron_id] = ACTIONS(1773),
    [sym_source_type_ip_address] = ACTIONS(1773),
    [sym_source_type_axlink] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1773),
    [sym_source_type_ip_socket_address] = ACTIONS(1773),
    [sym_source_type_rs232] = ACTIONS(1773),
    [sym_source_type_internal] = ACTIONS(1773),
    [sym_source_type_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1773),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1773),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1773),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1773),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_file_read_only] = ACTIONS(1773),
    [sym_file_rw_new] = ACTIONS(1773),
    [sym_file_rw_append] = ACTIONS(1773),
    [sym_ip_tcp] = ACTIONS(1773),
    [sym_ip_udp] = ACTIONS(1773),
    [sym_ip_udp_2way] = ACTIONS(1773),
    [sym_xml_encode_types] = ACTIONS(1773),
    [sym_xml_encode_char_as_list] = ACTIONS(1773),
    [sym_xml_encode_le] = ACTIONS(1773),
    [sym_xml_decode_types] = ACTIONS(1773),
    [sym_xml_decode_no_preserve] = ACTIONS(1773),
    [sym_duet_dev_type_display_device] = ACTIONS(1773),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_io_device] = ACTIONS(1773),
    [sym_duet_dev_type_relay_device] = ACTIONS(1773),
    [sym_duet_dev_type_ups] = ACTIONS(1773),
    [sym_duet_dev_type_amplifier] = ACTIONS(1773),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1773),
    [sym_duet_dev_type_camera] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1773),
    [sym_duet_dev_type_dss] = ACTIONS(1773),
    [sym_duet_dev_type_dvr] = ACTIONS(1773),
    [sym_duet_dev_type_disc_device] = ACTIONS(1773),
    [sym_duet_dev_type_document_camera] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1773),
    [sym_duet_dev_type_hvac] = ACTIONS(1773),
    [sym_duet_dev_type_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_light] = ACTIONS(1773),
    [sym_duet_dev_type_monitor] = ACTIONS(1773),
    [sym_duet_dev_type_motor] = ACTIONS(1773),
    [sym_duet_dev_type_multi_window] = ACTIONS(1773),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1773),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1773),
    [sym_duet_dev_type_receiver] = ACTIONS(1773),
    [sym_duet_dev_type_security_system] = ACTIONS(1773),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1773),
    [sym_duet_dev_type_settop_box] = ACTIONS(1773),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1773),
    [sym_duet_dev_type_switcher] = ACTIONS(1773),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_tv] = ACTIONS(1773),
    [sym_duet_dev_type_utility] = ACTIONS(1773),
    [sym_duet_dev_type_vcr] = ACTIONS(1773),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_video_processor] = ACTIONS(1773),
    [sym_duet_dev_type_video_projector] = ACTIONS(1773),
    [sym_duet_dev_type_video_wall] = ACTIONS(1773),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1773),
    [sym_duet_dev_type_weather] = ACTIONS(1773),
    [sym_duet_dev_not_polled] = ACTIONS(1773),
    [sym_duet_dev_polled] = ACTIONS(1773),
    [sym_clkmgr_mode_network] = ACTIONS(1773),
    [sym_clkmgr_mode_standalone] = ACTIONS(1773),
    [sym_normal_standby] = ACTIONS(1773),
    [sym_normal_wake] = ACTIONS(1773),
    [sym_ip_multicast_ttl_option] = ACTIONS(1773),
    [sym_ip_ttl_subnet] = ACTIONS(1773),
    [sym_ip_ttl_site] = ACTIONS(1773),
    [sym_ip_ttl_region] = ACTIONS(1773),
    [sym_ip_ttl_continent] = ACTIONS(1773),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1773),
    [sym_ip_nodelay_on] = ACTIONS(1773),
    [sym_ip_nodelay_off] = ACTIONS(1773),
    [sym_amx_error] = ACTIONS(1773),
    [sym_amx_warning] = ACTIONS(1773),
    [sym_amx_info] = ACTIONS(1773),
    [sym_amx_debug] = ACTIONS(1773),
    [sym_netlinx_login_success] = ACTIONS(1773),
    [sym_netlinx_login_fail] = ACTIONS(1773),
    [sym_netlinx_logout] = ACTIONS(1773),
    [sym_null_str] = ACTIONS(1773),
    [sym_smtp_address] = ACTIONS(1773),
    [sym_smtp_port_number] = ACTIONS(1773),
    [sym_smtp_username] = ACTIONS(1773),
    [sym_smtp_password] = ACTIONS(1773),
    [sym_smtp_require_tls] = ACTIONS(1773),
    [sym_smtp_from] = ACTIONS(1773),
    [sym_smtp_tls_true] = ACTIONS(1773),
    [sym_smtp_tls_false] = ACTIONS(1773),
    [sym_listview_on_row_select_event] = ACTIONS(1773),
    [sym_data_structure_unknown] = ACTIONS(1773),
    [sym_data_structure_datafeed] = ACTIONS(1773),
    [sym_data_structure_datarecordset] = ACTIONS(1773),
    [sym_data_structure_datarecord] = ACTIONS(1773),
    [sym_data_structure_datafield] = ACTIONS(1773),
    [sym_data_max_content_field_count] = ACTIONS(1773),
    [sym_data_max_metadata_field_count] = ACTIONS(1773),
    [sym_data_max_id_length] = ACTIONS(1773),
    [sym_data_max_name_length] = ACTIONS(1773),
    [sym_data_max_description_length] = ACTIONS(1773),
    [sym_data_max_type_length] = ACTIONS(1773),
    [sym_data_max_format_length] = ACTIONS(1773),
    [sym_data_max_source_length] = ACTIONS(1773),
    [sym_data_max_value_length] = ACTIONS(1773),
    [sym_data_max_label_length] = ACTIONS(1773),
    [sym_data_max_url_length] = ACTIONS(1773),
    [sym_max_last_login_info_length] = ACTIONS(1773),
    [sym_data_type_unknown] = ACTIONS(1773),
    [sym_data_type_string] = ACTIONS(1773),
    [sym_data_type_datetime] = ACTIONS(1773),
    [sym_data_type_date] = ACTIONS(1773),
    [sym_data_type_time] = ACTIONS(1773),
    [sym_data_type_image] = ACTIONS(1773),
    [sym_data_format_url] = ACTIONS(1773),
    [sym_data_format_phone] = ACTIONS(1773),
    [sym_data_format_email] = ACTIONS(1773),
    [sym_data_format_iso8601] = ACTIONS(1773),
    [sym_auditlog_priv] = ACTIONS(1773),
    [sym_configuration_priv] = ACTIONS(1773),
    [sym_deviceconfig_priv] = ACTIONS(1773),
    [sym_ftp_priv] = ACTIONS(1773),
    [sym_http_priv] = ACTIONS(1773),
    [sym_networkconfig_priv] = ACTIONS(1773),
    [sym_programport_priv] = ACTIONS(1773),
    [sym_remoteui_priv] = ACTIONS(1773),
    [sym_securitycontrol_priv] = ACTIONS(1773),
    [sym_softwaremanagement_priv] = ACTIONS(1773),
    [sym_terminal_priv] = ACTIONS(1773),
    [sym_tpadmin_priv] = ACTIONS(1773),
    [sym_user1_priv] = ACTIONS(1773),
    [sym_user2_priv] = ACTIONS(1773),
    [sym_user3_priv] = ACTIONS(1773),
    [sym_user4_priv] = ACTIONS(1773),
    [sym_usermanagement_priv] = ACTIONS(1773),
    [sym_valid_account] = ACTIONS(1773),
    [sym_err_name_invalid] = ACTIONS(1773),
    [sym_err_password_invalid] = ACTIONS(1773),
    [sym_err_invalid_account] = ACTIONS(1773),
    [sym_err_no_authorization] = ACTIONS(1773),
    [sym_err_invalid_parameter] = ACTIONS(1773),
    [sym_err_user_locked_out] = ACTIONS(1773),
    [sym_err_user_account_expired] = ACTIONS(1773),
    [sym_err_general_error] = ACTIONS(1773),
    [sym_err_authentication_error] = ACTIONS(1773),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1773),
    [sym_tls_validate_certificate] = ACTIONS(1773),
    [sym_tdata] = ACTIONS(1773),
    [sym_tchannel] = ACTIONS(1773),
    [sym_tlevel] = ACTIONS(1773),
    [sym_tbutton] = ACTIONS(1773),
    [sym_ttimeline] = ACTIONS(1773),
    [sym_tcustom] = ACTIONS(1773),
    [sym_url_struct] = ACTIONS(1773),
    [sym_dns_struct] = ACTIONS(1773),
    [sym_ip_address_struct] = ACTIONS(1773),
    [sym_dev_info_struct] = ACTIONS(1773),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1773),
    [sym_data_feed] = ACTIONS(1773),
    [sym_data_field] = ACTIONS(1773),
    [sym_data_record] = ACTIONS(1773),
    [sym_wc_data_feed] = ACTIONS(1773),
    [sym_wc_data_field] = ACTIONS(1773),
    [sym_wc_data_record] = ACTIONS(1773),
    [sym_last_login_info] = ACTIONS(1773),
    [sym____reserved___] = ACTIONS(1773),
    [sym_date] = ACTIONS(1773),
    [sym_day] = ACTIONS(1773),
    [sym_time] = ACTIONS(1773),
    [sym_ldate] = ACTIONS(1773),
    [sym_push_device] = ACTIONS(1773),
    [sym_push_channel] = ACTIONS(1773),
    [sym_push_devchan] = ACTIONS(1773),
    [sym_release_device] = ACTIONS(1773),
    [sym_release_channel] = ACTIONS(1773),
    [sym_release_devchan] = ACTIONS(1773),
    [sym_master_slot] = ACTIONS(1773),
    [sym_get_pulse_time] = ACTIONS(1773),
    [sym_get_timer] = ACTIONS(1773),
    [sym_master_sn] = ACTIONS(1773),
    [sym_system_number] = ACTIONS(1773),
    [sym_button] = ACTIONS(1773),
    [sym_dv_channel] = ACTIONS(1773),
    [sym_data] = ACTIONS(1773),
    [sym_level] = ACTIONS(1773),
    [sym_timeline] = ACTIONS(1773),
    [sym_channel] = ACTIONS(1773),
    [sym_custom] = ACTIONS(1773),
    [sym_atoi] = ACTIONS(1773),
    [sym_atol] = ACTIONS(1773),
    [sym_atof] = ACTIONS(1773),
    [sym_itoa] = ACTIONS(1773),
    [sym_itohex] = ACTIONS(1773),
    [sym_hextoi] = ACTIONS(1773),
    [sym_ftoa] = ACTIONS(1773),
    [sym_find_string] = ACTIONS(1773),
    [sym_left_string] = ACTIONS(1773),
    [sym_right_string] = ACTIONS(1773),
    [sym_mid_string] = ACTIONS(1773),
    [sym_remove_string] = ACTIONS(1773),
    [sym_get_buffer_string] = ACTIONS(1773),
    [sym_get_buffer_char] = ACTIONS(1773),
    [sym_get_multi_buffer_string] = ACTIONS(1773),
    [sym_length_string] = ACTIONS(1773),
    [sym_max_length_string] = ACTIONS(1773),
    [sym_set_length_string] = ACTIONS(1773),
    [sym_compare_string] = ACTIONS(1773),
    [sym_upper_string] = ACTIONS(1773),
    [sym_lower_string] = ACTIONS(1773),
    [sym_get_last] = ACTIONS(1773),
    [sym_date_to_day] = ACTIONS(1773),
    [sym_date_to_month] = ACTIONS(1773),
    [sym_date_to_year] = ACTIONS(1773),
    [sym_time_to_hour] = ACTIONS(1773),
    [sym_time_to_minute] = ACTIONS(1773),
    [sym_time_to_second] = ACTIONS(1773),
    [sym_day_of_week] = ACTIONS(1773),
    [sym_set_pulse_time] = ACTIONS(1773),
    [sym_set_timer] = ACTIONS(1773),
    [sym_file_open] = ACTIONS(1773),
    [sym_file_close] = ACTIONS(1773),
    [sym_file_read] = ACTIONS(1773),
    [sym_file_read_line] = ACTIONS(1773),
    [sym_file_write] = ACTIONS(1773),
    [sym_file_write_line] = ACTIONS(1773),
    [sym_file_copy] = ACTIONS(1773),
    [sym_file_delete] = ACTIONS(1773),
    [sym_file_rename] = ACTIONS(1773),
    [sym_file_dir] = ACTIONS(1773),
    [sym_file_getdir] = ACTIONS(1773),
    [sym_file_setdir] = ACTIONS(1773),
    [sym_file_create_dir] = ACTIONS(1773),
    [sym_file_remove_dir] = ACTIONS(1773),
    [sym_file_seek] = ACTIONS(1773),
    [sym_length_array] = ACTIONS(1773),
    [sym_max_length_array] = ACTIONS(1773),
    [sym_set_length_array] = ACTIONS(1773),
    [sym_ip_client_open] = ACTIONS(1773),
    [sym_ip_client_close] = ACTIONS(1773),
    [sym_ip_server_open] = ACTIONS(1773),
    [sym_ip_server_close] = ACTIONS(1773),
    [sym_ip_bound_client_open] = ACTIONS(1773),
    [sym_ip_mc_server_open] = ACTIONS(1773),
    [sym_ip_set_option] = ACTIONS(1773),
    [sym_ssh_client_open] = ACTIONS(1773),
    [sym_ssh_client_close] = ACTIONS(1773),
    [sym_tls_client_open] = ACTIONS(1773),
    [sym_tls_client_close] = ACTIONS(1773),
    [sym_do_push] = ACTIONS(1773),
    [sym_do_push_timed] = ACTIONS(1773),
    [sym_do_release] = ACTIONS(1773),
    [sym_device_id] = ACTIONS(1773),
    [sym_device_info] = ACTIONS(1773),
    [sym_device_id_string] = ACTIONS(1773),
    [sym_device_standby] = ACTIONS(1773),
    [sym_device_wake] = ACTIONS(1773),
    [sym_timeline_create] = ACTIONS(1773),
    [sym_timeline_kill] = ACTIONS(1773),
    [sym_timeline_reload] = ACTIONS(1773),
    [sym_timeline_pause] = ACTIONS(1773),
    [sym_timeline_restart] = ACTIONS(1773),
    [sym_timeline_set] = ACTIONS(1773),
    [sym_timeline_get] = ACTIONS(1773),
    [sym_timeline_active] = ACTIONS(1773),
    [sym_timeline_dynamic_id] = ACTIONS(1773),
    [sym_variable_to_string] = ACTIONS(1773),
    [sym_string_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_string] = ACTIONS(1773),
    [sym_type_cast] = ACTIONS(1773),
    [sym_variable_to_xml] = ACTIONS(1773),
    [sym_xml_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_xml] = ACTIONS(1773),
    [sym_redirect_string] = ACTIONS(1773),
    [sym_random_number] = ACTIONS(1773),
    [sym_get_unique_id] = ACTIONS(1773),
    [sym_get_system_number] = ACTIONS(1773),
    [sym_set_system_number] = ACTIONS(1773),
    [sym_get_serial_number] = ACTIONS(1773),
    [sym_get_master_build] = ACTIONS(1773),
    [sym_set_validation_code] = ACTIONS(1773),
    [sym_reboot] = ACTIONS(1773),
    [sym_load_duet_module] = ACTIONS(1773),
    [sym_unload_duet_module] = ACTIONS(1773),
    [sym_duet_mem_size_set] = ACTIONS(1773),
    [sym_duet_mem_size_get] = ACTIONS(1773),
    [sym_dynamic_application_device] = ACTIONS(1773),
    [sym_dynamic_polled_port] = ACTIONS(1773),
    [sym_static_port_binding] = ACTIONS(1773),
    [sym_static_ip_binding] = ACTIONS(1773),
    [sym_get_dns_list] = ACTIONS(1773),
    [sym_set_dns_list] = ACTIONS(1773),
    [sym_get_ip_address] = ACTIONS(1773),
    [sym_set_ip_address] = ACTIONS(1773),
    [sym_get_url_list] = ACTIONS(1773),
    [sym_add_url_entry] = ACTIONS(1773),
    [sym_delete_url_entry] = ACTIONS(1773),
    [sym_abs_value] = ACTIONS(1773),
    [sym_max_value] = ACTIONS(1773),
    [sym_min_value] = ACTIONS(1773),
    [sym_exp_value] = ACTIONS(1773),
    [sym_log_value] = ACTIONS(1773),
    [sym_log10_value] = ACTIONS(1773),
    [sym_power_value] = ACTIONS(1773),
    [sym_sqrt_value] = ACTIONS(1773),
    [sym_raw_be] = ACTIONS(1773),
    [sym_raw_le] = ACTIONS(1773),
    [sym_internal_queue_size_set] = ACTIONS(1773),
    [sym_internal_queue_size_get] = ACTIONS(1773),
    [sym_internal_threshold_set] = ACTIONS(1773),
    [sym_internal_threshold_get] = ACTIONS(1773),
    [sym_rebuilt_event] = ACTIONS(1773),
    [sym_do_custom_event] = ACTIONS(1773),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1773),
    [sym_clkmgr_set_clk_source] = ACTIONS(1773),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1773),
    [sym_clkmgr_get_timezone] = ACTIONS(1773),
    [sym_clkmgr_set_timezone] = ACTIONS(1773),
    [sym_clkmgr_get_resync_period] = ACTIONS(1773),
    [sym_clkmgr_set_resync_period] = ACTIONS(1773),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_timeservers] = ACTIONS(1773),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_set_log_level] = ACTIONS(1773),
    [sym_get_log_level] = ACTIONS(1773),
    [sym_amx_log] = ACTIONS(1773),
    [sym_smtp_server_config_set] = ACTIONS(1773),
    [sym_smtp_server_config_get] = ACTIONS(1773),
    [sym_smtp_send] = ACTIONS(1773),
    [sym_validate_netlinx_account] = ACTIONS(1773),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1773),
    [sym_authenticate_certificate] = ACTIONS(1773),
    [sym_audit_netlinx_session_event] = ACTIONS(1773),
    [sym_audit_netlinx_generic_event] = ACTIONS(1773),
    [sym_get_available_flash_disk_space] = ACTIONS(1773),
    [sym_get_max_flash_disk_space] = ACTIONS(1773),
    [sym_data_create_feed] = ACTIONS(1773),
    [sym_data_delete_feed] = ACTIONS(1773),
    [sym_data_publish_feed] = ACTIONS(1773),
    [sym_data_get_published_feed] = ACTIONS(1773),
    [sym_data_add_record] = ACTIONS(1773),
    [sym_data_get_event_record] = ACTIONS(1773),
    [sym__wc_data_create_feed] = ACTIONS(1773),
    [sym__wc_data_add_record] = ACTIONS(1773),
    [sym__wc_data_get_event_record] = ACTIONS(1773),
    [sym_format] = ACTIONS(1773),
    [sym_astro_clock] = ACTIONS(1773),
    [sym_set_outdoor_temperature] = ACTIONS(1773),
    [sym_set_virtual_level_count] = ACTIONS(1773),
    [sym_set_virtual_channel_count] = ACTIONS(1773),
    [sym_set_virtual_port_count] = ACTIONS(1773),
    [sym__wc_encode_int] = ACTIONS(1773),
    [sym__wc_decode_int] = ACTIONS(1773),
    [sym__wc_int] = ACTIONS(1773),
    [sym__wc_to_ch_int] = ACTIONS(1773),
    [sym__ch_to_wc_int] = ACTIONS(1773),
    [sym__wc_find_string_int] = ACTIONS(1773),
    [sym__wc_left_string_int] = ACTIONS(1773),
    [sym__wc_lower_string_int] = ACTIONS(1773),
    [sym__wc_mid_string_int] = ACTIONS(1773),
    [sym__wc_remove_string_int] = ACTIONS(1773),
    [sym__wc_right_string_int] = ACTIONS(1773),
    [sym__wc_upper_string_int] = ACTIONS(1773),
    [sym__wc_compare_string_int] = ACTIONS(1773),
    [sym__wc_get_buffer_char_int] = ACTIONS(1773),
    [sym__wc_get_buffer_string_int] = ACTIONS(1773),
    [sym__wc_concat_string_int] = ACTIONS(1773),
    [sym__wc_file_open_int] = ACTIONS(1773),
    [sym__wc_file_close_int] = ACTIONS(1773),
    [sym__wc_file_read_int] = ACTIONS(1773),
    [sym__wc_file_read_line_int] = ACTIONS(1773),
    [sym__wc_file_write_int] = ACTIONS(1773),
    [sym__wc_file_write_line_int] = ACTIONS(1773),
    [aux_sym_preproc_include_token1] = ACTIONS(1773),
    [aux_sym_preproc_define_token1] = ACTIONS(1775),
    [aux_sym_preproc_warn_token1] = ACTIONS(1775),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1775),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_else_token1] = ACTIONS(1775),
    [anon_sym_LBRACK] = ACTIONS(1775),
    [anon_sym_LBRACE] = ACTIONS(1775),
    [anon_sym_LPAREN] = ACTIONS(1773),
    [anon_sym_BANG] = ACTIONS(1775),
    [anon_sym_TILDE] = ACTIONS(1775),
    [anon_sym_DASH] = ACTIONS(1773),
    [anon_sym_PLUS] = ACTIONS(1773),
    [anon_sym_DASH_DASH] = ACTIONS(1775),
    [anon_sym_PLUS_PLUS] = ACTIONS(1775),
    [anon_sym_DQUOTE] = ACTIONS(1775),
    [anon_sym_SQUOTE] = ACTIONS(1775),
    [sym_number_literal] = ACTIONS(1775),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(230)] = {
    [sym_identifier] = ACTIONS(1777),
    [sym_program_name_keyword] = ACTIONS(1777),
    [sym_module_name_keyword] = ACTIONS(1777),
    [sym_define_device_keyword] = ACTIONS(1777),
    [sym_define_combine_keyword] = ACTIONS(1777),
    [sym_define_constant_keyword] = ACTIONS(1777),
    [sym_define_type_keyword] = ACTIONS(1777),
    [sym_define_variable_keyword] = ACTIONS(1777),
    [sym_define_system_variable_keyword] = ACTIONS(1777),
    [sym_define_connect_level_keyword] = ACTIONS(1777),
    [sym_define_latching_keyword] = ACTIONS(1777),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1777),
    [sym_define_toggling_keyword] = ACTIONS(1777),
    [sym_define_call_keyword] = ACTIONS(1777),
    [sym_define_function_keyword] = ACTIONS(1777),
    [sym_define_library_function_keyword] = ACTIONS(1777),
    [sym_define_start_keyword] = ACTIONS(1777),
    [sym_define_event_keyword] = ACTIONS(1777),
    [sym_define_program_keyword] = ACTIONS(1777),
    [sym_define_module_keyword] = ACTIONS(1777),
    [sym_char_keyword] = ACTIONS(1777),
    [sym_widechar_keyword] = ACTIONS(1777),
    [sym_integer_keyword] = ACTIONS(1777),
    [sym_sinteger_keyword] = ACTIONS(1777),
    [sym_long_keyword] = ACTIONS(1777),
    [sym_slong_keyword] = ACTIONS(1777),
    [sym_float_keyword] = ACTIONS(1777),
    [sym_double_keyword] = ACTIONS(1777),
    [sym_variant_keyword] = ACTIONS(1777),
    [sym_variantarray_keyword] = ACTIONS(1777),
    [sym_local_var_keyword] = ACTIONS(1777),
    [sym_stack_var_keyword] = ACTIONS(1777),
    [sym_constant_keyword] = ACTIONS(1777),
    [sym_volatile_keyword] = ACTIONS(1777),
    [sym_non_volatile_keyword] = ACTIONS(1777),
    [sym_persistent_keyword] = ACTIONS(1777),
    [sym_dev_keyword] = ACTIONS(1777),
    [sym_devlev_keyword] = ACTIONS(1777),
    [sym_devchan_keyword] = ACTIONS(1777),
    [sym_if_keyword] = ACTIONS(1777),
    [sym_else_keyword] = ACTIONS(1777),
    [sym_switch_keyword] = ACTIONS(1777),
    [sym_case_keyword] = ACTIONS(1777),
    [sym_default_keyword] = ACTIONS(1777),
    [sym_while_keyword] = ACTIONS(1777),
    [sym_for_keyword] = ACTIONS(1777),
    [sym_break_keyword] = ACTIONS(1777),
    [sym_continue_keyword] = ACTIONS(1777),
    [sym_return_keyword] = ACTIONS(1777),
    [sym_select_keyword] = ACTIONS(1777),
    [sym_active_keyword] = ACTIONS(1777),
    [sym_struct_keyword] = ACTIONS(1777),
    [sym_structure_keyword] = ACTIONS(1777),
    [sym_button_event_keyword] = ACTIONS(1777),
    [sym_channel_event_keyword] = ACTIONS(1777),
    [sym_level_event_keyword] = ACTIONS(1777),
    [sym_data_event_keyword] = ACTIONS(1777),
    [sym_timeline_event_keyword] = ACTIONS(1777),
    [sym_custom_event_keyword] = ACTIONS(1777),
    [sym_push_keyword] = ACTIONS(1777),
    [sym_release_keyword] = ACTIONS(1777),
    [aux_sym_on_keyword_token1] = ACTIONS(1777),
    [aux_sym_off_keyword_token1] = ACTIONS(1777),
    [sym_send_level_keyword] = ACTIONS(1777),
    [sym_send_string_keyword] = ACTIONS(1777),
    [sym_send_command_keyword] = ACTIONS(1777),
    [sym_clear_buffer_keyword] = ACTIONS(1777),
    [sym_create_buffer_keyword] = ACTIONS(1777),
    [sym_create_multi_buffer_keyword] = ACTIONS(1777),
    [sym_call_keyword] = ACTIONS(1777),
    [sym_system_call_keyword] = ACTIONS(1777),
    [sym_devchan_to_keyword] = ACTIONS(1777),
    [sym_devchan_min_to_keyword] = ACTIONS(1777),
    [sym_devchan_total_off_keyword] = ACTIONS(1777),
    [sym_devchan_pulse_keyword] = ACTIONS(1777),
    [sym_wait_keyword] = ACTIONS(1777),
    [sym_cancel_wait_keyword] = ACTIONS(1777),
    [sym_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1777),
    [sym_not] = ACTIONS(1777),
    [sym_bnot] = ACTIONS(1777),
    [sym___file__] = ACTIONS(1777),
    [sym___date__] = ACTIONS(1777),
    [sym___ldate__] = ACTIONS(1777),
    [sym___time__] = ACTIONS(1777),
    [sym___line__] = ACTIONS(1777),
    [sym___name__] = ACTIONS(1777),
    [sym___netlinx__] = ACTIONS(1777),
    [sym_netlinx_axi_version] = ACTIONS(1777),
    [sym_url_flg_tcp] = ACTIONS(1777),
    [sym_url_flg_accinfopresent] = ACTIONS(1777),
    [sym_url_flg_temp] = ACTIONS(1777),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1777),
    [sym_url_flg_stat_mask] = ACTIONS(1777),
    [sym_url_flg_stat_lookup] = ACTIONS(1777),
    [sym_url_flg_stat_connecting] = ACTIONS(1777),
    [sym_url_flg_stat_waiting] = ACTIONS(1777),
    [sym_url_flg_stat_connected] = ACTIONS(1777),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1777),
    [sym_first_virtual_device] = ACTIONS(1777),
    [sym_first_local_port] = ACTIONS(1777),
    [sym_dynamic_virtual_device] = ACTIONS(1777),
    [sym_timeline_once] = ACTIONS(1777),
    [sym_timeline_repeat] = ACTIONS(1777),
    [sym_timeline_absolute] = ACTIONS(1777),
    [sym_timeline_relative] = ACTIONS(1777),
    [sym_do_push_timed_infinite] = ACTIONS(1777),
    [sym_source_type_no_address] = ACTIONS(1777),
    [sym_source_type_neuron_id] = ACTIONS(1777),
    [sym_source_type_ip_address] = ACTIONS(1777),
    [sym_source_type_axlink] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1777),
    [sym_source_type_ip_socket_address] = ACTIONS(1777),
    [sym_source_type_rs232] = ACTIONS(1777),
    [sym_source_type_internal] = ACTIONS(1777),
    [sym_source_type_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1777),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1777),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1777),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1777),
    [sym_true] = ACTIONS(1777),
    [sym_false] = ACTIONS(1777),
    [sym_file_read_only] = ACTIONS(1777),
    [sym_file_rw_new] = ACTIONS(1777),
    [sym_file_rw_append] = ACTIONS(1777),
    [sym_ip_tcp] = ACTIONS(1777),
    [sym_ip_udp] = ACTIONS(1777),
    [sym_ip_udp_2way] = ACTIONS(1777),
    [sym_xml_encode_types] = ACTIONS(1777),
    [sym_xml_encode_char_as_list] = ACTIONS(1777),
    [sym_xml_encode_le] = ACTIONS(1777),
    [sym_xml_decode_types] = ACTIONS(1777),
    [sym_xml_decode_no_preserve] = ACTIONS(1777),
    [sym_duet_dev_type_display_device] = ACTIONS(1777),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_io_device] = ACTIONS(1777),
    [sym_duet_dev_type_relay_device] = ACTIONS(1777),
    [sym_duet_dev_type_ups] = ACTIONS(1777),
    [sym_duet_dev_type_amplifier] = ACTIONS(1777),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1777),
    [sym_duet_dev_type_camera] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1777),
    [sym_duet_dev_type_dss] = ACTIONS(1777),
    [sym_duet_dev_type_dvr] = ACTIONS(1777),
    [sym_duet_dev_type_disc_device] = ACTIONS(1777),
    [sym_duet_dev_type_document_camera] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1777),
    [sym_duet_dev_type_hvac] = ACTIONS(1777),
    [sym_duet_dev_type_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_light] = ACTIONS(1777),
    [sym_duet_dev_type_monitor] = ACTIONS(1777),
    [sym_duet_dev_type_motor] = ACTIONS(1777),
    [sym_duet_dev_type_multi_window] = ACTIONS(1777),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1777),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1777),
    [sym_duet_dev_type_receiver] = ACTIONS(1777),
    [sym_duet_dev_type_security_system] = ACTIONS(1777),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1777),
    [sym_duet_dev_type_settop_box] = ACTIONS(1777),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1777),
    [sym_duet_dev_type_switcher] = ACTIONS(1777),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_tv] = ACTIONS(1777),
    [sym_duet_dev_type_utility] = ACTIONS(1777),
    [sym_duet_dev_type_vcr] = ACTIONS(1777),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_video_processor] = ACTIONS(1777),
    [sym_duet_dev_type_video_projector] = ACTIONS(1777),
    [sym_duet_dev_type_video_wall] = ACTIONS(1777),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1777),
    [sym_duet_dev_type_weather] = ACTIONS(1777),
    [sym_duet_dev_not_polled] = ACTIONS(1777),
    [sym_duet_dev_polled] = ACTIONS(1777),
    [sym_clkmgr_mode_network] = ACTIONS(1777),
    [sym_clkmgr_mode_standalone] = ACTIONS(1777),
    [sym_normal_standby] = ACTIONS(1777),
    [sym_normal_wake] = ACTIONS(1777),
    [sym_ip_multicast_ttl_option] = ACTIONS(1777),
    [sym_ip_ttl_subnet] = ACTIONS(1777),
    [sym_ip_ttl_site] = ACTIONS(1777),
    [sym_ip_ttl_region] = ACTIONS(1777),
    [sym_ip_ttl_continent] = ACTIONS(1777),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1777),
    [sym_ip_nodelay_on] = ACTIONS(1777),
    [sym_ip_nodelay_off] = ACTIONS(1777),
    [sym_amx_error] = ACTIONS(1777),
    [sym_amx_warning] = ACTIONS(1777),
    [sym_amx_info] = ACTIONS(1777),
    [sym_amx_debug] = ACTIONS(1777),
    [sym_netlinx_login_success] = ACTIONS(1777),
    [sym_netlinx_login_fail] = ACTIONS(1777),
    [sym_netlinx_logout] = ACTIONS(1777),
    [sym_null_str] = ACTIONS(1777),
    [sym_smtp_address] = ACTIONS(1777),
    [sym_smtp_port_number] = ACTIONS(1777),
    [sym_smtp_username] = ACTIONS(1777),
    [sym_smtp_password] = ACTIONS(1777),
    [sym_smtp_require_tls] = ACTIONS(1777),
    [sym_smtp_from] = ACTIONS(1777),
    [sym_smtp_tls_true] = ACTIONS(1777),
    [sym_smtp_tls_false] = ACTIONS(1777),
    [sym_listview_on_row_select_event] = ACTIONS(1777),
    [sym_data_structure_unknown] = ACTIONS(1777),
    [sym_data_structure_datafeed] = ACTIONS(1777),
    [sym_data_structure_datarecordset] = ACTIONS(1777),
    [sym_data_structure_datarecord] = ACTIONS(1777),
    [sym_data_structure_datafield] = ACTIONS(1777),
    [sym_data_max_content_field_count] = ACTIONS(1777),
    [sym_data_max_metadata_field_count] = ACTIONS(1777),
    [sym_data_max_id_length] = ACTIONS(1777),
    [sym_data_max_name_length] = ACTIONS(1777),
    [sym_data_max_description_length] = ACTIONS(1777),
    [sym_data_max_type_length] = ACTIONS(1777),
    [sym_data_max_format_length] = ACTIONS(1777),
    [sym_data_max_source_length] = ACTIONS(1777),
    [sym_data_max_value_length] = ACTIONS(1777),
    [sym_data_max_label_length] = ACTIONS(1777),
    [sym_data_max_url_length] = ACTIONS(1777),
    [sym_max_last_login_info_length] = ACTIONS(1777),
    [sym_data_type_unknown] = ACTIONS(1777),
    [sym_data_type_string] = ACTIONS(1777),
    [sym_data_type_datetime] = ACTIONS(1777),
    [sym_data_type_date] = ACTIONS(1777),
    [sym_data_type_time] = ACTIONS(1777),
    [sym_data_type_image] = ACTIONS(1777),
    [sym_data_format_url] = ACTIONS(1777),
    [sym_data_format_phone] = ACTIONS(1777),
    [sym_data_format_email] = ACTIONS(1777),
    [sym_data_format_iso8601] = ACTIONS(1777),
    [sym_auditlog_priv] = ACTIONS(1777),
    [sym_configuration_priv] = ACTIONS(1777),
    [sym_deviceconfig_priv] = ACTIONS(1777),
    [sym_ftp_priv] = ACTIONS(1777),
    [sym_http_priv] = ACTIONS(1777),
    [sym_networkconfig_priv] = ACTIONS(1777),
    [sym_programport_priv] = ACTIONS(1777),
    [sym_remoteui_priv] = ACTIONS(1777),
    [sym_securitycontrol_priv] = ACTIONS(1777),
    [sym_softwaremanagement_priv] = ACTIONS(1777),
    [sym_terminal_priv] = ACTIONS(1777),
    [sym_tpadmin_priv] = ACTIONS(1777),
    [sym_user1_priv] = ACTIONS(1777),
    [sym_user2_priv] = ACTIONS(1777),
    [sym_user3_priv] = ACTIONS(1777),
    [sym_user4_priv] = ACTIONS(1777),
    [sym_usermanagement_priv] = ACTIONS(1777),
    [sym_valid_account] = ACTIONS(1777),
    [sym_err_name_invalid] = ACTIONS(1777),
    [sym_err_password_invalid] = ACTIONS(1777),
    [sym_err_invalid_account] = ACTIONS(1777),
    [sym_err_no_authorization] = ACTIONS(1777),
    [sym_err_invalid_parameter] = ACTIONS(1777),
    [sym_err_user_locked_out] = ACTIONS(1777),
    [sym_err_user_account_expired] = ACTIONS(1777),
    [sym_err_general_error] = ACTIONS(1777),
    [sym_err_authentication_error] = ACTIONS(1777),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1777),
    [sym_tls_validate_certificate] = ACTIONS(1777),
    [sym_tdata] = ACTIONS(1777),
    [sym_tchannel] = ACTIONS(1777),
    [sym_tlevel] = ACTIONS(1777),
    [sym_tbutton] = ACTIONS(1777),
    [sym_ttimeline] = ACTIONS(1777),
    [sym_tcustom] = ACTIONS(1777),
    [sym_url_struct] = ACTIONS(1777),
    [sym_dns_struct] = ACTIONS(1777),
    [sym_ip_address_struct] = ACTIONS(1777),
    [sym_dev_info_struct] = ACTIONS(1777),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1777),
    [sym_data_feed] = ACTIONS(1777),
    [sym_data_field] = ACTIONS(1777),
    [sym_data_record] = ACTIONS(1777),
    [sym_wc_data_feed] = ACTIONS(1777),
    [sym_wc_data_field] = ACTIONS(1777),
    [sym_wc_data_record] = ACTIONS(1777),
    [sym_last_login_info] = ACTIONS(1777),
    [sym____reserved___] = ACTIONS(1777),
    [sym_date] = ACTIONS(1777),
    [sym_day] = ACTIONS(1777),
    [sym_time] = ACTIONS(1777),
    [sym_ldate] = ACTIONS(1777),
    [sym_push_device] = ACTIONS(1777),
    [sym_push_channel] = ACTIONS(1777),
    [sym_push_devchan] = ACTIONS(1777),
    [sym_release_device] = ACTIONS(1777),
    [sym_release_channel] = ACTIONS(1777),
    [sym_release_devchan] = ACTIONS(1777),
    [sym_master_slot] = ACTIONS(1777),
    [sym_get_pulse_time] = ACTIONS(1777),
    [sym_get_timer] = ACTIONS(1777),
    [sym_master_sn] = ACTIONS(1777),
    [sym_system_number] = ACTIONS(1777),
    [sym_button] = ACTIONS(1777),
    [sym_dv_channel] = ACTIONS(1777),
    [sym_data] = ACTIONS(1777),
    [sym_level] = ACTIONS(1777),
    [sym_timeline] = ACTIONS(1777),
    [sym_channel] = ACTIONS(1777),
    [sym_custom] = ACTIONS(1777),
    [sym_atoi] = ACTIONS(1777),
    [sym_atol] = ACTIONS(1777),
    [sym_atof] = ACTIONS(1777),
    [sym_itoa] = ACTIONS(1777),
    [sym_itohex] = ACTIONS(1777),
    [sym_hextoi] = ACTIONS(1777),
    [sym_ftoa] = ACTIONS(1777),
    [sym_find_string] = ACTIONS(1777),
    [sym_left_string] = ACTIONS(1777),
    [sym_right_string] = ACTIONS(1777),
    [sym_mid_string] = ACTIONS(1777),
    [sym_remove_string] = ACTIONS(1777),
    [sym_get_buffer_string] = ACTIONS(1777),
    [sym_get_buffer_char] = ACTIONS(1777),
    [sym_get_multi_buffer_string] = ACTIONS(1777),
    [sym_length_string] = ACTIONS(1777),
    [sym_max_length_string] = ACTIONS(1777),
    [sym_set_length_string] = ACTIONS(1777),
    [sym_compare_string] = ACTIONS(1777),
    [sym_upper_string] = ACTIONS(1777),
    [sym_lower_string] = ACTIONS(1777),
    [sym_get_last] = ACTIONS(1777),
    [sym_date_to_day] = ACTIONS(1777),
    [sym_date_to_month] = ACTIONS(1777),
    [sym_date_to_year] = ACTIONS(1777),
    [sym_time_to_hour] = ACTIONS(1777),
    [sym_time_to_minute] = ACTIONS(1777),
    [sym_time_to_second] = ACTIONS(1777),
    [sym_day_of_week] = ACTIONS(1777),
    [sym_set_pulse_time] = ACTIONS(1777),
    [sym_set_timer] = ACTIONS(1777),
    [sym_file_open] = ACTIONS(1777),
    [sym_file_close] = ACTIONS(1777),
    [sym_file_read] = ACTIONS(1777),
    [sym_file_read_line] = ACTIONS(1777),
    [sym_file_write] = ACTIONS(1777),
    [sym_file_write_line] = ACTIONS(1777),
    [sym_file_copy] = ACTIONS(1777),
    [sym_file_delete] = ACTIONS(1777),
    [sym_file_rename] = ACTIONS(1777),
    [sym_file_dir] = ACTIONS(1777),
    [sym_file_getdir] = ACTIONS(1777),
    [sym_file_setdir] = ACTIONS(1777),
    [sym_file_create_dir] = ACTIONS(1777),
    [sym_file_remove_dir] = ACTIONS(1777),
    [sym_file_seek] = ACTIONS(1777),
    [sym_length_array] = ACTIONS(1777),
    [sym_max_length_array] = ACTIONS(1777),
    [sym_set_length_array] = ACTIONS(1777),
    [sym_ip_client_open] = ACTIONS(1777),
    [sym_ip_client_close] = ACTIONS(1777),
    [sym_ip_server_open] = ACTIONS(1777),
    [sym_ip_server_close] = ACTIONS(1777),
    [sym_ip_bound_client_open] = ACTIONS(1777),
    [sym_ip_mc_server_open] = ACTIONS(1777),
    [sym_ip_set_option] = ACTIONS(1777),
    [sym_ssh_client_open] = ACTIONS(1777),
    [sym_ssh_client_close] = ACTIONS(1777),
    [sym_tls_client_open] = ACTIONS(1777),
    [sym_tls_client_close] = ACTIONS(1777),
    [sym_do_push] = ACTIONS(1777),
    [sym_do_push_timed] = ACTIONS(1777),
    [sym_do_release] = ACTIONS(1777),
    [sym_device_id] = ACTIONS(1777),
    [sym_device_info] = ACTIONS(1777),
    [sym_device_id_string] = ACTIONS(1777),
    [sym_device_standby] = ACTIONS(1777),
    [sym_device_wake] = ACTIONS(1777),
    [sym_timeline_create] = ACTIONS(1777),
    [sym_timeline_kill] = ACTIONS(1777),
    [sym_timeline_reload] = ACTIONS(1777),
    [sym_timeline_pause] = ACTIONS(1777),
    [sym_timeline_restart] = ACTIONS(1777),
    [sym_timeline_set] = ACTIONS(1777),
    [sym_timeline_get] = ACTIONS(1777),
    [sym_timeline_active] = ACTIONS(1777),
    [sym_timeline_dynamic_id] = ACTIONS(1777),
    [sym_variable_to_string] = ACTIONS(1777),
    [sym_string_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_string] = ACTIONS(1777),
    [sym_type_cast] = ACTIONS(1777),
    [sym_variable_to_xml] = ACTIONS(1777),
    [sym_xml_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_xml] = ACTIONS(1777),
    [sym_redirect_string] = ACTIONS(1777),
    [sym_random_number] = ACTIONS(1777),
    [sym_get_unique_id] = ACTIONS(1777),
    [sym_get_system_number] = ACTIONS(1777),
    [sym_set_system_number] = ACTIONS(1777),
    [sym_get_serial_number] = ACTIONS(1777),
    [sym_get_master_build] = ACTIONS(1777),
    [sym_set_validation_code] = ACTIONS(1777),
    [sym_reboot] = ACTIONS(1777),
    [sym_load_duet_module] = ACTIONS(1777),
    [sym_unload_duet_module] = ACTIONS(1777),
    [sym_duet_mem_size_set] = ACTIONS(1777),
    [sym_duet_mem_size_get] = ACTIONS(1777),
    [sym_dynamic_application_device] = ACTIONS(1777),
    [sym_dynamic_polled_port] = ACTIONS(1777),
    [sym_static_port_binding] = ACTIONS(1777),
    [sym_static_ip_binding] = ACTIONS(1777),
    [sym_get_dns_list] = ACTIONS(1777),
    [sym_set_dns_list] = ACTIONS(1777),
    [sym_get_ip_address] = ACTIONS(1777),
    [sym_set_ip_address] = ACTIONS(1777),
    [sym_get_url_list] = ACTIONS(1777),
    [sym_add_url_entry] = ACTIONS(1777),
    [sym_delete_url_entry] = ACTIONS(1777),
    [sym_abs_value] = ACTIONS(1777),
    [sym_max_value] = ACTIONS(1777),
    [sym_min_value] = ACTIONS(1777),
    [sym_exp_value] = ACTIONS(1777),
    [sym_log_value] = ACTIONS(1777),
    [sym_log10_value] = ACTIONS(1777),
    [sym_power_value] = ACTIONS(1777),
    [sym_sqrt_value] = ACTIONS(1777),
    [sym_raw_be] = ACTIONS(1777),
    [sym_raw_le] = ACTIONS(1777),
    [sym_internal_queue_size_set] = ACTIONS(1777),
    [sym_internal_queue_size_get] = ACTIONS(1777),
    [sym_internal_threshold_set] = ACTIONS(1777),
    [sym_internal_threshold_get] = ACTIONS(1777),
    [sym_rebuilt_event] = ACTIONS(1777),
    [sym_do_custom_event] = ACTIONS(1777),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1777),
    [sym_clkmgr_set_clk_source] = ACTIONS(1777),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1777),
    [sym_clkmgr_get_timezone] = ACTIONS(1777),
    [sym_clkmgr_set_timezone] = ACTIONS(1777),
    [sym_clkmgr_get_resync_period] = ACTIONS(1777),
    [sym_clkmgr_set_resync_period] = ACTIONS(1777),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_timeservers] = ACTIONS(1777),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_set_log_level] = ACTIONS(1777),
    [sym_get_log_level] = ACTIONS(1777),
    [sym_amx_log] = ACTIONS(1777),
    [sym_smtp_server_config_set] = ACTIONS(1777),
    [sym_smtp_server_config_get] = ACTIONS(1777),
    [sym_smtp_send] = ACTIONS(1777),
    [sym_validate_netlinx_account] = ACTIONS(1777),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1777),
    [sym_authenticate_certificate] = ACTIONS(1777),
    [sym_audit_netlinx_session_event] = ACTIONS(1777),
    [sym_audit_netlinx_generic_event] = ACTIONS(1777),
    [sym_get_available_flash_disk_space] = ACTIONS(1777),
    [sym_get_max_flash_disk_space] = ACTIONS(1777),
    [sym_data_create_feed] = ACTIONS(1777),
    [sym_data_delete_feed] = ACTIONS(1777),
    [sym_data_publish_feed] = ACTIONS(1777),
    [sym_data_get_published_feed] = ACTIONS(1777),
    [sym_data_add_record] = ACTIONS(1777),
    [sym_data_get_event_record] = ACTIONS(1777),
    [sym__wc_data_create_feed] = ACTIONS(1777),
    [sym__wc_data_add_record] = ACTIONS(1777),
    [sym__wc_data_get_event_record] = ACTIONS(1777),
    [sym_format] = ACTIONS(1777),
    [sym_astro_clock] = ACTIONS(1777),
    [sym_set_outdoor_temperature] = ACTIONS(1777),
    [sym_set_virtual_level_count] = ACTIONS(1777),
    [sym_set_virtual_channel_count] = ACTIONS(1777),
    [sym_set_virtual_port_count] = ACTIONS(1777),
    [sym__wc_encode_int] = ACTIONS(1777),
    [sym__wc_decode_int] = ACTIONS(1777),
    [sym__wc_int] = ACTIONS(1777),
    [sym__wc_to_ch_int] = ACTIONS(1777),
    [sym__ch_to_wc_int] = ACTIONS(1777),
    [sym__wc_find_string_int] = ACTIONS(1777),
    [sym__wc_left_string_int] = ACTIONS(1777),
    [sym__wc_lower_string_int] = ACTIONS(1777),
    [sym__wc_mid_string_int] = ACTIONS(1777),
    [sym__wc_remove_string_int] = ACTIONS(1777),
    [sym__wc_right_string_int] = ACTIONS(1777),
    [sym__wc_upper_string_int] = ACTIONS(1777),
    [sym__wc_compare_string_int] = ACTIONS(1777),
    [sym__wc_get_buffer_char_int] = ACTIONS(1777),
    [sym__wc_get_buffer_string_int] = ACTIONS(1777),
    [sym__wc_concat_string_int] = ACTIONS(1777),
    [sym__wc_file_open_int] = ACTIONS(1777),
    [sym__wc_file_close_int] = ACTIONS(1777),
    [sym__wc_file_read_int] = ACTIONS(1777),
    [sym__wc_file_read_line_int] = ACTIONS(1777),
    [sym__wc_file_write_int] = ACTIONS(1777),
    [sym__wc_file_write_line_int] = ACTIONS(1777),
    [aux_sym_preproc_include_token1] = ACTIONS(1777),
    [aux_sym_preproc_define_token1] = ACTIONS(1779),
    [aux_sym_preproc_warn_token1] = ACTIONS(1779),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1779),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_else_token1] = ACTIONS(1779),
    [anon_sym_LBRACK] = ACTIONS(1779),
    [anon_sym_LBRACE] = ACTIONS(1779),
    [anon_sym_LPAREN] = ACTIONS(1777),
    [anon_sym_BANG] = ACTIONS(1779),
    [anon_sym_TILDE] = ACTIONS(1779),
    [anon_sym_DASH] = ACTIONS(1777),
    [anon_sym_PLUS] = ACTIONS(1777),
    [anon_sym_DASH_DASH] = ACTIONS(1779),
    [anon_sym_PLUS_PLUS] = ACTIONS(1779),
    [anon_sym_DQUOTE] = ACTIONS(1779),
    [anon_sym_SQUOTE] = ACTIONS(1779),
    [sym_number_literal] = ACTIONS(1779),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(231)] = {
    [sym_identifier] = ACTIONS(1777),
    [sym_program_name_keyword] = ACTIONS(1777),
    [sym_module_name_keyword] = ACTIONS(1777),
    [sym_define_device_keyword] = ACTIONS(1777),
    [sym_define_combine_keyword] = ACTIONS(1777),
    [sym_define_constant_keyword] = ACTIONS(1777),
    [sym_define_type_keyword] = ACTIONS(1777),
    [sym_define_variable_keyword] = ACTIONS(1777),
    [sym_define_system_variable_keyword] = ACTIONS(1777),
    [sym_define_connect_level_keyword] = ACTIONS(1777),
    [sym_define_latching_keyword] = ACTIONS(1777),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1777),
    [sym_define_toggling_keyword] = ACTIONS(1777),
    [sym_define_call_keyword] = ACTIONS(1777),
    [sym_define_function_keyword] = ACTIONS(1777),
    [sym_define_library_function_keyword] = ACTIONS(1777),
    [sym_define_start_keyword] = ACTIONS(1777),
    [sym_define_event_keyword] = ACTIONS(1777),
    [sym_define_program_keyword] = ACTIONS(1777),
    [sym_define_module_keyword] = ACTIONS(1777),
    [sym_char_keyword] = ACTIONS(1777),
    [sym_widechar_keyword] = ACTIONS(1777),
    [sym_integer_keyword] = ACTIONS(1777),
    [sym_sinteger_keyword] = ACTIONS(1777),
    [sym_long_keyword] = ACTIONS(1777),
    [sym_slong_keyword] = ACTIONS(1777),
    [sym_float_keyword] = ACTIONS(1777),
    [sym_double_keyword] = ACTIONS(1777),
    [sym_variant_keyword] = ACTIONS(1777),
    [sym_variantarray_keyword] = ACTIONS(1777),
    [sym_local_var_keyword] = ACTIONS(1777),
    [sym_stack_var_keyword] = ACTIONS(1777),
    [sym_constant_keyword] = ACTIONS(1777),
    [sym_volatile_keyword] = ACTIONS(1777),
    [sym_non_volatile_keyword] = ACTIONS(1777),
    [sym_persistent_keyword] = ACTIONS(1777),
    [sym_dev_keyword] = ACTIONS(1777),
    [sym_devlev_keyword] = ACTIONS(1777),
    [sym_devchan_keyword] = ACTIONS(1777),
    [sym_if_keyword] = ACTIONS(1777),
    [sym_else_keyword] = ACTIONS(1777),
    [sym_switch_keyword] = ACTIONS(1777),
    [sym_case_keyword] = ACTIONS(1777),
    [sym_default_keyword] = ACTIONS(1777),
    [sym_while_keyword] = ACTIONS(1777),
    [sym_for_keyword] = ACTIONS(1777),
    [sym_break_keyword] = ACTIONS(1777),
    [sym_continue_keyword] = ACTIONS(1777),
    [sym_return_keyword] = ACTIONS(1777),
    [sym_select_keyword] = ACTIONS(1777),
    [sym_active_keyword] = ACTIONS(1777),
    [sym_struct_keyword] = ACTIONS(1777),
    [sym_structure_keyword] = ACTIONS(1777),
    [sym_button_event_keyword] = ACTIONS(1777),
    [sym_channel_event_keyword] = ACTIONS(1777),
    [sym_level_event_keyword] = ACTIONS(1777),
    [sym_data_event_keyword] = ACTIONS(1777),
    [sym_timeline_event_keyword] = ACTIONS(1777),
    [sym_custom_event_keyword] = ACTIONS(1777),
    [sym_push_keyword] = ACTIONS(1777),
    [sym_release_keyword] = ACTIONS(1777),
    [aux_sym_on_keyword_token1] = ACTIONS(1777),
    [aux_sym_off_keyword_token1] = ACTIONS(1777),
    [sym_send_level_keyword] = ACTIONS(1777),
    [sym_send_string_keyword] = ACTIONS(1777),
    [sym_send_command_keyword] = ACTIONS(1777),
    [sym_clear_buffer_keyword] = ACTIONS(1777),
    [sym_create_buffer_keyword] = ACTIONS(1777),
    [sym_create_multi_buffer_keyword] = ACTIONS(1777),
    [sym_call_keyword] = ACTIONS(1777),
    [sym_system_call_keyword] = ACTIONS(1777),
    [sym_devchan_to_keyword] = ACTIONS(1777),
    [sym_devchan_min_to_keyword] = ACTIONS(1777),
    [sym_devchan_total_off_keyword] = ACTIONS(1777),
    [sym_devchan_pulse_keyword] = ACTIONS(1777),
    [sym_wait_keyword] = ACTIONS(1777),
    [sym_cancel_wait_keyword] = ACTIONS(1777),
    [sym_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1777),
    [sym_not] = ACTIONS(1777),
    [sym_bnot] = ACTIONS(1777),
    [sym___file__] = ACTIONS(1777),
    [sym___date__] = ACTIONS(1777),
    [sym___ldate__] = ACTIONS(1777),
    [sym___time__] = ACTIONS(1777),
    [sym___line__] = ACTIONS(1777),
    [sym___name__] = ACTIONS(1777),
    [sym___netlinx__] = ACTIONS(1777),
    [sym_netlinx_axi_version] = ACTIONS(1777),
    [sym_url_flg_tcp] = ACTIONS(1777),
    [sym_url_flg_accinfopresent] = ACTIONS(1777),
    [sym_url_flg_temp] = ACTIONS(1777),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1777),
    [sym_url_flg_stat_mask] = ACTIONS(1777),
    [sym_url_flg_stat_lookup] = ACTIONS(1777),
    [sym_url_flg_stat_connecting] = ACTIONS(1777),
    [sym_url_flg_stat_waiting] = ACTIONS(1777),
    [sym_url_flg_stat_connected] = ACTIONS(1777),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1777),
    [sym_first_virtual_device] = ACTIONS(1777),
    [sym_first_local_port] = ACTIONS(1777),
    [sym_dynamic_virtual_device] = ACTIONS(1777),
    [sym_timeline_once] = ACTIONS(1777),
    [sym_timeline_repeat] = ACTIONS(1777),
    [sym_timeline_absolute] = ACTIONS(1777),
    [sym_timeline_relative] = ACTIONS(1777),
    [sym_do_push_timed_infinite] = ACTIONS(1777),
    [sym_source_type_no_address] = ACTIONS(1777),
    [sym_source_type_neuron_id] = ACTIONS(1777),
    [sym_source_type_ip_address] = ACTIONS(1777),
    [sym_source_type_axlink] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1777),
    [sym_source_type_ip_socket_address] = ACTIONS(1777),
    [sym_source_type_rs232] = ACTIONS(1777),
    [sym_source_type_internal] = ACTIONS(1777),
    [sym_source_type_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1777),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1777),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1777),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1777),
    [sym_true] = ACTIONS(1777),
    [sym_false] = ACTIONS(1777),
    [sym_file_read_only] = ACTIONS(1777),
    [sym_file_rw_new] = ACTIONS(1777),
    [sym_file_rw_append] = ACTIONS(1777),
    [sym_ip_tcp] = ACTIONS(1777),
    [sym_ip_udp] = ACTIONS(1777),
    [sym_ip_udp_2way] = ACTIONS(1777),
    [sym_xml_encode_types] = ACTIONS(1777),
    [sym_xml_encode_char_as_list] = ACTIONS(1777),
    [sym_xml_encode_le] = ACTIONS(1777),
    [sym_xml_decode_types] = ACTIONS(1777),
    [sym_xml_decode_no_preserve] = ACTIONS(1777),
    [sym_duet_dev_type_display_device] = ACTIONS(1777),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_io_device] = ACTIONS(1777),
    [sym_duet_dev_type_relay_device] = ACTIONS(1777),
    [sym_duet_dev_type_ups] = ACTIONS(1777),
    [sym_duet_dev_type_amplifier] = ACTIONS(1777),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1777),
    [sym_duet_dev_type_camera] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1777),
    [sym_duet_dev_type_dss] = ACTIONS(1777),
    [sym_duet_dev_type_dvr] = ACTIONS(1777),
    [sym_duet_dev_type_disc_device] = ACTIONS(1777),
    [sym_duet_dev_type_document_camera] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1777),
    [sym_duet_dev_type_hvac] = ACTIONS(1777),
    [sym_duet_dev_type_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_light] = ACTIONS(1777),
    [sym_duet_dev_type_monitor] = ACTIONS(1777),
    [sym_duet_dev_type_motor] = ACTIONS(1777),
    [sym_duet_dev_type_multi_window] = ACTIONS(1777),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1777),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1777),
    [sym_duet_dev_type_receiver] = ACTIONS(1777),
    [sym_duet_dev_type_security_system] = ACTIONS(1777),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1777),
    [sym_duet_dev_type_settop_box] = ACTIONS(1777),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1777),
    [sym_duet_dev_type_switcher] = ACTIONS(1777),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_tv] = ACTIONS(1777),
    [sym_duet_dev_type_utility] = ACTIONS(1777),
    [sym_duet_dev_type_vcr] = ACTIONS(1777),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_video_processor] = ACTIONS(1777),
    [sym_duet_dev_type_video_projector] = ACTIONS(1777),
    [sym_duet_dev_type_video_wall] = ACTIONS(1777),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1777),
    [sym_duet_dev_type_weather] = ACTIONS(1777),
    [sym_duet_dev_not_polled] = ACTIONS(1777),
    [sym_duet_dev_polled] = ACTIONS(1777),
    [sym_clkmgr_mode_network] = ACTIONS(1777),
    [sym_clkmgr_mode_standalone] = ACTIONS(1777),
    [sym_normal_standby] = ACTIONS(1777),
    [sym_normal_wake] = ACTIONS(1777),
    [sym_ip_multicast_ttl_option] = ACTIONS(1777),
    [sym_ip_ttl_subnet] = ACTIONS(1777),
    [sym_ip_ttl_site] = ACTIONS(1777),
    [sym_ip_ttl_region] = ACTIONS(1777),
    [sym_ip_ttl_continent] = ACTIONS(1777),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1777),
    [sym_ip_nodelay_on] = ACTIONS(1777),
    [sym_ip_nodelay_off] = ACTIONS(1777),
    [sym_amx_error] = ACTIONS(1777),
    [sym_amx_warning] = ACTIONS(1777),
    [sym_amx_info] = ACTIONS(1777),
    [sym_amx_debug] = ACTIONS(1777),
    [sym_netlinx_login_success] = ACTIONS(1777),
    [sym_netlinx_login_fail] = ACTIONS(1777),
    [sym_netlinx_logout] = ACTIONS(1777),
    [sym_null_str] = ACTIONS(1777),
    [sym_smtp_address] = ACTIONS(1777),
    [sym_smtp_port_number] = ACTIONS(1777),
    [sym_smtp_username] = ACTIONS(1777),
    [sym_smtp_password] = ACTIONS(1777),
    [sym_smtp_require_tls] = ACTIONS(1777),
    [sym_smtp_from] = ACTIONS(1777),
    [sym_smtp_tls_true] = ACTIONS(1777),
    [sym_smtp_tls_false] = ACTIONS(1777),
    [sym_listview_on_row_select_event] = ACTIONS(1777),
    [sym_data_structure_unknown] = ACTIONS(1777),
    [sym_data_structure_datafeed] = ACTIONS(1777),
    [sym_data_structure_datarecordset] = ACTIONS(1777),
    [sym_data_structure_datarecord] = ACTIONS(1777),
    [sym_data_structure_datafield] = ACTIONS(1777),
    [sym_data_max_content_field_count] = ACTIONS(1777),
    [sym_data_max_metadata_field_count] = ACTIONS(1777),
    [sym_data_max_id_length] = ACTIONS(1777),
    [sym_data_max_name_length] = ACTIONS(1777),
    [sym_data_max_description_length] = ACTIONS(1777),
    [sym_data_max_type_length] = ACTIONS(1777),
    [sym_data_max_format_length] = ACTIONS(1777),
    [sym_data_max_source_length] = ACTIONS(1777),
    [sym_data_max_value_length] = ACTIONS(1777),
    [sym_data_max_label_length] = ACTIONS(1777),
    [sym_data_max_url_length] = ACTIONS(1777),
    [sym_max_last_login_info_length] = ACTIONS(1777),
    [sym_data_type_unknown] = ACTIONS(1777),
    [sym_data_type_string] = ACTIONS(1777),
    [sym_data_type_datetime] = ACTIONS(1777),
    [sym_data_type_date] = ACTIONS(1777),
    [sym_data_type_time] = ACTIONS(1777),
    [sym_data_type_image] = ACTIONS(1777),
    [sym_data_format_url] = ACTIONS(1777),
    [sym_data_format_phone] = ACTIONS(1777),
    [sym_data_format_email] = ACTIONS(1777),
    [sym_data_format_iso8601] = ACTIONS(1777),
    [sym_auditlog_priv] = ACTIONS(1777),
    [sym_configuration_priv] = ACTIONS(1777),
    [sym_deviceconfig_priv] = ACTIONS(1777),
    [sym_ftp_priv] = ACTIONS(1777),
    [sym_http_priv] = ACTIONS(1777),
    [sym_networkconfig_priv] = ACTIONS(1777),
    [sym_programport_priv] = ACTIONS(1777),
    [sym_remoteui_priv] = ACTIONS(1777),
    [sym_securitycontrol_priv] = ACTIONS(1777),
    [sym_softwaremanagement_priv] = ACTIONS(1777),
    [sym_terminal_priv] = ACTIONS(1777),
    [sym_tpadmin_priv] = ACTIONS(1777),
    [sym_user1_priv] = ACTIONS(1777),
    [sym_user2_priv] = ACTIONS(1777),
    [sym_user3_priv] = ACTIONS(1777),
    [sym_user4_priv] = ACTIONS(1777),
    [sym_usermanagement_priv] = ACTIONS(1777),
    [sym_valid_account] = ACTIONS(1777),
    [sym_err_name_invalid] = ACTIONS(1777),
    [sym_err_password_invalid] = ACTIONS(1777),
    [sym_err_invalid_account] = ACTIONS(1777),
    [sym_err_no_authorization] = ACTIONS(1777),
    [sym_err_invalid_parameter] = ACTIONS(1777),
    [sym_err_user_locked_out] = ACTIONS(1777),
    [sym_err_user_account_expired] = ACTIONS(1777),
    [sym_err_general_error] = ACTIONS(1777),
    [sym_err_authentication_error] = ACTIONS(1777),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1777),
    [sym_tls_validate_certificate] = ACTIONS(1777),
    [sym_tdata] = ACTIONS(1777),
    [sym_tchannel] = ACTIONS(1777),
    [sym_tlevel] = ACTIONS(1777),
    [sym_tbutton] = ACTIONS(1777),
    [sym_ttimeline] = ACTIONS(1777),
    [sym_tcustom] = ACTIONS(1777),
    [sym_url_struct] = ACTIONS(1777),
    [sym_dns_struct] = ACTIONS(1777),
    [sym_ip_address_struct] = ACTIONS(1777),
    [sym_dev_info_struct] = ACTIONS(1777),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1777),
    [sym_data_feed] = ACTIONS(1777),
    [sym_data_field] = ACTIONS(1777),
    [sym_data_record] = ACTIONS(1777),
    [sym_wc_data_feed] = ACTIONS(1777),
    [sym_wc_data_field] = ACTIONS(1777),
    [sym_wc_data_record] = ACTIONS(1777),
    [sym_last_login_info] = ACTIONS(1777),
    [sym____reserved___] = ACTIONS(1777),
    [sym_date] = ACTIONS(1777),
    [sym_day] = ACTIONS(1777),
    [sym_time] = ACTIONS(1777),
    [sym_ldate] = ACTIONS(1777),
    [sym_push_device] = ACTIONS(1777),
    [sym_push_channel] = ACTIONS(1777),
    [sym_push_devchan] = ACTIONS(1777),
    [sym_release_device] = ACTIONS(1777),
    [sym_release_channel] = ACTIONS(1777),
    [sym_release_devchan] = ACTIONS(1777),
    [sym_master_slot] = ACTIONS(1777),
    [sym_get_pulse_time] = ACTIONS(1777),
    [sym_get_timer] = ACTIONS(1777),
    [sym_master_sn] = ACTIONS(1777),
    [sym_system_number] = ACTIONS(1777),
    [sym_button] = ACTIONS(1777),
    [sym_dv_channel] = ACTIONS(1777),
    [sym_data] = ACTIONS(1777),
    [sym_level] = ACTIONS(1777),
    [sym_timeline] = ACTIONS(1777),
    [sym_channel] = ACTIONS(1777),
    [sym_custom] = ACTIONS(1777),
    [sym_atoi] = ACTIONS(1777),
    [sym_atol] = ACTIONS(1777),
    [sym_atof] = ACTIONS(1777),
    [sym_itoa] = ACTIONS(1777),
    [sym_itohex] = ACTIONS(1777),
    [sym_hextoi] = ACTIONS(1777),
    [sym_ftoa] = ACTIONS(1777),
    [sym_find_string] = ACTIONS(1777),
    [sym_left_string] = ACTIONS(1777),
    [sym_right_string] = ACTIONS(1777),
    [sym_mid_string] = ACTIONS(1777),
    [sym_remove_string] = ACTIONS(1777),
    [sym_get_buffer_string] = ACTIONS(1777),
    [sym_get_buffer_char] = ACTIONS(1777),
    [sym_get_multi_buffer_string] = ACTIONS(1777),
    [sym_length_string] = ACTIONS(1777),
    [sym_max_length_string] = ACTIONS(1777),
    [sym_set_length_string] = ACTIONS(1777),
    [sym_compare_string] = ACTIONS(1777),
    [sym_upper_string] = ACTIONS(1777),
    [sym_lower_string] = ACTIONS(1777),
    [sym_get_last] = ACTIONS(1777),
    [sym_date_to_day] = ACTIONS(1777),
    [sym_date_to_month] = ACTIONS(1777),
    [sym_date_to_year] = ACTIONS(1777),
    [sym_time_to_hour] = ACTIONS(1777),
    [sym_time_to_minute] = ACTIONS(1777),
    [sym_time_to_second] = ACTIONS(1777),
    [sym_day_of_week] = ACTIONS(1777),
    [sym_set_pulse_time] = ACTIONS(1777),
    [sym_set_timer] = ACTIONS(1777),
    [sym_file_open] = ACTIONS(1777),
    [sym_file_close] = ACTIONS(1777),
    [sym_file_read] = ACTIONS(1777),
    [sym_file_read_line] = ACTIONS(1777),
    [sym_file_write] = ACTIONS(1777),
    [sym_file_write_line] = ACTIONS(1777),
    [sym_file_copy] = ACTIONS(1777),
    [sym_file_delete] = ACTIONS(1777),
    [sym_file_rename] = ACTIONS(1777),
    [sym_file_dir] = ACTIONS(1777),
    [sym_file_getdir] = ACTIONS(1777),
    [sym_file_setdir] = ACTIONS(1777),
    [sym_file_create_dir] = ACTIONS(1777),
    [sym_file_remove_dir] = ACTIONS(1777),
    [sym_file_seek] = ACTIONS(1777),
    [sym_length_array] = ACTIONS(1777),
    [sym_max_length_array] = ACTIONS(1777),
    [sym_set_length_array] = ACTIONS(1777),
    [sym_ip_client_open] = ACTIONS(1777),
    [sym_ip_client_close] = ACTIONS(1777),
    [sym_ip_server_open] = ACTIONS(1777),
    [sym_ip_server_close] = ACTIONS(1777),
    [sym_ip_bound_client_open] = ACTIONS(1777),
    [sym_ip_mc_server_open] = ACTIONS(1777),
    [sym_ip_set_option] = ACTIONS(1777),
    [sym_ssh_client_open] = ACTIONS(1777),
    [sym_ssh_client_close] = ACTIONS(1777),
    [sym_tls_client_open] = ACTIONS(1777),
    [sym_tls_client_close] = ACTIONS(1777),
    [sym_do_push] = ACTIONS(1777),
    [sym_do_push_timed] = ACTIONS(1777),
    [sym_do_release] = ACTIONS(1777),
    [sym_device_id] = ACTIONS(1777),
    [sym_device_info] = ACTIONS(1777),
    [sym_device_id_string] = ACTIONS(1777),
    [sym_device_standby] = ACTIONS(1777),
    [sym_device_wake] = ACTIONS(1777),
    [sym_timeline_create] = ACTIONS(1777),
    [sym_timeline_kill] = ACTIONS(1777),
    [sym_timeline_reload] = ACTIONS(1777),
    [sym_timeline_pause] = ACTIONS(1777),
    [sym_timeline_restart] = ACTIONS(1777),
    [sym_timeline_set] = ACTIONS(1777),
    [sym_timeline_get] = ACTIONS(1777),
    [sym_timeline_active] = ACTIONS(1777),
    [sym_timeline_dynamic_id] = ACTIONS(1777),
    [sym_variable_to_string] = ACTIONS(1777),
    [sym_string_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_string] = ACTIONS(1777),
    [sym_type_cast] = ACTIONS(1777),
    [sym_variable_to_xml] = ACTIONS(1777),
    [sym_xml_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_xml] = ACTIONS(1777),
    [sym_redirect_string] = ACTIONS(1777),
    [sym_random_number] = ACTIONS(1777),
    [sym_get_unique_id] = ACTIONS(1777),
    [sym_get_system_number] = ACTIONS(1777),
    [sym_set_system_number] = ACTIONS(1777),
    [sym_get_serial_number] = ACTIONS(1777),
    [sym_get_master_build] = ACTIONS(1777),
    [sym_set_validation_code] = ACTIONS(1777),
    [sym_reboot] = ACTIONS(1777),
    [sym_load_duet_module] = ACTIONS(1777),
    [sym_unload_duet_module] = ACTIONS(1777),
    [sym_duet_mem_size_set] = ACTIONS(1777),
    [sym_duet_mem_size_get] = ACTIONS(1777),
    [sym_dynamic_application_device] = ACTIONS(1777),
    [sym_dynamic_polled_port] = ACTIONS(1777),
    [sym_static_port_binding] = ACTIONS(1777),
    [sym_static_ip_binding] = ACTIONS(1777),
    [sym_get_dns_list] = ACTIONS(1777),
    [sym_set_dns_list] = ACTIONS(1777),
    [sym_get_ip_address] = ACTIONS(1777),
    [sym_set_ip_address] = ACTIONS(1777),
    [sym_get_url_list] = ACTIONS(1777),
    [sym_add_url_entry] = ACTIONS(1777),
    [sym_delete_url_entry] = ACTIONS(1777),
    [sym_abs_value] = ACTIONS(1777),
    [sym_max_value] = ACTIONS(1777),
    [sym_min_value] = ACTIONS(1777),
    [sym_exp_value] = ACTIONS(1777),
    [sym_log_value] = ACTIONS(1777),
    [sym_log10_value] = ACTIONS(1777),
    [sym_power_value] = ACTIONS(1777),
    [sym_sqrt_value] = ACTIONS(1777),
    [sym_raw_be] = ACTIONS(1777),
    [sym_raw_le] = ACTIONS(1777),
    [sym_internal_queue_size_set] = ACTIONS(1777),
    [sym_internal_queue_size_get] = ACTIONS(1777),
    [sym_internal_threshold_set] = ACTIONS(1777),
    [sym_internal_threshold_get] = ACTIONS(1777),
    [sym_rebuilt_event] = ACTIONS(1777),
    [sym_do_custom_event] = ACTIONS(1777),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1777),
    [sym_clkmgr_set_clk_source] = ACTIONS(1777),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1777),
    [sym_clkmgr_get_timezone] = ACTIONS(1777),
    [sym_clkmgr_set_timezone] = ACTIONS(1777),
    [sym_clkmgr_get_resync_period] = ACTIONS(1777),
    [sym_clkmgr_set_resync_period] = ACTIONS(1777),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_timeservers] = ACTIONS(1777),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_set_log_level] = ACTIONS(1777),
    [sym_get_log_level] = ACTIONS(1777),
    [sym_amx_log] = ACTIONS(1777),
    [sym_smtp_server_config_set] = ACTIONS(1777),
    [sym_smtp_server_config_get] = ACTIONS(1777),
    [sym_smtp_send] = ACTIONS(1777),
    [sym_validate_netlinx_account] = ACTIONS(1777),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1777),
    [sym_authenticate_certificate] = ACTIONS(1777),
    [sym_audit_netlinx_session_event] = ACTIONS(1777),
    [sym_audit_netlinx_generic_event] = ACTIONS(1777),
    [sym_get_available_flash_disk_space] = ACTIONS(1777),
    [sym_get_max_flash_disk_space] = ACTIONS(1777),
    [sym_data_create_feed] = ACTIONS(1777),
    [sym_data_delete_feed] = ACTIONS(1777),
    [sym_data_publish_feed] = ACTIONS(1777),
    [sym_data_get_published_feed] = ACTIONS(1777),
    [sym_data_add_record] = ACTIONS(1777),
    [sym_data_get_event_record] = ACTIONS(1777),
    [sym__wc_data_create_feed] = ACTIONS(1777),
    [sym__wc_data_add_record] = ACTIONS(1777),
    [sym__wc_data_get_event_record] = ACTIONS(1777),
    [sym_format] = ACTIONS(1777),
    [sym_astro_clock] = ACTIONS(1777),
    [sym_set_outdoor_temperature] = ACTIONS(1777),
    [sym_set_virtual_level_count] = ACTIONS(1777),
    [sym_set_virtual_channel_count] = ACTIONS(1777),
    [sym_set_virtual_port_count] = ACTIONS(1777),
    [sym__wc_encode_int] = ACTIONS(1777),
    [sym__wc_decode_int] = ACTIONS(1777),
    [sym__wc_int] = ACTIONS(1777),
    [sym__wc_to_ch_int] = ACTIONS(1777),
    [sym__ch_to_wc_int] = ACTIONS(1777),
    [sym__wc_find_string_int] = ACTIONS(1777),
    [sym__wc_left_string_int] = ACTIONS(1777),
    [sym__wc_lower_string_int] = ACTIONS(1777),
    [sym__wc_mid_string_int] = ACTIONS(1777),
    [sym__wc_remove_string_int] = ACTIONS(1777),
    [sym__wc_right_string_int] = ACTIONS(1777),
    [sym__wc_upper_string_int] = ACTIONS(1777),
    [sym__wc_compare_string_int] = ACTIONS(1777),
    [sym__wc_get_buffer_char_int] = ACTIONS(1777),
    [sym__wc_get_buffer_string_int] = ACTIONS(1777),
    [sym__wc_concat_string_int] = ACTIONS(1777),
    [sym__wc_file_open_int] = ACTIONS(1777),
    [sym__wc_file_close_int] = ACTIONS(1777),
    [sym__wc_file_read_int] = ACTIONS(1777),
    [sym__wc_file_read_line_int] = ACTIONS(1777),
    [sym__wc_file_write_int] = ACTIONS(1777),
    [sym__wc_file_write_line_int] = ACTIONS(1777),
    [aux_sym_preproc_include_token1] = ACTIONS(1777),
    [aux_sym_preproc_define_token1] = ACTIONS(1779),
    [aux_sym_preproc_warn_token1] = ACTIONS(1779),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1779),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_else_token1] = ACTIONS(1779),
    [anon_sym_LBRACK] = ACTIONS(1779),
    [anon_sym_LBRACE] = ACTIONS(1779),
    [anon_sym_LPAREN] = ACTIONS(1777),
    [anon_sym_BANG] = ACTIONS(1779),
    [anon_sym_TILDE] = ACTIONS(1779),
    [anon_sym_DASH] = ACTIONS(1777),
    [anon_sym_PLUS] = ACTIONS(1777),
    [anon_sym_DASH_DASH] = ACTIONS(1779),
    [anon_sym_PLUS_PLUS] = ACTIONS(1779),
    [anon_sym_DQUOTE] = ACTIONS(1779),
    [anon_sym_SQUOTE] = ACTIONS(1779),
    [sym_number_literal] = ACTIONS(1779),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(232)] = {
    [sym_identifier] = ACTIONS(1777),
    [sym_program_name_keyword] = ACTIONS(1777),
    [sym_module_name_keyword] = ACTIONS(1777),
    [sym_define_device_keyword] = ACTIONS(1777),
    [sym_define_combine_keyword] = ACTIONS(1777),
    [sym_define_constant_keyword] = ACTIONS(1777),
    [sym_define_type_keyword] = ACTIONS(1777),
    [sym_define_variable_keyword] = ACTIONS(1777),
    [sym_define_system_variable_keyword] = ACTIONS(1777),
    [sym_define_connect_level_keyword] = ACTIONS(1777),
    [sym_define_latching_keyword] = ACTIONS(1777),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1777),
    [sym_define_toggling_keyword] = ACTIONS(1777),
    [sym_define_call_keyword] = ACTIONS(1777),
    [sym_define_function_keyword] = ACTIONS(1777),
    [sym_define_library_function_keyword] = ACTIONS(1777),
    [sym_define_start_keyword] = ACTIONS(1777),
    [sym_define_event_keyword] = ACTIONS(1777),
    [sym_define_program_keyword] = ACTIONS(1777),
    [sym_define_module_keyword] = ACTIONS(1777),
    [sym_char_keyword] = ACTIONS(1777),
    [sym_widechar_keyword] = ACTIONS(1777),
    [sym_integer_keyword] = ACTIONS(1777),
    [sym_sinteger_keyword] = ACTIONS(1777),
    [sym_long_keyword] = ACTIONS(1777),
    [sym_slong_keyword] = ACTIONS(1777),
    [sym_float_keyword] = ACTIONS(1777),
    [sym_double_keyword] = ACTIONS(1777),
    [sym_variant_keyword] = ACTIONS(1777),
    [sym_variantarray_keyword] = ACTIONS(1777),
    [sym_local_var_keyword] = ACTIONS(1777),
    [sym_stack_var_keyword] = ACTIONS(1777),
    [sym_constant_keyword] = ACTIONS(1777),
    [sym_volatile_keyword] = ACTIONS(1777),
    [sym_non_volatile_keyword] = ACTIONS(1777),
    [sym_persistent_keyword] = ACTIONS(1777),
    [sym_dev_keyword] = ACTIONS(1777),
    [sym_devlev_keyword] = ACTIONS(1777),
    [sym_devchan_keyword] = ACTIONS(1777),
    [sym_if_keyword] = ACTIONS(1777),
    [sym_else_keyword] = ACTIONS(1777),
    [sym_switch_keyword] = ACTIONS(1777),
    [sym_case_keyword] = ACTIONS(1777),
    [sym_default_keyword] = ACTIONS(1777),
    [sym_while_keyword] = ACTIONS(1777),
    [sym_for_keyword] = ACTIONS(1777),
    [sym_break_keyword] = ACTIONS(1777),
    [sym_continue_keyword] = ACTIONS(1777),
    [sym_return_keyword] = ACTIONS(1777),
    [sym_select_keyword] = ACTIONS(1777),
    [sym_active_keyword] = ACTIONS(1777),
    [sym_struct_keyword] = ACTIONS(1777),
    [sym_structure_keyword] = ACTIONS(1777),
    [sym_button_event_keyword] = ACTIONS(1777),
    [sym_channel_event_keyword] = ACTIONS(1777),
    [sym_level_event_keyword] = ACTIONS(1777),
    [sym_data_event_keyword] = ACTIONS(1777),
    [sym_timeline_event_keyword] = ACTIONS(1777),
    [sym_custom_event_keyword] = ACTIONS(1777),
    [sym_push_keyword] = ACTIONS(1777),
    [sym_release_keyword] = ACTIONS(1777),
    [aux_sym_on_keyword_token1] = ACTIONS(1777),
    [aux_sym_off_keyword_token1] = ACTIONS(1777),
    [sym_send_level_keyword] = ACTIONS(1777),
    [sym_send_string_keyword] = ACTIONS(1777),
    [sym_send_command_keyword] = ACTIONS(1777),
    [sym_clear_buffer_keyword] = ACTIONS(1777),
    [sym_create_buffer_keyword] = ACTIONS(1777),
    [sym_create_multi_buffer_keyword] = ACTIONS(1777),
    [sym_call_keyword] = ACTIONS(1777),
    [sym_system_call_keyword] = ACTIONS(1777),
    [sym_devchan_to_keyword] = ACTIONS(1777),
    [sym_devchan_min_to_keyword] = ACTIONS(1777),
    [sym_devchan_total_off_keyword] = ACTIONS(1777),
    [sym_devchan_pulse_keyword] = ACTIONS(1777),
    [sym_wait_keyword] = ACTIONS(1777),
    [sym_cancel_wait_keyword] = ACTIONS(1777),
    [sym_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1777),
    [sym_not] = ACTIONS(1777),
    [sym_bnot] = ACTIONS(1777),
    [sym___file__] = ACTIONS(1777),
    [sym___date__] = ACTIONS(1777),
    [sym___ldate__] = ACTIONS(1777),
    [sym___time__] = ACTIONS(1777),
    [sym___line__] = ACTIONS(1777),
    [sym___name__] = ACTIONS(1777),
    [sym___netlinx__] = ACTIONS(1777),
    [sym_netlinx_axi_version] = ACTIONS(1777),
    [sym_url_flg_tcp] = ACTIONS(1777),
    [sym_url_flg_accinfopresent] = ACTIONS(1777),
    [sym_url_flg_temp] = ACTIONS(1777),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1777),
    [sym_url_flg_stat_mask] = ACTIONS(1777),
    [sym_url_flg_stat_lookup] = ACTIONS(1777),
    [sym_url_flg_stat_connecting] = ACTIONS(1777),
    [sym_url_flg_stat_waiting] = ACTIONS(1777),
    [sym_url_flg_stat_connected] = ACTIONS(1777),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1777),
    [sym_first_virtual_device] = ACTIONS(1777),
    [sym_first_local_port] = ACTIONS(1777),
    [sym_dynamic_virtual_device] = ACTIONS(1777),
    [sym_timeline_once] = ACTIONS(1777),
    [sym_timeline_repeat] = ACTIONS(1777),
    [sym_timeline_absolute] = ACTIONS(1777),
    [sym_timeline_relative] = ACTIONS(1777),
    [sym_do_push_timed_infinite] = ACTIONS(1777),
    [sym_source_type_no_address] = ACTIONS(1777),
    [sym_source_type_neuron_id] = ACTIONS(1777),
    [sym_source_type_ip_address] = ACTIONS(1777),
    [sym_source_type_axlink] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1777),
    [sym_source_type_ip_socket_address] = ACTIONS(1777),
    [sym_source_type_rs232] = ACTIONS(1777),
    [sym_source_type_internal] = ACTIONS(1777),
    [sym_source_type_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1777),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1777),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1777),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1777),
    [sym_true] = ACTIONS(1777),
    [sym_false] = ACTIONS(1777),
    [sym_file_read_only] = ACTIONS(1777),
    [sym_file_rw_new] = ACTIONS(1777),
    [sym_file_rw_append] = ACTIONS(1777),
    [sym_ip_tcp] = ACTIONS(1777),
    [sym_ip_udp] = ACTIONS(1777),
    [sym_ip_udp_2way] = ACTIONS(1777),
    [sym_xml_encode_types] = ACTIONS(1777),
    [sym_xml_encode_char_as_list] = ACTIONS(1777),
    [sym_xml_encode_le] = ACTIONS(1777),
    [sym_xml_decode_types] = ACTIONS(1777),
    [sym_xml_decode_no_preserve] = ACTIONS(1777),
    [sym_duet_dev_type_display_device] = ACTIONS(1777),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_io_device] = ACTIONS(1777),
    [sym_duet_dev_type_relay_device] = ACTIONS(1777),
    [sym_duet_dev_type_ups] = ACTIONS(1777),
    [sym_duet_dev_type_amplifier] = ACTIONS(1777),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1777),
    [sym_duet_dev_type_camera] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1777),
    [sym_duet_dev_type_dss] = ACTIONS(1777),
    [sym_duet_dev_type_dvr] = ACTIONS(1777),
    [sym_duet_dev_type_disc_device] = ACTIONS(1777),
    [sym_duet_dev_type_document_camera] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1777),
    [sym_duet_dev_type_hvac] = ACTIONS(1777),
    [sym_duet_dev_type_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_light] = ACTIONS(1777),
    [sym_duet_dev_type_monitor] = ACTIONS(1777),
    [sym_duet_dev_type_motor] = ACTIONS(1777),
    [sym_duet_dev_type_multi_window] = ACTIONS(1777),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1777),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1777),
    [sym_duet_dev_type_receiver] = ACTIONS(1777),
    [sym_duet_dev_type_security_system] = ACTIONS(1777),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1777),
    [sym_duet_dev_type_settop_box] = ACTIONS(1777),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1777),
    [sym_duet_dev_type_switcher] = ACTIONS(1777),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_tv] = ACTIONS(1777),
    [sym_duet_dev_type_utility] = ACTIONS(1777),
    [sym_duet_dev_type_vcr] = ACTIONS(1777),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_video_processor] = ACTIONS(1777),
    [sym_duet_dev_type_video_projector] = ACTIONS(1777),
    [sym_duet_dev_type_video_wall] = ACTIONS(1777),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1777),
    [sym_duet_dev_type_weather] = ACTIONS(1777),
    [sym_duet_dev_not_polled] = ACTIONS(1777),
    [sym_duet_dev_polled] = ACTIONS(1777),
    [sym_clkmgr_mode_network] = ACTIONS(1777),
    [sym_clkmgr_mode_standalone] = ACTIONS(1777),
    [sym_normal_standby] = ACTIONS(1777),
    [sym_normal_wake] = ACTIONS(1777),
    [sym_ip_multicast_ttl_option] = ACTIONS(1777),
    [sym_ip_ttl_subnet] = ACTIONS(1777),
    [sym_ip_ttl_site] = ACTIONS(1777),
    [sym_ip_ttl_region] = ACTIONS(1777),
    [sym_ip_ttl_continent] = ACTIONS(1777),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1777),
    [sym_ip_nodelay_on] = ACTIONS(1777),
    [sym_ip_nodelay_off] = ACTIONS(1777),
    [sym_amx_error] = ACTIONS(1777),
    [sym_amx_warning] = ACTIONS(1777),
    [sym_amx_info] = ACTIONS(1777),
    [sym_amx_debug] = ACTIONS(1777),
    [sym_netlinx_login_success] = ACTIONS(1777),
    [sym_netlinx_login_fail] = ACTIONS(1777),
    [sym_netlinx_logout] = ACTIONS(1777),
    [sym_null_str] = ACTIONS(1777),
    [sym_smtp_address] = ACTIONS(1777),
    [sym_smtp_port_number] = ACTIONS(1777),
    [sym_smtp_username] = ACTIONS(1777),
    [sym_smtp_password] = ACTIONS(1777),
    [sym_smtp_require_tls] = ACTIONS(1777),
    [sym_smtp_from] = ACTIONS(1777),
    [sym_smtp_tls_true] = ACTIONS(1777),
    [sym_smtp_tls_false] = ACTIONS(1777),
    [sym_listview_on_row_select_event] = ACTIONS(1777),
    [sym_data_structure_unknown] = ACTIONS(1777),
    [sym_data_structure_datafeed] = ACTIONS(1777),
    [sym_data_structure_datarecordset] = ACTIONS(1777),
    [sym_data_structure_datarecord] = ACTIONS(1777),
    [sym_data_structure_datafield] = ACTIONS(1777),
    [sym_data_max_content_field_count] = ACTIONS(1777),
    [sym_data_max_metadata_field_count] = ACTIONS(1777),
    [sym_data_max_id_length] = ACTIONS(1777),
    [sym_data_max_name_length] = ACTIONS(1777),
    [sym_data_max_description_length] = ACTIONS(1777),
    [sym_data_max_type_length] = ACTIONS(1777),
    [sym_data_max_format_length] = ACTIONS(1777),
    [sym_data_max_source_length] = ACTIONS(1777),
    [sym_data_max_value_length] = ACTIONS(1777),
    [sym_data_max_label_length] = ACTIONS(1777),
    [sym_data_max_url_length] = ACTIONS(1777),
    [sym_max_last_login_info_length] = ACTIONS(1777),
    [sym_data_type_unknown] = ACTIONS(1777),
    [sym_data_type_string] = ACTIONS(1777),
    [sym_data_type_datetime] = ACTIONS(1777),
    [sym_data_type_date] = ACTIONS(1777),
    [sym_data_type_time] = ACTIONS(1777),
    [sym_data_type_image] = ACTIONS(1777),
    [sym_data_format_url] = ACTIONS(1777),
    [sym_data_format_phone] = ACTIONS(1777),
    [sym_data_format_email] = ACTIONS(1777),
    [sym_data_format_iso8601] = ACTIONS(1777),
    [sym_auditlog_priv] = ACTIONS(1777),
    [sym_configuration_priv] = ACTIONS(1777),
    [sym_deviceconfig_priv] = ACTIONS(1777),
    [sym_ftp_priv] = ACTIONS(1777),
    [sym_http_priv] = ACTIONS(1777),
    [sym_networkconfig_priv] = ACTIONS(1777),
    [sym_programport_priv] = ACTIONS(1777),
    [sym_remoteui_priv] = ACTIONS(1777),
    [sym_securitycontrol_priv] = ACTIONS(1777),
    [sym_softwaremanagement_priv] = ACTIONS(1777),
    [sym_terminal_priv] = ACTIONS(1777),
    [sym_tpadmin_priv] = ACTIONS(1777),
    [sym_user1_priv] = ACTIONS(1777),
    [sym_user2_priv] = ACTIONS(1777),
    [sym_user3_priv] = ACTIONS(1777),
    [sym_user4_priv] = ACTIONS(1777),
    [sym_usermanagement_priv] = ACTIONS(1777),
    [sym_valid_account] = ACTIONS(1777),
    [sym_err_name_invalid] = ACTIONS(1777),
    [sym_err_password_invalid] = ACTIONS(1777),
    [sym_err_invalid_account] = ACTIONS(1777),
    [sym_err_no_authorization] = ACTIONS(1777),
    [sym_err_invalid_parameter] = ACTIONS(1777),
    [sym_err_user_locked_out] = ACTIONS(1777),
    [sym_err_user_account_expired] = ACTIONS(1777),
    [sym_err_general_error] = ACTIONS(1777),
    [sym_err_authentication_error] = ACTIONS(1777),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1777),
    [sym_tls_validate_certificate] = ACTIONS(1777),
    [sym_tdata] = ACTIONS(1777),
    [sym_tchannel] = ACTIONS(1777),
    [sym_tlevel] = ACTIONS(1777),
    [sym_tbutton] = ACTIONS(1777),
    [sym_ttimeline] = ACTIONS(1777),
    [sym_tcustom] = ACTIONS(1777),
    [sym_url_struct] = ACTIONS(1777),
    [sym_dns_struct] = ACTIONS(1777),
    [sym_ip_address_struct] = ACTIONS(1777),
    [sym_dev_info_struct] = ACTIONS(1777),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1777),
    [sym_data_feed] = ACTIONS(1777),
    [sym_data_field] = ACTIONS(1777),
    [sym_data_record] = ACTIONS(1777),
    [sym_wc_data_feed] = ACTIONS(1777),
    [sym_wc_data_field] = ACTIONS(1777),
    [sym_wc_data_record] = ACTIONS(1777),
    [sym_last_login_info] = ACTIONS(1777),
    [sym____reserved___] = ACTIONS(1777),
    [sym_date] = ACTIONS(1777),
    [sym_day] = ACTIONS(1777),
    [sym_time] = ACTIONS(1777),
    [sym_ldate] = ACTIONS(1777),
    [sym_push_device] = ACTIONS(1777),
    [sym_push_channel] = ACTIONS(1777),
    [sym_push_devchan] = ACTIONS(1777),
    [sym_release_device] = ACTIONS(1777),
    [sym_release_channel] = ACTIONS(1777),
    [sym_release_devchan] = ACTIONS(1777),
    [sym_master_slot] = ACTIONS(1777),
    [sym_get_pulse_time] = ACTIONS(1777),
    [sym_get_timer] = ACTIONS(1777),
    [sym_master_sn] = ACTIONS(1777),
    [sym_system_number] = ACTIONS(1777),
    [sym_button] = ACTIONS(1777),
    [sym_dv_channel] = ACTIONS(1777),
    [sym_data] = ACTIONS(1777),
    [sym_level] = ACTIONS(1777),
    [sym_timeline] = ACTIONS(1777),
    [sym_channel] = ACTIONS(1777),
    [sym_custom] = ACTIONS(1777),
    [sym_atoi] = ACTIONS(1777),
    [sym_atol] = ACTIONS(1777),
    [sym_atof] = ACTIONS(1777),
    [sym_itoa] = ACTIONS(1777),
    [sym_itohex] = ACTIONS(1777),
    [sym_hextoi] = ACTIONS(1777),
    [sym_ftoa] = ACTIONS(1777),
    [sym_find_string] = ACTIONS(1777),
    [sym_left_string] = ACTIONS(1777),
    [sym_right_string] = ACTIONS(1777),
    [sym_mid_string] = ACTIONS(1777),
    [sym_remove_string] = ACTIONS(1777),
    [sym_get_buffer_string] = ACTIONS(1777),
    [sym_get_buffer_char] = ACTIONS(1777),
    [sym_get_multi_buffer_string] = ACTIONS(1777),
    [sym_length_string] = ACTIONS(1777),
    [sym_max_length_string] = ACTIONS(1777),
    [sym_set_length_string] = ACTIONS(1777),
    [sym_compare_string] = ACTIONS(1777),
    [sym_upper_string] = ACTIONS(1777),
    [sym_lower_string] = ACTIONS(1777),
    [sym_get_last] = ACTIONS(1777),
    [sym_date_to_day] = ACTIONS(1777),
    [sym_date_to_month] = ACTIONS(1777),
    [sym_date_to_year] = ACTIONS(1777),
    [sym_time_to_hour] = ACTIONS(1777),
    [sym_time_to_minute] = ACTIONS(1777),
    [sym_time_to_second] = ACTIONS(1777),
    [sym_day_of_week] = ACTIONS(1777),
    [sym_set_pulse_time] = ACTIONS(1777),
    [sym_set_timer] = ACTIONS(1777),
    [sym_file_open] = ACTIONS(1777),
    [sym_file_close] = ACTIONS(1777),
    [sym_file_read] = ACTIONS(1777),
    [sym_file_read_line] = ACTIONS(1777),
    [sym_file_write] = ACTIONS(1777),
    [sym_file_write_line] = ACTIONS(1777),
    [sym_file_copy] = ACTIONS(1777),
    [sym_file_delete] = ACTIONS(1777),
    [sym_file_rename] = ACTIONS(1777),
    [sym_file_dir] = ACTIONS(1777),
    [sym_file_getdir] = ACTIONS(1777),
    [sym_file_setdir] = ACTIONS(1777),
    [sym_file_create_dir] = ACTIONS(1777),
    [sym_file_remove_dir] = ACTIONS(1777),
    [sym_file_seek] = ACTIONS(1777),
    [sym_length_array] = ACTIONS(1777),
    [sym_max_length_array] = ACTIONS(1777),
    [sym_set_length_array] = ACTIONS(1777),
    [sym_ip_client_open] = ACTIONS(1777),
    [sym_ip_client_close] = ACTIONS(1777),
    [sym_ip_server_open] = ACTIONS(1777),
    [sym_ip_server_close] = ACTIONS(1777),
    [sym_ip_bound_client_open] = ACTIONS(1777),
    [sym_ip_mc_server_open] = ACTIONS(1777),
    [sym_ip_set_option] = ACTIONS(1777),
    [sym_ssh_client_open] = ACTIONS(1777),
    [sym_ssh_client_close] = ACTIONS(1777),
    [sym_tls_client_open] = ACTIONS(1777),
    [sym_tls_client_close] = ACTIONS(1777),
    [sym_do_push] = ACTIONS(1777),
    [sym_do_push_timed] = ACTIONS(1777),
    [sym_do_release] = ACTIONS(1777),
    [sym_device_id] = ACTIONS(1777),
    [sym_device_info] = ACTIONS(1777),
    [sym_device_id_string] = ACTIONS(1777),
    [sym_device_standby] = ACTIONS(1777),
    [sym_device_wake] = ACTIONS(1777),
    [sym_timeline_create] = ACTIONS(1777),
    [sym_timeline_kill] = ACTIONS(1777),
    [sym_timeline_reload] = ACTIONS(1777),
    [sym_timeline_pause] = ACTIONS(1777),
    [sym_timeline_restart] = ACTIONS(1777),
    [sym_timeline_set] = ACTIONS(1777),
    [sym_timeline_get] = ACTIONS(1777),
    [sym_timeline_active] = ACTIONS(1777),
    [sym_timeline_dynamic_id] = ACTIONS(1777),
    [sym_variable_to_string] = ACTIONS(1777),
    [sym_string_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_string] = ACTIONS(1777),
    [sym_type_cast] = ACTIONS(1777),
    [sym_variable_to_xml] = ACTIONS(1777),
    [sym_xml_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_xml] = ACTIONS(1777),
    [sym_redirect_string] = ACTIONS(1777),
    [sym_random_number] = ACTIONS(1777),
    [sym_get_unique_id] = ACTIONS(1777),
    [sym_get_system_number] = ACTIONS(1777),
    [sym_set_system_number] = ACTIONS(1777),
    [sym_get_serial_number] = ACTIONS(1777),
    [sym_get_master_build] = ACTIONS(1777),
    [sym_set_validation_code] = ACTIONS(1777),
    [sym_reboot] = ACTIONS(1777),
    [sym_load_duet_module] = ACTIONS(1777),
    [sym_unload_duet_module] = ACTIONS(1777),
    [sym_duet_mem_size_set] = ACTIONS(1777),
    [sym_duet_mem_size_get] = ACTIONS(1777),
    [sym_dynamic_application_device] = ACTIONS(1777),
    [sym_dynamic_polled_port] = ACTIONS(1777),
    [sym_static_port_binding] = ACTIONS(1777),
    [sym_static_ip_binding] = ACTIONS(1777),
    [sym_get_dns_list] = ACTIONS(1777),
    [sym_set_dns_list] = ACTIONS(1777),
    [sym_get_ip_address] = ACTIONS(1777),
    [sym_set_ip_address] = ACTIONS(1777),
    [sym_get_url_list] = ACTIONS(1777),
    [sym_add_url_entry] = ACTIONS(1777),
    [sym_delete_url_entry] = ACTIONS(1777),
    [sym_abs_value] = ACTIONS(1777),
    [sym_max_value] = ACTIONS(1777),
    [sym_min_value] = ACTIONS(1777),
    [sym_exp_value] = ACTIONS(1777),
    [sym_log_value] = ACTIONS(1777),
    [sym_log10_value] = ACTIONS(1777),
    [sym_power_value] = ACTIONS(1777),
    [sym_sqrt_value] = ACTIONS(1777),
    [sym_raw_be] = ACTIONS(1777),
    [sym_raw_le] = ACTIONS(1777),
    [sym_internal_queue_size_set] = ACTIONS(1777),
    [sym_internal_queue_size_get] = ACTIONS(1777),
    [sym_internal_threshold_set] = ACTIONS(1777),
    [sym_internal_threshold_get] = ACTIONS(1777),
    [sym_rebuilt_event] = ACTIONS(1777),
    [sym_do_custom_event] = ACTIONS(1777),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1777),
    [sym_clkmgr_set_clk_source] = ACTIONS(1777),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1777),
    [sym_clkmgr_get_timezone] = ACTIONS(1777),
    [sym_clkmgr_set_timezone] = ACTIONS(1777),
    [sym_clkmgr_get_resync_period] = ACTIONS(1777),
    [sym_clkmgr_set_resync_period] = ACTIONS(1777),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_timeservers] = ACTIONS(1777),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_set_log_level] = ACTIONS(1777),
    [sym_get_log_level] = ACTIONS(1777),
    [sym_amx_log] = ACTIONS(1777),
    [sym_smtp_server_config_set] = ACTIONS(1777),
    [sym_smtp_server_config_get] = ACTIONS(1777),
    [sym_smtp_send] = ACTIONS(1777),
    [sym_validate_netlinx_account] = ACTIONS(1777),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1777),
    [sym_authenticate_certificate] = ACTIONS(1777),
    [sym_audit_netlinx_session_event] = ACTIONS(1777),
    [sym_audit_netlinx_generic_event] = ACTIONS(1777),
    [sym_get_available_flash_disk_space] = ACTIONS(1777),
    [sym_get_max_flash_disk_space] = ACTIONS(1777),
    [sym_data_create_feed] = ACTIONS(1777),
    [sym_data_delete_feed] = ACTIONS(1777),
    [sym_data_publish_feed] = ACTIONS(1777),
    [sym_data_get_published_feed] = ACTIONS(1777),
    [sym_data_add_record] = ACTIONS(1777),
    [sym_data_get_event_record] = ACTIONS(1777),
    [sym__wc_data_create_feed] = ACTIONS(1777),
    [sym__wc_data_add_record] = ACTIONS(1777),
    [sym__wc_data_get_event_record] = ACTIONS(1777),
    [sym_format] = ACTIONS(1777),
    [sym_astro_clock] = ACTIONS(1777),
    [sym_set_outdoor_temperature] = ACTIONS(1777),
    [sym_set_virtual_level_count] = ACTIONS(1777),
    [sym_set_virtual_channel_count] = ACTIONS(1777),
    [sym_set_virtual_port_count] = ACTIONS(1777),
    [sym__wc_encode_int] = ACTIONS(1777),
    [sym__wc_decode_int] = ACTIONS(1777),
    [sym__wc_int] = ACTIONS(1777),
    [sym__wc_to_ch_int] = ACTIONS(1777),
    [sym__ch_to_wc_int] = ACTIONS(1777),
    [sym__wc_find_string_int] = ACTIONS(1777),
    [sym__wc_left_string_int] = ACTIONS(1777),
    [sym__wc_lower_string_int] = ACTIONS(1777),
    [sym__wc_mid_string_int] = ACTIONS(1777),
    [sym__wc_remove_string_int] = ACTIONS(1777),
    [sym__wc_right_string_int] = ACTIONS(1777),
    [sym__wc_upper_string_int] = ACTIONS(1777),
    [sym__wc_compare_string_int] = ACTIONS(1777),
    [sym__wc_get_buffer_char_int] = ACTIONS(1777),
    [sym__wc_get_buffer_string_int] = ACTIONS(1777),
    [sym__wc_concat_string_int] = ACTIONS(1777),
    [sym__wc_file_open_int] = ACTIONS(1777),
    [sym__wc_file_close_int] = ACTIONS(1777),
    [sym__wc_file_read_int] = ACTIONS(1777),
    [sym__wc_file_read_line_int] = ACTIONS(1777),
    [sym__wc_file_write_int] = ACTIONS(1777),
    [sym__wc_file_write_line_int] = ACTIONS(1777),
    [aux_sym_preproc_include_token1] = ACTIONS(1777),
    [aux_sym_preproc_define_token1] = ACTIONS(1779),
    [aux_sym_preproc_warn_token1] = ACTIONS(1779),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1779),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_else_token1] = ACTIONS(1779),
    [anon_sym_LBRACK] = ACTIONS(1779),
    [anon_sym_LBRACE] = ACTIONS(1779),
    [anon_sym_LPAREN] = ACTIONS(1777),
    [anon_sym_BANG] = ACTIONS(1779),
    [anon_sym_TILDE] = ACTIONS(1779),
    [anon_sym_DASH] = ACTIONS(1777),
    [anon_sym_PLUS] = ACTIONS(1777),
    [anon_sym_DASH_DASH] = ACTIONS(1779),
    [anon_sym_PLUS_PLUS] = ACTIONS(1779),
    [anon_sym_DQUOTE] = ACTIONS(1779),
    [anon_sym_SQUOTE] = ACTIONS(1779),
    [sym_number_literal] = ACTIONS(1779),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(233)] = {
    [sym_identifier] = ACTIONS(1777),
    [sym_program_name_keyword] = ACTIONS(1777),
    [sym_module_name_keyword] = ACTIONS(1777),
    [sym_define_device_keyword] = ACTIONS(1777),
    [sym_define_combine_keyword] = ACTIONS(1777),
    [sym_define_constant_keyword] = ACTIONS(1777),
    [sym_define_type_keyword] = ACTIONS(1777),
    [sym_define_variable_keyword] = ACTIONS(1777),
    [sym_define_system_variable_keyword] = ACTIONS(1777),
    [sym_define_connect_level_keyword] = ACTIONS(1777),
    [sym_define_latching_keyword] = ACTIONS(1777),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1777),
    [sym_define_toggling_keyword] = ACTIONS(1777),
    [sym_define_call_keyword] = ACTIONS(1777),
    [sym_define_function_keyword] = ACTIONS(1777),
    [sym_define_library_function_keyword] = ACTIONS(1777),
    [sym_define_start_keyword] = ACTIONS(1777),
    [sym_define_event_keyword] = ACTIONS(1777),
    [sym_define_program_keyword] = ACTIONS(1777),
    [sym_define_module_keyword] = ACTIONS(1777),
    [sym_char_keyword] = ACTIONS(1777),
    [sym_widechar_keyword] = ACTIONS(1777),
    [sym_integer_keyword] = ACTIONS(1777),
    [sym_sinteger_keyword] = ACTIONS(1777),
    [sym_long_keyword] = ACTIONS(1777),
    [sym_slong_keyword] = ACTIONS(1777),
    [sym_float_keyword] = ACTIONS(1777),
    [sym_double_keyword] = ACTIONS(1777),
    [sym_variant_keyword] = ACTIONS(1777),
    [sym_variantarray_keyword] = ACTIONS(1777),
    [sym_local_var_keyword] = ACTIONS(1777),
    [sym_stack_var_keyword] = ACTIONS(1777),
    [sym_constant_keyword] = ACTIONS(1777),
    [sym_volatile_keyword] = ACTIONS(1777),
    [sym_non_volatile_keyword] = ACTIONS(1777),
    [sym_persistent_keyword] = ACTIONS(1777),
    [sym_dev_keyword] = ACTIONS(1777),
    [sym_devlev_keyword] = ACTIONS(1777),
    [sym_devchan_keyword] = ACTIONS(1777),
    [sym_if_keyword] = ACTIONS(1777),
    [sym_else_keyword] = ACTIONS(1777),
    [sym_switch_keyword] = ACTIONS(1777),
    [sym_case_keyword] = ACTIONS(1777),
    [sym_default_keyword] = ACTIONS(1777),
    [sym_while_keyword] = ACTIONS(1777),
    [sym_for_keyword] = ACTIONS(1777),
    [sym_break_keyword] = ACTIONS(1777),
    [sym_continue_keyword] = ACTIONS(1777),
    [sym_return_keyword] = ACTIONS(1777),
    [sym_select_keyword] = ACTIONS(1777),
    [sym_active_keyword] = ACTIONS(1777),
    [sym_struct_keyword] = ACTIONS(1777),
    [sym_structure_keyword] = ACTIONS(1777),
    [sym_button_event_keyword] = ACTIONS(1777),
    [sym_channel_event_keyword] = ACTIONS(1777),
    [sym_level_event_keyword] = ACTIONS(1777),
    [sym_data_event_keyword] = ACTIONS(1777),
    [sym_timeline_event_keyword] = ACTIONS(1777),
    [sym_custom_event_keyword] = ACTIONS(1777),
    [sym_push_keyword] = ACTIONS(1777),
    [sym_release_keyword] = ACTIONS(1777),
    [aux_sym_on_keyword_token1] = ACTIONS(1777),
    [aux_sym_off_keyword_token1] = ACTIONS(1777),
    [sym_send_level_keyword] = ACTIONS(1777),
    [sym_send_string_keyword] = ACTIONS(1777),
    [sym_send_command_keyword] = ACTIONS(1777),
    [sym_clear_buffer_keyword] = ACTIONS(1777),
    [sym_create_buffer_keyword] = ACTIONS(1777),
    [sym_create_multi_buffer_keyword] = ACTIONS(1777),
    [sym_call_keyword] = ACTIONS(1777),
    [sym_system_call_keyword] = ACTIONS(1777),
    [sym_devchan_to_keyword] = ACTIONS(1777),
    [sym_devchan_min_to_keyword] = ACTIONS(1777),
    [sym_devchan_total_off_keyword] = ACTIONS(1777),
    [sym_devchan_pulse_keyword] = ACTIONS(1777),
    [sym_wait_keyword] = ACTIONS(1777),
    [sym_cancel_wait_keyword] = ACTIONS(1777),
    [sym_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1777),
    [sym_not] = ACTIONS(1777),
    [sym_bnot] = ACTIONS(1777),
    [sym___file__] = ACTIONS(1777),
    [sym___date__] = ACTIONS(1777),
    [sym___ldate__] = ACTIONS(1777),
    [sym___time__] = ACTIONS(1777),
    [sym___line__] = ACTIONS(1777),
    [sym___name__] = ACTIONS(1777),
    [sym___netlinx__] = ACTIONS(1777),
    [sym_netlinx_axi_version] = ACTIONS(1777),
    [sym_url_flg_tcp] = ACTIONS(1777),
    [sym_url_flg_accinfopresent] = ACTIONS(1777),
    [sym_url_flg_temp] = ACTIONS(1777),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1777),
    [sym_url_flg_stat_mask] = ACTIONS(1777),
    [sym_url_flg_stat_lookup] = ACTIONS(1777),
    [sym_url_flg_stat_connecting] = ACTIONS(1777),
    [sym_url_flg_stat_waiting] = ACTIONS(1777),
    [sym_url_flg_stat_connected] = ACTIONS(1777),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1777),
    [sym_first_virtual_device] = ACTIONS(1777),
    [sym_first_local_port] = ACTIONS(1777),
    [sym_dynamic_virtual_device] = ACTIONS(1777),
    [sym_timeline_once] = ACTIONS(1777),
    [sym_timeline_repeat] = ACTIONS(1777),
    [sym_timeline_absolute] = ACTIONS(1777),
    [sym_timeline_relative] = ACTIONS(1777),
    [sym_do_push_timed_infinite] = ACTIONS(1777),
    [sym_source_type_no_address] = ACTIONS(1777),
    [sym_source_type_neuron_id] = ACTIONS(1777),
    [sym_source_type_ip_address] = ACTIONS(1777),
    [sym_source_type_axlink] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1777),
    [sym_source_type_ip_socket_address] = ACTIONS(1777),
    [sym_source_type_rs232] = ACTIONS(1777),
    [sym_source_type_internal] = ACTIONS(1777),
    [sym_source_type_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1777),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1777),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1777),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1777),
    [sym_true] = ACTIONS(1777),
    [sym_false] = ACTIONS(1777),
    [sym_file_read_only] = ACTIONS(1777),
    [sym_file_rw_new] = ACTIONS(1777),
    [sym_file_rw_append] = ACTIONS(1777),
    [sym_ip_tcp] = ACTIONS(1777),
    [sym_ip_udp] = ACTIONS(1777),
    [sym_ip_udp_2way] = ACTIONS(1777),
    [sym_xml_encode_types] = ACTIONS(1777),
    [sym_xml_encode_char_as_list] = ACTIONS(1777),
    [sym_xml_encode_le] = ACTIONS(1777),
    [sym_xml_decode_types] = ACTIONS(1777),
    [sym_xml_decode_no_preserve] = ACTIONS(1777),
    [sym_duet_dev_type_display_device] = ACTIONS(1777),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_io_device] = ACTIONS(1777),
    [sym_duet_dev_type_relay_device] = ACTIONS(1777),
    [sym_duet_dev_type_ups] = ACTIONS(1777),
    [sym_duet_dev_type_amplifier] = ACTIONS(1777),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1777),
    [sym_duet_dev_type_camera] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1777),
    [sym_duet_dev_type_dss] = ACTIONS(1777),
    [sym_duet_dev_type_dvr] = ACTIONS(1777),
    [sym_duet_dev_type_disc_device] = ACTIONS(1777),
    [sym_duet_dev_type_document_camera] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1777),
    [sym_duet_dev_type_hvac] = ACTIONS(1777),
    [sym_duet_dev_type_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_light] = ACTIONS(1777),
    [sym_duet_dev_type_monitor] = ACTIONS(1777),
    [sym_duet_dev_type_motor] = ACTIONS(1777),
    [sym_duet_dev_type_multi_window] = ACTIONS(1777),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1777),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1777),
    [sym_duet_dev_type_receiver] = ACTIONS(1777),
    [sym_duet_dev_type_security_system] = ACTIONS(1777),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1777),
    [sym_duet_dev_type_settop_box] = ACTIONS(1777),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1777),
    [sym_duet_dev_type_switcher] = ACTIONS(1777),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_tv] = ACTIONS(1777),
    [sym_duet_dev_type_utility] = ACTIONS(1777),
    [sym_duet_dev_type_vcr] = ACTIONS(1777),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_video_processor] = ACTIONS(1777),
    [sym_duet_dev_type_video_projector] = ACTIONS(1777),
    [sym_duet_dev_type_video_wall] = ACTIONS(1777),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1777),
    [sym_duet_dev_type_weather] = ACTIONS(1777),
    [sym_duet_dev_not_polled] = ACTIONS(1777),
    [sym_duet_dev_polled] = ACTIONS(1777),
    [sym_clkmgr_mode_network] = ACTIONS(1777),
    [sym_clkmgr_mode_standalone] = ACTIONS(1777),
    [sym_normal_standby] = ACTIONS(1777),
    [sym_normal_wake] = ACTIONS(1777),
    [sym_ip_multicast_ttl_option] = ACTIONS(1777),
    [sym_ip_ttl_subnet] = ACTIONS(1777),
    [sym_ip_ttl_site] = ACTIONS(1777),
    [sym_ip_ttl_region] = ACTIONS(1777),
    [sym_ip_ttl_continent] = ACTIONS(1777),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1777),
    [sym_ip_nodelay_on] = ACTIONS(1777),
    [sym_ip_nodelay_off] = ACTIONS(1777),
    [sym_amx_error] = ACTIONS(1777),
    [sym_amx_warning] = ACTIONS(1777),
    [sym_amx_info] = ACTIONS(1777),
    [sym_amx_debug] = ACTIONS(1777),
    [sym_netlinx_login_success] = ACTIONS(1777),
    [sym_netlinx_login_fail] = ACTIONS(1777),
    [sym_netlinx_logout] = ACTIONS(1777),
    [sym_null_str] = ACTIONS(1777),
    [sym_smtp_address] = ACTIONS(1777),
    [sym_smtp_port_number] = ACTIONS(1777),
    [sym_smtp_username] = ACTIONS(1777),
    [sym_smtp_password] = ACTIONS(1777),
    [sym_smtp_require_tls] = ACTIONS(1777),
    [sym_smtp_from] = ACTIONS(1777),
    [sym_smtp_tls_true] = ACTIONS(1777),
    [sym_smtp_tls_false] = ACTIONS(1777),
    [sym_listview_on_row_select_event] = ACTIONS(1777),
    [sym_data_structure_unknown] = ACTIONS(1777),
    [sym_data_structure_datafeed] = ACTIONS(1777),
    [sym_data_structure_datarecordset] = ACTIONS(1777),
    [sym_data_structure_datarecord] = ACTIONS(1777),
    [sym_data_structure_datafield] = ACTIONS(1777),
    [sym_data_max_content_field_count] = ACTIONS(1777),
    [sym_data_max_metadata_field_count] = ACTIONS(1777),
    [sym_data_max_id_length] = ACTIONS(1777),
    [sym_data_max_name_length] = ACTIONS(1777),
    [sym_data_max_description_length] = ACTIONS(1777),
    [sym_data_max_type_length] = ACTIONS(1777),
    [sym_data_max_format_length] = ACTIONS(1777),
    [sym_data_max_source_length] = ACTIONS(1777),
    [sym_data_max_value_length] = ACTIONS(1777),
    [sym_data_max_label_length] = ACTIONS(1777),
    [sym_data_max_url_length] = ACTIONS(1777),
    [sym_max_last_login_info_length] = ACTIONS(1777),
    [sym_data_type_unknown] = ACTIONS(1777),
    [sym_data_type_string] = ACTIONS(1777),
    [sym_data_type_datetime] = ACTIONS(1777),
    [sym_data_type_date] = ACTIONS(1777),
    [sym_data_type_time] = ACTIONS(1777),
    [sym_data_type_image] = ACTIONS(1777),
    [sym_data_format_url] = ACTIONS(1777),
    [sym_data_format_phone] = ACTIONS(1777),
    [sym_data_format_email] = ACTIONS(1777),
    [sym_data_format_iso8601] = ACTIONS(1777),
    [sym_auditlog_priv] = ACTIONS(1777),
    [sym_configuration_priv] = ACTIONS(1777),
    [sym_deviceconfig_priv] = ACTIONS(1777),
    [sym_ftp_priv] = ACTIONS(1777),
    [sym_http_priv] = ACTIONS(1777),
    [sym_networkconfig_priv] = ACTIONS(1777),
    [sym_programport_priv] = ACTIONS(1777),
    [sym_remoteui_priv] = ACTIONS(1777),
    [sym_securitycontrol_priv] = ACTIONS(1777),
    [sym_softwaremanagement_priv] = ACTIONS(1777),
    [sym_terminal_priv] = ACTIONS(1777),
    [sym_tpadmin_priv] = ACTIONS(1777),
    [sym_user1_priv] = ACTIONS(1777),
    [sym_user2_priv] = ACTIONS(1777),
    [sym_user3_priv] = ACTIONS(1777),
    [sym_user4_priv] = ACTIONS(1777),
    [sym_usermanagement_priv] = ACTIONS(1777),
    [sym_valid_account] = ACTIONS(1777),
    [sym_err_name_invalid] = ACTIONS(1777),
    [sym_err_password_invalid] = ACTIONS(1777),
    [sym_err_invalid_account] = ACTIONS(1777),
    [sym_err_no_authorization] = ACTIONS(1777),
    [sym_err_invalid_parameter] = ACTIONS(1777),
    [sym_err_user_locked_out] = ACTIONS(1777),
    [sym_err_user_account_expired] = ACTIONS(1777),
    [sym_err_general_error] = ACTIONS(1777),
    [sym_err_authentication_error] = ACTIONS(1777),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1777),
    [sym_tls_validate_certificate] = ACTIONS(1777),
    [sym_tdata] = ACTIONS(1777),
    [sym_tchannel] = ACTIONS(1777),
    [sym_tlevel] = ACTIONS(1777),
    [sym_tbutton] = ACTIONS(1777),
    [sym_ttimeline] = ACTIONS(1777),
    [sym_tcustom] = ACTIONS(1777),
    [sym_url_struct] = ACTIONS(1777),
    [sym_dns_struct] = ACTIONS(1777),
    [sym_ip_address_struct] = ACTIONS(1777),
    [sym_dev_info_struct] = ACTIONS(1777),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1777),
    [sym_data_feed] = ACTIONS(1777),
    [sym_data_field] = ACTIONS(1777),
    [sym_data_record] = ACTIONS(1777),
    [sym_wc_data_feed] = ACTIONS(1777),
    [sym_wc_data_field] = ACTIONS(1777),
    [sym_wc_data_record] = ACTIONS(1777),
    [sym_last_login_info] = ACTIONS(1777),
    [sym____reserved___] = ACTIONS(1777),
    [sym_date] = ACTIONS(1777),
    [sym_day] = ACTIONS(1777),
    [sym_time] = ACTIONS(1777),
    [sym_ldate] = ACTIONS(1777),
    [sym_push_device] = ACTIONS(1777),
    [sym_push_channel] = ACTIONS(1777),
    [sym_push_devchan] = ACTIONS(1777),
    [sym_release_device] = ACTIONS(1777),
    [sym_release_channel] = ACTIONS(1777),
    [sym_release_devchan] = ACTIONS(1777),
    [sym_master_slot] = ACTIONS(1777),
    [sym_get_pulse_time] = ACTIONS(1777),
    [sym_get_timer] = ACTIONS(1777),
    [sym_master_sn] = ACTIONS(1777),
    [sym_system_number] = ACTIONS(1777),
    [sym_button] = ACTIONS(1777),
    [sym_dv_channel] = ACTIONS(1777),
    [sym_data] = ACTIONS(1777),
    [sym_level] = ACTIONS(1777),
    [sym_timeline] = ACTIONS(1777),
    [sym_channel] = ACTIONS(1777),
    [sym_custom] = ACTIONS(1777),
    [sym_atoi] = ACTIONS(1777),
    [sym_atol] = ACTIONS(1777),
    [sym_atof] = ACTIONS(1777),
    [sym_itoa] = ACTIONS(1777),
    [sym_itohex] = ACTIONS(1777),
    [sym_hextoi] = ACTIONS(1777),
    [sym_ftoa] = ACTIONS(1777),
    [sym_find_string] = ACTIONS(1777),
    [sym_left_string] = ACTIONS(1777),
    [sym_right_string] = ACTIONS(1777),
    [sym_mid_string] = ACTIONS(1777),
    [sym_remove_string] = ACTIONS(1777),
    [sym_get_buffer_string] = ACTIONS(1777),
    [sym_get_buffer_char] = ACTIONS(1777),
    [sym_get_multi_buffer_string] = ACTIONS(1777),
    [sym_length_string] = ACTIONS(1777),
    [sym_max_length_string] = ACTIONS(1777),
    [sym_set_length_string] = ACTIONS(1777),
    [sym_compare_string] = ACTIONS(1777),
    [sym_upper_string] = ACTIONS(1777),
    [sym_lower_string] = ACTIONS(1777),
    [sym_get_last] = ACTIONS(1777),
    [sym_date_to_day] = ACTIONS(1777),
    [sym_date_to_month] = ACTIONS(1777),
    [sym_date_to_year] = ACTIONS(1777),
    [sym_time_to_hour] = ACTIONS(1777),
    [sym_time_to_minute] = ACTIONS(1777),
    [sym_time_to_second] = ACTIONS(1777),
    [sym_day_of_week] = ACTIONS(1777),
    [sym_set_pulse_time] = ACTIONS(1777),
    [sym_set_timer] = ACTIONS(1777),
    [sym_file_open] = ACTIONS(1777),
    [sym_file_close] = ACTIONS(1777),
    [sym_file_read] = ACTIONS(1777),
    [sym_file_read_line] = ACTIONS(1777),
    [sym_file_write] = ACTIONS(1777),
    [sym_file_write_line] = ACTIONS(1777),
    [sym_file_copy] = ACTIONS(1777),
    [sym_file_delete] = ACTIONS(1777),
    [sym_file_rename] = ACTIONS(1777),
    [sym_file_dir] = ACTIONS(1777),
    [sym_file_getdir] = ACTIONS(1777),
    [sym_file_setdir] = ACTIONS(1777),
    [sym_file_create_dir] = ACTIONS(1777),
    [sym_file_remove_dir] = ACTIONS(1777),
    [sym_file_seek] = ACTIONS(1777),
    [sym_length_array] = ACTIONS(1777),
    [sym_max_length_array] = ACTIONS(1777),
    [sym_set_length_array] = ACTIONS(1777),
    [sym_ip_client_open] = ACTIONS(1777),
    [sym_ip_client_close] = ACTIONS(1777),
    [sym_ip_server_open] = ACTIONS(1777),
    [sym_ip_server_close] = ACTIONS(1777),
    [sym_ip_bound_client_open] = ACTIONS(1777),
    [sym_ip_mc_server_open] = ACTIONS(1777),
    [sym_ip_set_option] = ACTIONS(1777),
    [sym_ssh_client_open] = ACTIONS(1777),
    [sym_ssh_client_close] = ACTIONS(1777),
    [sym_tls_client_open] = ACTIONS(1777),
    [sym_tls_client_close] = ACTIONS(1777),
    [sym_do_push] = ACTIONS(1777),
    [sym_do_push_timed] = ACTIONS(1777),
    [sym_do_release] = ACTIONS(1777),
    [sym_device_id] = ACTIONS(1777),
    [sym_device_info] = ACTIONS(1777),
    [sym_device_id_string] = ACTIONS(1777),
    [sym_device_standby] = ACTIONS(1777),
    [sym_device_wake] = ACTIONS(1777),
    [sym_timeline_create] = ACTIONS(1777),
    [sym_timeline_kill] = ACTIONS(1777),
    [sym_timeline_reload] = ACTIONS(1777),
    [sym_timeline_pause] = ACTIONS(1777),
    [sym_timeline_restart] = ACTIONS(1777),
    [sym_timeline_set] = ACTIONS(1777),
    [sym_timeline_get] = ACTIONS(1777),
    [sym_timeline_active] = ACTIONS(1777),
    [sym_timeline_dynamic_id] = ACTIONS(1777),
    [sym_variable_to_string] = ACTIONS(1777),
    [sym_string_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_string] = ACTIONS(1777),
    [sym_type_cast] = ACTIONS(1777),
    [sym_variable_to_xml] = ACTIONS(1777),
    [sym_xml_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_xml] = ACTIONS(1777),
    [sym_redirect_string] = ACTIONS(1777),
    [sym_random_number] = ACTIONS(1777),
    [sym_get_unique_id] = ACTIONS(1777),
    [sym_get_system_number] = ACTIONS(1777),
    [sym_set_system_number] = ACTIONS(1777),
    [sym_get_serial_number] = ACTIONS(1777),
    [sym_get_master_build] = ACTIONS(1777),
    [sym_set_validation_code] = ACTIONS(1777),
    [sym_reboot] = ACTIONS(1777),
    [sym_load_duet_module] = ACTIONS(1777),
    [sym_unload_duet_module] = ACTIONS(1777),
    [sym_duet_mem_size_set] = ACTIONS(1777),
    [sym_duet_mem_size_get] = ACTIONS(1777),
    [sym_dynamic_application_device] = ACTIONS(1777),
    [sym_dynamic_polled_port] = ACTIONS(1777),
    [sym_static_port_binding] = ACTIONS(1777),
    [sym_static_ip_binding] = ACTIONS(1777),
    [sym_get_dns_list] = ACTIONS(1777),
    [sym_set_dns_list] = ACTIONS(1777),
    [sym_get_ip_address] = ACTIONS(1777),
    [sym_set_ip_address] = ACTIONS(1777),
    [sym_get_url_list] = ACTIONS(1777),
    [sym_add_url_entry] = ACTIONS(1777),
    [sym_delete_url_entry] = ACTIONS(1777),
    [sym_abs_value] = ACTIONS(1777),
    [sym_max_value] = ACTIONS(1777),
    [sym_min_value] = ACTIONS(1777),
    [sym_exp_value] = ACTIONS(1777),
    [sym_log_value] = ACTIONS(1777),
    [sym_log10_value] = ACTIONS(1777),
    [sym_power_value] = ACTIONS(1777),
    [sym_sqrt_value] = ACTIONS(1777),
    [sym_raw_be] = ACTIONS(1777),
    [sym_raw_le] = ACTIONS(1777),
    [sym_internal_queue_size_set] = ACTIONS(1777),
    [sym_internal_queue_size_get] = ACTIONS(1777),
    [sym_internal_threshold_set] = ACTIONS(1777),
    [sym_internal_threshold_get] = ACTIONS(1777),
    [sym_rebuilt_event] = ACTIONS(1777),
    [sym_do_custom_event] = ACTIONS(1777),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1777),
    [sym_clkmgr_set_clk_source] = ACTIONS(1777),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1777),
    [sym_clkmgr_get_timezone] = ACTIONS(1777),
    [sym_clkmgr_set_timezone] = ACTIONS(1777),
    [sym_clkmgr_get_resync_period] = ACTIONS(1777),
    [sym_clkmgr_set_resync_period] = ACTIONS(1777),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_timeservers] = ACTIONS(1777),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_set_log_level] = ACTIONS(1777),
    [sym_get_log_level] = ACTIONS(1777),
    [sym_amx_log] = ACTIONS(1777),
    [sym_smtp_server_config_set] = ACTIONS(1777),
    [sym_smtp_server_config_get] = ACTIONS(1777),
    [sym_smtp_send] = ACTIONS(1777),
    [sym_validate_netlinx_account] = ACTIONS(1777),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1777),
    [sym_authenticate_certificate] = ACTIONS(1777),
    [sym_audit_netlinx_session_event] = ACTIONS(1777),
    [sym_audit_netlinx_generic_event] = ACTIONS(1777),
    [sym_get_available_flash_disk_space] = ACTIONS(1777),
    [sym_get_max_flash_disk_space] = ACTIONS(1777),
    [sym_data_create_feed] = ACTIONS(1777),
    [sym_data_delete_feed] = ACTIONS(1777),
    [sym_data_publish_feed] = ACTIONS(1777),
    [sym_data_get_published_feed] = ACTIONS(1777),
    [sym_data_add_record] = ACTIONS(1777),
    [sym_data_get_event_record] = ACTIONS(1777),
    [sym__wc_data_create_feed] = ACTIONS(1777),
    [sym__wc_data_add_record] = ACTIONS(1777),
    [sym__wc_data_get_event_record] = ACTIONS(1777),
    [sym_format] = ACTIONS(1777),
    [sym_astro_clock] = ACTIONS(1777),
    [sym_set_outdoor_temperature] = ACTIONS(1777),
    [sym_set_virtual_level_count] = ACTIONS(1777),
    [sym_set_virtual_channel_count] = ACTIONS(1777),
    [sym_set_virtual_port_count] = ACTIONS(1777),
    [sym__wc_encode_int] = ACTIONS(1777),
    [sym__wc_decode_int] = ACTIONS(1777),
    [sym__wc_int] = ACTIONS(1777),
    [sym__wc_to_ch_int] = ACTIONS(1777),
    [sym__ch_to_wc_int] = ACTIONS(1777),
    [sym__wc_find_string_int] = ACTIONS(1777),
    [sym__wc_left_string_int] = ACTIONS(1777),
    [sym__wc_lower_string_int] = ACTIONS(1777),
    [sym__wc_mid_string_int] = ACTIONS(1777),
    [sym__wc_remove_string_int] = ACTIONS(1777),
    [sym__wc_right_string_int] = ACTIONS(1777),
    [sym__wc_upper_string_int] = ACTIONS(1777),
    [sym__wc_compare_string_int] = ACTIONS(1777),
    [sym__wc_get_buffer_char_int] = ACTIONS(1777),
    [sym__wc_get_buffer_string_int] = ACTIONS(1777),
    [sym__wc_concat_string_int] = ACTIONS(1777),
    [sym__wc_file_open_int] = ACTIONS(1777),
    [sym__wc_file_close_int] = ACTIONS(1777),
    [sym__wc_file_read_int] = ACTIONS(1777),
    [sym__wc_file_read_line_int] = ACTIONS(1777),
    [sym__wc_file_write_int] = ACTIONS(1777),
    [sym__wc_file_write_line_int] = ACTIONS(1777),
    [aux_sym_preproc_include_token1] = ACTIONS(1777),
    [aux_sym_preproc_define_token1] = ACTIONS(1779),
    [aux_sym_preproc_warn_token1] = ACTIONS(1779),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1779),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_else_token1] = ACTIONS(1779),
    [anon_sym_LBRACK] = ACTIONS(1779),
    [anon_sym_LBRACE] = ACTIONS(1779),
    [anon_sym_LPAREN] = ACTIONS(1777),
    [anon_sym_BANG] = ACTIONS(1779),
    [anon_sym_TILDE] = ACTIONS(1779),
    [anon_sym_DASH] = ACTIONS(1777),
    [anon_sym_PLUS] = ACTIONS(1777),
    [anon_sym_DASH_DASH] = ACTIONS(1779),
    [anon_sym_PLUS_PLUS] = ACTIONS(1779),
    [anon_sym_DQUOTE] = ACTIONS(1779),
    [anon_sym_SQUOTE] = ACTIONS(1779),
    [sym_number_literal] = ACTIONS(1779),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(234)] = {
    [sym_identifier] = ACTIONS(1781),
    [sym_program_name_keyword] = ACTIONS(1781),
    [sym_module_name_keyword] = ACTIONS(1781),
    [sym_define_device_keyword] = ACTIONS(1781),
    [sym_define_combine_keyword] = ACTIONS(1781),
    [sym_define_constant_keyword] = ACTIONS(1781),
    [sym_define_type_keyword] = ACTIONS(1781),
    [sym_define_variable_keyword] = ACTIONS(1781),
    [sym_define_system_variable_keyword] = ACTIONS(1781),
    [sym_define_connect_level_keyword] = ACTIONS(1781),
    [sym_define_latching_keyword] = ACTIONS(1781),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1781),
    [sym_define_toggling_keyword] = ACTIONS(1781),
    [sym_define_call_keyword] = ACTIONS(1781),
    [sym_define_function_keyword] = ACTIONS(1781),
    [sym_define_library_function_keyword] = ACTIONS(1781),
    [sym_define_start_keyword] = ACTIONS(1781),
    [sym_define_event_keyword] = ACTIONS(1781),
    [sym_define_program_keyword] = ACTIONS(1781),
    [sym_define_module_keyword] = ACTIONS(1781),
    [sym_char_keyword] = ACTIONS(1781),
    [sym_widechar_keyword] = ACTIONS(1781),
    [sym_integer_keyword] = ACTIONS(1781),
    [sym_sinteger_keyword] = ACTIONS(1781),
    [sym_long_keyword] = ACTIONS(1781),
    [sym_slong_keyword] = ACTIONS(1781),
    [sym_float_keyword] = ACTIONS(1781),
    [sym_double_keyword] = ACTIONS(1781),
    [sym_variant_keyword] = ACTIONS(1781),
    [sym_variantarray_keyword] = ACTIONS(1781),
    [sym_local_var_keyword] = ACTIONS(1781),
    [sym_stack_var_keyword] = ACTIONS(1781),
    [sym_constant_keyword] = ACTIONS(1781),
    [sym_volatile_keyword] = ACTIONS(1781),
    [sym_non_volatile_keyword] = ACTIONS(1781),
    [sym_persistent_keyword] = ACTIONS(1781),
    [sym_dev_keyword] = ACTIONS(1781),
    [sym_devlev_keyword] = ACTIONS(1781),
    [sym_devchan_keyword] = ACTIONS(1781),
    [sym_if_keyword] = ACTIONS(1781),
    [sym_else_keyword] = ACTIONS(1781),
    [sym_switch_keyword] = ACTIONS(1781),
    [sym_case_keyword] = ACTIONS(1781),
    [sym_default_keyword] = ACTIONS(1781),
    [sym_while_keyword] = ACTIONS(1781),
    [sym_for_keyword] = ACTIONS(1781),
    [sym_break_keyword] = ACTIONS(1781),
    [sym_continue_keyword] = ACTIONS(1781),
    [sym_return_keyword] = ACTIONS(1781),
    [sym_select_keyword] = ACTIONS(1781),
    [sym_active_keyword] = ACTIONS(1781),
    [sym_struct_keyword] = ACTIONS(1781),
    [sym_structure_keyword] = ACTIONS(1781),
    [sym_button_event_keyword] = ACTIONS(1781),
    [sym_channel_event_keyword] = ACTIONS(1781),
    [sym_level_event_keyword] = ACTIONS(1781),
    [sym_data_event_keyword] = ACTIONS(1781),
    [sym_timeline_event_keyword] = ACTIONS(1781),
    [sym_custom_event_keyword] = ACTIONS(1781),
    [sym_push_keyword] = ACTIONS(1781),
    [sym_release_keyword] = ACTIONS(1781),
    [aux_sym_on_keyword_token1] = ACTIONS(1781),
    [aux_sym_off_keyword_token1] = ACTIONS(1781),
    [sym_send_level_keyword] = ACTIONS(1781),
    [sym_send_string_keyword] = ACTIONS(1781),
    [sym_send_command_keyword] = ACTIONS(1781),
    [sym_clear_buffer_keyword] = ACTIONS(1781),
    [sym_create_buffer_keyword] = ACTIONS(1781),
    [sym_create_multi_buffer_keyword] = ACTIONS(1781),
    [sym_call_keyword] = ACTIONS(1781),
    [sym_system_call_keyword] = ACTIONS(1781),
    [sym_devchan_to_keyword] = ACTIONS(1781),
    [sym_devchan_min_to_keyword] = ACTIONS(1781),
    [sym_devchan_total_off_keyword] = ACTIONS(1781),
    [sym_devchan_pulse_keyword] = ACTIONS(1781),
    [sym_wait_keyword] = ACTIONS(1781),
    [sym_cancel_wait_keyword] = ACTIONS(1781),
    [sym_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1781),
    [sym_not] = ACTIONS(1781),
    [sym_bnot] = ACTIONS(1781),
    [sym___file__] = ACTIONS(1781),
    [sym___date__] = ACTIONS(1781),
    [sym___ldate__] = ACTIONS(1781),
    [sym___time__] = ACTIONS(1781),
    [sym___line__] = ACTIONS(1781),
    [sym___name__] = ACTIONS(1781),
    [sym___netlinx__] = ACTIONS(1781),
    [sym_netlinx_axi_version] = ACTIONS(1781),
    [sym_url_flg_tcp] = ACTIONS(1781),
    [sym_url_flg_accinfopresent] = ACTIONS(1781),
    [sym_url_flg_temp] = ACTIONS(1781),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1781),
    [sym_url_flg_stat_mask] = ACTIONS(1781),
    [sym_url_flg_stat_lookup] = ACTIONS(1781),
    [sym_url_flg_stat_connecting] = ACTIONS(1781),
    [sym_url_flg_stat_waiting] = ACTIONS(1781),
    [sym_url_flg_stat_connected] = ACTIONS(1781),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1781),
    [sym_first_virtual_device] = ACTIONS(1781),
    [sym_first_local_port] = ACTIONS(1781),
    [sym_dynamic_virtual_device] = ACTIONS(1781),
    [sym_timeline_once] = ACTIONS(1781),
    [sym_timeline_repeat] = ACTIONS(1781),
    [sym_timeline_absolute] = ACTIONS(1781),
    [sym_timeline_relative] = ACTIONS(1781),
    [sym_do_push_timed_infinite] = ACTIONS(1781),
    [sym_source_type_no_address] = ACTIONS(1781),
    [sym_source_type_neuron_id] = ACTIONS(1781),
    [sym_source_type_ip_address] = ACTIONS(1781),
    [sym_source_type_axlink] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1781),
    [sym_source_type_ip_socket_address] = ACTIONS(1781),
    [sym_source_type_rs232] = ACTIONS(1781),
    [sym_source_type_internal] = ACTIONS(1781),
    [sym_source_type_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1781),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1781),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1781),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1781),
    [sym_true] = ACTIONS(1781),
    [sym_false] = ACTIONS(1781),
    [sym_file_read_only] = ACTIONS(1781),
    [sym_file_rw_new] = ACTIONS(1781),
    [sym_file_rw_append] = ACTIONS(1781),
    [sym_ip_tcp] = ACTIONS(1781),
    [sym_ip_udp] = ACTIONS(1781),
    [sym_ip_udp_2way] = ACTIONS(1781),
    [sym_xml_encode_types] = ACTIONS(1781),
    [sym_xml_encode_char_as_list] = ACTIONS(1781),
    [sym_xml_encode_le] = ACTIONS(1781),
    [sym_xml_decode_types] = ACTIONS(1781),
    [sym_xml_decode_no_preserve] = ACTIONS(1781),
    [sym_duet_dev_type_display_device] = ACTIONS(1781),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_io_device] = ACTIONS(1781),
    [sym_duet_dev_type_relay_device] = ACTIONS(1781),
    [sym_duet_dev_type_ups] = ACTIONS(1781),
    [sym_duet_dev_type_amplifier] = ACTIONS(1781),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1781),
    [sym_duet_dev_type_camera] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1781),
    [sym_duet_dev_type_dss] = ACTIONS(1781),
    [sym_duet_dev_type_dvr] = ACTIONS(1781),
    [sym_duet_dev_type_disc_device] = ACTIONS(1781),
    [sym_duet_dev_type_document_camera] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1781),
    [sym_duet_dev_type_hvac] = ACTIONS(1781),
    [sym_duet_dev_type_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_light] = ACTIONS(1781),
    [sym_duet_dev_type_monitor] = ACTIONS(1781),
    [sym_duet_dev_type_motor] = ACTIONS(1781),
    [sym_duet_dev_type_multi_window] = ACTIONS(1781),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1781),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1781),
    [sym_duet_dev_type_receiver] = ACTIONS(1781),
    [sym_duet_dev_type_security_system] = ACTIONS(1781),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1781),
    [sym_duet_dev_type_settop_box] = ACTIONS(1781),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1781),
    [sym_duet_dev_type_switcher] = ACTIONS(1781),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_tv] = ACTIONS(1781),
    [sym_duet_dev_type_utility] = ACTIONS(1781),
    [sym_duet_dev_type_vcr] = ACTIONS(1781),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_video_processor] = ACTIONS(1781),
    [sym_duet_dev_type_video_projector] = ACTIONS(1781),
    [sym_duet_dev_type_video_wall] = ACTIONS(1781),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1781),
    [sym_duet_dev_type_weather] = ACTIONS(1781),
    [sym_duet_dev_not_polled] = ACTIONS(1781),
    [sym_duet_dev_polled] = ACTIONS(1781),
    [sym_clkmgr_mode_network] = ACTIONS(1781),
    [sym_clkmgr_mode_standalone] = ACTIONS(1781),
    [sym_normal_standby] = ACTIONS(1781),
    [sym_normal_wake] = ACTIONS(1781),
    [sym_ip_multicast_ttl_option] = ACTIONS(1781),
    [sym_ip_ttl_subnet] = ACTIONS(1781),
    [sym_ip_ttl_site] = ACTIONS(1781),
    [sym_ip_ttl_region] = ACTIONS(1781),
    [sym_ip_ttl_continent] = ACTIONS(1781),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1781),
    [sym_ip_nodelay_on] = ACTIONS(1781),
    [sym_ip_nodelay_off] = ACTIONS(1781),
    [sym_amx_error] = ACTIONS(1781),
    [sym_amx_warning] = ACTIONS(1781),
    [sym_amx_info] = ACTIONS(1781),
    [sym_amx_debug] = ACTIONS(1781),
    [sym_netlinx_login_success] = ACTIONS(1781),
    [sym_netlinx_login_fail] = ACTIONS(1781),
    [sym_netlinx_logout] = ACTIONS(1781),
    [sym_null_str] = ACTIONS(1781),
    [sym_smtp_address] = ACTIONS(1781),
    [sym_smtp_port_number] = ACTIONS(1781),
    [sym_smtp_username] = ACTIONS(1781),
    [sym_smtp_password] = ACTIONS(1781),
    [sym_smtp_require_tls] = ACTIONS(1781),
    [sym_smtp_from] = ACTIONS(1781),
    [sym_smtp_tls_true] = ACTIONS(1781),
    [sym_smtp_tls_false] = ACTIONS(1781),
    [sym_listview_on_row_select_event] = ACTIONS(1781),
    [sym_data_structure_unknown] = ACTIONS(1781),
    [sym_data_structure_datafeed] = ACTIONS(1781),
    [sym_data_structure_datarecordset] = ACTIONS(1781),
    [sym_data_structure_datarecord] = ACTIONS(1781),
    [sym_data_structure_datafield] = ACTIONS(1781),
    [sym_data_max_content_field_count] = ACTIONS(1781),
    [sym_data_max_metadata_field_count] = ACTIONS(1781),
    [sym_data_max_id_length] = ACTIONS(1781),
    [sym_data_max_name_length] = ACTIONS(1781),
    [sym_data_max_description_length] = ACTIONS(1781),
    [sym_data_max_type_length] = ACTIONS(1781),
    [sym_data_max_format_length] = ACTIONS(1781),
    [sym_data_max_source_length] = ACTIONS(1781),
    [sym_data_max_value_length] = ACTIONS(1781),
    [sym_data_max_label_length] = ACTIONS(1781),
    [sym_data_max_url_length] = ACTIONS(1781),
    [sym_max_last_login_info_length] = ACTIONS(1781),
    [sym_data_type_unknown] = ACTIONS(1781),
    [sym_data_type_string] = ACTIONS(1781),
    [sym_data_type_datetime] = ACTIONS(1781),
    [sym_data_type_date] = ACTIONS(1781),
    [sym_data_type_time] = ACTIONS(1781),
    [sym_data_type_image] = ACTIONS(1781),
    [sym_data_format_url] = ACTIONS(1781),
    [sym_data_format_phone] = ACTIONS(1781),
    [sym_data_format_email] = ACTIONS(1781),
    [sym_data_format_iso8601] = ACTIONS(1781),
    [sym_auditlog_priv] = ACTIONS(1781),
    [sym_configuration_priv] = ACTIONS(1781),
    [sym_deviceconfig_priv] = ACTIONS(1781),
    [sym_ftp_priv] = ACTIONS(1781),
    [sym_http_priv] = ACTIONS(1781),
    [sym_networkconfig_priv] = ACTIONS(1781),
    [sym_programport_priv] = ACTIONS(1781),
    [sym_remoteui_priv] = ACTIONS(1781),
    [sym_securitycontrol_priv] = ACTIONS(1781),
    [sym_softwaremanagement_priv] = ACTIONS(1781),
    [sym_terminal_priv] = ACTIONS(1781),
    [sym_tpadmin_priv] = ACTIONS(1781),
    [sym_user1_priv] = ACTIONS(1781),
    [sym_user2_priv] = ACTIONS(1781),
    [sym_user3_priv] = ACTIONS(1781),
    [sym_user4_priv] = ACTIONS(1781),
    [sym_usermanagement_priv] = ACTIONS(1781),
    [sym_valid_account] = ACTIONS(1781),
    [sym_err_name_invalid] = ACTIONS(1781),
    [sym_err_password_invalid] = ACTIONS(1781),
    [sym_err_invalid_account] = ACTIONS(1781),
    [sym_err_no_authorization] = ACTIONS(1781),
    [sym_err_invalid_parameter] = ACTIONS(1781),
    [sym_err_user_locked_out] = ACTIONS(1781),
    [sym_err_user_account_expired] = ACTIONS(1781),
    [sym_err_general_error] = ACTIONS(1781),
    [sym_err_authentication_error] = ACTIONS(1781),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1781),
    [sym_tls_validate_certificate] = ACTIONS(1781),
    [sym_tdata] = ACTIONS(1781),
    [sym_tchannel] = ACTIONS(1781),
    [sym_tlevel] = ACTIONS(1781),
    [sym_tbutton] = ACTIONS(1781),
    [sym_ttimeline] = ACTIONS(1781),
    [sym_tcustom] = ACTIONS(1781),
    [sym_url_struct] = ACTIONS(1781),
    [sym_dns_struct] = ACTIONS(1781),
    [sym_ip_address_struct] = ACTIONS(1781),
    [sym_dev_info_struct] = ACTIONS(1781),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1781),
    [sym_data_feed] = ACTIONS(1781),
    [sym_data_field] = ACTIONS(1781),
    [sym_data_record] = ACTIONS(1781),
    [sym_wc_data_feed] = ACTIONS(1781),
    [sym_wc_data_field] = ACTIONS(1781),
    [sym_wc_data_record] = ACTIONS(1781),
    [sym_last_login_info] = ACTIONS(1781),
    [sym____reserved___] = ACTIONS(1781),
    [sym_date] = ACTIONS(1781),
    [sym_day] = ACTIONS(1781),
    [sym_time] = ACTIONS(1781),
    [sym_ldate] = ACTIONS(1781),
    [sym_push_device] = ACTIONS(1781),
    [sym_push_channel] = ACTIONS(1781),
    [sym_push_devchan] = ACTIONS(1781),
    [sym_release_device] = ACTIONS(1781),
    [sym_release_channel] = ACTIONS(1781),
    [sym_release_devchan] = ACTIONS(1781),
    [sym_master_slot] = ACTIONS(1781),
    [sym_get_pulse_time] = ACTIONS(1781),
    [sym_get_timer] = ACTIONS(1781),
    [sym_master_sn] = ACTIONS(1781),
    [sym_system_number] = ACTIONS(1781),
    [sym_button] = ACTIONS(1781),
    [sym_dv_channel] = ACTIONS(1781),
    [sym_data] = ACTIONS(1781),
    [sym_level] = ACTIONS(1781),
    [sym_timeline] = ACTIONS(1781),
    [sym_channel] = ACTIONS(1781),
    [sym_custom] = ACTIONS(1781),
    [sym_atoi] = ACTIONS(1781),
    [sym_atol] = ACTIONS(1781),
    [sym_atof] = ACTIONS(1781),
    [sym_itoa] = ACTIONS(1781),
    [sym_itohex] = ACTIONS(1781),
    [sym_hextoi] = ACTIONS(1781),
    [sym_ftoa] = ACTIONS(1781),
    [sym_find_string] = ACTIONS(1781),
    [sym_left_string] = ACTIONS(1781),
    [sym_right_string] = ACTIONS(1781),
    [sym_mid_string] = ACTIONS(1781),
    [sym_remove_string] = ACTIONS(1781),
    [sym_get_buffer_string] = ACTIONS(1781),
    [sym_get_buffer_char] = ACTIONS(1781),
    [sym_get_multi_buffer_string] = ACTIONS(1781),
    [sym_length_string] = ACTIONS(1781),
    [sym_max_length_string] = ACTIONS(1781),
    [sym_set_length_string] = ACTIONS(1781),
    [sym_compare_string] = ACTIONS(1781),
    [sym_upper_string] = ACTIONS(1781),
    [sym_lower_string] = ACTIONS(1781),
    [sym_get_last] = ACTIONS(1781),
    [sym_date_to_day] = ACTIONS(1781),
    [sym_date_to_month] = ACTIONS(1781),
    [sym_date_to_year] = ACTIONS(1781),
    [sym_time_to_hour] = ACTIONS(1781),
    [sym_time_to_minute] = ACTIONS(1781),
    [sym_time_to_second] = ACTIONS(1781),
    [sym_day_of_week] = ACTIONS(1781),
    [sym_set_pulse_time] = ACTIONS(1781),
    [sym_set_timer] = ACTIONS(1781),
    [sym_file_open] = ACTIONS(1781),
    [sym_file_close] = ACTIONS(1781),
    [sym_file_read] = ACTIONS(1781),
    [sym_file_read_line] = ACTIONS(1781),
    [sym_file_write] = ACTIONS(1781),
    [sym_file_write_line] = ACTIONS(1781),
    [sym_file_copy] = ACTIONS(1781),
    [sym_file_delete] = ACTIONS(1781),
    [sym_file_rename] = ACTIONS(1781),
    [sym_file_dir] = ACTIONS(1781),
    [sym_file_getdir] = ACTIONS(1781),
    [sym_file_setdir] = ACTIONS(1781),
    [sym_file_create_dir] = ACTIONS(1781),
    [sym_file_remove_dir] = ACTIONS(1781),
    [sym_file_seek] = ACTIONS(1781),
    [sym_length_array] = ACTIONS(1781),
    [sym_max_length_array] = ACTIONS(1781),
    [sym_set_length_array] = ACTIONS(1781),
    [sym_ip_client_open] = ACTIONS(1781),
    [sym_ip_client_close] = ACTIONS(1781),
    [sym_ip_server_open] = ACTIONS(1781),
    [sym_ip_server_close] = ACTIONS(1781),
    [sym_ip_bound_client_open] = ACTIONS(1781),
    [sym_ip_mc_server_open] = ACTIONS(1781),
    [sym_ip_set_option] = ACTIONS(1781),
    [sym_ssh_client_open] = ACTIONS(1781),
    [sym_ssh_client_close] = ACTIONS(1781),
    [sym_tls_client_open] = ACTIONS(1781),
    [sym_tls_client_close] = ACTIONS(1781),
    [sym_do_push] = ACTIONS(1781),
    [sym_do_push_timed] = ACTIONS(1781),
    [sym_do_release] = ACTIONS(1781),
    [sym_device_id] = ACTIONS(1781),
    [sym_device_info] = ACTIONS(1781),
    [sym_device_id_string] = ACTIONS(1781),
    [sym_device_standby] = ACTIONS(1781),
    [sym_device_wake] = ACTIONS(1781),
    [sym_timeline_create] = ACTIONS(1781),
    [sym_timeline_kill] = ACTIONS(1781),
    [sym_timeline_reload] = ACTIONS(1781),
    [sym_timeline_pause] = ACTIONS(1781),
    [sym_timeline_restart] = ACTIONS(1781),
    [sym_timeline_set] = ACTIONS(1781),
    [sym_timeline_get] = ACTIONS(1781),
    [sym_timeline_active] = ACTIONS(1781),
    [sym_timeline_dynamic_id] = ACTIONS(1781),
    [sym_variable_to_string] = ACTIONS(1781),
    [sym_string_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_string] = ACTIONS(1781),
    [sym_type_cast] = ACTIONS(1781),
    [sym_variable_to_xml] = ACTIONS(1781),
    [sym_xml_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_xml] = ACTIONS(1781),
    [sym_redirect_string] = ACTIONS(1781),
    [sym_random_number] = ACTIONS(1781),
    [sym_get_unique_id] = ACTIONS(1781),
    [sym_get_system_number] = ACTIONS(1781),
    [sym_set_system_number] = ACTIONS(1781),
    [sym_get_serial_number] = ACTIONS(1781),
    [sym_get_master_build] = ACTIONS(1781),
    [sym_set_validation_code] = ACTIONS(1781),
    [sym_reboot] = ACTIONS(1781),
    [sym_load_duet_module] = ACTIONS(1781),
    [sym_unload_duet_module] = ACTIONS(1781),
    [sym_duet_mem_size_set] = ACTIONS(1781),
    [sym_duet_mem_size_get] = ACTIONS(1781),
    [sym_dynamic_application_device] = ACTIONS(1781),
    [sym_dynamic_polled_port] = ACTIONS(1781),
    [sym_static_port_binding] = ACTIONS(1781),
    [sym_static_ip_binding] = ACTIONS(1781),
    [sym_get_dns_list] = ACTIONS(1781),
    [sym_set_dns_list] = ACTIONS(1781),
    [sym_get_ip_address] = ACTIONS(1781),
    [sym_set_ip_address] = ACTIONS(1781),
    [sym_get_url_list] = ACTIONS(1781),
    [sym_add_url_entry] = ACTIONS(1781),
    [sym_delete_url_entry] = ACTIONS(1781),
    [sym_abs_value] = ACTIONS(1781),
    [sym_max_value] = ACTIONS(1781),
    [sym_min_value] = ACTIONS(1781),
    [sym_exp_value] = ACTIONS(1781),
    [sym_log_value] = ACTIONS(1781),
    [sym_log10_value] = ACTIONS(1781),
    [sym_power_value] = ACTIONS(1781),
    [sym_sqrt_value] = ACTIONS(1781),
    [sym_raw_be] = ACTIONS(1781),
    [sym_raw_le] = ACTIONS(1781),
    [sym_internal_queue_size_set] = ACTIONS(1781),
    [sym_internal_queue_size_get] = ACTIONS(1781),
    [sym_internal_threshold_set] = ACTIONS(1781),
    [sym_internal_threshold_get] = ACTIONS(1781),
    [sym_rebuilt_event] = ACTIONS(1781),
    [sym_do_custom_event] = ACTIONS(1781),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1781),
    [sym_clkmgr_set_clk_source] = ACTIONS(1781),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1781),
    [sym_clkmgr_get_timezone] = ACTIONS(1781),
    [sym_clkmgr_set_timezone] = ACTIONS(1781),
    [sym_clkmgr_get_resync_period] = ACTIONS(1781),
    [sym_clkmgr_set_resync_period] = ACTIONS(1781),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_timeservers] = ACTIONS(1781),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_set_log_level] = ACTIONS(1781),
    [sym_get_log_level] = ACTIONS(1781),
    [sym_amx_log] = ACTIONS(1781),
    [sym_smtp_server_config_set] = ACTIONS(1781),
    [sym_smtp_server_config_get] = ACTIONS(1781),
    [sym_smtp_send] = ACTIONS(1781),
    [sym_validate_netlinx_account] = ACTIONS(1781),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1781),
    [sym_authenticate_certificate] = ACTIONS(1781),
    [sym_audit_netlinx_session_event] = ACTIONS(1781),
    [sym_audit_netlinx_generic_event] = ACTIONS(1781),
    [sym_get_available_flash_disk_space] = ACTIONS(1781),
    [sym_get_max_flash_disk_space] = ACTIONS(1781),
    [sym_data_create_feed] = ACTIONS(1781),
    [sym_data_delete_feed] = ACTIONS(1781),
    [sym_data_publish_feed] = ACTIONS(1781),
    [sym_data_get_published_feed] = ACTIONS(1781),
    [sym_data_add_record] = ACTIONS(1781),
    [sym_data_get_event_record] = ACTIONS(1781),
    [sym__wc_data_create_feed] = ACTIONS(1781),
    [sym__wc_data_add_record] = ACTIONS(1781),
    [sym__wc_data_get_event_record] = ACTIONS(1781),
    [sym_format] = ACTIONS(1781),
    [sym_astro_clock] = ACTIONS(1781),
    [sym_set_outdoor_temperature] = ACTIONS(1781),
    [sym_set_virtual_level_count] = ACTIONS(1781),
    [sym_set_virtual_channel_count] = ACTIONS(1781),
    [sym_set_virtual_port_count] = ACTIONS(1781),
    [sym__wc_encode_int] = ACTIONS(1781),
    [sym__wc_decode_int] = ACTIONS(1781),
    [sym__wc_int] = ACTIONS(1781),
    [sym__wc_to_ch_int] = ACTIONS(1781),
    [sym__ch_to_wc_int] = ACTIONS(1781),
    [sym__wc_find_string_int] = ACTIONS(1781),
    [sym__wc_left_string_int] = ACTIONS(1781),
    [sym__wc_lower_string_int] = ACTIONS(1781),
    [sym__wc_mid_string_int] = ACTIONS(1781),
    [sym__wc_remove_string_int] = ACTIONS(1781),
    [sym__wc_right_string_int] = ACTIONS(1781),
    [sym__wc_upper_string_int] = ACTIONS(1781),
    [sym__wc_compare_string_int] = ACTIONS(1781),
    [sym__wc_get_buffer_char_int] = ACTIONS(1781),
    [sym__wc_get_buffer_string_int] = ACTIONS(1781),
    [sym__wc_concat_string_int] = ACTIONS(1781),
    [sym__wc_file_open_int] = ACTIONS(1781),
    [sym__wc_file_close_int] = ACTIONS(1781),
    [sym__wc_file_read_int] = ACTIONS(1781),
    [sym__wc_file_read_line_int] = ACTIONS(1781),
    [sym__wc_file_write_int] = ACTIONS(1781),
    [sym__wc_file_write_line_int] = ACTIONS(1781),
    [aux_sym_preproc_include_token1] = ACTIONS(1781),
    [aux_sym_preproc_define_token1] = ACTIONS(1783),
    [aux_sym_preproc_warn_token1] = ACTIONS(1783),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1783),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_else_token1] = ACTIONS(1783),
    [anon_sym_LBRACK] = ACTIONS(1783),
    [anon_sym_LBRACE] = ACTIONS(1783),
    [anon_sym_LPAREN] = ACTIONS(1781),
    [anon_sym_BANG] = ACTIONS(1783),
    [anon_sym_TILDE] = ACTIONS(1783),
    [anon_sym_DASH] = ACTIONS(1781),
    [anon_sym_PLUS] = ACTIONS(1781),
    [anon_sym_DASH_DASH] = ACTIONS(1783),
    [anon_sym_PLUS_PLUS] = ACTIONS(1783),
    [anon_sym_DQUOTE] = ACTIONS(1783),
    [anon_sym_SQUOTE] = ACTIONS(1783),
    [sym_number_literal] = ACTIONS(1783),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(235)] = {
    [sym_identifier] = ACTIONS(1781),
    [sym_program_name_keyword] = ACTIONS(1781),
    [sym_module_name_keyword] = ACTIONS(1781),
    [sym_define_device_keyword] = ACTIONS(1781),
    [sym_define_combine_keyword] = ACTIONS(1781),
    [sym_define_constant_keyword] = ACTIONS(1781),
    [sym_define_type_keyword] = ACTIONS(1781),
    [sym_define_variable_keyword] = ACTIONS(1781),
    [sym_define_system_variable_keyword] = ACTIONS(1781),
    [sym_define_connect_level_keyword] = ACTIONS(1781),
    [sym_define_latching_keyword] = ACTIONS(1781),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1781),
    [sym_define_toggling_keyword] = ACTIONS(1781),
    [sym_define_call_keyword] = ACTIONS(1781),
    [sym_define_function_keyword] = ACTIONS(1781),
    [sym_define_library_function_keyword] = ACTIONS(1781),
    [sym_define_start_keyword] = ACTIONS(1781),
    [sym_define_event_keyword] = ACTIONS(1781),
    [sym_define_program_keyword] = ACTIONS(1781),
    [sym_define_module_keyword] = ACTIONS(1781),
    [sym_char_keyword] = ACTIONS(1781),
    [sym_widechar_keyword] = ACTIONS(1781),
    [sym_integer_keyword] = ACTIONS(1781),
    [sym_sinteger_keyword] = ACTIONS(1781),
    [sym_long_keyword] = ACTIONS(1781),
    [sym_slong_keyword] = ACTIONS(1781),
    [sym_float_keyword] = ACTIONS(1781),
    [sym_double_keyword] = ACTIONS(1781),
    [sym_variant_keyword] = ACTIONS(1781),
    [sym_variantarray_keyword] = ACTIONS(1781),
    [sym_local_var_keyword] = ACTIONS(1781),
    [sym_stack_var_keyword] = ACTIONS(1781),
    [sym_constant_keyword] = ACTIONS(1781),
    [sym_volatile_keyword] = ACTIONS(1781),
    [sym_non_volatile_keyword] = ACTIONS(1781),
    [sym_persistent_keyword] = ACTIONS(1781),
    [sym_dev_keyword] = ACTIONS(1781),
    [sym_devlev_keyword] = ACTIONS(1781),
    [sym_devchan_keyword] = ACTIONS(1781),
    [sym_if_keyword] = ACTIONS(1781),
    [sym_else_keyword] = ACTIONS(1781),
    [sym_switch_keyword] = ACTIONS(1781),
    [sym_case_keyword] = ACTIONS(1781),
    [sym_default_keyword] = ACTIONS(1781),
    [sym_while_keyword] = ACTIONS(1781),
    [sym_for_keyword] = ACTIONS(1781),
    [sym_break_keyword] = ACTIONS(1781),
    [sym_continue_keyword] = ACTIONS(1781),
    [sym_return_keyword] = ACTIONS(1781),
    [sym_select_keyword] = ACTIONS(1781),
    [sym_active_keyword] = ACTIONS(1781),
    [sym_struct_keyword] = ACTIONS(1781),
    [sym_structure_keyword] = ACTIONS(1781),
    [sym_button_event_keyword] = ACTIONS(1781),
    [sym_channel_event_keyword] = ACTIONS(1781),
    [sym_level_event_keyword] = ACTIONS(1781),
    [sym_data_event_keyword] = ACTIONS(1781),
    [sym_timeline_event_keyword] = ACTIONS(1781),
    [sym_custom_event_keyword] = ACTIONS(1781),
    [sym_push_keyword] = ACTIONS(1781),
    [sym_release_keyword] = ACTIONS(1781),
    [aux_sym_on_keyword_token1] = ACTIONS(1781),
    [aux_sym_off_keyword_token1] = ACTIONS(1781),
    [sym_send_level_keyword] = ACTIONS(1781),
    [sym_send_string_keyword] = ACTIONS(1781),
    [sym_send_command_keyword] = ACTIONS(1781),
    [sym_clear_buffer_keyword] = ACTIONS(1781),
    [sym_create_buffer_keyword] = ACTIONS(1781),
    [sym_create_multi_buffer_keyword] = ACTIONS(1781),
    [sym_call_keyword] = ACTIONS(1781),
    [sym_system_call_keyword] = ACTIONS(1781),
    [sym_devchan_to_keyword] = ACTIONS(1781),
    [sym_devchan_min_to_keyword] = ACTIONS(1781),
    [sym_devchan_total_off_keyword] = ACTIONS(1781),
    [sym_devchan_pulse_keyword] = ACTIONS(1781),
    [sym_wait_keyword] = ACTIONS(1781),
    [sym_cancel_wait_keyword] = ACTIONS(1781),
    [sym_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1781),
    [sym_not] = ACTIONS(1781),
    [sym_bnot] = ACTIONS(1781),
    [sym___file__] = ACTIONS(1781),
    [sym___date__] = ACTIONS(1781),
    [sym___ldate__] = ACTIONS(1781),
    [sym___time__] = ACTIONS(1781),
    [sym___line__] = ACTIONS(1781),
    [sym___name__] = ACTIONS(1781),
    [sym___netlinx__] = ACTIONS(1781),
    [sym_netlinx_axi_version] = ACTIONS(1781),
    [sym_url_flg_tcp] = ACTIONS(1781),
    [sym_url_flg_accinfopresent] = ACTIONS(1781),
    [sym_url_flg_temp] = ACTIONS(1781),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1781),
    [sym_url_flg_stat_mask] = ACTIONS(1781),
    [sym_url_flg_stat_lookup] = ACTIONS(1781),
    [sym_url_flg_stat_connecting] = ACTIONS(1781),
    [sym_url_flg_stat_waiting] = ACTIONS(1781),
    [sym_url_flg_stat_connected] = ACTIONS(1781),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1781),
    [sym_first_virtual_device] = ACTIONS(1781),
    [sym_first_local_port] = ACTIONS(1781),
    [sym_dynamic_virtual_device] = ACTIONS(1781),
    [sym_timeline_once] = ACTIONS(1781),
    [sym_timeline_repeat] = ACTIONS(1781),
    [sym_timeline_absolute] = ACTIONS(1781),
    [sym_timeline_relative] = ACTIONS(1781),
    [sym_do_push_timed_infinite] = ACTIONS(1781),
    [sym_source_type_no_address] = ACTIONS(1781),
    [sym_source_type_neuron_id] = ACTIONS(1781),
    [sym_source_type_ip_address] = ACTIONS(1781),
    [sym_source_type_axlink] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1781),
    [sym_source_type_ip_socket_address] = ACTIONS(1781),
    [sym_source_type_rs232] = ACTIONS(1781),
    [sym_source_type_internal] = ACTIONS(1781),
    [sym_source_type_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1781),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1781),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1781),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1781),
    [sym_true] = ACTIONS(1781),
    [sym_false] = ACTIONS(1781),
    [sym_file_read_only] = ACTIONS(1781),
    [sym_file_rw_new] = ACTIONS(1781),
    [sym_file_rw_append] = ACTIONS(1781),
    [sym_ip_tcp] = ACTIONS(1781),
    [sym_ip_udp] = ACTIONS(1781),
    [sym_ip_udp_2way] = ACTIONS(1781),
    [sym_xml_encode_types] = ACTIONS(1781),
    [sym_xml_encode_char_as_list] = ACTIONS(1781),
    [sym_xml_encode_le] = ACTIONS(1781),
    [sym_xml_decode_types] = ACTIONS(1781),
    [sym_xml_decode_no_preserve] = ACTIONS(1781),
    [sym_duet_dev_type_display_device] = ACTIONS(1781),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_io_device] = ACTIONS(1781),
    [sym_duet_dev_type_relay_device] = ACTIONS(1781),
    [sym_duet_dev_type_ups] = ACTIONS(1781),
    [sym_duet_dev_type_amplifier] = ACTIONS(1781),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1781),
    [sym_duet_dev_type_camera] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1781),
    [sym_duet_dev_type_dss] = ACTIONS(1781),
    [sym_duet_dev_type_dvr] = ACTIONS(1781),
    [sym_duet_dev_type_disc_device] = ACTIONS(1781),
    [sym_duet_dev_type_document_camera] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1781),
    [sym_duet_dev_type_hvac] = ACTIONS(1781),
    [sym_duet_dev_type_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_light] = ACTIONS(1781),
    [sym_duet_dev_type_monitor] = ACTIONS(1781),
    [sym_duet_dev_type_motor] = ACTIONS(1781),
    [sym_duet_dev_type_multi_window] = ACTIONS(1781),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1781),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1781),
    [sym_duet_dev_type_receiver] = ACTIONS(1781),
    [sym_duet_dev_type_security_system] = ACTIONS(1781),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1781),
    [sym_duet_dev_type_settop_box] = ACTIONS(1781),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1781),
    [sym_duet_dev_type_switcher] = ACTIONS(1781),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_tv] = ACTIONS(1781),
    [sym_duet_dev_type_utility] = ACTIONS(1781),
    [sym_duet_dev_type_vcr] = ACTIONS(1781),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_video_processor] = ACTIONS(1781),
    [sym_duet_dev_type_video_projector] = ACTIONS(1781),
    [sym_duet_dev_type_video_wall] = ACTIONS(1781),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1781),
    [sym_duet_dev_type_weather] = ACTIONS(1781),
    [sym_duet_dev_not_polled] = ACTIONS(1781),
    [sym_duet_dev_polled] = ACTIONS(1781),
    [sym_clkmgr_mode_network] = ACTIONS(1781),
    [sym_clkmgr_mode_standalone] = ACTIONS(1781),
    [sym_normal_standby] = ACTIONS(1781),
    [sym_normal_wake] = ACTIONS(1781),
    [sym_ip_multicast_ttl_option] = ACTIONS(1781),
    [sym_ip_ttl_subnet] = ACTIONS(1781),
    [sym_ip_ttl_site] = ACTIONS(1781),
    [sym_ip_ttl_region] = ACTIONS(1781),
    [sym_ip_ttl_continent] = ACTIONS(1781),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1781),
    [sym_ip_nodelay_on] = ACTIONS(1781),
    [sym_ip_nodelay_off] = ACTIONS(1781),
    [sym_amx_error] = ACTIONS(1781),
    [sym_amx_warning] = ACTIONS(1781),
    [sym_amx_info] = ACTIONS(1781),
    [sym_amx_debug] = ACTIONS(1781),
    [sym_netlinx_login_success] = ACTIONS(1781),
    [sym_netlinx_login_fail] = ACTIONS(1781),
    [sym_netlinx_logout] = ACTIONS(1781),
    [sym_null_str] = ACTIONS(1781),
    [sym_smtp_address] = ACTIONS(1781),
    [sym_smtp_port_number] = ACTIONS(1781),
    [sym_smtp_username] = ACTIONS(1781),
    [sym_smtp_password] = ACTIONS(1781),
    [sym_smtp_require_tls] = ACTIONS(1781),
    [sym_smtp_from] = ACTIONS(1781),
    [sym_smtp_tls_true] = ACTIONS(1781),
    [sym_smtp_tls_false] = ACTIONS(1781),
    [sym_listview_on_row_select_event] = ACTIONS(1781),
    [sym_data_structure_unknown] = ACTIONS(1781),
    [sym_data_structure_datafeed] = ACTIONS(1781),
    [sym_data_structure_datarecordset] = ACTIONS(1781),
    [sym_data_structure_datarecord] = ACTIONS(1781),
    [sym_data_structure_datafield] = ACTIONS(1781),
    [sym_data_max_content_field_count] = ACTIONS(1781),
    [sym_data_max_metadata_field_count] = ACTIONS(1781),
    [sym_data_max_id_length] = ACTIONS(1781),
    [sym_data_max_name_length] = ACTIONS(1781),
    [sym_data_max_description_length] = ACTIONS(1781),
    [sym_data_max_type_length] = ACTIONS(1781),
    [sym_data_max_format_length] = ACTIONS(1781),
    [sym_data_max_source_length] = ACTIONS(1781),
    [sym_data_max_value_length] = ACTIONS(1781),
    [sym_data_max_label_length] = ACTIONS(1781),
    [sym_data_max_url_length] = ACTIONS(1781),
    [sym_max_last_login_info_length] = ACTIONS(1781),
    [sym_data_type_unknown] = ACTIONS(1781),
    [sym_data_type_string] = ACTIONS(1781),
    [sym_data_type_datetime] = ACTIONS(1781),
    [sym_data_type_date] = ACTIONS(1781),
    [sym_data_type_time] = ACTIONS(1781),
    [sym_data_type_image] = ACTIONS(1781),
    [sym_data_format_url] = ACTIONS(1781),
    [sym_data_format_phone] = ACTIONS(1781),
    [sym_data_format_email] = ACTIONS(1781),
    [sym_data_format_iso8601] = ACTIONS(1781),
    [sym_auditlog_priv] = ACTIONS(1781),
    [sym_configuration_priv] = ACTIONS(1781),
    [sym_deviceconfig_priv] = ACTIONS(1781),
    [sym_ftp_priv] = ACTIONS(1781),
    [sym_http_priv] = ACTIONS(1781),
    [sym_networkconfig_priv] = ACTIONS(1781),
    [sym_programport_priv] = ACTIONS(1781),
    [sym_remoteui_priv] = ACTIONS(1781),
    [sym_securitycontrol_priv] = ACTIONS(1781),
    [sym_softwaremanagement_priv] = ACTIONS(1781),
    [sym_terminal_priv] = ACTIONS(1781),
    [sym_tpadmin_priv] = ACTIONS(1781),
    [sym_user1_priv] = ACTIONS(1781),
    [sym_user2_priv] = ACTIONS(1781),
    [sym_user3_priv] = ACTIONS(1781),
    [sym_user4_priv] = ACTIONS(1781),
    [sym_usermanagement_priv] = ACTIONS(1781),
    [sym_valid_account] = ACTIONS(1781),
    [sym_err_name_invalid] = ACTIONS(1781),
    [sym_err_password_invalid] = ACTIONS(1781),
    [sym_err_invalid_account] = ACTIONS(1781),
    [sym_err_no_authorization] = ACTIONS(1781),
    [sym_err_invalid_parameter] = ACTIONS(1781),
    [sym_err_user_locked_out] = ACTIONS(1781),
    [sym_err_user_account_expired] = ACTIONS(1781),
    [sym_err_general_error] = ACTIONS(1781),
    [sym_err_authentication_error] = ACTIONS(1781),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1781),
    [sym_tls_validate_certificate] = ACTIONS(1781),
    [sym_tdata] = ACTIONS(1781),
    [sym_tchannel] = ACTIONS(1781),
    [sym_tlevel] = ACTIONS(1781),
    [sym_tbutton] = ACTIONS(1781),
    [sym_ttimeline] = ACTIONS(1781),
    [sym_tcustom] = ACTIONS(1781),
    [sym_url_struct] = ACTIONS(1781),
    [sym_dns_struct] = ACTIONS(1781),
    [sym_ip_address_struct] = ACTIONS(1781),
    [sym_dev_info_struct] = ACTIONS(1781),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1781),
    [sym_data_feed] = ACTIONS(1781),
    [sym_data_field] = ACTIONS(1781),
    [sym_data_record] = ACTIONS(1781),
    [sym_wc_data_feed] = ACTIONS(1781),
    [sym_wc_data_field] = ACTIONS(1781),
    [sym_wc_data_record] = ACTIONS(1781),
    [sym_last_login_info] = ACTIONS(1781),
    [sym____reserved___] = ACTIONS(1781),
    [sym_date] = ACTIONS(1781),
    [sym_day] = ACTIONS(1781),
    [sym_time] = ACTIONS(1781),
    [sym_ldate] = ACTIONS(1781),
    [sym_push_device] = ACTIONS(1781),
    [sym_push_channel] = ACTIONS(1781),
    [sym_push_devchan] = ACTIONS(1781),
    [sym_release_device] = ACTIONS(1781),
    [sym_release_channel] = ACTIONS(1781),
    [sym_release_devchan] = ACTIONS(1781),
    [sym_master_slot] = ACTIONS(1781),
    [sym_get_pulse_time] = ACTIONS(1781),
    [sym_get_timer] = ACTIONS(1781),
    [sym_master_sn] = ACTIONS(1781),
    [sym_system_number] = ACTIONS(1781),
    [sym_button] = ACTIONS(1781),
    [sym_dv_channel] = ACTIONS(1781),
    [sym_data] = ACTIONS(1781),
    [sym_level] = ACTIONS(1781),
    [sym_timeline] = ACTIONS(1781),
    [sym_channel] = ACTIONS(1781),
    [sym_custom] = ACTIONS(1781),
    [sym_atoi] = ACTIONS(1781),
    [sym_atol] = ACTIONS(1781),
    [sym_atof] = ACTIONS(1781),
    [sym_itoa] = ACTIONS(1781),
    [sym_itohex] = ACTIONS(1781),
    [sym_hextoi] = ACTIONS(1781),
    [sym_ftoa] = ACTIONS(1781),
    [sym_find_string] = ACTIONS(1781),
    [sym_left_string] = ACTIONS(1781),
    [sym_right_string] = ACTIONS(1781),
    [sym_mid_string] = ACTIONS(1781),
    [sym_remove_string] = ACTIONS(1781),
    [sym_get_buffer_string] = ACTIONS(1781),
    [sym_get_buffer_char] = ACTIONS(1781),
    [sym_get_multi_buffer_string] = ACTIONS(1781),
    [sym_length_string] = ACTIONS(1781),
    [sym_max_length_string] = ACTIONS(1781),
    [sym_set_length_string] = ACTIONS(1781),
    [sym_compare_string] = ACTIONS(1781),
    [sym_upper_string] = ACTIONS(1781),
    [sym_lower_string] = ACTIONS(1781),
    [sym_get_last] = ACTIONS(1781),
    [sym_date_to_day] = ACTIONS(1781),
    [sym_date_to_month] = ACTIONS(1781),
    [sym_date_to_year] = ACTIONS(1781),
    [sym_time_to_hour] = ACTIONS(1781),
    [sym_time_to_minute] = ACTIONS(1781),
    [sym_time_to_second] = ACTIONS(1781),
    [sym_day_of_week] = ACTIONS(1781),
    [sym_set_pulse_time] = ACTIONS(1781),
    [sym_set_timer] = ACTIONS(1781),
    [sym_file_open] = ACTIONS(1781),
    [sym_file_close] = ACTIONS(1781),
    [sym_file_read] = ACTIONS(1781),
    [sym_file_read_line] = ACTIONS(1781),
    [sym_file_write] = ACTIONS(1781),
    [sym_file_write_line] = ACTIONS(1781),
    [sym_file_copy] = ACTIONS(1781),
    [sym_file_delete] = ACTIONS(1781),
    [sym_file_rename] = ACTIONS(1781),
    [sym_file_dir] = ACTIONS(1781),
    [sym_file_getdir] = ACTIONS(1781),
    [sym_file_setdir] = ACTIONS(1781),
    [sym_file_create_dir] = ACTIONS(1781),
    [sym_file_remove_dir] = ACTIONS(1781),
    [sym_file_seek] = ACTIONS(1781),
    [sym_length_array] = ACTIONS(1781),
    [sym_max_length_array] = ACTIONS(1781),
    [sym_set_length_array] = ACTIONS(1781),
    [sym_ip_client_open] = ACTIONS(1781),
    [sym_ip_client_close] = ACTIONS(1781),
    [sym_ip_server_open] = ACTIONS(1781),
    [sym_ip_server_close] = ACTIONS(1781),
    [sym_ip_bound_client_open] = ACTIONS(1781),
    [sym_ip_mc_server_open] = ACTIONS(1781),
    [sym_ip_set_option] = ACTIONS(1781),
    [sym_ssh_client_open] = ACTIONS(1781),
    [sym_ssh_client_close] = ACTIONS(1781),
    [sym_tls_client_open] = ACTIONS(1781),
    [sym_tls_client_close] = ACTIONS(1781),
    [sym_do_push] = ACTIONS(1781),
    [sym_do_push_timed] = ACTIONS(1781),
    [sym_do_release] = ACTIONS(1781),
    [sym_device_id] = ACTIONS(1781),
    [sym_device_info] = ACTIONS(1781),
    [sym_device_id_string] = ACTIONS(1781),
    [sym_device_standby] = ACTIONS(1781),
    [sym_device_wake] = ACTIONS(1781),
    [sym_timeline_create] = ACTIONS(1781),
    [sym_timeline_kill] = ACTIONS(1781),
    [sym_timeline_reload] = ACTIONS(1781),
    [sym_timeline_pause] = ACTIONS(1781),
    [sym_timeline_restart] = ACTIONS(1781),
    [sym_timeline_set] = ACTIONS(1781),
    [sym_timeline_get] = ACTIONS(1781),
    [sym_timeline_active] = ACTIONS(1781),
    [sym_timeline_dynamic_id] = ACTIONS(1781),
    [sym_variable_to_string] = ACTIONS(1781),
    [sym_string_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_string] = ACTIONS(1781),
    [sym_type_cast] = ACTIONS(1781),
    [sym_variable_to_xml] = ACTIONS(1781),
    [sym_xml_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_xml] = ACTIONS(1781),
    [sym_redirect_string] = ACTIONS(1781),
    [sym_random_number] = ACTIONS(1781),
    [sym_get_unique_id] = ACTIONS(1781),
    [sym_get_system_number] = ACTIONS(1781),
    [sym_set_system_number] = ACTIONS(1781),
    [sym_get_serial_number] = ACTIONS(1781),
    [sym_get_master_build] = ACTIONS(1781),
    [sym_set_validation_code] = ACTIONS(1781),
    [sym_reboot] = ACTIONS(1781),
    [sym_load_duet_module] = ACTIONS(1781),
    [sym_unload_duet_module] = ACTIONS(1781),
    [sym_duet_mem_size_set] = ACTIONS(1781),
    [sym_duet_mem_size_get] = ACTIONS(1781),
    [sym_dynamic_application_device] = ACTIONS(1781),
    [sym_dynamic_polled_port] = ACTIONS(1781),
    [sym_static_port_binding] = ACTIONS(1781),
    [sym_static_ip_binding] = ACTIONS(1781),
    [sym_get_dns_list] = ACTIONS(1781),
    [sym_set_dns_list] = ACTIONS(1781),
    [sym_get_ip_address] = ACTIONS(1781),
    [sym_set_ip_address] = ACTIONS(1781),
    [sym_get_url_list] = ACTIONS(1781),
    [sym_add_url_entry] = ACTIONS(1781),
    [sym_delete_url_entry] = ACTIONS(1781),
    [sym_abs_value] = ACTIONS(1781),
    [sym_max_value] = ACTIONS(1781),
    [sym_min_value] = ACTIONS(1781),
    [sym_exp_value] = ACTIONS(1781),
    [sym_log_value] = ACTIONS(1781),
    [sym_log10_value] = ACTIONS(1781),
    [sym_power_value] = ACTIONS(1781),
    [sym_sqrt_value] = ACTIONS(1781),
    [sym_raw_be] = ACTIONS(1781),
    [sym_raw_le] = ACTIONS(1781),
    [sym_internal_queue_size_set] = ACTIONS(1781),
    [sym_internal_queue_size_get] = ACTIONS(1781),
    [sym_internal_threshold_set] = ACTIONS(1781),
    [sym_internal_threshold_get] = ACTIONS(1781),
    [sym_rebuilt_event] = ACTIONS(1781),
    [sym_do_custom_event] = ACTIONS(1781),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1781),
    [sym_clkmgr_set_clk_source] = ACTIONS(1781),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1781),
    [sym_clkmgr_get_timezone] = ACTIONS(1781),
    [sym_clkmgr_set_timezone] = ACTIONS(1781),
    [sym_clkmgr_get_resync_period] = ACTIONS(1781),
    [sym_clkmgr_set_resync_period] = ACTIONS(1781),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_timeservers] = ACTIONS(1781),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_set_log_level] = ACTIONS(1781),
    [sym_get_log_level] = ACTIONS(1781),
    [sym_amx_log] = ACTIONS(1781),
    [sym_smtp_server_config_set] = ACTIONS(1781),
    [sym_smtp_server_config_get] = ACTIONS(1781),
    [sym_smtp_send] = ACTIONS(1781),
    [sym_validate_netlinx_account] = ACTIONS(1781),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1781),
    [sym_authenticate_certificate] = ACTIONS(1781),
    [sym_audit_netlinx_session_event] = ACTIONS(1781),
    [sym_audit_netlinx_generic_event] = ACTIONS(1781),
    [sym_get_available_flash_disk_space] = ACTIONS(1781),
    [sym_get_max_flash_disk_space] = ACTIONS(1781),
    [sym_data_create_feed] = ACTIONS(1781),
    [sym_data_delete_feed] = ACTIONS(1781),
    [sym_data_publish_feed] = ACTIONS(1781),
    [sym_data_get_published_feed] = ACTIONS(1781),
    [sym_data_add_record] = ACTIONS(1781),
    [sym_data_get_event_record] = ACTIONS(1781),
    [sym__wc_data_create_feed] = ACTIONS(1781),
    [sym__wc_data_add_record] = ACTIONS(1781),
    [sym__wc_data_get_event_record] = ACTIONS(1781),
    [sym_format] = ACTIONS(1781),
    [sym_astro_clock] = ACTIONS(1781),
    [sym_set_outdoor_temperature] = ACTIONS(1781),
    [sym_set_virtual_level_count] = ACTIONS(1781),
    [sym_set_virtual_channel_count] = ACTIONS(1781),
    [sym_set_virtual_port_count] = ACTIONS(1781),
    [sym__wc_encode_int] = ACTIONS(1781),
    [sym__wc_decode_int] = ACTIONS(1781),
    [sym__wc_int] = ACTIONS(1781),
    [sym__wc_to_ch_int] = ACTIONS(1781),
    [sym__ch_to_wc_int] = ACTIONS(1781),
    [sym__wc_find_string_int] = ACTIONS(1781),
    [sym__wc_left_string_int] = ACTIONS(1781),
    [sym__wc_lower_string_int] = ACTIONS(1781),
    [sym__wc_mid_string_int] = ACTIONS(1781),
    [sym__wc_remove_string_int] = ACTIONS(1781),
    [sym__wc_right_string_int] = ACTIONS(1781),
    [sym__wc_upper_string_int] = ACTIONS(1781),
    [sym__wc_compare_string_int] = ACTIONS(1781),
    [sym__wc_get_buffer_char_int] = ACTIONS(1781),
    [sym__wc_get_buffer_string_int] = ACTIONS(1781),
    [sym__wc_concat_string_int] = ACTIONS(1781),
    [sym__wc_file_open_int] = ACTIONS(1781),
    [sym__wc_file_close_int] = ACTIONS(1781),
    [sym__wc_file_read_int] = ACTIONS(1781),
    [sym__wc_file_read_line_int] = ACTIONS(1781),
    [sym__wc_file_write_int] = ACTIONS(1781),
    [sym__wc_file_write_line_int] = ACTIONS(1781),
    [aux_sym_preproc_include_token1] = ACTIONS(1781),
    [aux_sym_preproc_define_token1] = ACTIONS(1783),
    [aux_sym_preproc_warn_token1] = ACTIONS(1783),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1783),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_else_token1] = ACTIONS(1783),
    [anon_sym_LBRACK] = ACTIONS(1783),
    [anon_sym_LBRACE] = ACTIONS(1783),
    [anon_sym_LPAREN] = ACTIONS(1781),
    [anon_sym_BANG] = ACTIONS(1783),
    [anon_sym_TILDE] = ACTIONS(1783),
    [anon_sym_DASH] = ACTIONS(1781),
    [anon_sym_PLUS] = ACTIONS(1781),
    [anon_sym_DASH_DASH] = ACTIONS(1783),
    [anon_sym_PLUS_PLUS] = ACTIONS(1783),
    [anon_sym_DQUOTE] = ACTIONS(1783),
    [anon_sym_SQUOTE] = ACTIONS(1783),
    [sym_number_literal] = ACTIONS(1783),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(236)] = {
    [sym_identifier] = ACTIONS(1785),
    [sym_program_name_keyword] = ACTIONS(1785),
    [sym_module_name_keyword] = ACTIONS(1785),
    [sym_define_device_keyword] = ACTIONS(1785),
    [sym_define_combine_keyword] = ACTIONS(1785),
    [sym_define_constant_keyword] = ACTIONS(1785),
    [sym_define_type_keyword] = ACTIONS(1785),
    [sym_define_variable_keyword] = ACTIONS(1785),
    [sym_define_system_variable_keyword] = ACTIONS(1785),
    [sym_define_connect_level_keyword] = ACTIONS(1785),
    [sym_define_latching_keyword] = ACTIONS(1785),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1785),
    [sym_define_toggling_keyword] = ACTIONS(1785),
    [sym_define_call_keyword] = ACTIONS(1785),
    [sym_define_function_keyword] = ACTIONS(1785),
    [sym_define_library_function_keyword] = ACTIONS(1785),
    [sym_define_start_keyword] = ACTIONS(1785),
    [sym_define_event_keyword] = ACTIONS(1785),
    [sym_define_program_keyword] = ACTIONS(1785),
    [sym_define_module_keyword] = ACTIONS(1785),
    [sym_char_keyword] = ACTIONS(1785),
    [sym_widechar_keyword] = ACTIONS(1785),
    [sym_integer_keyword] = ACTIONS(1785),
    [sym_sinteger_keyword] = ACTIONS(1785),
    [sym_long_keyword] = ACTIONS(1785),
    [sym_slong_keyword] = ACTIONS(1785),
    [sym_float_keyword] = ACTIONS(1785),
    [sym_double_keyword] = ACTIONS(1785),
    [sym_variant_keyword] = ACTIONS(1785),
    [sym_variantarray_keyword] = ACTIONS(1785),
    [sym_local_var_keyword] = ACTIONS(1785),
    [sym_stack_var_keyword] = ACTIONS(1785),
    [sym_constant_keyword] = ACTIONS(1785),
    [sym_volatile_keyword] = ACTIONS(1785),
    [sym_non_volatile_keyword] = ACTIONS(1785),
    [sym_persistent_keyword] = ACTIONS(1785),
    [sym_dev_keyword] = ACTIONS(1785),
    [sym_devlev_keyword] = ACTIONS(1785),
    [sym_devchan_keyword] = ACTIONS(1785),
    [sym_if_keyword] = ACTIONS(1785),
    [sym_else_keyword] = ACTIONS(1785),
    [sym_switch_keyword] = ACTIONS(1785),
    [sym_case_keyword] = ACTIONS(1785),
    [sym_default_keyword] = ACTIONS(1785),
    [sym_while_keyword] = ACTIONS(1785),
    [sym_for_keyword] = ACTIONS(1785),
    [sym_break_keyword] = ACTIONS(1785),
    [sym_continue_keyword] = ACTIONS(1785),
    [sym_return_keyword] = ACTIONS(1785),
    [sym_select_keyword] = ACTIONS(1785),
    [sym_active_keyword] = ACTIONS(1785),
    [sym_struct_keyword] = ACTIONS(1785),
    [sym_structure_keyword] = ACTIONS(1785),
    [sym_button_event_keyword] = ACTIONS(1785),
    [sym_channel_event_keyword] = ACTIONS(1785),
    [sym_level_event_keyword] = ACTIONS(1785),
    [sym_data_event_keyword] = ACTIONS(1785),
    [sym_timeline_event_keyword] = ACTIONS(1785),
    [sym_custom_event_keyword] = ACTIONS(1785),
    [sym_push_keyword] = ACTIONS(1785),
    [sym_release_keyword] = ACTIONS(1785),
    [aux_sym_on_keyword_token1] = ACTIONS(1785),
    [aux_sym_off_keyword_token1] = ACTIONS(1785),
    [sym_send_level_keyword] = ACTIONS(1785),
    [sym_send_string_keyword] = ACTIONS(1785),
    [sym_send_command_keyword] = ACTIONS(1785),
    [sym_clear_buffer_keyword] = ACTIONS(1785),
    [sym_create_buffer_keyword] = ACTIONS(1785),
    [sym_create_multi_buffer_keyword] = ACTIONS(1785),
    [sym_call_keyword] = ACTIONS(1785),
    [sym_system_call_keyword] = ACTIONS(1785),
    [sym_devchan_to_keyword] = ACTIONS(1785),
    [sym_devchan_min_to_keyword] = ACTIONS(1785),
    [sym_devchan_total_off_keyword] = ACTIONS(1785),
    [sym_devchan_pulse_keyword] = ACTIONS(1785),
    [sym_wait_keyword] = ACTIONS(1785),
    [sym_cancel_wait_keyword] = ACTIONS(1785),
    [sym_wait_until_keyword] = ACTIONS(1785),
    [sym_cancel_wait_until_keyword] = ACTIONS(1785),
    [sym_cancel_all_wait_keyword] = ACTIONS(1785),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1785),
    [sym_not] = ACTIONS(1785),
    [sym_bnot] = ACTIONS(1785),
    [sym___file__] = ACTIONS(1785),
    [sym___date__] = ACTIONS(1785),
    [sym___ldate__] = ACTIONS(1785),
    [sym___time__] = ACTIONS(1785),
    [sym___line__] = ACTIONS(1785),
    [sym___name__] = ACTIONS(1785),
    [sym___netlinx__] = ACTIONS(1785),
    [sym_netlinx_axi_version] = ACTIONS(1785),
    [sym_url_flg_tcp] = ACTIONS(1785),
    [sym_url_flg_accinfopresent] = ACTIONS(1785),
    [sym_url_flg_temp] = ACTIONS(1785),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1785),
    [sym_url_flg_stat_mask] = ACTIONS(1785),
    [sym_url_flg_stat_lookup] = ACTIONS(1785),
    [sym_url_flg_stat_connecting] = ACTIONS(1785),
    [sym_url_flg_stat_waiting] = ACTIONS(1785),
    [sym_url_flg_stat_connected] = ACTIONS(1785),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1785),
    [sym_first_virtual_device] = ACTIONS(1785),
    [sym_first_local_port] = ACTIONS(1785),
    [sym_dynamic_virtual_device] = ACTIONS(1785),
    [sym_timeline_once] = ACTIONS(1785),
    [sym_timeline_repeat] = ACTIONS(1785),
    [sym_timeline_absolute] = ACTIONS(1785),
    [sym_timeline_relative] = ACTIONS(1785),
    [sym_do_push_timed_infinite] = ACTIONS(1785),
    [sym_source_type_no_address] = ACTIONS(1785),
    [sym_source_type_neuron_id] = ACTIONS(1785),
    [sym_source_type_ip_address] = ACTIONS(1785),
    [sym_source_type_axlink] = ACTIONS(1785),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1785),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1785),
    [sym_source_type_ip_socket_address] = ACTIONS(1785),
    [sym_source_type_rs232] = ACTIONS(1785),
    [sym_source_type_internal] = ACTIONS(1785),
    [sym_source_type_mac_address] = ACTIONS(1785),
    [sym_source_type_ipv4_port] = ACTIONS(1785),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1785),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1785),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1785),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1785),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1785),
    [sym_true] = ACTIONS(1785),
    [sym_false] = ACTIONS(1785),
    [sym_file_read_only] = ACTIONS(1785),
    [sym_file_rw_new] = ACTIONS(1785),
    [sym_file_rw_append] = ACTIONS(1785),
    [sym_ip_tcp] = ACTIONS(1785),
    [sym_ip_udp] = ACTIONS(1785),
    [sym_ip_udp_2way] = ACTIONS(1785),
    [sym_xml_encode_types] = ACTIONS(1785),
    [sym_xml_encode_char_as_list] = ACTIONS(1785),
    [sym_xml_encode_le] = ACTIONS(1785),
    [sym_xml_decode_types] = ACTIONS(1785),
    [sym_xml_decode_no_preserve] = ACTIONS(1785),
    [sym_duet_dev_type_display_device] = ACTIONS(1785),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1785),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1785),
    [sym_duet_dev_type_io_device] = ACTIONS(1785),
    [sym_duet_dev_type_relay_device] = ACTIONS(1785),
    [sym_duet_dev_type_ups] = ACTIONS(1785),
    [sym_duet_dev_type_amplifier] = ACTIONS(1785),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1785),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1785),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1785),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1785),
    [sym_duet_dev_type_camera] = ACTIONS(1785),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1785),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1785),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1785),
    [sym_duet_dev_type_dss] = ACTIONS(1785),
    [sym_duet_dev_type_dvr] = ACTIONS(1785),
    [sym_duet_dev_type_disc_device] = ACTIONS(1785),
    [sym_duet_dev_type_document_camera] = ACTIONS(1785),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1785),
    [sym_duet_dev_type_hvac] = ACTIONS(1785),
    [sym_duet_dev_type_keypad] = ACTIONS(1785),
    [sym_duet_dev_type_light] = ACTIONS(1785),
    [sym_duet_dev_type_monitor] = ACTIONS(1785),
    [sym_duet_dev_type_motor] = ACTIONS(1785),
    [sym_duet_dev_type_multi_window] = ACTIONS(1785),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1785),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1785),
    [sym_duet_dev_type_receiver] = ACTIONS(1785),
    [sym_duet_dev_type_security_system] = ACTIONS(1785),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1785),
    [sym_duet_dev_type_settop_box] = ACTIONS(1785),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1785),
    [sym_duet_dev_type_switcher] = ACTIONS(1785),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1785),
    [sym_duet_dev_type_tv] = ACTIONS(1785),
    [sym_duet_dev_type_utility] = ACTIONS(1785),
    [sym_duet_dev_type_vcr] = ACTIONS(1785),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1785),
    [sym_duet_dev_type_video_processor] = ACTIONS(1785),
    [sym_duet_dev_type_video_projector] = ACTIONS(1785),
    [sym_duet_dev_type_video_wall] = ACTIONS(1785),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1785),
    [sym_duet_dev_type_weather] = ACTIONS(1785),
    [sym_duet_dev_not_polled] = ACTIONS(1785),
    [sym_duet_dev_polled] = ACTIONS(1785),
    [sym_clkmgr_mode_network] = ACTIONS(1785),
    [sym_clkmgr_mode_standalone] = ACTIONS(1785),
    [sym_normal_standby] = ACTIONS(1785),
    [sym_normal_wake] = ACTIONS(1785),
    [sym_ip_multicast_ttl_option] = ACTIONS(1785),
    [sym_ip_ttl_subnet] = ACTIONS(1785),
    [sym_ip_ttl_site] = ACTIONS(1785),
    [sym_ip_ttl_region] = ACTIONS(1785),
    [sym_ip_ttl_continent] = ACTIONS(1785),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1785),
    [sym_ip_nodelay_on] = ACTIONS(1785),
    [sym_ip_nodelay_off] = ACTIONS(1785),
    [sym_amx_error] = ACTIONS(1785),
    [sym_amx_warning] = ACTIONS(1785),
    [sym_amx_info] = ACTIONS(1785),
    [sym_amx_debug] = ACTIONS(1785),
    [sym_netlinx_login_success] = ACTIONS(1785),
    [sym_netlinx_login_fail] = ACTIONS(1785),
    [sym_netlinx_logout] = ACTIONS(1785),
    [sym_null_str] = ACTIONS(1785),
    [sym_smtp_address] = ACTIONS(1785),
    [sym_smtp_port_number] = ACTIONS(1785),
    [sym_smtp_username] = ACTIONS(1785),
    [sym_smtp_password] = ACTIONS(1785),
    [sym_smtp_require_tls] = ACTIONS(1785),
    [sym_smtp_from] = ACTIONS(1785),
    [sym_smtp_tls_true] = ACTIONS(1785),
    [sym_smtp_tls_false] = ACTIONS(1785),
    [sym_listview_on_row_select_event] = ACTIONS(1785),
    [sym_data_structure_unknown] = ACTIONS(1785),
    [sym_data_structure_datafeed] = ACTIONS(1785),
    [sym_data_structure_datarecordset] = ACTIONS(1785),
    [sym_data_structure_datarecord] = ACTIONS(1785),
    [sym_data_structure_datafield] = ACTIONS(1785),
    [sym_data_max_content_field_count] = ACTIONS(1785),
    [sym_data_max_metadata_field_count] = ACTIONS(1785),
    [sym_data_max_id_length] = ACTIONS(1785),
    [sym_data_max_name_length] = ACTIONS(1785),
    [sym_data_max_description_length] = ACTIONS(1785),
    [sym_data_max_type_length] = ACTIONS(1785),
    [sym_data_max_format_length] = ACTIONS(1785),
    [sym_data_max_source_length] = ACTIONS(1785),
    [sym_data_max_value_length] = ACTIONS(1785),
    [sym_data_max_label_length] = ACTIONS(1785),
    [sym_data_max_url_length] = ACTIONS(1785),
    [sym_max_last_login_info_length] = ACTIONS(1785),
    [sym_data_type_unknown] = ACTIONS(1785),
    [sym_data_type_string] = ACTIONS(1785),
    [sym_data_type_datetime] = ACTIONS(1785),
    [sym_data_type_date] = ACTIONS(1785),
    [sym_data_type_time] = ACTIONS(1785),
    [sym_data_type_image] = ACTIONS(1785),
    [sym_data_format_url] = ACTIONS(1785),
    [sym_data_format_phone] = ACTIONS(1785),
    [sym_data_format_email] = ACTIONS(1785),
    [sym_data_format_iso8601] = ACTIONS(1785),
    [sym_auditlog_priv] = ACTIONS(1785),
    [sym_configuration_priv] = ACTIONS(1785),
    [sym_deviceconfig_priv] = ACTIONS(1785),
    [sym_ftp_priv] = ACTIONS(1785),
    [sym_http_priv] = ACTIONS(1785),
    [sym_networkconfig_priv] = ACTIONS(1785),
    [sym_programport_priv] = ACTIONS(1785),
    [sym_remoteui_priv] = ACTIONS(1785),
    [sym_securitycontrol_priv] = ACTIONS(1785),
    [sym_softwaremanagement_priv] = ACTIONS(1785),
    [sym_terminal_priv] = ACTIONS(1785),
    [sym_tpadmin_priv] = ACTIONS(1785),
    [sym_user1_priv] = ACTIONS(1785),
    [sym_user2_priv] = ACTIONS(1785),
    [sym_user3_priv] = ACTIONS(1785),
    [sym_user4_priv] = ACTIONS(1785),
    [sym_usermanagement_priv] = ACTIONS(1785),
    [sym_valid_account] = ACTIONS(1785),
    [sym_err_name_invalid] = ACTIONS(1785),
    [sym_err_password_invalid] = ACTIONS(1785),
    [sym_err_invalid_account] = ACTIONS(1785),
    [sym_err_no_authorization] = ACTIONS(1785),
    [sym_err_invalid_parameter] = ACTIONS(1785),
    [sym_err_user_locked_out] = ACTIONS(1785),
    [sym_err_user_account_expired] = ACTIONS(1785),
    [sym_err_general_error] = ACTIONS(1785),
    [sym_err_authentication_error] = ACTIONS(1785),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1785),
    [sym_tls_validate_certificate] = ACTIONS(1785),
    [sym_tdata] = ACTIONS(1785),
    [sym_tchannel] = ACTIONS(1785),
    [sym_tlevel] = ACTIONS(1785),
    [sym_tbutton] = ACTIONS(1785),
    [sym_ttimeline] = ACTIONS(1785),
    [sym_tcustom] = ACTIONS(1785),
    [sym_url_struct] = ACTIONS(1785),
    [sym_dns_struct] = ACTIONS(1785),
    [sym_ip_address_struct] = ACTIONS(1785),
    [sym_dev_info_struct] = ACTIONS(1785),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1785),
    [sym_data_feed] = ACTIONS(1785),
    [sym_data_field] = ACTIONS(1785),
    [sym_data_record] = ACTIONS(1785),
    [sym_wc_data_feed] = ACTIONS(1785),
    [sym_wc_data_field] = ACTIONS(1785),
    [sym_wc_data_record] = ACTIONS(1785),
    [sym_last_login_info] = ACTIONS(1785),
    [sym____reserved___] = ACTIONS(1785),
    [sym_date] = ACTIONS(1785),
    [sym_day] = ACTIONS(1785),
    [sym_time] = ACTIONS(1785),
    [sym_ldate] = ACTIONS(1785),
    [sym_push_device] = ACTIONS(1785),
    [sym_push_channel] = ACTIONS(1785),
    [sym_push_devchan] = ACTIONS(1785),
    [sym_release_device] = ACTIONS(1785),
    [sym_release_channel] = ACTIONS(1785),
    [sym_release_devchan] = ACTIONS(1785),
    [sym_master_slot] = ACTIONS(1785),
    [sym_get_pulse_time] = ACTIONS(1785),
    [sym_get_timer] = ACTIONS(1785),
    [sym_master_sn] = ACTIONS(1785),
    [sym_system_number] = ACTIONS(1785),
    [sym_button] = ACTIONS(1785),
    [sym_dv_channel] = ACTIONS(1785),
    [sym_data] = ACTIONS(1785),
    [sym_level] = ACTIONS(1785),
    [sym_timeline] = ACTIONS(1785),
    [sym_channel] = ACTIONS(1785),
    [sym_custom] = ACTIONS(1785),
    [sym_atoi] = ACTIONS(1785),
    [sym_atol] = ACTIONS(1785),
    [sym_atof] = ACTIONS(1785),
    [sym_itoa] = ACTIONS(1785),
    [sym_itohex] = ACTIONS(1785),
    [sym_hextoi] = ACTIONS(1785),
    [sym_ftoa] = ACTIONS(1785),
    [sym_find_string] = ACTIONS(1785),
    [sym_left_string] = ACTIONS(1785),
    [sym_right_string] = ACTIONS(1785),
    [sym_mid_string] = ACTIONS(1785),
    [sym_remove_string] = ACTIONS(1785),
    [sym_get_buffer_string] = ACTIONS(1785),
    [sym_get_buffer_char] = ACTIONS(1785),
    [sym_get_multi_buffer_string] = ACTIONS(1785),
    [sym_length_string] = ACTIONS(1785),
    [sym_max_length_string] = ACTIONS(1785),
    [sym_set_length_string] = ACTIONS(1785),
    [sym_compare_string] = ACTIONS(1785),
    [sym_upper_string] = ACTIONS(1785),
    [sym_lower_string] = ACTIONS(1785),
    [sym_get_last] = ACTIONS(1785),
    [sym_date_to_day] = ACTIONS(1785),
    [sym_date_to_month] = ACTIONS(1785),
    [sym_date_to_year] = ACTIONS(1785),
    [sym_time_to_hour] = ACTIONS(1785),
    [sym_time_to_minute] = ACTIONS(1785),
    [sym_time_to_second] = ACTIONS(1785),
    [sym_day_of_week] = ACTIONS(1785),
    [sym_set_pulse_time] = ACTIONS(1785),
    [sym_set_timer] = ACTIONS(1785),
    [sym_file_open] = ACTIONS(1785),
    [sym_file_close] = ACTIONS(1785),
    [sym_file_read] = ACTIONS(1785),
    [sym_file_read_line] = ACTIONS(1785),
    [sym_file_write] = ACTIONS(1785),
    [sym_file_write_line] = ACTIONS(1785),
    [sym_file_copy] = ACTIONS(1785),
    [sym_file_delete] = ACTIONS(1785),
    [sym_file_rename] = ACTIONS(1785),
    [sym_file_dir] = ACTIONS(1785),
    [sym_file_getdir] = ACTIONS(1785),
    [sym_file_setdir] = ACTIONS(1785),
    [sym_file_create_dir] = ACTIONS(1785),
    [sym_file_remove_dir] = ACTIONS(1785),
    [sym_file_seek] = ACTIONS(1785),
    [sym_length_array] = ACTIONS(1785),
    [sym_max_length_array] = ACTIONS(1785),
    [sym_set_length_array] = ACTIONS(1785),
    [sym_ip_client_open] = ACTIONS(1785),
    [sym_ip_client_close] = ACTIONS(1785),
    [sym_ip_server_open] = ACTIONS(1785),
    [sym_ip_server_close] = ACTIONS(1785),
    [sym_ip_bound_client_open] = ACTIONS(1785),
    [sym_ip_mc_server_open] = ACTIONS(1785),
    [sym_ip_set_option] = ACTIONS(1785),
    [sym_ssh_client_open] = ACTIONS(1785),
    [sym_ssh_client_close] = ACTIONS(1785),
    [sym_tls_client_open] = ACTIONS(1785),
    [sym_tls_client_close] = ACTIONS(1785),
    [sym_do_push] = ACTIONS(1785),
    [sym_do_push_timed] = ACTIONS(1785),
    [sym_do_release] = ACTIONS(1785),
    [sym_device_id] = ACTIONS(1785),
    [sym_device_info] = ACTIONS(1785),
    [sym_device_id_string] = ACTIONS(1785),
    [sym_device_standby] = ACTIONS(1785),
    [sym_device_wake] = ACTIONS(1785),
    [sym_timeline_create] = ACTIONS(1785),
    [sym_timeline_kill] = ACTIONS(1785),
    [sym_timeline_reload] = ACTIONS(1785),
    [sym_timeline_pause] = ACTIONS(1785),
    [sym_timeline_restart] = ACTIONS(1785),
    [sym_timeline_set] = ACTIONS(1785),
    [sym_timeline_get] = ACTIONS(1785),
    [sym_timeline_active] = ACTIONS(1785),
    [sym_timeline_dynamic_id] = ACTIONS(1785),
    [sym_variable_to_string] = ACTIONS(1785),
    [sym_string_to_variable] = ACTIONS(1785),
    [sym_length_variable_to_string] = ACTIONS(1785),
    [sym_type_cast] = ACTIONS(1785),
    [sym_variable_to_xml] = ACTIONS(1785),
    [sym_xml_to_variable] = ACTIONS(1785),
    [sym_length_variable_to_xml] = ACTIONS(1785),
    [sym_redirect_string] = ACTIONS(1785),
    [sym_random_number] = ACTIONS(1785),
    [sym_get_unique_id] = ACTIONS(1785),
    [sym_get_system_number] = ACTIONS(1785),
    [sym_set_system_number] = ACTIONS(1785),
    [sym_get_serial_number] = ACTIONS(1785),
    [sym_get_master_build] = ACTIONS(1785),
    [sym_set_validation_code] = ACTIONS(1785),
    [sym_reboot] = ACTIONS(1785),
    [sym_load_duet_module] = ACTIONS(1785),
    [sym_unload_duet_module] = ACTIONS(1785),
    [sym_duet_mem_size_set] = ACTIONS(1785),
    [sym_duet_mem_size_get] = ACTIONS(1785),
    [sym_dynamic_application_device] = ACTIONS(1785),
    [sym_dynamic_polled_port] = ACTIONS(1785),
    [sym_static_port_binding] = ACTIONS(1785),
    [sym_static_ip_binding] = ACTIONS(1785),
    [sym_get_dns_list] = ACTIONS(1785),
    [sym_set_dns_list] = ACTIONS(1785),
    [sym_get_ip_address] = ACTIONS(1785),
    [sym_set_ip_address] = ACTIONS(1785),
    [sym_get_url_list] = ACTIONS(1785),
    [sym_add_url_entry] = ACTIONS(1785),
    [sym_delete_url_entry] = ACTIONS(1785),
    [sym_abs_value] = ACTIONS(1785),
    [sym_max_value] = ACTIONS(1785),
    [sym_min_value] = ACTIONS(1785),
    [sym_exp_value] = ACTIONS(1785),
    [sym_log_value] = ACTIONS(1785),
    [sym_log10_value] = ACTIONS(1785),
    [sym_power_value] = ACTIONS(1785),
    [sym_sqrt_value] = ACTIONS(1785),
    [sym_raw_be] = ACTIONS(1785),
    [sym_raw_le] = ACTIONS(1785),
    [sym_internal_queue_size_set] = ACTIONS(1785),
    [sym_internal_queue_size_get] = ACTIONS(1785),
    [sym_internal_threshold_set] = ACTIONS(1785),
    [sym_internal_threshold_get] = ACTIONS(1785),
    [sym_rebuilt_event] = ACTIONS(1785),
    [sym_do_custom_event] = ACTIONS(1785),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1785),
    [sym_clkmgr_set_clk_source] = ACTIONS(1785),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1785),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1785),
    [sym_clkmgr_get_timezone] = ACTIONS(1785),
    [sym_clkmgr_set_timezone] = ACTIONS(1785),
    [sym_clkmgr_get_resync_period] = ACTIONS(1785),
    [sym_clkmgr_set_resync_period] = ACTIONS(1785),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1785),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1785),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1785),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1785),
    [sym_clkmgr_get_timeservers] = ACTIONS(1785),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1785),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1785),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1785),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1785),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1785),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1785),
    [sym_set_log_level] = ACTIONS(1785),
    [sym_get_log_level] = ACTIONS(1785),
    [sym_amx_log] = ACTIONS(1785),
    [sym_smtp_server_config_set] = ACTIONS(1785),
    [sym_smtp_server_config_get] = ACTIONS(1785),
    [sym_smtp_send] = ACTIONS(1785),
    [sym_validate_netlinx_account] = ACTIONS(1785),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1785),
    [sym_authenticate_certificate] = ACTIONS(1785),
    [sym_audit_netlinx_session_event] = ACTIONS(1785),
    [sym_audit_netlinx_generic_event] = ACTIONS(1785),
    [sym_get_available_flash_disk_space] = ACTIONS(1785),
    [sym_get_max_flash_disk_space] = ACTIONS(1785),
    [sym_data_create_feed] = ACTIONS(1785),
    [sym_data_delete_feed] = ACTIONS(1785),
    [sym_data_publish_feed] = ACTIONS(1785),
    [sym_data_get_published_feed] = ACTIONS(1785),
    [sym_data_add_record] = ACTIONS(1785),
    [sym_data_get_event_record] = ACTIONS(1785),
    [sym__wc_data_create_feed] = ACTIONS(1785),
    [sym__wc_data_add_record] = ACTIONS(1785),
    [sym__wc_data_get_event_record] = ACTIONS(1785),
    [sym_format] = ACTIONS(1785),
    [sym_astro_clock] = ACTIONS(1785),
    [sym_set_outdoor_temperature] = ACTIONS(1785),
    [sym_set_virtual_level_count] = ACTIONS(1785),
    [sym_set_virtual_channel_count] = ACTIONS(1785),
    [sym_set_virtual_port_count] = ACTIONS(1785),
    [sym__wc_encode_int] = ACTIONS(1785),
    [sym__wc_decode_int] = ACTIONS(1785),
    [sym__wc_int] = ACTIONS(1785),
    [sym__wc_to_ch_int] = ACTIONS(1785),
    [sym__ch_to_wc_int] = ACTIONS(1785),
    [sym__wc_find_string_int] = ACTIONS(1785),
    [sym__wc_left_string_int] = ACTIONS(1785),
    [sym__wc_lower_string_int] = ACTIONS(1785),
    [sym__wc_mid_string_int] = ACTIONS(1785),
    [sym__wc_remove_string_int] = ACTIONS(1785),
    [sym__wc_right_string_int] = ACTIONS(1785),
    [sym__wc_upper_string_int] = ACTIONS(1785),
    [sym__wc_compare_string_int] = ACTIONS(1785),
    [sym__wc_get_buffer_char_int] = ACTIONS(1785),
    [sym__wc_get_buffer_string_int] = ACTIONS(1785),
    [sym__wc_concat_string_int] = ACTIONS(1785),
    [sym__wc_file_open_int] = ACTIONS(1785),
    [sym__wc_file_close_int] = ACTIONS(1785),
    [sym__wc_file_read_int] = ACTIONS(1785),
    [sym__wc_file_read_line_int] = ACTIONS(1785),
    [sym__wc_file_write_int] = ACTIONS(1785),
    [sym__wc_file_write_line_int] = ACTIONS(1785),
    [aux_sym_preproc_include_token1] = ACTIONS(1785),
    [aux_sym_preproc_define_token1] = ACTIONS(1787),
    [aux_sym_preproc_warn_token1] = ACTIONS(1787),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1787),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1787),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1787),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1787),
    [aux_sym_preproc_else_token1] = ACTIONS(1787),
    [anon_sym_LBRACK] = ACTIONS(1787),
    [anon_sym_LBRACE] = ACTIONS(1787),
    [anon_sym_LPAREN] = ACTIONS(1785),
    [anon_sym_BANG] = ACTIONS(1787),
    [anon_sym_TILDE] = ACTIONS(1787),
    [anon_sym_DASH] = ACTIONS(1785),
    [anon_sym_PLUS] = ACTIONS(1785),
    [anon_sym_DASH_DASH] = ACTIONS(1787),
    [anon_sym_PLUS_PLUS] = ACTIONS(1787),
    [anon_sym_DQUOTE] = ACTIONS(1787),
    [anon_sym_SQUOTE] = ACTIONS(1787),
    [sym_number_literal] = ACTIONS(1787),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(237)] = {
    [sym_identifier] = ACTIONS(1781),
    [sym_program_name_keyword] = ACTIONS(1781),
    [sym_module_name_keyword] = ACTIONS(1781),
    [sym_define_device_keyword] = ACTIONS(1781),
    [sym_define_combine_keyword] = ACTIONS(1781),
    [sym_define_constant_keyword] = ACTIONS(1781),
    [sym_define_type_keyword] = ACTIONS(1781),
    [sym_define_variable_keyword] = ACTIONS(1781),
    [sym_define_system_variable_keyword] = ACTIONS(1781),
    [sym_define_connect_level_keyword] = ACTIONS(1781),
    [sym_define_latching_keyword] = ACTIONS(1781),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1781),
    [sym_define_toggling_keyword] = ACTIONS(1781),
    [sym_define_call_keyword] = ACTIONS(1781),
    [sym_define_function_keyword] = ACTIONS(1781),
    [sym_define_library_function_keyword] = ACTIONS(1781),
    [sym_define_start_keyword] = ACTIONS(1781),
    [sym_define_event_keyword] = ACTIONS(1781),
    [sym_define_program_keyword] = ACTIONS(1781),
    [sym_define_module_keyword] = ACTIONS(1781),
    [sym_char_keyword] = ACTIONS(1781),
    [sym_widechar_keyword] = ACTIONS(1781),
    [sym_integer_keyword] = ACTIONS(1781),
    [sym_sinteger_keyword] = ACTIONS(1781),
    [sym_long_keyword] = ACTIONS(1781),
    [sym_slong_keyword] = ACTIONS(1781),
    [sym_float_keyword] = ACTIONS(1781),
    [sym_double_keyword] = ACTIONS(1781),
    [sym_variant_keyword] = ACTIONS(1781),
    [sym_variantarray_keyword] = ACTIONS(1781),
    [sym_local_var_keyword] = ACTIONS(1781),
    [sym_stack_var_keyword] = ACTIONS(1781),
    [sym_constant_keyword] = ACTIONS(1781),
    [sym_volatile_keyword] = ACTIONS(1781),
    [sym_non_volatile_keyword] = ACTIONS(1781),
    [sym_persistent_keyword] = ACTIONS(1781),
    [sym_dev_keyword] = ACTIONS(1781),
    [sym_devlev_keyword] = ACTIONS(1781),
    [sym_devchan_keyword] = ACTIONS(1781),
    [sym_if_keyword] = ACTIONS(1781),
    [sym_else_keyword] = ACTIONS(1781),
    [sym_switch_keyword] = ACTIONS(1781),
    [sym_case_keyword] = ACTIONS(1781),
    [sym_default_keyword] = ACTIONS(1781),
    [sym_while_keyword] = ACTIONS(1781),
    [sym_for_keyword] = ACTIONS(1781),
    [sym_break_keyword] = ACTIONS(1781),
    [sym_continue_keyword] = ACTIONS(1781),
    [sym_return_keyword] = ACTIONS(1781),
    [sym_select_keyword] = ACTIONS(1781),
    [sym_active_keyword] = ACTIONS(1781),
    [sym_struct_keyword] = ACTIONS(1781),
    [sym_structure_keyword] = ACTIONS(1781),
    [sym_button_event_keyword] = ACTIONS(1781),
    [sym_channel_event_keyword] = ACTIONS(1781),
    [sym_level_event_keyword] = ACTIONS(1781),
    [sym_data_event_keyword] = ACTIONS(1781),
    [sym_timeline_event_keyword] = ACTIONS(1781),
    [sym_custom_event_keyword] = ACTIONS(1781),
    [sym_push_keyword] = ACTIONS(1781),
    [sym_release_keyword] = ACTIONS(1781),
    [aux_sym_on_keyword_token1] = ACTIONS(1781),
    [aux_sym_off_keyword_token1] = ACTIONS(1781),
    [sym_send_level_keyword] = ACTIONS(1781),
    [sym_send_string_keyword] = ACTIONS(1781),
    [sym_send_command_keyword] = ACTIONS(1781),
    [sym_clear_buffer_keyword] = ACTIONS(1781),
    [sym_create_buffer_keyword] = ACTIONS(1781),
    [sym_create_multi_buffer_keyword] = ACTIONS(1781),
    [sym_call_keyword] = ACTIONS(1781),
    [sym_system_call_keyword] = ACTIONS(1781),
    [sym_devchan_to_keyword] = ACTIONS(1781),
    [sym_devchan_min_to_keyword] = ACTIONS(1781),
    [sym_devchan_total_off_keyword] = ACTIONS(1781),
    [sym_devchan_pulse_keyword] = ACTIONS(1781),
    [sym_wait_keyword] = ACTIONS(1781),
    [sym_cancel_wait_keyword] = ACTIONS(1781),
    [sym_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1781),
    [sym_not] = ACTIONS(1781),
    [sym_bnot] = ACTIONS(1781),
    [sym___file__] = ACTIONS(1781),
    [sym___date__] = ACTIONS(1781),
    [sym___ldate__] = ACTIONS(1781),
    [sym___time__] = ACTIONS(1781),
    [sym___line__] = ACTIONS(1781),
    [sym___name__] = ACTIONS(1781),
    [sym___netlinx__] = ACTIONS(1781),
    [sym_netlinx_axi_version] = ACTIONS(1781),
    [sym_url_flg_tcp] = ACTIONS(1781),
    [sym_url_flg_accinfopresent] = ACTIONS(1781),
    [sym_url_flg_temp] = ACTIONS(1781),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1781),
    [sym_url_flg_stat_mask] = ACTIONS(1781),
    [sym_url_flg_stat_lookup] = ACTIONS(1781),
    [sym_url_flg_stat_connecting] = ACTIONS(1781),
    [sym_url_flg_stat_waiting] = ACTIONS(1781),
    [sym_url_flg_stat_connected] = ACTIONS(1781),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1781),
    [sym_first_virtual_device] = ACTIONS(1781),
    [sym_first_local_port] = ACTIONS(1781),
    [sym_dynamic_virtual_device] = ACTIONS(1781),
    [sym_timeline_once] = ACTIONS(1781),
    [sym_timeline_repeat] = ACTIONS(1781),
    [sym_timeline_absolute] = ACTIONS(1781),
    [sym_timeline_relative] = ACTIONS(1781),
    [sym_do_push_timed_infinite] = ACTIONS(1781),
    [sym_source_type_no_address] = ACTIONS(1781),
    [sym_source_type_neuron_id] = ACTIONS(1781),
    [sym_source_type_ip_address] = ACTIONS(1781),
    [sym_source_type_axlink] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1781),
    [sym_source_type_ip_socket_address] = ACTIONS(1781),
    [sym_source_type_rs232] = ACTIONS(1781),
    [sym_source_type_internal] = ACTIONS(1781),
    [sym_source_type_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1781),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1781),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1781),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1781),
    [sym_true] = ACTIONS(1781),
    [sym_false] = ACTIONS(1781),
    [sym_file_read_only] = ACTIONS(1781),
    [sym_file_rw_new] = ACTIONS(1781),
    [sym_file_rw_append] = ACTIONS(1781),
    [sym_ip_tcp] = ACTIONS(1781),
    [sym_ip_udp] = ACTIONS(1781),
    [sym_ip_udp_2way] = ACTIONS(1781),
    [sym_xml_encode_types] = ACTIONS(1781),
    [sym_xml_encode_char_as_list] = ACTIONS(1781),
    [sym_xml_encode_le] = ACTIONS(1781),
    [sym_xml_decode_types] = ACTIONS(1781),
    [sym_xml_decode_no_preserve] = ACTIONS(1781),
    [sym_duet_dev_type_display_device] = ACTIONS(1781),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_io_device] = ACTIONS(1781),
    [sym_duet_dev_type_relay_device] = ACTIONS(1781),
    [sym_duet_dev_type_ups] = ACTIONS(1781),
    [sym_duet_dev_type_amplifier] = ACTIONS(1781),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1781),
    [sym_duet_dev_type_camera] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1781),
    [sym_duet_dev_type_dss] = ACTIONS(1781),
    [sym_duet_dev_type_dvr] = ACTIONS(1781),
    [sym_duet_dev_type_disc_device] = ACTIONS(1781),
    [sym_duet_dev_type_document_camera] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1781),
    [sym_duet_dev_type_hvac] = ACTIONS(1781),
    [sym_duet_dev_type_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_light] = ACTIONS(1781),
    [sym_duet_dev_type_monitor] = ACTIONS(1781),
    [sym_duet_dev_type_motor] = ACTIONS(1781),
    [sym_duet_dev_type_multi_window] = ACTIONS(1781),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1781),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1781),
    [sym_duet_dev_type_receiver] = ACTIONS(1781),
    [sym_duet_dev_type_security_system] = ACTIONS(1781),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1781),
    [sym_duet_dev_type_settop_box] = ACTIONS(1781),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1781),
    [sym_duet_dev_type_switcher] = ACTIONS(1781),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_tv] = ACTIONS(1781),
    [sym_duet_dev_type_utility] = ACTIONS(1781),
    [sym_duet_dev_type_vcr] = ACTIONS(1781),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_video_processor] = ACTIONS(1781),
    [sym_duet_dev_type_video_projector] = ACTIONS(1781),
    [sym_duet_dev_type_video_wall] = ACTIONS(1781),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1781),
    [sym_duet_dev_type_weather] = ACTIONS(1781),
    [sym_duet_dev_not_polled] = ACTIONS(1781),
    [sym_duet_dev_polled] = ACTIONS(1781),
    [sym_clkmgr_mode_network] = ACTIONS(1781),
    [sym_clkmgr_mode_standalone] = ACTIONS(1781),
    [sym_normal_standby] = ACTIONS(1781),
    [sym_normal_wake] = ACTIONS(1781),
    [sym_ip_multicast_ttl_option] = ACTIONS(1781),
    [sym_ip_ttl_subnet] = ACTIONS(1781),
    [sym_ip_ttl_site] = ACTIONS(1781),
    [sym_ip_ttl_region] = ACTIONS(1781),
    [sym_ip_ttl_continent] = ACTIONS(1781),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1781),
    [sym_ip_nodelay_on] = ACTIONS(1781),
    [sym_ip_nodelay_off] = ACTIONS(1781),
    [sym_amx_error] = ACTIONS(1781),
    [sym_amx_warning] = ACTIONS(1781),
    [sym_amx_info] = ACTIONS(1781),
    [sym_amx_debug] = ACTIONS(1781),
    [sym_netlinx_login_success] = ACTIONS(1781),
    [sym_netlinx_login_fail] = ACTIONS(1781),
    [sym_netlinx_logout] = ACTIONS(1781),
    [sym_null_str] = ACTIONS(1781),
    [sym_smtp_address] = ACTIONS(1781),
    [sym_smtp_port_number] = ACTIONS(1781),
    [sym_smtp_username] = ACTIONS(1781),
    [sym_smtp_password] = ACTIONS(1781),
    [sym_smtp_require_tls] = ACTIONS(1781),
    [sym_smtp_from] = ACTIONS(1781),
    [sym_smtp_tls_true] = ACTIONS(1781),
    [sym_smtp_tls_false] = ACTIONS(1781),
    [sym_listview_on_row_select_event] = ACTIONS(1781),
    [sym_data_structure_unknown] = ACTIONS(1781),
    [sym_data_structure_datafeed] = ACTIONS(1781),
    [sym_data_structure_datarecordset] = ACTIONS(1781),
    [sym_data_structure_datarecord] = ACTIONS(1781),
    [sym_data_structure_datafield] = ACTIONS(1781),
    [sym_data_max_content_field_count] = ACTIONS(1781),
    [sym_data_max_metadata_field_count] = ACTIONS(1781),
    [sym_data_max_id_length] = ACTIONS(1781),
    [sym_data_max_name_length] = ACTIONS(1781),
    [sym_data_max_description_length] = ACTIONS(1781),
    [sym_data_max_type_length] = ACTIONS(1781),
    [sym_data_max_format_length] = ACTIONS(1781),
    [sym_data_max_source_length] = ACTIONS(1781),
    [sym_data_max_value_length] = ACTIONS(1781),
    [sym_data_max_label_length] = ACTIONS(1781),
    [sym_data_max_url_length] = ACTIONS(1781),
    [sym_max_last_login_info_length] = ACTIONS(1781),
    [sym_data_type_unknown] = ACTIONS(1781),
    [sym_data_type_string] = ACTIONS(1781),
    [sym_data_type_datetime] = ACTIONS(1781),
    [sym_data_type_date] = ACTIONS(1781),
    [sym_data_type_time] = ACTIONS(1781),
    [sym_data_type_image] = ACTIONS(1781),
    [sym_data_format_url] = ACTIONS(1781),
    [sym_data_format_phone] = ACTIONS(1781),
    [sym_data_format_email] = ACTIONS(1781),
    [sym_data_format_iso8601] = ACTIONS(1781),
    [sym_auditlog_priv] = ACTIONS(1781),
    [sym_configuration_priv] = ACTIONS(1781),
    [sym_deviceconfig_priv] = ACTIONS(1781),
    [sym_ftp_priv] = ACTIONS(1781),
    [sym_http_priv] = ACTIONS(1781),
    [sym_networkconfig_priv] = ACTIONS(1781),
    [sym_programport_priv] = ACTIONS(1781),
    [sym_remoteui_priv] = ACTIONS(1781),
    [sym_securitycontrol_priv] = ACTIONS(1781),
    [sym_softwaremanagement_priv] = ACTIONS(1781),
    [sym_terminal_priv] = ACTIONS(1781),
    [sym_tpadmin_priv] = ACTIONS(1781),
    [sym_user1_priv] = ACTIONS(1781),
    [sym_user2_priv] = ACTIONS(1781),
    [sym_user3_priv] = ACTIONS(1781),
    [sym_user4_priv] = ACTIONS(1781),
    [sym_usermanagement_priv] = ACTIONS(1781),
    [sym_valid_account] = ACTIONS(1781),
    [sym_err_name_invalid] = ACTIONS(1781),
    [sym_err_password_invalid] = ACTIONS(1781),
    [sym_err_invalid_account] = ACTIONS(1781),
    [sym_err_no_authorization] = ACTIONS(1781),
    [sym_err_invalid_parameter] = ACTIONS(1781),
    [sym_err_user_locked_out] = ACTIONS(1781),
    [sym_err_user_account_expired] = ACTIONS(1781),
    [sym_err_general_error] = ACTIONS(1781),
    [sym_err_authentication_error] = ACTIONS(1781),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1781),
    [sym_tls_validate_certificate] = ACTIONS(1781),
    [sym_tdata] = ACTIONS(1781),
    [sym_tchannel] = ACTIONS(1781),
    [sym_tlevel] = ACTIONS(1781),
    [sym_tbutton] = ACTIONS(1781),
    [sym_ttimeline] = ACTIONS(1781),
    [sym_tcustom] = ACTIONS(1781),
    [sym_url_struct] = ACTIONS(1781),
    [sym_dns_struct] = ACTIONS(1781),
    [sym_ip_address_struct] = ACTIONS(1781),
    [sym_dev_info_struct] = ACTIONS(1781),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1781),
    [sym_data_feed] = ACTIONS(1781),
    [sym_data_field] = ACTIONS(1781),
    [sym_data_record] = ACTIONS(1781),
    [sym_wc_data_feed] = ACTIONS(1781),
    [sym_wc_data_field] = ACTIONS(1781),
    [sym_wc_data_record] = ACTIONS(1781),
    [sym_last_login_info] = ACTIONS(1781),
    [sym____reserved___] = ACTIONS(1781),
    [sym_date] = ACTIONS(1781),
    [sym_day] = ACTIONS(1781),
    [sym_time] = ACTIONS(1781),
    [sym_ldate] = ACTIONS(1781),
    [sym_push_device] = ACTIONS(1781),
    [sym_push_channel] = ACTIONS(1781),
    [sym_push_devchan] = ACTIONS(1781),
    [sym_release_device] = ACTIONS(1781),
    [sym_release_channel] = ACTIONS(1781),
    [sym_release_devchan] = ACTIONS(1781),
    [sym_master_slot] = ACTIONS(1781),
    [sym_get_pulse_time] = ACTIONS(1781),
    [sym_get_timer] = ACTIONS(1781),
    [sym_master_sn] = ACTIONS(1781),
    [sym_system_number] = ACTIONS(1781),
    [sym_button] = ACTIONS(1781),
    [sym_dv_channel] = ACTIONS(1781),
    [sym_data] = ACTIONS(1781),
    [sym_level] = ACTIONS(1781),
    [sym_timeline] = ACTIONS(1781),
    [sym_channel] = ACTIONS(1781),
    [sym_custom] = ACTIONS(1781),
    [sym_atoi] = ACTIONS(1781),
    [sym_atol] = ACTIONS(1781),
    [sym_atof] = ACTIONS(1781),
    [sym_itoa] = ACTIONS(1781),
    [sym_itohex] = ACTIONS(1781),
    [sym_hextoi] = ACTIONS(1781),
    [sym_ftoa] = ACTIONS(1781),
    [sym_find_string] = ACTIONS(1781),
    [sym_left_string] = ACTIONS(1781),
    [sym_right_string] = ACTIONS(1781),
    [sym_mid_string] = ACTIONS(1781),
    [sym_remove_string] = ACTIONS(1781),
    [sym_get_buffer_string] = ACTIONS(1781),
    [sym_get_buffer_char] = ACTIONS(1781),
    [sym_get_multi_buffer_string] = ACTIONS(1781),
    [sym_length_string] = ACTIONS(1781),
    [sym_max_length_string] = ACTIONS(1781),
    [sym_set_length_string] = ACTIONS(1781),
    [sym_compare_string] = ACTIONS(1781),
    [sym_upper_string] = ACTIONS(1781),
    [sym_lower_string] = ACTIONS(1781),
    [sym_get_last] = ACTIONS(1781),
    [sym_date_to_day] = ACTIONS(1781),
    [sym_date_to_month] = ACTIONS(1781),
    [sym_date_to_year] = ACTIONS(1781),
    [sym_time_to_hour] = ACTIONS(1781),
    [sym_time_to_minute] = ACTIONS(1781),
    [sym_time_to_second] = ACTIONS(1781),
    [sym_day_of_week] = ACTIONS(1781),
    [sym_set_pulse_time] = ACTIONS(1781),
    [sym_set_timer] = ACTIONS(1781),
    [sym_file_open] = ACTIONS(1781),
    [sym_file_close] = ACTIONS(1781),
    [sym_file_read] = ACTIONS(1781),
    [sym_file_read_line] = ACTIONS(1781),
    [sym_file_write] = ACTIONS(1781),
    [sym_file_write_line] = ACTIONS(1781),
    [sym_file_copy] = ACTIONS(1781),
    [sym_file_delete] = ACTIONS(1781),
    [sym_file_rename] = ACTIONS(1781),
    [sym_file_dir] = ACTIONS(1781),
    [sym_file_getdir] = ACTIONS(1781),
    [sym_file_setdir] = ACTIONS(1781),
    [sym_file_create_dir] = ACTIONS(1781),
    [sym_file_remove_dir] = ACTIONS(1781),
    [sym_file_seek] = ACTIONS(1781),
    [sym_length_array] = ACTIONS(1781),
    [sym_max_length_array] = ACTIONS(1781),
    [sym_set_length_array] = ACTIONS(1781),
    [sym_ip_client_open] = ACTIONS(1781),
    [sym_ip_client_close] = ACTIONS(1781),
    [sym_ip_server_open] = ACTIONS(1781),
    [sym_ip_server_close] = ACTIONS(1781),
    [sym_ip_bound_client_open] = ACTIONS(1781),
    [sym_ip_mc_server_open] = ACTIONS(1781),
    [sym_ip_set_option] = ACTIONS(1781),
    [sym_ssh_client_open] = ACTIONS(1781),
    [sym_ssh_client_close] = ACTIONS(1781),
    [sym_tls_client_open] = ACTIONS(1781),
    [sym_tls_client_close] = ACTIONS(1781),
    [sym_do_push] = ACTIONS(1781),
    [sym_do_push_timed] = ACTIONS(1781),
    [sym_do_release] = ACTIONS(1781),
    [sym_device_id] = ACTIONS(1781),
    [sym_device_info] = ACTIONS(1781),
    [sym_device_id_string] = ACTIONS(1781),
    [sym_device_standby] = ACTIONS(1781),
    [sym_device_wake] = ACTIONS(1781),
    [sym_timeline_create] = ACTIONS(1781),
    [sym_timeline_kill] = ACTIONS(1781),
    [sym_timeline_reload] = ACTIONS(1781),
    [sym_timeline_pause] = ACTIONS(1781),
    [sym_timeline_restart] = ACTIONS(1781),
    [sym_timeline_set] = ACTIONS(1781),
    [sym_timeline_get] = ACTIONS(1781),
    [sym_timeline_active] = ACTIONS(1781),
    [sym_timeline_dynamic_id] = ACTIONS(1781),
    [sym_variable_to_string] = ACTIONS(1781),
    [sym_string_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_string] = ACTIONS(1781),
    [sym_type_cast] = ACTIONS(1781),
    [sym_variable_to_xml] = ACTIONS(1781),
    [sym_xml_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_xml] = ACTIONS(1781),
    [sym_redirect_string] = ACTIONS(1781),
    [sym_random_number] = ACTIONS(1781),
    [sym_get_unique_id] = ACTIONS(1781),
    [sym_get_system_number] = ACTIONS(1781),
    [sym_set_system_number] = ACTIONS(1781),
    [sym_get_serial_number] = ACTIONS(1781),
    [sym_get_master_build] = ACTIONS(1781),
    [sym_set_validation_code] = ACTIONS(1781),
    [sym_reboot] = ACTIONS(1781),
    [sym_load_duet_module] = ACTIONS(1781),
    [sym_unload_duet_module] = ACTIONS(1781),
    [sym_duet_mem_size_set] = ACTIONS(1781),
    [sym_duet_mem_size_get] = ACTIONS(1781),
    [sym_dynamic_application_device] = ACTIONS(1781),
    [sym_dynamic_polled_port] = ACTIONS(1781),
    [sym_static_port_binding] = ACTIONS(1781),
    [sym_static_ip_binding] = ACTIONS(1781),
    [sym_get_dns_list] = ACTIONS(1781),
    [sym_set_dns_list] = ACTIONS(1781),
    [sym_get_ip_address] = ACTIONS(1781),
    [sym_set_ip_address] = ACTIONS(1781),
    [sym_get_url_list] = ACTIONS(1781),
    [sym_add_url_entry] = ACTIONS(1781),
    [sym_delete_url_entry] = ACTIONS(1781),
    [sym_abs_value] = ACTIONS(1781),
    [sym_max_value] = ACTIONS(1781),
    [sym_min_value] = ACTIONS(1781),
    [sym_exp_value] = ACTIONS(1781),
    [sym_log_value] = ACTIONS(1781),
    [sym_log10_value] = ACTIONS(1781),
    [sym_power_value] = ACTIONS(1781),
    [sym_sqrt_value] = ACTIONS(1781),
    [sym_raw_be] = ACTIONS(1781),
    [sym_raw_le] = ACTIONS(1781),
    [sym_internal_queue_size_set] = ACTIONS(1781),
    [sym_internal_queue_size_get] = ACTIONS(1781),
    [sym_internal_threshold_set] = ACTIONS(1781),
    [sym_internal_threshold_get] = ACTIONS(1781),
    [sym_rebuilt_event] = ACTIONS(1781),
    [sym_do_custom_event] = ACTIONS(1781),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1781),
    [sym_clkmgr_set_clk_source] = ACTIONS(1781),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1781),
    [sym_clkmgr_get_timezone] = ACTIONS(1781),
    [sym_clkmgr_set_timezone] = ACTIONS(1781),
    [sym_clkmgr_get_resync_period] = ACTIONS(1781),
    [sym_clkmgr_set_resync_period] = ACTIONS(1781),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_timeservers] = ACTIONS(1781),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_set_log_level] = ACTIONS(1781),
    [sym_get_log_level] = ACTIONS(1781),
    [sym_amx_log] = ACTIONS(1781),
    [sym_smtp_server_config_set] = ACTIONS(1781),
    [sym_smtp_server_config_get] = ACTIONS(1781),
    [sym_smtp_send] = ACTIONS(1781),
    [sym_validate_netlinx_account] = ACTIONS(1781),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1781),
    [sym_authenticate_certificate] = ACTIONS(1781),
    [sym_audit_netlinx_session_event] = ACTIONS(1781),
    [sym_audit_netlinx_generic_event] = ACTIONS(1781),
    [sym_get_available_flash_disk_space] = ACTIONS(1781),
    [sym_get_max_flash_disk_space] = ACTIONS(1781),
    [sym_data_create_feed] = ACTIONS(1781),
    [sym_data_delete_feed] = ACTIONS(1781),
    [sym_data_publish_feed] = ACTIONS(1781),
    [sym_data_get_published_feed] = ACTIONS(1781),
    [sym_data_add_record] = ACTIONS(1781),
    [sym_data_get_event_record] = ACTIONS(1781),
    [sym__wc_data_create_feed] = ACTIONS(1781),
    [sym__wc_data_add_record] = ACTIONS(1781),
    [sym__wc_data_get_event_record] = ACTIONS(1781),
    [sym_format] = ACTIONS(1781),
    [sym_astro_clock] = ACTIONS(1781),
    [sym_set_outdoor_temperature] = ACTIONS(1781),
    [sym_set_virtual_level_count] = ACTIONS(1781),
    [sym_set_virtual_channel_count] = ACTIONS(1781),
    [sym_set_virtual_port_count] = ACTIONS(1781),
    [sym__wc_encode_int] = ACTIONS(1781),
    [sym__wc_decode_int] = ACTIONS(1781),
    [sym__wc_int] = ACTIONS(1781),
    [sym__wc_to_ch_int] = ACTIONS(1781),
    [sym__ch_to_wc_int] = ACTIONS(1781),
    [sym__wc_find_string_int] = ACTIONS(1781),
    [sym__wc_left_string_int] = ACTIONS(1781),
    [sym__wc_lower_string_int] = ACTIONS(1781),
    [sym__wc_mid_string_int] = ACTIONS(1781),
    [sym__wc_remove_string_int] = ACTIONS(1781),
    [sym__wc_right_string_int] = ACTIONS(1781),
    [sym__wc_upper_string_int] = ACTIONS(1781),
    [sym__wc_compare_string_int] = ACTIONS(1781),
    [sym__wc_get_buffer_char_int] = ACTIONS(1781),
    [sym__wc_get_buffer_string_int] = ACTIONS(1781),
    [sym__wc_concat_string_int] = ACTIONS(1781),
    [sym__wc_file_open_int] = ACTIONS(1781),
    [sym__wc_file_close_int] = ACTIONS(1781),
    [sym__wc_file_read_int] = ACTIONS(1781),
    [sym__wc_file_read_line_int] = ACTIONS(1781),
    [sym__wc_file_write_int] = ACTIONS(1781),
    [sym__wc_file_write_line_int] = ACTIONS(1781),
    [aux_sym_preproc_include_token1] = ACTIONS(1781),
    [aux_sym_preproc_define_token1] = ACTIONS(1783),
    [aux_sym_preproc_warn_token1] = ACTIONS(1783),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1783),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_else_token1] = ACTIONS(1783),
    [anon_sym_LBRACK] = ACTIONS(1783),
    [anon_sym_LBRACE] = ACTIONS(1783),
    [anon_sym_LPAREN] = ACTIONS(1781),
    [anon_sym_BANG] = ACTIONS(1783),
    [anon_sym_TILDE] = ACTIONS(1783),
    [anon_sym_DASH] = ACTIONS(1781),
    [anon_sym_PLUS] = ACTIONS(1781),
    [anon_sym_DASH_DASH] = ACTIONS(1783),
    [anon_sym_PLUS_PLUS] = ACTIONS(1783),
    [anon_sym_DQUOTE] = ACTIONS(1783),
    [anon_sym_SQUOTE] = ACTIONS(1783),
    [sym_number_literal] = ACTIONS(1783),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(238)] = {
    [sym_identifier] = ACTIONS(1789),
    [sym_program_name_keyword] = ACTIONS(1789),
    [sym_module_name_keyword] = ACTIONS(1789),
    [sym_define_device_keyword] = ACTIONS(1789),
    [sym_define_combine_keyword] = ACTIONS(1789),
    [sym_define_constant_keyword] = ACTIONS(1789),
    [sym_define_type_keyword] = ACTIONS(1789),
    [sym_define_variable_keyword] = ACTIONS(1789),
    [sym_define_system_variable_keyword] = ACTIONS(1789),
    [sym_define_connect_level_keyword] = ACTIONS(1789),
    [sym_define_latching_keyword] = ACTIONS(1789),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1789),
    [sym_define_toggling_keyword] = ACTIONS(1789),
    [sym_define_call_keyword] = ACTIONS(1789),
    [sym_define_function_keyword] = ACTIONS(1789),
    [sym_define_library_function_keyword] = ACTIONS(1789),
    [sym_define_start_keyword] = ACTIONS(1789),
    [sym_define_event_keyword] = ACTIONS(1789),
    [sym_define_program_keyword] = ACTIONS(1789),
    [sym_define_module_keyword] = ACTIONS(1789),
    [sym_char_keyword] = ACTIONS(1789),
    [sym_widechar_keyword] = ACTIONS(1789),
    [sym_integer_keyword] = ACTIONS(1789),
    [sym_sinteger_keyword] = ACTIONS(1789),
    [sym_long_keyword] = ACTIONS(1789),
    [sym_slong_keyword] = ACTIONS(1789),
    [sym_float_keyword] = ACTIONS(1789),
    [sym_double_keyword] = ACTIONS(1789),
    [sym_variant_keyword] = ACTIONS(1789),
    [sym_variantarray_keyword] = ACTIONS(1789),
    [sym_local_var_keyword] = ACTIONS(1789),
    [sym_stack_var_keyword] = ACTIONS(1789),
    [sym_constant_keyword] = ACTIONS(1789),
    [sym_volatile_keyword] = ACTIONS(1789),
    [sym_non_volatile_keyword] = ACTIONS(1789),
    [sym_persistent_keyword] = ACTIONS(1789),
    [sym_dev_keyword] = ACTIONS(1789),
    [sym_devlev_keyword] = ACTIONS(1789),
    [sym_devchan_keyword] = ACTIONS(1789),
    [sym_if_keyword] = ACTIONS(1789),
    [sym_else_keyword] = ACTIONS(1789),
    [sym_switch_keyword] = ACTIONS(1789),
    [sym_case_keyword] = ACTIONS(1789),
    [sym_default_keyword] = ACTIONS(1789),
    [sym_while_keyword] = ACTIONS(1789),
    [sym_for_keyword] = ACTIONS(1789),
    [sym_break_keyword] = ACTIONS(1789),
    [sym_continue_keyword] = ACTIONS(1789),
    [sym_return_keyword] = ACTIONS(1789),
    [sym_select_keyword] = ACTIONS(1789),
    [sym_active_keyword] = ACTIONS(1789),
    [sym_struct_keyword] = ACTIONS(1789),
    [sym_structure_keyword] = ACTIONS(1789),
    [sym_button_event_keyword] = ACTIONS(1789),
    [sym_channel_event_keyword] = ACTIONS(1789),
    [sym_level_event_keyword] = ACTIONS(1789),
    [sym_data_event_keyword] = ACTIONS(1789),
    [sym_timeline_event_keyword] = ACTIONS(1789),
    [sym_custom_event_keyword] = ACTIONS(1789),
    [sym_push_keyword] = ACTIONS(1789),
    [sym_release_keyword] = ACTIONS(1789),
    [aux_sym_on_keyword_token1] = ACTIONS(1789),
    [aux_sym_off_keyword_token1] = ACTIONS(1789),
    [sym_send_level_keyword] = ACTIONS(1789),
    [sym_send_string_keyword] = ACTIONS(1789),
    [sym_send_command_keyword] = ACTIONS(1789),
    [sym_clear_buffer_keyword] = ACTIONS(1789),
    [sym_create_buffer_keyword] = ACTIONS(1789),
    [sym_create_multi_buffer_keyword] = ACTIONS(1789),
    [sym_call_keyword] = ACTIONS(1789),
    [sym_system_call_keyword] = ACTIONS(1789),
    [sym_devchan_to_keyword] = ACTIONS(1789),
    [sym_devchan_min_to_keyword] = ACTIONS(1789),
    [sym_devchan_total_off_keyword] = ACTIONS(1789),
    [sym_devchan_pulse_keyword] = ACTIONS(1789),
    [sym_wait_keyword] = ACTIONS(1789),
    [sym_cancel_wait_keyword] = ACTIONS(1789),
    [sym_wait_until_keyword] = ACTIONS(1789),
    [sym_cancel_wait_until_keyword] = ACTIONS(1789),
    [sym_cancel_all_wait_keyword] = ACTIONS(1789),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1789),
    [sym_not] = ACTIONS(1789),
    [sym_bnot] = ACTIONS(1789),
    [sym___file__] = ACTIONS(1789),
    [sym___date__] = ACTIONS(1789),
    [sym___ldate__] = ACTIONS(1789),
    [sym___time__] = ACTIONS(1789),
    [sym___line__] = ACTIONS(1789),
    [sym___name__] = ACTIONS(1789),
    [sym___netlinx__] = ACTIONS(1789),
    [sym_netlinx_axi_version] = ACTIONS(1789),
    [sym_url_flg_tcp] = ACTIONS(1789),
    [sym_url_flg_accinfopresent] = ACTIONS(1789),
    [sym_url_flg_temp] = ACTIONS(1789),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1789),
    [sym_url_flg_stat_mask] = ACTIONS(1789),
    [sym_url_flg_stat_lookup] = ACTIONS(1789),
    [sym_url_flg_stat_connecting] = ACTIONS(1789),
    [sym_url_flg_stat_waiting] = ACTIONS(1789),
    [sym_url_flg_stat_connected] = ACTIONS(1789),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1789),
    [sym_first_virtual_device] = ACTIONS(1789),
    [sym_first_local_port] = ACTIONS(1789),
    [sym_dynamic_virtual_device] = ACTIONS(1789),
    [sym_timeline_once] = ACTIONS(1789),
    [sym_timeline_repeat] = ACTIONS(1789),
    [sym_timeline_absolute] = ACTIONS(1789),
    [sym_timeline_relative] = ACTIONS(1789),
    [sym_do_push_timed_infinite] = ACTIONS(1789),
    [sym_source_type_no_address] = ACTIONS(1789),
    [sym_source_type_neuron_id] = ACTIONS(1789),
    [sym_source_type_ip_address] = ACTIONS(1789),
    [sym_source_type_axlink] = ACTIONS(1789),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1789),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1789),
    [sym_source_type_ip_socket_address] = ACTIONS(1789),
    [sym_source_type_rs232] = ACTIONS(1789),
    [sym_source_type_internal] = ACTIONS(1789),
    [sym_source_type_mac_address] = ACTIONS(1789),
    [sym_source_type_ipv4_port] = ACTIONS(1789),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1789),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1789),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1789),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1789),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1789),
    [sym_true] = ACTIONS(1789),
    [sym_false] = ACTIONS(1789),
    [sym_file_read_only] = ACTIONS(1789),
    [sym_file_rw_new] = ACTIONS(1789),
    [sym_file_rw_append] = ACTIONS(1789),
    [sym_ip_tcp] = ACTIONS(1789),
    [sym_ip_udp] = ACTIONS(1789),
    [sym_ip_udp_2way] = ACTIONS(1789),
    [sym_xml_encode_types] = ACTIONS(1789),
    [sym_xml_encode_char_as_list] = ACTIONS(1789),
    [sym_xml_encode_le] = ACTIONS(1789),
    [sym_xml_decode_types] = ACTIONS(1789),
    [sym_xml_decode_no_preserve] = ACTIONS(1789),
    [sym_duet_dev_type_display_device] = ACTIONS(1789),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1789),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1789),
    [sym_duet_dev_type_io_device] = ACTIONS(1789),
    [sym_duet_dev_type_relay_device] = ACTIONS(1789),
    [sym_duet_dev_type_ups] = ACTIONS(1789),
    [sym_duet_dev_type_amplifier] = ACTIONS(1789),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1789),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1789),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1789),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1789),
    [sym_duet_dev_type_camera] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1789),
    [sym_duet_dev_type_dss] = ACTIONS(1789),
    [sym_duet_dev_type_dvr] = ACTIONS(1789),
    [sym_duet_dev_type_disc_device] = ACTIONS(1789),
    [sym_duet_dev_type_document_camera] = ACTIONS(1789),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1789),
    [sym_duet_dev_type_hvac] = ACTIONS(1789),
    [sym_duet_dev_type_keypad] = ACTIONS(1789),
    [sym_duet_dev_type_light] = ACTIONS(1789),
    [sym_duet_dev_type_monitor] = ACTIONS(1789),
    [sym_duet_dev_type_motor] = ACTIONS(1789),
    [sym_duet_dev_type_multi_window] = ACTIONS(1789),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1789),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1789),
    [sym_duet_dev_type_receiver] = ACTIONS(1789),
    [sym_duet_dev_type_security_system] = ACTIONS(1789),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1789),
    [sym_duet_dev_type_settop_box] = ACTIONS(1789),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1789),
    [sym_duet_dev_type_switcher] = ACTIONS(1789),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1789),
    [sym_duet_dev_type_tv] = ACTIONS(1789),
    [sym_duet_dev_type_utility] = ACTIONS(1789),
    [sym_duet_dev_type_vcr] = ACTIONS(1789),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1789),
    [sym_duet_dev_type_video_processor] = ACTIONS(1789),
    [sym_duet_dev_type_video_projector] = ACTIONS(1789),
    [sym_duet_dev_type_video_wall] = ACTIONS(1789),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1789),
    [sym_duet_dev_type_weather] = ACTIONS(1789),
    [sym_duet_dev_not_polled] = ACTIONS(1789),
    [sym_duet_dev_polled] = ACTIONS(1789),
    [sym_clkmgr_mode_network] = ACTIONS(1789),
    [sym_clkmgr_mode_standalone] = ACTIONS(1789),
    [sym_normal_standby] = ACTIONS(1789),
    [sym_normal_wake] = ACTIONS(1789),
    [sym_ip_multicast_ttl_option] = ACTIONS(1789),
    [sym_ip_ttl_subnet] = ACTIONS(1789),
    [sym_ip_ttl_site] = ACTIONS(1789),
    [sym_ip_ttl_region] = ACTIONS(1789),
    [sym_ip_ttl_continent] = ACTIONS(1789),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1789),
    [sym_ip_nodelay_on] = ACTIONS(1789),
    [sym_ip_nodelay_off] = ACTIONS(1789),
    [sym_amx_error] = ACTIONS(1789),
    [sym_amx_warning] = ACTIONS(1789),
    [sym_amx_info] = ACTIONS(1789),
    [sym_amx_debug] = ACTIONS(1789),
    [sym_netlinx_login_success] = ACTIONS(1789),
    [sym_netlinx_login_fail] = ACTIONS(1789),
    [sym_netlinx_logout] = ACTIONS(1789),
    [sym_null_str] = ACTIONS(1789),
    [sym_smtp_address] = ACTIONS(1789),
    [sym_smtp_port_number] = ACTIONS(1789),
    [sym_smtp_username] = ACTIONS(1789),
    [sym_smtp_password] = ACTIONS(1789),
    [sym_smtp_require_tls] = ACTIONS(1789),
    [sym_smtp_from] = ACTIONS(1789),
    [sym_smtp_tls_true] = ACTIONS(1789),
    [sym_smtp_tls_false] = ACTIONS(1789),
    [sym_listview_on_row_select_event] = ACTIONS(1789),
    [sym_data_structure_unknown] = ACTIONS(1789),
    [sym_data_structure_datafeed] = ACTIONS(1789),
    [sym_data_structure_datarecordset] = ACTIONS(1789),
    [sym_data_structure_datarecord] = ACTIONS(1789),
    [sym_data_structure_datafield] = ACTIONS(1789),
    [sym_data_max_content_field_count] = ACTIONS(1789),
    [sym_data_max_metadata_field_count] = ACTIONS(1789),
    [sym_data_max_id_length] = ACTIONS(1789),
    [sym_data_max_name_length] = ACTIONS(1789),
    [sym_data_max_description_length] = ACTIONS(1789),
    [sym_data_max_type_length] = ACTIONS(1789),
    [sym_data_max_format_length] = ACTIONS(1789),
    [sym_data_max_source_length] = ACTIONS(1789),
    [sym_data_max_value_length] = ACTIONS(1789),
    [sym_data_max_label_length] = ACTIONS(1789),
    [sym_data_max_url_length] = ACTIONS(1789),
    [sym_max_last_login_info_length] = ACTIONS(1789),
    [sym_data_type_unknown] = ACTIONS(1789),
    [sym_data_type_string] = ACTIONS(1789),
    [sym_data_type_datetime] = ACTIONS(1789),
    [sym_data_type_date] = ACTIONS(1789),
    [sym_data_type_time] = ACTIONS(1789),
    [sym_data_type_image] = ACTIONS(1789),
    [sym_data_format_url] = ACTIONS(1789),
    [sym_data_format_phone] = ACTIONS(1789),
    [sym_data_format_email] = ACTIONS(1789),
    [sym_data_format_iso8601] = ACTIONS(1789),
    [sym_auditlog_priv] = ACTIONS(1789),
    [sym_configuration_priv] = ACTIONS(1789),
    [sym_deviceconfig_priv] = ACTIONS(1789),
    [sym_ftp_priv] = ACTIONS(1789),
    [sym_http_priv] = ACTIONS(1789),
    [sym_networkconfig_priv] = ACTIONS(1789),
    [sym_programport_priv] = ACTIONS(1789),
    [sym_remoteui_priv] = ACTIONS(1789),
    [sym_securitycontrol_priv] = ACTIONS(1789),
    [sym_softwaremanagement_priv] = ACTIONS(1789),
    [sym_terminal_priv] = ACTIONS(1789),
    [sym_tpadmin_priv] = ACTIONS(1789),
    [sym_user1_priv] = ACTIONS(1789),
    [sym_user2_priv] = ACTIONS(1789),
    [sym_user3_priv] = ACTIONS(1789),
    [sym_user4_priv] = ACTIONS(1789),
    [sym_usermanagement_priv] = ACTIONS(1789),
    [sym_valid_account] = ACTIONS(1789),
    [sym_err_name_invalid] = ACTIONS(1789),
    [sym_err_password_invalid] = ACTIONS(1789),
    [sym_err_invalid_account] = ACTIONS(1789),
    [sym_err_no_authorization] = ACTIONS(1789),
    [sym_err_invalid_parameter] = ACTIONS(1789),
    [sym_err_user_locked_out] = ACTIONS(1789),
    [sym_err_user_account_expired] = ACTIONS(1789),
    [sym_err_general_error] = ACTIONS(1789),
    [sym_err_authentication_error] = ACTIONS(1789),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1789),
    [sym_tls_validate_certificate] = ACTIONS(1789),
    [sym_tdata] = ACTIONS(1789),
    [sym_tchannel] = ACTIONS(1789),
    [sym_tlevel] = ACTIONS(1789),
    [sym_tbutton] = ACTIONS(1789),
    [sym_ttimeline] = ACTIONS(1789),
    [sym_tcustom] = ACTIONS(1789),
    [sym_url_struct] = ACTIONS(1789),
    [sym_dns_struct] = ACTIONS(1789),
    [sym_ip_address_struct] = ACTIONS(1789),
    [sym_dev_info_struct] = ACTIONS(1789),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1789),
    [sym_data_feed] = ACTIONS(1789),
    [sym_data_field] = ACTIONS(1789),
    [sym_data_record] = ACTIONS(1789),
    [sym_wc_data_feed] = ACTIONS(1789),
    [sym_wc_data_field] = ACTIONS(1789),
    [sym_wc_data_record] = ACTIONS(1789),
    [sym_last_login_info] = ACTIONS(1789),
    [sym____reserved___] = ACTIONS(1789),
    [sym_date] = ACTIONS(1789),
    [sym_day] = ACTIONS(1789),
    [sym_time] = ACTIONS(1789),
    [sym_ldate] = ACTIONS(1789),
    [sym_push_device] = ACTIONS(1789),
    [sym_push_channel] = ACTIONS(1789),
    [sym_push_devchan] = ACTIONS(1789),
    [sym_release_device] = ACTIONS(1789),
    [sym_release_channel] = ACTIONS(1789),
    [sym_release_devchan] = ACTIONS(1789),
    [sym_master_slot] = ACTIONS(1789),
    [sym_get_pulse_time] = ACTIONS(1789),
    [sym_get_timer] = ACTIONS(1789),
    [sym_master_sn] = ACTIONS(1789),
    [sym_system_number] = ACTIONS(1789),
    [sym_button] = ACTIONS(1789),
    [sym_dv_channel] = ACTIONS(1789),
    [sym_data] = ACTIONS(1789),
    [sym_level] = ACTIONS(1789),
    [sym_timeline] = ACTIONS(1789),
    [sym_channel] = ACTIONS(1789),
    [sym_custom] = ACTIONS(1789),
    [sym_atoi] = ACTIONS(1789),
    [sym_atol] = ACTIONS(1789),
    [sym_atof] = ACTIONS(1789),
    [sym_itoa] = ACTIONS(1789),
    [sym_itohex] = ACTIONS(1789),
    [sym_hextoi] = ACTIONS(1789),
    [sym_ftoa] = ACTIONS(1789),
    [sym_find_string] = ACTIONS(1789),
    [sym_left_string] = ACTIONS(1789),
    [sym_right_string] = ACTIONS(1789),
    [sym_mid_string] = ACTIONS(1789),
    [sym_remove_string] = ACTIONS(1789),
    [sym_get_buffer_string] = ACTIONS(1789),
    [sym_get_buffer_char] = ACTIONS(1789),
    [sym_get_multi_buffer_string] = ACTIONS(1789),
    [sym_length_string] = ACTIONS(1789),
    [sym_max_length_string] = ACTIONS(1789),
    [sym_set_length_string] = ACTIONS(1789),
    [sym_compare_string] = ACTIONS(1789),
    [sym_upper_string] = ACTIONS(1789),
    [sym_lower_string] = ACTIONS(1789),
    [sym_get_last] = ACTIONS(1789),
    [sym_date_to_day] = ACTIONS(1789),
    [sym_date_to_month] = ACTIONS(1789),
    [sym_date_to_year] = ACTIONS(1789),
    [sym_time_to_hour] = ACTIONS(1789),
    [sym_time_to_minute] = ACTIONS(1789),
    [sym_time_to_second] = ACTIONS(1789),
    [sym_day_of_week] = ACTIONS(1789),
    [sym_set_pulse_time] = ACTIONS(1789),
    [sym_set_timer] = ACTIONS(1789),
    [sym_file_open] = ACTIONS(1789),
    [sym_file_close] = ACTIONS(1789),
    [sym_file_read] = ACTIONS(1789),
    [sym_file_read_line] = ACTIONS(1789),
    [sym_file_write] = ACTIONS(1789),
    [sym_file_write_line] = ACTIONS(1789),
    [sym_file_copy] = ACTIONS(1789),
    [sym_file_delete] = ACTIONS(1789),
    [sym_file_rename] = ACTIONS(1789),
    [sym_file_dir] = ACTIONS(1789),
    [sym_file_getdir] = ACTIONS(1789),
    [sym_file_setdir] = ACTIONS(1789),
    [sym_file_create_dir] = ACTIONS(1789),
    [sym_file_remove_dir] = ACTIONS(1789),
    [sym_file_seek] = ACTIONS(1789),
    [sym_length_array] = ACTIONS(1789),
    [sym_max_length_array] = ACTIONS(1789),
    [sym_set_length_array] = ACTIONS(1789),
    [sym_ip_client_open] = ACTIONS(1789),
    [sym_ip_client_close] = ACTIONS(1789),
    [sym_ip_server_open] = ACTIONS(1789),
    [sym_ip_server_close] = ACTIONS(1789),
    [sym_ip_bound_client_open] = ACTIONS(1789),
    [sym_ip_mc_server_open] = ACTIONS(1789),
    [sym_ip_set_option] = ACTIONS(1789),
    [sym_ssh_client_open] = ACTIONS(1789),
    [sym_ssh_client_close] = ACTIONS(1789),
    [sym_tls_client_open] = ACTIONS(1789),
    [sym_tls_client_close] = ACTIONS(1789),
    [sym_do_push] = ACTIONS(1789),
    [sym_do_push_timed] = ACTIONS(1789),
    [sym_do_release] = ACTIONS(1789),
    [sym_device_id] = ACTIONS(1789),
    [sym_device_info] = ACTIONS(1789),
    [sym_device_id_string] = ACTIONS(1789),
    [sym_device_standby] = ACTIONS(1789),
    [sym_device_wake] = ACTIONS(1789),
    [sym_timeline_create] = ACTIONS(1789),
    [sym_timeline_kill] = ACTIONS(1789),
    [sym_timeline_reload] = ACTIONS(1789),
    [sym_timeline_pause] = ACTIONS(1789),
    [sym_timeline_restart] = ACTIONS(1789),
    [sym_timeline_set] = ACTIONS(1789),
    [sym_timeline_get] = ACTIONS(1789),
    [sym_timeline_active] = ACTIONS(1789),
    [sym_timeline_dynamic_id] = ACTIONS(1789),
    [sym_variable_to_string] = ACTIONS(1789),
    [sym_string_to_variable] = ACTIONS(1789),
    [sym_length_variable_to_string] = ACTIONS(1789),
    [sym_type_cast] = ACTIONS(1789),
    [sym_variable_to_xml] = ACTIONS(1789),
    [sym_xml_to_variable] = ACTIONS(1789),
    [sym_length_variable_to_xml] = ACTIONS(1789),
    [sym_redirect_string] = ACTIONS(1789),
    [sym_random_number] = ACTIONS(1789),
    [sym_get_unique_id] = ACTIONS(1789),
    [sym_get_system_number] = ACTIONS(1789),
    [sym_set_system_number] = ACTIONS(1789),
    [sym_get_serial_number] = ACTIONS(1789),
    [sym_get_master_build] = ACTIONS(1789),
    [sym_set_validation_code] = ACTIONS(1789),
    [sym_reboot] = ACTIONS(1789),
    [sym_load_duet_module] = ACTIONS(1789),
    [sym_unload_duet_module] = ACTIONS(1789),
    [sym_duet_mem_size_set] = ACTIONS(1789),
    [sym_duet_mem_size_get] = ACTIONS(1789),
    [sym_dynamic_application_device] = ACTIONS(1789),
    [sym_dynamic_polled_port] = ACTIONS(1789),
    [sym_static_port_binding] = ACTIONS(1789),
    [sym_static_ip_binding] = ACTIONS(1789),
    [sym_get_dns_list] = ACTIONS(1789),
    [sym_set_dns_list] = ACTIONS(1789),
    [sym_get_ip_address] = ACTIONS(1789),
    [sym_set_ip_address] = ACTIONS(1789),
    [sym_get_url_list] = ACTIONS(1789),
    [sym_add_url_entry] = ACTIONS(1789),
    [sym_delete_url_entry] = ACTIONS(1789),
    [sym_abs_value] = ACTIONS(1789),
    [sym_max_value] = ACTIONS(1789),
    [sym_min_value] = ACTIONS(1789),
    [sym_exp_value] = ACTIONS(1789),
    [sym_log_value] = ACTIONS(1789),
    [sym_log10_value] = ACTIONS(1789),
    [sym_power_value] = ACTIONS(1789),
    [sym_sqrt_value] = ACTIONS(1789),
    [sym_raw_be] = ACTIONS(1789),
    [sym_raw_le] = ACTIONS(1789),
    [sym_internal_queue_size_set] = ACTIONS(1789),
    [sym_internal_queue_size_get] = ACTIONS(1789),
    [sym_internal_threshold_set] = ACTIONS(1789),
    [sym_internal_threshold_get] = ACTIONS(1789),
    [sym_rebuilt_event] = ACTIONS(1789),
    [sym_do_custom_event] = ACTIONS(1789),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1789),
    [sym_clkmgr_set_clk_source] = ACTIONS(1789),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1789),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1789),
    [sym_clkmgr_get_timezone] = ACTIONS(1789),
    [sym_clkmgr_set_timezone] = ACTIONS(1789),
    [sym_clkmgr_get_resync_period] = ACTIONS(1789),
    [sym_clkmgr_set_resync_period] = ACTIONS(1789),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1789),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1789),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1789),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1789),
    [sym_clkmgr_get_timeservers] = ACTIONS(1789),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1789),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1789),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1789),
    [sym_set_log_level] = ACTIONS(1789),
    [sym_get_log_level] = ACTIONS(1789),
    [sym_amx_log] = ACTIONS(1789),
    [sym_smtp_server_config_set] = ACTIONS(1789),
    [sym_smtp_server_config_get] = ACTIONS(1789),
    [sym_smtp_send] = ACTIONS(1789),
    [sym_validate_netlinx_account] = ACTIONS(1789),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1789),
    [sym_authenticate_certificate] = ACTIONS(1789),
    [sym_audit_netlinx_session_event] = ACTIONS(1789),
    [sym_audit_netlinx_generic_event] = ACTIONS(1789),
    [sym_get_available_flash_disk_space] = ACTIONS(1789),
    [sym_get_max_flash_disk_space] = ACTIONS(1789),
    [sym_data_create_feed] = ACTIONS(1789),
    [sym_data_delete_feed] = ACTIONS(1789),
    [sym_data_publish_feed] = ACTIONS(1789),
    [sym_data_get_published_feed] = ACTIONS(1789),
    [sym_data_add_record] = ACTIONS(1789),
    [sym_data_get_event_record] = ACTIONS(1789),
    [sym__wc_data_create_feed] = ACTIONS(1789),
    [sym__wc_data_add_record] = ACTIONS(1789),
    [sym__wc_data_get_event_record] = ACTIONS(1789),
    [sym_format] = ACTIONS(1789),
    [sym_astro_clock] = ACTIONS(1789),
    [sym_set_outdoor_temperature] = ACTIONS(1789),
    [sym_set_virtual_level_count] = ACTIONS(1789),
    [sym_set_virtual_channel_count] = ACTIONS(1789),
    [sym_set_virtual_port_count] = ACTIONS(1789),
    [sym__wc_encode_int] = ACTIONS(1789),
    [sym__wc_decode_int] = ACTIONS(1789),
    [sym__wc_int] = ACTIONS(1789),
    [sym__wc_to_ch_int] = ACTIONS(1789),
    [sym__ch_to_wc_int] = ACTIONS(1789),
    [sym__wc_find_string_int] = ACTIONS(1789),
    [sym__wc_left_string_int] = ACTIONS(1789),
    [sym__wc_lower_string_int] = ACTIONS(1789),
    [sym__wc_mid_string_int] = ACTIONS(1789),
    [sym__wc_remove_string_int] = ACTIONS(1789),
    [sym__wc_right_string_int] = ACTIONS(1789),
    [sym__wc_upper_string_int] = ACTIONS(1789),
    [sym__wc_compare_string_int] = ACTIONS(1789),
    [sym__wc_get_buffer_char_int] = ACTIONS(1789),
    [sym__wc_get_buffer_string_int] = ACTIONS(1789),
    [sym__wc_concat_string_int] = ACTIONS(1789),
    [sym__wc_file_open_int] = ACTIONS(1789),
    [sym__wc_file_close_int] = ACTIONS(1789),
    [sym__wc_file_read_int] = ACTIONS(1789),
    [sym__wc_file_read_line_int] = ACTIONS(1789),
    [sym__wc_file_write_int] = ACTIONS(1789),
    [sym__wc_file_write_line_int] = ACTIONS(1789),
    [aux_sym_preproc_include_token1] = ACTIONS(1789),
    [aux_sym_preproc_define_token1] = ACTIONS(1791),
    [aux_sym_preproc_warn_token1] = ACTIONS(1791),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1791),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1791),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1791),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1791),
    [aux_sym_preproc_else_token1] = ACTIONS(1791),
    [anon_sym_LBRACK] = ACTIONS(1791),
    [anon_sym_LBRACE] = ACTIONS(1791),
    [anon_sym_LPAREN] = ACTIONS(1789),
    [anon_sym_BANG] = ACTIONS(1791),
    [anon_sym_TILDE] = ACTIONS(1791),
    [anon_sym_DASH] = ACTIONS(1789),
    [anon_sym_PLUS] = ACTIONS(1789),
    [anon_sym_DASH_DASH] = ACTIONS(1791),
    [anon_sym_PLUS_PLUS] = ACTIONS(1791),
    [anon_sym_DQUOTE] = ACTIONS(1791),
    [anon_sym_SQUOTE] = ACTIONS(1791),
    [sym_number_literal] = ACTIONS(1791),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(239)] = {
    [sym_identifier] = ACTIONS(1789),
    [sym_program_name_keyword] = ACTIONS(1789),
    [sym_module_name_keyword] = ACTIONS(1789),
    [sym_define_device_keyword] = ACTIONS(1789),
    [sym_define_combine_keyword] = ACTIONS(1789),
    [sym_define_constant_keyword] = ACTIONS(1789),
    [sym_define_type_keyword] = ACTIONS(1789),
    [sym_define_variable_keyword] = ACTIONS(1789),
    [sym_define_system_variable_keyword] = ACTIONS(1789),
    [sym_define_connect_level_keyword] = ACTIONS(1789),
    [sym_define_latching_keyword] = ACTIONS(1789),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1789),
    [sym_define_toggling_keyword] = ACTIONS(1789),
    [sym_define_call_keyword] = ACTIONS(1789),
    [sym_define_function_keyword] = ACTIONS(1789),
    [sym_define_library_function_keyword] = ACTIONS(1789),
    [sym_define_start_keyword] = ACTIONS(1789),
    [sym_define_event_keyword] = ACTIONS(1789),
    [sym_define_program_keyword] = ACTIONS(1789),
    [sym_define_module_keyword] = ACTIONS(1789),
    [sym_char_keyword] = ACTIONS(1789),
    [sym_widechar_keyword] = ACTIONS(1789),
    [sym_integer_keyword] = ACTIONS(1789),
    [sym_sinteger_keyword] = ACTIONS(1789),
    [sym_long_keyword] = ACTIONS(1789),
    [sym_slong_keyword] = ACTIONS(1789),
    [sym_float_keyword] = ACTIONS(1789),
    [sym_double_keyword] = ACTIONS(1789),
    [sym_variant_keyword] = ACTIONS(1789),
    [sym_variantarray_keyword] = ACTIONS(1789),
    [sym_local_var_keyword] = ACTIONS(1789),
    [sym_stack_var_keyword] = ACTIONS(1789),
    [sym_constant_keyword] = ACTIONS(1789),
    [sym_volatile_keyword] = ACTIONS(1789),
    [sym_non_volatile_keyword] = ACTIONS(1789),
    [sym_persistent_keyword] = ACTIONS(1789),
    [sym_dev_keyword] = ACTIONS(1789),
    [sym_devlev_keyword] = ACTIONS(1789),
    [sym_devchan_keyword] = ACTIONS(1789),
    [sym_if_keyword] = ACTIONS(1789),
    [sym_else_keyword] = ACTIONS(1789),
    [sym_switch_keyword] = ACTIONS(1789),
    [sym_case_keyword] = ACTIONS(1789),
    [sym_default_keyword] = ACTIONS(1789),
    [sym_while_keyword] = ACTIONS(1789),
    [sym_for_keyword] = ACTIONS(1789),
    [sym_break_keyword] = ACTIONS(1789),
    [sym_continue_keyword] = ACTIONS(1789),
    [sym_return_keyword] = ACTIONS(1789),
    [sym_select_keyword] = ACTIONS(1789),
    [sym_active_keyword] = ACTIONS(1789),
    [sym_struct_keyword] = ACTIONS(1789),
    [sym_structure_keyword] = ACTIONS(1789),
    [sym_button_event_keyword] = ACTIONS(1789),
    [sym_channel_event_keyword] = ACTIONS(1789),
    [sym_level_event_keyword] = ACTIONS(1789),
    [sym_data_event_keyword] = ACTIONS(1789),
    [sym_timeline_event_keyword] = ACTIONS(1789),
    [sym_custom_event_keyword] = ACTIONS(1789),
    [sym_push_keyword] = ACTIONS(1789),
    [sym_release_keyword] = ACTIONS(1789),
    [aux_sym_on_keyword_token1] = ACTIONS(1789),
    [aux_sym_off_keyword_token1] = ACTIONS(1789),
    [sym_send_level_keyword] = ACTIONS(1789),
    [sym_send_string_keyword] = ACTIONS(1789),
    [sym_send_command_keyword] = ACTIONS(1789),
    [sym_clear_buffer_keyword] = ACTIONS(1789),
    [sym_create_buffer_keyword] = ACTIONS(1789),
    [sym_create_multi_buffer_keyword] = ACTIONS(1789),
    [sym_call_keyword] = ACTIONS(1789),
    [sym_system_call_keyword] = ACTIONS(1789),
    [sym_devchan_to_keyword] = ACTIONS(1789),
    [sym_devchan_min_to_keyword] = ACTIONS(1789),
    [sym_devchan_total_off_keyword] = ACTIONS(1789),
    [sym_devchan_pulse_keyword] = ACTIONS(1789),
    [sym_wait_keyword] = ACTIONS(1789),
    [sym_cancel_wait_keyword] = ACTIONS(1789),
    [sym_wait_until_keyword] = ACTIONS(1789),
    [sym_cancel_wait_until_keyword] = ACTIONS(1789),
    [sym_cancel_all_wait_keyword] = ACTIONS(1789),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1789),
    [sym_not] = ACTIONS(1789),
    [sym_bnot] = ACTIONS(1789),
    [sym___file__] = ACTIONS(1789),
    [sym___date__] = ACTIONS(1789),
    [sym___ldate__] = ACTIONS(1789),
    [sym___time__] = ACTIONS(1789),
    [sym___line__] = ACTIONS(1789),
    [sym___name__] = ACTIONS(1789),
    [sym___netlinx__] = ACTIONS(1789),
    [sym_netlinx_axi_version] = ACTIONS(1789),
    [sym_url_flg_tcp] = ACTIONS(1789),
    [sym_url_flg_accinfopresent] = ACTIONS(1789),
    [sym_url_flg_temp] = ACTIONS(1789),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1789),
    [sym_url_flg_stat_mask] = ACTIONS(1789),
    [sym_url_flg_stat_lookup] = ACTIONS(1789),
    [sym_url_flg_stat_connecting] = ACTIONS(1789),
    [sym_url_flg_stat_waiting] = ACTIONS(1789),
    [sym_url_flg_stat_connected] = ACTIONS(1789),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1789),
    [sym_first_virtual_device] = ACTIONS(1789),
    [sym_first_local_port] = ACTIONS(1789),
    [sym_dynamic_virtual_device] = ACTIONS(1789),
    [sym_timeline_once] = ACTIONS(1789),
    [sym_timeline_repeat] = ACTIONS(1789),
    [sym_timeline_absolute] = ACTIONS(1789),
    [sym_timeline_relative] = ACTIONS(1789),
    [sym_do_push_timed_infinite] = ACTIONS(1789),
    [sym_source_type_no_address] = ACTIONS(1789),
    [sym_source_type_neuron_id] = ACTIONS(1789),
    [sym_source_type_ip_address] = ACTIONS(1789),
    [sym_source_type_axlink] = ACTIONS(1789),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1789),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1789),
    [sym_source_type_ip_socket_address] = ACTIONS(1789),
    [sym_source_type_rs232] = ACTIONS(1789),
    [sym_source_type_internal] = ACTIONS(1789),
    [sym_source_type_mac_address] = ACTIONS(1789),
    [sym_source_type_ipv4_port] = ACTIONS(1789),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1789),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1789),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1789),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1789),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1789),
    [sym_true] = ACTIONS(1789),
    [sym_false] = ACTIONS(1789),
    [sym_file_read_only] = ACTIONS(1789),
    [sym_file_rw_new] = ACTIONS(1789),
    [sym_file_rw_append] = ACTIONS(1789),
    [sym_ip_tcp] = ACTIONS(1789),
    [sym_ip_udp] = ACTIONS(1789),
    [sym_ip_udp_2way] = ACTIONS(1789),
    [sym_xml_encode_types] = ACTIONS(1789),
    [sym_xml_encode_char_as_list] = ACTIONS(1789),
    [sym_xml_encode_le] = ACTIONS(1789),
    [sym_xml_decode_types] = ACTIONS(1789),
    [sym_xml_decode_no_preserve] = ACTIONS(1789),
    [sym_duet_dev_type_display_device] = ACTIONS(1789),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1789),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1789),
    [sym_duet_dev_type_io_device] = ACTIONS(1789),
    [sym_duet_dev_type_relay_device] = ACTIONS(1789),
    [sym_duet_dev_type_ups] = ACTIONS(1789),
    [sym_duet_dev_type_amplifier] = ACTIONS(1789),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1789),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1789),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1789),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1789),
    [sym_duet_dev_type_camera] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1789),
    [sym_duet_dev_type_dss] = ACTIONS(1789),
    [sym_duet_dev_type_dvr] = ACTIONS(1789),
    [sym_duet_dev_type_disc_device] = ACTIONS(1789),
    [sym_duet_dev_type_document_camera] = ACTIONS(1789),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1789),
    [sym_duet_dev_type_hvac] = ACTIONS(1789),
    [sym_duet_dev_type_keypad] = ACTIONS(1789),
    [sym_duet_dev_type_light] = ACTIONS(1789),
    [sym_duet_dev_type_monitor] = ACTIONS(1789),
    [sym_duet_dev_type_motor] = ACTIONS(1789),
    [sym_duet_dev_type_multi_window] = ACTIONS(1789),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1789),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1789),
    [sym_duet_dev_type_receiver] = ACTIONS(1789),
    [sym_duet_dev_type_security_system] = ACTIONS(1789),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1789),
    [sym_duet_dev_type_settop_box] = ACTIONS(1789),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1789),
    [sym_duet_dev_type_switcher] = ACTIONS(1789),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1789),
    [sym_duet_dev_type_tv] = ACTIONS(1789),
    [sym_duet_dev_type_utility] = ACTIONS(1789),
    [sym_duet_dev_type_vcr] = ACTIONS(1789),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1789),
    [sym_duet_dev_type_video_processor] = ACTIONS(1789),
    [sym_duet_dev_type_video_projector] = ACTIONS(1789),
    [sym_duet_dev_type_video_wall] = ACTIONS(1789),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1789),
    [sym_duet_dev_type_weather] = ACTIONS(1789),
    [sym_duet_dev_not_polled] = ACTIONS(1789),
    [sym_duet_dev_polled] = ACTIONS(1789),
    [sym_clkmgr_mode_network] = ACTIONS(1789),
    [sym_clkmgr_mode_standalone] = ACTIONS(1789),
    [sym_normal_standby] = ACTIONS(1789),
    [sym_normal_wake] = ACTIONS(1789),
    [sym_ip_multicast_ttl_option] = ACTIONS(1789),
    [sym_ip_ttl_subnet] = ACTIONS(1789),
    [sym_ip_ttl_site] = ACTIONS(1789),
    [sym_ip_ttl_region] = ACTIONS(1789),
    [sym_ip_ttl_continent] = ACTIONS(1789),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1789),
    [sym_ip_nodelay_on] = ACTIONS(1789),
    [sym_ip_nodelay_off] = ACTIONS(1789),
    [sym_amx_error] = ACTIONS(1789),
    [sym_amx_warning] = ACTIONS(1789),
    [sym_amx_info] = ACTIONS(1789),
    [sym_amx_debug] = ACTIONS(1789),
    [sym_netlinx_login_success] = ACTIONS(1789),
    [sym_netlinx_login_fail] = ACTIONS(1789),
    [sym_netlinx_logout] = ACTIONS(1789),
    [sym_null_str] = ACTIONS(1789),
    [sym_smtp_address] = ACTIONS(1789),
    [sym_smtp_port_number] = ACTIONS(1789),
    [sym_smtp_username] = ACTIONS(1789),
    [sym_smtp_password] = ACTIONS(1789),
    [sym_smtp_require_tls] = ACTIONS(1789),
    [sym_smtp_from] = ACTIONS(1789),
    [sym_smtp_tls_true] = ACTIONS(1789),
    [sym_smtp_tls_false] = ACTIONS(1789),
    [sym_listview_on_row_select_event] = ACTIONS(1789),
    [sym_data_structure_unknown] = ACTIONS(1789),
    [sym_data_structure_datafeed] = ACTIONS(1789),
    [sym_data_structure_datarecordset] = ACTIONS(1789),
    [sym_data_structure_datarecord] = ACTIONS(1789),
    [sym_data_structure_datafield] = ACTIONS(1789),
    [sym_data_max_content_field_count] = ACTIONS(1789),
    [sym_data_max_metadata_field_count] = ACTIONS(1789),
    [sym_data_max_id_length] = ACTIONS(1789),
    [sym_data_max_name_length] = ACTIONS(1789),
    [sym_data_max_description_length] = ACTIONS(1789),
    [sym_data_max_type_length] = ACTIONS(1789),
    [sym_data_max_format_length] = ACTIONS(1789),
    [sym_data_max_source_length] = ACTIONS(1789),
    [sym_data_max_value_length] = ACTIONS(1789),
    [sym_data_max_label_length] = ACTIONS(1789),
    [sym_data_max_url_length] = ACTIONS(1789),
    [sym_max_last_login_info_length] = ACTIONS(1789),
    [sym_data_type_unknown] = ACTIONS(1789),
    [sym_data_type_string] = ACTIONS(1789),
    [sym_data_type_datetime] = ACTIONS(1789),
    [sym_data_type_date] = ACTIONS(1789),
    [sym_data_type_time] = ACTIONS(1789),
    [sym_data_type_image] = ACTIONS(1789),
    [sym_data_format_url] = ACTIONS(1789),
    [sym_data_format_phone] = ACTIONS(1789),
    [sym_data_format_email] = ACTIONS(1789),
    [sym_data_format_iso8601] = ACTIONS(1789),
    [sym_auditlog_priv] = ACTIONS(1789),
    [sym_configuration_priv] = ACTIONS(1789),
    [sym_deviceconfig_priv] = ACTIONS(1789),
    [sym_ftp_priv] = ACTIONS(1789),
    [sym_http_priv] = ACTIONS(1789),
    [sym_networkconfig_priv] = ACTIONS(1789),
    [sym_programport_priv] = ACTIONS(1789),
    [sym_remoteui_priv] = ACTIONS(1789),
    [sym_securitycontrol_priv] = ACTIONS(1789),
    [sym_softwaremanagement_priv] = ACTIONS(1789),
    [sym_terminal_priv] = ACTIONS(1789),
    [sym_tpadmin_priv] = ACTIONS(1789),
    [sym_user1_priv] = ACTIONS(1789),
    [sym_user2_priv] = ACTIONS(1789),
    [sym_user3_priv] = ACTIONS(1789),
    [sym_user4_priv] = ACTIONS(1789),
    [sym_usermanagement_priv] = ACTIONS(1789),
    [sym_valid_account] = ACTIONS(1789),
    [sym_err_name_invalid] = ACTIONS(1789),
    [sym_err_password_invalid] = ACTIONS(1789),
    [sym_err_invalid_account] = ACTIONS(1789),
    [sym_err_no_authorization] = ACTIONS(1789),
    [sym_err_invalid_parameter] = ACTIONS(1789),
    [sym_err_user_locked_out] = ACTIONS(1789),
    [sym_err_user_account_expired] = ACTIONS(1789),
    [sym_err_general_error] = ACTIONS(1789),
    [sym_err_authentication_error] = ACTIONS(1789),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1789),
    [sym_tls_validate_certificate] = ACTIONS(1789),
    [sym_tdata] = ACTIONS(1789),
    [sym_tchannel] = ACTIONS(1789),
    [sym_tlevel] = ACTIONS(1789),
    [sym_tbutton] = ACTIONS(1789),
    [sym_ttimeline] = ACTIONS(1789),
    [sym_tcustom] = ACTIONS(1789),
    [sym_url_struct] = ACTIONS(1789),
    [sym_dns_struct] = ACTIONS(1789),
    [sym_ip_address_struct] = ACTIONS(1789),
    [sym_dev_info_struct] = ACTIONS(1789),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1789),
    [sym_data_feed] = ACTIONS(1789),
    [sym_data_field] = ACTIONS(1789),
    [sym_data_record] = ACTIONS(1789),
    [sym_wc_data_feed] = ACTIONS(1789),
    [sym_wc_data_field] = ACTIONS(1789),
    [sym_wc_data_record] = ACTIONS(1789),
    [sym_last_login_info] = ACTIONS(1789),
    [sym____reserved___] = ACTIONS(1789),
    [sym_date] = ACTIONS(1789),
    [sym_day] = ACTIONS(1789),
    [sym_time] = ACTIONS(1789),
    [sym_ldate] = ACTIONS(1789),
    [sym_push_device] = ACTIONS(1789),
    [sym_push_channel] = ACTIONS(1789),
    [sym_push_devchan] = ACTIONS(1789),
    [sym_release_device] = ACTIONS(1789),
    [sym_release_channel] = ACTIONS(1789),
    [sym_release_devchan] = ACTIONS(1789),
    [sym_master_slot] = ACTIONS(1789),
    [sym_get_pulse_time] = ACTIONS(1789),
    [sym_get_timer] = ACTIONS(1789),
    [sym_master_sn] = ACTIONS(1789),
    [sym_system_number] = ACTIONS(1789),
    [sym_button] = ACTIONS(1789),
    [sym_dv_channel] = ACTIONS(1789),
    [sym_data] = ACTIONS(1789),
    [sym_level] = ACTIONS(1789),
    [sym_timeline] = ACTIONS(1789),
    [sym_channel] = ACTIONS(1789),
    [sym_custom] = ACTIONS(1789),
    [sym_atoi] = ACTIONS(1789),
    [sym_atol] = ACTIONS(1789),
    [sym_atof] = ACTIONS(1789),
    [sym_itoa] = ACTIONS(1789),
    [sym_itohex] = ACTIONS(1789),
    [sym_hextoi] = ACTIONS(1789),
    [sym_ftoa] = ACTIONS(1789),
    [sym_find_string] = ACTIONS(1789),
    [sym_left_string] = ACTIONS(1789),
    [sym_right_string] = ACTIONS(1789),
    [sym_mid_string] = ACTIONS(1789),
    [sym_remove_string] = ACTIONS(1789),
    [sym_get_buffer_string] = ACTIONS(1789),
    [sym_get_buffer_char] = ACTIONS(1789),
    [sym_get_multi_buffer_string] = ACTIONS(1789),
    [sym_length_string] = ACTIONS(1789),
    [sym_max_length_string] = ACTIONS(1789),
    [sym_set_length_string] = ACTIONS(1789),
    [sym_compare_string] = ACTIONS(1789),
    [sym_upper_string] = ACTIONS(1789),
    [sym_lower_string] = ACTIONS(1789),
    [sym_get_last] = ACTIONS(1789),
    [sym_date_to_day] = ACTIONS(1789),
    [sym_date_to_month] = ACTIONS(1789),
    [sym_date_to_year] = ACTIONS(1789),
    [sym_time_to_hour] = ACTIONS(1789),
    [sym_time_to_minute] = ACTIONS(1789),
    [sym_time_to_second] = ACTIONS(1789),
    [sym_day_of_week] = ACTIONS(1789),
    [sym_set_pulse_time] = ACTIONS(1789),
    [sym_set_timer] = ACTIONS(1789),
    [sym_file_open] = ACTIONS(1789),
    [sym_file_close] = ACTIONS(1789),
    [sym_file_read] = ACTIONS(1789),
    [sym_file_read_line] = ACTIONS(1789),
    [sym_file_write] = ACTIONS(1789),
    [sym_file_write_line] = ACTIONS(1789),
    [sym_file_copy] = ACTIONS(1789),
    [sym_file_delete] = ACTIONS(1789),
    [sym_file_rename] = ACTIONS(1789),
    [sym_file_dir] = ACTIONS(1789),
    [sym_file_getdir] = ACTIONS(1789),
    [sym_file_setdir] = ACTIONS(1789),
    [sym_file_create_dir] = ACTIONS(1789),
    [sym_file_remove_dir] = ACTIONS(1789),
    [sym_file_seek] = ACTIONS(1789),
    [sym_length_array] = ACTIONS(1789),
    [sym_max_length_array] = ACTIONS(1789),
    [sym_set_length_array] = ACTIONS(1789),
    [sym_ip_client_open] = ACTIONS(1789),
    [sym_ip_client_close] = ACTIONS(1789),
    [sym_ip_server_open] = ACTIONS(1789),
    [sym_ip_server_close] = ACTIONS(1789),
    [sym_ip_bound_client_open] = ACTIONS(1789),
    [sym_ip_mc_server_open] = ACTIONS(1789),
    [sym_ip_set_option] = ACTIONS(1789),
    [sym_ssh_client_open] = ACTIONS(1789),
    [sym_ssh_client_close] = ACTIONS(1789),
    [sym_tls_client_open] = ACTIONS(1789),
    [sym_tls_client_close] = ACTIONS(1789),
    [sym_do_push] = ACTIONS(1789),
    [sym_do_push_timed] = ACTIONS(1789),
    [sym_do_release] = ACTIONS(1789),
    [sym_device_id] = ACTIONS(1789),
    [sym_device_info] = ACTIONS(1789),
    [sym_device_id_string] = ACTIONS(1789),
    [sym_device_standby] = ACTIONS(1789),
    [sym_device_wake] = ACTIONS(1789),
    [sym_timeline_create] = ACTIONS(1789),
    [sym_timeline_kill] = ACTIONS(1789),
    [sym_timeline_reload] = ACTIONS(1789),
    [sym_timeline_pause] = ACTIONS(1789),
    [sym_timeline_restart] = ACTIONS(1789),
    [sym_timeline_set] = ACTIONS(1789),
    [sym_timeline_get] = ACTIONS(1789),
    [sym_timeline_active] = ACTIONS(1789),
    [sym_timeline_dynamic_id] = ACTIONS(1789),
    [sym_variable_to_string] = ACTIONS(1789),
    [sym_string_to_variable] = ACTIONS(1789),
    [sym_length_variable_to_string] = ACTIONS(1789),
    [sym_type_cast] = ACTIONS(1789),
    [sym_variable_to_xml] = ACTIONS(1789),
    [sym_xml_to_variable] = ACTIONS(1789),
    [sym_length_variable_to_xml] = ACTIONS(1789),
    [sym_redirect_string] = ACTIONS(1789),
    [sym_random_number] = ACTIONS(1789),
    [sym_get_unique_id] = ACTIONS(1789),
    [sym_get_system_number] = ACTIONS(1789),
    [sym_set_system_number] = ACTIONS(1789),
    [sym_get_serial_number] = ACTIONS(1789),
    [sym_get_master_build] = ACTIONS(1789),
    [sym_set_validation_code] = ACTIONS(1789),
    [sym_reboot] = ACTIONS(1789),
    [sym_load_duet_module] = ACTIONS(1789),
    [sym_unload_duet_module] = ACTIONS(1789),
    [sym_duet_mem_size_set] = ACTIONS(1789),
    [sym_duet_mem_size_get] = ACTIONS(1789),
    [sym_dynamic_application_device] = ACTIONS(1789),
    [sym_dynamic_polled_port] = ACTIONS(1789),
    [sym_static_port_binding] = ACTIONS(1789),
    [sym_static_ip_binding] = ACTIONS(1789),
    [sym_get_dns_list] = ACTIONS(1789),
    [sym_set_dns_list] = ACTIONS(1789),
    [sym_get_ip_address] = ACTIONS(1789),
    [sym_set_ip_address] = ACTIONS(1789),
    [sym_get_url_list] = ACTIONS(1789),
    [sym_add_url_entry] = ACTIONS(1789),
    [sym_delete_url_entry] = ACTIONS(1789),
    [sym_abs_value] = ACTIONS(1789),
    [sym_max_value] = ACTIONS(1789),
    [sym_min_value] = ACTIONS(1789),
    [sym_exp_value] = ACTIONS(1789),
    [sym_log_value] = ACTIONS(1789),
    [sym_log10_value] = ACTIONS(1789),
    [sym_power_value] = ACTIONS(1789),
    [sym_sqrt_value] = ACTIONS(1789),
    [sym_raw_be] = ACTIONS(1789),
    [sym_raw_le] = ACTIONS(1789),
    [sym_internal_queue_size_set] = ACTIONS(1789),
    [sym_internal_queue_size_get] = ACTIONS(1789),
    [sym_internal_threshold_set] = ACTIONS(1789),
    [sym_internal_threshold_get] = ACTIONS(1789),
    [sym_rebuilt_event] = ACTIONS(1789),
    [sym_do_custom_event] = ACTIONS(1789),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1789),
    [sym_clkmgr_set_clk_source] = ACTIONS(1789),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1789),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1789),
    [sym_clkmgr_get_timezone] = ACTIONS(1789),
    [sym_clkmgr_set_timezone] = ACTIONS(1789),
    [sym_clkmgr_get_resync_period] = ACTIONS(1789),
    [sym_clkmgr_set_resync_period] = ACTIONS(1789),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1789),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1789),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1789),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1789),
    [sym_clkmgr_get_timeservers] = ACTIONS(1789),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1789),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1789),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1789),
    [sym_set_log_level] = ACTIONS(1789),
    [sym_get_log_level] = ACTIONS(1789),
    [sym_amx_log] = ACTIONS(1789),
    [sym_smtp_server_config_set] = ACTIONS(1789),
    [sym_smtp_server_config_get] = ACTIONS(1789),
    [sym_smtp_send] = ACTIONS(1789),
    [sym_validate_netlinx_account] = ACTIONS(1789),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1789),
    [sym_authenticate_certificate] = ACTIONS(1789),
    [sym_audit_netlinx_session_event] = ACTIONS(1789),
    [sym_audit_netlinx_generic_event] = ACTIONS(1789),
    [sym_get_available_flash_disk_space] = ACTIONS(1789),
    [sym_get_max_flash_disk_space] = ACTIONS(1789),
    [sym_data_create_feed] = ACTIONS(1789),
    [sym_data_delete_feed] = ACTIONS(1789),
    [sym_data_publish_feed] = ACTIONS(1789),
    [sym_data_get_published_feed] = ACTIONS(1789),
    [sym_data_add_record] = ACTIONS(1789),
    [sym_data_get_event_record] = ACTIONS(1789),
    [sym__wc_data_create_feed] = ACTIONS(1789),
    [sym__wc_data_add_record] = ACTIONS(1789),
    [sym__wc_data_get_event_record] = ACTIONS(1789),
    [sym_format] = ACTIONS(1789),
    [sym_astro_clock] = ACTIONS(1789),
    [sym_set_outdoor_temperature] = ACTIONS(1789),
    [sym_set_virtual_level_count] = ACTIONS(1789),
    [sym_set_virtual_channel_count] = ACTIONS(1789),
    [sym_set_virtual_port_count] = ACTIONS(1789),
    [sym__wc_encode_int] = ACTIONS(1789),
    [sym__wc_decode_int] = ACTIONS(1789),
    [sym__wc_int] = ACTIONS(1789),
    [sym__wc_to_ch_int] = ACTIONS(1789),
    [sym__ch_to_wc_int] = ACTIONS(1789),
    [sym__wc_find_string_int] = ACTIONS(1789),
    [sym__wc_left_string_int] = ACTIONS(1789),
    [sym__wc_lower_string_int] = ACTIONS(1789),
    [sym__wc_mid_string_int] = ACTIONS(1789),
    [sym__wc_remove_string_int] = ACTIONS(1789),
    [sym__wc_right_string_int] = ACTIONS(1789),
    [sym__wc_upper_string_int] = ACTIONS(1789),
    [sym__wc_compare_string_int] = ACTIONS(1789),
    [sym__wc_get_buffer_char_int] = ACTIONS(1789),
    [sym__wc_get_buffer_string_int] = ACTIONS(1789),
    [sym__wc_concat_string_int] = ACTIONS(1789),
    [sym__wc_file_open_int] = ACTIONS(1789),
    [sym__wc_file_close_int] = ACTIONS(1789),
    [sym__wc_file_read_int] = ACTIONS(1789),
    [sym__wc_file_read_line_int] = ACTIONS(1789),
    [sym__wc_file_write_int] = ACTIONS(1789),
    [sym__wc_file_write_line_int] = ACTIONS(1789),
    [aux_sym_preproc_include_token1] = ACTIONS(1789),
    [aux_sym_preproc_define_token1] = ACTIONS(1791),
    [aux_sym_preproc_warn_token1] = ACTIONS(1791),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1791),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1791),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1791),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1791),
    [aux_sym_preproc_else_token1] = ACTIONS(1791),
    [anon_sym_LBRACK] = ACTIONS(1791),
    [anon_sym_LBRACE] = ACTIONS(1791),
    [anon_sym_LPAREN] = ACTIONS(1789),
    [anon_sym_BANG] = ACTIONS(1791),
    [anon_sym_TILDE] = ACTIONS(1791),
    [anon_sym_DASH] = ACTIONS(1789),
    [anon_sym_PLUS] = ACTIONS(1789),
    [anon_sym_DASH_DASH] = ACTIONS(1791),
    [anon_sym_PLUS_PLUS] = ACTIONS(1791),
    [anon_sym_DQUOTE] = ACTIONS(1791),
    [anon_sym_SQUOTE] = ACTIONS(1791),
    [sym_number_literal] = ACTIONS(1791),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(240)] = {
    [sym_identifier] = ACTIONS(1793),
    [sym_program_name_keyword] = ACTIONS(1793),
    [sym_module_name_keyword] = ACTIONS(1793),
    [sym_define_device_keyword] = ACTIONS(1793),
    [sym_define_combine_keyword] = ACTIONS(1793),
    [sym_define_constant_keyword] = ACTIONS(1793),
    [sym_define_type_keyword] = ACTIONS(1793),
    [sym_define_variable_keyword] = ACTIONS(1793),
    [sym_define_system_variable_keyword] = ACTIONS(1793),
    [sym_define_connect_level_keyword] = ACTIONS(1793),
    [sym_define_latching_keyword] = ACTIONS(1793),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1793),
    [sym_define_toggling_keyword] = ACTIONS(1793),
    [sym_define_call_keyword] = ACTIONS(1793),
    [sym_define_function_keyword] = ACTIONS(1793),
    [sym_define_library_function_keyword] = ACTIONS(1793),
    [sym_define_start_keyword] = ACTIONS(1793),
    [sym_define_event_keyword] = ACTIONS(1793),
    [sym_define_program_keyword] = ACTIONS(1793),
    [sym_define_module_keyword] = ACTIONS(1793),
    [sym_char_keyword] = ACTIONS(1793),
    [sym_widechar_keyword] = ACTIONS(1793),
    [sym_integer_keyword] = ACTIONS(1793),
    [sym_sinteger_keyword] = ACTIONS(1793),
    [sym_long_keyword] = ACTIONS(1793),
    [sym_slong_keyword] = ACTIONS(1793),
    [sym_float_keyword] = ACTIONS(1793),
    [sym_double_keyword] = ACTIONS(1793),
    [sym_variant_keyword] = ACTIONS(1793),
    [sym_variantarray_keyword] = ACTIONS(1793),
    [sym_local_var_keyword] = ACTIONS(1793),
    [sym_stack_var_keyword] = ACTIONS(1793),
    [sym_constant_keyword] = ACTIONS(1793),
    [sym_volatile_keyword] = ACTIONS(1793),
    [sym_non_volatile_keyword] = ACTIONS(1793),
    [sym_persistent_keyword] = ACTIONS(1793),
    [sym_dev_keyword] = ACTIONS(1793),
    [sym_devlev_keyword] = ACTIONS(1793),
    [sym_devchan_keyword] = ACTIONS(1793),
    [sym_if_keyword] = ACTIONS(1793),
    [sym_else_keyword] = ACTIONS(1793),
    [sym_switch_keyword] = ACTIONS(1793),
    [sym_case_keyword] = ACTIONS(1793),
    [sym_default_keyword] = ACTIONS(1793),
    [sym_while_keyword] = ACTIONS(1793),
    [sym_for_keyword] = ACTIONS(1793),
    [sym_break_keyword] = ACTIONS(1793),
    [sym_continue_keyword] = ACTIONS(1793),
    [sym_return_keyword] = ACTIONS(1793),
    [sym_select_keyword] = ACTIONS(1793),
    [sym_active_keyword] = ACTIONS(1793),
    [sym_struct_keyword] = ACTIONS(1793),
    [sym_structure_keyword] = ACTIONS(1793),
    [sym_button_event_keyword] = ACTIONS(1793),
    [sym_channel_event_keyword] = ACTIONS(1793),
    [sym_level_event_keyword] = ACTIONS(1793),
    [sym_data_event_keyword] = ACTIONS(1793),
    [sym_timeline_event_keyword] = ACTIONS(1793),
    [sym_custom_event_keyword] = ACTIONS(1793),
    [sym_push_keyword] = ACTIONS(1793),
    [sym_release_keyword] = ACTIONS(1793),
    [aux_sym_on_keyword_token1] = ACTIONS(1793),
    [aux_sym_off_keyword_token1] = ACTIONS(1793),
    [sym_send_level_keyword] = ACTIONS(1793),
    [sym_send_string_keyword] = ACTIONS(1793),
    [sym_send_command_keyword] = ACTIONS(1793),
    [sym_clear_buffer_keyword] = ACTIONS(1793),
    [sym_create_buffer_keyword] = ACTIONS(1793),
    [sym_create_multi_buffer_keyword] = ACTIONS(1793),
    [sym_call_keyword] = ACTIONS(1793),
    [sym_system_call_keyword] = ACTIONS(1793),
    [sym_devchan_to_keyword] = ACTIONS(1793),
    [sym_devchan_min_to_keyword] = ACTIONS(1793),
    [sym_devchan_total_off_keyword] = ACTIONS(1793),
    [sym_devchan_pulse_keyword] = ACTIONS(1793),
    [sym_wait_keyword] = ACTIONS(1793),
    [sym_cancel_wait_keyword] = ACTIONS(1793),
    [sym_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1793),
    [sym_not] = ACTIONS(1793),
    [sym_bnot] = ACTIONS(1793),
    [sym___file__] = ACTIONS(1793),
    [sym___date__] = ACTIONS(1793),
    [sym___ldate__] = ACTIONS(1793),
    [sym___time__] = ACTIONS(1793),
    [sym___line__] = ACTIONS(1793),
    [sym___name__] = ACTIONS(1793),
    [sym___netlinx__] = ACTIONS(1793),
    [sym_netlinx_axi_version] = ACTIONS(1793),
    [sym_url_flg_tcp] = ACTIONS(1793),
    [sym_url_flg_accinfopresent] = ACTIONS(1793),
    [sym_url_flg_temp] = ACTIONS(1793),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1793),
    [sym_url_flg_stat_mask] = ACTIONS(1793),
    [sym_url_flg_stat_lookup] = ACTIONS(1793),
    [sym_url_flg_stat_connecting] = ACTIONS(1793),
    [sym_url_flg_stat_waiting] = ACTIONS(1793),
    [sym_url_flg_stat_connected] = ACTIONS(1793),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1793),
    [sym_first_virtual_device] = ACTIONS(1793),
    [sym_first_local_port] = ACTIONS(1793),
    [sym_dynamic_virtual_device] = ACTIONS(1793),
    [sym_timeline_once] = ACTIONS(1793),
    [sym_timeline_repeat] = ACTIONS(1793),
    [sym_timeline_absolute] = ACTIONS(1793),
    [sym_timeline_relative] = ACTIONS(1793),
    [sym_do_push_timed_infinite] = ACTIONS(1793),
    [sym_source_type_no_address] = ACTIONS(1793),
    [sym_source_type_neuron_id] = ACTIONS(1793),
    [sym_source_type_ip_address] = ACTIONS(1793),
    [sym_source_type_axlink] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1793),
    [sym_source_type_ip_socket_address] = ACTIONS(1793),
    [sym_source_type_rs232] = ACTIONS(1793),
    [sym_source_type_internal] = ACTIONS(1793),
    [sym_source_type_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1793),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1793),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1793),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1793),
    [sym_true] = ACTIONS(1793),
    [sym_false] = ACTIONS(1793),
    [sym_file_read_only] = ACTIONS(1793),
    [sym_file_rw_new] = ACTIONS(1793),
    [sym_file_rw_append] = ACTIONS(1793),
    [sym_ip_tcp] = ACTIONS(1793),
    [sym_ip_udp] = ACTIONS(1793),
    [sym_ip_udp_2way] = ACTIONS(1793),
    [sym_xml_encode_types] = ACTIONS(1793),
    [sym_xml_encode_char_as_list] = ACTIONS(1793),
    [sym_xml_encode_le] = ACTIONS(1793),
    [sym_xml_decode_types] = ACTIONS(1793),
    [sym_xml_decode_no_preserve] = ACTIONS(1793),
    [sym_duet_dev_type_display_device] = ACTIONS(1793),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_io_device] = ACTIONS(1793),
    [sym_duet_dev_type_relay_device] = ACTIONS(1793),
    [sym_duet_dev_type_ups] = ACTIONS(1793),
    [sym_duet_dev_type_amplifier] = ACTIONS(1793),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1793),
    [sym_duet_dev_type_camera] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1793),
    [sym_duet_dev_type_dss] = ACTIONS(1793),
    [sym_duet_dev_type_dvr] = ACTIONS(1793),
    [sym_duet_dev_type_disc_device] = ACTIONS(1793),
    [sym_duet_dev_type_document_camera] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1793),
    [sym_duet_dev_type_hvac] = ACTIONS(1793),
    [sym_duet_dev_type_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_light] = ACTIONS(1793),
    [sym_duet_dev_type_monitor] = ACTIONS(1793),
    [sym_duet_dev_type_motor] = ACTIONS(1793),
    [sym_duet_dev_type_multi_window] = ACTIONS(1793),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1793),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1793),
    [sym_duet_dev_type_receiver] = ACTIONS(1793),
    [sym_duet_dev_type_security_system] = ACTIONS(1793),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1793),
    [sym_duet_dev_type_settop_box] = ACTIONS(1793),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1793),
    [sym_duet_dev_type_switcher] = ACTIONS(1793),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_tv] = ACTIONS(1793),
    [sym_duet_dev_type_utility] = ACTIONS(1793),
    [sym_duet_dev_type_vcr] = ACTIONS(1793),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_video_processor] = ACTIONS(1793),
    [sym_duet_dev_type_video_projector] = ACTIONS(1793),
    [sym_duet_dev_type_video_wall] = ACTIONS(1793),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1793),
    [sym_duet_dev_type_weather] = ACTIONS(1793),
    [sym_duet_dev_not_polled] = ACTIONS(1793),
    [sym_duet_dev_polled] = ACTIONS(1793),
    [sym_clkmgr_mode_network] = ACTIONS(1793),
    [sym_clkmgr_mode_standalone] = ACTIONS(1793),
    [sym_normal_standby] = ACTIONS(1793),
    [sym_normal_wake] = ACTIONS(1793),
    [sym_ip_multicast_ttl_option] = ACTIONS(1793),
    [sym_ip_ttl_subnet] = ACTIONS(1793),
    [sym_ip_ttl_site] = ACTIONS(1793),
    [sym_ip_ttl_region] = ACTIONS(1793),
    [sym_ip_ttl_continent] = ACTIONS(1793),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1793),
    [sym_ip_nodelay_on] = ACTIONS(1793),
    [sym_ip_nodelay_off] = ACTIONS(1793),
    [sym_amx_error] = ACTIONS(1793),
    [sym_amx_warning] = ACTIONS(1793),
    [sym_amx_info] = ACTIONS(1793),
    [sym_amx_debug] = ACTIONS(1793),
    [sym_netlinx_login_success] = ACTIONS(1793),
    [sym_netlinx_login_fail] = ACTIONS(1793),
    [sym_netlinx_logout] = ACTIONS(1793),
    [sym_null_str] = ACTIONS(1793),
    [sym_smtp_address] = ACTIONS(1793),
    [sym_smtp_port_number] = ACTIONS(1793),
    [sym_smtp_username] = ACTIONS(1793),
    [sym_smtp_password] = ACTIONS(1793),
    [sym_smtp_require_tls] = ACTIONS(1793),
    [sym_smtp_from] = ACTIONS(1793),
    [sym_smtp_tls_true] = ACTIONS(1793),
    [sym_smtp_tls_false] = ACTIONS(1793),
    [sym_listview_on_row_select_event] = ACTIONS(1793),
    [sym_data_structure_unknown] = ACTIONS(1793),
    [sym_data_structure_datafeed] = ACTIONS(1793),
    [sym_data_structure_datarecordset] = ACTIONS(1793),
    [sym_data_structure_datarecord] = ACTIONS(1793),
    [sym_data_structure_datafield] = ACTIONS(1793),
    [sym_data_max_content_field_count] = ACTIONS(1793),
    [sym_data_max_metadata_field_count] = ACTIONS(1793),
    [sym_data_max_id_length] = ACTIONS(1793),
    [sym_data_max_name_length] = ACTIONS(1793),
    [sym_data_max_description_length] = ACTIONS(1793),
    [sym_data_max_type_length] = ACTIONS(1793),
    [sym_data_max_format_length] = ACTIONS(1793),
    [sym_data_max_source_length] = ACTIONS(1793),
    [sym_data_max_value_length] = ACTIONS(1793),
    [sym_data_max_label_length] = ACTIONS(1793),
    [sym_data_max_url_length] = ACTIONS(1793),
    [sym_max_last_login_info_length] = ACTIONS(1793),
    [sym_data_type_unknown] = ACTIONS(1793),
    [sym_data_type_string] = ACTIONS(1793),
    [sym_data_type_datetime] = ACTIONS(1793),
    [sym_data_type_date] = ACTIONS(1793),
    [sym_data_type_time] = ACTIONS(1793),
    [sym_data_type_image] = ACTIONS(1793),
    [sym_data_format_url] = ACTIONS(1793),
    [sym_data_format_phone] = ACTIONS(1793),
    [sym_data_format_email] = ACTIONS(1793),
    [sym_data_format_iso8601] = ACTIONS(1793),
    [sym_auditlog_priv] = ACTIONS(1793),
    [sym_configuration_priv] = ACTIONS(1793),
    [sym_deviceconfig_priv] = ACTIONS(1793),
    [sym_ftp_priv] = ACTIONS(1793),
    [sym_http_priv] = ACTIONS(1793),
    [sym_networkconfig_priv] = ACTIONS(1793),
    [sym_programport_priv] = ACTIONS(1793),
    [sym_remoteui_priv] = ACTIONS(1793),
    [sym_securitycontrol_priv] = ACTIONS(1793),
    [sym_softwaremanagement_priv] = ACTIONS(1793),
    [sym_terminal_priv] = ACTIONS(1793),
    [sym_tpadmin_priv] = ACTIONS(1793),
    [sym_user1_priv] = ACTIONS(1793),
    [sym_user2_priv] = ACTIONS(1793),
    [sym_user3_priv] = ACTIONS(1793),
    [sym_user4_priv] = ACTIONS(1793),
    [sym_usermanagement_priv] = ACTIONS(1793),
    [sym_valid_account] = ACTIONS(1793),
    [sym_err_name_invalid] = ACTIONS(1793),
    [sym_err_password_invalid] = ACTIONS(1793),
    [sym_err_invalid_account] = ACTIONS(1793),
    [sym_err_no_authorization] = ACTIONS(1793),
    [sym_err_invalid_parameter] = ACTIONS(1793),
    [sym_err_user_locked_out] = ACTIONS(1793),
    [sym_err_user_account_expired] = ACTIONS(1793),
    [sym_err_general_error] = ACTIONS(1793),
    [sym_err_authentication_error] = ACTIONS(1793),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1793),
    [sym_tls_validate_certificate] = ACTIONS(1793),
    [sym_tdata] = ACTIONS(1793),
    [sym_tchannel] = ACTIONS(1793),
    [sym_tlevel] = ACTIONS(1793),
    [sym_tbutton] = ACTIONS(1793),
    [sym_ttimeline] = ACTIONS(1793),
    [sym_tcustom] = ACTIONS(1793),
    [sym_url_struct] = ACTIONS(1793),
    [sym_dns_struct] = ACTIONS(1793),
    [sym_ip_address_struct] = ACTIONS(1793),
    [sym_dev_info_struct] = ACTIONS(1793),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1793),
    [sym_data_feed] = ACTIONS(1793),
    [sym_data_field] = ACTIONS(1793),
    [sym_data_record] = ACTIONS(1793),
    [sym_wc_data_feed] = ACTIONS(1793),
    [sym_wc_data_field] = ACTIONS(1793),
    [sym_wc_data_record] = ACTIONS(1793),
    [sym_last_login_info] = ACTIONS(1793),
    [sym____reserved___] = ACTIONS(1793),
    [sym_date] = ACTIONS(1793),
    [sym_day] = ACTIONS(1793),
    [sym_time] = ACTIONS(1793),
    [sym_ldate] = ACTIONS(1793),
    [sym_push_device] = ACTIONS(1793),
    [sym_push_channel] = ACTIONS(1793),
    [sym_push_devchan] = ACTIONS(1793),
    [sym_release_device] = ACTIONS(1793),
    [sym_release_channel] = ACTIONS(1793),
    [sym_release_devchan] = ACTIONS(1793),
    [sym_master_slot] = ACTIONS(1793),
    [sym_get_pulse_time] = ACTIONS(1793),
    [sym_get_timer] = ACTIONS(1793),
    [sym_master_sn] = ACTIONS(1793),
    [sym_system_number] = ACTIONS(1793),
    [sym_button] = ACTIONS(1793),
    [sym_dv_channel] = ACTIONS(1793),
    [sym_data] = ACTIONS(1793),
    [sym_level] = ACTIONS(1793),
    [sym_timeline] = ACTIONS(1793),
    [sym_channel] = ACTIONS(1793),
    [sym_custom] = ACTIONS(1793),
    [sym_atoi] = ACTIONS(1793),
    [sym_atol] = ACTIONS(1793),
    [sym_atof] = ACTIONS(1793),
    [sym_itoa] = ACTIONS(1793),
    [sym_itohex] = ACTIONS(1793),
    [sym_hextoi] = ACTIONS(1793),
    [sym_ftoa] = ACTIONS(1793),
    [sym_find_string] = ACTIONS(1793),
    [sym_left_string] = ACTIONS(1793),
    [sym_right_string] = ACTIONS(1793),
    [sym_mid_string] = ACTIONS(1793),
    [sym_remove_string] = ACTIONS(1793),
    [sym_get_buffer_string] = ACTIONS(1793),
    [sym_get_buffer_char] = ACTIONS(1793),
    [sym_get_multi_buffer_string] = ACTIONS(1793),
    [sym_length_string] = ACTIONS(1793),
    [sym_max_length_string] = ACTIONS(1793),
    [sym_set_length_string] = ACTIONS(1793),
    [sym_compare_string] = ACTIONS(1793),
    [sym_upper_string] = ACTIONS(1793),
    [sym_lower_string] = ACTIONS(1793),
    [sym_get_last] = ACTIONS(1793),
    [sym_date_to_day] = ACTIONS(1793),
    [sym_date_to_month] = ACTIONS(1793),
    [sym_date_to_year] = ACTIONS(1793),
    [sym_time_to_hour] = ACTIONS(1793),
    [sym_time_to_minute] = ACTIONS(1793),
    [sym_time_to_second] = ACTIONS(1793),
    [sym_day_of_week] = ACTIONS(1793),
    [sym_set_pulse_time] = ACTIONS(1793),
    [sym_set_timer] = ACTIONS(1793),
    [sym_file_open] = ACTIONS(1793),
    [sym_file_close] = ACTIONS(1793),
    [sym_file_read] = ACTIONS(1793),
    [sym_file_read_line] = ACTIONS(1793),
    [sym_file_write] = ACTIONS(1793),
    [sym_file_write_line] = ACTIONS(1793),
    [sym_file_copy] = ACTIONS(1793),
    [sym_file_delete] = ACTIONS(1793),
    [sym_file_rename] = ACTIONS(1793),
    [sym_file_dir] = ACTIONS(1793),
    [sym_file_getdir] = ACTIONS(1793),
    [sym_file_setdir] = ACTIONS(1793),
    [sym_file_create_dir] = ACTIONS(1793),
    [sym_file_remove_dir] = ACTIONS(1793),
    [sym_file_seek] = ACTIONS(1793),
    [sym_length_array] = ACTIONS(1793),
    [sym_max_length_array] = ACTIONS(1793),
    [sym_set_length_array] = ACTIONS(1793),
    [sym_ip_client_open] = ACTIONS(1793),
    [sym_ip_client_close] = ACTIONS(1793),
    [sym_ip_server_open] = ACTIONS(1793),
    [sym_ip_server_close] = ACTIONS(1793),
    [sym_ip_bound_client_open] = ACTIONS(1793),
    [sym_ip_mc_server_open] = ACTIONS(1793),
    [sym_ip_set_option] = ACTIONS(1793),
    [sym_ssh_client_open] = ACTIONS(1793),
    [sym_ssh_client_close] = ACTIONS(1793),
    [sym_tls_client_open] = ACTIONS(1793),
    [sym_tls_client_close] = ACTIONS(1793),
    [sym_do_push] = ACTIONS(1793),
    [sym_do_push_timed] = ACTIONS(1793),
    [sym_do_release] = ACTIONS(1793),
    [sym_device_id] = ACTIONS(1793),
    [sym_device_info] = ACTIONS(1793),
    [sym_device_id_string] = ACTIONS(1793),
    [sym_device_standby] = ACTIONS(1793),
    [sym_device_wake] = ACTIONS(1793),
    [sym_timeline_create] = ACTIONS(1793),
    [sym_timeline_kill] = ACTIONS(1793),
    [sym_timeline_reload] = ACTIONS(1793),
    [sym_timeline_pause] = ACTIONS(1793),
    [sym_timeline_restart] = ACTIONS(1793),
    [sym_timeline_set] = ACTIONS(1793),
    [sym_timeline_get] = ACTIONS(1793),
    [sym_timeline_active] = ACTIONS(1793),
    [sym_timeline_dynamic_id] = ACTIONS(1793),
    [sym_variable_to_string] = ACTIONS(1793),
    [sym_string_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_string] = ACTIONS(1793),
    [sym_type_cast] = ACTIONS(1793),
    [sym_variable_to_xml] = ACTIONS(1793),
    [sym_xml_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_xml] = ACTIONS(1793),
    [sym_redirect_string] = ACTIONS(1793),
    [sym_random_number] = ACTIONS(1793),
    [sym_get_unique_id] = ACTIONS(1793),
    [sym_get_system_number] = ACTIONS(1793),
    [sym_set_system_number] = ACTIONS(1793),
    [sym_get_serial_number] = ACTIONS(1793),
    [sym_get_master_build] = ACTIONS(1793),
    [sym_set_validation_code] = ACTIONS(1793),
    [sym_reboot] = ACTIONS(1793),
    [sym_load_duet_module] = ACTIONS(1793),
    [sym_unload_duet_module] = ACTIONS(1793),
    [sym_duet_mem_size_set] = ACTIONS(1793),
    [sym_duet_mem_size_get] = ACTIONS(1793),
    [sym_dynamic_application_device] = ACTIONS(1793),
    [sym_dynamic_polled_port] = ACTIONS(1793),
    [sym_static_port_binding] = ACTIONS(1793),
    [sym_static_ip_binding] = ACTIONS(1793),
    [sym_get_dns_list] = ACTIONS(1793),
    [sym_set_dns_list] = ACTIONS(1793),
    [sym_get_ip_address] = ACTIONS(1793),
    [sym_set_ip_address] = ACTIONS(1793),
    [sym_get_url_list] = ACTIONS(1793),
    [sym_add_url_entry] = ACTIONS(1793),
    [sym_delete_url_entry] = ACTIONS(1793),
    [sym_abs_value] = ACTIONS(1793),
    [sym_max_value] = ACTIONS(1793),
    [sym_min_value] = ACTIONS(1793),
    [sym_exp_value] = ACTIONS(1793),
    [sym_log_value] = ACTIONS(1793),
    [sym_log10_value] = ACTIONS(1793),
    [sym_power_value] = ACTIONS(1793),
    [sym_sqrt_value] = ACTIONS(1793),
    [sym_raw_be] = ACTIONS(1793),
    [sym_raw_le] = ACTIONS(1793),
    [sym_internal_queue_size_set] = ACTIONS(1793),
    [sym_internal_queue_size_get] = ACTIONS(1793),
    [sym_internal_threshold_set] = ACTIONS(1793),
    [sym_internal_threshold_get] = ACTIONS(1793),
    [sym_rebuilt_event] = ACTIONS(1793),
    [sym_do_custom_event] = ACTIONS(1793),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1793),
    [sym_clkmgr_set_clk_source] = ACTIONS(1793),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1793),
    [sym_clkmgr_get_timezone] = ACTIONS(1793),
    [sym_clkmgr_set_timezone] = ACTIONS(1793),
    [sym_clkmgr_get_resync_period] = ACTIONS(1793),
    [sym_clkmgr_set_resync_period] = ACTIONS(1793),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_timeservers] = ACTIONS(1793),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_set_log_level] = ACTIONS(1793),
    [sym_get_log_level] = ACTIONS(1793),
    [sym_amx_log] = ACTIONS(1793),
    [sym_smtp_server_config_set] = ACTIONS(1793),
    [sym_smtp_server_config_get] = ACTIONS(1793),
    [sym_smtp_send] = ACTIONS(1793),
    [sym_validate_netlinx_account] = ACTIONS(1793),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1793),
    [sym_authenticate_certificate] = ACTIONS(1793),
    [sym_audit_netlinx_session_event] = ACTIONS(1793),
    [sym_audit_netlinx_generic_event] = ACTIONS(1793),
    [sym_get_available_flash_disk_space] = ACTIONS(1793),
    [sym_get_max_flash_disk_space] = ACTIONS(1793),
    [sym_data_create_feed] = ACTIONS(1793),
    [sym_data_delete_feed] = ACTIONS(1793),
    [sym_data_publish_feed] = ACTIONS(1793),
    [sym_data_get_published_feed] = ACTIONS(1793),
    [sym_data_add_record] = ACTIONS(1793),
    [sym_data_get_event_record] = ACTIONS(1793),
    [sym__wc_data_create_feed] = ACTIONS(1793),
    [sym__wc_data_add_record] = ACTIONS(1793),
    [sym__wc_data_get_event_record] = ACTIONS(1793),
    [sym_format] = ACTIONS(1793),
    [sym_astro_clock] = ACTIONS(1793),
    [sym_set_outdoor_temperature] = ACTIONS(1793),
    [sym_set_virtual_level_count] = ACTIONS(1793),
    [sym_set_virtual_channel_count] = ACTIONS(1793),
    [sym_set_virtual_port_count] = ACTIONS(1793),
    [sym__wc_encode_int] = ACTIONS(1793),
    [sym__wc_decode_int] = ACTIONS(1793),
    [sym__wc_int] = ACTIONS(1793),
    [sym__wc_to_ch_int] = ACTIONS(1793),
    [sym__ch_to_wc_int] = ACTIONS(1793),
    [sym__wc_find_string_int] = ACTIONS(1793),
    [sym__wc_left_string_int] = ACTIONS(1793),
    [sym__wc_lower_string_int] = ACTIONS(1793),
    [sym__wc_mid_string_int] = ACTIONS(1793),
    [sym__wc_remove_string_int] = ACTIONS(1793),
    [sym__wc_right_string_int] = ACTIONS(1793),
    [sym__wc_upper_string_int] = ACTIONS(1793),
    [sym__wc_compare_string_int] = ACTIONS(1793),
    [sym__wc_get_buffer_char_int] = ACTIONS(1793),
    [sym__wc_get_buffer_string_int] = ACTIONS(1793),
    [sym__wc_concat_string_int] = ACTIONS(1793),
    [sym__wc_file_open_int] = ACTIONS(1793),
    [sym__wc_file_close_int] = ACTIONS(1793),
    [sym__wc_file_read_int] = ACTIONS(1793),
    [sym__wc_file_read_line_int] = ACTIONS(1793),
    [sym__wc_file_write_int] = ACTIONS(1793),
    [sym__wc_file_write_line_int] = ACTIONS(1793),
    [aux_sym_preproc_include_token1] = ACTIONS(1793),
    [aux_sym_preproc_define_token1] = ACTIONS(1795),
    [aux_sym_preproc_warn_token1] = ACTIONS(1795),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1795),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_else_token1] = ACTIONS(1795),
    [anon_sym_LBRACK] = ACTIONS(1795),
    [anon_sym_LBRACE] = ACTIONS(1795),
    [anon_sym_LPAREN] = ACTIONS(1793),
    [anon_sym_BANG] = ACTIONS(1795),
    [anon_sym_TILDE] = ACTIONS(1795),
    [anon_sym_DASH] = ACTIONS(1793),
    [anon_sym_PLUS] = ACTIONS(1793),
    [anon_sym_DASH_DASH] = ACTIONS(1795),
    [anon_sym_PLUS_PLUS] = ACTIONS(1795),
    [anon_sym_DQUOTE] = ACTIONS(1795),
    [anon_sym_SQUOTE] = ACTIONS(1795),
    [sym_number_literal] = ACTIONS(1795),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(241)] = {
    [sym_identifier] = ACTIONS(1793),
    [sym_program_name_keyword] = ACTIONS(1793),
    [sym_module_name_keyword] = ACTIONS(1793),
    [sym_define_device_keyword] = ACTIONS(1793),
    [sym_define_combine_keyword] = ACTIONS(1793),
    [sym_define_constant_keyword] = ACTIONS(1793),
    [sym_define_type_keyword] = ACTIONS(1793),
    [sym_define_variable_keyword] = ACTIONS(1793),
    [sym_define_system_variable_keyword] = ACTIONS(1793),
    [sym_define_connect_level_keyword] = ACTIONS(1793),
    [sym_define_latching_keyword] = ACTIONS(1793),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1793),
    [sym_define_toggling_keyword] = ACTIONS(1793),
    [sym_define_call_keyword] = ACTIONS(1793),
    [sym_define_function_keyword] = ACTIONS(1793),
    [sym_define_library_function_keyword] = ACTIONS(1793),
    [sym_define_start_keyword] = ACTIONS(1793),
    [sym_define_event_keyword] = ACTIONS(1793),
    [sym_define_program_keyword] = ACTIONS(1793),
    [sym_define_module_keyword] = ACTIONS(1793),
    [sym_char_keyword] = ACTIONS(1793),
    [sym_widechar_keyword] = ACTIONS(1793),
    [sym_integer_keyword] = ACTIONS(1793),
    [sym_sinteger_keyword] = ACTIONS(1793),
    [sym_long_keyword] = ACTIONS(1793),
    [sym_slong_keyword] = ACTIONS(1793),
    [sym_float_keyword] = ACTIONS(1793),
    [sym_double_keyword] = ACTIONS(1793),
    [sym_variant_keyword] = ACTIONS(1793),
    [sym_variantarray_keyword] = ACTIONS(1793),
    [sym_local_var_keyword] = ACTIONS(1793),
    [sym_stack_var_keyword] = ACTIONS(1793),
    [sym_constant_keyword] = ACTIONS(1793),
    [sym_volatile_keyword] = ACTIONS(1793),
    [sym_non_volatile_keyword] = ACTIONS(1793),
    [sym_persistent_keyword] = ACTIONS(1793),
    [sym_dev_keyword] = ACTIONS(1793),
    [sym_devlev_keyword] = ACTIONS(1793),
    [sym_devchan_keyword] = ACTIONS(1793),
    [sym_if_keyword] = ACTIONS(1793),
    [sym_else_keyword] = ACTIONS(1793),
    [sym_switch_keyword] = ACTIONS(1793),
    [sym_case_keyword] = ACTIONS(1793),
    [sym_default_keyword] = ACTIONS(1793),
    [sym_while_keyword] = ACTIONS(1793),
    [sym_for_keyword] = ACTIONS(1793),
    [sym_break_keyword] = ACTIONS(1793),
    [sym_continue_keyword] = ACTIONS(1793),
    [sym_return_keyword] = ACTIONS(1793),
    [sym_select_keyword] = ACTIONS(1793),
    [sym_active_keyword] = ACTIONS(1793),
    [sym_struct_keyword] = ACTIONS(1793),
    [sym_structure_keyword] = ACTIONS(1793),
    [sym_button_event_keyword] = ACTIONS(1793),
    [sym_channel_event_keyword] = ACTIONS(1793),
    [sym_level_event_keyword] = ACTIONS(1793),
    [sym_data_event_keyword] = ACTIONS(1793),
    [sym_timeline_event_keyword] = ACTIONS(1793),
    [sym_custom_event_keyword] = ACTIONS(1793),
    [sym_push_keyword] = ACTIONS(1793),
    [sym_release_keyword] = ACTIONS(1793),
    [aux_sym_on_keyword_token1] = ACTIONS(1793),
    [aux_sym_off_keyword_token1] = ACTIONS(1793),
    [sym_send_level_keyword] = ACTIONS(1793),
    [sym_send_string_keyword] = ACTIONS(1793),
    [sym_send_command_keyword] = ACTIONS(1793),
    [sym_clear_buffer_keyword] = ACTIONS(1793),
    [sym_create_buffer_keyword] = ACTIONS(1793),
    [sym_create_multi_buffer_keyword] = ACTIONS(1793),
    [sym_call_keyword] = ACTIONS(1793),
    [sym_system_call_keyword] = ACTIONS(1793),
    [sym_devchan_to_keyword] = ACTIONS(1793),
    [sym_devchan_min_to_keyword] = ACTIONS(1793),
    [sym_devchan_total_off_keyword] = ACTIONS(1793),
    [sym_devchan_pulse_keyword] = ACTIONS(1793),
    [sym_wait_keyword] = ACTIONS(1793),
    [sym_cancel_wait_keyword] = ACTIONS(1793),
    [sym_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1793),
    [sym_not] = ACTIONS(1793),
    [sym_bnot] = ACTIONS(1793),
    [sym___file__] = ACTIONS(1793),
    [sym___date__] = ACTIONS(1793),
    [sym___ldate__] = ACTIONS(1793),
    [sym___time__] = ACTIONS(1793),
    [sym___line__] = ACTIONS(1793),
    [sym___name__] = ACTIONS(1793),
    [sym___netlinx__] = ACTIONS(1793),
    [sym_netlinx_axi_version] = ACTIONS(1793),
    [sym_url_flg_tcp] = ACTIONS(1793),
    [sym_url_flg_accinfopresent] = ACTIONS(1793),
    [sym_url_flg_temp] = ACTIONS(1793),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1793),
    [sym_url_flg_stat_mask] = ACTIONS(1793),
    [sym_url_flg_stat_lookup] = ACTIONS(1793),
    [sym_url_flg_stat_connecting] = ACTIONS(1793),
    [sym_url_flg_stat_waiting] = ACTIONS(1793),
    [sym_url_flg_stat_connected] = ACTIONS(1793),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1793),
    [sym_first_virtual_device] = ACTIONS(1793),
    [sym_first_local_port] = ACTIONS(1793),
    [sym_dynamic_virtual_device] = ACTIONS(1793),
    [sym_timeline_once] = ACTIONS(1793),
    [sym_timeline_repeat] = ACTIONS(1793),
    [sym_timeline_absolute] = ACTIONS(1793),
    [sym_timeline_relative] = ACTIONS(1793),
    [sym_do_push_timed_infinite] = ACTIONS(1793),
    [sym_source_type_no_address] = ACTIONS(1793),
    [sym_source_type_neuron_id] = ACTIONS(1793),
    [sym_source_type_ip_address] = ACTIONS(1793),
    [sym_source_type_axlink] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1793),
    [sym_source_type_ip_socket_address] = ACTIONS(1793),
    [sym_source_type_rs232] = ACTIONS(1793),
    [sym_source_type_internal] = ACTIONS(1793),
    [sym_source_type_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1793),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1793),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1793),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1793),
    [sym_true] = ACTIONS(1793),
    [sym_false] = ACTIONS(1793),
    [sym_file_read_only] = ACTIONS(1793),
    [sym_file_rw_new] = ACTIONS(1793),
    [sym_file_rw_append] = ACTIONS(1793),
    [sym_ip_tcp] = ACTIONS(1793),
    [sym_ip_udp] = ACTIONS(1793),
    [sym_ip_udp_2way] = ACTIONS(1793),
    [sym_xml_encode_types] = ACTIONS(1793),
    [sym_xml_encode_char_as_list] = ACTIONS(1793),
    [sym_xml_encode_le] = ACTIONS(1793),
    [sym_xml_decode_types] = ACTIONS(1793),
    [sym_xml_decode_no_preserve] = ACTIONS(1793),
    [sym_duet_dev_type_display_device] = ACTIONS(1793),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_io_device] = ACTIONS(1793),
    [sym_duet_dev_type_relay_device] = ACTIONS(1793),
    [sym_duet_dev_type_ups] = ACTIONS(1793),
    [sym_duet_dev_type_amplifier] = ACTIONS(1793),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1793),
    [sym_duet_dev_type_camera] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1793),
    [sym_duet_dev_type_dss] = ACTIONS(1793),
    [sym_duet_dev_type_dvr] = ACTIONS(1793),
    [sym_duet_dev_type_disc_device] = ACTIONS(1793),
    [sym_duet_dev_type_document_camera] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1793),
    [sym_duet_dev_type_hvac] = ACTIONS(1793),
    [sym_duet_dev_type_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_light] = ACTIONS(1793),
    [sym_duet_dev_type_monitor] = ACTIONS(1793),
    [sym_duet_dev_type_motor] = ACTIONS(1793),
    [sym_duet_dev_type_multi_window] = ACTIONS(1793),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1793),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1793),
    [sym_duet_dev_type_receiver] = ACTIONS(1793),
    [sym_duet_dev_type_security_system] = ACTIONS(1793),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1793),
    [sym_duet_dev_type_settop_box] = ACTIONS(1793),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1793),
    [sym_duet_dev_type_switcher] = ACTIONS(1793),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_tv] = ACTIONS(1793),
    [sym_duet_dev_type_utility] = ACTIONS(1793),
    [sym_duet_dev_type_vcr] = ACTIONS(1793),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_video_processor] = ACTIONS(1793),
    [sym_duet_dev_type_video_projector] = ACTIONS(1793),
    [sym_duet_dev_type_video_wall] = ACTIONS(1793),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1793),
    [sym_duet_dev_type_weather] = ACTIONS(1793),
    [sym_duet_dev_not_polled] = ACTIONS(1793),
    [sym_duet_dev_polled] = ACTIONS(1793),
    [sym_clkmgr_mode_network] = ACTIONS(1793),
    [sym_clkmgr_mode_standalone] = ACTIONS(1793),
    [sym_normal_standby] = ACTIONS(1793),
    [sym_normal_wake] = ACTIONS(1793),
    [sym_ip_multicast_ttl_option] = ACTIONS(1793),
    [sym_ip_ttl_subnet] = ACTIONS(1793),
    [sym_ip_ttl_site] = ACTIONS(1793),
    [sym_ip_ttl_region] = ACTIONS(1793),
    [sym_ip_ttl_continent] = ACTIONS(1793),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1793),
    [sym_ip_nodelay_on] = ACTIONS(1793),
    [sym_ip_nodelay_off] = ACTIONS(1793),
    [sym_amx_error] = ACTIONS(1793),
    [sym_amx_warning] = ACTIONS(1793),
    [sym_amx_info] = ACTIONS(1793),
    [sym_amx_debug] = ACTIONS(1793),
    [sym_netlinx_login_success] = ACTIONS(1793),
    [sym_netlinx_login_fail] = ACTIONS(1793),
    [sym_netlinx_logout] = ACTIONS(1793),
    [sym_null_str] = ACTIONS(1793),
    [sym_smtp_address] = ACTIONS(1793),
    [sym_smtp_port_number] = ACTIONS(1793),
    [sym_smtp_username] = ACTIONS(1793),
    [sym_smtp_password] = ACTIONS(1793),
    [sym_smtp_require_tls] = ACTIONS(1793),
    [sym_smtp_from] = ACTIONS(1793),
    [sym_smtp_tls_true] = ACTIONS(1793),
    [sym_smtp_tls_false] = ACTIONS(1793),
    [sym_listview_on_row_select_event] = ACTIONS(1793),
    [sym_data_structure_unknown] = ACTIONS(1793),
    [sym_data_structure_datafeed] = ACTIONS(1793),
    [sym_data_structure_datarecordset] = ACTIONS(1793),
    [sym_data_structure_datarecord] = ACTIONS(1793),
    [sym_data_structure_datafield] = ACTIONS(1793),
    [sym_data_max_content_field_count] = ACTIONS(1793),
    [sym_data_max_metadata_field_count] = ACTIONS(1793),
    [sym_data_max_id_length] = ACTIONS(1793),
    [sym_data_max_name_length] = ACTIONS(1793),
    [sym_data_max_description_length] = ACTIONS(1793),
    [sym_data_max_type_length] = ACTIONS(1793),
    [sym_data_max_format_length] = ACTIONS(1793),
    [sym_data_max_source_length] = ACTIONS(1793),
    [sym_data_max_value_length] = ACTIONS(1793),
    [sym_data_max_label_length] = ACTIONS(1793),
    [sym_data_max_url_length] = ACTIONS(1793),
    [sym_max_last_login_info_length] = ACTIONS(1793),
    [sym_data_type_unknown] = ACTIONS(1793),
    [sym_data_type_string] = ACTIONS(1793),
    [sym_data_type_datetime] = ACTIONS(1793),
    [sym_data_type_date] = ACTIONS(1793),
    [sym_data_type_time] = ACTIONS(1793),
    [sym_data_type_image] = ACTIONS(1793),
    [sym_data_format_url] = ACTIONS(1793),
    [sym_data_format_phone] = ACTIONS(1793),
    [sym_data_format_email] = ACTIONS(1793),
    [sym_data_format_iso8601] = ACTIONS(1793),
    [sym_auditlog_priv] = ACTIONS(1793),
    [sym_configuration_priv] = ACTIONS(1793),
    [sym_deviceconfig_priv] = ACTIONS(1793),
    [sym_ftp_priv] = ACTIONS(1793),
    [sym_http_priv] = ACTIONS(1793),
    [sym_networkconfig_priv] = ACTIONS(1793),
    [sym_programport_priv] = ACTIONS(1793),
    [sym_remoteui_priv] = ACTIONS(1793),
    [sym_securitycontrol_priv] = ACTIONS(1793),
    [sym_softwaremanagement_priv] = ACTIONS(1793),
    [sym_terminal_priv] = ACTIONS(1793),
    [sym_tpadmin_priv] = ACTIONS(1793),
    [sym_user1_priv] = ACTIONS(1793),
    [sym_user2_priv] = ACTIONS(1793),
    [sym_user3_priv] = ACTIONS(1793),
    [sym_user4_priv] = ACTIONS(1793),
    [sym_usermanagement_priv] = ACTIONS(1793),
    [sym_valid_account] = ACTIONS(1793),
    [sym_err_name_invalid] = ACTIONS(1793),
    [sym_err_password_invalid] = ACTIONS(1793),
    [sym_err_invalid_account] = ACTIONS(1793),
    [sym_err_no_authorization] = ACTIONS(1793),
    [sym_err_invalid_parameter] = ACTIONS(1793),
    [sym_err_user_locked_out] = ACTIONS(1793),
    [sym_err_user_account_expired] = ACTIONS(1793),
    [sym_err_general_error] = ACTIONS(1793),
    [sym_err_authentication_error] = ACTIONS(1793),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1793),
    [sym_tls_validate_certificate] = ACTIONS(1793),
    [sym_tdata] = ACTIONS(1793),
    [sym_tchannel] = ACTIONS(1793),
    [sym_tlevel] = ACTIONS(1793),
    [sym_tbutton] = ACTIONS(1793),
    [sym_ttimeline] = ACTIONS(1793),
    [sym_tcustom] = ACTIONS(1793),
    [sym_url_struct] = ACTIONS(1793),
    [sym_dns_struct] = ACTIONS(1793),
    [sym_ip_address_struct] = ACTIONS(1793),
    [sym_dev_info_struct] = ACTIONS(1793),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1793),
    [sym_data_feed] = ACTIONS(1793),
    [sym_data_field] = ACTIONS(1793),
    [sym_data_record] = ACTIONS(1793),
    [sym_wc_data_feed] = ACTIONS(1793),
    [sym_wc_data_field] = ACTIONS(1793),
    [sym_wc_data_record] = ACTIONS(1793),
    [sym_last_login_info] = ACTIONS(1793),
    [sym____reserved___] = ACTIONS(1793),
    [sym_date] = ACTIONS(1793),
    [sym_day] = ACTIONS(1793),
    [sym_time] = ACTIONS(1793),
    [sym_ldate] = ACTIONS(1793),
    [sym_push_device] = ACTIONS(1793),
    [sym_push_channel] = ACTIONS(1793),
    [sym_push_devchan] = ACTIONS(1793),
    [sym_release_device] = ACTIONS(1793),
    [sym_release_channel] = ACTIONS(1793),
    [sym_release_devchan] = ACTIONS(1793),
    [sym_master_slot] = ACTIONS(1793),
    [sym_get_pulse_time] = ACTIONS(1793),
    [sym_get_timer] = ACTIONS(1793),
    [sym_master_sn] = ACTIONS(1793),
    [sym_system_number] = ACTIONS(1793),
    [sym_button] = ACTIONS(1793),
    [sym_dv_channel] = ACTIONS(1793),
    [sym_data] = ACTIONS(1793),
    [sym_level] = ACTIONS(1793),
    [sym_timeline] = ACTIONS(1793),
    [sym_channel] = ACTIONS(1793),
    [sym_custom] = ACTIONS(1793),
    [sym_atoi] = ACTIONS(1793),
    [sym_atol] = ACTIONS(1793),
    [sym_atof] = ACTIONS(1793),
    [sym_itoa] = ACTIONS(1793),
    [sym_itohex] = ACTIONS(1793),
    [sym_hextoi] = ACTIONS(1793),
    [sym_ftoa] = ACTIONS(1793),
    [sym_find_string] = ACTIONS(1793),
    [sym_left_string] = ACTIONS(1793),
    [sym_right_string] = ACTIONS(1793),
    [sym_mid_string] = ACTIONS(1793),
    [sym_remove_string] = ACTIONS(1793),
    [sym_get_buffer_string] = ACTIONS(1793),
    [sym_get_buffer_char] = ACTIONS(1793),
    [sym_get_multi_buffer_string] = ACTIONS(1793),
    [sym_length_string] = ACTIONS(1793),
    [sym_max_length_string] = ACTIONS(1793),
    [sym_set_length_string] = ACTIONS(1793),
    [sym_compare_string] = ACTIONS(1793),
    [sym_upper_string] = ACTIONS(1793),
    [sym_lower_string] = ACTIONS(1793),
    [sym_get_last] = ACTIONS(1793),
    [sym_date_to_day] = ACTIONS(1793),
    [sym_date_to_month] = ACTIONS(1793),
    [sym_date_to_year] = ACTIONS(1793),
    [sym_time_to_hour] = ACTIONS(1793),
    [sym_time_to_minute] = ACTIONS(1793),
    [sym_time_to_second] = ACTIONS(1793),
    [sym_day_of_week] = ACTIONS(1793),
    [sym_set_pulse_time] = ACTIONS(1793),
    [sym_set_timer] = ACTIONS(1793),
    [sym_file_open] = ACTIONS(1793),
    [sym_file_close] = ACTIONS(1793),
    [sym_file_read] = ACTIONS(1793),
    [sym_file_read_line] = ACTIONS(1793),
    [sym_file_write] = ACTIONS(1793),
    [sym_file_write_line] = ACTIONS(1793),
    [sym_file_copy] = ACTIONS(1793),
    [sym_file_delete] = ACTIONS(1793),
    [sym_file_rename] = ACTIONS(1793),
    [sym_file_dir] = ACTIONS(1793),
    [sym_file_getdir] = ACTIONS(1793),
    [sym_file_setdir] = ACTIONS(1793),
    [sym_file_create_dir] = ACTIONS(1793),
    [sym_file_remove_dir] = ACTIONS(1793),
    [sym_file_seek] = ACTIONS(1793),
    [sym_length_array] = ACTIONS(1793),
    [sym_max_length_array] = ACTIONS(1793),
    [sym_set_length_array] = ACTIONS(1793),
    [sym_ip_client_open] = ACTIONS(1793),
    [sym_ip_client_close] = ACTIONS(1793),
    [sym_ip_server_open] = ACTIONS(1793),
    [sym_ip_server_close] = ACTIONS(1793),
    [sym_ip_bound_client_open] = ACTIONS(1793),
    [sym_ip_mc_server_open] = ACTIONS(1793),
    [sym_ip_set_option] = ACTIONS(1793),
    [sym_ssh_client_open] = ACTIONS(1793),
    [sym_ssh_client_close] = ACTIONS(1793),
    [sym_tls_client_open] = ACTIONS(1793),
    [sym_tls_client_close] = ACTIONS(1793),
    [sym_do_push] = ACTIONS(1793),
    [sym_do_push_timed] = ACTIONS(1793),
    [sym_do_release] = ACTIONS(1793),
    [sym_device_id] = ACTIONS(1793),
    [sym_device_info] = ACTIONS(1793),
    [sym_device_id_string] = ACTIONS(1793),
    [sym_device_standby] = ACTIONS(1793),
    [sym_device_wake] = ACTIONS(1793),
    [sym_timeline_create] = ACTIONS(1793),
    [sym_timeline_kill] = ACTIONS(1793),
    [sym_timeline_reload] = ACTIONS(1793),
    [sym_timeline_pause] = ACTIONS(1793),
    [sym_timeline_restart] = ACTIONS(1793),
    [sym_timeline_set] = ACTIONS(1793),
    [sym_timeline_get] = ACTIONS(1793),
    [sym_timeline_active] = ACTIONS(1793),
    [sym_timeline_dynamic_id] = ACTIONS(1793),
    [sym_variable_to_string] = ACTIONS(1793),
    [sym_string_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_string] = ACTIONS(1793),
    [sym_type_cast] = ACTIONS(1793),
    [sym_variable_to_xml] = ACTIONS(1793),
    [sym_xml_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_xml] = ACTIONS(1793),
    [sym_redirect_string] = ACTIONS(1793),
    [sym_random_number] = ACTIONS(1793),
    [sym_get_unique_id] = ACTIONS(1793),
    [sym_get_system_number] = ACTIONS(1793),
    [sym_set_system_number] = ACTIONS(1793),
    [sym_get_serial_number] = ACTIONS(1793),
    [sym_get_master_build] = ACTIONS(1793),
    [sym_set_validation_code] = ACTIONS(1793),
    [sym_reboot] = ACTIONS(1793),
    [sym_load_duet_module] = ACTIONS(1793),
    [sym_unload_duet_module] = ACTIONS(1793),
    [sym_duet_mem_size_set] = ACTIONS(1793),
    [sym_duet_mem_size_get] = ACTIONS(1793),
    [sym_dynamic_application_device] = ACTIONS(1793),
    [sym_dynamic_polled_port] = ACTIONS(1793),
    [sym_static_port_binding] = ACTIONS(1793),
    [sym_static_ip_binding] = ACTIONS(1793),
    [sym_get_dns_list] = ACTIONS(1793),
    [sym_set_dns_list] = ACTIONS(1793),
    [sym_get_ip_address] = ACTIONS(1793),
    [sym_set_ip_address] = ACTIONS(1793),
    [sym_get_url_list] = ACTIONS(1793),
    [sym_add_url_entry] = ACTIONS(1793),
    [sym_delete_url_entry] = ACTIONS(1793),
    [sym_abs_value] = ACTIONS(1793),
    [sym_max_value] = ACTIONS(1793),
    [sym_min_value] = ACTIONS(1793),
    [sym_exp_value] = ACTIONS(1793),
    [sym_log_value] = ACTIONS(1793),
    [sym_log10_value] = ACTIONS(1793),
    [sym_power_value] = ACTIONS(1793),
    [sym_sqrt_value] = ACTIONS(1793),
    [sym_raw_be] = ACTIONS(1793),
    [sym_raw_le] = ACTIONS(1793),
    [sym_internal_queue_size_set] = ACTIONS(1793),
    [sym_internal_queue_size_get] = ACTIONS(1793),
    [sym_internal_threshold_set] = ACTIONS(1793),
    [sym_internal_threshold_get] = ACTIONS(1793),
    [sym_rebuilt_event] = ACTIONS(1793),
    [sym_do_custom_event] = ACTIONS(1793),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1793),
    [sym_clkmgr_set_clk_source] = ACTIONS(1793),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1793),
    [sym_clkmgr_get_timezone] = ACTIONS(1793),
    [sym_clkmgr_set_timezone] = ACTIONS(1793),
    [sym_clkmgr_get_resync_period] = ACTIONS(1793),
    [sym_clkmgr_set_resync_period] = ACTIONS(1793),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_timeservers] = ACTIONS(1793),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_set_log_level] = ACTIONS(1793),
    [sym_get_log_level] = ACTIONS(1793),
    [sym_amx_log] = ACTIONS(1793),
    [sym_smtp_server_config_set] = ACTIONS(1793),
    [sym_smtp_server_config_get] = ACTIONS(1793),
    [sym_smtp_send] = ACTIONS(1793),
    [sym_validate_netlinx_account] = ACTIONS(1793),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1793),
    [sym_authenticate_certificate] = ACTIONS(1793),
    [sym_audit_netlinx_session_event] = ACTIONS(1793),
    [sym_audit_netlinx_generic_event] = ACTIONS(1793),
    [sym_get_available_flash_disk_space] = ACTIONS(1793),
    [sym_get_max_flash_disk_space] = ACTIONS(1793),
    [sym_data_create_feed] = ACTIONS(1793),
    [sym_data_delete_feed] = ACTIONS(1793),
    [sym_data_publish_feed] = ACTIONS(1793),
    [sym_data_get_published_feed] = ACTIONS(1793),
    [sym_data_add_record] = ACTIONS(1793),
    [sym_data_get_event_record] = ACTIONS(1793),
    [sym__wc_data_create_feed] = ACTIONS(1793),
    [sym__wc_data_add_record] = ACTIONS(1793),
    [sym__wc_data_get_event_record] = ACTIONS(1793),
    [sym_format] = ACTIONS(1793),
    [sym_astro_clock] = ACTIONS(1793),
    [sym_set_outdoor_temperature] = ACTIONS(1793),
    [sym_set_virtual_level_count] = ACTIONS(1793),
    [sym_set_virtual_channel_count] = ACTIONS(1793),
    [sym_set_virtual_port_count] = ACTIONS(1793),
    [sym__wc_encode_int] = ACTIONS(1793),
    [sym__wc_decode_int] = ACTIONS(1793),
    [sym__wc_int] = ACTIONS(1793),
    [sym__wc_to_ch_int] = ACTIONS(1793),
    [sym__ch_to_wc_int] = ACTIONS(1793),
    [sym__wc_find_string_int] = ACTIONS(1793),
    [sym__wc_left_string_int] = ACTIONS(1793),
    [sym__wc_lower_string_int] = ACTIONS(1793),
    [sym__wc_mid_string_int] = ACTIONS(1793),
    [sym__wc_remove_string_int] = ACTIONS(1793),
    [sym__wc_right_string_int] = ACTIONS(1793),
    [sym__wc_upper_string_int] = ACTIONS(1793),
    [sym__wc_compare_string_int] = ACTIONS(1793),
    [sym__wc_get_buffer_char_int] = ACTIONS(1793),
    [sym__wc_get_buffer_string_int] = ACTIONS(1793),
    [sym__wc_concat_string_int] = ACTIONS(1793),
    [sym__wc_file_open_int] = ACTIONS(1793),
    [sym__wc_file_close_int] = ACTIONS(1793),
    [sym__wc_file_read_int] = ACTIONS(1793),
    [sym__wc_file_read_line_int] = ACTIONS(1793),
    [sym__wc_file_write_int] = ACTIONS(1793),
    [sym__wc_file_write_line_int] = ACTIONS(1793),
    [aux_sym_preproc_include_token1] = ACTIONS(1793),
    [aux_sym_preproc_define_token1] = ACTIONS(1795),
    [aux_sym_preproc_warn_token1] = ACTIONS(1795),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1795),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_else_token1] = ACTIONS(1795),
    [anon_sym_LBRACK] = ACTIONS(1795),
    [anon_sym_LBRACE] = ACTIONS(1795),
    [anon_sym_LPAREN] = ACTIONS(1793),
    [anon_sym_BANG] = ACTIONS(1795),
    [anon_sym_TILDE] = ACTIONS(1795),
    [anon_sym_DASH] = ACTIONS(1793),
    [anon_sym_PLUS] = ACTIONS(1793),
    [anon_sym_DASH_DASH] = ACTIONS(1795),
    [anon_sym_PLUS_PLUS] = ACTIONS(1795),
    [anon_sym_DQUOTE] = ACTIONS(1795),
    [anon_sym_SQUOTE] = ACTIONS(1795),
    [sym_number_literal] = ACTIONS(1795),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(242)] = {
    [sym_identifier] = ACTIONS(1793),
    [sym_program_name_keyword] = ACTIONS(1793),
    [sym_module_name_keyword] = ACTIONS(1793),
    [sym_define_device_keyword] = ACTIONS(1793),
    [sym_define_combine_keyword] = ACTIONS(1793),
    [sym_define_constant_keyword] = ACTIONS(1793),
    [sym_define_type_keyword] = ACTIONS(1793),
    [sym_define_variable_keyword] = ACTIONS(1793),
    [sym_define_system_variable_keyword] = ACTIONS(1793),
    [sym_define_connect_level_keyword] = ACTIONS(1793),
    [sym_define_latching_keyword] = ACTIONS(1793),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1793),
    [sym_define_toggling_keyword] = ACTIONS(1793),
    [sym_define_call_keyword] = ACTIONS(1793),
    [sym_define_function_keyword] = ACTIONS(1793),
    [sym_define_library_function_keyword] = ACTIONS(1793),
    [sym_define_start_keyword] = ACTIONS(1793),
    [sym_define_event_keyword] = ACTIONS(1793),
    [sym_define_program_keyword] = ACTIONS(1793),
    [sym_define_module_keyword] = ACTIONS(1793),
    [sym_char_keyword] = ACTIONS(1793),
    [sym_widechar_keyword] = ACTIONS(1793),
    [sym_integer_keyword] = ACTIONS(1793),
    [sym_sinteger_keyword] = ACTIONS(1793),
    [sym_long_keyword] = ACTIONS(1793),
    [sym_slong_keyword] = ACTIONS(1793),
    [sym_float_keyword] = ACTIONS(1793),
    [sym_double_keyword] = ACTIONS(1793),
    [sym_variant_keyword] = ACTIONS(1793),
    [sym_variantarray_keyword] = ACTIONS(1793),
    [sym_local_var_keyword] = ACTIONS(1793),
    [sym_stack_var_keyword] = ACTIONS(1793),
    [sym_constant_keyword] = ACTIONS(1793),
    [sym_volatile_keyword] = ACTIONS(1793),
    [sym_non_volatile_keyword] = ACTIONS(1793),
    [sym_persistent_keyword] = ACTIONS(1793),
    [sym_dev_keyword] = ACTIONS(1793),
    [sym_devlev_keyword] = ACTIONS(1793),
    [sym_devchan_keyword] = ACTIONS(1793),
    [sym_if_keyword] = ACTIONS(1793),
    [sym_else_keyword] = ACTIONS(1793),
    [sym_switch_keyword] = ACTIONS(1793),
    [sym_case_keyword] = ACTIONS(1793),
    [sym_default_keyword] = ACTIONS(1793),
    [sym_while_keyword] = ACTIONS(1793),
    [sym_for_keyword] = ACTIONS(1793),
    [sym_break_keyword] = ACTIONS(1793),
    [sym_continue_keyword] = ACTIONS(1793),
    [sym_return_keyword] = ACTIONS(1793),
    [sym_select_keyword] = ACTIONS(1793),
    [sym_active_keyword] = ACTIONS(1793),
    [sym_struct_keyword] = ACTIONS(1793),
    [sym_structure_keyword] = ACTIONS(1793),
    [sym_button_event_keyword] = ACTIONS(1793),
    [sym_channel_event_keyword] = ACTIONS(1793),
    [sym_level_event_keyword] = ACTIONS(1793),
    [sym_data_event_keyword] = ACTIONS(1793),
    [sym_timeline_event_keyword] = ACTIONS(1793),
    [sym_custom_event_keyword] = ACTIONS(1793),
    [sym_push_keyword] = ACTIONS(1793),
    [sym_release_keyword] = ACTIONS(1793),
    [aux_sym_on_keyword_token1] = ACTIONS(1793),
    [aux_sym_off_keyword_token1] = ACTIONS(1793),
    [sym_send_level_keyword] = ACTIONS(1793),
    [sym_send_string_keyword] = ACTIONS(1793),
    [sym_send_command_keyword] = ACTIONS(1793),
    [sym_clear_buffer_keyword] = ACTIONS(1793),
    [sym_create_buffer_keyword] = ACTIONS(1793),
    [sym_create_multi_buffer_keyword] = ACTIONS(1793),
    [sym_call_keyword] = ACTIONS(1793),
    [sym_system_call_keyword] = ACTIONS(1793),
    [sym_devchan_to_keyword] = ACTIONS(1793),
    [sym_devchan_min_to_keyword] = ACTIONS(1793),
    [sym_devchan_total_off_keyword] = ACTIONS(1793),
    [sym_devchan_pulse_keyword] = ACTIONS(1793),
    [sym_wait_keyword] = ACTIONS(1793),
    [sym_cancel_wait_keyword] = ACTIONS(1793),
    [sym_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1793),
    [sym_not] = ACTIONS(1793),
    [sym_bnot] = ACTIONS(1793),
    [sym___file__] = ACTIONS(1793),
    [sym___date__] = ACTIONS(1793),
    [sym___ldate__] = ACTIONS(1793),
    [sym___time__] = ACTIONS(1793),
    [sym___line__] = ACTIONS(1793),
    [sym___name__] = ACTIONS(1793),
    [sym___netlinx__] = ACTIONS(1793),
    [sym_netlinx_axi_version] = ACTIONS(1793),
    [sym_url_flg_tcp] = ACTIONS(1793),
    [sym_url_flg_accinfopresent] = ACTIONS(1793),
    [sym_url_flg_temp] = ACTIONS(1793),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1793),
    [sym_url_flg_stat_mask] = ACTIONS(1793),
    [sym_url_flg_stat_lookup] = ACTIONS(1793),
    [sym_url_flg_stat_connecting] = ACTIONS(1793),
    [sym_url_flg_stat_waiting] = ACTIONS(1793),
    [sym_url_flg_stat_connected] = ACTIONS(1793),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1793),
    [sym_first_virtual_device] = ACTIONS(1793),
    [sym_first_local_port] = ACTIONS(1793),
    [sym_dynamic_virtual_device] = ACTIONS(1793),
    [sym_timeline_once] = ACTIONS(1793),
    [sym_timeline_repeat] = ACTIONS(1793),
    [sym_timeline_absolute] = ACTIONS(1793),
    [sym_timeline_relative] = ACTIONS(1793),
    [sym_do_push_timed_infinite] = ACTIONS(1793),
    [sym_source_type_no_address] = ACTIONS(1793),
    [sym_source_type_neuron_id] = ACTIONS(1793),
    [sym_source_type_ip_address] = ACTIONS(1793),
    [sym_source_type_axlink] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1793),
    [sym_source_type_ip_socket_address] = ACTIONS(1793),
    [sym_source_type_rs232] = ACTIONS(1793),
    [sym_source_type_internal] = ACTIONS(1793),
    [sym_source_type_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1793),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1793),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1793),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1793),
    [sym_true] = ACTIONS(1793),
    [sym_false] = ACTIONS(1793),
    [sym_file_read_only] = ACTIONS(1793),
    [sym_file_rw_new] = ACTIONS(1793),
    [sym_file_rw_append] = ACTIONS(1793),
    [sym_ip_tcp] = ACTIONS(1793),
    [sym_ip_udp] = ACTIONS(1793),
    [sym_ip_udp_2way] = ACTIONS(1793),
    [sym_xml_encode_types] = ACTIONS(1793),
    [sym_xml_encode_char_as_list] = ACTIONS(1793),
    [sym_xml_encode_le] = ACTIONS(1793),
    [sym_xml_decode_types] = ACTIONS(1793),
    [sym_xml_decode_no_preserve] = ACTIONS(1793),
    [sym_duet_dev_type_display_device] = ACTIONS(1793),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_io_device] = ACTIONS(1793),
    [sym_duet_dev_type_relay_device] = ACTIONS(1793),
    [sym_duet_dev_type_ups] = ACTIONS(1793),
    [sym_duet_dev_type_amplifier] = ACTIONS(1793),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1793),
    [sym_duet_dev_type_camera] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1793),
    [sym_duet_dev_type_dss] = ACTIONS(1793),
    [sym_duet_dev_type_dvr] = ACTIONS(1793),
    [sym_duet_dev_type_disc_device] = ACTIONS(1793),
    [sym_duet_dev_type_document_camera] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1793),
    [sym_duet_dev_type_hvac] = ACTIONS(1793),
    [sym_duet_dev_type_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_light] = ACTIONS(1793),
    [sym_duet_dev_type_monitor] = ACTIONS(1793),
    [sym_duet_dev_type_motor] = ACTIONS(1793),
    [sym_duet_dev_type_multi_window] = ACTIONS(1793),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1793),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1793),
    [sym_duet_dev_type_receiver] = ACTIONS(1793),
    [sym_duet_dev_type_security_system] = ACTIONS(1793),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1793),
    [sym_duet_dev_type_settop_box] = ACTIONS(1793),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1793),
    [sym_duet_dev_type_switcher] = ACTIONS(1793),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_tv] = ACTIONS(1793),
    [sym_duet_dev_type_utility] = ACTIONS(1793),
    [sym_duet_dev_type_vcr] = ACTIONS(1793),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_video_processor] = ACTIONS(1793),
    [sym_duet_dev_type_video_projector] = ACTIONS(1793),
    [sym_duet_dev_type_video_wall] = ACTIONS(1793),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1793),
    [sym_duet_dev_type_weather] = ACTIONS(1793),
    [sym_duet_dev_not_polled] = ACTIONS(1793),
    [sym_duet_dev_polled] = ACTIONS(1793),
    [sym_clkmgr_mode_network] = ACTIONS(1793),
    [sym_clkmgr_mode_standalone] = ACTIONS(1793),
    [sym_normal_standby] = ACTIONS(1793),
    [sym_normal_wake] = ACTIONS(1793),
    [sym_ip_multicast_ttl_option] = ACTIONS(1793),
    [sym_ip_ttl_subnet] = ACTIONS(1793),
    [sym_ip_ttl_site] = ACTIONS(1793),
    [sym_ip_ttl_region] = ACTIONS(1793),
    [sym_ip_ttl_continent] = ACTIONS(1793),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1793),
    [sym_ip_nodelay_on] = ACTIONS(1793),
    [sym_ip_nodelay_off] = ACTIONS(1793),
    [sym_amx_error] = ACTIONS(1793),
    [sym_amx_warning] = ACTIONS(1793),
    [sym_amx_info] = ACTIONS(1793),
    [sym_amx_debug] = ACTIONS(1793),
    [sym_netlinx_login_success] = ACTIONS(1793),
    [sym_netlinx_login_fail] = ACTIONS(1793),
    [sym_netlinx_logout] = ACTIONS(1793),
    [sym_null_str] = ACTIONS(1793),
    [sym_smtp_address] = ACTIONS(1793),
    [sym_smtp_port_number] = ACTIONS(1793),
    [sym_smtp_username] = ACTIONS(1793),
    [sym_smtp_password] = ACTIONS(1793),
    [sym_smtp_require_tls] = ACTIONS(1793),
    [sym_smtp_from] = ACTIONS(1793),
    [sym_smtp_tls_true] = ACTIONS(1793),
    [sym_smtp_tls_false] = ACTIONS(1793),
    [sym_listview_on_row_select_event] = ACTIONS(1793),
    [sym_data_structure_unknown] = ACTIONS(1793),
    [sym_data_structure_datafeed] = ACTIONS(1793),
    [sym_data_structure_datarecordset] = ACTIONS(1793),
    [sym_data_structure_datarecord] = ACTIONS(1793),
    [sym_data_structure_datafield] = ACTIONS(1793),
    [sym_data_max_content_field_count] = ACTIONS(1793),
    [sym_data_max_metadata_field_count] = ACTIONS(1793),
    [sym_data_max_id_length] = ACTIONS(1793),
    [sym_data_max_name_length] = ACTIONS(1793),
    [sym_data_max_description_length] = ACTIONS(1793),
    [sym_data_max_type_length] = ACTIONS(1793),
    [sym_data_max_format_length] = ACTIONS(1793),
    [sym_data_max_source_length] = ACTIONS(1793),
    [sym_data_max_value_length] = ACTIONS(1793),
    [sym_data_max_label_length] = ACTIONS(1793),
    [sym_data_max_url_length] = ACTIONS(1793),
    [sym_max_last_login_info_length] = ACTIONS(1793),
    [sym_data_type_unknown] = ACTIONS(1793),
    [sym_data_type_string] = ACTIONS(1793),
    [sym_data_type_datetime] = ACTIONS(1793),
    [sym_data_type_date] = ACTIONS(1793),
    [sym_data_type_time] = ACTIONS(1793),
    [sym_data_type_image] = ACTIONS(1793),
    [sym_data_format_url] = ACTIONS(1793),
    [sym_data_format_phone] = ACTIONS(1793),
    [sym_data_format_email] = ACTIONS(1793),
    [sym_data_format_iso8601] = ACTIONS(1793),
    [sym_auditlog_priv] = ACTIONS(1793),
    [sym_configuration_priv] = ACTIONS(1793),
    [sym_deviceconfig_priv] = ACTIONS(1793),
    [sym_ftp_priv] = ACTIONS(1793),
    [sym_http_priv] = ACTIONS(1793),
    [sym_networkconfig_priv] = ACTIONS(1793),
    [sym_programport_priv] = ACTIONS(1793),
    [sym_remoteui_priv] = ACTIONS(1793),
    [sym_securitycontrol_priv] = ACTIONS(1793),
    [sym_softwaremanagement_priv] = ACTIONS(1793),
    [sym_terminal_priv] = ACTIONS(1793),
    [sym_tpadmin_priv] = ACTIONS(1793),
    [sym_user1_priv] = ACTIONS(1793),
    [sym_user2_priv] = ACTIONS(1793),
    [sym_user3_priv] = ACTIONS(1793),
    [sym_user4_priv] = ACTIONS(1793),
    [sym_usermanagement_priv] = ACTIONS(1793),
    [sym_valid_account] = ACTIONS(1793),
    [sym_err_name_invalid] = ACTIONS(1793),
    [sym_err_password_invalid] = ACTIONS(1793),
    [sym_err_invalid_account] = ACTIONS(1793),
    [sym_err_no_authorization] = ACTIONS(1793),
    [sym_err_invalid_parameter] = ACTIONS(1793),
    [sym_err_user_locked_out] = ACTIONS(1793),
    [sym_err_user_account_expired] = ACTIONS(1793),
    [sym_err_general_error] = ACTIONS(1793),
    [sym_err_authentication_error] = ACTIONS(1793),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1793),
    [sym_tls_validate_certificate] = ACTIONS(1793),
    [sym_tdata] = ACTIONS(1793),
    [sym_tchannel] = ACTIONS(1793),
    [sym_tlevel] = ACTIONS(1793),
    [sym_tbutton] = ACTIONS(1793),
    [sym_ttimeline] = ACTIONS(1793),
    [sym_tcustom] = ACTIONS(1793),
    [sym_url_struct] = ACTIONS(1793),
    [sym_dns_struct] = ACTIONS(1793),
    [sym_ip_address_struct] = ACTIONS(1793),
    [sym_dev_info_struct] = ACTIONS(1793),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1793),
    [sym_data_feed] = ACTIONS(1793),
    [sym_data_field] = ACTIONS(1793),
    [sym_data_record] = ACTIONS(1793),
    [sym_wc_data_feed] = ACTIONS(1793),
    [sym_wc_data_field] = ACTIONS(1793),
    [sym_wc_data_record] = ACTIONS(1793),
    [sym_last_login_info] = ACTIONS(1793),
    [sym____reserved___] = ACTIONS(1793),
    [sym_date] = ACTIONS(1793),
    [sym_day] = ACTIONS(1793),
    [sym_time] = ACTIONS(1793),
    [sym_ldate] = ACTIONS(1793),
    [sym_push_device] = ACTIONS(1793),
    [sym_push_channel] = ACTIONS(1793),
    [sym_push_devchan] = ACTIONS(1793),
    [sym_release_device] = ACTIONS(1793),
    [sym_release_channel] = ACTIONS(1793),
    [sym_release_devchan] = ACTIONS(1793),
    [sym_master_slot] = ACTIONS(1793),
    [sym_get_pulse_time] = ACTIONS(1793),
    [sym_get_timer] = ACTIONS(1793),
    [sym_master_sn] = ACTIONS(1793),
    [sym_system_number] = ACTIONS(1793),
    [sym_button] = ACTIONS(1793),
    [sym_dv_channel] = ACTIONS(1793),
    [sym_data] = ACTIONS(1793),
    [sym_level] = ACTIONS(1793),
    [sym_timeline] = ACTIONS(1793),
    [sym_channel] = ACTIONS(1793),
    [sym_custom] = ACTIONS(1793),
    [sym_atoi] = ACTIONS(1793),
    [sym_atol] = ACTIONS(1793),
    [sym_atof] = ACTIONS(1793),
    [sym_itoa] = ACTIONS(1793),
    [sym_itohex] = ACTIONS(1793),
    [sym_hextoi] = ACTIONS(1793),
    [sym_ftoa] = ACTIONS(1793),
    [sym_find_string] = ACTIONS(1793),
    [sym_left_string] = ACTIONS(1793),
    [sym_right_string] = ACTIONS(1793),
    [sym_mid_string] = ACTIONS(1793),
    [sym_remove_string] = ACTIONS(1793),
    [sym_get_buffer_string] = ACTIONS(1793),
    [sym_get_buffer_char] = ACTIONS(1793),
    [sym_get_multi_buffer_string] = ACTIONS(1793),
    [sym_length_string] = ACTIONS(1793),
    [sym_max_length_string] = ACTIONS(1793),
    [sym_set_length_string] = ACTIONS(1793),
    [sym_compare_string] = ACTIONS(1793),
    [sym_upper_string] = ACTIONS(1793),
    [sym_lower_string] = ACTIONS(1793),
    [sym_get_last] = ACTIONS(1793),
    [sym_date_to_day] = ACTIONS(1793),
    [sym_date_to_month] = ACTIONS(1793),
    [sym_date_to_year] = ACTIONS(1793),
    [sym_time_to_hour] = ACTIONS(1793),
    [sym_time_to_minute] = ACTIONS(1793),
    [sym_time_to_second] = ACTIONS(1793),
    [sym_day_of_week] = ACTIONS(1793),
    [sym_set_pulse_time] = ACTIONS(1793),
    [sym_set_timer] = ACTIONS(1793),
    [sym_file_open] = ACTIONS(1793),
    [sym_file_close] = ACTIONS(1793),
    [sym_file_read] = ACTIONS(1793),
    [sym_file_read_line] = ACTIONS(1793),
    [sym_file_write] = ACTIONS(1793),
    [sym_file_write_line] = ACTIONS(1793),
    [sym_file_copy] = ACTIONS(1793),
    [sym_file_delete] = ACTIONS(1793),
    [sym_file_rename] = ACTIONS(1793),
    [sym_file_dir] = ACTIONS(1793),
    [sym_file_getdir] = ACTIONS(1793),
    [sym_file_setdir] = ACTIONS(1793),
    [sym_file_create_dir] = ACTIONS(1793),
    [sym_file_remove_dir] = ACTIONS(1793),
    [sym_file_seek] = ACTIONS(1793),
    [sym_length_array] = ACTIONS(1793),
    [sym_max_length_array] = ACTIONS(1793),
    [sym_set_length_array] = ACTIONS(1793),
    [sym_ip_client_open] = ACTIONS(1793),
    [sym_ip_client_close] = ACTIONS(1793),
    [sym_ip_server_open] = ACTIONS(1793),
    [sym_ip_server_close] = ACTIONS(1793),
    [sym_ip_bound_client_open] = ACTIONS(1793),
    [sym_ip_mc_server_open] = ACTIONS(1793),
    [sym_ip_set_option] = ACTIONS(1793),
    [sym_ssh_client_open] = ACTIONS(1793),
    [sym_ssh_client_close] = ACTIONS(1793),
    [sym_tls_client_open] = ACTIONS(1793),
    [sym_tls_client_close] = ACTIONS(1793),
    [sym_do_push] = ACTIONS(1793),
    [sym_do_push_timed] = ACTIONS(1793),
    [sym_do_release] = ACTIONS(1793),
    [sym_device_id] = ACTIONS(1793),
    [sym_device_info] = ACTIONS(1793),
    [sym_device_id_string] = ACTIONS(1793),
    [sym_device_standby] = ACTIONS(1793),
    [sym_device_wake] = ACTIONS(1793),
    [sym_timeline_create] = ACTIONS(1793),
    [sym_timeline_kill] = ACTIONS(1793),
    [sym_timeline_reload] = ACTIONS(1793),
    [sym_timeline_pause] = ACTIONS(1793),
    [sym_timeline_restart] = ACTIONS(1793),
    [sym_timeline_set] = ACTIONS(1793),
    [sym_timeline_get] = ACTIONS(1793),
    [sym_timeline_active] = ACTIONS(1793),
    [sym_timeline_dynamic_id] = ACTIONS(1793),
    [sym_variable_to_string] = ACTIONS(1793),
    [sym_string_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_string] = ACTIONS(1793),
    [sym_type_cast] = ACTIONS(1793),
    [sym_variable_to_xml] = ACTIONS(1793),
    [sym_xml_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_xml] = ACTIONS(1793),
    [sym_redirect_string] = ACTIONS(1793),
    [sym_random_number] = ACTIONS(1793),
    [sym_get_unique_id] = ACTIONS(1793),
    [sym_get_system_number] = ACTIONS(1793),
    [sym_set_system_number] = ACTIONS(1793),
    [sym_get_serial_number] = ACTIONS(1793),
    [sym_get_master_build] = ACTIONS(1793),
    [sym_set_validation_code] = ACTIONS(1793),
    [sym_reboot] = ACTIONS(1793),
    [sym_load_duet_module] = ACTIONS(1793),
    [sym_unload_duet_module] = ACTIONS(1793),
    [sym_duet_mem_size_set] = ACTIONS(1793),
    [sym_duet_mem_size_get] = ACTIONS(1793),
    [sym_dynamic_application_device] = ACTIONS(1793),
    [sym_dynamic_polled_port] = ACTIONS(1793),
    [sym_static_port_binding] = ACTIONS(1793),
    [sym_static_ip_binding] = ACTIONS(1793),
    [sym_get_dns_list] = ACTIONS(1793),
    [sym_set_dns_list] = ACTIONS(1793),
    [sym_get_ip_address] = ACTIONS(1793),
    [sym_set_ip_address] = ACTIONS(1793),
    [sym_get_url_list] = ACTIONS(1793),
    [sym_add_url_entry] = ACTIONS(1793),
    [sym_delete_url_entry] = ACTIONS(1793),
    [sym_abs_value] = ACTIONS(1793),
    [sym_max_value] = ACTIONS(1793),
    [sym_min_value] = ACTIONS(1793),
    [sym_exp_value] = ACTIONS(1793),
    [sym_log_value] = ACTIONS(1793),
    [sym_log10_value] = ACTIONS(1793),
    [sym_power_value] = ACTIONS(1793),
    [sym_sqrt_value] = ACTIONS(1793),
    [sym_raw_be] = ACTIONS(1793),
    [sym_raw_le] = ACTIONS(1793),
    [sym_internal_queue_size_set] = ACTIONS(1793),
    [sym_internal_queue_size_get] = ACTIONS(1793),
    [sym_internal_threshold_set] = ACTIONS(1793),
    [sym_internal_threshold_get] = ACTIONS(1793),
    [sym_rebuilt_event] = ACTIONS(1793),
    [sym_do_custom_event] = ACTIONS(1793),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1793),
    [sym_clkmgr_set_clk_source] = ACTIONS(1793),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1793),
    [sym_clkmgr_get_timezone] = ACTIONS(1793),
    [sym_clkmgr_set_timezone] = ACTIONS(1793),
    [sym_clkmgr_get_resync_period] = ACTIONS(1793),
    [sym_clkmgr_set_resync_period] = ACTIONS(1793),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_timeservers] = ACTIONS(1793),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_set_log_level] = ACTIONS(1793),
    [sym_get_log_level] = ACTIONS(1793),
    [sym_amx_log] = ACTIONS(1793),
    [sym_smtp_server_config_set] = ACTIONS(1793),
    [sym_smtp_server_config_get] = ACTIONS(1793),
    [sym_smtp_send] = ACTIONS(1793),
    [sym_validate_netlinx_account] = ACTIONS(1793),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1793),
    [sym_authenticate_certificate] = ACTIONS(1793),
    [sym_audit_netlinx_session_event] = ACTIONS(1793),
    [sym_audit_netlinx_generic_event] = ACTIONS(1793),
    [sym_get_available_flash_disk_space] = ACTIONS(1793),
    [sym_get_max_flash_disk_space] = ACTIONS(1793),
    [sym_data_create_feed] = ACTIONS(1793),
    [sym_data_delete_feed] = ACTIONS(1793),
    [sym_data_publish_feed] = ACTIONS(1793),
    [sym_data_get_published_feed] = ACTIONS(1793),
    [sym_data_add_record] = ACTIONS(1793),
    [sym_data_get_event_record] = ACTIONS(1793),
    [sym__wc_data_create_feed] = ACTIONS(1793),
    [sym__wc_data_add_record] = ACTIONS(1793),
    [sym__wc_data_get_event_record] = ACTIONS(1793),
    [sym_format] = ACTIONS(1793),
    [sym_astro_clock] = ACTIONS(1793),
    [sym_set_outdoor_temperature] = ACTIONS(1793),
    [sym_set_virtual_level_count] = ACTIONS(1793),
    [sym_set_virtual_channel_count] = ACTIONS(1793),
    [sym_set_virtual_port_count] = ACTIONS(1793),
    [sym__wc_encode_int] = ACTIONS(1793),
    [sym__wc_decode_int] = ACTIONS(1793),
    [sym__wc_int] = ACTIONS(1793),
    [sym__wc_to_ch_int] = ACTIONS(1793),
    [sym__ch_to_wc_int] = ACTIONS(1793),
    [sym__wc_find_string_int] = ACTIONS(1793),
    [sym__wc_left_string_int] = ACTIONS(1793),
    [sym__wc_lower_string_int] = ACTIONS(1793),
    [sym__wc_mid_string_int] = ACTIONS(1793),
    [sym__wc_remove_string_int] = ACTIONS(1793),
    [sym__wc_right_string_int] = ACTIONS(1793),
    [sym__wc_upper_string_int] = ACTIONS(1793),
    [sym__wc_compare_string_int] = ACTIONS(1793),
    [sym__wc_get_buffer_char_int] = ACTIONS(1793),
    [sym__wc_get_buffer_string_int] = ACTIONS(1793),
    [sym__wc_concat_string_int] = ACTIONS(1793),
    [sym__wc_file_open_int] = ACTIONS(1793),
    [sym__wc_file_close_int] = ACTIONS(1793),
    [sym__wc_file_read_int] = ACTIONS(1793),
    [sym__wc_file_read_line_int] = ACTIONS(1793),
    [sym__wc_file_write_int] = ACTIONS(1793),
    [sym__wc_file_write_line_int] = ACTIONS(1793),
    [aux_sym_preproc_include_token1] = ACTIONS(1793),
    [aux_sym_preproc_define_token1] = ACTIONS(1795),
    [aux_sym_preproc_warn_token1] = ACTIONS(1795),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1795),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_else_token1] = ACTIONS(1795),
    [anon_sym_LBRACK] = ACTIONS(1795),
    [anon_sym_LBRACE] = ACTIONS(1795),
    [anon_sym_LPAREN] = ACTIONS(1793),
    [anon_sym_BANG] = ACTIONS(1795),
    [anon_sym_TILDE] = ACTIONS(1795),
    [anon_sym_DASH] = ACTIONS(1793),
    [anon_sym_PLUS] = ACTIONS(1793),
    [anon_sym_DASH_DASH] = ACTIONS(1795),
    [anon_sym_PLUS_PLUS] = ACTIONS(1795),
    [anon_sym_DQUOTE] = ACTIONS(1795),
    [anon_sym_SQUOTE] = ACTIONS(1795),
    [sym_number_literal] = ACTIONS(1795),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(243)] = {
    [sym_identifier] = ACTIONS(1793),
    [sym_program_name_keyword] = ACTIONS(1793),
    [sym_module_name_keyword] = ACTIONS(1793),
    [sym_define_device_keyword] = ACTIONS(1793),
    [sym_define_combine_keyword] = ACTIONS(1793),
    [sym_define_constant_keyword] = ACTIONS(1793),
    [sym_define_type_keyword] = ACTIONS(1793),
    [sym_define_variable_keyword] = ACTIONS(1793),
    [sym_define_system_variable_keyword] = ACTIONS(1793),
    [sym_define_connect_level_keyword] = ACTIONS(1793),
    [sym_define_latching_keyword] = ACTIONS(1793),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1793),
    [sym_define_toggling_keyword] = ACTIONS(1793),
    [sym_define_call_keyword] = ACTIONS(1793),
    [sym_define_function_keyword] = ACTIONS(1793),
    [sym_define_library_function_keyword] = ACTIONS(1793),
    [sym_define_start_keyword] = ACTIONS(1793),
    [sym_define_event_keyword] = ACTIONS(1793),
    [sym_define_program_keyword] = ACTIONS(1793),
    [sym_define_module_keyword] = ACTIONS(1793),
    [sym_char_keyword] = ACTIONS(1793),
    [sym_widechar_keyword] = ACTIONS(1793),
    [sym_integer_keyword] = ACTIONS(1793),
    [sym_sinteger_keyword] = ACTIONS(1793),
    [sym_long_keyword] = ACTIONS(1793),
    [sym_slong_keyword] = ACTIONS(1793),
    [sym_float_keyword] = ACTIONS(1793),
    [sym_double_keyword] = ACTIONS(1793),
    [sym_variant_keyword] = ACTIONS(1793),
    [sym_variantarray_keyword] = ACTIONS(1793),
    [sym_local_var_keyword] = ACTIONS(1793),
    [sym_stack_var_keyword] = ACTIONS(1793),
    [sym_constant_keyword] = ACTIONS(1793),
    [sym_volatile_keyword] = ACTIONS(1793),
    [sym_non_volatile_keyword] = ACTIONS(1793),
    [sym_persistent_keyword] = ACTIONS(1793),
    [sym_dev_keyword] = ACTIONS(1793),
    [sym_devlev_keyword] = ACTIONS(1793),
    [sym_devchan_keyword] = ACTIONS(1793),
    [sym_if_keyword] = ACTIONS(1793),
    [sym_else_keyword] = ACTIONS(1793),
    [sym_switch_keyword] = ACTIONS(1793),
    [sym_case_keyword] = ACTIONS(1793),
    [sym_default_keyword] = ACTIONS(1793),
    [sym_while_keyword] = ACTIONS(1793),
    [sym_for_keyword] = ACTIONS(1793),
    [sym_break_keyword] = ACTIONS(1793),
    [sym_continue_keyword] = ACTIONS(1793),
    [sym_return_keyword] = ACTIONS(1793),
    [sym_select_keyword] = ACTIONS(1793),
    [sym_active_keyword] = ACTIONS(1793),
    [sym_struct_keyword] = ACTIONS(1793),
    [sym_structure_keyword] = ACTIONS(1793),
    [sym_button_event_keyword] = ACTIONS(1793),
    [sym_channel_event_keyword] = ACTIONS(1793),
    [sym_level_event_keyword] = ACTIONS(1793),
    [sym_data_event_keyword] = ACTIONS(1793),
    [sym_timeline_event_keyword] = ACTIONS(1793),
    [sym_custom_event_keyword] = ACTIONS(1793),
    [sym_push_keyword] = ACTIONS(1793),
    [sym_release_keyword] = ACTIONS(1793),
    [aux_sym_on_keyword_token1] = ACTIONS(1793),
    [aux_sym_off_keyword_token1] = ACTIONS(1793),
    [sym_send_level_keyword] = ACTIONS(1793),
    [sym_send_string_keyword] = ACTIONS(1793),
    [sym_send_command_keyword] = ACTIONS(1793),
    [sym_clear_buffer_keyword] = ACTIONS(1793),
    [sym_create_buffer_keyword] = ACTIONS(1793),
    [sym_create_multi_buffer_keyword] = ACTIONS(1793),
    [sym_call_keyword] = ACTIONS(1793),
    [sym_system_call_keyword] = ACTIONS(1793),
    [sym_devchan_to_keyword] = ACTIONS(1793),
    [sym_devchan_min_to_keyword] = ACTIONS(1793),
    [sym_devchan_total_off_keyword] = ACTIONS(1793),
    [sym_devchan_pulse_keyword] = ACTIONS(1793),
    [sym_wait_keyword] = ACTIONS(1793),
    [sym_cancel_wait_keyword] = ACTIONS(1793),
    [sym_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1793),
    [sym_not] = ACTIONS(1793),
    [sym_bnot] = ACTIONS(1793),
    [sym___file__] = ACTIONS(1793),
    [sym___date__] = ACTIONS(1793),
    [sym___ldate__] = ACTIONS(1793),
    [sym___time__] = ACTIONS(1793),
    [sym___line__] = ACTIONS(1793),
    [sym___name__] = ACTIONS(1793),
    [sym___netlinx__] = ACTIONS(1793),
    [sym_netlinx_axi_version] = ACTIONS(1793),
    [sym_url_flg_tcp] = ACTIONS(1793),
    [sym_url_flg_accinfopresent] = ACTIONS(1793),
    [sym_url_flg_temp] = ACTIONS(1793),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1793),
    [sym_url_flg_stat_mask] = ACTIONS(1793),
    [sym_url_flg_stat_lookup] = ACTIONS(1793),
    [sym_url_flg_stat_connecting] = ACTIONS(1793),
    [sym_url_flg_stat_waiting] = ACTIONS(1793),
    [sym_url_flg_stat_connected] = ACTIONS(1793),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1793),
    [sym_first_virtual_device] = ACTIONS(1793),
    [sym_first_local_port] = ACTIONS(1793),
    [sym_dynamic_virtual_device] = ACTIONS(1793),
    [sym_timeline_once] = ACTIONS(1793),
    [sym_timeline_repeat] = ACTIONS(1793),
    [sym_timeline_absolute] = ACTIONS(1793),
    [sym_timeline_relative] = ACTIONS(1793),
    [sym_do_push_timed_infinite] = ACTIONS(1793),
    [sym_source_type_no_address] = ACTIONS(1793),
    [sym_source_type_neuron_id] = ACTIONS(1793),
    [sym_source_type_ip_address] = ACTIONS(1793),
    [sym_source_type_axlink] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1793),
    [sym_source_type_ip_socket_address] = ACTIONS(1793),
    [sym_source_type_rs232] = ACTIONS(1793),
    [sym_source_type_internal] = ACTIONS(1793),
    [sym_source_type_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1793),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1793),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1793),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1793),
    [sym_true] = ACTIONS(1793),
    [sym_false] = ACTIONS(1793),
    [sym_file_read_only] = ACTIONS(1793),
    [sym_file_rw_new] = ACTIONS(1793),
    [sym_file_rw_append] = ACTIONS(1793),
    [sym_ip_tcp] = ACTIONS(1793),
    [sym_ip_udp] = ACTIONS(1793),
    [sym_ip_udp_2way] = ACTIONS(1793),
    [sym_xml_encode_types] = ACTIONS(1793),
    [sym_xml_encode_char_as_list] = ACTIONS(1793),
    [sym_xml_encode_le] = ACTIONS(1793),
    [sym_xml_decode_types] = ACTIONS(1793),
    [sym_xml_decode_no_preserve] = ACTIONS(1793),
    [sym_duet_dev_type_display_device] = ACTIONS(1793),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_io_device] = ACTIONS(1793),
    [sym_duet_dev_type_relay_device] = ACTIONS(1793),
    [sym_duet_dev_type_ups] = ACTIONS(1793),
    [sym_duet_dev_type_amplifier] = ACTIONS(1793),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1793),
    [sym_duet_dev_type_camera] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1793),
    [sym_duet_dev_type_dss] = ACTIONS(1793),
    [sym_duet_dev_type_dvr] = ACTIONS(1793),
    [sym_duet_dev_type_disc_device] = ACTIONS(1793),
    [sym_duet_dev_type_document_camera] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1793),
    [sym_duet_dev_type_hvac] = ACTIONS(1793),
    [sym_duet_dev_type_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_light] = ACTIONS(1793),
    [sym_duet_dev_type_monitor] = ACTIONS(1793),
    [sym_duet_dev_type_motor] = ACTIONS(1793),
    [sym_duet_dev_type_multi_window] = ACTIONS(1793),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1793),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1793),
    [sym_duet_dev_type_receiver] = ACTIONS(1793),
    [sym_duet_dev_type_security_system] = ACTIONS(1793),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1793),
    [sym_duet_dev_type_settop_box] = ACTIONS(1793),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1793),
    [sym_duet_dev_type_switcher] = ACTIONS(1793),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_tv] = ACTIONS(1793),
    [sym_duet_dev_type_utility] = ACTIONS(1793),
    [sym_duet_dev_type_vcr] = ACTIONS(1793),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_video_processor] = ACTIONS(1793),
    [sym_duet_dev_type_video_projector] = ACTIONS(1793),
    [sym_duet_dev_type_video_wall] = ACTIONS(1793),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1793),
    [sym_duet_dev_type_weather] = ACTIONS(1793),
    [sym_duet_dev_not_polled] = ACTIONS(1793),
    [sym_duet_dev_polled] = ACTIONS(1793),
    [sym_clkmgr_mode_network] = ACTIONS(1793),
    [sym_clkmgr_mode_standalone] = ACTIONS(1793),
    [sym_normal_standby] = ACTIONS(1793),
    [sym_normal_wake] = ACTIONS(1793),
    [sym_ip_multicast_ttl_option] = ACTIONS(1793),
    [sym_ip_ttl_subnet] = ACTIONS(1793),
    [sym_ip_ttl_site] = ACTIONS(1793),
    [sym_ip_ttl_region] = ACTIONS(1793),
    [sym_ip_ttl_continent] = ACTIONS(1793),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1793),
    [sym_ip_nodelay_on] = ACTIONS(1793),
    [sym_ip_nodelay_off] = ACTIONS(1793),
    [sym_amx_error] = ACTIONS(1793),
    [sym_amx_warning] = ACTIONS(1793),
    [sym_amx_info] = ACTIONS(1793),
    [sym_amx_debug] = ACTIONS(1793),
    [sym_netlinx_login_success] = ACTIONS(1793),
    [sym_netlinx_login_fail] = ACTIONS(1793),
    [sym_netlinx_logout] = ACTIONS(1793),
    [sym_null_str] = ACTIONS(1793),
    [sym_smtp_address] = ACTIONS(1793),
    [sym_smtp_port_number] = ACTIONS(1793),
    [sym_smtp_username] = ACTIONS(1793),
    [sym_smtp_password] = ACTIONS(1793),
    [sym_smtp_require_tls] = ACTIONS(1793),
    [sym_smtp_from] = ACTIONS(1793),
    [sym_smtp_tls_true] = ACTIONS(1793),
    [sym_smtp_tls_false] = ACTIONS(1793),
    [sym_listview_on_row_select_event] = ACTIONS(1793),
    [sym_data_structure_unknown] = ACTIONS(1793),
    [sym_data_structure_datafeed] = ACTIONS(1793),
    [sym_data_structure_datarecordset] = ACTIONS(1793),
    [sym_data_structure_datarecord] = ACTIONS(1793),
    [sym_data_structure_datafield] = ACTIONS(1793),
    [sym_data_max_content_field_count] = ACTIONS(1793),
    [sym_data_max_metadata_field_count] = ACTIONS(1793),
    [sym_data_max_id_length] = ACTIONS(1793),
    [sym_data_max_name_length] = ACTIONS(1793),
    [sym_data_max_description_length] = ACTIONS(1793),
    [sym_data_max_type_length] = ACTIONS(1793),
    [sym_data_max_format_length] = ACTIONS(1793),
    [sym_data_max_source_length] = ACTIONS(1793),
    [sym_data_max_value_length] = ACTIONS(1793),
    [sym_data_max_label_length] = ACTIONS(1793),
    [sym_data_max_url_length] = ACTIONS(1793),
    [sym_max_last_login_info_length] = ACTIONS(1793),
    [sym_data_type_unknown] = ACTIONS(1793),
    [sym_data_type_string] = ACTIONS(1793),
    [sym_data_type_datetime] = ACTIONS(1793),
    [sym_data_type_date] = ACTIONS(1793),
    [sym_data_type_time] = ACTIONS(1793),
    [sym_data_type_image] = ACTIONS(1793),
    [sym_data_format_url] = ACTIONS(1793),
    [sym_data_format_phone] = ACTIONS(1793),
    [sym_data_format_email] = ACTIONS(1793),
    [sym_data_format_iso8601] = ACTIONS(1793),
    [sym_auditlog_priv] = ACTIONS(1793),
    [sym_configuration_priv] = ACTIONS(1793),
    [sym_deviceconfig_priv] = ACTIONS(1793),
    [sym_ftp_priv] = ACTIONS(1793),
    [sym_http_priv] = ACTIONS(1793),
    [sym_networkconfig_priv] = ACTIONS(1793),
    [sym_programport_priv] = ACTIONS(1793),
    [sym_remoteui_priv] = ACTIONS(1793),
    [sym_securitycontrol_priv] = ACTIONS(1793),
    [sym_softwaremanagement_priv] = ACTIONS(1793),
    [sym_terminal_priv] = ACTIONS(1793),
    [sym_tpadmin_priv] = ACTIONS(1793),
    [sym_user1_priv] = ACTIONS(1793),
    [sym_user2_priv] = ACTIONS(1793),
    [sym_user3_priv] = ACTIONS(1793),
    [sym_user4_priv] = ACTIONS(1793),
    [sym_usermanagement_priv] = ACTIONS(1793),
    [sym_valid_account] = ACTIONS(1793),
    [sym_err_name_invalid] = ACTIONS(1793),
    [sym_err_password_invalid] = ACTIONS(1793),
    [sym_err_invalid_account] = ACTIONS(1793),
    [sym_err_no_authorization] = ACTIONS(1793),
    [sym_err_invalid_parameter] = ACTIONS(1793),
    [sym_err_user_locked_out] = ACTIONS(1793),
    [sym_err_user_account_expired] = ACTIONS(1793),
    [sym_err_general_error] = ACTIONS(1793),
    [sym_err_authentication_error] = ACTIONS(1793),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1793),
    [sym_tls_validate_certificate] = ACTIONS(1793),
    [sym_tdata] = ACTIONS(1793),
    [sym_tchannel] = ACTIONS(1793),
    [sym_tlevel] = ACTIONS(1793),
    [sym_tbutton] = ACTIONS(1793),
    [sym_ttimeline] = ACTIONS(1793),
    [sym_tcustom] = ACTIONS(1793),
    [sym_url_struct] = ACTIONS(1793),
    [sym_dns_struct] = ACTIONS(1793),
    [sym_ip_address_struct] = ACTIONS(1793),
    [sym_dev_info_struct] = ACTIONS(1793),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1793),
    [sym_data_feed] = ACTIONS(1793),
    [sym_data_field] = ACTIONS(1793),
    [sym_data_record] = ACTIONS(1793),
    [sym_wc_data_feed] = ACTIONS(1793),
    [sym_wc_data_field] = ACTIONS(1793),
    [sym_wc_data_record] = ACTIONS(1793),
    [sym_last_login_info] = ACTIONS(1793),
    [sym____reserved___] = ACTIONS(1793),
    [sym_date] = ACTIONS(1793),
    [sym_day] = ACTIONS(1793),
    [sym_time] = ACTIONS(1793),
    [sym_ldate] = ACTIONS(1793),
    [sym_push_device] = ACTIONS(1793),
    [sym_push_channel] = ACTIONS(1793),
    [sym_push_devchan] = ACTIONS(1793),
    [sym_release_device] = ACTIONS(1793),
    [sym_release_channel] = ACTIONS(1793),
    [sym_release_devchan] = ACTIONS(1793),
    [sym_master_slot] = ACTIONS(1793),
    [sym_get_pulse_time] = ACTIONS(1793),
    [sym_get_timer] = ACTIONS(1793),
    [sym_master_sn] = ACTIONS(1793),
    [sym_system_number] = ACTIONS(1793),
    [sym_button] = ACTIONS(1793),
    [sym_dv_channel] = ACTIONS(1793),
    [sym_data] = ACTIONS(1793),
    [sym_level] = ACTIONS(1793),
    [sym_timeline] = ACTIONS(1793),
    [sym_channel] = ACTIONS(1793),
    [sym_custom] = ACTIONS(1793),
    [sym_atoi] = ACTIONS(1793),
    [sym_atol] = ACTIONS(1793),
    [sym_atof] = ACTIONS(1793),
    [sym_itoa] = ACTIONS(1793),
    [sym_itohex] = ACTIONS(1793),
    [sym_hextoi] = ACTIONS(1793),
    [sym_ftoa] = ACTIONS(1793),
    [sym_find_string] = ACTIONS(1793),
    [sym_left_string] = ACTIONS(1793),
    [sym_right_string] = ACTIONS(1793),
    [sym_mid_string] = ACTIONS(1793),
    [sym_remove_string] = ACTIONS(1793),
    [sym_get_buffer_string] = ACTIONS(1793),
    [sym_get_buffer_char] = ACTIONS(1793),
    [sym_get_multi_buffer_string] = ACTIONS(1793),
    [sym_length_string] = ACTIONS(1793),
    [sym_max_length_string] = ACTIONS(1793),
    [sym_set_length_string] = ACTIONS(1793),
    [sym_compare_string] = ACTIONS(1793),
    [sym_upper_string] = ACTIONS(1793),
    [sym_lower_string] = ACTIONS(1793),
    [sym_get_last] = ACTIONS(1793),
    [sym_date_to_day] = ACTIONS(1793),
    [sym_date_to_month] = ACTIONS(1793),
    [sym_date_to_year] = ACTIONS(1793),
    [sym_time_to_hour] = ACTIONS(1793),
    [sym_time_to_minute] = ACTIONS(1793),
    [sym_time_to_second] = ACTIONS(1793),
    [sym_day_of_week] = ACTIONS(1793),
    [sym_set_pulse_time] = ACTIONS(1793),
    [sym_set_timer] = ACTIONS(1793),
    [sym_file_open] = ACTIONS(1793),
    [sym_file_close] = ACTIONS(1793),
    [sym_file_read] = ACTIONS(1793),
    [sym_file_read_line] = ACTIONS(1793),
    [sym_file_write] = ACTIONS(1793),
    [sym_file_write_line] = ACTIONS(1793),
    [sym_file_copy] = ACTIONS(1793),
    [sym_file_delete] = ACTIONS(1793),
    [sym_file_rename] = ACTIONS(1793),
    [sym_file_dir] = ACTIONS(1793),
    [sym_file_getdir] = ACTIONS(1793),
    [sym_file_setdir] = ACTIONS(1793),
    [sym_file_create_dir] = ACTIONS(1793),
    [sym_file_remove_dir] = ACTIONS(1793),
    [sym_file_seek] = ACTIONS(1793),
    [sym_length_array] = ACTIONS(1793),
    [sym_max_length_array] = ACTIONS(1793),
    [sym_set_length_array] = ACTIONS(1793),
    [sym_ip_client_open] = ACTIONS(1793),
    [sym_ip_client_close] = ACTIONS(1793),
    [sym_ip_server_open] = ACTIONS(1793),
    [sym_ip_server_close] = ACTIONS(1793),
    [sym_ip_bound_client_open] = ACTIONS(1793),
    [sym_ip_mc_server_open] = ACTIONS(1793),
    [sym_ip_set_option] = ACTIONS(1793),
    [sym_ssh_client_open] = ACTIONS(1793),
    [sym_ssh_client_close] = ACTIONS(1793),
    [sym_tls_client_open] = ACTIONS(1793),
    [sym_tls_client_close] = ACTIONS(1793),
    [sym_do_push] = ACTIONS(1793),
    [sym_do_push_timed] = ACTIONS(1793),
    [sym_do_release] = ACTIONS(1793),
    [sym_device_id] = ACTIONS(1793),
    [sym_device_info] = ACTIONS(1793),
    [sym_device_id_string] = ACTIONS(1793),
    [sym_device_standby] = ACTIONS(1793),
    [sym_device_wake] = ACTIONS(1793),
    [sym_timeline_create] = ACTIONS(1793),
    [sym_timeline_kill] = ACTIONS(1793),
    [sym_timeline_reload] = ACTIONS(1793),
    [sym_timeline_pause] = ACTIONS(1793),
    [sym_timeline_restart] = ACTIONS(1793),
    [sym_timeline_set] = ACTIONS(1793),
    [sym_timeline_get] = ACTIONS(1793),
    [sym_timeline_active] = ACTIONS(1793),
    [sym_timeline_dynamic_id] = ACTIONS(1793),
    [sym_variable_to_string] = ACTIONS(1793),
    [sym_string_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_string] = ACTIONS(1793),
    [sym_type_cast] = ACTIONS(1793),
    [sym_variable_to_xml] = ACTIONS(1793),
    [sym_xml_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_xml] = ACTIONS(1793),
    [sym_redirect_string] = ACTIONS(1793),
    [sym_random_number] = ACTIONS(1793),
    [sym_get_unique_id] = ACTIONS(1793),
    [sym_get_system_number] = ACTIONS(1793),
    [sym_set_system_number] = ACTIONS(1793),
    [sym_get_serial_number] = ACTIONS(1793),
    [sym_get_master_build] = ACTIONS(1793),
    [sym_set_validation_code] = ACTIONS(1793),
    [sym_reboot] = ACTIONS(1793),
    [sym_load_duet_module] = ACTIONS(1793),
    [sym_unload_duet_module] = ACTIONS(1793),
    [sym_duet_mem_size_set] = ACTIONS(1793),
    [sym_duet_mem_size_get] = ACTIONS(1793),
    [sym_dynamic_application_device] = ACTIONS(1793),
    [sym_dynamic_polled_port] = ACTIONS(1793),
    [sym_static_port_binding] = ACTIONS(1793),
    [sym_static_ip_binding] = ACTIONS(1793),
    [sym_get_dns_list] = ACTIONS(1793),
    [sym_set_dns_list] = ACTIONS(1793),
    [sym_get_ip_address] = ACTIONS(1793),
    [sym_set_ip_address] = ACTIONS(1793),
    [sym_get_url_list] = ACTIONS(1793),
    [sym_add_url_entry] = ACTIONS(1793),
    [sym_delete_url_entry] = ACTIONS(1793),
    [sym_abs_value] = ACTIONS(1793),
    [sym_max_value] = ACTIONS(1793),
    [sym_min_value] = ACTIONS(1793),
    [sym_exp_value] = ACTIONS(1793),
    [sym_log_value] = ACTIONS(1793),
    [sym_log10_value] = ACTIONS(1793),
    [sym_power_value] = ACTIONS(1793),
    [sym_sqrt_value] = ACTIONS(1793),
    [sym_raw_be] = ACTIONS(1793),
    [sym_raw_le] = ACTIONS(1793),
    [sym_internal_queue_size_set] = ACTIONS(1793),
    [sym_internal_queue_size_get] = ACTIONS(1793),
    [sym_internal_threshold_set] = ACTIONS(1793),
    [sym_internal_threshold_get] = ACTIONS(1793),
    [sym_rebuilt_event] = ACTIONS(1793),
    [sym_do_custom_event] = ACTIONS(1793),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1793),
    [sym_clkmgr_set_clk_source] = ACTIONS(1793),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1793),
    [sym_clkmgr_get_timezone] = ACTIONS(1793),
    [sym_clkmgr_set_timezone] = ACTIONS(1793),
    [sym_clkmgr_get_resync_period] = ACTIONS(1793),
    [sym_clkmgr_set_resync_period] = ACTIONS(1793),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_timeservers] = ACTIONS(1793),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_set_log_level] = ACTIONS(1793),
    [sym_get_log_level] = ACTIONS(1793),
    [sym_amx_log] = ACTIONS(1793),
    [sym_smtp_server_config_set] = ACTIONS(1793),
    [sym_smtp_server_config_get] = ACTIONS(1793),
    [sym_smtp_send] = ACTIONS(1793),
    [sym_validate_netlinx_account] = ACTIONS(1793),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1793),
    [sym_authenticate_certificate] = ACTIONS(1793),
    [sym_audit_netlinx_session_event] = ACTIONS(1793),
    [sym_audit_netlinx_generic_event] = ACTIONS(1793),
    [sym_get_available_flash_disk_space] = ACTIONS(1793),
    [sym_get_max_flash_disk_space] = ACTIONS(1793),
    [sym_data_create_feed] = ACTIONS(1793),
    [sym_data_delete_feed] = ACTIONS(1793),
    [sym_data_publish_feed] = ACTIONS(1793),
    [sym_data_get_published_feed] = ACTIONS(1793),
    [sym_data_add_record] = ACTIONS(1793),
    [sym_data_get_event_record] = ACTIONS(1793),
    [sym__wc_data_create_feed] = ACTIONS(1793),
    [sym__wc_data_add_record] = ACTIONS(1793),
    [sym__wc_data_get_event_record] = ACTIONS(1793),
    [sym_format] = ACTIONS(1793),
    [sym_astro_clock] = ACTIONS(1793),
    [sym_set_outdoor_temperature] = ACTIONS(1793),
    [sym_set_virtual_level_count] = ACTIONS(1793),
    [sym_set_virtual_channel_count] = ACTIONS(1793),
    [sym_set_virtual_port_count] = ACTIONS(1793),
    [sym__wc_encode_int] = ACTIONS(1793),
    [sym__wc_decode_int] = ACTIONS(1793),
    [sym__wc_int] = ACTIONS(1793),
    [sym__wc_to_ch_int] = ACTIONS(1793),
    [sym__ch_to_wc_int] = ACTIONS(1793),
    [sym__wc_find_string_int] = ACTIONS(1793),
    [sym__wc_left_string_int] = ACTIONS(1793),
    [sym__wc_lower_string_int] = ACTIONS(1793),
    [sym__wc_mid_string_int] = ACTIONS(1793),
    [sym__wc_remove_string_int] = ACTIONS(1793),
    [sym__wc_right_string_int] = ACTIONS(1793),
    [sym__wc_upper_string_int] = ACTIONS(1793),
    [sym__wc_compare_string_int] = ACTIONS(1793),
    [sym__wc_get_buffer_char_int] = ACTIONS(1793),
    [sym__wc_get_buffer_string_int] = ACTIONS(1793),
    [sym__wc_concat_string_int] = ACTIONS(1793),
    [sym__wc_file_open_int] = ACTIONS(1793),
    [sym__wc_file_close_int] = ACTIONS(1793),
    [sym__wc_file_read_int] = ACTIONS(1793),
    [sym__wc_file_read_line_int] = ACTIONS(1793),
    [sym__wc_file_write_int] = ACTIONS(1793),
    [sym__wc_file_write_line_int] = ACTIONS(1793),
    [aux_sym_preproc_include_token1] = ACTIONS(1793),
    [aux_sym_preproc_define_token1] = ACTIONS(1795),
    [aux_sym_preproc_warn_token1] = ACTIONS(1795),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1795),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_else_token1] = ACTIONS(1795),
    [anon_sym_LBRACK] = ACTIONS(1795),
    [anon_sym_LBRACE] = ACTIONS(1795),
    [anon_sym_LPAREN] = ACTIONS(1793),
    [anon_sym_BANG] = ACTIONS(1795),
    [anon_sym_TILDE] = ACTIONS(1795),
    [anon_sym_DASH] = ACTIONS(1793),
    [anon_sym_PLUS] = ACTIONS(1793),
    [anon_sym_DASH_DASH] = ACTIONS(1795),
    [anon_sym_PLUS_PLUS] = ACTIONS(1795),
    [anon_sym_DQUOTE] = ACTIONS(1795),
    [anon_sym_SQUOTE] = ACTIONS(1795),
    [sym_number_literal] = ACTIONS(1795),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(244)] = {
    [sym_parameter_list] = STATE(292),
    [sym_identifier] = ACTIONS(1797),
    [sym_program_name_keyword] = ACTIONS(1797),
    [sym_module_name_keyword] = ACTIONS(1797),
    [sym_define_device_keyword] = ACTIONS(1797),
    [sym_define_combine_keyword] = ACTIONS(1797),
    [sym_define_constant_keyword] = ACTIONS(1797),
    [sym_define_type_keyword] = ACTIONS(1797),
    [sym_define_variable_keyword] = ACTIONS(1797),
    [sym_define_system_variable_keyword] = ACTIONS(1797),
    [sym_define_connect_level_keyword] = ACTIONS(1797),
    [sym_define_latching_keyword] = ACTIONS(1797),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1797),
    [sym_define_toggling_keyword] = ACTIONS(1797),
    [sym_define_call_keyword] = ACTIONS(1797),
    [sym_define_function_keyword] = ACTIONS(1797),
    [sym_define_library_function_keyword] = ACTIONS(1797),
    [sym_define_start_keyword] = ACTIONS(1797),
    [sym_define_event_keyword] = ACTIONS(1797),
    [sym_define_program_keyword] = ACTIONS(1797),
    [sym_define_module_keyword] = ACTIONS(1797),
    [sym_char_keyword] = ACTIONS(1797),
    [sym_widechar_keyword] = ACTIONS(1797),
    [sym_integer_keyword] = ACTIONS(1797),
    [sym_sinteger_keyword] = ACTIONS(1797),
    [sym_long_keyword] = ACTIONS(1797),
    [sym_slong_keyword] = ACTIONS(1797),
    [sym_float_keyword] = ACTIONS(1797),
    [sym_double_keyword] = ACTIONS(1797),
    [sym_variant_keyword] = ACTIONS(1797),
    [sym_variantarray_keyword] = ACTIONS(1797),
    [sym_local_var_keyword] = ACTIONS(1797),
    [sym_stack_var_keyword] = ACTIONS(1797),
    [sym_constant_keyword] = ACTIONS(1797),
    [sym_volatile_keyword] = ACTIONS(1797),
    [sym_non_volatile_keyword] = ACTIONS(1797),
    [sym_persistent_keyword] = ACTIONS(1797),
    [sym_dev_keyword] = ACTIONS(1797),
    [sym_devlev_keyword] = ACTIONS(1797),
    [sym_devchan_keyword] = ACTIONS(1797),
    [sym_if_keyword] = ACTIONS(1797),
    [sym_switch_keyword] = ACTIONS(1797),
    [sym_case_keyword] = ACTIONS(1797),
    [sym_default_keyword] = ACTIONS(1797),
    [sym_while_keyword] = ACTIONS(1797),
    [sym_for_keyword] = ACTIONS(1797),
    [sym_break_keyword] = ACTIONS(1797),
    [sym_continue_keyword] = ACTIONS(1797),
    [sym_return_keyword] = ACTIONS(1797),
    [sym_select_keyword] = ACTIONS(1797),
    [sym_active_keyword] = ACTIONS(1797),
    [sym_struct_keyword] = ACTIONS(1797),
    [sym_structure_keyword] = ACTIONS(1797),
    [sym_button_event_keyword] = ACTIONS(1797),
    [sym_channel_event_keyword] = ACTIONS(1797),
    [sym_level_event_keyword] = ACTIONS(1797),
    [sym_data_event_keyword] = ACTIONS(1797),
    [sym_timeline_event_keyword] = ACTIONS(1797),
    [sym_custom_event_keyword] = ACTIONS(1797),
    [sym_push_keyword] = ACTIONS(1797),
    [sym_release_keyword] = ACTIONS(1797),
    [aux_sym_on_keyword_token1] = ACTIONS(1797),
    [aux_sym_off_keyword_token1] = ACTIONS(1797),
    [sym_send_level_keyword] = ACTIONS(1797),
    [sym_send_string_keyword] = ACTIONS(1797),
    [sym_send_command_keyword] = ACTIONS(1797),
    [sym_clear_buffer_keyword] = ACTIONS(1797),
    [sym_create_buffer_keyword] = ACTIONS(1797),
    [sym_create_multi_buffer_keyword] = ACTIONS(1797),
    [sym_call_keyword] = ACTIONS(1797),
    [sym_system_call_keyword] = ACTIONS(1797),
    [sym_devchan_to_keyword] = ACTIONS(1797),
    [sym_devchan_min_to_keyword] = ACTIONS(1797),
    [sym_devchan_total_off_keyword] = ACTIONS(1797),
    [sym_devchan_pulse_keyword] = ACTIONS(1797),
    [sym_wait_keyword] = ACTIONS(1797),
    [sym_cancel_wait_keyword] = ACTIONS(1797),
    [sym_wait_until_keyword] = ACTIONS(1797),
    [sym_cancel_wait_until_keyword] = ACTIONS(1797),
    [sym_cancel_all_wait_keyword] = ACTIONS(1797),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1797),
    [sym_not] = ACTIONS(1797),
    [sym_bnot] = ACTIONS(1797),
    [sym___file__] = ACTIONS(1797),
    [sym___date__] = ACTIONS(1797),
    [sym___ldate__] = ACTIONS(1797),
    [sym___time__] = ACTIONS(1797),
    [sym___line__] = ACTIONS(1797),
    [sym___name__] = ACTIONS(1797),
    [sym___netlinx__] = ACTIONS(1797),
    [sym_netlinx_axi_version] = ACTIONS(1797),
    [sym_url_flg_tcp] = ACTIONS(1797),
    [sym_url_flg_accinfopresent] = ACTIONS(1797),
    [sym_url_flg_temp] = ACTIONS(1797),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1797),
    [sym_url_flg_stat_mask] = ACTIONS(1797),
    [sym_url_flg_stat_lookup] = ACTIONS(1797),
    [sym_url_flg_stat_connecting] = ACTIONS(1797),
    [sym_url_flg_stat_waiting] = ACTIONS(1797),
    [sym_url_flg_stat_connected] = ACTIONS(1797),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1797),
    [sym_first_virtual_device] = ACTIONS(1797),
    [sym_first_local_port] = ACTIONS(1797),
    [sym_dynamic_virtual_device] = ACTIONS(1797),
    [sym_timeline_once] = ACTIONS(1797),
    [sym_timeline_repeat] = ACTIONS(1797),
    [sym_timeline_absolute] = ACTIONS(1797),
    [sym_timeline_relative] = ACTIONS(1797),
    [sym_do_push_timed_infinite] = ACTIONS(1797),
    [sym_source_type_no_address] = ACTIONS(1797),
    [sym_source_type_neuron_id] = ACTIONS(1797),
    [sym_source_type_ip_address] = ACTIONS(1797),
    [sym_source_type_axlink] = ACTIONS(1797),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1797),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1797),
    [sym_source_type_ip_socket_address] = ACTIONS(1797),
    [sym_source_type_rs232] = ACTIONS(1797),
    [sym_source_type_internal] = ACTIONS(1797),
    [sym_source_type_mac_address] = ACTIONS(1797),
    [sym_source_type_ipv4_port] = ACTIONS(1797),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1797),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1797),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1797),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1797),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1797),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1797),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1797),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1797),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1797),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1797),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1797),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1797),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1797),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1797),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1797),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1797),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1797),
    [sym_true] = ACTIONS(1797),
    [sym_false] = ACTIONS(1797),
    [sym_file_read_only] = ACTIONS(1797),
    [sym_file_rw_new] = ACTIONS(1797),
    [sym_file_rw_append] = ACTIONS(1797),
    [sym_ip_tcp] = ACTIONS(1797),
    [sym_ip_udp] = ACTIONS(1797),
    [sym_ip_udp_2way] = ACTIONS(1797),
    [sym_xml_encode_types] = ACTIONS(1797),
    [sym_xml_encode_char_as_list] = ACTIONS(1797),
    [sym_xml_encode_le] = ACTIONS(1797),
    [sym_xml_decode_types] = ACTIONS(1797),
    [sym_xml_decode_no_preserve] = ACTIONS(1797),
    [sym_duet_dev_type_display_device] = ACTIONS(1797),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1797),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1797),
    [sym_duet_dev_type_io_device] = ACTIONS(1797),
    [sym_duet_dev_type_relay_device] = ACTIONS(1797),
    [sym_duet_dev_type_ups] = ACTIONS(1797),
    [sym_duet_dev_type_amplifier] = ACTIONS(1797),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1797),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1797),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1797),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1797),
    [sym_duet_dev_type_camera] = ACTIONS(1797),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1797),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1797),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1797),
    [sym_duet_dev_type_dss] = ACTIONS(1797),
    [sym_duet_dev_type_dvr] = ACTIONS(1797),
    [sym_duet_dev_type_disc_device] = ACTIONS(1797),
    [sym_duet_dev_type_document_camera] = ACTIONS(1797),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1797),
    [sym_duet_dev_type_hvac] = ACTIONS(1797),
    [sym_duet_dev_type_keypad] = ACTIONS(1797),
    [sym_duet_dev_type_light] = ACTIONS(1797),
    [sym_duet_dev_type_monitor] = ACTIONS(1797),
    [sym_duet_dev_type_motor] = ACTIONS(1797),
    [sym_duet_dev_type_multi_window] = ACTIONS(1797),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1797),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1797),
    [sym_duet_dev_type_receiver] = ACTIONS(1797),
    [sym_duet_dev_type_security_system] = ACTIONS(1797),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1797),
    [sym_duet_dev_type_settop_box] = ACTIONS(1797),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1797),
    [sym_duet_dev_type_switcher] = ACTIONS(1797),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1797),
    [sym_duet_dev_type_tv] = ACTIONS(1797),
    [sym_duet_dev_type_utility] = ACTIONS(1797),
    [sym_duet_dev_type_vcr] = ACTIONS(1797),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1797),
    [sym_duet_dev_type_video_processor] = ACTIONS(1797),
    [sym_duet_dev_type_video_projector] = ACTIONS(1797),
    [sym_duet_dev_type_video_wall] = ACTIONS(1797),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1797),
    [sym_duet_dev_type_weather] = ACTIONS(1797),
    [sym_duet_dev_not_polled] = ACTIONS(1797),
    [sym_duet_dev_polled] = ACTIONS(1797),
    [sym_clkmgr_mode_network] = ACTIONS(1797),
    [sym_clkmgr_mode_standalone] = ACTIONS(1797),
    [sym_normal_standby] = ACTIONS(1797),
    [sym_normal_wake] = ACTIONS(1797),
    [sym_ip_multicast_ttl_option] = ACTIONS(1797),
    [sym_ip_ttl_subnet] = ACTIONS(1797),
    [sym_ip_ttl_site] = ACTIONS(1797),
    [sym_ip_ttl_region] = ACTIONS(1797),
    [sym_ip_ttl_continent] = ACTIONS(1797),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1797),
    [sym_ip_nodelay_on] = ACTIONS(1797),
    [sym_ip_nodelay_off] = ACTIONS(1797),
    [sym_amx_error] = ACTIONS(1797),
    [sym_amx_warning] = ACTIONS(1797),
    [sym_amx_info] = ACTIONS(1797),
    [sym_amx_debug] = ACTIONS(1797),
    [sym_netlinx_login_success] = ACTIONS(1797),
    [sym_netlinx_login_fail] = ACTIONS(1797),
    [sym_netlinx_logout] = ACTIONS(1797),
    [sym_null_str] = ACTIONS(1797),
    [sym_smtp_address] = ACTIONS(1797),
    [sym_smtp_port_number] = ACTIONS(1797),
    [sym_smtp_username] = ACTIONS(1797),
    [sym_smtp_password] = ACTIONS(1797),
    [sym_smtp_require_tls] = ACTIONS(1797),
    [sym_smtp_from] = ACTIONS(1797),
    [sym_smtp_tls_true] = ACTIONS(1797),
    [sym_smtp_tls_false] = ACTIONS(1797),
    [sym_listview_on_row_select_event] = ACTIONS(1797),
    [sym_data_structure_unknown] = ACTIONS(1797),
    [sym_data_structure_datafeed] = ACTIONS(1797),
    [sym_data_structure_datarecordset] = ACTIONS(1797),
    [sym_data_structure_datarecord] = ACTIONS(1797),
    [sym_data_structure_datafield] = ACTIONS(1797),
    [sym_data_max_content_field_count] = ACTIONS(1797),
    [sym_data_max_metadata_field_count] = ACTIONS(1797),
    [sym_data_max_id_length] = ACTIONS(1797),
    [sym_data_max_name_length] = ACTIONS(1797),
    [sym_data_max_description_length] = ACTIONS(1797),
    [sym_data_max_type_length] = ACTIONS(1797),
    [sym_data_max_format_length] = ACTIONS(1797),
    [sym_data_max_source_length] = ACTIONS(1797),
    [sym_data_max_value_length] = ACTIONS(1797),
    [sym_data_max_label_length] = ACTIONS(1797),
    [sym_data_max_url_length] = ACTIONS(1797),
    [sym_max_last_login_info_length] = ACTIONS(1797),
    [sym_data_type_unknown] = ACTIONS(1797),
    [sym_data_type_string] = ACTIONS(1797),
    [sym_data_type_datetime] = ACTIONS(1797),
    [sym_data_type_date] = ACTIONS(1797),
    [sym_data_type_time] = ACTIONS(1797),
    [sym_data_type_image] = ACTIONS(1797),
    [sym_data_format_url] = ACTIONS(1797),
    [sym_data_format_phone] = ACTIONS(1797),
    [sym_data_format_email] = ACTIONS(1797),
    [sym_data_format_iso8601] = ACTIONS(1797),
    [sym_auditlog_priv] = ACTIONS(1797),
    [sym_configuration_priv] = ACTIONS(1797),
    [sym_deviceconfig_priv] = ACTIONS(1797),
    [sym_ftp_priv] = ACTIONS(1797),
    [sym_http_priv] = ACTIONS(1797),
    [sym_networkconfig_priv] = ACTIONS(1797),
    [sym_programport_priv] = ACTIONS(1797),
    [sym_remoteui_priv] = ACTIONS(1797),
    [sym_securitycontrol_priv] = ACTIONS(1797),
    [sym_softwaremanagement_priv] = ACTIONS(1797),
    [sym_terminal_priv] = ACTIONS(1797),
    [sym_tpadmin_priv] = ACTIONS(1797),
    [sym_user1_priv] = ACTIONS(1797),
    [sym_user2_priv] = ACTIONS(1797),
    [sym_user3_priv] = ACTIONS(1797),
    [sym_user4_priv] = ACTIONS(1797),
    [sym_usermanagement_priv] = ACTIONS(1797),
    [sym_valid_account] = ACTIONS(1797),
    [sym_err_name_invalid] = ACTIONS(1797),
    [sym_err_password_invalid] = ACTIONS(1797),
    [sym_err_invalid_account] = ACTIONS(1797),
    [sym_err_no_authorization] = ACTIONS(1797),
    [sym_err_invalid_parameter] = ACTIONS(1797),
    [sym_err_user_locked_out] = ACTIONS(1797),
    [sym_err_user_account_expired] = ACTIONS(1797),
    [sym_err_general_error] = ACTIONS(1797),
    [sym_err_authentication_error] = ACTIONS(1797),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1797),
    [sym_tls_validate_certificate] = ACTIONS(1797),
    [sym_tdata] = ACTIONS(1797),
    [sym_tchannel] = ACTIONS(1797),
    [sym_tlevel] = ACTIONS(1797),
    [sym_tbutton] = ACTIONS(1797),
    [sym_ttimeline] = ACTIONS(1797),
    [sym_tcustom] = ACTIONS(1797),
    [sym_url_struct] = ACTIONS(1797),
    [sym_dns_struct] = ACTIONS(1797),
    [sym_ip_address_struct] = ACTIONS(1797),
    [sym_dev_info_struct] = ACTIONS(1797),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1797),
    [sym_data_feed] = ACTIONS(1797),
    [sym_data_field] = ACTIONS(1797),
    [sym_data_record] = ACTIONS(1797),
    [sym_wc_data_feed] = ACTIONS(1797),
    [sym_wc_data_field] = ACTIONS(1797),
    [sym_wc_data_record] = ACTIONS(1797),
    [sym_last_login_info] = ACTIONS(1797),
    [sym____reserved___] = ACTIONS(1797),
    [sym_date] = ACTIONS(1797),
    [sym_day] = ACTIONS(1797),
    [sym_time] = ACTIONS(1797),
    [sym_ldate] = ACTIONS(1797),
    [sym_push_device] = ACTIONS(1797),
    [sym_push_channel] = ACTIONS(1797),
    [sym_push_devchan] = ACTIONS(1797),
    [sym_release_device] = ACTIONS(1797),
    [sym_release_channel] = ACTIONS(1797),
    [sym_release_devchan] = ACTIONS(1797),
    [sym_master_slot] = ACTIONS(1797),
    [sym_get_pulse_time] = ACTIONS(1797),
    [sym_get_timer] = ACTIONS(1797),
    [sym_master_sn] = ACTIONS(1797),
    [sym_system_number] = ACTIONS(1797),
    [sym_button] = ACTIONS(1797),
    [sym_dv_channel] = ACTIONS(1797),
    [sym_data] = ACTIONS(1797),
    [sym_level] = ACTIONS(1797),
    [sym_timeline] = ACTIONS(1797),
    [sym_channel] = ACTIONS(1797),
    [sym_custom] = ACTIONS(1797),
    [sym_atoi] = ACTIONS(1797),
    [sym_atol] = ACTIONS(1797),
    [sym_atof] = ACTIONS(1797),
    [sym_itoa] = ACTIONS(1797),
    [sym_itohex] = ACTIONS(1797),
    [sym_hextoi] = ACTIONS(1797),
    [sym_ftoa] = ACTIONS(1797),
    [sym_find_string] = ACTIONS(1797),
    [sym_left_string] = ACTIONS(1797),
    [sym_right_string] = ACTIONS(1797),
    [sym_mid_string] = ACTIONS(1797),
    [sym_remove_string] = ACTIONS(1797),
    [sym_get_buffer_string] = ACTIONS(1797),
    [sym_get_buffer_char] = ACTIONS(1797),
    [sym_get_multi_buffer_string] = ACTIONS(1797),
    [sym_length_string] = ACTIONS(1797),
    [sym_max_length_string] = ACTIONS(1797),
    [sym_set_length_string] = ACTIONS(1797),
    [sym_compare_string] = ACTIONS(1797),
    [sym_upper_string] = ACTIONS(1797),
    [sym_lower_string] = ACTIONS(1797),
    [sym_get_last] = ACTIONS(1797),
    [sym_date_to_day] = ACTIONS(1797),
    [sym_date_to_month] = ACTIONS(1797),
    [sym_date_to_year] = ACTIONS(1797),
    [sym_time_to_hour] = ACTIONS(1797),
    [sym_time_to_minute] = ACTIONS(1797),
    [sym_time_to_second] = ACTIONS(1797),
    [sym_day_of_week] = ACTIONS(1797),
    [sym_set_pulse_time] = ACTIONS(1797),
    [sym_set_timer] = ACTIONS(1797),
    [sym_file_open] = ACTIONS(1797),
    [sym_file_close] = ACTIONS(1797),
    [sym_file_read] = ACTIONS(1797),
    [sym_file_read_line] = ACTIONS(1797),
    [sym_file_write] = ACTIONS(1797),
    [sym_file_write_line] = ACTIONS(1797),
    [sym_file_copy] = ACTIONS(1797),
    [sym_file_delete] = ACTIONS(1797),
    [sym_file_rename] = ACTIONS(1797),
    [sym_file_dir] = ACTIONS(1797),
    [sym_file_getdir] = ACTIONS(1797),
    [sym_file_setdir] = ACTIONS(1797),
    [sym_file_create_dir] = ACTIONS(1797),
    [sym_file_remove_dir] = ACTIONS(1797),
    [sym_file_seek] = ACTIONS(1797),
    [sym_length_array] = ACTIONS(1797),
    [sym_max_length_array] = ACTIONS(1797),
    [sym_set_length_array] = ACTIONS(1797),
    [sym_ip_client_open] = ACTIONS(1797),
    [sym_ip_client_close] = ACTIONS(1797),
    [sym_ip_server_open] = ACTIONS(1797),
    [sym_ip_server_close] = ACTIONS(1797),
    [sym_ip_bound_client_open] = ACTIONS(1797),
    [sym_ip_mc_server_open] = ACTIONS(1797),
    [sym_ip_set_option] = ACTIONS(1797),
    [sym_ssh_client_open] = ACTIONS(1797),
    [sym_ssh_client_close] = ACTIONS(1797),
    [sym_tls_client_open] = ACTIONS(1797),
    [sym_tls_client_close] = ACTIONS(1797),
    [sym_do_push] = ACTIONS(1797),
    [sym_do_push_timed] = ACTIONS(1797),
    [sym_do_release] = ACTIONS(1797),
    [sym_device_id] = ACTIONS(1797),
    [sym_device_info] = ACTIONS(1797),
    [sym_device_id_string] = ACTIONS(1797),
    [sym_device_standby] = ACTIONS(1797),
    [sym_device_wake] = ACTIONS(1797),
    [sym_timeline_create] = ACTIONS(1797),
    [sym_timeline_kill] = ACTIONS(1797),
    [sym_timeline_reload] = ACTIONS(1797),
    [sym_timeline_pause] = ACTIONS(1797),
    [sym_timeline_restart] = ACTIONS(1797),
    [sym_timeline_set] = ACTIONS(1797),
    [sym_timeline_get] = ACTIONS(1797),
    [sym_timeline_active] = ACTIONS(1797),
    [sym_timeline_dynamic_id] = ACTIONS(1797),
    [sym_variable_to_string] = ACTIONS(1797),
    [sym_string_to_variable] = ACTIONS(1797),
    [sym_length_variable_to_string] = ACTIONS(1797),
    [sym_type_cast] = ACTIONS(1797),
    [sym_variable_to_xml] = ACTIONS(1797),
    [sym_xml_to_variable] = ACTIONS(1797),
    [sym_length_variable_to_xml] = ACTIONS(1797),
    [sym_redirect_string] = ACTIONS(1797),
    [sym_random_number] = ACTIONS(1797),
    [sym_get_unique_id] = ACTIONS(1797),
    [sym_get_system_number] = ACTIONS(1797),
    [sym_set_system_number] = ACTIONS(1797),
    [sym_get_serial_number] = ACTIONS(1797),
    [sym_get_master_build] = ACTIONS(1797),
    [sym_set_validation_code] = ACTIONS(1797),
    [sym_reboot] = ACTIONS(1797),
    [sym_load_duet_module] = ACTIONS(1797),
    [sym_unload_duet_module] = ACTIONS(1797),
    [sym_duet_mem_size_set] = ACTIONS(1797),
    [sym_duet_mem_size_get] = ACTIONS(1797),
    [sym_dynamic_application_device] = ACTIONS(1797),
    [sym_dynamic_polled_port] = ACTIONS(1797),
    [sym_static_port_binding] = ACTIONS(1797),
    [sym_static_ip_binding] = ACTIONS(1797),
    [sym_get_dns_list] = ACTIONS(1797),
    [sym_set_dns_list] = ACTIONS(1797),
    [sym_get_ip_address] = ACTIONS(1797),
    [sym_set_ip_address] = ACTIONS(1797),
    [sym_get_url_list] = ACTIONS(1797),
    [sym_add_url_entry] = ACTIONS(1797),
    [sym_delete_url_entry] = ACTIONS(1797),
    [sym_abs_value] = ACTIONS(1797),
    [sym_max_value] = ACTIONS(1797),
    [sym_min_value] = ACTIONS(1797),
    [sym_exp_value] = ACTIONS(1797),
    [sym_log_value] = ACTIONS(1797),
    [sym_log10_value] = ACTIONS(1797),
    [sym_power_value] = ACTIONS(1797),
    [sym_sqrt_value] = ACTIONS(1797),
    [sym_raw_be] = ACTIONS(1797),
    [sym_raw_le] = ACTIONS(1797),
    [sym_internal_queue_size_set] = ACTIONS(1797),
    [sym_internal_queue_size_get] = ACTIONS(1797),
    [sym_internal_threshold_set] = ACTIONS(1797),
    [sym_internal_threshold_get] = ACTIONS(1797),
    [sym_rebuilt_event] = ACTIONS(1797),
    [sym_do_custom_event] = ACTIONS(1797),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1797),
    [sym_clkmgr_set_clk_source] = ACTIONS(1797),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1797),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1797),
    [sym_clkmgr_get_timezone] = ACTIONS(1797),
    [sym_clkmgr_set_timezone] = ACTIONS(1797),
    [sym_clkmgr_get_resync_period] = ACTIONS(1797),
    [sym_clkmgr_set_resync_period] = ACTIONS(1797),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1797),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1797),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1797),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1797),
    [sym_clkmgr_get_timeservers] = ACTIONS(1797),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1797),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1797),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1797),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1797),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1797),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1797),
    [sym_set_log_level] = ACTIONS(1797),
    [sym_get_log_level] = ACTIONS(1797),
    [sym_amx_log] = ACTIONS(1797),
    [sym_smtp_server_config_set] = ACTIONS(1797),
    [sym_smtp_server_config_get] = ACTIONS(1797),
    [sym_smtp_send] = ACTIONS(1797),
    [sym_validate_netlinx_account] = ACTIONS(1797),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1797),
    [sym_authenticate_certificate] = ACTIONS(1797),
    [sym_audit_netlinx_session_event] = ACTIONS(1797),
    [sym_audit_netlinx_generic_event] = ACTIONS(1797),
    [sym_get_available_flash_disk_space] = ACTIONS(1797),
    [sym_get_max_flash_disk_space] = ACTIONS(1797),
    [sym_data_create_feed] = ACTIONS(1797),
    [sym_data_delete_feed] = ACTIONS(1797),
    [sym_data_publish_feed] = ACTIONS(1797),
    [sym_data_get_published_feed] = ACTIONS(1797),
    [sym_data_add_record] = ACTIONS(1797),
    [sym_data_get_event_record] = ACTIONS(1797),
    [sym__wc_data_create_feed] = ACTIONS(1797),
    [sym__wc_data_add_record] = ACTIONS(1797),
    [sym__wc_data_get_event_record] = ACTIONS(1797),
    [sym_format] = ACTIONS(1797),
    [sym_astro_clock] = ACTIONS(1797),
    [sym_set_outdoor_temperature] = ACTIONS(1797),
    [sym_set_virtual_level_count] = ACTIONS(1797),
    [sym_set_virtual_channel_count] = ACTIONS(1797),
    [sym_set_virtual_port_count] = ACTIONS(1797),
    [sym__wc_encode_int] = ACTIONS(1797),
    [sym__wc_decode_int] = ACTIONS(1797),
    [sym__wc_int] = ACTIONS(1797),
    [sym__wc_to_ch_int] = ACTIONS(1797),
    [sym__ch_to_wc_int] = ACTIONS(1797),
    [sym__wc_find_string_int] = ACTIONS(1797),
    [sym__wc_left_string_int] = ACTIONS(1797),
    [sym__wc_lower_string_int] = ACTIONS(1797),
    [sym__wc_mid_string_int] = ACTIONS(1797),
    [sym__wc_remove_string_int] = ACTIONS(1797),
    [sym__wc_right_string_int] = ACTIONS(1797),
    [sym__wc_upper_string_int] = ACTIONS(1797),
    [sym__wc_compare_string_int] = ACTIONS(1797),
    [sym__wc_get_buffer_char_int] = ACTIONS(1797),
    [sym__wc_get_buffer_string_int] = ACTIONS(1797),
    [sym__wc_concat_string_int] = ACTIONS(1797),
    [sym__wc_file_open_int] = ACTIONS(1797),
    [sym__wc_file_close_int] = ACTIONS(1797),
    [sym__wc_file_read_int] = ACTIONS(1797),
    [sym__wc_file_read_line_int] = ACTIONS(1797),
    [sym__wc_file_write_int] = ACTIONS(1797),
    [sym__wc_file_write_line_int] = ACTIONS(1797),
    [aux_sym_preproc_include_token1] = ACTIONS(1797),
    [aux_sym_preproc_define_token1] = ACTIONS(1799),
    [aux_sym_preproc_warn_token1] = ACTIONS(1799),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1799),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1799),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1799),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1799),
    [aux_sym_preproc_else_token1] = ACTIONS(1799),
    [anon_sym_LBRACK] = ACTIONS(1799),
    [anon_sym_LBRACE] = ACTIONS(1799),
    [anon_sym_LPAREN] = ACTIONS(1801),
    [anon_sym_BANG] = ACTIONS(1799),
    [anon_sym_TILDE] = ACTIONS(1799),
    [anon_sym_DASH] = ACTIONS(1797),
    [anon_sym_PLUS] = ACTIONS(1797),
    [anon_sym_DASH_DASH] = ACTIONS(1799),
    [anon_sym_PLUS_PLUS] = ACTIONS(1799),
    [anon_sym_DQUOTE] = ACTIONS(1799),
    [anon_sym_SQUOTE] = ACTIONS(1799),
    [sym_number_literal] = ACTIONS(1799),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(245)] = {
    [sym_identifier] = ACTIONS(1803),
    [sym_program_name_keyword] = ACTIONS(1803),
    [sym_module_name_keyword] = ACTIONS(1803),
    [sym_define_device_keyword] = ACTIONS(1803),
    [sym_define_combine_keyword] = ACTIONS(1803),
    [sym_define_constant_keyword] = ACTIONS(1803),
    [sym_define_type_keyword] = ACTIONS(1803),
    [sym_define_variable_keyword] = ACTIONS(1803),
    [sym_define_system_variable_keyword] = ACTIONS(1803),
    [sym_define_connect_level_keyword] = ACTIONS(1803),
    [sym_define_latching_keyword] = ACTIONS(1803),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1803),
    [sym_define_toggling_keyword] = ACTIONS(1803),
    [sym_define_call_keyword] = ACTIONS(1803),
    [sym_define_function_keyword] = ACTIONS(1803),
    [sym_define_library_function_keyword] = ACTIONS(1803),
    [sym_define_start_keyword] = ACTIONS(1803),
    [sym_define_event_keyword] = ACTIONS(1803),
    [sym_define_program_keyword] = ACTIONS(1803),
    [sym_define_module_keyword] = ACTIONS(1803),
    [sym_char_keyword] = ACTIONS(1803),
    [sym_widechar_keyword] = ACTIONS(1803),
    [sym_integer_keyword] = ACTIONS(1803),
    [sym_sinteger_keyword] = ACTIONS(1803),
    [sym_long_keyword] = ACTIONS(1803),
    [sym_slong_keyword] = ACTIONS(1803),
    [sym_float_keyword] = ACTIONS(1803),
    [sym_double_keyword] = ACTIONS(1803),
    [sym_variant_keyword] = ACTIONS(1803),
    [sym_variantarray_keyword] = ACTIONS(1803),
    [sym_local_var_keyword] = ACTIONS(1803),
    [sym_stack_var_keyword] = ACTIONS(1803),
    [sym_constant_keyword] = ACTIONS(1803),
    [sym_volatile_keyword] = ACTIONS(1803),
    [sym_non_volatile_keyword] = ACTIONS(1803),
    [sym_persistent_keyword] = ACTIONS(1803),
    [sym_dev_keyword] = ACTIONS(1803),
    [sym_devlev_keyword] = ACTIONS(1803),
    [sym_devchan_keyword] = ACTIONS(1803),
    [sym_if_keyword] = ACTIONS(1803),
    [sym_else_keyword] = ACTIONS(1803),
    [sym_switch_keyword] = ACTIONS(1803),
    [sym_case_keyword] = ACTIONS(1803),
    [sym_default_keyword] = ACTIONS(1803),
    [sym_while_keyword] = ACTIONS(1803),
    [sym_for_keyword] = ACTIONS(1803),
    [sym_break_keyword] = ACTIONS(1803),
    [sym_continue_keyword] = ACTIONS(1803),
    [sym_return_keyword] = ACTIONS(1803),
    [sym_select_keyword] = ACTIONS(1803),
    [sym_active_keyword] = ACTIONS(1803),
    [sym_struct_keyword] = ACTIONS(1803),
    [sym_structure_keyword] = ACTIONS(1803),
    [sym_button_event_keyword] = ACTIONS(1803),
    [sym_channel_event_keyword] = ACTIONS(1803),
    [sym_level_event_keyword] = ACTIONS(1803),
    [sym_data_event_keyword] = ACTIONS(1803),
    [sym_timeline_event_keyword] = ACTIONS(1803),
    [sym_custom_event_keyword] = ACTIONS(1803),
    [sym_push_keyword] = ACTIONS(1803),
    [sym_release_keyword] = ACTIONS(1803),
    [aux_sym_on_keyword_token1] = ACTIONS(1803),
    [aux_sym_off_keyword_token1] = ACTIONS(1803),
    [sym_send_level_keyword] = ACTIONS(1803),
    [sym_send_string_keyword] = ACTIONS(1803),
    [sym_send_command_keyword] = ACTIONS(1803),
    [sym_clear_buffer_keyword] = ACTIONS(1803),
    [sym_create_buffer_keyword] = ACTIONS(1803),
    [sym_create_multi_buffer_keyword] = ACTIONS(1803),
    [sym_call_keyword] = ACTIONS(1803),
    [sym_system_call_keyword] = ACTIONS(1803),
    [sym_devchan_to_keyword] = ACTIONS(1803),
    [sym_devchan_min_to_keyword] = ACTIONS(1803),
    [sym_devchan_total_off_keyword] = ACTIONS(1803),
    [sym_devchan_pulse_keyword] = ACTIONS(1803),
    [sym_wait_keyword] = ACTIONS(1803),
    [sym_cancel_wait_keyword] = ACTIONS(1803),
    [sym_wait_until_keyword] = ACTIONS(1803),
    [sym_cancel_wait_until_keyword] = ACTIONS(1803),
    [sym_cancel_all_wait_keyword] = ACTIONS(1803),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1803),
    [sym_not] = ACTIONS(1803),
    [sym_bnot] = ACTIONS(1803),
    [sym___file__] = ACTIONS(1803),
    [sym___date__] = ACTIONS(1803),
    [sym___ldate__] = ACTIONS(1803),
    [sym___time__] = ACTIONS(1803),
    [sym___line__] = ACTIONS(1803),
    [sym___name__] = ACTIONS(1803),
    [sym___netlinx__] = ACTIONS(1803),
    [sym_netlinx_axi_version] = ACTIONS(1803),
    [sym_url_flg_tcp] = ACTIONS(1803),
    [sym_url_flg_accinfopresent] = ACTIONS(1803),
    [sym_url_flg_temp] = ACTIONS(1803),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1803),
    [sym_url_flg_stat_mask] = ACTIONS(1803),
    [sym_url_flg_stat_lookup] = ACTIONS(1803),
    [sym_url_flg_stat_connecting] = ACTIONS(1803),
    [sym_url_flg_stat_waiting] = ACTIONS(1803),
    [sym_url_flg_stat_connected] = ACTIONS(1803),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1803),
    [sym_first_virtual_device] = ACTIONS(1803),
    [sym_first_local_port] = ACTIONS(1803),
    [sym_dynamic_virtual_device] = ACTIONS(1803),
    [sym_timeline_once] = ACTIONS(1803),
    [sym_timeline_repeat] = ACTIONS(1803),
    [sym_timeline_absolute] = ACTIONS(1803),
    [sym_timeline_relative] = ACTIONS(1803),
    [sym_do_push_timed_infinite] = ACTIONS(1803),
    [sym_source_type_no_address] = ACTIONS(1803),
    [sym_source_type_neuron_id] = ACTIONS(1803),
    [sym_source_type_ip_address] = ACTIONS(1803),
    [sym_source_type_axlink] = ACTIONS(1803),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1803),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1803),
    [sym_source_type_ip_socket_address] = ACTIONS(1803),
    [sym_source_type_rs232] = ACTIONS(1803),
    [sym_source_type_internal] = ACTIONS(1803),
    [sym_source_type_mac_address] = ACTIONS(1803),
    [sym_source_type_ipv4_port] = ACTIONS(1803),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1803),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1803),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1803),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1803),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1803),
    [sym_true] = ACTIONS(1803),
    [sym_false] = ACTIONS(1803),
    [sym_file_read_only] = ACTIONS(1803),
    [sym_file_rw_new] = ACTIONS(1803),
    [sym_file_rw_append] = ACTIONS(1803),
    [sym_ip_tcp] = ACTIONS(1803),
    [sym_ip_udp] = ACTIONS(1803),
    [sym_ip_udp_2way] = ACTIONS(1803),
    [sym_xml_encode_types] = ACTIONS(1803),
    [sym_xml_encode_char_as_list] = ACTIONS(1803),
    [sym_xml_encode_le] = ACTIONS(1803),
    [sym_xml_decode_types] = ACTIONS(1803),
    [sym_xml_decode_no_preserve] = ACTIONS(1803),
    [sym_duet_dev_type_display_device] = ACTIONS(1803),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1803),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1803),
    [sym_duet_dev_type_io_device] = ACTIONS(1803),
    [sym_duet_dev_type_relay_device] = ACTIONS(1803),
    [sym_duet_dev_type_ups] = ACTIONS(1803),
    [sym_duet_dev_type_amplifier] = ACTIONS(1803),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1803),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1803),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1803),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1803),
    [sym_duet_dev_type_camera] = ACTIONS(1803),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1803),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1803),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1803),
    [sym_duet_dev_type_dss] = ACTIONS(1803),
    [sym_duet_dev_type_dvr] = ACTIONS(1803),
    [sym_duet_dev_type_disc_device] = ACTIONS(1803),
    [sym_duet_dev_type_document_camera] = ACTIONS(1803),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1803),
    [sym_duet_dev_type_hvac] = ACTIONS(1803),
    [sym_duet_dev_type_keypad] = ACTIONS(1803),
    [sym_duet_dev_type_light] = ACTIONS(1803),
    [sym_duet_dev_type_monitor] = ACTIONS(1803),
    [sym_duet_dev_type_motor] = ACTIONS(1803),
    [sym_duet_dev_type_multi_window] = ACTIONS(1803),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1803),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1803),
    [sym_duet_dev_type_receiver] = ACTIONS(1803),
    [sym_duet_dev_type_security_system] = ACTIONS(1803),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1803),
    [sym_duet_dev_type_settop_box] = ACTIONS(1803),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1803),
    [sym_duet_dev_type_switcher] = ACTIONS(1803),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1803),
    [sym_duet_dev_type_tv] = ACTIONS(1803),
    [sym_duet_dev_type_utility] = ACTIONS(1803),
    [sym_duet_dev_type_vcr] = ACTIONS(1803),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1803),
    [sym_duet_dev_type_video_processor] = ACTIONS(1803),
    [sym_duet_dev_type_video_projector] = ACTIONS(1803),
    [sym_duet_dev_type_video_wall] = ACTIONS(1803),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1803),
    [sym_duet_dev_type_weather] = ACTIONS(1803),
    [sym_duet_dev_not_polled] = ACTIONS(1803),
    [sym_duet_dev_polled] = ACTIONS(1803),
    [sym_clkmgr_mode_network] = ACTIONS(1803),
    [sym_clkmgr_mode_standalone] = ACTIONS(1803),
    [sym_normal_standby] = ACTIONS(1803),
    [sym_normal_wake] = ACTIONS(1803),
    [sym_ip_multicast_ttl_option] = ACTIONS(1803),
    [sym_ip_ttl_subnet] = ACTIONS(1803),
    [sym_ip_ttl_site] = ACTIONS(1803),
    [sym_ip_ttl_region] = ACTIONS(1803),
    [sym_ip_ttl_continent] = ACTIONS(1803),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1803),
    [sym_ip_nodelay_on] = ACTIONS(1803),
    [sym_ip_nodelay_off] = ACTIONS(1803),
    [sym_amx_error] = ACTIONS(1803),
    [sym_amx_warning] = ACTIONS(1803),
    [sym_amx_info] = ACTIONS(1803),
    [sym_amx_debug] = ACTIONS(1803),
    [sym_netlinx_login_success] = ACTIONS(1803),
    [sym_netlinx_login_fail] = ACTIONS(1803),
    [sym_netlinx_logout] = ACTIONS(1803),
    [sym_null_str] = ACTIONS(1803),
    [sym_smtp_address] = ACTIONS(1803),
    [sym_smtp_port_number] = ACTIONS(1803),
    [sym_smtp_username] = ACTIONS(1803),
    [sym_smtp_password] = ACTIONS(1803),
    [sym_smtp_require_tls] = ACTIONS(1803),
    [sym_smtp_from] = ACTIONS(1803),
    [sym_smtp_tls_true] = ACTIONS(1803),
    [sym_smtp_tls_false] = ACTIONS(1803),
    [sym_listview_on_row_select_event] = ACTIONS(1803),
    [sym_data_structure_unknown] = ACTIONS(1803),
    [sym_data_structure_datafeed] = ACTIONS(1803),
    [sym_data_structure_datarecordset] = ACTIONS(1803),
    [sym_data_structure_datarecord] = ACTIONS(1803),
    [sym_data_structure_datafield] = ACTIONS(1803),
    [sym_data_max_content_field_count] = ACTIONS(1803),
    [sym_data_max_metadata_field_count] = ACTIONS(1803),
    [sym_data_max_id_length] = ACTIONS(1803),
    [sym_data_max_name_length] = ACTIONS(1803),
    [sym_data_max_description_length] = ACTIONS(1803),
    [sym_data_max_type_length] = ACTIONS(1803),
    [sym_data_max_format_length] = ACTIONS(1803),
    [sym_data_max_source_length] = ACTIONS(1803),
    [sym_data_max_value_length] = ACTIONS(1803),
    [sym_data_max_label_length] = ACTIONS(1803),
    [sym_data_max_url_length] = ACTIONS(1803),
    [sym_max_last_login_info_length] = ACTIONS(1803),
    [sym_data_type_unknown] = ACTIONS(1803),
    [sym_data_type_string] = ACTIONS(1803),
    [sym_data_type_datetime] = ACTIONS(1803),
    [sym_data_type_date] = ACTIONS(1803),
    [sym_data_type_time] = ACTIONS(1803),
    [sym_data_type_image] = ACTIONS(1803),
    [sym_data_format_url] = ACTIONS(1803),
    [sym_data_format_phone] = ACTIONS(1803),
    [sym_data_format_email] = ACTIONS(1803),
    [sym_data_format_iso8601] = ACTIONS(1803),
    [sym_auditlog_priv] = ACTIONS(1803),
    [sym_configuration_priv] = ACTIONS(1803),
    [sym_deviceconfig_priv] = ACTIONS(1803),
    [sym_ftp_priv] = ACTIONS(1803),
    [sym_http_priv] = ACTIONS(1803),
    [sym_networkconfig_priv] = ACTIONS(1803),
    [sym_programport_priv] = ACTIONS(1803),
    [sym_remoteui_priv] = ACTIONS(1803),
    [sym_securitycontrol_priv] = ACTIONS(1803),
    [sym_softwaremanagement_priv] = ACTIONS(1803),
    [sym_terminal_priv] = ACTIONS(1803),
    [sym_tpadmin_priv] = ACTIONS(1803),
    [sym_user1_priv] = ACTIONS(1803),
    [sym_user2_priv] = ACTIONS(1803),
    [sym_user3_priv] = ACTIONS(1803),
    [sym_user4_priv] = ACTIONS(1803),
    [sym_usermanagement_priv] = ACTIONS(1803),
    [sym_valid_account] = ACTIONS(1803),
    [sym_err_name_invalid] = ACTIONS(1803),
    [sym_err_password_invalid] = ACTIONS(1803),
    [sym_err_invalid_account] = ACTIONS(1803),
    [sym_err_no_authorization] = ACTIONS(1803),
    [sym_err_invalid_parameter] = ACTIONS(1803),
    [sym_err_user_locked_out] = ACTIONS(1803),
    [sym_err_user_account_expired] = ACTIONS(1803),
    [sym_err_general_error] = ACTIONS(1803),
    [sym_err_authentication_error] = ACTIONS(1803),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1803),
    [sym_tls_validate_certificate] = ACTIONS(1803),
    [sym_tdata] = ACTIONS(1803),
    [sym_tchannel] = ACTIONS(1803),
    [sym_tlevel] = ACTIONS(1803),
    [sym_tbutton] = ACTIONS(1803),
    [sym_ttimeline] = ACTIONS(1803),
    [sym_tcustom] = ACTIONS(1803),
    [sym_url_struct] = ACTIONS(1803),
    [sym_dns_struct] = ACTIONS(1803),
    [sym_ip_address_struct] = ACTIONS(1803),
    [sym_dev_info_struct] = ACTIONS(1803),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1803),
    [sym_data_feed] = ACTIONS(1803),
    [sym_data_field] = ACTIONS(1803),
    [sym_data_record] = ACTIONS(1803),
    [sym_wc_data_feed] = ACTIONS(1803),
    [sym_wc_data_field] = ACTIONS(1803),
    [sym_wc_data_record] = ACTIONS(1803),
    [sym_last_login_info] = ACTIONS(1803),
    [sym____reserved___] = ACTIONS(1803),
    [sym_date] = ACTIONS(1803),
    [sym_day] = ACTIONS(1803),
    [sym_time] = ACTIONS(1803),
    [sym_ldate] = ACTIONS(1803),
    [sym_push_device] = ACTIONS(1803),
    [sym_push_channel] = ACTIONS(1803),
    [sym_push_devchan] = ACTIONS(1803),
    [sym_release_device] = ACTIONS(1803),
    [sym_release_channel] = ACTIONS(1803),
    [sym_release_devchan] = ACTIONS(1803),
    [sym_master_slot] = ACTIONS(1803),
    [sym_get_pulse_time] = ACTIONS(1803),
    [sym_get_timer] = ACTIONS(1803),
    [sym_master_sn] = ACTIONS(1803),
    [sym_system_number] = ACTIONS(1803),
    [sym_button] = ACTIONS(1803),
    [sym_dv_channel] = ACTIONS(1803),
    [sym_data] = ACTIONS(1803),
    [sym_level] = ACTIONS(1803),
    [sym_timeline] = ACTIONS(1803),
    [sym_channel] = ACTIONS(1803),
    [sym_custom] = ACTIONS(1803),
    [sym_atoi] = ACTIONS(1803),
    [sym_atol] = ACTIONS(1803),
    [sym_atof] = ACTIONS(1803),
    [sym_itoa] = ACTIONS(1803),
    [sym_itohex] = ACTIONS(1803),
    [sym_hextoi] = ACTIONS(1803),
    [sym_ftoa] = ACTIONS(1803),
    [sym_find_string] = ACTIONS(1803),
    [sym_left_string] = ACTIONS(1803),
    [sym_right_string] = ACTIONS(1803),
    [sym_mid_string] = ACTIONS(1803),
    [sym_remove_string] = ACTIONS(1803),
    [sym_get_buffer_string] = ACTIONS(1803),
    [sym_get_buffer_char] = ACTIONS(1803),
    [sym_get_multi_buffer_string] = ACTIONS(1803),
    [sym_length_string] = ACTIONS(1803),
    [sym_max_length_string] = ACTIONS(1803),
    [sym_set_length_string] = ACTIONS(1803),
    [sym_compare_string] = ACTIONS(1803),
    [sym_upper_string] = ACTIONS(1803),
    [sym_lower_string] = ACTIONS(1803),
    [sym_get_last] = ACTIONS(1803),
    [sym_date_to_day] = ACTIONS(1803),
    [sym_date_to_month] = ACTIONS(1803),
    [sym_date_to_year] = ACTIONS(1803),
    [sym_time_to_hour] = ACTIONS(1803),
    [sym_time_to_minute] = ACTIONS(1803),
    [sym_time_to_second] = ACTIONS(1803),
    [sym_day_of_week] = ACTIONS(1803),
    [sym_set_pulse_time] = ACTIONS(1803),
    [sym_set_timer] = ACTIONS(1803),
    [sym_file_open] = ACTIONS(1803),
    [sym_file_close] = ACTIONS(1803),
    [sym_file_read] = ACTIONS(1803),
    [sym_file_read_line] = ACTIONS(1803),
    [sym_file_write] = ACTIONS(1803),
    [sym_file_write_line] = ACTIONS(1803),
    [sym_file_copy] = ACTIONS(1803),
    [sym_file_delete] = ACTIONS(1803),
    [sym_file_rename] = ACTIONS(1803),
    [sym_file_dir] = ACTIONS(1803),
    [sym_file_getdir] = ACTIONS(1803),
    [sym_file_setdir] = ACTIONS(1803),
    [sym_file_create_dir] = ACTIONS(1803),
    [sym_file_remove_dir] = ACTIONS(1803),
    [sym_file_seek] = ACTIONS(1803),
    [sym_length_array] = ACTIONS(1803),
    [sym_max_length_array] = ACTIONS(1803),
    [sym_set_length_array] = ACTIONS(1803),
    [sym_ip_client_open] = ACTIONS(1803),
    [sym_ip_client_close] = ACTIONS(1803),
    [sym_ip_server_open] = ACTIONS(1803),
    [sym_ip_server_close] = ACTIONS(1803),
    [sym_ip_bound_client_open] = ACTIONS(1803),
    [sym_ip_mc_server_open] = ACTIONS(1803),
    [sym_ip_set_option] = ACTIONS(1803),
    [sym_ssh_client_open] = ACTIONS(1803),
    [sym_ssh_client_close] = ACTIONS(1803),
    [sym_tls_client_open] = ACTIONS(1803),
    [sym_tls_client_close] = ACTIONS(1803),
    [sym_do_push] = ACTIONS(1803),
    [sym_do_push_timed] = ACTIONS(1803),
    [sym_do_release] = ACTIONS(1803),
    [sym_device_id] = ACTIONS(1803),
    [sym_device_info] = ACTIONS(1803),
    [sym_device_id_string] = ACTIONS(1803),
    [sym_device_standby] = ACTIONS(1803),
    [sym_device_wake] = ACTIONS(1803),
    [sym_timeline_create] = ACTIONS(1803),
    [sym_timeline_kill] = ACTIONS(1803),
    [sym_timeline_reload] = ACTIONS(1803),
    [sym_timeline_pause] = ACTIONS(1803),
    [sym_timeline_restart] = ACTIONS(1803),
    [sym_timeline_set] = ACTIONS(1803),
    [sym_timeline_get] = ACTIONS(1803),
    [sym_timeline_active] = ACTIONS(1803),
    [sym_timeline_dynamic_id] = ACTIONS(1803),
    [sym_variable_to_string] = ACTIONS(1803),
    [sym_string_to_variable] = ACTIONS(1803),
    [sym_length_variable_to_string] = ACTIONS(1803),
    [sym_type_cast] = ACTIONS(1803),
    [sym_variable_to_xml] = ACTIONS(1803),
    [sym_xml_to_variable] = ACTIONS(1803),
    [sym_length_variable_to_xml] = ACTIONS(1803),
    [sym_redirect_string] = ACTIONS(1803),
    [sym_random_number] = ACTIONS(1803),
    [sym_get_unique_id] = ACTIONS(1803),
    [sym_get_system_number] = ACTIONS(1803),
    [sym_set_system_number] = ACTIONS(1803),
    [sym_get_serial_number] = ACTIONS(1803),
    [sym_get_master_build] = ACTIONS(1803),
    [sym_set_validation_code] = ACTIONS(1803),
    [sym_reboot] = ACTIONS(1803),
    [sym_load_duet_module] = ACTIONS(1803),
    [sym_unload_duet_module] = ACTIONS(1803),
    [sym_duet_mem_size_set] = ACTIONS(1803),
    [sym_duet_mem_size_get] = ACTIONS(1803),
    [sym_dynamic_application_device] = ACTIONS(1803),
    [sym_dynamic_polled_port] = ACTIONS(1803),
    [sym_static_port_binding] = ACTIONS(1803),
    [sym_static_ip_binding] = ACTIONS(1803),
    [sym_get_dns_list] = ACTIONS(1803),
    [sym_set_dns_list] = ACTIONS(1803),
    [sym_get_ip_address] = ACTIONS(1803),
    [sym_set_ip_address] = ACTIONS(1803),
    [sym_get_url_list] = ACTIONS(1803),
    [sym_add_url_entry] = ACTIONS(1803),
    [sym_delete_url_entry] = ACTIONS(1803),
    [sym_abs_value] = ACTIONS(1803),
    [sym_max_value] = ACTIONS(1803),
    [sym_min_value] = ACTIONS(1803),
    [sym_exp_value] = ACTIONS(1803),
    [sym_log_value] = ACTIONS(1803),
    [sym_log10_value] = ACTIONS(1803),
    [sym_power_value] = ACTIONS(1803),
    [sym_sqrt_value] = ACTIONS(1803),
    [sym_raw_be] = ACTIONS(1803),
    [sym_raw_le] = ACTIONS(1803),
    [sym_internal_queue_size_set] = ACTIONS(1803),
    [sym_internal_queue_size_get] = ACTIONS(1803),
    [sym_internal_threshold_set] = ACTIONS(1803),
    [sym_internal_threshold_get] = ACTIONS(1803),
    [sym_rebuilt_event] = ACTIONS(1803),
    [sym_do_custom_event] = ACTIONS(1803),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1803),
    [sym_clkmgr_set_clk_source] = ACTIONS(1803),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1803),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1803),
    [sym_clkmgr_get_timezone] = ACTIONS(1803),
    [sym_clkmgr_set_timezone] = ACTIONS(1803),
    [sym_clkmgr_get_resync_period] = ACTIONS(1803),
    [sym_clkmgr_set_resync_period] = ACTIONS(1803),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1803),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1803),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1803),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1803),
    [sym_clkmgr_get_timeservers] = ACTIONS(1803),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1803),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1803),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1803),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1803),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1803),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1803),
    [sym_set_log_level] = ACTIONS(1803),
    [sym_get_log_level] = ACTIONS(1803),
    [sym_amx_log] = ACTIONS(1803),
    [sym_smtp_server_config_set] = ACTIONS(1803),
    [sym_smtp_server_config_get] = ACTIONS(1803),
    [sym_smtp_send] = ACTIONS(1803),
    [sym_validate_netlinx_account] = ACTIONS(1803),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1803),
    [sym_authenticate_certificate] = ACTIONS(1803),
    [sym_audit_netlinx_session_event] = ACTIONS(1803),
    [sym_audit_netlinx_generic_event] = ACTIONS(1803),
    [sym_get_available_flash_disk_space] = ACTIONS(1803),
    [sym_get_max_flash_disk_space] = ACTIONS(1803),
    [sym_data_create_feed] = ACTIONS(1803),
    [sym_data_delete_feed] = ACTIONS(1803),
    [sym_data_publish_feed] = ACTIONS(1803),
    [sym_data_get_published_feed] = ACTIONS(1803),
    [sym_data_add_record] = ACTIONS(1803),
    [sym_data_get_event_record] = ACTIONS(1803),
    [sym__wc_data_create_feed] = ACTIONS(1803),
    [sym__wc_data_add_record] = ACTIONS(1803),
    [sym__wc_data_get_event_record] = ACTIONS(1803),
    [sym_format] = ACTIONS(1803),
    [sym_astro_clock] = ACTIONS(1803),
    [sym_set_outdoor_temperature] = ACTIONS(1803),
    [sym_set_virtual_level_count] = ACTIONS(1803),
    [sym_set_virtual_channel_count] = ACTIONS(1803),
    [sym_set_virtual_port_count] = ACTIONS(1803),
    [sym__wc_encode_int] = ACTIONS(1803),
    [sym__wc_decode_int] = ACTIONS(1803),
    [sym__wc_int] = ACTIONS(1803),
    [sym__wc_to_ch_int] = ACTIONS(1803),
    [sym__ch_to_wc_int] = ACTIONS(1803),
    [sym__wc_find_string_int] = ACTIONS(1803),
    [sym__wc_left_string_int] = ACTIONS(1803),
    [sym__wc_lower_string_int] = ACTIONS(1803),
    [sym__wc_mid_string_int] = ACTIONS(1803),
    [sym__wc_remove_string_int] = ACTIONS(1803),
    [sym__wc_right_string_int] = ACTIONS(1803),
    [sym__wc_upper_string_int] = ACTIONS(1803),
    [sym__wc_compare_string_int] = ACTIONS(1803),
    [sym__wc_get_buffer_char_int] = ACTIONS(1803),
    [sym__wc_get_buffer_string_int] = ACTIONS(1803),
    [sym__wc_concat_string_int] = ACTIONS(1803),
    [sym__wc_file_open_int] = ACTIONS(1803),
    [sym__wc_file_close_int] = ACTIONS(1803),
    [sym__wc_file_read_int] = ACTIONS(1803),
    [sym__wc_file_read_line_int] = ACTIONS(1803),
    [sym__wc_file_write_int] = ACTIONS(1803),
    [sym__wc_file_write_line_int] = ACTIONS(1803),
    [aux_sym_preproc_include_token1] = ACTIONS(1803),
    [aux_sym_preproc_define_token1] = ACTIONS(1805),
    [aux_sym_preproc_warn_token1] = ACTIONS(1805),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1805),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1805),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1805),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1805),
    [aux_sym_preproc_else_token1] = ACTIONS(1805),
    [anon_sym_LBRACK] = ACTIONS(1805),
    [anon_sym_LBRACE] = ACTIONS(1805),
    [anon_sym_LPAREN] = ACTIONS(1803),
    [anon_sym_BANG] = ACTIONS(1805),
    [anon_sym_TILDE] = ACTIONS(1805),
    [anon_sym_DASH] = ACTIONS(1803),
    [anon_sym_PLUS] = ACTIONS(1803),
    [anon_sym_DASH_DASH] = ACTIONS(1805),
    [anon_sym_PLUS_PLUS] = ACTIONS(1805),
    [anon_sym_DQUOTE] = ACTIONS(1805),
    [anon_sym_SQUOTE] = ACTIONS(1805),
    [sym_number_literal] = ACTIONS(1805),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(246)] = {
    [sym_identifier] = ACTIONS(1807),
    [sym_program_name_keyword] = ACTIONS(1807),
    [sym_module_name_keyword] = ACTIONS(1807),
    [sym_define_device_keyword] = ACTIONS(1807),
    [sym_define_combine_keyword] = ACTIONS(1807),
    [sym_define_constant_keyword] = ACTIONS(1807),
    [sym_define_type_keyword] = ACTIONS(1807),
    [sym_define_variable_keyword] = ACTIONS(1807),
    [sym_define_system_variable_keyword] = ACTIONS(1807),
    [sym_define_connect_level_keyword] = ACTIONS(1807),
    [sym_define_latching_keyword] = ACTIONS(1807),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1807),
    [sym_define_toggling_keyword] = ACTIONS(1807),
    [sym_define_call_keyword] = ACTIONS(1807),
    [sym_define_function_keyword] = ACTIONS(1807),
    [sym_define_library_function_keyword] = ACTIONS(1807),
    [sym_define_start_keyword] = ACTIONS(1807),
    [sym_define_event_keyword] = ACTIONS(1807),
    [sym_define_program_keyword] = ACTIONS(1807),
    [sym_define_module_keyword] = ACTIONS(1807),
    [sym_char_keyword] = ACTIONS(1807),
    [sym_widechar_keyword] = ACTIONS(1807),
    [sym_integer_keyword] = ACTIONS(1807),
    [sym_sinteger_keyword] = ACTIONS(1807),
    [sym_long_keyword] = ACTIONS(1807),
    [sym_slong_keyword] = ACTIONS(1807),
    [sym_float_keyword] = ACTIONS(1807),
    [sym_double_keyword] = ACTIONS(1807),
    [sym_variant_keyword] = ACTIONS(1807),
    [sym_variantarray_keyword] = ACTIONS(1807),
    [sym_local_var_keyword] = ACTIONS(1807),
    [sym_stack_var_keyword] = ACTIONS(1807),
    [sym_constant_keyword] = ACTIONS(1807),
    [sym_volatile_keyword] = ACTIONS(1807),
    [sym_non_volatile_keyword] = ACTIONS(1807),
    [sym_persistent_keyword] = ACTIONS(1807),
    [sym_dev_keyword] = ACTIONS(1807),
    [sym_devlev_keyword] = ACTIONS(1807),
    [sym_devchan_keyword] = ACTIONS(1807),
    [sym_if_keyword] = ACTIONS(1807),
    [sym_else_keyword] = ACTIONS(1807),
    [sym_switch_keyword] = ACTIONS(1807),
    [sym_case_keyword] = ACTIONS(1807),
    [sym_default_keyword] = ACTIONS(1807),
    [sym_while_keyword] = ACTIONS(1807),
    [sym_for_keyword] = ACTIONS(1807),
    [sym_break_keyword] = ACTIONS(1807),
    [sym_continue_keyword] = ACTIONS(1807),
    [sym_return_keyword] = ACTIONS(1807),
    [sym_select_keyword] = ACTIONS(1807),
    [sym_active_keyword] = ACTIONS(1807),
    [sym_struct_keyword] = ACTIONS(1807),
    [sym_structure_keyword] = ACTIONS(1807),
    [sym_button_event_keyword] = ACTIONS(1807),
    [sym_channel_event_keyword] = ACTIONS(1807),
    [sym_level_event_keyword] = ACTIONS(1807),
    [sym_data_event_keyword] = ACTIONS(1807),
    [sym_timeline_event_keyword] = ACTIONS(1807),
    [sym_custom_event_keyword] = ACTIONS(1807),
    [sym_push_keyword] = ACTIONS(1807),
    [sym_release_keyword] = ACTIONS(1807),
    [aux_sym_on_keyword_token1] = ACTIONS(1807),
    [aux_sym_off_keyword_token1] = ACTIONS(1807),
    [sym_send_level_keyword] = ACTIONS(1807),
    [sym_send_string_keyword] = ACTIONS(1807),
    [sym_send_command_keyword] = ACTIONS(1807),
    [sym_clear_buffer_keyword] = ACTIONS(1807),
    [sym_create_buffer_keyword] = ACTIONS(1807),
    [sym_create_multi_buffer_keyword] = ACTIONS(1807),
    [sym_call_keyword] = ACTIONS(1807),
    [sym_system_call_keyword] = ACTIONS(1807),
    [sym_devchan_to_keyword] = ACTIONS(1807),
    [sym_devchan_min_to_keyword] = ACTIONS(1807),
    [sym_devchan_total_off_keyword] = ACTIONS(1807),
    [sym_devchan_pulse_keyword] = ACTIONS(1807),
    [sym_wait_keyword] = ACTIONS(1807),
    [sym_cancel_wait_keyword] = ACTIONS(1807),
    [sym_wait_until_keyword] = ACTIONS(1807),
    [sym_cancel_wait_until_keyword] = ACTIONS(1807),
    [sym_cancel_all_wait_keyword] = ACTIONS(1807),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1807),
    [sym_not] = ACTIONS(1807),
    [sym_bnot] = ACTIONS(1807),
    [sym___file__] = ACTIONS(1807),
    [sym___date__] = ACTIONS(1807),
    [sym___ldate__] = ACTIONS(1807),
    [sym___time__] = ACTIONS(1807),
    [sym___line__] = ACTIONS(1807),
    [sym___name__] = ACTIONS(1807),
    [sym___netlinx__] = ACTIONS(1807),
    [sym_netlinx_axi_version] = ACTIONS(1807),
    [sym_url_flg_tcp] = ACTIONS(1807),
    [sym_url_flg_accinfopresent] = ACTIONS(1807),
    [sym_url_flg_temp] = ACTIONS(1807),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1807),
    [sym_url_flg_stat_mask] = ACTIONS(1807),
    [sym_url_flg_stat_lookup] = ACTIONS(1807),
    [sym_url_flg_stat_connecting] = ACTIONS(1807),
    [sym_url_flg_stat_waiting] = ACTIONS(1807),
    [sym_url_flg_stat_connected] = ACTIONS(1807),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1807),
    [sym_first_virtual_device] = ACTIONS(1807),
    [sym_first_local_port] = ACTIONS(1807),
    [sym_dynamic_virtual_device] = ACTIONS(1807),
    [sym_timeline_once] = ACTIONS(1807),
    [sym_timeline_repeat] = ACTIONS(1807),
    [sym_timeline_absolute] = ACTIONS(1807),
    [sym_timeline_relative] = ACTIONS(1807),
    [sym_do_push_timed_infinite] = ACTIONS(1807),
    [sym_source_type_no_address] = ACTIONS(1807),
    [sym_source_type_neuron_id] = ACTIONS(1807),
    [sym_source_type_ip_address] = ACTIONS(1807),
    [sym_source_type_axlink] = ACTIONS(1807),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1807),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1807),
    [sym_source_type_ip_socket_address] = ACTIONS(1807),
    [sym_source_type_rs232] = ACTIONS(1807),
    [sym_source_type_internal] = ACTIONS(1807),
    [sym_source_type_mac_address] = ACTIONS(1807),
    [sym_source_type_ipv4_port] = ACTIONS(1807),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1807),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1807),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1807),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1807),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1807),
    [sym_true] = ACTIONS(1807),
    [sym_false] = ACTIONS(1807),
    [sym_file_read_only] = ACTIONS(1807),
    [sym_file_rw_new] = ACTIONS(1807),
    [sym_file_rw_append] = ACTIONS(1807),
    [sym_ip_tcp] = ACTIONS(1807),
    [sym_ip_udp] = ACTIONS(1807),
    [sym_ip_udp_2way] = ACTIONS(1807),
    [sym_xml_encode_types] = ACTIONS(1807),
    [sym_xml_encode_char_as_list] = ACTIONS(1807),
    [sym_xml_encode_le] = ACTIONS(1807),
    [sym_xml_decode_types] = ACTIONS(1807),
    [sym_xml_decode_no_preserve] = ACTIONS(1807),
    [sym_duet_dev_type_display_device] = ACTIONS(1807),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1807),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1807),
    [sym_duet_dev_type_io_device] = ACTIONS(1807),
    [sym_duet_dev_type_relay_device] = ACTIONS(1807),
    [sym_duet_dev_type_ups] = ACTIONS(1807),
    [sym_duet_dev_type_amplifier] = ACTIONS(1807),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1807),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1807),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1807),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1807),
    [sym_duet_dev_type_camera] = ACTIONS(1807),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1807),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1807),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1807),
    [sym_duet_dev_type_dss] = ACTIONS(1807),
    [sym_duet_dev_type_dvr] = ACTIONS(1807),
    [sym_duet_dev_type_disc_device] = ACTIONS(1807),
    [sym_duet_dev_type_document_camera] = ACTIONS(1807),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1807),
    [sym_duet_dev_type_hvac] = ACTIONS(1807),
    [sym_duet_dev_type_keypad] = ACTIONS(1807),
    [sym_duet_dev_type_light] = ACTIONS(1807),
    [sym_duet_dev_type_monitor] = ACTIONS(1807),
    [sym_duet_dev_type_motor] = ACTIONS(1807),
    [sym_duet_dev_type_multi_window] = ACTIONS(1807),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1807),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1807),
    [sym_duet_dev_type_receiver] = ACTIONS(1807),
    [sym_duet_dev_type_security_system] = ACTIONS(1807),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1807),
    [sym_duet_dev_type_settop_box] = ACTIONS(1807),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1807),
    [sym_duet_dev_type_switcher] = ACTIONS(1807),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1807),
    [sym_duet_dev_type_tv] = ACTIONS(1807),
    [sym_duet_dev_type_utility] = ACTIONS(1807),
    [sym_duet_dev_type_vcr] = ACTIONS(1807),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1807),
    [sym_duet_dev_type_video_processor] = ACTIONS(1807),
    [sym_duet_dev_type_video_projector] = ACTIONS(1807),
    [sym_duet_dev_type_video_wall] = ACTIONS(1807),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1807),
    [sym_duet_dev_type_weather] = ACTIONS(1807),
    [sym_duet_dev_not_polled] = ACTIONS(1807),
    [sym_duet_dev_polled] = ACTIONS(1807),
    [sym_clkmgr_mode_network] = ACTIONS(1807),
    [sym_clkmgr_mode_standalone] = ACTIONS(1807),
    [sym_normal_standby] = ACTIONS(1807),
    [sym_normal_wake] = ACTIONS(1807),
    [sym_ip_multicast_ttl_option] = ACTIONS(1807),
    [sym_ip_ttl_subnet] = ACTIONS(1807),
    [sym_ip_ttl_site] = ACTIONS(1807),
    [sym_ip_ttl_region] = ACTIONS(1807),
    [sym_ip_ttl_continent] = ACTIONS(1807),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1807),
    [sym_ip_nodelay_on] = ACTIONS(1807),
    [sym_ip_nodelay_off] = ACTIONS(1807),
    [sym_amx_error] = ACTIONS(1807),
    [sym_amx_warning] = ACTIONS(1807),
    [sym_amx_info] = ACTIONS(1807),
    [sym_amx_debug] = ACTIONS(1807),
    [sym_netlinx_login_success] = ACTIONS(1807),
    [sym_netlinx_login_fail] = ACTIONS(1807),
    [sym_netlinx_logout] = ACTIONS(1807),
    [sym_null_str] = ACTIONS(1807),
    [sym_smtp_address] = ACTIONS(1807),
    [sym_smtp_port_number] = ACTIONS(1807),
    [sym_smtp_username] = ACTIONS(1807),
    [sym_smtp_password] = ACTIONS(1807),
    [sym_smtp_require_tls] = ACTIONS(1807),
    [sym_smtp_from] = ACTIONS(1807),
    [sym_smtp_tls_true] = ACTIONS(1807),
    [sym_smtp_tls_false] = ACTIONS(1807),
    [sym_listview_on_row_select_event] = ACTIONS(1807),
    [sym_data_structure_unknown] = ACTIONS(1807),
    [sym_data_structure_datafeed] = ACTIONS(1807),
    [sym_data_structure_datarecordset] = ACTIONS(1807),
    [sym_data_structure_datarecord] = ACTIONS(1807),
    [sym_data_structure_datafield] = ACTIONS(1807),
    [sym_data_max_content_field_count] = ACTIONS(1807),
    [sym_data_max_metadata_field_count] = ACTIONS(1807),
    [sym_data_max_id_length] = ACTIONS(1807),
    [sym_data_max_name_length] = ACTIONS(1807),
    [sym_data_max_description_length] = ACTIONS(1807),
    [sym_data_max_type_length] = ACTIONS(1807),
    [sym_data_max_format_length] = ACTIONS(1807),
    [sym_data_max_source_length] = ACTIONS(1807),
    [sym_data_max_value_length] = ACTIONS(1807),
    [sym_data_max_label_length] = ACTIONS(1807),
    [sym_data_max_url_length] = ACTIONS(1807),
    [sym_max_last_login_info_length] = ACTIONS(1807),
    [sym_data_type_unknown] = ACTIONS(1807),
    [sym_data_type_string] = ACTIONS(1807),
    [sym_data_type_datetime] = ACTIONS(1807),
    [sym_data_type_date] = ACTIONS(1807),
    [sym_data_type_time] = ACTIONS(1807),
    [sym_data_type_image] = ACTIONS(1807),
    [sym_data_format_url] = ACTIONS(1807),
    [sym_data_format_phone] = ACTIONS(1807),
    [sym_data_format_email] = ACTIONS(1807),
    [sym_data_format_iso8601] = ACTIONS(1807),
    [sym_auditlog_priv] = ACTIONS(1807),
    [sym_configuration_priv] = ACTIONS(1807),
    [sym_deviceconfig_priv] = ACTIONS(1807),
    [sym_ftp_priv] = ACTIONS(1807),
    [sym_http_priv] = ACTIONS(1807),
    [sym_networkconfig_priv] = ACTIONS(1807),
    [sym_programport_priv] = ACTIONS(1807),
    [sym_remoteui_priv] = ACTIONS(1807),
    [sym_securitycontrol_priv] = ACTIONS(1807),
    [sym_softwaremanagement_priv] = ACTIONS(1807),
    [sym_terminal_priv] = ACTIONS(1807),
    [sym_tpadmin_priv] = ACTIONS(1807),
    [sym_user1_priv] = ACTIONS(1807),
    [sym_user2_priv] = ACTIONS(1807),
    [sym_user3_priv] = ACTIONS(1807),
    [sym_user4_priv] = ACTIONS(1807),
    [sym_usermanagement_priv] = ACTIONS(1807),
    [sym_valid_account] = ACTIONS(1807),
    [sym_err_name_invalid] = ACTIONS(1807),
    [sym_err_password_invalid] = ACTIONS(1807),
    [sym_err_invalid_account] = ACTIONS(1807),
    [sym_err_no_authorization] = ACTIONS(1807),
    [sym_err_invalid_parameter] = ACTIONS(1807),
    [sym_err_user_locked_out] = ACTIONS(1807),
    [sym_err_user_account_expired] = ACTIONS(1807),
    [sym_err_general_error] = ACTIONS(1807),
    [sym_err_authentication_error] = ACTIONS(1807),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1807),
    [sym_tls_validate_certificate] = ACTIONS(1807),
    [sym_tdata] = ACTIONS(1807),
    [sym_tchannel] = ACTIONS(1807),
    [sym_tlevel] = ACTIONS(1807),
    [sym_tbutton] = ACTIONS(1807),
    [sym_ttimeline] = ACTIONS(1807),
    [sym_tcustom] = ACTIONS(1807),
    [sym_url_struct] = ACTIONS(1807),
    [sym_dns_struct] = ACTIONS(1807),
    [sym_ip_address_struct] = ACTIONS(1807),
    [sym_dev_info_struct] = ACTIONS(1807),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1807),
    [sym_data_feed] = ACTIONS(1807),
    [sym_data_field] = ACTIONS(1807),
    [sym_data_record] = ACTIONS(1807),
    [sym_wc_data_feed] = ACTIONS(1807),
    [sym_wc_data_field] = ACTIONS(1807),
    [sym_wc_data_record] = ACTIONS(1807),
    [sym_last_login_info] = ACTIONS(1807),
    [sym____reserved___] = ACTIONS(1807),
    [sym_date] = ACTIONS(1807),
    [sym_day] = ACTIONS(1807),
    [sym_time] = ACTIONS(1807),
    [sym_ldate] = ACTIONS(1807),
    [sym_push_device] = ACTIONS(1807),
    [sym_push_channel] = ACTIONS(1807),
    [sym_push_devchan] = ACTIONS(1807),
    [sym_release_device] = ACTIONS(1807),
    [sym_release_channel] = ACTIONS(1807),
    [sym_release_devchan] = ACTIONS(1807),
    [sym_master_slot] = ACTIONS(1807),
    [sym_get_pulse_time] = ACTIONS(1807),
    [sym_get_timer] = ACTIONS(1807),
    [sym_master_sn] = ACTIONS(1807),
    [sym_system_number] = ACTIONS(1807),
    [sym_button] = ACTIONS(1807),
    [sym_dv_channel] = ACTIONS(1807),
    [sym_data] = ACTIONS(1807),
    [sym_level] = ACTIONS(1807),
    [sym_timeline] = ACTIONS(1807),
    [sym_channel] = ACTIONS(1807),
    [sym_custom] = ACTIONS(1807),
    [sym_atoi] = ACTIONS(1807),
    [sym_atol] = ACTIONS(1807),
    [sym_atof] = ACTIONS(1807),
    [sym_itoa] = ACTIONS(1807),
    [sym_itohex] = ACTIONS(1807),
    [sym_hextoi] = ACTIONS(1807),
    [sym_ftoa] = ACTIONS(1807),
    [sym_find_string] = ACTIONS(1807),
    [sym_left_string] = ACTIONS(1807),
    [sym_right_string] = ACTIONS(1807),
    [sym_mid_string] = ACTIONS(1807),
    [sym_remove_string] = ACTIONS(1807),
    [sym_get_buffer_string] = ACTIONS(1807),
    [sym_get_buffer_char] = ACTIONS(1807),
    [sym_get_multi_buffer_string] = ACTIONS(1807),
    [sym_length_string] = ACTIONS(1807),
    [sym_max_length_string] = ACTIONS(1807),
    [sym_set_length_string] = ACTIONS(1807),
    [sym_compare_string] = ACTIONS(1807),
    [sym_upper_string] = ACTIONS(1807),
    [sym_lower_string] = ACTIONS(1807),
    [sym_get_last] = ACTIONS(1807),
    [sym_date_to_day] = ACTIONS(1807),
    [sym_date_to_month] = ACTIONS(1807),
    [sym_date_to_year] = ACTIONS(1807),
    [sym_time_to_hour] = ACTIONS(1807),
    [sym_time_to_minute] = ACTIONS(1807),
    [sym_time_to_second] = ACTIONS(1807),
    [sym_day_of_week] = ACTIONS(1807),
    [sym_set_pulse_time] = ACTIONS(1807),
    [sym_set_timer] = ACTIONS(1807),
    [sym_file_open] = ACTIONS(1807),
    [sym_file_close] = ACTIONS(1807),
    [sym_file_read] = ACTIONS(1807),
    [sym_file_read_line] = ACTIONS(1807),
    [sym_file_write] = ACTIONS(1807),
    [sym_file_write_line] = ACTIONS(1807),
    [sym_file_copy] = ACTIONS(1807),
    [sym_file_delete] = ACTIONS(1807),
    [sym_file_rename] = ACTIONS(1807),
    [sym_file_dir] = ACTIONS(1807),
    [sym_file_getdir] = ACTIONS(1807),
    [sym_file_setdir] = ACTIONS(1807),
    [sym_file_create_dir] = ACTIONS(1807),
    [sym_file_remove_dir] = ACTIONS(1807),
    [sym_file_seek] = ACTIONS(1807),
    [sym_length_array] = ACTIONS(1807),
    [sym_max_length_array] = ACTIONS(1807),
    [sym_set_length_array] = ACTIONS(1807),
    [sym_ip_client_open] = ACTIONS(1807),
    [sym_ip_client_close] = ACTIONS(1807),
    [sym_ip_server_open] = ACTIONS(1807),
    [sym_ip_server_close] = ACTIONS(1807),
    [sym_ip_bound_client_open] = ACTIONS(1807),
    [sym_ip_mc_server_open] = ACTIONS(1807),
    [sym_ip_set_option] = ACTIONS(1807),
    [sym_ssh_client_open] = ACTIONS(1807),
    [sym_ssh_client_close] = ACTIONS(1807),
    [sym_tls_client_open] = ACTIONS(1807),
    [sym_tls_client_close] = ACTIONS(1807),
    [sym_do_push] = ACTIONS(1807),
    [sym_do_push_timed] = ACTIONS(1807),
    [sym_do_release] = ACTIONS(1807),
    [sym_device_id] = ACTIONS(1807),
    [sym_device_info] = ACTIONS(1807),
    [sym_device_id_string] = ACTIONS(1807),
    [sym_device_standby] = ACTIONS(1807),
    [sym_device_wake] = ACTIONS(1807),
    [sym_timeline_create] = ACTIONS(1807),
    [sym_timeline_kill] = ACTIONS(1807),
    [sym_timeline_reload] = ACTIONS(1807),
    [sym_timeline_pause] = ACTIONS(1807),
    [sym_timeline_restart] = ACTIONS(1807),
    [sym_timeline_set] = ACTIONS(1807),
    [sym_timeline_get] = ACTIONS(1807),
    [sym_timeline_active] = ACTIONS(1807),
    [sym_timeline_dynamic_id] = ACTIONS(1807),
    [sym_variable_to_string] = ACTIONS(1807),
    [sym_string_to_variable] = ACTIONS(1807),
    [sym_length_variable_to_string] = ACTIONS(1807),
    [sym_type_cast] = ACTIONS(1807),
    [sym_variable_to_xml] = ACTIONS(1807),
    [sym_xml_to_variable] = ACTIONS(1807),
    [sym_length_variable_to_xml] = ACTIONS(1807),
    [sym_redirect_string] = ACTIONS(1807),
    [sym_random_number] = ACTIONS(1807),
    [sym_get_unique_id] = ACTIONS(1807),
    [sym_get_system_number] = ACTIONS(1807),
    [sym_set_system_number] = ACTIONS(1807),
    [sym_get_serial_number] = ACTIONS(1807),
    [sym_get_master_build] = ACTIONS(1807),
    [sym_set_validation_code] = ACTIONS(1807),
    [sym_reboot] = ACTIONS(1807),
    [sym_load_duet_module] = ACTIONS(1807),
    [sym_unload_duet_module] = ACTIONS(1807),
    [sym_duet_mem_size_set] = ACTIONS(1807),
    [sym_duet_mem_size_get] = ACTIONS(1807),
    [sym_dynamic_application_device] = ACTIONS(1807),
    [sym_dynamic_polled_port] = ACTIONS(1807),
    [sym_static_port_binding] = ACTIONS(1807),
    [sym_static_ip_binding] = ACTIONS(1807),
    [sym_get_dns_list] = ACTIONS(1807),
    [sym_set_dns_list] = ACTIONS(1807),
    [sym_get_ip_address] = ACTIONS(1807),
    [sym_set_ip_address] = ACTIONS(1807),
    [sym_get_url_list] = ACTIONS(1807),
    [sym_add_url_entry] = ACTIONS(1807),
    [sym_delete_url_entry] = ACTIONS(1807),
    [sym_abs_value] = ACTIONS(1807),
    [sym_max_value] = ACTIONS(1807),
    [sym_min_value] = ACTIONS(1807),
    [sym_exp_value] = ACTIONS(1807),
    [sym_log_value] = ACTIONS(1807),
    [sym_log10_value] = ACTIONS(1807),
    [sym_power_value] = ACTIONS(1807),
    [sym_sqrt_value] = ACTIONS(1807),
    [sym_raw_be] = ACTIONS(1807),
    [sym_raw_le] = ACTIONS(1807),
    [sym_internal_queue_size_set] = ACTIONS(1807),
    [sym_internal_queue_size_get] = ACTIONS(1807),
    [sym_internal_threshold_set] = ACTIONS(1807),
    [sym_internal_threshold_get] = ACTIONS(1807),
    [sym_rebuilt_event] = ACTIONS(1807),
    [sym_do_custom_event] = ACTIONS(1807),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1807),
    [sym_clkmgr_set_clk_source] = ACTIONS(1807),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1807),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1807),
    [sym_clkmgr_get_timezone] = ACTIONS(1807),
    [sym_clkmgr_set_timezone] = ACTIONS(1807),
    [sym_clkmgr_get_resync_period] = ACTIONS(1807),
    [sym_clkmgr_set_resync_period] = ACTIONS(1807),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1807),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1807),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1807),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1807),
    [sym_clkmgr_get_timeservers] = ACTIONS(1807),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1807),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1807),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1807),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1807),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1807),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1807),
    [sym_set_log_level] = ACTIONS(1807),
    [sym_get_log_level] = ACTIONS(1807),
    [sym_amx_log] = ACTIONS(1807),
    [sym_smtp_server_config_set] = ACTIONS(1807),
    [sym_smtp_server_config_get] = ACTIONS(1807),
    [sym_smtp_send] = ACTIONS(1807),
    [sym_validate_netlinx_account] = ACTIONS(1807),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1807),
    [sym_authenticate_certificate] = ACTIONS(1807),
    [sym_audit_netlinx_session_event] = ACTIONS(1807),
    [sym_audit_netlinx_generic_event] = ACTIONS(1807),
    [sym_get_available_flash_disk_space] = ACTIONS(1807),
    [sym_get_max_flash_disk_space] = ACTIONS(1807),
    [sym_data_create_feed] = ACTIONS(1807),
    [sym_data_delete_feed] = ACTIONS(1807),
    [sym_data_publish_feed] = ACTIONS(1807),
    [sym_data_get_published_feed] = ACTIONS(1807),
    [sym_data_add_record] = ACTIONS(1807),
    [sym_data_get_event_record] = ACTIONS(1807),
    [sym__wc_data_create_feed] = ACTIONS(1807),
    [sym__wc_data_add_record] = ACTIONS(1807),
    [sym__wc_data_get_event_record] = ACTIONS(1807),
    [sym_format] = ACTIONS(1807),
    [sym_astro_clock] = ACTIONS(1807),
    [sym_set_outdoor_temperature] = ACTIONS(1807),
    [sym_set_virtual_level_count] = ACTIONS(1807),
    [sym_set_virtual_channel_count] = ACTIONS(1807),
    [sym_set_virtual_port_count] = ACTIONS(1807),
    [sym__wc_encode_int] = ACTIONS(1807),
    [sym__wc_decode_int] = ACTIONS(1807),
    [sym__wc_int] = ACTIONS(1807),
    [sym__wc_to_ch_int] = ACTIONS(1807),
    [sym__ch_to_wc_int] = ACTIONS(1807),
    [sym__wc_find_string_int] = ACTIONS(1807),
    [sym__wc_left_string_int] = ACTIONS(1807),
    [sym__wc_lower_string_int] = ACTIONS(1807),
    [sym__wc_mid_string_int] = ACTIONS(1807),
    [sym__wc_remove_string_int] = ACTIONS(1807),
    [sym__wc_right_string_int] = ACTIONS(1807),
    [sym__wc_upper_string_int] = ACTIONS(1807),
    [sym__wc_compare_string_int] = ACTIONS(1807),
    [sym__wc_get_buffer_char_int] = ACTIONS(1807),
    [sym__wc_get_buffer_string_int] = ACTIONS(1807),
    [sym__wc_concat_string_int] = ACTIONS(1807),
    [sym__wc_file_open_int] = ACTIONS(1807),
    [sym__wc_file_close_int] = ACTIONS(1807),
    [sym__wc_file_read_int] = ACTIONS(1807),
    [sym__wc_file_read_line_int] = ACTIONS(1807),
    [sym__wc_file_write_int] = ACTIONS(1807),
    [sym__wc_file_write_line_int] = ACTIONS(1807),
    [aux_sym_preproc_include_token1] = ACTIONS(1807),
    [aux_sym_preproc_define_token1] = ACTIONS(1809),
    [aux_sym_preproc_warn_token1] = ACTIONS(1809),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1809),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1809),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1809),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1809),
    [aux_sym_preproc_else_token1] = ACTIONS(1809),
    [anon_sym_LBRACK] = ACTIONS(1809),
    [anon_sym_LBRACE] = ACTIONS(1809),
    [anon_sym_LPAREN] = ACTIONS(1807),
    [anon_sym_BANG] = ACTIONS(1809),
    [anon_sym_TILDE] = ACTIONS(1809),
    [anon_sym_DASH] = ACTIONS(1807),
    [anon_sym_PLUS] = ACTIONS(1807),
    [anon_sym_DASH_DASH] = ACTIONS(1809),
    [anon_sym_PLUS_PLUS] = ACTIONS(1809),
    [anon_sym_DQUOTE] = ACTIONS(1809),
    [anon_sym_SQUOTE] = ACTIONS(1809),
    [sym_number_literal] = ACTIONS(1809),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(247)] = {
    [sym_identifier] = ACTIONS(1811),
    [sym_program_name_keyword] = ACTIONS(1811),
    [sym_module_name_keyword] = ACTIONS(1811),
    [sym_define_device_keyword] = ACTIONS(1811),
    [sym_define_combine_keyword] = ACTIONS(1811),
    [sym_define_constant_keyword] = ACTIONS(1811),
    [sym_define_type_keyword] = ACTIONS(1811),
    [sym_define_variable_keyword] = ACTIONS(1811),
    [sym_define_system_variable_keyword] = ACTIONS(1811),
    [sym_define_connect_level_keyword] = ACTIONS(1811),
    [sym_define_latching_keyword] = ACTIONS(1811),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1811),
    [sym_define_toggling_keyword] = ACTIONS(1811),
    [sym_define_call_keyword] = ACTIONS(1811),
    [sym_define_function_keyword] = ACTIONS(1811),
    [sym_define_library_function_keyword] = ACTIONS(1811),
    [sym_define_start_keyword] = ACTIONS(1811),
    [sym_define_event_keyword] = ACTIONS(1811),
    [sym_define_program_keyword] = ACTIONS(1811),
    [sym_define_module_keyword] = ACTIONS(1811),
    [sym_char_keyword] = ACTIONS(1811),
    [sym_widechar_keyword] = ACTIONS(1811),
    [sym_integer_keyword] = ACTIONS(1811),
    [sym_sinteger_keyword] = ACTIONS(1811),
    [sym_long_keyword] = ACTIONS(1811),
    [sym_slong_keyword] = ACTIONS(1811),
    [sym_float_keyword] = ACTIONS(1811),
    [sym_double_keyword] = ACTIONS(1811),
    [sym_variant_keyword] = ACTIONS(1811),
    [sym_variantarray_keyword] = ACTIONS(1811),
    [sym_local_var_keyword] = ACTIONS(1811),
    [sym_stack_var_keyword] = ACTIONS(1811),
    [sym_constant_keyword] = ACTIONS(1811),
    [sym_volatile_keyword] = ACTIONS(1811),
    [sym_non_volatile_keyword] = ACTIONS(1811),
    [sym_persistent_keyword] = ACTIONS(1811),
    [sym_dev_keyword] = ACTIONS(1811),
    [sym_devlev_keyword] = ACTIONS(1811),
    [sym_devchan_keyword] = ACTIONS(1811),
    [sym_if_keyword] = ACTIONS(1811),
    [sym_else_keyword] = ACTIONS(1811),
    [sym_switch_keyword] = ACTIONS(1811),
    [sym_case_keyword] = ACTIONS(1811),
    [sym_default_keyword] = ACTIONS(1811),
    [sym_while_keyword] = ACTIONS(1811),
    [sym_for_keyword] = ACTIONS(1811),
    [sym_break_keyword] = ACTIONS(1811),
    [sym_continue_keyword] = ACTIONS(1811),
    [sym_return_keyword] = ACTIONS(1811),
    [sym_select_keyword] = ACTIONS(1811),
    [sym_active_keyword] = ACTIONS(1811),
    [sym_struct_keyword] = ACTIONS(1811),
    [sym_structure_keyword] = ACTIONS(1811),
    [sym_button_event_keyword] = ACTIONS(1811),
    [sym_channel_event_keyword] = ACTIONS(1811),
    [sym_level_event_keyword] = ACTIONS(1811),
    [sym_data_event_keyword] = ACTIONS(1811),
    [sym_timeline_event_keyword] = ACTIONS(1811),
    [sym_custom_event_keyword] = ACTIONS(1811),
    [sym_push_keyword] = ACTIONS(1811),
    [sym_release_keyword] = ACTIONS(1811),
    [aux_sym_on_keyword_token1] = ACTIONS(1811),
    [aux_sym_off_keyword_token1] = ACTIONS(1811),
    [sym_send_level_keyword] = ACTIONS(1811),
    [sym_send_string_keyword] = ACTIONS(1811),
    [sym_send_command_keyword] = ACTIONS(1811),
    [sym_clear_buffer_keyword] = ACTIONS(1811),
    [sym_create_buffer_keyword] = ACTIONS(1811),
    [sym_create_multi_buffer_keyword] = ACTIONS(1811),
    [sym_call_keyword] = ACTIONS(1811),
    [sym_system_call_keyword] = ACTIONS(1811),
    [sym_devchan_to_keyword] = ACTIONS(1811),
    [sym_devchan_min_to_keyword] = ACTIONS(1811),
    [sym_devchan_total_off_keyword] = ACTIONS(1811),
    [sym_devchan_pulse_keyword] = ACTIONS(1811),
    [sym_wait_keyword] = ACTIONS(1811),
    [sym_cancel_wait_keyword] = ACTIONS(1811),
    [sym_wait_until_keyword] = ACTIONS(1811),
    [sym_cancel_wait_until_keyword] = ACTIONS(1811),
    [sym_cancel_all_wait_keyword] = ACTIONS(1811),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1811),
    [sym_not] = ACTIONS(1811),
    [sym_bnot] = ACTIONS(1811),
    [sym___file__] = ACTIONS(1811),
    [sym___date__] = ACTIONS(1811),
    [sym___ldate__] = ACTIONS(1811),
    [sym___time__] = ACTIONS(1811),
    [sym___line__] = ACTIONS(1811),
    [sym___name__] = ACTIONS(1811),
    [sym___netlinx__] = ACTIONS(1811),
    [sym_netlinx_axi_version] = ACTIONS(1811),
    [sym_url_flg_tcp] = ACTIONS(1811),
    [sym_url_flg_accinfopresent] = ACTIONS(1811),
    [sym_url_flg_temp] = ACTIONS(1811),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1811),
    [sym_url_flg_stat_mask] = ACTIONS(1811),
    [sym_url_flg_stat_lookup] = ACTIONS(1811),
    [sym_url_flg_stat_connecting] = ACTIONS(1811),
    [sym_url_flg_stat_waiting] = ACTIONS(1811),
    [sym_url_flg_stat_connected] = ACTIONS(1811),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1811),
    [sym_first_virtual_device] = ACTIONS(1811),
    [sym_first_local_port] = ACTIONS(1811),
    [sym_dynamic_virtual_device] = ACTIONS(1811),
    [sym_timeline_once] = ACTIONS(1811),
    [sym_timeline_repeat] = ACTIONS(1811),
    [sym_timeline_absolute] = ACTIONS(1811),
    [sym_timeline_relative] = ACTIONS(1811),
    [sym_do_push_timed_infinite] = ACTIONS(1811),
    [sym_source_type_no_address] = ACTIONS(1811),
    [sym_source_type_neuron_id] = ACTIONS(1811),
    [sym_source_type_ip_address] = ACTIONS(1811),
    [sym_source_type_axlink] = ACTIONS(1811),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1811),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1811),
    [sym_source_type_ip_socket_address] = ACTIONS(1811),
    [sym_source_type_rs232] = ACTIONS(1811),
    [sym_source_type_internal] = ACTIONS(1811),
    [sym_source_type_mac_address] = ACTIONS(1811),
    [sym_source_type_ipv4_port] = ACTIONS(1811),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1811),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1811),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1811),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1811),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1811),
    [sym_true] = ACTIONS(1811),
    [sym_false] = ACTIONS(1811),
    [sym_file_read_only] = ACTIONS(1811),
    [sym_file_rw_new] = ACTIONS(1811),
    [sym_file_rw_append] = ACTIONS(1811),
    [sym_ip_tcp] = ACTIONS(1811),
    [sym_ip_udp] = ACTIONS(1811),
    [sym_ip_udp_2way] = ACTIONS(1811),
    [sym_xml_encode_types] = ACTIONS(1811),
    [sym_xml_encode_char_as_list] = ACTIONS(1811),
    [sym_xml_encode_le] = ACTIONS(1811),
    [sym_xml_decode_types] = ACTIONS(1811),
    [sym_xml_decode_no_preserve] = ACTIONS(1811),
    [sym_duet_dev_type_display_device] = ACTIONS(1811),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1811),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1811),
    [sym_duet_dev_type_io_device] = ACTIONS(1811),
    [sym_duet_dev_type_relay_device] = ACTIONS(1811),
    [sym_duet_dev_type_ups] = ACTIONS(1811),
    [sym_duet_dev_type_amplifier] = ACTIONS(1811),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1811),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1811),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1811),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1811),
    [sym_duet_dev_type_camera] = ACTIONS(1811),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1811),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1811),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1811),
    [sym_duet_dev_type_dss] = ACTIONS(1811),
    [sym_duet_dev_type_dvr] = ACTIONS(1811),
    [sym_duet_dev_type_disc_device] = ACTIONS(1811),
    [sym_duet_dev_type_document_camera] = ACTIONS(1811),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1811),
    [sym_duet_dev_type_hvac] = ACTIONS(1811),
    [sym_duet_dev_type_keypad] = ACTIONS(1811),
    [sym_duet_dev_type_light] = ACTIONS(1811),
    [sym_duet_dev_type_monitor] = ACTIONS(1811),
    [sym_duet_dev_type_motor] = ACTIONS(1811),
    [sym_duet_dev_type_multi_window] = ACTIONS(1811),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1811),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1811),
    [sym_duet_dev_type_receiver] = ACTIONS(1811),
    [sym_duet_dev_type_security_system] = ACTIONS(1811),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1811),
    [sym_duet_dev_type_settop_box] = ACTIONS(1811),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1811),
    [sym_duet_dev_type_switcher] = ACTIONS(1811),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1811),
    [sym_duet_dev_type_tv] = ACTIONS(1811),
    [sym_duet_dev_type_utility] = ACTIONS(1811),
    [sym_duet_dev_type_vcr] = ACTIONS(1811),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1811),
    [sym_duet_dev_type_video_processor] = ACTIONS(1811),
    [sym_duet_dev_type_video_projector] = ACTIONS(1811),
    [sym_duet_dev_type_video_wall] = ACTIONS(1811),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1811),
    [sym_duet_dev_type_weather] = ACTIONS(1811),
    [sym_duet_dev_not_polled] = ACTIONS(1811),
    [sym_duet_dev_polled] = ACTIONS(1811),
    [sym_clkmgr_mode_network] = ACTIONS(1811),
    [sym_clkmgr_mode_standalone] = ACTIONS(1811),
    [sym_normal_standby] = ACTIONS(1811),
    [sym_normal_wake] = ACTIONS(1811),
    [sym_ip_multicast_ttl_option] = ACTIONS(1811),
    [sym_ip_ttl_subnet] = ACTIONS(1811),
    [sym_ip_ttl_site] = ACTIONS(1811),
    [sym_ip_ttl_region] = ACTIONS(1811),
    [sym_ip_ttl_continent] = ACTIONS(1811),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1811),
    [sym_ip_nodelay_on] = ACTIONS(1811),
    [sym_ip_nodelay_off] = ACTIONS(1811),
    [sym_amx_error] = ACTIONS(1811),
    [sym_amx_warning] = ACTIONS(1811),
    [sym_amx_info] = ACTIONS(1811),
    [sym_amx_debug] = ACTIONS(1811),
    [sym_netlinx_login_success] = ACTIONS(1811),
    [sym_netlinx_login_fail] = ACTIONS(1811),
    [sym_netlinx_logout] = ACTIONS(1811),
    [sym_null_str] = ACTIONS(1811),
    [sym_smtp_address] = ACTIONS(1811),
    [sym_smtp_port_number] = ACTIONS(1811),
    [sym_smtp_username] = ACTIONS(1811),
    [sym_smtp_password] = ACTIONS(1811),
    [sym_smtp_require_tls] = ACTIONS(1811),
    [sym_smtp_from] = ACTIONS(1811),
    [sym_smtp_tls_true] = ACTIONS(1811),
    [sym_smtp_tls_false] = ACTIONS(1811),
    [sym_listview_on_row_select_event] = ACTIONS(1811),
    [sym_data_structure_unknown] = ACTIONS(1811),
    [sym_data_structure_datafeed] = ACTIONS(1811),
    [sym_data_structure_datarecordset] = ACTIONS(1811),
    [sym_data_structure_datarecord] = ACTIONS(1811),
    [sym_data_structure_datafield] = ACTIONS(1811),
    [sym_data_max_content_field_count] = ACTIONS(1811),
    [sym_data_max_metadata_field_count] = ACTIONS(1811),
    [sym_data_max_id_length] = ACTIONS(1811),
    [sym_data_max_name_length] = ACTIONS(1811),
    [sym_data_max_description_length] = ACTIONS(1811),
    [sym_data_max_type_length] = ACTIONS(1811),
    [sym_data_max_format_length] = ACTIONS(1811),
    [sym_data_max_source_length] = ACTIONS(1811),
    [sym_data_max_value_length] = ACTIONS(1811),
    [sym_data_max_label_length] = ACTIONS(1811),
    [sym_data_max_url_length] = ACTIONS(1811),
    [sym_max_last_login_info_length] = ACTIONS(1811),
    [sym_data_type_unknown] = ACTIONS(1811),
    [sym_data_type_string] = ACTIONS(1811),
    [sym_data_type_datetime] = ACTIONS(1811),
    [sym_data_type_date] = ACTIONS(1811),
    [sym_data_type_time] = ACTIONS(1811),
    [sym_data_type_image] = ACTIONS(1811),
    [sym_data_format_url] = ACTIONS(1811),
    [sym_data_format_phone] = ACTIONS(1811),
    [sym_data_format_email] = ACTIONS(1811),
    [sym_data_format_iso8601] = ACTIONS(1811),
    [sym_auditlog_priv] = ACTIONS(1811),
    [sym_configuration_priv] = ACTIONS(1811),
    [sym_deviceconfig_priv] = ACTIONS(1811),
    [sym_ftp_priv] = ACTIONS(1811),
    [sym_http_priv] = ACTIONS(1811),
    [sym_networkconfig_priv] = ACTIONS(1811),
    [sym_programport_priv] = ACTIONS(1811),
    [sym_remoteui_priv] = ACTIONS(1811),
    [sym_securitycontrol_priv] = ACTIONS(1811),
    [sym_softwaremanagement_priv] = ACTIONS(1811),
    [sym_terminal_priv] = ACTIONS(1811),
    [sym_tpadmin_priv] = ACTIONS(1811),
    [sym_user1_priv] = ACTIONS(1811),
    [sym_user2_priv] = ACTIONS(1811),
    [sym_user3_priv] = ACTIONS(1811),
    [sym_user4_priv] = ACTIONS(1811),
    [sym_usermanagement_priv] = ACTIONS(1811),
    [sym_valid_account] = ACTIONS(1811),
    [sym_err_name_invalid] = ACTIONS(1811),
    [sym_err_password_invalid] = ACTIONS(1811),
    [sym_err_invalid_account] = ACTIONS(1811),
    [sym_err_no_authorization] = ACTIONS(1811),
    [sym_err_invalid_parameter] = ACTIONS(1811),
    [sym_err_user_locked_out] = ACTIONS(1811),
    [sym_err_user_account_expired] = ACTIONS(1811),
    [sym_err_general_error] = ACTIONS(1811),
    [sym_err_authentication_error] = ACTIONS(1811),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1811),
    [sym_tls_validate_certificate] = ACTIONS(1811),
    [sym_tdata] = ACTIONS(1811),
    [sym_tchannel] = ACTIONS(1811),
    [sym_tlevel] = ACTIONS(1811),
    [sym_tbutton] = ACTIONS(1811),
    [sym_ttimeline] = ACTIONS(1811),
    [sym_tcustom] = ACTIONS(1811),
    [sym_url_struct] = ACTIONS(1811),
    [sym_dns_struct] = ACTIONS(1811),
    [sym_ip_address_struct] = ACTIONS(1811),
    [sym_dev_info_struct] = ACTIONS(1811),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1811),
    [sym_data_feed] = ACTIONS(1811),
    [sym_data_field] = ACTIONS(1811),
    [sym_data_record] = ACTIONS(1811),
    [sym_wc_data_feed] = ACTIONS(1811),
    [sym_wc_data_field] = ACTIONS(1811),
    [sym_wc_data_record] = ACTIONS(1811),
    [sym_last_login_info] = ACTIONS(1811),
    [sym____reserved___] = ACTIONS(1811),
    [sym_date] = ACTIONS(1811),
    [sym_day] = ACTIONS(1811),
    [sym_time] = ACTIONS(1811),
    [sym_ldate] = ACTIONS(1811),
    [sym_push_device] = ACTIONS(1811),
    [sym_push_channel] = ACTIONS(1811),
    [sym_push_devchan] = ACTIONS(1811),
    [sym_release_device] = ACTIONS(1811),
    [sym_release_channel] = ACTIONS(1811),
    [sym_release_devchan] = ACTIONS(1811),
    [sym_master_slot] = ACTIONS(1811),
    [sym_get_pulse_time] = ACTIONS(1811),
    [sym_get_timer] = ACTIONS(1811),
    [sym_master_sn] = ACTIONS(1811),
    [sym_system_number] = ACTIONS(1811),
    [sym_button] = ACTIONS(1811),
    [sym_dv_channel] = ACTIONS(1811),
    [sym_data] = ACTIONS(1811),
    [sym_level] = ACTIONS(1811),
    [sym_timeline] = ACTIONS(1811),
    [sym_channel] = ACTIONS(1811),
    [sym_custom] = ACTIONS(1811),
    [sym_atoi] = ACTIONS(1811),
    [sym_atol] = ACTIONS(1811),
    [sym_atof] = ACTIONS(1811),
    [sym_itoa] = ACTIONS(1811),
    [sym_itohex] = ACTIONS(1811),
    [sym_hextoi] = ACTIONS(1811),
    [sym_ftoa] = ACTIONS(1811),
    [sym_find_string] = ACTIONS(1811),
    [sym_left_string] = ACTIONS(1811),
    [sym_right_string] = ACTIONS(1811),
    [sym_mid_string] = ACTIONS(1811),
    [sym_remove_string] = ACTIONS(1811),
    [sym_get_buffer_string] = ACTIONS(1811),
    [sym_get_buffer_char] = ACTIONS(1811),
    [sym_get_multi_buffer_string] = ACTIONS(1811),
    [sym_length_string] = ACTIONS(1811),
    [sym_max_length_string] = ACTIONS(1811),
    [sym_set_length_string] = ACTIONS(1811),
    [sym_compare_string] = ACTIONS(1811),
    [sym_upper_string] = ACTIONS(1811),
    [sym_lower_string] = ACTIONS(1811),
    [sym_get_last] = ACTIONS(1811),
    [sym_date_to_day] = ACTIONS(1811),
    [sym_date_to_month] = ACTIONS(1811),
    [sym_date_to_year] = ACTIONS(1811),
    [sym_time_to_hour] = ACTIONS(1811),
    [sym_time_to_minute] = ACTIONS(1811),
    [sym_time_to_second] = ACTIONS(1811),
    [sym_day_of_week] = ACTIONS(1811),
    [sym_set_pulse_time] = ACTIONS(1811),
    [sym_set_timer] = ACTIONS(1811),
    [sym_file_open] = ACTIONS(1811),
    [sym_file_close] = ACTIONS(1811),
    [sym_file_read] = ACTIONS(1811),
    [sym_file_read_line] = ACTIONS(1811),
    [sym_file_write] = ACTIONS(1811),
    [sym_file_write_line] = ACTIONS(1811),
    [sym_file_copy] = ACTIONS(1811),
    [sym_file_delete] = ACTIONS(1811),
    [sym_file_rename] = ACTIONS(1811),
    [sym_file_dir] = ACTIONS(1811),
    [sym_file_getdir] = ACTIONS(1811),
    [sym_file_setdir] = ACTIONS(1811),
    [sym_file_create_dir] = ACTIONS(1811),
    [sym_file_remove_dir] = ACTIONS(1811),
    [sym_file_seek] = ACTIONS(1811),
    [sym_length_array] = ACTIONS(1811),
    [sym_max_length_array] = ACTIONS(1811),
    [sym_set_length_array] = ACTIONS(1811),
    [sym_ip_client_open] = ACTIONS(1811),
    [sym_ip_client_close] = ACTIONS(1811),
    [sym_ip_server_open] = ACTIONS(1811),
    [sym_ip_server_close] = ACTIONS(1811),
    [sym_ip_bound_client_open] = ACTIONS(1811),
    [sym_ip_mc_server_open] = ACTIONS(1811),
    [sym_ip_set_option] = ACTIONS(1811),
    [sym_ssh_client_open] = ACTIONS(1811),
    [sym_ssh_client_close] = ACTIONS(1811),
    [sym_tls_client_open] = ACTIONS(1811),
    [sym_tls_client_close] = ACTIONS(1811),
    [sym_do_push] = ACTIONS(1811),
    [sym_do_push_timed] = ACTIONS(1811),
    [sym_do_release] = ACTIONS(1811),
    [sym_device_id] = ACTIONS(1811),
    [sym_device_info] = ACTIONS(1811),
    [sym_device_id_string] = ACTIONS(1811),
    [sym_device_standby] = ACTIONS(1811),
    [sym_device_wake] = ACTIONS(1811),
    [sym_timeline_create] = ACTIONS(1811),
    [sym_timeline_kill] = ACTIONS(1811),
    [sym_timeline_reload] = ACTIONS(1811),
    [sym_timeline_pause] = ACTIONS(1811),
    [sym_timeline_restart] = ACTIONS(1811),
    [sym_timeline_set] = ACTIONS(1811),
    [sym_timeline_get] = ACTIONS(1811),
    [sym_timeline_active] = ACTIONS(1811),
    [sym_timeline_dynamic_id] = ACTIONS(1811),
    [sym_variable_to_string] = ACTIONS(1811),
    [sym_string_to_variable] = ACTIONS(1811),
    [sym_length_variable_to_string] = ACTIONS(1811),
    [sym_type_cast] = ACTIONS(1811),
    [sym_variable_to_xml] = ACTIONS(1811),
    [sym_xml_to_variable] = ACTIONS(1811),
    [sym_length_variable_to_xml] = ACTIONS(1811),
    [sym_redirect_string] = ACTIONS(1811),
    [sym_random_number] = ACTIONS(1811),
    [sym_get_unique_id] = ACTIONS(1811),
    [sym_get_system_number] = ACTIONS(1811),
    [sym_set_system_number] = ACTIONS(1811),
    [sym_get_serial_number] = ACTIONS(1811),
    [sym_get_master_build] = ACTIONS(1811),
    [sym_set_validation_code] = ACTIONS(1811),
    [sym_reboot] = ACTIONS(1811),
    [sym_load_duet_module] = ACTIONS(1811),
    [sym_unload_duet_module] = ACTIONS(1811),
    [sym_duet_mem_size_set] = ACTIONS(1811),
    [sym_duet_mem_size_get] = ACTIONS(1811),
    [sym_dynamic_application_device] = ACTIONS(1811),
    [sym_dynamic_polled_port] = ACTIONS(1811),
    [sym_static_port_binding] = ACTIONS(1811),
    [sym_static_ip_binding] = ACTIONS(1811),
    [sym_get_dns_list] = ACTIONS(1811),
    [sym_set_dns_list] = ACTIONS(1811),
    [sym_get_ip_address] = ACTIONS(1811),
    [sym_set_ip_address] = ACTIONS(1811),
    [sym_get_url_list] = ACTIONS(1811),
    [sym_add_url_entry] = ACTIONS(1811),
    [sym_delete_url_entry] = ACTIONS(1811),
    [sym_abs_value] = ACTIONS(1811),
    [sym_max_value] = ACTIONS(1811),
    [sym_min_value] = ACTIONS(1811),
    [sym_exp_value] = ACTIONS(1811),
    [sym_log_value] = ACTIONS(1811),
    [sym_log10_value] = ACTIONS(1811),
    [sym_power_value] = ACTIONS(1811),
    [sym_sqrt_value] = ACTIONS(1811),
    [sym_raw_be] = ACTIONS(1811),
    [sym_raw_le] = ACTIONS(1811),
    [sym_internal_queue_size_set] = ACTIONS(1811),
    [sym_internal_queue_size_get] = ACTIONS(1811),
    [sym_internal_threshold_set] = ACTIONS(1811),
    [sym_internal_threshold_get] = ACTIONS(1811),
    [sym_rebuilt_event] = ACTIONS(1811),
    [sym_do_custom_event] = ACTIONS(1811),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1811),
    [sym_clkmgr_set_clk_source] = ACTIONS(1811),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1811),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1811),
    [sym_clkmgr_get_timezone] = ACTIONS(1811),
    [sym_clkmgr_set_timezone] = ACTIONS(1811),
    [sym_clkmgr_get_resync_period] = ACTIONS(1811),
    [sym_clkmgr_set_resync_period] = ACTIONS(1811),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1811),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1811),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1811),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1811),
    [sym_clkmgr_get_timeservers] = ACTIONS(1811),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1811),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1811),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1811),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1811),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1811),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1811),
    [sym_set_log_level] = ACTIONS(1811),
    [sym_get_log_level] = ACTIONS(1811),
    [sym_amx_log] = ACTIONS(1811),
    [sym_smtp_server_config_set] = ACTIONS(1811),
    [sym_smtp_server_config_get] = ACTIONS(1811),
    [sym_smtp_send] = ACTIONS(1811),
    [sym_validate_netlinx_account] = ACTIONS(1811),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1811),
    [sym_authenticate_certificate] = ACTIONS(1811),
    [sym_audit_netlinx_session_event] = ACTIONS(1811),
    [sym_audit_netlinx_generic_event] = ACTIONS(1811),
    [sym_get_available_flash_disk_space] = ACTIONS(1811),
    [sym_get_max_flash_disk_space] = ACTIONS(1811),
    [sym_data_create_feed] = ACTIONS(1811),
    [sym_data_delete_feed] = ACTIONS(1811),
    [sym_data_publish_feed] = ACTIONS(1811),
    [sym_data_get_published_feed] = ACTIONS(1811),
    [sym_data_add_record] = ACTIONS(1811),
    [sym_data_get_event_record] = ACTIONS(1811),
    [sym__wc_data_create_feed] = ACTIONS(1811),
    [sym__wc_data_add_record] = ACTIONS(1811),
    [sym__wc_data_get_event_record] = ACTIONS(1811),
    [sym_format] = ACTIONS(1811),
    [sym_astro_clock] = ACTIONS(1811),
    [sym_set_outdoor_temperature] = ACTIONS(1811),
    [sym_set_virtual_level_count] = ACTIONS(1811),
    [sym_set_virtual_channel_count] = ACTIONS(1811),
    [sym_set_virtual_port_count] = ACTIONS(1811),
    [sym__wc_encode_int] = ACTIONS(1811),
    [sym__wc_decode_int] = ACTIONS(1811),
    [sym__wc_int] = ACTIONS(1811),
    [sym__wc_to_ch_int] = ACTIONS(1811),
    [sym__ch_to_wc_int] = ACTIONS(1811),
    [sym__wc_find_string_int] = ACTIONS(1811),
    [sym__wc_left_string_int] = ACTIONS(1811),
    [sym__wc_lower_string_int] = ACTIONS(1811),
    [sym__wc_mid_string_int] = ACTIONS(1811),
    [sym__wc_remove_string_int] = ACTIONS(1811),
    [sym__wc_right_string_int] = ACTIONS(1811),
    [sym__wc_upper_string_int] = ACTIONS(1811),
    [sym__wc_compare_string_int] = ACTIONS(1811),
    [sym__wc_get_buffer_char_int] = ACTIONS(1811),
    [sym__wc_get_buffer_string_int] = ACTIONS(1811),
    [sym__wc_concat_string_int] = ACTIONS(1811),
    [sym__wc_file_open_int] = ACTIONS(1811),
    [sym__wc_file_close_int] = ACTIONS(1811),
    [sym__wc_file_read_int] = ACTIONS(1811),
    [sym__wc_file_read_line_int] = ACTIONS(1811),
    [sym__wc_file_write_int] = ACTIONS(1811),
    [sym__wc_file_write_line_int] = ACTIONS(1811),
    [aux_sym_preproc_include_token1] = ACTIONS(1811),
    [aux_sym_preproc_define_token1] = ACTIONS(1813),
    [aux_sym_preproc_warn_token1] = ACTIONS(1813),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1813),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1813),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1813),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1813),
    [aux_sym_preproc_else_token1] = ACTIONS(1813),
    [anon_sym_LBRACK] = ACTIONS(1813),
    [anon_sym_LBRACE] = ACTIONS(1813),
    [anon_sym_LPAREN] = ACTIONS(1811),
    [anon_sym_BANG] = ACTIONS(1813),
    [anon_sym_TILDE] = ACTIONS(1813),
    [anon_sym_DASH] = ACTIONS(1811),
    [anon_sym_PLUS] = ACTIONS(1811),
    [anon_sym_DASH_DASH] = ACTIONS(1813),
    [anon_sym_PLUS_PLUS] = ACTIONS(1813),
    [anon_sym_DQUOTE] = ACTIONS(1813),
    [anon_sym_SQUOTE] = ACTIONS(1813),
    [sym_number_literal] = ACTIONS(1813),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(248)] = {
    [sym_identifier] = ACTIONS(1815),
    [sym_program_name_keyword] = ACTIONS(1815),
    [sym_module_name_keyword] = ACTIONS(1815),
    [sym_define_device_keyword] = ACTIONS(1815),
    [sym_define_combine_keyword] = ACTIONS(1815),
    [sym_define_constant_keyword] = ACTIONS(1815),
    [sym_define_type_keyword] = ACTIONS(1815),
    [sym_define_variable_keyword] = ACTIONS(1815),
    [sym_define_system_variable_keyword] = ACTIONS(1815),
    [sym_define_connect_level_keyword] = ACTIONS(1815),
    [sym_define_latching_keyword] = ACTIONS(1815),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1815),
    [sym_define_toggling_keyword] = ACTIONS(1815),
    [sym_define_call_keyword] = ACTIONS(1815),
    [sym_define_function_keyword] = ACTIONS(1815),
    [sym_define_library_function_keyword] = ACTIONS(1815),
    [sym_define_start_keyword] = ACTIONS(1815),
    [sym_define_event_keyword] = ACTIONS(1815),
    [sym_define_program_keyword] = ACTIONS(1815),
    [sym_define_module_keyword] = ACTIONS(1815),
    [sym_char_keyword] = ACTIONS(1815),
    [sym_widechar_keyword] = ACTIONS(1815),
    [sym_integer_keyword] = ACTIONS(1815),
    [sym_sinteger_keyword] = ACTIONS(1815),
    [sym_long_keyword] = ACTIONS(1815),
    [sym_slong_keyword] = ACTIONS(1815),
    [sym_float_keyword] = ACTIONS(1815),
    [sym_double_keyword] = ACTIONS(1815),
    [sym_variant_keyword] = ACTIONS(1815),
    [sym_variantarray_keyword] = ACTIONS(1815),
    [sym_local_var_keyword] = ACTIONS(1815),
    [sym_stack_var_keyword] = ACTIONS(1815),
    [sym_constant_keyword] = ACTIONS(1815),
    [sym_volatile_keyword] = ACTIONS(1815),
    [sym_non_volatile_keyword] = ACTIONS(1815),
    [sym_persistent_keyword] = ACTIONS(1815),
    [sym_dev_keyword] = ACTIONS(1815),
    [sym_devlev_keyword] = ACTIONS(1815),
    [sym_devchan_keyword] = ACTIONS(1815),
    [sym_if_keyword] = ACTIONS(1815),
    [sym_else_keyword] = ACTIONS(1815),
    [sym_switch_keyword] = ACTIONS(1815),
    [sym_case_keyword] = ACTIONS(1815),
    [sym_default_keyword] = ACTIONS(1815),
    [sym_while_keyword] = ACTIONS(1815),
    [sym_for_keyword] = ACTIONS(1815),
    [sym_break_keyword] = ACTIONS(1815),
    [sym_continue_keyword] = ACTIONS(1815),
    [sym_return_keyword] = ACTIONS(1815),
    [sym_select_keyword] = ACTIONS(1815),
    [sym_active_keyword] = ACTIONS(1815),
    [sym_struct_keyword] = ACTIONS(1815),
    [sym_structure_keyword] = ACTIONS(1815),
    [sym_button_event_keyword] = ACTIONS(1815),
    [sym_channel_event_keyword] = ACTIONS(1815),
    [sym_level_event_keyword] = ACTIONS(1815),
    [sym_data_event_keyword] = ACTIONS(1815),
    [sym_timeline_event_keyword] = ACTIONS(1815),
    [sym_custom_event_keyword] = ACTIONS(1815),
    [sym_push_keyword] = ACTIONS(1815),
    [sym_release_keyword] = ACTIONS(1815),
    [aux_sym_on_keyword_token1] = ACTIONS(1815),
    [aux_sym_off_keyword_token1] = ACTIONS(1815),
    [sym_send_level_keyword] = ACTIONS(1815),
    [sym_send_string_keyword] = ACTIONS(1815),
    [sym_send_command_keyword] = ACTIONS(1815),
    [sym_clear_buffer_keyword] = ACTIONS(1815),
    [sym_create_buffer_keyword] = ACTIONS(1815),
    [sym_create_multi_buffer_keyword] = ACTIONS(1815),
    [sym_call_keyword] = ACTIONS(1815),
    [sym_system_call_keyword] = ACTIONS(1815),
    [sym_devchan_to_keyword] = ACTIONS(1815),
    [sym_devchan_min_to_keyword] = ACTIONS(1815),
    [sym_devchan_total_off_keyword] = ACTIONS(1815),
    [sym_devchan_pulse_keyword] = ACTIONS(1815),
    [sym_wait_keyword] = ACTIONS(1815),
    [sym_cancel_wait_keyword] = ACTIONS(1815),
    [sym_wait_until_keyword] = ACTIONS(1815),
    [sym_cancel_wait_until_keyword] = ACTIONS(1815),
    [sym_cancel_all_wait_keyword] = ACTIONS(1815),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1815),
    [sym_not] = ACTIONS(1815),
    [sym_bnot] = ACTIONS(1815),
    [sym___file__] = ACTIONS(1815),
    [sym___date__] = ACTIONS(1815),
    [sym___ldate__] = ACTIONS(1815),
    [sym___time__] = ACTIONS(1815),
    [sym___line__] = ACTIONS(1815),
    [sym___name__] = ACTIONS(1815),
    [sym___netlinx__] = ACTIONS(1815),
    [sym_netlinx_axi_version] = ACTIONS(1815),
    [sym_url_flg_tcp] = ACTIONS(1815),
    [sym_url_flg_accinfopresent] = ACTIONS(1815),
    [sym_url_flg_temp] = ACTIONS(1815),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1815),
    [sym_url_flg_stat_mask] = ACTIONS(1815),
    [sym_url_flg_stat_lookup] = ACTIONS(1815),
    [sym_url_flg_stat_connecting] = ACTIONS(1815),
    [sym_url_flg_stat_waiting] = ACTIONS(1815),
    [sym_url_flg_stat_connected] = ACTIONS(1815),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1815),
    [sym_first_virtual_device] = ACTIONS(1815),
    [sym_first_local_port] = ACTIONS(1815),
    [sym_dynamic_virtual_device] = ACTIONS(1815),
    [sym_timeline_once] = ACTIONS(1815),
    [sym_timeline_repeat] = ACTIONS(1815),
    [sym_timeline_absolute] = ACTIONS(1815),
    [sym_timeline_relative] = ACTIONS(1815),
    [sym_do_push_timed_infinite] = ACTIONS(1815),
    [sym_source_type_no_address] = ACTIONS(1815),
    [sym_source_type_neuron_id] = ACTIONS(1815),
    [sym_source_type_ip_address] = ACTIONS(1815),
    [sym_source_type_axlink] = ACTIONS(1815),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1815),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1815),
    [sym_source_type_ip_socket_address] = ACTIONS(1815),
    [sym_source_type_rs232] = ACTIONS(1815),
    [sym_source_type_internal] = ACTIONS(1815),
    [sym_source_type_mac_address] = ACTIONS(1815),
    [sym_source_type_ipv4_port] = ACTIONS(1815),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1815),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1815),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1815),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1815),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1815),
    [sym_true] = ACTIONS(1815),
    [sym_false] = ACTIONS(1815),
    [sym_file_read_only] = ACTIONS(1815),
    [sym_file_rw_new] = ACTIONS(1815),
    [sym_file_rw_append] = ACTIONS(1815),
    [sym_ip_tcp] = ACTIONS(1815),
    [sym_ip_udp] = ACTIONS(1815),
    [sym_ip_udp_2way] = ACTIONS(1815),
    [sym_xml_encode_types] = ACTIONS(1815),
    [sym_xml_encode_char_as_list] = ACTIONS(1815),
    [sym_xml_encode_le] = ACTIONS(1815),
    [sym_xml_decode_types] = ACTIONS(1815),
    [sym_xml_decode_no_preserve] = ACTIONS(1815),
    [sym_duet_dev_type_display_device] = ACTIONS(1815),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1815),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1815),
    [sym_duet_dev_type_io_device] = ACTIONS(1815),
    [sym_duet_dev_type_relay_device] = ACTIONS(1815),
    [sym_duet_dev_type_ups] = ACTIONS(1815),
    [sym_duet_dev_type_amplifier] = ACTIONS(1815),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1815),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1815),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1815),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1815),
    [sym_duet_dev_type_camera] = ACTIONS(1815),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1815),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1815),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1815),
    [sym_duet_dev_type_dss] = ACTIONS(1815),
    [sym_duet_dev_type_dvr] = ACTIONS(1815),
    [sym_duet_dev_type_disc_device] = ACTIONS(1815),
    [sym_duet_dev_type_document_camera] = ACTIONS(1815),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1815),
    [sym_duet_dev_type_hvac] = ACTIONS(1815),
    [sym_duet_dev_type_keypad] = ACTIONS(1815),
    [sym_duet_dev_type_light] = ACTIONS(1815),
    [sym_duet_dev_type_monitor] = ACTIONS(1815),
    [sym_duet_dev_type_motor] = ACTIONS(1815),
    [sym_duet_dev_type_multi_window] = ACTIONS(1815),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1815),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1815),
    [sym_duet_dev_type_receiver] = ACTIONS(1815),
    [sym_duet_dev_type_security_system] = ACTIONS(1815),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1815),
    [sym_duet_dev_type_settop_box] = ACTIONS(1815),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1815),
    [sym_duet_dev_type_switcher] = ACTIONS(1815),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1815),
    [sym_duet_dev_type_tv] = ACTIONS(1815),
    [sym_duet_dev_type_utility] = ACTIONS(1815),
    [sym_duet_dev_type_vcr] = ACTIONS(1815),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1815),
    [sym_duet_dev_type_video_processor] = ACTIONS(1815),
    [sym_duet_dev_type_video_projector] = ACTIONS(1815),
    [sym_duet_dev_type_video_wall] = ACTIONS(1815),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1815),
    [sym_duet_dev_type_weather] = ACTIONS(1815),
    [sym_duet_dev_not_polled] = ACTIONS(1815),
    [sym_duet_dev_polled] = ACTIONS(1815),
    [sym_clkmgr_mode_network] = ACTIONS(1815),
    [sym_clkmgr_mode_standalone] = ACTIONS(1815),
    [sym_normal_standby] = ACTIONS(1815),
    [sym_normal_wake] = ACTIONS(1815),
    [sym_ip_multicast_ttl_option] = ACTIONS(1815),
    [sym_ip_ttl_subnet] = ACTIONS(1815),
    [sym_ip_ttl_site] = ACTIONS(1815),
    [sym_ip_ttl_region] = ACTIONS(1815),
    [sym_ip_ttl_continent] = ACTIONS(1815),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1815),
    [sym_ip_nodelay_on] = ACTIONS(1815),
    [sym_ip_nodelay_off] = ACTIONS(1815),
    [sym_amx_error] = ACTIONS(1815),
    [sym_amx_warning] = ACTIONS(1815),
    [sym_amx_info] = ACTIONS(1815),
    [sym_amx_debug] = ACTIONS(1815),
    [sym_netlinx_login_success] = ACTIONS(1815),
    [sym_netlinx_login_fail] = ACTIONS(1815),
    [sym_netlinx_logout] = ACTIONS(1815),
    [sym_null_str] = ACTIONS(1815),
    [sym_smtp_address] = ACTIONS(1815),
    [sym_smtp_port_number] = ACTIONS(1815),
    [sym_smtp_username] = ACTIONS(1815),
    [sym_smtp_password] = ACTIONS(1815),
    [sym_smtp_require_tls] = ACTIONS(1815),
    [sym_smtp_from] = ACTIONS(1815),
    [sym_smtp_tls_true] = ACTIONS(1815),
    [sym_smtp_tls_false] = ACTIONS(1815),
    [sym_listview_on_row_select_event] = ACTIONS(1815),
    [sym_data_structure_unknown] = ACTIONS(1815),
    [sym_data_structure_datafeed] = ACTIONS(1815),
    [sym_data_structure_datarecordset] = ACTIONS(1815),
    [sym_data_structure_datarecord] = ACTIONS(1815),
    [sym_data_structure_datafield] = ACTIONS(1815),
    [sym_data_max_content_field_count] = ACTIONS(1815),
    [sym_data_max_metadata_field_count] = ACTIONS(1815),
    [sym_data_max_id_length] = ACTIONS(1815),
    [sym_data_max_name_length] = ACTIONS(1815),
    [sym_data_max_description_length] = ACTIONS(1815),
    [sym_data_max_type_length] = ACTIONS(1815),
    [sym_data_max_format_length] = ACTIONS(1815),
    [sym_data_max_source_length] = ACTIONS(1815),
    [sym_data_max_value_length] = ACTIONS(1815),
    [sym_data_max_label_length] = ACTIONS(1815),
    [sym_data_max_url_length] = ACTIONS(1815),
    [sym_max_last_login_info_length] = ACTIONS(1815),
    [sym_data_type_unknown] = ACTIONS(1815),
    [sym_data_type_string] = ACTIONS(1815),
    [sym_data_type_datetime] = ACTIONS(1815),
    [sym_data_type_date] = ACTIONS(1815),
    [sym_data_type_time] = ACTIONS(1815),
    [sym_data_type_image] = ACTIONS(1815),
    [sym_data_format_url] = ACTIONS(1815),
    [sym_data_format_phone] = ACTIONS(1815),
    [sym_data_format_email] = ACTIONS(1815),
    [sym_data_format_iso8601] = ACTIONS(1815),
    [sym_auditlog_priv] = ACTIONS(1815),
    [sym_configuration_priv] = ACTIONS(1815),
    [sym_deviceconfig_priv] = ACTIONS(1815),
    [sym_ftp_priv] = ACTIONS(1815),
    [sym_http_priv] = ACTIONS(1815),
    [sym_networkconfig_priv] = ACTIONS(1815),
    [sym_programport_priv] = ACTIONS(1815),
    [sym_remoteui_priv] = ACTIONS(1815),
    [sym_securitycontrol_priv] = ACTIONS(1815),
    [sym_softwaremanagement_priv] = ACTIONS(1815),
    [sym_terminal_priv] = ACTIONS(1815),
    [sym_tpadmin_priv] = ACTIONS(1815),
    [sym_user1_priv] = ACTIONS(1815),
    [sym_user2_priv] = ACTIONS(1815),
    [sym_user3_priv] = ACTIONS(1815),
    [sym_user4_priv] = ACTIONS(1815),
    [sym_usermanagement_priv] = ACTIONS(1815),
    [sym_valid_account] = ACTIONS(1815),
    [sym_err_name_invalid] = ACTIONS(1815),
    [sym_err_password_invalid] = ACTIONS(1815),
    [sym_err_invalid_account] = ACTIONS(1815),
    [sym_err_no_authorization] = ACTIONS(1815),
    [sym_err_invalid_parameter] = ACTIONS(1815),
    [sym_err_user_locked_out] = ACTIONS(1815),
    [sym_err_user_account_expired] = ACTIONS(1815),
    [sym_err_general_error] = ACTIONS(1815),
    [sym_err_authentication_error] = ACTIONS(1815),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1815),
    [sym_tls_validate_certificate] = ACTIONS(1815),
    [sym_tdata] = ACTIONS(1815),
    [sym_tchannel] = ACTIONS(1815),
    [sym_tlevel] = ACTIONS(1815),
    [sym_tbutton] = ACTIONS(1815),
    [sym_ttimeline] = ACTIONS(1815),
    [sym_tcustom] = ACTIONS(1815),
    [sym_url_struct] = ACTIONS(1815),
    [sym_dns_struct] = ACTIONS(1815),
    [sym_ip_address_struct] = ACTIONS(1815),
    [sym_dev_info_struct] = ACTIONS(1815),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1815),
    [sym_data_feed] = ACTIONS(1815),
    [sym_data_field] = ACTIONS(1815),
    [sym_data_record] = ACTIONS(1815),
    [sym_wc_data_feed] = ACTIONS(1815),
    [sym_wc_data_field] = ACTIONS(1815),
    [sym_wc_data_record] = ACTIONS(1815),
    [sym_last_login_info] = ACTIONS(1815),
    [sym____reserved___] = ACTIONS(1815),
    [sym_date] = ACTIONS(1815),
    [sym_day] = ACTIONS(1815),
    [sym_time] = ACTIONS(1815),
    [sym_ldate] = ACTIONS(1815),
    [sym_push_device] = ACTIONS(1815),
    [sym_push_channel] = ACTIONS(1815),
    [sym_push_devchan] = ACTIONS(1815),
    [sym_release_device] = ACTIONS(1815),
    [sym_release_channel] = ACTIONS(1815),
    [sym_release_devchan] = ACTIONS(1815),
    [sym_master_slot] = ACTIONS(1815),
    [sym_get_pulse_time] = ACTIONS(1815),
    [sym_get_timer] = ACTIONS(1815),
    [sym_master_sn] = ACTIONS(1815),
    [sym_system_number] = ACTIONS(1815),
    [sym_button] = ACTIONS(1815),
    [sym_dv_channel] = ACTIONS(1815),
    [sym_data] = ACTIONS(1815),
    [sym_level] = ACTIONS(1815),
    [sym_timeline] = ACTIONS(1815),
    [sym_channel] = ACTIONS(1815),
    [sym_custom] = ACTIONS(1815),
    [sym_atoi] = ACTIONS(1815),
    [sym_atol] = ACTIONS(1815),
    [sym_atof] = ACTIONS(1815),
    [sym_itoa] = ACTIONS(1815),
    [sym_itohex] = ACTIONS(1815),
    [sym_hextoi] = ACTIONS(1815),
    [sym_ftoa] = ACTIONS(1815),
    [sym_find_string] = ACTIONS(1815),
    [sym_left_string] = ACTIONS(1815),
    [sym_right_string] = ACTIONS(1815),
    [sym_mid_string] = ACTIONS(1815),
    [sym_remove_string] = ACTIONS(1815),
    [sym_get_buffer_string] = ACTIONS(1815),
    [sym_get_buffer_char] = ACTIONS(1815),
    [sym_get_multi_buffer_string] = ACTIONS(1815),
    [sym_length_string] = ACTIONS(1815),
    [sym_max_length_string] = ACTIONS(1815),
    [sym_set_length_string] = ACTIONS(1815),
    [sym_compare_string] = ACTIONS(1815),
    [sym_upper_string] = ACTIONS(1815),
    [sym_lower_string] = ACTIONS(1815),
    [sym_get_last] = ACTIONS(1815),
    [sym_date_to_day] = ACTIONS(1815),
    [sym_date_to_month] = ACTIONS(1815),
    [sym_date_to_year] = ACTIONS(1815),
    [sym_time_to_hour] = ACTIONS(1815),
    [sym_time_to_minute] = ACTIONS(1815),
    [sym_time_to_second] = ACTIONS(1815),
    [sym_day_of_week] = ACTIONS(1815),
    [sym_set_pulse_time] = ACTIONS(1815),
    [sym_set_timer] = ACTIONS(1815),
    [sym_file_open] = ACTIONS(1815),
    [sym_file_close] = ACTIONS(1815),
    [sym_file_read] = ACTIONS(1815),
    [sym_file_read_line] = ACTIONS(1815),
    [sym_file_write] = ACTIONS(1815),
    [sym_file_write_line] = ACTIONS(1815),
    [sym_file_copy] = ACTIONS(1815),
    [sym_file_delete] = ACTIONS(1815),
    [sym_file_rename] = ACTIONS(1815),
    [sym_file_dir] = ACTIONS(1815),
    [sym_file_getdir] = ACTIONS(1815),
    [sym_file_setdir] = ACTIONS(1815),
    [sym_file_create_dir] = ACTIONS(1815),
    [sym_file_remove_dir] = ACTIONS(1815),
    [sym_file_seek] = ACTIONS(1815),
    [sym_length_array] = ACTIONS(1815),
    [sym_max_length_array] = ACTIONS(1815),
    [sym_set_length_array] = ACTIONS(1815),
    [sym_ip_client_open] = ACTIONS(1815),
    [sym_ip_client_close] = ACTIONS(1815),
    [sym_ip_server_open] = ACTIONS(1815),
    [sym_ip_server_close] = ACTIONS(1815),
    [sym_ip_bound_client_open] = ACTIONS(1815),
    [sym_ip_mc_server_open] = ACTIONS(1815),
    [sym_ip_set_option] = ACTIONS(1815),
    [sym_ssh_client_open] = ACTIONS(1815),
    [sym_ssh_client_close] = ACTIONS(1815),
    [sym_tls_client_open] = ACTIONS(1815),
    [sym_tls_client_close] = ACTIONS(1815),
    [sym_do_push] = ACTIONS(1815),
    [sym_do_push_timed] = ACTIONS(1815),
    [sym_do_release] = ACTIONS(1815),
    [sym_device_id] = ACTIONS(1815),
    [sym_device_info] = ACTIONS(1815),
    [sym_device_id_string] = ACTIONS(1815),
    [sym_device_standby] = ACTIONS(1815),
    [sym_device_wake] = ACTIONS(1815),
    [sym_timeline_create] = ACTIONS(1815),
    [sym_timeline_kill] = ACTIONS(1815),
    [sym_timeline_reload] = ACTIONS(1815),
    [sym_timeline_pause] = ACTIONS(1815),
    [sym_timeline_restart] = ACTIONS(1815),
    [sym_timeline_set] = ACTIONS(1815),
    [sym_timeline_get] = ACTIONS(1815),
    [sym_timeline_active] = ACTIONS(1815),
    [sym_timeline_dynamic_id] = ACTIONS(1815),
    [sym_variable_to_string] = ACTIONS(1815),
    [sym_string_to_variable] = ACTIONS(1815),
    [sym_length_variable_to_string] = ACTIONS(1815),
    [sym_type_cast] = ACTIONS(1815),
    [sym_variable_to_xml] = ACTIONS(1815),
    [sym_xml_to_variable] = ACTIONS(1815),
    [sym_length_variable_to_xml] = ACTIONS(1815),
    [sym_redirect_string] = ACTIONS(1815),
    [sym_random_number] = ACTIONS(1815),
    [sym_get_unique_id] = ACTIONS(1815),
    [sym_get_system_number] = ACTIONS(1815),
    [sym_set_system_number] = ACTIONS(1815),
    [sym_get_serial_number] = ACTIONS(1815),
    [sym_get_master_build] = ACTIONS(1815),
    [sym_set_validation_code] = ACTIONS(1815),
    [sym_reboot] = ACTIONS(1815),
    [sym_load_duet_module] = ACTIONS(1815),
    [sym_unload_duet_module] = ACTIONS(1815),
    [sym_duet_mem_size_set] = ACTIONS(1815),
    [sym_duet_mem_size_get] = ACTIONS(1815),
    [sym_dynamic_application_device] = ACTIONS(1815),
    [sym_dynamic_polled_port] = ACTIONS(1815),
    [sym_static_port_binding] = ACTIONS(1815),
    [sym_static_ip_binding] = ACTIONS(1815),
    [sym_get_dns_list] = ACTIONS(1815),
    [sym_set_dns_list] = ACTIONS(1815),
    [sym_get_ip_address] = ACTIONS(1815),
    [sym_set_ip_address] = ACTIONS(1815),
    [sym_get_url_list] = ACTIONS(1815),
    [sym_add_url_entry] = ACTIONS(1815),
    [sym_delete_url_entry] = ACTIONS(1815),
    [sym_abs_value] = ACTIONS(1815),
    [sym_max_value] = ACTIONS(1815),
    [sym_min_value] = ACTIONS(1815),
    [sym_exp_value] = ACTIONS(1815),
    [sym_log_value] = ACTIONS(1815),
    [sym_log10_value] = ACTIONS(1815),
    [sym_power_value] = ACTIONS(1815),
    [sym_sqrt_value] = ACTIONS(1815),
    [sym_raw_be] = ACTIONS(1815),
    [sym_raw_le] = ACTIONS(1815),
    [sym_internal_queue_size_set] = ACTIONS(1815),
    [sym_internal_queue_size_get] = ACTIONS(1815),
    [sym_internal_threshold_set] = ACTIONS(1815),
    [sym_internal_threshold_get] = ACTIONS(1815),
    [sym_rebuilt_event] = ACTIONS(1815),
    [sym_do_custom_event] = ACTIONS(1815),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1815),
    [sym_clkmgr_set_clk_source] = ACTIONS(1815),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1815),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1815),
    [sym_clkmgr_get_timezone] = ACTIONS(1815),
    [sym_clkmgr_set_timezone] = ACTIONS(1815),
    [sym_clkmgr_get_resync_period] = ACTIONS(1815),
    [sym_clkmgr_set_resync_period] = ACTIONS(1815),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1815),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1815),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1815),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1815),
    [sym_clkmgr_get_timeservers] = ACTIONS(1815),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1815),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1815),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1815),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1815),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1815),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1815),
    [sym_set_log_level] = ACTIONS(1815),
    [sym_get_log_level] = ACTIONS(1815),
    [sym_amx_log] = ACTIONS(1815),
    [sym_smtp_server_config_set] = ACTIONS(1815),
    [sym_smtp_server_config_get] = ACTIONS(1815),
    [sym_smtp_send] = ACTIONS(1815),
    [sym_validate_netlinx_account] = ACTIONS(1815),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1815),
    [sym_authenticate_certificate] = ACTIONS(1815),
    [sym_audit_netlinx_session_event] = ACTIONS(1815),
    [sym_audit_netlinx_generic_event] = ACTIONS(1815),
    [sym_get_available_flash_disk_space] = ACTIONS(1815),
    [sym_get_max_flash_disk_space] = ACTIONS(1815),
    [sym_data_create_feed] = ACTIONS(1815),
    [sym_data_delete_feed] = ACTIONS(1815),
    [sym_data_publish_feed] = ACTIONS(1815),
    [sym_data_get_published_feed] = ACTIONS(1815),
    [sym_data_add_record] = ACTIONS(1815),
    [sym_data_get_event_record] = ACTIONS(1815),
    [sym__wc_data_create_feed] = ACTIONS(1815),
    [sym__wc_data_add_record] = ACTIONS(1815),
    [sym__wc_data_get_event_record] = ACTIONS(1815),
    [sym_format] = ACTIONS(1815),
    [sym_astro_clock] = ACTIONS(1815),
    [sym_set_outdoor_temperature] = ACTIONS(1815),
    [sym_set_virtual_level_count] = ACTIONS(1815),
    [sym_set_virtual_channel_count] = ACTIONS(1815),
    [sym_set_virtual_port_count] = ACTIONS(1815),
    [sym__wc_encode_int] = ACTIONS(1815),
    [sym__wc_decode_int] = ACTIONS(1815),
    [sym__wc_int] = ACTIONS(1815),
    [sym__wc_to_ch_int] = ACTIONS(1815),
    [sym__ch_to_wc_int] = ACTIONS(1815),
    [sym__wc_find_string_int] = ACTIONS(1815),
    [sym__wc_left_string_int] = ACTIONS(1815),
    [sym__wc_lower_string_int] = ACTIONS(1815),
    [sym__wc_mid_string_int] = ACTIONS(1815),
    [sym__wc_remove_string_int] = ACTIONS(1815),
    [sym__wc_right_string_int] = ACTIONS(1815),
    [sym__wc_upper_string_int] = ACTIONS(1815),
    [sym__wc_compare_string_int] = ACTIONS(1815),
    [sym__wc_get_buffer_char_int] = ACTIONS(1815),
    [sym__wc_get_buffer_string_int] = ACTIONS(1815),
    [sym__wc_concat_string_int] = ACTIONS(1815),
    [sym__wc_file_open_int] = ACTIONS(1815),
    [sym__wc_file_close_int] = ACTIONS(1815),
    [sym__wc_file_read_int] = ACTIONS(1815),
    [sym__wc_file_read_line_int] = ACTIONS(1815),
    [sym__wc_file_write_int] = ACTIONS(1815),
    [sym__wc_file_write_line_int] = ACTIONS(1815),
    [aux_sym_preproc_include_token1] = ACTIONS(1815),
    [aux_sym_preproc_define_token1] = ACTIONS(1817),
    [aux_sym_preproc_warn_token1] = ACTIONS(1817),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1817),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1817),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1817),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1817),
    [aux_sym_preproc_else_token1] = ACTIONS(1817),
    [anon_sym_LBRACK] = ACTIONS(1817),
    [anon_sym_LBRACE] = ACTIONS(1817),
    [anon_sym_LPAREN] = ACTIONS(1815),
    [anon_sym_BANG] = ACTIONS(1817),
    [anon_sym_TILDE] = ACTIONS(1817),
    [anon_sym_DASH] = ACTIONS(1815),
    [anon_sym_PLUS] = ACTIONS(1815),
    [anon_sym_DASH_DASH] = ACTIONS(1817),
    [anon_sym_PLUS_PLUS] = ACTIONS(1817),
    [anon_sym_DQUOTE] = ACTIONS(1817),
    [anon_sym_SQUOTE] = ACTIONS(1817),
    [sym_number_literal] = ACTIONS(1817),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(249)] = {
    [sym_identifier] = ACTIONS(1819),
    [sym_program_name_keyword] = ACTIONS(1819),
    [sym_module_name_keyword] = ACTIONS(1819),
    [sym_define_device_keyword] = ACTIONS(1819),
    [sym_define_combine_keyword] = ACTIONS(1819),
    [sym_define_constant_keyword] = ACTIONS(1819),
    [sym_define_type_keyword] = ACTIONS(1819),
    [sym_define_variable_keyword] = ACTIONS(1819),
    [sym_define_system_variable_keyword] = ACTIONS(1819),
    [sym_define_connect_level_keyword] = ACTIONS(1819),
    [sym_define_latching_keyword] = ACTIONS(1819),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1819),
    [sym_define_toggling_keyword] = ACTIONS(1819),
    [sym_define_call_keyword] = ACTIONS(1819),
    [sym_define_function_keyword] = ACTIONS(1819),
    [sym_define_library_function_keyword] = ACTIONS(1819),
    [sym_define_start_keyword] = ACTIONS(1819),
    [sym_define_event_keyword] = ACTIONS(1819),
    [sym_define_program_keyword] = ACTIONS(1819),
    [sym_define_module_keyword] = ACTIONS(1819),
    [sym_char_keyword] = ACTIONS(1819),
    [sym_widechar_keyword] = ACTIONS(1819),
    [sym_integer_keyword] = ACTIONS(1819),
    [sym_sinteger_keyword] = ACTIONS(1819),
    [sym_long_keyword] = ACTIONS(1819),
    [sym_slong_keyword] = ACTIONS(1819),
    [sym_float_keyword] = ACTIONS(1819),
    [sym_double_keyword] = ACTIONS(1819),
    [sym_variant_keyword] = ACTIONS(1819),
    [sym_variantarray_keyword] = ACTIONS(1819),
    [sym_local_var_keyword] = ACTIONS(1819),
    [sym_stack_var_keyword] = ACTIONS(1819),
    [sym_constant_keyword] = ACTIONS(1819),
    [sym_volatile_keyword] = ACTIONS(1819),
    [sym_non_volatile_keyword] = ACTIONS(1819),
    [sym_persistent_keyword] = ACTIONS(1819),
    [sym_dev_keyword] = ACTIONS(1819),
    [sym_devlev_keyword] = ACTIONS(1819),
    [sym_devchan_keyword] = ACTIONS(1819),
    [sym_if_keyword] = ACTIONS(1819),
    [sym_else_keyword] = ACTIONS(1819),
    [sym_switch_keyword] = ACTIONS(1819),
    [sym_case_keyword] = ACTIONS(1819),
    [sym_default_keyword] = ACTIONS(1819),
    [sym_while_keyword] = ACTIONS(1819),
    [sym_for_keyword] = ACTIONS(1819),
    [sym_break_keyword] = ACTIONS(1819),
    [sym_continue_keyword] = ACTIONS(1819),
    [sym_return_keyword] = ACTIONS(1819),
    [sym_select_keyword] = ACTIONS(1819),
    [sym_active_keyword] = ACTIONS(1819),
    [sym_struct_keyword] = ACTIONS(1819),
    [sym_structure_keyword] = ACTIONS(1819),
    [sym_button_event_keyword] = ACTIONS(1819),
    [sym_channel_event_keyword] = ACTIONS(1819),
    [sym_level_event_keyword] = ACTIONS(1819),
    [sym_data_event_keyword] = ACTIONS(1819),
    [sym_timeline_event_keyword] = ACTIONS(1819),
    [sym_custom_event_keyword] = ACTIONS(1819),
    [sym_push_keyword] = ACTIONS(1819),
    [sym_release_keyword] = ACTIONS(1819),
    [aux_sym_on_keyword_token1] = ACTIONS(1819),
    [aux_sym_off_keyword_token1] = ACTIONS(1819),
    [sym_send_level_keyword] = ACTIONS(1819),
    [sym_send_string_keyword] = ACTIONS(1819),
    [sym_send_command_keyword] = ACTIONS(1819),
    [sym_clear_buffer_keyword] = ACTIONS(1819),
    [sym_create_buffer_keyword] = ACTIONS(1819),
    [sym_create_multi_buffer_keyword] = ACTIONS(1819),
    [sym_call_keyword] = ACTIONS(1819),
    [sym_system_call_keyword] = ACTIONS(1819),
    [sym_devchan_to_keyword] = ACTIONS(1819),
    [sym_devchan_min_to_keyword] = ACTIONS(1819),
    [sym_devchan_total_off_keyword] = ACTIONS(1819),
    [sym_devchan_pulse_keyword] = ACTIONS(1819),
    [sym_wait_keyword] = ACTIONS(1819),
    [sym_cancel_wait_keyword] = ACTIONS(1819),
    [sym_wait_until_keyword] = ACTIONS(1819),
    [sym_cancel_wait_until_keyword] = ACTIONS(1819),
    [sym_cancel_all_wait_keyword] = ACTIONS(1819),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1819),
    [sym_not] = ACTIONS(1819),
    [sym_bnot] = ACTIONS(1819),
    [sym___file__] = ACTIONS(1819),
    [sym___date__] = ACTIONS(1819),
    [sym___ldate__] = ACTIONS(1819),
    [sym___time__] = ACTIONS(1819),
    [sym___line__] = ACTIONS(1819),
    [sym___name__] = ACTIONS(1819),
    [sym___netlinx__] = ACTIONS(1819),
    [sym_netlinx_axi_version] = ACTIONS(1819),
    [sym_url_flg_tcp] = ACTIONS(1819),
    [sym_url_flg_accinfopresent] = ACTIONS(1819),
    [sym_url_flg_temp] = ACTIONS(1819),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1819),
    [sym_url_flg_stat_mask] = ACTIONS(1819),
    [sym_url_flg_stat_lookup] = ACTIONS(1819),
    [sym_url_flg_stat_connecting] = ACTIONS(1819),
    [sym_url_flg_stat_waiting] = ACTIONS(1819),
    [sym_url_flg_stat_connected] = ACTIONS(1819),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1819),
    [sym_first_virtual_device] = ACTIONS(1819),
    [sym_first_local_port] = ACTIONS(1819),
    [sym_dynamic_virtual_device] = ACTIONS(1819),
    [sym_timeline_once] = ACTIONS(1819),
    [sym_timeline_repeat] = ACTIONS(1819),
    [sym_timeline_absolute] = ACTIONS(1819),
    [sym_timeline_relative] = ACTIONS(1819),
    [sym_do_push_timed_infinite] = ACTIONS(1819),
    [sym_source_type_no_address] = ACTIONS(1819),
    [sym_source_type_neuron_id] = ACTIONS(1819),
    [sym_source_type_ip_address] = ACTIONS(1819),
    [sym_source_type_axlink] = ACTIONS(1819),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1819),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1819),
    [sym_source_type_ip_socket_address] = ACTIONS(1819),
    [sym_source_type_rs232] = ACTIONS(1819),
    [sym_source_type_internal] = ACTIONS(1819),
    [sym_source_type_mac_address] = ACTIONS(1819),
    [sym_source_type_ipv4_port] = ACTIONS(1819),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1819),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1819),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1819),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1819),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1819),
    [sym_true] = ACTIONS(1819),
    [sym_false] = ACTIONS(1819),
    [sym_file_read_only] = ACTIONS(1819),
    [sym_file_rw_new] = ACTIONS(1819),
    [sym_file_rw_append] = ACTIONS(1819),
    [sym_ip_tcp] = ACTIONS(1819),
    [sym_ip_udp] = ACTIONS(1819),
    [sym_ip_udp_2way] = ACTIONS(1819),
    [sym_xml_encode_types] = ACTIONS(1819),
    [sym_xml_encode_char_as_list] = ACTIONS(1819),
    [sym_xml_encode_le] = ACTIONS(1819),
    [sym_xml_decode_types] = ACTIONS(1819),
    [sym_xml_decode_no_preserve] = ACTIONS(1819),
    [sym_duet_dev_type_display_device] = ACTIONS(1819),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1819),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1819),
    [sym_duet_dev_type_io_device] = ACTIONS(1819),
    [sym_duet_dev_type_relay_device] = ACTIONS(1819),
    [sym_duet_dev_type_ups] = ACTIONS(1819),
    [sym_duet_dev_type_amplifier] = ACTIONS(1819),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1819),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1819),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1819),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1819),
    [sym_duet_dev_type_camera] = ACTIONS(1819),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1819),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1819),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1819),
    [sym_duet_dev_type_dss] = ACTIONS(1819),
    [sym_duet_dev_type_dvr] = ACTIONS(1819),
    [sym_duet_dev_type_disc_device] = ACTIONS(1819),
    [sym_duet_dev_type_document_camera] = ACTIONS(1819),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1819),
    [sym_duet_dev_type_hvac] = ACTIONS(1819),
    [sym_duet_dev_type_keypad] = ACTIONS(1819),
    [sym_duet_dev_type_light] = ACTIONS(1819),
    [sym_duet_dev_type_monitor] = ACTIONS(1819),
    [sym_duet_dev_type_motor] = ACTIONS(1819),
    [sym_duet_dev_type_multi_window] = ACTIONS(1819),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1819),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1819),
    [sym_duet_dev_type_receiver] = ACTIONS(1819),
    [sym_duet_dev_type_security_system] = ACTIONS(1819),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1819),
    [sym_duet_dev_type_settop_box] = ACTIONS(1819),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1819),
    [sym_duet_dev_type_switcher] = ACTIONS(1819),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1819),
    [sym_duet_dev_type_tv] = ACTIONS(1819),
    [sym_duet_dev_type_utility] = ACTIONS(1819),
    [sym_duet_dev_type_vcr] = ACTIONS(1819),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1819),
    [sym_duet_dev_type_video_processor] = ACTIONS(1819),
    [sym_duet_dev_type_video_projector] = ACTIONS(1819),
    [sym_duet_dev_type_video_wall] = ACTIONS(1819),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1819),
    [sym_duet_dev_type_weather] = ACTIONS(1819),
    [sym_duet_dev_not_polled] = ACTIONS(1819),
    [sym_duet_dev_polled] = ACTIONS(1819),
    [sym_clkmgr_mode_network] = ACTIONS(1819),
    [sym_clkmgr_mode_standalone] = ACTIONS(1819),
    [sym_normal_standby] = ACTIONS(1819),
    [sym_normal_wake] = ACTIONS(1819),
    [sym_ip_multicast_ttl_option] = ACTIONS(1819),
    [sym_ip_ttl_subnet] = ACTIONS(1819),
    [sym_ip_ttl_site] = ACTIONS(1819),
    [sym_ip_ttl_region] = ACTIONS(1819),
    [sym_ip_ttl_continent] = ACTIONS(1819),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1819),
    [sym_ip_nodelay_on] = ACTIONS(1819),
    [sym_ip_nodelay_off] = ACTIONS(1819),
    [sym_amx_error] = ACTIONS(1819),
    [sym_amx_warning] = ACTIONS(1819),
    [sym_amx_info] = ACTIONS(1819),
    [sym_amx_debug] = ACTIONS(1819),
    [sym_netlinx_login_success] = ACTIONS(1819),
    [sym_netlinx_login_fail] = ACTIONS(1819),
    [sym_netlinx_logout] = ACTIONS(1819),
    [sym_null_str] = ACTIONS(1819),
    [sym_smtp_address] = ACTIONS(1819),
    [sym_smtp_port_number] = ACTIONS(1819),
    [sym_smtp_username] = ACTIONS(1819),
    [sym_smtp_password] = ACTIONS(1819),
    [sym_smtp_require_tls] = ACTIONS(1819),
    [sym_smtp_from] = ACTIONS(1819),
    [sym_smtp_tls_true] = ACTIONS(1819),
    [sym_smtp_tls_false] = ACTIONS(1819),
    [sym_listview_on_row_select_event] = ACTIONS(1819),
    [sym_data_structure_unknown] = ACTIONS(1819),
    [sym_data_structure_datafeed] = ACTIONS(1819),
    [sym_data_structure_datarecordset] = ACTIONS(1819),
    [sym_data_structure_datarecord] = ACTIONS(1819),
    [sym_data_structure_datafield] = ACTIONS(1819),
    [sym_data_max_content_field_count] = ACTIONS(1819),
    [sym_data_max_metadata_field_count] = ACTIONS(1819),
    [sym_data_max_id_length] = ACTIONS(1819),
    [sym_data_max_name_length] = ACTIONS(1819),
    [sym_data_max_description_length] = ACTIONS(1819),
    [sym_data_max_type_length] = ACTIONS(1819),
    [sym_data_max_format_length] = ACTIONS(1819),
    [sym_data_max_source_length] = ACTIONS(1819),
    [sym_data_max_value_length] = ACTIONS(1819),
    [sym_data_max_label_length] = ACTIONS(1819),
    [sym_data_max_url_length] = ACTIONS(1819),
    [sym_max_last_login_info_length] = ACTIONS(1819),
    [sym_data_type_unknown] = ACTIONS(1819),
    [sym_data_type_string] = ACTIONS(1819),
    [sym_data_type_datetime] = ACTIONS(1819),
    [sym_data_type_date] = ACTIONS(1819),
    [sym_data_type_time] = ACTIONS(1819),
    [sym_data_type_image] = ACTIONS(1819),
    [sym_data_format_url] = ACTIONS(1819),
    [sym_data_format_phone] = ACTIONS(1819),
    [sym_data_format_email] = ACTIONS(1819),
    [sym_data_format_iso8601] = ACTIONS(1819),
    [sym_auditlog_priv] = ACTIONS(1819),
    [sym_configuration_priv] = ACTIONS(1819),
    [sym_deviceconfig_priv] = ACTIONS(1819),
    [sym_ftp_priv] = ACTIONS(1819),
    [sym_http_priv] = ACTIONS(1819),
    [sym_networkconfig_priv] = ACTIONS(1819),
    [sym_programport_priv] = ACTIONS(1819),
    [sym_remoteui_priv] = ACTIONS(1819),
    [sym_securitycontrol_priv] = ACTIONS(1819),
    [sym_softwaremanagement_priv] = ACTIONS(1819),
    [sym_terminal_priv] = ACTIONS(1819),
    [sym_tpadmin_priv] = ACTIONS(1819),
    [sym_user1_priv] = ACTIONS(1819),
    [sym_user2_priv] = ACTIONS(1819),
    [sym_user3_priv] = ACTIONS(1819),
    [sym_user4_priv] = ACTIONS(1819),
    [sym_usermanagement_priv] = ACTIONS(1819),
    [sym_valid_account] = ACTIONS(1819),
    [sym_err_name_invalid] = ACTIONS(1819),
    [sym_err_password_invalid] = ACTIONS(1819),
    [sym_err_invalid_account] = ACTIONS(1819),
    [sym_err_no_authorization] = ACTIONS(1819),
    [sym_err_invalid_parameter] = ACTIONS(1819),
    [sym_err_user_locked_out] = ACTIONS(1819),
    [sym_err_user_account_expired] = ACTIONS(1819),
    [sym_err_general_error] = ACTIONS(1819),
    [sym_err_authentication_error] = ACTIONS(1819),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1819),
    [sym_tls_validate_certificate] = ACTIONS(1819),
    [sym_tdata] = ACTIONS(1819),
    [sym_tchannel] = ACTIONS(1819),
    [sym_tlevel] = ACTIONS(1819),
    [sym_tbutton] = ACTIONS(1819),
    [sym_ttimeline] = ACTIONS(1819),
    [sym_tcustom] = ACTIONS(1819),
    [sym_url_struct] = ACTIONS(1819),
    [sym_dns_struct] = ACTIONS(1819),
    [sym_ip_address_struct] = ACTIONS(1819),
    [sym_dev_info_struct] = ACTIONS(1819),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1819),
    [sym_data_feed] = ACTIONS(1819),
    [sym_data_field] = ACTIONS(1819),
    [sym_data_record] = ACTIONS(1819),
    [sym_wc_data_feed] = ACTIONS(1819),
    [sym_wc_data_field] = ACTIONS(1819),
    [sym_wc_data_record] = ACTIONS(1819),
    [sym_last_login_info] = ACTIONS(1819),
    [sym____reserved___] = ACTIONS(1819),
    [sym_date] = ACTIONS(1819),
    [sym_day] = ACTIONS(1819),
    [sym_time] = ACTIONS(1819),
    [sym_ldate] = ACTIONS(1819),
    [sym_push_device] = ACTIONS(1819),
    [sym_push_channel] = ACTIONS(1819),
    [sym_push_devchan] = ACTIONS(1819),
    [sym_release_device] = ACTIONS(1819),
    [sym_release_channel] = ACTIONS(1819),
    [sym_release_devchan] = ACTIONS(1819),
    [sym_master_slot] = ACTIONS(1819),
    [sym_get_pulse_time] = ACTIONS(1819),
    [sym_get_timer] = ACTIONS(1819),
    [sym_master_sn] = ACTIONS(1819),
    [sym_system_number] = ACTIONS(1819),
    [sym_button] = ACTIONS(1819),
    [sym_dv_channel] = ACTIONS(1819),
    [sym_data] = ACTIONS(1819),
    [sym_level] = ACTIONS(1819),
    [sym_timeline] = ACTIONS(1819),
    [sym_channel] = ACTIONS(1819),
    [sym_custom] = ACTIONS(1819),
    [sym_atoi] = ACTIONS(1819),
    [sym_atol] = ACTIONS(1819),
    [sym_atof] = ACTIONS(1819),
    [sym_itoa] = ACTIONS(1819),
    [sym_itohex] = ACTIONS(1819),
    [sym_hextoi] = ACTIONS(1819),
    [sym_ftoa] = ACTIONS(1819),
    [sym_find_string] = ACTIONS(1819),
    [sym_left_string] = ACTIONS(1819),
    [sym_right_string] = ACTIONS(1819),
    [sym_mid_string] = ACTIONS(1819),
    [sym_remove_string] = ACTIONS(1819),
    [sym_get_buffer_string] = ACTIONS(1819),
    [sym_get_buffer_char] = ACTIONS(1819),
    [sym_get_multi_buffer_string] = ACTIONS(1819),
    [sym_length_string] = ACTIONS(1819),
    [sym_max_length_string] = ACTIONS(1819),
    [sym_set_length_string] = ACTIONS(1819),
    [sym_compare_string] = ACTIONS(1819),
    [sym_upper_string] = ACTIONS(1819),
    [sym_lower_string] = ACTIONS(1819),
    [sym_get_last] = ACTIONS(1819),
    [sym_date_to_day] = ACTIONS(1819),
    [sym_date_to_month] = ACTIONS(1819),
    [sym_date_to_year] = ACTIONS(1819),
    [sym_time_to_hour] = ACTIONS(1819),
    [sym_time_to_minute] = ACTIONS(1819),
    [sym_time_to_second] = ACTIONS(1819),
    [sym_day_of_week] = ACTIONS(1819),
    [sym_set_pulse_time] = ACTIONS(1819),
    [sym_set_timer] = ACTIONS(1819),
    [sym_file_open] = ACTIONS(1819),
    [sym_file_close] = ACTIONS(1819),
    [sym_file_read] = ACTIONS(1819),
    [sym_file_read_line] = ACTIONS(1819),
    [sym_file_write] = ACTIONS(1819),
    [sym_file_write_line] = ACTIONS(1819),
    [sym_file_copy] = ACTIONS(1819),
    [sym_file_delete] = ACTIONS(1819),
    [sym_file_rename] = ACTIONS(1819),
    [sym_file_dir] = ACTIONS(1819),
    [sym_file_getdir] = ACTIONS(1819),
    [sym_file_setdir] = ACTIONS(1819),
    [sym_file_create_dir] = ACTIONS(1819),
    [sym_file_remove_dir] = ACTIONS(1819),
    [sym_file_seek] = ACTIONS(1819),
    [sym_length_array] = ACTIONS(1819),
    [sym_max_length_array] = ACTIONS(1819),
    [sym_set_length_array] = ACTIONS(1819),
    [sym_ip_client_open] = ACTIONS(1819),
    [sym_ip_client_close] = ACTIONS(1819),
    [sym_ip_server_open] = ACTIONS(1819),
    [sym_ip_server_close] = ACTIONS(1819),
    [sym_ip_bound_client_open] = ACTIONS(1819),
    [sym_ip_mc_server_open] = ACTIONS(1819),
    [sym_ip_set_option] = ACTIONS(1819),
    [sym_ssh_client_open] = ACTIONS(1819),
    [sym_ssh_client_close] = ACTIONS(1819),
    [sym_tls_client_open] = ACTIONS(1819),
    [sym_tls_client_close] = ACTIONS(1819),
    [sym_do_push] = ACTIONS(1819),
    [sym_do_push_timed] = ACTIONS(1819),
    [sym_do_release] = ACTIONS(1819),
    [sym_device_id] = ACTIONS(1819),
    [sym_device_info] = ACTIONS(1819),
    [sym_device_id_string] = ACTIONS(1819),
    [sym_device_standby] = ACTIONS(1819),
    [sym_device_wake] = ACTIONS(1819),
    [sym_timeline_create] = ACTIONS(1819),
    [sym_timeline_kill] = ACTIONS(1819),
    [sym_timeline_reload] = ACTIONS(1819),
    [sym_timeline_pause] = ACTIONS(1819),
    [sym_timeline_restart] = ACTIONS(1819),
    [sym_timeline_set] = ACTIONS(1819),
    [sym_timeline_get] = ACTIONS(1819),
    [sym_timeline_active] = ACTIONS(1819),
    [sym_timeline_dynamic_id] = ACTIONS(1819),
    [sym_variable_to_string] = ACTIONS(1819),
    [sym_string_to_variable] = ACTIONS(1819),
    [sym_length_variable_to_string] = ACTIONS(1819),
    [sym_type_cast] = ACTIONS(1819),
    [sym_variable_to_xml] = ACTIONS(1819),
    [sym_xml_to_variable] = ACTIONS(1819),
    [sym_length_variable_to_xml] = ACTIONS(1819),
    [sym_redirect_string] = ACTIONS(1819),
    [sym_random_number] = ACTIONS(1819),
    [sym_get_unique_id] = ACTIONS(1819),
    [sym_get_system_number] = ACTIONS(1819),
    [sym_set_system_number] = ACTIONS(1819),
    [sym_get_serial_number] = ACTIONS(1819),
    [sym_get_master_build] = ACTIONS(1819),
    [sym_set_validation_code] = ACTIONS(1819),
    [sym_reboot] = ACTIONS(1819),
    [sym_load_duet_module] = ACTIONS(1819),
    [sym_unload_duet_module] = ACTIONS(1819),
    [sym_duet_mem_size_set] = ACTIONS(1819),
    [sym_duet_mem_size_get] = ACTIONS(1819),
    [sym_dynamic_application_device] = ACTIONS(1819),
    [sym_dynamic_polled_port] = ACTIONS(1819),
    [sym_static_port_binding] = ACTIONS(1819),
    [sym_static_ip_binding] = ACTIONS(1819),
    [sym_get_dns_list] = ACTIONS(1819),
    [sym_set_dns_list] = ACTIONS(1819),
    [sym_get_ip_address] = ACTIONS(1819),
    [sym_set_ip_address] = ACTIONS(1819),
    [sym_get_url_list] = ACTIONS(1819),
    [sym_add_url_entry] = ACTIONS(1819),
    [sym_delete_url_entry] = ACTIONS(1819),
    [sym_abs_value] = ACTIONS(1819),
    [sym_max_value] = ACTIONS(1819),
    [sym_min_value] = ACTIONS(1819),
    [sym_exp_value] = ACTIONS(1819),
    [sym_log_value] = ACTIONS(1819),
    [sym_log10_value] = ACTIONS(1819),
    [sym_power_value] = ACTIONS(1819),
    [sym_sqrt_value] = ACTIONS(1819),
    [sym_raw_be] = ACTIONS(1819),
    [sym_raw_le] = ACTIONS(1819),
    [sym_internal_queue_size_set] = ACTIONS(1819),
    [sym_internal_queue_size_get] = ACTIONS(1819),
    [sym_internal_threshold_set] = ACTIONS(1819),
    [sym_internal_threshold_get] = ACTIONS(1819),
    [sym_rebuilt_event] = ACTIONS(1819),
    [sym_do_custom_event] = ACTIONS(1819),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1819),
    [sym_clkmgr_set_clk_source] = ACTIONS(1819),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1819),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1819),
    [sym_clkmgr_get_timezone] = ACTIONS(1819),
    [sym_clkmgr_set_timezone] = ACTIONS(1819),
    [sym_clkmgr_get_resync_period] = ACTIONS(1819),
    [sym_clkmgr_set_resync_period] = ACTIONS(1819),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1819),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1819),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1819),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1819),
    [sym_clkmgr_get_timeservers] = ACTIONS(1819),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1819),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1819),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1819),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1819),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1819),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1819),
    [sym_set_log_level] = ACTIONS(1819),
    [sym_get_log_level] = ACTIONS(1819),
    [sym_amx_log] = ACTIONS(1819),
    [sym_smtp_server_config_set] = ACTIONS(1819),
    [sym_smtp_server_config_get] = ACTIONS(1819),
    [sym_smtp_send] = ACTIONS(1819),
    [sym_validate_netlinx_account] = ACTIONS(1819),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1819),
    [sym_authenticate_certificate] = ACTIONS(1819),
    [sym_audit_netlinx_session_event] = ACTIONS(1819),
    [sym_audit_netlinx_generic_event] = ACTIONS(1819),
    [sym_get_available_flash_disk_space] = ACTIONS(1819),
    [sym_get_max_flash_disk_space] = ACTIONS(1819),
    [sym_data_create_feed] = ACTIONS(1819),
    [sym_data_delete_feed] = ACTIONS(1819),
    [sym_data_publish_feed] = ACTIONS(1819),
    [sym_data_get_published_feed] = ACTIONS(1819),
    [sym_data_add_record] = ACTIONS(1819),
    [sym_data_get_event_record] = ACTIONS(1819),
    [sym__wc_data_create_feed] = ACTIONS(1819),
    [sym__wc_data_add_record] = ACTIONS(1819),
    [sym__wc_data_get_event_record] = ACTIONS(1819),
    [sym_format] = ACTIONS(1819),
    [sym_astro_clock] = ACTIONS(1819),
    [sym_set_outdoor_temperature] = ACTIONS(1819),
    [sym_set_virtual_level_count] = ACTIONS(1819),
    [sym_set_virtual_channel_count] = ACTIONS(1819),
    [sym_set_virtual_port_count] = ACTIONS(1819),
    [sym__wc_encode_int] = ACTIONS(1819),
    [sym__wc_decode_int] = ACTIONS(1819),
    [sym__wc_int] = ACTIONS(1819),
    [sym__wc_to_ch_int] = ACTIONS(1819),
    [sym__ch_to_wc_int] = ACTIONS(1819),
    [sym__wc_find_string_int] = ACTIONS(1819),
    [sym__wc_left_string_int] = ACTIONS(1819),
    [sym__wc_lower_string_int] = ACTIONS(1819),
    [sym__wc_mid_string_int] = ACTIONS(1819),
    [sym__wc_remove_string_int] = ACTIONS(1819),
    [sym__wc_right_string_int] = ACTIONS(1819),
    [sym__wc_upper_string_int] = ACTIONS(1819),
    [sym__wc_compare_string_int] = ACTIONS(1819),
    [sym__wc_get_buffer_char_int] = ACTIONS(1819),
    [sym__wc_get_buffer_string_int] = ACTIONS(1819),
    [sym__wc_concat_string_int] = ACTIONS(1819),
    [sym__wc_file_open_int] = ACTIONS(1819),
    [sym__wc_file_close_int] = ACTIONS(1819),
    [sym__wc_file_read_int] = ACTIONS(1819),
    [sym__wc_file_read_line_int] = ACTIONS(1819),
    [sym__wc_file_write_int] = ACTIONS(1819),
    [sym__wc_file_write_line_int] = ACTIONS(1819),
    [aux_sym_preproc_include_token1] = ACTIONS(1819),
    [aux_sym_preproc_define_token1] = ACTIONS(1821),
    [aux_sym_preproc_warn_token1] = ACTIONS(1821),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1821),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1821),
    [aux_sym_preproc_else_token1] = ACTIONS(1821),
    [anon_sym_LBRACK] = ACTIONS(1821),
    [anon_sym_LBRACE] = ACTIONS(1821),
    [anon_sym_LPAREN] = ACTIONS(1819),
    [anon_sym_BANG] = ACTIONS(1821),
    [anon_sym_TILDE] = ACTIONS(1821),
    [anon_sym_DASH] = ACTIONS(1819),
    [anon_sym_PLUS] = ACTIONS(1819),
    [anon_sym_DASH_DASH] = ACTIONS(1821),
    [anon_sym_PLUS_PLUS] = ACTIONS(1821),
    [anon_sym_DQUOTE] = ACTIONS(1821),
    [anon_sym_SQUOTE] = ACTIONS(1821),
    [sym_number_literal] = ACTIONS(1821),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(250)] = {
    [sym_identifier] = ACTIONS(1823),
    [sym_program_name_keyword] = ACTIONS(1823),
    [sym_module_name_keyword] = ACTIONS(1823),
    [sym_define_device_keyword] = ACTIONS(1823),
    [sym_define_combine_keyword] = ACTIONS(1823),
    [sym_define_constant_keyword] = ACTIONS(1823),
    [sym_define_type_keyword] = ACTIONS(1823),
    [sym_define_variable_keyword] = ACTIONS(1823),
    [sym_define_system_variable_keyword] = ACTIONS(1823),
    [sym_define_connect_level_keyword] = ACTIONS(1823),
    [sym_define_latching_keyword] = ACTIONS(1823),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1823),
    [sym_define_toggling_keyword] = ACTIONS(1823),
    [sym_define_call_keyword] = ACTIONS(1823),
    [sym_define_function_keyword] = ACTIONS(1823),
    [sym_define_library_function_keyword] = ACTIONS(1823),
    [sym_define_start_keyword] = ACTIONS(1823),
    [sym_define_event_keyword] = ACTIONS(1823),
    [sym_define_program_keyword] = ACTIONS(1823),
    [sym_define_module_keyword] = ACTIONS(1823),
    [sym_char_keyword] = ACTIONS(1823),
    [sym_widechar_keyword] = ACTIONS(1823),
    [sym_integer_keyword] = ACTIONS(1823),
    [sym_sinteger_keyword] = ACTIONS(1823),
    [sym_long_keyword] = ACTIONS(1823),
    [sym_slong_keyword] = ACTIONS(1823),
    [sym_float_keyword] = ACTIONS(1823),
    [sym_double_keyword] = ACTIONS(1823),
    [sym_variant_keyword] = ACTIONS(1823),
    [sym_variantarray_keyword] = ACTIONS(1823),
    [sym_local_var_keyword] = ACTIONS(1823),
    [sym_stack_var_keyword] = ACTIONS(1823),
    [sym_constant_keyword] = ACTIONS(1823),
    [sym_volatile_keyword] = ACTIONS(1823),
    [sym_non_volatile_keyword] = ACTIONS(1823),
    [sym_persistent_keyword] = ACTIONS(1823),
    [sym_dev_keyword] = ACTIONS(1823),
    [sym_devlev_keyword] = ACTIONS(1823),
    [sym_devchan_keyword] = ACTIONS(1823),
    [sym_if_keyword] = ACTIONS(1823),
    [sym_else_keyword] = ACTIONS(1823),
    [sym_switch_keyword] = ACTIONS(1823),
    [sym_case_keyword] = ACTIONS(1823),
    [sym_default_keyword] = ACTIONS(1823),
    [sym_while_keyword] = ACTIONS(1823),
    [sym_for_keyword] = ACTIONS(1823),
    [sym_break_keyword] = ACTIONS(1823),
    [sym_continue_keyword] = ACTIONS(1823),
    [sym_return_keyword] = ACTIONS(1823),
    [sym_select_keyword] = ACTIONS(1823),
    [sym_active_keyword] = ACTIONS(1823),
    [sym_struct_keyword] = ACTIONS(1823),
    [sym_structure_keyword] = ACTIONS(1823),
    [sym_button_event_keyword] = ACTIONS(1823),
    [sym_channel_event_keyword] = ACTIONS(1823),
    [sym_level_event_keyword] = ACTIONS(1823),
    [sym_data_event_keyword] = ACTIONS(1823),
    [sym_timeline_event_keyword] = ACTIONS(1823),
    [sym_custom_event_keyword] = ACTIONS(1823),
    [sym_push_keyword] = ACTIONS(1823),
    [sym_release_keyword] = ACTIONS(1823),
    [aux_sym_on_keyword_token1] = ACTIONS(1823),
    [aux_sym_off_keyword_token1] = ACTIONS(1823),
    [sym_send_level_keyword] = ACTIONS(1823),
    [sym_send_string_keyword] = ACTIONS(1823),
    [sym_send_command_keyword] = ACTIONS(1823),
    [sym_clear_buffer_keyword] = ACTIONS(1823),
    [sym_create_buffer_keyword] = ACTIONS(1823),
    [sym_create_multi_buffer_keyword] = ACTIONS(1823),
    [sym_call_keyword] = ACTIONS(1823),
    [sym_system_call_keyword] = ACTIONS(1823),
    [sym_devchan_to_keyword] = ACTIONS(1823),
    [sym_devchan_min_to_keyword] = ACTIONS(1823),
    [sym_devchan_total_off_keyword] = ACTIONS(1823),
    [sym_devchan_pulse_keyword] = ACTIONS(1823),
    [sym_wait_keyword] = ACTIONS(1823),
    [sym_cancel_wait_keyword] = ACTIONS(1823),
    [sym_wait_until_keyword] = ACTIONS(1823),
    [sym_cancel_wait_until_keyword] = ACTIONS(1823),
    [sym_cancel_all_wait_keyword] = ACTIONS(1823),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1823),
    [sym_not] = ACTIONS(1823),
    [sym_bnot] = ACTIONS(1823),
    [sym___file__] = ACTIONS(1823),
    [sym___date__] = ACTIONS(1823),
    [sym___ldate__] = ACTIONS(1823),
    [sym___time__] = ACTIONS(1823),
    [sym___line__] = ACTIONS(1823),
    [sym___name__] = ACTIONS(1823),
    [sym___netlinx__] = ACTIONS(1823),
    [sym_netlinx_axi_version] = ACTIONS(1823),
    [sym_url_flg_tcp] = ACTIONS(1823),
    [sym_url_flg_accinfopresent] = ACTIONS(1823),
    [sym_url_flg_temp] = ACTIONS(1823),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1823),
    [sym_url_flg_stat_mask] = ACTIONS(1823),
    [sym_url_flg_stat_lookup] = ACTIONS(1823),
    [sym_url_flg_stat_connecting] = ACTIONS(1823),
    [sym_url_flg_stat_waiting] = ACTIONS(1823),
    [sym_url_flg_stat_connected] = ACTIONS(1823),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1823),
    [sym_first_virtual_device] = ACTIONS(1823),
    [sym_first_local_port] = ACTIONS(1823),
    [sym_dynamic_virtual_device] = ACTIONS(1823),
    [sym_timeline_once] = ACTIONS(1823),
    [sym_timeline_repeat] = ACTIONS(1823),
    [sym_timeline_absolute] = ACTIONS(1823),
    [sym_timeline_relative] = ACTIONS(1823),
    [sym_do_push_timed_infinite] = ACTIONS(1823),
    [sym_source_type_no_address] = ACTIONS(1823),
    [sym_source_type_neuron_id] = ACTIONS(1823),
    [sym_source_type_ip_address] = ACTIONS(1823),
    [sym_source_type_axlink] = ACTIONS(1823),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1823),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1823),
    [sym_source_type_ip_socket_address] = ACTIONS(1823),
    [sym_source_type_rs232] = ACTIONS(1823),
    [sym_source_type_internal] = ACTIONS(1823),
    [sym_source_type_mac_address] = ACTIONS(1823),
    [sym_source_type_ipv4_port] = ACTIONS(1823),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1823),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1823),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1823),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1823),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1823),
    [sym_true] = ACTIONS(1823),
    [sym_false] = ACTIONS(1823),
    [sym_file_read_only] = ACTIONS(1823),
    [sym_file_rw_new] = ACTIONS(1823),
    [sym_file_rw_append] = ACTIONS(1823),
    [sym_ip_tcp] = ACTIONS(1823),
    [sym_ip_udp] = ACTIONS(1823),
    [sym_ip_udp_2way] = ACTIONS(1823),
    [sym_xml_encode_types] = ACTIONS(1823),
    [sym_xml_encode_char_as_list] = ACTIONS(1823),
    [sym_xml_encode_le] = ACTIONS(1823),
    [sym_xml_decode_types] = ACTIONS(1823),
    [sym_xml_decode_no_preserve] = ACTIONS(1823),
    [sym_duet_dev_type_display_device] = ACTIONS(1823),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1823),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1823),
    [sym_duet_dev_type_io_device] = ACTIONS(1823),
    [sym_duet_dev_type_relay_device] = ACTIONS(1823),
    [sym_duet_dev_type_ups] = ACTIONS(1823),
    [sym_duet_dev_type_amplifier] = ACTIONS(1823),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1823),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1823),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1823),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1823),
    [sym_duet_dev_type_camera] = ACTIONS(1823),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1823),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1823),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1823),
    [sym_duet_dev_type_dss] = ACTIONS(1823),
    [sym_duet_dev_type_dvr] = ACTIONS(1823),
    [sym_duet_dev_type_disc_device] = ACTIONS(1823),
    [sym_duet_dev_type_document_camera] = ACTIONS(1823),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1823),
    [sym_duet_dev_type_hvac] = ACTIONS(1823),
    [sym_duet_dev_type_keypad] = ACTIONS(1823),
    [sym_duet_dev_type_light] = ACTIONS(1823),
    [sym_duet_dev_type_monitor] = ACTIONS(1823),
    [sym_duet_dev_type_motor] = ACTIONS(1823),
    [sym_duet_dev_type_multi_window] = ACTIONS(1823),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1823),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1823),
    [sym_duet_dev_type_receiver] = ACTIONS(1823),
    [sym_duet_dev_type_security_system] = ACTIONS(1823),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1823),
    [sym_duet_dev_type_settop_box] = ACTIONS(1823),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1823),
    [sym_duet_dev_type_switcher] = ACTIONS(1823),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1823),
    [sym_duet_dev_type_tv] = ACTIONS(1823),
    [sym_duet_dev_type_utility] = ACTIONS(1823),
    [sym_duet_dev_type_vcr] = ACTIONS(1823),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1823),
    [sym_duet_dev_type_video_processor] = ACTIONS(1823),
    [sym_duet_dev_type_video_projector] = ACTIONS(1823),
    [sym_duet_dev_type_video_wall] = ACTIONS(1823),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1823),
    [sym_duet_dev_type_weather] = ACTIONS(1823),
    [sym_duet_dev_not_polled] = ACTIONS(1823),
    [sym_duet_dev_polled] = ACTIONS(1823),
    [sym_clkmgr_mode_network] = ACTIONS(1823),
    [sym_clkmgr_mode_standalone] = ACTIONS(1823),
    [sym_normal_standby] = ACTIONS(1823),
    [sym_normal_wake] = ACTIONS(1823),
    [sym_ip_multicast_ttl_option] = ACTIONS(1823),
    [sym_ip_ttl_subnet] = ACTIONS(1823),
    [sym_ip_ttl_site] = ACTIONS(1823),
    [sym_ip_ttl_region] = ACTIONS(1823),
    [sym_ip_ttl_continent] = ACTIONS(1823),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1823),
    [sym_ip_nodelay_on] = ACTIONS(1823),
    [sym_ip_nodelay_off] = ACTIONS(1823),
    [sym_amx_error] = ACTIONS(1823),
    [sym_amx_warning] = ACTIONS(1823),
    [sym_amx_info] = ACTIONS(1823),
    [sym_amx_debug] = ACTIONS(1823),
    [sym_netlinx_login_success] = ACTIONS(1823),
    [sym_netlinx_login_fail] = ACTIONS(1823),
    [sym_netlinx_logout] = ACTIONS(1823),
    [sym_null_str] = ACTIONS(1823),
    [sym_smtp_address] = ACTIONS(1823),
    [sym_smtp_port_number] = ACTIONS(1823),
    [sym_smtp_username] = ACTIONS(1823),
    [sym_smtp_password] = ACTIONS(1823),
    [sym_smtp_require_tls] = ACTIONS(1823),
    [sym_smtp_from] = ACTIONS(1823),
    [sym_smtp_tls_true] = ACTIONS(1823),
    [sym_smtp_tls_false] = ACTIONS(1823),
    [sym_listview_on_row_select_event] = ACTIONS(1823),
    [sym_data_structure_unknown] = ACTIONS(1823),
    [sym_data_structure_datafeed] = ACTIONS(1823),
    [sym_data_structure_datarecordset] = ACTIONS(1823),
    [sym_data_structure_datarecord] = ACTIONS(1823),
    [sym_data_structure_datafield] = ACTIONS(1823),
    [sym_data_max_content_field_count] = ACTIONS(1823),
    [sym_data_max_metadata_field_count] = ACTIONS(1823),
    [sym_data_max_id_length] = ACTIONS(1823),
    [sym_data_max_name_length] = ACTIONS(1823),
    [sym_data_max_description_length] = ACTIONS(1823),
    [sym_data_max_type_length] = ACTIONS(1823),
    [sym_data_max_format_length] = ACTIONS(1823),
    [sym_data_max_source_length] = ACTIONS(1823),
    [sym_data_max_value_length] = ACTIONS(1823),
    [sym_data_max_label_length] = ACTIONS(1823),
    [sym_data_max_url_length] = ACTIONS(1823),
    [sym_max_last_login_info_length] = ACTIONS(1823),
    [sym_data_type_unknown] = ACTIONS(1823),
    [sym_data_type_string] = ACTIONS(1823),
    [sym_data_type_datetime] = ACTIONS(1823),
    [sym_data_type_date] = ACTIONS(1823),
    [sym_data_type_time] = ACTIONS(1823),
    [sym_data_type_image] = ACTIONS(1823),
    [sym_data_format_url] = ACTIONS(1823),
    [sym_data_format_phone] = ACTIONS(1823),
    [sym_data_format_email] = ACTIONS(1823),
    [sym_data_format_iso8601] = ACTIONS(1823),
    [sym_auditlog_priv] = ACTIONS(1823),
    [sym_configuration_priv] = ACTIONS(1823),
    [sym_deviceconfig_priv] = ACTIONS(1823),
    [sym_ftp_priv] = ACTIONS(1823),
    [sym_http_priv] = ACTIONS(1823),
    [sym_networkconfig_priv] = ACTIONS(1823),
    [sym_programport_priv] = ACTIONS(1823),
    [sym_remoteui_priv] = ACTIONS(1823),
    [sym_securitycontrol_priv] = ACTIONS(1823),
    [sym_softwaremanagement_priv] = ACTIONS(1823),
    [sym_terminal_priv] = ACTIONS(1823),
    [sym_tpadmin_priv] = ACTIONS(1823),
    [sym_user1_priv] = ACTIONS(1823),
    [sym_user2_priv] = ACTIONS(1823),
    [sym_user3_priv] = ACTIONS(1823),
    [sym_user4_priv] = ACTIONS(1823),
    [sym_usermanagement_priv] = ACTIONS(1823),
    [sym_valid_account] = ACTIONS(1823),
    [sym_err_name_invalid] = ACTIONS(1823),
    [sym_err_password_invalid] = ACTIONS(1823),
    [sym_err_invalid_account] = ACTIONS(1823),
    [sym_err_no_authorization] = ACTIONS(1823),
    [sym_err_invalid_parameter] = ACTIONS(1823),
    [sym_err_user_locked_out] = ACTIONS(1823),
    [sym_err_user_account_expired] = ACTIONS(1823),
    [sym_err_general_error] = ACTIONS(1823),
    [sym_err_authentication_error] = ACTIONS(1823),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1823),
    [sym_tls_validate_certificate] = ACTIONS(1823),
    [sym_tdata] = ACTIONS(1823),
    [sym_tchannel] = ACTIONS(1823),
    [sym_tlevel] = ACTIONS(1823),
    [sym_tbutton] = ACTIONS(1823),
    [sym_ttimeline] = ACTIONS(1823),
    [sym_tcustom] = ACTIONS(1823),
    [sym_url_struct] = ACTIONS(1823),
    [sym_dns_struct] = ACTIONS(1823),
    [sym_ip_address_struct] = ACTIONS(1823),
    [sym_dev_info_struct] = ACTIONS(1823),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1823),
    [sym_data_feed] = ACTIONS(1823),
    [sym_data_field] = ACTIONS(1823),
    [sym_data_record] = ACTIONS(1823),
    [sym_wc_data_feed] = ACTIONS(1823),
    [sym_wc_data_field] = ACTIONS(1823),
    [sym_wc_data_record] = ACTIONS(1823),
    [sym_last_login_info] = ACTIONS(1823),
    [sym____reserved___] = ACTIONS(1823),
    [sym_date] = ACTIONS(1823),
    [sym_day] = ACTIONS(1823),
    [sym_time] = ACTIONS(1823),
    [sym_ldate] = ACTIONS(1823),
    [sym_push_device] = ACTIONS(1823),
    [sym_push_channel] = ACTIONS(1823),
    [sym_push_devchan] = ACTIONS(1823),
    [sym_release_device] = ACTIONS(1823),
    [sym_release_channel] = ACTIONS(1823),
    [sym_release_devchan] = ACTIONS(1823),
    [sym_master_slot] = ACTIONS(1823),
    [sym_get_pulse_time] = ACTIONS(1823),
    [sym_get_timer] = ACTIONS(1823),
    [sym_master_sn] = ACTIONS(1823),
    [sym_system_number] = ACTIONS(1823),
    [sym_button] = ACTIONS(1823),
    [sym_dv_channel] = ACTIONS(1823),
    [sym_data] = ACTIONS(1823),
    [sym_level] = ACTIONS(1823),
    [sym_timeline] = ACTIONS(1823),
    [sym_channel] = ACTIONS(1823),
    [sym_custom] = ACTIONS(1823),
    [sym_atoi] = ACTIONS(1823),
    [sym_atol] = ACTIONS(1823),
    [sym_atof] = ACTIONS(1823),
    [sym_itoa] = ACTIONS(1823),
    [sym_itohex] = ACTIONS(1823),
    [sym_hextoi] = ACTIONS(1823),
    [sym_ftoa] = ACTIONS(1823),
    [sym_find_string] = ACTIONS(1823),
    [sym_left_string] = ACTIONS(1823),
    [sym_right_string] = ACTIONS(1823),
    [sym_mid_string] = ACTIONS(1823),
    [sym_remove_string] = ACTIONS(1823),
    [sym_get_buffer_string] = ACTIONS(1823),
    [sym_get_buffer_char] = ACTIONS(1823),
    [sym_get_multi_buffer_string] = ACTIONS(1823),
    [sym_length_string] = ACTIONS(1823),
    [sym_max_length_string] = ACTIONS(1823),
    [sym_set_length_string] = ACTIONS(1823),
    [sym_compare_string] = ACTIONS(1823),
    [sym_upper_string] = ACTIONS(1823),
    [sym_lower_string] = ACTIONS(1823),
    [sym_get_last] = ACTIONS(1823),
    [sym_date_to_day] = ACTIONS(1823),
    [sym_date_to_month] = ACTIONS(1823),
    [sym_date_to_year] = ACTIONS(1823),
    [sym_time_to_hour] = ACTIONS(1823),
    [sym_time_to_minute] = ACTIONS(1823),
    [sym_time_to_second] = ACTIONS(1823),
    [sym_day_of_week] = ACTIONS(1823),
    [sym_set_pulse_time] = ACTIONS(1823),
    [sym_set_timer] = ACTIONS(1823),
    [sym_file_open] = ACTIONS(1823),
    [sym_file_close] = ACTIONS(1823),
    [sym_file_read] = ACTIONS(1823),
    [sym_file_read_line] = ACTIONS(1823),
    [sym_file_write] = ACTIONS(1823),
    [sym_file_write_line] = ACTIONS(1823),
    [sym_file_copy] = ACTIONS(1823),
    [sym_file_delete] = ACTIONS(1823),
    [sym_file_rename] = ACTIONS(1823),
    [sym_file_dir] = ACTIONS(1823),
    [sym_file_getdir] = ACTIONS(1823),
    [sym_file_setdir] = ACTIONS(1823),
    [sym_file_create_dir] = ACTIONS(1823),
    [sym_file_remove_dir] = ACTIONS(1823),
    [sym_file_seek] = ACTIONS(1823),
    [sym_length_array] = ACTIONS(1823),
    [sym_max_length_array] = ACTIONS(1823),
    [sym_set_length_array] = ACTIONS(1823),
    [sym_ip_client_open] = ACTIONS(1823),
    [sym_ip_client_close] = ACTIONS(1823),
    [sym_ip_server_open] = ACTIONS(1823),
    [sym_ip_server_close] = ACTIONS(1823),
    [sym_ip_bound_client_open] = ACTIONS(1823),
    [sym_ip_mc_server_open] = ACTIONS(1823),
    [sym_ip_set_option] = ACTIONS(1823),
    [sym_ssh_client_open] = ACTIONS(1823),
    [sym_ssh_client_close] = ACTIONS(1823),
    [sym_tls_client_open] = ACTIONS(1823),
    [sym_tls_client_close] = ACTIONS(1823),
    [sym_do_push] = ACTIONS(1823),
    [sym_do_push_timed] = ACTIONS(1823),
    [sym_do_release] = ACTIONS(1823),
    [sym_device_id] = ACTIONS(1823),
    [sym_device_info] = ACTIONS(1823),
    [sym_device_id_string] = ACTIONS(1823),
    [sym_device_standby] = ACTIONS(1823),
    [sym_device_wake] = ACTIONS(1823),
    [sym_timeline_create] = ACTIONS(1823),
    [sym_timeline_kill] = ACTIONS(1823),
    [sym_timeline_reload] = ACTIONS(1823),
    [sym_timeline_pause] = ACTIONS(1823),
    [sym_timeline_restart] = ACTIONS(1823),
    [sym_timeline_set] = ACTIONS(1823),
    [sym_timeline_get] = ACTIONS(1823),
    [sym_timeline_active] = ACTIONS(1823),
    [sym_timeline_dynamic_id] = ACTIONS(1823),
    [sym_variable_to_string] = ACTIONS(1823),
    [sym_string_to_variable] = ACTIONS(1823),
    [sym_length_variable_to_string] = ACTIONS(1823),
    [sym_type_cast] = ACTIONS(1823),
    [sym_variable_to_xml] = ACTIONS(1823),
    [sym_xml_to_variable] = ACTIONS(1823),
    [sym_length_variable_to_xml] = ACTIONS(1823),
    [sym_redirect_string] = ACTIONS(1823),
    [sym_random_number] = ACTIONS(1823),
    [sym_get_unique_id] = ACTIONS(1823),
    [sym_get_system_number] = ACTIONS(1823),
    [sym_set_system_number] = ACTIONS(1823),
    [sym_get_serial_number] = ACTIONS(1823),
    [sym_get_master_build] = ACTIONS(1823),
    [sym_set_validation_code] = ACTIONS(1823),
    [sym_reboot] = ACTIONS(1823),
    [sym_load_duet_module] = ACTIONS(1823),
    [sym_unload_duet_module] = ACTIONS(1823),
    [sym_duet_mem_size_set] = ACTIONS(1823),
    [sym_duet_mem_size_get] = ACTIONS(1823),
    [sym_dynamic_application_device] = ACTIONS(1823),
    [sym_dynamic_polled_port] = ACTIONS(1823),
    [sym_static_port_binding] = ACTIONS(1823),
    [sym_static_ip_binding] = ACTIONS(1823),
    [sym_get_dns_list] = ACTIONS(1823),
    [sym_set_dns_list] = ACTIONS(1823),
    [sym_get_ip_address] = ACTIONS(1823),
    [sym_set_ip_address] = ACTIONS(1823),
    [sym_get_url_list] = ACTIONS(1823),
    [sym_add_url_entry] = ACTIONS(1823),
    [sym_delete_url_entry] = ACTIONS(1823),
    [sym_abs_value] = ACTIONS(1823),
    [sym_max_value] = ACTIONS(1823),
    [sym_min_value] = ACTIONS(1823),
    [sym_exp_value] = ACTIONS(1823),
    [sym_log_value] = ACTIONS(1823),
    [sym_log10_value] = ACTIONS(1823),
    [sym_power_value] = ACTIONS(1823),
    [sym_sqrt_value] = ACTIONS(1823),
    [sym_raw_be] = ACTIONS(1823),
    [sym_raw_le] = ACTIONS(1823),
    [sym_internal_queue_size_set] = ACTIONS(1823),
    [sym_internal_queue_size_get] = ACTIONS(1823),
    [sym_internal_threshold_set] = ACTIONS(1823),
    [sym_internal_threshold_get] = ACTIONS(1823),
    [sym_rebuilt_event] = ACTIONS(1823),
    [sym_do_custom_event] = ACTIONS(1823),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1823),
    [sym_clkmgr_set_clk_source] = ACTIONS(1823),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1823),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1823),
    [sym_clkmgr_get_timezone] = ACTIONS(1823),
    [sym_clkmgr_set_timezone] = ACTIONS(1823),
    [sym_clkmgr_get_resync_period] = ACTIONS(1823),
    [sym_clkmgr_set_resync_period] = ACTIONS(1823),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1823),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1823),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1823),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1823),
    [sym_clkmgr_get_timeservers] = ACTIONS(1823),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1823),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1823),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1823),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1823),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1823),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1823),
    [sym_set_log_level] = ACTIONS(1823),
    [sym_get_log_level] = ACTIONS(1823),
    [sym_amx_log] = ACTIONS(1823),
    [sym_smtp_server_config_set] = ACTIONS(1823),
    [sym_smtp_server_config_get] = ACTIONS(1823),
    [sym_smtp_send] = ACTIONS(1823),
    [sym_validate_netlinx_account] = ACTIONS(1823),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1823),
    [sym_authenticate_certificate] = ACTIONS(1823),
    [sym_audit_netlinx_session_event] = ACTIONS(1823),
    [sym_audit_netlinx_generic_event] = ACTIONS(1823),
    [sym_get_available_flash_disk_space] = ACTIONS(1823),
    [sym_get_max_flash_disk_space] = ACTIONS(1823),
    [sym_data_create_feed] = ACTIONS(1823),
    [sym_data_delete_feed] = ACTIONS(1823),
    [sym_data_publish_feed] = ACTIONS(1823),
    [sym_data_get_published_feed] = ACTIONS(1823),
    [sym_data_add_record] = ACTIONS(1823),
    [sym_data_get_event_record] = ACTIONS(1823),
    [sym__wc_data_create_feed] = ACTIONS(1823),
    [sym__wc_data_add_record] = ACTIONS(1823),
    [sym__wc_data_get_event_record] = ACTIONS(1823),
    [sym_format] = ACTIONS(1823),
    [sym_astro_clock] = ACTIONS(1823),
    [sym_set_outdoor_temperature] = ACTIONS(1823),
    [sym_set_virtual_level_count] = ACTIONS(1823),
    [sym_set_virtual_channel_count] = ACTIONS(1823),
    [sym_set_virtual_port_count] = ACTIONS(1823),
    [sym__wc_encode_int] = ACTIONS(1823),
    [sym__wc_decode_int] = ACTIONS(1823),
    [sym__wc_int] = ACTIONS(1823),
    [sym__wc_to_ch_int] = ACTIONS(1823),
    [sym__ch_to_wc_int] = ACTIONS(1823),
    [sym__wc_find_string_int] = ACTIONS(1823),
    [sym__wc_left_string_int] = ACTIONS(1823),
    [sym__wc_lower_string_int] = ACTIONS(1823),
    [sym__wc_mid_string_int] = ACTIONS(1823),
    [sym__wc_remove_string_int] = ACTIONS(1823),
    [sym__wc_right_string_int] = ACTIONS(1823),
    [sym__wc_upper_string_int] = ACTIONS(1823),
    [sym__wc_compare_string_int] = ACTIONS(1823),
    [sym__wc_get_buffer_char_int] = ACTIONS(1823),
    [sym__wc_get_buffer_string_int] = ACTIONS(1823),
    [sym__wc_concat_string_int] = ACTIONS(1823),
    [sym__wc_file_open_int] = ACTIONS(1823),
    [sym__wc_file_close_int] = ACTIONS(1823),
    [sym__wc_file_read_int] = ACTIONS(1823),
    [sym__wc_file_read_line_int] = ACTIONS(1823),
    [sym__wc_file_write_int] = ACTIONS(1823),
    [sym__wc_file_write_line_int] = ACTIONS(1823),
    [aux_sym_preproc_include_token1] = ACTIONS(1823),
    [aux_sym_preproc_define_token1] = ACTIONS(1825),
    [aux_sym_preproc_warn_token1] = ACTIONS(1825),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1825),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1825),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1825),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1825),
    [aux_sym_preproc_else_token1] = ACTIONS(1825),
    [anon_sym_LBRACK] = ACTIONS(1825),
    [anon_sym_LBRACE] = ACTIONS(1825),
    [anon_sym_LPAREN] = ACTIONS(1823),
    [anon_sym_BANG] = ACTIONS(1825),
    [anon_sym_TILDE] = ACTIONS(1825),
    [anon_sym_DASH] = ACTIONS(1823),
    [anon_sym_PLUS] = ACTIONS(1823),
    [anon_sym_DASH_DASH] = ACTIONS(1825),
    [anon_sym_PLUS_PLUS] = ACTIONS(1825),
    [anon_sym_DQUOTE] = ACTIONS(1825),
    [anon_sym_SQUOTE] = ACTIONS(1825),
    [sym_number_literal] = ACTIONS(1825),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(251)] = {
    [sym_identifier] = ACTIONS(1671),
    [sym_program_name_keyword] = ACTIONS(1671),
    [sym_module_name_keyword] = ACTIONS(1671),
    [sym_define_device_keyword] = ACTIONS(1671),
    [sym_define_combine_keyword] = ACTIONS(1671),
    [sym_define_constant_keyword] = ACTIONS(1671),
    [sym_define_type_keyword] = ACTIONS(1671),
    [sym_define_variable_keyword] = ACTIONS(1671),
    [sym_define_system_variable_keyword] = ACTIONS(1671),
    [sym_define_connect_level_keyword] = ACTIONS(1671),
    [sym_define_latching_keyword] = ACTIONS(1671),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1671),
    [sym_define_toggling_keyword] = ACTIONS(1671),
    [sym_define_call_keyword] = ACTIONS(1671),
    [sym_define_function_keyword] = ACTIONS(1671),
    [sym_define_library_function_keyword] = ACTIONS(1671),
    [sym_define_start_keyword] = ACTIONS(1671),
    [sym_define_event_keyword] = ACTIONS(1671),
    [sym_define_program_keyword] = ACTIONS(1671),
    [sym_define_module_keyword] = ACTIONS(1671),
    [sym_char_keyword] = ACTIONS(1671),
    [sym_widechar_keyword] = ACTIONS(1671),
    [sym_integer_keyword] = ACTIONS(1671),
    [sym_sinteger_keyword] = ACTIONS(1671),
    [sym_long_keyword] = ACTIONS(1671),
    [sym_slong_keyword] = ACTIONS(1671),
    [sym_float_keyword] = ACTIONS(1671),
    [sym_double_keyword] = ACTIONS(1671),
    [sym_variant_keyword] = ACTIONS(1671),
    [sym_variantarray_keyword] = ACTIONS(1671),
    [sym_local_var_keyword] = ACTIONS(1671),
    [sym_stack_var_keyword] = ACTIONS(1671),
    [sym_constant_keyword] = ACTIONS(1671),
    [sym_volatile_keyword] = ACTIONS(1671),
    [sym_non_volatile_keyword] = ACTIONS(1671),
    [sym_persistent_keyword] = ACTIONS(1671),
    [sym_dev_keyword] = ACTIONS(1671),
    [sym_devlev_keyword] = ACTIONS(1671),
    [sym_devchan_keyword] = ACTIONS(1671),
    [sym_if_keyword] = ACTIONS(1671),
    [sym_else_keyword] = ACTIONS(1671),
    [sym_switch_keyword] = ACTIONS(1671),
    [sym_case_keyword] = ACTIONS(1671),
    [sym_default_keyword] = ACTIONS(1671),
    [sym_while_keyword] = ACTIONS(1671),
    [sym_for_keyword] = ACTIONS(1671),
    [sym_break_keyword] = ACTIONS(1671),
    [sym_continue_keyword] = ACTIONS(1671),
    [sym_return_keyword] = ACTIONS(1671),
    [sym_select_keyword] = ACTIONS(1671),
    [sym_active_keyword] = ACTIONS(1671),
    [sym_struct_keyword] = ACTIONS(1671),
    [sym_structure_keyword] = ACTIONS(1671),
    [sym_button_event_keyword] = ACTIONS(1671),
    [sym_channel_event_keyword] = ACTIONS(1671),
    [sym_level_event_keyword] = ACTIONS(1671),
    [sym_data_event_keyword] = ACTIONS(1671),
    [sym_timeline_event_keyword] = ACTIONS(1671),
    [sym_custom_event_keyword] = ACTIONS(1671),
    [sym_push_keyword] = ACTIONS(1671),
    [sym_release_keyword] = ACTIONS(1671),
    [aux_sym_on_keyword_token1] = ACTIONS(1671),
    [aux_sym_off_keyword_token1] = ACTIONS(1671),
    [sym_send_level_keyword] = ACTIONS(1671),
    [sym_send_string_keyword] = ACTIONS(1671),
    [sym_send_command_keyword] = ACTIONS(1671),
    [sym_clear_buffer_keyword] = ACTIONS(1671),
    [sym_create_buffer_keyword] = ACTIONS(1671),
    [sym_create_multi_buffer_keyword] = ACTIONS(1671),
    [sym_call_keyword] = ACTIONS(1671),
    [sym_system_call_keyword] = ACTIONS(1671),
    [sym_devchan_to_keyword] = ACTIONS(1671),
    [sym_devchan_min_to_keyword] = ACTIONS(1671),
    [sym_devchan_total_off_keyword] = ACTIONS(1671),
    [sym_devchan_pulse_keyword] = ACTIONS(1671),
    [sym_wait_keyword] = ACTIONS(1671),
    [sym_cancel_wait_keyword] = ACTIONS(1671),
    [sym_wait_until_keyword] = ACTIONS(1671),
    [sym_cancel_wait_until_keyword] = ACTIONS(1671),
    [sym_cancel_all_wait_keyword] = ACTIONS(1671),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1671),
    [sym_not] = ACTIONS(1671),
    [sym_bnot] = ACTIONS(1671),
    [sym___file__] = ACTIONS(1671),
    [sym___date__] = ACTIONS(1671),
    [sym___ldate__] = ACTIONS(1671),
    [sym___time__] = ACTIONS(1671),
    [sym___line__] = ACTIONS(1671),
    [sym___name__] = ACTIONS(1671),
    [sym___netlinx__] = ACTIONS(1671),
    [sym_netlinx_axi_version] = ACTIONS(1671),
    [sym_url_flg_tcp] = ACTIONS(1671),
    [sym_url_flg_accinfopresent] = ACTIONS(1671),
    [sym_url_flg_temp] = ACTIONS(1671),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1671),
    [sym_url_flg_stat_mask] = ACTIONS(1671),
    [sym_url_flg_stat_lookup] = ACTIONS(1671),
    [sym_url_flg_stat_connecting] = ACTIONS(1671),
    [sym_url_flg_stat_waiting] = ACTIONS(1671),
    [sym_url_flg_stat_connected] = ACTIONS(1671),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1671),
    [sym_first_virtual_device] = ACTIONS(1671),
    [sym_first_local_port] = ACTIONS(1671),
    [sym_dynamic_virtual_device] = ACTIONS(1671),
    [sym_timeline_once] = ACTIONS(1671),
    [sym_timeline_repeat] = ACTIONS(1671),
    [sym_timeline_absolute] = ACTIONS(1671),
    [sym_timeline_relative] = ACTIONS(1671),
    [sym_do_push_timed_infinite] = ACTIONS(1671),
    [sym_source_type_no_address] = ACTIONS(1671),
    [sym_source_type_neuron_id] = ACTIONS(1671),
    [sym_source_type_ip_address] = ACTIONS(1671),
    [sym_source_type_axlink] = ACTIONS(1671),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1671),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1671),
    [sym_source_type_ip_socket_address] = ACTIONS(1671),
    [sym_source_type_rs232] = ACTIONS(1671),
    [sym_source_type_internal] = ACTIONS(1671),
    [sym_source_type_mac_address] = ACTIONS(1671),
    [sym_source_type_ipv4_port] = ACTIONS(1671),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1671),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1671),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1671),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1671),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1671),
    [sym_true] = ACTIONS(1671),
    [sym_false] = ACTIONS(1671),
    [sym_file_read_only] = ACTIONS(1671),
    [sym_file_rw_new] = ACTIONS(1671),
    [sym_file_rw_append] = ACTIONS(1671),
    [sym_ip_tcp] = ACTIONS(1671),
    [sym_ip_udp] = ACTIONS(1671),
    [sym_ip_udp_2way] = ACTIONS(1671),
    [sym_xml_encode_types] = ACTIONS(1671),
    [sym_xml_encode_char_as_list] = ACTIONS(1671),
    [sym_xml_encode_le] = ACTIONS(1671),
    [sym_xml_decode_types] = ACTIONS(1671),
    [sym_xml_decode_no_preserve] = ACTIONS(1671),
    [sym_duet_dev_type_display_device] = ACTIONS(1671),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1671),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1671),
    [sym_duet_dev_type_io_device] = ACTIONS(1671),
    [sym_duet_dev_type_relay_device] = ACTIONS(1671),
    [sym_duet_dev_type_ups] = ACTIONS(1671),
    [sym_duet_dev_type_amplifier] = ACTIONS(1671),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1671),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1671),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1671),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1671),
    [sym_duet_dev_type_camera] = ACTIONS(1671),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1671),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1671),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1671),
    [sym_duet_dev_type_dss] = ACTIONS(1671),
    [sym_duet_dev_type_dvr] = ACTIONS(1671),
    [sym_duet_dev_type_disc_device] = ACTIONS(1671),
    [sym_duet_dev_type_document_camera] = ACTIONS(1671),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1671),
    [sym_duet_dev_type_hvac] = ACTIONS(1671),
    [sym_duet_dev_type_keypad] = ACTIONS(1671),
    [sym_duet_dev_type_light] = ACTIONS(1671),
    [sym_duet_dev_type_monitor] = ACTIONS(1671),
    [sym_duet_dev_type_motor] = ACTIONS(1671),
    [sym_duet_dev_type_multi_window] = ACTIONS(1671),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1671),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1671),
    [sym_duet_dev_type_receiver] = ACTIONS(1671),
    [sym_duet_dev_type_security_system] = ACTIONS(1671),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1671),
    [sym_duet_dev_type_settop_box] = ACTIONS(1671),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1671),
    [sym_duet_dev_type_switcher] = ACTIONS(1671),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1671),
    [sym_duet_dev_type_tv] = ACTIONS(1671),
    [sym_duet_dev_type_utility] = ACTIONS(1671),
    [sym_duet_dev_type_vcr] = ACTIONS(1671),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1671),
    [sym_duet_dev_type_video_processor] = ACTIONS(1671),
    [sym_duet_dev_type_video_projector] = ACTIONS(1671),
    [sym_duet_dev_type_video_wall] = ACTIONS(1671),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1671),
    [sym_duet_dev_type_weather] = ACTIONS(1671),
    [sym_duet_dev_not_polled] = ACTIONS(1671),
    [sym_duet_dev_polled] = ACTIONS(1671),
    [sym_clkmgr_mode_network] = ACTIONS(1671),
    [sym_clkmgr_mode_standalone] = ACTIONS(1671),
    [sym_normal_standby] = ACTIONS(1671),
    [sym_normal_wake] = ACTIONS(1671),
    [sym_ip_multicast_ttl_option] = ACTIONS(1671),
    [sym_ip_ttl_subnet] = ACTIONS(1671),
    [sym_ip_ttl_site] = ACTIONS(1671),
    [sym_ip_ttl_region] = ACTIONS(1671),
    [sym_ip_ttl_continent] = ACTIONS(1671),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1671),
    [sym_ip_nodelay_on] = ACTIONS(1671),
    [sym_ip_nodelay_off] = ACTIONS(1671),
    [sym_amx_error] = ACTIONS(1671),
    [sym_amx_warning] = ACTIONS(1671),
    [sym_amx_info] = ACTIONS(1671),
    [sym_amx_debug] = ACTIONS(1671),
    [sym_netlinx_login_success] = ACTIONS(1671),
    [sym_netlinx_login_fail] = ACTIONS(1671),
    [sym_netlinx_logout] = ACTIONS(1671),
    [sym_null_str] = ACTIONS(1671),
    [sym_smtp_address] = ACTIONS(1671),
    [sym_smtp_port_number] = ACTIONS(1671),
    [sym_smtp_username] = ACTIONS(1671),
    [sym_smtp_password] = ACTIONS(1671),
    [sym_smtp_require_tls] = ACTIONS(1671),
    [sym_smtp_from] = ACTIONS(1671),
    [sym_smtp_tls_true] = ACTIONS(1671),
    [sym_smtp_tls_false] = ACTIONS(1671),
    [sym_listview_on_row_select_event] = ACTIONS(1671),
    [sym_data_structure_unknown] = ACTIONS(1671),
    [sym_data_structure_datafeed] = ACTIONS(1671),
    [sym_data_structure_datarecordset] = ACTIONS(1671),
    [sym_data_structure_datarecord] = ACTIONS(1671),
    [sym_data_structure_datafield] = ACTIONS(1671),
    [sym_data_max_content_field_count] = ACTIONS(1671),
    [sym_data_max_metadata_field_count] = ACTIONS(1671),
    [sym_data_max_id_length] = ACTIONS(1671),
    [sym_data_max_name_length] = ACTIONS(1671),
    [sym_data_max_description_length] = ACTIONS(1671),
    [sym_data_max_type_length] = ACTIONS(1671),
    [sym_data_max_format_length] = ACTIONS(1671),
    [sym_data_max_source_length] = ACTIONS(1671),
    [sym_data_max_value_length] = ACTIONS(1671),
    [sym_data_max_label_length] = ACTIONS(1671),
    [sym_data_max_url_length] = ACTIONS(1671),
    [sym_max_last_login_info_length] = ACTIONS(1671),
    [sym_data_type_unknown] = ACTIONS(1671),
    [sym_data_type_string] = ACTIONS(1671),
    [sym_data_type_datetime] = ACTIONS(1671),
    [sym_data_type_date] = ACTIONS(1671),
    [sym_data_type_time] = ACTIONS(1671),
    [sym_data_type_image] = ACTIONS(1671),
    [sym_data_format_url] = ACTIONS(1671),
    [sym_data_format_phone] = ACTIONS(1671),
    [sym_data_format_email] = ACTIONS(1671),
    [sym_data_format_iso8601] = ACTIONS(1671),
    [sym_auditlog_priv] = ACTIONS(1671),
    [sym_configuration_priv] = ACTIONS(1671),
    [sym_deviceconfig_priv] = ACTIONS(1671),
    [sym_ftp_priv] = ACTIONS(1671),
    [sym_http_priv] = ACTIONS(1671),
    [sym_networkconfig_priv] = ACTIONS(1671),
    [sym_programport_priv] = ACTIONS(1671),
    [sym_remoteui_priv] = ACTIONS(1671),
    [sym_securitycontrol_priv] = ACTIONS(1671),
    [sym_softwaremanagement_priv] = ACTIONS(1671),
    [sym_terminal_priv] = ACTIONS(1671),
    [sym_tpadmin_priv] = ACTIONS(1671),
    [sym_user1_priv] = ACTIONS(1671),
    [sym_user2_priv] = ACTIONS(1671),
    [sym_user3_priv] = ACTIONS(1671),
    [sym_user4_priv] = ACTIONS(1671),
    [sym_usermanagement_priv] = ACTIONS(1671),
    [sym_valid_account] = ACTIONS(1671),
    [sym_err_name_invalid] = ACTIONS(1671),
    [sym_err_password_invalid] = ACTIONS(1671),
    [sym_err_invalid_account] = ACTIONS(1671),
    [sym_err_no_authorization] = ACTIONS(1671),
    [sym_err_invalid_parameter] = ACTIONS(1671),
    [sym_err_user_locked_out] = ACTIONS(1671),
    [sym_err_user_account_expired] = ACTIONS(1671),
    [sym_err_general_error] = ACTIONS(1671),
    [sym_err_authentication_error] = ACTIONS(1671),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1671),
    [sym_tls_validate_certificate] = ACTIONS(1671),
    [sym_tdata] = ACTIONS(1671),
    [sym_tchannel] = ACTIONS(1671),
    [sym_tlevel] = ACTIONS(1671),
    [sym_tbutton] = ACTIONS(1671),
    [sym_ttimeline] = ACTIONS(1671),
    [sym_tcustom] = ACTIONS(1671),
    [sym_url_struct] = ACTIONS(1671),
    [sym_dns_struct] = ACTIONS(1671),
    [sym_ip_address_struct] = ACTIONS(1671),
    [sym_dev_info_struct] = ACTIONS(1671),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1671),
    [sym_data_feed] = ACTIONS(1671),
    [sym_data_field] = ACTIONS(1671),
    [sym_data_record] = ACTIONS(1671),
    [sym_wc_data_feed] = ACTIONS(1671),
    [sym_wc_data_field] = ACTIONS(1671),
    [sym_wc_data_record] = ACTIONS(1671),
    [sym_last_login_info] = ACTIONS(1671),
    [sym____reserved___] = ACTIONS(1671),
    [sym_date] = ACTIONS(1671),
    [sym_day] = ACTIONS(1671),
    [sym_time] = ACTIONS(1671),
    [sym_ldate] = ACTIONS(1671),
    [sym_push_device] = ACTIONS(1671),
    [sym_push_channel] = ACTIONS(1671),
    [sym_push_devchan] = ACTIONS(1671),
    [sym_release_device] = ACTIONS(1671),
    [sym_release_channel] = ACTIONS(1671),
    [sym_release_devchan] = ACTIONS(1671),
    [sym_master_slot] = ACTIONS(1671),
    [sym_get_pulse_time] = ACTIONS(1671),
    [sym_get_timer] = ACTIONS(1671),
    [sym_master_sn] = ACTIONS(1671),
    [sym_system_number] = ACTIONS(1671),
    [sym_button] = ACTIONS(1671),
    [sym_dv_channel] = ACTIONS(1671),
    [sym_data] = ACTIONS(1671),
    [sym_level] = ACTIONS(1671),
    [sym_timeline] = ACTIONS(1671),
    [sym_channel] = ACTIONS(1671),
    [sym_custom] = ACTIONS(1671),
    [sym_atoi] = ACTIONS(1671),
    [sym_atol] = ACTIONS(1671),
    [sym_atof] = ACTIONS(1671),
    [sym_itoa] = ACTIONS(1671),
    [sym_itohex] = ACTIONS(1671),
    [sym_hextoi] = ACTIONS(1671),
    [sym_ftoa] = ACTIONS(1671),
    [sym_find_string] = ACTIONS(1671),
    [sym_left_string] = ACTIONS(1671),
    [sym_right_string] = ACTIONS(1671),
    [sym_mid_string] = ACTIONS(1671),
    [sym_remove_string] = ACTIONS(1671),
    [sym_get_buffer_string] = ACTIONS(1671),
    [sym_get_buffer_char] = ACTIONS(1671),
    [sym_get_multi_buffer_string] = ACTIONS(1671),
    [sym_length_string] = ACTIONS(1671),
    [sym_max_length_string] = ACTIONS(1671),
    [sym_set_length_string] = ACTIONS(1671),
    [sym_compare_string] = ACTIONS(1671),
    [sym_upper_string] = ACTIONS(1671),
    [sym_lower_string] = ACTIONS(1671),
    [sym_get_last] = ACTIONS(1671),
    [sym_date_to_day] = ACTIONS(1671),
    [sym_date_to_month] = ACTIONS(1671),
    [sym_date_to_year] = ACTIONS(1671),
    [sym_time_to_hour] = ACTIONS(1671),
    [sym_time_to_minute] = ACTIONS(1671),
    [sym_time_to_second] = ACTIONS(1671),
    [sym_day_of_week] = ACTIONS(1671),
    [sym_set_pulse_time] = ACTIONS(1671),
    [sym_set_timer] = ACTIONS(1671),
    [sym_file_open] = ACTIONS(1671),
    [sym_file_close] = ACTIONS(1671),
    [sym_file_read] = ACTIONS(1671),
    [sym_file_read_line] = ACTIONS(1671),
    [sym_file_write] = ACTIONS(1671),
    [sym_file_write_line] = ACTIONS(1671),
    [sym_file_copy] = ACTIONS(1671),
    [sym_file_delete] = ACTIONS(1671),
    [sym_file_rename] = ACTIONS(1671),
    [sym_file_dir] = ACTIONS(1671),
    [sym_file_getdir] = ACTIONS(1671),
    [sym_file_setdir] = ACTIONS(1671),
    [sym_file_create_dir] = ACTIONS(1671),
    [sym_file_remove_dir] = ACTIONS(1671),
    [sym_file_seek] = ACTIONS(1671),
    [sym_length_array] = ACTIONS(1671),
    [sym_max_length_array] = ACTIONS(1671),
    [sym_set_length_array] = ACTIONS(1671),
    [sym_ip_client_open] = ACTIONS(1671),
    [sym_ip_client_close] = ACTIONS(1671),
    [sym_ip_server_open] = ACTIONS(1671),
    [sym_ip_server_close] = ACTIONS(1671),
    [sym_ip_bound_client_open] = ACTIONS(1671),
    [sym_ip_mc_server_open] = ACTIONS(1671),
    [sym_ip_set_option] = ACTIONS(1671),
    [sym_ssh_client_open] = ACTIONS(1671),
    [sym_ssh_client_close] = ACTIONS(1671),
    [sym_tls_client_open] = ACTIONS(1671),
    [sym_tls_client_close] = ACTIONS(1671),
    [sym_do_push] = ACTIONS(1671),
    [sym_do_push_timed] = ACTIONS(1671),
    [sym_do_release] = ACTIONS(1671),
    [sym_device_id] = ACTIONS(1671),
    [sym_device_info] = ACTIONS(1671),
    [sym_device_id_string] = ACTIONS(1671),
    [sym_device_standby] = ACTIONS(1671),
    [sym_device_wake] = ACTIONS(1671),
    [sym_timeline_create] = ACTIONS(1671),
    [sym_timeline_kill] = ACTIONS(1671),
    [sym_timeline_reload] = ACTIONS(1671),
    [sym_timeline_pause] = ACTIONS(1671),
    [sym_timeline_restart] = ACTIONS(1671),
    [sym_timeline_set] = ACTIONS(1671),
    [sym_timeline_get] = ACTIONS(1671),
    [sym_timeline_active] = ACTIONS(1671),
    [sym_timeline_dynamic_id] = ACTIONS(1671),
    [sym_variable_to_string] = ACTIONS(1671),
    [sym_string_to_variable] = ACTIONS(1671),
    [sym_length_variable_to_string] = ACTIONS(1671),
    [sym_type_cast] = ACTIONS(1671),
    [sym_variable_to_xml] = ACTIONS(1671),
    [sym_xml_to_variable] = ACTIONS(1671),
    [sym_length_variable_to_xml] = ACTIONS(1671),
    [sym_redirect_string] = ACTIONS(1671),
    [sym_random_number] = ACTIONS(1671),
    [sym_get_unique_id] = ACTIONS(1671),
    [sym_get_system_number] = ACTIONS(1671),
    [sym_set_system_number] = ACTIONS(1671),
    [sym_get_serial_number] = ACTIONS(1671),
    [sym_get_master_build] = ACTIONS(1671),
    [sym_set_validation_code] = ACTIONS(1671),
    [sym_reboot] = ACTIONS(1671),
    [sym_load_duet_module] = ACTIONS(1671),
    [sym_unload_duet_module] = ACTIONS(1671),
    [sym_duet_mem_size_set] = ACTIONS(1671),
    [sym_duet_mem_size_get] = ACTIONS(1671),
    [sym_dynamic_application_device] = ACTIONS(1671),
    [sym_dynamic_polled_port] = ACTIONS(1671),
    [sym_static_port_binding] = ACTIONS(1671),
    [sym_static_ip_binding] = ACTIONS(1671),
    [sym_get_dns_list] = ACTIONS(1671),
    [sym_set_dns_list] = ACTIONS(1671),
    [sym_get_ip_address] = ACTIONS(1671),
    [sym_set_ip_address] = ACTIONS(1671),
    [sym_get_url_list] = ACTIONS(1671),
    [sym_add_url_entry] = ACTIONS(1671),
    [sym_delete_url_entry] = ACTIONS(1671),
    [sym_abs_value] = ACTIONS(1671),
    [sym_max_value] = ACTIONS(1671),
    [sym_min_value] = ACTIONS(1671),
    [sym_exp_value] = ACTIONS(1671),
    [sym_log_value] = ACTIONS(1671),
    [sym_log10_value] = ACTIONS(1671),
    [sym_power_value] = ACTIONS(1671),
    [sym_sqrt_value] = ACTIONS(1671),
    [sym_raw_be] = ACTIONS(1671),
    [sym_raw_le] = ACTIONS(1671),
    [sym_internal_queue_size_set] = ACTIONS(1671),
    [sym_internal_queue_size_get] = ACTIONS(1671),
    [sym_internal_threshold_set] = ACTIONS(1671),
    [sym_internal_threshold_get] = ACTIONS(1671),
    [sym_rebuilt_event] = ACTIONS(1671),
    [sym_do_custom_event] = ACTIONS(1671),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1671),
    [sym_clkmgr_set_clk_source] = ACTIONS(1671),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1671),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1671),
    [sym_clkmgr_get_timezone] = ACTIONS(1671),
    [sym_clkmgr_set_timezone] = ACTIONS(1671),
    [sym_clkmgr_get_resync_period] = ACTIONS(1671),
    [sym_clkmgr_set_resync_period] = ACTIONS(1671),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1671),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1671),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1671),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1671),
    [sym_clkmgr_get_timeservers] = ACTIONS(1671),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1671),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1671),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1671),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1671),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1671),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1671),
    [sym_set_log_level] = ACTIONS(1671),
    [sym_get_log_level] = ACTIONS(1671),
    [sym_amx_log] = ACTIONS(1671),
    [sym_smtp_server_config_set] = ACTIONS(1671),
    [sym_smtp_server_config_get] = ACTIONS(1671),
    [sym_smtp_send] = ACTIONS(1671),
    [sym_validate_netlinx_account] = ACTIONS(1671),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1671),
    [sym_authenticate_certificate] = ACTIONS(1671),
    [sym_audit_netlinx_session_event] = ACTIONS(1671),
    [sym_audit_netlinx_generic_event] = ACTIONS(1671),
    [sym_get_available_flash_disk_space] = ACTIONS(1671),
    [sym_get_max_flash_disk_space] = ACTIONS(1671),
    [sym_data_create_feed] = ACTIONS(1671),
    [sym_data_delete_feed] = ACTIONS(1671),
    [sym_data_publish_feed] = ACTIONS(1671),
    [sym_data_get_published_feed] = ACTIONS(1671),
    [sym_data_add_record] = ACTIONS(1671),
    [sym_data_get_event_record] = ACTIONS(1671),
    [sym__wc_data_create_feed] = ACTIONS(1671),
    [sym__wc_data_add_record] = ACTIONS(1671),
    [sym__wc_data_get_event_record] = ACTIONS(1671),
    [sym_format] = ACTIONS(1671),
    [sym_astro_clock] = ACTIONS(1671),
    [sym_set_outdoor_temperature] = ACTIONS(1671),
    [sym_set_virtual_level_count] = ACTIONS(1671),
    [sym_set_virtual_channel_count] = ACTIONS(1671),
    [sym_set_virtual_port_count] = ACTIONS(1671),
    [sym__wc_encode_int] = ACTIONS(1671),
    [sym__wc_decode_int] = ACTIONS(1671),
    [sym__wc_int] = ACTIONS(1671),
    [sym__wc_to_ch_int] = ACTIONS(1671),
    [sym__ch_to_wc_int] = ACTIONS(1671),
    [sym__wc_find_string_int] = ACTIONS(1671),
    [sym__wc_left_string_int] = ACTIONS(1671),
    [sym__wc_lower_string_int] = ACTIONS(1671),
    [sym__wc_mid_string_int] = ACTIONS(1671),
    [sym__wc_remove_string_int] = ACTIONS(1671),
    [sym__wc_right_string_int] = ACTIONS(1671),
    [sym__wc_upper_string_int] = ACTIONS(1671),
    [sym__wc_compare_string_int] = ACTIONS(1671),
    [sym__wc_get_buffer_char_int] = ACTIONS(1671),
    [sym__wc_get_buffer_string_int] = ACTIONS(1671),
    [sym__wc_concat_string_int] = ACTIONS(1671),
    [sym__wc_file_open_int] = ACTIONS(1671),
    [sym__wc_file_close_int] = ACTIONS(1671),
    [sym__wc_file_read_int] = ACTIONS(1671),
    [sym__wc_file_read_line_int] = ACTIONS(1671),
    [sym__wc_file_write_int] = ACTIONS(1671),
    [sym__wc_file_write_line_int] = ACTIONS(1671),
    [aux_sym_preproc_include_token1] = ACTIONS(1671),
    [aux_sym_preproc_define_token1] = ACTIONS(1669),
    [aux_sym_preproc_warn_token1] = ACTIONS(1669),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1669),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1669),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1669),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1669),
    [aux_sym_preproc_else_token1] = ACTIONS(1669),
    [anon_sym_LBRACK] = ACTIONS(1669),
    [anon_sym_LBRACE] = ACTIONS(1669),
    [anon_sym_LPAREN] = ACTIONS(1671),
    [anon_sym_BANG] = ACTIONS(1669),
    [anon_sym_TILDE] = ACTIONS(1669),
    [anon_sym_DASH] = ACTIONS(1671),
    [anon_sym_PLUS] = ACTIONS(1671),
    [anon_sym_DASH_DASH] = ACTIONS(1669),
    [anon_sym_PLUS_PLUS] = ACTIONS(1669),
    [anon_sym_DQUOTE] = ACTIONS(1669),
    [anon_sym_SQUOTE] = ACTIONS(1669),
    [sym_number_literal] = ACTIONS(1669),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(252)] = {
    [sym_identifier] = ACTIONS(1827),
    [sym_program_name_keyword] = ACTIONS(1827),
    [sym_module_name_keyword] = ACTIONS(1827),
    [sym_define_device_keyword] = ACTIONS(1827),
    [sym_define_combine_keyword] = ACTIONS(1827),
    [sym_define_constant_keyword] = ACTIONS(1827),
    [sym_define_type_keyword] = ACTIONS(1827),
    [sym_define_variable_keyword] = ACTIONS(1827),
    [sym_define_system_variable_keyword] = ACTIONS(1827),
    [sym_define_connect_level_keyword] = ACTIONS(1827),
    [sym_define_latching_keyword] = ACTIONS(1827),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1827),
    [sym_define_toggling_keyword] = ACTIONS(1827),
    [sym_define_call_keyword] = ACTIONS(1827),
    [sym_define_function_keyword] = ACTIONS(1827),
    [sym_define_library_function_keyword] = ACTIONS(1827),
    [sym_define_start_keyword] = ACTIONS(1827),
    [sym_define_event_keyword] = ACTIONS(1827),
    [sym_define_program_keyword] = ACTIONS(1827),
    [sym_define_module_keyword] = ACTIONS(1827),
    [sym_char_keyword] = ACTIONS(1827),
    [sym_widechar_keyword] = ACTIONS(1827),
    [sym_integer_keyword] = ACTIONS(1827),
    [sym_sinteger_keyword] = ACTIONS(1827),
    [sym_long_keyword] = ACTIONS(1827),
    [sym_slong_keyword] = ACTIONS(1827),
    [sym_float_keyword] = ACTIONS(1827),
    [sym_double_keyword] = ACTIONS(1827),
    [sym_variant_keyword] = ACTIONS(1827),
    [sym_variantarray_keyword] = ACTIONS(1827),
    [sym_local_var_keyword] = ACTIONS(1827),
    [sym_stack_var_keyword] = ACTIONS(1827),
    [sym_constant_keyword] = ACTIONS(1827),
    [sym_volatile_keyword] = ACTIONS(1827),
    [sym_non_volatile_keyword] = ACTIONS(1827),
    [sym_persistent_keyword] = ACTIONS(1827),
    [sym_dev_keyword] = ACTIONS(1827),
    [sym_devlev_keyword] = ACTIONS(1827),
    [sym_devchan_keyword] = ACTIONS(1827),
    [sym_if_keyword] = ACTIONS(1827),
    [sym_else_keyword] = ACTIONS(1827),
    [sym_switch_keyword] = ACTIONS(1827),
    [sym_case_keyword] = ACTIONS(1827),
    [sym_default_keyword] = ACTIONS(1827),
    [sym_while_keyword] = ACTIONS(1827),
    [sym_for_keyword] = ACTIONS(1827),
    [sym_break_keyword] = ACTIONS(1827),
    [sym_continue_keyword] = ACTIONS(1827),
    [sym_return_keyword] = ACTIONS(1827),
    [sym_select_keyword] = ACTIONS(1827),
    [sym_active_keyword] = ACTIONS(1827),
    [sym_struct_keyword] = ACTIONS(1827),
    [sym_structure_keyword] = ACTIONS(1827),
    [sym_button_event_keyword] = ACTIONS(1827),
    [sym_channel_event_keyword] = ACTIONS(1827),
    [sym_level_event_keyword] = ACTIONS(1827),
    [sym_data_event_keyword] = ACTIONS(1827),
    [sym_timeline_event_keyword] = ACTIONS(1827),
    [sym_custom_event_keyword] = ACTIONS(1827),
    [sym_push_keyword] = ACTIONS(1827),
    [sym_release_keyword] = ACTIONS(1827),
    [aux_sym_on_keyword_token1] = ACTIONS(1827),
    [aux_sym_off_keyword_token1] = ACTIONS(1827),
    [sym_send_level_keyword] = ACTIONS(1827),
    [sym_send_string_keyword] = ACTIONS(1827),
    [sym_send_command_keyword] = ACTIONS(1827),
    [sym_clear_buffer_keyword] = ACTIONS(1827),
    [sym_create_buffer_keyword] = ACTIONS(1827),
    [sym_create_multi_buffer_keyword] = ACTIONS(1827),
    [sym_call_keyword] = ACTIONS(1827),
    [sym_system_call_keyword] = ACTIONS(1827),
    [sym_devchan_to_keyword] = ACTIONS(1827),
    [sym_devchan_min_to_keyword] = ACTIONS(1827),
    [sym_devchan_total_off_keyword] = ACTIONS(1827),
    [sym_devchan_pulse_keyword] = ACTIONS(1827),
    [sym_wait_keyword] = ACTIONS(1827),
    [sym_cancel_wait_keyword] = ACTIONS(1827),
    [sym_wait_until_keyword] = ACTIONS(1827),
    [sym_cancel_wait_until_keyword] = ACTIONS(1827),
    [sym_cancel_all_wait_keyword] = ACTIONS(1827),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1827),
    [sym_not] = ACTIONS(1827),
    [sym_bnot] = ACTIONS(1827),
    [sym___file__] = ACTIONS(1827),
    [sym___date__] = ACTIONS(1827),
    [sym___ldate__] = ACTIONS(1827),
    [sym___time__] = ACTIONS(1827),
    [sym___line__] = ACTIONS(1827),
    [sym___name__] = ACTIONS(1827),
    [sym___netlinx__] = ACTIONS(1827),
    [sym_netlinx_axi_version] = ACTIONS(1827),
    [sym_url_flg_tcp] = ACTIONS(1827),
    [sym_url_flg_accinfopresent] = ACTIONS(1827),
    [sym_url_flg_temp] = ACTIONS(1827),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1827),
    [sym_url_flg_stat_mask] = ACTIONS(1827),
    [sym_url_flg_stat_lookup] = ACTIONS(1827),
    [sym_url_flg_stat_connecting] = ACTIONS(1827),
    [sym_url_flg_stat_waiting] = ACTIONS(1827),
    [sym_url_flg_stat_connected] = ACTIONS(1827),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1827),
    [sym_first_virtual_device] = ACTIONS(1827),
    [sym_first_local_port] = ACTIONS(1827),
    [sym_dynamic_virtual_device] = ACTIONS(1827),
    [sym_timeline_once] = ACTIONS(1827),
    [sym_timeline_repeat] = ACTIONS(1827),
    [sym_timeline_absolute] = ACTIONS(1827),
    [sym_timeline_relative] = ACTIONS(1827),
    [sym_do_push_timed_infinite] = ACTIONS(1827),
    [sym_source_type_no_address] = ACTIONS(1827),
    [sym_source_type_neuron_id] = ACTIONS(1827),
    [sym_source_type_ip_address] = ACTIONS(1827),
    [sym_source_type_axlink] = ACTIONS(1827),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1827),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1827),
    [sym_source_type_ip_socket_address] = ACTIONS(1827),
    [sym_source_type_rs232] = ACTIONS(1827),
    [sym_source_type_internal] = ACTIONS(1827),
    [sym_source_type_mac_address] = ACTIONS(1827),
    [sym_source_type_ipv4_port] = ACTIONS(1827),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1827),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1827),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1827),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1827),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1827),
    [sym_true] = ACTIONS(1827),
    [sym_false] = ACTIONS(1827),
    [sym_file_read_only] = ACTIONS(1827),
    [sym_file_rw_new] = ACTIONS(1827),
    [sym_file_rw_append] = ACTIONS(1827),
    [sym_ip_tcp] = ACTIONS(1827),
    [sym_ip_udp] = ACTIONS(1827),
    [sym_ip_udp_2way] = ACTIONS(1827),
    [sym_xml_encode_types] = ACTIONS(1827),
    [sym_xml_encode_char_as_list] = ACTIONS(1827),
    [sym_xml_encode_le] = ACTIONS(1827),
    [sym_xml_decode_types] = ACTIONS(1827),
    [sym_xml_decode_no_preserve] = ACTIONS(1827),
    [sym_duet_dev_type_display_device] = ACTIONS(1827),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1827),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1827),
    [sym_duet_dev_type_io_device] = ACTIONS(1827),
    [sym_duet_dev_type_relay_device] = ACTIONS(1827),
    [sym_duet_dev_type_ups] = ACTIONS(1827),
    [sym_duet_dev_type_amplifier] = ACTIONS(1827),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1827),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1827),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1827),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1827),
    [sym_duet_dev_type_camera] = ACTIONS(1827),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1827),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1827),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1827),
    [sym_duet_dev_type_dss] = ACTIONS(1827),
    [sym_duet_dev_type_dvr] = ACTIONS(1827),
    [sym_duet_dev_type_disc_device] = ACTIONS(1827),
    [sym_duet_dev_type_document_camera] = ACTIONS(1827),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1827),
    [sym_duet_dev_type_hvac] = ACTIONS(1827),
    [sym_duet_dev_type_keypad] = ACTIONS(1827),
    [sym_duet_dev_type_light] = ACTIONS(1827),
    [sym_duet_dev_type_monitor] = ACTIONS(1827),
    [sym_duet_dev_type_motor] = ACTIONS(1827),
    [sym_duet_dev_type_multi_window] = ACTIONS(1827),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1827),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1827),
    [sym_duet_dev_type_receiver] = ACTIONS(1827),
    [sym_duet_dev_type_security_system] = ACTIONS(1827),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1827),
    [sym_duet_dev_type_settop_box] = ACTIONS(1827),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1827),
    [sym_duet_dev_type_switcher] = ACTIONS(1827),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1827),
    [sym_duet_dev_type_tv] = ACTIONS(1827),
    [sym_duet_dev_type_utility] = ACTIONS(1827),
    [sym_duet_dev_type_vcr] = ACTIONS(1827),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1827),
    [sym_duet_dev_type_video_processor] = ACTIONS(1827),
    [sym_duet_dev_type_video_projector] = ACTIONS(1827),
    [sym_duet_dev_type_video_wall] = ACTIONS(1827),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1827),
    [sym_duet_dev_type_weather] = ACTIONS(1827),
    [sym_duet_dev_not_polled] = ACTIONS(1827),
    [sym_duet_dev_polled] = ACTIONS(1827),
    [sym_clkmgr_mode_network] = ACTIONS(1827),
    [sym_clkmgr_mode_standalone] = ACTIONS(1827),
    [sym_normal_standby] = ACTIONS(1827),
    [sym_normal_wake] = ACTIONS(1827),
    [sym_ip_multicast_ttl_option] = ACTIONS(1827),
    [sym_ip_ttl_subnet] = ACTIONS(1827),
    [sym_ip_ttl_site] = ACTIONS(1827),
    [sym_ip_ttl_region] = ACTIONS(1827),
    [sym_ip_ttl_continent] = ACTIONS(1827),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1827),
    [sym_ip_nodelay_on] = ACTIONS(1827),
    [sym_ip_nodelay_off] = ACTIONS(1827),
    [sym_amx_error] = ACTIONS(1827),
    [sym_amx_warning] = ACTIONS(1827),
    [sym_amx_info] = ACTIONS(1827),
    [sym_amx_debug] = ACTIONS(1827),
    [sym_netlinx_login_success] = ACTIONS(1827),
    [sym_netlinx_login_fail] = ACTIONS(1827),
    [sym_netlinx_logout] = ACTIONS(1827),
    [sym_null_str] = ACTIONS(1827),
    [sym_smtp_address] = ACTIONS(1827),
    [sym_smtp_port_number] = ACTIONS(1827),
    [sym_smtp_username] = ACTIONS(1827),
    [sym_smtp_password] = ACTIONS(1827),
    [sym_smtp_require_tls] = ACTIONS(1827),
    [sym_smtp_from] = ACTIONS(1827),
    [sym_smtp_tls_true] = ACTIONS(1827),
    [sym_smtp_tls_false] = ACTIONS(1827),
    [sym_listview_on_row_select_event] = ACTIONS(1827),
    [sym_data_structure_unknown] = ACTIONS(1827),
    [sym_data_structure_datafeed] = ACTIONS(1827),
    [sym_data_structure_datarecordset] = ACTIONS(1827),
    [sym_data_structure_datarecord] = ACTIONS(1827),
    [sym_data_structure_datafield] = ACTIONS(1827),
    [sym_data_max_content_field_count] = ACTIONS(1827),
    [sym_data_max_metadata_field_count] = ACTIONS(1827),
    [sym_data_max_id_length] = ACTIONS(1827),
    [sym_data_max_name_length] = ACTIONS(1827),
    [sym_data_max_description_length] = ACTIONS(1827),
    [sym_data_max_type_length] = ACTIONS(1827),
    [sym_data_max_format_length] = ACTIONS(1827),
    [sym_data_max_source_length] = ACTIONS(1827),
    [sym_data_max_value_length] = ACTIONS(1827),
    [sym_data_max_label_length] = ACTIONS(1827),
    [sym_data_max_url_length] = ACTIONS(1827),
    [sym_max_last_login_info_length] = ACTIONS(1827),
    [sym_data_type_unknown] = ACTIONS(1827),
    [sym_data_type_string] = ACTIONS(1827),
    [sym_data_type_datetime] = ACTIONS(1827),
    [sym_data_type_date] = ACTIONS(1827),
    [sym_data_type_time] = ACTIONS(1827),
    [sym_data_type_image] = ACTIONS(1827),
    [sym_data_format_url] = ACTIONS(1827),
    [sym_data_format_phone] = ACTIONS(1827),
    [sym_data_format_email] = ACTIONS(1827),
    [sym_data_format_iso8601] = ACTIONS(1827),
    [sym_auditlog_priv] = ACTIONS(1827),
    [sym_configuration_priv] = ACTIONS(1827),
    [sym_deviceconfig_priv] = ACTIONS(1827),
    [sym_ftp_priv] = ACTIONS(1827),
    [sym_http_priv] = ACTIONS(1827),
    [sym_networkconfig_priv] = ACTIONS(1827),
    [sym_programport_priv] = ACTIONS(1827),
    [sym_remoteui_priv] = ACTIONS(1827),
    [sym_securitycontrol_priv] = ACTIONS(1827),
    [sym_softwaremanagement_priv] = ACTIONS(1827),
    [sym_terminal_priv] = ACTIONS(1827),
    [sym_tpadmin_priv] = ACTIONS(1827),
    [sym_user1_priv] = ACTIONS(1827),
    [sym_user2_priv] = ACTIONS(1827),
    [sym_user3_priv] = ACTIONS(1827),
    [sym_user4_priv] = ACTIONS(1827),
    [sym_usermanagement_priv] = ACTIONS(1827),
    [sym_valid_account] = ACTIONS(1827),
    [sym_err_name_invalid] = ACTIONS(1827),
    [sym_err_password_invalid] = ACTIONS(1827),
    [sym_err_invalid_account] = ACTIONS(1827),
    [sym_err_no_authorization] = ACTIONS(1827),
    [sym_err_invalid_parameter] = ACTIONS(1827),
    [sym_err_user_locked_out] = ACTIONS(1827),
    [sym_err_user_account_expired] = ACTIONS(1827),
    [sym_err_general_error] = ACTIONS(1827),
    [sym_err_authentication_error] = ACTIONS(1827),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1827),
    [sym_tls_validate_certificate] = ACTIONS(1827),
    [sym_tdata] = ACTIONS(1827),
    [sym_tchannel] = ACTIONS(1827),
    [sym_tlevel] = ACTIONS(1827),
    [sym_tbutton] = ACTIONS(1827),
    [sym_ttimeline] = ACTIONS(1827),
    [sym_tcustom] = ACTIONS(1827),
    [sym_url_struct] = ACTIONS(1827),
    [sym_dns_struct] = ACTIONS(1827),
    [sym_ip_address_struct] = ACTIONS(1827),
    [sym_dev_info_struct] = ACTIONS(1827),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1827),
    [sym_data_feed] = ACTIONS(1827),
    [sym_data_field] = ACTIONS(1827),
    [sym_data_record] = ACTIONS(1827),
    [sym_wc_data_feed] = ACTIONS(1827),
    [sym_wc_data_field] = ACTIONS(1827),
    [sym_wc_data_record] = ACTIONS(1827),
    [sym_last_login_info] = ACTIONS(1827),
    [sym____reserved___] = ACTIONS(1827),
    [sym_date] = ACTIONS(1827),
    [sym_day] = ACTIONS(1827),
    [sym_time] = ACTIONS(1827),
    [sym_ldate] = ACTIONS(1827),
    [sym_push_device] = ACTIONS(1827),
    [sym_push_channel] = ACTIONS(1827),
    [sym_push_devchan] = ACTIONS(1827),
    [sym_release_device] = ACTIONS(1827),
    [sym_release_channel] = ACTIONS(1827),
    [sym_release_devchan] = ACTIONS(1827),
    [sym_master_slot] = ACTIONS(1827),
    [sym_get_pulse_time] = ACTIONS(1827),
    [sym_get_timer] = ACTIONS(1827),
    [sym_master_sn] = ACTIONS(1827),
    [sym_system_number] = ACTIONS(1827),
    [sym_button] = ACTIONS(1827),
    [sym_dv_channel] = ACTIONS(1827),
    [sym_data] = ACTIONS(1827),
    [sym_level] = ACTIONS(1827),
    [sym_timeline] = ACTIONS(1827),
    [sym_channel] = ACTIONS(1827),
    [sym_custom] = ACTIONS(1827),
    [sym_atoi] = ACTIONS(1827),
    [sym_atol] = ACTIONS(1827),
    [sym_atof] = ACTIONS(1827),
    [sym_itoa] = ACTIONS(1827),
    [sym_itohex] = ACTIONS(1827),
    [sym_hextoi] = ACTIONS(1827),
    [sym_ftoa] = ACTIONS(1827),
    [sym_find_string] = ACTIONS(1827),
    [sym_left_string] = ACTIONS(1827),
    [sym_right_string] = ACTIONS(1827),
    [sym_mid_string] = ACTIONS(1827),
    [sym_remove_string] = ACTIONS(1827),
    [sym_get_buffer_string] = ACTIONS(1827),
    [sym_get_buffer_char] = ACTIONS(1827),
    [sym_get_multi_buffer_string] = ACTIONS(1827),
    [sym_length_string] = ACTIONS(1827),
    [sym_max_length_string] = ACTIONS(1827),
    [sym_set_length_string] = ACTIONS(1827),
    [sym_compare_string] = ACTIONS(1827),
    [sym_upper_string] = ACTIONS(1827),
    [sym_lower_string] = ACTIONS(1827),
    [sym_get_last] = ACTIONS(1827),
    [sym_date_to_day] = ACTIONS(1827),
    [sym_date_to_month] = ACTIONS(1827),
    [sym_date_to_year] = ACTIONS(1827),
    [sym_time_to_hour] = ACTIONS(1827),
    [sym_time_to_minute] = ACTIONS(1827),
    [sym_time_to_second] = ACTIONS(1827),
    [sym_day_of_week] = ACTIONS(1827),
    [sym_set_pulse_time] = ACTIONS(1827),
    [sym_set_timer] = ACTIONS(1827),
    [sym_file_open] = ACTIONS(1827),
    [sym_file_close] = ACTIONS(1827),
    [sym_file_read] = ACTIONS(1827),
    [sym_file_read_line] = ACTIONS(1827),
    [sym_file_write] = ACTIONS(1827),
    [sym_file_write_line] = ACTIONS(1827),
    [sym_file_copy] = ACTIONS(1827),
    [sym_file_delete] = ACTIONS(1827),
    [sym_file_rename] = ACTIONS(1827),
    [sym_file_dir] = ACTIONS(1827),
    [sym_file_getdir] = ACTIONS(1827),
    [sym_file_setdir] = ACTIONS(1827),
    [sym_file_create_dir] = ACTIONS(1827),
    [sym_file_remove_dir] = ACTIONS(1827),
    [sym_file_seek] = ACTIONS(1827),
    [sym_length_array] = ACTIONS(1827),
    [sym_max_length_array] = ACTIONS(1827),
    [sym_set_length_array] = ACTIONS(1827),
    [sym_ip_client_open] = ACTIONS(1827),
    [sym_ip_client_close] = ACTIONS(1827),
    [sym_ip_server_open] = ACTIONS(1827),
    [sym_ip_server_close] = ACTIONS(1827),
    [sym_ip_bound_client_open] = ACTIONS(1827),
    [sym_ip_mc_server_open] = ACTIONS(1827),
    [sym_ip_set_option] = ACTIONS(1827),
    [sym_ssh_client_open] = ACTIONS(1827),
    [sym_ssh_client_close] = ACTIONS(1827),
    [sym_tls_client_open] = ACTIONS(1827),
    [sym_tls_client_close] = ACTIONS(1827),
    [sym_do_push] = ACTIONS(1827),
    [sym_do_push_timed] = ACTIONS(1827),
    [sym_do_release] = ACTIONS(1827),
    [sym_device_id] = ACTIONS(1827),
    [sym_device_info] = ACTIONS(1827),
    [sym_device_id_string] = ACTIONS(1827),
    [sym_device_standby] = ACTIONS(1827),
    [sym_device_wake] = ACTIONS(1827),
    [sym_timeline_create] = ACTIONS(1827),
    [sym_timeline_kill] = ACTIONS(1827),
    [sym_timeline_reload] = ACTIONS(1827),
    [sym_timeline_pause] = ACTIONS(1827),
    [sym_timeline_restart] = ACTIONS(1827),
    [sym_timeline_set] = ACTIONS(1827),
    [sym_timeline_get] = ACTIONS(1827),
    [sym_timeline_active] = ACTIONS(1827),
    [sym_timeline_dynamic_id] = ACTIONS(1827),
    [sym_variable_to_string] = ACTIONS(1827),
    [sym_string_to_variable] = ACTIONS(1827),
    [sym_length_variable_to_string] = ACTIONS(1827),
    [sym_type_cast] = ACTIONS(1827),
    [sym_variable_to_xml] = ACTIONS(1827),
    [sym_xml_to_variable] = ACTIONS(1827),
    [sym_length_variable_to_xml] = ACTIONS(1827),
    [sym_redirect_string] = ACTIONS(1827),
    [sym_random_number] = ACTIONS(1827),
    [sym_get_unique_id] = ACTIONS(1827),
    [sym_get_system_number] = ACTIONS(1827),
    [sym_set_system_number] = ACTIONS(1827),
    [sym_get_serial_number] = ACTIONS(1827),
    [sym_get_master_build] = ACTIONS(1827),
    [sym_set_validation_code] = ACTIONS(1827),
    [sym_reboot] = ACTIONS(1827),
    [sym_load_duet_module] = ACTIONS(1827),
    [sym_unload_duet_module] = ACTIONS(1827),
    [sym_duet_mem_size_set] = ACTIONS(1827),
    [sym_duet_mem_size_get] = ACTIONS(1827),
    [sym_dynamic_application_device] = ACTIONS(1827),
    [sym_dynamic_polled_port] = ACTIONS(1827),
    [sym_static_port_binding] = ACTIONS(1827),
    [sym_static_ip_binding] = ACTIONS(1827),
    [sym_get_dns_list] = ACTIONS(1827),
    [sym_set_dns_list] = ACTIONS(1827),
    [sym_get_ip_address] = ACTIONS(1827),
    [sym_set_ip_address] = ACTIONS(1827),
    [sym_get_url_list] = ACTIONS(1827),
    [sym_add_url_entry] = ACTIONS(1827),
    [sym_delete_url_entry] = ACTIONS(1827),
    [sym_abs_value] = ACTIONS(1827),
    [sym_max_value] = ACTIONS(1827),
    [sym_min_value] = ACTIONS(1827),
    [sym_exp_value] = ACTIONS(1827),
    [sym_log_value] = ACTIONS(1827),
    [sym_log10_value] = ACTIONS(1827),
    [sym_power_value] = ACTIONS(1827),
    [sym_sqrt_value] = ACTIONS(1827),
    [sym_raw_be] = ACTIONS(1827),
    [sym_raw_le] = ACTIONS(1827),
    [sym_internal_queue_size_set] = ACTIONS(1827),
    [sym_internal_queue_size_get] = ACTIONS(1827),
    [sym_internal_threshold_set] = ACTIONS(1827),
    [sym_internal_threshold_get] = ACTIONS(1827),
    [sym_rebuilt_event] = ACTIONS(1827),
    [sym_do_custom_event] = ACTIONS(1827),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1827),
    [sym_clkmgr_set_clk_source] = ACTIONS(1827),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1827),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1827),
    [sym_clkmgr_get_timezone] = ACTIONS(1827),
    [sym_clkmgr_set_timezone] = ACTIONS(1827),
    [sym_clkmgr_get_resync_period] = ACTIONS(1827),
    [sym_clkmgr_set_resync_period] = ACTIONS(1827),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1827),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1827),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1827),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1827),
    [sym_clkmgr_get_timeservers] = ACTIONS(1827),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1827),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1827),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1827),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1827),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1827),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1827),
    [sym_set_log_level] = ACTIONS(1827),
    [sym_get_log_level] = ACTIONS(1827),
    [sym_amx_log] = ACTIONS(1827),
    [sym_smtp_server_config_set] = ACTIONS(1827),
    [sym_smtp_server_config_get] = ACTIONS(1827),
    [sym_smtp_send] = ACTIONS(1827),
    [sym_validate_netlinx_account] = ACTIONS(1827),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1827),
    [sym_authenticate_certificate] = ACTIONS(1827),
    [sym_audit_netlinx_session_event] = ACTIONS(1827),
    [sym_audit_netlinx_generic_event] = ACTIONS(1827),
    [sym_get_available_flash_disk_space] = ACTIONS(1827),
    [sym_get_max_flash_disk_space] = ACTIONS(1827),
    [sym_data_create_feed] = ACTIONS(1827),
    [sym_data_delete_feed] = ACTIONS(1827),
    [sym_data_publish_feed] = ACTIONS(1827),
    [sym_data_get_published_feed] = ACTIONS(1827),
    [sym_data_add_record] = ACTIONS(1827),
    [sym_data_get_event_record] = ACTIONS(1827),
    [sym__wc_data_create_feed] = ACTIONS(1827),
    [sym__wc_data_add_record] = ACTIONS(1827),
    [sym__wc_data_get_event_record] = ACTIONS(1827),
    [sym_format] = ACTIONS(1827),
    [sym_astro_clock] = ACTIONS(1827),
    [sym_set_outdoor_temperature] = ACTIONS(1827),
    [sym_set_virtual_level_count] = ACTIONS(1827),
    [sym_set_virtual_channel_count] = ACTIONS(1827),
    [sym_set_virtual_port_count] = ACTIONS(1827),
    [sym__wc_encode_int] = ACTIONS(1827),
    [sym__wc_decode_int] = ACTIONS(1827),
    [sym__wc_int] = ACTIONS(1827),
    [sym__wc_to_ch_int] = ACTIONS(1827),
    [sym__ch_to_wc_int] = ACTIONS(1827),
    [sym__wc_find_string_int] = ACTIONS(1827),
    [sym__wc_left_string_int] = ACTIONS(1827),
    [sym__wc_lower_string_int] = ACTIONS(1827),
    [sym__wc_mid_string_int] = ACTIONS(1827),
    [sym__wc_remove_string_int] = ACTIONS(1827),
    [sym__wc_right_string_int] = ACTIONS(1827),
    [sym__wc_upper_string_int] = ACTIONS(1827),
    [sym__wc_compare_string_int] = ACTIONS(1827),
    [sym__wc_get_buffer_char_int] = ACTIONS(1827),
    [sym__wc_get_buffer_string_int] = ACTIONS(1827),
    [sym__wc_concat_string_int] = ACTIONS(1827),
    [sym__wc_file_open_int] = ACTIONS(1827),
    [sym__wc_file_close_int] = ACTIONS(1827),
    [sym__wc_file_read_int] = ACTIONS(1827),
    [sym__wc_file_read_line_int] = ACTIONS(1827),
    [sym__wc_file_write_int] = ACTIONS(1827),
    [sym__wc_file_write_line_int] = ACTIONS(1827),
    [aux_sym_preproc_include_token1] = ACTIONS(1827),
    [aux_sym_preproc_define_token1] = ACTIONS(1829),
    [aux_sym_preproc_warn_token1] = ACTIONS(1829),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1829),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1829),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1829),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1829),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [anon_sym_LBRACK] = ACTIONS(1829),
    [anon_sym_LBRACE] = ACTIONS(1829),
    [anon_sym_LPAREN] = ACTIONS(1827),
    [anon_sym_BANG] = ACTIONS(1829),
    [anon_sym_TILDE] = ACTIONS(1829),
    [anon_sym_DASH] = ACTIONS(1827),
    [anon_sym_PLUS] = ACTIONS(1827),
    [anon_sym_DASH_DASH] = ACTIONS(1829),
    [anon_sym_PLUS_PLUS] = ACTIONS(1829),
    [anon_sym_DQUOTE] = ACTIONS(1829),
    [anon_sym_SQUOTE] = ACTIONS(1829),
    [sym_number_literal] = ACTIONS(1829),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(253)] = {
    [sym_identifier] = ACTIONS(1831),
    [sym_program_name_keyword] = ACTIONS(1831),
    [sym_module_name_keyword] = ACTIONS(1831),
    [sym_define_device_keyword] = ACTIONS(1831),
    [sym_define_combine_keyword] = ACTIONS(1831),
    [sym_define_constant_keyword] = ACTIONS(1831),
    [sym_define_type_keyword] = ACTIONS(1831),
    [sym_define_variable_keyword] = ACTIONS(1831),
    [sym_define_system_variable_keyword] = ACTIONS(1831),
    [sym_define_connect_level_keyword] = ACTIONS(1831),
    [sym_define_latching_keyword] = ACTIONS(1831),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1831),
    [sym_define_toggling_keyword] = ACTIONS(1831),
    [sym_define_call_keyword] = ACTIONS(1831),
    [sym_define_function_keyword] = ACTIONS(1831),
    [sym_define_library_function_keyword] = ACTIONS(1831),
    [sym_define_start_keyword] = ACTIONS(1831),
    [sym_define_event_keyword] = ACTIONS(1831),
    [sym_define_program_keyword] = ACTIONS(1831),
    [sym_define_module_keyword] = ACTIONS(1831),
    [sym_char_keyword] = ACTIONS(1831),
    [sym_widechar_keyword] = ACTIONS(1831),
    [sym_integer_keyword] = ACTIONS(1831),
    [sym_sinteger_keyword] = ACTIONS(1831),
    [sym_long_keyword] = ACTIONS(1831),
    [sym_slong_keyword] = ACTIONS(1831),
    [sym_float_keyword] = ACTIONS(1831),
    [sym_double_keyword] = ACTIONS(1831),
    [sym_variant_keyword] = ACTIONS(1831),
    [sym_variantarray_keyword] = ACTIONS(1831),
    [sym_local_var_keyword] = ACTIONS(1831),
    [sym_stack_var_keyword] = ACTIONS(1831),
    [sym_constant_keyword] = ACTIONS(1831),
    [sym_volatile_keyword] = ACTIONS(1831),
    [sym_non_volatile_keyword] = ACTIONS(1831),
    [sym_persistent_keyword] = ACTIONS(1831),
    [sym_dev_keyword] = ACTIONS(1831),
    [sym_devlev_keyword] = ACTIONS(1831),
    [sym_devchan_keyword] = ACTIONS(1831),
    [sym_if_keyword] = ACTIONS(1831),
    [sym_else_keyword] = ACTIONS(1831),
    [sym_switch_keyword] = ACTIONS(1831),
    [sym_case_keyword] = ACTIONS(1831),
    [sym_default_keyword] = ACTIONS(1831),
    [sym_while_keyword] = ACTIONS(1831),
    [sym_for_keyword] = ACTIONS(1831),
    [sym_break_keyword] = ACTIONS(1831),
    [sym_continue_keyword] = ACTIONS(1831),
    [sym_return_keyword] = ACTIONS(1831),
    [sym_select_keyword] = ACTIONS(1831),
    [sym_active_keyword] = ACTIONS(1831),
    [sym_struct_keyword] = ACTIONS(1831),
    [sym_structure_keyword] = ACTIONS(1831),
    [sym_button_event_keyword] = ACTIONS(1831),
    [sym_channel_event_keyword] = ACTIONS(1831),
    [sym_level_event_keyword] = ACTIONS(1831),
    [sym_data_event_keyword] = ACTIONS(1831),
    [sym_timeline_event_keyword] = ACTIONS(1831),
    [sym_custom_event_keyword] = ACTIONS(1831),
    [sym_push_keyword] = ACTIONS(1831),
    [sym_release_keyword] = ACTIONS(1831),
    [aux_sym_on_keyword_token1] = ACTIONS(1831),
    [aux_sym_off_keyword_token1] = ACTIONS(1831),
    [sym_send_level_keyword] = ACTIONS(1831),
    [sym_send_string_keyword] = ACTIONS(1831),
    [sym_send_command_keyword] = ACTIONS(1831),
    [sym_clear_buffer_keyword] = ACTIONS(1831),
    [sym_create_buffer_keyword] = ACTIONS(1831),
    [sym_create_multi_buffer_keyword] = ACTIONS(1831),
    [sym_call_keyword] = ACTIONS(1831),
    [sym_system_call_keyword] = ACTIONS(1831),
    [sym_devchan_to_keyword] = ACTIONS(1831),
    [sym_devchan_min_to_keyword] = ACTIONS(1831),
    [sym_devchan_total_off_keyword] = ACTIONS(1831),
    [sym_devchan_pulse_keyword] = ACTIONS(1831),
    [sym_wait_keyword] = ACTIONS(1831),
    [sym_cancel_wait_keyword] = ACTIONS(1831),
    [sym_wait_until_keyword] = ACTIONS(1831),
    [sym_cancel_wait_until_keyword] = ACTIONS(1831),
    [sym_cancel_all_wait_keyword] = ACTIONS(1831),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1831),
    [sym_not] = ACTIONS(1831),
    [sym_bnot] = ACTIONS(1831),
    [sym___file__] = ACTIONS(1831),
    [sym___date__] = ACTIONS(1831),
    [sym___ldate__] = ACTIONS(1831),
    [sym___time__] = ACTIONS(1831),
    [sym___line__] = ACTIONS(1831),
    [sym___name__] = ACTIONS(1831),
    [sym___netlinx__] = ACTIONS(1831),
    [sym_netlinx_axi_version] = ACTIONS(1831),
    [sym_url_flg_tcp] = ACTIONS(1831),
    [sym_url_flg_accinfopresent] = ACTIONS(1831),
    [sym_url_flg_temp] = ACTIONS(1831),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1831),
    [sym_url_flg_stat_mask] = ACTIONS(1831),
    [sym_url_flg_stat_lookup] = ACTIONS(1831),
    [sym_url_flg_stat_connecting] = ACTIONS(1831),
    [sym_url_flg_stat_waiting] = ACTIONS(1831),
    [sym_url_flg_stat_connected] = ACTIONS(1831),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1831),
    [sym_first_virtual_device] = ACTIONS(1831),
    [sym_first_local_port] = ACTIONS(1831),
    [sym_dynamic_virtual_device] = ACTIONS(1831),
    [sym_timeline_once] = ACTIONS(1831),
    [sym_timeline_repeat] = ACTIONS(1831),
    [sym_timeline_absolute] = ACTIONS(1831),
    [sym_timeline_relative] = ACTIONS(1831),
    [sym_do_push_timed_infinite] = ACTIONS(1831),
    [sym_source_type_no_address] = ACTIONS(1831),
    [sym_source_type_neuron_id] = ACTIONS(1831),
    [sym_source_type_ip_address] = ACTIONS(1831),
    [sym_source_type_axlink] = ACTIONS(1831),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1831),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1831),
    [sym_source_type_ip_socket_address] = ACTIONS(1831),
    [sym_source_type_rs232] = ACTIONS(1831),
    [sym_source_type_internal] = ACTIONS(1831),
    [sym_source_type_mac_address] = ACTIONS(1831),
    [sym_source_type_ipv4_port] = ACTIONS(1831),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1831),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1831),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1831),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1831),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1831),
    [sym_true] = ACTIONS(1831),
    [sym_false] = ACTIONS(1831),
    [sym_file_read_only] = ACTIONS(1831),
    [sym_file_rw_new] = ACTIONS(1831),
    [sym_file_rw_append] = ACTIONS(1831),
    [sym_ip_tcp] = ACTIONS(1831),
    [sym_ip_udp] = ACTIONS(1831),
    [sym_ip_udp_2way] = ACTIONS(1831),
    [sym_xml_encode_types] = ACTIONS(1831),
    [sym_xml_encode_char_as_list] = ACTIONS(1831),
    [sym_xml_encode_le] = ACTIONS(1831),
    [sym_xml_decode_types] = ACTIONS(1831),
    [sym_xml_decode_no_preserve] = ACTIONS(1831),
    [sym_duet_dev_type_display_device] = ACTIONS(1831),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1831),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1831),
    [sym_duet_dev_type_io_device] = ACTIONS(1831),
    [sym_duet_dev_type_relay_device] = ACTIONS(1831),
    [sym_duet_dev_type_ups] = ACTIONS(1831),
    [sym_duet_dev_type_amplifier] = ACTIONS(1831),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1831),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1831),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1831),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1831),
    [sym_duet_dev_type_camera] = ACTIONS(1831),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1831),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1831),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1831),
    [sym_duet_dev_type_dss] = ACTIONS(1831),
    [sym_duet_dev_type_dvr] = ACTIONS(1831),
    [sym_duet_dev_type_disc_device] = ACTIONS(1831),
    [sym_duet_dev_type_document_camera] = ACTIONS(1831),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1831),
    [sym_duet_dev_type_hvac] = ACTIONS(1831),
    [sym_duet_dev_type_keypad] = ACTIONS(1831),
    [sym_duet_dev_type_light] = ACTIONS(1831),
    [sym_duet_dev_type_monitor] = ACTIONS(1831),
    [sym_duet_dev_type_motor] = ACTIONS(1831),
    [sym_duet_dev_type_multi_window] = ACTIONS(1831),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1831),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1831),
    [sym_duet_dev_type_receiver] = ACTIONS(1831),
    [sym_duet_dev_type_security_system] = ACTIONS(1831),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1831),
    [sym_duet_dev_type_settop_box] = ACTIONS(1831),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1831),
    [sym_duet_dev_type_switcher] = ACTIONS(1831),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1831),
    [sym_duet_dev_type_tv] = ACTIONS(1831),
    [sym_duet_dev_type_utility] = ACTIONS(1831),
    [sym_duet_dev_type_vcr] = ACTIONS(1831),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1831),
    [sym_duet_dev_type_video_processor] = ACTIONS(1831),
    [sym_duet_dev_type_video_projector] = ACTIONS(1831),
    [sym_duet_dev_type_video_wall] = ACTIONS(1831),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1831),
    [sym_duet_dev_type_weather] = ACTIONS(1831),
    [sym_duet_dev_not_polled] = ACTIONS(1831),
    [sym_duet_dev_polled] = ACTIONS(1831),
    [sym_clkmgr_mode_network] = ACTIONS(1831),
    [sym_clkmgr_mode_standalone] = ACTIONS(1831),
    [sym_normal_standby] = ACTIONS(1831),
    [sym_normal_wake] = ACTIONS(1831),
    [sym_ip_multicast_ttl_option] = ACTIONS(1831),
    [sym_ip_ttl_subnet] = ACTIONS(1831),
    [sym_ip_ttl_site] = ACTIONS(1831),
    [sym_ip_ttl_region] = ACTIONS(1831),
    [sym_ip_ttl_continent] = ACTIONS(1831),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1831),
    [sym_ip_nodelay_on] = ACTIONS(1831),
    [sym_ip_nodelay_off] = ACTIONS(1831),
    [sym_amx_error] = ACTIONS(1831),
    [sym_amx_warning] = ACTIONS(1831),
    [sym_amx_info] = ACTIONS(1831),
    [sym_amx_debug] = ACTIONS(1831),
    [sym_netlinx_login_success] = ACTIONS(1831),
    [sym_netlinx_login_fail] = ACTIONS(1831),
    [sym_netlinx_logout] = ACTIONS(1831),
    [sym_null_str] = ACTIONS(1831),
    [sym_smtp_address] = ACTIONS(1831),
    [sym_smtp_port_number] = ACTIONS(1831),
    [sym_smtp_username] = ACTIONS(1831),
    [sym_smtp_password] = ACTIONS(1831),
    [sym_smtp_require_tls] = ACTIONS(1831),
    [sym_smtp_from] = ACTIONS(1831),
    [sym_smtp_tls_true] = ACTIONS(1831),
    [sym_smtp_tls_false] = ACTIONS(1831),
    [sym_listview_on_row_select_event] = ACTIONS(1831),
    [sym_data_structure_unknown] = ACTIONS(1831),
    [sym_data_structure_datafeed] = ACTIONS(1831),
    [sym_data_structure_datarecordset] = ACTIONS(1831),
    [sym_data_structure_datarecord] = ACTIONS(1831),
    [sym_data_structure_datafield] = ACTIONS(1831),
    [sym_data_max_content_field_count] = ACTIONS(1831),
    [sym_data_max_metadata_field_count] = ACTIONS(1831),
    [sym_data_max_id_length] = ACTIONS(1831),
    [sym_data_max_name_length] = ACTIONS(1831),
    [sym_data_max_description_length] = ACTIONS(1831),
    [sym_data_max_type_length] = ACTIONS(1831),
    [sym_data_max_format_length] = ACTIONS(1831),
    [sym_data_max_source_length] = ACTIONS(1831),
    [sym_data_max_value_length] = ACTIONS(1831),
    [sym_data_max_label_length] = ACTIONS(1831),
    [sym_data_max_url_length] = ACTIONS(1831),
    [sym_max_last_login_info_length] = ACTIONS(1831),
    [sym_data_type_unknown] = ACTIONS(1831),
    [sym_data_type_string] = ACTIONS(1831),
    [sym_data_type_datetime] = ACTIONS(1831),
    [sym_data_type_date] = ACTIONS(1831),
    [sym_data_type_time] = ACTIONS(1831),
    [sym_data_type_image] = ACTIONS(1831),
    [sym_data_format_url] = ACTIONS(1831),
    [sym_data_format_phone] = ACTIONS(1831),
    [sym_data_format_email] = ACTIONS(1831),
    [sym_data_format_iso8601] = ACTIONS(1831),
    [sym_auditlog_priv] = ACTIONS(1831),
    [sym_configuration_priv] = ACTIONS(1831),
    [sym_deviceconfig_priv] = ACTIONS(1831),
    [sym_ftp_priv] = ACTIONS(1831),
    [sym_http_priv] = ACTIONS(1831),
    [sym_networkconfig_priv] = ACTIONS(1831),
    [sym_programport_priv] = ACTIONS(1831),
    [sym_remoteui_priv] = ACTIONS(1831),
    [sym_securitycontrol_priv] = ACTIONS(1831),
    [sym_softwaremanagement_priv] = ACTIONS(1831),
    [sym_terminal_priv] = ACTIONS(1831),
    [sym_tpadmin_priv] = ACTIONS(1831),
    [sym_user1_priv] = ACTIONS(1831),
    [sym_user2_priv] = ACTIONS(1831),
    [sym_user3_priv] = ACTIONS(1831),
    [sym_user4_priv] = ACTIONS(1831),
    [sym_usermanagement_priv] = ACTIONS(1831),
    [sym_valid_account] = ACTIONS(1831),
    [sym_err_name_invalid] = ACTIONS(1831),
    [sym_err_password_invalid] = ACTIONS(1831),
    [sym_err_invalid_account] = ACTIONS(1831),
    [sym_err_no_authorization] = ACTIONS(1831),
    [sym_err_invalid_parameter] = ACTIONS(1831),
    [sym_err_user_locked_out] = ACTIONS(1831),
    [sym_err_user_account_expired] = ACTIONS(1831),
    [sym_err_general_error] = ACTIONS(1831),
    [sym_err_authentication_error] = ACTIONS(1831),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1831),
    [sym_tls_validate_certificate] = ACTIONS(1831),
    [sym_tdata] = ACTIONS(1831),
    [sym_tchannel] = ACTIONS(1831),
    [sym_tlevel] = ACTIONS(1831),
    [sym_tbutton] = ACTIONS(1831),
    [sym_ttimeline] = ACTIONS(1831),
    [sym_tcustom] = ACTIONS(1831),
    [sym_url_struct] = ACTIONS(1831),
    [sym_dns_struct] = ACTIONS(1831),
    [sym_ip_address_struct] = ACTIONS(1831),
    [sym_dev_info_struct] = ACTIONS(1831),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1831),
    [sym_data_feed] = ACTIONS(1831),
    [sym_data_field] = ACTIONS(1831),
    [sym_data_record] = ACTIONS(1831),
    [sym_wc_data_feed] = ACTIONS(1831),
    [sym_wc_data_field] = ACTIONS(1831),
    [sym_wc_data_record] = ACTIONS(1831),
    [sym_last_login_info] = ACTIONS(1831),
    [sym____reserved___] = ACTIONS(1831),
    [sym_date] = ACTIONS(1831),
    [sym_day] = ACTIONS(1831),
    [sym_time] = ACTIONS(1831),
    [sym_ldate] = ACTIONS(1831),
    [sym_push_device] = ACTIONS(1831),
    [sym_push_channel] = ACTIONS(1831),
    [sym_push_devchan] = ACTIONS(1831),
    [sym_release_device] = ACTIONS(1831),
    [sym_release_channel] = ACTIONS(1831),
    [sym_release_devchan] = ACTIONS(1831),
    [sym_master_slot] = ACTIONS(1831),
    [sym_get_pulse_time] = ACTIONS(1831),
    [sym_get_timer] = ACTIONS(1831),
    [sym_master_sn] = ACTIONS(1831),
    [sym_system_number] = ACTIONS(1831),
    [sym_button] = ACTIONS(1831),
    [sym_dv_channel] = ACTIONS(1831),
    [sym_data] = ACTIONS(1831),
    [sym_level] = ACTIONS(1831),
    [sym_timeline] = ACTIONS(1831),
    [sym_channel] = ACTIONS(1831),
    [sym_custom] = ACTIONS(1831),
    [sym_atoi] = ACTIONS(1831),
    [sym_atol] = ACTIONS(1831),
    [sym_atof] = ACTIONS(1831),
    [sym_itoa] = ACTIONS(1831),
    [sym_itohex] = ACTIONS(1831),
    [sym_hextoi] = ACTIONS(1831),
    [sym_ftoa] = ACTIONS(1831),
    [sym_find_string] = ACTIONS(1831),
    [sym_left_string] = ACTIONS(1831),
    [sym_right_string] = ACTIONS(1831),
    [sym_mid_string] = ACTIONS(1831),
    [sym_remove_string] = ACTIONS(1831),
    [sym_get_buffer_string] = ACTIONS(1831),
    [sym_get_buffer_char] = ACTIONS(1831),
    [sym_get_multi_buffer_string] = ACTIONS(1831),
    [sym_length_string] = ACTIONS(1831),
    [sym_max_length_string] = ACTIONS(1831),
    [sym_set_length_string] = ACTIONS(1831),
    [sym_compare_string] = ACTIONS(1831),
    [sym_upper_string] = ACTIONS(1831),
    [sym_lower_string] = ACTIONS(1831),
    [sym_get_last] = ACTIONS(1831),
    [sym_date_to_day] = ACTIONS(1831),
    [sym_date_to_month] = ACTIONS(1831),
    [sym_date_to_year] = ACTIONS(1831),
    [sym_time_to_hour] = ACTIONS(1831),
    [sym_time_to_minute] = ACTIONS(1831),
    [sym_time_to_second] = ACTIONS(1831),
    [sym_day_of_week] = ACTIONS(1831),
    [sym_set_pulse_time] = ACTIONS(1831),
    [sym_set_timer] = ACTIONS(1831),
    [sym_file_open] = ACTIONS(1831),
    [sym_file_close] = ACTIONS(1831),
    [sym_file_read] = ACTIONS(1831),
    [sym_file_read_line] = ACTIONS(1831),
    [sym_file_write] = ACTIONS(1831),
    [sym_file_write_line] = ACTIONS(1831),
    [sym_file_copy] = ACTIONS(1831),
    [sym_file_delete] = ACTIONS(1831),
    [sym_file_rename] = ACTIONS(1831),
    [sym_file_dir] = ACTIONS(1831),
    [sym_file_getdir] = ACTIONS(1831),
    [sym_file_setdir] = ACTIONS(1831),
    [sym_file_create_dir] = ACTIONS(1831),
    [sym_file_remove_dir] = ACTIONS(1831),
    [sym_file_seek] = ACTIONS(1831),
    [sym_length_array] = ACTIONS(1831),
    [sym_max_length_array] = ACTIONS(1831),
    [sym_set_length_array] = ACTIONS(1831),
    [sym_ip_client_open] = ACTIONS(1831),
    [sym_ip_client_close] = ACTIONS(1831),
    [sym_ip_server_open] = ACTIONS(1831),
    [sym_ip_server_close] = ACTIONS(1831),
    [sym_ip_bound_client_open] = ACTIONS(1831),
    [sym_ip_mc_server_open] = ACTIONS(1831),
    [sym_ip_set_option] = ACTIONS(1831),
    [sym_ssh_client_open] = ACTIONS(1831),
    [sym_ssh_client_close] = ACTIONS(1831),
    [sym_tls_client_open] = ACTIONS(1831),
    [sym_tls_client_close] = ACTIONS(1831),
    [sym_do_push] = ACTIONS(1831),
    [sym_do_push_timed] = ACTIONS(1831),
    [sym_do_release] = ACTIONS(1831),
    [sym_device_id] = ACTIONS(1831),
    [sym_device_info] = ACTIONS(1831),
    [sym_device_id_string] = ACTIONS(1831),
    [sym_device_standby] = ACTIONS(1831),
    [sym_device_wake] = ACTIONS(1831),
    [sym_timeline_create] = ACTIONS(1831),
    [sym_timeline_kill] = ACTIONS(1831),
    [sym_timeline_reload] = ACTIONS(1831),
    [sym_timeline_pause] = ACTIONS(1831),
    [sym_timeline_restart] = ACTIONS(1831),
    [sym_timeline_set] = ACTIONS(1831),
    [sym_timeline_get] = ACTIONS(1831),
    [sym_timeline_active] = ACTIONS(1831),
    [sym_timeline_dynamic_id] = ACTIONS(1831),
    [sym_variable_to_string] = ACTIONS(1831),
    [sym_string_to_variable] = ACTIONS(1831),
    [sym_length_variable_to_string] = ACTIONS(1831),
    [sym_type_cast] = ACTIONS(1831),
    [sym_variable_to_xml] = ACTIONS(1831),
    [sym_xml_to_variable] = ACTIONS(1831),
    [sym_length_variable_to_xml] = ACTIONS(1831),
    [sym_redirect_string] = ACTIONS(1831),
    [sym_random_number] = ACTIONS(1831),
    [sym_get_unique_id] = ACTIONS(1831),
    [sym_get_system_number] = ACTIONS(1831),
    [sym_set_system_number] = ACTIONS(1831),
    [sym_get_serial_number] = ACTIONS(1831),
    [sym_get_master_build] = ACTIONS(1831),
    [sym_set_validation_code] = ACTIONS(1831),
    [sym_reboot] = ACTIONS(1831),
    [sym_load_duet_module] = ACTIONS(1831),
    [sym_unload_duet_module] = ACTIONS(1831),
    [sym_duet_mem_size_set] = ACTIONS(1831),
    [sym_duet_mem_size_get] = ACTIONS(1831),
    [sym_dynamic_application_device] = ACTIONS(1831),
    [sym_dynamic_polled_port] = ACTIONS(1831),
    [sym_static_port_binding] = ACTIONS(1831),
    [sym_static_ip_binding] = ACTIONS(1831),
    [sym_get_dns_list] = ACTIONS(1831),
    [sym_set_dns_list] = ACTIONS(1831),
    [sym_get_ip_address] = ACTIONS(1831),
    [sym_set_ip_address] = ACTIONS(1831),
    [sym_get_url_list] = ACTIONS(1831),
    [sym_add_url_entry] = ACTIONS(1831),
    [sym_delete_url_entry] = ACTIONS(1831),
    [sym_abs_value] = ACTIONS(1831),
    [sym_max_value] = ACTIONS(1831),
    [sym_min_value] = ACTIONS(1831),
    [sym_exp_value] = ACTIONS(1831),
    [sym_log_value] = ACTIONS(1831),
    [sym_log10_value] = ACTIONS(1831),
    [sym_power_value] = ACTIONS(1831),
    [sym_sqrt_value] = ACTIONS(1831),
    [sym_raw_be] = ACTIONS(1831),
    [sym_raw_le] = ACTIONS(1831),
    [sym_internal_queue_size_set] = ACTIONS(1831),
    [sym_internal_queue_size_get] = ACTIONS(1831),
    [sym_internal_threshold_set] = ACTIONS(1831),
    [sym_internal_threshold_get] = ACTIONS(1831),
    [sym_rebuilt_event] = ACTIONS(1831),
    [sym_do_custom_event] = ACTIONS(1831),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1831),
    [sym_clkmgr_set_clk_source] = ACTIONS(1831),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1831),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1831),
    [sym_clkmgr_get_timezone] = ACTIONS(1831),
    [sym_clkmgr_set_timezone] = ACTIONS(1831),
    [sym_clkmgr_get_resync_period] = ACTIONS(1831),
    [sym_clkmgr_set_resync_period] = ACTIONS(1831),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1831),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1831),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1831),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1831),
    [sym_clkmgr_get_timeservers] = ACTIONS(1831),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1831),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1831),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1831),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1831),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1831),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1831),
    [sym_set_log_level] = ACTIONS(1831),
    [sym_get_log_level] = ACTIONS(1831),
    [sym_amx_log] = ACTIONS(1831),
    [sym_smtp_server_config_set] = ACTIONS(1831),
    [sym_smtp_server_config_get] = ACTIONS(1831),
    [sym_smtp_send] = ACTIONS(1831),
    [sym_validate_netlinx_account] = ACTIONS(1831),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1831),
    [sym_authenticate_certificate] = ACTIONS(1831),
    [sym_audit_netlinx_session_event] = ACTIONS(1831),
    [sym_audit_netlinx_generic_event] = ACTIONS(1831),
    [sym_get_available_flash_disk_space] = ACTIONS(1831),
    [sym_get_max_flash_disk_space] = ACTIONS(1831),
    [sym_data_create_feed] = ACTIONS(1831),
    [sym_data_delete_feed] = ACTIONS(1831),
    [sym_data_publish_feed] = ACTIONS(1831),
    [sym_data_get_published_feed] = ACTIONS(1831),
    [sym_data_add_record] = ACTIONS(1831),
    [sym_data_get_event_record] = ACTIONS(1831),
    [sym__wc_data_create_feed] = ACTIONS(1831),
    [sym__wc_data_add_record] = ACTIONS(1831),
    [sym__wc_data_get_event_record] = ACTIONS(1831),
    [sym_format] = ACTIONS(1831),
    [sym_astro_clock] = ACTIONS(1831),
    [sym_set_outdoor_temperature] = ACTIONS(1831),
    [sym_set_virtual_level_count] = ACTIONS(1831),
    [sym_set_virtual_channel_count] = ACTIONS(1831),
    [sym_set_virtual_port_count] = ACTIONS(1831),
    [sym__wc_encode_int] = ACTIONS(1831),
    [sym__wc_decode_int] = ACTIONS(1831),
    [sym__wc_int] = ACTIONS(1831),
    [sym__wc_to_ch_int] = ACTIONS(1831),
    [sym__ch_to_wc_int] = ACTIONS(1831),
    [sym__wc_find_string_int] = ACTIONS(1831),
    [sym__wc_left_string_int] = ACTIONS(1831),
    [sym__wc_lower_string_int] = ACTIONS(1831),
    [sym__wc_mid_string_int] = ACTIONS(1831),
    [sym__wc_remove_string_int] = ACTIONS(1831),
    [sym__wc_right_string_int] = ACTIONS(1831),
    [sym__wc_upper_string_int] = ACTIONS(1831),
    [sym__wc_compare_string_int] = ACTIONS(1831),
    [sym__wc_get_buffer_char_int] = ACTIONS(1831),
    [sym__wc_get_buffer_string_int] = ACTIONS(1831),
    [sym__wc_concat_string_int] = ACTIONS(1831),
    [sym__wc_file_open_int] = ACTIONS(1831),
    [sym__wc_file_close_int] = ACTIONS(1831),
    [sym__wc_file_read_int] = ACTIONS(1831),
    [sym__wc_file_read_line_int] = ACTIONS(1831),
    [sym__wc_file_write_int] = ACTIONS(1831),
    [sym__wc_file_write_line_int] = ACTIONS(1831),
    [aux_sym_preproc_include_token1] = ACTIONS(1831),
    [aux_sym_preproc_define_token1] = ACTIONS(1833),
    [aux_sym_preproc_warn_token1] = ACTIONS(1833),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1833),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1833),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1833),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1833),
    [aux_sym_preproc_else_token1] = ACTIONS(1833),
    [anon_sym_LBRACK] = ACTIONS(1833),
    [anon_sym_LBRACE] = ACTIONS(1833),
    [anon_sym_LPAREN] = ACTIONS(1831),
    [anon_sym_BANG] = ACTIONS(1833),
    [anon_sym_TILDE] = ACTIONS(1833),
    [anon_sym_DASH] = ACTIONS(1831),
    [anon_sym_PLUS] = ACTIONS(1831),
    [anon_sym_DASH_DASH] = ACTIONS(1833),
    [anon_sym_PLUS_PLUS] = ACTIONS(1833),
    [anon_sym_DQUOTE] = ACTIONS(1833),
    [anon_sym_SQUOTE] = ACTIONS(1833),
    [sym_number_literal] = ACTIONS(1833),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(254)] = {
    [sym_identifier] = ACTIONS(1835),
    [sym_program_name_keyword] = ACTIONS(1835),
    [sym_module_name_keyword] = ACTIONS(1835),
    [sym_define_device_keyword] = ACTIONS(1835),
    [sym_define_combine_keyword] = ACTIONS(1835),
    [sym_define_constant_keyword] = ACTIONS(1835),
    [sym_define_type_keyword] = ACTIONS(1835),
    [sym_define_variable_keyword] = ACTIONS(1835),
    [sym_define_system_variable_keyword] = ACTIONS(1835),
    [sym_define_connect_level_keyword] = ACTIONS(1835),
    [sym_define_latching_keyword] = ACTIONS(1835),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1835),
    [sym_define_toggling_keyword] = ACTIONS(1835),
    [sym_define_call_keyword] = ACTIONS(1835),
    [sym_define_function_keyword] = ACTIONS(1835),
    [sym_define_library_function_keyword] = ACTIONS(1835),
    [sym_define_start_keyword] = ACTIONS(1835),
    [sym_define_event_keyword] = ACTIONS(1835),
    [sym_define_program_keyword] = ACTIONS(1835),
    [sym_define_module_keyword] = ACTIONS(1835),
    [sym_char_keyword] = ACTIONS(1835),
    [sym_widechar_keyword] = ACTIONS(1835),
    [sym_integer_keyword] = ACTIONS(1835),
    [sym_sinteger_keyword] = ACTIONS(1835),
    [sym_long_keyword] = ACTIONS(1835),
    [sym_slong_keyword] = ACTIONS(1835),
    [sym_float_keyword] = ACTIONS(1835),
    [sym_double_keyword] = ACTIONS(1835),
    [sym_variant_keyword] = ACTIONS(1835),
    [sym_variantarray_keyword] = ACTIONS(1835),
    [sym_local_var_keyword] = ACTIONS(1835),
    [sym_stack_var_keyword] = ACTIONS(1835),
    [sym_constant_keyword] = ACTIONS(1835),
    [sym_volatile_keyword] = ACTIONS(1835),
    [sym_non_volatile_keyword] = ACTIONS(1835),
    [sym_persistent_keyword] = ACTIONS(1835),
    [sym_dev_keyword] = ACTIONS(1835),
    [sym_devlev_keyword] = ACTIONS(1835),
    [sym_devchan_keyword] = ACTIONS(1835),
    [sym_if_keyword] = ACTIONS(1835),
    [sym_else_keyword] = ACTIONS(1835),
    [sym_switch_keyword] = ACTIONS(1835),
    [sym_case_keyword] = ACTIONS(1835),
    [sym_default_keyword] = ACTIONS(1835),
    [sym_while_keyword] = ACTIONS(1835),
    [sym_for_keyword] = ACTIONS(1835),
    [sym_break_keyword] = ACTIONS(1835),
    [sym_continue_keyword] = ACTIONS(1835),
    [sym_return_keyword] = ACTIONS(1835),
    [sym_select_keyword] = ACTIONS(1835),
    [sym_active_keyword] = ACTIONS(1835),
    [sym_struct_keyword] = ACTIONS(1835),
    [sym_structure_keyword] = ACTIONS(1835),
    [sym_button_event_keyword] = ACTIONS(1835),
    [sym_channel_event_keyword] = ACTIONS(1835),
    [sym_level_event_keyword] = ACTIONS(1835),
    [sym_data_event_keyword] = ACTIONS(1835),
    [sym_timeline_event_keyword] = ACTIONS(1835),
    [sym_custom_event_keyword] = ACTIONS(1835),
    [sym_push_keyword] = ACTIONS(1835),
    [sym_release_keyword] = ACTIONS(1835),
    [aux_sym_on_keyword_token1] = ACTIONS(1835),
    [aux_sym_off_keyword_token1] = ACTIONS(1835),
    [sym_send_level_keyword] = ACTIONS(1835),
    [sym_send_string_keyword] = ACTIONS(1835),
    [sym_send_command_keyword] = ACTIONS(1835),
    [sym_clear_buffer_keyword] = ACTIONS(1835),
    [sym_create_buffer_keyword] = ACTIONS(1835),
    [sym_create_multi_buffer_keyword] = ACTIONS(1835),
    [sym_call_keyword] = ACTIONS(1835),
    [sym_system_call_keyword] = ACTIONS(1835),
    [sym_devchan_to_keyword] = ACTIONS(1835),
    [sym_devchan_min_to_keyword] = ACTIONS(1835),
    [sym_devchan_total_off_keyword] = ACTIONS(1835),
    [sym_devchan_pulse_keyword] = ACTIONS(1835),
    [sym_wait_keyword] = ACTIONS(1835),
    [sym_cancel_wait_keyword] = ACTIONS(1835),
    [sym_wait_until_keyword] = ACTIONS(1835),
    [sym_cancel_wait_until_keyword] = ACTIONS(1835),
    [sym_cancel_all_wait_keyword] = ACTIONS(1835),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1835),
    [sym_not] = ACTIONS(1835),
    [sym_bnot] = ACTIONS(1835),
    [sym___file__] = ACTIONS(1835),
    [sym___date__] = ACTIONS(1835),
    [sym___ldate__] = ACTIONS(1835),
    [sym___time__] = ACTIONS(1835),
    [sym___line__] = ACTIONS(1835),
    [sym___name__] = ACTIONS(1835),
    [sym___netlinx__] = ACTIONS(1835),
    [sym_netlinx_axi_version] = ACTIONS(1835),
    [sym_url_flg_tcp] = ACTIONS(1835),
    [sym_url_flg_accinfopresent] = ACTIONS(1835),
    [sym_url_flg_temp] = ACTIONS(1835),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1835),
    [sym_url_flg_stat_mask] = ACTIONS(1835),
    [sym_url_flg_stat_lookup] = ACTIONS(1835),
    [sym_url_flg_stat_connecting] = ACTIONS(1835),
    [sym_url_flg_stat_waiting] = ACTIONS(1835),
    [sym_url_flg_stat_connected] = ACTIONS(1835),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1835),
    [sym_first_virtual_device] = ACTIONS(1835),
    [sym_first_local_port] = ACTIONS(1835),
    [sym_dynamic_virtual_device] = ACTIONS(1835),
    [sym_timeline_once] = ACTIONS(1835),
    [sym_timeline_repeat] = ACTIONS(1835),
    [sym_timeline_absolute] = ACTIONS(1835),
    [sym_timeline_relative] = ACTIONS(1835),
    [sym_do_push_timed_infinite] = ACTIONS(1835),
    [sym_source_type_no_address] = ACTIONS(1835),
    [sym_source_type_neuron_id] = ACTIONS(1835),
    [sym_source_type_ip_address] = ACTIONS(1835),
    [sym_source_type_axlink] = ACTIONS(1835),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1835),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1835),
    [sym_source_type_ip_socket_address] = ACTIONS(1835),
    [sym_source_type_rs232] = ACTIONS(1835),
    [sym_source_type_internal] = ACTIONS(1835),
    [sym_source_type_mac_address] = ACTIONS(1835),
    [sym_source_type_ipv4_port] = ACTIONS(1835),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1835),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1835),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1835),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1835),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1835),
    [sym_true] = ACTIONS(1835),
    [sym_false] = ACTIONS(1835),
    [sym_file_read_only] = ACTIONS(1835),
    [sym_file_rw_new] = ACTIONS(1835),
    [sym_file_rw_append] = ACTIONS(1835),
    [sym_ip_tcp] = ACTIONS(1835),
    [sym_ip_udp] = ACTIONS(1835),
    [sym_ip_udp_2way] = ACTIONS(1835),
    [sym_xml_encode_types] = ACTIONS(1835),
    [sym_xml_encode_char_as_list] = ACTIONS(1835),
    [sym_xml_encode_le] = ACTIONS(1835),
    [sym_xml_decode_types] = ACTIONS(1835),
    [sym_xml_decode_no_preserve] = ACTIONS(1835),
    [sym_duet_dev_type_display_device] = ACTIONS(1835),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1835),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1835),
    [sym_duet_dev_type_io_device] = ACTIONS(1835),
    [sym_duet_dev_type_relay_device] = ACTIONS(1835),
    [sym_duet_dev_type_ups] = ACTIONS(1835),
    [sym_duet_dev_type_amplifier] = ACTIONS(1835),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1835),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1835),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1835),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1835),
    [sym_duet_dev_type_camera] = ACTIONS(1835),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1835),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1835),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1835),
    [sym_duet_dev_type_dss] = ACTIONS(1835),
    [sym_duet_dev_type_dvr] = ACTIONS(1835),
    [sym_duet_dev_type_disc_device] = ACTIONS(1835),
    [sym_duet_dev_type_document_camera] = ACTIONS(1835),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1835),
    [sym_duet_dev_type_hvac] = ACTIONS(1835),
    [sym_duet_dev_type_keypad] = ACTIONS(1835),
    [sym_duet_dev_type_light] = ACTIONS(1835),
    [sym_duet_dev_type_monitor] = ACTIONS(1835),
    [sym_duet_dev_type_motor] = ACTIONS(1835),
    [sym_duet_dev_type_multi_window] = ACTIONS(1835),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1835),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1835),
    [sym_duet_dev_type_receiver] = ACTIONS(1835),
    [sym_duet_dev_type_security_system] = ACTIONS(1835),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1835),
    [sym_duet_dev_type_settop_box] = ACTIONS(1835),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1835),
    [sym_duet_dev_type_switcher] = ACTIONS(1835),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1835),
    [sym_duet_dev_type_tv] = ACTIONS(1835),
    [sym_duet_dev_type_utility] = ACTIONS(1835),
    [sym_duet_dev_type_vcr] = ACTIONS(1835),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1835),
    [sym_duet_dev_type_video_processor] = ACTIONS(1835),
    [sym_duet_dev_type_video_projector] = ACTIONS(1835),
    [sym_duet_dev_type_video_wall] = ACTIONS(1835),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1835),
    [sym_duet_dev_type_weather] = ACTIONS(1835),
    [sym_duet_dev_not_polled] = ACTIONS(1835),
    [sym_duet_dev_polled] = ACTIONS(1835),
    [sym_clkmgr_mode_network] = ACTIONS(1835),
    [sym_clkmgr_mode_standalone] = ACTIONS(1835),
    [sym_normal_standby] = ACTIONS(1835),
    [sym_normal_wake] = ACTIONS(1835),
    [sym_ip_multicast_ttl_option] = ACTIONS(1835),
    [sym_ip_ttl_subnet] = ACTIONS(1835),
    [sym_ip_ttl_site] = ACTIONS(1835),
    [sym_ip_ttl_region] = ACTIONS(1835),
    [sym_ip_ttl_continent] = ACTIONS(1835),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1835),
    [sym_ip_nodelay_on] = ACTIONS(1835),
    [sym_ip_nodelay_off] = ACTIONS(1835),
    [sym_amx_error] = ACTIONS(1835),
    [sym_amx_warning] = ACTIONS(1835),
    [sym_amx_info] = ACTIONS(1835),
    [sym_amx_debug] = ACTIONS(1835),
    [sym_netlinx_login_success] = ACTIONS(1835),
    [sym_netlinx_login_fail] = ACTIONS(1835),
    [sym_netlinx_logout] = ACTIONS(1835),
    [sym_null_str] = ACTIONS(1835),
    [sym_smtp_address] = ACTIONS(1835),
    [sym_smtp_port_number] = ACTIONS(1835),
    [sym_smtp_username] = ACTIONS(1835),
    [sym_smtp_password] = ACTIONS(1835),
    [sym_smtp_require_tls] = ACTIONS(1835),
    [sym_smtp_from] = ACTIONS(1835),
    [sym_smtp_tls_true] = ACTIONS(1835),
    [sym_smtp_tls_false] = ACTIONS(1835),
    [sym_listview_on_row_select_event] = ACTIONS(1835),
    [sym_data_structure_unknown] = ACTIONS(1835),
    [sym_data_structure_datafeed] = ACTIONS(1835),
    [sym_data_structure_datarecordset] = ACTIONS(1835),
    [sym_data_structure_datarecord] = ACTIONS(1835),
    [sym_data_structure_datafield] = ACTIONS(1835),
    [sym_data_max_content_field_count] = ACTIONS(1835),
    [sym_data_max_metadata_field_count] = ACTIONS(1835),
    [sym_data_max_id_length] = ACTIONS(1835),
    [sym_data_max_name_length] = ACTIONS(1835),
    [sym_data_max_description_length] = ACTIONS(1835),
    [sym_data_max_type_length] = ACTIONS(1835),
    [sym_data_max_format_length] = ACTIONS(1835),
    [sym_data_max_source_length] = ACTIONS(1835),
    [sym_data_max_value_length] = ACTIONS(1835),
    [sym_data_max_label_length] = ACTIONS(1835),
    [sym_data_max_url_length] = ACTIONS(1835),
    [sym_max_last_login_info_length] = ACTIONS(1835),
    [sym_data_type_unknown] = ACTIONS(1835),
    [sym_data_type_string] = ACTIONS(1835),
    [sym_data_type_datetime] = ACTIONS(1835),
    [sym_data_type_date] = ACTIONS(1835),
    [sym_data_type_time] = ACTIONS(1835),
    [sym_data_type_image] = ACTIONS(1835),
    [sym_data_format_url] = ACTIONS(1835),
    [sym_data_format_phone] = ACTIONS(1835),
    [sym_data_format_email] = ACTIONS(1835),
    [sym_data_format_iso8601] = ACTIONS(1835),
    [sym_auditlog_priv] = ACTIONS(1835),
    [sym_configuration_priv] = ACTIONS(1835),
    [sym_deviceconfig_priv] = ACTIONS(1835),
    [sym_ftp_priv] = ACTIONS(1835),
    [sym_http_priv] = ACTIONS(1835),
    [sym_networkconfig_priv] = ACTIONS(1835),
    [sym_programport_priv] = ACTIONS(1835),
    [sym_remoteui_priv] = ACTIONS(1835),
    [sym_securitycontrol_priv] = ACTIONS(1835),
    [sym_softwaremanagement_priv] = ACTIONS(1835),
    [sym_terminal_priv] = ACTIONS(1835),
    [sym_tpadmin_priv] = ACTIONS(1835),
    [sym_user1_priv] = ACTIONS(1835),
    [sym_user2_priv] = ACTIONS(1835),
    [sym_user3_priv] = ACTIONS(1835),
    [sym_user4_priv] = ACTIONS(1835),
    [sym_usermanagement_priv] = ACTIONS(1835),
    [sym_valid_account] = ACTIONS(1835),
    [sym_err_name_invalid] = ACTIONS(1835),
    [sym_err_password_invalid] = ACTIONS(1835),
    [sym_err_invalid_account] = ACTIONS(1835),
    [sym_err_no_authorization] = ACTIONS(1835),
    [sym_err_invalid_parameter] = ACTIONS(1835),
    [sym_err_user_locked_out] = ACTIONS(1835),
    [sym_err_user_account_expired] = ACTIONS(1835),
    [sym_err_general_error] = ACTIONS(1835),
    [sym_err_authentication_error] = ACTIONS(1835),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1835),
    [sym_tls_validate_certificate] = ACTIONS(1835),
    [sym_tdata] = ACTIONS(1835),
    [sym_tchannel] = ACTIONS(1835),
    [sym_tlevel] = ACTIONS(1835),
    [sym_tbutton] = ACTIONS(1835),
    [sym_ttimeline] = ACTIONS(1835),
    [sym_tcustom] = ACTIONS(1835),
    [sym_url_struct] = ACTIONS(1835),
    [sym_dns_struct] = ACTIONS(1835),
    [sym_ip_address_struct] = ACTIONS(1835),
    [sym_dev_info_struct] = ACTIONS(1835),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1835),
    [sym_data_feed] = ACTIONS(1835),
    [sym_data_field] = ACTIONS(1835),
    [sym_data_record] = ACTIONS(1835),
    [sym_wc_data_feed] = ACTIONS(1835),
    [sym_wc_data_field] = ACTIONS(1835),
    [sym_wc_data_record] = ACTIONS(1835),
    [sym_last_login_info] = ACTIONS(1835),
    [sym____reserved___] = ACTIONS(1835),
    [sym_date] = ACTIONS(1835),
    [sym_day] = ACTIONS(1835),
    [sym_time] = ACTIONS(1835),
    [sym_ldate] = ACTIONS(1835),
    [sym_push_device] = ACTIONS(1835),
    [sym_push_channel] = ACTIONS(1835),
    [sym_push_devchan] = ACTIONS(1835),
    [sym_release_device] = ACTIONS(1835),
    [sym_release_channel] = ACTIONS(1835),
    [sym_release_devchan] = ACTIONS(1835),
    [sym_master_slot] = ACTIONS(1835),
    [sym_get_pulse_time] = ACTIONS(1835),
    [sym_get_timer] = ACTIONS(1835),
    [sym_master_sn] = ACTIONS(1835),
    [sym_system_number] = ACTIONS(1835),
    [sym_button] = ACTIONS(1835),
    [sym_dv_channel] = ACTIONS(1835),
    [sym_data] = ACTIONS(1835),
    [sym_level] = ACTIONS(1835),
    [sym_timeline] = ACTIONS(1835),
    [sym_channel] = ACTIONS(1835),
    [sym_custom] = ACTIONS(1835),
    [sym_atoi] = ACTIONS(1835),
    [sym_atol] = ACTIONS(1835),
    [sym_atof] = ACTIONS(1835),
    [sym_itoa] = ACTIONS(1835),
    [sym_itohex] = ACTIONS(1835),
    [sym_hextoi] = ACTIONS(1835),
    [sym_ftoa] = ACTIONS(1835),
    [sym_find_string] = ACTIONS(1835),
    [sym_left_string] = ACTIONS(1835),
    [sym_right_string] = ACTIONS(1835),
    [sym_mid_string] = ACTIONS(1835),
    [sym_remove_string] = ACTIONS(1835),
    [sym_get_buffer_string] = ACTIONS(1835),
    [sym_get_buffer_char] = ACTIONS(1835),
    [sym_get_multi_buffer_string] = ACTIONS(1835),
    [sym_length_string] = ACTIONS(1835),
    [sym_max_length_string] = ACTIONS(1835),
    [sym_set_length_string] = ACTIONS(1835),
    [sym_compare_string] = ACTIONS(1835),
    [sym_upper_string] = ACTIONS(1835),
    [sym_lower_string] = ACTIONS(1835),
    [sym_get_last] = ACTIONS(1835),
    [sym_date_to_day] = ACTIONS(1835),
    [sym_date_to_month] = ACTIONS(1835),
    [sym_date_to_year] = ACTIONS(1835),
    [sym_time_to_hour] = ACTIONS(1835),
    [sym_time_to_minute] = ACTIONS(1835),
    [sym_time_to_second] = ACTIONS(1835),
    [sym_day_of_week] = ACTIONS(1835),
    [sym_set_pulse_time] = ACTIONS(1835),
    [sym_set_timer] = ACTIONS(1835),
    [sym_file_open] = ACTIONS(1835),
    [sym_file_close] = ACTIONS(1835),
    [sym_file_read] = ACTIONS(1835),
    [sym_file_read_line] = ACTIONS(1835),
    [sym_file_write] = ACTIONS(1835),
    [sym_file_write_line] = ACTIONS(1835),
    [sym_file_copy] = ACTIONS(1835),
    [sym_file_delete] = ACTIONS(1835),
    [sym_file_rename] = ACTIONS(1835),
    [sym_file_dir] = ACTIONS(1835),
    [sym_file_getdir] = ACTIONS(1835),
    [sym_file_setdir] = ACTIONS(1835),
    [sym_file_create_dir] = ACTIONS(1835),
    [sym_file_remove_dir] = ACTIONS(1835),
    [sym_file_seek] = ACTIONS(1835),
    [sym_length_array] = ACTIONS(1835),
    [sym_max_length_array] = ACTIONS(1835),
    [sym_set_length_array] = ACTIONS(1835),
    [sym_ip_client_open] = ACTIONS(1835),
    [sym_ip_client_close] = ACTIONS(1835),
    [sym_ip_server_open] = ACTIONS(1835),
    [sym_ip_server_close] = ACTIONS(1835),
    [sym_ip_bound_client_open] = ACTIONS(1835),
    [sym_ip_mc_server_open] = ACTIONS(1835),
    [sym_ip_set_option] = ACTIONS(1835),
    [sym_ssh_client_open] = ACTIONS(1835),
    [sym_ssh_client_close] = ACTIONS(1835),
    [sym_tls_client_open] = ACTIONS(1835),
    [sym_tls_client_close] = ACTIONS(1835),
    [sym_do_push] = ACTIONS(1835),
    [sym_do_push_timed] = ACTIONS(1835),
    [sym_do_release] = ACTIONS(1835),
    [sym_device_id] = ACTIONS(1835),
    [sym_device_info] = ACTIONS(1835),
    [sym_device_id_string] = ACTIONS(1835),
    [sym_device_standby] = ACTIONS(1835),
    [sym_device_wake] = ACTIONS(1835),
    [sym_timeline_create] = ACTIONS(1835),
    [sym_timeline_kill] = ACTIONS(1835),
    [sym_timeline_reload] = ACTIONS(1835),
    [sym_timeline_pause] = ACTIONS(1835),
    [sym_timeline_restart] = ACTIONS(1835),
    [sym_timeline_set] = ACTIONS(1835),
    [sym_timeline_get] = ACTIONS(1835),
    [sym_timeline_active] = ACTIONS(1835),
    [sym_timeline_dynamic_id] = ACTIONS(1835),
    [sym_variable_to_string] = ACTIONS(1835),
    [sym_string_to_variable] = ACTIONS(1835),
    [sym_length_variable_to_string] = ACTIONS(1835),
    [sym_type_cast] = ACTIONS(1835),
    [sym_variable_to_xml] = ACTIONS(1835),
    [sym_xml_to_variable] = ACTIONS(1835),
    [sym_length_variable_to_xml] = ACTIONS(1835),
    [sym_redirect_string] = ACTIONS(1835),
    [sym_random_number] = ACTIONS(1835),
    [sym_get_unique_id] = ACTIONS(1835),
    [sym_get_system_number] = ACTIONS(1835),
    [sym_set_system_number] = ACTIONS(1835),
    [sym_get_serial_number] = ACTIONS(1835),
    [sym_get_master_build] = ACTIONS(1835),
    [sym_set_validation_code] = ACTIONS(1835),
    [sym_reboot] = ACTIONS(1835),
    [sym_load_duet_module] = ACTIONS(1835),
    [sym_unload_duet_module] = ACTIONS(1835),
    [sym_duet_mem_size_set] = ACTIONS(1835),
    [sym_duet_mem_size_get] = ACTIONS(1835),
    [sym_dynamic_application_device] = ACTIONS(1835),
    [sym_dynamic_polled_port] = ACTIONS(1835),
    [sym_static_port_binding] = ACTIONS(1835),
    [sym_static_ip_binding] = ACTIONS(1835),
    [sym_get_dns_list] = ACTIONS(1835),
    [sym_set_dns_list] = ACTIONS(1835),
    [sym_get_ip_address] = ACTIONS(1835),
    [sym_set_ip_address] = ACTIONS(1835),
    [sym_get_url_list] = ACTIONS(1835),
    [sym_add_url_entry] = ACTIONS(1835),
    [sym_delete_url_entry] = ACTIONS(1835),
    [sym_abs_value] = ACTIONS(1835),
    [sym_max_value] = ACTIONS(1835),
    [sym_min_value] = ACTIONS(1835),
    [sym_exp_value] = ACTIONS(1835),
    [sym_log_value] = ACTIONS(1835),
    [sym_log10_value] = ACTIONS(1835),
    [sym_power_value] = ACTIONS(1835),
    [sym_sqrt_value] = ACTIONS(1835),
    [sym_raw_be] = ACTIONS(1835),
    [sym_raw_le] = ACTIONS(1835),
    [sym_internal_queue_size_set] = ACTIONS(1835),
    [sym_internal_queue_size_get] = ACTIONS(1835),
    [sym_internal_threshold_set] = ACTIONS(1835),
    [sym_internal_threshold_get] = ACTIONS(1835),
    [sym_rebuilt_event] = ACTIONS(1835),
    [sym_do_custom_event] = ACTIONS(1835),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1835),
    [sym_clkmgr_set_clk_source] = ACTIONS(1835),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1835),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1835),
    [sym_clkmgr_get_timezone] = ACTIONS(1835),
    [sym_clkmgr_set_timezone] = ACTIONS(1835),
    [sym_clkmgr_get_resync_period] = ACTIONS(1835),
    [sym_clkmgr_set_resync_period] = ACTIONS(1835),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1835),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1835),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1835),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1835),
    [sym_clkmgr_get_timeservers] = ACTIONS(1835),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1835),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1835),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1835),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1835),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1835),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1835),
    [sym_set_log_level] = ACTIONS(1835),
    [sym_get_log_level] = ACTIONS(1835),
    [sym_amx_log] = ACTIONS(1835),
    [sym_smtp_server_config_set] = ACTIONS(1835),
    [sym_smtp_server_config_get] = ACTIONS(1835),
    [sym_smtp_send] = ACTIONS(1835),
    [sym_validate_netlinx_account] = ACTIONS(1835),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1835),
    [sym_authenticate_certificate] = ACTIONS(1835),
    [sym_audit_netlinx_session_event] = ACTIONS(1835),
    [sym_audit_netlinx_generic_event] = ACTIONS(1835),
    [sym_get_available_flash_disk_space] = ACTIONS(1835),
    [sym_get_max_flash_disk_space] = ACTIONS(1835),
    [sym_data_create_feed] = ACTIONS(1835),
    [sym_data_delete_feed] = ACTIONS(1835),
    [sym_data_publish_feed] = ACTIONS(1835),
    [sym_data_get_published_feed] = ACTIONS(1835),
    [sym_data_add_record] = ACTIONS(1835),
    [sym_data_get_event_record] = ACTIONS(1835),
    [sym__wc_data_create_feed] = ACTIONS(1835),
    [sym__wc_data_add_record] = ACTIONS(1835),
    [sym__wc_data_get_event_record] = ACTIONS(1835),
    [sym_format] = ACTIONS(1835),
    [sym_astro_clock] = ACTIONS(1835),
    [sym_set_outdoor_temperature] = ACTIONS(1835),
    [sym_set_virtual_level_count] = ACTIONS(1835),
    [sym_set_virtual_channel_count] = ACTIONS(1835),
    [sym_set_virtual_port_count] = ACTIONS(1835),
    [sym__wc_encode_int] = ACTIONS(1835),
    [sym__wc_decode_int] = ACTIONS(1835),
    [sym__wc_int] = ACTIONS(1835),
    [sym__wc_to_ch_int] = ACTIONS(1835),
    [sym__ch_to_wc_int] = ACTIONS(1835),
    [sym__wc_find_string_int] = ACTIONS(1835),
    [sym__wc_left_string_int] = ACTIONS(1835),
    [sym__wc_lower_string_int] = ACTIONS(1835),
    [sym__wc_mid_string_int] = ACTIONS(1835),
    [sym__wc_remove_string_int] = ACTIONS(1835),
    [sym__wc_right_string_int] = ACTIONS(1835),
    [sym__wc_upper_string_int] = ACTIONS(1835),
    [sym__wc_compare_string_int] = ACTIONS(1835),
    [sym__wc_get_buffer_char_int] = ACTIONS(1835),
    [sym__wc_get_buffer_string_int] = ACTIONS(1835),
    [sym__wc_concat_string_int] = ACTIONS(1835),
    [sym__wc_file_open_int] = ACTIONS(1835),
    [sym__wc_file_close_int] = ACTIONS(1835),
    [sym__wc_file_read_int] = ACTIONS(1835),
    [sym__wc_file_read_line_int] = ACTIONS(1835),
    [sym__wc_file_write_int] = ACTIONS(1835),
    [sym__wc_file_write_line_int] = ACTIONS(1835),
    [aux_sym_preproc_include_token1] = ACTIONS(1835),
    [aux_sym_preproc_define_token1] = ACTIONS(1837),
    [aux_sym_preproc_warn_token1] = ACTIONS(1837),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1837),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1837),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1837),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1837),
    [aux_sym_preproc_else_token1] = ACTIONS(1837),
    [anon_sym_LBRACK] = ACTIONS(1837),
    [anon_sym_LBRACE] = ACTIONS(1837),
    [anon_sym_LPAREN] = ACTIONS(1835),
    [anon_sym_BANG] = ACTIONS(1837),
    [anon_sym_TILDE] = ACTIONS(1837),
    [anon_sym_DASH] = ACTIONS(1835),
    [anon_sym_PLUS] = ACTIONS(1835),
    [anon_sym_DASH_DASH] = ACTIONS(1837),
    [anon_sym_PLUS_PLUS] = ACTIONS(1837),
    [anon_sym_DQUOTE] = ACTIONS(1837),
    [anon_sym_SQUOTE] = ACTIONS(1837),
    [sym_number_literal] = ACTIONS(1837),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(255)] = {
    [sym_identifier] = ACTIONS(1839),
    [sym_program_name_keyword] = ACTIONS(1839),
    [sym_module_name_keyword] = ACTIONS(1839),
    [sym_define_device_keyword] = ACTIONS(1839),
    [sym_define_combine_keyword] = ACTIONS(1839),
    [sym_define_constant_keyword] = ACTIONS(1839),
    [sym_define_type_keyword] = ACTIONS(1839),
    [sym_define_variable_keyword] = ACTIONS(1839),
    [sym_define_system_variable_keyword] = ACTIONS(1839),
    [sym_define_connect_level_keyword] = ACTIONS(1839),
    [sym_define_latching_keyword] = ACTIONS(1839),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1839),
    [sym_define_toggling_keyword] = ACTIONS(1839),
    [sym_define_call_keyword] = ACTIONS(1839),
    [sym_define_function_keyword] = ACTIONS(1839),
    [sym_define_library_function_keyword] = ACTIONS(1839),
    [sym_define_start_keyword] = ACTIONS(1839),
    [sym_define_event_keyword] = ACTIONS(1839),
    [sym_define_program_keyword] = ACTIONS(1839),
    [sym_define_module_keyword] = ACTIONS(1839),
    [sym_char_keyword] = ACTIONS(1839),
    [sym_widechar_keyword] = ACTIONS(1839),
    [sym_integer_keyword] = ACTIONS(1839),
    [sym_sinteger_keyword] = ACTIONS(1839),
    [sym_long_keyword] = ACTIONS(1839),
    [sym_slong_keyword] = ACTIONS(1839),
    [sym_float_keyword] = ACTIONS(1839),
    [sym_double_keyword] = ACTIONS(1839),
    [sym_variant_keyword] = ACTIONS(1839),
    [sym_variantarray_keyword] = ACTIONS(1839),
    [sym_local_var_keyword] = ACTIONS(1839),
    [sym_stack_var_keyword] = ACTIONS(1839),
    [sym_constant_keyword] = ACTIONS(1839),
    [sym_volatile_keyword] = ACTIONS(1839),
    [sym_non_volatile_keyword] = ACTIONS(1839),
    [sym_persistent_keyword] = ACTIONS(1839),
    [sym_dev_keyword] = ACTIONS(1839),
    [sym_devlev_keyword] = ACTIONS(1839),
    [sym_devchan_keyword] = ACTIONS(1839),
    [sym_if_keyword] = ACTIONS(1839),
    [sym_else_keyword] = ACTIONS(1839),
    [sym_switch_keyword] = ACTIONS(1839),
    [sym_case_keyword] = ACTIONS(1839),
    [sym_default_keyword] = ACTIONS(1839),
    [sym_while_keyword] = ACTIONS(1839),
    [sym_for_keyword] = ACTIONS(1839),
    [sym_break_keyword] = ACTIONS(1839),
    [sym_continue_keyword] = ACTIONS(1839),
    [sym_return_keyword] = ACTIONS(1839),
    [sym_select_keyword] = ACTIONS(1839),
    [sym_active_keyword] = ACTIONS(1839),
    [sym_struct_keyword] = ACTIONS(1839),
    [sym_structure_keyword] = ACTIONS(1839),
    [sym_button_event_keyword] = ACTIONS(1839),
    [sym_channel_event_keyword] = ACTIONS(1839),
    [sym_level_event_keyword] = ACTIONS(1839),
    [sym_data_event_keyword] = ACTIONS(1839),
    [sym_timeline_event_keyword] = ACTIONS(1839),
    [sym_custom_event_keyword] = ACTIONS(1839),
    [sym_push_keyword] = ACTIONS(1839),
    [sym_release_keyword] = ACTIONS(1839),
    [aux_sym_on_keyword_token1] = ACTIONS(1839),
    [aux_sym_off_keyword_token1] = ACTIONS(1839),
    [sym_send_level_keyword] = ACTIONS(1839),
    [sym_send_string_keyword] = ACTIONS(1839),
    [sym_send_command_keyword] = ACTIONS(1839),
    [sym_clear_buffer_keyword] = ACTIONS(1839),
    [sym_create_buffer_keyword] = ACTIONS(1839),
    [sym_create_multi_buffer_keyword] = ACTIONS(1839),
    [sym_call_keyword] = ACTIONS(1839),
    [sym_system_call_keyword] = ACTIONS(1839),
    [sym_devchan_to_keyword] = ACTIONS(1839),
    [sym_devchan_min_to_keyword] = ACTIONS(1839),
    [sym_devchan_total_off_keyword] = ACTIONS(1839),
    [sym_devchan_pulse_keyword] = ACTIONS(1839),
    [sym_wait_keyword] = ACTIONS(1839),
    [sym_cancel_wait_keyword] = ACTIONS(1839),
    [sym_wait_until_keyword] = ACTIONS(1839),
    [sym_cancel_wait_until_keyword] = ACTIONS(1839),
    [sym_cancel_all_wait_keyword] = ACTIONS(1839),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1839),
    [sym_not] = ACTIONS(1839),
    [sym_bnot] = ACTIONS(1839),
    [sym___file__] = ACTIONS(1839),
    [sym___date__] = ACTIONS(1839),
    [sym___ldate__] = ACTIONS(1839),
    [sym___time__] = ACTIONS(1839),
    [sym___line__] = ACTIONS(1839),
    [sym___name__] = ACTIONS(1839),
    [sym___netlinx__] = ACTIONS(1839),
    [sym_netlinx_axi_version] = ACTIONS(1839),
    [sym_url_flg_tcp] = ACTIONS(1839),
    [sym_url_flg_accinfopresent] = ACTIONS(1839),
    [sym_url_flg_temp] = ACTIONS(1839),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1839),
    [sym_url_flg_stat_mask] = ACTIONS(1839),
    [sym_url_flg_stat_lookup] = ACTIONS(1839),
    [sym_url_flg_stat_connecting] = ACTIONS(1839),
    [sym_url_flg_stat_waiting] = ACTIONS(1839),
    [sym_url_flg_stat_connected] = ACTIONS(1839),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1839),
    [sym_first_virtual_device] = ACTIONS(1839),
    [sym_first_local_port] = ACTIONS(1839),
    [sym_dynamic_virtual_device] = ACTIONS(1839),
    [sym_timeline_once] = ACTIONS(1839),
    [sym_timeline_repeat] = ACTIONS(1839),
    [sym_timeline_absolute] = ACTIONS(1839),
    [sym_timeline_relative] = ACTIONS(1839),
    [sym_do_push_timed_infinite] = ACTIONS(1839),
    [sym_source_type_no_address] = ACTIONS(1839),
    [sym_source_type_neuron_id] = ACTIONS(1839),
    [sym_source_type_ip_address] = ACTIONS(1839),
    [sym_source_type_axlink] = ACTIONS(1839),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1839),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1839),
    [sym_source_type_ip_socket_address] = ACTIONS(1839),
    [sym_source_type_rs232] = ACTIONS(1839),
    [sym_source_type_internal] = ACTIONS(1839),
    [sym_source_type_mac_address] = ACTIONS(1839),
    [sym_source_type_ipv4_port] = ACTIONS(1839),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1839),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1839),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1839),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1839),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1839),
    [sym_true] = ACTIONS(1839),
    [sym_false] = ACTIONS(1839),
    [sym_file_read_only] = ACTIONS(1839),
    [sym_file_rw_new] = ACTIONS(1839),
    [sym_file_rw_append] = ACTIONS(1839),
    [sym_ip_tcp] = ACTIONS(1839),
    [sym_ip_udp] = ACTIONS(1839),
    [sym_ip_udp_2way] = ACTIONS(1839),
    [sym_xml_encode_types] = ACTIONS(1839),
    [sym_xml_encode_char_as_list] = ACTIONS(1839),
    [sym_xml_encode_le] = ACTIONS(1839),
    [sym_xml_decode_types] = ACTIONS(1839),
    [sym_xml_decode_no_preserve] = ACTIONS(1839),
    [sym_duet_dev_type_display_device] = ACTIONS(1839),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1839),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1839),
    [sym_duet_dev_type_io_device] = ACTIONS(1839),
    [sym_duet_dev_type_relay_device] = ACTIONS(1839),
    [sym_duet_dev_type_ups] = ACTIONS(1839),
    [sym_duet_dev_type_amplifier] = ACTIONS(1839),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1839),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1839),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1839),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1839),
    [sym_duet_dev_type_camera] = ACTIONS(1839),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1839),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1839),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1839),
    [sym_duet_dev_type_dss] = ACTIONS(1839),
    [sym_duet_dev_type_dvr] = ACTIONS(1839),
    [sym_duet_dev_type_disc_device] = ACTIONS(1839),
    [sym_duet_dev_type_document_camera] = ACTIONS(1839),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1839),
    [sym_duet_dev_type_hvac] = ACTIONS(1839),
    [sym_duet_dev_type_keypad] = ACTIONS(1839),
    [sym_duet_dev_type_light] = ACTIONS(1839),
    [sym_duet_dev_type_monitor] = ACTIONS(1839),
    [sym_duet_dev_type_motor] = ACTIONS(1839),
    [sym_duet_dev_type_multi_window] = ACTIONS(1839),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1839),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1839),
    [sym_duet_dev_type_receiver] = ACTIONS(1839),
    [sym_duet_dev_type_security_system] = ACTIONS(1839),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1839),
    [sym_duet_dev_type_settop_box] = ACTIONS(1839),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1839),
    [sym_duet_dev_type_switcher] = ACTIONS(1839),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1839),
    [sym_duet_dev_type_tv] = ACTIONS(1839),
    [sym_duet_dev_type_utility] = ACTIONS(1839),
    [sym_duet_dev_type_vcr] = ACTIONS(1839),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1839),
    [sym_duet_dev_type_video_processor] = ACTIONS(1839),
    [sym_duet_dev_type_video_projector] = ACTIONS(1839),
    [sym_duet_dev_type_video_wall] = ACTIONS(1839),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1839),
    [sym_duet_dev_type_weather] = ACTIONS(1839),
    [sym_duet_dev_not_polled] = ACTIONS(1839),
    [sym_duet_dev_polled] = ACTIONS(1839),
    [sym_clkmgr_mode_network] = ACTIONS(1839),
    [sym_clkmgr_mode_standalone] = ACTIONS(1839),
    [sym_normal_standby] = ACTIONS(1839),
    [sym_normal_wake] = ACTIONS(1839),
    [sym_ip_multicast_ttl_option] = ACTIONS(1839),
    [sym_ip_ttl_subnet] = ACTIONS(1839),
    [sym_ip_ttl_site] = ACTIONS(1839),
    [sym_ip_ttl_region] = ACTIONS(1839),
    [sym_ip_ttl_continent] = ACTIONS(1839),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1839),
    [sym_ip_nodelay_on] = ACTIONS(1839),
    [sym_ip_nodelay_off] = ACTIONS(1839),
    [sym_amx_error] = ACTIONS(1839),
    [sym_amx_warning] = ACTIONS(1839),
    [sym_amx_info] = ACTIONS(1839),
    [sym_amx_debug] = ACTIONS(1839),
    [sym_netlinx_login_success] = ACTIONS(1839),
    [sym_netlinx_login_fail] = ACTIONS(1839),
    [sym_netlinx_logout] = ACTIONS(1839),
    [sym_null_str] = ACTIONS(1839),
    [sym_smtp_address] = ACTIONS(1839),
    [sym_smtp_port_number] = ACTIONS(1839),
    [sym_smtp_username] = ACTIONS(1839),
    [sym_smtp_password] = ACTIONS(1839),
    [sym_smtp_require_tls] = ACTIONS(1839),
    [sym_smtp_from] = ACTIONS(1839),
    [sym_smtp_tls_true] = ACTIONS(1839),
    [sym_smtp_tls_false] = ACTIONS(1839),
    [sym_listview_on_row_select_event] = ACTIONS(1839),
    [sym_data_structure_unknown] = ACTIONS(1839),
    [sym_data_structure_datafeed] = ACTIONS(1839),
    [sym_data_structure_datarecordset] = ACTIONS(1839),
    [sym_data_structure_datarecord] = ACTIONS(1839),
    [sym_data_structure_datafield] = ACTIONS(1839),
    [sym_data_max_content_field_count] = ACTIONS(1839),
    [sym_data_max_metadata_field_count] = ACTIONS(1839),
    [sym_data_max_id_length] = ACTIONS(1839),
    [sym_data_max_name_length] = ACTIONS(1839),
    [sym_data_max_description_length] = ACTIONS(1839),
    [sym_data_max_type_length] = ACTIONS(1839),
    [sym_data_max_format_length] = ACTIONS(1839),
    [sym_data_max_source_length] = ACTIONS(1839),
    [sym_data_max_value_length] = ACTIONS(1839),
    [sym_data_max_label_length] = ACTIONS(1839),
    [sym_data_max_url_length] = ACTIONS(1839),
    [sym_max_last_login_info_length] = ACTIONS(1839),
    [sym_data_type_unknown] = ACTIONS(1839),
    [sym_data_type_string] = ACTIONS(1839),
    [sym_data_type_datetime] = ACTIONS(1839),
    [sym_data_type_date] = ACTIONS(1839),
    [sym_data_type_time] = ACTIONS(1839),
    [sym_data_type_image] = ACTIONS(1839),
    [sym_data_format_url] = ACTIONS(1839),
    [sym_data_format_phone] = ACTIONS(1839),
    [sym_data_format_email] = ACTIONS(1839),
    [sym_data_format_iso8601] = ACTIONS(1839),
    [sym_auditlog_priv] = ACTIONS(1839),
    [sym_configuration_priv] = ACTIONS(1839),
    [sym_deviceconfig_priv] = ACTIONS(1839),
    [sym_ftp_priv] = ACTIONS(1839),
    [sym_http_priv] = ACTIONS(1839),
    [sym_networkconfig_priv] = ACTIONS(1839),
    [sym_programport_priv] = ACTIONS(1839),
    [sym_remoteui_priv] = ACTIONS(1839),
    [sym_securitycontrol_priv] = ACTIONS(1839),
    [sym_softwaremanagement_priv] = ACTIONS(1839),
    [sym_terminal_priv] = ACTIONS(1839),
    [sym_tpadmin_priv] = ACTIONS(1839),
    [sym_user1_priv] = ACTIONS(1839),
    [sym_user2_priv] = ACTIONS(1839),
    [sym_user3_priv] = ACTIONS(1839),
    [sym_user4_priv] = ACTIONS(1839),
    [sym_usermanagement_priv] = ACTIONS(1839),
    [sym_valid_account] = ACTIONS(1839),
    [sym_err_name_invalid] = ACTIONS(1839),
    [sym_err_password_invalid] = ACTIONS(1839),
    [sym_err_invalid_account] = ACTIONS(1839),
    [sym_err_no_authorization] = ACTIONS(1839),
    [sym_err_invalid_parameter] = ACTIONS(1839),
    [sym_err_user_locked_out] = ACTIONS(1839),
    [sym_err_user_account_expired] = ACTIONS(1839),
    [sym_err_general_error] = ACTIONS(1839),
    [sym_err_authentication_error] = ACTIONS(1839),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1839),
    [sym_tls_validate_certificate] = ACTIONS(1839),
    [sym_tdata] = ACTIONS(1839),
    [sym_tchannel] = ACTIONS(1839),
    [sym_tlevel] = ACTIONS(1839),
    [sym_tbutton] = ACTIONS(1839),
    [sym_ttimeline] = ACTIONS(1839),
    [sym_tcustom] = ACTIONS(1839),
    [sym_url_struct] = ACTIONS(1839),
    [sym_dns_struct] = ACTIONS(1839),
    [sym_ip_address_struct] = ACTIONS(1839),
    [sym_dev_info_struct] = ACTIONS(1839),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1839),
    [sym_data_feed] = ACTIONS(1839),
    [sym_data_field] = ACTIONS(1839),
    [sym_data_record] = ACTIONS(1839),
    [sym_wc_data_feed] = ACTIONS(1839),
    [sym_wc_data_field] = ACTIONS(1839),
    [sym_wc_data_record] = ACTIONS(1839),
    [sym_last_login_info] = ACTIONS(1839),
    [sym____reserved___] = ACTIONS(1839),
    [sym_date] = ACTIONS(1839),
    [sym_day] = ACTIONS(1839),
    [sym_time] = ACTIONS(1839),
    [sym_ldate] = ACTIONS(1839),
    [sym_push_device] = ACTIONS(1839),
    [sym_push_channel] = ACTIONS(1839),
    [sym_push_devchan] = ACTIONS(1839),
    [sym_release_device] = ACTIONS(1839),
    [sym_release_channel] = ACTIONS(1839),
    [sym_release_devchan] = ACTIONS(1839),
    [sym_master_slot] = ACTIONS(1839),
    [sym_get_pulse_time] = ACTIONS(1839),
    [sym_get_timer] = ACTIONS(1839),
    [sym_master_sn] = ACTIONS(1839),
    [sym_system_number] = ACTIONS(1839),
    [sym_button] = ACTIONS(1839),
    [sym_dv_channel] = ACTIONS(1839),
    [sym_data] = ACTIONS(1839),
    [sym_level] = ACTIONS(1839),
    [sym_timeline] = ACTIONS(1839),
    [sym_channel] = ACTIONS(1839),
    [sym_custom] = ACTIONS(1839),
    [sym_atoi] = ACTIONS(1839),
    [sym_atol] = ACTIONS(1839),
    [sym_atof] = ACTIONS(1839),
    [sym_itoa] = ACTIONS(1839),
    [sym_itohex] = ACTIONS(1839),
    [sym_hextoi] = ACTIONS(1839),
    [sym_ftoa] = ACTIONS(1839),
    [sym_find_string] = ACTIONS(1839),
    [sym_left_string] = ACTIONS(1839),
    [sym_right_string] = ACTIONS(1839),
    [sym_mid_string] = ACTIONS(1839),
    [sym_remove_string] = ACTIONS(1839),
    [sym_get_buffer_string] = ACTIONS(1839),
    [sym_get_buffer_char] = ACTIONS(1839),
    [sym_get_multi_buffer_string] = ACTIONS(1839),
    [sym_length_string] = ACTIONS(1839),
    [sym_max_length_string] = ACTIONS(1839),
    [sym_set_length_string] = ACTIONS(1839),
    [sym_compare_string] = ACTIONS(1839),
    [sym_upper_string] = ACTIONS(1839),
    [sym_lower_string] = ACTIONS(1839),
    [sym_get_last] = ACTIONS(1839),
    [sym_date_to_day] = ACTIONS(1839),
    [sym_date_to_month] = ACTIONS(1839),
    [sym_date_to_year] = ACTIONS(1839),
    [sym_time_to_hour] = ACTIONS(1839),
    [sym_time_to_minute] = ACTIONS(1839),
    [sym_time_to_second] = ACTIONS(1839),
    [sym_day_of_week] = ACTIONS(1839),
    [sym_set_pulse_time] = ACTIONS(1839),
    [sym_set_timer] = ACTIONS(1839),
    [sym_file_open] = ACTIONS(1839),
    [sym_file_close] = ACTIONS(1839),
    [sym_file_read] = ACTIONS(1839),
    [sym_file_read_line] = ACTIONS(1839),
    [sym_file_write] = ACTIONS(1839),
    [sym_file_write_line] = ACTIONS(1839),
    [sym_file_copy] = ACTIONS(1839),
    [sym_file_delete] = ACTIONS(1839),
    [sym_file_rename] = ACTIONS(1839),
    [sym_file_dir] = ACTIONS(1839),
    [sym_file_getdir] = ACTIONS(1839),
    [sym_file_setdir] = ACTIONS(1839),
    [sym_file_create_dir] = ACTIONS(1839),
    [sym_file_remove_dir] = ACTIONS(1839),
    [sym_file_seek] = ACTIONS(1839),
    [sym_length_array] = ACTIONS(1839),
    [sym_max_length_array] = ACTIONS(1839),
    [sym_set_length_array] = ACTIONS(1839),
    [sym_ip_client_open] = ACTIONS(1839),
    [sym_ip_client_close] = ACTIONS(1839),
    [sym_ip_server_open] = ACTIONS(1839),
    [sym_ip_server_close] = ACTIONS(1839),
    [sym_ip_bound_client_open] = ACTIONS(1839),
    [sym_ip_mc_server_open] = ACTIONS(1839),
    [sym_ip_set_option] = ACTIONS(1839),
    [sym_ssh_client_open] = ACTIONS(1839),
    [sym_ssh_client_close] = ACTIONS(1839),
    [sym_tls_client_open] = ACTIONS(1839),
    [sym_tls_client_close] = ACTIONS(1839),
    [sym_do_push] = ACTIONS(1839),
    [sym_do_push_timed] = ACTIONS(1839),
    [sym_do_release] = ACTIONS(1839),
    [sym_device_id] = ACTIONS(1839),
    [sym_device_info] = ACTIONS(1839),
    [sym_device_id_string] = ACTIONS(1839),
    [sym_device_standby] = ACTIONS(1839),
    [sym_device_wake] = ACTIONS(1839),
    [sym_timeline_create] = ACTIONS(1839),
    [sym_timeline_kill] = ACTIONS(1839),
    [sym_timeline_reload] = ACTIONS(1839),
    [sym_timeline_pause] = ACTIONS(1839),
    [sym_timeline_restart] = ACTIONS(1839),
    [sym_timeline_set] = ACTIONS(1839),
    [sym_timeline_get] = ACTIONS(1839),
    [sym_timeline_active] = ACTIONS(1839),
    [sym_timeline_dynamic_id] = ACTIONS(1839),
    [sym_variable_to_string] = ACTIONS(1839),
    [sym_string_to_variable] = ACTIONS(1839),
    [sym_length_variable_to_string] = ACTIONS(1839),
    [sym_type_cast] = ACTIONS(1839),
    [sym_variable_to_xml] = ACTIONS(1839),
    [sym_xml_to_variable] = ACTIONS(1839),
    [sym_length_variable_to_xml] = ACTIONS(1839),
    [sym_redirect_string] = ACTIONS(1839),
    [sym_random_number] = ACTIONS(1839),
    [sym_get_unique_id] = ACTIONS(1839),
    [sym_get_system_number] = ACTIONS(1839),
    [sym_set_system_number] = ACTIONS(1839),
    [sym_get_serial_number] = ACTIONS(1839),
    [sym_get_master_build] = ACTIONS(1839),
    [sym_set_validation_code] = ACTIONS(1839),
    [sym_reboot] = ACTIONS(1839),
    [sym_load_duet_module] = ACTIONS(1839),
    [sym_unload_duet_module] = ACTIONS(1839),
    [sym_duet_mem_size_set] = ACTIONS(1839),
    [sym_duet_mem_size_get] = ACTIONS(1839),
    [sym_dynamic_application_device] = ACTIONS(1839),
    [sym_dynamic_polled_port] = ACTIONS(1839),
    [sym_static_port_binding] = ACTIONS(1839),
    [sym_static_ip_binding] = ACTIONS(1839),
    [sym_get_dns_list] = ACTIONS(1839),
    [sym_set_dns_list] = ACTIONS(1839),
    [sym_get_ip_address] = ACTIONS(1839),
    [sym_set_ip_address] = ACTIONS(1839),
    [sym_get_url_list] = ACTIONS(1839),
    [sym_add_url_entry] = ACTIONS(1839),
    [sym_delete_url_entry] = ACTIONS(1839),
    [sym_abs_value] = ACTIONS(1839),
    [sym_max_value] = ACTIONS(1839),
    [sym_min_value] = ACTIONS(1839),
    [sym_exp_value] = ACTIONS(1839),
    [sym_log_value] = ACTIONS(1839),
    [sym_log10_value] = ACTIONS(1839),
    [sym_power_value] = ACTIONS(1839),
    [sym_sqrt_value] = ACTIONS(1839),
    [sym_raw_be] = ACTIONS(1839),
    [sym_raw_le] = ACTIONS(1839),
    [sym_internal_queue_size_set] = ACTIONS(1839),
    [sym_internal_queue_size_get] = ACTIONS(1839),
    [sym_internal_threshold_set] = ACTIONS(1839),
    [sym_internal_threshold_get] = ACTIONS(1839),
    [sym_rebuilt_event] = ACTIONS(1839),
    [sym_do_custom_event] = ACTIONS(1839),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1839),
    [sym_clkmgr_set_clk_source] = ACTIONS(1839),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1839),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1839),
    [sym_clkmgr_get_timezone] = ACTIONS(1839),
    [sym_clkmgr_set_timezone] = ACTIONS(1839),
    [sym_clkmgr_get_resync_period] = ACTIONS(1839),
    [sym_clkmgr_set_resync_period] = ACTIONS(1839),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1839),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1839),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1839),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1839),
    [sym_clkmgr_get_timeservers] = ACTIONS(1839),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1839),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1839),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1839),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1839),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1839),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1839),
    [sym_set_log_level] = ACTIONS(1839),
    [sym_get_log_level] = ACTIONS(1839),
    [sym_amx_log] = ACTIONS(1839),
    [sym_smtp_server_config_set] = ACTIONS(1839),
    [sym_smtp_server_config_get] = ACTIONS(1839),
    [sym_smtp_send] = ACTIONS(1839),
    [sym_validate_netlinx_account] = ACTIONS(1839),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1839),
    [sym_authenticate_certificate] = ACTIONS(1839),
    [sym_audit_netlinx_session_event] = ACTIONS(1839),
    [sym_audit_netlinx_generic_event] = ACTIONS(1839),
    [sym_get_available_flash_disk_space] = ACTIONS(1839),
    [sym_get_max_flash_disk_space] = ACTIONS(1839),
    [sym_data_create_feed] = ACTIONS(1839),
    [sym_data_delete_feed] = ACTIONS(1839),
    [sym_data_publish_feed] = ACTIONS(1839),
    [sym_data_get_published_feed] = ACTIONS(1839),
    [sym_data_add_record] = ACTIONS(1839),
    [sym_data_get_event_record] = ACTIONS(1839),
    [sym__wc_data_create_feed] = ACTIONS(1839),
    [sym__wc_data_add_record] = ACTIONS(1839),
    [sym__wc_data_get_event_record] = ACTIONS(1839),
    [sym_format] = ACTIONS(1839),
    [sym_astro_clock] = ACTIONS(1839),
    [sym_set_outdoor_temperature] = ACTIONS(1839),
    [sym_set_virtual_level_count] = ACTIONS(1839),
    [sym_set_virtual_channel_count] = ACTIONS(1839),
    [sym_set_virtual_port_count] = ACTIONS(1839),
    [sym__wc_encode_int] = ACTIONS(1839),
    [sym__wc_decode_int] = ACTIONS(1839),
    [sym__wc_int] = ACTIONS(1839),
    [sym__wc_to_ch_int] = ACTIONS(1839),
    [sym__ch_to_wc_int] = ACTIONS(1839),
    [sym__wc_find_string_int] = ACTIONS(1839),
    [sym__wc_left_string_int] = ACTIONS(1839),
    [sym__wc_lower_string_int] = ACTIONS(1839),
    [sym__wc_mid_string_int] = ACTIONS(1839),
    [sym__wc_remove_string_int] = ACTIONS(1839),
    [sym__wc_right_string_int] = ACTIONS(1839),
    [sym__wc_upper_string_int] = ACTIONS(1839),
    [sym__wc_compare_string_int] = ACTIONS(1839),
    [sym__wc_get_buffer_char_int] = ACTIONS(1839),
    [sym__wc_get_buffer_string_int] = ACTIONS(1839),
    [sym__wc_concat_string_int] = ACTIONS(1839),
    [sym__wc_file_open_int] = ACTIONS(1839),
    [sym__wc_file_close_int] = ACTIONS(1839),
    [sym__wc_file_read_int] = ACTIONS(1839),
    [sym__wc_file_read_line_int] = ACTIONS(1839),
    [sym__wc_file_write_int] = ACTIONS(1839),
    [sym__wc_file_write_line_int] = ACTIONS(1839),
    [aux_sym_preproc_include_token1] = ACTIONS(1839),
    [aux_sym_preproc_define_token1] = ACTIONS(1841),
    [aux_sym_preproc_warn_token1] = ACTIONS(1841),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1841),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1841),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1841),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1841),
    [aux_sym_preproc_else_token1] = ACTIONS(1841),
    [anon_sym_LBRACK] = ACTIONS(1841),
    [anon_sym_LBRACE] = ACTIONS(1841),
    [anon_sym_LPAREN] = ACTIONS(1839),
    [anon_sym_BANG] = ACTIONS(1841),
    [anon_sym_TILDE] = ACTIONS(1841),
    [anon_sym_DASH] = ACTIONS(1839),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_DASH_DASH] = ACTIONS(1841),
    [anon_sym_PLUS_PLUS] = ACTIONS(1841),
    [anon_sym_DQUOTE] = ACTIONS(1841),
    [anon_sym_SQUOTE] = ACTIONS(1841),
    [sym_number_literal] = ACTIONS(1841),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(256)] = {
    [sym_identifier] = ACTIONS(1843),
    [sym_program_name_keyword] = ACTIONS(1843),
    [sym_module_name_keyword] = ACTIONS(1843),
    [sym_define_device_keyword] = ACTIONS(1843),
    [sym_define_combine_keyword] = ACTIONS(1843),
    [sym_define_constant_keyword] = ACTIONS(1843),
    [sym_define_type_keyword] = ACTIONS(1843),
    [sym_define_variable_keyword] = ACTIONS(1843),
    [sym_define_system_variable_keyword] = ACTIONS(1843),
    [sym_define_connect_level_keyword] = ACTIONS(1843),
    [sym_define_latching_keyword] = ACTIONS(1843),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1843),
    [sym_define_toggling_keyword] = ACTIONS(1843),
    [sym_define_call_keyword] = ACTIONS(1843),
    [sym_define_function_keyword] = ACTIONS(1843),
    [sym_define_library_function_keyword] = ACTIONS(1843),
    [sym_define_start_keyword] = ACTIONS(1843),
    [sym_define_event_keyword] = ACTIONS(1843),
    [sym_define_program_keyword] = ACTIONS(1843),
    [sym_define_module_keyword] = ACTIONS(1843),
    [sym_char_keyword] = ACTIONS(1843),
    [sym_widechar_keyword] = ACTIONS(1843),
    [sym_integer_keyword] = ACTIONS(1843),
    [sym_sinteger_keyword] = ACTIONS(1843),
    [sym_long_keyword] = ACTIONS(1843),
    [sym_slong_keyword] = ACTIONS(1843),
    [sym_float_keyword] = ACTIONS(1843),
    [sym_double_keyword] = ACTIONS(1843),
    [sym_variant_keyword] = ACTIONS(1843),
    [sym_variantarray_keyword] = ACTIONS(1843),
    [sym_local_var_keyword] = ACTIONS(1843),
    [sym_stack_var_keyword] = ACTIONS(1843),
    [sym_constant_keyword] = ACTIONS(1843),
    [sym_volatile_keyword] = ACTIONS(1843),
    [sym_non_volatile_keyword] = ACTIONS(1843),
    [sym_persistent_keyword] = ACTIONS(1843),
    [sym_dev_keyword] = ACTIONS(1843),
    [sym_devlev_keyword] = ACTIONS(1843),
    [sym_devchan_keyword] = ACTIONS(1843),
    [sym_if_keyword] = ACTIONS(1843),
    [sym_else_keyword] = ACTIONS(1843),
    [sym_switch_keyword] = ACTIONS(1843),
    [sym_case_keyword] = ACTIONS(1843),
    [sym_default_keyword] = ACTIONS(1843),
    [sym_while_keyword] = ACTIONS(1843),
    [sym_for_keyword] = ACTIONS(1843),
    [sym_break_keyword] = ACTIONS(1843),
    [sym_continue_keyword] = ACTIONS(1843),
    [sym_return_keyword] = ACTIONS(1843),
    [sym_select_keyword] = ACTIONS(1843),
    [sym_active_keyword] = ACTIONS(1843),
    [sym_struct_keyword] = ACTIONS(1843),
    [sym_structure_keyword] = ACTIONS(1843),
    [sym_button_event_keyword] = ACTIONS(1843),
    [sym_channel_event_keyword] = ACTIONS(1843),
    [sym_level_event_keyword] = ACTIONS(1843),
    [sym_data_event_keyword] = ACTIONS(1843),
    [sym_timeline_event_keyword] = ACTIONS(1843),
    [sym_custom_event_keyword] = ACTIONS(1843),
    [sym_push_keyword] = ACTIONS(1843),
    [sym_release_keyword] = ACTIONS(1843),
    [aux_sym_on_keyword_token1] = ACTIONS(1843),
    [aux_sym_off_keyword_token1] = ACTIONS(1843),
    [sym_send_level_keyword] = ACTIONS(1843),
    [sym_send_string_keyword] = ACTIONS(1843),
    [sym_send_command_keyword] = ACTIONS(1843),
    [sym_clear_buffer_keyword] = ACTIONS(1843),
    [sym_create_buffer_keyword] = ACTIONS(1843),
    [sym_create_multi_buffer_keyword] = ACTIONS(1843),
    [sym_call_keyword] = ACTIONS(1843),
    [sym_system_call_keyword] = ACTIONS(1843),
    [sym_devchan_to_keyword] = ACTIONS(1843),
    [sym_devchan_min_to_keyword] = ACTIONS(1843),
    [sym_devchan_total_off_keyword] = ACTIONS(1843),
    [sym_devchan_pulse_keyword] = ACTIONS(1843),
    [sym_wait_keyword] = ACTIONS(1843),
    [sym_cancel_wait_keyword] = ACTIONS(1843),
    [sym_wait_until_keyword] = ACTIONS(1843),
    [sym_cancel_wait_until_keyword] = ACTIONS(1843),
    [sym_cancel_all_wait_keyword] = ACTIONS(1843),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1843),
    [sym_not] = ACTIONS(1843),
    [sym_bnot] = ACTIONS(1843),
    [sym___file__] = ACTIONS(1843),
    [sym___date__] = ACTIONS(1843),
    [sym___ldate__] = ACTIONS(1843),
    [sym___time__] = ACTIONS(1843),
    [sym___line__] = ACTIONS(1843),
    [sym___name__] = ACTIONS(1843),
    [sym___netlinx__] = ACTIONS(1843),
    [sym_netlinx_axi_version] = ACTIONS(1843),
    [sym_url_flg_tcp] = ACTIONS(1843),
    [sym_url_flg_accinfopresent] = ACTIONS(1843),
    [sym_url_flg_temp] = ACTIONS(1843),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1843),
    [sym_url_flg_stat_mask] = ACTIONS(1843),
    [sym_url_flg_stat_lookup] = ACTIONS(1843),
    [sym_url_flg_stat_connecting] = ACTIONS(1843),
    [sym_url_flg_stat_waiting] = ACTIONS(1843),
    [sym_url_flg_stat_connected] = ACTIONS(1843),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1843),
    [sym_first_virtual_device] = ACTIONS(1843),
    [sym_first_local_port] = ACTIONS(1843),
    [sym_dynamic_virtual_device] = ACTIONS(1843),
    [sym_timeline_once] = ACTIONS(1843),
    [sym_timeline_repeat] = ACTIONS(1843),
    [sym_timeline_absolute] = ACTIONS(1843),
    [sym_timeline_relative] = ACTIONS(1843),
    [sym_do_push_timed_infinite] = ACTIONS(1843),
    [sym_source_type_no_address] = ACTIONS(1843),
    [sym_source_type_neuron_id] = ACTIONS(1843),
    [sym_source_type_ip_address] = ACTIONS(1843),
    [sym_source_type_axlink] = ACTIONS(1843),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1843),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1843),
    [sym_source_type_ip_socket_address] = ACTIONS(1843),
    [sym_source_type_rs232] = ACTIONS(1843),
    [sym_source_type_internal] = ACTIONS(1843),
    [sym_source_type_mac_address] = ACTIONS(1843),
    [sym_source_type_ipv4_port] = ACTIONS(1843),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1843),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1843),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1843),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1843),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1843),
    [sym_true] = ACTIONS(1843),
    [sym_false] = ACTIONS(1843),
    [sym_file_read_only] = ACTIONS(1843),
    [sym_file_rw_new] = ACTIONS(1843),
    [sym_file_rw_append] = ACTIONS(1843),
    [sym_ip_tcp] = ACTIONS(1843),
    [sym_ip_udp] = ACTIONS(1843),
    [sym_ip_udp_2way] = ACTIONS(1843),
    [sym_xml_encode_types] = ACTIONS(1843),
    [sym_xml_encode_char_as_list] = ACTIONS(1843),
    [sym_xml_encode_le] = ACTIONS(1843),
    [sym_xml_decode_types] = ACTIONS(1843),
    [sym_xml_decode_no_preserve] = ACTIONS(1843),
    [sym_duet_dev_type_display_device] = ACTIONS(1843),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1843),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1843),
    [sym_duet_dev_type_io_device] = ACTIONS(1843),
    [sym_duet_dev_type_relay_device] = ACTIONS(1843),
    [sym_duet_dev_type_ups] = ACTIONS(1843),
    [sym_duet_dev_type_amplifier] = ACTIONS(1843),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1843),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1843),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1843),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1843),
    [sym_duet_dev_type_camera] = ACTIONS(1843),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1843),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1843),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1843),
    [sym_duet_dev_type_dss] = ACTIONS(1843),
    [sym_duet_dev_type_dvr] = ACTIONS(1843),
    [sym_duet_dev_type_disc_device] = ACTIONS(1843),
    [sym_duet_dev_type_document_camera] = ACTIONS(1843),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1843),
    [sym_duet_dev_type_hvac] = ACTIONS(1843),
    [sym_duet_dev_type_keypad] = ACTIONS(1843),
    [sym_duet_dev_type_light] = ACTIONS(1843),
    [sym_duet_dev_type_monitor] = ACTIONS(1843),
    [sym_duet_dev_type_motor] = ACTIONS(1843),
    [sym_duet_dev_type_multi_window] = ACTIONS(1843),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1843),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1843),
    [sym_duet_dev_type_receiver] = ACTIONS(1843),
    [sym_duet_dev_type_security_system] = ACTIONS(1843),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1843),
    [sym_duet_dev_type_settop_box] = ACTIONS(1843),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1843),
    [sym_duet_dev_type_switcher] = ACTIONS(1843),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1843),
    [sym_duet_dev_type_tv] = ACTIONS(1843),
    [sym_duet_dev_type_utility] = ACTIONS(1843),
    [sym_duet_dev_type_vcr] = ACTIONS(1843),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1843),
    [sym_duet_dev_type_video_processor] = ACTIONS(1843),
    [sym_duet_dev_type_video_projector] = ACTIONS(1843),
    [sym_duet_dev_type_video_wall] = ACTIONS(1843),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1843),
    [sym_duet_dev_type_weather] = ACTIONS(1843),
    [sym_duet_dev_not_polled] = ACTIONS(1843),
    [sym_duet_dev_polled] = ACTIONS(1843),
    [sym_clkmgr_mode_network] = ACTIONS(1843),
    [sym_clkmgr_mode_standalone] = ACTIONS(1843),
    [sym_normal_standby] = ACTIONS(1843),
    [sym_normal_wake] = ACTIONS(1843),
    [sym_ip_multicast_ttl_option] = ACTIONS(1843),
    [sym_ip_ttl_subnet] = ACTIONS(1843),
    [sym_ip_ttl_site] = ACTIONS(1843),
    [sym_ip_ttl_region] = ACTIONS(1843),
    [sym_ip_ttl_continent] = ACTIONS(1843),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1843),
    [sym_ip_nodelay_on] = ACTIONS(1843),
    [sym_ip_nodelay_off] = ACTIONS(1843),
    [sym_amx_error] = ACTIONS(1843),
    [sym_amx_warning] = ACTIONS(1843),
    [sym_amx_info] = ACTIONS(1843),
    [sym_amx_debug] = ACTIONS(1843),
    [sym_netlinx_login_success] = ACTIONS(1843),
    [sym_netlinx_login_fail] = ACTIONS(1843),
    [sym_netlinx_logout] = ACTIONS(1843),
    [sym_null_str] = ACTIONS(1843),
    [sym_smtp_address] = ACTIONS(1843),
    [sym_smtp_port_number] = ACTIONS(1843),
    [sym_smtp_username] = ACTIONS(1843),
    [sym_smtp_password] = ACTIONS(1843),
    [sym_smtp_require_tls] = ACTIONS(1843),
    [sym_smtp_from] = ACTIONS(1843),
    [sym_smtp_tls_true] = ACTIONS(1843),
    [sym_smtp_tls_false] = ACTIONS(1843),
    [sym_listview_on_row_select_event] = ACTIONS(1843),
    [sym_data_structure_unknown] = ACTIONS(1843),
    [sym_data_structure_datafeed] = ACTIONS(1843),
    [sym_data_structure_datarecordset] = ACTIONS(1843),
    [sym_data_structure_datarecord] = ACTIONS(1843),
    [sym_data_structure_datafield] = ACTIONS(1843),
    [sym_data_max_content_field_count] = ACTIONS(1843),
    [sym_data_max_metadata_field_count] = ACTIONS(1843),
    [sym_data_max_id_length] = ACTIONS(1843),
    [sym_data_max_name_length] = ACTIONS(1843),
    [sym_data_max_description_length] = ACTIONS(1843),
    [sym_data_max_type_length] = ACTIONS(1843),
    [sym_data_max_format_length] = ACTIONS(1843),
    [sym_data_max_source_length] = ACTIONS(1843),
    [sym_data_max_value_length] = ACTIONS(1843),
    [sym_data_max_label_length] = ACTIONS(1843),
    [sym_data_max_url_length] = ACTIONS(1843),
    [sym_max_last_login_info_length] = ACTIONS(1843),
    [sym_data_type_unknown] = ACTIONS(1843),
    [sym_data_type_string] = ACTIONS(1843),
    [sym_data_type_datetime] = ACTIONS(1843),
    [sym_data_type_date] = ACTIONS(1843),
    [sym_data_type_time] = ACTIONS(1843),
    [sym_data_type_image] = ACTIONS(1843),
    [sym_data_format_url] = ACTIONS(1843),
    [sym_data_format_phone] = ACTIONS(1843),
    [sym_data_format_email] = ACTIONS(1843),
    [sym_data_format_iso8601] = ACTIONS(1843),
    [sym_auditlog_priv] = ACTIONS(1843),
    [sym_configuration_priv] = ACTIONS(1843),
    [sym_deviceconfig_priv] = ACTIONS(1843),
    [sym_ftp_priv] = ACTIONS(1843),
    [sym_http_priv] = ACTIONS(1843),
    [sym_networkconfig_priv] = ACTIONS(1843),
    [sym_programport_priv] = ACTIONS(1843),
    [sym_remoteui_priv] = ACTIONS(1843),
    [sym_securitycontrol_priv] = ACTIONS(1843),
    [sym_softwaremanagement_priv] = ACTIONS(1843),
    [sym_terminal_priv] = ACTIONS(1843),
    [sym_tpadmin_priv] = ACTIONS(1843),
    [sym_user1_priv] = ACTIONS(1843),
    [sym_user2_priv] = ACTIONS(1843),
    [sym_user3_priv] = ACTIONS(1843),
    [sym_user4_priv] = ACTIONS(1843),
    [sym_usermanagement_priv] = ACTIONS(1843),
    [sym_valid_account] = ACTIONS(1843),
    [sym_err_name_invalid] = ACTIONS(1843),
    [sym_err_password_invalid] = ACTIONS(1843),
    [sym_err_invalid_account] = ACTIONS(1843),
    [sym_err_no_authorization] = ACTIONS(1843),
    [sym_err_invalid_parameter] = ACTIONS(1843),
    [sym_err_user_locked_out] = ACTIONS(1843),
    [sym_err_user_account_expired] = ACTIONS(1843),
    [sym_err_general_error] = ACTIONS(1843),
    [sym_err_authentication_error] = ACTIONS(1843),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1843),
    [sym_tls_validate_certificate] = ACTIONS(1843),
    [sym_tdata] = ACTIONS(1843),
    [sym_tchannel] = ACTIONS(1843),
    [sym_tlevel] = ACTIONS(1843),
    [sym_tbutton] = ACTIONS(1843),
    [sym_ttimeline] = ACTIONS(1843),
    [sym_tcustom] = ACTIONS(1843),
    [sym_url_struct] = ACTIONS(1843),
    [sym_dns_struct] = ACTIONS(1843),
    [sym_ip_address_struct] = ACTIONS(1843),
    [sym_dev_info_struct] = ACTIONS(1843),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1843),
    [sym_data_feed] = ACTIONS(1843),
    [sym_data_field] = ACTIONS(1843),
    [sym_data_record] = ACTIONS(1843),
    [sym_wc_data_feed] = ACTIONS(1843),
    [sym_wc_data_field] = ACTIONS(1843),
    [sym_wc_data_record] = ACTIONS(1843),
    [sym_last_login_info] = ACTIONS(1843),
    [sym____reserved___] = ACTIONS(1843),
    [sym_date] = ACTIONS(1843),
    [sym_day] = ACTIONS(1843),
    [sym_time] = ACTIONS(1843),
    [sym_ldate] = ACTIONS(1843),
    [sym_push_device] = ACTIONS(1843),
    [sym_push_channel] = ACTIONS(1843),
    [sym_push_devchan] = ACTIONS(1843),
    [sym_release_device] = ACTIONS(1843),
    [sym_release_channel] = ACTIONS(1843),
    [sym_release_devchan] = ACTIONS(1843),
    [sym_master_slot] = ACTIONS(1843),
    [sym_get_pulse_time] = ACTIONS(1843),
    [sym_get_timer] = ACTIONS(1843),
    [sym_master_sn] = ACTIONS(1843),
    [sym_system_number] = ACTIONS(1843),
    [sym_button] = ACTIONS(1843),
    [sym_dv_channel] = ACTIONS(1843),
    [sym_data] = ACTIONS(1843),
    [sym_level] = ACTIONS(1843),
    [sym_timeline] = ACTIONS(1843),
    [sym_channel] = ACTIONS(1843),
    [sym_custom] = ACTIONS(1843),
    [sym_atoi] = ACTIONS(1843),
    [sym_atol] = ACTIONS(1843),
    [sym_atof] = ACTIONS(1843),
    [sym_itoa] = ACTIONS(1843),
    [sym_itohex] = ACTIONS(1843),
    [sym_hextoi] = ACTIONS(1843),
    [sym_ftoa] = ACTIONS(1843),
    [sym_find_string] = ACTIONS(1843),
    [sym_left_string] = ACTIONS(1843),
    [sym_right_string] = ACTIONS(1843),
    [sym_mid_string] = ACTIONS(1843),
    [sym_remove_string] = ACTIONS(1843),
    [sym_get_buffer_string] = ACTIONS(1843),
    [sym_get_buffer_char] = ACTIONS(1843),
    [sym_get_multi_buffer_string] = ACTIONS(1843),
    [sym_length_string] = ACTIONS(1843),
    [sym_max_length_string] = ACTIONS(1843),
    [sym_set_length_string] = ACTIONS(1843),
    [sym_compare_string] = ACTIONS(1843),
    [sym_upper_string] = ACTIONS(1843),
    [sym_lower_string] = ACTIONS(1843),
    [sym_get_last] = ACTIONS(1843),
    [sym_date_to_day] = ACTIONS(1843),
    [sym_date_to_month] = ACTIONS(1843),
    [sym_date_to_year] = ACTIONS(1843),
    [sym_time_to_hour] = ACTIONS(1843),
    [sym_time_to_minute] = ACTIONS(1843),
    [sym_time_to_second] = ACTIONS(1843),
    [sym_day_of_week] = ACTIONS(1843),
    [sym_set_pulse_time] = ACTIONS(1843),
    [sym_set_timer] = ACTIONS(1843),
    [sym_file_open] = ACTIONS(1843),
    [sym_file_close] = ACTIONS(1843),
    [sym_file_read] = ACTIONS(1843),
    [sym_file_read_line] = ACTIONS(1843),
    [sym_file_write] = ACTIONS(1843),
    [sym_file_write_line] = ACTIONS(1843),
    [sym_file_copy] = ACTIONS(1843),
    [sym_file_delete] = ACTIONS(1843),
    [sym_file_rename] = ACTIONS(1843),
    [sym_file_dir] = ACTIONS(1843),
    [sym_file_getdir] = ACTIONS(1843),
    [sym_file_setdir] = ACTIONS(1843),
    [sym_file_create_dir] = ACTIONS(1843),
    [sym_file_remove_dir] = ACTIONS(1843),
    [sym_file_seek] = ACTIONS(1843),
    [sym_length_array] = ACTIONS(1843),
    [sym_max_length_array] = ACTIONS(1843),
    [sym_set_length_array] = ACTIONS(1843),
    [sym_ip_client_open] = ACTIONS(1843),
    [sym_ip_client_close] = ACTIONS(1843),
    [sym_ip_server_open] = ACTIONS(1843),
    [sym_ip_server_close] = ACTIONS(1843),
    [sym_ip_bound_client_open] = ACTIONS(1843),
    [sym_ip_mc_server_open] = ACTIONS(1843),
    [sym_ip_set_option] = ACTIONS(1843),
    [sym_ssh_client_open] = ACTIONS(1843),
    [sym_ssh_client_close] = ACTIONS(1843),
    [sym_tls_client_open] = ACTIONS(1843),
    [sym_tls_client_close] = ACTIONS(1843),
    [sym_do_push] = ACTIONS(1843),
    [sym_do_push_timed] = ACTIONS(1843),
    [sym_do_release] = ACTIONS(1843),
    [sym_device_id] = ACTIONS(1843),
    [sym_device_info] = ACTIONS(1843),
    [sym_device_id_string] = ACTIONS(1843),
    [sym_device_standby] = ACTIONS(1843),
    [sym_device_wake] = ACTIONS(1843),
    [sym_timeline_create] = ACTIONS(1843),
    [sym_timeline_kill] = ACTIONS(1843),
    [sym_timeline_reload] = ACTIONS(1843),
    [sym_timeline_pause] = ACTIONS(1843),
    [sym_timeline_restart] = ACTIONS(1843),
    [sym_timeline_set] = ACTIONS(1843),
    [sym_timeline_get] = ACTIONS(1843),
    [sym_timeline_active] = ACTIONS(1843),
    [sym_timeline_dynamic_id] = ACTIONS(1843),
    [sym_variable_to_string] = ACTIONS(1843),
    [sym_string_to_variable] = ACTIONS(1843),
    [sym_length_variable_to_string] = ACTIONS(1843),
    [sym_type_cast] = ACTIONS(1843),
    [sym_variable_to_xml] = ACTIONS(1843),
    [sym_xml_to_variable] = ACTIONS(1843),
    [sym_length_variable_to_xml] = ACTIONS(1843),
    [sym_redirect_string] = ACTIONS(1843),
    [sym_random_number] = ACTIONS(1843),
    [sym_get_unique_id] = ACTIONS(1843),
    [sym_get_system_number] = ACTIONS(1843),
    [sym_set_system_number] = ACTIONS(1843),
    [sym_get_serial_number] = ACTIONS(1843),
    [sym_get_master_build] = ACTIONS(1843),
    [sym_set_validation_code] = ACTIONS(1843),
    [sym_reboot] = ACTIONS(1843),
    [sym_load_duet_module] = ACTIONS(1843),
    [sym_unload_duet_module] = ACTIONS(1843),
    [sym_duet_mem_size_set] = ACTIONS(1843),
    [sym_duet_mem_size_get] = ACTIONS(1843),
    [sym_dynamic_application_device] = ACTIONS(1843),
    [sym_dynamic_polled_port] = ACTIONS(1843),
    [sym_static_port_binding] = ACTIONS(1843),
    [sym_static_ip_binding] = ACTIONS(1843),
    [sym_get_dns_list] = ACTIONS(1843),
    [sym_set_dns_list] = ACTIONS(1843),
    [sym_get_ip_address] = ACTIONS(1843),
    [sym_set_ip_address] = ACTIONS(1843),
    [sym_get_url_list] = ACTIONS(1843),
    [sym_add_url_entry] = ACTIONS(1843),
    [sym_delete_url_entry] = ACTIONS(1843),
    [sym_abs_value] = ACTIONS(1843),
    [sym_max_value] = ACTIONS(1843),
    [sym_min_value] = ACTIONS(1843),
    [sym_exp_value] = ACTIONS(1843),
    [sym_log_value] = ACTIONS(1843),
    [sym_log10_value] = ACTIONS(1843),
    [sym_power_value] = ACTIONS(1843),
    [sym_sqrt_value] = ACTIONS(1843),
    [sym_raw_be] = ACTIONS(1843),
    [sym_raw_le] = ACTIONS(1843),
    [sym_internal_queue_size_set] = ACTIONS(1843),
    [sym_internal_queue_size_get] = ACTIONS(1843),
    [sym_internal_threshold_set] = ACTIONS(1843),
    [sym_internal_threshold_get] = ACTIONS(1843),
    [sym_rebuilt_event] = ACTIONS(1843),
    [sym_do_custom_event] = ACTIONS(1843),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1843),
    [sym_clkmgr_set_clk_source] = ACTIONS(1843),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1843),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1843),
    [sym_clkmgr_get_timezone] = ACTIONS(1843),
    [sym_clkmgr_set_timezone] = ACTIONS(1843),
    [sym_clkmgr_get_resync_period] = ACTIONS(1843),
    [sym_clkmgr_set_resync_period] = ACTIONS(1843),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1843),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1843),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1843),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1843),
    [sym_clkmgr_get_timeservers] = ACTIONS(1843),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1843),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1843),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1843),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1843),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1843),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1843),
    [sym_set_log_level] = ACTIONS(1843),
    [sym_get_log_level] = ACTIONS(1843),
    [sym_amx_log] = ACTIONS(1843),
    [sym_smtp_server_config_set] = ACTIONS(1843),
    [sym_smtp_server_config_get] = ACTIONS(1843),
    [sym_smtp_send] = ACTIONS(1843),
    [sym_validate_netlinx_account] = ACTIONS(1843),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1843),
    [sym_authenticate_certificate] = ACTIONS(1843),
    [sym_audit_netlinx_session_event] = ACTIONS(1843),
    [sym_audit_netlinx_generic_event] = ACTIONS(1843),
    [sym_get_available_flash_disk_space] = ACTIONS(1843),
    [sym_get_max_flash_disk_space] = ACTIONS(1843),
    [sym_data_create_feed] = ACTIONS(1843),
    [sym_data_delete_feed] = ACTIONS(1843),
    [sym_data_publish_feed] = ACTIONS(1843),
    [sym_data_get_published_feed] = ACTIONS(1843),
    [sym_data_add_record] = ACTIONS(1843),
    [sym_data_get_event_record] = ACTIONS(1843),
    [sym__wc_data_create_feed] = ACTIONS(1843),
    [sym__wc_data_add_record] = ACTIONS(1843),
    [sym__wc_data_get_event_record] = ACTIONS(1843),
    [sym_format] = ACTIONS(1843),
    [sym_astro_clock] = ACTIONS(1843),
    [sym_set_outdoor_temperature] = ACTIONS(1843),
    [sym_set_virtual_level_count] = ACTIONS(1843),
    [sym_set_virtual_channel_count] = ACTIONS(1843),
    [sym_set_virtual_port_count] = ACTIONS(1843),
    [sym__wc_encode_int] = ACTIONS(1843),
    [sym__wc_decode_int] = ACTIONS(1843),
    [sym__wc_int] = ACTIONS(1843),
    [sym__wc_to_ch_int] = ACTIONS(1843),
    [sym__ch_to_wc_int] = ACTIONS(1843),
    [sym__wc_find_string_int] = ACTIONS(1843),
    [sym__wc_left_string_int] = ACTIONS(1843),
    [sym__wc_lower_string_int] = ACTIONS(1843),
    [sym__wc_mid_string_int] = ACTIONS(1843),
    [sym__wc_remove_string_int] = ACTIONS(1843),
    [sym__wc_right_string_int] = ACTIONS(1843),
    [sym__wc_upper_string_int] = ACTIONS(1843),
    [sym__wc_compare_string_int] = ACTIONS(1843),
    [sym__wc_get_buffer_char_int] = ACTIONS(1843),
    [sym__wc_get_buffer_string_int] = ACTIONS(1843),
    [sym__wc_concat_string_int] = ACTIONS(1843),
    [sym__wc_file_open_int] = ACTIONS(1843),
    [sym__wc_file_close_int] = ACTIONS(1843),
    [sym__wc_file_read_int] = ACTIONS(1843),
    [sym__wc_file_read_line_int] = ACTIONS(1843),
    [sym__wc_file_write_int] = ACTIONS(1843),
    [sym__wc_file_write_line_int] = ACTIONS(1843),
    [aux_sym_preproc_include_token1] = ACTIONS(1843),
    [aux_sym_preproc_define_token1] = ACTIONS(1845),
    [aux_sym_preproc_warn_token1] = ACTIONS(1845),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1845),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1845),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1845),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1845),
    [aux_sym_preproc_else_token1] = ACTIONS(1845),
    [anon_sym_LBRACK] = ACTIONS(1845),
    [anon_sym_LBRACE] = ACTIONS(1845),
    [anon_sym_LPAREN] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [anon_sym_DASH] = ACTIONS(1843),
    [anon_sym_PLUS] = ACTIONS(1843),
    [anon_sym_DASH_DASH] = ACTIONS(1845),
    [anon_sym_PLUS_PLUS] = ACTIONS(1845),
    [anon_sym_DQUOTE] = ACTIONS(1845),
    [anon_sym_SQUOTE] = ACTIONS(1845),
    [sym_number_literal] = ACTIONS(1845),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(257)] = {
    [sym_identifier] = ACTIONS(1847),
    [sym_program_name_keyword] = ACTIONS(1847),
    [sym_module_name_keyword] = ACTIONS(1847),
    [sym_define_device_keyword] = ACTIONS(1847),
    [sym_define_combine_keyword] = ACTIONS(1847),
    [sym_define_constant_keyword] = ACTIONS(1847),
    [sym_define_type_keyword] = ACTIONS(1847),
    [sym_define_variable_keyword] = ACTIONS(1847),
    [sym_define_system_variable_keyword] = ACTIONS(1847),
    [sym_define_connect_level_keyword] = ACTIONS(1847),
    [sym_define_latching_keyword] = ACTIONS(1847),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1847),
    [sym_define_toggling_keyword] = ACTIONS(1847),
    [sym_define_call_keyword] = ACTIONS(1847),
    [sym_define_function_keyword] = ACTIONS(1847),
    [sym_define_library_function_keyword] = ACTIONS(1847),
    [sym_define_start_keyword] = ACTIONS(1847),
    [sym_define_event_keyword] = ACTIONS(1847),
    [sym_define_program_keyword] = ACTIONS(1847),
    [sym_define_module_keyword] = ACTIONS(1847),
    [sym_char_keyword] = ACTIONS(1847),
    [sym_widechar_keyword] = ACTIONS(1847),
    [sym_integer_keyword] = ACTIONS(1847),
    [sym_sinteger_keyword] = ACTIONS(1847),
    [sym_long_keyword] = ACTIONS(1847),
    [sym_slong_keyword] = ACTIONS(1847),
    [sym_float_keyword] = ACTIONS(1847),
    [sym_double_keyword] = ACTIONS(1847),
    [sym_variant_keyword] = ACTIONS(1847),
    [sym_variantarray_keyword] = ACTIONS(1847),
    [sym_local_var_keyword] = ACTIONS(1847),
    [sym_stack_var_keyword] = ACTIONS(1847),
    [sym_constant_keyword] = ACTIONS(1847),
    [sym_volatile_keyword] = ACTIONS(1847),
    [sym_non_volatile_keyword] = ACTIONS(1847),
    [sym_persistent_keyword] = ACTIONS(1847),
    [sym_dev_keyword] = ACTIONS(1847),
    [sym_devlev_keyword] = ACTIONS(1847),
    [sym_devchan_keyword] = ACTIONS(1847),
    [sym_if_keyword] = ACTIONS(1847),
    [sym_else_keyword] = ACTIONS(1847),
    [sym_switch_keyword] = ACTIONS(1847),
    [sym_case_keyword] = ACTIONS(1847),
    [sym_default_keyword] = ACTIONS(1847),
    [sym_while_keyword] = ACTIONS(1847),
    [sym_for_keyword] = ACTIONS(1847),
    [sym_break_keyword] = ACTIONS(1847),
    [sym_continue_keyword] = ACTIONS(1847),
    [sym_return_keyword] = ACTIONS(1847),
    [sym_select_keyword] = ACTIONS(1847),
    [sym_active_keyword] = ACTIONS(1847),
    [sym_struct_keyword] = ACTIONS(1847),
    [sym_structure_keyword] = ACTIONS(1847),
    [sym_button_event_keyword] = ACTIONS(1847),
    [sym_channel_event_keyword] = ACTIONS(1847),
    [sym_level_event_keyword] = ACTIONS(1847),
    [sym_data_event_keyword] = ACTIONS(1847),
    [sym_timeline_event_keyword] = ACTIONS(1847),
    [sym_custom_event_keyword] = ACTIONS(1847),
    [sym_push_keyword] = ACTIONS(1847),
    [sym_release_keyword] = ACTIONS(1847),
    [aux_sym_on_keyword_token1] = ACTIONS(1847),
    [aux_sym_off_keyword_token1] = ACTIONS(1847),
    [sym_send_level_keyword] = ACTIONS(1847),
    [sym_send_string_keyword] = ACTIONS(1847),
    [sym_send_command_keyword] = ACTIONS(1847),
    [sym_clear_buffer_keyword] = ACTIONS(1847),
    [sym_create_buffer_keyword] = ACTIONS(1847),
    [sym_create_multi_buffer_keyword] = ACTIONS(1847),
    [sym_call_keyword] = ACTIONS(1847),
    [sym_system_call_keyword] = ACTIONS(1847),
    [sym_devchan_to_keyword] = ACTIONS(1847),
    [sym_devchan_min_to_keyword] = ACTIONS(1847),
    [sym_devchan_total_off_keyword] = ACTIONS(1847),
    [sym_devchan_pulse_keyword] = ACTIONS(1847),
    [sym_wait_keyword] = ACTIONS(1847),
    [sym_cancel_wait_keyword] = ACTIONS(1847),
    [sym_wait_until_keyword] = ACTIONS(1847),
    [sym_cancel_wait_until_keyword] = ACTIONS(1847),
    [sym_cancel_all_wait_keyword] = ACTIONS(1847),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1847),
    [sym_not] = ACTIONS(1847),
    [sym_bnot] = ACTIONS(1847),
    [sym___file__] = ACTIONS(1847),
    [sym___date__] = ACTIONS(1847),
    [sym___ldate__] = ACTIONS(1847),
    [sym___time__] = ACTIONS(1847),
    [sym___line__] = ACTIONS(1847),
    [sym___name__] = ACTIONS(1847),
    [sym___netlinx__] = ACTIONS(1847),
    [sym_netlinx_axi_version] = ACTIONS(1847),
    [sym_url_flg_tcp] = ACTIONS(1847),
    [sym_url_flg_accinfopresent] = ACTIONS(1847),
    [sym_url_flg_temp] = ACTIONS(1847),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1847),
    [sym_url_flg_stat_mask] = ACTIONS(1847),
    [sym_url_flg_stat_lookup] = ACTIONS(1847),
    [sym_url_flg_stat_connecting] = ACTIONS(1847),
    [sym_url_flg_stat_waiting] = ACTIONS(1847),
    [sym_url_flg_stat_connected] = ACTIONS(1847),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1847),
    [sym_first_virtual_device] = ACTIONS(1847),
    [sym_first_local_port] = ACTIONS(1847),
    [sym_dynamic_virtual_device] = ACTIONS(1847),
    [sym_timeline_once] = ACTIONS(1847),
    [sym_timeline_repeat] = ACTIONS(1847),
    [sym_timeline_absolute] = ACTIONS(1847),
    [sym_timeline_relative] = ACTIONS(1847),
    [sym_do_push_timed_infinite] = ACTIONS(1847),
    [sym_source_type_no_address] = ACTIONS(1847),
    [sym_source_type_neuron_id] = ACTIONS(1847),
    [sym_source_type_ip_address] = ACTIONS(1847),
    [sym_source_type_axlink] = ACTIONS(1847),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1847),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1847),
    [sym_source_type_ip_socket_address] = ACTIONS(1847),
    [sym_source_type_rs232] = ACTIONS(1847),
    [sym_source_type_internal] = ACTIONS(1847),
    [sym_source_type_mac_address] = ACTIONS(1847),
    [sym_source_type_ipv4_port] = ACTIONS(1847),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1847),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1847),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1847),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1847),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1847),
    [sym_true] = ACTIONS(1847),
    [sym_false] = ACTIONS(1847),
    [sym_file_read_only] = ACTIONS(1847),
    [sym_file_rw_new] = ACTIONS(1847),
    [sym_file_rw_append] = ACTIONS(1847),
    [sym_ip_tcp] = ACTIONS(1847),
    [sym_ip_udp] = ACTIONS(1847),
    [sym_ip_udp_2way] = ACTIONS(1847),
    [sym_xml_encode_types] = ACTIONS(1847),
    [sym_xml_encode_char_as_list] = ACTIONS(1847),
    [sym_xml_encode_le] = ACTIONS(1847),
    [sym_xml_decode_types] = ACTIONS(1847),
    [sym_xml_decode_no_preserve] = ACTIONS(1847),
    [sym_duet_dev_type_display_device] = ACTIONS(1847),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1847),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1847),
    [sym_duet_dev_type_io_device] = ACTIONS(1847),
    [sym_duet_dev_type_relay_device] = ACTIONS(1847),
    [sym_duet_dev_type_ups] = ACTIONS(1847),
    [sym_duet_dev_type_amplifier] = ACTIONS(1847),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1847),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1847),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1847),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1847),
    [sym_duet_dev_type_camera] = ACTIONS(1847),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1847),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1847),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1847),
    [sym_duet_dev_type_dss] = ACTIONS(1847),
    [sym_duet_dev_type_dvr] = ACTIONS(1847),
    [sym_duet_dev_type_disc_device] = ACTIONS(1847),
    [sym_duet_dev_type_document_camera] = ACTIONS(1847),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1847),
    [sym_duet_dev_type_hvac] = ACTIONS(1847),
    [sym_duet_dev_type_keypad] = ACTIONS(1847),
    [sym_duet_dev_type_light] = ACTIONS(1847),
    [sym_duet_dev_type_monitor] = ACTIONS(1847),
    [sym_duet_dev_type_motor] = ACTIONS(1847),
    [sym_duet_dev_type_multi_window] = ACTIONS(1847),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1847),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1847),
    [sym_duet_dev_type_receiver] = ACTIONS(1847),
    [sym_duet_dev_type_security_system] = ACTIONS(1847),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1847),
    [sym_duet_dev_type_settop_box] = ACTIONS(1847),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1847),
    [sym_duet_dev_type_switcher] = ACTIONS(1847),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1847),
    [sym_duet_dev_type_tv] = ACTIONS(1847),
    [sym_duet_dev_type_utility] = ACTIONS(1847),
    [sym_duet_dev_type_vcr] = ACTIONS(1847),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1847),
    [sym_duet_dev_type_video_processor] = ACTIONS(1847),
    [sym_duet_dev_type_video_projector] = ACTIONS(1847),
    [sym_duet_dev_type_video_wall] = ACTIONS(1847),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1847),
    [sym_duet_dev_type_weather] = ACTIONS(1847),
    [sym_duet_dev_not_polled] = ACTIONS(1847),
    [sym_duet_dev_polled] = ACTIONS(1847),
    [sym_clkmgr_mode_network] = ACTIONS(1847),
    [sym_clkmgr_mode_standalone] = ACTIONS(1847),
    [sym_normal_standby] = ACTIONS(1847),
    [sym_normal_wake] = ACTIONS(1847),
    [sym_ip_multicast_ttl_option] = ACTIONS(1847),
    [sym_ip_ttl_subnet] = ACTIONS(1847),
    [sym_ip_ttl_site] = ACTIONS(1847),
    [sym_ip_ttl_region] = ACTIONS(1847),
    [sym_ip_ttl_continent] = ACTIONS(1847),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1847),
    [sym_ip_nodelay_on] = ACTIONS(1847),
    [sym_ip_nodelay_off] = ACTIONS(1847),
    [sym_amx_error] = ACTIONS(1847),
    [sym_amx_warning] = ACTIONS(1847),
    [sym_amx_info] = ACTIONS(1847),
    [sym_amx_debug] = ACTIONS(1847),
    [sym_netlinx_login_success] = ACTIONS(1847),
    [sym_netlinx_login_fail] = ACTIONS(1847),
    [sym_netlinx_logout] = ACTIONS(1847),
    [sym_null_str] = ACTIONS(1847),
    [sym_smtp_address] = ACTIONS(1847),
    [sym_smtp_port_number] = ACTIONS(1847),
    [sym_smtp_username] = ACTIONS(1847),
    [sym_smtp_password] = ACTIONS(1847),
    [sym_smtp_require_tls] = ACTIONS(1847),
    [sym_smtp_from] = ACTIONS(1847),
    [sym_smtp_tls_true] = ACTIONS(1847),
    [sym_smtp_tls_false] = ACTIONS(1847),
    [sym_listview_on_row_select_event] = ACTIONS(1847),
    [sym_data_structure_unknown] = ACTIONS(1847),
    [sym_data_structure_datafeed] = ACTIONS(1847),
    [sym_data_structure_datarecordset] = ACTIONS(1847),
    [sym_data_structure_datarecord] = ACTIONS(1847),
    [sym_data_structure_datafield] = ACTIONS(1847),
    [sym_data_max_content_field_count] = ACTIONS(1847),
    [sym_data_max_metadata_field_count] = ACTIONS(1847),
    [sym_data_max_id_length] = ACTIONS(1847),
    [sym_data_max_name_length] = ACTIONS(1847),
    [sym_data_max_description_length] = ACTIONS(1847),
    [sym_data_max_type_length] = ACTIONS(1847),
    [sym_data_max_format_length] = ACTIONS(1847),
    [sym_data_max_source_length] = ACTIONS(1847),
    [sym_data_max_value_length] = ACTIONS(1847),
    [sym_data_max_label_length] = ACTIONS(1847),
    [sym_data_max_url_length] = ACTIONS(1847),
    [sym_max_last_login_info_length] = ACTIONS(1847),
    [sym_data_type_unknown] = ACTIONS(1847),
    [sym_data_type_string] = ACTIONS(1847),
    [sym_data_type_datetime] = ACTIONS(1847),
    [sym_data_type_date] = ACTIONS(1847),
    [sym_data_type_time] = ACTIONS(1847),
    [sym_data_type_image] = ACTIONS(1847),
    [sym_data_format_url] = ACTIONS(1847),
    [sym_data_format_phone] = ACTIONS(1847),
    [sym_data_format_email] = ACTIONS(1847),
    [sym_data_format_iso8601] = ACTIONS(1847),
    [sym_auditlog_priv] = ACTIONS(1847),
    [sym_configuration_priv] = ACTIONS(1847),
    [sym_deviceconfig_priv] = ACTIONS(1847),
    [sym_ftp_priv] = ACTIONS(1847),
    [sym_http_priv] = ACTIONS(1847),
    [sym_networkconfig_priv] = ACTIONS(1847),
    [sym_programport_priv] = ACTIONS(1847),
    [sym_remoteui_priv] = ACTIONS(1847),
    [sym_securitycontrol_priv] = ACTIONS(1847),
    [sym_softwaremanagement_priv] = ACTIONS(1847),
    [sym_terminal_priv] = ACTIONS(1847),
    [sym_tpadmin_priv] = ACTIONS(1847),
    [sym_user1_priv] = ACTIONS(1847),
    [sym_user2_priv] = ACTIONS(1847),
    [sym_user3_priv] = ACTIONS(1847),
    [sym_user4_priv] = ACTIONS(1847),
    [sym_usermanagement_priv] = ACTIONS(1847),
    [sym_valid_account] = ACTIONS(1847),
    [sym_err_name_invalid] = ACTIONS(1847),
    [sym_err_password_invalid] = ACTIONS(1847),
    [sym_err_invalid_account] = ACTIONS(1847),
    [sym_err_no_authorization] = ACTIONS(1847),
    [sym_err_invalid_parameter] = ACTIONS(1847),
    [sym_err_user_locked_out] = ACTIONS(1847),
    [sym_err_user_account_expired] = ACTIONS(1847),
    [sym_err_general_error] = ACTIONS(1847),
    [sym_err_authentication_error] = ACTIONS(1847),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1847),
    [sym_tls_validate_certificate] = ACTIONS(1847),
    [sym_tdata] = ACTIONS(1847),
    [sym_tchannel] = ACTIONS(1847),
    [sym_tlevel] = ACTIONS(1847),
    [sym_tbutton] = ACTIONS(1847),
    [sym_ttimeline] = ACTIONS(1847),
    [sym_tcustom] = ACTIONS(1847),
    [sym_url_struct] = ACTIONS(1847),
    [sym_dns_struct] = ACTIONS(1847),
    [sym_ip_address_struct] = ACTIONS(1847),
    [sym_dev_info_struct] = ACTIONS(1847),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1847),
    [sym_data_feed] = ACTIONS(1847),
    [sym_data_field] = ACTIONS(1847),
    [sym_data_record] = ACTIONS(1847),
    [sym_wc_data_feed] = ACTIONS(1847),
    [sym_wc_data_field] = ACTIONS(1847),
    [sym_wc_data_record] = ACTIONS(1847),
    [sym_last_login_info] = ACTIONS(1847),
    [sym____reserved___] = ACTIONS(1847),
    [sym_date] = ACTIONS(1847),
    [sym_day] = ACTIONS(1847),
    [sym_time] = ACTIONS(1847),
    [sym_ldate] = ACTIONS(1847),
    [sym_push_device] = ACTIONS(1847),
    [sym_push_channel] = ACTIONS(1847),
    [sym_push_devchan] = ACTIONS(1847),
    [sym_release_device] = ACTIONS(1847),
    [sym_release_channel] = ACTIONS(1847),
    [sym_release_devchan] = ACTIONS(1847),
    [sym_master_slot] = ACTIONS(1847),
    [sym_get_pulse_time] = ACTIONS(1847),
    [sym_get_timer] = ACTIONS(1847),
    [sym_master_sn] = ACTIONS(1847),
    [sym_system_number] = ACTIONS(1847),
    [sym_button] = ACTIONS(1847),
    [sym_dv_channel] = ACTIONS(1847),
    [sym_data] = ACTIONS(1847),
    [sym_level] = ACTIONS(1847),
    [sym_timeline] = ACTIONS(1847),
    [sym_channel] = ACTIONS(1847),
    [sym_custom] = ACTIONS(1847),
    [sym_atoi] = ACTIONS(1847),
    [sym_atol] = ACTIONS(1847),
    [sym_atof] = ACTIONS(1847),
    [sym_itoa] = ACTIONS(1847),
    [sym_itohex] = ACTIONS(1847),
    [sym_hextoi] = ACTIONS(1847),
    [sym_ftoa] = ACTIONS(1847),
    [sym_find_string] = ACTIONS(1847),
    [sym_left_string] = ACTIONS(1847),
    [sym_right_string] = ACTIONS(1847),
    [sym_mid_string] = ACTIONS(1847),
    [sym_remove_string] = ACTIONS(1847),
    [sym_get_buffer_string] = ACTIONS(1847),
    [sym_get_buffer_char] = ACTIONS(1847),
    [sym_get_multi_buffer_string] = ACTIONS(1847),
    [sym_length_string] = ACTIONS(1847),
    [sym_max_length_string] = ACTIONS(1847),
    [sym_set_length_string] = ACTIONS(1847),
    [sym_compare_string] = ACTIONS(1847),
    [sym_upper_string] = ACTIONS(1847),
    [sym_lower_string] = ACTIONS(1847),
    [sym_get_last] = ACTIONS(1847),
    [sym_date_to_day] = ACTIONS(1847),
    [sym_date_to_month] = ACTIONS(1847),
    [sym_date_to_year] = ACTIONS(1847),
    [sym_time_to_hour] = ACTIONS(1847),
    [sym_time_to_minute] = ACTIONS(1847),
    [sym_time_to_second] = ACTIONS(1847),
    [sym_day_of_week] = ACTIONS(1847),
    [sym_set_pulse_time] = ACTIONS(1847),
    [sym_set_timer] = ACTIONS(1847),
    [sym_file_open] = ACTIONS(1847),
    [sym_file_close] = ACTIONS(1847),
    [sym_file_read] = ACTIONS(1847),
    [sym_file_read_line] = ACTIONS(1847),
    [sym_file_write] = ACTIONS(1847),
    [sym_file_write_line] = ACTIONS(1847),
    [sym_file_copy] = ACTIONS(1847),
    [sym_file_delete] = ACTIONS(1847),
    [sym_file_rename] = ACTIONS(1847),
    [sym_file_dir] = ACTIONS(1847),
    [sym_file_getdir] = ACTIONS(1847),
    [sym_file_setdir] = ACTIONS(1847),
    [sym_file_create_dir] = ACTIONS(1847),
    [sym_file_remove_dir] = ACTIONS(1847),
    [sym_file_seek] = ACTIONS(1847),
    [sym_length_array] = ACTIONS(1847),
    [sym_max_length_array] = ACTIONS(1847),
    [sym_set_length_array] = ACTIONS(1847),
    [sym_ip_client_open] = ACTIONS(1847),
    [sym_ip_client_close] = ACTIONS(1847),
    [sym_ip_server_open] = ACTIONS(1847),
    [sym_ip_server_close] = ACTIONS(1847),
    [sym_ip_bound_client_open] = ACTIONS(1847),
    [sym_ip_mc_server_open] = ACTIONS(1847),
    [sym_ip_set_option] = ACTIONS(1847),
    [sym_ssh_client_open] = ACTIONS(1847),
    [sym_ssh_client_close] = ACTIONS(1847),
    [sym_tls_client_open] = ACTIONS(1847),
    [sym_tls_client_close] = ACTIONS(1847),
    [sym_do_push] = ACTIONS(1847),
    [sym_do_push_timed] = ACTIONS(1847),
    [sym_do_release] = ACTIONS(1847),
    [sym_device_id] = ACTIONS(1847),
    [sym_device_info] = ACTIONS(1847),
    [sym_device_id_string] = ACTIONS(1847),
    [sym_device_standby] = ACTIONS(1847),
    [sym_device_wake] = ACTIONS(1847),
    [sym_timeline_create] = ACTIONS(1847),
    [sym_timeline_kill] = ACTIONS(1847),
    [sym_timeline_reload] = ACTIONS(1847),
    [sym_timeline_pause] = ACTIONS(1847),
    [sym_timeline_restart] = ACTIONS(1847),
    [sym_timeline_set] = ACTIONS(1847),
    [sym_timeline_get] = ACTIONS(1847),
    [sym_timeline_active] = ACTIONS(1847),
    [sym_timeline_dynamic_id] = ACTIONS(1847),
    [sym_variable_to_string] = ACTIONS(1847),
    [sym_string_to_variable] = ACTIONS(1847),
    [sym_length_variable_to_string] = ACTIONS(1847),
    [sym_type_cast] = ACTIONS(1847),
    [sym_variable_to_xml] = ACTIONS(1847),
    [sym_xml_to_variable] = ACTIONS(1847),
    [sym_length_variable_to_xml] = ACTIONS(1847),
    [sym_redirect_string] = ACTIONS(1847),
    [sym_random_number] = ACTIONS(1847),
    [sym_get_unique_id] = ACTIONS(1847),
    [sym_get_system_number] = ACTIONS(1847),
    [sym_set_system_number] = ACTIONS(1847),
    [sym_get_serial_number] = ACTIONS(1847),
    [sym_get_master_build] = ACTIONS(1847),
    [sym_set_validation_code] = ACTIONS(1847),
    [sym_reboot] = ACTIONS(1847),
    [sym_load_duet_module] = ACTIONS(1847),
    [sym_unload_duet_module] = ACTIONS(1847),
    [sym_duet_mem_size_set] = ACTIONS(1847),
    [sym_duet_mem_size_get] = ACTIONS(1847),
    [sym_dynamic_application_device] = ACTIONS(1847),
    [sym_dynamic_polled_port] = ACTIONS(1847),
    [sym_static_port_binding] = ACTIONS(1847),
    [sym_static_ip_binding] = ACTIONS(1847),
    [sym_get_dns_list] = ACTIONS(1847),
    [sym_set_dns_list] = ACTIONS(1847),
    [sym_get_ip_address] = ACTIONS(1847),
    [sym_set_ip_address] = ACTIONS(1847),
    [sym_get_url_list] = ACTIONS(1847),
    [sym_add_url_entry] = ACTIONS(1847),
    [sym_delete_url_entry] = ACTIONS(1847),
    [sym_abs_value] = ACTIONS(1847),
    [sym_max_value] = ACTIONS(1847),
    [sym_min_value] = ACTIONS(1847),
    [sym_exp_value] = ACTIONS(1847),
    [sym_log_value] = ACTIONS(1847),
    [sym_log10_value] = ACTIONS(1847),
    [sym_power_value] = ACTIONS(1847),
    [sym_sqrt_value] = ACTIONS(1847),
    [sym_raw_be] = ACTIONS(1847),
    [sym_raw_le] = ACTIONS(1847),
    [sym_internal_queue_size_set] = ACTIONS(1847),
    [sym_internal_queue_size_get] = ACTIONS(1847),
    [sym_internal_threshold_set] = ACTIONS(1847),
    [sym_internal_threshold_get] = ACTIONS(1847),
    [sym_rebuilt_event] = ACTIONS(1847),
    [sym_do_custom_event] = ACTIONS(1847),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1847),
    [sym_clkmgr_set_clk_source] = ACTIONS(1847),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1847),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1847),
    [sym_clkmgr_get_timezone] = ACTIONS(1847),
    [sym_clkmgr_set_timezone] = ACTIONS(1847),
    [sym_clkmgr_get_resync_period] = ACTIONS(1847),
    [sym_clkmgr_set_resync_period] = ACTIONS(1847),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1847),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1847),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1847),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1847),
    [sym_clkmgr_get_timeservers] = ACTIONS(1847),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1847),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1847),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1847),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1847),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1847),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1847),
    [sym_set_log_level] = ACTIONS(1847),
    [sym_get_log_level] = ACTIONS(1847),
    [sym_amx_log] = ACTIONS(1847),
    [sym_smtp_server_config_set] = ACTIONS(1847),
    [sym_smtp_server_config_get] = ACTIONS(1847),
    [sym_smtp_send] = ACTIONS(1847),
    [sym_validate_netlinx_account] = ACTIONS(1847),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1847),
    [sym_authenticate_certificate] = ACTIONS(1847),
    [sym_audit_netlinx_session_event] = ACTIONS(1847),
    [sym_audit_netlinx_generic_event] = ACTIONS(1847),
    [sym_get_available_flash_disk_space] = ACTIONS(1847),
    [sym_get_max_flash_disk_space] = ACTIONS(1847),
    [sym_data_create_feed] = ACTIONS(1847),
    [sym_data_delete_feed] = ACTIONS(1847),
    [sym_data_publish_feed] = ACTIONS(1847),
    [sym_data_get_published_feed] = ACTIONS(1847),
    [sym_data_add_record] = ACTIONS(1847),
    [sym_data_get_event_record] = ACTIONS(1847),
    [sym__wc_data_create_feed] = ACTIONS(1847),
    [sym__wc_data_add_record] = ACTIONS(1847),
    [sym__wc_data_get_event_record] = ACTIONS(1847),
    [sym_format] = ACTIONS(1847),
    [sym_astro_clock] = ACTIONS(1847),
    [sym_set_outdoor_temperature] = ACTIONS(1847),
    [sym_set_virtual_level_count] = ACTIONS(1847),
    [sym_set_virtual_channel_count] = ACTIONS(1847),
    [sym_set_virtual_port_count] = ACTIONS(1847),
    [sym__wc_encode_int] = ACTIONS(1847),
    [sym__wc_decode_int] = ACTIONS(1847),
    [sym__wc_int] = ACTIONS(1847),
    [sym__wc_to_ch_int] = ACTIONS(1847),
    [sym__ch_to_wc_int] = ACTIONS(1847),
    [sym__wc_find_string_int] = ACTIONS(1847),
    [sym__wc_left_string_int] = ACTIONS(1847),
    [sym__wc_lower_string_int] = ACTIONS(1847),
    [sym__wc_mid_string_int] = ACTIONS(1847),
    [sym__wc_remove_string_int] = ACTIONS(1847),
    [sym__wc_right_string_int] = ACTIONS(1847),
    [sym__wc_upper_string_int] = ACTIONS(1847),
    [sym__wc_compare_string_int] = ACTIONS(1847),
    [sym__wc_get_buffer_char_int] = ACTIONS(1847),
    [sym__wc_get_buffer_string_int] = ACTIONS(1847),
    [sym__wc_concat_string_int] = ACTIONS(1847),
    [sym__wc_file_open_int] = ACTIONS(1847),
    [sym__wc_file_close_int] = ACTIONS(1847),
    [sym__wc_file_read_int] = ACTIONS(1847),
    [sym__wc_file_read_line_int] = ACTIONS(1847),
    [sym__wc_file_write_int] = ACTIONS(1847),
    [sym__wc_file_write_line_int] = ACTIONS(1847),
    [aux_sym_preproc_include_token1] = ACTIONS(1847),
    [aux_sym_preproc_define_token1] = ACTIONS(1849),
    [aux_sym_preproc_warn_token1] = ACTIONS(1849),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1849),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1849),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1849),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1849),
    [aux_sym_preproc_else_token1] = ACTIONS(1849),
    [anon_sym_LBRACK] = ACTIONS(1849),
    [anon_sym_LBRACE] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(1847),
    [anon_sym_BANG] = ACTIONS(1849),
    [anon_sym_TILDE] = ACTIONS(1849),
    [anon_sym_DASH] = ACTIONS(1847),
    [anon_sym_PLUS] = ACTIONS(1847),
    [anon_sym_DASH_DASH] = ACTIONS(1849),
    [anon_sym_PLUS_PLUS] = ACTIONS(1849),
    [anon_sym_DQUOTE] = ACTIONS(1849),
    [anon_sym_SQUOTE] = ACTIONS(1849),
    [sym_number_literal] = ACTIONS(1849),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(258)] = {
    [sym_identifier] = ACTIONS(1851),
    [sym_program_name_keyword] = ACTIONS(1851),
    [sym_module_name_keyword] = ACTIONS(1851),
    [sym_define_device_keyword] = ACTIONS(1851),
    [sym_define_combine_keyword] = ACTIONS(1851),
    [sym_define_constant_keyword] = ACTIONS(1851),
    [sym_define_type_keyword] = ACTIONS(1851),
    [sym_define_variable_keyword] = ACTIONS(1851),
    [sym_define_system_variable_keyword] = ACTIONS(1851),
    [sym_define_connect_level_keyword] = ACTIONS(1851),
    [sym_define_latching_keyword] = ACTIONS(1851),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1851),
    [sym_define_toggling_keyword] = ACTIONS(1851),
    [sym_define_call_keyword] = ACTIONS(1851),
    [sym_define_function_keyword] = ACTIONS(1851),
    [sym_define_library_function_keyword] = ACTIONS(1851),
    [sym_define_start_keyword] = ACTIONS(1851),
    [sym_define_event_keyword] = ACTIONS(1851),
    [sym_define_program_keyword] = ACTIONS(1851),
    [sym_define_module_keyword] = ACTIONS(1851),
    [sym_char_keyword] = ACTIONS(1851),
    [sym_widechar_keyword] = ACTIONS(1851),
    [sym_integer_keyword] = ACTIONS(1851),
    [sym_sinteger_keyword] = ACTIONS(1851),
    [sym_long_keyword] = ACTIONS(1851),
    [sym_slong_keyword] = ACTIONS(1851),
    [sym_float_keyword] = ACTIONS(1851),
    [sym_double_keyword] = ACTIONS(1851),
    [sym_variant_keyword] = ACTIONS(1851),
    [sym_variantarray_keyword] = ACTIONS(1851),
    [sym_local_var_keyword] = ACTIONS(1851),
    [sym_stack_var_keyword] = ACTIONS(1851),
    [sym_constant_keyword] = ACTIONS(1851),
    [sym_volatile_keyword] = ACTIONS(1851),
    [sym_non_volatile_keyword] = ACTIONS(1851),
    [sym_persistent_keyword] = ACTIONS(1851),
    [sym_dev_keyword] = ACTIONS(1851),
    [sym_devlev_keyword] = ACTIONS(1851),
    [sym_devchan_keyword] = ACTIONS(1851),
    [sym_if_keyword] = ACTIONS(1851),
    [sym_else_keyword] = ACTIONS(1851),
    [sym_switch_keyword] = ACTIONS(1851),
    [sym_case_keyword] = ACTIONS(1851),
    [sym_default_keyword] = ACTIONS(1851),
    [sym_while_keyword] = ACTIONS(1851),
    [sym_for_keyword] = ACTIONS(1851),
    [sym_break_keyword] = ACTIONS(1851),
    [sym_continue_keyword] = ACTIONS(1851),
    [sym_return_keyword] = ACTIONS(1851),
    [sym_select_keyword] = ACTIONS(1851),
    [sym_active_keyword] = ACTIONS(1851),
    [sym_struct_keyword] = ACTIONS(1851),
    [sym_structure_keyword] = ACTIONS(1851),
    [sym_button_event_keyword] = ACTIONS(1851),
    [sym_channel_event_keyword] = ACTIONS(1851),
    [sym_level_event_keyword] = ACTIONS(1851),
    [sym_data_event_keyword] = ACTIONS(1851),
    [sym_timeline_event_keyword] = ACTIONS(1851),
    [sym_custom_event_keyword] = ACTIONS(1851),
    [sym_push_keyword] = ACTIONS(1851),
    [sym_release_keyword] = ACTIONS(1851),
    [aux_sym_on_keyword_token1] = ACTIONS(1851),
    [aux_sym_off_keyword_token1] = ACTIONS(1851),
    [sym_send_level_keyword] = ACTIONS(1851),
    [sym_send_string_keyword] = ACTIONS(1851),
    [sym_send_command_keyword] = ACTIONS(1851),
    [sym_clear_buffer_keyword] = ACTIONS(1851),
    [sym_create_buffer_keyword] = ACTIONS(1851),
    [sym_create_multi_buffer_keyword] = ACTIONS(1851),
    [sym_call_keyword] = ACTIONS(1851),
    [sym_system_call_keyword] = ACTIONS(1851),
    [sym_devchan_to_keyword] = ACTIONS(1851),
    [sym_devchan_min_to_keyword] = ACTIONS(1851),
    [sym_devchan_total_off_keyword] = ACTIONS(1851),
    [sym_devchan_pulse_keyword] = ACTIONS(1851),
    [sym_wait_keyword] = ACTIONS(1851),
    [sym_cancel_wait_keyword] = ACTIONS(1851),
    [sym_wait_until_keyword] = ACTIONS(1851),
    [sym_cancel_wait_until_keyword] = ACTIONS(1851),
    [sym_cancel_all_wait_keyword] = ACTIONS(1851),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1851),
    [sym_not] = ACTIONS(1851),
    [sym_bnot] = ACTIONS(1851),
    [sym___file__] = ACTIONS(1851),
    [sym___date__] = ACTIONS(1851),
    [sym___ldate__] = ACTIONS(1851),
    [sym___time__] = ACTIONS(1851),
    [sym___line__] = ACTIONS(1851),
    [sym___name__] = ACTIONS(1851),
    [sym___netlinx__] = ACTIONS(1851),
    [sym_netlinx_axi_version] = ACTIONS(1851),
    [sym_url_flg_tcp] = ACTIONS(1851),
    [sym_url_flg_accinfopresent] = ACTIONS(1851),
    [sym_url_flg_temp] = ACTIONS(1851),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1851),
    [sym_url_flg_stat_mask] = ACTIONS(1851),
    [sym_url_flg_stat_lookup] = ACTIONS(1851),
    [sym_url_flg_stat_connecting] = ACTIONS(1851),
    [sym_url_flg_stat_waiting] = ACTIONS(1851),
    [sym_url_flg_stat_connected] = ACTIONS(1851),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1851),
    [sym_first_virtual_device] = ACTIONS(1851),
    [sym_first_local_port] = ACTIONS(1851),
    [sym_dynamic_virtual_device] = ACTIONS(1851),
    [sym_timeline_once] = ACTIONS(1851),
    [sym_timeline_repeat] = ACTIONS(1851),
    [sym_timeline_absolute] = ACTIONS(1851),
    [sym_timeline_relative] = ACTIONS(1851),
    [sym_do_push_timed_infinite] = ACTIONS(1851),
    [sym_source_type_no_address] = ACTIONS(1851),
    [sym_source_type_neuron_id] = ACTIONS(1851),
    [sym_source_type_ip_address] = ACTIONS(1851),
    [sym_source_type_axlink] = ACTIONS(1851),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1851),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1851),
    [sym_source_type_ip_socket_address] = ACTIONS(1851),
    [sym_source_type_rs232] = ACTIONS(1851),
    [sym_source_type_internal] = ACTIONS(1851),
    [sym_source_type_mac_address] = ACTIONS(1851),
    [sym_source_type_ipv4_port] = ACTIONS(1851),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1851),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1851),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1851),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1851),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1851),
    [sym_true] = ACTIONS(1851),
    [sym_false] = ACTIONS(1851),
    [sym_file_read_only] = ACTIONS(1851),
    [sym_file_rw_new] = ACTIONS(1851),
    [sym_file_rw_append] = ACTIONS(1851),
    [sym_ip_tcp] = ACTIONS(1851),
    [sym_ip_udp] = ACTIONS(1851),
    [sym_ip_udp_2way] = ACTIONS(1851),
    [sym_xml_encode_types] = ACTIONS(1851),
    [sym_xml_encode_char_as_list] = ACTIONS(1851),
    [sym_xml_encode_le] = ACTIONS(1851),
    [sym_xml_decode_types] = ACTIONS(1851),
    [sym_xml_decode_no_preserve] = ACTIONS(1851),
    [sym_duet_dev_type_display_device] = ACTIONS(1851),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1851),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1851),
    [sym_duet_dev_type_io_device] = ACTIONS(1851),
    [sym_duet_dev_type_relay_device] = ACTIONS(1851),
    [sym_duet_dev_type_ups] = ACTIONS(1851),
    [sym_duet_dev_type_amplifier] = ACTIONS(1851),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1851),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1851),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1851),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1851),
    [sym_duet_dev_type_camera] = ACTIONS(1851),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1851),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1851),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1851),
    [sym_duet_dev_type_dss] = ACTIONS(1851),
    [sym_duet_dev_type_dvr] = ACTIONS(1851),
    [sym_duet_dev_type_disc_device] = ACTIONS(1851),
    [sym_duet_dev_type_document_camera] = ACTIONS(1851),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1851),
    [sym_duet_dev_type_hvac] = ACTIONS(1851),
    [sym_duet_dev_type_keypad] = ACTIONS(1851),
    [sym_duet_dev_type_light] = ACTIONS(1851),
    [sym_duet_dev_type_monitor] = ACTIONS(1851),
    [sym_duet_dev_type_motor] = ACTIONS(1851),
    [sym_duet_dev_type_multi_window] = ACTIONS(1851),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1851),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1851),
    [sym_duet_dev_type_receiver] = ACTIONS(1851),
    [sym_duet_dev_type_security_system] = ACTIONS(1851),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1851),
    [sym_duet_dev_type_settop_box] = ACTIONS(1851),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1851),
    [sym_duet_dev_type_switcher] = ACTIONS(1851),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1851),
    [sym_duet_dev_type_tv] = ACTIONS(1851),
    [sym_duet_dev_type_utility] = ACTIONS(1851),
    [sym_duet_dev_type_vcr] = ACTIONS(1851),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1851),
    [sym_duet_dev_type_video_processor] = ACTIONS(1851),
    [sym_duet_dev_type_video_projector] = ACTIONS(1851),
    [sym_duet_dev_type_video_wall] = ACTIONS(1851),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1851),
    [sym_duet_dev_type_weather] = ACTIONS(1851),
    [sym_duet_dev_not_polled] = ACTIONS(1851),
    [sym_duet_dev_polled] = ACTIONS(1851),
    [sym_clkmgr_mode_network] = ACTIONS(1851),
    [sym_clkmgr_mode_standalone] = ACTIONS(1851),
    [sym_normal_standby] = ACTIONS(1851),
    [sym_normal_wake] = ACTIONS(1851),
    [sym_ip_multicast_ttl_option] = ACTIONS(1851),
    [sym_ip_ttl_subnet] = ACTIONS(1851),
    [sym_ip_ttl_site] = ACTIONS(1851),
    [sym_ip_ttl_region] = ACTIONS(1851),
    [sym_ip_ttl_continent] = ACTIONS(1851),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1851),
    [sym_ip_nodelay_on] = ACTIONS(1851),
    [sym_ip_nodelay_off] = ACTIONS(1851),
    [sym_amx_error] = ACTIONS(1851),
    [sym_amx_warning] = ACTIONS(1851),
    [sym_amx_info] = ACTIONS(1851),
    [sym_amx_debug] = ACTIONS(1851),
    [sym_netlinx_login_success] = ACTIONS(1851),
    [sym_netlinx_login_fail] = ACTIONS(1851),
    [sym_netlinx_logout] = ACTIONS(1851),
    [sym_null_str] = ACTIONS(1851),
    [sym_smtp_address] = ACTIONS(1851),
    [sym_smtp_port_number] = ACTIONS(1851),
    [sym_smtp_username] = ACTIONS(1851),
    [sym_smtp_password] = ACTIONS(1851),
    [sym_smtp_require_tls] = ACTIONS(1851),
    [sym_smtp_from] = ACTIONS(1851),
    [sym_smtp_tls_true] = ACTIONS(1851),
    [sym_smtp_tls_false] = ACTIONS(1851),
    [sym_listview_on_row_select_event] = ACTIONS(1851),
    [sym_data_structure_unknown] = ACTIONS(1851),
    [sym_data_structure_datafeed] = ACTIONS(1851),
    [sym_data_structure_datarecordset] = ACTIONS(1851),
    [sym_data_structure_datarecord] = ACTIONS(1851),
    [sym_data_structure_datafield] = ACTIONS(1851),
    [sym_data_max_content_field_count] = ACTIONS(1851),
    [sym_data_max_metadata_field_count] = ACTIONS(1851),
    [sym_data_max_id_length] = ACTIONS(1851),
    [sym_data_max_name_length] = ACTIONS(1851),
    [sym_data_max_description_length] = ACTIONS(1851),
    [sym_data_max_type_length] = ACTIONS(1851),
    [sym_data_max_format_length] = ACTIONS(1851),
    [sym_data_max_source_length] = ACTIONS(1851),
    [sym_data_max_value_length] = ACTIONS(1851),
    [sym_data_max_label_length] = ACTIONS(1851),
    [sym_data_max_url_length] = ACTIONS(1851),
    [sym_max_last_login_info_length] = ACTIONS(1851),
    [sym_data_type_unknown] = ACTIONS(1851),
    [sym_data_type_string] = ACTIONS(1851),
    [sym_data_type_datetime] = ACTIONS(1851),
    [sym_data_type_date] = ACTIONS(1851),
    [sym_data_type_time] = ACTIONS(1851),
    [sym_data_type_image] = ACTIONS(1851),
    [sym_data_format_url] = ACTIONS(1851),
    [sym_data_format_phone] = ACTIONS(1851),
    [sym_data_format_email] = ACTIONS(1851),
    [sym_data_format_iso8601] = ACTIONS(1851),
    [sym_auditlog_priv] = ACTIONS(1851),
    [sym_configuration_priv] = ACTIONS(1851),
    [sym_deviceconfig_priv] = ACTIONS(1851),
    [sym_ftp_priv] = ACTIONS(1851),
    [sym_http_priv] = ACTIONS(1851),
    [sym_networkconfig_priv] = ACTIONS(1851),
    [sym_programport_priv] = ACTIONS(1851),
    [sym_remoteui_priv] = ACTIONS(1851),
    [sym_securitycontrol_priv] = ACTIONS(1851),
    [sym_softwaremanagement_priv] = ACTIONS(1851),
    [sym_terminal_priv] = ACTIONS(1851),
    [sym_tpadmin_priv] = ACTIONS(1851),
    [sym_user1_priv] = ACTIONS(1851),
    [sym_user2_priv] = ACTIONS(1851),
    [sym_user3_priv] = ACTIONS(1851),
    [sym_user4_priv] = ACTIONS(1851),
    [sym_usermanagement_priv] = ACTIONS(1851),
    [sym_valid_account] = ACTIONS(1851),
    [sym_err_name_invalid] = ACTIONS(1851),
    [sym_err_password_invalid] = ACTIONS(1851),
    [sym_err_invalid_account] = ACTIONS(1851),
    [sym_err_no_authorization] = ACTIONS(1851),
    [sym_err_invalid_parameter] = ACTIONS(1851),
    [sym_err_user_locked_out] = ACTIONS(1851),
    [sym_err_user_account_expired] = ACTIONS(1851),
    [sym_err_general_error] = ACTIONS(1851),
    [sym_err_authentication_error] = ACTIONS(1851),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1851),
    [sym_tls_validate_certificate] = ACTIONS(1851),
    [sym_tdata] = ACTIONS(1851),
    [sym_tchannel] = ACTIONS(1851),
    [sym_tlevel] = ACTIONS(1851),
    [sym_tbutton] = ACTIONS(1851),
    [sym_ttimeline] = ACTIONS(1851),
    [sym_tcustom] = ACTIONS(1851),
    [sym_url_struct] = ACTIONS(1851),
    [sym_dns_struct] = ACTIONS(1851),
    [sym_ip_address_struct] = ACTIONS(1851),
    [sym_dev_info_struct] = ACTIONS(1851),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1851),
    [sym_data_feed] = ACTIONS(1851),
    [sym_data_field] = ACTIONS(1851),
    [sym_data_record] = ACTIONS(1851),
    [sym_wc_data_feed] = ACTIONS(1851),
    [sym_wc_data_field] = ACTIONS(1851),
    [sym_wc_data_record] = ACTIONS(1851),
    [sym_last_login_info] = ACTIONS(1851),
    [sym____reserved___] = ACTIONS(1851),
    [sym_date] = ACTIONS(1851),
    [sym_day] = ACTIONS(1851),
    [sym_time] = ACTIONS(1851),
    [sym_ldate] = ACTIONS(1851),
    [sym_push_device] = ACTIONS(1851),
    [sym_push_channel] = ACTIONS(1851),
    [sym_push_devchan] = ACTIONS(1851),
    [sym_release_device] = ACTIONS(1851),
    [sym_release_channel] = ACTIONS(1851),
    [sym_release_devchan] = ACTIONS(1851),
    [sym_master_slot] = ACTIONS(1851),
    [sym_get_pulse_time] = ACTIONS(1851),
    [sym_get_timer] = ACTIONS(1851),
    [sym_master_sn] = ACTIONS(1851),
    [sym_system_number] = ACTIONS(1851),
    [sym_button] = ACTIONS(1851),
    [sym_dv_channel] = ACTIONS(1851),
    [sym_data] = ACTIONS(1851),
    [sym_level] = ACTIONS(1851),
    [sym_timeline] = ACTIONS(1851),
    [sym_channel] = ACTIONS(1851),
    [sym_custom] = ACTIONS(1851),
    [sym_atoi] = ACTIONS(1851),
    [sym_atol] = ACTIONS(1851),
    [sym_atof] = ACTIONS(1851),
    [sym_itoa] = ACTIONS(1851),
    [sym_itohex] = ACTIONS(1851),
    [sym_hextoi] = ACTIONS(1851),
    [sym_ftoa] = ACTIONS(1851),
    [sym_find_string] = ACTIONS(1851),
    [sym_left_string] = ACTIONS(1851),
    [sym_right_string] = ACTIONS(1851),
    [sym_mid_string] = ACTIONS(1851),
    [sym_remove_string] = ACTIONS(1851),
    [sym_get_buffer_string] = ACTIONS(1851),
    [sym_get_buffer_char] = ACTIONS(1851),
    [sym_get_multi_buffer_string] = ACTIONS(1851),
    [sym_length_string] = ACTIONS(1851),
    [sym_max_length_string] = ACTIONS(1851),
    [sym_set_length_string] = ACTIONS(1851),
    [sym_compare_string] = ACTIONS(1851),
    [sym_upper_string] = ACTIONS(1851),
    [sym_lower_string] = ACTIONS(1851),
    [sym_get_last] = ACTIONS(1851),
    [sym_date_to_day] = ACTIONS(1851),
    [sym_date_to_month] = ACTIONS(1851),
    [sym_date_to_year] = ACTIONS(1851),
    [sym_time_to_hour] = ACTIONS(1851),
    [sym_time_to_minute] = ACTIONS(1851),
    [sym_time_to_second] = ACTIONS(1851),
    [sym_day_of_week] = ACTIONS(1851),
    [sym_set_pulse_time] = ACTIONS(1851),
    [sym_set_timer] = ACTIONS(1851),
    [sym_file_open] = ACTIONS(1851),
    [sym_file_close] = ACTIONS(1851),
    [sym_file_read] = ACTIONS(1851),
    [sym_file_read_line] = ACTIONS(1851),
    [sym_file_write] = ACTIONS(1851),
    [sym_file_write_line] = ACTIONS(1851),
    [sym_file_copy] = ACTIONS(1851),
    [sym_file_delete] = ACTIONS(1851),
    [sym_file_rename] = ACTIONS(1851),
    [sym_file_dir] = ACTIONS(1851),
    [sym_file_getdir] = ACTIONS(1851),
    [sym_file_setdir] = ACTIONS(1851),
    [sym_file_create_dir] = ACTIONS(1851),
    [sym_file_remove_dir] = ACTIONS(1851),
    [sym_file_seek] = ACTIONS(1851),
    [sym_length_array] = ACTIONS(1851),
    [sym_max_length_array] = ACTIONS(1851),
    [sym_set_length_array] = ACTIONS(1851),
    [sym_ip_client_open] = ACTIONS(1851),
    [sym_ip_client_close] = ACTIONS(1851),
    [sym_ip_server_open] = ACTIONS(1851),
    [sym_ip_server_close] = ACTIONS(1851),
    [sym_ip_bound_client_open] = ACTIONS(1851),
    [sym_ip_mc_server_open] = ACTIONS(1851),
    [sym_ip_set_option] = ACTIONS(1851),
    [sym_ssh_client_open] = ACTIONS(1851),
    [sym_ssh_client_close] = ACTIONS(1851),
    [sym_tls_client_open] = ACTIONS(1851),
    [sym_tls_client_close] = ACTIONS(1851),
    [sym_do_push] = ACTIONS(1851),
    [sym_do_push_timed] = ACTIONS(1851),
    [sym_do_release] = ACTIONS(1851),
    [sym_device_id] = ACTIONS(1851),
    [sym_device_info] = ACTIONS(1851),
    [sym_device_id_string] = ACTIONS(1851),
    [sym_device_standby] = ACTIONS(1851),
    [sym_device_wake] = ACTIONS(1851),
    [sym_timeline_create] = ACTIONS(1851),
    [sym_timeline_kill] = ACTIONS(1851),
    [sym_timeline_reload] = ACTIONS(1851),
    [sym_timeline_pause] = ACTIONS(1851),
    [sym_timeline_restart] = ACTIONS(1851),
    [sym_timeline_set] = ACTIONS(1851),
    [sym_timeline_get] = ACTIONS(1851),
    [sym_timeline_active] = ACTIONS(1851),
    [sym_timeline_dynamic_id] = ACTIONS(1851),
    [sym_variable_to_string] = ACTIONS(1851),
    [sym_string_to_variable] = ACTIONS(1851),
    [sym_length_variable_to_string] = ACTIONS(1851),
    [sym_type_cast] = ACTIONS(1851),
    [sym_variable_to_xml] = ACTIONS(1851),
    [sym_xml_to_variable] = ACTIONS(1851),
    [sym_length_variable_to_xml] = ACTIONS(1851),
    [sym_redirect_string] = ACTIONS(1851),
    [sym_random_number] = ACTIONS(1851),
    [sym_get_unique_id] = ACTIONS(1851),
    [sym_get_system_number] = ACTIONS(1851),
    [sym_set_system_number] = ACTIONS(1851),
    [sym_get_serial_number] = ACTIONS(1851),
    [sym_get_master_build] = ACTIONS(1851),
    [sym_set_validation_code] = ACTIONS(1851),
    [sym_reboot] = ACTIONS(1851),
    [sym_load_duet_module] = ACTIONS(1851),
    [sym_unload_duet_module] = ACTIONS(1851),
    [sym_duet_mem_size_set] = ACTIONS(1851),
    [sym_duet_mem_size_get] = ACTIONS(1851),
    [sym_dynamic_application_device] = ACTIONS(1851),
    [sym_dynamic_polled_port] = ACTIONS(1851),
    [sym_static_port_binding] = ACTIONS(1851),
    [sym_static_ip_binding] = ACTIONS(1851),
    [sym_get_dns_list] = ACTIONS(1851),
    [sym_set_dns_list] = ACTIONS(1851),
    [sym_get_ip_address] = ACTIONS(1851),
    [sym_set_ip_address] = ACTIONS(1851),
    [sym_get_url_list] = ACTIONS(1851),
    [sym_add_url_entry] = ACTIONS(1851),
    [sym_delete_url_entry] = ACTIONS(1851),
    [sym_abs_value] = ACTIONS(1851),
    [sym_max_value] = ACTIONS(1851),
    [sym_min_value] = ACTIONS(1851),
    [sym_exp_value] = ACTIONS(1851),
    [sym_log_value] = ACTIONS(1851),
    [sym_log10_value] = ACTIONS(1851),
    [sym_power_value] = ACTIONS(1851),
    [sym_sqrt_value] = ACTIONS(1851),
    [sym_raw_be] = ACTIONS(1851),
    [sym_raw_le] = ACTIONS(1851),
    [sym_internal_queue_size_set] = ACTIONS(1851),
    [sym_internal_queue_size_get] = ACTIONS(1851),
    [sym_internal_threshold_set] = ACTIONS(1851),
    [sym_internal_threshold_get] = ACTIONS(1851),
    [sym_rebuilt_event] = ACTIONS(1851),
    [sym_do_custom_event] = ACTIONS(1851),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1851),
    [sym_clkmgr_set_clk_source] = ACTIONS(1851),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1851),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1851),
    [sym_clkmgr_get_timezone] = ACTIONS(1851),
    [sym_clkmgr_set_timezone] = ACTIONS(1851),
    [sym_clkmgr_get_resync_period] = ACTIONS(1851),
    [sym_clkmgr_set_resync_period] = ACTIONS(1851),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1851),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1851),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1851),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1851),
    [sym_clkmgr_get_timeservers] = ACTIONS(1851),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1851),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1851),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1851),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1851),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1851),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1851),
    [sym_set_log_level] = ACTIONS(1851),
    [sym_get_log_level] = ACTIONS(1851),
    [sym_amx_log] = ACTIONS(1851),
    [sym_smtp_server_config_set] = ACTIONS(1851),
    [sym_smtp_server_config_get] = ACTIONS(1851),
    [sym_smtp_send] = ACTIONS(1851),
    [sym_validate_netlinx_account] = ACTIONS(1851),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1851),
    [sym_authenticate_certificate] = ACTIONS(1851),
    [sym_audit_netlinx_session_event] = ACTIONS(1851),
    [sym_audit_netlinx_generic_event] = ACTIONS(1851),
    [sym_get_available_flash_disk_space] = ACTIONS(1851),
    [sym_get_max_flash_disk_space] = ACTIONS(1851),
    [sym_data_create_feed] = ACTIONS(1851),
    [sym_data_delete_feed] = ACTIONS(1851),
    [sym_data_publish_feed] = ACTIONS(1851),
    [sym_data_get_published_feed] = ACTIONS(1851),
    [sym_data_add_record] = ACTIONS(1851),
    [sym_data_get_event_record] = ACTIONS(1851),
    [sym__wc_data_create_feed] = ACTIONS(1851),
    [sym__wc_data_add_record] = ACTIONS(1851),
    [sym__wc_data_get_event_record] = ACTIONS(1851),
    [sym_format] = ACTIONS(1851),
    [sym_astro_clock] = ACTIONS(1851),
    [sym_set_outdoor_temperature] = ACTIONS(1851),
    [sym_set_virtual_level_count] = ACTIONS(1851),
    [sym_set_virtual_channel_count] = ACTIONS(1851),
    [sym_set_virtual_port_count] = ACTIONS(1851),
    [sym__wc_encode_int] = ACTIONS(1851),
    [sym__wc_decode_int] = ACTIONS(1851),
    [sym__wc_int] = ACTIONS(1851),
    [sym__wc_to_ch_int] = ACTIONS(1851),
    [sym__ch_to_wc_int] = ACTIONS(1851),
    [sym__wc_find_string_int] = ACTIONS(1851),
    [sym__wc_left_string_int] = ACTIONS(1851),
    [sym__wc_lower_string_int] = ACTIONS(1851),
    [sym__wc_mid_string_int] = ACTIONS(1851),
    [sym__wc_remove_string_int] = ACTIONS(1851),
    [sym__wc_right_string_int] = ACTIONS(1851),
    [sym__wc_upper_string_int] = ACTIONS(1851),
    [sym__wc_compare_string_int] = ACTIONS(1851),
    [sym__wc_get_buffer_char_int] = ACTIONS(1851),
    [sym__wc_get_buffer_string_int] = ACTIONS(1851),
    [sym__wc_concat_string_int] = ACTIONS(1851),
    [sym__wc_file_open_int] = ACTIONS(1851),
    [sym__wc_file_close_int] = ACTIONS(1851),
    [sym__wc_file_read_int] = ACTIONS(1851),
    [sym__wc_file_read_line_int] = ACTIONS(1851),
    [sym__wc_file_write_int] = ACTIONS(1851),
    [sym__wc_file_write_line_int] = ACTIONS(1851),
    [aux_sym_preproc_include_token1] = ACTIONS(1851),
    [aux_sym_preproc_define_token1] = ACTIONS(1853),
    [aux_sym_preproc_warn_token1] = ACTIONS(1853),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1853),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1853),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1853),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1853),
    [aux_sym_preproc_else_token1] = ACTIONS(1853),
    [anon_sym_LBRACK] = ACTIONS(1853),
    [anon_sym_LBRACE] = ACTIONS(1853),
    [anon_sym_LPAREN] = ACTIONS(1851),
    [anon_sym_BANG] = ACTIONS(1853),
    [anon_sym_TILDE] = ACTIONS(1853),
    [anon_sym_DASH] = ACTIONS(1851),
    [anon_sym_PLUS] = ACTIONS(1851),
    [anon_sym_DASH_DASH] = ACTIONS(1853),
    [anon_sym_PLUS_PLUS] = ACTIONS(1853),
    [anon_sym_DQUOTE] = ACTIONS(1853),
    [anon_sym_SQUOTE] = ACTIONS(1853),
    [sym_number_literal] = ACTIONS(1853),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(259)] = {
    [sym_identifier] = ACTIONS(1855),
    [sym_program_name_keyword] = ACTIONS(1855),
    [sym_module_name_keyword] = ACTIONS(1855),
    [sym_define_device_keyword] = ACTIONS(1855),
    [sym_define_combine_keyword] = ACTIONS(1855),
    [sym_define_constant_keyword] = ACTIONS(1855),
    [sym_define_type_keyword] = ACTIONS(1855),
    [sym_define_variable_keyword] = ACTIONS(1855),
    [sym_define_system_variable_keyword] = ACTIONS(1855),
    [sym_define_connect_level_keyword] = ACTIONS(1855),
    [sym_define_latching_keyword] = ACTIONS(1855),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1855),
    [sym_define_toggling_keyword] = ACTIONS(1855),
    [sym_define_call_keyword] = ACTIONS(1855),
    [sym_define_function_keyword] = ACTIONS(1855),
    [sym_define_library_function_keyword] = ACTIONS(1855),
    [sym_define_start_keyword] = ACTIONS(1855),
    [sym_define_event_keyword] = ACTIONS(1855),
    [sym_define_program_keyword] = ACTIONS(1855),
    [sym_define_module_keyword] = ACTIONS(1855),
    [sym_char_keyword] = ACTIONS(1855),
    [sym_widechar_keyword] = ACTIONS(1855),
    [sym_integer_keyword] = ACTIONS(1855),
    [sym_sinteger_keyword] = ACTIONS(1855),
    [sym_long_keyword] = ACTIONS(1855),
    [sym_slong_keyword] = ACTIONS(1855),
    [sym_float_keyword] = ACTIONS(1855),
    [sym_double_keyword] = ACTIONS(1855),
    [sym_variant_keyword] = ACTIONS(1855),
    [sym_variantarray_keyword] = ACTIONS(1855),
    [sym_local_var_keyword] = ACTIONS(1855),
    [sym_stack_var_keyword] = ACTIONS(1855),
    [sym_constant_keyword] = ACTIONS(1855),
    [sym_volatile_keyword] = ACTIONS(1855),
    [sym_non_volatile_keyword] = ACTIONS(1855),
    [sym_persistent_keyword] = ACTIONS(1855),
    [sym_dev_keyword] = ACTIONS(1855),
    [sym_devlev_keyword] = ACTIONS(1855),
    [sym_devchan_keyword] = ACTIONS(1855),
    [sym_if_keyword] = ACTIONS(1855),
    [sym_else_keyword] = ACTIONS(1855),
    [sym_switch_keyword] = ACTIONS(1855),
    [sym_case_keyword] = ACTIONS(1855),
    [sym_default_keyword] = ACTIONS(1855),
    [sym_while_keyword] = ACTIONS(1855),
    [sym_for_keyword] = ACTIONS(1855),
    [sym_break_keyword] = ACTIONS(1855),
    [sym_continue_keyword] = ACTIONS(1855),
    [sym_return_keyword] = ACTIONS(1855),
    [sym_select_keyword] = ACTIONS(1855),
    [sym_active_keyword] = ACTIONS(1855),
    [sym_struct_keyword] = ACTIONS(1855),
    [sym_structure_keyword] = ACTIONS(1855),
    [sym_button_event_keyword] = ACTIONS(1855),
    [sym_channel_event_keyword] = ACTIONS(1855),
    [sym_level_event_keyword] = ACTIONS(1855),
    [sym_data_event_keyword] = ACTIONS(1855),
    [sym_timeline_event_keyword] = ACTIONS(1855),
    [sym_custom_event_keyword] = ACTIONS(1855),
    [sym_push_keyword] = ACTIONS(1855),
    [sym_release_keyword] = ACTIONS(1855),
    [aux_sym_on_keyword_token1] = ACTIONS(1855),
    [aux_sym_off_keyword_token1] = ACTIONS(1855),
    [sym_send_level_keyword] = ACTIONS(1855),
    [sym_send_string_keyword] = ACTIONS(1855),
    [sym_send_command_keyword] = ACTIONS(1855),
    [sym_clear_buffer_keyword] = ACTIONS(1855),
    [sym_create_buffer_keyword] = ACTIONS(1855),
    [sym_create_multi_buffer_keyword] = ACTIONS(1855),
    [sym_call_keyword] = ACTIONS(1855),
    [sym_system_call_keyword] = ACTIONS(1855),
    [sym_devchan_to_keyword] = ACTIONS(1855),
    [sym_devchan_min_to_keyword] = ACTIONS(1855),
    [sym_devchan_total_off_keyword] = ACTIONS(1855),
    [sym_devchan_pulse_keyword] = ACTIONS(1855),
    [sym_wait_keyword] = ACTIONS(1855),
    [sym_cancel_wait_keyword] = ACTIONS(1855),
    [sym_wait_until_keyword] = ACTIONS(1855),
    [sym_cancel_wait_until_keyword] = ACTIONS(1855),
    [sym_cancel_all_wait_keyword] = ACTIONS(1855),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1855),
    [sym_not] = ACTIONS(1855),
    [sym_bnot] = ACTIONS(1855),
    [sym___file__] = ACTIONS(1855),
    [sym___date__] = ACTIONS(1855),
    [sym___ldate__] = ACTIONS(1855),
    [sym___time__] = ACTIONS(1855),
    [sym___line__] = ACTIONS(1855),
    [sym___name__] = ACTIONS(1855),
    [sym___netlinx__] = ACTIONS(1855),
    [sym_netlinx_axi_version] = ACTIONS(1855),
    [sym_url_flg_tcp] = ACTIONS(1855),
    [sym_url_flg_accinfopresent] = ACTIONS(1855),
    [sym_url_flg_temp] = ACTIONS(1855),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1855),
    [sym_url_flg_stat_mask] = ACTIONS(1855),
    [sym_url_flg_stat_lookup] = ACTIONS(1855),
    [sym_url_flg_stat_connecting] = ACTIONS(1855),
    [sym_url_flg_stat_waiting] = ACTIONS(1855),
    [sym_url_flg_stat_connected] = ACTIONS(1855),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1855),
    [sym_first_virtual_device] = ACTIONS(1855),
    [sym_first_local_port] = ACTIONS(1855),
    [sym_dynamic_virtual_device] = ACTIONS(1855),
    [sym_timeline_once] = ACTIONS(1855),
    [sym_timeline_repeat] = ACTIONS(1855),
    [sym_timeline_absolute] = ACTIONS(1855),
    [sym_timeline_relative] = ACTIONS(1855),
    [sym_do_push_timed_infinite] = ACTIONS(1855),
    [sym_source_type_no_address] = ACTIONS(1855),
    [sym_source_type_neuron_id] = ACTIONS(1855),
    [sym_source_type_ip_address] = ACTIONS(1855),
    [sym_source_type_axlink] = ACTIONS(1855),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1855),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1855),
    [sym_source_type_ip_socket_address] = ACTIONS(1855),
    [sym_source_type_rs232] = ACTIONS(1855),
    [sym_source_type_internal] = ACTIONS(1855),
    [sym_source_type_mac_address] = ACTIONS(1855),
    [sym_source_type_ipv4_port] = ACTIONS(1855),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1855),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1855),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1855),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1855),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1855),
    [sym_true] = ACTIONS(1855),
    [sym_false] = ACTIONS(1855),
    [sym_file_read_only] = ACTIONS(1855),
    [sym_file_rw_new] = ACTIONS(1855),
    [sym_file_rw_append] = ACTIONS(1855),
    [sym_ip_tcp] = ACTIONS(1855),
    [sym_ip_udp] = ACTIONS(1855),
    [sym_ip_udp_2way] = ACTIONS(1855),
    [sym_xml_encode_types] = ACTIONS(1855),
    [sym_xml_encode_char_as_list] = ACTIONS(1855),
    [sym_xml_encode_le] = ACTIONS(1855),
    [sym_xml_decode_types] = ACTIONS(1855),
    [sym_xml_decode_no_preserve] = ACTIONS(1855),
    [sym_duet_dev_type_display_device] = ACTIONS(1855),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1855),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1855),
    [sym_duet_dev_type_io_device] = ACTIONS(1855),
    [sym_duet_dev_type_relay_device] = ACTIONS(1855),
    [sym_duet_dev_type_ups] = ACTIONS(1855),
    [sym_duet_dev_type_amplifier] = ACTIONS(1855),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1855),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1855),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1855),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1855),
    [sym_duet_dev_type_camera] = ACTIONS(1855),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1855),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1855),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1855),
    [sym_duet_dev_type_dss] = ACTIONS(1855),
    [sym_duet_dev_type_dvr] = ACTIONS(1855),
    [sym_duet_dev_type_disc_device] = ACTIONS(1855),
    [sym_duet_dev_type_document_camera] = ACTIONS(1855),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1855),
    [sym_duet_dev_type_hvac] = ACTIONS(1855),
    [sym_duet_dev_type_keypad] = ACTIONS(1855),
    [sym_duet_dev_type_light] = ACTIONS(1855),
    [sym_duet_dev_type_monitor] = ACTIONS(1855),
    [sym_duet_dev_type_motor] = ACTIONS(1855),
    [sym_duet_dev_type_multi_window] = ACTIONS(1855),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1855),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1855),
    [sym_duet_dev_type_receiver] = ACTIONS(1855),
    [sym_duet_dev_type_security_system] = ACTIONS(1855),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1855),
    [sym_duet_dev_type_settop_box] = ACTIONS(1855),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1855),
    [sym_duet_dev_type_switcher] = ACTIONS(1855),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1855),
    [sym_duet_dev_type_tv] = ACTIONS(1855),
    [sym_duet_dev_type_utility] = ACTIONS(1855),
    [sym_duet_dev_type_vcr] = ACTIONS(1855),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1855),
    [sym_duet_dev_type_video_processor] = ACTIONS(1855),
    [sym_duet_dev_type_video_projector] = ACTIONS(1855),
    [sym_duet_dev_type_video_wall] = ACTIONS(1855),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1855),
    [sym_duet_dev_type_weather] = ACTIONS(1855),
    [sym_duet_dev_not_polled] = ACTIONS(1855),
    [sym_duet_dev_polled] = ACTIONS(1855),
    [sym_clkmgr_mode_network] = ACTIONS(1855),
    [sym_clkmgr_mode_standalone] = ACTIONS(1855),
    [sym_normal_standby] = ACTIONS(1855),
    [sym_normal_wake] = ACTIONS(1855),
    [sym_ip_multicast_ttl_option] = ACTIONS(1855),
    [sym_ip_ttl_subnet] = ACTIONS(1855),
    [sym_ip_ttl_site] = ACTIONS(1855),
    [sym_ip_ttl_region] = ACTIONS(1855),
    [sym_ip_ttl_continent] = ACTIONS(1855),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1855),
    [sym_ip_nodelay_on] = ACTIONS(1855),
    [sym_ip_nodelay_off] = ACTIONS(1855),
    [sym_amx_error] = ACTIONS(1855),
    [sym_amx_warning] = ACTIONS(1855),
    [sym_amx_info] = ACTIONS(1855),
    [sym_amx_debug] = ACTIONS(1855),
    [sym_netlinx_login_success] = ACTIONS(1855),
    [sym_netlinx_login_fail] = ACTIONS(1855),
    [sym_netlinx_logout] = ACTIONS(1855),
    [sym_null_str] = ACTIONS(1855),
    [sym_smtp_address] = ACTIONS(1855),
    [sym_smtp_port_number] = ACTIONS(1855),
    [sym_smtp_username] = ACTIONS(1855),
    [sym_smtp_password] = ACTIONS(1855),
    [sym_smtp_require_tls] = ACTIONS(1855),
    [sym_smtp_from] = ACTIONS(1855),
    [sym_smtp_tls_true] = ACTIONS(1855),
    [sym_smtp_tls_false] = ACTIONS(1855),
    [sym_listview_on_row_select_event] = ACTIONS(1855),
    [sym_data_structure_unknown] = ACTIONS(1855),
    [sym_data_structure_datafeed] = ACTIONS(1855),
    [sym_data_structure_datarecordset] = ACTIONS(1855),
    [sym_data_structure_datarecord] = ACTIONS(1855),
    [sym_data_structure_datafield] = ACTIONS(1855),
    [sym_data_max_content_field_count] = ACTIONS(1855),
    [sym_data_max_metadata_field_count] = ACTIONS(1855),
    [sym_data_max_id_length] = ACTIONS(1855),
    [sym_data_max_name_length] = ACTIONS(1855),
    [sym_data_max_description_length] = ACTIONS(1855),
    [sym_data_max_type_length] = ACTIONS(1855),
    [sym_data_max_format_length] = ACTIONS(1855),
    [sym_data_max_source_length] = ACTIONS(1855),
    [sym_data_max_value_length] = ACTIONS(1855),
    [sym_data_max_label_length] = ACTIONS(1855),
    [sym_data_max_url_length] = ACTIONS(1855),
    [sym_max_last_login_info_length] = ACTIONS(1855),
    [sym_data_type_unknown] = ACTIONS(1855),
    [sym_data_type_string] = ACTIONS(1855),
    [sym_data_type_datetime] = ACTIONS(1855),
    [sym_data_type_date] = ACTIONS(1855),
    [sym_data_type_time] = ACTIONS(1855),
    [sym_data_type_image] = ACTIONS(1855),
    [sym_data_format_url] = ACTIONS(1855),
    [sym_data_format_phone] = ACTIONS(1855),
    [sym_data_format_email] = ACTIONS(1855),
    [sym_data_format_iso8601] = ACTIONS(1855),
    [sym_auditlog_priv] = ACTIONS(1855),
    [sym_configuration_priv] = ACTIONS(1855),
    [sym_deviceconfig_priv] = ACTIONS(1855),
    [sym_ftp_priv] = ACTIONS(1855),
    [sym_http_priv] = ACTIONS(1855),
    [sym_networkconfig_priv] = ACTIONS(1855),
    [sym_programport_priv] = ACTIONS(1855),
    [sym_remoteui_priv] = ACTIONS(1855),
    [sym_securitycontrol_priv] = ACTIONS(1855),
    [sym_softwaremanagement_priv] = ACTIONS(1855),
    [sym_terminal_priv] = ACTIONS(1855),
    [sym_tpadmin_priv] = ACTIONS(1855),
    [sym_user1_priv] = ACTIONS(1855),
    [sym_user2_priv] = ACTIONS(1855),
    [sym_user3_priv] = ACTIONS(1855),
    [sym_user4_priv] = ACTIONS(1855),
    [sym_usermanagement_priv] = ACTIONS(1855),
    [sym_valid_account] = ACTIONS(1855),
    [sym_err_name_invalid] = ACTIONS(1855),
    [sym_err_password_invalid] = ACTIONS(1855),
    [sym_err_invalid_account] = ACTIONS(1855),
    [sym_err_no_authorization] = ACTIONS(1855),
    [sym_err_invalid_parameter] = ACTIONS(1855),
    [sym_err_user_locked_out] = ACTIONS(1855),
    [sym_err_user_account_expired] = ACTIONS(1855),
    [sym_err_general_error] = ACTIONS(1855),
    [sym_err_authentication_error] = ACTIONS(1855),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1855),
    [sym_tls_validate_certificate] = ACTIONS(1855),
    [sym_tdata] = ACTIONS(1855),
    [sym_tchannel] = ACTIONS(1855),
    [sym_tlevel] = ACTIONS(1855),
    [sym_tbutton] = ACTIONS(1855),
    [sym_ttimeline] = ACTIONS(1855),
    [sym_tcustom] = ACTIONS(1855),
    [sym_url_struct] = ACTIONS(1855),
    [sym_dns_struct] = ACTIONS(1855),
    [sym_ip_address_struct] = ACTIONS(1855),
    [sym_dev_info_struct] = ACTIONS(1855),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1855),
    [sym_data_feed] = ACTIONS(1855),
    [sym_data_field] = ACTIONS(1855),
    [sym_data_record] = ACTIONS(1855),
    [sym_wc_data_feed] = ACTIONS(1855),
    [sym_wc_data_field] = ACTIONS(1855),
    [sym_wc_data_record] = ACTIONS(1855),
    [sym_last_login_info] = ACTIONS(1855),
    [sym____reserved___] = ACTIONS(1855),
    [sym_date] = ACTIONS(1855),
    [sym_day] = ACTIONS(1855),
    [sym_time] = ACTIONS(1855),
    [sym_ldate] = ACTIONS(1855),
    [sym_push_device] = ACTIONS(1855),
    [sym_push_channel] = ACTIONS(1855),
    [sym_push_devchan] = ACTIONS(1855),
    [sym_release_device] = ACTIONS(1855),
    [sym_release_channel] = ACTIONS(1855),
    [sym_release_devchan] = ACTIONS(1855),
    [sym_master_slot] = ACTIONS(1855),
    [sym_get_pulse_time] = ACTIONS(1855),
    [sym_get_timer] = ACTIONS(1855),
    [sym_master_sn] = ACTIONS(1855),
    [sym_system_number] = ACTIONS(1855),
    [sym_button] = ACTIONS(1855),
    [sym_dv_channel] = ACTIONS(1855),
    [sym_data] = ACTIONS(1855),
    [sym_level] = ACTIONS(1855),
    [sym_timeline] = ACTIONS(1855),
    [sym_channel] = ACTIONS(1855),
    [sym_custom] = ACTIONS(1855),
    [sym_atoi] = ACTIONS(1855),
    [sym_atol] = ACTIONS(1855),
    [sym_atof] = ACTIONS(1855),
    [sym_itoa] = ACTIONS(1855),
    [sym_itohex] = ACTIONS(1855),
    [sym_hextoi] = ACTIONS(1855),
    [sym_ftoa] = ACTIONS(1855),
    [sym_find_string] = ACTIONS(1855),
    [sym_left_string] = ACTIONS(1855),
    [sym_right_string] = ACTIONS(1855),
    [sym_mid_string] = ACTIONS(1855),
    [sym_remove_string] = ACTIONS(1855),
    [sym_get_buffer_string] = ACTIONS(1855),
    [sym_get_buffer_char] = ACTIONS(1855),
    [sym_get_multi_buffer_string] = ACTIONS(1855),
    [sym_length_string] = ACTIONS(1855),
    [sym_max_length_string] = ACTIONS(1855),
    [sym_set_length_string] = ACTIONS(1855),
    [sym_compare_string] = ACTIONS(1855),
    [sym_upper_string] = ACTIONS(1855),
    [sym_lower_string] = ACTIONS(1855),
    [sym_get_last] = ACTIONS(1855),
    [sym_date_to_day] = ACTIONS(1855),
    [sym_date_to_month] = ACTIONS(1855),
    [sym_date_to_year] = ACTIONS(1855),
    [sym_time_to_hour] = ACTIONS(1855),
    [sym_time_to_minute] = ACTIONS(1855),
    [sym_time_to_second] = ACTIONS(1855),
    [sym_day_of_week] = ACTIONS(1855),
    [sym_set_pulse_time] = ACTIONS(1855),
    [sym_set_timer] = ACTIONS(1855),
    [sym_file_open] = ACTIONS(1855),
    [sym_file_close] = ACTIONS(1855),
    [sym_file_read] = ACTIONS(1855),
    [sym_file_read_line] = ACTIONS(1855),
    [sym_file_write] = ACTIONS(1855),
    [sym_file_write_line] = ACTIONS(1855),
    [sym_file_copy] = ACTIONS(1855),
    [sym_file_delete] = ACTIONS(1855),
    [sym_file_rename] = ACTIONS(1855),
    [sym_file_dir] = ACTIONS(1855),
    [sym_file_getdir] = ACTIONS(1855),
    [sym_file_setdir] = ACTIONS(1855),
    [sym_file_create_dir] = ACTIONS(1855),
    [sym_file_remove_dir] = ACTIONS(1855),
    [sym_file_seek] = ACTIONS(1855),
    [sym_length_array] = ACTIONS(1855),
    [sym_max_length_array] = ACTIONS(1855),
    [sym_set_length_array] = ACTIONS(1855),
    [sym_ip_client_open] = ACTIONS(1855),
    [sym_ip_client_close] = ACTIONS(1855),
    [sym_ip_server_open] = ACTIONS(1855),
    [sym_ip_server_close] = ACTIONS(1855),
    [sym_ip_bound_client_open] = ACTIONS(1855),
    [sym_ip_mc_server_open] = ACTIONS(1855),
    [sym_ip_set_option] = ACTIONS(1855),
    [sym_ssh_client_open] = ACTIONS(1855),
    [sym_ssh_client_close] = ACTIONS(1855),
    [sym_tls_client_open] = ACTIONS(1855),
    [sym_tls_client_close] = ACTIONS(1855),
    [sym_do_push] = ACTIONS(1855),
    [sym_do_push_timed] = ACTIONS(1855),
    [sym_do_release] = ACTIONS(1855),
    [sym_device_id] = ACTIONS(1855),
    [sym_device_info] = ACTIONS(1855),
    [sym_device_id_string] = ACTIONS(1855),
    [sym_device_standby] = ACTIONS(1855),
    [sym_device_wake] = ACTIONS(1855),
    [sym_timeline_create] = ACTIONS(1855),
    [sym_timeline_kill] = ACTIONS(1855),
    [sym_timeline_reload] = ACTIONS(1855),
    [sym_timeline_pause] = ACTIONS(1855),
    [sym_timeline_restart] = ACTIONS(1855),
    [sym_timeline_set] = ACTIONS(1855),
    [sym_timeline_get] = ACTIONS(1855),
    [sym_timeline_active] = ACTIONS(1855),
    [sym_timeline_dynamic_id] = ACTIONS(1855),
    [sym_variable_to_string] = ACTIONS(1855),
    [sym_string_to_variable] = ACTIONS(1855),
    [sym_length_variable_to_string] = ACTIONS(1855),
    [sym_type_cast] = ACTIONS(1855),
    [sym_variable_to_xml] = ACTIONS(1855),
    [sym_xml_to_variable] = ACTIONS(1855),
    [sym_length_variable_to_xml] = ACTIONS(1855),
    [sym_redirect_string] = ACTIONS(1855),
    [sym_random_number] = ACTIONS(1855),
    [sym_get_unique_id] = ACTIONS(1855),
    [sym_get_system_number] = ACTIONS(1855),
    [sym_set_system_number] = ACTIONS(1855),
    [sym_get_serial_number] = ACTIONS(1855),
    [sym_get_master_build] = ACTIONS(1855),
    [sym_set_validation_code] = ACTIONS(1855),
    [sym_reboot] = ACTIONS(1855),
    [sym_load_duet_module] = ACTIONS(1855),
    [sym_unload_duet_module] = ACTIONS(1855),
    [sym_duet_mem_size_set] = ACTIONS(1855),
    [sym_duet_mem_size_get] = ACTIONS(1855),
    [sym_dynamic_application_device] = ACTIONS(1855),
    [sym_dynamic_polled_port] = ACTIONS(1855),
    [sym_static_port_binding] = ACTIONS(1855),
    [sym_static_ip_binding] = ACTIONS(1855),
    [sym_get_dns_list] = ACTIONS(1855),
    [sym_set_dns_list] = ACTIONS(1855),
    [sym_get_ip_address] = ACTIONS(1855),
    [sym_set_ip_address] = ACTIONS(1855),
    [sym_get_url_list] = ACTIONS(1855),
    [sym_add_url_entry] = ACTIONS(1855),
    [sym_delete_url_entry] = ACTIONS(1855),
    [sym_abs_value] = ACTIONS(1855),
    [sym_max_value] = ACTIONS(1855),
    [sym_min_value] = ACTIONS(1855),
    [sym_exp_value] = ACTIONS(1855),
    [sym_log_value] = ACTIONS(1855),
    [sym_log10_value] = ACTIONS(1855),
    [sym_power_value] = ACTIONS(1855),
    [sym_sqrt_value] = ACTIONS(1855),
    [sym_raw_be] = ACTIONS(1855),
    [sym_raw_le] = ACTIONS(1855),
    [sym_internal_queue_size_set] = ACTIONS(1855),
    [sym_internal_queue_size_get] = ACTIONS(1855),
    [sym_internal_threshold_set] = ACTIONS(1855),
    [sym_internal_threshold_get] = ACTIONS(1855),
    [sym_rebuilt_event] = ACTIONS(1855),
    [sym_do_custom_event] = ACTIONS(1855),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1855),
    [sym_clkmgr_set_clk_source] = ACTIONS(1855),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1855),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1855),
    [sym_clkmgr_get_timezone] = ACTIONS(1855),
    [sym_clkmgr_set_timezone] = ACTIONS(1855),
    [sym_clkmgr_get_resync_period] = ACTIONS(1855),
    [sym_clkmgr_set_resync_period] = ACTIONS(1855),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1855),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1855),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1855),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1855),
    [sym_clkmgr_get_timeservers] = ACTIONS(1855),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1855),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1855),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1855),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1855),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1855),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1855),
    [sym_set_log_level] = ACTIONS(1855),
    [sym_get_log_level] = ACTIONS(1855),
    [sym_amx_log] = ACTIONS(1855),
    [sym_smtp_server_config_set] = ACTIONS(1855),
    [sym_smtp_server_config_get] = ACTIONS(1855),
    [sym_smtp_send] = ACTIONS(1855),
    [sym_validate_netlinx_account] = ACTIONS(1855),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1855),
    [sym_authenticate_certificate] = ACTIONS(1855),
    [sym_audit_netlinx_session_event] = ACTIONS(1855),
    [sym_audit_netlinx_generic_event] = ACTIONS(1855),
    [sym_get_available_flash_disk_space] = ACTIONS(1855),
    [sym_get_max_flash_disk_space] = ACTIONS(1855),
    [sym_data_create_feed] = ACTIONS(1855),
    [sym_data_delete_feed] = ACTIONS(1855),
    [sym_data_publish_feed] = ACTIONS(1855),
    [sym_data_get_published_feed] = ACTIONS(1855),
    [sym_data_add_record] = ACTIONS(1855),
    [sym_data_get_event_record] = ACTIONS(1855),
    [sym__wc_data_create_feed] = ACTIONS(1855),
    [sym__wc_data_add_record] = ACTIONS(1855),
    [sym__wc_data_get_event_record] = ACTIONS(1855),
    [sym_format] = ACTIONS(1855),
    [sym_astro_clock] = ACTIONS(1855),
    [sym_set_outdoor_temperature] = ACTIONS(1855),
    [sym_set_virtual_level_count] = ACTIONS(1855),
    [sym_set_virtual_channel_count] = ACTIONS(1855),
    [sym_set_virtual_port_count] = ACTIONS(1855),
    [sym__wc_encode_int] = ACTIONS(1855),
    [sym__wc_decode_int] = ACTIONS(1855),
    [sym__wc_int] = ACTIONS(1855),
    [sym__wc_to_ch_int] = ACTIONS(1855),
    [sym__ch_to_wc_int] = ACTIONS(1855),
    [sym__wc_find_string_int] = ACTIONS(1855),
    [sym__wc_left_string_int] = ACTIONS(1855),
    [sym__wc_lower_string_int] = ACTIONS(1855),
    [sym__wc_mid_string_int] = ACTIONS(1855),
    [sym__wc_remove_string_int] = ACTIONS(1855),
    [sym__wc_right_string_int] = ACTIONS(1855),
    [sym__wc_upper_string_int] = ACTIONS(1855),
    [sym__wc_compare_string_int] = ACTIONS(1855),
    [sym__wc_get_buffer_char_int] = ACTIONS(1855),
    [sym__wc_get_buffer_string_int] = ACTIONS(1855),
    [sym__wc_concat_string_int] = ACTIONS(1855),
    [sym__wc_file_open_int] = ACTIONS(1855),
    [sym__wc_file_close_int] = ACTIONS(1855),
    [sym__wc_file_read_int] = ACTIONS(1855),
    [sym__wc_file_read_line_int] = ACTIONS(1855),
    [sym__wc_file_write_int] = ACTIONS(1855),
    [sym__wc_file_write_line_int] = ACTIONS(1855),
    [aux_sym_preproc_include_token1] = ACTIONS(1855),
    [aux_sym_preproc_define_token1] = ACTIONS(1857),
    [aux_sym_preproc_warn_token1] = ACTIONS(1857),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1857),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1857),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1857),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1857),
    [aux_sym_preproc_else_token1] = ACTIONS(1857),
    [anon_sym_LBRACK] = ACTIONS(1857),
    [anon_sym_LBRACE] = ACTIONS(1857),
    [anon_sym_LPAREN] = ACTIONS(1855),
    [anon_sym_BANG] = ACTIONS(1857),
    [anon_sym_TILDE] = ACTIONS(1857),
    [anon_sym_DASH] = ACTIONS(1855),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_DASH_DASH] = ACTIONS(1857),
    [anon_sym_PLUS_PLUS] = ACTIONS(1857),
    [anon_sym_DQUOTE] = ACTIONS(1857),
    [anon_sym_SQUOTE] = ACTIONS(1857),
    [sym_number_literal] = ACTIONS(1857),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(260)] = {
    [sym_identifier] = ACTIONS(1859),
    [sym_program_name_keyword] = ACTIONS(1859),
    [sym_module_name_keyword] = ACTIONS(1859),
    [sym_define_device_keyword] = ACTIONS(1859),
    [sym_define_combine_keyword] = ACTIONS(1859),
    [sym_define_constant_keyword] = ACTIONS(1859),
    [sym_define_type_keyword] = ACTIONS(1859),
    [sym_define_variable_keyword] = ACTIONS(1859),
    [sym_define_system_variable_keyword] = ACTIONS(1859),
    [sym_define_connect_level_keyword] = ACTIONS(1859),
    [sym_define_latching_keyword] = ACTIONS(1859),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1859),
    [sym_define_toggling_keyword] = ACTIONS(1859),
    [sym_define_call_keyword] = ACTIONS(1859),
    [sym_define_function_keyword] = ACTIONS(1859),
    [sym_define_library_function_keyword] = ACTIONS(1859),
    [sym_define_start_keyword] = ACTIONS(1859),
    [sym_define_event_keyword] = ACTIONS(1859),
    [sym_define_program_keyword] = ACTIONS(1859),
    [sym_define_module_keyword] = ACTIONS(1859),
    [sym_char_keyword] = ACTIONS(1859),
    [sym_widechar_keyword] = ACTIONS(1859),
    [sym_integer_keyword] = ACTIONS(1859),
    [sym_sinteger_keyword] = ACTIONS(1859),
    [sym_long_keyword] = ACTIONS(1859),
    [sym_slong_keyword] = ACTIONS(1859),
    [sym_float_keyword] = ACTIONS(1859),
    [sym_double_keyword] = ACTIONS(1859),
    [sym_variant_keyword] = ACTIONS(1859),
    [sym_variantarray_keyword] = ACTIONS(1859),
    [sym_local_var_keyword] = ACTIONS(1859),
    [sym_stack_var_keyword] = ACTIONS(1859),
    [sym_constant_keyword] = ACTIONS(1859),
    [sym_volatile_keyword] = ACTIONS(1859),
    [sym_non_volatile_keyword] = ACTIONS(1859),
    [sym_persistent_keyword] = ACTIONS(1859),
    [sym_dev_keyword] = ACTIONS(1859),
    [sym_devlev_keyword] = ACTIONS(1859),
    [sym_devchan_keyword] = ACTIONS(1859),
    [sym_if_keyword] = ACTIONS(1859),
    [sym_else_keyword] = ACTIONS(1859),
    [sym_switch_keyword] = ACTIONS(1859),
    [sym_case_keyword] = ACTIONS(1859),
    [sym_default_keyword] = ACTIONS(1859),
    [sym_while_keyword] = ACTIONS(1859),
    [sym_for_keyword] = ACTIONS(1859),
    [sym_break_keyword] = ACTIONS(1859),
    [sym_continue_keyword] = ACTIONS(1859),
    [sym_return_keyword] = ACTIONS(1859),
    [sym_select_keyword] = ACTIONS(1859),
    [sym_active_keyword] = ACTIONS(1859),
    [sym_struct_keyword] = ACTIONS(1859),
    [sym_structure_keyword] = ACTIONS(1859),
    [sym_button_event_keyword] = ACTIONS(1859),
    [sym_channel_event_keyword] = ACTIONS(1859),
    [sym_level_event_keyword] = ACTIONS(1859),
    [sym_data_event_keyword] = ACTIONS(1859),
    [sym_timeline_event_keyword] = ACTIONS(1859),
    [sym_custom_event_keyword] = ACTIONS(1859),
    [sym_push_keyword] = ACTIONS(1859),
    [sym_release_keyword] = ACTIONS(1859),
    [aux_sym_on_keyword_token1] = ACTIONS(1859),
    [aux_sym_off_keyword_token1] = ACTIONS(1859),
    [sym_send_level_keyword] = ACTIONS(1859),
    [sym_send_string_keyword] = ACTIONS(1859),
    [sym_send_command_keyword] = ACTIONS(1859),
    [sym_clear_buffer_keyword] = ACTIONS(1859),
    [sym_create_buffer_keyword] = ACTIONS(1859),
    [sym_create_multi_buffer_keyword] = ACTIONS(1859),
    [sym_call_keyword] = ACTIONS(1859),
    [sym_system_call_keyword] = ACTIONS(1859),
    [sym_devchan_to_keyword] = ACTIONS(1859),
    [sym_devchan_min_to_keyword] = ACTIONS(1859),
    [sym_devchan_total_off_keyword] = ACTIONS(1859),
    [sym_devchan_pulse_keyword] = ACTIONS(1859),
    [sym_wait_keyword] = ACTIONS(1859),
    [sym_cancel_wait_keyword] = ACTIONS(1859),
    [sym_wait_until_keyword] = ACTIONS(1859),
    [sym_cancel_wait_until_keyword] = ACTIONS(1859),
    [sym_cancel_all_wait_keyword] = ACTIONS(1859),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1859),
    [sym_not] = ACTIONS(1859),
    [sym_bnot] = ACTIONS(1859),
    [sym___file__] = ACTIONS(1859),
    [sym___date__] = ACTIONS(1859),
    [sym___ldate__] = ACTIONS(1859),
    [sym___time__] = ACTIONS(1859),
    [sym___line__] = ACTIONS(1859),
    [sym___name__] = ACTIONS(1859),
    [sym___netlinx__] = ACTIONS(1859),
    [sym_netlinx_axi_version] = ACTIONS(1859),
    [sym_url_flg_tcp] = ACTIONS(1859),
    [sym_url_flg_accinfopresent] = ACTIONS(1859),
    [sym_url_flg_temp] = ACTIONS(1859),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1859),
    [sym_url_flg_stat_mask] = ACTIONS(1859),
    [sym_url_flg_stat_lookup] = ACTIONS(1859),
    [sym_url_flg_stat_connecting] = ACTIONS(1859),
    [sym_url_flg_stat_waiting] = ACTIONS(1859),
    [sym_url_flg_stat_connected] = ACTIONS(1859),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1859),
    [sym_first_virtual_device] = ACTIONS(1859),
    [sym_first_local_port] = ACTIONS(1859),
    [sym_dynamic_virtual_device] = ACTIONS(1859),
    [sym_timeline_once] = ACTIONS(1859),
    [sym_timeline_repeat] = ACTIONS(1859),
    [sym_timeline_absolute] = ACTIONS(1859),
    [sym_timeline_relative] = ACTIONS(1859),
    [sym_do_push_timed_infinite] = ACTIONS(1859),
    [sym_source_type_no_address] = ACTIONS(1859),
    [sym_source_type_neuron_id] = ACTIONS(1859),
    [sym_source_type_ip_address] = ACTIONS(1859),
    [sym_source_type_axlink] = ACTIONS(1859),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1859),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1859),
    [sym_source_type_ip_socket_address] = ACTIONS(1859),
    [sym_source_type_rs232] = ACTIONS(1859),
    [sym_source_type_internal] = ACTIONS(1859),
    [sym_source_type_mac_address] = ACTIONS(1859),
    [sym_source_type_ipv4_port] = ACTIONS(1859),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1859),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1859),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1859),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1859),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1859),
    [sym_true] = ACTIONS(1859),
    [sym_false] = ACTIONS(1859),
    [sym_file_read_only] = ACTIONS(1859),
    [sym_file_rw_new] = ACTIONS(1859),
    [sym_file_rw_append] = ACTIONS(1859),
    [sym_ip_tcp] = ACTIONS(1859),
    [sym_ip_udp] = ACTIONS(1859),
    [sym_ip_udp_2way] = ACTIONS(1859),
    [sym_xml_encode_types] = ACTIONS(1859),
    [sym_xml_encode_char_as_list] = ACTIONS(1859),
    [sym_xml_encode_le] = ACTIONS(1859),
    [sym_xml_decode_types] = ACTIONS(1859),
    [sym_xml_decode_no_preserve] = ACTIONS(1859),
    [sym_duet_dev_type_display_device] = ACTIONS(1859),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1859),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1859),
    [sym_duet_dev_type_io_device] = ACTIONS(1859),
    [sym_duet_dev_type_relay_device] = ACTIONS(1859),
    [sym_duet_dev_type_ups] = ACTIONS(1859),
    [sym_duet_dev_type_amplifier] = ACTIONS(1859),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1859),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1859),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1859),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1859),
    [sym_duet_dev_type_camera] = ACTIONS(1859),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1859),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1859),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1859),
    [sym_duet_dev_type_dss] = ACTIONS(1859),
    [sym_duet_dev_type_dvr] = ACTIONS(1859),
    [sym_duet_dev_type_disc_device] = ACTIONS(1859),
    [sym_duet_dev_type_document_camera] = ACTIONS(1859),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1859),
    [sym_duet_dev_type_hvac] = ACTIONS(1859),
    [sym_duet_dev_type_keypad] = ACTIONS(1859),
    [sym_duet_dev_type_light] = ACTIONS(1859),
    [sym_duet_dev_type_monitor] = ACTIONS(1859),
    [sym_duet_dev_type_motor] = ACTIONS(1859),
    [sym_duet_dev_type_multi_window] = ACTIONS(1859),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1859),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1859),
    [sym_duet_dev_type_receiver] = ACTIONS(1859),
    [sym_duet_dev_type_security_system] = ACTIONS(1859),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1859),
    [sym_duet_dev_type_settop_box] = ACTIONS(1859),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1859),
    [sym_duet_dev_type_switcher] = ACTIONS(1859),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1859),
    [sym_duet_dev_type_tv] = ACTIONS(1859),
    [sym_duet_dev_type_utility] = ACTIONS(1859),
    [sym_duet_dev_type_vcr] = ACTIONS(1859),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1859),
    [sym_duet_dev_type_video_processor] = ACTIONS(1859),
    [sym_duet_dev_type_video_projector] = ACTIONS(1859),
    [sym_duet_dev_type_video_wall] = ACTIONS(1859),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1859),
    [sym_duet_dev_type_weather] = ACTIONS(1859),
    [sym_duet_dev_not_polled] = ACTIONS(1859),
    [sym_duet_dev_polled] = ACTIONS(1859),
    [sym_clkmgr_mode_network] = ACTIONS(1859),
    [sym_clkmgr_mode_standalone] = ACTIONS(1859),
    [sym_normal_standby] = ACTIONS(1859),
    [sym_normal_wake] = ACTIONS(1859),
    [sym_ip_multicast_ttl_option] = ACTIONS(1859),
    [sym_ip_ttl_subnet] = ACTIONS(1859),
    [sym_ip_ttl_site] = ACTIONS(1859),
    [sym_ip_ttl_region] = ACTIONS(1859),
    [sym_ip_ttl_continent] = ACTIONS(1859),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1859),
    [sym_ip_nodelay_on] = ACTIONS(1859),
    [sym_ip_nodelay_off] = ACTIONS(1859),
    [sym_amx_error] = ACTIONS(1859),
    [sym_amx_warning] = ACTIONS(1859),
    [sym_amx_info] = ACTIONS(1859),
    [sym_amx_debug] = ACTIONS(1859),
    [sym_netlinx_login_success] = ACTIONS(1859),
    [sym_netlinx_login_fail] = ACTIONS(1859),
    [sym_netlinx_logout] = ACTIONS(1859),
    [sym_null_str] = ACTIONS(1859),
    [sym_smtp_address] = ACTIONS(1859),
    [sym_smtp_port_number] = ACTIONS(1859),
    [sym_smtp_username] = ACTIONS(1859),
    [sym_smtp_password] = ACTIONS(1859),
    [sym_smtp_require_tls] = ACTIONS(1859),
    [sym_smtp_from] = ACTIONS(1859),
    [sym_smtp_tls_true] = ACTIONS(1859),
    [sym_smtp_tls_false] = ACTIONS(1859),
    [sym_listview_on_row_select_event] = ACTIONS(1859),
    [sym_data_structure_unknown] = ACTIONS(1859),
    [sym_data_structure_datafeed] = ACTIONS(1859),
    [sym_data_structure_datarecordset] = ACTIONS(1859),
    [sym_data_structure_datarecord] = ACTIONS(1859),
    [sym_data_structure_datafield] = ACTIONS(1859),
    [sym_data_max_content_field_count] = ACTIONS(1859),
    [sym_data_max_metadata_field_count] = ACTIONS(1859),
    [sym_data_max_id_length] = ACTIONS(1859),
    [sym_data_max_name_length] = ACTIONS(1859),
    [sym_data_max_description_length] = ACTIONS(1859),
    [sym_data_max_type_length] = ACTIONS(1859),
    [sym_data_max_format_length] = ACTIONS(1859),
    [sym_data_max_source_length] = ACTIONS(1859),
    [sym_data_max_value_length] = ACTIONS(1859),
    [sym_data_max_label_length] = ACTIONS(1859),
    [sym_data_max_url_length] = ACTIONS(1859),
    [sym_max_last_login_info_length] = ACTIONS(1859),
    [sym_data_type_unknown] = ACTIONS(1859),
    [sym_data_type_string] = ACTIONS(1859),
    [sym_data_type_datetime] = ACTIONS(1859),
    [sym_data_type_date] = ACTIONS(1859),
    [sym_data_type_time] = ACTIONS(1859),
    [sym_data_type_image] = ACTIONS(1859),
    [sym_data_format_url] = ACTIONS(1859),
    [sym_data_format_phone] = ACTIONS(1859),
    [sym_data_format_email] = ACTIONS(1859),
    [sym_data_format_iso8601] = ACTIONS(1859),
    [sym_auditlog_priv] = ACTIONS(1859),
    [sym_configuration_priv] = ACTIONS(1859),
    [sym_deviceconfig_priv] = ACTIONS(1859),
    [sym_ftp_priv] = ACTIONS(1859),
    [sym_http_priv] = ACTIONS(1859),
    [sym_networkconfig_priv] = ACTIONS(1859),
    [sym_programport_priv] = ACTIONS(1859),
    [sym_remoteui_priv] = ACTIONS(1859),
    [sym_securitycontrol_priv] = ACTIONS(1859),
    [sym_softwaremanagement_priv] = ACTIONS(1859),
    [sym_terminal_priv] = ACTIONS(1859),
    [sym_tpadmin_priv] = ACTIONS(1859),
    [sym_user1_priv] = ACTIONS(1859),
    [sym_user2_priv] = ACTIONS(1859),
    [sym_user3_priv] = ACTIONS(1859),
    [sym_user4_priv] = ACTIONS(1859),
    [sym_usermanagement_priv] = ACTIONS(1859),
    [sym_valid_account] = ACTIONS(1859),
    [sym_err_name_invalid] = ACTIONS(1859),
    [sym_err_password_invalid] = ACTIONS(1859),
    [sym_err_invalid_account] = ACTIONS(1859),
    [sym_err_no_authorization] = ACTIONS(1859),
    [sym_err_invalid_parameter] = ACTIONS(1859),
    [sym_err_user_locked_out] = ACTIONS(1859),
    [sym_err_user_account_expired] = ACTIONS(1859),
    [sym_err_general_error] = ACTIONS(1859),
    [sym_err_authentication_error] = ACTIONS(1859),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1859),
    [sym_tls_validate_certificate] = ACTIONS(1859),
    [sym_tdata] = ACTIONS(1859),
    [sym_tchannel] = ACTIONS(1859),
    [sym_tlevel] = ACTIONS(1859),
    [sym_tbutton] = ACTIONS(1859),
    [sym_ttimeline] = ACTIONS(1859),
    [sym_tcustom] = ACTIONS(1859),
    [sym_url_struct] = ACTIONS(1859),
    [sym_dns_struct] = ACTIONS(1859),
    [sym_ip_address_struct] = ACTIONS(1859),
    [sym_dev_info_struct] = ACTIONS(1859),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1859),
    [sym_data_feed] = ACTIONS(1859),
    [sym_data_field] = ACTIONS(1859),
    [sym_data_record] = ACTIONS(1859),
    [sym_wc_data_feed] = ACTIONS(1859),
    [sym_wc_data_field] = ACTIONS(1859),
    [sym_wc_data_record] = ACTIONS(1859),
    [sym_last_login_info] = ACTIONS(1859),
    [sym____reserved___] = ACTIONS(1859),
    [sym_date] = ACTIONS(1859),
    [sym_day] = ACTIONS(1859),
    [sym_time] = ACTIONS(1859),
    [sym_ldate] = ACTIONS(1859),
    [sym_push_device] = ACTIONS(1859),
    [sym_push_channel] = ACTIONS(1859),
    [sym_push_devchan] = ACTIONS(1859),
    [sym_release_device] = ACTIONS(1859),
    [sym_release_channel] = ACTIONS(1859),
    [sym_release_devchan] = ACTIONS(1859),
    [sym_master_slot] = ACTIONS(1859),
    [sym_get_pulse_time] = ACTIONS(1859),
    [sym_get_timer] = ACTIONS(1859),
    [sym_master_sn] = ACTIONS(1859),
    [sym_system_number] = ACTIONS(1859),
    [sym_button] = ACTIONS(1859),
    [sym_dv_channel] = ACTIONS(1859),
    [sym_data] = ACTIONS(1859),
    [sym_level] = ACTIONS(1859),
    [sym_timeline] = ACTIONS(1859),
    [sym_channel] = ACTIONS(1859),
    [sym_custom] = ACTIONS(1859),
    [sym_atoi] = ACTIONS(1859),
    [sym_atol] = ACTIONS(1859),
    [sym_atof] = ACTIONS(1859),
    [sym_itoa] = ACTIONS(1859),
    [sym_itohex] = ACTIONS(1859),
    [sym_hextoi] = ACTIONS(1859),
    [sym_ftoa] = ACTIONS(1859),
    [sym_find_string] = ACTIONS(1859),
    [sym_left_string] = ACTIONS(1859),
    [sym_right_string] = ACTIONS(1859),
    [sym_mid_string] = ACTIONS(1859),
    [sym_remove_string] = ACTIONS(1859),
    [sym_get_buffer_string] = ACTIONS(1859),
    [sym_get_buffer_char] = ACTIONS(1859),
    [sym_get_multi_buffer_string] = ACTIONS(1859),
    [sym_length_string] = ACTIONS(1859),
    [sym_max_length_string] = ACTIONS(1859),
    [sym_set_length_string] = ACTIONS(1859),
    [sym_compare_string] = ACTIONS(1859),
    [sym_upper_string] = ACTIONS(1859),
    [sym_lower_string] = ACTIONS(1859),
    [sym_get_last] = ACTIONS(1859),
    [sym_date_to_day] = ACTIONS(1859),
    [sym_date_to_month] = ACTIONS(1859),
    [sym_date_to_year] = ACTIONS(1859),
    [sym_time_to_hour] = ACTIONS(1859),
    [sym_time_to_minute] = ACTIONS(1859),
    [sym_time_to_second] = ACTIONS(1859),
    [sym_day_of_week] = ACTIONS(1859),
    [sym_set_pulse_time] = ACTIONS(1859),
    [sym_set_timer] = ACTIONS(1859),
    [sym_file_open] = ACTIONS(1859),
    [sym_file_close] = ACTIONS(1859),
    [sym_file_read] = ACTIONS(1859),
    [sym_file_read_line] = ACTIONS(1859),
    [sym_file_write] = ACTIONS(1859),
    [sym_file_write_line] = ACTIONS(1859),
    [sym_file_copy] = ACTIONS(1859),
    [sym_file_delete] = ACTIONS(1859),
    [sym_file_rename] = ACTIONS(1859),
    [sym_file_dir] = ACTIONS(1859),
    [sym_file_getdir] = ACTIONS(1859),
    [sym_file_setdir] = ACTIONS(1859),
    [sym_file_create_dir] = ACTIONS(1859),
    [sym_file_remove_dir] = ACTIONS(1859),
    [sym_file_seek] = ACTIONS(1859),
    [sym_length_array] = ACTIONS(1859),
    [sym_max_length_array] = ACTIONS(1859),
    [sym_set_length_array] = ACTIONS(1859),
    [sym_ip_client_open] = ACTIONS(1859),
    [sym_ip_client_close] = ACTIONS(1859),
    [sym_ip_server_open] = ACTIONS(1859),
    [sym_ip_server_close] = ACTIONS(1859),
    [sym_ip_bound_client_open] = ACTIONS(1859),
    [sym_ip_mc_server_open] = ACTIONS(1859),
    [sym_ip_set_option] = ACTIONS(1859),
    [sym_ssh_client_open] = ACTIONS(1859),
    [sym_ssh_client_close] = ACTIONS(1859),
    [sym_tls_client_open] = ACTIONS(1859),
    [sym_tls_client_close] = ACTIONS(1859),
    [sym_do_push] = ACTIONS(1859),
    [sym_do_push_timed] = ACTIONS(1859),
    [sym_do_release] = ACTIONS(1859),
    [sym_device_id] = ACTIONS(1859),
    [sym_device_info] = ACTIONS(1859),
    [sym_device_id_string] = ACTIONS(1859),
    [sym_device_standby] = ACTIONS(1859),
    [sym_device_wake] = ACTIONS(1859),
    [sym_timeline_create] = ACTIONS(1859),
    [sym_timeline_kill] = ACTIONS(1859),
    [sym_timeline_reload] = ACTIONS(1859),
    [sym_timeline_pause] = ACTIONS(1859),
    [sym_timeline_restart] = ACTIONS(1859),
    [sym_timeline_set] = ACTIONS(1859),
    [sym_timeline_get] = ACTIONS(1859),
    [sym_timeline_active] = ACTIONS(1859),
    [sym_timeline_dynamic_id] = ACTIONS(1859),
    [sym_variable_to_string] = ACTIONS(1859),
    [sym_string_to_variable] = ACTIONS(1859),
    [sym_length_variable_to_string] = ACTIONS(1859),
    [sym_type_cast] = ACTIONS(1859),
    [sym_variable_to_xml] = ACTIONS(1859),
    [sym_xml_to_variable] = ACTIONS(1859),
    [sym_length_variable_to_xml] = ACTIONS(1859),
    [sym_redirect_string] = ACTIONS(1859),
    [sym_random_number] = ACTIONS(1859),
    [sym_get_unique_id] = ACTIONS(1859),
    [sym_get_system_number] = ACTIONS(1859),
    [sym_set_system_number] = ACTIONS(1859),
    [sym_get_serial_number] = ACTIONS(1859),
    [sym_get_master_build] = ACTIONS(1859),
    [sym_set_validation_code] = ACTIONS(1859),
    [sym_reboot] = ACTIONS(1859),
    [sym_load_duet_module] = ACTIONS(1859),
    [sym_unload_duet_module] = ACTIONS(1859),
    [sym_duet_mem_size_set] = ACTIONS(1859),
    [sym_duet_mem_size_get] = ACTIONS(1859),
    [sym_dynamic_application_device] = ACTIONS(1859),
    [sym_dynamic_polled_port] = ACTIONS(1859),
    [sym_static_port_binding] = ACTIONS(1859),
    [sym_static_ip_binding] = ACTIONS(1859),
    [sym_get_dns_list] = ACTIONS(1859),
    [sym_set_dns_list] = ACTIONS(1859),
    [sym_get_ip_address] = ACTIONS(1859),
    [sym_set_ip_address] = ACTIONS(1859),
    [sym_get_url_list] = ACTIONS(1859),
    [sym_add_url_entry] = ACTIONS(1859),
    [sym_delete_url_entry] = ACTIONS(1859),
    [sym_abs_value] = ACTIONS(1859),
    [sym_max_value] = ACTIONS(1859),
    [sym_min_value] = ACTIONS(1859),
    [sym_exp_value] = ACTIONS(1859),
    [sym_log_value] = ACTIONS(1859),
    [sym_log10_value] = ACTIONS(1859),
    [sym_power_value] = ACTIONS(1859),
    [sym_sqrt_value] = ACTIONS(1859),
    [sym_raw_be] = ACTIONS(1859),
    [sym_raw_le] = ACTIONS(1859),
    [sym_internal_queue_size_set] = ACTIONS(1859),
    [sym_internal_queue_size_get] = ACTIONS(1859),
    [sym_internal_threshold_set] = ACTIONS(1859),
    [sym_internal_threshold_get] = ACTIONS(1859),
    [sym_rebuilt_event] = ACTIONS(1859),
    [sym_do_custom_event] = ACTIONS(1859),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1859),
    [sym_clkmgr_set_clk_source] = ACTIONS(1859),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1859),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1859),
    [sym_clkmgr_get_timezone] = ACTIONS(1859),
    [sym_clkmgr_set_timezone] = ACTIONS(1859),
    [sym_clkmgr_get_resync_period] = ACTIONS(1859),
    [sym_clkmgr_set_resync_period] = ACTIONS(1859),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1859),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1859),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1859),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1859),
    [sym_clkmgr_get_timeservers] = ACTIONS(1859),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1859),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1859),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1859),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1859),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1859),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1859),
    [sym_set_log_level] = ACTIONS(1859),
    [sym_get_log_level] = ACTIONS(1859),
    [sym_amx_log] = ACTIONS(1859),
    [sym_smtp_server_config_set] = ACTIONS(1859),
    [sym_smtp_server_config_get] = ACTIONS(1859),
    [sym_smtp_send] = ACTIONS(1859),
    [sym_validate_netlinx_account] = ACTIONS(1859),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1859),
    [sym_authenticate_certificate] = ACTIONS(1859),
    [sym_audit_netlinx_session_event] = ACTIONS(1859),
    [sym_audit_netlinx_generic_event] = ACTIONS(1859),
    [sym_get_available_flash_disk_space] = ACTIONS(1859),
    [sym_get_max_flash_disk_space] = ACTIONS(1859),
    [sym_data_create_feed] = ACTIONS(1859),
    [sym_data_delete_feed] = ACTIONS(1859),
    [sym_data_publish_feed] = ACTIONS(1859),
    [sym_data_get_published_feed] = ACTIONS(1859),
    [sym_data_add_record] = ACTIONS(1859),
    [sym_data_get_event_record] = ACTIONS(1859),
    [sym__wc_data_create_feed] = ACTIONS(1859),
    [sym__wc_data_add_record] = ACTIONS(1859),
    [sym__wc_data_get_event_record] = ACTIONS(1859),
    [sym_format] = ACTIONS(1859),
    [sym_astro_clock] = ACTIONS(1859),
    [sym_set_outdoor_temperature] = ACTIONS(1859),
    [sym_set_virtual_level_count] = ACTIONS(1859),
    [sym_set_virtual_channel_count] = ACTIONS(1859),
    [sym_set_virtual_port_count] = ACTIONS(1859),
    [sym__wc_encode_int] = ACTIONS(1859),
    [sym__wc_decode_int] = ACTIONS(1859),
    [sym__wc_int] = ACTIONS(1859),
    [sym__wc_to_ch_int] = ACTIONS(1859),
    [sym__ch_to_wc_int] = ACTIONS(1859),
    [sym__wc_find_string_int] = ACTIONS(1859),
    [sym__wc_left_string_int] = ACTIONS(1859),
    [sym__wc_lower_string_int] = ACTIONS(1859),
    [sym__wc_mid_string_int] = ACTIONS(1859),
    [sym__wc_remove_string_int] = ACTIONS(1859),
    [sym__wc_right_string_int] = ACTIONS(1859),
    [sym__wc_upper_string_int] = ACTIONS(1859),
    [sym__wc_compare_string_int] = ACTIONS(1859),
    [sym__wc_get_buffer_char_int] = ACTIONS(1859),
    [sym__wc_get_buffer_string_int] = ACTIONS(1859),
    [sym__wc_concat_string_int] = ACTIONS(1859),
    [sym__wc_file_open_int] = ACTIONS(1859),
    [sym__wc_file_close_int] = ACTIONS(1859),
    [sym__wc_file_read_int] = ACTIONS(1859),
    [sym__wc_file_read_line_int] = ACTIONS(1859),
    [sym__wc_file_write_int] = ACTIONS(1859),
    [sym__wc_file_write_line_int] = ACTIONS(1859),
    [aux_sym_preproc_include_token1] = ACTIONS(1859),
    [aux_sym_preproc_define_token1] = ACTIONS(1861),
    [aux_sym_preproc_warn_token1] = ACTIONS(1861),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1861),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1861),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1861),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1861),
    [aux_sym_preproc_else_token1] = ACTIONS(1861),
    [anon_sym_LBRACK] = ACTIONS(1861),
    [anon_sym_LBRACE] = ACTIONS(1861),
    [anon_sym_LPAREN] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1861),
    [anon_sym_TILDE] = ACTIONS(1861),
    [anon_sym_DASH] = ACTIONS(1859),
    [anon_sym_PLUS] = ACTIONS(1859),
    [anon_sym_DASH_DASH] = ACTIONS(1861),
    [anon_sym_PLUS_PLUS] = ACTIONS(1861),
    [anon_sym_DQUOTE] = ACTIONS(1861),
    [anon_sym_SQUOTE] = ACTIONS(1861),
    [sym_number_literal] = ACTIONS(1861),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(261)] = {
    [sym_identifier] = ACTIONS(1863),
    [sym_program_name_keyword] = ACTIONS(1863),
    [sym_module_name_keyword] = ACTIONS(1863),
    [sym_define_device_keyword] = ACTIONS(1863),
    [sym_define_combine_keyword] = ACTIONS(1863),
    [sym_define_constant_keyword] = ACTIONS(1863),
    [sym_define_type_keyword] = ACTIONS(1863),
    [sym_define_variable_keyword] = ACTIONS(1863),
    [sym_define_system_variable_keyword] = ACTIONS(1863),
    [sym_define_connect_level_keyword] = ACTIONS(1863),
    [sym_define_latching_keyword] = ACTIONS(1863),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1863),
    [sym_define_toggling_keyword] = ACTIONS(1863),
    [sym_define_call_keyword] = ACTIONS(1863),
    [sym_define_function_keyword] = ACTIONS(1863),
    [sym_define_library_function_keyword] = ACTIONS(1863),
    [sym_define_start_keyword] = ACTIONS(1863),
    [sym_define_event_keyword] = ACTIONS(1863),
    [sym_define_program_keyword] = ACTIONS(1863),
    [sym_define_module_keyword] = ACTIONS(1863),
    [sym_char_keyword] = ACTIONS(1863),
    [sym_widechar_keyword] = ACTIONS(1863),
    [sym_integer_keyword] = ACTIONS(1863),
    [sym_sinteger_keyword] = ACTIONS(1863),
    [sym_long_keyword] = ACTIONS(1863),
    [sym_slong_keyword] = ACTIONS(1863),
    [sym_float_keyword] = ACTIONS(1863),
    [sym_double_keyword] = ACTIONS(1863),
    [sym_variant_keyword] = ACTIONS(1863),
    [sym_variantarray_keyword] = ACTIONS(1863),
    [sym_local_var_keyword] = ACTIONS(1863),
    [sym_stack_var_keyword] = ACTIONS(1863),
    [sym_constant_keyword] = ACTIONS(1863),
    [sym_volatile_keyword] = ACTIONS(1863),
    [sym_non_volatile_keyword] = ACTIONS(1863),
    [sym_persistent_keyword] = ACTIONS(1863),
    [sym_dev_keyword] = ACTIONS(1863),
    [sym_devlev_keyword] = ACTIONS(1863),
    [sym_devchan_keyword] = ACTIONS(1863),
    [sym_if_keyword] = ACTIONS(1863),
    [sym_else_keyword] = ACTIONS(1863),
    [sym_switch_keyword] = ACTIONS(1863),
    [sym_case_keyword] = ACTIONS(1863),
    [sym_default_keyword] = ACTIONS(1863),
    [sym_while_keyword] = ACTIONS(1863),
    [sym_for_keyword] = ACTIONS(1863),
    [sym_break_keyword] = ACTIONS(1863),
    [sym_continue_keyword] = ACTIONS(1863),
    [sym_return_keyword] = ACTIONS(1863),
    [sym_select_keyword] = ACTIONS(1863),
    [sym_active_keyword] = ACTIONS(1863),
    [sym_struct_keyword] = ACTIONS(1863),
    [sym_structure_keyword] = ACTIONS(1863),
    [sym_button_event_keyword] = ACTIONS(1863),
    [sym_channel_event_keyword] = ACTIONS(1863),
    [sym_level_event_keyword] = ACTIONS(1863),
    [sym_data_event_keyword] = ACTIONS(1863),
    [sym_timeline_event_keyword] = ACTIONS(1863),
    [sym_custom_event_keyword] = ACTIONS(1863),
    [sym_push_keyword] = ACTIONS(1863),
    [sym_release_keyword] = ACTIONS(1863),
    [aux_sym_on_keyword_token1] = ACTIONS(1863),
    [aux_sym_off_keyword_token1] = ACTIONS(1863),
    [sym_send_level_keyword] = ACTIONS(1863),
    [sym_send_string_keyword] = ACTIONS(1863),
    [sym_send_command_keyword] = ACTIONS(1863),
    [sym_clear_buffer_keyword] = ACTIONS(1863),
    [sym_create_buffer_keyword] = ACTIONS(1863),
    [sym_create_multi_buffer_keyword] = ACTIONS(1863),
    [sym_call_keyword] = ACTIONS(1863),
    [sym_system_call_keyword] = ACTIONS(1863),
    [sym_devchan_to_keyword] = ACTIONS(1863),
    [sym_devchan_min_to_keyword] = ACTIONS(1863),
    [sym_devchan_total_off_keyword] = ACTIONS(1863),
    [sym_devchan_pulse_keyword] = ACTIONS(1863),
    [sym_wait_keyword] = ACTIONS(1863),
    [sym_cancel_wait_keyword] = ACTIONS(1863),
    [sym_wait_until_keyword] = ACTIONS(1863),
    [sym_cancel_wait_until_keyword] = ACTIONS(1863),
    [sym_cancel_all_wait_keyword] = ACTIONS(1863),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1863),
    [sym_not] = ACTIONS(1863),
    [sym_bnot] = ACTIONS(1863),
    [sym___file__] = ACTIONS(1863),
    [sym___date__] = ACTIONS(1863),
    [sym___ldate__] = ACTIONS(1863),
    [sym___time__] = ACTIONS(1863),
    [sym___line__] = ACTIONS(1863),
    [sym___name__] = ACTIONS(1863),
    [sym___netlinx__] = ACTIONS(1863),
    [sym_netlinx_axi_version] = ACTIONS(1863),
    [sym_url_flg_tcp] = ACTIONS(1863),
    [sym_url_flg_accinfopresent] = ACTIONS(1863),
    [sym_url_flg_temp] = ACTIONS(1863),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1863),
    [sym_url_flg_stat_mask] = ACTIONS(1863),
    [sym_url_flg_stat_lookup] = ACTIONS(1863),
    [sym_url_flg_stat_connecting] = ACTIONS(1863),
    [sym_url_flg_stat_waiting] = ACTIONS(1863),
    [sym_url_flg_stat_connected] = ACTIONS(1863),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1863),
    [sym_first_virtual_device] = ACTIONS(1863),
    [sym_first_local_port] = ACTIONS(1863),
    [sym_dynamic_virtual_device] = ACTIONS(1863),
    [sym_timeline_once] = ACTIONS(1863),
    [sym_timeline_repeat] = ACTIONS(1863),
    [sym_timeline_absolute] = ACTIONS(1863),
    [sym_timeline_relative] = ACTIONS(1863),
    [sym_do_push_timed_infinite] = ACTIONS(1863),
    [sym_source_type_no_address] = ACTIONS(1863),
    [sym_source_type_neuron_id] = ACTIONS(1863),
    [sym_source_type_ip_address] = ACTIONS(1863),
    [sym_source_type_axlink] = ACTIONS(1863),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1863),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1863),
    [sym_source_type_ip_socket_address] = ACTIONS(1863),
    [sym_source_type_rs232] = ACTIONS(1863),
    [sym_source_type_internal] = ACTIONS(1863),
    [sym_source_type_mac_address] = ACTIONS(1863),
    [sym_source_type_ipv4_port] = ACTIONS(1863),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1863),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1863),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1863),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1863),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1863),
    [sym_true] = ACTIONS(1863),
    [sym_false] = ACTIONS(1863),
    [sym_file_read_only] = ACTIONS(1863),
    [sym_file_rw_new] = ACTIONS(1863),
    [sym_file_rw_append] = ACTIONS(1863),
    [sym_ip_tcp] = ACTIONS(1863),
    [sym_ip_udp] = ACTIONS(1863),
    [sym_ip_udp_2way] = ACTIONS(1863),
    [sym_xml_encode_types] = ACTIONS(1863),
    [sym_xml_encode_char_as_list] = ACTIONS(1863),
    [sym_xml_encode_le] = ACTIONS(1863),
    [sym_xml_decode_types] = ACTIONS(1863),
    [sym_xml_decode_no_preserve] = ACTIONS(1863),
    [sym_duet_dev_type_display_device] = ACTIONS(1863),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1863),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1863),
    [sym_duet_dev_type_io_device] = ACTIONS(1863),
    [sym_duet_dev_type_relay_device] = ACTIONS(1863),
    [sym_duet_dev_type_ups] = ACTIONS(1863),
    [sym_duet_dev_type_amplifier] = ACTIONS(1863),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1863),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1863),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1863),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1863),
    [sym_duet_dev_type_camera] = ACTIONS(1863),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1863),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1863),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1863),
    [sym_duet_dev_type_dss] = ACTIONS(1863),
    [sym_duet_dev_type_dvr] = ACTIONS(1863),
    [sym_duet_dev_type_disc_device] = ACTIONS(1863),
    [sym_duet_dev_type_document_camera] = ACTIONS(1863),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1863),
    [sym_duet_dev_type_hvac] = ACTIONS(1863),
    [sym_duet_dev_type_keypad] = ACTIONS(1863),
    [sym_duet_dev_type_light] = ACTIONS(1863),
    [sym_duet_dev_type_monitor] = ACTIONS(1863),
    [sym_duet_dev_type_motor] = ACTIONS(1863),
    [sym_duet_dev_type_multi_window] = ACTIONS(1863),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1863),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1863),
    [sym_duet_dev_type_receiver] = ACTIONS(1863),
    [sym_duet_dev_type_security_system] = ACTIONS(1863),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1863),
    [sym_duet_dev_type_settop_box] = ACTIONS(1863),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1863),
    [sym_duet_dev_type_switcher] = ACTIONS(1863),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1863),
    [sym_duet_dev_type_tv] = ACTIONS(1863),
    [sym_duet_dev_type_utility] = ACTIONS(1863),
    [sym_duet_dev_type_vcr] = ACTIONS(1863),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1863),
    [sym_duet_dev_type_video_processor] = ACTIONS(1863),
    [sym_duet_dev_type_video_projector] = ACTIONS(1863),
    [sym_duet_dev_type_video_wall] = ACTIONS(1863),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1863),
    [sym_duet_dev_type_weather] = ACTIONS(1863),
    [sym_duet_dev_not_polled] = ACTIONS(1863),
    [sym_duet_dev_polled] = ACTIONS(1863),
    [sym_clkmgr_mode_network] = ACTIONS(1863),
    [sym_clkmgr_mode_standalone] = ACTIONS(1863),
    [sym_normal_standby] = ACTIONS(1863),
    [sym_normal_wake] = ACTIONS(1863),
    [sym_ip_multicast_ttl_option] = ACTIONS(1863),
    [sym_ip_ttl_subnet] = ACTIONS(1863),
    [sym_ip_ttl_site] = ACTIONS(1863),
    [sym_ip_ttl_region] = ACTIONS(1863),
    [sym_ip_ttl_continent] = ACTIONS(1863),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1863),
    [sym_ip_nodelay_on] = ACTIONS(1863),
    [sym_ip_nodelay_off] = ACTIONS(1863),
    [sym_amx_error] = ACTIONS(1863),
    [sym_amx_warning] = ACTIONS(1863),
    [sym_amx_info] = ACTIONS(1863),
    [sym_amx_debug] = ACTIONS(1863),
    [sym_netlinx_login_success] = ACTIONS(1863),
    [sym_netlinx_login_fail] = ACTIONS(1863),
    [sym_netlinx_logout] = ACTIONS(1863),
    [sym_null_str] = ACTIONS(1863),
    [sym_smtp_address] = ACTIONS(1863),
    [sym_smtp_port_number] = ACTIONS(1863),
    [sym_smtp_username] = ACTIONS(1863),
    [sym_smtp_password] = ACTIONS(1863),
    [sym_smtp_require_tls] = ACTIONS(1863),
    [sym_smtp_from] = ACTIONS(1863),
    [sym_smtp_tls_true] = ACTIONS(1863),
    [sym_smtp_tls_false] = ACTIONS(1863),
    [sym_listview_on_row_select_event] = ACTIONS(1863),
    [sym_data_structure_unknown] = ACTIONS(1863),
    [sym_data_structure_datafeed] = ACTIONS(1863),
    [sym_data_structure_datarecordset] = ACTIONS(1863),
    [sym_data_structure_datarecord] = ACTIONS(1863),
    [sym_data_structure_datafield] = ACTIONS(1863),
    [sym_data_max_content_field_count] = ACTIONS(1863),
    [sym_data_max_metadata_field_count] = ACTIONS(1863),
    [sym_data_max_id_length] = ACTIONS(1863),
    [sym_data_max_name_length] = ACTIONS(1863),
    [sym_data_max_description_length] = ACTIONS(1863),
    [sym_data_max_type_length] = ACTIONS(1863),
    [sym_data_max_format_length] = ACTIONS(1863),
    [sym_data_max_source_length] = ACTIONS(1863),
    [sym_data_max_value_length] = ACTIONS(1863),
    [sym_data_max_label_length] = ACTIONS(1863),
    [sym_data_max_url_length] = ACTIONS(1863),
    [sym_max_last_login_info_length] = ACTIONS(1863),
    [sym_data_type_unknown] = ACTIONS(1863),
    [sym_data_type_string] = ACTIONS(1863),
    [sym_data_type_datetime] = ACTIONS(1863),
    [sym_data_type_date] = ACTIONS(1863),
    [sym_data_type_time] = ACTIONS(1863),
    [sym_data_type_image] = ACTIONS(1863),
    [sym_data_format_url] = ACTIONS(1863),
    [sym_data_format_phone] = ACTIONS(1863),
    [sym_data_format_email] = ACTIONS(1863),
    [sym_data_format_iso8601] = ACTIONS(1863),
    [sym_auditlog_priv] = ACTIONS(1863),
    [sym_configuration_priv] = ACTIONS(1863),
    [sym_deviceconfig_priv] = ACTIONS(1863),
    [sym_ftp_priv] = ACTIONS(1863),
    [sym_http_priv] = ACTIONS(1863),
    [sym_networkconfig_priv] = ACTIONS(1863),
    [sym_programport_priv] = ACTIONS(1863),
    [sym_remoteui_priv] = ACTIONS(1863),
    [sym_securitycontrol_priv] = ACTIONS(1863),
    [sym_softwaremanagement_priv] = ACTIONS(1863),
    [sym_terminal_priv] = ACTIONS(1863),
    [sym_tpadmin_priv] = ACTIONS(1863),
    [sym_user1_priv] = ACTIONS(1863),
    [sym_user2_priv] = ACTIONS(1863),
    [sym_user3_priv] = ACTIONS(1863),
    [sym_user4_priv] = ACTIONS(1863),
    [sym_usermanagement_priv] = ACTIONS(1863),
    [sym_valid_account] = ACTIONS(1863),
    [sym_err_name_invalid] = ACTIONS(1863),
    [sym_err_password_invalid] = ACTIONS(1863),
    [sym_err_invalid_account] = ACTIONS(1863),
    [sym_err_no_authorization] = ACTIONS(1863),
    [sym_err_invalid_parameter] = ACTIONS(1863),
    [sym_err_user_locked_out] = ACTIONS(1863),
    [sym_err_user_account_expired] = ACTIONS(1863),
    [sym_err_general_error] = ACTIONS(1863),
    [sym_err_authentication_error] = ACTIONS(1863),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1863),
    [sym_tls_validate_certificate] = ACTIONS(1863),
    [sym_tdata] = ACTIONS(1863),
    [sym_tchannel] = ACTIONS(1863),
    [sym_tlevel] = ACTIONS(1863),
    [sym_tbutton] = ACTIONS(1863),
    [sym_ttimeline] = ACTIONS(1863),
    [sym_tcustom] = ACTIONS(1863),
    [sym_url_struct] = ACTIONS(1863),
    [sym_dns_struct] = ACTIONS(1863),
    [sym_ip_address_struct] = ACTIONS(1863),
    [sym_dev_info_struct] = ACTIONS(1863),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1863),
    [sym_data_feed] = ACTIONS(1863),
    [sym_data_field] = ACTIONS(1863),
    [sym_data_record] = ACTIONS(1863),
    [sym_wc_data_feed] = ACTIONS(1863),
    [sym_wc_data_field] = ACTIONS(1863),
    [sym_wc_data_record] = ACTIONS(1863),
    [sym_last_login_info] = ACTIONS(1863),
    [sym____reserved___] = ACTIONS(1863),
    [sym_date] = ACTIONS(1863),
    [sym_day] = ACTIONS(1863),
    [sym_time] = ACTIONS(1863),
    [sym_ldate] = ACTIONS(1863),
    [sym_push_device] = ACTIONS(1863),
    [sym_push_channel] = ACTIONS(1863),
    [sym_push_devchan] = ACTIONS(1863),
    [sym_release_device] = ACTIONS(1863),
    [sym_release_channel] = ACTIONS(1863),
    [sym_release_devchan] = ACTIONS(1863),
    [sym_master_slot] = ACTIONS(1863),
    [sym_get_pulse_time] = ACTIONS(1863),
    [sym_get_timer] = ACTIONS(1863),
    [sym_master_sn] = ACTIONS(1863),
    [sym_system_number] = ACTIONS(1863),
    [sym_button] = ACTIONS(1863),
    [sym_dv_channel] = ACTIONS(1863),
    [sym_data] = ACTIONS(1863),
    [sym_level] = ACTIONS(1863),
    [sym_timeline] = ACTIONS(1863),
    [sym_channel] = ACTIONS(1863),
    [sym_custom] = ACTIONS(1863),
    [sym_atoi] = ACTIONS(1863),
    [sym_atol] = ACTIONS(1863),
    [sym_atof] = ACTIONS(1863),
    [sym_itoa] = ACTIONS(1863),
    [sym_itohex] = ACTIONS(1863),
    [sym_hextoi] = ACTIONS(1863),
    [sym_ftoa] = ACTIONS(1863),
    [sym_find_string] = ACTIONS(1863),
    [sym_left_string] = ACTIONS(1863),
    [sym_right_string] = ACTIONS(1863),
    [sym_mid_string] = ACTIONS(1863),
    [sym_remove_string] = ACTIONS(1863),
    [sym_get_buffer_string] = ACTIONS(1863),
    [sym_get_buffer_char] = ACTIONS(1863),
    [sym_get_multi_buffer_string] = ACTIONS(1863),
    [sym_length_string] = ACTIONS(1863),
    [sym_max_length_string] = ACTIONS(1863),
    [sym_set_length_string] = ACTIONS(1863),
    [sym_compare_string] = ACTIONS(1863),
    [sym_upper_string] = ACTIONS(1863),
    [sym_lower_string] = ACTIONS(1863),
    [sym_get_last] = ACTIONS(1863),
    [sym_date_to_day] = ACTIONS(1863),
    [sym_date_to_month] = ACTIONS(1863),
    [sym_date_to_year] = ACTIONS(1863),
    [sym_time_to_hour] = ACTIONS(1863),
    [sym_time_to_minute] = ACTIONS(1863),
    [sym_time_to_second] = ACTIONS(1863),
    [sym_day_of_week] = ACTIONS(1863),
    [sym_set_pulse_time] = ACTIONS(1863),
    [sym_set_timer] = ACTIONS(1863),
    [sym_file_open] = ACTIONS(1863),
    [sym_file_close] = ACTIONS(1863),
    [sym_file_read] = ACTIONS(1863),
    [sym_file_read_line] = ACTIONS(1863),
    [sym_file_write] = ACTIONS(1863),
    [sym_file_write_line] = ACTIONS(1863),
    [sym_file_copy] = ACTIONS(1863),
    [sym_file_delete] = ACTIONS(1863),
    [sym_file_rename] = ACTIONS(1863),
    [sym_file_dir] = ACTIONS(1863),
    [sym_file_getdir] = ACTIONS(1863),
    [sym_file_setdir] = ACTIONS(1863),
    [sym_file_create_dir] = ACTIONS(1863),
    [sym_file_remove_dir] = ACTIONS(1863),
    [sym_file_seek] = ACTIONS(1863),
    [sym_length_array] = ACTIONS(1863),
    [sym_max_length_array] = ACTIONS(1863),
    [sym_set_length_array] = ACTIONS(1863),
    [sym_ip_client_open] = ACTIONS(1863),
    [sym_ip_client_close] = ACTIONS(1863),
    [sym_ip_server_open] = ACTIONS(1863),
    [sym_ip_server_close] = ACTIONS(1863),
    [sym_ip_bound_client_open] = ACTIONS(1863),
    [sym_ip_mc_server_open] = ACTIONS(1863),
    [sym_ip_set_option] = ACTIONS(1863),
    [sym_ssh_client_open] = ACTIONS(1863),
    [sym_ssh_client_close] = ACTIONS(1863),
    [sym_tls_client_open] = ACTIONS(1863),
    [sym_tls_client_close] = ACTIONS(1863),
    [sym_do_push] = ACTIONS(1863),
    [sym_do_push_timed] = ACTIONS(1863),
    [sym_do_release] = ACTIONS(1863),
    [sym_device_id] = ACTIONS(1863),
    [sym_device_info] = ACTIONS(1863),
    [sym_device_id_string] = ACTIONS(1863),
    [sym_device_standby] = ACTIONS(1863),
    [sym_device_wake] = ACTIONS(1863),
    [sym_timeline_create] = ACTIONS(1863),
    [sym_timeline_kill] = ACTIONS(1863),
    [sym_timeline_reload] = ACTIONS(1863),
    [sym_timeline_pause] = ACTIONS(1863),
    [sym_timeline_restart] = ACTIONS(1863),
    [sym_timeline_set] = ACTIONS(1863),
    [sym_timeline_get] = ACTIONS(1863),
    [sym_timeline_active] = ACTIONS(1863),
    [sym_timeline_dynamic_id] = ACTIONS(1863),
    [sym_variable_to_string] = ACTIONS(1863),
    [sym_string_to_variable] = ACTIONS(1863),
    [sym_length_variable_to_string] = ACTIONS(1863),
    [sym_type_cast] = ACTIONS(1863),
    [sym_variable_to_xml] = ACTIONS(1863),
    [sym_xml_to_variable] = ACTIONS(1863),
    [sym_length_variable_to_xml] = ACTIONS(1863),
    [sym_redirect_string] = ACTIONS(1863),
    [sym_random_number] = ACTIONS(1863),
    [sym_get_unique_id] = ACTIONS(1863),
    [sym_get_system_number] = ACTIONS(1863),
    [sym_set_system_number] = ACTIONS(1863),
    [sym_get_serial_number] = ACTIONS(1863),
    [sym_get_master_build] = ACTIONS(1863),
    [sym_set_validation_code] = ACTIONS(1863),
    [sym_reboot] = ACTIONS(1863),
    [sym_load_duet_module] = ACTIONS(1863),
    [sym_unload_duet_module] = ACTIONS(1863),
    [sym_duet_mem_size_set] = ACTIONS(1863),
    [sym_duet_mem_size_get] = ACTIONS(1863),
    [sym_dynamic_application_device] = ACTIONS(1863),
    [sym_dynamic_polled_port] = ACTIONS(1863),
    [sym_static_port_binding] = ACTIONS(1863),
    [sym_static_ip_binding] = ACTIONS(1863),
    [sym_get_dns_list] = ACTIONS(1863),
    [sym_set_dns_list] = ACTIONS(1863),
    [sym_get_ip_address] = ACTIONS(1863),
    [sym_set_ip_address] = ACTIONS(1863),
    [sym_get_url_list] = ACTIONS(1863),
    [sym_add_url_entry] = ACTIONS(1863),
    [sym_delete_url_entry] = ACTIONS(1863),
    [sym_abs_value] = ACTIONS(1863),
    [sym_max_value] = ACTIONS(1863),
    [sym_min_value] = ACTIONS(1863),
    [sym_exp_value] = ACTIONS(1863),
    [sym_log_value] = ACTIONS(1863),
    [sym_log10_value] = ACTIONS(1863),
    [sym_power_value] = ACTIONS(1863),
    [sym_sqrt_value] = ACTIONS(1863),
    [sym_raw_be] = ACTIONS(1863),
    [sym_raw_le] = ACTIONS(1863),
    [sym_internal_queue_size_set] = ACTIONS(1863),
    [sym_internal_queue_size_get] = ACTIONS(1863),
    [sym_internal_threshold_set] = ACTIONS(1863),
    [sym_internal_threshold_get] = ACTIONS(1863),
    [sym_rebuilt_event] = ACTIONS(1863),
    [sym_do_custom_event] = ACTIONS(1863),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1863),
    [sym_clkmgr_set_clk_source] = ACTIONS(1863),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1863),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1863),
    [sym_clkmgr_get_timezone] = ACTIONS(1863),
    [sym_clkmgr_set_timezone] = ACTIONS(1863),
    [sym_clkmgr_get_resync_period] = ACTIONS(1863),
    [sym_clkmgr_set_resync_period] = ACTIONS(1863),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1863),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1863),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1863),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1863),
    [sym_clkmgr_get_timeservers] = ACTIONS(1863),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1863),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1863),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1863),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1863),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1863),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1863),
    [sym_set_log_level] = ACTIONS(1863),
    [sym_get_log_level] = ACTIONS(1863),
    [sym_amx_log] = ACTIONS(1863),
    [sym_smtp_server_config_set] = ACTIONS(1863),
    [sym_smtp_server_config_get] = ACTIONS(1863),
    [sym_smtp_send] = ACTIONS(1863),
    [sym_validate_netlinx_account] = ACTIONS(1863),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1863),
    [sym_authenticate_certificate] = ACTIONS(1863),
    [sym_audit_netlinx_session_event] = ACTIONS(1863),
    [sym_audit_netlinx_generic_event] = ACTIONS(1863),
    [sym_get_available_flash_disk_space] = ACTIONS(1863),
    [sym_get_max_flash_disk_space] = ACTIONS(1863),
    [sym_data_create_feed] = ACTIONS(1863),
    [sym_data_delete_feed] = ACTIONS(1863),
    [sym_data_publish_feed] = ACTIONS(1863),
    [sym_data_get_published_feed] = ACTIONS(1863),
    [sym_data_add_record] = ACTIONS(1863),
    [sym_data_get_event_record] = ACTIONS(1863),
    [sym__wc_data_create_feed] = ACTIONS(1863),
    [sym__wc_data_add_record] = ACTIONS(1863),
    [sym__wc_data_get_event_record] = ACTIONS(1863),
    [sym_format] = ACTIONS(1863),
    [sym_astro_clock] = ACTIONS(1863),
    [sym_set_outdoor_temperature] = ACTIONS(1863),
    [sym_set_virtual_level_count] = ACTIONS(1863),
    [sym_set_virtual_channel_count] = ACTIONS(1863),
    [sym_set_virtual_port_count] = ACTIONS(1863),
    [sym__wc_encode_int] = ACTIONS(1863),
    [sym__wc_decode_int] = ACTIONS(1863),
    [sym__wc_int] = ACTIONS(1863),
    [sym__wc_to_ch_int] = ACTIONS(1863),
    [sym__ch_to_wc_int] = ACTIONS(1863),
    [sym__wc_find_string_int] = ACTIONS(1863),
    [sym__wc_left_string_int] = ACTIONS(1863),
    [sym__wc_lower_string_int] = ACTIONS(1863),
    [sym__wc_mid_string_int] = ACTIONS(1863),
    [sym__wc_remove_string_int] = ACTIONS(1863),
    [sym__wc_right_string_int] = ACTIONS(1863),
    [sym__wc_upper_string_int] = ACTIONS(1863),
    [sym__wc_compare_string_int] = ACTIONS(1863),
    [sym__wc_get_buffer_char_int] = ACTIONS(1863),
    [sym__wc_get_buffer_string_int] = ACTIONS(1863),
    [sym__wc_concat_string_int] = ACTIONS(1863),
    [sym__wc_file_open_int] = ACTIONS(1863),
    [sym__wc_file_close_int] = ACTIONS(1863),
    [sym__wc_file_read_int] = ACTIONS(1863),
    [sym__wc_file_read_line_int] = ACTIONS(1863),
    [sym__wc_file_write_int] = ACTIONS(1863),
    [sym__wc_file_write_line_int] = ACTIONS(1863),
    [aux_sym_preproc_include_token1] = ACTIONS(1863),
    [aux_sym_preproc_define_token1] = ACTIONS(1865),
    [aux_sym_preproc_warn_token1] = ACTIONS(1865),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1865),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1865),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1865),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1865),
    [aux_sym_preproc_else_token1] = ACTIONS(1865),
    [anon_sym_LBRACK] = ACTIONS(1865),
    [anon_sym_LBRACE] = ACTIONS(1865),
    [anon_sym_LPAREN] = ACTIONS(1863),
    [anon_sym_BANG] = ACTIONS(1865),
    [anon_sym_TILDE] = ACTIONS(1865),
    [anon_sym_DASH] = ACTIONS(1863),
    [anon_sym_PLUS] = ACTIONS(1863),
    [anon_sym_DASH_DASH] = ACTIONS(1865),
    [anon_sym_PLUS_PLUS] = ACTIONS(1865),
    [anon_sym_DQUOTE] = ACTIONS(1865),
    [anon_sym_SQUOTE] = ACTIONS(1865),
    [sym_number_literal] = ACTIONS(1865),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(262)] = {
    [sym_identifier] = ACTIONS(1867),
    [sym_program_name_keyword] = ACTIONS(1867),
    [sym_module_name_keyword] = ACTIONS(1867),
    [sym_define_device_keyword] = ACTIONS(1867),
    [sym_define_combine_keyword] = ACTIONS(1867),
    [sym_define_constant_keyword] = ACTIONS(1867),
    [sym_define_type_keyword] = ACTIONS(1867),
    [sym_define_variable_keyword] = ACTIONS(1867),
    [sym_define_system_variable_keyword] = ACTIONS(1867),
    [sym_define_connect_level_keyword] = ACTIONS(1867),
    [sym_define_latching_keyword] = ACTIONS(1867),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1867),
    [sym_define_toggling_keyword] = ACTIONS(1867),
    [sym_define_call_keyword] = ACTIONS(1867),
    [sym_define_function_keyword] = ACTIONS(1867),
    [sym_define_library_function_keyword] = ACTIONS(1867),
    [sym_define_start_keyword] = ACTIONS(1867),
    [sym_define_event_keyword] = ACTIONS(1867),
    [sym_define_program_keyword] = ACTIONS(1867),
    [sym_define_module_keyword] = ACTIONS(1867),
    [sym_char_keyword] = ACTIONS(1867),
    [sym_widechar_keyword] = ACTIONS(1867),
    [sym_integer_keyword] = ACTIONS(1867),
    [sym_sinteger_keyword] = ACTIONS(1867),
    [sym_long_keyword] = ACTIONS(1867),
    [sym_slong_keyword] = ACTIONS(1867),
    [sym_float_keyword] = ACTIONS(1867),
    [sym_double_keyword] = ACTIONS(1867),
    [sym_variant_keyword] = ACTIONS(1867),
    [sym_variantarray_keyword] = ACTIONS(1867),
    [sym_local_var_keyword] = ACTIONS(1867),
    [sym_stack_var_keyword] = ACTIONS(1867),
    [sym_constant_keyword] = ACTIONS(1867),
    [sym_volatile_keyword] = ACTIONS(1867),
    [sym_non_volatile_keyword] = ACTIONS(1867),
    [sym_persistent_keyword] = ACTIONS(1867),
    [sym_dev_keyword] = ACTIONS(1867),
    [sym_devlev_keyword] = ACTIONS(1867),
    [sym_devchan_keyword] = ACTIONS(1867),
    [sym_if_keyword] = ACTIONS(1867),
    [sym_else_keyword] = ACTIONS(1867),
    [sym_switch_keyword] = ACTIONS(1867),
    [sym_case_keyword] = ACTIONS(1867),
    [sym_default_keyword] = ACTIONS(1867),
    [sym_while_keyword] = ACTIONS(1867),
    [sym_for_keyword] = ACTIONS(1867),
    [sym_break_keyword] = ACTIONS(1867),
    [sym_continue_keyword] = ACTIONS(1867),
    [sym_return_keyword] = ACTIONS(1867),
    [sym_select_keyword] = ACTIONS(1867),
    [sym_active_keyword] = ACTIONS(1867),
    [sym_struct_keyword] = ACTIONS(1867),
    [sym_structure_keyword] = ACTIONS(1867),
    [sym_button_event_keyword] = ACTIONS(1867),
    [sym_channel_event_keyword] = ACTIONS(1867),
    [sym_level_event_keyword] = ACTIONS(1867),
    [sym_data_event_keyword] = ACTIONS(1867),
    [sym_timeline_event_keyword] = ACTIONS(1867),
    [sym_custom_event_keyword] = ACTIONS(1867),
    [sym_push_keyword] = ACTIONS(1867),
    [sym_release_keyword] = ACTIONS(1867),
    [aux_sym_on_keyword_token1] = ACTIONS(1867),
    [aux_sym_off_keyword_token1] = ACTIONS(1867),
    [sym_send_level_keyword] = ACTIONS(1867),
    [sym_send_string_keyword] = ACTIONS(1867),
    [sym_send_command_keyword] = ACTIONS(1867),
    [sym_clear_buffer_keyword] = ACTIONS(1867),
    [sym_create_buffer_keyword] = ACTIONS(1867),
    [sym_create_multi_buffer_keyword] = ACTIONS(1867),
    [sym_call_keyword] = ACTIONS(1867),
    [sym_system_call_keyword] = ACTIONS(1867),
    [sym_devchan_to_keyword] = ACTIONS(1867),
    [sym_devchan_min_to_keyword] = ACTIONS(1867),
    [sym_devchan_total_off_keyword] = ACTIONS(1867),
    [sym_devchan_pulse_keyword] = ACTIONS(1867),
    [sym_wait_keyword] = ACTIONS(1867),
    [sym_cancel_wait_keyword] = ACTIONS(1867),
    [sym_wait_until_keyword] = ACTIONS(1867),
    [sym_cancel_wait_until_keyword] = ACTIONS(1867),
    [sym_cancel_all_wait_keyword] = ACTIONS(1867),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1867),
    [sym_not] = ACTIONS(1867),
    [sym_bnot] = ACTIONS(1867),
    [sym___file__] = ACTIONS(1867),
    [sym___date__] = ACTIONS(1867),
    [sym___ldate__] = ACTIONS(1867),
    [sym___time__] = ACTIONS(1867),
    [sym___line__] = ACTIONS(1867),
    [sym___name__] = ACTIONS(1867),
    [sym___netlinx__] = ACTIONS(1867),
    [sym_netlinx_axi_version] = ACTIONS(1867),
    [sym_url_flg_tcp] = ACTIONS(1867),
    [sym_url_flg_accinfopresent] = ACTIONS(1867),
    [sym_url_flg_temp] = ACTIONS(1867),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1867),
    [sym_url_flg_stat_mask] = ACTIONS(1867),
    [sym_url_flg_stat_lookup] = ACTIONS(1867),
    [sym_url_flg_stat_connecting] = ACTIONS(1867),
    [sym_url_flg_stat_waiting] = ACTIONS(1867),
    [sym_url_flg_stat_connected] = ACTIONS(1867),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1867),
    [sym_first_virtual_device] = ACTIONS(1867),
    [sym_first_local_port] = ACTIONS(1867),
    [sym_dynamic_virtual_device] = ACTIONS(1867),
    [sym_timeline_once] = ACTIONS(1867),
    [sym_timeline_repeat] = ACTIONS(1867),
    [sym_timeline_absolute] = ACTIONS(1867),
    [sym_timeline_relative] = ACTIONS(1867),
    [sym_do_push_timed_infinite] = ACTIONS(1867),
    [sym_source_type_no_address] = ACTIONS(1867),
    [sym_source_type_neuron_id] = ACTIONS(1867),
    [sym_source_type_ip_address] = ACTIONS(1867),
    [sym_source_type_axlink] = ACTIONS(1867),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1867),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1867),
    [sym_source_type_ip_socket_address] = ACTIONS(1867),
    [sym_source_type_rs232] = ACTIONS(1867),
    [sym_source_type_internal] = ACTIONS(1867),
    [sym_source_type_mac_address] = ACTIONS(1867),
    [sym_source_type_ipv4_port] = ACTIONS(1867),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1867),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1867),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1867),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1867),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1867),
    [sym_true] = ACTIONS(1867),
    [sym_false] = ACTIONS(1867),
    [sym_file_read_only] = ACTIONS(1867),
    [sym_file_rw_new] = ACTIONS(1867),
    [sym_file_rw_append] = ACTIONS(1867),
    [sym_ip_tcp] = ACTIONS(1867),
    [sym_ip_udp] = ACTIONS(1867),
    [sym_ip_udp_2way] = ACTIONS(1867),
    [sym_xml_encode_types] = ACTIONS(1867),
    [sym_xml_encode_char_as_list] = ACTIONS(1867),
    [sym_xml_encode_le] = ACTIONS(1867),
    [sym_xml_decode_types] = ACTIONS(1867),
    [sym_xml_decode_no_preserve] = ACTIONS(1867),
    [sym_duet_dev_type_display_device] = ACTIONS(1867),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1867),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1867),
    [sym_duet_dev_type_io_device] = ACTIONS(1867),
    [sym_duet_dev_type_relay_device] = ACTIONS(1867),
    [sym_duet_dev_type_ups] = ACTIONS(1867),
    [sym_duet_dev_type_amplifier] = ACTIONS(1867),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1867),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1867),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1867),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1867),
    [sym_duet_dev_type_camera] = ACTIONS(1867),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1867),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1867),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1867),
    [sym_duet_dev_type_dss] = ACTIONS(1867),
    [sym_duet_dev_type_dvr] = ACTIONS(1867),
    [sym_duet_dev_type_disc_device] = ACTIONS(1867),
    [sym_duet_dev_type_document_camera] = ACTIONS(1867),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1867),
    [sym_duet_dev_type_hvac] = ACTIONS(1867),
    [sym_duet_dev_type_keypad] = ACTIONS(1867),
    [sym_duet_dev_type_light] = ACTIONS(1867),
    [sym_duet_dev_type_monitor] = ACTIONS(1867),
    [sym_duet_dev_type_motor] = ACTIONS(1867),
    [sym_duet_dev_type_multi_window] = ACTIONS(1867),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1867),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1867),
    [sym_duet_dev_type_receiver] = ACTIONS(1867),
    [sym_duet_dev_type_security_system] = ACTIONS(1867),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1867),
    [sym_duet_dev_type_settop_box] = ACTIONS(1867),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1867),
    [sym_duet_dev_type_switcher] = ACTIONS(1867),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1867),
    [sym_duet_dev_type_tv] = ACTIONS(1867),
    [sym_duet_dev_type_utility] = ACTIONS(1867),
    [sym_duet_dev_type_vcr] = ACTIONS(1867),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1867),
    [sym_duet_dev_type_video_processor] = ACTIONS(1867),
    [sym_duet_dev_type_video_projector] = ACTIONS(1867),
    [sym_duet_dev_type_video_wall] = ACTIONS(1867),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1867),
    [sym_duet_dev_type_weather] = ACTIONS(1867),
    [sym_duet_dev_not_polled] = ACTIONS(1867),
    [sym_duet_dev_polled] = ACTIONS(1867),
    [sym_clkmgr_mode_network] = ACTIONS(1867),
    [sym_clkmgr_mode_standalone] = ACTIONS(1867),
    [sym_normal_standby] = ACTIONS(1867),
    [sym_normal_wake] = ACTIONS(1867),
    [sym_ip_multicast_ttl_option] = ACTIONS(1867),
    [sym_ip_ttl_subnet] = ACTIONS(1867),
    [sym_ip_ttl_site] = ACTIONS(1867),
    [sym_ip_ttl_region] = ACTIONS(1867),
    [sym_ip_ttl_continent] = ACTIONS(1867),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1867),
    [sym_ip_nodelay_on] = ACTIONS(1867),
    [sym_ip_nodelay_off] = ACTIONS(1867),
    [sym_amx_error] = ACTIONS(1867),
    [sym_amx_warning] = ACTIONS(1867),
    [sym_amx_info] = ACTIONS(1867),
    [sym_amx_debug] = ACTIONS(1867),
    [sym_netlinx_login_success] = ACTIONS(1867),
    [sym_netlinx_login_fail] = ACTIONS(1867),
    [sym_netlinx_logout] = ACTIONS(1867),
    [sym_null_str] = ACTIONS(1867),
    [sym_smtp_address] = ACTIONS(1867),
    [sym_smtp_port_number] = ACTIONS(1867),
    [sym_smtp_username] = ACTIONS(1867),
    [sym_smtp_password] = ACTIONS(1867),
    [sym_smtp_require_tls] = ACTIONS(1867),
    [sym_smtp_from] = ACTIONS(1867),
    [sym_smtp_tls_true] = ACTIONS(1867),
    [sym_smtp_tls_false] = ACTIONS(1867),
    [sym_listview_on_row_select_event] = ACTIONS(1867),
    [sym_data_structure_unknown] = ACTIONS(1867),
    [sym_data_structure_datafeed] = ACTIONS(1867),
    [sym_data_structure_datarecordset] = ACTIONS(1867),
    [sym_data_structure_datarecord] = ACTIONS(1867),
    [sym_data_structure_datafield] = ACTIONS(1867),
    [sym_data_max_content_field_count] = ACTIONS(1867),
    [sym_data_max_metadata_field_count] = ACTIONS(1867),
    [sym_data_max_id_length] = ACTIONS(1867),
    [sym_data_max_name_length] = ACTIONS(1867),
    [sym_data_max_description_length] = ACTIONS(1867),
    [sym_data_max_type_length] = ACTIONS(1867),
    [sym_data_max_format_length] = ACTIONS(1867),
    [sym_data_max_source_length] = ACTIONS(1867),
    [sym_data_max_value_length] = ACTIONS(1867),
    [sym_data_max_label_length] = ACTIONS(1867),
    [sym_data_max_url_length] = ACTIONS(1867),
    [sym_max_last_login_info_length] = ACTIONS(1867),
    [sym_data_type_unknown] = ACTIONS(1867),
    [sym_data_type_string] = ACTIONS(1867),
    [sym_data_type_datetime] = ACTIONS(1867),
    [sym_data_type_date] = ACTIONS(1867),
    [sym_data_type_time] = ACTIONS(1867),
    [sym_data_type_image] = ACTIONS(1867),
    [sym_data_format_url] = ACTIONS(1867),
    [sym_data_format_phone] = ACTIONS(1867),
    [sym_data_format_email] = ACTIONS(1867),
    [sym_data_format_iso8601] = ACTIONS(1867),
    [sym_auditlog_priv] = ACTIONS(1867),
    [sym_configuration_priv] = ACTIONS(1867),
    [sym_deviceconfig_priv] = ACTIONS(1867),
    [sym_ftp_priv] = ACTIONS(1867),
    [sym_http_priv] = ACTIONS(1867),
    [sym_networkconfig_priv] = ACTIONS(1867),
    [sym_programport_priv] = ACTIONS(1867),
    [sym_remoteui_priv] = ACTIONS(1867),
    [sym_securitycontrol_priv] = ACTIONS(1867),
    [sym_softwaremanagement_priv] = ACTIONS(1867),
    [sym_terminal_priv] = ACTIONS(1867),
    [sym_tpadmin_priv] = ACTIONS(1867),
    [sym_user1_priv] = ACTIONS(1867),
    [sym_user2_priv] = ACTIONS(1867),
    [sym_user3_priv] = ACTIONS(1867),
    [sym_user4_priv] = ACTIONS(1867),
    [sym_usermanagement_priv] = ACTIONS(1867),
    [sym_valid_account] = ACTIONS(1867),
    [sym_err_name_invalid] = ACTIONS(1867),
    [sym_err_password_invalid] = ACTIONS(1867),
    [sym_err_invalid_account] = ACTIONS(1867),
    [sym_err_no_authorization] = ACTIONS(1867),
    [sym_err_invalid_parameter] = ACTIONS(1867),
    [sym_err_user_locked_out] = ACTIONS(1867),
    [sym_err_user_account_expired] = ACTIONS(1867),
    [sym_err_general_error] = ACTIONS(1867),
    [sym_err_authentication_error] = ACTIONS(1867),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1867),
    [sym_tls_validate_certificate] = ACTIONS(1867),
    [sym_tdata] = ACTIONS(1867),
    [sym_tchannel] = ACTIONS(1867),
    [sym_tlevel] = ACTIONS(1867),
    [sym_tbutton] = ACTIONS(1867),
    [sym_ttimeline] = ACTIONS(1867),
    [sym_tcustom] = ACTIONS(1867),
    [sym_url_struct] = ACTIONS(1867),
    [sym_dns_struct] = ACTIONS(1867),
    [sym_ip_address_struct] = ACTIONS(1867),
    [sym_dev_info_struct] = ACTIONS(1867),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1867),
    [sym_data_feed] = ACTIONS(1867),
    [sym_data_field] = ACTIONS(1867),
    [sym_data_record] = ACTIONS(1867),
    [sym_wc_data_feed] = ACTIONS(1867),
    [sym_wc_data_field] = ACTIONS(1867),
    [sym_wc_data_record] = ACTIONS(1867),
    [sym_last_login_info] = ACTIONS(1867),
    [sym____reserved___] = ACTIONS(1867),
    [sym_date] = ACTIONS(1867),
    [sym_day] = ACTIONS(1867),
    [sym_time] = ACTIONS(1867),
    [sym_ldate] = ACTIONS(1867),
    [sym_push_device] = ACTIONS(1867),
    [sym_push_channel] = ACTIONS(1867),
    [sym_push_devchan] = ACTIONS(1867),
    [sym_release_device] = ACTIONS(1867),
    [sym_release_channel] = ACTIONS(1867),
    [sym_release_devchan] = ACTIONS(1867),
    [sym_master_slot] = ACTIONS(1867),
    [sym_get_pulse_time] = ACTIONS(1867),
    [sym_get_timer] = ACTIONS(1867),
    [sym_master_sn] = ACTIONS(1867),
    [sym_system_number] = ACTIONS(1867),
    [sym_button] = ACTIONS(1867),
    [sym_dv_channel] = ACTIONS(1867),
    [sym_data] = ACTIONS(1867),
    [sym_level] = ACTIONS(1867),
    [sym_timeline] = ACTIONS(1867),
    [sym_channel] = ACTIONS(1867),
    [sym_custom] = ACTIONS(1867),
    [sym_atoi] = ACTIONS(1867),
    [sym_atol] = ACTIONS(1867),
    [sym_atof] = ACTIONS(1867),
    [sym_itoa] = ACTIONS(1867),
    [sym_itohex] = ACTIONS(1867),
    [sym_hextoi] = ACTIONS(1867),
    [sym_ftoa] = ACTIONS(1867),
    [sym_find_string] = ACTIONS(1867),
    [sym_left_string] = ACTIONS(1867),
    [sym_right_string] = ACTIONS(1867),
    [sym_mid_string] = ACTIONS(1867),
    [sym_remove_string] = ACTIONS(1867),
    [sym_get_buffer_string] = ACTIONS(1867),
    [sym_get_buffer_char] = ACTIONS(1867),
    [sym_get_multi_buffer_string] = ACTIONS(1867),
    [sym_length_string] = ACTIONS(1867),
    [sym_max_length_string] = ACTIONS(1867),
    [sym_set_length_string] = ACTIONS(1867),
    [sym_compare_string] = ACTIONS(1867),
    [sym_upper_string] = ACTIONS(1867),
    [sym_lower_string] = ACTIONS(1867),
    [sym_get_last] = ACTIONS(1867),
    [sym_date_to_day] = ACTIONS(1867),
    [sym_date_to_month] = ACTIONS(1867),
    [sym_date_to_year] = ACTIONS(1867),
    [sym_time_to_hour] = ACTIONS(1867),
    [sym_time_to_minute] = ACTIONS(1867),
    [sym_time_to_second] = ACTIONS(1867),
    [sym_day_of_week] = ACTIONS(1867),
    [sym_set_pulse_time] = ACTIONS(1867),
    [sym_set_timer] = ACTIONS(1867),
    [sym_file_open] = ACTIONS(1867),
    [sym_file_close] = ACTIONS(1867),
    [sym_file_read] = ACTIONS(1867),
    [sym_file_read_line] = ACTIONS(1867),
    [sym_file_write] = ACTIONS(1867),
    [sym_file_write_line] = ACTIONS(1867),
    [sym_file_copy] = ACTIONS(1867),
    [sym_file_delete] = ACTIONS(1867),
    [sym_file_rename] = ACTIONS(1867),
    [sym_file_dir] = ACTIONS(1867),
    [sym_file_getdir] = ACTIONS(1867),
    [sym_file_setdir] = ACTIONS(1867),
    [sym_file_create_dir] = ACTIONS(1867),
    [sym_file_remove_dir] = ACTIONS(1867),
    [sym_file_seek] = ACTIONS(1867),
    [sym_length_array] = ACTIONS(1867),
    [sym_max_length_array] = ACTIONS(1867),
    [sym_set_length_array] = ACTIONS(1867),
    [sym_ip_client_open] = ACTIONS(1867),
    [sym_ip_client_close] = ACTIONS(1867),
    [sym_ip_server_open] = ACTIONS(1867),
    [sym_ip_server_close] = ACTIONS(1867),
    [sym_ip_bound_client_open] = ACTIONS(1867),
    [sym_ip_mc_server_open] = ACTIONS(1867),
    [sym_ip_set_option] = ACTIONS(1867),
    [sym_ssh_client_open] = ACTIONS(1867),
    [sym_ssh_client_close] = ACTIONS(1867),
    [sym_tls_client_open] = ACTIONS(1867),
    [sym_tls_client_close] = ACTIONS(1867),
    [sym_do_push] = ACTIONS(1867),
    [sym_do_push_timed] = ACTIONS(1867),
    [sym_do_release] = ACTIONS(1867),
    [sym_device_id] = ACTIONS(1867),
    [sym_device_info] = ACTIONS(1867),
    [sym_device_id_string] = ACTIONS(1867),
    [sym_device_standby] = ACTIONS(1867),
    [sym_device_wake] = ACTIONS(1867),
    [sym_timeline_create] = ACTIONS(1867),
    [sym_timeline_kill] = ACTIONS(1867),
    [sym_timeline_reload] = ACTIONS(1867),
    [sym_timeline_pause] = ACTIONS(1867),
    [sym_timeline_restart] = ACTIONS(1867),
    [sym_timeline_set] = ACTIONS(1867),
    [sym_timeline_get] = ACTIONS(1867),
    [sym_timeline_active] = ACTIONS(1867),
    [sym_timeline_dynamic_id] = ACTIONS(1867),
    [sym_variable_to_string] = ACTIONS(1867),
    [sym_string_to_variable] = ACTIONS(1867),
    [sym_length_variable_to_string] = ACTIONS(1867),
    [sym_type_cast] = ACTIONS(1867),
    [sym_variable_to_xml] = ACTIONS(1867),
    [sym_xml_to_variable] = ACTIONS(1867),
    [sym_length_variable_to_xml] = ACTIONS(1867),
    [sym_redirect_string] = ACTIONS(1867),
    [sym_random_number] = ACTIONS(1867),
    [sym_get_unique_id] = ACTIONS(1867),
    [sym_get_system_number] = ACTIONS(1867),
    [sym_set_system_number] = ACTIONS(1867),
    [sym_get_serial_number] = ACTIONS(1867),
    [sym_get_master_build] = ACTIONS(1867),
    [sym_set_validation_code] = ACTIONS(1867),
    [sym_reboot] = ACTIONS(1867),
    [sym_load_duet_module] = ACTIONS(1867),
    [sym_unload_duet_module] = ACTIONS(1867),
    [sym_duet_mem_size_set] = ACTIONS(1867),
    [sym_duet_mem_size_get] = ACTIONS(1867),
    [sym_dynamic_application_device] = ACTIONS(1867),
    [sym_dynamic_polled_port] = ACTIONS(1867),
    [sym_static_port_binding] = ACTIONS(1867),
    [sym_static_ip_binding] = ACTIONS(1867),
    [sym_get_dns_list] = ACTIONS(1867),
    [sym_set_dns_list] = ACTIONS(1867),
    [sym_get_ip_address] = ACTIONS(1867),
    [sym_set_ip_address] = ACTIONS(1867),
    [sym_get_url_list] = ACTIONS(1867),
    [sym_add_url_entry] = ACTIONS(1867),
    [sym_delete_url_entry] = ACTIONS(1867),
    [sym_abs_value] = ACTIONS(1867),
    [sym_max_value] = ACTIONS(1867),
    [sym_min_value] = ACTIONS(1867),
    [sym_exp_value] = ACTIONS(1867),
    [sym_log_value] = ACTIONS(1867),
    [sym_log10_value] = ACTIONS(1867),
    [sym_power_value] = ACTIONS(1867),
    [sym_sqrt_value] = ACTIONS(1867),
    [sym_raw_be] = ACTIONS(1867),
    [sym_raw_le] = ACTIONS(1867),
    [sym_internal_queue_size_set] = ACTIONS(1867),
    [sym_internal_queue_size_get] = ACTIONS(1867),
    [sym_internal_threshold_set] = ACTIONS(1867),
    [sym_internal_threshold_get] = ACTIONS(1867),
    [sym_rebuilt_event] = ACTIONS(1867),
    [sym_do_custom_event] = ACTIONS(1867),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1867),
    [sym_clkmgr_set_clk_source] = ACTIONS(1867),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1867),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1867),
    [sym_clkmgr_get_timezone] = ACTIONS(1867),
    [sym_clkmgr_set_timezone] = ACTIONS(1867),
    [sym_clkmgr_get_resync_period] = ACTIONS(1867),
    [sym_clkmgr_set_resync_period] = ACTIONS(1867),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1867),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1867),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1867),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1867),
    [sym_clkmgr_get_timeservers] = ACTIONS(1867),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1867),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1867),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1867),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1867),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1867),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1867),
    [sym_set_log_level] = ACTIONS(1867),
    [sym_get_log_level] = ACTIONS(1867),
    [sym_amx_log] = ACTIONS(1867),
    [sym_smtp_server_config_set] = ACTIONS(1867),
    [sym_smtp_server_config_get] = ACTIONS(1867),
    [sym_smtp_send] = ACTIONS(1867),
    [sym_validate_netlinx_account] = ACTIONS(1867),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1867),
    [sym_authenticate_certificate] = ACTIONS(1867),
    [sym_audit_netlinx_session_event] = ACTIONS(1867),
    [sym_audit_netlinx_generic_event] = ACTIONS(1867),
    [sym_get_available_flash_disk_space] = ACTIONS(1867),
    [sym_get_max_flash_disk_space] = ACTIONS(1867),
    [sym_data_create_feed] = ACTIONS(1867),
    [sym_data_delete_feed] = ACTIONS(1867),
    [sym_data_publish_feed] = ACTIONS(1867),
    [sym_data_get_published_feed] = ACTIONS(1867),
    [sym_data_add_record] = ACTIONS(1867),
    [sym_data_get_event_record] = ACTIONS(1867),
    [sym__wc_data_create_feed] = ACTIONS(1867),
    [sym__wc_data_add_record] = ACTIONS(1867),
    [sym__wc_data_get_event_record] = ACTIONS(1867),
    [sym_format] = ACTIONS(1867),
    [sym_astro_clock] = ACTIONS(1867),
    [sym_set_outdoor_temperature] = ACTIONS(1867),
    [sym_set_virtual_level_count] = ACTIONS(1867),
    [sym_set_virtual_channel_count] = ACTIONS(1867),
    [sym_set_virtual_port_count] = ACTIONS(1867),
    [sym__wc_encode_int] = ACTIONS(1867),
    [sym__wc_decode_int] = ACTIONS(1867),
    [sym__wc_int] = ACTIONS(1867),
    [sym__wc_to_ch_int] = ACTIONS(1867),
    [sym__ch_to_wc_int] = ACTIONS(1867),
    [sym__wc_find_string_int] = ACTIONS(1867),
    [sym__wc_left_string_int] = ACTIONS(1867),
    [sym__wc_lower_string_int] = ACTIONS(1867),
    [sym__wc_mid_string_int] = ACTIONS(1867),
    [sym__wc_remove_string_int] = ACTIONS(1867),
    [sym__wc_right_string_int] = ACTIONS(1867),
    [sym__wc_upper_string_int] = ACTIONS(1867),
    [sym__wc_compare_string_int] = ACTIONS(1867),
    [sym__wc_get_buffer_char_int] = ACTIONS(1867),
    [sym__wc_get_buffer_string_int] = ACTIONS(1867),
    [sym__wc_concat_string_int] = ACTIONS(1867),
    [sym__wc_file_open_int] = ACTIONS(1867),
    [sym__wc_file_close_int] = ACTIONS(1867),
    [sym__wc_file_read_int] = ACTIONS(1867),
    [sym__wc_file_read_line_int] = ACTIONS(1867),
    [sym__wc_file_write_int] = ACTIONS(1867),
    [sym__wc_file_write_line_int] = ACTIONS(1867),
    [aux_sym_preproc_include_token1] = ACTIONS(1867),
    [aux_sym_preproc_define_token1] = ACTIONS(1869),
    [aux_sym_preproc_warn_token1] = ACTIONS(1869),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1869),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1869),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1869),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1869),
    [aux_sym_preproc_else_token1] = ACTIONS(1869),
    [anon_sym_LBRACK] = ACTIONS(1869),
    [anon_sym_LBRACE] = ACTIONS(1869),
    [anon_sym_LPAREN] = ACTIONS(1867),
    [anon_sym_BANG] = ACTIONS(1869),
    [anon_sym_TILDE] = ACTIONS(1869),
    [anon_sym_DASH] = ACTIONS(1867),
    [anon_sym_PLUS] = ACTIONS(1867),
    [anon_sym_DASH_DASH] = ACTIONS(1869),
    [anon_sym_PLUS_PLUS] = ACTIONS(1869),
    [anon_sym_DQUOTE] = ACTIONS(1869),
    [anon_sym_SQUOTE] = ACTIONS(1869),
    [sym_number_literal] = ACTIONS(1869),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(263)] = {
    [sym_identifier] = ACTIONS(1871),
    [sym_program_name_keyword] = ACTIONS(1871),
    [sym_module_name_keyword] = ACTIONS(1871),
    [sym_define_device_keyword] = ACTIONS(1871),
    [sym_define_combine_keyword] = ACTIONS(1871),
    [sym_define_constant_keyword] = ACTIONS(1871),
    [sym_define_type_keyword] = ACTIONS(1871),
    [sym_define_variable_keyword] = ACTIONS(1871),
    [sym_define_system_variable_keyword] = ACTIONS(1871),
    [sym_define_connect_level_keyword] = ACTIONS(1871),
    [sym_define_latching_keyword] = ACTIONS(1871),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1871),
    [sym_define_toggling_keyword] = ACTIONS(1871),
    [sym_define_call_keyword] = ACTIONS(1871),
    [sym_define_function_keyword] = ACTIONS(1871),
    [sym_define_library_function_keyword] = ACTIONS(1871),
    [sym_define_start_keyword] = ACTIONS(1871),
    [sym_define_event_keyword] = ACTIONS(1871),
    [sym_define_program_keyword] = ACTIONS(1871),
    [sym_define_module_keyword] = ACTIONS(1871),
    [sym_char_keyword] = ACTIONS(1871),
    [sym_widechar_keyword] = ACTIONS(1871),
    [sym_integer_keyword] = ACTIONS(1871),
    [sym_sinteger_keyword] = ACTIONS(1871),
    [sym_long_keyword] = ACTIONS(1871),
    [sym_slong_keyword] = ACTIONS(1871),
    [sym_float_keyword] = ACTIONS(1871),
    [sym_double_keyword] = ACTIONS(1871),
    [sym_variant_keyword] = ACTIONS(1871),
    [sym_variantarray_keyword] = ACTIONS(1871),
    [sym_local_var_keyword] = ACTIONS(1871),
    [sym_stack_var_keyword] = ACTIONS(1871),
    [sym_constant_keyword] = ACTIONS(1871),
    [sym_volatile_keyword] = ACTIONS(1871),
    [sym_non_volatile_keyword] = ACTIONS(1871),
    [sym_persistent_keyword] = ACTIONS(1871),
    [sym_dev_keyword] = ACTIONS(1871),
    [sym_devlev_keyword] = ACTIONS(1871),
    [sym_devchan_keyword] = ACTIONS(1871),
    [sym_if_keyword] = ACTIONS(1871),
    [sym_else_keyword] = ACTIONS(1871),
    [sym_switch_keyword] = ACTIONS(1871),
    [sym_case_keyword] = ACTIONS(1871),
    [sym_default_keyword] = ACTIONS(1871),
    [sym_while_keyword] = ACTIONS(1871),
    [sym_for_keyword] = ACTIONS(1871),
    [sym_break_keyword] = ACTIONS(1871),
    [sym_continue_keyword] = ACTIONS(1871),
    [sym_return_keyword] = ACTIONS(1871),
    [sym_select_keyword] = ACTIONS(1871),
    [sym_active_keyword] = ACTIONS(1871),
    [sym_struct_keyword] = ACTIONS(1871),
    [sym_structure_keyword] = ACTIONS(1871),
    [sym_button_event_keyword] = ACTIONS(1871),
    [sym_channel_event_keyword] = ACTIONS(1871),
    [sym_level_event_keyword] = ACTIONS(1871),
    [sym_data_event_keyword] = ACTIONS(1871),
    [sym_timeline_event_keyword] = ACTIONS(1871),
    [sym_custom_event_keyword] = ACTIONS(1871),
    [sym_push_keyword] = ACTIONS(1871),
    [sym_release_keyword] = ACTIONS(1871),
    [aux_sym_on_keyword_token1] = ACTIONS(1871),
    [aux_sym_off_keyword_token1] = ACTIONS(1871),
    [sym_send_level_keyword] = ACTIONS(1871),
    [sym_send_string_keyword] = ACTIONS(1871),
    [sym_send_command_keyword] = ACTIONS(1871),
    [sym_clear_buffer_keyword] = ACTIONS(1871),
    [sym_create_buffer_keyword] = ACTIONS(1871),
    [sym_create_multi_buffer_keyword] = ACTIONS(1871),
    [sym_call_keyword] = ACTIONS(1871),
    [sym_system_call_keyword] = ACTIONS(1871),
    [sym_devchan_to_keyword] = ACTIONS(1871),
    [sym_devchan_min_to_keyword] = ACTIONS(1871),
    [sym_devchan_total_off_keyword] = ACTIONS(1871),
    [sym_devchan_pulse_keyword] = ACTIONS(1871),
    [sym_wait_keyword] = ACTIONS(1871),
    [sym_cancel_wait_keyword] = ACTIONS(1871),
    [sym_wait_until_keyword] = ACTIONS(1871),
    [sym_cancel_wait_until_keyword] = ACTIONS(1871),
    [sym_cancel_all_wait_keyword] = ACTIONS(1871),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1871),
    [sym_not] = ACTIONS(1871),
    [sym_bnot] = ACTIONS(1871),
    [sym___file__] = ACTIONS(1871),
    [sym___date__] = ACTIONS(1871),
    [sym___ldate__] = ACTIONS(1871),
    [sym___time__] = ACTIONS(1871),
    [sym___line__] = ACTIONS(1871),
    [sym___name__] = ACTIONS(1871),
    [sym___netlinx__] = ACTIONS(1871),
    [sym_netlinx_axi_version] = ACTIONS(1871),
    [sym_url_flg_tcp] = ACTIONS(1871),
    [sym_url_flg_accinfopresent] = ACTIONS(1871),
    [sym_url_flg_temp] = ACTIONS(1871),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1871),
    [sym_url_flg_stat_mask] = ACTIONS(1871),
    [sym_url_flg_stat_lookup] = ACTIONS(1871),
    [sym_url_flg_stat_connecting] = ACTIONS(1871),
    [sym_url_flg_stat_waiting] = ACTIONS(1871),
    [sym_url_flg_stat_connected] = ACTIONS(1871),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1871),
    [sym_first_virtual_device] = ACTIONS(1871),
    [sym_first_local_port] = ACTIONS(1871),
    [sym_dynamic_virtual_device] = ACTIONS(1871),
    [sym_timeline_once] = ACTIONS(1871),
    [sym_timeline_repeat] = ACTIONS(1871),
    [sym_timeline_absolute] = ACTIONS(1871),
    [sym_timeline_relative] = ACTIONS(1871),
    [sym_do_push_timed_infinite] = ACTIONS(1871),
    [sym_source_type_no_address] = ACTIONS(1871),
    [sym_source_type_neuron_id] = ACTIONS(1871),
    [sym_source_type_ip_address] = ACTIONS(1871),
    [sym_source_type_axlink] = ACTIONS(1871),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1871),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1871),
    [sym_source_type_ip_socket_address] = ACTIONS(1871),
    [sym_source_type_rs232] = ACTIONS(1871),
    [sym_source_type_internal] = ACTIONS(1871),
    [sym_source_type_mac_address] = ACTIONS(1871),
    [sym_source_type_ipv4_port] = ACTIONS(1871),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1871),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1871),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1871),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1871),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1871),
    [sym_true] = ACTIONS(1871),
    [sym_false] = ACTIONS(1871),
    [sym_file_read_only] = ACTIONS(1871),
    [sym_file_rw_new] = ACTIONS(1871),
    [sym_file_rw_append] = ACTIONS(1871),
    [sym_ip_tcp] = ACTIONS(1871),
    [sym_ip_udp] = ACTIONS(1871),
    [sym_ip_udp_2way] = ACTIONS(1871),
    [sym_xml_encode_types] = ACTIONS(1871),
    [sym_xml_encode_char_as_list] = ACTIONS(1871),
    [sym_xml_encode_le] = ACTIONS(1871),
    [sym_xml_decode_types] = ACTIONS(1871),
    [sym_xml_decode_no_preserve] = ACTIONS(1871),
    [sym_duet_dev_type_display_device] = ACTIONS(1871),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1871),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1871),
    [sym_duet_dev_type_io_device] = ACTIONS(1871),
    [sym_duet_dev_type_relay_device] = ACTIONS(1871),
    [sym_duet_dev_type_ups] = ACTIONS(1871),
    [sym_duet_dev_type_amplifier] = ACTIONS(1871),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1871),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1871),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1871),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1871),
    [sym_duet_dev_type_camera] = ACTIONS(1871),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1871),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1871),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1871),
    [sym_duet_dev_type_dss] = ACTIONS(1871),
    [sym_duet_dev_type_dvr] = ACTIONS(1871),
    [sym_duet_dev_type_disc_device] = ACTIONS(1871),
    [sym_duet_dev_type_document_camera] = ACTIONS(1871),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1871),
    [sym_duet_dev_type_hvac] = ACTIONS(1871),
    [sym_duet_dev_type_keypad] = ACTIONS(1871),
    [sym_duet_dev_type_light] = ACTIONS(1871),
    [sym_duet_dev_type_monitor] = ACTIONS(1871),
    [sym_duet_dev_type_motor] = ACTIONS(1871),
    [sym_duet_dev_type_multi_window] = ACTIONS(1871),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1871),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1871),
    [sym_duet_dev_type_receiver] = ACTIONS(1871),
    [sym_duet_dev_type_security_system] = ACTIONS(1871),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1871),
    [sym_duet_dev_type_settop_box] = ACTIONS(1871),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1871),
    [sym_duet_dev_type_switcher] = ACTIONS(1871),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1871),
    [sym_duet_dev_type_tv] = ACTIONS(1871),
    [sym_duet_dev_type_utility] = ACTIONS(1871),
    [sym_duet_dev_type_vcr] = ACTIONS(1871),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1871),
    [sym_duet_dev_type_video_processor] = ACTIONS(1871),
    [sym_duet_dev_type_video_projector] = ACTIONS(1871),
    [sym_duet_dev_type_video_wall] = ACTIONS(1871),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1871),
    [sym_duet_dev_type_weather] = ACTIONS(1871),
    [sym_duet_dev_not_polled] = ACTIONS(1871),
    [sym_duet_dev_polled] = ACTIONS(1871),
    [sym_clkmgr_mode_network] = ACTIONS(1871),
    [sym_clkmgr_mode_standalone] = ACTIONS(1871),
    [sym_normal_standby] = ACTIONS(1871),
    [sym_normal_wake] = ACTIONS(1871),
    [sym_ip_multicast_ttl_option] = ACTIONS(1871),
    [sym_ip_ttl_subnet] = ACTIONS(1871),
    [sym_ip_ttl_site] = ACTIONS(1871),
    [sym_ip_ttl_region] = ACTIONS(1871),
    [sym_ip_ttl_continent] = ACTIONS(1871),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1871),
    [sym_ip_nodelay_on] = ACTIONS(1871),
    [sym_ip_nodelay_off] = ACTIONS(1871),
    [sym_amx_error] = ACTIONS(1871),
    [sym_amx_warning] = ACTIONS(1871),
    [sym_amx_info] = ACTIONS(1871),
    [sym_amx_debug] = ACTIONS(1871),
    [sym_netlinx_login_success] = ACTIONS(1871),
    [sym_netlinx_login_fail] = ACTIONS(1871),
    [sym_netlinx_logout] = ACTIONS(1871),
    [sym_null_str] = ACTIONS(1871),
    [sym_smtp_address] = ACTIONS(1871),
    [sym_smtp_port_number] = ACTIONS(1871),
    [sym_smtp_username] = ACTIONS(1871),
    [sym_smtp_password] = ACTIONS(1871),
    [sym_smtp_require_tls] = ACTIONS(1871),
    [sym_smtp_from] = ACTIONS(1871),
    [sym_smtp_tls_true] = ACTIONS(1871),
    [sym_smtp_tls_false] = ACTIONS(1871),
    [sym_listview_on_row_select_event] = ACTIONS(1871),
    [sym_data_structure_unknown] = ACTIONS(1871),
    [sym_data_structure_datafeed] = ACTIONS(1871),
    [sym_data_structure_datarecordset] = ACTIONS(1871),
    [sym_data_structure_datarecord] = ACTIONS(1871),
    [sym_data_structure_datafield] = ACTIONS(1871),
    [sym_data_max_content_field_count] = ACTIONS(1871),
    [sym_data_max_metadata_field_count] = ACTIONS(1871),
    [sym_data_max_id_length] = ACTIONS(1871),
    [sym_data_max_name_length] = ACTIONS(1871),
    [sym_data_max_description_length] = ACTIONS(1871),
    [sym_data_max_type_length] = ACTIONS(1871),
    [sym_data_max_format_length] = ACTIONS(1871),
    [sym_data_max_source_length] = ACTIONS(1871),
    [sym_data_max_value_length] = ACTIONS(1871),
    [sym_data_max_label_length] = ACTIONS(1871),
    [sym_data_max_url_length] = ACTIONS(1871),
    [sym_max_last_login_info_length] = ACTIONS(1871),
    [sym_data_type_unknown] = ACTIONS(1871),
    [sym_data_type_string] = ACTIONS(1871),
    [sym_data_type_datetime] = ACTIONS(1871),
    [sym_data_type_date] = ACTIONS(1871),
    [sym_data_type_time] = ACTIONS(1871),
    [sym_data_type_image] = ACTIONS(1871),
    [sym_data_format_url] = ACTIONS(1871),
    [sym_data_format_phone] = ACTIONS(1871),
    [sym_data_format_email] = ACTIONS(1871),
    [sym_data_format_iso8601] = ACTIONS(1871),
    [sym_auditlog_priv] = ACTIONS(1871),
    [sym_configuration_priv] = ACTIONS(1871),
    [sym_deviceconfig_priv] = ACTIONS(1871),
    [sym_ftp_priv] = ACTIONS(1871),
    [sym_http_priv] = ACTIONS(1871),
    [sym_networkconfig_priv] = ACTIONS(1871),
    [sym_programport_priv] = ACTIONS(1871),
    [sym_remoteui_priv] = ACTIONS(1871),
    [sym_securitycontrol_priv] = ACTIONS(1871),
    [sym_softwaremanagement_priv] = ACTIONS(1871),
    [sym_terminal_priv] = ACTIONS(1871),
    [sym_tpadmin_priv] = ACTIONS(1871),
    [sym_user1_priv] = ACTIONS(1871),
    [sym_user2_priv] = ACTIONS(1871),
    [sym_user3_priv] = ACTIONS(1871),
    [sym_user4_priv] = ACTIONS(1871),
    [sym_usermanagement_priv] = ACTIONS(1871),
    [sym_valid_account] = ACTIONS(1871),
    [sym_err_name_invalid] = ACTIONS(1871),
    [sym_err_password_invalid] = ACTIONS(1871),
    [sym_err_invalid_account] = ACTIONS(1871),
    [sym_err_no_authorization] = ACTIONS(1871),
    [sym_err_invalid_parameter] = ACTIONS(1871),
    [sym_err_user_locked_out] = ACTIONS(1871),
    [sym_err_user_account_expired] = ACTIONS(1871),
    [sym_err_general_error] = ACTIONS(1871),
    [sym_err_authentication_error] = ACTIONS(1871),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1871),
    [sym_tls_validate_certificate] = ACTIONS(1871),
    [sym_tdata] = ACTIONS(1871),
    [sym_tchannel] = ACTIONS(1871),
    [sym_tlevel] = ACTIONS(1871),
    [sym_tbutton] = ACTIONS(1871),
    [sym_ttimeline] = ACTIONS(1871),
    [sym_tcustom] = ACTIONS(1871),
    [sym_url_struct] = ACTIONS(1871),
    [sym_dns_struct] = ACTIONS(1871),
    [sym_ip_address_struct] = ACTIONS(1871),
    [sym_dev_info_struct] = ACTIONS(1871),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1871),
    [sym_data_feed] = ACTIONS(1871),
    [sym_data_field] = ACTIONS(1871),
    [sym_data_record] = ACTIONS(1871),
    [sym_wc_data_feed] = ACTIONS(1871),
    [sym_wc_data_field] = ACTIONS(1871),
    [sym_wc_data_record] = ACTIONS(1871),
    [sym_last_login_info] = ACTIONS(1871),
    [sym____reserved___] = ACTIONS(1871),
    [sym_date] = ACTIONS(1871),
    [sym_day] = ACTIONS(1871),
    [sym_time] = ACTIONS(1871),
    [sym_ldate] = ACTIONS(1871),
    [sym_push_device] = ACTIONS(1871),
    [sym_push_channel] = ACTIONS(1871),
    [sym_push_devchan] = ACTIONS(1871),
    [sym_release_device] = ACTIONS(1871),
    [sym_release_channel] = ACTIONS(1871),
    [sym_release_devchan] = ACTIONS(1871),
    [sym_master_slot] = ACTIONS(1871),
    [sym_get_pulse_time] = ACTIONS(1871),
    [sym_get_timer] = ACTIONS(1871),
    [sym_master_sn] = ACTIONS(1871),
    [sym_system_number] = ACTIONS(1871),
    [sym_button] = ACTIONS(1871),
    [sym_dv_channel] = ACTIONS(1871),
    [sym_data] = ACTIONS(1871),
    [sym_level] = ACTIONS(1871),
    [sym_timeline] = ACTIONS(1871),
    [sym_channel] = ACTIONS(1871),
    [sym_custom] = ACTIONS(1871),
    [sym_atoi] = ACTIONS(1871),
    [sym_atol] = ACTIONS(1871),
    [sym_atof] = ACTIONS(1871),
    [sym_itoa] = ACTIONS(1871),
    [sym_itohex] = ACTIONS(1871),
    [sym_hextoi] = ACTIONS(1871),
    [sym_ftoa] = ACTIONS(1871),
    [sym_find_string] = ACTIONS(1871),
    [sym_left_string] = ACTIONS(1871),
    [sym_right_string] = ACTIONS(1871),
    [sym_mid_string] = ACTIONS(1871),
    [sym_remove_string] = ACTIONS(1871),
    [sym_get_buffer_string] = ACTIONS(1871),
    [sym_get_buffer_char] = ACTIONS(1871),
    [sym_get_multi_buffer_string] = ACTIONS(1871),
    [sym_length_string] = ACTIONS(1871),
    [sym_max_length_string] = ACTIONS(1871),
    [sym_set_length_string] = ACTIONS(1871),
    [sym_compare_string] = ACTIONS(1871),
    [sym_upper_string] = ACTIONS(1871),
    [sym_lower_string] = ACTIONS(1871),
    [sym_get_last] = ACTIONS(1871),
    [sym_date_to_day] = ACTIONS(1871),
    [sym_date_to_month] = ACTIONS(1871),
    [sym_date_to_year] = ACTIONS(1871),
    [sym_time_to_hour] = ACTIONS(1871),
    [sym_time_to_minute] = ACTIONS(1871),
    [sym_time_to_second] = ACTIONS(1871),
    [sym_day_of_week] = ACTIONS(1871),
    [sym_set_pulse_time] = ACTIONS(1871),
    [sym_set_timer] = ACTIONS(1871),
    [sym_file_open] = ACTIONS(1871),
    [sym_file_close] = ACTIONS(1871),
    [sym_file_read] = ACTIONS(1871),
    [sym_file_read_line] = ACTIONS(1871),
    [sym_file_write] = ACTIONS(1871),
    [sym_file_write_line] = ACTIONS(1871),
    [sym_file_copy] = ACTIONS(1871),
    [sym_file_delete] = ACTIONS(1871),
    [sym_file_rename] = ACTIONS(1871),
    [sym_file_dir] = ACTIONS(1871),
    [sym_file_getdir] = ACTIONS(1871),
    [sym_file_setdir] = ACTIONS(1871),
    [sym_file_create_dir] = ACTIONS(1871),
    [sym_file_remove_dir] = ACTIONS(1871),
    [sym_file_seek] = ACTIONS(1871),
    [sym_length_array] = ACTIONS(1871),
    [sym_max_length_array] = ACTIONS(1871),
    [sym_set_length_array] = ACTIONS(1871),
    [sym_ip_client_open] = ACTIONS(1871),
    [sym_ip_client_close] = ACTIONS(1871),
    [sym_ip_server_open] = ACTIONS(1871),
    [sym_ip_server_close] = ACTIONS(1871),
    [sym_ip_bound_client_open] = ACTIONS(1871),
    [sym_ip_mc_server_open] = ACTIONS(1871),
    [sym_ip_set_option] = ACTIONS(1871),
    [sym_ssh_client_open] = ACTIONS(1871),
    [sym_ssh_client_close] = ACTIONS(1871),
    [sym_tls_client_open] = ACTIONS(1871),
    [sym_tls_client_close] = ACTIONS(1871),
    [sym_do_push] = ACTIONS(1871),
    [sym_do_push_timed] = ACTIONS(1871),
    [sym_do_release] = ACTIONS(1871),
    [sym_device_id] = ACTIONS(1871),
    [sym_device_info] = ACTIONS(1871),
    [sym_device_id_string] = ACTIONS(1871),
    [sym_device_standby] = ACTIONS(1871),
    [sym_device_wake] = ACTIONS(1871),
    [sym_timeline_create] = ACTIONS(1871),
    [sym_timeline_kill] = ACTIONS(1871),
    [sym_timeline_reload] = ACTIONS(1871),
    [sym_timeline_pause] = ACTIONS(1871),
    [sym_timeline_restart] = ACTIONS(1871),
    [sym_timeline_set] = ACTIONS(1871),
    [sym_timeline_get] = ACTIONS(1871),
    [sym_timeline_active] = ACTIONS(1871),
    [sym_timeline_dynamic_id] = ACTIONS(1871),
    [sym_variable_to_string] = ACTIONS(1871),
    [sym_string_to_variable] = ACTIONS(1871),
    [sym_length_variable_to_string] = ACTIONS(1871),
    [sym_type_cast] = ACTIONS(1871),
    [sym_variable_to_xml] = ACTIONS(1871),
    [sym_xml_to_variable] = ACTIONS(1871),
    [sym_length_variable_to_xml] = ACTIONS(1871),
    [sym_redirect_string] = ACTIONS(1871),
    [sym_random_number] = ACTIONS(1871),
    [sym_get_unique_id] = ACTIONS(1871),
    [sym_get_system_number] = ACTIONS(1871),
    [sym_set_system_number] = ACTIONS(1871),
    [sym_get_serial_number] = ACTIONS(1871),
    [sym_get_master_build] = ACTIONS(1871),
    [sym_set_validation_code] = ACTIONS(1871),
    [sym_reboot] = ACTIONS(1871),
    [sym_load_duet_module] = ACTIONS(1871),
    [sym_unload_duet_module] = ACTIONS(1871),
    [sym_duet_mem_size_set] = ACTIONS(1871),
    [sym_duet_mem_size_get] = ACTIONS(1871),
    [sym_dynamic_application_device] = ACTIONS(1871),
    [sym_dynamic_polled_port] = ACTIONS(1871),
    [sym_static_port_binding] = ACTIONS(1871),
    [sym_static_ip_binding] = ACTIONS(1871),
    [sym_get_dns_list] = ACTIONS(1871),
    [sym_set_dns_list] = ACTIONS(1871),
    [sym_get_ip_address] = ACTIONS(1871),
    [sym_set_ip_address] = ACTIONS(1871),
    [sym_get_url_list] = ACTIONS(1871),
    [sym_add_url_entry] = ACTIONS(1871),
    [sym_delete_url_entry] = ACTIONS(1871),
    [sym_abs_value] = ACTIONS(1871),
    [sym_max_value] = ACTIONS(1871),
    [sym_min_value] = ACTIONS(1871),
    [sym_exp_value] = ACTIONS(1871),
    [sym_log_value] = ACTIONS(1871),
    [sym_log10_value] = ACTIONS(1871),
    [sym_power_value] = ACTIONS(1871),
    [sym_sqrt_value] = ACTIONS(1871),
    [sym_raw_be] = ACTIONS(1871),
    [sym_raw_le] = ACTIONS(1871),
    [sym_internal_queue_size_set] = ACTIONS(1871),
    [sym_internal_queue_size_get] = ACTIONS(1871),
    [sym_internal_threshold_set] = ACTIONS(1871),
    [sym_internal_threshold_get] = ACTIONS(1871),
    [sym_rebuilt_event] = ACTIONS(1871),
    [sym_do_custom_event] = ACTIONS(1871),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1871),
    [sym_clkmgr_set_clk_source] = ACTIONS(1871),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1871),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1871),
    [sym_clkmgr_get_timezone] = ACTIONS(1871),
    [sym_clkmgr_set_timezone] = ACTIONS(1871),
    [sym_clkmgr_get_resync_period] = ACTIONS(1871),
    [sym_clkmgr_set_resync_period] = ACTIONS(1871),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1871),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1871),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1871),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1871),
    [sym_clkmgr_get_timeservers] = ACTIONS(1871),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1871),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1871),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1871),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1871),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1871),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1871),
    [sym_set_log_level] = ACTIONS(1871),
    [sym_get_log_level] = ACTIONS(1871),
    [sym_amx_log] = ACTIONS(1871),
    [sym_smtp_server_config_set] = ACTIONS(1871),
    [sym_smtp_server_config_get] = ACTIONS(1871),
    [sym_smtp_send] = ACTIONS(1871),
    [sym_validate_netlinx_account] = ACTIONS(1871),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1871),
    [sym_authenticate_certificate] = ACTIONS(1871),
    [sym_audit_netlinx_session_event] = ACTIONS(1871),
    [sym_audit_netlinx_generic_event] = ACTIONS(1871),
    [sym_get_available_flash_disk_space] = ACTIONS(1871),
    [sym_get_max_flash_disk_space] = ACTIONS(1871),
    [sym_data_create_feed] = ACTIONS(1871),
    [sym_data_delete_feed] = ACTIONS(1871),
    [sym_data_publish_feed] = ACTIONS(1871),
    [sym_data_get_published_feed] = ACTIONS(1871),
    [sym_data_add_record] = ACTIONS(1871),
    [sym_data_get_event_record] = ACTIONS(1871),
    [sym__wc_data_create_feed] = ACTIONS(1871),
    [sym__wc_data_add_record] = ACTIONS(1871),
    [sym__wc_data_get_event_record] = ACTIONS(1871),
    [sym_format] = ACTIONS(1871),
    [sym_astro_clock] = ACTIONS(1871),
    [sym_set_outdoor_temperature] = ACTIONS(1871),
    [sym_set_virtual_level_count] = ACTIONS(1871),
    [sym_set_virtual_channel_count] = ACTIONS(1871),
    [sym_set_virtual_port_count] = ACTIONS(1871),
    [sym__wc_encode_int] = ACTIONS(1871),
    [sym__wc_decode_int] = ACTIONS(1871),
    [sym__wc_int] = ACTIONS(1871),
    [sym__wc_to_ch_int] = ACTIONS(1871),
    [sym__ch_to_wc_int] = ACTIONS(1871),
    [sym__wc_find_string_int] = ACTIONS(1871),
    [sym__wc_left_string_int] = ACTIONS(1871),
    [sym__wc_lower_string_int] = ACTIONS(1871),
    [sym__wc_mid_string_int] = ACTIONS(1871),
    [sym__wc_remove_string_int] = ACTIONS(1871),
    [sym__wc_right_string_int] = ACTIONS(1871),
    [sym__wc_upper_string_int] = ACTIONS(1871),
    [sym__wc_compare_string_int] = ACTIONS(1871),
    [sym__wc_get_buffer_char_int] = ACTIONS(1871),
    [sym__wc_get_buffer_string_int] = ACTIONS(1871),
    [sym__wc_concat_string_int] = ACTIONS(1871),
    [sym__wc_file_open_int] = ACTIONS(1871),
    [sym__wc_file_close_int] = ACTIONS(1871),
    [sym__wc_file_read_int] = ACTIONS(1871),
    [sym__wc_file_read_line_int] = ACTIONS(1871),
    [sym__wc_file_write_int] = ACTIONS(1871),
    [sym__wc_file_write_line_int] = ACTIONS(1871),
    [aux_sym_preproc_include_token1] = ACTIONS(1871),
    [aux_sym_preproc_define_token1] = ACTIONS(1873),
    [aux_sym_preproc_warn_token1] = ACTIONS(1873),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1873),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1873),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1873),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1873),
    [aux_sym_preproc_else_token1] = ACTIONS(1873),
    [anon_sym_LBRACK] = ACTIONS(1873),
    [anon_sym_LBRACE] = ACTIONS(1873),
    [anon_sym_LPAREN] = ACTIONS(1871),
    [anon_sym_BANG] = ACTIONS(1873),
    [anon_sym_TILDE] = ACTIONS(1873),
    [anon_sym_DASH] = ACTIONS(1871),
    [anon_sym_PLUS] = ACTIONS(1871),
    [anon_sym_DASH_DASH] = ACTIONS(1873),
    [anon_sym_PLUS_PLUS] = ACTIONS(1873),
    [anon_sym_DQUOTE] = ACTIONS(1873),
    [anon_sym_SQUOTE] = ACTIONS(1873),
    [sym_number_literal] = ACTIONS(1873),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(264)] = {
    [sym_identifier] = ACTIONS(1875),
    [sym_program_name_keyword] = ACTIONS(1875),
    [sym_module_name_keyword] = ACTIONS(1875),
    [sym_define_device_keyword] = ACTIONS(1875),
    [sym_define_combine_keyword] = ACTIONS(1875),
    [sym_define_constant_keyword] = ACTIONS(1875),
    [sym_define_type_keyword] = ACTIONS(1875),
    [sym_define_variable_keyword] = ACTIONS(1875),
    [sym_define_system_variable_keyword] = ACTIONS(1875),
    [sym_define_connect_level_keyword] = ACTIONS(1875),
    [sym_define_latching_keyword] = ACTIONS(1875),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1875),
    [sym_define_toggling_keyword] = ACTIONS(1875),
    [sym_define_call_keyword] = ACTIONS(1875),
    [sym_define_function_keyword] = ACTIONS(1875),
    [sym_define_library_function_keyword] = ACTIONS(1875),
    [sym_define_start_keyword] = ACTIONS(1875),
    [sym_define_event_keyword] = ACTIONS(1875),
    [sym_define_program_keyword] = ACTIONS(1875),
    [sym_define_module_keyword] = ACTIONS(1875),
    [sym_char_keyword] = ACTIONS(1875),
    [sym_widechar_keyword] = ACTIONS(1875),
    [sym_integer_keyword] = ACTIONS(1875),
    [sym_sinteger_keyword] = ACTIONS(1875),
    [sym_long_keyword] = ACTIONS(1875),
    [sym_slong_keyword] = ACTIONS(1875),
    [sym_float_keyword] = ACTIONS(1875),
    [sym_double_keyword] = ACTIONS(1875),
    [sym_variant_keyword] = ACTIONS(1875),
    [sym_variantarray_keyword] = ACTIONS(1875),
    [sym_local_var_keyword] = ACTIONS(1875),
    [sym_stack_var_keyword] = ACTIONS(1875),
    [sym_constant_keyword] = ACTIONS(1875),
    [sym_volatile_keyword] = ACTIONS(1875),
    [sym_non_volatile_keyword] = ACTIONS(1875),
    [sym_persistent_keyword] = ACTIONS(1875),
    [sym_dev_keyword] = ACTIONS(1875),
    [sym_devlev_keyword] = ACTIONS(1875),
    [sym_devchan_keyword] = ACTIONS(1875),
    [sym_if_keyword] = ACTIONS(1875),
    [sym_else_keyword] = ACTIONS(1875),
    [sym_switch_keyword] = ACTIONS(1875),
    [sym_case_keyword] = ACTIONS(1875),
    [sym_default_keyword] = ACTIONS(1875),
    [sym_while_keyword] = ACTIONS(1875),
    [sym_for_keyword] = ACTIONS(1875),
    [sym_break_keyword] = ACTIONS(1875),
    [sym_continue_keyword] = ACTIONS(1875),
    [sym_return_keyword] = ACTIONS(1875),
    [sym_select_keyword] = ACTIONS(1875),
    [sym_active_keyword] = ACTIONS(1875),
    [sym_struct_keyword] = ACTIONS(1875),
    [sym_structure_keyword] = ACTIONS(1875),
    [sym_button_event_keyword] = ACTIONS(1875),
    [sym_channel_event_keyword] = ACTIONS(1875),
    [sym_level_event_keyword] = ACTIONS(1875),
    [sym_data_event_keyword] = ACTIONS(1875),
    [sym_timeline_event_keyword] = ACTIONS(1875),
    [sym_custom_event_keyword] = ACTIONS(1875),
    [sym_push_keyword] = ACTIONS(1875),
    [sym_release_keyword] = ACTIONS(1875),
    [aux_sym_on_keyword_token1] = ACTIONS(1875),
    [aux_sym_off_keyword_token1] = ACTIONS(1875),
    [sym_send_level_keyword] = ACTIONS(1875),
    [sym_send_string_keyword] = ACTIONS(1875),
    [sym_send_command_keyword] = ACTIONS(1875),
    [sym_clear_buffer_keyword] = ACTIONS(1875),
    [sym_create_buffer_keyword] = ACTIONS(1875),
    [sym_create_multi_buffer_keyword] = ACTIONS(1875),
    [sym_call_keyword] = ACTIONS(1875),
    [sym_system_call_keyword] = ACTIONS(1875),
    [sym_devchan_to_keyword] = ACTIONS(1875),
    [sym_devchan_min_to_keyword] = ACTIONS(1875),
    [sym_devchan_total_off_keyword] = ACTIONS(1875),
    [sym_devchan_pulse_keyword] = ACTIONS(1875),
    [sym_wait_keyword] = ACTIONS(1875),
    [sym_cancel_wait_keyword] = ACTIONS(1875),
    [sym_wait_until_keyword] = ACTIONS(1875),
    [sym_cancel_wait_until_keyword] = ACTIONS(1875),
    [sym_cancel_all_wait_keyword] = ACTIONS(1875),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1875),
    [sym_not] = ACTIONS(1875),
    [sym_bnot] = ACTIONS(1875),
    [sym___file__] = ACTIONS(1875),
    [sym___date__] = ACTIONS(1875),
    [sym___ldate__] = ACTIONS(1875),
    [sym___time__] = ACTIONS(1875),
    [sym___line__] = ACTIONS(1875),
    [sym___name__] = ACTIONS(1875),
    [sym___netlinx__] = ACTIONS(1875),
    [sym_netlinx_axi_version] = ACTIONS(1875),
    [sym_url_flg_tcp] = ACTIONS(1875),
    [sym_url_flg_accinfopresent] = ACTIONS(1875),
    [sym_url_flg_temp] = ACTIONS(1875),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1875),
    [sym_url_flg_stat_mask] = ACTIONS(1875),
    [sym_url_flg_stat_lookup] = ACTIONS(1875),
    [sym_url_flg_stat_connecting] = ACTIONS(1875),
    [sym_url_flg_stat_waiting] = ACTIONS(1875),
    [sym_url_flg_stat_connected] = ACTIONS(1875),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1875),
    [sym_first_virtual_device] = ACTIONS(1875),
    [sym_first_local_port] = ACTIONS(1875),
    [sym_dynamic_virtual_device] = ACTIONS(1875),
    [sym_timeline_once] = ACTIONS(1875),
    [sym_timeline_repeat] = ACTIONS(1875),
    [sym_timeline_absolute] = ACTIONS(1875),
    [sym_timeline_relative] = ACTIONS(1875),
    [sym_do_push_timed_infinite] = ACTIONS(1875),
    [sym_source_type_no_address] = ACTIONS(1875),
    [sym_source_type_neuron_id] = ACTIONS(1875),
    [sym_source_type_ip_address] = ACTIONS(1875),
    [sym_source_type_axlink] = ACTIONS(1875),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1875),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1875),
    [sym_source_type_ip_socket_address] = ACTIONS(1875),
    [sym_source_type_rs232] = ACTIONS(1875),
    [sym_source_type_internal] = ACTIONS(1875),
    [sym_source_type_mac_address] = ACTIONS(1875),
    [sym_source_type_ipv4_port] = ACTIONS(1875),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1875),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1875),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1875),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1875),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1875),
    [sym_true] = ACTIONS(1875),
    [sym_false] = ACTIONS(1875),
    [sym_file_read_only] = ACTIONS(1875),
    [sym_file_rw_new] = ACTIONS(1875),
    [sym_file_rw_append] = ACTIONS(1875),
    [sym_ip_tcp] = ACTIONS(1875),
    [sym_ip_udp] = ACTIONS(1875),
    [sym_ip_udp_2way] = ACTIONS(1875),
    [sym_xml_encode_types] = ACTIONS(1875),
    [sym_xml_encode_char_as_list] = ACTIONS(1875),
    [sym_xml_encode_le] = ACTIONS(1875),
    [sym_xml_decode_types] = ACTIONS(1875),
    [sym_xml_decode_no_preserve] = ACTIONS(1875),
    [sym_duet_dev_type_display_device] = ACTIONS(1875),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1875),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1875),
    [sym_duet_dev_type_io_device] = ACTIONS(1875),
    [sym_duet_dev_type_relay_device] = ACTIONS(1875),
    [sym_duet_dev_type_ups] = ACTIONS(1875),
    [sym_duet_dev_type_amplifier] = ACTIONS(1875),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1875),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1875),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1875),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1875),
    [sym_duet_dev_type_camera] = ACTIONS(1875),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1875),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1875),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1875),
    [sym_duet_dev_type_dss] = ACTIONS(1875),
    [sym_duet_dev_type_dvr] = ACTIONS(1875),
    [sym_duet_dev_type_disc_device] = ACTIONS(1875),
    [sym_duet_dev_type_document_camera] = ACTIONS(1875),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1875),
    [sym_duet_dev_type_hvac] = ACTIONS(1875),
    [sym_duet_dev_type_keypad] = ACTIONS(1875),
    [sym_duet_dev_type_light] = ACTIONS(1875),
    [sym_duet_dev_type_monitor] = ACTIONS(1875),
    [sym_duet_dev_type_motor] = ACTIONS(1875),
    [sym_duet_dev_type_multi_window] = ACTIONS(1875),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1875),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1875),
    [sym_duet_dev_type_receiver] = ACTIONS(1875),
    [sym_duet_dev_type_security_system] = ACTIONS(1875),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1875),
    [sym_duet_dev_type_settop_box] = ACTIONS(1875),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1875),
    [sym_duet_dev_type_switcher] = ACTIONS(1875),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1875),
    [sym_duet_dev_type_tv] = ACTIONS(1875),
    [sym_duet_dev_type_utility] = ACTIONS(1875),
    [sym_duet_dev_type_vcr] = ACTIONS(1875),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1875),
    [sym_duet_dev_type_video_processor] = ACTIONS(1875),
    [sym_duet_dev_type_video_projector] = ACTIONS(1875),
    [sym_duet_dev_type_video_wall] = ACTIONS(1875),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1875),
    [sym_duet_dev_type_weather] = ACTIONS(1875),
    [sym_duet_dev_not_polled] = ACTIONS(1875),
    [sym_duet_dev_polled] = ACTIONS(1875),
    [sym_clkmgr_mode_network] = ACTIONS(1875),
    [sym_clkmgr_mode_standalone] = ACTIONS(1875),
    [sym_normal_standby] = ACTIONS(1875),
    [sym_normal_wake] = ACTIONS(1875),
    [sym_ip_multicast_ttl_option] = ACTIONS(1875),
    [sym_ip_ttl_subnet] = ACTIONS(1875),
    [sym_ip_ttl_site] = ACTIONS(1875),
    [sym_ip_ttl_region] = ACTIONS(1875),
    [sym_ip_ttl_continent] = ACTIONS(1875),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1875),
    [sym_ip_nodelay_on] = ACTIONS(1875),
    [sym_ip_nodelay_off] = ACTIONS(1875),
    [sym_amx_error] = ACTIONS(1875),
    [sym_amx_warning] = ACTIONS(1875),
    [sym_amx_info] = ACTIONS(1875),
    [sym_amx_debug] = ACTIONS(1875),
    [sym_netlinx_login_success] = ACTIONS(1875),
    [sym_netlinx_login_fail] = ACTIONS(1875),
    [sym_netlinx_logout] = ACTIONS(1875),
    [sym_null_str] = ACTIONS(1875),
    [sym_smtp_address] = ACTIONS(1875),
    [sym_smtp_port_number] = ACTIONS(1875),
    [sym_smtp_username] = ACTIONS(1875),
    [sym_smtp_password] = ACTIONS(1875),
    [sym_smtp_require_tls] = ACTIONS(1875),
    [sym_smtp_from] = ACTIONS(1875),
    [sym_smtp_tls_true] = ACTIONS(1875),
    [sym_smtp_tls_false] = ACTIONS(1875),
    [sym_listview_on_row_select_event] = ACTIONS(1875),
    [sym_data_structure_unknown] = ACTIONS(1875),
    [sym_data_structure_datafeed] = ACTIONS(1875),
    [sym_data_structure_datarecordset] = ACTIONS(1875),
    [sym_data_structure_datarecord] = ACTIONS(1875),
    [sym_data_structure_datafield] = ACTIONS(1875),
    [sym_data_max_content_field_count] = ACTIONS(1875),
    [sym_data_max_metadata_field_count] = ACTIONS(1875),
    [sym_data_max_id_length] = ACTIONS(1875),
    [sym_data_max_name_length] = ACTIONS(1875),
    [sym_data_max_description_length] = ACTIONS(1875),
    [sym_data_max_type_length] = ACTIONS(1875),
    [sym_data_max_format_length] = ACTIONS(1875),
    [sym_data_max_source_length] = ACTIONS(1875),
    [sym_data_max_value_length] = ACTIONS(1875),
    [sym_data_max_label_length] = ACTIONS(1875),
    [sym_data_max_url_length] = ACTIONS(1875),
    [sym_max_last_login_info_length] = ACTIONS(1875),
    [sym_data_type_unknown] = ACTIONS(1875),
    [sym_data_type_string] = ACTIONS(1875),
    [sym_data_type_datetime] = ACTIONS(1875),
    [sym_data_type_date] = ACTIONS(1875),
    [sym_data_type_time] = ACTIONS(1875),
    [sym_data_type_image] = ACTIONS(1875),
    [sym_data_format_url] = ACTIONS(1875),
    [sym_data_format_phone] = ACTIONS(1875),
    [sym_data_format_email] = ACTIONS(1875),
    [sym_data_format_iso8601] = ACTIONS(1875),
    [sym_auditlog_priv] = ACTIONS(1875),
    [sym_configuration_priv] = ACTIONS(1875),
    [sym_deviceconfig_priv] = ACTIONS(1875),
    [sym_ftp_priv] = ACTIONS(1875),
    [sym_http_priv] = ACTIONS(1875),
    [sym_networkconfig_priv] = ACTIONS(1875),
    [sym_programport_priv] = ACTIONS(1875),
    [sym_remoteui_priv] = ACTIONS(1875),
    [sym_securitycontrol_priv] = ACTIONS(1875),
    [sym_softwaremanagement_priv] = ACTIONS(1875),
    [sym_terminal_priv] = ACTIONS(1875),
    [sym_tpadmin_priv] = ACTIONS(1875),
    [sym_user1_priv] = ACTIONS(1875),
    [sym_user2_priv] = ACTIONS(1875),
    [sym_user3_priv] = ACTIONS(1875),
    [sym_user4_priv] = ACTIONS(1875),
    [sym_usermanagement_priv] = ACTIONS(1875),
    [sym_valid_account] = ACTIONS(1875),
    [sym_err_name_invalid] = ACTIONS(1875),
    [sym_err_password_invalid] = ACTIONS(1875),
    [sym_err_invalid_account] = ACTIONS(1875),
    [sym_err_no_authorization] = ACTIONS(1875),
    [sym_err_invalid_parameter] = ACTIONS(1875),
    [sym_err_user_locked_out] = ACTIONS(1875),
    [sym_err_user_account_expired] = ACTIONS(1875),
    [sym_err_general_error] = ACTIONS(1875),
    [sym_err_authentication_error] = ACTIONS(1875),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1875),
    [sym_tls_validate_certificate] = ACTIONS(1875),
    [sym_tdata] = ACTIONS(1875),
    [sym_tchannel] = ACTIONS(1875),
    [sym_tlevel] = ACTIONS(1875),
    [sym_tbutton] = ACTIONS(1875),
    [sym_ttimeline] = ACTIONS(1875),
    [sym_tcustom] = ACTIONS(1875),
    [sym_url_struct] = ACTIONS(1875),
    [sym_dns_struct] = ACTIONS(1875),
    [sym_ip_address_struct] = ACTIONS(1875),
    [sym_dev_info_struct] = ACTIONS(1875),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1875),
    [sym_data_feed] = ACTIONS(1875),
    [sym_data_field] = ACTIONS(1875),
    [sym_data_record] = ACTIONS(1875),
    [sym_wc_data_feed] = ACTIONS(1875),
    [sym_wc_data_field] = ACTIONS(1875),
    [sym_wc_data_record] = ACTIONS(1875),
    [sym_last_login_info] = ACTIONS(1875),
    [sym____reserved___] = ACTIONS(1875),
    [sym_date] = ACTIONS(1875),
    [sym_day] = ACTIONS(1875),
    [sym_time] = ACTIONS(1875),
    [sym_ldate] = ACTIONS(1875),
    [sym_push_device] = ACTIONS(1875),
    [sym_push_channel] = ACTIONS(1875),
    [sym_push_devchan] = ACTIONS(1875),
    [sym_release_device] = ACTIONS(1875),
    [sym_release_channel] = ACTIONS(1875),
    [sym_release_devchan] = ACTIONS(1875),
    [sym_master_slot] = ACTIONS(1875),
    [sym_get_pulse_time] = ACTIONS(1875),
    [sym_get_timer] = ACTIONS(1875),
    [sym_master_sn] = ACTIONS(1875),
    [sym_system_number] = ACTIONS(1875),
    [sym_button] = ACTIONS(1875),
    [sym_dv_channel] = ACTIONS(1875),
    [sym_data] = ACTIONS(1875),
    [sym_level] = ACTIONS(1875),
    [sym_timeline] = ACTIONS(1875),
    [sym_channel] = ACTIONS(1875),
    [sym_custom] = ACTIONS(1875),
    [sym_atoi] = ACTIONS(1875),
    [sym_atol] = ACTIONS(1875),
    [sym_atof] = ACTIONS(1875),
    [sym_itoa] = ACTIONS(1875),
    [sym_itohex] = ACTIONS(1875),
    [sym_hextoi] = ACTIONS(1875),
    [sym_ftoa] = ACTIONS(1875),
    [sym_find_string] = ACTIONS(1875),
    [sym_left_string] = ACTIONS(1875),
    [sym_right_string] = ACTIONS(1875),
    [sym_mid_string] = ACTIONS(1875),
    [sym_remove_string] = ACTIONS(1875),
    [sym_get_buffer_string] = ACTIONS(1875),
    [sym_get_buffer_char] = ACTIONS(1875),
    [sym_get_multi_buffer_string] = ACTIONS(1875),
    [sym_length_string] = ACTIONS(1875),
    [sym_max_length_string] = ACTIONS(1875),
    [sym_set_length_string] = ACTIONS(1875),
    [sym_compare_string] = ACTIONS(1875),
    [sym_upper_string] = ACTIONS(1875),
    [sym_lower_string] = ACTIONS(1875),
    [sym_get_last] = ACTIONS(1875),
    [sym_date_to_day] = ACTIONS(1875),
    [sym_date_to_month] = ACTIONS(1875),
    [sym_date_to_year] = ACTIONS(1875),
    [sym_time_to_hour] = ACTIONS(1875),
    [sym_time_to_minute] = ACTIONS(1875),
    [sym_time_to_second] = ACTIONS(1875),
    [sym_day_of_week] = ACTIONS(1875),
    [sym_set_pulse_time] = ACTIONS(1875),
    [sym_set_timer] = ACTIONS(1875),
    [sym_file_open] = ACTIONS(1875),
    [sym_file_close] = ACTIONS(1875),
    [sym_file_read] = ACTIONS(1875),
    [sym_file_read_line] = ACTIONS(1875),
    [sym_file_write] = ACTIONS(1875),
    [sym_file_write_line] = ACTIONS(1875),
    [sym_file_copy] = ACTIONS(1875),
    [sym_file_delete] = ACTIONS(1875),
    [sym_file_rename] = ACTIONS(1875),
    [sym_file_dir] = ACTIONS(1875),
    [sym_file_getdir] = ACTIONS(1875),
    [sym_file_setdir] = ACTIONS(1875),
    [sym_file_create_dir] = ACTIONS(1875),
    [sym_file_remove_dir] = ACTIONS(1875),
    [sym_file_seek] = ACTIONS(1875),
    [sym_length_array] = ACTIONS(1875),
    [sym_max_length_array] = ACTIONS(1875),
    [sym_set_length_array] = ACTIONS(1875),
    [sym_ip_client_open] = ACTIONS(1875),
    [sym_ip_client_close] = ACTIONS(1875),
    [sym_ip_server_open] = ACTIONS(1875),
    [sym_ip_server_close] = ACTIONS(1875),
    [sym_ip_bound_client_open] = ACTIONS(1875),
    [sym_ip_mc_server_open] = ACTIONS(1875),
    [sym_ip_set_option] = ACTIONS(1875),
    [sym_ssh_client_open] = ACTIONS(1875),
    [sym_ssh_client_close] = ACTIONS(1875),
    [sym_tls_client_open] = ACTIONS(1875),
    [sym_tls_client_close] = ACTIONS(1875),
    [sym_do_push] = ACTIONS(1875),
    [sym_do_push_timed] = ACTIONS(1875),
    [sym_do_release] = ACTIONS(1875),
    [sym_device_id] = ACTIONS(1875),
    [sym_device_info] = ACTIONS(1875),
    [sym_device_id_string] = ACTIONS(1875),
    [sym_device_standby] = ACTIONS(1875),
    [sym_device_wake] = ACTIONS(1875),
    [sym_timeline_create] = ACTIONS(1875),
    [sym_timeline_kill] = ACTIONS(1875),
    [sym_timeline_reload] = ACTIONS(1875),
    [sym_timeline_pause] = ACTIONS(1875),
    [sym_timeline_restart] = ACTIONS(1875),
    [sym_timeline_set] = ACTIONS(1875),
    [sym_timeline_get] = ACTIONS(1875),
    [sym_timeline_active] = ACTIONS(1875),
    [sym_timeline_dynamic_id] = ACTIONS(1875),
    [sym_variable_to_string] = ACTIONS(1875),
    [sym_string_to_variable] = ACTIONS(1875),
    [sym_length_variable_to_string] = ACTIONS(1875),
    [sym_type_cast] = ACTIONS(1875),
    [sym_variable_to_xml] = ACTIONS(1875),
    [sym_xml_to_variable] = ACTIONS(1875),
    [sym_length_variable_to_xml] = ACTIONS(1875),
    [sym_redirect_string] = ACTIONS(1875),
    [sym_random_number] = ACTIONS(1875),
    [sym_get_unique_id] = ACTIONS(1875),
    [sym_get_system_number] = ACTIONS(1875),
    [sym_set_system_number] = ACTIONS(1875),
    [sym_get_serial_number] = ACTIONS(1875),
    [sym_get_master_build] = ACTIONS(1875),
    [sym_set_validation_code] = ACTIONS(1875),
    [sym_reboot] = ACTIONS(1875),
    [sym_load_duet_module] = ACTIONS(1875),
    [sym_unload_duet_module] = ACTIONS(1875),
    [sym_duet_mem_size_set] = ACTIONS(1875),
    [sym_duet_mem_size_get] = ACTIONS(1875),
    [sym_dynamic_application_device] = ACTIONS(1875),
    [sym_dynamic_polled_port] = ACTIONS(1875),
    [sym_static_port_binding] = ACTIONS(1875),
    [sym_static_ip_binding] = ACTIONS(1875),
    [sym_get_dns_list] = ACTIONS(1875),
    [sym_set_dns_list] = ACTIONS(1875),
    [sym_get_ip_address] = ACTIONS(1875),
    [sym_set_ip_address] = ACTIONS(1875),
    [sym_get_url_list] = ACTIONS(1875),
    [sym_add_url_entry] = ACTIONS(1875),
    [sym_delete_url_entry] = ACTIONS(1875),
    [sym_abs_value] = ACTIONS(1875),
    [sym_max_value] = ACTIONS(1875),
    [sym_min_value] = ACTIONS(1875),
    [sym_exp_value] = ACTIONS(1875),
    [sym_log_value] = ACTIONS(1875),
    [sym_log10_value] = ACTIONS(1875),
    [sym_power_value] = ACTIONS(1875),
    [sym_sqrt_value] = ACTIONS(1875),
    [sym_raw_be] = ACTIONS(1875),
    [sym_raw_le] = ACTIONS(1875),
    [sym_internal_queue_size_set] = ACTIONS(1875),
    [sym_internal_queue_size_get] = ACTIONS(1875),
    [sym_internal_threshold_set] = ACTIONS(1875),
    [sym_internal_threshold_get] = ACTIONS(1875),
    [sym_rebuilt_event] = ACTIONS(1875),
    [sym_do_custom_event] = ACTIONS(1875),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1875),
    [sym_clkmgr_set_clk_source] = ACTIONS(1875),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1875),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1875),
    [sym_clkmgr_get_timezone] = ACTIONS(1875),
    [sym_clkmgr_set_timezone] = ACTIONS(1875),
    [sym_clkmgr_get_resync_period] = ACTIONS(1875),
    [sym_clkmgr_set_resync_period] = ACTIONS(1875),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1875),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1875),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1875),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1875),
    [sym_clkmgr_get_timeservers] = ACTIONS(1875),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1875),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1875),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1875),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1875),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1875),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1875),
    [sym_set_log_level] = ACTIONS(1875),
    [sym_get_log_level] = ACTIONS(1875),
    [sym_amx_log] = ACTIONS(1875),
    [sym_smtp_server_config_set] = ACTIONS(1875),
    [sym_smtp_server_config_get] = ACTIONS(1875),
    [sym_smtp_send] = ACTIONS(1875),
    [sym_validate_netlinx_account] = ACTIONS(1875),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1875),
    [sym_authenticate_certificate] = ACTIONS(1875),
    [sym_audit_netlinx_session_event] = ACTIONS(1875),
    [sym_audit_netlinx_generic_event] = ACTIONS(1875),
    [sym_get_available_flash_disk_space] = ACTIONS(1875),
    [sym_get_max_flash_disk_space] = ACTIONS(1875),
    [sym_data_create_feed] = ACTIONS(1875),
    [sym_data_delete_feed] = ACTIONS(1875),
    [sym_data_publish_feed] = ACTIONS(1875),
    [sym_data_get_published_feed] = ACTIONS(1875),
    [sym_data_add_record] = ACTIONS(1875),
    [sym_data_get_event_record] = ACTIONS(1875),
    [sym__wc_data_create_feed] = ACTIONS(1875),
    [sym__wc_data_add_record] = ACTIONS(1875),
    [sym__wc_data_get_event_record] = ACTIONS(1875),
    [sym_format] = ACTIONS(1875),
    [sym_astro_clock] = ACTIONS(1875),
    [sym_set_outdoor_temperature] = ACTIONS(1875),
    [sym_set_virtual_level_count] = ACTIONS(1875),
    [sym_set_virtual_channel_count] = ACTIONS(1875),
    [sym_set_virtual_port_count] = ACTIONS(1875),
    [sym__wc_encode_int] = ACTIONS(1875),
    [sym__wc_decode_int] = ACTIONS(1875),
    [sym__wc_int] = ACTIONS(1875),
    [sym__wc_to_ch_int] = ACTIONS(1875),
    [sym__ch_to_wc_int] = ACTIONS(1875),
    [sym__wc_find_string_int] = ACTIONS(1875),
    [sym__wc_left_string_int] = ACTIONS(1875),
    [sym__wc_lower_string_int] = ACTIONS(1875),
    [sym__wc_mid_string_int] = ACTIONS(1875),
    [sym__wc_remove_string_int] = ACTIONS(1875),
    [sym__wc_right_string_int] = ACTIONS(1875),
    [sym__wc_upper_string_int] = ACTIONS(1875),
    [sym__wc_compare_string_int] = ACTIONS(1875),
    [sym__wc_get_buffer_char_int] = ACTIONS(1875),
    [sym__wc_get_buffer_string_int] = ACTIONS(1875),
    [sym__wc_concat_string_int] = ACTIONS(1875),
    [sym__wc_file_open_int] = ACTIONS(1875),
    [sym__wc_file_close_int] = ACTIONS(1875),
    [sym__wc_file_read_int] = ACTIONS(1875),
    [sym__wc_file_read_line_int] = ACTIONS(1875),
    [sym__wc_file_write_int] = ACTIONS(1875),
    [sym__wc_file_write_line_int] = ACTIONS(1875),
    [aux_sym_preproc_include_token1] = ACTIONS(1875),
    [aux_sym_preproc_define_token1] = ACTIONS(1877),
    [aux_sym_preproc_warn_token1] = ACTIONS(1877),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1877),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1877),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1877),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1877),
    [aux_sym_preproc_else_token1] = ACTIONS(1877),
    [anon_sym_LBRACK] = ACTIONS(1877),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_LPAREN] = ACTIONS(1875),
    [anon_sym_BANG] = ACTIONS(1877),
    [anon_sym_TILDE] = ACTIONS(1877),
    [anon_sym_DASH] = ACTIONS(1875),
    [anon_sym_PLUS] = ACTIONS(1875),
    [anon_sym_DASH_DASH] = ACTIONS(1877),
    [anon_sym_PLUS_PLUS] = ACTIONS(1877),
    [anon_sym_DQUOTE] = ACTIONS(1877),
    [anon_sym_SQUOTE] = ACTIONS(1877),
    [sym_number_literal] = ACTIONS(1877),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(265)] = {
    [sym_identifier] = ACTIONS(1879),
    [sym_program_name_keyword] = ACTIONS(1879),
    [sym_module_name_keyword] = ACTIONS(1879),
    [sym_define_device_keyword] = ACTIONS(1879),
    [sym_define_combine_keyword] = ACTIONS(1879),
    [sym_define_constant_keyword] = ACTIONS(1879),
    [sym_define_type_keyword] = ACTIONS(1879),
    [sym_define_variable_keyword] = ACTIONS(1879),
    [sym_define_system_variable_keyword] = ACTIONS(1879),
    [sym_define_connect_level_keyword] = ACTIONS(1879),
    [sym_define_latching_keyword] = ACTIONS(1879),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1879),
    [sym_define_toggling_keyword] = ACTIONS(1879),
    [sym_define_call_keyword] = ACTIONS(1879),
    [sym_define_function_keyword] = ACTIONS(1879),
    [sym_define_library_function_keyword] = ACTIONS(1879),
    [sym_define_start_keyword] = ACTIONS(1879),
    [sym_define_event_keyword] = ACTIONS(1879),
    [sym_define_program_keyword] = ACTIONS(1879),
    [sym_define_module_keyword] = ACTIONS(1879),
    [sym_char_keyword] = ACTIONS(1879),
    [sym_widechar_keyword] = ACTIONS(1879),
    [sym_integer_keyword] = ACTIONS(1879),
    [sym_sinteger_keyword] = ACTIONS(1879),
    [sym_long_keyword] = ACTIONS(1879),
    [sym_slong_keyword] = ACTIONS(1879),
    [sym_float_keyword] = ACTIONS(1879),
    [sym_double_keyword] = ACTIONS(1879),
    [sym_variant_keyword] = ACTIONS(1879),
    [sym_variantarray_keyword] = ACTIONS(1879),
    [sym_local_var_keyword] = ACTIONS(1879),
    [sym_stack_var_keyword] = ACTIONS(1879),
    [sym_constant_keyword] = ACTIONS(1879),
    [sym_volatile_keyword] = ACTIONS(1879),
    [sym_non_volatile_keyword] = ACTIONS(1879),
    [sym_persistent_keyword] = ACTIONS(1879),
    [sym_dev_keyword] = ACTIONS(1879),
    [sym_devlev_keyword] = ACTIONS(1879),
    [sym_devchan_keyword] = ACTIONS(1879),
    [sym_if_keyword] = ACTIONS(1879),
    [sym_else_keyword] = ACTIONS(1879),
    [sym_switch_keyword] = ACTIONS(1879),
    [sym_case_keyword] = ACTIONS(1879),
    [sym_default_keyword] = ACTIONS(1879),
    [sym_while_keyword] = ACTIONS(1879),
    [sym_for_keyword] = ACTIONS(1879),
    [sym_break_keyword] = ACTIONS(1879),
    [sym_continue_keyword] = ACTIONS(1879),
    [sym_return_keyword] = ACTIONS(1879),
    [sym_select_keyword] = ACTIONS(1879),
    [sym_active_keyword] = ACTIONS(1879),
    [sym_struct_keyword] = ACTIONS(1879),
    [sym_structure_keyword] = ACTIONS(1879),
    [sym_button_event_keyword] = ACTIONS(1879),
    [sym_channel_event_keyword] = ACTIONS(1879),
    [sym_level_event_keyword] = ACTIONS(1879),
    [sym_data_event_keyword] = ACTIONS(1879),
    [sym_timeline_event_keyword] = ACTIONS(1879),
    [sym_custom_event_keyword] = ACTIONS(1879),
    [sym_push_keyword] = ACTIONS(1879),
    [sym_release_keyword] = ACTIONS(1879),
    [aux_sym_on_keyword_token1] = ACTIONS(1879),
    [aux_sym_off_keyword_token1] = ACTIONS(1879),
    [sym_send_level_keyword] = ACTIONS(1879),
    [sym_send_string_keyword] = ACTIONS(1879),
    [sym_send_command_keyword] = ACTIONS(1879),
    [sym_clear_buffer_keyword] = ACTIONS(1879),
    [sym_create_buffer_keyword] = ACTIONS(1879),
    [sym_create_multi_buffer_keyword] = ACTIONS(1879),
    [sym_call_keyword] = ACTIONS(1879),
    [sym_system_call_keyword] = ACTIONS(1879),
    [sym_devchan_to_keyword] = ACTIONS(1879),
    [sym_devchan_min_to_keyword] = ACTIONS(1879),
    [sym_devchan_total_off_keyword] = ACTIONS(1879),
    [sym_devchan_pulse_keyword] = ACTIONS(1879),
    [sym_wait_keyword] = ACTIONS(1879),
    [sym_cancel_wait_keyword] = ACTIONS(1879),
    [sym_wait_until_keyword] = ACTIONS(1879),
    [sym_cancel_wait_until_keyword] = ACTIONS(1879),
    [sym_cancel_all_wait_keyword] = ACTIONS(1879),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1879),
    [sym_not] = ACTIONS(1879),
    [sym_bnot] = ACTIONS(1879),
    [sym___file__] = ACTIONS(1879),
    [sym___date__] = ACTIONS(1879),
    [sym___ldate__] = ACTIONS(1879),
    [sym___time__] = ACTIONS(1879),
    [sym___line__] = ACTIONS(1879),
    [sym___name__] = ACTIONS(1879),
    [sym___netlinx__] = ACTIONS(1879),
    [sym_netlinx_axi_version] = ACTIONS(1879),
    [sym_url_flg_tcp] = ACTIONS(1879),
    [sym_url_flg_accinfopresent] = ACTIONS(1879),
    [sym_url_flg_temp] = ACTIONS(1879),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1879),
    [sym_url_flg_stat_mask] = ACTIONS(1879),
    [sym_url_flg_stat_lookup] = ACTIONS(1879),
    [sym_url_flg_stat_connecting] = ACTIONS(1879),
    [sym_url_flg_stat_waiting] = ACTIONS(1879),
    [sym_url_flg_stat_connected] = ACTIONS(1879),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1879),
    [sym_first_virtual_device] = ACTIONS(1879),
    [sym_first_local_port] = ACTIONS(1879),
    [sym_dynamic_virtual_device] = ACTIONS(1879),
    [sym_timeline_once] = ACTIONS(1879),
    [sym_timeline_repeat] = ACTIONS(1879),
    [sym_timeline_absolute] = ACTIONS(1879),
    [sym_timeline_relative] = ACTIONS(1879),
    [sym_do_push_timed_infinite] = ACTIONS(1879),
    [sym_source_type_no_address] = ACTIONS(1879),
    [sym_source_type_neuron_id] = ACTIONS(1879),
    [sym_source_type_ip_address] = ACTIONS(1879),
    [sym_source_type_axlink] = ACTIONS(1879),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1879),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1879),
    [sym_source_type_ip_socket_address] = ACTIONS(1879),
    [sym_source_type_rs232] = ACTIONS(1879),
    [sym_source_type_internal] = ACTIONS(1879),
    [sym_source_type_mac_address] = ACTIONS(1879),
    [sym_source_type_ipv4_port] = ACTIONS(1879),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1879),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1879),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1879),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1879),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1879),
    [sym_true] = ACTIONS(1879),
    [sym_false] = ACTIONS(1879),
    [sym_file_read_only] = ACTIONS(1879),
    [sym_file_rw_new] = ACTIONS(1879),
    [sym_file_rw_append] = ACTIONS(1879),
    [sym_ip_tcp] = ACTIONS(1879),
    [sym_ip_udp] = ACTIONS(1879),
    [sym_ip_udp_2way] = ACTIONS(1879),
    [sym_xml_encode_types] = ACTIONS(1879),
    [sym_xml_encode_char_as_list] = ACTIONS(1879),
    [sym_xml_encode_le] = ACTIONS(1879),
    [sym_xml_decode_types] = ACTIONS(1879),
    [sym_xml_decode_no_preserve] = ACTIONS(1879),
    [sym_duet_dev_type_display_device] = ACTIONS(1879),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1879),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1879),
    [sym_duet_dev_type_io_device] = ACTIONS(1879),
    [sym_duet_dev_type_relay_device] = ACTIONS(1879),
    [sym_duet_dev_type_ups] = ACTIONS(1879),
    [sym_duet_dev_type_amplifier] = ACTIONS(1879),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1879),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1879),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1879),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1879),
    [sym_duet_dev_type_camera] = ACTIONS(1879),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1879),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1879),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1879),
    [sym_duet_dev_type_dss] = ACTIONS(1879),
    [sym_duet_dev_type_dvr] = ACTIONS(1879),
    [sym_duet_dev_type_disc_device] = ACTIONS(1879),
    [sym_duet_dev_type_document_camera] = ACTIONS(1879),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1879),
    [sym_duet_dev_type_hvac] = ACTIONS(1879),
    [sym_duet_dev_type_keypad] = ACTIONS(1879),
    [sym_duet_dev_type_light] = ACTIONS(1879),
    [sym_duet_dev_type_monitor] = ACTIONS(1879),
    [sym_duet_dev_type_motor] = ACTIONS(1879),
    [sym_duet_dev_type_multi_window] = ACTIONS(1879),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1879),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1879),
    [sym_duet_dev_type_receiver] = ACTIONS(1879),
    [sym_duet_dev_type_security_system] = ACTIONS(1879),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1879),
    [sym_duet_dev_type_settop_box] = ACTIONS(1879),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1879),
    [sym_duet_dev_type_switcher] = ACTIONS(1879),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1879),
    [sym_duet_dev_type_tv] = ACTIONS(1879),
    [sym_duet_dev_type_utility] = ACTIONS(1879),
    [sym_duet_dev_type_vcr] = ACTIONS(1879),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1879),
    [sym_duet_dev_type_video_processor] = ACTIONS(1879),
    [sym_duet_dev_type_video_projector] = ACTIONS(1879),
    [sym_duet_dev_type_video_wall] = ACTIONS(1879),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1879),
    [sym_duet_dev_type_weather] = ACTIONS(1879),
    [sym_duet_dev_not_polled] = ACTIONS(1879),
    [sym_duet_dev_polled] = ACTIONS(1879),
    [sym_clkmgr_mode_network] = ACTIONS(1879),
    [sym_clkmgr_mode_standalone] = ACTIONS(1879),
    [sym_normal_standby] = ACTIONS(1879),
    [sym_normal_wake] = ACTIONS(1879),
    [sym_ip_multicast_ttl_option] = ACTIONS(1879),
    [sym_ip_ttl_subnet] = ACTIONS(1879),
    [sym_ip_ttl_site] = ACTIONS(1879),
    [sym_ip_ttl_region] = ACTIONS(1879),
    [sym_ip_ttl_continent] = ACTIONS(1879),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1879),
    [sym_ip_nodelay_on] = ACTIONS(1879),
    [sym_ip_nodelay_off] = ACTIONS(1879),
    [sym_amx_error] = ACTIONS(1879),
    [sym_amx_warning] = ACTIONS(1879),
    [sym_amx_info] = ACTIONS(1879),
    [sym_amx_debug] = ACTIONS(1879),
    [sym_netlinx_login_success] = ACTIONS(1879),
    [sym_netlinx_login_fail] = ACTIONS(1879),
    [sym_netlinx_logout] = ACTIONS(1879),
    [sym_null_str] = ACTIONS(1879),
    [sym_smtp_address] = ACTIONS(1879),
    [sym_smtp_port_number] = ACTIONS(1879),
    [sym_smtp_username] = ACTIONS(1879),
    [sym_smtp_password] = ACTIONS(1879),
    [sym_smtp_require_tls] = ACTIONS(1879),
    [sym_smtp_from] = ACTIONS(1879),
    [sym_smtp_tls_true] = ACTIONS(1879),
    [sym_smtp_tls_false] = ACTIONS(1879),
    [sym_listview_on_row_select_event] = ACTIONS(1879),
    [sym_data_structure_unknown] = ACTIONS(1879),
    [sym_data_structure_datafeed] = ACTIONS(1879),
    [sym_data_structure_datarecordset] = ACTIONS(1879),
    [sym_data_structure_datarecord] = ACTIONS(1879),
    [sym_data_structure_datafield] = ACTIONS(1879),
    [sym_data_max_content_field_count] = ACTIONS(1879),
    [sym_data_max_metadata_field_count] = ACTIONS(1879),
    [sym_data_max_id_length] = ACTIONS(1879),
    [sym_data_max_name_length] = ACTIONS(1879),
    [sym_data_max_description_length] = ACTIONS(1879),
    [sym_data_max_type_length] = ACTIONS(1879),
    [sym_data_max_format_length] = ACTIONS(1879),
    [sym_data_max_source_length] = ACTIONS(1879),
    [sym_data_max_value_length] = ACTIONS(1879),
    [sym_data_max_label_length] = ACTIONS(1879),
    [sym_data_max_url_length] = ACTIONS(1879),
    [sym_max_last_login_info_length] = ACTIONS(1879),
    [sym_data_type_unknown] = ACTIONS(1879),
    [sym_data_type_string] = ACTIONS(1879),
    [sym_data_type_datetime] = ACTIONS(1879),
    [sym_data_type_date] = ACTIONS(1879),
    [sym_data_type_time] = ACTIONS(1879),
    [sym_data_type_image] = ACTIONS(1879),
    [sym_data_format_url] = ACTIONS(1879),
    [sym_data_format_phone] = ACTIONS(1879),
    [sym_data_format_email] = ACTIONS(1879),
    [sym_data_format_iso8601] = ACTIONS(1879),
    [sym_auditlog_priv] = ACTIONS(1879),
    [sym_configuration_priv] = ACTIONS(1879),
    [sym_deviceconfig_priv] = ACTIONS(1879),
    [sym_ftp_priv] = ACTIONS(1879),
    [sym_http_priv] = ACTIONS(1879),
    [sym_networkconfig_priv] = ACTIONS(1879),
    [sym_programport_priv] = ACTIONS(1879),
    [sym_remoteui_priv] = ACTIONS(1879),
    [sym_securitycontrol_priv] = ACTIONS(1879),
    [sym_softwaremanagement_priv] = ACTIONS(1879),
    [sym_terminal_priv] = ACTIONS(1879),
    [sym_tpadmin_priv] = ACTIONS(1879),
    [sym_user1_priv] = ACTIONS(1879),
    [sym_user2_priv] = ACTIONS(1879),
    [sym_user3_priv] = ACTIONS(1879),
    [sym_user4_priv] = ACTIONS(1879),
    [sym_usermanagement_priv] = ACTIONS(1879),
    [sym_valid_account] = ACTIONS(1879),
    [sym_err_name_invalid] = ACTIONS(1879),
    [sym_err_password_invalid] = ACTIONS(1879),
    [sym_err_invalid_account] = ACTIONS(1879),
    [sym_err_no_authorization] = ACTIONS(1879),
    [sym_err_invalid_parameter] = ACTIONS(1879),
    [sym_err_user_locked_out] = ACTIONS(1879),
    [sym_err_user_account_expired] = ACTIONS(1879),
    [sym_err_general_error] = ACTIONS(1879),
    [sym_err_authentication_error] = ACTIONS(1879),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1879),
    [sym_tls_validate_certificate] = ACTIONS(1879),
    [sym_tdata] = ACTIONS(1879),
    [sym_tchannel] = ACTIONS(1879),
    [sym_tlevel] = ACTIONS(1879),
    [sym_tbutton] = ACTIONS(1879),
    [sym_ttimeline] = ACTIONS(1879),
    [sym_tcustom] = ACTIONS(1879),
    [sym_url_struct] = ACTIONS(1879),
    [sym_dns_struct] = ACTIONS(1879),
    [sym_ip_address_struct] = ACTIONS(1879),
    [sym_dev_info_struct] = ACTIONS(1879),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1879),
    [sym_data_feed] = ACTIONS(1879),
    [sym_data_field] = ACTIONS(1879),
    [sym_data_record] = ACTIONS(1879),
    [sym_wc_data_feed] = ACTIONS(1879),
    [sym_wc_data_field] = ACTIONS(1879),
    [sym_wc_data_record] = ACTIONS(1879),
    [sym_last_login_info] = ACTIONS(1879),
    [sym____reserved___] = ACTIONS(1879),
    [sym_date] = ACTIONS(1879),
    [sym_day] = ACTIONS(1879),
    [sym_time] = ACTIONS(1879),
    [sym_ldate] = ACTIONS(1879),
    [sym_push_device] = ACTIONS(1879),
    [sym_push_channel] = ACTIONS(1879),
    [sym_push_devchan] = ACTIONS(1879),
    [sym_release_device] = ACTIONS(1879),
    [sym_release_channel] = ACTIONS(1879),
    [sym_release_devchan] = ACTIONS(1879),
    [sym_master_slot] = ACTIONS(1879),
    [sym_get_pulse_time] = ACTIONS(1879),
    [sym_get_timer] = ACTIONS(1879),
    [sym_master_sn] = ACTIONS(1879),
    [sym_system_number] = ACTIONS(1879),
    [sym_button] = ACTIONS(1879),
    [sym_dv_channel] = ACTIONS(1879),
    [sym_data] = ACTIONS(1879),
    [sym_level] = ACTIONS(1879),
    [sym_timeline] = ACTIONS(1879),
    [sym_channel] = ACTIONS(1879),
    [sym_custom] = ACTIONS(1879),
    [sym_atoi] = ACTIONS(1879),
    [sym_atol] = ACTIONS(1879),
    [sym_atof] = ACTIONS(1879),
    [sym_itoa] = ACTIONS(1879),
    [sym_itohex] = ACTIONS(1879),
    [sym_hextoi] = ACTIONS(1879),
    [sym_ftoa] = ACTIONS(1879),
    [sym_find_string] = ACTIONS(1879),
    [sym_left_string] = ACTIONS(1879),
    [sym_right_string] = ACTIONS(1879),
    [sym_mid_string] = ACTIONS(1879),
    [sym_remove_string] = ACTIONS(1879),
    [sym_get_buffer_string] = ACTIONS(1879),
    [sym_get_buffer_char] = ACTIONS(1879),
    [sym_get_multi_buffer_string] = ACTIONS(1879),
    [sym_length_string] = ACTIONS(1879),
    [sym_max_length_string] = ACTIONS(1879),
    [sym_set_length_string] = ACTIONS(1879),
    [sym_compare_string] = ACTIONS(1879),
    [sym_upper_string] = ACTIONS(1879),
    [sym_lower_string] = ACTIONS(1879),
    [sym_get_last] = ACTIONS(1879),
    [sym_date_to_day] = ACTIONS(1879),
    [sym_date_to_month] = ACTIONS(1879),
    [sym_date_to_year] = ACTIONS(1879),
    [sym_time_to_hour] = ACTIONS(1879),
    [sym_time_to_minute] = ACTIONS(1879),
    [sym_time_to_second] = ACTIONS(1879),
    [sym_day_of_week] = ACTIONS(1879),
    [sym_set_pulse_time] = ACTIONS(1879),
    [sym_set_timer] = ACTIONS(1879),
    [sym_file_open] = ACTIONS(1879),
    [sym_file_close] = ACTIONS(1879),
    [sym_file_read] = ACTIONS(1879),
    [sym_file_read_line] = ACTIONS(1879),
    [sym_file_write] = ACTIONS(1879),
    [sym_file_write_line] = ACTIONS(1879),
    [sym_file_copy] = ACTIONS(1879),
    [sym_file_delete] = ACTIONS(1879),
    [sym_file_rename] = ACTIONS(1879),
    [sym_file_dir] = ACTIONS(1879),
    [sym_file_getdir] = ACTIONS(1879),
    [sym_file_setdir] = ACTIONS(1879),
    [sym_file_create_dir] = ACTIONS(1879),
    [sym_file_remove_dir] = ACTIONS(1879),
    [sym_file_seek] = ACTIONS(1879),
    [sym_length_array] = ACTIONS(1879),
    [sym_max_length_array] = ACTIONS(1879),
    [sym_set_length_array] = ACTIONS(1879),
    [sym_ip_client_open] = ACTIONS(1879),
    [sym_ip_client_close] = ACTIONS(1879),
    [sym_ip_server_open] = ACTIONS(1879),
    [sym_ip_server_close] = ACTIONS(1879),
    [sym_ip_bound_client_open] = ACTIONS(1879),
    [sym_ip_mc_server_open] = ACTIONS(1879),
    [sym_ip_set_option] = ACTIONS(1879),
    [sym_ssh_client_open] = ACTIONS(1879),
    [sym_ssh_client_close] = ACTIONS(1879),
    [sym_tls_client_open] = ACTIONS(1879),
    [sym_tls_client_close] = ACTIONS(1879),
    [sym_do_push] = ACTIONS(1879),
    [sym_do_push_timed] = ACTIONS(1879),
    [sym_do_release] = ACTIONS(1879),
    [sym_device_id] = ACTIONS(1879),
    [sym_device_info] = ACTIONS(1879),
    [sym_device_id_string] = ACTIONS(1879),
    [sym_device_standby] = ACTIONS(1879),
    [sym_device_wake] = ACTIONS(1879),
    [sym_timeline_create] = ACTIONS(1879),
    [sym_timeline_kill] = ACTIONS(1879),
    [sym_timeline_reload] = ACTIONS(1879),
    [sym_timeline_pause] = ACTIONS(1879),
    [sym_timeline_restart] = ACTIONS(1879),
    [sym_timeline_set] = ACTIONS(1879),
    [sym_timeline_get] = ACTIONS(1879),
    [sym_timeline_active] = ACTIONS(1879),
    [sym_timeline_dynamic_id] = ACTIONS(1879),
    [sym_variable_to_string] = ACTIONS(1879),
    [sym_string_to_variable] = ACTIONS(1879),
    [sym_length_variable_to_string] = ACTIONS(1879),
    [sym_type_cast] = ACTIONS(1879),
    [sym_variable_to_xml] = ACTIONS(1879),
    [sym_xml_to_variable] = ACTIONS(1879),
    [sym_length_variable_to_xml] = ACTIONS(1879),
    [sym_redirect_string] = ACTIONS(1879),
    [sym_random_number] = ACTIONS(1879),
    [sym_get_unique_id] = ACTIONS(1879),
    [sym_get_system_number] = ACTIONS(1879),
    [sym_set_system_number] = ACTIONS(1879),
    [sym_get_serial_number] = ACTIONS(1879),
    [sym_get_master_build] = ACTIONS(1879),
    [sym_set_validation_code] = ACTIONS(1879),
    [sym_reboot] = ACTIONS(1879),
    [sym_load_duet_module] = ACTIONS(1879),
    [sym_unload_duet_module] = ACTIONS(1879),
    [sym_duet_mem_size_set] = ACTIONS(1879),
    [sym_duet_mem_size_get] = ACTIONS(1879),
    [sym_dynamic_application_device] = ACTIONS(1879),
    [sym_dynamic_polled_port] = ACTIONS(1879),
    [sym_static_port_binding] = ACTIONS(1879),
    [sym_static_ip_binding] = ACTIONS(1879),
    [sym_get_dns_list] = ACTIONS(1879),
    [sym_set_dns_list] = ACTIONS(1879),
    [sym_get_ip_address] = ACTIONS(1879),
    [sym_set_ip_address] = ACTIONS(1879),
    [sym_get_url_list] = ACTIONS(1879),
    [sym_add_url_entry] = ACTIONS(1879),
    [sym_delete_url_entry] = ACTIONS(1879),
    [sym_abs_value] = ACTIONS(1879),
    [sym_max_value] = ACTIONS(1879),
    [sym_min_value] = ACTIONS(1879),
    [sym_exp_value] = ACTIONS(1879),
    [sym_log_value] = ACTIONS(1879),
    [sym_log10_value] = ACTIONS(1879),
    [sym_power_value] = ACTIONS(1879),
    [sym_sqrt_value] = ACTIONS(1879),
    [sym_raw_be] = ACTIONS(1879),
    [sym_raw_le] = ACTIONS(1879),
    [sym_internal_queue_size_set] = ACTIONS(1879),
    [sym_internal_queue_size_get] = ACTIONS(1879),
    [sym_internal_threshold_set] = ACTIONS(1879),
    [sym_internal_threshold_get] = ACTIONS(1879),
    [sym_rebuilt_event] = ACTIONS(1879),
    [sym_do_custom_event] = ACTIONS(1879),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1879),
    [sym_clkmgr_set_clk_source] = ACTIONS(1879),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1879),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1879),
    [sym_clkmgr_get_timezone] = ACTIONS(1879),
    [sym_clkmgr_set_timezone] = ACTIONS(1879),
    [sym_clkmgr_get_resync_period] = ACTIONS(1879),
    [sym_clkmgr_set_resync_period] = ACTIONS(1879),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1879),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1879),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1879),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1879),
    [sym_clkmgr_get_timeservers] = ACTIONS(1879),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1879),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1879),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1879),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1879),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1879),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1879),
    [sym_set_log_level] = ACTIONS(1879),
    [sym_get_log_level] = ACTIONS(1879),
    [sym_amx_log] = ACTIONS(1879),
    [sym_smtp_server_config_set] = ACTIONS(1879),
    [sym_smtp_server_config_get] = ACTIONS(1879),
    [sym_smtp_send] = ACTIONS(1879),
    [sym_validate_netlinx_account] = ACTIONS(1879),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1879),
    [sym_authenticate_certificate] = ACTIONS(1879),
    [sym_audit_netlinx_session_event] = ACTIONS(1879),
    [sym_audit_netlinx_generic_event] = ACTIONS(1879),
    [sym_get_available_flash_disk_space] = ACTIONS(1879),
    [sym_get_max_flash_disk_space] = ACTIONS(1879),
    [sym_data_create_feed] = ACTIONS(1879),
    [sym_data_delete_feed] = ACTIONS(1879),
    [sym_data_publish_feed] = ACTIONS(1879),
    [sym_data_get_published_feed] = ACTIONS(1879),
    [sym_data_add_record] = ACTIONS(1879),
    [sym_data_get_event_record] = ACTIONS(1879),
    [sym__wc_data_create_feed] = ACTIONS(1879),
    [sym__wc_data_add_record] = ACTIONS(1879),
    [sym__wc_data_get_event_record] = ACTIONS(1879),
    [sym_format] = ACTIONS(1879),
    [sym_astro_clock] = ACTIONS(1879),
    [sym_set_outdoor_temperature] = ACTIONS(1879),
    [sym_set_virtual_level_count] = ACTIONS(1879),
    [sym_set_virtual_channel_count] = ACTIONS(1879),
    [sym_set_virtual_port_count] = ACTIONS(1879),
    [sym__wc_encode_int] = ACTIONS(1879),
    [sym__wc_decode_int] = ACTIONS(1879),
    [sym__wc_int] = ACTIONS(1879),
    [sym__wc_to_ch_int] = ACTIONS(1879),
    [sym__ch_to_wc_int] = ACTIONS(1879),
    [sym__wc_find_string_int] = ACTIONS(1879),
    [sym__wc_left_string_int] = ACTIONS(1879),
    [sym__wc_lower_string_int] = ACTIONS(1879),
    [sym__wc_mid_string_int] = ACTIONS(1879),
    [sym__wc_remove_string_int] = ACTIONS(1879),
    [sym__wc_right_string_int] = ACTIONS(1879),
    [sym__wc_upper_string_int] = ACTIONS(1879),
    [sym__wc_compare_string_int] = ACTIONS(1879),
    [sym__wc_get_buffer_char_int] = ACTIONS(1879),
    [sym__wc_get_buffer_string_int] = ACTIONS(1879),
    [sym__wc_concat_string_int] = ACTIONS(1879),
    [sym__wc_file_open_int] = ACTIONS(1879),
    [sym__wc_file_close_int] = ACTIONS(1879),
    [sym__wc_file_read_int] = ACTIONS(1879),
    [sym__wc_file_read_line_int] = ACTIONS(1879),
    [sym__wc_file_write_int] = ACTIONS(1879),
    [sym__wc_file_write_line_int] = ACTIONS(1879),
    [aux_sym_preproc_include_token1] = ACTIONS(1879),
    [aux_sym_preproc_define_token1] = ACTIONS(1881),
    [aux_sym_preproc_warn_token1] = ACTIONS(1881),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1881),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1881),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1881),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1881),
    [aux_sym_preproc_else_token1] = ACTIONS(1881),
    [anon_sym_LBRACK] = ACTIONS(1881),
    [anon_sym_LBRACE] = ACTIONS(1881),
    [anon_sym_LPAREN] = ACTIONS(1879),
    [anon_sym_BANG] = ACTIONS(1881),
    [anon_sym_TILDE] = ACTIONS(1881),
    [anon_sym_DASH] = ACTIONS(1879),
    [anon_sym_PLUS] = ACTIONS(1879),
    [anon_sym_DASH_DASH] = ACTIONS(1881),
    [anon_sym_PLUS_PLUS] = ACTIONS(1881),
    [anon_sym_DQUOTE] = ACTIONS(1881),
    [anon_sym_SQUOTE] = ACTIONS(1881),
    [sym_number_literal] = ACTIONS(1881),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(266)] = {
    [sym_identifier] = ACTIONS(1667),
    [sym_program_name_keyword] = ACTIONS(1667),
    [sym_module_name_keyword] = ACTIONS(1667),
    [sym_define_device_keyword] = ACTIONS(1667),
    [sym_define_combine_keyword] = ACTIONS(1667),
    [sym_define_constant_keyword] = ACTIONS(1667),
    [sym_define_type_keyword] = ACTIONS(1667),
    [sym_define_variable_keyword] = ACTIONS(1667),
    [sym_define_system_variable_keyword] = ACTIONS(1667),
    [sym_define_connect_level_keyword] = ACTIONS(1667),
    [sym_define_latching_keyword] = ACTIONS(1667),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1667),
    [sym_define_toggling_keyword] = ACTIONS(1667),
    [sym_define_call_keyword] = ACTIONS(1667),
    [sym_define_function_keyword] = ACTIONS(1667),
    [sym_define_library_function_keyword] = ACTIONS(1667),
    [sym_define_start_keyword] = ACTIONS(1667),
    [sym_define_event_keyword] = ACTIONS(1667),
    [sym_define_program_keyword] = ACTIONS(1667),
    [sym_define_module_keyword] = ACTIONS(1667),
    [sym_char_keyword] = ACTIONS(1667),
    [sym_widechar_keyword] = ACTIONS(1667),
    [sym_integer_keyword] = ACTIONS(1667),
    [sym_sinteger_keyword] = ACTIONS(1667),
    [sym_long_keyword] = ACTIONS(1667),
    [sym_slong_keyword] = ACTIONS(1667),
    [sym_float_keyword] = ACTIONS(1667),
    [sym_double_keyword] = ACTIONS(1667),
    [sym_variant_keyword] = ACTIONS(1667),
    [sym_variantarray_keyword] = ACTIONS(1667),
    [sym_local_var_keyword] = ACTIONS(1667),
    [sym_stack_var_keyword] = ACTIONS(1667),
    [sym_constant_keyword] = ACTIONS(1667),
    [sym_volatile_keyword] = ACTIONS(1667),
    [sym_non_volatile_keyword] = ACTIONS(1667),
    [sym_persistent_keyword] = ACTIONS(1667),
    [sym_dev_keyword] = ACTIONS(1667),
    [sym_devlev_keyword] = ACTIONS(1667),
    [sym_devchan_keyword] = ACTIONS(1667),
    [sym_if_keyword] = ACTIONS(1667),
    [sym_else_keyword] = ACTIONS(1667),
    [sym_switch_keyword] = ACTIONS(1667),
    [sym_case_keyword] = ACTIONS(1667),
    [sym_default_keyword] = ACTIONS(1667),
    [sym_while_keyword] = ACTIONS(1667),
    [sym_for_keyword] = ACTIONS(1667),
    [sym_break_keyword] = ACTIONS(1667),
    [sym_continue_keyword] = ACTIONS(1667),
    [sym_return_keyword] = ACTIONS(1667),
    [sym_select_keyword] = ACTIONS(1667),
    [sym_active_keyword] = ACTIONS(1667),
    [sym_struct_keyword] = ACTIONS(1667),
    [sym_structure_keyword] = ACTIONS(1667),
    [sym_button_event_keyword] = ACTIONS(1667),
    [sym_channel_event_keyword] = ACTIONS(1667),
    [sym_level_event_keyword] = ACTIONS(1667),
    [sym_data_event_keyword] = ACTIONS(1667),
    [sym_timeline_event_keyword] = ACTIONS(1667),
    [sym_custom_event_keyword] = ACTIONS(1667),
    [sym_push_keyword] = ACTIONS(1667),
    [sym_release_keyword] = ACTIONS(1667),
    [aux_sym_on_keyword_token1] = ACTIONS(1667),
    [aux_sym_off_keyword_token1] = ACTIONS(1667),
    [sym_send_level_keyword] = ACTIONS(1667),
    [sym_send_string_keyword] = ACTIONS(1667),
    [sym_send_command_keyword] = ACTIONS(1667),
    [sym_clear_buffer_keyword] = ACTIONS(1667),
    [sym_create_buffer_keyword] = ACTIONS(1667),
    [sym_create_multi_buffer_keyword] = ACTIONS(1667),
    [sym_call_keyword] = ACTIONS(1667),
    [sym_system_call_keyword] = ACTIONS(1667),
    [sym_devchan_to_keyword] = ACTIONS(1667),
    [sym_devchan_min_to_keyword] = ACTIONS(1667),
    [sym_devchan_total_off_keyword] = ACTIONS(1667),
    [sym_devchan_pulse_keyword] = ACTIONS(1667),
    [sym_wait_keyword] = ACTIONS(1667),
    [sym_cancel_wait_keyword] = ACTIONS(1667),
    [sym_wait_until_keyword] = ACTIONS(1667),
    [sym_cancel_wait_until_keyword] = ACTIONS(1667),
    [sym_cancel_all_wait_keyword] = ACTIONS(1667),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1667),
    [sym_not] = ACTIONS(1667),
    [sym_bnot] = ACTIONS(1667),
    [sym___file__] = ACTIONS(1667),
    [sym___date__] = ACTIONS(1667),
    [sym___ldate__] = ACTIONS(1667),
    [sym___time__] = ACTIONS(1667),
    [sym___line__] = ACTIONS(1667),
    [sym___name__] = ACTIONS(1667),
    [sym___netlinx__] = ACTIONS(1667),
    [sym_netlinx_axi_version] = ACTIONS(1667),
    [sym_url_flg_tcp] = ACTIONS(1667),
    [sym_url_flg_accinfopresent] = ACTIONS(1667),
    [sym_url_flg_temp] = ACTIONS(1667),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1667),
    [sym_url_flg_stat_mask] = ACTIONS(1667),
    [sym_url_flg_stat_lookup] = ACTIONS(1667),
    [sym_url_flg_stat_connecting] = ACTIONS(1667),
    [sym_url_flg_stat_waiting] = ACTIONS(1667),
    [sym_url_flg_stat_connected] = ACTIONS(1667),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1667),
    [sym_first_virtual_device] = ACTIONS(1667),
    [sym_first_local_port] = ACTIONS(1667),
    [sym_dynamic_virtual_device] = ACTIONS(1667),
    [sym_timeline_once] = ACTIONS(1667),
    [sym_timeline_repeat] = ACTIONS(1667),
    [sym_timeline_absolute] = ACTIONS(1667),
    [sym_timeline_relative] = ACTIONS(1667),
    [sym_do_push_timed_infinite] = ACTIONS(1667),
    [sym_source_type_no_address] = ACTIONS(1667),
    [sym_source_type_neuron_id] = ACTIONS(1667),
    [sym_source_type_ip_address] = ACTIONS(1667),
    [sym_source_type_axlink] = ACTIONS(1667),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1667),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1667),
    [sym_source_type_ip_socket_address] = ACTIONS(1667),
    [sym_source_type_rs232] = ACTIONS(1667),
    [sym_source_type_internal] = ACTIONS(1667),
    [sym_source_type_mac_address] = ACTIONS(1667),
    [sym_source_type_ipv4_port] = ACTIONS(1667),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1667),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1667),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1667),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1667),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1667),
    [sym_true] = ACTIONS(1667),
    [sym_false] = ACTIONS(1667),
    [sym_file_read_only] = ACTIONS(1667),
    [sym_file_rw_new] = ACTIONS(1667),
    [sym_file_rw_append] = ACTIONS(1667),
    [sym_ip_tcp] = ACTIONS(1667),
    [sym_ip_udp] = ACTIONS(1667),
    [sym_ip_udp_2way] = ACTIONS(1667),
    [sym_xml_encode_types] = ACTIONS(1667),
    [sym_xml_encode_char_as_list] = ACTIONS(1667),
    [sym_xml_encode_le] = ACTIONS(1667),
    [sym_xml_decode_types] = ACTIONS(1667),
    [sym_xml_decode_no_preserve] = ACTIONS(1667),
    [sym_duet_dev_type_display_device] = ACTIONS(1667),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1667),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1667),
    [sym_duet_dev_type_io_device] = ACTIONS(1667),
    [sym_duet_dev_type_relay_device] = ACTIONS(1667),
    [sym_duet_dev_type_ups] = ACTIONS(1667),
    [sym_duet_dev_type_amplifier] = ACTIONS(1667),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1667),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1667),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1667),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1667),
    [sym_duet_dev_type_camera] = ACTIONS(1667),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1667),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1667),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1667),
    [sym_duet_dev_type_dss] = ACTIONS(1667),
    [sym_duet_dev_type_dvr] = ACTIONS(1667),
    [sym_duet_dev_type_disc_device] = ACTIONS(1667),
    [sym_duet_dev_type_document_camera] = ACTIONS(1667),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1667),
    [sym_duet_dev_type_hvac] = ACTIONS(1667),
    [sym_duet_dev_type_keypad] = ACTIONS(1667),
    [sym_duet_dev_type_light] = ACTIONS(1667),
    [sym_duet_dev_type_monitor] = ACTIONS(1667),
    [sym_duet_dev_type_motor] = ACTIONS(1667),
    [sym_duet_dev_type_multi_window] = ACTIONS(1667),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1667),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1667),
    [sym_duet_dev_type_receiver] = ACTIONS(1667),
    [sym_duet_dev_type_security_system] = ACTIONS(1667),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1667),
    [sym_duet_dev_type_settop_box] = ACTIONS(1667),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1667),
    [sym_duet_dev_type_switcher] = ACTIONS(1667),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1667),
    [sym_duet_dev_type_tv] = ACTIONS(1667),
    [sym_duet_dev_type_utility] = ACTIONS(1667),
    [sym_duet_dev_type_vcr] = ACTIONS(1667),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1667),
    [sym_duet_dev_type_video_processor] = ACTIONS(1667),
    [sym_duet_dev_type_video_projector] = ACTIONS(1667),
    [sym_duet_dev_type_video_wall] = ACTIONS(1667),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1667),
    [sym_duet_dev_type_weather] = ACTIONS(1667),
    [sym_duet_dev_not_polled] = ACTIONS(1667),
    [sym_duet_dev_polled] = ACTIONS(1667),
    [sym_clkmgr_mode_network] = ACTIONS(1667),
    [sym_clkmgr_mode_standalone] = ACTIONS(1667),
    [sym_normal_standby] = ACTIONS(1667),
    [sym_normal_wake] = ACTIONS(1667),
    [sym_ip_multicast_ttl_option] = ACTIONS(1667),
    [sym_ip_ttl_subnet] = ACTIONS(1667),
    [sym_ip_ttl_site] = ACTIONS(1667),
    [sym_ip_ttl_region] = ACTIONS(1667),
    [sym_ip_ttl_continent] = ACTIONS(1667),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1667),
    [sym_ip_nodelay_on] = ACTIONS(1667),
    [sym_ip_nodelay_off] = ACTIONS(1667),
    [sym_amx_error] = ACTIONS(1667),
    [sym_amx_warning] = ACTIONS(1667),
    [sym_amx_info] = ACTIONS(1667),
    [sym_amx_debug] = ACTIONS(1667),
    [sym_netlinx_login_success] = ACTIONS(1667),
    [sym_netlinx_login_fail] = ACTIONS(1667),
    [sym_netlinx_logout] = ACTIONS(1667),
    [sym_null_str] = ACTIONS(1667),
    [sym_smtp_address] = ACTIONS(1667),
    [sym_smtp_port_number] = ACTIONS(1667),
    [sym_smtp_username] = ACTIONS(1667),
    [sym_smtp_password] = ACTIONS(1667),
    [sym_smtp_require_tls] = ACTIONS(1667),
    [sym_smtp_from] = ACTIONS(1667),
    [sym_smtp_tls_true] = ACTIONS(1667),
    [sym_smtp_tls_false] = ACTIONS(1667),
    [sym_listview_on_row_select_event] = ACTIONS(1667),
    [sym_data_structure_unknown] = ACTIONS(1667),
    [sym_data_structure_datafeed] = ACTIONS(1667),
    [sym_data_structure_datarecordset] = ACTIONS(1667),
    [sym_data_structure_datarecord] = ACTIONS(1667),
    [sym_data_structure_datafield] = ACTIONS(1667),
    [sym_data_max_content_field_count] = ACTIONS(1667),
    [sym_data_max_metadata_field_count] = ACTIONS(1667),
    [sym_data_max_id_length] = ACTIONS(1667),
    [sym_data_max_name_length] = ACTIONS(1667),
    [sym_data_max_description_length] = ACTIONS(1667),
    [sym_data_max_type_length] = ACTIONS(1667),
    [sym_data_max_format_length] = ACTIONS(1667),
    [sym_data_max_source_length] = ACTIONS(1667),
    [sym_data_max_value_length] = ACTIONS(1667),
    [sym_data_max_label_length] = ACTIONS(1667),
    [sym_data_max_url_length] = ACTIONS(1667),
    [sym_max_last_login_info_length] = ACTIONS(1667),
    [sym_data_type_unknown] = ACTIONS(1667),
    [sym_data_type_string] = ACTIONS(1667),
    [sym_data_type_datetime] = ACTIONS(1667),
    [sym_data_type_date] = ACTIONS(1667),
    [sym_data_type_time] = ACTIONS(1667),
    [sym_data_type_image] = ACTIONS(1667),
    [sym_data_format_url] = ACTIONS(1667),
    [sym_data_format_phone] = ACTIONS(1667),
    [sym_data_format_email] = ACTIONS(1667),
    [sym_data_format_iso8601] = ACTIONS(1667),
    [sym_auditlog_priv] = ACTIONS(1667),
    [sym_configuration_priv] = ACTIONS(1667),
    [sym_deviceconfig_priv] = ACTIONS(1667),
    [sym_ftp_priv] = ACTIONS(1667),
    [sym_http_priv] = ACTIONS(1667),
    [sym_networkconfig_priv] = ACTIONS(1667),
    [sym_programport_priv] = ACTIONS(1667),
    [sym_remoteui_priv] = ACTIONS(1667),
    [sym_securitycontrol_priv] = ACTIONS(1667),
    [sym_softwaremanagement_priv] = ACTIONS(1667),
    [sym_terminal_priv] = ACTIONS(1667),
    [sym_tpadmin_priv] = ACTIONS(1667),
    [sym_user1_priv] = ACTIONS(1667),
    [sym_user2_priv] = ACTIONS(1667),
    [sym_user3_priv] = ACTIONS(1667),
    [sym_user4_priv] = ACTIONS(1667),
    [sym_usermanagement_priv] = ACTIONS(1667),
    [sym_valid_account] = ACTIONS(1667),
    [sym_err_name_invalid] = ACTIONS(1667),
    [sym_err_password_invalid] = ACTIONS(1667),
    [sym_err_invalid_account] = ACTIONS(1667),
    [sym_err_no_authorization] = ACTIONS(1667),
    [sym_err_invalid_parameter] = ACTIONS(1667),
    [sym_err_user_locked_out] = ACTIONS(1667),
    [sym_err_user_account_expired] = ACTIONS(1667),
    [sym_err_general_error] = ACTIONS(1667),
    [sym_err_authentication_error] = ACTIONS(1667),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1667),
    [sym_tls_validate_certificate] = ACTIONS(1667),
    [sym_tdata] = ACTIONS(1667),
    [sym_tchannel] = ACTIONS(1667),
    [sym_tlevel] = ACTIONS(1667),
    [sym_tbutton] = ACTIONS(1667),
    [sym_ttimeline] = ACTIONS(1667),
    [sym_tcustom] = ACTIONS(1667),
    [sym_url_struct] = ACTIONS(1667),
    [sym_dns_struct] = ACTIONS(1667),
    [sym_ip_address_struct] = ACTIONS(1667),
    [sym_dev_info_struct] = ACTIONS(1667),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1667),
    [sym_data_feed] = ACTIONS(1667),
    [sym_data_field] = ACTIONS(1667),
    [sym_data_record] = ACTIONS(1667),
    [sym_wc_data_feed] = ACTIONS(1667),
    [sym_wc_data_field] = ACTIONS(1667),
    [sym_wc_data_record] = ACTIONS(1667),
    [sym_last_login_info] = ACTIONS(1667),
    [sym____reserved___] = ACTIONS(1667),
    [sym_date] = ACTIONS(1667),
    [sym_day] = ACTIONS(1667),
    [sym_time] = ACTIONS(1667),
    [sym_ldate] = ACTIONS(1667),
    [sym_push_device] = ACTIONS(1667),
    [sym_push_channel] = ACTIONS(1667),
    [sym_push_devchan] = ACTIONS(1667),
    [sym_release_device] = ACTIONS(1667),
    [sym_release_channel] = ACTIONS(1667),
    [sym_release_devchan] = ACTIONS(1667),
    [sym_master_slot] = ACTIONS(1667),
    [sym_get_pulse_time] = ACTIONS(1667),
    [sym_get_timer] = ACTIONS(1667),
    [sym_master_sn] = ACTIONS(1667),
    [sym_system_number] = ACTIONS(1667),
    [sym_button] = ACTIONS(1667),
    [sym_dv_channel] = ACTIONS(1667),
    [sym_data] = ACTIONS(1667),
    [sym_level] = ACTIONS(1667),
    [sym_timeline] = ACTIONS(1667),
    [sym_channel] = ACTIONS(1667),
    [sym_custom] = ACTIONS(1667),
    [sym_atoi] = ACTIONS(1667),
    [sym_atol] = ACTIONS(1667),
    [sym_atof] = ACTIONS(1667),
    [sym_itoa] = ACTIONS(1667),
    [sym_itohex] = ACTIONS(1667),
    [sym_hextoi] = ACTIONS(1667),
    [sym_ftoa] = ACTIONS(1667),
    [sym_find_string] = ACTIONS(1667),
    [sym_left_string] = ACTIONS(1667),
    [sym_right_string] = ACTIONS(1667),
    [sym_mid_string] = ACTIONS(1667),
    [sym_remove_string] = ACTIONS(1667),
    [sym_get_buffer_string] = ACTIONS(1667),
    [sym_get_buffer_char] = ACTIONS(1667),
    [sym_get_multi_buffer_string] = ACTIONS(1667),
    [sym_length_string] = ACTIONS(1667),
    [sym_max_length_string] = ACTIONS(1667),
    [sym_set_length_string] = ACTIONS(1667),
    [sym_compare_string] = ACTIONS(1667),
    [sym_upper_string] = ACTIONS(1667),
    [sym_lower_string] = ACTIONS(1667),
    [sym_get_last] = ACTIONS(1667),
    [sym_date_to_day] = ACTIONS(1667),
    [sym_date_to_month] = ACTIONS(1667),
    [sym_date_to_year] = ACTIONS(1667),
    [sym_time_to_hour] = ACTIONS(1667),
    [sym_time_to_minute] = ACTIONS(1667),
    [sym_time_to_second] = ACTIONS(1667),
    [sym_day_of_week] = ACTIONS(1667),
    [sym_set_pulse_time] = ACTIONS(1667),
    [sym_set_timer] = ACTIONS(1667),
    [sym_file_open] = ACTIONS(1667),
    [sym_file_close] = ACTIONS(1667),
    [sym_file_read] = ACTIONS(1667),
    [sym_file_read_line] = ACTIONS(1667),
    [sym_file_write] = ACTIONS(1667),
    [sym_file_write_line] = ACTIONS(1667),
    [sym_file_copy] = ACTIONS(1667),
    [sym_file_delete] = ACTIONS(1667),
    [sym_file_rename] = ACTIONS(1667),
    [sym_file_dir] = ACTIONS(1667),
    [sym_file_getdir] = ACTIONS(1667),
    [sym_file_setdir] = ACTIONS(1667),
    [sym_file_create_dir] = ACTIONS(1667),
    [sym_file_remove_dir] = ACTIONS(1667),
    [sym_file_seek] = ACTIONS(1667),
    [sym_length_array] = ACTIONS(1667),
    [sym_max_length_array] = ACTIONS(1667),
    [sym_set_length_array] = ACTIONS(1667),
    [sym_ip_client_open] = ACTIONS(1667),
    [sym_ip_client_close] = ACTIONS(1667),
    [sym_ip_server_open] = ACTIONS(1667),
    [sym_ip_server_close] = ACTIONS(1667),
    [sym_ip_bound_client_open] = ACTIONS(1667),
    [sym_ip_mc_server_open] = ACTIONS(1667),
    [sym_ip_set_option] = ACTIONS(1667),
    [sym_ssh_client_open] = ACTIONS(1667),
    [sym_ssh_client_close] = ACTIONS(1667),
    [sym_tls_client_open] = ACTIONS(1667),
    [sym_tls_client_close] = ACTIONS(1667),
    [sym_do_push] = ACTIONS(1667),
    [sym_do_push_timed] = ACTIONS(1667),
    [sym_do_release] = ACTIONS(1667),
    [sym_device_id] = ACTIONS(1667),
    [sym_device_info] = ACTIONS(1667),
    [sym_device_id_string] = ACTIONS(1667),
    [sym_device_standby] = ACTIONS(1667),
    [sym_device_wake] = ACTIONS(1667),
    [sym_timeline_create] = ACTIONS(1667),
    [sym_timeline_kill] = ACTIONS(1667),
    [sym_timeline_reload] = ACTIONS(1667),
    [sym_timeline_pause] = ACTIONS(1667),
    [sym_timeline_restart] = ACTIONS(1667),
    [sym_timeline_set] = ACTIONS(1667),
    [sym_timeline_get] = ACTIONS(1667),
    [sym_timeline_active] = ACTIONS(1667),
    [sym_timeline_dynamic_id] = ACTIONS(1667),
    [sym_variable_to_string] = ACTIONS(1667),
    [sym_string_to_variable] = ACTIONS(1667),
    [sym_length_variable_to_string] = ACTIONS(1667),
    [sym_type_cast] = ACTIONS(1667),
    [sym_variable_to_xml] = ACTIONS(1667),
    [sym_xml_to_variable] = ACTIONS(1667),
    [sym_length_variable_to_xml] = ACTIONS(1667),
    [sym_redirect_string] = ACTIONS(1667),
    [sym_random_number] = ACTIONS(1667),
    [sym_get_unique_id] = ACTIONS(1667),
    [sym_get_system_number] = ACTIONS(1667),
    [sym_set_system_number] = ACTIONS(1667),
    [sym_get_serial_number] = ACTIONS(1667),
    [sym_get_master_build] = ACTIONS(1667),
    [sym_set_validation_code] = ACTIONS(1667),
    [sym_reboot] = ACTIONS(1667),
    [sym_load_duet_module] = ACTIONS(1667),
    [sym_unload_duet_module] = ACTIONS(1667),
    [sym_duet_mem_size_set] = ACTIONS(1667),
    [sym_duet_mem_size_get] = ACTIONS(1667),
    [sym_dynamic_application_device] = ACTIONS(1667),
    [sym_dynamic_polled_port] = ACTIONS(1667),
    [sym_static_port_binding] = ACTIONS(1667),
    [sym_static_ip_binding] = ACTIONS(1667),
    [sym_get_dns_list] = ACTIONS(1667),
    [sym_set_dns_list] = ACTIONS(1667),
    [sym_get_ip_address] = ACTIONS(1667),
    [sym_set_ip_address] = ACTIONS(1667),
    [sym_get_url_list] = ACTIONS(1667),
    [sym_add_url_entry] = ACTIONS(1667),
    [sym_delete_url_entry] = ACTIONS(1667),
    [sym_abs_value] = ACTIONS(1667),
    [sym_max_value] = ACTIONS(1667),
    [sym_min_value] = ACTIONS(1667),
    [sym_exp_value] = ACTIONS(1667),
    [sym_log_value] = ACTIONS(1667),
    [sym_log10_value] = ACTIONS(1667),
    [sym_power_value] = ACTIONS(1667),
    [sym_sqrt_value] = ACTIONS(1667),
    [sym_raw_be] = ACTIONS(1667),
    [sym_raw_le] = ACTIONS(1667),
    [sym_internal_queue_size_set] = ACTIONS(1667),
    [sym_internal_queue_size_get] = ACTIONS(1667),
    [sym_internal_threshold_set] = ACTIONS(1667),
    [sym_internal_threshold_get] = ACTIONS(1667),
    [sym_rebuilt_event] = ACTIONS(1667),
    [sym_do_custom_event] = ACTIONS(1667),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1667),
    [sym_clkmgr_set_clk_source] = ACTIONS(1667),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1667),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1667),
    [sym_clkmgr_get_timezone] = ACTIONS(1667),
    [sym_clkmgr_set_timezone] = ACTIONS(1667),
    [sym_clkmgr_get_resync_period] = ACTIONS(1667),
    [sym_clkmgr_set_resync_period] = ACTIONS(1667),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1667),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1667),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1667),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1667),
    [sym_clkmgr_get_timeservers] = ACTIONS(1667),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1667),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1667),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1667),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1667),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1667),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1667),
    [sym_set_log_level] = ACTIONS(1667),
    [sym_get_log_level] = ACTIONS(1667),
    [sym_amx_log] = ACTIONS(1667),
    [sym_smtp_server_config_set] = ACTIONS(1667),
    [sym_smtp_server_config_get] = ACTIONS(1667),
    [sym_smtp_send] = ACTIONS(1667),
    [sym_validate_netlinx_account] = ACTIONS(1667),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1667),
    [sym_authenticate_certificate] = ACTIONS(1667),
    [sym_audit_netlinx_session_event] = ACTIONS(1667),
    [sym_audit_netlinx_generic_event] = ACTIONS(1667),
    [sym_get_available_flash_disk_space] = ACTIONS(1667),
    [sym_get_max_flash_disk_space] = ACTIONS(1667),
    [sym_data_create_feed] = ACTIONS(1667),
    [sym_data_delete_feed] = ACTIONS(1667),
    [sym_data_publish_feed] = ACTIONS(1667),
    [sym_data_get_published_feed] = ACTIONS(1667),
    [sym_data_add_record] = ACTIONS(1667),
    [sym_data_get_event_record] = ACTIONS(1667),
    [sym__wc_data_create_feed] = ACTIONS(1667),
    [sym__wc_data_add_record] = ACTIONS(1667),
    [sym__wc_data_get_event_record] = ACTIONS(1667),
    [sym_format] = ACTIONS(1667),
    [sym_astro_clock] = ACTIONS(1667),
    [sym_set_outdoor_temperature] = ACTIONS(1667),
    [sym_set_virtual_level_count] = ACTIONS(1667),
    [sym_set_virtual_channel_count] = ACTIONS(1667),
    [sym_set_virtual_port_count] = ACTIONS(1667),
    [sym__wc_encode_int] = ACTIONS(1667),
    [sym__wc_decode_int] = ACTIONS(1667),
    [sym__wc_int] = ACTIONS(1667),
    [sym__wc_to_ch_int] = ACTIONS(1667),
    [sym__ch_to_wc_int] = ACTIONS(1667),
    [sym__wc_find_string_int] = ACTIONS(1667),
    [sym__wc_left_string_int] = ACTIONS(1667),
    [sym__wc_lower_string_int] = ACTIONS(1667),
    [sym__wc_mid_string_int] = ACTIONS(1667),
    [sym__wc_remove_string_int] = ACTIONS(1667),
    [sym__wc_right_string_int] = ACTIONS(1667),
    [sym__wc_upper_string_int] = ACTIONS(1667),
    [sym__wc_compare_string_int] = ACTIONS(1667),
    [sym__wc_get_buffer_char_int] = ACTIONS(1667),
    [sym__wc_get_buffer_string_int] = ACTIONS(1667),
    [sym__wc_concat_string_int] = ACTIONS(1667),
    [sym__wc_file_open_int] = ACTIONS(1667),
    [sym__wc_file_close_int] = ACTIONS(1667),
    [sym__wc_file_read_int] = ACTIONS(1667),
    [sym__wc_file_read_line_int] = ACTIONS(1667),
    [sym__wc_file_write_int] = ACTIONS(1667),
    [sym__wc_file_write_line_int] = ACTIONS(1667),
    [aux_sym_preproc_include_token1] = ACTIONS(1667),
    [aux_sym_preproc_define_token1] = ACTIONS(1665),
    [aux_sym_preproc_warn_token1] = ACTIONS(1665),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1665),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1665),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1665),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1665),
    [aux_sym_preproc_else_token1] = ACTIONS(1665),
    [anon_sym_LBRACK] = ACTIONS(1665),
    [anon_sym_LBRACE] = ACTIONS(1665),
    [anon_sym_LPAREN] = ACTIONS(1667),
    [anon_sym_BANG] = ACTIONS(1665),
    [anon_sym_TILDE] = ACTIONS(1665),
    [anon_sym_DASH] = ACTIONS(1667),
    [anon_sym_PLUS] = ACTIONS(1667),
    [anon_sym_DASH_DASH] = ACTIONS(1665),
    [anon_sym_PLUS_PLUS] = ACTIONS(1665),
    [anon_sym_DQUOTE] = ACTIONS(1665),
    [anon_sym_SQUOTE] = ACTIONS(1665),
    [sym_number_literal] = ACTIONS(1665),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(267)] = {
    [sym_identifier] = ACTIONS(1781),
    [sym_program_name_keyword] = ACTIONS(1781),
    [sym_module_name_keyword] = ACTIONS(1781),
    [sym_define_device_keyword] = ACTIONS(1781),
    [sym_define_combine_keyword] = ACTIONS(1781),
    [sym_define_constant_keyword] = ACTIONS(1781),
    [sym_define_type_keyword] = ACTIONS(1781),
    [sym_define_variable_keyword] = ACTIONS(1781),
    [sym_define_system_variable_keyword] = ACTIONS(1781),
    [sym_define_connect_level_keyword] = ACTIONS(1781),
    [sym_define_latching_keyword] = ACTIONS(1781),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1781),
    [sym_define_toggling_keyword] = ACTIONS(1781),
    [sym_define_call_keyword] = ACTIONS(1781),
    [sym_define_function_keyword] = ACTIONS(1781),
    [sym_define_library_function_keyword] = ACTIONS(1781),
    [sym_define_start_keyword] = ACTIONS(1781),
    [sym_define_event_keyword] = ACTIONS(1781),
    [sym_define_program_keyword] = ACTIONS(1781),
    [sym_define_module_keyword] = ACTIONS(1781),
    [sym_char_keyword] = ACTIONS(1781),
    [sym_widechar_keyword] = ACTIONS(1781),
    [sym_integer_keyword] = ACTIONS(1781),
    [sym_sinteger_keyword] = ACTIONS(1781),
    [sym_long_keyword] = ACTIONS(1781),
    [sym_slong_keyword] = ACTIONS(1781),
    [sym_float_keyword] = ACTIONS(1781),
    [sym_double_keyword] = ACTIONS(1781),
    [sym_variant_keyword] = ACTIONS(1781),
    [sym_variantarray_keyword] = ACTIONS(1781),
    [sym_local_var_keyword] = ACTIONS(1781),
    [sym_stack_var_keyword] = ACTIONS(1781),
    [sym_constant_keyword] = ACTIONS(1781),
    [sym_volatile_keyword] = ACTIONS(1781),
    [sym_non_volatile_keyword] = ACTIONS(1781),
    [sym_persistent_keyword] = ACTIONS(1781),
    [sym_dev_keyword] = ACTIONS(1781),
    [sym_devlev_keyword] = ACTIONS(1781),
    [sym_devchan_keyword] = ACTIONS(1781),
    [sym_if_keyword] = ACTIONS(1781),
    [sym_else_keyword] = ACTIONS(1781),
    [sym_switch_keyword] = ACTIONS(1781),
    [sym_case_keyword] = ACTIONS(1781),
    [sym_default_keyword] = ACTIONS(1781),
    [sym_while_keyword] = ACTIONS(1781),
    [sym_for_keyword] = ACTIONS(1781),
    [sym_break_keyword] = ACTIONS(1781),
    [sym_continue_keyword] = ACTIONS(1781),
    [sym_return_keyword] = ACTIONS(1781),
    [sym_select_keyword] = ACTIONS(1781),
    [sym_active_keyword] = ACTIONS(1781),
    [sym_struct_keyword] = ACTIONS(1781),
    [sym_structure_keyword] = ACTIONS(1781),
    [sym_button_event_keyword] = ACTIONS(1781),
    [sym_channel_event_keyword] = ACTIONS(1781),
    [sym_level_event_keyword] = ACTIONS(1781),
    [sym_data_event_keyword] = ACTIONS(1781),
    [sym_timeline_event_keyword] = ACTIONS(1781),
    [sym_custom_event_keyword] = ACTIONS(1781),
    [sym_push_keyword] = ACTIONS(1781),
    [sym_release_keyword] = ACTIONS(1781),
    [aux_sym_on_keyword_token1] = ACTIONS(1781),
    [aux_sym_off_keyword_token1] = ACTIONS(1781),
    [sym_send_level_keyword] = ACTIONS(1781),
    [sym_send_string_keyword] = ACTIONS(1781),
    [sym_send_command_keyword] = ACTIONS(1781),
    [sym_clear_buffer_keyword] = ACTIONS(1781),
    [sym_create_buffer_keyword] = ACTIONS(1781),
    [sym_create_multi_buffer_keyword] = ACTIONS(1781),
    [sym_call_keyword] = ACTIONS(1781),
    [sym_system_call_keyword] = ACTIONS(1781),
    [sym_devchan_to_keyword] = ACTIONS(1781),
    [sym_devchan_min_to_keyword] = ACTIONS(1781),
    [sym_devchan_total_off_keyword] = ACTIONS(1781),
    [sym_devchan_pulse_keyword] = ACTIONS(1781),
    [sym_wait_keyword] = ACTIONS(1781),
    [sym_cancel_wait_keyword] = ACTIONS(1781),
    [sym_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1781),
    [sym_not] = ACTIONS(1781),
    [sym_bnot] = ACTIONS(1781),
    [sym___file__] = ACTIONS(1781),
    [sym___date__] = ACTIONS(1781),
    [sym___ldate__] = ACTIONS(1781),
    [sym___time__] = ACTIONS(1781),
    [sym___line__] = ACTIONS(1781),
    [sym___name__] = ACTIONS(1781),
    [sym___netlinx__] = ACTIONS(1781),
    [sym_netlinx_axi_version] = ACTIONS(1781),
    [sym_url_flg_tcp] = ACTIONS(1781),
    [sym_url_flg_accinfopresent] = ACTIONS(1781),
    [sym_url_flg_temp] = ACTIONS(1781),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1781),
    [sym_url_flg_stat_mask] = ACTIONS(1781),
    [sym_url_flg_stat_lookup] = ACTIONS(1781),
    [sym_url_flg_stat_connecting] = ACTIONS(1781),
    [sym_url_flg_stat_waiting] = ACTIONS(1781),
    [sym_url_flg_stat_connected] = ACTIONS(1781),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1781),
    [sym_first_virtual_device] = ACTIONS(1781),
    [sym_first_local_port] = ACTIONS(1781),
    [sym_dynamic_virtual_device] = ACTIONS(1781),
    [sym_timeline_once] = ACTIONS(1781),
    [sym_timeline_repeat] = ACTIONS(1781),
    [sym_timeline_absolute] = ACTIONS(1781),
    [sym_timeline_relative] = ACTIONS(1781),
    [sym_do_push_timed_infinite] = ACTIONS(1781),
    [sym_source_type_no_address] = ACTIONS(1781),
    [sym_source_type_neuron_id] = ACTIONS(1781),
    [sym_source_type_ip_address] = ACTIONS(1781),
    [sym_source_type_axlink] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1781),
    [sym_source_type_ip_socket_address] = ACTIONS(1781),
    [sym_source_type_rs232] = ACTIONS(1781),
    [sym_source_type_internal] = ACTIONS(1781),
    [sym_source_type_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1781),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1781),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1781),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1781),
    [sym_true] = ACTIONS(1781),
    [sym_false] = ACTIONS(1781),
    [sym_file_read_only] = ACTIONS(1781),
    [sym_file_rw_new] = ACTIONS(1781),
    [sym_file_rw_append] = ACTIONS(1781),
    [sym_ip_tcp] = ACTIONS(1781),
    [sym_ip_udp] = ACTIONS(1781),
    [sym_ip_udp_2way] = ACTIONS(1781),
    [sym_xml_encode_types] = ACTIONS(1781),
    [sym_xml_encode_char_as_list] = ACTIONS(1781),
    [sym_xml_encode_le] = ACTIONS(1781),
    [sym_xml_decode_types] = ACTIONS(1781),
    [sym_xml_decode_no_preserve] = ACTIONS(1781),
    [sym_duet_dev_type_display_device] = ACTIONS(1781),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_io_device] = ACTIONS(1781),
    [sym_duet_dev_type_relay_device] = ACTIONS(1781),
    [sym_duet_dev_type_ups] = ACTIONS(1781),
    [sym_duet_dev_type_amplifier] = ACTIONS(1781),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1781),
    [sym_duet_dev_type_camera] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1781),
    [sym_duet_dev_type_dss] = ACTIONS(1781),
    [sym_duet_dev_type_dvr] = ACTIONS(1781),
    [sym_duet_dev_type_disc_device] = ACTIONS(1781),
    [sym_duet_dev_type_document_camera] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1781),
    [sym_duet_dev_type_hvac] = ACTIONS(1781),
    [sym_duet_dev_type_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_light] = ACTIONS(1781),
    [sym_duet_dev_type_monitor] = ACTIONS(1781),
    [sym_duet_dev_type_motor] = ACTIONS(1781),
    [sym_duet_dev_type_multi_window] = ACTIONS(1781),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1781),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1781),
    [sym_duet_dev_type_receiver] = ACTIONS(1781),
    [sym_duet_dev_type_security_system] = ACTIONS(1781),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1781),
    [sym_duet_dev_type_settop_box] = ACTIONS(1781),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1781),
    [sym_duet_dev_type_switcher] = ACTIONS(1781),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_tv] = ACTIONS(1781),
    [sym_duet_dev_type_utility] = ACTIONS(1781),
    [sym_duet_dev_type_vcr] = ACTIONS(1781),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_video_processor] = ACTIONS(1781),
    [sym_duet_dev_type_video_projector] = ACTIONS(1781),
    [sym_duet_dev_type_video_wall] = ACTIONS(1781),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1781),
    [sym_duet_dev_type_weather] = ACTIONS(1781),
    [sym_duet_dev_not_polled] = ACTIONS(1781),
    [sym_duet_dev_polled] = ACTIONS(1781),
    [sym_clkmgr_mode_network] = ACTIONS(1781),
    [sym_clkmgr_mode_standalone] = ACTIONS(1781),
    [sym_normal_standby] = ACTIONS(1781),
    [sym_normal_wake] = ACTIONS(1781),
    [sym_ip_multicast_ttl_option] = ACTIONS(1781),
    [sym_ip_ttl_subnet] = ACTIONS(1781),
    [sym_ip_ttl_site] = ACTIONS(1781),
    [sym_ip_ttl_region] = ACTIONS(1781),
    [sym_ip_ttl_continent] = ACTIONS(1781),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1781),
    [sym_ip_nodelay_on] = ACTIONS(1781),
    [sym_ip_nodelay_off] = ACTIONS(1781),
    [sym_amx_error] = ACTIONS(1781),
    [sym_amx_warning] = ACTIONS(1781),
    [sym_amx_info] = ACTIONS(1781),
    [sym_amx_debug] = ACTIONS(1781),
    [sym_netlinx_login_success] = ACTIONS(1781),
    [sym_netlinx_login_fail] = ACTIONS(1781),
    [sym_netlinx_logout] = ACTIONS(1781),
    [sym_null_str] = ACTIONS(1781),
    [sym_smtp_address] = ACTIONS(1781),
    [sym_smtp_port_number] = ACTIONS(1781),
    [sym_smtp_username] = ACTIONS(1781),
    [sym_smtp_password] = ACTIONS(1781),
    [sym_smtp_require_tls] = ACTIONS(1781),
    [sym_smtp_from] = ACTIONS(1781),
    [sym_smtp_tls_true] = ACTIONS(1781),
    [sym_smtp_tls_false] = ACTIONS(1781),
    [sym_listview_on_row_select_event] = ACTIONS(1781),
    [sym_data_structure_unknown] = ACTIONS(1781),
    [sym_data_structure_datafeed] = ACTIONS(1781),
    [sym_data_structure_datarecordset] = ACTIONS(1781),
    [sym_data_structure_datarecord] = ACTIONS(1781),
    [sym_data_structure_datafield] = ACTIONS(1781),
    [sym_data_max_content_field_count] = ACTIONS(1781),
    [sym_data_max_metadata_field_count] = ACTIONS(1781),
    [sym_data_max_id_length] = ACTIONS(1781),
    [sym_data_max_name_length] = ACTIONS(1781),
    [sym_data_max_description_length] = ACTIONS(1781),
    [sym_data_max_type_length] = ACTIONS(1781),
    [sym_data_max_format_length] = ACTIONS(1781),
    [sym_data_max_source_length] = ACTIONS(1781),
    [sym_data_max_value_length] = ACTIONS(1781),
    [sym_data_max_label_length] = ACTIONS(1781),
    [sym_data_max_url_length] = ACTIONS(1781),
    [sym_max_last_login_info_length] = ACTIONS(1781),
    [sym_data_type_unknown] = ACTIONS(1781),
    [sym_data_type_string] = ACTIONS(1781),
    [sym_data_type_datetime] = ACTIONS(1781),
    [sym_data_type_date] = ACTIONS(1781),
    [sym_data_type_time] = ACTIONS(1781),
    [sym_data_type_image] = ACTIONS(1781),
    [sym_data_format_url] = ACTIONS(1781),
    [sym_data_format_phone] = ACTIONS(1781),
    [sym_data_format_email] = ACTIONS(1781),
    [sym_data_format_iso8601] = ACTIONS(1781),
    [sym_auditlog_priv] = ACTIONS(1781),
    [sym_configuration_priv] = ACTIONS(1781),
    [sym_deviceconfig_priv] = ACTIONS(1781),
    [sym_ftp_priv] = ACTIONS(1781),
    [sym_http_priv] = ACTIONS(1781),
    [sym_networkconfig_priv] = ACTIONS(1781),
    [sym_programport_priv] = ACTIONS(1781),
    [sym_remoteui_priv] = ACTIONS(1781),
    [sym_securitycontrol_priv] = ACTIONS(1781),
    [sym_softwaremanagement_priv] = ACTIONS(1781),
    [sym_terminal_priv] = ACTIONS(1781),
    [sym_tpadmin_priv] = ACTIONS(1781),
    [sym_user1_priv] = ACTIONS(1781),
    [sym_user2_priv] = ACTIONS(1781),
    [sym_user3_priv] = ACTIONS(1781),
    [sym_user4_priv] = ACTIONS(1781),
    [sym_usermanagement_priv] = ACTIONS(1781),
    [sym_valid_account] = ACTIONS(1781),
    [sym_err_name_invalid] = ACTIONS(1781),
    [sym_err_password_invalid] = ACTIONS(1781),
    [sym_err_invalid_account] = ACTIONS(1781),
    [sym_err_no_authorization] = ACTIONS(1781),
    [sym_err_invalid_parameter] = ACTIONS(1781),
    [sym_err_user_locked_out] = ACTIONS(1781),
    [sym_err_user_account_expired] = ACTIONS(1781),
    [sym_err_general_error] = ACTIONS(1781),
    [sym_err_authentication_error] = ACTIONS(1781),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1781),
    [sym_tls_validate_certificate] = ACTIONS(1781),
    [sym_tdata] = ACTIONS(1781),
    [sym_tchannel] = ACTIONS(1781),
    [sym_tlevel] = ACTIONS(1781),
    [sym_tbutton] = ACTIONS(1781),
    [sym_ttimeline] = ACTIONS(1781),
    [sym_tcustom] = ACTIONS(1781),
    [sym_url_struct] = ACTIONS(1781),
    [sym_dns_struct] = ACTIONS(1781),
    [sym_ip_address_struct] = ACTIONS(1781),
    [sym_dev_info_struct] = ACTIONS(1781),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1781),
    [sym_data_feed] = ACTIONS(1781),
    [sym_data_field] = ACTIONS(1781),
    [sym_data_record] = ACTIONS(1781),
    [sym_wc_data_feed] = ACTIONS(1781),
    [sym_wc_data_field] = ACTIONS(1781),
    [sym_wc_data_record] = ACTIONS(1781),
    [sym_last_login_info] = ACTIONS(1781),
    [sym____reserved___] = ACTIONS(1781),
    [sym_date] = ACTIONS(1781),
    [sym_day] = ACTIONS(1781),
    [sym_time] = ACTIONS(1781),
    [sym_ldate] = ACTIONS(1781),
    [sym_push_device] = ACTIONS(1781),
    [sym_push_channel] = ACTIONS(1781),
    [sym_push_devchan] = ACTIONS(1781),
    [sym_release_device] = ACTIONS(1781),
    [sym_release_channel] = ACTIONS(1781),
    [sym_release_devchan] = ACTIONS(1781),
    [sym_master_slot] = ACTIONS(1781),
    [sym_get_pulse_time] = ACTIONS(1781),
    [sym_get_timer] = ACTIONS(1781),
    [sym_master_sn] = ACTIONS(1781),
    [sym_system_number] = ACTIONS(1781),
    [sym_button] = ACTIONS(1781),
    [sym_dv_channel] = ACTIONS(1781),
    [sym_data] = ACTIONS(1781),
    [sym_level] = ACTIONS(1781),
    [sym_timeline] = ACTIONS(1781),
    [sym_channel] = ACTIONS(1781),
    [sym_custom] = ACTIONS(1781),
    [sym_atoi] = ACTIONS(1781),
    [sym_atol] = ACTIONS(1781),
    [sym_atof] = ACTIONS(1781),
    [sym_itoa] = ACTIONS(1781),
    [sym_itohex] = ACTIONS(1781),
    [sym_hextoi] = ACTIONS(1781),
    [sym_ftoa] = ACTIONS(1781),
    [sym_find_string] = ACTIONS(1781),
    [sym_left_string] = ACTIONS(1781),
    [sym_right_string] = ACTIONS(1781),
    [sym_mid_string] = ACTIONS(1781),
    [sym_remove_string] = ACTIONS(1781),
    [sym_get_buffer_string] = ACTIONS(1781),
    [sym_get_buffer_char] = ACTIONS(1781),
    [sym_get_multi_buffer_string] = ACTIONS(1781),
    [sym_length_string] = ACTIONS(1781),
    [sym_max_length_string] = ACTIONS(1781),
    [sym_set_length_string] = ACTIONS(1781),
    [sym_compare_string] = ACTIONS(1781),
    [sym_upper_string] = ACTIONS(1781),
    [sym_lower_string] = ACTIONS(1781),
    [sym_get_last] = ACTIONS(1781),
    [sym_date_to_day] = ACTIONS(1781),
    [sym_date_to_month] = ACTIONS(1781),
    [sym_date_to_year] = ACTIONS(1781),
    [sym_time_to_hour] = ACTIONS(1781),
    [sym_time_to_minute] = ACTIONS(1781),
    [sym_time_to_second] = ACTIONS(1781),
    [sym_day_of_week] = ACTIONS(1781),
    [sym_set_pulse_time] = ACTIONS(1781),
    [sym_set_timer] = ACTIONS(1781),
    [sym_file_open] = ACTIONS(1781),
    [sym_file_close] = ACTIONS(1781),
    [sym_file_read] = ACTIONS(1781),
    [sym_file_read_line] = ACTIONS(1781),
    [sym_file_write] = ACTIONS(1781),
    [sym_file_write_line] = ACTIONS(1781),
    [sym_file_copy] = ACTIONS(1781),
    [sym_file_delete] = ACTIONS(1781),
    [sym_file_rename] = ACTIONS(1781),
    [sym_file_dir] = ACTIONS(1781),
    [sym_file_getdir] = ACTIONS(1781),
    [sym_file_setdir] = ACTIONS(1781),
    [sym_file_create_dir] = ACTIONS(1781),
    [sym_file_remove_dir] = ACTIONS(1781),
    [sym_file_seek] = ACTIONS(1781),
    [sym_length_array] = ACTIONS(1781),
    [sym_max_length_array] = ACTIONS(1781),
    [sym_set_length_array] = ACTIONS(1781),
    [sym_ip_client_open] = ACTIONS(1781),
    [sym_ip_client_close] = ACTIONS(1781),
    [sym_ip_server_open] = ACTIONS(1781),
    [sym_ip_server_close] = ACTIONS(1781),
    [sym_ip_bound_client_open] = ACTIONS(1781),
    [sym_ip_mc_server_open] = ACTIONS(1781),
    [sym_ip_set_option] = ACTIONS(1781),
    [sym_ssh_client_open] = ACTIONS(1781),
    [sym_ssh_client_close] = ACTIONS(1781),
    [sym_tls_client_open] = ACTIONS(1781),
    [sym_tls_client_close] = ACTIONS(1781),
    [sym_do_push] = ACTIONS(1781),
    [sym_do_push_timed] = ACTIONS(1781),
    [sym_do_release] = ACTIONS(1781),
    [sym_device_id] = ACTIONS(1781),
    [sym_device_info] = ACTIONS(1781),
    [sym_device_id_string] = ACTIONS(1781),
    [sym_device_standby] = ACTIONS(1781),
    [sym_device_wake] = ACTIONS(1781),
    [sym_timeline_create] = ACTIONS(1781),
    [sym_timeline_kill] = ACTIONS(1781),
    [sym_timeline_reload] = ACTIONS(1781),
    [sym_timeline_pause] = ACTIONS(1781),
    [sym_timeline_restart] = ACTIONS(1781),
    [sym_timeline_set] = ACTIONS(1781),
    [sym_timeline_get] = ACTIONS(1781),
    [sym_timeline_active] = ACTIONS(1781),
    [sym_timeline_dynamic_id] = ACTIONS(1781),
    [sym_variable_to_string] = ACTIONS(1781),
    [sym_string_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_string] = ACTIONS(1781),
    [sym_type_cast] = ACTIONS(1781),
    [sym_variable_to_xml] = ACTIONS(1781),
    [sym_xml_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_xml] = ACTIONS(1781),
    [sym_redirect_string] = ACTIONS(1781),
    [sym_random_number] = ACTIONS(1781),
    [sym_get_unique_id] = ACTIONS(1781),
    [sym_get_system_number] = ACTIONS(1781),
    [sym_set_system_number] = ACTIONS(1781),
    [sym_get_serial_number] = ACTIONS(1781),
    [sym_get_master_build] = ACTIONS(1781),
    [sym_set_validation_code] = ACTIONS(1781),
    [sym_reboot] = ACTIONS(1781),
    [sym_load_duet_module] = ACTIONS(1781),
    [sym_unload_duet_module] = ACTIONS(1781),
    [sym_duet_mem_size_set] = ACTIONS(1781),
    [sym_duet_mem_size_get] = ACTIONS(1781),
    [sym_dynamic_application_device] = ACTIONS(1781),
    [sym_dynamic_polled_port] = ACTIONS(1781),
    [sym_static_port_binding] = ACTIONS(1781),
    [sym_static_ip_binding] = ACTIONS(1781),
    [sym_get_dns_list] = ACTIONS(1781),
    [sym_set_dns_list] = ACTIONS(1781),
    [sym_get_ip_address] = ACTIONS(1781),
    [sym_set_ip_address] = ACTIONS(1781),
    [sym_get_url_list] = ACTIONS(1781),
    [sym_add_url_entry] = ACTIONS(1781),
    [sym_delete_url_entry] = ACTIONS(1781),
    [sym_abs_value] = ACTIONS(1781),
    [sym_max_value] = ACTIONS(1781),
    [sym_min_value] = ACTIONS(1781),
    [sym_exp_value] = ACTIONS(1781),
    [sym_log_value] = ACTIONS(1781),
    [sym_log10_value] = ACTIONS(1781),
    [sym_power_value] = ACTIONS(1781),
    [sym_sqrt_value] = ACTIONS(1781),
    [sym_raw_be] = ACTIONS(1781),
    [sym_raw_le] = ACTIONS(1781),
    [sym_internal_queue_size_set] = ACTIONS(1781),
    [sym_internal_queue_size_get] = ACTIONS(1781),
    [sym_internal_threshold_set] = ACTIONS(1781),
    [sym_internal_threshold_get] = ACTIONS(1781),
    [sym_rebuilt_event] = ACTIONS(1781),
    [sym_do_custom_event] = ACTIONS(1781),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1781),
    [sym_clkmgr_set_clk_source] = ACTIONS(1781),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1781),
    [sym_clkmgr_get_timezone] = ACTIONS(1781),
    [sym_clkmgr_set_timezone] = ACTIONS(1781),
    [sym_clkmgr_get_resync_period] = ACTIONS(1781),
    [sym_clkmgr_set_resync_period] = ACTIONS(1781),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_timeservers] = ACTIONS(1781),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_set_log_level] = ACTIONS(1781),
    [sym_get_log_level] = ACTIONS(1781),
    [sym_amx_log] = ACTIONS(1781),
    [sym_smtp_server_config_set] = ACTIONS(1781),
    [sym_smtp_server_config_get] = ACTIONS(1781),
    [sym_smtp_send] = ACTIONS(1781),
    [sym_validate_netlinx_account] = ACTIONS(1781),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1781),
    [sym_authenticate_certificate] = ACTIONS(1781),
    [sym_audit_netlinx_session_event] = ACTIONS(1781),
    [sym_audit_netlinx_generic_event] = ACTIONS(1781),
    [sym_get_available_flash_disk_space] = ACTIONS(1781),
    [sym_get_max_flash_disk_space] = ACTIONS(1781),
    [sym_data_create_feed] = ACTIONS(1781),
    [sym_data_delete_feed] = ACTIONS(1781),
    [sym_data_publish_feed] = ACTIONS(1781),
    [sym_data_get_published_feed] = ACTIONS(1781),
    [sym_data_add_record] = ACTIONS(1781),
    [sym_data_get_event_record] = ACTIONS(1781),
    [sym__wc_data_create_feed] = ACTIONS(1781),
    [sym__wc_data_add_record] = ACTIONS(1781),
    [sym__wc_data_get_event_record] = ACTIONS(1781),
    [sym_format] = ACTIONS(1781),
    [sym_astro_clock] = ACTIONS(1781),
    [sym_set_outdoor_temperature] = ACTIONS(1781),
    [sym_set_virtual_level_count] = ACTIONS(1781),
    [sym_set_virtual_channel_count] = ACTIONS(1781),
    [sym_set_virtual_port_count] = ACTIONS(1781),
    [sym__wc_encode_int] = ACTIONS(1781),
    [sym__wc_decode_int] = ACTIONS(1781),
    [sym__wc_int] = ACTIONS(1781),
    [sym__wc_to_ch_int] = ACTIONS(1781),
    [sym__ch_to_wc_int] = ACTIONS(1781),
    [sym__wc_find_string_int] = ACTIONS(1781),
    [sym__wc_left_string_int] = ACTIONS(1781),
    [sym__wc_lower_string_int] = ACTIONS(1781),
    [sym__wc_mid_string_int] = ACTIONS(1781),
    [sym__wc_remove_string_int] = ACTIONS(1781),
    [sym__wc_right_string_int] = ACTIONS(1781),
    [sym__wc_upper_string_int] = ACTIONS(1781),
    [sym__wc_compare_string_int] = ACTIONS(1781),
    [sym__wc_get_buffer_char_int] = ACTIONS(1781),
    [sym__wc_get_buffer_string_int] = ACTIONS(1781),
    [sym__wc_concat_string_int] = ACTIONS(1781),
    [sym__wc_file_open_int] = ACTIONS(1781),
    [sym__wc_file_close_int] = ACTIONS(1781),
    [sym__wc_file_read_int] = ACTIONS(1781),
    [sym__wc_file_read_line_int] = ACTIONS(1781),
    [sym__wc_file_write_int] = ACTIONS(1781),
    [sym__wc_file_write_line_int] = ACTIONS(1781),
    [aux_sym_preproc_include_token1] = ACTIONS(1781),
    [aux_sym_preproc_define_token1] = ACTIONS(1783),
    [aux_sym_preproc_warn_token1] = ACTIONS(1783),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1783),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_else_token1] = ACTIONS(1783),
    [anon_sym_LBRACK] = ACTIONS(1783),
    [anon_sym_LBRACE] = ACTIONS(1783),
    [anon_sym_LPAREN] = ACTIONS(1781),
    [anon_sym_BANG] = ACTIONS(1783),
    [anon_sym_TILDE] = ACTIONS(1783),
    [anon_sym_DASH] = ACTIONS(1781),
    [anon_sym_PLUS] = ACTIONS(1781),
    [anon_sym_DASH_DASH] = ACTIONS(1783),
    [anon_sym_PLUS_PLUS] = ACTIONS(1783),
    [anon_sym_DQUOTE] = ACTIONS(1783),
    [anon_sym_SQUOTE] = ACTIONS(1783),
    [sym_number_literal] = ACTIONS(1783),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(268)] = {
    [ts_builtin_sym_end] = ACTIONS(1857),
    [sym_identifier] = ACTIONS(1855),
    [sym_program_name_keyword] = ACTIONS(1855),
    [sym_module_name_keyword] = ACTIONS(1855),
    [sym_define_device_keyword] = ACTIONS(1855),
    [sym_define_combine_keyword] = ACTIONS(1855),
    [sym_define_constant_keyword] = ACTIONS(1855),
    [sym_define_type_keyword] = ACTIONS(1855),
    [sym_define_variable_keyword] = ACTIONS(1855),
    [sym_define_system_variable_keyword] = ACTIONS(1855),
    [sym_define_connect_level_keyword] = ACTIONS(1855),
    [sym_define_latching_keyword] = ACTIONS(1855),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1855),
    [sym_define_toggling_keyword] = ACTIONS(1855),
    [sym_define_call_keyword] = ACTIONS(1855),
    [sym_define_function_keyword] = ACTIONS(1855),
    [sym_define_library_function_keyword] = ACTIONS(1855),
    [sym_define_start_keyword] = ACTIONS(1855),
    [sym_define_event_keyword] = ACTIONS(1855),
    [sym_define_program_keyword] = ACTIONS(1855),
    [sym_define_module_keyword] = ACTIONS(1855),
    [sym_char_keyword] = ACTIONS(1855),
    [sym_widechar_keyword] = ACTIONS(1855),
    [sym_integer_keyword] = ACTIONS(1855),
    [sym_sinteger_keyword] = ACTIONS(1855),
    [sym_long_keyword] = ACTIONS(1855),
    [sym_slong_keyword] = ACTIONS(1855),
    [sym_float_keyword] = ACTIONS(1855),
    [sym_double_keyword] = ACTIONS(1855),
    [sym_variant_keyword] = ACTIONS(1855),
    [sym_variantarray_keyword] = ACTIONS(1855),
    [sym_local_var_keyword] = ACTIONS(1855),
    [sym_stack_var_keyword] = ACTIONS(1855),
    [sym_constant_keyword] = ACTIONS(1855),
    [sym_volatile_keyword] = ACTIONS(1855),
    [sym_non_volatile_keyword] = ACTIONS(1855),
    [sym_persistent_keyword] = ACTIONS(1855),
    [sym_dev_keyword] = ACTIONS(1855),
    [sym_devlev_keyword] = ACTIONS(1855),
    [sym_devchan_keyword] = ACTIONS(1855),
    [sym_if_keyword] = ACTIONS(1855),
    [sym_else_keyword] = ACTIONS(1855),
    [sym_switch_keyword] = ACTIONS(1855),
    [sym_case_keyword] = ACTIONS(1855),
    [sym_default_keyword] = ACTIONS(1855),
    [sym_while_keyword] = ACTIONS(1855),
    [sym_for_keyword] = ACTIONS(1855),
    [sym_break_keyword] = ACTIONS(1855),
    [sym_continue_keyword] = ACTIONS(1855),
    [sym_return_keyword] = ACTIONS(1855),
    [sym_select_keyword] = ACTIONS(1855),
    [sym_active_keyword] = ACTIONS(1855),
    [sym_struct_keyword] = ACTIONS(1855),
    [sym_structure_keyword] = ACTIONS(1855),
    [sym_button_event_keyword] = ACTIONS(1855),
    [sym_channel_event_keyword] = ACTIONS(1855),
    [sym_level_event_keyword] = ACTIONS(1855),
    [sym_data_event_keyword] = ACTIONS(1855),
    [sym_timeline_event_keyword] = ACTIONS(1855),
    [sym_custom_event_keyword] = ACTIONS(1855),
    [sym_push_keyword] = ACTIONS(1855),
    [sym_release_keyword] = ACTIONS(1855),
    [aux_sym_on_keyword_token1] = ACTIONS(1855),
    [aux_sym_off_keyword_token1] = ACTIONS(1855),
    [sym_send_level_keyword] = ACTIONS(1855),
    [sym_send_string_keyword] = ACTIONS(1855),
    [sym_send_command_keyword] = ACTIONS(1855),
    [sym_clear_buffer_keyword] = ACTIONS(1855),
    [sym_create_buffer_keyword] = ACTIONS(1855),
    [sym_create_multi_buffer_keyword] = ACTIONS(1855),
    [sym_call_keyword] = ACTIONS(1855),
    [sym_system_call_keyword] = ACTIONS(1855),
    [sym_devchan_to_keyword] = ACTIONS(1855),
    [sym_devchan_min_to_keyword] = ACTIONS(1855),
    [sym_devchan_total_off_keyword] = ACTIONS(1855),
    [sym_devchan_pulse_keyword] = ACTIONS(1855),
    [sym_wait_keyword] = ACTIONS(1855),
    [sym_cancel_wait_keyword] = ACTIONS(1855),
    [sym_wait_until_keyword] = ACTIONS(1855),
    [sym_cancel_wait_until_keyword] = ACTIONS(1855),
    [sym_cancel_all_wait_keyword] = ACTIONS(1855),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1855),
    [sym_not] = ACTIONS(1855),
    [sym_bnot] = ACTIONS(1855),
    [sym___file__] = ACTIONS(1855),
    [sym___date__] = ACTIONS(1855),
    [sym___ldate__] = ACTIONS(1855),
    [sym___time__] = ACTIONS(1855),
    [sym___line__] = ACTIONS(1855),
    [sym___name__] = ACTIONS(1855),
    [sym___netlinx__] = ACTIONS(1855),
    [sym_netlinx_axi_version] = ACTIONS(1855),
    [sym_url_flg_tcp] = ACTIONS(1855),
    [sym_url_flg_accinfopresent] = ACTIONS(1855),
    [sym_url_flg_temp] = ACTIONS(1855),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1855),
    [sym_url_flg_stat_mask] = ACTIONS(1855),
    [sym_url_flg_stat_lookup] = ACTIONS(1855),
    [sym_url_flg_stat_connecting] = ACTIONS(1855),
    [sym_url_flg_stat_waiting] = ACTIONS(1855),
    [sym_url_flg_stat_connected] = ACTIONS(1855),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1855),
    [sym_first_virtual_device] = ACTIONS(1855),
    [sym_first_local_port] = ACTIONS(1855),
    [sym_dynamic_virtual_device] = ACTIONS(1855),
    [sym_timeline_once] = ACTIONS(1855),
    [sym_timeline_repeat] = ACTIONS(1855),
    [sym_timeline_absolute] = ACTIONS(1855),
    [sym_timeline_relative] = ACTIONS(1855),
    [sym_do_push_timed_infinite] = ACTIONS(1855),
    [sym_source_type_no_address] = ACTIONS(1855),
    [sym_source_type_neuron_id] = ACTIONS(1855),
    [sym_source_type_ip_address] = ACTIONS(1855),
    [sym_source_type_axlink] = ACTIONS(1855),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1855),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1855),
    [sym_source_type_ip_socket_address] = ACTIONS(1855),
    [sym_source_type_rs232] = ACTIONS(1855),
    [sym_source_type_internal] = ACTIONS(1855),
    [sym_source_type_mac_address] = ACTIONS(1855),
    [sym_source_type_ipv4_port] = ACTIONS(1855),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1855),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1855),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1855),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1855),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1855),
    [sym_true] = ACTIONS(1855),
    [sym_false] = ACTIONS(1855),
    [sym_file_read_only] = ACTIONS(1855),
    [sym_file_rw_new] = ACTIONS(1855),
    [sym_file_rw_append] = ACTIONS(1855),
    [sym_ip_tcp] = ACTIONS(1855),
    [sym_ip_udp] = ACTIONS(1855),
    [sym_ip_udp_2way] = ACTIONS(1855),
    [sym_xml_encode_types] = ACTIONS(1855),
    [sym_xml_encode_char_as_list] = ACTIONS(1855),
    [sym_xml_encode_le] = ACTIONS(1855),
    [sym_xml_decode_types] = ACTIONS(1855),
    [sym_xml_decode_no_preserve] = ACTIONS(1855),
    [sym_duet_dev_type_display_device] = ACTIONS(1855),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1855),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1855),
    [sym_duet_dev_type_io_device] = ACTIONS(1855),
    [sym_duet_dev_type_relay_device] = ACTIONS(1855),
    [sym_duet_dev_type_ups] = ACTIONS(1855),
    [sym_duet_dev_type_amplifier] = ACTIONS(1855),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1855),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1855),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1855),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1855),
    [sym_duet_dev_type_camera] = ACTIONS(1855),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1855),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1855),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1855),
    [sym_duet_dev_type_dss] = ACTIONS(1855),
    [sym_duet_dev_type_dvr] = ACTIONS(1855),
    [sym_duet_dev_type_disc_device] = ACTIONS(1855),
    [sym_duet_dev_type_document_camera] = ACTIONS(1855),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1855),
    [sym_duet_dev_type_hvac] = ACTIONS(1855),
    [sym_duet_dev_type_keypad] = ACTIONS(1855),
    [sym_duet_dev_type_light] = ACTIONS(1855),
    [sym_duet_dev_type_monitor] = ACTIONS(1855),
    [sym_duet_dev_type_motor] = ACTIONS(1855),
    [sym_duet_dev_type_multi_window] = ACTIONS(1855),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1855),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1855),
    [sym_duet_dev_type_receiver] = ACTIONS(1855),
    [sym_duet_dev_type_security_system] = ACTIONS(1855),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1855),
    [sym_duet_dev_type_settop_box] = ACTIONS(1855),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1855),
    [sym_duet_dev_type_switcher] = ACTIONS(1855),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1855),
    [sym_duet_dev_type_tv] = ACTIONS(1855),
    [sym_duet_dev_type_utility] = ACTIONS(1855),
    [sym_duet_dev_type_vcr] = ACTIONS(1855),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1855),
    [sym_duet_dev_type_video_processor] = ACTIONS(1855),
    [sym_duet_dev_type_video_projector] = ACTIONS(1855),
    [sym_duet_dev_type_video_wall] = ACTIONS(1855),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1855),
    [sym_duet_dev_type_weather] = ACTIONS(1855),
    [sym_duet_dev_not_polled] = ACTIONS(1855),
    [sym_duet_dev_polled] = ACTIONS(1855),
    [sym_clkmgr_mode_network] = ACTIONS(1855),
    [sym_clkmgr_mode_standalone] = ACTIONS(1855),
    [sym_normal_standby] = ACTIONS(1855),
    [sym_normal_wake] = ACTIONS(1855),
    [sym_ip_multicast_ttl_option] = ACTIONS(1855),
    [sym_ip_ttl_subnet] = ACTIONS(1855),
    [sym_ip_ttl_site] = ACTIONS(1855),
    [sym_ip_ttl_region] = ACTIONS(1855),
    [sym_ip_ttl_continent] = ACTIONS(1855),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1855),
    [sym_ip_nodelay_on] = ACTIONS(1855),
    [sym_ip_nodelay_off] = ACTIONS(1855),
    [sym_amx_error] = ACTIONS(1855),
    [sym_amx_warning] = ACTIONS(1855),
    [sym_amx_info] = ACTIONS(1855),
    [sym_amx_debug] = ACTIONS(1855),
    [sym_netlinx_login_success] = ACTIONS(1855),
    [sym_netlinx_login_fail] = ACTIONS(1855),
    [sym_netlinx_logout] = ACTIONS(1855),
    [sym_null_str] = ACTIONS(1855),
    [sym_smtp_address] = ACTIONS(1855),
    [sym_smtp_port_number] = ACTIONS(1855),
    [sym_smtp_username] = ACTIONS(1855),
    [sym_smtp_password] = ACTIONS(1855),
    [sym_smtp_require_tls] = ACTIONS(1855),
    [sym_smtp_from] = ACTIONS(1855),
    [sym_smtp_tls_true] = ACTIONS(1855),
    [sym_smtp_tls_false] = ACTIONS(1855),
    [sym_listview_on_row_select_event] = ACTIONS(1855),
    [sym_data_structure_unknown] = ACTIONS(1855),
    [sym_data_structure_datafeed] = ACTIONS(1855),
    [sym_data_structure_datarecordset] = ACTIONS(1855),
    [sym_data_structure_datarecord] = ACTIONS(1855),
    [sym_data_structure_datafield] = ACTIONS(1855),
    [sym_data_max_content_field_count] = ACTIONS(1855),
    [sym_data_max_metadata_field_count] = ACTIONS(1855),
    [sym_data_max_id_length] = ACTIONS(1855),
    [sym_data_max_name_length] = ACTIONS(1855),
    [sym_data_max_description_length] = ACTIONS(1855),
    [sym_data_max_type_length] = ACTIONS(1855),
    [sym_data_max_format_length] = ACTIONS(1855),
    [sym_data_max_source_length] = ACTIONS(1855),
    [sym_data_max_value_length] = ACTIONS(1855),
    [sym_data_max_label_length] = ACTIONS(1855),
    [sym_data_max_url_length] = ACTIONS(1855),
    [sym_max_last_login_info_length] = ACTIONS(1855),
    [sym_data_type_unknown] = ACTIONS(1855),
    [sym_data_type_string] = ACTIONS(1855),
    [sym_data_type_datetime] = ACTIONS(1855),
    [sym_data_type_date] = ACTIONS(1855),
    [sym_data_type_time] = ACTIONS(1855),
    [sym_data_type_image] = ACTIONS(1855),
    [sym_data_format_url] = ACTIONS(1855),
    [sym_data_format_phone] = ACTIONS(1855),
    [sym_data_format_email] = ACTIONS(1855),
    [sym_data_format_iso8601] = ACTIONS(1855),
    [sym_auditlog_priv] = ACTIONS(1855),
    [sym_configuration_priv] = ACTIONS(1855),
    [sym_deviceconfig_priv] = ACTIONS(1855),
    [sym_ftp_priv] = ACTIONS(1855),
    [sym_http_priv] = ACTIONS(1855),
    [sym_networkconfig_priv] = ACTIONS(1855),
    [sym_programport_priv] = ACTIONS(1855),
    [sym_remoteui_priv] = ACTIONS(1855),
    [sym_securitycontrol_priv] = ACTIONS(1855),
    [sym_softwaremanagement_priv] = ACTIONS(1855),
    [sym_terminal_priv] = ACTIONS(1855),
    [sym_tpadmin_priv] = ACTIONS(1855),
    [sym_user1_priv] = ACTIONS(1855),
    [sym_user2_priv] = ACTIONS(1855),
    [sym_user3_priv] = ACTIONS(1855),
    [sym_user4_priv] = ACTIONS(1855),
    [sym_usermanagement_priv] = ACTIONS(1855),
    [sym_valid_account] = ACTIONS(1855),
    [sym_err_name_invalid] = ACTIONS(1855),
    [sym_err_password_invalid] = ACTIONS(1855),
    [sym_err_invalid_account] = ACTIONS(1855),
    [sym_err_no_authorization] = ACTIONS(1855),
    [sym_err_invalid_parameter] = ACTIONS(1855),
    [sym_err_user_locked_out] = ACTIONS(1855),
    [sym_err_user_account_expired] = ACTIONS(1855),
    [sym_err_general_error] = ACTIONS(1855),
    [sym_err_authentication_error] = ACTIONS(1855),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1855),
    [sym_tls_validate_certificate] = ACTIONS(1855),
    [sym_tdata] = ACTIONS(1855),
    [sym_tchannel] = ACTIONS(1855),
    [sym_tlevel] = ACTIONS(1855),
    [sym_tbutton] = ACTIONS(1855),
    [sym_ttimeline] = ACTIONS(1855),
    [sym_tcustom] = ACTIONS(1855),
    [sym_url_struct] = ACTIONS(1855),
    [sym_dns_struct] = ACTIONS(1855),
    [sym_ip_address_struct] = ACTIONS(1855),
    [sym_dev_info_struct] = ACTIONS(1855),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1855),
    [sym_data_feed] = ACTIONS(1855),
    [sym_data_field] = ACTIONS(1855),
    [sym_data_record] = ACTIONS(1855),
    [sym_wc_data_feed] = ACTIONS(1855),
    [sym_wc_data_field] = ACTIONS(1855),
    [sym_wc_data_record] = ACTIONS(1855),
    [sym_last_login_info] = ACTIONS(1855),
    [sym____reserved___] = ACTIONS(1855),
    [sym_date] = ACTIONS(1855),
    [sym_day] = ACTIONS(1855),
    [sym_time] = ACTIONS(1855),
    [sym_ldate] = ACTIONS(1855),
    [sym_push_device] = ACTIONS(1855),
    [sym_push_channel] = ACTIONS(1855),
    [sym_push_devchan] = ACTIONS(1855),
    [sym_release_device] = ACTIONS(1855),
    [sym_release_channel] = ACTIONS(1855),
    [sym_release_devchan] = ACTIONS(1855),
    [sym_master_slot] = ACTIONS(1855),
    [sym_get_pulse_time] = ACTIONS(1855),
    [sym_get_timer] = ACTIONS(1855),
    [sym_master_sn] = ACTIONS(1855),
    [sym_system_number] = ACTIONS(1855),
    [sym_button] = ACTIONS(1855),
    [sym_dv_channel] = ACTIONS(1855),
    [sym_data] = ACTIONS(1855),
    [sym_level] = ACTIONS(1855),
    [sym_timeline] = ACTIONS(1855),
    [sym_channel] = ACTIONS(1855),
    [sym_custom] = ACTIONS(1855),
    [sym_atoi] = ACTIONS(1855),
    [sym_atol] = ACTIONS(1855),
    [sym_atof] = ACTIONS(1855),
    [sym_itoa] = ACTIONS(1855),
    [sym_itohex] = ACTIONS(1855),
    [sym_hextoi] = ACTIONS(1855),
    [sym_ftoa] = ACTIONS(1855),
    [sym_find_string] = ACTIONS(1855),
    [sym_left_string] = ACTIONS(1855),
    [sym_right_string] = ACTIONS(1855),
    [sym_mid_string] = ACTIONS(1855),
    [sym_remove_string] = ACTIONS(1855),
    [sym_get_buffer_string] = ACTIONS(1855),
    [sym_get_buffer_char] = ACTIONS(1855),
    [sym_get_multi_buffer_string] = ACTIONS(1855),
    [sym_length_string] = ACTIONS(1855),
    [sym_max_length_string] = ACTIONS(1855),
    [sym_set_length_string] = ACTIONS(1855),
    [sym_compare_string] = ACTIONS(1855),
    [sym_upper_string] = ACTIONS(1855),
    [sym_lower_string] = ACTIONS(1855),
    [sym_get_last] = ACTIONS(1855),
    [sym_date_to_day] = ACTIONS(1855),
    [sym_date_to_month] = ACTIONS(1855),
    [sym_date_to_year] = ACTIONS(1855),
    [sym_time_to_hour] = ACTIONS(1855),
    [sym_time_to_minute] = ACTIONS(1855),
    [sym_time_to_second] = ACTIONS(1855),
    [sym_day_of_week] = ACTIONS(1855),
    [sym_set_pulse_time] = ACTIONS(1855),
    [sym_set_timer] = ACTIONS(1855),
    [sym_file_open] = ACTIONS(1855),
    [sym_file_close] = ACTIONS(1855),
    [sym_file_read] = ACTIONS(1855),
    [sym_file_read_line] = ACTIONS(1855),
    [sym_file_write] = ACTIONS(1855),
    [sym_file_write_line] = ACTIONS(1855),
    [sym_file_copy] = ACTIONS(1855),
    [sym_file_delete] = ACTIONS(1855),
    [sym_file_rename] = ACTIONS(1855),
    [sym_file_dir] = ACTIONS(1855),
    [sym_file_getdir] = ACTIONS(1855),
    [sym_file_setdir] = ACTIONS(1855),
    [sym_file_create_dir] = ACTIONS(1855),
    [sym_file_remove_dir] = ACTIONS(1855),
    [sym_file_seek] = ACTIONS(1855),
    [sym_length_array] = ACTIONS(1855),
    [sym_max_length_array] = ACTIONS(1855),
    [sym_set_length_array] = ACTIONS(1855),
    [sym_ip_client_open] = ACTIONS(1855),
    [sym_ip_client_close] = ACTIONS(1855),
    [sym_ip_server_open] = ACTIONS(1855),
    [sym_ip_server_close] = ACTIONS(1855),
    [sym_ip_bound_client_open] = ACTIONS(1855),
    [sym_ip_mc_server_open] = ACTIONS(1855),
    [sym_ip_set_option] = ACTIONS(1855),
    [sym_ssh_client_open] = ACTIONS(1855),
    [sym_ssh_client_close] = ACTIONS(1855),
    [sym_tls_client_open] = ACTIONS(1855),
    [sym_tls_client_close] = ACTIONS(1855),
    [sym_do_push] = ACTIONS(1855),
    [sym_do_push_timed] = ACTIONS(1855),
    [sym_do_release] = ACTIONS(1855),
    [sym_device_id] = ACTIONS(1855),
    [sym_device_info] = ACTIONS(1855),
    [sym_device_id_string] = ACTIONS(1855),
    [sym_device_standby] = ACTIONS(1855),
    [sym_device_wake] = ACTIONS(1855),
    [sym_timeline_create] = ACTIONS(1855),
    [sym_timeline_kill] = ACTIONS(1855),
    [sym_timeline_reload] = ACTIONS(1855),
    [sym_timeline_pause] = ACTIONS(1855),
    [sym_timeline_restart] = ACTIONS(1855),
    [sym_timeline_set] = ACTIONS(1855),
    [sym_timeline_get] = ACTIONS(1855),
    [sym_timeline_active] = ACTIONS(1855),
    [sym_timeline_dynamic_id] = ACTIONS(1855),
    [sym_variable_to_string] = ACTIONS(1855),
    [sym_string_to_variable] = ACTIONS(1855),
    [sym_length_variable_to_string] = ACTIONS(1855),
    [sym_type_cast] = ACTIONS(1855),
    [sym_variable_to_xml] = ACTIONS(1855),
    [sym_xml_to_variable] = ACTIONS(1855),
    [sym_length_variable_to_xml] = ACTIONS(1855),
    [sym_redirect_string] = ACTIONS(1855),
    [sym_random_number] = ACTIONS(1855),
    [sym_get_unique_id] = ACTIONS(1855),
    [sym_get_system_number] = ACTIONS(1855),
    [sym_set_system_number] = ACTIONS(1855),
    [sym_get_serial_number] = ACTIONS(1855),
    [sym_get_master_build] = ACTIONS(1855),
    [sym_set_validation_code] = ACTIONS(1855),
    [sym_reboot] = ACTIONS(1855),
    [sym_load_duet_module] = ACTIONS(1855),
    [sym_unload_duet_module] = ACTIONS(1855),
    [sym_duet_mem_size_set] = ACTIONS(1855),
    [sym_duet_mem_size_get] = ACTIONS(1855),
    [sym_dynamic_application_device] = ACTIONS(1855),
    [sym_dynamic_polled_port] = ACTIONS(1855),
    [sym_static_port_binding] = ACTIONS(1855),
    [sym_static_ip_binding] = ACTIONS(1855),
    [sym_get_dns_list] = ACTIONS(1855),
    [sym_set_dns_list] = ACTIONS(1855),
    [sym_get_ip_address] = ACTIONS(1855),
    [sym_set_ip_address] = ACTIONS(1855),
    [sym_get_url_list] = ACTIONS(1855),
    [sym_add_url_entry] = ACTIONS(1855),
    [sym_delete_url_entry] = ACTIONS(1855),
    [sym_abs_value] = ACTIONS(1855),
    [sym_max_value] = ACTIONS(1855),
    [sym_min_value] = ACTIONS(1855),
    [sym_exp_value] = ACTIONS(1855),
    [sym_log_value] = ACTIONS(1855),
    [sym_log10_value] = ACTIONS(1855),
    [sym_power_value] = ACTIONS(1855),
    [sym_sqrt_value] = ACTIONS(1855),
    [sym_raw_be] = ACTIONS(1855),
    [sym_raw_le] = ACTIONS(1855),
    [sym_internal_queue_size_set] = ACTIONS(1855),
    [sym_internal_queue_size_get] = ACTIONS(1855),
    [sym_internal_threshold_set] = ACTIONS(1855),
    [sym_internal_threshold_get] = ACTIONS(1855),
    [sym_rebuilt_event] = ACTIONS(1855),
    [sym_do_custom_event] = ACTIONS(1855),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1855),
    [sym_clkmgr_set_clk_source] = ACTIONS(1855),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1855),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1855),
    [sym_clkmgr_get_timezone] = ACTIONS(1855),
    [sym_clkmgr_set_timezone] = ACTIONS(1855),
    [sym_clkmgr_get_resync_period] = ACTIONS(1855),
    [sym_clkmgr_set_resync_period] = ACTIONS(1855),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1855),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1855),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1855),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1855),
    [sym_clkmgr_get_timeservers] = ACTIONS(1855),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1855),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1855),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1855),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1855),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1855),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1855),
    [sym_set_log_level] = ACTIONS(1855),
    [sym_get_log_level] = ACTIONS(1855),
    [sym_amx_log] = ACTIONS(1855),
    [sym_smtp_server_config_set] = ACTIONS(1855),
    [sym_smtp_server_config_get] = ACTIONS(1855),
    [sym_smtp_send] = ACTIONS(1855),
    [sym_validate_netlinx_account] = ACTIONS(1855),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1855),
    [sym_authenticate_certificate] = ACTIONS(1855),
    [sym_audit_netlinx_session_event] = ACTIONS(1855),
    [sym_audit_netlinx_generic_event] = ACTIONS(1855),
    [sym_get_available_flash_disk_space] = ACTIONS(1855),
    [sym_get_max_flash_disk_space] = ACTIONS(1855),
    [sym_data_create_feed] = ACTIONS(1855),
    [sym_data_delete_feed] = ACTIONS(1855),
    [sym_data_publish_feed] = ACTIONS(1855),
    [sym_data_get_published_feed] = ACTIONS(1855),
    [sym_data_add_record] = ACTIONS(1855),
    [sym_data_get_event_record] = ACTIONS(1855),
    [sym__wc_data_create_feed] = ACTIONS(1855),
    [sym__wc_data_add_record] = ACTIONS(1855),
    [sym__wc_data_get_event_record] = ACTIONS(1855),
    [sym_format] = ACTIONS(1855),
    [sym_astro_clock] = ACTIONS(1855),
    [sym_set_outdoor_temperature] = ACTIONS(1855),
    [sym_set_virtual_level_count] = ACTIONS(1855),
    [sym_set_virtual_channel_count] = ACTIONS(1855),
    [sym_set_virtual_port_count] = ACTIONS(1855),
    [sym__wc_encode_int] = ACTIONS(1855),
    [sym__wc_decode_int] = ACTIONS(1855),
    [sym__wc_int] = ACTIONS(1855),
    [sym__wc_to_ch_int] = ACTIONS(1855),
    [sym__ch_to_wc_int] = ACTIONS(1855),
    [sym__wc_find_string_int] = ACTIONS(1855),
    [sym__wc_left_string_int] = ACTIONS(1855),
    [sym__wc_lower_string_int] = ACTIONS(1855),
    [sym__wc_mid_string_int] = ACTIONS(1855),
    [sym__wc_remove_string_int] = ACTIONS(1855),
    [sym__wc_right_string_int] = ACTIONS(1855),
    [sym__wc_upper_string_int] = ACTIONS(1855),
    [sym__wc_compare_string_int] = ACTIONS(1855),
    [sym__wc_get_buffer_char_int] = ACTIONS(1855),
    [sym__wc_get_buffer_string_int] = ACTIONS(1855),
    [sym__wc_concat_string_int] = ACTIONS(1855),
    [sym__wc_file_open_int] = ACTIONS(1855),
    [sym__wc_file_close_int] = ACTIONS(1855),
    [sym__wc_file_read_int] = ACTIONS(1855),
    [sym__wc_file_read_line_int] = ACTIONS(1855),
    [sym__wc_file_write_int] = ACTIONS(1855),
    [sym__wc_file_write_line_int] = ACTIONS(1855),
    [aux_sym_preproc_include_token1] = ACTIONS(1855),
    [aux_sym_preproc_define_token1] = ACTIONS(1857),
    [aux_sym_preproc_warn_token1] = ACTIONS(1857),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1857),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1857),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1857),
    [anon_sym_LBRACK] = ACTIONS(1857),
    [anon_sym_LBRACE] = ACTIONS(1857),
    [anon_sym_LPAREN] = ACTIONS(1855),
    [anon_sym_BANG] = ACTIONS(1857),
    [anon_sym_TILDE] = ACTIONS(1857),
    [anon_sym_DASH] = ACTIONS(1855),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_DASH_DASH] = ACTIONS(1857),
    [anon_sym_PLUS_PLUS] = ACTIONS(1857),
    [anon_sym_DQUOTE] = ACTIONS(1857),
    [anon_sym_SQUOTE] = ACTIONS(1857),
    [sym_number_literal] = ACTIONS(1857),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(269)] = {
    [sym_identifier] = ACTIONS(1883),
    [sym_program_name_keyword] = ACTIONS(1883),
    [sym_module_name_keyword] = ACTIONS(1883),
    [sym_define_device_keyword] = ACTIONS(1883),
    [sym_define_combine_keyword] = ACTIONS(1883),
    [sym_define_constant_keyword] = ACTIONS(1883),
    [sym_define_type_keyword] = ACTIONS(1883),
    [sym_define_variable_keyword] = ACTIONS(1883),
    [sym_define_system_variable_keyword] = ACTIONS(1883),
    [sym_define_connect_level_keyword] = ACTIONS(1883),
    [sym_define_latching_keyword] = ACTIONS(1883),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1883),
    [sym_define_toggling_keyword] = ACTIONS(1883),
    [sym_define_call_keyword] = ACTIONS(1883),
    [sym_define_function_keyword] = ACTIONS(1883),
    [sym_define_library_function_keyword] = ACTIONS(1883),
    [sym_define_start_keyword] = ACTIONS(1883),
    [sym_define_event_keyword] = ACTIONS(1883),
    [sym_define_program_keyword] = ACTIONS(1883),
    [sym_define_module_keyword] = ACTIONS(1883),
    [sym_char_keyword] = ACTIONS(1883),
    [sym_widechar_keyword] = ACTIONS(1883),
    [sym_integer_keyword] = ACTIONS(1883),
    [sym_sinteger_keyword] = ACTIONS(1883),
    [sym_long_keyword] = ACTIONS(1883),
    [sym_slong_keyword] = ACTIONS(1883),
    [sym_float_keyword] = ACTIONS(1883),
    [sym_double_keyword] = ACTIONS(1883),
    [sym_variant_keyword] = ACTIONS(1883),
    [sym_variantarray_keyword] = ACTIONS(1883),
    [sym_local_var_keyword] = ACTIONS(1883),
    [sym_stack_var_keyword] = ACTIONS(1883),
    [sym_constant_keyword] = ACTIONS(1883),
    [sym_volatile_keyword] = ACTIONS(1883),
    [sym_non_volatile_keyword] = ACTIONS(1883),
    [sym_persistent_keyword] = ACTIONS(1883),
    [sym_dev_keyword] = ACTIONS(1883),
    [sym_devlev_keyword] = ACTIONS(1883),
    [sym_devchan_keyword] = ACTIONS(1883),
    [sym_if_keyword] = ACTIONS(1883),
    [sym_switch_keyword] = ACTIONS(1883),
    [sym_case_keyword] = ACTIONS(1883),
    [sym_default_keyword] = ACTIONS(1883),
    [sym_while_keyword] = ACTIONS(1883),
    [sym_for_keyword] = ACTIONS(1883),
    [sym_break_keyword] = ACTIONS(1883),
    [sym_continue_keyword] = ACTIONS(1883),
    [sym_return_keyword] = ACTIONS(1883),
    [sym_select_keyword] = ACTIONS(1883),
    [sym_active_keyword] = ACTIONS(1883),
    [sym_struct_keyword] = ACTIONS(1883),
    [sym_structure_keyword] = ACTIONS(1883),
    [sym_button_event_keyword] = ACTIONS(1883),
    [sym_channel_event_keyword] = ACTIONS(1883),
    [sym_level_event_keyword] = ACTIONS(1883),
    [sym_data_event_keyword] = ACTIONS(1883),
    [sym_timeline_event_keyword] = ACTIONS(1883),
    [sym_custom_event_keyword] = ACTIONS(1883),
    [sym_push_keyword] = ACTIONS(1883),
    [sym_release_keyword] = ACTIONS(1883),
    [aux_sym_on_keyword_token1] = ACTIONS(1883),
    [aux_sym_off_keyword_token1] = ACTIONS(1883),
    [sym_send_level_keyword] = ACTIONS(1883),
    [sym_send_string_keyword] = ACTIONS(1883),
    [sym_send_command_keyword] = ACTIONS(1883),
    [sym_clear_buffer_keyword] = ACTIONS(1883),
    [sym_create_buffer_keyword] = ACTIONS(1883),
    [sym_create_multi_buffer_keyword] = ACTIONS(1883),
    [sym_call_keyword] = ACTIONS(1883),
    [sym_system_call_keyword] = ACTIONS(1883),
    [sym_devchan_to_keyword] = ACTIONS(1883),
    [sym_devchan_min_to_keyword] = ACTIONS(1883),
    [sym_devchan_total_off_keyword] = ACTIONS(1883),
    [sym_devchan_pulse_keyword] = ACTIONS(1883),
    [sym_wait_keyword] = ACTIONS(1883),
    [sym_cancel_wait_keyword] = ACTIONS(1883),
    [sym_wait_until_keyword] = ACTIONS(1883),
    [sym_cancel_wait_until_keyword] = ACTIONS(1883),
    [sym_cancel_all_wait_keyword] = ACTIONS(1883),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1883),
    [sym_not] = ACTIONS(1883),
    [sym_bnot] = ACTIONS(1883),
    [sym___file__] = ACTIONS(1883),
    [sym___date__] = ACTIONS(1883),
    [sym___ldate__] = ACTIONS(1883),
    [sym___time__] = ACTIONS(1883),
    [sym___line__] = ACTIONS(1883),
    [sym___name__] = ACTIONS(1883),
    [sym___netlinx__] = ACTIONS(1883),
    [sym_netlinx_axi_version] = ACTIONS(1883),
    [sym_url_flg_tcp] = ACTIONS(1883),
    [sym_url_flg_accinfopresent] = ACTIONS(1883),
    [sym_url_flg_temp] = ACTIONS(1883),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1883),
    [sym_url_flg_stat_mask] = ACTIONS(1883),
    [sym_url_flg_stat_lookup] = ACTIONS(1883),
    [sym_url_flg_stat_connecting] = ACTIONS(1883),
    [sym_url_flg_stat_waiting] = ACTIONS(1883),
    [sym_url_flg_stat_connected] = ACTIONS(1883),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1883),
    [sym_first_virtual_device] = ACTIONS(1883),
    [sym_first_local_port] = ACTIONS(1883),
    [sym_dynamic_virtual_device] = ACTIONS(1883),
    [sym_timeline_once] = ACTIONS(1883),
    [sym_timeline_repeat] = ACTIONS(1883),
    [sym_timeline_absolute] = ACTIONS(1883),
    [sym_timeline_relative] = ACTIONS(1883),
    [sym_do_push_timed_infinite] = ACTIONS(1883),
    [sym_source_type_no_address] = ACTIONS(1883),
    [sym_source_type_neuron_id] = ACTIONS(1883),
    [sym_source_type_ip_address] = ACTIONS(1883),
    [sym_source_type_axlink] = ACTIONS(1883),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1883),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1883),
    [sym_source_type_ip_socket_address] = ACTIONS(1883),
    [sym_source_type_rs232] = ACTIONS(1883),
    [sym_source_type_internal] = ACTIONS(1883),
    [sym_source_type_mac_address] = ACTIONS(1883),
    [sym_source_type_ipv4_port] = ACTIONS(1883),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1883),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1883),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1883),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1883),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1883),
    [sym_true] = ACTIONS(1883),
    [sym_false] = ACTIONS(1883),
    [sym_file_read_only] = ACTIONS(1883),
    [sym_file_rw_new] = ACTIONS(1883),
    [sym_file_rw_append] = ACTIONS(1883),
    [sym_ip_tcp] = ACTIONS(1883),
    [sym_ip_udp] = ACTIONS(1883),
    [sym_ip_udp_2way] = ACTIONS(1883),
    [sym_xml_encode_types] = ACTIONS(1883),
    [sym_xml_encode_char_as_list] = ACTIONS(1883),
    [sym_xml_encode_le] = ACTIONS(1883),
    [sym_xml_decode_types] = ACTIONS(1883),
    [sym_xml_decode_no_preserve] = ACTIONS(1883),
    [sym_duet_dev_type_display_device] = ACTIONS(1883),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1883),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1883),
    [sym_duet_dev_type_io_device] = ACTIONS(1883),
    [sym_duet_dev_type_relay_device] = ACTIONS(1883),
    [sym_duet_dev_type_ups] = ACTIONS(1883),
    [sym_duet_dev_type_amplifier] = ACTIONS(1883),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1883),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1883),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1883),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1883),
    [sym_duet_dev_type_camera] = ACTIONS(1883),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1883),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1883),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1883),
    [sym_duet_dev_type_dss] = ACTIONS(1883),
    [sym_duet_dev_type_dvr] = ACTIONS(1883),
    [sym_duet_dev_type_disc_device] = ACTIONS(1883),
    [sym_duet_dev_type_document_camera] = ACTIONS(1883),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1883),
    [sym_duet_dev_type_hvac] = ACTIONS(1883),
    [sym_duet_dev_type_keypad] = ACTIONS(1883),
    [sym_duet_dev_type_light] = ACTIONS(1883),
    [sym_duet_dev_type_monitor] = ACTIONS(1883),
    [sym_duet_dev_type_motor] = ACTIONS(1883),
    [sym_duet_dev_type_multi_window] = ACTIONS(1883),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1883),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1883),
    [sym_duet_dev_type_receiver] = ACTIONS(1883),
    [sym_duet_dev_type_security_system] = ACTIONS(1883),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1883),
    [sym_duet_dev_type_settop_box] = ACTIONS(1883),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1883),
    [sym_duet_dev_type_switcher] = ACTIONS(1883),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1883),
    [sym_duet_dev_type_tv] = ACTIONS(1883),
    [sym_duet_dev_type_utility] = ACTIONS(1883),
    [sym_duet_dev_type_vcr] = ACTIONS(1883),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1883),
    [sym_duet_dev_type_video_processor] = ACTIONS(1883),
    [sym_duet_dev_type_video_projector] = ACTIONS(1883),
    [sym_duet_dev_type_video_wall] = ACTIONS(1883),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1883),
    [sym_duet_dev_type_weather] = ACTIONS(1883),
    [sym_duet_dev_not_polled] = ACTIONS(1883),
    [sym_duet_dev_polled] = ACTIONS(1883),
    [sym_clkmgr_mode_network] = ACTIONS(1883),
    [sym_clkmgr_mode_standalone] = ACTIONS(1883),
    [sym_normal_standby] = ACTIONS(1883),
    [sym_normal_wake] = ACTIONS(1883),
    [sym_ip_multicast_ttl_option] = ACTIONS(1883),
    [sym_ip_ttl_subnet] = ACTIONS(1883),
    [sym_ip_ttl_site] = ACTIONS(1883),
    [sym_ip_ttl_region] = ACTIONS(1883),
    [sym_ip_ttl_continent] = ACTIONS(1883),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1883),
    [sym_ip_nodelay_on] = ACTIONS(1883),
    [sym_ip_nodelay_off] = ACTIONS(1883),
    [sym_amx_error] = ACTIONS(1883),
    [sym_amx_warning] = ACTIONS(1883),
    [sym_amx_info] = ACTIONS(1883),
    [sym_amx_debug] = ACTIONS(1883),
    [sym_netlinx_login_success] = ACTIONS(1883),
    [sym_netlinx_login_fail] = ACTIONS(1883),
    [sym_netlinx_logout] = ACTIONS(1883),
    [sym_null_str] = ACTIONS(1883),
    [sym_smtp_address] = ACTIONS(1883),
    [sym_smtp_port_number] = ACTIONS(1883),
    [sym_smtp_username] = ACTIONS(1883),
    [sym_smtp_password] = ACTIONS(1883),
    [sym_smtp_require_tls] = ACTIONS(1883),
    [sym_smtp_from] = ACTIONS(1883),
    [sym_smtp_tls_true] = ACTIONS(1883),
    [sym_smtp_tls_false] = ACTIONS(1883),
    [sym_listview_on_row_select_event] = ACTIONS(1883),
    [sym_data_structure_unknown] = ACTIONS(1883),
    [sym_data_structure_datafeed] = ACTIONS(1883),
    [sym_data_structure_datarecordset] = ACTIONS(1883),
    [sym_data_structure_datarecord] = ACTIONS(1883),
    [sym_data_structure_datafield] = ACTIONS(1883),
    [sym_data_max_content_field_count] = ACTIONS(1883),
    [sym_data_max_metadata_field_count] = ACTIONS(1883),
    [sym_data_max_id_length] = ACTIONS(1883),
    [sym_data_max_name_length] = ACTIONS(1883),
    [sym_data_max_description_length] = ACTIONS(1883),
    [sym_data_max_type_length] = ACTIONS(1883),
    [sym_data_max_format_length] = ACTIONS(1883),
    [sym_data_max_source_length] = ACTIONS(1883),
    [sym_data_max_value_length] = ACTIONS(1883),
    [sym_data_max_label_length] = ACTIONS(1883),
    [sym_data_max_url_length] = ACTIONS(1883),
    [sym_max_last_login_info_length] = ACTIONS(1883),
    [sym_data_type_unknown] = ACTIONS(1883),
    [sym_data_type_string] = ACTIONS(1883),
    [sym_data_type_datetime] = ACTIONS(1883),
    [sym_data_type_date] = ACTIONS(1883),
    [sym_data_type_time] = ACTIONS(1883),
    [sym_data_type_image] = ACTIONS(1883),
    [sym_data_format_url] = ACTIONS(1883),
    [sym_data_format_phone] = ACTIONS(1883),
    [sym_data_format_email] = ACTIONS(1883),
    [sym_data_format_iso8601] = ACTIONS(1883),
    [sym_auditlog_priv] = ACTIONS(1883),
    [sym_configuration_priv] = ACTIONS(1883),
    [sym_deviceconfig_priv] = ACTIONS(1883),
    [sym_ftp_priv] = ACTIONS(1883),
    [sym_http_priv] = ACTIONS(1883),
    [sym_networkconfig_priv] = ACTIONS(1883),
    [sym_programport_priv] = ACTIONS(1883),
    [sym_remoteui_priv] = ACTIONS(1883),
    [sym_securitycontrol_priv] = ACTIONS(1883),
    [sym_softwaremanagement_priv] = ACTIONS(1883),
    [sym_terminal_priv] = ACTIONS(1883),
    [sym_tpadmin_priv] = ACTIONS(1883),
    [sym_user1_priv] = ACTIONS(1883),
    [sym_user2_priv] = ACTIONS(1883),
    [sym_user3_priv] = ACTIONS(1883),
    [sym_user4_priv] = ACTIONS(1883),
    [sym_usermanagement_priv] = ACTIONS(1883),
    [sym_valid_account] = ACTIONS(1883),
    [sym_err_name_invalid] = ACTIONS(1883),
    [sym_err_password_invalid] = ACTIONS(1883),
    [sym_err_invalid_account] = ACTIONS(1883),
    [sym_err_no_authorization] = ACTIONS(1883),
    [sym_err_invalid_parameter] = ACTIONS(1883),
    [sym_err_user_locked_out] = ACTIONS(1883),
    [sym_err_user_account_expired] = ACTIONS(1883),
    [sym_err_general_error] = ACTIONS(1883),
    [sym_err_authentication_error] = ACTIONS(1883),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1883),
    [sym_tls_validate_certificate] = ACTIONS(1883),
    [sym_tdata] = ACTIONS(1883),
    [sym_tchannel] = ACTIONS(1883),
    [sym_tlevel] = ACTIONS(1883),
    [sym_tbutton] = ACTIONS(1883),
    [sym_ttimeline] = ACTIONS(1883),
    [sym_tcustom] = ACTIONS(1883),
    [sym_url_struct] = ACTIONS(1883),
    [sym_dns_struct] = ACTIONS(1883),
    [sym_ip_address_struct] = ACTIONS(1883),
    [sym_dev_info_struct] = ACTIONS(1883),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1883),
    [sym_data_feed] = ACTIONS(1883),
    [sym_data_field] = ACTIONS(1883),
    [sym_data_record] = ACTIONS(1883),
    [sym_wc_data_feed] = ACTIONS(1883),
    [sym_wc_data_field] = ACTIONS(1883),
    [sym_wc_data_record] = ACTIONS(1883),
    [sym_last_login_info] = ACTIONS(1883),
    [sym____reserved___] = ACTIONS(1883),
    [sym_date] = ACTIONS(1883),
    [sym_day] = ACTIONS(1883),
    [sym_time] = ACTIONS(1883),
    [sym_ldate] = ACTIONS(1883),
    [sym_push_device] = ACTIONS(1883),
    [sym_push_channel] = ACTIONS(1883),
    [sym_push_devchan] = ACTIONS(1883),
    [sym_release_device] = ACTIONS(1883),
    [sym_release_channel] = ACTIONS(1883),
    [sym_release_devchan] = ACTIONS(1883),
    [sym_master_slot] = ACTIONS(1883),
    [sym_get_pulse_time] = ACTIONS(1883),
    [sym_get_timer] = ACTIONS(1883),
    [sym_master_sn] = ACTIONS(1883),
    [sym_system_number] = ACTIONS(1883),
    [sym_button] = ACTIONS(1883),
    [sym_dv_channel] = ACTIONS(1883),
    [sym_data] = ACTIONS(1883),
    [sym_level] = ACTIONS(1883),
    [sym_timeline] = ACTIONS(1883),
    [sym_channel] = ACTIONS(1883),
    [sym_custom] = ACTIONS(1883),
    [sym_atoi] = ACTIONS(1883),
    [sym_atol] = ACTIONS(1883),
    [sym_atof] = ACTIONS(1883),
    [sym_itoa] = ACTIONS(1883),
    [sym_itohex] = ACTIONS(1883),
    [sym_hextoi] = ACTIONS(1883),
    [sym_ftoa] = ACTIONS(1883),
    [sym_find_string] = ACTIONS(1883),
    [sym_left_string] = ACTIONS(1883),
    [sym_right_string] = ACTIONS(1883),
    [sym_mid_string] = ACTIONS(1883),
    [sym_remove_string] = ACTIONS(1883),
    [sym_get_buffer_string] = ACTIONS(1883),
    [sym_get_buffer_char] = ACTIONS(1883),
    [sym_get_multi_buffer_string] = ACTIONS(1883),
    [sym_length_string] = ACTIONS(1883),
    [sym_max_length_string] = ACTIONS(1883),
    [sym_set_length_string] = ACTIONS(1883),
    [sym_compare_string] = ACTIONS(1883),
    [sym_upper_string] = ACTIONS(1883),
    [sym_lower_string] = ACTIONS(1883),
    [sym_get_last] = ACTIONS(1883),
    [sym_date_to_day] = ACTIONS(1883),
    [sym_date_to_month] = ACTIONS(1883),
    [sym_date_to_year] = ACTIONS(1883),
    [sym_time_to_hour] = ACTIONS(1883),
    [sym_time_to_minute] = ACTIONS(1883),
    [sym_time_to_second] = ACTIONS(1883),
    [sym_day_of_week] = ACTIONS(1883),
    [sym_set_pulse_time] = ACTIONS(1883),
    [sym_set_timer] = ACTIONS(1883),
    [sym_file_open] = ACTIONS(1883),
    [sym_file_close] = ACTIONS(1883),
    [sym_file_read] = ACTIONS(1883),
    [sym_file_read_line] = ACTIONS(1883),
    [sym_file_write] = ACTIONS(1883),
    [sym_file_write_line] = ACTIONS(1883),
    [sym_file_copy] = ACTIONS(1883),
    [sym_file_delete] = ACTIONS(1883),
    [sym_file_rename] = ACTIONS(1883),
    [sym_file_dir] = ACTIONS(1883),
    [sym_file_getdir] = ACTIONS(1883),
    [sym_file_setdir] = ACTIONS(1883),
    [sym_file_create_dir] = ACTIONS(1883),
    [sym_file_remove_dir] = ACTIONS(1883),
    [sym_file_seek] = ACTIONS(1883),
    [sym_length_array] = ACTIONS(1883),
    [sym_max_length_array] = ACTIONS(1883),
    [sym_set_length_array] = ACTIONS(1883),
    [sym_ip_client_open] = ACTIONS(1883),
    [sym_ip_client_close] = ACTIONS(1883),
    [sym_ip_server_open] = ACTIONS(1883),
    [sym_ip_server_close] = ACTIONS(1883),
    [sym_ip_bound_client_open] = ACTIONS(1883),
    [sym_ip_mc_server_open] = ACTIONS(1883),
    [sym_ip_set_option] = ACTIONS(1883),
    [sym_ssh_client_open] = ACTIONS(1883),
    [sym_ssh_client_close] = ACTIONS(1883),
    [sym_tls_client_open] = ACTIONS(1883),
    [sym_tls_client_close] = ACTIONS(1883),
    [sym_do_push] = ACTIONS(1883),
    [sym_do_push_timed] = ACTIONS(1883),
    [sym_do_release] = ACTIONS(1883),
    [sym_device_id] = ACTIONS(1883),
    [sym_device_info] = ACTIONS(1883),
    [sym_device_id_string] = ACTIONS(1883),
    [sym_device_standby] = ACTIONS(1883),
    [sym_device_wake] = ACTIONS(1883),
    [sym_timeline_create] = ACTIONS(1883),
    [sym_timeline_kill] = ACTIONS(1883),
    [sym_timeline_reload] = ACTIONS(1883),
    [sym_timeline_pause] = ACTIONS(1883),
    [sym_timeline_restart] = ACTIONS(1883),
    [sym_timeline_set] = ACTIONS(1883),
    [sym_timeline_get] = ACTIONS(1883),
    [sym_timeline_active] = ACTIONS(1883),
    [sym_timeline_dynamic_id] = ACTIONS(1883),
    [sym_variable_to_string] = ACTIONS(1883),
    [sym_string_to_variable] = ACTIONS(1883),
    [sym_length_variable_to_string] = ACTIONS(1883),
    [sym_type_cast] = ACTIONS(1883),
    [sym_variable_to_xml] = ACTIONS(1883),
    [sym_xml_to_variable] = ACTIONS(1883),
    [sym_length_variable_to_xml] = ACTIONS(1883),
    [sym_redirect_string] = ACTIONS(1883),
    [sym_random_number] = ACTIONS(1883),
    [sym_get_unique_id] = ACTIONS(1883),
    [sym_get_system_number] = ACTIONS(1883),
    [sym_set_system_number] = ACTIONS(1883),
    [sym_get_serial_number] = ACTIONS(1883),
    [sym_get_master_build] = ACTIONS(1883),
    [sym_set_validation_code] = ACTIONS(1883),
    [sym_reboot] = ACTIONS(1883),
    [sym_load_duet_module] = ACTIONS(1883),
    [sym_unload_duet_module] = ACTIONS(1883),
    [sym_duet_mem_size_set] = ACTIONS(1883),
    [sym_duet_mem_size_get] = ACTIONS(1883),
    [sym_dynamic_application_device] = ACTIONS(1883),
    [sym_dynamic_polled_port] = ACTIONS(1883),
    [sym_static_port_binding] = ACTIONS(1883),
    [sym_static_ip_binding] = ACTIONS(1883),
    [sym_get_dns_list] = ACTIONS(1883),
    [sym_set_dns_list] = ACTIONS(1883),
    [sym_get_ip_address] = ACTIONS(1883),
    [sym_set_ip_address] = ACTIONS(1883),
    [sym_get_url_list] = ACTIONS(1883),
    [sym_add_url_entry] = ACTIONS(1883),
    [sym_delete_url_entry] = ACTIONS(1883),
    [sym_abs_value] = ACTIONS(1883),
    [sym_max_value] = ACTIONS(1883),
    [sym_min_value] = ACTIONS(1883),
    [sym_exp_value] = ACTIONS(1883),
    [sym_log_value] = ACTIONS(1883),
    [sym_log10_value] = ACTIONS(1883),
    [sym_power_value] = ACTIONS(1883),
    [sym_sqrt_value] = ACTIONS(1883),
    [sym_raw_be] = ACTIONS(1883),
    [sym_raw_le] = ACTIONS(1883),
    [sym_internal_queue_size_set] = ACTIONS(1883),
    [sym_internal_queue_size_get] = ACTIONS(1883),
    [sym_internal_threshold_set] = ACTIONS(1883),
    [sym_internal_threshold_get] = ACTIONS(1883),
    [sym_rebuilt_event] = ACTIONS(1883),
    [sym_do_custom_event] = ACTIONS(1883),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1883),
    [sym_clkmgr_set_clk_source] = ACTIONS(1883),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1883),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1883),
    [sym_clkmgr_get_timezone] = ACTIONS(1883),
    [sym_clkmgr_set_timezone] = ACTIONS(1883),
    [sym_clkmgr_get_resync_period] = ACTIONS(1883),
    [sym_clkmgr_set_resync_period] = ACTIONS(1883),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1883),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1883),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1883),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1883),
    [sym_clkmgr_get_timeservers] = ACTIONS(1883),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1883),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1883),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1883),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1883),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1883),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1883),
    [sym_set_log_level] = ACTIONS(1883),
    [sym_get_log_level] = ACTIONS(1883),
    [sym_amx_log] = ACTIONS(1883),
    [sym_smtp_server_config_set] = ACTIONS(1883),
    [sym_smtp_server_config_get] = ACTIONS(1883),
    [sym_smtp_send] = ACTIONS(1883),
    [sym_validate_netlinx_account] = ACTIONS(1883),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1883),
    [sym_authenticate_certificate] = ACTIONS(1883),
    [sym_audit_netlinx_session_event] = ACTIONS(1883),
    [sym_audit_netlinx_generic_event] = ACTIONS(1883),
    [sym_get_available_flash_disk_space] = ACTIONS(1883),
    [sym_get_max_flash_disk_space] = ACTIONS(1883),
    [sym_data_create_feed] = ACTIONS(1883),
    [sym_data_delete_feed] = ACTIONS(1883),
    [sym_data_publish_feed] = ACTIONS(1883),
    [sym_data_get_published_feed] = ACTIONS(1883),
    [sym_data_add_record] = ACTIONS(1883),
    [sym_data_get_event_record] = ACTIONS(1883),
    [sym__wc_data_create_feed] = ACTIONS(1883),
    [sym__wc_data_add_record] = ACTIONS(1883),
    [sym__wc_data_get_event_record] = ACTIONS(1883),
    [sym_format] = ACTIONS(1883),
    [sym_astro_clock] = ACTIONS(1883),
    [sym_set_outdoor_temperature] = ACTIONS(1883),
    [sym_set_virtual_level_count] = ACTIONS(1883),
    [sym_set_virtual_channel_count] = ACTIONS(1883),
    [sym_set_virtual_port_count] = ACTIONS(1883),
    [sym__wc_encode_int] = ACTIONS(1883),
    [sym__wc_decode_int] = ACTIONS(1883),
    [sym__wc_int] = ACTIONS(1883),
    [sym__wc_to_ch_int] = ACTIONS(1883),
    [sym__ch_to_wc_int] = ACTIONS(1883),
    [sym__wc_find_string_int] = ACTIONS(1883),
    [sym__wc_left_string_int] = ACTIONS(1883),
    [sym__wc_lower_string_int] = ACTIONS(1883),
    [sym__wc_mid_string_int] = ACTIONS(1883),
    [sym__wc_remove_string_int] = ACTIONS(1883),
    [sym__wc_right_string_int] = ACTIONS(1883),
    [sym__wc_upper_string_int] = ACTIONS(1883),
    [sym__wc_compare_string_int] = ACTIONS(1883),
    [sym__wc_get_buffer_char_int] = ACTIONS(1883),
    [sym__wc_get_buffer_string_int] = ACTIONS(1883),
    [sym__wc_concat_string_int] = ACTIONS(1883),
    [sym__wc_file_open_int] = ACTIONS(1883),
    [sym__wc_file_close_int] = ACTIONS(1883),
    [sym__wc_file_read_int] = ACTIONS(1883),
    [sym__wc_file_read_line_int] = ACTIONS(1883),
    [sym__wc_file_write_int] = ACTIONS(1883),
    [sym__wc_file_write_line_int] = ACTIONS(1883),
    [aux_sym_preproc_include_token1] = ACTIONS(1883),
    [aux_sym_preproc_define_token1] = ACTIONS(1885),
    [aux_sym_preproc_warn_token1] = ACTIONS(1885),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1885),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1885),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1885),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1885),
    [aux_sym_preproc_else_token1] = ACTIONS(1885),
    [anon_sym_LBRACK] = ACTIONS(1885),
    [anon_sym_LBRACE] = ACTIONS(1885),
    [anon_sym_LPAREN] = ACTIONS(1883),
    [anon_sym_BANG] = ACTIONS(1885),
    [anon_sym_TILDE] = ACTIONS(1885),
    [anon_sym_DASH] = ACTIONS(1883),
    [anon_sym_PLUS] = ACTIONS(1883),
    [anon_sym_DASH_DASH] = ACTIONS(1885),
    [anon_sym_PLUS_PLUS] = ACTIONS(1885),
    [anon_sym_DQUOTE] = ACTIONS(1885),
    [anon_sym_SQUOTE] = ACTIONS(1885),
    [sym_number_literal] = ACTIONS(1885),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(270)] = {
    [sym_identifier] = ACTIONS(1887),
    [sym_program_name_keyword] = ACTIONS(1887),
    [sym_module_name_keyword] = ACTIONS(1887),
    [sym_define_device_keyword] = ACTIONS(1887),
    [sym_define_combine_keyword] = ACTIONS(1887),
    [sym_define_constant_keyword] = ACTIONS(1887),
    [sym_define_type_keyword] = ACTIONS(1887),
    [sym_define_variable_keyword] = ACTIONS(1887),
    [sym_define_system_variable_keyword] = ACTIONS(1887),
    [sym_define_connect_level_keyword] = ACTIONS(1887),
    [sym_define_latching_keyword] = ACTIONS(1887),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1887),
    [sym_define_toggling_keyword] = ACTIONS(1887),
    [sym_define_call_keyword] = ACTIONS(1887),
    [sym_define_function_keyword] = ACTIONS(1887),
    [sym_define_library_function_keyword] = ACTIONS(1887),
    [sym_define_start_keyword] = ACTIONS(1887),
    [sym_define_event_keyword] = ACTIONS(1887),
    [sym_define_program_keyword] = ACTIONS(1887),
    [sym_define_module_keyword] = ACTIONS(1887),
    [sym_char_keyword] = ACTIONS(1887),
    [sym_widechar_keyword] = ACTIONS(1887),
    [sym_integer_keyword] = ACTIONS(1887),
    [sym_sinteger_keyword] = ACTIONS(1887),
    [sym_long_keyword] = ACTIONS(1887),
    [sym_slong_keyword] = ACTIONS(1887),
    [sym_float_keyword] = ACTIONS(1887),
    [sym_double_keyword] = ACTIONS(1887),
    [sym_variant_keyword] = ACTIONS(1887),
    [sym_variantarray_keyword] = ACTIONS(1887),
    [sym_local_var_keyword] = ACTIONS(1887),
    [sym_stack_var_keyword] = ACTIONS(1887),
    [sym_constant_keyword] = ACTIONS(1887),
    [sym_volatile_keyword] = ACTIONS(1887),
    [sym_non_volatile_keyword] = ACTIONS(1887),
    [sym_persistent_keyword] = ACTIONS(1887),
    [sym_dev_keyword] = ACTIONS(1887),
    [sym_devlev_keyword] = ACTIONS(1887),
    [sym_devchan_keyword] = ACTIONS(1887),
    [sym_if_keyword] = ACTIONS(1887),
    [sym_switch_keyword] = ACTIONS(1887),
    [sym_case_keyword] = ACTIONS(1887),
    [sym_default_keyword] = ACTIONS(1887),
    [sym_while_keyword] = ACTIONS(1887),
    [sym_for_keyword] = ACTIONS(1887),
    [sym_break_keyword] = ACTIONS(1887),
    [sym_continue_keyword] = ACTIONS(1887),
    [sym_return_keyword] = ACTIONS(1887),
    [sym_select_keyword] = ACTIONS(1887),
    [sym_active_keyword] = ACTIONS(1887),
    [sym_struct_keyword] = ACTIONS(1887),
    [sym_structure_keyword] = ACTIONS(1887),
    [sym_button_event_keyword] = ACTIONS(1887),
    [sym_channel_event_keyword] = ACTIONS(1887),
    [sym_level_event_keyword] = ACTIONS(1887),
    [sym_data_event_keyword] = ACTIONS(1887),
    [sym_timeline_event_keyword] = ACTIONS(1887),
    [sym_custom_event_keyword] = ACTIONS(1887),
    [sym_push_keyword] = ACTIONS(1887),
    [sym_release_keyword] = ACTIONS(1887),
    [aux_sym_on_keyword_token1] = ACTIONS(1887),
    [aux_sym_off_keyword_token1] = ACTIONS(1887),
    [sym_send_level_keyword] = ACTIONS(1887),
    [sym_send_string_keyword] = ACTIONS(1887),
    [sym_send_command_keyword] = ACTIONS(1887),
    [sym_clear_buffer_keyword] = ACTIONS(1887),
    [sym_create_buffer_keyword] = ACTIONS(1887),
    [sym_create_multi_buffer_keyword] = ACTIONS(1887),
    [sym_call_keyword] = ACTIONS(1887),
    [sym_system_call_keyword] = ACTIONS(1887),
    [sym_devchan_to_keyword] = ACTIONS(1887),
    [sym_devchan_min_to_keyword] = ACTIONS(1887),
    [sym_devchan_total_off_keyword] = ACTIONS(1887),
    [sym_devchan_pulse_keyword] = ACTIONS(1887),
    [sym_wait_keyword] = ACTIONS(1887),
    [sym_cancel_wait_keyword] = ACTIONS(1887),
    [sym_wait_until_keyword] = ACTIONS(1887),
    [sym_cancel_wait_until_keyword] = ACTIONS(1887),
    [sym_cancel_all_wait_keyword] = ACTIONS(1887),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1887),
    [sym_not] = ACTIONS(1887),
    [sym_bnot] = ACTIONS(1887),
    [sym___file__] = ACTIONS(1887),
    [sym___date__] = ACTIONS(1887),
    [sym___ldate__] = ACTIONS(1887),
    [sym___time__] = ACTIONS(1887),
    [sym___line__] = ACTIONS(1887),
    [sym___name__] = ACTIONS(1887),
    [sym___netlinx__] = ACTIONS(1887),
    [sym_netlinx_axi_version] = ACTIONS(1887),
    [sym_url_flg_tcp] = ACTIONS(1887),
    [sym_url_flg_accinfopresent] = ACTIONS(1887),
    [sym_url_flg_temp] = ACTIONS(1887),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1887),
    [sym_url_flg_stat_mask] = ACTIONS(1887),
    [sym_url_flg_stat_lookup] = ACTIONS(1887),
    [sym_url_flg_stat_connecting] = ACTIONS(1887),
    [sym_url_flg_stat_waiting] = ACTIONS(1887),
    [sym_url_flg_stat_connected] = ACTIONS(1887),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1887),
    [sym_first_virtual_device] = ACTIONS(1887),
    [sym_first_local_port] = ACTIONS(1887),
    [sym_dynamic_virtual_device] = ACTIONS(1887),
    [sym_timeline_once] = ACTIONS(1887),
    [sym_timeline_repeat] = ACTIONS(1887),
    [sym_timeline_absolute] = ACTIONS(1887),
    [sym_timeline_relative] = ACTIONS(1887),
    [sym_do_push_timed_infinite] = ACTIONS(1887),
    [sym_source_type_no_address] = ACTIONS(1887),
    [sym_source_type_neuron_id] = ACTIONS(1887),
    [sym_source_type_ip_address] = ACTIONS(1887),
    [sym_source_type_axlink] = ACTIONS(1887),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1887),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1887),
    [sym_source_type_ip_socket_address] = ACTIONS(1887),
    [sym_source_type_rs232] = ACTIONS(1887),
    [sym_source_type_internal] = ACTIONS(1887),
    [sym_source_type_mac_address] = ACTIONS(1887),
    [sym_source_type_ipv4_port] = ACTIONS(1887),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1887),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1887),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1887),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1887),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1887),
    [sym_true] = ACTIONS(1887),
    [sym_false] = ACTIONS(1887),
    [sym_file_read_only] = ACTIONS(1887),
    [sym_file_rw_new] = ACTIONS(1887),
    [sym_file_rw_append] = ACTIONS(1887),
    [sym_ip_tcp] = ACTIONS(1887),
    [sym_ip_udp] = ACTIONS(1887),
    [sym_ip_udp_2way] = ACTIONS(1887),
    [sym_xml_encode_types] = ACTIONS(1887),
    [sym_xml_encode_char_as_list] = ACTIONS(1887),
    [sym_xml_encode_le] = ACTIONS(1887),
    [sym_xml_decode_types] = ACTIONS(1887),
    [sym_xml_decode_no_preserve] = ACTIONS(1887),
    [sym_duet_dev_type_display_device] = ACTIONS(1887),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1887),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1887),
    [sym_duet_dev_type_io_device] = ACTIONS(1887),
    [sym_duet_dev_type_relay_device] = ACTIONS(1887),
    [sym_duet_dev_type_ups] = ACTIONS(1887),
    [sym_duet_dev_type_amplifier] = ACTIONS(1887),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1887),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1887),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1887),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1887),
    [sym_duet_dev_type_camera] = ACTIONS(1887),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1887),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1887),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1887),
    [sym_duet_dev_type_dss] = ACTIONS(1887),
    [sym_duet_dev_type_dvr] = ACTIONS(1887),
    [sym_duet_dev_type_disc_device] = ACTIONS(1887),
    [sym_duet_dev_type_document_camera] = ACTIONS(1887),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1887),
    [sym_duet_dev_type_hvac] = ACTIONS(1887),
    [sym_duet_dev_type_keypad] = ACTIONS(1887),
    [sym_duet_dev_type_light] = ACTIONS(1887),
    [sym_duet_dev_type_monitor] = ACTIONS(1887),
    [sym_duet_dev_type_motor] = ACTIONS(1887),
    [sym_duet_dev_type_multi_window] = ACTIONS(1887),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1887),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1887),
    [sym_duet_dev_type_receiver] = ACTIONS(1887),
    [sym_duet_dev_type_security_system] = ACTIONS(1887),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1887),
    [sym_duet_dev_type_settop_box] = ACTIONS(1887),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1887),
    [sym_duet_dev_type_switcher] = ACTIONS(1887),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1887),
    [sym_duet_dev_type_tv] = ACTIONS(1887),
    [sym_duet_dev_type_utility] = ACTIONS(1887),
    [sym_duet_dev_type_vcr] = ACTIONS(1887),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1887),
    [sym_duet_dev_type_video_processor] = ACTIONS(1887),
    [sym_duet_dev_type_video_projector] = ACTIONS(1887),
    [sym_duet_dev_type_video_wall] = ACTIONS(1887),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1887),
    [sym_duet_dev_type_weather] = ACTIONS(1887),
    [sym_duet_dev_not_polled] = ACTIONS(1887),
    [sym_duet_dev_polled] = ACTIONS(1887),
    [sym_clkmgr_mode_network] = ACTIONS(1887),
    [sym_clkmgr_mode_standalone] = ACTIONS(1887),
    [sym_normal_standby] = ACTIONS(1887),
    [sym_normal_wake] = ACTIONS(1887),
    [sym_ip_multicast_ttl_option] = ACTIONS(1887),
    [sym_ip_ttl_subnet] = ACTIONS(1887),
    [sym_ip_ttl_site] = ACTIONS(1887),
    [sym_ip_ttl_region] = ACTIONS(1887),
    [sym_ip_ttl_continent] = ACTIONS(1887),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1887),
    [sym_ip_nodelay_on] = ACTIONS(1887),
    [sym_ip_nodelay_off] = ACTIONS(1887),
    [sym_amx_error] = ACTIONS(1887),
    [sym_amx_warning] = ACTIONS(1887),
    [sym_amx_info] = ACTIONS(1887),
    [sym_amx_debug] = ACTIONS(1887),
    [sym_netlinx_login_success] = ACTIONS(1887),
    [sym_netlinx_login_fail] = ACTIONS(1887),
    [sym_netlinx_logout] = ACTIONS(1887),
    [sym_null_str] = ACTIONS(1887),
    [sym_smtp_address] = ACTIONS(1887),
    [sym_smtp_port_number] = ACTIONS(1887),
    [sym_smtp_username] = ACTIONS(1887),
    [sym_smtp_password] = ACTIONS(1887),
    [sym_smtp_require_tls] = ACTIONS(1887),
    [sym_smtp_from] = ACTIONS(1887),
    [sym_smtp_tls_true] = ACTIONS(1887),
    [sym_smtp_tls_false] = ACTIONS(1887),
    [sym_listview_on_row_select_event] = ACTIONS(1887),
    [sym_data_structure_unknown] = ACTIONS(1887),
    [sym_data_structure_datafeed] = ACTIONS(1887),
    [sym_data_structure_datarecordset] = ACTIONS(1887),
    [sym_data_structure_datarecord] = ACTIONS(1887),
    [sym_data_structure_datafield] = ACTIONS(1887),
    [sym_data_max_content_field_count] = ACTIONS(1887),
    [sym_data_max_metadata_field_count] = ACTIONS(1887),
    [sym_data_max_id_length] = ACTIONS(1887),
    [sym_data_max_name_length] = ACTIONS(1887),
    [sym_data_max_description_length] = ACTIONS(1887),
    [sym_data_max_type_length] = ACTIONS(1887),
    [sym_data_max_format_length] = ACTIONS(1887),
    [sym_data_max_source_length] = ACTIONS(1887),
    [sym_data_max_value_length] = ACTIONS(1887),
    [sym_data_max_label_length] = ACTIONS(1887),
    [sym_data_max_url_length] = ACTIONS(1887),
    [sym_max_last_login_info_length] = ACTIONS(1887),
    [sym_data_type_unknown] = ACTIONS(1887),
    [sym_data_type_string] = ACTIONS(1887),
    [sym_data_type_datetime] = ACTIONS(1887),
    [sym_data_type_date] = ACTIONS(1887),
    [sym_data_type_time] = ACTIONS(1887),
    [sym_data_type_image] = ACTIONS(1887),
    [sym_data_format_url] = ACTIONS(1887),
    [sym_data_format_phone] = ACTIONS(1887),
    [sym_data_format_email] = ACTIONS(1887),
    [sym_data_format_iso8601] = ACTIONS(1887),
    [sym_auditlog_priv] = ACTIONS(1887),
    [sym_configuration_priv] = ACTIONS(1887),
    [sym_deviceconfig_priv] = ACTIONS(1887),
    [sym_ftp_priv] = ACTIONS(1887),
    [sym_http_priv] = ACTIONS(1887),
    [sym_networkconfig_priv] = ACTIONS(1887),
    [sym_programport_priv] = ACTIONS(1887),
    [sym_remoteui_priv] = ACTIONS(1887),
    [sym_securitycontrol_priv] = ACTIONS(1887),
    [sym_softwaremanagement_priv] = ACTIONS(1887),
    [sym_terminal_priv] = ACTIONS(1887),
    [sym_tpadmin_priv] = ACTIONS(1887),
    [sym_user1_priv] = ACTIONS(1887),
    [sym_user2_priv] = ACTIONS(1887),
    [sym_user3_priv] = ACTIONS(1887),
    [sym_user4_priv] = ACTIONS(1887),
    [sym_usermanagement_priv] = ACTIONS(1887),
    [sym_valid_account] = ACTIONS(1887),
    [sym_err_name_invalid] = ACTIONS(1887),
    [sym_err_password_invalid] = ACTIONS(1887),
    [sym_err_invalid_account] = ACTIONS(1887),
    [sym_err_no_authorization] = ACTIONS(1887),
    [sym_err_invalid_parameter] = ACTIONS(1887),
    [sym_err_user_locked_out] = ACTIONS(1887),
    [sym_err_user_account_expired] = ACTIONS(1887),
    [sym_err_general_error] = ACTIONS(1887),
    [sym_err_authentication_error] = ACTIONS(1887),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1887),
    [sym_tls_validate_certificate] = ACTIONS(1887),
    [sym_tdata] = ACTIONS(1887),
    [sym_tchannel] = ACTIONS(1887),
    [sym_tlevel] = ACTIONS(1887),
    [sym_tbutton] = ACTIONS(1887),
    [sym_ttimeline] = ACTIONS(1887),
    [sym_tcustom] = ACTIONS(1887),
    [sym_url_struct] = ACTIONS(1887),
    [sym_dns_struct] = ACTIONS(1887),
    [sym_ip_address_struct] = ACTIONS(1887),
    [sym_dev_info_struct] = ACTIONS(1887),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1887),
    [sym_data_feed] = ACTIONS(1887),
    [sym_data_field] = ACTIONS(1887),
    [sym_data_record] = ACTIONS(1887),
    [sym_wc_data_feed] = ACTIONS(1887),
    [sym_wc_data_field] = ACTIONS(1887),
    [sym_wc_data_record] = ACTIONS(1887),
    [sym_last_login_info] = ACTIONS(1887),
    [sym____reserved___] = ACTIONS(1887),
    [sym_date] = ACTIONS(1887),
    [sym_day] = ACTIONS(1887),
    [sym_time] = ACTIONS(1887),
    [sym_ldate] = ACTIONS(1887),
    [sym_push_device] = ACTIONS(1887),
    [sym_push_channel] = ACTIONS(1887),
    [sym_push_devchan] = ACTIONS(1887),
    [sym_release_device] = ACTIONS(1887),
    [sym_release_channel] = ACTIONS(1887),
    [sym_release_devchan] = ACTIONS(1887),
    [sym_master_slot] = ACTIONS(1887),
    [sym_get_pulse_time] = ACTIONS(1887),
    [sym_get_timer] = ACTIONS(1887),
    [sym_master_sn] = ACTIONS(1887),
    [sym_system_number] = ACTIONS(1887),
    [sym_button] = ACTIONS(1887),
    [sym_dv_channel] = ACTIONS(1887),
    [sym_data] = ACTIONS(1887),
    [sym_level] = ACTIONS(1887),
    [sym_timeline] = ACTIONS(1887),
    [sym_channel] = ACTIONS(1887),
    [sym_custom] = ACTIONS(1887),
    [sym_atoi] = ACTIONS(1887),
    [sym_atol] = ACTIONS(1887),
    [sym_atof] = ACTIONS(1887),
    [sym_itoa] = ACTIONS(1887),
    [sym_itohex] = ACTIONS(1887),
    [sym_hextoi] = ACTIONS(1887),
    [sym_ftoa] = ACTIONS(1887),
    [sym_find_string] = ACTIONS(1887),
    [sym_left_string] = ACTIONS(1887),
    [sym_right_string] = ACTIONS(1887),
    [sym_mid_string] = ACTIONS(1887),
    [sym_remove_string] = ACTIONS(1887),
    [sym_get_buffer_string] = ACTIONS(1887),
    [sym_get_buffer_char] = ACTIONS(1887),
    [sym_get_multi_buffer_string] = ACTIONS(1887),
    [sym_length_string] = ACTIONS(1887),
    [sym_max_length_string] = ACTIONS(1887),
    [sym_set_length_string] = ACTIONS(1887),
    [sym_compare_string] = ACTIONS(1887),
    [sym_upper_string] = ACTIONS(1887),
    [sym_lower_string] = ACTIONS(1887),
    [sym_get_last] = ACTIONS(1887),
    [sym_date_to_day] = ACTIONS(1887),
    [sym_date_to_month] = ACTIONS(1887),
    [sym_date_to_year] = ACTIONS(1887),
    [sym_time_to_hour] = ACTIONS(1887),
    [sym_time_to_minute] = ACTIONS(1887),
    [sym_time_to_second] = ACTIONS(1887),
    [sym_day_of_week] = ACTIONS(1887),
    [sym_set_pulse_time] = ACTIONS(1887),
    [sym_set_timer] = ACTIONS(1887),
    [sym_file_open] = ACTIONS(1887),
    [sym_file_close] = ACTIONS(1887),
    [sym_file_read] = ACTIONS(1887),
    [sym_file_read_line] = ACTIONS(1887),
    [sym_file_write] = ACTIONS(1887),
    [sym_file_write_line] = ACTIONS(1887),
    [sym_file_copy] = ACTIONS(1887),
    [sym_file_delete] = ACTIONS(1887),
    [sym_file_rename] = ACTIONS(1887),
    [sym_file_dir] = ACTIONS(1887),
    [sym_file_getdir] = ACTIONS(1887),
    [sym_file_setdir] = ACTIONS(1887),
    [sym_file_create_dir] = ACTIONS(1887),
    [sym_file_remove_dir] = ACTIONS(1887),
    [sym_file_seek] = ACTIONS(1887),
    [sym_length_array] = ACTIONS(1887),
    [sym_max_length_array] = ACTIONS(1887),
    [sym_set_length_array] = ACTIONS(1887),
    [sym_ip_client_open] = ACTIONS(1887),
    [sym_ip_client_close] = ACTIONS(1887),
    [sym_ip_server_open] = ACTIONS(1887),
    [sym_ip_server_close] = ACTIONS(1887),
    [sym_ip_bound_client_open] = ACTIONS(1887),
    [sym_ip_mc_server_open] = ACTIONS(1887),
    [sym_ip_set_option] = ACTIONS(1887),
    [sym_ssh_client_open] = ACTIONS(1887),
    [sym_ssh_client_close] = ACTIONS(1887),
    [sym_tls_client_open] = ACTIONS(1887),
    [sym_tls_client_close] = ACTIONS(1887),
    [sym_do_push] = ACTIONS(1887),
    [sym_do_push_timed] = ACTIONS(1887),
    [sym_do_release] = ACTIONS(1887),
    [sym_device_id] = ACTIONS(1887),
    [sym_device_info] = ACTIONS(1887),
    [sym_device_id_string] = ACTIONS(1887),
    [sym_device_standby] = ACTIONS(1887),
    [sym_device_wake] = ACTIONS(1887),
    [sym_timeline_create] = ACTIONS(1887),
    [sym_timeline_kill] = ACTIONS(1887),
    [sym_timeline_reload] = ACTIONS(1887),
    [sym_timeline_pause] = ACTIONS(1887),
    [sym_timeline_restart] = ACTIONS(1887),
    [sym_timeline_set] = ACTIONS(1887),
    [sym_timeline_get] = ACTIONS(1887),
    [sym_timeline_active] = ACTIONS(1887),
    [sym_timeline_dynamic_id] = ACTIONS(1887),
    [sym_variable_to_string] = ACTIONS(1887),
    [sym_string_to_variable] = ACTIONS(1887),
    [sym_length_variable_to_string] = ACTIONS(1887),
    [sym_type_cast] = ACTIONS(1887),
    [sym_variable_to_xml] = ACTIONS(1887),
    [sym_xml_to_variable] = ACTIONS(1887),
    [sym_length_variable_to_xml] = ACTIONS(1887),
    [sym_redirect_string] = ACTIONS(1887),
    [sym_random_number] = ACTIONS(1887),
    [sym_get_unique_id] = ACTIONS(1887),
    [sym_get_system_number] = ACTIONS(1887),
    [sym_set_system_number] = ACTIONS(1887),
    [sym_get_serial_number] = ACTIONS(1887),
    [sym_get_master_build] = ACTIONS(1887),
    [sym_set_validation_code] = ACTIONS(1887),
    [sym_reboot] = ACTIONS(1887),
    [sym_load_duet_module] = ACTIONS(1887),
    [sym_unload_duet_module] = ACTIONS(1887),
    [sym_duet_mem_size_set] = ACTIONS(1887),
    [sym_duet_mem_size_get] = ACTIONS(1887),
    [sym_dynamic_application_device] = ACTIONS(1887),
    [sym_dynamic_polled_port] = ACTIONS(1887),
    [sym_static_port_binding] = ACTIONS(1887),
    [sym_static_ip_binding] = ACTIONS(1887),
    [sym_get_dns_list] = ACTIONS(1887),
    [sym_set_dns_list] = ACTIONS(1887),
    [sym_get_ip_address] = ACTIONS(1887),
    [sym_set_ip_address] = ACTIONS(1887),
    [sym_get_url_list] = ACTIONS(1887),
    [sym_add_url_entry] = ACTIONS(1887),
    [sym_delete_url_entry] = ACTIONS(1887),
    [sym_abs_value] = ACTIONS(1887),
    [sym_max_value] = ACTIONS(1887),
    [sym_min_value] = ACTIONS(1887),
    [sym_exp_value] = ACTIONS(1887),
    [sym_log_value] = ACTIONS(1887),
    [sym_log10_value] = ACTIONS(1887),
    [sym_power_value] = ACTIONS(1887),
    [sym_sqrt_value] = ACTIONS(1887),
    [sym_raw_be] = ACTIONS(1887),
    [sym_raw_le] = ACTIONS(1887),
    [sym_internal_queue_size_set] = ACTIONS(1887),
    [sym_internal_queue_size_get] = ACTIONS(1887),
    [sym_internal_threshold_set] = ACTIONS(1887),
    [sym_internal_threshold_get] = ACTIONS(1887),
    [sym_rebuilt_event] = ACTIONS(1887),
    [sym_do_custom_event] = ACTIONS(1887),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1887),
    [sym_clkmgr_set_clk_source] = ACTIONS(1887),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1887),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1887),
    [sym_clkmgr_get_timezone] = ACTIONS(1887),
    [sym_clkmgr_set_timezone] = ACTIONS(1887),
    [sym_clkmgr_get_resync_period] = ACTIONS(1887),
    [sym_clkmgr_set_resync_period] = ACTIONS(1887),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1887),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1887),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1887),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1887),
    [sym_clkmgr_get_timeservers] = ACTIONS(1887),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1887),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1887),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1887),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1887),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1887),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1887),
    [sym_set_log_level] = ACTIONS(1887),
    [sym_get_log_level] = ACTIONS(1887),
    [sym_amx_log] = ACTIONS(1887),
    [sym_smtp_server_config_set] = ACTIONS(1887),
    [sym_smtp_server_config_get] = ACTIONS(1887),
    [sym_smtp_send] = ACTIONS(1887),
    [sym_validate_netlinx_account] = ACTIONS(1887),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1887),
    [sym_authenticate_certificate] = ACTIONS(1887),
    [sym_audit_netlinx_session_event] = ACTIONS(1887),
    [sym_audit_netlinx_generic_event] = ACTIONS(1887),
    [sym_get_available_flash_disk_space] = ACTIONS(1887),
    [sym_get_max_flash_disk_space] = ACTIONS(1887),
    [sym_data_create_feed] = ACTIONS(1887),
    [sym_data_delete_feed] = ACTIONS(1887),
    [sym_data_publish_feed] = ACTIONS(1887),
    [sym_data_get_published_feed] = ACTIONS(1887),
    [sym_data_add_record] = ACTIONS(1887),
    [sym_data_get_event_record] = ACTIONS(1887),
    [sym__wc_data_create_feed] = ACTIONS(1887),
    [sym__wc_data_add_record] = ACTIONS(1887),
    [sym__wc_data_get_event_record] = ACTIONS(1887),
    [sym_format] = ACTIONS(1887),
    [sym_astro_clock] = ACTIONS(1887),
    [sym_set_outdoor_temperature] = ACTIONS(1887),
    [sym_set_virtual_level_count] = ACTIONS(1887),
    [sym_set_virtual_channel_count] = ACTIONS(1887),
    [sym_set_virtual_port_count] = ACTIONS(1887),
    [sym__wc_encode_int] = ACTIONS(1887),
    [sym__wc_decode_int] = ACTIONS(1887),
    [sym__wc_int] = ACTIONS(1887),
    [sym__wc_to_ch_int] = ACTIONS(1887),
    [sym__ch_to_wc_int] = ACTIONS(1887),
    [sym__wc_find_string_int] = ACTIONS(1887),
    [sym__wc_left_string_int] = ACTIONS(1887),
    [sym__wc_lower_string_int] = ACTIONS(1887),
    [sym__wc_mid_string_int] = ACTIONS(1887),
    [sym__wc_remove_string_int] = ACTIONS(1887),
    [sym__wc_right_string_int] = ACTIONS(1887),
    [sym__wc_upper_string_int] = ACTIONS(1887),
    [sym__wc_compare_string_int] = ACTIONS(1887),
    [sym__wc_get_buffer_char_int] = ACTIONS(1887),
    [sym__wc_get_buffer_string_int] = ACTIONS(1887),
    [sym__wc_concat_string_int] = ACTIONS(1887),
    [sym__wc_file_open_int] = ACTIONS(1887),
    [sym__wc_file_close_int] = ACTIONS(1887),
    [sym__wc_file_read_int] = ACTIONS(1887),
    [sym__wc_file_read_line_int] = ACTIONS(1887),
    [sym__wc_file_write_int] = ACTIONS(1887),
    [sym__wc_file_write_line_int] = ACTIONS(1887),
    [aux_sym_preproc_include_token1] = ACTIONS(1887),
    [aux_sym_preproc_define_token1] = ACTIONS(1889),
    [aux_sym_preproc_warn_token1] = ACTIONS(1889),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1889),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1889),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1889),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1889),
    [aux_sym_preproc_else_token1] = ACTIONS(1889),
    [anon_sym_LBRACK] = ACTIONS(1889),
    [anon_sym_LBRACE] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(1887),
    [anon_sym_BANG] = ACTIONS(1889),
    [anon_sym_TILDE] = ACTIONS(1889),
    [anon_sym_DASH] = ACTIONS(1887),
    [anon_sym_PLUS] = ACTIONS(1887),
    [anon_sym_DASH_DASH] = ACTIONS(1889),
    [anon_sym_PLUS_PLUS] = ACTIONS(1889),
    [anon_sym_DQUOTE] = ACTIONS(1889),
    [anon_sym_SQUOTE] = ACTIONS(1889),
    [sym_number_literal] = ACTIONS(1889),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(271)] = {
    [sym_identifier] = ACTIONS(1891),
    [sym_program_name_keyword] = ACTIONS(1891),
    [sym_module_name_keyword] = ACTIONS(1891),
    [sym_define_device_keyword] = ACTIONS(1891),
    [sym_define_combine_keyword] = ACTIONS(1891),
    [sym_define_constant_keyword] = ACTIONS(1891),
    [sym_define_type_keyword] = ACTIONS(1891),
    [sym_define_variable_keyword] = ACTIONS(1891),
    [sym_define_system_variable_keyword] = ACTIONS(1891),
    [sym_define_connect_level_keyword] = ACTIONS(1891),
    [sym_define_latching_keyword] = ACTIONS(1891),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1891),
    [sym_define_toggling_keyword] = ACTIONS(1891),
    [sym_define_call_keyword] = ACTIONS(1891),
    [sym_define_function_keyword] = ACTIONS(1891),
    [sym_define_library_function_keyword] = ACTIONS(1891),
    [sym_define_start_keyword] = ACTIONS(1891),
    [sym_define_event_keyword] = ACTIONS(1891),
    [sym_define_program_keyword] = ACTIONS(1891),
    [sym_define_module_keyword] = ACTIONS(1891),
    [sym_char_keyword] = ACTIONS(1891),
    [sym_widechar_keyword] = ACTIONS(1891),
    [sym_integer_keyword] = ACTIONS(1891),
    [sym_sinteger_keyword] = ACTIONS(1891),
    [sym_long_keyword] = ACTIONS(1891),
    [sym_slong_keyword] = ACTIONS(1891),
    [sym_float_keyword] = ACTIONS(1891),
    [sym_double_keyword] = ACTIONS(1891),
    [sym_variant_keyword] = ACTIONS(1891),
    [sym_variantarray_keyword] = ACTIONS(1891),
    [sym_local_var_keyword] = ACTIONS(1891),
    [sym_stack_var_keyword] = ACTIONS(1891),
    [sym_constant_keyword] = ACTIONS(1891),
    [sym_volatile_keyword] = ACTIONS(1891),
    [sym_non_volatile_keyword] = ACTIONS(1891),
    [sym_persistent_keyword] = ACTIONS(1891),
    [sym_dev_keyword] = ACTIONS(1891),
    [sym_devlev_keyword] = ACTIONS(1891),
    [sym_devchan_keyword] = ACTIONS(1891),
    [sym_if_keyword] = ACTIONS(1891),
    [sym_switch_keyword] = ACTIONS(1891),
    [sym_case_keyword] = ACTIONS(1891),
    [sym_default_keyword] = ACTIONS(1891),
    [sym_while_keyword] = ACTIONS(1891),
    [sym_for_keyword] = ACTIONS(1891),
    [sym_break_keyword] = ACTIONS(1891),
    [sym_continue_keyword] = ACTIONS(1891),
    [sym_return_keyword] = ACTIONS(1891),
    [sym_select_keyword] = ACTIONS(1891),
    [sym_active_keyword] = ACTIONS(1891),
    [sym_struct_keyword] = ACTIONS(1891),
    [sym_structure_keyword] = ACTIONS(1891),
    [sym_button_event_keyword] = ACTIONS(1891),
    [sym_channel_event_keyword] = ACTIONS(1891),
    [sym_level_event_keyword] = ACTIONS(1891),
    [sym_data_event_keyword] = ACTIONS(1891),
    [sym_timeline_event_keyword] = ACTIONS(1891),
    [sym_custom_event_keyword] = ACTIONS(1891),
    [sym_push_keyword] = ACTIONS(1891),
    [sym_release_keyword] = ACTIONS(1891),
    [aux_sym_on_keyword_token1] = ACTIONS(1891),
    [aux_sym_off_keyword_token1] = ACTIONS(1891),
    [sym_send_level_keyword] = ACTIONS(1891),
    [sym_send_string_keyword] = ACTIONS(1891),
    [sym_send_command_keyword] = ACTIONS(1891),
    [sym_clear_buffer_keyword] = ACTIONS(1891),
    [sym_create_buffer_keyword] = ACTIONS(1891),
    [sym_create_multi_buffer_keyword] = ACTIONS(1891),
    [sym_call_keyword] = ACTIONS(1891),
    [sym_system_call_keyword] = ACTIONS(1891),
    [sym_devchan_to_keyword] = ACTIONS(1891),
    [sym_devchan_min_to_keyword] = ACTIONS(1891),
    [sym_devchan_total_off_keyword] = ACTIONS(1891),
    [sym_devchan_pulse_keyword] = ACTIONS(1891),
    [sym_wait_keyword] = ACTIONS(1891),
    [sym_cancel_wait_keyword] = ACTIONS(1891),
    [sym_wait_until_keyword] = ACTIONS(1891),
    [sym_cancel_wait_until_keyword] = ACTIONS(1891),
    [sym_cancel_all_wait_keyword] = ACTIONS(1891),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1891),
    [sym_not] = ACTIONS(1891),
    [sym_bnot] = ACTIONS(1891),
    [sym___file__] = ACTIONS(1891),
    [sym___date__] = ACTIONS(1891),
    [sym___ldate__] = ACTIONS(1891),
    [sym___time__] = ACTIONS(1891),
    [sym___line__] = ACTIONS(1891),
    [sym___name__] = ACTIONS(1891),
    [sym___netlinx__] = ACTIONS(1891),
    [sym_netlinx_axi_version] = ACTIONS(1891),
    [sym_url_flg_tcp] = ACTIONS(1891),
    [sym_url_flg_accinfopresent] = ACTIONS(1891),
    [sym_url_flg_temp] = ACTIONS(1891),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1891),
    [sym_url_flg_stat_mask] = ACTIONS(1891),
    [sym_url_flg_stat_lookup] = ACTIONS(1891),
    [sym_url_flg_stat_connecting] = ACTIONS(1891),
    [sym_url_flg_stat_waiting] = ACTIONS(1891),
    [sym_url_flg_stat_connected] = ACTIONS(1891),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1891),
    [sym_first_virtual_device] = ACTIONS(1891),
    [sym_first_local_port] = ACTIONS(1891),
    [sym_dynamic_virtual_device] = ACTIONS(1891),
    [sym_timeline_once] = ACTIONS(1891),
    [sym_timeline_repeat] = ACTIONS(1891),
    [sym_timeline_absolute] = ACTIONS(1891),
    [sym_timeline_relative] = ACTIONS(1891),
    [sym_do_push_timed_infinite] = ACTIONS(1891),
    [sym_source_type_no_address] = ACTIONS(1891),
    [sym_source_type_neuron_id] = ACTIONS(1891),
    [sym_source_type_ip_address] = ACTIONS(1891),
    [sym_source_type_axlink] = ACTIONS(1891),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1891),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1891),
    [sym_source_type_ip_socket_address] = ACTIONS(1891),
    [sym_source_type_rs232] = ACTIONS(1891),
    [sym_source_type_internal] = ACTIONS(1891),
    [sym_source_type_mac_address] = ACTIONS(1891),
    [sym_source_type_ipv4_port] = ACTIONS(1891),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1891),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1891),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1891),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1891),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1891),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1891),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1891),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1891),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1891),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1891),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1891),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1891),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1891),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1891),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1891),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1891),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1891),
    [sym_true] = ACTIONS(1891),
    [sym_false] = ACTIONS(1891),
    [sym_file_read_only] = ACTIONS(1891),
    [sym_file_rw_new] = ACTIONS(1891),
    [sym_file_rw_append] = ACTIONS(1891),
    [sym_ip_tcp] = ACTIONS(1891),
    [sym_ip_udp] = ACTIONS(1891),
    [sym_ip_udp_2way] = ACTIONS(1891),
    [sym_xml_encode_types] = ACTIONS(1891),
    [sym_xml_encode_char_as_list] = ACTIONS(1891),
    [sym_xml_encode_le] = ACTIONS(1891),
    [sym_xml_decode_types] = ACTIONS(1891),
    [sym_xml_decode_no_preserve] = ACTIONS(1891),
    [sym_duet_dev_type_display_device] = ACTIONS(1891),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1891),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1891),
    [sym_duet_dev_type_io_device] = ACTIONS(1891),
    [sym_duet_dev_type_relay_device] = ACTIONS(1891),
    [sym_duet_dev_type_ups] = ACTIONS(1891),
    [sym_duet_dev_type_amplifier] = ACTIONS(1891),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1891),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1891),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1891),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1891),
    [sym_duet_dev_type_camera] = ACTIONS(1891),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1891),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1891),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1891),
    [sym_duet_dev_type_dss] = ACTIONS(1891),
    [sym_duet_dev_type_dvr] = ACTIONS(1891),
    [sym_duet_dev_type_disc_device] = ACTIONS(1891),
    [sym_duet_dev_type_document_camera] = ACTIONS(1891),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1891),
    [sym_duet_dev_type_hvac] = ACTIONS(1891),
    [sym_duet_dev_type_keypad] = ACTIONS(1891),
    [sym_duet_dev_type_light] = ACTIONS(1891),
    [sym_duet_dev_type_monitor] = ACTIONS(1891),
    [sym_duet_dev_type_motor] = ACTIONS(1891),
    [sym_duet_dev_type_multi_window] = ACTIONS(1891),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1891),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1891),
    [sym_duet_dev_type_receiver] = ACTIONS(1891),
    [sym_duet_dev_type_security_system] = ACTIONS(1891),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1891),
    [sym_duet_dev_type_settop_box] = ACTIONS(1891),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1891),
    [sym_duet_dev_type_switcher] = ACTIONS(1891),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1891),
    [sym_duet_dev_type_tv] = ACTIONS(1891),
    [sym_duet_dev_type_utility] = ACTIONS(1891),
    [sym_duet_dev_type_vcr] = ACTIONS(1891),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1891),
    [sym_duet_dev_type_video_processor] = ACTIONS(1891),
    [sym_duet_dev_type_video_projector] = ACTIONS(1891),
    [sym_duet_dev_type_video_wall] = ACTIONS(1891),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1891),
    [sym_duet_dev_type_weather] = ACTIONS(1891),
    [sym_duet_dev_not_polled] = ACTIONS(1891),
    [sym_duet_dev_polled] = ACTIONS(1891),
    [sym_clkmgr_mode_network] = ACTIONS(1891),
    [sym_clkmgr_mode_standalone] = ACTIONS(1891),
    [sym_normal_standby] = ACTIONS(1891),
    [sym_normal_wake] = ACTIONS(1891),
    [sym_ip_multicast_ttl_option] = ACTIONS(1891),
    [sym_ip_ttl_subnet] = ACTIONS(1891),
    [sym_ip_ttl_site] = ACTIONS(1891),
    [sym_ip_ttl_region] = ACTIONS(1891),
    [sym_ip_ttl_continent] = ACTIONS(1891),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1891),
    [sym_ip_nodelay_on] = ACTIONS(1891),
    [sym_ip_nodelay_off] = ACTIONS(1891),
    [sym_amx_error] = ACTIONS(1891),
    [sym_amx_warning] = ACTIONS(1891),
    [sym_amx_info] = ACTIONS(1891),
    [sym_amx_debug] = ACTIONS(1891),
    [sym_netlinx_login_success] = ACTIONS(1891),
    [sym_netlinx_login_fail] = ACTIONS(1891),
    [sym_netlinx_logout] = ACTIONS(1891),
    [sym_null_str] = ACTIONS(1891),
    [sym_smtp_address] = ACTIONS(1891),
    [sym_smtp_port_number] = ACTIONS(1891),
    [sym_smtp_username] = ACTIONS(1891),
    [sym_smtp_password] = ACTIONS(1891),
    [sym_smtp_require_tls] = ACTIONS(1891),
    [sym_smtp_from] = ACTIONS(1891),
    [sym_smtp_tls_true] = ACTIONS(1891),
    [sym_smtp_tls_false] = ACTIONS(1891),
    [sym_listview_on_row_select_event] = ACTIONS(1891),
    [sym_data_structure_unknown] = ACTIONS(1891),
    [sym_data_structure_datafeed] = ACTIONS(1891),
    [sym_data_structure_datarecordset] = ACTIONS(1891),
    [sym_data_structure_datarecord] = ACTIONS(1891),
    [sym_data_structure_datafield] = ACTIONS(1891),
    [sym_data_max_content_field_count] = ACTIONS(1891),
    [sym_data_max_metadata_field_count] = ACTIONS(1891),
    [sym_data_max_id_length] = ACTIONS(1891),
    [sym_data_max_name_length] = ACTIONS(1891),
    [sym_data_max_description_length] = ACTIONS(1891),
    [sym_data_max_type_length] = ACTIONS(1891),
    [sym_data_max_format_length] = ACTIONS(1891),
    [sym_data_max_source_length] = ACTIONS(1891),
    [sym_data_max_value_length] = ACTIONS(1891),
    [sym_data_max_label_length] = ACTIONS(1891),
    [sym_data_max_url_length] = ACTIONS(1891),
    [sym_max_last_login_info_length] = ACTIONS(1891),
    [sym_data_type_unknown] = ACTIONS(1891),
    [sym_data_type_string] = ACTIONS(1891),
    [sym_data_type_datetime] = ACTIONS(1891),
    [sym_data_type_date] = ACTIONS(1891),
    [sym_data_type_time] = ACTIONS(1891),
    [sym_data_type_image] = ACTIONS(1891),
    [sym_data_format_url] = ACTIONS(1891),
    [sym_data_format_phone] = ACTIONS(1891),
    [sym_data_format_email] = ACTIONS(1891),
    [sym_data_format_iso8601] = ACTIONS(1891),
    [sym_auditlog_priv] = ACTIONS(1891),
    [sym_configuration_priv] = ACTIONS(1891),
    [sym_deviceconfig_priv] = ACTIONS(1891),
    [sym_ftp_priv] = ACTIONS(1891),
    [sym_http_priv] = ACTIONS(1891),
    [sym_networkconfig_priv] = ACTIONS(1891),
    [sym_programport_priv] = ACTIONS(1891),
    [sym_remoteui_priv] = ACTIONS(1891),
    [sym_securitycontrol_priv] = ACTIONS(1891),
    [sym_softwaremanagement_priv] = ACTIONS(1891),
    [sym_terminal_priv] = ACTIONS(1891),
    [sym_tpadmin_priv] = ACTIONS(1891),
    [sym_user1_priv] = ACTIONS(1891),
    [sym_user2_priv] = ACTIONS(1891),
    [sym_user3_priv] = ACTIONS(1891),
    [sym_user4_priv] = ACTIONS(1891),
    [sym_usermanagement_priv] = ACTIONS(1891),
    [sym_valid_account] = ACTIONS(1891),
    [sym_err_name_invalid] = ACTIONS(1891),
    [sym_err_password_invalid] = ACTIONS(1891),
    [sym_err_invalid_account] = ACTIONS(1891),
    [sym_err_no_authorization] = ACTIONS(1891),
    [sym_err_invalid_parameter] = ACTIONS(1891),
    [sym_err_user_locked_out] = ACTIONS(1891),
    [sym_err_user_account_expired] = ACTIONS(1891),
    [sym_err_general_error] = ACTIONS(1891),
    [sym_err_authentication_error] = ACTIONS(1891),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1891),
    [sym_tls_validate_certificate] = ACTIONS(1891),
    [sym_tdata] = ACTIONS(1891),
    [sym_tchannel] = ACTIONS(1891),
    [sym_tlevel] = ACTIONS(1891),
    [sym_tbutton] = ACTIONS(1891),
    [sym_ttimeline] = ACTIONS(1891),
    [sym_tcustom] = ACTIONS(1891),
    [sym_url_struct] = ACTIONS(1891),
    [sym_dns_struct] = ACTIONS(1891),
    [sym_ip_address_struct] = ACTIONS(1891),
    [sym_dev_info_struct] = ACTIONS(1891),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1891),
    [sym_data_feed] = ACTIONS(1891),
    [sym_data_field] = ACTIONS(1891),
    [sym_data_record] = ACTIONS(1891),
    [sym_wc_data_feed] = ACTIONS(1891),
    [sym_wc_data_field] = ACTIONS(1891),
    [sym_wc_data_record] = ACTIONS(1891),
    [sym_last_login_info] = ACTIONS(1891),
    [sym____reserved___] = ACTIONS(1891),
    [sym_date] = ACTIONS(1891),
    [sym_day] = ACTIONS(1891),
    [sym_time] = ACTIONS(1891),
    [sym_ldate] = ACTIONS(1891),
    [sym_push_device] = ACTIONS(1891),
    [sym_push_channel] = ACTIONS(1891),
    [sym_push_devchan] = ACTIONS(1891),
    [sym_release_device] = ACTIONS(1891),
    [sym_release_channel] = ACTIONS(1891),
    [sym_release_devchan] = ACTIONS(1891),
    [sym_master_slot] = ACTIONS(1891),
    [sym_get_pulse_time] = ACTIONS(1891),
    [sym_get_timer] = ACTIONS(1891),
    [sym_master_sn] = ACTIONS(1891),
    [sym_system_number] = ACTIONS(1891),
    [sym_button] = ACTIONS(1891),
    [sym_dv_channel] = ACTIONS(1891),
    [sym_data] = ACTIONS(1891),
    [sym_level] = ACTIONS(1891),
    [sym_timeline] = ACTIONS(1891),
    [sym_channel] = ACTIONS(1891),
    [sym_custom] = ACTIONS(1891),
    [sym_atoi] = ACTIONS(1891),
    [sym_atol] = ACTIONS(1891),
    [sym_atof] = ACTIONS(1891),
    [sym_itoa] = ACTIONS(1891),
    [sym_itohex] = ACTIONS(1891),
    [sym_hextoi] = ACTIONS(1891),
    [sym_ftoa] = ACTIONS(1891),
    [sym_find_string] = ACTIONS(1891),
    [sym_left_string] = ACTIONS(1891),
    [sym_right_string] = ACTIONS(1891),
    [sym_mid_string] = ACTIONS(1891),
    [sym_remove_string] = ACTIONS(1891),
    [sym_get_buffer_string] = ACTIONS(1891),
    [sym_get_buffer_char] = ACTIONS(1891),
    [sym_get_multi_buffer_string] = ACTIONS(1891),
    [sym_length_string] = ACTIONS(1891),
    [sym_max_length_string] = ACTIONS(1891),
    [sym_set_length_string] = ACTIONS(1891),
    [sym_compare_string] = ACTIONS(1891),
    [sym_upper_string] = ACTIONS(1891),
    [sym_lower_string] = ACTIONS(1891),
    [sym_get_last] = ACTIONS(1891),
    [sym_date_to_day] = ACTIONS(1891),
    [sym_date_to_month] = ACTIONS(1891),
    [sym_date_to_year] = ACTIONS(1891),
    [sym_time_to_hour] = ACTIONS(1891),
    [sym_time_to_minute] = ACTIONS(1891),
    [sym_time_to_second] = ACTIONS(1891),
    [sym_day_of_week] = ACTIONS(1891),
    [sym_set_pulse_time] = ACTIONS(1891),
    [sym_set_timer] = ACTIONS(1891),
    [sym_file_open] = ACTIONS(1891),
    [sym_file_close] = ACTIONS(1891),
    [sym_file_read] = ACTIONS(1891),
    [sym_file_read_line] = ACTIONS(1891),
    [sym_file_write] = ACTIONS(1891),
    [sym_file_write_line] = ACTIONS(1891),
    [sym_file_copy] = ACTIONS(1891),
    [sym_file_delete] = ACTIONS(1891),
    [sym_file_rename] = ACTIONS(1891),
    [sym_file_dir] = ACTIONS(1891),
    [sym_file_getdir] = ACTIONS(1891),
    [sym_file_setdir] = ACTIONS(1891),
    [sym_file_create_dir] = ACTIONS(1891),
    [sym_file_remove_dir] = ACTIONS(1891),
    [sym_file_seek] = ACTIONS(1891),
    [sym_length_array] = ACTIONS(1891),
    [sym_max_length_array] = ACTIONS(1891),
    [sym_set_length_array] = ACTIONS(1891),
    [sym_ip_client_open] = ACTIONS(1891),
    [sym_ip_client_close] = ACTIONS(1891),
    [sym_ip_server_open] = ACTIONS(1891),
    [sym_ip_server_close] = ACTIONS(1891),
    [sym_ip_bound_client_open] = ACTIONS(1891),
    [sym_ip_mc_server_open] = ACTIONS(1891),
    [sym_ip_set_option] = ACTIONS(1891),
    [sym_ssh_client_open] = ACTIONS(1891),
    [sym_ssh_client_close] = ACTIONS(1891),
    [sym_tls_client_open] = ACTIONS(1891),
    [sym_tls_client_close] = ACTIONS(1891),
    [sym_do_push] = ACTIONS(1891),
    [sym_do_push_timed] = ACTIONS(1891),
    [sym_do_release] = ACTIONS(1891),
    [sym_device_id] = ACTIONS(1891),
    [sym_device_info] = ACTIONS(1891),
    [sym_device_id_string] = ACTIONS(1891),
    [sym_device_standby] = ACTIONS(1891),
    [sym_device_wake] = ACTIONS(1891),
    [sym_timeline_create] = ACTIONS(1891),
    [sym_timeline_kill] = ACTIONS(1891),
    [sym_timeline_reload] = ACTIONS(1891),
    [sym_timeline_pause] = ACTIONS(1891),
    [sym_timeline_restart] = ACTIONS(1891),
    [sym_timeline_set] = ACTIONS(1891),
    [sym_timeline_get] = ACTIONS(1891),
    [sym_timeline_active] = ACTIONS(1891),
    [sym_timeline_dynamic_id] = ACTIONS(1891),
    [sym_variable_to_string] = ACTIONS(1891),
    [sym_string_to_variable] = ACTIONS(1891),
    [sym_length_variable_to_string] = ACTIONS(1891),
    [sym_type_cast] = ACTIONS(1891),
    [sym_variable_to_xml] = ACTIONS(1891),
    [sym_xml_to_variable] = ACTIONS(1891),
    [sym_length_variable_to_xml] = ACTIONS(1891),
    [sym_redirect_string] = ACTIONS(1891),
    [sym_random_number] = ACTIONS(1891),
    [sym_get_unique_id] = ACTIONS(1891),
    [sym_get_system_number] = ACTIONS(1891),
    [sym_set_system_number] = ACTIONS(1891),
    [sym_get_serial_number] = ACTIONS(1891),
    [sym_get_master_build] = ACTIONS(1891),
    [sym_set_validation_code] = ACTIONS(1891),
    [sym_reboot] = ACTIONS(1891),
    [sym_load_duet_module] = ACTIONS(1891),
    [sym_unload_duet_module] = ACTIONS(1891),
    [sym_duet_mem_size_set] = ACTIONS(1891),
    [sym_duet_mem_size_get] = ACTIONS(1891),
    [sym_dynamic_application_device] = ACTIONS(1891),
    [sym_dynamic_polled_port] = ACTIONS(1891),
    [sym_static_port_binding] = ACTIONS(1891),
    [sym_static_ip_binding] = ACTIONS(1891),
    [sym_get_dns_list] = ACTIONS(1891),
    [sym_set_dns_list] = ACTIONS(1891),
    [sym_get_ip_address] = ACTIONS(1891),
    [sym_set_ip_address] = ACTIONS(1891),
    [sym_get_url_list] = ACTIONS(1891),
    [sym_add_url_entry] = ACTIONS(1891),
    [sym_delete_url_entry] = ACTIONS(1891),
    [sym_abs_value] = ACTIONS(1891),
    [sym_max_value] = ACTIONS(1891),
    [sym_min_value] = ACTIONS(1891),
    [sym_exp_value] = ACTIONS(1891),
    [sym_log_value] = ACTIONS(1891),
    [sym_log10_value] = ACTIONS(1891),
    [sym_power_value] = ACTIONS(1891),
    [sym_sqrt_value] = ACTIONS(1891),
    [sym_raw_be] = ACTIONS(1891),
    [sym_raw_le] = ACTIONS(1891),
    [sym_internal_queue_size_set] = ACTIONS(1891),
    [sym_internal_queue_size_get] = ACTIONS(1891),
    [sym_internal_threshold_set] = ACTIONS(1891),
    [sym_internal_threshold_get] = ACTIONS(1891),
    [sym_rebuilt_event] = ACTIONS(1891),
    [sym_do_custom_event] = ACTIONS(1891),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1891),
    [sym_clkmgr_set_clk_source] = ACTIONS(1891),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1891),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1891),
    [sym_clkmgr_get_timezone] = ACTIONS(1891),
    [sym_clkmgr_set_timezone] = ACTIONS(1891),
    [sym_clkmgr_get_resync_period] = ACTIONS(1891),
    [sym_clkmgr_set_resync_period] = ACTIONS(1891),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1891),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1891),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1891),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1891),
    [sym_clkmgr_get_timeservers] = ACTIONS(1891),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1891),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1891),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1891),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1891),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1891),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1891),
    [sym_set_log_level] = ACTIONS(1891),
    [sym_get_log_level] = ACTIONS(1891),
    [sym_amx_log] = ACTIONS(1891),
    [sym_smtp_server_config_set] = ACTIONS(1891),
    [sym_smtp_server_config_get] = ACTIONS(1891),
    [sym_smtp_send] = ACTIONS(1891),
    [sym_validate_netlinx_account] = ACTIONS(1891),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1891),
    [sym_authenticate_certificate] = ACTIONS(1891),
    [sym_audit_netlinx_session_event] = ACTIONS(1891),
    [sym_audit_netlinx_generic_event] = ACTIONS(1891),
    [sym_get_available_flash_disk_space] = ACTIONS(1891),
    [sym_get_max_flash_disk_space] = ACTIONS(1891),
    [sym_data_create_feed] = ACTIONS(1891),
    [sym_data_delete_feed] = ACTIONS(1891),
    [sym_data_publish_feed] = ACTIONS(1891),
    [sym_data_get_published_feed] = ACTIONS(1891),
    [sym_data_add_record] = ACTIONS(1891),
    [sym_data_get_event_record] = ACTIONS(1891),
    [sym__wc_data_create_feed] = ACTIONS(1891),
    [sym__wc_data_add_record] = ACTIONS(1891),
    [sym__wc_data_get_event_record] = ACTIONS(1891),
    [sym_format] = ACTIONS(1891),
    [sym_astro_clock] = ACTIONS(1891),
    [sym_set_outdoor_temperature] = ACTIONS(1891),
    [sym_set_virtual_level_count] = ACTIONS(1891),
    [sym_set_virtual_channel_count] = ACTIONS(1891),
    [sym_set_virtual_port_count] = ACTIONS(1891),
    [sym__wc_encode_int] = ACTIONS(1891),
    [sym__wc_decode_int] = ACTIONS(1891),
    [sym__wc_int] = ACTIONS(1891),
    [sym__wc_to_ch_int] = ACTIONS(1891),
    [sym__ch_to_wc_int] = ACTIONS(1891),
    [sym__wc_find_string_int] = ACTIONS(1891),
    [sym__wc_left_string_int] = ACTIONS(1891),
    [sym__wc_lower_string_int] = ACTIONS(1891),
    [sym__wc_mid_string_int] = ACTIONS(1891),
    [sym__wc_remove_string_int] = ACTIONS(1891),
    [sym__wc_right_string_int] = ACTIONS(1891),
    [sym__wc_upper_string_int] = ACTIONS(1891),
    [sym__wc_compare_string_int] = ACTIONS(1891),
    [sym__wc_get_buffer_char_int] = ACTIONS(1891),
    [sym__wc_get_buffer_string_int] = ACTIONS(1891),
    [sym__wc_concat_string_int] = ACTIONS(1891),
    [sym__wc_file_open_int] = ACTIONS(1891),
    [sym__wc_file_close_int] = ACTIONS(1891),
    [sym__wc_file_read_int] = ACTIONS(1891),
    [sym__wc_file_read_line_int] = ACTIONS(1891),
    [sym__wc_file_write_int] = ACTIONS(1891),
    [sym__wc_file_write_line_int] = ACTIONS(1891),
    [aux_sym_preproc_include_token1] = ACTIONS(1891),
    [aux_sym_preproc_define_token1] = ACTIONS(1893),
    [aux_sym_preproc_warn_token1] = ACTIONS(1893),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1893),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1893),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1893),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1893),
    [aux_sym_preproc_else_token1] = ACTIONS(1893),
    [anon_sym_LBRACK] = ACTIONS(1893),
    [anon_sym_LBRACE] = ACTIONS(1893),
    [anon_sym_LPAREN] = ACTIONS(1891),
    [anon_sym_BANG] = ACTIONS(1893),
    [anon_sym_TILDE] = ACTIONS(1893),
    [anon_sym_DASH] = ACTIONS(1891),
    [anon_sym_PLUS] = ACTIONS(1891),
    [anon_sym_DASH_DASH] = ACTIONS(1893),
    [anon_sym_PLUS_PLUS] = ACTIONS(1893),
    [anon_sym_DQUOTE] = ACTIONS(1893),
    [anon_sym_SQUOTE] = ACTIONS(1893),
    [sym_number_literal] = ACTIONS(1893),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(272)] = {
    [sym_identifier] = ACTIONS(1895),
    [sym_program_name_keyword] = ACTIONS(1895),
    [sym_module_name_keyword] = ACTIONS(1895),
    [sym_define_device_keyword] = ACTIONS(1895),
    [sym_define_combine_keyword] = ACTIONS(1895),
    [sym_define_constant_keyword] = ACTIONS(1895),
    [sym_define_type_keyword] = ACTIONS(1895),
    [sym_define_variable_keyword] = ACTIONS(1895),
    [sym_define_system_variable_keyword] = ACTIONS(1895),
    [sym_define_connect_level_keyword] = ACTIONS(1895),
    [sym_define_latching_keyword] = ACTIONS(1895),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1895),
    [sym_define_toggling_keyword] = ACTIONS(1895),
    [sym_define_call_keyword] = ACTIONS(1895),
    [sym_define_function_keyword] = ACTIONS(1895),
    [sym_define_library_function_keyword] = ACTIONS(1895),
    [sym_define_start_keyword] = ACTIONS(1895),
    [sym_define_event_keyword] = ACTIONS(1895),
    [sym_define_program_keyword] = ACTIONS(1895),
    [sym_define_module_keyword] = ACTIONS(1895),
    [sym_char_keyword] = ACTIONS(1895),
    [sym_widechar_keyword] = ACTIONS(1895),
    [sym_integer_keyword] = ACTIONS(1895),
    [sym_sinteger_keyword] = ACTIONS(1895),
    [sym_long_keyword] = ACTIONS(1895),
    [sym_slong_keyword] = ACTIONS(1895),
    [sym_float_keyword] = ACTIONS(1895),
    [sym_double_keyword] = ACTIONS(1895),
    [sym_variant_keyword] = ACTIONS(1895),
    [sym_variantarray_keyword] = ACTIONS(1895),
    [sym_local_var_keyword] = ACTIONS(1895),
    [sym_stack_var_keyword] = ACTIONS(1895),
    [sym_constant_keyword] = ACTIONS(1895),
    [sym_volatile_keyword] = ACTIONS(1895),
    [sym_non_volatile_keyword] = ACTIONS(1895),
    [sym_persistent_keyword] = ACTIONS(1895),
    [sym_dev_keyword] = ACTIONS(1895),
    [sym_devlev_keyword] = ACTIONS(1895),
    [sym_devchan_keyword] = ACTIONS(1895),
    [sym_if_keyword] = ACTIONS(1895),
    [sym_switch_keyword] = ACTIONS(1895),
    [sym_case_keyword] = ACTIONS(1895),
    [sym_default_keyword] = ACTIONS(1895),
    [sym_while_keyword] = ACTIONS(1895),
    [sym_for_keyword] = ACTIONS(1895),
    [sym_break_keyword] = ACTIONS(1895),
    [sym_continue_keyword] = ACTIONS(1895),
    [sym_return_keyword] = ACTIONS(1895),
    [sym_select_keyword] = ACTIONS(1895),
    [sym_active_keyword] = ACTIONS(1895),
    [sym_struct_keyword] = ACTIONS(1895),
    [sym_structure_keyword] = ACTIONS(1895),
    [sym_button_event_keyword] = ACTIONS(1895),
    [sym_channel_event_keyword] = ACTIONS(1895),
    [sym_level_event_keyword] = ACTIONS(1895),
    [sym_data_event_keyword] = ACTIONS(1895),
    [sym_timeline_event_keyword] = ACTIONS(1895),
    [sym_custom_event_keyword] = ACTIONS(1895),
    [sym_push_keyword] = ACTIONS(1895),
    [sym_release_keyword] = ACTIONS(1895),
    [aux_sym_on_keyword_token1] = ACTIONS(1895),
    [aux_sym_off_keyword_token1] = ACTIONS(1895),
    [sym_send_level_keyword] = ACTIONS(1895),
    [sym_send_string_keyword] = ACTIONS(1895),
    [sym_send_command_keyword] = ACTIONS(1895),
    [sym_clear_buffer_keyword] = ACTIONS(1895),
    [sym_create_buffer_keyword] = ACTIONS(1895),
    [sym_create_multi_buffer_keyword] = ACTIONS(1895),
    [sym_call_keyword] = ACTIONS(1895),
    [sym_system_call_keyword] = ACTIONS(1895),
    [sym_devchan_to_keyword] = ACTIONS(1895),
    [sym_devchan_min_to_keyword] = ACTIONS(1895),
    [sym_devchan_total_off_keyword] = ACTIONS(1895),
    [sym_devchan_pulse_keyword] = ACTIONS(1895),
    [sym_wait_keyword] = ACTIONS(1895),
    [sym_cancel_wait_keyword] = ACTIONS(1895),
    [sym_wait_until_keyword] = ACTIONS(1895),
    [sym_cancel_wait_until_keyword] = ACTIONS(1895),
    [sym_cancel_all_wait_keyword] = ACTIONS(1895),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1895),
    [sym_not] = ACTIONS(1895),
    [sym_bnot] = ACTIONS(1895),
    [sym___file__] = ACTIONS(1895),
    [sym___date__] = ACTIONS(1895),
    [sym___ldate__] = ACTIONS(1895),
    [sym___time__] = ACTIONS(1895),
    [sym___line__] = ACTIONS(1895),
    [sym___name__] = ACTIONS(1895),
    [sym___netlinx__] = ACTIONS(1895),
    [sym_netlinx_axi_version] = ACTIONS(1895),
    [sym_url_flg_tcp] = ACTIONS(1895),
    [sym_url_flg_accinfopresent] = ACTIONS(1895),
    [sym_url_flg_temp] = ACTIONS(1895),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1895),
    [sym_url_flg_stat_mask] = ACTIONS(1895),
    [sym_url_flg_stat_lookup] = ACTIONS(1895),
    [sym_url_flg_stat_connecting] = ACTIONS(1895),
    [sym_url_flg_stat_waiting] = ACTIONS(1895),
    [sym_url_flg_stat_connected] = ACTIONS(1895),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1895),
    [sym_first_virtual_device] = ACTIONS(1895),
    [sym_first_local_port] = ACTIONS(1895),
    [sym_dynamic_virtual_device] = ACTIONS(1895),
    [sym_timeline_once] = ACTIONS(1895),
    [sym_timeline_repeat] = ACTIONS(1895),
    [sym_timeline_absolute] = ACTIONS(1895),
    [sym_timeline_relative] = ACTIONS(1895),
    [sym_do_push_timed_infinite] = ACTIONS(1895),
    [sym_source_type_no_address] = ACTIONS(1895),
    [sym_source_type_neuron_id] = ACTIONS(1895),
    [sym_source_type_ip_address] = ACTIONS(1895),
    [sym_source_type_axlink] = ACTIONS(1895),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1895),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1895),
    [sym_source_type_ip_socket_address] = ACTIONS(1895),
    [sym_source_type_rs232] = ACTIONS(1895),
    [sym_source_type_internal] = ACTIONS(1895),
    [sym_source_type_mac_address] = ACTIONS(1895),
    [sym_source_type_ipv4_port] = ACTIONS(1895),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1895),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1895),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1895),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1895),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1895),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1895),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1895),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1895),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1895),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1895),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1895),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1895),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1895),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1895),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1895),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1895),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1895),
    [sym_true] = ACTIONS(1895),
    [sym_false] = ACTIONS(1895),
    [sym_file_read_only] = ACTIONS(1895),
    [sym_file_rw_new] = ACTIONS(1895),
    [sym_file_rw_append] = ACTIONS(1895),
    [sym_ip_tcp] = ACTIONS(1895),
    [sym_ip_udp] = ACTIONS(1895),
    [sym_ip_udp_2way] = ACTIONS(1895),
    [sym_xml_encode_types] = ACTIONS(1895),
    [sym_xml_encode_char_as_list] = ACTIONS(1895),
    [sym_xml_encode_le] = ACTIONS(1895),
    [sym_xml_decode_types] = ACTIONS(1895),
    [sym_xml_decode_no_preserve] = ACTIONS(1895),
    [sym_duet_dev_type_display_device] = ACTIONS(1895),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1895),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1895),
    [sym_duet_dev_type_io_device] = ACTIONS(1895),
    [sym_duet_dev_type_relay_device] = ACTIONS(1895),
    [sym_duet_dev_type_ups] = ACTIONS(1895),
    [sym_duet_dev_type_amplifier] = ACTIONS(1895),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1895),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1895),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1895),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1895),
    [sym_duet_dev_type_camera] = ACTIONS(1895),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1895),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1895),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1895),
    [sym_duet_dev_type_dss] = ACTIONS(1895),
    [sym_duet_dev_type_dvr] = ACTIONS(1895),
    [sym_duet_dev_type_disc_device] = ACTIONS(1895),
    [sym_duet_dev_type_document_camera] = ACTIONS(1895),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1895),
    [sym_duet_dev_type_hvac] = ACTIONS(1895),
    [sym_duet_dev_type_keypad] = ACTIONS(1895),
    [sym_duet_dev_type_light] = ACTIONS(1895),
    [sym_duet_dev_type_monitor] = ACTIONS(1895),
    [sym_duet_dev_type_motor] = ACTIONS(1895),
    [sym_duet_dev_type_multi_window] = ACTIONS(1895),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1895),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1895),
    [sym_duet_dev_type_receiver] = ACTIONS(1895),
    [sym_duet_dev_type_security_system] = ACTIONS(1895),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1895),
    [sym_duet_dev_type_settop_box] = ACTIONS(1895),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1895),
    [sym_duet_dev_type_switcher] = ACTIONS(1895),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1895),
    [sym_duet_dev_type_tv] = ACTIONS(1895),
    [sym_duet_dev_type_utility] = ACTIONS(1895),
    [sym_duet_dev_type_vcr] = ACTIONS(1895),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1895),
    [sym_duet_dev_type_video_processor] = ACTIONS(1895),
    [sym_duet_dev_type_video_projector] = ACTIONS(1895),
    [sym_duet_dev_type_video_wall] = ACTIONS(1895),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1895),
    [sym_duet_dev_type_weather] = ACTIONS(1895),
    [sym_duet_dev_not_polled] = ACTIONS(1895),
    [sym_duet_dev_polled] = ACTIONS(1895),
    [sym_clkmgr_mode_network] = ACTIONS(1895),
    [sym_clkmgr_mode_standalone] = ACTIONS(1895),
    [sym_normal_standby] = ACTIONS(1895),
    [sym_normal_wake] = ACTIONS(1895),
    [sym_ip_multicast_ttl_option] = ACTIONS(1895),
    [sym_ip_ttl_subnet] = ACTIONS(1895),
    [sym_ip_ttl_site] = ACTIONS(1895),
    [sym_ip_ttl_region] = ACTIONS(1895),
    [sym_ip_ttl_continent] = ACTIONS(1895),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1895),
    [sym_ip_nodelay_on] = ACTIONS(1895),
    [sym_ip_nodelay_off] = ACTIONS(1895),
    [sym_amx_error] = ACTIONS(1895),
    [sym_amx_warning] = ACTIONS(1895),
    [sym_amx_info] = ACTIONS(1895),
    [sym_amx_debug] = ACTIONS(1895),
    [sym_netlinx_login_success] = ACTIONS(1895),
    [sym_netlinx_login_fail] = ACTIONS(1895),
    [sym_netlinx_logout] = ACTIONS(1895),
    [sym_null_str] = ACTIONS(1895),
    [sym_smtp_address] = ACTIONS(1895),
    [sym_smtp_port_number] = ACTIONS(1895),
    [sym_smtp_username] = ACTIONS(1895),
    [sym_smtp_password] = ACTIONS(1895),
    [sym_smtp_require_tls] = ACTIONS(1895),
    [sym_smtp_from] = ACTIONS(1895),
    [sym_smtp_tls_true] = ACTIONS(1895),
    [sym_smtp_tls_false] = ACTIONS(1895),
    [sym_listview_on_row_select_event] = ACTIONS(1895),
    [sym_data_structure_unknown] = ACTIONS(1895),
    [sym_data_structure_datafeed] = ACTIONS(1895),
    [sym_data_structure_datarecordset] = ACTIONS(1895),
    [sym_data_structure_datarecord] = ACTIONS(1895),
    [sym_data_structure_datafield] = ACTIONS(1895),
    [sym_data_max_content_field_count] = ACTIONS(1895),
    [sym_data_max_metadata_field_count] = ACTIONS(1895),
    [sym_data_max_id_length] = ACTIONS(1895),
    [sym_data_max_name_length] = ACTIONS(1895),
    [sym_data_max_description_length] = ACTIONS(1895),
    [sym_data_max_type_length] = ACTIONS(1895),
    [sym_data_max_format_length] = ACTIONS(1895),
    [sym_data_max_source_length] = ACTIONS(1895),
    [sym_data_max_value_length] = ACTIONS(1895),
    [sym_data_max_label_length] = ACTIONS(1895),
    [sym_data_max_url_length] = ACTIONS(1895),
    [sym_max_last_login_info_length] = ACTIONS(1895),
    [sym_data_type_unknown] = ACTIONS(1895),
    [sym_data_type_string] = ACTIONS(1895),
    [sym_data_type_datetime] = ACTIONS(1895),
    [sym_data_type_date] = ACTIONS(1895),
    [sym_data_type_time] = ACTIONS(1895),
    [sym_data_type_image] = ACTIONS(1895),
    [sym_data_format_url] = ACTIONS(1895),
    [sym_data_format_phone] = ACTIONS(1895),
    [sym_data_format_email] = ACTIONS(1895),
    [sym_data_format_iso8601] = ACTIONS(1895),
    [sym_auditlog_priv] = ACTIONS(1895),
    [sym_configuration_priv] = ACTIONS(1895),
    [sym_deviceconfig_priv] = ACTIONS(1895),
    [sym_ftp_priv] = ACTIONS(1895),
    [sym_http_priv] = ACTIONS(1895),
    [sym_networkconfig_priv] = ACTIONS(1895),
    [sym_programport_priv] = ACTIONS(1895),
    [sym_remoteui_priv] = ACTIONS(1895),
    [sym_securitycontrol_priv] = ACTIONS(1895),
    [sym_softwaremanagement_priv] = ACTIONS(1895),
    [sym_terminal_priv] = ACTIONS(1895),
    [sym_tpadmin_priv] = ACTIONS(1895),
    [sym_user1_priv] = ACTIONS(1895),
    [sym_user2_priv] = ACTIONS(1895),
    [sym_user3_priv] = ACTIONS(1895),
    [sym_user4_priv] = ACTIONS(1895),
    [sym_usermanagement_priv] = ACTIONS(1895),
    [sym_valid_account] = ACTIONS(1895),
    [sym_err_name_invalid] = ACTIONS(1895),
    [sym_err_password_invalid] = ACTIONS(1895),
    [sym_err_invalid_account] = ACTIONS(1895),
    [sym_err_no_authorization] = ACTIONS(1895),
    [sym_err_invalid_parameter] = ACTIONS(1895),
    [sym_err_user_locked_out] = ACTIONS(1895),
    [sym_err_user_account_expired] = ACTIONS(1895),
    [sym_err_general_error] = ACTIONS(1895),
    [sym_err_authentication_error] = ACTIONS(1895),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1895),
    [sym_tls_validate_certificate] = ACTIONS(1895),
    [sym_tdata] = ACTIONS(1895),
    [sym_tchannel] = ACTIONS(1895),
    [sym_tlevel] = ACTIONS(1895),
    [sym_tbutton] = ACTIONS(1895),
    [sym_ttimeline] = ACTIONS(1895),
    [sym_tcustom] = ACTIONS(1895),
    [sym_url_struct] = ACTIONS(1895),
    [sym_dns_struct] = ACTIONS(1895),
    [sym_ip_address_struct] = ACTIONS(1895),
    [sym_dev_info_struct] = ACTIONS(1895),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1895),
    [sym_data_feed] = ACTIONS(1895),
    [sym_data_field] = ACTIONS(1895),
    [sym_data_record] = ACTIONS(1895),
    [sym_wc_data_feed] = ACTIONS(1895),
    [sym_wc_data_field] = ACTIONS(1895),
    [sym_wc_data_record] = ACTIONS(1895),
    [sym_last_login_info] = ACTIONS(1895),
    [sym____reserved___] = ACTIONS(1895),
    [sym_date] = ACTIONS(1895),
    [sym_day] = ACTIONS(1895),
    [sym_time] = ACTIONS(1895),
    [sym_ldate] = ACTIONS(1895),
    [sym_push_device] = ACTIONS(1895),
    [sym_push_channel] = ACTIONS(1895),
    [sym_push_devchan] = ACTIONS(1895),
    [sym_release_device] = ACTIONS(1895),
    [sym_release_channel] = ACTIONS(1895),
    [sym_release_devchan] = ACTIONS(1895),
    [sym_master_slot] = ACTIONS(1895),
    [sym_get_pulse_time] = ACTIONS(1895),
    [sym_get_timer] = ACTIONS(1895),
    [sym_master_sn] = ACTIONS(1895),
    [sym_system_number] = ACTIONS(1895),
    [sym_button] = ACTIONS(1895),
    [sym_dv_channel] = ACTIONS(1895),
    [sym_data] = ACTIONS(1895),
    [sym_level] = ACTIONS(1895),
    [sym_timeline] = ACTIONS(1895),
    [sym_channel] = ACTIONS(1895),
    [sym_custom] = ACTIONS(1895),
    [sym_atoi] = ACTIONS(1895),
    [sym_atol] = ACTIONS(1895),
    [sym_atof] = ACTIONS(1895),
    [sym_itoa] = ACTIONS(1895),
    [sym_itohex] = ACTIONS(1895),
    [sym_hextoi] = ACTIONS(1895),
    [sym_ftoa] = ACTIONS(1895),
    [sym_find_string] = ACTIONS(1895),
    [sym_left_string] = ACTIONS(1895),
    [sym_right_string] = ACTIONS(1895),
    [sym_mid_string] = ACTIONS(1895),
    [sym_remove_string] = ACTIONS(1895),
    [sym_get_buffer_string] = ACTIONS(1895),
    [sym_get_buffer_char] = ACTIONS(1895),
    [sym_get_multi_buffer_string] = ACTIONS(1895),
    [sym_length_string] = ACTIONS(1895),
    [sym_max_length_string] = ACTIONS(1895),
    [sym_set_length_string] = ACTIONS(1895),
    [sym_compare_string] = ACTIONS(1895),
    [sym_upper_string] = ACTIONS(1895),
    [sym_lower_string] = ACTIONS(1895),
    [sym_get_last] = ACTIONS(1895),
    [sym_date_to_day] = ACTIONS(1895),
    [sym_date_to_month] = ACTIONS(1895),
    [sym_date_to_year] = ACTIONS(1895),
    [sym_time_to_hour] = ACTIONS(1895),
    [sym_time_to_minute] = ACTIONS(1895),
    [sym_time_to_second] = ACTIONS(1895),
    [sym_day_of_week] = ACTIONS(1895),
    [sym_set_pulse_time] = ACTIONS(1895),
    [sym_set_timer] = ACTIONS(1895),
    [sym_file_open] = ACTIONS(1895),
    [sym_file_close] = ACTIONS(1895),
    [sym_file_read] = ACTIONS(1895),
    [sym_file_read_line] = ACTIONS(1895),
    [sym_file_write] = ACTIONS(1895),
    [sym_file_write_line] = ACTIONS(1895),
    [sym_file_copy] = ACTIONS(1895),
    [sym_file_delete] = ACTIONS(1895),
    [sym_file_rename] = ACTIONS(1895),
    [sym_file_dir] = ACTIONS(1895),
    [sym_file_getdir] = ACTIONS(1895),
    [sym_file_setdir] = ACTIONS(1895),
    [sym_file_create_dir] = ACTIONS(1895),
    [sym_file_remove_dir] = ACTIONS(1895),
    [sym_file_seek] = ACTIONS(1895),
    [sym_length_array] = ACTIONS(1895),
    [sym_max_length_array] = ACTIONS(1895),
    [sym_set_length_array] = ACTIONS(1895),
    [sym_ip_client_open] = ACTIONS(1895),
    [sym_ip_client_close] = ACTIONS(1895),
    [sym_ip_server_open] = ACTIONS(1895),
    [sym_ip_server_close] = ACTIONS(1895),
    [sym_ip_bound_client_open] = ACTIONS(1895),
    [sym_ip_mc_server_open] = ACTIONS(1895),
    [sym_ip_set_option] = ACTIONS(1895),
    [sym_ssh_client_open] = ACTIONS(1895),
    [sym_ssh_client_close] = ACTIONS(1895),
    [sym_tls_client_open] = ACTIONS(1895),
    [sym_tls_client_close] = ACTIONS(1895),
    [sym_do_push] = ACTIONS(1895),
    [sym_do_push_timed] = ACTIONS(1895),
    [sym_do_release] = ACTIONS(1895),
    [sym_device_id] = ACTIONS(1895),
    [sym_device_info] = ACTIONS(1895),
    [sym_device_id_string] = ACTIONS(1895),
    [sym_device_standby] = ACTIONS(1895),
    [sym_device_wake] = ACTIONS(1895),
    [sym_timeline_create] = ACTIONS(1895),
    [sym_timeline_kill] = ACTIONS(1895),
    [sym_timeline_reload] = ACTIONS(1895),
    [sym_timeline_pause] = ACTIONS(1895),
    [sym_timeline_restart] = ACTIONS(1895),
    [sym_timeline_set] = ACTIONS(1895),
    [sym_timeline_get] = ACTIONS(1895),
    [sym_timeline_active] = ACTIONS(1895),
    [sym_timeline_dynamic_id] = ACTIONS(1895),
    [sym_variable_to_string] = ACTIONS(1895),
    [sym_string_to_variable] = ACTIONS(1895),
    [sym_length_variable_to_string] = ACTIONS(1895),
    [sym_type_cast] = ACTIONS(1895),
    [sym_variable_to_xml] = ACTIONS(1895),
    [sym_xml_to_variable] = ACTIONS(1895),
    [sym_length_variable_to_xml] = ACTIONS(1895),
    [sym_redirect_string] = ACTIONS(1895),
    [sym_random_number] = ACTIONS(1895),
    [sym_get_unique_id] = ACTIONS(1895),
    [sym_get_system_number] = ACTIONS(1895),
    [sym_set_system_number] = ACTIONS(1895),
    [sym_get_serial_number] = ACTIONS(1895),
    [sym_get_master_build] = ACTIONS(1895),
    [sym_set_validation_code] = ACTIONS(1895),
    [sym_reboot] = ACTIONS(1895),
    [sym_load_duet_module] = ACTIONS(1895),
    [sym_unload_duet_module] = ACTIONS(1895),
    [sym_duet_mem_size_set] = ACTIONS(1895),
    [sym_duet_mem_size_get] = ACTIONS(1895),
    [sym_dynamic_application_device] = ACTIONS(1895),
    [sym_dynamic_polled_port] = ACTIONS(1895),
    [sym_static_port_binding] = ACTIONS(1895),
    [sym_static_ip_binding] = ACTIONS(1895),
    [sym_get_dns_list] = ACTIONS(1895),
    [sym_set_dns_list] = ACTIONS(1895),
    [sym_get_ip_address] = ACTIONS(1895),
    [sym_set_ip_address] = ACTIONS(1895),
    [sym_get_url_list] = ACTIONS(1895),
    [sym_add_url_entry] = ACTIONS(1895),
    [sym_delete_url_entry] = ACTIONS(1895),
    [sym_abs_value] = ACTIONS(1895),
    [sym_max_value] = ACTIONS(1895),
    [sym_min_value] = ACTIONS(1895),
    [sym_exp_value] = ACTIONS(1895),
    [sym_log_value] = ACTIONS(1895),
    [sym_log10_value] = ACTIONS(1895),
    [sym_power_value] = ACTIONS(1895),
    [sym_sqrt_value] = ACTIONS(1895),
    [sym_raw_be] = ACTIONS(1895),
    [sym_raw_le] = ACTIONS(1895),
    [sym_internal_queue_size_set] = ACTIONS(1895),
    [sym_internal_queue_size_get] = ACTIONS(1895),
    [sym_internal_threshold_set] = ACTIONS(1895),
    [sym_internal_threshold_get] = ACTIONS(1895),
    [sym_rebuilt_event] = ACTIONS(1895),
    [sym_do_custom_event] = ACTIONS(1895),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1895),
    [sym_clkmgr_set_clk_source] = ACTIONS(1895),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1895),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1895),
    [sym_clkmgr_get_timezone] = ACTIONS(1895),
    [sym_clkmgr_set_timezone] = ACTIONS(1895),
    [sym_clkmgr_get_resync_period] = ACTIONS(1895),
    [sym_clkmgr_set_resync_period] = ACTIONS(1895),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1895),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1895),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1895),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1895),
    [sym_clkmgr_get_timeservers] = ACTIONS(1895),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1895),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1895),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1895),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1895),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1895),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1895),
    [sym_set_log_level] = ACTIONS(1895),
    [sym_get_log_level] = ACTIONS(1895),
    [sym_amx_log] = ACTIONS(1895),
    [sym_smtp_server_config_set] = ACTIONS(1895),
    [sym_smtp_server_config_get] = ACTIONS(1895),
    [sym_smtp_send] = ACTIONS(1895),
    [sym_validate_netlinx_account] = ACTIONS(1895),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1895),
    [sym_authenticate_certificate] = ACTIONS(1895),
    [sym_audit_netlinx_session_event] = ACTIONS(1895),
    [sym_audit_netlinx_generic_event] = ACTIONS(1895),
    [sym_get_available_flash_disk_space] = ACTIONS(1895),
    [sym_get_max_flash_disk_space] = ACTIONS(1895),
    [sym_data_create_feed] = ACTIONS(1895),
    [sym_data_delete_feed] = ACTIONS(1895),
    [sym_data_publish_feed] = ACTIONS(1895),
    [sym_data_get_published_feed] = ACTIONS(1895),
    [sym_data_add_record] = ACTIONS(1895),
    [sym_data_get_event_record] = ACTIONS(1895),
    [sym__wc_data_create_feed] = ACTIONS(1895),
    [sym__wc_data_add_record] = ACTIONS(1895),
    [sym__wc_data_get_event_record] = ACTIONS(1895),
    [sym_format] = ACTIONS(1895),
    [sym_astro_clock] = ACTIONS(1895),
    [sym_set_outdoor_temperature] = ACTIONS(1895),
    [sym_set_virtual_level_count] = ACTIONS(1895),
    [sym_set_virtual_channel_count] = ACTIONS(1895),
    [sym_set_virtual_port_count] = ACTIONS(1895),
    [sym__wc_encode_int] = ACTIONS(1895),
    [sym__wc_decode_int] = ACTIONS(1895),
    [sym__wc_int] = ACTIONS(1895),
    [sym__wc_to_ch_int] = ACTIONS(1895),
    [sym__ch_to_wc_int] = ACTIONS(1895),
    [sym__wc_find_string_int] = ACTIONS(1895),
    [sym__wc_left_string_int] = ACTIONS(1895),
    [sym__wc_lower_string_int] = ACTIONS(1895),
    [sym__wc_mid_string_int] = ACTIONS(1895),
    [sym__wc_remove_string_int] = ACTIONS(1895),
    [sym__wc_right_string_int] = ACTIONS(1895),
    [sym__wc_upper_string_int] = ACTIONS(1895),
    [sym__wc_compare_string_int] = ACTIONS(1895),
    [sym__wc_get_buffer_char_int] = ACTIONS(1895),
    [sym__wc_get_buffer_string_int] = ACTIONS(1895),
    [sym__wc_concat_string_int] = ACTIONS(1895),
    [sym__wc_file_open_int] = ACTIONS(1895),
    [sym__wc_file_close_int] = ACTIONS(1895),
    [sym__wc_file_read_int] = ACTIONS(1895),
    [sym__wc_file_read_line_int] = ACTIONS(1895),
    [sym__wc_file_write_int] = ACTIONS(1895),
    [sym__wc_file_write_line_int] = ACTIONS(1895),
    [aux_sym_preproc_include_token1] = ACTIONS(1895),
    [aux_sym_preproc_define_token1] = ACTIONS(1897),
    [aux_sym_preproc_warn_token1] = ACTIONS(1897),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1897),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1897),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1897),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1897),
    [aux_sym_preproc_else_token1] = ACTIONS(1897),
    [anon_sym_LBRACK] = ACTIONS(1897),
    [anon_sym_LBRACE] = ACTIONS(1897),
    [anon_sym_LPAREN] = ACTIONS(1895),
    [anon_sym_BANG] = ACTIONS(1897),
    [anon_sym_TILDE] = ACTIONS(1897),
    [anon_sym_DASH] = ACTIONS(1895),
    [anon_sym_PLUS] = ACTIONS(1895),
    [anon_sym_DASH_DASH] = ACTIONS(1897),
    [anon_sym_PLUS_PLUS] = ACTIONS(1897),
    [anon_sym_DQUOTE] = ACTIONS(1897),
    [anon_sym_SQUOTE] = ACTIONS(1897),
    [sym_number_literal] = ACTIONS(1897),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(273)] = {
    [ts_builtin_sym_end] = ACTIONS(1741),
    [sym_identifier] = ACTIONS(1739),
    [sym_program_name_keyword] = ACTIONS(1739),
    [sym_module_name_keyword] = ACTIONS(1739),
    [sym_define_device_keyword] = ACTIONS(1739),
    [sym_define_combine_keyword] = ACTIONS(1739),
    [sym_define_constant_keyword] = ACTIONS(1739),
    [sym_define_type_keyword] = ACTIONS(1739),
    [sym_define_variable_keyword] = ACTIONS(1739),
    [sym_define_system_variable_keyword] = ACTIONS(1739),
    [sym_define_connect_level_keyword] = ACTIONS(1739),
    [sym_define_latching_keyword] = ACTIONS(1739),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1739),
    [sym_define_toggling_keyword] = ACTIONS(1739),
    [sym_define_call_keyword] = ACTIONS(1739),
    [sym_define_function_keyword] = ACTIONS(1739),
    [sym_define_library_function_keyword] = ACTIONS(1739),
    [sym_define_start_keyword] = ACTIONS(1739),
    [sym_define_event_keyword] = ACTIONS(1739),
    [sym_define_program_keyword] = ACTIONS(1739),
    [sym_define_module_keyword] = ACTIONS(1739),
    [sym_char_keyword] = ACTIONS(1739),
    [sym_widechar_keyword] = ACTIONS(1739),
    [sym_integer_keyword] = ACTIONS(1739),
    [sym_sinteger_keyword] = ACTIONS(1739),
    [sym_long_keyword] = ACTIONS(1739),
    [sym_slong_keyword] = ACTIONS(1739),
    [sym_float_keyword] = ACTIONS(1739),
    [sym_double_keyword] = ACTIONS(1739),
    [sym_variant_keyword] = ACTIONS(1739),
    [sym_variantarray_keyword] = ACTIONS(1739),
    [sym_local_var_keyword] = ACTIONS(1739),
    [sym_stack_var_keyword] = ACTIONS(1739),
    [sym_constant_keyword] = ACTIONS(1739),
    [sym_volatile_keyword] = ACTIONS(1739),
    [sym_non_volatile_keyword] = ACTIONS(1739),
    [sym_persistent_keyword] = ACTIONS(1739),
    [sym_dev_keyword] = ACTIONS(1739),
    [sym_devlev_keyword] = ACTIONS(1739),
    [sym_devchan_keyword] = ACTIONS(1739),
    [sym_if_keyword] = ACTIONS(1739),
    [sym_else_keyword] = ACTIONS(1739),
    [sym_switch_keyword] = ACTIONS(1739),
    [sym_case_keyword] = ACTIONS(1739),
    [sym_default_keyword] = ACTIONS(1739),
    [sym_while_keyword] = ACTIONS(1739),
    [sym_for_keyword] = ACTIONS(1739),
    [sym_break_keyword] = ACTIONS(1739),
    [sym_continue_keyword] = ACTIONS(1739),
    [sym_return_keyword] = ACTIONS(1739),
    [sym_select_keyword] = ACTIONS(1739),
    [sym_active_keyword] = ACTIONS(1739),
    [sym_struct_keyword] = ACTIONS(1739),
    [sym_structure_keyword] = ACTIONS(1739),
    [sym_button_event_keyword] = ACTIONS(1739),
    [sym_channel_event_keyword] = ACTIONS(1739),
    [sym_level_event_keyword] = ACTIONS(1739),
    [sym_data_event_keyword] = ACTIONS(1739),
    [sym_timeline_event_keyword] = ACTIONS(1739),
    [sym_custom_event_keyword] = ACTIONS(1739),
    [sym_push_keyword] = ACTIONS(1739),
    [sym_release_keyword] = ACTIONS(1739),
    [aux_sym_on_keyword_token1] = ACTIONS(1739),
    [aux_sym_off_keyword_token1] = ACTIONS(1739),
    [sym_send_level_keyword] = ACTIONS(1739),
    [sym_send_string_keyword] = ACTIONS(1739),
    [sym_send_command_keyword] = ACTIONS(1739),
    [sym_clear_buffer_keyword] = ACTIONS(1739),
    [sym_create_buffer_keyword] = ACTIONS(1739),
    [sym_create_multi_buffer_keyword] = ACTIONS(1739),
    [sym_call_keyword] = ACTIONS(1739),
    [sym_system_call_keyword] = ACTIONS(1739),
    [sym_devchan_to_keyword] = ACTIONS(1739),
    [sym_devchan_min_to_keyword] = ACTIONS(1739),
    [sym_devchan_total_off_keyword] = ACTIONS(1739),
    [sym_devchan_pulse_keyword] = ACTIONS(1739),
    [sym_wait_keyword] = ACTIONS(1739),
    [sym_cancel_wait_keyword] = ACTIONS(1739),
    [sym_wait_until_keyword] = ACTIONS(1739),
    [sym_cancel_wait_until_keyword] = ACTIONS(1739),
    [sym_cancel_all_wait_keyword] = ACTIONS(1739),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1739),
    [sym_not] = ACTIONS(1739),
    [sym_bnot] = ACTIONS(1739),
    [sym___file__] = ACTIONS(1739),
    [sym___date__] = ACTIONS(1739),
    [sym___ldate__] = ACTIONS(1739),
    [sym___time__] = ACTIONS(1739),
    [sym___line__] = ACTIONS(1739),
    [sym___name__] = ACTIONS(1739),
    [sym___netlinx__] = ACTIONS(1739),
    [sym_netlinx_axi_version] = ACTIONS(1739),
    [sym_url_flg_tcp] = ACTIONS(1739),
    [sym_url_flg_accinfopresent] = ACTIONS(1739),
    [sym_url_flg_temp] = ACTIONS(1739),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1739),
    [sym_url_flg_stat_mask] = ACTIONS(1739),
    [sym_url_flg_stat_lookup] = ACTIONS(1739),
    [sym_url_flg_stat_connecting] = ACTIONS(1739),
    [sym_url_flg_stat_waiting] = ACTIONS(1739),
    [sym_url_flg_stat_connected] = ACTIONS(1739),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1739),
    [sym_first_virtual_device] = ACTIONS(1739),
    [sym_first_local_port] = ACTIONS(1739),
    [sym_dynamic_virtual_device] = ACTIONS(1739),
    [sym_timeline_once] = ACTIONS(1739),
    [sym_timeline_repeat] = ACTIONS(1739),
    [sym_timeline_absolute] = ACTIONS(1739),
    [sym_timeline_relative] = ACTIONS(1739),
    [sym_do_push_timed_infinite] = ACTIONS(1739),
    [sym_source_type_no_address] = ACTIONS(1739),
    [sym_source_type_neuron_id] = ACTIONS(1739),
    [sym_source_type_ip_address] = ACTIONS(1739),
    [sym_source_type_axlink] = ACTIONS(1739),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1739),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1739),
    [sym_source_type_ip_socket_address] = ACTIONS(1739),
    [sym_source_type_rs232] = ACTIONS(1739),
    [sym_source_type_internal] = ACTIONS(1739),
    [sym_source_type_mac_address] = ACTIONS(1739),
    [sym_source_type_ipv4_port] = ACTIONS(1739),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1739),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1739),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1739),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1739),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1739),
    [sym_true] = ACTIONS(1739),
    [sym_false] = ACTIONS(1739),
    [sym_file_read_only] = ACTIONS(1739),
    [sym_file_rw_new] = ACTIONS(1739),
    [sym_file_rw_append] = ACTIONS(1739),
    [sym_ip_tcp] = ACTIONS(1739),
    [sym_ip_udp] = ACTIONS(1739),
    [sym_ip_udp_2way] = ACTIONS(1739),
    [sym_xml_encode_types] = ACTIONS(1739),
    [sym_xml_encode_char_as_list] = ACTIONS(1739),
    [sym_xml_encode_le] = ACTIONS(1739),
    [sym_xml_decode_types] = ACTIONS(1739),
    [sym_xml_decode_no_preserve] = ACTIONS(1739),
    [sym_duet_dev_type_display_device] = ACTIONS(1739),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1739),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1739),
    [sym_duet_dev_type_io_device] = ACTIONS(1739),
    [sym_duet_dev_type_relay_device] = ACTIONS(1739),
    [sym_duet_dev_type_ups] = ACTIONS(1739),
    [sym_duet_dev_type_amplifier] = ACTIONS(1739),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1739),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1739),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1739),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1739),
    [sym_duet_dev_type_camera] = ACTIONS(1739),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1739),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1739),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1739),
    [sym_duet_dev_type_dss] = ACTIONS(1739),
    [sym_duet_dev_type_dvr] = ACTIONS(1739),
    [sym_duet_dev_type_disc_device] = ACTIONS(1739),
    [sym_duet_dev_type_document_camera] = ACTIONS(1739),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1739),
    [sym_duet_dev_type_hvac] = ACTIONS(1739),
    [sym_duet_dev_type_keypad] = ACTIONS(1739),
    [sym_duet_dev_type_light] = ACTIONS(1739),
    [sym_duet_dev_type_monitor] = ACTIONS(1739),
    [sym_duet_dev_type_motor] = ACTIONS(1739),
    [sym_duet_dev_type_multi_window] = ACTIONS(1739),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1739),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1739),
    [sym_duet_dev_type_receiver] = ACTIONS(1739),
    [sym_duet_dev_type_security_system] = ACTIONS(1739),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1739),
    [sym_duet_dev_type_settop_box] = ACTIONS(1739),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1739),
    [sym_duet_dev_type_switcher] = ACTIONS(1739),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1739),
    [sym_duet_dev_type_tv] = ACTIONS(1739),
    [sym_duet_dev_type_utility] = ACTIONS(1739),
    [sym_duet_dev_type_vcr] = ACTIONS(1739),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1739),
    [sym_duet_dev_type_video_processor] = ACTIONS(1739),
    [sym_duet_dev_type_video_projector] = ACTIONS(1739),
    [sym_duet_dev_type_video_wall] = ACTIONS(1739),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1739),
    [sym_duet_dev_type_weather] = ACTIONS(1739),
    [sym_duet_dev_not_polled] = ACTIONS(1739),
    [sym_duet_dev_polled] = ACTIONS(1739),
    [sym_clkmgr_mode_network] = ACTIONS(1739),
    [sym_clkmgr_mode_standalone] = ACTIONS(1739),
    [sym_normal_standby] = ACTIONS(1739),
    [sym_normal_wake] = ACTIONS(1739),
    [sym_ip_multicast_ttl_option] = ACTIONS(1739),
    [sym_ip_ttl_subnet] = ACTIONS(1739),
    [sym_ip_ttl_site] = ACTIONS(1739),
    [sym_ip_ttl_region] = ACTIONS(1739),
    [sym_ip_ttl_continent] = ACTIONS(1739),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1739),
    [sym_ip_nodelay_on] = ACTIONS(1739),
    [sym_ip_nodelay_off] = ACTIONS(1739),
    [sym_amx_error] = ACTIONS(1739),
    [sym_amx_warning] = ACTIONS(1739),
    [sym_amx_info] = ACTIONS(1739),
    [sym_amx_debug] = ACTIONS(1739),
    [sym_netlinx_login_success] = ACTIONS(1739),
    [sym_netlinx_login_fail] = ACTIONS(1739),
    [sym_netlinx_logout] = ACTIONS(1739),
    [sym_null_str] = ACTIONS(1739),
    [sym_smtp_address] = ACTIONS(1739),
    [sym_smtp_port_number] = ACTIONS(1739),
    [sym_smtp_username] = ACTIONS(1739),
    [sym_smtp_password] = ACTIONS(1739),
    [sym_smtp_require_tls] = ACTIONS(1739),
    [sym_smtp_from] = ACTIONS(1739),
    [sym_smtp_tls_true] = ACTIONS(1739),
    [sym_smtp_tls_false] = ACTIONS(1739),
    [sym_listview_on_row_select_event] = ACTIONS(1739),
    [sym_data_structure_unknown] = ACTIONS(1739),
    [sym_data_structure_datafeed] = ACTIONS(1739),
    [sym_data_structure_datarecordset] = ACTIONS(1739),
    [sym_data_structure_datarecord] = ACTIONS(1739),
    [sym_data_structure_datafield] = ACTIONS(1739),
    [sym_data_max_content_field_count] = ACTIONS(1739),
    [sym_data_max_metadata_field_count] = ACTIONS(1739),
    [sym_data_max_id_length] = ACTIONS(1739),
    [sym_data_max_name_length] = ACTIONS(1739),
    [sym_data_max_description_length] = ACTIONS(1739),
    [sym_data_max_type_length] = ACTIONS(1739),
    [sym_data_max_format_length] = ACTIONS(1739),
    [sym_data_max_source_length] = ACTIONS(1739),
    [sym_data_max_value_length] = ACTIONS(1739),
    [sym_data_max_label_length] = ACTIONS(1739),
    [sym_data_max_url_length] = ACTIONS(1739),
    [sym_max_last_login_info_length] = ACTIONS(1739),
    [sym_data_type_unknown] = ACTIONS(1739),
    [sym_data_type_string] = ACTIONS(1739),
    [sym_data_type_datetime] = ACTIONS(1739),
    [sym_data_type_date] = ACTIONS(1739),
    [sym_data_type_time] = ACTIONS(1739),
    [sym_data_type_image] = ACTIONS(1739),
    [sym_data_format_url] = ACTIONS(1739),
    [sym_data_format_phone] = ACTIONS(1739),
    [sym_data_format_email] = ACTIONS(1739),
    [sym_data_format_iso8601] = ACTIONS(1739),
    [sym_auditlog_priv] = ACTIONS(1739),
    [sym_configuration_priv] = ACTIONS(1739),
    [sym_deviceconfig_priv] = ACTIONS(1739),
    [sym_ftp_priv] = ACTIONS(1739),
    [sym_http_priv] = ACTIONS(1739),
    [sym_networkconfig_priv] = ACTIONS(1739),
    [sym_programport_priv] = ACTIONS(1739),
    [sym_remoteui_priv] = ACTIONS(1739),
    [sym_securitycontrol_priv] = ACTIONS(1739),
    [sym_softwaremanagement_priv] = ACTIONS(1739),
    [sym_terminal_priv] = ACTIONS(1739),
    [sym_tpadmin_priv] = ACTIONS(1739),
    [sym_user1_priv] = ACTIONS(1739),
    [sym_user2_priv] = ACTIONS(1739),
    [sym_user3_priv] = ACTIONS(1739),
    [sym_user4_priv] = ACTIONS(1739),
    [sym_usermanagement_priv] = ACTIONS(1739),
    [sym_valid_account] = ACTIONS(1739),
    [sym_err_name_invalid] = ACTIONS(1739),
    [sym_err_password_invalid] = ACTIONS(1739),
    [sym_err_invalid_account] = ACTIONS(1739),
    [sym_err_no_authorization] = ACTIONS(1739),
    [sym_err_invalid_parameter] = ACTIONS(1739),
    [sym_err_user_locked_out] = ACTIONS(1739),
    [sym_err_user_account_expired] = ACTIONS(1739),
    [sym_err_general_error] = ACTIONS(1739),
    [sym_err_authentication_error] = ACTIONS(1739),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1739),
    [sym_tls_validate_certificate] = ACTIONS(1739),
    [sym_tdata] = ACTIONS(1739),
    [sym_tchannel] = ACTIONS(1739),
    [sym_tlevel] = ACTIONS(1739),
    [sym_tbutton] = ACTIONS(1739),
    [sym_ttimeline] = ACTIONS(1739),
    [sym_tcustom] = ACTIONS(1739),
    [sym_url_struct] = ACTIONS(1739),
    [sym_dns_struct] = ACTIONS(1739),
    [sym_ip_address_struct] = ACTIONS(1739),
    [sym_dev_info_struct] = ACTIONS(1739),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1739),
    [sym_data_feed] = ACTIONS(1739),
    [sym_data_field] = ACTIONS(1739),
    [sym_data_record] = ACTIONS(1739),
    [sym_wc_data_feed] = ACTIONS(1739),
    [sym_wc_data_field] = ACTIONS(1739),
    [sym_wc_data_record] = ACTIONS(1739),
    [sym_last_login_info] = ACTIONS(1739),
    [sym____reserved___] = ACTIONS(1739),
    [sym_date] = ACTIONS(1739),
    [sym_day] = ACTIONS(1739),
    [sym_time] = ACTIONS(1739),
    [sym_ldate] = ACTIONS(1739),
    [sym_push_device] = ACTIONS(1739),
    [sym_push_channel] = ACTIONS(1739),
    [sym_push_devchan] = ACTIONS(1739),
    [sym_release_device] = ACTIONS(1739),
    [sym_release_channel] = ACTIONS(1739),
    [sym_release_devchan] = ACTIONS(1739),
    [sym_master_slot] = ACTIONS(1739),
    [sym_get_pulse_time] = ACTIONS(1739),
    [sym_get_timer] = ACTIONS(1739),
    [sym_master_sn] = ACTIONS(1739),
    [sym_system_number] = ACTIONS(1739),
    [sym_button] = ACTIONS(1739),
    [sym_dv_channel] = ACTIONS(1739),
    [sym_data] = ACTIONS(1739),
    [sym_level] = ACTIONS(1739),
    [sym_timeline] = ACTIONS(1739),
    [sym_channel] = ACTIONS(1739),
    [sym_custom] = ACTIONS(1739),
    [sym_atoi] = ACTIONS(1739),
    [sym_atol] = ACTIONS(1739),
    [sym_atof] = ACTIONS(1739),
    [sym_itoa] = ACTIONS(1739),
    [sym_itohex] = ACTIONS(1739),
    [sym_hextoi] = ACTIONS(1739),
    [sym_ftoa] = ACTIONS(1739),
    [sym_find_string] = ACTIONS(1739),
    [sym_left_string] = ACTIONS(1739),
    [sym_right_string] = ACTIONS(1739),
    [sym_mid_string] = ACTIONS(1739),
    [sym_remove_string] = ACTIONS(1739),
    [sym_get_buffer_string] = ACTIONS(1739),
    [sym_get_buffer_char] = ACTIONS(1739),
    [sym_get_multi_buffer_string] = ACTIONS(1739),
    [sym_length_string] = ACTIONS(1739),
    [sym_max_length_string] = ACTIONS(1739),
    [sym_set_length_string] = ACTIONS(1739),
    [sym_compare_string] = ACTIONS(1739),
    [sym_upper_string] = ACTIONS(1739),
    [sym_lower_string] = ACTIONS(1739),
    [sym_get_last] = ACTIONS(1739),
    [sym_date_to_day] = ACTIONS(1739),
    [sym_date_to_month] = ACTIONS(1739),
    [sym_date_to_year] = ACTIONS(1739),
    [sym_time_to_hour] = ACTIONS(1739),
    [sym_time_to_minute] = ACTIONS(1739),
    [sym_time_to_second] = ACTIONS(1739),
    [sym_day_of_week] = ACTIONS(1739),
    [sym_set_pulse_time] = ACTIONS(1739),
    [sym_set_timer] = ACTIONS(1739),
    [sym_file_open] = ACTIONS(1739),
    [sym_file_close] = ACTIONS(1739),
    [sym_file_read] = ACTIONS(1739),
    [sym_file_read_line] = ACTIONS(1739),
    [sym_file_write] = ACTIONS(1739),
    [sym_file_write_line] = ACTIONS(1739),
    [sym_file_copy] = ACTIONS(1739),
    [sym_file_delete] = ACTIONS(1739),
    [sym_file_rename] = ACTIONS(1739),
    [sym_file_dir] = ACTIONS(1739),
    [sym_file_getdir] = ACTIONS(1739),
    [sym_file_setdir] = ACTIONS(1739),
    [sym_file_create_dir] = ACTIONS(1739),
    [sym_file_remove_dir] = ACTIONS(1739),
    [sym_file_seek] = ACTIONS(1739),
    [sym_length_array] = ACTIONS(1739),
    [sym_max_length_array] = ACTIONS(1739),
    [sym_set_length_array] = ACTIONS(1739),
    [sym_ip_client_open] = ACTIONS(1739),
    [sym_ip_client_close] = ACTIONS(1739),
    [sym_ip_server_open] = ACTIONS(1739),
    [sym_ip_server_close] = ACTIONS(1739),
    [sym_ip_bound_client_open] = ACTIONS(1739),
    [sym_ip_mc_server_open] = ACTIONS(1739),
    [sym_ip_set_option] = ACTIONS(1739),
    [sym_ssh_client_open] = ACTIONS(1739),
    [sym_ssh_client_close] = ACTIONS(1739),
    [sym_tls_client_open] = ACTIONS(1739),
    [sym_tls_client_close] = ACTIONS(1739),
    [sym_do_push] = ACTIONS(1739),
    [sym_do_push_timed] = ACTIONS(1739),
    [sym_do_release] = ACTIONS(1739),
    [sym_device_id] = ACTIONS(1739),
    [sym_device_info] = ACTIONS(1739),
    [sym_device_id_string] = ACTIONS(1739),
    [sym_device_standby] = ACTIONS(1739),
    [sym_device_wake] = ACTIONS(1739),
    [sym_timeline_create] = ACTIONS(1739),
    [sym_timeline_kill] = ACTIONS(1739),
    [sym_timeline_reload] = ACTIONS(1739),
    [sym_timeline_pause] = ACTIONS(1739),
    [sym_timeline_restart] = ACTIONS(1739),
    [sym_timeline_set] = ACTIONS(1739),
    [sym_timeline_get] = ACTIONS(1739),
    [sym_timeline_active] = ACTIONS(1739),
    [sym_timeline_dynamic_id] = ACTIONS(1739),
    [sym_variable_to_string] = ACTIONS(1739),
    [sym_string_to_variable] = ACTIONS(1739),
    [sym_length_variable_to_string] = ACTIONS(1739),
    [sym_type_cast] = ACTIONS(1739),
    [sym_variable_to_xml] = ACTIONS(1739),
    [sym_xml_to_variable] = ACTIONS(1739),
    [sym_length_variable_to_xml] = ACTIONS(1739),
    [sym_redirect_string] = ACTIONS(1739),
    [sym_random_number] = ACTIONS(1739),
    [sym_get_unique_id] = ACTIONS(1739),
    [sym_get_system_number] = ACTIONS(1739),
    [sym_set_system_number] = ACTIONS(1739),
    [sym_get_serial_number] = ACTIONS(1739),
    [sym_get_master_build] = ACTIONS(1739),
    [sym_set_validation_code] = ACTIONS(1739),
    [sym_reboot] = ACTIONS(1739),
    [sym_load_duet_module] = ACTIONS(1739),
    [sym_unload_duet_module] = ACTIONS(1739),
    [sym_duet_mem_size_set] = ACTIONS(1739),
    [sym_duet_mem_size_get] = ACTIONS(1739),
    [sym_dynamic_application_device] = ACTIONS(1739),
    [sym_dynamic_polled_port] = ACTIONS(1739),
    [sym_static_port_binding] = ACTIONS(1739),
    [sym_static_ip_binding] = ACTIONS(1739),
    [sym_get_dns_list] = ACTIONS(1739),
    [sym_set_dns_list] = ACTIONS(1739),
    [sym_get_ip_address] = ACTIONS(1739),
    [sym_set_ip_address] = ACTIONS(1739),
    [sym_get_url_list] = ACTIONS(1739),
    [sym_add_url_entry] = ACTIONS(1739),
    [sym_delete_url_entry] = ACTIONS(1739),
    [sym_abs_value] = ACTIONS(1739),
    [sym_max_value] = ACTIONS(1739),
    [sym_min_value] = ACTIONS(1739),
    [sym_exp_value] = ACTIONS(1739),
    [sym_log_value] = ACTIONS(1739),
    [sym_log10_value] = ACTIONS(1739),
    [sym_power_value] = ACTIONS(1739),
    [sym_sqrt_value] = ACTIONS(1739),
    [sym_raw_be] = ACTIONS(1739),
    [sym_raw_le] = ACTIONS(1739),
    [sym_internal_queue_size_set] = ACTIONS(1739),
    [sym_internal_queue_size_get] = ACTIONS(1739),
    [sym_internal_threshold_set] = ACTIONS(1739),
    [sym_internal_threshold_get] = ACTIONS(1739),
    [sym_rebuilt_event] = ACTIONS(1739),
    [sym_do_custom_event] = ACTIONS(1739),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1739),
    [sym_clkmgr_set_clk_source] = ACTIONS(1739),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1739),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1739),
    [sym_clkmgr_get_timezone] = ACTIONS(1739),
    [sym_clkmgr_set_timezone] = ACTIONS(1739),
    [sym_clkmgr_get_resync_period] = ACTIONS(1739),
    [sym_clkmgr_set_resync_period] = ACTIONS(1739),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1739),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1739),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1739),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1739),
    [sym_clkmgr_get_timeservers] = ACTIONS(1739),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1739),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1739),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1739),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1739),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1739),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1739),
    [sym_set_log_level] = ACTIONS(1739),
    [sym_get_log_level] = ACTIONS(1739),
    [sym_amx_log] = ACTIONS(1739),
    [sym_smtp_server_config_set] = ACTIONS(1739),
    [sym_smtp_server_config_get] = ACTIONS(1739),
    [sym_smtp_send] = ACTIONS(1739),
    [sym_validate_netlinx_account] = ACTIONS(1739),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1739),
    [sym_authenticate_certificate] = ACTIONS(1739),
    [sym_audit_netlinx_session_event] = ACTIONS(1739),
    [sym_audit_netlinx_generic_event] = ACTIONS(1739),
    [sym_get_available_flash_disk_space] = ACTIONS(1739),
    [sym_get_max_flash_disk_space] = ACTIONS(1739),
    [sym_data_create_feed] = ACTIONS(1739),
    [sym_data_delete_feed] = ACTIONS(1739),
    [sym_data_publish_feed] = ACTIONS(1739),
    [sym_data_get_published_feed] = ACTIONS(1739),
    [sym_data_add_record] = ACTIONS(1739),
    [sym_data_get_event_record] = ACTIONS(1739),
    [sym__wc_data_create_feed] = ACTIONS(1739),
    [sym__wc_data_add_record] = ACTIONS(1739),
    [sym__wc_data_get_event_record] = ACTIONS(1739),
    [sym_format] = ACTIONS(1739),
    [sym_astro_clock] = ACTIONS(1739),
    [sym_set_outdoor_temperature] = ACTIONS(1739),
    [sym_set_virtual_level_count] = ACTIONS(1739),
    [sym_set_virtual_channel_count] = ACTIONS(1739),
    [sym_set_virtual_port_count] = ACTIONS(1739),
    [sym__wc_encode_int] = ACTIONS(1739),
    [sym__wc_decode_int] = ACTIONS(1739),
    [sym__wc_int] = ACTIONS(1739),
    [sym__wc_to_ch_int] = ACTIONS(1739),
    [sym__ch_to_wc_int] = ACTIONS(1739),
    [sym__wc_find_string_int] = ACTIONS(1739),
    [sym__wc_left_string_int] = ACTIONS(1739),
    [sym__wc_lower_string_int] = ACTIONS(1739),
    [sym__wc_mid_string_int] = ACTIONS(1739),
    [sym__wc_remove_string_int] = ACTIONS(1739),
    [sym__wc_right_string_int] = ACTIONS(1739),
    [sym__wc_upper_string_int] = ACTIONS(1739),
    [sym__wc_compare_string_int] = ACTIONS(1739),
    [sym__wc_get_buffer_char_int] = ACTIONS(1739),
    [sym__wc_get_buffer_string_int] = ACTIONS(1739),
    [sym__wc_concat_string_int] = ACTIONS(1739),
    [sym__wc_file_open_int] = ACTIONS(1739),
    [sym__wc_file_close_int] = ACTIONS(1739),
    [sym__wc_file_read_int] = ACTIONS(1739),
    [sym__wc_file_read_line_int] = ACTIONS(1739),
    [sym__wc_file_write_int] = ACTIONS(1739),
    [sym__wc_file_write_line_int] = ACTIONS(1739),
    [aux_sym_preproc_include_token1] = ACTIONS(1739),
    [aux_sym_preproc_define_token1] = ACTIONS(1741),
    [aux_sym_preproc_warn_token1] = ACTIONS(1741),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1741),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1741),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1741),
    [anon_sym_LBRACK] = ACTIONS(1741),
    [anon_sym_LBRACE] = ACTIONS(1741),
    [anon_sym_LPAREN] = ACTIONS(1739),
    [anon_sym_BANG] = ACTIONS(1741),
    [anon_sym_TILDE] = ACTIONS(1741),
    [anon_sym_DASH] = ACTIONS(1739),
    [anon_sym_PLUS] = ACTIONS(1739),
    [anon_sym_DASH_DASH] = ACTIONS(1741),
    [anon_sym_PLUS_PLUS] = ACTIONS(1741),
    [anon_sym_DQUOTE] = ACTIONS(1741),
    [anon_sym_SQUOTE] = ACTIONS(1741),
    [sym_number_literal] = ACTIONS(1741),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(274)] = {
    [sym_parameter_list] = STATE(415),
    [ts_builtin_sym_end] = ACTIONS(1799),
    [sym_identifier] = ACTIONS(1797),
    [sym_program_name_keyword] = ACTIONS(1797),
    [sym_module_name_keyword] = ACTIONS(1797),
    [sym_define_device_keyword] = ACTIONS(1797),
    [sym_define_combine_keyword] = ACTIONS(1797),
    [sym_define_constant_keyword] = ACTIONS(1797),
    [sym_define_type_keyword] = ACTIONS(1797),
    [sym_define_variable_keyword] = ACTIONS(1797),
    [sym_define_system_variable_keyword] = ACTIONS(1797),
    [sym_define_connect_level_keyword] = ACTIONS(1797),
    [sym_define_latching_keyword] = ACTIONS(1797),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1797),
    [sym_define_toggling_keyword] = ACTIONS(1797),
    [sym_define_call_keyword] = ACTIONS(1797),
    [sym_define_function_keyword] = ACTIONS(1797),
    [sym_define_library_function_keyword] = ACTIONS(1797),
    [sym_define_start_keyword] = ACTIONS(1797),
    [sym_define_event_keyword] = ACTIONS(1797),
    [sym_define_program_keyword] = ACTIONS(1797),
    [sym_define_module_keyword] = ACTIONS(1797),
    [sym_char_keyword] = ACTIONS(1797),
    [sym_widechar_keyword] = ACTIONS(1797),
    [sym_integer_keyword] = ACTIONS(1797),
    [sym_sinteger_keyword] = ACTIONS(1797),
    [sym_long_keyword] = ACTIONS(1797),
    [sym_slong_keyword] = ACTIONS(1797),
    [sym_float_keyword] = ACTIONS(1797),
    [sym_double_keyword] = ACTIONS(1797),
    [sym_variant_keyword] = ACTIONS(1797),
    [sym_variantarray_keyword] = ACTIONS(1797),
    [sym_local_var_keyword] = ACTIONS(1797),
    [sym_stack_var_keyword] = ACTIONS(1797),
    [sym_constant_keyword] = ACTIONS(1797),
    [sym_volatile_keyword] = ACTIONS(1797),
    [sym_non_volatile_keyword] = ACTIONS(1797),
    [sym_persistent_keyword] = ACTIONS(1797),
    [sym_dev_keyword] = ACTIONS(1797),
    [sym_devlev_keyword] = ACTIONS(1797),
    [sym_devchan_keyword] = ACTIONS(1797),
    [sym_if_keyword] = ACTIONS(1797),
    [sym_switch_keyword] = ACTIONS(1797),
    [sym_case_keyword] = ACTIONS(1797),
    [sym_default_keyword] = ACTIONS(1797),
    [sym_while_keyword] = ACTIONS(1797),
    [sym_for_keyword] = ACTIONS(1797),
    [sym_break_keyword] = ACTIONS(1797),
    [sym_continue_keyword] = ACTIONS(1797),
    [sym_return_keyword] = ACTIONS(1797),
    [sym_select_keyword] = ACTIONS(1797),
    [sym_active_keyword] = ACTIONS(1797),
    [sym_struct_keyword] = ACTIONS(1797),
    [sym_structure_keyword] = ACTIONS(1797),
    [sym_button_event_keyword] = ACTIONS(1797),
    [sym_channel_event_keyword] = ACTIONS(1797),
    [sym_level_event_keyword] = ACTIONS(1797),
    [sym_data_event_keyword] = ACTIONS(1797),
    [sym_timeline_event_keyword] = ACTIONS(1797),
    [sym_custom_event_keyword] = ACTIONS(1797),
    [sym_push_keyword] = ACTIONS(1797),
    [sym_release_keyword] = ACTIONS(1797),
    [aux_sym_on_keyword_token1] = ACTIONS(1797),
    [aux_sym_off_keyword_token1] = ACTIONS(1797),
    [sym_send_level_keyword] = ACTIONS(1797),
    [sym_send_string_keyword] = ACTIONS(1797),
    [sym_send_command_keyword] = ACTIONS(1797),
    [sym_clear_buffer_keyword] = ACTIONS(1797),
    [sym_create_buffer_keyword] = ACTIONS(1797),
    [sym_create_multi_buffer_keyword] = ACTIONS(1797),
    [sym_call_keyword] = ACTIONS(1797),
    [sym_system_call_keyword] = ACTIONS(1797),
    [sym_devchan_to_keyword] = ACTIONS(1797),
    [sym_devchan_min_to_keyword] = ACTIONS(1797),
    [sym_devchan_total_off_keyword] = ACTIONS(1797),
    [sym_devchan_pulse_keyword] = ACTIONS(1797),
    [sym_wait_keyword] = ACTIONS(1797),
    [sym_cancel_wait_keyword] = ACTIONS(1797),
    [sym_wait_until_keyword] = ACTIONS(1797),
    [sym_cancel_wait_until_keyword] = ACTIONS(1797),
    [sym_cancel_all_wait_keyword] = ACTIONS(1797),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1797),
    [sym_not] = ACTIONS(1797),
    [sym_bnot] = ACTIONS(1797),
    [sym___file__] = ACTIONS(1797),
    [sym___date__] = ACTIONS(1797),
    [sym___ldate__] = ACTIONS(1797),
    [sym___time__] = ACTIONS(1797),
    [sym___line__] = ACTIONS(1797),
    [sym___name__] = ACTIONS(1797),
    [sym___netlinx__] = ACTIONS(1797),
    [sym_netlinx_axi_version] = ACTIONS(1797),
    [sym_url_flg_tcp] = ACTIONS(1797),
    [sym_url_flg_accinfopresent] = ACTIONS(1797),
    [sym_url_flg_temp] = ACTIONS(1797),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1797),
    [sym_url_flg_stat_mask] = ACTIONS(1797),
    [sym_url_flg_stat_lookup] = ACTIONS(1797),
    [sym_url_flg_stat_connecting] = ACTIONS(1797),
    [sym_url_flg_stat_waiting] = ACTIONS(1797),
    [sym_url_flg_stat_connected] = ACTIONS(1797),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1797),
    [sym_first_virtual_device] = ACTIONS(1797),
    [sym_first_local_port] = ACTIONS(1797),
    [sym_dynamic_virtual_device] = ACTIONS(1797),
    [sym_timeline_once] = ACTIONS(1797),
    [sym_timeline_repeat] = ACTIONS(1797),
    [sym_timeline_absolute] = ACTIONS(1797),
    [sym_timeline_relative] = ACTIONS(1797),
    [sym_do_push_timed_infinite] = ACTIONS(1797),
    [sym_source_type_no_address] = ACTIONS(1797),
    [sym_source_type_neuron_id] = ACTIONS(1797),
    [sym_source_type_ip_address] = ACTIONS(1797),
    [sym_source_type_axlink] = ACTIONS(1797),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1797),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1797),
    [sym_source_type_ip_socket_address] = ACTIONS(1797),
    [sym_source_type_rs232] = ACTIONS(1797),
    [sym_source_type_internal] = ACTIONS(1797),
    [sym_source_type_mac_address] = ACTIONS(1797),
    [sym_source_type_ipv4_port] = ACTIONS(1797),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1797),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1797),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1797),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1797),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1797),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1797),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1797),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1797),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1797),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1797),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1797),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1797),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1797),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1797),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1797),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1797),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1797),
    [sym_true] = ACTIONS(1797),
    [sym_false] = ACTIONS(1797),
    [sym_file_read_only] = ACTIONS(1797),
    [sym_file_rw_new] = ACTIONS(1797),
    [sym_file_rw_append] = ACTIONS(1797),
    [sym_ip_tcp] = ACTIONS(1797),
    [sym_ip_udp] = ACTIONS(1797),
    [sym_ip_udp_2way] = ACTIONS(1797),
    [sym_xml_encode_types] = ACTIONS(1797),
    [sym_xml_encode_char_as_list] = ACTIONS(1797),
    [sym_xml_encode_le] = ACTIONS(1797),
    [sym_xml_decode_types] = ACTIONS(1797),
    [sym_xml_decode_no_preserve] = ACTIONS(1797),
    [sym_duet_dev_type_display_device] = ACTIONS(1797),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1797),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1797),
    [sym_duet_dev_type_io_device] = ACTIONS(1797),
    [sym_duet_dev_type_relay_device] = ACTIONS(1797),
    [sym_duet_dev_type_ups] = ACTIONS(1797),
    [sym_duet_dev_type_amplifier] = ACTIONS(1797),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1797),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1797),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1797),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1797),
    [sym_duet_dev_type_camera] = ACTIONS(1797),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1797),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1797),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1797),
    [sym_duet_dev_type_dss] = ACTIONS(1797),
    [sym_duet_dev_type_dvr] = ACTIONS(1797),
    [sym_duet_dev_type_disc_device] = ACTIONS(1797),
    [sym_duet_dev_type_document_camera] = ACTIONS(1797),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1797),
    [sym_duet_dev_type_hvac] = ACTIONS(1797),
    [sym_duet_dev_type_keypad] = ACTIONS(1797),
    [sym_duet_dev_type_light] = ACTIONS(1797),
    [sym_duet_dev_type_monitor] = ACTIONS(1797),
    [sym_duet_dev_type_motor] = ACTIONS(1797),
    [sym_duet_dev_type_multi_window] = ACTIONS(1797),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1797),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1797),
    [sym_duet_dev_type_receiver] = ACTIONS(1797),
    [sym_duet_dev_type_security_system] = ACTIONS(1797),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1797),
    [sym_duet_dev_type_settop_box] = ACTIONS(1797),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1797),
    [sym_duet_dev_type_switcher] = ACTIONS(1797),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1797),
    [sym_duet_dev_type_tv] = ACTIONS(1797),
    [sym_duet_dev_type_utility] = ACTIONS(1797),
    [sym_duet_dev_type_vcr] = ACTIONS(1797),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1797),
    [sym_duet_dev_type_video_processor] = ACTIONS(1797),
    [sym_duet_dev_type_video_projector] = ACTIONS(1797),
    [sym_duet_dev_type_video_wall] = ACTIONS(1797),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1797),
    [sym_duet_dev_type_weather] = ACTIONS(1797),
    [sym_duet_dev_not_polled] = ACTIONS(1797),
    [sym_duet_dev_polled] = ACTIONS(1797),
    [sym_clkmgr_mode_network] = ACTIONS(1797),
    [sym_clkmgr_mode_standalone] = ACTIONS(1797),
    [sym_normal_standby] = ACTIONS(1797),
    [sym_normal_wake] = ACTIONS(1797),
    [sym_ip_multicast_ttl_option] = ACTIONS(1797),
    [sym_ip_ttl_subnet] = ACTIONS(1797),
    [sym_ip_ttl_site] = ACTIONS(1797),
    [sym_ip_ttl_region] = ACTIONS(1797),
    [sym_ip_ttl_continent] = ACTIONS(1797),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1797),
    [sym_ip_nodelay_on] = ACTIONS(1797),
    [sym_ip_nodelay_off] = ACTIONS(1797),
    [sym_amx_error] = ACTIONS(1797),
    [sym_amx_warning] = ACTIONS(1797),
    [sym_amx_info] = ACTIONS(1797),
    [sym_amx_debug] = ACTIONS(1797),
    [sym_netlinx_login_success] = ACTIONS(1797),
    [sym_netlinx_login_fail] = ACTIONS(1797),
    [sym_netlinx_logout] = ACTIONS(1797),
    [sym_null_str] = ACTIONS(1797),
    [sym_smtp_address] = ACTIONS(1797),
    [sym_smtp_port_number] = ACTIONS(1797),
    [sym_smtp_username] = ACTIONS(1797),
    [sym_smtp_password] = ACTIONS(1797),
    [sym_smtp_require_tls] = ACTIONS(1797),
    [sym_smtp_from] = ACTIONS(1797),
    [sym_smtp_tls_true] = ACTIONS(1797),
    [sym_smtp_tls_false] = ACTIONS(1797),
    [sym_listview_on_row_select_event] = ACTIONS(1797),
    [sym_data_structure_unknown] = ACTIONS(1797),
    [sym_data_structure_datafeed] = ACTIONS(1797),
    [sym_data_structure_datarecordset] = ACTIONS(1797),
    [sym_data_structure_datarecord] = ACTIONS(1797),
    [sym_data_structure_datafield] = ACTIONS(1797),
    [sym_data_max_content_field_count] = ACTIONS(1797),
    [sym_data_max_metadata_field_count] = ACTIONS(1797),
    [sym_data_max_id_length] = ACTIONS(1797),
    [sym_data_max_name_length] = ACTIONS(1797),
    [sym_data_max_description_length] = ACTIONS(1797),
    [sym_data_max_type_length] = ACTIONS(1797),
    [sym_data_max_format_length] = ACTIONS(1797),
    [sym_data_max_source_length] = ACTIONS(1797),
    [sym_data_max_value_length] = ACTIONS(1797),
    [sym_data_max_label_length] = ACTIONS(1797),
    [sym_data_max_url_length] = ACTIONS(1797),
    [sym_max_last_login_info_length] = ACTIONS(1797),
    [sym_data_type_unknown] = ACTIONS(1797),
    [sym_data_type_string] = ACTIONS(1797),
    [sym_data_type_datetime] = ACTIONS(1797),
    [sym_data_type_date] = ACTIONS(1797),
    [sym_data_type_time] = ACTIONS(1797),
    [sym_data_type_image] = ACTIONS(1797),
    [sym_data_format_url] = ACTIONS(1797),
    [sym_data_format_phone] = ACTIONS(1797),
    [sym_data_format_email] = ACTIONS(1797),
    [sym_data_format_iso8601] = ACTIONS(1797),
    [sym_auditlog_priv] = ACTIONS(1797),
    [sym_configuration_priv] = ACTIONS(1797),
    [sym_deviceconfig_priv] = ACTIONS(1797),
    [sym_ftp_priv] = ACTIONS(1797),
    [sym_http_priv] = ACTIONS(1797),
    [sym_networkconfig_priv] = ACTIONS(1797),
    [sym_programport_priv] = ACTIONS(1797),
    [sym_remoteui_priv] = ACTIONS(1797),
    [sym_securitycontrol_priv] = ACTIONS(1797),
    [sym_softwaremanagement_priv] = ACTIONS(1797),
    [sym_terminal_priv] = ACTIONS(1797),
    [sym_tpadmin_priv] = ACTIONS(1797),
    [sym_user1_priv] = ACTIONS(1797),
    [sym_user2_priv] = ACTIONS(1797),
    [sym_user3_priv] = ACTIONS(1797),
    [sym_user4_priv] = ACTIONS(1797),
    [sym_usermanagement_priv] = ACTIONS(1797),
    [sym_valid_account] = ACTIONS(1797),
    [sym_err_name_invalid] = ACTIONS(1797),
    [sym_err_password_invalid] = ACTIONS(1797),
    [sym_err_invalid_account] = ACTIONS(1797),
    [sym_err_no_authorization] = ACTIONS(1797),
    [sym_err_invalid_parameter] = ACTIONS(1797),
    [sym_err_user_locked_out] = ACTIONS(1797),
    [sym_err_user_account_expired] = ACTIONS(1797),
    [sym_err_general_error] = ACTIONS(1797),
    [sym_err_authentication_error] = ACTIONS(1797),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1797),
    [sym_tls_validate_certificate] = ACTIONS(1797),
    [sym_tdata] = ACTIONS(1797),
    [sym_tchannel] = ACTIONS(1797),
    [sym_tlevel] = ACTIONS(1797),
    [sym_tbutton] = ACTIONS(1797),
    [sym_ttimeline] = ACTIONS(1797),
    [sym_tcustom] = ACTIONS(1797),
    [sym_url_struct] = ACTIONS(1797),
    [sym_dns_struct] = ACTIONS(1797),
    [sym_ip_address_struct] = ACTIONS(1797),
    [sym_dev_info_struct] = ACTIONS(1797),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1797),
    [sym_data_feed] = ACTIONS(1797),
    [sym_data_field] = ACTIONS(1797),
    [sym_data_record] = ACTIONS(1797),
    [sym_wc_data_feed] = ACTIONS(1797),
    [sym_wc_data_field] = ACTIONS(1797),
    [sym_wc_data_record] = ACTIONS(1797),
    [sym_last_login_info] = ACTIONS(1797),
    [sym____reserved___] = ACTIONS(1797),
    [sym_date] = ACTIONS(1797),
    [sym_day] = ACTIONS(1797),
    [sym_time] = ACTIONS(1797),
    [sym_ldate] = ACTIONS(1797),
    [sym_push_device] = ACTIONS(1797),
    [sym_push_channel] = ACTIONS(1797),
    [sym_push_devchan] = ACTIONS(1797),
    [sym_release_device] = ACTIONS(1797),
    [sym_release_channel] = ACTIONS(1797),
    [sym_release_devchan] = ACTIONS(1797),
    [sym_master_slot] = ACTIONS(1797),
    [sym_get_pulse_time] = ACTIONS(1797),
    [sym_get_timer] = ACTIONS(1797),
    [sym_master_sn] = ACTIONS(1797),
    [sym_system_number] = ACTIONS(1797),
    [sym_button] = ACTIONS(1797),
    [sym_dv_channel] = ACTIONS(1797),
    [sym_data] = ACTIONS(1797),
    [sym_level] = ACTIONS(1797),
    [sym_timeline] = ACTIONS(1797),
    [sym_channel] = ACTIONS(1797),
    [sym_custom] = ACTIONS(1797),
    [sym_atoi] = ACTIONS(1797),
    [sym_atol] = ACTIONS(1797),
    [sym_atof] = ACTIONS(1797),
    [sym_itoa] = ACTIONS(1797),
    [sym_itohex] = ACTIONS(1797),
    [sym_hextoi] = ACTIONS(1797),
    [sym_ftoa] = ACTIONS(1797),
    [sym_find_string] = ACTIONS(1797),
    [sym_left_string] = ACTIONS(1797),
    [sym_right_string] = ACTIONS(1797),
    [sym_mid_string] = ACTIONS(1797),
    [sym_remove_string] = ACTIONS(1797),
    [sym_get_buffer_string] = ACTIONS(1797),
    [sym_get_buffer_char] = ACTIONS(1797),
    [sym_get_multi_buffer_string] = ACTIONS(1797),
    [sym_length_string] = ACTIONS(1797),
    [sym_max_length_string] = ACTIONS(1797),
    [sym_set_length_string] = ACTIONS(1797),
    [sym_compare_string] = ACTIONS(1797),
    [sym_upper_string] = ACTIONS(1797),
    [sym_lower_string] = ACTIONS(1797),
    [sym_get_last] = ACTIONS(1797),
    [sym_date_to_day] = ACTIONS(1797),
    [sym_date_to_month] = ACTIONS(1797),
    [sym_date_to_year] = ACTIONS(1797),
    [sym_time_to_hour] = ACTIONS(1797),
    [sym_time_to_minute] = ACTIONS(1797),
    [sym_time_to_second] = ACTIONS(1797),
    [sym_day_of_week] = ACTIONS(1797),
    [sym_set_pulse_time] = ACTIONS(1797),
    [sym_set_timer] = ACTIONS(1797),
    [sym_file_open] = ACTIONS(1797),
    [sym_file_close] = ACTIONS(1797),
    [sym_file_read] = ACTIONS(1797),
    [sym_file_read_line] = ACTIONS(1797),
    [sym_file_write] = ACTIONS(1797),
    [sym_file_write_line] = ACTIONS(1797),
    [sym_file_copy] = ACTIONS(1797),
    [sym_file_delete] = ACTIONS(1797),
    [sym_file_rename] = ACTIONS(1797),
    [sym_file_dir] = ACTIONS(1797),
    [sym_file_getdir] = ACTIONS(1797),
    [sym_file_setdir] = ACTIONS(1797),
    [sym_file_create_dir] = ACTIONS(1797),
    [sym_file_remove_dir] = ACTIONS(1797),
    [sym_file_seek] = ACTIONS(1797),
    [sym_length_array] = ACTIONS(1797),
    [sym_max_length_array] = ACTIONS(1797),
    [sym_set_length_array] = ACTIONS(1797),
    [sym_ip_client_open] = ACTIONS(1797),
    [sym_ip_client_close] = ACTIONS(1797),
    [sym_ip_server_open] = ACTIONS(1797),
    [sym_ip_server_close] = ACTIONS(1797),
    [sym_ip_bound_client_open] = ACTIONS(1797),
    [sym_ip_mc_server_open] = ACTIONS(1797),
    [sym_ip_set_option] = ACTIONS(1797),
    [sym_ssh_client_open] = ACTIONS(1797),
    [sym_ssh_client_close] = ACTIONS(1797),
    [sym_tls_client_open] = ACTIONS(1797),
    [sym_tls_client_close] = ACTIONS(1797),
    [sym_do_push] = ACTIONS(1797),
    [sym_do_push_timed] = ACTIONS(1797),
    [sym_do_release] = ACTIONS(1797),
    [sym_device_id] = ACTIONS(1797),
    [sym_device_info] = ACTIONS(1797),
    [sym_device_id_string] = ACTIONS(1797),
    [sym_device_standby] = ACTIONS(1797),
    [sym_device_wake] = ACTIONS(1797),
    [sym_timeline_create] = ACTIONS(1797),
    [sym_timeline_kill] = ACTIONS(1797),
    [sym_timeline_reload] = ACTIONS(1797),
    [sym_timeline_pause] = ACTIONS(1797),
    [sym_timeline_restart] = ACTIONS(1797),
    [sym_timeline_set] = ACTIONS(1797),
    [sym_timeline_get] = ACTIONS(1797),
    [sym_timeline_active] = ACTIONS(1797),
    [sym_timeline_dynamic_id] = ACTIONS(1797),
    [sym_variable_to_string] = ACTIONS(1797),
    [sym_string_to_variable] = ACTIONS(1797),
    [sym_length_variable_to_string] = ACTIONS(1797),
    [sym_type_cast] = ACTIONS(1797),
    [sym_variable_to_xml] = ACTIONS(1797),
    [sym_xml_to_variable] = ACTIONS(1797),
    [sym_length_variable_to_xml] = ACTIONS(1797),
    [sym_redirect_string] = ACTIONS(1797),
    [sym_random_number] = ACTIONS(1797),
    [sym_get_unique_id] = ACTIONS(1797),
    [sym_get_system_number] = ACTIONS(1797),
    [sym_set_system_number] = ACTIONS(1797),
    [sym_get_serial_number] = ACTIONS(1797),
    [sym_get_master_build] = ACTIONS(1797),
    [sym_set_validation_code] = ACTIONS(1797),
    [sym_reboot] = ACTIONS(1797),
    [sym_load_duet_module] = ACTIONS(1797),
    [sym_unload_duet_module] = ACTIONS(1797),
    [sym_duet_mem_size_set] = ACTIONS(1797),
    [sym_duet_mem_size_get] = ACTIONS(1797),
    [sym_dynamic_application_device] = ACTIONS(1797),
    [sym_dynamic_polled_port] = ACTIONS(1797),
    [sym_static_port_binding] = ACTIONS(1797),
    [sym_static_ip_binding] = ACTIONS(1797),
    [sym_get_dns_list] = ACTIONS(1797),
    [sym_set_dns_list] = ACTIONS(1797),
    [sym_get_ip_address] = ACTIONS(1797),
    [sym_set_ip_address] = ACTIONS(1797),
    [sym_get_url_list] = ACTIONS(1797),
    [sym_add_url_entry] = ACTIONS(1797),
    [sym_delete_url_entry] = ACTIONS(1797),
    [sym_abs_value] = ACTIONS(1797),
    [sym_max_value] = ACTIONS(1797),
    [sym_min_value] = ACTIONS(1797),
    [sym_exp_value] = ACTIONS(1797),
    [sym_log_value] = ACTIONS(1797),
    [sym_log10_value] = ACTIONS(1797),
    [sym_power_value] = ACTIONS(1797),
    [sym_sqrt_value] = ACTIONS(1797),
    [sym_raw_be] = ACTIONS(1797),
    [sym_raw_le] = ACTIONS(1797),
    [sym_internal_queue_size_set] = ACTIONS(1797),
    [sym_internal_queue_size_get] = ACTIONS(1797),
    [sym_internal_threshold_set] = ACTIONS(1797),
    [sym_internal_threshold_get] = ACTIONS(1797),
    [sym_rebuilt_event] = ACTIONS(1797),
    [sym_do_custom_event] = ACTIONS(1797),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1797),
    [sym_clkmgr_set_clk_source] = ACTIONS(1797),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1797),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1797),
    [sym_clkmgr_get_timezone] = ACTIONS(1797),
    [sym_clkmgr_set_timezone] = ACTIONS(1797),
    [sym_clkmgr_get_resync_period] = ACTIONS(1797),
    [sym_clkmgr_set_resync_period] = ACTIONS(1797),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1797),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1797),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1797),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1797),
    [sym_clkmgr_get_timeservers] = ACTIONS(1797),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1797),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1797),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1797),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1797),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1797),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1797),
    [sym_set_log_level] = ACTIONS(1797),
    [sym_get_log_level] = ACTIONS(1797),
    [sym_amx_log] = ACTIONS(1797),
    [sym_smtp_server_config_set] = ACTIONS(1797),
    [sym_smtp_server_config_get] = ACTIONS(1797),
    [sym_smtp_send] = ACTIONS(1797),
    [sym_validate_netlinx_account] = ACTIONS(1797),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1797),
    [sym_authenticate_certificate] = ACTIONS(1797),
    [sym_audit_netlinx_session_event] = ACTIONS(1797),
    [sym_audit_netlinx_generic_event] = ACTIONS(1797),
    [sym_get_available_flash_disk_space] = ACTIONS(1797),
    [sym_get_max_flash_disk_space] = ACTIONS(1797),
    [sym_data_create_feed] = ACTIONS(1797),
    [sym_data_delete_feed] = ACTIONS(1797),
    [sym_data_publish_feed] = ACTIONS(1797),
    [sym_data_get_published_feed] = ACTIONS(1797),
    [sym_data_add_record] = ACTIONS(1797),
    [sym_data_get_event_record] = ACTIONS(1797),
    [sym__wc_data_create_feed] = ACTIONS(1797),
    [sym__wc_data_add_record] = ACTIONS(1797),
    [sym__wc_data_get_event_record] = ACTIONS(1797),
    [sym_format] = ACTIONS(1797),
    [sym_astro_clock] = ACTIONS(1797),
    [sym_set_outdoor_temperature] = ACTIONS(1797),
    [sym_set_virtual_level_count] = ACTIONS(1797),
    [sym_set_virtual_channel_count] = ACTIONS(1797),
    [sym_set_virtual_port_count] = ACTIONS(1797),
    [sym__wc_encode_int] = ACTIONS(1797),
    [sym__wc_decode_int] = ACTIONS(1797),
    [sym__wc_int] = ACTIONS(1797),
    [sym__wc_to_ch_int] = ACTIONS(1797),
    [sym__ch_to_wc_int] = ACTIONS(1797),
    [sym__wc_find_string_int] = ACTIONS(1797),
    [sym__wc_left_string_int] = ACTIONS(1797),
    [sym__wc_lower_string_int] = ACTIONS(1797),
    [sym__wc_mid_string_int] = ACTIONS(1797),
    [sym__wc_remove_string_int] = ACTIONS(1797),
    [sym__wc_right_string_int] = ACTIONS(1797),
    [sym__wc_upper_string_int] = ACTIONS(1797),
    [sym__wc_compare_string_int] = ACTIONS(1797),
    [sym__wc_get_buffer_char_int] = ACTIONS(1797),
    [sym__wc_get_buffer_string_int] = ACTIONS(1797),
    [sym__wc_concat_string_int] = ACTIONS(1797),
    [sym__wc_file_open_int] = ACTIONS(1797),
    [sym__wc_file_close_int] = ACTIONS(1797),
    [sym__wc_file_read_int] = ACTIONS(1797),
    [sym__wc_file_read_line_int] = ACTIONS(1797),
    [sym__wc_file_write_int] = ACTIONS(1797),
    [sym__wc_file_write_line_int] = ACTIONS(1797),
    [aux_sym_preproc_include_token1] = ACTIONS(1797),
    [aux_sym_preproc_define_token1] = ACTIONS(1799),
    [aux_sym_preproc_warn_token1] = ACTIONS(1799),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1799),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1799),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1799),
    [anon_sym_LBRACK] = ACTIONS(1799),
    [anon_sym_LBRACE] = ACTIONS(1799),
    [anon_sym_LPAREN] = ACTIONS(1899),
    [anon_sym_BANG] = ACTIONS(1799),
    [anon_sym_TILDE] = ACTIONS(1799),
    [anon_sym_DASH] = ACTIONS(1797),
    [anon_sym_PLUS] = ACTIONS(1797),
    [anon_sym_DASH_DASH] = ACTIONS(1799),
    [anon_sym_PLUS_PLUS] = ACTIONS(1799),
    [anon_sym_DQUOTE] = ACTIONS(1799),
    [anon_sym_SQUOTE] = ACTIONS(1799),
    [sym_number_literal] = ACTIONS(1799),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(275)] = {
    [sym_identifier] = ACTIONS(1901),
    [sym_program_name_keyword] = ACTIONS(1901),
    [sym_module_name_keyword] = ACTIONS(1901),
    [sym_define_device_keyword] = ACTIONS(1901),
    [sym_define_combine_keyword] = ACTIONS(1901),
    [sym_define_constant_keyword] = ACTIONS(1901),
    [sym_define_type_keyword] = ACTIONS(1901),
    [sym_define_variable_keyword] = ACTIONS(1901),
    [sym_define_system_variable_keyword] = ACTIONS(1901),
    [sym_define_connect_level_keyword] = ACTIONS(1901),
    [sym_define_latching_keyword] = ACTIONS(1901),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1901),
    [sym_define_toggling_keyword] = ACTIONS(1901),
    [sym_define_call_keyword] = ACTIONS(1901),
    [sym_define_function_keyword] = ACTIONS(1901),
    [sym_define_library_function_keyword] = ACTIONS(1901),
    [sym_define_start_keyword] = ACTIONS(1901),
    [sym_define_event_keyword] = ACTIONS(1901),
    [sym_define_program_keyword] = ACTIONS(1901),
    [sym_define_module_keyword] = ACTIONS(1901),
    [sym_char_keyword] = ACTIONS(1901),
    [sym_widechar_keyword] = ACTIONS(1901),
    [sym_integer_keyword] = ACTIONS(1901),
    [sym_sinteger_keyword] = ACTIONS(1901),
    [sym_long_keyword] = ACTIONS(1901),
    [sym_slong_keyword] = ACTIONS(1901),
    [sym_float_keyword] = ACTIONS(1901),
    [sym_double_keyword] = ACTIONS(1901),
    [sym_variant_keyword] = ACTIONS(1901),
    [sym_variantarray_keyword] = ACTIONS(1901),
    [sym_local_var_keyword] = ACTIONS(1901),
    [sym_stack_var_keyword] = ACTIONS(1901),
    [sym_constant_keyword] = ACTIONS(1901),
    [sym_volatile_keyword] = ACTIONS(1901),
    [sym_non_volatile_keyword] = ACTIONS(1901),
    [sym_persistent_keyword] = ACTIONS(1901),
    [sym_dev_keyword] = ACTIONS(1901),
    [sym_devlev_keyword] = ACTIONS(1901),
    [sym_devchan_keyword] = ACTIONS(1901),
    [sym_if_keyword] = ACTIONS(1901),
    [sym_switch_keyword] = ACTIONS(1901),
    [sym_case_keyword] = ACTIONS(1901),
    [sym_default_keyword] = ACTIONS(1901),
    [sym_while_keyword] = ACTIONS(1901),
    [sym_for_keyword] = ACTIONS(1901),
    [sym_break_keyword] = ACTIONS(1901),
    [sym_continue_keyword] = ACTIONS(1901),
    [sym_return_keyword] = ACTIONS(1901),
    [sym_select_keyword] = ACTIONS(1901),
    [sym_active_keyword] = ACTIONS(1901),
    [sym_struct_keyword] = ACTIONS(1901),
    [sym_structure_keyword] = ACTIONS(1901),
    [sym_button_event_keyword] = ACTIONS(1901),
    [sym_channel_event_keyword] = ACTIONS(1901),
    [sym_level_event_keyword] = ACTIONS(1901),
    [sym_data_event_keyword] = ACTIONS(1901),
    [sym_timeline_event_keyword] = ACTIONS(1901),
    [sym_custom_event_keyword] = ACTIONS(1901),
    [sym_push_keyword] = ACTIONS(1901),
    [sym_release_keyword] = ACTIONS(1901),
    [aux_sym_on_keyword_token1] = ACTIONS(1901),
    [aux_sym_off_keyword_token1] = ACTIONS(1901),
    [sym_send_level_keyword] = ACTIONS(1901),
    [sym_send_string_keyword] = ACTIONS(1901),
    [sym_send_command_keyword] = ACTIONS(1901),
    [sym_clear_buffer_keyword] = ACTIONS(1901),
    [sym_create_buffer_keyword] = ACTIONS(1901),
    [sym_create_multi_buffer_keyword] = ACTIONS(1901),
    [sym_call_keyword] = ACTIONS(1901),
    [sym_system_call_keyword] = ACTIONS(1901),
    [sym_devchan_to_keyword] = ACTIONS(1901),
    [sym_devchan_min_to_keyword] = ACTIONS(1901),
    [sym_devchan_total_off_keyword] = ACTIONS(1901),
    [sym_devchan_pulse_keyword] = ACTIONS(1901),
    [sym_wait_keyword] = ACTIONS(1901),
    [sym_cancel_wait_keyword] = ACTIONS(1901),
    [sym_wait_until_keyword] = ACTIONS(1901),
    [sym_cancel_wait_until_keyword] = ACTIONS(1901),
    [sym_cancel_all_wait_keyword] = ACTIONS(1901),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1901),
    [sym_not] = ACTIONS(1901),
    [sym_bnot] = ACTIONS(1901),
    [sym___file__] = ACTIONS(1901),
    [sym___date__] = ACTIONS(1901),
    [sym___ldate__] = ACTIONS(1901),
    [sym___time__] = ACTIONS(1901),
    [sym___line__] = ACTIONS(1901),
    [sym___name__] = ACTIONS(1901),
    [sym___netlinx__] = ACTIONS(1901),
    [sym_netlinx_axi_version] = ACTIONS(1901),
    [sym_url_flg_tcp] = ACTIONS(1901),
    [sym_url_flg_accinfopresent] = ACTIONS(1901),
    [sym_url_flg_temp] = ACTIONS(1901),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1901),
    [sym_url_flg_stat_mask] = ACTIONS(1901),
    [sym_url_flg_stat_lookup] = ACTIONS(1901),
    [sym_url_flg_stat_connecting] = ACTIONS(1901),
    [sym_url_flg_stat_waiting] = ACTIONS(1901),
    [sym_url_flg_stat_connected] = ACTIONS(1901),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1901),
    [sym_first_virtual_device] = ACTIONS(1901),
    [sym_first_local_port] = ACTIONS(1901),
    [sym_dynamic_virtual_device] = ACTIONS(1901),
    [sym_timeline_once] = ACTIONS(1901),
    [sym_timeline_repeat] = ACTIONS(1901),
    [sym_timeline_absolute] = ACTIONS(1901),
    [sym_timeline_relative] = ACTIONS(1901),
    [sym_do_push_timed_infinite] = ACTIONS(1901),
    [sym_source_type_no_address] = ACTIONS(1901),
    [sym_source_type_neuron_id] = ACTIONS(1901),
    [sym_source_type_ip_address] = ACTIONS(1901),
    [sym_source_type_axlink] = ACTIONS(1901),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1901),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1901),
    [sym_source_type_ip_socket_address] = ACTIONS(1901),
    [sym_source_type_rs232] = ACTIONS(1901),
    [sym_source_type_internal] = ACTIONS(1901),
    [sym_source_type_mac_address] = ACTIONS(1901),
    [sym_source_type_ipv4_port] = ACTIONS(1901),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1901),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1901),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1901),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1901),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1901),
    [sym_true] = ACTIONS(1901),
    [sym_false] = ACTIONS(1901),
    [sym_file_read_only] = ACTIONS(1901),
    [sym_file_rw_new] = ACTIONS(1901),
    [sym_file_rw_append] = ACTIONS(1901),
    [sym_ip_tcp] = ACTIONS(1901),
    [sym_ip_udp] = ACTIONS(1901),
    [sym_ip_udp_2way] = ACTIONS(1901),
    [sym_xml_encode_types] = ACTIONS(1901),
    [sym_xml_encode_char_as_list] = ACTIONS(1901),
    [sym_xml_encode_le] = ACTIONS(1901),
    [sym_xml_decode_types] = ACTIONS(1901),
    [sym_xml_decode_no_preserve] = ACTIONS(1901),
    [sym_duet_dev_type_display_device] = ACTIONS(1901),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1901),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1901),
    [sym_duet_dev_type_io_device] = ACTIONS(1901),
    [sym_duet_dev_type_relay_device] = ACTIONS(1901),
    [sym_duet_dev_type_ups] = ACTIONS(1901),
    [sym_duet_dev_type_amplifier] = ACTIONS(1901),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1901),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1901),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1901),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1901),
    [sym_duet_dev_type_camera] = ACTIONS(1901),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1901),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1901),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1901),
    [sym_duet_dev_type_dss] = ACTIONS(1901),
    [sym_duet_dev_type_dvr] = ACTIONS(1901),
    [sym_duet_dev_type_disc_device] = ACTIONS(1901),
    [sym_duet_dev_type_document_camera] = ACTIONS(1901),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1901),
    [sym_duet_dev_type_hvac] = ACTIONS(1901),
    [sym_duet_dev_type_keypad] = ACTIONS(1901),
    [sym_duet_dev_type_light] = ACTIONS(1901),
    [sym_duet_dev_type_monitor] = ACTIONS(1901),
    [sym_duet_dev_type_motor] = ACTIONS(1901),
    [sym_duet_dev_type_multi_window] = ACTIONS(1901),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1901),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1901),
    [sym_duet_dev_type_receiver] = ACTIONS(1901),
    [sym_duet_dev_type_security_system] = ACTIONS(1901),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1901),
    [sym_duet_dev_type_settop_box] = ACTIONS(1901),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1901),
    [sym_duet_dev_type_switcher] = ACTIONS(1901),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1901),
    [sym_duet_dev_type_tv] = ACTIONS(1901),
    [sym_duet_dev_type_utility] = ACTIONS(1901),
    [sym_duet_dev_type_vcr] = ACTIONS(1901),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1901),
    [sym_duet_dev_type_video_processor] = ACTIONS(1901),
    [sym_duet_dev_type_video_projector] = ACTIONS(1901),
    [sym_duet_dev_type_video_wall] = ACTIONS(1901),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1901),
    [sym_duet_dev_type_weather] = ACTIONS(1901),
    [sym_duet_dev_not_polled] = ACTIONS(1901),
    [sym_duet_dev_polled] = ACTIONS(1901),
    [sym_clkmgr_mode_network] = ACTIONS(1901),
    [sym_clkmgr_mode_standalone] = ACTIONS(1901),
    [sym_normal_standby] = ACTIONS(1901),
    [sym_normal_wake] = ACTIONS(1901),
    [sym_ip_multicast_ttl_option] = ACTIONS(1901),
    [sym_ip_ttl_subnet] = ACTIONS(1901),
    [sym_ip_ttl_site] = ACTIONS(1901),
    [sym_ip_ttl_region] = ACTIONS(1901),
    [sym_ip_ttl_continent] = ACTIONS(1901),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1901),
    [sym_ip_nodelay_on] = ACTIONS(1901),
    [sym_ip_nodelay_off] = ACTIONS(1901),
    [sym_amx_error] = ACTIONS(1901),
    [sym_amx_warning] = ACTIONS(1901),
    [sym_amx_info] = ACTIONS(1901),
    [sym_amx_debug] = ACTIONS(1901),
    [sym_netlinx_login_success] = ACTIONS(1901),
    [sym_netlinx_login_fail] = ACTIONS(1901),
    [sym_netlinx_logout] = ACTIONS(1901),
    [sym_null_str] = ACTIONS(1901),
    [sym_smtp_address] = ACTIONS(1901),
    [sym_smtp_port_number] = ACTIONS(1901),
    [sym_smtp_username] = ACTIONS(1901),
    [sym_smtp_password] = ACTIONS(1901),
    [sym_smtp_require_tls] = ACTIONS(1901),
    [sym_smtp_from] = ACTIONS(1901),
    [sym_smtp_tls_true] = ACTIONS(1901),
    [sym_smtp_tls_false] = ACTIONS(1901),
    [sym_listview_on_row_select_event] = ACTIONS(1901),
    [sym_data_structure_unknown] = ACTIONS(1901),
    [sym_data_structure_datafeed] = ACTIONS(1901),
    [sym_data_structure_datarecordset] = ACTIONS(1901),
    [sym_data_structure_datarecord] = ACTIONS(1901),
    [sym_data_structure_datafield] = ACTIONS(1901),
    [sym_data_max_content_field_count] = ACTIONS(1901),
    [sym_data_max_metadata_field_count] = ACTIONS(1901),
    [sym_data_max_id_length] = ACTIONS(1901),
    [sym_data_max_name_length] = ACTIONS(1901),
    [sym_data_max_description_length] = ACTIONS(1901),
    [sym_data_max_type_length] = ACTIONS(1901),
    [sym_data_max_format_length] = ACTIONS(1901),
    [sym_data_max_source_length] = ACTIONS(1901),
    [sym_data_max_value_length] = ACTIONS(1901),
    [sym_data_max_label_length] = ACTIONS(1901),
    [sym_data_max_url_length] = ACTIONS(1901),
    [sym_max_last_login_info_length] = ACTIONS(1901),
    [sym_data_type_unknown] = ACTIONS(1901),
    [sym_data_type_string] = ACTIONS(1901),
    [sym_data_type_datetime] = ACTIONS(1901),
    [sym_data_type_date] = ACTIONS(1901),
    [sym_data_type_time] = ACTIONS(1901),
    [sym_data_type_image] = ACTIONS(1901),
    [sym_data_format_url] = ACTIONS(1901),
    [sym_data_format_phone] = ACTIONS(1901),
    [sym_data_format_email] = ACTIONS(1901),
    [sym_data_format_iso8601] = ACTIONS(1901),
    [sym_auditlog_priv] = ACTIONS(1901),
    [sym_configuration_priv] = ACTIONS(1901),
    [sym_deviceconfig_priv] = ACTIONS(1901),
    [sym_ftp_priv] = ACTIONS(1901),
    [sym_http_priv] = ACTIONS(1901),
    [sym_networkconfig_priv] = ACTIONS(1901),
    [sym_programport_priv] = ACTIONS(1901),
    [sym_remoteui_priv] = ACTIONS(1901),
    [sym_securitycontrol_priv] = ACTIONS(1901),
    [sym_softwaremanagement_priv] = ACTIONS(1901),
    [sym_terminal_priv] = ACTIONS(1901),
    [sym_tpadmin_priv] = ACTIONS(1901),
    [sym_user1_priv] = ACTIONS(1901),
    [sym_user2_priv] = ACTIONS(1901),
    [sym_user3_priv] = ACTIONS(1901),
    [sym_user4_priv] = ACTIONS(1901),
    [sym_usermanagement_priv] = ACTIONS(1901),
    [sym_valid_account] = ACTIONS(1901),
    [sym_err_name_invalid] = ACTIONS(1901),
    [sym_err_password_invalid] = ACTIONS(1901),
    [sym_err_invalid_account] = ACTIONS(1901),
    [sym_err_no_authorization] = ACTIONS(1901),
    [sym_err_invalid_parameter] = ACTIONS(1901),
    [sym_err_user_locked_out] = ACTIONS(1901),
    [sym_err_user_account_expired] = ACTIONS(1901),
    [sym_err_general_error] = ACTIONS(1901),
    [sym_err_authentication_error] = ACTIONS(1901),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1901),
    [sym_tls_validate_certificate] = ACTIONS(1901),
    [sym_tdata] = ACTIONS(1901),
    [sym_tchannel] = ACTIONS(1901),
    [sym_tlevel] = ACTIONS(1901),
    [sym_tbutton] = ACTIONS(1901),
    [sym_ttimeline] = ACTIONS(1901),
    [sym_tcustom] = ACTIONS(1901),
    [sym_url_struct] = ACTIONS(1901),
    [sym_dns_struct] = ACTIONS(1901),
    [sym_ip_address_struct] = ACTIONS(1901),
    [sym_dev_info_struct] = ACTIONS(1901),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1901),
    [sym_data_feed] = ACTIONS(1901),
    [sym_data_field] = ACTIONS(1901),
    [sym_data_record] = ACTIONS(1901),
    [sym_wc_data_feed] = ACTIONS(1901),
    [sym_wc_data_field] = ACTIONS(1901),
    [sym_wc_data_record] = ACTIONS(1901),
    [sym_last_login_info] = ACTIONS(1901),
    [sym____reserved___] = ACTIONS(1901),
    [sym_date] = ACTIONS(1901),
    [sym_day] = ACTIONS(1901),
    [sym_time] = ACTIONS(1901),
    [sym_ldate] = ACTIONS(1901),
    [sym_push_device] = ACTIONS(1901),
    [sym_push_channel] = ACTIONS(1901),
    [sym_push_devchan] = ACTIONS(1901),
    [sym_release_device] = ACTIONS(1901),
    [sym_release_channel] = ACTIONS(1901),
    [sym_release_devchan] = ACTIONS(1901),
    [sym_master_slot] = ACTIONS(1901),
    [sym_get_pulse_time] = ACTIONS(1901),
    [sym_get_timer] = ACTIONS(1901),
    [sym_master_sn] = ACTIONS(1901),
    [sym_system_number] = ACTIONS(1901),
    [sym_button] = ACTIONS(1901),
    [sym_dv_channel] = ACTIONS(1901),
    [sym_data] = ACTIONS(1901),
    [sym_level] = ACTIONS(1901),
    [sym_timeline] = ACTIONS(1901),
    [sym_channel] = ACTIONS(1901),
    [sym_custom] = ACTIONS(1901),
    [sym_atoi] = ACTIONS(1901),
    [sym_atol] = ACTIONS(1901),
    [sym_atof] = ACTIONS(1901),
    [sym_itoa] = ACTIONS(1901),
    [sym_itohex] = ACTIONS(1901),
    [sym_hextoi] = ACTIONS(1901),
    [sym_ftoa] = ACTIONS(1901),
    [sym_find_string] = ACTIONS(1901),
    [sym_left_string] = ACTIONS(1901),
    [sym_right_string] = ACTIONS(1901),
    [sym_mid_string] = ACTIONS(1901),
    [sym_remove_string] = ACTIONS(1901),
    [sym_get_buffer_string] = ACTIONS(1901),
    [sym_get_buffer_char] = ACTIONS(1901),
    [sym_get_multi_buffer_string] = ACTIONS(1901),
    [sym_length_string] = ACTIONS(1901),
    [sym_max_length_string] = ACTIONS(1901),
    [sym_set_length_string] = ACTIONS(1901),
    [sym_compare_string] = ACTIONS(1901),
    [sym_upper_string] = ACTIONS(1901),
    [sym_lower_string] = ACTIONS(1901),
    [sym_get_last] = ACTIONS(1901),
    [sym_date_to_day] = ACTIONS(1901),
    [sym_date_to_month] = ACTIONS(1901),
    [sym_date_to_year] = ACTIONS(1901),
    [sym_time_to_hour] = ACTIONS(1901),
    [sym_time_to_minute] = ACTIONS(1901),
    [sym_time_to_second] = ACTIONS(1901),
    [sym_day_of_week] = ACTIONS(1901),
    [sym_set_pulse_time] = ACTIONS(1901),
    [sym_set_timer] = ACTIONS(1901),
    [sym_file_open] = ACTIONS(1901),
    [sym_file_close] = ACTIONS(1901),
    [sym_file_read] = ACTIONS(1901),
    [sym_file_read_line] = ACTIONS(1901),
    [sym_file_write] = ACTIONS(1901),
    [sym_file_write_line] = ACTIONS(1901),
    [sym_file_copy] = ACTIONS(1901),
    [sym_file_delete] = ACTIONS(1901),
    [sym_file_rename] = ACTIONS(1901),
    [sym_file_dir] = ACTIONS(1901),
    [sym_file_getdir] = ACTIONS(1901),
    [sym_file_setdir] = ACTIONS(1901),
    [sym_file_create_dir] = ACTIONS(1901),
    [sym_file_remove_dir] = ACTIONS(1901),
    [sym_file_seek] = ACTIONS(1901),
    [sym_length_array] = ACTIONS(1901),
    [sym_max_length_array] = ACTIONS(1901),
    [sym_set_length_array] = ACTIONS(1901),
    [sym_ip_client_open] = ACTIONS(1901),
    [sym_ip_client_close] = ACTIONS(1901),
    [sym_ip_server_open] = ACTIONS(1901),
    [sym_ip_server_close] = ACTIONS(1901),
    [sym_ip_bound_client_open] = ACTIONS(1901),
    [sym_ip_mc_server_open] = ACTIONS(1901),
    [sym_ip_set_option] = ACTIONS(1901),
    [sym_ssh_client_open] = ACTIONS(1901),
    [sym_ssh_client_close] = ACTIONS(1901),
    [sym_tls_client_open] = ACTIONS(1901),
    [sym_tls_client_close] = ACTIONS(1901),
    [sym_do_push] = ACTIONS(1901),
    [sym_do_push_timed] = ACTIONS(1901),
    [sym_do_release] = ACTIONS(1901),
    [sym_device_id] = ACTIONS(1901),
    [sym_device_info] = ACTIONS(1901),
    [sym_device_id_string] = ACTIONS(1901),
    [sym_device_standby] = ACTIONS(1901),
    [sym_device_wake] = ACTIONS(1901),
    [sym_timeline_create] = ACTIONS(1901),
    [sym_timeline_kill] = ACTIONS(1901),
    [sym_timeline_reload] = ACTIONS(1901),
    [sym_timeline_pause] = ACTIONS(1901),
    [sym_timeline_restart] = ACTIONS(1901),
    [sym_timeline_set] = ACTIONS(1901),
    [sym_timeline_get] = ACTIONS(1901),
    [sym_timeline_active] = ACTIONS(1901),
    [sym_timeline_dynamic_id] = ACTIONS(1901),
    [sym_variable_to_string] = ACTIONS(1901),
    [sym_string_to_variable] = ACTIONS(1901),
    [sym_length_variable_to_string] = ACTIONS(1901),
    [sym_type_cast] = ACTIONS(1901),
    [sym_variable_to_xml] = ACTIONS(1901),
    [sym_xml_to_variable] = ACTIONS(1901),
    [sym_length_variable_to_xml] = ACTIONS(1901),
    [sym_redirect_string] = ACTIONS(1901),
    [sym_random_number] = ACTIONS(1901),
    [sym_get_unique_id] = ACTIONS(1901),
    [sym_get_system_number] = ACTIONS(1901),
    [sym_set_system_number] = ACTIONS(1901),
    [sym_get_serial_number] = ACTIONS(1901),
    [sym_get_master_build] = ACTIONS(1901),
    [sym_set_validation_code] = ACTIONS(1901),
    [sym_reboot] = ACTIONS(1901),
    [sym_load_duet_module] = ACTIONS(1901),
    [sym_unload_duet_module] = ACTIONS(1901),
    [sym_duet_mem_size_set] = ACTIONS(1901),
    [sym_duet_mem_size_get] = ACTIONS(1901),
    [sym_dynamic_application_device] = ACTIONS(1901),
    [sym_dynamic_polled_port] = ACTIONS(1901),
    [sym_static_port_binding] = ACTIONS(1901),
    [sym_static_ip_binding] = ACTIONS(1901),
    [sym_get_dns_list] = ACTIONS(1901),
    [sym_set_dns_list] = ACTIONS(1901),
    [sym_get_ip_address] = ACTIONS(1901),
    [sym_set_ip_address] = ACTIONS(1901),
    [sym_get_url_list] = ACTIONS(1901),
    [sym_add_url_entry] = ACTIONS(1901),
    [sym_delete_url_entry] = ACTIONS(1901),
    [sym_abs_value] = ACTIONS(1901),
    [sym_max_value] = ACTIONS(1901),
    [sym_min_value] = ACTIONS(1901),
    [sym_exp_value] = ACTIONS(1901),
    [sym_log_value] = ACTIONS(1901),
    [sym_log10_value] = ACTIONS(1901),
    [sym_power_value] = ACTIONS(1901),
    [sym_sqrt_value] = ACTIONS(1901),
    [sym_raw_be] = ACTIONS(1901),
    [sym_raw_le] = ACTIONS(1901),
    [sym_internal_queue_size_set] = ACTIONS(1901),
    [sym_internal_queue_size_get] = ACTIONS(1901),
    [sym_internal_threshold_set] = ACTIONS(1901),
    [sym_internal_threshold_get] = ACTIONS(1901),
    [sym_rebuilt_event] = ACTIONS(1901),
    [sym_do_custom_event] = ACTIONS(1901),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1901),
    [sym_clkmgr_set_clk_source] = ACTIONS(1901),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1901),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1901),
    [sym_clkmgr_get_timezone] = ACTIONS(1901),
    [sym_clkmgr_set_timezone] = ACTIONS(1901),
    [sym_clkmgr_get_resync_period] = ACTIONS(1901),
    [sym_clkmgr_set_resync_period] = ACTIONS(1901),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1901),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1901),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1901),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1901),
    [sym_clkmgr_get_timeservers] = ACTIONS(1901),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1901),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1901),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1901),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1901),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1901),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1901),
    [sym_set_log_level] = ACTIONS(1901),
    [sym_get_log_level] = ACTIONS(1901),
    [sym_amx_log] = ACTIONS(1901),
    [sym_smtp_server_config_set] = ACTIONS(1901),
    [sym_smtp_server_config_get] = ACTIONS(1901),
    [sym_smtp_send] = ACTIONS(1901),
    [sym_validate_netlinx_account] = ACTIONS(1901),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1901),
    [sym_authenticate_certificate] = ACTIONS(1901),
    [sym_audit_netlinx_session_event] = ACTIONS(1901),
    [sym_audit_netlinx_generic_event] = ACTIONS(1901),
    [sym_get_available_flash_disk_space] = ACTIONS(1901),
    [sym_get_max_flash_disk_space] = ACTIONS(1901),
    [sym_data_create_feed] = ACTIONS(1901),
    [sym_data_delete_feed] = ACTIONS(1901),
    [sym_data_publish_feed] = ACTIONS(1901),
    [sym_data_get_published_feed] = ACTIONS(1901),
    [sym_data_add_record] = ACTIONS(1901),
    [sym_data_get_event_record] = ACTIONS(1901),
    [sym__wc_data_create_feed] = ACTIONS(1901),
    [sym__wc_data_add_record] = ACTIONS(1901),
    [sym__wc_data_get_event_record] = ACTIONS(1901),
    [sym_format] = ACTIONS(1901),
    [sym_astro_clock] = ACTIONS(1901),
    [sym_set_outdoor_temperature] = ACTIONS(1901),
    [sym_set_virtual_level_count] = ACTIONS(1901),
    [sym_set_virtual_channel_count] = ACTIONS(1901),
    [sym_set_virtual_port_count] = ACTIONS(1901),
    [sym__wc_encode_int] = ACTIONS(1901),
    [sym__wc_decode_int] = ACTIONS(1901),
    [sym__wc_int] = ACTIONS(1901),
    [sym__wc_to_ch_int] = ACTIONS(1901),
    [sym__ch_to_wc_int] = ACTIONS(1901),
    [sym__wc_find_string_int] = ACTIONS(1901),
    [sym__wc_left_string_int] = ACTIONS(1901),
    [sym__wc_lower_string_int] = ACTIONS(1901),
    [sym__wc_mid_string_int] = ACTIONS(1901),
    [sym__wc_remove_string_int] = ACTIONS(1901),
    [sym__wc_right_string_int] = ACTIONS(1901),
    [sym__wc_upper_string_int] = ACTIONS(1901),
    [sym__wc_compare_string_int] = ACTIONS(1901),
    [sym__wc_get_buffer_char_int] = ACTIONS(1901),
    [sym__wc_get_buffer_string_int] = ACTIONS(1901),
    [sym__wc_concat_string_int] = ACTIONS(1901),
    [sym__wc_file_open_int] = ACTIONS(1901),
    [sym__wc_file_close_int] = ACTIONS(1901),
    [sym__wc_file_read_int] = ACTIONS(1901),
    [sym__wc_file_read_line_int] = ACTIONS(1901),
    [sym__wc_file_write_int] = ACTIONS(1901),
    [sym__wc_file_write_line_int] = ACTIONS(1901),
    [aux_sym_preproc_include_token1] = ACTIONS(1901),
    [aux_sym_preproc_define_token1] = ACTIONS(1903),
    [aux_sym_preproc_warn_token1] = ACTIONS(1903),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1903),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1903),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1903),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1903),
    [aux_sym_preproc_else_token1] = ACTIONS(1903),
    [anon_sym_LBRACK] = ACTIONS(1903),
    [anon_sym_LBRACE] = ACTIONS(1903),
    [anon_sym_LPAREN] = ACTIONS(1901),
    [anon_sym_BANG] = ACTIONS(1903),
    [anon_sym_TILDE] = ACTIONS(1903),
    [anon_sym_DASH] = ACTIONS(1901),
    [anon_sym_PLUS] = ACTIONS(1901),
    [anon_sym_DASH_DASH] = ACTIONS(1903),
    [anon_sym_PLUS_PLUS] = ACTIONS(1903),
    [anon_sym_DQUOTE] = ACTIONS(1903),
    [anon_sym_SQUOTE] = ACTIONS(1903),
    [sym_number_literal] = ACTIONS(1903),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(276)] = {
    [sym_identifier] = ACTIONS(1905),
    [sym_program_name_keyword] = ACTIONS(1905),
    [sym_module_name_keyword] = ACTIONS(1905),
    [sym_define_device_keyword] = ACTIONS(1905),
    [sym_define_combine_keyword] = ACTIONS(1905),
    [sym_define_constant_keyword] = ACTIONS(1905),
    [sym_define_type_keyword] = ACTIONS(1905),
    [sym_define_variable_keyword] = ACTIONS(1905),
    [sym_define_system_variable_keyword] = ACTIONS(1905),
    [sym_define_connect_level_keyword] = ACTIONS(1905),
    [sym_define_latching_keyword] = ACTIONS(1905),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1905),
    [sym_define_toggling_keyword] = ACTIONS(1905),
    [sym_define_call_keyword] = ACTIONS(1905),
    [sym_define_function_keyword] = ACTIONS(1905),
    [sym_define_library_function_keyword] = ACTIONS(1905),
    [sym_define_start_keyword] = ACTIONS(1905),
    [sym_define_event_keyword] = ACTIONS(1905),
    [sym_define_program_keyword] = ACTIONS(1905),
    [sym_define_module_keyword] = ACTIONS(1905),
    [sym_char_keyword] = ACTIONS(1905),
    [sym_widechar_keyword] = ACTIONS(1905),
    [sym_integer_keyword] = ACTIONS(1905),
    [sym_sinteger_keyword] = ACTIONS(1905),
    [sym_long_keyword] = ACTIONS(1905),
    [sym_slong_keyword] = ACTIONS(1905),
    [sym_float_keyword] = ACTIONS(1905),
    [sym_double_keyword] = ACTIONS(1905),
    [sym_variant_keyword] = ACTIONS(1905),
    [sym_variantarray_keyword] = ACTIONS(1905),
    [sym_local_var_keyword] = ACTIONS(1905),
    [sym_stack_var_keyword] = ACTIONS(1905),
    [sym_constant_keyword] = ACTIONS(1905),
    [sym_volatile_keyword] = ACTIONS(1905),
    [sym_non_volatile_keyword] = ACTIONS(1905),
    [sym_persistent_keyword] = ACTIONS(1905),
    [sym_dev_keyword] = ACTIONS(1905),
    [sym_devlev_keyword] = ACTIONS(1905),
    [sym_devchan_keyword] = ACTIONS(1905),
    [sym_if_keyword] = ACTIONS(1905),
    [sym_switch_keyword] = ACTIONS(1905),
    [sym_case_keyword] = ACTIONS(1905),
    [sym_default_keyword] = ACTIONS(1905),
    [sym_while_keyword] = ACTIONS(1905),
    [sym_for_keyword] = ACTIONS(1905),
    [sym_break_keyword] = ACTIONS(1905),
    [sym_continue_keyword] = ACTIONS(1905),
    [sym_return_keyword] = ACTIONS(1905),
    [sym_select_keyword] = ACTIONS(1905),
    [sym_active_keyword] = ACTIONS(1905),
    [sym_struct_keyword] = ACTIONS(1905),
    [sym_structure_keyword] = ACTIONS(1905),
    [sym_button_event_keyword] = ACTIONS(1905),
    [sym_channel_event_keyword] = ACTIONS(1905),
    [sym_level_event_keyword] = ACTIONS(1905),
    [sym_data_event_keyword] = ACTIONS(1905),
    [sym_timeline_event_keyword] = ACTIONS(1905),
    [sym_custom_event_keyword] = ACTIONS(1905),
    [sym_push_keyword] = ACTIONS(1905),
    [sym_release_keyword] = ACTIONS(1905),
    [aux_sym_on_keyword_token1] = ACTIONS(1905),
    [aux_sym_off_keyword_token1] = ACTIONS(1905),
    [sym_send_level_keyword] = ACTIONS(1905),
    [sym_send_string_keyword] = ACTIONS(1905),
    [sym_send_command_keyword] = ACTIONS(1905),
    [sym_clear_buffer_keyword] = ACTIONS(1905),
    [sym_create_buffer_keyword] = ACTIONS(1905),
    [sym_create_multi_buffer_keyword] = ACTIONS(1905),
    [sym_call_keyword] = ACTIONS(1905),
    [sym_system_call_keyword] = ACTIONS(1905),
    [sym_devchan_to_keyword] = ACTIONS(1905),
    [sym_devchan_min_to_keyword] = ACTIONS(1905),
    [sym_devchan_total_off_keyword] = ACTIONS(1905),
    [sym_devchan_pulse_keyword] = ACTIONS(1905),
    [sym_wait_keyword] = ACTIONS(1905),
    [sym_cancel_wait_keyword] = ACTIONS(1905),
    [sym_wait_until_keyword] = ACTIONS(1905),
    [sym_cancel_wait_until_keyword] = ACTIONS(1905),
    [sym_cancel_all_wait_keyword] = ACTIONS(1905),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1905),
    [sym_not] = ACTIONS(1905),
    [sym_bnot] = ACTIONS(1905),
    [sym___file__] = ACTIONS(1905),
    [sym___date__] = ACTIONS(1905),
    [sym___ldate__] = ACTIONS(1905),
    [sym___time__] = ACTIONS(1905),
    [sym___line__] = ACTIONS(1905),
    [sym___name__] = ACTIONS(1905),
    [sym___netlinx__] = ACTIONS(1905),
    [sym_netlinx_axi_version] = ACTIONS(1905),
    [sym_url_flg_tcp] = ACTIONS(1905),
    [sym_url_flg_accinfopresent] = ACTIONS(1905),
    [sym_url_flg_temp] = ACTIONS(1905),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1905),
    [sym_url_flg_stat_mask] = ACTIONS(1905),
    [sym_url_flg_stat_lookup] = ACTIONS(1905),
    [sym_url_flg_stat_connecting] = ACTIONS(1905),
    [sym_url_flg_stat_waiting] = ACTIONS(1905),
    [sym_url_flg_stat_connected] = ACTIONS(1905),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1905),
    [sym_first_virtual_device] = ACTIONS(1905),
    [sym_first_local_port] = ACTIONS(1905),
    [sym_dynamic_virtual_device] = ACTIONS(1905),
    [sym_timeline_once] = ACTIONS(1905),
    [sym_timeline_repeat] = ACTIONS(1905),
    [sym_timeline_absolute] = ACTIONS(1905),
    [sym_timeline_relative] = ACTIONS(1905),
    [sym_do_push_timed_infinite] = ACTIONS(1905),
    [sym_source_type_no_address] = ACTIONS(1905),
    [sym_source_type_neuron_id] = ACTIONS(1905),
    [sym_source_type_ip_address] = ACTIONS(1905),
    [sym_source_type_axlink] = ACTIONS(1905),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1905),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1905),
    [sym_source_type_ip_socket_address] = ACTIONS(1905),
    [sym_source_type_rs232] = ACTIONS(1905),
    [sym_source_type_internal] = ACTIONS(1905),
    [sym_source_type_mac_address] = ACTIONS(1905),
    [sym_source_type_ipv4_port] = ACTIONS(1905),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1905),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1905),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1905),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1905),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1905),
    [sym_true] = ACTIONS(1905),
    [sym_false] = ACTIONS(1905),
    [sym_file_read_only] = ACTIONS(1905),
    [sym_file_rw_new] = ACTIONS(1905),
    [sym_file_rw_append] = ACTIONS(1905),
    [sym_ip_tcp] = ACTIONS(1905),
    [sym_ip_udp] = ACTIONS(1905),
    [sym_ip_udp_2way] = ACTIONS(1905),
    [sym_xml_encode_types] = ACTIONS(1905),
    [sym_xml_encode_char_as_list] = ACTIONS(1905),
    [sym_xml_encode_le] = ACTIONS(1905),
    [sym_xml_decode_types] = ACTIONS(1905),
    [sym_xml_decode_no_preserve] = ACTIONS(1905),
    [sym_duet_dev_type_display_device] = ACTIONS(1905),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1905),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1905),
    [sym_duet_dev_type_io_device] = ACTIONS(1905),
    [sym_duet_dev_type_relay_device] = ACTIONS(1905),
    [sym_duet_dev_type_ups] = ACTIONS(1905),
    [sym_duet_dev_type_amplifier] = ACTIONS(1905),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1905),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1905),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1905),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1905),
    [sym_duet_dev_type_camera] = ACTIONS(1905),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1905),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1905),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1905),
    [sym_duet_dev_type_dss] = ACTIONS(1905),
    [sym_duet_dev_type_dvr] = ACTIONS(1905),
    [sym_duet_dev_type_disc_device] = ACTIONS(1905),
    [sym_duet_dev_type_document_camera] = ACTIONS(1905),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1905),
    [sym_duet_dev_type_hvac] = ACTIONS(1905),
    [sym_duet_dev_type_keypad] = ACTIONS(1905),
    [sym_duet_dev_type_light] = ACTIONS(1905),
    [sym_duet_dev_type_monitor] = ACTIONS(1905),
    [sym_duet_dev_type_motor] = ACTIONS(1905),
    [sym_duet_dev_type_multi_window] = ACTIONS(1905),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1905),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1905),
    [sym_duet_dev_type_receiver] = ACTIONS(1905),
    [sym_duet_dev_type_security_system] = ACTIONS(1905),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1905),
    [sym_duet_dev_type_settop_box] = ACTIONS(1905),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1905),
    [sym_duet_dev_type_switcher] = ACTIONS(1905),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1905),
    [sym_duet_dev_type_tv] = ACTIONS(1905),
    [sym_duet_dev_type_utility] = ACTIONS(1905),
    [sym_duet_dev_type_vcr] = ACTIONS(1905),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1905),
    [sym_duet_dev_type_video_processor] = ACTIONS(1905),
    [sym_duet_dev_type_video_projector] = ACTIONS(1905),
    [sym_duet_dev_type_video_wall] = ACTIONS(1905),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1905),
    [sym_duet_dev_type_weather] = ACTIONS(1905),
    [sym_duet_dev_not_polled] = ACTIONS(1905),
    [sym_duet_dev_polled] = ACTIONS(1905),
    [sym_clkmgr_mode_network] = ACTIONS(1905),
    [sym_clkmgr_mode_standalone] = ACTIONS(1905),
    [sym_normal_standby] = ACTIONS(1905),
    [sym_normal_wake] = ACTIONS(1905),
    [sym_ip_multicast_ttl_option] = ACTIONS(1905),
    [sym_ip_ttl_subnet] = ACTIONS(1905),
    [sym_ip_ttl_site] = ACTIONS(1905),
    [sym_ip_ttl_region] = ACTIONS(1905),
    [sym_ip_ttl_continent] = ACTIONS(1905),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1905),
    [sym_ip_nodelay_on] = ACTIONS(1905),
    [sym_ip_nodelay_off] = ACTIONS(1905),
    [sym_amx_error] = ACTIONS(1905),
    [sym_amx_warning] = ACTIONS(1905),
    [sym_amx_info] = ACTIONS(1905),
    [sym_amx_debug] = ACTIONS(1905),
    [sym_netlinx_login_success] = ACTIONS(1905),
    [sym_netlinx_login_fail] = ACTIONS(1905),
    [sym_netlinx_logout] = ACTIONS(1905),
    [sym_null_str] = ACTIONS(1905),
    [sym_smtp_address] = ACTIONS(1905),
    [sym_smtp_port_number] = ACTIONS(1905),
    [sym_smtp_username] = ACTIONS(1905),
    [sym_smtp_password] = ACTIONS(1905),
    [sym_smtp_require_tls] = ACTIONS(1905),
    [sym_smtp_from] = ACTIONS(1905),
    [sym_smtp_tls_true] = ACTIONS(1905),
    [sym_smtp_tls_false] = ACTIONS(1905),
    [sym_listview_on_row_select_event] = ACTIONS(1905),
    [sym_data_structure_unknown] = ACTIONS(1905),
    [sym_data_structure_datafeed] = ACTIONS(1905),
    [sym_data_structure_datarecordset] = ACTIONS(1905),
    [sym_data_structure_datarecord] = ACTIONS(1905),
    [sym_data_structure_datafield] = ACTIONS(1905),
    [sym_data_max_content_field_count] = ACTIONS(1905),
    [sym_data_max_metadata_field_count] = ACTIONS(1905),
    [sym_data_max_id_length] = ACTIONS(1905),
    [sym_data_max_name_length] = ACTIONS(1905),
    [sym_data_max_description_length] = ACTIONS(1905),
    [sym_data_max_type_length] = ACTIONS(1905),
    [sym_data_max_format_length] = ACTIONS(1905),
    [sym_data_max_source_length] = ACTIONS(1905),
    [sym_data_max_value_length] = ACTIONS(1905),
    [sym_data_max_label_length] = ACTIONS(1905),
    [sym_data_max_url_length] = ACTIONS(1905),
    [sym_max_last_login_info_length] = ACTIONS(1905),
    [sym_data_type_unknown] = ACTIONS(1905),
    [sym_data_type_string] = ACTIONS(1905),
    [sym_data_type_datetime] = ACTIONS(1905),
    [sym_data_type_date] = ACTIONS(1905),
    [sym_data_type_time] = ACTIONS(1905),
    [sym_data_type_image] = ACTIONS(1905),
    [sym_data_format_url] = ACTIONS(1905),
    [sym_data_format_phone] = ACTIONS(1905),
    [sym_data_format_email] = ACTIONS(1905),
    [sym_data_format_iso8601] = ACTIONS(1905),
    [sym_auditlog_priv] = ACTIONS(1905),
    [sym_configuration_priv] = ACTIONS(1905),
    [sym_deviceconfig_priv] = ACTIONS(1905),
    [sym_ftp_priv] = ACTIONS(1905),
    [sym_http_priv] = ACTIONS(1905),
    [sym_networkconfig_priv] = ACTIONS(1905),
    [sym_programport_priv] = ACTIONS(1905),
    [sym_remoteui_priv] = ACTIONS(1905),
    [sym_securitycontrol_priv] = ACTIONS(1905),
    [sym_softwaremanagement_priv] = ACTIONS(1905),
    [sym_terminal_priv] = ACTIONS(1905),
    [sym_tpadmin_priv] = ACTIONS(1905),
    [sym_user1_priv] = ACTIONS(1905),
    [sym_user2_priv] = ACTIONS(1905),
    [sym_user3_priv] = ACTIONS(1905),
    [sym_user4_priv] = ACTIONS(1905),
    [sym_usermanagement_priv] = ACTIONS(1905),
    [sym_valid_account] = ACTIONS(1905),
    [sym_err_name_invalid] = ACTIONS(1905),
    [sym_err_password_invalid] = ACTIONS(1905),
    [sym_err_invalid_account] = ACTIONS(1905),
    [sym_err_no_authorization] = ACTIONS(1905),
    [sym_err_invalid_parameter] = ACTIONS(1905),
    [sym_err_user_locked_out] = ACTIONS(1905),
    [sym_err_user_account_expired] = ACTIONS(1905),
    [sym_err_general_error] = ACTIONS(1905),
    [sym_err_authentication_error] = ACTIONS(1905),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1905),
    [sym_tls_validate_certificate] = ACTIONS(1905),
    [sym_tdata] = ACTIONS(1905),
    [sym_tchannel] = ACTIONS(1905),
    [sym_tlevel] = ACTIONS(1905),
    [sym_tbutton] = ACTIONS(1905),
    [sym_ttimeline] = ACTIONS(1905),
    [sym_tcustom] = ACTIONS(1905),
    [sym_url_struct] = ACTIONS(1905),
    [sym_dns_struct] = ACTIONS(1905),
    [sym_ip_address_struct] = ACTIONS(1905),
    [sym_dev_info_struct] = ACTIONS(1905),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1905),
    [sym_data_feed] = ACTIONS(1905),
    [sym_data_field] = ACTIONS(1905),
    [sym_data_record] = ACTIONS(1905),
    [sym_wc_data_feed] = ACTIONS(1905),
    [sym_wc_data_field] = ACTIONS(1905),
    [sym_wc_data_record] = ACTIONS(1905),
    [sym_last_login_info] = ACTIONS(1905),
    [sym____reserved___] = ACTIONS(1905),
    [sym_date] = ACTIONS(1905),
    [sym_day] = ACTIONS(1905),
    [sym_time] = ACTIONS(1905),
    [sym_ldate] = ACTIONS(1905),
    [sym_push_device] = ACTIONS(1905),
    [sym_push_channel] = ACTIONS(1905),
    [sym_push_devchan] = ACTIONS(1905),
    [sym_release_device] = ACTIONS(1905),
    [sym_release_channel] = ACTIONS(1905),
    [sym_release_devchan] = ACTIONS(1905),
    [sym_master_slot] = ACTIONS(1905),
    [sym_get_pulse_time] = ACTIONS(1905),
    [sym_get_timer] = ACTIONS(1905),
    [sym_master_sn] = ACTIONS(1905),
    [sym_system_number] = ACTIONS(1905),
    [sym_button] = ACTIONS(1905),
    [sym_dv_channel] = ACTIONS(1905),
    [sym_data] = ACTIONS(1905),
    [sym_level] = ACTIONS(1905),
    [sym_timeline] = ACTIONS(1905),
    [sym_channel] = ACTIONS(1905),
    [sym_custom] = ACTIONS(1905),
    [sym_atoi] = ACTIONS(1905),
    [sym_atol] = ACTIONS(1905),
    [sym_atof] = ACTIONS(1905),
    [sym_itoa] = ACTIONS(1905),
    [sym_itohex] = ACTIONS(1905),
    [sym_hextoi] = ACTIONS(1905),
    [sym_ftoa] = ACTIONS(1905),
    [sym_find_string] = ACTIONS(1905),
    [sym_left_string] = ACTIONS(1905),
    [sym_right_string] = ACTIONS(1905),
    [sym_mid_string] = ACTIONS(1905),
    [sym_remove_string] = ACTIONS(1905),
    [sym_get_buffer_string] = ACTIONS(1905),
    [sym_get_buffer_char] = ACTIONS(1905),
    [sym_get_multi_buffer_string] = ACTIONS(1905),
    [sym_length_string] = ACTIONS(1905),
    [sym_max_length_string] = ACTIONS(1905),
    [sym_set_length_string] = ACTIONS(1905),
    [sym_compare_string] = ACTIONS(1905),
    [sym_upper_string] = ACTIONS(1905),
    [sym_lower_string] = ACTIONS(1905),
    [sym_get_last] = ACTIONS(1905),
    [sym_date_to_day] = ACTIONS(1905),
    [sym_date_to_month] = ACTIONS(1905),
    [sym_date_to_year] = ACTIONS(1905),
    [sym_time_to_hour] = ACTIONS(1905),
    [sym_time_to_minute] = ACTIONS(1905),
    [sym_time_to_second] = ACTIONS(1905),
    [sym_day_of_week] = ACTIONS(1905),
    [sym_set_pulse_time] = ACTIONS(1905),
    [sym_set_timer] = ACTIONS(1905),
    [sym_file_open] = ACTIONS(1905),
    [sym_file_close] = ACTIONS(1905),
    [sym_file_read] = ACTIONS(1905),
    [sym_file_read_line] = ACTIONS(1905),
    [sym_file_write] = ACTIONS(1905),
    [sym_file_write_line] = ACTIONS(1905),
    [sym_file_copy] = ACTIONS(1905),
    [sym_file_delete] = ACTIONS(1905),
    [sym_file_rename] = ACTIONS(1905),
    [sym_file_dir] = ACTIONS(1905),
    [sym_file_getdir] = ACTIONS(1905),
    [sym_file_setdir] = ACTIONS(1905),
    [sym_file_create_dir] = ACTIONS(1905),
    [sym_file_remove_dir] = ACTIONS(1905),
    [sym_file_seek] = ACTIONS(1905),
    [sym_length_array] = ACTIONS(1905),
    [sym_max_length_array] = ACTIONS(1905),
    [sym_set_length_array] = ACTIONS(1905),
    [sym_ip_client_open] = ACTIONS(1905),
    [sym_ip_client_close] = ACTIONS(1905),
    [sym_ip_server_open] = ACTIONS(1905),
    [sym_ip_server_close] = ACTIONS(1905),
    [sym_ip_bound_client_open] = ACTIONS(1905),
    [sym_ip_mc_server_open] = ACTIONS(1905),
    [sym_ip_set_option] = ACTIONS(1905),
    [sym_ssh_client_open] = ACTIONS(1905),
    [sym_ssh_client_close] = ACTIONS(1905),
    [sym_tls_client_open] = ACTIONS(1905),
    [sym_tls_client_close] = ACTIONS(1905),
    [sym_do_push] = ACTIONS(1905),
    [sym_do_push_timed] = ACTIONS(1905),
    [sym_do_release] = ACTIONS(1905),
    [sym_device_id] = ACTIONS(1905),
    [sym_device_info] = ACTIONS(1905),
    [sym_device_id_string] = ACTIONS(1905),
    [sym_device_standby] = ACTIONS(1905),
    [sym_device_wake] = ACTIONS(1905),
    [sym_timeline_create] = ACTIONS(1905),
    [sym_timeline_kill] = ACTIONS(1905),
    [sym_timeline_reload] = ACTIONS(1905),
    [sym_timeline_pause] = ACTIONS(1905),
    [sym_timeline_restart] = ACTIONS(1905),
    [sym_timeline_set] = ACTIONS(1905),
    [sym_timeline_get] = ACTIONS(1905),
    [sym_timeline_active] = ACTIONS(1905),
    [sym_timeline_dynamic_id] = ACTIONS(1905),
    [sym_variable_to_string] = ACTIONS(1905),
    [sym_string_to_variable] = ACTIONS(1905),
    [sym_length_variable_to_string] = ACTIONS(1905),
    [sym_type_cast] = ACTIONS(1905),
    [sym_variable_to_xml] = ACTIONS(1905),
    [sym_xml_to_variable] = ACTIONS(1905),
    [sym_length_variable_to_xml] = ACTIONS(1905),
    [sym_redirect_string] = ACTIONS(1905),
    [sym_random_number] = ACTIONS(1905),
    [sym_get_unique_id] = ACTIONS(1905),
    [sym_get_system_number] = ACTIONS(1905),
    [sym_set_system_number] = ACTIONS(1905),
    [sym_get_serial_number] = ACTIONS(1905),
    [sym_get_master_build] = ACTIONS(1905),
    [sym_set_validation_code] = ACTIONS(1905),
    [sym_reboot] = ACTIONS(1905),
    [sym_load_duet_module] = ACTIONS(1905),
    [sym_unload_duet_module] = ACTIONS(1905),
    [sym_duet_mem_size_set] = ACTIONS(1905),
    [sym_duet_mem_size_get] = ACTIONS(1905),
    [sym_dynamic_application_device] = ACTIONS(1905),
    [sym_dynamic_polled_port] = ACTIONS(1905),
    [sym_static_port_binding] = ACTIONS(1905),
    [sym_static_ip_binding] = ACTIONS(1905),
    [sym_get_dns_list] = ACTIONS(1905),
    [sym_set_dns_list] = ACTIONS(1905),
    [sym_get_ip_address] = ACTIONS(1905),
    [sym_set_ip_address] = ACTIONS(1905),
    [sym_get_url_list] = ACTIONS(1905),
    [sym_add_url_entry] = ACTIONS(1905),
    [sym_delete_url_entry] = ACTIONS(1905),
    [sym_abs_value] = ACTIONS(1905),
    [sym_max_value] = ACTIONS(1905),
    [sym_min_value] = ACTIONS(1905),
    [sym_exp_value] = ACTIONS(1905),
    [sym_log_value] = ACTIONS(1905),
    [sym_log10_value] = ACTIONS(1905),
    [sym_power_value] = ACTIONS(1905),
    [sym_sqrt_value] = ACTIONS(1905),
    [sym_raw_be] = ACTIONS(1905),
    [sym_raw_le] = ACTIONS(1905),
    [sym_internal_queue_size_set] = ACTIONS(1905),
    [sym_internal_queue_size_get] = ACTIONS(1905),
    [sym_internal_threshold_set] = ACTIONS(1905),
    [sym_internal_threshold_get] = ACTIONS(1905),
    [sym_rebuilt_event] = ACTIONS(1905),
    [sym_do_custom_event] = ACTIONS(1905),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1905),
    [sym_clkmgr_set_clk_source] = ACTIONS(1905),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1905),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1905),
    [sym_clkmgr_get_timezone] = ACTIONS(1905),
    [sym_clkmgr_set_timezone] = ACTIONS(1905),
    [sym_clkmgr_get_resync_period] = ACTIONS(1905),
    [sym_clkmgr_set_resync_period] = ACTIONS(1905),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1905),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1905),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1905),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1905),
    [sym_clkmgr_get_timeservers] = ACTIONS(1905),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1905),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1905),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1905),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1905),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1905),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1905),
    [sym_set_log_level] = ACTIONS(1905),
    [sym_get_log_level] = ACTIONS(1905),
    [sym_amx_log] = ACTIONS(1905),
    [sym_smtp_server_config_set] = ACTIONS(1905),
    [sym_smtp_server_config_get] = ACTIONS(1905),
    [sym_smtp_send] = ACTIONS(1905),
    [sym_validate_netlinx_account] = ACTIONS(1905),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1905),
    [sym_authenticate_certificate] = ACTIONS(1905),
    [sym_audit_netlinx_session_event] = ACTIONS(1905),
    [sym_audit_netlinx_generic_event] = ACTIONS(1905),
    [sym_get_available_flash_disk_space] = ACTIONS(1905),
    [sym_get_max_flash_disk_space] = ACTIONS(1905),
    [sym_data_create_feed] = ACTIONS(1905),
    [sym_data_delete_feed] = ACTIONS(1905),
    [sym_data_publish_feed] = ACTIONS(1905),
    [sym_data_get_published_feed] = ACTIONS(1905),
    [sym_data_add_record] = ACTIONS(1905),
    [sym_data_get_event_record] = ACTIONS(1905),
    [sym__wc_data_create_feed] = ACTIONS(1905),
    [sym__wc_data_add_record] = ACTIONS(1905),
    [sym__wc_data_get_event_record] = ACTIONS(1905),
    [sym_format] = ACTIONS(1905),
    [sym_astro_clock] = ACTIONS(1905),
    [sym_set_outdoor_temperature] = ACTIONS(1905),
    [sym_set_virtual_level_count] = ACTIONS(1905),
    [sym_set_virtual_channel_count] = ACTIONS(1905),
    [sym_set_virtual_port_count] = ACTIONS(1905),
    [sym__wc_encode_int] = ACTIONS(1905),
    [sym__wc_decode_int] = ACTIONS(1905),
    [sym__wc_int] = ACTIONS(1905),
    [sym__wc_to_ch_int] = ACTIONS(1905),
    [sym__ch_to_wc_int] = ACTIONS(1905),
    [sym__wc_find_string_int] = ACTIONS(1905),
    [sym__wc_left_string_int] = ACTIONS(1905),
    [sym__wc_lower_string_int] = ACTIONS(1905),
    [sym__wc_mid_string_int] = ACTIONS(1905),
    [sym__wc_remove_string_int] = ACTIONS(1905),
    [sym__wc_right_string_int] = ACTIONS(1905),
    [sym__wc_upper_string_int] = ACTIONS(1905),
    [sym__wc_compare_string_int] = ACTIONS(1905),
    [sym__wc_get_buffer_char_int] = ACTIONS(1905),
    [sym__wc_get_buffer_string_int] = ACTIONS(1905),
    [sym__wc_concat_string_int] = ACTIONS(1905),
    [sym__wc_file_open_int] = ACTIONS(1905),
    [sym__wc_file_close_int] = ACTIONS(1905),
    [sym__wc_file_read_int] = ACTIONS(1905),
    [sym__wc_file_read_line_int] = ACTIONS(1905),
    [sym__wc_file_write_int] = ACTIONS(1905),
    [sym__wc_file_write_line_int] = ACTIONS(1905),
    [aux_sym_preproc_include_token1] = ACTIONS(1905),
    [aux_sym_preproc_define_token1] = ACTIONS(1907),
    [aux_sym_preproc_warn_token1] = ACTIONS(1907),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1907),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1907),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1907),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1907),
    [aux_sym_preproc_else_token1] = ACTIONS(1907),
    [anon_sym_LBRACK] = ACTIONS(1907),
    [anon_sym_LBRACE] = ACTIONS(1907),
    [anon_sym_LPAREN] = ACTIONS(1905),
    [anon_sym_BANG] = ACTIONS(1907),
    [anon_sym_TILDE] = ACTIONS(1907),
    [anon_sym_DASH] = ACTIONS(1905),
    [anon_sym_PLUS] = ACTIONS(1905),
    [anon_sym_DASH_DASH] = ACTIONS(1907),
    [anon_sym_PLUS_PLUS] = ACTIONS(1907),
    [anon_sym_DQUOTE] = ACTIONS(1907),
    [anon_sym_SQUOTE] = ACTIONS(1907),
    [sym_number_literal] = ACTIONS(1907),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(277)] = {
    [sym_identifier] = ACTIONS(1909),
    [sym_program_name_keyword] = ACTIONS(1909),
    [sym_module_name_keyword] = ACTIONS(1909),
    [sym_define_device_keyword] = ACTIONS(1909),
    [sym_define_combine_keyword] = ACTIONS(1909),
    [sym_define_constant_keyword] = ACTIONS(1909),
    [sym_define_type_keyword] = ACTIONS(1909),
    [sym_define_variable_keyword] = ACTIONS(1909),
    [sym_define_system_variable_keyword] = ACTIONS(1909),
    [sym_define_connect_level_keyword] = ACTIONS(1909),
    [sym_define_latching_keyword] = ACTIONS(1909),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1909),
    [sym_define_toggling_keyword] = ACTIONS(1909),
    [sym_define_call_keyword] = ACTIONS(1909),
    [sym_define_function_keyword] = ACTIONS(1909),
    [sym_define_library_function_keyword] = ACTIONS(1909),
    [sym_define_start_keyword] = ACTIONS(1909),
    [sym_define_event_keyword] = ACTIONS(1909),
    [sym_define_program_keyword] = ACTIONS(1909),
    [sym_define_module_keyword] = ACTIONS(1909),
    [sym_char_keyword] = ACTIONS(1909),
    [sym_widechar_keyword] = ACTIONS(1909),
    [sym_integer_keyword] = ACTIONS(1909),
    [sym_sinteger_keyword] = ACTIONS(1909),
    [sym_long_keyword] = ACTIONS(1909),
    [sym_slong_keyword] = ACTIONS(1909),
    [sym_float_keyword] = ACTIONS(1909),
    [sym_double_keyword] = ACTIONS(1909),
    [sym_variant_keyword] = ACTIONS(1909),
    [sym_variantarray_keyword] = ACTIONS(1909),
    [sym_local_var_keyword] = ACTIONS(1909),
    [sym_stack_var_keyword] = ACTIONS(1909),
    [sym_constant_keyword] = ACTIONS(1909),
    [sym_volatile_keyword] = ACTIONS(1909),
    [sym_non_volatile_keyword] = ACTIONS(1909),
    [sym_persistent_keyword] = ACTIONS(1909),
    [sym_dev_keyword] = ACTIONS(1909),
    [sym_devlev_keyword] = ACTIONS(1909),
    [sym_devchan_keyword] = ACTIONS(1909),
    [sym_if_keyword] = ACTIONS(1909),
    [sym_switch_keyword] = ACTIONS(1909),
    [sym_case_keyword] = ACTIONS(1909),
    [sym_default_keyword] = ACTIONS(1909),
    [sym_while_keyword] = ACTIONS(1909),
    [sym_for_keyword] = ACTIONS(1909),
    [sym_break_keyword] = ACTIONS(1909),
    [sym_continue_keyword] = ACTIONS(1909),
    [sym_return_keyword] = ACTIONS(1909),
    [sym_select_keyword] = ACTIONS(1909),
    [sym_active_keyword] = ACTIONS(1909),
    [sym_struct_keyword] = ACTIONS(1909),
    [sym_structure_keyword] = ACTIONS(1909),
    [sym_button_event_keyword] = ACTIONS(1909),
    [sym_channel_event_keyword] = ACTIONS(1909),
    [sym_level_event_keyword] = ACTIONS(1909),
    [sym_data_event_keyword] = ACTIONS(1909),
    [sym_timeline_event_keyword] = ACTIONS(1909),
    [sym_custom_event_keyword] = ACTIONS(1909),
    [sym_push_keyword] = ACTIONS(1909),
    [sym_release_keyword] = ACTIONS(1909),
    [aux_sym_on_keyword_token1] = ACTIONS(1909),
    [aux_sym_off_keyword_token1] = ACTIONS(1909),
    [sym_send_level_keyword] = ACTIONS(1909),
    [sym_send_string_keyword] = ACTIONS(1909),
    [sym_send_command_keyword] = ACTIONS(1909),
    [sym_clear_buffer_keyword] = ACTIONS(1909),
    [sym_create_buffer_keyword] = ACTIONS(1909),
    [sym_create_multi_buffer_keyword] = ACTIONS(1909),
    [sym_call_keyword] = ACTIONS(1909),
    [sym_system_call_keyword] = ACTIONS(1909),
    [sym_devchan_to_keyword] = ACTIONS(1909),
    [sym_devchan_min_to_keyword] = ACTIONS(1909),
    [sym_devchan_total_off_keyword] = ACTIONS(1909),
    [sym_devchan_pulse_keyword] = ACTIONS(1909),
    [sym_wait_keyword] = ACTIONS(1909),
    [sym_cancel_wait_keyword] = ACTIONS(1909),
    [sym_wait_until_keyword] = ACTIONS(1909),
    [sym_cancel_wait_until_keyword] = ACTIONS(1909),
    [sym_cancel_all_wait_keyword] = ACTIONS(1909),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1909),
    [sym_not] = ACTIONS(1909),
    [sym_bnot] = ACTIONS(1909),
    [sym___file__] = ACTIONS(1909),
    [sym___date__] = ACTIONS(1909),
    [sym___ldate__] = ACTIONS(1909),
    [sym___time__] = ACTIONS(1909),
    [sym___line__] = ACTIONS(1909),
    [sym___name__] = ACTIONS(1909),
    [sym___netlinx__] = ACTIONS(1909),
    [sym_netlinx_axi_version] = ACTIONS(1909),
    [sym_url_flg_tcp] = ACTIONS(1909),
    [sym_url_flg_accinfopresent] = ACTIONS(1909),
    [sym_url_flg_temp] = ACTIONS(1909),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1909),
    [sym_url_flg_stat_mask] = ACTIONS(1909),
    [sym_url_flg_stat_lookup] = ACTIONS(1909),
    [sym_url_flg_stat_connecting] = ACTIONS(1909),
    [sym_url_flg_stat_waiting] = ACTIONS(1909),
    [sym_url_flg_stat_connected] = ACTIONS(1909),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1909),
    [sym_first_virtual_device] = ACTIONS(1909),
    [sym_first_local_port] = ACTIONS(1909),
    [sym_dynamic_virtual_device] = ACTIONS(1909),
    [sym_timeline_once] = ACTIONS(1909),
    [sym_timeline_repeat] = ACTIONS(1909),
    [sym_timeline_absolute] = ACTIONS(1909),
    [sym_timeline_relative] = ACTIONS(1909),
    [sym_do_push_timed_infinite] = ACTIONS(1909),
    [sym_source_type_no_address] = ACTIONS(1909),
    [sym_source_type_neuron_id] = ACTIONS(1909),
    [sym_source_type_ip_address] = ACTIONS(1909),
    [sym_source_type_axlink] = ACTIONS(1909),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1909),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1909),
    [sym_source_type_ip_socket_address] = ACTIONS(1909),
    [sym_source_type_rs232] = ACTIONS(1909),
    [sym_source_type_internal] = ACTIONS(1909),
    [sym_source_type_mac_address] = ACTIONS(1909),
    [sym_source_type_ipv4_port] = ACTIONS(1909),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1909),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1909),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1909),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1909),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1909),
    [sym_true] = ACTIONS(1909),
    [sym_false] = ACTIONS(1909),
    [sym_file_read_only] = ACTIONS(1909),
    [sym_file_rw_new] = ACTIONS(1909),
    [sym_file_rw_append] = ACTIONS(1909),
    [sym_ip_tcp] = ACTIONS(1909),
    [sym_ip_udp] = ACTIONS(1909),
    [sym_ip_udp_2way] = ACTIONS(1909),
    [sym_xml_encode_types] = ACTIONS(1909),
    [sym_xml_encode_char_as_list] = ACTIONS(1909),
    [sym_xml_encode_le] = ACTIONS(1909),
    [sym_xml_decode_types] = ACTIONS(1909),
    [sym_xml_decode_no_preserve] = ACTIONS(1909),
    [sym_duet_dev_type_display_device] = ACTIONS(1909),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1909),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1909),
    [sym_duet_dev_type_io_device] = ACTIONS(1909),
    [sym_duet_dev_type_relay_device] = ACTIONS(1909),
    [sym_duet_dev_type_ups] = ACTIONS(1909),
    [sym_duet_dev_type_amplifier] = ACTIONS(1909),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1909),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1909),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1909),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1909),
    [sym_duet_dev_type_camera] = ACTIONS(1909),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1909),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1909),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1909),
    [sym_duet_dev_type_dss] = ACTIONS(1909),
    [sym_duet_dev_type_dvr] = ACTIONS(1909),
    [sym_duet_dev_type_disc_device] = ACTIONS(1909),
    [sym_duet_dev_type_document_camera] = ACTIONS(1909),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1909),
    [sym_duet_dev_type_hvac] = ACTIONS(1909),
    [sym_duet_dev_type_keypad] = ACTIONS(1909),
    [sym_duet_dev_type_light] = ACTIONS(1909),
    [sym_duet_dev_type_monitor] = ACTIONS(1909),
    [sym_duet_dev_type_motor] = ACTIONS(1909),
    [sym_duet_dev_type_multi_window] = ACTIONS(1909),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1909),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1909),
    [sym_duet_dev_type_receiver] = ACTIONS(1909),
    [sym_duet_dev_type_security_system] = ACTIONS(1909),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1909),
    [sym_duet_dev_type_settop_box] = ACTIONS(1909),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1909),
    [sym_duet_dev_type_switcher] = ACTIONS(1909),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1909),
    [sym_duet_dev_type_tv] = ACTIONS(1909),
    [sym_duet_dev_type_utility] = ACTIONS(1909),
    [sym_duet_dev_type_vcr] = ACTIONS(1909),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1909),
    [sym_duet_dev_type_video_processor] = ACTIONS(1909),
    [sym_duet_dev_type_video_projector] = ACTIONS(1909),
    [sym_duet_dev_type_video_wall] = ACTIONS(1909),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1909),
    [sym_duet_dev_type_weather] = ACTIONS(1909),
    [sym_duet_dev_not_polled] = ACTIONS(1909),
    [sym_duet_dev_polled] = ACTIONS(1909),
    [sym_clkmgr_mode_network] = ACTIONS(1909),
    [sym_clkmgr_mode_standalone] = ACTIONS(1909),
    [sym_normal_standby] = ACTIONS(1909),
    [sym_normal_wake] = ACTIONS(1909),
    [sym_ip_multicast_ttl_option] = ACTIONS(1909),
    [sym_ip_ttl_subnet] = ACTIONS(1909),
    [sym_ip_ttl_site] = ACTIONS(1909),
    [sym_ip_ttl_region] = ACTIONS(1909),
    [sym_ip_ttl_continent] = ACTIONS(1909),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1909),
    [sym_ip_nodelay_on] = ACTIONS(1909),
    [sym_ip_nodelay_off] = ACTIONS(1909),
    [sym_amx_error] = ACTIONS(1909),
    [sym_amx_warning] = ACTIONS(1909),
    [sym_amx_info] = ACTIONS(1909),
    [sym_amx_debug] = ACTIONS(1909),
    [sym_netlinx_login_success] = ACTIONS(1909),
    [sym_netlinx_login_fail] = ACTIONS(1909),
    [sym_netlinx_logout] = ACTIONS(1909),
    [sym_null_str] = ACTIONS(1909),
    [sym_smtp_address] = ACTIONS(1909),
    [sym_smtp_port_number] = ACTIONS(1909),
    [sym_smtp_username] = ACTIONS(1909),
    [sym_smtp_password] = ACTIONS(1909),
    [sym_smtp_require_tls] = ACTIONS(1909),
    [sym_smtp_from] = ACTIONS(1909),
    [sym_smtp_tls_true] = ACTIONS(1909),
    [sym_smtp_tls_false] = ACTIONS(1909),
    [sym_listview_on_row_select_event] = ACTIONS(1909),
    [sym_data_structure_unknown] = ACTIONS(1909),
    [sym_data_structure_datafeed] = ACTIONS(1909),
    [sym_data_structure_datarecordset] = ACTIONS(1909),
    [sym_data_structure_datarecord] = ACTIONS(1909),
    [sym_data_structure_datafield] = ACTIONS(1909),
    [sym_data_max_content_field_count] = ACTIONS(1909),
    [sym_data_max_metadata_field_count] = ACTIONS(1909),
    [sym_data_max_id_length] = ACTIONS(1909),
    [sym_data_max_name_length] = ACTIONS(1909),
    [sym_data_max_description_length] = ACTIONS(1909),
    [sym_data_max_type_length] = ACTIONS(1909),
    [sym_data_max_format_length] = ACTIONS(1909),
    [sym_data_max_source_length] = ACTIONS(1909),
    [sym_data_max_value_length] = ACTIONS(1909),
    [sym_data_max_label_length] = ACTIONS(1909),
    [sym_data_max_url_length] = ACTIONS(1909),
    [sym_max_last_login_info_length] = ACTIONS(1909),
    [sym_data_type_unknown] = ACTIONS(1909),
    [sym_data_type_string] = ACTIONS(1909),
    [sym_data_type_datetime] = ACTIONS(1909),
    [sym_data_type_date] = ACTIONS(1909),
    [sym_data_type_time] = ACTIONS(1909),
    [sym_data_type_image] = ACTIONS(1909),
    [sym_data_format_url] = ACTIONS(1909),
    [sym_data_format_phone] = ACTIONS(1909),
    [sym_data_format_email] = ACTIONS(1909),
    [sym_data_format_iso8601] = ACTIONS(1909),
    [sym_auditlog_priv] = ACTIONS(1909),
    [sym_configuration_priv] = ACTIONS(1909),
    [sym_deviceconfig_priv] = ACTIONS(1909),
    [sym_ftp_priv] = ACTIONS(1909),
    [sym_http_priv] = ACTIONS(1909),
    [sym_networkconfig_priv] = ACTIONS(1909),
    [sym_programport_priv] = ACTIONS(1909),
    [sym_remoteui_priv] = ACTIONS(1909),
    [sym_securitycontrol_priv] = ACTIONS(1909),
    [sym_softwaremanagement_priv] = ACTIONS(1909),
    [sym_terminal_priv] = ACTIONS(1909),
    [sym_tpadmin_priv] = ACTIONS(1909),
    [sym_user1_priv] = ACTIONS(1909),
    [sym_user2_priv] = ACTIONS(1909),
    [sym_user3_priv] = ACTIONS(1909),
    [sym_user4_priv] = ACTIONS(1909),
    [sym_usermanagement_priv] = ACTIONS(1909),
    [sym_valid_account] = ACTIONS(1909),
    [sym_err_name_invalid] = ACTIONS(1909),
    [sym_err_password_invalid] = ACTIONS(1909),
    [sym_err_invalid_account] = ACTIONS(1909),
    [sym_err_no_authorization] = ACTIONS(1909),
    [sym_err_invalid_parameter] = ACTIONS(1909),
    [sym_err_user_locked_out] = ACTIONS(1909),
    [sym_err_user_account_expired] = ACTIONS(1909),
    [sym_err_general_error] = ACTIONS(1909),
    [sym_err_authentication_error] = ACTIONS(1909),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1909),
    [sym_tls_validate_certificate] = ACTIONS(1909),
    [sym_tdata] = ACTIONS(1909),
    [sym_tchannel] = ACTIONS(1909),
    [sym_tlevel] = ACTIONS(1909),
    [sym_tbutton] = ACTIONS(1909),
    [sym_ttimeline] = ACTIONS(1909),
    [sym_tcustom] = ACTIONS(1909),
    [sym_url_struct] = ACTIONS(1909),
    [sym_dns_struct] = ACTIONS(1909),
    [sym_ip_address_struct] = ACTIONS(1909),
    [sym_dev_info_struct] = ACTIONS(1909),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1909),
    [sym_data_feed] = ACTIONS(1909),
    [sym_data_field] = ACTIONS(1909),
    [sym_data_record] = ACTIONS(1909),
    [sym_wc_data_feed] = ACTIONS(1909),
    [sym_wc_data_field] = ACTIONS(1909),
    [sym_wc_data_record] = ACTIONS(1909),
    [sym_last_login_info] = ACTIONS(1909),
    [sym____reserved___] = ACTIONS(1909),
    [sym_date] = ACTIONS(1909),
    [sym_day] = ACTIONS(1909),
    [sym_time] = ACTIONS(1909),
    [sym_ldate] = ACTIONS(1909),
    [sym_push_device] = ACTIONS(1909),
    [sym_push_channel] = ACTIONS(1909),
    [sym_push_devchan] = ACTIONS(1909),
    [sym_release_device] = ACTIONS(1909),
    [sym_release_channel] = ACTIONS(1909),
    [sym_release_devchan] = ACTIONS(1909),
    [sym_master_slot] = ACTIONS(1909),
    [sym_get_pulse_time] = ACTIONS(1909),
    [sym_get_timer] = ACTIONS(1909),
    [sym_master_sn] = ACTIONS(1909),
    [sym_system_number] = ACTIONS(1909),
    [sym_button] = ACTIONS(1909),
    [sym_dv_channel] = ACTIONS(1909),
    [sym_data] = ACTIONS(1909),
    [sym_level] = ACTIONS(1909),
    [sym_timeline] = ACTIONS(1909),
    [sym_channel] = ACTIONS(1909),
    [sym_custom] = ACTIONS(1909),
    [sym_atoi] = ACTIONS(1909),
    [sym_atol] = ACTIONS(1909),
    [sym_atof] = ACTIONS(1909),
    [sym_itoa] = ACTIONS(1909),
    [sym_itohex] = ACTIONS(1909),
    [sym_hextoi] = ACTIONS(1909),
    [sym_ftoa] = ACTIONS(1909),
    [sym_find_string] = ACTIONS(1909),
    [sym_left_string] = ACTIONS(1909),
    [sym_right_string] = ACTIONS(1909),
    [sym_mid_string] = ACTIONS(1909),
    [sym_remove_string] = ACTIONS(1909),
    [sym_get_buffer_string] = ACTIONS(1909),
    [sym_get_buffer_char] = ACTIONS(1909),
    [sym_get_multi_buffer_string] = ACTIONS(1909),
    [sym_length_string] = ACTIONS(1909),
    [sym_max_length_string] = ACTIONS(1909),
    [sym_set_length_string] = ACTIONS(1909),
    [sym_compare_string] = ACTIONS(1909),
    [sym_upper_string] = ACTIONS(1909),
    [sym_lower_string] = ACTIONS(1909),
    [sym_get_last] = ACTIONS(1909),
    [sym_date_to_day] = ACTIONS(1909),
    [sym_date_to_month] = ACTIONS(1909),
    [sym_date_to_year] = ACTIONS(1909),
    [sym_time_to_hour] = ACTIONS(1909),
    [sym_time_to_minute] = ACTIONS(1909),
    [sym_time_to_second] = ACTIONS(1909),
    [sym_day_of_week] = ACTIONS(1909),
    [sym_set_pulse_time] = ACTIONS(1909),
    [sym_set_timer] = ACTIONS(1909),
    [sym_file_open] = ACTIONS(1909),
    [sym_file_close] = ACTIONS(1909),
    [sym_file_read] = ACTIONS(1909),
    [sym_file_read_line] = ACTIONS(1909),
    [sym_file_write] = ACTIONS(1909),
    [sym_file_write_line] = ACTIONS(1909),
    [sym_file_copy] = ACTIONS(1909),
    [sym_file_delete] = ACTIONS(1909),
    [sym_file_rename] = ACTIONS(1909),
    [sym_file_dir] = ACTIONS(1909),
    [sym_file_getdir] = ACTIONS(1909),
    [sym_file_setdir] = ACTIONS(1909),
    [sym_file_create_dir] = ACTIONS(1909),
    [sym_file_remove_dir] = ACTIONS(1909),
    [sym_file_seek] = ACTIONS(1909),
    [sym_length_array] = ACTIONS(1909),
    [sym_max_length_array] = ACTIONS(1909),
    [sym_set_length_array] = ACTIONS(1909),
    [sym_ip_client_open] = ACTIONS(1909),
    [sym_ip_client_close] = ACTIONS(1909),
    [sym_ip_server_open] = ACTIONS(1909),
    [sym_ip_server_close] = ACTIONS(1909),
    [sym_ip_bound_client_open] = ACTIONS(1909),
    [sym_ip_mc_server_open] = ACTIONS(1909),
    [sym_ip_set_option] = ACTIONS(1909),
    [sym_ssh_client_open] = ACTIONS(1909),
    [sym_ssh_client_close] = ACTIONS(1909),
    [sym_tls_client_open] = ACTIONS(1909),
    [sym_tls_client_close] = ACTIONS(1909),
    [sym_do_push] = ACTIONS(1909),
    [sym_do_push_timed] = ACTIONS(1909),
    [sym_do_release] = ACTIONS(1909),
    [sym_device_id] = ACTIONS(1909),
    [sym_device_info] = ACTIONS(1909),
    [sym_device_id_string] = ACTIONS(1909),
    [sym_device_standby] = ACTIONS(1909),
    [sym_device_wake] = ACTIONS(1909),
    [sym_timeline_create] = ACTIONS(1909),
    [sym_timeline_kill] = ACTIONS(1909),
    [sym_timeline_reload] = ACTIONS(1909),
    [sym_timeline_pause] = ACTIONS(1909),
    [sym_timeline_restart] = ACTIONS(1909),
    [sym_timeline_set] = ACTIONS(1909),
    [sym_timeline_get] = ACTIONS(1909),
    [sym_timeline_active] = ACTIONS(1909),
    [sym_timeline_dynamic_id] = ACTIONS(1909),
    [sym_variable_to_string] = ACTIONS(1909),
    [sym_string_to_variable] = ACTIONS(1909),
    [sym_length_variable_to_string] = ACTIONS(1909),
    [sym_type_cast] = ACTIONS(1909),
    [sym_variable_to_xml] = ACTIONS(1909),
    [sym_xml_to_variable] = ACTIONS(1909),
    [sym_length_variable_to_xml] = ACTIONS(1909),
    [sym_redirect_string] = ACTIONS(1909),
    [sym_random_number] = ACTIONS(1909),
    [sym_get_unique_id] = ACTIONS(1909),
    [sym_get_system_number] = ACTIONS(1909),
    [sym_set_system_number] = ACTIONS(1909),
    [sym_get_serial_number] = ACTIONS(1909),
    [sym_get_master_build] = ACTIONS(1909),
    [sym_set_validation_code] = ACTIONS(1909),
    [sym_reboot] = ACTIONS(1909),
    [sym_load_duet_module] = ACTIONS(1909),
    [sym_unload_duet_module] = ACTIONS(1909),
    [sym_duet_mem_size_set] = ACTIONS(1909),
    [sym_duet_mem_size_get] = ACTIONS(1909),
    [sym_dynamic_application_device] = ACTIONS(1909),
    [sym_dynamic_polled_port] = ACTIONS(1909),
    [sym_static_port_binding] = ACTIONS(1909),
    [sym_static_ip_binding] = ACTIONS(1909),
    [sym_get_dns_list] = ACTIONS(1909),
    [sym_set_dns_list] = ACTIONS(1909),
    [sym_get_ip_address] = ACTIONS(1909),
    [sym_set_ip_address] = ACTIONS(1909),
    [sym_get_url_list] = ACTIONS(1909),
    [sym_add_url_entry] = ACTIONS(1909),
    [sym_delete_url_entry] = ACTIONS(1909),
    [sym_abs_value] = ACTIONS(1909),
    [sym_max_value] = ACTIONS(1909),
    [sym_min_value] = ACTIONS(1909),
    [sym_exp_value] = ACTIONS(1909),
    [sym_log_value] = ACTIONS(1909),
    [sym_log10_value] = ACTIONS(1909),
    [sym_power_value] = ACTIONS(1909),
    [sym_sqrt_value] = ACTIONS(1909),
    [sym_raw_be] = ACTIONS(1909),
    [sym_raw_le] = ACTIONS(1909),
    [sym_internal_queue_size_set] = ACTIONS(1909),
    [sym_internal_queue_size_get] = ACTIONS(1909),
    [sym_internal_threshold_set] = ACTIONS(1909),
    [sym_internal_threshold_get] = ACTIONS(1909),
    [sym_rebuilt_event] = ACTIONS(1909),
    [sym_do_custom_event] = ACTIONS(1909),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1909),
    [sym_clkmgr_set_clk_source] = ACTIONS(1909),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1909),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1909),
    [sym_clkmgr_get_timezone] = ACTIONS(1909),
    [sym_clkmgr_set_timezone] = ACTIONS(1909),
    [sym_clkmgr_get_resync_period] = ACTIONS(1909),
    [sym_clkmgr_set_resync_period] = ACTIONS(1909),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1909),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1909),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1909),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1909),
    [sym_clkmgr_get_timeservers] = ACTIONS(1909),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1909),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1909),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1909),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1909),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1909),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1909),
    [sym_set_log_level] = ACTIONS(1909),
    [sym_get_log_level] = ACTIONS(1909),
    [sym_amx_log] = ACTIONS(1909),
    [sym_smtp_server_config_set] = ACTIONS(1909),
    [sym_smtp_server_config_get] = ACTIONS(1909),
    [sym_smtp_send] = ACTIONS(1909),
    [sym_validate_netlinx_account] = ACTIONS(1909),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1909),
    [sym_authenticate_certificate] = ACTIONS(1909),
    [sym_audit_netlinx_session_event] = ACTIONS(1909),
    [sym_audit_netlinx_generic_event] = ACTIONS(1909),
    [sym_get_available_flash_disk_space] = ACTIONS(1909),
    [sym_get_max_flash_disk_space] = ACTIONS(1909),
    [sym_data_create_feed] = ACTIONS(1909),
    [sym_data_delete_feed] = ACTIONS(1909),
    [sym_data_publish_feed] = ACTIONS(1909),
    [sym_data_get_published_feed] = ACTIONS(1909),
    [sym_data_add_record] = ACTIONS(1909),
    [sym_data_get_event_record] = ACTIONS(1909),
    [sym__wc_data_create_feed] = ACTIONS(1909),
    [sym__wc_data_add_record] = ACTIONS(1909),
    [sym__wc_data_get_event_record] = ACTIONS(1909),
    [sym_format] = ACTIONS(1909),
    [sym_astro_clock] = ACTIONS(1909),
    [sym_set_outdoor_temperature] = ACTIONS(1909),
    [sym_set_virtual_level_count] = ACTIONS(1909),
    [sym_set_virtual_channel_count] = ACTIONS(1909),
    [sym_set_virtual_port_count] = ACTIONS(1909),
    [sym__wc_encode_int] = ACTIONS(1909),
    [sym__wc_decode_int] = ACTIONS(1909),
    [sym__wc_int] = ACTIONS(1909),
    [sym__wc_to_ch_int] = ACTIONS(1909),
    [sym__ch_to_wc_int] = ACTIONS(1909),
    [sym__wc_find_string_int] = ACTIONS(1909),
    [sym__wc_left_string_int] = ACTIONS(1909),
    [sym__wc_lower_string_int] = ACTIONS(1909),
    [sym__wc_mid_string_int] = ACTIONS(1909),
    [sym__wc_remove_string_int] = ACTIONS(1909),
    [sym__wc_right_string_int] = ACTIONS(1909),
    [sym__wc_upper_string_int] = ACTIONS(1909),
    [sym__wc_compare_string_int] = ACTIONS(1909),
    [sym__wc_get_buffer_char_int] = ACTIONS(1909),
    [sym__wc_get_buffer_string_int] = ACTIONS(1909),
    [sym__wc_concat_string_int] = ACTIONS(1909),
    [sym__wc_file_open_int] = ACTIONS(1909),
    [sym__wc_file_close_int] = ACTIONS(1909),
    [sym__wc_file_read_int] = ACTIONS(1909),
    [sym__wc_file_read_line_int] = ACTIONS(1909),
    [sym__wc_file_write_int] = ACTIONS(1909),
    [sym__wc_file_write_line_int] = ACTIONS(1909),
    [aux_sym_preproc_include_token1] = ACTIONS(1909),
    [aux_sym_preproc_define_token1] = ACTIONS(1911),
    [aux_sym_preproc_warn_token1] = ACTIONS(1911),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1911),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1911),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1911),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1911),
    [aux_sym_preproc_else_token1] = ACTIONS(1911),
    [anon_sym_LBRACK] = ACTIONS(1911),
    [anon_sym_LBRACE] = ACTIONS(1911),
    [anon_sym_LPAREN] = ACTIONS(1909),
    [anon_sym_BANG] = ACTIONS(1911),
    [anon_sym_TILDE] = ACTIONS(1911),
    [anon_sym_DASH] = ACTIONS(1909),
    [anon_sym_PLUS] = ACTIONS(1909),
    [anon_sym_DASH_DASH] = ACTIONS(1911),
    [anon_sym_PLUS_PLUS] = ACTIONS(1911),
    [anon_sym_DQUOTE] = ACTIONS(1911),
    [anon_sym_SQUOTE] = ACTIONS(1911),
    [sym_number_literal] = ACTIONS(1911),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(278)] = {
    [sym_identifier] = ACTIONS(1913),
    [sym_program_name_keyword] = ACTIONS(1913),
    [sym_module_name_keyword] = ACTIONS(1913),
    [sym_define_device_keyword] = ACTIONS(1913),
    [sym_define_combine_keyword] = ACTIONS(1913),
    [sym_define_constant_keyword] = ACTIONS(1913),
    [sym_define_type_keyword] = ACTIONS(1913),
    [sym_define_variable_keyword] = ACTIONS(1913),
    [sym_define_system_variable_keyword] = ACTIONS(1913),
    [sym_define_connect_level_keyword] = ACTIONS(1913),
    [sym_define_latching_keyword] = ACTIONS(1913),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1913),
    [sym_define_toggling_keyword] = ACTIONS(1913),
    [sym_define_call_keyword] = ACTIONS(1913),
    [sym_define_function_keyword] = ACTIONS(1913),
    [sym_define_library_function_keyword] = ACTIONS(1913),
    [sym_define_start_keyword] = ACTIONS(1913),
    [sym_define_event_keyword] = ACTIONS(1913),
    [sym_define_program_keyword] = ACTIONS(1913),
    [sym_define_module_keyword] = ACTIONS(1913),
    [sym_char_keyword] = ACTIONS(1913),
    [sym_widechar_keyword] = ACTIONS(1913),
    [sym_integer_keyword] = ACTIONS(1913),
    [sym_sinteger_keyword] = ACTIONS(1913),
    [sym_long_keyword] = ACTIONS(1913),
    [sym_slong_keyword] = ACTIONS(1913),
    [sym_float_keyword] = ACTIONS(1913),
    [sym_double_keyword] = ACTIONS(1913),
    [sym_variant_keyword] = ACTIONS(1913),
    [sym_variantarray_keyword] = ACTIONS(1913),
    [sym_local_var_keyword] = ACTIONS(1913),
    [sym_stack_var_keyword] = ACTIONS(1913),
    [sym_constant_keyword] = ACTIONS(1913),
    [sym_volatile_keyword] = ACTIONS(1913),
    [sym_non_volatile_keyword] = ACTIONS(1913),
    [sym_persistent_keyword] = ACTIONS(1913),
    [sym_dev_keyword] = ACTIONS(1913),
    [sym_devlev_keyword] = ACTIONS(1913),
    [sym_devchan_keyword] = ACTIONS(1913),
    [sym_if_keyword] = ACTIONS(1913),
    [sym_switch_keyword] = ACTIONS(1913),
    [sym_case_keyword] = ACTIONS(1913),
    [sym_default_keyword] = ACTIONS(1913),
    [sym_while_keyword] = ACTIONS(1913),
    [sym_for_keyword] = ACTIONS(1913),
    [sym_break_keyword] = ACTIONS(1913),
    [sym_continue_keyword] = ACTIONS(1913),
    [sym_return_keyword] = ACTIONS(1913),
    [sym_select_keyword] = ACTIONS(1913),
    [sym_active_keyword] = ACTIONS(1913),
    [sym_struct_keyword] = ACTIONS(1913),
    [sym_structure_keyword] = ACTIONS(1913),
    [sym_button_event_keyword] = ACTIONS(1913),
    [sym_channel_event_keyword] = ACTIONS(1913),
    [sym_level_event_keyword] = ACTIONS(1913),
    [sym_data_event_keyword] = ACTIONS(1913),
    [sym_timeline_event_keyword] = ACTIONS(1913),
    [sym_custom_event_keyword] = ACTIONS(1913),
    [sym_push_keyword] = ACTIONS(1913),
    [sym_release_keyword] = ACTIONS(1913),
    [aux_sym_on_keyword_token1] = ACTIONS(1913),
    [aux_sym_off_keyword_token1] = ACTIONS(1913),
    [sym_send_level_keyword] = ACTIONS(1913),
    [sym_send_string_keyword] = ACTIONS(1913),
    [sym_send_command_keyword] = ACTIONS(1913),
    [sym_clear_buffer_keyword] = ACTIONS(1913),
    [sym_create_buffer_keyword] = ACTIONS(1913),
    [sym_create_multi_buffer_keyword] = ACTIONS(1913),
    [sym_call_keyword] = ACTIONS(1913),
    [sym_system_call_keyword] = ACTIONS(1913),
    [sym_devchan_to_keyword] = ACTIONS(1913),
    [sym_devchan_min_to_keyword] = ACTIONS(1913),
    [sym_devchan_total_off_keyword] = ACTIONS(1913),
    [sym_devchan_pulse_keyword] = ACTIONS(1913),
    [sym_wait_keyword] = ACTIONS(1913),
    [sym_cancel_wait_keyword] = ACTIONS(1913),
    [sym_wait_until_keyword] = ACTIONS(1913),
    [sym_cancel_wait_until_keyword] = ACTIONS(1913),
    [sym_cancel_all_wait_keyword] = ACTIONS(1913),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1913),
    [sym_not] = ACTIONS(1913),
    [sym_bnot] = ACTIONS(1913),
    [sym___file__] = ACTIONS(1913),
    [sym___date__] = ACTIONS(1913),
    [sym___ldate__] = ACTIONS(1913),
    [sym___time__] = ACTIONS(1913),
    [sym___line__] = ACTIONS(1913),
    [sym___name__] = ACTIONS(1913),
    [sym___netlinx__] = ACTIONS(1913),
    [sym_netlinx_axi_version] = ACTIONS(1913),
    [sym_url_flg_tcp] = ACTIONS(1913),
    [sym_url_flg_accinfopresent] = ACTIONS(1913),
    [sym_url_flg_temp] = ACTIONS(1913),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1913),
    [sym_url_flg_stat_mask] = ACTIONS(1913),
    [sym_url_flg_stat_lookup] = ACTIONS(1913),
    [sym_url_flg_stat_connecting] = ACTIONS(1913),
    [sym_url_flg_stat_waiting] = ACTIONS(1913),
    [sym_url_flg_stat_connected] = ACTIONS(1913),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1913),
    [sym_first_virtual_device] = ACTIONS(1913),
    [sym_first_local_port] = ACTIONS(1913),
    [sym_dynamic_virtual_device] = ACTIONS(1913),
    [sym_timeline_once] = ACTIONS(1913),
    [sym_timeline_repeat] = ACTIONS(1913),
    [sym_timeline_absolute] = ACTIONS(1913),
    [sym_timeline_relative] = ACTIONS(1913),
    [sym_do_push_timed_infinite] = ACTIONS(1913),
    [sym_source_type_no_address] = ACTIONS(1913),
    [sym_source_type_neuron_id] = ACTIONS(1913),
    [sym_source_type_ip_address] = ACTIONS(1913),
    [sym_source_type_axlink] = ACTIONS(1913),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1913),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1913),
    [sym_source_type_ip_socket_address] = ACTIONS(1913),
    [sym_source_type_rs232] = ACTIONS(1913),
    [sym_source_type_internal] = ACTIONS(1913),
    [sym_source_type_mac_address] = ACTIONS(1913),
    [sym_source_type_ipv4_port] = ACTIONS(1913),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1913),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1913),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1913),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1913),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1913),
    [sym_true] = ACTIONS(1913),
    [sym_false] = ACTIONS(1913),
    [sym_file_read_only] = ACTIONS(1913),
    [sym_file_rw_new] = ACTIONS(1913),
    [sym_file_rw_append] = ACTIONS(1913),
    [sym_ip_tcp] = ACTIONS(1913),
    [sym_ip_udp] = ACTIONS(1913),
    [sym_ip_udp_2way] = ACTIONS(1913),
    [sym_xml_encode_types] = ACTIONS(1913),
    [sym_xml_encode_char_as_list] = ACTIONS(1913),
    [sym_xml_encode_le] = ACTIONS(1913),
    [sym_xml_decode_types] = ACTIONS(1913),
    [sym_xml_decode_no_preserve] = ACTIONS(1913),
    [sym_duet_dev_type_display_device] = ACTIONS(1913),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1913),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1913),
    [sym_duet_dev_type_io_device] = ACTIONS(1913),
    [sym_duet_dev_type_relay_device] = ACTIONS(1913),
    [sym_duet_dev_type_ups] = ACTIONS(1913),
    [sym_duet_dev_type_amplifier] = ACTIONS(1913),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1913),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1913),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1913),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1913),
    [sym_duet_dev_type_camera] = ACTIONS(1913),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1913),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1913),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1913),
    [sym_duet_dev_type_dss] = ACTIONS(1913),
    [sym_duet_dev_type_dvr] = ACTIONS(1913),
    [sym_duet_dev_type_disc_device] = ACTIONS(1913),
    [sym_duet_dev_type_document_camera] = ACTIONS(1913),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1913),
    [sym_duet_dev_type_hvac] = ACTIONS(1913),
    [sym_duet_dev_type_keypad] = ACTIONS(1913),
    [sym_duet_dev_type_light] = ACTIONS(1913),
    [sym_duet_dev_type_monitor] = ACTIONS(1913),
    [sym_duet_dev_type_motor] = ACTIONS(1913),
    [sym_duet_dev_type_multi_window] = ACTIONS(1913),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1913),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1913),
    [sym_duet_dev_type_receiver] = ACTIONS(1913),
    [sym_duet_dev_type_security_system] = ACTIONS(1913),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1913),
    [sym_duet_dev_type_settop_box] = ACTIONS(1913),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1913),
    [sym_duet_dev_type_switcher] = ACTIONS(1913),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1913),
    [sym_duet_dev_type_tv] = ACTIONS(1913),
    [sym_duet_dev_type_utility] = ACTIONS(1913),
    [sym_duet_dev_type_vcr] = ACTIONS(1913),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1913),
    [sym_duet_dev_type_video_processor] = ACTIONS(1913),
    [sym_duet_dev_type_video_projector] = ACTIONS(1913),
    [sym_duet_dev_type_video_wall] = ACTIONS(1913),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1913),
    [sym_duet_dev_type_weather] = ACTIONS(1913),
    [sym_duet_dev_not_polled] = ACTIONS(1913),
    [sym_duet_dev_polled] = ACTIONS(1913),
    [sym_clkmgr_mode_network] = ACTIONS(1913),
    [sym_clkmgr_mode_standalone] = ACTIONS(1913),
    [sym_normal_standby] = ACTIONS(1913),
    [sym_normal_wake] = ACTIONS(1913),
    [sym_ip_multicast_ttl_option] = ACTIONS(1913),
    [sym_ip_ttl_subnet] = ACTIONS(1913),
    [sym_ip_ttl_site] = ACTIONS(1913),
    [sym_ip_ttl_region] = ACTIONS(1913),
    [sym_ip_ttl_continent] = ACTIONS(1913),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1913),
    [sym_ip_nodelay_on] = ACTIONS(1913),
    [sym_ip_nodelay_off] = ACTIONS(1913),
    [sym_amx_error] = ACTIONS(1913),
    [sym_amx_warning] = ACTIONS(1913),
    [sym_amx_info] = ACTIONS(1913),
    [sym_amx_debug] = ACTIONS(1913),
    [sym_netlinx_login_success] = ACTIONS(1913),
    [sym_netlinx_login_fail] = ACTIONS(1913),
    [sym_netlinx_logout] = ACTIONS(1913),
    [sym_null_str] = ACTIONS(1913),
    [sym_smtp_address] = ACTIONS(1913),
    [sym_smtp_port_number] = ACTIONS(1913),
    [sym_smtp_username] = ACTIONS(1913),
    [sym_smtp_password] = ACTIONS(1913),
    [sym_smtp_require_tls] = ACTIONS(1913),
    [sym_smtp_from] = ACTIONS(1913),
    [sym_smtp_tls_true] = ACTIONS(1913),
    [sym_smtp_tls_false] = ACTIONS(1913),
    [sym_listview_on_row_select_event] = ACTIONS(1913),
    [sym_data_structure_unknown] = ACTIONS(1913),
    [sym_data_structure_datafeed] = ACTIONS(1913),
    [sym_data_structure_datarecordset] = ACTIONS(1913),
    [sym_data_structure_datarecord] = ACTIONS(1913),
    [sym_data_structure_datafield] = ACTIONS(1913),
    [sym_data_max_content_field_count] = ACTIONS(1913),
    [sym_data_max_metadata_field_count] = ACTIONS(1913),
    [sym_data_max_id_length] = ACTIONS(1913),
    [sym_data_max_name_length] = ACTIONS(1913),
    [sym_data_max_description_length] = ACTIONS(1913),
    [sym_data_max_type_length] = ACTIONS(1913),
    [sym_data_max_format_length] = ACTIONS(1913),
    [sym_data_max_source_length] = ACTIONS(1913),
    [sym_data_max_value_length] = ACTIONS(1913),
    [sym_data_max_label_length] = ACTIONS(1913),
    [sym_data_max_url_length] = ACTIONS(1913),
    [sym_max_last_login_info_length] = ACTIONS(1913),
    [sym_data_type_unknown] = ACTIONS(1913),
    [sym_data_type_string] = ACTIONS(1913),
    [sym_data_type_datetime] = ACTIONS(1913),
    [sym_data_type_date] = ACTIONS(1913),
    [sym_data_type_time] = ACTIONS(1913),
    [sym_data_type_image] = ACTIONS(1913),
    [sym_data_format_url] = ACTIONS(1913),
    [sym_data_format_phone] = ACTIONS(1913),
    [sym_data_format_email] = ACTIONS(1913),
    [sym_data_format_iso8601] = ACTIONS(1913),
    [sym_auditlog_priv] = ACTIONS(1913),
    [sym_configuration_priv] = ACTIONS(1913),
    [sym_deviceconfig_priv] = ACTIONS(1913),
    [sym_ftp_priv] = ACTIONS(1913),
    [sym_http_priv] = ACTIONS(1913),
    [sym_networkconfig_priv] = ACTIONS(1913),
    [sym_programport_priv] = ACTIONS(1913),
    [sym_remoteui_priv] = ACTIONS(1913),
    [sym_securitycontrol_priv] = ACTIONS(1913),
    [sym_softwaremanagement_priv] = ACTIONS(1913),
    [sym_terminal_priv] = ACTIONS(1913),
    [sym_tpadmin_priv] = ACTIONS(1913),
    [sym_user1_priv] = ACTIONS(1913),
    [sym_user2_priv] = ACTIONS(1913),
    [sym_user3_priv] = ACTIONS(1913),
    [sym_user4_priv] = ACTIONS(1913),
    [sym_usermanagement_priv] = ACTIONS(1913),
    [sym_valid_account] = ACTIONS(1913),
    [sym_err_name_invalid] = ACTIONS(1913),
    [sym_err_password_invalid] = ACTIONS(1913),
    [sym_err_invalid_account] = ACTIONS(1913),
    [sym_err_no_authorization] = ACTIONS(1913),
    [sym_err_invalid_parameter] = ACTIONS(1913),
    [sym_err_user_locked_out] = ACTIONS(1913),
    [sym_err_user_account_expired] = ACTIONS(1913),
    [sym_err_general_error] = ACTIONS(1913),
    [sym_err_authentication_error] = ACTIONS(1913),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1913),
    [sym_tls_validate_certificate] = ACTIONS(1913),
    [sym_tdata] = ACTIONS(1913),
    [sym_tchannel] = ACTIONS(1913),
    [sym_tlevel] = ACTIONS(1913),
    [sym_tbutton] = ACTIONS(1913),
    [sym_ttimeline] = ACTIONS(1913),
    [sym_tcustom] = ACTIONS(1913),
    [sym_url_struct] = ACTIONS(1913),
    [sym_dns_struct] = ACTIONS(1913),
    [sym_ip_address_struct] = ACTIONS(1913),
    [sym_dev_info_struct] = ACTIONS(1913),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1913),
    [sym_data_feed] = ACTIONS(1913),
    [sym_data_field] = ACTIONS(1913),
    [sym_data_record] = ACTIONS(1913),
    [sym_wc_data_feed] = ACTIONS(1913),
    [sym_wc_data_field] = ACTIONS(1913),
    [sym_wc_data_record] = ACTIONS(1913),
    [sym_last_login_info] = ACTIONS(1913),
    [sym____reserved___] = ACTIONS(1913),
    [sym_date] = ACTIONS(1913),
    [sym_day] = ACTIONS(1913),
    [sym_time] = ACTIONS(1913),
    [sym_ldate] = ACTIONS(1913),
    [sym_push_device] = ACTIONS(1913),
    [sym_push_channel] = ACTIONS(1913),
    [sym_push_devchan] = ACTIONS(1913),
    [sym_release_device] = ACTIONS(1913),
    [sym_release_channel] = ACTIONS(1913),
    [sym_release_devchan] = ACTIONS(1913),
    [sym_master_slot] = ACTIONS(1913),
    [sym_get_pulse_time] = ACTIONS(1913),
    [sym_get_timer] = ACTIONS(1913),
    [sym_master_sn] = ACTIONS(1913),
    [sym_system_number] = ACTIONS(1913),
    [sym_button] = ACTIONS(1913),
    [sym_dv_channel] = ACTIONS(1913),
    [sym_data] = ACTIONS(1913),
    [sym_level] = ACTIONS(1913),
    [sym_timeline] = ACTIONS(1913),
    [sym_channel] = ACTIONS(1913),
    [sym_custom] = ACTIONS(1913),
    [sym_atoi] = ACTIONS(1913),
    [sym_atol] = ACTIONS(1913),
    [sym_atof] = ACTIONS(1913),
    [sym_itoa] = ACTIONS(1913),
    [sym_itohex] = ACTIONS(1913),
    [sym_hextoi] = ACTIONS(1913),
    [sym_ftoa] = ACTIONS(1913),
    [sym_find_string] = ACTIONS(1913),
    [sym_left_string] = ACTIONS(1913),
    [sym_right_string] = ACTIONS(1913),
    [sym_mid_string] = ACTIONS(1913),
    [sym_remove_string] = ACTIONS(1913),
    [sym_get_buffer_string] = ACTIONS(1913),
    [sym_get_buffer_char] = ACTIONS(1913),
    [sym_get_multi_buffer_string] = ACTIONS(1913),
    [sym_length_string] = ACTIONS(1913),
    [sym_max_length_string] = ACTIONS(1913),
    [sym_set_length_string] = ACTIONS(1913),
    [sym_compare_string] = ACTIONS(1913),
    [sym_upper_string] = ACTIONS(1913),
    [sym_lower_string] = ACTIONS(1913),
    [sym_get_last] = ACTIONS(1913),
    [sym_date_to_day] = ACTIONS(1913),
    [sym_date_to_month] = ACTIONS(1913),
    [sym_date_to_year] = ACTIONS(1913),
    [sym_time_to_hour] = ACTIONS(1913),
    [sym_time_to_minute] = ACTIONS(1913),
    [sym_time_to_second] = ACTIONS(1913),
    [sym_day_of_week] = ACTIONS(1913),
    [sym_set_pulse_time] = ACTIONS(1913),
    [sym_set_timer] = ACTIONS(1913),
    [sym_file_open] = ACTIONS(1913),
    [sym_file_close] = ACTIONS(1913),
    [sym_file_read] = ACTIONS(1913),
    [sym_file_read_line] = ACTIONS(1913),
    [sym_file_write] = ACTIONS(1913),
    [sym_file_write_line] = ACTIONS(1913),
    [sym_file_copy] = ACTIONS(1913),
    [sym_file_delete] = ACTIONS(1913),
    [sym_file_rename] = ACTIONS(1913),
    [sym_file_dir] = ACTIONS(1913),
    [sym_file_getdir] = ACTIONS(1913),
    [sym_file_setdir] = ACTIONS(1913),
    [sym_file_create_dir] = ACTIONS(1913),
    [sym_file_remove_dir] = ACTIONS(1913),
    [sym_file_seek] = ACTIONS(1913),
    [sym_length_array] = ACTIONS(1913),
    [sym_max_length_array] = ACTIONS(1913),
    [sym_set_length_array] = ACTIONS(1913),
    [sym_ip_client_open] = ACTIONS(1913),
    [sym_ip_client_close] = ACTIONS(1913),
    [sym_ip_server_open] = ACTIONS(1913),
    [sym_ip_server_close] = ACTIONS(1913),
    [sym_ip_bound_client_open] = ACTIONS(1913),
    [sym_ip_mc_server_open] = ACTIONS(1913),
    [sym_ip_set_option] = ACTIONS(1913),
    [sym_ssh_client_open] = ACTIONS(1913),
    [sym_ssh_client_close] = ACTIONS(1913),
    [sym_tls_client_open] = ACTIONS(1913),
    [sym_tls_client_close] = ACTIONS(1913),
    [sym_do_push] = ACTIONS(1913),
    [sym_do_push_timed] = ACTIONS(1913),
    [sym_do_release] = ACTIONS(1913),
    [sym_device_id] = ACTIONS(1913),
    [sym_device_info] = ACTIONS(1913),
    [sym_device_id_string] = ACTIONS(1913),
    [sym_device_standby] = ACTIONS(1913),
    [sym_device_wake] = ACTIONS(1913),
    [sym_timeline_create] = ACTIONS(1913),
    [sym_timeline_kill] = ACTIONS(1913),
    [sym_timeline_reload] = ACTIONS(1913),
    [sym_timeline_pause] = ACTIONS(1913),
    [sym_timeline_restart] = ACTIONS(1913),
    [sym_timeline_set] = ACTIONS(1913),
    [sym_timeline_get] = ACTIONS(1913),
    [sym_timeline_active] = ACTIONS(1913),
    [sym_timeline_dynamic_id] = ACTIONS(1913),
    [sym_variable_to_string] = ACTIONS(1913),
    [sym_string_to_variable] = ACTIONS(1913),
    [sym_length_variable_to_string] = ACTIONS(1913),
    [sym_type_cast] = ACTIONS(1913),
    [sym_variable_to_xml] = ACTIONS(1913),
    [sym_xml_to_variable] = ACTIONS(1913),
    [sym_length_variable_to_xml] = ACTIONS(1913),
    [sym_redirect_string] = ACTIONS(1913),
    [sym_random_number] = ACTIONS(1913),
    [sym_get_unique_id] = ACTIONS(1913),
    [sym_get_system_number] = ACTIONS(1913),
    [sym_set_system_number] = ACTIONS(1913),
    [sym_get_serial_number] = ACTIONS(1913),
    [sym_get_master_build] = ACTIONS(1913),
    [sym_set_validation_code] = ACTIONS(1913),
    [sym_reboot] = ACTIONS(1913),
    [sym_load_duet_module] = ACTIONS(1913),
    [sym_unload_duet_module] = ACTIONS(1913),
    [sym_duet_mem_size_set] = ACTIONS(1913),
    [sym_duet_mem_size_get] = ACTIONS(1913),
    [sym_dynamic_application_device] = ACTIONS(1913),
    [sym_dynamic_polled_port] = ACTIONS(1913),
    [sym_static_port_binding] = ACTIONS(1913),
    [sym_static_ip_binding] = ACTIONS(1913),
    [sym_get_dns_list] = ACTIONS(1913),
    [sym_set_dns_list] = ACTIONS(1913),
    [sym_get_ip_address] = ACTIONS(1913),
    [sym_set_ip_address] = ACTIONS(1913),
    [sym_get_url_list] = ACTIONS(1913),
    [sym_add_url_entry] = ACTIONS(1913),
    [sym_delete_url_entry] = ACTIONS(1913),
    [sym_abs_value] = ACTIONS(1913),
    [sym_max_value] = ACTIONS(1913),
    [sym_min_value] = ACTIONS(1913),
    [sym_exp_value] = ACTIONS(1913),
    [sym_log_value] = ACTIONS(1913),
    [sym_log10_value] = ACTIONS(1913),
    [sym_power_value] = ACTIONS(1913),
    [sym_sqrt_value] = ACTIONS(1913),
    [sym_raw_be] = ACTIONS(1913),
    [sym_raw_le] = ACTIONS(1913),
    [sym_internal_queue_size_set] = ACTIONS(1913),
    [sym_internal_queue_size_get] = ACTIONS(1913),
    [sym_internal_threshold_set] = ACTIONS(1913),
    [sym_internal_threshold_get] = ACTIONS(1913),
    [sym_rebuilt_event] = ACTIONS(1913),
    [sym_do_custom_event] = ACTIONS(1913),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1913),
    [sym_clkmgr_set_clk_source] = ACTIONS(1913),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1913),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1913),
    [sym_clkmgr_get_timezone] = ACTIONS(1913),
    [sym_clkmgr_set_timezone] = ACTIONS(1913),
    [sym_clkmgr_get_resync_period] = ACTIONS(1913),
    [sym_clkmgr_set_resync_period] = ACTIONS(1913),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1913),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1913),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1913),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1913),
    [sym_clkmgr_get_timeservers] = ACTIONS(1913),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1913),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1913),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1913),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1913),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1913),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1913),
    [sym_set_log_level] = ACTIONS(1913),
    [sym_get_log_level] = ACTIONS(1913),
    [sym_amx_log] = ACTIONS(1913),
    [sym_smtp_server_config_set] = ACTIONS(1913),
    [sym_smtp_server_config_get] = ACTIONS(1913),
    [sym_smtp_send] = ACTIONS(1913),
    [sym_validate_netlinx_account] = ACTIONS(1913),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1913),
    [sym_authenticate_certificate] = ACTIONS(1913),
    [sym_audit_netlinx_session_event] = ACTIONS(1913),
    [sym_audit_netlinx_generic_event] = ACTIONS(1913),
    [sym_get_available_flash_disk_space] = ACTIONS(1913),
    [sym_get_max_flash_disk_space] = ACTIONS(1913),
    [sym_data_create_feed] = ACTIONS(1913),
    [sym_data_delete_feed] = ACTIONS(1913),
    [sym_data_publish_feed] = ACTIONS(1913),
    [sym_data_get_published_feed] = ACTIONS(1913),
    [sym_data_add_record] = ACTIONS(1913),
    [sym_data_get_event_record] = ACTIONS(1913),
    [sym__wc_data_create_feed] = ACTIONS(1913),
    [sym__wc_data_add_record] = ACTIONS(1913),
    [sym__wc_data_get_event_record] = ACTIONS(1913),
    [sym_format] = ACTIONS(1913),
    [sym_astro_clock] = ACTIONS(1913),
    [sym_set_outdoor_temperature] = ACTIONS(1913),
    [sym_set_virtual_level_count] = ACTIONS(1913),
    [sym_set_virtual_channel_count] = ACTIONS(1913),
    [sym_set_virtual_port_count] = ACTIONS(1913),
    [sym__wc_encode_int] = ACTIONS(1913),
    [sym__wc_decode_int] = ACTIONS(1913),
    [sym__wc_int] = ACTIONS(1913),
    [sym__wc_to_ch_int] = ACTIONS(1913),
    [sym__ch_to_wc_int] = ACTIONS(1913),
    [sym__wc_find_string_int] = ACTIONS(1913),
    [sym__wc_left_string_int] = ACTIONS(1913),
    [sym__wc_lower_string_int] = ACTIONS(1913),
    [sym__wc_mid_string_int] = ACTIONS(1913),
    [sym__wc_remove_string_int] = ACTIONS(1913),
    [sym__wc_right_string_int] = ACTIONS(1913),
    [sym__wc_upper_string_int] = ACTIONS(1913),
    [sym__wc_compare_string_int] = ACTIONS(1913),
    [sym__wc_get_buffer_char_int] = ACTIONS(1913),
    [sym__wc_get_buffer_string_int] = ACTIONS(1913),
    [sym__wc_concat_string_int] = ACTIONS(1913),
    [sym__wc_file_open_int] = ACTIONS(1913),
    [sym__wc_file_close_int] = ACTIONS(1913),
    [sym__wc_file_read_int] = ACTIONS(1913),
    [sym__wc_file_read_line_int] = ACTIONS(1913),
    [sym__wc_file_write_int] = ACTIONS(1913),
    [sym__wc_file_write_line_int] = ACTIONS(1913),
    [aux_sym_preproc_include_token1] = ACTIONS(1913),
    [aux_sym_preproc_define_token1] = ACTIONS(1915),
    [aux_sym_preproc_warn_token1] = ACTIONS(1915),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1915),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1915),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1915),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1915),
    [aux_sym_preproc_else_token1] = ACTIONS(1915),
    [anon_sym_LBRACK] = ACTIONS(1915),
    [anon_sym_LBRACE] = ACTIONS(1915),
    [anon_sym_LPAREN] = ACTIONS(1913),
    [anon_sym_BANG] = ACTIONS(1915),
    [anon_sym_TILDE] = ACTIONS(1915),
    [anon_sym_DASH] = ACTIONS(1913),
    [anon_sym_PLUS] = ACTIONS(1913),
    [anon_sym_DASH_DASH] = ACTIONS(1915),
    [anon_sym_PLUS_PLUS] = ACTIONS(1915),
    [anon_sym_DQUOTE] = ACTIONS(1915),
    [anon_sym_SQUOTE] = ACTIONS(1915),
    [sym_number_literal] = ACTIONS(1915),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(279)] = {
    [sym_identifier] = ACTIONS(1917),
    [sym_program_name_keyword] = ACTIONS(1917),
    [sym_module_name_keyword] = ACTIONS(1917),
    [sym_define_device_keyword] = ACTIONS(1917),
    [sym_define_combine_keyword] = ACTIONS(1917),
    [sym_define_constant_keyword] = ACTIONS(1917),
    [sym_define_type_keyword] = ACTIONS(1917),
    [sym_define_variable_keyword] = ACTIONS(1917),
    [sym_define_system_variable_keyword] = ACTIONS(1917),
    [sym_define_connect_level_keyword] = ACTIONS(1917),
    [sym_define_latching_keyword] = ACTIONS(1917),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1917),
    [sym_define_toggling_keyword] = ACTIONS(1917),
    [sym_define_call_keyword] = ACTIONS(1917),
    [sym_define_function_keyword] = ACTIONS(1917),
    [sym_define_library_function_keyword] = ACTIONS(1917),
    [sym_define_start_keyword] = ACTIONS(1917),
    [sym_define_event_keyword] = ACTIONS(1917),
    [sym_define_program_keyword] = ACTIONS(1917),
    [sym_define_module_keyword] = ACTIONS(1917),
    [sym_char_keyword] = ACTIONS(1917),
    [sym_widechar_keyword] = ACTIONS(1917),
    [sym_integer_keyword] = ACTIONS(1917),
    [sym_sinteger_keyword] = ACTIONS(1917),
    [sym_long_keyword] = ACTIONS(1917),
    [sym_slong_keyword] = ACTIONS(1917),
    [sym_float_keyword] = ACTIONS(1917),
    [sym_double_keyword] = ACTIONS(1917),
    [sym_variant_keyword] = ACTIONS(1917),
    [sym_variantarray_keyword] = ACTIONS(1917),
    [sym_local_var_keyword] = ACTIONS(1917),
    [sym_stack_var_keyword] = ACTIONS(1917),
    [sym_constant_keyword] = ACTIONS(1917),
    [sym_volatile_keyword] = ACTIONS(1917),
    [sym_non_volatile_keyword] = ACTIONS(1917),
    [sym_persistent_keyword] = ACTIONS(1917),
    [sym_dev_keyword] = ACTIONS(1917),
    [sym_devlev_keyword] = ACTIONS(1917),
    [sym_devchan_keyword] = ACTIONS(1917),
    [sym_if_keyword] = ACTIONS(1917),
    [sym_switch_keyword] = ACTIONS(1917),
    [sym_case_keyword] = ACTIONS(1917),
    [sym_default_keyword] = ACTIONS(1917),
    [sym_while_keyword] = ACTIONS(1917),
    [sym_for_keyword] = ACTIONS(1917),
    [sym_break_keyword] = ACTIONS(1917),
    [sym_continue_keyword] = ACTIONS(1917),
    [sym_return_keyword] = ACTIONS(1917),
    [sym_select_keyword] = ACTIONS(1917),
    [sym_active_keyword] = ACTIONS(1917),
    [sym_struct_keyword] = ACTIONS(1917),
    [sym_structure_keyword] = ACTIONS(1917),
    [sym_button_event_keyword] = ACTIONS(1917),
    [sym_channel_event_keyword] = ACTIONS(1917),
    [sym_level_event_keyword] = ACTIONS(1917),
    [sym_data_event_keyword] = ACTIONS(1917),
    [sym_timeline_event_keyword] = ACTIONS(1917),
    [sym_custom_event_keyword] = ACTIONS(1917),
    [sym_push_keyword] = ACTIONS(1917),
    [sym_release_keyword] = ACTIONS(1917),
    [aux_sym_on_keyword_token1] = ACTIONS(1917),
    [aux_sym_off_keyword_token1] = ACTIONS(1917),
    [sym_send_level_keyword] = ACTIONS(1917),
    [sym_send_string_keyword] = ACTIONS(1917),
    [sym_send_command_keyword] = ACTIONS(1917),
    [sym_clear_buffer_keyword] = ACTIONS(1917),
    [sym_create_buffer_keyword] = ACTIONS(1917),
    [sym_create_multi_buffer_keyword] = ACTIONS(1917),
    [sym_call_keyword] = ACTIONS(1917),
    [sym_system_call_keyword] = ACTIONS(1917),
    [sym_devchan_to_keyword] = ACTIONS(1917),
    [sym_devchan_min_to_keyword] = ACTIONS(1917),
    [sym_devchan_total_off_keyword] = ACTIONS(1917),
    [sym_devchan_pulse_keyword] = ACTIONS(1917),
    [sym_wait_keyword] = ACTIONS(1917),
    [sym_cancel_wait_keyword] = ACTIONS(1917),
    [sym_wait_until_keyword] = ACTIONS(1917),
    [sym_cancel_wait_until_keyword] = ACTIONS(1917),
    [sym_cancel_all_wait_keyword] = ACTIONS(1917),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1917),
    [sym_not] = ACTIONS(1917),
    [sym_bnot] = ACTIONS(1917),
    [sym___file__] = ACTIONS(1917),
    [sym___date__] = ACTIONS(1917),
    [sym___ldate__] = ACTIONS(1917),
    [sym___time__] = ACTIONS(1917),
    [sym___line__] = ACTIONS(1917),
    [sym___name__] = ACTIONS(1917),
    [sym___netlinx__] = ACTIONS(1917),
    [sym_netlinx_axi_version] = ACTIONS(1917),
    [sym_url_flg_tcp] = ACTIONS(1917),
    [sym_url_flg_accinfopresent] = ACTIONS(1917),
    [sym_url_flg_temp] = ACTIONS(1917),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1917),
    [sym_url_flg_stat_mask] = ACTIONS(1917),
    [sym_url_flg_stat_lookup] = ACTIONS(1917),
    [sym_url_flg_stat_connecting] = ACTIONS(1917),
    [sym_url_flg_stat_waiting] = ACTIONS(1917),
    [sym_url_flg_stat_connected] = ACTIONS(1917),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1917),
    [sym_first_virtual_device] = ACTIONS(1917),
    [sym_first_local_port] = ACTIONS(1917),
    [sym_dynamic_virtual_device] = ACTIONS(1917),
    [sym_timeline_once] = ACTIONS(1917),
    [sym_timeline_repeat] = ACTIONS(1917),
    [sym_timeline_absolute] = ACTIONS(1917),
    [sym_timeline_relative] = ACTIONS(1917),
    [sym_do_push_timed_infinite] = ACTIONS(1917),
    [sym_source_type_no_address] = ACTIONS(1917),
    [sym_source_type_neuron_id] = ACTIONS(1917),
    [sym_source_type_ip_address] = ACTIONS(1917),
    [sym_source_type_axlink] = ACTIONS(1917),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1917),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1917),
    [sym_source_type_ip_socket_address] = ACTIONS(1917),
    [sym_source_type_rs232] = ACTIONS(1917),
    [sym_source_type_internal] = ACTIONS(1917),
    [sym_source_type_mac_address] = ACTIONS(1917),
    [sym_source_type_ipv4_port] = ACTIONS(1917),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1917),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1917),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1917),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1917),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1917),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1917),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1917),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1917),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1917),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1917),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1917),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1917),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1917),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1917),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1917),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1917),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1917),
    [sym_true] = ACTIONS(1917),
    [sym_false] = ACTIONS(1917),
    [sym_file_read_only] = ACTIONS(1917),
    [sym_file_rw_new] = ACTIONS(1917),
    [sym_file_rw_append] = ACTIONS(1917),
    [sym_ip_tcp] = ACTIONS(1917),
    [sym_ip_udp] = ACTIONS(1917),
    [sym_ip_udp_2way] = ACTIONS(1917),
    [sym_xml_encode_types] = ACTIONS(1917),
    [sym_xml_encode_char_as_list] = ACTIONS(1917),
    [sym_xml_encode_le] = ACTIONS(1917),
    [sym_xml_decode_types] = ACTIONS(1917),
    [sym_xml_decode_no_preserve] = ACTIONS(1917),
    [sym_duet_dev_type_display_device] = ACTIONS(1917),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1917),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1917),
    [sym_duet_dev_type_io_device] = ACTIONS(1917),
    [sym_duet_dev_type_relay_device] = ACTIONS(1917),
    [sym_duet_dev_type_ups] = ACTIONS(1917),
    [sym_duet_dev_type_amplifier] = ACTIONS(1917),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1917),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1917),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1917),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1917),
    [sym_duet_dev_type_camera] = ACTIONS(1917),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1917),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1917),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1917),
    [sym_duet_dev_type_dss] = ACTIONS(1917),
    [sym_duet_dev_type_dvr] = ACTIONS(1917),
    [sym_duet_dev_type_disc_device] = ACTIONS(1917),
    [sym_duet_dev_type_document_camera] = ACTIONS(1917),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1917),
    [sym_duet_dev_type_hvac] = ACTIONS(1917),
    [sym_duet_dev_type_keypad] = ACTIONS(1917),
    [sym_duet_dev_type_light] = ACTIONS(1917),
    [sym_duet_dev_type_monitor] = ACTIONS(1917),
    [sym_duet_dev_type_motor] = ACTIONS(1917),
    [sym_duet_dev_type_multi_window] = ACTIONS(1917),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1917),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1917),
    [sym_duet_dev_type_receiver] = ACTIONS(1917),
    [sym_duet_dev_type_security_system] = ACTIONS(1917),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1917),
    [sym_duet_dev_type_settop_box] = ACTIONS(1917),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1917),
    [sym_duet_dev_type_switcher] = ACTIONS(1917),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1917),
    [sym_duet_dev_type_tv] = ACTIONS(1917),
    [sym_duet_dev_type_utility] = ACTIONS(1917),
    [sym_duet_dev_type_vcr] = ACTIONS(1917),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1917),
    [sym_duet_dev_type_video_processor] = ACTIONS(1917),
    [sym_duet_dev_type_video_projector] = ACTIONS(1917),
    [sym_duet_dev_type_video_wall] = ACTIONS(1917),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1917),
    [sym_duet_dev_type_weather] = ACTIONS(1917),
    [sym_duet_dev_not_polled] = ACTIONS(1917),
    [sym_duet_dev_polled] = ACTIONS(1917),
    [sym_clkmgr_mode_network] = ACTIONS(1917),
    [sym_clkmgr_mode_standalone] = ACTIONS(1917),
    [sym_normal_standby] = ACTIONS(1917),
    [sym_normal_wake] = ACTIONS(1917),
    [sym_ip_multicast_ttl_option] = ACTIONS(1917),
    [sym_ip_ttl_subnet] = ACTIONS(1917),
    [sym_ip_ttl_site] = ACTIONS(1917),
    [sym_ip_ttl_region] = ACTIONS(1917),
    [sym_ip_ttl_continent] = ACTIONS(1917),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1917),
    [sym_ip_nodelay_on] = ACTIONS(1917),
    [sym_ip_nodelay_off] = ACTIONS(1917),
    [sym_amx_error] = ACTIONS(1917),
    [sym_amx_warning] = ACTIONS(1917),
    [sym_amx_info] = ACTIONS(1917),
    [sym_amx_debug] = ACTIONS(1917),
    [sym_netlinx_login_success] = ACTIONS(1917),
    [sym_netlinx_login_fail] = ACTIONS(1917),
    [sym_netlinx_logout] = ACTIONS(1917),
    [sym_null_str] = ACTIONS(1917),
    [sym_smtp_address] = ACTIONS(1917),
    [sym_smtp_port_number] = ACTIONS(1917),
    [sym_smtp_username] = ACTIONS(1917),
    [sym_smtp_password] = ACTIONS(1917),
    [sym_smtp_require_tls] = ACTIONS(1917),
    [sym_smtp_from] = ACTIONS(1917),
    [sym_smtp_tls_true] = ACTIONS(1917),
    [sym_smtp_tls_false] = ACTIONS(1917),
    [sym_listview_on_row_select_event] = ACTIONS(1917),
    [sym_data_structure_unknown] = ACTIONS(1917),
    [sym_data_structure_datafeed] = ACTIONS(1917),
    [sym_data_structure_datarecordset] = ACTIONS(1917),
    [sym_data_structure_datarecord] = ACTIONS(1917),
    [sym_data_structure_datafield] = ACTIONS(1917),
    [sym_data_max_content_field_count] = ACTIONS(1917),
    [sym_data_max_metadata_field_count] = ACTIONS(1917),
    [sym_data_max_id_length] = ACTIONS(1917),
    [sym_data_max_name_length] = ACTIONS(1917),
    [sym_data_max_description_length] = ACTIONS(1917),
    [sym_data_max_type_length] = ACTIONS(1917),
    [sym_data_max_format_length] = ACTIONS(1917),
    [sym_data_max_source_length] = ACTIONS(1917),
    [sym_data_max_value_length] = ACTIONS(1917),
    [sym_data_max_label_length] = ACTIONS(1917),
    [sym_data_max_url_length] = ACTIONS(1917),
    [sym_max_last_login_info_length] = ACTIONS(1917),
    [sym_data_type_unknown] = ACTIONS(1917),
    [sym_data_type_string] = ACTIONS(1917),
    [sym_data_type_datetime] = ACTIONS(1917),
    [sym_data_type_date] = ACTIONS(1917),
    [sym_data_type_time] = ACTIONS(1917),
    [sym_data_type_image] = ACTIONS(1917),
    [sym_data_format_url] = ACTIONS(1917),
    [sym_data_format_phone] = ACTIONS(1917),
    [sym_data_format_email] = ACTIONS(1917),
    [sym_data_format_iso8601] = ACTIONS(1917),
    [sym_auditlog_priv] = ACTIONS(1917),
    [sym_configuration_priv] = ACTIONS(1917),
    [sym_deviceconfig_priv] = ACTIONS(1917),
    [sym_ftp_priv] = ACTIONS(1917),
    [sym_http_priv] = ACTIONS(1917),
    [sym_networkconfig_priv] = ACTIONS(1917),
    [sym_programport_priv] = ACTIONS(1917),
    [sym_remoteui_priv] = ACTIONS(1917),
    [sym_securitycontrol_priv] = ACTIONS(1917),
    [sym_softwaremanagement_priv] = ACTIONS(1917),
    [sym_terminal_priv] = ACTIONS(1917),
    [sym_tpadmin_priv] = ACTIONS(1917),
    [sym_user1_priv] = ACTIONS(1917),
    [sym_user2_priv] = ACTIONS(1917),
    [sym_user3_priv] = ACTIONS(1917),
    [sym_user4_priv] = ACTIONS(1917),
    [sym_usermanagement_priv] = ACTIONS(1917),
    [sym_valid_account] = ACTIONS(1917),
    [sym_err_name_invalid] = ACTIONS(1917),
    [sym_err_password_invalid] = ACTIONS(1917),
    [sym_err_invalid_account] = ACTIONS(1917),
    [sym_err_no_authorization] = ACTIONS(1917),
    [sym_err_invalid_parameter] = ACTIONS(1917),
    [sym_err_user_locked_out] = ACTIONS(1917),
    [sym_err_user_account_expired] = ACTIONS(1917),
    [sym_err_general_error] = ACTIONS(1917),
    [sym_err_authentication_error] = ACTIONS(1917),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1917),
    [sym_tls_validate_certificate] = ACTIONS(1917),
    [sym_tdata] = ACTIONS(1917),
    [sym_tchannel] = ACTIONS(1917),
    [sym_tlevel] = ACTIONS(1917),
    [sym_tbutton] = ACTIONS(1917),
    [sym_ttimeline] = ACTIONS(1917),
    [sym_tcustom] = ACTIONS(1917),
    [sym_url_struct] = ACTIONS(1917),
    [sym_dns_struct] = ACTIONS(1917),
    [sym_ip_address_struct] = ACTIONS(1917),
    [sym_dev_info_struct] = ACTIONS(1917),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1917),
    [sym_data_feed] = ACTIONS(1917),
    [sym_data_field] = ACTIONS(1917),
    [sym_data_record] = ACTIONS(1917),
    [sym_wc_data_feed] = ACTIONS(1917),
    [sym_wc_data_field] = ACTIONS(1917),
    [sym_wc_data_record] = ACTIONS(1917),
    [sym_last_login_info] = ACTIONS(1917),
    [sym____reserved___] = ACTIONS(1917),
    [sym_date] = ACTIONS(1917),
    [sym_day] = ACTIONS(1917),
    [sym_time] = ACTIONS(1917),
    [sym_ldate] = ACTIONS(1917),
    [sym_push_device] = ACTIONS(1917),
    [sym_push_channel] = ACTIONS(1917),
    [sym_push_devchan] = ACTIONS(1917),
    [sym_release_device] = ACTIONS(1917),
    [sym_release_channel] = ACTIONS(1917),
    [sym_release_devchan] = ACTIONS(1917),
    [sym_master_slot] = ACTIONS(1917),
    [sym_get_pulse_time] = ACTIONS(1917),
    [sym_get_timer] = ACTIONS(1917),
    [sym_master_sn] = ACTIONS(1917),
    [sym_system_number] = ACTIONS(1917),
    [sym_button] = ACTIONS(1917),
    [sym_dv_channel] = ACTIONS(1917),
    [sym_data] = ACTIONS(1917),
    [sym_level] = ACTIONS(1917),
    [sym_timeline] = ACTIONS(1917),
    [sym_channel] = ACTIONS(1917),
    [sym_custom] = ACTIONS(1917),
    [sym_atoi] = ACTIONS(1917),
    [sym_atol] = ACTIONS(1917),
    [sym_atof] = ACTIONS(1917),
    [sym_itoa] = ACTIONS(1917),
    [sym_itohex] = ACTIONS(1917),
    [sym_hextoi] = ACTIONS(1917),
    [sym_ftoa] = ACTIONS(1917),
    [sym_find_string] = ACTIONS(1917),
    [sym_left_string] = ACTIONS(1917),
    [sym_right_string] = ACTIONS(1917),
    [sym_mid_string] = ACTIONS(1917),
    [sym_remove_string] = ACTIONS(1917),
    [sym_get_buffer_string] = ACTIONS(1917),
    [sym_get_buffer_char] = ACTIONS(1917),
    [sym_get_multi_buffer_string] = ACTIONS(1917),
    [sym_length_string] = ACTIONS(1917),
    [sym_max_length_string] = ACTIONS(1917),
    [sym_set_length_string] = ACTIONS(1917),
    [sym_compare_string] = ACTIONS(1917),
    [sym_upper_string] = ACTIONS(1917),
    [sym_lower_string] = ACTIONS(1917),
    [sym_get_last] = ACTIONS(1917),
    [sym_date_to_day] = ACTIONS(1917),
    [sym_date_to_month] = ACTIONS(1917),
    [sym_date_to_year] = ACTIONS(1917),
    [sym_time_to_hour] = ACTIONS(1917),
    [sym_time_to_minute] = ACTIONS(1917),
    [sym_time_to_second] = ACTIONS(1917),
    [sym_day_of_week] = ACTIONS(1917),
    [sym_set_pulse_time] = ACTIONS(1917),
    [sym_set_timer] = ACTIONS(1917),
    [sym_file_open] = ACTIONS(1917),
    [sym_file_close] = ACTIONS(1917),
    [sym_file_read] = ACTIONS(1917),
    [sym_file_read_line] = ACTIONS(1917),
    [sym_file_write] = ACTIONS(1917),
    [sym_file_write_line] = ACTIONS(1917),
    [sym_file_copy] = ACTIONS(1917),
    [sym_file_delete] = ACTIONS(1917),
    [sym_file_rename] = ACTIONS(1917),
    [sym_file_dir] = ACTIONS(1917),
    [sym_file_getdir] = ACTIONS(1917),
    [sym_file_setdir] = ACTIONS(1917),
    [sym_file_create_dir] = ACTIONS(1917),
    [sym_file_remove_dir] = ACTIONS(1917),
    [sym_file_seek] = ACTIONS(1917),
    [sym_length_array] = ACTIONS(1917),
    [sym_max_length_array] = ACTIONS(1917),
    [sym_set_length_array] = ACTIONS(1917),
    [sym_ip_client_open] = ACTIONS(1917),
    [sym_ip_client_close] = ACTIONS(1917),
    [sym_ip_server_open] = ACTIONS(1917),
    [sym_ip_server_close] = ACTIONS(1917),
    [sym_ip_bound_client_open] = ACTIONS(1917),
    [sym_ip_mc_server_open] = ACTIONS(1917),
    [sym_ip_set_option] = ACTIONS(1917),
    [sym_ssh_client_open] = ACTIONS(1917),
    [sym_ssh_client_close] = ACTIONS(1917),
    [sym_tls_client_open] = ACTIONS(1917),
    [sym_tls_client_close] = ACTIONS(1917),
    [sym_do_push] = ACTIONS(1917),
    [sym_do_push_timed] = ACTIONS(1917),
    [sym_do_release] = ACTIONS(1917),
    [sym_device_id] = ACTIONS(1917),
    [sym_device_info] = ACTIONS(1917),
    [sym_device_id_string] = ACTIONS(1917),
    [sym_device_standby] = ACTIONS(1917),
    [sym_device_wake] = ACTIONS(1917),
    [sym_timeline_create] = ACTIONS(1917),
    [sym_timeline_kill] = ACTIONS(1917),
    [sym_timeline_reload] = ACTIONS(1917),
    [sym_timeline_pause] = ACTIONS(1917),
    [sym_timeline_restart] = ACTIONS(1917),
    [sym_timeline_set] = ACTIONS(1917),
    [sym_timeline_get] = ACTIONS(1917),
    [sym_timeline_active] = ACTIONS(1917),
    [sym_timeline_dynamic_id] = ACTIONS(1917),
    [sym_variable_to_string] = ACTIONS(1917),
    [sym_string_to_variable] = ACTIONS(1917),
    [sym_length_variable_to_string] = ACTIONS(1917),
    [sym_type_cast] = ACTIONS(1917),
    [sym_variable_to_xml] = ACTIONS(1917),
    [sym_xml_to_variable] = ACTIONS(1917),
    [sym_length_variable_to_xml] = ACTIONS(1917),
    [sym_redirect_string] = ACTIONS(1917),
    [sym_random_number] = ACTIONS(1917),
    [sym_get_unique_id] = ACTIONS(1917),
    [sym_get_system_number] = ACTIONS(1917),
    [sym_set_system_number] = ACTIONS(1917),
    [sym_get_serial_number] = ACTIONS(1917),
    [sym_get_master_build] = ACTIONS(1917),
    [sym_set_validation_code] = ACTIONS(1917),
    [sym_reboot] = ACTIONS(1917),
    [sym_load_duet_module] = ACTIONS(1917),
    [sym_unload_duet_module] = ACTIONS(1917),
    [sym_duet_mem_size_set] = ACTIONS(1917),
    [sym_duet_mem_size_get] = ACTIONS(1917),
    [sym_dynamic_application_device] = ACTIONS(1917),
    [sym_dynamic_polled_port] = ACTIONS(1917),
    [sym_static_port_binding] = ACTIONS(1917),
    [sym_static_ip_binding] = ACTIONS(1917),
    [sym_get_dns_list] = ACTIONS(1917),
    [sym_set_dns_list] = ACTIONS(1917),
    [sym_get_ip_address] = ACTIONS(1917),
    [sym_set_ip_address] = ACTIONS(1917),
    [sym_get_url_list] = ACTIONS(1917),
    [sym_add_url_entry] = ACTIONS(1917),
    [sym_delete_url_entry] = ACTIONS(1917),
    [sym_abs_value] = ACTIONS(1917),
    [sym_max_value] = ACTIONS(1917),
    [sym_min_value] = ACTIONS(1917),
    [sym_exp_value] = ACTIONS(1917),
    [sym_log_value] = ACTIONS(1917),
    [sym_log10_value] = ACTIONS(1917),
    [sym_power_value] = ACTIONS(1917),
    [sym_sqrt_value] = ACTIONS(1917),
    [sym_raw_be] = ACTIONS(1917),
    [sym_raw_le] = ACTIONS(1917),
    [sym_internal_queue_size_set] = ACTIONS(1917),
    [sym_internal_queue_size_get] = ACTIONS(1917),
    [sym_internal_threshold_set] = ACTIONS(1917),
    [sym_internal_threshold_get] = ACTIONS(1917),
    [sym_rebuilt_event] = ACTIONS(1917),
    [sym_do_custom_event] = ACTIONS(1917),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1917),
    [sym_clkmgr_set_clk_source] = ACTIONS(1917),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1917),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1917),
    [sym_clkmgr_get_timezone] = ACTIONS(1917),
    [sym_clkmgr_set_timezone] = ACTIONS(1917),
    [sym_clkmgr_get_resync_period] = ACTIONS(1917),
    [sym_clkmgr_set_resync_period] = ACTIONS(1917),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1917),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1917),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1917),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1917),
    [sym_clkmgr_get_timeservers] = ACTIONS(1917),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1917),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1917),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1917),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1917),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1917),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1917),
    [sym_set_log_level] = ACTIONS(1917),
    [sym_get_log_level] = ACTIONS(1917),
    [sym_amx_log] = ACTIONS(1917),
    [sym_smtp_server_config_set] = ACTIONS(1917),
    [sym_smtp_server_config_get] = ACTIONS(1917),
    [sym_smtp_send] = ACTIONS(1917),
    [sym_validate_netlinx_account] = ACTIONS(1917),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1917),
    [sym_authenticate_certificate] = ACTIONS(1917),
    [sym_audit_netlinx_session_event] = ACTIONS(1917),
    [sym_audit_netlinx_generic_event] = ACTIONS(1917),
    [sym_get_available_flash_disk_space] = ACTIONS(1917),
    [sym_get_max_flash_disk_space] = ACTIONS(1917),
    [sym_data_create_feed] = ACTIONS(1917),
    [sym_data_delete_feed] = ACTIONS(1917),
    [sym_data_publish_feed] = ACTIONS(1917),
    [sym_data_get_published_feed] = ACTIONS(1917),
    [sym_data_add_record] = ACTIONS(1917),
    [sym_data_get_event_record] = ACTIONS(1917),
    [sym__wc_data_create_feed] = ACTIONS(1917),
    [sym__wc_data_add_record] = ACTIONS(1917),
    [sym__wc_data_get_event_record] = ACTIONS(1917),
    [sym_format] = ACTIONS(1917),
    [sym_astro_clock] = ACTIONS(1917),
    [sym_set_outdoor_temperature] = ACTIONS(1917),
    [sym_set_virtual_level_count] = ACTIONS(1917),
    [sym_set_virtual_channel_count] = ACTIONS(1917),
    [sym_set_virtual_port_count] = ACTIONS(1917),
    [sym__wc_encode_int] = ACTIONS(1917),
    [sym__wc_decode_int] = ACTIONS(1917),
    [sym__wc_int] = ACTIONS(1917),
    [sym__wc_to_ch_int] = ACTIONS(1917),
    [sym__ch_to_wc_int] = ACTIONS(1917),
    [sym__wc_find_string_int] = ACTIONS(1917),
    [sym__wc_left_string_int] = ACTIONS(1917),
    [sym__wc_lower_string_int] = ACTIONS(1917),
    [sym__wc_mid_string_int] = ACTIONS(1917),
    [sym__wc_remove_string_int] = ACTIONS(1917),
    [sym__wc_right_string_int] = ACTIONS(1917),
    [sym__wc_upper_string_int] = ACTIONS(1917),
    [sym__wc_compare_string_int] = ACTIONS(1917),
    [sym__wc_get_buffer_char_int] = ACTIONS(1917),
    [sym__wc_get_buffer_string_int] = ACTIONS(1917),
    [sym__wc_concat_string_int] = ACTIONS(1917),
    [sym__wc_file_open_int] = ACTIONS(1917),
    [sym__wc_file_close_int] = ACTIONS(1917),
    [sym__wc_file_read_int] = ACTIONS(1917),
    [sym__wc_file_read_line_int] = ACTIONS(1917),
    [sym__wc_file_write_int] = ACTIONS(1917),
    [sym__wc_file_write_line_int] = ACTIONS(1917),
    [aux_sym_preproc_include_token1] = ACTIONS(1917),
    [aux_sym_preproc_define_token1] = ACTIONS(1919),
    [aux_sym_preproc_warn_token1] = ACTIONS(1919),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1919),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1919),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1919),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1919),
    [aux_sym_preproc_else_token1] = ACTIONS(1919),
    [anon_sym_LBRACK] = ACTIONS(1919),
    [anon_sym_LBRACE] = ACTIONS(1919),
    [anon_sym_LPAREN] = ACTIONS(1917),
    [anon_sym_BANG] = ACTIONS(1919),
    [anon_sym_TILDE] = ACTIONS(1919),
    [anon_sym_DASH] = ACTIONS(1917),
    [anon_sym_PLUS] = ACTIONS(1917),
    [anon_sym_DASH_DASH] = ACTIONS(1919),
    [anon_sym_PLUS_PLUS] = ACTIONS(1919),
    [anon_sym_DQUOTE] = ACTIONS(1919),
    [anon_sym_SQUOTE] = ACTIONS(1919),
    [sym_number_literal] = ACTIONS(1919),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(280)] = {
    [sym_identifier] = ACTIONS(1921),
    [sym_program_name_keyword] = ACTIONS(1921),
    [sym_module_name_keyword] = ACTIONS(1921),
    [sym_define_device_keyword] = ACTIONS(1921),
    [sym_define_combine_keyword] = ACTIONS(1921),
    [sym_define_constant_keyword] = ACTIONS(1921),
    [sym_define_type_keyword] = ACTIONS(1921),
    [sym_define_variable_keyword] = ACTIONS(1921),
    [sym_define_system_variable_keyword] = ACTIONS(1921),
    [sym_define_connect_level_keyword] = ACTIONS(1921),
    [sym_define_latching_keyword] = ACTIONS(1921),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1921),
    [sym_define_toggling_keyword] = ACTIONS(1921),
    [sym_define_call_keyword] = ACTIONS(1921),
    [sym_define_function_keyword] = ACTIONS(1921),
    [sym_define_library_function_keyword] = ACTIONS(1921),
    [sym_define_start_keyword] = ACTIONS(1921),
    [sym_define_event_keyword] = ACTIONS(1921),
    [sym_define_program_keyword] = ACTIONS(1921),
    [sym_define_module_keyword] = ACTIONS(1921),
    [sym_char_keyword] = ACTIONS(1921),
    [sym_widechar_keyword] = ACTIONS(1921),
    [sym_integer_keyword] = ACTIONS(1921),
    [sym_sinteger_keyword] = ACTIONS(1921),
    [sym_long_keyword] = ACTIONS(1921),
    [sym_slong_keyword] = ACTIONS(1921),
    [sym_float_keyword] = ACTIONS(1921),
    [sym_double_keyword] = ACTIONS(1921),
    [sym_variant_keyword] = ACTIONS(1921),
    [sym_variantarray_keyword] = ACTIONS(1921),
    [sym_local_var_keyword] = ACTIONS(1921),
    [sym_stack_var_keyword] = ACTIONS(1921),
    [sym_constant_keyword] = ACTIONS(1921),
    [sym_volatile_keyword] = ACTIONS(1921),
    [sym_non_volatile_keyword] = ACTIONS(1921),
    [sym_persistent_keyword] = ACTIONS(1921),
    [sym_dev_keyword] = ACTIONS(1921),
    [sym_devlev_keyword] = ACTIONS(1921),
    [sym_devchan_keyword] = ACTIONS(1921),
    [sym_if_keyword] = ACTIONS(1921),
    [sym_switch_keyword] = ACTIONS(1921),
    [sym_case_keyword] = ACTIONS(1921),
    [sym_default_keyword] = ACTIONS(1921),
    [sym_while_keyword] = ACTIONS(1921),
    [sym_for_keyword] = ACTIONS(1921),
    [sym_break_keyword] = ACTIONS(1921),
    [sym_continue_keyword] = ACTIONS(1921),
    [sym_return_keyword] = ACTIONS(1921),
    [sym_select_keyword] = ACTIONS(1921),
    [sym_active_keyword] = ACTIONS(1921),
    [sym_struct_keyword] = ACTIONS(1921),
    [sym_structure_keyword] = ACTIONS(1921),
    [sym_button_event_keyword] = ACTIONS(1921),
    [sym_channel_event_keyword] = ACTIONS(1921),
    [sym_level_event_keyword] = ACTIONS(1921),
    [sym_data_event_keyword] = ACTIONS(1921),
    [sym_timeline_event_keyword] = ACTIONS(1921),
    [sym_custom_event_keyword] = ACTIONS(1921),
    [sym_push_keyword] = ACTIONS(1921),
    [sym_release_keyword] = ACTIONS(1921),
    [aux_sym_on_keyword_token1] = ACTIONS(1921),
    [aux_sym_off_keyword_token1] = ACTIONS(1921),
    [sym_send_level_keyword] = ACTIONS(1921),
    [sym_send_string_keyword] = ACTIONS(1921),
    [sym_send_command_keyword] = ACTIONS(1921),
    [sym_clear_buffer_keyword] = ACTIONS(1921),
    [sym_create_buffer_keyword] = ACTIONS(1921),
    [sym_create_multi_buffer_keyword] = ACTIONS(1921),
    [sym_call_keyword] = ACTIONS(1921),
    [sym_system_call_keyword] = ACTIONS(1921),
    [sym_devchan_to_keyword] = ACTIONS(1921),
    [sym_devchan_min_to_keyword] = ACTIONS(1921),
    [sym_devchan_total_off_keyword] = ACTIONS(1921),
    [sym_devchan_pulse_keyword] = ACTIONS(1921),
    [sym_wait_keyword] = ACTIONS(1921),
    [sym_cancel_wait_keyword] = ACTIONS(1921),
    [sym_wait_until_keyword] = ACTIONS(1921),
    [sym_cancel_wait_until_keyword] = ACTIONS(1921),
    [sym_cancel_all_wait_keyword] = ACTIONS(1921),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1921),
    [sym_not] = ACTIONS(1921),
    [sym_bnot] = ACTIONS(1921),
    [sym___file__] = ACTIONS(1921),
    [sym___date__] = ACTIONS(1921),
    [sym___ldate__] = ACTIONS(1921),
    [sym___time__] = ACTIONS(1921),
    [sym___line__] = ACTIONS(1921),
    [sym___name__] = ACTIONS(1921),
    [sym___netlinx__] = ACTIONS(1921),
    [sym_netlinx_axi_version] = ACTIONS(1921),
    [sym_url_flg_tcp] = ACTIONS(1921),
    [sym_url_flg_accinfopresent] = ACTIONS(1921),
    [sym_url_flg_temp] = ACTIONS(1921),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1921),
    [sym_url_flg_stat_mask] = ACTIONS(1921),
    [sym_url_flg_stat_lookup] = ACTIONS(1921),
    [sym_url_flg_stat_connecting] = ACTIONS(1921),
    [sym_url_flg_stat_waiting] = ACTIONS(1921),
    [sym_url_flg_stat_connected] = ACTIONS(1921),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1921),
    [sym_first_virtual_device] = ACTIONS(1921),
    [sym_first_local_port] = ACTIONS(1921),
    [sym_dynamic_virtual_device] = ACTIONS(1921),
    [sym_timeline_once] = ACTIONS(1921),
    [sym_timeline_repeat] = ACTIONS(1921),
    [sym_timeline_absolute] = ACTIONS(1921),
    [sym_timeline_relative] = ACTIONS(1921),
    [sym_do_push_timed_infinite] = ACTIONS(1921),
    [sym_source_type_no_address] = ACTIONS(1921),
    [sym_source_type_neuron_id] = ACTIONS(1921),
    [sym_source_type_ip_address] = ACTIONS(1921),
    [sym_source_type_axlink] = ACTIONS(1921),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1921),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1921),
    [sym_source_type_ip_socket_address] = ACTIONS(1921),
    [sym_source_type_rs232] = ACTIONS(1921),
    [sym_source_type_internal] = ACTIONS(1921),
    [sym_source_type_mac_address] = ACTIONS(1921),
    [sym_source_type_ipv4_port] = ACTIONS(1921),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1921),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1921),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1921),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1921),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1921),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1921),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1921),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1921),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1921),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1921),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1921),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1921),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1921),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1921),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1921),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1921),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1921),
    [sym_true] = ACTIONS(1921),
    [sym_false] = ACTIONS(1921),
    [sym_file_read_only] = ACTIONS(1921),
    [sym_file_rw_new] = ACTIONS(1921),
    [sym_file_rw_append] = ACTIONS(1921),
    [sym_ip_tcp] = ACTIONS(1921),
    [sym_ip_udp] = ACTIONS(1921),
    [sym_ip_udp_2way] = ACTIONS(1921),
    [sym_xml_encode_types] = ACTIONS(1921),
    [sym_xml_encode_char_as_list] = ACTIONS(1921),
    [sym_xml_encode_le] = ACTIONS(1921),
    [sym_xml_decode_types] = ACTIONS(1921),
    [sym_xml_decode_no_preserve] = ACTIONS(1921),
    [sym_duet_dev_type_display_device] = ACTIONS(1921),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1921),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1921),
    [sym_duet_dev_type_io_device] = ACTIONS(1921),
    [sym_duet_dev_type_relay_device] = ACTIONS(1921),
    [sym_duet_dev_type_ups] = ACTIONS(1921),
    [sym_duet_dev_type_amplifier] = ACTIONS(1921),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1921),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1921),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1921),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1921),
    [sym_duet_dev_type_camera] = ACTIONS(1921),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1921),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1921),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1921),
    [sym_duet_dev_type_dss] = ACTIONS(1921),
    [sym_duet_dev_type_dvr] = ACTIONS(1921),
    [sym_duet_dev_type_disc_device] = ACTIONS(1921),
    [sym_duet_dev_type_document_camera] = ACTIONS(1921),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1921),
    [sym_duet_dev_type_hvac] = ACTIONS(1921),
    [sym_duet_dev_type_keypad] = ACTIONS(1921),
    [sym_duet_dev_type_light] = ACTIONS(1921),
    [sym_duet_dev_type_monitor] = ACTIONS(1921),
    [sym_duet_dev_type_motor] = ACTIONS(1921),
    [sym_duet_dev_type_multi_window] = ACTIONS(1921),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1921),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1921),
    [sym_duet_dev_type_receiver] = ACTIONS(1921),
    [sym_duet_dev_type_security_system] = ACTIONS(1921),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1921),
    [sym_duet_dev_type_settop_box] = ACTIONS(1921),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1921),
    [sym_duet_dev_type_switcher] = ACTIONS(1921),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1921),
    [sym_duet_dev_type_tv] = ACTIONS(1921),
    [sym_duet_dev_type_utility] = ACTIONS(1921),
    [sym_duet_dev_type_vcr] = ACTIONS(1921),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1921),
    [sym_duet_dev_type_video_processor] = ACTIONS(1921),
    [sym_duet_dev_type_video_projector] = ACTIONS(1921),
    [sym_duet_dev_type_video_wall] = ACTIONS(1921),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1921),
    [sym_duet_dev_type_weather] = ACTIONS(1921),
    [sym_duet_dev_not_polled] = ACTIONS(1921),
    [sym_duet_dev_polled] = ACTIONS(1921),
    [sym_clkmgr_mode_network] = ACTIONS(1921),
    [sym_clkmgr_mode_standalone] = ACTIONS(1921),
    [sym_normal_standby] = ACTIONS(1921),
    [sym_normal_wake] = ACTIONS(1921),
    [sym_ip_multicast_ttl_option] = ACTIONS(1921),
    [sym_ip_ttl_subnet] = ACTIONS(1921),
    [sym_ip_ttl_site] = ACTIONS(1921),
    [sym_ip_ttl_region] = ACTIONS(1921),
    [sym_ip_ttl_continent] = ACTIONS(1921),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1921),
    [sym_ip_nodelay_on] = ACTIONS(1921),
    [sym_ip_nodelay_off] = ACTIONS(1921),
    [sym_amx_error] = ACTIONS(1921),
    [sym_amx_warning] = ACTIONS(1921),
    [sym_amx_info] = ACTIONS(1921),
    [sym_amx_debug] = ACTIONS(1921),
    [sym_netlinx_login_success] = ACTIONS(1921),
    [sym_netlinx_login_fail] = ACTIONS(1921),
    [sym_netlinx_logout] = ACTIONS(1921),
    [sym_null_str] = ACTIONS(1921),
    [sym_smtp_address] = ACTIONS(1921),
    [sym_smtp_port_number] = ACTIONS(1921),
    [sym_smtp_username] = ACTIONS(1921),
    [sym_smtp_password] = ACTIONS(1921),
    [sym_smtp_require_tls] = ACTIONS(1921),
    [sym_smtp_from] = ACTIONS(1921),
    [sym_smtp_tls_true] = ACTIONS(1921),
    [sym_smtp_tls_false] = ACTIONS(1921),
    [sym_listview_on_row_select_event] = ACTIONS(1921),
    [sym_data_structure_unknown] = ACTIONS(1921),
    [sym_data_structure_datafeed] = ACTIONS(1921),
    [sym_data_structure_datarecordset] = ACTIONS(1921),
    [sym_data_structure_datarecord] = ACTIONS(1921),
    [sym_data_structure_datafield] = ACTIONS(1921),
    [sym_data_max_content_field_count] = ACTIONS(1921),
    [sym_data_max_metadata_field_count] = ACTIONS(1921),
    [sym_data_max_id_length] = ACTIONS(1921),
    [sym_data_max_name_length] = ACTIONS(1921),
    [sym_data_max_description_length] = ACTIONS(1921),
    [sym_data_max_type_length] = ACTIONS(1921),
    [sym_data_max_format_length] = ACTIONS(1921),
    [sym_data_max_source_length] = ACTIONS(1921),
    [sym_data_max_value_length] = ACTIONS(1921),
    [sym_data_max_label_length] = ACTIONS(1921),
    [sym_data_max_url_length] = ACTIONS(1921),
    [sym_max_last_login_info_length] = ACTIONS(1921),
    [sym_data_type_unknown] = ACTIONS(1921),
    [sym_data_type_string] = ACTIONS(1921),
    [sym_data_type_datetime] = ACTIONS(1921),
    [sym_data_type_date] = ACTIONS(1921),
    [sym_data_type_time] = ACTIONS(1921),
    [sym_data_type_image] = ACTIONS(1921),
    [sym_data_format_url] = ACTIONS(1921),
    [sym_data_format_phone] = ACTIONS(1921),
    [sym_data_format_email] = ACTIONS(1921),
    [sym_data_format_iso8601] = ACTIONS(1921),
    [sym_auditlog_priv] = ACTIONS(1921),
    [sym_configuration_priv] = ACTIONS(1921),
    [sym_deviceconfig_priv] = ACTIONS(1921),
    [sym_ftp_priv] = ACTIONS(1921),
    [sym_http_priv] = ACTIONS(1921),
    [sym_networkconfig_priv] = ACTIONS(1921),
    [sym_programport_priv] = ACTIONS(1921),
    [sym_remoteui_priv] = ACTIONS(1921),
    [sym_securitycontrol_priv] = ACTIONS(1921),
    [sym_softwaremanagement_priv] = ACTIONS(1921),
    [sym_terminal_priv] = ACTIONS(1921),
    [sym_tpadmin_priv] = ACTIONS(1921),
    [sym_user1_priv] = ACTIONS(1921),
    [sym_user2_priv] = ACTIONS(1921),
    [sym_user3_priv] = ACTIONS(1921),
    [sym_user4_priv] = ACTIONS(1921),
    [sym_usermanagement_priv] = ACTIONS(1921),
    [sym_valid_account] = ACTIONS(1921),
    [sym_err_name_invalid] = ACTIONS(1921),
    [sym_err_password_invalid] = ACTIONS(1921),
    [sym_err_invalid_account] = ACTIONS(1921),
    [sym_err_no_authorization] = ACTIONS(1921),
    [sym_err_invalid_parameter] = ACTIONS(1921),
    [sym_err_user_locked_out] = ACTIONS(1921),
    [sym_err_user_account_expired] = ACTIONS(1921),
    [sym_err_general_error] = ACTIONS(1921),
    [sym_err_authentication_error] = ACTIONS(1921),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1921),
    [sym_tls_validate_certificate] = ACTIONS(1921),
    [sym_tdata] = ACTIONS(1921),
    [sym_tchannel] = ACTIONS(1921),
    [sym_tlevel] = ACTIONS(1921),
    [sym_tbutton] = ACTIONS(1921),
    [sym_ttimeline] = ACTIONS(1921),
    [sym_tcustom] = ACTIONS(1921),
    [sym_url_struct] = ACTIONS(1921),
    [sym_dns_struct] = ACTIONS(1921),
    [sym_ip_address_struct] = ACTIONS(1921),
    [sym_dev_info_struct] = ACTIONS(1921),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1921),
    [sym_data_feed] = ACTIONS(1921),
    [sym_data_field] = ACTIONS(1921),
    [sym_data_record] = ACTIONS(1921),
    [sym_wc_data_feed] = ACTIONS(1921),
    [sym_wc_data_field] = ACTIONS(1921),
    [sym_wc_data_record] = ACTIONS(1921),
    [sym_last_login_info] = ACTIONS(1921),
    [sym____reserved___] = ACTIONS(1921),
    [sym_date] = ACTIONS(1921),
    [sym_day] = ACTIONS(1921),
    [sym_time] = ACTIONS(1921),
    [sym_ldate] = ACTIONS(1921),
    [sym_push_device] = ACTIONS(1921),
    [sym_push_channel] = ACTIONS(1921),
    [sym_push_devchan] = ACTIONS(1921),
    [sym_release_device] = ACTIONS(1921),
    [sym_release_channel] = ACTIONS(1921),
    [sym_release_devchan] = ACTIONS(1921),
    [sym_master_slot] = ACTIONS(1921),
    [sym_get_pulse_time] = ACTIONS(1921),
    [sym_get_timer] = ACTIONS(1921),
    [sym_master_sn] = ACTIONS(1921),
    [sym_system_number] = ACTIONS(1921),
    [sym_button] = ACTIONS(1921),
    [sym_dv_channel] = ACTIONS(1921),
    [sym_data] = ACTIONS(1921),
    [sym_level] = ACTIONS(1921),
    [sym_timeline] = ACTIONS(1921),
    [sym_channel] = ACTIONS(1921),
    [sym_custom] = ACTIONS(1921),
    [sym_atoi] = ACTIONS(1921),
    [sym_atol] = ACTIONS(1921),
    [sym_atof] = ACTIONS(1921),
    [sym_itoa] = ACTIONS(1921),
    [sym_itohex] = ACTIONS(1921),
    [sym_hextoi] = ACTIONS(1921),
    [sym_ftoa] = ACTIONS(1921),
    [sym_find_string] = ACTIONS(1921),
    [sym_left_string] = ACTIONS(1921),
    [sym_right_string] = ACTIONS(1921),
    [sym_mid_string] = ACTIONS(1921),
    [sym_remove_string] = ACTIONS(1921),
    [sym_get_buffer_string] = ACTIONS(1921),
    [sym_get_buffer_char] = ACTIONS(1921),
    [sym_get_multi_buffer_string] = ACTIONS(1921),
    [sym_length_string] = ACTIONS(1921),
    [sym_max_length_string] = ACTIONS(1921),
    [sym_set_length_string] = ACTIONS(1921),
    [sym_compare_string] = ACTIONS(1921),
    [sym_upper_string] = ACTIONS(1921),
    [sym_lower_string] = ACTIONS(1921),
    [sym_get_last] = ACTIONS(1921),
    [sym_date_to_day] = ACTIONS(1921),
    [sym_date_to_month] = ACTIONS(1921),
    [sym_date_to_year] = ACTIONS(1921),
    [sym_time_to_hour] = ACTIONS(1921),
    [sym_time_to_minute] = ACTIONS(1921),
    [sym_time_to_second] = ACTIONS(1921),
    [sym_day_of_week] = ACTIONS(1921),
    [sym_set_pulse_time] = ACTIONS(1921),
    [sym_set_timer] = ACTIONS(1921),
    [sym_file_open] = ACTIONS(1921),
    [sym_file_close] = ACTIONS(1921),
    [sym_file_read] = ACTIONS(1921),
    [sym_file_read_line] = ACTIONS(1921),
    [sym_file_write] = ACTIONS(1921),
    [sym_file_write_line] = ACTIONS(1921),
    [sym_file_copy] = ACTIONS(1921),
    [sym_file_delete] = ACTIONS(1921),
    [sym_file_rename] = ACTIONS(1921),
    [sym_file_dir] = ACTIONS(1921),
    [sym_file_getdir] = ACTIONS(1921),
    [sym_file_setdir] = ACTIONS(1921),
    [sym_file_create_dir] = ACTIONS(1921),
    [sym_file_remove_dir] = ACTIONS(1921),
    [sym_file_seek] = ACTIONS(1921),
    [sym_length_array] = ACTIONS(1921),
    [sym_max_length_array] = ACTIONS(1921),
    [sym_set_length_array] = ACTIONS(1921),
    [sym_ip_client_open] = ACTIONS(1921),
    [sym_ip_client_close] = ACTIONS(1921),
    [sym_ip_server_open] = ACTIONS(1921),
    [sym_ip_server_close] = ACTIONS(1921),
    [sym_ip_bound_client_open] = ACTIONS(1921),
    [sym_ip_mc_server_open] = ACTIONS(1921),
    [sym_ip_set_option] = ACTIONS(1921),
    [sym_ssh_client_open] = ACTIONS(1921),
    [sym_ssh_client_close] = ACTIONS(1921),
    [sym_tls_client_open] = ACTIONS(1921),
    [sym_tls_client_close] = ACTIONS(1921),
    [sym_do_push] = ACTIONS(1921),
    [sym_do_push_timed] = ACTIONS(1921),
    [sym_do_release] = ACTIONS(1921),
    [sym_device_id] = ACTIONS(1921),
    [sym_device_info] = ACTIONS(1921),
    [sym_device_id_string] = ACTIONS(1921),
    [sym_device_standby] = ACTIONS(1921),
    [sym_device_wake] = ACTIONS(1921),
    [sym_timeline_create] = ACTIONS(1921),
    [sym_timeline_kill] = ACTIONS(1921),
    [sym_timeline_reload] = ACTIONS(1921),
    [sym_timeline_pause] = ACTIONS(1921),
    [sym_timeline_restart] = ACTIONS(1921),
    [sym_timeline_set] = ACTIONS(1921),
    [sym_timeline_get] = ACTIONS(1921),
    [sym_timeline_active] = ACTIONS(1921),
    [sym_timeline_dynamic_id] = ACTIONS(1921),
    [sym_variable_to_string] = ACTIONS(1921),
    [sym_string_to_variable] = ACTIONS(1921),
    [sym_length_variable_to_string] = ACTIONS(1921),
    [sym_type_cast] = ACTIONS(1921),
    [sym_variable_to_xml] = ACTIONS(1921),
    [sym_xml_to_variable] = ACTIONS(1921),
    [sym_length_variable_to_xml] = ACTIONS(1921),
    [sym_redirect_string] = ACTIONS(1921),
    [sym_random_number] = ACTIONS(1921),
    [sym_get_unique_id] = ACTIONS(1921),
    [sym_get_system_number] = ACTIONS(1921),
    [sym_set_system_number] = ACTIONS(1921),
    [sym_get_serial_number] = ACTIONS(1921),
    [sym_get_master_build] = ACTIONS(1921),
    [sym_set_validation_code] = ACTIONS(1921),
    [sym_reboot] = ACTIONS(1921),
    [sym_load_duet_module] = ACTIONS(1921),
    [sym_unload_duet_module] = ACTIONS(1921),
    [sym_duet_mem_size_set] = ACTIONS(1921),
    [sym_duet_mem_size_get] = ACTIONS(1921),
    [sym_dynamic_application_device] = ACTIONS(1921),
    [sym_dynamic_polled_port] = ACTIONS(1921),
    [sym_static_port_binding] = ACTIONS(1921),
    [sym_static_ip_binding] = ACTIONS(1921),
    [sym_get_dns_list] = ACTIONS(1921),
    [sym_set_dns_list] = ACTIONS(1921),
    [sym_get_ip_address] = ACTIONS(1921),
    [sym_set_ip_address] = ACTIONS(1921),
    [sym_get_url_list] = ACTIONS(1921),
    [sym_add_url_entry] = ACTIONS(1921),
    [sym_delete_url_entry] = ACTIONS(1921),
    [sym_abs_value] = ACTIONS(1921),
    [sym_max_value] = ACTIONS(1921),
    [sym_min_value] = ACTIONS(1921),
    [sym_exp_value] = ACTIONS(1921),
    [sym_log_value] = ACTIONS(1921),
    [sym_log10_value] = ACTIONS(1921),
    [sym_power_value] = ACTIONS(1921),
    [sym_sqrt_value] = ACTIONS(1921),
    [sym_raw_be] = ACTIONS(1921),
    [sym_raw_le] = ACTIONS(1921),
    [sym_internal_queue_size_set] = ACTIONS(1921),
    [sym_internal_queue_size_get] = ACTIONS(1921),
    [sym_internal_threshold_set] = ACTIONS(1921),
    [sym_internal_threshold_get] = ACTIONS(1921),
    [sym_rebuilt_event] = ACTIONS(1921),
    [sym_do_custom_event] = ACTIONS(1921),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1921),
    [sym_clkmgr_set_clk_source] = ACTIONS(1921),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1921),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1921),
    [sym_clkmgr_get_timezone] = ACTIONS(1921),
    [sym_clkmgr_set_timezone] = ACTIONS(1921),
    [sym_clkmgr_get_resync_period] = ACTIONS(1921),
    [sym_clkmgr_set_resync_period] = ACTIONS(1921),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1921),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1921),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1921),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1921),
    [sym_clkmgr_get_timeservers] = ACTIONS(1921),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1921),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1921),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1921),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1921),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1921),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1921),
    [sym_set_log_level] = ACTIONS(1921),
    [sym_get_log_level] = ACTIONS(1921),
    [sym_amx_log] = ACTIONS(1921),
    [sym_smtp_server_config_set] = ACTIONS(1921),
    [sym_smtp_server_config_get] = ACTIONS(1921),
    [sym_smtp_send] = ACTIONS(1921),
    [sym_validate_netlinx_account] = ACTIONS(1921),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1921),
    [sym_authenticate_certificate] = ACTIONS(1921),
    [sym_audit_netlinx_session_event] = ACTIONS(1921),
    [sym_audit_netlinx_generic_event] = ACTIONS(1921),
    [sym_get_available_flash_disk_space] = ACTIONS(1921),
    [sym_get_max_flash_disk_space] = ACTIONS(1921),
    [sym_data_create_feed] = ACTIONS(1921),
    [sym_data_delete_feed] = ACTIONS(1921),
    [sym_data_publish_feed] = ACTIONS(1921),
    [sym_data_get_published_feed] = ACTIONS(1921),
    [sym_data_add_record] = ACTIONS(1921),
    [sym_data_get_event_record] = ACTIONS(1921),
    [sym__wc_data_create_feed] = ACTIONS(1921),
    [sym__wc_data_add_record] = ACTIONS(1921),
    [sym__wc_data_get_event_record] = ACTIONS(1921),
    [sym_format] = ACTIONS(1921),
    [sym_astro_clock] = ACTIONS(1921),
    [sym_set_outdoor_temperature] = ACTIONS(1921),
    [sym_set_virtual_level_count] = ACTIONS(1921),
    [sym_set_virtual_channel_count] = ACTIONS(1921),
    [sym_set_virtual_port_count] = ACTIONS(1921),
    [sym__wc_encode_int] = ACTIONS(1921),
    [sym__wc_decode_int] = ACTIONS(1921),
    [sym__wc_int] = ACTIONS(1921),
    [sym__wc_to_ch_int] = ACTIONS(1921),
    [sym__ch_to_wc_int] = ACTIONS(1921),
    [sym__wc_find_string_int] = ACTIONS(1921),
    [sym__wc_left_string_int] = ACTIONS(1921),
    [sym__wc_lower_string_int] = ACTIONS(1921),
    [sym__wc_mid_string_int] = ACTIONS(1921),
    [sym__wc_remove_string_int] = ACTIONS(1921),
    [sym__wc_right_string_int] = ACTIONS(1921),
    [sym__wc_upper_string_int] = ACTIONS(1921),
    [sym__wc_compare_string_int] = ACTIONS(1921),
    [sym__wc_get_buffer_char_int] = ACTIONS(1921),
    [sym__wc_get_buffer_string_int] = ACTIONS(1921),
    [sym__wc_concat_string_int] = ACTIONS(1921),
    [sym__wc_file_open_int] = ACTIONS(1921),
    [sym__wc_file_close_int] = ACTIONS(1921),
    [sym__wc_file_read_int] = ACTIONS(1921),
    [sym__wc_file_read_line_int] = ACTIONS(1921),
    [sym__wc_file_write_int] = ACTIONS(1921),
    [sym__wc_file_write_line_int] = ACTIONS(1921),
    [aux_sym_preproc_include_token1] = ACTIONS(1921),
    [aux_sym_preproc_define_token1] = ACTIONS(1923),
    [aux_sym_preproc_warn_token1] = ACTIONS(1923),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1923),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1923),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1923),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1923),
    [aux_sym_preproc_else_token1] = ACTIONS(1923),
    [anon_sym_LBRACK] = ACTIONS(1923),
    [anon_sym_LBRACE] = ACTIONS(1923),
    [anon_sym_LPAREN] = ACTIONS(1921),
    [anon_sym_BANG] = ACTIONS(1923),
    [anon_sym_TILDE] = ACTIONS(1923),
    [anon_sym_DASH] = ACTIONS(1921),
    [anon_sym_PLUS] = ACTIONS(1921),
    [anon_sym_DASH_DASH] = ACTIONS(1923),
    [anon_sym_PLUS_PLUS] = ACTIONS(1923),
    [anon_sym_DQUOTE] = ACTIONS(1923),
    [anon_sym_SQUOTE] = ACTIONS(1923),
    [sym_number_literal] = ACTIONS(1923),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(281)] = {
    [sym_identifier] = ACTIONS(1925),
    [sym_program_name_keyword] = ACTIONS(1925),
    [sym_module_name_keyword] = ACTIONS(1925),
    [sym_define_device_keyword] = ACTIONS(1925),
    [sym_define_combine_keyword] = ACTIONS(1925),
    [sym_define_constant_keyword] = ACTIONS(1925),
    [sym_define_type_keyword] = ACTIONS(1925),
    [sym_define_variable_keyword] = ACTIONS(1925),
    [sym_define_system_variable_keyword] = ACTIONS(1925),
    [sym_define_connect_level_keyword] = ACTIONS(1925),
    [sym_define_latching_keyword] = ACTIONS(1925),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1925),
    [sym_define_toggling_keyword] = ACTIONS(1925),
    [sym_define_call_keyword] = ACTIONS(1925),
    [sym_define_function_keyword] = ACTIONS(1925),
    [sym_define_library_function_keyword] = ACTIONS(1925),
    [sym_define_start_keyword] = ACTIONS(1925),
    [sym_define_event_keyword] = ACTIONS(1925),
    [sym_define_program_keyword] = ACTIONS(1925),
    [sym_define_module_keyword] = ACTIONS(1925),
    [sym_char_keyword] = ACTIONS(1925),
    [sym_widechar_keyword] = ACTIONS(1925),
    [sym_integer_keyword] = ACTIONS(1925),
    [sym_sinteger_keyword] = ACTIONS(1925),
    [sym_long_keyword] = ACTIONS(1925),
    [sym_slong_keyword] = ACTIONS(1925),
    [sym_float_keyword] = ACTIONS(1925),
    [sym_double_keyword] = ACTIONS(1925),
    [sym_variant_keyword] = ACTIONS(1925),
    [sym_variantarray_keyword] = ACTIONS(1925),
    [sym_local_var_keyword] = ACTIONS(1925),
    [sym_stack_var_keyword] = ACTIONS(1925),
    [sym_constant_keyword] = ACTIONS(1925),
    [sym_volatile_keyword] = ACTIONS(1925),
    [sym_non_volatile_keyword] = ACTIONS(1925),
    [sym_persistent_keyword] = ACTIONS(1925),
    [sym_dev_keyword] = ACTIONS(1925),
    [sym_devlev_keyword] = ACTIONS(1925),
    [sym_devchan_keyword] = ACTIONS(1925),
    [sym_if_keyword] = ACTIONS(1925),
    [sym_switch_keyword] = ACTIONS(1925),
    [sym_case_keyword] = ACTIONS(1925),
    [sym_default_keyword] = ACTIONS(1925),
    [sym_while_keyword] = ACTIONS(1925),
    [sym_for_keyword] = ACTIONS(1925),
    [sym_break_keyword] = ACTIONS(1925),
    [sym_continue_keyword] = ACTIONS(1925),
    [sym_return_keyword] = ACTIONS(1925),
    [sym_select_keyword] = ACTIONS(1925),
    [sym_active_keyword] = ACTIONS(1925),
    [sym_struct_keyword] = ACTIONS(1925),
    [sym_structure_keyword] = ACTIONS(1925),
    [sym_button_event_keyword] = ACTIONS(1925),
    [sym_channel_event_keyword] = ACTIONS(1925),
    [sym_level_event_keyword] = ACTIONS(1925),
    [sym_data_event_keyword] = ACTIONS(1925),
    [sym_timeline_event_keyword] = ACTIONS(1925),
    [sym_custom_event_keyword] = ACTIONS(1925),
    [sym_push_keyword] = ACTIONS(1925),
    [sym_release_keyword] = ACTIONS(1925),
    [aux_sym_on_keyword_token1] = ACTIONS(1925),
    [aux_sym_off_keyword_token1] = ACTIONS(1925),
    [sym_send_level_keyword] = ACTIONS(1925),
    [sym_send_string_keyword] = ACTIONS(1925),
    [sym_send_command_keyword] = ACTIONS(1925),
    [sym_clear_buffer_keyword] = ACTIONS(1925),
    [sym_create_buffer_keyword] = ACTIONS(1925),
    [sym_create_multi_buffer_keyword] = ACTIONS(1925),
    [sym_call_keyword] = ACTIONS(1925),
    [sym_system_call_keyword] = ACTIONS(1925),
    [sym_devchan_to_keyword] = ACTIONS(1925),
    [sym_devchan_min_to_keyword] = ACTIONS(1925),
    [sym_devchan_total_off_keyword] = ACTIONS(1925),
    [sym_devchan_pulse_keyword] = ACTIONS(1925),
    [sym_wait_keyword] = ACTIONS(1925),
    [sym_cancel_wait_keyword] = ACTIONS(1925),
    [sym_wait_until_keyword] = ACTIONS(1925),
    [sym_cancel_wait_until_keyword] = ACTIONS(1925),
    [sym_cancel_all_wait_keyword] = ACTIONS(1925),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1925),
    [sym_not] = ACTIONS(1925),
    [sym_bnot] = ACTIONS(1925),
    [sym___file__] = ACTIONS(1925),
    [sym___date__] = ACTIONS(1925),
    [sym___ldate__] = ACTIONS(1925),
    [sym___time__] = ACTIONS(1925),
    [sym___line__] = ACTIONS(1925),
    [sym___name__] = ACTIONS(1925),
    [sym___netlinx__] = ACTIONS(1925),
    [sym_netlinx_axi_version] = ACTIONS(1925),
    [sym_url_flg_tcp] = ACTIONS(1925),
    [sym_url_flg_accinfopresent] = ACTIONS(1925),
    [sym_url_flg_temp] = ACTIONS(1925),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1925),
    [sym_url_flg_stat_mask] = ACTIONS(1925),
    [sym_url_flg_stat_lookup] = ACTIONS(1925),
    [sym_url_flg_stat_connecting] = ACTIONS(1925),
    [sym_url_flg_stat_waiting] = ACTIONS(1925),
    [sym_url_flg_stat_connected] = ACTIONS(1925),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1925),
    [sym_first_virtual_device] = ACTIONS(1925),
    [sym_first_local_port] = ACTIONS(1925),
    [sym_dynamic_virtual_device] = ACTIONS(1925),
    [sym_timeline_once] = ACTIONS(1925),
    [sym_timeline_repeat] = ACTIONS(1925),
    [sym_timeline_absolute] = ACTIONS(1925),
    [sym_timeline_relative] = ACTIONS(1925),
    [sym_do_push_timed_infinite] = ACTIONS(1925),
    [sym_source_type_no_address] = ACTIONS(1925),
    [sym_source_type_neuron_id] = ACTIONS(1925),
    [sym_source_type_ip_address] = ACTIONS(1925),
    [sym_source_type_axlink] = ACTIONS(1925),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1925),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1925),
    [sym_source_type_ip_socket_address] = ACTIONS(1925),
    [sym_source_type_rs232] = ACTIONS(1925),
    [sym_source_type_internal] = ACTIONS(1925),
    [sym_source_type_mac_address] = ACTIONS(1925),
    [sym_source_type_ipv4_port] = ACTIONS(1925),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1925),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1925),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1925),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1925),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1925),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1925),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1925),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1925),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1925),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1925),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1925),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1925),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1925),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1925),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1925),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1925),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1925),
    [sym_true] = ACTIONS(1925),
    [sym_false] = ACTIONS(1925),
    [sym_file_read_only] = ACTIONS(1925),
    [sym_file_rw_new] = ACTIONS(1925),
    [sym_file_rw_append] = ACTIONS(1925),
    [sym_ip_tcp] = ACTIONS(1925),
    [sym_ip_udp] = ACTIONS(1925),
    [sym_ip_udp_2way] = ACTIONS(1925),
    [sym_xml_encode_types] = ACTIONS(1925),
    [sym_xml_encode_char_as_list] = ACTIONS(1925),
    [sym_xml_encode_le] = ACTIONS(1925),
    [sym_xml_decode_types] = ACTIONS(1925),
    [sym_xml_decode_no_preserve] = ACTIONS(1925),
    [sym_duet_dev_type_display_device] = ACTIONS(1925),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1925),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1925),
    [sym_duet_dev_type_io_device] = ACTIONS(1925),
    [sym_duet_dev_type_relay_device] = ACTIONS(1925),
    [sym_duet_dev_type_ups] = ACTIONS(1925),
    [sym_duet_dev_type_amplifier] = ACTIONS(1925),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1925),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1925),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1925),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1925),
    [sym_duet_dev_type_camera] = ACTIONS(1925),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1925),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1925),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1925),
    [sym_duet_dev_type_dss] = ACTIONS(1925),
    [sym_duet_dev_type_dvr] = ACTIONS(1925),
    [sym_duet_dev_type_disc_device] = ACTIONS(1925),
    [sym_duet_dev_type_document_camera] = ACTIONS(1925),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1925),
    [sym_duet_dev_type_hvac] = ACTIONS(1925),
    [sym_duet_dev_type_keypad] = ACTIONS(1925),
    [sym_duet_dev_type_light] = ACTIONS(1925),
    [sym_duet_dev_type_monitor] = ACTIONS(1925),
    [sym_duet_dev_type_motor] = ACTIONS(1925),
    [sym_duet_dev_type_multi_window] = ACTIONS(1925),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1925),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1925),
    [sym_duet_dev_type_receiver] = ACTIONS(1925),
    [sym_duet_dev_type_security_system] = ACTIONS(1925),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1925),
    [sym_duet_dev_type_settop_box] = ACTIONS(1925),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1925),
    [sym_duet_dev_type_switcher] = ACTIONS(1925),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1925),
    [sym_duet_dev_type_tv] = ACTIONS(1925),
    [sym_duet_dev_type_utility] = ACTIONS(1925),
    [sym_duet_dev_type_vcr] = ACTIONS(1925),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1925),
    [sym_duet_dev_type_video_processor] = ACTIONS(1925),
    [sym_duet_dev_type_video_projector] = ACTIONS(1925),
    [sym_duet_dev_type_video_wall] = ACTIONS(1925),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1925),
    [sym_duet_dev_type_weather] = ACTIONS(1925),
    [sym_duet_dev_not_polled] = ACTIONS(1925),
    [sym_duet_dev_polled] = ACTIONS(1925),
    [sym_clkmgr_mode_network] = ACTIONS(1925),
    [sym_clkmgr_mode_standalone] = ACTIONS(1925),
    [sym_normal_standby] = ACTIONS(1925),
    [sym_normal_wake] = ACTIONS(1925),
    [sym_ip_multicast_ttl_option] = ACTIONS(1925),
    [sym_ip_ttl_subnet] = ACTIONS(1925),
    [sym_ip_ttl_site] = ACTIONS(1925),
    [sym_ip_ttl_region] = ACTIONS(1925),
    [sym_ip_ttl_continent] = ACTIONS(1925),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1925),
    [sym_ip_nodelay_on] = ACTIONS(1925),
    [sym_ip_nodelay_off] = ACTIONS(1925),
    [sym_amx_error] = ACTIONS(1925),
    [sym_amx_warning] = ACTIONS(1925),
    [sym_amx_info] = ACTIONS(1925),
    [sym_amx_debug] = ACTIONS(1925),
    [sym_netlinx_login_success] = ACTIONS(1925),
    [sym_netlinx_login_fail] = ACTIONS(1925),
    [sym_netlinx_logout] = ACTIONS(1925),
    [sym_null_str] = ACTIONS(1925),
    [sym_smtp_address] = ACTIONS(1925),
    [sym_smtp_port_number] = ACTIONS(1925),
    [sym_smtp_username] = ACTIONS(1925),
    [sym_smtp_password] = ACTIONS(1925),
    [sym_smtp_require_tls] = ACTIONS(1925),
    [sym_smtp_from] = ACTIONS(1925),
    [sym_smtp_tls_true] = ACTIONS(1925),
    [sym_smtp_tls_false] = ACTIONS(1925),
    [sym_listview_on_row_select_event] = ACTIONS(1925),
    [sym_data_structure_unknown] = ACTIONS(1925),
    [sym_data_structure_datafeed] = ACTIONS(1925),
    [sym_data_structure_datarecordset] = ACTIONS(1925),
    [sym_data_structure_datarecord] = ACTIONS(1925),
    [sym_data_structure_datafield] = ACTIONS(1925),
    [sym_data_max_content_field_count] = ACTIONS(1925),
    [sym_data_max_metadata_field_count] = ACTIONS(1925),
    [sym_data_max_id_length] = ACTIONS(1925),
    [sym_data_max_name_length] = ACTIONS(1925),
    [sym_data_max_description_length] = ACTIONS(1925),
    [sym_data_max_type_length] = ACTIONS(1925),
    [sym_data_max_format_length] = ACTIONS(1925),
    [sym_data_max_source_length] = ACTIONS(1925),
    [sym_data_max_value_length] = ACTIONS(1925),
    [sym_data_max_label_length] = ACTIONS(1925),
    [sym_data_max_url_length] = ACTIONS(1925),
    [sym_max_last_login_info_length] = ACTIONS(1925),
    [sym_data_type_unknown] = ACTIONS(1925),
    [sym_data_type_string] = ACTIONS(1925),
    [sym_data_type_datetime] = ACTIONS(1925),
    [sym_data_type_date] = ACTIONS(1925),
    [sym_data_type_time] = ACTIONS(1925),
    [sym_data_type_image] = ACTIONS(1925),
    [sym_data_format_url] = ACTIONS(1925),
    [sym_data_format_phone] = ACTIONS(1925),
    [sym_data_format_email] = ACTIONS(1925),
    [sym_data_format_iso8601] = ACTIONS(1925),
    [sym_auditlog_priv] = ACTIONS(1925),
    [sym_configuration_priv] = ACTIONS(1925),
    [sym_deviceconfig_priv] = ACTIONS(1925),
    [sym_ftp_priv] = ACTIONS(1925),
    [sym_http_priv] = ACTIONS(1925),
    [sym_networkconfig_priv] = ACTIONS(1925),
    [sym_programport_priv] = ACTIONS(1925),
    [sym_remoteui_priv] = ACTIONS(1925),
    [sym_securitycontrol_priv] = ACTIONS(1925),
    [sym_softwaremanagement_priv] = ACTIONS(1925),
    [sym_terminal_priv] = ACTIONS(1925),
    [sym_tpadmin_priv] = ACTIONS(1925),
    [sym_user1_priv] = ACTIONS(1925),
    [sym_user2_priv] = ACTIONS(1925),
    [sym_user3_priv] = ACTIONS(1925),
    [sym_user4_priv] = ACTIONS(1925),
    [sym_usermanagement_priv] = ACTIONS(1925),
    [sym_valid_account] = ACTIONS(1925),
    [sym_err_name_invalid] = ACTIONS(1925),
    [sym_err_password_invalid] = ACTIONS(1925),
    [sym_err_invalid_account] = ACTIONS(1925),
    [sym_err_no_authorization] = ACTIONS(1925),
    [sym_err_invalid_parameter] = ACTIONS(1925),
    [sym_err_user_locked_out] = ACTIONS(1925),
    [sym_err_user_account_expired] = ACTIONS(1925),
    [sym_err_general_error] = ACTIONS(1925),
    [sym_err_authentication_error] = ACTIONS(1925),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1925),
    [sym_tls_validate_certificate] = ACTIONS(1925),
    [sym_tdata] = ACTIONS(1925),
    [sym_tchannel] = ACTIONS(1925),
    [sym_tlevel] = ACTIONS(1925),
    [sym_tbutton] = ACTIONS(1925),
    [sym_ttimeline] = ACTIONS(1925),
    [sym_tcustom] = ACTIONS(1925),
    [sym_url_struct] = ACTIONS(1925),
    [sym_dns_struct] = ACTIONS(1925),
    [sym_ip_address_struct] = ACTIONS(1925),
    [sym_dev_info_struct] = ACTIONS(1925),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1925),
    [sym_data_feed] = ACTIONS(1925),
    [sym_data_field] = ACTIONS(1925),
    [sym_data_record] = ACTIONS(1925),
    [sym_wc_data_feed] = ACTIONS(1925),
    [sym_wc_data_field] = ACTIONS(1925),
    [sym_wc_data_record] = ACTIONS(1925),
    [sym_last_login_info] = ACTIONS(1925),
    [sym____reserved___] = ACTIONS(1925),
    [sym_date] = ACTIONS(1925),
    [sym_day] = ACTIONS(1925),
    [sym_time] = ACTIONS(1925),
    [sym_ldate] = ACTIONS(1925),
    [sym_push_device] = ACTIONS(1925),
    [sym_push_channel] = ACTIONS(1925),
    [sym_push_devchan] = ACTIONS(1925),
    [sym_release_device] = ACTIONS(1925),
    [sym_release_channel] = ACTIONS(1925),
    [sym_release_devchan] = ACTIONS(1925),
    [sym_master_slot] = ACTIONS(1925),
    [sym_get_pulse_time] = ACTIONS(1925),
    [sym_get_timer] = ACTIONS(1925),
    [sym_master_sn] = ACTIONS(1925),
    [sym_system_number] = ACTIONS(1925),
    [sym_button] = ACTIONS(1925),
    [sym_dv_channel] = ACTIONS(1925),
    [sym_data] = ACTIONS(1925),
    [sym_level] = ACTIONS(1925),
    [sym_timeline] = ACTIONS(1925),
    [sym_channel] = ACTIONS(1925),
    [sym_custom] = ACTIONS(1925),
    [sym_atoi] = ACTIONS(1925),
    [sym_atol] = ACTIONS(1925),
    [sym_atof] = ACTIONS(1925),
    [sym_itoa] = ACTIONS(1925),
    [sym_itohex] = ACTIONS(1925),
    [sym_hextoi] = ACTIONS(1925),
    [sym_ftoa] = ACTIONS(1925),
    [sym_find_string] = ACTIONS(1925),
    [sym_left_string] = ACTIONS(1925),
    [sym_right_string] = ACTIONS(1925),
    [sym_mid_string] = ACTIONS(1925),
    [sym_remove_string] = ACTIONS(1925),
    [sym_get_buffer_string] = ACTIONS(1925),
    [sym_get_buffer_char] = ACTIONS(1925),
    [sym_get_multi_buffer_string] = ACTIONS(1925),
    [sym_length_string] = ACTIONS(1925),
    [sym_max_length_string] = ACTIONS(1925),
    [sym_set_length_string] = ACTIONS(1925),
    [sym_compare_string] = ACTIONS(1925),
    [sym_upper_string] = ACTIONS(1925),
    [sym_lower_string] = ACTIONS(1925),
    [sym_get_last] = ACTIONS(1925),
    [sym_date_to_day] = ACTIONS(1925),
    [sym_date_to_month] = ACTIONS(1925),
    [sym_date_to_year] = ACTIONS(1925),
    [sym_time_to_hour] = ACTIONS(1925),
    [sym_time_to_minute] = ACTIONS(1925),
    [sym_time_to_second] = ACTIONS(1925),
    [sym_day_of_week] = ACTIONS(1925),
    [sym_set_pulse_time] = ACTIONS(1925),
    [sym_set_timer] = ACTIONS(1925),
    [sym_file_open] = ACTIONS(1925),
    [sym_file_close] = ACTIONS(1925),
    [sym_file_read] = ACTIONS(1925),
    [sym_file_read_line] = ACTIONS(1925),
    [sym_file_write] = ACTIONS(1925),
    [sym_file_write_line] = ACTIONS(1925),
    [sym_file_copy] = ACTIONS(1925),
    [sym_file_delete] = ACTIONS(1925),
    [sym_file_rename] = ACTIONS(1925),
    [sym_file_dir] = ACTIONS(1925),
    [sym_file_getdir] = ACTIONS(1925),
    [sym_file_setdir] = ACTIONS(1925),
    [sym_file_create_dir] = ACTIONS(1925),
    [sym_file_remove_dir] = ACTIONS(1925),
    [sym_file_seek] = ACTIONS(1925),
    [sym_length_array] = ACTIONS(1925),
    [sym_max_length_array] = ACTIONS(1925),
    [sym_set_length_array] = ACTIONS(1925),
    [sym_ip_client_open] = ACTIONS(1925),
    [sym_ip_client_close] = ACTIONS(1925),
    [sym_ip_server_open] = ACTIONS(1925),
    [sym_ip_server_close] = ACTIONS(1925),
    [sym_ip_bound_client_open] = ACTIONS(1925),
    [sym_ip_mc_server_open] = ACTIONS(1925),
    [sym_ip_set_option] = ACTIONS(1925),
    [sym_ssh_client_open] = ACTIONS(1925),
    [sym_ssh_client_close] = ACTIONS(1925),
    [sym_tls_client_open] = ACTIONS(1925),
    [sym_tls_client_close] = ACTIONS(1925),
    [sym_do_push] = ACTIONS(1925),
    [sym_do_push_timed] = ACTIONS(1925),
    [sym_do_release] = ACTIONS(1925),
    [sym_device_id] = ACTIONS(1925),
    [sym_device_info] = ACTIONS(1925),
    [sym_device_id_string] = ACTIONS(1925),
    [sym_device_standby] = ACTIONS(1925),
    [sym_device_wake] = ACTIONS(1925),
    [sym_timeline_create] = ACTIONS(1925),
    [sym_timeline_kill] = ACTIONS(1925),
    [sym_timeline_reload] = ACTIONS(1925),
    [sym_timeline_pause] = ACTIONS(1925),
    [sym_timeline_restart] = ACTIONS(1925),
    [sym_timeline_set] = ACTIONS(1925),
    [sym_timeline_get] = ACTIONS(1925),
    [sym_timeline_active] = ACTIONS(1925),
    [sym_timeline_dynamic_id] = ACTIONS(1925),
    [sym_variable_to_string] = ACTIONS(1925),
    [sym_string_to_variable] = ACTIONS(1925),
    [sym_length_variable_to_string] = ACTIONS(1925),
    [sym_type_cast] = ACTIONS(1925),
    [sym_variable_to_xml] = ACTIONS(1925),
    [sym_xml_to_variable] = ACTIONS(1925),
    [sym_length_variable_to_xml] = ACTIONS(1925),
    [sym_redirect_string] = ACTIONS(1925),
    [sym_random_number] = ACTIONS(1925),
    [sym_get_unique_id] = ACTIONS(1925),
    [sym_get_system_number] = ACTIONS(1925),
    [sym_set_system_number] = ACTIONS(1925),
    [sym_get_serial_number] = ACTIONS(1925),
    [sym_get_master_build] = ACTIONS(1925),
    [sym_set_validation_code] = ACTIONS(1925),
    [sym_reboot] = ACTIONS(1925),
    [sym_load_duet_module] = ACTIONS(1925),
    [sym_unload_duet_module] = ACTIONS(1925),
    [sym_duet_mem_size_set] = ACTIONS(1925),
    [sym_duet_mem_size_get] = ACTIONS(1925),
    [sym_dynamic_application_device] = ACTIONS(1925),
    [sym_dynamic_polled_port] = ACTIONS(1925),
    [sym_static_port_binding] = ACTIONS(1925),
    [sym_static_ip_binding] = ACTIONS(1925),
    [sym_get_dns_list] = ACTIONS(1925),
    [sym_set_dns_list] = ACTIONS(1925),
    [sym_get_ip_address] = ACTIONS(1925),
    [sym_set_ip_address] = ACTIONS(1925),
    [sym_get_url_list] = ACTIONS(1925),
    [sym_add_url_entry] = ACTIONS(1925),
    [sym_delete_url_entry] = ACTIONS(1925),
    [sym_abs_value] = ACTIONS(1925),
    [sym_max_value] = ACTIONS(1925),
    [sym_min_value] = ACTIONS(1925),
    [sym_exp_value] = ACTIONS(1925),
    [sym_log_value] = ACTIONS(1925),
    [sym_log10_value] = ACTIONS(1925),
    [sym_power_value] = ACTIONS(1925),
    [sym_sqrt_value] = ACTIONS(1925),
    [sym_raw_be] = ACTIONS(1925),
    [sym_raw_le] = ACTIONS(1925),
    [sym_internal_queue_size_set] = ACTIONS(1925),
    [sym_internal_queue_size_get] = ACTIONS(1925),
    [sym_internal_threshold_set] = ACTIONS(1925),
    [sym_internal_threshold_get] = ACTIONS(1925),
    [sym_rebuilt_event] = ACTIONS(1925),
    [sym_do_custom_event] = ACTIONS(1925),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1925),
    [sym_clkmgr_set_clk_source] = ACTIONS(1925),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1925),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1925),
    [sym_clkmgr_get_timezone] = ACTIONS(1925),
    [sym_clkmgr_set_timezone] = ACTIONS(1925),
    [sym_clkmgr_get_resync_period] = ACTIONS(1925),
    [sym_clkmgr_set_resync_period] = ACTIONS(1925),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1925),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1925),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1925),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1925),
    [sym_clkmgr_get_timeservers] = ACTIONS(1925),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1925),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1925),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1925),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1925),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1925),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1925),
    [sym_set_log_level] = ACTIONS(1925),
    [sym_get_log_level] = ACTIONS(1925),
    [sym_amx_log] = ACTIONS(1925),
    [sym_smtp_server_config_set] = ACTIONS(1925),
    [sym_smtp_server_config_get] = ACTIONS(1925),
    [sym_smtp_send] = ACTIONS(1925),
    [sym_validate_netlinx_account] = ACTIONS(1925),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1925),
    [sym_authenticate_certificate] = ACTIONS(1925),
    [sym_audit_netlinx_session_event] = ACTIONS(1925),
    [sym_audit_netlinx_generic_event] = ACTIONS(1925),
    [sym_get_available_flash_disk_space] = ACTIONS(1925),
    [sym_get_max_flash_disk_space] = ACTIONS(1925),
    [sym_data_create_feed] = ACTIONS(1925),
    [sym_data_delete_feed] = ACTIONS(1925),
    [sym_data_publish_feed] = ACTIONS(1925),
    [sym_data_get_published_feed] = ACTIONS(1925),
    [sym_data_add_record] = ACTIONS(1925),
    [sym_data_get_event_record] = ACTIONS(1925),
    [sym__wc_data_create_feed] = ACTIONS(1925),
    [sym__wc_data_add_record] = ACTIONS(1925),
    [sym__wc_data_get_event_record] = ACTIONS(1925),
    [sym_format] = ACTIONS(1925),
    [sym_astro_clock] = ACTIONS(1925),
    [sym_set_outdoor_temperature] = ACTIONS(1925),
    [sym_set_virtual_level_count] = ACTIONS(1925),
    [sym_set_virtual_channel_count] = ACTIONS(1925),
    [sym_set_virtual_port_count] = ACTIONS(1925),
    [sym__wc_encode_int] = ACTIONS(1925),
    [sym__wc_decode_int] = ACTIONS(1925),
    [sym__wc_int] = ACTIONS(1925),
    [sym__wc_to_ch_int] = ACTIONS(1925),
    [sym__ch_to_wc_int] = ACTIONS(1925),
    [sym__wc_find_string_int] = ACTIONS(1925),
    [sym__wc_left_string_int] = ACTIONS(1925),
    [sym__wc_lower_string_int] = ACTIONS(1925),
    [sym__wc_mid_string_int] = ACTIONS(1925),
    [sym__wc_remove_string_int] = ACTIONS(1925),
    [sym__wc_right_string_int] = ACTIONS(1925),
    [sym__wc_upper_string_int] = ACTIONS(1925),
    [sym__wc_compare_string_int] = ACTIONS(1925),
    [sym__wc_get_buffer_char_int] = ACTIONS(1925),
    [sym__wc_get_buffer_string_int] = ACTIONS(1925),
    [sym__wc_concat_string_int] = ACTIONS(1925),
    [sym__wc_file_open_int] = ACTIONS(1925),
    [sym__wc_file_close_int] = ACTIONS(1925),
    [sym__wc_file_read_int] = ACTIONS(1925),
    [sym__wc_file_read_line_int] = ACTIONS(1925),
    [sym__wc_file_write_int] = ACTIONS(1925),
    [sym__wc_file_write_line_int] = ACTIONS(1925),
    [aux_sym_preproc_include_token1] = ACTIONS(1925),
    [aux_sym_preproc_define_token1] = ACTIONS(1927),
    [aux_sym_preproc_warn_token1] = ACTIONS(1927),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1927),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1927),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1927),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1927),
    [aux_sym_preproc_else_token1] = ACTIONS(1927),
    [anon_sym_LBRACK] = ACTIONS(1927),
    [anon_sym_LBRACE] = ACTIONS(1927),
    [anon_sym_LPAREN] = ACTIONS(1925),
    [anon_sym_BANG] = ACTIONS(1927),
    [anon_sym_TILDE] = ACTIONS(1927),
    [anon_sym_DASH] = ACTIONS(1925),
    [anon_sym_PLUS] = ACTIONS(1925),
    [anon_sym_DASH_DASH] = ACTIONS(1927),
    [anon_sym_PLUS_PLUS] = ACTIONS(1927),
    [anon_sym_DQUOTE] = ACTIONS(1927),
    [anon_sym_SQUOTE] = ACTIONS(1927),
    [sym_number_literal] = ACTIONS(1927),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(282)] = {
    [sym_identifier] = ACTIONS(1929),
    [sym_program_name_keyword] = ACTIONS(1929),
    [sym_module_name_keyword] = ACTIONS(1929),
    [sym_define_device_keyword] = ACTIONS(1929),
    [sym_define_combine_keyword] = ACTIONS(1929),
    [sym_define_constant_keyword] = ACTIONS(1929),
    [sym_define_type_keyword] = ACTIONS(1929),
    [sym_define_variable_keyword] = ACTIONS(1929),
    [sym_define_system_variable_keyword] = ACTIONS(1929),
    [sym_define_connect_level_keyword] = ACTIONS(1929),
    [sym_define_latching_keyword] = ACTIONS(1929),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1929),
    [sym_define_toggling_keyword] = ACTIONS(1929),
    [sym_define_call_keyword] = ACTIONS(1929),
    [sym_define_function_keyword] = ACTIONS(1929),
    [sym_define_library_function_keyword] = ACTIONS(1929),
    [sym_define_start_keyword] = ACTIONS(1929),
    [sym_define_event_keyword] = ACTIONS(1929),
    [sym_define_program_keyword] = ACTIONS(1929),
    [sym_define_module_keyword] = ACTIONS(1929),
    [sym_char_keyword] = ACTIONS(1929),
    [sym_widechar_keyword] = ACTIONS(1929),
    [sym_integer_keyword] = ACTIONS(1929),
    [sym_sinteger_keyword] = ACTIONS(1929),
    [sym_long_keyword] = ACTIONS(1929),
    [sym_slong_keyword] = ACTIONS(1929),
    [sym_float_keyword] = ACTIONS(1929),
    [sym_double_keyword] = ACTIONS(1929),
    [sym_variant_keyword] = ACTIONS(1929),
    [sym_variantarray_keyword] = ACTIONS(1929),
    [sym_local_var_keyword] = ACTIONS(1929),
    [sym_stack_var_keyword] = ACTIONS(1929),
    [sym_constant_keyword] = ACTIONS(1929),
    [sym_volatile_keyword] = ACTIONS(1929),
    [sym_non_volatile_keyword] = ACTIONS(1929),
    [sym_persistent_keyword] = ACTIONS(1929),
    [sym_dev_keyword] = ACTIONS(1929),
    [sym_devlev_keyword] = ACTIONS(1929),
    [sym_devchan_keyword] = ACTIONS(1929),
    [sym_if_keyword] = ACTIONS(1929),
    [sym_switch_keyword] = ACTIONS(1929),
    [sym_case_keyword] = ACTIONS(1929),
    [sym_default_keyword] = ACTIONS(1929),
    [sym_while_keyword] = ACTIONS(1929),
    [sym_for_keyword] = ACTIONS(1929),
    [sym_break_keyword] = ACTIONS(1929),
    [sym_continue_keyword] = ACTIONS(1929),
    [sym_return_keyword] = ACTIONS(1929),
    [sym_select_keyword] = ACTIONS(1929),
    [sym_active_keyword] = ACTIONS(1929),
    [sym_struct_keyword] = ACTIONS(1929),
    [sym_structure_keyword] = ACTIONS(1929),
    [sym_button_event_keyword] = ACTIONS(1929),
    [sym_channel_event_keyword] = ACTIONS(1929),
    [sym_level_event_keyword] = ACTIONS(1929),
    [sym_data_event_keyword] = ACTIONS(1929),
    [sym_timeline_event_keyword] = ACTIONS(1929),
    [sym_custom_event_keyword] = ACTIONS(1929),
    [sym_push_keyword] = ACTIONS(1929),
    [sym_release_keyword] = ACTIONS(1929),
    [aux_sym_on_keyword_token1] = ACTIONS(1929),
    [aux_sym_off_keyword_token1] = ACTIONS(1929),
    [sym_send_level_keyword] = ACTIONS(1929),
    [sym_send_string_keyword] = ACTIONS(1929),
    [sym_send_command_keyword] = ACTIONS(1929),
    [sym_clear_buffer_keyword] = ACTIONS(1929),
    [sym_create_buffer_keyword] = ACTIONS(1929),
    [sym_create_multi_buffer_keyword] = ACTIONS(1929),
    [sym_call_keyword] = ACTIONS(1929),
    [sym_system_call_keyword] = ACTIONS(1929),
    [sym_devchan_to_keyword] = ACTIONS(1929),
    [sym_devchan_min_to_keyword] = ACTIONS(1929),
    [sym_devchan_total_off_keyword] = ACTIONS(1929),
    [sym_devchan_pulse_keyword] = ACTIONS(1929),
    [sym_wait_keyword] = ACTIONS(1929),
    [sym_cancel_wait_keyword] = ACTIONS(1929),
    [sym_wait_until_keyword] = ACTIONS(1929),
    [sym_cancel_wait_until_keyword] = ACTIONS(1929),
    [sym_cancel_all_wait_keyword] = ACTIONS(1929),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1929),
    [sym_not] = ACTIONS(1929),
    [sym_bnot] = ACTIONS(1929),
    [sym___file__] = ACTIONS(1929),
    [sym___date__] = ACTIONS(1929),
    [sym___ldate__] = ACTIONS(1929),
    [sym___time__] = ACTIONS(1929),
    [sym___line__] = ACTIONS(1929),
    [sym___name__] = ACTIONS(1929),
    [sym___netlinx__] = ACTIONS(1929),
    [sym_netlinx_axi_version] = ACTIONS(1929),
    [sym_url_flg_tcp] = ACTIONS(1929),
    [sym_url_flg_accinfopresent] = ACTIONS(1929),
    [sym_url_flg_temp] = ACTIONS(1929),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1929),
    [sym_url_flg_stat_mask] = ACTIONS(1929),
    [sym_url_flg_stat_lookup] = ACTIONS(1929),
    [sym_url_flg_stat_connecting] = ACTIONS(1929),
    [sym_url_flg_stat_waiting] = ACTIONS(1929),
    [sym_url_flg_stat_connected] = ACTIONS(1929),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1929),
    [sym_first_virtual_device] = ACTIONS(1929),
    [sym_first_local_port] = ACTIONS(1929),
    [sym_dynamic_virtual_device] = ACTIONS(1929),
    [sym_timeline_once] = ACTIONS(1929),
    [sym_timeline_repeat] = ACTIONS(1929),
    [sym_timeline_absolute] = ACTIONS(1929),
    [sym_timeline_relative] = ACTIONS(1929),
    [sym_do_push_timed_infinite] = ACTIONS(1929),
    [sym_source_type_no_address] = ACTIONS(1929),
    [sym_source_type_neuron_id] = ACTIONS(1929),
    [sym_source_type_ip_address] = ACTIONS(1929),
    [sym_source_type_axlink] = ACTIONS(1929),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1929),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1929),
    [sym_source_type_ip_socket_address] = ACTIONS(1929),
    [sym_source_type_rs232] = ACTIONS(1929),
    [sym_source_type_internal] = ACTIONS(1929),
    [sym_source_type_mac_address] = ACTIONS(1929),
    [sym_source_type_ipv4_port] = ACTIONS(1929),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1929),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1929),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1929),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1929),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1929),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1929),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1929),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1929),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1929),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1929),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1929),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1929),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1929),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1929),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1929),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1929),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1929),
    [sym_true] = ACTIONS(1929),
    [sym_false] = ACTIONS(1929),
    [sym_file_read_only] = ACTIONS(1929),
    [sym_file_rw_new] = ACTIONS(1929),
    [sym_file_rw_append] = ACTIONS(1929),
    [sym_ip_tcp] = ACTIONS(1929),
    [sym_ip_udp] = ACTIONS(1929),
    [sym_ip_udp_2way] = ACTIONS(1929),
    [sym_xml_encode_types] = ACTIONS(1929),
    [sym_xml_encode_char_as_list] = ACTIONS(1929),
    [sym_xml_encode_le] = ACTIONS(1929),
    [sym_xml_decode_types] = ACTIONS(1929),
    [sym_xml_decode_no_preserve] = ACTIONS(1929),
    [sym_duet_dev_type_display_device] = ACTIONS(1929),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1929),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1929),
    [sym_duet_dev_type_io_device] = ACTIONS(1929),
    [sym_duet_dev_type_relay_device] = ACTIONS(1929),
    [sym_duet_dev_type_ups] = ACTIONS(1929),
    [sym_duet_dev_type_amplifier] = ACTIONS(1929),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1929),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1929),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1929),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1929),
    [sym_duet_dev_type_camera] = ACTIONS(1929),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1929),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1929),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1929),
    [sym_duet_dev_type_dss] = ACTIONS(1929),
    [sym_duet_dev_type_dvr] = ACTIONS(1929),
    [sym_duet_dev_type_disc_device] = ACTIONS(1929),
    [sym_duet_dev_type_document_camera] = ACTIONS(1929),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1929),
    [sym_duet_dev_type_hvac] = ACTIONS(1929),
    [sym_duet_dev_type_keypad] = ACTIONS(1929),
    [sym_duet_dev_type_light] = ACTIONS(1929),
    [sym_duet_dev_type_monitor] = ACTIONS(1929),
    [sym_duet_dev_type_motor] = ACTIONS(1929),
    [sym_duet_dev_type_multi_window] = ACTIONS(1929),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1929),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1929),
    [sym_duet_dev_type_receiver] = ACTIONS(1929),
    [sym_duet_dev_type_security_system] = ACTIONS(1929),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1929),
    [sym_duet_dev_type_settop_box] = ACTIONS(1929),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1929),
    [sym_duet_dev_type_switcher] = ACTIONS(1929),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1929),
    [sym_duet_dev_type_tv] = ACTIONS(1929),
    [sym_duet_dev_type_utility] = ACTIONS(1929),
    [sym_duet_dev_type_vcr] = ACTIONS(1929),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1929),
    [sym_duet_dev_type_video_processor] = ACTIONS(1929),
    [sym_duet_dev_type_video_projector] = ACTIONS(1929),
    [sym_duet_dev_type_video_wall] = ACTIONS(1929),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1929),
    [sym_duet_dev_type_weather] = ACTIONS(1929),
    [sym_duet_dev_not_polled] = ACTIONS(1929),
    [sym_duet_dev_polled] = ACTIONS(1929),
    [sym_clkmgr_mode_network] = ACTIONS(1929),
    [sym_clkmgr_mode_standalone] = ACTIONS(1929),
    [sym_normal_standby] = ACTIONS(1929),
    [sym_normal_wake] = ACTIONS(1929),
    [sym_ip_multicast_ttl_option] = ACTIONS(1929),
    [sym_ip_ttl_subnet] = ACTIONS(1929),
    [sym_ip_ttl_site] = ACTIONS(1929),
    [sym_ip_ttl_region] = ACTIONS(1929),
    [sym_ip_ttl_continent] = ACTIONS(1929),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1929),
    [sym_ip_nodelay_on] = ACTIONS(1929),
    [sym_ip_nodelay_off] = ACTIONS(1929),
    [sym_amx_error] = ACTIONS(1929),
    [sym_amx_warning] = ACTIONS(1929),
    [sym_amx_info] = ACTIONS(1929),
    [sym_amx_debug] = ACTIONS(1929),
    [sym_netlinx_login_success] = ACTIONS(1929),
    [sym_netlinx_login_fail] = ACTIONS(1929),
    [sym_netlinx_logout] = ACTIONS(1929),
    [sym_null_str] = ACTIONS(1929),
    [sym_smtp_address] = ACTIONS(1929),
    [sym_smtp_port_number] = ACTIONS(1929),
    [sym_smtp_username] = ACTIONS(1929),
    [sym_smtp_password] = ACTIONS(1929),
    [sym_smtp_require_tls] = ACTIONS(1929),
    [sym_smtp_from] = ACTIONS(1929),
    [sym_smtp_tls_true] = ACTIONS(1929),
    [sym_smtp_tls_false] = ACTIONS(1929),
    [sym_listview_on_row_select_event] = ACTIONS(1929),
    [sym_data_structure_unknown] = ACTIONS(1929),
    [sym_data_structure_datafeed] = ACTIONS(1929),
    [sym_data_structure_datarecordset] = ACTIONS(1929),
    [sym_data_structure_datarecord] = ACTIONS(1929),
    [sym_data_structure_datafield] = ACTIONS(1929),
    [sym_data_max_content_field_count] = ACTIONS(1929),
    [sym_data_max_metadata_field_count] = ACTIONS(1929),
    [sym_data_max_id_length] = ACTIONS(1929),
    [sym_data_max_name_length] = ACTIONS(1929),
    [sym_data_max_description_length] = ACTIONS(1929),
    [sym_data_max_type_length] = ACTIONS(1929),
    [sym_data_max_format_length] = ACTIONS(1929),
    [sym_data_max_source_length] = ACTIONS(1929),
    [sym_data_max_value_length] = ACTIONS(1929),
    [sym_data_max_label_length] = ACTIONS(1929),
    [sym_data_max_url_length] = ACTIONS(1929),
    [sym_max_last_login_info_length] = ACTIONS(1929),
    [sym_data_type_unknown] = ACTIONS(1929),
    [sym_data_type_string] = ACTIONS(1929),
    [sym_data_type_datetime] = ACTIONS(1929),
    [sym_data_type_date] = ACTIONS(1929),
    [sym_data_type_time] = ACTIONS(1929),
    [sym_data_type_image] = ACTIONS(1929),
    [sym_data_format_url] = ACTIONS(1929),
    [sym_data_format_phone] = ACTIONS(1929),
    [sym_data_format_email] = ACTIONS(1929),
    [sym_data_format_iso8601] = ACTIONS(1929),
    [sym_auditlog_priv] = ACTIONS(1929),
    [sym_configuration_priv] = ACTIONS(1929),
    [sym_deviceconfig_priv] = ACTIONS(1929),
    [sym_ftp_priv] = ACTIONS(1929),
    [sym_http_priv] = ACTIONS(1929),
    [sym_networkconfig_priv] = ACTIONS(1929),
    [sym_programport_priv] = ACTIONS(1929),
    [sym_remoteui_priv] = ACTIONS(1929),
    [sym_securitycontrol_priv] = ACTIONS(1929),
    [sym_softwaremanagement_priv] = ACTIONS(1929),
    [sym_terminal_priv] = ACTIONS(1929),
    [sym_tpadmin_priv] = ACTIONS(1929),
    [sym_user1_priv] = ACTIONS(1929),
    [sym_user2_priv] = ACTIONS(1929),
    [sym_user3_priv] = ACTIONS(1929),
    [sym_user4_priv] = ACTIONS(1929),
    [sym_usermanagement_priv] = ACTIONS(1929),
    [sym_valid_account] = ACTIONS(1929),
    [sym_err_name_invalid] = ACTIONS(1929),
    [sym_err_password_invalid] = ACTIONS(1929),
    [sym_err_invalid_account] = ACTIONS(1929),
    [sym_err_no_authorization] = ACTIONS(1929),
    [sym_err_invalid_parameter] = ACTIONS(1929),
    [sym_err_user_locked_out] = ACTIONS(1929),
    [sym_err_user_account_expired] = ACTIONS(1929),
    [sym_err_general_error] = ACTIONS(1929),
    [sym_err_authentication_error] = ACTIONS(1929),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1929),
    [sym_tls_validate_certificate] = ACTIONS(1929),
    [sym_tdata] = ACTIONS(1929),
    [sym_tchannel] = ACTIONS(1929),
    [sym_tlevel] = ACTIONS(1929),
    [sym_tbutton] = ACTIONS(1929),
    [sym_ttimeline] = ACTIONS(1929),
    [sym_tcustom] = ACTIONS(1929),
    [sym_url_struct] = ACTIONS(1929),
    [sym_dns_struct] = ACTIONS(1929),
    [sym_ip_address_struct] = ACTIONS(1929),
    [sym_dev_info_struct] = ACTIONS(1929),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1929),
    [sym_data_feed] = ACTIONS(1929),
    [sym_data_field] = ACTIONS(1929),
    [sym_data_record] = ACTIONS(1929),
    [sym_wc_data_feed] = ACTIONS(1929),
    [sym_wc_data_field] = ACTIONS(1929),
    [sym_wc_data_record] = ACTIONS(1929),
    [sym_last_login_info] = ACTIONS(1929),
    [sym____reserved___] = ACTIONS(1929),
    [sym_date] = ACTIONS(1929),
    [sym_day] = ACTIONS(1929),
    [sym_time] = ACTIONS(1929),
    [sym_ldate] = ACTIONS(1929),
    [sym_push_device] = ACTIONS(1929),
    [sym_push_channel] = ACTIONS(1929),
    [sym_push_devchan] = ACTIONS(1929),
    [sym_release_device] = ACTIONS(1929),
    [sym_release_channel] = ACTIONS(1929),
    [sym_release_devchan] = ACTIONS(1929),
    [sym_master_slot] = ACTIONS(1929),
    [sym_get_pulse_time] = ACTIONS(1929),
    [sym_get_timer] = ACTIONS(1929),
    [sym_master_sn] = ACTIONS(1929),
    [sym_system_number] = ACTIONS(1929),
    [sym_button] = ACTIONS(1929),
    [sym_dv_channel] = ACTIONS(1929),
    [sym_data] = ACTIONS(1929),
    [sym_level] = ACTIONS(1929),
    [sym_timeline] = ACTIONS(1929),
    [sym_channel] = ACTIONS(1929),
    [sym_custom] = ACTIONS(1929),
    [sym_atoi] = ACTIONS(1929),
    [sym_atol] = ACTIONS(1929),
    [sym_atof] = ACTIONS(1929),
    [sym_itoa] = ACTIONS(1929),
    [sym_itohex] = ACTIONS(1929),
    [sym_hextoi] = ACTIONS(1929),
    [sym_ftoa] = ACTIONS(1929),
    [sym_find_string] = ACTIONS(1929),
    [sym_left_string] = ACTIONS(1929),
    [sym_right_string] = ACTIONS(1929),
    [sym_mid_string] = ACTIONS(1929),
    [sym_remove_string] = ACTIONS(1929),
    [sym_get_buffer_string] = ACTIONS(1929),
    [sym_get_buffer_char] = ACTIONS(1929),
    [sym_get_multi_buffer_string] = ACTIONS(1929),
    [sym_length_string] = ACTIONS(1929),
    [sym_max_length_string] = ACTIONS(1929),
    [sym_set_length_string] = ACTIONS(1929),
    [sym_compare_string] = ACTIONS(1929),
    [sym_upper_string] = ACTIONS(1929),
    [sym_lower_string] = ACTIONS(1929),
    [sym_get_last] = ACTIONS(1929),
    [sym_date_to_day] = ACTIONS(1929),
    [sym_date_to_month] = ACTIONS(1929),
    [sym_date_to_year] = ACTIONS(1929),
    [sym_time_to_hour] = ACTIONS(1929),
    [sym_time_to_minute] = ACTIONS(1929),
    [sym_time_to_second] = ACTIONS(1929),
    [sym_day_of_week] = ACTIONS(1929),
    [sym_set_pulse_time] = ACTIONS(1929),
    [sym_set_timer] = ACTIONS(1929),
    [sym_file_open] = ACTIONS(1929),
    [sym_file_close] = ACTIONS(1929),
    [sym_file_read] = ACTIONS(1929),
    [sym_file_read_line] = ACTIONS(1929),
    [sym_file_write] = ACTIONS(1929),
    [sym_file_write_line] = ACTIONS(1929),
    [sym_file_copy] = ACTIONS(1929),
    [sym_file_delete] = ACTIONS(1929),
    [sym_file_rename] = ACTIONS(1929),
    [sym_file_dir] = ACTIONS(1929),
    [sym_file_getdir] = ACTIONS(1929),
    [sym_file_setdir] = ACTIONS(1929),
    [sym_file_create_dir] = ACTIONS(1929),
    [sym_file_remove_dir] = ACTIONS(1929),
    [sym_file_seek] = ACTIONS(1929),
    [sym_length_array] = ACTIONS(1929),
    [sym_max_length_array] = ACTIONS(1929),
    [sym_set_length_array] = ACTIONS(1929),
    [sym_ip_client_open] = ACTIONS(1929),
    [sym_ip_client_close] = ACTIONS(1929),
    [sym_ip_server_open] = ACTIONS(1929),
    [sym_ip_server_close] = ACTIONS(1929),
    [sym_ip_bound_client_open] = ACTIONS(1929),
    [sym_ip_mc_server_open] = ACTIONS(1929),
    [sym_ip_set_option] = ACTIONS(1929),
    [sym_ssh_client_open] = ACTIONS(1929),
    [sym_ssh_client_close] = ACTIONS(1929),
    [sym_tls_client_open] = ACTIONS(1929),
    [sym_tls_client_close] = ACTIONS(1929),
    [sym_do_push] = ACTIONS(1929),
    [sym_do_push_timed] = ACTIONS(1929),
    [sym_do_release] = ACTIONS(1929),
    [sym_device_id] = ACTIONS(1929),
    [sym_device_info] = ACTIONS(1929),
    [sym_device_id_string] = ACTIONS(1929),
    [sym_device_standby] = ACTIONS(1929),
    [sym_device_wake] = ACTIONS(1929),
    [sym_timeline_create] = ACTIONS(1929),
    [sym_timeline_kill] = ACTIONS(1929),
    [sym_timeline_reload] = ACTIONS(1929),
    [sym_timeline_pause] = ACTIONS(1929),
    [sym_timeline_restart] = ACTIONS(1929),
    [sym_timeline_set] = ACTIONS(1929),
    [sym_timeline_get] = ACTIONS(1929),
    [sym_timeline_active] = ACTIONS(1929),
    [sym_timeline_dynamic_id] = ACTIONS(1929),
    [sym_variable_to_string] = ACTIONS(1929),
    [sym_string_to_variable] = ACTIONS(1929),
    [sym_length_variable_to_string] = ACTIONS(1929),
    [sym_type_cast] = ACTIONS(1929),
    [sym_variable_to_xml] = ACTIONS(1929),
    [sym_xml_to_variable] = ACTIONS(1929),
    [sym_length_variable_to_xml] = ACTIONS(1929),
    [sym_redirect_string] = ACTIONS(1929),
    [sym_random_number] = ACTIONS(1929),
    [sym_get_unique_id] = ACTIONS(1929),
    [sym_get_system_number] = ACTIONS(1929),
    [sym_set_system_number] = ACTIONS(1929),
    [sym_get_serial_number] = ACTIONS(1929),
    [sym_get_master_build] = ACTIONS(1929),
    [sym_set_validation_code] = ACTIONS(1929),
    [sym_reboot] = ACTIONS(1929),
    [sym_load_duet_module] = ACTIONS(1929),
    [sym_unload_duet_module] = ACTIONS(1929),
    [sym_duet_mem_size_set] = ACTIONS(1929),
    [sym_duet_mem_size_get] = ACTIONS(1929),
    [sym_dynamic_application_device] = ACTIONS(1929),
    [sym_dynamic_polled_port] = ACTIONS(1929),
    [sym_static_port_binding] = ACTIONS(1929),
    [sym_static_ip_binding] = ACTIONS(1929),
    [sym_get_dns_list] = ACTIONS(1929),
    [sym_set_dns_list] = ACTIONS(1929),
    [sym_get_ip_address] = ACTIONS(1929),
    [sym_set_ip_address] = ACTIONS(1929),
    [sym_get_url_list] = ACTIONS(1929),
    [sym_add_url_entry] = ACTIONS(1929),
    [sym_delete_url_entry] = ACTIONS(1929),
    [sym_abs_value] = ACTIONS(1929),
    [sym_max_value] = ACTIONS(1929),
    [sym_min_value] = ACTIONS(1929),
    [sym_exp_value] = ACTIONS(1929),
    [sym_log_value] = ACTIONS(1929),
    [sym_log10_value] = ACTIONS(1929),
    [sym_power_value] = ACTIONS(1929),
    [sym_sqrt_value] = ACTIONS(1929),
    [sym_raw_be] = ACTIONS(1929),
    [sym_raw_le] = ACTIONS(1929),
    [sym_internal_queue_size_set] = ACTIONS(1929),
    [sym_internal_queue_size_get] = ACTIONS(1929),
    [sym_internal_threshold_set] = ACTIONS(1929),
    [sym_internal_threshold_get] = ACTIONS(1929),
    [sym_rebuilt_event] = ACTIONS(1929),
    [sym_do_custom_event] = ACTIONS(1929),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1929),
    [sym_clkmgr_set_clk_source] = ACTIONS(1929),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1929),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1929),
    [sym_clkmgr_get_timezone] = ACTIONS(1929),
    [sym_clkmgr_set_timezone] = ACTIONS(1929),
    [sym_clkmgr_get_resync_period] = ACTIONS(1929),
    [sym_clkmgr_set_resync_period] = ACTIONS(1929),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1929),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1929),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1929),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1929),
    [sym_clkmgr_get_timeservers] = ACTIONS(1929),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1929),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1929),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1929),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1929),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1929),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1929),
    [sym_set_log_level] = ACTIONS(1929),
    [sym_get_log_level] = ACTIONS(1929),
    [sym_amx_log] = ACTIONS(1929),
    [sym_smtp_server_config_set] = ACTIONS(1929),
    [sym_smtp_server_config_get] = ACTIONS(1929),
    [sym_smtp_send] = ACTIONS(1929),
    [sym_validate_netlinx_account] = ACTIONS(1929),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1929),
    [sym_authenticate_certificate] = ACTIONS(1929),
    [sym_audit_netlinx_session_event] = ACTIONS(1929),
    [sym_audit_netlinx_generic_event] = ACTIONS(1929),
    [sym_get_available_flash_disk_space] = ACTIONS(1929),
    [sym_get_max_flash_disk_space] = ACTIONS(1929),
    [sym_data_create_feed] = ACTIONS(1929),
    [sym_data_delete_feed] = ACTIONS(1929),
    [sym_data_publish_feed] = ACTIONS(1929),
    [sym_data_get_published_feed] = ACTIONS(1929),
    [sym_data_add_record] = ACTIONS(1929),
    [sym_data_get_event_record] = ACTIONS(1929),
    [sym__wc_data_create_feed] = ACTIONS(1929),
    [sym__wc_data_add_record] = ACTIONS(1929),
    [sym__wc_data_get_event_record] = ACTIONS(1929),
    [sym_format] = ACTIONS(1929),
    [sym_astro_clock] = ACTIONS(1929),
    [sym_set_outdoor_temperature] = ACTIONS(1929),
    [sym_set_virtual_level_count] = ACTIONS(1929),
    [sym_set_virtual_channel_count] = ACTIONS(1929),
    [sym_set_virtual_port_count] = ACTIONS(1929),
    [sym__wc_encode_int] = ACTIONS(1929),
    [sym__wc_decode_int] = ACTIONS(1929),
    [sym__wc_int] = ACTIONS(1929),
    [sym__wc_to_ch_int] = ACTIONS(1929),
    [sym__ch_to_wc_int] = ACTIONS(1929),
    [sym__wc_find_string_int] = ACTIONS(1929),
    [sym__wc_left_string_int] = ACTIONS(1929),
    [sym__wc_lower_string_int] = ACTIONS(1929),
    [sym__wc_mid_string_int] = ACTIONS(1929),
    [sym__wc_remove_string_int] = ACTIONS(1929),
    [sym__wc_right_string_int] = ACTIONS(1929),
    [sym__wc_upper_string_int] = ACTIONS(1929),
    [sym__wc_compare_string_int] = ACTIONS(1929),
    [sym__wc_get_buffer_char_int] = ACTIONS(1929),
    [sym__wc_get_buffer_string_int] = ACTIONS(1929),
    [sym__wc_concat_string_int] = ACTIONS(1929),
    [sym__wc_file_open_int] = ACTIONS(1929),
    [sym__wc_file_close_int] = ACTIONS(1929),
    [sym__wc_file_read_int] = ACTIONS(1929),
    [sym__wc_file_read_line_int] = ACTIONS(1929),
    [sym__wc_file_write_int] = ACTIONS(1929),
    [sym__wc_file_write_line_int] = ACTIONS(1929),
    [aux_sym_preproc_include_token1] = ACTIONS(1929),
    [aux_sym_preproc_define_token1] = ACTIONS(1931),
    [aux_sym_preproc_warn_token1] = ACTIONS(1931),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1931),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1931),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1931),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1931),
    [aux_sym_preproc_else_token1] = ACTIONS(1931),
    [anon_sym_LBRACK] = ACTIONS(1931),
    [anon_sym_LBRACE] = ACTIONS(1931),
    [anon_sym_LPAREN] = ACTIONS(1929),
    [anon_sym_BANG] = ACTIONS(1931),
    [anon_sym_TILDE] = ACTIONS(1931),
    [anon_sym_DASH] = ACTIONS(1929),
    [anon_sym_PLUS] = ACTIONS(1929),
    [anon_sym_DASH_DASH] = ACTIONS(1931),
    [anon_sym_PLUS_PLUS] = ACTIONS(1931),
    [anon_sym_DQUOTE] = ACTIONS(1931),
    [anon_sym_SQUOTE] = ACTIONS(1931),
    [sym_number_literal] = ACTIONS(1931),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(283)] = {
    [sym_identifier] = ACTIONS(1933),
    [sym_program_name_keyword] = ACTIONS(1933),
    [sym_module_name_keyword] = ACTIONS(1933),
    [sym_define_device_keyword] = ACTIONS(1933),
    [sym_define_combine_keyword] = ACTIONS(1933),
    [sym_define_constant_keyword] = ACTIONS(1933),
    [sym_define_type_keyword] = ACTIONS(1933),
    [sym_define_variable_keyword] = ACTIONS(1933),
    [sym_define_system_variable_keyword] = ACTIONS(1933),
    [sym_define_connect_level_keyword] = ACTIONS(1933),
    [sym_define_latching_keyword] = ACTIONS(1933),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1933),
    [sym_define_toggling_keyword] = ACTIONS(1933),
    [sym_define_call_keyword] = ACTIONS(1933),
    [sym_define_function_keyword] = ACTIONS(1933),
    [sym_define_library_function_keyword] = ACTIONS(1933),
    [sym_define_start_keyword] = ACTIONS(1933),
    [sym_define_event_keyword] = ACTIONS(1933),
    [sym_define_program_keyword] = ACTIONS(1933),
    [sym_define_module_keyword] = ACTIONS(1933),
    [sym_char_keyword] = ACTIONS(1933),
    [sym_widechar_keyword] = ACTIONS(1933),
    [sym_integer_keyword] = ACTIONS(1933),
    [sym_sinteger_keyword] = ACTIONS(1933),
    [sym_long_keyword] = ACTIONS(1933),
    [sym_slong_keyword] = ACTIONS(1933),
    [sym_float_keyword] = ACTIONS(1933),
    [sym_double_keyword] = ACTIONS(1933),
    [sym_variant_keyword] = ACTIONS(1933),
    [sym_variantarray_keyword] = ACTIONS(1933),
    [sym_local_var_keyword] = ACTIONS(1933),
    [sym_stack_var_keyword] = ACTIONS(1933),
    [sym_constant_keyword] = ACTIONS(1933),
    [sym_volatile_keyword] = ACTIONS(1933),
    [sym_non_volatile_keyword] = ACTIONS(1933),
    [sym_persistent_keyword] = ACTIONS(1933),
    [sym_dev_keyword] = ACTIONS(1933),
    [sym_devlev_keyword] = ACTIONS(1933),
    [sym_devchan_keyword] = ACTIONS(1933),
    [sym_if_keyword] = ACTIONS(1933),
    [sym_switch_keyword] = ACTIONS(1933),
    [sym_case_keyword] = ACTIONS(1933),
    [sym_default_keyword] = ACTIONS(1933),
    [sym_while_keyword] = ACTIONS(1933),
    [sym_for_keyword] = ACTIONS(1933),
    [sym_break_keyword] = ACTIONS(1933),
    [sym_continue_keyword] = ACTIONS(1933),
    [sym_return_keyword] = ACTIONS(1933),
    [sym_select_keyword] = ACTIONS(1933),
    [sym_active_keyword] = ACTIONS(1933),
    [sym_struct_keyword] = ACTIONS(1933),
    [sym_structure_keyword] = ACTIONS(1933),
    [sym_button_event_keyword] = ACTIONS(1933),
    [sym_channel_event_keyword] = ACTIONS(1933),
    [sym_level_event_keyword] = ACTIONS(1933),
    [sym_data_event_keyword] = ACTIONS(1933),
    [sym_timeline_event_keyword] = ACTIONS(1933),
    [sym_custom_event_keyword] = ACTIONS(1933),
    [sym_push_keyword] = ACTIONS(1933),
    [sym_release_keyword] = ACTIONS(1933),
    [aux_sym_on_keyword_token1] = ACTIONS(1933),
    [aux_sym_off_keyword_token1] = ACTIONS(1933),
    [sym_send_level_keyword] = ACTIONS(1933),
    [sym_send_string_keyword] = ACTIONS(1933),
    [sym_send_command_keyword] = ACTIONS(1933),
    [sym_clear_buffer_keyword] = ACTIONS(1933),
    [sym_create_buffer_keyword] = ACTIONS(1933),
    [sym_create_multi_buffer_keyword] = ACTIONS(1933),
    [sym_call_keyword] = ACTIONS(1933),
    [sym_system_call_keyword] = ACTIONS(1933),
    [sym_devchan_to_keyword] = ACTIONS(1933),
    [sym_devchan_min_to_keyword] = ACTIONS(1933),
    [sym_devchan_total_off_keyword] = ACTIONS(1933),
    [sym_devchan_pulse_keyword] = ACTIONS(1933),
    [sym_wait_keyword] = ACTIONS(1933),
    [sym_cancel_wait_keyword] = ACTIONS(1933),
    [sym_wait_until_keyword] = ACTIONS(1933),
    [sym_cancel_wait_until_keyword] = ACTIONS(1933),
    [sym_cancel_all_wait_keyword] = ACTIONS(1933),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1933),
    [sym_not] = ACTIONS(1933),
    [sym_bnot] = ACTIONS(1933),
    [sym___file__] = ACTIONS(1933),
    [sym___date__] = ACTIONS(1933),
    [sym___ldate__] = ACTIONS(1933),
    [sym___time__] = ACTIONS(1933),
    [sym___line__] = ACTIONS(1933),
    [sym___name__] = ACTIONS(1933),
    [sym___netlinx__] = ACTIONS(1933),
    [sym_netlinx_axi_version] = ACTIONS(1933),
    [sym_url_flg_tcp] = ACTIONS(1933),
    [sym_url_flg_accinfopresent] = ACTIONS(1933),
    [sym_url_flg_temp] = ACTIONS(1933),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1933),
    [sym_url_flg_stat_mask] = ACTIONS(1933),
    [sym_url_flg_stat_lookup] = ACTIONS(1933),
    [sym_url_flg_stat_connecting] = ACTIONS(1933),
    [sym_url_flg_stat_waiting] = ACTIONS(1933),
    [sym_url_flg_stat_connected] = ACTIONS(1933),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1933),
    [sym_first_virtual_device] = ACTIONS(1933),
    [sym_first_local_port] = ACTIONS(1933),
    [sym_dynamic_virtual_device] = ACTIONS(1933),
    [sym_timeline_once] = ACTIONS(1933),
    [sym_timeline_repeat] = ACTIONS(1933),
    [sym_timeline_absolute] = ACTIONS(1933),
    [sym_timeline_relative] = ACTIONS(1933),
    [sym_do_push_timed_infinite] = ACTIONS(1933),
    [sym_source_type_no_address] = ACTIONS(1933),
    [sym_source_type_neuron_id] = ACTIONS(1933),
    [sym_source_type_ip_address] = ACTIONS(1933),
    [sym_source_type_axlink] = ACTIONS(1933),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1933),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1933),
    [sym_source_type_ip_socket_address] = ACTIONS(1933),
    [sym_source_type_rs232] = ACTIONS(1933),
    [sym_source_type_internal] = ACTIONS(1933),
    [sym_source_type_mac_address] = ACTIONS(1933),
    [sym_source_type_ipv4_port] = ACTIONS(1933),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1933),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1933),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1933),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1933),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1933),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1933),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1933),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1933),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1933),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1933),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1933),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1933),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1933),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1933),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1933),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1933),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1933),
    [sym_true] = ACTIONS(1933),
    [sym_false] = ACTIONS(1933),
    [sym_file_read_only] = ACTIONS(1933),
    [sym_file_rw_new] = ACTIONS(1933),
    [sym_file_rw_append] = ACTIONS(1933),
    [sym_ip_tcp] = ACTIONS(1933),
    [sym_ip_udp] = ACTIONS(1933),
    [sym_ip_udp_2way] = ACTIONS(1933),
    [sym_xml_encode_types] = ACTIONS(1933),
    [sym_xml_encode_char_as_list] = ACTIONS(1933),
    [sym_xml_encode_le] = ACTIONS(1933),
    [sym_xml_decode_types] = ACTIONS(1933),
    [sym_xml_decode_no_preserve] = ACTIONS(1933),
    [sym_duet_dev_type_display_device] = ACTIONS(1933),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1933),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1933),
    [sym_duet_dev_type_io_device] = ACTIONS(1933),
    [sym_duet_dev_type_relay_device] = ACTIONS(1933),
    [sym_duet_dev_type_ups] = ACTIONS(1933),
    [sym_duet_dev_type_amplifier] = ACTIONS(1933),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1933),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1933),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1933),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1933),
    [sym_duet_dev_type_camera] = ACTIONS(1933),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1933),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1933),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1933),
    [sym_duet_dev_type_dss] = ACTIONS(1933),
    [sym_duet_dev_type_dvr] = ACTIONS(1933),
    [sym_duet_dev_type_disc_device] = ACTIONS(1933),
    [sym_duet_dev_type_document_camera] = ACTIONS(1933),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1933),
    [sym_duet_dev_type_hvac] = ACTIONS(1933),
    [sym_duet_dev_type_keypad] = ACTIONS(1933),
    [sym_duet_dev_type_light] = ACTIONS(1933),
    [sym_duet_dev_type_monitor] = ACTIONS(1933),
    [sym_duet_dev_type_motor] = ACTIONS(1933),
    [sym_duet_dev_type_multi_window] = ACTIONS(1933),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1933),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1933),
    [sym_duet_dev_type_receiver] = ACTIONS(1933),
    [sym_duet_dev_type_security_system] = ACTIONS(1933),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1933),
    [sym_duet_dev_type_settop_box] = ACTIONS(1933),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1933),
    [sym_duet_dev_type_switcher] = ACTIONS(1933),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1933),
    [sym_duet_dev_type_tv] = ACTIONS(1933),
    [sym_duet_dev_type_utility] = ACTIONS(1933),
    [sym_duet_dev_type_vcr] = ACTIONS(1933),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1933),
    [sym_duet_dev_type_video_processor] = ACTIONS(1933),
    [sym_duet_dev_type_video_projector] = ACTIONS(1933),
    [sym_duet_dev_type_video_wall] = ACTIONS(1933),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1933),
    [sym_duet_dev_type_weather] = ACTIONS(1933),
    [sym_duet_dev_not_polled] = ACTIONS(1933),
    [sym_duet_dev_polled] = ACTIONS(1933),
    [sym_clkmgr_mode_network] = ACTIONS(1933),
    [sym_clkmgr_mode_standalone] = ACTIONS(1933),
    [sym_normal_standby] = ACTIONS(1933),
    [sym_normal_wake] = ACTIONS(1933),
    [sym_ip_multicast_ttl_option] = ACTIONS(1933),
    [sym_ip_ttl_subnet] = ACTIONS(1933),
    [sym_ip_ttl_site] = ACTIONS(1933),
    [sym_ip_ttl_region] = ACTIONS(1933),
    [sym_ip_ttl_continent] = ACTIONS(1933),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1933),
    [sym_ip_nodelay_on] = ACTIONS(1933),
    [sym_ip_nodelay_off] = ACTIONS(1933),
    [sym_amx_error] = ACTIONS(1933),
    [sym_amx_warning] = ACTIONS(1933),
    [sym_amx_info] = ACTIONS(1933),
    [sym_amx_debug] = ACTIONS(1933),
    [sym_netlinx_login_success] = ACTIONS(1933),
    [sym_netlinx_login_fail] = ACTIONS(1933),
    [sym_netlinx_logout] = ACTIONS(1933),
    [sym_null_str] = ACTIONS(1933),
    [sym_smtp_address] = ACTIONS(1933),
    [sym_smtp_port_number] = ACTIONS(1933),
    [sym_smtp_username] = ACTIONS(1933),
    [sym_smtp_password] = ACTIONS(1933),
    [sym_smtp_require_tls] = ACTIONS(1933),
    [sym_smtp_from] = ACTIONS(1933),
    [sym_smtp_tls_true] = ACTIONS(1933),
    [sym_smtp_tls_false] = ACTIONS(1933),
    [sym_listview_on_row_select_event] = ACTIONS(1933),
    [sym_data_structure_unknown] = ACTIONS(1933),
    [sym_data_structure_datafeed] = ACTIONS(1933),
    [sym_data_structure_datarecordset] = ACTIONS(1933),
    [sym_data_structure_datarecord] = ACTIONS(1933),
    [sym_data_structure_datafield] = ACTIONS(1933),
    [sym_data_max_content_field_count] = ACTIONS(1933),
    [sym_data_max_metadata_field_count] = ACTIONS(1933),
    [sym_data_max_id_length] = ACTIONS(1933),
    [sym_data_max_name_length] = ACTIONS(1933),
    [sym_data_max_description_length] = ACTIONS(1933),
    [sym_data_max_type_length] = ACTIONS(1933),
    [sym_data_max_format_length] = ACTIONS(1933),
    [sym_data_max_source_length] = ACTIONS(1933),
    [sym_data_max_value_length] = ACTIONS(1933),
    [sym_data_max_label_length] = ACTIONS(1933),
    [sym_data_max_url_length] = ACTIONS(1933),
    [sym_max_last_login_info_length] = ACTIONS(1933),
    [sym_data_type_unknown] = ACTIONS(1933),
    [sym_data_type_string] = ACTIONS(1933),
    [sym_data_type_datetime] = ACTIONS(1933),
    [sym_data_type_date] = ACTIONS(1933),
    [sym_data_type_time] = ACTIONS(1933),
    [sym_data_type_image] = ACTIONS(1933),
    [sym_data_format_url] = ACTIONS(1933),
    [sym_data_format_phone] = ACTIONS(1933),
    [sym_data_format_email] = ACTIONS(1933),
    [sym_data_format_iso8601] = ACTIONS(1933),
    [sym_auditlog_priv] = ACTIONS(1933),
    [sym_configuration_priv] = ACTIONS(1933),
    [sym_deviceconfig_priv] = ACTIONS(1933),
    [sym_ftp_priv] = ACTIONS(1933),
    [sym_http_priv] = ACTIONS(1933),
    [sym_networkconfig_priv] = ACTIONS(1933),
    [sym_programport_priv] = ACTIONS(1933),
    [sym_remoteui_priv] = ACTIONS(1933),
    [sym_securitycontrol_priv] = ACTIONS(1933),
    [sym_softwaremanagement_priv] = ACTIONS(1933),
    [sym_terminal_priv] = ACTIONS(1933),
    [sym_tpadmin_priv] = ACTIONS(1933),
    [sym_user1_priv] = ACTIONS(1933),
    [sym_user2_priv] = ACTIONS(1933),
    [sym_user3_priv] = ACTIONS(1933),
    [sym_user4_priv] = ACTIONS(1933),
    [sym_usermanagement_priv] = ACTIONS(1933),
    [sym_valid_account] = ACTIONS(1933),
    [sym_err_name_invalid] = ACTIONS(1933),
    [sym_err_password_invalid] = ACTIONS(1933),
    [sym_err_invalid_account] = ACTIONS(1933),
    [sym_err_no_authorization] = ACTIONS(1933),
    [sym_err_invalid_parameter] = ACTIONS(1933),
    [sym_err_user_locked_out] = ACTIONS(1933),
    [sym_err_user_account_expired] = ACTIONS(1933),
    [sym_err_general_error] = ACTIONS(1933),
    [sym_err_authentication_error] = ACTIONS(1933),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1933),
    [sym_tls_validate_certificate] = ACTIONS(1933),
    [sym_tdata] = ACTIONS(1933),
    [sym_tchannel] = ACTIONS(1933),
    [sym_tlevel] = ACTIONS(1933),
    [sym_tbutton] = ACTIONS(1933),
    [sym_ttimeline] = ACTIONS(1933),
    [sym_tcustom] = ACTIONS(1933),
    [sym_url_struct] = ACTIONS(1933),
    [sym_dns_struct] = ACTIONS(1933),
    [sym_ip_address_struct] = ACTIONS(1933),
    [sym_dev_info_struct] = ACTIONS(1933),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1933),
    [sym_data_feed] = ACTIONS(1933),
    [sym_data_field] = ACTIONS(1933),
    [sym_data_record] = ACTIONS(1933),
    [sym_wc_data_feed] = ACTIONS(1933),
    [sym_wc_data_field] = ACTIONS(1933),
    [sym_wc_data_record] = ACTIONS(1933),
    [sym_last_login_info] = ACTIONS(1933),
    [sym____reserved___] = ACTIONS(1933),
    [sym_date] = ACTIONS(1933),
    [sym_day] = ACTIONS(1933),
    [sym_time] = ACTIONS(1933),
    [sym_ldate] = ACTIONS(1933),
    [sym_push_device] = ACTIONS(1933),
    [sym_push_channel] = ACTIONS(1933),
    [sym_push_devchan] = ACTIONS(1933),
    [sym_release_device] = ACTIONS(1933),
    [sym_release_channel] = ACTIONS(1933),
    [sym_release_devchan] = ACTIONS(1933),
    [sym_master_slot] = ACTIONS(1933),
    [sym_get_pulse_time] = ACTIONS(1933),
    [sym_get_timer] = ACTIONS(1933),
    [sym_master_sn] = ACTIONS(1933),
    [sym_system_number] = ACTIONS(1933),
    [sym_button] = ACTIONS(1933),
    [sym_dv_channel] = ACTIONS(1933),
    [sym_data] = ACTIONS(1933),
    [sym_level] = ACTIONS(1933),
    [sym_timeline] = ACTIONS(1933),
    [sym_channel] = ACTIONS(1933),
    [sym_custom] = ACTIONS(1933),
    [sym_atoi] = ACTIONS(1933),
    [sym_atol] = ACTIONS(1933),
    [sym_atof] = ACTIONS(1933),
    [sym_itoa] = ACTIONS(1933),
    [sym_itohex] = ACTIONS(1933),
    [sym_hextoi] = ACTIONS(1933),
    [sym_ftoa] = ACTIONS(1933),
    [sym_find_string] = ACTIONS(1933),
    [sym_left_string] = ACTIONS(1933),
    [sym_right_string] = ACTIONS(1933),
    [sym_mid_string] = ACTIONS(1933),
    [sym_remove_string] = ACTIONS(1933),
    [sym_get_buffer_string] = ACTIONS(1933),
    [sym_get_buffer_char] = ACTIONS(1933),
    [sym_get_multi_buffer_string] = ACTIONS(1933),
    [sym_length_string] = ACTIONS(1933),
    [sym_max_length_string] = ACTIONS(1933),
    [sym_set_length_string] = ACTIONS(1933),
    [sym_compare_string] = ACTIONS(1933),
    [sym_upper_string] = ACTIONS(1933),
    [sym_lower_string] = ACTIONS(1933),
    [sym_get_last] = ACTIONS(1933),
    [sym_date_to_day] = ACTIONS(1933),
    [sym_date_to_month] = ACTIONS(1933),
    [sym_date_to_year] = ACTIONS(1933),
    [sym_time_to_hour] = ACTIONS(1933),
    [sym_time_to_minute] = ACTIONS(1933),
    [sym_time_to_second] = ACTIONS(1933),
    [sym_day_of_week] = ACTIONS(1933),
    [sym_set_pulse_time] = ACTIONS(1933),
    [sym_set_timer] = ACTIONS(1933),
    [sym_file_open] = ACTIONS(1933),
    [sym_file_close] = ACTIONS(1933),
    [sym_file_read] = ACTIONS(1933),
    [sym_file_read_line] = ACTIONS(1933),
    [sym_file_write] = ACTIONS(1933),
    [sym_file_write_line] = ACTIONS(1933),
    [sym_file_copy] = ACTIONS(1933),
    [sym_file_delete] = ACTIONS(1933),
    [sym_file_rename] = ACTIONS(1933),
    [sym_file_dir] = ACTIONS(1933),
    [sym_file_getdir] = ACTIONS(1933),
    [sym_file_setdir] = ACTIONS(1933),
    [sym_file_create_dir] = ACTIONS(1933),
    [sym_file_remove_dir] = ACTIONS(1933),
    [sym_file_seek] = ACTIONS(1933),
    [sym_length_array] = ACTIONS(1933),
    [sym_max_length_array] = ACTIONS(1933),
    [sym_set_length_array] = ACTIONS(1933),
    [sym_ip_client_open] = ACTIONS(1933),
    [sym_ip_client_close] = ACTIONS(1933),
    [sym_ip_server_open] = ACTIONS(1933),
    [sym_ip_server_close] = ACTIONS(1933),
    [sym_ip_bound_client_open] = ACTIONS(1933),
    [sym_ip_mc_server_open] = ACTIONS(1933),
    [sym_ip_set_option] = ACTIONS(1933),
    [sym_ssh_client_open] = ACTIONS(1933),
    [sym_ssh_client_close] = ACTIONS(1933),
    [sym_tls_client_open] = ACTIONS(1933),
    [sym_tls_client_close] = ACTIONS(1933),
    [sym_do_push] = ACTIONS(1933),
    [sym_do_push_timed] = ACTIONS(1933),
    [sym_do_release] = ACTIONS(1933),
    [sym_device_id] = ACTIONS(1933),
    [sym_device_info] = ACTIONS(1933),
    [sym_device_id_string] = ACTIONS(1933),
    [sym_device_standby] = ACTIONS(1933),
    [sym_device_wake] = ACTIONS(1933),
    [sym_timeline_create] = ACTIONS(1933),
    [sym_timeline_kill] = ACTIONS(1933),
    [sym_timeline_reload] = ACTIONS(1933),
    [sym_timeline_pause] = ACTIONS(1933),
    [sym_timeline_restart] = ACTIONS(1933),
    [sym_timeline_set] = ACTIONS(1933),
    [sym_timeline_get] = ACTIONS(1933),
    [sym_timeline_active] = ACTIONS(1933),
    [sym_timeline_dynamic_id] = ACTIONS(1933),
    [sym_variable_to_string] = ACTIONS(1933),
    [sym_string_to_variable] = ACTIONS(1933),
    [sym_length_variable_to_string] = ACTIONS(1933),
    [sym_type_cast] = ACTIONS(1933),
    [sym_variable_to_xml] = ACTIONS(1933),
    [sym_xml_to_variable] = ACTIONS(1933),
    [sym_length_variable_to_xml] = ACTIONS(1933),
    [sym_redirect_string] = ACTIONS(1933),
    [sym_random_number] = ACTIONS(1933),
    [sym_get_unique_id] = ACTIONS(1933),
    [sym_get_system_number] = ACTIONS(1933),
    [sym_set_system_number] = ACTIONS(1933),
    [sym_get_serial_number] = ACTIONS(1933),
    [sym_get_master_build] = ACTIONS(1933),
    [sym_set_validation_code] = ACTIONS(1933),
    [sym_reboot] = ACTIONS(1933),
    [sym_load_duet_module] = ACTIONS(1933),
    [sym_unload_duet_module] = ACTIONS(1933),
    [sym_duet_mem_size_set] = ACTIONS(1933),
    [sym_duet_mem_size_get] = ACTIONS(1933),
    [sym_dynamic_application_device] = ACTIONS(1933),
    [sym_dynamic_polled_port] = ACTIONS(1933),
    [sym_static_port_binding] = ACTIONS(1933),
    [sym_static_ip_binding] = ACTIONS(1933),
    [sym_get_dns_list] = ACTIONS(1933),
    [sym_set_dns_list] = ACTIONS(1933),
    [sym_get_ip_address] = ACTIONS(1933),
    [sym_set_ip_address] = ACTIONS(1933),
    [sym_get_url_list] = ACTIONS(1933),
    [sym_add_url_entry] = ACTIONS(1933),
    [sym_delete_url_entry] = ACTIONS(1933),
    [sym_abs_value] = ACTIONS(1933),
    [sym_max_value] = ACTIONS(1933),
    [sym_min_value] = ACTIONS(1933),
    [sym_exp_value] = ACTIONS(1933),
    [sym_log_value] = ACTIONS(1933),
    [sym_log10_value] = ACTIONS(1933),
    [sym_power_value] = ACTIONS(1933),
    [sym_sqrt_value] = ACTIONS(1933),
    [sym_raw_be] = ACTIONS(1933),
    [sym_raw_le] = ACTIONS(1933),
    [sym_internal_queue_size_set] = ACTIONS(1933),
    [sym_internal_queue_size_get] = ACTIONS(1933),
    [sym_internal_threshold_set] = ACTIONS(1933),
    [sym_internal_threshold_get] = ACTIONS(1933),
    [sym_rebuilt_event] = ACTIONS(1933),
    [sym_do_custom_event] = ACTIONS(1933),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1933),
    [sym_clkmgr_set_clk_source] = ACTIONS(1933),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1933),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1933),
    [sym_clkmgr_get_timezone] = ACTIONS(1933),
    [sym_clkmgr_set_timezone] = ACTIONS(1933),
    [sym_clkmgr_get_resync_period] = ACTIONS(1933),
    [sym_clkmgr_set_resync_period] = ACTIONS(1933),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1933),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1933),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1933),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1933),
    [sym_clkmgr_get_timeservers] = ACTIONS(1933),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1933),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1933),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1933),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1933),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1933),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1933),
    [sym_set_log_level] = ACTIONS(1933),
    [sym_get_log_level] = ACTIONS(1933),
    [sym_amx_log] = ACTIONS(1933),
    [sym_smtp_server_config_set] = ACTIONS(1933),
    [sym_smtp_server_config_get] = ACTIONS(1933),
    [sym_smtp_send] = ACTIONS(1933),
    [sym_validate_netlinx_account] = ACTIONS(1933),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1933),
    [sym_authenticate_certificate] = ACTIONS(1933),
    [sym_audit_netlinx_session_event] = ACTIONS(1933),
    [sym_audit_netlinx_generic_event] = ACTIONS(1933),
    [sym_get_available_flash_disk_space] = ACTIONS(1933),
    [sym_get_max_flash_disk_space] = ACTIONS(1933),
    [sym_data_create_feed] = ACTIONS(1933),
    [sym_data_delete_feed] = ACTIONS(1933),
    [sym_data_publish_feed] = ACTIONS(1933),
    [sym_data_get_published_feed] = ACTIONS(1933),
    [sym_data_add_record] = ACTIONS(1933),
    [sym_data_get_event_record] = ACTIONS(1933),
    [sym__wc_data_create_feed] = ACTIONS(1933),
    [sym__wc_data_add_record] = ACTIONS(1933),
    [sym__wc_data_get_event_record] = ACTIONS(1933),
    [sym_format] = ACTIONS(1933),
    [sym_astro_clock] = ACTIONS(1933),
    [sym_set_outdoor_temperature] = ACTIONS(1933),
    [sym_set_virtual_level_count] = ACTIONS(1933),
    [sym_set_virtual_channel_count] = ACTIONS(1933),
    [sym_set_virtual_port_count] = ACTIONS(1933),
    [sym__wc_encode_int] = ACTIONS(1933),
    [sym__wc_decode_int] = ACTIONS(1933),
    [sym__wc_int] = ACTIONS(1933),
    [sym__wc_to_ch_int] = ACTIONS(1933),
    [sym__ch_to_wc_int] = ACTIONS(1933),
    [sym__wc_find_string_int] = ACTIONS(1933),
    [sym__wc_left_string_int] = ACTIONS(1933),
    [sym__wc_lower_string_int] = ACTIONS(1933),
    [sym__wc_mid_string_int] = ACTIONS(1933),
    [sym__wc_remove_string_int] = ACTIONS(1933),
    [sym__wc_right_string_int] = ACTIONS(1933),
    [sym__wc_upper_string_int] = ACTIONS(1933),
    [sym__wc_compare_string_int] = ACTIONS(1933),
    [sym__wc_get_buffer_char_int] = ACTIONS(1933),
    [sym__wc_get_buffer_string_int] = ACTIONS(1933),
    [sym__wc_concat_string_int] = ACTIONS(1933),
    [sym__wc_file_open_int] = ACTIONS(1933),
    [sym__wc_file_close_int] = ACTIONS(1933),
    [sym__wc_file_read_int] = ACTIONS(1933),
    [sym__wc_file_read_line_int] = ACTIONS(1933),
    [sym__wc_file_write_int] = ACTIONS(1933),
    [sym__wc_file_write_line_int] = ACTIONS(1933),
    [aux_sym_preproc_include_token1] = ACTIONS(1933),
    [aux_sym_preproc_define_token1] = ACTIONS(1935),
    [aux_sym_preproc_warn_token1] = ACTIONS(1935),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1935),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1935),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1935),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1935),
    [aux_sym_preproc_else_token1] = ACTIONS(1935),
    [anon_sym_LBRACK] = ACTIONS(1935),
    [anon_sym_LBRACE] = ACTIONS(1935),
    [anon_sym_LPAREN] = ACTIONS(1933),
    [anon_sym_BANG] = ACTIONS(1935),
    [anon_sym_TILDE] = ACTIONS(1935),
    [anon_sym_DASH] = ACTIONS(1933),
    [anon_sym_PLUS] = ACTIONS(1933),
    [anon_sym_DASH_DASH] = ACTIONS(1935),
    [anon_sym_PLUS_PLUS] = ACTIONS(1935),
    [anon_sym_DQUOTE] = ACTIONS(1935),
    [anon_sym_SQUOTE] = ACTIONS(1935),
    [sym_number_literal] = ACTIONS(1935),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(284)] = {
    [ts_builtin_sym_end] = ACTIONS(1821),
    [sym_identifier] = ACTIONS(1819),
    [sym_program_name_keyword] = ACTIONS(1819),
    [sym_module_name_keyword] = ACTIONS(1819),
    [sym_define_device_keyword] = ACTIONS(1819),
    [sym_define_combine_keyword] = ACTIONS(1819),
    [sym_define_constant_keyword] = ACTIONS(1819),
    [sym_define_type_keyword] = ACTIONS(1819),
    [sym_define_variable_keyword] = ACTIONS(1819),
    [sym_define_system_variable_keyword] = ACTIONS(1819),
    [sym_define_connect_level_keyword] = ACTIONS(1819),
    [sym_define_latching_keyword] = ACTIONS(1819),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1819),
    [sym_define_toggling_keyword] = ACTIONS(1819),
    [sym_define_call_keyword] = ACTIONS(1819),
    [sym_define_function_keyword] = ACTIONS(1819),
    [sym_define_library_function_keyword] = ACTIONS(1819),
    [sym_define_start_keyword] = ACTIONS(1819),
    [sym_define_event_keyword] = ACTIONS(1819),
    [sym_define_program_keyword] = ACTIONS(1819),
    [sym_define_module_keyword] = ACTIONS(1819),
    [sym_char_keyword] = ACTIONS(1819),
    [sym_widechar_keyword] = ACTIONS(1819),
    [sym_integer_keyword] = ACTIONS(1819),
    [sym_sinteger_keyword] = ACTIONS(1819),
    [sym_long_keyword] = ACTIONS(1819),
    [sym_slong_keyword] = ACTIONS(1819),
    [sym_float_keyword] = ACTIONS(1819),
    [sym_double_keyword] = ACTIONS(1819),
    [sym_variant_keyword] = ACTIONS(1819),
    [sym_variantarray_keyword] = ACTIONS(1819),
    [sym_local_var_keyword] = ACTIONS(1819),
    [sym_stack_var_keyword] = ACTIONS(1819),
    [sym_constant_keyword] = ACTIONS(1819),
    [sym_volatile_keyword] = ACTIONS(1819),
    [sym_non_volatile_keyword] = ACTIONS(1819),
    [sym_persistent_keyword] = ACTIONS(1819),
    [sym_dev_keyword] = ACTIONS(1819),
    [sym_devlev_keyword] = ACTIONS(1819),
    [sym_devchan_keyword] = ACTIONS(1819),
    [sym_if_keyword] = ACTIONS(1819),
    [sym_else_keyword] = ACTIONS(1819),
    [sym_switch_keyword] = ACTIONS(1819),
    [sym_case_keyword] = ACTIONS(1819),
    [sym_default_keyword] = ACTIONS(1819),
    [sym_while_keyword] = ACTIONS(1819),
    [sym_for_keyword] = ACTIONS(1819),
    [sym_break_keyword] = ACTIONS(1819),
    [sym_continue_keyword] = ACTIONS(1819),
    [sym_return_keyword] = ACTIONS(1819),
    [sym_select_keyword] = ACTIONS(1819),
    [sym_active_keyword] = ACTIONS(1819),
    [sym_struct_keyword] = ACTIONS(1819),
    [sym_structure_keyword] = ACTIONS(1819),
    [sym_button_event_keyword] = ACTIONS(1819),
    [sym_channel_event_keyword] = ACTIONS(1819),
    [sym_level_event_keyword] = ACTIONS(1819),
    [sym_data_event_keyword] = ACTIONS(1819),
    [sym_timeline_event_keyword] = ACTIONS(1819),
    [sym_custom_event_keyword] = ACTIONS(1819),
    [sym_push_keyword] = ACTIONS(1819),
    [sym_release_keyword] = ACTIONS(1819),
    [aux_sym_on_keyword_token1] = ACTIONS(1819),
    [aux_sym_off_keyword_token1] = ACTIONS(1819),
    [sym_send_level_keyword] = ACTIONS(1819),
    [sym_send_string_keyword] = ACTIONS(1819),
    [sym_send_command_keyword] = ACTIONS(1819),
    [sym_clear_buffer_keyword] = ACTIONS(1819),
    [sym_create_buffer_keyword] = ACTIONS(1819),
    [sym_create_multi_buffer_keyword] = ACTIONS(1819),
    [sym_call_keyword] = ACTIONS(1819),
    [sym_system_call_keyword] = ACTIONS(1819),
    [sym_devchan_to_keyword] = ACTIONS(1819),
    [sym_devchan_min_to_keyword] = ACTIONS(1819),
    [sym_devchan_total_off_keyword] = ACTIONS(1819),
    [sym_devchan_pulse_keyword] = ACTIONS(1819),
    [sym_wait_keyword] = ACTIONS(1819),
    [sym_cancel_wait_keyword] = ACTIONS(1819),
    [sym_wait_until_keyword] = ACTIONS(1819),
    [sym_cancel_wait_until_keyword] = ACTIONS(1819),
    [sym_cancel_all_wait_keyword] = ACTIONS(1819),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1819),
    [sym_not] = ACTIONS(1819),
    [sym_bnot] = ACTIONS(1819),
    [sym___file__] = ACTIONS(1819),
    [sym___date__] = ACTIONS(1819),
    [sym___ldate__] = ACTIONS(1819),
    [sym___time__] = ACTIONS(1819),
    [sym___line__] = ACTIONS(1819),
    [sym___name__] = ACTIONS(1819),
    [sym___netlinx__] = ACTIONS(1819),
    [sym_netlinx_axi_version] = ACTIONS(1819),
    [sym_url_flg_tcp] = ACTIONS(1819),
    [sym_url_flg_accinfopresent] = ACTIONS(1819),
    [sym_url_flg_temp] = ACTIONS(1819),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1819),
    [sym_url_flg_stat_mask] = ACTIONS(1819),
    [sym_url_flg_stat_lookup] = ACTIONS(1819),
    [sym_url_flg_stat_connecting] = ACTIONS(1819),
    [sym_url_flg_stat_waiting] = ACTIONS(1819),
    [sym_url_flg_stat_connected] = ACTIONS(1819),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1819),
    [sym_first_virtual_device] = ACTIONS(1819),
    [sym_first_local_port] = ACTIONS(1819),
    [sym_dynamic_virtual_device] = ACTIONS(1819),
    [sym_timeline_once] = ACTIONS(1819),
    [sym_timeline_repeat] = ACTIONS(1819),
    [sym_timeline_absolute] = ACTIONS(1819),
    [sym_timeline_relative] = ACTIONS(1819),
    [sym_do_push_timed_infinite] = ACTIONS(1819),
    [sym_source_type_no_address] = ACTIONS(1819),
    [sym_source_type_neuron_id] = ACTIONS(1819),
    [sym_source_type_ip_address] = ACTIONS(1819),
    [sym_source_type_axlink] = ACTIONS(1819),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1819),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1819),
    [sym_source_type_ip_socket_address] = ACTIONS(1819),
    [sym_source_type_rs232] = ACTIONS(1819),
    [sym_source_type_internal] = ACTIONS(1819),
    [sym_source_type_mac_address] = ACTIONS(1819),
    [sym_source_type_ipv4_port] = ACTIONS(1819),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1819),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1819),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1819),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1819),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1819),
    [sym_true] = ACTIONS(1819),
    [sym_false] = ACTIONS(1819),
    [sym_file_read_only] = ACTIONS(1819),
    [sym_file_rw_new] = ACTIONS(1819),
    [sym_file_rw_append] = ACTIONS(1819),
    [sym_ip_tcp] = ACTIONS(1819),
    [sym_ip_udp] = ACTIONS(1819),
    [sym_ip_udp_2way] = ACTIONS(1819),
    [sym_xml_encode_types] = ACTIONS(1819),
    [sym_xml_encode_char_as_list] = ACTIONS(1819),
    [sym_xml_encode_le] = ACTIONS(1819),
    [sym_xml_decode_types] = ACTIONS(1819),
    [sym_xml_decode_no_preserve] = ACTIONS(1819),
    [sym_duet_dev_type_display_device] = ACTIONS(1819),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1819),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1819),
    [sym_duet_dev_type_io_device] = ACTIONS(1819),
    [sym_duet_dev_type_relay_device] = ACTIONS(1819),
    [sym_duet_dev_type_ups] = ACTIONS(1819),
    [sym_duet_dev_type_amplifier] = ACTIONS(1819),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1819),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1819),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1819),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1819),
    [sym_duet_dev_type_camera] = ACTIONS(1819),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1819),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1819),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1819),
    [sym_duet_dev_type_dss] = ACTIONS(1819),
    [sym_duet_dev_type_dvr] = ACTIONS(1819),
    [sym_duet_dev_type_disc_device] = ACTIONS(1819),
    [sym_duet_dev_type_document_camera] = ACTIONS(1819),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1819),
    [sym_duet_dev_type_hvac] = ACTIONS(1819),
    [sym_duet_dev_type_keypad] = ACTIONS(1819),
    [sym_duet_dev_type_light] = ACTIONS(1819),
    [sym_duet_dev_type_monitor] = ACTIONS(1819),
    [sym_duet_dev_type_motor] = ACTIONS(1819),
    [sym_duet_dev_type_multi_window] = ACTIONS(1819),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1819),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1819),
    [sym_duet_dev_type_receiver] = ACTIONS(1819),
    [sym_duet_dev_type_security_system] = ACTIONS(1819),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1819),
    [sym_duet_dev_type_settop_box] = ACTIONS(1819),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1819),
    [sym_duet_dev_type_switcher] = ACTIONS(1819),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1819),
    [sym_duet_dev_type_tv] = ACTIONS(1819),
    [sym_duet_dev_type_utility] = ACTIONS(1819),
    [sym_duet_dev_type_vcr] = ACTIONS(1819),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1819),
    [sym_duet_dev_type_video_processor] = ACTIONS(1819),
    [sym_duet_dev_type_video_projector] = ACTIONS(1819),
    [sym_duet_dev_type_video_wall] = ACTIONS(1819),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1819),
    [sym_duet_dev_type_weather] = ACTIONS(1819),
    [sym_duet_dev_not_polled] = ACTIONS(1819),
    [sym_duet_dev_polled] = ACTIONS(1819),
    [sym_clkmgr_mode_network] = ACTIONS(1819),
    [sym_clkmgr_mode_standalone] = ACTIONS(1819),
    [sym_normal_standby] = ACTIONS(1819),
    [sym_normal_wake] = ACTIONS(1819),
    [sym_ip_multicast_ttl_option] = ACTIONS(1819),
    [sym_ip_ttl_subnet] = ACTIONS(1819),
    [sym_ip_ttl_site] = ACTIONS(1819),
    [sym_ip_ttl_region] = ACTIONS(1819),
    [sym_ip_ttl_continent] = ACTIONS(1819),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1819),
    [sym_ip_nodelay_on] = ACTIONS(1819),
    [sym_ip_nodelay_off] = ACTIONS(1819),
    [sym_amx_error] = ACTIONS(1819),
    [sym_amx_warning] = ACTIONS(1819),
    [sym_amx_info] = ACTIONS(1819),
    [sym_amx_debug] = ACTIONS(1819),
    [sym_netlinx_login_success] = ACTIONS(1819),
    [sym_netlinx_login_fail] = ACTIONS(1819),
    [sym_netlinx_logout] = ACTIONS(1819),
    [sym_null_str] = ACTIONS(1819),
    [sym_smtp_address] = ACTIONS(1819),
    [sym_smtp_port_number] = ACTIONS(1819),
    [sym_smtp_username] = ACTIONS(1819),
    [sym_smtp_password] = ACTIONS(1819),
    [sym_smtp_require_tls] = ACTIONS(1819),
    [sym_smtp_from] = ACTIONS(1819),
    [sym_smtp_tls_true] = ACTIONS(1819),
    [sym_smtp_tls_false] = ACTIONS(1819),
    [sym_listview_on_row_select_event] = ACTIONS(1819),
    [sym_data_structure_unknown] = ACTIONS(1819),
    [sym_data_structure_datafeed] = ACTIONS(1819),
    [sym_data_structure_datarecordset] = ACTIONS(1819),
    [sym_data_structure_datarecord] = ACTIONS(1819),
    [sym_data_structure_datafield] = ACTIONS(1819),
    [sym_data_max_content_field_count] = ACTIONS(1819),
    [sym_data_max_metadata_field_count] = ACTIONS(1819),
    [sym_data_max_id_length] = ACTIONS(1819),
    [sym_data_max_name_length] = ACTIONS(1819),
    [sym_data_max_description_length] = ACTIONS(1819),
    [sym_data_max_type_length] = ACTIONS(1819),
    [sym_data_max_format_length] = ACTIONS(1819),
    [sym_data_max_source_length] = ACTIONS(1819),
    [sym_data_max_value_length] = ACTIONS(1819),
    [sym_data_max_label_length] = ACTIONS(1819),
    [sym_data_max_url_length] = ACTIONS(1819),
    [sym_max_last_login_info_length] = ACTIONS(1819),
    [sym_data_type_unknown] = ACTIONS(1819),
    [sym_data_type_string] = ACTIONS(1819),
    [sym_data_type_datetime] = ACTIONS(1819),
    [sym_data_type_date] = ACTIONS(1819),
    [sym_data_type_time] = ACTIONS(1819),
    [sym_data_type_image] = ACTIONS(1819),
    [sym_data_format_url] = ACTIONS(1819),
    [sym_data_format_phone] = ACTIONS(1819),
    [sym_data_format_email] = ACTIONS(1819),
    [sym_data_format_iso8601] = ACTIONS(1819),
    [sym_auditlog_priv] = ACTIONS(1819),
    [sym_configuration_priv] = ACTIONS(1819),
    [sym_deviceconfig_priv] = ACTIONS(1819),
    [sym_ftp_priv] = ACTIONS(1819),
    [sym_http_priv] = ACTIONS(1819),
    [sym_networkconfig_priv] = ACTIONS(1819),
    [sym_programport_priv] = ACTIONS(1819),
    [sym_remoteui_priv] = ACTIONS(1819),
    [sym_securitycontrol_priv] = ACTIONS(1819),
    [sym_softwaremanagement_priv] = ACTIONS(1819),
    [sym_terminal_priv] = ACTIONS(1819),
    [sym_tpadmin_priv] = ACTIONS(1819),
    [sym_user1_priv] = ACTIONS(1819),
    [sym_user2_priv] = ACTIONS(1819),
    [sym_user3_priv] = ACTIONS(1819),
    [sym_user4_priv] = ACTIONS(1819),
    [sym_usermanagement_priv] = ACTIONS(1819),
    [sym_valid_account] = ACTIONS(1819),
    [sym_err_name_invalid] = ACTIONS(1819),
    [sym_err_password_invalid] = ACTIONS(1819),
    [sym_err_invalid_account] = ACTIONS(1819),
    [sym_err_no_authorization] = ACTIONS(1819),
    [sym_err_invalid_parameter] = ACTIONS(1819),
    [sym_err_user_locked_out] = ACTIONS(1819),
    [sym_err_user_account_expired] = ACTIONS(1819),
    [sym_err_general_error] = ACTIONS(1819),
    [sym_err_authentication_error] = ACTIONS(1819),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1819),
    [sym_tls_validate_certificate] = ACTIONS(1819),
    [sym_tdata] = ACTIONS(1819),
    [sym_tchannel] = ACTIONS(1819),
    [sym_tlevel] = ACTIONS(1819),
    [sym_tbutton] = ACTIONS(1819),
    [sym_ttimeline] = ACTIONS(1819),
    [sym_tcustom] = ACTIONS(1819),
    [sym_url_struct] = ACTIONS(1819),
    [sym_dns_struct] = ACTIONS(1819),
    [sym_ip_address_struct] = ACTIONS(1819),
    [sym_dev_info_struct] = ACTIONS(1819),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1819),
    [sym_data_feed] = ACTIONS(1819),
    [sym_data_field] = ACTIONS(1819),
    [sym_data_record] = ACTIONS(1819),
    [sym_wc_data_feed] = ACTIONS(1819),
    [sym_wc_data_field] = ACTIONS(1819),
    [sym_wc_data_record] = ACTIONS(1819),
    [sym_last_login_info] = ACTIONS(1819),
    [sym____reserved___] = ACTIONS(1819),
    [sym_date] = ACTIONS(1819),
    [sym_day] = ACTIONS(1819),
    [sym_time] = ACTIONS(1819),
    [sym_ldate] = ACTIONS(1819),
    [sym_push_device] = ACTIONS(1819),
    [sym_push_channel] = ACTIONS(1819),
    [sym_push_devchan] = ACTIONS(1819),
    [sym_release_device] = ACTIONS(1819),
    [sym_release_channel] = ACTIONS(1819),
    [sym_release_devchan] = ACTIONS(1819),
    [sym_master_slot] = ACTIONS(1819),
    [sym_get_pulse_time] = ACTIONS(1819),
    [sym_get_timer] = ACTIONS(1819),
    [sym_master_sn] = ACTIONS(1819),
    [sym_system_number] = ACTIONS(1819),
    [sym_button] = ACTIONS(1819),
    [sym_dv_channel] = ACTIONS(1819),
    [sym_data] = ACTIONS(1819),
    [sym_level] = ACTIONS(1819),
    [sym_timeline] = ACTIONS(1819),
    [sym_channel] = ACTIONS(1819),
    [sym_custom] = ACTIONS(1819),
    [sym_atoi] = ACTIONS(1819),
    [sym_atol] = ACTIONS(1819),
    [sym_atof] = ACTIONS(1819),
    [sym_itoa] = ACTIONS(1819),
    [sym_itohex] = ACTIONS(1819),
    [sym_hextoi] = ACTIONS(1819),
    [sym_ftoa] = ACTIONS(1819),
    [sym_find_string] = ACTIONS(1819),
    [sym_left_string] = ACTIONS(1819),
    [sym_right_string] = ACTIONS(1819),
    [sym_mid_string] = ACTIONS(1819),
    [sym_remove_string] = ACTIONS(1819),
    [sym_get_buffer_string] = ACTIONS(1819),
    [sym_get_buffer_char] = ACTIONS(1819),
    [sym_get_multi_buffer_string] = ACTIONS(1819),
    [sym_length_string] = ACTIONS(1819),
    [sym_max_length_string] = ACTIONS(1819),
    [sym_set_length_string] = ACTIONS(1819),
    [sym_compare_string] = ACTIONS(1819),
    [sym_upper_string] = ACTIONS(1819),
    [sym_lower_string] = ACTIONS(1819),
    [sym_get_last] = ACTIONS(1819),
    [sym_date_to_day] = ACTIONS(1819),
    [sym_date_to_month] = ACTIONS(1819),
    [sym_date_to_year] = ACTIONS(1819),
    [sym_time_to_hour] = ACTIONS(1819),
    [sym_time_to_minute] = ACTIONS(1819),
    [sym_time_to_second] = ACTIONS(1819),
    [sym_day_of_week] = ACTIONS(1819),
    [sym_set_pulse_time] = ACTIONS(1819),
    [sym_set_timer] = ACTIONS(1819),
    [sym_file_open] = ACTIONS(1819),
    [sym_file_close] = ACTIONS(1819),
    [sym_file_read] = ACTIONS(1819),
    [sym_file_read_line] = ACTIONS(1819),
    [sym_file_write] = ACTIONS(1819),
    [sym_file_write_line] = ACTIONS(1819),
    [sym_file_copy] = ACTIONS(1819),
    [sym_file_delete] = ACTIONS(1819),
    [sym_file_rename] = ACTIONS(1819),
    [sym_file_dir] = ACTIONS(1819),
    [sym_file_getdir] = ACTIONS(1819),
    [sym_file_setdir] = ACTIONS(1819),
    [sym_file_create_dir] = ACTIONS(1819),
    [sym_file_remove_dir] = ACTIONS(1819),
    [sym_file_seek] = ACTIONS(1819),
    [sym_length_array] = ACTIONS(1819),
    [sym_max_length_array] = ACTIONS(1819),
    [sym_set_length_array] = ACTIONS(1819),
    [sym_ip_client_open] = ACTIONS(1819),
    [sym_ip_client_close] = ACTIONS(1819),
    [sym_ip_server_open] = ACTIONS(1819),
    [sym_ip_server_close] = ACTIONS(1819),
    [sym_ip_bound_client_open] = ACTIONS(1819),
    [sym_ip_mc_server_open] = ACTIONS(1819),
    [sym_ip_set_option] = ACTIONS(1819),
    [sym_ssh_client_open] = ACTIONS(1819),
    [sym_ssh_client_close] = ACTIONS(1819),
    [sym_tls_client_open] = ACTIONS(1819),
    [sym_tls_client_close] = ACTIONS(1819),
    [sym_do_push] = ACTIONS(1819),
    [sym_do_push_timed] = ACTIONS(1819),
    [sym_do_release] = ACTIONS(1819),
    [sym_device_id] = ACTIONS(1819),
    [sym_device_info] = ACTIONS(1819),
    [sym_device_id_string] = ACTIONS(1819),
    [sym_device_standby] = ACTIONS(1819),
    [sym_device_wake] = ACTIONS(1819),
    [sym_timeline_create] = ACTIONS(1819),
    [sym_timeline_kill] = ACTIONS(1819),
    [sym_timeline_reload] = ACTIONS(1819),
    [sym_timeline_pause] = ACTIONS(1819),
    [sym_timeline_restart] = ACTIONS(1819),
    [sym_timeline_set] = ACTIONS(1819),
    [sym_timeline_get] = ACTIONS(1819),
    [sym_timeline_active] = ACTIONS(1819),
    [sym_timeline_dynamic_id] = ACTIONS(1819),
    [sym_variable_to_string] = ACTIONS(1819),
    [sym_string_to_variable] = ACTIONS(1819),
    [sym_length_variable_to_string] = ACTIONS(1819),
    [sym_type_cast] = ACTIONS(1819),
    [sym_variable_to_xml] = ACTIONS(1819),
    [sym_xml_to_variable] = ACTIONS(1819),
    [sym_length_variable_to_xml] = ACTIONS(1819),
    [sym_redirect_string] = ACTIONS(1819),
    [sym_random_number] = ACTIONS(1819),
    [sym_get_unique_id] = ACTIONS(1819),
    [sym_get_system_number] = ACTIONS(1819),
    [sym_set_system_number] = ACTIONS(1819),
    [sym_get_serial_number] = ACTIONS(1819),
    [sym_get_master_build] = ACTIONS(1819),
    [sym_set_validation_code] = ACTIONS(1819),
    [sym_reboot] = ACTIONS(1819),
    [sym_load_duet_module] = ACTIONS(1819),
    [sym_unload_duet_module] = ACTIONS(1819),
    [sym_duet_mem_size_set] = ACTIONS(1819),
    [sym_duet_mem_size_get] = ACTIONS(1819),
    [sym_dynamic_application_device] = ACTIONS(1819),
    [sym_dynamic_polled_port] = ACTIONS(1819),
    [sym_static_port_binding] = ACTIONS(1819),
    [sym_static_ip_binding] = ACTIONS(1819),
    [sym_get_dns_list] = ACTIONS(1819),
    [sym_set_dns_list] = ACTIONS(1819),
    [sym_get_ip_address] = ACTIONS(1819),
    [sym_set_ip_address] = ACTIONS(1819),
    [sym_get_url_list] = ACTIONS(1819),
    [sym_add_url_entry] = ACTIONS(1819),
    [sym_delete_url_entry] = ACTIONS(1819),
    [sym_abs_value] = ACTIONS(1819),
    [sym_max_value] = ACTIONS(1819),
    [sym_min_value] = ACTIONS(1819),
    [sym_exp_value] = ACTIONS(1819),
    [sym_log_value] = ACTIONS(1819),
    [sym_log10_value] = ACTIONS(1819),
    [sym_power_value] = ACTIONS(1819),
    [sym_sqrt_value] = ACTIONS(1819),
    [sym_raw_be] = ACTIONS(1819),
    [sym_raw_le] = ACTIONS(1819),
    [sym_internal_queue_size_set] = ACTIONS(1819),
    [sym_internal_queue_size_get] = ACTIONS(1819),
    [sym_internal_threshold_set] = ACTIONS(1819),
    [sym_internal_threshold_get] = ACTIONS(1819),
    [sym_rebuilt_event] = ACTIONS(1819),
    [sym_do_custom_event] = ACTIONS(1819),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1819),
    [sym_clkmgr_set_clk_source] = ACTIONS(1819),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1819),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1819),
    [sym_clkmgr_get_timezone] = ACTIONS(1819),
    [sym_clkmgr_set_timezone] = ACTIONS(1819),
    [sym_clkmgr_get_resync_period] = ACTIONS(1819),
    [sym_clkmgr_set_resync_period] = ACTIONS(1819),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1819),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1819),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1819),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1819),
    [sym_clkmgr_get_timeservers] = ACTIONS(1819),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1819),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1819),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1819),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1819),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1819),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1819),
    [sym_set_log_level] = ACTIONS(1819),
    [sym_get_log_level] = ACTIONS(1819),
    [sym_amx_log] = ACTIONS(1819),
    [sym_smtp_server_config_set] = ACTIONS(1819),
    [sym_smtp_server_config_get] = ACTIONS(1819),
    [sym_smtp_send] = ACTIONS(1819),
    [sym_validate_netlinx_account] = ACTIONS(1819),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1819),
    [sym_authenticate_certificate] = ACTIONS(1819),
    [sym_audit_netlinx_session_event] = ACTIONS(1819),
    [sym_audit_netlinx_generic_event] = ACTIONS(1819),
    [sym_get_available_flash_disk_space] = ACTIONS(1819),
    [sym_get_max_flash_disk_space] = ACTIONS(1819),
    [sym_data_create_feed] = ACTIONS(1819),
    [sym_data_delete_feed] = ACTIONS(1819),
    [sym_data_publish_feed] = ACTIONS(1819),
    [sym_data_get_published_feed] = ACTIONS(1819),
    [sym_data_add_record] = ACTIONS(1819),
    [sym_data_get_event_record] = ACTIONS(1819),
    [sym__wc_data_create_feed] = ACTIONS(1819),
    [sym__wc_data_add_record] = ACTIONS(1819),
    [sym__wc_data_get_event_record] = ACTIONS(1819),
    [sym_format] = ACTIONS(1819),
    [sym_astro_clock] = ACTIONS(1819),
    [sym_set_outdoor_temperature] = ACTIONS(1819),
    [sym_set_virtual_level_count] = ACTIONS(1819),
    [sym_set_virtual_channel_count] = ACTIONS(1819),
    [sym_set_virtual_port_count] = ACTIONS(1819),
    [sym__wc_encode_int] = ACTIONS(1819),
    [sym__wc_decode_int] = ACTIONS(1819),
    [sym__wc_int] = ACTIONS(1819),
    [sym__wc_to_ch_int] = ACTIONS(1819),
    [sym__ch_to_wc_int] = ACTIONS(1819),
    [sym__wc_find_string_int] = ACTIONS(1819),
    [sym__wc_left_string_int] = ACTIONS(1819),
    [sym__wc_lower_string_int] = ACTIONS(1819),
    [sym__wc_mid_string_int] = ACTIONS(1819),
    [sym__wc_remove_string_int] = ACTIONS(1819),
    [sym__wc_right_string_int] = ACTIONS(1819),
    [sym__wc_upper_string_int] = ACTIONS(1819),
    [sym__wc_compare_string_int] = ACTIONS(1819),
    [sym__wc_get_buffer_char_int] = ACTIONS(1819),
    [sym__wc_get_buffer_string_int] = ACTIONS(1819),
    [sym__wc_concat_string_int] = ACTIONS(1819),
    [sym__wc_file_open_int] = ACTIONS(1819),
    [sym__wc_file_close_int] = ACTIONS(1819),
    [sym__wc_file_read_int] = ACTIONS(1819),
    [sym__wc_file_read_line_int] = ACTIONS(1819),
    [sym__wc_file_write_int] = ACTIONS(1819),
    [sym__wc_file_write_line_int] = ACTIONS(1819),
    [aux_sym_preproc_include_token1] = ACTIONS(1819),
    [aux_sym_preproc_define_token1] = ACTIONS(1821),
    [aux_sym_preproc_warn_token1] = ACTIONS(1821),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1821),
    [anon_sym_LBRACK] = ACTIONS(1821),
    [anon_sym_LBRACE] = ACTIONS(1821),
    [anon_sym_LPAREN] = ACTIONS(1819),
    [anon_sym_BANG] = ACTIONS(1821),
    [anon_sym_TILDE] = ACTIONS(1821),
    [anon_sym_DASH] = ACTIONS(1819),
    [anon_sym_PLUS] = ACTIONS(1819),
    [anon_sym_DASH_DASH] = ACTIONS(1821),
    [anon_sym_PLUS_PLUS] = ACTIONS(1821),
    [anon_sym_DQUOTE] = ACTIONS(1821),
    [anon_sym_SQUOTE] = ACTIONS(1821),
    [sym_number_literal] = ACTIONS(1821),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(285)] = {
    [sym_identifier] = ACTIONS(1937),
    [sym_program_name_keyword] = ACTIONS(1937),
    [sym_module_name_keyword] = ACTIONS(1937),
    [sym_define_device_keyword] = ACTIONS(1937),
    [sym_define_combine_keyword] = ACTIONS(1937),
    [sym_define_constant_keyword] = ACTIONS(1937),
    [sym_define_type_keyword] = ACTIONS(1937),
    [sym_define_variable_keyword] = ACTIONS(1937),
    [sym_define_system_variable_keyword] = ACTIONS(1937),
    [sym_define_connect_level_keyword] = ACTIONS(1937),
    [sym_define_latching_keyword] = ACTIONS(1937),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1937),
    [sym_define_toggling_keyword] = ACTIONS(1937),
    [sym_define_call_keyword] = ACTIONS(1937),
    [sym_define_function_keyword] = ACTIONS(1937),
    [sym_define_library_function_keyword] = ACTIONS(1937),
    [sym_define_start_keyword] = ACTIONS(1937),
    [sym_define_event_keyword] = ACTIONS(1937),
    [sym_define_program_keyword] = ACTIONS(1937),
    [sym_define_module_keyword] = ACTIONS(1937),
    [sym_char_keyword] = ACTIONS(1937),
    [sym_widechar_keyword] = ACTIONS(1937),
    [sym_integer_keyword] = ACTIONS(1937),
    [sym_sinteger_keyword] = ACTIONS(1937),
    [sym_long_keyword] = ACTIONS(1937),
    [sym_slong_keyword] = ACTIONS(1937),
    [sym_float_keyword] = ACTIONS(1937),
    [sym_double_keyword] = ACTIONS(1937),
    [sym_variant_keyword] = ACTIONS(1937),
    [sym_variantarray_keyword] = ACTIONS(1937),
    [sym_local_var_keyword] = ACTIONS(1937),
    [sym_stack_var_keyword] = ACTIONS(1937),
    [sym_constant_keyword] = ACTIONS(1937),
    [sym_volatile_keyword] = ACTIONS(1937),
    [sym_non_volatile_keyword] = ACTIONS(1937),
    [sym_persistent_keyword] = ACTIONS(1937),
    [sym_dev_keyword] = ACTIONS(1937),
    [sym_devlev_keyword] = ACTIONS(1937),
    [sym_devchan_keyword] = ACTIONS(1937),
    [sym_if_keyword] = ACTIONS(1937),
    [sym_switch_keyword] = ACTIONS(1937),
    [sym_case_keyword] = ACTIONS(1937),
    [sym_default_keyword] = ACTIONS(1937),
    [sym_while_keyword] = ACTIONS(1937),
    [sym_for_keyword] = ACTIONS(1937),
    [sym_break_keyword] = ACTIONS(1937),
    [sym_continue_keyword] = ACTIONS(1937),
    [sym_return_keyword] = ACTIONS(1937),
    [sym_select_keyword] = ACTIONS(1937),
    [sym_active_keyword] = ACTIONS(1937),
    [sym_struct_keyword] = ACTIONS(1937),
    [sym_structure_keyword] = ACTIONS(1937),
    [sym_button_event_keyword] = ACTIONS(1937),
    [sym_channel_event_keyword] = ACTIONS(1937),
    [sym_level_event_keyword] = ACTIONS(1937),
    [sym_data_event_keyword] = ACTIONS(1937),
    [sym_timeline_event_keyword] = ACTIONS(1937),
    [sym_custom_event_keyword] = ACTIONS(1937),
    [sym_push_keyword] = ACTIONS(1937),
    [sym_release_keyword] = ACTIONS(1937),
    [aux_sym_on_keyword_token1] = ACTIONS(1937),
    [aux_sym_off_keyword_token1] = ACTIONS(1937),
    [sym_send_level_keyword] = ACTIONS(1937),
    [sym_send_string_keyword] = ACTIONS(1937),
    [sym_send_command_keyword] = ACTIONS(1937),
    [sym_clear_buffer_keyword] = ACTIONS(1937),
    [sym_create_buffer_keyword] = ACTIONS(1937),
    [sym_create_multi_buffer_keyword] = ACTIONS(1937),
    [sym_call_keyword] = ACTIONS(1937),
    [sym_system_call_keyword] = ACTIONS(1937),
    [sym_devchan_to_keyword] = ACTIONS(1937),
    [sym_devchan_min_to_keyword] = ACTIONS(1937),
    [sym_devchan_total_off_keyword] = ACTIONS(1937),
    [sym_devchan_pulse_keyword] = ACTIONS(1937),
    [sym_wait_keyword] = ACTIONS(1937),
    [sym_cancel_wait_keyword] = ACTIONS(1937),
    [sym_wait_until_keyword] = ACTIONS(1937),
    [sym_cancel_wait_until_keyword] = ACTIONS(1937),
    [sym_cancel_all_wait_keyword] = ACTIONS(1937),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1937),
    [sym_not] = ACTIONS(1937),
    [sym_bnot] = ACTIONS(1937),
    [sym___file__] = ACTIONS(1937),
    [sym___date__] = ACTIONS(1937),
    [sym___ldate__] = ACTIONS(1937),
    [sym___time__] = ACTIONS(1937),
    [sym___line__] = ACTIONS(1937),
    [sym___name__] = ACTIONS(1937),
    [sym___netlinx__] = ACTIONS(1937),
    [sym_netlinx_axi_version] = ACTIONS(1937),
    [sym_url_flg_tcp] = ACTIONS(1937),
    [sym_url_flg_accinfopresent] = ACTIONS(1937),
    [sym_url_flg_temp] = ACTIONS(1937),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1937),
    [sym_url_flg_stat_mask] = ACTIONS(1937),
    [sym_url_flg_stat_lookup] = ACTIONS(1937),
    [sym_url_flg_stat_connecting] = ACTIONS(1937),
    [sym_url_flg_stat_waiting] = ACTIONS(1937),
    [sym_url_flg_stat_connected] = ACTIONS(1937),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1937),
    [sym_first_virtual_device] = ACTIONS(1937),
    [sym_first_local_port] = ACTIONS(1937),
    [sym_dynamic_virtual_device] = ACTIONS(1937),
    [sym_timeline_once] = ACTIONS(1937),
    [sym_timeline_repeat] = ACTIONS(1937),
    [sym_timeline_absolute] = ACTIONS(1937),
    [sym_timeline_relative] = ACTIONS(1937),
    [sym_do_push_timed_infinite] = ACTIONS(1937),
    [sym_source_type_no_address] = ACTIONS(1937),
    [sym_source_type_neuron_id] = ACTIONS(1937),
    [sym_source_type_ip_address] = ACTIONS(1937),
    [sym_source_type_axlink] = ACTIONS(1937),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1937),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1937),
    [sym_source_type_ip_socket_address] = ACTIONS(1937),
    [sym_source_type_rs232] = ACTIONS(1937),
    [sym_source_type_internal] = ACTIONS(1937),
    [sym_source_type_mac_address] = ACTIONS(1937),
    [sym_source_type_ipv4_port] = ACTIONS(1937),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1937),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1937),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1937),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1937),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1937),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1937),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1937),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1937),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1937),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1937),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1937),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1937),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1937),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1937),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1937),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1937),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1937),
    [sym_true] = ACTIONS(1937),
    [sym_false] = ACTIONS(1937),
    [sym_file_read_only] = ACTIONS(1937),
    [sym_file_rw_new] = ACTIONS(1937),
    [sym_file_rw_append] = ACTIONS(1937),
    [sym_ip_tcp] = ACTIONS(1937),
    [sym_ip_udp] = ACTIONS(1937),
    [sym_ip_udp_2way] = ACTIONS(1937),
    [sym_xml_encode_types] = ACTIONS(1937),
    [sym_xml_encode_char_as_list] = ACTIONS(1937),
    [sym_xml_encode_le] = ACTIONS(1937),
    [sym_xml_decode_types] = ACTIONS(1937),
    [sym_xml_decode_no_preserve] = ACTIONS(1937),
    [sym_duet_dev_type_display_device] = ACTIONS(1937),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1937),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1937),
    [sym_duet_dev_type_io_device] = ACTIONS(1937),
    [sym_duet_dev_type_relay_device] = ACTIONS(1937),
    [sym_duet_dev_type_ups] = ACTIONS(1937),
    [sym_duet_dev_type_amplifier] = ACTIONS(1937),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1937),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1937),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1937),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1937),
    [sym_duet_dev_type_camera] = ACTIONS(1937),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1937),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1937),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1937),
    [sym_duet_dev_type_dss] = ACTIONS(1937),
    [sym_duet_dev_type_dvr] = ACTIONS(1937),
    [sym_duet_dev_type_disc_device] = ACTIONS(1937),
    [sym_duet_dev_type_document_camera] = ACTIONS(1937),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1937),
    [sym_duet_dev_type_hvac] = ACTIONS(1937),
    [sym_duet_dev_type_keypad] = ACTIONS(1937),
    [sym_duet_dev_type_light] = ACTIONS(1937),
    [sym_duet_dev_type_monitor] = ACTIONS(1937),
    [sym_duet_dev_type_motor] = ACTIONS(1937),
    [sym_duet_dev_type_multi_window] = ACTIONS(1937),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1937),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1937),
    [sym_duet_dev_type_receiver] = ACTIONS(1937),
    [sym_duet_dev_type_security_system] = ACTIONS(1937),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1937),
    [sym_duet_dev_type_settop_box] = ACTIONS(1937),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1937),
    [sym_duet_dev_type_switcher] = ACTIONS(1937),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1937),
    [sym_duet_dev_type_tv] = ACTIONS(1937),
    [sym_duet_dev_type_utility] = ACTIONS(1937),
    [sym_duet_dev_type_vcr] = ACTIONS(1937),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1937),
    [sym_duet_dev_type_video_processor] = ACTIONS(1937),
    [sym_duet_dev_type_video_projector] = ACTIONS(1937),
    [sym_duet_dev_type_video_wall] = ACTIONS(1937),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1937),
    [sym_duet_dev_type_weather] = ACTIONS(1937),
    [sym_duet_dev_not_polled] = ACTIONS(1937),
    [sym_duet_dev_polled] = ACTIONS(1937),
    [sym_clkmgr_mode_network] = ACTIONS(1937),
    [sym_clkmgr_mode_standalone] = ACTIONS(1937),
    [sym_normal_standby] = ACTIONS(1937),
    [sym_normal_wake] = ACTIONS(1937),
    [sym_ip_multicast_ttl_option] = ACTIONS(1937),
    [sym_ip_ttl_subnet] = ACTIONS(1937),
    [sym_ip_ttl_site] = ACTIONS(1937),
    [sym_ip_ttl_region] = ACTIONS(1937),
    [sym_ip_ttl_continent] = ACTIONS(1937),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1937),
    [sym_ip_nodelay_on] = ACTIONS(1937),
    [sym_ip_nodelay_off] = ACTIONS(1937),
    [sym_amx_error] = ACTIONS(1937),
    [sym_amx_warning] = ACTIONS(1937),
    [sym_amx_info] = ACTIONS(1937),
    [sym_amx_debug] = ACTIONS(1937),
    [sym_netlinx_login_success] = ACTIONS(1937),
    [sym_netlinx_login_fail] = ACTIONS(1937),
    [sym_netlinx_logout] = ACTIONS(1937),
    [sym_null_str] = ACTIONS(1937),
    [sym_smtp_address] = ACTIONS(1937),
    [sym_smtp_port_number] = ACTIONS(1937),
    [sym_smtp_username] = ACTIONS(1937),
    [sym_smtp_password] = ACTIONS(1937),
    [sym_smtp_require_tls] = ACTIONS(1937),
    [sym_smtp_from] = ACTIONS(1937),
    [sym_smtp_tls_true] = ACTIONS(1937),
    [sym_smtp_tls_false] = ACTIONS(1937),
    [sym_listview_on_row_select_event] = ACTIONS(1937),
    [sym_data_structure_unknown] = ACTIONS(1937),
    [sym_data_structure_datafeed] = ACTIONS(1937),
    [sym_data_structure_datarecordset] = ACTIONS(1937),
    [sym_data_structure_datarecord] = ACTIONS(1937),
    [sym_data_structure_datafield] = ACTIONS(1937),
    [sym_data_max_content_field_count] = ACTIONS(1937),
    [sym_data_max_metadata_field_count] = ACTIONS(1937),
    [sym_data_max_id_length] = ACTIONS(1937),
    [sym_data_max_name_length] = ACTIONS(1937),
    [sym_data_max_description_length] = ACTIONS(1937),
    [sym_data_max_type_length] = ACTIONS(1937),
    [sym_data_max_format_length] = ACTIONS(1937),
    [sym_data_max_source_length] = ACTIONS(1937),
    [sym_data_max_value_length] = ACTIONS(1937),
    [sym_data_max_label_length] = ACTIONS(1937),
    [sym_data_max_url_length] = ACTIONS(1937),
    [sym_max_last_login_info_length] = ACTIONS(1937),
    [sym_data_type_unknown] = ACTIONS(1937),
    [sym_data_type_string] = ACTIONS(1937),
    [sym_data_type_datetime] = ACTIONS(1937),
    [sym_data_type_date] = ACTIONS(1937),
    [sym_data_type_time] = ACTIONS(1937),
    [sym_data_type_image] = ACTIONS(1937),
    [sym_data_format_url] = ACTIONS(1937),
    [sym_data_format_phone] = ACTIONS(1937),
    [sym_data_format_email] = ACTIONS(1937),
    [sym_data_format_iso8601] = ACTIONS(1937),
    [sym_auditlog_priv] = ACTIONS(1937),
    [sym_configuration_priv] = ACTIONS(1937),
    [sym_deviceconfig_priv] = ACTIONS(1937),
    [sym_ftp_priv] = ACTIONS(1937),
    [sym_http_priv] = ACTIONS(1937),
    [sym_networkconfig_priv] = ACTIONS(1937),
    [sym_programport_priv] = ACTIONS(1937),
    [sym_remoteui_priv] = ACTIONS(1937),
    [sym_securitycontrol_priv] = ACTIONS(1937),
    [sym_softwaremanagement_priv] = ACTIONS(1937),
    [sym_terminal_priv] = ACTIONS(1937),
    [sym_tpadmin_priv] = ACTIONS(1937),
    [sym_user1_priv] = ACTIONS(1937),
    [sym_user2_priv] = ACTIONS(1937),
    [sym_user3_priv] = ACTIONS(1937),
    [sym_user4_priv] = ACTIONS(1937),
    [sym_usermanagement_priv] = ACTIONS(1937),
    [sym_valid_account] = ACTIONS(1937),
    [sym_err_name_invalid] = ACTIONS(1937),
    [sym_err_password_invalid] = ACTIONS(1937),
    [sym_err_invalid_account] = ACTIONS(1937),
    [sym_err_no_authorization] = ACTIONS(1937),
    [sym_err_invalid_parameter] = ACTIONS(1937),
    [sym_err_user_locked_out] = ACTIONS(1937),
    [sym_err_user_account_expired] = ACTIONS(1937),
    [sym_err_general_error] = ACTIONS(1937),
    [sym_err_authentication_error] = ACTIONS(1937),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1937),
    [sym_tls_validate_certificate] = ACTIONS(1937),
    [sym_tdata] = ACTIONS(1937),
    [sym_tchannel] = ACTIONS(1937),
    [sym_tlevel] = ACTIONS(1937),
    [sym_tbutton] = ACTIONS(1937),
    [sym_ttimeline] = ACTIONS(1937),
    [sym_tcustom] = ACTIONS(1937),
    [sym_url_struct] = ACTIONS(1937),
    [sym_dns_struct] = ACTIONS(1937),
    [sym_ip_address_struct] = ACTIONS(1937),
    [sym_dev_info_struct] = ACTIONS(1937),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1937),
    [sym_data_feed] = ACTIONS(1937),
    [sym_data_field] = ACTIONS(1937),
    [sym_data_record] = ACTIONS(1937),
    [sym_wc_data_feed] = ACTIONS(1937),
    [sym_wc_data_field] = ACTIONS(1937),
    [sym_wc_data_record] = ACTIONS(1937),
    [sym_last_login_info] = ACTIONS(1937),
    [sym____reserved___] = ACTIONS(1937),
    [sym_date] = ACTIONS(1937),
    [sym_day] = ACTIONS(1937),
    [sym_time] = ACTIONS(1937),
    [sym_ldate] = ACTIONS(1937),
    [sym_push_device] = ACTIONS(1937),
    [sym_push_channel] = ACTIONS(1937),
    [sym_push_devchan] = ACTIONS(1937),
    [sym_release_device] = ACTIONS(1937),
    [sym_release_channel] = ACTIONS(1937),
    [sym_release_devchan] = ACTIONS(1937),
    [sym_master_slot] = ACTIONS(1937),
    [sym_get_pulse_time] = ACTIONS(1937),
    [sym_get_timer] = ACTIONS(1937),
    [sym_master_sn] = ACTIONS(1937),
    [sym_system_number] = ACTIONS(1937),
    [sym_button] = ACTIONS(1937),
    [sym_dv_channel] = ACTIONS(1937),
    [sym_data] = ACTIONS(1937),
    [sym_level] = ACTIONS(1937),
    [sym_timeline] = ACTIONS(1937),
    [sym_channel] = ACTIONS(1937),
    [sym_custom] = ACTIONS(1937),
    [sym_atoi] = ACTIONS(1937),
    [sym_atol] = ACTIONS(1937),
    [sym_atof] = ACTIONS(1937),
    [sym_itoa] = ACTIONS(1937),
    [sym_itohex] = ACTIONS(1937),
    [sym_hextoi] = ACTIONS(1937),
    [sym_ftoa] = ACTIONS(1937),
    [sym_find_string] = ACTIONS(1937),
    [sym_left_string] = ACTIONS(1937),
    [sym_right_string] = ACTIONS(1937),
    [sym_mid_string] = ACTIONS(1937),
    [sym_remove_string] = ACTIONS(1937),
    [sym_get_buffer_string] = ACTIONS(1937),
    [sym_get_buffer_char] = ACTIONS(1937),
    [sym_get_multi_buffer_string] = ACTIONS(1937),
    [sym_length_string] = ACTIONS(1937),
    [sym_max_length_string] = ACTIONS(1937),
    [sym_set_length_string] = ACTIONS(1937),
    [sym_compare_string] = ACTIONS(1937),
    [sym_upper_string] = ACTIONS(1937),
    [sym_lower_string] = ACTIONS(1937),
    [sym_get_last] = ACTIONS(1937),
    [sym_date_to_day] = ACTIONS(1937),
    [sym_date_to_month] = ACTIONS(1937),
    [sym_date_to_year] = ACTIONS(1937),
    [sym_time_to_hour] = ACTIONS(1937),
    [sym_time_to_minute] = ACTIONS(1937),
    [sym_time_to_second] = ACTIONS(1937),
    [sym_day_of_week] = ACTIONS(1937),
    [sym_set_pulse_time] = ACTIONS(1937),
    [sym_set_timer] = ACTIONS(1937),
    [sym_file_open] = ACTIONS(1937),
    [sym_file_close] = ACTIONS(1937),
    [sym_file_read] = ACTIONS(1937),
    [sym_file_read_line] = ACTIONS(1937),
    [sym_file_write] = ACTIONS(1937),
    [sym_file_write_line] = ACTIONS(1937),
    [sym_file_copy] = ACTIONS(1937),
    [sym_file_delete] = ACTIONS(1937),
    [sym_file_rename] = ACTIONS(1937),
    [sym_file_dir] = ACTIONS(1937),
    [sym_file_getdir] = ACTIONS(1937),
    [sym_file_setdir] = ACTIONS(1937),
    [sym_file_create_dir] = ACTIONS(1937),
    [sym_file_remove_dir] = ACTIONS(1937),
    [sym_file_seek] = ACTIONS(1937),
    [sym_length_array] = ACTIONS(1937),
    [sym_max_length_array] = ACTIONS(1937),
    [sym_set_length_array] = ACTIONS(1937),
    [sym_ip_client_open] = ACTIONS(1937),
    [sym_ip_client_close] = ACTIONS(1937),
    [sym_ip_server_open] = ACTIONS(1937),
    [sym_ip_server_close] = ACTIONS(1937),
    [sym_ip_bound_client_open] = ACTIONS(1937),
    [sym_ip_mc_server_open] = ACTIONS(1937),
    [sym_ip_set_option] = ACTIONS(1937),
    [sym_ssh_client_open] = ACTIONS(1937),
    [sym_ssh_client_close] = ACTIONS(1937),
    [sym_tls_client_open] = ACTIONS(1937),
    [sym_tls_client_close] = ACTIONS(1937),
    [sym_do_push] = ACTIONS(1937),
    [sym_do_push_timed] = ACTIONS(1937),
    [sym_do_release] = ACTIONS(1937),
    [sym_device_id] = ACTIONS(1937),
    [sym_device_info] = ACTIONS(1937),
    [sym_device_id_string] = ACTIONS(1937),
    [sym_device_standby] = ACTIONS(1937),
    [sym_device_wake] = ACTIONS(1937),
    [sym_timeline_create] = ACTIONS(1937),
    [sym_timeline_kill] = ACTIONS(1937),
    [sym_timeline_reload] = ACTIONS(1937),
    [sym_timeline_pause] = ACTIONS(1937),
    [sym_timeline_restart] = ACTIONS(1937),
    [sym_timeline_set] = ACTIONS(1937),
    [sym_timeline_get] = ACTIONS(1937),
    [sym_timeline_active] = ACTIONS(1937),
    [sym_timeline_dynamic_id] = ACTIONS(1937),
    [sym_variable_to_string] = ACTIONS(1937),
    [sym_string_to_variable] = ACTIONS(1937),
    [sym_length_variable_to_string] = ACTIONS(1937),
    [sym_type_cast] = ACTIONS(1937),
    [sym_variable_to_xml] = ACTIONS(1937),
    [sym_xml_to_variable] = ACTIONS(1937),
    [sym_length_variable_to_xml] = ACTIONS(1937),
    [sym_redirect_string] = ACTIONS(1937),
    [sym_random_number] = ACTIONS(1937),
    [sym_get_unique_id] = ACTIONS(1937),
    [sym_get_system_number] = ACTIONS(1937),
    [sym_set_system_number] = ACTIONS(1937),
    [sym_get_serial_number] = ACTIONS(1937),
    [sym_get_master_build] = ACTIONS(1937),
    [sym_set_validation_code] = ACTIONS(1937),
    [sym_reboot] = ACTIONS(1937),
    [sym_load_duet_module] = ACTIONS(1937),
    [sym_unload_duet_module] = ACTIONS(1937),
    [sym_duet_mem_size_set] = ACTIONS(1937),
    [sym_duet_mem_size_get] = ACTIONS(1937),
    [sym_dynamic_application_device] = ACTIONS(1937),
    [sym_dynamic_polled_port] = ACTIONS(1937),
    [sym_static_port_binding] = ACTIONS(1937),
    [sym_static_ip_binding] = ACTIONS(1937),
    [sym_get_dns_list] = ACTIONS(1937),
    [sym_set_dns_list] = ACTIONS(1937),
    [sym_get_ip_address] = ACTIONS(1937),
    [sym_set_ip_address] = ACTIONS(1937),
    [sym_get_url_list] = ACTIONS(1937),
    [sym_add_url_entry] = ACTIONS(1937),
    [sym_delete_url_entry] = ACTIONS(1937),
    [sym_abs_value] = ACTIONS(1937),
    [sym_max_value] = ACTIONS(1937),
    [sym_min_value] = ACTIONS(1937),
    [sym_exp_value] = ACTIONS(1937),
    [sym_log_value] = ACTIONS(1937),
    [sym_log10_value] = ACTIONS(1937),
    [sym_power_value] = ACTIONS(1937),
    [sym_sqrt_value] = ACTIONS(1937),
    [sym_raw_be] = ACTIONS(1937),
    [sym_raw_le] = ACTIONS(1937),
    [sym_internal_queue_size_set] = ACTIONS(1937),
    [sym_internal_queue_size_get] = ACTIONS(1937),
    [sym_internal_threshold_set] = ACTIONS(1937),
    [sym_internal_threshold_get] = ACTIONS(1937),
    [sym_rebuilt_event] = ACTIONS(1937),
    [sym_do_custom_event] = ACTIONS(1937),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1937),
    [sym_clkmgr_set_clk_source] = ACTIONS(1937),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1937),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1937),
    [sym_clkmgr_get_timezone] = ACTIONS(1937),
    [sym_clkmgr_set_timezone] = ACTIONS(1937),
    [sym_clkmgr_get_resync_period] = ACTIONS(1937),
    [sym_clkmgr_set_resync_period] = ACTIONS(1937),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1937),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1937),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1937),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1937),
    [sym_clkmgr_get_timeservers] = ACTIONS(1937),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1937),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1937),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1937),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1937),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1937),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1937),
    [sym_set_log_level] = ACTIONS(1937),
    [sym_get_log_level] = ACTIONS(1937),
    [sym_amx_log] = ACTIONS(1937),
    [sym_smtp_server_config_set] = ACTIONS(1937),
    [sym_smtp_server_config_get] = ACTIONS(1937),
    [sym_smtp_send] = ACTIONS(1937),
    [sym_validate_netlinx_account] = ACTIONS(1937),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1937),
    [sym_authenticate_certificate] = ACTIONS(1937),
    [sym_audit_netlinx_session_event] = ACTIONS(1937),
    [sym_audit_netlinx_generic_event] = ACTIONS(1937),
    [sym_get_available_flash_disk_space] = ACTIONS(1937),
    [sym_get_max_flash_disk_space] = ACTIONS(1937),
    [sym_data_create_feed] = ACTIONS(1937),
    [sym_data_delete_feed] = ACTIONS(1937),
    [sym_data_publish_feed] = ACTIONS(1937),
    [sym_data_get_published_feed] = ACTIONS(1937),
    [sym_data_add_record] = ACTIONS(1937),
    [sym_data_get_event_record] = ACTIONS(1937),
    [sym__wc_data_create_feed] = ACTIONS(1937),
    [sym__wc_data_add_record] = ACTIONS(1937),
    [sym__wc_data_get_event_record] = ACTIONS(1937),
    [sym_format] = ACTIONS(1937),
    [sym_astro_clock] = ACTIONS(1937),
    [sym_set_outdoor_temperature] = ACTIONS(1937),
    [sym_set_virtual_level_count] = ACTIONS(1937),
    [sym_set_virtual_channel_count] = ACTIONS(1937),
    [sym_set_virtual_port_count] = ACTIONS(1937),
    [sym__wc_encode_int] = ACTIONS(1937),
    [sym__wc_decode_int] = ACTIONS(1937),
    [sym__wc_int] = ACTIONS(1937),
    [sym__wc_to_ch_int] = ACTIONS(1937),
    [sym__ch_to_wc_int] = ACTIONS(1937),
    [sym__wc_find_string_int] = ACTIONS(1937),
    [sym__wc_left_string_int] = ACTIONS(1937),
    [sym__wc_lower_string_int] = ACTIONS(1937),
    [sym__wc_mid_string_int] = ACTIONS(1937),
    [sym__wc_remove_string_int] = ACTIONS(1937),
    [sym__wc_right_string_int] = ACTIONS(1937),
    [sym__wc_upper_string_int] = ACTIONS(1937),
    [sym__wc_compare_string_int] = ACTIONS(1937),
    [sym__wc_get_buffer_char_int] = ACTIONS(1937),
    [sym__wc_get_buffer_string_int] = ACTIONS(1937),
    [sym__wc_concat_string_int] = ACTIONS(1937),
    [sym__wc_file_open_int] = ACTIONS(1937),
    [sym__wc_file_close_int] = ACTIONS(1937),
    [sym__wc_file_read_int] = ACTIONS(1937),
    [sym__wc_file_read_line_int] = ACTIONS(1937),
    [sym__wc_file_write_int] = ACTIONS(1937),
    [sym__wc_file_write_line_int] = ACTIONS(1937),
    [aux_sym_preproc_include_token1] = ACTIONS(1937),
    [aux_sym_preproc_define_token1] = ACTIONS(1939),
    [aux_sym_preproc_warn_token1] = ACTIONS(1939),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1939),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1939),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1939),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1939),
    [aux_sym_preproc_else_token1] = ACTIONS(1939),
    [anon_sym_LBRACK] = ACTIONS(1939),
    [anon_sym_LBRACE] = ACTIONS(1939),
    [anon_sym_LPAREN] = ACTIONS(1937),
    [anon_sym_BANG] = ACTIONS(1939),
    [anon_sym_TILDE] = ACTIONS(1939),
    [anon_sym_DASH] = ACTIONS(1937),
    [anon_sym_PLUS] = ACTIONS(1937),
    [anon_sym_DASH_DASH] = ACTIONS(1939),
    [anon_sym_PLUS_PLUS] = ACTIONS(1939),
    [anon_sym_DQUOTE] = ACTIONS(1939),
    [anon_sym_SQUOTE] = ACTIONS(1939),
    [sym_number_literal] = ACTIONS(1939),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(286)] = {
    [sym_identifier] = ACTIONS(1941),
    [sym_program_name_keyword] = ACTIONS(1941),
    [sym_module_name_keyword] = ACTIONS(1941),
    [sym_define_device_keyword] = ACTIONS(1941),
    [sym_define_combine_keyword] = ACTIONS(1941),
    [sym_define_constant_keyword] = ACTIONS(1941),
    [sym_define_type_keyword] = ACTIONS(1941),
    [sym_define_variable_keyword] = ACTIONS(1941),
    [sym_define_system_variable_keyword] = ACTIONS(1941),
    [sym_define_connect_level_keyword] = ACTIONS(1941),
    [sym_define_latching_keyword] = ACTIONS(1941),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1941),
    [sym_define_toggling_keyword] = ACTIONS(1941),
    [sym_define_call_keyword] = ACTIONS(1941),
    [sym_define_function_keyword] = ACTIONS(1941),
    [sym_define_library_function_keyword] = ACTIONS(1941),
    [sym_define_start_keyword] = ACTIONS(1941),
    [sym_define_event_keyword] = ACTIONS(1941),
    [sym_define_program_keyword] = ACTIONS(1941),
    [sym_define_module_keyword] = ACTIONS(1941),
    [sym_char_keyword] = ACTIONS(1941),
    [sym_widechar_keyword] = ACTIONS(1941),
    [sym_integer_keyword] = ACTIONS(1941),
    [sym_sinteger_keyword] = ACTIONS(1941),
    [sym_long_keyword] = ACTIONS(1941),
    [sym_slong_keyword] = ACTIONS(1941),
    [sym_float_keyword] = ACTIONS(1941),
    [sym_double_keyword] = ACTIONS(1941),
    [sym_variant_keyword] = ACTIONS(1941),
    [sym_variantarray_keyword] = ACTIONS(1941),
    [sym_local_var_keyword] = ACTIONS(1941),
    [sym_stack_var_keyword] = ACTIONS(1941),
    [sym_constant_keyword] = ACTIONS(1941),
    [sym_volatile_keyword] = ACTIONS(1941),
    [sym_non_volatile_keyword] = ACTIONS(1941),
    [sym_persistent_keyword] = ACTIONS(1941),
    [sym_dev_keyword] = ACTIONS(1941),
    [sym_devlev_keyword] = ACTIONS(1941),
    [sym_devchan_keyword] = ACTIONS(1941),
    [sym_if_keyword] = ACTIONS(1941),
    [sym_switch_keyword] = ACTIONS(1941),
    [sym_case_keyword] = ACTIONS(1941),
    [sym_default_keyword] = ACTIONS(1941),
    [sym_while_keyword] = ACTIONS(1941),
    [sym_for_keyword] = ACTIONS(1941),
    [sym_break_keyword] = ACTIONS(1941),
    [sym_continue_keyword] = ACTIONS(1941),
    [sym_return_keyword] = ACTIONS(1941),
    [sym_select_keyword] = ACTIONS(1941),
    [sym_active_keyword] = ACTIONS(1941),
    [sym_struct_keyword] = ACTIONS(1941),
    [sym_structure_keyword] = ACTIONS(1941),
    [sym_button_event_keyword] = ACTIONS(1941),
    [sym_channel_event_keyword] = ACTIONS(1941),
    [sym_level_event_keyword] = ACTIONS(1941),
    [sym_data_event_keyword] = ACTIONS(1941),
    [sym_timeline_event_keyword] = ACTIONS(1941),
    [sym_custom_event_keyword] = ACTIONS(1941),
    [sym_push_keyword] = ACTIONS(1941),
    [sym_release_keyword] = ACTIONS(1941),
    [aux_sym_on_keyword_token1] = ACTIONS(1941),
    [aux_sym_off_keyword_token1] = ACTIONS(1941),
    [sym_send_level_keyword] = ACTIONS(1941),
    [sym_send_string_keyword] = ACTIONS(1941),
    [sym_send_command_keyword] = ACTIONS(1941),
    [sym_clear_buffer_keyword] = ACTIONS(1941),
    [sym_create_buffer_keyword] = ACTIONS(1941),
    [sym_create_multi_buffer_keyword] = ACTIONS(1941),
    [sym_call_keyword] = ACTIONS(1941),
    [sym_system_call_keyword] = ACTIONS(1941),
    [sym_devchan_to_keyword] = ACTIONS(1941),
    [sym_devchan_min_to_keyword] = ACTIONS(1941),
    [sym_devchan_total_off_keyword] = ACTIONS(1941),
    [sym_devchan_pulse_keyword] = ACTIONS(1941),
    [sym_wait_keyword] = ACTIONS(1941),
    [sym_cancel_wait_keyword] = ACTIONS(1941),
    [sym_wait_until_keyword] = ACTIONS(1941),
    [sym_cancel_wait_until_keyword] = ACTIONS(1941),
    [sym_cancel_all_wait_keyword] = ACTIONS(1941),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1941),
    [sym_not] = ACTIONS(1941),
    [sym_bnot] = ACTIONS(1941),
    [sym___file__] = ACTIONS(1941),
    [sym___date__] = ACTIONS(1941),
    [sym___ldate__] = ACTIONS(1941),
    [sym___time__] = ACTIONS(1941),
    [sym___line__] = ACTIONS(1941),
    [sym___name__] = ACTIONS(1941),
    [sym___netlinx__] = ACTIONS(1941),
    [sym_netlinx_axi_version] = ACTIONS(1941),
    [sym_url_flg_tcp] = ACTIONS(1941),
    [sym_url_flg_accinfopresent] = ACTIONS(1941),
    [sym_url_flg_temp] = ACTIONS(1941),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1941),
    [sym_url_flg_stat_mask] = ACTIONS(1941),
    [sym_url_flg_stat_lookup] = ACTIONS(1941),
    [sym_url_flg_stat_connecting] = ACTIONS(1941),
    [sym_url_flg_stat_waiting] = ACTIONS(1941),
    [sym_url_flg_stat_connected] = ACTIONS(1941),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1941),
    [sym_first_virtual_device] = ACTIONS(1941),
    [sym_first_local_port] = ACTIONS(1941),
    [sym_dynamic_virtual_device] = ACTIONS(1941),
    [sym_timeline_once] = ACTIONS(1941),
    [sym_timeline_repeat] = ACTIONS(1941),
    [sym_timeline_absolute] = ACTIONS(1941),
    [sym_timeline_relative] = ACTIONS(1941),
    [sym_do_push_timed_infinite] = ACTIONS(1941),
    [sym_source_type_no_address] = ACTIONS(1941),
    [sym_source_type_neuron_id] = ACTIONS(1941),
    [sym_source_type_ip_address] = ACTIONS(1941),
    [sym_source_type_axlink] = ACTIONS(1941),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1941),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1941),
    [sym_source_type_ip_socket_address] = ACTIONS(1941),
    [sym_source_type_rs232] = ACTIONS(1941),
    [sym_source_type_internal] = ACTIONS(1941),
    [sym_source_type_mac_address] = ACTIONS(1941),
    [sym_source_type_ipv4_port] = ACTIONS(1941),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1941),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1941),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1941),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1941),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1941),
    [sym_true] = ACTIONS(1941),
    [sym_false] = ACTIONS(1941),
    [sym_file_read_only] = ACTIONS(1941),
    [sym_file_rw_new] = ACTIONS(1941),
    [sym_file_rw_append] = ACTIONS(1941),
    [sym_ip_tcp] = ACTIONS(1941),
    [sym_ip_udp] = ACTIONS(1941),
    [sym_ip_udp_2way] = ACTIONS(1941),
    [sym_xml_encode_types] = ACTIONS(1941),
    [sym_xml_encode_char_as_list] = ACTIONS(1941),
    [sym_xml_encode_le] = ACTIONS(1941),
    [sym_xml_decode_types] = ACTIONS(1941),
    [sym_xml_decode_no_preserve] = ACTIONS(1941),
    [sym_duet_dev_type_display_device] = ACTIONS(1941),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1941),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1941),
    [sym_duet_dev_type_io_device] = ACTIONS(1941),
    [sym_duet_dev_type_relay_device] = ACTIONS(1941),
    [sym_duet_dev_type_ups] = ACTIONS(1941),
    [sym_duet_dev_type_amplifier] = ACTIONS(1941),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1941),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1941),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1941),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1941),
    [sym_duet_dev_type_camera] = ACTIONS(1941),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1941),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1941),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1941),
    [sym_duet_dev_type_dss] = ACTIONS(1941),
    [sym_duet_dev_type_dvr] = ACTIONS(1941),
    [sym_duet_dev_type_disc_device] = ACTIONS(1941),
    [sym_duet_dev_type_document_camera] = ACTIONS(1941),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1941),
    [sym_duet_dev_type_hvac] = ACTIONS(1941),
    [sym_duet_dev_type_keypad] = ACTIONS(1941),
    [sym_duet_dev_type_light] = ACTIONS(1941),
    [sym_duet_dev_type_monitor] = ACTIONS(1941),
    [sym_duet_dev_type_motor] = ACTIONS(1941),
    [sym_duet_dev_type_multi_window] = ACTIONS(1941),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1941),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1941),
    [sym_duet_dev_type_receiver] = ACTIONS(1941),
    [sym_duet_dev_type_security_system] = ACTIONS(1941),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1941),
    [sym_duet_dev_type_settop_box] = ACTIONS(1941),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1941),
    [sym_duet_dev_type_switcher] = ACTIONS(1941),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1941),
    [sym_duet_dev_type_tv] = ACTIONS(1941),
    [sym_duet_dev_type_utility] = ACTIONS(1941),
    [sym_duet_dev_type_vcr] = ACTIONS(1941),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1941),
    [sym_duet_dev_type_video_processor] = ACTIONS(1941),
    [sym_duet_dev_type_video_projector] = ACTIONS(1941),
    [sym_duet_dev_type_video_wall] = ACTIONS(1941),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1941),
    [sym_duet_dev_type_weather] = ACTIONS(1941),
    [sym_duet_dev_not_polled] = ACTIONS(1941),
    [sym_duet_dev_polled] = ACTIONS(1941),
    [sym_clkmgr_mode_network] = ACTIONS(1941),
    [sym_clkmgr_mode_standalone] = ACTIONS(1941),
    [sym_normal_standby] = ACTIONS(1941),
    [sym_normal_wake] = ACTIONS(1941),
    [sym_ip_multicast_ttl_option] = ACTIONS(1941),
    [sym_ip_ttl_subnet] = ACTIONS(1941),
    [sym_ip_ttl_site] = ACTIONS(1941),
    [sym_ip_ttl_region] = ACTIONS(1941),
    [sym_ip_ttl_continent] = ACTIONS(1941),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1941),
    [sym_ip_nodelay_on] = ACTIONS(1941),
    [sym_ip_nodelay_off] = ACTIONS(1941),
    [sym_amx_error] = ACTIONS(1941),
    [sym_amx_warning] = ACTIONS(1941),
    [sym_amx_info] = ACTIONS(1941),
    [sym_amx_debug] = ACTIONS(1941),
    [sym_netlinx_login_success] = ACTIONS(1941),
    [sym_netlinx_login_fail] = ACTIONS(1941),
    [sym_netlinx_logout] = ACTIONS(1941),
    [sym_null_str] = ACTIONS(1941),
    [sym_smtp_address] = ACTIONS(1941),
    [sym_smtp_port_number] = ACTIONS(1941),
    [sym_smtp_username] = ACTIONS(1941),
    [sym_smtp_password] = ACTIONS(1941),
    [sym_smtp_require_tls] = ACTIONS(1941),
    [sym_smtp_from] = ACTIONS(1941),
    [sym_smtp_tls_true] = ACTIONS(1941),
    [sym_smtp_tls_false] = ACTIONS(1941),
    [sym_listview_on_row_select_event] = ACTIONS(1941),
    [sym_data_structure_unknown] = ACTIONS(1941),
    [sym_data_structure_datafeed] = ACTIONS(1941),
    [sym_data_structure_datarecordset] = ACTIONS(1941),
    [sym_data_structure_datarecord] = ACTIONS(1941),
    [sym_data_structure_datafield] = ACTIONS(1941),
    [sym_data_max_content_field_count] = ACTIONS(1941),
    [sym_data_max_metadata_field_count] = ACTIONS(1941),
    [sym_data_max_id_length] = ACTIONS(1941),
    [sym_data_max_name_length] = ACTIONS(1941),
    [sym_data_max_description_length] = ACTIONS(1941),
    [sym_data_max_type_length] = ACTIONS(1941),
    [sym_data_max_format_length] = ACTIONS(1941),
    [sym_data_max_source_length] = ACTIONS(1941),
    [sym_data_max_value_length] = ACTIONS(1941),
    [sym_data_max_label_length] = ACTIONS(1941),
    [sym_data_max_url_length] = ACTIONS(1941),
    [sym_max_last_login_info_length] = ACTIONS(1941),
    [sym_data_type_unknown] = ACTIONS(1941),
    [sym_data_type_string] = ACTIONS(1941),
    [sym_data_type_datetime] = ACTIONS(1941),
    [sym_data_type_date] = ACTIONS(1941),
    [sym_data_type_time] = ACTIONS(1941),
    [sym_data_type_image] = ACTIONS(1941),
    [sym_data_format_url] = ACTIONS(1941),
    [sym_data_format_phone] = ACTIONS(1941),
    [sym_data_format_email] = ACTIONS(1941),
    [sym_data_format_iso8601] = ACTIONS(1941),
    [sym_auditlog_priv] = ACTIONS(1941),
    [sym_configuration_priv] = ACTIONS(1941),
    [sym_deviceconfig_priv] = ACTIONS(1941),
    [sym_ftp_priv] = ACTIONS(1941),
    [sym_http_priv] = ACTIONS(1941),
    [sym_networkconfig_priv] = ACTIONS(1941),
    [sym_programport_priv] = ACTIONS(1941),
    [sym_remoteui_priv] = ACTIONS(1941),
    [sym_securitycontrol_priv] = ACTIONS(1941),
    [sym_softwaremanagement_priv] = ACTIONS(1941),
    [sym_terminal_priv] = ACTIONS(1941),
    [sym_tpadmin_priv] = ACTIONS(1941),
    [sym_user1_priv] = ACTIONS(1941),
    [sym_user2_priv] = ACTIONS(1941),
    [sym_user3_priv] = ACTIONS(1941),
    [sym_user4_priv] = ACTIONS(1941),
    [sym_usermanagement_priv] = ACTIONS(1941),
    [sym_valid_account] = ACTIONS(1941),
    [sym_err_name_invalid] = ACTIONS(1941),
    [sym_err_password_invalid] = ACTIONS(1941),
    [sym_err_invalid_account] = ACTIONS(1941),
    [sym_err_no_authorization] = ACTIONS(1941),
    [sym_err_invalid_parameter] = ACTIONS(1941),
    [sym_err_user_locked_out] = ACTIONS(1941),
    [sym_err_user_account_expired] = ACTIONS(1941),
    [sym_err_general_error] = ACTIONS(1941),
    [sym_err_authentication_error] = ACTIONS(1941),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1941),
    [sym_tls_validate_certificate] = ACTIONS(1941),
    [sym_tdata] = ACTIONS(1941),
    [sym_tchannel] = ACTIONS(1941),
    [sym_tlevel] = ACTIONS(1941),
    [sym_tbutton] = ACTIONS(1941),
    [sym_ttimeline] = ACTIONS(1941),
    [sym_tcustom] = ACTIONS(1941),
    [sym_url_struct] = ACTIONS(1941),
    [sym_dns_struct] = ACTIONS(1941),
    [sym_ip_address_struct] = ACTIONS(1941),
    [sym_dev_info_struct] = ACTIONS(1941),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1941),
    [sym_data_feed] = ACTIONS(1941),
    [sym_data_field] = ACTIONS(1941),
    [sym_data_record] = ACTIONS(1941),
    [sym_wc_data_feed] = ACTIONS(1941),
    [sym_wc_data_field] = ACTIONS(1941),
    [sym_wc_data_record] = ACTIONS(1941),
    [sym_last_login_info] = ACTIONS(1941),
    [sym____reserved___] = ACTIONS(1941),
    [sym_date] = ACTIONS(1941),
    [sym_day] = ACTIONS(1941),
    [sym_time] = ACTIONS(1941),
    [sym_ldate] = ACTIONS(1941),
    [sym_push_device] = ACTIONS(1941),
    [sym_push_channel] = ACTIONS(1941),
    [sym_push_devchan] = ACTIONS(1941),
    [sym_release_device] = ACTIONS(1941),
    [sym_release_channel] = ACTIONS(1941),
    [sym_release_devchan] = ACTIONS(1941),
    [sym_master_slot] = ACTIONS(1941),
    [sym_get_pulse_time] = ACTIONS(1941),
    [sym_get_timer] = ACTIONS(1941),
    [sym_master_sn] = ACTIONS(1941),
    [sym_system_number] = ACTIONS(1941),
    [sym_button] = ACTIONS(1941),
    [sym_dv_channel] = ACTIONS(1941),
    [sym_data] = ACTIONS(1941),
    [sym_level] = ACTIONS(1941),
    [sym_timeline] = ACTIONS(1941),
    [sym_channel] = ACTIONS(1941),
    [sym_custom] = ACTIONS(1941),
    [sym_atoi] = ACTIONS(1941),
    [sym_atol] = ACTIONS(1941),
    [sym_atof] = ACTIONS(1941),
    [sym_itoa] = ACTIONS(1941),
    [sym_itohex] = ACTIONS(1941),
    [sym_hextoi] = ACTIONS(1941),
    [sym_ftoa] = ACTIONS(1941),
    [sym_find_string] = ACTIONS(1941),
    [sym_left_string] = ACTIONS(1941),
    [sym_right_string] = ACTIONS(1941),
    [sym_mid_string] = ACTIONS(1941),
    [sym_remove_string] = ACTIONS(1941),
    [sym_get_buffer_string] = ACTIONS(1941),
    [sym_get_buffer_char] = ACTIONS(1941),
    [sym_get_multi_buffer_string] = ACTIONS(1941),
    [sym_length_string] = ACTIONS(1941),
    [sym_max_length_string] = ACTIONS(1941),
    [sym_set_length_string] = ACTIONS(1941),
    [sym_compare_string] = ACTIONS(1941),
    [sym_upper_string] = ACTIONS(1941),
    [sym_lower_string] = ACTIONS(1941),
    [sym_get_last] = ACTIONS(1941),
    [sym_date_to_day] = ACTIONS(1941),
    [sym_date_to_month] = ACTIONS(1941),
    [sym_date_to_year] = ACTIONS(1941),
    [sym_time_to_hour] = ACTIONS(1941),
    [sym_time_to_minute] = ACTIONS(1941),
    [sym_time_to_second] = ACTIONS(1941),
    [sym_day_of_week] = ACTIONS(1941),
    [sym_set_pulse_time] = ACTIONS(1941),
    [sym_set_timer] = ACTIONS(1941),
    [sym_file_open] = ACTIONS(1941),
    [sym_file_close] = ACTIONS(1941),
    [sym_file_read] = ACTIONS(1941),
    [sym_file_read_line] = ACTIONS(1941),
    [sym_file_write] = ACTIONS(1941),
    [sym_file_write_line] = ACTIONS(1941),
    [sym_file_copy] = ACTIONS(1941),
    [sym_file_delete] = ACTIONS(1941),
    [sym_file_rename] = ACTIONS(1941),
    [sym_file_dir] = ACTIONS(1941),
    [sym_file_getdir] = ACTIONS(1941),
    [sym_file_setdir] = ACTIONS(1941),
    [sym_file_create_dir] = ACTIONS(1941),
    [sym_file_remove_dir] = ACTIONS(1941),
    [sym_file_seek] = ACTIONS(1941),
    [sym_length_array] = ACTIONS(1941),
    [sym_max_length_array] = ACTIONS(1941),
    [sym_set_length_array] = ACTIONS(1941),
    [sym_ip_client_open] = ACTIONS(1941),
    [sym_ip_client_close] = ACTIONS(1941),
    [sym_ip_server_open] = ACTIONS(1941),
    [sym_ip_server_close] = ACTIONS(1941),
    [sym_ip_bound_client_open] = ACTIONS(1941),
    [sym_ip_mc_server_open] = ACTIONS(1941),
    [sym_ip_set_option] = ACTIONS(1941),
    [sym_ssh_client_open] = ACTIONS(1941),
    [sym_ssh_client_close] = ACTIONS(1941),
    [sym_tls_client_open] = ACTIONS(1941),
    [sym_tls_client_close] = ACTIONS(1941),
    [sym_do_push] = ACTIONS(1941),
    [sym_do_push_timed] = ACTIONS(1941),
    [sym_do_release] = ACTIONS(1941),
    [sym_device_id] = ACTIONS(1941),
    [sym_device_info] = ACTIONS(1941),
    [sym_device_id_string] = ACTIONS(1941),
    [sym_device_standby] = ACTIONS(1941),
    [sym_device_wake] = ACTIONS(1941),
    [sym_timeline_create] = ACTIONS(1941),
    [sym_timeline_kill] = ACTIONS(1941),
    [sym_timeline_reload] = ACTIONS(1941),
    [sym_timeline_pause] = ACTIONS(1941),
    [sym_timeline_restart] = ACTIONS(1941),
    [sym_timeline_set] = ACTIONS(1941),
    [sym_timeline_get] = ACTIONS(1941),
    [sym_timeline_active] = ACTIONS(1941),
    [sym_timeline_dynamic_id] = ACTIONS(1941),
    [sym_variable_to_string] = ACTIONS(1941),
    [sym_string_to_variable] = ACTIONS(1941),
    [sym_length_variable_to_string] = ACTIONS(1941),
    [sym_type_cast] = ACTIONS(1941),
    [sym_variable_to_xml] = ACTIONS(1941),
    [sym_xml_to_variable] = ACTIONS(1941),
    [sym_length_variable_to_xml] = ACTIONS(1941),
    [sym_redirect_string] = ACTIONS(1941),
    [sym_random_number] = ACTIONS(1941),
    [sym_get_unique_id] = ACTIONS(1941),
    [sym_get_system_number] = ACTIONS(1941),
    [sym_set_system_number] = ACTIONS(1941),
    [sym_get_serial_number] = ACTIONS(1941),
    [sym_get_master_build] = ACTIONS(1941),
    [sym_set_validation_code] = ACTIONS(1941),
    [sym_reboot] = ACTIONS(1941),
    [sym_load_duet_module] = ACTIONS(1941),
    [sym_unload_duet_module] = ACTIONS(1941),
    [sym_duet_mem_size_set] = ACTIONS(1941),
    [sym_duet_mem_size_get] = ACTIONS(1941),
    [sym_dynamic_application_device] = ACTIONS(1941),
    [sym_dynamic_polled_port] = ACTIONS(1941),
    [sym_static_port_binding] = ACTIONS(1941),
    [sym_static_ip_binding] = ACTIONS(1941),
    [sym_get_dns_list] = ACTIONS(1941),
    [sym_set_dns_list] = ACTIONS(1941),
    [sym_get_ip_address] = ACTIONS(1941),
    [sym_set_ip_address] = ACTIONS(1941),
    [sym_get_url_list] = ACTIONS(1941),
    [sym_add_url_entry] = ACTIONS(1941),
    [sym_delete_url_entry] = ACTIONS(1941),
    [sym_abs_value] = ACTIONS(1941),
    [sym_max_value] = ACTIONS(1941),
    [sym_min_value] = ACTIONS(1941),
    [sym_exp_value] = ACTIONS(1941),
    [sym_log_value] = ACTIONS(1941),
    [sym_log10_value] = ACTIONS(1941),
    [sym_power_value] = ACTIONS(1941),
    [sym_sqrt_value] = ACTIONS(1941),
    [sym_raw_be] = ACTIONS(1941),
    [sym_raw_le] = ACTIONS(1941),
    [sym_internal_queue_size_set] = ACTIONS(1941),
    [sym_internal_queue_size_get] = ACTIONS(1941),
    [sym_internal_threshold_set] = ACTIONS(1941),
    [sym_internal_threshold_get] = ACTIONS(1941),
    [sym_rebuilt_event] = ACTIONS(1941),
    [sym_do_custom_event] = ACTIONS(1941),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1941),
    [sym_clkmgr_set_clk_source] = ACTIONS(1941),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1941),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1941),
    [sym_clkmgr_get_timezone] = ACTIONS(1941),
    [sym_clkmgr_set_timezone] = ACTIONS(1941),
    [sym_clkmgr_get_resync_period] = ACTIONS(1941),
    [sym_clkmgr_set_resync_period] = ACTIONS(1941),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1941),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1941),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1941),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1941),
    [sym_clkmgr_get_timeservers] = ACTIONS(1941),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1941),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1941),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1941),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1941),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1941),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1941),
    [sym_set_log_level] = ACTIONS(1941),
    [sym_get_log_level] = ACTIONS(1941),
    [sym_amx_log] = ACTIONS(1941),
    [sym_smtp_server_config_set] = ACTIONS(1941),
    [sym_smtp_server_config_get] = ACTIONS(1941),
    [sym_smtp_send] = ACTIONS(1941),
    [sym_validate_netlinx_account] = ACTIONS(1941),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1941),
    [sym_authenticate_certificate] = ACTIONS(1941),
    [sym_audit_netlinx_session_event] = ACTIONS(1941),
    [sym_audit_netlinx_generic_event] = ACTIONS(1941),
    [sym_get_available_flash_disk_space] = ACTIONS(1941),
    [sym_get_max_flash_disk_space] = ACTIONS(1941),
    [sym_data_create_feed] = ACTIONS(1941),
    [sym_data_delete_feed] = ACTIONS(1941),
    [sym_data_publish_feed] = ACTIONS(1941),
    [sym_data_get_published_feed] = ACTIONS(1941),
    [sym_data_add_record] = ACTIONS(1941),
    [sym_data_get_event_record] = ACTIONS(1941),
    [sym__wc_data_create_feed] = ACTIONS(1941),
    [sym__wc_data_add_record] = ACTIONS(1941),
    [sym__wc_data_get_event_record] = ACTIONS(1941),
    [sym_format] = ACTIONS(1941),
    [sym_astro_clock] = ACTIONS(1941),
    [sym_set_outdoor_temperature] = ACTIONS(1941),
    [sym_set_virtual_level_count] = ACTIONS(1941),
    [sym_set_virtual_channel_count] = ACTIONS(1941),
    [sym_set_virtual_port_count] = ACTIONS(1941),
    [sym__wc_encode_int] = ACTIONS(1941),
    [sym__wc_decode_int] = ACTIONS(1941),
    [sym__wc_int] = ACTIONS(1941),
    [sym__wc_to_ch_int] = ACTIONS(1941),
    [sym__ch_to_wc_int] = ACTIONS(1941),
    [sym__wc_find_string_int] = ACTIONS(1941),
    [sym__wc_left_string_int] = ACTIONS(1941),
    [sym__wc_lower_string_int] = ACTIONS(1941),
    [sym__wc_mid_string_int] = ACTIONS(1941),
    [sym__wc_remove_string_int] = ACTIONS(1941),
    [sym__wc_right_string_int] = ACTIONS(1941),
    [sym__wc_upper_string_int] = ACTIONS(1941),
    [sym__wc_compare_string_int] = ACTIONS(1941),
    [sym__wc_get_buffer_char_int] = ACTIONS(1941),
    [sym__wc_get_buffer_string_int] = ACTIONS(1941),
    [sym__wc_concat_string_int] = ACTIONS(1941),
    [sym__wc_file_open_int] = ACTIONS(1941),
    [sym__wc_file_close_int] = ACTIONS(1941),
    [sym__wc_file_read_int] = ACTIONS(1941),
    [sym__wc_file_read_line_int] = ACTIONS(1941),
    [sym__wc_file_write_int] = ACTIONS(1941),
    [sym__wc_file_write_line_int] = ACTIONS(1941),
    [aux_sym_preproc_include_token1] = ACTIONS(1941),
    [aux_sym_preproc_define_token1] = ACTIONS(1943),
    [aux_sym_preproc_warn_token1] = ACTIONS(1943),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1943),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1943),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1943),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1943),
    [aux_sym_preproc_else_token1] = ACTIONS(1943),
    [anon_sym_LBRACK] = ACTIONS(1943),
    [anon_sym_LBRACE] = ACTIONS(1943),
    [anon_sym_LPAREN] = ACTIONS(1941),
    [anon_sym_BANG] = ACTIONS(1943),
    [anon_sym_TILDE] = ACTIONS(1943),
    [anon_sym_DASH] = ACTIONS(1941),
    [anon_sym_PLUS] = ACTIONS(1941),
    [anon_sym_DASH_DASH] = ACTIONS(1943),
    [anon_sym_PLUS_PLUS] = ACTIONS(1943),
    [anon_sym_DQUOTE] = ACTIONS(1943),
    [anon_sym_SQUOTE] = ACTIONS(1943),
    [sym_number_literal] = ACTIONS(1943),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(287)] = {
    [ts_builtin_sym_end] = ACTIONS(1841),
    [sym_identifier] = ACTIONS(1839),
    [sym_program_name_keyword] = ACTIONS(1839),
    [sym_module_name_keyword] = ACTIONS(1839),
    [sym_define_device_keyword] = ACTIONS(1839),
    [sym_define_combine_keyword] = ACTIONS(1839),
    [sym_define_constant_keyword] = ACTIONS(1839),
    [sym_define_type_keyword] = ACTIONS(1839),
    [sym_define_variable_keyword] = ACTIONS(1839),
    [sym_define_system_variable_keyword] = ACTIONS(1839),
    [sym_define_connect_level_keyword] = ACTIONS(1839),
    [sym_define_latching_keyword] = ACTIONS(1839),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1839),
    [sym_define_toggling_keyword] = ACTIONS(1839),
    [sym_define_call_keyword] = ACTIONS(1839),
    [sym_define_function_keyword] = ACTIONS(1839),
    [sym_define_library_function_keyword] = ACTIONS(1839),
    [sym_define_start_keyword] = ACTIONS(1839),
    [sym_define_event_keyword] = ACTIONS(1839),
    [sym_define_program_keyword] = ACTIONS(1839),
    [sym_define_module_keyword] = ACTIONS(1839),
    [sym_char_keyword] = ACTIONS(1839),
    [sym_widechar_keyword] = ACTIONS(1839),
    [sym_integer_keyword] = ACTIONS(1839),
    [sym_sinteger_keyword] = ACTIONS(1839),
    [sym_long_keyword] = ACTIONS(1839),
    [sym_slong_keyword] = ACTIONS(1839),
    [sym_float_keyword] = ACTIONS(1839),
    [sym_double_keyword] = ACTIONS(1839),
    [sym_variant_keyword] = ACTIONS(1839),
    [sym_variantarray_keyword] = ACTIONS(1839),
    [sym_local_var_keyword] = ACTIONS(1839),
    [sym_stack_var_keyword] = ACTIONS(1839),
    [sym_constant_keyword] = ACTIONS(1839),
    [sym_volatile_keyword] = ACTIONS(1839),
    [sym_non_volatile_keyword] = ACTIONS(1839),
    [sym_persistent_keyword] = ACTIONS(1839),
    [sym_dev_keyword] = ACTIONS(1839),
    [sym_devlev_keyword] = ACTIONS(1839),
    [sym_devchan_keyword] = ACTIONS(1839),
    [sym_if_keyword] = ACTIONS(1839),
    [sym_else_keyword] = ACTIONS(1839),
    [sym_switch_keyword] = ACTIONS(1839),
    [sym_case_keyword] = ACTIONS(1839),
    [sym_default_keyword] = ACTIONS(1839),
    [sym_while_keyword] = ACTIONS(1839),
    [sym_for_keyword] = ACTIONS(1839),
    [sym_break_keyword] = ACTIONS(1839),
    [sym_continue_keyword] = ACTIONS(1839),
    [sym_return_keyword] = ACTIONS(1839),
    [sym_select_keyword] = ACTIONS(1839),
    [sym_active_keyword] = ACTIONS(1839),
    [sym_struct_keyword] = ACTIONS(1839),
    [sym_structure_keyword] = ACTIONS(1839),
    [sym_button_event_keyword] = ACTIONS(1839),
    [sym_channel_event_keyword] = ACTIONS(1839),
    [sym_level_event_keyword] = ACTIONS(1839),
    [sym_data_event_keyword] = ACTIONS(1839),
    [sym_timeline_event_keyword] = ACTIONS(1839),
    [sym_custom_event_keyword] = ACTIONS(1839),
    [sym_push_keyword] = ACTIONS(1839),
    [sym_release_keyword] = ACTIONS(1839),
    [aux_sym_on_keyword_token1] = ACTIONS(1839),
    [aux_sym_off_keyword_token1] = ACTIONS(1839),
    [sym_send_level_keyword] = ACTIONS(1839),
    [sym_send_string_keyword] = ACTIONS(1839),
    [sym_send_command_keyword] = ACTIONS(1839),
    [sym_clear_buffer_keyword] = ACTIONS(1839),
    [sym_create_buffer_keyword] = ACTIONS(1839),
    [sym_create_multi_buffer_keyword] = ACTIONS(1839),
    [sym_call_keyword] = ACTIONS(1839),
    [sym_system_call_keyword] = ACTIONS(1839),
    [sym_devchan_to_keyword] = ACTIONS(1839),
    [sym_devchan_min_to_keyword] = ACTIONS(1839),
    [sym_devchan_total_off_keyword] = ACTIONS(1839),
    [sym_devchan_pulse_keyword] = ACTIONS(1839),
    [sym_wait_keyword] = ACTIONS(1839),
    [sym_cancel_wait_keyword] = ACTIONS(1839),
    [sym_wait_until_keyword] = ACTIONS(1839),
    [sym_cancel_wait_until_keyword] = ACTIONS(1839),
    [sym_cancel_all_wait_keyword] = ACTIONS(1839),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1839),
    [sym_not] = ACTIONS(1839),
    [sym_bnot] = ACTIONS(1839),
    [sym___file__] = ACTIONS(1839),
    [sym___date__] = ACTIONS(1839),
    [sym___ldate__] = ACTIONS(1839),
    [sym___time__] = ACTIONS(1839),
    [sym___line__] = ACTIONS(1839),
    [sym___name__] = ACTIONS(1839),
    [sym___netlinx__] = ACTIONS(1839),
    [sym_netlinx_axi_version] = ACTIONS(1839),
    [sym_url_flg_tcp] = ACTIONS(1839),
    [sym_url_flg_accinfopresent] = ACTIONS(1839),
    [sym_url_flg_temp] = ACTIONS(1839),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1839),
    [sym_url_flg_stat_mask] = ACTIONS(1839),
    [sym_url_flg_stat_lookup] = ACTIONS(1839),
    [sym_url_flg_stat_connecting] = ACTIONS(1839),
    [sym_url_flg_stat_waiting] = ACTIONS(1839),
    [sym_url_flg_stat_connected] = ACTIONS(1839),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1839),
    [sym_first_virtual_device] = ACTIONS(1839),
    [sym_first_local_port] = ACTIONS(1839),
    [sym_dynamic_virtual_device] = ACTIONS(1839),
    [sym_timeline_once] = ACTIONS(1839),
    [sym_timeline_repeat] = ACTIONS(1839),
    [sym_timeline_absolute] = ACTIONS(1839),
    [sym_timeline_relative] = ACTIONS(1839),
    [sym_do_push_timed_infinite] = ACTIONS(1839),
    [sym_source_type_no_address] = ACTIONS(1839),
    [sym_source_type_neuron_id] = ACTIONS(1839),
    [sym_source_type_ip_address] = ACTIONS(1839),
    [sym_source_type_axlink] = ACTIONS(1839),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1839),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1839),
    [sym_source_type_ip_socket_address] = ACTIONS(1839),
    [sym_source_type_rs232] = ACTIONS(1839),
    [sym_source_type_internal] = ACTIONS(1839),
    [sym_source_type_mac_address] = ACTIONS(1839),
    [sym_source_type_ipv4_port] = ACTIONS(1839),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1839),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1839),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1839),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1839),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1839),
    [sym_true] = ACTIONS(1839),
    [sym_false] = ACTIONS(1839),
    [sym_file_read_only] = ACTIONS(1839),
    [sym_file_rw_new] = ACTIONS(1839),
    [sym_file_rw_append] = ACTIONS(1839),
    [sym_ip_tcp] = ACTIONS(1839),
    [sym_ip_udp] = ACTIONS(1839),
    [sym_ip_udp_2way] = ACTIONS(1839),
    [sym_xml_encode_types] = ACTIONS(1839),
    [sym_xml_encode_char_as_list] = ACTIONS(1839),
    [sym_xml_encode_le] = ACTIONS(1839),
    [sym_xml_decode_types] = ACTIONS(1839),
    [sym_xml_decode_no_preserve] = ACTIONS(1839),
    [sym_duet_dev_type_display_device] = ACTIONS(1839),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1839),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1839),
    [sym_duet_dev_type_io_device] = ACTIONS(1839),
    [sym_duet_dev_type_relay_device] = ACTIONS(1839),
    [sym_duet_dev_type_ups] = ACTIONS(1839),
    [sym_duet_dev_type_amplifier] = ACTIONS(1839),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1839),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1839),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1839),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1839),
    [sym_duet_dev_type_camera] = ACTIONS(1839),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1839),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1839),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1839),
    [sym_duet_dev_type_dss] = ACTIONS(1839),
    [sym_duet_dev_type_dvr] = ACTIONS(1839),
    [sym_duet_dev_type_disc_device] = ACTIONS(1839),
    [sym_duet_dev_type_document_camera] = ACTIONS(1839),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1839),
    [sym_duet_dev_type_hvac] = ACTIONS(1839),
    [sym_duet_dev_type_keypad] = ACTIONS(1839),
    [sym_duet_dev_type_light] = ACTIONS(1839),
    [sym_duet_dev_type_monitor] = ACTIONS(1839),
    [sym_duet_dev_type_motor] = ACTIONS(1839),
    [sym_duet_dev_type_multi_window] = ACTIONS(1839),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1839),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1839),
    [sym_duet_dev_type_receiver] = ACTIONS(1839),
    [sym_duet_dev_type_security_system] = ACTIONS(1839),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1839),
    [sym_duet_dev_type_settop_box] = ACTIONS(1839),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1839),
    [sym_duet_dev_type_switcher] = ACTIONS(1839),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1839),
    [sym_duet_dev_type_tv] = ACTIONS(1839),
    [sym_duet_dev_type_utility] = ACTIONS(1839),
    [sym_duet_dev_type_vcr] = ACTIONS(1839),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1839),
    [sym_duet_dev_type_video_processor] = ACTIONS(1839),
    [sym_duet_dev_type_video_projector] = ACTIONS(1839),
    [sym_duet_dev_type_video_wall] = ACTIONS(1839),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1839),
    [sym_duet_dev_type_weather] = ACTIONS(1839),
    [sym_duet_dev_not_polled] = ACTIONS(1839),
    [sym_duet_dev_polled] = ACTIONS(1839),
    [sym_clkmgr_mode_network] = ACTIONS(1839),
    [sym_clkmgr_mode_standalone] = ACTIONS(1839),
    [sym_normal_standby] = ACTIONS(1839),
    [sym_normal_wake] = ACTIONS(1839),
    [sym_ip_multicast_ttl_option] = ACTIONS(1839),
    [sym_ip_ttl_subnet] = ACTIONS(1839),
    [sym_ip_ttl_site] = ACTIONS(1839),
    [sym_ip_ttl_region] = ACTIONS(1839),
    [sym_ip_ttl_continent] = ACTIONS(1839),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1839),
    [sym_ip_nodelay_on] = ACTIONS(1839),
    [sym_ip_nodelay_off] = ACTIONS(1839),
    [sym_amx_error] = ACTIONS(1839),
    [sym_amx_warning] = ACTIONS(1839),
    [sym_amx_info] = ACTIONS(1839),
    [sym_amx_debug] = ACTIONS(1839),
    [sym_netlinx_login_success] = ACTIONS(1839),
    [sym_netlinx_login_fail] = ACTIONS(1839),
    [sym_netlinx_logout] = ACTIONS(1839),
    [sym_null_str] = ACTIONS(1839),
    [sym_smtp_address] = ACTIONS(1839),
    [sym_smtp_port_number] = ACTIONS(1839),
    [sym_smtp_username] = ACTIONS(1839),
    [sym_smtp_password] = ACTIONS(1839),
    [sym_smtp_require_tls] = ACTIONS(1839),
    [sym_smtp_from] = ACTIONS(1839),
    [sym_smtp_tls_true] = ACTIONS(1839),
    [sym_smtp_tls_false] = ACTIONS(1839),
    [sym_listview_on_row_select_event] = ACTIONS(1839),
    [sym_data_structure_unknown] = ACTIONS(1839),
    [sym_data_structure_datafeed] = ACTIONS(1839),
    [sym_data_structure_datarecordset] = ACTIONS(1839),
    [sym_data_structure_datarecord] = ACTIONS(1839),
    [sym_data_structure_datafield] = ACTIONS(1839),
    [sym_data_max_content_field_count] = ACTIONS(1839),
    [sym_data_max_metadata_field_count] = ACTIONS(1839),
    [sym_data_max_id_length] = ACTIONS(1839),
    [sym_data_max_name_length] = ACTIONS(1839),
    [sym_data_max_description_length] = ACTIONS(1839),
    [sym_data_max_type_length] = ACTIONS(1839),
    [sym_data_max_format_length] = ACTIONS(1839),
    [sym_data_max_source_length] = ACTIONS(1839),
    [sym_data_max_value_length] = ACTIONS(1839),
    [sym_data_max_label_length] = ACTIONS(1839),
    [sym_data_max_url_length] = ACTIONS(1839),
    [sym_max_last_login_info_length] = ACTIONS(1839),
    [sym_data_type_unknown] = ACTIONS(1839),
    [sym_data_type_string] = ACTIONS(1839),
    [sym_data_type_datetime] = ACTIONS(1839),
    [sym_data_type_date] = ACTIONS(1839),
    [sym_data_type_time] = ACTIONS(1839),
    [sym_data_type_image] = ACTIONS(1839),
    [sym_data_format_url] = ACTIONS(1839),
    [sym_data_format_phone] = ACTIONS(1839),
    [sym_data_format_email] = ACTIONS(1839),
    [sym_data_format_iso8601] = ACTIONS(1839),
    [sym_auditlog_priv] = ACTIONS(1839),
    [sym_configuration_priv] = ACTIONS(1839),
    [sym_deviceconfig_priv] = ACTIONS(1839),
    [sym_ftp_priv] = ACTIONS(1839),
    [sym_http_priv] = ACTIONS(1839),
    [sym_networkconfig_priv] = ACTIONS(1839),
    [sym_programport_priv] = ACTIONS(1839),
    [sym_remoteui_priv] = ACTIONS(1839),
    [sym_securitycontrol_priv] = ACTIONS(1839),
    [sym_softwaremanagement_priv] = ACTIONS(1839),
    [sym_terminal_priv] = ACTIONS(1839),
    [sym_tpadmin_priv] = ACTIONS(1839),
    [sym_user1_priv] = ACTIONS(1839),
    [sym_user2_priv] = ACTIONS(1839),
    [sym_user3_priv] = ACTIONS(1839),
    [sym_user4_priv] = ACTIONS(1839),
    [sym_usermanagement_priv] = ACTIONS(1839),
    [sym_valid_account] = ACTIONS(1839),
    [sym_err_name_invalid] = ACTIONS(1839),
    [sym_err_password_invalid] = ACTIONS(1839),
    [sym_err_invalid_account] = ACTIONS(1839),
    [sym_err_no_authorization] = ACTIONS(1839),
    [sym_err_invalid_parameter] = ACTIONS(1839),
    [sym_err_user_locked_out] = ACTIONS(1839),
    [sym_err_user_account_expired] = ACTIONS(1839),
    [sym_err_general_error] = ACTIONS(1839),
    [sym_err_authentication_error] = ACTIONS(1839),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1839),
    [sym_tls_validate_certificate] = ACTIONS(1839),
    [sym_tdata] = ACTIONS(1839),
    [sym_tchannel] = ACTIONS(1839),
    [sym_tlevel] = ACTIONS(1839),
    [sym_tbutton] = ACTIONS(1839),
    [sym_ttimeline] = ACTIONS(1839),
    [sym_tcustom] = ACTIONS(1839),
    [sym_url_struct] = ACTIONS(1839),
    [sym_dns_struct] = ACTIONS(1839),
    [sym_ip_address_struct] = ACTIONS(1839),
    [sym_dev_info_struct] = ACTIONS(1839),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1839),
    [sym_data_feed] = ACTIONS(1839),
    [sym_data_field] = ACTIONS(1839),
    [sym_data_record] = ACTIONS(1839),
    [sym_wc_data_feed] = ACTIONS(1839),
    [sym_wc_data_field] = ACTIONS(1839),
    [sym_wc_data_record] = ACTIONS(1839),
    [sym_last_login_info] = ACTIONS(1839),
    [sym____reserved___] = ACTIONS(1839),
    [sym_date] = ACTIONS(1839),
    [sym_day] = ACTIONS(1839),
    [sym_time] = ACTIONS(1839),
    [sym_ldate] = ACTIONS(1839),
    [sym_push_device] = ACTIONS(1839),
    [sym_push_channel] = ACTIONS(1839),
    [sym_push_devchan] = ACTIONS(1839),
    [sym_release_device] = ACTIONS(1839),
    [sym_release_channel] = ACTIONS(1839),
    [sym_release_devchan] = ACTIONS(1839),
    [sym_master_slot] = ACTIONS(1839),
    [sym_get_pulse_time] = ACTIONS(1839),
    [sym_get_timer] = ACTIONS(1839),
    [sym_master_sn] = ACTIONS(1839),
    [sym_system_number] = ACTIONS(1839),
    [sym_button] = ACTIONS(1839),
    [sym_dv_channel] = ACTIONS(1839),
    [sym_data] = ACTIONS(1839),
    [sym_level] = ACTIONS(1839),
    [sym_timeline] = ACTIONS(1839),
    [sym_channel] = ACTIONS(1839),
    [sym_custom] = ACTIONS(1839),
    [sym_atoi] = ACTIONS(1839),
    [sym_atol] = ACTIONS(1839),
    [sym_atof] = ACTIONS(1839),
    [sym_itoa] = ACTIONS(1839),
    [sym_itohex] = ACTIONS(1839),
    [sym_hextoi] = ACTIONS(1839),
    [sym_ftoa] = ACTIONS(1839),
    [sym_find_string] = ACTIONS(1839),
    [sym_left_string] = ACTIONS(1839),
    [sym_right_string] = ACTIONS(1839),
    [sym_mid_string] = ACTIONS(1839),
    [sym_remove_string] = ACTIONS(1839),
    [sym_get_buffer_string] = ACTIONS(1839),
    [sym_get_buffer_char] = ACTIONS(1839),
    [sym_get_multi_buffer_string] = ACTIONS(1839),
    [sym_length_string] = ACTIONS(1839),
    [sym_max_length_string] = ACTIONS(1839),
    [sym_set_length_string] = ACTIONS(1839),
    [sym_compare_string] = ACTIONS(1839),
    [sym_upper_string] = ACTIONS(1839),
    [sym_lower_string] = ACTIONS(1839),
    [sym_get_last] = ACTIONS(1839),
    [sym_date_to_day] = ACTIONS(1839),
    [sym_date_to_month] = ACTIONS(1839),
    [sym_date_to_year] = ACTIONS(1839),
    [sym_time_to_hour] = ACTIONS(1839),
    [sym_time_to_minute] = ACTIONS(1839),
    [sym_time_to_second] = ACTIONS(1839),
    [sym_day_of_week] = ACTIONS(1839),
    [sym_set_pulse_time] = ACTIONS(1839),
    [sym_set_timer] = ACTIONS(1839),
    [sym_file_open] = ACTIONS(1839),
    [sym_file_close] = ACTIONS(1839),
    [sym_file_read] = ACTIONS(1839),
    [sym_file_read_line] = ACTIONS(1839),
    [sym_file_write] = ACTIONS(1839),
    [sym_file_write_line] = ACTIONS(1839),
    [sym_file_copy] = ACTIONS(1839),
    [sym_file_delete] = ACTIONS(1839),
    [sym_file_rename] = ACTIONS(1839),
    [sym_file_dir] = ACTIONS(1839),
    [sym_file_getdir] = ACTIONS(1839),
    [sym_file_setdir] = ACTIONS(1839),
    [sym_file_create_dir] = ACTIONS(1839),
    [sym_file_remove_dir] = ACTIONS(1839),
    [sym_file_seek] = ACTIONS(1839),
    [sym_length_array] = ACTIONS(1839),
    [sym_max_length_array] = ACTIONS(1839),
    [sym_set_length_array] = ACTIONS(1839),
    [sym_ip_client_open] = ACTIONS(1839),
    [sym_ip_client_close] = ACTIONS(1839),
    [sym_ip_server_open] = ACTIONS(1839),
    [sym_ip_server_close] = ACTIONS(1839),
    [sym_ip_bound_client_open] = ACTIONS(1839),
    [sym_ip_mc_server_open] = ACTIONS(1839),
    [sym_ip_set_option] = ACTIONS(1839),
    [sym_ssh_client_open] = ACTIONS(1839),
    [sym_ssh_client_close] = ACTIONS(1839),
    [sym_tls_client_open] = ACTIONS(1839),
    [sym_tls_client_close] = ACTIONS(1839),
    [sym_do_push] = ACTIONS(1839),
    [sym_do_push_timed] = ACTIONS(1839),
    [sym_do_release] = ACTIONS(1839),
    [sym_device_id] = ACTIONS(1839),
    [sym_device_info] = ACTIONS(1839),
    [sym_device_id_string] = ACTIONS(1839),
    [sym_device_standby] = ACTIONS(1839),
    [sym_device_wake] = ACTIONS(1839),
    [sym_timeline_create] = ACTIONS(1839),
    [sym_timeline_kill] = ACTIONS(1839),
    [sym_timeline_reload] = ACTIONS(1839),
    [sym_timeline_pause] = ACTIONS(1839),
    [sym_timeline_restart] = ACTIONS(1839),
    [sym_timeline_set] = ACTIONS(1839),
    [sym_timeline_get] = ACTIONS(1839),
    [sym_timeline_active] = ACTIONS(1839),
    [sym_timeline_dynamic_id] = ACTIONS(1839),
    [sym_variable_to_string] = ACTIONS(1839),
    [sym_string_to_variable] = ACTIONS(1839),
    [sym_length_variable_to_string] = ACTIONS(1839),
    [sym_type_cast] = ACTIONS(1839),
    [sym_variable_to_xml] = ACTIONS(1839),
    [sym_xml_to_variable] = ACTIONS(1839),
    [sym_length_variable_to_xml] = ACTIONS(1839),
    [sym_redirect_string] = ACTIONS(1839),
    [sym_random_number] = ACTIONS(1839),
    [sym_get_unique_id] = ACTIONS(1839),
    [sym_get_system_number] = ACTIONS(1839),
    [sym_set_system_number] = ACTIONS(1839),
    [sym_get_serial_number] = ACTIONS(1839),
    [sym_get_master_build] = ACTIONS(1839),
    [sym_set_validation_code] = ACTIONS(1839),
    [sym_reboot] = ACTIONS(1839),
    [sym_load_duet_module] = ACTIONS(1839),
    [sym_unload_duet_module] = ACTIONS(1839),
    [sym_duet_mem_size_set] = ACTIONS(1839),
    [sym_duet_mem_size_get] = ACTIONS(1839),
    [sym_dynamic_application_device] = ACTIONS(1839),
    [sym_dynamic_polled_port] = ACTIONS(1839),
    [sym_static_port_binding] = ACTIONS(1839),
    [sym_static_ip_binding] = ACTIONS(1839),
    [sym_get_dns_list] = ACTIONS(1839),
    [sym_set_dns_list] = ACTIONS(1839),
    [sym_get_ip_address] = ACTIONS(1839),
    [sym_set_ip_address] = ACTIONS(1839),
    [sym_get_url_list] = ACTIONS(1839),
    [sym_add_url_entry] = ACTIONS(1839),
    [sym_delete_url_entry] = ACTIONS(1839),
    [sym_abs_value] = ACTIONS(1839),
    [sym_max_value] = ACTIONS(1839),
    [sym_min_value] = ACTIONS(1839),
    [sym_exp_value] = ACTIONS(1839),
    [sym_log_value] = ACTIONS(1839),
    [sym_log10_value] = ACTIONS(1839),
    [sym_power_value] = ACTIONS(1839),
    [sym_sqrt_value] = ACTIONS(1839),
    [sym_raw_be] = ACTIONS(1839),
    [sym_raw_le] = ACTIONS(1839),
    [sym_internal_queue_size_set] = ACTIONS(1839),
    [sym_internal_queue_size_get] = ACTIONS(1839),
    [sym_internal_threshold_set] = ACTIONS(1839),
    [sym_internal_threshold_get] = ACTIONS(1839),
    [sym_rebuilt_event] = ACTIONS(1839),
    [sym_do_custom_event] = ACTIONS(1839),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1839),
    [sym_clkmgr_set_clk_source] = ACTIONS(1839),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1839),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1839),
    [sym_clkmgr_get_timezone] = ACTIONS(1839),
    [sym_clkmgr_set_timezone] = ACTIONS(1839),
    [sym_clkmgr_get_resync_period] = ACTIONS(1839),
    [sym_clkmgr_set_resync_period] = ACTIONS(1839),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1839),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1839),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1839),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1839),
    [sym_clkmgr_get_timeservers] = ACTIONS(1839),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1839),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1839),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1839),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1839),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1839),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1839),
    [sym_set_log_level] = ACTIONS(1839),
    [sym_get_log_level] = ACTIONS(1839),
    [sym_amx_log] = ACTIONS(1839),
    [sym_smtp_server_config_set] = ACTIONS(1839),
    [sym_smtp_server_config_get] = ACTIONS(1839),
    [sym_smtp_send] = ACTIONS(1839),
    [sym_validate_netlinx_account] = ACTIONS(1839),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1839),
    [sym_authenticate_certificate] = ACTIONS(1839),
    [sym_audit_netlinx_session_event] = ACTIONS(1839),
    [sym_audit_netlinx_generic_event] = ACTIONS(1839),
    [sym_get_available_flash_disk_space] = ACTIONS(1839),
    [sym_get_max_flash_disk_space] = ACTIONS(1839),
    [sym_data_create_feed] = ACTIONS(1839),
    [sym_data_delete_feed] = ACTIONS(1839),
    [sym_data_publish_feed] = ACTIONS(1839),
    [sym_data_get_published_feed] = ACTIONS(1839),
    [sym_data_add_record] = ACTIONS(1839),
    [sym_data_get_event_record] = ACTIONS(1839),
    [sym__wc_data_create_feed] = ACTIONS(1839),
    [sym__wc_data_add_record] = ACTIONS(1839),
    [sym__wc_data_get_event_record] = ACTIONS(1839),
    [sym_format] = ACTIONS(1839),
    [sym_astro_clock] = ACTIONS(1839),
    [sym_set_outdoor_temperature] = ACTIONS(1839),
    [sym_set_virtual_level_count] = ACTIONS(1839),
    [sym_set_virtual_channel_count] = ACTIONS(1839),
    [sym_set_virtual_port_count] = ACTIONS(1839),
    [sym__wc_encode_int] = ACTIONS(1839),
    [sym__wc_decode_int] = ACTIONS(1839),
    [sym__wc_int] = ACTIONS(1839),
    [sym__wc_to_ch_int] = ACTIONS(1839),
    [sym__ch_to_wc_int] = ACTIONS(1839),
    [sym__wc_find_string_int] = ACTIONS(1839),
    [sym__wc_left_string_int] = ACTIONS(1839),
    [sym__wc_lower_string_int] = ACTIONS(1839),
    [sym__wc_mid_string_int] = ACTIONS(1839),
    [sym__wc_remove_string_int] = ACTIONS(1839),
    [sym__wc_right_string_int] = ACTIONS(1839),
    [sym__wc_upper_string_int] = ACTIONS(1839),
    [sym__wc_compare_string_int] = ACTIONS(1839),
    [sym__wc_get_buffer_char_int] = ACTIONS(1839),
    [sym__wc_get_buffer_string_int] = ACTIONS(1839),
    [sym__wc_concat_string_int] = ACTIONS(1839),
    [sym__wc_file_open_int] = ACTIONS(1839),
    [sym__wc_file_close_int] = ACTIONS(1839),
    [sym__wc_file_read_int] = ACTIONS(1839),
    [sym__wc_file_read_line_int] = ACTIONS(1839),
    [sym__wc_file_write_int] = ACTIONS(1839),
    [sym__wc_file_write_line_int] = ACTIONS(1839),
    [aux_sym_preproc_include_token1] = ACTIONS(1839),
    [aux_sym_preproc_define_token1] = ACTIONS(1841),
    [aux_sym_preproc_warn_token1] = ACTIONS(1841),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1841),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1841),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1841),
    [anon_sym_LBRACK] = ACTIONS(1841),
    [anon_sym_LBRACE] = ACTIONS(1841),
    [anon_sym_LPAREN] = ACTIONS(1839),
    [anon_sym_BANG] = ACTIONS(1841),
    [anon_sym_TILDE] = ACTIONS(1841),
    [anon_sym_DASH] = ACTIONS(1839),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_DASH_DASH] = ACTIONS(1841),
    [anon_sym_PLUS_PLUS] = ACTIONS(1841),
    [anon_sym_DQUOTE] = ACTIONS(1841),
    [anon_sym_SQUOTE] = ACTIONS(1841),
    [sym_number_literal] = ACTIONS(1841),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(288)] = {
    [ts_builtin_sym_end] = ACTIONS(1845),
    [sym_identifier] = ACTIONS(1843),
    [sym_program_name_keyword] = ACTIONS(1843),
    [sym_module_name_keyword] = ACTIONS(1843),
    [sym_define_device_keyword] = ACTIONS(1843),
    [sym_define_combine_keyword] = ACTIONS(1843),
    [sym_define_constant_keyword] = ACTIONS(1843),
    [sym_define_type_keyword] = ACTIONS(1843),
    [sym_define_variable_keyword] = ACTIONS(1843),
    [sym_define_system_variable_keyword] = ACTIONS(1843),
    [sym_define_connect_level_keyword] = ACTIONS(1843),
    [sym_define_latching_keyword] = ACTIONS(1843),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1843),
    [sym_define_toggling_keyword] = ACTIONS(1843),
    [sym_define_call_keyword] = ACTIONS(1843),
    [sym_define_function_keyword] = ACTIONS(1843),
    [sym_define_library_function_keyword] = ACTIONS(1843),
    [sym_define_start_keyword] = ACTIONS(1843),
    [sym_define_event_keyword] = ACTIONS(1843),
    [sym_define_program_keyword] = ACTIONS(1843),
    [sym_define_module_keyword] = ACTIONS(1843),
    [sym_char_keyword] = ACTIONS(1843),
    [sym_widechar_keyword] = ACTIONS(1843),
    [sym_integer_keyword] = ACTIONS(1843),
    [sym_sinteger_keyword] = ACTIONS(1843),
    [sym_long_keyword] = ACTIONS(1843),
    [sym_slong_keyword] = ACTIONS(1843),
    [sym_float_keyword] = ACTIONS(1843),
    [sym_double_keyword] = ACTIONS(1843),
    [sym_variant_keyword] = ACTIONS(1843),
    [sym_variantarray_keyword] = ACTIONS(1843),
    [sym_local_var_keyword] = ACTIONS(1843),
    [sym_stack_var_keyword] = ACTIONS(1843),
    [sym_constant_keyword] = ACTIONS(1843),
    [sym_volatile_keyword] = ACTIONS(1843),
    [sym_non_volatile_keyword] = ACTIONS(1843),
    [sym_persistent_keyword] = ACTIONS(1843),
    [sym_dev_keyword] = ACTIONS(1843),
    [sym_devlev_keyword] = ACTIONS(1843),
    [sym_devchan_keyword] = ACTIONS(1843),
    [sym_if_keyword] = ACTIONS(1843),
    [sym_else_keyword] = ACTIONS(1843),
    [sym_switch_keyword] = ACTIONS(1843),
    [sym_case_keyword] = ACTIONS(1843),
    [sym_default_keyword] = ACTIONS(1843),
    [sym_while_keyword] = ACTIONS(1843),
    [sym_for_keyword] = ACTIONS(1843),
    [sym_break_keyword] = ACTIONS(1843),
    [sym_continue_keyword] = ACTIONS(1843),
    [sym_return_keyword] = ACTIONS(1843),
    [sym_select_keyword] = ACTIONS(1843),
    [sym_active_keyword] = ACTIONS(1843),
    [sym_struct_keyword] = ACTIONS(1843),
    [sym_structure_keyword] = ACTIONS(1843),
    [sym_button_event_keyword] = ACTIONS(1843),
    [sym_channel_event_keyword] = ACTIONS(1843),
    [sym_level_event_keyword] = ACTIONS(1843),
    [sym_data_event_keyword] = ACTIONS(1843),
    [sym_timeline_event_keyword] = ACTIONS(1843),
    [sym_custom_event_keyword] = ACTIONS(1843),
    [sym_push_keyword] = ACTIONS(1843),
    [sym_release_keyword] = ACTIONS(1843),
    [aux_sym_on_keyword_token1] = ACTIONS(1843),
    [aux_sym_off_keyword_token1] = ACTIONS(1843),
    [sym_send_level_keyword] = ACTIONS(1843),
    [sym_send_string_keyword] = ACTIONS(1843),
    [sym_send_command_keyword] = ACTIONS(1843),
    [sym_clear_buffer_keyword] = ACTIONS(1843),
    [sym_create_buffer_keyword] = ACTIONS(1843),
    [sym_create_multi_buffer_keyword] = ACTIONS(1843),
    [sym_call_keyword] = ACTIONS(1843),
    [sym_system_call_keyword] = ACTIONS(1843),
    [sym_devchan_to_keyword] = ACTIONS(1843),
    [sym_devchan_min_to_keyword] = ACTIONS(1843),
    [sym_devchan_total_off_keyword] = ACTIONS(1843),
    [sym_devchan_pulse_keyword] = ACTIONS(1843),
    [sym_wait_keyword] = ACTIONS(1843),
    [sym_cancel_wait_keyword] = ACTIONS(1843),
    [sym_wait_until_keyword] = ACTIONS(1843),
    [sym_cancel_wait_until_keyword] = ACTIONS(1843),
    [sym_cancel_all_wait_keyword] = ACTIONS(1843),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1843),
    [sym_not] = ACTIONS(1843),
    [sym_bnot] = ACTIONS(1843),
    [sym___file__] = ACTIONS(1843),
    [sym___date__] = ACTIONS(1843),
    [sym___ldate__] = ACTIONS(1843),
    [sym___time__] = ACTIONS(1843),
    [sym___line__] = ACTIONS(1843),
    [sym___name__] = ACTIONS(1843),
    [sym___netlinx__] = ACTIONS(1843),
    [sym_netlinx_axi_version] = ACTIONS(1843),
    [sym_url_flg_tcp] = ACTIONS(1843),
    [sym_url_flg_accinfopresent] = ACTIONS(1843),
    [sym_url_flg_temp] = ACTIONS(1843),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1843),
    [sym_url_flg_stat_mask] = ACTIONS(1843),
    [sym_url_flg_stat_lookup] = ACTIONS(1843),
    [sym_url_flg_stat_connecting] = ACTIONS(1843),
    [sym_url_flg_stat_waiting] = ACTIONS(1843),
    [sym_url_flg_stat_connected] = ACTIONS(1843),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1843),
    [sym_first_virtual_device] = ACTIONS(1843),
    [sym_first_local_port] = ACTIONS(1843),
    [sym_dynamic_virtual_device] = ACTIONS(1843),
    [sym_timeline_once] = ACTIONS(1843),
    [sym_timeline_repeat] = ACTIONS(1843),
    [sym_timeline_absolute] = ACTIONS(1843),
    [sym_timeline_relative] = ACTIONS(1843),
    [sym_do_push_timed_infinite] = ACTIONS(1843),
    [sym_source_type_no_address] = ACTIONS(1843),
    [sym_source_type_neuron_id] = ACTIONS(1843),
    [sym_source_type_ip_address] = ACTIONS(1843),
    [sym_source_type_axlink] = ACTIONS(1843),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1843),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1843),
    [sym_source_type_ip_socket_address] = ACTIONS(1843),
    [sym_source_type_rs232] = ACTIONS(1843),
    [sym_source_type_internal] = ACTIONS(1843),
    [sym_source_type_mac_address] = ACTIONS(1843),
    [sym_source_type_ipv4_port] = ACTIONS(1843),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1843),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1843),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1843),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1843),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1843),
    [sym_true] = ACTIONS(1843),
    [sym_false] = ACTIONS(1843),
    [sym_file_read_only] = ACTIONS(1843),
    [sym_file_rw_new] = ACTIONS(1843),
    [sym_file_rw_append] = ACTIONS(1843),
    [sym_ip_tcp] = ACTIONS(1843),
    [sym_ip_udp] = ACTIONS(1843),
    [sym_ip_udp_2way] = ACTIONS(1843),
    [sym_xml_encode_types] = ACTIONS(1843),
    [sym_xml_encode_char_as_list] = ACTIONS(1843),
    [sym_xml_encode_le] = ACTIONS(1843),
    [sym_xml_decode_types] = ACTIONS(1843),
    [sym_xml_decode_no_preserve] = ACTIONS(1843),
    [sym_duet_dev_type_display_device] = ACTIONS(1843),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1843),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1843),
    [sym_duet_dev_type_io_device] = ACTIONS(1843),
    [sym_duet_dev_type_relay_device] = ACTIONS(1843),
    [sym_duet_dev_type_ups] = ACTIONS(1843),
    [sym_duet_dev_type_amplifier] = ACTIONS(1843),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1843),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1843),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1843),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1843),
    [sym_duet_dev_type_camera] = ACTIONS(1843),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1843),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1843),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1843),
    [sym_duet_dev_type_dss] = ACTIONS(1843),
    [sym_duet_dev_type_dvr] = ACTIONS(1843),
    [sym_duet_dev_type_disc_device] = ACTIONS(1843),
    [sym_duet_dev_type_document_camera] = ACTIONS(1843),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1843),
    [sym_duet_dev_type_hvac] = ACTIONS(1843),
    [sym_duet_dev_type_keypad] = ACTIONS(1843),
    [sym_duet_dev_type_light] = ACTIONS(1843),
    [sym_duet_dev_type_monitor] = ACTIONS(1843),
    [sym_duet_dev_type_motor] = ACTIONS(1843),
    [sym_duet_dev_type_multi_window] = ACTIONS(1843),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1843),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1843),
    [sym_duet_dev_type_receiver] = ACTIONS(1843),
    [sym_duet_dev_type_security_system] = ACTIONS(1843),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1843),
    [sym_duet_dev_type_settop_box] = ACTIONS(1843),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1843),
    [sym_duet_dev_type_switcher] = ACTIONS(1843),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1843),
    [sym_duet_dev_type_tv] = ACTIONS(1843),
    [sym_duet_dev_type_utility] = ACTIONS(1843),
    [sym_duet_dev_type_vcr] = ACTIONS(1843),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1843),
    [sym_duet_dev_type_video_processor] = ACTIONS(1843),
    [sym_duet_dev_type_video_projector] = ACTIONS(1843),
    [sym_duet_dev_type_video_wall] = ACTIONS(1843),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1843),
    [sym_duet_dev_type_weather] = ACTIONS(1843),
    [sym_duet_dev_not_polled] = ACTIONS(1843),
    [sym_duet_dev_polled] = ACTIONS(1843),
    [sym_clkmgr_mode_network] = ACTIONS(1843),
    [sym_clkmgr_mode_standalone] = ACTIONS(1843),
    [sym_normal_standby] = ACTIONS(1843),
    [sym_normal_wake] = ACTIONS(1843),
    [sym_ip_multicast_ttl_option] = ACTIONS(1843),
    [sym_ip_ttl_subnet] = ACTIONS(1843),
    [sym_ip_ttl_site] = ACTIONS(1843),
    [sym_ip_ttl_region] = ACTIONS(1843),
    [sym_ip_ttl_continent] = ACTIONS(1843),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1843),
    [sym_ip_nodelay_on] = ACTIONS(1843),
    [sym_ip_nodelay_off] = ACTIONS(1843),
    [sym_amx_error] = ACTIONS(1843),
    [sym_amx_warning] = ACTIONS(1843),
    [sym_amx_info] = ACTIONS(1843),
    [sym_amx_debug] = ACTIONS(1843),
    [sym_netlinx_login_success] = ACTIONS(1843),
    [sym_netlinx_login_fail] = ACTIONS(1843),
    [sym_netlinx_logout] = ACTIONS(1843),
    [sym_null_str] = ACTIONS(1843),
    [sym_smtp_address] = ACTIONS(1843),
    [sym_smtp_port_number] = ACTIONS(1843),
    [sym_smtp_username] = ACTIONS(1843),
    [sym_smtp_password] = ACTIONS(1843),
    [sym_smtp_require_tls] = ACTIONS(1843),
    [sym_smtp_from] = ACTIONS(1843),
    [sym_smtp_tls_true] = ACTIONS(1843),
    [sym_smtp_tls_false] = ACTIONS(1843),
    [sym_listview_on_row_select_event] = ACTIONS(1843),
    [sym_data_structure_unknown] = ACTIONS(1843),
    [sym_data_structure_datafeed] = ACTIONS(1843),
    [sym_data_structure_datarecordset] = ACTIONS(1843),
    [sym_data_structure_datarecord] = ACTIONS(1843),
    [sym_data_structure_datafield] = ACTIONS(1843),
    [sym_data_max_content_field_count] = ACTIONS(1843),
    [sym_data_max_metadata_field_count] = ACTIONS(1843),
    [sym_data_max_id_length] = ACTIONS(1843),
    [sym_data_max_name_length] = ACTIONS(1843),
    [sym_data_max_description_length] = ACTIONS(1843),
    [sym_data_max_type_length] = ACTIONS(1843),
    [sym_data_max_format_length] = ACTIONS(1843),
    [sym_data_max_source_length] = ACTIONS(1843),
    [sym_data_max_value_length] = ACTIONS(1843),
    [sym_data_max_label_length] = ACTIONS(1843),
    [sym_data_max_url_length] = ACTIONS(1843),
    [sym_max_last_login_info_length] = ACTIONS(1843),
    [sym_data_type_unknown] = ACTIONS(1843),
    [sym_data_type_string] = ACTIONS(1843),
    [sym_data_type_datetime] = ACTIONS(1843),
    [sym_data_type_date] = ACTIONS(1843),
    [sym_data_type_time] = ACTIONS(1843),
    [sym_data_type_image] = ACTIONS(1843),
    [sym_data_format_url] = ACTIONS(1843),
    [sym_data_format_phone] = ACTIONS(1843),
    [sym_data_format_email] = ACTIONS(1843),
    [sym_data_format_iso8601] = ACTIONS(1843),
    [sym_auditlog_priv] = ACTIONS(1843),
    [sym_configuration_priv] = ACTIONS(1843),
    [sym_deviceconfig_priv] = ACTIONS(1843),
    [sym_ftp_priv] = ACTIONS(1843),
    [sym_http_priv] = ACTIONS(1843),
    [sym_networkconfig_priv] = ACTIONS(1843),
    [sym_programport_priv] = ACTIONS(1843),
    [sym_remoteui_priv] = ACTIONS(1843),
    [sym_securitycontrol_priv] = ACTIONS(1843),
    [sym_softwaremanagement_priv] = ACTIONS(1843),
    [sym_terminal_priv] = ACTIONS(1843),
    [sym_tpadmin_priv] = ACTIONS(1843),
    [sym_user1_priv] = ACTIONS(1843),
    [sym_user2_priv] = ACTIONS(1843),
    [sym_user3_priv] = ACTIONS(1843),
    [sym_user4_priv] = ACTIONS(1843),
    [sym_usermanagement_priv] = ACTIONS(1843),
    [sym_valid_account] = ACTIONS(1843),
    [sym_err_name_invalid] = ACTIONS(1843),
    [sym_err_password_invalid] = ACTIONS(1843),
    [sym_err_invalid_account] = ACTIONS(1843),
    [sym_err_no_authorization] = ACTIONS(1843),
    [sym_err_invalid_parameter] = ACTIONS(1843),
    [sym_err_user_locked_out] = ACTIONS(1843),
    [sym_err_user_account_expired] = ACTIONS(1843),
    [sym_err_general_error] = ACTIONS(1843),
    [sym_err_authentication_error] = ACTIONS(1843),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1843),
    [sym_tls_validate_certificate] = ACTIONS(1843),
    [sym_tdata] = ACTIONS(1843),
    [sym_tchannel] = ACTIONS(1843),
    [sym_tlevel] = ACTIONS(1843),
    [sym_tbutton] = ACTIONS(1843),
    [sym_ttimeline] = ACTIONS(1843),
    [sym_tcustom] = ACTIONS(1843),
    [sym_url_struct] = ACTIONS(1843),
    [sym_dns_struct] = ACTIONS(1843),
    [sym_ip_address_struct] = ACTIONS(1843),
    [sym_dev_info_struct] = ACTIONS(1843),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1843),
    [sym_data_feed] = ACTIONS(1843),
    [sym_data_field] = ACTIONS(1843),
    [sym_data_record] = ACTIONS(1843),
    [sym_wc_data_feed] = ACTIONS(1843),
    [sym_wc_data_field] = ACTIONS(1843),
    [sym_wc_data_record] = ACTIONS(1843),
    [sym_last_login_info] = ACTIONS(1843),
    [sym____reserved___] = ACTIONS(1843),
    [sym_date] = ACTIONS(1843),
    [sym_day] = ACTIONS(1843),
    [sym_time] = ACTIONS(1843),
    [sym_ldate] = ACTIONS(1843),
    [sym_push_device] = ACTIONS(1843),
    [sym_push_channel] = ACTIONS(1843),
    [sym_push_devchan] = ACTIONS(1843),
    [sym_release_device] = ACTIONS(1843),
    [sym_release_channel] = ACTIONS(1843),
    [sym_release_devchan] = ACTIONS(1843),
    [sym_master_slot] = ACTIONS(1843),
    [sym_get_pulse_time] = ACTIONS(1843),
    [sym_get_timer] = ACTIONS(1843),
    [sym_master_sn] = ACTIONS(1843),
    [sym_system_number] = ACTIONS(1843),
    [sym_button] = ACTIONS(1843),
    [sym_dv_channel] = ACTIONS(1843),
    [sym_data] = ACTIONS(1843),
    [sym_level] = ACTIONS(1843),
    [sym_timeline] = ACTIONS(1843),
    [sym_channel] = ACTIONS(1843),
    [sym_custom] = ACTIONS(1843),
    [sym_atoi] = ACTIONS(1843),
    [sym_atol] = ACTIONS(1843),
    [sym_atof] = ACTIONS(1843),
    [sym_itoa] = ACTIONS(1843),
    [sym_itohex] = ACTIONS(1843),
    [sym_hextoi] = ACTIONS(1843),
    [sym_ftoa] = ACTIONS(1843),
    [sym_find_string] = ACTIONS(1843),
    [sym_left_string] = ACTIONS(1843),
    [sym_right_string] = ACTIONS(1843),
    [sym_mid_string] = ACTIONS(1843),
    [sym_remove_string] = ACTIONS(1843),
    [sym_get_buffer_string] = ACTIONS(1843),
    [sym_get_buffer_char] = ACTIONS(1843),
    [sym_get_multi_buffer_string] = ACTIONS(1843),
    [sym_length_string] = ACTIONS(1843),
    [sym_max_length_string] = ACTIONS(1843),
    [sym_set_length_string] = ACTIONS(1843),
    [sym_compare_string] = ACTIONS(1843),
    [sym_upper_string] = ACTIONS(1843),
    [sym_lower_string] = ACTIONS(1843),
    [sym_get_last] = ACTIONS(1843),
    [sym_date_to_day] = ACTIONS(1843),
    [sym_date_to_month] = ACTIONS(1843),
    [sym_date_to_year] = ACTIONS(1843),
    [sym_time_to_hour] = ACTIONS(1843),
    [sym_time_to_minute] = ACTIONS(1843),
    [sym_time_to_second] = ACTIONS(1843),
    [sym_day_of_week] = ACTIONS(1843),
    [sym_set_pulse_time] = ACTIONS(1843),
    [sym_set_timer] = ACTIONS(1843),
    [sym_file_open] = ACTIONS(1843),
    [sym_file_close] = ACTIONS(1843),
    [sym_file_read] = ACTIONS(1843),
    [sym_file_read_line] = ACTIONS(1843),
    [sym_file_write] = ACTIONS(1843),
    [sym_file_write_line] = ACTIONS(1843),
    [sym_file_copy] = ACTIONS(1843),
    [sym_file_delete] = ACTIONS(1843),
    [sym_file_rename] = ACTIONS(1843),
    [sym_file_dir] = ACTIONS(1843),
    [sym_file_getdir] = ACTIONS(1843),
    [sym_file_setdir] = ACTIONS(1843),
    [sym_file_create_dir] = ACTIONS(1843),
    [sym_file_remove_dir] = ACTIONS(1843),
    [sym_file_seek] = ACTIONS(1843),
    [sym_length_array] = ACTIONS(1843),
    [sym_max_length_array] = ACTIONS(1843),
    [sym_set_length_array] = ACTIONS(1843),
    [sym_ip_client_open] = ACTIONS(1843),
    [sym_ip_client_close] = ACTIONS(1843),
    [sym_ip_server_open] = ACTIONS(1843),
    [sym_ip_server_close] = ACTIONS(1843),
    [sym_ip_bound_client_open] = ACTIONS(1843),
    [sym_ip_mc_server_open] = ACTIONS(1843),
    [sym_ip_set_option] = ACTIONS(1843),
    [sym_ssh_client_open] = ACTIONS(1843),
    [sym_ssh_client_close] = ACTIONS(1843),
    [sym_tls_client_open] = ACTIONS(1843),
    [sym_tls_client_close] = ACTIONS(1843),
    [sym_do_push] = ACTIONS(1843),
    [sym_do_push_timed] = ACTIONS(1843),
    [sym_do_release] = ACTIONS(1843),
    [sym_device_id] = ACTIONS(1843),
    [sym_device_info] = ACTIONS(1843),
    [sym_device_id_string] = ACTIONS(1843),
    [sym_device_standby] = ACTIONS(1843),
    [sym_device_wake] = ACTIONS(1843),
    [sym_timeline_create] = ACTIONS(1843),
    [sym_timeline_kill] = ACTIONS(1843),
    [sym_timeline_reload] = ACTIONS(1843),
    [sym_timeline_pause] = ACTIONS(1843),
    [sym_timeline_restart] = ACTIONS(1843),
    [sym_timeline_set] = ACTIONS(1843),
    [sym_timeline_get] = ACTIONS(1843),
    [sym_timeline_active] = ACTIONS(1843),
    [sym_timeline_dynamic_id] = ACTIONS(1843),
    [sym_variable_to_string] = ACTIONS(1843),
    [sym_string_to_variable] = ACTIONS(1843),
    [sym_length_variable_to_string] = ACTIONS(1843),
    [sym_type_cast] = ACTIONS(1843),
    [sym_variable_to_xml] = ACTIONS(1843),
    [sym_xml_to_variable] = ACTIONS(1843),
    [sym_length_variable_to_xml] = ACTIONS(1843),
    [sym_redirect_string] = ACTIONS(1843),
    [sym_random_number] = ACTIONS(1843),
    [sym_get_unique_id] = ACTIONS(1843),
    [sym_get_system_number] = ACTIONS(1843),
    [sym_set_system_number] = ACTIONS(1843),
    [sym_get_serial_number] = ACTIONS(1843),
    [sym_get_master_build] = ACTIONS(1843),
    [sym_set_validation_code] = ACTIONS(1843),
    [sym_reboot] = ACTIONS(1843),
    [sym_load_duet_module] = ACTIONS(1843),
    [sym_unload_duet_module] = ACTIONS(1843),
    [sym_duet_mem_size_set] = ACTIONS(1843),
    [sym_duet_mem_size_get] = ACTIONS(1843),
    [sym_dynamic_application_device] = ACTIONS(1843),
    [sym_dynamic_polled_port] = ACTIONS(1843),
    [sym_static_port_binding] = ACTIONS(1843),
    [sym_static_ip_binding] = ACTIONS(1843),
    [sym_get_dns_list] = ACTIONS(1843),
    [sym_set_dns_list] = ACTIONS(1843),
    [sym_get_ip_address] = ACTIONS(1843),
    [sym_set_ip_address] = ACTIONS(1843),
    [sym_get_url_list] = ACTIONS(1843),
    [sym_add_url_entry] = ACTIONS(1843),
    [sym_delete_url_entry] = ACTIONS(1843),
    [sym_abs_value] = ACTIONS(1843),
    [sym_max_value] = ACTIONS(1843),
    [sym_min_value] = ACTIONS(1843),
    [sym_exp_value] = ACTIONS(1843),
    [sym_log_value] = ACTIONS(1843),
    [sym_log10_value] = ACTIONS(1843),
    [sym_power_value] = ACTIONS(1843),
    [sym_sqrt_value] = ACTIONS(1843),
    [sym_raw_be] = ACTIONS(1843),
    [sym_raw_le] = ACTIONS(1843),
    [sym_internal_queue_size_set] = ACTIONS(1843),
    [sym_internal_queue_size_get] = ACTIONS(1843),
    [sym_internal_threshold_set] = ACTIONS(1843),
    [sym_internal_threshold_get] = ACTIONS(1843),
    [sym_rebuilt_event] = ACTIONS(1843),
    [sym_do_custom_event] = ACTIONS(1843),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1843),
    [sym_clkmgr_set_clk_source] = ACTIONS(1843),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1843),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1843),
    [sym_clkmgr_get_timezone] = ACTIONS(1843),
    [sym_clkmgr_set_timezone] = ACTIONS(1843),
    [sym_clkmgr_get_resync_period] = ACTIONS(1843),
    [sym_clkmgr_set_resync_period] = ACTIONS(1843),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1843),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1843),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1843),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1843),
    [sym_clkmgr_get_timeservers] = ACTIONS(1843),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1843),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1843),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1843),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1843),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1843),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1843),
    [sym_set_log_level] = ACTIONS(1843),
    [sym_get_log_level] = ACTIONS(1843),
    [sym_amx_log] = ACTIONS(1843),
    [sym_smtp_server_config_set] = ACTIONS(1843),
    [sym_smtp_server_config_get] = ACTIONS(1843),
    [sym_smtp_send] = ACTIONS(1843),
    [sym_validate_netlinx_account] = ACTIONS(1843),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1843),
    [sym_authenticate_certificate] = ACTIONS(1843),
    [sym_audit_netlinx_session_event] = ACTIONS(1843),
    [sym_audit_netlinx_generic_event] = ACTIONS(1843),
    [sym_get_available_flash_disk_space] = ACTIONS(1843),
    [sym_get_max_flash_disk_space] = ACTIONS(1843),
    [sym_data_create_feed] = ACTIONS(1843),
    [sym_data_delete_feed] = ACTIONS(1843),
    [sym_data_publish_feed] = ACTIONS(1843),
    [sym_data_get_published_feed] = ACTIONS(1843),
    [sym_data_add_record] = ACTIONS(1843),
    [sym_data_get_event_record] = ACTIONS(1843),
    [sym__wc_data_create_feed] = ACTIONS(1843),
    [sym__wc_data_add_record] = ACTIONS(1843),
    [sym__wc_data_get_event_record] = ACTIONS(1843),
    [sym_format] = ACTIONS(1843),
    [sym_astro_clock] = ACTIONS(1843),
    [sym_set_outdoor_temperature] = ACTIONS(1843),
    [sym_set_virtual_level_count] = ACTIONS(1843),
    [sym_set_virtual_channel_count] = ACTIONS(1843),
    [sym_set_virtual_port_count] = ACTIONS(1843),
    [sym__wc_encode_int] = ACTIONS(1843),
    [sym__wc_decode_int] = ACTIONS(1843),
    [sym__wc_int] = ACTIONS(1843),
    [sym__wc_to_ch_int] = ACTIONS(1843),
    [sym__ch_to_wc_int] = ACTIONS(1843),
    [sym__wc_find_string_int] = ACTIONS(1843),
    [sym__wc_left_string_int] = ACTIONS(1843),
    [sym__wc_lower_string_int] = ACTIONS(1843),
    [sym__wc_mid_string_int] = ACTIONS(1843),
    [sym__wc_remove_string_int] = ACTIONS(1843),
    [sym__wc_right_string_int] = ACTIONS(1843),
    [sym__wc_upper_string_int] = ACTIONS(1843),
    [sym__wc_compare_string_int] = ACTIONS(1843),
    [sym__wc_get_buffer_char_int] = ACTIONS(1843),
    [sym__wc_get_buffer_string_int] = ACTIONS(1843),
    [sym__wc_concat_string_int] = ACTIONS(1843),
    [sym__wc_file_open_int] = ACTIONS(1843),
    [sym__wc_file_close_int] = ACTIONS(1843),
    [sym__wc_file_read_int] = ACTIONS(1843),
    [sym__wc_file_read_line_int] = ACTIONS(1843),
    [sym__wc_file_write_int] = ACTIONS(1843),
    [sym__wc_file_write_line_int] = ACTIONS(1843),
    [aux_sym_preproc_include_token1] = ACTIONS(1843),
    [aux_sym_preproc_define_token1] = ACTIONS(1845),
    [aux_sym_preproc_warn_token1] = ACTIONS(1845),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1845),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1845),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1845),
    [anon_sym_LBRACK] = ACTIONS(1845),
    [anon_sym_LBRACE] = ACTIONS(1845),
    [anon_sym_LPAREN] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [anon_sym_DASH] = ACTIONS(1843),
    [anon_sym_PLUS] = ACTIONS(1843),
    [anon_sym_DASH_DASH] = ACTIONS(1845),
    [anon_sym_PLUS_PLUS] = ACTIONS(1845),
    [anon_sym_DQUOTE] = ACTIONS(1845),
    [anon_sym_SQUOTE] = ACTIONS(1845),
    [sym_number_literal] = ACTIONS(1845),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(289)] = {
    [sym_identifier] = ACTIONS(1945),
    [sym_program_name_keyword] = ACTIONS(1945),
    [sym_module_name_keyword] = ACTIONS(1945),
    [sym_define_device_keyword] = ACTIONS(1945),
    [sym_define_combine_keyword] = ACTIONS(1945),
    [sym_define_constant_keyword] = ACTIONS(1945),
    [sym_define_type_keyword] = ACTIONS(1945),
    [sym_define_variable_keyword] = ACTIONS(1945),
    [sym_define_system_variable_keyword] = ACTIONS(1945),
    [sym_define_connect_level_keyword] = ACTIONS(1945),
    [sym_define_latching_keyword] = ACTIONS(1945),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1945),
    [sym_define_toggling_keyword] = ACTIONS(1945),
    [sym_define_call_keyword] = ACTIONS(1945),
    [sym_define_function_keyword] = ACTIONS(1945),
    [sym_define_library_function_keyword] = ACTIONS(1945),
    [sym_define_start_keyword] = ACTIONS(1945),
    [sym_define_event_keyword] = ACTIONS(1945),
    [sym_define_program_keyword] = ACTIONS(1945),
    [sym_define_module_keyword] = ACTIONS(1945),
    [sym_char_keyword] = ACTIONS(1945),
    [sym_widechar_keyword] = ACTIONS(1945),
    [sym_integer_keyword] = ACTIONS(1945),
    [sym_sinteger_keyword] = ACTIONS(1945),
    [sym_long_keyword] = ACTIONS(1945),
    [sym_slong_keyword] = ACTIONS(1945),
    [sym_float_keyword] = ACTIONS(1945),
    [sym_double_keyword] = ACTIONS(1945),
    [sym_variant_keyword] = ACTIONS(1945),
    [sym_variantarray_keyword] = ACTIONS(1945),
    [sym_local_var_keyword] = ACTIONS(1945),
    [sym_stack_var_keyword] = ACTIONS(1945),
    [sym_constant_keyword] = ACTIONS(1945),
    [sym_volatile_keyword] = ACTIONS(1945),
    [sym_non_volatile_keyword] = ACTIONS(1945),
    [sym_persistent_keyword] = ACTIONS(1945),
    [sym_dev_keyword] = ACTIONS(1945),
    [sym_devlev_keyword] = ACTIONS(1945),
    [sym_devchan_keyword] = ACTIONS(1945),
    [sym_if_keyword] = ACTIONS(1945),
    [sym_switch_keyword] = ACTIONS(1945),
    [sym_case_keyword] = ACTIONS(1945),
    [sym_default_keyword] = ACTIONS(1945),
    [sym_while_keyword] = ACTIONS(1945),
    [sym_for_keyword] = ACTIONS(1945),
    [sym_break_keyword] = ACTIONS(1945),
    [sym_continue_keyword] = ACTIONS(1945),
    [sym_return_keyword] = ACTIONS(1945),
    [sym_select_keyword] = ACTIONS(1945),
    [sym_active_keyword] = ACTIONS(1945),
    [sym_struct_keyword] = ACTIONS(1945),
    [sym_structure_keyword] = ACTIONS(1945),
    [sym_button_event_keyword] = ACTIONS(1945),
    [sym_channel_event_keyword] = ACTIONS(1945),
    [sym_level_event_keyword] = ACTIONS(1945),
    [sym_data_event_keyword] = ACTIONS(1945),
    [sym_timeline_event_keyword] = ACTIONS(1945),
    [sym_custom_event_keyword] = ACTIONS(1945),
    [sym_push_keyword] = ACTIONS(1945),
    [sym_release_keyword] = ACTIONS(1945),
    [aux_sym_on_keyword_token1] = ACTIONS(1945),
    [aux_sym_off_keyword_token1] = ACTIONS(1945),
    [sym_send_level_keyword] = ACTIONS(1945),
    [sym_send_string_keyword] = ACTIONS(1945),
    [sym_send_command_keyword] = ACTIONS(1945),
    [sym_clear_buffer_keyword] = ACTIONS(1945),
    [sym_create_buffer_keyword] = ACTIONS(1945),
    [sym_create_multi_buffer_keyword] = ACTIONS(1945),
    [sym_call_keyword] = ACTIONS(1945),
    [sym_system_call_keyword] = ACTIONS(1945),
    [sym_devchan_to_keyword] = ACTIONS(1945),
    [sym_devchan_min_to_keyword] = ACTIONS(1945),
    [sym_devchan_total_off_keyword] = ACTIONS(1945),
    [sym_devchan_pulse_keyword] = ACTIONS(1945),
    [sym_wait_keyword] = ACTIONS(1945),
    [sym_cancel_wait_keyword] = ACTIONS(1945),
    [sym_wait_until_keyword] = ACTIONS(1945),
    [sym_cancel_wait_until_keyword] = ACTIONS(1945),
    [sym_cancel_all_wait_keyword] = ACTIONS(1945),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1945),
    [sym_not] = ACTIONS(1945),
    [sym_bnot] = ACTIONS(1945),
    [sym___file__] = ACTIONS(1945),
    [sym___date__] = ACTIONS(1945),
    [sym___ldate__] = ACTIONS(1945),
    [sym___time__] = ACTIONS(1945),
    [sym___line__] = ACTIONS(1945),
    [sym___name__] = ACTIONS(1945),
    [sym___netlinx__] = ACTIONS(1945),
    [sym_netlinx_axi_version] = ACTIONS(1945),
    [sym_url_flg_tcp] = ACTIONS(1945),
    [sym_url_flg_accinfopresent] = ACTIONS(1945),
    [sym_url_flg_temp] = ACTIONS(1945),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1945),
    [sym_url_flg_stat_mask] = ACTIONS(1945),
    [sym_url_flg_stat_lookup] = ACTIONS(1945),
    [sym_url_flg_stat_connecting] = ACTIONS(1945),
    [sym_url_flg_stat_waiting] = ACTIONS(1945),
    [sym_url_flg_stat_connected] = ACTIONS(1945),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1945),
    [sym_first_virtual_device] = ACTIONS(1945),
    [sym_first_local_port] = ACTIONS(1945),
    [sym_dynamic_virtual_device] = ACTIONS(1945),
    [sym_timeline_once] = ACTIONS(1945),
    [sym_timeline_repeat] = ACTIONS(1945),
    [sym_timeline_absolute] = ACTIONS(1945),
    [sym_timeline_relative] = ACTIONS(1945),
    [sym_do_push_timed_infinite] = ACTIONS(1945),
    [sym_source_type_no_address] = ACTIONS(1945),
    [sym_source_type_neuron_id] = ACTIONS(1945),
    [sym_source_type_ip_address] = ACTIONS(1945),
    [sym_source_type_axlink] = ACTIONS(1945),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1945),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1945),
    [sym_source_type_ip_socket_address] = ACTIONS(1945),
    [sym_source_type_rs232] = ACTIONS(1945),
    [sym_source_type_internal] = ACTIONS(1945),
    [sym_source_type_mac_address] = ACTIONS(1945),
    [sym_source_type_ipv4_port] = ACTIONS(1945),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1945),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1945),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1945),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1945),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1945),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1945),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1945),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1945),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1945),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1945),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1945),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1945),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1945),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1945),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1945),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1945),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1945),
    [sym_true] = ACTIONS(1945),
    [sym_false] = ACTIONS(1945),
    [sym_file_read_only] = ACTIONS(1945),
    [sym_file_rw_new] = ACTIONS(1945),
    [sym_file_rw_append] = ACTIONS(1945),
    [sym_ip_tcp] = ACTIONS(1945),
    [sym_ip_udp] = ACTIONS(1945),
    [sym_ip_udp_2way] = ACTIONS(1945),
    [sym_xml_encode_types] = ACTIONS(1945),
    [sym_xml_encode_char_as_list] = ACTIONS(1945),
    [sym_xml_encode_le] = ACTIONS(1945),
    [sym_xml_decode_types] = ACTIONS(1945),
    [sym_xml_decode_no_preserve] = ACTIONS(1945),
    [sym_duet_dev_type_display_device] = ACTIONS(1945),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1945),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1945),
    [sym_duet_dev_type_io_device] = ACTIONS(1945),
    [sym_duet_dev_type_relay_device] = ACTIONS(1945),
    [sym_duet_dev_type_ups] = ACTIONS(1945),
    [sym_duet_dev_type_amplifier] = ACTIONS(1945),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1945),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1945),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1945),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1945),
    [sym_duet_dev_type_camera] = ACTIONS(1945),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1945),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1945),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1945),
    [sym_duet_dev_type_dss] = ACTIONS(1945),
    [sym_duet_dev_type_dvr] = ACTIONS(1945),
    [sym_duet_dev_type_disc_device] = ACTIONS(1945),
    [sym_duet_dev_type_document_camera] = ACTIONS(1945),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1945),
    [sym_duet_dev_type_hvac] = ACTIONS(1945),
    [sym_duet_dev_type_keypad] = ACTIONS(1945),
    [sym_duet_dev_type_light] = ACTIONS(1945),
    [sym_duet_dev_type_monitor] = ACTIONS(1945),
    [sym_duet_dev_type_motor] = ACTIONS(1945),
    [sym_duet_dev_type_multi_window] = ACTIONS(1945),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1945),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1945),
    [sym_duet_dev_type_receiver] = ACTIONS(1945),
    [sym_duet_dev_type_security_system] = ACTIONS(1945),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1945),
    [sym_duet_dev_type_settop_box] = ACTIONS(1945),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1945),
    [sym_duet_dev_type_switcher] = ACTIONS(1945),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1945),
    [sym_duet_dev_type_tv] = ACTIONS(1945),
    [sym_duet_dev_type_utility] = ACTIONS(1945),
    [sym_duet_dev_type_vcr] = ACTIONS(1945),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1945),
    [sym_duet_dev_type_video_processor] = ACTIONS(1945),
    [sym_duet_dev_type_video_projector] = ACTIONS(1945),
    [sym_duet_dev_type_video_wall] = ACTIONS(1945),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1945),
    [sym_duet_dev_type_weather] = ACTIONS(1945),
    [sym_duet_dev_not_polled] = ACTIONS(1945),
    [sym_duet_dev_polled] = ACTIONS(1945),
    [sym_clkmgr_mode_network] = ACTIONS(1945),
    [sym_clkmgr_mode_standalone] = ACTIONS(1945),
    [sym_normal_standby] = ACTIONS(1945),
    [sym_normal_wake] = ACTIONS(1945),
    [sym_ip_multicast_ttl_option] = ACTIONS(1945),
    [sym_ip_ttl_subnet] = ACTIONS(1945),
    [sym_ip_ttl_site] = ACTIONS(1945),
    [sym_ip_ttl_region] = ACTIONS(1945),
    [sym_ip_ttl_continent] = ACTIONS(1945),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1945),
    [sym_ip_nodelay_on] = ACTIONS(1945),
    [sym_ip_nodelay_off] = ACTIONS(1945),
    [sym_amx_error] = ACTIONS(1945),
    [sym_amx_warning] = ACTIONS(1945),
    [sym_amx_info] = ACTIONS(1945),
    [sym_amx_debug] = ACTIONS(1945),
    [sym_netlinx_login_success] = ACTIONS(1945),
    [sym_netlinx_login_fail] = ACTIONS(1945),
    [sym_netlinx_logout] = ACTIONS(1945),
    [sym_null_str] = ACTIONS(1945),
    [sym_smtp_address] = ACTIONS(1945),
    [sym_smtp_port_number] = ACTIONS(1945),
    [sym_smtp_username] = ACTIONS(1945),
    [sym_smtp_password] = ACTIONS(1945),
    [sym_smtp_require_tls] = ACTIONS(1945),
    [sym_smtp_from] = ACTIONS(1945),
    [sym_smtp_tls_true] = ACTIONS(1945),
    [sym_smtp_tls_false] = ACTIONS(1945),
    [sym_listview_on_row_select_event] = ACTIONS(1945),
    [sym_data_structure_unknown] = ACTIONS(1945),
    [sym_data_structure_datafeed] = ACTIONS(1945),
    [sym_data_structure_datarecordset] = ACTIONS(1945),
    [sym_data_structure_datarecord] = ACTIONS(1945),
    [sym_data_structure_datafield] = ACTIONS(1945),
    [sym_data_max_content_field_count] = ACTIONS(1945),
    [sym_data_max_metadata_field_count] = ACTIONS(1945),
    [sym_data_max_id_length] = ACTIONS(1945),
    [sym_data_max_name_length] = ACTIONS(1945),
    [sym_data_max_description_length] = ACTIONS(1945),
    [sym_data_max_type_length] = ACTIONS(1945),
    [sym_data_max_format_length] = ACTIONS(1945),
    [sym_data_max_source_length] = ACTIONS(1945),
    [sym_data_max_value_length] = ACTIONS(1945),
    [sym_data_max_label_length] = ACTIONS(1945),
    [sym_data_max_url_length] = ACTIONS(1945),
    [sym_max_last_login_info_length] = ACTIONS(1945),
    [sym_data_type_unknown] = ACTIONS(1945),
    [sym_data_type_string] = ACTIONS(1945),
    [sym_data_type_datetime] = ACTIONS(1945),
    [sym_data_type_date] = ACTIONS(1945),
    [sym_data_type_time] = ACTIONS(1945),
    [sym_data_type_image] = ACTIONS(1945),
    [sym_data_format_url] = ACTIONS(1945),
    [sym_data_format_phone] = ACTIONS(1945),
    [sym_data_format_email] = ACTIONS(1945),
    [sym_data_format_iso8601] = ACTIONS(1945),
    [sym_auditlog_priv] = ACTIONS(1945),
    [sym_configuration_priv] = ACTIONS(1945),
    [sym_deviceconfig_priv] = ACTIONS(1945),
    [sym_ftp_priv] = ACTIONS(1945),
    [sym_http_priv] = ACTIONS(1945),
    [sym_networkconfig_priv] = ACTIONS(1945),
    [sym_programport_priv] = ACTIONS(1945),
    [sym_remoteui_priv] = ACTIONS(1945),
    [sym_securitycontrol_priv] = ACTIONS(1945),
    [sym_softwaremanagement_priv] = ACTIONS(1945),
    [sym_terminal_priv] = ACTIONS(1945),
    [sym_tpadmin_priv] = ACTIONS(1945),
    [sym_user1_priv] = ACTIONS(1945),
    [sym_user2_priv] = ACTIONS(1945),
    [sym_user3_priv] = ACTIONS(1945),
    [sym_user4_priv] = ACTIONS(1945),
    [sym_usermanagement_priv] = ACTIONS(1945),
    [sym_valid_account] = ACTIONS(1945),
    [sym_err_name_invalid] = ACTIONS(1945),
    [sym_err_password_invalid] = ACTIONS(1945),
    [sym_err_invalid_account] = ACTIONS(1945),
    [sym_err_no_authorization] = ACTIONS(1945),
    [sym_err_invalid_parameter] = ACTIONS(1945),
    [sym_err_user_locked_out] = ACTIONS(1945),
    [sym_err_user_account_expired] = ACTIONS(1945),
    [sym_err_general_error] = ACTIONS(1945),
    [sym_err_authentication_error] = ACTIONS(1945),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1945),
    [sym_tls_validate_certificate] = ACTIONS(1945),
    [sym_tdata] = ACTIONS(1945),
    [sym_tchannel] = ACTIONS(1945),
    [sym_tlevel] = ACTIONS(1945),
    [sym_tbutton] = ACTIONS(1945),
    [sym_ttimeline] = ACTIONS(1945),
    [sym_tcustom] = ACTIONS(1945),
    [sym_url_struct] = ACTIONS(1945),
    [sym_dns_struct] = ACTIONS(1945),
    [sym_ip_address_struct] = ACTIONS(1945),
    [sym_dev_info_struct] = ACTIONS(1945),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1945),
    [sym_data_feed] = ACTIONS(1945),
    [sym_data_field] = ACTIONS(1945),
    [sym_data_record] = ACTIONS(1945),
    [sym_wc_data_feed] = ACTIONS(1945),
    [sym_wc_data_field] = ACTIONS(1945),
    [sym_wc_data_record] = ACTIONS(1945),
    [sym_last_login_info] = ACTIONS(1945),
    [sym____reserved___] = ACTIONS(1945),
    [sym_date] = ACTIONS(1945),
    [sym_day] = ACTIONS(1945),
    [sym_time] = ACTIONS(1945),
    [sym_ldate] = ACTIONS(1945),
    [sym_push_device] = ACTIONS(1945),
    [sym_push_channel] = ACTIONS(1945),
    [sym_push_devchan] = ACTIONS(1945),
    [sym_release_device] = ACTIONS(1945),
    [sym_release_channel] = ACTIONS(1945),
    [sym_release_devchan] = ACTIONS(1945),
    [sym_master_slot] = ACTIONS(1945),
    [sym_get_pulse_time] = ACTIONS(1945),
    [sym_get_timer] = ACTIONS(1945),
    [sym_master_sn] = ACTIONS(1945),
    [sym_system_number] = ACTIONS(1945),
    [sym_button] = ACTIONS(1945),
    [sym_dv_channel] = ACTIONS(1945),
    [sym_data] = ACTIONS(1945),
    [sym_level] = ACTIONS(1945),
    [sym_timeline] = ACTIONS(1945),
    [sym_channel] = ACTIONS(1945),
    [sym_custom] = ACTIONS(1945),
    [sym_atoi] = ACTIONS(1945),
    [sym_atol] = ACTIONS(1945),
    [sym_atof] = ACTIONS(1945),
    [sym_itoa] = ACTIONS(1945),
    [sym_itohex] = ACTIONS(1945),
    [sym_hextoi] = ACTIONS(1945),
    [sym_ftoa] = ACTIONS(1945),
    [sym_find_string] = ACTIONS(1945),
    [sym_left_string] = ACTIONS(1945),
    [sym_right_string] = ACTIONS(1945),
    [sym_mid_string] = ACTIONS(1945),
    [sym_remove_string] = ACTIONS(1945),
    [sym_get_buffer_string] = ACTIONS(1945),
    [sym_get_buffer_char] = ACTIONS(1945),
    [sym_get_multi_buffer_string] = ACTIONS(1945),
    [sym_length_string] = ACTIONS(1945),
    [sym_max_length_string] = ACTIONS(1945),
    [sym_set_length_string] = ACTIONS(1945),
    [sym_compare_string] = ACTIONS(1945),
    [sym_upper_string] = ACTIONS(1945),
    [sym_lower_string] = ACTIONS(1945),
    [sym_get_last] = ACTIONS(1945),
    [sym_date_to_day] = ACTIONS(1945),
    [sym_date_to_month] = ACTIONS(1945),
    [sym_date_to_year] = ACTIONS(1945),
    [sym_time_to_hour] = ACTIONS(1945),
    [sym_time_to_minute] = ACTIONS(1945),
    [sym_time_to_second] = ACTIONS(1945),
    [sym_day_of_week] = ACTIONS(1945),
    [sym_set_pulse_time] = ACTIONS(1945),
    [sym_set_timer] = ACTIONS(1945),
    [sym_file_open] = ACTIONS(1945),
    [sym_file_close] = ACTIONS(1945),
    [sym_file_read] = ACTIONS(1945),
    [sym_file_read_line] = ACTIONS(1945),
    [sym_file_write] = ACTIONS(1945),
    [sym_file_write_line] = ACTIONS(1945),
    [sym_file_copy] = ACTIONS(1945),
    [sym_file_delete] = ACTIONS(1945),
    [sym_file_rename] = ACTIONS(1945),
    [sym_file_dir] = ACTIONS(1945),
    [sym_file_getdir] = ACTIONS(1945),
    [sym_file_setdir] = ACTIONS(1945),
    [sym_file_create_dir] = ACTIONS(1945),
    [sym_file_remove_dir] = ACTIONS(1945),
    [sym_file_seek] = ACTIONS(1945),
    [sym_length_array] = ACTIONS(1945),
    [sym_max_length_array] = ACTIONS(1945),
    [sym_set_length_array] = ACTIONS(1945),
    [sym_ip_client_open] = ACTIONS(1945),
    [sym_ip_client_close] = ACTIONS(1945),
    [sym_ip_server_open] = ACTIONS(1945),
    [sym_ip_server_close] = ACTIONS(1945),
    [sym_ip_bound_client_open] = ACTIONS(1945),
    [sym_ip_mc_server_open] = ACTIONS(1945),
    [sym_ip_set_option] = ACTIONS(1945),
    [sym_ssh_client_open] = ACTIONS(1945),
    [sym_ssh_client_close] = ACTIONS(1945),
    [sym_tls_client_open] = ACTIONS(1945),
    [sym_tls_client_close] = ACTIONS(1945),
    [sym_do_push] = ACTIONS(1945),
    [sym_do_push_timed] = ACTIONS(1945),
    [sym_do_release] = ACTIONS(1945),
    [sym_device_id] = ACTIONS(1945),
    [sym_device_info] = ACTIONS(1945),
    [sym_device_id_string] = ACTIONS(1945),
    [sym_device_standby] = ACTIONS(1945),
    [sym_device_wake] = ACTIONS(1945),
    [sym_timeline_create] = ACTIONS(1945),
    [sym_timeline_kill] = ACTIONS(1945),
    [sym_timeline_reload] = ACTIONS(1945),
    [sym_timeline_pause] = ACTIONS(1945),
    [sym_timeline_restart] = ACTIONS(1945),
    [sym_timeline_set] = ACTIONS(1945),
    [sym_timeline_get] = ACTIONS(1945),
    [sym_timeline_active] = ACTIONS(1945),
    [sym_timeline_dynamic_id] = ACTIONS(1945),
    [sym_variable_to_string] = ACTIONS(1945),
    [sym_string_to_variable] = ACTIONS(1945),
    [sym_length_variable_to_string] = ACTIONS(1945),
    [sym_type_cast] = ACTIONS(1945),
    [sym_variable_to_xml] = ACTIONS(1945),
    [sym_xml_to_variable] = ACTIONS(1945),
    [sym_length_variable_to_xml] = ACTIONS(1945),
    [sym_redirect_string] = ACTIONS(1945),
    [sym_random_number] = ACTIONS(1945),
    [sym_get_unique_id] = ACTIONS(1945),
    [sym_get_system_number] = ACTIONS(1945),
    [sym_set_system_number] = ACTIONS(1945),
    [sym_get_serial_number] = ACTIONS(1945),
    [sym_get_master_build] = ACTIONS(1945),
    [sym_set_validation_code] = ACTIONS(1945),
    [sym_reboot] = ACTIONS(1945),
    [sym_load_duet_module] = ACTIONS(1945),
    [sym_unload_duet_module] = ACTIONS(1945),
    [sym_duet_mem_size_set] = ACTIONS(1945),
    [sym_duet_mem_size_get] = ACTIONS(1945),
    [sym_dynamic_application_device] = ACTIONS(1945),
    [sym_dynamic_polled_port] = ACTIONS(1945),
    [sym_static_port_binding] = ACTIONS(1945),
    [sym_static_ip_binding] = ACTIONS(1945),
    [sym_get_dns_list] = ACTIONS(1945),
    [sym_set_dns_list] = ACTIONS(1945),
    [sym_get_ip_address] = ACTIONS(1945),
    [sym_set_ip_address] = ACTIONS(1945),
    [sym_get_url_list] = ACTIONS(1945),
    [sym_add_url_entry] = ACTIONS(1945),
    [sym_delete_url_entry] = ACTIONS(1945),
    [sym_abs_value] = ACTIONS(1945),
    [sym_max_value] = ACTIONS(1945),
    [sym_min_value] = ACTIONS(1945),
    [sym_exp_value] = ACTIONS(1945),
    [sym_log_value] = ACTIONS(1945),
    [sym_log10_value] = ACTIONS(1945),
    [sym_power_value] = ACTIONS(1945),
    [sym_sqrt_value] = ACTIONS(1945),
    [sym_raw_be] = ACTIONS(1945),
    [sym_raw_le] = ACTIONS(1945),
    [sym_internal_queue_size_set] = ACTIONS(1945),
    [sym_internal_queue_size_get] = ACTIONS(1945),
    [sym_internal_threshold_set] = ACTIONS(1945),
    [sym_internal_threshold_get] = ACTIONS(1945),
    [sym_rebuilt_event] = ACTIONS(1945),
    [sym_do_custom_event] = ACTIONS(1945),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1945),
    [sym_clkmgr_set_clk_source] = ACTIONS(1945),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1945),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1945),
    [sym_clkmgr_get_timezone] = ACTIONS(1945),
    [sym_clkmgr_set_timezone] = ACTIONS(1945),
    [sym_clkmgr_get_resync_period] = ACTIONS(1945),
    [sym_clkmgr_set_resync_period] = ACTIONS(1945),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1945),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1945),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1945),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1945),
    [sym_clkmgr_get_timeservers] = ACTIONS(1945),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1945),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1945),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1945),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1945),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1945),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1945),
    [sym_set_log_level] = ACTIONS(1945),
    [sym_get_log_level] = ACTIONS(1945),
    [sym_amx_log] = ACTIONS(1945),
    [sym_smtp_server_config_set] = ACTIONS(1945),
    [sym_smtp_server_config_get] = ACTIONS(1945),
    [sym_smtp_send] = ACTIONS(1945),
    [sym_validate_netlinx_account] = ACTIONS(1945),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1945),
    [sym_authenticate_certificate] = ACTIONS(1945),
    [sym_audit_netlinx_session_event] = ACTIONS(1945),
    [sym_audit_netlinx_generic_event] = ACTIONS(1945),
    [sym_get_available_flash_disk_space] = ACTIONS(1945),
    [sym_get_max_flash_disk_space] = ACTIONS(1945),
    [sym_data_create_feed] = ACTIONS(1945),
    [sym_data_delete_feed] = ACTIONS(1945),
    [sym_data_publish_feed] = ACTIONS(1945),
    [sym_data_get_published_feed] = ACTIONS(1945),
    [sym_data_add_record] = ACTIONS(1945),
    [sym_data_get_event_record] = ACTIONS(1945),
    [sym__wc_data_create_feed] = ACTIONS(1945),
    [sym__wc_data_add_record] = ACTIONS(1945),
    [sym__wc_data_get_event_record] = ACTIONS(1945),
    [sym_format] = ACTIONS(1945),
    [sym_astro_clock] = ACTIONS(1945),
    [sym_set_outdoor_temperature] = ACTIONS(1945),
    [sym_set_virtual_level_count] = ACTIONS(1945),
    [sym_set_virtual_channel_count] = ACTIONS(1945),
    [sym_set_virtual_port_count] = ACTIONS(1945),
    [sym__wc_encode_int] = ACTIONS(1945),
    [sym__wc_decode_int] = ACTIONS(1945),
    [sym__wc_int] = ACTIONS(1945),
    [sym__wc_to_ch_int] = ACTIONS(1945),
    [sym__ch_to_wc_int] = ACTIONS(1945),
    [sym__wc_find_string_int] = ACTIONS(1945),
    [sym__wc_left_string_int] = ACTIONS(1945),
    [sym__wc_lower_string_int] = ACTIONS(1945),
    [sym__wc_mid_string_int] = ACTIONS(1945),
    [sym__wc_remove_string_int] = ACTIONS(1945),
    [sym__wc_right_string_int] = ACTIONS(1945),
    [sym__wc_upper_string_int] = ACTIONS(1945),
    [sym__wc_compare_string_int] = ACTIONS(1945),
    [sym__wc_get_buffer_char_int] = ACTIONS(1945),
    [sym__wc_get_buffer_string_int] = ACTIONS(1945),
    [sym__wc_concat_string_int] = ACTIONS(1945),
    [sym__wc_file_open_int] = ACTIONS(1945),
    [sym__wc_file_close_int] = ACTIONS(1945),
    [sym__wc_file_read_int] = ACTIONS(1945),
    [sym__wc_file_read_line_int] = ACTIONS(1945),
    [sym__wc_file_write_int] = ACTIONS(1945),
    [sym__wc_file_write_line_int] = ACTIONS(1945),
    [aux_sym_preproc_include_token1] = ACTIONS(1945),
    [aux_sym_preproc_define_token1] = ACTIONS(1947),
    [aux_sym_preproc_warn_token1] = ACTIONS(1947),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1947),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1947),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1947),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1947),
    [aux_sym_preproc_else_token1] = ACTIONS(1947),
    [anon_sym_LBRACK] = ACTIONS(1947),
    [anon_sym_LBRACE] = ACTIONS(1947),
    [anon_sym_LPAREN] = ACTIONS(1945),
    [anon_sym_BANG] = ACTIONS(1947),
    [anon_sym_TILDE] = ACTIONS(1947),
    [anon_sym_DASH] = ACTIONS(1945),
    [anon_sym_PLUS] = ACTIONS(1945),
    [anon_sym_DASH_DASH] = ACTIONS(1947),
    [anon_sym_PLUS_PLUS] = ACTIONS(1947),
    [anon_sym_DQUOTE] = ACTIONS(1947),
    [anon_sym_SQUOTE] = ACTIONS(1947),
    [sym_number_literal] = ACTIONS(1947),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(290)] = {
    [sym_identifier] = ACTIONS(1949),
    [sym_program_name_keyword] = ACTIONS(1949),
    [sym_module_name_keyword] = ACTIONS(1949),
    [sym_define_device_keyword] = ACTIONS(1949),
    [sym_define_combine_keyword] = ACTIONS(1949),
    [sym_define_constant_keyword] = ACTIONS(1949),
    [sym_define_type_keyword] = ACTIONS(1949),
    [sym_define_variable_keyword] = ACTIONS(1949),
    [sym_define_system_variable_keyword] = ACTIONS(1949),
    [sym_define_connect_level_keyword] = ACTIONS(1949),
    [sym_define_latching_keyword] = ACTIONS(1949),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1949),
    [sym_define_toggling_keyword] = ACTIONS(1949),
    [sym_define_call_keyword] = ACTIONS(1949),
    [sym_define_function_keyword] = ACTIONS(1949),
    [sym_define_library_function_keyword] = ACTIONS(1949),
    [sym_define_start_keyword] = ACTIONS(1949),
    [sym_define_event_keyword] = ACTIONS(1949),
    [sym_define_program_keyword] = ACTIONS(1949),
    [sym_define_module_keyword] = ACTIONS(1949),
    [sym_char_keyword] = ACTIONS(1949),
    [sym_widechar_keyword] = ACTIONS(1949),
    [sym_integer_keyword] = ACTIONS(1949),
    [sym_sinteger_keyword] = ACTIONS(1949),
    [sym_long_keyword] = ACTIONS(1949),
    [sym_slong_keyword] = ACTIONS(1949),
    [sym_float_keyword] = ACTIONS(1949),
    [sym_double_keyword] = ACTIONS(1949),
    [sym_variant_keyword] = ACTIONS(1949),
    [sym_variantarray_keyword] = ACTIONS(1949),
    [sym_local_var_keyword] = ACTIONS(1949),
    [sym_stack_var_keyword] = ACTIONS(1949),
    [sym_constant_keyword] = ACTIONS(1949),
    [sym_volatile_keyword] = ACTIONS(1949),
    [sym_non_volatile_keyword] = ACTIONS(1949),
    [sym_persistent_keyword] = ACTIONS(1949),
    [sym_dev_keyword] = ACTIONS(1949),
    [sym_devlev_keyword] = ACTIONS(1949),
    [sym_devchan_keyword] = ACTIONS(1949),
    [sym_if_keyword] = ACTIONS(1949),
    [sym_switch_keyword] = ACTIONS(1949),
    [sym_case_keyword] = ACTIONS(1949),
    [sym_default_keyword] = ACTIONS(1949),
    [sym_while_keyword] = ACTIONS(1949),
    [sym_for_keyword] = ACTIONS(1949),
    [sym_break_keyword] = ACTIONS(1949),
    [sym_continue_keyword] = ACTIONS(1949),
    [sym_return_keyword] = ACTIONS(1949),
    [sym_select_keyword] = ACTIONS(1949),
    [sym_active_keyword] = ACTIONS(1949),
    [sym_struct_keyword] = ACTIONS(1949),
    [sym_structure_keyword] = ACTIONS(1949),
    [sym_button_event_keyword] = ACTIONS(1949),
    [sym_channel_event_keyword] = ACTIONS(1949),
    [sym_level_event_keyword] = ACTIONS(1949),
    [sym_data_event_keyword] = ACTIONS(1949),
    [sym_timeline_event_keyword] = ACTIONS(1949),
    [sym_custom_event_keyword] = ACTIONS(1949),
    [sym_push_keyword] = ACTIONS(1949),
    [sym_release_keyword] = ACTIONS(1949),
    [aux_sym_on_keyword_token1] = ACTIONS(1949),
    [aux_sym_off_keyword_token1] = ACTIONS(1949),
    [sym_send_level_keyword] = ACTIONS(1949),
    [sym_send_string_keyword] = ACTIONS(1949),
    [sym_send_command_keyword] = ACTIONS(1949),
    [sym_clear_buffer_keyword] = ACTIONS(1949),
    [sym_create_buffer_keyword] = ACTIONS(1949),
    [sym_create_multi_buffer_keyword] = ACTIONS(1949),
    [sym_call_keyword] = ACTIONS(1949),
    [sym_system_call_keyword] = ACTIONS(1949),
    [sym_devchan_to_keyword] = ACTIONS(1949),
    [sym_devchan_min_to_keyword] = ACTIONS(1949),
    [sym_devchan_total_off_keyword] = ACTIONS(1949),
    [sym_devchan_pulse_keyword] = ACTIONS(1949),
    [sym_wait_keyword] = ACTIONS(1949),
    [sym_cancel_wait_keyword] = ACTIONS(1949),
    [sym_wait_until_keyword] = ACTIONS(1949),
    [sym_cancel_wait_until_keyword] = ACTIONS(1949),
    [sym_cancel_all_wait_keyword] = ACTIONS(1949),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1949),
    [sym_not] = ACTIONS(1949),
    [sym_bnot] = ACTIONS(1949),
    [sym___file__] = ACTIONS(1949),
    [sym___date__] = ACTIONS(1949),
    [sym___ldate__] = ACTIONS(1949),
    [sym___time__] = ACTIONS(1949),
    [sym___line__] = ACTIONS(1949),
    [sym___name__] = ACTIONS(1949),
    [sym___netlinx__] = ACTIONS(1949),
    [sym_netlinx_axi_version] = ACTIONS(1949),
    [sym_url_flg_tcp] = ACTIONS(1949),
    [sym_url_flg_accinfopresent] = ACTIONS(1949),
    [sym_url_flg_temp] = ACTIONS(1949),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1949),
    [sym_url_flg_stat_mask] = ACTIONS(1949),
    [sym_url_flg_stat_lookup] = ACTIONS(1949),
    [sym_url_flg_stat_connecting] = ACTIONS(1949),
    [sym_url_flg_stat_waiting] = ACTIONS(1949),
    [sym_url_flg_stat_connected] = ACTIONS(1949),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1949),
    [sym_first_virtual_device] = ACTIONS(1949),
    [sym_first_local_port] = ACTIONS(1949),
    [sym_dynamic_virtual_device] = ACTIONS(1949),
    [sym_timeline_once] = ACTIONS(1949),
    [sym_timeline_repeat] = ACTIONS(1949),
    [sym_timeline_absolute] = ACTIONS(1949),
    [sym_timeline_relative] = ACTIONS(1949),
    [sym_do_push_timed_infinite] = ACTIONS(1949),
    [sym_source_type_no_address] = ACTIONS(1949),
    [sym_source_type_neuron_id] = ACTIONS(1949),
    [sym_source_type_ip_address] = ACTIONS(1949),
    [sym_source_type_axlink] = ACTIONS(1949),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1949),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1949),
    [sym_source_type_ip_socket_address] = ACTIONS(1949),
    [sym_source_type_rs232] = ACTIONS(1949),
    [sym_source_type_internal] = ACTIONS(1949),
    [sym_source_type_mac_address] = ACTIONS(1949),
    [sym_source_type_ipv4_port] = ACTIONS(1949),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1949),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1949),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1949),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1949),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1949),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1949),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1949),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1949),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1949),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1949),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1949),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1949),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1949),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1949),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1949),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1949),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1949),
    [sym_true] = ACTIONS(1949),
    [sym_false] = ACTIONS(1949),
    [sym_file_read_only] = ACTIONS(1949),
    [sym_file_rw_new] = ACTIONS(1949),
    [sym_file_rw_append] = ACTIONS(1949),
    [sym_ip_tcp] = ACTIONS(1949),
    [sym_ip_udp] = ACTIONS(1949),
    [sym_ip_udp_2way] = ACTIONS(1949),
    [sym_xml_encode_types] = ACTIONS(1949),
    [sym_xml_encode_char_as_list] = ACTIONS(1949),
    [sym_xml_encode_le] = ACTIONS(1949),
    [sym_xml_decode_types] = ACTIONS(1949),
    [sym_xml_decode_no_preserve] = ACTIONS(1949),
    [sym_duet_dev_type_display_device] = ACTIONS(1949),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1949),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1949),
    [sym_duet_dev_type_io_device] = ACTIONS(1949),
    [sym_duet_dev_type_relay_device] = ACTIONS(1949),
    [sym_duet_dev_type_ups] = ACTIONS(1949),
    [sym_duet_dev_type_amplifier] = ACTIONS(1949),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1949),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1949),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1949),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1949),
    [sym_duet_dev_type_camera] = ACTIONS(1949),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1949),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1949),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1949),
    [sym_duet_dev_type_dss] = ACTIONS(1949),
    [sym_duet_dev_type_dvr] = ACTIONS(1949),
    [sym_duet_dev_type_disc_device] = ACTIONS(1949),
    [sym_duet_dev_type_document_camera] = ACTIONS(1949),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1949),
    [sym_duet_dev_type_hvac] = ACTIONS(1949),
    [sym_duet_dev_type_keypad] = ACTIONS(1949),
    [sym_duet_dev_type_light] = ACTIONS(1949),
    [sym_duet_dev_type_monitor] = ACTIONS(1949),
    [sym_duet_dev_type_motor] = ACTIONS(1949),
    [sym_duet_dev_type_multi_window] = ACTIONS(1949),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1949),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1949),
    [sym_duet_dev_type_receiver] = ACTIONS(1949),
    [sym_duet_dev_type_security_system] = ACTIONS(1949),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1949),
    [sym_duet_dev_type_settop_box] = ACTIONS(1949),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1949),
    [sym_duet_dev_type_switcher] = ACTIONS(1949),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1949),
    [sym_duet_dev_type_tv] = ACTIONS(1949),
    [sym_duet_dev_type_utility] = ACTIONS(1949),
    [sym_duet_dev_type_vcr] = ACTIONS(1949),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1949),
    [sym_duet_dev_type_video_processor] = ACTIONS(1949),
    [sym_duet_dev_type_video_projector] = ACTIONS(1949),
    [sym_duet_dev_type_video_wall] = ACTIONS(1949),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1949),
    [sym_duet_dev_type_weather] = ACTIONS(1949),
    [sym_duet_dev_not_polled] = ACTIONS(1949),
    [sym_duet_dev_polled] = ACTIONS(1949),
    [sym_clkmgr_mode_network] = ACTIONS(1949),
    [sym_clkmgr_mode_standalone] = ACTIONS(1949),
    [sym_normal_standby] = ACTIONS(1949),
    [sym_normal_wake] = ACTIONS(1949),
    [sym_ip_multicast_ttl_option] = ACTIONS(1949),
    [sym_ip_ttl_subnet] = ACTIONS(1949),
    [sym_ip_ttl_site] = ACTIONS(1949),
    [sym_ip_ttl_region] = ACTIONS(1949),
    [sym_ip_ttl_continent] = ACTIONS(1949),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1949),
    [sym_ip_nodelay_on] = ACTIONS(1949),
    [sym_ip_nodelay_off] = ACTIONS(1949),
    [sym_amx_error] = ACTIONS(1949),
    [sym_amx_warning] = ACTIONS(1949),
    [sym_amx_info] = ACTIONS(1949),
    [sym_amx_debug] = ACTIONS(1949),
    [sym_netlinx_login_success] = ACTIONS(1949),
    [sym_netlinx_login_fail] = ACTIONS(1949),
    [sym_netlinx_logout] = ACTIONS(1949),
    [sym_null_str] = ACTIONS(1949),
    [sym_smtp_address] = ACTIONS(1949),
    [sym_smtp_port_number] = ACTIONS(1949),
    [sym_smtp_username] = ACTIONS(1949),
    [sym_smtp_password] = ACTIONS(1949),
    [sym_smtp_require_tls] = ACTIONS(1949),
    [sym_smtp_from] = ACTIONS(1949),
    [sym_smtp_tls_true] = ACTIONS(1949),
    [sym_smtp_tls_false] = ACTIONS(1949),
    [sym_listview_on_row_select_event] = ACTIONS(1949),
    [sym_data_structure_unknown] = ACTIONS(1949),
    [sym_data_structure_datafeed] = ACTIONS(1949),
    [sym_data_structure_datarecordset] = ACTIONS(1949),
    [sym_data_structure_datarecord] = ACTIONS(1949),
    [sym_data_structure_datafield] = ACTIONS(1949),
    [sym_data_max_content_field_count] = ACTIONS(1949),
    [sym_data_max_metadata_field_count] = ACTIONS(1949),
    [sym_data_max_id_length] = ACTIONS(1949),
    [sym_data_max_name_length] = ACTIONS(1949),
    [sym_data_max_description_length] = ACTIONS(1949),
    [sym_data_max_type_length] = ACTIONS(1949),
    [sym_data_max_format_length] = ACTIONS(1949),
    [sym_data_max_source_length] = ACTIONS(1949),
    [sym_data_max_value_length] = ACTIONS(1949),
    [sym_data_max_label_length] = ACTIONS(1949),
    [sym_data_max_url_length] = ACTIONS(1949),
    [sym_max_last_login_info_length] = ACTIONS(1949),
    [sym_data_type_unknown] = ACTIONS(1949),
    [sym_data_type_string] = ACTIONS(1949),
    [sym_data_type_datetime] = ACTIONS(1949),
    [sym_data_type_date] = ACTIONS(1949),
    [sym_data_type_time] = ACTIONS(1949),
    [sym_data_type_image] = ACTIONS(1949),
    [sym_data_format_url] = ACTIONS(1949),
    [sym_data_format_phone] = ACTIONS(1949),
    [sym_data_format_email] = ACTIONS(1949),
    [sym_data_format_iso8601] = ACTIONS(1949),
    [sym_auditlog_priv] = ACTIONS(1949),
    [sym_configuration_priv] = ACTIONS(1949),
    [sym_deviceconfig_priv] = ACTIONS(1949),
    [sym_ftp_priv] = ACTIONS(1949),
    [sym_http_priv] = ACTIONS(1949),
    [sym_networkconfig_priv] = ACTIONS(1949),
    [sym_programport_priv] = ACTIONS(1949),
    [sym_remoteui_priv] = ACTIONS(1949),
    [sym_securitycontrol_priv] = ACTIONS(1949),
    [sym_softwaremanagement_priv] = ACTIONS(1949),
    [sym_terminal_priv] = ACTIONS(1949),
    [sym_tpadmin_priv] = ACTIONS(1949),
    [sym_user1_priv] = ACTIONS(1949),
    [sym_user2_priv] = ACTIONS(1949),
    [sym_user3_priv] = ACTIONS(1949),
    [sym_user4_priv] = ACTIONS(1949),
    [sym_usermanagement_priv] = ACTIONS(1949),
    [sym_valid_account] = ACTIONS(1949),
    [sym_err_name_invalid] = ACTIONS(1949),
    [sym_err_password_invalid] = ACTIONS(1949),
    [sym_err_invalid_account] = ACTIONS(1949),
    [sym_err_no_authorization] = ACTIONS(1949),
    [sym_err_invalid_parameter] = ACTIONS(1949),
    [sym_err_user_locked_out] = ACTIONS(1949),
    [sym_err_user_account_expired] = ACTIONS(1949),
    [sym_err_general_error] = ACTIONS(1949),
    [sym_err_authentication_error] = ACTIONS(1949),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1949),
    [sym_tls_validate_certificate] = ACTIONS(1949),
    [sym_tdata] = ACTIONS(1949),
    [sym_tchannel] = ACTIONS(1949),
    [sym_tlevel] = ACTIONS(1949),
    [sym_tbutton] = ACTIONS(1949),
    [sym_ttimeline] = ACTIONS(1949),
    [sym_tcustom] = ACTIONS(1949),
    [sym_url_struct] = ACTIONS(1949),
    [sym_dns_struct] = ACTIONS(1949),
    [sym_ip_address_struct] = ACTIONS(1949),
    [sym_dev_info_struct] = ACTIONS(1949),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1949),
    [sym_data_feed] = ACTIONS(1949),
    [sym_data_field] = ACTIONS(1949),
    [sym_data_record] = ACTIONS(1949),
    [sym_wc_data_feed] = ACTIONS(1949),
    [sym_wc_data_field] = ACTIONS(1949),
    [sym_wc_data_record] = ACTIONS(1949),
    [sym_last_login_info] = ACTIONS(1949),
    [sym____reserved___] = ACTIONS(1949),
    [sym_date] = ACTIONS(1949),
    [sym_day] = ACTIONS(1949),
    [sym_time] = ACTIONS(1949),
    [sym_ldate] = ACTIONS(1949),
    [sym_push_device] = ACTIONS(1949),
    [sym_push_channel] = ACTIONS(1949),
    [sym_push_devchan] = ACTIONS(1949),
    [sym_release_device] = ACTIONS(1949),
    [sym_release_channel] = ACTIONS(1949),
    [sym_release_devchan] = ACTIONS(1949),
    [sym_master_slot] = ACTIONS(1949),
    [sym_get_pulse_time] = ACTIONS(1949),
    [sym_get_timer] = ACTIONS(1949),
    [sym_master_sn] = ACTIONS(1949),
    [sym_system_number] = ACTIONS(1949),
    [sym_button] = ACTIONS(1949),
    [sym_dv_channel] = ACTIONS(1949),
    [sym_data] = ACTIONS(1949),
    [sym_level] = ACTIONS(1949),
    [sym_timeline] = ACTIONS(1949),
    [sym_channel] = ACTIONS(1949),
    [sym_custom] = ACTIONS(1949),
    [sym_atoi] = ACTIONS(1949),
    [sym_atol] = ACTIONS(1949),
    [sym_atof] = ACTIONS(1949),
    [sym_itoa] = ACTIONS(1949),
    [sym_itohex] = ACTIONS(1949),
    [sym_hextoi] = ACTIONS(1949),
    [sym_ftoa] = ACTIONS(1949),
    [sym_find_string] = ACTIONS(1949),
    [sym_left_string] = ACTIONS(1949),
    [sym_right_string] = ACTIONS(1949),
    [sym_mid_string] = ACTIONS(1949),
    [sym_remove_string] = ACTIONS(1949),
    [sym_get_buffer_string] = ACTIONS(1949),
    [sym_get_buffer_char] = ACTIONS(1949),
    [sym_get_multi_buffer_string] = ACTIONS(1949),
    [sym_length_string] = ACTIONS(1949),
    [sym_max_length_string] = ACTIONS(1949),
    [sym_set_length_string] = ACTIONS(1949),
    [sym_compare_string] = ACTIONS(1949),
    [sym_upper_string] = ACTIONS(1949),
    [sym_lower_string] = ACTIONS(1949),
    [sym_get_last] = ACTIONS(1949),
    [sym_date_to_day] = ACTIONS(1949),
    [sym_date_to_month] = ACTIONS(1949),
    [sym_date_to_year] = ACTIONS(1949),
    [sym_time_to_hour] = ACTIONS(1949),
    [sym_time_to_minute] = ACTIONS(1949),
    [sym_time_to_second] = ACTIONS(1949),
    [sym_day_of_week] = ACTIONS(1949),
    [sym_set_pulse_time] = ACTIONS(1949),
    [sym_set_timer] = ACTIONS(1949),
    [sym_file_open] = ACTIONS(1949),
    [sym_file_close] = ACTIONS(1949),
    [sym_file_read] = ACTIONS(1949),
    [sym_file_read_line] = ACTIONS(1949),
    [sym_file_write] = ACTIONS(1949),
    [sym_file_write_line] = ACTIONS(1949),
    [sym_file_copy] = ACTIONS(1949),
    [sym_file_delete] = ACTIONS(1949),
    [sym_file_rename] = ACTIONS(1949),
    [sym_file_dir] = ACTIONS(1949),
    [sym_file_getdir] = ACTIONS(1949),
    [sym_file_setdir] = ACTIONS(1949),
    [sym_file_create_dir] = ACTIONS(1949),
    [sym_file_remove_dir] = ACTIONS(1949),
    [sym_file_seek] = ACTIONS(1949),
    [sym_length_array] = ACTIONS(1949),
    [sym_max_length_array] = ACTIONS(1949),
    [sym_set_length_array] = ACTIONS(1949),
    [sym_ip_client_open] = ACTIONS(1949),
    [sym_ip_client_close] = ACTIONS(1949),
    [sym_ip_server_open] = ACTIONS(1949),
    [sym_ip_server_close] = ACTIONS(1949),
    [sym_ip_bound_client_open] = ACTIONS(1949),
    [sym_ip_mc_server_open] = ACTIONS(1949),
    [sym_ip_set_option] = ACTIONS(1949),
    [sym_ssh_client_open] = ACTIONS(1949),
    [sym_ssh_client_close] = ACTIONS(1949),
    [sym_tls_client_open] = ACTIONS(1949),
    [sym_tls_client_close] = ACTIONS(1949),
    [sym_do_push] = ACTIONS(1949),
    [sym_do_push_timed] = ACTIONS(1949),
    [sym_do_release] = ACTIONS(1949),
    [sym_device_id] = ACTIONS(1949),
    [sym_device_info] = ACTIONS(1949),
    [sym_device_id_string] = ACTIONS(1949),
    [sym_device_standby] = ACTIONS(1949),
    [sym_device_wake] = ACTIONS(1949),
    [sym_timeline_create] = ACTIONS(1949),
    [sym_timeline_kill] = ACTIONS(1949),
    [sym_timeline_reload] = ACTIONS(1949),
    [sym_timeline_pause] = ACTIONS(1949),
    [sym_timeline_restart] = ACTIONS(1949),
    [sym_timeline_set] = ACTIONS(1949),
    [sym_timeline_get] = ACTIONS(1949),
    [sym_timeline_active] = ACTIONS(1949),
    [sym_timeline_dynamic_id] = ACTIONS(1949),
    [sym_variable_to_string] = ACTIONS(1949),
    [sym_string_to_variable] = ACTIONS(1949),
    [sym_length_variable_to_string] = ACTIONS(1949),
    [sym_type_cast] = ACTIONS(1949),
    [sym_variable_to_xml] = ACTIONS(1949),
    [sym_xml_to_variable] = ACTIONS(1949),
    [sym_length_variable_to_xml] = ACTIONS(1949),
    [sym_redirect_string] = ACTIONS(1949),
    [sym_random_number] = ACTIONS(1949),
    [sym_get_unique_id] = ACTIONS(1949),
    [sym_get_system_number] = ACTIONS(1949),
    [sym_set_system_number] = ACTIONS(1949),
    [sym_get_serial_number] = ACTIONS(1949),
    [sym_get_master_build] = ACTIONS(1949),
    [sym_set_validation_code] = ACTIONS(1949),
    [sym_reboot] = ACTIONS(1949),
    [sym_load_duet_module] = ACTIONS(1949),
    [sym_unload_duet_module] = ACTIONS(1949),
    [sym_duet_mem_size_set] = ACTIONS(1949),
    [sym_duet_mem_size_get] = ACTIONS(1949),
    [sym_dynamic_application_device] = ACTIONS(1949),
    [sym_dynamic_polled_port] = ACTIONS(1949),
    [sym_static_port_binding] = ACTIONS(1949),
    [sym_static_ip_binding] = ACTIONS(1949),
    [sym_get_dns_list] = ACTIONS(1949),
    [sym_set_dns_list] = ACTIONS(1949),
    [sym_get_ip_address] = ACTIONS(1949),
    [sym_set_ip_address] = ACTIONS(1949),
    [sym_get_url_list] = ACTIONS(1949),
    [sym_add_url_entry] = ACTIONS(1949),
    [sym_delete_url_entry] = ACTIONS(1949),
    [sym_abs_value] = ACTIONS(1949),
    [sym_max_value] = ACTIONS(1949),
    [sym_min_value] = ACTIONS(1949),
    [sym_exp_value] = ACTIONS(1949),
    [sym_log_value] = ACTIONS(1949),
    [sym_log10_value] = ACTIONS(1949),
    [sym_power_value] = ACTIONS(1949),
    [sym_sqrt_value] = ACTIONS(1949),
    [sym_raw_be] = ACTIONS(1949),
    [sym_raw_le] = ACTIONS(1949),
    [sym_internal_queue_size_set] = ACTIONS(1949),
    [sym_internal_queue_size_get] = ACTIONS(1949),
    [sym_internal_threshold_set] = ACTIONS(1949),
    [sym_internal_threshold_get] = ACTIONS(1949),
    [sym_rebuilt_event] = ACTIONS(1949),
    [sym_do_custom_event] = ACTIONS(1949),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1949),
    [sym_clkmgr_set_clk_source] = ACTIONS(1949),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1949),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1949),
    [sym_clkmgr_get_timezone] = ACTIONS(1949),
    [sym_clkmgr_set_timezone] = ACTIONS(1949),
    [sym_clkmgr_get_resync_period] = ACTIONS(1949),
    [sym_clkmgr_set_resync_period] = ACTIONS(1949),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1949),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1949),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1949),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1949),
    [sym_clkmgr_get_timeservers] = ACTIONS(1949),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1949),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1949),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1949),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1949),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1949),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1949),
    [sym_set_log_level] = ACTIONS(1949),
    [sym_get_log_level] = ACTIONS(1949),
    [sym_amx_log] = ACTIONS(1949),
    [sym_smtp_server_config_set] = ACTIONS(1949),
    [sym_smtp_server_config_get] = ACTIONS(1949),
    [sym_smtp_send] = ACTIONS(1949),
    [sym_validate_netlinx_account] = ACTIONS(1949),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1949),
    [sym_authenticate_certificate] = ACTIONS(1949),
    [sym_audit_netlinx_session_event] = ACTIONS(1949),
    [sym_audit_netlinx_generic_event] = ACTIONS(1949),
    [sym_get_available_flash_disk_space] = ACTIONS(1949),
    [sym_get_max_flash_disk_space] = ACTIONS(1949),
    [sym_data_create_feed] = ACTIONS(1949),
    [sym_data_delete_feed] = ACTIONS(1949),
    [sym_data_publish_feed] = ACTIONS(1949),
    [sym_data_get_published_feed] = ACTIONS(1949),
    [sym_data_add_record] = ACTIONS(1949),
    [sym_data_get_event_record] = ACTIONS(1949),
    [sym__wc_data_create_feed] = ACTIONS(1949),
    [sym__wc_data_add_record] = ACTIONS(1949),
    [sym__wc_data_get_event_record] = ACTIONS(1949),
    [sym_format] = ACTIONS(1949),
    [sym_astro_clock] = ACTIONS(1949),
    [sym_set_outdoor_temperature] = ACTIONS(1949),
    [sym_set_virtual_level_count] = ACTIONS(1949),
    [sym_set_virtual_channel_count] = ACTIONS(1949),
    [sym_set_virtual_port_count] = ACTIONS(1949),
    [sym__wc_encode_int] = ACTIONS(1949),
    [sym__wc_decode_int] = ACTIONS(1949),
    [sym__wc_int] = ACTIONS(1949),
    [sym__wc_to_ch_int] = ACTIONS(1949),
    [sym__ch_to_wc_int] = ACTIONS(1949),
    [sym__wc_find_string_int] = ACTIONS(1949),
    [sym__wc_left_string_int] = ACTIONS(1949),
    [sym__wc_lower_string_int] = ACTIONS(1949),
    [sym__wc_mid_string_int] = ACTIONS(1949),
    [sym__wc_remove_string_int] = ACTIONS(1949),
    [sym__wc_right_string_int] = ACTIONS(1949),
    [sym__wc_upper_string_int] = ACTIONS(1949),
    [sym__wc_compare_string_int] = ACTIONS(1949),
    [sym__wc_get_buffer_char_int] = ACTIONS(1949),
    [sym__wc_get_buffer_string_int] = ACTIONS(1949),
    [sym__wc_concat_string_int] = ACTIONS(1949),
    [sym__wc_file_open_int] = ACTIONS(1949),
    [sym__wc_file_close_int] = ACTIONS(1949),
    [sym__wc_file_read_int] = ACTIONS(1949),
    [sym__wc_file_read_line_int] = ACTIONS(1949),
    [sym__wc_file_write_int] = ACTIONS(1949),
    [sym__wc_file_write_line_int] = ACTIONS(1949),
    [aux_sym_preproc_include_token1] = ACTIONS(1949),
    [aux_sym_preproc_define_token1] = ACTIONS(1951),
    [aux_sym_preproc_warn_token1] = ACTIONS(1951),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1951),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1951),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1951),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1951),
    [aux_sym_preproc_else_token1] = ACTIONS(1951),
    [anon_sym_LBRACK] = ACTIONS(1951),
    [anon_sym_LBRACE] = ACTIONS(1951),
    [anon_sym_LPAREN] = ACTIONS(1949),
    [anon_sym_BANG] = ACTIONS(1951),
    [anon_sym_TILDE] = ACTIONS(1951),
    [anon_sym_DASH] = ACTIONS(1949),
    [anon_sym_PLUS] = ACTIONS(1949),
    [anon_sym_DASH_DASH] = ACTIONS(1951),
    [anon_sym_PLUS_PLUS] = ACTIONS(1951),
    [anon_sym_DQUOTE] = ACTIONS(1951),
    [anon_sym_SQUOTE] = ACTIONS(1951),
    [sym_number_literal] = ACTIONS(1951),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(291)] = {
    [sym_identifier] = ACTIONS(1953),
    [sym_program_name_keyword] = ACTIONS(1953),
    [sym_module_name_keyword] = ACTIONS(1953),
    [sym_define_device_keyword] = ACTIONS(1953),
    [sym_define_combine_keyword] = ACTIONS(1953),
    [sym_define_constant_keyword] = ACTIONS(1953),
    [sym_define_type_keyword] = ACTIONS(1953),
    [sym_define_variable_keyword] = ACTIONS(1953),
    [sym_define_system_variable_keyword] = ACTIONS(1953),
    [sym_define_connect_level_keyword] = ACTIONS(1953),
    [sym_define_latching_keyword] = ACTIONS(1953),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1953),
    [sym_define_toggling_keyword] = ACTIONS(1953),
    [sym_define_call_keyword] = ACTIONS(1953),
    [sym_define_function_keyword] = ACTIONS(1953),
    [sym_define_library_function_keyword] = ACTIONS(1953),
    [sym_define_start_keyword] = ACTIONS(1953),
    [sym_define_event_keyword] = ACTIONS(1953),
    [sym_define_program_keyword] = ACTIONS(1953),
    [sym_define_module_keyword] = ACTIONS(1953),
    [sym_char_keyword] = ACTIONS(1953),
    [sym_widechar_keyword] = ACTIONS(1953),
    [sym_integer_keyword] = ACTIONS(1953),
    [sym_sinteger_keyword] = ACTIONS(1953),
    [sym_long_keyword] = ACTIONS(1953),
    [sym_slong_keyword] = ACTIONS(1953),
    [sym_float_keyword] = ACTIONS(1953),
    [sym_double_keyword] = ACTIONS(1953),
    [sym_variant_keyword] = ACTIONS(1953),
    [sym_variantarray_keyword] = ACTIONS(1953),
    [sym_local_var_keyword] = ACTIONS(1953),
    [sym_stack_var_keyword] = ACTIONS(1953),
    [sym_constant_keyword] = ACTIONS(1953),
    [sym_volatile_keyword] = ACTIONS(1953),
    [sym_non_volatile_keyword] = ACTIONS(1953),
    [sym_persistent_keyword] = ACTIONS(1953),
    [sym_dev_keyword] = ACTIONS(1953),
    [sym_devlev_keyword] = ACTIONS(1953),
    [sym_devchan_keyword] = ACTIONS(1953),
    [sym_if_keyword] = ACTIONS(1953),
    [sym_switch_keyword] = ACTIONS(1953),
    [sym_case_keyword] = ACTIONS(1953),
    [sym_default_keyword] = ACTIONS(1953),
    [sym_while_keyword] = ACTIONS(1953),
    [sym_for_keyword] = ACTIONS(1953),
    [sym_break_keyword] = ACTIONS(1953),
    [sym_continue_keyword] = ACTIONS(1953),
    [sym_return_keyword] = ACTIONS(1953),
    [sym_select_keyword] = ACTIONS(1953),
    [sym_active_keyword] = ACTIONS(1953),
    [sym_struct_keyword] = ACTIONS(1953),
    [sym_structure_keyword] = ACTIONS(1953),
    [sym_button_event_keyword] = ACTIONS(1953),
    [sym_channel_event_keyword] = ACTIONS(1953),
    [sym_level_event_keyword] = ACTIONS(1953),
    [sym_data_event_keyword] = ACTIONS(1953),
    [sym_timeline_event_keyword] = ACTIONS(1953),
    [sym_custom_event_keyword] = ACTIONS(1953),
    [sym_push_keyword] = ACTIONS(1953),
    [sym_release_keyword] = ACTIONS(1953),
    [aux_sym_on_keyword_token1] = ACTIONS(1953),
    [aux_sym_off_keyword_token1] = ACTIONS(1953),
    [sym_send_level_keyword] = ACTIONS(1953),
    [sym_send_string_keyword] = ACTIONS(1953),
    [sym_send_command_keyword] = ACTIONS(1953),
    [sym_clear_buffer_keyword] = ACTIONS(1953),
    [sym_create_buffer_keyword] = ACTIONS(1953),
    [sym_create_multi_buffer_keyword] = ACTIONS(1953),
    [sym_call_keyword] = ACTIONS(1953),
    [sym_system_call_keyword] = ACTIONS(1953),
    [sym_devchan_to_keyword] = ACTIONS(1953),
    [sym_devchan_min_to_keyword] = ACTIONS(1953),
    [sym_devchan_total_off_keyword] = ACTIONS(1953),
    [sym_devchan_pulse_keyword] = ACTIONS(1953),
    [sym_wait_keyword] = ACTIONS(1953),
    [sym_cancel_wait_keyword] = ACTIONS(1953),
    [sym_wait_until_keyword] = ACTIONS(1953),
    [sym_cancel_wait_until_keyword] = ACTIONS(1953),
    [sym_cancel_all_wait_keyword] = ACTIONS(1953),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1953),
    [sym_not] = ACTIONS(1953),
    [sym_bnot] = ACTIONS(1953),
    [sym___file__] = ACTIONS(1953),
    [sym___date__] = ACTIONS(1953),
    [sym___ldate__] = ACTIONS(1953),
    [sym___time__] = ACTIONS(1953),
    [sym___line__] = ACTIONS(1953),
    [sym___name__] = ACTIONS(1953),
    [sym___netlinx__] = ACTIONS(1953),
    [sym_netlinx_axi_version] = ACTIONS(1953),
    [sym_url_flg_tcp] = ACTIONS(1953),
    [sym_url_flg_accinfopresent] = ACTIONS(1953),
    [sym_url_flg_temp] = ACTIONS(1953),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1953),
    [sym_url_flg_stat_mask] = ACTIONS(1953),
    [sym_url_flg_stat_lookup] = ACTIONS(1953),
    [sym_url_flg_stat_connecting] = ACTIONS(1953),
    [sym_url_flg_stat_waiting] = ACTIONS(1953),
    [sym_url_flg_stat_connected] = ACTIONS(1953),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1953),
    [sym_first_virtual_device] = ACTIONS(1953),
    [sym_first_local_port] = ACTIONS(1953),
    [sym_dynamic_virtual_device] = ACTIONS(1953),
    [sym_timeline_once] = ACTIONS(1953),
    [sym_timeline_repeat] = ACTIONS(1953),
    [sym_timeline_absolute] = ACTIONS(1953),
    [sym_timeline_relative] = ACTIONS(1953),
    [sym_do_push_timed_infinite] = ACTIONS(1953),
    [sym_source_type_no_address] = ACTIONS(1953),
    [sym_source_type_neuron_id] = ACTIONS(1953),
    [sym_source_type_ip_address] = ACTIONS(1953),
    [sym_source_type_axlink] = ACTIONS(1953),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1953),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1953),
    [sym_source_type_ip_socket_address] = ACTIONS(1953),
    [sym_source_type_rs232] = ACTIONS(1953),
    [sym_source_type_internal] = ACTIONS(1953),
    [sym_source_type_mac_address] = ACTIONS(1953),
    [sym_source_type_ipv4_port] = ACTIONS(1953),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1953),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1953),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1953),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1953),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1953),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1953),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1953),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1953),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1953),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1953),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1953),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1953),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1953),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1953),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1953),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1953),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1953),
    [sym_true] = ACTIONS(1953),
    [sym_false] = ACTIONS(1953),
    [sym_file_read_only] = ACTIONS(1953),
    [sym_file_rw_new] = ACTIONS(1953),
    [sym_file_rw_append] = ACTIONS(1953),
    [sym_ip_tcp] = ACTIONS(1953),
    [sym_ip_udp] = ACTIONS(1953),
    [sym_ip_udp_2way] = ACTIONS(1953),
    [sym_xml_encode_types] = ACTIONS(1953),
    [sym_xml_encode_char_as_list] = ACTIONS(1953),
    [sym_xml_encode_le] = ACTIONS(1953),
    [sym_xml_decode_types] = ACTIONS(1953),
    [sym_xml_decode_no_preserve] = ACTIONS(1953),
    [sym_duet_dev_type_display_device] = ACTIONS(1953),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1953),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1953),
    [sym_duet_dev_type_io_device] = ACTIONS(1953),
    [sym_duet_dev_type_relay_device] = ACTIONS(1953),
    [sym_duet_dev_type_ups] = ACTIONS(1953),
    [sym_duet_dev_type_amplifier] = ACTIONS(1953),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1953),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1953),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1953),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1953),
    [sym_duet_dev_type_camera] = ACTIONS(1953),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1953),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1953),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1953),
    [sym_duet_dev_type_dss] = ACTIONS(1953),
    [sym_duet_dev_type_dvr] = ACTIONS(1953),
    [sym_duet_dev_type_disc_device] = ACTIONS(1953),
    [sym_duet_dev_type_document_camera] = ACTIONS(1953),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1953),
    [sym_duet_dev_type_hvac] = ACTIONS(1953),
    [sym_duet_dev_type_keypad] = ACTIONS(1953),
    [sym_duet_dev_type_light] = ACTIONS(1953),
    [sym_duet_dev_type_monitor] = ACTIONS(1953),
    [sym_duet_dev_type_motor] = ACTIONS(1953),
    [sym_duet_dev_type_multi_window] = ACTIONS(1953),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1953),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1953),
    [sym_duet_dev_type_receiver] = ACTIONS(1953),
    [sym_duet_dev_type_security_system] = ACTIONS(1953),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1953),
    [sym_duet_dev_type_settop_box] = ACTIONS(1953),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1953),
    [sym_duet_dev_type_switcher] = ACTIONS(1953),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1953),
    [sym_duet_dev_type_tv] = ACTIONS(1953),
    [sym_duet_dev_type_utility] = ACTIONS(1953),
    [sym_duet_dev_type_vcr] = ACTIONS(1953),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1953),
    [sym_duet_dev_type_video_processor] = ACTIONS(1953),
    [sym_duet_dev_type_video_projector] = ACTIONS(1953),
    [sym_duet_dev_type_video_wall] = ACTIONS(1953),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1953),
    [sym_duet_dev_type_weather] = ACTIONS(1953),
    [sym_duet_dev_not_polled] = ACTIONS(1953),
    [sym_duet_dev_polled] = ACTIONS(1953),
    [sym_clkmgr_mode_network] = ACTIONS(1953),
    [sym_clkmgr_mode_standalone] = ACTIONS(1953),
    [sym_normal_standby] = ACTIONS(1953),
    [sym_normal_wake] = ACTIONS(1953),
    [sym_ip_multicast_ttl_option] = ACTIONS(1953),
    [sym_ip_ttl_subnet] = ACTIONS(1953),
    [sym_ip_ttl_site] = ACTIONS(1953),
    [sym_ip_ttl_region] = ACTIONS(1953),
    [sym_ip_ttl_continent] = ACTIONS(1953),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1953),
    [sym_ip_nodelay_on] = ACTIONS(1953),
    [sym_ip_nodelay_off] = ACTIONS(1953),
    [sym_amx_error] = ACTIONS(1953),
    [sym_amx_warning] = ACTIONS(1953),
    [sym_amx_info] = ACTIONS(1953),
    [sym_amx_debug] = ACTIONS(1953),
    [sym_netlinx_login_success] = ACTIONS(1953),
    [sym_netlinx_login_fail] = ACTIONS(1953),
    [sym_netlinx_logout] = ACTIONS(1953),
    [sym_null_str] = ACTIONS(1953),
    [sym_smtp_address] = ACTIONS(1953),
    [sym_smtp_port_number] = ACTIONS(1953),
    [sym_smtp_username] = ACTIONS(1953),
    [sym_smtp_password] = ACTIONS(1953),
    [sym_smtp_require_tls] = ACTIONS(1953),
    [sym_smtp_from] = ACTIONS(1953),
    [sym_smtp_tls_true] = ACTIONS(1953),
    [sym_smtp_tls_false] = ACTIONS(1953),
    [sym_listview_on_row_select_event] = ACTIONS(1953),
    [sym_data_structure_unknown] = ACTIONS(1953),
    [sym_data_structure_datafeed] = ACTIONS(1953),
    [sym_data_structure_datarecordset] = ACTIONS(1953),
    [sym_data_structure_datarecord] = ACTIONS(1953),
    [sym_data_structure_datafield] = ACTIONS(1953),
    [sym_data_max_content_field_count] = ACTIONS(1953),
    [sym_data_max_metadata_field_count] = ACTIONS(1953),
    [sym_data_max_id_length] = ACTIONS(1953),
    [sym_data_max_name_length] = ACTIONS(1953),
    [sym_data_max_description_length] = ACTIONS(1953),
    [sym_data_max_type_length] = ACTIONS(1953),
    [sym_data_max_format_length] = ACTIONS(1953),
    [sym_data_max_source_length] = ACTIONS(1953),
    [sym_data_max_value_length] = ACTIONS(1953),
    [sym_data_max_label_length] = ACTIONS(1953),
    [sym_data_max_url_length] = ACTIONS(1953),
    [sym_max_last_login_info_length] = ACTIONS(1953),
    [sym_data_type_unknown] = ACTIONS(1953),
    [sym_data_type_string] = ACTIONS(1953),
    [sym_data_type_datetime] = ACTIONS(1953),
    [sym_data_type_date] = ACTIONS(1953),
    [sym_data_type_time] = ACTIONS(1953),
    [sym_data_type_image] = ACTIONS(1953),
    [sym_data_format_url] = ACTIONS(1953),
    [sym_data_format_phone] = ACTIONS(1953),
    [sym_data_format_email] = ACTIONS(1953),
    [sym_data_format_iso8601] = ACTIONS(1953),
    [sym_auditlog_priv] = ACTIONS(1953),
    [sym_configuration_priv] = ACTIONS(1953),
    [sym_deviceconfig_priv] = ACTIONS(1953),
    [sym_ftp_priv] = ACTIONS(1953),
    [sym_http_priv] = ACTIONS(1953),
    [sym_networkconfig_priv] = ACTIONS(1953),
    [sym_programport_priv] = ACTIONS(1953),
    [sym_remoteui_priv] = ACTIONS(1953),
    [sym_securitycontrol_priv] = ACTIONS(1953),
    [sym_softwaremanagement_priv] = ACTIONS(1953),
    [sym_terminal_priv] = ACTIONS(1953),
    [sym_tpadmin_priv] = ACTIONS(1953),
    [sym_user1_priv] = ACTIONS(1953),
    [sym_user2_priv] = ACTIONS(1953),
    [sym_user3_priv] = ACTIONS(1953),
    [sym_user4_priv] = ACTIONS(1953),
    [sym_usermanagement_priv] = ACTIONS(1953),
    [sym_valid_account] = ACTIONS(1953),
    [sym_err_name_invalid] = ACTIONS(1953),
    [sym_err_password_invalid] = ACTIONS(1953),
    [sym_err_invalid_account] = ACTIONS(1953),
    [sym_err_no_authorization] = ACTIONS(1953),
    [sym_err_invalid_parameter] = ACTIONS(1953),
    [sym_err_user_locked_out] = ACTIONS(1953),
    [sym_err_user_account_expired] = ACTIONS(1953),
    [sym_err_general_error] = ACTIONS(1953),
    [sym_err_authentication_error] = ACTIONS(1953),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1953),
    [sym_tls_validate_certificate] = ACTIONS(1953),
    [sym_tdata] = ACTIONS(1953),
    [sym_tchannel] = ACTIONS(1953),
    [sym_tlevel] = ACTIONS(1953),
    [sym_tbutton] = ACTIONS(1953),
    [sym_ttimeline] = ACTIONS(1953),
    [sym_tcustom] = ACTIONS(1953),
    [sym_url_struct] = ACTIONS(1953),
    [sym_dns_struct] = ACTIONS(1953),
    [sym_ip_address_struct] = ACTIONS(1953),
    [sym_dev_info_struct] = ACTIONS(1953),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1953),
    [sym_data_feed] = ACTIONS(1953),
    [sym_data_field] = ACTIONS(1953),
    [sym_data_record] = ACTIONS(1953),
    [sym_wc_data_feed] = ACTIONS(1953),
    [sym_wc_data_field] = ACTIONS(1953),
    [sym_wc_data_record] = ACTIONS(1953),
    [sym_last_login_info] = ACTIONS(1953),
    [sym____reserved___] = ACTIONS(1953),
    [sym_date] = ACTIONS(1953),
    [sym_day] = ACTIONS(1953),
    [sym_time] = ACTIONS(1953),
    [sym_ldate] = ACTIONS(1953),
    [sym_push_device] = ACTIONS(1953),
    [sym_push_channel] = ACTIONS(1953),
    [sym_push_devchan] = ACTIONS(1953),
    [sym_release_device] = ACTIONS(1953),
    [sym_release_channel] = ACTIONS(1953),
    [sym_release_devchan] = ACTIONS(1953),
    [sym_master_slot] = ACTIONS(1953),
    [sym_get_pulse_time] = ACTIONS(1953),
    [sym_get_timer] = ACTIONS(1953),
    [sym_master_sn] = ACTIONS(1953),
    [sym_system_number] = ACTIONS(1953),
    [sym_button] = ACTIONS(1953),
    [sym_dv_channel] = ACTIONS(1953),
    [sym_data] = ACTIONS(1953),
    [sym_level] = ACTIONS(1953),
    [sym_timeline] = ACTIONS(1953),
    [sym_channel] = ACTIONS(1953),
    [sym_custom] = ACTIONS(1953),
    [sym_atoi] = ACTIONS(1953),
    [sym_atol] = ACTIONS(1953),
    [sym_atof] = ACTIONS(1953),
    [sym_itoa] = ACTIONS(1953),
    [sym_itohex] = ACTIONS(1953),
    [sym_hextoi] = ACTIONS(1953),
    [sym_ftoa] = ACTIONS(1953),
    [sym_find_string] = ACTIONS(1953),
    [sym_left_string] = ACTIONS(1953),
    [sym_right_string] = ACTIONS(1953),
    [sym_mid_string] = ACTIONS(1953),
    [sym_remove_string] = ACTIONS(1953),
    [sym_get_buffer_string] = ACTIONS(1953),
    [sym_get_buffer_char] = ACTIONS(1953),
    [sym_get_multi_buffer_string] = ACTIONS(1953),
    [sym_length_string] = ACTIONS(1953),
    [sym_max_length_string] = ACTIONS(1953),
    [sym_set_length_string] = ACTIONS(1953),
    [sym_compare_string] = ACTIONS(1953),
    [sym_upper_string] = ACTIONS(1953),
    [sym_lower_string] = ACTIONS(1953),
    [sym_get_last] = ACTIONS(1953),
    [sym_date_to_day] = ACTIONS(1953),
    [sym_date_to_month] = ACTIONS(1953),
    [sym_date_to_year] = ACTIONS(1953),
    [sym_time_to_hour] = ACTIONS(1953),
    [sym_time_to_minute] = ACTIONS(1953),
    [sym_time_to_second] = ACTIONS(1953),
    [sym_day_of_week] = ACTIONS(1953),
    [sym_set_pulse_time] = ACTIONS(1953),
    [sym_set_timer] = ACTIONS(1953),
    [sym_file_open] = ACTIONS(1953),
    [sym_file_close] = ACTIONS(1953),
    [sym_file_read] = ACTIONS(1953),
    [sym_file_read_line] = ACTIONS(1953),
    [sym_file_write] = ACTIONS(1953),
    [sym_file_write_line] = ACTIONS(1953),
    [sym_file_copy] = ACTIONS(1953),
    [sym_file_delete] = ACTIONS(1953),
    [sym_file_rename] = ACTIONS(1953),
    [sym_file_dir] = ACTIONS(1953),
    [sym_file_getdir] = ACTIONS(1953),
    [sym_file_setdir] = ACTIONS(1953),
    [sym_file_create_dir] = ACTIONS(1953),
    [sym_file_remove_dir] = ACTIONS(1953),
    [sym_file_seek] = ACTIONS(1953),
    [sym_length_array] = ACTIONS(1953),
    [sym_max_length_array] = ACTIONS(1953),
    [sym_set_length_array] = ACTIONS(1953),
    [sym_ip_client_open] = ACTIONS(1953),
    [sym_ip_client_close] = ACTIONS(1953),
    [sym_ip_server_open] = ACTIONS(1953),
    [sym_ip_server_close] = ACTIONS(1953),
    [sym_ip_bound_client_open] = ACTIONS(1953),
    [sym_ip_mc_server_open] = ACTIONS(1953),
    [sym_ip_set_option] = ACTIONS(1953),
    [sym_ssh_client_open] = ACTIONS(1953),
    [sym_ssh_client_close] = ACTIONS(1953),
    [sym_tls_client_open] = ACTIONS(1953),
    [sym_tls_client_close] = ACTIONS(1953),
    [sym_do_push] = ACTIONS(1953),
    [sym_do_push_timed] = ACTIONS(1953),
    [sym_do_release] = ACTIONS(1953),
    [sym_device_id] = ACTIONS(1953),
    [sym_device_info] = ACTIONS(1953),
    [sym_device_id_string] = ACTIONS(1953),
    [sym_device_standby] = ACTIONS(1953),
    [sym_device_wake] = ACTIONS(1953),
    [sym_timeline_create] = ACTIONS(1953),
    [sym_timeline_kill] = ACTIONS(1953),
    [sym_timeline_reload] = ACTIONS(1953),
    [sym_timeline_pause] = ACTIONS(1953),
    [sym_timeline_restart] = ACTIONS(1953),
    [sym_timeline_set] = ACTIONS(1953),
    [sym_timeline_get] = ACTIONS(1953),
    [sym_timeline_active] = ACTIONS(1953),
    [sym_timeline_dynamic_id] = ACTIONS(1953),
    [sym_variable_to_string] = ACTIONS(1953),
    [sym_string_to_variable] = ACTIONS(1953),
    [sym_length_variable_to_string] = ACTIONS(1953),
    [sym_type_cast] = ACTIONS(1953),
    [sym_variable_to_xml] = ACTIONS(1953),
    [sym_xml_to_variable] = ACTIONS(1953),
    [sym_length_variable_to_xml] = ACTIONS(1953),
    [sym_redirect_string] = ACTIONS(1953),
    [sym_random_number] = ACTIONS(1953),
    [sym_get_unique_id] = ACTIONS(1953),
    [sym_get_system_number] = ACTIONS(1953),
    [sym_set_system_number] = ACTIONS(1953),
    [sym_get_serial_number] = ACTIONS(1953),
    [sym_get_master_build] = ACTIONS(1953),
    [sym_set_validation_code] = ACTIONS(1953),
    [sym_reboot] = ACTIONS(1953),
    [sym_load_duet_module] = ACTIONS(1953),
    [sym_unload_duet_module] = ACTIONS(1953),
    [sym_duet_mem_size_set] = ACTIONS(1953),
    [sym_duet_mem_size_get] = ACTIONS(1953),
    [sym_dynamic_application_device] = ACTIONS(1953),
    [sym_dynamic_polled_port] = ACTIONS(1953),
    [sym_static_port_binding] = ACTIONS(1953),
    [sym_static_ip_binding] = ACTIONS(1953),
    [sym_get_dns_list] = ACTIONS(1953),
    [sym_set_dns_list] = ACTIONS(1953),
    [sym_get_ip_address] = ACTIONS(1953),
    [sym_set_ip_address] = ACTIONS(1953),
    [sym_get_url_list] = ACTIONS(1953),
    [sym_add_url_entry] = ACTIONS(1953),
    [sym_delete_url_entry] = ACTIONS(1953),
    [sym_abs_value] = ACTIONS(1953),
    [sym_max_value] = ACTIONS(1953),
    [sym_min_value] = ACTIONS(1953),
    [sym_exp_value] = ACTIONS(1953),
    [sym_log_value] = ACTIONS(1953),
    [sym_log10_value] = ACTIONS(1953),
    [sym_power_value] = ACTIONS(1953),
    [sym_sqrt_value] = ACTIONS(1953),
    [sym_raw_be] = ACTIONS(1953),
    [sym_raw_le] = ACTIONS(1953),
    [sym_internal_queue_size_set] = ACTIONS(1953),
    [sym_internal_queue_size_get] = ACTIONS(1953),
    [sym_internal_threshold_set] = ACTIONS(1953),
    [sym_internal_threshold_get] = ACTIONS(1953),
    [sym_rebuilt_event] = ACTIONS(1953),
    [sym_do_custom_event] = ACTIONS(1953),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1953),
    [sym_clkmgr_set_clk_source] = ACTIONS(1953),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1953),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1953),
    [sym_clkmgr_get_timezone] = ACTIONS(1953),
    [sym_clkmgr_set_timezone] = ACTIONS(1953),
    [sym_clkmgr_get_resync_period] = ACTIONS(1953),
    [sym_clkmgr_set_resync_period] = ACTIONS(1953),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1953),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1953),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1953),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1953),
    [sym_clkmgr_get_timeservers] = ACTIONS(1953),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1953),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1953),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1953),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1953),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1953),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1953),
    [sym_set_log_level] = ACTIONS(1953),
    [sym_get_log_level] = ACTIONS(1953),
    [sym_amx_log] = ACTIONS(1953),
    [sym_smtp_server_config_set] = ACTIONS(1953),
    [sym_smtp_server_config_get] = ACTIONS(1953),
    [sym_smtp_send] = ACTIONS(1953),
    [sym_validate_netlinx_account] = ACTIONS(1953),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1953),
    [sym_authenticate_certificate] = ACTIONS(1953),
    [sym_audit_netlinx_session_event] = ACTIONS(1953),
    [sym_audit_netlinx_generic_event] = ACTIONS(1953),
    [sym_get_available_flash_disk_space] = ACTIONS(1953),
    [sym_get_max_flash_disk_space] = ACTIONS(1953),
    [sym_data_create_feed] = ACTIONS(1953),
    [sym_data_delete_feed] = ACTIONS(1953),
    [sym_data_publish_feed] = ACTIONS(1953),
    [sym_data_get_published_feed] = ACTIONS(1953),
    [sym_data_add_record] = ACTIONS(1953),
    [sym_data_get_event_record] = ACTIONS(1953),
    [sym__wc_data_create_feed] = ACTIONS(1953),
    [sym__wc_data_add_record] = ACTIONS(1953),
    [sym__wc_data_get_event_record] = ACTIONS(1953),
    [sym_format] = ACTIONS(1953),
    [sym_astro_clock] = ACTIONS(1953),
    [sym_set_outdoor_temperature] = ACTIONS(1953),
    [sym_set_virtual_level_count] = ACTIONS(1953),
    [sym_set_virtual_channel_count] = ACTIONS(1953),
    [sym_set_virtual_port_count] = ACTIONS(1953),
    [sym__wc_encode_int] = ACTIONS(1953),
    [sym__wc_decode_int] = ACTIONS(1953),
    [sym__wc_int] = ACTIONS(1953),
    [sym__wc_to_ch_int] = ACTIONS(1953),
    [sym__ch_to_wc_int] = ACTIONS(1953),
    [sym__wc_find_string_int] = ACTIONS(1953),
    [sym__wc_left_string_int] = ACTIONS(1953),
    [sym__wc_lower_string_int] = ACTIONS(1953),
    [sym__wc_mid_string_int] = ACTIONS(1953),
    [sym__wc_remove_string_int] = ACTIONS(1953),
    [sym__wc_right_string_int] = ACTIONS(1953),
    [sym__wc_upper_string_int] = ACTIONS(1953),
    [sym__wc_compare_string_int] = ACTIONS(1953),
    [sym__wc_get_buffer_char_int] = ACTIONS(1953),
    [sym__wc_get_buffer_string_int] = ACTIONS(1953),
    [sym__wc_concat_string_int] = ACTIONS(1953),
    [sym__wc_file_open_int] = ACTIONS(1953),
    [sym__wc_file_close_int] = ACTIONS(1953),
    [sym__wc_file_read_int] = ACTIONS(1953),
    [sym__wc_file_read_line_int] = ACTIONS(1953),
    [sym__wc_file_write_int] = ACTIONS(1953),
    [sym__wc_file_write_line_int] = ACTIONS(1953),
    [aux_sym_preproc_include_token1] = ACTIONS(1953),
    [aux_sym_preproc_define_token1] = ACTIONS(1955),
    [aux_sym_preproc_warn_token1] = ACTIONS(1955),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1955),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1955),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1955),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1955),
    [aux_sym_preproc_else_token1] = ACTIONS(1955),
    [anon_sym_LBRACK] = ACTIONS(1955),
    [anon_sym_LBRACE] = ACTIONS(1955),
    [anon_sym_LPAREN] = ACTIONS(1953),
    [anon_sym_BANG] = ACTIONS(1955),
    [anon_sym_TILDE] = ACTIONS(1955),
    [anon_sym_DASH] = ACTIONS(1953),
    [anon_sym_PLUS] = ACTIONS(1953),
    [anon_sym_DASH_DASH] = ACTIONS(1955),
    [anon_sym_PLUS_PLUS] = ACTIONS(1955),
    [anon_sym_DQUOTE] = ACTIONS(1955),
    [anon_sym_SQUOTE] = ACTIONS(1955),
    [sym_number_literal] = ACTIONS(1955),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(292)] = {
    [sym_identifier] = ACTIONS(1957),
    [sym_program_name_keyword] = ACTIONS(1957),
    [sym_module_name_keyword] = ACTIONS(1957),
    [sym_define_device_keyword] = ACTIONS(1957),
    [sym_define_combine_keyword] = ACTIONS(1957),
    [sym_define_constant_keyword] = ACTIONS(1957),
    [sym_define_type_keyword] = ACTIONS(1957),
    [sym_define_variable_keyword] = ACTIONS(1957),
    [sym_define_system_variable_keyword] = ACTIONS(1957),
    [sym_define_connect_level_keyword] = ACTIONS(1957),
    [sym_define_latching_keyword] = ACTIONS(1957),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1957),
    [sym_define_toggling_keyword] = ACTIONS(1957),
    [sym_define_call_keyword] = ACTIONS(1957),
    [sym_define_function_keyword] = ACTIONS(1957),
    [sym_define_library_function_keyword] = ACTIONS(1957),
    [sym_define_start_keyword] = ACTIONS(1957),
    [sym_define_event_keyword] = ACTIONS(1957),
    [sym_define_program_keyword] = ACTIONS(1957),
    [sym_define_module_keyword] = ACTIONS(1957),
    [sym_char_keyword] = ACTIONS(1957),
    [sym_widechar_keyword] = ACTIONS(1957),
    [sym_integer_keyword] = ACTIONS(1957),
    [sym_sinteger_keyword] = ACTIONS(1957),
    [sym_long_keyword] = ACTIONS(1957),
    [sym_slong_keyword] = ACTIONS(1957),
    [sym_float_keyword] = ACTIONS(1957),
    [sym_double_keyword] = ACTIONS(1957),
    [sym_variant_keyword] = ACTIONS(1957),
    [sym_variantarray_keyword] = ACTIONS(1957),
    [sym_local_var_keyword] = ACTIONS(1957),
    [sym_stack_var_keyword] = ACTIONS(1957),
    [sym_constant_keyword] = ACTIONS(1957),
    [sym_volatile_keyword] = ACTIONS(1957),
    [sym_non_volatile_keyword] = ACTIONS(1957),
    [sym_persistent_keyword] = ACTIONS(1957),
    [sym_dev_keyword] = ACTIONS(1957),
    [sym_devlev_keyword] = ACTIONS(1957),
    [sym_devchan_keyword] = ACTIONS(1957),
    [sym_if_keyword] = ACTIONS(1957),
    [sym_switch_keyword] = ACTIONS(1957),
    [sym_case_keyword] = ACTIONS(1957),
    [sym_default_keyword] = ACTIONS(1957),
    [sym_while_keyword] = ACTIONS(1957),
    [sym_for_keyword] = ACTIONS(1957),
    [sym_break_keyword] = ACTIONS(1957),
    [sym_continue_keyword] = ACTIONS(1957),
    [sym_return_keyword] = ACTIONS(1957),
    [sym_select_keyword] = ACTIONS(1957),
    [sym_active_keyword] = ACTIONS(1957),
    [sym_struct_keyword] = ACTIONS(1957),
    [sym_structure_keyword] = ACTIONS(1957),
    [sym_button_event_keyword] = ACTIONS(1957),
    [sym_channel_event_keyword] = ACTIONS(1957),
    [sym_level_event_keyword] = ACTIONS(1957),
    [sym_data_event_keyword] = ACTIONS(1957),
    [sym_timeline_event_keyword] = ACTIONS(1957),
    [sym_custom_event_keyword] = ACTIONS(1957),
    [sym_push_keyword] = ACTIONS(1957),
    [sym_release_keyword] = ACTIONS(1957),
    [aux_sym_on_keyword_token1] = ACTIONS(1957),
    [aux_sym_off_keyword_token1] = ACTIONS(1957),
    [sym_send_level_keyword] = ACTIONS(1957),
    [sym_send_string_keyword] = ACTIONS(1957),
    [sym_send_command_keyword] = ACTIONS(1957),
    [sym_clear_buffer_keyword] = ACTIONS(1957),
    [sym_create_buffer_keyword] = ACTIONS(1957),
    [sym_create_multi_buffer_keyword] = ACTIONS(1957),
    [sym_call_keyword] = ACTIONS(1957),
    [sym_system_call_keyword] = ACTIONS(1957),
    [sym_devchan_to_keyword] = ACTIONS(1957),
    [sym_devchan_min_to_keyword] = ACTIONS(1957),
    [sym_devchan_total_off_keyword] = ACTIONS(1957),
    [sym_devchan_pulse_keyword] = ACTIONS(1957),
    [sym_wait_keyword] = ACTIONS(1957),
    [sym_cancel_wait_keyword] = ACTIONS(1957),
    [sym_wait_until_keyword] = ACTIONS(1957),
    [sym_cancel_wait_until_keyword] = ACTIONS(1957),
    [sym_cancel_all_wait_keyword] = ACTIONS(1957),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1957),
    [sym_not] = ACTIONS(1957),
    [sym_bnot] = ACTIONS(1957),
    [sym___file__] = ACTIONS(1957),
    [sym___date__] = ACTIONS(1957),
    [sym___ldate__] = ACTIONS(1957),
    [sym___time__] = ACTIONS(1957),
    [sym___line__] = ACTIONS(1957),
    [sym___name__] = ACTIONS(1957),
    [sym___netlinx__] = ACTIONS(1957),
    [sym_netlinx_axi_version] = ACTIONS(1957),
    [sym_url_flg_tcp] = ACTIONS(1957),
    [sym_url_flg_accinfopresent] = ACTIONS(1957),
    [sym_url_flg_temp] = ACTIONS(1957),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1957),
    [sym_url_flg_stat_mask] = ACTIONS(1957),
    [sym_url_flg_stat_lookup] = ACTIONS(1957),
    [sym_url_flg_stat_connecting] = ACTIONS(1957),
    [sym_url_flg_stat_waiting] = ACTIONS(1957),
    [sym_url_flg_stat_connected] = ACTIONS(1957),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1957),
    [sym_first_virtual_device] = ACTIONS(1957),
    [sym_first_local_port] = ACTIONS(1957),
    [sym_dynamic_virtual_device] = ACTIONS(1957),
    [sym_timeline_once] = ACTIONS(1957),
    [sym_timeline_repeat] = ACTIONS(1957),
    [sym_timeline_absolute] = ACTIONS(1957),
    [sym_timeline_relative] = ACTIONS(1957),
    [sym_do_push_timed_infinite] = ACTIONS(1957),
    [sym_source_type_no_address] = ACTIONS(1957),
    [sym_source_type_neuron_id] = ACTIONS(1957),
    [sym_source_type_ip_address] = ACTIONS(1957),
    [sym_source_type_axlink] = ACTIONS(1957),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1957),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1957),
    [sym_source_type_ip_socket_address] = ACTIONS(1957),
    [sym_source_type_rs232] = ACTIONS(1957),
    [sym_source_type_internal] = ACTIONS(1957),
    [sym_source_type_mac_address] = ACTIONS(1957),
    [sym_source_type_ipv4_port] = ACTIONS(1957),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1957),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1957),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1957),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1957),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1957),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1957),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1957),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1957),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1957),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1957),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1957),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1957),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1957),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1957),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1957),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1957),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1957),
    [sym_true] = ACTIONS(1957),
    [sym_false] = ACTIONS(1957),
    [sym_file_read_only] = ACTIONS(1957),
    [sym_file_rw_new] = ACTIONS(1957),
    [sym_file_rw_append] = ACTIONS(1957),
    [sym_ip_tcp] = ACTIONS(1957),
    [sym_ip_udp] = ACTIONS(1957),
    [sym_ip_udp_2way] = ACTIONS(1957),
    [sym_xml_encode_types] = ACTIONS(1957),
    [sym_xml_encode_char_as_list] = ACTIONS(1957),
    [sym_xml_encode_le] = ACTIONS(1957),
    [sym_xml_decode_types] = ACTIONS(1957),
    [sym_xml_decode_no_preserve] = ACTIONS(1957),
    [sym_duet_dev_type_display_device] = ACTIONS(1957),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1957),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1957),
    [sym_duet_dev_type_io_device] = ACTIONS(1957),
    [sym_duet_dev_type_relay_device] = ACTIONS(1957),
    [sym_duet_dev_type_ups] = ACTIONS(1957),
    [sym_duet_dev_type_amplifier] = ACTIONS(1957),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1957),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1957),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1957),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1957),
    [sym_duet_dev_type_camera] = ACTIONS(1957),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1957),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1957),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1957),
    [sym_duet_dev_type_dss] = ACTIONS(1957),
    [sym_duet_dev_type_dvr] = ACTIONS(1957),
    [sym_duet_dev_type_disc_device] = ACTIONS(1957),
    [sym_duet_dev_type_document_camera] = ACTIONS(1957),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1957),
    [sym_duet_dev_type_hvac] = ACTIONS(1957),
    [sym_duet_dev_type_keypad] = ACTIONS(1957),
    [sym_duet_dev_type_light] = ACTIONS(1957),
    [sym_duet_dev_type_monitor] = ACTIONS(1957),
    [sym_duet_dev_type_motor] = ACTIONS(1957),
    [sym_duet_dev_type_multi_window] = ACTIONS(1957),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1957),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1957),
    [sym_duet_dev_type_receiver] = ACTIONS(1957),
    [sym_duet_dev_type_security_system] = ACTIONS(1957),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1957),
    [sym_duet_dev_type_settop_box] = ACTIONS(1957),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1957),
    [sym_duet_dev_type_switcher] = ACTIONS(1957),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1957),
    [sym_duet_dev_type_tv] = ACTIONS(1957),
    [sym_duet_dev_type_utility] = ACTIONS(1957),
    [sym_duet_dev_type_vcr] = ACTIONS(1957),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1957),
    [sym_duet_dev_type_video_processor] = ACTIONS(1957),
    [sym_duet_dev_type_video_projector] = ACTIONS(1957),
    [sym_duet_dev_type_video_wall] = ACTIONS(1957),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1957),
    [sym_duet_dev_type_weather] = ACTIONS(1957),
    [sym_duet_dev_not_polled] = ACTIONS(1957),
    [sym_duet_dev_polled] = ACTIONS(1957),
    [sym_clkmgr_mode_network] = ACTIONS(1957),
    [sym_clkmgr_mode_standalone] = ACTIONS(1957),
    [sym_normal_standby] = ACTIONS(1957),
    [sym_normal_wake] = ACTIONS(1957),
    [sym_ip_multicast_ttl_option] = ACTIONS(1957),
    [sym_ip_ttl_subnet] = ACTIONS(1957),
    [sym_ip_ttl_site] = ACTIONS(1957),
    [sym_ip_ttl_region] = ACTIONS(1957),
    [sym_ip_ttl_continent] = ACTIONS(1957),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1957),
    [sym_ip_nodelay_on] = ACTIONS(1957),
    [sym_ip_nodelay_off] = ACTIONS(1957),
    [sym_amx_error] = ACTIONS(1957),
    [sym_amx_warning] = ACTIONS(1957),
    [sym_amx_info] = ACTIONS(1957),
    [sym_amx_debug] = ACTIONS(1957),
    [sym_netlinx_login_success] = ACTIONS(1957),
    [sym_netlinx_login_fail] = ACTIONS(1957),
    [sym_netlinx_logout] = ACTIONS(1957),
    [sym_null_str] = ACTIONS(1957),
    [sym_smtp_address] = ACTIONS(1957),
    [sym_smtp_port_number] = ACTIONS(1957),
    [sym_smtp_username] = ACTIONS(1957),
    [sym_smtp_password] = ACTIONS(1957),
    [sym_smtp_require_tls] = ACTIONS(1957),
    [sym_smtp_from] = ACTIONS(1957),
    [sym_smtp_tls_true] = ACTIONS(1957),
    [sym_smtp_tls_false] = ACTIONS(1957),
    [sym_listview_on_row_select_event] = ACTIONS(1957),
    [sym_data_structure_unknown] = ACTIONS(1957),
    [sym_data_structure_datafeed] = ACTIONS(1957),
    [sym_data_structure_datarecordset] = ACTIONS(1957),
    [sym_data_structure_datarecord] = ACTIONS(1957),
    [sym_data_structure_datafield] = ACTIONS(1957),
    [sym_data_max_content_field_count] = ACTIONS(1957),
    [sym_data_max_metadata_field_count] = ACTIONS(1957),
    [sym_data_max_id_length] = ACTIONS(1957),
    [sym_data_max_name_length] = ACTIONS(1957),
    [sym_data_max_description_length] = ACTIONS(1957),
    [sym_data_max_type_length] = ACTIONS(1957),
    [sym_data_max_format_length] = ACTIONS(1957),
    [sym_data_max_source_length] = ACTIONS(1957),
    [sym_data_max_value_length] = ACTIONS(1957),
    [sym_data_max_label_length] = ACTIONS(1957),
    [sym_data_max_url_length] = ACTIONS(1957),
    [sym_max_last_login_info_length] = ACTIONS(1957),
    [sym_data_type_unknown] = ACTIONS(1957),
    [sym_data_type_string] = ACTIONS(1957),
    [sym_data_type_datetime] = ACTIONS(1957),
    [sym_data_type_date] = ACTIONS(1957),
    [sym_data_type_time] = ACTIONS(1957),
    [sym_data_type_image] = ACTIONS(1957),
    [sym_data_format_url] = ACTIONS(1957),
    [sym_data_format_phone] = ACTIONS(1957),
    [sym_data_format_email] = ACTIONS(1957),
    [sym_data_format_iso8601] = ACTIONS(1957),
    [sym_auditlog_priv] = ACTIONS(1957),
    [sym_configuration_priv] = ACTIONS(1957),
    [sym_deviceconfig_priv] = ACTIONS(1957),
    [sym_ftp_priv] = ACTIONS(1957),
    [sym_http_priv] = ACTIONS(1957),
    [sym_networkconfig_priv] = ACTIONS(1957),
    [sym_programport_priv] = ACTIONS(1957),
    [sym_remoteui_priv] = ACTIONS(1957),
    [sym_securitycontrol_priv] = ACTIONS(1957),
    [sym_softwaremanagement_priv] = ACTIONS(1957),
    [sym_terminal_priv] = ACTIONS(1957),
    [sym_tpadmin_priv] = ACTIONS(1957),
    [sym_user1_priv] = ACTIONS(1957),
    [sym_user2_priv] = ACTIONS(1957),
    [sym_user3_priv] = ACTIONS(1957),
    [sym_user4_priv] = ACTIONS(1957),
    [sym_usermanagement_priv] = ACTIONS(1957),
    [sym_valid_account] = ACTIONS(1957),
    [sym_err_name_invalid] = ACTIONS(1957),
    [sym_err_password_invalid] = ACTIONS(1957),
    [sym_err_invalid_account] = ACTIONS(1957),
    [sym_err_no_authorization] = ACTIONS(1957),
    [sym_err_invalid_parameter] = ACTIONS(1957),
    [sym_err_user_locked_out] = ACTIONS(1957),
    [sym_err_user_account_expired] = ACTIONS(1957),
    [sym_err_general_error] = ACTIONS(1957),
    [sym_err_authentication_error] = ACTIONS(1957),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1957),
    [sym_tls_validate_certificate] = ACTIONS(1957),
    [sym_tdata] = ACTIONS(1957),
    [sym_tchannel] = ACTIONS(1957),
    [sym_tlevel] = ACTIONS(1957),
    [sym_tbutton] = ACTIONS(1957),
    [sym_ttimeline] = ACTIONS(1957),
    [sym_tcustom] = ACTIONS(1957),
    [sym_url_struct] = ACTIONS(1957),
    [sym_dns_struct] = ACTIONS(1957),
    [sym_ip_address_struct] = ACTIONS(1957),
    [sym_dev_info_struct] = ACTIONS(1957),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1957),
    [sym_data_feed] = ACTIONS(1957),
    [sym_data_field] = ACTIONS(1957),
    [sym_data_record] = ACTIONS(1957),
    [sym_wc_data_feed] = ACTIONS(1957),
    [sym_wc_data_field] = ACTIONS(1957),
    [sym_wc_data_record] = ACTIONS(1957),
    [sym_last_login_info] = ACTIONS(1957),
    [sym____reserved___] = ACTIONS(1957),
    [sym_date] = ACTIONS(1957),
    [sym_day] = ACTIONS(1957),
    [sym_time] = ACTIONS(1957),
    [sym_ldate] = ACTIONS(1957),
    [sym_push_device] = ACTIONS(1957),
    [sym_push_channel] = ACTIONS(1957),
    [sym_push_devchan] = ACTIONS(1957),
    [sym_release_device] = ACTIONS(1957),
    [sym_release_channel] = ACTIONS(1957),
    [sym_release_devchan] = ACTIONS(1957),
    [sym_master_slot] = ACTIONS(1957),
    [sym_get_pulse_time] = ACTIONS(1957),
    [sym_get_timer] = ACTIONS(1957),
    [sym_master_sn] = ACTIONS(1957),
    [sym_system_number] = ACTIONS(1957),
    [sym_button] = ACTIONS(1957),
    [sym_dv_channel] = ACTIONS(1957),
    [sym_data] = ACTIONS(1957),
    [sym_level] = ACTIONS(1957),
    [sym_timeline] = ACTIONS(1957),
    [sym_channel] = ACTIONS(1957),
    [sym_custom] = ACTIONS(1957),
    [sym_atoi] = ACTIONS(1957),
    [sym_atol] = ACTIONS(1957),
    [sym_atof] = ACTIONS(1957),
    [sym_itoa] = ACTIONS(1957),
    [sym_itohex] = ACTIONS(1957),
    [sym_hextoi] = ACTIONS(1957),
    [sym_ftoa] = ACTIONS(1957),
    [sym_find_string] = ACTIONS(1957),
    [sym_left_string] = ACTIONS(1957),
    [sym_right_string] = ACTIONS(1957),
    [sym_mid_string] = ACTIONS(1957),
    [sym_remove_string] = ACTIONS(1957),
    [sym_get_buffer_string] = ACTIONS(1957),
    [sym_get_buffer_char] = ACTIONS(1957),
    [sym_get_multi_buffer_string] = ACTIONS(1957),
    [sym_length_string] = ACTIONS(1957),
    [sym_max_length_string] = ACTIONS(1957),
    [sym_set_length_string] = ACTIONS(1957),
    [sym_compare_string] = ACTIONS(1957),
    [sym_upper_string] = ACTIONS(1957),
    [sym_lower_string] = ACTIONS(1957),
    [sym_get_last] = ACTIONS(1957),
    [sym_date_to_day] = ACTIONS(1957),
    [sym_date_to_month] = ACTIONS(1957),
    [sym_date_to_year] = ACTIONS(1957),
    [sym_time_to_hour] = ACTIONS(1957),
    [sym_time_to_minute] = ACTIONS(1957),
    [sym_time_to_second] = ACTIONS(1957),
    [sym_day_of_week] = ACTIONS(1957),
    [sym_set_pulse_time] = ACTIONS(1957),
    [sym_set_timer] = ACTIONS(1957),
    [sym_file_open] = ACTIONS(1957),
    [sym_file_close] = ACTIONS(1957),
    [sym_file_read] = ACTIONS(1957),
    [sym_file_read_line] = ACTIONS(1957),
    [sym_file_write] = ACTIONS(1957),
    [sym_file_write_line] = ACTIONS(1957),
    [sym_file_copy] = ACTIONS(1957),
    [sym_file_delete] = ACTIONS(1957),
    [sym_file_rename] = ACTIONS(1957),
    [sym_file_dir] = ACTIONS(1957),
    [sym_file_getdir] = ACTIONS(1957),
    [sym_file_setdir] = ACTIONS(1957),
    [sym_file_create_dir] = ACTIONS(1957),
    [sym_file_remove_dir] = ACTIONS(1957),
    [sym_file_seek] = ACTIONS(1957),
    [sym_length_array] = ACTIONS(1957),
    [sym_max_length_array] = ACTIONS(1957),
    [sym_set_length_array] = ACTIONS(1957),
    [sym_ip_client_open] = ACTIONS(1957),
    [sym_ip_client_close] = ACTIONS(1957),
    [sym_ip_server_open] = ACTIONS(1957),
    [sym_ip_server_close] = ACTIONS(1957),
    [sym_ip_bound_client_open] = ACTIONS(1957),
    [sym_ip_mc_server_open] = ACTIONS(1957),
    [sym_ip_set_option] = ACTIONS(1957),
    [sym_ssh_client_open] = ACTIONS(1957),
    [sym_ssh_client_close] = ACTIONS(1957),
    [sym_tls_client_open] = ACTIONS(1957),
    [sym_tls_client_close] = ACTIONS(1957),
    [sym_do_push] = ACTIONS(1957),
    [sym_do_push_timed] = ACTIONS(1957),
    [sym_do_release] = ACTIONS(1957),
    [sym_device_id] = ACTIONS(1957),
    [sym_device_info] = ACTIONS(1957),
    [sym_device_id_string] = ACTIONS(1957),
    [sym_device_standby] = ACTIONS(1957),
    [sym_device_wake] = ACTIONS(1957),
    [sym_timeline_create] = ACTIONS(1957),
    [sym_timeline_kill] = ACTIONS(1957),
    [sym_timeline_reload] = ACTIONS(1957),
    [sym_timeline_pause] = ACTIONS(1957),
    [sym_timeline_restart] = ACTIONS(1957),
    [sym_timeline_set] = ACTIONS(1957),
    [sym_timeline_get] = ACTIONS(1957),
    [sym_timeline_active] = ACTIONS(1957),
    [sym_timeline_dynamic_id] = ACTIONS(1957),
    [sym_variable_to_string] = ACTIONS(1957),
    [sym_string_to_variable] = ACTIONS(1957),
    [sym_length_variable_to_string] = ACTIONS(1957),
    [sym_type_cast] = ACTIONS(1957),
    [sym_variable_to_xml] = ACTIONS(1957),
    [sym_xml_to_variable] = ACTIONS(1957),
    [sym_length_variable_to_xml] = ACTIONS(1957),
    [sym_redirect_string] = ACTIONS(1957),
    [sym_random_number] = ACTIONS(1957),
    [sym_get_unique_id] = ACTIONS(1957),
    [sym_get_system_number] = ACTIONS(1957),
    [sym_set_system_number] = ACTIONS(1957),
    [sym_get_serial_number] = ACTIONS(1957),
    [sym_get_master_build] = ACTIONS(1957),
    [sym_set_validation_code] = ACTIONS(1957),
    [sym_reboot] = ACTIONS(1957),
    [sym_load_duet_module] = ACTIONS(1957),
    [sym_unload_duet_module] = ACTIONS(1957),
    [sym_duet_mem_size_set] = ACTIONS(1957),
    [sym_duet_mem_size_get] = ACTIONS(1957),
    [sym_dynamic_application_device] = ACTIONS(1957),
    [sym_dynamic_polled_port] = ACTIONS(1957),
    [sym_static_port_binding] = ACTIONS(1957),
    [sym_static_ip_binding] = ACTIONS(1957),
    [sym_get_dns_list] = ACTIONS(1957),
    [sym_set_dns_list] = ACTIONS(1957),
    [sym_get_ip_address] = ACTIONS(1957),
    [sym_set_ip_address] = ACTIONS(1957),
    [sym_get_url_list] = ACTIONS(1957),
    [sym_add_url_entry] = ACTIONS(1957),
    [sym_delete_url_entry] = ACTIONS(1957),
    [sym_abs_value] = ACTIONS(1957),
    [sym_max_value] = ACTIONS(1957),
    [sym_min_value] = ACTIONS(1957),
    [sym_exp_value] = ACTIONS(1957),
    [sym_log_value] = ACTIONS(1957),
    [sym_log10_value] = ACTIONS(1957),
    [sym_power_value] = ACTIONS(1957),
    [sym_sqrt_value] = ACTIONS(1957),
    [sym_raw_be] = ACTIONS(1957),
    [sym_raw_le] = ACTIONS(1957),
    [sym_internal_queue_size_set] = ACTIONS(1957),
    [sym_internal_queue_size_get] = ACTIONS(1957),
    [sym_internal_threshold_set] = ACTIONS(1957),
    [sym_internal_threshold_get] = ACTIONS(1957),
    [sym_rebuilt_event] = ACTIONS(1957),
    [sym_do_custom_event] = ACTIONS(1957),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1957),
    [sym_clkmgr_set_clk_source] = ACTIONS(1957),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1957),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1957),
    [sym_clkmgr_get_timezone] = ACTIONS(1957),
    [sym_clkmgr_set_timezone] = ACTIONS(1957),
    [sym_clkmgr_get_resync_period] = ACTIONS(1957),
    [sym_clkmgr_set_resync_period] = ACTIONS(1957),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1957),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1957),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1957),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1957),
    [sym_clkmgr_get_timeservers] = ACTIONS(1957),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1957),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1957),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1957),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1957),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1957),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1957),
    [sym_set_log_level] = ACTIONS(1957),
    [sym_get_log_level] = ACTIONS(1957),
    [sym_amx_log] = ACTIONS(1957),
    [sym_smtp_server_config_set] = ACTIONS(1957),
    [sym_smtp_server_config_get] = ACTIONS(1957),
    [sym_smtp_send] = ACTIONS(1957),
    [sym_validate_netlinx_account] = ACTIONS(1957),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1957),
    [sym_authenticate_certificate] = ACTIONS(1957),
    [sym_audit_netlinx_session_event] = ACTIONS(1957),
    [sym_audit_netlinx_generic_event] = ACTIONS(1957),
    [sym_get_available_flash_disk_space] = ACTIONS(1957),
    [sym_get_max_flash_disk_space] = ACTIONS(1957),
    [sym_data_create_feed] = ACTIONS(1957),
    [sym_data_delete_feed] = ACTIONS(1957),
    [sym_data_publish_feed] = ACTIONS(1957),
    [sym_data_get_published_feed] = ACTIONS(1957),
    [sym_data_add_record] = ACTIONS(1957),
    [sym_data_get_event_record] = ACTIONS(1957),
    [sym__wc_data_create_feed] = ACTIONS(1957),
    [sym__wc_data_add_record] = ACTIONS(1957),
    [sym__wc_data_get_event_record] = ACTIONS(1957),
    [sym_format] = ACTIONS(1957),
    [sym_astro_clock] = ACTIONS(1957),
    [sym_set_outdoor_temperature] = ACTIONS(1957),
    [sym_set_virtual_level_count] = ACTIONS(1957),
    [sym_set_virtual_channel_count] = ACTIONS(1957),
    [sym_set_virtual_port_count] = ACTIONS(1957),
    [sym__wc_encode_int] = ACTIONS(1957),
    [sym__wc_decode_int] = ACTIONS(1957),
    [sym__wc_int] = ACTIONS(1957),
    [sym__wc_to_ch_int] = ACTIONS(1957),
    [sym__ch_to_wc_int] = ACTIONS(1957),
    [sym__wc_find_string_int] = ACTIONS(1957),
    [sym__wc_left_string_int] = ACTIONS(1957),
    [sym__wc_lower_string_int] = ACTIONS(1957),
    [sym__wc_mid_string_int] = ACTIONS(1957),
    [sym__wc_remove_string_int] = ACTIONS(1957),
    [sym__wc_right_string_int] = ACTIONS(1957),
    [sym__wc_upper_string_int] = ACTIONS(1957),
    [sym__wc_compare_string_int] = ACTIONS(1957),
    [sym__wc_get_buffer_char_int] = ACTIONS(1957),
    [sym__wc_get_buffer_string_int] = ACTIONS(1957),
    [sym__wc_concat_string_int] = ACTIONS(1957),
    [sym__wc_file_open_int] = ACTIONS(1957),
    [sym__wc_file_close_int] = ACTIONS(1957),
    [sym__wc_file_read_int] = ACTIONS(1957),
    [sym__wc_file_read_line_int] = ACTIONS(1957),
    [sym__wc_file_write_int] = ACTIONS(1957),
    [sym__wc_file_write_line_int] = ACTIONS(1957),
    [aux_sym_preproc_include_token1] = ACTIONS(1957),
    [aux_sym_preproc_define_token1] = ACTIONS(1959),
    [aux_sym_preproc_warn_token1] = ACTIONS(1959),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1959),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1959),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1959),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1959),
    [aux_sym_preproc_else_token1] = ACTIONS(1959),
    [anon_sym_LBRACK] = ACTIONS(1959),
    [anon_sym_LBRACE] = ACTIONS(1959),
    [anon_sym_LPAREN] = ACTIONS(1957),
    [anon_sym_BANG] = ACTIONS(1959),
    [anon_sym_TILDE] = ACTIONS(1959),
    [anon_sym_DASH] = ACTIONS(1957),
    [anon_sym_PLUS] = ACTIONS(1957),
    [anon_sym_DASH_DASH] = ACTIONS(1959),
    [anon_sym_PLUS_PLUS] = ACTIONS(1959),
    [anon_sym_DQUOTE] = ACTIONS(1959),
    [anon_sym_SQUOTE] = ACTIONS(1959),
    [sym_number_literal] = ACTIONS(1959),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(293)] = {
    [ts_builtin_sym_end] = ACTIONS(1849),
    [sym_identifier] = ACTIONS(1847),
    [sym_program_name_keyword] = ACTIONS(1847),
    [sym_module_name_keyword] = ACTIONS(1847),
    [sym_define_device_keyword] = ACTIONS(1847),
    [sym_define_combine_keyword] = ACTIONS(1847),
    [sym_define_constant_keyword] = ACTIONS(1847),
    [sym_define_type_keyword] = ACTIONS(1847),
    [sym_define_variable_keyword] = ACTIONS(1847),
    [sym_define_system_variable_keyword] = ACTIONS(1847),
    [sym_define_connect_level_keyword] = ACTIONS(1847),
    [sym_define_latching_keyword] = ACTIONS(1847),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1847),
    [sym_define_toggling_keyword] = ACTIONS(1847),
    [sym_define_call_keyword] = ACTIONS(1847),
    [sym_define_function_keyword] = ACTIONS(1847),
    [sym_define_library_function_keyword] = ACTIONS(1847),
    [sym_define_start_keyword] = ACTIONS(1847),
    [sym_define_event_keyword] = ACTIONS(1847),
    [sym_define_program_keyword] = ACTIONS(1847),
    [sym_define_module_keyword] = ACTIONS(1847),
    [sym_char_keyword] = ACTIONS(1847),
    [sym_widechar_keyword] = ACTIONS(1847),
    [sym_integer_keyword] = ACTIONS(1847),
    [sym_sinteger_keyword] = ACTIONS(1847),
    [sym_long_keyword] = ACTIONS(1847),
    [sym_slong_keyword] = ACTIONS(1847),
    [sym_float_keyword] = ACTIONS(1847),
    [sym_double_keyword] = ACTIONS(1847),
    [sym_variant_keyword] = ACTIONS(1847),
    [sym_variantarray_keyword] = ACTIONS(1847),
    [sym_local_var_keyword] = ACTIONS(1847),
    [sym_stack_var_keyword] = ACTIONS(1847),
    [sym_constant_keyword] = ACTIONS(1847),
    [sym_volatile_keyword] = ACTIONS(1847),
    [sym_non_volatile_keyword] = ACTIONS(1847),
    [sym_persistent_keyword] = ACTIONS(1847),
    [sym_dev_keyword] = ACTIONS(1847),
    [sym_devlev_keyword] = ACTIONS(1847),
    [sym_devchan_keyword] = ACTIONS(1847),
    [sym_if_keyword] = ACTIONS(1847),
    [sym_else_keyword] = ACTIONS(1847),
    [sym_switch_keyword] = ACTIONS(1847),
    [sym_case_keyword] = ACTIONS(1847),
    [sym_default_keyword] = ACTIONS(1847),
    [sym_while_keyword] = ACTIONS(1847),
    [sym_for_keyword] = ACTIONS(1847),
    [sym_break_keyword] = ACTIONS(1847),
    [sym_continue_keyword] = ACTIONS(1847),
    [sym_return_keyword] = ACTIONS(1847),
    [sym_select_keyword] = ACTIONS(1847),
    [sym_active_keyword] = ACTIONS(1847),
    [sym_struct_keyword] = ACTIONS(1847),
    [sym_structure_keyword] = ACTIONS(1847),
    [sym_button_event_keyword] = ACTIONS(1847),
    [sym_channel_event_keyword] = ACTIONS(1847),
    [sym_level_event_keyword] = ACTIONS(1847),
    [sym_data_event_keyword] = ACTIONS(1847),
    [sym_timeline_event_keyword] = ACTIONS(1847),
    [sym_custom_event_keyword] = ACTIONS(1847),
    [sym_push_keyword] = ACTIONS(1847),
    [sym_release_keyword] = ACTIONS(1847),
    [aux_sym_on_keyword_token1] = ACTIONS(1847),
    [aux_sym_off_keyword_token1] = ACTIONS(1847),
    [sym_send_level_keyword] = ACTIONS(1847),
    [sym_send_string_keyword] = ACTIONS(1847),
    [sym_send_command_keyword] = ACTIONS(1847),
    [sym_clear_buffer_keyword] = ACTIONS(1847),
    [sym_create_buffer_keyword] = ACTIONS(1847),
    [sym_create_multi_buffer_keyword] = ACTIONS(1847),
    [sym_call_keyword] = ACTIONS(1847),
    [sym_system_call_keyword] = ACTIONS(1847),
    [sym_devchan_to_keyword] = ACTIONS(1847),
    [sym_devchan_min_to_keyword] = ACTIONS(1847),
    [sym_devchan_total_off_keyword] = ACTIONS(1847),
    [sym_devchan_pulse_keyword] = ACTIONS(1847),
    [sym_wait_keyword] = ACTIONS(1847),
    [sym_cancel_wait_keyword] = ACTIONS(1847),
    [sym_wait_until_keyword] = ACTIONS(1847),
    [sym_cancel_wait_until_keyword] = ACTIONS(1847),
    [sym_cancel_all_wait_keyword] = ACTIONS(1847),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1847),
    [sym_not] = ACTIONS(1847),
    [sym_bnot] = ACTIONS(1847),
    [sym___file__] = ACTIONS(1847),
    [sym___date__] = ACTIONS(1847),
    [sym___ldate__] = ACTIONS(1847),
    [sym___time__] = ACTIONS(1847),
    [sym___line__] = ACTIONS(1847),
    [sym___name__] = ACTIONS(1847),
    [sym___netlinx__] = ACTIONS(1847),
    [sym_netlinx_axi_version] = ACTIONS(1847),
    [sym_url_flg_tcp] = ACTIONS(1847),
    [sym_url_flg_accinfopresent] = ACTIONS(1847),
    [sym_url_flg_temp] = ACTIONS(1847),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1847),
    [sym_url_flg_stat_mask] = ACTIONS(1847),
    [sym_url_flg_stat_lookup] = ACTIONS(1847),
    [sym_url_flg_stat_connecting] = ACTIONS(1847),
    [sym_url_flg_stat_waiting] = ACTIONS(1847),
    [sym_url_flg_stat_connected] = ACTIONS(1847),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1847),
    [sym_first_virtual_device] = ACTIONS(1847),
    [sym_first_local_port] = ACTIONS(1847),
    [sym_dynamic_virtual_device] = ACTIONS(1847),
    [sym_timeline_once] = ACTIONS(1847),
    [sym_timeline_repeat] = ACTIONS(1847),
    [sym_timeline_absolute] = ACTIONS(1847),
    [sym_timeline_relative] = ACTIONS(1847),
    [sym_do_push_timed_infinite] = ACTIONS(1847),
    [sym_source_type_no_address] = ACTIONS(1847),
    [sym_source_type_neuron_id] = ACTIONS(1847),
    [sym_source_type_ip_address] = ACTIONS(1847),
    [sym_source_type_axlink] = ACTIONS(1847),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1847),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1847),
    [sym_source_type_ip_socket_address] = ACTIONS(1847),
    [sym_source_type_rs232] = ACTIONS(1847),
    [sym_source_type_internal] = ACTIONS(1847),
    [sym_source_type_mac_address] = ACTIONS(1847),
    [sym_source_type_ipv4_port] = ACTIONS(1847),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1847),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1847),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1847),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1847),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1847),
    [sym_true] = ACTIONS(1847),
    [sym_false] = ACTIONS(1847),
    [sym_file_read_only] = ACTIONS(1847),
    [sym_file_rw_new] = ACTIONS(1847),
    [sym_file_rw_append] = ACTIONS(1847),
    [sym_ip_tcp] = ACTIONS(1847),
    [sym_ip_udp] = ACTIONS(1847),
    [sym_ip_udp_2way] = ACTIONS(1847),
    [sym_xml_encode_types] = ACTIONS(1847),
    [sym_xml_encode_char_as_list] = ACTIONS(1847),
    [sym_xml_encode_le] = ACTIONS(1847),
    [sym_xml_decode_types] = ACTIONS(1847),
    [sym_xml_decode_no_preserve] = ACTIONS(1847),
    [sym_duet_dev_type_display_device] = ACTIONS(1847),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1847),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1847),
    [sym_duet_dev_type_io_device] = ACTIONS(1847),
    [sym_duet_dev_type_relay_device] = ACTIONS(1847),
    [sym_duet_dev_type_ups] = ACTIONS(1847),
    [sym_duet_dev_type_amplifier] = ACTIONS(1847),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1847),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1847),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1847),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1847),
    [sym_duet_dev_type_camera] = ACTIONS(1847),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1847),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1847),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1847),
    [sym_duet_dev_type_dss] = ACTIONS(1847),
    [sym_duet_dev_type_dvr] = ACTIONS(1847),
    [sym_duet_dev_type_disc_device] = ACTIONS(1847),
    [sym_duet_dev_type_document_camera] = ACTIONS(1847),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1847),
    [sym_duet_dev_type_hvac] = ACTIONS(1847),
    [sym_duet_dev_type_keypad] = ACTIONS(1847),
    [sym_duet_dev_type_light] = ACTIONS(1847),
    [sym_duet_dev_type_monitor] = ACTIONS(1847),
    [sym_duet_dev_type_motor] = ACTIONS(1847),
    [sym_duet_dev_type_multi_window] = ACTIONS(1847),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1847),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1847),
    [sym_duet_dev_type_receiver] = ACTIONS(1847),
    [sym_duet_dev_type_security_system] = ACTIONS(1847),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1847),
    [sym_duet_dev_type_settop_box] = ACTIONS(1847),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1847),
    [sym_duet_dev_type_switcher] = ACTIONS(1847),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1847),
    [sym_duet_dev_type_tv] = ACTIONS(1847),
    [sym_duet_dev_type_utility] = ACTIONS(1847),
    [sym_duet_dev_type_vcr] = ACTIONS(1847),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1847),
    [sym_duet_dev_type_video_processor] = ACTIONS(1847),
    [sym_duet_dev_type_video_projector] = ACTIONS(1847),
    [sym_duet_dev_type_video_wall] = ACTIONS(1847),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1847),
    [sym_duet_dev_type_weather] = ACTIONS(1847),
    [sym_duet_dev_not_polled] = ACTIONS(1847),
    [sym_duet_dev_polled] = ACTIONS(1847),
    [sym_clkmgr_mode_network] = ACTIONS(1847),
    [sym_clkmgr_mode_standalone] = ACTIONS(1847),
    [sym_normal_standby] = ACTIONS(1847),
    [sym_normal_wake] = ACTIONS(1847),
    [sym_ip_multicast_ttl_option] = ACTIONS(1847),
    [sym_ip_ttl_subnet] = ACTIONS(1847),
    [sym_ip_ttl_site] = ACTIONS(1847),
    [sym_ip_ttl_region] = ACTIONS(1847),
    [sym_ip_ttl_continent] = ACTIONS(1847),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1847),
    [sym_ip_nodelay_on] = ACTIONS(1847),
    [sym_ip_nodelay_off] = ACTIONS(1847),
    [sym_amx_error] = ACTIONS(1847),
    [sym_amx_warning] = ACTIONS(1847),
    [sym_amx_info] = ACTIONS(1847),
    [sym_amx_debug] = ACTIONS(1847),
    [sym_netlinx_login_success] = ACTIONS(1847),
    [sym_netlinx_login_fail] = ACTIONS(1847),
    [sym_netlinx_logout] = ACTIONS(1847),
    [sym_null_str] = ACTIONS(1847),
    [sym_smtp_address] = ACTIONS(1847),
    [sym_smtp_port_number] = ACTIONS(1847),
    [sym_smtp_username] = ACTIONS(1847),
    [sym_smtp_password] = ACTIONS(1847),
    [sym_smtp_require_tls] = ACTIONS(1847),
    [sym_smtp_from] = ACTIONS(1847),
    [sym_smtp_tls_true] = ACTIONS(1847),
    [sym_smtp_tls_false] = ACTIONS(1847),
    [sym_listview_on_row_select_event] = ACTIONS(1847),
    [sym_data_structure_unknown] = ACTIONS(1847),
    [sym_data_structure_datafeed] = ACTIONS(1847),
    [sym_data_structure_datarecordset] = ACTIONS(1847),
    [sym_data_structure_datarecord] = ACTIONS(1847),
    [sym_data_structure_datafield] = ACTIONS(1847),
    [sym_data_max_content_field_count] = ACTIONS(1847),
    [sym_data_max_metadata_field_count] = ACTIONS(1847),
    [sym_data_max_id_length] = ACTIONS(1847),
    [sym_data_max_name_length] = ACTIONS(1847),
    [sym_data_max_description_length] = ACTIONS(1847),
    [sym_data_max_type_length] = ACTIONS(1847),
    [sym_data_max_format_length] = ACTIONS(1847),
    [sym_data_max_source_length] = ACTIONS(1847),
    [sym_data_max_value_length] = ACTIONS(1847),
    [sym_data_max_label_length] = ACTIONS(1847),
    [sym_data_max_url_length] = ACTIONS(1847),
    [sym_max_last_login_info_length] = ACTIONS(1847),
    [sym_data_type_unknown] = ACTIONS(1847),
    [sym_data_type_string] = ACTIONS(1847),
    [sym_data_type_datetime] = ACTIONS(1847),
    [sym_data_type_date] = ACTIONS(1847),
    [sym_data_type_time] = ACTIONS(1847),
    [sym_data_type_image] = ACTIONS(1847),
    [sym_data_format_url] = ACTIONS(1847),
    [sym_data_format_phone] = ACTIONS(1847),
    [sym_data_format_email] = ACTIONS(1847),
    [sym_data_format_iso8601] = ACTIONS(1847),
    [sym_auditlog_priv] = ACTIONS(1847),
    [sym_configuration_priv] = ACTIONS(1847),
    [sym_deviceconfig_priv] = ACTIONS(1847),
    [sym_ftp_priv] = ACTIONS(1847),
    [sym_http_priv] = ACTIONS(1847),
    [sym_networkconfig_priv] = ACTIONS(1847),
    [sym_programport_priv] = ACTIONS(1847),
    [sym_remoteui_priv] = ACTIONS(1847),
    [sym_securitycontrol_priv] = ACTIONS(1847),
    [sym_softwaremanagement_priv] = ACTIONS(1847),
    [sym_terminal_priv] = ACTIONS(1847),
    [sym_tpadmin_priv] = ACTIONS(1847),
    [sym_user1_priv] = ACTIONS(1847),
    [sym_user2_priv] = ACTIONS(1847),
    [sym_user3_priv] = ACTIONS(1847),
    [sym_user4_priv] = ACTIONS(1847),
    [sym_usermanagement_priv] = ACTIONS(1847),
    [sym_valid_account] = ACTIONS(1847),
    [sym_err_name_invalid] = ACTIONS(1847),
    [sym_err_password_invalid] = ACTIONS(1847),
    [sym_err_invalid_account] = ACTIONS(1847),
    [sym_err_no_authorization] = ACTIONS(1847),
    [sym_err_invalid_parameter] = ACTIONS(1847),
    [sym_err_user_locked_out] = ACTIONS(1847),
    [sym_err_user_account_expired] = ACTIONS(1847),
    [sym_err_general_error] = ACTIONS(1847),
    [sym_err_authentication_error] = ACTIONS(1847),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1847),
    [sym_tls_validate_certificate] = ACTIONS(1847),
    [sym_tdata] = ACTIONS(1847),
    [sym_tchannel] = ACTIONS(1847),
    [sym_tlevel] = ACTIONS(1847),
    [sym_tbutton] = ACTIONS(1847),
    [sym_ttimeline] = ACTIONS(1847),
    [sym_tcustom] = ACTIONS(1847),
    [sym_url_struct] = ACTIONS(1847),
    [sym_dns_struct] = ACTIONS(1847),
    [sym_ip_address_struct] = ACTIONS(1847),
    [sym_dev_info_struct] = ACTIONS(1847),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1847),
    [sym_data_feed] = ACTIONS(1847),
    [sym_data_field] = ACTIONS(1847),
    [sym_data_record] = ACTIONS(1847),
    [sym_wc_data_feed] = ACTIONS(1847),
    [sym_wc_data_field] = ACTIONS(1847),
    [sym_wc_data_record] = ACTIONS(1847),
    [sym_last_login_info] = ACTIONS(1847),
    [sym____reserved___] = ACTIONS(1847),
    [sym_date] = ACTIONS(1847),
    [sym_day] = ACTIONS(1847),
    [sym_time] = ACTIONS(1847),
    [sym_ldate] = ACTIONS(1847),
    [sym_push_device] = ACTIONS(1847),
    [sym_push_channel] = ACTIONS(1847),
    [sym_push_devchan] = ACTIONS(1847),
    [sym_release_device] = ACTIONS(1847),
    [sym_release_channel] = ACTIONS(1847),
    [sym_release_devchan] = ACTIONS(1847),
    [sym_master_slot] = ACTIONS(1847),
    [sym_get_pulse_time] = ACTIONS(1847),
    [sym_get_timer] = ACTIONS(1847),
    [sym_master_sn] = ACTIONS(1847),
    [sym_system_number] = ACTIONS(1847),
    [sym_button] = ACTIONS(1847),
    [sym_dv_channel] = ACTIONS(1847),
    [sym_data] = ACTIONS(1847),
    [sym_level] = ACTIONS(1847),
    [sym_timeline] = ACTIONS(1847),
    [sym_channel] = ACTIONS(1847),
    [sym_custom] = ACTIONS(1847),
    [sym_atoi] = ACTIONS(1847),
    [sym_atol] = ACTIONS(1847),
    [sym_atof] = ACTIONS(1847),
    [sym_itoa] = ACTIONS(1847),
    [sym_itohex] = ACTIONS(1847),
    [sym_hextoi] = ACTIONS(1847),
    [sym_ftoa] = ACTIONS(1847),
    [sym_find_string] = ACTIONS(1847),
    [sym_left_string] = ACTIONS(1847),
    [sym_right_string] = ACTIONS(1847),
    [sym_mid_string] = ACTIONS(1847),
    [sym_remove_string] = ACTIONS(1847),
    [sym_get_buffer_string] = ACTIONS(1847),
    [sym_get_buffer_char] = ACTIONS(1847),
    [sym_get_multi_buffer_string] = ACTIONS(1847),
    [sym_length_string] = ACTIONS(1847),
    [sym_max_length_string] = ACTIONS(1847),
    [sym_set_length_string] = ACTIONS(1847),
    [sym_compare_string] = ACTIONS(1847),
    [sym_upper_string] = ACTIONS(1847),
    [sym_lower_string] = ACTIONS(1847),
    [sym_get_last] = ACTIONS(1847),
    [sym_date_to_day] = ACTIONS(1847),
    [sym_date_to_month] = ACTIONS(1847),
    [sym_date_to_year] = ACTIONS(1847),
    [sym_time_to_hour] = ACTIONS(1847),
    [sym_time_to_minute] = ACTIONS(1847),
    [sym_time_to_second] = ACTIONS(1847),
    [sym_day_of_week] = ACTIONS(1847),
    [sym_set_pulse_time] = ACTIONS(1847),
    [sym_set_timer] = ACTIONS(1847),
    [sym_file_open] = ACTIONS(1847),
    [sym_file_close] = ACTIONS(1847),
    [sym_file_read] = ACTIONS(1847),
    [sym_file_read_line] = ACTIONS(1847),
    [sym_file_write] = ACTIONS(1847),
    [sym_file_write_line] = ACTIONS(1847),
    [sym_file_copy] = ACTIONS(1847),
    [sym_file_delete] = ACTIONS(1847),
    [sym_file_rename] = ACTIONS(1847),
    [sym_file_dir] = ACTIONS(1847),
    [sym_file_getdir] = ACTIONS(1847),
    [sym_file_setdir] = ACTIONS(1847),
    [sym_file_create_dir] = ACTIONS(1847),
    [sym_file_remove_dir] = ACTIONS(1847),
    [sym_file_seek] = ACTIONS(1847),
    [sym_length_array] = ACTIONS(1847),
    [sym_max_length_array] = ACTIONS(1847),
    [sym_set_length_array] = ACTIONS(1847),
    [sym_ip_client_open] = ACTIONS(1847),
    [sym_ip_client_close] = ACTIONS(1847),
    [sym_ip_server_open] = ACTIONS(1847),
    [sym_ip_server_close] = ACTIONS(1847),
    [sym_ip_bound_client_open] = ACTIONS(1847),
    [sym_ip_mc_server_open] = ACTIONS(1847),
    [sym_ip_set_option] = ACTIONS(1847),
    [sym_ssh_client_open] = ACTIONS(1847),
    [sym_ssh_client_close] = ACTIONS(1847),
    [sym_tls_client_open] = ACTIONS(1847),
    [sym_tls_client_close] = ACTIONS(1847),
    [sym_do_push] = ACTIONS(1847),
    [sym_do_push_timed] = ACTIONS(1847),
    [sym_do_release] = ACTIONS(1847),
    [sym_device_id] = ACTIONS(1847),
    [sym_device_info] = ACTIONS(1847),
    [sym_device_id_string] = ACTIONS(1847),
    [sym_device_standby] = ACTIONS(1847),
    [sym_device_wake] = ACTIONS(1847),
    [sym_timeline_create] = ACTIONS(1847),
    [sym_timeline_kill] = ACTIONS(1847),
    [sym_timeline_reload] = ACTIONS(1847),
    [sym_timeline_pause] = ACTIONS(1847),
    [sym_timeline_restart] = ACTIONS(1847),
    [sym_timeline_set] = ACTIONS(1847),
    [sym_timeline_get] = ACTIONS(1847),
    [sym_timeline_active] = ACTIONS(1847),
    [sym_timeline_dynamic_id] = ACTIONS(1847),
    [sym_variable_to_string] = ACTIONS(1847),
    [sym_string_to_variable] = ACTIONS(1847),
    [sym_length_variable_to_string] = ACTIONS(1847),
    [sym_type_cast] = ACTIONS(1847),
    [sym_variable_to_xml] = ACTIONS(1847),
    [sym_xml_to_variable] = ACTIONS(1847),
    [sym_length_variable_to_xml] = ACTIONS(1847),
    [sym_redirect_string] = ACTIONS(1847),
    [sym_random_number] = ACTIONS(1847),
    [sym_get_unique_id] = ACTIONS(1847),
    [sym_get_system_number] = ACTIONS(1847),
    [sym_set_system_number] = ACTIONS(1847),
    [sym_get_serial_number] = ACTIONS(1847),
    [sym_get_master_build] = ACTIONS(1847),
    [sym_set_validation_code] = ACTIONS(1847),
    [sym_reboot] = ACTIONS(1847),
    [sym_load_duet_module] = ACTIONS(1847),
    [sym_unload_duet_module] = ACTIONS(1847),
    [sym_duet_mem_size_set] = ACTIONS(1847),
    [sym_duet_mem_size_get] = ACTIONS(1847),
    [sym_dynamic_application_device] = ACTIONS(1847),
    [sym_dynamic_polled_port] = ACTIONS(1847),
    [sym_static_port_binding] = ACTIONS(1847),
    [sym_static_ip_binding] = ACTIONS(1847),
    [sym_get_dns_list] = ACTIONS(1847),
    [sym_set_dns_list] = ACTIONS(1847),
    [sym_get_ip_address] = ACTIONS(1847),
    [sym_set_ip_address] = ACTIONS(1847),
    [sym_get_url_list] = ACTIONS(1847),
    [sym_add_url_entry] = ACTIONS(1847),
    [sym_delete_url_entry] = ACTIONS(1847),
    [sym_abs_value] = ACTIONS(1847),
    [sym_max_value] = ACTIONS(1847),
    [sym_min_value] = ACTIONS(1847),
    [sym_exp_value] = ACTIONS(1847),
    [sym_log_value] = ACTIONS(1847),
    [sym_log10_value] = ACTIONS(1847),
    [sym_power_value] = ACTIONS(1847),
    [sym_sqrt_value] = ACTIONS(1847),
    [sym_raw_be] = ACTIONS(1847),
    [sym_raw_le] = ACTIONS(1847),
    [sym_internal_queue_size_set] = ACTIONS(1847),
    [sym_internal_queue_size_get] = ACTIONS(1847),
    [sym_internal_threshold_set] = ACTIONS(1847),
    [sym_internal_threshold_get] = ACTIONS(1847),
    [sym_rebuilt_event] = ACTIONS(1847),
    [sym_do_custom_event] = ACTIONS(1847),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1847),
    [sym_clkmgr_set_clk_source] = ACTIONS(1847),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1847),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1847),
    [sym_clkmgr_get_timezone] = ACTIONS(1847),
    [sym_clkmgr_set_timezone] = ACTIONS(1847),
    [sym_clkmgr_get_resync_period] = ACTIONS(1847),
    [sym_clkmgr_set_resync_period] = ACTIONS(1847),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1847),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1847),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1847),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1847),
    [sym_clkmgr_get_timeservers] = ACTIONS(1847),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1847),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1847),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1847),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1847),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1847),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1847),
    [sym_set_log_level] = ACTIONS(1847),
    [sym_get_log_level] = ACTIONS(1847),
    [sym_amx_log] = ACTIONS(1847),
    [sym_smtp_server_config_set] = ACTIONS(1847),
    [sym_smtp_server_config_get] = ACTIONS(1847),
    [sym_smtp_send] = ACTIONS(1847),
    [sym_validate_netlinx_account] = ACTIONS(1847),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1847),
    [sym_authenticate_certificate] = ACTIONS(1847),
    [sym_audit_netlinx_session_event] = ACTIONS(1847),
    [sym_audit_netlinx_generic_event] = ACTIONS(1847),
    [sym_get_available_flash_disk_space] = ACTIONS(1847),
    [sym_get_max_flash_disk_space] = ACTIONS(1847),
    [sym_data_create_feed] = ACTIONS(1847),
    [sym_data_delete_feed] = ACTIONS(1847),
    [sym_data_publish_feed] = ACTIONS(1847),
    [sym_data_get_published_feed] = ACTIONS(1847),
    [sym_data_add_record] = ACTIONS(1847),
    [sym_data_get_event_record] = ACTIONS(1847),
    [sym__wc_data_create_feed] = ACTIONS(1847),
    [sym__wc_data_add_record] = ACTIONS(1847),
    [sym__wc_data_get_event_record] = ACTIONS(1847),
    [sym_format] = ACTIONS(1847),
    [sym_astro_clock] = ACTIONS(1847),
    [sym_set_outdoor_temperature] = ACTIONS(1847),
    [sym_set_virtual_level_count] = ACTIONS(1847),
    [sym_set_virtual_channel_count] = ACTIONS(1847),
    [sym_set_virtual_port_count] = ACTIONS(1847),
    [sym__wc_encode_int] = ACTIONS(1847),
    [sym__wc_decode_int] = ACTIONS(1847),
    [sym__wc_int] = ACTIONS(1847),
    [sym__wc_to_ch_int] = ACTIONS(1847),
    [sym__ch_to_wc_int] = ACTIONS(1847),
    [sym__wc_find_string_int] = ACTIONS(1847),
    [sym__wc_left_string_int] = ACTIONS(1847),
    [sym__wc_lower_string_int] = ACTIONS(1847),
    [sym__wc_mid_string_int] = ACTIONS(1847),
    [sym__wc_remove_string_int] = ACTIONS(1847),
    [sym__wc_right_string_int] = ACTIONS(1847),
    [sym__wc_upper_string_int] = ACTIONS(1847),
    [sym__wc_compare_string_int] = ACTIONS(1847),
    [sym__wc_get_buffer_char_int] = ACTIONS(1847),
    [sym__wc_get_buffer_string_int] = ACTIONS(1847),
    [sym__wc_concat_string_int] = ACTIONS(1847),
    [sym__wc_file_open_int] = ACTIONS(1847),
    [sym__wc_file_close_int] = ACTIONS(1847),
    [sym__wc_file_read_int] = ACTIONS(1847),
    [sym__wc_file_read_line_int] = ACTIONS(1847),
    [sym__wc_file_write_int] = ACTIONS(1847),
    [sym__wc_file_write_line_int] = ACTIONS(1847),
    [aux_sym_preproc_include_token1] = ACTIONS(1847),
    [aux_sym_preproc_define_token1] = ACTIONS(1849),
    [aux_sym_preproc_warn_token1] = ACTIONS(1849),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1849),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1849),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1849),
    [anon_sym_LBRACK] = ACTIONS(1849),
    [anon_sym_LBRACE] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(1847),
    [anon_sym_BANG] = ACTIONS(1849),
    [anon_sym_TILDE] = ACTIONS(1849),
    [anon_sym_DASH] = ACTIONS(1847),
    [anon_sym_PLUS] = ACTIONS(1847),
    [anon_sym_DASH_DASH] = ACTIONS(1849),
    [anon_sym_PLUS_PLUS] = ACTIONS(1849),
    [anon_sym_DQUOTE] = ACTIONS(1849),
    [anon_sym_SQUOTE] = ACTIONS(1849),
    [sym_number_literal] = ACTIONS(1849),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(294)] = {
    [ts_builtin_sym_end] = ACTIONS(1853),
    [sym_identifier] = ACTIONS(1851),
    [sym_program_name_keyword] = ACTIONS(1851),
    [sym_module_name_keyword] = ACTIONS(1851),
    [sym_define_device_keyword] = ACTIONS(1851),
    [sym_define_combine_keyword] = ACTIONS(1851),
    [sym_define_constant_keyword] = ACTIONS(1851),
    [sym_define_type_keyword] = ACTIONS(1851),
    [sym_define_variable_keyword] = ACTIONS(1851),
    [sym_define_system_variable_keyword] = ACTIONS(1851),
    [sym_define_connect_level_keyword] = ACTIONS(1851),
    [sym_define_latching_keyword] = ACTIONS(1851),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1851),
    [sym_define_toggling_keyword] = ACTIONS(1851),
    [sym_define_call_keyword] = ACTIONS(1851),
    [sym_define_function_keyword] = ACTIONS(1851),
    [sym_define_library_function_keyword] = ACTIONS(1851),
    [sym_define_start_keyword] = ACTIONS(1851),
    [sym_define_event_keyword] = ACTIONS(1851),
    [sym_define_program_keyword] = ACTIONS(1851),
    [sym_define_module_keyword] = ACTIONS(1851),
    [sym_char_keyword] = ACTIONS(1851),
    [sym_widechar_keyword] = ACTIONS(1851),
    [sym_integer_keyword] = ACTIONS(1851),
    [sym_sinteger_keyword] = ACTIONS(1851),
    [sym_long_keyword] = ACTIONS(1851),
    [sym_slong_keyword] = ACTIONS(1851),
    [sym_float_keyword] = ACTIONS(1851),
    [sym_double_keyword] = ACTIONS(1851),
    [sym_variant_keyword] = ACTIONS(1851),
    [sym_variantarray_keyword] = ACTIONS(1851),
    [sym_local_var_keyword] = ACTIONS(1851),
    [sym_stack_var_keyword] = ACTIONS(1851),
    [sym_constant_keyword] = ACTIONS(1851),
    [sym_volatile_keyword] = ACTIONS(1851),
    [sym_non_volatile_keyword] = ACTIONS(1851),
    [sym_persistent_keyword] = ACTIONS(1851),
    [sym_dev_keyword] = ACTIONS(1851),
    [sym_devlev_keyword] = ACTIONS(1851),
    [sym_devchan_keyword] = ACTIONS(1851),
    [sym_if_keyword] = ACTIONS(1851),
    [sym_else_keyword] = ACTIONS(1851),
    [sym_switch_keyword] = ACTIONS(1851),
    [sym_case_keyword] = ACTIONS(1851),
    [sym_default_keyword] = ACTIONS(1851),
    [sym_while_keyword] = ACTIONS(1851),
    [sym_for_keyword] = ACTIONS(1851),
    [sym_break_keyword] = ACTIONS(1851),
    [sym_continue_keyword] = ACTIONS(1851),
    [sym_return_keyword] = ACTIONS(1851),
    [sym_select_keyword] = ACTIONS(1851),
    [sym_active_keyword] = ACTIONS(1851),
    [sym_struct_keyword] = ACTIONS(1851),
    [sym_structure_keyword] = ACTIONS(1851),
    [sym_button_event_keyword] = ACTIONS(1851),
    [sym_channel_event_keyword] = ACTIONS(1851),
    [sym_level_event_keyword] = ACTIONS(1851),
    [sym_data_event_keyword] = ACTIONS(1851),
    [sym_timeline_event_keyword] = ACTIONS(1851),
    [sym_custom_event_keyword] = ACTIONS(1851),
    [sym_push_keyword] = ACTIONS(1851),
    [sym_release_keyword] = ACTIONS(1851),
    [aux_sym_on_keyword_token1] = ACTIONS(1851),
    [aux_sym_off_keyword_token1] = ACTIONS(1851),
    [sym_send_level_keyword] = ACTIONS(1851),
    [sym_send_string_keyword] = ACTIONS(1851),
    [sym_send_command_keyword] = ACTIONS(1851),
    [sym_clear_buffer_keyword] = ACTIONS(1851),
    [sym_create_buffer_keyword] = ACTIONS(1851),
    [sym_create_multi_buffer_keyword] = ACTIONS(1851),
    [sym_call_keyword] = ACTIONS(1851),
    [sym_system_call_keyword] = ACTIONS(1851),
    [sym_devchan_to_keyword] = ACTIONS(1851),
    [sym_devchan_min_to_keyword] = ACTIONS(1851),
    [sym_devchan_total_off_keyword] = ACTIONS(1851),
    [sym_devchan_pulse_keyword] = ACTIONS(1851),
    [sym_wait_keyword] = ACTIONS(1851),
    [sym_cancel_wait_keyword] = ACTIONS(1851),
    [sym_wait_until_keyword] = ACTIONS(1851),
    [sym_cancel_wait_until_keyword] = ACTIONS(1851),
    [sym_cancel_all_wait_keyword] = ACTIONS(1851),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1851),
    [sym_not] = ACTIONS(1851),
    [sym_bnot] = ACTIONS(1851),
    [sym___file__] = ACTIONS(1851),
    [sym___date__] = ACTIONS(1851),
    [sym___ldate__] = ACTIONS(1851),
    [sym___time__] = ACTIONS(1851),
    [sym___line__] = ACTIONS(1851),
    [sym___name__] = ACTIONS(1851),
    [sym___netlinx__] = ACTIONS(1851),
    [sym_netlinx_axi_version] = ACTIONS(1851),
    [sym_url_flg_tcp] = ACTIONS(1851),
    [sym_url_flg_accinfopresent] = ACTIONS(1851),
    [sym_url_flg_temp] = ACTIONS(1851),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1851),
    [sym_url_flg_stat_mask] = ACTIONS(1851),
    [sym_url_flg_stat_lookup] = ACTIONS(1851),
    [sym_url_flg_stat_connecting] = ACTIONS(1851),
    [sym_url_flg_stat_waiting] = ACTIONS(1851),
    [sym_url_flg_stat_connected] = ACTIONS(1851),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1851),
    [sym_first_virtual_device] = ACTIONS(1851),
    [sym_first_local_port] = ACTIONS(1851),
    [sym_dynamic_virtual_device] = ACTIONS(1851),
    [sym_timeline_once] = ACTIONS(1851),
    [sym_timeline_repeat] = ACTIONS(1851),
    [sym_timeline_absolute] = ACTIONS(1851),
    [sym_timeline_relative] = ACTIONS(1851),
    [sym_do_push_timed_infinite] = ACTIONS(1851),
    [sym_source_type_no_address] = ACTIONS(1851),
    [sym_source_type_neuron_id] = ACTIONS(1851),
    [sym_source_type_ip_address] = ACTIONS(1851),
    [sym_source_type_axlink] = ACTIONS(1851),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1851),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1851),
    [sym_source_type_ip_socket_address] = ACTIONS(1851),
    [sym_source_type_rs232] = ACTIONS(1851),
    [sym_source_type_internal] = ACTIONS(1851),
    [sym_source_type_mac_address] = ACTIONS(1851),
    [sym_source_type_ipv4_port] = ACTIONS(1851),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1851),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1851),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1851),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1851),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1851),
    [sym_true] = ACTIONS(1851),
    [sym_false] = ACTIONS(1851),
    [sym_file_read_only] = ACTIONS(1851),
    [sym_file_rw_new] = ACTIONS(1851),
    [sym_file_rw_append] = ACTIONS(1851),
    [sym_ip_tcp] = ACTIONS(1851),
    [sym_ip_udp] = ACTIONS(1851),
    [sym_ip_udp_2way] = ACTIONS(1851),
    [sym_xml_encode_types] = ACTIONS(1851),
    [sym_xml_encode_char_as_list] = ACTIONS(1851),
    [sym_xml_encode_le] = ACTIONS(1851),
    [sym_xml_decode_types] = ACTIONS(1851),
    [sym_xml_decode_no_preserve] = ACTIONS(1851),
    [sym_duet_dev_type_display_device] = ACTIONS(1851),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1851),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1851),
    [sym_duet_dev_type_io_device] = ACTIONS(1851),
    [sym_duet_dev_type_relay_device] = ACTIONS(1851),
    [sym_duet_dev_type_ups] = ACTIONS(1851),
    [sym_duet_dev_type_amplifier] = ACTIONS(1851),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1851),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1851),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1851),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1851),
    [sym_duet_dev_type_camera] = ACTIONS(1851),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1851),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1851),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1851),
    [sym_duet_dev_type_dss] = ACTIONS(1851),
    [sym_duet_dev_type_dvr] = ACTIONS(1851),
    [sym_duet_dev_type_disc_device] = ACTIONS(1851),
    [sym_duet_dev_type_document_camera] = ACTIONS(1851),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1851),
    [sym_duet_dev_type_hvac] = ACTIONS(1851),
    [sym_duet_dev_type_keypad] = ACTIONS(1851),
    [sym_duet_dev_type_light] = ACTIONS(1851),
    [sym_duet_dev_type_monitor] = ACTIONS(1851),
    [sym_duet_dev_type_motor] = ACTIONS(1851),
    [sym_duet_dev_type_multi_window] = ACTIONS(1851),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1851),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1851),
    [sym_duet_dev_type_receiver] = ACTIONS(1851),
    [sym_duet_dev_type_security_system] = ACTIONS(1851),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1851),
    [sym_duet_dev_type_settop_box] = ACTIONS(1851),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1851),
    [sym_duet_dev_type_switcher] = ACTIONS(1851),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1851),
    [sym_duet_dev_type_tv] = ACTIONS(1851),
    [sym_duet_dev_type_utility] = ACTIONS(1851),
    [sym_duet_dev_type_vcr] = ACTIONS(1851),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1851),
    [sym_duet_dev_type_video_processor] = ACTIONS(1851),
    [sym_duet_dev_type_video_projector] = ACTIONS(1851),
    [sym_duet_dev_type_video_wall] = ACTIONS(1851),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1851),
    [sym_duet_dev_type_weather] = ACTIONS(1851),
    [sym_duet_dev_not_polled] = ACTIONS(1851),
    [sym_duet_dev_polled] = ACTIONS(1851),
    [sym_clkmgr_mode_network] = ACTIONS(1851),
    [sym_clkmgr_mode_standalone] = ACTIONS(1851),
    [sym_normal_standby] = ACTIONS(1851),
    [sym_normal_wake] = ACTIONS(1851),
    [sym_ip_multicast_ttl_option] = ACTIONS(1851),
    [sym_ip_ttl_subnet] = ACTIONS(1851),
    [sym_ip_ttl_site] = ACTIONS(1851),
    [sym_ip_ttl_region] = ACTIONS(1851),
    [sym_ip_ttl_continent] = ACTIONS(1851),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1851),
    [sym_ip_nodelay_on] = ACTIONS(1851),
    [sym_ip_nodelay_off] = ACTIONS(1851),
    [sym_amx_error] = ACTIONS(1851),
    [sym_amx_warning] = ACTIONS(1851),
    [sym_amx_info] = ACTIONS(1851),
    [sym_amx_debug] = ACTIONS(1851),
    [sym_netlinx_login_success] = ACTIONS(1851),
    [sym_netlinx_login_fail] = ACTIONS(1851),
    [sym_netlinx_logout] = ACTIONS(1851),
    [sym_null_str] = ACTIONS(1851),
    [sym_smtp_address] = ACTIONS(1851),
    [sym_smtp_port_number] = ACTIONS(1851),
    [sym_smtp_username] = ACTIONS(1851),
    [sym_smtp_password] = ACTIONS(1851),
    [sym_smtp_require_tls] = ACTIONS(1851),
    [sym_smtp_from] = ACTIONS(1851),
    [sym_smtp_tls_true] = ACTIONS(1851),
    [sym_smtp_tls_false] = ACTIONS(1851),
    [sym_listview_on_row_select_event] = ACTIONS(1851),
    [sym_data_structure_unknown] = ACTIONS(1851),
    [sym_data_structure_datafeed] = ACTIONS(1851),
    [sym_data_structure_datarecordset] = ACTIONS(1851),
    [sym_data_structure_datarecord] = ACTIONS(1851),
    [sym_data_structure_datafield] = ACTIONS(1851),
    [sym_data_max_content_field_count] = ACTIONS(1851),
    [sym_data_max_metadata_field_count] = ACTIONS(1851),
    [sym_data_max_id_length] = ACTIONS(1851),
    [sym_data_max_name_length] = ACTIONS(1851),
    [sym_data_max_description_length] = ACTIONS(1851),
    [sym_data_max_type_length] = ACTIONS(1851),
    [sym_data_max_format_length] = ACTIONS(1851),
    [sym_data_max_source_length] = ACTIONS(1851),
    [sym_data_max_value_length] = ACTIONS(1851),
    [sym_data_max_label_length] = ACTIONS(1851),
    [sym_data_max_url_length] = ACTIONS(1851),
    [sym_max_last_login_info_length] = ACTIONS(1851),
    [sym_data_type_unknown] = ACTIONS(1851),
    [sym_data_type_string] = ACTIONS(1851),
    [sym_data_type_datetime] = ACTIONS(1851),
    [sym_data_type_date] = ACTIONS(1851),
    [sym_data_type_time] = ACTIONS(1851),
    [sym_data_type_image] = ACTIONS(1851),
    [sym_data_format_url] = ACTIONS(1851),
    [sym_data_format_phone] = ACTIONS(1851),
    [sym_data_format_email] = ACTIONS(1851),
    [sym_data_format_iso8601] = ACTIONS(1851),
    [sym_auditlog_priv] = ACTIONS(1851),
    [sym_configuration_priv] = ACTIONS(1851),
    [sym_deviceconfig_priv] = ACTIONS(1851),
    [sym_ftp_priv] = ACTIONS(1851),
    [sym_http_priv] = ACTIONS(1851),
    [sym_networkconfig_priv] = ACTIONS(1851),
    [sym_programport_priv] = ACTIONS(1851),
    [sym_remoteui_priv] = ACTIONS(1851),
    [sym_securitycontrol_priv] = ACTIONS(1851),
    [sym_softwaremanagement_priv] = ACTIONS(1851),
    [sym_terminal_priv] = ACTIONS(1851),
    [sym_tpadmin_priv] = ACTIONS(1851),
    [sym_user1_priv] = ACTIONS(1851),
    [sym_user2_priv] = ACTIONS(1851),
    [sym_user3_priv] = ACTIONS(1851),
    [sym_user4_priv] = ACTIONS(1851),
    [sym_usermanagement_priv] = ACTIONS(1851),
    [sym_valid_account] = ACTIONS(1851),
    [sym_err_name_invalid] = ACTIONS(1851),
    [sym_err_password_invalid] = ACTIONS(1851),
    [sym_err_invalid_account] = ACTIONS(1851),
    [sym_err_no_authorization] = ACTIONS(1851),
    [sym_err_invalid_parameter] = ACTIONS(1851),
    [sym_err_user_locked_out] = ACTIONS(1851),
    [sym_err_user_account_expired] = ACTIONS(1851),
    [sym_err_general_error] = ACTIONS(1851),
    [sym_err_authentication_error] = ACTIONS(1851),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1851),
    [sym_tls_validate_certificate] = ACTIONS(1851),
    [sym_tdata] = ACTIONS(1851),
    [sym_tchannel] = ACTIONS(1851),
    [sym_tlevel] = ACTIONS(1851),
    [sym_tbutton] = ACTIONS(1851),
    [sym_ttimeline] = ACTIONS(1851),
    [sym_tcustom] = ACTIONS(1851),
    [sym_url_struct] = ACTIONS(1851),
    [sym_dns_struct] = ACTIONS(1851),
    [sym_ip_address_struct] = ACTIONS(1851),
    [sym_dev_info_struct] = ACTIONS(1851),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1851),
    [sym_data_feed] = ACTIONS(1851),
    [sym_data_field] = ACTIONS(1851),
    [sym_data_record] = ACTIONS(1851),
    [sym_wc_data_feed] = ACTIONS(1851),
    [sym_wc_data_field] = ACTIONS(1851),
    [sym_wc_data_record] = ACTIONS(1851),
    [sym_last_login_info] = ACTIONS(1851),
    [sym____reserved___] = ACTIONS(1851),
    [sym_date] = ACTIONS(1851),
    [sym_day] = ACTIONS(1851),
    [sym_time] = ACTIONS(1851),
    [sym_ldate] = ACTIONS(1851),
    [sym_push_device] = ACTIONS(1851),
    [sym_push_channel] = ACTIONS(1851),
    [sym_push_devchan] = ACTIONS(1851),
    [sym_release_device] = ACTIONS(1851),
    [sym_release_channel] = ACTIONS(1851),
    [sym_release_devchan] = ACTIONS(1851),
    [sym_master_slot] = ACTIONS(1851),
    [sym_get_pulse_time] = ACTIONS(1851),
    [sym_get_timer] = ACTIONS(1851),
    [sym_master_sn] = ACTIONS(1851),
    [sym_system_number] = ACTIONS(1851),
    [sym_button] = ACTIONS(1851),
    [sym_dv_channel] = ACTIONS(1851),
    [sym_data] = ACTIONS(1851),
    [sym_level] = ACTIONS(1851),
    [sym_timeline] = ACTIONS(1851),
    [sym_channel] = ACTIONS(1851),
    [sym_custom] = ACTIONS(1851),
    [sym_atoi] = ACTIONS(1851),
    [sym_atol] = ACTIONS(1851),
    [sym_atof] = ACTIONS(1851),
    [sym_itoa] = ACTIONS(1851),
    [sym_itohex] = ACTIONS(1851),
    [sym_hextoi] = ACTIONS(1851),
    [sym_ftoa] = ACTIONS(1851),
    [sym_find_string] = ACTIONS(1851),
    [sym_left_string] = ACTIONS(1851),
    [sym_right_string] = ACTIONS(1851),
    [sym_mid_string] = ACTIONS(1851),
    [sym_remove_string] = ACTIONS(1851),
    [sym_get_buffer_string] = ACTIONS(1851),
    [sym_get_buffer_char] = ACTIONS(1851),
    [sym_get_multi_buffer_string] = ACTIONS(1851),
    [sym_length_string] = ACTIONS(1851),
    [sym_max_length_string] = ACTIONS(1851),
    [sym_set_length_string] = ACTIONS(1851),
    [sym_compare_string] = ACTIONS(1851),
    [sym_upper_string] = ACTIONS(1851),
    [sym_lower_string] = ACTIONS(1851),
    [sym_get_last] = ACTIONS(1851),
    [sym_date_to_day] = ACTIONS(1851),
    [sym_date_to_month] = ACTIONS(1851),
    [sym_date_to_year] = ACTIONS(1851),
    [sym_time_to_hour] = ACTIONS(1851),
    [sym_time_to_minute] = ACTIONS(1851),
    [sym_time_to_second] = ACTIONS(1851),
    [sym_day_of_week] = ACTIONS(1851),
    [sym_set_pulse_time] = ACTIONS(1851),
    [sym_set_timer] = ACTIONS(1851),
    [sym_file_open] = ACTIONS(1851),
    [sym_file_close] = ACTIONS(1851),
    [sym_file_read] = ACTIONS(1851),
    [sym_file_read_line] = ACTIONS(1851),
    [sym_file_write] = ACTIONS(1851),
    [sym_file_write_line] = ACTIONS(1851),
    [sym_file_copy] = ACTIONS(1851),
    [sym_file_delete] = ACTIONS(1851),
    [sym_file_rename] = ACTIONS(1851),
    [sym_file_dir] = ACTIONS(1851),
    [sym_file_getdir] = ACTIONS(1851),
    [sym_file_setdir] = ACTIONS(1851),
    [sym_file_create_dir] = ACTIONS(1851),
    [sym_file_remove_dir] = ACTIONS(1851),
    [sym_file_seek] = ACTIONS(1851),
    [sym_length_array] = ACTIONS(1851),
    [sym_max_length_array] = ACTIONS(1851),
    [sym_set_length_array] = ACTIONS(1851),
    [sym_ip_client_open] = ACTIONS(1851),
    [sym_ip_client_close] = ACTIONS(1851),
    [sym_ip_server_open] = ACTIONS(1851),
    [sym_ip_server_close] = ACTIONS(1851),
    [sym_ip_bound_client_open] = ACTIONS(1851),
    [sym_ip_mc_server_open] = ACTIONS(1851),
    [sym_ip_set_option] = ACTIONS(1851),
    [sym_ssh_client_open] = ACTIONS(1851),
    [sym_ssh_client_close] = ACTIONS(1851),
    [sym_tls_client_open] = ACTIONS(1851),
    [sym_tls_client_close] = ACTIONS(1851),
    [sym_do_push] = ACTIONS(1851),
    [sym_do_push_timed] = ACTIONS(1851),
    [sym_do_release] = ACTIONS(1851),
    [sym_device_id] = ACTIONS(1851),
    [sym_device_info] = ACTIONS(1851),
    [sym_device_id_string] = ACTIONS(1851),
    [sym_device_standby] = ACTIONS(1851),
    [sym_device_wake] = ACTIONS(1851),
    [sym_timeline_create] = ACTIONS(1851),
    [sym_timeline_kill] = ACTIONS(1851),
    [sym_timeline_reload] = ACTIONS(1851),
    [sym_timeline_pause] = ACTIONS(1851),
    [sym_timeline_restart] = ACTIONS(1851),
    [sym_timeline_set] = ACTIONS(1851),
    [sym_timeline_get] = ACTIONS(1851),
    [sym_timeline_active] = ACTIONS(1851),
    [sym_timeline_dynamic_id] = ACTIONS(1851),
    [sym_variable_to_string] = ACTIONS(1851),
    [sym_string_to_variable] = ACTIONS(1851),
    [sym_length_variable_to_string] = ACTIONS(1851),
    [sym_type_cast] = ACTIONS(1851),
    [sym_variable_to_xml] = ACTIONS(1851),
    [sym_xml_to_variable] = ACTIONS(1851),
    [sym_length_variable_to_xml] = ACTIONS(1851),
    [sym_redirect_string] = ACTIONS(1851),
    [sym_random_number] = ACTIONS(1851),
    [sym_get_unique_id] = ACTIONS(1851),
    [sym_get_system_number] = ACTIONS(1851),
    [sym_set_system_number] = ACTIONS(1851),
    [sym_get_serial_number] = ACTIONS(1851),
    [sym_get_master_build] = ACTIONS(1851),
    [sym_set_validation_code] = ACTIONS(1851),
    [sym_reboot] = ACTIONS(1851),
    [sym_load_duet_module] = ACTIONS(1851),
    [sym_unload_duet_module] = ACTIONS(1851),
    [sym_duet_mem_size_set] = ACTIONS(1851),
    [sym_duet_mem_size_get] = ACTIONS(1851),
    [sym_dynamic_application_device] = ACTIONS(1851),
    [sym_dynamic_polled_port] = ACTIONS(1851),
    [sym_static_port_binding] = ACTIONS(1851),
    [sym_static_ip_binding] = ACTIONS(1851),
    [sym_get_dns_list] = ACTIONS(1851),
    [sym_set_dns_list] = ACTIONS(1851),
    [sym_get_ip_address] = ACTIONS(1851),
    [sym_set_ip_address] = ACTIONS(1851),
    [sym_get_url_list] = ACTIONS(1851),
    [sym_add_url_entry] = ACTIONS(1851),
    [sym_delete_url_entry] = ACTIONS(1851),
    [sym_abs_value] = ACTIONS(1851),
    [sym_max_value] = ACTIONS(1851),
    [sym_min_value] = ACTIONS(1851),
    [sym_exp_value] = ACTIONS(1851),
    [sym_log_value] = ACTIONS(1851),
    [sym_log10_value] = ACTIONS(1851),
    [sym_power_value] = ACTIONS(1851),
    [sym_sqrt_value] = ACTIONS(1851),
    [sym_raw_be] = ACTIONS(1851),
    [sym_raw_le] = ACTIONS(1851),
    [sym_internal_queue_size_set] = ACTIONS(1851),
    [sym_internal_queue_size_get] = ACTIONS(1851),
    [sym_internal_threshold_set] = ACTIONS(1851),
    [sym_internal_threshold_get] = ACTIONS(1851),
    [sym_rebuilt_event] = ACTIONS(1851),
    [sym_do_custom_event] = ACTIONS(1851),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1851),
    [sym_clkmgr_set_clk_source] = ACTIONS(1851),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1851),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1851),
    [sym_clkmgr_get_timezone] = ACTIONS(1851),
    [sym_clkmgr_set_timezone] = ACTIONS(1851),
    [sym_clkmgr_get_resync_period] = ACTIONS(1851),
    [sym_clkmgr_set_resync_period] = ACTIONS(1851),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1851),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1851),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1851),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1851),
    [sym_clkmgr_get_timeservers] = ACTIONS(1851),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1851),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1851),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1851),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1851),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1851),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1851),
    [sym_set_log_level] = ACTIONS(1851),
    [sym_get_log_level] = ACTIONS(1851),
    [sym_amx_log] = ACTIONS(1851),
    [sym_smtp_server_config_set] = ACTIONS(1851),
    [sym_smtp_server_config_get] = ACTIONS(1851),
    [sym_smtp_send] = ACTIONS(1851),
    [sym_validate_netlinx_account] = ACTIONS(1851),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1851),
    [sym_authenticate_certificate] = ACTIONS(1851),
    [sym_audit_netlinx_session_event] = ACTIONS(1851),
    [sym_audit_netlinx_generic_event] = ACTIONS(1851),
    [sym_get_available_flash_disk_space] = ACTIONS(1851),
    [sym_get_max_flash_disk_space] = ACTIONS(1851),
    [sym_data_create_feed] = ACTIONS(1851),
    [sym_data_delete_feed] = ACTIONS(1851),
    [sym_data_publish_feed] = ACTIONS(1851),
    [sym_data_get_published_feed] = ACTIONS(1851),
    [sym_data_add_record] = ACTIONS(1851),
    [sym_data_get_event_record] = ACTIONS(1851),
    [sym__wc_data_create_feed] = ACTIONS(1851),
    [sym__wc_data_add_record] = ACTIONS(1851),
    [sym__wc_data_get_event_record] = ACTIONS(1851),
    [sym_format] = ACTIONS(1851),
    [sym_astro_clock] = ACTIONS(1851),
    [sym_set_outdoor_temperature] = ACTIONS(1851),
    [sym_set_virtual_level_count] = ACTIONS(1851),
    [sym_set_virtual_channel_count] = ACTIONS(1851),
    [sym_set_virtual_port_count] = ACTIONS(1851),
    [sym__wc_encode_int] = ACTIONS(1851),
    [sym__wc_decode_int] = ACTIONS(1851),
    [sym__wc_int] = ACTIONS(1851),
    [sym__wc_to_ch_int] = ACTIONS(1851),
    [sym__ch_to_wc_int] = ACTIONS(1851),
    [sym__wc_find_string_int] = ACTIONS(1851),
    [sym__wc_left_string_int] = ACTIONS(1851),
    [sym__wc_lower_string_int] = ACTIONS(1851),
    [sym__wc_mid_string_int] = ACTIONS(1851),
    [sym__wc_remove_string_int] = ACTIONS(1851),
    [sym__wc_right_string_int] = ACTIONS(1851),
    [sym__wc_upper_string_int] = ACTIONS(1851),
    [sym__wc_compare_string_int] = ACTIONS(1851),
    [sym__wc_get_buffer_char_int] = ACTIONS(1851),
    [sym__wc_get_buffer_string_int] = ACTIONS(1851),
    [sym__wc_concat_string_int] = ACTIONS(1851),
    [sym__wc_file_open_int] = ACTIONS(1851),
    [sym__wc_file_close_int] = ACTIONS(1851),
    [sym__wc_file_read_int] = ACTIONS(1851),
    [sym__wc_file_read_line_int] = ACTIONS(1851),
    [sym__wc_file_write_int] = ACTIONS(1851),
    [sym__wc_file_write_line_int] = ACTIONS(1851),
    [aux_sym_preproc_include_token1] = ACTIONS(1851),
    [aux_sym_preproc_define_token1] = ACTIONS(1853),
    [aux_sym_preproc_warn_token1] = ACTIONS(1853),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1853),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1853),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1853),
    [anon_sym_LBRACK] = ACTIONS(1853),
    [anon_sym_LBRACE] = ACTIONS(1853),
    [anon_sym_LPAREN] = ACTIONS(1851),
    [anon_sym_BANG] = ACTIONS(1853),
    [anon_sym_TILDE] = ACTIONS(1853),
    [anon_sym_DASH] = ACTIONS(1851),
    [anon_sym_PLUS] = ACTIONS(1851),
    [anon_sym_DASH_DASH] = ACTIONS(1853),
    [anon_sym_PLUS_PLUS] = ACTIONS(1853),
    [anon_sym_DQUOTE] = ACTIONS(1853),
    [anon_sym_SQUOTE] = ACTIONS(1853),
    [sym_number_literal] = ACTIONS(1853),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(295)] = {
    [sym_identifier] = ACTIONS(1961),
    [sym_program_name_keyword] = ACTIONS(1961),
    [sym_module_name_keyword] = ACTIONS(1961),
    [sym_define_device_keyword] = ACTIONS(1961),
    [sym_define_combine_keyword] = ACTIONS(1961),
    [sym_define_constant_keyword] = ACTIONS(1961),
    [sym_define_type_keyword] = ACTIONS(1961),
    [sym_define_variable_keyword] = ACTIONS(1961),
    [sym_define_system_variable_keyword] = ACTIONS(1961),
    [sym_define_connect_level_keyword] = ACTIONS(1961),
    [sym_define_latching_keyword] = ACTIONS(1961),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1961),
    [sym_define_toggling_keyword] = ACTIONS(1961),
    [sym_define_call_keyword] = ACTIONS(1961),
    [sym_define_function_keyword] = ACTIONS(1961),
    [sym_define_library_function_keyword] = ACTIONS(1961),
    [sym_define_start_keyword] = ACTIONS(1961),
    [sym_define_event_keyword] = ACTIONS(1961),
    [sym_define_program_keyword] = ACTIONS(1961),
    [sym_define_module_keyword] = ACTIONS(1961),
    [sym_char_keyword] = ACTIONS(1961),
    [sym_widechar_keyword] = ACTIONS(1961),
    [sym_integer_keyword] = ACTIONS(1961),
    [sym_sinteger_keyword] = ACTIONS(1961),
    [sym_long_keyword] = ACTIONS(1961),
    [sym_slong_keyword] = ACTIONS(1961),
    [sym_float_keyword] = ACTIONS(1961),
    [sym_double_keyword] = ACTIONS(1961),
    [sym_variant_keyword] = ACTIONS(1961),
    [sym_variantarray_keyword] = ACTIONS(1961),
    [sym_local_var_keyword] = ACTIONS(1961),
    [sym_stack_var_keyword] = ACTIONS(1961),
    [sym_constant_keyword] = ACTIONS(1961),
    [sym_volatile_keyword] = ACTIONS(1961),
    [sym_non_volatile_keyword] = ACTIONS(1961),
    [sym_persistent_keyword] = ACTIONS(1961),
    [sym_dev_keyword] = ACTIONS(1961),
    [sym_devlev_keyword] = ACTIONS(1961),
    [sym_devchan_keyword] = ACTIONS(1961),
    [sym_if_keyword] = ACTIONS(1961),
    [sym_switch_keyword] = ACTIONS(1961),
    [sym_case_keyword] = ACTIONS(1961),
    [sym_default_keyword] = ACTIONS(1961),
    [sym_while_keyword] = ACTIONS(1961),
    [sym_for_keyword] = ACTIONS(1961),
    [sym_break_keyword] = ACTIONS(1961),
    [sym_continue_keyword] = ACTIONS(1961),
    [sym_return_keyword] = ACTIONS(1961),
    [sym_select_keyword] = ACTIONS(1961),
    [sym_active_keyword] = ACTIONS(1961),
    [sym_struct_keyword] = ACTIONS(1961),
    [sym_structure_keyword] = ACTIONS(1961),
    [sym_button_event_keyword] = ACTIONS(1961),
    [sym_channel_event_keyword] = ACTIONS(1961),
    [sym_level_event_keyword] = ACTIONS(1961),
    [sym_data_event_keyword] = ACTIONS(1961),
    [sym_timeline_event_keyword] = ACTIONS(1961),
    [sym_custom_event_keyword] = ACTIONS(1961),
    [sym_push_keyword] = ACTIONS(1961),
    [sym_release_keyword] = ACTIONS(1961),
    [aux_sym_on_keyword_token1] = ACTIONS(1961),
    [aux_sym_off_keyword_token1] = ACTIONS(1961),
    [sym_send_level_keyword] = ACTIONS(1961),
    [sym_send_string_keyword] = ACTIONS(1961),
    [sym_send_command_keyword] = ACTIONS(1961),
    [sym_clear_buffer_keyword] = ACTIONS(1961),
    [sym_create_buffer_keyword] = ACTIONS(1961),
    [sym_create_multi_buffer_keyword] = ACTIONS(1961),
    [sym_call_keyword] = ACTIONS(1961),
    [sym_system_call_keyword] = ACTIONS(1961),
    [sym_devchan_to_keyword] = ACTIONS(1961),
    [sym_devchan_min_to_keyword] = ACTIONS(1961),
    [sym_devchan_total_off_keyword] = ACTIONS(1961),
    [sym_devchan_pulse_keyword] = ACTIONS(1961),
    [sym_wait_keyword] = ACTIONS(1961),
    [sym_cancel_wait_keyword] = ACTIONS(1961),
    [sym_wait_until_keyword] = ACTIONS(1961),
    [sym_cancel_wait_until_keyword] = ACTIONS(1961),
    [sym_cancel_all_wait_keyword] = ACTIONS(1961),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1961),
    [sym_not] = ACTIONS(1961),
    [sym_bnot] = ACTIONS(1961),
    [sym___file__] = ACTIONS(1961),
    [sym___date__] = ACTIONS(1961),
    [sym___ldate__] = ACTIONS(1961),
    [sym___time__] = ACTIONS(1961),
    [sym___line__] = ACTIONS(1961),
    [sym___name__] = ACTIONS(1961),
    [sym___netlinx__] = ACTIONS(1961),
    [sym_netlinx_axi_version] = ACTIONS(1961),
    [sym_url_flg_tcp] = ACTIONS(1961),
    [sym_url_flg_accinfopresent] = ACTIONS(1961),
    [sym_url_flg_temp] = ACTIONS(1961),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1961),
    [sym_url_flg_stat_mask] = ACTIONS(1961),
    [sym_url_flg_stat_lookup] = ACTIONS(1961),
    [sym_url_flg_stat_connecting] = ACTIONS(1961),
    [sym_url_flg_stat_waiting] = ACTIONS(1961),
    [sym_url_flg_stat_connected] = ACTIONS(1961),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1961),
    [sym_first_virtual_device] = ACTIONS(1961),
    [sym_first_local_port] = ACTIONS(1961),
    [sym_dynamic_virtual_device] = ACTIONS(1961),
    [sym_timeline_once] = ACTIONS(1961),
    [sym_timeline_repeat] = ACTIONS(1961),
    [sym_timeline_absolute] = ACTIONS(1961),
    [sym_timeline_relative] = ACTIONS(1961),
    [sym_do_push_timed_infinite] = ACTIONS(1961),
    [sym_source_type_no_address] = ACTIONS(1961),
    [sym_source_type_neuron_id] = ACTIONS(1961),
    [sym_source_type_ip_address] = ACTIONS(1961),
    [sym_source_type_axlink] = ACTIONS(1961),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1961),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1961),
    [sym_source_type_ip_socket_address] = ACTIONS(1961),
    [sym_source_type_rs232] = ACTIONS(1961),
    [sym_source_type_internal] = ACTIONS(1961),
    [sym_source_type_mac_address] = ACTIONS(1961),
    [sym_source_type_ipv4_port] = ACTIONS(1961),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1961),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1961),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1961),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1961),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1961),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1961),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1961),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1961),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1961),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1961),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1961),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1961),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1961),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1961),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1961),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1961),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1961),
    [sym_true] = ACTIONS(1961),
    [sym_false] = ACTIONS(1961),
    [sym_file_read_only] = ACTIONS(1961),
    [sym_file_rw_new] = ACTIONS(1961),
    [sym_file_rw_append] = ACTIONS(1961),
    [sym_ip_tcp] = ACTIONS(1961),
    [sym_ip_udp] = ACTIONS(1961),
    [sym_ip_udp_2way] = ACTIONS(1961),
    [sym_xml_encode_types] = ACTIONS(1961),
    [sym_xml_encode_char_as_list] = ACTIONS(1961),
    [sym_xml_encode_le] = ACTIONS(1961),
    [sym_xml_decode_types] = ACTIONS(1961),
    [sym_xml_decode_no_preserve] = ACTIONS(1961),
    [sym_duet_dev_type_display_device] = ACTIONS(1961),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1961),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1961),
    [sym_duet_dev_type_io_device] = ACTIONS(1961),
    [sym_duet_dev_type_relay_device] = ACTIONS(1961),
    [sym_duet_dev_type_ups] = ACTIONS(1961),
    [sym_duet_dev_type_amplifier] = ACTIONS(1961),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1961),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1961),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1961),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1961),
    [sym_duet_dev_type_camera] = ACTIONS(1961),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1961),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1961),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1961),
    [sym_duet_dev_type_dss] = ACTIONS(1961),
    [sym_duet_dev_type_dvr] = ACTIONS(1961),
    [sym_duet_dev_type_disc_device] = ACTIONS(1961),
    [sym_duet_dev_type_document_camera] = ACTIONS(1961),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1961),
    [sym_duet_dev_type_hvac] = ACTIONS(1961),
    [sym_duet_dev_type_keypad] = ACTIONS(1961),
    [sym_duet_dev_type_light] = ACTIONS(1961),
    [sym_duet_dev_type_monitor] = ACTIONS(1961),
    [sym_duet_dev_type_motor] = ACTIONS(1961),
    [sym_duet_dev_type_multi_window] = ACTIONS(1961),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1961),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1961),
    [sym_duet_dev_type_receiver] = ACTIONS(1961),
    [sym_duet_dev_type_security_system] = ACTIONS(1961),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1961),
    [sym_duet_dev_type_settop_box] = ACTIONS(1961),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1961),
    [sym_duet_dev_type_switcher] = ACTIONS(1961),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1961),
    [sym_duet_dev_type_tv] = ACTIONS(1961),
    [sym_duet_dev_type_utility] = ACTIONS(1961),
    [sym_duet_dev_type_vcr] = ACTIONS(1961),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1961),
    [sym_duet_dev_type_video_processor] = ACTIONS(1961),
    [sym_duet_dev_type_video_projector] = ACTIONS(1961),
    [sym_duet_dev_type_video_wall] = ACTIONS(1961),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1961),
    [sym_duet_dev_type_weather] = ACTIONS(1961),
    [sym_duet_dev_not_polled] = ACTIONS(1961),
    [sym_duet_dev_polled] = ACTIONS(1961),
    [sym_clkmgr_mode_network] = ACTIONS(1961),
    [sym_clkmgr_mode_standalone] = ACTIONS(1961),
    [sym_normal_standby] = ACTIONS(1961),
    [sym_normal_wake] = ACTIONS(1961),
    [sym_ip_multicast_ttl_option] = ACTIONS(1961),
    [sym_ip_ttl_subnet] = ACTIONS(1961),
    [sym_ip_ttl_site] = ACTIONS(1961),
    [sym_ip_ttl_region] = ACTIONS(1961),
    [sym_ip_ttl_continent] = ACTIONS(1961),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1961),
    [sym_ip_nodelay_on] = ACTIONS(1961),
    [sym_ip_nodelay_off] = ACTIONS(1961),
    [sym_amx_error] = ACTIONS(1961),
    [sym_amx_warning] = ACTIONS(1961),
    [sym_amx_info] = ACTIONS(1961),
    [sym_amx_debug] = ACTIONS(1961),
    [sym_netlinx_login_success] = ACTIONS(1961),
    [sym_netlinx_login_fail] = ACTIONS(1961),
    [sym_netlinx_logout] = ACTIONS(1961),
    [sym_null_str] = ACTIONS(1961),
    [sym_smtp_address] = ACTIONS(1961),
    [sym_smtp_port_number] = ACTIONS(1961),
    [sym_smtp_username] = ACTIONS(1961),
    [sym_smtp_password] = ACTIONS(1961),
    [sym_smtp_require_tls] = ACTIONS(1961),
    [sym_smtp_from] = ACTIONS(1961),
    [sym_smtp_tls_true] = ACTIONS(1961),
    [sym_smtp_tls_false] = ACTIONS(1961),
    [sym_listview_on_row_select_event] = ACTIONS(1961),
    [sym_data_structure_unknown] = ACTIONS(1961),
    [sym_data_structure_datafeed] = ACTIONS(1961),
    [sym_data_structure_datarecordset] = ACTIONS(1961),
    [sym_data_structure_datarecord] = ACTIONS(1961),
    [sym_data_structure_datafield] = ACTIONS(1961),
    [sym_data_max_content_field_count] = ACTIONS(1961),
    [sym_data_max_metadata_field_count] = ACTIONS(1961),
    [sym_data_max_id_length] = ACTIONS(1961),
    [sym_data_max_name_length] = ACTIONS(1961),
    [sym_data_max_description_length] = ACTIONS(1961),
    [sym_data_max_type_length] = ACTIONS(1961),
    [sym_data_max_format_length] = ACTIONS(1961),
    [sym_data_max_source_length] = ACTIONS(1961),
    [sym_data_max_value_length] = ACTIONS(1961),
    [sym_data_max_label_length] = ACTIONS(1961),
    [sym_data_max_url_length] = ACTIONS(1961),
    [sym_max_last_login_info_length] = ACTIONS(1961),
    [sym_data_type_unknown] = ACTIONS(1961),
    [sym_data_type_string] = ACTIONS(1961),
    [sym_data_type_datetime] = ACTIONS(1961),
    [sym_data_type_date] = ACTIONS(1961),
    [sym_data_type_time] = ACTIONS(1961),
    [sym_data_type_image] = ACTIONS(1961),
    [sym_data_format_url] = ACTIONS(1961),
    [sym_data_format_phone] = ACTIONS(1961),
    [sym_data_format_email] = ACTIONS(1961),
    [sym_data_format_iso8601] = ACTIONS(1961),
    [sym_auditlog_priv] = ACTIONS(1961),
    [sym_configuration_priv] = ACTIONS(1961),
    [sym_deviceconfig_priv] = ACTIONS(1961),
    [sym_ftp_priv] = ACTIONS(1961),
    [sym_http_priv] = ACTIONS(1961),
    [sym_networkconfig_priv] = ACTIONS(1961),
    [sym_programport_priv] = ACTIONS(1961),
    [sym_remoteui_priv] = ACTIONS(1961),
    [sym_securitycontrol_priv] = ACTIONS(1961),
    [sym_softwaremanagement_priv] = ACTIONS(1961),
    [sym_terminal_priv] = ACTIONS(1961),
    [sym_tpadmin_priv] = ACTIONS(1961),
    [sym_user1_priv] = ACTIONS(1961),
    [sym_user2_priv] = ACTIONS(1961),
    [sym_user3_priv] = ACTIONS(1961),
    [sym_user4_priv] = ACTIONS(1961),
    [sym_usermanagement_priv] = ACTIONS(1961),
    [sym_valid_account] = ACTIONS(1961),
    [sym_err_name_invalid] = ACTIONS(1961),
    [sym_err_password_invalid] = ACTIONS(1961),
    [sym_err_invalid_account] = ACTIONS(1961),
    [sym_err_no_authorization] = ACTIONS(1961),
    [sym_err_invalid_parameter] = ACTIONS(1961),
    [sym_err_user_locked_out] = ACTIONS(1961),
    [sym_err_user_account_expired] = ACTIONS(1961),
    [sym_err_general_error] = ACTIONS(1961),
    [sym_err_authentication_error] = ACTIONS(1961),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1961),
    [sym_tls_validate_certificate] = ACTIONS(1961),
    [sym_tdata] = ACTIONS(1961),
    [sym_tchannel] = ACTIONS(1961),
    [sym_tlevel] = ACTIONS(1961),
    [sym_tbutton] = ACTIONS(1961),
    [sym_ttimeline] = ACTIONS(1961),
    [sym_tcustom] = ACTIONS(1961),
    [sym_url_struct] = ACTIONS(1961),
    [sym_dns_struct] = ACTIONS(1961),
    [sym_ip_address_struct] = ACTIONS(1961),
    [sym_dev_info_struct] = ACTIONS(1961),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1961),
    [sym_data_feed] = ACTIONS(1961),
    [sym_data_field] = ACTIONS(1961),
    [sym_data_record] = ACTIONS(1961),
    [sym_wc_data_feed] = ACTIONS(1961),
    [sym_wc_data_field] = ACTIONS(1961),
    [sym_wc_data_record] = ACTIONS(1961),
    [sym_last_login_info] = ACTIONS(1961),
    [sym____reserved___] = ACTIONS(1961),
    [sym_date] = ACTIONS(1961),
    [sym_day] = ACTIONS(1961),
    [sym_time] = ACTIONS(1961),
    [sym_ldate] = ACTIONS(1961),
    [sym_push_device] = ACTIONS(1961),
    [sym_push_channel] = ACTIONS(1961),
    [sym_push_devchan] = ACTIONS(1961),
    [sym_release_device] = ACTIONS(1961),
    [sym_release_channel] = ACTIONS(1961),
    [sym_release_devchan] = ACTIONS(1961),
    [sym_master_slot] = ACTIONS(1961),
    [sym_get_pulse_time] = ACTIONS(1961),
    [sym_get_timer] = ACTIONS(1961),
    [sym_master_sn] = ACTIONS(1961),
    [sym_system_number] = ACTIONS(1961),
    [sym_button] = ACTIONS(1961),
    [sym_dv_channel] = ACTIONS(1961),
    [sym_data] = ACTIONS(1961),
    [sym_level] = ACTIONS(1961),
    [sym_timeline] = ACTIONS(1961),
    [sym_channel] = ACTIONS(1961),
    [sym_custom] = ACTIONS(1961),
    [sym_atoi] = ACTIONS(1961),
    [sym_atol] = ACTIONS(1961),
    [sym_atof] = ACTIONS(1961),
    [sym_itoa] = ACTIONS(1961),
    [sym_itohex] = ACTIONS(1961),
    [sym_hextoi] = ACTIONS(1961),
    [sym_ftoa] = ACTIONS(1961),
    [sym_find_string] = ACTIONS(1961),
    [sym_left_string] = ACTIONS(1961),
    [sym_right_string] = ACTIONS(1961),
    [sym_mid_string] = ACTIONS(1961),
    [sym_remove_string] = ACTIONS(1961),
    [sym_get_buffer_string] = ACTIONS(1961),
    [sym_get_buffer_char] = ACTIONS(1961),
    [sym_get_multi_buffer_string] = ACTIONS(1961),
    [sym_length_string] = ACTIONS(1961),
    [sym_max_length_string] = ACTIONS(1961),
    [sym_set_length_string] = ACTIONS(1961),
    [sym_compare_string] = ACTIONS(1961),
    [sym_upper_string] = ACTIONS(1961),
    [sym_lower_string] = ACTIONS(1961),
    [sym_get_last] = ACTIONS(1961),
    [sym_date_to_day] = ACTIONS(1961),
    [sym_date_to_month] = ACTIONS(1961),
    [sym_date_to_year] = ACTIONS(1961),
    [sym_time_to_hour] = ACTIONS(1961),
    [sym_time_to_minute] = ACTIONS(1961),
    [sym_time_to_second] = ACTIONS(1961),
    [sym_day_of_week] = ACTIONS(1961),
    [sym_set_pulse_time] = ACTIONS(1961),
    [sym_set_timer] = ACTIONS(1961),
    [sym_file_open] = ACTIONS(1961),
    [sym_file_close] = ACTIONS(1961),
    [sym_file_read] = ACTIONS(1961),
    [sym_file_read_line] = ACTIONS(1961),
    [sym_file_write] = ACTIONS(1961),
    [sym_file_write_line] = ACTIONS(1961),
    [sym_file_copy] = ACTIONS(1961),
    [sym_file_delete] = ACTIONS(1961),
    [sym_file_rename] = ACTIONS(1961),
    [sym_file_dir] = ACTIONS(1961),
    [sym_file_getdir] = ACTIONS(1961),
    [sym_file_setdir] = ACTIONS(1961),
    [sym_file_create_dir] = ACTIONS(1961),
    [sym_file_remove_dir] = ACTIONS(1961),
    [sym_file_seek] = ACTIONS(1961),
    [sym_length_array] = ACTIONS(1961),
    [sym_max_length_array] = ACTIONS(1961),
    [sym_set_length_array] = ACTIONS(1961),
    [sym_ip_client_open] = ACTIONS(1961),
    [sym_ip_client_close] = ACTIONS(1961),
    [sym_ip_server_open] = ACTIONS(1961),
    [sym_ip_server_close] = ACTIONS(1961),
    [sym_ip_bound_client_open] = ACTIONS(1961),
    [sym_ip_mc_server_open] = ACTIONS(1961),
    [sym_ip_set_option] = ACTIONS(1961),
    [sym_ssh_client_open] = ACTIONS(1961),
    [sym_ssh_client_close] = ACTIONS(1961),
    [sym_tls_client_open] = ACTIONS(1961),
    [sym_tls_client_close] = ACTIONS(1961),
    [sym_do_push] = ACTIONS(1961),
    [sym_do_push_timed] = ACTIONS(1961),
    [sym_do_release] = ACTIONS(1961),
    [sym_device_id] = ACTIONS(1961),
    [sym_device_info] = ACTIONS(1961),
    [sym_device_id_string] = ACTIONS(1961),
    [sym_device_standby] = ACTIONS(1961),
    [sym_device_wake] = ACTIONS(1961),
    [sym_timeline_create] = ACTIONS(1961),
    [sym_timeline_kill] = ACTIONS(1961),
    [sym_timeline_reload] = ACTIONS(1961),
    [sym_timeline_pause] = ACTIONS(1961),
    [sym_timeline_restart] = ACTIONS(1961),
    [sym_timeline_set] = ACTIONS(1961),
    [sym_timeline_get] = ACTIONS(1961),
    [sym_timeline_active] = ACTIONS(1961),
    [sym_timeline_dynamic_id] = ACTIONS(1961),
    [sym_variable_to_string] = ACTIONS(1961),
    [sym_string_to_variable] = ACTIONS(1961),
    [sym_length_variable_to_string] = ACTIONS(1961),
    [sym_type_cast] = ACTIONS(1961),
    [sym_variable_to_xml] = ACTIONS(1961),
    [sym_xml_to_variable] = ACTIONS(1961),
    [sym_length_variable_to_xml] = ACTIONS(1961),
    [sym_redirect_string] = ACTIONS(1961),
    [sym_random_number] = ACTIONS(1961),
    [sym_get_unique_id] = ACTIONS(1961),
    [sym_get_system_number] = ACTIONS(1961),
    [sym_set_system_number] = ACTIONS(1961),
    [sym_get_serial_number] = ACTIONS(1961),
    [sym_get_master_build] = ACTIONS(1961),
    [sym_set_validation_code] = ACTIONS(1961),
    [sym_reboot] = ACTIONS(1961),
    [sym_load_duet_module] = ACTIONS(1961),
    [sym_unload_duet_module] = ACTIONS(1961),
    [sym_duet_mem_size_set] = ACTIONS(1961),
    [sym_duet_mem_size_get] = ACTIONS(1961),
    [sym_dynamic_application_device] = ACTIONS(1961),
    [sym_dynamic_polled_port] = ACTIONS(1961),
    [sym_static_port_binding] = ACTIONS(1961),
    [sym_static_ip_binding] = ACTIONS(1961),
    [sym_get_dns_list] = ACTIONS(1961),
    [sym_set_dns_list] = ACTIONS(1961),
    [sym_get_ip_address] = ACTIONS(1961),
    [sym_set_ip_address] = ACTIONS(1961),
    [sym_get_url_list] = ACTIONS(1961),
    [sym_add_url_entry] = ACTIONS(1961),
    [sym_delete_url_entry] = ACTIONS(1961),
    [sym_abs_value] = ACTIONS(1961),
    [sym_max_value] = ACTIONS(1961),
    [sym_min_value] = ACTIONS(1961),
    [sym_exp_value] = ACTIONS(1961),
    [sym_log_value] = ACTIONS(1961),
    [sym_log10_value] = ACTIONS(1961),
    [sym_power_value] = ACTIONS(1961),
    [sym_sqrt_value] = ACTIONS(1961),
    [sym_raw_be] = ACTIONS(1961),
    [sym_raw_le] = ACTIONS(1961),
    [sym_internal_queue_size_set] = ACTIONS(1961),
    [sym_internal_queue_size_get] = ACTIONS(1961),
    [sym_internal_threshold_set] = ACTIONS(1961),
    [sym_internal_threshold_get] = ACTIONS(1961),
    [sym_rebuilt_event] = ACTIONS(1961),
    [sym_do_custom_event] = ACTIONS(1961),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1961),
    [sym_clkmgr_set_clk_source] = ACTIONS(1961),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1961),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1961),
    [sym_clkmgr_get_timezone] = ACTIONS(1961),
    [sym_clkmgr_set_timezone] = ACTIONS(1961),
    [sym_clkmgr_get_resync_period] = ACTIONS(1961),
    [sym_clkmgr_set_resync_period] = ACTIONS(1961),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1961),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1961),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1961),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1961),
    [sym_clkmgr_get_timeservers] = ACTIONS(1961),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1961),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1961),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1961),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1961),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1961),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1961),
    [sym_set_log_level] = ACTIONS(1961),
    [sym_get_log_level] = ACTIONS(1961),
    [sym_amx_log] = ACTIONS(1961),
    [sym_smtp_server_config_set] = ACTIONS(1961),
    [sym_smtp_server_config_get] = ACTIONS(1961),
    [sym_smtp_send] = ACTIONS(1961),
    [sym_validate_netlinx_account] = ACTIONS(1961),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1961),
    [sym_authenticate_certificate] = ACTIONS(1961),
    [sym_audit_netlinx_session_event] = ACTIONS(1961),
    [sym_audit_netlinx_generic_event] = ACTIONS(1961),
    [sym_get_available_flash_disk_space] = ACTIONS(1961),
    [sym_get_max_flash_disk_space] = ACTIONS(1961),
    [sym_data_create_feed] = ACTIONS(1961),
    [sym_data_delete_feed] = ACTIONS(1961),
    [sym_data_publish_feed] = ACTIONS(1961),
    [sym_data_get_published_feed] = ACTIONS(1961),
    [sym_data_add_record] = ACTIONS(1961),
    [sym_data_get_event_record] = ACTIONS(1961),
    [sym__wc_data_create_feed] = ACTIONS(1961),
    [sym__wc_data_add_record] = ACTIONS(1961),
    [sym__wc_data_get_event_record] = ACTIONS(1961),
    [sym_format] = ACTIONS(1961),
    [sym_astro_clock] = ACTIONS(1961),
    [sym_set_outdoor_temperature] = ACTIONS(1961),
    [sym_set_virtual_level_count] = ACTIONS(1961),
    [sym_set_virtual_channel_count] = ACTIONS(1961),
    [sym_set_virtual_port_count] = ACTIONS(1961),
    [sym__wc_encode_int] = ACTIONS(1961),
    [sym__wc_decode_int] = ACTIONS(1961),
    [sym__wc_int] = ACTIONS(1961),
    [sym__wc_to_ch_int] = ACTIONS(1961),
    [sym__ch_to_wc_int] = ACTIONS(1961),
    [sym__wc_find_string_int] = ACTIONS(1961),
    [sym__wc_left_string_int] = ACTIONS(1961),
    [sym__wc_lower_string_int] = ACTIONS(1961),
    [sym__wc_mid_string_int] = ACTIONS(1961),
    [sym__wc_remove_string_int] = ACTIONS(1961),
    [sym__wc_right_string_int] = ACTIONS(1961),
    [sym__wc_upper_string_int] = ACTIONS(1961),
    [sym__wc_compare_string_int] = ACTIONS(1961),
    [sym__wc_get_buffer_char_int] = ACTIONS(1961),
    [sym__wc_get_buffer_string_int] = ACTIONS(1961),
    [sym__wc_concat_string_int] = ACTIONS(1961),
    [sym__wc_file_open_int] = ACTIONS(1961),
    [sym__wc_file_close_int] = ACTIONS(1961),
    [sym__wc_file_read_int] = ACTIONS(1961),
    [sym__wc_file_read_line_int] = ACTIONS(1961),
    [sym__wc_file_write_int] = ACTIONS(1961),
    [sym__wc_file_write_line_int] = ACTIONS(1961),
    [aux_sym_preproc_include_token1] = ACTIONS(1961),
    [aux_sym_preproc_define_token1] = ACTIONS(1963),
    [aux_sym_preproc_warn_token1] = ACTIONS(1963),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1963),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1963),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1963),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1963),
    [aux_sym_preproc_else_token1] = ACTIONS(1963),
    [anon_sym_LBRACK] = ACTIONS(1963),
    [anon_sym_LBRACE] = ACTIONS(1963),
    [anon_sym_LPAREN] = ACTIONS(1961),
    [anon_sym_BANG] = ACTIONS(1963),
    [anon_sym_TILDE] = ACTIONS(1963),
    [anon_sym_DASH] = ACTIONS(1961),
    [anon_sym_PLUS] = ACTIONS(1961),
    [anon_sym_DASH_DASH] = ACTIONS(1963),
    [anon_sym_PLUS_PLUS] = ACTIONS(1963),
    [anon_sym_DQUOTE] = ACTIONS(1963),
    [anon_sym_SQUOTE] = ACTIONS(1963),
    [sym_number_literal] = ACTIONS(1963),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(296)] = {
    [sym_identifier] = ACTIONS(1965),
    [sym_program_name_keyword] = ACTIONS(1965),
    [sym_module_name_keyword] = ACTIONS(1965),
    [sym_define_device_keyword] = ACTIONS(1965),
    [sym_define_combine_keyword] = ACTIONS(1965),
    [sym_define_constant_keyword] = ACTIONS(1965),
    [sym_define_type_keyword] = ACTIONS(1965),
    [sym_define_variable_keyword] = ACTIONS(1965),
    [sym_define_system_variable_keyword] = ACTIONS(1965),
    [sym_define_connect_level_keyword] = ACTIONS(1965),
    [sym_define_latching_keyword] = ACTIONS(1965),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1965),
    [sym_define_toggling_keyword] = ACTIONS(1965),
    [sym_define_call_keyword] = ACTIONS(1965),
    [sym_define_function_keyword] = ACTIONS(1965),
    [sym_define_library_function_keyword] = ACTIONS(1965),
    [sym_define_start_keyword] = ACTIONS(1965),
    [sym_define_event_keyword] = ACTIONS(1965),
    [sym_define_program_keyword] = ACTIONS(1965),
    [sym_define_module_keyword] = ACTIONS(1965),
    [sym_char_keyword] = ACTIONS(1965),
    [sym_widechar_keyword] = ACTIONS(1965),
    [sym_integer_keyword] = ACTIONS(1965),
    [sym_sinteger_keyword] = ACTIONS(1965),
    [sym_long_keyword] = ACTIONS(1965),
    [sym_slong_keyword] = ACTIONS(1965),
    [sym_float_keyword] = ACTIONS(1965),
    [sym_double_keyword] = ACTIONS(1965),
    [sym_variant_keyword] = ACTIONS(1965),
    [sym_variantarray_keyword] = ACTIONS(1965),
    [sym_local_var_keyword] = ACTIONS(1965),
    [sym_stack_var_keyword] = ACTIONS(1965),
    [sym_constant_keyword] = ACTIONS(1965),
    [sym_volatile_keyword] = ACTIONS(1965),
    [sym_non_volatile_keyword] = ACTIONS(1965),
    [sym_persistent_keyword] = ACTIONS(1965),
    [sym_dev_keyword] = ACTIONS(1965),
    [sym_devlev_keyword] = ACTIONS(1965),
    [sym_devchan_keyword] = ACTIONS(1965),
    [sym_if_keyword] = ACTIONS(1965),
    [sym_switch_keyword] = ACTIONS(1965),
    [sym_case_keyword] = ACTIONS(1965),
    [sym_default_keyword] = ACTIONS(1965),
    [sym_while_keyword] = ACTIONS(1965),
    [sym_for_keyword] = ACTIONS(1965),
    [sym_break_keyword] = ACTIONS(1965),
    [sym_continue_keyword] = ACTIONS(1965),
    [sym_return_keyword] = ACTIONS(1965),
    [sym_select_keyword] = ACTIONS(1965),
    [sym_active_keyword] = ACTIONS(1965),
    [sym_struct_keyword] = ACTIONS(1965),
    [sym_structure_keyword] = ACTIONS(1965),
    [sym_button_event_keyword] = ACTIONS(1965),
    [sym_channel_event_keyword] = ACTIONS(1965),
    [sym_level_event_keyword] = ACTIONS(1965),
    [sym_data_event_keyword] = ACTIONS(1965),
    [sym_timeline_event_keyword] = ACTIONS(1965),
    [sym_custom_event_keyword] = ACTIONS(1965),
    [sym_push_keyword] = ACTIONS(1965),
    [sym_release_keyword] = ACTIONS(1965),
    [aux_sym_on_keyword_token1] = ACTIONS(1965),
    [aux_sym_off_keyword_token1] = ACTIONS(1965),
    [sym_send_level_keyword] = ACTIONS(1965),
    [sym_send_string_keyword] = ACTIONS(1965),
    [sym_send_command_keyword] = ACTIONS(1965),
    [sym_clear_buffer_keyword] = ACTIONS(1965),
    [sym_create_buffer_keyword] = ACTIONS(1965),
    [sym_create_multi_buffer_keyword] = ACTIONS(1965),
    [sym_call_keyword] = ACTIONS(1965),
    [sym_system_call_keyword] = ACTIONS(1965),
    [sym_devchan_to_keyword] = ACTIONS(1965),
    [sym_devchan_min_to_keyword] = ACTIONS(1965),
    [sym_devchan_total_off_keyword] = ACTIONS(1965),
    [sym_devchan_pulse_keyword] = ACTIONS(1965),
    [sym_wait_keyword] = ACTIONS(1965),
    [sym_cancel_wait_keyword] = ACTIONS(1965),
    [sym_wait_until_keyword] = ACTIONS(1965),
    [sym_cancel_wait_until_keyword] = ACTIONS(1965),
    [sym_cancel_all_wait_keyword] = ACTIONS(1965),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1965),
    [sym_not] = ACTIONS(1965),
    [sym_bnot] = ACTIONS(1965),
    [sym___file__] = ACTIONS(1965),
    [sym___date__] = ACTIONS(1965),
    [sym___ldate__] = ACTIONS(1965),
    [sym___time__] = ACTIONS(1965),
    [sym___line__] = ACTIONS(1965),
    [sym___name__] = ACTIONS(1965),
    [sym___netlinx__] = ACTIONS(1965),
    [sym_netlinx_axi_version] = ACTIONS(1965),
    [sym_url_flg_tcp] = ACTIONS(1965),
    [sym_url_flg_accinfopresent] = ACTIONS(1965),
    [sym_url_flg_temp] = ACTIONS(1965),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1965),
    [sym_url_flg_stat_mask] = ACTIONS(1965),
    [sym_url_flg_stat_lookup] = ACTIONS(1965),
    [sym_url_flg_stat_connecting] = ACTIONS(1965),
    [sym_url_flg_stat_waiting] = ACTIONS(1965),
    [sym_url_flg_stat_connected] = ACTIONS(1965),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1965),
    [sym_first_virtual_device] = ACTIONS(1965),
    [sym_first_local_port] = ACTIONS(1965),
    [sym_dynamic_virtual_device] = ACTIONS(1965),
    [sym_timeline_once] = ACTIONS(1965),
    [sym_timeline_repeat] = ACTIONS(1965),
    [sym_timeline_absolute] = ACTIONS(1965),
    [sym_timeline_relative] = ACTIONS(1965),
    [sym_do_push_timed_infinite] = ACTIONS(1965),
    [sym_source_type_no_address] = ACTIONS(1965),
    [sym_source_type_neuron_id] = ACTIONS(1965),
    [sym_source_type_ip_address] = ACTIONS(1965),
    [sym_source_type_axlink] = ACTIONS(1965),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1965),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1965),
    [sym_source_type_ip_socket_address] = ACTIONS(1965),
    [sym_source_type_rs232] = ACTIONS(1965),
    [sym_source_type_internal] = ACTIONS(1965),
    [sym_source_type_mac_address] = ACTIONS(1965),
    [sym_source_type_ipv4_port] = ACTIONS(1965),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1965),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1965),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1965),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1965),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1965),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1965),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1965),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1965),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1965),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1965),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1965),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1965),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1965),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1965),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1965),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1965),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1965),
    [sym_true] = ACTIONS(1965),
    [sym_false] = ACTIONS(1965),
    [sym_file_read_only] = ACTIONS(1965),
    [sym_file_rw_new] = ACTIONS(1965),
    [sym_file_rw_append] = ACTIONS(1965),
    [sym_ip_tcp] = ACTIONS(1965),
    [sym_ip_udp] = ACTIONS(1965),
    [sym_ip_udp_2way] = ACTIONS(1965),
    [sym_xml_encode_types] = ACTIONS(1965),
    [sym_xml_encode_char_as_list] = ACTIONS(1965),
    [sym_xml_encode_le] = ACTIONS(1965),
    [sym_xml_decode_types] = ACTIONS(1965),
    [sym_xml_decode_no_preserve] = ACTIONS(1965),
    [sym_duet_dev_type_display_device] = ACTIONS(1965),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1965),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1965),
    [sym_duet_dev_type_io_device] = ACTIONS(1965),
    [sym_duet_dev_type_relay_device] = ACTIONS(1965),
    [sym_duet_dev_type_ups] = ACTIONS(1965),
    [sym_duet_dev_type_amplifier] = ACTIONS(1965),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1965),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1965),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1965),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1965),
    [sym_duet_dev_type_camera] = ACTIONS(1965),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1965),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1965),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1965),
    [sym_duet_dev_type_dss] = ACTIONS(1965),
    [sym_duet_dev_type_dvr] = ACTIONS(1965),
    [sym_duet_dev_type_disc_device] = ACTIONS(1965),
    [sym_duet_dev_type_document_camera] = ACTIONS(1965),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1965),
    [sym_duet_dev_type_hvac] = ACTIONS(1965),
    [sym_duet_dev_type_keypad] = ACTIONS(1965),
    [sym_duet_dev_type_light] = ACTIONS(1965),
    [sym_duet_dev_type_monitor] = ACTIONS(1965),
    [sym_duet_dev_type_motor] = ACTIONS(1965),
    [sym_duet_dev_type_multi_window] = ACTIONS(1965),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1965),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1965),
    [sym_duet_dev_type_receiver] = ACTIONS(1965),
    [sym_duet_dev_type_security_system] = ACTIONS(1965),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1965),
    [sym_duet_dev_type_settop_box] = ACTIONS(1965),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1965),
    [sym_duet_dev_type_switcher] = ACTIONS(1965),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1965),
    [sym_duet_dev_type_tv] = ACTIONS(1965),
    [sym_duet_dev_type_utility] = ACTIONS(1965),
    [sym_duet_dev_type_vcr] = ACTIONS(1965),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1965),
    [sym_duet_dev_type_video_processor] = ACTIONS(1965),
    [sym_duet_dev_type_video_projector] = ACTIONS(1965),
    [sym_duet_dev_type_video_wall] = ACTIONS(1965),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1965),
    [sym_duet_dev_type_weather] = ACTIONS(1965),
    [sym_duet_dev_not_polled] = ACTIONS(1965),
    [sym_duet_dev_polled] = ACTIONS(1965),
    [sym_clkmgr_mode_network] = ACTIONS(1965),
    [sym_clkmgr_mode_standalone] = ACTIONS(1965),
    [sym_normal_standby] = ACTIONS(1965),
    [sym_normal_wake] = ACTIONS(1965),
    [sym_ip_multicast_ttl_option] = ACTIONS(1965),
    [sym_ip_ttl_subnet] = ACTIONS(1965),
    [sym_ip_ttl_site] = ACTIONS(1965),
    [sym_ip_ttl_region] = ACTIONS(1965),
    [sym_ip_ttl_continent] = ACTIONS(1965),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1965),
    [sym_ip_nodelay_on] = ACTIONS(1965),
    [sym_ip_nodelay_off] = ACTIONS(1965),
    [sym_amx_error] = ACTIONS(1965),
    [sym_amx_warning] = ACTIONS(1965),
    [sym_amx_info] = ACTIONS(1965),
    [sym_amx_debug] = ACTIONS(1965),
    [sym_netlinx_login_success] = ACTIONS(1965),
    [sym_netlinx_login_fail] = ACTIONS(1965),
    [sym_netlinx_logout] = ACTIONS(1965),
    [sym_null_str] = ACTIONS(1965),
    [sym_smtp_address] = ACTIONS(1965),
    [sym_smtp_port_number] = ACTIONS(1965),
    [sym_smtp_username] = ACTIONS(1965),
    [sym_smtp_password] = ACTIONS(1965),
    [sym_smtp_require_tls] = ACTIONS(1965),
    [sym_smtp_from] = ACTIONS(1965),
    [sym_smtp_tls_true] = ACTIONS(1965),
    [sym_smtp_tls_false] = ACTIONS(1965),
    [sym_listview_on_row_select_event] = ACTIONS(1965),
    [sym_data_structure_unknown] = ACTIONS(1965),
    [sym_data_structure_datafeed] = ACTIONS(1965),
    [sym_data_structure_datarecordset] = ACTIONS(1965),
    [sym_data_structure_datarecord] = ACTIONS(1965),
    [sym_data_structure_datafield] = ACTIONS(1965),
    [sym_data_max_content_field_count] = ACTIONS(1965),
    [sym_data_max_metadata_field_count] = ACTIONS(1965),
    [sym_data_max_id_length] = ACTIONS(1965),
    [sym_data_max_name_length] = ACTIONS(1965),
    [sym_data_max_description_length] = ACTIONS(1965),
    [sym_data_max_type_length] = ACTIONS(1965),
    [sym_data_max_format_length] = ACTIONS(1965),
    [sym_data_max_source_length] = ACTIONS(1965),
    [sym_data_max_value_length] = ACTIONS(1965),
    [sym_data_max_label_length] = ACTIONS(1965),
    [sym_data_max_url_length] = ACTIONS(1965),
    [sym_max_last_login_info_length] = ACTIONS(1965),
    [sym_data_type_unknown] = ACTIONS(1965),
    [sym_data_type_string] = ACTIONS(1965),
    [sym_data_type_datetime] = ACTIONS(1965),
    [sym_data_type_date] = ACTIONS(1965),
    [sym_data_type_time] = ACTIONS(1965),
    [sym_data_type_image] = ACTIONS(1965),
    [sym_data_format_url] = ACTIONS(1965),
    [sym_data_format_phone] = ACTIONS(1965),
    [sym_data_format_email] = ACTIONS(1965),
    [sym_data_format_iso8601] = ACTIONS(1965),
    [sym_auditlog_priv] = ACTIONS(1965),
    [sym_configuration_priv] = ACTIONS(1965),
    [sym_deviceconfig_priv] = ACTIONS(1965),
    [sym_ftp_priv] = ACTIONS(1965),
    [sym_http_priv] = ACTIONS(1965),
    [sym_networkconfig_priv] = ACTIONS(1965),
    [sym_programport_priv] = ACTIONS(1965),
    [sym_remoteui_priv] = ACTIONS(1965),
    [sym_securitycontrol_priv] = ACTIONS(1965),
    [sym_softwaremanagement_priv] = ACTIONS(1965),
    [sym_terminal_priv] = ACTIONS(1965),
    [sym_tpadmin_priv] = ACTIONS(1965),
    [sym_user1_priv] = ACTIONS(1965),
    [sym_user2_priv] = ACTIONS(1965),
    [sym_user3_priv] = ACTIONS(1965),
    [sym_user4_priv] = ACTIONS(1965),
    [sym_usermanagement_priv] = ACTIONS(1965),
    [sym_valid_account] = ACTIONS(1965),
    [sym_err_name_invalid] = ACTIONS(1965),
    [sym_err_password_invalid] = ACTIONS(1965),
    [sym_err_invalid_account] = ACTIONS(1965),
    [sym_err_no_authorization] = ACTIONS(1965),
    [sym_err_invalid_parameter] = ACTIONS(1965),
    [sym_err_user_locked_out] = ACTIONS(1965),
    [sym_err_user_account_expired] = ACTIONS(1965),
    [sym_err_general_error] = ACTIONS(1965),
    [sym_err_authentication_error] = ACTIONS(1965),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1965),
    [sym_tls_validate_certificate] = ACTIONS(1965),
    [sym_tdata] = ACTIONS(1965),
    [sym_tchannel] = ACTIONS(1965),
    [sym_tlevel] = ACTIONS(1965),
    [sym_tbutton] = ACTIONS(1965),
    [sym_ttimeline] = ACTIONS(1965),
    [sym_tcustom] = ACTIONS(1965),
    [sym_url_struct] = ACTIONS(1965),
    [sym_dns_struct] = ACTIONS(1965),
    [sym_ip_address_struct] = ACTIONS(1965),
    [sym_dev_info_struct] = ACTIONS(1965),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1965),
    [sym_data_feed] = ACTIONS(1965),
    [sym_data_field] = ACTIONS(1965),
    [sym_data_record] = ACTIONS(1965),
    [sym_wc_data_feed] = ACTIONS(1965),
    [sym_wc_data_field] = ACTIONS(1965),
    [sym_wc_data_record] = ACTIONS(1965),
    [sym_last_login_info] = ACTIONS(1965),
    [sym____reserved___] = ACTIONS(1965),
    [sym_date] = ACTIONS(1965),
    [sym_day] = ACTIONS(1965),
    [sym_time] = ACTIONS(1965),
    [sym_ldate] = ACTIONS(1965),
    [sym_push_device] = ACTIONS(1965),
    [sym_push_channel] = ACTIONS(1965),
    [sym_push_devchan] = ACTIONS(1965),
    [sym_release_device] = ACTIONS(1965),
    [sym_release_channel] = ACTIONS(1965),
    [sym_release_devchan] = ACTIONS(1965),
    [sym_master_slot] = ACTIONS(1965),
    [sym_get_pulse_time] = ACTIONS(1965),
    [sym_get_timer] = ACTIONS(1965),
    [sym_master_sn] = ACTIONS(1965),
    [sym_system_number] = ACTIONS(1965),
    [sym_button] = ACTIONS(1965),
    [sym_dv_channel] = ACTIONS(1965),
    [sym_data] = ACTIONS(1965),
    [sym_level] = ACTIONS(1965),
    [sym_timeline] = ACTIONS(1965),
    [sym_channel] = ACTIONS(1965),
    [sym_custom] = ACTIONS(1965),
    [sym_atoi] = ACTIONS(1965),
    [sym_atol] = ACTIONS(1965),
    [sym_atof] = ACTIONS(1965),
    [sym_itoa] = ACTIONS(1965),
    [sym_itohex] = ACTIONS(1965),
    [sym_hextoi] = ACTIONS(1965),
    [sym_ftoa] = ACTIONS(1965),
    [sym_find_string] = ACTIONS(1965),
    [sym_left_string] = ACTIONS(1965),
    [sym_right_string] = ACTIONS(1965),
    [sym_mid_string] = ACTIONS(1965),
    [sym_remove_string] = ACTIONS(1965),
    [sym_get_buffer_string] = ACTIONS(1965),
    [sym_get_buffer_char] = ACTIONS(1965),
    [sym_get_multi_buffer_string] = ACTIONS(1965),
    [sym_length_string] = ACTIONS(1965),
    [sym_max_length_string] = ACTIONS(1965),
    [sym_set_length_string] = ACTIONS(1965),
    [sym_compare_string] = ACTIONS(1965),
    [sym_upper_string] = ACTIONS(1965),
    [sym_lower_string] = ACTIONS(1965),
    [sym_get_last] = ACTIONS(1965),
    [sym_date_to_day] = ACTIONS(1965),
    [sym_date_to_month] = ACTIONS(1965),
    [sym_date_to_year] = ACTIONS(1965),
    [sym_time_to_hour] = ACTIONS(1965),
    [sym_time_to_minute] = ACTIONS(1965),
    [sym_time_to_second] = ACTIONS(1965),
    [sym_day_of_week] = ACTIONS(1965),
    [sym_set_pulse_time] = ACTIONS(1965),
    [sym_set_timer] = ACTIONS(1965),
    [sym_file_open] = ACTIONS(1965),
    [sym_file_close] = ACTIONS(1965),
    [sym_file_read] = ACTIONS(1965),
    [sym_file_read_line] = ACTIONS(1965),
    [sym_file_write] = ACTIONS(1965),
    [sym_file_write_line] = ACTIONS(1965),
    [sym_file_copy] = ACTIONS(1965),
    [sym_file_delete] = ACTIONS(1965),
    [sym_file_rename] = ACTIONS(1965),
    [sym_file_dir] = ACTIONS(1965),
    [sym_file_getdir] = ACTIONS(1965),
    [sym_file_setdir] = ACTIONS(1965),
    [sym_file_create_dir] = ACTIONS(1965),
    [sym_file_remove_dir] = ACTIONS(1965),
    [sym_file_seek] = ACTIONS(1965),
    [sym_length_array] = ACTIONS(1965),
    [sym_max_length_array] = ACTIONS(1965),
    [sym_set_length_array] = ACTIONS(1965),
    [sym_ip_client_open] = ACTIONS(1965),
    [sym_ip_client_close] = ACTIONS(1965),
    [sym_ip_server_open] = ACTIONS(1965),
    [sym_ip_server_close] = ACTIONS(1965),
    [sym_ip_bound_client_open] = ACTIONS(1965),
    [sym_ip_mc_server_open] = ACTIONS(1965),
    [sym_ip_set_option] = ACTIONS(1965),
    [sym_ssh_client_open] = ACTIONS(1965),
    [sym_ssh_client_close] = ACTIONS(1965),
    [sym_tls_client_open] = ACTIONS(1965),
    [sym_tls_client_close] = ACTIONS(1965),
    [sym_do_push] = ACTIONS(1965),
    [sym_do_push_timed] = ACTIONS(1965),
    [sym_do_release] = ACTIONS(1965),
    [sym_device_id] = ACTIONS(1965),
    [sym_device_info] = ACTIONS(1965),
    [sym_device_id_string] = ACTIONS(1965),
    [sym_device_standby] = ACTIONS(1965),
    [sym_device_wake] = ACTIONS(1965),
    [sym_timeline_create] = ACTIONS(1965),
    [sym_timeline_kill] = ACTIONS(1965),
    [sym_timeline_reload] = ACTIONS(1965),
    [sym_timeline_pause] = ACTIONS(1965),
    [sym_timeline_restart] = ACTIONS(1965),
    [sym_timeline_set] = ACTIONS(1965),
    [sym_timeline_get] = ACTIONS(1965),
    [sym_timeline_active] = ACTIONS(1965),
    [sym_timeline_dynamic_id] = ACTIONS(1965),
    [sym_variable_to_string] = ACTIONS(1965),
    [sym_string_to_variable] = ACTIONS(1965),
    [sym_length_variable_to_string] = ACTIONS(1965),
    [sym_type_cast] = ACTIONS(1965),
    [sym_variable_to_xml] = ACTIONS(1965),
    [sym_xml_to_variable] = ACTIONS(1965),
    [sym_length_variable_to_xml] = ACTIONS(1965),
    [sym_redirect_string] = ACTIONS(1965),
    [sym_random_number] = ACTIONS(1965),
    [sym_get_unique_id] = ACTIONS(1965),
    [sym_get_system_number] = ACTIONS(1965),
    [sym_set_system_number] = ACTIONS(1965),
    [sym_get_serial_number] = ACTIONS(1965),
    [sym_get_master_build] = ACTIONS(1965),
    [sym_set_validation_code] = ACTIONS(1965),
    [sym_reboot] = ACTIONS(1965),
    [sym_load_duet_module] = ACTIONS(1965),
    [sym_unload_duet_module] = ACTIONS(1965),
    [sym_duet_mem_size_set] = ACTIONS(1965),
    [sym_duet_mem_size_get] = ACTIONS(1965),
    [sym_dynamic_application_device] = ACTIONS(1965),
    [sym_dynamic_polled_port] = ACTIONS(1965),
    [sym_static_port_binding] = ACTIONS(1965),
    [sym_static_ip_binding] = ACTIONS(1965),
    [sym_get_dns_list] = ACTIONS(1965),
    [sym_set_dns_list] = ACTIONS(1965),
    [sym_get_ip_address] = ACTIONS(1965),
    [sym_set_ip_address] = ACTIONS(1965),
    [sym_get_url_list] = ACTIONS(1965),
    [sym_add_url_entry] = ACTIONS(1965),
    [sym_delete_url_entry] = ACTIONS(1965),
    [sym_abs_value] = ACTIONS(1965),
    [sym_max_value] = ACTIONS(1965),
    [sym_min_value] = ACTIONS(1965),
    [sym_exp_value] = ACTIONS(1965),
    [sym_log_value] = ACTIONS(1965),
    [sym_log10_value] = ACTIONS(1965),
    [sym_power_value] = ACTIONS(1965),
    [sym_sqrt_value] = ACTIONS(1965),
    [sym_raw_be] = ACTIONS(1965),
    [sym_raw_le] = ACTIONS(1965),
    [sym_internal_queue_size_set] = ACTIONS(1965),
    [sym_internal_queue_size_get] = ACTIONS(1965),
    [sym_internal_threshold_set] = ACTIONS(1965),
    [sym_internal_threshold_get] = ACTIONS(1965),
    [sym_rebuilt_event] = ACTIONS(1965),
    [sym_do_custom_event] = ACTIONS(1965),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1965),
    [sym_clkmgr_set_clk_source] = ACTIONS(1965),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1965),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1965),
    [sym_clkmgr_get_timezone] = ACTIONS(1965),
    [sym_clkmgr_set_timezone] = ACTIONS(1965),
    [sym_clkmgr_get_resync_period] = ACTIONS(1965),
    [sym_clkmgr_set_resync_period] = ACTIONS(1965),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1965),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1965),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1965),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1965),
    [sym_clkmgr_get_timeservers] = ACTIONS(1965),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1965),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1965),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1965),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1965),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1965),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1965),
    [sym_set_log_level] = ACTIONS(1965),
    [sym_get_log_level] = ACTIONS(1965),
    [sym_amx_log] = ACTIONS(1965),
    [sym_smtp_server_config_set] = ACTIONS(1965),
    [sym_smtp_server_config_get] = ACTIONS(1965),
    [sym_smtp_send] = ACTIONS(1965),
    [sym_validate_netlinx_account] = ACTIONS(1965),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1965),
    [sym_authenticate_certificate] = ACTIONS(1965),
    [sym_audit_netlinx_session_event] = ACTIONS(1965),
    [sym_audit_netlinx_generic_event] = ACTIONS(1965),
    [sym_get_available_flash_disk_space] = ACTIONS(1965),
    [sym_get_max_flash_disk_space] = ACTIONS(1965),
    [sym_data_create_feed] = ACTIONS(1965),
    [sym_data_delete_feed] = ACTIONS(1965),
    [sym_data_publish_feed] = ACTIONS(1965),
    [sym_data_get_published_feed] = ACTIONS(1965),
    [sym_data_add_record] = ACTIONS(1965),
    [sym_data_get_event_record] = ACTIONS(1965),
    [sym__wc_data_create_feed] = ACTIONS(1965),
    [sym__wc_data_add_record] = ACTIONS(1965),
    [sym__wc_data_get_event_record] = ACTIONS(1965),
    [sym_format] = ACTIONS(1965),
    [sym_astro_clock] = ACTIONS(1965),
    [sym_set_outdoor_temperature] = ACTIONS(1965),
    [sym_set_virtual_level_count] = ACTIONS(1965),
    [sym_set_virtual_channel_count] = ACTIONS(1965),
    [sym_set_virtual_port_count] = ACTIONS(1965),
    [sym__wc_encode_int] = ACTIONS(1965),
    [sym__wc_decode_int] = ACTIONS(1965),
    [sym__wc_int] = ACTIONS(1965),
    [sym__wc_to_ch_int] = ACTIONS(1965),
    [sym__ch_to_wc_int] = ACTIONS(1965),
    [sym__wc_find_string_int] = ACTIONS(1965),
    [sym__wc_left_string_int] = ACTIONS(1965),
    [sym__wc_lower_string_int] = ACTIONS(1965),
    [sym__wc_mid_string_int] = ACTIONS(1965),
    [sym__wc_remove_string_int] = ACTIONS(1965),
    [sym__wc_right_string_int] = ACTIONS(1965),
    [sym__wc_upper_string_int] = ACTIONS(1965),
    [sym__wc_compare_string_int] = ACTIONS(1965),
    [sym__wc_get_buffer_char_int] = ACTIONS(1965),
    [sym__wc_get_buffer_string_int] = ACTIONS(1965),
    [sym__wc_concat_string_int] = ACTIONS(1965),
    [sym__wc_file_open_int] = ACTIONS(1965),
    [sym__wc_file_close_int] = ACTIONS(1965),
    [sym__wc_file_read_int] = ACTIONS(1965),
    [sym__wc_file_read_line_int] = ACTIONS(1965),
    [sym__wc_file_write_int] = ACTIONS(1965),
    [sym__wc_file_write_line_int] = ACTIONS(1965),
    [aux_sym_preproc_include_token1] = ACTIONS(1965),
    [aux_sym_preproc_define_token1] = ACTIONS(1967),
    [aux_sym_preproc_warn_token1] = ACTIONS(1967),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1967),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1967),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1967),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1967),
    [aux_sym_preproc_else_token1] = ACTIONS(1967),
    [anon_sym_LBRACK] = ACTIONS(1967),
    [anon_sym_LBRACE] = ACTIONS(1967),
    [anon_sym_LPAREN] = ACTIONS(1965),
    [anon_sym_BANG] = ACTIONS(1967),
    [anon_sym_TILDE] = ACTIONS(1967),
    [anon_sym_DASH] = ACTIONS(1965),
    [anon_sym_PLUS] = ACTIONS(1965),
    [anon_sym_DASH_DASH] = ACTIONS(1967),
    [anon_sym_PLUS_PLUS] = ACTIONS(1967),
    [anon_sym_DQUOTE] = ACTIONS(1967),
    [anon_sym_SQUOTE] = ACTIONS(1967),
    [sym_number_literal] = ACTIONS(1967),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(297)] = {
    [sym_identifier] = ACTIONS(1969),
    [sym_program_name_keyword] = ACTIONS(1969),
    [sym_module_name_keyword] = ACTIONS(1969),
    [sym_define_device_keyword] = ACTIONS(1969),
    [sym_define_combine_keyword] = ACTIONS(1969),
    [sym_define_constant_keyword] = ACTIONS(1969),
    [sym_define_type_keyword] = ACTIONS(1969),
    [sym_define_variable_keyword] = ACTIONS(1969),
    [sym_define_system_variable_keyword] = ACTIONS(1969),
    [sym_define_connect_level_keyword] = ACTIONS(1969),
    [sym_define_latching_keyword] = ACTIONS(1969),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1969),
    [sym_define_toggling_keyword] = ACTIONS(1969),
    [sym_define_call_keyword] = ACTIONS(1969),
    [sym_define_function_keyword] = ACTIONS(1969),
    [sym_define_library_function_keyword] = ACTIONS(1969),
    [sym_define_start_keyword] = ACTIONS(1969),
    [sym_define_event_keyword] = ACTIONS(1969),
    [sym_define_program_keyword] = ACTIONS(1969),
    [sym_define_module_keyword] = ACTIONS(1969),
    [sym_char_keyword] = ACTIONS(1969),
    [sym_widechar_keyword] = ACTIONS(1969),
    [sym_integer_keyword] = ACTIONS(1969),
    [sym_sinteger_keyword] = ACTIONS(1969),
    [sym_long_keyword] = ACTIONS(1969),
    [sym_slong_keyword] = ACTIONS(1969),
    [sym_float_keyword] = ACTIONS(1969),
    [sym_double_keyword] = ACTIONS(1969),
    [sym_variant_keyword] = ACTIONS(1969),
    [sym_variantarray_keyword] = ACTIONS(1969),
    [sym_local_var_keyword] = ACTIONS(1969),
    [sym_stack_var_keyword] = ACTIONS(1969),
    [sym_constant_keyword] = ACTIONS(1969),
    [sym_volatile_keyword] = ACTIONS(1969),
    [sym_non_volatile_keyword] = ACTIONS(1969),
    [sym_persistent_keyword] = ACTIONS(1969),
    [sym_dev_keyword] = ACTIONS(1969),
    [sym_devlev_keyword] = ACTIONS(1969),
    [sym_devchan_keyword] = ACTIONS(1969),
    [sym_if_keyword] = ACTIONS(1969),
    [sym_switch_keyword] = ACTIONS(1969),
    [sym_case_keyword] = ACTIONS(1969),
    [sym_default_keyword] = ACTIONS(1969),
    [sym_while_keyword] = ACTIONS(1969),
    [sym_for_keyword] = ACTIONS(1969),
    [sym_break_keyword] = ACTIONS(1969),
    [sym_continue_keyword] = ACTIONS(1969),
    [sym_return_keyword] = ACTIONS(1969),
    [sym_select_keyword] = ACTIONS(1969),
    [sym_active_keyword] = ACTIONS(1969),
    [sym_struct_keyword] = ACTIONS(1969),
    [sym_structure_keyword] = ACTIONS(1969),
    [sym_button_event_keyword] = ACTIONS(1969),
    [sym_channel_event_keyword] = ACTIONS(1969),
    [sym_level_event_keyword] = ACTIONS(1969),
    [sym_data_event_keyword] = ACTIONS(1969),
    [sym_timeline_event_keyword] = ACTIONS(1969),
    [sym_custom_event_keyword] = ACTIONS(1969),
    [sym_push_keyword] = ACTIONS(1969),
    [sym_release_keyword] = ACTIONS(1969),
    [aux_sym_on_keyword_token1] = ACTIONS(1969),
    [aux_sym_off_keyword_token1] = ACTIONS(1969),
    [sym_send_level_keyword] = ACTIONS(1969),
    [sym_send_string_keyword] = ACTIONS(1969),
    [sym_send_command_keyword] = ACTIONS(1969),
    [sym_clear_buffer_keyword] = ACTIONS(1969),
    [sym_create_buffer_keyword] = ACTIONS(1969),
    [sym_create_multi_buffer_keyword] = ACTIONS(1969),
    [sym_call_keyword] = ACTIONS(1969),
    [sym_system_call_keyword] = ACTIONS(1969),
    [sym_devchan_to_keyword] = ACTIONS(1969),
    [sym_devchan_min_to_keyword] = ACTIONS(1969),
    [sym_devchan_total_off_keyword] = ACTIONS(1969),
    [sym_devchan_pulse_keyword] = ACTIONS(1969),
    [sym_wait_keyword] = ACTIONS(1969),
    [sym_cancel_wait_keyword] = ACTIONS(1969),
    [sym_wait_until_keyword] = ACTIONS(1969),
    [sym_cancel_wait_until_keyword] = ACTIONS(1969),
    [sym_cancel_all_wait_keyword] = ACTIONS(1969),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1969),
    [sym_not] = ACTIONS(1969),
    [sym_bnot] = ACTIONS(1969),
    [sym___file__] = ACTIONS(1969),
    [sym___date__] = ACTIONS(1969),
    [sym___ldate__] = ACTIONS(1969),
    [sym___time__] = ACTIONS(1969),
    [sym___line__] = ACTIONS(1969),
    [sym___name__] = ACTIONS(1969),
    [sym___netlinx__] = ACTIONS(1969),
    [sym_netlinx_axi_version] = ACTIONS(1969),
    [sym_url_flg_tcp] = ACTIONS(1969),
    [sym_url_flg_accinfopresent] = ACTIONS(1969),
    [sym_url_flg_temp] = ACTIONS(1969),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1969),
    [sym_url_flg_stat_mask] = ACTIONS(1969),
    [sym_url_flg_stat_lookup] = ACTIONS(1969),
    [sym_url_flg_stat_connecting] = ACTIONS(1969),
    [sym_url_flg_stat_waiting] = ACTIONS(1969),
    [sym_url_flg_stat_connected] = ACTIONS(1969),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1969),
    [sym_first_virtual_device] = ACTIONS(1969),
    [sym_first_local_port] = ACTIONS(1969),
    [sym_dynamic_virtual_device] = ACTIONS(1969),
    [sym_timeline_once] = ACTIONS(1969),
    [sym_timeline_repeat] = ACTIONS(1969),
    [sym_timeline_absolute] = ACTIONS(1969),
    [sym_timeline_relative] = ACTIONS(1969),
    [sym_do_push_timed_infinite] = ACTIONS(1969),
    [sym_source_type_no_address] = ACTIONS(1969),
    [sym_source_type_neuron_id] = ACTIONS(1969),
    [sym_source_type_ip_address] = ACTIONS(1969),
    [sym_source_type_axlink] = ACTIONS(1969),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1969),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1969),
    [sym_source_type_ip_socket_address] = ACTIONS(1969),
    [sym_source_type_rs232] = ACTIONS(1969),
    [sym_source_type_internal] = ACTIONS(1969),
    [sym_source_type_mac_address] = ACTIONS(1969),
    [sym_source_type_ipv4_port] = ACTIONS(1969),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1969),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1969),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1969),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1969),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1969),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1969),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1969),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1969),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1969),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1969),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1969),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1969),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1969),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1969),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1969),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1969),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1969),
    [sym_true] = ACTIONS(1969),
    [sym_false] = ACTIONS(1969),
    [sym_file_read_only] = ACTIONS(1969),
    [sym_file_rw_new] = ACTIONS(1969),
    [sym_file_rw_append] = ACTIONS(1969),
    [sym_ip_tcp] = ACTIONS(1969),
    [sym_ip_udp] = ACTIONS(1969),
    [sym_ip_udp_2way] = ACTIONS(1969),
    [sym_xml_encode_types] = ACTIONS(1969),
    [sym_xml_encode_char_as_list] = ACTIONS(1969),
    [sym_xml_encode_le] = ACTIONS(1969),
    [sym_xml_decode_types] = ACTIONS(1969),
    [sym_xml_decode_no_preserve] = ACTIONS(1969),
    [sym_duet_dev_type_display_device] = ACTIONS(1969),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1969),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1969),
    [sym_duet_dev_type_io_device] = ACTIONS(1969),
    [sym_duet_dev_type_relay_device] = ACTIONS(1969),
    [sym_duet_dev_type_ups] = ACTIONS(1969),
    [sym_duet_dev_type_amplifier] = ACTIONS(1969),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1969),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1969),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1969),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1969),
    [sym_duet_dev_type_camera] = ACTIONS(1969),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1969),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1969),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1969),
    [sym_duet_dev_type_dss] = ACTIONS(1969),
    [sym_duet_dev_type_dvr] = ACTIONS(1969),
    [sym_duet_dev_type_disc_device] = ACTIONS(1969),
    [sym_duet_dev_type_document_camera] = ACTIONS(1969),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1969),
    [sym_duet_dev_type_hvac] = ACTIONS(1969),
    [sym_duet_dev_type_keypad] = ACTIONS(1969),
    [sym_duet_dev_type_light] = ACTIONS(1969),
    [sym_duet_dev_type_monitor] = ACTIONS(1969),
    [sym_duet_dev_type_motor] = ACTIONS(1969),
    [sym_duet_dev_type_multi_window] = ACTIONS(1969),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1969),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1969),
    [sym_duet_dev_type_receiver] = ACTIONS(1969),
    [sym_duet_dev_type_security_system] = ACTIONS(1969),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1969),
    [sym_duet_dev_type_settop_box] = ACTIONS(1969),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1969),
    [sym_duet_dev_type_switcher] = ACTIONS(1969),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1969),
    [sym_duet_dev_type_tv] = ACTIONS(1969),
    [sym_duet_dev_type_utility] = ACTIONS(1969),
    [sym_duet_dev_type_vcr] = ACTIONS(1969),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1969),
    [sym_duet_dev_type_video_processor] = ACTIONS(1969),
    [sym_duet_dev_type_video_projector] = ACTIONS(1969),
    [sym_duet_dev_type_video_wall] = ACTIONS(1969),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1969),
    [sym_duet_dev_type_weather] = ACTIONS(1969),
    [sym_duet_dev_not_polled] = ACTIONS(1969),
    [sym_duet_dev_polled] = ACTIONS(1969),
    [sym_clkmgr_mode_network] = ACTIONS(1969),
    [sym_clkmgr_mode_standalone] = ACTIONS(1969),
    [sym_normal_standby] = ACTIONS(1969),
    [sym_normal_wake] = ACTIONS(1969),
    [sym_ip_multicast_ttl_option] = ACTIONS(1969),
    [sym_ip_ttl_subnet] = ACTIONS(1969),
    [sym_ip_ttl_site] = ACTIONS(1969),
    [sym_ip_ttl_region] = ACTIONS(1969),
    [sym_ip_ttl_continent] = ACTIONS(1969),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1969),
    [sym_ip_nodelay_on] = ACTIONS(1969),
    [sym_ip_nodelay_off] = ACTIONS(1969),
    [sym_amx_error] = ACTIONS(1969),
    [sym_amx_warning] = ACTIONS(1969),
    [sym_amx_info] = ACTIONS(1969),
    [sym_amx_debug] = ACTIONS(1969),
    [sym_netlinx_login_success] = ACTIONS(1969),
    [sym_netlinx_login_fail] = ACTIONS(1969),
    [sym_netlinx_logout] = ACTIONS(1969),
    [sym_null_str] = ACTIONS(1969),
    [sym_smtp_address] = ACTIONS(1969),
    [sym_smtp_port_number] = ACTIONS(1969),
    [sym_smtp_username] = ACTIONS(1969),
    [sym_smtp_password] = ACTIONS(1969),
    [sym_smtp_require_tls] = ACTIONS(1969),
    [sym_smtp_from] = ACTIONS(1969),
    [sym_smtp_tls_true] = ACTIONS(1969),
    [sym_smtp_tls_false] = ACTIONS(1969),
    [sym_listview_on_row_select_event] = ACTIONS(1969),
    [sym_data_structure_unknown] = ACTIONS(1969),
    [sym_data_structure_datafeed] = ACTIONS(1969),
    [sym_data_structure_datarecordset] = ACTIONS(1969),
    [sym_data_structure_datarecord] = ACTIONS(1969),
    [sym_data_structure_datafield] = ACTIONS(1969),
    [sym_data_max_content_field_count] = ACTIONS(1969),
    [sym_data_max_metadata_field_count] = ACTIONS(1969),
    [sym_data_max_id_length] = ACTIONS(1969),
    [sym_data_max_name_length] = ACTIONS(1969),
    [sym_data_max_description_length] = ACTIONS(1969),
    [sym_data_max_type_length] = ACTIONS(1969),
    [sym_data_max_format_length] = ACTIONS(1969),
    [sym_data_max_source_length] = ACTIONS(1969),
    [sym_data_max_value_length] = ACTIONS(1969),
    [sym_data_max_label_length] = ACTIONS(1969),
    [sym_data_max_url_length] = ACTIONS(1969),
    [sym_max_last_login_info_length] = ACTIONS(1969),
    [sym_data_type_unknown] = ACTIONS(1969),
    [sym_data_type_string] = ACTIONS(1969),
    [sym_data_type_datetime] = ACTIONS(1969),
    [sym_data_type_date] = ACTIONS(1969),
    [sym_data_type_time] = ACTIONS(1969),
    [sym_data_type_image] = ACTIONS(1969),
    [sym_data_format_url] = ACTIONS(1969),
    [sym_data_format_phone] = ACTIONS(1969),
    [sym_data_format_email] = ACTIONS(1969),
    [sym_data_format_iso8601] = ACTIONS(1969),
    [sym_auditlog_priv] = ACTIONS(1969),
    [sym_configuration_priv] = ACTIONS(1969),
    [sym_deviceconfig_priv] = ACTIONS(1969),
    [sym_ftp_priv] = ACTIONS(1969),
    [sym_http_priv] = ACTIONS(1969),
    [sym_networkconfig_priv] = ACTIONS(1969),
    [sym_programport_priv] = ACTIONS(1969),
    [sym_remoteui_priv] = ACTIONS(1969),
    [sym_securitycontrol_priv] = ACTIONS(1969),
    [sym_softwaremanagement_priv] = ACTIONS(1969),
    [sym_terminal_priv] = ACTIONS(1969),
    [sym_tpadmin_priv] = ACTIONS(1969),
    [sym_user1_priv] = ACTIONS(1969),
    [sym_user2_priv] = ACTIONS(1969),
    [sym_user3_priv] = ACTIONS(1969),
    [sym_user4_priv] = ACTIONS(1969),
    [sym_usermanagement_priv] = ACTIONS(1969),
    [sym_valid_account] = ACTIONS(1969),
    [sym_err_name_invalid] = ACTIONS(1969),
    [sym_err_password_invalid] = ACTIONS(1969),
    [sym_err_invalid_account] = ACTIONS(1969),
    [sym_err_no_authorization] = ACTIONS(1969),
    [sym_err_invalid_parameter] = ACTIONS(1969),
    [sym_err_user_locked_out] = ACTIONS(1969),
    [sym_err_user_account_expired] = ACTIONS(1969),
    [sym_err_general_error] = ACTIONS(1969),
    [sym_err_authentication_error] = ACTIONS(1969),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1969),
    [sym_tls_validate_certificate] = ACTIONS(1969),
    [sym_tdata] = ACTIONS(1969),
    [sym_tchannel] = ACTIONS(1969),
    [sym_tlevel] = ACTIONS(1969),
    [sym_tbutton] = ACTIONS(1969),
    [sym_ttimeline] = ACTIONS(1969),
    [sym_tcustom] = ACTIONS(1969),
    [sym_url_struct] = ACTIONS(1969),
    [sym_dns_struct] = ACTIONS(1969),
    [sym_ip_address_struct] = ACTIONS(1969),
    [sym_dev_info_struct] = ACTIONS(1969),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1969),
    [sym_data_feed] = ACTIONS(1969),
    [sym_data_field] = ACTIONS(1969),
    [sym_data_record] = ACTIONS(1969),
    [sym_wc_data_feed] = ACTIONS(1969),
    [sym_wc_data_field] = ACTIONS(1969),
    [sym_wc_data_record] = ACTIONS(1969),
    [sym_last_login_info] = ACTIONS(1969),
    [sym____reserved___] = ACTIONS(1969),
    [sym_date] = ACTIONS(1969),
    [sym_day] = ACTIONS(1969),
    [sym_time] = ACTIONS(1969),
    [sym_ldate] = ACTIONS(1969),
    [sym_push_device] = ACTIONS(1969),
    [sym_push_channel] = ACTIONS(1969),
    [sym_push_devchan] = ACTIONS(1969),
    [sym_release_device] = ACTIONS(1969),
    [sym_release_channel] = ACTIONS(1969),
    [sym_release_devchan] = ACTIONS(1969),
    [sym_master_slot] = ACTIONS(1969),
    [sym_get_pulse_time] = ACTIONS(1969),
    [sym_get_timer] = ACTIONS(1969),
    [sym_master_sn] = ACTIONS(1969),
    [sym_system_number] = ACTIONS(1969),
    [sym_button] = ACTIONS(1969),
    [sym_dv_channel] = ACTIONS(1969),
    [sym_data] = ACTIONS(1969),
    [sym_level] = ACTIONS(1969),
    [sym_timeline] = ACTIONS(1969),
    [sym_channel] = ACTIONS(1969),
    [sym_custom] = ACTIONS(1969),
    [sym_atoi] = ACTIONS(1969),
    [sym_atol] = ACTIONS(1969),
    [sym_atof] = ACTIONS(1969),
    [sym_itoa] = ACTIONS(1969),
    [sym_itohex] = ACTIONS(1969),
    [sym_hextoi] = ACTIONS(1969),
    [sym_ftoa] = ACTIONS(1969),
    [sym_find_string] = ACTIONS(1969),
    [sym_left_string] = ACTIONS(1969),
    [sym_right_string] = ACTIONS(1969),
    [sym_mid_string] = ACTIONS(1969),
    [sym_remove_string] = ACTIONS(1969),
    [sym_get_buffer_string] = ACTIONS(1969),
    [sym_get_buffer_char] = ACTIONS(1969),
    [sym_get_multi_buffer_string] = ACTIONS(1969),
    [sym_length_string] = ACTIONS(1969),
    [sym_max_length_string] = ACTIONS(1969),
    [sym_set_length_string] = ACTIONS(1969),
    [sym_compare_string] = ACTIONS(1969),
    [sym_upper_string] = ACTIONS(1969),
    [sym_lower_string] = ACTIONS(1969),
    [sym_get_last] = ACTIONS(1969),
    [sym_date_to_day] = ACTIONS(1969),
    [sym_date_to_month] = ACTIONS(1969),
    [sym_date_to_year] = ACTIONS(1969),
    [sym_time_to_hour] = ACTIONS(1969),
    [sym_time_to_minute] = ACTIONS(1969),
    [sym_time_to_second] = ACTIONS(1969),
    [sym_day_of_week] = ACTIONS(1969),
    [sym_set_pulse_time] = ACTIONS(1969),
    [sym_set_timer] = ACTIONS(1969),
    [sym_file_open] = ACTIONS(1969),
    [sym_file_close] = ACTIONS(1969),
    [sym_file_read] = ACTIONS(1969),
    [sym_file_read_line] = ACTIONS(1969),
    [sym_file_write] = ACTIONS(1969),
    [sym_file_write_line] = ACTIONS(1969),
    [sym_file_copy] = ACTIONS(1969),
    [sym_file_delete] = ACTIONS(1969),
    [sym_file_rename] = ACTIONS(1969),
    [sym_file_dir] = ACTIONS(1969),
    [sym_file_getdir] = ACTIONS(1969),
    [sym_file_setdir] = ACTIONS(1969),
    [sym_file_create_dir] = ACTIONS(1969),
    [sym_file_remove_dir] = ACTIONS(1969),
    [sym_file_seek] = ACTIONS(1969),
    [sym_length_array] = ACTIONS(1969),
    [sym_max_length_array] = ACTIONS(1969),
    [sym_set_length_array] = ACTIONS(1969),
    [sym_ip_client_open] = ACTIONS(1969),
    [sym_ip_client_close] = ACTIONS(1969),
    [sym_ip_server_open] = ACTIONS(1969),
    [sym_ip_server_close] = ACTIONS(1969),
    [sym_ip_bound_client_open] = ACTIONS(1969),
    [sym_ip_mc_server_open] = ACTIONS(1969),
    [sym_ip_set_option] = ACTIONS(1969),
    [sym_ssh_client_open] = ACTIONS(1969),
    [sym_ssh_client_close] = ACTIONS(1969),
    [sym_tls_client_open] = ACTIONS(1969),
    [sym_tls_client_close] = ACTIONS(1969),
    [sym_do_push] = ACTIONS(1969),
    [sym_do_push_timed] = ACTIONS(1969),
    [sym_do_release] = ACTIONS(1969),
    [sym_device_id] = ACTIONS(1969),
    [sym_device_info] = ACTIONS(1969),
    [sym_device_id_string] = ACTIONS(1969),
    [sym_device_standby] = ACTIONS(1969),
    [sym_device_wake] = ACTIONS(1969),
    [sym_timeline_create] = ACTIONS(1969),
    [sym_timeline_kill] = ACTIONS(1969),
    [sym_timeline_reload] = ACTIONS(1969),
    [sym_timeline_pause] = ACTIONS(1969),
    [sym_timeline_restart] = ACTIONS(1969),
    [sym_timeline_set] = ACTIONS(1969),
    [sym_timeline_get] = ACTIONS(1969),
    [sym_timeline_active] = ACTIONS(1969),
    [sym_timeline_dynamic_id] = ACTIONS(1969),
    [sym_variable_to_string] = ACTIONS(1969),
    [sym_string_to_variable] = ACTIONS(1969),
    [sym_length_variable_to_string] = ACTIONS(1969),
    [sym_type_cast] = ACTIONS(1969),
    [sym_variable_to_xml] = ACTIONS(1969),
    [sym_xml_to_variable] = ACTIONS(1969),
    [sym_length_variable_to_xml] = ACTIONS(1969),
    [sym_redirect_string] = ACTIONS(1969),
    [sym_random_number] = ACTIONS(1969),
    [sym_get_unique_id] = ACTIONS(1969),
    [sym_get_system_number] = ACTIONS(1969),
    [sym_set_system_number] = ACTIONS(1969),
    [sym_get_serial_number] = ACTIONS(1969),
    [sym_get_master_build] = ACTIONS(1969),
    [sym_set_validation_code] = ACTIONS(1969),
    [sym_reboot] = ACTIONS(1969),
    [sym_load_duet_module] = ACTIONS(1969),
    [sym_unload_duet_module] = ACTIONS(1969),
    [sym_duet_mem_size_set] = ACTIONS(1969),
    [sym_duet_mem_size_get] = ACTIONS(1969),
    [sym_dynamic_application_device] = ACTIONS(1969),
    [sym_dynamic_polled_port] = ACTIONS(1969),
    [sym_static_port_binding] = ACTIONS(1969),
    [sym_static_ip_binding] = ACTIONS(1969),
    [sym_get_dns_list] = ACTIONS(1969),
    [sym_set_dns_list] = ACTIONS(1969),
    [sym_get_ip_address] = ACTIONS(1969),
    [sym_set_ip_address] = ACTIONS(1969),
    [sym_get_url_list] = ACTIONS(1969),
    [sym_add_url_entry] = ACTIONS(1969),
    [sym_delete_url_entry] = ACTIONS(1969),
    [sym_abs_value] = ACTIONS(1969),
    [sym_max_value] = ACTIONS(1969),
    [sym_min_value] = ACTIONS(1969),
    [sym_exp_value] = ACTIONS(1969),
    [sym_log_value] = ACTIONS(1969),
    [sym_log10_value] = ACTIONS(1969),
    [sym_power_value] = ACTIONS(1969),
    [sym_sqrt_value] = ACTIONS(1969),
    [sym_raw_be] = ACTIONS(1969),
    [sym_raw_le] = ACTIONS(1969),
    [sym_internal_queue_size_set] = ACTIONS(1969),
    [sym_internal_queue_size_get] = ACTIONS(1969),
    [sym_internal_threshold_set] = ACTIONS(1969),
    [sym_internal_threshold_get] = ACTIONS(1969),
    [sym_rebuilt_event] = ACTIONS(1969),
    [sym_do_custom_event] = ACTIONS(1969),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1969),
    [sym_clkmgr_set_clk_source] = ACTIONS(1969),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1969),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1969),
    [sym_clkmgr_get_timezone] = ACTIONS(1969),
    [sym_clkmgr_set_timezone] = ACTIONS(1969),
    [sym_clkmgr_get_resync_period] = ACTIONS(1969),
    [sym_clkmgr_set_resync_period] = ACTIONS(1969),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1969),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1969),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1969),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1969),
    [sym_clkmgr_get_timeservers] = ACTIONS(1969),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1969),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1969),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1969),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1969),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1969),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1969),
    [sym_set_log_level] = ACTIONS(1969),
    [sym_get_log_level] = ACTIONS(1969),
    [sym_amx_log] = ACTIONS(1969),
    [sym_smtp_server_config_set] = ACTIONS(1969),
    [sym_smtp_server_config_get] = ACTIONS(1969),
    [sym_smtp_send] = ACTIONS(1969),
    [sym_validate_netlinx_account] = ACTIONS(1969),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1969),
    [sym_authenticate_certificate] = ACTIONS(1969),
    [sym_audit_netlinx_session_event] = ACTIONS(1969),
    [sym_audit_netlinx_generic_event] = ACTIONS(1969),
    [sym_get_available_flash_disk_space] = ACTIONS(1969),
    [sym_get_max_flash_disk_space] = ACTIONS(1969),
    [sym_data_create_feed] = ACTIONS(1969),
    [sym_data_delete_feed] = ACTIONS(1969),
    [sym_data_publish_feed] = ACTIONS(1969),
    [sym_data_get_published_feed] = ACTIONS(1969),
    [sym_data_add_record] = ACTIONS(1969),
    [sym_data_get_event_record] = ACTIONS(1969),
    [sym__wc_data_create_feed] = ACTIONS(1969),
    [sym__wc_data_add_record] = ACTIONS(1969),
    [sym__wc_data_get_event_record] = ACTIONS(1969),
    [sym_format] = ACTIONS(1969),
    [sym_astro_clock] = ACTIONS(1969),
    [sym_set_outdoor_temperature] = ACTIONS(1969),
    [sym_set_virtual_level_count] = ACTIONS(1969),
    [sym_set_virtual_channel_count] = ACTIONS(1969),
    [sym_set_virtual_port_count] = ACTIONS(1969),
    [sym__wc_encode_int] = ACTIONS(1969),
    [sym__wc_decode_int] = ACTIONS(1969),
    [sym__wc_int] = ACTIONS(1969),
    [sym__wc_to_ch_int] = ACTIONS(1969),
    [sym__ch_to_wc_int] = ACTIONS(1969),
    [sym__wc_find_string_int] = ACTIONS(1969),
    [sym__wc_left_string_int] = ACTIONS(1969),
    [sym__wc_lower_string_int] = ACTIONS(1969),
    [sym__wc_mid_string_int] = ACTIONS(1969),
    [sym__wc_remove_string_int] = ACTIONS(1969),
    [sym__wc_right_string_int] = ACTIONS(1969),
    [sym__wc_upper_string_int] = ACTIONS(1969),
    [sym__wc_compare_string_int] = ACTIONS(1969),
    [sym__wc_get_buffer_char_int] = ACTIONS(1969),
    [sym__wc_get_buffer_string_int] = ACTIONS(1969),
    [sym__wc_concat_string_int] = ACTIONS(1969),
    [sym__wc_file_open_int] = ACTIONS(1969),
    [sym__wc_file_close_int] = ACTIONS(1969),
    [sym__wc_file_read_int] = ACTIONS(1969),
    [sym__wc_file_read_line_int] = ACTIONS(1969),
    [sym__wc_file_write_int] = ACTIONS(1969),
    [sym__wc_file_write_line_int] = ACTIONS(1969),
    [aux_sym_preproc_include_token1] = ACTIONS(1969),
    [aux_sym_preproc_define_token1] = ACTIONS(1971),
    [aux_sym_preproc_warn_token1] = ACTIONS(1971),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1971),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1971),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1971),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1971),
    [aux_sym_preproc_else_token1] = ACTIONS(1971),
    [anon_sym_LBRACK] = ACTIONS(1971),
    [anon_sym_LBRACE] = ACTIONS(1971),
    [anon_sym_LPAREN] = ACTIONS(1969),
    [anon_sym_BANG] = ACTIONS(1971),
    [anon_sym_TILDE] = ACTIONS(1971),
    [anon_sym_DASH] = ACTIONS(1969),
    [anon_sym_PLUS] = ACTIONS(1969),
    [anon_sym_DASH_DASH] = ACTIONS(1971),
    [anon_sym_PLUS_PLUS] = ACTIONS(1971),
    [anon_sym_DQUOTE] = ACTIONS(1971),
    [anon_sym_SQUOTE] = ACTIONS(1971),
    [sym_number_literal] = ACTIONS(1971),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(298)] = {
    [ts_builtin_sym_end] = ACTIONS(1837),
    [sym_identifier] = ACTIONS(1835),
    [sym_program_name_keyword] = ACTIONS(1835),
    [sym_module_name_keyword] = ACTIONS(1835),
    [sym_define_device_keyword] = ACTIONS(1835),
    [sym_define_combine_keyword] = ACTIONS(1835),
    [sym_define_constant_keyword] = ACTIONS(1835),
    [sym_define_type_keyword] = ACTIONS(1835),
    [sym_define_variable_keyword] = ACTIONS(1835),
    [sym_define_system_variable_keyword] = ACTIONS(1835),
    [sym_define_connect_level_keyword] = ACTIONS(1835),
    [sym_define_latching_keyword] = ACTIONS(1835),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1835),
    [sym_define_toggling_keyword] = ACTIONS(1835),
    [sym_define_call_keyword] = ACTIONS(1835),
    [sym_define_function_keyword] = ACTIONS(1835),
    [sym_define_library_function_keyword] = ACTIONS(1835),
    [sym_define_start_keyword] = ACTIONS(1835),
    [sym_define_event_keyword] = ACTIONS(1835),
    [sym_define_program_keyword] = ACTIONS(1835),
    [sym_define_module_keyword] = ACTIONS(1835),
    [sym_char_keyword] = ACTIONS(1835),
    [sym_widechar_keyword] = ACTIONS(1835),
    [sym_integer_keyword] = ACTIONS(1835),
    [sym_sinteger_keyword] = ACTIONS(1835),
    [sym_long_keyword] = ACTIONS(1835),
    [sym_slong_keyword] = ACTIONS(1835),
    [sym_float_keyword] = ACTIONS(1835),
    [sym_double_keyword] = ACTIONS(1835),
    [sym_variant_keyword] = ACTIONS(1835),
    [sym_variantarray_keyword] = ACTIONS(1835),
    [sym_local_var_keyword] = ACTIONS(1835),
    [sym_stack_var_keyword] = ACTIONS(1835),
    [sym_constant_keyword] = ACTIONS(1835),
    [sym_volatile_keyword] = ACTIONS(1835),
    [sym_non_volatile_keyword] = ACTIONS(1835),
    [sym_persistent_keyword] = ACTIONS(1835),
    [sym_dev_keyword] = ACTIONS(1835),
    [sym_devlev_keyword] = ACTIONS(1835),
    [sym_devchan_keyword] = ACTIONS(1835),
    [sym_if_keyword] = ACTIONS(1835),
    [sym_else_keyword] = ACTIONS(1835),
    [sym_switch_keyword] = ACTIONS(1835),
    [sym_case_keyword] = ACTIONS(1835),
    [sym_default_keyword] = ACTIONS(1835),
    [sym_while_keyword] = ACTIONS(1835),
    [sym_for_keyword] = ACTIONS(1835),
    [sym_break_keyword] = ACTIONS(1835),
    [sym_continue_keyword] = ACTIONS(1835),
    [sym_return_keyword] = ACTIONS(1835),
    [sym_select_keyword] = ACTIONS(1835),
    [sym_active_keyword] = ACTIONS(1835),
    [sym_struct_keyword] = ACTIONS(1835),
    [sym_structure_keyword] = ACTIONS(1835),
    [sym_button_event_keyword] = ACTIONS(1835),
    [sym_channel_event_keyword] = ACTIONS(1835),
    [sym_level_event_keyword] = ACTIONS(1835),
    [sym_data_event_keyword] = ACTIONS(1835),
    [sym_timeline_event_keyword] = ACTIONS(1835),
    [sym_custom_event_keyword] = ACTIONS(1835),
    [sym_push_keyword] = ACTIONS(1835),
    [sym_release_keyword] = ACTIONS(1835),
    [aux_sym_on_keyword_token1] = ACTIONS(1835),
    [aux_sym_off_keyword_token1] = ACTIONS(1835),
    [sym_send_level_keyword] = ACTIONS(1835),
    [sym_send_string_keyword] = ACTIONS(1835),
    [sym_send_command_keyword] = ACTIONS(1835),
    [sym_clear_buffer_keyword] = ACTIONS(1835),
    [sym_create_buffer_keyword] = ACTIONS(1835),
    [sym_create_multi_buffer_keyword] = ACTIONS(1835),
    [sym_call_keyword] = ACTIONS(1835),
    [sym_system_call_keyword] = ACTIONS(1835),
    [sym_devchan_to_keyword] = ACTIONS(1835),
    [sym_devchan_min_to_keyword] = ACTIONS(1835),
    [sym_devchan_total_off_keyword] = ACTIONS(1835),
    [sym_devchan_pulse_keyword] = ACTIONS(1835),
    [sym_wait_keyword] = ACTIONS(1835),
    [sym_cancel_wait_keyword] = ACTIONS(1835),
    [sym_wait_until_keyword] = ACTIONS(1835),
    [sym_cancel_wait_until_keyword] = ACTIONS(1835),
    [sym_cancel_all_wait_keyword] = ACTIONS(1835),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1835),
    [sym_not] = ACTIONS(1835),
    [sym_bnot] = ACTIONS(1835),
    [sym___file__] = ACTIONS(1835),
    [sym___date__] = ACTIONS(1835),
    [sym___ldate__] = ACTIONS(1835),
    [sym___time__] = ACTIONS(1835),
    [sym___line__] = ACTIONS(1835),
    [sym___name__] = ACTIONS(1835),
    [sym___netlinx__] = ACTIONS(1835),
    [sym_netlinx_axi_version] = ACTIONS(1835),
    [sym_url_flg_tcp] = ACTIONS(1835),
    [sym_url_flg_accinfopresent] = ACTIONS(1835),
    [sym_url_flg_temp] = ACTIONS(1835),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1835),
    [sym_url_flg_stat_mask] = ACTIONS(1835),
    [sym_url_flg_stat_lookup] = ACTIONS(1835),
    [sym_url_flg_stat_connecting] = ACTIONS(1835),
    [sym_url_flg_stat_waiting] = ACTIONS(1835),
    [sym_url_flg_stat_connected] = ACTIONS(1835),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1835),
    [sym_first_virtual_device] = ACTIONS(1835),
    [sym_first_local_port] = ACTIONS(1835),
    [sym_dynamic_virtual_device] = ACTIONS(1835),
    [sym_timeline_once] = ACTIONS(1835),
    [sym_timeline_repeat] = ACTIONS(1835),
    [sym_timeline_absolute] = ACTIONS(1835),
    [sym_timeline_relative] = ACTIONS(1835),
    [sym_do_push_timed_infinite] = ACTIONS(1835),
    [sym_source_type_no_address] = ACTIONS(1835),
    [sym_source_type_neuron_id] = ACTIONS(1835),
    [sym_source_type_ip_address] = ACTIONS(1835),
    [sym_source_type_axlink] = ACTIONS(1835),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1835),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1835),
    [sym_source_type_ip_socket_address] = ACTIONS(1835),
    [sym_source_type_rs232] = ACTIONS(1835),
    [sym_source_type_internal] = ACTIONS(1835),
    [sym_source_type_mac_address] = ACTIONS(1835),
    [sym_source_type_ipv4_port] = ACTIONS(1835),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1835),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1835),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1835),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1835),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1835),
    [sym_true] = ACTIONS(1835),
    [sym_false] = ACTIONS(1835),
    [sym_file_read_only] = ACTIONS(1835),
    [sym_file_rw_new] = ACTIONS(1835),
    [sym_file_rw_append] = ACTIONS(1835),
    [sym_ip_tcp] = ACTIONS(1835),
    [sym_ip_udp] = ACTIONS(1835),
    [sym_ip_udp_2way] = ACTIONS(1835),
    [sym_xml_encode_types] = ACTIONS(1835),
    [sym_xml_encode_char_as_list] = ACTIONS(1835),
    [sym_xml_encode_le] = ACTIONS(1835),
    [sym_xml_decode_types] = ACTIONS(1835),
    [sym_xml_decode_no_preserve] = ACTIONS(1835),
    [sym_duet_dev_type_display_device] = ACTIONS(1835),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1835),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1835),
    [sym_duet_dev_type_io_device] = ACTIONS(1835),
    [sym_duet_dev_type_relay_device] = ACTIONS(1835),
    [sym_duet_dev_type_ups] = ACTIONS(1835),
    [sym_duet_dev_type_amplifier] = ACTIONS(1835),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1835),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1835),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1835),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1835),
    [sym_duet_dev_type_camera] = ACTIONS(1835),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1835),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1835),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1835),
    [sym_duet_dev_type_dss] = ACTIONS(1835),
    [sym_duet_dev_type_dvr] = ACTIONS(1835),
    [sym_duet_dev_type_disc_device] = ACTIONS(1835),
    [sym_duet_dev_type_document_camera] = ACTIONS(1835),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1835),
    [sym_duet_dev_type_hvac] = ACTIONS(1835),
    [sym_duet_dev_type_keypad] = ACTIONS(1835),
    [sym_duet_dev_type_light] = ACTIONS(1835),
    [sym_duet_dev_type_monitor] = ACTIONS(1835),
    [sym_duet_dev_type_motor] = ACTIONS(1835),
    [sym_duet_dev_type_multi_window] = ACTIONS(1835),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1835),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1835),
    [sym_duet_dev_type_receiver] = ACTIONS(1835),
    [sym_duet_dev_type_security_system] = ACTIONS(1835),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1835),
    [sym_duet_dev_type_settop_box] = ACTIONS(1835),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1835),
    [sym_duet_dev_type_switcher] = ACTIONS(1835),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1835),
    [sym_duet_dev_type_tv] = ACTIONS(1835),
    [sym_duet_dev_type_utility] = ACTIONS(1835),
    [sym_duet_dev_type_vcr] = ACTIONS(1835),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1835),
    [sym_duet_dev_type_video_processor] = ACTIONS(1835),
    [sym_duet_dev_type_video_projector] = ACTIONS(1835),
    [sym_duet_dev_type_video_wall] = ACTIONS(1835),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1835),
    [sym_duet_dev_type_weather] = ACTIONS(1835),
    [sym_duet_dev_not_polled] = ACTIONS(1835),
    [sym_duet_dev_polled] = ACTIONS(1835),
    [sym_clkmgr_mode_network] = ACTIONS(1835),
    [sym_clkmgr_mode_standalone] = ACTIONS(1835),
    [sym_normal_standby] = ACTIONS(1835),
    [sym_normal_wake] = ACTIONS(1835),
    [sym_ip_multicast_ttl_option] = ACTIONS(1835),
    [sym_ip_ttl_subnet] = ACTIONS(1835),
    [sym_ip_ttl_site] = ACTIONS(1835),
    [sym_ip_ttl_region] = ACTIONS(1835),
    [sym_ip_ttl_continent] = ACTIONS(1835),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1835),
    [sym_ip_nodelay_on] = ACTIONS(1835),
    [sym_ip_nodelay_off] = ACTIONS(1835),
    [sym_amx_error] = ACTIONS(1835),
    [sym_amx_warning] = ACTIONS(1835),
    [sym_amx_info] = ACTIONS(1835),
    [sym_amx_debug] = ACTIONS(1835),
    [sym_netlinx_login_success] = ACTIONS(1835),
    [sym_netlinx_login_fail] = ACTIONS(1835),
    [sym_netlinx_logout] = ACTIONS(1835),
    [sym_null_str] = ACTIONS(1835),
    [sym_smtp_address] = ACTIONS(1835),
    [sym_smtp_port_number] = ACTIONS(1835),
    [sym_smtp_username] = ACTIONS(1835),
    [sym_smtp_password] = ACTIONS(1835),
    [sym_smtp_require_tls] = ACTIONS(1835),
    [sym_smtp_from] = ACTIONS(1835),
    [sym_smtp_tls_true] = ACTIONS(1835),
    [sym_smtp_tls_false] = ACTIONS(1835),
    [sym_listview_on_row_select_event] = ACTIONS(1835),
    [sym_data_structure_unknown] = ACTIONS(1835),
    [sym_data_structure_datafeed] = ACTIONS(1835),
    [sym_data_structure_datarecordset] = ACTIONS(1835),
    [sym_data_structure_datarecord] = ACTIONS(1835),
    [sym_data_structure_datafield] = ACTIONS(1835),
    [sym_data_max_content_field_count] = ACTIONS(1835),
    [sym_data_max_metadata_field_count] = ACTIONS(1835),
    [sym_data_max_id_length] = ACTIONS(1835),
    [sym_data_max_name_length] = ACTIONS(1835),
    [sym_data_max_description_length] = ACTIONS(1835),
    [sym_data_max_type_length] = ACTIONS(1835),
    [sym_data_max_format_length] = ACTIONS(1835),
    [sym_data_max_source_length] = ACTIONS(1835),
    [sym_data_max_value_length] = ACTIONS(1835),
    [sym_data_max_label_length] = ACTIONS(1835),
    [sym_data_max_url_length] = ACTIONS(1835),
    [sym_max_last_login_info_length] = ACTIONS(1835),
    [sym_data_type_unknown] = ACTIONS(1835),
    [sym_data_type_string] = ACTIONS(1835),
    [sym_data_type_datetime] = ACTIONS(1835),
    [sym_data_type_date] = ACTIONS(1835),
    [sym_data_type_time] = ACTIONS(1835),
    [sym_data_type_image] = ACTIONS(1835),
    [sym_data_format_url] = ACTIONS(1835),
    [sym_data_format_phone] = ACTIONS(1835),
    [sym_data_format_email] = ACTIONS(1835),
    [sym_data_format_iso8601] = ACTIONS(1835),
    [sym_auditlog_priv] = ACTIONS(1835),
    [sym_configuration_priv] = ACTIONS(1835),
    [sym_deviceconfig_priv] = ACTIONS(1835),
    [sym_ftp_priv] = ACTIONS(1835),
    [sym_http_priv] = ACTIONS(1835),
    [sym_networkconfig_priv] = ACTIONS(1835),
    [sym_programport_priv] = ACTIONS(1835),
    [sym_remoteui_priv] = ACTIONS(1835),
    [sym_securitycontrol_priv] = ACTIONS(1835),
    [sym_softwaremanagement_priv] = ACTIONS(1835),
    [sym_terminal_priv] = ACTIONS(1835),
    [sym_tpadmin_priv] = ACTIONS(1835),
    [sym_user1_priv] = ACTIONS(1835),
    [sym_user2_priv] = ACTIONS(1835),
    [sym_user3_priv] = ACTIONS(1835),
    [sym_user4_priv] = ACTIONS(1835),
    [sym_usermanagement_priv] = ACTIONS(1835),
    [sym_valid_account] = ACTIONS(1835),
    [sym_err_name_invalid] = ACTIONS(1835),
    [sym_err_password_invalid] = ACTIONS(1835),
    [sym_err_invalid_account] = ACTIONS(1835),
    [sym_err_no_authorization] = ACTIONS(1835),
    [sym_err_invalid_parameter] = ACTIONS(1835),
    [sym_err_user_locked_out] = ACTIONS(1835),
    [sym_err_user_account_expired] = ACTIONS(1835),
    [sym_err_general_error] = ACTIONS(1835),
    [sym_err_authentication_error] = ACTIONS(1835),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1835),
    [sym_tls_validate_certificate] = ACTIONS(1835),
    [sym_tdata] = ACTIONS(1835),
    [sym_tchannel] = ACTIONS(1835),
    [sym_tlevel] = ACTIONS(1835),
    [sym_tbutton] = ACTIONS(1835),
    [sym_ttimeline] = ACTIONS(1835),
    [sym_tcustom] = ACTIONS(1835),
    [sym_url_struct] = ACTIONS(1835),
    [sym_dns_struct] = ACTIONS(1835),
    [sym_ip_address_struct] = ACTIONS(1835),
    [sym_dev_info_struct] = ACTIONS(1835),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1835),
    [sym_data_feed] = ACTIONS(1835),
    [sym_data_field] = ACTIONS(1835),
    [sym_data_record] = ACTIONS(1835),
    [sym_wc_data_feed] = ACTIONS(1835),
    [sym_wc_data_field] = ACTIONS(1835),
    [sym_wc_data_record] = ACTIONS(1835),
    [sym_last_login_info] = ACTIONS(1835),
    [sym____reserved___] = ACTIONS(1835),
    [sym_date] = ACTIONS(1835),
    [sym_day] = ACTIONS(1835),
    [sym_time] = ACTIONS(1835),
    [sym_ldate] = ACTIONS(1835),
    [sym_push_device] = ACTIONS(1835),
    [sym_push_channel] = ACTIONS(1835),
    [sym_push_devchan] = ACTIONS(1835),
    [sym_release_device] = ACTIONS(1835),
    [sym_release_channel] = ACTIONS(1835),
    [sym_release_devchan] = ACTIONS(1835),
    [sym_master_slot] = ACTIONS(1835),
    [sym_get_pulse_time] = ACTIONS(1835),
    [sym_get_timer] = ACTIONS(1835),
    [sym_master_sn] = ACTIONS(1835),
    [sym_system_number] = ACTIONS(1835),
    [sym_button] = ACTIONS(1835),
    [sym_dv_channel] = ACTIONS(1835),
    [sym_data] = ACTIONS(1835),
    [sym_level] = ACTIONS(1835),
    [sym_timeline] = ACTIONS(1835),
    [sym_channel] = ACTIONS(1835),
    [sym_custom] = ACTIONS(1835),
    [sym_atoi] = ACTIONS(1835),
    [sym_atol] = ACTIONS(1835),
    [sym_atof] = ACTIONS(1835),
    [sym_itoa] = ACTIONS(1835),
    [sym_itohex] = ACTIONS(1835),
    [sym_hextoi] = ACTIONS(1835),
    [sym_ftoa] = ACTIONS(1835),
    [sym_find_string] = ACTIONS(1835),
    [sym_left_string] = ACTIONS(1835),
    [sym_right_string] = ACTIONS(1835),
    [sym_mid_string] = ACTIONS(1835),
    [sym_remove_string] = ACTIONS(1835),
    [sym_get_buffer_string] = ACTIONS(1835),
    [sym_get_buffer_char] = ACTIONS(1835),
    [sym_get_multi_buffer_string] = ACTIONS(1835),
    [sym_length_string] = ACTIONS(1835),
    [sym_max_length_string] = ACTIONS(1835),
    [sym_set_length_string] = ACTIONS(1835),
    [sym_compare_string] = ACTIONS(1835),
    [sym_upper_string] = ACTIONS(1835),
    [sym_lower_string] = ACTIONS(1835),
    [sym_get_last] = ACTIONS(1835),
    [sym_date_to_day] = ACTIONS(1835),
    [sym_date_to_month] = ACTIONS(1835),
    [sym_date_to_year] = ACTIONS(1835),
    [sym_time_to_hour] = ACTIONS(1835),
    [sym_time_to_minute] = ACTIONS(1835),
    [sym_time_to_second] = ACTIONS(1835),
    [sym_day_of_week] = ACTIONS(1835),
    [sym_set_pulse_time] = ACTIONS(1835),
    [sym_set_timer] = ACTIONS(1835),
    [sym_file_open] = ACTIONS(1835),
    [sym_file_close] = ACTIONS(1835),
    [sym_file_read] = ACTIONS(1835),
    [sym_file_read_line] = ACTIONS(1835),
    [sym_file_write] = ACTIONS(1835),
    [sym_file_write_line] = ACTIONS(1835),
    [sym_file_copy] = ACTIONS(1835),
    [sym_file_delete] = ACTIONS(1835),
    [sym_file_rename] = ACTIONS(1835),
    [sym_file_dir] = ACTIONS(1835),
    [sym_file_getdir] = ACTIONS(1835),
    [sym_file_setdir] = ACTIONS(1835),
    [sym_file_create_dir] = ACTIONS(1835),
    [sym_file_remove_dir] = ACTIONS(1835),
    [sym_file_seek] = ACTIONS(1835),
    [sym_length_array] = ACTIONS(1835),
    [sym_max_length_array] = ACTIONS(1835),
    [sym_set_length_array] = ACTIONS(1835),
    [sym_ip_client_open] = ACTIONS(1835),
    [sym_ip_client_close] = ACTIONS(1835),
    [sym_ip_server_open] = ACTIONS(1835),
    [sym_ip_server_close] = ACTIONS(1835),
    [sym_ip_bound_client_open] = ACTIONS(1835),
    [sym_ip_mc_server_open] = ACTIONS(1835),
    [sym_ip_set_option] = ACTIONS(1835),
    [sym_ssh_client_open] = ACTIONS(1835),
    [sym_ssh_client_close] = ACTIONS(1835),
    [sym_tls_client_open] = ACTIONS(1835),
    [sym_tls_client_close] = ACTIONS(1835),
    [sym_do_push] = ACTIONS(1835),
    [sym_do_push_timed] = ACTIONS(1835),
    [sym_do_release] = ACTIONS(1835),
    [sym_device_id] = ACTIONS(1835),
    [sym_device_info] = ACTIONS(1835),
    [sym_device_id_string] = ACTIONS(1835),
    [sym_device_standby] = ACTIONS(1835),
    [sym_device_wake] = ACTIONS(1835),
    [sym_timeline_create] = ACTIONS(1835),
    [sym_timeline_kill] = ACTIONS(1835),
    [sym_timeline_reload] = ACTIONS(1835),
    [sym_timeline_pause] = ACTIONS(1835),
    [sym_timeline_restart] = ACTIONS(1835),
    [sym_timeline_set] = ACTIONS(1835),
    [sym_timeline_get] = ACTIONS(1835),
    [sym_timeline_active] = ACTIONS(1835),
    [sym_timeline_dynamic_id] = ACTIONS(1835),
    [sym_variable_to_string] = ACTIONS(1835),
    [sym_string_to_variable] = ACTIONS(1835),
    [sym_length_variable_to_string] = ACTIONS(1835),
    [sym_type_cast] = ACTIONS(1835),
    [sym_variable_to_xml] = ACTIONS(1835),
    [sym_xml_to_variable] = ACTIONS(1835),
    [sym_length_variable_to_xml] = ACTIONS(1835),
    [sym_redirect_string] = ACTIONS(1835),
    [sym_random_number] = ACTIONS(1835),
    [sym_get_unique_id] = ACTIONS(1835),
    [sym_get_system_number] = ACTIONS(1835),
    [sym_set_system_number] = ACTIONS(1835),
    [sym_get_serial_number] = ACTIONS(1835),
    [sym_get_master_build] = ACTIONS(1835),
    [sym_set_validation_code] = ACTIONS(1835),
    [sym_reboot] = ACTIONS(1835),
    [sym_load_duet_module] = ACTIONS(1835),
    [sym_unload_duet_module] = ACTIONS(1835),
    [sym_duet_mem_size_set] = ACTIONS(1835),
    [sym_duet_mem_size_get] = ACTIONS(1835),
    [sym_dynamic_application_device] = ACTIONS(1835),
    [sym_dynamic_polled_port] = ACTIONS(1835),
    [sym_static_port_binding] = ACTIONS(1835),
    [sym_static_ip_binding] = ACTIONS(1835),
    [sym_get_dns_list] = ACTIONS(1835),
    [sym_set_dns_list] = ACTIONS(1835),
    [sym_get_ip_address] = ACTIONS(1835),
    [sym_set_ip_address] = ACTIONS(1835),
    [sym_get_url_list] = ACTIONS(1835),
    [sym_add_url_entry] = ACTIONS(1835),
    [sym_delete_url_entry] = ACTIONS(1835),
    [sym_abs_value] = ACTIONS(1835),
    [sym_max_value] = ACTIONS(1835),
    [sym_min_value] = ACTIONS(1835),
    [sym_exp_value] = ACTIONS(1835),
    [sym_log_value] = ACTIONS(1835),
    [sym_log10_value] = ACTIONS(1835),
    [sym_power_value] = ACTIONS(1835),
    [sym_sqrt_value] = ACTIONS(1835),
    [sym_raw_be] = ACTIONS(1835),
    [sym_raw_le] = ACTIONS(1835),
    [sym_internal_queue_size_set] = ACTIONS(1835),
    [sym_internal_queue_size_get] = ACTIONS(1835),
    [sym_internal_threshold_set] = ACTIONS(1835),
    [sym_internal_threshold_get] = ACTIONS(1835),
    [sym_rebuilt_event] = ACTIONS(1835),
    [sym_do_custom_event] = ACTIONS(1835),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1835),
    [sym_clkmgr_set_clk_source] = ACTIONS(1835),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1835),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1835),
    [sym_clkmgr_get_timezone] = ACTIONS(1835),
    [sym_clkmgr_set_timezone] = ACTIONS(1835),
    [sym_clkmgr_get_resync_period] = ACTIONS(1835),
    [sym_clkmgr_set_resync_period] = ACTIONS(1835),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1835),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1835),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1835),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1835),
    [sym_clkmgr_get_timeservers] = ACTIONS(1835),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1835),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1835),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1835),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1835),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1835),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1835),
    [sym_set_log_level] = ACTIONS(1835),
    [sym_get_log_level] = ACTIONS(1835),
    [sym_amx_log] = ACTIONS(1835),
    [sym_smtp_server_config_set] = ACTIONS(1835),
    [sym_smtp_server_config_get] = ACTIONS(1835),
    [sym_smtp_send] = ACTIONS(1835),
    [sym_validate_netlinx_account] = ACTIONS(1835),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1835),
    [sym_authenticate_certificate] = ACTIONS(1835),
    [sym_audit_netlinx_session_event] = ACTIONS(1835),
    [sym_audit_netlinx_generic_event] = ACTIONS(1835),
    [sym_get_available_flash_disk_space] = ACTIONS(1835),
    [sym_get_max_flash_disk_space] = ACTIONS(1835),
    [sym_data_create_feed] = ACTIONS(1835),
    [sym_data_delete_feed] = ACTIONS(1835),
    [sym_data_publish_feed] = ACTIONS(1835),
    [sym_data_get_published_feed] = ACTIONS(1835),
    [sym_data_add_record] = ACTIONS(1835),
    [sym_data_get_event_record] = ACTIONS(1835),
    [sym__wc_data_create_feed] = ACTIONS(1835),
    [sym__wc_data_add_record] = ACTIONS(1835),
    [sym__wc_data_get_event_record] = ACTIONS(1835),
    [sym_format] = ACTIONS(1835),
    [sym_astro_clock] = ACTIONS(1835),
    [sym_set_outdoor_temperature] = ACTIONS(1835),
    [sym_set_virtual_level_count] = ACTIONS(1835),
    [sym_set_virtual_channel_count] = ACTIONS(1835),
    [sym_set_virtual_port_count] = ACTIONS(1835),
    [sym__wc_encode_int] = ACTIONS(1835),
    [sym__wc_decode_int] = ACTIONS(1835),
    [sym__wc_int] = ACTIONS(1835),
    [sym__wc_to_ch_int] = ACTIONS(1835),
    [sym__ch_to_wc_int] = ACTIONS(1835),
    [sym__wc_find_string_int] = ACTIONS(1835),
    [sym__wc_left_string_int] = ACTIONS(1835),
    [sym__wc_lower_string_int] = ACTIONS(1835),
    [sym__wc_mid_string_int] = ACTIONS(1835),
    [sym__wc_remove_string_int] = ACTIONS(1835),
    [sym__wc_right_string_int] = ACTIONS(1835),
    [sym__wc_upper_string_int] = ACTIONS(1835),
    [sym__wc_compare_string_int] = ACTIONS(1835),
    [sym__wc_get_buffer_char_int] = ACTIONS(1835),
    [sym__wc_get_buffer_string_int] = ACTIONS(1835),
    [sym__wc_concat_string_int] = ACTIONS(1835),
    [sym__wc_file_open_int] = ACTIONS(1835),
    [sym__wc_file_close_int] = ACTIONS(1835),
    [sym__wc_file_read_int] = ACTIONS(1835),
    [sym__wc_file_read_line_int] = ACTIONS(1835),
    [sym__wc_file_write_int] = ACTIONS(1835),
    [sym__wc_file_write_line_int] = ACTIONS(1835),
    [aux_sym_preproc_include_token1] = ACTIONS(1835),
    [aux_sym_preproc_define_token1] = ACTIONS(1837),
    [aux_sym_preproc_warn_token1] = ACTIONS(1837),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1837),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1837),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1837),
    [anon_sym_LBRACK] = ACTIONS(1837),
    [anon_sym_LBRACE] = ACTIONS(1837),
    [anon_sym_LPAREN] = ACTIONS(1835),
    [anon_sym_BANG] = ACTIONS(1837),
    [anon_sym_TILDE] = ACTIONS(1837),
    [anon_sym_DASH] = ACTIONS(1835),
    [anon_sym_PLUS] = ACTIONS(1835),
    [anon_sym_DASH_DASH] = ACTIONS(1837),
    [anon_sym_PLUS_PLUS] = ACTIONS(1837),
    [anon_sym_DQUOTE] = ACTIONS(1837),
    [anon_sym_SQUOTE] = ACTIONS(1837),
    [sym_number_literal] = ACTIONS(1837),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(299)] = {
    [ts_builtin_sym_end] = ACTIONS(1717),
    [sym_identifier] = ACTIONS(1715),
    [sym_program_name_keyword] = ACTIONS(1715),
    [sym_module_name_keyword] = ACTIONS(1715),
    [sym_define_device_keyword] = ACTIONS(1715),
    [sym_define_combine_keyword] = ACTIONS(1715),
    [sym_define_constant_keyword] = ACTIONS(1715),
    [sym_define_type_keyword] = ACTIONS(1715),
    [sym_define_variable_keyword] = ACTIONS(1715),
    [sym_define_system_variable_keyword] = ACTIONS(1715),
    [sym_define_connect_level_keyword] = ACTIONS(1715),
    [sym_define_latching_keyword] = ACTIONS(1715),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1715),
    [sym_define_toggling_keyword] = ACTIONS(1715),
    [sym_define_call_keyword] = ACTIONS(1715),
    [sym_define_function_keyword] = ACTIONS(1715),
    [sym_define_library_function_keyword] = ACTIONS(1715),
    [sym_define_start_keyword] = ACTIONS(1715),
    [sym_define_event_keyword] = ACTIONS(1715),
    [sym_define_program_keyword] = ACTIONS(1715),
    [sym_define_module_keyword] = ACTIONS(1715),
    [sym_char_keyword] = ACTIONS(1715),
    [sym_widechar_keyword] = ACTIONS(1715),
    [sym_integer_keyword] = ACTIONS(1715),
    [sym_sinteger_keyword] = ACTIONS(1715),
    [sym_long_keyword] = ACTIONS(1715),
    [sym_slong_keyword] = ACTIONS(1715),
    [sym_float_keyword] = ACTIONS(1715),
    [sym_double_keyword] = ACTIONS(1715),
    [sym_variant_keyword] = ACTIONS(1715),
    [sym_variantarray_keyword] = ACTIONS(1715),
    [sym_local_var_keyword] = ACTIONS(1715),
    [sym_stack_var_keyword] = ACTIONS(1715),
    [sym_constant_keyword] = ACTIONS(1715),
    [sym_volatile_keyword] = ACTIONS(1715),
    [sym_non_volatile_keyword] = ACTIONS(1715),
    [sym_persistent_keyword] = ACTIONS(1715),
    [sym_dev_keyword] = ACTIONS(1715),
    [sym_devlev_keyword] = ACTIONS(1715),
    [sym_devchan_keyword] = ACTIONS(1715),
    [sym_if_keyword] = ACTIONS(1715),
    [sym_else_keyword] = ACTIONS(1715),
    [sym_switch_keyword] = ACTIONS(1715),
    [sym_case_keyword] = ACTIONS(1715),
    [sym_default_keyword] = ACTIONS(1715),
    [sym_while_keyword] = ACTIONS(1715),
    [sym_for_keyword] = ACTIONS(1715),
    [sym_break_keyword] = ACTIONS(1715),
    [sym_continue_keyword] = ACTIONS(1715),
    [sym_return_keyword] = ACTIONS(1715),
    [sym_select_keyword] = ACTIONS(1715),
    [sym_active_keyword] = ACTIONS(1715),
    [sym_struct_keyword] = ACTIONS(1715),
    [sym_structure_keyword] = ACTIONS(1715),
    [sym_button_event_keyword] = ACTIONS(1715),
    [sym_channel_event_keyword] = ACTIONS(1715),
    [sym_level_event_keyword] = ACTIONS(1715),
    [sym_data_event_keyword] = ACTIONS(1715),
    [sym_timeline_event_keyword] = ACTIONS(1715),
    [sym_custom_event_keyword] = ACTIONS(1715),
    [sym_push_keyword] = ACTIONS(1715),
    [sym_release_keyword] = ACTIONS(1715),
    [aux_sym_on_keyword_token1] = ACTIONS(1715),
    [aux_sym_off_keyword_token1] = ACTIONS(1715),
    [sym_send_level_keyword] = ACTIONS(1715),
    [sym_send_string_keyword] = ACTIONS(1715),
    [sym_send_command_keyword] = ACTIONS(1715),
    [sym_clear_buffer_keyword] = ACTIONS(1715),
    [sym_create_buffer_keyword] = ACTIONS(1715),
    [sym_create_multi_buffer_keyword] = ACTIONS(1715),
    [sym_call_keyword] = ACTIONS(1715),
    [sym_system_call_keyword] = ACTIONS(1715),
    [sym_devchan_to_keyword] = ACTIONS(1715),
    [sym_devchan_min_to_keyword] = ACTIONS(1715),
    [sym_devchan_total_off_keyword] = ACTIONS(1715),
    [sym_devchan_pulse_keyword] = ACTIONS(1715),
    [sym_wait_keyword] = ACTIONS(1715),
    [sym_cancel_wait_keyword] = ACTIONS(1715),
    [sym_wait_until_keyword] = ACTIONS(1715),
    [sym_cancel_wait_until_keyword] = ACTIONS(1715),
    [sym_cancel_all_wait_keyword] = ACTIONS(1715),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1715),
    [sym_not] = ACTIONS(1715),
    [sym_bnot] = ACTIONS(1715),
    [sym___file__] = ACTIONS(1715),
    [sym___date__] = ACTIONS(1715),
    [sym___ldate__] = ACTIONS(1715),
    [sym___time__] = ACTIONS(1715),
    [sym___line__] = ACTIONS(1715),
    [sym___name__] = ACTIONS(1715),
    [sym___netlinx__] = ACTIONS(1715),
    [sym_netlinx_axi_version] = ACTIONS(1715),
    [sym_url_flg_tcp] = ACTIONS(1715),
    [sym_url_flg_accinfopresent] = ACTIONS(1715),
    [sym_url_flg_temp] = ACTIONS(1715),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1715),
    [sym_url_flg_stat_mask] = ACTIONS(1715),
    [sym_url_flg_stat_lookup] = ACTIONS(1715),
    [sym_url_flg_stat_connecting] = ACTIONS(1715),
    [sym_url_flg_stat_waiting] = ACTIONS(1715),
    [sym_url_flg_stat_connected] = ACTIONS(1715),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1715),
    [sym_first_virtual_device] = ACTIONS(1715),
    [sym_first_local_port] = ACTIONS(1715),
    [sym_dynamic_virtual_device] = ACTIONS(1715),
    [sym_timeline_once] = ACTIONS(1715),
    [sym_timeline_repeat] = ACTIONS(1715),
    [sym_timeline_absolute] = ACTIONS(1715),
    [sym_timeline_relative] = ACTIONS(1715),
    [sym_do_push_timed_infinite] = ACTIONS(1715),
    [sym_source_type_no_address] = ACTIONS(1715),
    [sym_source_type_neuron_id] = ACTIONS(1715),
    [sym_source_type_ip_address] = ACTIONS(1715),
    [sym_source_type_axlink] = ACTIONS(1715),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1715),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1715),
    [sym_source_type_ip_socket_address] = ACTIONS(1715),
    [sym_source_type_rs232] = ACTIONS(1715),
    [sym_source_type_internal] = ACTIONS(1715),
    [sym_source_type_mac_address] = ACTIONS(1715),
    [sym_source_type_ipv4_port] = ACTIONS(1715),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1715),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1715),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1715),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1715),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1715),
    [sym_true] = ACTIONS(1715),
    [sym_false] = ACTIONS(1715),
    [sym_file_read_only] = ACTIONS(1715),
    [sym_file_rw_new] = ACTIONS(1715),
    [sym_file_rw_append] = ACTIONS(1715),
    [sym_ip_tcp] = ACTIONS(1715),
    [sym_ip_udp] = ACTIONS(1715),
    [sym_ip_udp_2way] = ACTIONS(1715),
    [sym_xml_encode_types] = ACTIONS(1715),
    [sym_xml_encode_char_as_list] = ACTIONS(1715),
    [sym_xml_encode_le] = ACTIONS(1715),
    [sym_xml_decode_types] = ACTIONS(1715),
    [sym_xml_decode_no_preserve] = ACTIONS(1715),
    [sym_duet_dev_type_display_device] = ACTIONS(1715),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1715),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1715),
    [sym_duet_dev_type_io_device] = ACTIONS(1715),
    [sym_duet_dev_type_relay_device] = ACTIONS(1715),
    [sym_duet_dev_type_ups] = ACTIONS(1715),
    [sym_duet_dev_type_amplifier] = ACTIONS(1715),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1715),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1715),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1715),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1715),
    [sym_duet_dev_type_camera] = ACTIONS(1715),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1715),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1715),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1715),
    [sym_duet_dev_type_dss] = ACTIONS(1715),
    [sym_duet_dev_type_dvr] = ACTIONS(1715),
    [sym_duet_dev_type_disc_device] = ACTIONS(1715),
    [sym_duet_dev_type_document_camera] = ACTIONS(1715),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1715),
    [sym_duet_dev_type_hvac] = ACTIONS(1715),
    [sym_duet_dev_type_keypad] = ACTIONS(1715),
    [sym_duet_dev_type_light] = ACTIONS(1715),
    [sym_duet_dev_type_monitor] = ACTIONS(1715),
    [sym_duet_dev_type_motor] = ACTIONS(1715),
    [sym_duet_dev_type_multi_window] = ACTIONS(1715),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1715),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1715),
    [sym_duet_dev_type_receiver] = ACTIONS(1715),
    [sym_duet_dev_type_security_system] = ACTIONS(1715),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1715),
    [sym_duet_dev_type_settop_box] = ACTIONS(1715),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1715),
    [sym_duet_dev_type_switcher] = ACTIONS(1715),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1715),
    [sym_duet_dev_type_tv] = ACTIONS(1715),
    [sym_duet_dev_type_utility] = ACTIONS(1715),
    [sym_duet_dev_type_vcr] = ACTIONS(1715),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1715),
    [sym_duet_dev_type_video_processor] = ACTIONS(1715),
    [sym_duet_dev_type_video_projector] = ACTIONS(1715),
    [sym_duet_dev_type_video_wall] = ACTIONS(1715),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1715),
    [sym_duet_dev_type_weather] = ACTIONS(1715),
    [sym_duet_dev_not_polled] = ACTIONS(1715),
    [sym_duet_dev_polled] = ACTIONS(1715),
    [sym_clkmgr_mode_network] = ACTIONS(1715),
    [sym_clkmgr_mode_standalone] = ACTIONS(1715),
    [sym_normal_standby] = ACTIONS(1715),
    [sym_normal_wake] = ACTIONS(1715),
    [sym_ip_multicast_ttl_option] = ACTIONS(1715),
    [sym_ip_ttl_subnet] = ACTIONS(1715),
    [sym_ip_ttl_site] = ACTIONS(1715),
    [sym_ip_ttl_region] = ACTIONS(1715),
    [sym_ip_ttl_continent] = ACTIONS(1715),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1715),
    [sym_ip_nodelay_on] = ACTIONS(1715),
    [sym_ip_nodelay_off] = ACTIONS(1715),
    [sym_amx_error] = ACTIONS(1715),
    [sym_amx_warning] = ACTIONS(1715),
    [sym_amx_info] = ACTIONS(1715),
    [sym_amx_debug] = ACTIONS(1715),
    [sym_netlinx_login_success] = ACTIONS(1715),
    [sym_netlinx_login_fail] = ACTIONS(1715),
    [sym_netlinx_logout] = ACTIONS(1715),
    [sym_null_str] = ACTIONS(1715),
    [sym_smtp_address] = ACTIONS(1715),
    [sym_smtp_port_number] = ACTIONS(1715),
    [sym_smtp_username] = ACTIONS(1715),
    [sym_smtp_password] = ACTIONS(1715),
    [sym_smtp_require_tls] = ACTIONS(1715),
    [sym_smtp_from] = ACTIONS(1715),
    [sym_smtp_tls_true] = ACTIONS(1715),
    [sym_smtp_tls_false] = ACTIONS(1715),
    [sym_listview_on_row_select_event] = ACTIONS(1715),
    [sym_data_structure_unknown] = ACTIONS(1715),
    [sym_data_structure_datafeed] = ACTIONS(1715),
    [sym_data_structure_datarecordset] = ACTIONS(1715),
    [sym_data_structure_datarecord] = ACTIONS(1715),
    [sym_data_structure_datafield] = ACTIONS(1715),
    [sym_data_max_content_field_count] = ACTIONS(1715),
    [sym_data_max_metadata_field_count] = ACTIONS(1715),
    [sym_data_max_id_length] = ACTIONS(1715),
    [sym_data_max_name_length] = ACTIONS(1715),
    [sym_data_max_description_length] = ACTIONS(1715),
    [sym_data_max_type_length] = ACTIONS(1715),
    [sym_data_max_format_length] = ACTIONS(1715),
    [sym_data_max_source_length] = ACTIONS(1715),
    [sym_data_max_value_length] = ACTIONS(1715),
    [sym_data_max_label_length] = ACTIONS(1715),
    [sym_data_max_url_length] = ACTIONS(1715),
    [sym_max_last_login_info_length] = ACTIONS(1715),
    [sym_data_type_unknown] = ACTIONS(1715),
    [sym_data_type_string] = ACTIONS(1715),
    [sym_data_type_datetime] = ACTIONS(1715),
    [sym_data_type_date] = ACTIONS(1715),
    [sym_data_type_time] = ACTIONS(1715),
    [sym_data_type_image] = ACTIONS(1715),
    [sym_data_format_url] = ACTIONS(1715),
    [sym_data_format_phone] = ACTIONS(1715),
    [sym_data_format_email] = ACTIONS(1715),
    [sym_data_format_iso8601] = ACTIONS(1715),
    [sym_auditlog_priv] = ACTIONS(1715),
    [sym_configuration_priv] = ACTIONS(1715),
    [sym_deviceconfig_priv] = ACTIONS(1715),
    [sym_ftp_priv] = ACTIONS(1715),
    [sym_http_priv] = ACTIONS(1715),
    [sym_networkconfig_priv] = ACTIONS(1715),
    [sym_programport_priv] = ACTIONS(1715),
    [sym_remoteui_priv] = ACTIONS(1715),
    [sym_securitycontrol_priv] = ACTIONS(1715),
    [sym_softwaremanagement_priv] = ACTIONS(1715),
    [sym_terminal_priv] = ACTIONS(1715),
    [sym_tpadmin_priv] = ACTIONS(1715),
    [sym_user1_priv] = ACTIONS(1715),
    [sym_user2_priv] = ACTIONS(1715),
    [sym_user3_priv] = ACTIONS(1715),
    [sym_user4_priv] = ACTIONS(1715),
    [sym_usermanagement_priv] = ACTIONS(1715),
    [sym_valid_account] = ACTIONS(1715),
    [sym_err_name_invalid] = ACTIONS(1715),
    [sym_err_password_invalid] = ACTIONS(1715),
    [sym_err_invalid_account] = ACTIONS(1715),
    [sym_err_no_authorization] = ACTIONS(1715),
    [sym_err_invalid_parameter] = ACTIONS(1715),
    [sym_err_user_locked_out] = ACTIONS(1715),
    [sym_err_user_account_expired] = ACTIONS(1715),
    [sym_err_general_error] = ACTIONS(1715),
    [sym_err_authentication_error] = ACTIONS(1715),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1715),
    [sym_tls_validate_certificate] = ACTIONS(1715),
    [sym_tdata] = ACTIONS(1715),
    [sym_tchannel] = ACTIONS(1715),
    [sym_tlevel] = ACTIONS(1715),
    [sym_tbutton] = ACTIONS(1715),
    [sym_ttimeline] = ACTIONS(1715),
    [sym_tcustom] = ACTIONS(1715),
    [sym_url_struct] = ACTIONS(1715),
    [sym_dns_struct] = ACTIONS(1715),
    [sym_ip_address_struct] = ACTIONS(1715),
    [sym_dev_info_struct] = ACTIONS(1715),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1715),
    [sym_data_feed] = ACTIONS(1715),
    [sym_data_field] = ACTIONS(1715),
    [sym_data_record] = ACTIONS(1715),
    [sym_wc_data_feed] = ACTIONS(1715),
    [sym_wc_data_field] = ACTIONS(1715),
    [sym_wc_data_record] = ACTIONS(1715),
    [sym_last_login_info] = ACTIONS(1715),
    [sym____reserved___] = ACTIONS(1715),
    [sym_date] = ACTIONS(1715),
    [sym_day] = ACTIONS(1715),
    [sym_time] = ACTIONS(1715),
    [sym_ldate] = ACTIONS(1715),
    [sym_push_device] = ACTIONS(1715),
    [sym_push_channel] = ACTIONS(1715),
    [sym_push_devchan] = ACTIONS(1715),
    [sym_release_device] = ACTIONS(1715),
    [sym_release_channel] = ACTIONS(1715),
    [sym_release_devchan] = ACTIONS(1715),
    [sym_master_slot] = ACTIONS(1715),
    [sym_get_pulse_time] = ACTIONS(1715),
    [sym_get_timer] = ACTIONS(1715),
    [sym_master_sn] = ACTIONS(1715),
    [sym_system_number] = ACTIONS(1715),
    [sym_button] = ACTIONS(1715),
    [sym_dv_channel] = ACTIONS(1715),
    [sym_data] = ACTIONS(1715),
    [sym_level] = ACTIONS(1715),
    [sym_timeline] = ACTIONS(1715),
    [sym_channel] = ACTIONS(1715),
    [sym_custom] = ACTIONS(1715),
    [sym_atoi] = ACTIONS(1715),
    [sym_atol] = ACTIONS(1715),
    [sym_atof] = ACTIONS(1715),
    [sym_itoa] = ACTIONS(1715),
    [sym_itohex] = ACTIONS(1715),
    [sym_hextoi] = ACTIONS(1715),
    [sym_ftoa] = ACTIONS(1715),
    [sym_find_string] = ACTIONS(1715),
    [sym_left_string] = ACTIONS(1715),
    [sym_right_string] = ACTIONS(1715),
    [sym_mid_string] = ACTIONS(1715),
    [sym_remove_string] = ACTIONS(1715),
    [sym_get_buffer_string] = ACTIONS(1715),
    [sym_get_buffer_char] = ACTIONS(1715),
    [sym_get_multi_buffer_string] = ACTIONS(1715),
    [sym_length_string] = ACTIONS(1715),
    [sym_max_length_string] = ACTIONS(1715),
    [sym_set_length_string] = ACTIONS(1715),
    [sym_compare_string] = ACTIONS(1715),
    [sym_upper_string] = ACTIONS(1715),
    [sym_lower_string] = ACTIONS(1715),
    [sym_get_last] = ACTIONS(1715),
    [sym_date_to_day] = ACTIONS(1715),
    [sym_date_to_month] = ACTIONS(1715),
    [sym_date_to_year] = ACTIONS(1715),
    [sym_time_to_hour] = ACTIONS(1715),
    [sym_time_to_minute] = ACTIONS(1715),
    [sym_time_to_second] = ACTIONS(1715),
    [sym_day_of_week] = ACTIONS(1715),
    [sym_set_pulse_time] = ACTIONS(1715),
    [sym_set_timer] = ACTIONS(1715),
    [sym_file_open] = ACTIONS(1715),
    [sym_file_close] = ACTIONS(1715),
    [sym_file_read] = ACTIONS(1715),
    [sym_file_read_line] = ACTIONS(1715),
    [sym_file_write] = ACTIONS(1715),
    [sym_file_write_line] = ACTIONS(1715),
    [sym_file_copy] = ACTIONS(1715),
    [sym_file_delete] = ACTIONS(1715),
    [sym_file_rename] = ACTIONS(1715),
    [sym_file_dir] = ACTIONS(1715),
    [sym_file_getdir] = ACTIONS(1715),
    [sym_file_setdir] = ACTIONS(1715),
    [sym_file_create_dir] = ACTIONS(1715),
    [sym_file_remove_dir] = ACTIONS(1715),
    [sym_file_seek] = ACTIONS(1715),
    [sym_length_array] = ACTIONS(1715),
    [sym_max_length_array] = ACTIONS(1715),
    [sym_set_length_array] = ACTIONS(1715),
    [sym_ip_client_open] = ACTIONS(1715),
    [sym_ip_client_close] = ACTIONS(1715),
    [sym_ip_server_open] = ACTIONS(1715),
    [sym_ip_server_close] = ACTIONS(1715),
    [sym_ip_bound_client_open] = ACTIONS(1715),
    [sym_ip_mc_server_open] = ACTIONS(1715),
    [sym_ip_set_option] = ACTIONS(1715),
    [sym_ssh_client_open] = ACTIONS(1715),
    [sym_ssh_client_close] = ACTIONS(1715),
    [sym_tls_client_open] = ACTIONS(1715),
    [sym_tls_client_close] = ACTIONS(1715),
    [sym_do_push] = ACTIONS(1715),
    [sym_do_push_timed] = ACTIONS(1715),
    [sym_do_release] = ACTIONS(1715),
    [sym_device_id] = ACTIONS(1715),
    [sym_device_info] = ACTIONS(1715),
    [sym_device_id_string] = ACTIONS(1715),
    [sym_device_standby] = ACTIONS(1715),
    [sym_device_wake] = ACTIONS(1715),
    [sym_timeline_create] = ACTIONS(1715),
    [sym_timeline_kill] = ACTIONS(1715),
    [sym_timeline_reload] = ACTIONS(1715),
    [sym_timeline_pause] = ACTIONS(1715),
    [sym_timeline_restart] = ACTIONS(1715),
    [sym_timeline_set] = ACTIONS(1715),
    [sym_timeline_get] = ACTIONS(1715),
    [sym_timeline_active] = ACTIONS(1715),
    [sym_timeline_dynamic_id] = ACTIONS(1715),
    [sym_variable_to_string] = ACTIONS(1715),
    [sym_string_to_variable] = ACTIONS(1715),
    [sym_length_variable_to_string] = ACTIONS(1715),
    [sym_type_cast] = ACTIONS(1715),
    [sym_variable_to_xml] = ACTIONS(1715),
    [sym_xml_to_variable] = ACTIONS(1715),
    [sym_length_variable_to_xml] = ACTIONS(1715),
    [sym_redirect_string] = ACTIONS(1715),
    [sym_random_number] = ACTIONS(1715),
    [sym_get_unique_id] = ACTIONS(1715),
    [sym_get_system_number] = ACTIONS(1715),
    [sym_set_system_number] = ACTIONS(1715),
    [sym_get_serial_number] = ACTIONS(1715),
    [sym_get_master_build] = ACTIONS(1715),
    [sym_set_validation_code] = ACTIONS(1715),
    [sym_reboot] = ACTIONS(1715),
    [sym_load_duet_module] = ACTIONS(1715),
    [sym_unload_duet_module] = ACTIONS(1715),
    [sym_duet_mem_size_set] = ACTIONS(1715),
    [sym_duet_mem_size_get] = ACTIONS(1715),
    [sym_dynamic_application_device] = ACTIONS(1715),
    [sym_dynamic_polled_port] = ACTIONS(1715),
    [sym_static_port_binding] = ACTIONS(1715),
    [sym_static_ip_binding] = ACTIONS(1715),
    [sym_get_dns_list] = ACTIONS(1715),
    [sym_set_dns_list] = ACTIONS(1715),
    [sym_get_ip_address] = ACTIONS(1715),
    [sym_set_ip_address] = ACTIONS(1715),
    [sym_get_url_list] = ACTIONS(1715),
    [sym_add_url_entry] = ACTIONS(1715),
    [sym_delete_url_entry] = ACTIONS(1715),
    [sym_abs_value] = ACTIONS(1715),
    [sym_max_value] = ACTIONS(1715),
    [sym_min_value] = ACTIONS(1715),
    [sym_exp_value] = ACTIONS(1715),
    [sym_log_value] = ACTIONS(1715),
    [sym_log10_value] = ACTIONS(1715),
    [sym_power_value] = ACTIONS(1715),
    [sym_sqrt_value] = ACTIONS(1715),
    [sym_raw_be] = ACTIONS(1715),
    [sym_raw_le] = ACTIONS(1715),
    [sym_internal_queue_size_set] = ACTIONS(1715),
    [sym_internal_queue_size_get] = ACTIONS(1715),
    [sym_internal_threshold_set] = ACTIONS(1715),
    [sym_internal_threshold_get] = ACTIONS(1715),
    [sym_rebuilt_event] = ACTIONS(1715),
    [sym_do_custom_event] = ACTIONS(1715),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1715),
    [sym_clkmgr_set_clk_source] = ACTIONS(1715),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1715),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1715),
    [sym_clkmgr_get_timezone] = ACTIONS(1715),
    [sym_clkmgr_set_timezone] = ACTIONS(1715),
    [sym_clkmgr_get_resync_period] = ACTIONS(1715),
    [sym_clkmgr_set_resync_period] = ACTIONS(1715),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1715),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1715),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1715),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1715),
    [sym_clkmgr_get_timeservers] = ACTIONS(1715),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1715),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1715),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1715),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1715),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1715),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1715),
    [sym_set_log_level] = ACTIONS(1715),
    [sym_get_log_level] = ACTIONS(1715),
    [sym_amx_log] = ACTIONS(1715),
    [sym_smtp_server_config_set] = ACTIONS(1715),
    [sym_smtp_server_config_get] = ACTIONS(1715),
    [sym_smtp_send] = ACTIONS(1715),
    [sym_validate_netlinx_account] = ACTIONS(1715),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1715),
    [sym_authenticate_certificate] = ACTIONS(1715),
    [sym_audit_netlinx_session_event] = ACTIONS(1715),
    [sym_audit_netlinx_generic_event] = ACTIONS(1715),
    [sym_get_available_flash_disk_space] = ACTIONS(1715),
    [sym_get_max_flash_disk_space] = ACTIONS(1715),
    [sym_data_create_feed] = ACTIONS(1715),
    [sym_data_delete_feed] = ACTIONS(1715),
    [sym_data_publish_feed] = ACTIONS(1715),
    [sym_data_get_published_feed] = ACTIONS(1715),
    [sym_data_add_record] = ACTIONS(1715),
    [sym_data_get_event_record] = ACTIONS(1715),
    [sym__wc_data_create_feed] = ACTIONS(1715),
    [sym__wc_data_add_record] = ACTIONS(1715),
    [sym__wc_data_get_event_record] = ACTIONS(1715),
    [sym_format] = ACTIONS(1715),
    [sym_astro_clock] = ACTIONS(1715),
    [sym_set_outdoor_temperature] = ACTIONS(1715),
    [sym_set_virtual_level_count] = ACTIONS(1715),
    [sym_set_virtual_channel_count] = ACTIONS(1715),
    [sym_set_virtual_port_count] = ACTIONS(1715),
    [sym__wc_encode_int] = ACTIONS(1715),
    [sym__wc_decode_int] = ACTIONS(1715),
    [sym__wc_int] = ACTIONS(1715),
    [sym__wc_to_ch_int] = ACTIONS(1715),
    [sym__ch_to_wc_int] = ACTIONS(1715),
    [sym__wc_find_string_int] = ACTIONS(1715),
    [sym__wc_left_string_int] = ACTIONS(1715),
    [sym__wc_lower_string_int] = ACTIONS(1715),
    [sym__wc_mid_string_int] = ACTIONS(1715),
    [sym__wc_remove_string_int] = ACTIONS(1715),
    [sym__wc_right_string_int] = ACTIONS(1715),
    [sym__wc_upper_string_int] = ACTIONS(1715),
    [sym__wc_compare_string_int] = ACTIONS(1715),
    [sym__wc_get_buffer_char_int] = ACTIONS(1715),
    [sym__wc_get_buffer_string_int] = ACTIONS(1715),
    [sym__wc_concat_string_int] = ACTIONS(1715),
    [sym__wc_file_open_int] = ACTIONS(1715),
    [sym__wc_file_close_int] = ACTIONS(1715),
    [sym__wc_file_read_int] = ACTIONS(1715),
    [sym__wc_file_read_line_int] = ACTIONS(1715),
    [sym__wc_file_write_int] = ACTIONS(1715),
    [sym__wc_file_write_line_int] = ACTIONS(1715),
    [aux_sym_preproc_include_token1] = ACTIONS(1715),
    [aux_sym_preproc_define_token1] = ACTIONS(1717),
    [aux_sym_preproc_warn_token1] = ACTIONS(1717),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1717),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1717),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1717),
    [anon_sym_LBRACK] = ACTIONS(1717),
    [anon_sym_LBRACE] = ACTIONS(1717),
    [anon_sym_LPAREN] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [anon_sym_DASH] = ACTIONS(1715),
    [anon_sym_PLUS] = ACTIONS(1715),
    [anon_sym_DASH_DASH] = ACTIONS(1717),
    [anon_sym_PLUS_PLUS] = ACTIONS(1717),
    [anon_sym_DQUOTE] = ACTIONS(1717),
    [anon_sym_SQUOTE] = ACTIONS(1717),
    [sym_number_literal] = ACTIONS(1717),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(300)] = {
    [ts_builtin_sym_end] = ACTIONS(1775),
    [sym_identifier] = ACTIONS(1773),
    [sym_program_name_keyword] = ACTIONS(1773),
    [sym_module_name_keyword] = ACTIONS(1773),
    [sym_define_device_keyword] = ACTIONS(1773),
    [sym_define_combine_keyword] = ACTIONS(1773),
    [sym_define_constant_keyword] = ACTIONS(1773),
    [sym_define_type_keyword] = ACTIONS(1773),
    [sym_define_variable_keyword] = ACTIONS(1773),
    [sym_define_system_variable_keyword] = ACTIONS(1773),
    [sym_define_connect_level_keyword] = ACTIONS(1773),
    [sym_define_latching_keyword] = ACTIONS(1773),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1773),
    [sym_define_toggling_keyword] = ACTIONS(1773),
    [sym_define_call_keyword] = ACTIONS(1773),
    [sym_define_function_keyword] = ACTIONS(1773),
    [sym_define_library_function_keyword] = ACTIONS(1773),
    [sym_define_start_keyword] = ACTIONS(1773),
    [sym_define_event_keyword] = ACTIONS(1773),
    [sym_define_program_keyword] = ACTIONS(1773),
    [sym_define_module_keyword] = ACTIONS(1773),
    [sym_char_keyword] = ACTIONS(1773),
    [sym_widechar_keyword] = ACTIONS(1773),
    [sym_integer_keyword] = ACTIONS(1773),
    [sym_sinteger_keyword] = ACTIONS(1773),
    [sym_long_keyword] = ACTIONS(1773),
    [sym_slong_keyword] = ACTIONS(1773),
    [sym_float_keyword] = ACTIONS(1773),
    [sym_double_keyword] = ACTIONS(1773),
    [sym_variant_keyword] = ACTIONS(1773),
    [sym_variantarray_keyword] = ACTIONS(1773),
    [sym_local_var_keyword] = ACTIONS(1773),
    [sym_stack_var_keyword] = ACTIONS(1773),
    [sym_constant_keyword] = ACTIONS(1773),
    [sym_volatile_keyword] = ACTIONS(1773),
    [sym_non_volatile_keyword] = ACTIONS(1773),
    [sym_persistent_keyword] = ACTIONS(1773),
    [sym_dev_keyword] = ACTIONS(1773),
    [sym_devlev_keyword] = ACTIONS(1773),
    [sym_devchan_keyword] = ACTIONS(1773),
    [sym_if_keyword] = ACTIONS(1773),
    [sym_else_keyword] = ACTIONS(1773),
    [sym_switch_keyword] = ACTIONS(1773),
    [sym_case_keyword] = ACTIONS(1773),
    [sym_default_keyword] = ACTIONS(1773),
    [sym_while_keyword] = ACTIONS(1773),
    [sym_for_keyword] = ACTIONS(1773),
    [sym_break_keyword] = ACTIONS(1773),
    [sym_continue_keyword] = ACTIONS(1773),
    [sym_return_keyword] = ACTIONS(1773),
    [sym_select_keyword] = ACTIONS(1773),
    [sym_active_keyword] = ACTIONS(1773),
    [sym_struct_keyword] = ACTIONS(1773),
    [sym_structure_keyword] = ACTIONS(1773),
    [sym_button_event_keyword] = ACTIONS(1773),
    [sym_channel_event_keyword] = ACTIONS(1773),
    [sym_level_event_keyword] = ACTIONS(1773),
    [sym_data_event_keyword] = ACTIONS(1773),
    [sym_timeline_event_keyword] = ACTIONS(1773),
    [sym_custom_event_keyword] = ACTIONS(1773),
    [sym_push_keyword] = ACTIONS(1773),
    [sym_release_keyword] = ACTIONS(1773),
    [aux_sym_on_keyword_token1] = ACTIONS(1773),
    [aux_sym_off_keyword_token1] = ACTIONS(1773),
    [sym_send_level_keyword] = ACTIONS(1773),
    [sym_send_string_keyword] = ACTIONS(1773),
    [sym_send_command_keyword] = ACTIONS(1773),
    [sym_clear_buffer_keyword] = ACTIONS(1773),
    [sym_create_buffer_keyword] = ACTIONS(1773),
    [sym_create_multi_buffer_keyword] = ACTIONS(1773),
    [sym_call_keyword] = ACTIONS(1773),
    [sym_system_call_keyword] = ACTIONS(1773),
    [sym_devchan_to_keyword] = ACTIONS(1773),
    [sym_devchan_min_to_keyword] = ACTIONS(1773),
    [sym_devchan_total_off_keyword] = ACTIONS(1773),
    [sym_devchan_pulse_keyword] = ACTIONS(1773),
    [sym_wait_keyword] = ACTIONS(1773),
    [sym_cancel_wait_keyword] = ACTIONS(1773),
    [sym_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1773),
    [sym_not] = ACTIONS(1773),
    [sym_bnot] = ACTIONS(1773),
    [sym___file__] = ACTIONS(1773),
    [sym___date__] = ACTIONS(1773),
    [sym___ldate__] = ACTIONS(1773),
    [sym___time__] = ACTIONS(1773),
    [sym___line__] = ACTIONS(1773),
    [sym___name__] = ACTIONS(1773),
    [sym___netlinx__] = ACTIONS(1773),
    [sym_netlinx_axi_version] = ACTIONS(1773),
    [sym_url_flg_tcp] = ACTIONS(1773),
    [sym_url_flg_accinfopresent] = ACTIONS(1773),
    [sym_url_flg_temp] = ACTIONS(1773),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1773),
    [sym_url_flg_stat_mask] = ACTIONS(1773),
    [sym_url_flg_stat_lookup] = ACTIONS(1773),
    [sym_url_flg_stat_connecting] = ACTIONS(1773),
    [sym_url_flg_stat_waiting] = ACTIONS(1773),
    [sym_url_flg_stat_connected] = ACTIONS(1773),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1773),
    [sym_first_virtual_device] = ACTIONS(1773),
    [sym_first_local_port] = ACTIONS(1773),
    [sym_dynamic_virtual_device] = ACTIONS(1773),
    [sym_timeline_once] = ACTIONS(1773),
    [sym_timeline_repeat] = ACTIONS(1773),
    [sym_timeline_absolute] = ACTIONS(1773),
    [sym_timeline_relative] = ACTIONS(1773),
    [sym_do_push_timed_infinite] = ACTIONS(1773),
    [sym_source_type_no_address] = ACTIONS(1773),
    [sym_source_type_neuron_id] = ACTIONS(1773),
    [sym_source_type_ip_address] = ACTIONS(1773),
    [sym_source_type_axlink] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1773),
    [sym_source_type_ip_socket_address] = ACTIONS(1773),
    [sym_source_type_rs232] = ACTIONS(1773),
    [sym_source_type_internal] = ACTIONS(1773),
    [sym_source_type_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1773),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1773),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1773),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1773),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_file_read_only] = ACTIONS(1773),
    [sym_file_rw_new] = ACTIONS(1773),
    [sym_file_rw_append] = ACTIONS(1773),
    [sym_ip_tcp] = ACTIONS(1773),
    [sym_ip_udp] = ACTIONS(1773),
    [sym_ip_udp_2way] = ACTIONS(1773),
    [sym_xml_encode_types] = ACTIONS(1773),
    [sym_xml_encode_char_as_list] = ACTIONS(1773),
    [sym_xml_encode_le] = ACTIONS(1773),
    [sym_xml_decode_types] = ACTIONS(1773),
    [sym_xml_decode_no_preserve] = ACTIONS(1773),
    [sym_duet_dev_type_display_device] = ACTIONS(1773),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_io_device] = ACTIONS(1773),
    [sym_duet_dev_type_relay_device] = ACTIONS(1773),
    [sym_duet_dev_type_ups] = ACTIONS(1773),
    [sym_duet_dev_type_amplifier] = ACTIONS(1773),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1773),
    [sym_duet_dev_type_camera] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1773),
    [sym_duet_dev_type_dss] = ACTIONS(1773),
    [sym_duet_dev_type_dvr] = ACTIONS(1773),
    [sym_duet_dev_type_disc_device] = ACTIONS(1773),
    [sym_duet_dev_type_document_camera] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1773),
    [sym_duet_dev_type_hvac] = ACTIONS(1773),
    [sym_duet_dev_type_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_light] = ACTIONS(1773),
    [sym_duet_dev_type_monitor] = ACTIONS(1773),
    [sym_duet_dev_type_motor] = ACTIONS(1773),
    [sym_duet_dev_type_multi_window] = ACTIONS(1773),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1773),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1773),
    [sym_duet_dev_type_receiver] = ACTIONS(1773),
    [sym_duet_dev_type_security_system] = ACTIONS(1773),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1773),
    [sym_duet_dev_type_settop_box] = ACTIONS(1773),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1773),
    [sym_duet_dev_type_switcher] = ACTIONS(1773),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_tv] = ACTIONS(1773),
    [sym_duet_dev_type_utility] = ACTIONS(1773),
    [sym_duet_dev_type_vcr] = ACTIONS(1773),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_video_processor] = ACTIONS(1773),
    [sym_duet_dev_type_video_projector] = ACTIONS(1773),
    [sym_duet_dev_type_video_wall] = ACTIONS(1773),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1773),
    [sym_duet_dev_type_weather] = ACTIONS(1773),
    [sym_duet_dev_not_polled] = ACTIONS(1773),
    [sym_duet_dev_polled] = ACTIONS(1773),
    [sym_clkmgr_mode_network] = ACTIONS(1773),
    [sym_clkmgr_mode_standalone] = ACTIONS(1773),
    [sym_normal_standby] = ACTIONS(1773),
    [sym_normal_wake] = ACTIONS(1773),
    [sym_ip_multicast_ttl_option] = ACTIONS(1773),
    [sym_ip_ttl_subnet] = ACTIONS(1773),
    [sym_ip_ttl_site] = ACTIONS(1773),
    [sym_ip_ttl_region] = ACTIONS(1773),
    [sym_ip_ttl_continent] = ACTIONS(1773),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1773),
    [sym_ip_nodelay_on] = ACTIONS(1773),
    [sym_ip_nodelay_off] = ACTIONS(1773),
    [sym_amx_error] = ACTIONS(1773),
    [sym_amx_warning] = ACTIONS(1773),
    [sym_amx_info] = ACTIONS(1773),
    [sym_amx_debug] = ACTIONS(1773),
    [sym_netlinx_login_success] = ACTIONS(1773),
    [sym_netlinx_login_fail] = ACTIONS(1773),
    [sym_netlinx_logout] = ACTIONS(1773),
    [sym_null_str] = ACTIONS(1773),
    [sym_smtp_address] = ACTIONS(1773),
    [sym_smtp_port_number] = ACTIONS(1773),
    [sym_smtp_username] = ACTIONS(1773),
    [sym_smtp_password] = ACTIONS(1773),
    [sym_smtp_require_tls] = ACTIONS(1773),
    [sym_smtp_from] = ACTIONS(1773),
    [sym_smtp_tls_true] = ACTIONS(1773),
    [sym_smtp_tls_false] = ACTIONS(1773),
    [sym_listview_on_row_select_event] = ACTIONS(1773),
    [sym_data_structure_unknown] = ACTIONS(1773),
    [sym_data_structure_datafeed] = ACTIONS(1773),
    [sym_data_structure_datarecordset] = ACTIONS(1773),
    [sym_data_structure_datarecord] = ACTIONS(1773),
    [sym_data_structure_datafield] = ACTIONS(1773),
    [sym_data_max_content_field_count] = ACTIONS(1773),
    [sym_data_max_metadata_field_count] = ACTIONS(1773),
    [sym_data_max_id_length] = ACTIONS(1773),
    [sym_data_max_name_length] = ACTIONS(1773),
    [sym_data_max_description_length] = ACTIONS(1773),
    [sym_data_max_type_length] = ACTIONS(1773),
    [sym_data_max_format_length] = ACTIONS(1773),
    [sym_data_max_source_length] = ACTIONS(1773),
    [sym_data_max_value_length] = ACTIONS(1773),
    [sym_data_max_label_length] = ACTIONS(1773),
    [sym_data_max_url_length] = ACTIONS(1773),
    [sym_max_last_login_info_length] = ACTIONS(1773),
    [sym_data_type_unknown] = ACTIONS(1773),
    [sym_data_type_string] = ACTIONS(1773),
    [sym_data_type_datetime] = ACTIONS(1773),
    [sym_data_type_date] = ACTIONS(1773),
    [sym_data_type_time] = ACTIONS(1773),
    [sym_data_type_image] = ACTIONS(1773),
    [sym_data_format_url] = ACTIONS(1773),
    [sym_data_format_phone] = ACTIONS(1773),
    [sym_data_format_email] = ACTIONS(1773),
    [sym_data_format_iso8601] = ACTIONS(1773),
    [sym_auditlog_priv] = ACTIONS(1773),
    [sym_configuration_priv] = ACTIONS(1773),
    [sym_deviceconfig_priv] = ACTIONS(1773),
    [sym_ftp_priv] = ACTIONS(1773),
    [sym_http_priv] = ACTIONS(1773),
    [sym_networkconfig_priv] = ACTIONS(1773),
    [sym_programport_priv] = ACTIONS(1773),
    [sym_remoteui_priv] = ACTIONS(1773),
    [sym_securitycontrol_priv] = ACTIONS(1773),
    [sym_softwaremanagement_priv] = ACTIONS(1773),
    [sym_terminal_priv] = ACTIONS(1773),
    [sym_tpadmin_priv] = ACTIONS(1773),
    [sym_user1_priv] = ACTIONS(1773),
    [sym_user2_priv] = ACTIONS(1773),
    [sym_user3_priv] = ACTIONS(1773),
    [sym_user4_priv] = ACTIONS(1773),
    [sym_usermanagement_priv] = ACTIONS(1773),
    [sym_valid_account] = ACTIONS(1773),
    [sym_err_name_invalid] = ACTIONS(1773),
    [sym_err_password_invalid] = ACTIONS(1773),
    [sym_err_invalid_account] = ACTIONS(1773),
    [sym_err_no_authorization] = ACTIONS(1773),
    [sym_err_invalid_parameter] = ACTIONS(1773),
    [sym_err_user_locked_out] = ACTIONS(1773),
    [sym_err_user_account_expired] = ACTIONS(1773),
    [sym_err_general_error] = ACTIONS(1773),
    [sym_err_authentication_error] = ACTIONS(1773),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1773),
    [sym_tls_validate_certificate] = ACTIONS(1773),
    [sym_tdata] = ACTIONS(1773),
    [sym_tchannel] = ACTIONS(1773),
    [sym_tlevel] = ACTIONS(1773),
    [sym_tbutton] = ACTIONS(1773),
    [sym_ttimeline] = ACTIONS(1773),
    [sym_tcustom] = ACTIONS(1773),
    [sym_url_struct] = ACTIONS(1773),
    [sym_dns_struct] = ACTIONS(1773),
    [sym_ip_address_struct] = ACTIONS(1773),
    [sym_dev_info_struct] = ACTIONS(1773),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1773),
    [sym_data_feed] = ACTIONS(1773),
    [sym_data_field] = ACTIONS(1773),
    [sym_data_record] = ACTIONS(1773),
    [sym_wc_data_feed] = ACTIONS(1773),
    [sym_wc_data_field] = ACTIONS(1773),
    [sym_wc_data_record] = ACTIONS(1773),
    [sym_last_login_info] = ACTIONS(1773),
    [sym____reserved___] = ACTIONS(1773),
    [sym_date] = ACTIONS(1773),
    [sym_day] = ACTIONS(1773),
    [sym_time] = ACTIONS(1773),
    [sym_ldate] = ACTIONS(1773),
    [sym_push_device] = ACTIONS(1773),
    [sym_push_channel] = ACTIONS(1773),
    [sym_push_devchan] = ACTIONS(1773),
    [sym_release_device] = ACTIONS(1773),
    [sym_release_channel] = ACTIONS(1773),
    [sym_release_devchan] = ACTIONS(1773),
    [sym_master_slot] = ACTIONS(1773),
    [sym_get_pulse_time] = ACTIONS(1773),
    [sym_get_timer] = ACTIONS(1773),
    [sym_master_sn] = ACTIONS(1773),
    [sym_system_number] = ACTIONS(1773),
    [sym_button] = ACTIONS(1773),
    [sym_dv_channel] = ACTIONS(1773),
    [sym_data] = ACTIONS(1773),
    [sym_level] = ACTIONS(1773),
    [sym_timeline] = ACTIONS(1773),
    [sym_channel] = ACTIONS(1773),
    [sym_custom] = ACTIONS(1773),
    [sym_atoi] = ACTIONS(1773),
    [sym_atol] = ACTIONS(1773),
    [sym_atof] = ACTIONS(1773),
    [sym_itoa] = ACTIONS(1773),
    [sym_itohex] = ACTIONS(1773),
    [sym_hextoi] = ACTIONS(1773),
    [sym_ftoa] = ACTIONS(1773),
    [sym_find_string] = ACTIONS(1773),
    [sym_left_string] = ACTIONS(1773),
    [sym_right_string] = ACTIONS(1773),
    [sym_mid_string] = ACTIONS(1773),
    [sym_remove_string] = ACTIONS(1773),
    [sym_get_buffer_string] = ACTIONS(1773),
    [sym_get_buffer_char] = ACTIONS(1773),
    [sym_get_multi_buffer_string] = ACTIONS(1773),
    [sym_length_string] = ACTIONS(1773),
    [sym_max_length_string] = ACTIONS(1773),
    [sym_set_length_string] = ACTIONS(1773),
    [sym_compare_string] = ACTIONS(1773),
    [sym_upper_string] = ACTIONS(1773),
    [sym_lower_string] = ACTIONS(1773),
    [sym_get_last] = ACTIONS(1773),
    [sym_date_to_day] = ACTIONS(1773),
    [sym_date_to_month] = ACTIONS(1773),
    [sym_date_to_year] = ACTIONS(1773),
    [sym_time_to_hour] = ACTIONS(1773),
    [sym_time_to_minute] = ACTIONS(1773),
    [sym_time_to_second] = ACTIONS(1773),
    [sym_day_of_week] = ACTIONS(1773),
    [sym_set_pulse_time] = ACTIONS(1773),
    [sym_set_timer] = ACTIONS(1773),
    [sym_file_open] = ACTIONS(1773),
    [sym_file_close] = ACTIONS(1773),
    [sym_file_read] = ACTIONS(1773),
    [sym_file_read_line] = ACTIONS(1773),
    [sym_file_write] = ACTIONS(1773),
    [sym_file_write_line] = ACTIONS(1773),
    [sym_file_copy] = ACTIONS(1773),
    [sym_file_delete] = ACTIONS(1773),
    [sym_file_rename] = ACTIONS(1773),
    [sym_file_dir] = ACTIONS(1773),
    [sym_file_getdir] = ACTIONS(1773),
    [sym_file_setdir] = ACTIONS(1773),
    [sym_file_create_dir] = ACTIONS(1773),
    [sym_file_remove_dir] = ACTIONS(1773),
    [sym_file_seek] = ACTIONS(1773),
    [sym_length_array] = ACTIONS(1773),
    [sym_max_length_array] = ACTIONS(1773),
    [sym_set_length_array] = ACTIONS(1773),
    [sym_ip_client_open] = ACTIONS(1773),
    [sym_ip_client_close] = ACTIONS(1773),
    [sym_ip_server_open] = ACTIONS(1773),
    [sym_ip_server_close] = ACTIONS(1773),
    [sym_ip_bound_client_open] = ACTIONS(1773),
    [sym_ip_mc_server_open] = ACTIONS(1773),
    [sym_ip_set_option] = ACTIONS(1773),
    [sym_ssh_client_open] = ACTIONS(1773),
    [sym_ssh_client_close] = ACTIONS(1773),
    [sym_tls_client_open] = ACTIONS(1773),
    [sym_tls_client_close] = ACTIONS(1773),
    [sym_do_push] = ACTIONS(1773),
    [sym_do_push_timed] = ACTIONS(1773),
    [sym_do_release] = ACTIONS(1773),
    [sym_device_id] = ACTIONS(1773),
    [sym_device_info] = ACTIONS(1773),
    [sym_device_id_string] = ACTIONS(1773),
    [sym_device_standby] = ACTIONS(1773),
    [sym_device_wake] = ACTIONS(1773),
    [sym_timeline_create] = ACTIONS(1773),
    [sym_timeline_kill] = ACTIONS(1773),
    [sym_timeline_reload] = ACTIONS(1773),
    [sym_timeline_pause] = ACTIONS(1773),
    [sym_timeline_restart] = ACTIONS(1773),
    [sym_timeline_set] = ACTIONS(1773),
    [sym_timeline_get] = ACTIONS(1773),
    [sym_timeline_active] = ACTIONS(1773),
    [sym_timeline_dynamic_id] = ACTIONS(1773),
    [sym_variable_to_string] = ACTIONS(1773),
    [sym_string_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_string] = ACTIONS(1773),
    [sym_type_cast] = ACTIONS(1773),
    [sym_variable_to_xml] = ACTIONS(1773),
    [sym_xml_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_xml] = ACTIONS(1773),
    [sym_redirect_string] = ACTIONS(1773),
    [sym_random_number] = ACTIONS(1773),
    [sym_get_unique_id] = ACTIONS(1773),
    [sym_get_system_number] = ACTIONS(1773),
    [sym_set_system_number] = ACTIONS(1773),
    [sym_get_serial_number] = ACTIONS(1773),
    [sym_get_master_build] = ACTIONS(1773),
    [sym_set_validation_code] = ACTIONS(1773),
    [sym_reboot] = ACTIONS(1773),
    [sym_load_duet_module] = ACTIONS(1773),
    [sym_unload_duet_module] = ACTIONS(1773),
    [sym_duet_mem_size_set] = ACTIONS(1773),
    [sym_duet_mem_size_get] = ACTIONS(1773),
    [sym_dynamic_application_device] = ACTIONS(1773),
    [sym_dynamic_polled_port] = ACTIONS(1773),
    [sym_static_port_binding] = ACTIONS(1773),
    [sym_static_ip_binding] = ACTIONS(1773),
    [sym_get_dns_list] = ACTIONS(1773),
    [sym_set_dns_list] = ACTIONS(1773),
    [sym_get_ip_address] = ACTIONS(1773),
    [sym_set_ip_address] = ACTIONS(1773),
    [sym_get_url_list] = ACTIONS(1773),
    [sym_add_url_entry] = ACTIONS(1773),
    [sym_delete_url_entry] = ACTIONS(1773),
    [sym_abs_value] = ACTIONS(1773),
    [sym_max_value] = ACTIONS(1773),
    [sym_min_value] = ACTIONS(1773),
    [sym_exp_value] = ACTIONS(1773),
    [sym_log_value] = ACTIONS(1773),
    [sym_log10_value] = ACTIONS(1773),
    [sym_power_value] = ACTIONS(1773),
    [sym_sqrt_value] = ACTIONS(1773),
    [sym_raw_be] = ACTIONS(1773),
    [sym_raw_le] = ACTIONS(1773),
    [sym_internal_queue_size_set] = ACTIONS(1773),
    [sym_internal_queue_size_get] = ACTIONS(1773),
    [sym_internal_threshold_set] = ACTIONS(1773),
    [sym_internal_threshold_get] = ACTIONS(1773),
    [sym_rebuilt_event] = ACTIONS(1773),
    [sym_do_custom_event] = ACTIONS(1773),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1773),
    [sym_clkmgr_set_clk_source] = ACTIONS(1773),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1773),
    [sym_clkmgr_get_timezone] = ACTIONS(1773),
    [sym_clkmgr_set_timezone] = ACTIONS(1773),
    [sym_clkmgr_get_resync_period] = ACTIONS(1773),
    [sym_clkmgr_set_resync_period] = ACTIONS(1773),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_timeservers] = ACTIONS(1773),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_set_log_level] = ACTIONS(1773),
    [sym_get_log_level] = ACTIONS(1773),
    [sym_amx_log] = ACTIONS(1773),
    [sym_smtp_server_config_set] = ACTIONS(1773),
    [sym_smtp_server_config_get] = ACTIONS(1773),
    [sym_smtp_send] = ACTIONS(1773),
    [sym_validate_netlinx_account] = ACTIONS(1773),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1773),
    [sym_authenticate_certificate] = ACTIONS(1773),
    [sym_audit_netlinx_session_event] = ACTIONS(1773),
    [sym_audit_netlinx_generic_event] = ACTIONS(1773),
    [sym_get_available_flash_disk_space] = ACTIONS(1773),
    [sym_get_max_flash_disk_space] = ACTIONS(1773),
    [sym_data_create_feed] = ACTIONS(1773),
    [sym_data_delete_feed] = ACTIONS(1773),
    [sym_data_publish_feed] = ACTIONS(1773),
    [sym_data_get_published_feed] = ACTIONS(1773),
    [sym_data_add_record] = ACTIONS(1773),
    [sym_data_get_event_record] = ACTIONS(1773),
    [sym__wc_data_create_feed] = ACTIONS(1773),
    [sym__wc_data_add_record] = ACTIONS(1773),
    [sym__wc_data_get_event_record] = ACTIONS(1773),
    [sym_format] = ACTIONS(1773),
    [sym_astro_clock] = ACTIONS(1773),
    [sym_set_outdoor_temperature] = ACTIONS(1773),
    [sym_set_virtual_level_count] = ACTIONS(1773),
    [sym_set_virtual_channel_count] = ACTIONS(1773),
    [sym_set_virtual_port_count] = ACTIONS(1773),
    [sym__wc_encode_int] = ACTIONS(1773),
    [sym__wc_decode_int] = ACTIONS(1773),
    [sym__wc_int] = ACTIONS(1773),
    [sym__wc_to_ch_int] = ACTIONS(1773),
    [sym__ch_to_wc_int] = ACTIONS(1773),
    [sym__wc_find_string_int] = ACTIONS(1773),
    [sym__wc_left_string_int] = ACTIONS(1773),
    [sym__wc_lower_string_int] = ACTIONS(1773),
    [sym__wc_mid_string_int] = ACTIONS(1773),
    [sym__wc_remove_string_int] = ACTIONS(1773),
    [sym__wc_right_string_int] = ACTIONS(1773),
    [sym__wc_upper_string_int] = ACTIONS(1773),
    [sym__wc_compare_string_int] = ACTIONS(1773),
    [sym__wc_get_buffer_char_int] = ACTIONS(1773),
    [sym__wc_get_buffer_string_int] = ACTIONS(1773),
    [sym__wc_concat_string_int] = ACTIONS(1773),
    [sym__wc_file_open_int] = ACTIONS(1773),
    [sym__wc_file_close_int] = ACTIONS(1773),
    [sym__wc_file_read_int] = ACTIONS(1773),
    [sym__wc_file_read_line_int] = ACTIONS(1773),
    [sym__wc_file_write_int] = ACTIONS(1773),
    [sym__wc_file_write_line_int] = ACTIONS(1773),
    [aux_sym_preproc_include_token1] = ACTIONS(1773),
    [aux_sym_preproc_define_token1] = ACTIONS(1775),
    [aux_sym_preproc_warn_token1] = ACTIONS(1775),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1775),
    [anon_sym_LBRACK] = ACTIONS(1775),
    [anon_sym_LBRACE] = ACTIONS(1775),
    [anon_sym_LPAREN] = ACTIONS(1773),
    [anon_sym_BANG] = ACTIONS(1775),
    [anon_sym_TILDE] = ACTIONS(1775),
    [anon_sym_DASH] = ACTIONS(1773),
    [anon_sym_PLUS] = ACTIONS(1773),
    [anon_sym_DASH_DASH] = ACTIONS(1775),
    [anon_sym_PLUS_PLUS] = ACTIONS(1775),
    [anon_sym_DQUOTE] = ACTIONS(1775),
    [anon_sym_SQUOTE] = ACTIONS(1775),
    [sym_number_literal] = ACTIONS(1775),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(301)] = {
    [ts_builtin_sym_end] = ACTIONS(1775),
    [sym_identifier] = ACTIONS(1773),
    [sym_program_name_keyword] = ACTIONS(1773),
    [sym_module_name_keyword] = ACTIONS(1773),
    [sym_define_device_keyword] = ACTIONS(1773),
    [sym_define_combine_keyword] = ACTIONS(1773),
    [sym_define_constant_keyword] = ACTIONS(1773),
    [sym_define_type_keyword] = ACTIONS(1773),
    [sym_define_variable_keyword] = ACTIONS(1773),
    [sym_define_system_variable_keyword] = ACTIONS(1773),
    [sym_define_connect_level_keyword] = ACTIONS(1773),
    [sym_define_latching_keyword] = ACTIONS(1773),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1773),
    [sym_define_toggling_keyword] = ACTIONS(1773),
    [sym_define_call_keyword] = ACTIONS(1773),
    [sym_define_function_keyword] = ACTIONS(1773),
    [sym_define_library_function_keyword] = ACTIONS(1773),
    [sym_define_start_keyword] = ACTIONS(1773),
    [sym_define_event_keyword] = ACTIONS(1773),
    [sym_define_program_keyword] = ACTIONS(1773),
    [sym_define_module_keyword] = ACTIONS(1773),
    [sym_char_keyword] = ACTIONS(1773),
    [sym_widechar_keyword] = ACTIONS(1773),
    [sym_integer_keyword] = ACTIONS(1773),
    [sym_sinteger_keyword] = ACTIONS(1773),
    [sym_long_keyword] = ACTIONS(1773),
    [sym_slong_keyword] = ACTIONS(1773),
    [sym_float_keyword] = ACTIONS(1773),
    [sym_double_keyword] = ACTIONS(1773),
    [sym_variant_keyword] = ACTIONS(1773),
    [sym_variantarray_keyword] = ACTIONS(1773),
    [sym_local_var_keyword] = ACTIONS(1773),
    [sym_stack_var_keyword] = ACTIONS(1773),
    [sym_constant_keyword] = ACTIONS(1773),
    [sym_volatile_keyword] = ACTIONS(1773),
    [sym_non_volatile_keyword] = ACTIONS(1773),
    [sym_persistent_keyword] = ACTIONS(1773),
    [sym_dev_keyword] = ACTIONS(1773),
    [sym_devlev_keyword] = ACTIONS(1773),
    [sym_devchan_keyword] = ACTIONS(1773),
    [sym_if_keyword] = ACTIONS(1773),
    [sym_else_keyword] = ACTIONS(1773),
    [sym_switch_keyword] = ACTIONS(1773),
    [sym_case_keyword] = ACTIONS(1773),
    [sym_default_keyword] = ACTIONS(1773),
    [sym_while_keyword] = ACTIONS(1773),
    [sym_for_keyword] = ACTIONS(1773),
    [sym_break_keyword] = ACTIONS(1773),
    [sym_continue_keyword] = ACTIONS(1773),
    [sym_return_keyword] = ACTIONS(1773),
    [sym_select_keyword] = ACTIONS(1773),
    [sym_active_keyword] = ACTIONS(1773),
    [sym_struct_keyword] = ACTIONS(1773),
    [sym_structure_keyword] = ACTIONS(1773),
    [sym_button_event_keyword] = ACTIONS(1773),
    [sym_channel_event_keyword] = ACTIONS(1773),
    [sym_level_event_keyword] = ACTIONS(1773),
    [sym_data_event_keyword] = ACTIONS(1773),
    [sym_timeline_event_keyword] = ACTIONS(1773),
    [sym_custom_event_keyword] = ACTIONS(1773),
    [sym_push_keyword] = ACTIONS(1773),
    [sym_release_keyword] = ACTIONS(1773),
    [aux_sym_on_keyword_token1] = ACTIONS(1773),
    [aux_sym_off_keyword_token1] = ACTIONS(1773),
    [sym_send_level_keyword] = ACTIONS(1773),
    [sym_send_string_keyword] = ACTIONS(1773),
    [sym_send_command_keyword] = ACTIONS(1773),
    [sym_clear_buffer_keyword] = ACTIONS(1773),
    [sym_create_buffer_keyword] = ACTIONS(1773),
    [sym_create_multi_buffer_keyword] = ACTIONS(1773),
    [sym_call_keyword] = ACTIONS(1773),
    [sym_system_call_keyword] = ACTIONS(1773),
    [sym_devchan_to_keyword] = ACTIONS(1773),
    [sym_devchan_min_to_keyword] = ACTIONS(1773),
    [sym_devchan_total_off_keyword] = ACTIONS(1773),
    [sym_devchan_pulse_keyword] = ACTIONS(1773),
    [sym_wait_keyword] = ACTIONS(1773),
    [sym_cancel_wait_keyword] = ACTIONS(1773),
    [sym_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1773),
    [sym_not] = ACTIONS(1773),
    [sym_bnot] = ACTIONS(1773),
    [sym___file__] = ACTIONS(1773),
    [sym___date__] = ACTIONS(1773),
    [sym___ldate__] = ACTIONS(1773),
    [sym___time__] = ACTIONS(1773),
    [sym___line__] = ACTIONS(1773),
    [sym___name__] = ACTIONS(1773),
    [sym___netlinx__] = ACTIONS(1773),
    [sym_netlinx_axi_version] = ACTIONS(1773),
    [sym_url_flg_tcp] = ACTIONS(1773),
    [sym_url_flg_accinfopresent] = ACTIONS(1773),
    [sym_url_flg_temp] = ACTIONS(1773),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1773),
    [sym_url_flg_stat_mask] = ACTIONS(1773),
    [sym_url_flg_stat_lookup] = ACTIONS(1773),
    [sym_url_flg_stat_connecting] = ACTIONS(1773),
    [sym_url_flg_stat_waiting] = ACTIONS(1773),
    [sym_url_flg_stat_connected] = ACTIONS(1773),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1773),
    [sym_first_virtual_device] = ACTIONS(1773),
    [sym_first_local_port] = ACTIONS(1773),
    [sym_dynamic_virtual_device] = ACTIONS(1773),
    [sym_timeline_once] = ACTIONS(1773),
    [sym_timeline_repeat] = ACTIONS(1773),
    [sym_timeline_absolute] = ACTIONS(1773),
    [sym_timeline_relative] = ACTIONS(1773),
    [sym_do_push_timed_infinite] = ACTIONS(1773),
    [sym_source_type_no_address] = ACTIONS(1773),
    [sym_source_type_neuron_id] = ACTIONS(1773),
    [sym_source_type_ip_address] = ACTIONS(1773),
    [sym_source_type_axlink] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1773),
    [sym_source_type_ip_socket_address] = ACTIONS(1773),
    [sym_source_type_rs232] = ACTIONS(1773),
    [sym_source_type_internal] = ACTIONS(1773),
    [sym_source_type_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1773),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1773),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1773),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1773),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_file_read_only] = ACTIONS(1773),
    [sym_file_rw_new] = ACTIONS(1773),
    [sym_file_rw_append] = ACTIONS(1773),
    [sym_ip_tcp] = ACTIONS(1773),
    [sym_ip_udp] = ACTIONS(1773),
    [sym_ip_udp_2way] = ACTIONS(1773),
    [sym_xml_encode_types] = ACTIONS(1773),
    [sym_xml_encode_char_as_list] = ACTIONS(1773),
    [sym_xml_encode_le] = ACTIONS(1773),
    [sym_xml_decode_types] = ACTIONS(1773),
    [sym_xml_decode_no_preserve] = ACTIONS(1773),
    [sym_duet_dev_type_display_device] = ACTIONS(1773),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_io_device] = ACTIONS(1773),
    [sym_duet_dev_type_relay_device] = ACTIONS(1773),
    [sym_duet_dev_type_ups] = ACTIONS(1773),
    [sym_duet_dev_type_amplifier] = ACTIONS(1773),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1773),
    [sym_duet_dev_type_camera] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1773),
    [sym_duet_dev_type_dss] = ACTIONS(1773),
    [sym_duet_dev_type_dvr] = ACTIONS(1773),
    [sym_duet_dev_type_disc_device] = ACTIONS(1773),
    [sym_duet_dev_type_document_camera] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1773),
    [sym_duet_dev_type_hvac] = ACTIONS(1773),
    [sym_duet_dev_type_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_light] = ACTIONS(1773),
    [sym_duet_dev_type_monitor] = ACTIONS(1773),
    [sym_duet_dev_type_motor] = ACTIONS(1773),
    [sym_duet_dev_type_multi_window] = ACTIONS(1773),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1773),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1773),
    [sym_duet_dev_type_receiver] = ACTIONS(1773),
    [sym_duet_dev_type_security_system] = ACTIONS(1773),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1773),
    [sym_duet_dev_type_settop_box] = ACTIONS(1773),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1773),
    [sym_duet_dev_type_switcher] = ACTIONS(1773),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_tv] = ACTIONS(1773),
    [sym_duet_dev_type_utility] = ACTIONS(1773),
    [sym_duet_dev_type_vcr] = ACTIONS(1773),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_video_processor] = ACTIONS(1773),
    [sym_duet_dev_type_video_projector] = ACTIONS(1773),
    [sym_duet_dev_type_video_wall] = ACTIONS(1773),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1773),
    [sym_duet_dev_type_weather] = ACTIONS(1773),
    [sym_duet_dev_not_polled] = ACTIONS(1773),
    [sym_duet_dev_polled] = ACTIONS(1773),
    [sym_clkmgr_mode_network] = ACTIONS(1773),
    [sym_clkmgr_mode_standalone] = ACTIONS(1773),
    [sym_normal_standby] = ACTIONS(1773),
    [sym_normal_wake] = ACTIONS(1773),
    [sym_ip_multicast_ttl_option] = ACTIONS(1773),
    [sym_ip_ttl_subnet] = ACTIONS(1773),
    [sym_ip_ttl_site] = ACTIONS(1773),
    [sym_ip_ttl_region] = ACTIONS(1773),
    [sym_ip_ttl_continent] = ACTIONS(1773),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1773),
    [sym_ip_nodelay_on] = ACTIONS(1773),
    [sym_ip_nodelay_off] = ACTIONS(1773),
    [sym_amx_error] = ACTIONS(1773),
    [sym_amx_warning] = ACTIONS(1773),
    [sym_amx_info] = ACTIONS(1773),
    [sym_amx_debug] = ACTIONS(1773),
    [sym_netlinx_login_success] = ACTIONS(1773),
    [sym_netlinx_login_fail] = ACTIONS(1773),
    [sym_netlinx_logout] = ACTIONS(1773),
    [sym_null_str] = ACTIONS(1773),
    [sym_smtp_address] = ACTIONS(1773),
    [sym_smtp_port_number] = ACTIONS(1773),
    [sym_smtp_username] = ACTIONS(1773),
    [sym_smtp_password] = ACTIONS(1773),
    [sym_smtp_require_tls] = ACTIONS(1773),
    [sym_smtp_from] = ACTIONS(1773),
    [sym_smtp_tls_true] = ACTIONS(1773),
    [sym_smtp_tls_false] = ACTIONS(1773),
    [sym_listview_on_row_select_event] = ACTIONS(1773),
    [sym_data_structure_unknown] = ACTIONS(1773),
    [sym_data_structure_datafeed] = ACTIONS(1773),
    [sym_data_structure_datarecordset] = ACTIONS(1773),
    [sym_data_structure_datarecord] = ACTIONS(1773),
    [sym_data_structure_datafield] = ACTIONS(1773),
    [sym_data_max_content_field_count] = ACTIONS(1773),
    [sym_data_max_metadata_field_count] = ACTIONS(1773),
    [sym_data_max_id_length] = ACTIONS(1773),
    [sym_data_max_name_length] = ACTIONS(1773),
    [sym_data_max_description_length] = ACTIONS(1773),
    [sym_data_max_type_length] = ACTIONS(1773),
    [sym_data_max_format_length] = ACTIONS(1773),
    [sym_data_max_source_length] = ACTIONS(1773),
    [sym_data_max_value_length] = ACTIONS(1773),
    [sym_data_max_label_length] = ACTIONS(1773),
    [sym_data_max_url_length] = ACTIONS(1773),
    [sym_max_last_login_info_length] = ACTIONS(1773),
    [sym_data_type_unknown] = ACTIONS(1773),
    [sym_data_type_string] = ACTIONS(1773),
    [sym_data_type_datetime] = ACTIONS(1773),
    [sym_data_type_date] = ACTIONS(1773),
    [sym_data_type_time] = ACTIONS(1773),
    [sym_data_type_image] = ACTIONS(1773),
    [sym_data_format_url] = ACTIONS(1773),
    [sym_data_format_phone] = ACTIONS(1773),
    [sym_data_format_email] = ACTIONS(1773),
    [sym_data_format_iso8601] = ACTIONS(1773),
    [sym_auditlog_priv] = ACTIONS(1773),
    [sym_configuration_priv] = ACTIONS(1773),
    [sym_deviceconfig_priv] = ACTIONS(1773),
    [sym_ftp_priv] = ACTIONS(1773),
    [sym_http_priv] = ACTIONS(1773),
    [sym_networkconfig_priv] = ACTIONS(1773),
    [sym_programport_priv] = ACTIONS(1773),
    [sym_remoteui_priv] = ACTIONS(1773),
    [sym_securitycontrol_priv] = ACTIONS(1773),
    [sym_softwaremanagement_priv] = ACTIONS(1773),
    [sym_terminal_priv] = ACTIONS(1773),
    [sym_tpadmin_priv] = ACTIONS(1773),
    [sym_user1_priv] = ACTIONS(1773),
    [sym_user2_priv] = ACTIONS(1773),
    [sym_user3_priv] = ACTIONS(1773),
    [sym_user4_priv] = ACTIONS(1773),
    [sym_usermanagement_priv] = ACTIONS(1773),
    [sym_valid_account] = ACTIONS(1773),
    [sym_err_name_invalid] = ACTIONS(1773),
    [sym_err_password_invalid] = ACTIONS(1773),
    [sym_err_invalid_account] = ACTIONS(1773),
    [sym_err_no_authorization] = ACTIONS(1773),
    [sym_err_invalid_parameter] = ACTIONS(1773),
    [sym_err_user_locked_out] = ACTIONS(1773),
    [sym_err_user_account_expired] = ACTIONS(1773),
    [sym_err_general_error] = ACTIONS(1773),
    [sym_err_authentication_error] = ACTIONS(1773),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1773),
    [sym_tls_validate_certificate] = ACTIONS(1773),
    [sym_tdata] = ACTIONS(1773),
    [sym_tchannel] = ACTIONS(1773),
    [sym_tlevel] = ACTIONS(1773),
    [sym_tbutton] = ACTIONS(1773),
    [sym_ttimeline] = ACTIONS(1773),
    [sym_tcustom] = ACTIONS(1773),
    [sym_url_struct] = ACTIONS(1773),
    [sym_dns_struct] = ACTIONS(1773),
    [sym_ip_address_struct] = ACTIONS(1773),
    [sym_dev_info_struct] = ACTIONS(1773),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1773),
    [sym_data_feed] = ACTIONS(1773),
    [sym_data_field] = ACTIONS(1773),
    [sym_data_record] = ACTIONS(1773),
    [sym_wc_data_feed] = ACTIONS(1773),
    [sym_wc_data_field] = ACTIONS(1773),
    [sym_wc_data_record] = ACTIONS(1773),
    [sym_last_login_info] = ACTIONS(1773),
    [sym____reserved___] = ACTIONS(1773),
    [sym_date] = ACTIONS(1773),
    [sym_day] = ACTIONS(1773),
    [sym_time] = ACTIONS(1773),
    [sym_ldate] = ACTIONS(1773),
    [sym_push_device] = ACTIONS(1773),
    [sym_push_channel] = ACTIONS(1773),
    [sym_push_devchan] = ACTIONS(1773),
    [sym_release_device] = ACTIONS(1773),
    [sym_release_channel] = ACTIONS(1773),
    [sym_release_devchan] = ACTIONS(1773),
    [sym_master_slot] = ACTIONS(1773),
    [sym_get_pulse_time] = ACTIONS(1773),
    [sym_get_timer] = ACTIONS(1773),
    [sym_master_sn] = ACTIONS(1773),
    [sym_system_number] = ACTIONS(1773),
    [sym_button] = ACTIONS(1773),
    [sym_dv_channel] = ACTIONS(1773),
    [sym_data] = ACTIONS(1773),
    [sym_level] = ACTIONS(1773),
    [sym_timeline] = ACTIONS(1773),
    [sym_channel] = ACTIONS(1773),
    [sym_custom] = ACTIONS(1773),
    [sym_atoi] = ACTIONS(1773),
    [sym_atol] = ACTIONS(1773),
    [sym_atof] = ACTIONS(1773),
    [sym_itoa] = ACTIONS(1773),
    [sym_itohex] = ACTIONS(1773),
    [sym_hextoi] = ACTIONS(1773),
    [sym_ftoa] = ACTIONS(1773),
    [sym_find_string] = ACTIONS(1773),
    [sym_left_string] = ACTIONS(1773),
    [sym_right_string] = ACTIONS(1773),
    [sym_mid_string] = ACTIONS(1773),
    [sym_remove_string] = ACTIONS(1773),
    [sym_get_buffer_string] = ACTIONS(1773),
    [sym_get_buffer_char] = ACTIONS(1773),
    [sym_get_multi_buffer_string] = ACTIONS(1773),
    [sym_length_string] = ACTIONS(1773),
    [sym_max_length_string] = ACTIONS(1773),
    [sym_set_length_string] = ACTIONS(1773),
    [sym_compare_string] = ACTIONS(1773),
    [sym_upper_string] = ACTIONS(1773),
    [sym_lower_string] = ACTIONS(1773),
    [sym_get_last] = ACTIONS(1773),
    [sym_date_to_day] = ACTIONS(1773),
    [sym_date_to_month] = ACTIONS(1773),
    [sym_date_to_year] = ACTIONS(1773),
    [sym_time_to_hour] = ACTIONS(1773),
    [sym_time_to_minute] = ACTIONS(1773),
    [sym_time_to_second] = ACTIONS(1773),
    [sym_day_of_week] = ACTIONS(1773),
    [sym_set_pulse_time] = ACTIONS(1773),
    [sym_set_timer] = ACTIONS(1773),
    [sym_file_open] = ACTIONS(1773),
    [sym_file_close] = ACTIONS(1773),
    [sym_file_read] = ACTIONS(1773),
    [sym_file_read_line] = ACTIONS(1773),
    [sym_file_write] = ACTIONS(1773),
    [sym_file_write_line] = ACTIONS(1773),
    [sym_file_copy] = ACTIONS(1773),
    [sym_file_delete] = ACTIONS(1773),
    [sym_file_rename] = ACTIONS(1773),
    [sym_file_dir] = ACTIONS(1773),
    [sym_file_getdir] = ACTIONS(1773),
    [sym_file_setdir] = ACTIONS(1773),
    [sym_file_create_dir] = ACTIONS(1773),
    [sym_file_remove_dir] = ACTIONS(1773),
    [sym_file_seek] = ACTIONS(1773),
    [sym_length_array] = ACTIONS(1773),
    [sym_max_length_array] = ACTIONS(1773),
    [sym_set_length_array] = ACTIONS(1773),
    [sym_ip_client_open] = ACTIONS(1773),
    [sym_ip_client_close] = ACTIONS(1773),
    [sym_ip_server_open] = ACTIONS(1773),
    [sym_ip_server_close] = ACTIONS(1773),
    [sym_ip_bound_client_open] = ACTIONS(1773),
    [sym_ip_mc_server_open] = ACTIONS(1773),
    [sym_ip_set_option] = ACTIONS(1773),
    [sym_ssh_client_open] = ACTIONS(1773),
    [sym_ssh_client_close] = ACTIONS(1773),
    [sym_tls_client_open] = ACTIONS(1773),
    [sym_tls_client_close] = ACTIONS(1773),
    [sym_do_push] = ACTIONS(1773),
    [sym_do_push_timed] = ACTIONS(1773),
    [sym_do_release] = ACTIONS(1773),
    [sym_device_id] = ACTIONS(1773),
    [sym_device_info] = ACTIONS(1773),
    [sym_device_id_string] = ACTIONS(1773),
    [sym_device_standby] = ACTIONS(1773),
    [sym_device_wake] = ACTIONS(1773),
    [sym_timeline_create] = ACTIONS(1773),
    [sym_timeline_kill] = ACTIONS(1773),
    [sym_timeline_reload] = ACTIONS(1773),
    [sym_timeline_pause] = ACTIONS(1773),
    [sym_timeline_restart] = ACTIONS(1773),
    [sym_timeline_set] = ACTIONS(1773),
    [sym_timeline_get] = ACTIONS(1773),
    [sym_timeline_active] = ACTIONS(1773),
    [sym_timeline_dynamic_id] = ACTIONS(1773),
    [sym_variable_to_string] = ACTIONS(1773),
    [sym_string_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_string] = ACTIONS(1773),
    [sym_type_cast] = ACTIONS(1773),
    [sym_variable_to_xml] = ACTIONS(1773),
    [sym_xml_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_xml] = ACTIONS(1773),
    [sym_redirect_string] = ACTIONS(1773),
    [sym_random_number] = ACTIONS(1773),
    [sym_get_unique_id] = ACTIONS(1773),
    [sym_get_system_number] = ACTIONS(1773),
    [sym_set_system_number] = ACTIONS(1773),
    [sym_get_serial_number] = ACTIONS(1773),
    [sym_get_master_build] = ACTIONS(1773),
    [sym_set_validation_code] = ACTIONS(1773),
    [sym_reboot] = ACTIONS(1773),
    [sym_load_duet_module] = ACTIONS(1773),
    [sym_unload_duet_module] = ACTIONS(1773),
    [sym_duet_mem_size_set] = ACTIONS(1773),
    [sym_duet_mem_size_get] = ACTIONS(1773),
    [sym_dynamic_application_device] = ACTIONS(1773),
    [sym_dynamic_polled_port] = ACTIONS(1773),
    [sym_static_port_binding] = ACTIONS(1773),
    [sym_static_ip_binding] = ACTIONS(1773),
    [sym_get_dns_list] = ACTIONS(1773),
    [sym_set_dns_list] = ACTIONS(1773),
    [sym_get_ip_address] = ACTIONS(1773),
    [sym_set_ip_address] = ACTIONS(1773),
    [sym_get_url_list] = ACTIONS(1773),
    [sym_add_url_entry] = ACTIONS(1773),
    [sym_delete_url_entry] = ACTIONS(1773),
    [sym_abs_value] = ACTIONS(1773),
    [sym_max_value] = ACTIONS(1773),
    [sym_min_value] = ACTIONS(1773),
    [sym_exp_value] = ACTIONS(1773),
    [sym_log_value] = ACTIONS(1773),
    [sym_log10_value] = ACTIONS(1773),
    [sym_power_value] = ACTIONS(1773),
    [sym_sqrt_value] = ACTIONS(1773),
    [sym_raw_be] = ACTIONS(1773),
    [sym_raw_le] = ACTIONS(1773),
    [sym_internal_queue_size_set] = ACTIONS(1773),
    [sym_internal_queue_size_get] = ACTIONS(1773),
    [sym_internal_threshold_set] = ACTIONS(1773),
    [sym_internal_threshold_get] = ACTIONS(1773),
    [sym_rebuilt_event] = ACTIONS(1773),
    [sym_do_custom_event] = ACTIONS(1773),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1773),
    [sym_clkmgr_set_clk_source] = ACTIONS(1773),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1773),
    [sym_clkmgr_get_timezone] = ACTIONS(1773),
    [sym_clkmgr_set_timezone] = ACTIONS(1773),
    [sym_clkmgr_get_resync_period] = ACTIONS(1773),
    [sym_clkmgr_set_resync_period] = ACTIONS(1773),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_timeservers] = ACTIONS(1773),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_set_log_level] = ACTIONS(1773),
    [sym_get_log_level] = ACTIONS(1773),
    [sym_amx_log] = ACTIONS(1773),
    [sym_smtp_server_config_set] = ACTIONS(1773),
    [sym_smtp_server_config_get] = ACTIONS(1773),
    [sym_smtp_send] = ACTIONS(1773),
    [sym_validate_netlinx_account] = ACTIONS(1773),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1773),
    [sym_authenticate_certificate] = ACTIONS(1773),
    [sym_audit_netlinx_session_event] = ACTIONS(1773),
    [sym_audit_netlinx_generic_event] = ACTIONS(1773),
    [sym_get_available_flash_disk_space] = ACTIONS(1773),
    [sym_get_max_flash_disk_space] = ACTIONS(1773),
    [sym_data_create_feed] = ACTIONS(1773),
    [sym_data_delete_feed] = ACTIONS(1773),
    [sym_data_publish_feed] = ACTIONS(1773),
    [sym_data_get_published_feed] = ACTIONS(1773),
    [sym_data_add_record] = ACTIONS(1773),
    [sym_data_get_event_record] = ACTIONS(1773),
    [sym__wc_data_create_feed] = ACTIONS(1773),
    [sym__wc_data_add_record] = ACTIONS(1773),
    [sym__wc_data_get_event_record] = ACTIONS(1773),
    [sym_format] = ACTIONS(1773),
    [sym_astro_clock] = ACTIONS(1773),
    [sym_set_outdoor_temperature] = ACTIONS(1773),
    [sym_set_virtual_level_count] = ACTIONS(1773),
    [sym_set_virtual_channel_count] = ACTIONS(1773),
    [sym_set_virtual_port_count] = ACTIONS(1773),
    [sym__wc_encode_int] = ACTIONS(1773),
    [sym__wc_decode_int] = ACTIONS(1773),
    [sym__wc_int] = ACTIONS(1773),
    [sym__wc_to_ch_int] = ACTIONS(1773),
    [sym__ch_to_wc_int] = ACTIONS(1773),
    [sym__wc_find_string_int] = ACTIONS(1773),
    [sym__wc_left_string_int] = ACTIONS(1773),
    [sym__wc_lower_string_int] = ACTIONS(1773),
    [sym__wc_mid_string_int] = ACTIONS(1773),
    [sym__wc_remove_string_int] = ACTIONS(1773),
    [sym__wc_right_string_int] = ACTIONS(1773),
    [sym__wc_upper_string_int] = ACTIONS(1773),
    [sym__wc_compare_string_int] = ACTIONS(1773),
    [sym__wc_get_buffer_char_int] = ACTIONS(1773),
    [sym__wc_get_buffer_string_int] = ACTIONS(1773),
    [sym__wc_concat_string_int] = ACTIONS(1773),
    [sym__wc_file_open_int] = ACTIONS(1773),
    [sym__wc_file_close_int] = ACTIONS(1773),
    [sym__wc_file_read_int] = ACTIONS(1773),
    [sym__wc_file_read_line_int] = ACTIONS(1773),
    [sym__wc_file_write_int] = ACTIONS(1773),
    [sym__wc_file_write_line_int] = ACTIONS(1773),
    [aux_sym_preproc_include_token1] = ACTIONS(1773),
    [aux_sym_preproc_define_token1] = ACTIONS(1775),
    [aux_sym_preproc_warn_token1] = ACTIONS(1775),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1775),
    [anon_sym_LBRACK] = ACTIONS(1775),
    [anon_sym_LBRACE] = ACTIONS(1775),
    [anon_sym_LPAREN] = ACTIONS(1773),
    [anon_sym_BANG] = ACTIONS(1775),
    [anon_sym_TILDE] = ACTIONS(1775),
    [anon_sym_DASH] = ACTIONS(1773),
    [anon_sym_PLUS] = ACTIONS(1773),
    [anon_sym_DASH_DASH] = ACTIONS(1775),
    [anon_sym_PLUS_PLUS] = ACTIONS(1775),
    [anon_sym_DQUOTE] = ACTIONS(1775),
    [anon_sym_SQUOTE] = ACTIONS(1775),
    [sym_number_literal] = ACTIONS(1775),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(302)] = {
    [ts_builtin_sym_end] = ACTIONS(1775),
    [sym_identifier] = ACTIONS(1773),
    [sym_program_name_keyword] = ACTIONS(1773),
    [sym_module_name_keyword] = ACTIONS(1773),
    [sym_define_device_keyword] = ACTIONS(1773),
    [sym_define_combine_keyword] = ACTIONS(1773),
    [sym_define_constant_keyword] = ACTIONS(1773),
    [sym_define_type_keyword] = ACTIONS(1773),
    [sym_define_variable_keyword] = ACTIONS(1773),
    [sym_define_system_variable_keyword] = ACTIONS(1773),
    [sym_define_connect_level_keyword] = ACTIONS(1773),
    [sym_define_latching_keyword] = ACTIONS(1773),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1773),
    [sym_define_toggling_keyword] = ACTIONS(1773),
    [sym_define_call_keyword] = ACTIONS(1773),
    [sym_define_function_keyword] = ACTIONS(1773),
    [sym_define_library_function_keyword] = ACTIONS(1773),
    [sym_define_start_keyword] = ACTIONS(1773),
    [sym_define_event_keyword] = ACTIONS(1773),
    [sym_define_program_keyword] = ACTIONS(1773),
    [sym_define_module_keyword] = ACTIONS(1773),
    [sym_char_keyword] = ACTIONS(1773),
    [sym_widechar_keyword] = ACTIONS(1773),
    [sym_integer_keyword] = ACTIONS(1773),
    [sym_sinteger_keyword] = ACTIONS(1773),
    [sym_long_keyword] = ACTIONS(1773),
    [sym_slong_keyword] = ACTIONS(1773),
    [sym_float_keyword] = ACTIONS(1773),
    [sym_double_keyword] = ACTIONS(1773),
    [sym_variant_keyword] = ACTIONS(1773),
    [sym_variantarray_keyword] = ACTIONS(1773),
    [sym_local_var_keyword] = ACTIONS(1773),
    [sym_stack_var_keyword] = ACTIONS(1773),
    [sym_constant_keyword] = ACTIONS(1773),
    [sym_volatile_keyword] = ACTIONS(1773),
    [sym_non_volatile_keyword] = ACTIONS(1773),
    [sym_persistent_keyword] = ACTIONS(1773),
    [sym_dev_keyword] = ACTIONS(1773),
    [sym_devlev_keyword] = ACTIONS(1773),
    [sym_devchan_keyword] = ACTIONS(1773),
    [sym_if_keyword] = ACTIONS(1773),
    [sym_else_keyword] = ACTIONS(1773),
    [sym_switch_keyword] = ACTIONS(1773),
    [sym_case_keyword] = ACTIONS(1773),
    [sym_default_keyword] = ACTIONS(1773),
    [sym_while_keyword] = ACTIONS(1773),
    [sym_for_keyword] = ACTIONS(1773),
    [sym_break_keyword] = ACTIONS(1773),
    [sym_continue_keyword] = ACTIONS(1773),
    [sym_return_keyword] = ACTIONS(1773),
    [sym_select_keyword] = ACTIONS(1773),
    [sym_active_keyword] = ACTIONS(1773),
    [sym_struct_keyword] = ACTIONS(1773),
    [sym_structure_keyword] = ACTIONS(1773),
    [sym_button_event_keyword] = ACTIONS(1773),
    [sym_channel_event_keyword] = ACTIONS(1773),
    [sym_level_event_keyword] = ACTIONS(1773),
    [sym_data_event_keyword] = ACTIONS(1773),
    [sym_timeline_event_keyword] = ACTIONS(1773),
    [sym_custom_event_keyword] = ACTIONS(1773),
    [sym_push_keyword] = ACTIONS(1773),
    [sym_release_keyword] = ACTIONS(1773),
    [aux_sym_on_keyword_token1] = ACTIONS(1773),
    [aux_sym_off_keyword_token1] = ACTIONS(1773),
    [sym_send_level_keyword] = ACTIONS(1773),
    [sym_send_string_keyword] = ACTIONS(1773),
    [sym_send_command_keyword] = ACTIONS(1773),
    [sym_clear_buffer_keyword] = ACTIONS(1773),
    [sym_create_buffer_keyword] = ACTIONS(1773),
    [sym_create_multi_buffer_keyword] = ACTIONS(1773),
    [sym_call_keyword] = ACTIONS(1773),
    [sym_system_call_keyword] = ACTIONS(1773),
    [sym_devchan_to_keyword] = ACTIONS(1773),
    [sym_devchan_min_to_keyword] = ACTIONS(1773),
    [sym_devchan_total_off_keyword] = ACTIONS(1773),
    [sym_devchan_pulse_keyword] = ACTIONS(1773),
    [sym_wait_keyword] = ACTIONS(1773),
    [sym_cancel_wait_keyword] = ACTIONS(1773),
    [sym_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1773),
    [sym_not] = ACTIONS(1773),
    [sym_bnot] = ACTIONS(1773),
    [sym___file__] = ACTIONS(1773),
    [sym___date__] = ACTIONS(1773),
    [sym___ldate__] = ACTIONS(1773),
    [sym___time__] = ACTIONS(1773),
    [sym___line__] = ACTIONS(1773),
    [sym___name__] = ACTIONS(1773),
    [sym___netlinx__] = ACTIONS(1773),
    [sym_netlinx_axi_version] = ACTIONS(1773),
    [sym_url_flg_tcp] = ACTIONS(1773),
    [sym_url_flg_accinfopresent] = ACTIONS(1773),
    [sym_url_flg_temp] = ACTIONS(1773),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1773),
    [sym_url_flg_stat_mask] = ACTIONS(1773),
    [sym_url_flg_stat_lookup] = ACTIONS(1773),
    [sym_url_flg_stat_connecting] = ACTIONS(1773),
    [sym_url_flg_stat_waiting] = ACTIONS(1773),
    [sym_url_flg_stat_connected] = ACTIONS(1773),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1773),
    [sym_first_virtual_device] = ACTIONS(1773),
    [sym_first_local_port] = ACTIONS(1773),
    [sym_dynamic_virtual_device] = ACTIONS(1773),
    [sym_timeline_once] = ACTIONS(1773),
    [sym_timeline_repeat] = ACTIONS(1773),
    [sym_timeline_absolute] = ACTIONS(1773),
    [sym_timeline_relative] = ACTIONS(1773),
    [sym_do_push_timed_infinite] = ACTIONS(1773),
    [sym_source_type_no_address] = ACTIONS(1773),
    [sym_source_type_neuron_id] = ACTIONS(1773),
    [sym_source_type_ip_address] = ACTIONS(1773),
    [sym_source_type_axlink] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1773),
    [sym_source_type_ip_socket_address] = ACTIONS(1773),
    [sym_source_type_rs232] = ACTIONS(1773),
    [sym_source_type_internal] = ACTIONS(1773),
    [sym_source_type_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1773),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1773),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1773),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1773),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_file_read_only] = ACTIONS(1773),
    [sym_file_rw_new] = ACTIONS(1773),
    [sym_file_rw_append] = ACTIONS(1773),
    [sym_ip_tcp] = ACTIONS(1773),
    [sym_ip_udp] = ACTIONS(1773),
    [sym_ip_udp_2way] = ACTIONS(1773),
    [sym_xml_encode_types] = ACTIONS(1773),
    [sym_xml_encode_char_as_list] = ACTIONS(1773),
    [sym_xml_encode_le] = ACTIONS(1773),
    [sym_xml_decode_types] = ACTIONS(1773),
    [sym_xml_decode_no_preserve] = ACTIONS(1773),
    [sym_duet_dev_type_display_device] = ACTIONS(1773),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_io_device] = ACTIONS(1773),
    [sym_duet_dev_type_relay_device] = ACTIONS(1773),
    [sym_duet_dev_type_ups] = ACTIONS(1773),
    [sym_duet_dev_type_amplifier] = ACTIONS(1773),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1773),
    [sym_duet_dev_type_camera] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1773),
    [sym_duet_dev_type_dss] = ACTIONS(1773),
    [sym_duet_dev_type_dvr] = ACTIONS(1773),
    [sym_duet_dev_type_disc_device] = ACTIONS(1773),
    [sym_duet_dev_type_document_camera] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1773),
    [sym_duet_dev_type_hvac] = ACTIONS(1773),
    [sym_duet_dev_type_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_light] = ACTIONS(1773),
    [sym_duet_dev_type_monitor] = ACTIONS(1773),
    [sym_duet_dev_type_motor] = ACTIONS(1773),
    [sym_duet_dev_type_multi_window] = ACTIONS(1773),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1773),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1773),
    [sym_duet_dev_type_receiver] = ACTIONS(1773),
    [sym_duet_dev_type_security_system] = ACTIONS(1773),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1773),
    [sym_duet_dev_type_settop_box] = ACTIONS(1773),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1773),
    [sym_duet_dev_type_switcher] = ACTIONS(1773),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_tv] = ACTIONS(1773),
    [sym_duet_dev_type_utility] = ACTIONS(1773),
    [sym_duet_dev_type_vcr] = ACTIONS(1773),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_video_processor] = ACTIONS(1773),
    [sym_duet_dev_type_video_projector] = ACTIONS(1773),
    [sym_duet_dev_type_video_wall] = ACTIONS(1773),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1773),
    [sym_duet_dev_type_weather] = ACTIONS(1773),
    [sym_duet_dev_not_polled] = ACTIONS(1773),
    [sym_duet_dev_polled] = ACTIONS(1773),
    [sym_clkmgr_mode_network] = ACTIONS(1773),
    [sym_clkmgr_mode_standalone] = ACTIONS(1773),
    [sym_normal_standby] = ACTIONS(1773),
    [sym_normal_wake] = ACTIONS(1773),
    [sym_ip_multicast_ttl_option] = ACTIONS(1773),
    [sym_ip_ttl_subnet] = ACTIONS(1773),
    [sym_ip_ttl_site] = ACTIONS(1773),
    [sym_ip_ttl_region] = ACTIONS(1773),
    [sym_ip_ttl_continent] = ACTIONS(1773),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1773),
    [sym_ip_nodelay_on] = ACTIONS(1773),
    [sym_ip_nodelay_off] = ACTIONS(1773),
    [sym_amx_error] = ACTIONS(1773),
    [sym_amx_warning] = ACTIONS(1773),
    [sym_amx_info] = ACTIONS(1773),
    [sym_amx_debug] = ACTIONS(1773),
    [sym_netlinx_login_success] = ACTIONS(1773),
    [sym_netlinx_login_fail] = ACTIONS(1773),
    [sym_netlinx_logout] = ACTIONS(1773),
    [sym_null_str] = ACTIONS(1773),
    [sym_smtp_address] = ACTIONS(1773),
    [sym_smtp_port_number] = ACTIONS(1773),
    [sym_smtp_username] = ACTIONS(1773),
    [sym_smtp_password] = ACTIONS(1773),
    [sym_smtp_require_tls] = ACTIONS(1773),
    [sym_smtp_from] = ACTIONS(1773),
    [sym_smtp_tls_true] = ACTIONS(1773),
    [sym_smtp_tls_false] = ACTIONS(1773),
    [sym_listview_on_row_select_event] = ACTIONS(1773),
    [sym_data_structure_unknown] = ACTIONS(1773),
    [sym_data_structure_datafeed] = ACTIONS(1773),
    [sym_data_structure_datarecordset] = ACTIONS(1773),
    [sym_data_structure_datarecord] = ACTIONS(1773),
    [sym_data_structure_datafield] = ACTIONS(1773),
    [sym_data_max_content_field_count] = ACTIONS(1773),
    [sym_data_max_metadata_field_count] = ACTIONS(1773),
    [sym_data_max_id_length] = ACTIONS(1773),
    [sym_data_max_name_length] = ACTIONS(1773),
    [sym_data_max_description_length] = ACTIONS(1773),
    [sym_data_max_type_length] = ACTIONS(1773),
    [sym_data_max_format_length] = ACTIONS(1773),
    [sym_data_max_source_length] = ACTIONS(1773),
    [sym_data_max_value_length] = ACTIONS(1773),
    [sym_data_max_label_length] = ACTIONS(1773),
    [sym_data_max_url_length] = ACTIONS(1773),
    [sym_max_last_login_info_length] = ACTIONS(1773),
    [sym_data_type_unknown] = ACTIONS(1773),
    [sym_data_type_string] = ACTIONS(1773),
    [sym_data_type_datetime] = ACTIONS(1773),
    [sym_data_type_date] = ACTIONS(1773),
    [sym_data_type_time] = ACTIONS(1773),
    [sym_data_type_image] = ACTIONS(1773),
    [sym_data_format_url] = ACTIONS(1773),
    [sym_data_format_phone] = ACTIONS(1773),
    [sym_data_format_email] = ACTIONS(1773),
    [sym_data_format_iso8601] = ACTIONS(1773),
    [sym_auditlog_priv] = ACTIONS(1773),
    [sym_configuration_priv] = ACTIONS(1773),
    [sym_deviceconfig_priv] = ACTIONS(1773),
    [sym_ftp_priv] = ACTIONS(1773),
    [sym_http_priv] = ACTIONS(1773),
    [sym_networkconfig_priv] = ACTIONS(1773),
    [sym_programport_priv] = ACTIONS(1773),
    [sym_remoteui_priv] = ACTIONS(1773),
    [sym_securitycontrol_priv] = ACTIONS(1773),
    [sym_softwaremanagement_priv] = ACTIONS(1773),
    [sym_terminal_priv] = ACTIONS(1773),
    [sym_tpadmin_priv] = ACTIONS(1773),
    [sym_user1_priv] = ACTIONS(1773),
    [sym_user2_priv] = ACTIONS(1773),
    [sym_user3_priv] = ACTIONS(1773),
    [sym_user4_priv] = ACTIONS(1773),
    [sym_usermanagement_priv] = ACTIONS(1773),
    [sym_valid_account] = ACTIONS(1773),
    [sym_err_name_invalid] = ACTIONS(1773),
    [sym_err_password_invalid] = ACTIONS(1773),
    [sym_err_invalid_account] = ACTIONS(1773),
    [sym_err_no_authorization] = ACTIONS(1773),
    [sym_err_invalid_parameter] = ACTIONS(1773),
    [sym_err_user_locked_out] = ACTIONS(1773),
    [sym_err_user_account_expired] = ACTIONS(1773),
    [sym_err_general_error] = ACTIONS(1773),
    [sym_err_authentication_error] = ACTIONS(1773),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1773),
    [sym_tls_validate_certificate] = ACTIONS(1773),
    [sym_tdata] = ACTIONS(1773),
    [sym_tchannel] = ACTIONS(1773),
    [sym_tlevel] = ACTIONS(1773),
    [sym_tbutton] = ACTIONS(1773),
    [sym_ttimeline] = ACTIONS(1773),
    [sym_tcustom] = ACTIONS(1773),
    [sym_url_struct] = ACTIONS(1773),
    [sym_dns_struct] = ACTIONS(1773),
    [sym_ip_address_struct] = ACTIONS(1773),
    [sym_dev_info_struct] = ACTIONS(1773),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1773),
    [sym_data_feed] = ACTIONS(1773),
    [sym_data_field] = ACTIONS(1773),
    [sym_data_record] = ACTIONS(1773),
    [sym_wc_data_feed] = ACTIONS(1773),
    [sym_wc_data_field] = ACTIONS(1773),
    [sym_wc_data_record] = ACTIONS(1773),
    [sym_last_login_info] = ACTIONS(1773),
    [sym____reserved___] = ACTIONS(1773),
    [sym_date] = ACTIONS(1773),
    [sym_day] = ACTIONS(1773),
    [sym_time] = ACTIONS(1773),
    [sym_ldate] = ACTIONS(1773),
    [sym_push_device] = ACTIONS(1773),
    [sym_push_channel] = ACTIONS(1773),
    [sym_push_devchan] = ACTIONS(1773),
    [sym_release_device] = ACTIONS(1773),
    [sym_release_channel] = ACTIONS(1773),
    [sym_release_devchan] = ACTIONS(1773),
    [sym_master_slot] = ACTIONS(1773),
    [sym_get_pulse_time] = ACTIONS(1773),
    [sym_get_timer] = ACTIONS(1773),
    [sym_master_sn] = ACTIONS(1773),
    [sym_system_number] = ACTIONS(1773),
    [sym_button] = ACTIONS(1773),
    [sym_dv_channel] = ACTIONS(1773),
    [sym_data] = ACTIONS(1773),
    [sym_level] = ACTIONS(1773),
    [sym_timeline] = ACTIONS(1773),
    [sym_channel] = ACTIONS(1773),
    [sym_custom] = ACTIONS(1773),
    [sym_atoi] = ACTIONS(1773),
    [sym_atol] = ACTIONS(1773),
    [sym_atof] = ACTIONS(1773),
    [sym_itoa] = ACTIONS(1773),
    [sym_itohex] = ACTIONS(1773),
    [sym_hextoi] = ACTIONS(1773),
    [sym_ftoa] = ACTIONS(1773),
    [sym_find_string] = ACTIONS(1773),
    [sym_left_string] = ACTIONS(1773),
    [sym_right_string] = ACTIONS(1773),
    [sym_mid_string] = ACTIONS(1773),
    [sym_remove_string] = ACTIONS(1773),
    [sym_get_buffer_string] = ACTIONS(1773),
    [sym_get_buffer_char] = ACTIONS(1773),
    [sym_get_multi_buffer_string] = ACTIONS(1773),
    [sym_length_string] = ACTIONS(1773),
    [sym_max_length_string] = ACTIONS(1773),
    [sym_set_length_string] = ACTIONS(1773),
    [sym_compare_string] = ACTIONS(1773),
    [sym_upper_string] = ACTIONS(1773),
    [sym_lower_string] = ACTIONS(1773),
    [sym_get_last] = ACTIONS(1773),
    [sym_date_to_day] = ACTIONS(1773),
    [sym_date_to_month] = ACTIONS(1773),
    [sym_date_to_year] = ACTIONS(1773),
    [sym_time_to_hour] = ACTIONS(1773),
    [sym_time_to_minute] = ACTIONS(1773),
    [sym_time_to_second] = ACTIONS(1773),
    [sym_day_of_week] = ACTIONS(1773),
    [sym_set_pulse_time] = ACTIONS(1773),
    [sym_set_timer] = ACTIONS(1773),
    [sym_file_open] = ACTIONS(1773),
    [sym_file_close] = ACTIONS(1773),
    [sym_file_read] = ACTIONS(1773),
    [sym_file_read_line] = ACTIONS(1773),
    [sym_file_write] = ACTIONS(1773),
    [sym_file_write_line] = ACTIONS(1773),
    [sym_file_copy] = ACTIONS(1773),
    [sym_file_delete] = ACTIONS(1773),
    [sym_file_rename] = ACTIONS(1773),
    [sym_file_dir] = ACTIONS(1773),
    [sym_file_getdir] = ACTIONS(1773),
    [sym_file_setdir] = ACTIONS(1773),
    [sym_file_create_dir] = ACTIONS(1773),
    [sym_file_remove_dir] = ACTIONS(1773),
    [sym_file_seek] = ACTIONS(1773),
    [sym_length_array] = ACTIONS(1773),
    [sym_max_length_array] = ACTIONS(1773),
    [sym_set_length_array] = ACTIONS(1773),
    [sym_ip_client_open] = ACTIONS(1773),
    [sym_ip_client_close] = ACTIONS(1773),
    [sym_ip_server_open] = ACTIONS(1773),
    [sym_ip_server_close] = ACTIONS(1773),
    [sym_ip_bound_client_open] = ACTIONS(1773),
    [sym_ip_mc_server_open] = ACTIONS(1773),
    [sym_ip_set_option] = ACTIONS(1773),
    [sym_ssh_client_open] = ACTIONS(1773),
    [sym_ssh_client_close] = ACTIONS(1773),
    [sym_tls_client_open] = ACTIONS(1773),
    [sym_tls_client_close] = ACTIONS(1773),
    [sym_do_push] = ACTIONS(1773),
    [sym_do_push_timed] = ACTIONS(1773),
    [sym_do_release] = ACTIONS(1773),
    [sym_device_id] = ACTIONS(1773),
    [sym_device_info] = ACTIONS(1773),
    [sym_device_id_string] = ACTIONS(1773),
    [sym_device_standby] = ACTIONS(1773),
    [sym_device_wake] = ACTIONS(1773),
    [sym_timeline_create] = ACTIONS(1773),
    [sym_timeline_kill] = ACTIONS(1773),
    [sym_timeline_reload] = ACTIONS(1773),
    [sym_timeline_pause] = ACTIONS(1773),
    [sym_timeline_restart] = ACTIONS(1773),
    [sym_timeline_set] = ACTIONS(1773),
    [sym_timeline_get] = ACTIONS(1773),
    [sym_timeline_active] = ACTIONS(1773),
    [sym_timeline_dynamic_id] = ACTIONS(1773),
    [sym_variable_to_string] = ACTIONS(1773),
    [sym_string_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_string] = ACTIONS(1773),
    [sym_type_cast] = ACTIONS(1773),
    [sym_variable_to_xml] = ACTIONS(1773),
    [sym_xml_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_xml] = ACTIONS(1773),
    [sym_redirect_string] = ACTIONS(1773),
    [sym_random_number] = ACTIONS(1773),
    [sym_get_unique_id] = ACTIONS(1773),
    [sym_get_system_number] = ACTIONS(1773),
    [sym_set_system_number] = ACTIONS(1773),
    [sym_get_serial_number] = ACTIONS(1773),
    [sym_get_master_build] = ACTIONS(1773),
    [sym_set_validation_code] = ACTIONS(1773),
    [sym_reboot] = ACTIONS(1773),
    [sym_load_duet_module] = ACTIONS(1773),
    [sym_unload_duet_module] = ACTIONS(1773),
    [sym_duet_mem_size_set] = ACTIONS(1773),
    [sym_duet_mem_size_get] = ACTIONS(1773),
    [sym_dynamic_application_device] = ACTIONS(1773),
    [sym_dynamic_polled_port] = ACTIONS(1773),
    [sym_static_port_binding] = ACTIONS(1773),
    [sym_static_ip_binding] = ACTIONS(1773),
    [sym_get_dns_list] = ACTIONS(1773),
    [sym_set_dns_list] = ACTIONS(1773),
    [sym_get_ip_address] = ACTIONS(1773),
    [sym_set_ip_address] = ACTIONS(1773),
    [sym_get_url_list] = ACTIONS(1773),
    [sym_add_url_entry] = ACTIONS(1773),
    [sym_delete_url_entry] = ACTIONS(1773),
    [sym_abs_value] = ACTIONS(1773),
    [sym_max_value] = ACTIONS(1773),
    [sym_min_value] = ACTIONS(1773),
    [sym_exp_value] = ACTIONS(1773),
    [sym_log_value] = ACTIONS(1773),
    [sym_log10_value] = ACTIONS(1773),
    [sym_power_value] = ACTIONS(1773),
    [sym_sqrt_value] = ACTIONS(1773),
    [sym_raw_be] = ACTIONS(1773),
    [sym_raw_le] = ACTIONS(1773),
    [sym_internal_queue_size_set] = ACTIONS(1773),
    [sym_internal_queue_size_get] = ACTIONS(1773),
    [sym_internal_threshold_set] = ACTIONS(1773),
    [sym_internal_threshold_get] = ACTIONS(1773),
    [sym_rebuilt_event] = ACTIONS(1773),
    [sym_do_custom_event] = ACTIONS(1773),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1773),
    [sym_clkmgr_set_clk_source] = ACTIONS(1773),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1773),
    [sym_clkmgr_get_timezone] = ACTIONS(1773),
    [sym_clkmgr_set_timezone] = ACTIONS(1773),
    [sym_clkmgr_get_resync_period] = ACTIONS(1773),
    [sym_clkmgr_set_resync_period] = ACTIONS(1773),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_timeservers] = ACTIONS(1773),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_set_log_level] = ACTIONS(1773),
    [sym_get_log_level] = ACTIONS(1773),
    [sym_amx_log] = ACTIONS(1773),
    [sym_smtp_server_config_set] = ACTIONS(1773),
    [sym_smtp_server_config_get] = ACTIONS(1773),
    [sym_smtp_send] = ACTIONS(1773),
    [sym_validate_netlinx_account] = ACTIONS(1773),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1773),
    [sym_authenticate_certificate] = ACTIONS(1773),
    [sym_audit_netlinx_session_event] = ACTIONS(1773),
    [sym_audit_netlinx_generic_event] = ACTIONS(1773),
    [sym_get_available_flash_disk_space] = ACTIONS(1773),
    [sym_get_max_flash_disk_space] = ACTIONS(1773),
    [sym_data_create_feed] = ACTIONS(1773),
    [sym_data_delete_feed] = ACTIONS(1773),
    [sym_data_publish_feed] = ACTIONS(1773),
    [sym_data_get_published_feed] = ACTIONS(1773),
    [sym_data_add_record] = ACTIONS(1773),
    [sym_data_get_event_record] = ACTIONS(1773),
    [sym__wc_data_create_feed] = ACTIONS(1773),
    [sym__wc_data_add_record] = ACTIONS(1773),
    [sym__wc_data_get_event_record] = ACTIONS(1773),
    [sym_format] = ACTIONS(1773),
    [sym_astro_clock] = ACTIONS(1773),
    [sym_set_outdoor_temperature] = ACTIONS(1773),
    [sym_set_virtual_level_count] = ACTIONS(1773),
    [sym_set_virtual_channel_count] = ACTIONS(1773),
    [sym_set_virtual_port_count] = ACTIONS(1773),
    [sym__wc_encode_int] = ACTIONS(1773),
    [sym__wc_decode_int] = ACTIONS(1773),
    [sym__wc_int] = ACTIONS(1773),
    [sym__wc_to_ch_int] = ACTIONS(1773),
    [sym__ch_to_wc_int] = ACTIONS(1773),
    [sym__wc_find_string_int] = ACTIONS(1773),
    [sym__wc_left_string_int] = ACTIONS(1773),
    [sym__wc_lower_string_int] = ACTIONS(1773),
    [sym__wc_mid_string_int] = ACTIONS(1773),
    [sym__wc_remove_string_int] = ACTIONS(1773),
    [sym__wc_right_string_int] = ACTIONS(1773),
    [sym__wc_upper_string_int] = ACTIONS(1773),
    [sym__wc_compare_string_int] = ACTIONS(1773),
    [sym__wc_get_buffer_char_int] = ACTIONS(1773),
    [sym__wc_get_buffer_string_int] = ACTIONS(1773),
    [sym__wc_concat_string_int] = ACTIONS(1773),
    [sym__wc_file_open_int] = ACTIONS(1773),
    [sym__wc_file_close_int] = ACTIONS(1773),
    [sym__wc_file_read_int] = ACTIONS(1773),
    [sym__wc_file_read_line_int] = ACTIONS(1773),
    [sym__wc_file_write_int] = ACTIONS(1773),
    [sym__wc_file_write_line_int] = ACTIONS(1773),
    [aux_sym_preproc_include_token1] = ACTIONS(1773),
    [aux_sym_preproc_define_token1] = ACTIONS(1775),
    [aux_sym_preproc_warn_token1] = ACTIONS(1775),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1775),
    [anon_sym_LBRACK] = ACTIONS(1775),
    [anon_sym_LBRACE] = ACTIONS(1775),
    [anon_sym_LPAREN] = ACTIONS(1773),
    [anon_sym_BANG] = ACTIONS(1775),
    [anon_sym_TILDE] = ACTIONS(1775),
    [anon_sym_DASH] = ACTIONS(1773),
    [anon_sym_PLUS] = ACTIONS(1773),
    [anon_sym_DASH_DASH] = ACTIONS(1775),
    [anon_sym_PLUS_PLUS] = ACTIONS(1775),
    [anon_sym_DQUOTE] = ACTIONS(1775),
    [anon_sym_SQUOTE] = ACTIONS(1775),
    [sym_number_literal] = ACTIONS(1775),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(303)] = {
    [ts_builtin_sym_end] = ACTIONS(1775),
    [sym_identifier] = ACTIONS(1773),
    [sym_program_name_keyword] = ACTIONS(1773),
    [sym_module_name_keyword] = ACTIONS(1773),
    [sym_define_device_keyword] = ACTIONS(1773),
    [sym_define_combine_keyword] = ACTIONS(1773),
    [sym_define_constant_keyword] = ACTIONS(1773),
    [sym_define_type_keyword] = ACTIONS(1773),
    [sym_define_variable_keyword] = ACTIONS(1773),
    [sym_define_system_variable_keyword] = ACTIONS(1773),
    [sym_define_connect_level_keyword] = ACTIONS(1773),
    [sym_define_latching_keyword] = ACTIONS(1773),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1773),
    [sym_define_toggling_keyword] = ACTIONS(1773),
    [sym_define_call_keyword] = ACTIONS(1773),
    [sym_define_function_keyword] = ACTIONS(1773),
    [sym_define_library_function_keyword] = ACTIONS(1773),
    [sym_define_start_keyword] = ACTIONS(1773),
    [sym_define_event_keyword] = ACTIONS(1773),
    [sym_define_program_keyword] = ACTIONS(1773),
    [sym_define_module_keyword] = ACTIONS(1773),
    [sym_char_keyword] = ACTIONS(1773),
    [sym_widechar_keyword] = ACTIONS(1773),
    [sym_integer_keyword] = ACTIONS(1773),
    [sym_sinteger_keyword] = ACTIONS(1773),
    [sym_long_keyword] = ACTIONS(1773),
    [sym_slong_keyword] = ACTIONS(1773),
    [sym_float_keyword] = ACTIONS(1773),
    [sym_double_keyword] = ACTIONS(1773),
    [sym_variant_keyword] = ACTIONS(1773),
    [sym_variantarray_keyword] = ACTIONS(1773),
    [sym_local_var_keyword] = ACTIONS(1773),
    [sym_stack_var_keyword] = ACTIONS(1773),
    [sym_constant_keyword] = ACTIONS(1773),
    [sym_volatile_keyword] = ACTIONS(1773),
    [sym_non_volatile_keyword] = ACTIONS(1773),
    [sym_persistent_keyword] = ACTIONS(1773),
    [sym_dev_keyword] = ACTIONS(1773),
    [sym_devlev_keyword] = ACTIONS(1773),
    [sym_devchan_keyword] = ACTIONS(1773),
    [sym_if_keyword] = ACTIONS(1773),
    [sym_else_keyword] = ACTIONS(1773),
    [sym_switch_keyword] = ACTIONS(1773),
    [sym_case_keyword] = ACTIONS(1773),
    [sym_default_keyword] = ACTIONS(1773),
    [sym_while_keyword] = ACTIONS(1773),
    [sym_for_keyword] = ACTIONS(1773),
    [sym_break_keyword] = ACTIONS(1773),
    [sym_continue_keyword] = ACTIONS(1773),
    [sym_return_keyword] = ACTIONS(1773),
    [sym_select_keyword] = ACTIONS(1773),
    [sym_active_keyword] = ACTIONS(1773),
    [sym_struct_keyword] = ACTIONS(1773),
    [sym_structure_keyword] = ACTIONS(1773),
    [sym_button_event_keyword] = ACTIONS(1773),
    [sym_channel_event_keyword] = ACTIONS(1773),
    [sym_level_event_keyword] = ACTIONS(1773),
    [sym_data_event_keyword] = ACTIONS(1773),
    [sym_timeline_event_keyword] = ACTIONS(1773),
    [sym_custom_event_keyword] = ACTIONS(1773),
    [sym_push_keyword] = ACTIONS(1773),
    [sym_release_keyword] = ACTIONS(1773),
    [aux_sym_on_keyword_token1] = ACTIONS(1773),
    [aux_sym_off_keyword_token1] = ACTIONS(1773),
    [sym_send_level_keyword] = ACTIONS(1773),
    [sym_send_string_keyword] = ACTIONS(1773),
    [sym_send_command_keyword] = ACTIONS(1773),
    [sym_clear_buffer_keyword] = ACTIONS(1773),
    [sym_create_buffer_keyword] = ACTIONS(1773),
    [sym_create_multi_buffer_keyword] = ACTIONS(1773),
    [sym_call_keyword] = ACTIONS(1773),
    [sym_system_call_keyword] = ACTIONS(1773),
    [sym_devchan_to_keyword] = ACTIONS(1773),
    [sym_devchan_min_to_keyword] = ACTIONS(1773),
    [sym_devchan_total_off_keyword] = ACTIONS(1773),
    [sym_devchan_pulse_keyword] = ACTIONS(1773),
    [sym_wait_keyword] = ACTIONS(1773),
    [sym_cancel_wait_keyword] = ACTIONS(1773),
    [sym_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1773),
    [sym_not] = ACTIONS(1773),
    [sym_bnot] = ACTIONS(1773),
    [sym___file__] = ACTIONS(1773),
    [sym___date__] = ACTIONS(1773),
    [sym___ldate__] = ACTIONS(1773),
    [sym___time__] = ACTIONS(1773),
    [sym___line__] = ACTIONS(1773),
    [sym___name__] = ACTIONS(1773),
    [sym___netlinx__] = ACTIONS(1773),
    [sym_netlinx_axi_version] = ACTIONS(1773),
    [sym_url_flg_tcp] = ACTIONS(1773),
    [sym_url_flg_accinfopresent] = ACTIONS(1773),
    [sym_url_flg_temp] = ACTIONS(1773),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1773),
    [sym_url_flg_stat_mask] = ACTIONS(1773),
    [sym_url_flg_stat_lookup] = ACTIONS(1773),
    [sym_url_flg_stat_connecting] = ACTIONS(1773),
    [sym_url_flg_stat_waiting] = ACTIONS(1773),
    [sym_url_flg_stat_connected] = ACTIONS(1773),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1773),
    [sym_first_virtual_device] = ACTIONS(1773),
    [sym_first_local_port] = ACTIONS(1773),
    [sym_dynamic_virtual_device] = ACTIONS(1773),
    [sym_timeline_once] = ACTIONS(1773),
    [sym_timeline_repeat] = ACTIONS(1773),
    [sym_timeline_absolute] = ACTIONS(1773),
    [sym_timeline_relative] = ACTIONS(1773),
    [sym_do_push_timed_infinite] = ACTIONS(1773),
    [sym_source_type_no_address] = ACTIONS(1773),
    [sym_source_type_neuron_id] = ACTIONS(1773),
    [sym_source_type_ip_address] = ACTIONS(1773),
    [sym_source_type_axlink] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1773),
    [sym_source_type_ip_socket_address] = ACTIONS(1773),
    [sym_source_type_rs232] = ACTIONS(1773),
    [sym_source_type_internal] = ACTIONS(1773),
    [sym_source_type_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1773),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1773),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1773),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1773),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_file_read_only] = ACTIONS(1773),
    [sym_file_rw_new] = ACTIONS(1773),
    [sym_file_rw_append] = ACTIONS(1773),
    [sym_ip_tcp] = ACTIONS(1773),
    [sym_ip_udp] = ACTIONS(1773),
    [sym_ip_udp_2way] = ACTIONS(1773),
    [sym_xml_encode_types] = ACTIONS(1773),
    [sym_xml_encode_char_as_list] = ACTIONS(1773),
    [sym_xml_encode_le] = ACTIONS(1773),
    [sym_xml_decode_types] = ACTIONS(1773),
    [sym_xml_decode_no_preserve] = ACTIONS(1773),
    [sym_duet_dev_type_display_device] = ACTIONS(1773),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_io_device] = ACTIONS(1773),
    [sym_duet_dev_type_relay_device] = ACTIONS(1773),
    [sym_duet_dev_type_ups] = ACTIONS(1773),
    [sym_duet_dev_type_amplifier] = ACTIONS(1773),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1773),
    [sym_duet_dev_type_camera] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1773),
    [sym_duet_dev_type_dss] = ACTIONS(1773),
    [sym_duet_dev_type_dvr] = ACTIONS(1773),
    [sym_duet_dev_type_disc_device] = ACTIONS(1773),
    [sym_duet_dev_type_document_camera] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1773),
    [sym_duet_dev_type_hvac] = ACTIONS(1773),
    [sym_duet_dev_type_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_light] = ACTIONS(1773),
    [sym_duet_dev_type_monitor] = ACTIONS(1773),
    [sym_duet_dev_type_motor] = ACTIONS(1773),
    [sym_duet_dev_type_multi_window] = ACTIONS(1773),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1773),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1773),
    [sym_duet_dev_type_receiver] = ACTIONS(1773),
    [sym_duet_dev_type_security_system] = ACTIONS(1773),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1773),
    [sym_duet_dev_type_settop_box] = ACTIONS(1773),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1773),
    [sym_duet_dev_type_switcher] = ACTIONS(1773),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_tv] = ACTIONS(1773),
    [sym_duet_dev_type_utility] = ACTIONS(1773),
    [sym_duet_dev_type_vcr] = ACTIONS(1773),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_video_processor] = ACTIONS(1773),
    [sym_duet_dev_type_video_projector] = ACTIONS(1773),
    [sym_duet_dev_type_video_wall] = ACTIONS(1773),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1773),
    [sym_duet_dev_type_weather] = ACTIONS(1773),
    [sym_duet_dev_not_polled] = ACTIONS(1773),
    [sym_duet_dev_polled] = ACTIONS(1773),
    [sym_clkmgr_mode_network] = ACTIONS(1773),
    [sym_clkmgr_mode_standalone] = ACTIONS(1773),
    [sym_normal_standby] = ACTIONS(1773),
    [sym_normal_wake] = ACTIONS(1773),
    [sym_ip_multicast_ttl_option] = ACTIONS(1773),
    [sym_ip_ttl_subnet] = ACTIONS(1773),
    [sym_ip_ttl_site] = ACTIONS(1773),
    [sym_ip_ttl_region] = ACTIONS(1773),
    [sym_ip_ttl_continent] = ACTIONS(1773),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1773),
    [sym_ip_nodelay_on] = ACTIONS(1773),
    [sym_ip_nodelay_off] = ACTIONS(1773),
    [sym_amx_error] = ACTIONS(1773),
    [sym_amx_warning] = ACTIONS(1773),
    [sym_amx_info] = ACTIONS(1773),
    [sym_amx_debug] = ACTIONS(1773),
    [sym_netlinx_login_success] = ACTIONS(1773),
    [sym_netlinx_login_fail] = ACTIONS(1773),
    [sym_netlinx_logout] = ACTIONS(1773),
    [sym_null_str] = ACTIONS(1773),
    [sym_smtp_address] = ACTIONS(1773),
    [sym_smtp_port_number] = ACTIONS(1773),
    [sym_smtp_username] = ACTIONS(1773),
    [sym_smtp_password] = ACTIONS(1773),
    [sym_smtp_require_tls] = ACTIONS(1773),
    [sym_smtp_from] = ACTIONS(1773),
    [sym_smtp_tls_true] = ACTIONS(1773),
    [sym_smtp_tls_false] = ACTIONS(1773),
    [sym_listview_on_row_select_event] = ACTIONS(1773),
    [sym_data_structure_unknown] = ACTIONS(1773),
    [sym_data_structure_datafeed] = ACTIONS(1773),
    [sym_data_structure_datarecordset] = ACTIONS(1773),
    [sym_data_structure_datarecord] = ACTIONS(1773),
    [sym_data_structure_datafield] = ACTIONS(1773),
    [sym_data_max_content_field_count] = ACTIONS(1773),
    [sym_data_max_metadata_field_count] = ACTIONS(1773),
    [sym_data_max_id_length] = ACTIONS(1773),
    [sym_data_max_name_length] = ACTIONS(1773),
    [sym_data_max_description_length] = ACTIONS(1773),
    [sym_data_max_type_length] = ACTIONS(1773),
    [sym_data_max_format_length] = ACTIONS(1773),
    [sym_data_max_source_length] = ACTIONS(1773),
    [sym_data_max_value_length] = ACTIONS(1773),
    [sym_data_max_label_length] = ACTIONS(1773),
    [sym_data_max_url_length] = ACTIONS(1773),
    [sym_max_last_login_info_length] = ACTIONS(1773),
    [sym_data_type_unknown] = ACTIONS(1773),
    [sym_data_type_string] = ACTIONS(1773),
    [sym_data_type_datetime] = ACTIONS(1773),
    [sym_data_type_date] = ACTIONS(1773),
    [sym_data_type_time] = ACTIONS(1773),
    [sym_data_type_image] = ACTIONS(1773),
    [sym_data_format_url] = ACTIONS(1773),
    [sym_data_format_phone] = ACTIONS(1773),
    [sym_data_format_email] = ACTIONS(1773),
    [sym_data_format_iso8601] = ACTIONS(1773),
    [sym_auditlog_priv] = ACTIONS(1773),
    [sym_configuration_priv] = ACTIONS(1773),
    [sym_deviceconfig_priv] = ACTIONS(1773),
    [sym_ftp_priv] = ACTIONS(1773),
    [sym_http_priv] = ACTIONS(1773),
    [sym_networkconfig_priv] = ACTIONS(1773),
    [sym_programport_priv] = ACTIONS(1773),
    [sym_remoteui_priv] = ACTIONS(1773),
    [sym_securitycontrol_priv] = ACTIONS(1773),
    [sym_softwaremanagement_priv] = ACTIONS(1773),
    [sym_terminal_priv] = ACTIONS(1773),
    [sym_tpadmin_priv] = ACTIONS(1773),
    [sym_user1_priv] = ACTIONS(1773),
    [sym_user2_priv] = ACTIONS(1773),
    [sym_user3_priv] = ACTIONS(1773),
    [sym_user4_priv] = ACTIONS(1773),
    [sym_usermanagement_priv] = ACTIONS(1773),
    [sym_valid_account] = ACTIONS(1773),
    [sym_err_name_invalid] = ACTIONS(1773),
    [sym_err_password_invalid] = ACTIONS(1773),
    [sym_err_invalid_account] = ACTIONS(1773),
    [sym_err_no_authorization] = ACTIONS(1773),
    [sym_err_invalid_parameter] = ACTIONS(1773),
    [sym_err_user_locked_out] = ACTIONS(1773),
    [sym_err_user_account_expired] = ACTIONS(1773),
    [sym_err_general_error] = ACTIONS(1773),
    [sym_err_authentication_error] = ACTIONS(1773),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1773),
    [sym_tls_validate_certificate] = ACTIONS(1773),
    [sym_tdata] = ACTIONS(1773),
    [sym_tchannel] = ACTIONS(1773),
    [sym_tlevel] = ACTIONS(1773),
    [sym_tbutton] = ACTIONS(1773),
    [sym_ttimeline] = ACTIONS(1773),
    [sym_tcustom] = ACTIONS(1773),
    [sym_url_struct] = ACTIONS(1773),
    [sym_dns_struct] = ACTIONS(1773),
    [sym_ip_address_struct] = ACTIONS(1773),
    [sym_dev_info_struct] = ACTIONS(1773),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1773),
    [sym_data_feed] = ACTIONS(1773),
    [sym_data_field] = ACTIONS(1773),
    [sym_data_record] = ACTIONS(1773),
    [sym_wc_data_feed] = ACTIONS(1773),
    [sym_wc_data_field] = ACTIONS(1773),
    [sym_wc_data_record] = ACTIONS(1773),
    [sym_last_login_info] = ACTIONS(1773),
    [sym____reserved___] = ACTIONS(1773),
    [sym_date] = ACTIONS(1773),
    [sym_day] = ACTIONS(1773),
    [sym_time] = ACTIONS(1773),
    [sym_ldate] = ACTIONS(1773),
    [sym_push_device] = ACTIONS(1773),
    [sym_push_channel] = ACTIONS(1773),
    [sym_push_devchan] = ACTIONS(1773),
    [sym_release_device] = ACTIONS(1773),
    [sym_release_channel] = ACTIONS(1773),
    [sym_release_devchan] = ACTIONS(1773),
    [sym_master_slot] = ACTIONS(1773),
    [sym_get_pulse_time] = ACTIONS(1773),
    [sym_get_timer] = ACTIONS(1773),
    [sym_master_sn] = ACTIONS(1773),
    [sym_system_number] = ACTIONS(1773),
    [sym_button] = ACTIONS(1773),
    [sym_dv_channel] = ACTIONS(1773),
    [sym_data] = ACTIONS(1773),
    [sym_level] = ACTIONS(1773),
    [sym_timeline] = ACTIONS(1773),
    [sym_channel] = ACTIONS(1773),
    [sym_custom] = ACTIONS(1773),
    [sym_atoi] = ACTIONS(1773),
    [sym_atol] = ACTIONS(1773),
    [sym_atof] = ACTIONS(1773),
    [sym_itoa] = ACTIONS(1773),
    [sym_itohex] = ACTIONS(1773),
    [sym_hextoi] = ACTIONS(1773),
    [sym_ftoa] = ACTIONS(1773),
    [sym_find_string] = ACTIONS(1773),
    [sym_left_string] = ACTIONS(1773),
    [sym_right_string] = ACTIONS(1773),
    [sym_mid_string] = ACTIONS(1773),
    [sym_remove_string] = ACTIONS(1773),
    [sym_get_buffer_string] = ACTIONS(1773),
    [sym_get_buffer_char] = ACTIONS(1773),
    [sym_get_multi_buffer_string] = ACTIONS(1773),
    [sym_length_string] = ACTIONS(1773),
    [sym_max_length_string] = ACTIONS(1773),
    [sym_set_length_string] = ACTIONS(1773),
    [sym_compare_string] = ACTIONS(1773),
    [sym_upper_string] = ACTIONS(1773),
    [sym_lower_string] = ACTIONS(1773),
    [sym_get_last] = ACTIONS(1773),
    [sym_date_to_day] = ACTIONS(1773),
    [sym_date_to_month] = ACTIONS(1773),
    [sym_date_to_year] = ACTIONS(1773),
    [sym_time_to_hour] = ACTIONS(1773),
    [sym_time_to_minute] = ACTIONS(1773),
    [sym_time_to_second] = ACTIONS(1773),
    [sym_day_of_week] = ACTIONS(1773),
    [sym_set_pulse_time] = ACTIONS(1773),
    [sym_set_timer] = ACTIONS(1773),
    [sym_file_open] = ACTIONS(1773),
    [sym_file_close] = ACTIONS(1773),
    [sym_file_read] = ACTIONS(1773),
    [sym_file_read_line] = ACTIONS(1773),
    [sym_file_write] = ACTIONS(1773),
    [sym_file_write_line] = ACTIONS(1773),
    [sym_file_copy] = ACTIONS(1773),
    [sym_file_delete] = ACTIONS(1773),
    [sym_file_rename] = ACTIONS(1773),
    [sym_file_dir] = ACTIONS(1773),
    [sym_file_getdir] = ACTIONS(1773),
    [sym_file_setdir] = ACTIONS(1773),
    [sym_file_create_dir] = ACTIONS(1773),
    [sym_file_remove_dir] = ACTIONS(1773),
    [sym_file_seek] = ACTIONS(1773),
    [sym_length_array] = ACTIONS(1773),
    [sym_max_length_array] = ACTIONS(1773),
    [sym_set_length_array] = ACTIONS(1773),
    [sym_ip_client_open] = ACTIONS(1773),
    [sym_ip_client_close] = ACTIONS(1773),
    [sym_ip_server_open] = ACTIONS(1773),
    [sym_ip_server_close] = ACTIONS(1773),
    [sym_ip_bound_client_open] = ACTIONS(1773),
    [sym_ip_mc_server_open] = ACTIONS(1773),
    [sym_ip_set_option] = ACTIONS(1773),
    [sym_ssh_client_open] = ACTIONS(1773),
    [sym_ssh_client_close] = ACTIONS(1773),
    [sym_tls_client_open] = ACTIONS(1773),
    [sym_tls_client_close] = ACTIONS(1773),
    [sym_do_push] = ACTIONS(1773),
    [sym_do_push_timed] = ACTIONS(1773),
    [sym_do_release] = ACTIONS(1773),
    [sym_device_id] = ACTIONS(1773),
    [sym_device_info] = ACTIONS(1773),
    [sym_device_id_string] = ACTIONS(1773),
    [sym_device_standby] = ACTIONS(1773),
    [sym_device_wake] = ACTIONS(1773),
    [sym_timeline_create] = ACTIONS(1773),
    [sym_timeline_kill] = ACTIONS(1773),
    [sym_timeline_reload] = ACTIONS(1773),
    [sym_timeline_pause] = ACTIONS(1773),
    [sym_timeline_restart] = ACTIONS(1773),
    [sym_timeline_set] = ACTIONS(1773),
    [sym_timeline_get] = ACTIONS(1773),
    [sym_timeline_active] = ACTIONS(1773),
    [sym_timeline_dynamic_id] = ACTIONS(1773),
    [sym_variable_to_string] = ACTIONS(1773),
    [sym_string_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_string] = ACTIONS(1773),
    [sym_type_cast] = ACTIONS(1773),
    [sym_variable_to_xml] = ACTIONS(1773),
    [sym_xml_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_xml] = ACTIONS(1773),
    [sym_redirect_string] = ACTIONS(1773),
    [sym_random_number] = ACTIONS(1773),
    [sym_get_unique_id] = ACTIONS(1773),
    [sym_get_system_number] = ACTIONS(1773),
    [sym_set_system_number] = ACTIONS(1773),
    [sym_get_serial_number] = ACTIONS(1773),
    [sym_get_master_build] = ACTIONS(1773),
    [sym_set_validation_code] = ACTIONS(1773),
    [sym_reboot] = ACTIONS(1773),
    [sym_load_duet_module] = ACTIONS(1773),
    [sym_unload_duet_module] = ACTIONS(1773),
    [sym_duet_mem_size_set] = ACTIONS(1773),
    [sym_duet_mem_size_get] = ACTIONS(1773),
    [sym_dynamic_application_device] = ACTIONS(1773),
    [sym_dynamic_polled_port] = ACTIONS(1773),
    [sym_static_port_binding] = ACTIONS(1773),
    [sym_static_ip_binding] = ACTIONS(1773),
    [sym_get_dns_list] = ACTIONS(1773),
    [sym_set_dns_list] = ACTIONS(1773),
    [sym_get_ip_address] = ACTIONS(1773),
    [sym_set_ip_address] = ACTIONS(1773),
    [sym_get_url_list] = ACTIONS(1773),
    [sym_add_url_entry] = ACTIONS(1773),
    [sym_delete_url_entry] = ACTIONS(1773),
    [sym_abs_value] = ACTIONS(1773),
    [sym_max_value] = ACTIONS(1773),
    [sym_min_value] = ACTIONS(1773),
    [sym_exp_value] = ACTIONS(1773),
    [sym_log_value] = ACTIONS(1773),
    [sym_log10_value] = ACTIONS(1773),
    [sym_power_value] = ACTIONS(1773),
    [sym_sqrt_value] = ACTIONS(1773),
    [sym_raw_be] = ACTIONS(1773),
    [sym_raw_le] = ACTIONS(1773),
    [sym_internal_queue_size_set] = ACTIONS(1773),
    [sym_internal_queue_size_get] = ACTIONS(1773),
    [sym_internal_threshold_set] = ACTIONS(1773),
    [sym_internal_threshold_get] = ACTIONS(1773),
    [sym_rebuilt_event] = ACTIONS(1773),
    [sym_do_custom_event] = ACTIONS(1773),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1773),
    [sym_clkmgr_set_clk_source] = ACTIONS(1773),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1773),
    [sym_clkmgr_get_timezone] = ACTIONS(1773),
    [sym_clkmgr_set_timezone] = ACTIONS(1773),
    [sym_clkmgr_get_resync_period] = ACTIONS(1773),
    [sym_clkmgr_set_resync_period] = ACTIONS(1773),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_timeservers] = ACTIONS(1773),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_set_log_level] = ACTIONS(1773),
    [sym_get_log_level] = ACTIONS(1773),
    [sym_amx_log] = ACTIONS(1773),
    [sym_smtp_server_config_set] = ACTIONS(1773),
    [sym_smtp_server_config_get] = ACTIONS(1773),
    [sym_smtp_send] = ACTIONS(1773),
    [sym_validate_netlinx_account] = ACTIONS(1773),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1773),
    [sym_authenticate_certificate] = ACTIONS(1773),
    [sym_audit_netlinx_session_event] = ACTIONS(1773),
    [sym_audit_netlinx_generic_event] = ACTIONS(1773),
    [sym_get_available_flash_disk_space] = ACTIONS(1773),
    [sym_get_max_flash_disk_space] = ACTIONS(1773),
    [sym_data_create_feed] = ACTIONS(1773),
    [sym_data_delete_feed] = ACTIONS(1773),
    [sym_data_publish_feed] = ACTIONS(1773),
    [sym_data_get_published_feed] = ACTIONS(1773),
    [sym_data_add_record] = ACTIONS(1773),
    [sym_data_get_event_record] = ACTIONS(1773),
    [sym__wc_data_create_feed] = ACTIONS(1773),
    [sym__wc_data_add_record] = ACTIONS(1773),
    [sym__wc_data_get_event_record] = ACTIONS(1773),
    [sym_format] = ACTIONS(1773),
    [sym_astro_clock] = ACTIONS(1773),
    [sym_set_outdoor_temperature] = ACTIONS(1773),
    [sym_set_virtual_level_count] = ACTIONS(1773),
    [sym_set_virtual_channel_count] = ACTIONS(1773),
    [sym_set_virtual_port_count] = ACTIONS(1773),
    [sym__wc_encode_int] = ACTIONS(1773),
    [sym__wc_decode_int] = ACTIONS(1773),
    [sym__wc_int] = ACTIONS(1773),
    [sym__wc_to_ch_int] = ACTIONS(1773),
    [sym__ch_to_wc_int] = ACTIONS(1773),
    [sym__wc_find_string_int] = ACTIONS(1773),
    [sym__wc_left_string_int] = ACTIONS(1773),
    [sym__wc_lower_string_int] = ACTIONS(1773),
    [sym__wc_mid_string_int] = ACTIONS(1773),
    [sym__wc_remove_string_int] = ACTIONS(1773),
    [sym__wc_right_string_int] = ACTIONS(1773),
    [sym__wc_upper_string_int] = ACTIONS(1773),
    [sym__wc_compare_string_int] = ACTIONS(1773),
    [sym__wc_get_buffer_char_int] = ACTIONS(1773),
    [sym__wc_get_buffer_string_int] = ACTIONS(1773),
    [sym__wc_concat_string_int] = ACTIONS(1773),
    [sym__wc_file_open_int] = ACTIONS(1773),
    [sym__wc_file_close_int] = ACTIONS(1773),
    [sym__wc_file_read_int] = ACTIONS(1773),
    [sym__wc_file_read_line_int] = ACTIONS(1773),
    [sym__wc_file_write_int] = ACTIONS(1773),
    [sym__wc_file_write_line_int] = ACTIONS(1773),
    [aux_sym_preproc_include_token1] = ACTIONS(1773),
    [aux_sym_preproc_define_token1] = ACTIONS(1775),
    [aux_sym_preproc_warn_token1] = ACTIONS(1775),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1775),
    [anon_sym_LBRACK] = ACTIONS(1775),
    [anon_sym_LBRACE] = ACTIONS(1775),
    [anon_sym_LPAREN] = ACTIONS(1773),
    [anon_sym_BANG] = ACTIONS(1775),
    [anon_sym_TILDE] = ACTIONS(1775),
    [anon_sym_DASH] = ACTIONS(1773),
    [anon_sym_PLUS] = ACTIONS(1773),
    [anon_sym_DASH_DASH] = ACTIONS(1775),
    [anon_sym_PLUS_PLUS] = ACTIONS(1775),
    [anon_sym_DQUOTE] = ACTIONS(1775),
    [anon_sym_SQUOTE] = ACTIONS(1775),
    [sym_number_literal] = ACTIONS(1775),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(304)] = {
    [ts_builtin_sym_end] = ACTIONS(1779),
    [sym_identifier] = ACTIONS(1777),
    [sym_program_name_keyword] = ACTIONS(1777),
    [sym_module_name_keyword] = ACTIONS(1777),
    [sym_define_device_keyword] = ACTIONS(1777),
    [sym_define_combine_keyword] = ACTIONS(1777),
    [sym_define_constant_keyword] = ACTIONS(1777),
    [sym_define_type_keyword] = ACTIONS(1777),
    [sym_define_variable_keyword] = ACTIONS(1777),
    [sym_define_system_variable_keyword] = ACTIONS(1777),
    [sym_define_connect_level_keyword] = ACTIONS(1777),
    [sym_define_latching_keyword] = ACTIONS(1777),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1777),
    [sym_define_toggling_keyword] = ACTIONS(1777),
    [sym_define_call_keyword] = ACTIONS(1777),
    [sym_define_function_keyword] = ACTIONS(1777),
    [sym_define_library_function_keyword] = ACTIONS(1777),
    [sym_define_start_keyword] = ACTIONS(1777),
    [sym_define_event_keyword] = ACTIONS(1777),
    [sym_define_program_keyword] = ACTIONS(1777),
    [sym_define_module_keyword] = ACTIONS(1777),
    [sym_char_keyword] = ACTIONS(1777),
    [sym_widechar_keyword] = ACTIONS(1777),
    [sym_integer_keyword] = ACTIONS(1777),
    [sym_sinteger_keyword] = ACTIONS(1777),
    [sym_long_keyword] = ACTIONS(1777),
    [sym_slong_keyword] = ACTIONS(1777),
    [sym_float_keyword] = ACTIONS(1777),
    [sym_double_keyword] = ACTIONS(1777),
    [sym_variant_keyword] = ACTIONS(1777),
    [sym_variantarray_keyword] = ACTIONS(1777),
    [sym_local_var_keyword] = ACTIONS(1777),
    [sym_stack_var_keyword] = ACTIONS(1777),
    [sym_constant_keyword] = ACTIONS(1777),
    [sym_volatile_keyword] = ACTIONS(1777),
    [sym_non_volatile_keyword] = ACTIONS(1777),
    [sym_persistent_keyword] = ACTIONS(1777),
    [sym_dev_keyword] = ACTIONS(1777),
    [sym_devlev_keyword] = ACTIONS(1777),
    [sym_devchan_keyword] = ACTIONS(1777),
    [sym_if_keyword] = ACTIONS(1777),
    [sym_else_keyword] = ACTIONS(1777),
    [sym_switch_keyword] = ACTIONS(1777),
    [sym_case_keyword] = ACTIONS(1777),
    [sym_default_keyword] = ACTIONS(1777),
    [sym_while_keyword] = ACTIONS(1777),
    [sym_for_keyword] = ACTIONS(1777),
    [sym_break_keyword] = ACTIONS(1777),
    [sym_continue_keyword] = ACTIONS(1777),
    [sym_return_keyword] = ACTIONS(1777),
    [sym_select_keyword] = ACTIONS(1777),
    [sym_active_keyword] = ACTIONS(1777),
    [sym_struct_keyword] = ACTIONS(1777),
    [sym_structure_keyword] = ACTIONS(1777),
    [sym_button_event_keyword] = ACTIONS(1777),
    [sym_channel_event_keyword] = ACTIONS(1777),
    [sym_level_event_keyword] = ACTIONS(1777),
    [sym_data_event_keyword] = ACTIONS(1777),
    [sym_timeline_event_keyword] = ACTIONS(1777),
    [sym_custom_event_keyword] = ACTIONS(1777),
    [sym_push_keyword] = ACTIONS(1777),
    [sym_release_keyword] = ACTIONS(1777),
    [aux_sym_on_keyword_token1] = ACTIONS(1777),
    [aux_sym_off_keyword_token1] = ACTIONS(1777),
    [sym_send_level_keyword] = ACTIONS(1777),
    [sym_send_string_keyword] = ACTIONS(1777),
    [sym_send_command_keyword] = ACTIONS(1777),
    [sym_clear_buffer_keyword] = ACTIONS(1777),
    [sym_create_buffer_keyword] = ACTIONS(1777),
    [sym_create_multi_buffer_keyword] = ACTIONS(1777),
    [sym_call_keyword] = ACTIONS(1777),
    [sym_system_call_keyword] = ACTIONS(1777),
    [sym_devchan_to_keyword] = ACTIONS(1777),
    [sym_devchan_min_to_keyword] = ACTIONS(1777),
    [sym_devchan_total_off_keyword] = ACTIONS(1777),
    [sym_devchan_pulse_keyword] = ACTIONS(1777),
    [sym_wait_keyword] = ACTIONS(1777),
    [sym_cancel_wait_keyword] = ACTIONS(1777),
    [sym_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1777),
    [sym_not] = ACTIONS(1777),
    [sym_bnot] = ACTIONS(1777),
    [sym___file__] = ACTIONS(1777),
    [sym___date__] = ACTIONS(1777),
    [sym___ldate__] = ACTIONS(1777),
    [sym___time__] = ACTIONS(1777),
    [sym___line__] = ACTIONS(1777),
    [sym___name__] = ACTIONS(1777),
    [sym___netlinx__] = ACTIONS(1777),
    [sym_netlinx_axi_version] = ACTIONS(1777),
    [sym_url_flg_tcp] = ACTIONS(1777),
    [sym_url_flg_accinfopresent] = ACTIONS(1777),
    [sym_url_flg_temp] = ACTIONS(1777),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1777),
    [sym_url_flg_stat_mask] = ACTIONS(1777),
    [sym_url_flg_stat_lookup] = ACTIONS(1777),
    [sym_url_flg_stat_connecting] = ACTIONS(1777),
    [sym_url_flg_stat_waiting] = ACTIONS(1777),
    [sym_url_flg_stat_connected] = ACTIONS(1777),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1777),
    [sym_first_virtual_device] = ACTIONS(1777),
    [sym_first_local_port] = ACTIONS(1777),
    [sym_dynamic_virtual_device] = ACTIONS(1777),
    [sym_timeline_once] = ACTIONS(1777),
    [sym_timeline_repeat] = ACTIONS(1777),
    [sym_timeline_absolute] = ACTIONS(1777),
    [sym_timeline_relative] = ACTIONS(1777),
    [sym_do_push_timed_infinite] = ACTIONS(1777),
    [sym_source_type_no_address] = ACTIONS(1777),
    [sym_source_type_neuron_id] = ACTIONS(1777),
    [sym_source_type_ip_address] = ACTIONS(1777),
    [sym_source_type_axlink] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1777),
    [sym_source_type_ip_socket_address] = ACTIONS(1777),
    [sym_source_type_rs232] = ACTIONS(1777),
    [sym_source_type_internal] = ACTIONS(1777),
    [sym_source_type_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1777),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1777),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1777),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1777),
    [sym_true] = ACTIONS(1777),
    [sym_false] = ACTIONS(1777),
    [sym_file_read_only] = ACTIONS(1777),
    [sym_file_rw_new] = ACTIONS(1777),
    [sym_file_rw_append] = ACTIONS(1777),
    [sym_ip_tcp] = ACTIONS(1777),
    [sym_ip_udp] = ACTIONS(1777),
    [sym_ip_udp_2way] = ACTIONS(1777),
    [sym_xml_encode_types] = ACTIONS(1777),
    [sym_xml_encode_char_as_list] = ACTIONS(1777),
    [sym_xml_encode_le] = ACTIONS(1777),
    [sym_xml_decode_types] = ACTIONS(1777),
    [sym_xml_decode_no_preserve] = ACTIONS(1777),
    [sym_duet_dev_type_display_device] = ACTIONS(1777),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_io_device] = ACTIONS(1777),
    [sym_duet_dev_type_relay_device] = ACTIONS(1777),
    [sym_duet_dev_type_ups] = ACTIONS(1777),
    [sym_duet_dev_type_amplifier] = ACTIONS(1777),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1777),
    [sym_duet_dev_type_camera] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1777),
    [sym_duet_dev_type_dss] = ACTIONS(1777),
    [sym_duet_dev_type_dvr] = ACTIONS(1777),
    [sym_duet_dev_type_disc_device] = ACTIONS(1777),
    [sym_duet_dev_type_document_camera] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1777),
    [sym_duet_dev_type_hvac] = ACTIONS(1777),
    [sym_duet_dev_type_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_light] = ACTIONS(1777),
    [sym_duet_dev_type_monitor] = ACTIONS(1777),
    [sym_duet_dev_type_motor] = ACTIONS(1777),
    [sym_duet_dev_type_multi_window] = ACTIONS(1777),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1777),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1777),
    [sym_duet_dev_type_receiver] = ACTIONS(1777),
    [sym_duet_dev_type_security_system] = ACTIONS(1777),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1777),
    [sym_duet_dev_type_settop_box] = ACTIONS(1777),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1777),
    [sym_duet_dev_type_switcher] = ACTIONS(1777),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_tv] = ACTIONS(1777),
    [sym_duet_dev_type_utility] = ACTIONS(1777),
    [sym_duet_dev_type_vcr] = ACTIONS(1777),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_video_processor] = ACTIONS(1777),
    [sym_duet_dev_type_video_projector] = ACTIONS(1777),
    [sym_duet_dev_type_video_wall] = ACTIONS(1777),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1777),
    [sym_duet_dev_type_weather] = ACTIONS(1777),
    [sym_duet_dev_not_polled] = ACTIONS(1777),
    [sym_duet_dev_polled] = ACTIONS(1777),
    [sym_clkmgr_mode_network] = ACTIONS(1777),
    [sym_clkmgr_mode_standalone] = ACTIONS(1777),
    [sym_normal_standby] = ACTIONS(1777),
    [sym_normal_wake] = ACTIONS(1777),
    [sym_ip_multicast_ttl_option] = ACTIONS(1777),
    [sym_ip_ttl_subnet] = ACTIONS(1777),
    [sym_ip_ttl_site] = ACTIONS(1777),
    [sym_ip_ttl_region] = ACTIONS(1777),
    [sym_ip_ttl_continent] = ACTIONS(1777),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1777),
    [sym_ip_nodelay_on] = ACTIONS(1777),
    [sym_ip_nodelay_off] = ACTIONS(1777),
    [sym_amx_error] = ACTIONS(1777),
    [sym_amx_warning] = ACTIONS(1777),
    [sym_amx_info] = ACTIONS(1777),
    [sym_amx_debug] = ACTIONS(1777),
    [sym_netlinx_login_success] = ACTIONS(1777),
    [sym_netlinx_login_fail] = ACTIONS(1777),
    [sym_netlinx_logout] = ACTIONS(1777),
    [sym_null_str] = ACTIONS(1777),
    [sym_smtp_address] = ACTIONS(1777),
    [sym_smtp_port_number] = ACTIONS(1777),
    [sym_smtp_username] = ACTIONS(1777),
    [sym_smtp_password] = ACTIONS(1777),
    [sym_smtp_require_tls] = ACTIONS(1777),
    [sym_smtp_from] = ACTIONS(1777),
    [sym_smtp_tls_true] = ACTIONS(1777),
    [sym_smtp_tls_false] = ACTIONS(1777),
    [sym_listview_on_row_select_event] = ACTIONS(1777),
    [sym_data_structure_unknown] = ACTIONS(1777),
    [sym_data_structure_datafeed] = ACTIONS(1777),
    [sym_data_structure_datarecordset] = ACTIONS(1777),
    [sym_data_structure_datarecord] = ACTIONS(1777),
    [sym_data_structure_datafield] = ACTIONS(1777),
    [sym_data_max_content_field_count] = ACTIONS(1777),
    [sym_data_max_metadata_field_count] = ACTIONS(1777),
    [sym_data_max_id_length] = ACTIONS(1777),
    [sym_data_max_name_length] = ACTIONS(1777),
    [sym_data_max_description_length] = ACTIONS(1777),
    [sym_data_max_type_length] = ACTIONS(1777),
    [sym_data_max_format_length] = ACTIONS(1777),
    [sym_data_max_source_length] = ACTIONS(1777),
    [sym_data_max_value_length] = ACTIONS(1777),
    [sym_data_max_label_length] = ACTIONS(1777),
    [sym_data_max_url_length] = ACTIONS(1777),
    [sym_max_last_login_info_length] = ACTIONS(1777),
    [sym_data_type_unknown] = ACTIONS(1777),
    [sym_data_type_string] = ACTIONS(1777),
    [sym_data_type_datetime] = ACTIONS(1777),
    [sym_data_type_date] = ACTIONS(1777),
    [sym_data_type_time] = ACTIONS(1777),
    [sym_data_type_image] = ACTIONS(1777),
    [sym_data_format_url] = ACTIONS(1777),
    [sym_data_format_phone] = ACTIONS(1777),
    [sym_data_format_email] = ACTIONS(1777),
    [sym_data_format_iso8601] = ACTIONS(1777),
    [sym_auditlog_priv] = ACTIONS(1777),
    [sym_configuration_priv] = ACTIONS(1777),
    [sym_deviceconfig_priv] = ACTIONS(1777),
    [sym_ftp_priv] = ACTIONS(1777),
    [sym_http_priv] = ACTIONS(1777),
    [sym_networkconfig_priv] = ACTIONS(1777),
    [sym_programport_priv] = ACTIONS(1777),
    [sym_remoteui_priv] = ACTIONS(1777),
    [sym_securitycontrol_priv] = ACTIONS(1777),
    [sym_softwaremanagement_priv] = ACTIONS(1777),
    [sym_terminal_priv] = ACTIONS(1777),
    [sym_tpadmin_priv] = ACTIONS(1777),
    [sym_user1_priv] = ACTIONS(1777),
    [sym_user2_priv] = ACTIONS(1777),
    [sym_user3_priv] = ACTIONS(1777),
    [sym_user4_priv] = ACTIONS(1777),
    [sym_usermanagement_priv] = ACTIONS(1777),
    [sym_valid_account] = ACTIONS(1777),
    [sym_err_name_invalid] = ACTIONS(1777),
    [sym_err_password_invalid] = ACTIONS(1777),
    [sym_err_invalid_account] = ACTIONS(1777),
    [sym_err_no_authorization] = ACTIONS(1777),
    [sym_err_invalid_parameter] = ACTIONS(1777),
    [sym_err_user_locked_out] = ACTIONS(1777),
    [sym_err_user_account_expired] = ACTIONS(1777),
    [sym_err_general_error] = ACTIONS(1777),
    [sym_err_authentication_error] = ACTIONS(1777),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1777),
    [sym_tls_validate_certificate] = ACTIONS(1777),
    [sym_tdata] = ACTIONS(1777),
    [sym_tchannel] = ACTIONS(1777),
    [sym_tlevel] = ACTIONS(1777),
    [sym_tbutton] = ACTIONS(1777),
    [sym_ttimeline] = ACTIONS(1777),
    [sym_tcustom] = ACTIONS(1777),
    [sym_url_struct] = ACTIONS(1777),
    [sym_dns_struct] = ACTIONS(1777),
    [sym_ip_address_struct] = ACTIONS(1777),
    [sym_dev_info_struct] = ACTIONS(1777),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1777),
    [sym_data_feed] = ACTIONS(1777),
    [sym_data_field] = ACTIONS(1777),
    [sym_data_record] = ACTIONS(1777),
    [sym_wc_data_feed] = ACTIONS(1777),
    [sym_wc_data_field] = ACTIONS(1777),
    [sym_wc_data_record] = ACTIONS(1777),
    [sym_last_login_info] = ACTIONS(1777),
    [sym____reserved___] = ACTIONS(1777),
    [sym_date] = ACTIONS(1777),
    [sym_day] = ACTIONS(1777),
    [sym_time] = ACTIONS(1777),
    [sym_ldate] = ACTIONS(1777),
    [sym_push_device] = ACTIONS(1777),
    [sym_push_channel] = ACTIONS(1777),
    [sym_push_devchan] = ACTIONS(1777),
    [sym_release_device] = ACTIONS(1777),
    [sym_release_channel] = ACTIONS(1777),
    [sym_release_devchan] = ACTIONS(1777),
    [sym_master_slot] = ACTIONS(1777),
    [sym_get_pulse_time] = ACTIONS(1777),
    [sym_get_timer] = ACTIONS(1777),
    [sym_master_sn] = ACTIONS(1777),
    [sym_system_number] = ACTIONS(1777),
    [sym_button] = ACTIONS(1777),
    [sym_dv_channel] = ACTIONS(1777),
    [sym_data] = ACTIONS(1777),
    [sym_level] = ACTIONS(1777),
    [sym_timeline] = ACTIONS(1777),
    [sym_channel] = ACTIONS(1777),
    [sym_custom] = ACTIONS(1777),
    [sym_atoi] = ACTIONS(1777),
    [sym_atol] = ACTIONS(1777),
    [sym_atof] = ACTIONS(1777),
    [sym_itoa] = ACTIONS(1777),
    [sym_itohex] = ACTIONS(1777),
    [sym_hextoi] = ACTIONS(1777),
    [sym_ftoa] = ACTIONS(1777),
    [sym_find_string] = ACTIONS(1777),
    [sym_left_string] = ACTIONS(1777),
    [sym_right_string] = ACTIONS(1777),
    [sym_mid_string] = ACTIONS(1777),
    [sym_remove_string] = ACTIONS(1777),
    [sym_get_buffer_string] = ACTIONS(1777),
    [sym_get_buffer_char] = ACTIONS(1777),
    [sym_get_multi_buffer_string] = ACTIONS(1777),
    [sym_length_string] = ACTIONS(1777),
    [sym_max_length_string] = ACTIONS(1777),
    [sym_set_length_string] = ACTIONS(1777),
    [sym_compare_string] = ACTIONS(1777),
    [sym_upper_string] = ACTIONS(1777),
    [sym_lower_string] = ACTIONS(1777),
    [sym_get_last] = ACTIONS(1777),
    [sym_date_to_day] = ACTIONS(1777),
    [sym_date_to_month] = ACTIONS(1777),
    [sym_date_to_year] = ACTIONS(1777),
    [sym_time_to_hour] = ACTIONS(1777),
    [sym_time_to_minute] = ACTIONS(1777),
    [sym_time_to_second] = ACTIONS(1777),
    [sym_day_of_week] = ACTIONS(1777),
    [sym_set_pulse_time] = ACTIONS(1777),
    [sym_set_timer] = ACTIONS(1777),
    [sym_file_open] = ACTIONS(1777),
    [sym_file_close] = ACTIONS(1777),
    [sym_file_read] = ACTIONS(1777),
    [sym_file_read_line] = ACTIONS(1777),
    [sym_file_write] = ACTIONS(1777),
    [sym_file_write_line] = ACTIONS(1777),
    [sym_file_copy] = ACTIONS(1777),
    [sym_file_delete] = ACTIONS(1777),
    [sym_file_rename] = ACTIONS(1777),
    [sym_file_dir] = ACTIONS(1777),
    [sym_file_getdir] = ACTIONS(1777),
    [sym_file_setdir] = ACTIONS(1777),
    [sym_file_create_dir] = ACTIONS(1777),
    [sym_file_remove_dir] = ACTIONS(1777),
    [sym_file_seek] = ACTIONS(1777),
    [sym_length_array] = ACTIONS(1777),
    [sym_max_length_array] = ACTIONS(1777),
    [sym_set_length_array] = ACTIONS(1777),
    [sym_ip_client_open] = ACTIONS(1777),
    [sym_ip_client_close] = ACTIONS(1777),
    [sym_ip_server_open] = ACTIONS(1777),
    [sym_ip_server_close] = ACTIONS(1777),
    [sym_ip_bound_client_open] = ACTIONS(1777),
    [sym_ip_mc_server_open] = ACTIONS(1777),
    [sym_ip_set_option] = ACTIONS(1777),
    [sym_ssh_client_open] = ACTIONS(1777),
    [sym_ssh_client_close] = ACTIONS(1777),
    [sym_tls_client_open] = ACTIONS(1777),
    [sym_tls_client_close] = ACTIONS(1777),
    [sym_do_push] = ACTIONS(1777),
    [sym_do_push_timed] = ACTIONS(1777),
    [sym_do_release] = ACTIONS(1777),
    [sym_device_id] = ACTIONS(1777),
    [sym_device_info] = ACTIONS(1777),
    [sym_device_id_string] = ACTIONS(1777),
    [sym_device_standby] = ACTIONS(1777),
    [sym_device_wake] = ACTIONS(1777),
    [sym_timeline_create] = ACTIONS(1777),
    [sym_timeline_kill] = ACTIONS(1777),
    [sym_timeline_reload] = ACTIONS(1777),
    [sym_timeline_pause] = ACTIONS(1777),
    [sym_timeline_restart] = ACTIONS(1777),
    [sym_timeline_set] = ACTIONS(1777),
    [sym_timeline_get] = ACTIONS(1777),
    [sym_timeline_active] = ACTIONS(1777),
    [sym_timeline_dynamic_id] = ACTIONS(1777),
    [sym_variable_to_string] = ACTIONS(1777),
    [sym_string_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_string] = ACTIONS(1777),
    [sym_type_cast] = ACTIONS(1777),
    [sym_variable_to_xml] = ACTIONS(1777),
    [sym_xml_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_xml] = ACTIONS(1777),
    [sym_redirect_string] = ACTIONS(1777),
    [sym_random_number] = ACTIONS(1777),
    [sym_get_unique_id] = ACTIONS(1777),
    [sym_get_system_number] = ACTIONS(1777),
    [sym_set_system_number] = ACTIONS(1777),
    [sym_get_serial_number] = ACTIONS(1777),
    [sym_get_master_build] = ACTIONS(1777),
    [sym_set_validation_code] = ACTIONS(1777),
    [sym_reboot] = ACTIONS(1777),
    [sym_load_duet_module] = ACTIONS(1777),
    [sym_unload_duet_module] = ACTIONS(1777),
    [sym_duet_mem_size_set] = ACTIONS(1777),
    [sym_duet_mem_size_get] = ACTIONS(1777),
    [sym_dynamic_application_device] = ACTIONS(1777),
    [sym_dynamic_polled_port] = ACTIONS(1777),
    [sym_static_port_binding] = ACTIONS(1777),
    [sym_static_ip_binding] = ACTIONS(1777),
    [sym_get_dns_list] = ACTIONS(1777),
    [sym_set_dns_list] = ACTIONS(1777),
    [sym_get_ip_address] = ACTIONS(1777),
    [sym_set_ip_address] = ACTIONS(1777),
    [sym_get_url_list] = ACTIONS(1777),
    [sym_add_url_entry] = ACTIONS(1777),
    [sym_delete_url_entry] = ACTIONS(1777),
    [sym_abs_value] = ACTIONS(1777),
    [sym_max_value] = ACTIONS(1777),
    [sym_min_value] = ACTIONS(1777),
    [sym_exp_value] = ACTIONS(1777),
    [sym_log_value] = ACTIONS(1777),
    [sym_log10_value] = ACTIONS(1777),
    [sym_power_value] = ACTIONS(1777),
    [sym_sqrt_value] = ACTIONS(1777),
    [sym_raw_be] = ACTIONS(1777),
    [sym_raw_le] = ACTIONS(1777),
    [sym_internal_queue_size_set] = ACTIONS(1777),
    [sym_internal_queue_size_get] = ACTIONS(1777),
    [sym_internal_threshold_set] = ACTIONS(1777),
    [sym_internal_threshold_get] = ACTIONS(1777),
    [sym_rebuilt_event] = ACTIONS(1777),
    [sym_do_custom_event] = ACTIONS(1777),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1777),
    [sym_clkmgr_set_clk_source] = ACTIONS(1777),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1777),
    [sym_clkmgr_get_timezone] = ACTIONS(1777),
    [sym_clkmgr_set_timezone] = ACTIONS(1777),
    [sym_clkmgr_get_resync_period] = ACTIONS(1777),
    [sym_clkmgr_set_resync_period] = ACTIONS(1777),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_timeservers] = ACTIONS(1777),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_set_log_level] = ACTIONS(1777),
    [sym_get_log_level] = ACTIONS(1777),
    [sym_amx_log] = ACTIONS(1777),
    [sym_smtp_server_config_set] = ACTIONS(1777),
    [sym_smtp_server_config_get] = ACTIONS(1777),
    [sym_smtp_send] = ACTIONS(1777),
    [sym_validate_netlinx_account] = ACTIONS(1777),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1777),
    [sym_authenticate_certificate] = ACTIONS(1777),
    [sym_audit_netlinx_session_event] = ACTIONS(1777),
    [sym_audit_netlinx_generic_event] = ACTIONS(1777),
    [sym_get_available_flash_disk_space] = ACTIONS(1777),
    [sym_get_max_flash_disk_space] = ACTIONS(1777),
    [sym_data_create_feed] = ACTIONS(1777),
    [sym_data_delete_feed] = ACTIONS(1777),
    [sym_data_publish_feed] = ACTIONS(1777),
    [sym_data_get_published_feed] = ACTIONS(1777),
    [sym_data_add_record] = ACTIONS(1777),
    [sym_data_get_event_record] = ACTIONS(1777),
    [sym__wc_data_create_feed] = ACTIONS(1777),
    [sym__wc_data_add_record] = ACTIONS(1777),
    [sym__wc_data_get_event_record] = ACTIONS(1777),
    [sym_format] = ACTIONS(1777),
    [sym_astro_clock] = ACTIONS(1777),
    [sym_set_outdoor_temperature] = ACTIONS(1777),
    [sym_set_virtual_level_count] = ACTIONS(1777),
    [sym_set_virtual_channel_count] = ACTIONS(1777),
    [sym_set_virtual_port_count] = ACTIONS(1777),
    [sym__wc_encode_int] = ACTIONS(1777),
    [sym__wc_decode_int] = ACTIONS(1777),
    [sym__wc_int] = ACTIONS(1777),
    [sym__wc_to_ch_int] = ACTIONS(1777),
    [sym__ch_to_wc_int] = ACTIONS(1777),
    [sym__wc_find_string_int] = ACTIONS(1777),
    [sym__wc_left_string_int] = ACTIONS(1777),
    [sym__wc_lower_string_int] = ACTIONS(1777),
    [sym__wc_mid_string_int] = ACTIONS(1777),
    [sym__wc_remove_string_int] = ACTIONS(1777),
    [sym__wc_right_string_int] = ACTIONS(1777),
    [sym__wc_upper_string_int] = ACTIONS(1777),
    [sym__wc_compare_string_int] = ACTIONS(1777),
    [sym__wc_get_buffer_char_int] = ACTIONS(1777),
    [sym__wc_get_buffer_string_int] = ACTIONS(1777),
    [sym__wc_concat_string_int] = ACTIONS(1777),
    [sym__wc_file_open_int] = ACTIONS(1777),
    [sym__wc_file_close_int] = ACTIONS(1777),
    [sym__wc_file_read_int] = ACTIONS(1777),
    [sym__wc_file_read_line_int] = ACTIONS(1777),
    [sym__wc_file_write_int] = ACTIONS(1777),
    [sym__wc_file_write_line_int] = ACTIONS(1777),
    [aux_sym_preproc_include_token1] = ACTIONS(1777),
    [aux_sym_preproc_define_token1] = ACTIONS(1779),
    [aux_sym_preproc_warn_token1] = ACTIONS(1779),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1779),
    [anon_sym_LBRACK] = ACTIONS(1779),
    [anon_sym_LBRACE] = ACTIONS(1779),
    [anon_sym_LPAREN] = ACTIONS(1777),
    [anon_sym_BANG] = ACTIONS(1779),
    [anon_sym_TILDE] = ACTIONS(1779),
    [anon_sym_DASH] = ACTIONS(1777),
    [anon_sym_PLUS] = ACTIONS(1777),
    [anon_sym_DASH_DASH] = ACTIONS(1779),
    [anon_sym_PLUS_PLUS] = ACTIONS(1779),
    [anon_sym_DQUOTE] = ACTIONS(1779),
    [anon_sym_SQUOTE] = ACTIONS(1779),
    [sym_number_literal] = ACTIONS(1779),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(305)] = {
    [sym_identifier] = ACTIONS(1973),
    [sym_program_name_keyword] = ACTIONS(1973),
    [sym_module_name_keyword] = ACTIONS(1973),
    [sym_define_device_keyword] = ACTIONS(1973),
    [sym_define_combine_keyword] = ACTIONS(1973),
    [sym_define_constant_keyword] = ACTIONS(1973),
    [sym_define_type_keyword] = ACTIONS(1973),
    [sym_define_variable_keyword] = ACTIONS(1973),
    [sym_define_system_variable_keyword] = ACTIONS(1973),
    [sym_define_connect_level_keyword] = ACTIONS(1973),
    [sym_define_latching_keyword] = ACTIONS(1973),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1973),
    [sym_define_toggling_keyword] = ACTIONS(1973),
    [sym_define_call_keyword] = ACTIONS(1973),
    [sym_define_function_keyword] = ACTIONS(1973),
    [sym_define_library_function_keyword] = ACTIONS(1973),
    [sym_define_start_keyword] = ACTIONS(1973),
    [sym_define_event_keyword] = ACTIONS(1973),
    [sym_define_program_keyword] = ACTIONS(1973),
    [sym_define_module_keyword] = ACTIONS(1973),
    [sym_char_keyword] = ACTIONS(1973),
    [sym_widechar_keyword] = ACTIONS(1973),
    [sym_integer_keyword] = ACTIONS(1973),
    [sym_sinteger_keyword] = ACTIONS(1973),
    [sym_long_keyword] = ACTIONS(1973),
    [sym_slong_keyword] = ACTIONS(1973),
    [sym_float_keyword] = ACTIONS(1973),
    [sym_double_keyword] = ACTIONS(1973),
    [sym_variant_keyword] = ACTIONS(1973),
    [sym_variantarray_keyword] = ACTIONS(1973),
    [sym_local_var_keyword] = ACTIONS(1973),
    [sym_stack_var_keyword] = ACTIONS(1973),
    [sym_constant_keyword] = ACTIONS(1973),
    [sym_volatile_keyword] = ACTIONS(1973),
    [sym_non_volatile_keyword] = ACTIONS(1973),
    [sym_persistent_keyword] = ACTIONS(1973),
    [sym_dev_keyword] = ACTIONS(1973),
    [sym_devlev_keyword] = ACTIONS(1973),
    [sym_devchan_keyword] = ACTIONS(1973),
    [sym_if_keyword] = ACTIONS(1973),
    [sym_switch_keyword] = ACTIONS(1973),
    [sym_case_keyword] = ACTIONS(1973),
    [sym_default_keyword] = ACTIONS(1973),
    [sym_while_keyword] = ACTIONS(1973),
    [sym_for_keyword] = ACTIONS(1973),
    [sym_break_keyword] = ACTIONS(1973),
    [sym_continue_keyword] = ACTIONS(1973),
    [sym_return_keyword] = ACTIONS(1973),
    [sym_select_keyword] = ACTIONS(1973),
    [sym_active_keyword] = ACTIONS(1973),
    [sym_struct_keyword] = ACTIONS(1973),
    [sym_structure_keyword] = ACTIONS(1973),
    [sym_button_event_keyword] = ACTIONS(1973),
    [sym_channel_event_keyword] = ACTIONS(1973),
    [sym_level_event_keyword] = ACTIONS(1973),
    [sym_data_event_keyword] = ACTIONS(1973),
    [sym_timeline_event_keyword] = ACTIONS(1973),
    [sym_custom_event_keyword] = ACTIONS(1973),
    [sym_push_keyword] = ACTIONS(1973),
    [sym_release_keyword] = ACTIONS(1973),
    [aux_sym_on_keyword_token1] = ACTIONS(1973),
    [aux_sym_off_keyword_token1] = ACTIONS(1973),
    [sym_send_level_keyword] = ACTIONS(1973),
    [sym_send_string_keyword] = ACTIONS(1973),
    [sym_send_command_keyword] = ACTIONS(1973),
    [sym_clear_buffer_keyword] = ACTIONS(1973),
    [sym_create_buffer_keyword] = ACTIONS(1973),
    [sym_create_multi_buffer_keyword] = ACTIONS(1973),
    [sym_call_keyword] = ACTIONS(1973),
    [sym_system_call_keyword] = ACTIONS(1973),
    [sym_devchan_to_keyword] = ACTIONS(1973),
    [sym_devchan_min_to_keyword] = ACTIONS(1973),
    [sym_devchan_total_off_keyword] = ACTIONS(1973),
    [sym_devchan_pulse_keyword] = ACTIONS(1973),
    [sym_wait_keyword] = ACTIONS(1973),
    [sym_cancel_wait_keyword] = ACTIONS(1973),
    [sym_wait_until_keyword] = ACTIONS(1973),
    [sym_cancel_wait_until_keyword] = ACTIONS(1973),
    [sym_cancel_all_wait_keyword] = ACTIONS(1973),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1973),
    [sym_not] = ACTIONS(1973),
    [sym_bnot] = ACTIONS(1973),
    [sym___file__] = ACTIONS(1973),
    [sym___date__] = ACTIONS(1973),
    [sym___ldate__] = ACTIONS(1973),
    [sym___time__] = ACTIONS(1973),
    [sym___line__] = ACTIONS(1973),
    [sym___name__] = ACTIONS(1973),
    [sym___netlinx__] = ACTIONS(1973),
    [sym_netlinx_axi_version] = ACTIONS(1973),
    [sym_url_flg_tcp] = ACTIONS(1973),
    [sym_url_flg_accinfopresent] = ACTIONS(1973),
    [sym_url_flg_temp] = ACTIONS(1973),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1973),
    [sym_url_flg_stat_mask] = ACTIONS(1973),
    [sym_url_flg_stat_lookup] = ACTIONS(1973),
    [sym_url_flg_stat_connecting] = ACTIONS(1973),
    [sym_url_flg_stat_waiting] = ACTIONS(1973),
    [sym_url_flg_stat_connected] = ACTIONS(1973),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1973),
    [sym_first_virtual_device] = ACTIONS(1973),
    [sym_first_local_port] = ACTIONS(1973),
    [sym_dynamic_virtual_device] = ACTIONS(1973),
    [sym_timeline_once] = ACTIONS(1973),
    [sym_timeline_repeat] = ACTIONS(1973),
    [sym_timeline_absolute] = ACTIONS(1973),
    [sym_timeline_relative] = ACTIONS(1973),
    [sym_do_push_timed_infinite] = ACTIONS(1973),
    [sym_source_type_no_address] = ACTIONS(1973),
    [sym_source_type_neuron_id] = ACTIONS(1973),
    [sym_source_type_ip_address] = ACTIONS(1973),
    [sym_source_type_axlink] = ACTIONS(1973),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1973),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1973),
    [sym_source_type_ip_socket_address] = ACTIONS(1973),
    [sym_source_type_rs232] = ACTIONS(1973),
    [sym_source_type_internal] = ACTIONS(1973),
    [sym_source_type_mac_address] = ACTIONS(1973),
    [sym_source_type_ipv4_port] = ACTIONS(1973),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1973),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1973),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1973),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1973),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1973),
    [sym_true] = ACTIONS(1973),
    [sym_false] = ACTIONS(1973),
    [sym_file_read_only] = ACTIONS(1973),
    [sym_file_rw_new] = ACTIONS(1973),
    [sym_file_rw_append] = ACTIONS(1973),
    [sym_ip_tcp] = ACTIONS(1973),
    [sym_ip_udp] = ACTIONS(1973),
    [sym_ip_udp_2way] = ACTIONS(1973),
    [sym_xml_encode_types] = ACTIONS(1973),
    [sym_xml_encode_char_as_list] = ACTIONS(1973),
    [sym_xml_encode_le] = ACTIONS(1973),
    [sym_xml_decode_types] = ACTIONS(1973),
    [sym_xml_decode_no_preserve] = ACTIONS(1973),
    [sym_duet_dev_type_display_device] = ACTIONS(1973),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1973),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1973),
    [sym_duet_dev_type_io_device] = ACTIONS(1973),
    [sym_duet_dev_type_relay_device] = ACTIONS(1973),
    [sym_duet_dev_type_ups] = ACTIONS(1973),
    [sym_duet_dev_type_amplifier] = ACTIONS(1973),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1973),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1973),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1973),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1973),
    [sym_duet_dev_type_camera] = ACTIONS(1973),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1973),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1973),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1973),
    [sym_duet_dev_type_dss] = ACTIONS(1973),
    [sym_duet_dev_type_dvr] = ACTIONS(1973),
    [sym_duet_dev_type_disc_device] = ACTIONS(1973),
    [sym_duet_dev_type_document_camera] = ACTIONS(1973),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1973),
    [sym_duet_dev_type_hvac] = ACTIONS(1973),
    [sym_duet_dev_type_keypad] = ACTIONS(1973),
    [sym_duet_dev_type_light] = ACTIONS(1973),
    [sym_duet_dev_type_monitor] = ACTIONS(1973),
    [sym_duet_dev_type_motor] = ACTIONS(1973),
    [sym_duet_dev_type_multi_window] = ACTIONS(1973),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1973),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1973),
    [sym_duet_dev_type_receiver] = ACTIONS(1973),
    [sym_duet_dev_type_security_system] = ACTIONS(1973),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1973),
    [sym_duet_dev_type_settop_box] = ACTIONS(1973),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1973),
    [sym_duet_dev_type_switcher] = ACTIONS(1973),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1973),
    [sym_duet_dev_type_tv] = ACTIONS(1973),
    [sym_duet_dev_type_utility] = ACTIONS(1973),
    [sym_duet_dev_type_vcr] = ACTIONS(1973),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1973),
    [sym_duet_dev_type_video_processor] = ACTIONS(1973),
    [sym_duet_dev_type_video_projector] = ACTIONS(1973),
    [sym_duet_dev_type_video_wall] = ACTIONS(1973),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1973),
    [sym_duet_dev_type_weather] = ACTIONS(1973),
    [sym_duet_dev_not_polled] = ACTIONS(1973),
    [sym_duet_dev_polled] = ACTIONS(1973),
    [sym_clkmgr_mode_network] = ACTIONS(1973),
    [sym_clkmgr_mode_standalone] = ACTIONS(1973),
    [sym_normal_standby] = ACTIONS(1973),
    [sym_normal_wake] = ACTIONS(1973),
    [sym_ip_multicast_ttl_option] = ACTIONS(1973),
    [sym_ip_ttl_subnet] = ACTIONS(1973),
    [sym_ip_ttl_site] = ACTIONS(1973),
    [sym_ip_ttl_region] = ACTIONS(1973),
    [sym_ip_ttl_continent] = ACTIONS(1973),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1973),
    [sym_ip_nodelay_on] = ACTIONS(1973),
    [sym_ip_nodelay_off] = ACTIONS(1973),
    [sym_amx_error] = ACTIONS(1973),
    [sym_amx_warning] = ACTIONS(1973),
    [sym_amx_info] = ACTIONS(1973),
    [sym_amx_debug] = ACTIONS(1973),
    [sym_netlinx_login_success] = ACTIONS(1973),
    [sym_netlinx_login_fail] = ACTIONS(1973),
    [sym_netlinx_logout] = ACTIONS(1973),
    [sym_null_str] = ACTIONS(1973),
    [sym_smtp_address] = ACTIONS(1973),
    [sym_smtp_port_number] = ACTIONS(1973),
    [sym_smtp_username] = ACTIONS(1973),
    [sym_smtp_password] = ACTIONS(1973),
    [sym_smtp_require_tls] = ACTIONS(1973),
    [sym_smtp_from] = ACTIONS(1973),
    [sym_smtp_tls_true] = ACTIONS(1973),
    [sym_smtp_tls_false] = ACTIONS(1973),
    [sym_listview_on_row_select_event] = ACTIONS(1973),
    [sym_data_structure_unknown] = ACTIONS(1973),
    [sym_data_structure_datafeed] = ACTIONS(1973),
    [sym_data_structure_datarecordset] = ACTIONS(1973),
    [sym_data_structure_datarecord] = ACTIONS(1973),
    [sym_data_structure_datafield] = ACTIONS(1973),
    [sym_data_max_content_field_count] = ACTIONS(1973),
    [sym_data_max_metadata_field_count] = ACTIONS(1973),
    [sym_data_max_id_length] = ACTIONS(1973),
    [sym_data_max_name_length] = ACTIONS(1973),
    [sym_data_max_description_length] = ACTIONS(1973),
    [sym_data_max_type_length] = ACTIONS(1973),
    [sym_data_max_format_length] = ACTIONS(1973),
    [sym_data_max_source_length] = ACTIONS(1973),
    [sym_data_max_value_length] = ACTIONS(1973),
    [sym_data_max_label_length] = ACTIONS(1973),
    [sym_data_max_url_length] = ACTIONS(1973),
    [sym_max_last_login_info_length] = ACTIONS(1973),
    [sym_data_type_unknown] = ACTIONS(1973),
    [sym_data_type_string] = ACTIONS(1973),
    [sym_data_type_datetime] = ACTIONS(1973),
    [sym_data_type_date] = ACTIONS(1973),
    [sym_data_type_time] = ACTIONS(1973),
    [sym_data_type_image] = ACTIONS(1973),
    [sym_data_format_url] = ACTIONS(1973),
    [sym_data_format_phone] = ACTIONS(1973),
    [sym_data_format_email] = ACTIONS(1973),
    [sym_data_format_iso8601] = ACTIONS(1973),
    [sym_auditlog_priv] = ACTIONS(1973),
    [sym_configuration_priv] = ACTIONS(1973),
    [sym_deviceconfig_priv] = ACTIONS(1973),
    [sym_ftp_priv] = ACTIONS(1973),
    [sym_http_priv] = ACTIONS(1973),
    [sym_networkconfig_priv] = ACTIONS(1973),
    [sym_programport_priv] = ACTIONS(1973),
    [sym_remoteui_priv] = ACTIONS(1973),
    [sym_securitycontrol_priv] = ACTIONS(1973),
    [sym_softwaremanagement_priv] = ACTIONS(1973),
    [sym_terminal_priv] = ACTIONS(1973),
    [sym_tpadmin_priv] = ACTIONS(1973),
    [sym_user1_priv] = ACTIONS(1973),
    [sym_user2_priv] = ACTIONS(1973),
    [sym_user3_priv] = ACTIONS(1973),
    [sym_user4_priv] = ACTIONS(1973),
    [sym_usermanagement_priv] = ACTIONS(1973),
    [sym_valid_account] = ACTIONS(1973),
    [sym_err_name_invalid] = ACTIONS(1973),
    [sym_err_password_invalid] = ACTIONS(1973),
    [sym_err_invalid_account] = ACTIONS(1973),
    [sym_err_no_authorization] = ACTIONS(1973),
    [sym_err_invalid_parameter] = ACTIONS(1973),
    [sym_err_user_locked_out] = ACTIONS(1973),
    [sym_err_user_account_expired] = ACTIONS(1973),
    [sym_err_general_error] = ACTIONS(1973),
    [sym_err_authentication_error] = ACTIONS(1973),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1973),
    [sym_tls_validate_certificate] = ACTIONS(1973),
    [sym_tdata] = ACTIONS(1973),
    [sym_tchannel] = ACTIONS(1973),
    [sym_tlevel] = ACTIONS(1973),
    [sym_tbutton] = ACTIONS(1973),
    [sym_ttimeline] = ACTIONS(1973),
    [sym_tcustom] = ACTIONS(1973),
    [sym_url_struct] = ACTIONS(1973),
    [sym_dns_struct] = ACTIONS(1973),
    [sym_ip_address_struct] = ACTIONS(1973),
    [sym_dev_info_struct] = ACTIONS(1973),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1973),
    [sym_data_feed] = ACTIONS(1973),
    [sym_data_field] = ACTIONS(1973),
    [sym_data_record] = ACTIONS(1973),
    [sym_wc_data_feed] = ACTIONS(1973),
    [sym_wc_data_field] = ACTIONS(1973),
    [sym_wc_data_record] = ACTIONS(1973),
    [sym_last_login_info] = ACTIONS(1973),
    [sym____reserved___] = ACTIONS(1973),
    [sym_date] = ACTIONS(1973),
    [sym_day] = ACTIONS(1973),
    [sym_time] = ACTIONS(1973),
    [sym_ldate] = ACTIONS(1973),
    [sym_push_device] = ACTIONS(1973),
    [sym_push_channel] = ACTIONS(1973),
    [sym_push_devchan] = ACTIONS(1973),
    [sym_release_device] = ACTIONS(1973),
    [sym_release_channel] = ACTIONS(1973),
    [sym_release_devchan] = ACTIONS(1973),
    [sym_master_slot] = ACTIONS(1973),
    [sym_get_pulse_time] = ACTIONS(1973),
    [sym_get_timer] = ACTIONS(1973),
    [sym_master_sn] = ACTIONS(1973),
    [sym_system_number] = ACTIONS(1973),
    [sym_button] = ACTIONS(1973),
    [sym_dv_channel] = ACTIONS(1973),
    [sym_data] = ACTIONS(1973),
    [sym_level] = ACTIONS(1973),
    [sym_timeline] = ACTIONS(1973),
    [sym_channel] = ACTIONS(1973),
    [sym_custom] = ACTIONS(1973),
    [sym_atoi] = ACTIONS(1973),
    [sym_atol] = ACTIONS(1973),
    [sym_atof] = ACTIONS(1973),
    [sym_itoa] = ACTIONS(1973),
    [sym_itohex] = ACTIONS(1973),
    [sym_hextoi] = ACTIONS(1973),
    [sym_ftoa] = ACTIONS(1973),
    [sym_find_string] = ACTIONS(1973),
    [sym_left_string] = ACTIONS(1973),
    [sym_right_string] = ACTIONS(1973),
    [sym_mid_string] = ACTIONS(1973),
    [sym_remove_string] = ACTIONS(1973),
    [sym_get_buffer_string] = ACTIONS(1973),
    [sym_get_buffer_char] = ACTIONS(1973),
    [sym_get_multi_buffer_string] = ACTIONS(1973),
    [sym_length_string] = ACTIONS(1973),
    [sym_max_length_string] = ACTIONS(1973),
    [sym_set_length_string] = ACTIONS(1973),
    [sym_compare_string] = ACTIONS(1973),
    [sym_upper_string] = ACTIONS(1973),
    [sym_lower_string] = ACTIONS(1973),
    [sym_get_last] = ACTIONS(1973),
    [sym_date_to_day] = ACTIONS(1973),
    [sym_date_to_month] = ACTIONS(1973),
    [sym_date_to_year] = ACTIONS(1973),
    [sym_time_to_hour] = ACTIONS(1973),
    [sym_time_to_minute] = ACTIONS(1973),
    [sym_time_to_second] = ACTIONS(1973),
    [sym_day_of_week] = ACTIONS(1973),
    [sym_set_pulse_time] = ACTIONS(1973),
    [sym_set_timer] = ACTIONS(1973),
    [sym_file_open] = ACTIONS(1973),
    [sym_file_close] = ACTIONS(1973),
    [sym_file_read] = ACTIONS(1973),
    [sym_file_read_line] = ACTIONS(1973),
    [sym_file_write] = ACTIONS(1973),
    [sym_file_write_line] = ACTIONS(1973),
    [sym_file_copy] = ACTIONS(1973),
    [sym_file_delete] = ACTIONS(1973),
    [sym_file_rename] = ACTIONS(1973),
    [sym_file_dir] = ACTIONS(1973),
    [sym_file_getdir] = ACTIONS(1973),
    [sym_file_setdir] = ACTIONS(1973),
    [sym_file_create_dir] = ACTIONS(1973),
    [sym_file_remove_dir] = ACTIONS(1973),
    [sym_file_seek] = ACTIONS(1973),
    [sym_length_array] = ACTIONS(1973),
    [sym_max_length_array] = ACTIONS(1973),
    [sym_set_length_array] = ACTIONS(1973),
    [sym_ip_client_open] = ACTIONS(1973),
    [sym_ip_client_close] = ACTIONS(1973),
    [sym_ip_server_open] = ACTIONS(1973),
    [sym_ip_server_close] = ACTIONS(1973),
    [sym_ip_bound_client_open] = ACTIONS(1973),
    [sym_ip_mc_server_open] = ACTIONS(1973),
    [sym_ip_set_option] = ACTIONS(1973),
    [sym_ssh_client_open] = ACTIONS(1973),
    [sym_ssh_client_close] = ACTIONS(1973),
    [sym_tls_client_open] = ACTIONS(1973),
    [sym_tls_client_close] = ACTIONS(1973),
    [sym_do_push] = ACTIONS(1973),
    [sym_do_push_timed] = ACTIONS(1973),
    [sym_do_release] = ACTIONS(1973),
    [sym_device_id] = ACTIONS(1973),
    [sym_device_info] = ACTIONS(1973),
    [sym_device_id_string] = ACTIONS(1973),
    [sym_device_standby] = ACTIONS(1973),
    [sym_device_wake] = ACTIONS(1973),
    [sym_timeline_create] = ACTIONS(1973),
    [sym_timeline_kill] = ACTIONS(1973),
    [sym_timeline_reload] = ACTIONS(1973),
    [sym_timeline_pause] = ACTIONS(1973),
    [sym_timeline_restart] = ACTIONS(1973),
    [sym_timeline_set] = ACTIONS(1973),
    [sym_timeline_get] = ACTIONS(1973),
    [sym_timeline_active] = ACTIONS(1973),
    [sym_timeline_dynamic_id] = ACTIONS(1973),
    [sym_variable_to_string] = ACTIONS(1973),
    [sym_string_to_variable] = ACTIONS(1973),
    [sym_length_variable_to_string] = ACTIONS(1973),
    [sym_type_cast] = ACTIONS(1973),
    [sym_variable_to_xml] = ACTIONS(1973),
    [sym_xml_to_variable] = ACTIONS(1973),
    [sym_length_variable_to_xml] = ACTIONS(1973),
    [sym_redirect_string] = ACTIONS(1973),
    [sym_random_number] = ACTIONS(1973),
    [sym_get_unique_id] = ACTIONS(1973),
    [sym_get_system_number] = ACTIONS(1973),
    [sym_set_system_number] = ACTIONS(1973),
    [sym_get_serial_number] = ACTIONS(1973),
    [sym_get_master_build] = ACTIONS(1973),
    [sym_set_validation_code] = ACTIONS(1973),
    [sym_reboot] = ACTIONS(1973),
    [sym_load_duet_module] = ACTIONS(1973),
    [sym_unload_duet_module] = ACTIONS(1973),
    [sym_duet_mem_size_set] = ACTIONS(1973),
    [sym_duet_mem_size_get] = ACTIONS(1973),
    [sym_dynamic_application_device] = ACTIONS(1973),
    [sym_dynamic_polled_port] = ACTIONS(1973),
    [sym_static_port_binding] = ACTIONS(1973),
    [sym_static_ip_binding] = ACTIONS(1973),
    [sym_get_dns_list] = ACTIONS(1973),
    [sym_set_dns_list] = ACTIONS(1973),
    [sym_get_ip_address] = ACTIONS(1973),
    [sym_set_ip_address] = ACTIONS(1973),
    [sym_get_url_list] = ACTIONS(1973),
    [sym_add_url_entry] = ACTIONS(1973),
    [sym_delete_url_entry] = ACTIONS(1973),
    [sym_abs_value] = ACTIONS(1973),
    [sym_max_value] = ACTIONS(1973),
    [sym_min_value] = ACTIONS(1973),
    [sym_exp_value] = ACTIONS(1973),
    [sym_log_value] = ACTIONS(1973),
    [sym_log10_value] = ACTIONS(1973),
    [sym_power_value] = ACTIONS(1973),
    [sym_sqrt_value] = ACTIONS(1973),
    [sym_raw_be] = ACTIONS(1973),
    [sym_raw_le] = ACTIONS(1973),
    [sym_internal_queue_size_set] = ACTIONS(1973),
    [sym_internal_queue_size_get] = ACTIONS(1973),
    [sym_internal_threshold_set] = ACTIONS(1973),
    [sym_internal_threshold_get] = ACTIONS(1973),
    [sym_rebuilt_event] = ACTIONS(1973),
    [sym_do_custom_event] = ACTIONS(1973),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1973),
    [sym_clkmgr_set_clk_source] = ACTIONS(1973),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1973),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1973),
    [sym_clkmgr_get_timezone] = ACTIONS(1973),
    [sym_clkmgr_set_timezone] = ACTIONS(1973),
    [sym_clkmgr_get_resync_period] = ACTIONS(1973),
    [sym_clkmgr_set_resync_period] = ACTIONS(1973),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1973),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1973),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1973),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1973),
    [sym_clkmgr_get_timeservers] = ACTIONS(1973),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1973),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1973),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1973),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1973),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1973),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1973),
    [sym_set_log_level] = ACTIONS(1973),
    [sym_get_log_level] = ACTIONS(1973),
    [sym_amx_log] = ACTIONS(1973),
    [sym_smtp_server_config_set] = ACTIONS(1973),
    [sym_smtp_server_config_get] = ACTIONS(1973),
    [sym_smtp_send] = ACTIONS(1973),
    [sym_validate_netlinx_account] = ACTIONS(1973),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1973),
    [sym_authenticate_certificate] = ACTIONS(1973),
    [sym_audit_netlinx_session_event] = ACTIONS(1973),
    [sym_audit_netlinx_generic_event] = ACTIONS(1973),
    [sym_get_available_flash_disk_space] = ACTIONS(1973),
    [sym_get_max_flash_disk_space] = ACTIONS(1973),
    [sym_data_create_feed] = ACTIONS(1973),
    [sym_data_delete_feed] = ACTIONS(1973),
    [sym_data_publish_feed] = ACTIONS(1973),
    [sym_data_get_published_feed] = ACTIONS(1973),
    [sym_data_add_record] = ACTIONS(1973),
    [sym_data_get_event_record] = ACTIONS(1973),
    [sym__wc_data_create_feed] = ACTIONS(1973),
    [sym__wc_data_add_record] = ACTIONS(1973),
    [sym__wc_data_get_event_record] = ACTIONS(1973),
    [sym_format] = ACTIONS(1973),
    [sym_astro_clock] = ACTIONS(1973),
    [sym_set_outdoor_temperature] = ACTIONS(1973),
    [sym_set_virtual_level_count] = ACTIONS(1973),
    [sym_set_virtual_channel_count] = ACTIONS(1973),
    [sym_set_virtual_port_count] = ACTIONS(1973),
    [sym__wc_encode_int] = ACTIONS(1973),
    [sym__wc_decode_int] = ACTIONS(1973),
    [sym__wc_int] = ACTIONS(1973),
    [sym__wc_to_ch_int] = ACTIONS(1973),
    [sym__ch_to_wc_int] = ACTIONS(1973),
    [sym__wc_find_string_int] = ACTIONS(1973),
    [sym__wc_left_string_int] = ACTIONS(1973),
    [sym__wc_lower_string_int] = ACTIONS(1973),
    [sym__wc_mid_string_int] = ACTIONS(1973),
    [sym__wc_remove_string_int] = ACTIONS(1973),
    [sym__wc_right_string_int] = ACTIONS(1973),
    [sym__wc_upper_string_int] = ACTIONS(1973),
    [sym__wc_compare_string_int] = ACTIONS(1973),
    [sym__wc_get_buffer_char_int] = ACTIONS(1973),
    [sym__wc_get_buffer_string_int] = ACTIONS(1973),
    [sym__wc_concat_string_int] = ACTIONS(1973),
    [sym__wc_file_open_int] = ACTIONS(1973),
    [sym__wc_file_close_int] = ACTIONS(1973),
    [sym__wc_file_read_int] = ACTIONS(1973),
    [sym__wc_file_read_line_int] = ACTIONS(1973),
    [sym__wc_file_write_int] = ACTIONS(1973),
    [sym__wc_file_write_line_int] = ACTIONS(1973),
    [aux_sym_preproc_include_token1] = ACTIONS(1973),
    [aux_sym_preproc_define_token1] = ACTIONS(1975),
    [aux_sym_preproc_warn_token1] = ACTIONS(1975),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1975),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1975),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1975),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1975),
    [aux_sym_preproc_else_token1] = ACTIONS(1975),
    [anon_sym_LBRACK] = ACTIONS(1975),
    [anon_sym_LBRACE] = ACTIONS(1975),
    [anon_sym_LPAREN] = ACTIONS(1973),
    [anon_sym_BANG] = ACTIONS(1975),
    [anon_sym_TILDE] = ACTIONS(1975),
    [anon_sym_DASH] = ACTIONS(1973),
    [anon_sym_PLUS] = ACTIONS(1973),
    [anon_sym_DASH_DASH] = ACTIONS(1975),
    [anon_sym_PLUS_PLUS] = ACTIONS(1975),
    [anon_sym_DQUOTE] = ACTIONS(1975),
    [anon_sym_SQUOTE] = ACTIONS(1975),
    [sym_number_literal] = ACTIONS(1975),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(306)] = {
    [sym_identifier] = ACTIONS(1977),
    [sym_program_name_keyword] = ACTIONS(1977),
    [sym_module_name_keyword] = ACTIONS(1977),
    [sym_define_device_keyword] = ACTIONS(1977),
    [sym_define_combine_keyword] = ACTIONS(1977),
    [sym_define_constant_keyword] = ACTIONS(1977),
    [sym_define_type_keyword] = ACTIONS(1977),
    [sym_define_variable_keyword] = ACTIONS(1977),
    [sym_define_system_variable_keyword] = ACTIONS(1977),
    [sym_define_connect_level_keyword] = ACTIONS(1977),
    [sym_define_latching_keyword] = ACTIONS(1977),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1977),
    [sym_define_toggling_keyword] = ACTIONS(1977),
    [sym_define_call_keyword] = ACTIONS(1977),
    [sym_define_function_keyword] = ACTIONS(1977),
    [sym_define_library_function_keyword] = ACTIONS(1977),
    [sym_define_start_keyword] = ACTIONS(1977),
    [sym_define_event_keyword] = ACTIONS(1977),
    [sym_define_program_keyword] = ACTIONS(1977),
    [sym_define_module_keyword] = ACTIONS(1977),
    [sym_char_keyword] = ACTIONS(1977),
    [sym_widechar_keyword] = ACTIONS(1977),
    [sym_integer_keyword] = ACTIONS(1977),
    [sym_sinteger_keyword] = ACTIONS(1977),
    [sym_long_keyword] = ACTIONS(1977),
    [sym_slong_keyword] = ACTIONS(1977),
    [sym_float_keyword] = ACTIONS(1977),
    [sym_double_keyword] = ACTIONS(1977),
    [sym_variant_keyword] = ACTIONS(1977),
    [sym_variantarray_keyword] = ACTIONS(1977),
    [sym_local_var_keyword] = ACTIONS(1977),
    [sym_stack_var_keyword] = ACTIONS(1977),
    [sym_constant_keyword] = ACTIONS(1977),
    [sym_volatile_keyword] = ACTIONS(1977),
    [sym_non_volatile_keyword] = ACTIONS(1977),
    [sym_persistent_keyword] = ACTIONS(1977),
    [sym_dev_keyword] = ACTIONS(1977),
    [sym_devlev_keyword] = ACTIONS(1977),
    [sym_devchan_keyword] = ACTIONS(1977),
    [sym_if_keyword] = ACTIONS(1977),
    [sym_switch_keyword] = ACTIONS(1977),
    [sym_case_keyword] = ACTIONS(1977),
    [sym_default_keyword] = ACTIONS(1977),
    [sym_while_keyword] = ACTIONS(1977),
    [sym_for_keyword] = ACTIONS(1977),
    [sym_break_keyword] = ACTIONS(1977),
    [sym_continue_keyword] = ACTIONS(1977),
    [sym_return_keyword] = ACTIONS(1977),
    [sym_select_keyword] = ACTIONS(1977),
    [sym_active_keyword] = ACTIONS(1977),
    [sym_struct_keyword] = ACTIONS(1977),
    [sym_structure_keyword] = ACTIONS(1977),
    [sym_button_event_keyword] = ACTIONS(1977),
    [sym_channel_event_keyword] = ACTIONS(1977),
    [sym_level_event_keyword] = ACTIONS(1977),
    [sym_data_event_keyword] = ACTIONS(1977),
    [sym_timeline_event_keyword] = ACTIONS(1977),
    [sym_custom_event_keyword] = ACTIONS(1977),
    [sym_push_keyword] = ACTIONS(1977),
    [sym_release_keyword] = ACTIONS(1977),
    [aux_sym_on_keyword_token1] = ACTIONS(1977),
    [aux_sym_off_keyword_token1] = ACTIONS(1977),
    [sym_send_level_keyword] = ACTIONS(1977),
    [sym_send_string_keyword] = ACTIONS(1977),
    [sym_send_command_keyword] = ACTIONS(1977),
    [sym_clear_buffer_keyword] = ACTIONS(1977),
    [sym_create_buffer_keyword] = ACTIONS(1977),
    [sym_create_multi_buffer_keyword] = ACTIONS(1977),
    [sym_call_keyword] = ACTIONS(1977),
    [sym_system_call_keyword] = ACTIONS(1977),
    [sym_devchan_to_keyword] = ACTIONS(1977),
    [sym_devchan_min_to_keyword] = ACTIONS(1977),
    [sym_devchan_total_off_keyword] = ACTIONS(1977),
    [sym_devchan_pulse_keyword] = ACTIONS(1977),
    [sym_wait_keyword] = ACTIONS(1977),
    [sym_cancel_wait_keyword] = ACTIONS(1977),
    [sym_wait_until_keyword] = ACTIONS(1977),
    [sym_cancel_wait_until_keyword] = ACTIONS(1977),
    [sym_cancel_all_wait_keyword] = ACTIONS(1977),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1977),
    [sym_not] = ACTIONS(1977),
    [sym_bnot] = ACTIONS(1977),
    [sym___file__] = ACTIONS(1977),
    [sym___date__] = ACTIONS(1977),
    [sym___ldate__] = ACTIONS(1977),
    [sym___time__] = ACTIONS(1977),
    [sym___line__] = ACTIONS(1977),
    [sym___name__] = ACTIONS(1977),
    [sym___netlinx__] = ACTIONS(1977),
    [sym_netlinx_axi_version] = ACTIONS(1977),
    [sym_url_flg_tcp] = ACTIONS(1977),
    [sym_url_flg_accinfopresent] = ACTIONS(1977),
    [sym_url_flg_temp] = ACTIONS(1977),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1977),
    [sym_url_flg_stat_mask] = ACTIONS(1977),
    [sym_url_flg_stat_lookup] = ACTIONS(1977),
    [sym_url_flg_stat_connecting] = ACTIONS(1977),
    [sym_url_flg_stat_waiting] = ACTIONS(1977),
    [sym_url_flg_stat_connected] = ACTIONS(1977),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1977),
    [sym_first_virtual_device] = ACTIONS(1977),
    [sym_first_local_port] = ACTIONS(1977),
    [sym_dynamic_virtual_device] = ACTIONS(1977),
    [sym_timeline_once] = ACTIONS(1977),
    [sym_timeline_repeat] = ACTIONS(1977),
    [sym_timeline_absolute] = ACTIONS(1977),
    [sym_timeline_relative] = ACTIONS(1977),
    [sym_do_push_timed_infinite] = ACTIONS(1977),
    [sym_source_type_no_address] = ACTIONS(1977),
    [sym_source_type_neuron_id] = ACTIONS(1977),
    [sym_source_type_ip_address] = ACTIONS(1977),
    [sym_source_type_axlink] = ACTIONS(1977),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1977),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1977),
    [sym_source_type_ip_socket_address] = ACTIONS(1977),
    [sym_source_type_rs232] = ACTIONS(1977),
    [sym_source_type_internal] = ACTIONS(1977),
    [sym_source_type_mac_address] = ACTIONS(1977),
    [sym_source_type_ipv4_port] = ACTIONS(1977),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1977),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1977),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1977),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1977),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1977),
    [sym_true] = ACTIONS(1977),
    [sym_false] = ACTIONS(1977),
    [sym_file_read_only] = ACTIONS(1977),
    [sym_file_rw_new] = ACTIONS(1977),
    [sym_file_rw_append] = ACTIONS(1977),
    [sym_ip_tcp] = ACTIONS(1977),
    [sym_ip_udp] = ACTIONS(1977),
    [sym_ip_udp_2way] = ACTIONS(1977),
    [sym_xml_encode_types] = ACTIONS(1977),
    [sym_xml_encode_char_as_list] = ACTIONS(1977),
    [sym_xml_encode_le] = ACTIONS(1977),
    [sym_xml_decode_types] = ACTIONS(1977),
    [sym_xml_decode_no_preserve] = ACTIONS(1977),
    [sym_duet_dev_type_display_device] = ACTIONS(1977),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1977),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1977),
    [sym_duet_dev_type_io_device] = ACTIONS(1977),
    [sym_duet_dev_type_relay_device] = ACTIONS(1977),
    [sym_duet_dev_type_ups] = ACTIONS(1977),
    [sym_duet_dev_type_amplifier] = ACTIONS(1977),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1977),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1977),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1977),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1977),
    [sym_duet_dev_type_camera] = ACTIONS(1977),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1977),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1977),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1977),
    [sym_duet_dev_type_dss] = ACTIONS(1977),
    [sym_duet_dev_type_dvr] = ACTIONS(1977),
    [sym_duet_dev_type_disc_device] = ACTIONS(1977),
    [sym_duet_dev_type_document_camera] = ACTIONS(1977),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1977),
    [sym_duet_dev_type_hvac] = ACTIONS(1977),
    [sym_duet_dev_type_keypad] = ACTIONS(1977),
    [sym_duet_dev_type_light] = ACTIONS(1977),
    [sym_duet_dev_type_monitor] = ACTIONS(1977),
    [sym_duet_dev_type_motor] = ACTIONS(1977),
    [sym_duet_dev_type_multi_window] = ACTIONS(1977),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1977),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1977),
    [sym_duet_dev_type_receiver] = ACTIONS(1977),
    [sym_duet_dev_type_security_system] = ACTIONS(1977),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1977),
    [sym_duet_dev_type_settop_box] = ACTIONS(1977),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1977),
    [sym_duet_dev_type_switcher] = ACTIONS(1977),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1977),
    [sym_duet_dev_type_tv] = ACTIONS(1977),
    [sym_duet_dev_type_utility] = ACTIONS(1977),
    [sym_duet_dev_type_vcr] = ACTIONS(1977),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1977),
    [sym_duet_dev_type_video_processor] = ACTIONS(1977),
    [sym_duet_dev_type_video_projector] = ACTIONS(1977),
    [sym_duet_dev_type_video_wall] = ACTIONS(1977),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1977),
    [sym_duet_dev_type_weather] = ACTIONS(1977),
    [sym_duet_dev_not_polled] = ACTIONS(1977),
    [sym_duet_dev_polled] = ACTIONS(1977),
    [sym_clkmgr_mode_network] = ACTIONS(1977),
    [sym_clkmgr_mode_standalone] = ACTIONS(1977),
    [sym_normal_standby] = ACTIONS(1977),
    [sym_normal_wake] = ACTIONS(1977),
    [sym_ip_multicast_ttl_option] = ACTIONS(1977),
    [sym_ip_ttl_subnet] = ACTIONS(1977),
    [sym_ip_ttl_site] = ACTIONS(1977),
    [sym_ip_ttl_region] = ACTIONS(1977),
    [sym_ip_ttl_continent] = ACTIONS(1977),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1977),
    [sym_ip_nodelay_on] = ACTIONS(1977),
    [sym_ip_nodelay_off] = ACTIONS(1977),
    [sym_amx_error] = ACTIONS(1977),
    [sym_amx_warning] = ACTIONS(1977),
    [sym_amx_info] = ACTIONS(1977),
    [sym_amx_debug] = ACTIONS(1977),
    [sym_netlinx_login_success] = ACTIONS(1977),
    [sym_netlinx_login_fail] = ACTIONS(1977),
    [sym_netlinx_logout] = ACTIONS(1977),
    [sym_null_str] = ACTIONS(1977),
    [sym_smtp_address] = ACTIONS(1977),
    [sym_smtp_port_number] = ACTIONS(1977),
    [sym_smtp_username] = ACTIONS(1977),
    [sym_smtp_password] = ACTIONS(1977),
    [sym_smtp_require_tls] = ACTIONS(1977),
    [sym_smtp_from] = ACTIONS(1977),
    [sym_smtp_tls_true] = ACTIONS(1977),
    [sym_smtp_tls_false] = ACTIONS(1977),
    [sym_listview_on_row_select_event] = ACTIONS(1977),
    [sym_data_structure_unknown] = ACTIONS(1977),
    [sym_data_structure_datafeed] = ACTIONS(1977),
    [sym_data_structure_datarecordset] = ACTIONS(1977),
    [sym_data_structure_datarecord] = ACTIONS(1977),
    [sym_data_structure_datafield] = ACTIONS(1977),
    [sym_data_max_content_field_count] = ACTIONS(1977),
    [sym_data_max_metadata_field_count] = ACTIONS(1977),
    [sym_data_max_id_length] = ACTIONS(1977),
    [sym_data_max_name_length] = ACTIONS(1977),
    [sym_data_max_description_length] = ACTIONS(1977),
    [sym_data_max_type_length] = ACTIONS(1977),
    [sym_data_max_format_length] = ACTIONS(1977),
    [sym_data_max_source_length] = ACTIONS(1977),
    [sym_data_max_value_length] = ACTIONS(1977),
    [sym_data_max_label_length] = ACTIONS(1977),
    [sym_data_max_url_length] = ACTIONS(1977),
    [sym_max_last_login_info_length] = ACTIONS(1977),
    [sym_data_type_unknown] = ACTIONS(1977),
    [sym_data_type_string] = ACTIONS(1977),
    [sym_data_type_datetime] = ACTIONS(1977),
    [sym_data_type_date] = ACTIONS(1977),
    [sym_data_type_time] = ACTIONS(1977),
    [sym_data_type_image] = ACTIONS(1977),
    [sym_data_format_url] = ACTIONS(1977),
    [sym_data_format_phone] = ACTIONS(1977),
    [sym_data_format_email] = ACTIONS(1977),
    [sym_data_format_iso8601] = ACTIONS(1977),
    [sym_auditlog_priv] = ACTIONS(1977),
    [sym_configuration_priv] = ACTIONS(1977),
    [sym_deviceconfig_priv] = ACTIONS(1977),
    [sym_ftp_priv] = ACTIONS(1977),
    [sym_http_priv] = ACTIONS(1977),
    [sym_networkconfig_priv] = ACTIONS(1977),
    [sym_programport_priv] = ACTIONS(1977),
    [sym_remoteui_priv] = ACTIONS(1977),
    [sym_securitycontrol_priv] = ACTIONS(1977),
    [sym_softwaremanagement_priv] = ACTIONS(1977),
    [sym_terminal_priv] = ACTIONS(1977),
    [sym_tpadmin_priv] = ACTIONS(1977),
    [sym_user1_priv] = ACTIONS(1977),
    [sym_user2_priv] = ACTIONS(1977),
    [sym_user3_priv] = ACTIONS(1977),
    [sym_user4_priv] = ACTIONS(1977),
    [sym_usermanagement_priv] = ACTIONS(1977),
    [sym_valid_account] = ACTIONS(1977),
    [sym_err_name_invalid] = ACTIONS(1977),
    [sym_err_password_invalid] = ACTIONS(1977),
    [sym_err_invalid_account] = ACTIONS(1977),
    [sym_err_no_authorization] = ACTIONS(1977),
    [sym_err_invalid_parameter] = ACTIONS(1977),
    [sym_err_user_locked_out] = ACTIONS(1977),
    [sym_err_user_account_expired] = ACTIONS(1977),
    [sym_err_general_error] = ACTIONS(1977),
    [sym_err_authentication_error] = ACTIONS(1977),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1977),
    [sym_tls_validate_certificate] = ACTIONS(1977),
    [sym_tdata] = ACTIONS(1977),
    [sym_tchannel] = ACTIONS(1977),
    [sym_tlevel] = ACTIONS(1977),
    [sym_tbutton] = ACTIONS(1977),
    [sym_ttimeline] = ACTIONS(1977),
    [sym_tcustom] = ACTIONS(1977),
    [sym_url_struct] = ACTIONS(1977),
    [sym_dns_struct] = ACTIONS(1977),
    [sym_ip_address_struct] = ACTIONS(1977),
    [sym_dev_info_struct] = ACTIONS(1977),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1977),
    [sym_data_feed] = ACTIONS(1977),
    [sym_data_field] = ACTIONS(1977),
    [sym_data_record] = ACTIONS(1977),
    [sym_wc_data_feed] = ACTIONS(1977),
    [sym_wc_data_field] = ACTIONS(1977),
    [sym_wc_data_record] = ACTIONS(1977),
    [sym_last_login_info] = ACTIONS(1977),
    [sym____reserved___] = ACTIONS(1977),
    [sym_date] = ACTIONS(1977),
    [sym_day] = ACTIONS(1977),
    [sym_time] = ACTIONS(1977),
    [sym_ldate] = ACTIONS(1977),
    [sym_push_device] = ACTIONS(1977),
    [sym_push_channel] = ACTIONS(1977),
    [sym_push_devchan] = ACTIONS(1977),
    [sym_release_device] = ACTIONS(1977),
    [sym_release_channel] = ACTIONS(1977),
    [sym_release_devchan] = ACTIONS(1977),
    [sym_master_slot] = ACTIONS(1977),
    [sym_get_pulse_time] = ACTIONS(1977),
    [sym_get_timer] = ACTIONS(1977),
    [sym_master_sn] = ACTIONS(1977),
    [sym_system_number] = ACTIONS(1977),
    [sym_button] = ACTIONS(1977),
    [sym_dv_channel] = ACTIONS(1977),
    [sym_data] = ACTIONS(1977),
    [sym_level] = ACTIONS(1977),
    [sym_timeline] = ACTIONS(1977),
    [sym_channel] = ACTIONS(1977),
    [sym_custom] = ACTIONS(1977),
    [sym_atoi] = ACTIONS(1977),
    [sym_atol] = ACTIONS(1977),
    [sym_atof] = ACTIONS(1977),
    [sym_itoa] = ACTIONS(1977),
    [sym_itohex] = ACTIONS(1977),
    [sym_hextoi] = ACTIONS(1977),
    [sym_ftoa] = ACTIONS(1977),
    [sym_find_string] = ACTIONS(1977),
    [sym_left_string] = ACTIONS(1977),
    [sym_right_string] = ACTIONS(1977),
    [sym_mid_string] = ACTIONS(1977),
    [sym_remove_string] = ACTIONS(1977),
    [sym_get_buffer_string] = ACTIONS(1977),
    [sym_get_buffer_char] = ACTIONS(1977),
    [sym_get_multi_buffer_string] = ACTIONS(1977),
    [sym_length_string] = ACTIONS(1977),
    [sym_max_length_string] = ACTIONS(1977),
    [sym_set_length_string] = ACTIONS(1977),
    [sym_compare_string] = ACTIONS(1977),
    [sym_upper_string] = ACTIONS(1977),
    [sym_lower_string] = ACTIONS(1977),
    [sym_get_last] = ACTIONS(1977),
    [sym_date_to_day] = ACTIONS(1977),
    [sym_date_to_month] = ACTIONS(1977),
    [sym_date_to_year] = ACTIONS(1977),
    [sym_time_to_hour] = ACTIONS(1977),
    [sym_time_to_minute] = ACTIONS(1977),
    [sym_time_to_second] = ACTIONS(1977),
    [sym_day_of_week] = ACTIONS(1977),
    [sym_set_pulse_time] = ACTIONS(1977),
    [sym_set_timer] = ACTIONS(1977),
    [sym_file_open] = ACTIONS(1977),
    [sym_file_close] = ACTIONS(1977),
    [sym_file_read] = ACTIONS(1977),
    [sym_file_read_line] = ACTIONS(1977),
    [sym_file_write] = ACTIONS(1977),
    [sym_file_write_line] = ACTIONS(1977),
    [sym_file_copy] = ACTIONS(1977),
    [sym_file_delete] = ACTIONS(1977),
    [sym_file_rename] = ACTIONS(1977),
    [sym_file_dir] = ACTIONS(1977),
    [sym_file_getdir] = ACTIONS(1977),
    [sym_file_setdir] = ACTIONS(1977),
    [sym_file_create_dir] = ACTIONS(1977),
    [sym_file_remove_dir] = ACTIONS(1977),
    [sym_file_seek] = ACTIONS(1977),
    [sym_length_array] = ACTIONS(1977),
    [sym_max_length_array] = ACTIONS(1977),
    [sym_set_length_array] = ACTIONS(1977),
    [sym_ip_client_open] = ACTIONS(1977),
    [sym_ip_client_close] = ACTIONS(1977),
    [sym_ip_server_open] = ACTIONS(1977),
    [sym_ip_server_close] = ACTIONS(1977),
    [sym_ip_bound_client_open] = ACTIONS(1977),
    [sym_ip_mc_server_open] = ACTIONS(1977),
    [sym_ip_set_option] = ACTIONS(1977),
    [sym_ssh_client_open] = ACTIONS(1977),
    [sym_ssh_client_close] = ACTIONS(1977),
    [sym_tls_client_open] = ACTIONS(1977),
    [sym_tls_client_close] = ACTIONS(1977),
    [sym_do_push] = ACTIONS(1977),
    [sym_do_push_timed] = ACTIONS(1977),
    [sym_do_release] = ACTIONS(1977),
    [sym_device_id] = ACTIONS(1977),
    [sym_device_info] = ACTIONS(1977),
    [sym_device_id_string] = ACTIONS(1977),
    [sym_device_standby] = ACTIONS(1977),
    [sym_device_wake] = ACTIONS(1977),
    [sym_timeline_create] = ACTIONS(1977),
    [sym_timeline_kill] = ACTIONS(1977),
    [sym_timeline_reload] = ACTIONS(1977),
    [sym_timeline_pause] = ACTIONS(1977),
    [sym_timeline_restart] = ACTIONS(1977),
    [sym_timeline_set] = ACTIONS(1977),
    [sym_timeline_get] = ACTIONS(1977),
    [sym_timeline_active] = ACTIONS(1977),
    [sym_timeline_dynamic_id] = ACTIONS(1977),
    [sym_variable_to_string] = ACTIONS(1977),
    [sym_string_to_variable] = ACTIONS(1977),
    [sym_length_variable_to_string] = ACTIONS(1977),
    [sym_type_cast] = ACTIONS(1977),
    [sym_variable_to_xml] = ACTIONS(1977),
    [sym_xml_to_variable] = ACTIONS(1977),
    [sym_length_variable_to_xml] = ACTIONS(1977),
    [sym_redirect_string] = ACTIONS(1977),
    [sym_random_number] = ACTIONS(1977),
    [sym_get_unique_id] = ACTIONS(1977),
    [sym_get_system_number] = ACTIONS(1977),
    [sym_set_system_number] = ACTIONS(1977),
    [sym_get_serial_number] = ACTIONS(1977),
    [sym_get_master_build] = ACTIONS(1977),
    [sym_set_validation_code] = ACTIONS(1977),
    [sym_reboot] = ACTIONS(1977),
    [sym_load_duet_module] = ACTIONS(1977),
    [sym_unload_duet_module] = ACTIONS(1977),
    [sym_duet_mem_size_set] = ACTIONS(1977),
    [sym_duet_mem_size_get] = ACTIONS(1977),
    [sym_dynamic_application_device] = ACTIONS(1977),
    [sym_dynamic_polled_port] = ACTIONS(1977),
    [sym_static_port_binding] = ACTIONS(1977),
    [sym_static_ip_binding] = ACTIONS(1977),
    [sym_get_dns_list] = ACTIONS(1977),
    [sym_set_dns_list] = ACTIONS(1977),
    [sym_get_ip_address] = ACTIONS(1977),
    [sym_set_ip_address] = ACTIONS(1977),
    [sym_get_url_list] = ACTIONS(1977),
    [sym_add_url_entry] = ACTIONS(1977),
    [sym_delete_url_entry] = ACTIONS(1977),
    [sym_abs_value] = ACTIONS(1977),
    [sym_max_value] = ACTIONS(1977),
    [sym_min_value] = ACTIONS(1977),
    [sym_exp_value] = ACTIONS(1977),
    [sym_log_value] = ACTIONS(1977),
    [sym_log10_value] = ACTIONS(1977),
    [sym_power_value] = ACTIONS(1977),
    [sym_sqrt_value] = ACTIONS(1977),
    [sym_raw_be] = ACTIONS(1977),
    [sym_raw_le] = ACTIONS(1977),
    [sym_internal_queue_size_set] = ACTIONS(1977),
    [sym_internal_queue_size_get] = ACTIONS(1977),
    [sym_internal_threshold_set] = ACTIONS(1977),
    [sym_internal_threshold_get] = ACTIONS(1977),
    [sym_rebuilt_event] = ACTIONS(1977),
    [sym_do_custom_event] = ACTIONS(1977),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1977),
    [sym_clkmgr_set_clk_source] = ACTIONS(1977),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1977),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1977),
    [sym_clkmgr_get_timezone] = ACTIONS(1977),
    [sym_clkmgr_set_timezone] = ACTIONS(1977),
    [sym_clkmgr_get_resync_period] = ACTIONS(1977),
    [sym_clkmgr_set_resync_period] = ACTIONS(1977),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1977),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1977),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1977),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1977),
    [sym_clkmgr_get_timeservers] = ACTIONS(1977),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1977),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1977),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1977),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1977),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1977),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1977),
    [sym_set_log_level] = ACTIONS(1977),
    [sym_get_log_level] = ACTIONS(1977),
    [sym_amx_log] = ACTIONS(1977),
    [sym_smtp_server_config_set] = ACTIONS(1977),
    [sym_smtp_server_config_get] = ACTIONS(1977),
    [sym_smtp_send] = ACTIONS(1977),
    [sym_validate_netlinx_account] = ACTIONS(1977),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1977),
    [sym_authenticate_certificate] = ACTIONS(1977),
    [sym_audit_netlinx_session_event] = ACTIONS(1977),
    [sym_audit_netlinx_generic_event] = ACTIONS(1977),
    [sym_get_available_flash_disk_space] = ACTIONS(1977),
    [sym_get_max_flash_disk_space] = ACTIONS(1977),
    [sym_data_create_feed] = ACTIONS(1977),
    [sym_data_delete_feed] = ACTIONS(1977),
    [sym_data_publish_feed] = ACTIONS(1977),
    [sym_data_get_published_feed] = ACTIONS(1977),
    [sym_data_add_record] = ACTIONS(1977),
    [sym_data_get_event_record] = ACTIONS(1977),
    [sym__wc_data_create_feed] = ACTIONS(1977),
    [sym__wc_data_add_record] = ACTIONS(1977),
    [sym__wc_data_get_event_record] = ACTIONS(1977),
    [sym_format] = ACTIONS(1977),
    [sym_astro_clock] = ACTIONS(1977),
    [sym_set_outdoor_temperature] = ACTIONS(1977),
    [sym_set_virtual_level_count] = ACTIONS(1977),
    [sym_set_virtual_channel_count] = ACTIONS(1977),
    [sym_set_virtual_port_count] = ACTIONS(1977),
    [sym__wc_encode_int] = ACTIONS(1977),
    [sym__wc_decode_int] = ACTIONS(1977),
    [sym__wc_int] = ACTIONS(1977),
    [sym__wc_to_ch_int] = ACTIONS(1977),
    [sym__ch_to_wc_int] = ACTIONS(1977),
    [sym__wc_find_string_int] = ACTIONS(1977),
    [sym__wc_left_string_int] = ACTIONS(1977),
    [sym__wc_lower_string_int] = ACTIONS(1977),
    [sym__wc_mid_string_int] = ACTIONS(1977),
    [sym__wc_remove_string_int] = ACTIONS(1977),
    [sym__wc_right_string_int] = ACTIONS(1977),
    [sym__wc_upper_string_int] = ACTIONS(1977),
    [sym__wc_compare_string_int] = ACTIONS(1977),
    [sym__wc_get_buffer_char_int] = ACTIONS(1977),
    [sym__wc_get_buffer_string_int] = ACTIONS(1977),
    [sym__wc_concat_string_int] = ACTIONS(1977),
    [sym__wc_file_open_int] = ACTIONS(1977),
    [sym__wc_file_close_int] = ACTIONS(1977),
    [sym__wc_file_read_int] = ACTIONS(1977),
    [sym__wc_file_read_line_int] = ACTIONS(1977),
    [sym__wc_file_write_int] = ACTIONS(1977),
    [sym__wc_file_write_line_int] = ACTIONS(1977),
    [aux_sym_preproc_include_token1] = ACTIONS(1977),
    [aux_sym_preproc_define_token1] = ACTIONS(1979),
    [aux_sym_preproc_warn_token1] = ACTIONS(1979),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1979),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1979),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1979),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1979),
    [aux_sym_preproc_else_token1] = ACTIONS(1979),
    [anon_sym_LBRACK] = ACTIONS(1979),
    [anon_sym_LBRACE] = ACTIONS(1979),
    [anon_sym_LPAREN] = ACTIONS(1977),
    [anon_sym_BANG] = ACTIONS(1979),
    [anon_sym_TILDE] = ACTIONS(1979),
    [anon_sym_DASH] = ACTIONS(1977),
    [anon_sym_PLUS] = ACTIONS(1977),
    [anon_sym_DASH_DASH] = ACTIONS(1979),
    [anon_sym_PLUS_PLUS] = ACTIONS(1979),
    [anon_sym_DQUOTE] = ACTIONS(1979),
    [anon_sym_SQUOTE] = ACTIONS(1979),
    [sym_number_literal] = ACTIONS(1979),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(307)] = {
    [sym_identifier] = ACTIONS(1981),
    [sym_program_name_keyword] = ACTIONS(1981),
    [sym_module_name_keyword] = ACTIONS(1981),
    [sym_define_device_keyword] = ACTIONS(1981),
    [sym_define_combine_keyword] = ACTIONS(1981),
    [sym_define_constant_keyword] = ACTIONS(1981),
    [sym_define_type_keyword] = ACTIONS(1981),
    [sym_define_variable_keyword] = ACTIONS(1981),
    [sym_define_system_variable_keyword] = ACTIONS(1981),
    [sym_define_connect_level_keyword] = ACTIONS(1981),
    [sym_define_latching_keyword] = ACTIONS(1981),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1981),
    [sym_define_toggling_keyword] = ACTIONS(1981),
    [sym_define_call_keyword] = ACTIONS(1981),
    [sym_define_function_keyword] = ACTIONS(1981),
    [sym_define_library_function_keyword] = ACTIONS(1981),
    [sym_define_start_keyword] = ACTIONS(1981),
    [sym_define_event_keyword] = ACTIONS(1981),
    [sym_define_program_keyword] = ACTIONS(1981),
    [sym_define_module_keyword] = ACTIONS(1981),
    [sym_char_keyword] = ACTIONS(1981),
    [sym_widechar_keyword] = ACTIONS(1981),
    [sym_integer_keyword] = ACTIONS(1981),
    [sym_sinteger_keyword] = ACTIONS(1981),
    [sym_long_keyword] = ACTIONS(1981),
    [sym_slong_keyword] = ACTIONS(1981),
    [sym_float_keyword] = ACTIONS(1981),
    [sym_double_keyword] = ACTIONS(1981),
    [sym_variant_keyword] = ACTIONS(1981),
    [sym_variantarray_keyword] = ACTIONS(1981),
    [sym_local_var_keyword] = ACTIONS(1981),
    [sym_stack_var_keyword] = ACTIONS(1981),
    [sym_constant_keyword] = ACTIONS(1981),
    [sym_volatile_keyword] = ACTIONS(1981),
    [sym_non_volatile_keyword] = ACTIONS(1981),
    [sym_persistent_keyword] = ACTIONS(1981),
    [sym_dev_keyword] = ACTIONS(1981),
    [sym_devlev_keyword] = ACTIONS(1981),
    [sym_devchan_keyword] = ACTIONS(1981),
    [sym_if_keyword] = ACTIONS(1981),
    [sym_switch_keyword] = ACTIONS(1981),
    [sym_case_keyword] = ACTIONS(1981),
    [sym_default_keyword] = ACTIONS(1981),
    [sym_while_keyword] = ACTIONS(1981),
    [sym_for_keyword] = ACTIONS(1981),
    [sym_break_keyword] = ACTIONS(1981),
    [sym_continue_keyword] = ACTIONS(1981),
    [sym_return_keyword] = ACTIONS(1981),
    [sym_select_keyword] = ACTIONS(1981),
    [sym_active_keyword] = ACTIONS(1981),
    [sym_struct_keyword] = ACTIONS(1981),
    [sym_structure_keyword] = ACTIONS(1981),
    [sym_button_event_keyword] = ACTIONS(1981),
    [sym_channel_event_keyword] = ACTIONS(1981),
    [sym_level_event_keyword] = ACTIONS(1981),
    [sym_data_event_keyword] = ACTIONS(1981),
    [sym_timeline_event_keyword] = ACTIONS(1981),
    [sym_custom_event_keyword] = ACTIONS(1981),
    [sym_push_keyword] = ACTIONS(1981),
    [sym_release_keyword] = ACTIONS(1981),
    [aux_sym_on_keyword_token1] = ACTIONS(1981),
    [aux_sym_off_keyword_token1] = ACTIONS(1981),
    [sym_send_level_keyword] = ACTIONS(1981),
    [sym_send_string_keyword] = ACTIONS(1981),
    [sym_send_command_keyword] = ACTIONS(1981),
    [sym_clear_buffer_keyword] = ACTIONS(1981),
    [sym_create_buffer_keyword] = ACTIONS(1981),
    [sym_create_multi_buffer_keyword] = ACTIONS(1981),
    [sym_call_keyword] = ACTIONS(1981),
    [sym_system_call_keyword] = ACTIONS(1981),
    [sym_devchan_to_keyword] = ACTIONS(1981),
    [sym_devchan_min_to_keyword] = ACTIONS(1981),
    [sym_devchan_total_off_keyword] = ACTIONS(1981),
    [sym_devchan_pulse_keyword] = ACTIONS(1981),
    [sym_wait_keyword] = ACTIONS(1981),
    [sym_cancel_wait_keyword] = ACTIONS(1981),
    [sym_wait_until_keyword] = ACTIONS(1981),
    [sym_cancel_wait_until_keyword] = ACTIONS(1981),
    [sym_cancel_all_wait_keyword] = ACTIONS(1981),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1981),
    [sym_not] = ACTIONS(1981),
    [sym_bnot] = ACTIONS(1981),
    [sym___file__] = ACTIONS(1981),
    [sym___date__] = ACTIONS(1981),
    [sym___ldate__] = ACTIONS(1981),
    [sym___time__] = ACTIONS(1981),
    [sym___line__] = ACTIONS(1981),
    [sym___name__] = ACTIONS(1981),
    [sym___netlinx__] = ACTIONS(1981),
    [sym_netlinx_axi_version] = ACTIONS(1981),
    [sym_url_flg_tcp] = ACTIONS(1981),
    [sym_url_flg_accinfopresent] = ACTIONS(1981),
    [sym_url_flg_temp] = ACTIONS(1981),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1981),
    [sym_url_flg_stat_mask] = ACTIONS(1981),
    [sym_url_flg_stat_lookup] = ACTIONS(1981),
    [sym_url_flg_stat_connecting] = ACTIONS(1981),
    [sym_url_flg_stat_waiting] = ACTIONS(1981),
    [sym_url_flg_stat_connected] = ACTIONS(1981),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1981),
    [sym_first_virtual_device] = ACTIONS(1981),
    [sym_first_local_port] = ACTIONS(1981),
    [sym_dynamic_virtual_device] = ACTIONS(1981),
    [sym_timeline_once] = ACTIONS(1981),
    [sym_timeline_repeat] = ACTIONS(1981),
    [sym_timeline_absolute] = ACTIONS(1981),
    [sym_timeline_relative] = ACTIONS(1981),
    [sym_do_push_timed_infinite] = ACTIONS(1981),
    [sym_source_type_no_address] = ACTIONS(1981),
    [sym_source_type_neuron_id] = ACTIONS(1981),
    [sym_source_type_ip_address] = ACTIONS(1981),
    [sym_source_type_axlink] = ACTIONS(1981),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1981),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1981),
    [sym_source_type_ip_socket_address] = ACTIONS(1981),
    [sym_source_type_rs232] = ACTIONS(1981),
    [sym_source_type_internal] = ACTIONS(1981),
    [sym_source_type_mac_address] = ACTIONS(1981),
    [sym_source_type_ipv4_port] = ACTIONS(1981),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1981),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1981),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1981),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1981),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1981),
    [sym_true] = ACTIONS(1981),
    [sym_false] = ACTIONS(1981),
    [sym_file_read_only] = ACTIONS(1981),
    [sym_file_rw_new] = ACTIONS(1981),
    [sym_file_rw_append] = ACTIONS(1981),
    [sym_ip_tcp] = ACTIONS(1981),
    [sym_ip_udp] = ACTIONS(1981),
    [sym_ip_udp_2way] = ACTIONS(1981),
    [sym_xml_encode_types] = ACTIONS(1981),
    [sym_xml_encode_char_as_list] = ACTIONS(1981),
    [sym_xml_encode_le] = ACTIONS(1981),
    [sym_xml_decode_types] = ACTIONS(1981),
    [sym_xml_decode_no_preserve] = ACTIONS(1981),
    [sym_duet_dev_type_display_device] = ACTIONS(1981),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1981),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1981),
    [sym_duet_dev_type_io_device] = ACTIONS(1981),
    [sym_duet_dev_type_relay_device] = ACTIONS(1981),
    [sym_duet_dev_type_ups] = ACTIONS(1981),
    [sym_duet_dev_type_amplifier] = ACTIONS(1981),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1981),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1981),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1981),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1981),
    [sym_duet_dev_type_camera] = ACTIONS(1981),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1981),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1981),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1981),
    [sym_duet_dev_type_dss] = ACTIONS(1981),
    [sym_duet_dev_type_dvr] = ACTIONS(1981),
    [sym_duet_dev_type_disc_device] = ACTIONS(1981),
    [sym_duet_dev_type_document_camera] = ACTIONS(1981),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1981),
    [sym_duet_dev_type_hvac] = ACTIONS(1981),
    [sym_duet_dev_type_keypad] = ACTIONS(1981),
    [sym_duet_dev_type_light] = ACTIONS(1981),
    [sym_duet_dev_type_monitor] = ACTIONS(1981),
    [sym_duet_dev_type_motor] = ACTIONS(1981),
    [sym_duet_dev_type_multi_window] = ACTIONS(1981),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1981),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1981),
    [sym_duet_dev_type_receiver] = ACTIONS(1981),
    [sym_duet_dev_type_security_system] = ACTIONS(1981),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1981),
    [sym_duet_dev_type_settop_box] = ACTIONS(1981),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1981),
    [sym_duet_dev_type_switcher] = ACTIONS(1981),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1981),
    [sym_duet_dev_type_tv] = ACTIONS(1981),
    [sym_duet_dev_type_utility] = ACTIONS(1981),
    [sym_duet_dev_type_vcr] = ACTIONS(1981),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1981),
    [sym_duet_dev_type_video_processor] = ACTIONS(1981),
    [sym_duet_dev_type_video_projector] = ACTIONS(1981),
    [sym_duet_dev_type_video_wall] = ACTIONS(1981),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1981),
    [sym_duet_dev_type_weather] = ACTIONS(1981),
    [sym_duet_dev_not_polled] = ACTIONS(1981),
    [sym_duet_dev_polled] = ACTIONS(1981),
    [sym_clkmgr_mode_network] = ACTIONS(1981),
    [sym_clkmgr_mode_standalone] = ACTIONS(1981),
    [sym_normal_standby] = ACTIONS(1981),
    [sym_normal_wake] = ACTIONS(1981),
    [sym_ip_multicast_ttl_option] = ACTIONS(1981),
    [sym_ip_ttl_subnet] = ACTIONS(1981),
    [sym_ip_ttl_site] = ACTIONS(1981),
    [sym_ip_ttl_region] = ACTIONS(1981),
    [sym_ip_ttl_continent] = ACTIONS(1981),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1981),
    [sym_ip_nodelay_on] = ACTIONS(1981),
    [sym_ip_nodelay_off] = ACTIONS(1981),
    [sym_amx_error] = ACTIONS(1981),
    [sym_amx_warning] = ACTIONS(1981),
    [sym_amx_info] = ACTIONS(1981),
    [sym_amx_debug] = ACTIONS(1981),
    [sym_netlinx_login_success] = ACTIONS(1981),
    [sym_netlinx_login_fail] = ACTIONS(1981),
    [sym_netlinx_logout] = ACTIONS(1981),
    [sym_null_str] = ACTIONS(1981),
    [sym_smtp_address] = ACTIONS(1981),
    [sym_smtp_port_number] = ACTIONS(1981),
    [sym_smtp_username] = ACTIONS(1981),
    [sym_smtp_password] = ACTIONS(1981),
    [sym_smtp_require_tls] = ACTIONS(1981),
    [sym_smtp_from] = ACTIONS(1981),
    [sym_smtp_tls_true] = ACTIONS(1981),
    [sym_smtp_tls_false] = ACTIONS(1981),
    [sym_listview_on_row_select_event] = ACTIONS(1981),
    [sym_data_structure_unknown] = ACTIONS(1981),
    [sym_data_structure_datafeed] = ACTIONS(1981),
    [sym_data_structure_datarecordset] = ACTIONS(1981),
    [sym_data_structure_datarecord] = ACTIONS(1981),
    [sym_data_structure_datafield] = ACTIONS(1981),
    [sym_data_max_content_field_count] = ACTIONS(1981),
    [sym_data_max_metadata_field_count] = ACTIONS(1981),
    [sym_data_max_id_length] = ACTIONS(1981),
    [sym_data_max_name_length] = ACTIONS(1981),
    [sym_data_max_description_length] = ACTIONS(1981),
    [sym_data_max_type_length] = ACTIONS(1981),
    [sym_data_max_format_length] = ACTIONS(1981),
    [sym_data_max_source_length] = ACTIONS(1981),
    [sym_data_max_value_length] = ACTIONS(1981),
    [sym_data_max_label_length] = ACTIONS(1981),
    [sym_data_max_url_length] = ACTIONS(1981),
    [sym_max_last_login_info_length] = ACTIONS(1981),
    [sym_data_type_unknown] = ACTIONS(1981),
    [sym_data_type_string] = ACTIONS(1981),
    [sym_data_type_datetime] = ACTIONS(1981),
    [sym_data_type_date] = ACTIONS(1981),
    [sym_data_type_time] = ACTIONS(1981),
    [sym_data_type_image] = ACTIONS(1981),
    [sym_data_format_url] = ACTIONS(1981),
    [sym_data_format_phone] = ACTIONS(1981),
    [sym_data_format_email] = ACTIONS(1981),
    [sym_data_format_iso8601] = ACTIONS(1981),
    [sym_auditlog_priv] = ACTIONS(1981),
    [sym_configuration_priv] = ACTIONS(1981),
    [sym_deviceconfig_priv] = ACTIONS(1981),
    [sym_ftp_priv] = ACTIONS(1981),
    [sym_http_priv] = ACTIONS(1981),
    [sym_networkconfig_priv] = ACTIONS(1981),
    [sym_programport_priv] = ACTIONS(1981),
    [sym_remoteui_priv] = ACTIONS(1981),
    [sym_securitycontrol_priv] = ACTIONS(1981),
    [sym_softwaremanagement_priv] = ACTIONS(1981),
    [sym_terminal_priv] = ACTIONS(1981),
    [sym_tpadmin_priv] = ACTIONS(1981),
    [sym_user1_priv] = ACTIONS(1981),
    [sym_user2_priv] = ACTIONS(1981),
    [sym_user3_priv] = ACTIONS(1981),
    [sym_user4_priv] = ACTIONS(1981),
    [sym_usermanagement_priv] = ACTIONS(1981),
    [sym_valid_account] = ACTIONS(1981),
    [sym_err_name_invalid] = ACTIONS(1981),
    [sym_err_password_invalid] = ACTIONS(1981),
    [sym_err_invalid_account] = ACTIONS(1981),
    [sym_err_no_authorization] = ACTIONS(1981),
    [sym_err_invalid_parameter] = ACTIONS(1981),
    [sym_err_user_locked_out] = ACTIONS(1981),
    [sym_err_user_account_expired] = ACTIONS(1981),
    [sym_err_general_error] = ACTIONS(1981),
    [sym_err_authentication_error] = ACTIONS(1981),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1981),
    [sym_tls_validate_certificate] = ACTIONS(1981),
    [sym_tdata] = ACTIONS(1981),
    [sym_tchannel] = ACTIONS(1981),
    [sym_tlevel] = ACTIONS(1981),
    [sym_tbutton] = ACTIONS(1981),
    [sym_ttimeline] = ACTIONS(1981),
    [sym_tcustom] = ACTIONS(1981),
    [sym_url_struct] = ACTIONS(1981),
    [sym_dns_struct] = ACTIONS(1981),
    [sym_ip_address_struct] = ACTIONS(1981),
    [sym_dev_info_struct] = ACTIONS(1981),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1981),
    [sym_data_feed] = ACTIONS(1981),
    [sym_data_field] = ACTIONS(1981),
    [sym_data_record] = ACTIONS(1981),
    [sym_wc_data_feed] = ACTIONS(1981),
    [sym_wc_data_field] = ACTIONS(1981),
    [sym_wc_data_record] = ACTIONS(1981),
    [sym_last_login_info] = ACTIONS(1981),
    [sym____reserved___] = ACTIONS(1981),
    [sym_date] = ACTIONS(1981),
    [sym_day] = ACTIONS(1981),
    [sym_time] = ACTIONS(1981),
    [sym_ldate] = ACTIONS(1981),
    [sym_push_device] = ACTIONS(1981),
    [sym_push_channel] = ACTIONS(1981),
    [sym_push_devchan] = ACTIONS(1981),
    [sym_release_device] = ACTIONS(1981),
    [sym_release_channel] = ACTIONS(1981),
    [sym_release_devchan] = ACTIONS(1981),
    [sym_master_slot] = ACTIONS(1981),
    [sym_get_pulse_time] = ACTIONS(1981),
    [sym_get_timer] = ACTIONS(1981),
    [sym_master_sn] = ACTIONS(1981),
    [sym_system_number] = ACTIONS(1981),
    [sym_button] = ACTIONS(1981),
    [sym_dv_channel] = ACTIONS(1981),
    [sym_data] = ACTIONS(1981),
    [sym_level] = ACTIONS(1981),
    [sym_timeline] = ACTIONS(1981),
    [sym_channel] = ACTIONS(1981),
    [sym_custom] = ACTIONS(1981),
    [sym_atoi] = ACTIONS(1981),
    [sym_atol] = ACTIONS(1981),
    [sym_atof] = ACTIONS(1981),
    [sym_itoa] = ACTIONS(1981),
    [sym_itohex] = ACTIONS(1981),
    [sym_hextoi] = ACTIONS(1981),
    [sym_ftoa] = ACTIONS(1981),
    [sym_find_string] = ACTIONS(1981),
    [sym_left_string] = ACTIONS(1981),
    [sym_right_string] = ACTIONS(1981),
    [sym_mid_string] = ACTIONS(1981),
    [sym_remove_string] = ACTIONS(1981),
    [sym_get_buffer_string] = ACTIONS(1981),
    [sym_get_buffer_char] = ACTIONS(1981),
    [sym_get_multi_buffer_string] = ACTIONS(1981),
    [sym_length_string] = ACTIONS(1981),
    [sym_max_length_string] = ACTIONS(1981),
    [sym_set_length_string] = ACTIONS(1981),
    [sym_compare_string] = ACTIONS(1981),
    [sym_upper_string] = ACTIONS(1981),
    [sym_lower_string] = ACTIONS(1981),
    [sym_get_last] = ACTIONS(1981),
    [sym_date_to_day] = ACTIONS(1981),
    [sym_date_to_month] = ACTIONS(1981),
    [sym_date_to_year] = ACTIONS(1981),
    [sym_time_to_hour] = ACTIONS(1981),
    [sym_time_to_minute] = ACTIONS(1981),
    [sym_time_to_second] = ACTIONS(1981),
    [sym_day_of_week] = ACTIONS(1981),
    [sym_set_pulse_time] = ACTIONS(1981),
    [sym_set_timer] = ACTIONS(1981),
    [sym_file_open] = ACTIONS(1981),
    [sym_file_close] = ACTIONS(1981),
    [sym_file_read] = ACTIONS(1981),
    [sym_file_read_line] = ACTIONS(1981),
    [sym_file_write] = ACTIONS(1981),
    [sym_file_write_line] = ACTIONS(1981),
    [sym_file_copy] = ACTIONS(1981),
    [sym_file_delete] = ACTIONS(1981),
    [sym_file_rename] = ACTIONS(1981),
    [sym_file_dir] = ACTIONS(1981),
    [sym_file_getdir] = ACTIONS(1981),
    [sym_file_setdir] = ACTIONS(1981),
    [sym_file_create_dir] = ACTIONS(1981),
    [sym_file_remove_dir] = ACTIONS(1981),
    [sym_file_seek] = ACTIONS(1981),
    [sym_length_array] = ACTIONS(1981),
    [sym_max_length_array] = ACTIONS(1981),
    [sym_set_length_array] = ACTIONS(1981),
    [sym_ip_client_open] = ACTIONS(1981),
    [sym_ip_client_close] = ACTIONS(1981),
    [sym_ip_server_open] = ACTIONS(1981),
    [sym_ip_server_close] = ACTIONS(1981),
    [sym_ip_bound_client_open] = ACTIONS(1981),
    [sym_ip_mc_server_open] = ACTIONS(1981),
    [sym_ip_set_option] = ACTIONS(1981),
    [sym_ssh_client_open] = ACTIONS(1981),
    [sym_ssh_client_close] = ACTIONS(1981),
    [sym_tls_client_open] = ACTIONS(1981),
    [sym_tls_client_close] = ACTIONS(1981),
    [sym_do_push] = ACTIONS(1981),
    [sym_do_push_timed] = ACTIONS(1981),
    [sym_do_release] = ACTIONS(1981),
    [sym_device_id] = ACTIONS(1981),
    [sym_device_info] = ACTIONS(1981),
    [sym_device_id_string] = ACTIONS(1981),
    [sym_device_standby] = ACTIONS(1981),
    [sym_device_wake] = ACTIONS(1981),
    [sym_timeline_create] = ACTIONS(1981),
    [sym_timeline_kill] = ACTIONS(1981),
    [sym_timeline_reload] = ACTIONS(1981),
    [sym_timeline_pause] = ACTIONS(1981),
    [sym_timeline_restart] = ACTIONS(1981),
    [sym_timeline_set] = ACTIONS(1981),
    [sym_timeline_get] = ACTIONS(1981),
    [sym_timeline_active] = ACTIONS(1981),
    [sym_timeline_dynamic_id] = ACTIONS(1981),
    [sym_variable_to_string] = ACTIONS(1981),
    [sym_string_to_variable] = ACTIONS(1981),
    [sym_length_variable_to_string] = ACTIONS(1981),
    [sym_type_cast] = ACTIONS(1981),
    [sym_variable_to_xml] = ACTIONS(1981),
    [sym_xml_to_variable] = ACTIONS(1981),
    [sym_length_variable_to_xml] = ACTIONS(1981),
    [sym_redirect_string] = ACTIONS(1981),
    [sym_random_number] = ACTIONS(1981),
    [sym_get_unique_id] = ACTIONS(1981),
    [sym_get_system_number] = ACTIONS(1981),
    [sym_set_system_number] = ACTIONS(1981),
    [sym_get_serial_number] = ACTIONS(1981),
    [sym_get_master_build] = ACTIONS(1981),
    [sym_set_validation_code] = ACTIONS(1981),
    [sym_reboot] = ACTIONS(1981),
    [sym_load_duet_module] = ACTIONS(1981),
    [sym_unload_duet_module] = ACTIONS(1981),
    [sym_duet_mem_size_set] = ACTIONS(1981),
    [sym_duet_mem_size_get] = ACTIONS(1981),
    [sym_dynamic_application_device] = ACTIONS(1981),
    [sym_dynamic_polled_port] = ACTIONS(1981),
    [sym_static_port_binding] = ACTIONS(1981),
    [sym_static_ip_binding] = ACTIONS(1981),
    [sym_get_dns_list] = ACTIONS(1981),
    [sym_set_dns_list] = ACTIONS(1981),
    [sym_get_ip_address] = ACTIONS(1981),
    [sym_set_ip_address] = ACTIONS(1981),
    [sym_get_url_list] = ACTIONS(1981),
    [sym_add_url_entry] = ACTIONS(1981),
    [sym_delete_url_entry] = ACTIONS(1981),
    [sym_abs_value] = ACTIONS(1981),
    [sym_max_value] = ACTIONS(1981),
    [sym_min_value] = ACTIONS(1981),
    [sym_exp_value] = ACTIONS(1981),
    [sym_log_value] = ACTIONS(1981),
    [sym_log10_value] = ACTIONS(1981),
    [sym_power_value] = ACTIONS(1981),
    [sym_sqrt_value] = ACTIONS(1981),
    [sym_raw_be] = ACTIONS(1981),
    [sym_raw_le] = ACTIONS(1981),
    [sym_internal_queue_size_set] = ACTIONS(1981),
    [sym_internal_queue_size_get] = ACTIONS(1981),
    [sym_internal_threshold_set] = ACTIONS(1981),
    [sym_internal_threshold_get] = ACTIONS(1981),
    [sym_rebuilt_event] = ACTIONS(1981),
    [sym_do_custom_event] = ACTIONS(1981),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1981),
    [sym_clkmgr_set_clk_source] = ACTIONS(1981),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1981),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1981),
    [sym_clkmgr_get_timezone] = ACTIONS(1981),
    [sym_clkmgr_set_timezone] = ACTIONS(1981),
    [sym_clkmgr_get_resync_period] = ACTIONS(1981),
    [sym_clkmgr_set_resync_period] = ACTIONS(1981),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1981),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1981),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1981),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1981),
    [sym_clkmgr_get_timeservers] = ACTIONS(1981),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1981),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1981),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1981),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1981),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1981),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1981),
    [sym_set_log_level] = ACTIONS(1981),
    [sym_get_log_level] = ACTIONS(1981),
    [sym_amx_log] = ACTIONS(1981),
    [sym_smtp_server_config_set] = ACTIONS(1981),
    [sym_smtp_server_config_get] = ACTIONS(1981),
    [sym_smtp_send] = ACTIONS(1981),
    [sym_validate_netlinx_account] = ACTIONS(1981),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1981),
    [sym_authenticate_certificate] = ACTIONS(1981),
    [sym_audit_netlinx_session_event] = ACTIONS(1981),
    [sym_audit_netlinx_generic_event] = ACTIONS(1981),
    [sym_get_available_flash_disk_space] = ACTIONS(1981),
    [sym_get_max_flash_disk_space] = ACTIONS(1981),
    [sym_data_create_feed] = ACTIONS(1981),
    [sym_data_delete_feed] = ACTIONS(1981),
    [sym_data_publish_feed] = ACTIONS(1981),
    [sym_data_get_published_feed] = ACTIONS(1981),
    [sym_data_add_record] = ACTIONS(1981),
    [sym_data_get_event_record] = ACTIONS(1981),
    [sym__wc_data_create_feed] = ACTIONS(1981),
    [sym__wc_data_add_record] = ACTIONS(1981),
    [sym__wc_data_get_event_record] = ACTIONS(1981),
    [sym_format] = ACTIONS(1981),
    [sym_astro_clock] = ACTIONS(1981),
    [sym_set_outdoor_temperature] = ACTIONS(1981),
    [sym_set_virtual_level_count] = ACTIONS(1981),
    [sym_set_virtual_channel_count] = ACTIONS(1981),
    [sym_set_virtual_port_count] = ACTIONS(1981),
    [sym__wc_encode_int] = ACTIONS(1981),
    [sym__wc_decode_int] = ACTIONS(1981),
    [sym__wc_int] = ACTIONS(1981),
    [sym__wc_to_ch_int] = ACTIONS(1981),
    [sym__ch_to_wc_int] = ACTIONS(1981),
    [sym__wc_find_string_int] = ACTIONS(1981),
    [sym__wc_left_string_int] = ACTIONS(1981),
    [sym__wc_lower_string_int] = ACTIONS(1981),
    [sym__wc_mid_string_int] = ACTIONS(1981),
    [sym__wc_remove_string_int] = ACTIONS(1981),
    [sym__wc_right_string_int] = ACTIONS(1981),
    [sym__wc_upper_string_int] = ACTIONS(1981),
    [sym__wc_compare_string_int] = ACTIONS(1981),
    [sym__wc_get_buffer_char_int] = ACTIONS(1981),
    [sym__wc_get_buffer_string_int] = ACTIONS(1981),
    [sym__wc_concat_string_int] = ACTIONS(1981),
    [sym__wc_file_open_int] = ACTIONS(1981),
    [sym__wc_file_close_int] = ACTIONS(1981),
    [sym__wc_file_read_int] = ACTIONS(1981),
    [sym__wc_file_read_line_int] = ACTIONS(1981),
    [sym__wc_file_write_int] = ACTIONS(1981),
    [sym__wc_file_write_line_int] = ACTIONS(1981),
    [aux_sym_preproc_include_token1] = ACTIONS(1981),
    [aux_sym_preproc_define_token1] = ACTIONS(1983),
    [aux_sym_preproc_warn_token1] = ACTIONS(1983),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1983),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1983),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1983),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1983),
    [aux_sym_preproc_else_token1] = ACTIONS(1983),
    [anon_sym_LBRACK] = ACTIONS(1983),
    [anon_sym_LBRACE] = ACTIONS(1983),
    [anon_sym_LPAREN] = ACTIONS(1981),
    [anon_sym_BANG] = ACTIONS(1983),
    [anon_sym_TILDE] = ACTIONS(1983),
    [anon_sym_DASH] = ACTIONS(1981),
    [anon_sym_PLUS] = ACTIONS(1981),
    [anon_sym_DASH_DASH] = ACTIONS(1983),
    [anon_sym_PLUS_PLUS] = ACTIONS(1983),
    [anon_sym_DQUOTE] = ACTIONS(1983),
    [anon_sym_SQUOTE] = ACTIONS(1983),
    [sym_number_literal] = ACTIONS(1983),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(308)] = {
    [ts_builtin_sym_end] = ACTIONS(1737),
    [sym_identifier] = ACTIONS(1735),
    [sym_program_name_keyword] = ACTIONS(1735),
    [sym_module_name_keyword] = ACTIONS(1735),
    [sym_define_device_keyword] = ACTIONS(1735),
    [sym_define_combine_keyword] = ACTIONS(1735),
    [sym_define_constant_keyword] = ACTIONS(1735),
    [sym_define_type_keyword] = ACTIONS(1735),
    [sym_define_variable_keyword] = ACTIONS(1735),
    [sym_define_system_variable_keyword] = ACTIONS(1735),
    [sym_define_connect_level_keyword] = ACTIONS(1735),
    [sym_define_latching_keyword] = ACTIONS(1735),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1735),
    [sym_define_toggling_keyword] = ACTIONS(1735),
    [sym_define_call_keyword] = ACTIONS(1735),
    [sym_define_function_keyword] = ACTIONS(1735),
    [sym_define_library_function_keyword] = ACTIONS(1735),
    [sym_define_start_keyword] = ACTIONS(1735),
    [sym_define_event_keyword] = ACTIONS(1735),
    [sym_define_program_keyword] = ACTIONS(1735),
    [sym_define_module_keyword] = ACTIONS(1735),
    [sym_char_keyword] = ACTIONS(1735),
    [sym_widechar_keyword] = ACTIONS(1735),
    [sym_integer_keyword] = ACTIONS(1735),
    [sym_sinteger_keyword] = ACTIONS(1735),
    [sym_long_keyword] = ACTIONS(1735),
    [sym_slong_keyword] = ACTIONS(1735),
    [sym_float_keyword] = ACTIONS(1735),
    [sym_double_keyword] = ACTIONS(1735),
    [sym_variant_keyword] = ACTIONS(1735),
    [sym_variantarray_keyword] = ACTIONS(1735),
    [sym_local_var_keyword] = ACTIONS(1735),
    [sym_stack_var_keyword] = ACTIONS(1735),
    [sym_constant_keyword] = ACTIONS(1735),
    [sym_volatile_keyword] = ACTIONS(1735),
    [sym_non_volatile_keyword] = ACTIONS(1735),
    [sym_persistent_keyword] = ACTIONS(1735),
    [sym_dev_keyword] = ACTIONS(1735),
    [sym_devlev_keyword] = ACTIONS(1735),
    [sym_devchan_keyword] = ACTIONS(1735),
    [sym_if_keyword] = ACTIONS(1735),
    [sym_else_keyword] = ACTIONS(1735),
    [sym_switch_keyword] = ACTIONS(1735),
    [sym_case_keyword] = ACTIONS(1735),
    [sym_default_keyword] = ACTIONS(1735),
    [sym_while_keyword] = ACTIONS(1735),
    [sym_for_keyword] = ACTIONS(1735),
    [sym_break_keyword] = ACTIONS(1735),
    [sym_continue_keyword] = ACTIONS(1735),
    [sym_return_keyword] = ACTIONS(1735),
    [sym_select_keyword] = ACTIONS(1735),
    [sym_active_keyword] = ACTIONS(1735),
    [sym_struct_keyword] = ACTIONS(1735),
    [sym_structure_keyword] = ACTIONS(1735),
    [sym_button_event_keyword] = ACTIONS(1735),
    [sym_channel_event_keyword] = ACTIONS(1735),
    [sym_level_event_keyword] = ACTIONS(1735),
    [sym_data_event_keyword] = ACTIONS(1735),
    [sym_timeline_event_keyword] = ACTIONS(1735),
    [sym_custom_event_keyword] = ACTIONS(1735),
    [sym_push_keyword] = ACTIONS(1735),
    [sym_release_keyword] = ACTIONS(1735),
    [aux_sym_on_keyword_token1] = ACTIONS(1735),
    [aux_sym_off_keyword_token1] = ACTIONS(1735),
    [sym_send_level_keyword] = ACTIONS(1735),
    [sym_send_string_keyword] = ACTIONS(1735),
    [sym_send_command_keyword] = ACTIONS(1735),
    [sym_clear_buffer_keyword] = ACTIONS(1735),
    [sym_create_buffer_keyword] = ACTIONS(1735),
    [sym_create_multi_buffer_keyword] = ACTIONS(1735),
    [sym_call_keyword] = ACTIONS(1735),
    [sym_system_call_keyword] = ACTIONS(1735),
    [sym_devchan_to_keyword] = ACTIONS(1735),
    [sym_devchan_min_to_keyword] = ACTIONS(1735),
    [sym_devchan_total_off_keyword] = ACTIONS(1735),
    [sym_devchan_pulse_keyword] = ACTIONS(1735),
    [sym_wait_keyword] = ACTIONS(1735),
    [sym_cancel_wait_keyword] = ACTIONS(1735),
    [sym_wait_until_keyword] = ACTIONS(1735),
    [sym_cancel_wait_until_keyword] = ACTIONS(1735),
    [sym_cancel_all_wait_keyword] = ACTIONS(1735),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1735),
    [sym_not] = ACTIONS(1735),
    [sym_bnot] = ACTIONS(1735),
    [sym___file__] = ACTIONS(1735),
    [sym___date__] = ACTIONS(1735),
    [sym___ldate__] = ACTIONS(1735),
    [sym___time__] = ACTIONS(1735),
    [sym___line__] = ACTIONS(1735),
    [sym___name__] = ACTIONS(1735),
    [sym___netlinx__] = ACTIONS(1735),
    [sym_netlinx_axi_version] = ACTIONS(1735),
    [sym_url_flg_tcp] = ACTIONS(1735),
    [sym_url_flg_accinfopresent] = ACTIONS(1735),
    [sym_url_flg_temp] = ACTIONS(1735),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1735),
    [sym_url_flg_stat_mask] = ACTIONS(1735),
    [sym_url_flg_stat_lookup] = ACTIONS(1735),
    [sym_url_flg_stat_connecting] = ACTIONS(1735),
    [sym_url_flg_stat_waiting] = ACTIONS(1735),
    [sym_url_flg_stat_connected] = ACTIONS(1735),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1735),
    [sym_first_virtual_device] = ACTIONS(1735),
    [sym_first_local_port] = ACTIONS(1735),
    [sym_dynamic_virtual_device] = ACTIONS(1735),
    [sym_timeline_once] = ACTIONS(1735),
    [sym_timeline_repeat] = ACTIONS(1735),
    [sym_timeline_absolute] = ACTIONS(1735),
    [sym_timeline_relative] = ACTIONS(1735),
    [sym_do_push_timed_infinite] = ACTIONS(1735),
    [sym_source_type_no_address] = ACTIONS(1735),
    [sym_source_type_neuron_id] = ACTIONS(1735),
    [sym_source_type_ip_address] = ACTIONS(1735),
    [sym_source_type_axlink] = ACTIONS(1735),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1735),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1735),
    [sym_source_type_ip_socket_address] = ACTIONS(1735),
    [sym_source_type_rs232] = ACTIONS(1735),
    [sym_source_type_internal] = ACTIONS(1735),
    [sym_source_type_mac_address] = ACTIONS(1735),
    [sym_source_type_ipv4_port] = ACTIONS(1735),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1735),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1735),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1735),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1735),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1735),
    [sym_true] = ACTIONS(1735),
    [sym_false] = ACTIONS(1735),
    [sym_file_read_only] = ACTIONS(1735),
    [sym_file_rw_new] = ACTIONS(1735),
    [sym_file_rw_append] = ACTIONS(1735),
    [sym_ip_tcp] = ACTIONS(1735),
    [sym_ip_udp] = ACTIONS(1735),
    [sym_ip_udp_2way] = ACTIONS(1735),
    [sym_xml_encode_types] = ACTIONS(1735),
    [sym_xml_encode_char_as_list] = ACTIONS(1735),
    [sym_xml_encode_le] = ACTIONS(1735),
    [sym_xml_decode_types] = ACTIONS(1735),
    [sym_xml_decode_no_preserve] = ACTIONS(1735),
    [sym_duet_dev_type_display_device] = ACTIONS(1735),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1735),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1735),
    [sym_duet_dev_type_io_device] = ACTIONS(1735),
    [sym_duet_dev_type_relay_device] = ACTIONS(1735),
    [sym_duet_dev_type_ups] = ACTIONS(1735),
    [sym_duet_dev_type_amplifier] = ACTIONS(1735),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1735),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1735),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1735),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1735),
    [sym_duet_dev_type_camera] = ACTIONS(1735),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1735),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1735),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1735),
    [sym_duet_dev_type_dss] = ACTIONS(1735),
    [sym_duet_dev_type_dvr] = ACTIONS(1735),
    [sym_duet_dev_type_disc_device] = ACTIONS(1735),
    [sym_duet_dev_type_document_camera] = ACTIONS(1735),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1735),
    [sym_duet_dev_type_hvac] = ACTIONS(1735),
    [sym_duet_dev_type_keypad] = ACTIONS(1735),
    [sym_duet_dev_type_light] = ACTIONS(1735),
    [sym_duet_dev_type_monitor] = ACTIONS(1735),
    [sym_duet_dev_type_motor] = ACTIONS(1735),
    [sym_duet_dev_type_multi_window] = ACTIONS(1735),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1735),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1735),
    [sym_duet_dev_type_receiver] = ACTIONS(1735),
    [sym_duet_dev_type_security_system] = ACTIONS(1735),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1735),
    [sym_duet_dev_type_settop_box] = ACTIONS(1735),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1735),
    [sym_duet_dev_type_switcher] = ACTIONS(1735),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1735),
    [sym_duet_dev_type_tv] = ACTIONS(1735),
    [sym_duet_dev_type_utility] = ACTIONS(1735),
    [sym_duet_dev_type_vcr] = ACTIONS(1735),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1735),
    [sym_duet_dev_type_video_processor] = ACTIONS(1735),
    [sym_duet_dev_type_video_projector] = ACTIONS(1735),
    [sym_duet_dev_type_video_wall] = ACTIONS(1735),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1735),
    [sym_duet_dev_type_weather] = ACTIONS(1735),
    [sym_duet_dev_not_polled] = ACTIONS(1735),
    [sym_duet_dev_polled] = ACTIONS(1735),
    [sym_clkmgr_mode_network] = ACTIONS(1735),
    [sym_clkmgr_mode_standalone] = ACTIONS(1735),
    [sym_normal_standby] = ACTIONS(1735),
    [sym_normal_wake] = ACTIONS(1735),
    [sym_ip_multicast_ttl_option] = ACTIONS(1735),
    [sym_ip_ttl_subnet] = ACTIONS(1735),
    [sym_ip_ttl_site] = ACTIONS(1735),
    [sym_ip_ttl_region] = ACTIONS(1735),
    [sym_ip_ttl_continent] = ACTIONS(1735),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1735),
    [sym_ip_nodelay_on] = ACTIONS(1735),
    [sym_ip_nodelay_off] = ACTIONS(1735),
    [sym_amx_error] = ACTIONS(1735),
    [sym_amx_warning] = ACTIONS(1735),
    [sym_amx_info] = ACTIONS(1735),
    [sym_amx_debug] = ACTIONS(1735),
    [sym_netlinx_login_success] = ACTIONS(1735),
    [sym_netlinx_login_fail] = ACTIONS(1735),
    [sym_netlinx_logout] = ACTIONS(1735),
    [sym_null_str] = ACTIONS(1735),
    [sym_smtp_address] = ACTIONS(1735),
    [sym_smtp_port_number] = ACTIONS(1735),
    [sym_smtp_username] = ACTIONS(1735),
    [sym_smtp_password] = ACTIONS(1735),
    [sym_smtp_require_tls] = ACTIONS(1735),
    [sym_smtp_from] = ACTIONS(1735),
    [sym_smtp_tls_true] = ACTIONS(1735),
    [sym_smtp_tls_false] = ACTIONS(1735),
    [sym_listview_on_row_select_event] = ACTIONS(1735),
    [sym_data_structure_unknown] = ACTIONS(1735),
    [sym_data_structure_datafeed] = ACTIONS(1735),
    [sym_data_structure_datarecordset] = ACTIONS(1735),
    [sym_data_structure_datarecord] = ACTIONS(1735),
    [sym_data_structure_datafield] = ACTIONS(1735),
    [sym_data_max_content_field_count] = ACTIONS(1735),
    [sym_data_max_metadata_field_count] = ACTIONS(1735),
    [sym_data_max_id_length] = ACTIONS(1735),
    [sym_data_max_name_length] = ACTIONS(1735),
    [sym_data_max_description_length] = ACTIONS(1735),
    [sym_data_max_type_length] = ACTIONS(1735),
    [sym_data_max_format_length] = ACTIONS(1735),
    [sym_data_max_source_length] = ACTIONS(1735),
    [sym_data_max_value_length] = ACTIONS(1735),
    [sym_data_max_label_length] = ACTIONS(1735),
    [sym_data_max_url_length] = ACTIONS(1735),
    [sym_max_last_login_info_length] = ACTIONS(1735),
    [sym_data_type_unknown] = ACTIONS(1735),
    [sym_data_type_string] = ACTIONS(1735),
    [sym_data_type_datetime] = ACTIONS(1735),
    [sym_data_type_date] = ACTIONS(1735),
    [sym_data_type_time] = ACTIONS(1735),
    [sym_data_type_image] = ACTIONS(1735),
    [sym_data_format_url] = ACTIONS(1735),
    [sym_data_format_phone] = ACTIONS(1735),
    [sym_data_format_email] = ACTIONS(1735),
    [sym_data_format_iso8601] = ACTIONS(1735),
    [sym_auditlog_priv] = ACTIONS(1735),
    [sym_configuration_priv] = ACTIONS(1735),
    [sym_deviceconfig_priv] = ACTIONS(1735),
    [sym_ftp_priv] = ACTIONS(1735),
    [sym_http_priv] = ACTIONS(1735),
    [sym_networkconfig_priv] = ACTIONS(1735),
    [sym_programport_priv] = ACTIONS(1735),
    [sym_remoteui_priv] = ACTIONS(1735),
    [sym_securitycontrol_priv] = ACTIONS(1735),
    [sym_softwaremanagement_priv] = ACTIONS(1735),
    [sym_terminal_priv] = ACTIONS(1735),
    [sym_tpadmin_priv] = ACTIONS(1735),
    [sym_user1_priv] = ACTIONS(1735),
    [sym_user2_priv] = ACTIONS(1735),
    [sym_user3_priv] = ACTIONS(1735),
    [sym_user4_priv] = ACTIONS(1735),
    [sym_usermanagement_priv] = ACTIONS(1735),
    [sym_valid_account] = ACTIONS(1735),
    [sym_err_name_invalid] = ACTIONS(1735),
    [sym_err_password_invalid] = ACTIONS(1735),
    [sym_err_invalid_account] = ACTIONS(1735),
    [sym_err_no_authorization] = ACTIONS(1735),
    [sym_err_invalid_parameter] = ACTIONS(1735),
    [sym_err_user_locked_out] = ACTIONS(1735),
    [sym_err_user_account_expired] = ACTIONS(1735),
    [sym_err_general_error] = ACTIONS(1735),
    [sym_err_authentication_error] = ACTIONS(1735),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1735),
    [sym_tls_validate_certificate] = ACTIONS(1735),
    [sym_tdata] = ACTIONS(1735),
    [sym_tchannel] = ACTIONS(1735),
    [sym_tlevel] = ACTIONS(1735),
    [sym_tbutton] = ACTIONS(1735),
    [sym_ttimeline] = ACTIONS(1735),
    [sym_tcustom] = ACTIONS(1735),
    [sym_url_struct] = ACTIONS(1735),
    [sym_dns_struct] = ACTIONS(1735),
    [sym_ip_address_struct] = ACTIONS(1735),
    [sym_dev_info_struct] = ACTIONS(1735),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1735),
    [sym_data_feed] = ACTIONS(1735),
    [sym_data_field] = ACTIONS(1735),
    [sym_data_record] = ACTIONS(1735),
    [sym_wc_data_feed] = ACTIONS(1735),
    [sym_wc_data_field] = ACTIONS(1735),
    [sym_wc_data_record] = ACTIONS(1735),
    [sym_last_login_info] = ACTIONS(1735),
    [sym____reserved___] = ACTIONS(1735),
    [sym_date] = ACTIONS(1735),
    [sym_day] = ACTIONS(1735),
    [sym_time] = ACTIONS(1735),
    [sym_ldate] = ACTIONS(1735),
    [sym_push_device] = ACTIONS(1735),
    [sym_push_channel] = ACTIONS(1735),
    [sym_push_devchan] = ACTIONS(1735),
    [sym_release_device] = ACTIONS(1735),
    [sym_release_channel] = ACTIONS(1735),
    [sym_release_devchan] = ACTIONS(1735),
    [sym_master_slot] = ACTIONS(1735),
    [sym_get_pulse_time] = ACTIONS(1735),
    [sym_get_timer] = ACTIONS(1735),
    [sym_master_sn] = ACTIONS(1735),
    [sym_system_number] = ACTIONS(1735),
    [sym_button] = ACTIONS(1735),
    [sym_dv_channel] = ACTIONS(1735),
    [sym_data] = ACTIONS(1735),
    [sym_level] = ACTIONS(1735),
    [sym_timeline] = ACTIONS(1735),
    [sym_channel] = ACTIONS(1735),
    [sym_custom] = ACTIONS(1735),
    [sym_atoi] = ACTIONS(1735),
    [sym_atol] = ACTIONS(1735),
    [sym_atof] = ACTIONS(1735),
    [sym_itoa] = ACTIONS(1735),
    [sym_itohex] = ACTIONS(1735),
    [sym_hextoi] = ACTIONS(1735),
    [sym_ftoa] = ACTIONS(1735),
    [sym_find_string] = ACTIONS(1735),
    [sym_left_string] = ACTIONS(1735),
    [sym_right_string] = ACTIONS(1735),
    [sym_mid_string] = ACTIONS(1735),
    [sym_remove_string] = ACTIONS(1735),
    [sym_get_buffer_string] = ACTIONS(1735),
    [sym_get_buffer_char] = ACTIONS(1735),
    [sym_get_multi_buffer_string] = ACTIONS(1735),
    [sym_length_string] = ACTIONS(1735),
    [sym_max_length_string] = ACTIONS(1735),
    [sym_set_length_string] = ACTIONS(1735),
    [sym_compare_string] = ACTIONS(1735),
    [sym_upper_string] = ACTIONS(1735),
    [sym_lower_string] = ACTIONS(1735),
    [sym_get_last] = ACTIONS(1735),
    [sym_date_to_day] = ACTIONS(1735),
    [sym_date_to_month] = ACTIONS(1735),
    [sym_date_to_year] = ACTIONS(1735),
    [sym_time_to_hour] = ACTIONS(1735),
    [sym_time_to_minute] = ACTIONS(1735),
    [sym_time_to_second] = ACTIONS(1735),
    [sym_day_of_week] = ACTIONS(1735),
    [sym_set_pulse_time] = ACTIONS(1735),
    [sym_set_timer] = ACTIONS(1735),
    [sym_file_open] = ACTIONS(1735),
    [sym_file_close] = ACTIONS(1735),
    [sym_file_read] = ACTIONS(1735),
    [sym_file_read_line] = ACTIONS(1735),
    [sym_file_write] = ACTIONS(1735),
    [sym_file_write_line] = ACTIONS(1735),
    [sym_file_copy] = ACTIONS(1735),
    [sym_file_delete] = ACTIONS(1735),
    [sym_file_rename] = ACTIONS(1735),
    [sym_file_dir] = ACTIONS(1735),
    [sym_file_getdir] = ACTIONS(1735),
    [sym_file_setdir] = ACTIONS(1735),
    [sym_file_create_dir] = ACTIONS(1735),
    [sym_file_remove_dir] = ACTIONS(1735),
    [sym_file_seek] = ACTIONS(1735),
    [sym_length_array] = ACTIONS(1735),
    [sym_max_length_array] = ACTIONS(1735),
    [sym_set_length_array] = ACTIONS(1735),
    [sym_ip_client_open] = ACTIONS(1735),
    [sym_ip_client_close] = ACTIONS(1735),
    [sym_ip_server_open] = ACTIONS(1735),
    [sym_ip_server_close] = ACTIONS(1735),
    [sym_ip_bound_client_open] = ACTIONS(1735),
    [sym_ip_mc_server_open] = ACTIONS(1735),
    [sym_ip_set_option] = ACTIONS(1735),
    [sym_ssh_client_open] = ACTIONS(1735),
    [sym_ssh_client_close] = ACTIONS(1735),
    [sym_tls_client_open] = ACTIONS(1735),
    [sym_tls_client_close] = ACTIONS(1735),
    [sym_do_push] = ACTIONS(1735),
    [sym_do_push_timed] = ACTIONS(1735),
    [sym_do_release] = ACTIONS(1735),
    [sym_device_id] = ACTIONS(1735),
    [sym_device_info] = ACTIONS(1735),
    [sym_device_id_string] = ACTIONS(1735),
    [sym_device_standby] = ACTIONS(1735),
    [sym_device_wake] = ACTIONS(1735),
    [sym_timeline_create] = ACTIONS(1735),
    [sym_timeline_kill] = ACTIONS(1735),
    [sym_timeline_reload] = ACTIONS(1735),
    [sym_timeline_pause] = ACTIONS(1735),
    [sym_timeline_restart] = ACTIONS(1735),
    [sym_timeline_set] = ACTIONS(1735),
    [sym_timeline_get] = ACTIONS(1735),
    [sym_timeline_active] = ACTIONS(1735),
    [sym_timeline_dynamic_id] = ACTIONS(1735),
    [sym_variable_to_string] = ACTIONS(1735),
    [sym_string_to_variable] = ACTIONS(1735),
    [sym_length_variable_to_string] = ACTIONS(1735),
    [sym_type_cast] = ACTIONS(1735),
    [sym_variable_to_xml] = ACTIONS(1735),
    [sym_xml_to_variable] = ACTIONS(1735),
    [sym_length_variable_to_xml] = ACTIONS(1735),
    [sym_redirect_string] = ACTIONS(1735),
    [sym_random_number] = ACTIONS(1735),
    [sym_get_unique_id] = ACTIONS(1735),
    [sym_get_system_number] = ACTIONS(1735),
    [sym_set_system_number] = ACTIONS(1735),
    [sym_get_serial_number] = ACTIONS(1735),
    [sym_get_master_build] = ACTIONS(1735),
    [sym_set_validation_code] = ACTIONS(1735),
    [sym_reboot] = ACTIONS(1735),
    [sym_load_duet_module] = ACTIONS(1735),
    [sym_unload_duet_module] = ACTIONS(1735),
    [sym_duet_mem_size_set] = ACTIONS(1735),
    [sym_duet_mem_size_get] = ACTIONS(1735),
    [sym_dynamic_application_device] = ACTIONS(1735),
    [sym_dynamic_polled_port] = ACTIONS(1735),
    [sym_static_port_binding] = ACTIONS(1735),
    [sym_static_ip_binding] = ACTIONS(1735),
    [sym_get_dns_list] = ACTIONS(1735),
    [sym_set_dns_list] = ACTIONS(1735),
    [sym_get_ip_address] = ACTIONS(1735),
    [sym_set_ip_address] = ACTIONS(1735),
    [sym_get_url_list] = ACTIONS(1735),
    [sym_add_url_entry] = ACTIONS(1735),
    [sym_delete_url_entry] = ACTIONS(1735),
    [sym_abs_value] = ACTIONS(1735),
    [sym_max_value] = ACTIONS(1735),
    [sym_min_value] = ACTIONS(1735),
    [sym_exp_value] = ACTIONS(1735),
    [sym_log_value] = ACTIONS(1735),
    [sym_log10_value] = ACTIONS(1735),
    [sym_power_value] = ACTIONS(1735),
    [sym_sqrt_value] = ACTIONS(1735),
    [sym_raw_be] = ACTIONS(1735),
    [sym_raw_le] = ACTIONS(1735),
    [sym_internal_queue_size_set] = ACTIONS(1735),
    [sym_internal_queue_size_get] = ACTIONS(1735),
    [sym_internal_threshold_set] = ACTIONS(1735),
    [sym_internal_threshold_get] = ACTIONS(1735),
    [sym_rebuilt_event] = ACTIONS(1735),
    [sym_do_custom_event] = ACTIONS(1735),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1735),
    [sym_clkmgr_set_clk_source] = ACTIONS(1735),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1735),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1735),
    [sym_clkmgr_get_timezone] = ACTIONS(1735),
    [sym_clkmgr_set_timezone] = ACTIONS(1735),
    [sym_clkmgr_get_resync_period] = ACTIONS(1735),
    [sym_clkmgr_set_resync_period] = ACTIONS(1735),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1735),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1735),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1735),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1735),
    [sym_clkmgr_get_timeservers] = ACTIONS(1735),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1735),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1735),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1735),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1735),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1735),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1735),
    [sym_set_log_level] = ACTIONS(1735),
    [sym_get_log_level] = ACTIONS(1735),
    [sym_amx_log] = ACTIONS(1735),
    [sym_smtp_server_config_set] = ACTIONS(1735),
    [sym_smtp_server_config_get] = ACTIONS(1735),
    [sym_smtp_send] = ACTIONS(1735),
    [sym_validate_netlinx_account] = ACTIONS(1735),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1735),
    [sym_authenticate_certificate] = ACTIONS(1735),
    [sym_audit_netlinx_session_event] = ACTIONS(1735),
    [sym_audit_netlinx_generic_event] = ACTIONS(1735),
    [sym_get_available_flash_disk_space] = ACTIONS(1735),
    [sym_get_max_flash_disk_space] = ACTIONS(1735),
    [sym_data_create_feed] = ACTIONS(1735),
    [sym_data_delete_feed] = ACTIONS(1735),
    [sym_data_publish_feed] = ACTIONS(1735),
    [sym_data_get_published_feed] = ACTIONS(1735),
    [sym_data_add_record] = ACTIONS(1735),
    [sym_data_get_event_record] = ACTIONS(1735),
    [sym__wc_data_create_feed] = ACTIONS(1735),
    [sym__wc_data_add_record] = ACTIONS(1735),
    [sym__wc_data_get_event_record] = ACTIONS(1735),
    [sym_format] = ACTIONS(1735),
    [sym_astro_clock] = ACTIONS(1735),
    [sym_set_outdoor_temperature] = ACTIONS(1735),
    [sym_set_virtual_level_count] = ACTIONS(1735),
    [sym_set_virtual_channel_count] = ACTIONS(1735),
    [sym_set_virtual_port_count] = ACTIONS(1735),
    [sym__wc_encode_int] = ACTIONS(1735),
    [sym__wc_decode_int] = ACTIONS(1735),
    [sym__wc_int] = ACTIONS(1735),
    [sym__wc_to_ch_int] = ACTIONS(1735),
    [sym__ch_to_wc_int] = ACTIONS(1735),
    [sym__wc_find_string_int] = ACTIONS(1735),
    [sym__wc_left_string_int] = ACTIONS(1735),
    [sym__wc_lower_string_int] = ACTIONS(1735),
    [sym__wc_mid_string_int] = ACTIONS(1735),
    [sym__wc_remove_string_int] = ACTIONS(1735),
    [sym__wc_right_string_int] = ACTIONS(1735),
    [sym__wc_upper_string_int] = ACTIONS(1735),
    [sym__wc_compare_string_int] = ACTIONS(1735),
    [sym__wc_get_buffer_char_int] = ACTIONS(1735),
    [sym__wc_get_buffer_string_int] = ACTIONS(1735),
    [sym__wc_concat_string_int] = ACTIONS(1735),
    [sym__wc_file_open_int] = ACTIONS(1735),
    [sym__wc_file_close_int] = ACTIONS(1735),
    [sym__wc_file_read_int] = ACTIONS(1735),
    [sym__wc_file_read_line_int] = ACTIONS(1735),
    [sym__wc_file_write_int] = ACTIONS(1735),
    [sym__wc_file_write_line_int] = ACTIONS(1735),
    [aux_sym_preproc_include_token1] = ACTIONS(1735),
    [aux_sym_preproc_define_token1] = ACTIONS(1737),
    [aux_sym_preproc_warn_token1] = ACTIONS(1737),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1737),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1737),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1737),
    [anon_sym_LBRACK] = ACTIONS(1737),
    [anon_sym_LBRACE] = ACTIONS(1737),
    [anon_sym_LPAREN] = ACTIONS(1735),
    [anon_sym_BANG] = ACTIONS(1737),
    [anon_sym_TILDE] = ACTIONS(1737),
    [anon_sym_DASH] = ACTIONS(1735),
    [anon_sym_PLUS] = ACTIONS(1735),
    [anon_sym_DASH_DASH] = ACTIONS(1737),
    [anon_sym_PLUS_PLUS] = ACTIONS(1737),
    [anon_sym_DQUOTE] = ACTIONS(1737),
    [anon_sym_SQUOTE] = ACTIONS(1737),
    [sym_number_literal] = ACTIONS(1737),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(309)] = {
    [sym_identifier] = ACTIONS(1985),
    [sym_program_name_keyword] = ACTIONS(1985),
    [sym_module_name_keyword] = ACTIONS(1985),
    [sym_define_device_keyword] = ACTIONS(1985),
    [sym_define_combine_keyword] = ACTIONS(1985),
    [sym_define_constant_keyword] = ACTIONS(1985),
    [sym_define_type_keyword] = ACTIONS(1985),
    [sym_define_variable_keyword] = ACTIONS(1985),
    [sym_define_system_variable_keyword] = ACTIONS(1985),
    [sym_define_connect_level_keyword] = ACTIONS(1985),
    [sym_define_latching_keyword] = ACTIONS(1985),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1985),
    [sym_define_toggling_keyword] = ACTIONS(1985),
    [sym_define_call_keyword] = ACTIONS(1985),
    [sym_define_function_keyword] = ACTIONS(1985),
    [sym_define_library_function_keyword] = ACTIONS(1985),
    [sym_define_start_keyword] = ACTIONS(1985),
    [sym_define_event_keyword] = ACTIONS(1985),
    [sym_define_program_keyword] = ACTIONS(1985),
    [sym_define_module_keyword] = ACTIONS(1985),
    [sym_char_keyword] = ACTIONS(1985),
    [sym_widechar_keyword] = ACTIONS(1985),
    [sym_integer_keyword] = ACTIONS(1985),
    [sym_sinteger_keyword] = ACTIONS(1985),
    [sym_long_keyword] = ACTIONS(1985),
    [sym_slong_keyword] = ACTIONS(1985),
    [sym_float_keyword] = ACTIONS(1985),
    [sym_double_keyword] = ACTIONS(1985),
    [sym_variant_keyword] = ACTIONS(1985),
    [sym_variantarray_keyword] = ACTIONS(1985),
    [sym_local_var_keyword] = ACTIONS(1985),
    [sym_stack_var_keyword] = ACTIONS(1985),
    [sym_constant_keyword] = ACTIONS(1985),
    [sym_volatile_keyword] = ACTIONS(1985),
    [sym_non_volatile_keyword] = ACTIONS(1985),
    [sym_persistent_keyword] = ACTIONS(1985),
    [sym_dev_keyword] = ACTIONS(1985),
    [sym_devlev_keyword] = ACTIONS(1985),
    [sym_devchan_keyword] = ACTIONS(1985),
    [sym_if_keyword] = ACTIONS(1985),
    [sym_switch_keyword] = ACTIONS(1985),
    [sym_case_keyword] = ACTIONS(1985),
    [sym_default_keyword] = ACTIONS(1985),
    [sym_while_keyword] = ACTIONS(1985),
    [sym_for_keyword] = ACTIONS(1985),
    [sym_break_keyword] = ACTIONS(1985),
    [sym_continue_keyword] = ACTIONS(1985),
    [sym_return_keyword] = ACTIONS(1985),
    [sym_select_keyword] = ACTIONS(1985),
    [sym_active_keyword] = ACTIONS(1985),
    [sym_struct_keyword] = ACTIONS(1985),
    [sym_structure_keyword] = ACTIONS(1985),
    [sym_button_event_keyword] = ACTIONS(1985),
    [sym_channel_event_keyword] = ACTIONS(1985),
    [sym_level_event_keyword] = ACTIONS(1985),
    [sym_data_event_keyword] = ACTIONS(1985),
    [sym_timeline_event_keyword] = ACTIONS(1985),
    [sym_custom_event_keyword] = ACTIONS(1985),
    [sym_push_keyword] = ACTIONS(1985),
    [sym_release_keyword] = ACTIONS(1985),
    [aux_sym_on_keyword_token1] = ACTIONS(1985),
    [aux_sym_off_keyword_token1] = ACTIONS(1985),
    [sym_send_level_keyword] = ACTIONS(1985),
    [sym_send_string_keyword] = ACTIONS(1985),
    [sym_send_command_keyword] = ACTIONS(1985),
    [sym_clear_buffer_keyword] = ACTIONS(1985),
    [sym_create_buffer_keyword] = ACTIONS(1985),
    [sym_create_multi_buffer_keyword] = ACTIONS(1985),
    [sym_call_keyword] = ACTIONS(1985),
    [sym_system_call_keyword] = ACTIONS(1985),
    [sym_devchan_to_keyword] = ACTIONS(1985),
    [sym_devchan_min_to_keyword] = ACTIONS(1985),
    [sym_devchan_total_off_keyword] = ACTIONS(1985),
    [sym_devchan_pulse_keyword] = ACTIONS(1985),
    [sym_wait_keyword] = ACTIONS(1985),
    [sym_cancel_wait_keyword] = ACTIONS(1985),
    [sym_wait_until_keyword] = ACTIONS(1985),
    [sym_cancel_wait_until_keyword] = ACTIONS(1985),
    [sym_cancel_all_wait_keyword] = ACTIONS(1985),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1985),
    [sym_not] = ACTIONS(1985),
    [sym_bnot] = ACTIONS(1985),
    [sym___file__] = ACTIONS(1985),
    [sym___date__] = ACTIONS(1985),
    [sym___ldate__] = ACTIONS(1985),
    [sym___time__] = ACTIONS(1985),
    [sym___line__] = ACTIONS(1985),
    [sym___name__] = ACTIONS(1985),
    [sym___netlinx__] = ACTIONS(1985),
    [sym_netlinx_axi_version] = ACTIONS(1985),
    [sym_url_flg_tcp] = ACTIONS(1985),
    [sym_url_flg_accinfopresent] = ACTIONS(1985),
    [sym_url_flg_temp] = ACTIONS(1985),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1985),
    [sym_url_flg_stat_mask] = ACTIONS(1985),
    [sym_url_flg_stat_lookup] = ACTIONS(1985),
    [sym_url_flg_stat_connecting] = ACTIONS(1985),
    [sym_url_flg_stat_waiting] = ACTIONS(1985),
    [sym_url_flg_stat_connected] = ACTIONS(1985),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1985),
    [sym_first_virtual_device] = ACTIONS(1985),
    [sym_first_local_port] = ACTIONS(1985),
    [sym_dynamic_virtual_device] = ACTIONS(1985),
    [sym_timeline_once] = ACTIONS(1985),
    [sym_timeline_repeat] = ACTIONS(1985),
    [sym_timeline_absolute] = ACTIONS(1985),
    [sym_timeline_relative] = ACTIONS(1985),
    [sym_do_push_timed_infinite] = ACTIONS(1985),
    [sym_source_type_no_address] = ACTIONS(1985),
    [sym_source_type_neuron_id] = ACTIONS(1985),
    [sym_source_type_ip_address] = ACTIONS(1985),
    [sym_source_type_axlink] = ACTIONS(1985),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1985),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1985),
    [sym_source_type_ip_socket_address] = ACTIONS(1985),
    [sym_source_type_rs232] = ACTIONS(1985),
    [sym_source_type_internal] = ACTIONS(1985),
    [sym_source_type_mac_address] = ACTIONS(1985),
    [sym_source_type_ipv4_port] = ACTIONS(1985),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1985),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1985),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1985),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1985),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1985),
    [sym_true] = ACTIONS(1985),
    [sym_false] = ACTIONS(1985),
    [sym_file_read_only] = ACTIONS(1985),
    [sym_file_rw_new] = ACTIONS(1985),
    [sym_file_rw_append] = ACTIONS(1985),
    [sym_ip_tcp] = ACTIONS(1985),
    [sym_ip_udp] = ACTIONS(1985),
    [sym_ip_udp_2way] = ACTIONS(1985),
    [sym_xml_encode_types] = ACTIONS(1985),
    [sym_xml_encode_char_as_list] = ACTIONS(1985),
    [sym_xml_encode_le] = ACTIONS(1985),
    [sym_xml_decode_types] = ACTIONS(1985),
    [sym_xml_decode_no_preserve] = ACTIONS(1985),
    [sym_duet_dev_type_display_device] = ACTIONS(1985),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1985),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1985),
    [sym_duet_dev_type_io_device] = ACTIONS(1985),
    [sym_duet_dev_type_relay_device] = ACTIONS(1985),
    [sym_duet_dev_type_ups] = ACTIONS(1985),
    [sym_duet_dev_type_amplifier] = ACTIONS(1985),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1985),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1985),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1985),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1985),
    [sym_duet_dev_type_camera] = ACTIONS(1985),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1985),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1985),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1985),
    [sym_duet_dev_type_dss] = ACTIONS(1985),
    [sym_duet_dev_type_dvr] = ACTIONS(1985),
    [sym_duet_dev_type_disc_device] = ACTIONS(1985),
    [sym_duet_dev_type_document_camera] = ACTIONS(1985),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1985),
    [sym_duet_dev_type_hvac] = ACTIONS(1985),
    [sym_duet_dev_type_keypad] = ACTIONS(1985),
    [sym_duet_dev_type_light] = ACTIONS(1985),
    [sym_duet_dev_type_monitor] = ACTIONS(1985),
    [sym_duet_dev_type_motor] = ACTIONS(1985),
    [sym_duet_dev_type_multi_window] = ACTIONS(1985),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1985),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1985),
    [sym_duet_dev_type_receiver] = ACTIONS(1985),
    [sym_duet_dev_type_security_system] = ACTIONS(1985),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1985),
    [sym_duet_dev_type_settop_box] = ACTIONS(1985),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1985),
    [sym_duet_dev_type_switcher] = ACTIONS(1985),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1985),
    [sym_duet_dev_type_tv] = ACTIONS(1985),
    [sym_duet_dev_type_utility] = ACTIONS(1985),
    [sym_duet_dev_type_vcr] = ACTIONS(1985),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1985),
    [sym_duet_dev_type_video_processor] = ACTIONS(1985),
    [sym_duet_dev_type_video_projector] = ACTIONS(1985),
    [sym_duet_dev_type_video_wall] = ACTIONS(1985),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1985),
    [sym_duet_dev_type_weather] = ACTIONS(1985),
    [sym_duet_dev_not_polled] = ACTIONS(1985),
    [sym_duet_dev_polled] = ACTIONS(1985),
    [sym_clkmgr_mode_network] = ACTIONS(1985),
    [sym_clkmgr_mode_standalone] = ACTIONS(1985),
    [sym_normal_standby] = ACTIONS(1985),
    [sym_normal_wake] = ACTIONS(1985),
    [sym_ip_multicast_ttl_option] = ACTIONS(1985),
    [sym_ip_ttl_subnet] = ACTIONS(1985),
    [sym_ip_ttl_site] = ACTIONS(1985),
    [sym_ip_ttl_region] = ACTIONS(1985),
    [sym_ip_ttl_continent] = ACTIONS(1985),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1985),
    [sym_ip_nodelay_on] = ACTIONS(1985),
    [sym_ip_nodelay_off] = ACTIONS(1985),
    [sym_amx_error] = ACTIONS(1985),
    [sym_amx_warning] = ACTIONS(1985),
    [sym_amx_info] = ACTIONS(1985),
    [sym_amx_debug] = ACTIONS(1985),
    [sym_netlinx_login_success] = ACTIONS(1985),
    [sym_netlinx_login_fail] = ACTIONS(1985),
    [sym_netlinx_logout] = ACTIONS(1985),
    [sym_null_str] = ACTIONS(1985),
    [sym_smtp_address] = ACTIONS(1985),
    [sym_smtp_port_number] = ACTIONS(1985),
    [sym_smtp_username] = ACTIONS(1985),
    [sym_smtp_password] = ACTIONS(1985),
    [sym_smtp_require_tls] = ACTIONS(1985),
    [sym_smtp_from] = ACTIONS(1985),
    [sym_smtp_tls_true] = ACTIONS(1985),
    [sym_smtp_tls_false] = ACTIONS(1985),
    [sym_listview_on_row_select_event] = ACTIONS(1985),
    [sym_data_structure_unknown] = ACTIONS(1985),
    [sym_data_structure_datafeed] = ACTIONS(1985),
    [sym_data_structure_datarecordset] = ACTIONS(1985),
    [sym_data_structure_datarecord] = ACTIONS(1985),
    [sym_data_structure_datafield] = ACTIONS(1985),
    [sym_data_max_content_field_count] = ACTIONS(1985),
    [sym_data_max_metadata_field_count] = ACTIONS(1985),
    [sym_data_max_id_length] = ACTIONS(1985),
    [sym_data_max_name_length] = ACTIONS(1985),
    [sym_data_max_description_length] = ACTIONS(1985),
    [sym_data_max_type_length] = ACTIONS(1985),
    [sym_data_max_format_length] = ACTIONS(1985),
    [sym_data_max_source_length] = ACTIONS(1985),
    [sym_data_max_value_length] = ACTIONS(1985),
    [sym_data_max_label_length] = ACTIONS(1985),
    [sym_data_max_url_length] = ACTIONS(1985),
    [sym_max_last_login_info_length] = ACTIONS(1985),
    [sym_data_type_unknown] = ACTIONS(1985),
    [sym_data_type_string] = ACTIONS(1985),
    [sym_data_type_datetime] = ACTIONS(1985),
    [sym_data_type_date] = ACTIONS(1985),
    [sym_data_type_time] = ACTIONS(1985),
    [sym_data_type_image] = ACTIONS(1985),
    [sym_data_format_url] = ACTIONS(1985),
    [sym_data_format_phone] = ACTIONS(1985),
    [sym_data_format_email] = ACTIONS(1985),
    [sym_data_format_iso8601] = ACTIONS(1985),
    [sym_auditlog_priv] = ACTIONS(1985),
    [sym_configuration_priv] = ACTIONS(1985),
    [sym_deviceconfig_priv] = ACTIONS(1985),
    [sym_ftp_priv] = ACTIONS(1985),
    [sym_http_priv] = ACTIONS(1985),
    [sym_networkconfig_priv] = ACTIONS(1985),
    [sym_programport_priv] = ACTIONS(1985),
    [sym_remoteui_priv] = ACTIONS(1985),
    [sym_securitycontrol_priv] = ACTIONS(1985),
    [sym_softwaremanagement_priv] = ACTIONS(1985),
    [sym_terminal_priv] = ACTIONS(1985),
    [sym_tpadmin_priv] = ACTIONS(1985),
    [sym_user1_priv] = ACTIONS(1985),
    [sym_user2_priv] = ACTIONS(1985),
    [sym_user3_priv] = ACTIONS(1985),
    [sym_user4_priv] = ACTIONS(1985),
    [sym_usermanagement_priv] = ACTIONS(1985),
    [sym_valid_account] = ACTIONS(1985),
    [sym_err_name_invalid] = ACTIONS(1985),
    [sym_err_password_invalid] = ACTIONS(1985),
    [sym_err_invalid_account] = ACTIONS(1985),
    [sym_err_no_authorization] = ACTIONS(1985),
    [sym_err_invalid_parameter] = ACTIONS(1985),
    [sym_err_user_locked_out] = ACTIONS(1985),
    [sym_err_user_account_expired] = ACTIONS(1985),
    [sym_err_general_error] = ACTIONS(1985),
    [sym_err_authentication_error] = ACTIONS(1985),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1985),
    [sym_tls_validate_certificate] = ACTIONS(1985),
    [sym_tdata] = ACTIONS(1985),
    [sym_tchannel] = ACTIONS(1985),
    [sym_tlevel] = ACTIONS(1985),
    [sym_tbutton] = ACTIONS(1985),
    [sym_ttimeline] = ACTIONS(1985),
    [sym_tcustom] = ACTIONS(1985),
    [sym_url_struct] = ACTIONS(1985),
    [sym_dns_struct] = ACTIONS(1985),
    [sym_ip_address_struct] = ACTIONS(1985),
    [sym_dev_info_struct] = ACTIONS(1985),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1985),
    [sym_data_feed] = ACTIONS(1985),
    [sym_data_field] = ACTIONS(1985),
    [sym_data_record] = ACTIONS(1985),
    [sym_wc_data_feed] = ACTIONS(1985),
    [sym_wc_data_field] = ACTIONS(1985),
    [sym_wc_data_record] = ACTIONS(1985),
    [sym_last_login_info] = ACTIONS(1985),
    [sym____reserved___] = ACTIONS(1985),
    [sym_date] = ACTIONS(1985),
    [sym_day] = ACTIONS(1985),
    [sym_time] = ACTIONS(1985),
    [sym_ldate] = ACTIONS(1985),
    [sym_push_device] = ACTIONS(1985),
    [sym_push_channel] = ACTIONS(1985),
    [sym_push_devchan] = ACTIONS(1985),
    [sym_release_device] = ACTIONS(1985),
    [sym_release_channel] = ACTIONS(1985),
    [sym_release_devchan] = ACTIONS(1985),
    [sym_master_slot] = ACTIONS(1985),
    [sym_get_pulse_time] = ACTIONS(1985),
    [sym_get_timer] = ACTIONS(1985),
    [sym_master_sn] = ACTIONS(1985),
    [sym_system_number] = ACTIONS(1985),
    [sym_button] = ACTIONS(1985),
    [sym_dv_channel] = ACTIONS(1985),
    [sym_data] = ACTIONS(1985),
    [sym_level] = ACTIONS(1985),
    [sym_timeline] = ACTIONS(1985),
    [sym_channel] = ACTIONS(1985),
    [sym_custom] = ACTIONS(1985),
    [sym_atoi] = ACTIONS(1985),
    [sym_atol] = ACTIONS(1985),
    [sym_atof] = ACTIONS(1985),
    [sym_itoa] = ACTIONS(1985),
    [sym_itohex] = ACTIONS(1985),
    [sym_hextoi] = ACTIONS(1985),
    [sym_ftoa] = ACTIONS(1985),
    [sym_find_string] = ACTIONS(1985),
    [sym_left_string] = ACTIONS(1985),
    [sym_right_string] = ACTIONS(1985),
    [sym_mid_string] = ACTIONS(1985),
    [sym_remove_string] = ACTIONS(1985),
    [sym_get_buffer_string] = ACTIONS(1985),
    [sym_get_buffer_char] = ACTIONS(1985),
    [sym_get_multi_buffer_string] = ACTIONS(1985),
    [sym_length_string] = ACTIONS(1985),
    [sym_max_length_string] = ACTIONS(1985),
    [sym_set_length_string] = ACTIONS(1985),
    [sym_compare_string] = ACTIONS(1985),
    [sym_upper_string] = ACTIONS(1985),
    [sym_lower_string] = ACTIONS(1985),
    [sym_get_last] = ACTIONS(1985),
    [sym_date_to_day] = ACTIONS(1985),
    [sym_date_to_month] = ACTIONS(1985),
    [sym_date_to_year] = ACTIONS(1985),
    [sym_time_to_hour] = ACTIONS(1985),
    [sym_time_to_minute] = ACTIONS(1985),
    [sym_time_to_second] = ACTIONS(1985),
    [sym_day_of_week] = ACTIONS(1985),
    [sym_set_pulse_time] = ACTIONS(1985),
    [sym_set_timer] = ACTIONS(1985),
    [sym_file_open] = ACTIONS(1985),
    [sym_file_close] = ACTIONS(1985),
    [sym_file_read] = ACTIONS(1985),
    [sym_file_read_line] = ACTIONS(1985),
    [sym_file_write] = ACTIONS(1985),
    [sym_file_write_line] = ACTIONS(1985),
    [sym_file_copy] = ACTIONS(1985),
    [sym_file_delete] = ACTIONS(1985),
    [sym_file_rename] = ACTIONS(1985),
    [sym_file_dir] = ACTIONS(1985),
    [sym_file_getdir] = ACTIONS(1985),
    [sym_file_setdir] = ACTIONS(1985),
    [sym_file_create_dir] = ACTIONS(1985),
    [sym_file_remove_dir] = ACTIONS(1985),
    [sym_file_seek] = ACTIONS(1985),
    [sym_length_array] = ACTIONS(1985),
    [sym_max_length_array] = ACTIONS(1985),
    [sym_set_length_array] = ACTIONS(1985),
    [sym_ip_client_open] = ACTIONS(1985),
    [sym_ip_client_close] = ACTIONS(1985),
    [sym_ip_server_open] = ACTIONS(1985),
    [sym_ip_server_close] = ACTIONS(1985),
    [sym_ip_bound_client_open] = ACTIONS(1985),
    [sym_ip_mc_server_open] = ACTIONS(1985),
    [sym_ip_set_option] = ACTIONS(1985),
    [sym_ssh_client_open] = ACTIONS(1985),
    [sym_ssh_client_close] = ACTIONS(1985),
    [sym_tls_client_open] = ACTIONS(1985),
    [sym_tls_client_close] = ACTIONS(1985),
    [sym_do_push] = ACTIONS(1985),
    [sym_do_push_timed] = ACTIONS(1985),
    [sym_do_release] = ACTIONS(1985),
    [sym_device_id] = ACTIONS(1985),
    [sym_device_info] = ACTIONS(1985),
    [sym_device_id_string] = ACTIONS(1985),
    [sym_device_standby] = ACTIONS(1985),
    [sym_device_wake] = ACTIONS(1985),
    [sym_timeline_create] = ACTIONS(1985),
    [sym_timeline_kill] = ACTIONS(1985),
    [sym_timeline_reload] = ACTIONS(1985),
    [sym_timeline_pause] = ACTIONS(1985),
    [sym_timeline_restart] = ACTIONS(1985),
    [sym_timeline_set] = ACTIONS(1985),
    [sym_timeline_get] = ACTIONS(1985),
    [sym_timeline_active] = ACTIONS(1985),
    [sym_timeline_dynamic_id] = ACTIONS(1985),
    [sym_variable_to_string] = ACTIONS(1985),
    [sym_string_to_variable] = ACTIONS(1985),
    [sym_length_variable_to_string] = ACTIONS(1985),
    [sym_type_cast] = ACTIONS(1985),
    [sym_variable_to_xml] = ACTIONS(1985),
    [sym_xml_to_variable] = ACTIONS(1985),
    [sym_length_variable_to_xml] = ACTIONS(1985),
    [sym_redirect_string] = ACTIONS(1985),
    [sym_random_number] = ACTIONS(1985),
    [sym_get_unique_id] = ACTIONS(1985),
    [sym_get_system_number] = ACTIONS(1985),
    [sym_set_system_number] = ACTIONS(1985),
    [sym_get_serial_number] = ACTIONS(1985),
    [sym_get_master_build] = ACTIONS(1985),
    [sym_set_validation_code] = ACTIONS(1985),
    [sym_reboot] = ACTIONS(1985),
    [sym_load_duet_module] = ACTIONS(1985),
    [sym_unload_duet_module] = ACTIONS(1985),
    [sym_duet_mem_size_set] = ACTIONS(1985),
    [sym_duet_mem_size_get] = ACTIONS(1985),
    [sym_dynamic_application_device] = ACTIONS(1985),
    [sym_dynamic_polled_port] = ACTIONS(1985),
    [sym_static_port_binding] = ACTIONS(1985),
    [sym_static_ip_binding] = ACTIONS(1985),
    [sym_get_dns_list] = ACTIONS(1985),
    [sym_set_dns_list] = ACTIONS(1985),
    [sym_get_ip_address] = ACTIONS(1985),
    [sym_set_ip_address] = ACTIONS(1985),
    [sym_get_url_list] = ACTIONS(1985),
    [sym_add_url_entry] = ACTIONS(1985),
    [sym_delete_url_entry] = ACTIONS(1985),
    [sym_abs_value] = ACTIONS(1985),
    [sym_max_value] = ACTIONS(1985),
    [sym_min_value] = ACTIONS(1985),
    [sym_exp_value] = ACTIONS(1985),
    [sym_log_value] = ACTIONS(1985),
    [sym_log10_value] = ACTIONS(1985),
    [sym_power_value] = ACTIONS(1985),
    [sym_sqrt_value] = ACTIONS(1985),
    [sym_raw_be] = ACTIONS(1985),
    [sym_raw_le] = ACTIONS(1985),
    [sym_internal_queue_size_set] = ACTIONS(1985),
    [sym_internal_queue_size_get] = ACTIONS(1985),
    [sym_internal_threshold_set] = ACTIONS(1985),
    [sym_internal_threshold_get] = ACTIONS(1985),
    [sym_rebuilt_event] = ACTIONS(1985),
    [sym_do_custom_event] = ACTIONS(1985),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1985),
    [sym_clkmgr_set_clk_source] = ACTIONS(1985),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1985),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1985),
    [sym_clkmgr_get_timezone] = ACTIONS(1985),
    [sym_clkmgr_set_timezone] = ACTIONS(1985),
    [sym_clkmgr_get_resync_period] = ACTIONS(1985),
    [sym_clkmgr_set_resync_period] = ACTIONS(1985),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1985),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1985),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1985),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1985),
    [sym_clkmgr_get_timeservers] = ACTIONS(1985),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1985),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1985),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1985),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1985),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1985),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1985),
    [sym_set_log_level] = ACTIONS(1985),
    [sym_get_log_level] = ACTIONS(1985),
    [sym_amx_log] = ACTIONS(1985),
    [sym_smtp_server_config_set] = ACTIONS(1985),
    [sym_smtp_server_config_get] = ACTIONS(1985),
    [sym_smtp_send] = ACTIONS(1985),
    [sym_validate_netlinx_account] = ACTIONS(1985),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1985),
    [sym_authenticate_certificate] = ACTIONS(1985),
    [sym_audit_netlinx_session_event] = ACTIONS(1985),
    [sym_audit_netlinx_generic_event] = ACTIONS(1985),
    [sym_get_available_flash_disk_space] = ACTIONS(1985),
    [sym_get_max_flash_disk_space] = ACTIONS(1985),
    [sym_data_create_feed] = ACTIONS(1985),
    [sym_data_delete_feed] = ACTIONS(1985),
    [sym_data_publish_feed] = ACTIONS(1985),
    [sym_data_get_published_feed] = ACTIONS(1985),
    [sym_data_add_record] = ACTIONS(1985),
    [sym_data_get_event_record] = ACTIONS(1985),
    [sym__wc_data_create_feed] = ACTIONS(1985),
    [sym__wc_data_add_record] = ACTIONS(1985),
    [sym__wc_data_get_event_record] = ACTIONS(1985),
    [sym_format] = ACTIONS(1985),
    [sym_astro_clock] = ACTIONS(1985),
    [sym_set_outdoor_temperature] = ACTIONS(1985),
    [sym_set_virtual_level_count] = ACTIONS(1985),
    [sym_set_virtual_channel_count] = ACTIONS(1985),
    [sym_set_virtual_port_count] = ACTIONS(1985),
    [sym__wc_encode_int] = ACTIONS(1985),
    [sym__wc_decode_int] = ACTIONS(1985),
    [sym__wc_int] = ACTIONS(1985),
    [sym__wc_to_ch_int] = ACTIONS(1985),
    [sym__ch_to_wc_int] = ACTIONS(1985),
    [sym__wc_find_string_int] = ACTIONS(1985),
    [sym__wc_left_string_int] = ACTIONS(1985),
    [sym__wc_lower_string_int] = ACTIONS(1985),
    [sym__wc_mid_string_int] = ACTIONS(1985),
    [sym__wc_remove_string_int] = ACTIONS(1985),
    [sym__wc_right_string_int] = ACTIONS(1985),
    [sym__wc_upper_string_int] = ACTIONS(1985),
    [sym__wc_compare_string_int] = ACTIONS(1985),
    [sym__wc_get_buffer_char_int] = ACTIONS(1985),
    [sym__wc_get_buffer_string_int] = ACTIONS(1985),
    [sym__wc_concat_string_int] = ACTIONS(1985),
    [sym__wc_file_open_int] = ACTIONS(1985),
    [sym__wc_file_close_int] = ACTIONS(1985),
    [sym__wc_file_read_int] = ACTIONS(1985),
    [sym__wc_file_read_line_int] = ACTIONS(1985),
    [sym__wc_file_write_int] = ACTIONS(1985),
    [sym__wc_file_write_line_int] = ACTIONS(1985),
    [aux_sym_preproc_include_token1] = ACTIONS(1985),
    [aux_sym_preproc_define_token1] = ACTIONS(1987),
    [aux_sym_preproc_warn_token1] = ACTIONS(1987),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1987),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1987),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1987),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1987),
    [aux_sym_preproc_else_token1] = ACTIONS(1987),
    [anon_sym_LBRACK] = ACTIONS(1987),
    [anon_sym_LBRACE] = ACTIONS(1987),
    [anon_sym_LPAREN] = ACTIONS(1985),
    [anon_sym_BANG] = ACTIONS(1987),
    [anon_sym_TILDE] = ACTIONS(1987),
    [anon_sym_DASH] = ACTIONS(1985),
    [anon_sym_PLUS] = ACTIONS(1985),
    [anon_sym_DASH_DASH] = ACTIONS(1987),
    [anon_sym_PLUS_PLUS] = ACTIONS(1987),
    [anon_sym_DQUOTE] = ACTIONS(1987),
    [anon_sym_SQUOTE] = ACTIONS(1987),
    [sym_number_literal] = ACTIONS(1987),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(310)] = {
    [sym_identifier] = ACTIONS(1989),
    [sym_program_name_keyword] = ACTIONS(1989),
    [sym_module_name_keyword] = ACTIONS(1989),
    [sym_define_device_keyword] = ACTIONS(1989),
    [sym_define_combine_keyword] = ACTIONS(1989),
    [sym_define_constant_keyword] = ACTIONS(1989),
    [sym_define_type_keyword] = ACTIONS(1989),
    [sym_define_variable_keyword] = ACTIONS(1989),
    [sym_define_system_variable_keyword] = ACTIONS(1989),
    [sym_define_connect_level_keyword] = ACTIONS(1989),
    [sym_define_latching_keyword] = ACTIONS(1989),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1989),
    [sym_define_toggling_keyword] = ACTIONS(1989),
    [sym_define_call_keyword] = ACTIONS(1989),
    [sym_define_function_keyword] = ACTIONS(1989),
    [sym_define_library_function_keyword] = ACTIONS(1989),
    [sym_define_start_keyword] = ACTIONS(1989),
    [sym_define_event_keyword] = ACTIONS(1989),
    [sym_define_program_keyword] = ACTIONS(1989),
    [sym_define_module_keyword] = ACTIONS(1989),
    [sym_char_keyword] = ACTIONS(1989),
    [sym_widechar_keyword] = ACTIONS(1989),
    [sym_integer_keyword] = ACTIONS(1989),
    [sym_sinteger_keyword] = ACTIONS(1989),
    [sym_long_keyword] = ACTIONS(1989),
    [sym_slong_keyword] = ACTIONS(1989),
    [sym_float_keyword] = ACTIONS(1989),
    [sym_double_keyword] = ACTIONS(1989),
    [sym_variant_keyword] = ACTIONS(1989),
    [sym_variantarray_keyword] = ACTIONS(1989),
    [sym_local_var_keyword] = ACTIONS(1989),
    [sym_stack_var_keyword] = ACTIONS(1989),
    [sym_constant_keyword] = ACTIONS(1989),
    [sym_volatile_keyword] = ACTIONS(1989),
    [sym_non_volatile_keyword] = ACTIONS(1989),
    [sym_persistent_keyword] = ACTIONS(1989),
    [sym_dev_keyword] = ACTIONS(1989),
    [sym_devlev_keyword] = ACTIONS(1989),
    [sym_devchan_keyword] = ACTIONS(1989),
    [sym_if_keyword] = ACTIONS(1989),
    [sym_switch_keyword] = ACTIONS(1989),
    [sym_case_keyword] = ACTIONS(1989),
    [sym_default_keyword] = ACTIONS(1989),
    [sym_while_keyword] = ACTIONS(1989),
    [sym_for_keyword] = ACTIONS(1989),
    [sym_break_keyword] = ACTIONS(1989),
    [sym_continue_keyword] = ACTIONS(1989),
    [sym_return_keyword] = ACTIONS(1989),
    [sym_select_keyword] = ACTIONS(1989),
    [sym_active_keyword] = ACTIONS(1989),
    [sym_struct_keyword] = ACTIONS(1989),
    [sym_structure_keyword] = ACTIONS(1989),
    [sym_button_event_keyword] = ACTIONS(1989),
    [sym_channel_event_keyword] = ACTIONS(1989),
    [sym_level_event_keyword] = ACTIONS(1989),
    [sym_data_event_keyword] = ACTIONS(1989),
    [sym_timeline_event_keyword] = ACTIONS(1989),
    [sym_custom_event_keyword] = ACTIONS(1989),
    [sym_push_keyword] = ACTIONS(1989),
    [sym_release_keyword] = ACTIONS(1989),
    [aux_sym_on_keyword_token1] = ACTIONS(1989),
    [aux_sym_off_keyword_token1] = ACTIONS(1989),
    [sym_send_level_keyword] = ACTIONS(1989),
    [sym_send_string_keyword] = ACTIONS(1989),
    [sym_send_command_keyword] = ACTIONS(1989),
    [sym_clear_buffer_keyword] = ACTIONS(1989),
    [sym_create_buffer_keyword] = ACTIONS(1989),
    [sym_create_multi_buffer_keyword] = ACTIONS(1989),
    [sym_call_keyword] = ACTIONS(1989),
    [sym_system_call_keyword] = ACTIONS(1989),
    [sym_devchan_to_keyword] = ACTIONS(1989),
    [sym_devchan_min_to_keyword] = ACTIONS(1989),
    [sym_devchan_total_off_keyword] = ACTIONS(1989),
    [sym_devchan_pulse_keyword] = ACTIONS(1989),
    [sym_wait_keyword] = ACTIONS(1989),
    [sym_cancel_wait_keyword] = ACTIONS(1989),
    [sym_wait_until_keyword] = ACTIONS(1989),
    [sym_cancel_wait_until_keyword] = ACTIONS(1989),
    [sym_cancel_all_wait_keyword] = ACTIONS(1989),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1989),
    [sym_not] = ACTIONS(1989),
    [sym_bnot] = ACTIONS(1989),
    [sym___file__] = ACTIONS(1989),
    [sym___date__] = ACTIONS(1989),
    [sym___ldate__] = ACTIONS(1989),
    [sym___time__] = ACTIONS(1989),
    [sym___line__] = ACTIONS(1989),
    [sym___name__] = ACTIONS(1989),
    [sym___netlinx__] = ACTIONS(1989),
    [sym_netlinx_axi_version] = ACTIONS(1989),
    [sym_url_flg_tcp] = ACTIONS(1989),
    [sym_url_flg_accinfopresent] = ACTIONS(1989),
    [sym_url_flg_temp] = ACTIONS(1989),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1989),
    [sym_url_flg_stat_mask] = ACTIONS(1989),
    [sym_url_flg_stat_lookup] = ACTIONS(1989),
    [sym_url_flg_stat_connecting] = ACTIONS(1989),
    [sym_url_flg_stat_waiting] = ACTIONS(1989),
    [sym_url_flg_stat_connected] = ACTIONS(1989),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1989),
    [sym_first_virtual_device] = ACTIONS(1989),
    [sym_first_local_port] = ACTIONS(1989),
    [sym_dynamic_virtual_device] = ACTIONS(1989),
    [sym_timeline_once] = ACTIONS(1989),
    [sym_timeline_repeat] = ACTIONS(1989),
    [sym_timeline_absolute] = ACTIONS(1989),
    [sym_timeline_relative] = ACTIONS(1989),
    [sym_do_push_timed_infinite] = ACTIONS(1989),
    [sym_source_type_no_address] = ACTIONS(1989),
    [sym_source_type_neuron_id] = ACTIONS(1989),
    [sym_source_type_ip_address] = ACTIONS(1989),
    [sym_source_type_axlink] = ACTIONS(1989),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1989),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1989),
    [sym_source_type_ip_socket_address] = ACTIONS(1989),
    [sym_source_type_rs232] = ACTIONS(1989),
    [sym_source_type_internal] = ACTIONS(1989),
    [sym_source_type_mac_address] = ACTIONS(1989),
    [sym_source_type_ipv4_port] = ACTIONS(1989),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1989),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1989),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1989),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1989),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1989),
    [sym_true] = ACTIONS(1989),
    [sym_false] = ACTIONS(1989),
    [sym_file_read_only] = ACTIONS(1989),
    [sym_file_rw_new] = ACTIONS(1989),
    [sym_file_rw_append] = ACTIONS(1989),
    [sym_ip_tcp] = ACTIONS(1989),
    [sym_ip_udp] = ACTIONS(1989),
    [sym_ip_udp_2way] = ACTIONS(1989),
    [sym_xml_encode_types] = ACTIONS(1989),
    [sym_xml_encode_char_as_list] = ACTIONS(1989),
    [sym_xml_encode_le] = ACTIONS(1989),
    [sym_xml_decode_types] = ACTIONS(1989),
    [sym_xml_decode_no_preserve] = ACTIONS(1989),
    [sym_duet_dev_type_display_device] = ACTIONS(1989),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1989),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1989),
    [sym_duet_dev_type_io_device] = ACTIONS(1989),
    [sym_duet_dev_type_relay_device] = ACTIONS(1989),
    [sym_duet_dev_type_ups] = ACTIONS(1989),
    [sym_duet_dev_type_amplifier] = ACTIONS(1989),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1989),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1989),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1989),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1989),
    [sym_duet_dev_type_camera] = ACTIONS(1989),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1989),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1989),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1989),
    [sym_duet_dev_type_dss] = ACTIONS(1989),
    [sym_duet_dev_type_dvr] = ACTIONS(1989),
    [sym_duet_dev_type_disc_device] = ACTIONS(1989),
    [sym_duet_dev_type_document_camera] = ACTIONS(1989),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1989),
    [sym_duet_dev_type_hvac] = ACTIONS(1989),
    [sym_duet_dev_type_keypad] = ACTIONS(1989),
    [sym_duet_dev_type_light] = ACTIONS(1989),
    [sym_duet_dev_type_monitor] = ACTIONS(1989),
    [sym_duet_dev_type_motor] = ACTIONS(1989),
    [sym_duet_dev_type_multi_window] = ACTIONS(1989),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1989),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1989),
    [sym_duet_dev_type_receiver] = ACTIONS(1989),
    [sym_duet_dev_type_security_system] = ACTIONS(1989),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1989),
    [sym_duet_dev_type_settop_box] = ACTIONS(1989),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1989),
    [sym_duet_dev_type_switcher] = ACTIONS(1989),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1989),
    [sym_duet_dev_type_tv] = ACTIONS(1989),
    [sym_duet_dev_type_utility] = ACTIONS(1989),
    [sym_duet_dev_type_vcr] = ACTIONS(1989),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1989),
    [sym_duet_dev_type_video_processor] = ACTIONS(1989),
    [sym_duet_dev_type_video_projector] = ACTIONS(1989),
    [sym_duet_dev_type_video_wall] = ACTIONS(1989),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1989),
    [sym_duet_dev_type_weather] = ACTIONS(1989),
    [sym_duet_dev_not_polled] = ACTIONS(1989),
    [sym_duet_dev_polled] = ACTIONS(1989),
    [sym_clkmgr_mode_network] = ACTIONS(1989),
    [sym_clkmgr_mode_standalone] = ACTIONS(1989),
    [sym_normal_standby] = ACTIONS(1989),
    [sym_normal_wake] = ACTIONS(1989),
    [sym_ip_multicast_ttl_option] = ACTIONS(1989),
    [sym_ip_ttl_subnet] = ACTIONS(1989),
    [sym_ip_ttl_site] = ACTIONS(1989),
    [sym_ip_ttl_region] = ACTIONS(1989),
    [sym_ip_ttl_continent] = ACTIONS(1989),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1989),
    [sym_ip_nodelay_on] = ACTIONS(1989),
    [sym_ip_nodelay_off] = ACTIONS(1989),
    [sym_amx_error] = ACTIONS(1989),
    [sym_amx_warning] = ACTIONS(1989),
    [sym_amx_info] = ACTIONS(1989),
    [sym_amx_debug] = ACTIONS(1989),
    [sym_netlinx_login_success] = ACTIONS(1989),
    [sym_netlinx_login_fail] = ACTIONS(1989),
    [sym_netlinx_logout] = ACTIONS(1989),
    [sym_null_str] = ACTIONS(1989),
    [sym_smtp_address] = ACTIONS(1989),
    [sym_smtp_port_number] = ACTIONS(1989),
    [sym_smtp_username] = ACTIONS(1989),
    [sym_smtp_password] = ACTIONS(1989),
    [sym_smtp_require_tls] = ACTIONS(1989),
    [sym_smtp_from] = ACTIONS(1989),
    [sym_smtp_tls_true] = ACTIONS(1989),
    [sym_smtp_tls_false] = ACTIONS(1989),
    [sym_listview_on_row_select_event] = ACTIONS(1989),
    [sym_data_structure_unknown] = ACTIONS(1989),
    [sym_data_structure_datafeed] = ACTIONS(1989),
    [sym_data_structure_datarecordset] = ACTIONS(1989),
    [sym_data_structure_datarecord] = ACTIONS(1989),
    [sym_data_structure_datafield] = ACTIONS(1989),
    [sym_data_max_content_field_count] = ACTIONS(1989),
    [sym_data_max_metadata_field_count] = ACTIONS(1989),
    [sym_data_max_id_length] = ACTIONS(1989),
    [sym_data_max_name_length] = ACTIONS(1989),
    [sym_data_max_description_length] = ACTIONS(1989),
    [sym_data_max_type_length] = ACTIONS(1989),
    [sym_data_max_format_length] = ACTIONS(1989),
    [sym_data_max_source_length] = ACTIONS(1989),
    [sym_data_max_value_length] = ACTIONS(1989),
    [sym_data_max_label_length] = ACTIONS(1989),
    [sym_data_max_url_length] = ACTIONS(1989),
    [sym_max_last_login_info_length] = ACTIONS(1989),
    [sym_data_type_unknown] = ACTIONS(1989),
    [sym_data_type_string] = ACTIONS(1989),
    [sym_data_type_datetime] = ACTIONS(1989),
    [sym_data_type_date] = ACTIONS(1989),
    [sym_data_type_time] = ACTIONS(1989),
    [sym_data_type_image] = ACTIONS(1989),
    [sym_data_format_url] = ACTIONS(1989),
    [sym_data_format_phone] = ACTIONS(1989),
    [sym_data_format_email] = ACTIONS(1989),
    [sym_data_format_iso8601] = ACTIONS(1989),
    [sym_auditlog_priv] = ACTIONS(1989),
    [sym_configuration_priv] = ACTIONS(1989),
    [sym_deviceconfig_priv] = ACTIONS(1989),
    [sym_ftp_priv] = ACTIONS(1989),
    [sym_http_priv] = ACTIONS(1989),
    [sym_networkconfig_priv] = ACTIONS(1989),
    [sym_programport_priv] = ACTIONS(1989),
    [sym_remoteui_priv] = ACTIONS(1989),
    [sym_securitycontrol_priv] = ACTIONS(1989),
    [sym_softwaremanagement_priv] = ACTIONS(1989),
    [sym_terminal_priv] = ACTIONS(1989),
    [sym_tpadmin_priv] = ACTIONS(1989),
    [sym_user1_priv] = ACTIONS(1989),
    [sym_user2_priv] = ACTIONS(1989),
    [sym_user3_priv] = ACTIONS(1989),
    [sym_user4_priv] = ACTIONS(1989),
    [sym_usermanagement_priv] = ACTIONS(1989),
    [sym_valid_account] = ACTIONS(1989),
    [sym_err_name_invalid] = ACTIONS(1989),
    [sym_err_password_invalid] = ACTIONS(1989),
    [sym_err_invalid_account] = ACTIONS(1989),
    [sym_err_no_authorization] = ACTIONS(1989),
    [sym_err_invalid_parameter] = ACTIONS(1989),
    [sym_err_user_locked_out] = ACTIONS(1989),
    [sym_err_user_account_expired] = ACTIONS(1989),
    [sym_err_general_error] = ACTIONS(1989),
    [sym_err_authentication_error] = ACTIONS(1989),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1989),
    [sym_tls_validate_certificate] = ACTIONS(1989),
    [sym_tdata] = ACTIONS(1989),
    [sym_tchannel] = ACTIONS(1989),
    [sym_tlevel] = ACTIONS(1989),
    [sym_tbutton] = ACTIONS(1989),
    [sym_ttimeline] = ACTIONS(1989),
    [sym_tcustom] = ACTIONS(1989),
    [sym_url_struct] = ACTIONS(1989),
    [sym_dns_struct] = ACTIONS(1989),
    [sym_ip_address_struct] = ACTIONS(1989),
    [sym_dev_info_struct] = ACTIONS(1989),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1989),
    [sym_data_feed] = ACTIONS(1989),
    [sym_data_field] = ACTIONS(1989),
    [sym_data_record] = ACTIONS(1989),
    [sym_wc_data_feed] = ACTIONS(1989),
    [sym_wc_data_field] = ACTIONS(1989),
    [sym_wc_data_record] = ACTIONS(1989),
    [sym_last_login_info] = ACTIONS(1989),
    [sym____reserved___] = ACTIONS(1989),
    [sym_date] = ACTIONS(1989),
    [sym_day] = ACTIONS(1989),
    [sym_time] = ACTIONS(1989),
    [sym_ldate] = ACTIONS(1989),
    [sym_push_device] = ACTIONS(1989),
    [sym_push_channel] = ACTIONS(1989),
    [sym_push_devchan] = ACTIONS(1989),
    [sym_release_device] = ACTIONS(1989),
    [sym_release_channel] = ACTIONS(1989),
    [sym_release_devchan] = ACTIONS(1989),
    [sym_master_slot] = ACTIONS(1989),
    [sym_get_pulse_time] = ACTIONS(1989),
    [sym_get_timer] = ACTIONS(1989),
    [sym_master_sn] = ACTIONS(1989),
    [sym_system_number] = ACTIONS(1989),
    [sym_button] = ACTIONS(1989),
    [sym_dv_channel] = ACTIONS(1989),
    [sym_data] = ACTIONS(1989),
    [sym_level] = ACTIONS(1989),
    [sym_timeline] = ACTIONS(1989),
    [sym_channel] = ACTIONS(1989),
    [sym_custom] = ACTIONS(1989),
    [sym_atoi] = ACTIONS(1989),
    [sym_atol] = ACTIONS(1989),
    [sym_atof] = ACTIONS(1989),
    [sym_itoa] = ACTIONS(1989),
    [sym_itohex] = ACTIONS(1989),
    [sym_hextoi] = ACTIONS(1989),
    [sym_ftoa] = ACTIONS(1989),
    [sym_find_string] = ACTIONS(1989),
    [sym_left_string] = ACTIONS(1989),
    [sym_right_string] = ACTIONS(1989),
    [sym_mid_string] = ACTIONS(1989),
    [sym_remove_string] = ACTIONS(1989),
    [sym_get_buffer_string] = ACTIONS(1989),
    [sym_get_buffer_char] = ACTIONS(1989),
    [sym_get_multi_buffer_string] = ACTIONS(1989),
    [sym_length_string] = ACTIONS(1989),
    [sym_max_length_string] = ACTIONS(1989),
    [sym_set_length_string] = ACTIONS(1989),
    [sym_compare_string] = ACTIONS(1989),
    [sym_upper_string] = ACTIONS(1989),
    [sym_lower_string] = ACTIONS(1989),
    [sym_get_last] = ACTIONS(1989),
    [sym_date_to_day] = ACTIONS(1989),
    [sym_date_to_month] = ACTIONS(1989),
    [sym_date_to_year] = ACTIONS(1989),
    [sym_time_to_hour] = ACTIONS(1989),
    [sym_time_to_minute] = ACTIONS(1989),
    [sym_time_to_second] = ACTIONS(1989),
    [sym_day_of_week] = ACTIONS(1989),
    [sym_set_pulse_time] = ACTIONS(1989),
    [sym_set_timer] = ACTIONS(1989),
    [sym_file_open] = ACTIONS(1989),
    [sym_file_close] = ACTIONS(1989),
    [sym_file_read] = ACTIONS(1989),
    [sym_file_read_line] = ACTIONS(1989),
    [sym_file_write] = ACTIONS(1989),
    [sym_file_write_line] = ACTIONS(1989),
    [sym_file_copy] = ACTIONS(1989),
    [sym_file_delete] = ACTIONS(1989),
    [sym_file_rename] = ACTIONS(1989),
    [sym_file_dir] = ACTIONS(1989),
    [sym_file_getdir] = ACTIONS(1989),
    [sym_file_setdir] = ACTIONS(1989),
    [sym_file_create_dir] = ACTIONS(1989),
    [sym_file_remove_dir] = ACTIONS(1989),
    [sym_file_seek] = ACTIONS(1989),
    [sym_length_array] = ACTIONS(1989),
    [sym_max_length_array] = ACTIONS(1989),
    [sym_set_length_array] = ACTIONS(1989),
    [sym_ip_client_open] = ACTIONS(1989),
    [sym_ip_client_close] = ACTIONS(1989),
    [sym_ip_server_open] = ACTIONS(1989),
    [sym_ip_server_close] = ACTIONS(1989),
    [sym_ip_bound_client_open] = ACTIONS(1989),
    [sym_ip_mc_server_open] = ACTIONS(1989),
    [sym_ip_set_option] = ACTIONS(1989),
    [sym_ssh_client_open] = ACTIONS(1989),
    [sym_ssh_client_close] = ACTIONS(1989),
    [sym_tls_client_open] = ACTIONS(1989),
    [sym_tls_client_close] = ACTIONS(1989),
    [sym_do_push] = ACTIONS(1989),
    [sym_do_push_timed] = ACTIONS(1989),
    [sym_do_release] = ACTIONS(1989),
    [sym_device_id] = ACTIONS(1989),
    [sym_device_info] = ACTIONS(1989),
    [sym_device_id_string] = ACTIONS(1989),
    [sym_device_standby] = ACTIONS(1989),
    [sym_device_wake] = ACTIONS(1989),
    [sym_timeline_create] = ACTIONS(1989),
    [sym_timeline_kill] = ACTIONS(1989),
    [sym_timeline_reload] = ACTIONS(1989),
    [sym_timeline_pause] = ACTIONS(1989),
    [sym_timeline_restart] = ACTIONS(1989),
    [sym_timeline_set] = ACTIONS(1989),
    [sym_timeline_get] = ACTIONS(1989),
    [sym_timeline_active] = ACTIONS(1989),
    [sym_timeline_dynamic_id] = ACTIONS(1989),
    [sym_variable_to_string] = ACTIONS(1989),
    [sym_string_to_variable] = ACTIONS(1989),
    [sym_length_variable_to_string] = ACTIONS(1989),
    [sym_type_cast] = ACTIONS(1989),
    [sym_variable_to_xml] = ACTIONS(1989),
    [sym_xml_to_variable] = ACTIONS(1989),
    [sym_length_variable_to_xml] = ACTIONS(1989),
    [sym_redirect_string] = ACTIONS(1989),
    [sym_random_number] = ACTIONS(1989),
    [sym_get_unique_id] = ACTIONS(1989),
    [sym_get_system_number] = ACTIONS(1989),
    [sym_set_system_number] = ACTIONS(1989),
    [sym_get_serial_number] = ACTIONS(1989),
    [sym_get_master_build] = ACTIONS(1989),
    [sym_set_validation_code] = ACTIONS(1989),
    [sym_reboot] = ACTIONS(1989),
    [sym_load_duet_module] = ACTIONS(1989),
    [sym_unload_duet_module] = ACTIONS(1989),
    [sym_duet_mem_size_set] = ACTIONS(1989),
    [sym_duet_mem_size_get] = ACTIONS(1989),
    [sym_dynamic_application_device] = ACTIONS(1989),
    [sym_dynamic_polled_port] = ACTIONS(1989),
    [sym_static_port_binding] = ACTIONS(1989),
    [sym_static_ip_binding] = ACTIONS(1989),
    [sym_get_dns_list] = ACTIONS(1989),
    [sym_set_dns_list] = ACTIONS(1989),
    [sym_get_ip_address] = ACTIONS(1989),
    [sym_set_ip_address] = ACTIONS(1989),
    [sym_get_url_list] = ACTIONS(1989),
    [sym_add_url_entry] = ACTIONS(1989),
    [sym_delete_url_entry] = ACTIONS(1989),
    [sym_abs_value] = ACTIONS(1989),
    [sym_max_value] = ACTIONS(1989),
    [sym_min_value] = ACTIONS(1989),
    [sym_exp_value] = ACTIONS(1989),
    [sym_log_value] = ACTIONS(1989),
    [sym_log10_value] = ACTIONS(1989),
    [sym_power_value] = ACTIONS(1989),
    [sym_sqrt_value] = ACTIONS(1989),
    [sym_raw_be] = ACTIONS(1989),
    [sym_raw_le] = ACTIONS(1989),
    [sym_internal_queue_size_set] = ACTIONS(1989),
    [sym_internal_queue_size_get] = ACTIONS(1989),
    [sym_internal_threshold_set] = ACTIONS(1989),
    [sym_internal_threshold_get] = ACTIONS(1989),
    [sym_rebuilt_event] = ACTIONS(1989),
    [sym_do_custom_event] = ACTIONS(1989),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1989),
    [sym_clkmgr_set_clk_source] = ACTIONS(1989),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1989),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1989),
    [sym_clkmgr_get_timezone] = ACTIONS(1989),
    [sym_clkmgr_set_timezone] = ACTIONS(1989),
    [sym_clkmgr_get_resync_period] = ACTIONS(1989),
    [sym_clkmgr_set_resync_period] = ACTIONS(1989),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1989),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1989),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1989),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1989),
    [sym_clkmgr_get_timeservers] = ACTIONS(1989),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1989),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1989),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1989),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1989),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1989),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1989),
    [sym_set_log_level] = ACTIONS(1989),
    [sym_get_log_level] = ACTIONS(1989),
    [sym_amx_log] = ACTIONS(1989),
    [sym_smtp_server_config_set] = ACTIONS(1989),
    [sym_smtp_server_config_get] = ACTIONS(1989),
    [sym_smtp_send] = ACTIONS(1989),
    [sym_validate_netlinx_account] = ACTIONS(1989),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1989),
    [sym_authenticate_certificate] = ACTIONS(1989),
    [sym_audit_netlinx_session_event] = ACTIONS(1989),
    [sym_audit_netlinx_generic_event] = ACTIONS(1989),
    [sym_get_available_flash_disk_space] = ACTIONS(1989),
    [sym_get_max_flash_disk_space] = ACTIONS(1989),
    [sym_data_create_feed] = ACTIONS(1989),
    [sym_data_delete_feed] = ACTIONS(1989),
    [sym_data_publish_feed] = ACTIONS(1989),
    [sym_data_get_published_feed] = ACTIONS(1989),
    [sym_data_add_record] = ACTIONS(1989),
    [sym_data_get_event_record] = ACTIONS(1989),
    [sym__wc_data_create_feed] = ACTIONS(1989),
    [sym__wc_data_add_record] = ACTIONS(1989),
    [sym__wc_data_get_event_record] = ACTIONS(1989),
    [sym_format] = ACTIONS(1989),
    [sym_astro_clock] = ACTIONS(1989),
    [sym_set_outdoor_temperature] = ACTIONS(1989),
    [sym_set_virtual_level_count] = ACTIONS(1989),
    [sym_set_virtual_channel_count] = ACTIONS(1989),
    [sym_set_virtual_port_count] = ACTIONS(1989),
    [sym__wc_encode_int] = ACTIONS(1989),
    [sym__wc_decode_int] = ACTIONS(1989),
    [sym__wc_int] = ACTIONS(1989),
    [sym__wc_to_ch_int] = ACTIONS(1989),
    [sym__ch_to_wc_int] = ACTIONS(1989),
    [sym__wc_find_string_int] = ACTIONS(1989),
    [sym__wc_left_string_int] = ACTIONS(1989),
    [sym__wc_lower_string_int] = ACTIONS(1989),
    [sym__wc_mid_string_int] = ACTIONS(1989),
    [sym__wc_remove_string_int] = ACTIONS(1989),
    [sym__wc_right_string_int] = ACTIONS(1989),
    [sym__wc_upper_string_int] = ACTIONS(1989),
    [sym__wc_compare_string_int] = ACTIONS(1989),
    [sym__wc_get_buffer_char_int] = ACTIONS(1989),
    [sym__wc_get_buffer_string_int] = ACTIONS(1989),
    [sym__wc_concat_string_int] = ACTIONS(1989),
    [sym__wc_file_open_int] = ACTIONS(1989),
    [sym__wc_file_close_int] = ACTIONS(1989),
    [sym__wc_file_read_int] = ACTIONS(1989),
    [sym__wc_file_read_line_int] = ACTIONS(1989),
    [sym__wc_file_write_int] = ACTIONS(1989),
    [sym__wc_file_write_line_int] = ACTIONS(1989),
    [aux_sym_preproc_include_token1] = ACTIONS(1989),
    [aux_sym_preproc_define_token1] = ACTIONS(1991),
    [aux_sym_preproc_warn_token1] = ACTIONS(1991),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1991),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1991),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1991),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1991),
    [aux_sym_preproc_else_token1] = ACTIONS(1991),
    [anon_sym_LBRACK] = ACTIONS(1991),
    [anon_sym_LBRACE] = ACTIONS(1991),
    [anon_sym_LPAREN] = ACTIONS(1989),
    [anon_sym_BANG] = ACTIONS(1991),
    [anon_sym_TILDE] = ACTIONS(1991),
    [anon_sym_DASH] = ACTIONS(1989),
    [anon_sym_PLUS] = ACTIONS(1989),
    [anon_sym_DASH_DASH] = ACTIONS(1991),
    [anon_sym_PLUS_PLUS] = ACTIONS(1991),
    [anon_sym_DQUOTE] = ACTIONS(1991),
    [anon_sym_SQUOTE] = ACTIONS(1991),
    [sym_number_literal] = ACTIONS(1991),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(311)] = {
    [ts_builtin_sym_end] = ACTIONS(1829),
    [sym_identifier] = ACTIONS(1827),
    [sym_program_name_keyword] = ACTIONS(1827),
    [sym_module_name_keyword] = ACTIONS(1827),
    [sym_define_device_keyword] = ACTIONS(1827),
    [sym_define_combine_keyword] = ACTIONS(1827),
    [sym_define_constant_keyword] = ACTIONS(1827),
    [sym_define_type_keyword] = ACTIONS(1827),
    [sym_define_variable_keyword] = ACTIONS(1827),
    [sym_define_system_variable_keyword] = ACTIONS(1827),
    [sym_define_connect_level_keyword] = ACTIONS(1827),
    [sym_define_latching_keyword] = ACTIONS(1827),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1827),
    [sym_define_toggling_keyword] = ACTIONS(1827),
    [sym_define_call_keyword] = ACTIONS(1827),
    [sym_define_function_keyword] = ACTIONS(1827),
    [sym_define_library_function_keyword] = ACTIONS(1827),
    [sym_define_start_keyword] = ACTIONS(1827),
    [sym_define_event_keyword] = ACTIONS(1827),
    [sym_define_program_keyword] = ACTIONS(1827),
    [sym_define_module_keyword] = ACTIONS(1827),
    [sym_char_keyword] = ACTIONS(1827),
    [sym_widechar_keyword] = ACTIONS(1827),
    [sym_integer_keyword] = ACTIONS(1827),
    [sym_sinteger_keyword] = ACTIONS(1827),
    [sym_long_keyword] = ACTIONS(1827),
    [sym_slong_keyword] = ACTIONS(1827),
    [sym_float_keyword] = ACTIONS(1827),
    [sym_double_keyword] = ACTIONS(1827),
    [sym_variant_keyword] = ACTIONS(1827),
    [sym_variantarray_keyword] = ACTIONS(1827),
    [sym_local_var_keyword] = ACTIONS(1827),
    [sym_stack_var_keyword] = ACTIONS(1827),
    [sym_constant_keyword] = ACTIONS(1827),
    [sym_volatile_keyword] = ACTIONS(1827),
    [sym_non_volatile_keyword] = ACTIONS(1827),
    [sym_persistent_keyword] = ACTIONS(1827),
    [sym_dev_keyword] = ACTIONS(1827),
    [sym_devlev_keyword] = ACTIONS(1827),
    [sym_devchan_keyword] = ACTIONS(1827),
    [sym_if_keyword] = ACTIONS(1827),
    [sym_else_keyword] = ACTIONS(1827),
    [sym_switch_keyword] = ACTIONS(1827),
    [sym_case_keyword] = ACTIONS(1827),
    [sym_default_keyword] = ACTIONS(1827),
    [sym_while_keyword] = ACTIONS(1827),
    [sym_for_keyword] = ACTIONS(1827),
    [sym_break_keyword] = ACTIONS(1827),
    [sym_continue_keyword] = ACTIONS(1827),
    [sym_return_keyword] = ACTIONS(1827),
    [sym_select_keyword] = ACTIONS(1827),
    [sym_active_keyword] = ACTIONS(1827),
    [sym_struct_keyword] = ACTIONS(1827),
    [sym_structure_keyword] = ACTIONS(1827),
    [sym_button_event_keyword] = ACTIONS(1827),
    [sym_channel_event_keyword] = ACTIONS(1827),
    [sym_level_event_keyword] = ACTIONS(1827),
    [sym_data_event_keyword] = ACTIONS(1827),
    [sym_timeline_event_keyword] = ACTIONS(1827),
    [sym_custom_event_keyword] = ACTIONS(1827),
    [sym_push_keyword] = ACTIONS(1827),
    [sym_release_keyword] = ACTIONS(1827),
    [aux_sym_on_keyword_token1] = ACTIONS(1827),
    [aux_sym_off_keyword_token1] = ACTIONS(1827),
    [sym_send_level_keyword] = ACTIONS(1827),
    [sym_send_string_keyword] = ACTIONS(1827),
    [sym_send_command_keyword] = ACTIONS(1827),
    [sym_clear_buffer_keyword] = ACTIONS(1827),
    [sym_create_buffer_keyword] = ACTIONS(1827),
    [sym_create_multi_buffer_keyword] = ACTIONS(1827),
    [sym_call_keyword] = ACTIONS(1827),
    [sym_system_call_keyword] = ACTIONS(1827),
    [sym_devchan_to_keyword] = ACTIONS(1827),
    [sym_devchan_min_to_keyword] = ACTIONS(1827),
    [sym_devchan_total_off_keyword] = ACTIONS(1827),
    [sym_devchan_pulse_keyword] = ACTIONS(1827),
    [sym_wait_keyword] = ACTIONS(1827),
    [sym_cancel_wait_keyword] = ACTIONS(1827),
    [sym_wait_until_keyword] = ACTIONS(1827),
    [sym_cancel_wait_until_keyword] = ACTIONS(1827),
    [sym_cancel_all_wait_keyword] = ACTIONS(1827),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1827),
    [sym_not] = ACTIONS(1827),
    [sym_bnot] = ACTIONS(1827),
    [sym___file__] = ACTIONS(1827),
    [sym___date__] = ACTIONS(1827),
    [sym___ldate__] = ACTIONS(1827),
    [sym___time__] = ACTIONS(1827),
    [sym___line__] = ACTIONS(1827),
    [sym___name__] = ACTIONS(1827),
    [sym___netlinx__] = ACTIONS(1827),
    [sym_netlinx_axi_version] = ACTIONS(1827),
    [sym_url_flg_tcp] = ACTIONS(1827),
    [sym_url_flg_accinfopresent] = ACTIONS(1827),
    [sym_url_flg_temp] = ACTIONS(1827),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1827),
    [sym_url_flg_stat_mask] = ACTIONS(1827),
    [sym_url_flg_stat_lookup] = ACTIONS(1827),
    [sym_url_flg_stat_connecting] = ACTIONS(1827),
    [sym_url_flg_stat_waiting] = ACTIONS(1827),
    [sym_url_flg_stat_connected] = ACTIONS(1827),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1827),
    [sym_first_virtual_device] = ACTIONS(1827),
    [sym_first_local_port] = ACTIONS(1827),
    [sym_dynamic_virtual_device] = ACTIONS(1827),
    [sym_timeline_once] = ACTIONS(1827),
    [sym_timeline_repeat] = ACTIONS(1827),
    [sym_timeline_absolute] = ACTIONS(1827),
    [sym_timeline_relative] = ACTIONS(1827),
    [sym_do_push_timed_infinite] = ACTIONS(1827),
    [sym_source_type_no_address] = ACTIONS(1827),
    [sym_source_type_neuron_id] = ACTIONS(1827),
    [sym_source_type_ip_address] = ACTIONS(1827),
    [sym_source_type_axlink] = ACTIONS(1827),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1827),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1827),
    [sym_source_type_ip_socket_address] = ACTIONS(1827),
    [sym_source_type_rs232] = ACTIONS(1827),
    [sym_source_type_internal] = ACTIONS(1827),
    [sym_source_type_mac_address] = ACTIONS(1827),
    [sym_source_type_ipv4_port] = ACTIONS(1827),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1827),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1827),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1827),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1827),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1827),
    [sym_true] = ACTIONS(1827),
    [sym_false] = ACTIONS(1827),
    [sym_file_read_only] = ACTIONS(1827),
    [sym_file_rw_new] = ACTIONS(1827),
    [sym_file_rw_append] = ACTIONS(1827),
    [sym_ip_tcp] = ACTIONS(1827),
    [sym_ip_udp] = ACTIONS(1827),
    [sym_ip_udp_2way] = ACTIONS(1827),
    [sym_xml_encode_types] = ACTIONS(1827),
    [sym_xml_encode_char_as_list] = ACTIONS(1827),
    [sym_xml_encode_le] = ACTIONS(1827),
    [sym_xml_decode_types] = ACTIONS(1827),
    [sym_xml_decode_no_preserve] = ACTIONS(1827),
    [sym_duet_dev_type_display_device] = ACTIONS(1827),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1827),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1827),
    [sym_duet_dev_type_io_device] = ACTIONS(1827),
    [sym_duet_dev_type_relay_device] = ACTIONS(1827),
    [sym_duet_dev_type_ups] = ACTIONS(1827),
    [sym_duet_dev_type_amplifier] = ACTIONS(1827),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1827),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1827),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1827),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1827),
    [sym_duet_dev_type_camera] = ACTIONS(1827),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1827),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1827),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1827),
    [sym_duet_dev_type_dss] = ACTIONS(1827),
    [sym_duet_dev_type_dvr] = ACTIONS(1827),
    [sym_duet_dev_type_disc_device] = ACTIONS(1827),
    [sym_duet_dev_type_document_camera] = ACTIONS(1827),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1827),
    [sym_duet_dev_type_hvac] = ACTIONS(1827),
    [sym_duet_dev_type_keypad] = ACTIONS(1827),
    [sym_duet_dev_type_light] = ACTIONS(1827),
    [sym_duet_dev_type_monitor] = ACTIONS(1827),
    [sym_duet_dev_type_motor] = ACTIONS(1827),
    [sym_duet_dev_type_multi_window] = ACTIONS(1827),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1827),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1827),
    [sym_duet_dev_type_receiver] = ACTIONS(1827),
    [sym_duet_dev_type_security_system] = ACTIONS(1827),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1827),
    [sym_duet_dev_type_settop_box] = ACTIONS(1827),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1827),
    [sym_duet_dev_type_switcher] = ACTIONS(1827),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1827),
    [sym_duet_dev_type_tv] = ACTIONS(1827),
    [sym_duet_dev_type_utility] = ACTIONS(1827),
    [sym_duet_dev_type_vcr] = ACTIONS(1827),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1827),
    [sym_duet_dev_type_video_processor] = ACTIONS(1827),
    [sym_duet_dev_type_video_projector] = ACTIONS(1827),
    [sym_duet_dev_type_video_wall] = ACTIONS(1827),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1827),
    [sym_duet_dev_type_weather] = ACTIONS(1827),
    [sym_duet_dev_not_polled] = ACTIONS(1827),
    [sym_duet_dev_polled] = ACTIONS(1827),
    [sym_clkmgr_mode_network] = ACTIONS(1827),
    [sym_clkmgr_mode_standalone] = ACTIONS(1827),
    [sym_normal_standby] = ACTIONS(1827),
    [sym_normal_wake] = ACTIONS(1827),
    [sym_ip_multicast_ttl_option] = ACTIONS(1827),
    [sym_ip_ttl_subnet] = ACTIONS(1827),
    [sym_ip_ttl_site] = ACTIONS(1827),
    [sym_ip_ttl_region] = ACTIONS(1827),
    [sym_ip_ttl_continent] = ACTIONS(1827),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1827),
    [sym_ip_nodelay_on] = ACTIONS(1827),
    [sym_ip_nodelay_off] = ACTIONS(1827),
    [sym_amx_error] = ACTIONS(1827),
    [sym_amx_warning] = ACTIONS(1827),
    [sym_amx_info] = ACTIONS(1827),
    [sym_amx_debug] = ACTIONS(1827),
    [sym_netlinx_login_success] = ACTIONS(1827),
    [sym_netlinx_login_fail] = ACTIONS(1827),
    [sym_netlinx_logout] = ACTIONS(1827),
    [sym_null_str] = ACTIONS(1827),
    [sym_smtp_address] = ACTIONS(1827),
    [sym_smtp_port_number] = ACTIONS(1827),
    [sym_smtp_username] = ACTIONS(1827),
    [sym_smtp_password] = ACTIONS(1827),
    [sym_smtp_require_tls] = ACTIONS(1827),
    [sym_smtp_from] = ACTIONS(1827),
    [sym_smtp_tls_true] = ACTIONS(1827),
    [sym_smtp_tls_false] = ACTIONS(1827),
    [sym_listview_on_row_select_event] = ACTIONS(1827),
    [sym_data_structure_unknown] = ACTIONS(1827),
    [sym_data_structure_datafeed] = ACTIONS(1827),
    [sym_data_structure_datarecordset] = ACTIONS(1827),
    [sym_data_structure_datarecord] = ACTIONS(1827),
    [sym_data_structure_datafield] = ACTIONS(1827),
    [sym_data_max_content_field_count] = ACTIONS(1827),
    [sym_data_max_metadata_field_count] = ACTIONS(1827),
    [sym_data_max_id_length] = ACTIONS(1827),
    [sym_data_max_name_length] = ACTIONS(1827),
    [sym_data_max_description_length] = ACTIONS(1827),
    [sym_data_max_type_length] = ACTIONS(1827),
    [sym_data_max_format_length] = ACTIONS(1827),
    [sym_data_max_source_length] = ACTIONS(1827),
    [sym_data_max_value_length] = ACTIONS(1827),
    [sym_data_max_label_length] = ACTIONS(1827),
    [sym_data_max_url_length] = ACTIONS(1827),
    [sym_max_last_login_info_length] = ACTIONS(1827),
    [sym_data_type_unknown] = ACTIONS(1827),
    [sym_data_type_string] = ACTIONS(1827),
    [sym_data_type_datetime] = ACTIONS(1827),
    [sym_data_type_date] = ACTIONS(1827),
    [sym_data_type_time] = ACTIONS(1827),
    [sym_data_type_image] = ACTIONS(1827),
    [sym_data_format_url] = ACTIONS(1827),
    [sym_data_format_phone] = ACTIONS(1827),
    [sym_data_format_email] = ACTIONS(1827),
    [sym_data_format_iso8601] = ACTIONS(1827),
    [sym_auditlog_priv] = ACTIONS(1827),
    [sym_configuration_priv] = ACTIONS(1827),
    [sym_deviceconfig_priv] = ACTIONS(1827),
    [sym_ftp_priv] = ACTIONS(1827),
    [sym_http_priv] = ACTIONS(1827),
    [sym_networkconfig_priv] = ACTIONS(1827),
    [sym_programport_priv] = ACTIONS(1827),
    [sym_remoteui_priv] = ACTIONS(1827),
    [sym_securitycontrol_priv] = ACTIONS(1827),
    [sym_softwaremanagement_priv] = ACTIONS(1827),
    [sym_terminal_priv] = ACTIONS(1827),
    [sym_tpadmin_priv] = ACTIONS(1827),
    [sym_user1_priv] = ACTIONS(1827),
    [sym_user2_priv] = ACTIONS(1827),
    [sym_user3_priv] = ACTIONS(1827),
    [sym_user4_priv] = ACTIONS(1827),
    [sym_usermanagement_priv] = ACTIONS(1827),
    [sym_valid_account] = ACTIONS(1827),
    [sym_err_name_invalid] = ACTIONS(1827),
    [sym_err_password_invalid] = ACTIONS(1827),
    [sym_err_invalid_account] = ACTIONS(1827),
    [sym_err_no_authorization] = ACTIONS(1827),
    [sym_err_invalid_parameter] = ACTIONS(1827),
    [sym_err_user_locked_out] = ACTIONS(1827),
    [sym_err_user_account_expired] = ACTIONS(1827),
    [sym_err_general_error] = ACTIONS(1827),
    [sym_err_authentication_error] = ACTIONS(1827),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1827),
    [sym_tls_validate_certificate] = ACTIONS(1827),
    [sym_tdata] = ACTIONS(1827),
    [sym_tchannel] = ACTIONS(1827),
    [sym_tlevel] = ACTIONS(1827),
    [sym_tbutton] = ACTIONS(1827),
    [sym_ttimeline] = ACTIONS(1827),
    [sym_tcustom] = ACTIONS(1827),
    [sym_url_struct] = ACTIONS(1827),
    [sym_dns_struct] = ACTIONS(1827),
    [sym_ip_address_struct] = ACTIONS(1827),
    [sym_dev_info_struct] = ACTIONS(1827),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1827),
    [sym_data_feed] = ACTIONS(1827),
    [sym_data_field] = ACTIONS(1827),
    [sym_data_record] = ACTIONS(1827),
    [sym_wc_data_feed] = ACTIONS(1827),
    [sym_wc_data_field] = ACTIONS(1827),
    [sym_wc_data_record] = ACTIONS(1827),
    [sym_last_login_info] = ACTIONS(1827),
    [sym____reserved___] = ACTIONS(1827),
    [sym_date] = ACTIONS(1827),
    [sym_day] = ACTIONS(1827),
    [sym_time] = ACTIONS(1827),
    [sym_ldate] = ACTIONS(1827),
    [sym_push_device] = ACTIONS(1827),
    [sym_push_channel] = ACTIONS(1827),
    [sym_push_devchan] = ACTIONS(1827),
    [sym_release_device] = ACTIONS(1827),
    [sym_release_channel] = ACTIONS(1827),
    [sym_release_devchan] = ACTIONS(1827),
    [sym_master_slot] = ACTIONS(1827),
    [sym_get_pulse_time] = ACTIONS(1827),
    [sym_get_timer] = ACTIONS(1827),
    [sym_master_sn] = ACTIONS(1827),
    [sym_system_number] = ACTIONS(1827),
    [sym_button] = ACTIONS(1827),
    [sym_dv_channel] = ACTIONS(1827),
    [sym_data] = ACTIONS(1827),
    [sym_level] = ACTIONS(1827),
    [sym_timeline] = ACTIONS(1827),
    [sym_channel] = ACTIONS(1827),
    [sym_custom] = ACTIONS(1827),
    [sym_atoi] = ACTIONS(1827),
    [sym_atol] = ACTIONS(1827),
    [sym_atof] = ACTIONS(1827),
    [sym_itoa] = ACTIONS(1827),
    [sym_itohex] = ACTIONS(1827),
    [sym_hextoi] = ACTIONS(1827),
    [sym_ftoa] = ACTIONS(1827),
    [sym_find_string] = ACTIONS(1827),
    [sym_left_string] = ACTIONS(1827),
    [sym_right_string] = ACTIONS(1827),
    [sym_mid_string] = ACTIONS(1827),
    [sym_remove_string] = ACTIONS(1827),
    [sym_get_buffer_string] = ACTIONS(1827),
    [sym_get_buffer_char] = ACTIONS(1827),
    [sym_get_multi_buffer_string] = ACTIONS(1827),
    [sym_length_string] = ACTIONS(1827),
    [sym_max_length_string] = ACTIONS(1827),
    [sym_set_length_string] = ACTIONS(1827),
    [sym_compare_string] = ACTIONS(1827),
    [sym_upper_string] = ACTIONS(1827),
    [sym_lower_string] = ACTIONS(1827),
    [sym_get_last] = ACTIONS(1827),
    [sym_date_to_day] = ACTIONS(1827),
    [sym_date_to_month] = ACTIONS(1827),
    [sym_date_to_year] = ACTIONS(1827),
    [sym_time_to_hour] = ACTIONS(1827),
    [sym_time_to_minute] = ACTIONS(1827),
    [sym_time_to_second] = ACTIONS(1827),
    [sym_day_of_week] = ACTIONS(1827),
    [sym_set_pulse_time] = ACTIONS(1827),
    [sym_set_timer] = ACTIONS(1827),
    [sym_file_open] = ACTIONS(1827),
    [sym_file_close] = ACTIONS(1827),
    [sym_file_read] = ACTIONS(1827),
    [sym_file_read_line] = ACTIONS(1827),
    [sym_file_write] = ACTIONS(1827),
    [sym_file_write_line] = ACTIONS(1827),
    [sym_file_copy] = ACTIONS(1827),
    [sym_file_delete] = ACTIONS(1827),
    [sym_file_rename] = ACTIONS(1827),
    [sym_file_dir] = ACTIONS(1827),
    [sym_file_getdir] = ACTIONS(1827),
    [sym_file_setdir] = ACTIONS(1827),
    [sym_file_create_dir] = ACTIONS(1827),
    [sym_file_remove_dir] = ACTIONS(1827),
    [sym_file_seek] = ACTIONS(1827),
    [sym_length_array] = ACTIONS(1827),
    [sym_max_length_array] = ACTIONS(1827),
    [sym_set_length_array] = ACTIONS(1827),
    [sym_ip_client_open] = ACTIONS(1827),
    [sym_ip_client_close] = ACTIONS(1827),
    [sym_ip_server_open] = ACTIONS(1827),
    [sym_ip_server_close] = ACTIONS(1827),
    [sym_ip_bound_client_open] = ACTIONS(1827),
    [sym_ip_mc_server_open] = ACTIONS(1827),
    [sym_ip_set_option] = ACTIONS(1827),
    [sym_ssh_client_open] = ACTIONS(1827),
    [sym_ssh_client_close] = ACTIONS(1827),
    [sym_tls_client_open] = ACTIONS(1827),
    [sym_tls_client_close] = ACTIONS(1827),
    [sym_do_push] = ACTIONS(1827),
    [sym_do_push_timed] = ACTIONS(1827),
    [sym_do_release] = ACTIONS(1827),
    [sym_device_id] = ACTIONS(1827),
    [sym_device_info] = ACTIONS(1827),
    [sym_device_id_string] = ACTIONS(1827),
    [sym_device_standby] = ACTIONS(1827),
    [sym_device_wake] = ACTIONS(1827),
    [sym_timeline_create] = ACTIONS(1827),
    [sym_timeline_kill] = ACTIONS(1827),
    [sym_timeline_reload] = ACTIONS(1827),
    [sym_timeline_pause] = ACTIONS(1827),
    [sym_timeline_restart] = ACTIONS(1827),
    [sym_timeline_set] = ACTIONS(1827),
    [sym_timeline_get] = ACTIONS(1827),
    [sym_timeline_active] = ACTIONS(1827),
    [sym_timeline_dynamic_id] = ACTIONS(1827),
    [sym_variable_to_string] = ACTIONS(1827),
    [sym_string_to_variable] = ACTIONS(1827),
    [sym_length_variable_to_string] = ACTIONS(1827),
    [sym_type_cast] = ACTIONS(1827),
    [sym_variable_to_xml] = ACTIONS(1827),
    [sym_xml_to_variable] = ACTIONS(1827),
    [sym_length_variable_to_xml] = ACTIONS(1827),
    [sym_redirect_string] = ACTIONS(1827),
    [sym_random_number] = ACTIONS(1827),
    [sym_get_unique_id] = ACTIONS(1827),
    [sym_get_system_number] = ACTIONS(1827),
    [sym_set_system_number] = ACTIONS(1827),
    [sym_get_serial_number] = ACTIONS(1827),
    [sym_get_master_build] = ACTIONS(1827),
    [sym_set_validation_code] = ACTIONS(1827),
    [sym_reboot] = ACTIONS(1827),
    [sym_load_duet_module] = ACTIONS(1827),
    [sym_unload_duet_module] = ACTIONS(1827),
    [sym_duet_mem_size_set] = ACTIONS(1827),
    [sym_duet_mem_size_get] = ACTIONS(1827),
    [sym_dynamic_application_device] = ACTIONS(1827),
    [sym_dynamic_polled_port] = ACTIONS(1827),
    [sym_static_port_binding] = ACTIONS(1827),
    [sym_static_ip_binding] = ACTIONS(1827),
    [sym_get_dns_list] = ACTIONS(1827),
    [sym_set_dns_list] = ACTIONS(1827),
    [sym_get_ip_address] = ACTIONS(1827),
    [sym_set_ip_address] = ACTIONS(1827),
    [sym_get_url_list] = ACTIONS(1827),
    [sym_add_url_entry] = ACTIONS(1827),
    [sym_delete_url_entry] = ACTIONS(1827),
    [sym_abs_value] = ACTIONS(1827),
    [sym_max_value] = ACTIONS(1827),
    [sym_min_value] = ACTIONS(1827),
    [sym_exp_value] = ACTIONS(1827),
    [sym_log_value] = ACTIONS(1827),
    [sym_log10_value] = ACTIONS(1827),
    [sym_power_value] = ACTIONS(1827),
    [sym_sqrt_value] = ACTIONS(1827),
    [sym_raw_be] = ACTIONS(1827),
    [sym_raw_le] = ACTIONS(1827),
    [sym_internal_queue_size_set] = ACTIONS(1827),
    [sym_internal_queue_size_get] = ACTIONS(1827),
    [sym_internal_threshold_set] = ACTIONS(1827),
    [sym_internal_threshold_get] = ACTIONS(1827),
    [sym_rebuilt_event] = ACTIONS(1827),
    [sym_do_custom_event] = ACTIONS(1827),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1827),
    [sym_clkmgr_set_clk_source] = ACTIONS(1827),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1827),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1827),
    [sym_clkmgr_get_timezone] = ACTIONS(1827),
    [sym_clkmgr_set_timezone] = ACTIONS(1827),
    [sym_clkmgr_get_resync_period] = ACTIONS(1827),
    [sym_clkmgr_set_resync_period] = ACTIONS(1827),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1827),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1827),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1827),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1827),
    [sym_clkmgr_get_timeservers] = ACTIONS(1827),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1827),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1827),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1827),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1827),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1827),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1827),
    [sym_set_log_level] = ACTIONS(1827),
    [sym_get_log_level] = ACTIONS(1827),
    [sym_amx_log] = ACTIONS(1827),
    [sym_smtp_server_config_set] = ACTIONS(1827),
    [sym_smtp_server_config_get] = ACTIONS(1827),
    [sym_smtp_send] = ACTIONS(1827),
    [sym_validate_netlinx_account] = ACTIONS(1827),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1827),
    [sym_authenticate_certificate] = ACTIONS(1827),
    [sym_audit_netlinx_session_event] = ACTIONS(1827),
    [sym_audit_netlinx_generic_event] = ACTIONS(1827),
    [sym_get_available_flash_disk_space] = ACTIONS(1827),
    [sym_get_max_flash_disk_space] = ACTIONS(1827),
    [sym_data_create_feed] = ACTIONS(1827),
    [sym_data_delete_feed] = ACTIONS(1827),
    [sym_data_publish_feed] = ACTIONS(1827),
    [sym_data_get_published_feed] = ACTIONS(1827),
    [sym_data_add_record] = ACTIONS(1827),
    [sym_data_get_event_record] = ACTIONS(1827),
    [sym__wc_data_create_feed] = ACTIONS(1827),
    [sym__wc_data_add_record] = ACTIONS(1827),
    [sym__wc_data_get_event_record] = ACTIONS(1827),
    [sym_format] = ACTIONS(1827),
    [sym_astro_clock] = ACTIONS(1827),
    [sym_set_outdoor_temperature] = ACTIONS(1827),
    [sym_set_virtual_level_count] = ACTIONS(1827),
    [sym_set_virtual_channel_count] = ACTIONS(1827),
    [sym_set_virtual_port_count] = ACTIONS(1827),
    [sym__wc_encode_int] = ACTIONS(1827),
    [sym__wc_decode_int] = ACTIONS(1827),
    [sym__wc_int] = ACTIONS(1827),
    [sym__wc_to_ch_int] = ACTIONS(1827),
    [sym__ch_to_wc_int] = ACTIONS(1827),
    [sym__wc_find_string_int] = ACTIONS(1827),
    [sym__wc_left_string_int] = ACTIONS(1827),
    [sym__wc_lower_string_int] = ACTIONS(1827),
    [sym__wc_mid_string_int] = ACTIONS(1827),
    [sym__wc_remove_string_int] = ACTIONS(1827),
    [sym__wc_right_string_int] = ACTIONS(1827),
    [sym__wc_upper_string_int] = ACTIONS(1827),
    [sym__wc_compare_string_int] = ACTIONS(1827),
    [sym__wc_get_buffer_char_int] = ACTIONS(1827),
    [sym__wc_get_buffer_string_int] = ACTIONS(1827),
    [sym__wc_concat_string_int] = ACTIONS(1827),
    [sym__wc_file_open_int] = ACTIONS(1827),
    [sym__wc_file_close_int] = ACTIONS(1827),
    [sym__wc_file_read_int] = ACTIONS(1827),
    [sym__wc_file_read_line_int] = ACTIONS(1827),
    [sym__wc_file_write_int] = ACTIONS(1827),
    [sym__wc_file_write_line_int] = ACTIONS(1827),
    [aux_sym_preproc_include_token1] = ACTIONS(1827),
    [aux_sym_preproc_define_token1] = ACTIONS(1829),
    [aux_sym_preproc_warn_token1] = ACTIONS(1829),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1829),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1829),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1829),
    [anon_sym_LBRACK] = ACTIONS(1829),
    [anon_sym_LBRACE] = ACTIONS(1829),
    [anon_sym_LPAREN] = ACTIONS(1827),
    [anon_sym_BANG] = ACTIONS(1829),
    [anon_sym_TILDE] = ACTIONS(1829),
    [anon_sym_DASH] = ACTIONS(1827),
    [anon_sym_PLUS] = ACTIONS(1827),
    [anon_sym_DASH_DASH] = ACTIONS(1829),
    [anon_sym_PLUS_PLUS] = ACTIONS(1829),
    [anon_sym_DQUOTE] = ACTIONS(1829),
    [anon_sym_SQUOTE] = ACTIONS(1829),
    [sym_number_literal] = ACTIONS(1829),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(312)] = {
    [sym_identifier] = ACTIONS(1993),
    [sym_program_name_keyword] = ACTIONS(1993),
    [sym_module_name_keyword] = ACTIONS(1993),
    [sym_define_device_keyword] = ACTIONS(1993),
    [sym_define_combine_keyword] = ACTIONS(1993),
    [sym_define_constant_keyword] = ACTIONS(1993),
    [sym_define_type_keyword] = ACTIONS(1993),
    [sym_define_variable_keyword] = ACTIONS(1993),
    [sym_define_system_variable_keyword] = ACTIONS(1993),
    [sym_define_connect_level_keyword] = ACTIONS(1993),
    [sym_define_latching_keyword] = ACTIONS(1993),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1993),
    [sym_define_toggling_keyword] = ACTIONS(1993),
    [sym_define_call_keyword] = ACTIONS(1993),
    [sym_define_function_keyword] = ACTIONS(1993),
    [sym_define_library_function_keyword] = ACTIONS(1993),
    [sym_define_start_keyword] = ACTIONS(1993),
    [sym_define_event_keyword] = ACTIONS(1993),
    [sym_define_program_keyword] = ACTIONS(1993),
    [sym_define_module_keyword] = ACTIONS(1993),
    [sym_char_keyword] = ACTIONS(1993),
    [sym_widechar_keyword] = ACTIONS(1993),
    [sym_integer_keyword] = ACTIONS(1993),
    [sym_sinteger_keyword] = ACTIONS(1993),
    [sym_long_keyword] = ACTIONS(1993),
    [sym_slong_keyword] = ACTIONS(1993),
    [sym_float_keyword] = ACTIONS(1993),
    [sym_double_keyword] = ACTIONS(1993),
    [sym_variant_keyword] = ACTIONS(1993),
    [sym_variantarray_keyword] = ACTIONS(1993),
    [sym_local_var_keyword] = ACTIONS(1993),
    [sym_stack_var_keyword] = ACTIONS(1993),
    [sym_constant_keyword] = ACTIONS(1993),
    [sym_volatile_keyword] = ACTIONS(1993),
    [sym_non_volatile_keyword] = ACTIONS(1993),
    [sym_persistent_keyword] = ACTIONS(1993),
    [sym_dev_keyword] = ACTIONS(1993),
    [sym_devlev_keyword] = ACTIONS(1993),
    [sym_devchan_keyword] = ACTIONS(1993),
    [sym_if_keyword] = ACTIONS(1993),
    [sym_switch_keyword] = ACTIONS(1993),
    [sym_case_keyword] = ACTIONS(1993),
    [sym_default_keyword] = ACTIONS(1993),
    [sym_while_keyword] = ACTIONS(1993),
    [sym_for_keyword] = ACTIONS(1993),
    [sym_break_keyword] = ACTIONS(1993),
    [sym_continue_keyword] = ACTIONS(1993),
    [sym_return_keyword] = ACTIONS(1993),
    [sym_select_keyword] = ACTIONS(1993),
    [sym_active_keyword] = ACTIONS(1993),
    [sym_struct_keyword] = ACTIONS(1993),
    [sym_structure_keyword] = ACTIONS(1993),
    [sym_button_event_keyword] = ACTIONS(1993),
    [sym_channel_event_keyword] = ACTIONS(1993),
    [sym_level_event_keyword] = ACTIONS(1993),
    [sym_data_event_keyword] = ACTIONS(1993),
    [sym_timeline_event_keyword] = ACTIONS(1993),
    [sym_custom_event_keyword] = ACTIONS(1993),
    [sym_push_keyword] = ACTIONS(1993),
    [sym_release_keyword] = ACTIONS(1993),
    [aux_sym_on_keyword_token1] = ACTIONS(1993),
    [aux_sym_off_keyword_token1] = ACTIONS(1993),
    [sym_send_level_keyword] = ACTIONS(1993),
    [sym_send_string_keyword] = ACTIONS(1993),
    [sym_send_command_keyword] = ACTIONS(1993),
    [sym_clear_buffer_keyword] = ACTIONS(1993),
    [sym_create_buffer_keyword] = ACTIONS(1993),
    [sym_create_multi_buffer_keyword] = ACTIONS(1993),
    [sym_call_keyword] = ACTIONS(1993),
    [sym_system_call_keyword] = ACTIONS(1993),
    [sym_devchan_to_keyword] = ACTIONS(1993),
    [sym_devchan_min_to_keyword] = ACTIONS(1993),
    [sym_devchan_total_off_keyword] = ACTIONS(1993),
    [sym_devchan_pulse_keyword] = ACTIONS(1993),
    [sym_wait_keyword] = ACTIONS(1993),
    [sym_cancel_wait_keyword] = ACTIONS(1993),
    [sym_wait_until_keyword] = ACTIONS(1993),
    [sym_cancel_wait_until_keyword] = ACTIONS(1993),
    [sym_cancel_all_wait_keyword] = ACTIONS(1993),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1993),
    [sym_not] = ACTIONS(1993),
    [sym_bnot] = ACTIONS(1993),
    [sym___file__] = ACTIONS(1993),
    [sym___date__] = ACTIONS(1993),
    [sym___ldate__] = ACTIONS(1993),
    [sym___time__] = ACTIONS(1993),
    [sym___line__] = ACTIONS(1993),
    [sym___name__] = ACTIONS(1993),
    [sym___netlinx__] = ACTIONS(1993),
    [sym_netlinx_axi_version] = ACTIONS(1993),
    [sym_url_flg_tcp] = ACTIONS(1993),
    [sym_url_flg_accinfopresent] = ACTIONS(1993),
    [sym_url_flg_temp] = ACTIONS(1993),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1993),
    [sym_url_flg_stat_mask] = ACTIONS(1993),
    [sym_url_flg_stat_lookup] = ACTIONS(1993),
    [sym_url_flg_stat_connecting] = ACTIONS(1993),
    [sym_url_flg_stat_waiting] = ACTIONS(1993),
    [sym_url_flg_stat_connected] = ACTIONS(1993),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1993),
    [sym_first_virtual_device] = ACTIONS(1993),
    [sym_first_local_port] = ACTIONS(1993),
    [sym_dynamic_virtual_device] = ACTIONS(1993),
    [sym_timeline_once] = ACTIONS(1993),
    [sym_timeline_repeat] = ACTIONS(1993),
    [sym_timeline_absolute] = ACTIONS(1993),
    [sym_timeline_relative] = ACTIONS(1993),
    [sym_do_push_timed_infinite] = ACTIONS(1993),
    [sym_source_type_no_address] = ACTIONS(1993),
    [sym_source_type_neuron_id] = ACTIONS(1993),
    [sym_source_type_ip_address] = ACTIONS(1993),
    [sym_source_type_axlink] = ACTIONS(1993),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1993),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1993),
    [sym_source_type_ip_socket_address] = ACTIONS(1993),
    [sym_source_type_rs232] = ACTIONS(1993),
    [sym_source_type_internal] = ACTIONS(1993),
    [sym_source_type_mac_address] = ACTIONS(1993),
    [sym_source_type_ipv4_port] = ACTIONS(1993),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1993),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1993),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1993),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1993),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1993),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1993),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1993),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1993),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1993),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1993),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1993),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1993),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1993),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1993),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1993),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1993),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1993),
    [sym_true] = ACTIONS(1993),
    [sym_false] = ACTIONS(1993),
    [sym_file_read_only] = ACTIONS(1993),
    [sym_file_rw_new] = ACTIONS(1993),
    [sym_file_rw_append] = ACTIONS(1993),
    [sym_ip_tcp] = ACTIONS(1993),
    [sym_ip_udp] = ACTIONS(1993),
    [sym_ip_udp_2way] = ACTIONS(1993),
    [sym_xml_encode_types] = ACTIONS(1993),
    [sym_xml_encode_char_as_list] = ACTIONS(1993),
    [sym_xml_encode_le] = ACTIONS(1993),
    [sym_xml_decode_types] = ACTIONS(1993),
    [sym_xml_decode_no_preserve] = ACTIONS(1993),
    [sym_duet_dev_type_display_device] = ACTIONS(1993),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1993),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1993),
    [sym_duet_dev_type_io_device] = ACTIONS(1993),
    [sym_duet_dev_type_relay_device] = ACTIONS(1993),
    [sym_duet_dev_type_ups] = ACTIONS(1993),
    [sym_duet_dev_type_amplifier] = ACTIONS(1993),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1993),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1993),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1993),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1993),
    [sym_duet_dev_type_camera] = ACTIONS(1993),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1993),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1993),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1993),
    [sym_duet_dev_type_dss] = ACTIONS(1993),
    [sym_duet_dev_type_dvr] = ACTIONS(1993),
    [sym_duet_dev_type_disc_device] = ACTIONS(1993),
    [sym_duet_dev_type_document_camera] = ACTIONS(1993),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1993),
    [sym_duet_dev_type_hvac] = ACTIONS(1993),
    [sym_duet_dev_type_keypad] = ACTIONS(1993),
    [sym_duet_dev_type_light] = ACTIONS(1993),
    [sym_duet_dev_type_monitor] = ACTIONS(1993),
    [sym_duet_dev_type_motor] = ACTIONS(1993),
    [sym_duet_dev_type_multi_window] = ACTIONS(1993),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1993),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1993),
    [sym_duet_dev_type_receiver] = ACTIONS(1993),
    [sym_duet_dev_type_security_system] = ACTIONS(1993),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1993),
    [sym_duet_dev_type_settop_box] = ACTIONS(1993),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1993),
    [sym_duet_dev_type_switcher] = ACTIONS(1993),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1993),
    [sym_duet_dev_type_tv] = ACTIONS(1993),
    [sym_duet_dev_type_utility] = ACTIONS(1993),
    [sym_duet_dev_type_vcr] = ACTIONS(1993),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1993),
    [sym_duet_dev_type_video_processor] = ACTIONS(1993),
    [sym_duet_dev_type_video_projector] = ACTIONS(1993),
    [sym_duet_dev_type_video_wall] = ACTIONS(1993),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1993),
    [sym_duet_dev_type_weather] = ACTIONS(1993),
    [sym_duet_dev_not_polled] = ACTIONS(1993),
    [sym_duet_dev_polled] = ACTIONS(1993),
    [sym_clkmgr_mode_network] = ACTIONS(1993),
    [sym_clkmgr_mode_standalone] = ACTIONS(1993),
    [sym_normal_standby] = ACTIONS(1993),
    [sym_normal_wake] = ACTIONS(1993),
    [sym_ip_multicast_ttl_option] = ACTIONS(1993),
    [sym_ip_ttl_subnet] = ACTIONS(1993),
    [sym_ip_ttl_site] = ACTIONS(1993),
    [sym_ip_ttl_region] = ACTIONS(1993),
    [sym_ip_ttl_continent] = ACTIONS(1993),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1993),
    [sym_ip_nodelay_on] = ACTIONS(1993),
    [sym_ip_nodelay_off] = ACTIONS(1993),
    [sym_amx_error] = ACTIONS(1993),
    [sym_amx_warning] = ACTIONS(1993),
    [sym_amx_info] = ACTIONS(1993),
    [sym_amx_debug] = ACTIONS(1993),
    [sym_netlinx_login_success] = ACTIONS(1993),
    [sym_netlinx_login_fail] = ACTIONS(1993),
    [sym_netlinx_logout] = ACTIONS(1993),
    [sym_null_str] = ACTIONS(1993),
    [sym_smtp_address] = ACTIONS(1993),
    [sym_smtp_port_number] = ACTIONS(1993),
    [sym_smtp_username] = ACTIONS(1993),
    [sym_smtp_password] = ACTIONS(1993),
    [sym_smtp_require_tls] = ACTIONS(1993),
    [sym_smtp_from] = ACTIONS(1993),
    [sym_smtp_tls_true] = ACTIONS(1993),
    [sym_smtp_tls_false] = ACTIONS(1993),
    [sym_listview_on_row_select_event] = ACTIONS(1993),
    [sym_data_structure_unknown] = ACTIONS(1993),
    [sym_data_structure_datafeed] = ACTIONS(1993),
    [sym_data_structure_datarecordset] = ACTIONS(1993),
    [sym_data_structure_datarecord] = ACTIONS(1993),
    [sym_data_structure_datafield] = ACTIONS(1993),
    [sym_data_max_content_field_count] = ACTIONS(1993),
    [sym_data_max_metadata_field_count] = ACTIONS(1993),
    [sym_data_max_id_length] = ACTIONS(1993),
    [sym_data_max_name_length] = ACTIONS(1993),
    [sym_data_max_description_length] = ACTIONS(1993),
    [sym_data_max_type_length] = ACTIONS(1993),
    [sym_data_max_format_length] = ACTIONS(1993),
    [sym_data_max_source_length] = ACTIONS(1993),
    [sym_data_max_value_length] = ACTIONS(1993),
    [sym_data_max_label_length] = ACTIONS(1993),
    [sym_data_max_url_length] = ACTIONS(1993),
    [sym_max_last_login_info_length] = ACTIONS(1993),
    [sym_data_type_unknown] = ACTIONS(1993),
    [sym_data_type_string] = ACTIONS(1993),
    [sym_data_type_datetime] = ACTIONS(1993),
    [sym_data_type_date] = ACTIONS(1993),
    [sym_data_type_time] = ACTIONS(1993),
    [sym_data_type_image] = ACTIONS(1993),
    [sym_data_format_url] = ACTIONS(1993),
    [sym_data_format_phone] = ACTIONS(1993),
    [sym_data_format_email] = ACTIONS(1993),
    [sym_data_format_iso8601] = ACTIONS(1993),
    [sym_auditlog_priv] = ACTIONS(1993),
    [sym_configuration_priv] = ACTIONS(1993),
    [sym_deviceconfig_priv] = ACTIONS(1993),
    [sym_ftp_priv] = ACTIONS(1993),
    [sym_http_priv] = ACTIONS(1993),
    [sym_networkconfig_priv] = ACTIONS(1993),
    [sym_programport_priv] = ACTIONS(1993),
    [sym_remoteui_priv] = ACTIONS(1993),
    [sym_securitycontrol_priv] = ACTIONS(1993),
    [sym_softwaremanagement_priv] = ACTIONS(1993),
    [sym_terminal_priv] = ACTIONS(1993),
    [sym_tpadmin_priv] = ACTIONS(1993),
    [sym_user1_priv] = ACTIONS(1993),
    [sym_user2_priv] = ACTIONS(1993),
    [sym_user3_priv] = ACTIONS(1993),
    [sym_user4_priv] = ACTIONS(1993),
    [sym_usermanagement_priv] = ACTIONS(1993),
    [sym_valid_account] = ACTIONS(1993),
    [sym_err_name_invalid] = ACTIONS(1993),
    [sym_err_password_invalid] = ACTIONS(1993),
    [sym_err_invalid_account] = ACTIONS(1993),
    [sym_err_no_authorization] = ACTIONS(1993),
    [sym_err_invalid_parameter] = ACTIONS(1993),
    [sym_err_user_locked_out] = ACTIONS(1993),
    [sym_err_user_account_expired] = ACTIONS(1993),
    [sym_err_general_error] = ACTIONS(1993),
    [sym_err_authentication_error] = ACTIONS(1993),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1993),
    [sym_tls_validate_certificate] = ACTIONS(1993),
    [sym_tdata] = ACTIONS(1993),
    [sym_tchannel] = ACTIONS(1993),
    [sym_tlevel] = ACTIONS(1993),
    [sym_tbutton] = ACTIONS(1993),
    [sym_ttimeline] = ACTIONS(1993),
    [sym_tcustom] = ACTIONS(1993),
    [sym_url_struct] = ACTIONS(1993),
    [sym_dns_struct] = ACTIONS(1993),
    [sym_ip_address_struct] = ACTIONS(1993),
    [sym_dev_info_struct] = ACTIONS(1993),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1993),
    [sym_data_feed] = ACTIONS(1993),
    [sym_data_field] = ACTIONS(1993),
    [sym_data_record] = ACTIONS(1993),
    [sym_wc_data_feed] = ACTIONS(1993),
    [sym_wc_data_field] = ACTIONS(1993),
    [sym_wc_data_record] = ACTIONS(1993),
    [sym_last_login_info] = ACTIONS(1993),
    [sym____reserved___] = ACTIONS(1993),
    [sym_date] = ACTIONS(1993),
    [sym_day] = ACTIONS(1993),
    [sym_time] = ACTIONS(1993),
    [sym_ldate] = ACTIONS(1993),
    [sym_push_device] = ACTIONS(1993),
    [sym_push_channel] = ACTIONS(1993),
    [sym_push_devchan] = ACTIONS(1993),
    [sym_release_device] = ACTIONS(1993),
    [sym_release_channel] = ACTIONS(1993),
    [sym_release_devchan] = ACTIONS(1993),
    [sym_master_slot] = ACTIONS(1993),
    [sym_get_pulse_time] = ACTIONS(1993),
    [sym_get_timer] = ACTIONS(1993),
    [sym_master_sn] = ACTIONS(1993),
    [sym_system_number] = ACTIONS(1993),
    [sym_button] = ACTIONS(1993),
    [sym_dv_channel] = ACTIONS(1993),
    [sym_data] = ACTIONS(1993),
    [sym_level] = ACTIONS(1993),
    [sym_timeline] = ACTIONS(1993),
    [sym_channel] = ACTIONS(1993),
    [sym_custom] = ACTIONS(1993),
    [sym_atoi] = ACTIONS(1993),
    [sym_atol] = ACTIONS(1993),
    [sym_atof] = ACTIONS(1993),
    [sym_itoa] = ACTIONS(1993),
    [sym_itohex] = ACTIONS(1993),
    [sym_hextoi] = ACTIONS(1993),
    [sym_ftoa] = ACTIONS(1993),
    [sym_find_string] = ACTIONS(1993),
    [sym_left_string] = ACTIONS(1993),
    [sym_right_string] = ACTIONS(1993),
    [sym_mid_string] = ACTIONS(1993),
    [sym_remove_string] = ACTIONS(1993),
    [sym_get_buffer_string] = ACTIONS(1993),
    [sym_get_buffer_char] = ACTIONS(1993),
    [sym_get_multi_buffer_string] = ACTIONS(1993),
    [sym_length_string] = ACTIONS(1993),
    [sym_max_length_string] = ACTIONS(1993),
    [sym_set_length_string] = ACTIONS(1993),
    [sym_compare_string] = ACTIONS(1993),
    [sym_upper_string] = ACTIONS(1993),
    [sym_lower_string] = ACTIONS(1993),
    [sym_get_last] = ACTIONS(1993),
    [sym_date_to_day] = ACTIONS(1993),
    [sym_date_to_month] = ACTIONS(1993),
    [sym_date_to_year] = ACTIONS(1993),
    [sym_time_to_hour] = ACTIONS(1993),
    [sym_time_to_minute] = ACTIONS(1993),
    [sym_time_to_second] = ACTIONS(1993),
    [sym_day_of_week] = ACTIONS(1993),
    [sym_set_pulse_time] = ACTIONS(1993),
    [sym_set_timer] = ACTIONS(1993),
    [sym_file_open] = ACTIONS(1993),
    [sym_file_close] = ACTIONS(1993),
    [sym_file_read] = ACTIONS(1993),
    [sym_file_read_line] = ACTIONS(1993),
    [sym_file_write] = ACTIONS(1993),
    [sym_file_write_line] = ACTIONS(1993),
    [sym_file_copy] = ACTIONS(1993),
    [sym_file_delete] = ACTIONS(1993),
    [sym_file_rename] = ACTIONS(1993),
    [sym_file_dir] = ACTIONS(1993),
    [sym_file_getdir] = ACTIONS(1993),
    [sym_file_setdir] = ACTIONS(1993),
    [sym_file_create_dir] = ACTIONS(1993),
    [sym_file_remove_dir] = ACTIONS(1993),
    [sym_file_seek] = ACTIONS(1993),
    [sym_length_array] = ACTIONS(1993),
    [sym_max_length_array] = ACTIONS(1993),
    [sym_set_length_array] = ACTIONS(1993),
    [sym_ip_client_open] = ACTIONS(1993),
    [sym_ip_client_close] = ACTIONS(1993),
    [sym_ip_server_open] = ACTIONS(1993),
    [sym_ip_server_close] = ACTIONS(1993),
    [sym_ip_bound_client_open] = ACTIONS(1993),
    [sym_ip_mc_server_open] = ACTIONS(1993),
    [sym_ip_set_option] = ACTIONS(1993),
    [sym_ssh_client_open] = ACTIONS(1993),
    [sym_ssh_client_close] = ACTIONS(1993),
    [sym_tls_client_open] = ACTIONS(1993),
    [sym_tls_client_close] = ACTIONS(1993),
    [sym_do_push] = ACTIONS(1993),
    [sym_do_push_timed] = ACTIONS(1993),
    [sym_do_release] = ACTIONS(1993),
    [sym_device_id] = ACTIONS(1993),
    [sym_device_info] = ACTIONS(1993),
    [sym_device_id_string] = ACTIONS(1993),
    [sym_device_standby] = ACTIONS(1993),
    [sym_device_wake] = ACTIONS(1993),
    [sym_timeline_create] = ACTIONS(1993),
    [sym_timeline_kill] = ACTIONS(1993),
    [sym_timeline_reload] = ACTIONS(1993),
    [sym_timeline_pause] = ACTIONS(1993),
    [sym_timeline_restart] = ACTIONS(1993),
    [sym_timeline_set] = ACTIONS(1993),
    [sym_timeline_get] = ACTIONS(1993),
    [sym_timeline_active] = ACTIONS(1993),
    [sym_timeline_dynamic_id] = ACTIONS(1993),
    [sym_variable_to_string] = ACTIONS(1993),
    [sym_string_to_variable] = ACTIONS(1993),
    [sym_length_variable_to_string] = ACTIONS(1993),
    [sym_type_cast] = ACTIONS(1993),
    [sym_variable_to_xml] = ACTIONS(1993),
    [sym_xml_to_variable] = ACTIONS(1993),
    [sym_length_variable_to_xml] = ACTIONS(1993),
    [sym_redirect_string] = ACTIONS(1993),
    [sym_random_number] = ACTIONS(1993),
    [sym_get_unique_id] = ACTIONS(1993),
    [sym_get_system_number] = ACTIONS(1993),
    [sym_set_system_number] = ACTIONS(1993),
    [sym_get_serial_number] = ACTIONS(1993),
    [sym_get_master_build] = ACTIONS(1993),
    [sym_set_validation_code] = ACTIONS(1993),
    [sym_reboot] = ACTIONS(1993),
    [sym_load_duet_module] = ACTIONS(1993),
    [sym_unload_duet_module] = ACTIONS(1993),
    [sym_duet_mem_size_set] = ACTIONS(1993),
    [sym_duet_mem_size_get] = ACTIONS(1993),
    [sym_dynamic_application_device] = ACTIONS(1993),
    [sym_dynamic_polled_port] = ACTIONS(1993),
    [sym_static_port_binding] = ACTIONS(1993),
    [sym_static_ip_binding] = ACTIONS(1993),
    [sym_get_dns_list] = ACTIONS(1993),
    [sym_set_dns_list] = ACTIONS(1993),
    [sym_get_ip_address] = ACTIONS(1993),
    [sym_set_ip_address] = ACTIONS(1993),
    [sym_get_url_list] = ACTIONS(1993),
    [sym_add_url_entry] = ACTIONS(1993),
    [sym_delete_url_entry] = ACTIONS(1993),
    [sym_abs_value] = ACTIONS(1993),
    [sym_max_value] = ACTIONS(1993),
    [sym_min_value] = ACTIONS(1993),
    [sym_exp_value] = ACTIONS(1993),
    [sym_log_value] = ACTIONS(1993),
    [sym_log10_value] = ACTIONS(1993),
    [sym_power_value] = ACTIONS(1993),
    [sym_sqrt_value] = ACTIONS(1993),
    [sym_raw_be] = ACTIONS(1993),
    [sym_raw_le] = ACTIONS(1993),
    [sym_internal_queue_size_set] = ACTIONS(1993),
    [sym_internal_queue_size_get] = ACTIONS(1993),
    [sym_internal_threshold_set] = ACTIONS(1993),
    [sym_internal_threshold_get] = ACTIONS(1993),
    [sym_rebuilt_event] = ACTIONS(1993),
    [sym_do_custom_event] = ACTIONS(1993),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1993),
    [sym_clkmgr_set_clk_source] = ACTIONS(1993),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1993),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1993),
    [sym_clkmgr_get_timezone] = ACTIONS(1993),
    [sym_clkmgr_set_timezone] = ACTIONS(1993),
    [sym_clkmgr_get_resync_period] = ACTIONS(1993),
    [sym_clkmgr_set_resync_period] = ACTIONS(1993),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1993),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1993),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1993),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1993),
    [sym_clkmgr_get_timeservers] = ACTIONS(1993),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1993),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1993),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1993),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1993),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1993),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1993),
    [sym_set_log_level] = ACTIONS(1993),
    [sym_get_log_level] = ACTIONS(1993),
    [sym_amx_log] = ACTIONS(1993),
    [sym_smtp_server_config_set] = ACTIONS(1993),
    [sym_smtp_server_config_get] = ACTIONS(1993),
    [sym_smtp_send] = ACTIONS(1993),
    [sym_validate_netlinx_account] = ACTIONS(1993),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1993),
    [sym_authenticate_certificate] = ACTIONS(1993),
    [sym_audit_netlinx_session_event] = ACTIONS(1993),
    [sym_audit_netlinx_generic_event] = ACTIONS(1993),
    [sym_get_available_flash_disk_space] = ACTIONS(1993),
    [sym_get_max_flash_disk_space] = ACTIONS(1993),
    [sym_data_create_feed] = ACTIONS(1993),
    [sym_data_delete_feed] = ACTIONS(1993),
    [sym_data_publish_feed] = ACTIONS(1993),
    [sym_data_get_published_feed] = ACTIONS(1993),
    [sym_data_add_record] = ACTIONS(1993),
    [sym_data_get_event_record] = ACTIONS(1993),
    [sym__wc_data_create_feed] = ACTIONS(1993),
    [sym__wc_data_add_record] = ACTIONS(1993),
    [sym__wc_data_get_event_record] = ACTIONS(1993),
    [sym_format] = ACTIONS(1993),
    [sym_astro_clock] = ACTIONS(1993),
    [sym_set_outdoor_temperature] = ACTIONS(1993),
    [sym_set_virtual_level_count] = ACTIONS(1993),
    [sym_set_virtual_channel_count] = ACTIONS(1993),
    [sym_set_virtual_port_count] = ACTIONS(1993),
    [sym__wc_encode_int] = ACTIONS(1993),
    [sym__wc_decode_int] = ACTIONS(1993),
    [sym__wc_int] = ACTIONS(1993),
    [sym__wc_to_ch_int] = ACTIONS(1993),
    [sym__ch_to_wc_int] = ACTIONS(1993),
    [sym__wc_find_string_int] = ACTIONS(1993),
    [sym__wc_left_string_int] = ACTIONS(1993),
    [sym__wc_lower_string_int] = ACTIONS(1993),
    [sym__wc_mid_string_int] = ACTIONS(1993),
    [sym__wc_remove_string_int] = ACTIONS(1993),
    [sym__wc_right_string_int] = ACTIONS(1993),
    [sym__wc_upper_string_int] = ACTIONS(1993),
    [sym__wc_compare_string_int] = ACTIONS(1993),
    [sym__wc_get_buffer_char_int] = ACTIONS(1993),
    [sym__wc_get_buffer_string_int] = ACTIONS(1993),
    [sym__wc_concat_string_int] = ACTIONS(1993),
    [sym__wc_file_open_int] = ACTIONS(1993),
    [sym__wc_file_close_int] = ACTIONS(1993),
    [sym__wc_file_read_int] = ACTIONS(1993),
    [sym__wc_file_read_line_int] = ACTIONS(1993),
    [sym__wc_file_write_int] = ACTIONS(1993),
    [sym__wc_file_write_line_int] = ACTIONS(1993),
    [aux_sym_preproc_include_token1] = ACTIONS(1993),
    [aux_sym_preproc_define_token1] = ACTIONS(1995),
    [aux_sym_preproc_warn_token1] = ACTIONS(1995),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1995),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1995),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1995),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1995),
    [aux_sym_preproc_else_token1] = ACTIONS(1995),
    [anon_sym_LBRACK] = ACTIONS(1995),
    [anon_sym_LBRACE] = ACTIONS(1995),
    [anon_sym_LPAREN] = ACTIONS(1993),
    [anon_sym_BANG] = ACTIONS(1995),
    [anon_sym_TILDE] = ACTIONS(1995),
    [anon_sym_DASH] = ACTIONS(1993),
    [anon_sym_PLUS] = ACTIONS(1993),
    [anon_sym_DASH_DASH] = ACTIONS(1995),
    [anon_sym_PLUS_PLUS] = ACTIONS(1995),
    [anon_sym_DQUOTE] = ACTIONS(1995),
    [anon_sym_SQUOTE] = ACTIONS(1995),
    [sym_number_literal] = ACTIONS(1995),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(313)] = {
    [ts_builtin_sym_end] = ACTIONS(1861),
    [sym_identifier] = ACTIONS(1859),
    [sym_program_name_keyword] = ACTIONS(1859),
    [sym_module_name_keyword] = ACTIONS(1859),
    [sym_define_device_keyword] = ACTIONS(1859),
    [sym_define_combine_keyword] = ACTIONS(1859),
    [sym_define_constant_keyword] = ACTIONS(1859),
    [sym_define_type_keyword] = ACTIONS(1859),
    [sym_define_variable_keyword] = ACTIONS(1859),
    [sym_define_system_variable_keyword] = ACTIONS(1859),
    [sym_define_connect_level_keyword] = ACTIONS(1859),
    [sym_define_latching_keyword] = ACTIONS(1859),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1859),
    [sym_define_toggling_keyword] = ACTIONS(1859),
    [sym_define_call_keyword] = ACTIONS(1859),
    [sym_define_function_keyword] = ACTIONS(1859),
    [sym_define_library_function_keyword] = ACTIONS(1859),
    [sym_define_start_keyword] = ACTIONS(1859),
    [sym_define_event_keyword] = ACTIONS(1859),
    [sym_define_program_keyword] = ACTIONS(1859),
    [sym_define_module_keyword] = ACTIONS(1859),
    [sym_char_keyword] = ACTIONS(1859),
    [sym_widechar_keyword] = ACTIONS(1859),
    [sym_integer_keyword] = ACTIONS(1859),
    [sym_sinteger_keyword] = ACTIONS(1859),
    [sym_long_keyword] = ACTIONS(1859),
    [sym_slong_keyword] = ACTIONS(1859),
    [sym_float_keyword] = ACTIONS(1859),
    [sym_double_keyword] = ACTIONS(1859),
    [sym_variant_keyword] = ACTIONS(1859),
    [sym_variantarray_keyword] = ACTIONS(1859),
    [sym_local_var_keyword] = ACTIONS(1859),
    [sym_stack_var_keyword] = ACTIONS(1859),
    [sym_constant_keyword] = ACTIONS(1859),
    [sym_volatile_keyword] = ACTIONS(1859),
    [sym_non_volatile_keyword] = ACTIONS(1859),
    [sym_persistent_keyword] = ACTIONS(1859),
    [sym_dev_keyword] = ACTIONS(1859),
    [sym_devlev_keyword] = ACTIONS(1859),
    [sym_devchan_keyword] = ACTIONS(1859),
    [sym_if_keyword] = ACTIONS(1859),
    [sym_else_keyword] = ACTIONS(1859),
    [sym_switch_keyword] = ACTIONS(1859),
    [sym_case_keyword] = ACTIONS(1859),
    [sym_default_keyword] = ACTIONS(1859),
    [sym_while_keyword] = ACTIONS(1859),
    [sym_for_keyword] = ACTIONS(1859),
    [sym_break_keyword] = ACTIONS(1859),
    [sym_continue_keyword] = ACTIONS(1859),
    [sym_return_keyword] = ACTIONS(1859),
    [sym_select_keyword] = ACTIONS(1859),
    [sym_active_keyword] = ACTIONS(1859),
    [sym_struct_keyword] = ACTIONS(1859),
    [sym_structure_keyword] = ACTIONS(1859),
    [sym_button_event_keyword] = ACTIONS(1859),
    [sym_channel_event_keyword] = ACTIONS(1859),
    [sym_level_event_keyword] = ACTIONS(1859),
    [sym_data_event_keyword] = ACTIONS(1859),
    [sym_timeline_event_keyword] = ACTIONS(1859),
    [sym_custom_event_keyword] = ACTIONS(1859),
    [sym_push_keyword] = ACTIONS(1859),
    [sym_release_keyword] = ACTIONS(1859),
    [aux_sym_on_keyword_token1] = ACTIONS(1859),
    [aux_sym_off_keyword_token1] = ACTIONS(1859),
    [sym_send_level_keyword] = ACTIONS(1859),
    [sym_send_string_keyword] = ACTIONS(1859),
    [sym_send_command_keyword] = ACTIONS(1859),
    [sym_clear_buffer_keyword] = ACTIONS(1859),
    [sym_create_buffer_keyword] = ACTIONS(1859),
    [sym_create_multi_buffer_keyword] = ACTIONS(1859),
    [sym_call_keyword] = ACTIONS(1859),
    [sym_system_call_keyword] = ACTIONS(1859),
    [sym_devchan_to_keyword] = ACTIONS(1859),
    [sym_devchan_min_to_keyword] = ACTIONS(1859),
    [sym_devchan_total_off_keyword] = ACTIONS(1859),
    [sym_devchan_pulse_keyword] = ACTIONS(1859),
    [sym_wait_keyword] = ACTIONS(1859),
    [sym_cancel_wait_keyword] = ACTIONS(1859),
    [sym_wait_until_keyword] = ACTIONS(1859),
    [sym_cancel_wait_until_keyword] = ACTIONS(1859),
    [sym_cancel_all_wait_keyword] = ACTIONS(1859),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1859),
    [sym_not] = ACTIONS(1859),
    [sym_bnot] = ACTIONS(1859),
    [sym___file__] = ACTIONS(1859),
    [sym___date__] = ACTIONS(1859),
    [sym___ldate__] = ACTIONS(1859),
    [sym___time__] = ACTIONS(1859),
    [sym___line__] = ACTIONS(1859),
    [sym___name__] = ACTIONS(1859),
    [sym___netlinx__] = ACTIONS(1859),
    [sym_netlinx_axi_version] = ACTIONS(1859),
    [sym_url_flg_tcp] = ACTIONS(1859),
    [sym_url_flg_accinfopresent] = ACTIONS(1859),
    [sym_url_flg_temp] = ACTIONS(1859),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1859),
    [sym_url_flg_stat_mask] = ACTIONS(1859),
    [sym_url_flg_stat_lookup] = ACTIONS(1859),
    [sym_url_flg_stat_connecting] = ACTIONS(1859),
    [sym_url_flg_stat_waiting] = ACTIONS(1859),
    [sym_url_flg_stat_connected] = ACTIONS(1859),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1859),
    [sym_first_virtual_device] = ACTIONS(1859),
    [sym_first_local_port] = ACTIONS(1859),
    [sym_dynamic_virtual_device] = ACTIONS(1859),
    [sym_timeline_once] = ACTIONS(1859),
    [sym_timeline_repeat] = ACTIONS(1859),
    [sym_timeline_absolute] = ACTIONS(1859),
    [sym_timeline_relative] = ACTIONS(1859),
    [sym_do_push_timed_infinite] = ACTIONS(1859),
    [sym_source_type_no_address] = ACTIONS(1859),
    [sym_source_type_neuron_id] = ACTIONS(1859),
    [sym_source_type_ip_address] = ACTIONS(1859),
    [sym_source_type_axlink] = ACTIONS(1859),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1859),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1859),
    [sym_source_type_ip_socket_address] = ACTIONS(1859),
    [sym_source_type_rs232] = ACTIONS(1859),
    [sym_source_type_internal] = ACTIONS(1859),
    [sym_source_type_mac_address] = ACTIONS(1859),
    [sym_source_type_ipv4_port] = ACTIONS(1859),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1859),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1859),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1859),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1859),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1859),
    [sym_true] = ACTIONS(1859),
    [sym_false] = ACTIONS(1859),
    [sym_file_read_only] = ACTIONS(1859),
    [sym_file_rw_new] = ACTIONS(1859),
    [sym_file_rw_append] = ACTIONS(1859),
    [sym_ip_tcp] = ACTIONS(1859),
    [sym_ip_udp] = ACTIONS(1859),
    [sym_ip_udp_2way] = ACTIONS(1859),
    [sym_xml_encode_types] = ACTIONS(1859),
    [sym_xml_encode_char_as_list] = ACTIONS(1859),
    [sym_xml_encode_le] = ACTIONS(1859),
    [sym_xml_decode_types] = ACTIONS(1859),
    [sym_xml_decode_no_preserve] = ACTIONS(1859),
    [sym_duet_dev_type_display_device] = ACTIONS(1859),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1859),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1859),
    [sym_duet_dev_type_io_device] = ACTIONS(1859),
    [sym_duet_dev_type_relay_device] = ACTIONS(1859),
    [sym_duet_dev_type_ups] = ACTIONS(1859),
    [sym_duet_dev_type_amplifier] = ACTIONS(1859),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1859),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1859),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1859),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1859),
    [sym_duet_dev_type_camera] = ACTIONS(1859),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1859),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1859),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1859),
    [sym_duet_dev_type_dss] = ACTIONS(1859),
    [sym_duet_dev_type_dvr] = ACTIONS(1859),
    [sym_duet_dev_type_disc_device] = ACTIONS(1859),
    [sym_duet_dev_type_document_camera] = ACTIONS(1859),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1859),
    [sym_duet_dev_type_hvac] = ACTIONS(1859),
    [sym_duet_dev_type_keypad] = ACTIONS(1859),
    [sym_duet_dev_type_light] = ACTIONS(1859),
    [sym_duet_dev_type_monitor] = ACTIONS(1859),
    [sym_duet_dev_type_motor] = ACTIONS(1859),
    [sym_duet_dev_type_multi_window] = ACTIONS(1859),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1859),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1859),
    [sym_duet_dev_type_receiver] = ACTIONS(1859),
    [sym_duet_dev_type_security_system] = ACTIONS(1859),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1859),
    [sym_duet_dev_type_settop_box] = ACTIONS(1859),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1859),
    [sym_duet_dev_type_switcher] = ACTIONS(1859),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1859),
    [sym_duet_dev_type_tv] = ACTIONS(1859),
    [sym_duet_dev_type_utility] = ACTIONS(1859),
    [sym_duet_dev_type_vcr] = ACTIONS(1859),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1859),
    [sym_duet_dev_type_video_processor] = ACTIONS(1859),
    [sym_duet_dev_type_video_projector] = ACTIONS(1859),
    [sym_duet_dev_type_video_wall] = ACTIONS(1859),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1859),
    [sym_duet_dev_type_weather] = ACTIONS(1859),
    [sym_duet_dev_not_polled] = ACTIONS(1859),
    [sym_duet_dev_polled] = ACTIONS(1859),
    [sym_clkmgr_mode_network] = ACTIONS(1859),
    [sym_clkmgr_mode_standalone] = ACTIONS(1859),
    [sym_normal_standby] = ACTIONS(1859),
    [sym_normal_wake] = ACTIONS(1859),
    [sym_ip_multicast_ttl_option] = ACTIONS(1859),
    [sym_ip_ttl_subnet] = ACTIONS(1859),
    [sym_ip_ttl_site] = ACTIONS(1859),
    [sym_ip_ttl_region] = ACTIONS(1859),
    [sym_ip_ttl_continent] = ACTIONS(1859),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1859),
    [sym_ip_nodelay_on] = ACTIONS(1859),
    [sym_ip_nodelay_off] = ACTIONS(1859),
    [sym_amx_error] = ACTIONS(1859),
    [sym_amx_warning] = ACTIONS(1859),
    [sym_amx_info] = ACTIONS(1859),
    [sym_amx_debug] = ACTIONS(1859),
    [sym_netlinx_login_success] = ACTIONS(1859),
    [sym_netlinx_login_fail] = ACTIONS(1859),
    [sym_netlinx_logout] = ACTIONS(1859),
    [sym_null_str] = ACTIONS(1859),
    [sym_smtp_address] = ACTIONS(1859),
    [sym_smtp_port_number] = ACTIONS(1859),
    [sym_smtp_username] = ACTIONS(1859),
    [sym_smtp_password] = ACTIONS(1859),
    [sym_smtp_require_tls] = ACTIONS(1859),
    [sym_smtp_from] = ACTIONS(1859),
    [sym_smtp_tls_true] = ACTIONS(1859),
    [sym_smtp_tls_false] = ACTIONS(1859),
    [sym_listview_on_row_select_event] = ACTIONS(1859),
    [sym_data_structure_unknown] = ACTIONS(1859),
    [sym_data_structure_datafeed] = ACTIONS(1859),
    [sym_data_structure_datarecordset] = ACTIONS(1859),
    [sym_data_structure_datarecord] = ACTIONS(1859),
    [sym_data_structure_datafield] = ACTIONS(1859),
    [sym_data_max_content_field_count] = ACTIONS(1859),
    [sym_data_max_metadata_field_count] = ACTIONS(1859),
    [sym_data_max_id_length] = ACTIONS(1859),
    [sym_data_max_name_length] = ACTIONS(1859),
    [sym_data_max_description_length] = ACTIONS(1859),
    [sym_data_max_type_length] = ACTIONS(1859),
    [sym_data_max_format_length] = ACTIONS(1859),
    [sym_data_max_source_length] = ACTIONS(1859),
    [sym_data_max_value_length] = ACTIONS(1859),
    [sym_data_max_label_length] = ACTIONS(1859),
    [sym_data_max_url_length] = ACTIONS(1859),
    [sym_max_last_login_info_length] = ACTIONS(1859),
    [sym_data_type_unknown] = ACTIONS(1859),
    [sym_data_type_string] = ACTIONS(1859),
    [sym_data_type_datetime] = ACTIONS(1859),
    [sym_data_type_date] = ACTIONS(1859),
    [sym_data_type_time] = ACTIONS(1859),
    [sym_data_type_image] = ACTIONS(1859),
    [sym_data_format_url] = ACTIONS(1859),
    [sym_data_format_phone] = ACTIONS(1859),
    [sym_data_format_email] = ACTIONS(1859),
    [sym_data_format_iso8601] = ACTIONS(1859),
    [sym_auditlog_priv] = ACTIONS(1859),
    [sym_configuration_priv] = ACTIONS(1859),
    [sym_deviceconfig_priv] = ACTIONS(1859),
    [sym_ftp_priv] = ACTIONS(1859),
    [sym_http_priv] = ACTIONS(1859),
    [sym_networkconfig_priv] = ACTIONS(1859),
    [sym_programport_priv] = ACTIONS(1859),
    [sym_remoteui_priv] = ACTIONS(1859),
    [sym_securitycontrol_priv] = ACTIONS(1859),
    [sym_softwaremanagement_priv] = ACTIONS(1859),
    [sym_terminal_priv] = ACTIONS(1859),
    [sym_tpadmin_priv] = ACTIONS(1859),
    [sym_user1_priv] = ACTIONS(1859),
    [sym_user2_priv] = ACTIONS(1859),
    [sym_user3_priv] = ACTIONS(1859),
    [sym_user4_priv] = ACTIONS(1859),
    [sym_usermanagement_priv] = ACTIONS(1859),
    [sym_valid_account] = ACTIONS(1859),
    [sym_err_name_invalid] = ACTIONS(1859),
    [sym_err_password_invalid] = ACTIONS(1859),
    [sym_err_invalid_account] = ACTIONS(1859),
    [sym_err_no_authorization] = ACTIONS(1859),
    [sym_err_invalid_parameter] = ACTIONS(1859),
    [sym_err_user_locked_out] = ACTIONS(1859),
    [sym_err_user_account_expired] = ACTIONS(1859),
    [sym_err_general_error] = ACTIONS(1859),
    [sym_err_authentication_error] = ACTIONS(1859),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1859),
    [sym_tls_validate_certificate] = ACTIONS(1859),
    [sym_tdata] = ACTIONS(1859),
    [sym_tchannel] = ACTIONS(1859),
    [sym_tlevel] = ACTIONS(1859),
    [sym_tbutton] = ACTIONS(1859),
    [sym_ttimeline] = ACTIONS(1859),
    [sym_tcustom] = ACTIONS(1859),
    [sym_url_struct] = ACTIONS(1859),
    [sym_dns_struct] = ACTIONS(1859),
    [sym_ip_address_struct] = ACTIONS(1859),
    [sym_dev_info_struct] = ACTIONS(1859),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1859),
    [sym_data_feed] = ACTIONS(1859),
    [sym_data_field] = ACTIONS(1859),
    [sym_data_record] = ACTIONS(1859),
    [sym_wc_data_feed] = ACTIONS(1859),
    [sym_wc_data_field] = ACTIONS(1859),
    [sym_wc_data_record] = ACTIONS(1859),
    [sym_last_login_info] = ACTIONS(1859),
    [sym____reserved___] = ACTIONS(1859),
    [sym_date] = ACTIONS(1859),
    [sym_day] = ACTIONS(1859),
    [sym_time] = ACTIONS(1859),
    [sym_ldate] = ACTIONS(1859),
    [sym_push_device] = ACTIONS(1859),
    [sym_push_channel] = ACTIONS(1859),
    [sym_push_devchan] = ACTIONS(1859),
    [sym_release_device] = ACTIONS(1859),
    [sym_release_channel] = ACTIONS(1859),
    [sym_release_devchan] = ACTIONS(1859),
    [sym_master_slot] = ACTIONS(1859),
    [sym_get_pulse_time] = ACTIONS(1859),
    [sym_get_timer] = ACTIONS(1859),
    [sym_master_sn] = ACTIONS(1859),
    [sym_system_number] = ACTIONS(1859),
    [sym_button] = ACTIONS(1859),
    [sym_dv_channel] = ACTIONS(1859),
    [sym_data] = ACTIONS(1859),
    [sym_level] = ACTIONS(1859),
    [sym_timeline] = ACTIONS(1859),
    [sym_channel] = ACTIONS(1859),
    [sym_custom] = ACTIONS(1859),
    [sym_atoi] = ACTIONS(1859),
    [sym_atol] = ACTIONS(1859),
    [sym_atof] = ACTIONS(1859),
    [sym_itoa] = ACTIONS(1859),
    [sym_itohex] = ACTIONS(1859),
    [sym_hextoi] = ACTIONS(1859),
    [sym_ftoa] = ACTIONS(1859),
    [sym_find_string] = ACTIONS(1859),
    [sym_left_string] = ACTIONS(1859),
    [sym_right_string] = ACTIONS(1859),
    [sym_mid_string] = ACTIONS(1859),
    [sym_remove_string] = ACTIONS(1859),
    [sym_get_buffer_string] = ACTIONS(1859),
    [sym_get_buffer_char] = ACTIONS(1859),
    [sym_get_multi_buffer_string] = ACTIONS(1859),
    [sym_length_string] = ACTIONS(1859),
    [sym_max_length_string] = ACTIONS(1859),
    [sym_set_length_string] = ACTIONS(1859),
    [sym_compare_string] = ACTIONS(1859),
    [sym_upper_string] = ACTIONS(1859),
    [sym_lower_string] = ACTIONS(1859),
    [sym_get_last] = ACTIONS(1859),
    [sym_date_to_day] = ACTIONS(1859),
    [sym_date_to_month] = ACTIONS(1859),
    [sym_date_to_year] = ACTIONS(1859),
    [sym_time_to_hour] = ACTIONS(1859),
    [sym_time_to_minute] = ACTIONS(1859),
    [sym_time_to_second] = ACTIONS(1859),
    [sym_day_of_week] = ACTIONS(1859),
    [sym_set_pulse_time] = ACTIONS(1859),
    [sym_set_timer] = ACTIONS(1859),
    [sym_file_open] = ACTIONS(1859),
    [sym_file_close] = ACTIONS(1859),
    [sym_file_read] = ACTIONS(1859),
    [sym_file_read_line] = ACTIONS(1859),
    [sym_file_write] = ACTIONS(1859),
    [sym_file_write_line] = ACTIONS(1859),
    [sym_file_copy] = ACTIONS(1859),
    [sym_file_delete] = ACTIONS(1859),
    [sym_file_rename] = ACTIONS(1859),
    [sym_file_dir] = ACTIONS(1859),
    [sym_file_getdir] = ACTIONS(1859),
    [sym_file_setdir] = ACTIONS(1859),
    [sym_file_create_dir] = ACTIONS(1859),
    [sym_file_remove_dir] = ACTIONS(1859),
    [sym_file_seek] = ACTIONS(1859),
    [sym_length_array] = ACTIONS(1859),
    [sym_max_length_array] = ACTIONS(1859),
    [sym_set_length_array] = ACTIONS(1859),
    [sym_ip_client_open] = ACTIONS(1859),
    [sym_ip_client_close] = ACTIONS(1859),
    [sym_ip_server_open] = ACTIONS(1859),
    [sym_ip_server_close] = ACTIONS(1859),
    [sym_ip_bound_client_open] = ACTIONS(1859),
    [sym_ip_mc_server_open] = ACTIONS(1859),
    [sym_ip_set_option] = ACTIONS(1859),
    [sym_ssh_client_open] = ACTIONS(1859),
    [sym_ssh_client_close] = ACTIONS(1859),
    [sym_tls_client_open] = ACTIONS(1859),
    [sym_tls_client_close] = ACTIONS(1859),
    [sym_do_push] = ACTIONS(1859),
    [sym_do_push_timed] = ACTIONS(1859),
    [sym_do_release] = ACTIONS(1859),
    [sym_device_id] = ACTIONS(1859),
    [sym_device_info] = ACTIONS(1859),
    [sym_device_id_string] = ACTIONS(1859),
    [sym_device_standby] = ACTIONS(1859),
    [sym_device_wake] = ACTIONS(1859),
    [sym_timeline_create] = ACTIONS(1859),
    [sym_timeline_kill] = ACTIONS(1859),
    [sym_timeline_reload] = ACTIONS(1859),
    [sym_timeline_pause] = ACTIONS(1859),
    [sym_timeline_restart] = ACTIONS(1859),
    [sym_timeline_set] = ACTIONS(1859),
    [sym_timeline_get] = ACTIONS(1859),
    [sym_timeline_active] = ACTIONS(1859),
    [sym_timeline_dynamic_id] = ACTIONS(1859),
    [sym_variable_to_string] = ACTIONS(1859),
    [sym_string_to_variable] = ACTIONS(1859),
    [sym_length_variable_to_string] = ACTIONS(1859),
    [sym_type_cast] = ACTIONS(1859),
    [sym_variable_to_xml] = ACTIONS(1859),
    [sym_xml_to_variable] = ACTIONS(1859),
    [sym_length_variable_to_xml] = ACTIONS(1859),
    [sym_redirect_string] = ACTIONS(1859),
    [sym_random_number] = ACTIONS(1859),
    [sym_get_unique_id] = ACTIONS(1859),
    [sym_get_system_number] = ACTIONS(1859),
    [sym_set_system_number] = ACTIONS(1859),
    [sym_get_serial_number] = ACTIONS(1859),
    [sym_get_master_build] = ACTIONS(1859),
    [sym_set_validation_code] = ACTIONS(1859),
    [sym_reboot] = ACTIONS(1859),
    [sym_load_duet_module] = ACTIONS(1859),
    [sym_unload_duet_module] = ACTIONS(1859),
    [sym_duet_mem_size_set] = ACTIONS(1859),
    [sym_duet_mem_size_get] = ACTIONS(1859),
    [sym_dynamic_application_device] = ACTIONS(1859),
    [sym_dynamic_polled_port] = ACTIONS(1859),
    [sym_static_port_binding] = ACTIONS(1859),
    [sym_static_ip_binding] = ACTIONS(1859),
    [sym_get_dns_list] = ACTIONS(1859),
    [sym_set_dns_list] = ACTIONS(1859),
    [sym_get_ip_address] = ACTIONS(1859),
    [sym_set_ip_address] = ACTIONS(1859),
    [sym_get_url_list] = ACTIONS(1859),
    [sym_add_url_entry] = ACTIONS(1859),
    [sym_delete_url_entry] = ACTIONS(1859),
    [sym_abs_value] = ACTIONS(1859),
    [sym_max_value] = ACTIONS(1859),
    [sym_min_value] = ACTIONS(1859),
    [sym_exp_value] = ACTIONS(1859),
    [sym_log_value] = ACTIONS(1859),
    [sym_log10_value] = ACTIONS(1859),
    [sym_power_value] = ACTIONS(1859),
    [sym_sqrt_value] = ACTIONS(1859),
    [sym_raw_be] = ACTIONS(1859),
    [sym_raw_le] = ACTIONS(1859),
    [sym_internal_queue_size_set] = ACTIONS(1859),
    [sym_internal_queue_size_get] = ACTIONS(1859),
    [sym_internal_threshold_set] = ACTIONS(1859),
    [sym_internal_threshold_get] = ACTIONS(1859),
    [sym_rebuilt_event] = ACTIONS(1859),
    [sym_do_custom_event] = ACTIONS(1859),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1859),
    [sym_clkmgr_set_clk_source] = ACTIONS(1859),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1859),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1859),
    [sym_clkmgr_get_timezone] = ACTIONS(1859),
    [sym_clkmgr_set_timezone] = ACTIONS(1859),
    [sym_clkmgr_get_resync_period] = ACTIONS(1859),
    [sym_clkmgr_set_resync_period] = ACTIONS(1859),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1859),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1859),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1859),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1859),
    [sym_clkmgr_get_timeservers] = ACTIONS(1859),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1859),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1859),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1859),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1859),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1859),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1859),
    [sym_set_log_level] = ACTIONS(1859),
    [sym_get_log_level] = ACTIONS(1859),
    [sym_amx_log] = ACTIONS(1859),
    [sym_smtp_server_config_set] = ACTIONS(1859),
    [sym_smtp_server_config_get] = ACTIONS(1859),
    [sym_smtp_send] = ACTIONS(1859),
    [sym_validate_netlinx_account] = ACTIONS(1859),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1859),
    [sym_authenticate_certificate] = ACTIONS(1859),
    [sym_audit_netlinx_session_event] = ACTIONS(1859),
    [sym_audit_netlinx_generic_event] = ACTIONS(1859),
    [sym_get_available_flash_disk_space] = ACTIONS(1859),
    [sym_get_max_flash_disk_space] = ACTIONS(1859),
    [sym_data_create_feed] = ACTIONS(1859),
    [sym_data_delete_feed] = ACTIONS(1859),
    [sym_data_publish_feed] = ACTIONS(1859),
    [sym_data_get_published_feed] = ACTIONS(1859),
    [sym_data_add_record] = ACTIONS(1859),
    [sym_data_get_event_record] = ACTIONS(1859),
    [sym__wc_data_create_feed] = ACTIONS(1859),
    [sym__wc_data_add_record] = ACTIONS(1859),
    [sym__wc_data_get_event_record] = ACTIONS(1859),
    [sym_format] = ACTIONS(1859),
    [sym_astro_clock] = ACTIONS(1859),
    [sym_set_outdoor_temperature] = ACTIONS(1859),
    [sym_set_virtual_level_count] = ACTIONS(1859),
    [sym_set_virtual_channel_count] = ACTIONS(1859),
    [sym_set_virtual_port_count] = ACTIONS(1859),
    [sym__wc_encode_int] = ACTIONS(1859),
    [sym__wc_decode_int] = ACTIONS(1859),
    [sym__wc_int] = ACTIONS(1859),
    [sym__wc_to_ch_int] = ACTIONS(1859),
    [sym__ch_to_wc_int] = ACTIONS(1859),
    [sym__wc_find_string_int] = ACTIONS(1859),
    [sym__wc_left_string_int] = ACTIONS(1859),
    [sym__wc_lower_string_int] = ACTIONS(1859),
    [sym__wc_mid_string_int] = ACTIONS(1859),
    [sym__wc_remove_string_int] = ACTIONS(1859),
    [sym__wc_right_string_int] = ACTIONS(1859),
    [sym__wc_upper_string_int] = ACTIONS(1859),
    [sym__wc_compare_string_int] = ACTIONS(1859),
    [sym__wc_get_buffer_char_int] = ACTIONS(1859),
    [sym__wc_get_buffer_string_int] = ACTIONS(1859),
    [sym__wc_concat_string_int] = ACTIONS(1859),
    [sym__wc_file_open_int] = ACTIONS(1859),
    [sym__wc_file_close_int] = ACTIONS(1859),
    [sym__wc_file_read_int] = ACTIONS(1859),
    [sym__wc_file_read_line_int] = ACTIONS(1859),
    [sym__wc_file_write_int] = ACTIONS(1859),
    [sym__wc_file_write_line_int] = ACTIONS(1859),
    [aux_sym_preproc_include_token1] = ACTIONS(1859),
    [aux_sym_preproc_define_token1] = ACTIONS(1861),
    [aux_sym_preproc_warn_token1] = ACTIONS(1861),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1861),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1861),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1861),
    [anon_sym_LBRACK] = ACTIONS(1861),
    [anon_sym_LBRACE] = ACTIONS(1861),
    [anon_sym_LPAREN] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1861),
    [anon_sym_TILDE] = ACTIONS(1861),
    [anon_sym_DASH] = ACTIONS(1859),
    [anon_sym_PLUS] = ACTIONS(1859),
    [anon_sym_DASH_DASH] = ACTIONS(1861),
    [anon_sym_PLUS_PLUS] = ACTIONS(1861),
    [anon_sym_DQUOTE] = ACTIONS(1861),
    [anon_sym_SQUOTE] = ACTIONS(1861),
    [sym_number_literal] = ACTIONS(1861),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(314)] = {
    [ts_builtin_sym_end] = ACTIONS(1865),
    [sym_identifier] = ACTIONS(1863),
    [sym_program_name_keyword] = ACTIONS(1863),
    [sym_module_name_keyword] = ACTIONS(1863),
    [sym_define_device_keyword] = ACTIONS(1863),
    [sym_define_combine_keyword] = ACTIONS(1863),
    [sym_define_constant_keyword] = ACTIONS(1863),
    [sym_define_type_keyword] = ACTIONS(1863),
    [sym_define_variable_keyword] = ACTIONS(1863),
    [sym_define_system_variable_keyword] = ACTIONS(1863),
    [sym_define_connect_level_keyword] = ACTIONS(1863),
    [sym_define_latching_keyword] = ACTIONS(1863),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1863),
    [sym_define_toggling_keyword] = ACTIONS(1863),
    [sym_define_call_keyword] = ACTIONS(1863),
    [sym_define_function_keyword] = ACTIONS(1863),
    [sym_define_library_function_keyword] = ACTIONS(1863),
    [sym_define_start_keyword] = ACTIONS(1863),
    [sym_define_event_keyword] = ACTIONS(1863),
    [sym_define_program_keyword] = ACTIONS(1863),
    [sym_define_module_keyword] = ACTIONS(1863),
    [sym_char_keyword] = ACTIONS(1863),
    [sym_widechar_keyword] = ACTIONS(1863),
    [sym_integer_keyword] = ACTIONS(1863),
    [sym_sinteger_keyword] = ACTIONS(1863),
    [sym_long_keyword] = ACTIONS(1863),
    [sym_slong_keyword] = ACTIONS(1863),
    [sym_float_keyword] = ACTIONS(1863),
    [sym_double_keyword] = ACTIONS(1863),
    [sym_variant_keyword] = ACTIONS(1863),
    [sym_variantarray_keyword] = ACTIONS(1863),
    [sym_local_var_keyword] = ACTIONS(1863),
    [sym_stack_var_keyword] = ACTIONS(1863),
    [sym_constant_keyword] = ACTIONS(1863),
    [sym_volatile_keyword] = ACTIONS(1863),
    [sym_non_volatile_keyword] = ACTIONS(1863),
    [sym_persistent_keyword] = ACTIONS(1863),
    [sym_dev_keyword] = ACTIONS(1863),
    [sym_devlev_keyword] = ACTIONS(1863),
    [sym_devchan_keyword] = ACTIONS(1863),
    [sym_if_keyword] = ACTIONS(1863),
    [sym_else_keyword] = ACTIONS(1863),
    [sym_switch_keyword] = ACTIONS(1863),
    [sym_case_keyword] = ACTIONS(1863),
    [sym_default_keyword] = ACTIONS(1863),
    [sym_while_keyword] = ACTIONS(1863),
    [sym_for_keyword] = ACTIONS(1863),
    [sym_break_keyword] = ACTIONS(1863),
    [sym_continue_keyword] = ACTIONS(1863),
    [sym_return_keyword] = ACTIONS(1863),
    [sym_select_keyword] = ACTIONS(1863),
    [sym_active_keyword] = ACTIONS(1863),
    [sym_struct_keyword] = ACTIONS(1863),
    [sym_structure_keyword] = ACTIONS(1863),
    [sym_button_event_keyword] = ACTIONS(1863),
    [sym_channel_event_keyword] = ACTIONS(1863),
    [sym_level_event_keyword] = ACTIONS(1863),
    [sym_data_event_keyword] = ACTIONS(1863),
    [sym_timeline_event_keyword] = ACTIONS(1863),
    [sym_custom_event_keyword] = ACTIONS(1863),
    [sym_push_keyword] = ACTIONS(1863),
    [sym_release_keyword] = ACTIONS(1863),
    [aux_sym_on_keyword_token1] = ACTIONS(1863),
    [aux_sym_off_keyword_token1] = ACTIONS(1863),
    [sym_send_level_keyword] = ACTIONS(1863),
    [sym_send_string_keyword] = ACTIONS(1863),
    [sym_send_command_keyword] = ACTIONS(1863),
    [sym_clear_buffer_keyword] = ACTIONS(1863),
    [sym_create_buffer_keyword] = ACTIONS(1863),
    [sym_create_multi_buffer_keyword] = ACTIONS(1863),
    [sym_call_keyword] = ACTIONS(1863),
    [sym_system_call_keyword] = ACTIONS(1863),
    [sym_devchan_to_keyword] = ACTIONS(1863),
    [sym_devchan_min_to_keyword] = ACTIONS(1863),
    [sym_devchan_total_off_keyword] = ACTIONS(1863),
    [sym_devchan_pulse_keyword] = ACTIONS(1863),
    [sym_wait_keyword] = ACTIONS(1863),
    [sym_cancel_wait_keyword] = ACTIONS(1863),
    [sym_wait_until_keyword] = ACTIONS(1863),
    [sym_cancel_wait_until_keyword] = ACTIONS(1863),
    [sym_cancel_all_wait_keyword] = ACTIONS(1863),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1863),
    [sym_not] = ACTIONS(1863),
    [sym_bnot] = ACTIONS(1863),
    [sym___file__] = ACTIONS(1863),
    [sym___date__] = ACTIONS(1863),
    [sym___ldate__] = ACTIONS(1863),
    [sym___time__] = ACTIONS(1863),
    [sym___line__] = ACTIONS(1863),
    [sym___name__] = ACTIONS(1863),
    [sym___netlinx__] = ACTIONS(1863),
    [sym_netlinx_axi_version] = ACTIONS(1863),
    [sym_url_flg_tcp] = ACTIONS(1863),
    [sym_url_flg_accinfopresent] = ACTIONS(1863),
    [sym_url_flg_temp] = ACTIONS(1863),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1863),
    [sym_url_flg_stat_mask] = ACTIONS(1863),
    [sym_url_flg_stat_lookup] = ACTIONS(1863),
    [sym_url_flg_stat_connecting] = ACTIONS(1863),
    [sym_url_flg_stat_waiting] = ACTIONS(1863),
    [sym_url_flg_stat_connected] = ACTIONS(1863),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1863),
    [sym_first_virtual_device] = ACTIONS(1863),
    [sym_first_local_port] = ACTIONS(1863),
    [sym_dynamic_virtual_device] = ACTIONS(1863),
    [sym_timeline_once] = ACTIONS(1863),
    [sym_timeline_repeat] = ACTIONS(1863),
    [sym_timeline_absolute] = ACTIONS(1863),
    [sym_timeline_relative] = ACTIONS(1863),
    [sym_do_push_timed_infinite] = ACTIONS(1863),
    [sym_source_type_no_address] = ACTIONS(1863),
    [sym_source_type_neuron_id] = ACTIONS(1863),
    [sym_source_type_ip_address] = ACTIONS(1863),
    [sym_source_type_axlink] = ACTIONS(1863),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1863),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1863),
    [sym_source_type_ip_socket_address] = ACTIONS(1863),
    [sym_source_type_rs232] = ACTIONS(1863),
    [sym_source_type_internal] = ACTIONS(1863),
    [sym_source_type_mac_address] = ACTIONS(1863),
    [sym_source_type_ipv4_port] = ACTIONS(1863),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1863),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1863),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1863),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1863),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1863),
    [sym_true] = ACTIONS(1863),
    [sym_false] = ACTIONS(1863),
    [sym_file_read_only] = ACTIONS(1863),
    [sym_file_rw_new] = ACTIONS(1863),
    [sym_file_rw_append] = ACTIONS(1863),
    [sym_ip_tcp] = ACTIONS(1863),
    [sym_ip_udp] = ACTIONS(1863),
    [sym_ip_udp_2way] = ACTIONS(1863),
    [sym_xml_encode_types] = ACTIONS(1863),
    [sym_xml_encode_char_as_list] = ACTIONS(1863),
    [sym_xml_encode_le] = ACTIONS(1863),
    [sym_xml_decode_types] = ACTIONS(1863),
    [sym_xml_decode_no_preserve] = ACTIONS(1863),
    [sym_duet_dev_type_display_device] = ACTIONS(1863),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1863),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1863),
    [sym_duet_dev_type_io_device] = ACTIONS(1863),
    [sym_duet_dev_type_relay_device] = ACTIONS(1863),
    [sym_duet_dev_type_ups] = ACTIONS(1863),
    [sym_duet_dev_type_amplifier] = ACTIONS(1863),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1863),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1863),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1863),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1863),
    [sym_duet_dev_type_camera] = ACTIONS(1863),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1863),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1863),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1863),
    [sym_duet_dev_type_dss] = ACTIONS(1863),
    [sym_duet_dev_type_dvr] = ACTIONS(1863),
    [sym_duet_dev_type_disc_device] = ACTIONS(1863),
    [sym_duet_dev_type_document_camera] = ACTIONS(1863),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1863),
    [sym_duet_dev_type_hvac] = ACTIONS(1863),
    [sym_duet_dev_type_keypad] = ACTIONS(1863),
    [sym_duet_dev_type_light] = ACTIONS(1863),
    [sym_duet_dev_type_monitor] = ACTIONS(1863),
    [sym_duet_dev_type_motor] = ACTIONS(1863),
    [sym_duet_dev_type_multi_window] = ACTIONS(1863),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1863),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1863),
    [sym_duet_dev_type_receiver] = ACTIONS(1863),
    [sym_duet_dev_type_security_system] = ACTIONS(1863),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1863),
    [sym_duet_dev_type_settop_box] = ACTIONS(1863),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1863),
    [sym_duet_dev_type_switcher] = ACTIONS(1863),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1863),
    [sym_duet_dev_type_tv] = ACTIONS(1863),
    [sym_duet_dev_type_utility] = ACTIONS(1863),
    [sym_duet_dev_type_vcr] = ACTIONS(1863),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1863),
    [sym_duet_dev_type_video_processor] = ACTIONS(1863),
    [sym_duet_dev_type_video_projector] = ACTIONS(1863),
    [sym_duet_dev_type_video_wall] = ACTIONS(1863),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1863),
    [sym_duet_dev_type_weather] = ACTIONS(1863),
    [sym_duet_dev_not_polled] = ACTIONS(1863),
    [sym_duet_dev_polled] = ACTIONS(1863),
    [sym_clkmgr_mode_network] = ACTIONS(1863),
    [sym_clkmgr_mode_standalone] = ACTIONS(1863),
    [sym_normal_standby] = ACTIONS(1863),
    [sym_normal_wake] = ACTIONS(1863),
    [sym_ip_multicast_ttl_option] = ACTIONS(1863),
    [sym_ip_ttl_subnet] = ACTIONS(1863),
    [sym_ip_ttl_site] = ACTIONS(1863),
    [sym_ip_ttl_region] = ACTIONS(1863),
    [sym_ip_ttl_continent] = ACTIONS(1863),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1863),
    [sym_ip_nodelay_on] = ACTIONS(1863),
    [sym_ip_nodelay_off] = ACTIONS(1863),
    [sym_amx_error] = ACTIONS(1863),
    [sym_amx_warning] = ACTIONS(1863),
    [sym_amx_info] = ACTIONS(1863),
    [sym_amx_debug] = ACTIONS(1863),
    [sym_netlinx_login_success] = ACTIONS(1863),
    [sym_netlinx_login_fail] = ACTIONS(1863),
    [sym_netlinx_logout] = ACTIONS(1863),
    [sym_null_str] = ACTIONS(1863),
    [sym_smtp_address] = ACTIONS(1863),
    [sym_smtp_port_number] = ACTIONS(1863),
    [sym_smtp_username] = ACTIONS(1863),
    [sym_smtp_password] = ACTIONS(1863),
    [sym_smtp_require_tls] = ACTIONS(1863),
    [sym_smtp_from] = ACTIONS(1863),
    [sym_smtp_tls_true] = ACTIONS(1863),
    [sym_smtp_tls_false] = ACTIONS(1863),
    [sym_listview_on_row_select_event] = ACTIONS(1863),
    [sym_data_structure_unknown] = ACTIONS(1863),
    [sym_data_structure_datafeed] = ACTIONS(1863),
    [sym_data_structure_datarecordset] = ACTIONS(1863),
    [sym_data_structure_datarecord] = ACTIONS(1863),
    [sym_data_structure_datafield] = ACTIONS(1863),
    [sym_data_max_content_field_count] = ACTIONS(1863),
    [sym_data_max_metadata_field_count] = ACTIONS(1863),
    [sym_data_max_id_length] = ACTIONS(1863),
    [sym_data_max_name_length] = ACTIONS(1863),
    [sym_data_max_description_length] = ACTIONS(1863),
    [sym_data_max_type_length] = ACTIONS(1863),
    [sym_data_max_format_length] = ACTIONS(1863),
    [sym_data_max_source_length] = ACTIONS(1863),
    [sym_data_max_value_length] = ACTIONS(1863),
    [sym_data_max_label_length] = ACTIONS(1863),
    [sym_data_max_url_length] = ACTIONS(1863),
    [sym_max_last_login_info_length] = ACTIONS(1863),
    [sym_data_type_unknown] = ACTIONS(1863),
    [sym_data_type_string] = ACTIONS(1863),
    [sym_data_type_datetime] = ACTIONS(1863),
    [sym_data_type_date] = ACTIONS(1863),
    [sym_data_type_time] = ACTIONS(1863),
    [sym_data_type_image] = ACTIONS(1863),
    [sym_data_format_url] = ACTIONS(1863),
    [sym_data_format_phone] = ACTIONS(1863),
    [sym_data_format_email] = ACTIONS(1863),
    [sym_data_format_iso8601] = ACTIONS(1863),
    [sym_auditlog_priv] = ACTIONS(1863),
    [sym_configuration_priv] = ACTIONS(1863),
    [sym_deviceconfig_priv] = ACTIONS(1863),
    [sym_ftp_priv] = ACTIONS(1863),
    [sym_http_priv] = ACTIONS(1863),
    [sym_networkconfig_priv] = ACTIONS(1863),
    [sym_programport_priv] = ACTIONS(1863),
    [sym_remoteui_priv] = ACTIONS(1863),
    [sym_securitycontrol_priv] = ACTIONS(1863),
    [sym_softwaremanagement_priv] = ACTIONS(1863),
    [sym_terminal_priv] = ACTIONS(1863),
    [sym_tpadmin_priv] = ACTIONS(1863),
    [sym_user1_priv] = ACTIONS(1863),
    [sym_user2_priv] = ACTIONS(1863),
    [sym_user3_priv] = ACTIONS(1863),
    [sym_user4_priv] = ACTIONS(1863),
    [sym_usermanagement_priv] = ACTIONS(1863),
    [sym_valid_account] = ACTIONS(1863),
    [sym_err_name_invalid] = ACTIONS(1863),
    [sym_err_password_invalid] = ACTIONS(1863),
    [sym_err_invalid_account] = ACTIONS(1863),
    [sym_err_no_authorization] = ACTIONS(1863),
    [sym_err_invalid_parameter] = ACTIONS(1863),
    [sym_err_user_locked_out] = ACTIONS(1863),
    [sym_err_user_account_expired] = ACTIONS(1863),
    [sym_err_general_error] = ACTIONS(1863),
    [sym_err_authentication_error] = ACTIONS(1863),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1863),
    [sym_tls_validate_certificate] = ACTIONS(1863),
    [sym_tdata] = ACTIONS(1863),
    [sym_tchannel] = ACTIONS(1863),
    [sym_tlevel] = ACTIONS(1863),
    [sym_tbutton] = ACTIONS(1863),
    [sym_ttimeline] = ACTIONS(1863),
    [sym_tcustom] = ACTIONS(1863),
    [sym_url_struct] = ACTIONS(1863),
    [sym_dns_struct] = ACTIONS(1863),
    [sym_ip_address_struct] = ACTIONS(1863),
    [sym_dev_info_struct] = ACTIONS(1863),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1863),
    [sym_data_feed] = ACTIONS(1863),
    [sym_data_field] = ACTIONS(1863),
    [sym_data_record] = ACTIONS(1863),
    [sym_wc_data_feed] = ACTIONS(1863),
    [sym_wc_data_field] = ACTIONS(1863),
    [sym_wc_data_record] = ACTIONS(1863),
    [sym_last_login_info] = ACTIONS(1863),
    [sym____reserved___] = ACTIONS(1863),
    [sym_date] = ACTIONS(1863),
    [sym_day] = ACTIONS(1863),
    [sym_time] = ACTIONS(1863),
    [sym_ldate] = ACTIONS(1863),
    [sym_push_device] = ACTIONS(1863),
    [sym_push_channel] = ACTIONS(1863),
    [sym_push_devchan] = ACTIONS(1863),
    [sym_release_device] = ACTIONS(1863),
    [sym_release_channel] = ACTIONS(1863),
    [sym_release_devchan] = ACTIONS(1863),
    [sym_master_slot] = ACTIONS(1863),
    [sym_get_pulse_time] = ACTIONS(1863),
    [sym_get_timer] = ACTIONS(1863),
    [sym_master_sn] = ACTIONS(1863),
    [sym_system_number] = ACTIONS(1863),
    [sym_button] = ACTIONS(1863),
    [sym_dv_channel] = ACTIONS(1863),
    [sym_data] = ACTIONS(1863),
    [sym_level] = ACTIONS(1863),
    [sym_timeline] = ACTIONS(1863),
    [sym_channel] = ACTIONS(1863),
    [sym_custom] = ACTIONS(1863),
    [sym_atoi] = ACTIONS(1863),
    [sym_atol] = ACTIONS(1863),
    [sym_atof] = ACTIONS(1863),
    [sym_itoa] = ACTIONS(1863),
    [sym_itohex] = ACTIONS(1863),
    [sym_hextoi] = ACTIONS(1863),
    [sym_ftoa] = ACTIONS(1863),
    [sym_find_string] = ACTIONS(1863),
    [sym_left_string] = ACTIONS(1863),
    [sym_right_string] = ACTIONS(1863),
    [sym_mid_string] = ACTIONS(1863),
    [sym_remove_string] = ACTIONS(1863),
    [sym_get_buffer_string] = ACTIONS(1863),
    [sym_get_buffer_char] = ACTIONS(1863),
    [sym_get_multi_buffer_string] = ACTIONS(1863),
    [sym_length_string] = ACTIONS(1863),
    [sym_max_length_string] = ACTIONS(1863),
    [sym_set_length_string] = ACTIONS(1863),
    [sym_compare_string] = ACTIONS(1863),
    [sym_upper_string] = ACTIONS(1863),
    [sym_lower_string] = ACTIONS(1863),
    [sym_get_last] = ACTIONS(1863),
    [sym_date_to_day] = ACTIONS(1863),
    [sym_date_to_month] = ACTIONS(1863),
    [sym_date_to_year] = ACTIONS(1863),
    [sym_time_to_hour] = ACTIONS(1863),
    [sym_time_to_minute] = ACTIONS(1863),
    [sym_time_to_second] = ACTIONS(1863),
    [sym_day_of_week] = ACTIONS(1863),
    [sym_set_pulse_time] = ACTIONS(1863),
    [sym_set_timer] = ACTIONS(1863),
    [sym_file_open] = ACTIONS(1863),
    [sym_file_close] = ACTIONS(1863),
    [sym_file_read] = ACTIONS(1863),
    [sym_file_read_line] = ACTIONS(1863),
    [sym_file_write] = ACTIONS(1863),
    [sym_file_write_line] = ACTIONS(1863),
    [sym_file_copy] = ACTIONS(1863),
    [sym_file_delete] = ACTIONS(1863),
    [sym_file_rename] = ACTIONS(1863),
    [sym_file_dir] = ACTIONS(1863),
    [sym_file_getdir] = ACTIONS(1863),
    [sym_file_setdir] = ACTIONS(1863),
    [sym_file_create_dir] = ACTIONS(1863),
    [sym_file_remove_dir] = ACTIONS(1863),
    [sym_file_seek] = ACTIONS(1863),
    [sym_length_array] = ACTIONS(1863),
    [sym_max_length_array] = ACTIONS(1863),
    [sym_set_length_array] = ACTIONS(1863),
    [sym_ip_client_open] = ACTIONS(1863),
    [sym_ip_client_close] = ACTIONS(1863),
    [sym_ip_server_open] = ACTIONS(1863),
    [sym_ip_server_close] = ACTIONS(1863),
    [sym_ip_bound_client_open] = ACTIONS(1863),
    [sym_ip_mc_server_open] = ACTIONS(1863),
    [sym_ip_set_option] = ACTIONS(1863),
    [sym_ssh_client_open] = ACTIONS(1863),
    [sym_ssh_client_close] = ACTIONS(1863),
    [sym_tls_client_open] = ACTIONS(1863),
    [sym_tls_client_close] = ACTIONS(1863),
    [sym_do_push] = ACTIONS(1863),
    [sym_do_push_timed] = ACTIONS(1863),
    [sym_do_release] = ACTIONS(1863),
    [sym_device_id] = ACTIONS(1863),
    [sym_device_info] = ACTIONS(1863),
    [sym_device_id_string] = ACTIONS(1863),
    [sym_device_standby] = ACTIONS(1863),
    [sym_device_wake] = ACTIONS(1863),
    [sym_timeline_create] = ACTIONS(1863),
    [sym_timeline_kill] = ACTIONS(1863),
    [sym_timeline_reload] = ACTIONS(1863),
    [sym_timeline_pause] = ACTIONS(1863),
    [sym_timeline_restart] = ACTIONS(1863),
    [sym_timeline_set] = ACTIONS(1863),
    [sym_timeline_get] = ACTIONS(1863),
    [sym_timeline_active] = ACTIONS(1863),
    [sym_timeline_dynamic_id] = ACTIONS(1863),
    [sym_variable_to_string] = ACTIONS(1863),
    [sym_string_to_variable] = ACTIONS(1863),
    [sym_length_variable_to_string] = ACTIONS(1863),
    [sym_type_cast] = ACTIONS(1863),
    [sym_variable_to_xml] = ACTIONS(1863),
    [sym_xml_to_variable] = ACTIONS(1863),
    [sym_length_variable_to_xml] = ACTIONS(1863),
    [sym_redirect_string] = ACTIONS(1863),
    [sym_random_number] = ACTIONS(1863),
    [sym_get_unique_id] = ACTIONS(1863),
    [sym_get_system_number] = ACTIONS(1863),
    [sym_set_system_number] = ACTIONS(1863),
    [sym_get_serial_number] = ACTIONS(1863),
    [sym_get_master_build] = ACTIONS(1863),
    [sym_set_validation_code] = ACTIONS(1863),
    [sym_reboot] = ACTIONS(1863),
    [sym_load_duet_module] = ACTIONS(1863),
    [sym_unload_duet_module] = ACTIONS(1863),
    [sym_duet_mem_size_set] = ACTIONS(1863),
    [sym_duet_mem_size_get] = ACTIONS(1863),
    [sym_dynamic_application_device] = ACTIONS(1863),
    [sym_dynamic_polled_port] = ACTIONS(1863),
    [sym_static_port_binding] = ACTIONS(1863),
    [sym_static_ip_binding] = ACTIONS(1863),
    [sym_get_dns_list] = ACTIONS(1863),
    [sym_set_dns_list] = ACTIONS(1863),
    [sym_get_ip_address] = ACTIONS(1863),
    [sym_set_ip_address] = ACTIONS(1863),
    [sym_get_url_list] = ACTIONS(1863),
    [sym_add_url_entry] = ACTIONS(1863),
    [sym_delete_url_entry] = ACTIONS(1863),
    [sym_abs_value] = ACTIONS(1863),
    [sym_max_value] = ACTIONS(1863),
    [sym_min_value] = ACTIONS(1863),
    [sym_exp_value] = ACTIONS(1863),
    [sym_log_value] = ACTIONS(1863),
    [sym_log10_value] = ACTIONS(1863),
    [sym_power_value] = ACTIONS(1863),
    [sym_sqrt_value] = ACTIONS(1863),
    [sym_raw_be] = ACTIONS(1863),
    [sym_raw_le] = ACTIONS(1863),
    [sym_internal_queue_size_set] = ACTIONS(1863),
    [sym_internal_queue_size_get] = ACTIONS(1863),
    [sym_internal_threshold_set] = ACTIONS(1863),
    [sym_internal_threshold_get] = ACTIONS(1863),
    [sym_rebuilt_event] = ACTIONS(1863),
    [sym_do_custom_event] = ACTIONS(1863),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1863),
    [sym_clkmgr_set_clk_source] = ACTIONS(1863),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1863),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1863),
    [sym_clkmgr_get_timezone] = ACTIONS(1863),
    [sym_clkmgr_set_timezone] = ACTIONS(1863),
    [sym_clkmgr_get_resync_period] = ACTIONS(1863),
    [sym_clkmgr_set_resync_period] = ACTIONS(1863),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1863),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1863),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1863),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1863),
    [sym_clkmgr_get_timeservers] = ACTIONS(1863),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1863),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1863),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1863),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1863),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1863),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1863),
    [sym_set_log_level] = ACTIONS(1863),
    [sym_get_log_level] = ACTIONS(1863),
    [sym_amx_log] = ACTIONS(1863),
    [sym_smtp_server_config_set] = ACTIONS(1863),
    [sym_smtp_server_config_get] = ACTIONS(1863),
    [sym_smtp_send] = ACTIONS(1863),
    [sym_validate_netlinx_account] = ACTIONS(1863),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1863),
    [sym_authenticate_certificate] = ACTIONS(1863),
    [sym_audit_netlinx_session_event] = ACTIONS(1863),
    [sym_audit_netlinx_generic_event] = ACTIONS(1863),
    [sym_get_available_flash_disk_space] = ACTIONS(1863),
    [sym_get_max_flash_disk_space] = ACTIONS(1863),
    [sym_data_create_feed] = ACTIONS(1863),
    [sym_data_delete_feed] = ACTIONS(1863),
    [sym_data_publish_feed] = ACTIONS(1863),
    [sym_data_get_published_feed] = ACTIONS(1863),
    [sym_data_add_record] = ACTIONS(1863),
    [sym_data_get_event_record] = ACTIONS(1863),
    [sym__wc_data_create_feed] = ACTIONS(1863),
    [sym__wc_data_add_record] = ACTIONS(1863),
    [sym__wc_data_get_event_record] = ACTIONS(1863),
    [sym_format] = ACTIONS(1863),
    [sym_astro_clock] = ACTIONS(1863),
    [sym_set_outdoor_temperature] = ACTIONS(1863),
    [sym_set_virtual_level_count] = ACTIONS(1863),
    [sym_set_virtual_channel_count] = ACTIONS(1863),
    [sym_set_virtual_port_count] = ACTIONS(1863),
    [sym__wc_encode_int] = ACTIONS(1863),
    [sym__wc_decode_int] = ACTIONS(1863),
    [sym__wc_int] = ACTIONS(1863),
    [sym__wc_to_ch_int] = ACTIONS(1863),
    [sym__ch_to_wc_int] = ACTIONS(1863),
    [sym__wc_find_string_int] = ACTIONS(1863),
    [sym__wc_left_string_int] = ACTIONS(1863),
    [sym__wc_lower_string_int] = ACTIONS(1863),
    [sym__wc_mid_string_int] = ACTIONS(1863),
    [sym__wc_remove_string_int] = ACTIONS(1863),
    [sym__wc_right_string_int] = ACTIONS(1863),
    [sym__wc_upper_string_int] = ACTIONS(1863),
    [sym__wc_compare_string_int] = ACTIONS(1863),
    [sym__wc_get_buffer_char_int] = ACTIONS(1863),
    [sym__wc_get_buffer_string_int] = ACTIONS(1863),
    [sym__wc_concat_string_int] = ACTIONS(1863),
    [sym__wc_file_open_int] = ACTIONS(1863),
    [sym__wc_file_close_int] = ACTIONS(1863),
    [sym__wc_file_read_int] = ACTIONS(1863),
    [sym__wc_file_read_line_int] = ACTIONS(1863),
    [sym__wc_file_write_int] = ACTIONS(1863),
    [sym__wc_file_write_line_int] = ACTIONS(1863),
    [aux_sym_preproc_include_token1] = ACTIONS(1863),
    [aux_sym_preproc_define_token1] = ACTIONS(1865),
    [aux_sym_preproc_warn_token1] = ACTIONS(1865),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1865),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1865),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1865),
    [anon_sym_LBRACK] = ACTIONS(1865),
    [anon_sym_LBRACE] = ACTIONS(1865),
    [anon_sym_LPAREN] = ACTIONS(1863),
    [anon_sym_BANG] = ACTIONS(1865),
    [anon_sym_TILDE] = ACTIONS(1865),
    [anon_sym_DASH] = ACTIONS(1863),
    [anon_sym_PLUS] = ACTIONS(1863),
    [anon_sym_DASH_DASH] = ACTIONS(1865),
    [anon_sym_PLUS_PLUS] = ACTIONS(1865),
    [anon_sym_DQUOTE] = ACTIONS(1865),
    [anon_sym_SQUOTE] = ACTIONS(1865),
    [sym_number_literal] = ACTIONS(1865),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(315)] = {
    [sym_identifier] = ACTIONS(1997),
    [sym_program_name_keyword] = ACTIONS(1997),
    [sym_module_name_keyword] = ACTIONS(1997),
    [sym_define_device_keyword] = ACTIONS(1997),
    [sym_define_combine_keyword] = ACTIONS(1997),
    [sym_define_constant_keyword] = ACTIONS(1997),
    [sym_define_type_keyword] = ACTIONS(1997),
    [sym_define_variable_keyword] = ACTIONS(1997),
    [sym_define_system_variable_keyword] = ACTIONS(1997),
    [sym_define_connect_level_keyword] = ACTIONS(1997),
    [sym_define_latching_keyword] = ACTIONS(1997),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1997),
    [sym_define_toggling_keyword] = ACTIONS(1997),
    [sym_define_call_keyword] = ACTIONS(1997),
    [sym_define_function_keyword] = ACTIONS(1997),
    [sym_define_library_function_keyword] = ACTIONS(1997),
    [sym_define_start_keyword] = ACTIONS(1997),
    [sym_define_event_keyword] = ACTIONS(1997),
    [sym_define_program_keyword] = ACTIONS(1997),
    [sym_define_module_keyword] = ACTIONS(1997),
    [sym_char_keyword] = ACTIONS(1997),
    [sym_widechar_keyword] = ACTIONS(1997),
    [sym_integer_keyword] = ACTIONS(1997),
    [sym_sinteger_keyword] = ACTIONS(1997),
    [sym_long_keyword] = ACTIONS(1997),
    [sym_slong_keyword] = ACTIONS(1997),
    [sym_float_keyword] = ACTIONS(1997),
    [sym_double_keyword] = ACTIONS(1997),
    [sym_variant_keyword] = ACTIONS(1997),
    [sym_variantarray_keyword] = ACTIONS(1997),
    [sym_local_var_keyword] = ACTIONS(1997),
    [sym_stack_var_keyword] = ACTIONS(1997),
    [sym_constant_keyword] = ACTIONS(1997),
    [sym_volatile_keyword] = ACTIONS(1997),
    [sym_non_volatile_keyword] = ACTIONS(1997),
    [sym_persistent_keyword] = ACTIONS(1997),
    [sym_dev_keyword] = ACTIONS(1997),
    [sym_devlev_keyword] = ACTIONS(1997),
    [sym_devchan_keyword] = ACTIONS(1997),
    [sym_if_keyword] = ACTIONS(1997),
    [sym_switch_keyword] = ACTIONS(1997),
    [sym_case_keyword] = ACTIONS(1997),
    [sym_default_keyword] = ACTIONS(1997),
    [sym_while_keyword] = ACTIONS(1997),
    [sym_for_keyword] = ACTIONS(1997),
    [sym_break_keyword] = ACTIONS(1997),
    [sym_continue_keyword] = ACTIONS(1997),
    [sym_return_keyword] = ACTIONS(1997),
    [sym_select_keyword] = ACTIONS(1997),
    [sym_active_keyword] = ACTIONS(1997),
    [sym_struct_keyword] = ACTIONS(1997),
    [sym_structure_keyword] = ACTIONS(1997),
    [sym_button_event_keyword] = ACTIONS(1997),
    [sym_channel_event_keyword] = ACTIONS(1997),
    [sym_level_event_keyword] = ACTIONS(1997),
    [sym_data_event_keyword] = ACTIONS(1997),
    [sym_timeline_event_keyword] = ACTIONS(1997),
    [sym_custom_event_keyword] = ACTIONS(1997),
    [sym_push_keyword] = ACTIONS(1997),
    [sym_release_keyword] = ACTIONS(1997),
    [aux_sym_on_keyword_token1] = ACTIONS(1997),
    [aux_sym_off_keyword_token1] = ACTIONS(1997),
    [sym_send_level_keyword] = ACTIONS(1997),
    [sym_send_string_keyword] = ACTIONS(1997),
    [sym_send_command_keyword] = ACTIONS(1997),
    [sym_clear_buffer_keyword] = ACTIONS(1997),
    [sym_create_buffer_keyword] = ACTIONS(1997),
    [sym_create_multi_buffer_keyword] = ACTIONS(1997),
    [sym_call_keyword] = ACTIONS(1997),
    [sym_system_call_keyword] = ACTIONS(1997),
    [sym_devchan_to_keyword] = ACTIONS(1997),
    [sym_devchan_min_to_keyword] = ACTIONS(1997),
    [sym_devchan_total_off_keyword] = ACTIONS(1997),
    [sym_devchan_pulse_keyword] = ACTIONS(1997),
    [sym_wait_keyword] = ACTIONS(1997),
    [sym_cancel_wait_keyword] = ACTIONS(1997),
    [sym_wait_until_keyword] = ACTIONS(1997),
    [sym_cancel_wait_until_keyword] = ACTIONS(1997),
    [sym_cancel_all_wait_keyword] = ACTIONS(1997),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1997),
    [sym_not] = ACTIONS(1997),
    [sym_bnot] = ACTIONS(1997),
    [sym___file__] = ACTIONS(1997),
    [sym___date__] = ACTIONS(1997),
    [sym___ldate__] = ACTIONS(1997),
    [sym___time__] = ACTIONS(1997),
    [sym___line__] = ACTIONS(1997),
    [sym___name__] = ACTIONS(1997),
    [sym___netlinx__] = ACTIONS(1997),
    [sym_netlinx_axi_version] = ACTIONS(1997),
    [sym_url_flg_tcp] = ACTIONS(1997),
    [sym_url_flg_accinfopresent] = ACTIONS(1997),
    [sym_url_flg_temp] = ACTIONS(1997),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1997),
    [sym_url_flg_stat_mask] = ACTIONS(1997),
    [sym_url_flg_stat_lookup] = ACTIONS(1997),
    [sym_url_flg_stat_connecting] = ACTIONS(1997),
    [sym_url_flg_stat_waiting] = ACTIONS(1997),
    [sym_url_flg_stat_connected] = ACTIONS(1997),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1997),
    [sym_first_virtual_device] = ACTIONS(1997),
    [sym_first_local_port] = ACTIONS(1997),
    [sym_dynamic_virtual_device] = ACTIONS(1997),
    [sym_timeline_once] = ACTIONS(1997),
    [sym_timeline_repeat] = ACTIONS(1997),
    [sym_timeline_absolute] = ACTIONS(1997),
    [sym_timeline_relative] = ACTIONS(1997),
    [sym_do_push_timed_infinite] = ACTIONS(1997),
    [sym_source_type_no_address] = ACTIONS(1997),
    [sym_source_type_neuron_id] = ACTIONS(1997),
    [sym_source_type_ip_address] = ACTIONS(1997),
    [sym_source_type_axlink] = ACTIONS(1997),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1997),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1997),
    [sym_source_type_ip_socket_address] = ACTIONS(1997),
    [sym_source_type_rs232] = ACTIONS(1997),
    [sym_source_type_internal] = ACTIONS(1997),
    [sym_source_type_mac_address] = ACTIONS(1997),
    [sym_source_type_ipv4_port] = ACTIONS(1997),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1997),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1997),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1997),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1997),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1997),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1997),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1997),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1997),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1997),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1997),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1997),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1997),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1997),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1997),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1997),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1997),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1997),
    [sym_true] = ACTIONS(1997),
    [sym_false] = ACTIONS(1997),
    [sym_file_read_only] = ACTIONS(1997),
    [sym_file_rw_new] = ACTIONS(1997),
    [sym_file_rw_append] = ACTIONS(1997),
    [sym_ip_tcp] = ACTIONS(1997),
    [sym_ip_udp] = ACTIONS(1997),
    [sym_ip_udp_2way] = ACTIONS(1997),
    [sym_xml_encode_types] = ACTIONS(1997),
    [sym_xml_encode_char_as_list] = ACTIONS(1997),
    [sym_xml_encode_le] = ACTIONS(1997),
    [sym_xml_decode_types] = ACTIONS(1997),
    [sym_xml_decode_no_preserve] = ACTIONS(1997),
    [sym_duet_dev_type_display_device] = ACTIONS(1997),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1997),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1997),
    [sym_duet_dev_type_io_device] = ACTIONS(1997),
    [sym_duet_dev_type_relay_device] = ACTIONS(1997),
    [sym_duet_dev_type_ups] = ACTIONS(1997),
    [sym_duet_dev_type_amplifier] = ACTIONS(1997),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1997),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1997),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1997),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1997),
    [sym_duet_dev_type_camera] = ACTIONS(1997),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1997),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1997),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1997),
    [sym_duet_dev_type_dss] = ACTIONS(1997),
    [sym_duet_dev_type_dvr] = ACTIONS(1997),
    [sym_duet_dev_type_disc_device] = ACTIONS(1997),
    [sym_duet_dev_type_document_camera] = ACTIONS(1997),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1997),
    [sym_duet_dev_type_hvac] = ACTIONS(1997),
    [sym_duet_dev_type_keypad] = ACTIONS(1997),
    [sym_duet_dev_type_light] = ACTIONS(1997),
    [sym_duet_dev_type_monitor] = ACTIONS(1997),
    [sym_duet_dev_type_motor] = ACTIONS(1997),
    [sym_duet_dev_type_multi_window] = ACTIONS(1997),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1997),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1997),
    [sym_duet_dev_type_receiver] = ACTIONS(1997),
    [sym_duet_dev_type_security_system] = ACTIONS(1997),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1997),
    [sym_duet_dev_type_settop_box] = ACTIONS(1997),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1997),
    [sym_duet_dev_type_switcher] = ACTIONS(1997),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1997),
    [sym_duet_dev_type_tv] = ACTIONS(1997),
    [sym_duet_dev_type_utility] = ACTIONS(1997),
    [sym_duet_dev_type_vcr] = ACTIONS(1997),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1997),
    [sym_duet_dev_type_video_processor] = ACTIONS(1997),
    [sym_duet_dev_type_video_projector] = ACTIONS(1997),
    [sym_duet_dev_type_video_wall] = ACTIONS(1997),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1997),
    [sym_duet_dev_type_weather] = ACTIONS(1997),
    [sym_duet_dev_not_polled] = ACTIONS(1997),
    [sym_duet_dev_polled] = ACTIONS(1997),
    [sym_clkmgr_mode_network] = ACTIONS(1997),
    [sym_clkmgr_mode_standalone] = ACTIONS(1997),
    [sym_normal_standby] = ACTIONS(1997),
    [sym_normal_wake] = ACTIONS(1997),
    [sym_ip_multicast_ttl_option] = ACTIONS(1997),
    [sym_ip_ttl_subnet] = ACTIONS(1997),
    [sym_ip_ttl_site] = ACTIONS(1997),
    [sym_ip_ttl_region] = ACTIONS(1997),
    [sym_ip_ttl_continent] = ACTIONS(1997),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1997),
    [sym_ip_nodelay_on] = ACTIONS(1997),
    [sym_ip_nodelay_off] = ACTIONS(1997),
    [sym_amx_error] = ACTIONS(1997),
    [sym_amx_warning] = ACTIONS(1997),
    [sym_amx_info] = ACTIONS(1997),
    [sym_amx_debug] = ACTIONS(1997),
    [sym_netlinx_login_success] = ACTIONS(1997),
    [sym_netlinx_login_fail] = ACTIONS(1997),
    [sym_netlinx_logout] = ACTIONS(1997),
    [sym_null_str] = ACTIONS(1997),
    [sym_smtp_address] = ACTIONS(1997),
    [sym_smtp_port_number] = ACTIONS(1997),
    [sym_smtp_username] = ACTIONS(1997),
    [sym_smtp_password] = ACTIONS(1997),
    [sym_smtp_require_tls] = ACTIONS(1997),
    [sym_smtp_from] = ACTIONS(1997),
    [sym_smtp_tls_true] = ACTIONS(1997),
    [sym_smtp_tls_false] = ACTIONS(1997),
    [sym_listview_on_row_select_event] = ACTIONS(1997),
    [sym_data_structure_unknown] = ACTIONS(1997),
    [sym_data_structure_datafeed] = ACTIONS(1997),
    [sym_data_structure_datarecordset] = ACTIONS(1997),
    [sym_data_structure_datarecord] = ACTIONS(1997),
    [sym_data_structure_datafield] = ACTIONS(1997),
    [sym_data_max_content_field_count] = ACTIONS(1997),
    [sym_data_max_metadata_field_count] = ACTIONS(1997),
    [sym_data_max_id_length] = ACTIONS(1997),
    [sym_data_max_name_length] = ACTIONS(1997),
    [sym_data_max_description_length] = ACTIONS(1997),
    [sym_data_max_type_length] = ACTIONS(1997),
    [sym_data_max_format_length] = ACTIONS(1997),
    [sym_data_max_source_length] = ACTIONS(1997),
    [sym_data_max_value_length] = ACTIONS(1997),
    [sym_data_max_label_length] = ACTIONS(1997),
    [sym_data_max_url_length] = ACTIONS(1997),
    [sym_max_last_login_info_length] = ACTIONS(1997),
    [sym_data_type_unknown] = ACTIONS(1997),
    [sym_data_type_string] = ACTIONS(1997),
    [sym_data_type_datetime] = ACTIONS(1997),
    [sym_data_type_date] = ACTIONS(1997),
    [sym_data_type_time] = ACTIONS(1997),
    [sym_data_type_image] = ACTIONS(1997),
    [sym_data_format_url] = ACTIONS(1997),
    [sym_data_format_phone] = ACTIONS(1997),
    [sym_data_format_email] = ACTIONS(1997),
    [sym_data_format_iso8601] = ACTIONS(1997),
    [sym_auditlog_priv] = ACTIONS(1997),
    [sym_configuration_priv] = ACTIONS(1997),
    [sym_deviceconfig_priv] = ACTIONS(1997),
    [sym_ftp_priv] = ACTIONS(1997),
    [sym_http_priv] = ACTIONS(1997),
    [sym_networkconfig_priv] = ACTIONS(1997),
    [sym_programport_priv] = ACTIONS(1997),
    [sym_remoteui_priv] = ACTIONS(1997),
    [sym_securitycontrol_priv] = ACTIONS(1997),
    [sym_softwaremanagement_priv] = ACTIONS(1997),
    [sym_terminal_priv] = ACTIONS(1997),
    [sym_tpadmin_priv] = ACTIONS(1997),
    [sym_user1_priv] = ACTIONS(1997),
    [sym_user2_priv] = ACTIONS(1997),
    [sym_user3_priv] = ACTIONS(1997),
    [sym_user4_priv] = ACTIONS(1997),
    [sym_usermanagement_priv] = ACTIONS(1997),
    [sym_valid_account] = ACTIONS(1997),
    [sym_err_name_invalid] = ACTIONS(1997),
    [sym_err_password_invalid] = ACTIONS(1997),
    [sym_err_invalid_account] = ACTIONS(1997),
    [sym_err_no_authorization] = ACTIONS(1997),
    [sym_err_invalid_parameter] = ACTIONS(1997),
    [sym_err_user_locked_out] = ACTIONS(1997),
    [sym_err_user_account_expired] = ACTIONS(1997),
    [sym_err_general_error] = ACTIONS(1997),
    [sym_err_authentication_error] = ACTIONS(1997),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1997),
    [sym_tls_validate_certificate] = ACTIONS(1997),
    [sym_tdata] = ACTIONS(1997),
    [sym_tchannel] = ACTIONS(1997),
    [sym_tlevel] = ACTIONS(1997),
    [sym_tbutton] = ACTIONS(1997),
    [sym_ttimeline] = ACTIONS(1997),
    [sym_tcustom] = ACTIONS(1997),
    [sym_url_struct] = ACTIONS(1997),
    [sym_dns_struct] = ACTIONS(1997),
    [sym_ip_address_struct] = ACTIONS(1997),
    [sym_dev_info_struct] = ACTIONS(1997),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1997),
    [sym_data_feed] = ACTIONS(1997),
    [sym_data_field] = ACTIONS(1997),
    [sym_data_record] = ACTIONS(1997),
    [sym_wc_data_feed] = ACTIONS(1997),
    [sym_wc_data_field] = ACTIONS(1997),
    [sym_wc_data_record] = ACTIONS(1997),
    [sym_last_login_info] = ACTIONS(1997),
    [sym____reserved___] = ACTIONS(1997),
    [sym_date] = ACTIONS(1997),
    [sym_day] = ACTIONS(1997),
    [sym_time] = ACTIONS(1997),
    [sym_ldate] = ACTIONS(1997),
    [sym_push_device] = ACTIONS(1997),
    [sym_push_channel] = ACTIONS(1997),
    [sym_push_devchan] = ACTIONS(1997),
    [sym_release_device] = ACTIONS(1997),
    [sym_release_channel] = ACTIONS(1997),
    [sym_release_devchan] = ACTIONS(1997),
    [sym_master_slot] = ACTIONS(1997),
    [sym_get_pulse_time] = ACTIONS(1997),
    [sym_get_timer] = ACTIONS(1997),
    [sym_master_sn] = ACTIONS(1997),
    [sym_system_number] = ACTIONS(1997),
    [sym_button] = ACTIONS(1997),
    [sym_dv_channel] = ACTIONS(1997),
    [sym_data] = ACTIONS(1997),
    [sym_level] = ACTIONS(1997),
    [sym_timeline] = ACTIONS(1997),
    [sym_channel] = ACTIONS(1997),
    [sym_custom] = ACTIONS(1997),
    [sym_atoi] = ACTIONS(1997),
    [sym_atol] = ACTIONS(1997),
    [sym_atof] = ACTIONS(1997),
    [sym_itoa] = ACTIONS(1997),
    [sym_itohex] = ACTIONS(1997),
    [sym_hextoi] = ACTIONS(1997),
    [sym_ftoa] = ACTIONS(1997),
    [sym_find_string] = ACTIONS(1997),
    [sym_left_string] = ACTIONS(1997),
    [sym_right_string] = ACTIONS(1997),
    [sym_mid_string] = ACTIONS(1997),
    [sym_remove_string] = ACTIONS(1997),
    [sym_get_buffer_string] = ACTIONS(1997),
    [sym_get_buffer_char] = ACTIONS(1997),
    [sym_get_multi_buffer_string] = ACTIONS(1997),
    [sym_length_string] = ACTIONS(1997),
    [sym_max_length_string] = ACTIONS(1997),
    [sym_set_length_string] = ACTIONS(1997),
    [sym_compare_string] = ACTIONS(1997),
    [sym_upper_string] = ACTIONS(1997),
    [sym_lower_string] = ACTIONS(1997),
    [sym_get_last] = ACTIONS(1997),
    [sym_date_to_day] = ACTIONS(1997),
    [sym_date_to_month] = ACTIONS(1997),
    [sym_date_to_year] = ACTIONS(1997),
    [sym_time_to_hour] = ACTIONS(1997),
    [sym_time_to_minute] = ACTIONS(1997),
    [sym_time_to_second] = ACTIONS(1997),
    [sym_day_of_week] = ACTIONS(1997),
    [sym_set_pulse_time] = ACTIONS(1997),
    [sym_set_timer] = ACTIONS(1997),
    [sym_file_open] = ACTIONS(1997),
    [sym_file_close] = ACTIONS(1997),
    [sym_file_read] = ACTIONS(1997),
    [sym_file_read_line] = ACTIONS(1997),
    [sym_file_write] = ACTIONS(1997),
    [sym_file_write_line] = ACTIONS(1997),
    [sym_file_copy] = ACTIONS(1997),
    [sym_file_delete] = ACTIONS(1997),
    [sym_file_rename] = ACTIONS(1997),
    [sym_file_dir] = ACTIONS(1997),
    [sym_file_getdir] = ACTIONS(1997),
    [sym_file_setdir] = ACTIONS(1997),
    [sym_file_create_dir] = ACTIONS(1997),
    [sym_file_remove_dir] = ACTIONS(1997),
    [sym_file_seek] = ACTIONS(1997),
    [sym_length_array] = ACTIONS(1997),
    [sym_max_length_array] = ACTIONS(1997),
    [sym_set_length_array] = ACTIONS(1997),
    [sym_ip_client_open] = ACTIONS(1997),
    [sym_ip_client_close] = ACTIONS(1997),
    [sym_ip_server_open] = ACTIONS(1997),
    [sym_ip_server_close] = ACTIONS(1997),
    [sym_ip_bound_client_open] = ACTIONS(1997),
    [sym_ip_mc_server_open] = ACTIONS(1997),
    [sym_ip_set_option] = ACTIONS(1997),
    [sym_ssh_client_open] = ACTIONS(1997),
    [sym_ssh_client_close] = ACTIONS(1997),
    [sym_tls_client_open] = ACTIONS(1997),
    [sym_tls_client_close] = ACTIONS(1997),
    [sym_do_push] = ACTIONS(1997),
    [sym_do_push_timed] = ACTIONS(1997),
    [sym_do_release] = ACTIONS(1997),
    [sym_device_id] = ACTIONS(1997),
    [sym_device_info] = ACTIONS(1997),
    [sym_device_id_string] = ACTIONS(1997),
    [sym_device_standby] = ACTIONS(1997),
    [sym_device_wake] = ACTIONS(1997),
    [sym_timeline_create] = ACTIONS(1997),
    [sym_timeline_kill] = ACTIONS(1997),
    [sym_timeline_reload] = ACTIONS(1997),
    [sym_timeline_pause] = ACTIONS(1997),
    [sym_timeline_restart] = ACTIONS(1997),
    [sym_timeline_set] = ACTIONS(1997),
    [sym_timeline_get] = ACTIONS(1997),
    [sym_timeline_active] = ACTIONS(1997),
    [sym_timeline_dynamic_id] = ACTIONS(1997),
    [sym_variable_to_string] = ACTIONS(1997),
    [sym_string_to_variable] = ACTIONS(1997),
    [sym_length_variable_to_string] = ACTIONS(1997),
    [sym_type_cast] = ACTIONS(1997),
    [sym_variable_to_xml] = ACTIONS(1997),
    [sym_xml_to_variable] = ACTIONS(1997),
    [sym_length_variable_to_xml] = ACTIONS(1997),
    [sym_redirect_string] = ACTIONS(1997),
    [sym_random_number] = ACTIONS(1997),
    [sym_get_unique_id] = ACTIONS(1997),
    [sym_get_system_number] = ACTIONS(1997),
    [sym_set_system_number] = ACTIONS(1997),
    [sym_get_serial_number] = ACTIONS(1997),
    [sym_get_master_build] = ACTIONS(1997),
    [sym_set_validation_code] = ACTIONS(1997),
    [sym_reboot] = ACTIONS(1997),
    [sym_load_duet_module] = ACTIONS(1997),
    [sym_unload_duet_module] = ACTIONS(1997),
    [sym_duet_mem_size_set] = ACTIONS(1997),
    [sym_duet_mem_size_get] = ACTIONS(1997),
    [sym_dynamic_application_device] = ACTIONS(1997),
    [sym_dynamic_polled_port] = ACTIONS(1997),
    [sym_static_port_binding] = ACTIONS(1997),
    [sym_static_ip_binding] = ACTIONS(1997),
    [sym_get_dns_list] = ACTIONS(1997),
    [sym_set_dns_list] = ACTIONS(1997),
    [sym_get_ip_address] = ACTIONS(1997),
    [sym_set_ip_address] = ACTIONS(1997),
    [sym_get_url_list] = ACTIONS(1997),
    [sym_add_url_entry] = ACTIONS(1997),
    [sym_delete_url_entry] = ACTIONS(1997),
    [sym_abs_value] = ACTIONS(1997),
    [sym_max_value] = ACTIONS(1997),
    [sym_min_value] = ACTIONS(1997),
    [sym_exp_value] = ACTIONS(1997),
    [sym_log_value] = ACTIONS(1997),
    [sym_log10_value] = ACTIONS(1997),
    [sym_power_value] = ACTIONS(1997),
    [sym_sqrt_value] = ACTIONS(1997),
    [sym_raw_be] = ACTIONS(1997),
    [sym_raw_le] = ACTIONS(1997),
    [sym_internal_queue_size_set] = ACTIONS(1997),
    [sym_internal_queue_size_get] = ACTIONS(1997),
    [sym_internal_threshold_set] = ACTIONS(1997),
    [sym_internal_threshold_get] = ACTIONS(1997),
    [sym_rebuilt_event] = ACTIONS(1997),
    [sym_do_custom_event] = ACTIONS(1997),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1997),
    [sym_clkmgr_set_clk_source] = ACTIONS(1997),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1997),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1997),
    [sym_clkmgr_get_timezone] = ACTIONS(1997),
    [sym_clkmgr_set_timezone] = ACTIONS(1997),
    [sym_clkmgr_get_resync_period] = ACTIONS(1997),
    [sym_clkmgr_set_resync_period] = ACTIONS(1997),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1997),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1997),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1997),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1997),
    [sym_clkmgr_get_timeservers] = ACTIONS(1997),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1997),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1997),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1997),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1997),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1997),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1997),
    [sym_set_log_level] = ACTIONS(1997),
    [sym_get_log_level] = ACTIONS(1997),
    [sym_amx_log] = ACTIONS(1997),
    [sym_smtp_server_config_set] = ACTIONS(1997),
    [sym_smtp_server_config_get] = ACTIONS(1997),
    [sym_smtp_send] = ACTIONS(1997),
    [sym_validate_netlinx_account] = ACTIONS(1997),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1997),
    [sym_authenticate_certificate] = ACTIONS(1997),
    [sym_audit_netlinx_session_event] = ACTIONS(1997),
    [sym_audit_netlinx_generic_event] = ACTIONS(1997),
    [sym_get_available_flash_disk_space] = ACTIONS(1997),
    [sym_get_max_flash_disk_space] = ACTIONS(1997),
    [sym_data_create_feed] = ACTIONS(1997),
    [sym_data_delete_feed] = ACTIONS(1997),
    [sym_data_publish_feed] = ACTIONS(1997),
    [sym_data_get_published_feed] = ACTIONS(1997),
    [sym_data_add_record] = ACTIONS(1997),
    [sym_data_get_event_record] = ACTIONS(1997),
    [sym__wc_data_create_feed] = ACTIONS(1997),
    [sym__wc_data_add_record] = ACTIONS(1997),
    [sym__wc_data_get_event_record] = ACTIONS(1997),
    [sym_format] = ACTIONS(1997),
    [sym_astro_clock] = ACTIONS(1997),
    [sym_set_outdoor_temperature] = ACTIONS(1997),
    [sym_set_virtual_level_count] = ACTIONS(1997),
    [sym_set_virtual_channel_count] = ACTIONS(1997),
    [sym_set_virtual_port_count] = ACTIONS(1997),
    [sym__wc_encode_int] = ACTIONS(1997),
    [sym__wc_decode_int] = ACTIONS(1997),
    [sym__wc_int] = ACTIONS(1997),
    [sym__wc_to_ch_int] = ACTIONS(1997),
    [sym__ch_to_wc_int] = ACTIONS(1997),
    [sym__wc_find_string_int] = ACTIONS(1997),
    [sym__wc_left_string_int] = ACTIONS(1997),
    [sym__wc_lower_string_int] = ACTIONS(1997),
    [sym__wc_mid_string_int] = ACTIONS(1997),
    [sym__wc_remove_string_int] = ACTIONS(1997),
    [sym__wc_right_string_int] = ACTIONS(1997),
    [sym__wc_upper_string_int] = ACTIONS(1997),
    [sym__wc_compare_string_int] = ACTIONS(1997),
    [sym__wc_get_buffer_char_int] = ACTIONS(1997),
    [sym__wc_get_buffer_string_int] = ACTIONS(1997),
    [sym__wc_concat_string_int] = ACTIONS(1997),
    [sym__wc_file_open_int] = ACTIONS(1997),
    [sym__wc_file_close_int] = ACTIONS(1997),
    [sym__wc_file_read_int] = ACTIONS(1997),
    [sym__wc_file_read_line_int] = ACTIONS(1997),
    [sym__wc_file_write_int] = ACTIONS(1997),
    [sym__wc_file_write_line_int] = ACTIONS(1997),
    [aux_sym_preproc_include_token1] = ACTIONS(1997),
    [aux_sym_preproc_define_token1] = ACTIONS(1999),
    [aux_sym_preproc_warn_token1] = ACTIONS(1999),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1999),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1999),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1999),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1999),
    [aux_sym_preproc_else_token1] = ACTIONS(1999),
    [anon_sym_LBRACK] = ACTIONS(1999),
    [anon_sym_LBRACE] = ACTIONS(1999),
    [anon_sym_LPAREN] = ACTIONS(1997),
    [anon_sym_BANG] = ACTIONS(1999),
    [anon_sym_TILDE] = ACTIONS(1999),
    [anon_sym_DASH] = ACTIONS(1997),
    [anon_sym_PLUS] = ACTIONS(1997),
    [anon_sym_DASH_DASH] = ACTIONS(1999),
    [anon_sym_PLUS_PLUS] = ACTIONS(1999),
    [anon_sym_DQUOTE] = ACTIONS(1999),
    [anon_sym_SQUOTE] = ACTIONS(1999),
    [sym_number_literal] = ACTIONS(1999),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(316)] = {
    [sym_identifier] = ACTIONS(2001),
    [sym_program_name_keyword] = ACTIONS(2001),
    [sym_module_name_keyword] = ACTIONS(2001),
    [sym_define_device_keyword] = ACTIONS(2001),
    [sym_define_combine_keyword] = ACTIONS(2001),
    [sym_define_constant_keyword] = ACTIONS(2001),
    [sym_define_type_keyword] = ACTIONS(2001),
    [sym_define_variable_keyword] = ACTIONS(2001),
    [sym_define_system_variable_keyword] = ACTIONS(2001),
    [sym_define_connect_level_keyword] = ACTIONS(2001),
    [sym_define_latching_keyword] = ACTIONS(2001),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2001),
    [sym_define_toggling_keyword] = ACTIONS(2001),
    [sym_define_call_keyword] = ACTIONS(2001),
    [sym_define_function_keyword] = ACTIONS(2001),
    [sym_define_library_function_keyword] = ACTIONS(2001),
    [sym_define_start_keyword] = ACTIONS(2001),
    [sym_define_event_keyword] = ACTIONS(2001),
    [sym_define_program_keyword] = ACTIONS(2001),
    [sym_define_module_keyword] = ACTIONS(2001),
    [sym_char_keyword] = ACTIONS(2001),
    [sym_widechar_keyword] = ACTIONS(2001),
    [sym_integer_keyword] = ACTIONS(2001),
    [sym_sinteger_keyword] = ACTIONS(2001),
    [sym_long_keyword] = ACTIONS(2001),
    [sym_slong_keyword] = ACTIONS(2001),
    [sym_float_keyword] = ACTIONS(2001),
    [sym_double_keyword] = ACTIONS(2001),
    [sym_variant_keyword] = ACTIONS(2001),
    [sym_variantarray_keyword] = ACTIONS(2001),
    [sym_local_var_keyword] = ACTIONS(2001),
    [sym_stack_var_keyword] = ACTIONS(2001),
    [sym_constant_keyword] = ACTIONS(2001),
    [sym_volatile_keyword] = ACTIONS(2001),
    [sym_non_volatile_keyword] = ACTIONS(2001),
    [sym_persistent_keyword] = ACTIONS(2001),
    [sym_dev_keyword] = ACTIONS(2001),
    [sym_devlev_keyword] = ACTIONS(2001),
    [sym_devchan_keyword] = ACTIONS(2001),
    [sym_if_keyword] = ACTIONS(2001),
    [sym_switch_keyword] = ACTIONS(2001),
    [sym_case_keyword] = ACTIONS(2001),
    [sym_default_keyword] = ACTIONS(2001),
    [sym_while_keyword] = ACTIONS(2001),
    [sym_for_keyword] = ACTIONS(2001),
    [sym_break_keyword] = ACTIONS(2001),
    [sym_continue_keyword] = ACTIONS(2001),
    [sym_return_keyword] = ACTIONS(2001),
    [sym_select_keyword] = ACTIONS(2001),
    [sym_active_keyword] = ACTIONS(2001),
    [sym_struct_keyword] = ACTIONS(2001),
    [sym_structure_keyword] = ACTIONS(2001),
    [sym_button_event_keyword] = ACTIONS(2001),
    [sym_channel_event_keyword] = ACTIONS(2001),
    [sym_level_event_keyword] = ACTIONS(2001),
    [sym_data_event_keyword] = ACTIONS(2001),
    [sym_timeline_event_keyword] = ACTIONS(2001),
    [sym_custom_event_keyword] = ACTIONS(2001),
    [sym_push_keyword] = ACTIONS(2001),
    [sym_release_keyword] = ACTIONS(2001),
    [aux_sym_on_keyword_token1] = ACTIONS(2001),
    [aux_sym_off_keyword_token1] = ACTIONS(2001),
    [sym_send_level_keyword] = ACTIONS(2001),
    [sym_send_string_keyword] = ACTIONS(2001),
    [sym_send_command_keyword] = ACTIONS(2001),
    [sym_clear_buffer_keyword] = ACTIONS(2001),
    [sym_create_buffer_keyword] = ACTIONS(2001),
    [sym_create_multi_buffer_keyword] = ACTIONS(2001),
    [sym_call_keyword] = ACTIONS(2001),
    [sym_system_call_keyword] = ACTIONS(2001),
    [sym_devchan_to_keyword] = ACTIONS(2001),
    [sym_devchan_min_to_keyword] = ACTIONS(2001),
    [sym_devchan_total_off_keyword] = ACTIONS(2001),
    [sym_devchan_pulse_keyword] = ACTIONS(2001),
    [sym_wait_keyword] = ACTIONS(2001),
    [sym_cancel_wait_keyword] = ACTIONS(2001),
    [sym_wait_until_keyword] = ACTIONS(2001),
    [sym_cancel_wait_until_keyword] = ACTIONS(2001),
    [sym_cancel_all_wait_keyword] = ACTIONS(2001),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2001),
    [sym_not] = ACTIONS(2001),
    [sym_bnot] = ACTIONS(2001),
    [sym___file__] = ACTIONS(2001),
    [sym___date__] = ACTIONS(2001),
    [sym___ldate__] = ACTIONS(2001),
    [sym___time__] = ACTIONS(2001),
    [sym___line__] = ACTIONS(2001),
    [sym___name__] = ACTIONS(2001),
    [sym___netlinx__] = ACTIONS(2001),
    [sym_netlinx_axi_version] = ACTIONS(2001),
    [sym_url_flg_tcp] = ACTIONS(2001),
    [sym_url_flg_accinfopresent] = ACTIONS(2001),
    [sym_url_flg_temp] = ACTIONS(2001),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2001),
    [sym_url_flg_stat_mask] = ACTIONS(2001),
    [sym_url_flg_stat_lookup] = ACTIONS(2001),
    [sym_url_flg_stat_connecting] = ACTIONS(2001),
    [sym_url_flg_stat_waiting] = ACTIONS(2001),
    [sym_url_flg_stat_connected] = ACTIONS(2001),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2001),
    [sym_first_virtual_device] = ACTIONS(2001),
    [sym_first_local_port] = ACTIONS(2001),
    [sym_dynamic_virtual_device] = ACTIONS(2001),
    [sym_timeline_once] = ACTIONS(2001),
    [sym_timeline_repeat] = ACTIONS(2001),
    [sym_timeline_absolute] = ACTIONS(2001),
    [sym_timeline_relative] = ACTIONS(2001),
    [sym_do_push_timed_infinite] = ACTIONS(2001),
    [sym_source_type_no_address] = ACTIONS(2001),
    [sym_source_type_neuron_id] = ACTIONS(2001),
    [sym_source_type_ip_address] = ACTIONS(2001),
    [sym_source_type_axlink] = ACTIONS(2001),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2001),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2001),
    [sym_source_type_ip_socket_address] = ACTIONS(2001),
    [sym_source_type_rs232] = ACTIONS(2001),
    [sym_source_type_internal] = ACTIONS(2001),
    [sym_source_type_mac_address] = ACTIONS(2001),
    [sym_source_type_ipv4_port] = ACTIONS(2001),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2001),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2001),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2001),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2001),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2001),
    [sym_true] = ACTIONS(2001),
    [sym_false] = ACTIONS(2001),
    [sym_file_read_only] = ACTIONS(2001),
    [sym_file_rw_new] = ACTIONS(2001),
    [sym_file_rw_append] = ACTIONS(2001),
    [sym_ip_tcp] = ACTIONS(2001),
    [sym_ip_udp] = ACTIONS(2001),
    [sym_ip_udp_2way] = ACTIONS(2001),
    [sym_xml_encode_types] = ACTIONS(2001),
    [sym_xml_encode_char_as_list] = ACTIONS(2001),
    [sym_xml_encode_le] = ACTIONS(2001),
    [sym_xml_decode_types] = ACTIONS(2001),
    [sym_xml_decode_no_preserve] = ACTIONS(2001),
    [sym_duet_dev_type_display_device] = ACTIONS(2001),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2001),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2001),
    [sym_duet_dev_type_io_device] = ACTIONS(2001),
    [sym_duet_dev_type_relay_device] = ACTIONS(2001),
    [sym_duet_dev_type_ups] = ACTIONS(2001),
    [sym_duet_dev_type_amplifier] = ACTIONS(2001),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2001),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2001),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2001),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2001),
    [sym_duet_dev_type_camera] = ACTIONS(2001),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2001),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2001),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2001),
    [sym_duet_dev_type_dss] = ACTIONS(2001),
    [sym_duet_dev_type_dvr] = ACTIONS(2001),
    [sym_duet_dev_type_disc_device] = ACTIONS(2001),
    [sym_duet_dev_type_document_camera] = ACTIONS(2001),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2001),
    [sym_duet_dev_type_hvac] = ACTIONS(2001),
    [sym_duet_dev_type_keypad] = ACTIONS(2001),
    [sym_duet_dev_type_light] = ACTIONS(2001),
    [sym_duet_dev_type_monitor] = ACTIONS(2001),
    [sym_duet_dev_type_motor] = ACTIONS(2001),
    [sym_duet_dev_type_multi_window] = ACTIONS(2001),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2001),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2001),
    [sym_duet_dev_type_receiver] = ACTIONS(2001),
    [sym_duet_dev_type_security_system] = ACTIONS(2001),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2001),
    [sym_duet_dev_type_settop_box] = ACTIONS(2001),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2001),
    [sym_duet_dev_type_switcher] = ACTIONS(2001),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2001),
    [sym_duet_dev_type_tv] = ACTIONS(2001),
    [sym_duet_dev_type_utility] = ACTIONS(2001),
    [sym_duet_dev_type_vcr] = ACTIONS(2001),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2001),
    [sym_duet_dev_type_video_processor] = ACTIONS(2001),
    [sym_duet_dev_type_video_projector] = ACTIONS(2001),
    [sym_duet_dev_type_video_wall] = ACTIONS(2001),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2001),
    [sym_duet_dev_type_weather] = ACTIONS(2001),
    [sym_duet_dev_not_polled] = ACTIONS(2001),
    [sym_duet_dev_polled] = ACTIONS(2001),
    [sym_clkmgr_mode_network] = ACTIONS(2001),
    [sym_clkmgr_mode_standalone] = ACTIONS(2001),
    [sym_normal_standby] = ACTIONS(2001),
    [sym_normal_wake] = ACTIONS(2001),
    [sym_ip_multicast_ttl_option] = ACTIONS(2001),
    [sym_ip_ttl_subnet] = ACTIONS(2001),
    [sym_ip_ttl_site] = ACTIONS(2001),
    [sym_ip_ttl_region] = ACTIONS(2001),
    [sym_ip_ttl_continent] = ACTIONS(2001),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2001),
    [sym_ip_nodelay_on] = ACTIONS(2001),
    [sym_ip_nodelay_off] = ACTIONS(2001),
    [sym_amx_error] = ACTIONS(2001),
    [sym_amx_warning] = ACTIONS(2001),
    [sym_amx_info] = ACTIONS(2001),
    [sym_amx_debug] = ACTIONS(2001),
    [sym_netlinx_login_success] = ACTIONS(2001),
    [sym_netlinx_login_fail] = ACTIONS(2001),
    [sym_netlinx_logout] = ACTIONS(2001),
    [sym_null_str] = ACTIONS(2001),
    [sym_smtp_address] = ACTIONS(2001),
    [sym_smtp_port_number] = ACTIONS(2001),
    [sym_smtp_username] = ACTIONS(2001),
    [sym_smtp_password] = ACTIONS(2001),
    [sym_smtp_require_tls] = ACTIONS(2001),
    [sym_smtp_from] = ACTIONS(2001),
    [sym_smtp_tls_true] = ACTIONS(2001),
    [sym_smtp_tls_false] = ACTIONS(2001),
    [sym_listview_on_row_select_event] = ACTIONS(2001),
    [sym_data_structure_unknown] = ACTIONS(2001),
    [sym_data_structure_datafeed] = ACTIONS(2001),
    [sym_data_structure_datarecordset] = ACTIONS(2001),
    [sym_data_structure_datarecord] = ACTIONS(2001),
    [sym_data_structure_datafield] = ACTIONS(2001),
    [sym_data_max_content_field_count] = ACTIONS(2001),
    [sym_data_max_metadata_field_count] = ACTIONS(2001),
    [sym_data_max_id_length] = ACTIONS(2001),
    [sym_data_max_name_length] = ACTIONS(2001),
    [sym_data_max_description_length] = ACTIONS(2001),
    [sym_data_max_type_length] = ACTIONS(2001),
    [sym_data_max_format_length] = ACTIONS(2001),
    [sym_data_max_source_length] = ACTIONS(2001),
    [sym_data_max_value_length] = ACTIONS(2001),
    [sym_data_max_label_length] = ACTIONS(2001),
    [sym_data_max_url_length] = ACTIONS(2001),
    [sym_max_last_login_info_length] = ACTIONS(2001),
    [sym_data_type_unknown] = ACTIONS(2001),
    [sym_data_type_string] = ACTIONS(2001),
    [sym_data_type_datetime] = ACTIONS(2001),
    [sym_data_type_date] = ACTIONS(2001),
    [sym_data_type_time] = ACTIONS(2001),
    [sym_data_type_image] = ACTIONS(2001),
    [sym_data_format_url] = ACTIONS(2001),
    [sym_data_format_phone] = ACTIONS(2001),
    [sym_data_format_email] = ACTIONS(2001),
    [sym_data_format_iso8601] = ACTIONS(2001),
    [sym_auditlog_priv] = ACTIONS(2001),
    [sym_configuration_priv] = ACTIONS(2001),
    [sym_deviceconfig_priv] = ACTIONS(2001),
    [sym_ftp_priv] = ACTIONS(2001),
    [sym_http_priv] = ACTIONS(2001),
    [sym_networkconfig_priv] = ACTIONS(2001),
    [sym_programport_priv] = ACTIONS(2001),
    [sym_remoteui_priv] = ACTIONS(2001),
    [sym_securitycontrol_priv] = ACTIONS(2001),
    [sym_softwaremanagement_priv] = ACTIONS(2001),
    [sym_terminal_priv] = ACTIONS(2001),
    [sym_tpadmin_priv] = ACTIONS(2001),
    [sym_user1_priv] = ACTIONS(2001),
    [sym_user2_priv] = ACTIONS(2001),
    [sym_user3_priv] = ACTIONS(2001),
    [sym_user4_priv] = ACTIONS(2001),
    [sym_usermanagement_priv] = ACTIONS(2001),
    [sym_valid_account] = ACTIONS(2001),
    [sym_err_name_invalid] = ACTIONS(2001),
    [sym_err_password_invalid] = ACTIONS(2001),
    [sym_err_invalid_account] = ACTIONS(2001),
    [sym_err_no_authorization] = ACTIONS(2001),
    [sym_err_invalid_parameter] = ACTIONS(2001),
    [sym_err_user_locked_out] = ACTIONS(2001),
    [sym_err_user_account_expired] = ACTIONS(2001),
    [sym_err_general_error] = ACTIONS(2001),
    [sym_err_authentication_error] = ACTIONS(2001),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2001),
    [sym_tls_validate_certificate] = ACTIONS(2001),
    [sym_tdata] = ACTIONS(2001),
    [sym_tchannel] = ACTIONS(2001),
    [sym_tlevel] = ACTIONS(2001),
    [sym_tbutton] = ACTIONS(2001),
    [sym_ttimeline] = ACTIONS(2001),
    [sym_tcustom] = ACTIONS(2001),
    [sym_url_struct] = ACTIONS(2001),
    [sym_dns_struct] = ACTIONS(2001),
    [sym_ip_address_struct] = ACTIONS(2001),
    [sym_dev_info_struct] = ACTIONS(2001),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2001),
    [sym_data_feed] = ACTIONS(2001),
    [sym_data_field] = ACTIONS(2001),
    [sym_data_record] = ACTIONS(2001),
    [sym_wc_data_feed] = ACTIONS(2001),
    [sym_wc_data_field] = ACTIONS(2001),
    [sym_wc_data_record] = ACTIONS(2001),
    [sym_last_login_info] = ACTIONS(2001),
    [sym____reserved___] = ACTIONS(2001),
    [sym_date] = ACTIONS(2001),
    [sym_day] = ACTIONS(2001),
    [sym_time] = ACTIONS(2001),
    [sym_ldate] = ACTIONS(2001),
    [sym_push_device] = ACTIONS(2001),
    [sym_push_channel] = ACTIONS(2001),
    [sym_push_devchan] = ACTIONS(2001),
    [sym_release_device] = ACTIONS(2001),
    [sym_release_channel] = ACTIONS(2001),
    [sym_release_devchan] = ACTIONS(2001),
    [sym_master_slot] = ACTIONS(2001),
    [sym_get_pulse_time] = ACTIONS(2001),
    [sym_get_timer] = ACTIONS(2001),
    [sym_master_sn] = ACTIONS(2001),
    [sym_system_number] = ACTIONS(2001),
    [sym_button] = ACTIONS(2001),
    [sym_dv_channel] = ACTIONS(2001),
    [sym_data] = ACTIONS(2001),
    [sym_level] = ACTIONS(2001),
    [sym_timeline] = ACTIONS(2001),
    [sym_channel] = ACTIONS(2001),
    [sym_custom] = ACTIONS(2001),
    [sym_atoi] = ACTIONS(2001),
    [sym_atol] = ACTIONS(2001),
    [sym_atof] = ACTIONS(2001),
    [sym_itoa] = ACTIONS(2001),
    [sym_itohex] = ACTIONS(2001),
    [sym_hextoi] = ACTIONS(2001),
    [sym_ftoa] = ACTIONS(2001),
    [sym_find_string] = ACTIONS(2001),
    [sym_left_string] = ACTIONS(2001),
    [sym_right_string] = ACTIONS(2001),
    [sym_mid_string] = ACTIONS(2001),
    [sym_remove_string] = ACTIONS(2001),
    [sym_get_buffer_string] = ACTIONS(2001),
    [sym_get_buffer_char] = ACTIONS(2001),
    [sym_get_multi_buffer_string] = ACTIONS(2001),
    [sym_length_string] = ACTIONS(2001),
    [sym_max_length_string] = ACTIONS(2001),
    [sym_set_length_string] = ACTIONS(2001),
    [sym_compare_string] = ACTIONS(2001),
    [sym_upper_string] = ACTIONS(2001),
    [sym_lower_string] = ACTIONS(2001),
    [sym_get_last] = ACTIONS(2001),
    [sym_date_to_day] = ACTIONS(2001),
    [sym_date_to_month] = ACTIONS(2001),
    [sym_date_to_year] = ACTIONS(2001),
    [sym_time_to_hour] = ACTIONS(2001),
    [sym_time_to_minute] = ACTIONS(2001),
    [sym_time_to_second] = ACTIONS(2001),
    [sym_day_of_week] = ACTIONS(2001),
    [sym_set_pulse_time] = ACTIONS(2001),
    [sym_set_timer] = ACTIONS(2001),
    [sym_file_open] = ACTIONS(2001),
    [sym_file_close] = ACTIONS(2001),
    [sym_file_read] = ACTIONS(2001),
    [sym_file_read_line] = ACTIONS(2001),
    [sym_file_write] = ACTIONS(2001),
    [sym_file_write_line] = ACTIONS(2001),
    [sym_file_copy] = ACTIONS(2001),
    [sym_file_delete] = ACTIONS(2001),
    [sym_file_rename] = ACTIONS(2001),
    [sym_file_dir] = ACTIONS(2001),
    [sym_file_getdir] = ACTIONS(2001),
    [sym_file_setdir] = ACTIONS(2001),
    [sym_file_create_dir] = ACTIONS(2001),
    [sym_file_remove_dir] = ACTIONS(2001),
    [sym_file_seek] = ACTIONS(2001),
    [sym_length_array] = ACTIONS(2001),
    [sym_max_length_array] = ACTIONS(2001),
    [sym_set_length_array] = ACTIONS(2001),
    [sym_ip_client_open] = ACTIONS(2001),
    [sym_ip_client_close] = ACTIONS(2001),
    [sym_ip_server_open] = ACTIONS(2001),
    [sym_ip_server_close] = ACTIONS(2001),
    [sym_ip_bound_client_open] = ACTIONS(2001),
    [sym_ip_mc_server_open] = ACTIONS(2001),
    [sym_ip_set_option] = ACTIONS(2001),
    [sym_ssh_client_open] = ACTIONS(2001),
    [sym_ssh_client_close] = ACTIONS(2001),
    [sym_tls_client_open] = ACTIONS(2001),
    [sym_tls_client_close] = ACTIONS(2001),
    [sym_do_push] = ACTIONS(2001),
    [sym_do_push_timed] = ACTIONS(2001),
    [sym_do_release] = ACTIONS(2001),
    [sym_device_id] = ACTIONS(2001),
    [sym_device_info] = ACTIONS(2001),
    [sym_device_id_string] = ACTIONS(2001),
    [sym_device_standby] = ACTIONS(2001),
    [sym_device_wake] = ACTIONS(2001),
    [sym_timeline_create] = ACTIONS(2001),
    [sym_timeline_kill] = ACTIONS(2001),
    [sym_timeline_reload] = ACTIONS(2001),
    [sym_timeline_pause] = ACTIONS(2001),
    [sym_timeline_restart] = ACTIONS(2001),
    [sym_timeline_set] = ACTIONS(2001),
    [sym_timeline_get] = ACTIONS(2001),
    [sym_timeline_active] = ACTIONS(2001),
    [sym_timeline_dynamic_id] = ACTIONS(2001),
    [sym_variable_to_string] = ACTIONS(2001),
    [sym_string_to_variable] = ACTIONS(2001),
    [sym_length_variable_to_string] = ACTIONS(2001),
    [sym_type_cast] = ACTIONS(2001),
    [sym_variable_to_xml] = ACTIONS(2001),
    [sym_xml_to_variable] = ACTIONS(2001),
    [sym_length_variable_to_xml] = ACTIONS(2001),
    [sym_redirect_string] = ACTIONS(2001),
    [sym_random_number] = ACTIONS(2001),
    [sym_get_unique_id] = ACTIONS(2001),
    [sym_get_system_number] = ACTIONS(2001),
    [sym_set_system_number] = ACTIONS(2001),
    [sym_get_serial_number] = ACTIONS(2001),
    [sym_get_master_build] = ACTIONS(2001),
    [sym_set_validation_code] = ACTIONS(2001),
    [sym_reboot] = ACTIONS(2001),
    [sym_load_duet_module] = ACTIONS(2001),
    [sym_unload_duet_module] = ACTIONS(2001),
    [sym_duet_mem_size_set] = ACTIONS(2001),
    [sym_duet_mem_size_get] = ACTIONS(2001),
    [sym_dynamic_application_device] = ACTIONS(2001),
    [sym_dynamic_polled_port] = ACTIONS(2001),
    [sym_static_port_binding] = ACTIONS(2001),
    [sym_static_ip_binding] = ACTIONS(2001),
    [sym_get_dns_list] = ACTIONS(2001),
    [sym_set_dns_list] = ACTIONS(2001),
    [sym_get_ip_address] = ACTIONS(2001),
    [sym_set_ip_address] = ACTIONS(2001),
    [sym_get_url_list] = ACTIONS(2001),
    [sym_add_url_entry] = ACTIONS(2001),
    [sym_delete_url_entry] = ACTIONS(2001),
    [sym_abs_value] = ACTIONS(2001),
    [sym_max_value] = ACTIONS(2001),
    [sym_min_value] = ACTIONS(2001),
    [sym_exp_value] = ACTIONS(2001),
    [sym_log_value] = ACTIONS(2001),
    [sym_log10_value] = ACTIONS(2001),
    [sym_power_value] = ACTIONS(2001),
    [sym_sqrt_value] = ACTIONS(2001),
    [sym_raw_be] = ACTIONS(2001),
    [sym_raw_le] = ACTIONS(2001),
    [sym_internal_queue_size_set] = ACTIONS(2001),
    [sym_internal_queue_size_get] = ACTIONS(2001),
    [sym_internal_threshold_set] = ACTIONS(2001),
    [sym_internal_threshold_get] = ACTIONS(2001),
    [sym_rebuilt_event] = ACTIONS(2001),
    [sym_do_custom_event] = ACTIONS(2001),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2001),
    [sym_clkmgr_set_clk_source] = ACTIONS(2001),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2001),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2001),
    [sym_clkmgr_get_timezone] = ACTIONS(2001),
    [sym_clkmgr_set_timezone] = ACTIONS(2001),
    [sym_clkmgr_get_resync_period] = ACTIONS(2001),
    [sym_clkmgr_set_resync_period] = ACTIONS(2001),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2001),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2001),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2001),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2001),
    [sym_clkmgr_get_timeservers] = ACTIONS(2001),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2001),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2001),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2001),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2001),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2001),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2001),
    [sym_set_log_level] = ACTIONS(2001),
    [sym_get_log_level] = ACTIONS(2001),
    [sym_amx_log] = ACTIONS(2001),
    [sym_smtp_server_config_set] = ACTIONS(2001),
    [sym_smtp_server_config_get] = ACTIONS(2001),
    [sym_smtp_send] = ACTIONS(2001),
    [sym_validate_netlinx_account] = ACTIONS(2001),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2001),
    [sym_authenticate_certificate] = ACTIONS(2001),
    [sym_audit_netlinx_session_event] = ACTIONS(2001),
    [sym_audit_netlinx_generic_event] = ACTIONS(2001),
    [sym_get_available_flash_disk_space] = ACTIONS(2001),
    [sym_get_max_flash_disk_space] = ACTIONS(2001),
    [sym_data_create_feed] = ACTIONS(2001),
    [sym_data_delete_feed] = ACTIONS(2001),
    [sym_data_publish_feed] = ACTIONS(2001),
    [sym_data_get_published_feed] = ACTIONS(2001),
    [sym_data_add_record] = ACTIONS(2001),
    [sym_data_get_event_record] = ACTIONS(2001),
    [sym__wc_data_create_feed] = ACTIONS(2001),
    [sym__wc_data_add_record] = ACTIONS(2001),
    [sym__wc_data_get_event_record] = ACTIONS(2001),
    [sym_format] = ACTIONS(2001),
    [sym_astro_clock] = ACTIONS(2001),
    [sym_set_outdoor_temperature] = ACTIONS(2001),
    [sym_set_virtual_level_count] = ACTIONS(2001),
    [sym_set_virtual_channel_count] = ACTIONS(2001),
    [sym_set_virtual_port_count] = ACTIONS(2001),
    [sym__wc_encode_int] = ACTIONS(2001),
    [sym__wc_decode_int] = ACTIONS(2001),
    [sym__wc_int] = ACTIONS(2001),
    [sym__wc_to_ch_int] = ACTIONS(2001),
    [sym__ch_to_wc_int] = ACTIONS(2001),
    [sym__wc_find_string_int] = ACTIONS(2001),
    [sym__wc_left_string_int] = ACTIONS(2001),
    [sym__wc_lower_string_int] = ACTIONS(2001),
    [sym__wc_mid_string_int] = ACTIONS(2001),
    [sym__wc_remove_string_int] = ACTIONS(2001),
    [sym__wc_right_string_int] = ACTIONS(2001),
    [sym__wc_upper_string_int] = ACTIONS(2001),
    [sym__wc_compare_string_int] = ACTIONS(2001),
    [sym__wc_get_buffer_char_int] = ACTIONS(2001),
    [sym__wc_get_buffer_string_int] = ACTIONS(2001),
    [sym__wc_concat_string_int] = ACTIONS(2001),
    [sym__wc_file_open_int] = ACTIONS(2001),
    [sym__wc_file_close_int] = ACTIONS(2001),
    [sym__wc_file_read_int] = ACTIONS(2001),
    [sym__wc_file_read_line_int] = ACTIONS(2001),
    [sym__wc_file_write_int] = ACTIONS(2001),
    [sym__wc_file_write_line_int] = ACTIONS(2001),
    [aux_sym_preproc_include_token1] = ACTIONS(2001),
    [aux_sym_preproc_define_token1] = ACTIONS(2003),
    [aux_sym_preproc_warn_token1] = ACTIONS(2003),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2003),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2003),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2003),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2003),
    [aux_sym_preproc_else_token1] = ACTIONS(2003),
    [anon_sym_LBRACK] = ACTIONS(2003),
    [anon_sym_LBRACE] = ACTIONS(2003),
    [anon_sym_LPAREN] = ACTIONS(2001),
    [anon_sym_BANG] = ACTIONS(2003),
    [anon_sym_TILDE] = ACTIONS(2003),
    [anon_sym_DASH] = ACTIONS(2001),
    [anon_sym_PLUS] = ACTIONS(2001),
    [anon_sym_DASH_DASH] = ACTIONS(2003),
    [anon_sym_PLUS_PLUS] = ACTIONS(2003),
    [anon_sym_DQUOTE] = ACTIONS(2003),
    [anon_sym_SQUOTE] = ACTIONS(2003),
    [sym_number_literal] = ACTIONS(2003),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(317)] = {
    [ts_builtin_sym_end] = ACTIONS(1779),
    [sym_identifier] = ACTIONS(1777),
    [sym_program_name_keyword] = ACTIONS(1777),
    [sym_module_name_keyword] = ACTIONS(1777),
    [sym_define_device_keyword] = ACTIONS(1777),
    [sym_define_combine_keyword] = ACTIONS(1777),
    [sym_define_constant_keyword] = ACTIONS(1777),
    [sym_define_type_keyword] = ACTIONS(1777),
    [sym_define_variable_keyword] = ACTIONS(1777),
    [sym_define_system_variable_keyword] = ACTIONS(1777),
    [sym_define_connect_level_keyword] = ACTIONS(1777),
    [sym_define_latching_keyword] = ACTIONS(1777),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1777),
    [sym_define_toggling_keyword] = ACTIONS(1777),
    [sym_define_call_keyword] = ACTIONS(1777),
    [sym_define_function_keyword] = ACTIONS(1777),
    [sym_define_library_function_keyword] = ACTIONS(1777),
    [sym_define_start_keyword] = ACTIONS(1777),
    [sym_define_event_keyword] = ACTIONS(1777),
    [sym_define_program_keyword] = ACTIONS(1777),
    [sym_define_module_keyword] = ACTIONS(1777),
    [sym_char_keyword] = ACTIONS(1777),
    [sym_widechar_keyword] = ACTIONS(1777),
    [sym_integer_keyword] = ACTIONS(1777),
    [sym_sinteger_keyword] = ACTIONS(1777),
    [sym_long_keyword] = ACTIONS(1777),
    [sym_slong_keyword] = ACTIONS(1777),
    [sym_float_keyword] = ACTIONS(1777),
    [sym_double_keyword] = ACTIONS(1777),
    [sym_variant_keyword] = ACTIONS(1777),
    [sym_variantarray_keyword] = ACTIONS(1777),
    [sym_local_var_keyword] = ACTIONS(1777),
    [sym_stack_var_keyword] = ACTIONS(1777),
    [sym_constant_keyword] = ACTIONS(1777),
    [sym_volatile_keyword] = ACTIONS(1777),
    [sym_non_volatile_keyword] = ACTIONS(1777),
    [sym_persistent_keyword] = ACTIONS(1777),
    [sym_dev_keyword] = ACTIONS(1777),
    [sym_devlev_keyword] = ACTIONS(1777),
    [sym_devchan_keyword] = ACTIONS(1777),
    [sym_if_keyword] = ACTIONS(1777),
    [sym_else_keyword] = ACTIONS(1777),
    [sym_switch_keyword] = ACTIONS(1777),
    [sym_case_keyword] = ACTIONS(1777),
    [sym_default_keyword] = ACTIONS(1777),
    [sym_while_keyword] = ACTIONS(1777),
    [sym_for_keyword] = ACTIONS(1777),
    [sym_break_keyword] = ACTIONS(1777),
    [sym_continue_keyword] = ACTIONS(1777),
    [sym_return_keyword] = ACTIONS(1777),
    [sym_select_keyword] = ACTIONS(1777),
    [sym_active_keyword] = ACTIONS(1777),
    [sym_struct_keyword] = ACTIONS(1777),
    [sym_structure_keyword] = ACTIONS(1777),
    [sym_button_event_keyword] = ACTIONS(1777),
    [sym_channel_event_keyword] = ACTIONS(1777),
    [sym_level_event_keyword] = ACTIONS(1777),
    [sym_data_event_keyword] = ACTIONS(1777),
    [sym_timeline_event_keyword] = ACTIONS(1777),
    [sym_custom_event_keyword] = ACTIONS(1777),
    [sym_push_keyword] = ACTIONS(1777),
    [sym_release_keyword] = ACTIONS(1777),
    [aux_sym_on_keyword_token1] = ACTIONS(1777),
    [aux_sym_off_keyword_token1] = ACTIONS(1777),
    [sym_send_level_keyword] = ACTIONS(1777),
    [sym_send_string_keyword] = ACTIONS(1777),
    [sym_send_command_keyword] = ACTIONS(1777),
    [sym_clear_buffer_keyword] = ACTIONS(1777),
    [sym_create_buffer_keyword] = ACTIONS(1777),
    [sym_create_multi_buffer_keyword] = ACTIONS(1777),
    [sym_call_keyword] = ACTIONS(1777),
    [sym_system_call_keyword] = ACTIONS(1777),
    [sym_devchan_to_keyword] = ACTIONS(1777),
    [sym_devchan_min_to_keyword] = ACTIONS(1777),
    [sym_devchan_total_off_keyword] = ACTIONS(1777),
    [sym_devchan_pulse_keyword] = ACTIONS(1777),
    [sym_wait_keyword] = ACTIONS(1777),
    [sym_cancel_wait_keyword] = ACTIONS(1777),
    [sym_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1777),
    [sym_not] = ACTIONS(1777),
    [sym_bnot] = ACTIONS(1777),
    [sym___file__] = ACTIONS(1777),
    [sym___date__] = ACTIONS(1777),
    [sym___ldate__] = ACTIONS(1777),
    [sym___time__] = ACTIONS(1777),
    [sym___line__] = ACTIONS(1777),
    [sym___name__] = ACTIONS(1777),
    [sym___netlinx__] = ACTIONS(1777),
    [sym_netlinx_axi_version] = ACTIONS(1777),
    [sym_url_flg_tcp] = ACTIONS(1777),
    [sym_url_flg_accinfopresent] = ACTIONS(1777),
    [sym_url_flg_temp] = ACTIONS(1777),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1777),
    [sym_url_flg_stat_mask] = ACTIONS(1777),
    [sym_url_flg_stat_lookup] = ACTIONS(1777),
    [sym_url_flg_stat_connecting] = ACTIONS(1777),
    [sym_url_flg_stat_waiting] = ACTIONS(1777),
    [sym_url_flg_stat_connected] = ACTIONS(1777),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1777),
    [sym_first_virtual_device] = ACTIONS(1777),
    [sym_first_local_port] = ACTIONS(1777),
    [sym_dynamic_virtual_device] = ACTIONS(1777),
    [sym_timeline_once] = ACTIONS(1777),
    [sym_timeline_repeat] = ACTIONS(1777),
    [sym_timeline_absolute] = ACTIONS(1777),
    [sym_timeline_relative] = ACTIONS(1777),
    [sym_do_push_timed_infinite] = ACTIONS(1777),
    [sym_source_type_no_address] = ACTIONS(1777),
    [sym_source_type_neuron_id] = ACTIONS(1777),
    [sym_source_type_ip_address] = ACTIONS(1777),
    [sym_source_type_axlink] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1777),
    [sym_source_type_ip_socket_address] = ACTIONS(1777),
    [sym_source_type_rs232] = ACTIONS(1777),
    [sym_source_type_internal] = ACTIONS(1777),
    [sym_source_type_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1777),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1777),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1777),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1777),
    [sym_true] = ACTIONS(1777),
    [sym_false] = ACTIONS(1777),
    [sym_file_read_only] = ACTIONS(1777),
    [sym_file_rw_new] = ACTIONS(1777),
    [sym_file_rw_append] = ACTIONS(1777),
    [sym_ip_tcp] = ACTIONS(1777),
    [sym_ip_udp] = ACTIONS(1777),
    [sym_ip_udp_2way] = ACTIONS(1777),
    [sym_xml_encode_types] = ACTIONS(1777),
    [sym_xml_encode_char_as_list] = ACTIONS(1777),
    [sym_xml_encode_le] = ACTIONS(1777),
    [sym_xml_decode_types] = ACTIONS(1777),
    [sym_xml_decode_no_preserve] = ACTIONS(1777),
    [sym_duet_dev_type_display_device] = ACTIONS(1777),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_io_device] = ACTIONS(1777),
    [sym_duet_dev_type_relay_device] = ACTIONS(1777),
    [sym_duet_dev_type_ups] = ACTIONS(1777),
    [sym_duet_dev_type_amplifier] = ACTIONS(1777),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1777),
    [sym_duet_dev_type_camera] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1777),
    [sym_duet_dev_type_dss] = ACTIONS(1777),
    [sym_duet_dev_type_dvr] = ACTIONS(1777),
    [sym_duet_dev_type_disc_device] = ACTIONS(1777),
    [sym_duet_dev_type_document_camera] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1777),
    [sym_duet_dev_type_hvac] = ACTIONS(1777),
    [sym_duet_dev_type_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_light] = ACTIONS(1777),
    [sym_duet_dev_type_monitor] = ACTIONS(1777),
    [sym_duet_dev_type_motor] = ACTIONS(1777),
    [sym_duet_dev_type_multi_window] = ACTIONS(1777),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1777),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1777),
    [sym_duet_dev_type_receiver] = ACTIONS(1777),
    [sym_duet_dev_type_security_system] = ACTIONS(1777),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1777),
    [sym_duet_dev_type_settop_box] = ACTIONS(1777),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1777),
    [sym_duet_dev_type_switcher] = ACTIONS(1777),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_tv] = ACTIONS(1777),
    [sym_duet_dev_type_utility] = ACTIONS(1777),
    [sym_duet_dev_type_vcr] = ACTIONS(1777),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_video_processor] = ACTIONS(1777),
    [sym_duet_dev_type_video_projector] = ACTIONS(1777),
    [sym_duet_dev_type_video_wall] = ACTIONS(1777),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1777),
    [sym_duet_dev_type_weather] = ACTIONS(1777),
    [sym_duet_dev_not_polled] = ACTIONS(1777),
    [sym_duet_dev_polled] = ACTIONS(1777),
    [sym_clkmgr_mode_network] = ACTIONS(1777),
    [sym_clkmgr_mode_standalone] = ACTIONS(1777),
    [sym_normal_standby] = ACTIONS(1777),
    [sym_normal_wake] = ACTIONS(1777),
    [sym_ip_multicast_ttl_option] = ACTIONS(1777),
    [sym_ip_ttl_subnet] = ACTIONS(1777),
    [sym_ip_ttl_site] = ACTIONS(1777),
    [sym_ip_ttl_region] = ACTIONS(1777),
    [sym_ip_ttl_continent] = ACTIONS(1777),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1777),
    [sym_ip_nodelay_on] = ACTIONS(1777),
    [sym_ip_nodelay_off] = ACTIONS(1777),
    [sym_amx_error] = ACTIONS(1777),
    [sym_amx_warning] = ACTIONS(1777),
    [sym_amx_info] = ACTIONS(1777),
    [sym_amx_debug] = ACTIONS(1777),
    [sym_netlinx_login_success] = ACTIONS(1777),
    [sym_netlinx_login_fail] = ACTIONS(1777),
    [sym_netlinx_logout] = ACTIONS(1777),
    [sym_null_str] = ACTIONS(1777),
    [sym_smtp_address] = ACTIONS(1777),
    [sym_smtp_port_number] = ACTIONS(1777),
    [sym_smtp_username] = ACTIONS(1777),
    [sym_smtp_password] = ACTIONS(1777),
    [sym_smtp_require_tls] = ACTIONS(1777),
    [sym_smtp_from] = ACTIONS(1777),
    [sym_smtp_tls_true] = ACTIONS(1777),
    [sym_smtp_tls_false] = ACTIONS(1777),
    [sym_listview_on_row_select_event] = ACTIONS(1777),
    [sym_data_structure_unknown] = ACTIONS(1777),
    [sym_data_structure_datafeed] = ACTIONS(1777),
    [sym_data_structure_datarecordset] = ACTIONS(1777),
    [sym_data_structure_datarecord] = ACTIONS(1777),
    [sym_data_structure_datafield] = ACTIONS(1777),
    [sym_data_max_content_field_count] = ACTIONS(1777),
    [sym_data_max_metadata_field_count] = ACTIONS(1777),
    [sym_data_max_id_length] = ACTIONS(1777),
    [sym_data_max_name_length] = ACTIONS(1777),
    [sym_data_max_description_length] = ACTIONS(1777),
    [sym_data_max_type_length] = ACTIONS(1777),
    [sym_data_max_format_length] = ACTIONS(1777),
    [sym_data_max_source_length] = ACTIONS(1777),
    [sym_data_max_value_length] = ACTIONS(1777),
    [sym_data_max_label_length] = ACTIONS(1777),
    [sym_data_max_url_length] = ACTIONS(1777),
    [sym_max_last_login_info_length] = ACTIONS(1777),
    [sym_data_type_unknown] = ACTIONS(1777),
    [sym_data_type_string] = ACTIONS(1777),
    [sym_data_type_datetime] = ACTIONS(1777),
    [sym_data_type_date] = ACTIONS(1777),
    [sym_data_type_time] = ACTIONS(1777),
    [sym_data_type_image] = ACTIONS(1777),
    [sym_data_format_url] = ACTIONS(1777),
    [sym_data_format_phone] = ACTIONS(1777),
    [sym_data_format_email] = ACTIONS(1777),
    [sym_data_format_iso8601] = ACTIONS(1777),
    [sym_auditlog_priv] = ACTIONS(1777),
    [sym_configuration_priv] = ACTIONS(1777),
    [sym_deviceconfig_priv] = ACTIONS(1777),
    [sym_ftp_priv] = ACTIONS(1777),
    [sym_http_priv] = ACTIONS(1777),
    [sym_networkconfig_priv] = ACTIONS(1777),
    [sym_programport_priv] = ACTIONS(1777),
    [sym_remoteui_priv] = ACTIONS(1777),
    [sym_securitycontrol_priv] = ACTIONS(1777),
    [sym_softwaremanagement_priv] = ACTIONS(1777),
    [sym_terminal_priv] = ACTIONS(1777),
    [sym_tpadmin_priv] = ACTIONS(1777),
    [sym_user1_priv] = ACTIONS(1777),
    [sym_user2_priv] = ACTIONS(1777),
    [sym_user3_priv] = ACTIONS(1777),
    [sym_user4_priv] = ACTIONS(1777),
    [sym_usermanagement_priv] = ACTIONS(1777),
    [sym_valid_account] = ACTIONS(1777),
    [sym_err_name_invalid] = ACTIONS(1777),
    [sym_err_password_invalid] = ACTIONS(1777),
    [sym_err_invalid_account] = ACTIONS(1777),
    [sym_err_no_authorization] = ACTIONS(1777),
    [sym_err_invalid_parameter] = ACTIONS(1777),
    [sym_err_user_locked_out] = ACTIONS(1777),
    [sym_err_user_account_expired] = ACTIONS(1777),
    [sym_err_general_error] = ACTIONS(1777),
    [sym_err_authentication_error] = ACTIONS(1777),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1777),
    [sym_tls_validate_certificate] = ACTIONS(1777),
    [sym_tdata] = ACTIONS(1777),
    [sym_tchannel] = ACTIONS(1777),
    [sym_tlevel] = ACTIONS(1777),
    [sym_tbutton] = ACTIONS(1777),
    [sym_ttimeline] = ACTIONS(1777),
    [sym_tcustom] = ACTIONS(1777),
    [sym_url_struct] = ACTIONS(1777),
    [sym_dns_struct] = ACTIONS(1777),
    [sym_ip_address_struct] = ACTIONS(1777),
    [sym_dev_info_struct] = ACTIONS(1777),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1777),
    [sym_data_feed] = ACTIONS(1777),
    [sym_data_field] = ACTIONS(1777),
    [sym_data_record] = ACTIONS(1777),
    [sym_wc_data_feed] = ACTIONS(1777),
    [sym_wc_data_field] = ACTIONS(1777),
    [sym_wc_data_record] = ACTIONS(1777),
    [sym_last_login_info] = ACTIONS(1777),
    [sym____reserved___] = ACTIONS(1777),
    [sym_date] = ACTIONS(1777),
    [sym_day] = ACTIONS(1777),
    [sym_time] = ACTIONS(1777),
    [sym_ldate] = ACTIONS(1777),
    [sym_push_device] = ACTIONS(1777),
    [sym_push_channel] = ACTIONS(1777),
    [sym_push_devchan] = ACTIONS(1777),
    [sym_release_device] = ACTIONS(1777),
    [sym_release_channel] = ACTIONS(1777),
    [sym_release_devchan] = ACTIONS(1777),
    [sym_master_slot] = ACTIONS(1777),
    [sym_get_pulse_time] = ACTIONS(1777),
    [sym_get_timer] = ACTIONS(1777),
    [sym_master_sn] = ACTIONS(1777),
    [sym_system_number] = ACTIONS(1777),
    [sym_button] = ACTIONS(1777),
    [sym_dv_channel] = ACTIONS(1777),
    [sym_data] = ACTIONS(1777),
    [sym_level] = ACTIONS(1777),
    [sym_timeline] = ACTIONS(1777),
    [sym_channel] = ACTIONS(1777),
    [sym_custom] = ACTIONS(1777),
    [sym_atoi] = ACTIONS(1777),
    [sym_atol] = ACTIONS(1777),
    [sym_atof] = ACTIONS(1777),
    [sym_itoa] = ACTIONS(1777),
    [sym_itohex] = ACTIONS(1777),
    [sym_hextoi] = ACTIONS(1777),
    [sym_ftoa] = ACTIONS(1777),
    [sym_find_string] = ACTIONS(1777),
    [sym_left_string] = ACTIONS(1777),
    [sym_right_string] = ACTIONS(1777),
    [sym_mid_string] = ACTIONS(1777),
    [sym_remove_string] = ACTIONS(1777),
    [sym_get_buffer_string] = ACTIONS(1777),
    [sym_get_buffer_char] = ACTIONS(1777),
    [sym_get_multi_buffer_string] = ACTIONS(1777),
    [sym_length_string] = ACTIONS(1777),
    [sym_max_length_string] = ACTIONS(1777),
    [sym_set_length_string] = ACTIONS(1777),
    [sym_compare_string] = ACTIONS(1777),
    [sym_upper_string] = ACTIONS(1777),
    [sym_lower_string] = ACTIONS(1777),
    [sym_get_last] = ACTIONS(1777),
    [sym_date_to_day] = ACTIONS(1777),
    [sym_date_to_month] = ACTIONS(1777),
    [sym_date_to_year] = ACTIONS(1777),
    [sym_time_to_hour] = ACTIONS(1777),
    [sym_time_to_minute] = ACTIONS(1777),
    [sym_time_to_second] = ACTIONS(1777),
    [sym_day_of_week] = ACTIONS(1777),
    [sym_set_pulse_time] = ACTIONS(1777),
    [sym_set_timer] = ACTIONS(1777),
    [sym_file_open] = ACTIONS(1777),
    [sym_file_close] = ACTIONS(1777),
    [sym_file_read] = ACTIONS(1777),
    [sym_file_read_line] = ACTIONS(1777),
    [sym_file_write] = ACTIONS(1777),
    [sym_file_write_line] = ACTIONS(1777),
    [sym_file_copy] = ACTIONS(1777),
    [sym_file_delete] = ACTIONS(1777),
    [sym_file_rename] = ACTIONS(1777),
    [sym_file_dir] = ACTIONS(1777),
    [sym_file_getdir] = ACTIONS(1777),
    [sym_file_setdir] = ACTIONS(1777),
    [sym_file_create_dir] = ACTIONS(1777),
    [sym_file_remove_dir] = ACTIONS(1777),
    [sym_file_seek] = ACTIONS(1777),
    [sym_length_array] = ACTIONS(1777),
    [sym_max_length_array] = ACTIONS(1777),
    [sym_set_length_array] = ACTIONS(1777),
    [sym_ip_client_open] = ACTIONS(1777),
    [sym_ip_client_close] = ACTIONS(1777),
    [sym_ip_server_open] = ACTIONS(1777),
    [sym_ip_server_close] = ACTIONS(1777),
    [sym_ip_bound_client_open] = ACTIONS(1777),
    [sym_ip_mc_server_open] = ACTIONS(1777),
    [sym_ip_set_option] = ACTIONS(1777),
    [sym_ssh_client_open] = ACTIONS(1777),
    [sym_ssh_client_close] = ACTIONS(1777),
    [sym_tls_client_open] = ACTIONS(1777),
    [sym_tls_client_close] = ACTIONS(1777),
    [sym_do_push] = ACTIONS(1777),
    [sym_do_push_timed] = ACTIONS(1777),
    [sym_do_release] = ACTIONS(1777),
    [sym_device_id] = ACTIONS(1777),
    [sym_device_info] = ACTIONS(1777),
    [sym_device_id_string] = ACTIONS(1777),
    [sym_device_standby] = ACTIONS(1777),
    [sym_device_wake] = ACTIONS(1777),
    [sym_timeline_create] = ACTIONS(1777),
    [sym_timeline_kill] = ACTIONS(1777),
    [sym_timeline_reload] = ACTIONS(1777),
    [sym_timeline_pause] = ACTIONS(1777),
    [sym_timeline_restart] = ACTIONS(1777),
    [sym_timeline_set] = ACTIONS(1777),
    [sym_timeline_get] = ACTIONS(1777),
    [sym_timeline_active] = ACTIONS(1777),
    [sym_timeline_dynamic_id] = ACTIONS(1777),
    [sym_variable_to_string] = ACTIONS(1777),
    [sym_string_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_string] = ACTIONS(1777),
    [sym_type_cast] = ACTIONS(1777),
    [sym_variable_to_xml] = ACTIONS(1777),
    [sym_xml_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_xml] = ACTIONS(1777),
    [sym_redirect_string] = ACTIONS(1777),
    [sym_random_number] = ACTIONS(1777),
    [sym_get_unique_id] = ACTIONS(1777),
    [sym_get_system_number] = ACTIONS(1777),
    [sym_set_system_number] = ACTIONS(1777),
    [sym_get_serial_number] = ACTIONS(1777),
    [sym_get_master_build] = ACTIONS(1777),
    [sym_set_validation_code] = ACTIONS(1777),
    [sym_reboot] = ACTIONS(1777),
    [sym_load_duet_module] = ACTIONS(1777),
    [sym_unload_duet_module] = ACTIONS(1777),
    [sym_duet_mem_size_set] = ACTIONS(1777),
    [sym_duet_mem_size_get] = ACTIONS(1777),
    [sym_dynamic_application_device] = ACTIONS(1777),
    [sym_dynamic_polled_port] = ACTIONS(1777),
    [sym_static_port_binding] = ACTIONS(1777),
    [sym_static_ip_binding] = ACTIONS(1777),
    [sym_get_dns_list] = ACTIONS(1777),
    [sym_set_dns_list] = ACTIONS(1777),
    [sym_get_ip_address] = ACTIONS(1777),
    [sym_set_ip_address] = ACTIONS(1777),
    [sym_get_url_list] = ACTIONS(1777),
    [sym_add_url_entry] = ACTIONS(1777),
    [sym_delete_url_entry] = ACTIONS(1777),
    [sym_abs_value] = ACTIONS(1777),
    [sym_max_value] = ACTIONS(1777),
    [sym_min_value] = ACTIONS(1777),
    [sym_exp_value] = ACTIONS(1777),
    [sym_log_value] = ACTIONS(1777),
    [sym_log10_value] = ACTIONS(1777),
    [sym_power_value] = ACTIONS(1777),
    [sym_sqrt_value] = ACTIONS(1777),
    [sym_raw_be] = ACTIONS(1777),
    [sym_raw_le] = ACTIONS(1777),
    [sym_internal_queue_size_set] = ACTIONS(1777),
    [sym_internal_queue_size_get] = ACTIONS(1777),
    [sym_internal_threshold_set] = ACTIONS(1777),
    [sym_internal_threshold_get] = ACTIONS(1777),
    [sym_rebuilt_event] = ACTIONS(1777),
    [sym_do_custom_event] = ACTIONS(1777),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1777),
    [sym_clkmgr_set_clk_source] = ACTIONS(1777),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1777),
    [sym_clkmgr_get_timezone] = ACTIONS(1777),
    [sym_clkmgr_set_timezone] = ACTIONS(1777),
    [sym_clkmgr_get_resync_period] = ACTIONS(1777),
    [sym_clkmgr_set_resync_period] = ACTIONS(1777),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_timeservers] = ACTIONS(1777),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_set_log_level] = ACTIONS(1777),
    [sym_get_log_level] = ACTIONS(1777),
    [sym_amx_log] = ACTIONS(1777),
    [sym_smtp_server_config_set] = ACTIONS(1777),
    [sym_smtp_server_config_get] = ACTIONS(1777),
    [sym_smtp_send] = ACTIONS(1777),
    [sym_validate_netlinx_account] = ACTIONS(1777),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1777),
    [sym_authenticate_certificate] = ACTIONS(1777),
    [sym_audit_netlinx_session_event] = ACTIONS(1777),
    [sym_audit_netlinx_generic_event] = ACTIONS(1777),
    [sym_get_available_flash_disk_space] = ACTIONS(1777),
    [sym_get_max_flash_disk_space] = ACTIONS(1777),
    [sym_data_create_feed] = ACTIONS(1777),
    [sym_data_delete_feed] = ACTIONS(1777),
    [sym_data_publish_feed] = ACTIONS(1777),
    [sym_data_get_published_feed] = ACTIONS(1777),
    [sym_data_add_record] = ACTIONS(1777),
    [sym_data_get_event_record] = ACTIONS(1777),
    [sym__wc_data_create_feed] = ACTIONS(1777),
    [sym__wc_data_add_record] = ACTIONS(1777),
    [sym__wc_data_get_event_record] = ACTIONS(1777),
    [sym_format] = ACTIONS(1777),
    [sym_astro_clock] = ACTIONS(1777),
    [sym_set_outdoor_temperature] = ACTIONS(1777),
    [sym_set_virtual_level_count] = ACTIONS(1777),
    [sym_set_virtual_channel_count] = ACTIONS(1777),
    [sym_set_virtual_port_count] = ACTIONS(1777),
    [sym__wc_encode_int] = ACTIONS(1777),
    [sym__wc_decode_int] = ACTIONS(1777),
    [sym__wc_int] = ACTIONS(1777),
    [sym__wc_to_ch_int] = ACTIONS(1777),
    [sym__ch_to_wc_int] = ACTIONS(1777),
    [sym__wc_find_string_int] = ACTIONS(1777),
    [sym__wc_left_string_int] = ACTIONS(1777),
    [sym__wc_lower_string_int] = ACTIONS(1777),
    [sym__wc_mid_string_int] = ACTIONS(1777),
    [sym__wc_remove_string_int] = ACTIONS(1777),
    [sym__wc_right_string_int] = ACTIONS(1777),
    [sym__wc_upper_string_int] = ACTIONS(1777),
    [sym__wc_compare_string_int] = ACTIONS(1777),
    [sym__wc_get_buffer_char_int] = ACTIONS(1777),
    [sym__wc_get_buffer_string_int] = ACTIONS(1777),
    [sym__wc_concat_string_int] = ACTIONS(1777),
    [sym__wc_file_open_int] = ACTIONS(1777),
    [sym__wc_file_close_int] = ACTIONS(1777),
    [sym__wc_file_read_int] = ACTIONS(1777),
    [sym__wc_file_read_line_int] = ACTIONS(1777),
    [sym__wc_file_write_int] = ACTIONS(1777),
    [sym__wc_file_write_line_int] = ACTIONS(1777),
    [aux_sym_preproc_include_token1] = ACTIONS(1777),
    [aux_sym_preproc_define_token1] = ACTIONS(1779),
    [aux_sym_preproc_warn_token1] = ACTIONS(1779),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1779),
    [anon_sym_LBRACK] = ACTIONS(1779),
    [anon_sym_LBRACE] = ACTIONS(1779),
    [anon_sym_LPAREN] = ACTIONS(1777),
    [anon_sym_BANG] = ACTIONS(1779),
    [anon_sym_TILDE] = ACTIONS(1779),
    [anon_sym_DASH] = ACTIONS(1777),
    [anon_sym_PLUS] = ACTIONS(1777),
    [anon_sym_DASH_DASH] = ACTIONS(1779),
    [anon_sym_PLUS_PLUS] = ACTIONS(1779),
    [anon_sym_DQUOTE] = ACTIONS(1779),
    [anon_sym_SQUOTE] = ACTIONS(1779),
    [sym_number_literal] = ACTIONS(1779),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(318)] = {
    [sym_identifier] = ACTIONS(2005),
    [sym_program_name_keyword] = ACTIONS(2005),
    [sym_module_name_keyword] = ACTIONS(2005),
    [sym_define_device_keyword] = ACTIONS(2005),
    [sym_define_combine_keyword] = ACTIONS(2005),
    [sym_define_constant_keyword] = ACTIONS(2005),
    [sym_define_type_keyword] = ACTIONS(2005),
    [sym_define_variable_keyword] = ACTIONS(2005),
    [sym_define_system_variable_keyword] = ACTIONS(2005),
    [sym_define_connect_level_keyword] = ACTIONS(2005),
    [sym_define_latching_keyword] = ACTIONS(2005),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2005),
    [sym_define_toggling_keyword] = ACTIONS(2005),
    [sym_define_call_keyword] = ACTIONS(2005),
    [sym_define_function_keyword] = ACTIONS(2005),
    [sym_define_library_function_keyword] = ACTIONS(2005),
    [sym_define_start_keyword] = ACTIONS(2005),
    [sym_define_event_keyword] = ACTIONS(2005),
    [sym_define_program_keyword] = ACTIONS(2005),
    [sym_define_module_keyword] = ACTIONS(2005),
    [sym_char_keyword] = ACTIONS(2005),
    [sym_widechar_keyword] = ACTIONS(2005),
    [sym_integer_keyword] = ACTIONS(2005),
    [sym_sinteger_keyword] = ACTIONS(2005),
    [sym_long_keyword] = ACTIONS(2005),
    [sym_slong_keyword] = ACTIONS(2005),
    [sym_float_keyword] = ACTIONS(2005),
    [sym_double_keyword] = ACTIONS(2005),
    [sym_variant_keyword] = ACTIONS(2005),
    [sym_variantarray_keyword] = ACTIONS(2005),
    [sym_local_var_keyword] = ACTIONS(2005),
    [sym_stack_var_keyword] = ACTIONS(2005),
    [sym_constant_keyword] = ACTIONS(2005),
    [sym_volatile_keyword] = ACTIONS(2005),
    [sym_non_volatile_keyword] = ACTIONS(2005),
    [sym_persistent_keyword] = ACTIONS(2005),
    [sym_dev_keyword] = ACTIONS(2005),
    [sym_devlev_keyword] = ACTIONS(2005),
    [sym_devchan_keyword] = ACTIONS(2005),
    [sym_if_keyword] = ACTIONS(2005),
    [sym_switch_keyword] = ACTIONS(2005),
    [sym_case_keyword] = ACTIONS(2005),
    [sym_default_keyword] = ACTIONS(2005),
    [sym_while_keyword] = ACTIONS(2005),
    [sym_for_keyword] = ACTIONS(2005),
    [sym_break_keyword] = ACTIONS(2005),
    [sym_continue_keyword] = ACTIONS(2005),
    [sym_return_keyword] = ACTIONS(2005),
    [sym_select_keyword] = ACTIONS(2005),
    [sym_active_keyword] = ACTIONS(2005),
    [sym_struct_keyword] = ACTIONS(2005),
    [sym_structure_keyword] = ACTIONS(2005),
    [sym_button_event_keyword] = ACTIONS(2005),
    [sym_channel_event_keyword] = ACTIONS(2005),
    [sym_level_event_keyword] = ACTIONS(2005),
    [sym_data_event_keyword] = ACTIONS(2005),
    [sym_timeline_event_keyword] = ACTIONS(2005),
    [sym_custom_event_keyword] = ACTIONS(2005),
    [sym_push_keyword] = ACTIONS(2005),
    [sym_release_keyword] = ACTIONS(2005),
    [aux_sym_on_keyword_token1] = ACTIONS(2005),
    [aux_sym_off_keyword_token1] = ACTIONS(2005),
    [sym_send_level_keyword] = ACTIONS(2005),
    [sym_send_string_keyword] = ACTIONS(2005),
    [sym_send_command_keyword] = ACTIONS(2005),
    [sym_clear_buffer_keyword] = ACTIONS(2005),
    [sym_create_buffer_keyword] = ACTIONS(2005),
    [sym_create_multi_buffer_keyword] = ACTIONS(2005),
    [sym_call_keyword] = ACTIONS(2005),
    [sym_system_call_keyword] = ACTIONS(2005),
    [sym_devchan_to_keyword] = ACTIONS(2005),
    [sym_devchan_min_to_keyword] = ACTIONS(2005),
    [sym_devchan_total_off_keyword] = ACTIONS(2005),
    [sym_devchan_pulse_keyword] = ACTIONS(2005),
    [sym_wait_keyword] = ACTIONS(2005),
    [sym_cancel_wait_keyword] = ACTIONS(2005),
    [sym_wait_until_keyword] = ACTIONS(2005),
    [sym_cancel_wait_until_keyword] = ACTIONS(2005),
    [sym_cancel_all_wait_keyword] = ACTIONS(2005),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2005),
    [sym_not] = ACTIONS(2005),
    [sym_bnot] = ACTIONS(2005),
    [sym___file__] = ACTIONS(2005),
    [sym___date__] = ACTIONS(2005),
    [sym___ldate__] = ACTIONS(2005),
    [sym___time__] = ACTIONS(2005),
    [sym___line__] = ACTIONS(2005),
    [sym___name__] = ACTIONS(2005),
    [sym___netlinx__] = ACTIONS(2005),
    [sym_netlinx_axi_version] = ACTIONS(2005),
    [sym_url_flg_tcp] = ACTIONS(2005),
    [sym_url_flg_accinfopresent] = ACTIONS(2005),
    [sym_url_flg_temp] = ACTIONS(2005),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2005),
    [sym_url_flg_stat_mask] = ACTIONS(2005),
    [sym_url_flg_stat_lookup] = ACTIONS(2005),
    [sym_url_flg_stat_connecting] = ACTIONS(2005),
    [sym_url_flg_stat_waiting] = ACTIONS(2005),
    [sym_url_flg_stat_connected] = ACTIONS(2005),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2005),
    [sym_first_virtual_device] = ACTIONS(2005),
    [sym_first_local_port] = ACTIONS(2005),
    [sym_dynamic_virtual_device] = ACTIONS(2005),
    [sym_timeline_once] = ACTIONS(2005),
    [sym_timeline_repeat] = ACTIONS(2005),
    [sym_timeline_absolute] = ACTIONS(2005),
    [sym_timeline_relative] = ACTIONS(2005),
    [sym_do_push_timed_infinite] = ACTIONS(2005),
    [sym_source_type_no_address] = ACTIONS(2005),
    [sym_source_type_neuron_id] = ACTIONS(2005),
    [sym_source_type_ip_address] = ACTIONS(2005),
    [sym_source_type_axlink] = ACTIONS(2005),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2005),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2005),
    [sym_source_type_ip_socket_address] = ACTIONS(2005),
    [sym_source_type_rs232] = ACTIONS(2005),
    [sym_source_type_internal] = ACTIONS(2005),
    [sym_source_type_mac_address] = ACTIONS(2005),
    [sym_source_type_ipv4_port] = ACTIONS(2005),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2005),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2005),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2005),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2005),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2005),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2005),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2005),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2005),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2005),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2005),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2005),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2005),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2005),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2005),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2005),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2005),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2005),
    [sym_true] = ACTIONS(2005),
    [sym_false] = ACTIONS(2005),
    [sym_file_read_only] = ACTIONS(2005),
    [sym_file_rw_new] = ACTIONS(2005),
    [sym_file_rw_append] = ACTIONS(2005),
    [sym_ip_tcp] = ACTIONS(2005),
    [sym_ip_udp] = ACTIONS(2005),
    [sym_ip_udp_2way] = ACTIONS(2005),
    [sym_xml_encode_types] = ACTIONS(2005),
    [sym_xml_encode_char_as_list] = ACTIONS(2005),
    [sym_xml_encode_le] = ACTIONS(2005),
    [sym_xml_decode_types] = ACTIONS(2005),
    [sym_xml_decode_no_preserve] = ACTIONS(2005),
    [sym_duet_dev_type_display_device] = ACTIONS(2005),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2005),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2005),
    [sym_duet_dev_type_io_device] = ACTIONS(2005),
    [sym_duet_dev_type_relay_device] = ACTIONS(2005),
    [sym_duet_dev_type_ups] = ACTIONS(2005),
    [sym_duet_dev_type_amplifier] = ACTIONS(2005),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2005),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2005),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2005),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2005),
    [sym_duet_dev_type_camera] = ACTIONS(2005),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2005),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2005),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2005),
    [sym_duet_dev_type_dss] = ACTIONS(2005),
    [sym_duet_dev_type_dvr] = ACTIONS(2005),
    [sym_duet_dev_type_disc_device] = ACTIONS(2005),
    [sym_duet_dev_type_document_camera] = ACTIONS(2005),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2005),
    [sym_duet_dev_type_hvac] = ACTIONS(2005),
    [sym_duet_dev_type_keypad] = ACTIONS(2005),
    [sym_duet_dev_type_light] = ACTIONS(2005),
    [sym_duet_dev_type_monitor] = ACTIONS(2005),
    [sym_duet_dev_type_motor] = ACTIONS(2005),
    [sym_duet_dev_type_multi_window] = ACTIONS(2005),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2005),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2005),
    [sym_duet_dev_type_receiver] = ACTIONS(2005),
    [sym_duet_dev_type_security_system] = ACTIONS(2005),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2005),
    [sym_duet_dev_type_settop_box] = ACTIONS(2005),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2005),
    [sym_duet_dev_type_switcher] = ACTIONS(2005),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2005),
    [sym_duet_dev_type_tv] = ACTIONS(2005),
    [sym_duet_dev_type_utility] = ACTIONS(2005),
    [sym_duet_dev_type_vcr] = ACTIONS(2005),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2005),
    [sym_duet_dev_type_video_processor] = ACTIONS(2005),
    [sym_duet_dev_type_video_projector] = ACTIONS(2005),
    [sym_duet_dev_type_video_wall] = ACTIONS(2005),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2005),
    [sym_duet_dev_type_weather] = ACTIONS(2005),
    [sym_duet_dev_not_polled] = ACTIONS(2005),
    [sym_duet_dev_polled] = ACTIONS(2005),
    [sym_clkmgr_mode_network] = ACTIONS(2005),
    [sym_clkmgr_mode_standalone] = ACTIONS(2005),
    [sym_normal_standby] = ACTIONS(2005),
    [sym_normal_wake] = ACTIONS(2005),
    [sym_ip_multicast_ttl_option] = ACTIONS(2005),
    [sym_ip_ttl_subnet] = ACTIONS(2005),
    [sym_ip_ttl_site] = ACTIONS(2005),
    [sym_ip_ttl_region] = ACTIONS(2005),
    [sym_ip_ttl_continent] = ACTIONS(2005),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2005),
    [sym_ip_nodelay_on] = ACTIONS(2005),
    [sym_ip_nodelay_off] = ACTIONS(2005),
    [sym_amx_error] = ACTIONS(2005),
    [sym_amx_warning] = ACTIONS(2005),
    [sym_amx_info] = ACTIONS(2005),
    [sym_amx_debug] = ACTIONS(2005),
    [sym_netlinx_login_success] = ACTIONS(2005),
    [sym_netlinx_login_fail] = ACTIONS(2005),
    [sym_netlinx_logout] = ACTIONS(2005),
    [sym_null_str] = ACTIONS(2005),
    [sym_smtp_address] = ACTIONS(2005),
    [sym_smtp_port_number] = ACTIONS(2005),
    [sym_smtp_username] = ACTIONS(2005),
    [sym_smtp_password] = ACTIONS(2005),
    [sym_smtp_require_tls] = ACTIONS(2005),
    [sym_smtp_from] = ACTIONS(2005),
    [sym_smtp_tls_true] = ACTIONS(2005),
    [sym_smtp_tls_false] = ACTIONS(2005),
    [sym_listview_on_row_select_event] = ACTIONS(2005),
    [sym_data_structure_unknown] = ACTIONS(2005),
    [sym_data_structure_datafeed] = ACTIONS(2005),
    [sym_data_structure_datarecordset] = ACTIONS(2005),
    [sym_data_structure_datarecord] = ACTIONS(2005),
    [sym_data_structure_datafield] = ACTIONS(2005),
    [sym_data_max_content_field_count] = ACTIONS(2005),
    [sym_data_max_metadata_field_count] = ACTIONS(2005),
    [sym_data_max_id_length] = ACTIONS(2005),
    [sym_data_max_name_length] = ACTIONS(2005),
    [sym_data_max_description_length] = ACTIONS(2005),
    [sym_data_max_type_length] = ACTIONS(2005),
    [sym_data_max_format_length] = ACTIONS(2005),
    [sym_data_max_source_length] = ACTIONS(2005),
    [sym_data_max_value_length] = ACTIONS(2005),
    [sym_data_max_label_length] = ACTIONS(2005),
    [sym_data_max_url_length] = ACTIONS(2005),
    [sym_max_last_login_info_length] = ACTIONS(2005),
    [sym_data_type_unknown] = ACTIONS(2005),
    [sym_data_type_string] = ACTIONS(2005),
    [sym_data_type_datetime] = ACTIONS(2005),
    [sym_data_type_date] = ACTIONS(2005),
    [sym_data_type_time] = ACTIONS(2005),
    [sym_data_type_image] = ACTIONS(2005),
    [sym_data_format_url] = ACTIONS(2005),
    [sym_data_format_phone] = ACTIONS(2005),
    [sym_data_format_email] = ACTIONS(2005),
    [sym_data_format_iso8601] = ACTIONS(2005),
    [sym_auditlog_priv] = ACTIONS(2005),
    [sym_configuration_priv] = ACTIONS(2005),
    [sym_deviceconfig_priv] = ACTIONS(2005),
    [sym_ftp_priv] = ACTIONS(2005),
    [sym_http_priv] = ACTIONS(2005),
    [sym_networkconfig_priv] = ACTIONS(2005),
    [sym_programport_priv] = ACTIONS(2005),
    [sym_remoteui_priv] = ACTIONS(2005),
    [sym_securitycontrol_priv] = ACTIONS(2005),
    [sym_softwaremanagement_priv] = ACTIONS(2005),
    [sym_terminal_priv] = ACTIONS(2005),
    [sym_tpadmin_priv] = ACTIONS(2005),
    [sym_user1_priv] = ACTIONS(2005),
    [sym_user2_priv] = ACTIONS(2005),
    [sym_user3_priv] = ACTIONS(2005),
    [sym_user4_priv] = ACTIONS(2005),
    [sym_usermanagement_priv] = ACTIONS(2005),
    [sym_valid_account] = ACTIONS(2005),
    [sym_err_name_invalid] = ACTIONS(2005),
    [sym_err_password_invalid] = ACTIONS(2005),
    [sym_err_invalid_account] = ACTIONS(2005),
    [sym_err_no_authorization] = ACTIONS(2005),
    [sym_err_invalid_parameter] = ACTIONS(2005),
    [sym_err_user_locked_out] = ACTIONS(2005),
    [sym_err_user_account_expired] = ACTIONS(2005),
    [sym_err_general_error] = ACTIONS(2005),
    [sym_err_authentication_error] = ACTIONS(2005),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2005),
    [sym_tls_validate_certificate] = ACTIONS(2005),
    [sym_tdata] = ACTIONS(2005),
    [sym_tchannel] = ACTIONS(2005),
    [sym_tlevel] = ACTIONS(2005),
    [sym_tbutton] = ACTIONS(2005),
    [sym_ttimeline] = ACTIONS(2005),
    [sym_tcustom] = ACTIONS(2005),
    [sym_url_struct] = ACTIONS(2005),
    [sym_dns_struct] = ACTIONS(2005),
    [sym_ip_address_struct] = ACTIONS(2005),
    [sym_dev_info_struct] = ACTIONS(2005),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2005),
    [sym_data_feed] = ACTIONS(2005),
    [sym_data_field] = ACTIONS(2005),
    [sym_data_record] = ACTIONS(2005),
    [sym_wc_data_feed] = ACTIONS(2005),
    [sym_wc_data_field] = ACTIONS(2005),
    [sym_wc_data_record] = ACTIONS(2005),
    [sym_last_login_info] = ACTIONS(2005),
    [sym____reserved___] = ACTIONS(2005),
    [sym_date] = ACTIONS(2005),
    [sym_day] = ACTIONS(2005),
    [sym_time] = ACTIONS(2005),
    [sym_ldate] = ACTIONS(2005),
    [sym_push_device] = ACTIONS(2005),
    [sym_push_channel] = ACTIONS(2005),
    [sym_push_devchan] = ACTIONS(2005),
    [sym_release_device] = ACTIONS(2005),
    [sym_release_channel] = ACTIONS(2005),
    [sym_release_devchan] = ACTIONS(2005),
    [sym_master_slot] = ACTIONS(2005),
    [sym_get_pulse_time] = ACTIONS(2005),
    [sym_get_timer] = ACTIONS(2005),
    [sym_master_sn] = ACTIONS(2005),
    [sym_system_number] = ACTIONS(2005),
    [sym_button] = ACTIONS(2005),
    [sym_dv_channel] = ACTIONS(2005),
    [sym_data] = ACTIONS(2005),
    [sym_level] = ACTIONS(2005),
    [sym_timeline] = ACTIONS(2005),
    [sym_channel] = ACTIONS(2005),
    [sym_custom] = ACTIONS(2005),
    [sym_atoi] = ACTIONS(2005),
    [sym_atol] = ACTIONS(2005),
    [sym_atof] = ACTIONS(2005),
    [sym_itoa] = ACTIONS(2005),
    [sym_itohex] = ACTIONS(2005),
    [sym_hextoi] = ACTIONS(2005),
    [sym_ftoa] = ACTIONS(2005),
    [sym_find_string] = ACTIONS(2005),
    [sym_left_string] = ACTIONS(2005),
    [sym_right_string] = ACTIONS(2005),
    [sym_mid_string] = ACTIONS(2005),
    [sym_remove_string] = ACTIONS(2005),
    [sym_get_buffer_string] = ACTIONS(2005),
    [sym_get_buffer_char] = ACTIONS(2005),
    [sym_get_multi_buffer_string] = ACTIONS(2005),
    [sym_length_string] = ACTIONS(2005),
    [sym_max_length_string] = ACTIONS(2005),
    [sym_set_length_string] = ACTIONS(2005),
    [sym_compare_string] = ACTIONS(2005),
    [sym_upper_string] = ACTIONS(2005),
    [sym_lower_string] = ACTIONS(2005),
    [sym_get_last] = ACTIONS(2005),
    [sym_date_to_day] = ACTIONS(2005),
    [sym_date_to_month] = ACTIONS(2005),
    [sym_date_to_year] = ACTIONS(2005),
    [sym_time_to_hour] = ACTIONS(2005),
    [sym_time_to_minute] = ACTIONS(2005),
    [sym_time_to_second] = ACTIONS(2005),
    [sym_day_of_week] = ACTIONS(2005),
    [sym_set_pulse_time] = ACTIONS(2005),
    [sym_set_timer] = ACTIONS(2005),
    [sym_file_open] = ACTIONS(2005),
    [sym_file_close] = ACTIONS(2005),
    [sym_file_read] = ACTIONS(2005),
    [sym_file_read_line] = ACTIONS(2005),
    [sym_file_write] = ACTIONS(2005),
    [sym_file_write_line] = ACTIONS(2005),
    [sym_file_copy] = ACTIONS(2005),
    [sym_file_delete] = ACTIONS(2005),
    [sym_file_rename] = ACTIONS(2005),
    [sym_file_dir] = ACTIONS(2005),
    [sym_file_getdir] = ACTIONS(2005),
    [sym_file_setdir] = ACTIONS(2005),
    [sym_file_create_dir] = ACTIONS(2005),
    [sym_file_remove_dir] = ACTIONS(2005),
    [sym_file_seek] = ACTIONS(2005),
    [sym_length_array] = ACTIONS(2005),
    [sym_max_length_array] = ACTIONS(2005),
    [sym_set_length_array] = ACTIONS(2005),
    [sym_ip_client_open] = ACTIONS(2005),
    [sym_ip_client_close] = ACTIONS(2005),
    [sym_ip_server_open] = ACTIONS(2005),
    [sym_ip_server_close] = ACTIONS(2005),
    [sym_ip_bound_client_open] = ACTIONS(2005),
    [sym_ip_mc_server_open] = ACTIONS(2005),
    [sym_ip_set_option] = ACTIONS(2005),
    [sym_ssh_client_open] = ACTIONS(2005),
    [sym_ssh_client_close] = ACTIONS(2005),
    [sym_tls_client_open] = ACTIONS(2005),
    [sym_tls_client_close] = ACTIONS(2005),
    [sym_do_push] = ACTIONS(2005),
    [sym_do_push_timed] = ACTIONS(2005),
    [sym_do_release] = ACTIONS(2005),
    [sym_device_id] = ACTIONS(2005),
    [sym_device_info] = ACTIONS(2005),
    [sym_device_id_string] = ACTIONS(2005),
    [sym_device_standby] = ACTIONS(2005),
    [sym_device_wake] = ACTIONS(2005),
    [sym_timeline_create] = ACTIONS(2005),
    [sym_timeline_kill] = ACTIONS(2005),
    [sym_timeline_reload] = ACTIONS(2005),
    [sym_timeline_pause] = ACTIONS(2005),
    [sym_timeline_restart] = ACTIONS(2005),
    [sym_timeline_set] = ACTIONS(2005),
    [sym_timeline_get] = ACTIONS(2005),
    [sym_timeline_active] = ACTIONS(2005),
    [sym_timeline_dynamic_id] = ACTIONS(2005),
    [sym_variable_to_string] = ACTIONS(2005),
    [sym_string_to_variable] = ACTIONS(2005),
    [sym_length_variable_to_string] = ACTIONS(2005),
    [sym_type_cast] = ACTIONS(2005),
    [sym_variable_to_xml] = ACTIONS(2005),
    [sym_xml_to_variable] = ACTIONS(2005),
    [sym_length_variable_to_xml] = ACTIONS(2005),
    [sym_redirect_string] = ACTIONS(2005),
    [sym_random_number] = ACTIONS(2005),
    [sym_get_unique_id] = ACTIONS(2005),
    [sym_get_system_number] = ACTIONS(2005),
    [sym_set_system_number] = ACTIONS(2005),
    [sym_get_serial_number] = ACTIONS(2005),
    [sym_get_master_build] = ACTIONS(2005),
    [sym_set_validation_code] = ACTIONS(2005),
    [sym_reboot] = ACTIONS(2005),
    [sym_load_duet_module] = ACTIONS(2005),
    [sym_unload_duet_module] = ACTIONS(2005),
    [sym_duet_mem_size_set] = ACTIONS(2005),
    [sym_duet_mem_size_get] = ACTIONS(2005),
    [sym_dynamic_application_device] = ACTIONS(2005),
    [sym_dynamic_polled_port] = ACTIONS(2005),
    [sym_static_port_binding] = ACTIONS(2005),
    [sym_static_ip_binding] = ACTIONS(2005),
    [sym_get_dns_list] = ACTIONS(2005),
    [sym_set_dns_list] = ACTIONS(2005),
    [sym_get_ip_address] = ACTIONS(2005),
    [sym_set_ip_address] = ACTIONS(2005),
    [sym_get_url_list] = ACTIONS(2005),
    [sym_add_url_entry] = ACTIONS(2005),
    [sym_delete_url_entry] = ACTIONS(2005),
    [sym_abs_value] = ACTIONS(2005),
    [sym_max_value] = ACTIONS(2005),
    [sym_min_value] = ACTIONS(2005),
    [sym_exp_value] = ACTIONS(2005),
    [sym_log_value] = ACTIONS(2005),
    [sym_log10_value] = ACTIONS(2005),
    [sym_power_value] = ACTIONS(2005),
    [sym_sqrt_value] = ACTIONS(2005),
    [sym_raw_be] = ACTIONS(2005),
    [sym_raw_le] = ACTIONS(2005),
    [sym_internal_queue_size_set] = ACTIONS(2005),
    [sym_internal_queue_size_get] = ACTIONS(2005),
    [sym_internal_threshold_set] = ACTIONS(2005),
    [sym_internal_threshold_get] = ACTIONS(2005),
    [sym_rebuilt_event] = ACTIONS(2005),
    [sym_do_custom_event] = ACTIONS(2005),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2005),
    [sym_clkmgr_set_clk_source] = ACTIONS(2005),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2005),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2005),
    [sym_clkmgr_get_timezone] = ACTIONS(2005),
    [sym_clkmgr_set_timezone] = ACTIONS(2005),
    [sym_clkmgr_get_resync_period] = ACTIONS(2005),
    [sym_clkmgr_set_resync_period] = ACTIONS(2005),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2005),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2005),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2005),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2005),
    [sym_clkmgr_get_timeservers] = ACTIONS(2005),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2005),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2005),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2005),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2005),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2005),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2005),
    [sym_set_log_level] = ACTIONS(2005),
    [sym_get_log_level] = ACTIONS(2005),
    [sym_amx_log] = ACTIONS(2005),
    [sym_smtp_server_config_set] = ACTIONS(2005),
    [sym_smtp_server_config_get] = ACTIONS(2005),
    [sym_smtp_send] = ACTIONS(2005),
    [sym_validate_netlinx_account] = ACTIONS(2005),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2005),
    [sym_authenticate_certificate] = ACTIONS(2005),
    [sym_audit_netlinx_session_event] = ACTIONS(2005),
    [sym_audit_netlinx_generic_event] = ACTIONS(2005),
    [sym_get_available_flash_disk_space] = ACTIONS(2005),
    [sym_get_max_flash_disk_space] = ACTIONS(2005),
    [sym_data_create_feed] = ACTIONS(2005),
    [sym_data_delete_feed] = ACTIONS(2005),
    [sym_data_publish_feed] = ACTIONS(2005),
    [sym_data_get_published_feed] = ACTIONS(2005),
    [sym_data_add_record] = ACTIONS(2005),
    [sym_data_get_event_record] = ACTIONS(2005),
    [sym__wc_data_create_feed] = ACTIONS(2005),
    [sym__wc_data_add_record] = ACTIONS(2005),
    [sym__wc_data_get_event_record] = ACTIONS(2005),
    [sym_format] = ACTIONS(2005),
    [sym_astro_clock] = ACTIONS(2005),
    [sym_set_outdoor_temperature] = ACTIONS(2005),
    [sym_set_virtual_level_count] = ACTIONS(2005),
    [sym_set_virtual_channel_count] = ACTIONS(2005),
    [sym_set_virtual_port_count] = ACTIONS(2005),
    [sym__wc_encode_int] = ACTIONS(2005),
    [sym__wc_decode_int] = ACTIONS(2005),
    [sym__wc_int] = ACTIONS(2005),
    [sym__wc_to_ch_int] = ACTIONS(2005),
    [sym__ch_to_wc_int] = ACTIONS(2005),
    [sym__wc_find_string_int] = ACTIONS(2005),
    [sym__wc_left_string_int] = ACTIONS(2005),
    [sym__wc_lower_string_int] = ACTIONS(2005),
    [sym__wc_mid_string_int] = ACTIONS(2005),
    [sym__wc_remove_string_int] = ACTIONS(2005),
    [sym__wc_right_string_int] = ACTIONS(2005),
    [sym__wc_upper_string_int] = ACTIONS(2005),
    [sym__wc_compare_string_int] = ACTIONS(2005),
    [sym__wc_get_buffer_char_int] = ACTIONS(2005),
    [sym__wc_get_buffer_string_int] = ACTIONS(2005),
    [sym__wc_concat_string_int] = ACTIONS(2005),
    [sym__wc_file_open_int] = ACTIONS(2005),
    [sym__wc_file_close_int] = ACTIONS(2005),
    [sym__wc_file_read_int] = ACTIONS(2005),
    [sym__wc_file_read_line_int] = ACTIONS(2005),
    [sym__wc_file_write_int] = ACTIONS(2005),
    [sym__wc_file_write_line_int] = ACTIONS(2005),
    [aux_sym_preproc_include_token1] = ACTIONS(2005),
    [aux_sym_preproc_define_token1] = ACTIONS(2007),
    [aux_sym_preproc_warn_token1] = ACTIONS(2007),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2007),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2007),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2007),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2007),
    [aux_sym_preproc_else_token1] = ACTIONS(2007),
    [anon_sym_LBRACK] = ACTIONS(2007),
    [anon_sym_LBRACE] = ACTIONS(2007),
    [anon_sym_LPAREN] = ACTIONS(2005),
    [anon_sym_BANG] = ACTIONS(2007),
    [anon_sym_TILDE] = ACTIONS(2007),
    [anon_sym_DASH] = ACTIONS(2005),
    [anon_sym_PLUS] = ACTIONS(2005),
    [anon_sym_DASH_DASH] = ACTIONS(2007),
    [anon_sym_PLUS_PLUS] = ACTIONS(2007),
    [anon_sym_DQUOTE] = ACTIONS(2007),
    [anon_sym_SQUOTE] = ACTIONS(2007),
    [sym_number_literal] = ACTIONS(2007),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(319)] = {
    [sym_identifier] = ACTIONS(2009),
    [sym_program_name_keyword] = ACTIONS(2009),
    [sym_module_name_keyword] = ACTIONS(2009),
    [sym_define_device_keyword] = ACTIONS(2009),
    [sym_define_combine_keyword] = ACTIONS(2009),
    [sym_define_constant_keyword] = ACTIONS(2009),
    [sym_define_type_keyword] = ACTIONS(2009),
    [sym_define_variable_keyword] = ACTIONS(2009),
    [sym_define_system_variable_keyword] = ACTIONS(2009),
    [sym_define_connect_level_keyword] = ACTIONS(2009),
    [sym_define_latching_keyword] = ACTIONS(2009),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2009),
    [sym_define_toggling_keyword] = ACTIONS(2009),
    [sym_define_call_keyword] = ACTIONS(2009),
    [sym_define_function_keyword] = ACTIONS(2009),
    [sym_define_library_function_keyword] = ACTIONS(2009),
    [sym_define_start_keyword] = ACTIONS(2009),
    [sym_define_event_keyword] = ACTIONS(2009),
    [sym_define_program_keyword] = ACTIONS(2009),
    [sym_define_module_keyword] = ACTIONS(2009),
    [sym_char_keyword] = ACTIONS(2009),
    [sym_widechar_keyword] = ACTIONS(2009),
    [sym_integer_keyword] = ACTIONS(2009),
    [sym_sinteger_keyword] = ACTIONS(2009),
    [sym_long_keyword] = ACTIONS(2009),
    [sym_slong_keyword] = ACTIONS(2009),
    [sym_float_keyword] = ACTIONS(2009),
    [sym_double_keyword] = ACTIONS(2009),
    [sym_variant_keyword] = ACTIONS(2009),
    [sym_variantarray_keyword] = ACTIONS(2009),
    [sym_local_var_keyword] = ACTIONS(2009),
    [sym_stack_var_keyword] = ACTIONS(2009),
    [sym_constant_keyword] = ACTIONS(2009),
    [sym_volatile_keyword] = ACTIONS(2009),
    [sym_non_volatile_keyword] = ACTIONS(2009),
    [sym_persistent_keyword] = ACTIONS(2009),
    [sym_dev_keyword] = ACTIONS(2009),
    [sym_devlev_keyword] = ACTIONS(2009),
    [sym_devchan_keyword] = ACTIONS(2009),
    [sym_if_keyword] = ACTIONS(2009),
    [sym_switch_keyword] = ACTIONS(2009),
    [sym_case_keyword] = ACTIONS(2009),
    [sym_default_keyword] = ACTIONS(2009),
    [sym_while_keyword] = ACTIONS(2009),
    [sym_for_keyword] = ACTIONS(2009),
    [sym_break_keyword] = ACTIONS(2009),
    [sym_continue_keyword] = ACTIONS(2009),
    [sym_return_keyword] = ACTIONS(2009),
    [sym_select_keyword] = ACTIONS(2009),
    [sym_active_keyword] = ACTIONS(2009),
    [sym_struct_keyword] = ACTIONS(2009),
    [sym_structure_keyword] = ACTIONS(2009),
    [sym_button_event_keyword] = ACTIONS(2009),
    [sym_channel_event_keyword] = ACTIONS(2009),
    [sym_level_event_keyword] = ACTIONS(2009),
    [sym_data_event_keyword] = ACTIONS(2009),
    [sym_timeline_event_keyword] = ACTIONS(2009),
    [sym_custom_event_keyword] = ACTIONS(2009),
    [sym_push_keyword] = ACTIONS(2009),
    [sym_release_keyword] = ACTIONS(2009),
    [aux_sym_on_keyword_token1] = ACTIONS(2009),
    [aux_sym_off_keyword_token1] = ACTIONS(2009),
    [sym_send_level_keyword] = ACTIONS(2009),
    [sym_send_string_keyword] = ACTIONS(2009),
    [sym_send_command_keyword] = ACTIONS(2009),
    [sym_clear_buffer_keyword] = ACTIONS(2009),
    [sym_create_buffer_keyword] = ACTIONS(2009),
    [sym_create_multi_buffer_keyword] = ACTIONS(2009),
    [sym_call_keyword] = ACTIONS(2009),
    [sym_system_call_keyword] = ACTIONS(2009),
    [sym_devchan_to_keyword] = ACTIONS(2009),
    [sym_devchan_min_to_keyword] = ACTIONS(2009),
    [sym_devchan_total_off_keyword] = ACTIONS(2009),
    [sym_devchan_pulse_keyword] = ACTIONS(2009),
    [sym_wait_keyword] = ACTIONS(2009),
    [sym_cancel_wait_keyword] = ACTIONS(2009),
    [sym_wait_until_keyword] = ACTIONS(2009),
    [sym_cancel_wait_until_keyword] = ACTIONS(2009),
    [sym_cancel_all_wait_keyword] = ACTIONS(2009),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2009),
    [sym_not] = ACTIONS(2009),
    [sym_bnot] = ACTIONS(2009),
    [sym___file__] = ACTIONS(2009),
    [sym___date__] = ACTIONS(2009),
    [sym___ldate__] = ACTIONS(2009),
    [sym___time__] = ACTIONS(2009),
    [sym___line__] = ACTIONS(2009),
    [sym___name__] = ACTIONS(2009),
    [sym___netlinx__] = ACTIONS(2009),
    [sym_netlinx_axi_version] = ACTIONS(2009),
    [sym_url_flg_tcp] = ACTIONS(2009),
    [sym_url_flg_accinfopresent] = ACTIONS(2009),
    [sym_url_flg_temp] = ACTIONS(2009),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2009),
    [sym_url_flg_stat_mask] = ACTIONS(2009),
    [sym_url_flg_stat_lookup] = ACTIONS(2009),
    [sym_url_flg_stat_connecting] = ACTIONS(2009),
    [sym_url_flg_stat_waiting] = ACTIONS(2009),
    [sym_url_flg_stat_connected] = ACTIONS(2009),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2009),
    [sym_first_virtual_device] = ACTIONS(2009),
    [sym_first_local_port] = ACTIONS(2009),
    [sym_dynamic_virtual_device] = ACTIONS(2009),
    [sym_timeline_once] = ACTIONS(2009),
    [sym_timeline_repeat] = ACTIONS(2009),
    [sym_timeline_absolute] = ACTIONS(2009),
    [sym_timeline_relative] = ACTIONS(2009),
    [sym_do_push_timed_infinite] = ACTIONS(2009),
    [sym_source_type_no_address] = ACTIONS(2009),
    [sym_source_type_neuron_id] = ACTIONS(2009),
    [sym_source_type_ip_address] = ACTIONS(2009),
    [sym_source_type_axlink] = ACTIONS(2009),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2009),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2009),
    [sym_source_type_ip_socket_address] = ACTIONS(2009),
    [sym_source_type_rs232] = ACTIONS(2009),
    [sym_source_type_internal] = ACTIONS(2009),
    [sym_source_type_mac_address] = ACTIONS(2009),
    [sym_source_type_ipv4_port] = ACTIONS(2009),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2009),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2009),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2009),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2009),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2009),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2009),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2009),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2009),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2009),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2009),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2009),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2009),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2009),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2009),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2009),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2009),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2009),
    [sym_true] = ACTIONS(2009),
    [sym_false] = ACTIONS(2009),
    [sym_file_read_only] = ACTIONS(2009),
    [sym_file_rw_new] = ACTIONS(2009),
    [sym_file_rw_append] = ACTIONS(2009),
    [sym_ip_tcp] = ACTIONS(2009),
    [sym_ip_udp] = ACTIONS(2009),
    [sym_ip_udp_2way] = ACTIONS(2009),
    [sym_xml_encode_types] = ACTIONS(2009),
    [sym_xml_encode_char_as_list] = ACTIONS(2009),
    [sym_xml_encode_le] = ACTIONS(2009),
    [sym_xml_decode_types] = ACTIONS(2009),
    [sym_xml_decode_no_preserve] = ACTIONS(2009),
    [sym_duet_dev_type_display_device] = ACTIONS(2009),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2009),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2009),
    [sym_duet_dev_type_io_device] = ACTIONS(2009),
    [sym_duet_dev_type_relay_device] = ACTIONS(2009),
    [sym_duet_dev_type_ups] = ACTIONS(2009),
    [sym_duet_dev_type_amplifier] = ACTIONS(2009),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2009),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2009),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2009),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2009),
    [sym_duet_dev_type_camera] = ACTIONS(2009),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2009),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2009),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2009),
    [sym_duet_dev_type_dss] = ACTIONS(2009),
    [sym_duet_dev_type_dvr] = ACTIONS(2009),
    [sym_duet_dev_type_disc_device] = ACTIONS(2009),
    [sym_duet_dev_type_document_camera] = ACTIONS(2009),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2009),
    [sym_duet_dev_type_hvac] = ACTIONS(2009),
    [sym_duet_dev_type_keypad] = ACTIONS(2009),
    [sym_duet_dev_type_light] = ACTIONS(2009),
    [sym_duet_dev_type_monitor] = ACTIONS(2009),
    [sym_duet_dev_type_motor] = ACTIONS(2009),
    [sym_duet_dev_type_multi_window] = ACTIONS(2009),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2009),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2009),
    [sym_duet_dev_type_receiver] = ACTIONS(2009),
    [sym_duet_dev_type_security_system] = ACTIONS(2009),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2009),
    [sym_duet_dev_type_settop_box] = ACTIONS(2009),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2009),
    [sym_duet_dev_type_switcher] = ACTIONS(2009),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2009),
    [sym_duet_dev_type_tv] = ACTIONS(2009),
    [sym_duet_dev_type_utility] = ACTIONS(2009),
    [sym_duet_dev_type_vcr] = ACTIONS(2009),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2009),
    [sym_duet_dev_type_video_processor] = ACTIONS(2009),
    [sym_duet_dev_type_video_projector] = ACTIONS(2009),
    [sym_duet_dev_type_video_wall] = ACTIONS(2009),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2009),
    [sym_duet_dev_type_weather] = ACTIONS(2009),
    [sym_duet_dev_not_polled] = ACTIONS(2009),
    [sym_duet_dev_polled] = ACTIONS(2009),
    [sym_clkmgr_mode_network] = ACTIONS(2009),
    [sym_clkmgr_mode_standalone] = ACTIONS(2009),
    [sym_normal_standby] = ACTIONS(2009),
    [sym_normal_wake] = ACTIONS(2009),
    [sym_ip_multicast_ttl_option] = ACTIONS(2009),
    [sym_ip_ttl_subnet] = ACTIONS(2009),
    [sym_ip_ttl_site] = ACTIONS(2009),
    [sym_ip_ttl_region] = ACTIONS(2009),
    [sym_ip_ttl_continent] = ACTIONS(2009),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2009),
    [sym_ip_nodelay_on] = ACTIONS(2009),
    [sym_ip_nodelay_off] = ACTIONS(2009),
    [sym_amx_error] = ACTIONS(2009),
    [sym_amx_warning] = ACTIONS(2009),
    [sym_amx_info] = ACTIONS(2009),
    [sym_amx_debug] = ACTIONS(2009),
    [sym_netlinx_login_success] = ACTIONS(2009),
    [sym_netlinx_login_fail] = ACTIONS(2009),
    [sym_netlinx_logout] = ACTIONS(2009),
    [sym_null_str] = ACTIONS(2009),
    [sym_smtp_address] = ACTIONS(2009),
    [sym_smtp_port_number] = ACTIONS(2009),
    [sym_smtp_username] = ACTIONS(2009),
    [sym_smtp_password] = ACTIONS(2009),
    [sym_smtp_require_tls] = ACTIONS(2009),
    [sym_smtp_from] = ACTIONS(2009),
    [sym_smtp_tls_true] = ACTIONS(2009),
    [sym_smtp_tls_false] = ACTIONS(2009),
    [sym_listview_on_row_select_event] = ACTIONS(2009),
    [sym_data_structure_unknown] = ACTIONS(2009),
    [sym_data_structure_datafeed] = ACTIONS(2009),
    [sym_data_structure_datarecordset] = ACTIONS(2009),
    [sym_data_structure_datarecord] = ACTIONS(2009),
    [sym_data_structure_datafield] = ACTIONS(2009),
    [sym_data_max_content_field_count] = ACTIONS(2009),
    [sym_data_max_metadata_field_count] = ACTIONS(2009),
    [sym_data_max_id_length] = ACTIONS(2009),
    [sym_data_max_name_length] = ACTIONS(2009),
    [sym_data_max_description_length] = ACTIONS(2009),
    [sym_data_max_type_length] = ACTIONS(2009),
    [sym_data_max_format_length] = ACTIONS(2009),
    [sym_data_max_source_length] = ACTIONS(2009),
    [sym_data_max_value_length] = ACTIONS(2009),
    [sym_data_max_label_length] = ACTIONS(2009),
    [sym_data_max_url_length] = ACTIONS(2009),
    [sym_max_last_login_info_length] = ACTIONS(2009),
    [sym_data_type_unknown] = ACTIONS(2009),
    [sym_data_type_string] = ACTIONS(2009),
    [sym_data_type_datetime] = ACTIONS(2009),
    [sym_data_type_date] = ACTIONS(2009),
    [sym_data_type_time] = ACTIONS(2009),
    [sym_data_type_image] = ACTIONS(2009),
    [sym_data_format_url] = ACTIONS(2009),
    [sym_data_format_phone] = ACTIONS(2009),
    [sym_data_format_email] = ACTIONS(2009),
    [sym_data_format_iso8601] = ACTIONS(2009),
    [sym_auditlog_priv] = ACTIONS(2009),
    [sym_configuration_priv] = ACTIONS(2009),
    [sym_deviceconfig_priv] = ACTIONS(2009),
    [sym_ftp_priv] = ACTIONS(2009),
    [sym_http_priv] = ACTIONS(2009),
    [sym_networkconfig_priv] = ACTIONS(2009),
    [sym_programport_priv] = ACTIONS(2009),
    [sym_remoteui_priv] = ACTIONS(2009),
    [sym_securitycontrol_priv] = ACTIONS(2009),
    [sym_softwaremanagement_priv] = ACTIONS(2009),
    [sym_terminal_priv] = ACTIONS(2009),
    [sym_tpadmin_priv] = ACTIONS(2009),
    [sym_user1_priv] = ACTIONS(2009),
    [sym_user2_priv] = ACTIONS(2009),
    [sym_user3_priv] = ACTIONS(2009),
    [sym_user4_priv] = ACTIONS(2009),
    [sym_usermanagement_priv] = ACTIONS(2009),
    [sym_valid_account] = ACTIONS(2009),
    [sym_err_name_invalid] = ACTIONS(2009),
    [sym_err_password_invalid] = ACTIONS(2009),
    [sym_err_invalid_account] = ACTIONS(2009),
    [sym_err_no_authorization] = ACTIONS(2009),
    [sym_err_invalid_parameter] = ACTIONS(2009),
    [sym_err_user_locked_out] = ACTIONS(2009),
    [sym_err_user_account_expired] = ACTIONS(2009),
    [sym_err_general_error] = ACTIONS(2009),
    [sym_err_authentication_error] = ACTIONS(2009),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2009),
    [sym_tls_validate_certificate] = ACTIONS(2009),
    [sym_tdata] = ACTIONS(2009),
    [sym_tchannel] = ACTIONS(2009),
    [sym_tlevel] = ACTIONS(2009),
    [sym_tbutton] = ACTIONS(2009),
    [sym_ttimeline] = ACTIONS(2009),
    [sym_tcustom] = ACTIONS(2009),
    [sym_url_struct] = ACTIONS(2009),
    [sym_dns_struct] = ACTIONS(2009),
    [sym_ip_address_struct] = ACTIONS(2009),
    [sym_dev_info_struct] = ACTIONS(2009),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2009),
    [sym_data_feed] = ACTIONS(2009),
    [sym_data_field] = ACTIONS(2009),
    [sym_data_record] = ACTIONS(2009),
    [sym_wc_data_feed] = ACTIONS(2009),
    [sym_wc_data_field] = ACTIONS(2009),
    [sym_wc_data_record] = ACTIONS(2009),
    [sym_last_login_info] = ACTIONS(2009),
    [sym____reserved___] = ACTIONS(2009),
    [sym_date] = ACTIONS(2009),
    [sym_day] = ACTIONS(2009),
    [sym_time] = ACTIONS(2009),
    [sym_ldate] = ACTIONS(2009),
    [sym_push_device] = ACTIONS(2009),
    [sym_push_channel] = ACTIONS(2009),
    [sym_push_devchan] = ACTIONS(2009),
    [sym_release_device] = ACTIONS(2009),
    [sym_release_channel] = ACTIONS(2009),
    [sym_release_devchan] = ACTIONS(2009),
    [sym_master_slot] = ACTIONS(2009),
    [sym_get_pulse_time] = ACTIONS(2009),
    [sym_get_timer] = ACTIONS(2009),
    [sym_master_sn] = ACTIONS(2009),
    [sym_system_number] = ACTIONS(2009),
    [sym_button] = ACTIONS(2009),
    [sym_dv_channel] = ACTIONS(2009),
    [sym_data] = ACTIONS(2009),
    [sym_level] = ACTIONS(2009),
    [sym_timeline] = ACTIONS(2009),
    [sym_channel] = ACTIONS(2009),
    [sym_custom] = ACTIONS(2009),
    [sym_atoi] = ACTIONS(2009),
    [sym_atol] = ACTIONS(2009),
    [sym_atof] = ACTIONS(2009),
    [sym_itoa] = ACTIONS(2009),
    [sym_itohex] = ACTIONS(2009),
    [sym_hextoi] = ACTIONS(2009),
    [sym_ftoa] = ACTIONS(2009),
    [sym_find_string] = ACTIONS(2009),
    [sym_left_string] = ACTIONS(2009),
    [sym_right_string] = ACTIONS(2009),
    [sym_mid_string] = ACTIONS(2009),
    [sym_remove_string] = ACTIONS(2009),
    [sym_get_buffer_string] = ACTIONS(2009),
    [sym_get_buffer_char] = ACTIONS(2009),
    [sym_get_multi_buffer_string] = ACTIONS(2009),
    [sym_length_string] = ACTIONS(2009),
    [sym_max_length_string] = ACTIONS(2009),
    [sym_set_length_string] = ACTIONS(2009),
    [sym_compare_string] = ACTIONS(2009),
    [sym_upper_string] = ACTIONS(2009),
    [sym_lower_string] = ACTIONS(2009),
    [sym_get_last] = ACTIONS(2009),
    [sym_date_to_day] = ACTIONS(2009),
    [sym_date_to_month] = ACTIONS(2009),
    [sym_date_to_year] = ACTIONS(2009),
    [sym_time_to_hour] = ACTIONS(2009),
    [sym_time_to_minute] = ACTIONS(2009),
    [sym_time_to_second] = ACTIONS(2009),
    [sym_day_of_week] = ACTIONS(2009),
    [sym_set_pulse_time] = ACTIONS(2009),
    [sym_set_timer] = ACTIONS(2009),
    [sym_file_open] = ACTIONS(2009),
    [sym_file_close] = ACTIONS(2009),
    [sym_file_read] = ACTIONS(2009),
    [sym_file_read_line] = ACTIONS(2009),
    [sym_file_write] = ACTIONS(2009),
    [sym_file_write_line] = ACTIONS(2009),
    [sym_file_copy] = ACTIONS(2009),
    [sym_file_delete] = ACTIONS(2009),
    [sym_file_rename] = ACTIONS(2009),
    [sym_file_dir] = ACTIONS(2009),
    [sym_file_getdir] = ACTIONS(2009),
    [sym_file_setdir] = ACTIONS(2009),
    [sym_file_create_dir] = ACTIONS(2009),
    [sym_file_remove_dir] = ACTIONS(2009),
    [sym_file_seek] = ACTIONS(2009),
    [sym_length_array] = ACTIONS(2009),
    [sym_max_length_array] = ACTIONS(2009),
    [sym_set_length_array] = ACTIONS(2009),
    [sym_ip_client_open] = ACTIONS(2009),
    [sym_ip_client_close] = ACTIONS(2009),
    [sym_ip_server_open] = ACTIONS(2009),
    [sym_ip_server_close] = ACTIONS(2009),
    [sym_ip_bound_client_open] = ACTIONS(2009),
    [sym_ip_mc_server_open] = ACTIONS(2009),
    [sym_ip_set_option] = ACTIONS(2009),
    [sym_ssh_client_open] = ACTIONS(2009),
    [sym_ssh_client_close] = ACTIONS(2009),
    [sym_tls_client_open] = ACTIONS(2009),
    [sym_tls_client_close] = ACTIONS(2009),
    [sym_do_push] = ACTIONS(2009),
    [sym_do_push_timed] = ACTIONS(2009),
    [sym_do_release] = ACTIONS(2009),
    [sym_device_id] = ACTIONS(2009),
    [sym_device_info] = ACTIONS(2009),
    [sym_device_id_string] = ACTIONS(2009),
    [sym_device_standby] = ACTIONS(2009),
    [sym_device_wake] = ACTIONS(2009),
    [sym_timeline_create] = ACTIONS(2009),
    [sym_timeline_kill] = ACTIONS(2009),
    [sym_timeline_reload] = ACTIONS(2009),
    [sym_timeline_pause] = ACTIONS(2009),
    [sym_timeline_restart] = ACTIONS(2009),
    [sym_timeline_set] = ACTIONS(2009),
    [sym_timeline_get] = ACTIONS(2009),
    [sym_timeline_active] = ACTIONS(2009),
    [sym_timeline_dynamic_id] = ACTIONS(2009),
    [sym_variable_to_string] = ACTIONS(2009),
    [sym_string_to_variable] = ACTIONS(2009),
    [sym_length_variable_to_string] = ACTIONS(2009),
    [sym_type_cast] = ACTIONS(2009),
    [sym_variable_to_xml] = ACTIONS(2009),
    [sym_xml_to_variable] = ACTIONS(2009),
    [sym_length_variable_to_xml] = ACTIONS(2009),
    [sym_redirect_string] = ACTIONS(2009),
    [sym_random_number] = ACTIONS(2009),
    [sym_get_unique_id] = ACTIONS(2009),
    [sym_get_system_number] = ACTIONS(2009),
    [sym_set_system_number] = ACTIONS(2009),
    [sym_get_serial_number] = ACTIONS(2009),
    [sym_get_master_build] = ACTIONS(2009),
    [sym_set_validation_code] = ACTIONS(2009),
    [sym_reboot] = ACTIONS(2009),
    [sym_load_duet_module] = ACTIONS(2009),
    [sym_unload_duet_module] = ACTIONS(2009),
    [sym_duet_mem_size_set] = ACTIONS(2009),
    [sym_duet_mem_size_get] = ACTIONS(2009),
    [sym_dynamic_application_device] = ACTIONS(2009),
    [sym_dynamic_polled_port] = ACTIONS(2009),
    [sym_static_port_binding] = ACTIONS(2009),
    [sym_static_ip_binding] = ACTIONS(2009),
    [sym_get_dns_list] = ACTIONS(2009),
    [sym_set_dns_list] = ACTIONS(2009),
    [sym_get_ip_address] = ACTIONS(2009),
    [sym_set_ip_address] = ACTIONS(2009),
    [sym_get_url_list] = ACTIONS(2009),
    [sym_add_url_entry] = ACTIONS(2009),
    [sym_delete_url_entry] = ACTIONS(2009),
    [sym_abs_value] = ACTIONS(2009),
    [sym_max_value] = ACTIONS(2009),
    [sym_min_value] = ACTIONS(2009),
    [sym_exp_value] = ACTIONS(2009),
    [sym_log_value] = ACTIONS(2009),
    [sym_log10_value] = ACTIONS(2009),
    [sym_power_value] = ACTIONS(2009),
    [sym_sqrt_value] = ACTIONS(2009),
    [sym_raw_be] = ACTIONS(2009),
    [sym_raw_le] = ACTIONS(2009),
    [sym_internal_queue_size_set] = ACTIONS(2009),
    [sym_internal_queue_size_get] = ACTIONS(2009),
    [sym_internal_threshold_set] = ACTIONS(2009),
    [sym_internal_threshold_get] = ACTIONS(2009),
    [sym_rebuilt_event] = ACTIONS(2009),
    [sym_do_custom_event] = ACTIONS(2009),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2009),
    [sym_clkmgr_set_clk_source] = ACTIONS(2009),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2009),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2009),
    [sym_clkmgr_get_timezone] = ACTIONS(2009),
    [sym_clkmgr_set_timezone] = ACTIONS(2009),
    [sym_clkmgr_get_resync_period] = ACTIONS(2009),
    [sym_clkmgr_set_resync_period] = ACTIONS(2009),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2009),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2009),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2009),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2009),
    [sym_clkmgr_get_timeservers] = ACTIONS(2009),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2009),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2009),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2009),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2009),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2009),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2009),
    [sym_set_log_level] = ACTIONS(2009),
    [sym_get_log_level] = ACTIONS(2009),
    [sym_amx_log] = ACTIONS(2009),
    [sym_smtp_server_config_set] = ACTIONS(2009),
    [sym_smtp_server_config_get] = ACTIONS(2009),
    [sym_smtp_send] = ACTIONS(2009),
    [sym_validate_netlinx_account] = ACTIONS(2009),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2009),
    [sym_authenticate_certificate] = ACTIONS(2009),
    [sym_audit_netlinx_session_event] = ACTIONS(2009),
    [sym_audit_netlinx_generic_event] = ACTIONS(2009),
    [sym_get_available_flash_disk_space] = ACTIONS(2009),
    [sym_get_max_flash_disk_space] = ACTIONS(2009),
    [sym_data_create_feed] = ACTIONS(2009),
    [sym_data_delete_feed] = ACTIONS(2009),
    [sym_data_publish_feed] = ACTIONS(2009),
    [sym_data_get_published_feed] = ACTIONS(2009),
    [sym_data_add_record] = ACTIONS(2009),
    [sym_data_get_event_record] = ACTIONS(2009),
    [sym__wc_data_create_feed] = ACTIONS(2009),
    [sym__wc_data_add_record] = ACTIONS(2009),
    [sym__wc_data_get_event_record] = ACTIONS(2009),
    [sym_format] = ACTIONS(2009),
    [sym_astro_clock] = ACTIONS(2009),
    [sym_set_outdoor_temperature] = ACTIONS(2009),
    [sym_set_virtual_level_count] = ACTIONS(2009),
    [sym_set_virtual_channel_count] = ACTIONS(2009),
    [sym_set_virtual_port_count] = ACTIONS(2009),
    [sym__wc_encode_int] = ACTIONS(2009),
    [sym__wc_decode_int] = ACTIONS(2009),
    [sym__wc_int] = ACTIONS(2009),
    [sym__wc_to_ch_int] = ACTIONS(2009),
    [sym__ch_to_wc_int] = ACTIONS(2009),
    [sym__wc_find_string_int] = ACTIONS(2009),
    [sym__wc_left_string_int] = ACTIONS(2009),
    [sym__wc_lower_string_int] = ACTIONS(2009),
    [sym__wc_mid_string_int] = ACTIONS(2009),
    [sym__wc_remove_string_int] = ACTIONS(2009),
    [sym__wc_right_string_int] = ACTIONS(2009),
    [sym__wc_upper_string_int] = ACTIONS(2009),
    [sym__wc_compare_string_int] = ACTIONS(2009),
    [sym__wc_get_buffer_char_int] = ACTIONS(2009),
    [sym__wc_get_buffer_string_int] = ACTIONS(2009),
    [sym__wc_concat_string_int] = ACTIONS(2009),
    [sym__wc_file_open_int] = ACTIONS(2009),
    [sym__wc_file_close_int] = ACTIONS(2009),
    [sym__wc_file_read_int] = ACTIONS(2009),
    [sym__wc_file_read_line_int] = ACTIONS(2009),
    [sym__wc_file_write_int] = ACTIONS(2009),
    [sym__wc_file_write_line_int] = ACTIONS(2009),
    [aux_sym_preproc_include_token1] = ACTIONS(2009),
    [aux_sym_preproc_define_token1] = ACTIONS(2011),
    [aux_sym_preproc_warn_token1] = ACTIONS(2011),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2011),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2011),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2011),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2011),
    [aux_sym_preproc_else_token1] = ACTIONS(2011),
    [anon_sym_LBRACK] = ACTIONS(2011),
    [anon_sym_LBRACE] = ACTIONS(2011),
    [anon_sym_LPAREN] = ACTIONS(2009),
    [anon_sym_BANG] = ACTIONS(2011),
    [anon_sym_TILDE] = ACTIONS(2011),
    [anon_sym_DASH] = ACTIONS(2009),
    [anon_sym_PLUS] = ACTIONS(2009),
    [anon_sym_DASH_DASH] = ACTIONS(2011),
    [anon_sym_PLUS_PLUS] = ACTIONS(2011),
    [anon_sym_DQUOTE] = ACTIONS(2011),
    [anon_sym_SQUOTE] = ACTIONS(2011),
    [sym_number_literal] = ACTIONS(2011),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(320)] = {
    [ts_builtin_sym_end] = ACTIONS(1779),
    [sym_identifier] = ACTIONS(1777),
    [sym_program_name_keyword] = ACTIONS(1777),
    [sym_module_name_keyword] = ACTIONS(1777),
    [sym_define_device_keyword] = ACTIONS(1777),
    [sym_define_combine_keyword] = ACTIONS(1777),
    [sym_define_constant_keyword] = ACTIONS(1777),
    [sym_define_type_keyword] = ACTIONS(1777),
    [sym_define_variable_keyword] = ACTIONS(1777),
    [sym_define_system_variable_keyword] = ACTIONS(1777),
    [sym_define_connect_level_keyword] = ACTIONS(1777),
    [sym_define_latching_keyword] = ACTIONS(1777),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1777),
    [sym_define_toggling_keyword] = ACTIONS(1777),
    [sym_define_call_keyword] = ACTIONS(1777),
    [sym_define_function_keyword] = ACTIONS(1777),
    [sym_define_library_function_keyword] = ACTIONS(1777),
    [sym_define_start_keyword] = ACTIONS(1777),
    [sym_define_event_keyword] = ACTIONS(1777),
    [sym_define_program_keyword] = ACTIONS(1777),
    [sym_define_module_keyword] = ACTIONS(1777),
    [sym_char_keyword] = ACTIONS(1777),
    [sym_widechar_keyword] = ACTIONS(1777),
    [sym_integer_keyword] = ACTIONS(1777),
    [sym_sinteger_keyword] = ACTIONS(1777),
    [sym_long_keyword] = ACTIONS(1777),
    [sym_slong_keyword] = ACTIONS(1777),
    [sym_float_keyword] = ACTIONS(1777),
    [sym_double_keyword] = ACTIONS(1777),
    [sym_variant_keyword] = ACTIONS(1777),
    [sym_variantarray_keyword] = ACTIONS(1777),
    [sym_local_var_keyword] = ACTIONS(1777),
    [sym_stack_var_keyword] = ACTIONS(1777),
    [sym_constant_keyword] = ACTIONS(1777),
    [sym_volatile_keyword] = ACTIONS(1777),
    [sym_non_volatile_keyword] = ACTIONS(1777),
    [sym_persistent_keyword] = ACTIONS(1777),
    [sym_dev_keyword] = ACTIONS(1777),
    [sym_devlev_keyword] = ACTIONS(1777),
    [sym_devchan_keyword] = ACTIONS(1777),
    [sym_if_keyword] = ACTIONS(1777),
    [sym_else_keyword] = ACTIONS(1777),
    [sym_switch_keyword] = ACTIONS(1777),
    [sym_case_keyword] = ACTIONS(1777),
    [sym_default_keyword] = ACTIONS(1777),
    [sym_while_keyword] = ACTIONS(1777),
    [sym_for_keyword] = ACTIONS(1777),
    [sym_break_keyword] = ACTIONS(1777),
    [sym_continue_keyword] = ACTIONS(1777),
    [sym_return_keyword] = ACTIONS(1777),
    [sym_select_keyword] = ACTIONS(1777),
    [sym_active_keyword] = ACTIONS(1777),
    [sym_struct_keyword] = ACTIONS(1777),
    [sym_structure_keyword] = ACTIONS(1777),
    [sym_button_event_keyword] = ACTIONS(1777),
    [sym_channel_event_keyword] = ACTIONS(1777),
    [sym_level_event_keyword] = ACTIONS(1777),
    [sym_data_event_keyword] = ACTIONS(1777),
    [sym_timeline_event_keyword] = ACTIONS(1777),
    [sym_custom_event_keyword] = ACTIONS(1777),
    [sym_push_keyword] = ACTIONS(1777),
    [sym_release_keyword] = ACTIONS(1777),
    [aux_sym_on_keyword_token1] = ACTIONS(1777),
    [aux_sym_off_keyword_token1] = ACTIONS(1777),
    [sym_send_level_keyword] = ACTIONS(1777),
    [sym_send_string_keyword] = ACTIONS(1777),
    [sym_send_command_keyword] = ACTIONS(1777),
    [sym_clear_buffer_keyword] = ACTIONS(1777),
    [sym_create_buffer_keyword] = ACTIONS(1777),
    [sym_create_multi_buffer_keyword] = ACTIONS(1777),
    [sym_call_keyword] = ACTIONS(1777),
    [sym_system_call_keyword] = ACTIONS(1777),
    [sym_devchan_to_keyword] = ACTIONS(1777),
    [sym_devchan_min_to_keyword] = ACTIONS(1777),
    [sym_devchan_total_off_keyword] = ACTIONS(1777),
    [sym_devchan_pulse_keyword] = ACTIONS(1777),
    [sym_wait_keyword] = ACTIONS(1777),
    [sym_cancel_wait_keyword] = ACTIONS(1777),
    [sym_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1777),
    [sym_not] = ACTIONS(1777),
    [sym_bnot] = ACTIONS(1777),
    [sym___file__] = ACTIONS(1777),
    [sym___date__] = ACTIONS(1777),
    [sym___ldate__] = ACTIONS(1777),
    [sym___time__] = ACTIONS(1777),
    [sym___line__] = ACTIONS(1777),
    [sym___name__] = ACTIONS(1777),
    [sym___netlinx__] = ACTIONS(1777),
    [sym_netlinx_axi_version] = ACTIONS(1777),
    [sym_url_flg_tcp] = ACTIONS(1777),
    [sym_url_flg_accinfopresent] = ACTIONS(1777),
    [sym_url_flg_temp] = ACTIONS(1777),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1777),
    [sym_url_flg_stat_mask] = ACTIONS(1777),
    [sym_url_flg_stat_lookup] = ACTIONS(1777),
    [sym_url_flg_stat_connecting] = ACTIONS(1777),
    [sym_url_flg_stat_waiting] = ACTIONS(1777),
    [sym_url_flg_stat_connected] = ACTIONS(1777),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1777),
    [sym_first_virtual_device] = ACTIONS(1777),
    [sym_first_local_port] = ACTIONS(1777),
    [sym_dynamic_virtual_device] = ACTIONS(1777),
    [sym_timeline_once] = ACTIONS(1777),
    [sym_timeline_repeat] = ACTIONS(1777),
    [sym_timeline_absolute] = ACTIONS(1777),
    [sym_timeline_relative] = ACTIONS(1777),
    [sym_do_push_timed_infinite] = ACTIONS(1777),
    [sym_source_type_no_address] = ACTIONS(1777),
    [sym_source_type_neuron_id] = ACTIONS(1777),
    [sym_source_type_ip_address] = ACTIONS(1777),
    [sym_source_type_axlink] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1777),
    [sym_source_type_ip_socket_address] = ACTIONS(1777),
    [sym_source_type_rs232] = ACTIONS(1777),
    [sym_source_type_internal] = ACTIONS(1777),
    [sym_source_type_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1777),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1777),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1777),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1777),
    [sym_true] = ACTIONS(1777),
    [sym_false] = ACTIONS(1777),
    [sym_file_read_only] = ACTIONS(1777),
    [sym_file_rw_new] = ACTIONS(1777),
    [sym_file_rw_append] = ACTIONS(1777),
    [sym_ip_tcp] = ACTIONS(1777),
    [sym_ip_udp] = ACTIONS(1777),
    [sym_ip_udp_2way] = ACTIONS(1777),
    [sym_xml_encode_types] = ACTIONS(1777),
    [sym_xml_encode_char_as_list] = ACTIONS(1777),
    [sym_xml_encode_le] = ACTIONS(1777),
    [sym_xml_decode_types] = ACTIONS(1777),
    [sym_xml_decode_no_preserve] = ACTIONS(1777),
    [sym_duet_dev_type_display_device] = ACTIONS(1777),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_io_device] = ACTIONS(1777),
    [sym_duet_dev_type_relay_device] = ACTIONS(1777),
    [sym_duet_dev_type_ups] = ACTIONS(1777),
    [sym_duet_dev_type_amplifier] = ACTIONS(1777),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1777),
    [sym_duet_dev_type_camera] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1777),
    [sym_duet_dev_type_dss] = ACTIONS(1777),
    [sym_duet_dev_type_dvr] = ACTIONS(1777),
    [sym_duet_dev_type_disc_device] = ACTIONS(1777),
    [sym_duet_dev_type_document_camera] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1777),
    [sym_duet_dev_type_hvac] = ACTIONS(1777),
    [sym_duet_dev_type_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_light] = ACTIONS(1777),
    [sym_duet_dev_type_monitor] = ACTIONS(1777),
    [sym_duet_dev_type_motor] = ACTIONS(1777),
    [sym_duet_dev_type_multi_window] = ACTIONS(1777),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1777),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1777),
    [sym_duet_dev_type_receiver] = ACTIONS(1777),
    [sym_duet_dev_type_security_system] = ACTIONS(1777),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1777),
    [sym_duet_dev_type_settop_box] = ACTIONS(1777),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1777),
    [sym_duet_dev_type_switcher] = ACTIONS(1777),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_tv] = ACTIONS(1777),
    [sym_duet_dev_type_utility] = ACTIONS(1777),
    [sym_duet_dev_type_vcr] = ACTIONS(1777),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_video_processor] = ACTIONS(1777),
    [sym_duet_dev_type_video_projector] = ACTIONS(1777),
    [sym_duet_dev_type_video_wall] = ACTIONS(1777),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1777),
    [sym_duet_dev_type_weather] = ACTIONS(1777),
    [sym_duet_dev_not_polled] = ACTIONS(1777),
    [sym_duet_dev_polled] = ACTIONS(1777),
    [sym_clkmgr_mode_network] = ACTIONS(1777),
    [sym_clkmgr_mode_standalone] = ACTIONS(1777),
    [sym_normal_standby] = ACTIONS(1777),
    [sym_normal_wake] = ACTIONS(1777),
    [sym_ip_multicast_ttl_option] = ACTIONS(1777),
    [sym_ip_ttl_subnet] = ACTIONS(1777),
    [sym_ip_ttl_site] = ACTIONS(1777),
    [sym_ip_ttl_region] = ACTIONS(1777),
    [sym_ip_ttl_continent] = ACTIONS(1777),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1777),
    [sym_ip_nodelay_on] = ACTIONS(1777),
    [sym_ip_nodelay_off] = ACTIONS(1777),
    [sym_amx_error] = ACTIONS(1777),
    [sym_amx_warning] = ACTIONS(1777),
    [sym_amx_info] = ACTIONS(1777),
    [sym_amx_debug] = ACTIONS(1777),
    [sym_netlinx_login_success] = ACTIONS(1777),
    [sym_netlinx_login_fail] = ACTIONS(1777),
    [sym_netlinx_logout] = ACTIONS(1777),
    [sym_null_str] = ACTIONS(1777),
    [sym_smtp_address] = ACTIONS(1777),
    [sym_smtp_port_number] = ACTIONS(1777),
    [sym_smtp_username] = ACTIONS(1777),
    [sym_smtp_password] = ACTIONS(1777),
    [sym_smtp_require_tls] = ACTIONS(1777),
    [sym_smtp_from] = ACTIONS(1777),
    [sym_smtp_tls_true] = ACTIONS(1777),
    [sym_smtp_tls_false] = ACTIONS(1777),
    [sym_listview_on_row_select_event] = ACTIONS(1777),
    [sym_data_structure_unknown] = ACTIONS(1777),
    [sym_data_structure_datafeed] = ACTIONS(1777),
    [sym_data_structure_datarecordset] = ACTIONS(1777),
    [sym_data_structure_datarecord] = ACTIONS(1777),
    [sym_data_structure_datafield] = ACTIONS(1777),
    [sym_data_max_content_field_count] = ACTIONS(1777),
    [sym_data_max_metadata_field_count] = ACTIONS(1777),
    [sym_data_max_id_length] = ACTIONS(1777),
    [sym_data_max_name_length] = ACTIONS(1777),
    [sym_data_max_description_length] = ACTIONS(1777),
    [sym_data_max_type_length] = ACTIONS(1777),
    [sym_data_max_format_length] = ACTIONS(1777),
    [sym_data_max_source_length] = ACTIONS(1777),
    [sym_data_max_value_length] = ACTIONS(1777),
    [sym_data_max_label_length] = ACTIONS(1777),
    [sym_data_max_url_length] = ACTIONS(1777),
    [sym_max_last_login_info_length] = ACTIONS(1777),
    [sym_data_type_unknown] = ACTIONS(1777),
    [sym_data_type_string] = ACTIONS(1777),
    [sym_data_type_datetime] = ACTIONS(1777),
    [sym_data_type_date] = ACTIONS(1777),
    [sym_data_type_time] = ACTIONS(1777),
    [sym_data_type_image] = ACTIONS(1777),
    [sym_data_format_url] = ACTIONS(1777),
    [sym_data_format_phone] = ACTIONS(1777),
    [sym_data_format_email] = ACTIONS(1777),
    [sym_data_format_iso8601] = ACTIONS(1777),
    [sym_auditlog_priv] = ACTIONS(1777),
    [sym_configuration_priv] = ACTIONS(1777),
    [sym_deviceconfig_priv] = ACTIONS(1777),
    [sym_ftp_priv] = ACTIONS(1777),
    [sym_http_priv] = ACTIONS(1777),
    [sym_networkconfig_priv] = ACTIONS(1777),
    [sym_programport_priv] = ACTIONS(1777),
    [sym_remoteui_priv] = ACTIONS(1777),
    [sym_securitycontrol_priv] = ACTIONS(1777),
    [sym_softwaremanagement_priv] = ACTIONS(1777),
    [sym_terminal_priv] = ACTIONS(1777),
    [sym_tpadmin_priv] = ACTIONS(1777),
    [sym_user1_priv] = ACTIONS(1777),
    [sym_user2_priv] = ACTIONS(1777),
    [sym_user3_priv] = ACTIONS(1777),
    [sym_user4_priv] = ACTIONS(1777),
    [sym_usermanagement_priv] = ACTIONS(1777),
    [sym_valid_account] = ACTIONS(1777),
    [sym_err_name_invalid] = ACTIONS(1777),
    [sym_err_password_invalid] = ACTIONS(1777),
    [sym_err_invalid_account] = ACTIONS(1777),
    [sym_err_no_authorization] = ACTIONS(1777),
    [sym_err_invalid_parameter] = ACTIONS(1777),
    [sym_err_user_locked_out] = ACTIONS(1777),
    [sym_err_user_account_expired] = ACTIONS(1777),
    [sym_err_general_error] = ACTIONS(1777),
    [sym_err_authentication_error] = ACTIONS(1777),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1777),
    [sym_tls_validate_certificate] = ACTIONS(1777),
    [sym_tdata] = ACTIONS(1777),
    [sym_tchannel] = ACTIONS(1777),
    [sym_tlevel] = ACTIONS(1777),
    [sym_tbutton] = ACTIONS(1777),
    [sym_ttimeline] = ACTIONS(1777),
    [sym_tcustom] = ACTIONS(1777),
    [sym_url_struct] = ACTIONS(1777),
    [sym_dns_struct] = ACTIONS(1777),
    [sym_ip_address_struct] = ACTIONS(1777),
    [sym_dev_info_struct] = ACTIONS(1777),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1777),
    [sym_data_feed] = ACTIONS(1777),
    [sym_data_field] = ACTIONS(1777),
    [sym_data_record] = ACTIONS(1777),
    [sym_wc_data_feed] = ACTIONS(1777),
    [sym_wc_data_field] = ACTIONS(1777),
    [sym_wc_data_record] = ACTIONS(1777),
    [sym_last_login_info] = ACTIONS(1777),
    [sym____reserved___] = ACTIONS(1777),
    [sym_date] = ACTIONS(1777),
    [sym_day] = ACTIONS(1777),
    [sym_time] = ACTIONS(1777),
    [sym_ldate] = ACTIONS(1777),
    [sym_push_device] = ACTIONS(1777),
    [sym_push_channel] = ACTIONS(1777),
    [sym_push_devchan] = ACTIONS(1777),
    [sym_release_device] = ACTIONS(1777),
    [sym_release_channel] = ACTIONS(1777),
    [sym_release_devchan] = ACTIONS(1777),
    [sym_master_slot] = ACTIONS(1777),
    [sym_get_pulse_time] = ACTIONS(1777),
    [sym_get_timer] = ACTIONS(1777),
    [sym_master_sn] = ACTIONS(1777),
    [sym_system_number] = ACTIONS(1777),
    [sym_button] = ACTIONS(1777),
    [sym_dv_channel] = ACTIONS(1777),
    [sym_data] = ACTIONS(1777),
    [sym_level] = ACTIONS(1777),
    [sym_timeline] = ACTIONS(1777),
    [sym_channel] = ACTIONS(1777),
    [sym_custom] = ACTIONS(1777),
    [sym_atoi] = ACTIONS(1777),
    [sym_atol] = ACTIONS(1777),
    [sym_atof] = ACTIONS(1777),
    [sym_itoa] = ACTIONS(1777),
    [sym_itohex] = ACTIONS(1777),
    [sym_hextoi] = ACTIONS(1777),
    [sym_ftoa] = ACTIONS(1777),
    [sym_find_string] = ACTIONS(1777),
    [sym_left_string] = ACTIONS(1777),
    [sym_right_string] = ACTIONS(1777),
    [sym_mid_string] = ACTIONS(1777),
    [sym_remove_string] = ACTIONS(1777),
    [sym_get_buffer_string] = ACTIONS(1777),
    [sym_get_buffer_char] = ACTIONS(1777),
    [sym_get_multi_buffer_string] = ACTIONS(1777),
    [sym_length_string] = ACTIONS(1777),
    [sym_max_length_string] = ACTIONS(1777),
    [sym_set_length_string] = ACTIONS(1777),
    [sym_compare_string] = ACTIONS(1777),
    [sym_upper_string] = ACTIONS(1777),
    [sym_lower_string] = ACTIONS(1777),
    [sym_get_last] = ACTIONS(1777),
    [sym_date_to_day] = ACTIONS(1777),
    [sym_date_to_month] = ACTIONS(1777),
    [sym_date_to_year] = ACTIONS(1777),
    [sym_time_to_hour] = ACTIONS(1777),
    [sym_time_to_minute] = ACTIONS(1777),
    [sym_time_to_second] = ACTIONS(1777),
    [sym_day_of_week] = ACTIONS(1777),
    [sym_set_pulse_time] = ACTIONS(1777),
    [sym_set_timer] = ACTIONS(1777),
    [sym_file_open] = ACTIONS(1777),
    [sym_file_close] = ACTIONS(1777),
    [sym_file_read] = ACTIONS(1777),
    [sym_file_read_line] = ACTIONS(1777),
    [sym_file_write] = ACTIONS(1777),
    [sym_file_write_line] = ACTIONS(1777),
    [sym_file_copy] = ACTIONS(1777),
    [sym_file_delete] = ACTIONS(1777),
    [sym_file_rename] = ACTIONS(1777),
    [sym_file_dir] = ACTIONS(1777),
    [sym_file_getdir] = ACTIONS(1777),
    [sym_file_setdir] = ACTIONS(1777),
    [sym_file_create_dir] = ACTIONS(1777),
    [sym_file_remove_dir] = ACTIONS(1777),
    [sym_file_seek] = ACTIONS(1777),
    [sym_length_array] = ACTIONS(1777),
    [sym_max_length_array] = ACTIONS(1777),
    [sym_set_length_array] = ACTIONS(1777),
    [sym_ip_client_open] = ACTIONS(1777),
    [sym_ip_client_close] = ACTIONS(1777),
    [sym_ip_server_open] = ACTIONS(1777),
    [sym_ip_server_close] = ACTIONS(1777),
    [sym_ip_bound_client_open] = ACTIONS(1777),
    [sym_ip_mc_server_open] = ACTIONS(1777),
    [sym_ip_set_option] = ACTIONS(1777),
    [sym_ssh_client_open] = ACTIONS(1777),
    [sym_ssh_client_close] = ACTIONS(1777),
    [sym_tls_client_open] = ACTIONS(1777),
    [sym_tls_client_close] = ACTIONS(1777),
    [sym_do_push] = ACTIONS(1777),
    [sym_do_push_timed] = ACTIONS(1777),
    [sym_do_release] = ACTIONS(1777),
    [sym_device_id] = ACTIONS(1777),
    [sym_device_info] = ACTIONS(1777),
    [sym_device_id_string] = ACTIONS(1777),
    [sym_device_standby] = ACTIONS(1777),
    [sym_device_wake] = ACTIONS(1777),
    [sym_timeline_create] = ACTIONS(1777),
    [sym_timeline_kill] = ACTIONS(1777),
    [sym_timeline_reload] = ACTIONS(1777),
    [sym_timeline_pause] = ACTIONS(1777),
    [sym_timeline_restart] = ACTIONS(1777),
    [sym_timeline_set] = ACTIONS(1777),
    [sym_timeline_get] = ACTIONS(1777),
    [sym_timeline_active] = ACTIONS(1777),
    [sym_timeline_dynamic_id] = ACTIONS(1777),
    [sym_variable_to_string] = ACTIONS(1777),
    [sym_string_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_string] = ACTIONS(1777),
    [sym_type_cast] = ACTIONS(1777),
    [sym_variable_to_xml] = ACTIONS(1777),
    [sym_xml_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_xml] = ACTIONS(1777),
    [sym_redirect_string] = ACTIONS(1777),
    [sym_random_number] = ACTIONS(1777),
    [sym_get_unique_id] = ACTIONS(1777),
    [sym_get_system_number] = ACTIONS(1777),
    [sym_set_system_number] = ACTIONS(1777),
    [sym_get_serial_number] = ACTIONS(1777),
    [sym_get_master_build] = ACTIONS(1777),
    [sym_set_validation_code] = ACTIONS(1777),
    [sym_reboot] = ACTIONS(1777),
    [sym_load_duet_module] = ACTIONS(1777),
    [sym_unload_duet_module] = ACTIONS(1777),
    [sym_duet_mem_size_set] = ACTIONS(1777),
    [sym_duet_mem_size_get] = ACTIONS(1777),
    [sym_dynamic_application_device] = ACTIONS(1777),
    [sym_dynamic_polled_port] = ACTIONS(1777),
    [sym_static_port_binding] = ACTIONS(1777),
    [sym_static_ip_binding] = ACTIONS(1777),
    [sym_get_dns_list] = ACTIONS(1777),
    [sym_set_dns_list] = ACTIONS(1777),
    [sym_get_ip_address] = ACTIONS(1777),
    [sym_set_ip_address] = ACTIONS(1777),
    [sym_get_url_list] = ACTIONS(1777),
    [sym_add_url_entry] = ACTIONS(1777),
    [sym_delete_url_entry] = ACTIONS(1777),
    [sym_abs_value] = ACTIONS(1777),
    [sym_max_value] = ACTIONS(1777),
    [sym_min_value] = ACTIONS(1777),
    [sym_exp_value] = ACTIONS(1777),
    [sym_log_value] = ACTIONS(1777),
    [sym_log10_value] = ACTIONS(1777),
    [sym_power_value] = ACTIONS(1777),
    [sym_sqrt_value] = ACTIONS(1777),
    [sym_raw_be] = ACTIONS(1777),
    [sym_raw_le] = ACTIONS(1777),
    [sym_internal_queue_size_set] = ACTIONS(1777),
    [sym_internal_queue_size_get] = ACTIONS(1777),
    [sym_internal_threshold_set] = ACTIONS(1777),
    [sym_internal_threshold_get] = ACTIONS(1777),
    [sym_rebuilt_event] = ACTIONS(1777),
    [sym_do_custom_event] = ACTIONS(1777),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1777),
    [sym_clkmgr_set_clk_source] = ACTIONS(1777),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1777),
    [sym_clkmgr_get_timezone] = ACTIONS(1777),
    [sym_clkmgr_set_timezone] = ACTIONS(1777),
    [sym_clkmgr_get_resync_period] = ACTIONS(1777),
    [sym_clkmgr_set_resync_period] = ACTIONS(1777),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_timeservers] = ACTIONS(1777),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_set_log_level] = ACTIONS(1777),
    [sym_get_log_level] = ACTIONS(1777),
    [sym_amx_log] = ACTIONS(1777),
    [sym_smtp_server_config_set] = ACTIONS(1777),
    [sym_smtp_server_config_get] = ACTIONS(1777),
    [sym_smtp_send] = ACTIONS(1777),
    [sym_validate_netlinx_account] = ACTIONS(1777),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1777),
    [sym_authenticate_certificate] = ACTIONS(1777),
    [sym_audit_netlinx_session_event] = ACTIONS(1777),
    [sym_audit_netlinx_generic_event] = ACTIONS(1777),
    [sym_get_available_flash_disk_space] = ACTIONS(1777),
    [sym_get_max_flash_disk_space] = ACTIONS(1777),
    [sym_data_create_feed] = ACTIONS(1777),
    [sym_data_delete_feed] = ACTIONS(1777),
    [sym_data_publish_feed] = ACTIONS(1777),
    [sym_data_get_published_feed] = ACTIONS(1777),
    [sym_data_add_record] = ACTIONS(1777),
    [sym_data_get_event_record] = ACTIONS(1777),
    [sym__wc_data_create_feed] = ACTIONS(1777),
    [sym__wc_data_add_record] = ACTIONS(1777),
    [sym__wc_data_get_event_record] = ACTIONS(1777),
    [sym_format] = ACTIONS(1777),
    [sym_astro_clock] = ACTIONS(1777),
    [sym_set_outdoor_temperature] = ACTIONS(1777),
    [sym_set_virtual_level_count] = ACTIONS(1777),
    [sym_set_virtual_channel_count] = ACTIONS(1777),
    [sym_set_virtual_port_count] = ACTIONS(1777),
    [sym__wc_encode_int] = ACTIONS(1777),
    [sym__wc_decode_int] = ACTIONS(1777),
    [sym__wc_int] = ACTIONS(1777),
    [sym__wc_to_ch_int] = ACTIONS(1777),
    [sym__ch_to_wc_int] = ACTIONS(1777),
    [sym__wc_find_string_int] = ACTIONS(1777),
    [sym__wc_left_string_int] = ACTIONS(1777),
    [sym__wc_lower_string_int] = ACTIONS(1777),
    [sym__wc_mid_string_int] = ACTIONS(1777),
    [sym__wc_remove_string_int] = ACTIONS(1777),
    [sym__wc_right_string_int] = ACTIONS(1777),
    [sym__wc_upper_string_int] = ACTIONS(1777),
    [sym__wc_compare_string_int] = ACTIONS(1777),
    [sym__wc_get_buffer_char_int] = ACTIONS(1777),
    [sym__wc_get_buffer_string_int] = ACTIONS(1777),
    [sym__wc_concat_string_int] = ACTIONS(1777),
    [sym__wc_file_open_int] = ACTIONS(1777),
    [sym__wc_file_close_int] = ACTIONS(1777),
    [sym__wc_file_read_int] = ACTIONS(1777),
    [sym__wc_file_read_line_int] = ACTIONS(1777),
    [sym__wc_file_write_int] = ACTIONS(1777),
    [sym__wc_file_write_line_int] = ACTIONS(1777),
    [aux_sym_preproc_include_token1] = ACTIONS(1777),
    [aux_sym_preproc_define_token1] = ACTIONS(1779),
    [aux_sym_preproc_warn_token1] = ACTIONS(1779),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1779),
    [anon_sym_LBRACK] = ACTIONS(1779),
    [anon_sym_LBRACE] = ACTIONS(1779),
    [anon_sym_LPAREN] = ACTIONS(1777),
    [anon_sym_BANG] = ACTIONS(1779),
    [anon_sym_TILDE] = ACTIONS(1779),
    [anon_sym_DASH] = ACTIONS(1777),
    [anon_sym_PLUS] = ACTIONS(1777),
    [anon_sym_DASH_DASH] = ACTIONS(1779),
    [anon_sym_PLUS_PLUS] = ACTIONS(1779),
    [anon_sym_DQUOTE] = ACTIONS(1779),
    [anon_sym_SQUOTE] = ACTIONS(1779),
    [sym_number_literal] = ACTIONS(1779),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(321)] = {
    [ts_builtin_sym_end] = ACTIONS(1869),
    [sym_identifier] = ACTIONS(1867),
    [sym_program_name_keyword] = ACTIONS(1867),
    [sym_module_name_keyword] = ACTIONS(1867),
    [sym_define_device_keyword] = ACTIONS(1867),
    [sym_define_combine_keyword] = ACTIONS(1867),
    [sym_define_constant_keyword] = ACTIONS(1867),
    [sym_define_type_keyword] = ACTIONS(1867),
    [sym_define_variable_keyword] = ACTIONS(1867),
    [sym_define_system_variable_keyword] = ACTIONS(1867),
    [sym_define_connect_level_keyword] = ACTIONS(1867),
    [sym_define_latching_keyword] = ACTIONS(1867),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1867),
    [sym_define_toggling_keyword] = ACTIONS(1867),
    [sym_define_call_keyword] = ACTIONS(1867),
    [sym_define_function_keyword] = ACTIONS(1867),
    [sym_define_library_function_keyword] = ACTIONS(1867),
    [sym_define_start_keyword] = ACTIONS(1867),
    [sym_define_event_keyword] = ACTIONS(1867),
    [sym_define_program_keyword] = ACTIONS(1867),
    [sym_define_module_keyword] = ACTIONS(1867),
    [sym_char_keyword] = ACTIONS(1867),
    [sym_widechar_keyword] = ACTIONS(1867),
    [sym_integer_keyword] = ACTIONS(1867),
    [sym_sinteger_keyword] = ACTIONS(1867),
    [sym_long_keyword] = ACTIONS(1867),
    [sym_slong_keyword] = ACTIONS(1867),
    [sym_float_keyword] = ACTIONS(1867),
    [sym_double_keyword] = ACTIONS(1867),
    [sym_variant_keyword] = ACTIONS(1867),
    [sym_variantarray_keyword] = ACTIONS(1867),
    [sym_local_var_keyword] = ACTIONS(1867),
    [sym_stack_var_keyword] = ACTIONS(1867),
    [sym_constant_keyword] = ACTIONS(1867),
    [sym_volatile_keyword] = ACTIONS(1867),
    [sym_non_volatile_keyword] = ACTIONS(1867),
    [sym_persistent_keyword] = ACTIONS(1867),
    [sym_dev_keyword] = ACTIONS(1867),
    [sym_devlev_keyword] = ACTIONS(1867),
    [sym_devchan_keyword] = ACTIONS(1867),
    [sym_if_keyword] = ACTIONS(1867),
    [sym_else_keyword] = ACTIONS(1867),
    [sym_switch_keyword] = ACTIONS(1867),
    [sym_case_keyword] = ACTIONS(1867),
    [sym_default_keyword] = ACTIONS(1867),
    [sym_while_keyword] = ACTIONS(1867),
    [sym_for_keyword] = ACTIONS(1867),
    [sym_break_keyword] = ACTIONS(1867),
    [sym_continue_keyword] = ACTIONS(1867),
    [sym_return_keyword] = ACTIONS(1867),
    [sym_select_keyword] = ACTIONS(1867),
    [sym_active_keyword] = ACTIONS(1867),
    [sym_struct_keyword] = ACTIONS(1867),
    [sym_structure_keyword] = ACTIONS(1867),
    [sym_button_event_keyword] = ACTIONS(1867),
    [sym_channel_event_keyword] = ACTIONS(1867),
    [sym_level_event_keyword] = ACTIONS(1867),
    [sym_data_event_keyword] = ACTIONS(1867),
    [sym_timeline_event_keyword] = ACTIONS(1867),
    [sym_custom_event_keyword] = ACTIONS(1867),
    [sym_push_keyword] = ACTIONS(1867),
    [sym_release_keyword] = ACTIONS(1867),
    [aux_sym_on_keyword_token1] = ACTIONS(1867),
    [aux_sym_off_keyword_token1] = ACTIONS(1867),
    [sym_send_level_keyword] = ACTIONS(1867),
    [sym_send_string_keyword] = ACTIONS(1867),
    [sym_send_command_keyword] = ACTIONS(1867),
    [sym_clear_buffer_keyword] = ACTIONS(1867),
    [sym_create_buffer_keyword] = ACTIONS(1867),
    [sym_create_multi_buffer_keyword] = ACTIONS(1867),
    [sym_call_keyword] = ACTIONS(1867),
    [sym_system_call_keyword] = ACTIONS(1867),
    [sym_devchan_to_keyword] = ACTIONS(1867),
    [sym_devchan_min_to_keyword] = ACTIONS(1867),
    [sym_devchan_total_off_keyword] = ACTIONS(1867),
    [sym_devchan_pulse_keyword] = ACTIONS(1867),
    [sym_wait_keyword] = ACTIONS(1867),
    [sym_cancel_wait_keyword] = ACTIONS(1867),
    [sym_wait_until_keyword] = ACTIONS(1867),
    [sym_cancel_wait_until_keyword] = ACTIONS(1867),
    [sym_cancel_all_wait_keyword] = ACTIONS(1867),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1867),
    [sym_not] = ACTIONS(1867),
    [sym_bnot] = ACTIONS(1867),
    [sym___file__] = ACTIONS(1867),
    [sym___date__] = ACTIONS(1867),
    [sym___ldate__] = ACTIONS(1867),
    [sym___time__] = ACTIONS(1867),
    [sym___line__] = ACTIONS(1867),
    [sym___name__] = ACTIONS(1867),
    [sym___netlinx__] = ACTIONS(1867),
    [sym_netlinx_axi_version] = ACTIONS(1867),
    [sym_url_flg_tcp] = ACTIONS(1867),
    [sym_url_flg_accinfopresent] = ACTIONS(1867),
    [sym_url_flg_temp] = ACTIONS(1867),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1867),
    [sym_url_flg_stat_mask] = ACTIONS(1867),
    [sym_url_flg_stat_lookup] = ACTIONS(1867),
    [sym_url_flg_stat_connecting] = ACTIONS(1867),
    [sym_url_flg_stat_waiting] = ACTIONS(1867),
    [sym_url_flg_stat_connected] = ACTIONS(1867),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1867),
    [sym_first_virtual_device] = ACTIONS(1867),
    [sym_first_local_port] = ACTIONS(1867),
    [sym_dynamic_virtual_device] = ACTIONS(1867),
    [sym_timeline_once] = ACTIONS(1867),
    [sym_timeline_repeat] = ACTIONS(1867),
    [sym_timeline_absolute] = ACTIONS(1867),
    [sym_timeline_relative] = ACTIONS(1867),
    [sym_do_push_timed_infinite] = ACTIONS(1867),
    [sym_source_type_no_address] = ACTIONS(1867),
    [sym_source_type_neuron_id] = ACTIONS(1867),
    [sym_source_type_ip_address] = ACTIONS(1867),
    [sym_source_type_axlink] = ACTIONS(1867),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1867),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1867),
    [sym_source_type_ip_socket_address] = ACTIONS(1867),
    [sym_source_type_rs232] = ACTIONS(1867),
    [sym_source_type_internal] = ACTIONS(1867),
    [sym_source_type_mac_address] = ACTIONS(1867),
    [sym_source_type_ipv4_port] = ACTIONS(1867),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1867),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1867),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1867),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1867),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1867),
    [sym_true] = ACTIONS(1867),
    [sym_false] = ACTIONS(1867),
    [sym_file_read_only] = ACTIONS(1867),
    [sym_file_rw_new] = ACTIONS(1867),
    [sym_file_rw_append] = ACTIONS(1867),
    [sym_ip_tcp] = ACTIONS(1867),
    [sym_ip_udp] = ACTIONS(1867),
    [sym_ip_udp_2way] = ACTIONS(1867),
    [sym_xml_encode_types] = ACTIONS(1867),
    [sym_xml_encode_char_as_list] = ACTIONS(1867),
    [sym_xml_encode_le] = ACTIONS(1867),
    [sym_xml_decode_types] = ACTIONS(1867),
    [sym_xml_decode_no_preserve] = ACTIONS(1867),
    [sym_duet_dev_type_display_device] = ACTIONS(1867),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1867),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1867),
    [sym_duet_dev_type_io_device] = ACTIONS(1867),
    [sym_duet_dev_type_relay_device] = ACTIONS(1867),
    [sym_duet_dev_type_ups] = ACTIONS(1867),
    [sym_duet_dev_type_amplifier] = ACTIONS(1867),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1867),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1867),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1867),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1867),
    [sym_duet_dev_type_camera] = ACTIONS(1867),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1867),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1867),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1867),
    [sym_duet_dev_type_dss] = ACTIONS(1867),
    [sym_duet_dev_type_dvr] = ACTIONS(1867),
    [sym_duet_dev_type_disc_device] = ACTIONS(1867),
    [sym_duet_dev_type_document_camera] = ACTIONS(1867),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1867),
    [sym_duet_dev_type_hvac] = ACTIONS(1867),
    [sym_duet_dev_type_keypad] = ACTIONS(1867),
    [sym_duet_dev_type_light] = ACTIONS(1867),
    [sym_duet_dev_type_monitor] = ACTIONS(1867),
    [sym_duet_dev_type_motor] = ACTIONS(1867),
    [sym_duet_dev_type_multi_window] = ACTIONS(1867),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1867),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1867),
    [sym_duet_dev_type_receiver] = ACTIONS(1867),
    [sym_duet_dev_type_security_system] = ACTIONS(1867),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1867),
    [sym_duet_dev_type_settop_box] = ACTIONS(1867),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1867),
    [sym_duet_dev_type_switcher] = ACTIONS(1867),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1867),
    [sym_duet_dev_type_tv] = ACTIONS(1867),
    [sym_duet_dev_type_utility] = ACTIONS(1867),
    [sym_duet_dev_type_vcr] = ACTIONS(1867),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1867),
    [sym_duet_dev_type_video_processor] = ACTIONS(1867),
    [sym_duet_dev_type_video_projector] = ACTIONS(1867),
    [sym_duet_dev_type_video_wall] = ACTIONS(1867),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1867),
    [sym_duet_dev_type_weather] = ACTIONS(1867),
    [sym_duet_dev_not_polled] = ACTIONS(1867),
    [sym_duet_dev_polled] = ACTIONS(1867),
    [sym_clkmgr_mode_network] = ACTIONS(1867),
    [sym_clkmgr_mode_standalone] = ACTIONS(1867),
    [sym_normal_standby] = ACTIONS(1867),
    [sym_normal_wake] = ACTIONS(1867),
    [sym_ip_multicast_ttl_option] = ACTIONS(1867),
    [sym_ip_ttl_subnet] = ACTIONS(1867),
    [sym_ip_ttl_site] = ACTIONS(1867),
    [sym_ip_ttl_region] = ACTIONS(1867),
    [sym_ip_ttl_continent] = ACTIONS(1867),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1867),
    [sym_ip_nodelay_on] = ACTIONS(1867),
    [sym_ip_nodelay_off] = ACTIONS(1867),
    [sym_amx_error] = ACTIONS(1867),
    [sym_amx_warning] = ACTIONS(1867),
    [sym_amx_info] = ACTIONS(1867),
    [sym_amx_debug] = ACTIONS(1867),
    [sym_netlinx_login_success] = ACTIONS(1867),
    [sym_netlinx_login_fail] = ACTIONS(1867),
    [sym_netlinx_logout] = ACTIONS(1867),
    [sym_null_str] = ACTIONS(1867),
    [sym_smtp_address] = ACTIONS(1867),
    [sym_smtp_port_number] = ACTIONS(1867),
    [sym_smtp_username] = ACTIONS(1867),
    [sym_smtp_password] = ACTIONS(1867),
    [sym_smtp_require_tls] = ACTIONS(1867),
    [sym_smtp_from] = ACTIONS(1867),
    [sym_smtp_tls_true] = ACTIONS(1867),
    [sym_smtp_tls_false] = ACTIONS(1867),
    [sym_listview_on_row_select_event] = ACTIONS(1867),
    [sym_data_structure_unknown] = ACTIONS(1867),
    [sym_data_structure_datafeed] = ACTIONS(1867),
    [sym_data_structure_datarecordset] = ACTIONS(1867),
    [sym_data_structure_datarecord] = ACTIONS(1867),
    [sym_data_structure_datafield] = ACTIONS(1867),
    [sym_data_max_content_field_count] = ACTIONS(1867),
    [sym_data_max_metadata_field_count] = ACTIONS(1867),
    [sym_data_max_id_length] = ACTIONS(1867),
    [sym_data_max_name_length] = ACTIONS(1867),
    [sym_data_max_description_length] = ACTIONS(1867),
    [sym_data_max_type_length] = ACTIONS(1867),
    [sym_data_max_format_length] = ACTIONS(1867),
    [sym_data_max_source_length] = ACTIONS(1867),
    [sym_data_max_value_length] = ACTIONS(1867),
    [sym_data_max_label_length] = ACTIONS(1867),
    [sym_data_max_url_length] = ACTIONS(1867),
    [sym_max_last_login_info_length] = ACTIONS(1867),
    [sym_data_type_unknown] = ACTIONS(1867),
    [sym_data_type_string] = ACTIONS(1867),
    [sym_data_type_datetime] = ACTIONS(1867),
    [sym_data_type_date] = ACTIONS(1867),
    [sym_data_type_time] = ACTIONS(1867),
    [sym_data_type_image] = ACTIONS(1867),
    [sym_data_format_url] = ACTIONS(1867),
    [sym_data_format_phone] = ACTIONS(1867),
    [sym_data_format_email] = ACTIONS(1867),
    [sym_data_format_iso8601] = ACTIONS(1867),
    [sym_auditlog_priv] = ACTIONS(1867),
    [sym_configuration_priv] = ACTIONS(1867),
    [sym_deviceconfig_priv] = ACTIONS(1867),
    [sym_ftp_priv] = ACTIONS(1867),
    [sym_http_priv] = ACTIONS(1867),
    [sym_networkconfig_priv] = ACTIONS(1867),
    [sym_programport_priv] = ACTIONS(1867),
    [sym_remoteui_priv] = ACTIONS(1867),
    [sym_securitycontrol_priv] = ACTIONS(1867),
    [sym_softwaremanagement_priv] = ACTIONS(1867),
    [sym_terminal_priv] = ACTIONS(1867),
    [sym_tpadmin_priv] = ACTIONS(1867),
    [sym_user1_priv] = ACTIONS(1867),
    [sym_user2_priv] = ACTIONS(1867),
    [sym_user3_priv] = ACTIONS(1867),
    [sym_user4_priv] = ACTIONS(1867),
    [sym_usermanagement_priv] = ACTIONS(1867),
    [sym_valid_account] = ACTIONS(1867),
    [sym_err_name_invalid] = ACTIONS(1867),
    [sym_err_password_invalid] = ACTIONS(1867),
    [sym_err_invalid_account] = ACTIONS(1867),
    [sym_err_no_authorization] = ACTIONS(1867),
    [sym_err_invalid_parameter] = ACTIONS(1867),
    [sym_err_user_locked_out] = ACTIONS(1867),
    [sym_err_user_account_expired] = ACTIONS(1867),
    [sym_err_general_error] = ACTIONS(1867),
    [sym_err_authentication_error] = ACTIONS(1867),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1867),
    [sym_tls_validate_certificate] = ACTIONS(1867),
    [sym_tdata] = ACTIONS(1867),
    [sym_tchannel] = ACTIONS(1867),
    [sym_tlevel] = ACTIONS(1867),
    [sym_tbutton] = ACTIONS(1867),
    [sym_ttimeline] = ACTIONS(1867),
    [sym_tcustom] = ACTIONS(1867),
    [sym_url_struct] = ACTIONS(1867),
    [sym_dns_struct] = ACTIONS(1867),
    [sym_ip_address_struct] = ACTIONS(1867),
    [sym_dev_info_struct] = ACTIONS(1867),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1867),
    [sym_data_feed] = ACTIONS(1867),
    [sym_data_field] = ACTIONS(1867),
    [sym_data_record] = ACTIONS(1867),
    [sym_wc_data_feed] = ACTIONS(1867),
    [sym_wc_data_field] = ACTIONS(1867),
    [sym_wc_data_record] = ACTIONS(1867),
    [sym_last_login_info] = ACTIONS(1867),
    [sym____reserved___] = ACTIONS(1867),
    [sym_date] = ACTIONS(1867),
    [sym_day] = ACTIONS(1867),
    [sym_time] = ACTIONS(1867),
    [sym_ldate] = ACTIONS(1867),
    [sym_push_device] = ACTIONS(1867),
    [sym_push_channel] = ACTIONS(1867),
    [sym_push_devchan] = ACTIONS(1867),
    [sym_release_device] = ACTIONS(1867),
    [sym_release_channel] = ACTIONS(1867),
    [sym_release_devchan] = ACTIONS(1867),
    [sym_master_slot] = ACTIONS(1867),
    [sym_get_pulse_time] = ACTIONS(1867),
    [sym_get_timer] = ACTIONS(1867),
    [sym_master_sn] = ACTIONS(1867),
    [sym_system_number] = ACTIONS(1867),
    [sym_button] = ACTIONS(1867),
    [sym_dv_channel] = ACTIONS(1867),
    [sym_data] = ACTIONS(1867),
    [sym_level] = ACTIONS(1867),
    [sym_timeline] = ACTIONS(1867),
    [sym_channel] = ACTIONS(1867),
    [sym_custom] = ACTIONS(1867),
    [sym_atoi] = ACTIONS(1867),
    [sym_atol] = ACTIONS(1867),
    [sym_atof] = ACTIONS(1867),
    [sym_itoa] = ACTIONS(1867),
    [sym_itohex] = ACTIONS(1867),
    [sym_hextoi] = ACTIONS(1867),
    [sym_ftoa] = ACTIONS(1867),
    [sym_find_string] = ACTIONS(1867),
    [sym_left_string] = ACTIONS(1867),
    [sym_right_string] = ACTIONS(1867),
    [sym_mid_string] = ACTIONS(1867),
    [sym_remove_string] = ACTIONS(1867),
    [sym_get_buffer_string] = ACTIONS(1867),
    [sym_get_buffer_char] = ACTIONS(1867),
    [sym_get_multi_buffer_string] = ACTIONS(1867),
    [sym_length_string] = ACTIONS(1867),
    [sym_max_length_string] = ACTIONS(1867),
    [sym_set_length_string] = ACTIONS(1867),
    [sym_compare_string] = ACTIONS(1867),
    [sym_upper_string] = ACTIONS(1867),
    [sym_lower_string] = ACTIONS(1867),
    [sym_get_last] = ACTIONS(1867),
    [sym_date_to_day] = ACTIONS(1867),
    [sym_date_to_month] = ACTIONS(1867),
    [sym_date_to_year] = ACTIONS(1867),
    [sym_time_to_hour] = ACTIONS(1867),
    [sym_time_to_minute] = ACTIONS(1867),
    [sym_time_to_second] = ACTIONS(1867),
    [sym_day_of_week] = ACTIONS(1867),
    [sym_set_pulse_time] = ACTIONS(1867),
    [sym_set_timer] = ACTIONS(1867),
    [sym_file_open] = ACTIONS(1867),
    [sym_file_close] = ACTIONS(1867),
    [sym_file_read] = ACTIONS(1867),
    [sym_file_read_line] = ACTIONS(1867),
    [sym_file_write] = ACTIONS(1867),
    [sym_file_write_line] = ACTIONS(1867),
    [sym_file_copy] = ACTIONS(1867),
    [sym_file_delete] = ACTIONS(1867),
    [sym_file_rename] = ACTIONS(1867),
    [sym_file_dir] = ACTIONS(1867),
    [sym_file_getdir] = ACTIONS(1867),
    [sym_file_setdir] = ACTIONS(1867),
    [sym_file_create_dir] = ACTIONS(1867),
    [sym_file_remove_dir] = ACTIONS(1867),
    [sym_file_seek] = ACTIONS(1867),
    [sym_length_array] = ACTIONS(1867),
    [sym_max_length_array] = ACTIONS(1867),
    [sym_set_length_array] = ACTIONS(1867),
    [sym_ip_client_open] = ACTIONS(1867),
    [sym_ip_client_close] = ACTIONS(1867),
    [sym_ip_server_open] = ACTIONS(1867),
    [sym_ip_server_close] = ACTIONS(1867),
    [sym_ip_bound_client_open] = ACTIONS(1867),
    [sym_ip_mc_server_open] = ACTIONS(1867),
    [sym_ip_set_option] = ACTIONS(1867),
    [sym_ssh_client_open] = ACTIONS(1867),
    [sym_ssh_client_close] = ACTIONS(1867),
    [sym_tls_client_open] = ACTIONS(1867),
    [sym_tls_client_close] = ACTIONS(1867),
    [sym_do_push] = ACTIONS(1867),
    [sym_do_push_timed] = ACTIONS(1867),
    [sym_do_release] = ACTIONS(1867),
    [sym_device_id] = ACTIONS(1867),
    [sym_device_info] = ACTIONS(1867),
    [sym_device_id_string] = ACTIONS(1867),
    [sym_device_standby] = ACTIONS(1867),
    [sym_device_wake] = ACTIONS(1867),
    [sym_timeline_create] = ACTIONS(1867),
    [sym_timeline_kill] = ACTIONS(1867),
    [sym_timeline_reload] = ACTIONS(1867),
    [sym_timeline_pause] = ACTIONS(1867),
    [sym_timeline_restart] = ACTIONS(1867),
    [sym_timeline_set] = ACTIONS(1867),
    [sym_timeline_get] = ACTIONS(1867),
    [sym_timeline_active] = ACTIONS(1867),
    [sym_timeline_dynamic_id] = ACTIONS(1867),
    [sym_variable_to_string] = ACTIONS(1867),
    [sym_string_to_variable] = ACTIONS(1867),
    [sym_length_variable_to_string] = ACTIONS(1867),
    [sym_type_cast] = ACTIONS(1867),
    [sym_variable_to_xml] = ACTIONS(1867),
    [sym_xml_to_variable] = ACTIONS(1867),
    [sym_length_variable_to_xml] = ACTIONS(1867),
    [sym_redirect_string] = ACTIONS(1867),
    [sym_random_number] = ACTIONS(1867),
    [sym_get_unique_id] = ACTIONS(1867),
    [sym_get_system_number] = ACTIONS(1867),
    [sym_set_system_number] = ACTIONS(1867),
    [sym_get_serial_number] = ACTIONS(1867),
    [sym_get_master_build] = ACTIONS(1867),
    [sym_set_validation_code] = ACTIONS(1867),
    [sym_reboot] = ACTIONS(1867),
    [sym_load_duet_module] = ACTIONS(1867),
    [sym_unload_duet_module] = ACTIONS(1867),
    [sym_duet_mem_size_set] = ACTIONS(1867),
    [sym_duet_mem_size_get] = ACTIONS(1867),
    [sym_dynamic_application_device] = ACTIONS(1867),
    [sym_dynamic_polled_port] = ACTIONS(1867),
    [sym_static_port_binding] = ACTIONS(1867),
    [sym_static_ip_binding] = ACTIONS(1867),
    [sym_get_dns_list] = ACTIONS(1867),
    [sym_set_dns_list] = ACTIONS(1867),
    [sym_get_ip_address] = ACTIONS(1867),
    [sym_set_ip_address] = ACTIONS(1867),
    [sym_get_url_list] = ACTIONS(1867),
    [sym_add_url_entry] = ACTIONS(1867),
    [sym_delete_url_entry] = ACTIONS(1867),
    [sym_abs_value] = ACTIONS(1867),
    [sym_max_value] = ACTIONS(1867),
    [sym_min_value] = ACTIONS(1867),
    [sym_exp_value] = ACTIONS(1867),
    [sym_log_value] = ACTIONS(1867),
    [sym_log10_value] = ACTIONS(1867),
    [sym_power_value] = ACTIONS(1867),
    [sym_sqrt_value] = ACTIONS(1867),
    [sym_raw_be] = ACTIONS(1867),
    [sym_raw_le] = ACTIONS(1867),
    [sym_internal_queue_size_set] = ACTIONS(1867),
    [sym_internal_queue_size_get] = ACTIONS(1867),
    [sym_internal_threshold_set] = ACTIONS(1867),
    [sym_internal_threshold_get] = ACTIONS(1867),
    [sym_rebuilt_event] = ACTIONS(1867),
    [sym_do_custom_event] = ACTIONS(1867),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1867),
    [sym_clkmgr_set_clk_source] = ACTIONS(1867),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1867),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1867),
    [sym_clkmgr_get_timezone] = ACTIONS(1867),
    [sym_clkmgr_set_timezone] = ACTIONS(1867),
    [sym_clkmgr_get_resync_period] = ACTIONS(1867),
    [sym_clkmgr_set_resync_period] = ACTIONS(1867),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1867),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1867),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1867),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1867),
    [sym_clkmgr_get_timeservers] = ACTIONS(1867),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1867),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1867),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1867),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1867),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1867),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1867),
    [sym_set_log_level] = ACTIONS(1867),
    [sym_get_log_level] = ACTIONS(1867),
    [sym_amx_log] = ACTIONS(1867),
    [sym_smtp_server_config_set] = ACTIONS(1867),
    [sym_smtp_server_config_get] = ACTIONS(1867),
    [sym_smtp_send] = ACTIONS(1867),
    [sym_validate_netlinx_account] = ACTIONS(1867),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1867),
    [sym_authenticate_certificate] = ACTIONS(1867),
    [sym_audit_netlinx_session_event] = ACTIONS(1867),
    [sym_audit_netlinx_generic_event] = ACTIONS(1867),
    [sym_get_available_flash_disk_space] = ACTIONS(1867),
    [sym_get_max_flash_disk_space] = ACTIONS(1867),
    [sym_data_create_feed] = ACTIONS(1867),
    [sym_data_delete_feed] = ACTIONS(1867),
    [sym_data_publish_feed] = ACTIONS(1867),
    [sym_data_get_published_feed] = ACTIONS(1867),
    [sym_data_add_record] = ACTIONS(1867),
    [sym_data_get_event_record] = ACTIONS(1867),
    [sym__wc_data_create_feed] = ACTIONS(1867),
    [sym__wc_data_add_record] = ACTIONS(1867),
    [sym__wc_data_get_event_record] = ACTIONS(1867),
    [sym_format] = ACTIONS(1867),
    [sym_astro_clock] = ACTIONS(1867),
    [sym_set_outdoor_temperature] = ACTIONS(1867),
    [sym_set_virtual_level_count] = ACTIONS(1867),
    [sym_set_virtual_channel_count] = ACTIONS(1867),
    [sym_set_virtual_port_count] = ACTIONS(1867),
    [sym__wc_encode_int] = ACTIONS(1867),
    [sym__wc_decode_int] = ACTIONS(1867),
    [sym__wc_int] = ACTIONS(1867),
    [sym__wc_to_ch_int] = ACTIONS(1867),
    [sym__ch_to_wc_int] = ACTIONS(1867),
    [sym__wc_find_string_int] = ACTIONS(1867),
    [sym__wc_left_string_int] = ACTIONS(1867),
    [sym__wc_lower_string_int] = ACTIONS(1867),
    [sym__wc_mid_string_int] = ACTIONS(1867),
    [sym__wc_remove_string_int] = ACTIONS(1867),
    [sym__wc_right_string_int] = ACTIONS(1867),
    [sym__wc_upper_string_int] = ACTIONS(1867),
    [sym__wc_compare_string_int] = ACTIONS(1867),
    [sym__wc_get_buffer_char_int] = ACTIONS(1867),
    [sym__wc_get_buffer_string_int] = ACTIONS(1867),
    [sym__wc_concat_string_int] = ACTIONS(1867),
    [sym__wc_file_open_int] = ACTIONS(1867),
    [sym__wc_file_close_int] = ACTIONS(1867),
    [sym__wc_file_read_int] = ACTIONS(1867),
    [sym__wc_file_read_line_int] = ACTIONS(1867),
    [sym__wc_file_write_int] = ACTIONS(1867),
    [sym__wc_file_write_line_int] = ACTIONS(1867),
    [aux_sym_preproc_include_token1] = ACTIONS(1867),
    [aux_sym_preproc_define_token1] = ACTIONS(1869),
    [aux_sym_preproc_warn_token1] = ACTIONS(1869),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1869),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1869),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1869),
    [anon_sym_LBRACK] = ACTIONS(1869),
    [anon_sym_LBRACE] = ACTIONS(1869),
    [anon_sym_LPAREN] = ACTIONS(1867),
    [anon_sym_BANG] = ACTIONS(1869),
    [anon_sym_TILDE] = ACTIONS(1869),
    [anon_sym_DASH] = ACTIONS(1867),
    [anon_sym_PLUS] = ACTIONS(1867),
    [anon_sym_DASH_DASH] = ACTIONS(1869),
    [anon_sym_PLUS_PLUS] = ACTIONS(1869),
    [anon_sym_DQUOTE] = ACTIONS(1869),
    [anon_sym_SQUOTE] = ACTIONS(1869),
    [sym_number_literal] = ACTIONS(1869),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(322)] = {
    [sym_identifier] = ACTIONS(2013),
    [sym_program_name_keyword] = ACTIONS(2013),
    [sym_module_name_keyword] = ACTIONS(2013),
    [sym_define_device_keyword] = ACTIONS(2013),
    [sym_define_combine_keyword] = ACTIONS(2013),
    [sym_define_constant_keyword] = ACTIONS(2013),
    [sym_define_type_keyword] = ACTIONS(2013),
    [sym_define_variable_keyword] = ACTIONS(2013),
    [sym_define_system_variable_keyword] = ACTIONS(2013),
    [sym_define_connect_level_keyword] = ACTIONS(2013),
    [sym_define_latching_keyword] = ACTIONS(2013),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2013),
    [sym_define_toggling_keyword] = ACTIONS(2013),
    [sym_define_call_keyword] = ACTIONS(2013),
    [sym_define_function_keyword] = ACTIONS(2013),
    [sym_define_library_function_keyword] = ACTIONS(2013),
    [sym_define_start_keyword] = ACTIONS(2013),
    [sym_define_event_keyword] = ACTIONS(2013),
    [sym_define_program_keyword] = ACTIONS(2013),
    [sym_define_module_keyword] = ACTIONS(2013),
    [sym_char_keyword] = ACTIONS(2013),
    [sym_widechar_keyword] = ACTIONS(2013),
    [sym_integer_keyword] = ACTIONS(2013),
    [sym_sinteger_keyword] = ACTIONS(2013),
    [sym_long_keyword] = ACTIONS(2013),
    [sym_slong_keyword] = ACTIONS(2013),
    [sym_float_keyword] = ACTIONS(2013),
    [sym_double_keyword] = ACTIONS(2013),
    [sym_variant_keyword] = ACTIONS(2013),
    [sym_variantarray_keyword] = ACTIONS(2013),
    [sym_local_var_keyword] = ACTIONS(2013),
    [sym_stack_var_keyword] = ACTIONS(2013),
    [sym_constant_keyword] = ACTIONS(2013),
    [sym_volatile_keyword] = ACTIONS(2013),
    [sym_non_volatile_keyword] = ACTIONS(2013),
    [sym_persistent_keyword] = ACTIONS(2013),
    [sym_dev_keyword] = ACTIONS(2013),
    [sym_devlev_keyword] = ACTIONS(2013),
    [sym_devchan_keyword] = ACTIONS(2013),
    [sym_if_keyword] = ACTIONS(2013),
    [sym_switch_keyword] = ACTIONS(2013),
    [sym_case_keyword] = ACTIONS(2013),
    [sym_default_keyword] = ACTIONS(2013),
    [sym_while_keyword] = ACTIONS(2013),
    [sym_for_keyword] = ACTIONS(2013),
    [sym_break_keyword] = ACTIONS(2013),
    [sym_continue_keyword] = ACTIONS(2013),
    [sym_return_keyword] = ACTIONS(2013),
    [sym_select_keyword] = ACTIONS(2013),
    [sym_active_keyword] = ACTIONS(2013),
    [sym_struct_keyword] = ACTIONS(2013),
    [sym_structure_keyword] = ACTIONS(2013),
    [sym_button_event_keyword] = ACTIONS(2013),
    [sym_channel_event_keyword] = ACTIONS(2013),
    [sym_level_event_keyword] = ACTIONS(2013),
    [sym_data_event_keyword] = ACTIONS(2013),
    [sym_timeline_event_keyword] = ACTIONS(2013),
    [sym_custom_event_keyword] = ACTIONS(2013),
    [sym_push_keyword] = ACTIONS(2013),
    [sym_release_keyword] = ACTIONS(2013),
    [aux_sym_on_keyword_token1] = ACTIONS(2013),
    [aux_sym_off_keyword_token1] = ACTIONS(2013),
    [sym_send_level_keyword] = ACTIONS(2013),
    [sym_send_string_keyword] = ACTIONS(2013),
    [sym_send_command_keyword] = ACTIONS(2013),
    [sym_clear_buffer_keyword] = ACTIONS(2013),
    [sym_create_buffer_keyword] = ACTIONS(2013),
    [sym_create_multi_buffer_keyword] = ACTIONS(2013),
    [sym_call_keyword] = ACTIONS(2013),
    [sym_system_call_keyword] = ACTIONS(2013),
    [sym_devchan_to_keyword] = ACTIONS(2013),
    [sym_devchan_min_to_keyword] = ACTIONS(2013),
    [sym_devchan_total_off_keyword] = ACTIONS(2013),
    [sym_devchan_pulse_keyword] = ACTIONS(2013),
    [sym_wait_keyword] = ACTIONS(2013),
    [sym_cancel_wait_keyword] = ACTIONS(2013),
    [sym_wait_until_keyword] = ACTIONS(2013),
    [sym_cancel_wait_until_keyword] = ACTIONS(2013),
    [sym_cancel_all_wait_keyword] = ACTIONS(2013),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2013),
    [sym_not] = ACTIONS(2013),
    [sym_bnot] = ACTIONS(2013),
    [sym___file__] = ACTIONS(2013),
    [sym___date__] = ACTIONS(2013),
    [sym___ldate__] = ACTIONS(2013),
    [sym___time__] = ACTIONS(2013),
    [sym___line__] = ACTIONS(2013),
    [sym___name__] = ACTIONS(2013),
    [sym___netlinx__] = ACTIONS(2013),
    [sym_netlinx_axi_version] = ACTIONS(2013),
    [sym_url_flg_tcp] = ACTIONS(2013),
    [sym_url_flg_accinfopresent] = ACTIONS(2013),
    [sym_url_flg_temp] = ACTIONS(2013),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2013),
    [sym_url_flg_stat_mask] = ACTIONS(2013),
    [sym_url_flg_stat_lookup] = ACTIONS(2013),
    [sym_url_flg_stat_connecting] = ACTIONS(2013),
    [sym_url_flg_stat_waiting] = ACTIONS(2013),
    [sym_url_flg_stat_connected] = ACTIONS(2013),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2013),
    [sym_first_virtual_device] = ACTIONS(2013),
    [sym_first_local_port] = ACTIONS(2013),
    [sym_dynamic_virtual_device] = ACTIONS(2013),
    [sym_timeline_once] = ACTIONS(2013),
    [sym_timeline_repeat] = ACTIONS(2013),
    [sym_timeline_absolute] = ACTIONS(2013),
    [sym_timeline_relative] = ACTIONS(2013),
    [sym_do_push_timed_infinite] = ACTIONS(2013),
    [sym_source_type_no_address] = ACTIONS(2013),
    [sym_source_type_neuron_id] = ACTIONS(2013),
    [sym_source_type_ip_address] = ACTIONS(2013),
    [sym_source_type_axlink] = ACTIONS(2013),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2013),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2013),
    [sym_source_type_ip_socket_address] = ACTIONS(2013),
    [sym_source_type_rs232] = ACTIONS(2013),
    [sym_source_type_internal] = ACTIONS(2013),
    [sym_source_type_mac_address] = ACTIONS(2013),
    [sym_source_type_ipv4_port] = ACTIONS(2013),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2013),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2013),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2013),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2013),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2013),
    [sym_true] = ACTIONS(2013),
    [sym_false] = ACTIONS(2013),
    [sym_file_read_only] = ACTIONS(2013),
    [sym_file_rw_new] = ACTIONS(2013),
    [sym_file_rw_append] = ACTIONS(2013),
    [sym_ip_tcp] = ACTIONS(2013),
    [sym_ip_udp] = ACTIONS(2013),
    [sym_ip_udp_2way] = ACTIONS(2013),
    [sym_xml_encode_types] = ACTIONS(2013),
    [sym_xml_encode_char_as_list] = ACTIONS(2013),
    [sym_xml_encode_le] = ACTIONS(2013),
    [sym_xml_decode_types] = ACTIONS(2013),
    [sym_xml_decode_no_preserve] = ACTIONS(2013),
    [sym_duet_dev_type_display_device] = ACTIONS(2013),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2013),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2013),
    [sym_duet_dev_type_io_device] = ACTIONS(2013),
    [sym_duet_dev_type_relay_device] = ACTIONS(2013),
    [sym_duet_dev_type_ups] = ACTIONS(2013),
    [sym_duet_dev_type_amplifier] = ACTIONS(2013),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2013),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2013),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2013),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2013),
    [sym_duet_dev_type_camera] = ACTIONS(2013),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2013),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2013),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2013),
    [sym_duet_dev_type_dss] = ACTIONS(2013),
    [sym_duet_dev_type_dvr] = ACTIONS(2013),
    [sym_duet_dev_type_disc_device] = ACTIONS(2013),
    [sym_duet_dev_type_document_camera] = ACTIONS(2013),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2013),
    [sym_duet_dev_type_hvac] = ACTIONS(2013),
    [sym_duet_dev_type_keypad] = ACTIONS(2013),
    [sym_duet_dev_type_light] = ACTIONS(2013),
    [sym_duet_dev_type_monitor] = ACTIONS(2013),
    [sym_duet_dev_type_motor] = ACTIONS(2013),
    [sym_duet_dev_type_multi_window] = ACTIONS(2013),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2013),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2013),
    [sym_duet_dev_type_receiver] = ACTIONS(2013),
    [sym_duet_dev_type_security_system] = ACTIONS(2013),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2013),
    [sym_duet_dev_type_settop_box] = ACTIONS(2013),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2013),
    [sym_duet_dev_type_switcher] = ACTIONS(2013),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2013),
    [sym_duet_dev_type_tv] = ACTIONS(2013),
    [sym_duet_dev_type_utility] = ACTIONS(2013),
    [sym_duet_dev_type_vcr] = ACTIONS(2013),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2013),
    [sym_duet_dev_type_video_processor] = ACTIONS(2013),
    [sym_duet_dev_type_video_projector] = ACTIONS(2013),
    [sym_duet_dev_type_video_wall] = ACTIONS(2013),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2013),
    [sym_duet_dev_type_weather] = ACTIONS(2013),
    [sym_duet_dev_not_polled] = ACTIONS(2013),
    [sym_duet_dev_polled] = ACTIONS(2013),
    [sym_clkmgr_mode_network] = ACTIONS(2013),
    [sym_clkmgr_mode_standalone] = ACTIONS(2013),
    [sym_normal_standby] = ACTIONS(2013),
    [sym_normal_wake] = ACTIONS(2013),
    [sym_ip_multicast_ttl_option] = ACTIONS(2013),
    [sym_ip_ttl_subnet] = ACTIONS(2013),
    [sym_ip_ttl_site] = ACTIONS(2013),
    [sym_ip_ttl_region] = ACTIONS(2013),
    [sym_ip_ttl_continent] = ACTIONS(2013),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2013),
    [sym_ip_nodelay_on] = ACTIONS(2013),
    [sym_ip_nodelay_off] = ACTIONS(2013),
    [sym_amx_error] = ACTIONS(2013),
    [sym_amx_warning] = ACTIONS(2013),
    [sym_amx_info] = ACTIONS(2013),
    [sym_amx_debug] = ACTIONS(2013),
    [sym_netlinx_login_success] = ACTIONS(2013),
    [sym_netlinx_login_fail] = ACTIONS(2013),
    [sym_netlinx_logout] = ACTIONS(2013),
    [sym_null_str] = ACTIONS(2013),
    [sym_smtp_address] = ACTIONS(2013),
    [sym_smtp_port_number] = ACTIONS(2013),
    [sym_smtp_username] = ACTIONS(2013),
    [sym_smtp_password] = ACTIONS(2013),
    [sym_smtp_require_tls] = ACTIONS(2013),
    [sym_smtp_from] = ACTIONS(2013),
    [sym_smtp_tls_true] = ACTIONS(2013),
    [sym_smtp_tls_false] = ACTIONS(2013),
    [sym_listview_on_row_select_event] = ACTIONS(2013),
    [sym_data_structure_unknown] = ACTIONS(2013),
    [sym_data_structure_datafeed] = ACTIONS(2013),
    [sym_data_structure_datarecordset] = ACTIONS(2013),
    [sym_data_structure_datarecord] = ACTIONS(2013),
    [sym_data_structure_datafield] = ACTIONS(2013),
    [sym_data_max_content_field_count] = ACTIONS(2013),
    [sym_data_max_metadata_field_count] = ACTIONS(2013),
    [sym_data_max_id_length] = ACTIONS(2013),
    [sym_data_max_name_length] = ACTIONS(2013),
    [sym_data_max_description_length] = ACTIONS(2013),
    [sym_data_max_type_length] = ACTIONS(2013),
    [sym_data_max_format_length] = ACTIONS(2013),
    [sym_data_max_source_length] = ACTIONS(2013),
    [sym_data_max_value_length] = ACTIONS(2013),
    [sym_data_max_label_length] = ACTIONS(2013),
    [sym_data_max_url_length] = ACTIONS(2013),
    [sym_max_last_login_info_length] = ACTIONS(2013),
    [sym_data_type_unknown] = ACTIONS(2013),
    [sym_data_type_string] = ACTIONS(2013),
    [sym_data_type_datetime] = ACTIONS(2013),
    [sym_data_type_date] = ACTIONS(2013),
    [sym_data_type_time] = ACTIONS(2013),
    [sym_data_type_image] = ACTIONS(2013),
    [sym_data_format_url] = ACTIONS(2013),
    [sym_data_format_phone] = ACTIONS(2013),
    [sym_data_format_email] = ACTIONS(2013),
    [sym_data_format_iso8601] = ACTIONS(2013),
    [sym_auditlog_priv] = ACTIONS(2013),
    [sym_configuration_priv] = ACTIONS(2013),
    [sym_deviceconfig_priv] = ACTIONS(2013),
    [sym_ftp_priv] = ACTIONS(2013),
    [sym_http_priv] = ACTIONS(2013),
    [sym_networkconfig_priv] = ACTIONS(2013),
    [sym_programport_priv] = ACTIONS(2013),
    [sym_remoteui_priv] = ACTIONS(2013),
    [sym_securitycontrol_priv] = ACTIONS(2013),
    [sym_softwaremanagement_priv] = ACTIONS(2013),
    [sym_terminal_priv] = ACTIONS(2013),
    [sym_tpadmin_priv] = ACTIONS(2013),
    [sym_user1_priv] = ACTIONS(2013),
    [sym_user2_priv] = ACTIONS(2013),
    [sym_user3_priv] = ACTIONS(2013),
    [sym_user4_priv] = ACTIONS(2013),
    [sym_usermanagement_priv] = ACTIONS(2013),
    [sym_valid_account] = ACTIONS(2013),
    [sym_err_name_invalid] = ACTIONS(2013),
    [sym_err_password_invalid] = ACTIONS(2013),
    [sym_err_invalid_account] = ACTIONS(2013),
    [sym_err_no_authorization] = ACTIONS(2013),
    [sym_err_invalid_parameter] = ACTIONS(2013),
    [sym_err_user_locked_out] = ACTIONS(2013),
    [sym_err_user_account_expired] = ACTIONS(2013),
    [sym_err_general_error] = ACTIONS(2013),
    [sym_err_authentication_error] = ACTIONS(2013),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2013),
    [sym_tls_validate_certificate] = ACTIONS(2013),
    [sym_tdata] = ACTIONS(2013),
    [sym_tchannel] = ACTIONS(2013),
    [sym_tlevel] = ACTIONS(2013),
    [sym_tbutton] = ACTIONS(2013),
    [sym_ttimeline] = ACTIONS(2013),
    [sym_tcustom] = ACTIONS(2013),
    [sym_url_struct] = ACTIONS(2013),
    [sym_dns_struct] = ACTIONS(2013),
    [sym_ip_address_struct] = ACTIONS(2013),
    [sym_dev_info_struct] = ACTIONS(2013),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2013),
    [sym_data_feed] = ACTIONS(2013),
    [sym_data_field] = ACTIONS(2013),
    [sym_data_record] = ACTIONS(2013),
    [sym_wc_data_feed] = ACTIONS(2013),
    [sym_wc_data_field] = ACTIONS(2013),
    [sym_wc_data_record] = ACTIONS(2013),
    [sym_last_login_info] = ACTIONS(2013),
    [sym____reserved___] = ACTIONS(2013),
    [sym_date] = ACTIONS(2013),
    [sym_day] = ACTIONS(2013),
    [sym_time] = ACTIONS(2013),
    [sym_ldate] = ACTIONS(2013),
    [sym_push_device] = ACTIONS(2013),
    [sym_push_channel] = ACTIONS(2013),
    [sym_push_devchan] = ACTIONS(2013),
    [sym_release_device] = ACTIONS(2013),
    [sym_release_channel] = ACTIONS(2013),
    [sym_release_devchan] = ACTIONS(2013),
    [sym_master_slot] = ACTIONS(2013),
    [sym_get_pulse_time] = ACTIONS(2013),
    [sym_get_timer] = ACTIONS(2013),
    [sym_master_sn] = ACTIONS(2013),
    [sym_system_number] = ACTIONS(2013),
    [sym_button] = ACTIONS(2013),
    [sym_dv_channel] = ACTIONS(2013),
    [sym_data] = ACTIONS(2013),
    [sym_level] = ACTIONS(2013),
    [sym_timeline] = ACTIONS(2013),
    [sym_channel] = ACTIONS(2013),
    [sym_custom] = ACTIONS(2013),
    [sym_atoi] = ACTIONS(2013),
    [sym_atol] = ACTIONS(2013),
    [sym_atof] = ACTIONS(2013),
    [sym_itoa] = ACTIONS(2013),
    [sym_itohex] = ACTIONS(2013),
    [sym_hextoi] = ACTIONS(2013),
    [sym_ftoa] = ACTIONS(2013),
    [sym_find_string] = ACTIONS(2013),
    [sym_left_string] = ACTIONS(2013),
    [sym_right_string] = ACTIONS(2013),
    [sym_mid_string] = ACTIONS(2013),
    [sym_remove_string] = ACTIONS(2013),
    [sym_get_buffer_string] = ACTIONS(2013),
    [sym_get_buffer_char] = ACTIONS(2013),
    [sym_get_multi_buffer_string] = ACTIONS(2013),
    [sym_length_string] = ACTIONS(2013),
    [sym_max_length_string] = ACTIONS(2013),
    [sym_set_length_string] = ACTIONS(2013),
    [sym_compare_string] = ACTIONS(2013),
    [sym_upper_string] = ACTIONS(2013),
    [sym_lower_string] = ACTIONS(2013),
    [sym_get_last] = ACTIONS(2013),
    [sym_date_to_day] = ACTIONS(2013),
    [sym_date_to_month] = ACTIONS(2013),
    [sym_date_to_year] = ACTIONS(2013),
    [sym_time_to_hour] = ACTIONS(2013),
    [sym_time_to_minute] = ACTIONS(2013),
    [sym_time_to_second] = ACTIONS(2013),
    [sym_day_of_week] = ACTIONS(2013),
    [sym_set_pulse_time] = ACTIONS(2013),
    [sym_set_timer] = ACTIONS(2013),
    [sym_file_open] = ACTIONS(2013),
    [sym_file_close] = ACTIONS(2013),
    [sym_file_read] = ACTIONS(2013),
    [sym_file_read_line] = ACTIONS(2013),
    [sym_file_write] = ACTIONS(2013),
    [sym_file_write_line] = ACTIONS(2013),
    [sym_file_copy] = ACTIONS(2013),
    [sym_file_delete] = ACTIONS(2013),
    [sym_file_rename] = ACTIONS(2013),
    [sym_file_dir] = ACTIONS(2013),
    [sym_file_getdir] = ACTIONS(2013),
    [sym_file_setdir] = ACTIONS(2013),
    [sym_file_create_dir] = ACTIONS(2013),
    [sym_file_remove_dir] = ACTIONS(2013),
    [sym_file_seek] = ACTIONS(2013),
    [sym_length_array] = ACTIONS(2013),
    [sym_max_length_array] = ACTIONS(2013),
    [sym_set_length_array] = ACTIONS(2013),
    [sym_ip_client_open] = ACTIONS(2013),
    [sym_ip_client_close] = ACTIONS(2013),
    [sym_ip_server_open] = ACTIONS(2013),
    [sym_ip_server_close] = ACTIONS(2013),
    [sym_ip_bound_client_open] = ACTIONS(2013),
    [sym_ip_mc_server_open] = ACTIONS(2013),
    [sym_ip_set_option] = ACTIONS(2013),
    [sym_ssh_client_open] = ACTIONS(2013),
    [sym_ssh_client_close] = ACTIONS(2013),
    [sym_tls_client_open] = ACTIONS(2013),
    [sym_tls_client_close] = ACTIONS(2013),
    [sym_do_push] = ACTIONS(2013),
    [sym_do_push_timed] = ACTIONS(2013),
    [sym_do_release] = ACTIONS(2013),
    [sym_device_id] = ACTIONS(2013),
    [sym_device_info] = ACTIONS(2013),
    [sym_device_id_string] = ACTIONS(2013),
    [sym_device_standby] = ACTIONS(2013),
    [sym_device_wake] = ACTIONS(2013),
    [sym_timeline_create] = ACTIONS(2013),
    [sym_timeline_kill] = ACTIONS(2013),
    [sym_timeline_reload] = ACTIONS(2013),
    [sym_timeline_pause] = ACTIONS(2013),
    [sym_timeline_restart] = ACTIONS(2013),
    [sym_timeline_set] = ACTIONS(2013),
    [sym_timeline_get] = ACTIONS(2013),
    [sym_timeline_active] = ACTIONS(2013),
    [sym_timeline_dynamic_id] = ACTIONS(2013),
    [sym_variable_to_string] = ACTIONS(2013),
    [sym_string_to_variable] = ACTIONS(2013),
    [sym_length_variable_to_string] = ACTIONS(2013),
    [sym_type_cast] = ACTIONS(2013),
    [sym_variable_to_xml] = ACTIONS(2013),
    [sym_xml_to_variable] = ACTIONS(2013),
    [sym_length_variable_to_xml] = ACTIONS(2013),
    [sym_redirect_string] = ACTIONS(2013),
    [sym_random_number] = ACTIONS(2013),
    [sym_get_unique_id] = ACTIONS(2013),
    [sym_get_system_number] = ACTIONS(2013),
    [sym_set_system_number] = ACTIONS(2013),
    [sym_get_serial_number] = ACTIONS(2013),
    [sym_get_master_build] = ACTIONS(2013),
    [sym_set_validation_code] = ACTIONS(2013),
    [sym_reboot] = ACTIONS(2013),
    [sym_load_duet_module] = ACTIONS(2013),
    [sym_unload_duet_module] = ACTIONS(2013),
    [sym_duet_mem_size_set] = ACTIONS(2013),
    [sym_duet_mem_size_get] = ACTIONS(2013),
    [sym_dynamic_application_device] = ACTIONS(2013),
    [sym_dynamic_polled_port] = ACTIONS(2013),
    [sym_static_port_binding] = ACTIONS(2013),
    [sym_static_ip_binding] = ACTIONS(2013),
    [sym_get_dns_list] = ACTIONS(2013),
    [sym_set_dns_list] = ACTIONS(2013),
    [sym_get_ip_address] = ACTIONS(2013),
    [sym_set_ip_address] = ACTIONS(2013),
    [sym_get_url_list] = ACTIONS(2013),
    [sym_add_url_entry] = ACTIONS(2013),
    [sym_delete_url_entry] = ACTIONS(2013),
    [sym_abs_value] = ACTIONS(2013),
    [sym_max_value] = ACTIONS(2013),
    [sym_min_value] = ACTIONS(2013),
    [sym_exp_value] = ACTIONS(2013),
    [sym_log_value] = ACTIONS(2013),
    [sym_log10_value] = ACTIONS(2013),
    [sym_power_value] = ACTIONS(2013),
    [sym_sqrt_value] = ACTIONS(2013),
    [sym_raw_be] = ACTIONS(2013),
    [sym_raw_le] = ACTIONS(2013),
    [sym_internal_queue_size_set] = ACTIONS(2013),
    [sym_internal_queue_size_get] = ACTIONS(2013),
    [sym_internal_threshold_set] = ACTIONS(2013),
    [sym_internal_threshold_get] = ACTIONS(2013),
    [sym_rebuilt_event] = ACTIONS(2013),
    [sym_do_custom_event] = ACTIONS(2013),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2013),
    [sym_clkmgr_set_clk_source] = ACTIONS(2013),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2013),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2013),
    [sym_clkmgr_get_timezone] = ACTIONS(2013),
    [sym_clkmgr_set_timezone] = ACTIONS(2013),
    [sym_clkmgr_get_resync_period] = ACTIONS(2013),
    [sym_clkmgr_set_resync_period] = ACTIONS(2013),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2013),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2013),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2013),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2013),
    [sym_clkmgr_get_timeservers] = ACTIONS(2013),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2013),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2013),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2013),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2013),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2013),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2013),
    [sym_set_log_level] = ACTIONS(2013),
    [sym_get_log_level] = ACTIONS(2013),
    [sym_amx_log] = ACTIONS(2013),
    [sym_smtp_server_config_set] = ACTIONS(2013),
    [sym_smtp_server_config_get] = ACTIONS(2013),
    [sym_smtp_send] = ACTIONS(2013),
    [sym_validate_netlinx_account] = ACTIONS(2013),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2013),
    [sym_authenticate_certificate] = ACTIONS(2013),
    [sym_audit_netlinx_session_event] = ACTIONS(2013),
    [sym_audit_netlinx_generic_event] = ACTIONS(2013),
    [sym_get_available_flash_disk_space] = ACTIONS(2013),
    [sym_get_max_flash_disk_space] = ACTIONS(2013),
    [sym_data_create_feed] = ACTIONS(2013),
    [sym_data_delete_feed] = ACTIONS(2013),
    [sym_data_publish_feed] = ACTIONS(2013),
    [sym_data_get_published_feed] = ACTIONS(2013),
    [sym_data_add_record] = ACTIONS(2013),
    [sym_data_get_event_record] = ACTIONS(2013),
    [sym__wc_data_create_feed] = ACTIONS(2013),
    [sym__wc_data_add_record] = ACTIONS(2013),
    [sym__wc_data_get_event_record] = ACTIONS(2013),
    [sym_format] = ACTIONS(2013),
    [sym_astro_clock] = ACTIONS(2013),
    [sym_set_outdoor_temperature] = ACTIONS(2013),
    [sym_set_virtual_level_count] = ACTIONS(2013),
    [sym_set_virtual_channel_count] = ACTIONS(2013),
    [sym_set_virtual_port_count] = ACTIONS(2013),
    [sym__wc_encode_int] = ACTIONS(2013),
    [sym__wc_decode_int] = ACTIONS(2013),
    [sym__wc_int] = ACTIONS(2013),
    [sym__wc_to_ch_int] = ACTIONS(2013),
    [sym__ch_to_wc_int] = ACTIONS(2013),
    [sym__wc_find_string_int] = ACTIONS(2013),
    [sym__wc_left_string_int] = ACTIONS(2013),
    [sym__wc_lower_string_int] = ACTIONS(2013),
    [sym__wc_mid_string_int] = ACTIONS(2013),
    [sym__wc_remove_string_int] = ACTIONS(2013),
    [sym__wc_right_string_int] = ACTIONS(2013),
    [sym__wc_upper_string_int] = ACTIONS(2013),
    [sym__wc_compare_string_int] = ACTIONS(2013),
    [sym__wc_get_buffer_char_int] = ACTIONS(2013),
    [sym__wc_get_buffer_string_int] = ACTIONS(2013),
    [sym__wc_concat_string_int] = ACTIONS(2013),
    [sym__wc_file_open_int] = ACTIONS(2013),
    [sym__wc_file_close_int] = ACTIONS(2013),
    [sym__wc_file_read_int] = ACTIONS(2013),
    [sym__wc_file_read_line_int] = ACTIONS(2013),
    [sym__wc_file_write_int] = ACTIONS(2013),
    [sym__wc_file_write_line_int] = ACTIONS(2013),
    [aux_sym_preproc_include_token1] = ACTIONS(2013),
    [aux_sym_preproc_define_token1] = ACTIONS(2015),
    [aux_sym_preproc_warn_token1] = ACTIONS(2015),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2015),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2015),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2015),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2015),
    [aux_sym_preproc_else_token1] = ACTIONS(2015),
    [anon_sym_LBRACK] = ACTIONS(2015),
    [anon_sym_LBRACE] = ACTIONS(2015),
    [anon_sym_LPAREN] = ACTIONS(2013),
    [anon_sym_BANG] = ACTIONS(2015),
    [anon_sym_TILDE] = ACTIONS(2015),
    [anon_sym_DASH] = ACTIONS(2013),
    [anon_sym_PLUS] = ACTIONS(2013),
    [anon_sym_DASH_DASH] = ACTIONS(2015),
    [anon_sym_PLUS_PLUS] = ACTIONS(2015),
    [anon_sym_DQUOTE] = ACTIONS(2015),
    [anon_sym_SQUOTE] = ACTIONS(2015),
    [sym_number_literal] = ACTIONS(2015),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(323)] = {
    [sym_identifier] = ACTIONS(2017),
    [sym_program_name_keyword] = ACTIONS(2017),
    [sym_module_name_keyword] = ACTIONS(2017),
    [sym_define_device_keyword] = ACTIONS(2017),
    [sym_define_combine_keyword] = ACTIONS(2017),
    [sym_define_constant_keyword] = ACTIONS(2017),
    [sym_define_type_keyword] = ACTIONS(2017),
    [sym_define_variable_keyword] = ACTIONS(2017),
    [sym_define_system_variable_keyword] = ACTIONS(2017),
    [sym_define_connect_level_keyword] = ACTIONS(2017),
    [sym_define_latching_keyword] = ACTIONS(2017),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2017),
    [sym_define_toggling_keyword] = ACTIONS(2017),
    [sym_define_call_keyword] = ACTIONS(2017),
    [sym_define_function_keyword] = ACTIONS(2017),
    [sym_define_library_function_keyword] = ACTIONS(2017),
    [sym_define_start_keyword] = ACTIONS(2017),
    [sym_define_event_keyword] = ACTIONS(2017),
    [sym_define_program_keyword] = ACTIONS(2017),
    [sym_define_module_keyword] = ACTIONS(2017),
    [sym_char_keyword] = ACTIONS(2017),
    [sym_widechar_keyword] = ACTIONS(2017),
    [sym_integer_keyword] = ACTIONS(2017),
    [sym_sinteger_keyword] = ACTIONS(2017),
    [sym_long_keyword] = ACTIONS(2017),
    [sym_slong_keyword] = ACTIONS(2017),
    [sym_float_keyword] = ACTIONS(2017),
    [sym_double_keyword] = ACTIONS(2017),
    [sym_variant_keyword] = ACTIONS(2017),
    [sym_variantarray_keyword] = ACTIONS(2017),
    [sym_local_var_keyword] = ACTIONS(2017),
    [sym_stack_var_keyword] = ACTIONS(2017),
    [sym_constant_keyword] = ACTIONS(2017),
    [sym_volatile_keyword] = ACTIONS(2017),
    [sym_non_volatile_keyword] = ACTIONS(2017),
    [sym_persistent_keyword] = ACTIONS(2017),
    [sym_dev_keyword] = ACTIONS(2017),
    [sym_devlev_keyword] = ACTIONS(2017),
    [sym_devchan_keyword] = ACTIONS(2017),
    [sym_if_keyword] = ACTIONS(2017),
    [sym_switch_keyword] = ACTIONS(2017),
    [sym_case_keyword] = ACTIONS(2017),
    [sym_default_keyword] = ACTIONS(2017),
    [sym_while_keyword] = ACTIONS(2017),
    [sym_for_keyword] = ACTIONS(2017),
    [sym_break_keyword] = ACTIONS(2017),
    [sym_continue_keyword] = ACTIONS(2017),
    [sym_return_keyword] = ACTIONS(2017),
    [sym_select_keyword] = ACTIONS(2017),
    [sym_active_keyword] = ACTIONS(2017),
    [sym_struct_keyword] = ACTIONS(2017),
    [sym_structure_keyword] = ACTIONS(2017),
    [sym_button_event_keyword] = ACTIONS(2017),
    [sym_channel_event_keyword] = ACTIONS(2017),
    [sym_level_event_keyword] = ACTIONS(2017),
    [sym_data_event_keyword] = ACTIONS(2017),
    [sym_timeline_event_keyword] = ACTIONS(2017),
    [sym_custom_event_keyword] = ACTIONS(2017),
    [sym_push_keyword] = ACTIONS(2017),
    [sym_release_keyword] = ACTIONS(2017),
    [aux_sym_on_keyword_token1] = ACTIONS(2017),
    [aux_sym_off_keyword_token1] = ACTIONS(2017),
    [sym_send_level_keyword] = ACTIONS(2017),
    [sym_send_string_keyword] = ACTIONS(2017),
    [sym_send_command_keyword] = ACTIONS(2017),
    [sym_clear_buffer_keyword] = ACTIONS(2017),
    [sym_create_buffer_keyword] = ACTIONS(2017),
    [sym_create_multi_buffer_keyword] = ACTIONS(2017),
    [sym_call_keyword] = ACTIONS(2017),
    [sym_system_call_keyword] = ACTIONS(2017),
    [sym_devchan_to_keyword] = ACTIONS(2017),
    [sym_devchan_min_to_keyword] = ACTIONS(2017),
    [sym_devchan_total_off_keyword] = ACTIONS(2017),
    [sym_devchan_pulse_keyword] = ACTIONS(2017),
    [sym_wait_keyword] = ACTIONS(2017),
    [sym_cancel_wait_keyword] = ACTIONS(2017),
    [sym_wait_until_keyword] = ACTIONS(2017),
    [sym_cancel_wait_until_keyword] = ACTIONS(2017),
    [sym_cancel_all_wait_keyword] = ACTIONS(2017),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2017),
    [sym_not] = ACTIONS(2017),
    [sym_bnot] = ACTIONS(2017),
    [sym___file__] = ACTIONS(2017),
    [sym___date__] = ACTIONS(2017),
    [sym___ldate__] = ACTIONS(2017),
    [sym___time__] = ACTIONS(2017),
    [sym___line__] = ACTIONS(2017),
    [sym___name__] = ACTIONS(2017),
    [sym___netlinx__] = ACTIONS(2017),
    [sym_netlinx_axi_version] = ACTIONS(2017),
    [sym_url_flg_tcp] = ACTIONS(2017),
    [sym_url_flg_accinfopresent] = ACTIONS(2017),
    [sym_url_flg_temp] = ACTIONS(2017),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2017),
    [sym_url_flg_stat_mask] = ACTIONS(2017),
    [sym_url_flg_stat_lookup] = ACTIONS(2017),
    [sym_url_flg_stat_connecting] = ACTIONS(2017),
    [sym_url_flg_stat_waiting] = ACTIONS(2017),
    [sym_url_flg_stat_connected] = ACTIONS(2017),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2017),
    [sym_first_virtual_device] = ACTIONS(2017),
    [sym_first_local_port] = ACTIONS(2017),
    [sym_dynamic_virtual_device] = ACTIONS(2017),
    [sym_timeline_once] = ACTIONS(2017),
    [sym_timeline_repeat] = ACTIONS(2017),
    [sym_timeline_absolute] = ACTIONS(2017),
    [sym_timeline_relative] = ACTIONS(2017),
    [sym_do_push_timed_infinite] = ACTIONS(2017),
    [sym_source_type_no_address] = ACTIONS(2017),
    [sym_source_type_neuron_id] = ACTIONS(2017),
    [sym_source_type_ip_address] = ACTIONS(2017),
    [sym_source_type_axlink] = ACTIONS(2017),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2017),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2017),
    [sym_source_type_ip_socket_address] = ACTIONS(2017),
    [sym_source_type_rs232] = ACTIONS(2017),
    [sym_source_type_internal] = ACTIONS(2017),
    [sym_source_type_mac_address] = ACTIONS(2017),
    [sym_source_type_ipv4_port] = ACTIONS(2017),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2017),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2017),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2017),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2017),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2017),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2017),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2017),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2017),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2017),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2017),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2017),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2017),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2017),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2017),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2017),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2017),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2017),
    [sym_true] = ACTIONS(2017),
    [sym_false] = ACTIONS(2017),
    [sym_file_read_only] = ACTIONS(2017),
    [sym_file_rw_new] = ACTIONS(2017),
    [sym_file_rw_append] = ACTIONS(2017),
    [sym_ip_tcp] = ACTIONS(2017),
    [sym_ip_udp] = ACTIONS(2017),
    [sym_ip_udp_2way] = ACTIONS(2017),
    [sym_xml_encode_types] = ACTIONS(2017),
    [sym_xml_encode_char_as_list] = ACTIONS(2017),
    [sym_xml_encode_le] = ACTIONS(2017),
    [sym_xml_decode_types] = ACTIONS(2017),
    [sym_xml_decode_no_preserve] = ACTIONS(2017),
    [sym_duet_dev_type_display_device] = ACTIONS(2017),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2017),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2017),
    [sym_duet_dev_type_io_device] = ACTIONS(2017),
    [sym_duet_dev_type_relay_device] = ACTIONS(2017),
    [sym_duet_dev_type_ups] = ACTIONS(2017),
    [sym_duet_dev_type_amplifier] = ACTIONS(2017),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2017),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2017),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2017),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2017),
    [sym_duet_dev_type_camera] = ACTIONS(2017),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2017),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2017),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2017),
    [sym_duet_dev_type_dss] = ACTIONS(2017),
    [sym_duet_dev_type_dvr] = ACTIONS(2017),
    [sym_duet_dev_type_disc_device] = ACTIONS(2017),
    [sym_duet_dev_type_document_camera] = ACTIONS(2017),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2017),
    [sym_duet_dev_type_hvac] = ACTIONS(2017),
    [sym_duet_dev_type_keypad] = ACTIONS(2017),
    [sym_duet_dev_type_light] = ACTIONS(2017),
    [sym_duet_dev_type_monitor] = ACTIONS(2017),
    [sym_duet_dev_type_motor] = ACTIONS(2017),
    [sym_duet_dev_type_multi_window] = ACTIONS(2017),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2017),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2017),
    [sym_duet_dev_type_receiver] = ACTIONS(2017),
    [sym_duet_dev_type_security_system] = ACTIONS(2017),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2017),
    [sym_duet_dev_type_settop_box] = ACTIONS(2017),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2017),
    [sym_duet_dev_type_switcher] = ACTIONS(2017),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2017),
    [sym_duet_dev_type_tv] = ACTIONS(2017),
    [sym_duet_dev_type_utility] = ACTIONS(2017),
    [sym_duet_dev_type_vcr] = ACTIONS(2017),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2017),
    [sym_duet_dev_type_video_processor] = ACTIONS(2017),
    [sym_duet_dev_type_video_projector] = ACTIONS(2017),
    [sym_duet_dev_type_video_wall] = ACTIONS(2017),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2017),
    [sym_duet_dev_type_weather] = ACTIONS(2017),
    [sym_duet_dev_not_polled] = ACTIONS(2017),
    [sym_duet_dev_polled] = ACTIONS(2017),
    [sym_clkmgr_mode_network] = ACTIONS(2017),
    [sym_clkmgr_mode_standalone] = ACTIONS(2017),
    [sym_normal_standby] = ACTIONS(2017),
    [sym_normal_wake] = ACTIONS(2017),
    [sym_ip_multicast_ttl_option] = ACTIONS(2017),
    [sym_ip_ttl_subnet] = ACTIONS(2017),
    [sym_ip_ttl_site] = ACTIONS(2017),
    [sym_ip_ttl_region] = ACTIONS(2017),
    [sym_ip_ttl_continent] = ACTIONS(2017),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2017),
    [sym_ip_nodelay_on] = ACTIONS(2017),
    [sym_ip_nodelay_off] = ACTIONS(2017),
    [sym_amx_error] = ACTIONS(2017),
    [sym_amx_warning] = ACTIONS(2017),
    [sym_amx_info] = ACTIONS(2017),
    [sym_amx_debug] = ACTIONS(2017),
    [sym_netlinx_login_success] = ACTIONS(2017),
    [sym_netlinx_login_fail] = ACTIONS(2017),
    [sym_netlinx_logout] = ACTIONS(2017),
    [sym_null_str] = ACTIONS(2017),
    [sym_smtp_address] = ACTIONS(2017),
    [sym_smtp_port_number] = ACTIONS(2017),
    [sym_smtp_username] = ACTIONS(2017),
    [sym_smtp_password] = ACTIONS(2017),
    [sym_smtp_require_tls] = ACTIONS(2017),
    [sym_smtp_from] = ACTIONS(2017),
    [sym_smtp_tls_true] = ACTIONS(2017),
    [sym_smtp_tls_false] = ACTIONS(2017),
    [sym_listview_on_row_select_event] = ACTIONS(2017),
    [sym_data_structure_unknown] = ACTIONS(2017),
    [sym_data_structure_datafeed] = ACTIONS(2017),
    [sym_data_structure_datarecordset] = ACTIONS(2017),
    [sym_data_structure_datarecord] = ACTIONS(2017),
    [sym_data_structure_datafield] = ACTIONS(2017),
    [sym_data_max_content_field_count] = ACTIONS(2017),
    [sym_data_max_metadata_field_count] = ACTIONS(2017),
    [sym_data_max_id_length] = ACTIONS(2017),
    [sym_data_max_name_length] = ACTIONS(2017),
    [sym_data_max_description_length] = ACTIONS(2017),
    [sym_data_max_type_length] = ACTIONS(2017),
    [sym_data_max_format_length] = ACTIONS(2017),
    [sym_data_max_source_length] = ACTIONS(2017),
    [sym_data_max_value_length] = ACTIONS(2017),
    [sym_data_max_label_length] = ACTIONS(2017),
    [sym_data_max_url_length] = ACTIONS(2017),
    [sym_max_last_login_info_length] = ACTIONS(2017),
    [sym_data_type_unknown] = ACTIONS(2017),
    [sym_data_type_string] = ACTIONS(2017),
    [sym_data_type_datetime] = ACTIONS(2017),
    [sym_data_type_date] = ACTIONS(2017),
    [sym_data_type_time] = ACTIONS(2017),
    [sym_data_type_image] = ACTIONS(2017),
    [sym_data_format_url] = ACTIONS(2017),
    [sym_data_format_phone] = ACTIONS(2017),
    [sym_data_format_email] = ACTIONS(2017),
    [sym_data_format_iso8601] = ACTIONS(2017),
    [sym_auditlog_priv] = ACTIONS(2017),
    [sym_configuration_priv] = ACTIONS(2017),
    [sym_deviceconfig_priv] = ACTIONS(2017),
    [sym_ftp_priv] = ACTIONS(2017),
    [sym_http_priv] = ACTIONS(2017),
    [sym_networkconfig_priv] = ACTIONS(2017),
    [sym_programport_priv] = ACTIONS(2017),
    [sym_remoteui_priv] = ACTIONS(2017),
    [sym_securitycontrol_priv] = ACTIONS(2017),
    [sym_softwaremanagement_priv] = ACTIONS(2017),
    [sym_terminal_priv] = ACTIONS(2017),
    [sym_tpadmin_priv] = ACTIONS(2017),
    [sym_user1_priv] = ACTIONS(2017),
    [sym_user2_priv] = ACTIONS(2017),
    [sym_user3_priv] = ACTIONS(2017),
    [sym_user4_priv] = ACTIONS(2017),
    [sym_usermanagement_priv] = ACTIONS(2017),
    [sym_valid_account] = ACTIONS(2017),
    [sym_err_name_invalid] = ACTIONS(2017),
    [sym_err_password_invalid] = ACTIONS(2017),
    [sym_err_invalid_account] = ACTIONS(2017),
    [sym_err_no_authorization] = ACTIONS(2017),
    [sym_err_invalid_parameter] = ACTIONS(2017),
    [sym_err_user_locked_out] = ACTIONS(2017),
    [sym_err_user_account_expired] = ACTIONS(2017),
    [sym_err_general_error] = ACTIONS(2017),
    [sym_err_authentication_error] = ACTIONS(2017),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2017),
    [sym_tls_validate_certificate] = ACTIONS(2017),
    [sym_tdata] = ACTIONS(2017),
    [sym_tchannel] = ACTIONS(2017),
    [sym_tlevel] = ACTIONS(2017),
    [sym_tbutton] = ACTIONS(2017),
    [sym_ttimeline] = ACTIONS(2017),
    [sym_tcustom] = ACTIONS(2017),
    [sym_url_struct] = ACTIONS(2017),
    [sym_dns_struct] = ACTIONS(2017),
    [sym_ip_address_struct] = ACTIONS(2017),
    [sym_dev_info_struct] = ACTIONS(2017),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2017),
    [sym_data_feed] = ACTIONS(2017),
    [sym_data_field] = ACTIONS(2017),
    [sym_data_record] = ACTIONS(2017),
    [sym_wc_data_feed] = ACTIONS(2017),
    [sym_wc_data_field] = ACTIONS(2017),
    [sym_wc_data_record] = ACTIONS(2017),
    [sym_last_login_info] = ACTIONS(2017),
    [sym____reserved___] = ACTIONS(2017),
    [sym_date] = ACTIONS(2017),
    [sym_day] = ACTIONS(2017),
    [sym_time] = ACTIONS(2017),
    [sym_ldate] = ACTIONS(2017),
    [sym_push_device] = ACTIONS(2017),
    [sym_push_channel] = ACTIONS(2017),
    [sym_push_devchan] = ACTIONS(2017),
    [sym_release_device] = ACTIONS(2017),
    [sym_release_channel] = ACTIONS(2017),
    [sym_release_devchan] = ACTIONS(2017),
    [sym_master_slot] = ACTIONS(2017),
    [sym_get_pulse_time] = ACTIONS(2017),
    [sym_get_timer] = ACTIONS(2017),
    [sym_master_sn] = ACTIONS(2017),
    [sym_system_number] = ACTIONS(2017),
    [sym_button] = ACTIONS(2017),
    [sym_dv_channel] = ACTIONS(2017),
    [sym_data] = ACTIONS(2017),
    [sym_level] = ACTIONS(2017),
    [sym_timeline] = ACTIONS(2017),
    [sym_channel] = ACTIONS(2017),
    [sym_custom] = ACTIONS(2017),
    [sym_atoi] = ACTIONS(2017),
    [sym_atol] = ACTIONS(2017),
    [sym_atof] = ACTIONS(2017),
    [sym_itoa] = ACTIONS(2017),
    [sym_itohex] = ACTIONS(2017),
    [sym_hextoi] = ACTIONS(2017),
    [sym_ftoa] = ACTIONS(2017),
    [sym_find_string] = ACTIONS(2017),
    [sym_left_string] = ACTIONS(2017),
    [sym_right_string] = ACTIONS(2017),
    [sym_mid_string] = ACTIONS(2017),
    [sym_remove_string] = ACTIONS(2017),
    [sym_get_buffer_string] = ACTIONS(2017),
    [sym_get_buffer_char] = ACTIONS(2017),
    [sym_get_multi_buffer_string] = ACTIONS(2017),
    [sym_length_string] = ACTIONS(2017),
    [sym_max_length_string] = ACTIONS(2017),
    [sym_set_length_string] = ACTIONS(2017),
    [sym_compare_string] = ACTIONS(2017),
    [sym_upper_string] = ACTIONS(2017),
    [sym_lower_string] = ACTIONS(2017),
    [sym_get_last] = ACTIONS(2017),
    [sym_date_to_day] = ACTIONS(2017),
    [sym_date_to_month] = ACTIONS(2017),
    [sym_date_to_year] = ACTIONS(2017),
    [sym_time_to_hour] = ACTIONS(2017),
    [sym_time_to_minute] = ACTIONS(2017),
    [sym_time_to_second] = ACTIONS(2017),
    [sym_day_of_week] = ACTIONS(2017),
    [sym_set_pulse_time] = ACTIONS(2017),
    [sym_set_timer] = ACTIONS(2017),
    [sym_file_open] = ACTIONS(2017),
    [sym_file_close] = ACTIONS(2017),
    [sym_file_read] = ACTIONS(2017),
    [sym_file_read_line] = ACTIONS(2017),
    [sym_file_write] = ACTIONS(2017),
    [sym_file_write_line] = ACTIONS(2017),
    [sym_file_copy] = ACTIONS(2017),
    [sym_file_delete] = ACTIONS(2017),
    [sym_file_rename] = ACTIONS(2017),
    [sym_file_dir] = ACTIONS(2017),
    [sym_file_getdir] = ACTIONS(2017),
    [sym_file_setdir] = ACTIONS(2017),
    [sym_file_create_dir] = ACTIONS(2017),
    [sym_file_remove_dir] = ACTIONS(2017),
    [sym_file_seek] = ACTIONS(2017),
    [sym_length_array] = ACTIONS(2017),
    [sym_max_length_array] = ACTIONS(2017),
    [sym_set_length_array] = ACTIONS(2017),
    [sym_ip_client_open] = ACTIONS(2017),
    [sym_ip_client_close] = ACTIONS(2017),
    [sym_ip_server_open] = ACTIONS(2017),
    [sym_ip_server_close] = ACTIONS(2017),
    [sym_ip_bound_client_open] = ACTIONS(2017),
    [sym_ip_mc_server_open] = ACTIONS(2017),
    [sym_ip_set_option] = ACTIONS(2017),
    [sym_ssh_client_open] = ACTIONS(2017),
    [sym_ssh_client_close] = ACTIONS(2017),
    [sym_tls_client_open] = ACTIONS(2017),
    [sym_tls_client_close] = ACTIONS(2017),
    [sym_do_push] = ACTIONS(2017),
    [sym_do_push_timed] = ACTIONS(2017),
    [sym_do_release] = ACTIONS(2017),
    [sym_device_id] = ACTIONS(2017),
    [sym_device_info] = ACTIONS(2017),
    [sym_device_id_string] = ACTIONS(2017),
    [sym_device_standby] = ACTIONS(2017),
    [sym_device_wake] = ACTIONS(2017),
    [sym_timeline_create] = ACTIONS(2017),
    [sym_timeline_kill] = ACTIONS(2017),
    [sym_timeline_reload] = ACTIONS(2017),
    [sym_timeline_pause] = ACTIONS(2017),
    [sym_timeline_restart] = ACTIONS(2017),
    [sym_timeline_set] = ACTIONS(2017),
    [sym_timeline_get] = ACTIONS(2017),
    [sym_timeline_active] = ACTIONS(2017),
    [sym_timeline_dynamic_id] = ACTIONS(2017),
    [sym_variable_to_string] = ACTIONS(2017),
    [sym_string_to_variable] = ACTIONS(2017),
    [sym_length_variable_to_string] = ACTIONS(2017),
    [sym_type_cast] = ACTIONS(2017),
    [sym_variable_to_xml] = ACTIONS(2017),
    [sym_xml_to_variable] = ACTIONS(2017),
    [sym_length_variable_to_xml] = ACTIONS(2017),
    [sym_redirect_string] = ACTIONS(2017),
    [sym_random_number] = ACTIONS(2017),
    [sym_get_unique_id] = ACTIONS(2017),
    [sym_get_system_number] = ACTIONS(2017),
    [sym_set_system_number] = ACTIONS(2017),
    [sym_get_serial_number] = ACTIONS(2017),
    [sym_get_master_build] = ACTIONS(2017),
    [sym_set_validation_code] = ACTIONS(2017),
    [sym_reboot] = ACTIONS(2017),
    [sym_load_duet_module] = ACTIONS(2017),
    [sym_unload_duet_module] = ACTIONS(2017),
    [sym_duet_mem_size_set] = ACTIONS(2017),
    [sym_duet_mem_size_get] = ACTIONS(2017),
    [sym_dynamic_application_device] = ACTIONS(2017),
    [sym_dynamic_polled_port] = ACTIONS(2017),
    [sym_static_port_binding] = ACTIONS(2017),
    [sym_static_ip_binding] = ACTIONS(2017),
    [sym_get_dns_list] = ACTIONS(2017),
    [sym_set_dns_list] = ACTIONS(2017),
    [sym_get_ip_address] = ACTIONS(2017),
    [sym_set_ip_address] = ACTIONS(2017),
    [sym_get_url_list] = ACTIONS(2017),
    [sym_add_url_entry] = ACTIONS(2017),
    [sym_delete_url_entry] = ACTIONS(2017),
    [sym_abs_value] = ACTIONS(2017),
    [sym_max_value] = ACTIONS(2017),
    [sym_min_value] = ACTIONS(2017),
    [sym_exp_value] = ACTIONS(2017),
    [sym_log_value] = ACTIONS(2017),
    [sym_log10_value] = ACTIONS(2017),
    [sym_power_value] = ACTIONS(2017),
    [sym_sqrt_value] = ACTIONS(2017),
    [sym_raw_be] = ACTIONS(2017),
    [sym_raw_le] = ACTIONS(2017),
    [sym_internal_queue_size_set] = ACTIONS(2017),
    [sym_internal_queue_size_get] = ACTIONS(2017),
    [sym_internal_threshold_set] = ACTIONS(2017),
    [sym_internal_threshold_get] = ACTIONS(2017),
    [sym_rebuilt_event] = ACTIONS(2017),
    [sym_do_custom_event] = ACTIONS(2017),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2017),
    [sym_clkmgr_set_clk_source] = ACTIONS(2017),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2017),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2017),
    [sym_clkmgr_get_timezone] = ACTIONS(2017),
    [sym_clkmgr_set_timezone] = ACTIONS(2017),
    [sym_clkmgr_get_resync_period] = ACTIONS(2017),
    [sym_clkmgr_set_resync_period] = ACTIONS(2017),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2017),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2017),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2017),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2017),
    [sym_clkmgr_get_timeservers] = ACTIONS(2017),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2017),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2017),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2017),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2017),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2017),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2017),
    [sym_set_log_level] = ACTIONS(2017),
    [sym_get_log_level] = ACTIONS(2017),
    [sym_amx_log] = ACTIONS(2017),
    [sym_smtp_server_config_set] = ACTIONS(2017),
    [sym_smtp_server_config_get] = ACTIONS(2017),
    [sym_smtp_send] = ACTIONS(2017),
    [sym_validate_netlinx_account] = ACTIONS(2017),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2017),
    [sym_authenticate_certificate] = ACTIONS(2017),
    [sym_audit_netlinx_session_event] = ACTIONS(2017),
    [sym_audit_netlinx_generic_event] = ACTIONS(2017),
    [sym_get_available_flash_disk_space] = ACTIONS(2017),
    [sym_get_max_flash_disk_space] = ACTIONS(2017),
    [sym_data_create_feed] = ACTIONS(2017),
    [sym_data_delete_feed] = ACTIONS(2017),
    [sym_data_publish_feed] = ACTIONS(2017),
    [sym_data_get_published_feed] = ACTIONS(2017),
    [sym_data_add_record] = ACTIONS(2017),
    [sym_data_get_event_record] = ACTIONS(2017),
    [sym__wc_data_create_feed] = ACTIONS(2017),
    [sym__wc_data_add_record] = ACTIONS(2017),
    [sym__wc_data_get_event_record] = ACTIONS(2017),
    [sym_format] = ACTIONS(2017),
    [sym_astro_clock] = ACTIONS(2017),
    [sym_set_outdoor_temperature] = ACTIONS(2017),
    [sym_set_virtual_level_count] = ACTIONS(2017),
    [sym_set_virtual_channel_count] = ACTIONS(2017),
    [sym_set_virtual_port_count] = ACTIONS(2017),
    [sym__wc_encode_int] = ACTIONS(2017),
    [sym__wc_decode_int] = ACTIONS(2017),
    [sym__wc_int] = ACTIONS(2017),
    [sym__wc_to_ch_int] = ACTIONS(2017),
    [sym__ch_to_wc_int] = ACTIONS(2017),
    [sym__wc_find_string_int] = ACTIONS(2017),
    [sym__wc_left_string_int] = ACTIONS(2017),
    [sym__wc_lower_string_int] = ACTIONS(2017),
    [sym__wc_mid_string_int] = ACTIONS(2017),
    [sym__wc_remove_string_int] = ACTIONS(2017),
    [sym__wc_right_string_int] = ACTIONS(2017),
    [sym__wc_upper_string_int] = ACTIONS(2017),
    [sym__wc_compare_string_int] = ACTIONS(2017),
    [sym__wc_get_buffer_char_int] = ACTIONS(2017),
    [sym__wc_get_buffer_string_int] = ACTIONS(2017),
    [sym__wc_concat_string_int] = ACTIONS(2017),
    [sym__wc_file_open_int] = ACTIONS(2017),
    [sym__wc_file_close_int] = ACTIONS(2017),
    [sym__wc_file_read_int] = ACTIONS(2017),
    [sym__wc_file_read_line_int] = ACTIONS(2017),
    [sym__wc_file_write_int] = ACTIONS(2017),
    [sym__wc_file_write_line_int] = ACTIONS(2017),
    [aux_sym_preproc_include_token1] = ACTIONS(2017),
    [aux_sym_preproc_define_token1] = ACTIONS(2019),
    [aux_sym_preproc_warn_token1] = ACTIONS(2019),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2019),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2019),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2019),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2019),
    [aux_sym_preproc_else_token1] = ACTIONS(2019),
    [anon_sym_LBRACK] = ACTIONS(2019),
    [anon_sym_LBRACE] = ACTIONS(2019),
    [anon_sym_LPAREN] = ACTIONS(2017),
    [anon_sym_BANG] = ACTIONS(2019),
    [anon_sym_TILDE] = ACTIONS(2019),
    [anon_sym_DASH] = ACTIONS(2017),
    [anon_sym_PLUS] = ACTIONS(2017),
    [anon_sym_DASH_DASH] = ACTIONS(2019),
    [anon_sym_PLUS_PLUS] = ACTIONS(2019),
    [anon_sym_DQUOTE] = ACTIONS(2019),
    [anon_sym_SQUOTE] = ACTIONS(2019),
    [sym_number_literal] = ACTIONS(2019),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(324)] = {
    [ts_builtin_sym_end] = ACTIONS(1779),
    [sym_identifier] = ACTIONS(1777),
    [sym_program_name_keyword] = ACTIONS(1777),
    [sym_module_name_keyword] = ACTIONS(1777),
    [sym_define_device_keyword] = ACTIONS(1777),
    [sym_define_combine_keyword] = ACTIONS(1777),
    [sym_define_constant_keyword] = ACTIONS(1777),
    [sym_define_type_keyword] = ACTIONS(1777),
    [sym_define_variable_keyword] = ACTIONS(1777),
    [sym_define_system_variable_keyword] = ACTIONS(1777),
    [sym_define_connect_level_keyword] = ACTIONS(1777),
    [sym_define_latching_keyword] = ACTIONS(1777),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1777),
    [sym_define_toggling_keyword] = ACTIONS(1777),
    [sym_define_call_keyword] = ACTIONS(1777),
    [sym_define_function_keyword] = ACTIONS(1777),
    [sym_define_library_function_keyword] = ACTIONS(1777),
    [sym_define_start_keyword] = ACTIONS(1777),
    [sym_define_event_keyword] = ACTIONS(1777),
    [sym_define_program_keyword] = ACTIONS(1777),
    [sym_define_module_keyword] = ACTIONS(1777),
    [sym_char_keyword] = ACTIONS(1777),
    [sym_widechar_keyword] = ACTIONS(1777),
    [sym_integer_keyword] = ACTIONS(1777),
    [sym_sinteger_keyword] = ACTIONS(1777),
    [sym_long_keyword] = ACTIONS(1777),
    [sym_slong_keyword] = ACTIONS(1777),
    [sym_float_keyword] = ACTIONS(1777),
    [sym_double_keyword] = ACTIONS(1777),
    [sym_variant_keyword] = ACTIONS(1777),
    [sym_variantarray_keyword] = ACTIONS(1777),
    [sym_local_var_keyword] = ACTIONS(1777),
    [sym_stack_var_keyword] = ACTIONS(1777),
    [sym_constant_keyword] = ACTIONS(1777),
    [sym_volatile_keyword] = ACTIONS(1777),
    [sym_non_volatile_keyword] = ACTIONS(1777),
    [sym_persistent_keyword] = ACTIONS(1777),
    [sym_dev_keyword] = ACTIONS(1777),
    [sym_devlev_keyword] = ACTIONS(1777),
    [sym_devchan_keyword] = ACTIONS(1777),
    [sym_if_keyword] = ACTIONS(1777),
    [sym_else_keyword] = ACTIONS(1777),
    [sym_switch_keyword] = ACTIONS(1777),
    [sym_case_keyword] = ACTIONS(1777),
    [sym_default_keyword] = ACTIONS(1777),
    [sym_while_keyword] = ACTIONS(1777),
    [sym_for_keyword] = ACTIONS(1777),
    [sym_break_keyword] = ACTIONS(1777),
    [sym_continue_keyword] = ACTIONS(1777),
    [sym_return_keyword] = ACTIONS(1777),
    [sym_select_keyword] = ACTIONS(1777),
    [sym_active_keyword] = ACTIONS(1777),
    [sym_struct_keyword] = ACTIONS(1777),
    [sym_structure_keyword] = ACTIONS(1777),
    [sym_button_event_keyword] = ACTIONS(1777),
    [sym_channel_event_keyword] = ACTIONS(1777),
    [sym_level_event_keyword] = ACTIONS(1777),
    [sym_data_event_keyword] = ACTIONS(1777),
    [sym_timeline_event_keyword] = ACTIONS(1777),
    [sym_custom_event_keyword] = ACTIONS(1777),
    [sym_push_keyword] = ACTIONS(1777),
    [sym_release_keyword] = ACTIONS(1777),
    [aux_sym_on_keyword_token1] = ACTIONS(1777),
    [aux_sym_off_keyword_token1] = ACTIONS(1777),
    [sym_send_level_keyword] = ACTIONS(1777),
    [sym_send_string_keyword] = ACTIONS(1777),
    [sym_send_command_keyword] = ACTIONS(1777),
    [sym_clear_buffer_keyword] = ACTIONS(1777),
    [sym_create_buffer_keyword] = ACTIONS(1777),
    [sym_create_multi_buffer_keyword] = ACTIONS(1777),
    [sym_call_keyword] = ACTIONS(1777),
    [sym_system_call_keyword] = ACTIONS(1777),
    [sym_devchan_to_keyword] = ACTIONS(1777),
    [sym_devchan_min_to_keyword] = ACTIONS(1777),
    [sym_devchan_total_off_keyword] = ACTIONS(1777),
    [sym_devchan_pulse_keyword] = ACTIONS(1777),
    [sym_wait_keyword] = ACTIONS(1777),
    [sym_cancel_wait_keyword] = ACTIONS(1777),
    [sym_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1777),
    [sym_not] = ACTIONS(1777),
    [sym_bnot] = ACTIONS(1777),
    [sym___file__] = ACTIONS(1777),
    [sym___date__] = ACTIONS(1777),
    [sym___ldate__] = ACTIONS(1777),
    [sym___time__] = ACTIONS(1777),
    [sym___line__] = ACTIONS(1777),
    [sym___name__] = ACTIONS(1777),
    [sym___netlinx__] = ACTIONS(1777),
    [sym_netlinx_axi_version] = ACTIONS(1777),
    [sym_url_flg_tcp] = ACTIONS(1777),
    [sym_url_flg_accinfopresent] = ACTIONS(1777),
    [sym_url_flg_temp] = ACTIONS(1777),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1777),
    [sym_url_flg_stat_mask] = ACTIONS(1777),
    [sym_url_flg_stat_lookup] = ACTIONS(1777),
    [sym_url_flg_stat_connecting] = ACTIONS(1777),
    [sym_url_flg_stat_waiting] = ACTIONS(1777),
    [sym_url_flg_stat_connected] = ACTIONS(1777),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1777),
    [sym_first_virtual_device] = ACTIONS(1777),
    [sym_first_local_port] = ACTIONS(1777),
    [sym_dynamic_virtual_device] = ACTIONS(1777),
    [sym_timeline_once] = ACTIONS(1777),
    [sym_timeline_repeat] = ACTIONS(1777),
    [sym_timeline_absolute] = ACTIONS(1777),
    [sym_timeline_relative] = ACTIONS(1777),
    [sym_do_push_timed_infinite] = ACTIONS(1777),
    [sym_source_type_no_address] = ACTIONS(1777),
    [sym_source_type_neuron_id] = ACTIONS(1777),
    [sym_source_type_ip_address] = ACTIONS(1777),
    [sym_source_type_axlink] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1777),
    [sym_source_type_ip_socket_address] = ACTIONS(1777),
    [sym_source_type_rs232] = ACTIONS(1777),
    [sym_source_type_internal] = ACTIONS(1777),
    [sym_source_type_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1777),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1777),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1777),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1777),
    [sym_true] = ACTIONS(1777),
    [sym_false] = ACTIONS(1777),
    [sym_file_read_only] = ACTIONS(1777),
    [sym_file_rw_new] = ACTIONS(1777),
    [sym_file_rw_append] = ACTIONS(1777),
    [sym_ip_tcp] = ACTIONS(1777),
    [sym_ip_udp] = ACTIONS(1777),
    [sym_ip_udp_2way] = ACTIONS(1777),
    [sym_xml_encode_types] = ACTIONS(1777),
    [sym_xml_encode_char_as_list] = ACTIONS(1777),
    [sym_xml_encode_le] = ACTIONS(1777),
    [sym_xml_decode_types] = ACTIONS(1777),
    [sym_xml_decode_no_preserve] = ACTIONS(1777),
    [sym_duet_dev_type_display_device] = ACTIONS(1777),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_io_device] = ACTIONS(1777),
    [sym_duet_dev_type_relay_device] = ACTIONS(1777),
    [sym_duet_dev_type_ups] = ACTIONS(1777),
    [sym_duet_dev_type_amplifier] = ACTIONS(1777),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1777),
    [sym_duet_dev_type_camera] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1777),
    [sym_duet_dev_type_dss] = ACTIONS(1777),
    [sym_duet_dev_type_dvr] = ACTIONS(1777),
    [sym_duet_dev_type_disc_device] = ACTIONS(1777),
    [sym_duet_dev_type_document_camera] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1777),
    [sym_duet_dev_type_hvac] = ACTIONS(1777),
    [sym_duet_dev_type_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_light] = ACTIONS(1777),
    [sym_duet_dev_type_monitor] = ACTIONS(1777),
    [sym_duet_dev_type_motor] = ACTIONS(1777),
    [sym_duet_dev_type_multi_window] = ACTIONS(1777),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1777),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1777),
    [sym_duet_dev_type_receiver] = ACTIONS(1777),
    [sym_duet_dev_type_security_system] = ACTIONS(1777),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1777),
    [sym_duet_dev_type_settop_box] = ACTIONS(1777),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1777),
    [sym_duet_dev_type_switcher] = ACTIONS(1777),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_tv] = ACTIONS(1777),
    [sym_duet_dev_type_utility] = ACTIONS(1777),
    [sym_duet_dev_type_vcr] = ACTIONS(1777),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_video_processor] = ACTIONS(1777),
    [sym_duet_dev_type_video_projector] = ACTIONS(1777),
    [sym_duet_dev_type_video_wall] = ACTIONS(1777),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1777),
    [sym_duet_dev_type_weather] = ACTIONS(1777),
    [sym_duet_dev_not_polled] = ACTIONS(1777),
    [sym_duet_dev_polled] = ACTIONS(1777),
    [sym_clkmgr_mode_network] = ACTIONS(1777),
    [sym_clkmgr_mode_standalone] = ACTIONS(1777),
    [sym_normal_standby] = ACTIONS(1777),
    [sym_normal_wake] = ACTIONS(1777),
    [sym_ip_multicast_ttl_option] = ACTIONS(1777),
    [sym_ip_ttl_subnet] = ACTIONS(1777),
    [sym_ip_ttl_site] = ACTIONS(1777),
    [sym_ip_ttl_region] = ACTIONS(1777),
    [sym_ip_ttl_continent] = ACTIONS(1777),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1777),
    [sym_ip_nodelay_on] = ACTIONS(1777),
    [sym_ip_nodelay_off] = ACTIONS(1777),
    [sym_amx_error] = ACTIONS(1777),
    [sym_amx_warning] = ACTIONS(1777),
    [sym_amx_info] = ACTIONS(1777),
    [sym_amx_debug] = ACTIONS(1777),
    [sym_netlinx_login_success] = ACTIONS(1777),
    [sym_netlinx_login_fail] = ACTIONS(1777),
    [sym_netlinx_logout] = ACTIONS(1777),
    [sym_null_str] = ACTIONS(1777),
    [sym_smtp_address] = ACTIONS(1777),
    [sym_smtp_port_number] = ACTIONS(1777),
    [sym_smtp_username] = ACTIONS(1777),
    [sym_smtp_password] = ACTIONS(1777),
    [sym_smtp_require_tls] = ACTIONS(1777),
    [sym_smtp_from] = ACTIONS(1777),
    [sym_smtp_tls_true] = ACTIONS(1777),
    [sym_smtp_tls_false] = ACTIONS(1777),
    [sym_listview_on_row_select_event] = ACTIONS(1777),
    [sym_data_structure_unknown] = ACTIONS(1777),
    [sym_data_structure_datafeed] = ACTIONS(1777),
    [sym_data_structure_datarecordset] = ACTIONS(1777),
    [sym_data_structure_datarecord] = ACTIONS(1777),
    [sym_data_structure_datafield] = ACTIONS(1777),
    [sym_data_max_content_field_count] = ACTIONS(1777),
    [sym_data_max_metadata_field_count] = ACTIONS(1777),
    [sym_data_max_id_length] = ACTIONS(1777),
    [sym_data_max_name_length] = ACTIONS(1777),
    [sym_data_max_description_length] = ACTIONS(1777),
    [sym_data_max_type_length] = ACTIONS(1777),
    [sym_data_max_format_length] = ACTIONS(1777),
    [sym_data_max_source_length] = ACTIONS(1777),
    [sym_data_max_value_length] = ACTIONS(1777),
    [sym_data_max_label_length] = ACTIONS(1777),
    [sym_data_max_url_length] = ACTIONS(1777),
    [sym_max_last_login_info_length] = ACTIONS(1777),
    [sym_data_type_unknown] = ACTIONS(1777),
    [sym_data_type_string] = ACTIONS(1777),
    [sym_data_type_datetime] = ACTIONS(1777),
    [sym_data_type_date] = ACTIONS(1777),
    [sym_data_type_time] = ACTIONS(1777),
    [sym_data_type_image] = ACTIONS(1777),
    [sym_data_format_url] = ACTIONS(1777),
    [sym_data_format_phone] = ACTIONS(1777),
    [sym_data_format_email] = ACTIONS(1777),
    [sym_data_format_iso8601] = ACTIONS(1777),
    [sym_auditlog_priv] = ACTIONS(1777),
    [sym_configuration_priv] = ACTIONS(1777),
    [sym_deviceconfig_priv] = ACTIONS(1777),
    [sym_ftp_priv] = ACTIONS(1777),
    [sym_http_priv] = ACTIONS(1777),
    [sym_networkconfig_priv] = ACTIONS(1777),
    [sym_programport_priv] = ACTIONS(1777),
    [sym_remoteui_priv] = ACTIONS(1777),
    [sym_securitycontrol_priv] = ACTIONS(1777),
    [sym_softwaremanagement_priv] = ACTIONS(1777),
    [sym_terminal_priv] = ACTIONS(1777),
    [sym_tpadmin_priv] = ACTIONS(1777),
    [sym_user1_priv] = ACTIONS(1777),
    [sym_user2_priv] = ACTIONS(1777),
    [sym_user3_priv] = ACTIONS(1777),
    [sym_user4_priv] = ACTIONS(1777),
    [sym_usermanagement_priv] = ACTIONS(1777),
    [sym_valid_account] = ACTIONS(1777),
    [sym_err_name_invalid] = ACTIONS(1777),
    [sym_err_password_invalid] = ACTIONS(1777),
    [sym_err_invalid_account] = ACTIONS(1777),
    [sym_err_no_authorization] = ACTIONS(1777),
    [sym_err_invalid_parameter] = ACTIONS(1777),
    [sym_err_user_locked_out] = ACTIONS(1777),
    [sym_err_user_account_expired] = ACTIONS(1777),
    [sym_err_general_error] = ACTIONS(1777),
    [sym_err_authentication_error] = ACTIONS(1777),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1777),
    [sym_tls_validate_certificate] = ACTIONS(1777),
    [sym_tdata] = ACTIONS(1777),
    [sym_tchannel] = ACTIONS(1777),
    [sym_tlevel] = ACTIONS(1777),
    [sym_tbutton] = ACTIONS(1777),
    [sym_ttimeline] = ACTIONS(1777),
    [sym_tcustom] = ACTIONS(1777),
    [sym_url_struct] = ACTIONS(1777),
    [sym_dns_struct] = ACTIONS(1777),
    [sym_ip_address_struct] = ACTIONS(1777),
    [sym_dev_info_struct] = ACTIONS(1777),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1777),
    [sym_data_feed] = ACTIONS(1777),
    [sym_data_field] = ACTIONS(1777),
    [sym_data_record] = ACTIONS(1777),
    [sym_wc_data_feed] = ACTIONS(1777),
    [sym_wc_data_field] = ACTIONS(1777),
    [sym_wc_data_record] = ACTIONS(1777),
    [sym_last_login_info] = ACTIONS(1777),
    [sym____reserved___] = ACTIONS(1777),
    [sym_date] = ACTIONS(1777),
    [sym_day] = ACTIONS(1777),
    [sym_time] = ACTIONS(1777),
    [sym_ldate] = ACTIONS(1777),
    [sym_push_device] = ACTIONS(1777),
    [sym_push_channel] = ACTIONS(1777),
    [sym_push_devchan] = ACTIONS(1777),
    [sym_release_device] = ACTIONS(1777),
    [sym_release_channel] = ACTIONS(1777),
    [sym_release_devchan] = ACTIONS(1777),
    [sym_master_slot] = ACTIONS(1777),
    [sym_get_pulse_time] = ACTIONS(1777),
    [sym_get_timer] = ACTIONS(1777),
    [sym_master_sn] = ACTIONS(1777),
    [sym_system_number] = ACTIONS(1777),
    [sym_button] = ACTIONS(1777),
    [sym_dv_channel] = ACTIONS(1777),
    [sym_data] = ACTIONS(1777),
    [sym_level] = ACTIONS(1777),
    [sym_timeline] = ACTIONS(1777),
    [sym_channel] = ACTIONS(1777),
    [sym_custom] = ACTIONS(1777),
    [sym_atoi] = ACTIONS(1777),
    [sym_atol] = ACTIONS(1777),
    [sym_atof] = ACTIONS(1777),
    [sym_itoa] = ACTIONS(1777),
    [sym_itohex] = ACTIONS(1777),
    [sym_hextoi] = ACTIONS(1777),
    [sym_ftoa] = ACTIONS(1777),
    [sym_find_string] = ACTIONS(1777),
    [sym_left_string] = ACTIONS(1777),
    [sym_right_string] = ACTIONS(1777),
    [sym_mid_string] = ACTIONS(1777),
    [sym_remove_string] = ACTIONS(1777),
    [sym_get_buffer_string] = ACTIONS(1777),
    [sym_get_buffer_char] = ACTIONS(1777),
    [sym_get_multi_buffer_string] = ACTIONS(1777),
    [sym_length_string] = ACTIONS(1777),
    [sym_max_length_string] = ACTIONS(1777),
    [sym_set_length_string] = ACTIONS(1777),
    [sym_compare_string] = ACTIONS(1777),
    [sym_upper_string] = ACTIONS(1777),
    [sym_lower_string] = ACTIONS(1777),
    [sym_get_last] = ACTIONS(1777),
    [sym_date_to_day] = ACTIONS(1777),
    [sym_date_to_month] = ACTIONS(1777),
    [sym_date_to_year] = ACTIONS(1777),
    [sym_time_to_hour] = ACTIONS(1777),
    [sym_time_to_minute] = ACTIONS(1777),
    [sym_time_to_second] = ACTIONS(1777),
    [sym_day_of_week] = ACTIONS(1777),
    [sym_set_pulse_time] = ACTIONS(1777),
    [sym_set_timer] = ACTIONS(1777),
    [sym_file_open] = ACTIONS(1777),
    [sym_file_close] = ACTIONS(1777),
    [sym_file_read] = ACTIONS(1777),
    [sym_file_read_line] = ACTIONS(1777),
    [sym_file_write] = ACTIONS(1777),
    [sym_file_write_line] = ACTIONS(1777),
    [sym_file_copy] = ACTIONS(1777),
    [sym_file_delete] = ACTIONS(1777),
    [sym_file_rename] = ACTIONS(1777),
    [sym_file_dir] = ACTIONS(1777),
    [sym_file_getdir] = ACTIONS(1777),
    [sym_file_setdir] = ACTIONS(1777),
    [sym_file_create_dir] = ACTIONS(1777),
    [sym_file_remove_dir] = ACTIONS(1777),
    [sym_file_seek] = ACTIONS(1777),
    [sym_length_array] = ACTIONS(1777),
    [sym_max_length_array] = ACTIONS(1777),
    [sym_set_length_array] = ACTIONS(1777),
    [sym_ip_client_open] = ACTIONS(1777),
    [sym_ip_client_close] = ACTIONS(1777),
    [sym_ip_server_open] = ACTIONS(1777),
    [sym_ip_server_close] = ACTIONS(1777),
    [sym_ip_bound_client_open] = ACTIONS(1777),
    [sym_ip_mc_server_open] = ACTIONS(1777),
    [sym_ip_set_option] = ACTIONS(1777),
    [sym_ssh_client_open] = ACTIONS(1777),
    [sym_ssh_client_close] = ACTIONS(1777),
    [sym_tls_client_open] = ACTIONS(1777),
    [sym_tls_client_close] = ACTIONS(1777),
    [sym_do_push] = ACTIONS(1777),
    [sym_do_push_timed] = ACTIONS(1777),
    [sym_do_release] = ACTIONS(1777),
    [sym_device_id] = ACTIONS(1777),
    [sym_device_info] = ACTIONS(1777),
    [sym_device_id_string] = ACTIONS(1777),
    [sym_device_standby] = ACTIONS(1777),
    [sym_device_wake] = ACTIONS(1777),
    [sym_timeline_create] = ACTIONS(1777),
    [sym_timeline_kill] = ACTIONS(1777),
    [sym_timeline_reload] = ACTIONS(1777),
    [sym_timeline_pause] = ACTIONS(1777),
    [sym_timeline_restart] = ACTIONS(1777),
    [sym_timeline_set] = ACTIONS(1777),
    [sym_timeline_get] = ACTIONS(1777),
    [sym_timeline_active] = ACTIONS(1777),
    [sym_timeline_dynamic_id] = ACTIONS(1777),
    [sym_variable_to_string] = ACTIONS(1777),
    [sym_string_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_string] = ACTIONS(1777),
    [sym_type_cast] = ACTIONS(1777),
    [sym_variable_to_xml] = ACTIONS(1777),
    [sym_xml_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_xml] = ACTIONS(1777),
    [sym_redirect_string] = ACTIONS(1777),
    [sym_random_number] = ACTIONS(1777),
    [sym_get_unique_id] = ACTIONS(1777),
    [sym_get_system_number] = ACTIONS(1777),
    [sym_set_system_number] = ACTIONS(1777),
    [sym_get_serial_number] = ACTIONS(1777),
    [sym_get_master_build] = ACTIONS(1777),
    [sym_set_validation_code] = ACTIONS(1777),
    [sym_reboot] = ACTIONS(1777),
    [sym_load_duet_module] = ACTIONS(1777),
    [sym_unload_duet_module] = ACTIONS(1777),
    [sym_duet_mem_size_set] = ACTIONS(1777),
    [sym_duet_mem_size_get] = ACTIONS(1777),
    [sym_dynamic_application_device] = ACTIONS(1777),
    [sym_dynamic_polled_port] = ACTIONS(1777),
    [sym_static_port_binding] = ACTIONS(1777),
    [sym_static_ip_binding] = ACTIONS(1777),
    [sym_get_dns_list] = ACTIONS(1777),
    [sym_set_dns_list] = ACTIONS(1777),
    [sym_get_ip_address] = ACTIONS(1777),
    [sym_set_ip_address] = ACTIONS(1777),
    [sym_get_url_list] = ACTIONS(1777),
    [sym_add_url_entry] = ACTIONS(1777),
    [sym_delete_url_entry] = ACTIONS(1777),
    [sym_abs_value] = ACTIONS(1777),
    [sym_max_value] = ACTIONS(1777),
    [sym_min_value] = ACTIONS(1777),
    [sym_exp_value] = ACTIONS(1777),
    [sym_log_value] = ACTIONS(1777),
    [sym_log10_value] = ACTIONS(1777),
    [sym_power_value] = ACTIONS(1777),
    [sym_sqrt_value] = ACTIONS(1777),
    [sym_raw_be] = ACTIONS(1777),
    [sym_raw_le] = ACTIONS(1777),
    [sym_internal_queue_size_set] = ACTIONS(1777),
    [sym_internal_queue_size_get] = ACTIONS(1777),
    [sym_internal_threshold_set] = ACTIONS(1777),
    [sym_internal_threshold_get] = ACTIONS(1777),
    [sym_rebuilt_event] = ACTIONS(1777),
    [sym_do_custom_event] = ACTIONS(1777),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1777),
    [sym_clkmgr_set_clk_source] = ACTIONS(1777),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1777),
    [sym_clkmgr_get_timezone] = ACTIONS(1777),
    [sym_clkmgr_set_timezone] = ACTIONS(1777),
    [sym_clkmgr_get_resync_period] = ACTIONS(1777),
    [sym_clkmgr_set_resync_period] = ACTIONS(1777),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_timeservers] = ACTIONS(1777),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_set_log_level] = ACTIONS(1777),
    [sym_get_log_level] = ACTIONS(1777),
    [sym_amx_log] = ACTIONS(1777),
    [sym_smtp_server_config_set] = ACTIONS(1777),
    [sym_smtp_server_config_get] = ACTIONS(1777),
    [sym_smtp_send] = ACTIONS(1777),
    [sym_validate_netlinx_account] = ACTIONS(1777),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1777),
    [sym_authenticate_certificate] = ACTIONS(1777),
    [sym_audit_netlinx_session_event] = ACTIONS(1777),
    [sym_audit_netlinx_generic_event] = ACTIONS(1777),
    [sym_get_available_flash_disk_space] = ACTIONS(1777),
    [sym_get_max_flash_disk_space] = ACTIONS(1777),
    [sym_data_create_feed] = ACTIONS(1777),
    [sym_data_delete_feed] = ACTIONS(1777),
    [sym_data_publish_feed] = ACTIONS(1777),
    [sym_data_get_published_feed] = ACTIONS(1777),
    [sym_data_add_record] = ACTIONS(1777),
    [sym_data_get_event_record] = ACTIONS(1777),
    [sym__wc_data_create_feed] = ACTIONS(1777),
    [sym__wc_data_add_record] = ACTIONS(1777),
    [sym__wc_data_get_event_record] = ACTIONS(1777),
    [sym_format] = ACTIONS(1777),
    [sym_astro_clock] = ACTIONS(1777),
    [sym_set_outdoor_temperature] = ACTIONS(1777),
    [sym_set_virtual_level_count] = ACTIONS(1777),
    [sym_set_virtual_channel_count] = ACTIONS(1777),
    [sym_set_virtual_port_count] = ACTIONS(1777),
    [sym__wc_encode_int] = ACTIONS(1777),
    [sym__wc_decode_int] = ACTIONS(1777),
    [sym__wc_int] = ACTIONS(1777),
    [sym__wc_to_ch_int] = ACTIONS(1777),
    [sym__ch_to_wc_int] = ACTIONS(1777),
    [sym__wc_find_string_int] = ACTIONS(1777),
    [sym__wc_left_string_int] = ACTIONS(1777),
    [sym__wc_lower_string_int] = ACTIONS(1777),
    [sym__wc_mid_string_int] = ACTIONS(1777),
    [sym__wc_remove_string_int] = ACTIONS(1777),
    [sym__wc_right_string_int] = ACTIONS(1777),
    [sym__wc_upper_string_int] = ACTIONS(1777),
    [sym__wc_compare_string_int] = ACTIONS(1777),
    [sym__wc_get_buffer_char_int] = ACTIONS(1777),
    [sym__wc_get_buffer_string_int] = ACTIONS(1777),
    [sym__wc_concat_string_int] = ACTIONS(1777),
    [sym__wc_file_open_int] = ACTIONS(1777),
    [sym__wc_file_close_int] = ACTIONS(1777),
    [sym__wc_file_read_int] = ACTIONS(1777),
    [sym__wc_file_read_line_int] = ACTIONS(1777),
    [sym__wc_file_write_int] = ACTIONS(1777),
    [sym__wc_file_write_line_int] = ACTIONS(1777),
    [aux_sym_preproc_include_token1] = ACTIONS(1777),
    [aux_sym_preproc_define_token1] = ACTIONS(1779),
    [aux_sym_preproc_warn_token1] = ACTIONS(1779),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1779),
    [anon_sym_LBRACK] = ACTIONS(1779),
    [anon_sym_LBRACE] = ACTIONS(1779),
    [anon_sym_LPAREN] = ACTIONS(1777),
    [anon_sym_BANG] = ACTIONS(1779),
    [anon_sym_TILDE] = ACTIONS(1779),
    [anon_sym_DASH] = ACTIONS(1777),
    [anon_sym_PLUS] = ACTIONS(1777),
    [anon_sym_DASH_DASH] = ACTIONS(1779),
    [anon_sym_PLUS_PLUS] = ACTIONS(1779),
    [anon_sym_DQUOTE] = ACTIONS(1779),
    [anon_sym_SQUOTE] = ACTIONS(1779),
    [sym_number_literal] = ACTIONS(1779),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(325)] = {
    [ts_builtin_sym_end] = ACTIONS(1783),
    [sym_identifier] = ACTIONS(1781),
    [sym_program_name_keyword] = ACTIONS(1781),
    [sym_module_name_keyword] = ACTIONS(1781),
    [sym_define_device_keyword] = ACTIONS(1781),
    [sym_define_combine_keyword] = ACTIONS(1781),
    [sym_define_constant_keyword] = ACTIONS(1781),
    [sym_define_type_keyword] = ACTIONS(1781),
    [sym_define_variable_keyword] = ACTIONS(1781),
    [sym_define_system_variable_keyword] = ACTIONS(1781),
    [sym_define_connect_level_keyword] = ACTIONS(1781),
    [sym_define_latching_keyword] = ACTIONS(1781),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1781),
    [sym_define_toggling_keyword] = ACTIONS(1781),
    [sym_define_call_keyword] = ACTIONS(1781),
    [sym_define_function_keyword] = ACTIONS(1781),
    [sym_define_library_function_keyword] = ACTIONS(1781),
    [sym_define_start_keyword] = ACTIONS(1781),
    [sym_define_event_keyword] = ACTIONS(1781),
    [sym_define_program_keyword] = ACTIONS(1781),
    [sym_define_module_keyword] = ACTIONS(1781),
    [sym_char_keyword] = ACTIONS(1781),
    [sym_widechar_keyword] = ACTIONS(1781),
    [sym_integer_keyword] = ACTIONS(1781),
    [sym_sinteger_keyword] = ACTIONS(1781),
    [sym_long_keyword] = ACTIONS(1781),
    [sym_slong_keyword] = ACTIONS(1781),
    [sym_float_keyword] = ACTIONS(1781),
    [sym_double_keyword] = ACTIONS(1781),
    [sym_variant_keyword] = ACTIONS(1781),
    [sym_variantarray_keyword] = ACTIONS(1781),
    [sym_local_var_keyword] = ACTIONS(1781),
    [sym_stack_var_keyword] = ACTIONS(1781),
    [sym_constant_keyword] = ACTIONS(1781),
    [sym_volatile_keyword] = ACTIONS(1781),
    [sym_non_volatile_keyword] = ACTIONS(1781),
    [sym_persistent_keyword] = ACTIONS(1781),
    [sym_dev_keyword] = ACTIONS(1781),
    [sym_devlev_keyword] = ACTIONS(1781),
    [sym_devchan_keyword] = ACTIONS(1781),
    [sym_if_keyword] = ACTIONS(1781),
    [sym_else_keyword] = ACTIONS(1781),
    [sym_switch_keyword] = ACTIONS(1781),
    [sym_case_keyword] = ACTIONS(1781),
    [sym_default_keyword] = ACTIONS(1781),
    [sym_while_keyword] = ACTIONS(1781),
    [sym_for_keyword] = ACTIONS(1781),
    [sym_break_keyword] = ACTIONS(1781),
    [sym_continue_keyword] = ACTIONS(1781),
    [sym_return_keyword] = ACTIONS(1781),
    [sym_select_keyword] = ACTIONS(1781),
    [sym_active_keyword] = ACTIONS(1781),
    [sym_struct_keyword] = ACTIONS(1781),
    [sym_structure_keyword] = ACTIONS(1781),
    [sym_button_event_keyword] = ACTIONS(1781),
    [sym_channel_event_keyword] = ACTIONS(1781),
    [sym_level_event_keyword] = ACTIONS(1781),
    [sym_data_event_keyword] = ACTIONS(1781),
    [sym_timeline_event_keyword] = ACTIONS(1781),
    [sym_custom_event_keyword] = ACTIONS(1781),
    [sym_push_keyword] = ACTIONS(1781),
    [sym_release_keyword] = ACTIONS(1781),
    [aux_sym_on_keyword_token1] = ACTIONS(1781),
    [aux_sym_off_keyword_token1] = ACTIONS(1781),
    [sym_send_level_keyword] = ACTIONS(1781),
    [sym_send_string_keyword] = ACTIONS(1781),
    [sym_send_command_keyword] = ACTIONS(1781),
    [sym_clear_buffer_keyword] = ACTIONS(1781),
    [sym_create_buffer_keyword] = ACTIONS(1781),
    [sym_create_multi_buffer_keyword] = ACTIONS(1781),
    [sym_call_keyword] = ACTIONS(1781),
    [sym_system_call_keyword] = ACTIONS(1781),
    [sym_devchan_to_keyword] = ACTIONS(1781),
    [sym_devchan_min_to_keyword] = ACTIONS(1781),
    [sym_devchan_total_off_keyword] = ACTIONS(1781),
    [sym_devchan_pulse_keyword] = ACTIONS(1781),
    [sym_wait_keyword] = ACTIONS(1781),
    [sym_cancel_wait_keyword] = ACTIONS(1781),
    [sym_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1781),
    [sym_not] = ACTIONS(1781),
    [sym_bnot] = ACTIONS(1781),
    [sym___file__] = ACTIONS(1781),
    [sym___date__] = ACTIONS(1781),
    [sym___ldate__] = ACTIONS(1781),
    [sym___time__] = ACTIONS(1781),
    [sym___line__] = ACTIONS(1781),
    [sym___name__] = ACTIONS(1781),
    [sym___netlinx__] = ACTIONS(1781),
    [sym_netlinx_axi_version] = ACTIONS(1781),
    [sym_url_flg_tcp] = ACTIONS(1781),
    [sym_url_flg_accinfopresent] = ACTIONS(1781),
    [sym_url_flg_temp] = ACTIONS(1781),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1781),
    [sym_url_flg_stat_mask] = ACTIONS(1781),
    [sym_url_flg_stat_lookup] = ACTIONS(1781),
    [sym_url_flg_stat_connecting] = ACTIONS(1781),
    [sym_url_flg_stat_waiting] = ACTIONS(1781),
    [sym_url_flg_stat_connected] = ACTIONS(1781),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1781),
    [sym_first_virtual_device] = ACTIONS(1781),
    [sym_first_local_port] = ACTIONS(1781),
    [sym_dynamic_virtual_device] = ACTIONS(1781),
    [sym_timeline_once] = ACTIONS(1781),
    [sym_timeline_repeat] = ACTIONS(1781),
    [sym_timeline_absolute] = ACTIONS(1781),
    [sym_timeline_relative] = ACTIONS(1781),
    [sym_do_push_timed_infinite] = ACTIONS(1781),
    [sym_source_type_no_address] = ACTIONS(1781),
    [sym_source_type_neuron_id] = ACTIONS(1781),
    [sym_source_type_ip_address] = ACTIONS(1781),
    [sym_source_type_axlink] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1781),
    [sym_source_type_ip_socket_address] = ACTIONS(1781),
    [sym_source_type_rs232] = ACTIONS(1781),
    [sym_source_type_internal] = ACTIONS(1781),
    [sym_source_type_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1781),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1781),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1781),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1781),
    [sym_true] = ACTIONS(1781),
    [sym_false] = ACTIONS(1781),
    [sym_file_read_only] = ACTIONS(1781),
    [sym_file_rw_new] = ACTIONS(1781),
    [sym_file_rw_append] = ACTIONS(1781),
    [sym_ip_tcp] = ACTIONS(1781),
    [sym_ip_udp] = ACTIONS(1781),
    [sym_ip_udp_2way] = ACTIONS(1781),
    [sym_xml_encode_types] = ACTIONS(1781),
    [sym_xml_encode_char_as_list] = ACTIONS(1781),
    [sym_xml_encode_le] = ACTIONS(1781),
    [sym_xml_decode_types] = ACTIONS(1781),
    [sym_xml_decode_no_preserve] = ACTIONS(1781),
    [sym_duet_dev_type_display_device] = ACTIONS(1781),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_io_device] = ACTIONS(1781),
    [sym_duet_dev_type_relay_device] = ACTIONS(1781),
    [sym_duet_dev_type_ups] = ACTIONS(1781),
    [sym_duet_dev_type_amplifier] = ACTIONS(1781),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1781),
    [sym_duet_dev_type_camera] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1781),
    [sym_duet_dev_type_dss] = ACTIONS(1781),
    [sym_duet_dev_type_dvr] = ACTIONS(1781),
    [sym_duet_dev_type_disc_device] = ACTIONS(1781),
    [sym_duet_dev_type_document_camera] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1781),
    [sym_duet_dev_type_hvac] = ACTIONS(1781),
    [sym_duet_dev_type_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_light] = ACTIONS(1781),
    [sym_duet_dev_type_monitor] = ACTIONS(1781),
    [sym_duet_dev_type_motor] = ACTIONS(1781),
    [sym_duet_dev_type_multi_window] = ACTIONS(1781),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1781),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1781),
    [sym_duet_dev_type_receiver] = ACTIONS(1781),
    [sym_duet_dev_type_security_system] = ACTIONS(1781),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1781),
    [sym_duet_dev_type_settop_box] = ACTIONS(1781),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1781),
    [sym_duet_dev_type_switcher] = ACTIONS(1781),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_tv] = ACTIONS(1781),
    [sym_duet_dev_type_utility] = ACTIONS(1781),
    [sym_duet_dev_type_vcr] = ACTIONS(1781),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_video_processor] = ACTIONS(1781),
    [sym_duet_dev_type_video_projector] = ACTIONS(1781),
    [sym_duet_dev_type_video_wall] = ACTIONS(1781),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1781),
    [sym_duet_dev_type_weather] = ACTIONS(1781),
    [sym_duet_dev_not_polled] = ACTIONS(1781),
    [sym_duet_dev_polled] = ACTIONS(1781),
    [sym_clkmgr_mode_network] = ACTIONS(1781),
    [sym_clkmgr_mode_standalone] = ACTIONS(1781),
    [sym_normal_standby] = ACTIONS(1781),
    [sym_normal_wake] = ACTIONS(1781),
    [sym_ip_multicast_ttl_option] = ACTIONS(1781),
    [sym_ip_ttl_subnet] = ACTIONS(1781),
    [sym_ip_ttl_site] = ACTIONS(1781),
    [sym_ip_ttl_region] = ACTIONS(1781),
    [sym_ip_ttl_continent] = ACTIONS(1781),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1781),
    [sym_ip_nodelay_on] = ACTIONS(1781),
    [sym_ip_nodelay_off] = ACTIONS(1781),
    [sym_amx_error] = ACTIONS(1781),
    [sym_amx_warning] = ACTIONS(1781),
    [sym_amx_info] = ACTIONS(1781),
    [sym_amx_debug] = ACTIONS(1781),
    [sym_netlinx_login_success] = ACTIONS(1781),
    [sym_netlinx_login_fail] = ACTIONS(1781),
    [sym_netlinx_logout] = ACTIONS(1781),
    [sym_null_str] = ACTIONS(1781),
    [sym_smtp_address] = ACTIONS(1781),
    [sym_smtp_port_number] = ACTIONS(1781),
    [sym_smtp_username] = ACTIONS(1781),
    [sym_smtp_password] = ACTIONS(1781),
    [sym_smtp_require_tls] = ACTIONS(1781),
    [sym_smtp_from] = ACTIONS(1781),
    [sym_smtp_tls_true] = ACTIONS(1781),
    [sym_smtp_tls_false] = ACTIONS(1781),
    [sym_listview_on_row_select_event] = ACTIONS(1781),
    [sym_data_structure_unknown] = ACTIONS(1781),
    [sym_data_structure_datafeed] = ACTIONS(1781),
    [sym_data_structure_datarecordset] = ACTIONS(1781),
    [sym_data_structure_datarecord] = ACTIONS(1781),
    [sym_data_structure_datafield] = ACTIONS(1781),
    [sym_data_max_content_field_count] = ACTIONS(1781),
    [sym_data_max_metadata_field_count] = ACTIONS(1781),
    [sym_data_max_id_length] = ACTIONS(1781),
    [sym_data_max_name_length] = ACTIONS(1781),
    [sym_data_max_description_length] = ACTIONS(1781),
    [sym_data_max_type_length] = ACTIONS(1781),
    [sym_data_max_format_length] = ACTIONS(1781),
    [sym_data_max_source_length] = ACTIONS(1781),
    [sym_data_max_value_length] = ACTIONS(1781),
    [sym_data_max_label_length] = ACTIONS(1781),
    [sym_data_max_url_length] = ACTIONS(1781),
    [sym_max_last_login_info_length] = ACTIONS(1781),
    [sym_data_type_unknown] = ACTIONS(1781),
    [sym_data_type_string] = ACTIONS(1781),
    [sym_data_type_datetime] = ACTIONS(1781),
    [sym_data_type_date] = ACTIONS(1781),
    [sym_data_type_time] = ACTIONS(1781),
    [sym_data_type_image] = ACTIONS(1781),
    [sym_data_format_url] = ACTIONS(1781),
    [sym_data_format_phone] = ACTIONS(1781),
    [sym_data_format_email] = ACTIONS(1781),
    [sym_data_format_iso8601] = ACTIONS(1781),
    [sym_auditlog_priv] = ACTIONS(1781),
    [sym_configuration_priv] = ACTIONS(1781),
    [sym_deviceconfig_priv] = ACTIONS(1781),
    [sym_ftp_priv] = ACTIONS(1781),
    [sym_http_priv] = ACTIONS(1781),
    [sym_networkconfig_priv] = ACTIONS(1781),
    [sym_programport_priv] = ACTIONS(1781),
    [sym_remoteui_priv] = ACTIONS(1781),
    [sym_securitycontrol_priv] = ACTIONS(1781),
    [sym_softwaremanagement_priv] = ACTIONS(1781),
    [sym_terminal_priv] = ACTIONS(1781),
    [sym_tpadmin_priv] = ACTIONS(1781),
    [sym_user1_priv] = ACTIONS(1781),
    [sym_user2_priv] = ACTIONS(1781),
    [sym_user3_priv] = ACTIONS(1781),
    [sym_user4_priv] = ACTIONS(1781),
    [sym_usermanagement_priv] = ACTIONS(1781),
    [sym_valid_account] = ACTIONS(1781),
    [sym_err_name_invalid] = ACTIONS(1781),
    [sym_err_password_invalid] = ACTIONS(1781),
    [sym_err_invalid_account] = ACTIONS(1781),
    [sym_err_no_authorization] = ACTIONS(1781),
    [sym_err_invalid_parameter] = ACTIONS(1781),
    [sym_err_user_locked_out] = ACTIONS(1781),
    [sym_err_user_account_expired] = ACTIONS(1781),
    [sym_err_general_error] = ACTIONS(1781),
    [sym_err_authentication_error] = ACTIONS(1781),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1781),
    [sym_tls_validate_certificate] = ACTIONS(1781),
    [sym_tdata] = ACTIONS(1781),
    [sym_tchannel] = ACTIONS(1781),
    [sym_tlevel] = ACTIONS(1781),
    [sym_tbutton] = ACTIONS(1781),
    [sym_ttimeline] = ACTIONS(1781),
    [sym_tcustom] = ACTIONS(1781),
    [sym_url_struct] = ACTIONS(1781),
    [sym_dns_struct] = ACTIONS(1781),
    [sym_ip_address_struct] = ACTIONS(1781),
    [sym_dev_info_struct] = ACTIONS(1781),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1781),
    [sym_data_feed] = ACTIONS(1781),
    [sym_data_field] = ACTIONS(1781),
    [sym_data_record] = ACTIONS(1781),
    [sym_wc_data_feed] = ACTIONS(1781),
    [sym_wc_data_field] = ACTIONS(1781),
    [sym_wc_data_record] = ACTIONS(1781),
    [sym_last_login_info] = ACTIONS(1781),
    [sym____reserved___] = ACTIONS(1781),
    [sym_date] = ACTIONS(1781),
    [sym_day] = ACTIONS(1781),
    [sym_time] = ACTIONS(1781),
    [sym_ldate] = ACTIONS(1781),
    [sym_push_device] = ACTIONS(1781),
    [sym_push_channel] = ACTIONS(1781),
    [sym_push_devchan] = ACTIONS(1781),
    [sym_release_device] = ACTIONS(1781),
    [sym_release_channel] = ACTIONS(1781),
    [sym_release_devchan] = ACTIONS(1781),
    [sym_master_slot] = ACTIONS(1781),
    [sym_get_pulse_time] = ACTIONS(1781),
    [sym_get_timer] = ACTIONS(1781),
    [sym_master_sn] = ACTIONS(1781),
    [sym_system_number] = ACTIONS(1781),
    [sym_button] = ACTIONS(1781),
    [sym_dv_channel] = ACTIONS(1781),
    [sym_data] = ACTIONS(1781),
    [sym_level] = ACTIONS(1781),
    [sym_timeline] = ACTIONS(1781),
    [sym_channel] = ACTIONS(1781),
    [sym_custom] = ACTIONS(1781),
    [sym_atoi] = ACTIONS(1781),
    [sym_atol] = ACTIONS(1781),
    [sym_atof] = ACTIONS(1781),
    [sym_itoa] = ACTIONS(1781),
    [sym_itohex] = ACTIONS(1781),
    [sym_hextoi] = ACTIONS(1781),
    [sym_ftoa] = ACTIONS(1781),
    [sym_find_string] = ACTIONS(1781),
    [sym_left_string] = ACTIONS(1781),
    [sym_right_string] = ACTIONS(1781),
    [sym_mid_string] = ACTIONS(1781),
    [sym_remove_string] = ACTIONS(1781),
    [sym_get_buffer_string] = ACTIONS(1781),
    [sym_get_buffer_char] = ACTIONS(1781),
    [sym_get_multi_buffer_string] = ACTIONS(1781),
    [sym_length_string] = ACTIONS(1781),
    [sym_max_length_string] = ACTIONS(1781),
    [sym_set_length_string] = ACTIONS(1781),
    [sym_compare_string] = ACTIONS(1781),
    [sym_upper_string] = ACTIONS(1781),
    [sym_lower_string] = ACTIONS(1781),
    [sym_get_last] = ACTIONS(1781),
    [sym_date_to_day] = ACTIONS(1781),
    [sym_date_to_month] = ACTIONS(1781),
    [sym_date_to_year] = ACTIONS(1781),
    [sym_time_to_hour] = ACTIONS(1781),
    [sym_time_to_minute] = ACTIONS(1781),
    [sym_time_to_second] = ACTIONS(1781),
    [sym_day_of_week] = ACTIONS(1781),
    [sym_set_pulse_time] = ACTIONS(1781),
    [sym_set_timer] = ACTIONS(1781),
    [sym_file_open] = ACTIONS(1781),
    [sym_file_close] = ACTIONS(1781),
    [sym_file_read] = ACTIONS(1781),
    [sym_file_read_line] = ACTIONS(1781),
    [sym_file_write] = ACTIONS(1781),
    [sym_file_write_line] = ACTIONS(1781),
    [sym_file_copy] = ACTIONS(1781),
    [sym_file_delete] = ACTIONS(1781),
    [sym_file_rename] = ACTIONS(1781),
    [sym_file_dir] = ACTIONS(1781),
    [sym_file_getdir] = ACTIONS(1781),
    [sym_file_setdir] = ACTIONS(1781),
    [sym_file_create_dir] = ACTIONS(1781),
    [sym_file_remove_dir] = ACTIONS(1781),
    [sym_file_seek] = ACTIONS(1781),
    [sym_length_array] = ACTIONS(1781),
    [sym_max_length_array] = ACTIONS(1781),
    [sym_set_length_array] = ACTIONS(1781),
    [sym_ip_client_open] = ACTIONS(1781),
    [sym_ip_client_close] = ACTIONS(1781),
    [sym_ip_server_open] = ACTIONS(1781),
    [sym_ip_server_close] = ACTIONS(1781),
    [sym_ip_bound_client_open] = ACTIONS(1781),
    [sym_ip_mc_server_open] = ACTIONS(1781),
    [sym_ip_set_option] = ACTIONS(1781),
    [sym_ssh_client_open] = ACTIONS(1781),
    [sym_ssh_client_close] = ACTIONS(1781),
    [sym_tls_client_open] = ACTIONS(1781),
    [sym_tls_client_close] = ACTIONS(1781),
    [sym_do_push] = ACTIONS(1781),
    [sym_do_push_timed] = ACTIONS(1781),
    [sym_do_release] = ACTIONS(1781),
    [sym_device_id] = ACTIONS(1781),
    [sym_device_info] = ACTIONS(1781),
    [sym_device_id_string] = ACTIONS(1781),
    [sym_device_standby] = ACTIONS(1781),
    [sym_device_wake] = ACTIONS(1781),
    [sym_timeline_create] = ACTIONS(1781),
    [sym_timeline_kill] = ACTIONS(1781),
    [sym_timeline_reload] = ACTIONS(1781),
    [sym_timeline_pause] = ACTIONS(1781),
    [sym_timeline_restart] = ACTIONS(1781),
    [sym_timeline_set] = ACTIONS(1781),
    [sym_timeline_get] = ACTIONS(1781),
    [sym_timeline_active] = ACTIONS(1781),
    [sym_timeline_dynamic_id] = ACTIONS(1781),
    [sym_variable_to_string] = ACTIONS(1781),
    [sym_string_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_string] = ACTIONS(1781),
    [sym_type_cast] = ACTIONS(1781),
    [sym_variable_to_xml] = ACTIONS(1781),
    [sym_xml_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_xml] = ACTIONS(1781),
    [sym_redirect_string] = ACTIONS(1781),
    [sym_random_number] = ACTIONS(1781),
    [sym_get_unique_id] = ACTIONS(1781),
    [sym_get_system_number] = ACTIONS(1781),
    [sym_set_system_number] = ACTIONS(1781),
    [sym_get_serial_number] = ACTIONS(1781),
    [sym_get_master_build] = ACTIONS(1781),
    [sym_set_validation_code] = ACTIONS(1781),
    [sym_reboot] = ACTIONS(1781),
    [sym_load_duet_module] = ACTIONS(1781),
    [sym_unload_duet_module] = ACTIONS(1781),
    [sym_duet_mem_size_set] = ACTIONS(1781),
    [sym_duet_mem_size_get] = ACTIONS(1781),
    [sym_dynamic_application_device] = ACTIONS(1781),
    [sym_dynamic_polled_port] = ACTIONS(1781),
    [sym_static_port_binding] = ACTIONS(1781),
    [sym_static_ip_binding] = ACTIONS(1781),
    [sym_get_dns_list] = ACTIONS(1781),
    [sym_set_dns_list] = ACTIONS(1781),
    [sym_get_ip_address] = ACTIONS(1781),
    [sym_set_ip_address] = ACTIONS(1781),
    [sym_get_url_list] = ACTIONS(1781),
    [sym_add_url_entry] = ACTIONS(1781),
    [sym_delete_url_entry] = ACTIONS(1781),
    [sym_abs_value] = ACTIONS(1781),
    [sym_max_value] = ACTIONS(1781),
    [sym_min_value] = ACTIONS(1781),
    [sym_exp_value] = ACTIONS(1781),
    [sym_log_value] = ACTIONS(1781),
    [sym_log10_value] = ACTIONS(1781),
    [sym_power_value] = ACTIONS(1781),
    [sym_sqrt_value] = ACTIONS(1781),
    [sym_raw_be] = ACTIONS(1781),
    [sym_raw_le] = ACTIONS(1781),
    [sym_internal_queue_size_set] = ACTIONS(1781),
    [sym_internal_queue_size_get] = ACTIONS(1781),
    [sym_internal_threshold_set] = ACTIONS(1781),
    [sym_internal_threshold_get] = ACTIONS(1781),
    [sym_rebuilt_event] = ACTIONS(1781),
    [sym_do_custom_event] = ACTIONS(1781),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1781),
    [sym_clkmgr_set_clk_source] = ACTIONS(1781),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1781),
    [sym_clkmgr_get_timezone] = ACTIONS(1781),
    [sym_clkmgr_set_timezone] = ACTIONS(1781),
    [sym_clkmgr_get_resync_period] = ACTIONS(1781),
    [sym_clkmgr_set_resync_period] = ACTIONS(1781),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_timeservers] = ACTIONS(1781),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_set_log_level] = ACTIONS(1781),
    [sym_get_log_level] = ACTIONS(1781),
    [sym_amx_log] = ACTIONS(1781),
    [sym_smtp_server_config_set] = ACTIONS(1781),
    [sym_smtp_server_config_get] = ACTIONS(1781),
    [sym_smtp_send] = ACTIONS(1781),
    [sym_validate_netlinx_account] = ACTIONS(1781),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1781),
    [sym_authenticate_certificate] = ACTIONS(1781),
    [sym_audit_netlinx_session_event] = ACTIONS(1781),
    [sym_audit_netlinx_generic_event] = ACTIONS(1781),
    [sym_get_available_flash_disk_space] = ACTIONS(1781),
    [sym_get_max_flash_disk_space] = ACTIONS(1781),
    [sym_data_create_feed] = ACTIONS(1781),
    [sym_data_delete_feed] = ACTIONS(1781),
    [sym_data_publish_feed] = ACTIONS(1781),
    [sym_data_get_published_feed] = ACTIONS(1781),
    [sym_data_add_record] = ACTIONS(1781),
    [sym_data_get_event_record] = ACTIONS(1781),
    [sym__wc_data_create_feed] = ACTIONS(1781),
    [sym__wc_data_add_record] = ACTIONS(1781),
    [sym__wc_data_get_event_record] = ACTIONS(1781),
    [sym_format] = ACTIONS(1781),
    [sym_astro_clock] = ACTIONS(1781),
    [sym_set_outdoor_temperature] = ACTIONS(1781),
    [sym_set_virtual_level_count] = ACTIONS(1781),
    [sym_set_virtual_channel_count] = ACTIONS(1781),
    [sym_set_virtual_port_count] = ACTIONS(1781),
    [sym__wc_encode_int] = ACTIONS(1781),
    [sym__wc_decode_int] = ACTIONS(1781),
    [sym__wc_int] = ACTIONS(1781),
    [sym__wc_to_ch_int] = ACTIONS(1781),
    [sym__ch_to_wc_int] = ACTIONS(1781),
    [sym__wc_find_string_int] = ACTIONS(1781),
    [sym__wc_left_string_int] = ACTIONS(1781),
    [sym__wc_lower_string_int] = ACTIONS(1781),
    [sym__wc_mid_string_int] = ACTIONS(1781),
    [sym__wc_remove_string_int] = ACTIONS(1781),
    [sym__wc_right_string_int] = ACTIONS(1781),
    [sym__wc_upper_string_int] = ACTIONS(1781),
    [sym__wc_compare_string_int] = ACTIONS(1781),
    [sym__wc_get_buffer_char_int] = ACTIONS(1781),
    [sym__wc_get_buffer_string_int] = ACTIONS(1781),
    [sym__wc_concat_string_int] = ACTIONS(1781),
    [sym__wc_file_open_int] = ACTIONS(1781),
    [sym__wc_file_close_int] = ACTIONS(1781),
    [sym__wc_file_read_int] = ACTIONS(1781),
    [sym__wc_file_read_line_int] = ACTIONS(1781),
    [sym__wc_file_write_int] = ACTIONS(1781),
    [sym__wc_file_write_line_int] = ACTIONS(1781),
    [aux_sym_preproc_include_token1] = ACTIONS(1781),
    [aux_sym_preproc_define_token1] = ACTIONS(1783),
    [aux_sym_preproc_warn_token1] = ACTIONS(1783),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1783),
    [anon_sym_LBRACK] = ACTIONS(1783),
    [anon_sym_LBRACE] = ACTIONS(1783),
    [anon_sym_LPAREN] = ACTIONS(1781),
    [anon_sym_BANG] = ACTIONS(1783),
    [anon_sym_TILDE] = ACTIONS(1783),
    [anon_sym_DASH] = ACTIONS(1781),
    [anon_sym_PLUS] = ACTIONS(1781),
    [anon_sym_DASH_DASH] = ACTIONS(1783),
    [anon_sym_PLUS_PLUS] = ACTIONS(1783),
    [anon_sym_DQUOTE] = ACTIONS(1783),
    [anon_sym_SQUOTE] = ACTIONS(1783),
    [sym_number_literal] = ACTIONS(1783),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(326)] = {
    [ts_builtin_sym_end] = ACTIONS(1783),
    [sym_identifier] = ACTIONS(1781),
    [sym_program_name_keyword] = ACTIONS(1781),
    [sym_module_name_keyword] = ACTIONS(1781),
    [sym_define_device_keyword] = ACTIONS(1781),
    [sym_define_combine_keyword] = ACTIONS(1781),
    [sym_define_constant_keyword] = ACTIONS(1781),
    [sym_define_type_keyword] = ACTIONS(1781),
    [sym_define_variable_keyword] = ACTIONS(1781),
    [sym_define_system_variable_keyword] = ACTIONS(1781),
    [sym_define_connect_level_keyword] = ACTIONS(1781),
    [sym_define_latching_keyword] = ACTIONS(1781),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1781),
    [sym_define_toggling_keyword] = ACTIONS(1781),
    [sym_define_call_keyword] = ACTIONS(1781),
    [sym_define_function_keyword] = ACTIONS(1781),
    [sym_define_library_function_keyword] = ACTIONS(1781),
    [sym_define_start_keyword] = ACTIONS(1781),
    [sym_define_event_keyword] = ACTIONS(1781),
    [sym_define_program_keyword] = ACTIONS(1781),
    [sym_define_module_keyword] = ACTIONS(1781),
    [sym_char_keyword] = ACTIONS(1781),
    [sym_widechar_keyword] = ACTIONS(1781),
    [sym_integer_keyword] = ACTIONS(1781),
    [sym_sinteger_keyword] = ACTIONS(1781),
    [sym_long_keyword] = ACTIONS(1781),
    [sym_slong_keyword] = ACTIONS(1781),
    [sym_float_keyword] = ACTIONS(1781),
    [sym_double_keyword] = ACTIONS(1781),
    [sym_variant_keyword] = ACTIONS(1781),
    [sym_variantarray_keyword] = ACTIONS(1781),
    [sym_local_var_keyword] = ACTIONS(1781),
    [sym_stack_var_keyword] = ACTIONS(1781),
    [sym_constant_keyword] = ACTIONS(1781),
    [sym_volatile_keyword] = ACTIONS(1781),
    [sym_non_volatile_keyword] = ACTIONS(1781),
    [sym_persistent_keyword] = ACTIONS(1781),
    [sym_dev_keyword] = ACTIONS(1781),
    [sym_devlev_keyword] = ACTIONS(1781),
    [sym_devchan_keyword] = ACTIONS(1781),
    [sym_if_keyword] = ACTIONS(1781),
    [sym_else_keyword] = ACTIONS(1781),
    [sym_switch_keyword] = ACTIONS(1781),
    [sym_case_keyword] = ACTIONS(1781),
    [sym_default_keyword] = ACTIONS(1781),
    [sym_while_keyword] = ACTIONS(1781),
    [sym_for_keyword] = ACTIONS(1781),
    [sym_break_keyword] = ACTIONS(1781),
    [sym_continue_keyword] = ACTIONS(1781),
    [sym_return_keyword] = ACTIONS(1781),
    [sym_select_keyword] = ACTIONS(1781),
    [sym_active_keyword] = ACTIONS(1781),
    [sym_struct_keyword] = ACTIONS(1781),
    [sym_structure_keyword] = ACTIONS(1781),
    [sym_button_event_keyword] = ACTIONS(1781),
    [sym_channel_event_keyword] = ACTIONS(1781),
    [sym_level_event_keyword] = ACTIONS(1781),
    [sym_data_event_keyword] = ACTIONS(1781),
    [sym_timeline_event_keyword] = ACTIONS(1781),
    [sym_custom_event_keyword] = ACTIONS(1781),
    [sym_push_keyword] = ACTIONS(1781),
    [sym_release_keyword] = ACTIONS(1781),
    [aux_sym_on_keyword_token1] = ACTIONS(1781),
    [aux_sym_off_keyword_token1] = ACTIONS(1781),
    [sym_send_level_keyword] = ACTIONS(1781),
    [sym_send_string_keyword] = ACTIONS(1781),
    [sym_send_command_keyword] = ACTIONS(1781),
    [sym_clear_buffer_keyword] = ACTIONS(1781),
    [sym_create_buffer_keyword] = ACTIONS(1781),
    [sym_create_multi_buffer_keyword] = ACTIONS(1781),
    [sym_call_keyword] = ACTIONS(1781),
    [sym_system_call_keyword] = ACTIONS(1781),
    [sym_devchan_to_keyword] = ACTIONS(1781),
    [sym_devchan_min_to_keyword] = ACTIONS(1781),
    [sym_devchan_total_off_keyword] = ACTIONS(1781),
    [sym_devchan_pulse_keyword] = ACTIONS(1781),
    [sym_wait_keyword] = ACTIONS(1781),
    [sym_cancel_wait_keyword] = ACTIONS(1781),
    [sym_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1781),
    [sym_not] = ACTIONS(1781),
    [sym_bnot] = ACTIONS(1781),
    [sym___file__] = ACTIONS(1781),
    [sym___date__] = ACTIONS(1781),
    [sym___ldate__] = ACTIONS(1781),
    [sym___time__] = ACTIONS(1781),
    [sym___line__] = ACTIONS(1781),
    [sym___name__] = ACTIONS(1781),
    [sym___netlinx__] = ACTIONS(1781),
    [sym_netlinx_axi_version] = ACTIONS(1781),
    [sym_url_flg_tcp] = ACTIONS(1781),
    [sym_url_flg_accinfopresent] = ACTIONS(1781),
    [sym_url_flg_temp] = ACTIONS(1781),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1781),
    [sym_url_flg_stat_mask] = ACTIONS(1781),
    [sym_url_flg_stat_lookup] = ACTIONS(1781),
    [sym_url_flg_stat_connecting] = ACTIONS(1781),
    [sym_url_flg_stat_waiting] = ACTIONS(1781),
    [sym_url_flg_stat_connected] = ACTIONS(1781),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1781),
    [sym_first_virtual_device] = ACTIONS(1781),
    [sym_first_local_port] = ACTIONS(1781),
    [sym_dynamic_virtual_device] = ACTIONS(1781),
    [sym_timeline_once] = ACTIONS(1781),
    [sym_timeline_repeat] = ACTIONS(1781),
    [sym_timeline_absolute] = ACTIONS(1781),
    [sym_timeline_relative] = ACTIONS(1781),
    [sym_do_push_timed_infinite] = ACTIONS(1781),
    [sym_source_type_no_address] = ACTIONS(1781),
    [sym_source_type_neuron_id] = ACTIONS(1781),
    [sym_source_type_ip_address] = ACTIONS(1781),
    [sym_source_type_axlink] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1781),
    [sym_source_type_ip_socket_address] = ACTIONS(1781),
    [sym_source_type_rs232] = ACTIONS(1781),
    [sym_source_type_internal] = ACTIONS(1781),
    [sym_source_type_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1781),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1781),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1781),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1781),
    [sym_true] = ACTIONS(1781),
    [sym_false] = ACTIONS(1781),
    [sym_file_read_only] = ACTIONS(1781),
    [sym_file_rw_new] = ACTIONS(1781),
    [sym_file_rw_append] = ACTIONS(1781),
    [sym_ip_tcp] = ACTIONS(1781),
    [sym_ip_udp] = ACTIONS(1781),
    [sym_ip_udp_2way] = ACTIONS(1781),
    [sym_xml_encode_types] = ACTIONS(1781),
    [sym_xml_encode_char_as_list] = ACTIONS(1781),
    [sym_xml_encode_le] = ACTIONS(1781),
    [sym_xml_decode_types] = ACTIONS(1781),
    [sym_xml_decode_no_preserve] = ACTIONS(1781),
    [sym_duet_dev_type_display_device] = ACTIONS(1781),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_io_device] = ACTIONS(1781),
    [sym_duet_dev_type_relay_device] = ACTIONS(1781),
    [sym_duet_dev_type_ups] = ACTIONS(1781),
    [sym_duet_dev_type_amplifier] = ACTIONS(1781),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1781),
    [sym_duet_dev_type_camera] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1781),
    [sym_duet_dev_type_dss] = ACTIONS(1781),
    [sym_duet_dev_type_dvr] = ACTIONS(1781),
    [sym_duet_dev_type_disc_device] = ACTIONS(1781),
    [sym_duet_dev_type_document_camera] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1781),
    [sym_duet_dev_type_hvac] = ACTIONS(1781),
    [sym_duet_dev_type_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_light] = ACTIONS(1781),
    [sym_duet_dev_type_monitor] = ACTIONS(1781),
    [sym_duet_dev_type_motor] = ACTIONS(1781),
    [sym_duet_dev_type_multi_window] = ACTIONS(1781),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1781),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1781),
    [sym_duet_dev_type_receiver] = ACTIONS(1781),
    [sym_duet_dev_type_security_system] = ACTIONS(1781),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1781),
    [sym_duet_dev_type_settop_box] = ACTIONS(1781),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1781),
    [sym_duet_dev_type_switcher] = ACTIONS(1781),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_tv] = ACTIONS(1781),
    [sym_duet_dev_type_utility] = ACTIONS(1781),
    [sym_duet_dev_type_vcr] = ACTIONS(1781),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_video_processor] = ACTIONS(1781),
    [sym_duet_dev_type_video_projector] = ACTIONS(1781),
    [sym_duet_dev_type_video_wall] = ACTIONS(1781),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1781),
    [sym_duet_dev_type_weather] = ACTIONS(1781),
    [sym_duet_dev_not_polled] = ACTIONS(1781),
    [sym_duet_dev_polled] = ACTIONS(1781),
    [sym_clkmgr_mode_network] = ACTIONS(1781),
    [sym_clkmgr_mode_standalone] = ACTIONS(1781),
    [sym_normal_standby] = ACTIONS(1781),
    [sym_normal_wake] = ACTIONS(1781),
    [sym_ip_multicast_ttl_option] = ACTIONS(1781),
    [sym_ip_ttl_subnet] = ACTIONS(1781),
    [sym_ip_ttl_site] = ACTIONS(1781),
    [sym_ip_ttl_region] = ACTIONS(1781),
    [sym_ip_ttl_continent] = ACTIONS(1781),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1781),
    [sym_ip_nodelay_on] = ACTIONS(1781),
    [sym_ip_nodelay_off] = ACTIONS(1781),
    [sym_amx_error] = ACTIONS(1781),
    [sym_amx_warning] = ACTIONS(1781),
    [sym_amx_info] = ACTIONS(1781),
    [sym_amx_debug] = ACTIONS(1781),
    [sym_netlinx_login_success] = ACTIONS(1781),
    [sym_netlinx_login_fail] = ACTIONS(1781),
    [sym_netlinx_logout] = ACTIONS(1781),
    [sym_null_str] = ACTIONS(1781),
    [sym_smtp_address] = ACTIONS(1781),
    [sym_smtp_port_number] = ACTIONS(1781),
    [sym_smtp_username] = ACTIONS(1781),
    [sym_smtp_password] = ACTIONS(1781),
    [sym_smtp_require_tls] = ACTIONS(1781),
    [sym_smtp_from] = ACTIONS(1781),
    [sym_smtp_tls_true] = ACTIONS(1781),
    [sym_smtp_tls_false] = ACTIONS(1781),
    [sym_listview_on_row_select_event] = ACTIONS(1781),
    [sym_data_structure_unknown] = ACTIONS(1781),
    [sym_data_structure_datafeed] = ACTIONS(1781),
    [sym_data_structure_datarecordset] = ACTIONS(1781),
    [sym_data_structure_datarecord] = ACTIONS(1781),
    [sym_data_structure_datafield] = ACTIONS(1781),
    [sym_data_max_content_field_count] = ACTIONS(1781),
    [sym_data_max_metadata_field_count] = ACTIONS(1781),
    [sym_data_max_id_length] = ACTIONS(1781),
    [sym_data_max_name_length] = ACTIONS(1781),
    [sym_data_max_description_length] = ACTIONS(1781),
    [sym_data_max_type_length] = ACTIONS(1781),
    [sym_data_max_format_length] = ACTIONS(1781),
    [sym_data_max_source_length] = ACTIONS(1781),
    [sym_data_max_value_length] = ACTIONS(1781),
    [sym_data_max_label_length] = ACTIONS(1781),
    [sym_data_max_url_length] = ACTIONS(1781),
    [sym_max_last_login_info_length] = ACTIONS(1781),
    [sym_data_type_unknown] = ACTIONS(1781),
    [sym_data_type_string] = ACTIONS(1781),
    [sym_data_type_datetime] = ACTIONS(1781),
    [sym_data_type_date] = ACTIONS(1781),
    [sym_data_type_time] = ACTIONS(1781),
    [sym_data_type_image] = ACTIONS(1781),
    [sym_data_format_url] = ACTIONS(1781),
    [sym_data_format_phone] = ACTIONS(1781),
    [sym_data_format_email] = ACTIONS(1781),
    [sym_data_format_iso8601] = ACTIONS(1781),
    [sym_auditlog_priv] = ACTIONS(1781),
    [sym_configuration_priv] = ACTIONS(1781),
    [sym_deviceconfig_priv] = ACTIONS(1781),
    [sym_ftp_priv] = ACTIONS(1781),
    [sym_http_priv] = ACTIONS(1781),
    [sym_networkconfig_priv] = ACTIONS(1781),
    [sym_programport_priv] = ACTIONS(1781),
    [sym_remoteui_priv] = ACTIONS(1781),
    [sym_securitycontrol_priv] = ACTIONS(1781),
    [sym_softwaremanagement_priv] = ACTIONS(1781),
    [sym_terminal_priv] = ACTIONS(1781),
    [sym_tpadmin_priv] = ACTIONS(1781),
    [sym_user1_priv] = ACTIONS(1781),
    [sym_user2_priv] = ACTIONS(1781),
    [sym_user3_priv] = ACTIONS(1781),
    [sym_user4_priv] = ACTIONS(1781),
    [sym_usermanagement_priv] = ACTIONS(1781),
    [sym_valid_account] = ACTIONS(1781),
    [sym_err_name_invalid] = ACTIONS(1781),
    [sym_err_password_invalid] = ACTIONS(1781),
    [sym_err_invalid_account] = ACTIONS(1781),
    [sym_err_no_authorization] = ACTIONS(1781),
    [sym_err_invalid_parameter] = ACTIONS(1781),
    [sym_err_user_locked_out] = ACTIONS(1781),
    [sym_err_user_account_expired] = ACTIONS(1781),
    [sym_err_general_error] = ACTIONS(1781),
    [sym_err_authentication_error] = ACTIONS(1781),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1781),
    [sym_tls_validate_certificate] = ACTIONS(1781),
    [sym_tdata] = ACTIONS(1781),
    [sym_tchannel] = ACTIONS(1781),
    [sym_tlevel] = ACTIONS(1781),
    [sym_tbutton] = ACTIONS(1781),
    [sym_ttimeline] = ACTIONS(1781),
    [sym_tcustom] = ACTIONS(1781),
    [sym_url_struct] = ACTIONS(1781),
    [sym_dns_struct] = ACTIONS(1781),
    [sym_ip_address_struct] = ACTIONS(1781),
    [sym_dev_info_struct] = ACTIONS(1781),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1781),
    [sym_data_feed] = ACTIONS(1781),
    [sym_data_field] = ACTIONS(1781),
    [sym_data_record] = ACTIONS(1781),
    [sym_wc_data_feed] = ACTIONS(1781),
    [sym_wc_data_field] = ACTIONS(1781),
    [sym_wc_data_record] = ACTIONS(1781),
    [sym_last_login_info] = ACTIONS(1781),
    [sym____reserved___] = ACTIONS(1781),
    [sym_date] = ACTIONS(1781),
    [sym_day] = ACTIONS(1781),
    [sym_time] = ACTIONS(1781),
    [sym_ldate] = ACTIONS(1781),
    [sym_push_device] = ACTIONS(1781),
    [sym_push_channel] = ACTIONS(1781),
    [sym_push_devchan] = ACTIONS(1781),
    [sym_release_device] = ACTIONS(1781),
    [sym_release_channel] = ACTIONS(1781),
    [sym_release_devchan] = ACTIONS(1781),
    [sym_master_slot] = ACTIONS(1781),
    [sym_get_pulse_time] = ACTIONS(1781),
    [sym_get_timer] = ACTIONS(1781),
    [sym_master_sn] = ACTIONS(1781),
    [sym_system_number] = ACTIONS(1781),
    [sym_button] = ACTIONS(1781),
    [sym_dv_channel] = ACTIONS(1781),
    [sym_data] = ACTIONS(1781),
    [sym_level] = ACTIONS(1781),
    [sym_timeline] = ACTIONS(1781),
    [sym_channel] = ACTIONS(1781),
    [sym_custom] = ACTIONS(1781),
    [sym_atoi] = ACTIONS(1781),
    [sym_atol] = ACTIONS(1781),
    [sym_atof] = ACTIONS(1781),
    [sym_itoa] = ACTIONS(1781),
    [sym_itohex] = ACTIONS(1781),
    [sym_hextoi] = ACTIONS(1781),
    [sym_ftoa] = ACTIONS(1781),
    [sym_find_string] = ACTIONS(1781),
    [sym_left_string] = ACTIONS(1781),
    [sym_right_string] = ACTIONS(1781),
    [sym_mid_string] = ACTIONS(1781),
    [sym_remove_string] = ACTIONS(1781),
    [sym_get_buffer_string] = ACTIONS(1781),
    [sym_get_buffer_char] = ACTIONS(1781),
    [sym_get_multi_buffer_string] = ACTIONS(1781),
    [sym_length_string] = ACTIONS(1781),
    [sym_max_length_string] = ACTIONS(1781),
    [sym_set_length_string] = ACTIONS(1781),
    [sym_compare_string] = ACTIONS(1781),
    [sym_upper_string] = ACTIONS(1781),
    [sym_lower_string] = ACTIONS(1781),
    [sym_get_last] = ACTIONS(1781),
    [sym_date_to_day] = ACTIONS(1781),
    [sym_date_to_month] = ACTIONS(1781),
    [sym_date_to_year] = ACTIONS(1781),
    [sym_time_to_hour] = ACTIONS(1781),
    [sym_time_to_minute] = ACTIONS(1781),
    [sym_time_to_second] = ACTIONS(1781),
    [sym_day_of_week] = ACTIONS(1781),
    [sym_set_pulse_time] = ACTIONS(1781),
    [sym_set_timer] = ACTIONS(1781),
    [sym_file_open] = ACTIONS(1781),
    [sym_file_close] = ACTIONS(1781),
    [sym_file_read] = ACTIONS(1781),
    [sym_file_read_line] = ACTIONS(1781),
    [sym_file_write] = ACTIONS(1781),
    [sym_file_write_line] = ACTIONS(1781),
    [sym_file_copy] = ACTIONS(1781),
    [sym_file_delete] = ACTIONS(1781),
    [sym_file_rename] = ACTIONS(1781),
    [sym_file_dir] = ACTIONS(1781),
    [sym_file_getdir] = ACTIONS(1781),
    [sym_file_setdir] = ACTIONS(1781),
    [sym_file_create_dir] = ACTIONS(1781),
    [sym_file_remove_dir] = ACTIONS(1781),
    [sym_file_seek] = ACTIONS(1781),
    [sym_length_array] = ACTIONS(1781),
    [sym_max_length_array] = ACTIONS(1781),
    [sym_set_length_array] = ACTIONS(1781),
    [sym_ip_client_open] = ACTIONS(1781),
    [sym_ip_client_close] = ACTIONS(1781),
    [sym_ip_server_open] = ACTIONS(1781),
    [sym_ip_server_close] = ACTIONS(1781),
    [sym_ip_bound_client_open] = ACTIONS(1781),
    [sym_ip_mc_server_open] = ACTIONS(1781),
    [sym_ip_set_option] = ACTIONS(1781),
    [sym_ssh_client_open] = ACTIONS(1781),
    [sym_ssh_client_close] = ACTIONS(1781),
    [sym_tls_client_open] = ACTIONS(1781),
    [sym_tls_client_close] = ACTIONS(1781),
    [sym_do_push] = ACTIONS(1781),
    [sym_do_push_timed] = ACTIONS(1781),
    [sym_do_release] = ACTIONS(1781),
    [sym_device_id] = ACTIONS(1781),
    [sym_device_info] = ACTIONS(1781),
    [sym_device_id_string] = ACTIONS(1781),
    [sym_device_standby] = ACTIONS(1781),
    [sym_device_wake] = ACTIONS(1781),
    [sym_timeline_create] = ACTIONS(1781),
    [sym_timeline_kill] = ACTIONS(1781),
    [sym_timeline_reload] = ACTIONS(1781),
    [sym_timeline_pause] = ACTIONS(1781),
    [sym_timeline_restart] = ACTIONS(1781),
    [sym_timeline_set] = ACTIONS(1781),
    [sym_timeline_get] = ACTIONS(1781),
    [sym_timeline_active] = ACTIONS(1781),
    [sym_timeline_dynamic_id] = ACTIONS(1781),
    [sym_variable_to_string] = ACTIONS(1781),
    [sym_string_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_string] = ACTIONS(1781),
    [sym_type_cast] = ACTIONS(1781),
    [sym_variable_to_xml] = ACTIONS(1781),
    [sym_xml_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_xml] = ACTIONS(1781),
    [sym_redirect_string] = ACTIONS(1781),
    [sym_random_number] = ACTIONS(1781),
    [sym_get_unique_id] = ACTIONS(1781),
    [sym_get_system_number] = ACTIONS(1781),
    [sym_set_system_number] = ACTIONS(1781),
    [sym_get_serial_number] = ACTIONS(1781),
    [sym_get_master_build] = ACTIONS(1781),
    [sym_set_validation_code] = ACTIONS(1781),
    [sym_reboot] = ACTIONS(1781),
    [sym_load_duet_module] = ACTIONS(1781),
    [sym_unload_duet_module] = ACTIONS(1781),
    [sym_duet_mem_size_set] = ACTIONS(1781),
    [sym_duet_mem_size_get] = ACTIONS(1781),
    [sym_dynamic_application_device] = ACTIONS(1781),
    [sym_dynamic_polled_port] = ACTIONS(1781),
    [sym_static_port_binding] = ACTIONS(1781),
    [sym_static_ip_binding] = ACTIONS(1781),
    [sym_get_dns_list] = ACTIONS(1781),
    [sym_set_dns_list] = ACTIONS(1781),
    [sym_get_ip_address] = ACTIONS(1781),
    [sym_set_ip_address] = ACTIONS(1781),
    [sym_get_url_list] = ACTIONS(1781),
    [sym_add_url_entry] = ACTIONS(1781),
    [sym_delete_url_entry] = ACTIONS(1781),
    [sym_abs_value] = ACTIONS(1781),
    [sym_max_value] = ACTIONS(1781),
    [sym_min_value] = ACTIONS(1781),
    [sym_exp_value] = ACTIONS(1781),
    [sym_log_value] = ACTIONS(1781),
    [sym_log10_value] = ACTIONS(1781),
    [sym_power_value] = ACTIONS(1781),
    [sym_sqrt_value] = ACTIONS(1781),
    [sym_raw_be] = ACTIONS(1781),
    [sym_raw_le] = ACTIONS(1781),
    [sym_internal_queue_size_set] = ACTIONS(1781),
    [sym_internal_queue_size_get] = ACTIONS(1781),
    [sym_internal_threshold_set] = ACTIONS(1781),
    [sym_internal_threshold_get] = ACTIONS(1781),
    [sym_rebuilt_event] = ACTIONS(1781),
    [sym_do_custom_event] = ACTIONS(1781),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1781),
    [sym_clkmgr_set_clk_source] = ACTIONS(1781),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1781),
    [sym_clkmgr_get_timezone] = ACTIONS(1781),
    [sym_clkmgr_set_timezone] = ACTIONS(1781),
    [sym_clkmgr_get_resync_period] = ACTIONS(1781),
    [sym_clkmgr_set_resync_period] = ACTIONS(1781),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_timeservers] = ACTIONS(1781),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_set_log_level] = ACTIONS(1781),
    [sym_get_log_level] = ACTIONS(1781),
    [sym_amx_log] = ACTIONS(1781),
    [sym_smtp_server_config_set] = ACTIONS(1781),
    [sym_smtp_server_config_get] = ACTIONS(1781),
    [sym_smtp_send] = ACTIONS(1781),
    [sym_validate_netlinx_account] = ACTIONS(1781),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1781),
    [sym_authenticate_certificate] = ACTIONS(1781),
    [sym_audit_netlinx_session_event] = ACTIONS(1781),
    [sym_audit_netlinx_generic_event] = ACTIONS(1781),
    [sym_get_available_flash_disk_space] = ACTIONS(1781),
    [sym_get_max_flash_disk_space] = ACTIONS(1781),
    [sym_data_create_feed] = ACTIONS(1781),
    [sym_data_delete_feed] = ACTIONS(1781),
    [sym_data_publish_feed] = ACTIONS(1781),
    [sym_data_get_published_feed] = ACTIONS(1781),
    [sym_data_add_record] = ACTIONS(1781),
    [sym_data_get_event_record] = ACTIONS(1781),
    [sym__wc_data_create_feed] = ACTIONS(1781),
    [sym__wc_data_add_record] = ACTIONS(1781),
    [sym__wc_data_get_event_record] = ACTIONS(1781),
    [sym_format] = ACTIONS(1781),
    [sym_astro_clock] = ACTIONS(1781),
    [sym_set_outdoor_temperature] = ACTIONS(1781),
    [sym_set_virtual_level_count] = ACTIONS(1781),
    [sym_set_virtual_channel_count] = ACTIONS(1781),
    [sym_set_virtual_port_count] = ACTIONS(1781),
    [sym__wc_encode_int] = ACTIONS(1781),
    [sym__wc_decode_int] = ACTIONS(1781),
    [sym__wc_int] = ACTIONS(1781),
    [sym__wc_to_ch_int] = ACTIONS(1781),
    [sym__ch_to_wc_int] = ACTIONS(1781),
    [sym__wc_find_string_int] = ACTIONS(1781),
    [sym__wc_left_string_int] = ACTIONS(1781),
    [sym__wc_lower_string_int] = ACTIONS(1781),
    [sym__wc_mid_string_int] = ACTIONS(1781),
    [sym__wc_remove_string_int] = ACTIONS(1781),
    [sym__wc_right_string_int] = ACTIONS(1781),
    [sym__wc_upper_string_int] = ACTIONS(1781),
    [sym__wc_compare_string_int] = ACTIONS(1781),
    [sym__wc_get_buffer_char_int] = ACTIONS(1781),
    [sym__wc_get_buffer_string_int] = ACTIONS(1781),
    [sym__wc_concat_string_int] = ACTIONS(1781),
    [sym__wc_file_open_int] = ACTIONS(1781),
    [sym__wc_file_close_int] = ACTIONS(1781),
    [sym__wc_file_read_int] = ACTIONS(1781),
    [sym__wc_file_read_line_int] = ACTIONS(1781),
    [sym__wc_file_write_int] = ACTIONS(1781),
    [sym__wc_file_write_line_int] = ACTIONS(1781),
    [aux_sym_preproc_include_token1] = ACTIONS(1781),
    [aux_sym_preproc_define_token1] = ACTIONS(1783),
    [aux_sym_preproc_warn_token1] = ACTIONS(1783),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1783),
    [anon_sym_LBRACK] = ACTIONS(1783),
    [anon_sym_LBRACE] = ACTIONS(1783),
    [anon_sym_LPAREN] = ACTIONS(1781),
    [anon_sym_BANG] = ACTIONS(1783),
    [anon_sym_TILDE] = ACTIONS(1783),
    [anon_sym_DASH] = ACTIONS(1781),
    [anon_sym_PLUS] = ACTIONS(1781),
    [anon_sym_DASH_DASH] = ACTIONS(1783),
    [anon_sym_PLUS_PLUS] = ACTIONS(1783),
    [anon_sym_DQUOTE] = ACTIONS(1783),
    [anon_sym_SQUOTE] = ACTIONS(1783),
    [sym_number_literal] = ACTIONS(1783),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(327)] = {
    [sym_identifier] = ACTIONS(2021),
    [sym_program_name_keyword] = ACTIONS(2021),
    [sym_module_name_keyword] = ACTIONS(2021),
    [sym_define_device_keyword] = ACTIONS(2021),
    [sym_define_combine_keyword] = ACTIONS(2021),
    [sym_define_constant_keyword] = ACTIONS(2021),
    [sym_define_type_keyword] = ACTIONS(2021),
    [sym_define_variable_keyword] = ACTIONS(2021),
    [sym_define_system_variable_keyword] = ACTIONS(2021),
    [sym_define_connect_level_keyword] = ACTIONS(2021),
    [sym_define_latching_keyword] = ACTIONS(2021),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2021),
    [sym_define_toggling_keyword] = ACTIONS(2021),
    [sym_define_call_keyword] = ACTIONS(2021),
    [sym_define_function_keyword] = ACTIONS(2021),
    [sym_define_library_function_keyword] = ACTIONS(2021),
    [sym_define_start_keyword] = ACTIONS(2021),
    [sym_define_event_keyword] = ACTIONS(2021),
    [sym_define_program_keyword] = ACTIONS(2021),
    [sym_define_module_keyword] = ACTIONS(2021),
    [sym_char_keyword] = ACTIONS(2021),
    [sym_widechar_keyword] = ACTIONS(2021),
    [sym_integer_keyword] = ACTIONS(2021),
    [sym_sinteger_keyword] = ACTIONS(2021),
    [sym_long_keyword] = ACTIONS(2021),
    [sym_slong_keyword] = ACTIONS(2021),
    [sym_float_keyword] = ACTIONS(2021),
    [sym_double_keyword] = ACTIONS(2021),
    [sym_variant_keyword] = ACTIONS(2021),
    [sym_variantarray_keyword] = ACTIONS(2021),
    [sym_local_var_keyword] = ACTIONS(2021),
    [sym_stack_var_keyword] = ACTIONS(2021),
    [sym_constant_keyword] = ACTIONS(2021),
    [sym_volatile_keyword] = ACTIONS(2021),
    [sym_non_volatile_keyword] = ACTIONS(2021),
    [sym_persistent_keyword] = ACTIONS(2021),
    [sym_dev_keyword] = ACTIONS(2021),
    [sym_devlev_keyword] = ACTIONS(2021),
    [sym_devchan_keyword] = ACTIONS(2021),
    [sym_if_keyword] = ACTIONS(2021),
    [sym_switch_keyword] = ACTIONS(2021),
    [sym_case_keyword] = ACTIONS(2021),
    [sym_default_keyword] = ACTIONS(2021),
    [sym_while_keyword] = ACTIONS(2021),
    [sym_for_keyword] = ACTIONS(2021),
    [sym_break_keyword] = ACTIONS(2021),
    [sym_continue_keyword] = ACTIONS(2021),
    [sym_return_keyword] = ACTIONS(2021),
    [sym_select_keyword] = ACTIONS(2021),
    [sym_active_keyword] = ACTIONS(2021),
    [sym_struct_keyword] = ACTIONS(2021),
    [sym_structure_keyword] = ACTIONS(2021),
    [sym_button_event_keyword] = ACTIONS(2021),
    [sym_channel_event_keyword] = ACTIONS(2021),
    [sym_level_event_keyword] = ACTIONS(2021),
    [sym_data_event_keyword] = ACTIONS(2021),
    [sym_timeline_event_keyword] = ACTIONS(2021),
    [sym_custom_event_keyword] = ACTIONS(2021),
    [sym_push_keyword] = ACTIONS(2021),
    [sym_release_keyword] = ACTIONS(2021),
    [aux_sym_on_keyword_token1] = ACTIONS(2021),
    [aux_sym_off_keyword_token1] = ACTIONS(2021),
    [sym_send_level_keyword] = ACTIONS(2021),
    [sym_send_string_keyword] = ACTIONS(2021),
    [sym_send_command_keyword] = ACTIONS(2021),
    [sym_clear_buffer_keyword] = ACTIONS(2021),
    [sym_create_buffer_keyword] = ACTIONS(2021),
    [sym_create_multi_buffer_keyword] = ACTIONS(2021),
    [sym_call_keyword] = ACTIONS(2021),
    [sym_system_call_keyword] = ACTIONS(2021),
    [sym_devchan_to_keyword] = ACTIONS(2021),
    [sym_devchan_min_to_keyword] = ACTIONS(2021),
    [sym_devchan_total_off_keyword] = ACTIONS(2021),
    [sym_devchan_pulse_keyword] = ACTIONS(2021),
    [sym_wait_keyword] = ACTIONS(2021),
    [sym_cancel_wait_keyword] = ACTIONS(2021),
    [sym_wait_until_keyword] = ACTIONS(2021),
    [sym_cancel_wait_until_keyword] = ACTIONS(2021),
    [sym_cancel_all_wait_keyword] = ACTIONS(2021),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2021),
    [sym_not] = ACTIONS(2021),
    [sym_bnot] = ACTIONS(2021),
    [sym___file__] = ACTIONS(2021),
    [sym___date__] = ACTIONS(2021),
    [sym___ldate__] = ACTIONS(2021),
    [sym___time__] = ACTIONS(2021),
    [sym___line__] = ACTIONS(2021),
    [sym___name__] = ACTIONS(2021),
    [sym___netlinx__] = ACTIONS(2021),
    [sym_netlinx_axi_version] = ACTIONS(2021),
    [sym_url_flg_tcp] = ACTIONS(2021),
    [sym_url_flg_accinfopresent] = ACTIONS(2021),
    [sym_url_flg_temp] = ACTIONS(2021),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2021),
    [sym_url_flg_stat_mask] = ACTIONS(2021),
    [sym_url_flg_stat_lookup] = ACTIONS(2021),
    [sym_url_flg_stat_connecting] = ACTIONS(2021),
    [sym_url_flg_stat_waiting] = ACTIONS(2021),
    [sym_url_flg_stat_connected] = ACTIONS(2021),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2021),
    [sym_first_virtual_device] = ACTIONS(2021),
    [sym_first_local_port] = ACTIONS(2021),
    [sym_dynamic_virtual_device] = ACTIONS(2021),
    [sym_timeline_once] = ACTIONS(2021),
    [sym_timeline_repeat] = ACTIONS(2021),
    [sym_timeline_absolute] = ACTIONS(2021),
    [sym_timeline_relative] = ACTIONS(2021),
    [sym_do_push_timed_infinite] = ACTIONS(2021),
    [sym_source_type_no_address] = ACTIONS(2021),
    [sym_source_type_neuron_id] = ACTIONS(2021),
    [sym_source_type_ip_address] = ACTIONS(2021),
    [sym_source_type_axlink] = ACTIONS(2021),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2021),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2021),
    [sym_source_type_ip_socket_address] = ACTIONS(2021),
    [sym_source_type_rs232] = ACTIONS(2021),
    [sym_source_type_internal] = ACTIONS(2021),
    [sym_source_type_mac_address] = ACTIONS(2021),
    [sym_source_type_ipv4_port] = ACTIONS(2021),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2021),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2021),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2021),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2021),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2021),
    [sym_true] = ACTIONS(2021),
    [sym_false] = ACTIONS(2021),
    [sym_file_read_only] = ACTIONS(2021),
    [sym_file_rw_new] = ACTIONS(2021),
    [sym_file_rw_append] = ACTIONS(2021),
    [sym_ip_tcp] = ACTIONS(2021),
    [sym_ip_udp] = ACTIONS(2021),
    [sym_ip_udp_2way] = ACTIONS(2021),
    [sym_xml_encode_types] = ACTIONS(2021),
    [sym_xml_encode_char_as_list] = ACTIONS(2021),
    [sym_xml_encode_le] = ACTIONS(2021),
    [sym_xml_decode_types] = ACTIONS(2021),
    [sym_xml_decode_no_preserve] = ACTIONS(2021),
    [sym_duet_dev_type_display_device] = ACTIONS(2021),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2021),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2021),
    [sym_duet_dev_type_io_device] = ACTIONS(2021),
    [sym_duet_dev_type_relay_device] = ACTIONS(2021),
    [sym_duet_dev_type_ups] = ACTIONS(2021),
    [sym_duet_dev_type_amplifier] = ACTIONS(2021),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2021),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2021),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2021),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2021),
    [sym_duet_dev_type_camera] = ACTIONS(2021),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2021),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2021),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2021),
    [sym_duet_dev_type_dss] = ACTIONS(2021),
    [sym_duet_dev_type_dvr] = ACTIONS(2021),
    [sym_duet_dev_type_disc_device] = ACTIONS(2021),
    [sym_duet_dev_type_document_camera] = ACTIONS(2021),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2021),
    [sym_duet_dev_type_hvac] = ACTIONS(2021),
    [sym_duet_dev_type_keypad] = ACTIONS(2021),
    [sym_duet_dev_type_light] = ACTIONS(2021),
    [sym_duet_dev_type_monitor] = ACTIONS(2021),
    [sym_duet_dev_type_motor] = ACTIONS(2021),
    [sym_duet_dev_type_multi_window] = ACTIONS(2021),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2021),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2021),
    [sym_duet_dev_type_receiver] = ACTIONS(2021),
    [sym_duet_dev_type_security_system] = ACTIONS(2021),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2021),
    [sym_duet_dev_type_settop_box] = ACTIONS(2021),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2021),
    [sym_duet_dev_type_switcher] = ACTIONS(2021),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2021),
    [sym_duet_dev_type_tv] = ACTIONS(2021),
    [sym_duet_dev_type_utility] = ACTIONS(2021),
    [sym_duet_dev_type_vcr] = ACTIONS(2021),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2021),
    [sym_duet_dev_type_video_processor] = ACTIONS(2021),
    [sym_duet_dev_type_video_projector] = ACTIONS(2021),
    [sym_duet_dev_type_video_wall] = ACTIONS(2021),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2021),
    [sym_duet_dev_type_weather] = ACTIONS(2021),
    [sym_duet_dev_not_polled] = ACTIONS(2021),
    [sym_duet_dev_polled] = ACTIONS(2021),
    [sym_clkmgr_mode_network] = ACTIONS(2021),
    [sym_clkmgr_mode_standalone] = ACTIONS(2021),
    [sym_normal_standby] = ACTIONS(2021),
    [sym_normal_wake] = ACTIONS(2021),
    [sym_ip_multicast_ttl_option] = ACTIONS(2021),
    [sym_ip_ttl_subnet] = ACTIONS(2021),
    [sym_ip_ttl_site] = ACTIONS(2021),
    [sym_ip_ttl_region] = ACTIONS(2021),
    [sym_ip_ttl_continent] = ACTIONS(2021),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2021),
    [sym_ip_nodelay_on] = ACTIONS(2021),
    [sym_ip_nodelay_off] = ACTIONS(2021),
    [sym_amx_error] = ACTIONS(2021),
    [sym_amx_warning] = ACTIONS(2021),
    [sym_amx_info] = ACTIONS(2021),
    [sym_amx_debug] = ACTIONS(2021),
    [sym_netlinx_login_success] = ACTIONS(2021),
    [sym_netlinx_login_fail] = ACTIONS(2021),
    [sym_netlinx_logout] = ACTIONS(2021),
    [sym_null_str] = ACTIONS(2021),
    [sym_smtp_address] = ACTIONS(2021),
    [sym_smtp_port_number] = ACTIONS(2021),
    [sym_smtp_username] = ACTIONS(2021),
    [sym_smtp_password] = ACTIONS(2021),
    [sym_smtp_require_tls] = ACTIONS(2021),
    [sym_smtp_from] = ACTIONS(2021),
    [sym_smtp_tls_true] = ACTIONS(2021),
    [sym_smtp_tls_false] = ACTIONS(2021),
    [sym_listview_on_row_select_event] = ACTIONS(2021),
    [sym_data_structure_unknown] = ACTIONS(2021),
    [sym_data_structure_datafeed] = ACTIONS(2021),
    [sym_data_structure_datarecordset] = ACTIONS(2021),
    [sym_data_structure_datarecord] = ACTIONS(2021),
    [sym_data_structure_datafield] = ACTIONS(2021),
    [sym_data_max_content_field_count] = ACTIONS(2021),
    [sym_data_max_metadata_field_count] = ACTIONS(2021),
    [sym_data_max_id_length] = ACTIONS(2021),
    [sym_data_max_name_length] = ACTIONS(2021),
    [sym_data_max_description_length] = ACTIONS(2021),
    [sym_data_max_type_length] = ACTIONS(2021),
    [sym_data_max_format_length] = ACTIONS(2021),
    [sym_data_max_source_length] = ACTIONS(2021),
    [sym_data_max_value_length] = ACTIONS(2021),
    [sym_data_max_label_length] = ACTIONS(2021),
    [sym_data_max_url_length] = ACTIONS(2021),
    [sym_max_last_login_info_length] = ACTIONS(2021),
    [sym_data_type_unknown] = ACTIONS(2021),
    [sym_data_type_string] = ACTIONS(2021),
    [sym_data_type_datetime] = ACTIONS(2021),
    [sym_data_type_date] = ACTIONS(2021),
    [sym_data_type_time] = ACTIONS(2021),
    [sym_data_type_image] = ACTIONS(2021),
    [sym_data_format_url] = ACTIONS(2021),
    [sym_data_format_phone] = ACTIONS(2021),
    [sym_data_format_email] = ACTIONS(2021),
    [sym_data_format_iso8601] = ACTIONS(2021),
    [sym_auditlog_priv] = ACTIONS(2021),
    [sym_configuration_priv] = ACTIONS(2021),
    [sym_deviceconfig_priv] = ACTIONS(2021),
    [sym_ftp_priv] = ACTIONS(2021),
    [sym_http_priv] = ACTIONS(2021),
    [sym_networkconfig_priv] = ACTIONS(2021),
    [sym_programport_priv] = ACTIONS(2021),
    [sym_remoteui_priv] = ACTIONS(2021),
    [sym_securitycontrol_priv] = ACTIONS(2021),
    [sym_softwaremanagement_priv] = ACTIONS(2021),
    [sym_terminal_priv] = ACTIONS(2021),
    [sym_tpadmin_priv] = ACTIONS(2021),
    [sym_user1_priv] = ACTIONS(2021),
    [sym_user2_priv] = ACTIONS(2021),
    [sym_user3_priv] = ACTIONS(2021),
    [sym_user4_priv] = ACTIONS(2021),
    [sym_usermanagement_priv] = ACTIONS(2021),
    [sym_valid_account] = ACTIONS(2021),
    [sym_err_name_invalid] = ACTIONS(2021),
    [sym_err_password_invalid] = ACTIONS(2021),
    [sym_err_invalid_account] = ACTIONS(2021),
    [sym_err_no_authorization] = ACTIONS(2021),
    [sym_err_invalid_parameter] = ACTIONS(2021),
    [sym_err_user_locked_out] = ACTIONS(2021),
    [sym_err_user_account_expired] = ACTIONS(2021),
    [sym_err_general_error] = ACTIONS(2021),
    [sym_err_authentication_error] = ACTIONS(2021),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2021),
    [sym_tls_validate_certificate] = ACTIONS(2021),
    [sym_tdata] = ACTIONS(2021),
    [sym_tchannel] = ACTIONS(2021),
    [sym_tlevel] = ACTIONS(2021),
    [sym_tbutton] = ACTIONS(2021),
    [sym_ttimeline] = ACTIONS(2021),
    [sym_tcustom] = ACTIONS(2021),
    [sym_url_struct] = ACTIONS(2021),
    [sym_dns_struct] = ACTIONS(2021),
    [sym_ip_address_struct] = ACTIONS(2021),
    [sym_dev_info_struct] = ACTIONS(2021),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2021),
    [sym_data_feed] = ACTIONS(2021),
    [sym_data_field] = ACTIONS(2021),
    [sym_data_record] = ACTIONS(2021),
    [sym_wc_data_feed] = ACTIONS(2021),
    [sym_wc_data_field] = ACTIONS(2021),
    [sym_wc_data_record] = ACTIONS(2021),
    [sym_last_login_info] = ACTIONS(2021),
    [sym____reserved___] = ACTIONS(2021),
    [sym_date] = ACTIONS(2021),
    [sym_day] = ACTIONS(2021),
    [sym_time] = ACTIONS(2021),
    [sym_ldate] = ACTIONS(2021),
    [sym_push_device] = ACTIONS(2021),
    [sym_push_channel] = ACTIONS(2021),
    [sym_push_devchan] = ACTIONS(2021),
    [sym_release_device] = ACTIONS(2021),
    [sym_release_channel] = ACTIONS(2021),
    [sym_release_devchan] = ACTIONS(2021),
    [sym_master_slot] = ACTIONS(2021),
    [sym_get_pulse_time] = ACTIONS(2021),
    [sym_get_timer] = ACTIONS(2021),
    [sym_master_sn] = ACTIONS(2021),
    [sym_system_number] = ACTIONS(2021),
    [sym_button] = ACTIONS(2021),
    [sym_dv_channel] = ACTIONS(2021),
    [sym_data] = ACTIONS(2021),
    [sym_level] = ACTIONS(2021),
    [sym_timeline] = ACTIONS(2021),
    [sym_channel] = ACTIONS(2021),
    [sym_custom] = ACTIONS(2021),
    [sym_atoi] = ACTIONS(2021),
    [sym_atol] = ACTIONS(2021),
    [sym_atof] = ACTIONS(2021),
    [sym_itoa] = ACTIONS(2021),
    [sym_itohex] = ACTIONS(2021),
    [sym_hextoi] = ACTIONS(2021),
    [sym_ftoa] = ACTIONS(2021),
    [sym_find_string] = ACTIONS(2021),
    [sym_left_string] = ACTIONS(2021),
    [sym_right_string] = ACTIONS(2021),
    [sym_mid_string] = ACTIONS(2021),
    [sym_remove_string] = ACTIONS(2021),
    [sym_get_buffer_string] = ACTIONS(2021),
    [sym_get_buffer_char] = ACTIONS(2021),
    [sym_get_multi_buffer_string] = ACTIONS(2021),
    [sym_length_string] = ACTIONS(2021),
    [sym_max_length_string] = ACTIONS(2021),
    [sym_set_length_string] = ACTIONS(2021),
    [sym_compare_string] = ACTIONS(2021),
    [sym_upper_string] = ACTIONS(2021),
    [sym_lower_string] = ACTIONS(2021),
    [sym_get_last] = ACTIONS(2021),
    [sym_date_to_day] = ACTIONS(2021),
    [sym_date_to_month] = ACTIONS(2021),
    [sym_date_to_year] = ACTIONS(2021),
    [sym_time_to_hour] = ACTIONS(2021),
    [sym_time_to_minute] = ACTIONS(2021),
    [sym_time_to_second] = ACTIONS(2021),
    [sym_day_of_week] = ACTIONS(2021),
    [sym_set_pulse_time] = ACTIONS(2021),
    [sym_set_timer] = ACTIONS(2021),
    [sym_file_open] = ACTIONS(2021),
    [sym_file_close] = ACTIONS(2021),
    [sym_file_read] = ACTIONS(2021),
    [sym_file_read_line] = ACTIONS(2021),
    [sym_file_write] = ACTIONS(2021),
    [sym_file_write_line] = ACTIONS(2021),
    [sym_file_copy] = ACTIONS(2021),
    [sym_file_delete] = ACTIONS(2021),
    [sym_file_rename] = ACTIONS(2021),
    [sym_file_dir] = ACTIONS(2021),
    [sym_file_getdir] = ACTIONS(2021),
    [sym_file_setdir] = ACTIONS(2021),
    [sym_file_create_dir] = ACTIONS(2021),
    [sym_file_remove_dir] = ACTIONS(2021),
    [sym_file_seek] = ACTIONS(2021),
    [sym_length_array] = ACTIONS(2021),
    [sym_max_length_array] = ACTIONS(2021),
    [sym_set_length_array] = ACTIONS(2021),
    [sym_ip_client_open] = ACTIONS(2021),
    [sym_ip_client_close] = ACTIONS(2021),
    [sym_ip_server_open] = ACTIONS(2021),
    [sym_ip_server_close] = ACTIONS(2021),
    [sym_ip_bound_client_open] = ACTIONS(2021),
    [sym_ip_mc_server_open] = ACTIONS(2021),
    [sym_ip_set_option] = ACTIONS(2021),
    [sym_ssh_client_open] = ACTIONS(2021),
    [sym_ssh_client_close] = ACTIONS(2021),
    [sym_tls_client_open] = ACTIONS(2021),
    [sym_tls_client_close] = ACTIONS(2021),
    [sym_do_push] = ACTIONS(2021),
    [sym_do_push_timed] = ACTIONS(2021),
    [sym_do_release] = ACTIONS(2021),
    [sym_device_id] = ACTIONS(2021),
    [sym_device_info] = ACTIONS(2021),
    [sym_device_id_string] = ACTIONS(2021),
    [sym_device_standby] = ACTIONS(2021),
    [sym_device_wake] = ACTIONS(2021),
    [sym_timeline_create] = ACTIONS(2021),
    [sym_timeline_kill] = ACTIONS(2021),
    [sym_timeline_reload] = ACTIONS(2021),
    [sym_timeline_pause] = ACTIONS(2021),
    [sym_timeline_restart] = ACTIONS(2021),
    [sym_timeline_set] = ACTIONS(2021),
    [sym_timeline_get] = ACTIONS(2021),
    [sym_timeline_active] = ACTIONS(2021),
    [sym_timeline_dynamic_id] = ACTIONS(2021),
    [sym_variable_to_string] = ACTIONS(2021),
    [sym_string_to_variable] = ACTIONS(2021),
    [sym_length_variable_to_string] = ACTIONS(2021),
    [sym_type_cast] = ACTIONS(2021),
    [sym_variable_to_xml] = ACTIONS(2021),
    [sym_xml_to_variable] = ACTIONS(2021),
    [sym_length_variable_to_xml] = ACTIONS(2021),
    [sym_redirect_string] = ACTIONS(2021),
    [sym_random_number] = ACTIONS(2021),
    [sym_get_unique_id] = ACTIONS(2021),
    [sym_get_system_number] = ACTIONS(2021),
    [sym_set_system_number] = ACTIONS(2021),
    [sym_get_serial_number] = ACTIONS(2021),
    [sym_get_master_build] = ACTIONS(2021),
    [sym_set_validation_code] = ACTIONS(2021),
    [sym_reboot] = ACTIONS(2021),
    [sym_load_duet_module] = ACTIONS(2021),
    [sym_unload_duet_module] = ACTIONS(2021),
    [sym_duet_mem_size_set] = ACTIONS(2021),
    [sym_duet_mem_size_get] = ACTIONS(2021),
    [sym_dynamic_application_device] = ACTIONS(2021),
    [sym_dynamic_polled_port] = ACTIONS(2021),
    [sym_static_port_binding] = ACTIONS(2021),
    [sym_static_ip_binding] = ACTIONS(2021),
    [sym_get_dns_list] = ACTIONS(2021),
    [sym_set_dns_list] = ACTIONS(2021),
    [sym_get_ip_address] = ACTIONS(2021),
    [sym_set_ip_address] = ACTIONS(2021),
    [sym_get_url_list] = ACTIONS(2021),
    [sym_add_url_entry] = ACTIONS(2021),
    [sym_delete_url_entry] = ACTIONS(2021),
    [sym_abs_value] = ACTIONS(2021),
    [sym_max_value] = ACTIONS(2021),
    [sym_min_value] = ACTIONS(2021),
    [sym_exp_value] = ACTIONS(2021),
    [sym_log_value] = ACTIONS(2021),
    [sym_log10_value] = ACTIONS(2021),
    [sym_power_value] = ACTIONS(2021),
    [sym_sqrt_value] = ACTIONS(2021),
    [sym_raw_be] = ACTIONS(2021),
    [sym_raw_le] = ACTIONS(2021),
    [sym_internal_queue_size_set] = ACTIONS(2021),
    [sym_internal_queue_size_get] = ACTIONS(2021),
    [sym_internal_threshold_set] = ACTIONS(2021),
    [sym_internal_threshold_get] = ACTIONS(2021),
    [sym_rebuilt_event] = ACTIONS(2021),
    [sym_do_custom_event] = ACTIONS(2021),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2021),
    [sym_clkmgr_set_clk_source] = ACTIONS(2021),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2021),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2021),
    [sym_clkmgr_get_timezone] = ACTIONS(2021),
    [sym_clkmgr_set_timezone] = ACTIONS(2021),
    [sym_clkmgr_get_resync_period] = ACTIONS(2021),
    [sym_clkmgr_set_resync_period] = ACTIONS(2021),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2021),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2021),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2021),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2021),
    [sym_clkmgr_get_timeservers] = ACTIONS(2021),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2021),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2021),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2021),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2021),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2021),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2021),
    [sym_set_log_level] = ACTIONS(2021),
    [sym_get_log_level] = ACTIONS(2021),
    [sym_amx_log] = ACTIONS(2021),
    [sym_smtp_server_config_set] = ACTIONS(2021),
    [sym_smtp_server_config_get] = ACTIONS(2021),
    [sym_smtp_send] = ACTIONS(2021),
    [sym_validate_netlinx_account] = ACTIONS(2021),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2021),
    [sym_authenticate_certificate] = ACTIONS(2021),
    [sym_audit_netlinx_session_event] = ACTIONS(2021),
    [sym_audit_netlinx_generic_event] = ACTIONS(2021),
    [sym_get_available_flash_disk_space] = ACTIONS(2021),
    [sym_get_max_flash_disk_space] = ACTIONS(2021),
    [sym_data_create_feed] = ACTIONS(2021),
    [sym_data_delete_feed] = ACTIONS(2021),
    [sym_data_publish_feed] = ACTIONS(2021),
    [sym_data_get_published_feed] = ACTIONS(2021),
    [sym_data_add_record] = ACTIONS(2021),
    [sym_data_get_event_record] = ACTIONS(2021),
    [sym__wc_data_create_feed] = ACTIONS(2021),
    [sym__wc_data_add_record] = ACTIONS(2021),
    [sym__wc_data_get_event_record] = ACTIONS(2021),
    [sym_format] = ACTIONS(2021),
    [sym_astro_clock] = ACTIONS(2021),
    [sym_set_outdoor_temperature] = ACTIONS(2021),
    [sym_set_virtual_level_count] = ACTIONS(2021),
    [sym_set_virtual_channel_count] = ACTIONS(2021),
    [sym_set_virtual_port_count] = ACTIONS(2021),
    [sym__wc_encode_int] = ACTIONS(2021),
    [sym__wc_decode_int] = ACTIONS(2021),
    [sym__wc_int] = ACTIONS(2021),
    [sym__wc_to_ch_int] = ACTIONS(2021),
    [sym__ch_to_wc_int] = ACTIONS(2021),
    [sym__wc_find_string_int] = ACTIONS(2021),
    [sym__wc_left_string_int] = ACTIONS(2021),
    [sym__wc_lower_string_int] = ACTIONS(2021),
    [sym__wc_mid_string_int] = ACTIONS(2021),
    [sym__wc_remove_string_int] = ACTIONS(2021),
    [sym__wc_right_string_int] = ACTIONS(2021),
    [sym__wc_upper_string_int] = ACTIONS(2021),
    [sym__wc_compare_string_int] = ACTIONS(2021),
    [sym__wc_get_buffer_char_int] = ACTIONS(2021),
    [sym__wc_get_buffer_string_int] = ACTIONS(2021),
    [sym__wc_concat_string_int] = ACTIONS(2021),
    [sym__wc_file_open_int] = ACTIONS(2021),
    [sym__wc_file_close_int] = ACTIONS(2021),
    [sym__wc_file_read_int] = ACTIONS(2021),
    [sym__wc_file_read_line_int] = ACTIONS(2021),
    [sym__wc_file_write_int] = ACTIONS(2021),
    [sym__wc_file_write_line_int] = ACTIONS(2021),
    [aux_sym_preproc_include_token1] = ACTIONS(2021),
    [aux_sym_preproc_define_token1] = ACTIONS(2023),
    [aux_sym_preproc_warn_token1] = ACTIONS(2023),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2023),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2023),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2023),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2023),
    [aux_sym_preproc_else_token1] = ACTIONS(2023),
    [anon_sym_LBRACK] = ACTIONS(2023),
    [anon_sym_LBRACE] = ACTIONS(2023),
    [anon_sym_LPAREN] = ACTIONS(2021),
    [anon_sym_BANG] = ACTIONS(2023),
    [anon_sym_TILDE] = ACTIONS(2023),
    [anon_sym_DASH] = ACTIONS(2021),
    [anon_sym_PLUS] = ACTIONS(2021),
    [anon_sym_DASH_DASH] = ACTIONS(2023),
    [anon_sym_PLUS_PLUS] = ACTIONS(2023),
    [anon_sym_DQUOTE] = ACTIONS(2023),
    [anon_sym_SQUOTE] = ACTIONS(2023),
    [sym_number_literal] = ACTIONS(2023),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(328)] = {
    [ts_builtin_sym_end] = ACTIONS(1783),
    [sym_identifier] = ACTIONS(1781),
    [sym_program_name_keyword] = ACTIONS(1781),
    [sym_module_name_keyword] = ACTIONS(1781),
    [sym_define_device_keyword] = ACTIONS(1781),
    [sym_define_combine_keyword] = ACTIONS(1781),
    [sym_define_constant_keyword] = ACTIONS(1781),
    [sym_define_type_keyword] = ACTIONS(1781),
    [sym_define_variable_keyword] = ACTIONS(1781),
    [sym_define_system_variable_keyword] = ACTIONS(1781),
    [sym_define_connect_level_keyword] = ACTIONS(1781),
    [sym_define_latching_keyword] = ACTIONS(1781),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1781),
    [sym_define_toggling_keyword] = ACTIONS(1781),
    [sym_define_call_keyword] = ACTIONS(1781),
    [sym_define_function_keyword] = ACTIONS(1781),
    [sym_define_library_function_keyword] = ACTIONS(1781),
    [sym_define_start_keyword] = ACTIONS(1781),
    [sym_define_event_keyword] = ACTIONS(1781),
    [sym_define_program_keyword] = ACTIONS(1781),
    [sym_define_module_keyword] = ACTIONS(1781),
    [sym_char_keyword] = ACTIONS(1781),
    [sym_widechar_keyword] = ACTIONS(1781),
    [sym_integer_keyword] = ACTIONS(1781),
    [sym_sinteger_keyword] = ACTIONS(1781),
    [sym_long_keyword] = ACTIONS(1781),
    [sym_slong_keyword] = ACTIONS(1781),
    [sym_float_keyword] = ACTIONS(1781),
    [sym_double_keyword] = ACTIONS(1781),
    [sym_variant_keyword] = ACTIONS(1781),
    [sym_variantarray_keyword] = ACTIONS(1781),
    [sym_local_var_keyword] = ACTIONS(1781),
    [sym_stack_var_keyword] = ACTIONS(1781),
    [sym_constant_keyword] = ACTIONS(1781),
    [sym_volatile_keyword] = ACTIONS(1781),
    [sym_non_volatile_keyword] = ACTIONS(1781),
    [sym_persistent_keyword] = ACTIONS(1781),
    [sym_dev_keyword] = ACTIONS(1781),
    [sym_devlev_keyword] = ACTIONS(1781),
    [sym_devchan_keyword] = ACTIONS(1781),
    [sym_if_keyword] = ACTIONS(1781),
    [sym_else_keyword] = ACTIONS(1781),
    [sym_switch_keyword] = ACTIONS(1781),
    [sym_case_keyword] = ACTIONS(1781),
    [sym_default_keyword] = ACTIONS(1781),
    [sym_while_keyword] = ACTIONS(1781),
    [sym_for_keyword] = ACTIONS(1781),
    [sym_break_keyword] = ACTIONS(1781),
    [sym_continue_keyword] = ACTIONS(1781),
    [sym_return_keyword] = ACTIONS(1781),
    [sym_select_keyword] = ACTIONS(1781),
    [sym_active_keyword] = ACTIONS(1781),
    [sym_struct_keyword] = ACTIONS(1781),
    [sym_structure_keyword] = ACTIONS(1781),
    [sym_button_event_keyword] = ACTIONS(1781),
    [sym_channel_event_keyword] = ACTIONS(1781),
    [sym_level_event_keyword] = ACTIONS(1781),
    [sym_data_event_keyword] = ACTIONS(1781),
    [sym_timeline_event_keyword] = ACTIONS(1781),
    [sym_custom_event_keyword] = ACTIONS(1781),
    [sym_push_keyword] = ACTIONS(1781),
    [sym_release_keyword] = ACTIONS(1781),
    [aux_sym_on_keyword_token1] = ACTIONS(1781),
    [aux_sym_off_keyword_token1] = ACTIONS(1781),
    [sym_send_level_keyword] = ACTIONS(1781),
    [sym_send_string_keyword] = ACTIONS(1781),
    [sym_send_command_keyword] = ACTIONS(1781),
    [sym_clear_buffer_keyword] = ACTIONS(1781),
    [sym_create_buffer_keyword] = ACTIONS(1781),
    [sym_create_multi_buffer_keyword] = ACTIONS(1781),
    [sym_call_keyword] = ACTIONS(1781),
    [sym_system_call_keyword] = ACTIONS(1781),
    [sym_devchan_to_keyword] = ACTIONS(1781),
    [sym_devchan_min_to_keyword] = ACTIONS(1781),
    [sym_devchan_total_off_keyword] = ACTIONS(1781),
    [sym_devchan_pulse_keyword] = ACTIONS(1781),
    [sym_wait_keyword] = ACTIONS(1781),
    [sym_cancel_wait_keyword] = ACTIONS(1781),
    [sym_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1781),
    [sym_not] = ACTIONS(1781),
    [sym_bnot] = ACTIONS(1781),
    [sym___file__] = ACTIONS(1781),
    [sym___date__] = ACTIONS(1781),
    [sym___ldate__] = ACTIONS(1781),
    [sym___time__] = ACTIONS(1781),
    [sym___line__] = ACTIONS(1781),
    [sym___name__] = ACTIONS(1781),
    [sym___netlinx__] = ACTIONS(1781),
    [sym_netlinx_axi_version] = ACTIONS(1781),
    [sym_url_flg_tcp] = ACTIONS(1781),
    [sym_url_flg_accinfopresent] = ACTIONS(1781),
    [sym_url_flg_temp] = ACTIONS(1781),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1781),
    [sym_url_flg_stat_mask] = ACTIONS(1781),
    [sym_url_flg_stat_lookup] = ACTIONS(1781),
    [sym_url_flg_stat_connecting] = ACTIONS(1781),
    [sym_url_flg_stat_waiting] = ACTIONS(1781),
    [sym_url_flg_stat_connected] = ACTIONS(1781),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1781),
    [sym_first_virtual_device] = ACTIONS(1781),
    [sym_first_local_port] = ACTIONS(1781),
    [sym_dynamic_virtual_device] = ACTIONS(1781),
    [sym_timeline_once] = ACTIONS(1781),
    [sym_timeline_repeat] = ACTIONS(1781),
    [sym_timeline_absolute] = ACTIONS(1781),
    [sym_timeline_relative] = ACTIONS(1781),
    [sym_do_push_timed_infinite] = ACTIONS(1781),
    [sym_source_type_no_address] = ACTIONS(1781),
    [sym_source_type_neuron_id] = ACTIONS(1781),
    [sym_source_type_ip_address] = ACTIONS(1781),
    [sym_source_type_axlink] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1781),
    [sym_source_type_ip_socket_address] = ACTIONS(1781),
    [sym_source_type_rs232] = ACTIONS(1781),
    [sym_source_type_internal] = ACTIONS(1781),
    [sym_source_type_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1781),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1781),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1781),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1781),
    [sym_true] = ACTIONS(1781),
    [sym_false] = ACTIONS(1781),
    [sym_file_read_only] = ACTIONS(1781),
    [sym_file_rw_new] = ACTIONS(1781),
    [sym_file_rw_append] = ACTIONS(1781),
    [sym_ip_tcp] = ACTIONS(1781),
    [sym_ip_udp] = ACTIONS(1781),
    [sym_ip_udp_2way] = ACTIONS(1781),
    [sym_xml_encode_types] = ACTIONS(1781),
    [sym_xml_encode_char_as_list] = ACTIONS(1781),
    [sym_xml_encode_le] = ACTIONS(1781),
    [sym_xml_decode_types] = ACTIONS(1781),
    [sym_xml_decode_no_preserve] = ACTIONS(1781),
    [sym_duet_dev_type_display_device] = ACTIONS(1781),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_io_device] = ACTIONS(1781),
    [sym_duet_dev_type_relay_device] = ACTIONS(1781),
    [sym_duet_dev_type_ups] = ACTIONS(1781),
    [sym_duet_dev_type_amplifier] = ACTIONS(1781),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1781),
    [sym_duet_dev_type_camera] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1781),
    [sym_duet_dev_type_dss] = ACTIONS(1781),
    [sym_duet_dev_type_dvr] = ACTIONS(1781),
    [sym_duet_dev_type_disc_device] = ACTIONS(1781),
    [sym_duet_dev_type_document_camera] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1781),
    [sym_duet_dev_type_hvac] = ACTIONS(1781),
    [sym_duet_dev_type_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_light] = ACTIONS(1781),
    [sym_duet_dev_type_monitor] = ACTIONS(1781),
    [sym_duet_dev_type_motor] = ACTIONS(1781),
    [sym_duet_dev_type_multi_window] = ACTIONS(1781),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1781),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1781),
    [sym_duet_dev_type_receiver] = ACTIONS(1781),
    [sym_duet_dev_type_security_system] = ACTIONS(1781),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1781),
    [sym_duet_dev_type_settop_box] = ACTIONS(1781),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1781),
    [sym_duet_dev_type_switcher] = ACTIONS(1781),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_tv] = ACTIONS(1781),
    [sym_duet_dev_type_utility] = ACTIONS(1781),
    [sym_duet_dev_type_vcr] = ACTIONS(1781),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_video_processor] = ACTIONS(1781),
    [sym_duet_dev_type_video_projector] = ACTIONS(1781),
    [sym_duet_dev_type_video_wall] = ACTIONS(1781),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1781),
    [sym_duet_dev_type_weather] = ACTIONS(1781),
    [sym_duet_dev_not_polled] = ACTIONS(1781),
    [sym_duet_dev_polled] = ACTIONS(1781),
    [sym_clkmgr_mode_network] = ACTIONS(1781),
    [sym_clkmgr_mode_standalone] = ACTIONS(1781),
    [sym_normal_standby] = ACTIONS(1781),
    [sym_normal_wake] = ACTIONS(1781),
    [sym_ip_multicast_ttl_option] = ACTIONS(1781),
    [sym_ip_ttl_subnet] = ACTIONS(1781),
    [sym_ip_ttl_site] = ACTIONS(1781),
    [sym_ip_ttl_region] = ACTIONS(1781),
    [sym_ip_ttl_continent] = ACTIONS(1781),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1781),
    [sym_ip_nodelay_on] = ACTIONS(1781),
    [sym_ip_nodelay_off] = ACTIONS(1781),
    [sym_amx_error] = ACTIONS(1781),
    [sym_amx_warning] = ACTIONS(1781),
    [sym_amx_info] = ACTIONS(1781),
    [sym_amx_debug] = ACTIONS(1781),
    [sym_netlinx_login_success] = ACTIONS(1781),
    [sym_netlinx_login_fail] = ACTIONS(1781),
    [sym_netlinx_logout] = ACTIONS(1781),
    [sym_null_str] = ACTIONS(1781),
    [sym_smtp_address] = ACTIONS(1781),
    [sym_smtp_port_number] = ACTIONS(1781),
    [sym_smtp_username] = ACTIONS(1781),
    [sym_smtp_password] = ACTIONS(1781),
    [sym_smtp_require_tls] = ACTIONS(1781),
    [sym_smtp_from] = ACTIONS(1781),
    [sym_smtp_tls_true] = ACTIONS(1781),
    [sym_smtp_tls_false] = ACTIONS(1781),
    [sym_listview_on_row_select_event] = ACTIONS(1781),
    [sym_data_structure_unknown] = ACTIONS(1781),
    [sym_data_structure_datafeed] = ACTIONS(1781),
    [sym_data_structure_datarecordset] = ACTIONS(1781),
    [sym_data_structure_datarecord] = ACTIONS(1781),
    [sym_data_structure_datafield] = ACTIONS(1781),
    [sym_data_max_content_field_count] = ACTIONS(1781),
    [sym_data_max_metadata_field_count] = ACTIONS(1781),
    [sym_data_max_id_length] = ACTIONS(1781),
    [sym_data_max_name_length] = ACTIONS(1781),
    [sym_data_max_description_length] = ACTIONS(1781),
    [sym_data_max_type_length] = ACTIONS(1781),
    [sym_data_max_format_length] = ACTIONS(1781),
    [sym_data_max_source_length] = ACTIONS(1781),
    [sym_data_max_value_length] = ACTIONS(1781),
    [sym_data_max_label_length] = ACTIONS(1781),
    [sym_data_max_url_length] = ACTIONS(1781),
    [sym_max_last_login_info_length] = ACTIONS(1781),
    [sym_data_type_unknown] = ACTIONS(1781),
    [sym_data_type_string] = ACTIONS(1781),
    [sym_data_type_datetime] = ACTIONS(1781),
    [sym_data_type_date] = ACTIONS(1781),
    [sym_data_type_time] = ACTIONS(1781),
    [sym_data_type_image] = ACTIONS(1781),
    [sym_data_format_url] = ACTIONS(1781),
    [sym_data_format_phone] = ACTIONS(1781),
    [sym_data_format_email] = ACTIONS(1781),
    [sym_data_format_iso8601] = ACTIONS(1781),
    [sym_auditlog_priv] = ACTIONS(1781),
    [sym_configuration_priv] = ACTIONS(1781),
    [sym_deviceconfig_priv] = ACTIONS(1781),
    [sym_ftp_priv] = ACTIONS(1781),
    [sym_http_priv] = ACTIONS(1781),
    [sym_networkconfig_priv] = ACTIONS(1781),
    [sym_programport_priv] = ACTIONS(1781),
    [sym_remoteui_priv] = ACTIONS(1781),
    [sym_securitycontrol_priv] = ACTIONS(1781),
    [sym_softwaremanagement_priv] = ACTIONS(1781),
    [sym_terminal_priv] = ACTIONS(1781),
    [sym_tpadmin_priv] = ACTIONS(1781),
    [sym_user1_priv] = ACTIONS(1781),
    [sym_user2_priv] = ACTIONS(1781),
    [sym_user3_priv] = ACTIONS(1781),
    [sym_user4_priv] = ACTIONS(1781),
    [sym_usermanagement_priv] = ACTIONS(1781),
    [sym_valid_account] = ACTIONS(1781),
    [sym_err_name_invalid] = ACTIONS(1781),
    [sym_err_password_invalid] = ACTIONS(1781),
    [sym_err_invalid_account] = ACTIONS(1781),
    [sym_err_no_authorization] = ACTIONS(1781),
    [sym_err_invalid_parameter] = ACTIONS(1781),
    [sym_err_user_locked_out] = ACTIONS(1781),
    [sym_err_user_account_expired] = ACTIONS(1781),
    [sym_err_general_error] = ACTIONS(1781),
    [sym_err_authentication_error] = ACTIONS(1781),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1781),
    [sym_tls_validate_certificate] = ACTIONS(1781),
    [sym_tdata] = ACTIONS(1781),
    [sym_tchannel] = ACTIONS(1781),
    [sym_tlevel] = ACTIONS(1781),
    [sym_tbutton] = ACTIONS(1781),
    [sym_ttimeline] = ACTIONS(1781),
    [sym_tcustom] = ACTIONS(1781),
    [sym_url_struct] = ACTIONS(1781),
    [sym_dns_struct] = ACTIONS(1781),
    [sym_ip_address_struct] = ACTIONS(1781),
    [sym_dev_info_struct] = ACTIONS(1781),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1781),
    [sym_data_feed] = ACTIONS(1781),
    [sym_data_field] = ACTIONS(1781),
    [sym_data_record] = ACTIONS(1781),
    [sym_wc_data_feed] = ACTIONS(1781),
    [sym_wc_data_field] = ACTIONS(1781),
    [sym_wc_data_record] = ACTIONS(1781),
    [sym_last_login_info] = ACTIONS(1781),
    [sym____reserved___] = ACTIONS(1781),
    [sym_date] = ACTIONS(1781),
    [sym_day] = ACTIONS(1781),
    [sym_time] = ACTIONS(1781),
    [sym_ldate] = ACTIONS(1781),
    [sym_push_device] = ACTIONS(1781),
    [sym_push_channel] = ACTIONS(1781),
    [sym_push_devchan] = ACTIONS(1781),
    [sym_release_device] = ACTIONS(1781),
    [sym_release_channel] = ACTIONS(1781),
    [sym_release_devchan] = ACTIONS(1781),
    [sym_master_slot] = ACTIONS(1781),
    [sym_get_pulse_time] = ACTIONS(1781),
    [sym_get_timer] = ACTIONS(1781),
    [sym_master_sn] = ACTIONS(1781),
    [sym_system_number] = ACTIONS(1781),
    [sym_button] = ACTIONS(1781),
    [sym_dv_channel] = ACTIONS(1781),
    [sym_data] = ACTIONS(1781),
    [sym_level] = ACTIONS(1781),
    [sym_timeline] = ACTIONS(1781),
    [sym_channel] = ACTIONS(1781),
    [sym_custom] = ACTIONS(1781),
    [sym_atoi] = ACTIONS(1781),
    [sym_atol] = ACTIONS(1781),
    [sym_atof] = ACTIONS(1781),
    [sym_itoa] = ACTIONS(1781),
    [sym_itohex] = ACTIONS(1781),
    [sym_hextoi] = ACTIONS(1781),
    [sym_ftoa] = ACTIONS(1781),
    [sym_find_string] = ACTIONS(1781),
    [sym_left_string] = ACTIONS(1781),
    [sym_right_string] = ACTIONS(1781),
    [sym_mid_string] = ACTIONS(1781),
    [sym_remove_string] = ACTIONS(1781),
    [sym_get_buffer_string] = ACTIONS(1781),
    [sym_get_buffer_char] = ACTIONS(1781),
    [sym_get_multi_buffer_string] = ACTIONS(1781),
    [sym_length_string] = ACTIONS(1781),
    [sym_max_length_string] = ACTIONS(1781),
    [sym_set_length_string] = ACTIONS(1781),
    [sym_compare_string] = ACTIONS(1781),
    [sym_upper_string] = ACTIONS(1781),
    [sym_lower_string] = ACTIONS(1781),
    [sym_get_last] = ACTIONS(1781),
    [sym_date_to_day] = ACTIONS(1781),
    [sym_date_to_month] = ACTIONS(1781),
    [sym_date_to_year] = ACTIONS(1781),
    [sym_time_to_hour] = ACTIONS(1781),
    [sym_time_to_minute] = ACTIONS(1781),
    [sym_time_to_second] = ACTIONS(1781),
    [sym_day_of_week] = ACTIONS(1781),
    [sym_set_pulse_time] = ACTIONS(1781),
    [sym_set_timer] = ACTIONS(1781),
    [sym_file_open] = ACTIONS(1781),
    [sym_file_close] = ACTIONS(1781),
    [sym_file_read] = ACTIONS(1781),
    [sym_file_read_line] = ACTIONS(1781),
    [sym_file_write] = ACTIONS(1781),
    [sym_file_write_line] = ACTIONS(1781),
    [sym_file_copy] = ACTIONS(1781),
    [sym_file_delete] = ACTIONS(1781),
    [sym_file_rename] = ACTIONS(1781),
    [sym_file_dir] = ACTIONS(1781),
    [sym_file_getdir] = ACTIONS(1781),
    [sym_file_setdir] = ACTIONS(1781),
    [sym_file_create_dir] = ACTIONS(1781),
    [sym_file_remove_dir] = ACTIONS(1781),
    [sym_file_seek] = ACTIONS(1781),
    [sym_length_array] = ACTIONS(1781),
    [sym_max_length_array] = ACTIONS(1781),
    [sym_set_length_array] = ACTIONS(1781),
    [sym_ip_client_open] = ACTIONS(1781),
    [sym_ip_client_close] = ACTIONS(1781),
    [sym_ip_server_open] = ACTIONS(1781),
    [sym_ip_server_close] = ACTIONS(1781),
    [sym_ip_bound_client_open] = ACTIONS(1781),
    [sym_ip_mc_server_open] = ACTIONS(1781),
    [sym_ip_set_option] = ACTIONS(1781),
    [sym_ssh_client_open] = ACTIONS(1781),
    [sym_ssh_client_close] = ACTIONS(1781),
    [sym_tls_client_open] = ACTIONS(1781),
    [sym_tls_client_close] = ACTIONS(1781),
    [sym_do_push] = ACTIONS(1781),
    [sym_do_push_timed] = ACTIONS(1781),
    [sym_do_release] = ACTIONS(1781),
    [sym_device_id] = ACTIONS(1781),
    [sym_device_info] = ACTIONS(1781),
    [sym_device_id_string] = ACTIONS(1781),
    [sym_device_standby] = ACTIONS(1781),
    [sym_device_wake] = ACTIONS(1781),
    [sym_timeline_create] = ACTIONS(1781),
    [sym_timeline_kill] = ACTIONS(1781),
    [sym_timeline_reload] = ACTIONS(1781),
    [sym_timeline_pause] = ACTIONS(1781),
    [sym_timeline_restart] = ACTIONS(1781),
    [sym_timeline_set] = ACTIONS(1781),
    [sym_timeline_get] = ACTIONS(1781),
    [sym_timeline_active] = ACTIONS(1781),
    [sym_timeline_dynamic_id] = ACTIONS(1781),
    [sym_variable_to_string] = ACTIONS(1781),
    [sym_string_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_string] = ACTIONS(1781),
    [sym_type_cast] = ACTIONS(1781),
    [sym_variable_to_xml] = ACTIONS(1781),
    [sym_xml_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_xml] = ACTIONS(1781),
    [sym_redirect_string] = ACTIONS(1781),
    [sym_random_number] = ACTIONS(1781),
    [sym_get_unique_id] = ACTIONS(1781),
    [sym_get_system_number] = ACTIONS(1781),
    [sym_set_system_number] = ACTIONS(1781),
    [sym_get_serial_number] = ACTIONS(1781),
    [sym_get_master_build] = ACTIONS(1781),
    [sym_set_validation_code] = ACTIONS(1781),
    [sym_reboot] = ACTIONS(1781),
    [sym_load_duet_module] = ACTIONS(1781),
    [sym_unload_duet_module] = ACTIONS(1781),
    [sym_duet_mem_size_set] = ACTIONS(1781),
    [sym_duet_mem_size_get] = ACTIONS(1781),
    [sym_dynamic_application_device] = ACTIONS(1781),
    [sym_dynamic_polled_port] = ACTIONS(1781),
    [sym_static_port_binding] = ACTIONS(1781),
    [sym_static_ip_binding] = ACTIONS(1781),
    [sym_get_dns_list] = ACTIONS(1781),
    [sym_set_dns_list] = ACTIONS(1781),
    [sym_get_ip_address] = ACTIONS(1781),
    [sym_set_ip_address] = ACTIONS(1781),
    [sym_get_url_list] = ACTIONS(1781),
    [sym_add_url_entry] = ACTIONS(1781),
    [sym_delete_url_entry] = ACTIONS(1781),
    [sym_abs_value] = ACTIONS(1781),
    [sym_max_value] = ACTIONS(1781),
    [sym_min_value] = ACTIONS(1781),
    [sym_exp_value] = ACTIONS(1781),
    [sym_log_value] = ACTIONS(1781),
    [sym_log10_value] = ACTIONS(1781),
    [sym_power_value] = ACTIONS(1781),
    [sym_sqrt_value] = ACTIONS(1781),
    [sym_raw_be] = ACTIONS(1781),
    [sym_raw_le] = ACTIONS(1781),
    [sym_internal_queue_size_set] = ACTIONS(1781),
    [sym_internal_queue_size_get] = ACTIONS(1781),
    [sym_internal_threshold_set] = ACTIONS(1781),
    [sym_internal_threshold_get] = ACTIONS(1781),
    [sym_rebuilt_event] = ACTIONS(1781),
    [sym_do_custom_event] = ACTIONS(1781),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1781),
    [sym_clkmgr_set_clk_source] = ACTIONS(1781),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1781),
    [sym_clkmgr_get_timezone] = ACTIONS(1781),
    [sym_clkmgr_set_timezone] = ACTIONS(1781),
    [sym_clkmgr_get_resync_period] = ACTIONS(1781),
    [sym_clkmgr_set_resync_period] = ACTIONS(1781),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_timeservers] = ACTIONS(1781),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_set_log_level] = ACTIONS(1781),
    [sym_get_log_level] = ACTIONS(1781),
    [sym_amx_log] = ACTIONS(1781),
    [sym_smtp_server_config_set] = ACTIONS(1781),
    [sym_smtp_server_config_get] = ACTIONS(1781),
    [sym_smtp_send] = ACTIONS(1781),
    [sym_validate_netlinx_account] = ACTIONS(1781),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1781),
    [sym_authenticate_certificate] = ACTIONS(1781),
    [sym_audit_netlinx_session_event] = ACTIONS(1781),
    [sym_audit_netlinx_generic_event] = ACTIONS(1781),
    [sym_get_available_flash_disk_space] = ACTIONS(1781),
    [sym_get_max_flash_disk_space] = ACTIONS(1781),
    [sym_data_create_feed] = ACTIONS(1781),
    [sym_data_delete_feed] = ACTIONS(1781),
    [sym_data_publish_feed] = ACTIONS(1781),
    [sym_data_get_published_feed] = ACTIONS(1781),
    [sym_data_add_record] = ACTIONS(1781),
    [sym_data_get_event_record] = ACTIONS(1781),
    [sym__wc_data_create_feed] = ACTIONS(1781),
    [sym__wc_data_add_record] = ACTIONS(1781),
    [sym__wc_data_get_event_record] = ACTIONS(1781),
    [sym_format] = ACTIONS(1781),
    [sym_astro_clock] = ACTIONS(1781),
    [sym_set_outdoor_temperature] = ACTIONS(1781),
    [sym_set_virtual_level_count] = ACTIONS(1781),
    [sym_set_virtual_channel_count] = ACTIONS(1781),
    [sym_set_virtual_port_count] = ACTIONS(1781),
    [sym__wc_encode_int] = ACTIONS(1781),
    [sym__wc_decode_int] = ACTIONS(1781),
    [sym__wc_int] = ACTIONS(1781),
    [sym__wc_to_ch_int] = ACTIONS(1781),
    [sym__ch_to_wc_int] = ACTIONS(1781),
    [sym__wc_find_string_int] = ACTIONS(1781),
    [sym__wc_left_string_int] = ACTIONS(1781),
    [sym__wc_lower_string_int] = ACTIONS(1781),
    [sym__wc_mid_string_int] = ACTIONS(1781),
    [sym__wc_remove_string_int] = ACTIONS(1781),
    [sym__wc_right_string_int] = ACTIONS(1781),
    [sym__wc_upper_string_int] = ACTIONS(1781),
    [sym__wc_compare_string_int] = ACTIONS(1781),
    [sym__wc_get_buffer_char_int] = ACTIONS(1781),
    [sym__wc_get_buffer_string_int] = ACTIONS(1781),
    [sym__wc_concat_string_int] = ACTIONS(1781),
    [sym__wc_file_open_int] = ACTIONS(1781),
    [sym__wc_file_close_int] = ACTIONS(1781),
    [sym__wc_file_read_int] = ACTIONS(1781),
    [sym__wc_file_read_line_int] = ACTIONS(1781),
    [sym__wc_file_write_int] = ACTIONS(1781),
    [sym__wc_file_write_line_int] = ACTIONS(1781),
    [aux_sym_preproc_include_token1] = ACTIONS(1781),
    [aux_sym_preproc_define_token1] = ACTIONS(1783),
    [aux_sym_preproc_warn_token1] = ACTIONS(1783),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1783),
    [anon_sym_LBRACK] = ACTIONS(1783),
    [anon_sym_LBRACE] = ACTIONS(1783),
    [anon_sym_LPAREN] = ACTIONS(1781),
    [anon_sym_BANG] = ACTIONS(1783),
    [anon_sym_TILDE] = ACTIONS(1783),
    [anon_sym_DASH] = ACTIONS(1781),
    [anon_sym_PLUS] = ACTIONS(1781),
    [anon_sym_DASH_DASH] = ACTIONS(1783),
    [anon_sym_PLUS_PLUS] = ACTIONS(1783),
    [anon_sym_DQUOTE] = ACTIONS(1783),
    [anon_sym_SQUOTE] = ACTIONS(1783),
    [sym_number_literal] = ACTIONS(1783),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(329)] = {
    [sym_identifier] = ACTIONS(2025),
    [sym_program_name_keyword] = ACTIONS(2025),
    [sym_module_name_keyword] = ACTIONS(2025),
    [sym_define_device_keyword] = ACTIONS(2025),
    [sym_define_combine_keyword] = ACTIONS(2025),
    [sym_define_constant_keyword] = ACTIONS(2025),
    [sym_define_type_keyword] = ACTIONS(2025),
    [sym_define_variable_keyword] = ACTIONS(2025),
    [sym_define_system_variable_keyword] = ACTIONS(2025),
    [sym_define_connect_level_keyword] = ACTIONS(2025),
    [sym_define_latching_keyword] = ACTIONS(2025),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2025),
    [sym_define_toggling_keyword] = ACTIONS(2025),
    [sym_define_call_keyword] = ACTIONS(2025),
    [sym_define_function_keyword] = ACTIONS(2025),
    [sym_define_library_function_keyword] = ACTIONS(2025),
    [sym_define_start_keyword] = ACTIONS(2025),
    [sym_define_event_keyword] = ACTIONS(2025),
    [sym_define_program_keyword] = ACTIONS(2025),
    [sym_define_module_keyword] = ACTIONS(2025),
    [sym_char_keyword] = ACTIONS(2025),
    [sym_widechar_keyword] = ACTIONS(2025),
    [sym_integer_keyword] = ACTIONS(2025),
    [sym_sinteger_keyword] = ACTIONS(2025),
    [sym_long_keyword] = ACTIONS(2025),
    [sym_slong_keyword] = ACTIONS(2025),
    [sym_float_keyword] = ACTIONS(2025),
    [sym_double_keyword] = ACTIONS(2025),
    [sym_variant_keyword] = ACTIONS(2025),
    [sym_variantarray_keyword] = ACTIONS(2025),
    [sym_local_var_keyword] = ACTIONS(2025),
    [sym_stack_var_keyword] = ACTIONS(2025),
    [sym_constant_keyword] = ACTIONS(2025),
    [sym_volatile_keyword] = ACTIONS(2025),
    [sym_non_volatile_keyword] = ACTIONS(2025),
    [sym_persistent_keyword] = ACTIONS(2025),
    [sym_dev_keyword] = ACTIONS(2025),
    [sym_devlev_keyword] = ACTIONS(2025),
    [sym_devchan_keyword] = ACTIONS(2025),
    [sym_if_keyword] = ACTIONS(2025),
    [sym_switch_keyword] = ACTIONS(2025),
    [sym_case_keyword] = ACTIONS(2025),
    [sym_default_keyword] = ACTIONS(2025),
    [sym_while_keyword] = ACTIONS(2025),
    [sym_for_keyword] = ACTIONS(2025),
    [sym_break_keyword] = ACTIONS(2025),
    [sym_continue_keyword] = ACTIONS(2025),
    [sym_return_keyword] = ACTIONS(2025),
    [sym_select_keyword] = ACTIONS(2025),
    [sym_active_keyword] = ACTIONS(2025),
    [sym_struct_keyword] = ACTIONS(2025),
    [sym_structure_keyword] = ACTIONS(2025),
    [sym_button_event_keyword] = ACTIONS(2025),
    [sym_channel_event_keyword] = ACTIONS(2025),
    [sym_level_event_keyword] = ACTIONS(2025),
    [sym_data_event_keyword] = ACTIONS(2025),
    [sym_timeline_event_keyword] = ACTIONS(2025),
    [sym_custom_event_keyword] = ACTIONS(2025),
    [sym_push_keyword] = ACTIONS(2025),
    [sym_release_keyword] = ACTIONS(2025),
    [aux_sym_on_keyword_token1] = ACTIONS(2025),
    [aux_sym_off_keyword_token1] = ACTIONS(2025),
    [sym_send_level_keyword] = ACTIONS(2025),
    [sym_send_string_keyword] = ACTIONS(2025),
    [sym_send_command_keyword] = ACTIONS(2025),
    [sym_clear_buffer_keyword] = ACTIONS(2025),
    [sym_create_buffer_keyword] = ACTIONS(2025),
    [sym_create_multi_buffer_keyword] = ACTIONS(2025),
    [sym_call_keyword] = ACTIONS(2025),
    [sym_system_call_keyword] = ACTIONS(2025),
    [sym_devchan_to_keyword] = ACTIONS(2025),
    [sym_devchan_min_to_keyword] = ACTIONS(2025),
    [sym_devchan_total_off_keyword] = ACTIONS(2025),
    [sym_devchan_pulse_keyword] = ACTIONS(2025),
    [sym_wait_keyword] = ACTIONS(2025),
    [sym_cancel_wait_keyword] = ACTIONS(2025),
    [sym_wait_until_keyword] = ACTIONS(2025),
    [sym_cancel_wait_until_keyword] = ACTIONS(2025),
    [sym_cancel_all_wait_keyword] = ACTIONS(2025),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2025),
    [sym_not] = ACTIONS(2025),
    [sym_bnot] = ACTIONS(2025),
    [sym___file__] = ACTIONS(2025),
    [sym___date__] = ACTIONS(2025),
    [sym___ldate__] = ACTIONS(2025),
    [sym___time__] = ACTIONS(2025),
    [sym___line__] = ACTIONS(2025),
    [sym___name__] = ACTIONS(2025),
    [sym___netlinx__] = ACTIONS(2025),
    [sym_netlinx_axi_version] = ACTIONS(2025),
    [sym_url_flg_tcp] = ACTIONS(2025),
    [sym_url_flg_accinfopresent] = ACTIONS(2025),
    [sym_url_flg_temp] = ACTIONS(2025),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2025),
    [sym_url_flg_stat_mask] = ACTIONS(2025),
    [sym_url_flg_stat_lookup] = ACTIONS(2025),
    [sym_url_flg_stat_connecting] = ACTIONS(2025),
    [sym_url_flg_stat_waiting] = ACTIONS(2025),
    [sym_url_flg_stat_connected] = ACTIONS(2025),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2025),
    [sym_first_virtual_device] = ACTIONS(2025),
    [sym_first_local_port] = ACTIONS(2025),
    [sym_dynamic_virtual_device] = ACTIONS(2025),
    [sym_timeline_once] = ACTIONS(2025),
    [sym_timeline_repeat] = ACTIONS(2025),
    [sym_timeline_absolute] = ACTIONS(2025),
    [sym_timeline_relative] = ACTIONS(2025),
    [sym_do_push_timed_infinite] = ACTIONS(2025),
    [sym_source_type_no_address] = ACTIONS(2025),
    [sym_source_type_neuron_id] = ACTIONS(2025),
    [sym_source_type_ip_address] = ACTIONS(2025),
    [sym_source_type_axlink] = ACTIONS(2025),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2025),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2025),
    [sym_source_type_ip_socket_address] = ACTIONS(2025),
    [sym_source_type_rs232] = ACTIONS(2025),
    [sym_source_type_internal] = ACTIONS(2025),
    [sym_source_type_mac_address] = ACTIONS(2025),
    [sym_source_type_ipv4_port] = ACTIONS(2025),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2025),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2025),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2025),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2025),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2025),
    [sym_true] = ACTIONS(2025),
    [sym_false] = ACTIONS(2025),
    [sym_file_read_only] = ACTIONS(2025),
    [sym_file_rw_new] = ACTIONS(2025),
    [sym_file_rw_append] = ACTIONS(2025),
    [sym_ip_tcp] = ACTIONS(2025),
    [sym_ip_udp] = ACTIONS(2025),
    [sym_ip_udp_2way] = ACTIONS(2025),
    [sym_xml_encode_types] = ACTIONS(2025),
    [sym_xml_encode_char_as_list] = ACTIONS(2025),
    [sym_xml_encode_le] = ACTIONS(2025),
    [sym_xml_decode_types] = ACTIONS(2025),
    [sym_xml_decode_no_preserve] = ACTIONS(2025),
    [sym_duet_dev_type_display_device] = ACTIONS(2025),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2025),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2025),
    [sym_duet_dev_type_io_device] = ACTIONS(2025),
    [sym_duet_dev_type_relay_device] = ACTIONS(2025),
    [sym_duet_dev_type_ups] = ACTIONS(2025),
    [sym_duet_dev_type_amplifier] = ACTIONS(2025),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2025),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2025),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2025),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2025),
    [sym_duet_dev_type_camera] = ACTIONS(2025),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2025),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2025),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2025),
    [sym_duet_dev_type_dss] = ACTIONS(2025),
    [sym_duet_dev_type_dvr] = ACTIONS(2025),
    [sym_duet_dev_type_disc_device] = ACTIONS(2025),
    [sym_duet_dev_type_document_camera] = ACTIONS(2025),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2025),
    [sym_duet_dev_type_hvac] = ACTIONS(2025),
    [sym_duet_dev_type_keypad] = ACTIONS(2025),
    [sym_duet_dev_type_light] = ACTIONS(2025),
    [sym_duet_dev_type_monitor] = ACTIONS(2025),
    [sym_duet_dev_type_motor] = ACTIONS(2025),
    [sym_duet_dev_type_multi_window] = ACTIONS(2025),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2025),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2025),
    [sym_duet_dev_type_receiver] = ACTIONS(2025),
    [sym_duet_dev_type_security_system] = ACTIONS(2025),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2025),
    [sym_duet_dev_type_settop_box] = ACTIONS(2025),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2025),
    [sym_duet_dev_type_switcher] = ACTIONS(2025),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2025),
    [sym_duet_dev_type_tv] = ACTIONS(2025),
    [sym_duet_dev_type_utility] = ACTIONS(2025),
    [sym_duet_dev_type_vcr] = ACTIONS(2025),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2025),
    [sym_duet_dev_type_video_processor] = ACTIONS(2025),
    [sym_duet_dev_type_video_projector] = ACTIONS(2025),
    [sym_duet_dev_type_video_wall] = ACTIONS(2025),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2025),
    [sym_duet_dev_type_weather] = ACTIONS(2025),
    [sym_duet_dev_not_polled] = ACTIONS(2025),
    [sym_duet_dev_polled] = ACTIONS(2025),
    [sym_clkmgr_mode_network] = ACTIONS(2025),
    [sym_clkmgr_mode_standalone] = ACTIONS(2025),
    [sym_normal_standby] = ACTIONS(2025),
    [sym_normal_wake] = ACTIONS(2025),
    [sym_ip_multicast_ttl_option] = ACTIONS(2025),
    [sym_ip_ttl_subnet] = ACTIONS(2025),
    [sym_ip_ttl_site] = ACTIONS(2025),
    [sym_ip_ttl_region] = ACTIONS(2025),
    [sym_ip_ttl_continent] = ACTIONS(2025),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2025),
    [sym_ip_nodelay_on] = ACTIONS(2025),
    [sym_ip_nodelay_off] = ACTIONS(2025),
    [sym_amx_error] = ACTIONS(2025),
    [sym_amx_warning] = ACTIONS(2025),
    [sym_amx_info] = ACTIONS(2025),
    [sym_amx_debug] = ACTIONS(2025),
    [sym_netlinx_login_success] = ACTIONS(2025),
    [sym_netlinx_login_fail] = ACTIONS(2025),
    [sym_netlinx_logout] = ACTIONS(2025),
    [sym_null_str] = ACTIONS(2025),
    [sym_smtp_address] = ACTIONS(2025),
    [sym_smtp_port_number] = ACTIONS(2025),
    [sym_smtp_username] = ACTIONS(2025),
    [sym_smtp_password] = ACTIONS(2025),
    [sym_smtp_require_tls] = ACTIONS(2025),
    [sym_smtp_from] = ACTIONS(2025),
    [sym_smtp_tls_true] = ACTIONS(2025),
    [sym_smtp_tls_false] = ACTIONS(2025),
    [sym_listview_on_row_select_event] = ACTIONS(2025),
    [sym_data_structure_unknown] = ACTIONS(2025),
    [sym_data_structure_datafeed] = ACTIONS(2025),
    [sym_data_structure_datarecordset] = ACTIONS(2025),
    [sym_data_structure_datarecord] = ACTIONS(2025),
    [sym_data_structure_datafield] = ACTIONS(2025),
    [sym_data_max_content_field_count] = ACTIONS(2025),
    [sym_data_max_metadata_field_count] = ACTIONS(2025),
    [sym_data_max_id_length] = ACTIONS(2025),
    [sym_data_max_name_length] = ACTIONS(2025),
    [sym_data_max_description_length] = ACTIONS(2025),
    [sym_data_max_type_length] = ACTIONS(2025),
    [sym_data_max_format_length] = ACTIONS(2025),
    [sym_data_max_source_length] = ACTIONS(2025),
    [sym_data_max_value_length] = ACTIONS(2025),
    [sym_data_max_label_length] = ACTIONS(2025),
    [sym_data_max_url_length] = ACTIONS(2025),
    [sym_max_last_login_info_length] = ACTIONS(2025),
    [sym_data_type_unknown] = ACTIONS(2025),
    [sym_data_type_string] = ACTIONS(2025),
    [sym_data_type_datetime] = ACTIONS(2025),
    [sym_data_type_date] = ACTIONS(2025),
    [sym_data_type_time] = ACTIONS(2025),
    [sym_data_type_image] = ACTIONS(2025),
    [sym_data_format_url] = ACTIONS(2025),
    [sym_data_format_phone] = ACTIONS(2025),
    [sym_data_format_email] = ACTIONS(2025),
    [sym_data_format_iso8601] = ACTIONS(2025),
    [sym_auditlog_priv] = ACTIONS(2025),
    [sym_configuration_priv] = ACTIONS(2025),
    [sym_deviceconfig_priv] = ACTIONS(2025),
    [sym_ftp_priv] = ACTIONS(2025),
    [sym_http_priv] = ACTIONS(2025),
    [sym_networkconfig_priv] = ACTIONS(2025),
    [sym_programport_priv] = ACTIONS(2025),
    [sym_remoteui_priv] = ACTIONS(2025),
    [sym_securitycontrol_priv] = ACTIONS(2025),
    [sym_softwaremanagement_priv] = ACTIONS(2025),
    [sym_terminal_priv] = ACTIONS(2025),
    [sym_tpadmin_priv] = ACTIONS(2025),
    [sym_user1_priv] = ACTIONS(2025),
    [sym_user2_priv] = ACTIONS(2025),
    [sym_user3_priv] = ACTIONS(2025),
    [sym_user4_priv] = ACTIONS(2025),
    [sym_usermanagement_priv] = ACTIONS(2025),
    [sym_valid_account] = ACTIONS(2025),
    [sym_err_name_invalid] = ACTIONS(2025),
    [sym_err_password_invalid] = ACTIONS(2025),
    [sym_err_invalid_account] = ACTIONS(2025),
    [sym_err_no_authorization] = ACTIONS(2025),
    [sym_err_invalid_parameter] = ACTIONS(2025),
    [sym_err_user_locked_out] = ACTIONS(2025),
    [sym_err_user_account_expired] = ACTIONS(2025),
    [sym_err_general_error] = ACTIONS(2025),
    [sym_err_authentication_error] = ACTIONS(2025),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2025),
    [sym_tls_validate_certificate] = ACTIONS(2025),
    [sym_tdata] = ACTIONS(2025),
    [sym_tchannel] = ACTIONS(2025),
    [sym_tlevel] = ACTIONS(2025),
    [sym_tbutton] = ACTIONS(2025),
    [sym_ttimeline] = ACTIONS(2025),
    [sym_tcustom] = ACTIONS(2025),
    [sym_url_struct] = ACTIONS(2025),
    [sym_dns_struct] = ACTIONS(2025),
    [sym_ip_address_struct] = ACTIONS(2025),
    [sym_dev_info_struct] = ACTIONS(2025),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2025),
    [sym_data_feed] = ACTIONS(2025),
    [sym_data_field] = ACTIONS(2025),
    [sym_data_record] = ACTIONS(2025),
    [sym_wc_data_feed] = ACTIONS(2025),
    [sym_wc_data_field] = ACTIONS(2025),
    [sym_wc_data_record] = ACTIONS(2025),
    [sym_last_login_info] = ACTIONS(2025),
    [sym____reserved___] = ACTIONS(2025),
    [sym_date] = ACTIONS(2025),
    [sym_day] = ACTIONS(2025),
    [sym_time] = ACTIONS(2025),
    [sym_ldate] = ACTIONS(2025),
    [sym_push_device] = ACTIONS(2025),
    [sym_push_channel] = ACTIONS(2025),
    [sym_push_devchan] = ACTIONS(2025),
    [sym_release_device] = ACTIONS(2025),
    [sym_release_channel] = ACTIONS(2025),
    [sym_release_devchan] = ACTIONS(2025),
    [sym_master_slot] = ACTIONS(2025),
    [sym_get_pulse_time] = ACTIONS(2025),
    [sym_get_timer] = ACTIONS(2025),
    [sym_master_sn] = ACTIONS(2025),
    [sym_system_number] = ACTIONS(2025),
    [sym_button] = ACTIONS(2025),
    [sym_dv_channel] = ACTIONS(2025),
    [sym_data] = ACTIONS(2025),
    [sym_level] = ACTIONS(2025),
    [sym_timeline] = ACTIONS(2025),
    [sym_channel] = ACTIONS(2025),
    [sym_custom] = ACTIONS(2025),
    [sym_atoi] = ACTIONS(2025),
    [sym_atol] = ACTIONS(2025),
    [sym_atof] = ACTIONS(2025),
    [sym_itoa] = ACTIONS(2025),
    [sym_itohex] = ACTIONS(2025),
    [sym_hextoi] = ACTIONS(2025),
    [sym_ftoa] = ACTIONS(2025),
    [sym_find_string] = ACTIONS(2025),
    [sym_left_string] = ACTIONS(2025),
    [sym_right_string] = ACTIONS(2025),
    [sym_mid_string] = ACTIONS(2025),
    [sym_remove_string] = ACTIONS(2025),
    [sym_get_buffer_string] = ACTIONS(2025),
    [sym_get_buffer_char] = ACTIONS(2025),
    [sym_get_multi_buffer_string] = ACTIONS(2025),
    [sym_length_string] = ACTIONS(2025),
    [sym_max_length_string] = ACTIONS(2025),
    [sym_set_length_string] = ACTIONS(2025),
    [sym_compare_string] = ACTIONS(2025),
    [sym_upper_string] = ACTIONS(2025),
    [sym_lower_string] = ACTIONS(2025),
    [sym_get_last] = ACTIONS(2025),
    [sym_date_to_day] = ACTIONS(2025),
    [sym_date_to_month] = ACTIONS(2025),
    [sym_date_to_year] = ACTIONS(2025),
    [sym_time_to_hour] = ACTIONS(2025),
    [sym_time_to_minute] = ACTIONS(2025),
    [sym_time_to_second] = ACTIONS(2025),
    [sym_day_of_week] = ACTIONS(2025),
    [sym_set_pulse_time] = ACTIONS(2025),
    [sym_set_timer] = ACTIONS(2025),
    [sym_file_open] = ACTIONS(2025),
    [sym_file_close] = ACTIONS(2025),
    [sym_file_read] = ACTIONS(2025),
    [sym_file_read_line] = ACTIONS(2025),
    [sym_file_write] = ACTIONS(2025),
    [sym_file_write_line] = ACTIONS(2025),
    [sym_file_copy] = ACTIONS(2025),
    [sym_file_delete] = ACTIONS(2025),
    [sym_file_rename] = ACTIONS(2025),
    [sym_file_dir] = ACTIONS(2025),
    [sym_file_getdir] = ACTIONS(2025),
    [sym_file_setdir] = ACTIONS(2025),
    [sym_file_create_dir] = ACTIONS(2025),
    [sym_file_remove_dir] = ACTIONS(2025),
    [sym_file_seek] = ACTIONS(2025),
    [sym_length_array] = ACTIONS(2025),
    [sym_max_length_array] = ACTIONS(2025),
    [sym_set_length_array] = ACTIONS(2025),
    [sym_ip_client_open] = ACTIONS(2025),
    [sym_ip_client_close] = ACTIONS(2025),
    [sym_ip_server_open] = ACTIONS(2025),
    [sym_ip_server_close] = ACTIONS(2025),
    [sym_ip_bound_client_open] = ACTIONS(2025),
    [sym_ip_mc_server_open] = ACTIONS(2025),
    [sym_ip_set_option] = ACTIONS(2025),
    [sym_ssh_client_open] = ACTIONS(2025),
    [sym_ssh_client_close] = ACTIONS(2025),
    [sym_tls_client_open] = ACTIONS(2025),
    [sym_tls_client_close] = ACTIONS(2025),
    [sym_do_push] = ACTIONS(2025),
    [sym_do_push_timed] = ACTIONS(2025),
    [sym_do_release] = ACTIONS(2025),
    [sym_device_id] = ACTIONS(2025),
    [sym_device_info] = ACTIONS(2025),
    [sym_device_id_string] = ACTIONS(2025),
    [sym_device_standby] = ACTIONS(2025),
    [sym_device_wake] = ACTIONS(2025),
    [sym_timeline_create] = ACTIONS(2025),
    [sym_timeline_kill] = ACTIONS(2025),
    [sym_timeline_reload] = ACTIONS(2025),
    [sym_timeline_pause] = ACTIONS(2025),
    [sym_timeline_restart] = ACTIONS(2025),
    [sym_timeline_set] = ACTIONS(2025),
    [sym_timeline_get] = ACTIONS(2025),
    [sym_timeline_active] = ACTIONS(2025),
    [sym_timeline_dynamic_id] = ACTIONS(2025),
    [sym_variable_to_string] = ACTIONS(2025),
    [sym_string_to_variable] = ACTIONS(2025),
    [sym_length_variable_to_string] = ACTIONS(2025),
    [sym_type_cast] = ACTIONS(2025),
    [sym_variable_to_xml] = ACTIONS(2025),
    [sym_xml_to_variable] = ACTIONS(2025),
    [sym_length_variable_to_xml] = ACTIONS(2025),
    [sym_redirect_string] = ACTIONS(2025),
    [sym_random_number] = ACTIONS(2025),
    [sym_get_unique_id] = ACTIONS(2025),
    [sym_get_system_number] = ACTIONS(2025),
    [sym_set_system_number] = ACTIONS(2025),
    [sym_get_serial_number] = ACTIONS(2025),
    [sym_get_master_build] = ACTIONS(2025),
    [sym_set_validation_code] = ACTIONS(2025),
    [sym_reboot] = ACTIONS(2025),
    [sym_load_duet_module] = ACTIONS(2025),
    [sym_unload_duet_module] = ACTIONS(2025),
    [sym_duet_mem_size_set] = ACTIONS(2025),
    [sym_duet_mem_size_get] = ACTIONS(2025),
    [sym_dynamic_application_device] = ACTIONS(2025),
    [sym_dynamic_polled_port] = ACTIONS(2025),
    [sym_static_port_binding] = ACTIONS(2025),
    [sym_static_ip_binding] = ACTIONS(2025),
    [sym_get_dns_list] = ACTIONS(2025),
    [sym_set_dns_list] = ACTIONS(2025),
    [sym_get_ip_address] = ACTIONS(2025),
    [sym_set_ip_address] = ACTIONS(2025),
    [sym_get_url_list] = ACTIONS(2025),
    [sym_add_url_entry] = ACTIONS(2025),
    [sym_delete_url_entry] = ACTIONS(2025),
    [sym_abs_value] = ACTIONS(2025),
    [sym_max_value] = ACTIONS(2025),
    [sym_min_value] = ACTIONS(2025),
    [sym_exp_value] = ACTIONS(2025),
    [sym_log_value] = ACTIONS(2025),
    [sym_log10_value] = ACTIONS(2025),
    [sym_power_value] = ACTIONS(2025),
    [sym_sqrt_value] = ACTIONS(2025),
    [sym_raw_be] = ACTIONS(2025),
    [sym_raw_le] = ACTIONS(2025),
    [sym_internal_queue_size_set] = ACTIONS(2025),
    [sym_internal_queue_size_get] = ACTIONS(2025),
    [sym_internal_threshold_set] = ACTIONS(2025),
    [sym_internal_threshold_get] = ACTIONS(2025),
    [sym_rebuilt_event] = ACTIONS(2025),
    [sym_do_custom_event] = ACTIONS(2025),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2025),
    [sym_clkmgr_set_clk_source] = ACTIONS(2025),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2025),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2025),
    [sym_clkmgr_get_timezone] = ACTIONS(2025),
    [sym_clkmgr_set_timezone] = ACTIONS(2025),
    [sym_clkmgr_get_resync_period] = ACTIONS(2025),
    [sym_clkmgr_set_resync_period] = ACTIONS(2025),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2025),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2025),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2025),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2025),
    [sym_clkmgr_get_timeservers] = ACTIONS(2025),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2025),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2025),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2025),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2025),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2025),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2025),
    [sym_set_log_level] = ACTIONS(2025),
    [sym_get_log_level] = ACTIONS(2025),
    [sym_amx_log] = ACTIONS(2025),
    [sym_smtp_server_config_set] = ACTIONS(2025),
    [sym_smtp_server_config_get] = ACTIONS(2025),
    [sym_smtp_send] = ACTIONS(2025),
    [sym_validate_netlinx_account] = ACTIONS(2025),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2025),
    [sym_authenticate_certificate] = ACTIONS(2025),
    [sym_audit_netlinx_session_event] = ACTIONS(2025),
    [sym_audit_netlinx_generic_event] = ACTIONS(2025),
    [sym_get_available_flash_disk_space] = ACTIONS(2025),
    [sym_get_max_flash_disk_space] = ACTIONS(2025),
    [sym_data_create_feed] = ACTIONS(2025),
    [sym_data_delete_feed] = ACTIONS(2025),
    [sym_data_publish_feed] = ACTIONS(2025),
    [sym_data_get_published_feed] = ACTIONS(2025),
    [sym_data_add_record] = ACTIONS(2025),
    [sym_data_get_event_record] = ACTIONS(2025),
    [sym__wc_data_create_feed] = ACTIONS(2025),
    [sym__wc_data_add_record] = ACTIONS(2025),
    [sym__wc_data_get_event_record] = ACTIONS(2025),
    [sym_format] = ACTIONS(2025),
    [sym_astro_clock] = ACTIONS(2025),
    [sym_set_outdoor_temperature] = ACTIONS(2025),
    [sym_set_virtual_level_count] = ACTIONS(2025),
    [sym_set_virtual_channel_count] = ACTIONS(2025),
    [sym_set_virtual_port_count] = ACTIONS(2025),
    [sym__wc_encode_int] = ACTIONS(2025),
    [sym__wc_decode_int] = ACTIONS(2025),
    [sym__wc_int] = ACTIONS(2025),
    [sym__wc_to_ch_int] = ACTIONS(2025),
    [sym__ch_to_wc_int] = ACTIONS(2025),
    [sym__wc_find_string_int] = ACTIONS(2025),
    [sym__wc_left_string_int] = ACTIONS(2025),
    [sym__wc_lower_string_int] = ACTIONS(2025),
    [sym__wc_mid_string_int] = ACTIONS(2025),
    [sym__wc_remove_string_int] = ACTIONS(2025),
    [sym__wc_right_string_int] = ACTIONS(2025),
    [sym__wc_upper_string_int] = ACTIONS(2025),
    [sym__wc_compare_string_int] = ACTIONS(2025),
    [sym__wc_get_buffer_char_int] = ACTIONS(2025),
    [sym__wc_get_buffer_string_int] = ACTIONS(2025),
    [sym__wc_concat_string_int] = ACTIONS(2025),
    [sym__wc_file_open_int] = ACTIONS(2025),
    [sym__wc_file_close_int] = ACTIONS(2025),
    [sym__wc_file_read_int] = ACTIONS(2025),
    [sym__wc_file_read_line_int] = ACTIONS(2025),
    [sym__wc_file_write_int] = ACTIONS(2025),
    [sym__wc_file_write_line_int] = ACTIONS(2025),
    [aux_sym_preproc_include_token1] = ACTIONS(2025),
    [aux_sym_preproc_define_token1] = ACTIONS(2027),
    [aux_sym_preproc_warn_token1] = ACTIONS(2027),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2027),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2027),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2027),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2027),
    [aux_sym_preproc_else_token1] = ACTIONS(2027),
    [anon_sym_LBRACK] = ACTIONS(2027),
    [anon_sym_LBRACE] = ACTIONS(2027),
    [anon_sym_LPAREN] = ACTIONS(2025),
    [anon_sym_BANG] = ACTIONS(2027),
    [anon_sym_TILDE] = ACTIONS(2027),
    [anon_sym_DASH] = ACTIONS(2025),
    [anon_sym_PLUS] = ACTIONS(2025),
    [anon_sym_DASH_DASH] = ACTIONS(2027),
    [anon_sym_PLUS_PLUS] = ACTIONS(2027),
    [anon_sym_DQUOTE] = ACTIONS(2027),
    [anon_sym_SQUOTE] = ACTIONS(2027),
    [sym_number_literal] = ACTIONS(2027),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(330)] = {
    [ts_builtin_sym_end] = ACTIONS(1783),
    [sym_identifier] = ACTIONS(1781),
    [sym_program_name_keyword] = ACTIONS(1781),
    [sym_module_name_keyword] = ACTIONS(1781),
    [sym_define_device_keyword] = ACTIONS(1781),
    [sym_define_combine_keyword] = ACTIONS(1781),
    [sym_define_constant_keyword] = ACTIONS(1781),
    [sym_define_type_keyword] = ACTIONS(1781),
    [sym_define_variable_keyword] = ACTIONS(1781),
    [sym_define_system_variable_keyword] = ACTIONS(1781),
    [sym_define_connect_level_keyword] = ACTIONS(1781),
    [sym_define_latching_keyword] = ACTIONS(1781),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1781),
    [sym_define_toggling_keyword] = ACTIONS(1781),
    [sym_define_call_keyword] = ACTIONS(1781),
    [sym_define_function_keyword] = ACTIONS(1781),
    [sym_define_library_function_keyword] = ACTIONS(1781),
    [sym_define_start_keyword] = ACTIONS(1781),
    [sym_define_event_keyword] = ACTIONS(1781),
    [sym_define_program_keyword] = ACTIONS(1781),
    [sym_define_module_keyword] = ACTIONS(1781),
    [sym_char_keyword] = ACTIONS(1781),
    [sym_widechar_keyword] = ACTIONS(1781),
    [sym_integer_keyword] = ACTIONS(1781),
    [sym_sinteger_keyword] = ACTIONS(1781),
    [sym_long_keyword] = ACTIONS(1781),
    [sym_slong_keyword] = ACTIONS(1781),
    [sym_float_keyword] = ACTIONS(1781),
    [sym_double_keyword] = ACTIONS(1781),
    [sym_variant_keyword] = ACTIONS(1781),
    [sym_variantarray_keyword] = ACTIONS(1781),
    [sym_local_var_keyword] = ACTIONS(1781),
    [sym_stack_var_keyword] = ACTIONS(1781),
    [sym_constant_keyword] = ACTIONS(1781),
    [sym_volatile_keyword] = ACTIONS(1781),
    [sym_non_volatile_keyword] = ACTIONS(1781),
    [sym_persistent_keyword] = ACTIONS(1781),
    [sym_dev_keyword] = ACTIONS(1781),
    [sym_devlev_keyword] = ACTIONS(1781),
    [sym_devchan_keyword] = ACTIONS(1781),
    [sym_if_keyword] = ACTIONS(1781),
    [sym_else_keyword] = ACTIONS(1781),
    [sym_switch_keyword] = ACTIONS(1781),
    [sym_case_keyword] = ACTIONS(1781),
    [sym_default_keyword] = ACTIONS(1781),
    [sym_while_keyword] = ACTIONS(1781),
    [sym_for_keyword] = ACTIONS(1781),
    [sym_break_keyword] = ACTIONS(1781),
    [sym_continue_keyword] = ACTIONS(1781),
    [sym_return_keyword] = ACTIONS(1781),
    [sym_select_keyword] = ACTIONS(1781),
    [sym_active_keyword] = ACTIONS(1781),
    [sym_struct_keyword] = ACTIONS(1781),
    [sym_structure_keyword] = ACTIONS(1781),
    [sym_button_event_keyword] = ACTIONS(1781),
    [sym_channel_event_keyword] = ACTIONS(1781),
    [sym_level_event_keyword] = ACTIONS(1781),
    [sym_data_event_keyword] = ACTIONS(1781),
    [sym_timeline_event_keyword] = ACTIONS(1781),
    [sym_custom_event_keyword] = ACTIONS(1781),
    [sym_push_keyword] = ACTIONS(1781),
    [sym_release_keyword] = ACTIONS(1781),
    [aux_sym_on_keyword_token1] = ACTIONS(1781),
    [aux_sym_off_keyword_token1] = ACTIONS(1781),
    [sym_send_level_keyword] = ACTIONS(1781),
    [sym_send_string_keyword] = ACTIONS(1781),
    [sym_send_command_keyword] = ACTIONS(1781),
    [sym_clear_buffer_keyword] = ACTIONS(1781),
    [sym_create_buffer_keyword] = ACTIONS(1781),
    [sym_create_multi_buffer_keyword] = ACTIONS(1781),
    [sym_call_keyword] = ACTIONS(1781),
    [sym_system_call_keyword] = ACTIONS(1781),
    [sym_devchan_to_keyword] = ACTIONS(1781),
    [sym_devchan_min_to_keyword] = ACTIONS(1781),
    [sym_devchan_total_off_keyword] = ACTIONS(1781),
    [sym_devchan_pulse_keyword] = ACTIONS(1781),
    [sym_wait_keyword] = ACTIONS(1781),
    [sym_cancel_wait_keyword] = ACTIONS(1781),
    [sym_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1781),
    [sym_not] = ACTIONS(1781),
    [sym_bnot] = ACTIONS(1781),
    [sym___file__] = ACTIONS(1781),
    [sym___date__] = ACTIONS(1781),
    [sym___ldate__] = ACTIONS(1781),
    [sym___time__] = ACTIONS(1781),
    [sym___line__] = ACTIONS(1781),
    [sym___name__] = ACTIONS(1781),
    [sym___netlinx__] = ACTIONS(1781),
    [sym_netlinx_axi_version] = ACTIONS(1781),
    [sym_url_flg_tcp] = ACTIONS(1781),
    [sym_url_flg_accinfopresent] = ACTIONS(1781),
    [sym_url_flg_temp] = ACTIONS(1781),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1781),
    [sym_url_flg_stat_mask] = ACTIONS(1781),
    [sym_url_flg_stat_lookup] = ACTIONS(1781),
    [sym_url_flg_stat_connecting] = ACTIONS(1781),
    [sym_url_flg_stat_waiting] = ACTIONS(1781),
    [sym_url_flg_stat_connected] = ACTIONS(1781),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1781),
    [sym_first_virtual_device] = ACTIONS(1781),
    [sym_first_local_port] = ACTIONS(1781),
    [sym_dynamic_virtual_device] = ACTIONS(1781),
    [sym_timeline_once] = ACTIONS(1781),
    [sym_timeline_repeat] = ACTIONS(1781),
    [sym_timeline_absolute] = ACTIONS(1781),
    [sym_timeline_relative] = ACTIONS(1781),
    [sym_do_push_timed_infinite] = ACTIONS(1781),
    [sym_source_type_no_address] = ACTIONS(1781),
    [sym_source_type_neuron_id] = ACTIONS(1781),
    [sym_source_type_ip_address] = ACTIONS(1781),
    [sym_source_type_axlink] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1781),
    [sym_source_type_ip_socket_address] = ACTIONS(1781),
    [sym_source_type_rs232] = ACTIONS(1781),
    [sym_source_type_internal] = ACTIONS(1781),
    [sym_source_type_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1781),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1781),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1781),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1781),
    [sym_true] = ACTIONS(1781),
    [sym_false] = ACTIONS(1781),
    [sym_file_read_only] = ACTIONS(1781),
    [sym_file_rw_new] = ACTIONS(1781),
    [sym_file_rw_append] = ACTIONS(1781),
    [sym_ip_tcp] = ACTIONS(1781),
    [sym_ip_udp] = ACTIONS(1781),
    [sym_ip_udp_2way] = ACTIONS(1781),
    [sym_xml_encode_types] = ACTIONS(1781),
    [sym_xml_encode_char_as_list] = ACTIONS(1781),
    [sym_xml_encode_le] = ACTIONS(1781),
    [sym_xml_decode_types] = ACTIONS(1781),
    [sym_xml_decode_no_preserve] = ACTIONS(1781),
    [sym_duet_dev_type_display_device] = ACTIONS(1781),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_io_device] = ACTIONS(1781),
    [sym_duet_dev_type_relay_device] = ACTIONS(1781),
    [sym_duet_dev_type_ups] = ACTIONS(1781),
    [sym_duet_dev_type_amplifier] = ACTIONS(1781),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1781),
    [sym_duet_dev_type_camera] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1781),
    [sym_duet_dev_type_dss] = ACTIONS(1781),
    [sym_duet_dev_type_dvr] = ACTIONS(1781),
    [sym_duet_dev_type_disc_device] = ACTIONS(1781),
    [sym_duet_dev_type_document_camera] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1781),
    [sym_duet_dev_type_hvac] = ACTIONS(1781),
    [sym_duet_dev_type_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_light] = ACTIONS(1781),
    [sym_duet_dev_type_monitor] = ACTIONS(1781),
    [sym_duet_dev_type_motor] = ACTIONS(1781),
    [sym_duet_dev_type_multi_window] = ACTIONS(1781),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1781),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1781),
    [sym_duet_dev_type_receiver] = ACTIONS(1781),
    [sym_duet_dev_type_security_system] = ACTIONS(1781),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1781),
    [sym_duet_dev_type_settop_box] = ACTIONS(1781),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1781),
    [sym_duet_dev_type_switcher] = ACTIONS(1781),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_tv] = ACTIONS(1781),
    [sym_duet_dev_type_utility] = ACTIONS(1781),
    [sym_duet_dev_type_vcr] = ACTIONS(1781),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_video_processor] = ACTIONS(1781),
    [sym_duet_dev_type_video_projector] = ACTIONS(1781),
    [sym_duet_dev_type_video_wall] = ACTIONS(1781),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1781),
    [sym_duet_dev_type_weather] = ACTIONS(1781),
    [sym_duet_dev_not_polled] = ACTIONS(1781),
    [sym_duet_dev_polled] = ACTIONS(1781),
    [sym_clkmgr_mode_network] = ACTIONS(1781),
    [sym_clkmgr_mode_standalone] = ACTIONS(1781),
    [sym_normal_standby] = ACTIONS(1781),
    [sym_normal_wake] = ACTIONS(1781),
    [sym_ip_multicast_ttl_option] = ACTIONS(1781),
    [sym_ip_ttl_subnet] = ACTIONS(1781),
    [sym_ip_ttl_site] = ACTIONS(1781),
    [sym_ip_ttl_region] = ACTIONS(1781),
    [sym_ip_ttl_continent] = ACTIONS(1781),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1781),
    [sym_ip_nodelay_on] = ACTIONS(1781),
    [sym_ip_nodelay_off] = ACTIONS(1781),
    [sym_amx_error] = ACTIONS(1781),
    [sym_amx_warning] = ACTIONS(1781),
    [sym_amx_info] = ACTIONS(1781),
    [sym_amx_debug] = ACTIONS(1781),
    [sym_netlinx_login_success] = ACTIONS(1781),
    [sym_netlinx_login_fail] = ACTIONS(1781),
    [sym_netlinx_logout] = ACTIONS(1781),
    [sym_null_str] = ACTIONS(1781),
    [sym_smtp_address] = ACTIONS(1781),
    [sym_smtp_port_number] = ACTIONS(1781),
    [sym_smtp_username] = ACTIONS(1781),
    [sym_smtp_password] = ACTIONS(1781),
    [sym_smtp_require_tls] = ACTIONS(1781),
    [sym_smtp_from] = ACTIONS(1781),
    [sym_smtp_tls_true] = ACTIONS(1781),
    [sym_smtp_tls_false] = ACTIONS(1781),
    [sym_listview_on_row_select_event] = ACTIONS(1781),
    [sym_data_structure_unknown] = ACTIONS(1781),
    [sym_data_structure_datafeed] = ACTIONS(1781),
    [sym_data_structure_datarecordset] = ACTIONS(1781),
    [sym_data_structure_datarecord] = ACTIONS(1781),
    [sym_data_structure_datafield] = ACTIONS(1781),
    [sym_data_max_content_field_count] = ACTIONS(1781),
    [sym_data_max_metadata_field_count] = ACTIONS(1781),
    [sym_data_max_id_length] = ACTIONS(1781),
    [sym_data_max_name_length] = ACTIONS(1781),
    [sym_data_max_description_length] = ACTIONS(1781),
    [sym_data_max_type_length] = ACTIONS(1781),
    [sym_data_max_format_length] = ACTIONS(1781),
    [sym_data_max_source_length] = ACTIONS(1781),
    [sym_data_max_value_length] = ACTIONS(1781),
    [sym_data_max_label_length] = ACTIONS(1781),
    [sym_data_max_url_length] = ACTIONS(1781),
    [sym_max_last_login_info_length] = ACTIONS(1781),
    [sym_data_type_unknown] = ACTIONS(1781),
    [sym_data_type_string] = ACTIONS(1781),
    [sym_data_type_datetime] = ACTIONS(1781),
    [sym_data_type_date] = ACTIONS(1781),
    [sym_data_type_time] = ACTIONS(1781),
    [sym_data_type_image] = ACTIONS(1781),
    [sym_data_format_url] = ACTIONS(1781),
    [sym_data_format_phone] = ACTIONS(1781),
    [sym_data_format_email] = ACTIONS(1781),
    [sym_data_format_iso8601] = ACTIONS(1781),
    [sym_auditlog_priv] = ACTIONS(1781),
    [sym_configuration_priv] = ACTIONS(1781),
    [sym_deviceconfig_priv] = ACTIONS(1781),
    [sym_ftp_priv] = ACTIONS(1781),
    [sym_http_priv] = ACTIONS(1781),
    [sym_networkconfig_priv] = ACTIONS(1781),
    [sym_programport_priv] = ACTIONS(1781),
    [sym_remoteui_priv] = ACTIONS(1781),
    [sym_securitycontrol_priv] = ACTIONS(1781),
    [sym_softwaremanagement_priv] = ACTIONS(1781),
    [sym_terminal_priv] = ACTIONS(1781),
    [sym_tpadmin_priv] = ACTIONS(1781),
    [sym_user1_priv] = ACTIONS(1781),
    [sym_user2_priv] = ACTIONS(1781),
    [sym_user3_priv] = ACTIONS(1781),
    [sym_user4_priv] = ACTIONS(1781),
    [sym_usermanagement_priv] = ACTIONS(1781),
    [sym_valid_account] = ACTIONS(1781),
    [sym_err_name_invalid] = ACTIONS(1781),
    [sym_err_password_invalid] = ACTIONS(1781),
    [sym_err_invalid_account] = ACTIONS(1781),
    [sym_err_no_authorization] = ACTIONS(1781),
    [sym_err_invalid_parameter] = ACTIONS(1781),
    [sym_err_user_locked_out] = ACTIONS(1781),
    [sym_err_user_account_expired] = ACTIONS(1781),
    [sym_err_general_error] = ACTIONS(1781),
    [sym_err_authentication_error] = ACTIONS(1781),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1781),
    [sym_tls_validate_certificate] = ACTIONS(1781),
    [sym_tdata] = ACTIONS(1781),
    [sym_tchannel] = ACTIONS(1781),
    [sym_tlevel] = ACTIONS(1781),
    [sym_tbutton] = ACTIONS(1781),
    [sym_ttimeline] = ACTIONS(1781),
    [sym_tcustom] = ACTIONS(1781),
    [sym_url_struct] = ACTIONS(1781),
    [sym_dns_struct] = ACTIONS(1781),
    [sym_ip_address_struct] = ACTIONS(1781),
    [sym_dev_info_struct] = ACTIONS(1781),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1781),
    [sym_data_feed] = ACTIONS(1781),
    [sym_data_field] = ACTIONS(1781),
    [sym_data_record] = ACTIONS(1781),
    [sym_wc_data_feed] = ACTIONS(1781),
    [sym_wc_data_field] = ACTIONS(1781),
    [sym_wc_data_record] = ACTIONS(1781),
    [sym_last_login_info] = ACTIONS(1781),
    [sym____reserved___] = ACTIONS(1781),
    [sym_date] = ACTIONS(1781),
    [sym_day] = ACTIONS(1781),
    [sym_time] = ACTIONS(1781),
    [sym_ldate] = ACTIONS(1781),
    [sym_push_device] = ACTIONS(1781),
    [sym_push_channel] = ACTIONS(1781),
    [sym_push_devchan] = ACTIONS(1781),
    [sym_release_device] = ACTIONS(1781),
    [sym_release_channel] = ACTIONS(1781),
    [sym_release_devchan] = ACTIONS(1781),
    [sym_master_slot] = ACTIONS(1781),
    [sym_get_pulse_time] = ACTIONS(1781),
    [sym_get_timer] = ACTIONS(1781),
    [sym_master_sn] = ACTIONS(1781),
    [sym_system_number] = ACTIONS(1781),
    [sym_button] = ACTIONS(1781),
    [sym_dv_channel] = ACTIONS(1781),
    [sym_data] = ACTIONS(1781),
    [sym_level] = ACTIONS(1781),
    [sym_timeline] = ACTIONS(1781),
    [sym_channel] = ACTIONS(1781),
    [sym_custom] = ACTIONS(1781),
    [sym_atoi] = ACTIONS(1781),
    [sym_atol] = ACTIONS(1781),
    [sym_atof] = ACTIONS(1781),
    [sym_itoa] = ACTIONS(1781),
    [sym_itohex] = ACTIONS(1781),
    [sym_hextoi] = ACTIONS(1781),
    [sym_ftoa] = ACTIONS(1781),
    [sym_find_string] = ACTIONS(1781),
    [sym_left_string] = ACTIONS(1781),
    [sym_right_string] = ACTIONS(1781),
    [sym_mid_string] = ACTIONS(1781),
    [sym_remove_string] = ACTIONS(1781),
    [sym_get_buffer_string] = ACTIONS(1781),
    [sym_get_buffer_char] = ACTIONS(1781),
    [sym_get_multi_buffer_string] = ACTIONS(1781),
    [sym_length_string] = ACTIONS(1781),
    [sym_max_length_string] = ACTIONS(1781),
    [sym_set_length_string] = ACTIONS(1781),
    [sym_compare_string] = ACTIONS(1781),
    [sym_upper_string] = ACTIONS(1781),
    [sym_lower_string] = ACTIONS(1781),
    [sym_get_last] = ACTIONS(1781),
    [sym_date_to_day] = ACTIONS(1781),
    [sym_date_to_month] = ACTIONS(1781),
    [sym_date_to_year] = ACTIONS(1781),
    [sym_time_to_hour] = ACTIONS(1781),
    [sym_time_to_minute] = ACTIONS(1781),
    [sym_time_to_second] = ACTIONS(1781),
    [sym_day_of_week] = ACTIONS(1781),
    [sym_set_pulse_time] = ACTIONS(1781),
    [sym_set_timer] = ACTIONS(1781),
    [sym_file_open] = ACTIONS(1781),
    [sym_file_close] = ACTIONS(1781),
    [sym_file_read] = ACTIONS(1781),
    [sym_file_read_line] = ACTIONS(1781),
    [sym_file_write] = ACTIONS(1781),
    [sym_file_write_line] = ACTIONS(1781),
    [sym_file_copy] = ACTIONS(1781),
    [sym_file_delete] = ACTIONS(1781),
    [sym_file_rename] = ACTIONS(1781),
    [sym_file_dir] = ACTIONS(1781),
    [sym_file_getdir] = ACTIONS(1781),
    [sym_file_setdir] = ACTIONS(1781),
    [sym_file_create_dir] = ACTIONS(1781),
    [sym_file_remove_dir] = ACTIONS(1781),
    [sym_file_seek] = ACTIONS(1781),
    [sym_length_array] = ACTIONS(1781),
    [sym_max_length_array] = ACTIONS(1781),
    [sym_set_length_array] = ACTIONS(1781),
    [sym_ip_client_open] = ACTIONS(1781),
    [sym_ip_client_close] = ACTIONS(1781),
    [sym_ip_server_open] = ACTIONS(1781),
    [sym_ip_server_close] = ACTIONS(1781),
    [sym_ip_bound_client_open] = ACTIONS(1781),
    [sym_ip_mc_server_open] = ACTIONS(1781),
    [sym_ip_set_option] = ACTIONS(1781),
    [sym_ssh_client_open] = ACTIONS(1781),
    [sym_ssh_client_close] = ACTIONS(1781),
    [sym_tls_client_open] = ACTIONS(1781),
    [sym_tls_client_close] = ACTIONS(1781),
    [sym_do_push] = ACTIONS(1781),
    [sym_do_push_timed] = ACTIONS(1781),
    [sym_do_release] = ACTIONS(1781),
    [sym_device_id] = ACTIONS(1781),
    [sym_device_info] = ACTIONS(1781),
    [sym_device_id_string] = ACTIONS(1781),
    [sym_device_standby] = ACTIONS(1781),
    [sym_device_wake] = ACTIONS(1781),
    [sym_timeline_create] = ACTIONS(1781),
    [sym_timeline_kill] = ACTIONS(1781),
    [sym_timeline_reload] = ACTIONS(1781),
    [sym_timeline_pause] = ACTIONS(1781),
    [sym_timeline_restart] = ACTIONS(1781),
    [sym_timeline_set] = ACTIONS(1781),
    [sym_timeline_get] = ACTIONS(1781),
    [sym_timeline_active] = ACTIONS(1781),
    [sym_timeline_dynamic_id] = ACTIONS(1781),
    [sym_variable_to_string] = ACTIONS(1781),
    [sym_string_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_string] = ACTIONS(1781),
    [sym_type_cast] = ACTIONS(1781),
    [sym_variable_to_xml] = ACTIONS(1781),
    [sym_xml_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_xml] = ACTIONS(1781),
    [sym_redirect_string] = ACTIONS(1781),
    [sym_random_number] = ACTIONS(1781),
    [sym_get_unique_id] = ACTIONS(1781),
    [sym_get_system_number] = ACTIONS(1781),
    [sym_set_system_number] = ACTIONS(1781),
    [sym_get_serial_number] = ACTIONS(1781),
    [sym_get_master_build] = ACTIONS(1781),
    [sym_set_validation_code] = ACTIONS(1781),
    [sym_reboot] = ACTIONS(1781),
    [sym_load_duet_module] = ACTIONS(1781),
    [sym_unload_duet_module] = ACTIONS(1781),
    [sym_duet_mem_size_set] = ACTIONS(1781),
    [sym_duet_mem_size_get] = ACTIONS(1781),
    [sym_dynamic_application_device] = ACTIONS(1781),
    [sym_dynamic_polled_port] = ACTIONS(1781),
    [sym_static_port_binding] = ACTIONS(1781),
    [sym_static_ip_binding] = ACTIONS(1781),
    [sym_get_dns_list] = ACTIONS(1781),
    [sym_set_dns_list] = ACTIONS(1781),
    [sym_get_ip_address] = ACTIONS(1781),
    [sym_set_ip_address] = ACTIONS(1781),
    [sym_get_url_list] = ACTIONS(1781),
    [sym_add_url_entry] = ACTIONS(1781),
    [sym_delete_url_entry] = ACTIONS(1781),
    [sym_abs_value] = ACTIONS(1781),
    [sym_max_value] = ACTIONS(1781),
    [sym_min_value] = ACTIONS(1781),
    [sym_exp_value] = ACTIONS(1781),
    [sym_log_value] = ACTIONS(1781),
    [sym_log10_value] = ACTIONS(1781),
    [sym_power_value] = ACTIONS(1781),
    [sym_sqrt_value] = ACTIONS(1781),
    [sym_raw_be] = ACTIONS(1781),
    [sym_raw_le] = ACTIONS(1781),
    [sym_internal_queue_size_set] = ACTIONS(1781),
    [sym_internal_queue_size_get] = ACTIONS(1781),
    [sym_internal_threshold_set] = ACTIONS(1781),
    [sym_internal_threshold_get] = ACTIONS(1781),
    [sym_rebuilt_event] = ACTIONS(1781),
    [sym_do_custom_event] = ACTIONS(1781),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1781),
    [sym_clkmgr_set_clk_source] = ACTIONS(1781),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1781),
    [sym_clkmgr_get_timezone] = ACTIONS(1781),
    [sym_clkmgr_set_timezone] = ACTIONS(1781),
    [sym_clkmgr_get_resync_period] = ACTIONS(1781),
    [sym_clkmgr_set_resync_period] = ACTIONS(1781),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_timeservers] = ACTIONS(1781),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_set_log_level] = ACTIONS(1781),
    [sym_get_log_level] = ACTIONS(1781),
    [sym_amx_log] = ACTIONS(1781),
    [sym_smtp_server_config_set] = ACTIONS(1781),
    [sym_smtp_server_config_get] = ACTIONS(1781),
    [sym_smtp_send] = ACTIONS(1781),
    [sym_validate_netlinx_account] = ACTIONS(1781),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1781),
    [sym_authenticate_certificate] = ACTIONS(1781),
    [sym_audit_netlinx_session_event] = ACTIONS(1781),
    [sym_audit_netlinx_generic_event] = ACTIONS(1781),
    [sym_get_available_flash_disk_space] = ACTIONS(1781),
    [sym_get_max_flash_disk_space] = ACTIONS(1781),
    [sym_data_create_feed] = ACTIONS(1781),
    [sym_data_delete_feed] = ACTIONS(1781),
    [sym_data_publish_feed] = ACTIONS(1781),
    [sym_data_get_published_feed] = ACTIONS(1781),
    [sym_data_add_record] = ACTIONS(1781),
    [sym_data_get_event_record] = ACTIONS(1781),
    [sym__wc_data_create_feed] = ACTIONS(1781),
    [sym__wc_data_add_record] = ACTIONS(1781),
    [sym__wc_data_get_event_record] = ACTIONS(1781),
    [sym_format] = ACTIONS(1781),
    [sym_astro_clock] = ACTIONS(1781),
    [sym_set_outdoor_temperature] = ACTIONS(1781),
    [sym_set_virtual_level_count] = ACTIONS(1781),
    [sym_set_virtual_channel_count] = ACTIONS(1781),
    [sym_set_virtual_port_count] = ACTIONS(1781),
    [sym__wc_encode_int] = ACTIONS(1781),
    [sym__wc_decode_int] = ACTIONS(1781),
    [sym__wc_int] = ACTIONS(1781),
    [sym__wc_to_ch_int] = ACTIONS(1781),
    [sym__ch_to_wc_int] = ACTIONS(1781),
    [sym__wc_find_string_int] = ACTIONS(1781),
    [sym__wc_left_string_int] = ACTIONS(1781),
    [sym__wc_lower_string_int] = ACTIONS(1781),
    [sym__wc_mid_string_int] = ACTIONS(1781),
    [sym__wc_remove_string_int] = ACTIONS(1781),
    [sym__wc_right_string_int] = ACTIONS(1781),
    [sym__wc_upper_string_int] = ACTIONS(1781),
    [sym__wc_compare_string_int] = ACTIONS(1781),
    [sym__wc_get_buffer_char_int] = ACTIONS(1781),
    [sym__wc_get_buffer_string_int] = ACTIONS(1781),
    [sym__wc_concat_string_int] = ACTIONS(1781),
    [sym__wc_file_open_int] = ACTIONS(1781),
    [sym__wc_file_close_int] = ACTIONS(1781),
    [sym__wc_file_read_int] = ACTIONS(1781),
    [sym__wc_file_read_line_int] = ACTIONS(1781),
    [sym__wc_file_write_int] = ACTIONS(1781),
    [sym__wc_file_write_line_int] = ACTIONS(1781),
    [aux_sym_preproc_include_token1] = ACTIONS(1781),
    [aux_sym_preproc_define_token1] = ACTIONS(1783),
    [aux_sym_preproc_warn_token1] = ACTIONS(1783),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1783),
    [anon_sym_LBRACK] = ACTIONS(1783),
    [anon_sym_LBRACE] = ACTIONS(1783),
    [anon_sym_LPAREN] = ACTIONS(1781),
    [anon_sym_BANG] = ACTIONS(1783),
    [anon_sym_TILDE] = ACTIONS(1783),
    [anon_sym_DASH] = ACTIONS(1781),
    [anon_sym_PLUS] = ACTIONS(1781),
    [anon_sym_DASH_DASH] = ACTIONS(1783),
    [anon_sym_PLUS_PLUS] = ACTIONS(1783),
    [anon_sym_DQUOTE] = ACTIONS(1783),
    [anon_sym_SQUOTE] = ACTIONS(1783),
    [sym_number_literal] = ACTIONS(1783),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(331)] = {
    [ts_builtin_sym_end] = ACTIONS(1873),
    [sym_identifier] = ACTIONS(1871),
    [sym_program_name_keyword] = ACTIONS(1871),
    [sym_module_name_keyword] = ACTIONS(1871),
    [sym_define_device_keyword] = ACTIONS(1871),
    [sym_define_combine_keyword] = ACTIONS(1871),
    [sym_define_constant_keyword] = ACTIONS(1871),
    [sym_define_type_keyword] = ACTIONS(1871),
    [sym_define_variable_keyword] = ACTIONS(1871),
    [sym_define_system_variable_keyword] = ACTIONS(1871),
    [sym_define_connect_level_keyword] = ACTIONS(1871),
    [sym_define_latching_keyword] = ACTIONS(1871),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1871),
    [sym_define_toggling_keyword] = ACTIONS(1871),
    [sym_define_call_keyword] = ACTIONS(1871),
    [sym_define_function_keyword] = ACTIONS(1871),
    [sym_define_library_function_keyword] = ACTIONS(1871),
    [sym_define_start_keyword] = ACTIONS(1871),
    [sym_define_event_keyword] = ACTIONS(1871),
    [sym_define_program_keyword] = ACTIONS(1871),
    [sym_define_module_keyword] = ACTIONS(1871),
    [sym_char_keyword] = ACTIONS(1871),
    [sym_widechar_keyword] = ACTIONS(1871),
    [sym_integer_keyword] = ACTIONS(1871),
    [sym_sinteger_keyword] = ACTIONS(1871),
    [sym_long_keyword] = ACTIONS(1871),
    [sym_slong_keyword] = ACTIONS(1871),
    [sym_float_keyword] = ACTIONS(1871),
    [sym_double_keyword] = ACTIONS(1871),
    [sym_variant_keyword] = ACTIONS(1871),
    [sym_variantarray_keyword] = ACTIONS(1871),
    [sym_local_var_keyword] = ACTIONS(1871),
    [sym_stack_var_keyword] = ACTIONS(1871),
    [sym_constant_keyword] = ACTIONS(1871),
    [sym_volatile_keyword] = ACTIONS(1871),
    [sym_non_volatile_keyword] = ACTIONS(1871),
    [sym_persistent_keyword] = ACTIONS(1871),
    [sym_dev_keyword] = ACTIONS(1871),
    [sym_devlev_keyword] = ACTIONS(1871),
    [sym_devchan_keyword] = ACTIONS(1871),
    [sym_if_keyword] = ACTIONS(1871),
    [sym_else_keyword] = ACTIONS(1871),
    [sym_switch_keyword] = ACTIONS(1871),
    [sym_case_keyword] = ACTIONS(1871),
    [sym_default_keyword] = ACTIONS(1871),
    [sym_while_keyword] = ACTIONS(1871),
    [sym_for_keyword] = ACTIONS(1871),
    [sym_break_keyword] = ACTIONS(1871),
    [sym_continue_keyword] = ACTIONS(1871),
    [sym_return_keyword] = ACTIONS(1871),
    [sym_select_keyword] = ACTIONS(1871),
    [sym_active_keyword] = ACTIONS(1871),
    [sym_struct_keyword] = ACTIONS(1871),
    [sym_structure_keyword] = ACTIONS(1871),
    [sym_button_event_keyword] = ACTIONS(1871),
    [sym_channel_event_keyword] = ACTIONS(1871),
    [sym_level_event_keyword] = ACTIONS(1871),
    [sym_data_event_keyword] = ACTIONS(1871),
    [sym_timeline_event_keyword] = ACTIONS(1871),
    [sym_custom_event_keyword] = ACTIONS(1871),
    [sym_push_keyword] = ACTIONS(1871),
    [sym_release_keyword] = ACTIONS(1871),
    [aux_sym_on_keyword_token1] = ACTIONS(1871),
    [aux_sym_off_keyword_token1] = ACTIONS(1871),
    [sym_send_level_keyword] = ACTIONS(1871),
    [sym_send_string_keyword] = ACTIONS(1871),
    [sym_send_command_keyword] = ACTIONS(1871),
    [sym_clear_buffer_keyword] = ACTIONS(1871),
    [sym_create_buffer_keyword] = ACTIONS(1871),
    [sym_create_multi_buffer_keyword] = ACTIONS(1871),
    [sym_call_keyword] = ACTIONS(1871),
    [sym_system_call_keyword] = ACTIONS(1871),
    [sym_devchan_to_keyword] = ACTIONS(1871),
    [sym_devchan_min_to_keyword] = ACTIONS(1871),
    [sym_devchan_total_off_keyword] = ACTIONS(1871),
    [sym_devchan_pulse_keyword] = ACTIONS(1871),
    [sym_wait_keyword] = ACTIONS(1871),
    [sym_cancel_wait_keyword] = ACTIONS(1871),
    [sym_wait_until_keyword] = ACTIONS(1871),
    [sym_cancel_wait_until_keyword] = ACTIONS(1871),
    [sym_cancel_all_wait_keyword] = ACTIONS(1871),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1871),
    [sym_not] = ACTIONS(1871),
    [sym_bnot] = ACTIONS(1871),
    [sym___file__] = ACTIONS(1871),
    [sym___date__] = ACTIONS(1871),
    [sym___ldate__] = ACTIONS(1871),
    [sym___time__] = ACTIONS(1871),
    [sym___line__] = ACTIONS(1871),
    [sym___name__] = ACTIONS(1871),
    [sym___netlinx__] = ACTIONS(1871),
    [sym_netlinx_axi_version] = ACTIONS(1871),
    [sym_url_flg_tcp] = ACTIONS(1871),
    [sym_url_flg_accinfopresent] = ACTIONS(1871),
    [sym_url_flg_temp] = ACTIONS(1871),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1871),
    [sym_url_flg_stat_mask] = ACTIONS(1871),
    [sym_url_flg_stat_lookup] = ACTIONS(1871),
    [sym_url_flg_stat_connecting] = ACTIONS(1871),
    [sym_url_flg_stat_waiting] = ACTIONS(1871),
    [sym_url_flg_stat_connected] = ACTIONS(1871),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1871),
    [sym_first_virtual_device] = ACTIONS(1871),
    [sym_first_local_port] = ACTIONS(1871),
    [sym_dynamic_virtual_device] = ACTIONS(1871),
    [sym_timeline_once] = ACTIONS(1871),
    [sym_timeline_repeat] = ACTIONS(1871),
    [sym_timeline_absolute] = ACTIONS(1871),
    [sym_timeline_relative] = ACTIONS(1871),
    [sym_do_push_timed_infinite] = ACTIONS(1871),
    [sym_source_type_no_address] = ACTIONS(1871),
    [sym_source_type_neuron_id] = ACTIONS(1871),
    [sym_source_type_ip_address] = ACTIONS(1871),
    [sym_source_type_axlink] = ACTIONS(1871),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1871),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1871),
    [sym_source_type_ip_socket_address] = ACTIONS(1871),
    [sym_source_type_rs232] = ACTIONS(1871),
    [sym_source_type_internal] = ACTIONS(1871),
    [sym_source_type_mac_address] = ACTIONS(1871),
    [sym_source_type_ipv4_port] = ACTIONS(1871),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1871),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1871),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1871),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1871),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1871),
    [sym_true] = ACTIONS(1871),
    [sym_false] = ACTIONS(1871),
    [sym_file_read_only] = ACTIONS(1871),
    [sym_file_rw_new] = ACTIONS(1871),
    [sym_file_rw_append] = ACTIONS(1871),
    [sym_ip_tcp] = ACTIONS(1871),
    [sym_ip_udp] = ACTIONS(1871),
    [sym_ip_udp_2way] = ACTIONS(1871),
    [sym_xml_encode_types] = ACTIONS(1871),
    [sym_xml_encode_char_as_list] = ACTIONS(1871),
    [sym_xml_encode_le] = ACTIONS(1871),
    [sym_xml_decode_types] = ACTIONS(1871),
    [sym_xml_decode_no_preserve] = ACTIONS(1871),
    [sym_duet_dev_type_display_device] = ACTIONS(1871),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1871),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1871),
    [sym_duet_dev_type_io_device] = ACTIONS(1871),
    [sym_duet_dev_type_relay_device] = ACTIONS(1871),
    [sym_duet_dev_type_ups] = ACTIONS(1871),
    [sym_duet_dev_type_amplifier] = ACTIONS(1871),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1871),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1871),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1871),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1871),
    [sym_duet_dev_type_camera] = ACTIONS(1871),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1871),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1871),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1871),
    [sym_duet_dev_type_dss] = ACTIONS(1871),
    [sym_duet_dev_type_dvr] = ACTIONS(1871),
    [sym_duet_dev_type_disc_device] = ACTIONS(1871),
    [sym_duet_dev_type_document_camera] = ACTIONS(1871),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1871),
    [sym_duet_dev_type_hvac] = ACTIONS(1871),
    [sym_duet_dev_type_keypad] = ACTIONS(1871),
    [sym_duet_dev_type_light] = ACTIONS(1871),
    [sym_duet_dev_type_monitor] = ACTIONS(1871),
    [sym_duet_dev_type_motor] = ACTIONS(1871),
    [sym_duet_dev_type_multi_window] = ACTIONS(1871),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1871),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1871),
    [sym_duet_dev_type_receiver] = ACTIONS(1871),
    [sym_duet_dev_type_security_system] = ACTIONS(1871),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1871),
    [sym_duet_dev_type_settop_box] = ACTIONS(1871),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1871),
    [sym_duet_dev_type_switcher] = ACTIONS(1871),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1871),
    [sym_duet_dev_type_tv] = ACTIONS(1871),
    [sym_duet_dev_type_utility] = ACTIONS(1871),
    [sym_duet_dev_type_vcr] = ACTIONS(1871),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1871),
    [sym_duet_dev_type_video_processor] = ACTIONS(1871),
    [sym_duet_dev_type_video_projector] = ACTIONS(1871),
    [sym_duet_dev_type_video_wall] = ACTIONS(1871),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1871),
    [sym_duet_dev_type_weather] = ACTIONS(1871),
    [sym_duet_dev_not_polled] = ACTIONS(1871),
    [sym_duet_dev_polled] = ACTIONS(1871),
    [sym_clkmgr_mode_network] = ACTIONS(1871),
    [sym_clkmgr_mode_standalone] = ACTIONS(1871),
    [sym_normal_standby] = ACTIONS(1871),
    [sym_normal_wake] = ACTIONS(1871),
    [sym_ip_multicast_ttl_option] = ACTIONS(1871),
    [sym_ip_ttl_subnet] = ACTIONS(1871),
    [sym_ip_ttl_site] = ACTIONS(1871),
    [sym_ip_ttl_region] = ACTIONS(1871),
    [sym_ip_ttl_continent] = ACTIONS(1871),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1871),
    [sym_ip_nodelay_on] = ACTIONS(1871),
    [sym_ip_nodelay_off] = ACTIONS(1871),
    [sym_amx_error] = ACTIONS(1871),
    [sym_amx_warning] = ACTIONS(1871),
    [sym_amx_info] = ACTIONS(1871),
    [sym_amx_debug] = ACTIONS(1871),
    [sym_netlinx_login_success] = ACTIONS(1871),
    [sym_netlinx_login_fail] = ACTIONS(1871),
    [sym_netlinx_logout] = ACTIONS(1871),
    [sym_null_str] = ACTIONS(1871),
    [sym_smtp_address] = ACTIONS(1871),
    [sym_smtp_port_number] = ACTIONS(1871),
    [sym_smtp_username] = ACTIONS(1871),
    [sym_smtp_password] = ACTIONS(1871),
    [sym_smtp_require_tls] = ACTIONS(1871),
    [sym_smtp_from] = ACTIONS(1871),
    [sym_smtp_tls_true] = ACTIONS(1871),
    [sym_smtp_tls_false] = ACTIONS(1871),
    [sym_listview_on_row_select_event] = ACTIONS(1871),
    [sym_data_structure_unknown] = ACTIONS(1871),
    [sym_data_structure_datafeed] = ACTIONS(1871),
    [sym_data_structure_datarecordset] = ACTIONS(1871),
    [sym_data_structure_datarecord] = ACTIONS(1871),
    [sym_data_structure_datafield] = ACTIONS(1871),
    [sym_data_max_content_field_count] = ACTIONS(1871),
    [sym_data_max_metadata_field_count] = ACTIONS(1871),
    [sym_data_max_id_length] = ACTIONS(1871),
    [sym_data_max_name_length] = ACTIONS(1871),
    [sym_data_max_description_length] = ACTIONS(1871),
    [sym_data_max_type_length] = ACTIONS(1871),
    [sym_data_max_format_length] = ACTIONS(1871),
    [sym_data_max_source_length] = ACTIONS(1871),
    [sym_data_max_value_length] = ACTIONS(1871),
    [sym_data_max_label_length] = ACTIONS(1871),
    [sym_data_max_url_length] = ACTIONS(1871),
    [sym_max_last_login_info_length] = ACTIONS(1871),
    [sym_data_type_unknown] = ACTIONS(1871),
    [sym_data_type_string] = ACTIONS(1871),
    [sym_data_type_datetime] = ACTIONS(1871),
    [sym_data_type_date] = ACTIONS(1871),
    [sym_data_type_time] = ACTIONS(1871),
    [sym_data_type_image] = ACTIONS(1871),
    [sym_data_format_url] = ACTIONS(1871),
    [sym_data_format_phone] = ACTIONS(1871),
    [sym_data_format_email] = ACTIONS(1871),
    [sym_data_format_iso8601] = ACTIONS(1871),
    [sym_auditlog_priv] = ACTIONS(1871),
    [sym_configuration_priv] = ACTIONS(1871),
    [sym_deviceconfig_priv] = ACTIONS(1871),
    [sym_ftp_priv] = ACTIONS(1871),
    [sym_http_priv] = ACTIONS(1871),
    [sym_networkconfig_priv] = ACTIONS(1871),
    [sym_programport_priv] = ACTIONS(1871),
    [sym_remoteui_priv] = ACTIONS(1871),
    [sym_securitycontrol_priv] = ACTIONS(1871),
    [sym_softwaremanagement_priv] = ACTIONS(1871),
    [sym_terminal_priv] = ACTIONS(1871),
    [sym_tpadmin_priv] = ACTIONS(1871),
    [sym_user1_priv] = ACTIONS(1871),
    [sym_user2_priv] = ACTIONS(1871),
    [sym_user3_priv] = ACTIONS(1871),
    [sym_user4_priv] = ACTIONS(1871),
    [sym_usermanagement_priv] = ACTIONS(1871),
    [sym_valid_account] = ACTIONS(1871),
    [sym_err_name_invalid] = ACTIONS(1871),
    [sym_err_password_invalid] = ACTIONS(1871),
    [sym_err_invalid_account] = ACTIONS(1871),
    [sym_err_no_authorization] = ACTIONS(1871),
    [sym_err_invalid_parameter] = ACTIONS(1871),
    [sym_err_user_locked_out] = ACTIONS(1871),
    [sym_err_user_account_expired] = ACTIONS(1871),
    [sym_err_general_error] = ACTIONS(1871),
    [sym_err_authentication_error] = ACTIONS(1871),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1871),
    [sym_tls_validate_certificate] = ACTIONS(1871),
    [sym_tdata] = ACTIONS(1871),
    [sym_tchannel] = ACTIONS(1871),
    [sym_tlevel] = ACTIONS(1871),
    [sym_tbutton] = ACTIONS(1871),
    [sym_ttimeline] = ACTIONS(1871),
    [sym_tcustom] = ACTIONS(1871),
    [sym_url_struct] = ACTIONS(1871),
    [sym_dns_struct] = ACTIONS(1871),
    [sym_ip_address_struct] = ACTIONS(1871),
    [sym_dev_info_struct] = ACTIONS(1871),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1871),
    [sym_data_feed] = ACTIONS(1871),
    [sym_data_field] = ACTIONS(1871),
    [sym_data_record] = ACTIONS(1871),
    [sym_wc_data_feed] = ACTIONS(1871),
    [sym_wc_data_field] = ACTIONS(1871),
    [sym_wc_data_record] = ACTIONS(1871),
    [sym_last_login_info] = ACTIONS(1871),
    [sym____reserved___] = ACTIONS(1871),
    [sym_date] = ACTIONS(1871),
    [sym_day] = ACTIONS(1871),
    [sym_time] = ACTIONS(1871),
    [sym_ldate] = ACTIONS(1871),
    [sym_push_device] = ACTIONS(1871),
    [sym_push_channel] = ACTIONS(1871),
    [sym_push_devchan] = ACTIONS(1871),
    [sym_release_device] = ACTIONS(1871),
    [sym_release_channel] = ACTIONS(1871),
    [sym_release_devchan] = ACTIONS(1871),
    [sym_master_slot] = ACTIONS(1871),
    [sym_get_pulse_time] = ACTIONS(1871),
    [sym_get_timer] = ACTIONS(1871),
    [sym_master_sn] = ACTIONS(1871),
    [sym_system_number] = ACTIONS(1871),
    [sym_button] = ACTIONS(1871),
    [sym_dv_channel] = ACTIONS(1871),
    [sym_data] = ACTIONS(1871),
    [sym_level] = ACTIONS(1871),
    [sym_timeline] = ACTIONS(1871),
    [sym_channel] = ACTIONS(1871),
    [sym_custom] = ACTIONS(1871),
    [sym_atoi] = ACTIONS(1871),
    [sym_atol] = ACTIONS(1871),
    [sym_atof] = ACTIONS(1871),
    [sym_itoa] = ACTIONS(1871),
    [sym_itohex] = ACTIONS(1871),
    [sym_hextoi] = ACTIONS(1871),
    [sym_ftoa] = ACTIONS(1871),
    [sym_find_string] = ACTIONS(1871),
    [sym_left_string] = ACTIONS(1871),
    [sym_right_string] = ACTIONS(1871),
    [sym_mid_string] = ACTIONS(1871),
    [sym_remove_string] = ACTIONS(1871),
    [sym_get_buffer_string] = ACTIONS(1871),
    [sym_get_buffer_char] = ACTIONS(1871),
    [sym_get_multi_buffer_string] = ACTIONS(1871),
    [sym_length_string] = ACTIONS(1871),
    [sym_max_length_string] = ACTIONS(1871),
    [sym_set_length_string] = ACTIONS(1871),
    [sym_compare_string] = ACTIONS(1871),
    [sym_upper_string] = ACTIONS(1871),
    [sym_lower_string] = ACTIONS(1871),
    [sym_get_last] = ACTIONS(1871),
    [sym_date_to_day] = ACTIONS(1871),
    [sym_date_to_month] = ACTIONS(1871),
    [sym_date_to_year] = ACTIONS(1871),
    [sym_time_to_hour] = ACTIONS(1871),
    [sym_time_to_minute] = ACTIONS(1871),
    [sym_time_to_second] = ACTIONS(1871),
    [sym_day_of_week] = ACTIONS(1871),
    [sym_set_pulse_time] = ACTIONS(1871),
    [sym_set_timer] = ACTIONS(1871),
    [sym_file_open] = ACTIONS(1871),
    [sym_file_close] = ACTIONS(1871),
    [sym_file_read] = ACTIONS(1871),
    [sym_file_read_line] = ACTIONS(1871),
    [sym_file_write] = ACTIONS(1871),
    [sym_file_write_line] = ACTIONS(1871),
    [sym_file_copy] = ACTIONS(1871),
    [sym_file_delete] = ACTIONS(1871),
    [sym_file_rename] = ACTIONS(1871),
    [sym_file_dir] = ACTIONS(1871),
    [sym_file_getdir] = ACTIONS(1871),
    [sym_file_setdir] = ACTIONS(1871),
    [sym_file_create_dir] = ACTIONS(1871),
    [sym_file_remove_dir] = ACTIONS(1871),
    [sym_file_seek] = ACTIONS(1871),
    [sym_length_array] = ACTIONS(1871),
    [sym_max_length_array] = ACTIONS(1871),
    [sym_set_length_array] = ACTIONS(1871),
    [sym_ip_client_open] = ACTIONS(1871),
    [sym_ip_client_close] = ACTIONS(1871),
    [sym_ip_server_open] = ACTIONS(1871),
    [sym_ip_server_close] = ACTIONS(1871),
    [sym_ip_bound_client_open] = ACTIONS(1871),
    [sym_ip_mc_server_open] = ACTIONS(1871),
    [sym_ip_set_option] = ACTIONS(1871),
    [sym_ssh_client_open] = ACTIONS(1871),
    [sym_ssh_client_close] = ACTIONS(1871),
    [sym_tls_client_open] = ACTIONS(1871),
    [sym_tls_client_close] = ACTIONS(1871),
    [sym_do_push] = ACTIONS(1871),
    [sym_do_push_timed] = ACTIONS(1871),
    [sym_do_release] = ACTIONS(1871),
    [sym_device_id] = ACTIONS(1871),
    [sym_device_info] = ACTIONS(1871),
    [sym_device_id_string] = ACTIONS(1871),
    [sym_device_standby] = ACTIONS(1871),
    [sym_device_wake] = ACTIONS(1871),
    [sym_timeline_create] = ACTIONS(1871),
    [sym_timeline_kill] = ACTIONS(1871),
    [sym_timeline_reload] = ACTIONS(1871),
    [sym_timeline_pause] = ACTIONS(1871),
    [sym_timeline_restart] = ACTIONS(1871),
    [sym_timeline_set] = ACTIONS(1871),
    [sym_timeline_get] = ACTIONS(1871),
    [sym_timeline_active] = ACTIONS(1871),
    [sym_timeline_dynamic_id] = ACTIONS(1871),
    [sym_variable_to_string] = ACTIONS(1871),
    [sym_string_to_variable] = ACTIONS(1871),
    [sym_length_variable_to_string] = ACTIONS(1871),
    [sym_type_cast] = ACTIONS(1871),
    [sym_variable_to_xml] = ACTIONS(1871),
    [sym_xml_to_variable] = ACTIONS(1871),
    [sym_length_variable_to_xml] = ACTIONS(1871),
    [sym_redirect_string] = ACTIONS(1871),
    [sym_random_number] = ACTIONS(1871),
    [sym_get_unique_id] = ACTIONS(1871),
    [sym_get_system_number] = ACTIONS(1871),
    [sym_set_system_number] = ACTIONS(1871),
    [sym_get_serial_number] = ACTIONS(1871),
    [sym_get_master_build] = ACTIONS(1871),
    [sym_set_validation_code] = ACTIONS(1871),
    [sym_reboot] = ACTIONS(1871),
    [sym_load_duet_module] = ACTIONS(1871),
    [sym_unload_duet_module] = ACTIONS(1871),
    [sym_duet_mem_size_set] = ACTIONS(1871),
    [sym_duet_mem_size_get] = ACTIONS(1871),
    [sym_dynamic_application_device] = ACTIONS(1871),
    [sym_dynamic_polled_port] = ACTIONS(1871),
    [sym_static_port_binding] = ACTIONS(1871),
    [sym_static_ip_binding] = ACTIONS(1871),
    [sym_get_dns_list] = ACTIONS(1871),
    [sym_set_dns_list] = ACTIONS(1871),
    [sym_get_ip_address] = ACTIONS(1871),
    [sym_set_ip_address] = ACTIONS(1871),
    [sym_get_url_list] = ACTIONS(1871),
    [sym_add_url_entry] = ACTIONS(1871),
    [sym_delete_url_entry] = ACTIONS(1871),
    [sym_abs_value] = ACTIONS(1871),
    [sym_max_value] = ACTIONS(1871),
    [sym_min_value] = ACTIONS(1871),
    [sym_exp_value] = ACTIONS(1871),
    [sym_log_value] = ACTIONS(1871),
    [sym_log10_value] = ACTIONS(1871),
    [sym_power_value] = ACTIONS(1871),
    [sym_sqrt_value] = ACTIONS(1871),
    [sym_raw_be] = ACTIONS(1871),
    [sym_raw_le] = ACTIONS(1871),
    [sym_internal_queue_size_set] = ACTIONS(1871),
    [sym_internal_queue_size_get] = ACTIONS(1871),
    [sym_internal_threshold_set] = ACTIONS(1871),
    [sym_internal_threshold_get] = ACTIONS(1871),
    [sym_rebuilt_event] = ACTIONS(1871),
    [sym_do_custom_event] = ACTIONS(1871),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1871),
    [sym_clkmgr_set_clk_source] = ACTIONS(1871),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1871),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1871),
    [sym_clkmgr_get_timezone] = ACTIONS(1871),
    [sym_clkmgr_set_timezone] = ACTIONS(1871),
    [sym_clkmgr_get_resync_period] = ACTIONS(1871),
    [sym_clkmgr_set_resync_period] = ACTIONS(1871),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1871),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1871),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1871),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1871),
    [sym_clkmgr_get_timeservers] = ACTIONS(1871),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1871),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1871),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1871),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1871),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1871),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1871),
    [sym_set_log_level] = ACTIONS(1871),
    [sym_get_log_level] = ACTIONS(1871),
    [sym_amx_log] = ACTIONS(1871),
    [sym_smtp_server_config_set] = ACTIONS(1871),
    [sym_smtp_server_config_get] = ACTIONS(1871),
    [sym_smtp_send] = ACTIONS(1871),
    [sym_validate_netlinx_account] = ACTIONS(1871),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1871),
    [sym_authenticate_certificate] = ACTIONS(1871),
    [sym_audit_netlinx_session_event] = ACTIONS(1871),
    [sym_audit_netlinx_generic_event] = ACTIONS(1871),
    [sym_get_available_flash_disk_space] = ACTIONS(1871),
    [sym_get_max_flash_disk_space] = ACTIONS(1871),
    [sym_data_create_feed] = ACTIONS(1871),
    [sym_data_delete_feed] = ACTIONS(1871),
    [sym_data_publish_feed] = ACTIONS(1871),
    [sym_data_get_published_feed] = ACTIONS(1871),
    [sym_data_add_record] = ACTIONS(1871),
    [sym_data_get_event_record] = ACTIONS(1871),
    [sym__wc_data_create_feed] = ACTIONS(1871),
    [sym__wc_data_add_record] = ACTIONS(1871),
    [sym__wc_data_get_event_record] = ACTIONS(1871),
    [sym_format] = ACTIONS(1871),
    [sym_astro_clock] = ACTIONS(1871),
    [sym_set_outdoor_temperature] = ACTIONS(1871),
    [sym_set_virtual_level_count] = ACTIONS(1871),
    [sym_set_virtual_channel_count] = ACTIONS(1871),
    [sym_set_virtual_port_count] = ACTIONS(1871),
    [sym__wc_encode_int] = ACTIONS(1871),
    [sym__wc_decode_int] = ACTIONS(1871),
    [sym__wc_int] = ACTIONS(1871),
    [sym__wc_to_ch_int] = ACTIONS(1871),
    [sym__ch_to_wc_int] = ACTIONS(1871),
    [sym__wc_find_string_int] = ACTIONS(1871),
    [sym__wc_left_string_int] = ACTIONS(1871),
    [sym__wc_lower_string_int] = ACTIONS(1871),
    [sym__wc_mid_string_int] = ACTIONS(1871),
    [sym__wc_remove_string_int] = ACTIONS(1871),
    [sym__wc_right_string_int] = ACTIONS(1871),
    [sym__wc_upper_string_int] = ACTIONS(1871),
    [sym__wc_compare_string_int] = ACTIONS(1871),
    [sym__wc_get_buffer_char_int] = ACTIONS(1871),
    [sym__wc_get_buffer_string_int] = ACTIONS(1871),
    [sym__wc_concat_string_int] = ACTIONS(1871),
    [sym__wc_file_open_int] = ACTIONS(1871),
    [sym__wc_file_close_int] = ACTIONS(1871),
    [sym__wc_file_read_int] = ACTIONS(1871),
    [sym__wc_file_read_line_int] = ACTIONS(1871),
    [sym__wc_file_write_int] = ACTIONS(1871),
    [sym__wc_file_write_line_int] = ACTIONS(1871),
    [aux_sym_preproc_include_token1] = ACTIONS(1871),
    [aux_sym_preproc_define_token1] = ACTIONS(1873),
    [aux_sym_preproc_warn_token1] = ACTIONS(1873),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1873),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1873),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1873),
    [anon_sym_LBRACK] = ACTIONS(1873),
    [anon_sym_LBRACE] = ACTIONS(1873),
    [anon_sym_LPAREN] = ACTIONS(1871),
    [anon_sym_BANG] = ACTIONS(1873),
    [anon_sym_TILDE] = ACTIONS(1873),
    [anon_sym_DASH] = ACTIONS(1871),
    [anon_sym_PLUS] = ACTIONS(1871),
    [anon_sym_DASH_DASH] = ACTIONS(1873),
    [anon_sym_PLUS_PLUS] = ACTIONS(1873),
    [anon_sym_DQUOTE] = ACTIONS(1873),
    [anon_sym_SQUOTE] = ACTIONS(1873),
    [sym_number_literal] = ACTIONS(1873),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(332)] = {
    [sym_identifier] = ACTIONS(2029),
    [sym_program_name_keyword] = ACTIONS(2029),
    [sym_module_name_keyword] = ACTIONS(2029),
    [sym_define_device_keyword] = ACTIONS(2029),
    [sym_define_combine_keyword] = ACTIONS(2029),
    [sym_define_constant_keyword] = ACTIONS(2029),
    [sym_define_type_keyword] = ACTIONS(2029),
    [sym_define_variable_keyword] = ACTIONS(2029),
    [sym_define_system_variable_keyword] = ACTIONS(2029),
    [sym_define_connect_level_keyword] = ACTIONS(2029),
    [sym_define_latching_keyword] = ACTIONS(2029),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2029),
    [sym_define_toggling_keyword] = ACTIONS(2029),
    [sym_define_call_keyword] = ACTIONS(2029),
    [sym_define_function_keyword] = ACTIONS(2029),
    [sym_define_library_function_keyword] = ACTIONS(2029),
    [sym_define_start_keyword] = ACTIONS(2029),
    [sym_define_event_keyword] = ACTIONS(2029),
    [sym_define_program_keyword] = ACTIONS(2029),
    [sym_define_module_keyword] = ACTIONS(2029),
    [sym_char_keyword] = ACTIONS(2029),
    [sym_widechar_keyword] = ACTIONS(2029),
    [sym_integer_keyword] = ACTIONS(2029),
    [sym_sinteger_keyword] = ACTIONS(2029),
    [sym_long_keyword] = ACTIONS(2029),
    [sym_slong_keyword] = ACTIONS(2029),
    [sym_float_keyword] = ACTIONS(2029),
    [sym_double_keyword] = ACTIONS(2029),
    [sym_variant_keyword] = ACTIONS(2029),
    [sym_variantarray_keyword] = ACTIONS(2029),
    [sym_local_var_keyword] = ACTIONS(2029),
    [sym_stack_var_keyword] = ACTIONS(2029),
    [sym_constant_keyword] = ACTIONS(2029),
    [sym_volatile_keyword] = ACTIONS(2029),
    [sym_non_volatile_keyword] = ACTIONS(2029),
    [sym_persistent_keyword] = ACTIONS(2029),
    [sym_dev_keyword] = ACTIONS(2029),
    [sym_devlev_keyword] = ACTIONS(2029),
    [sym_devchan_keyword] = ACTIONS(2029),
    [sym_if_keyword] = ACTIONS(2029),
    [sym_switch_keyword] = ACTIONS(2029),
    [sym_case_keyword] = ACTIONS(2029),
    [sym_default_keyword] = ACTIONS(2029),
    [sym_while_keyword] = ACTIONS(2029),
    [sym_for_keyword] = ACTIONS(2029),
    [sym_break_keyword] = ACTIONS(2029),
    [sym_continue_keyword] = ACTIONS(2029),
    [sym_return_keyword] = ACTIONS(2029),
    [sym_select_keyword] = ACTIONS(2029),
    [sym_active_keyword] = ACTIONS(2029),
    [sym_struct_keyword] = ACTIONS(2029),
    [sym_structure_keyword] = ACTIONS(2029),
    [sym_button_event_keyword] = ACTIONS(2029),
    [sym_channel_event_keyword] = ACTIONS(2029),
    [sym_level_event_keyword] = ACTIONS(2029),
    [sym_data_event_keyword] = ACTIONS(2029),
    [sym_timeline_event_keyword] = ACTIONS(2029),
    [sym_custom_event_keyword] = ACTIONS(2029),
    [sym_push_keyword] = ACTIONS(2029),
    [sym_release_keyword] = ACTIONS(2029),
    [aux_sym_on_keyword_token1] = ACTIONS(2029),
    [aux_sym_off_keyword_token1] = ACTIONS(2029),
    [sym_send_level_keyword] = ACTIONS(2029),
    [sym_send_string_keyword] = ACTIONS(2029),
    [sym_send_command_keyword] = ACTIONS(2029),
    [sym_clear_buffer_keyword] = ACTIONS(2029),
    [sym_create_buffer_keyword] = ACTIONS(2029),
    [sym_create_multi_buffer_keyword] = ACTIONS(2029),
    [sym_call_keyword] = ACTIONS(2029),
    [sym_system_call_keyword] = ACTIONS(2029),
    [sym_devchan_to_keyword] = ACTIONS(2029),
    [sym_devchan_min_to_keyword] = ACTIONS(2029),
    [sym_devchan_total_off_keyword] = ACTIONS(2029),
    [sym_devchan_pulse_keyword] = ACTIONS(2029),
    [sym_wait_keyword] = ACTIONS(2029),
    [sym_cancel_wait_keyword] = ACTIONS(2029),
    [sym_wait_until_keyword] = ACTIONS(2029),
    [sym_cancel_wait_until_keyword] = ACTIONS(2029),
    [sym_cancel_all_wait_keyword] = ACTIONS(2029),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2029),
    [sym_not] = ACTIONS(2029),
    [sym_bnot] = ACTIONS(2029),
    [sym___file__] = ACTIONS(2029),
    [sym___date__] = ACTIONS(2029),
    [sym___ldate__] = ACTIONS(2029),
    [sym___time__] = ACTIONS(2029),
    [sym___line__] = ACTIONS(2029),
    [sym___name__] = ACTIONS(2029),
    [sym___netlinx__] = ACTIONS(2029),
    [sym_netlinx_axi_version] = ACTIONS(2029),
    [sym_url_flg_tcp] = ACTIONS(2029),
    [sym_url_flg_accinfopresent] = ACTIONS(2029),
    [sym_url_flg_temp] = ACTIONS(2029),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2029),
    [sym_url_flg_stat_mask] = ACTIONS(2029),
    [sym_url_flg_stat_lookup] = ACTIONS(2029),
    [sym_url_flg_stat_connecting] = ACTIONS(2029),
    [sym_url_flg_stat_waiting] = ACTIONS(2029),
    [sym_url_flg_stat_connected] = ACTIONS(2029),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2029),
    [sym_first_virtual_device] = ACTIONS(2029),
    [sym_first_local_port] = ACTIONS(2029),
    [sym_dynamic_virtual_device] = ACTIONS(2029),
    [sym_timeline_once] = ACTIONS(2029),
    [sym_timeline_repeat] = ACTIONS(2029),
    [sym_timeline_absolute] = ACTIONS(2029),
    [sym_timeline_relative] = ACTIONS(2029),
    [sym_do_push_timed_infinite] = ACTIONS(2029),
    [sym_source_type_no_address] = ACTIONS(2029),
    [sym_source_type_neuron_id] = ACTIONS(2029),
    [sym_source_type_ip_address] = ACTIONS(2029),
    [sym_source_type_axlink] = ACTIONS(2029),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2029),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2029),
    [sym_source_type_ip_socket_address] = ACTIONS(2029),
    [sym_source_type_rs232] = ACTIONS(2029),
    [sym_source_type_internal] = ACTIONS(2029),
    [sym_source_type_mac_address] = ACTIONS(2029),
    [sym_source_type_ipv4_port] = ACTIONS(2029),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2029),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2029),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2029),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2029),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2029),
    [sym_true] = ACTIONS(2029),
    [sym_false] = ACTIONS(2029),
    [sym_file_read_only] = ACTIONS(2029),
    [sym_file_rw_new] = ACTIONS(2029),
    [sym_file_rw_append] = ACTIONS(2029),
    [sym_ip_tcp] = ACTIONS(2029),
    [sym_ip_udp] = ACTIONS(2029),
    [sym_ip_udp_2way] = ACTIONS(2029),
    [sym_xml_encode_types] = ACTIONS(2029),
    [sym_xml_encode_char_as_list] = ACTIONS(2029),
    [sym_xml_encode_le] = ACTIONS(2029),
    [sym_xml_decode_types] = ACTIONS(2029),
    [sym_xml_decode_no_preserve] = ACTIONS(2029),
    [sym_duet_dev_type_display_device] = ACTIONS(2029),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2029),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2029),
    [sym_duet_dev_type_io_device] = ACTIONS(2029),
    [sym_duet_dev_type_relay_device] = ACTIONS(2029),
    [sym_duet_dev_type_ups] = ACTIONS(2029),
    [sym_duet_dev_type_amplifier] = ACTIONS(2029),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2029),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2029),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2029),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2029),
    [sym_duet_dev_type_camera] = ACTIONS(2029),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2029),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2029),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2029),
    [sym_duet_dev_type_dss] = ACTIONS(2029),
    [sym_duet_dev_type_dvr] = ACTIONS(2029),
    [sym_duet_dev_type_disc_device] = ACTIONS(2029),
    [sym_duet_dev_type_document_camera] = ACTIONS(2029),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2029),
    [sym_duet_dev_type_hvac] = ACTIONS(2029),
    [sym_duet_dev_type_keypad] = ACTIONS(2029),
    [sym_duet_dev_type_light] = ACTIONS(2029),
    [sym_duet_dev_type_monitor] = ACTIONS(2029),
    [sym_duet_dev_type_motor] = ACTIONS(2029),
    [sym_duet_dev_type_multi_window] = ACTIONS(2029),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2029),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2029),
    [sym_duet_dev_type_receiver] = ACTIONS(2029),
    [sym_duet_dev_type_security_system] = ACTIONS(2029),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2029),
    [sym_duet_dev_type_settop_box] = ACTIONS(2029),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2029),
    [sym_duet_dev_type_switcher] = ACTIONS(2029),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2029),
    [sym_duet_dev_type_tv] = ACTIONS(2029),
    [sym_duet_dev_type_utility] = ACTIONS(2029),
    [sym_duet_dev_type_vcr] = ACTIONS(2029),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2029),
    [sym_duet_dev_type_video_processor] = ACTIONS(2029),
    [sym_duet_dev_type_video_projector] = ACTIONS(2029),
    [sym_duet_dev_type_video_wall] = ACTIONS(2029),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2029),
    [sym_duet_dev_type_weather] = ACTIONS(2029),
    [sym_duet_dev_not_polled] = ACTIONS(2029),
    [sym_duet_dev_polled] = ACTIONS(2029),
    [sym_clkmgr_mode_network] = ACTIONS(2029),
    [sym_clkmgr_mode_standalone] = ACTIONS(2029),
    [sym_normal_standby] = ACTIONS(2029),
    [sym_normal_wake] = ACTIONS(2029),
    [sym_ip_multicast_ttl_option] = ACTIONS(2029),
    [sym_ip_ttl_subnet] = ACTIONS(2029),
    [sym_ip_ttl_site] = ACTIONS(2029),
    [sym_ip_ttl_region] = ACTIONS(2029),
    [sym_ip_ttl_continent] = ACTIONS(2029),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2029),
    [sym_ip_nodelay_on] = ACTIONS(2029),
    [sym_ip_nodelay_off] = ACTIONS(2029),
    [sym_amx_error] = ACTIONS(2029),
    [sym_amx_warning] = ACTIONS(2029),
    [sym_amx_info] = ACTIONS(2029),
    [sym_amx_debug] = ACTIONS(2029),
    [sym_netlinx_login_success] = ACTIONS(2029),
    [sym_netlinx_login_fail] = ACTIONS(2029),
    [sym_netlinx_logout] = ACTIONS(2029),
    [sym_null_str] = ACTIONS(2029),
    [sym_smtp_address] = ACTIONS(2029),
    [sym_smtp_port_number] = ACTIONS(2029),
    [sym_smtp_username] = ACTIONS(2029),
    [sym_smtp_password] = ACTIONS(2029),
    [sym_smtp_require_tls] = ACTIONS(2029),
    [sym_smtp_from] = ACTIONS(2029),
    [sym_smtp_tls_true] = ACTIONS(2029),
    [sym_smtp_tls_false] = ACTIONS(2029),
    [sym_listview_on_row_select_event] = ACTIONS(2029),
    [sym_data_structure_unknown] = ACTIONS(2029),
    [sym_data_structure_datafeed] = ACTIONS(2029),
    [sym_data_structure_datarecordset] = ACTIONS(2029),
    [sym_data_structure_datarecord] = ACTIONS(2029),
    [sym_data_structure_datafield] = ACTIONS(2029),
    [sym_data_max_content_field_count] = ACTIONS(2029),
    [sym_data_max_metadata_field_count] = ACTIONS(2029),
    [sym_data_max_id_length] = ACTIONS(2029),
    [sym_data_max_name_length] = ACTIONS(2029),
    [sym_data_max_description_length] = ACTIONS(2029),
    [sym_data_max_type_length] = ACTIONS(2029),
    [sym_data_max_format_length] = ACTIONS(2029),
    [sym_data_max_source_length] = ACTIONS(2029),
    [sym_data_max_value_length] = ACTIONS(2029),
    [sym_data_max_label_length] = ACTIONS(2029),
    [sym_data_max_url_length] = ACTIONS(2029),
    [sym_max_last_login_info_length] = ACTIONS(2029),
    [sym_data_type_unknown] = ACTIONS(2029),
    [sym_data_type_string] = ACTIONS(2029),
    [sym_data_type_datetime] = ACTIONS(2029),
    [sym_data_type_date] = ACTIONS(2029),
    [sym_data_type_time] = ACTIONS(2029),
    [sym_data_type_image] = ACTIONS(2029),
    [sym_data_format_url] = ACTIONS(2029),
    [sym_data_format_phone] = ACTIONS(2029),
    [sym_data_format_email] = ACTIONS(2029),
    [sym_data_format_iso8601] = ACTIONS(2029),
    [sym_auditlog_priv] = ACTIONS(2029),
    [sym_configuration_priv] = ACTIONS(2029),
    [sym_deviceconfig_priv] = ACTIONS(2029),
    [sym_ftp_priv] = ACTIONS(2029),
    [sym_http_priv] = ACTIONS(2029),
    [sym_networkconfig_priv] = ACTIONS(2029),
    [sym_programport_priv] = ACTIONS(2029),
    [sym_remoteui_priv] = ACTIONS(2029),
    [sym_securitycontrol_priv] = ACTIONS(2029),
    [sym_softwaremanagement_priv] = ACTIONS(2029),
    [sym_terminal_priv] = ACTIONS(2029),
    [sym_tpadmin_priv] = ACTIONS(2029),
    [sym_user1_priv] = ACTIONS(2029),
    [sym_user2_priv] = ACTIONS(2029),
    [sym_user3_priv] = ACTIONS(2029),
    [sym_user4_priv] = ACTIONS(2029),
    [sym_usermanagement_priv] = ACTIONS(2029),
    [sym_valid_account] = ACTIONS(2029),
    [sym_err_name_invalid] = ACTIONS(2029),
    [sym_err_password_invalid] = ACTIONS(2029),
    [sym_err_invalid_account] = ACTIONS(2029),
    [sym_err_no_authorization] = ACTIONS(2029),
    [sym_err_invalid_parameter] = ACTIONS(2029),
    [sym_err_user_locked_out] = ACTIONS(2029),
    [sym_err_user_account_expired] = ACTIONS(2029),
    [sym_err_general_error] = ACTIONS(2029),
    [sym_err_authentication_error] = ACTIONS(2029),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2029),
    [sym_tls_validate_certificate] = ACTIONS(2029),
    [sym_tdata] = ACTIONS(2029),
    [sym_tchannel] = ACTIONS(2029),
    [sym_tlevel] = ACTIONS(2029),
    [sym_tbutton] = ACTIONS(2029),
    [sym_ttimeline] = ACTIONS(2029),
    [sym_tcustom] = ACTIONS(2029),
    [sym_url_struct] = ACTIONS(2029),
    [sym_dns_struct] = ACTIONS(2029),
    [sym_ip_address_struct] = ACTIONS(2029),
    [sym_dev_info_struct] = ACTIONS(2029),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2029),
    [sym_data_feed] = ACTIONS(2029),
    [sym_data_field] = ACTIONS(2029),
    [sym_data_record] = ACTIONS(2029),
    [sym_wc_data_feed] = ACTIONS(2029),
    [sym_wc_data_field] = ACTIONS(2029),
    [sym_wc_data_record] = ACTIONS(2029),
    [sym_last_login_info] = ACTIONS(2029),
    [sym____reserved___] = ACTIONS(2029),
    [sym_date] = ACTIONS(2029),
    [sym_day] = ACTIONS(2029),
    [sym_time] = ACTIONS(2029),
    [sym_ldate] = ACTIONS(2029),
    [sym_push_device] = ACTIONS(2029),
    [sym_push_channel] = ACTIONS(2029),
    [sym_push_devchan] = ACTIONS(2029),
    [sym_release_device] = ACTIONS(2029),
    [sym_release_channel] = ACTIONS(2029),
    [sym_release_devchan] = ACTIONS(2029),
    [sym_master_slot] = ACTIONS(2029),
    [sym_get_pulse_time] = ACTIONS(2029),
    [sym_get_timer] = ACTIONS(2029),
    [sym_master_sn] = ACTIONS(2029),
    [sym_system_number] = ACTIONS(2029),
    [sym_button] = ACTIONS(2029),
    [sym_dv_channel] = ACTIONS(2029),
    [sym_data] = ACTIONS(2029),
    [sym_level] = ACTIONS(2029),
    [sym_timeline] = ACTIONS(2029),
    [sym_channel] = ACTIONS(2029),
    [sym_custom] = ACTIONS(2029),
    [sym_atoi] = ACTIONS(2029),
    [sym_atol] = ACTIONS(2029),
    [sym_atof] = ACTIONS(2029),
    [sym_itoa] = ACTIONS(2029),
    [sym_itohex] = ACTIONS(2029),
    [sym_hextoi] = ACTIONS(2029),
    [sym_ftoa] = ACTIONS(2029),
    [sym_find_string] = ACTIONS(2029),
    [sym_left_string] = ACTIONS(2029),
    [sym_right_string] = ACTIONS(2029),
    [sym_mid_string] = ACTIONS(2029),
    [sym_remove_string] = ACTIONS(2029),
    [sym_get_buffer_string] = ACTIONS(2029),
    [sym_get_buffer_char] = ACTIONS(2029),
    [sym_get_multi_buffer_string] = ACTIONS(2029),
    [sym_length_string] = ACTIONS(2029),
    [sym_max_length_string] = ACTIONS(2029),
    [sym_set_length_string] = ACTIONS(2029),
    [sym_compare_string] = ACTIONS(2029),
    [sym_upper_string] = ACTIONS(2029),
    [sym_lower_string] = ACTIONS(2029),
    [sym_get_last] = ACTIONS(2029),
    [sym_date_to_day] = ACTIONS(2029),
    [sym_date_to_month] = ACTIONS(2029),
    [sym_date_to_year] = ACTIONS(2029),
    [sym_time_to_hour] = ACTIONS(2029),
    [sym_time_to_minute] = ACTIONS(2029),
    [sym_time_to_second] = ACTIONS(2029),
    [sym_day_of_week] = ACTIONS(2029),
    [sym_set_pulse_time] = ACTIONS(2029),
    [sym_set_timer] = ACTIONS(2029),
    [sym_file_open] = ACTIONS(2029),
    [sym_file_close] = ACTIONS(2029),
    [sym_file_read] = ACTIONS(2029),
    [sym_file_read_line] = ACTIONS(2029),
    [sym_file_write] = ACTIONS(2029),
    [sym_file_write_line] = ACTIONS(2029),
    [sym_file_copy] = ACTIONS(2029),
    [sym_file_delete] = ACTIONS(2029),
    [sym_file_rename] = ACTIONS(2029),
    [sym_file_dir] = ACTIONS(2029),
    [sym_file_getdir] = ACTIONS(2029),
    [sym_file_setdir] = ACTIONS(2029),
    [sym_file_create_dir] = ACTIONS(2029),
    [sym_file_remove_dir] = ACTIONS(2029),
    [sym_file_seek] = ACTIONS(2029),
    [sym_length_array] = ACTIONS(2029),
    [sym_max_length_array] = ACTIONS(2029),
    [sym_set_length_array] = ACTIONS(2029),
    [sym_ip_client_open] = ACTIONS(2029),
    [sym_ip_client_close] = ACTIONS(2029),
    [sym_ip_server_open] = ACTIONS(2029),
    [sym_ip_server_close] = ACTIONS(2029),
    [sym_ip_bound_client_open] = ACTIONS(2029),
    [sym_ip_mc_server_open] = ACTIONS(2029),
    [sym_ip_set_option] = ACTIONS(2029),
    [sym_ssh_client_open] = ACTIONS(2029),
    [sym_ssh_client_close] = ACTIONS(2029),
    [sym_tls_client_open] = ACTIONS(2029),
    [sym_tls_client_close] = ACTIONS(2029),
    [sym_do_push] = ACTIONS(2029),
    [sym_do_push_timed] = ACTIONS(2029),
    [sym_do_release] = ACTIONS(2029),
    [sym_device_id] = ACTIONS(2029),
    [sym_device_info] = ACTIONS(2029),
    [sym_device_id_string] = ACTIONS(2029),
    [sym_device_standby] = ACTIONS(2029),
    [sym_device_wake] = ACTIONS(2029),
    [sym_timeline_create] = ACTIONS(2029),
    [sym_timeline_kill] = ACTIONS(2029),
    [sym_timeline_reload] = ACTIONS(2029),
    [sym_timeline_pause] = ACTIONS(2029),
    [sym_timeline_restart] = ACTIONS(2029),
    [sym_timeline_set] = ACTIONS(2029),
    [sym_timeline_get] = ACTIONS(2029),
    [sym_timeline_active] = ACTIONS(2029),
    [sym_timeline_dynamic_id] = ACTIONS(2029),
    [sym_variable_to_string] = ACTIONS(2029),
    [sym_string_to_variable] = ACTIONS(2029),
    [sym_length_variable_to_string] = ACTIONS(2029),
    [sym_type_cast] = ACTIONS(2029),
    [sym_variable_to_xml] = ACTIONS(2029),
    [sym_xml_to_variable] = ACTIONS(2029),
    [sym_length_variable_to_xml] = ACTIONS(2029),
    [sym_redirect_string] = ACTIONS(2029),
    [sym_random_number] = ACTIONS(2029),
    [sym_get_unique_id] = ACTIONS(2029),
    [sym_get_system_number] = ACTIONS(2029),
    [sym_set_system_number] = ACTIONS(2029),
    [sym_get_serial_number] = ACTIONS(2029),
    [sym_get_master_build] = ACTIONS(2029),
    [sym_set_validation_code] = ACTIONS(2029),
    [sym_reboot] = ACTIONS(2029),
    [sym_load_duet_module] = ACTIONS(2029),
    [sym_unload_duet_module] = ACTIONS(2029),
    [sym_duet_mem_size_set] = ACTIONS(2029),
    [sym_duet_mem_size_get] = ACTIONS(2029),
    [sym_dynamic_application_device] = ACTIONS(2029),
    [sym_dynamic_polled_port] = ACTIONS(2029),
    [sym_static_port_binding] = ACTIONS(2029),
    [sym_static_ip_binding] = ACTIONS(2029),
    [sym_get_dns_list] = ACTIONS(2029),
    [sym_set_dns_list] = ACTIONS(2029),
    [sym_get_ip_address] = ACTIONS(2029),
    [sym_set_ip_address] = ACTIONS(2029),
    [sym_get_url_list] = ACTIONS(2029),
    [sym_add_url_entry] = ACTIONS(2029),
    [sym_delete_url_entry] = ACTIONS(2029),
    [sym_abs_value] = ACTIONS(2029),
    [sym_max_value] = ACTIONS(2029),
    [sym_min_value] = ACTIONS(2029),
    [sym_exp_value] = ACTIONS(2029),
    [sym_log_value] = ACTIONS(2029),
    [sym_log10_value] = ACTIONS(2029),
    [sym_power_value] = ACTIONS(2029),
    [sym_sqrt_value] = ACTIONS(2029),
    [sym_raw_be] = ACTIONS(2029),
    [sym_raw_le] = ACTIONS(2029),
    [sym_internal_queue_size_set] = ACTIONS(2029),
    [sym_internal_queue_size_get] = ACTIONS(2029),
    [sym_internal_threshold_set] = ACTIONS(2029),
    [sym_internal_threshold_get] = ACTIONS(2029),
    [sym_rebuilt_event] = ACTIONS(2029),
    [sym_do_custom_event] = ACTIONS(2029),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2029),
    [sym_clkmgr_set_clk_source] = ACTIONS(2029),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2029),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2029),
    [sym_clkmgr_get_timezone] = ACTIONS(2029),
    [sym_clkmgr_set_timezone] = ACTIONS(2029),
    [sym_clkmgr_get_resync_period] = ACTIONS(2029),
    [sym_clkmgr_set_resync_period] = ACTIONS(2029),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2029),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2029),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2029),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2029),
    [sym_clkmgr_get_timeservers] = ACTIONS(2029),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2029),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2029),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2029),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2029),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2029),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2029),
    [sym_set_log_level] = ACTIONS(2029),
    [sym_get_log_level] = ACTIONS(2029),
    [sym_amx_log] = ACTIONS(2029),
    [sym_smtp_server_config_set] = ACTIONS(2029),
    [sym_smtp_server_config_get] = ACTIONS(2029),
    [sym_smtp_send] = ACTIONS(2029),
    [sym_validate_netlinx_account] = ACTIONS(2029),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2029),
    [sym_authenticate_certificate] = ACTIONS(2029),
    [sym_audit_netlinx_session_event] = ACTIONS(2029),
    [sym_audit_netlinx_generic_event] = ACTIONS(2029),
    [sym_get_available_flash_disk_space] = ACTIONS(2029),
    [sym_get_max_flash_disk_space] = ACTIONS(2029),
    [sym_data_create_feed] = ACTIONS(2029),
    [sym_data_delete_feed] = ACTIONS(2029),
    [sym_data_publish_feed] = ACTIONS(2029),
    [sym_data_get_published_feed] = ACTIONS(2029),
    [sym_data_add_record] = ACTIONS(2029),
    [sym_data_get_event_record] = ACTIONS(2029),
    [sym__wc_data_create_feed] = ACTIONS(2029),
    [sym__wc_data_add_record] = ACTIONS(2029),
    [sym__wc_data_get_event_record] = ACTIONS(2029),
    [sym_format] = ACTIONS(2029),
    [sym_astro_clock] = ACTIONS(2029),
    [sym_set_outdoor_temperature] = ACTIONS(2029),
    [sym_set_virtual_level_count] = ACTIONS(2029),
    [sym_set_virtual_channel_count] = ACTIONS(2029),
    [sym_set_virtual_port_count] = ACTIONS(2029),
    [sym__wc_encode_int] = ACTIONS(2029),
    [sym__wc_decode_int] = ACTIONS(2029),
    [sym__wc_int] = ACTIONS(2029),
    [sym__wc_to_ch_int] = ACTIONS(2029),
    [sym__ch_to_wc_int] = ACTIONS(2029),
    [sym__wc_find_string_int] = ACTIONS(2029),
    [sym__wc_left_string_int] = ACTIONS(2029),
    [sym__wc_lower_string_int] = ACTIONS(2029),
    [sym__wc_mid_string_int] = ACTIONS(2029),
    [sym__wc_remove_string_int] = ACTIONS(2029),
    [sym__wc_right_string_int] = ACTIONS(2029),
    [sym__wc_upper_string_int] = ACTIONS(2029),
    [sym__wc_compare_string_int] = ACTIONS(2029),
    [sym__wc_get_buffer_char_int] = ACTIONS(2029),
    [sym__wc_get_buffer_string_int] = ACTIONS(2029),
    [sym__wc_concat_string_int] = ACTIONS(2029),
    [sym__wc_file_open_int] = ACTIONS(2029),
    [sym__wc_file_close_int] = ACTIONS(2029),
    [sym__wc_file_read_int] = ACTIONS(2029),
    [sym__wc_file_read_line_int] = ACTIONS(2029),
    [sym__wc_file_write_int] = ACTIONS(2029),
    [sym__wc_file_write_line_int] = ACTIONS(2029),
    [aux_sym_preproc_include_token1] = ACTIONS(2029),
    [aux_sym_preproc_define_token1] = ACTIONS(2031),
    [aux_sym_preproc_warn_token1] = ACTIONS(2031),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2031),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2031),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2031),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2031),
    [aux_sym_preproc_else_token1] = ACTIONS(2031),
    [anon_sym_LBRACK] = ACTIONS(2031),
    [anon_sym_LBRACE] = ACTIONS(2031),
    [anon_sym_LPAREN] = ACTIONS(2029),
    [anon_sym_BANG] = ACTIONS(2031),
    [anon_sym_TILDE] = ACTIONS(2031),
    [anon_sym_DASH] = ACTIONS(2029),
    [anon_sym_PLUS] = ACTIONS(2029),
    [anon_sym_DASH_DASH] = ACTIONS(2031),
    [anon_sym_PLUS_PLUS] = ACTIONS(2031),
    [anon_sym_DQUOTE] = ACTIONS(2031),
    [anon_sym_SQUOTE] = ACTIONS(2031),
    [sym_number_literal] = ACTIONS(2031),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(333)] = {
    [sym_identifier] = ACTIONS(2033),
    [sym_program_name_keyword] = ACTIONS(2033),
    [sym_module_name_keyword] = ACTIONS(2033),
    [sym_define_device_keyword] = ACTIONS(2033),
    [sym_define_combine_keyword] = ACTIONS(2033),
    [sym_define_constant_keyword] = ACTIONS(2033),
    [sym_define_type_keyword] = ACTIONS(2033),
    [sym_define_variable_keyword] = ACTIONS(2033),
    [sym_define_system_variable_keyword] = ACTIONS(2033),
    [sym_define_connect_level_keyword] = ACTIONS(2033),
    [sym_define_latching_keyword] = ACTIONS(2033),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2033),
    [sym_define_toggling_keyword] = ACTIONS(2033),
    [sym_define_call_keyword] = ACTIONS(2033),
    [sym_define_function_keyword] = ACTIONS(2033),
    [sym_define_library_function_keyword] = ACTIONS(2033),
    [sym_define_start_keyword] = ACTIONS(2033),
    [sym_define_event_keyword] = ACTIONS(2033),
    [sym_define_program_keyword] = ACTIONS(2033),
    [sym_define_module_keyword] = ACTIONS(2033),
    [sym_char_keyword] = ACTIONS(2033),
    [sym_widechar_keyword] = ACTIONS(2033),
    [sym_integer_keyword] = ACTIONS(2033),
    [sym_sinteger_keyword] = ACTIONS(2033),
    [sym_long_keyword] = ACTIONS(2033),
    [sym_slong_keyword] = ACTIONS(2033),
    [sym_float_keyword] = ACTIONS(2033),
    [sym_double_keyword] = ACTIONS(2033),
    [sym_variant_keyword] = ACTIONS(2033),
    [sym_variantarray_keyword] = ACTIONS(2033),
    [sym_local_var_keyword] = ACTIONS(2033),
    [sym_stack_var_keyword] = ACTIONS(2033),
    [sym_constant_keyword] = ACTIONS(2033),
    [sym_volatile_keyword] = ACTIONS(2033),
    [sym_non_volatile_keyword] = ACTIONS(2033),
    [sym_persistent_keyword] = ACTIONS(2033),
    [sym_dev_keyword] = ACTIONS(2033),
    [sym_devlev_keyword] = ACTIONS(2033),
    [sym_devchan_keyword] = ACTIONS(2033),
    [sym_if_keyword] = ACTIONS(2033),
    [sym_switch_keyword] = ACTIONS(2033),
    [sym_case_keyword] = ACTIONS(2033),
    [sym_default_keyword] = ACTIONS(2033),
    [sym_while_keyword] = ACTIONS(2033),
    [sym_for_keyword] = ACTIONS(2033),
    [sym_break_keyword] = ACTIONS(2033),
    [sym_continue_keyword] = ACTIONS(2033),
    [sym_return_keyword] = ACTIONS(2033),
    [sym_select_keyword] = ACTIONS(2033),
    [sym_active_keyword] = ACTIONS(2033),
    [sym_struct_keyword] = ACTIONS(2033),
    [sym_structure_keyword] = ACTIONS(2033),
    [sym_button_event_keyword] = ACTIONS(2033),
    [sym_channel_event_keyword] = ACTIONS(2033),
    [sym_level_event_keyword] = ACTIONS(2033),
    [sym_data_event_keyword] = ACTIONS(2033),
    [sym_timeline_event_keyword] = ACTIONS(2033),
    [sym_custom_event_keyword] = ACTIONS(2033),
    [sym_push_keyword] = ACTIONS(2033),
    [sym_release_keyword] = ACTIONS(2033),
    [aux_sym_on_keyword_token1] = ACTIONS(2033),
    [aux_sym_off_keyword_token1] = ACTIONS(2033),
    [sym_send_level_keyword] = ACTIONS(2033),
    [sym_send_string_keyword] = ACTIONS(2033),
    [sym_send_command_keyword] = ACTIONS(2033),
    [sym_clear_buffer_keyword] = ACTIONS(2033),
    [sym_create_buffer_keyword] = ACTIONS(2033),
    [sym_create_multi_buffer_keyword] = ACTIONS(2033),
    [sym_call_keyword] = ACTIONS(2033),
    [sym_system_call_keyword] = ACTIONS(2033),
    [sym_devchan_to_keyword] = ACTIONS(2033),
    [sym_devchan_min_to_keyword] = ACTIONS(2033),
    [sym_devchan_total_off_keyword] = ACTIONS(2033),
    [sym_devchan_pulse_keyword] = ACTIONS(2033),
    [sym_wait_keyword] = ACTIONS(2033),
    [sym_cancel_wait_keyword] = ACTIONS(2033),
    [sym_wait_until_keyword] = ACTIONS(2033),
    [sym_cancel_wait_until_keyword] = ACTIONS(2033),
    [sym_cancel_all_wait_keyword] = ACTIONS(2033),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2033),
    [sym_not] = ACTIONS(2033),
    [sym_bnot] = ACTIONS(2033),
    [sym___file__] = ACTIONS(2033),
    [sym___date__] = ACTIONS(2033),
    [sym___ldate__] = ACTIONS(2033),
    [sym___time__] = ACTIONS(2033),
    [sym___line__] = ACTIONS(2033),
    [sym___name__] = ACTIONS(2033),
    [sym___netlinx__] = ACTIONS(2033),
    [sym_netlinx_axi_version] = ACTIONS(2033),
    [sym_url_flg_tcp] = ACTIONS(2033),
    [sym_url_flg_accinfopresent] = ACTIONS(2033),
    [sym_url_flg_temp] = ACTIONS(2033),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2033),
    [sym_url_flg_stat_mask] = ACTIONS(2033),
    [sym_url_flg_stat_lookup] = ACTIONS(2033),
    [sym_url_flg_stat_connecting] = ACTIONS(2033),
    [sym_url_flg_stat_waiting] = ACTIONS(2033),
    [sym_url_flg_stat_connected] = ACTIONS(2033),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2033),
    [sym_first_virtual_device] = ACTIONS(2033),
    [sym_first_local_port] = ACTIONS(2033),
    [sym_dynamic_virtual_device] = ACTIONS(2033),
    [sym_timeline_once] = ACTIONS(2033),
    [sym_timeline_repeat] = ACTIONS(2033),
    [sym_timeline_absolute] = ACTIONS(2033),
    [sym_timeline_relative] = ACTIONS(2033),
    [sym_do_push_timed_infinite] = ACTIONS(2033),
    [sym_source_type_no_address] = ACTIONS(2033),
    [sym_source_type_neuron_id] = ACTIONS(2033),
    [sym_source_type_ip_address] = ACTIONS(2033),
    [sym_source_type_axlink] = ACTIONS(2033),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2033),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2033),
    [sym_source_type_ip_socket_address] = ACTIONS(2033),
    [sym_source_type_rs232] = ACTIONS(2033),
    [sym_source_type_internal] = ACTIONS(2033),
    [sym_source_type_mac_address] = ACTIONS(2033),
    [sym_source_type_ipv4_port] = ACTIONS(2033),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2033),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2033),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2033),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2033),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2033),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2033),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2033),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2033),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2033),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2033),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2033),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2033),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2033),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2033),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2033),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2033),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2033),
    [sym_true] = ACTIONS(2033),
    [sym_false] = ACTIONS(2033),
    [sym_file_read_only] = ACTIONS(2033),
    [sym_file_rw_new] = ACTIONS(2033),
    [sym_file_rw_append] = ACTIONS(2033),
    [sym_ip_tcp] = ACTIONS(2033),
    [sym_ip_udp] = ACTIONS(2033),
    [sym_ip_udp_2way] = ACTIONS(2033),
    [sym_xml_encode_types] = ACTIONS(2033),
    [sym_xml_encode_char_as_list] = ACTIONS(2033),
    [sym_xml_encode_le] = ACTIONS(2033),
    [sym_xml_decode_types] = ACTIONS(2033),
    [sym_xml_decode_no_preserve] = ACTIONS(2033),
    [sym_duet_dev_type_display_device] = ACTIONS(2033),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2033),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2033),
    [sym_duet_dev_type_io_device] = ACTIONS(2033),
    [sym_duet_dev_type_relay_device] = ACTIONS(2033),
    [sym_duet_dev_type_ups] = ACTIONS(2033),
    [sym_duet_dev_type_amplifier] = ACTIONS(2033),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2033),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2033),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2033),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2033),
    [sym_duet_dev_type_camera] = ACTIONS(2033),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2033),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2033),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2033),
    [sym_duet_dev_type_dss] = ACTIONS(2033),
    [sym_duet_dev_type_dvr] = ACTIONS(2033),
    [sym_duet_dev_type_disc_device] = ACTIONS(2033),
    [sym_duet_dev_type_document_camera] = ACTIONS(2033),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2033),
    [sym_duet_dev_type_hvac] = ACTIONS(2033),
    [sym_duet_dev_type_keypad] = ACTIONS(2033),
    [sym_duet_dev_type_light] = ACTIONS(2033),
    [sym_duet_dev_type_monitor] = ACTIONS(2033),
    [sym_duet_dev_type_motor] = ACTIONS(2033),
    [sym_duet_dev_type_multi_window] = ACTIONS(2033),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2033),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2033),
    [sym_duet_dev_type_receiver] = ACTIONS(2033),
    [sym_duet_dev_type_security_system] = ACTIONS(2033),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2033),
    [sym_duet_dev_type_settop_box] = ACTIONS(2033),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2033),
    [sym_duet_dev_type_switcher] = ACTIONS(2033),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2033),
    [sym_duet_dev_type_tv] = ACTIONS(2033),
    [sym_duet_dev_type_utility] = ACTIONS(2033),
    [sym_duet_dev_type_vcr] = ACTIONS(2033),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2033),
    [sym_duet_dev_type_video_processor] = ACTIONS(2033),
    [sym_duet_dev_type_video_projector] = ACTIONS(2033),
    [sym_duet_dev_type_video_wall] = ACTIONS(2033),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2033),
    [sym_duet_dev_type_weather] = ACTIONS(2033),
    [sym_duet_dev_not_polled] = ACTIONS(2033),
    [sym_duet_dev_polled] = ACTIONS(2033),
    [sym_clkmgr_mode_network] = ACTIONS(2033),
    [sym_clkmgr_mode_standalone] = ACTIONS(2033),
    [sym_normal_standby] = ACTIONS(2033),
    [sym_normal_wake] = ACTIONS(2033),
    [sym_ip_multicast_ttl_option] = ACTIONS(2033),
    [sym_ip_ttl_subnet] = ACTIONS(2033),
    [sym_ip_ttl_site] = ACTIONS(2033),
    [sym_ip_ttl_region] = ACTIONS(2033),
    [sym_ip_ttl_continent] = ACTIONS(2033),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2033),
    [sym_ip_nodelay_on] = ACTIONS(2033),
    [sym_ip_nodelay_off] = ACTIONS(2033),
    [sym_amx_error] = ACTIONS(2033),
    [sym_amx_warning] = ACTIONS(2033),
    [sym_amx_info] = ACTIONS(2033),
    [sym_amx_debug] = ACTIONS(2033),
    [sym_netlinx_login_success] = ACTIONS(2033),
    [sym_netlinx_login_fail] = ACTIONS(2033),
    [sym_netlinx_logout] = ACTIONS(2033),
    [sym_null_str] = ACTIONS(2033),
    [sym_smtp_address] = ACTIONS(2033),
    [sym_smtp_port_number] = ACTIONS(2033),
    [sym_smtp_username] = ACTIONS(2033),
    [sym_smtp_password] = ACTIONS(2033),
    [sym_smtp_require_tls] = ACTIONS(2033),
    [sym_smtp_from] = ACTIONS(2033),
    [sym_smtp_tls_true] = ACTIONS(2033),
    [sym_smtp_tls_false] = ACTIONS(2033),
    [sym_listview_on_row_select_event] = ACTIONS(2033),
    [sym_data_structure_unknown] = ACTIONS(2033),
    [sym_data_structure_datafeed] = ACTIONS(2033),
    [sym_data_structure_datarecordset] = ACTIONS(2033),
    [sym_data_structure_datarecord] = ACTIONS(2033),
    [sym_data_structure_datafield] = ACTIONS(2033),
    [sym_data_max_content_field_count] = ACTIONS(2033),
    [sym_data_max_metadata_field_count] = ACTIONS(2033),
    [sym_data_max_id_length] = ACTIONS(2033),
    [sym_data_max_name_length] = ACTIONS(2033),
    [sym_data_max_description_length] = ACTIONS(2033),
    [sym_data_max_type_length] = ACTIONS(2033),
    [sym_data_max_format_length] = ACTIONS(2033),
    [sym_data_max_source_length] = ACTIONS(2033),
    [sym_data_max_value_length] = ACTIONS(2033),
    [sym_data_max_label_length] = ACTIONS(2033),
    [sym_data_max_url_length] = ACTIONS(2033),
    [sym_max_last_login_info_length] = ACTIONS(2033),
    [sym_data_type_unknown] = ACTIONS(2033),
    [sym_data_type_string] = ACTIONS(2033),
    [sym_data_type_datetime] = ACTIONS(2033),
    [sym_data_type_date] = ACTIONS(2033),
    [sym_data_type_time] = ACTIONS(2033),
    [sym_data_type_image] = ACTIONS(2033),
    [sym_data_format_url] = ACTIONS(2033),
    [sym_data_format_phone] = ACTIONS(2033),
    [sym_data_format_email] = ACTIONS(2033),
    [sym_data_format_iso8601] = ACTIONS(2033),
    [sym_auditlog_priv] = ACTIONS(2033),
    [sym_configuration_priv] = ACTIONS(2033),
    [sym_deviceconfig_priv] = ACTIONS(2033),
    [sym_ftp_priv] = ACTIONS(2033),
    [sym_http_priv] = ACTIONS(2033),
    [sym_networkconfig_priv] = ACTIONS(2033),
    [sym_programport_priv] = ACTIONS(2033),
    [sym_remoteui_priv] = ACTIONS(2033),
    [sym_securitycontrol_priv] = ACTIONS(2033),
    [sym_softwaremanagement_priv] = ACTIONS(2033),
    [sym_terminal_priv] = ACTIONS(2033),
    [sym_tpadmin_priv] = ACTIONS(2033),
    [sym_user1_priv] = ACTIONS(2033),
    [sym_user2_priv] = ACTIONS(2033),
    [sym_user3_priv] = ACTIONS(2033),
    [sym_user4_priv] = ACTIONS(2033),
    [sym_usermanagement_priv] = ACTIONS(2033),
    [sym_valid_account] = ACTIONS(2033),
    [sym_err_name_invalid] = ACTIONS(2033),
    [sym_err_password_invalid] = ACTIONS(2033),
    [sym_err_invalid_account] = ACTIONS(2033),
    [sym_err_no_authorization] = ACTIONS(2033),
    [sym_err_invalid_parameter] = ACTIONS(2033),
    [sym_err_user_locked_out] = ACTIONS(2033),
    [sym_err_user_account_expired] = ACTIONS(2033),
    [sym_err_general_error] = ACTIONS(2033),
    [sym_err_authentication_error] = ACTIONS(2033),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2033),
    [sym_tls_validate_certificate] = ACTIONS(2033),
    [sym_tdata] = ACTIONS(2033),
    [sym_tchannel] = ACTIONS(2033),
    [sym_tlevel] = ACTIONS(2033),
    [sym_tbutton] = ACTIONS(2033),
    [sym_ttimeline] = ACTIONS(2033),
    [sym_tcustom] = ACTIONS(2033),
    [sym_url_struct] = ACTIONS(2033),
    [sym_dns_struct] = ACTIONS(2033),
    [sym_ip_address_struct] = ACTIONS(2033),
    [sym_dev_info_struct] = ACTIONS(2033),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2033),
    [sym_data_feed] = ACTIONS(2033),
    [sym_data_field] = ACTIONS(2033),
    [sym_data_record] = ACTIONS(2033),
    [sym_wc_data_feed] = ACTIONS(2033),
    [sym_wc_data_field] = ACTIONS(2033),
    [sym_wc_data_record] = ACTIONS(2033),
    [sym_last_login_info] = ACTIONS(2033),
    [sym____reserved___] = ACTIONS(2033),
    [sym_date] = ACTIONS(2033),
    [sym_day] = ACTIONS(2033),
    [sym_time] = ACTIONS(2033),
    [sym_ldate] = ACTIONS(2033),
    [sym_push_device] = ACTIONS(2033),
    [sym_push_channel] = ACTIONS(2033),
    [sym_push_devchan] = ACTIONS(2033),
    [sym_release_device] = ACTIONS(2033),
    [sym_release_channel] = ACTIONS(2033),
    [sym_release_devchan] = ACTIONS(2033),
    [sym_master_slot] = ACTIONS(2033),
    [sym_get_pulse_time] = ACTIONS(2033),
    [sym_get_timer] = ACTIONS(2033),
    [sym_master_sn] = ACTIONS(2033),
    [sym_system_number] = ACTIONS(2033),
    [sym_button] = ACTIONS(2033),
    [sym_dv_channel] = ACTIONS(2033),
    [sym_data] = ACTIONS(2033),
    [sym_level] = ACTIONS(2033),
    [sym_timeline] = ACTIONS(2033),
    [sym_channel] = ACTIONS(2033),
    [sym_custom] = ACTIONS(2033),
    [sym_atoi] = ACTIONS(2033),
    [sym_atol] = ACTIONS(2033),
    [sym_atof] = ACTIONS(2033),
    [sym_itoa] = ACTIONS(2033),
    [sym_itohex] = ACTIONS(2033),
    [sym_hextoi] = ACTIONS(2033),
    [sym_ftoa] = ACTIONS(2033),
    [sym_find_string] = ACTIONS(2033),
    [sym_left_string] = ACTIONS(2033),
    [sym_right_string] = ACTIONS(2033),
    [sym_mid_string] = ACTIONS(2033),
    [sym_remove_string] = ACTIONS(2033),
    [sym_get_buffer_string] = ACTIONS(2033),
    [sym_get_buffer_char] = ACTIONS(2033),
    [sym_get_multi_buffer_string] = ACTIONS(2033),
    [sym_length_string] = ACTIONS(2033),
    [sym_max_length_string] = ACTIONS(2033),
    [sym_set_length_string] = ACTIONS(2033),
    [sym_compare_string] = ACTIONS(2033),
    [sym_upper_string] = ACTIONS(2033),
    [sym_lower_string] = ACTIONS(2033),
    [sym_get_last] = ACTIONS(2033),
    [sym_date_to_day] = ACTIONS(2033),
    [sym_date_to_month] = ACTIONS(2033),
    [sym_date_to_year] = ACTIONS(2033),
    [sym_time_to_hour] = ACTIONS(2033),
    [sym_time_to_minute] = ACTIONS(2033),
    [sym_time_to_second] = ACTIONS(2033),
    [sym_day_of_week] = ACTIONS(2033),
    [sym_set_pulse_time] = ACTIONS(2033),
    [sym_set_timer] = ACTIONS(2033),
    [sym_file_open] = ACTIONS(2033),
    [sym_file_close] = ACTIONS(2033),
    [sym_file_read] = ACTIONS(2033),
    [sym_file_read_line] = ACTIONS(2033),
    [sym_file_write] = ACTIONS(2033),
    [sym_file_write_line] = ACTIONS(2033),
    [sym_file_copy] = ACTIONS(2033),
    [sym_file_delete] = ACTIONS(2033),
    [sym_file_rename] = ACTIONS(2033),
    [sym_file_dir] = ACTIONS(2033),
    [sym_file_getdir] = ACTIONS(2033),
    [sym_file_setdir] = ACTIONS(2033),
    [sym_file_create_dir] = ACTIONS(2033),
    [sym_file_remove_dir] = ACTIONS(2033),
    [sym_file_seek] = ACTIONS(2033),
    [sym_length_array] = ACTIONS(2033),
    [sym_max_length_array] = ACTIONS(2033),
    [sym_set_length_array] = ACTIONS(2033),
    [sym_ip_client_open] = ACTIONS(2033),
    [sym_ip_client_close] = ACTIONS(2033),
    [sym_ip_server_open] = ACTIONS(2033),
    [sym_ip_server_close] = ACTIONS(2033),
    [sym_ip_bound_client_open] = ACTIONS(2033),
    [sym_ip_mc_server_open] = ACTIONS(2033),
    [sym_ip_set_option] = ACTIONS(2033),
    [sym_ssh_client_open] = ACTIONS(2033),
    [sym_ssh_client_close] = ACTIONS(2033),
    [sym_tls_client_open] = ACTIONS(2033),
    [sym_tls_client_close] = ACTIONS(2033),
    [sym_do_push] = ACTIONS(2033),
    [sym_do_push_timed] = ACTIONS(2033),
    [sym_do_release] = ACTIONS(2033),
    [sym_device_id] = ACTIONS(2033),
    [sym_device_info] = ACTIONS(2033),
    [sym_device_id_string] = ACTIONS(2033),
    [sym_device_standby] = ACTIONS(2033),
    [sym_device_wake] = ACTIONS(2033),
    [sym_timeline_create] = ACTIONS(2033),
    [sym_timeline_kill] = ACTIONS(2033),
    [sym_timeline_reload] = ACTIONS(2033),
    [sym_timeline_pause] = ACTIONS(2033),
    [sym_timeline_restart] = ACTIONS(2033),
    [sym_timeline_set] = ACTIONS(2033),
    [sym_timeline_get] = ACTIONS(2033),
    [sym_timeline_active] = ACTIONS(2033),
    [sym_timeline_dynamic_id] = ACTIONS(2033),
    [sym_variable_to_string] = ACTIONS(2033),
    [sym_string_to_variable] = ACTIONS(2033),
    [sym_length_variable_to_string] = ACTIONS(2033),
    [sym_type_cast] = ACTIONS(2033),
    [sym_variable_to_xml] = ACTIONS(2033),
    [sym_xml_to_variable] = ACTIONS(2033),
    [sym_length_variable_to_xml] = ACTIONS(2033),
    [sym_redirect_string] = ACTIONS(2033),
    [sym_random_number] = ACTIONS(2033),
    [sym_get_unique_id] = ACTIONS(2033),
    [sym_get_system_number] = ACTIONS(2033),
    [sym_set_system_number] = ACTIONS(2033),
    [sym_get_serial_number] = ACTIONS(2033),
    [sym_get_master_build] = ACTIONS(2033),
    [sym_set_validation_code] = ACTIONS(2033),
    [sym_reboot] = ACTIONS(2033),
    [sym_load_duet_module] = ACTIONS(2033),
    [sym_unload_duet_module] = ACTIONS(2033),
    [sym_duet_mem_size_set] = ACTIONS(2033),
    [sym_duet_mem_size_get] = ACTIONS(2033),
    [sym_dynamic_application_device] = ACTIONS(2033),
    [sym_dynamic_polled_port] = ACTIONS(2033),
    [sym_static_port_binding] = ACTIONS(2033),
    [sym_static_ip_binding] = ACTIONS(2033),
    [sym_get_dns_list] = ACTIONS(2033),
    [sym_set_dns_list] = ACTIONS(2033),
    [sym_get_ip_address] = ACTIONS(2033),
    [sym_set_ip_address] = ACTIONS(2033),
    [sym_get_url_list] = ACTIONS(2033),
    [sym_add_url_entry] = ACTIONS(2033),
    [sym_delete_url_entry] = ACTIONS(2033),
    [sym_abs_value] = ACTIONS(2033),
    [sym_max_value] = ACTIONS(2033),
    [sym_min_value] = ACTIONS(2033),
    [sym_exp_value] = ACTIONS(2033),
    [sym_log_value] = ACTIONS(2033),
    [sym_log10_value] = ACTIONS(2033),
    [sym_power_value] = ACTIONS(2033),
    [sym_sqrt_value] = ACTIONS(2033),
    [sym_raw_be] = ACTIONS(2033),
    [sym_raw_le] = ACTIONS(2033),
    [sym_internal_queue_size_set] = ACTIONS(2033),
    [sym_internal_queue_size_get] = ACTIONS(2033),
    [sym_internal_threshold_set] = ACTIONS(2033),
    [sym_internal_threshold_get] = ACTIONS(2033),
    [sym_rebuilt_event] = ACTIONS(2033),
    [sym_do_custom_event] = ACTIONS(2033),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2033),
    [sym_clkmgr_set_clk_source] = ACTIONS(2033),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2033),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2033),
    [sym_clkmgr_get_timezone] = ACTIONS(2033),
    [sym_clkmgr_set_timezone] = ACTIONS(2033),
    [sym_clkmgr_get_resync_period] = ACTIONS(2033),
    [sym_clkmgr_set_resync_period] = ACTIONS(2033),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2033),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2033),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2033),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2033),
    [sym_clkmgr_get_timeservers] = ACTIONS(2033),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2033),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2033),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2033),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2033),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2033),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2033),
    [sym_set_log_level] = ACTIONS(2033),
    [sym_get_log_level] = ACTIONS(2033),
    [sym_amx_log] = ACTIONS(2033),
    [sym_smtp_server_config_set] = ACTIONS(2033),
    [sym_smtp_server_config_get] = ACTIONS(2033),
    [sym_smtp_send] = ACTIONS(2033),
    [sym_validate_netlinx_account] = ACTIONS(2033),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2033),
    [sym_authenticate_certificate] = ACTIONS(2033),
    [sym_audit_netlinx_session_event] = ACTIONS(2033),
    [sym_audit_netlinx_generic_event] = ACTIONS(2033),
    [sym_get_available_flash_disk_space] = ACTIONS(2033),
    [sym_get_max_flash_disk_space] = ACTIONS(2033),
    [sym_data_create_feed] = ACTIONS(2033),
    [sym_data_delete_feed] = ACTIONS(2033),
    [sym_data_publish_feed] = ACTIONS(2033),
    [sym_data_get_published_feed] = ACTIONS(2033),
    [sym_data_add_record] = ACTIONS(2033),
    [sym_data_get_event_record] = ACTIONS(2033),
    [sym__wc_data_create_feed] = ACTIONS(2033),
    [sym__wc_data_add_record] = ACTIONS(2033),
    [sym__wc_data_get_event_record] = ACTIONS(2033),
    [sym_format] = ACTIONS(2033),
    [sym_astro_clock] = ACTIONS(2033),
    [sym_set_outdoor_temperature] = ACTIONS(2033),
    [sym_set_virtual_level_count] = ACTIONS(2033),
    [sym_set_virtual_channel_count] = ACTIONS(2033),
    [sym_set_virtual_port_count] = ACTIONS(2033),
    [sym__wc_encode_int] = ACTIONS(2033),
    [sym__wc_decode_int] = ACTIONS(2033),
    [sym__wc_int] = ACTIONS(2033),
    [sym__wc_to_ch_int] = ACTIONS(2033),
    [sym__ch_to_wc_int] = ACTIONS(2033),
    [sym__wc_find_string_int] = ACTIONS(2033),
    [sym__wc_left_string_int] = ACTIONS(2033),
    [sym__wc_lower_string_int] = ACTIONS(2033),
    [sym__wc_mid_string_int] = ACTIONS(2033),
    [sym__wc_remove_string_int] = ACTIONS(2033),
    [sym__wc_right_string_int] = ACTIONS(2033),
    [sym__wc_upper_string_int] = ACTIONS(2033),
    [sym__wc_compare_string_int] = ACTIONS(2033),
    [sym__wc_get_buffer_char_int] = ACTIONS(2033),
    [sym__wc_get_buffer_string_int] = ACTIONS(2033),
    [sym__wc_concat_string_int] = ACTIONS(2033),
    [sym__wc_file_open_int] = ACTIONS(2033),
    [sym__wc_file_close_int] = ACTIONS(2033),
    [sym__wc_file_read_int] = ACTIONS(2033),
    [sym__wc_file_read_line_int] = ACTIONS(2033),
    [sym__wc_file_write_int] = ACTIONS(2033),
    [sym__wc_file_write_line_int] = ACTIONS(2033),
    [aux_sym_preproc_include_token1] = ACTIONS(2033),
    [aux_sym_preproc_define_token1] = ACTIONS(2035),
    [aux_sym_preproc_warn_token1] = ACTIONS(2035),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2035),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2035),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2035),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2035),
    [aux_sym_preproc_else_token1] = ACTIONS(2035),
    [anon_sym_LBRACK] = ACTIONS(2035),
    [anon_sym_LBRACE] = ACTIONS(2035),
    [anon_sym_LPAREN] = ACTIONS(2033),
    [anon_sym_BANG] = ACTIONS(2035),
    [anon_sym_TILDE] = ACTIONS(2035),
    [anon_sym_DASH] = ACTIONS(2033),
    [anon_sym_PLUS] = ACTIONS(2033),
    [anon_sym_DASH_DASH] = ACTIONS(2035),
    [anon_sym_PLUS_PLUS] = ACTIONS(2035),
    [anon_sym_DQUOTE] = ACTIONS(2035),
    [anon_sym_SQUOTE] = ACTIONS(2035),
    [sym_number_literal] = ACTIONS(2035),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(334)] = {
    [ts_builtin_sym_end] = ACTIONS(1733),
    [sym_identifier] = ACTIONS(1731),
    [sym_program_name_keyword] = ACTIONS(1731),
    [sym_module_name_keyword] = ACTIONS(1731),
    [sym_define_device_keyword] = ACTIONS(1731),
    [sym_define_combine_keyword] = ACTIONS(1731),
    [sym_define_constant_keyword] = ACTIONS(1731),
    [sym_define_type_keyword] = ACTIONS(1731),
    [sym_define_variable_keyword] = ACTIONS(1731),
    [sym_define_system_variable_keyword] = ACTIONS(1731),
    [sym_define_connect_level_keyword] = ACTIONS(1731),
    [sym_define_latching_keyword] = ACTIONS(1731),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1731),
    [sym_define_toggling_keyword] = ACTIONS(1731),
    [sym_define_call_keyword] = ACTIONS(1731),
    [sym_define_function_keyword] = ACTIONS(1731),
    [sym_define_library_function_keyword] = ACTIONS(1731),
    [sym_define_start_keyword] = ACTIONS(1731),
    [sym_define_event_keyword] = ACTIONS(1731),
    [sym_define_program_keyword] = ACTIONS(1731),
    [sym_define_module_keyword] = ACTIONS(1731),
    [sym_char_keyword] = ACTIONS(1731),
    [sym_widechar_keyword] = ACTIONS(1731),
    [sym_integer_keyword] = ACTIONS(1731),
    [sym_sinteger_keyword] = ACTIONS(1731),
    [sym_long_keyword] = ACTIONS(1731),
    [sym_slong_keyword] = ACTIONS(1731),
    [sym_float_keyword] = ACTIONS(1731),
    [sym_double_keyword] = ACTIONS(1731),
    [sym_variant_keyword] = ACTIONS(1731),
    [sym_variantarray_keyword] = ACTIONS(1731),
    [sym_local_var_keyword] = ACTIONS(1731),
    [sym_stack_var_keyword] = ACTIONS(1731),
    [sym_constant_keyword] = ACTIONS(1731),
    [sym_volatile_keyword] = ACTIONS(1731),
    [sym_non_volatile_keyword] = ACTIONS(1731),
    [sym_persistent_keyword] = ACTIONS(1731),
    [sym_dev_keyword] = ACTIONS(1731),
    [sym_devlev_keyword] = ACTIONS(1731),
    [sym_devchan_keyword] = ACTIONS(1731),
    [sym_if_keyword] = ACTIONS(1731),
    [sym_else_keyword] = ACTIONS(1731),
    [sym_switch_keyword] = ACTIONS(1731),
    [sym_case_keyword] = ACTIONS(1731),
    [sym_default_keyword] = ACTIONS(1731),
    [sym_while_keyword] = ACTIONS(1731),
    [sym_for_keyword] = ACTIONS(1731),
    [sym_break_keyword] = ACTIONS(1731),
    [sym_continue_keyword] = ACTIONS(1731),
    [sym_return_keyword] = ACTIONS(1731),
    [sym_select_keyword] = ACTIONS(1731),
    [sym_active_keyword] = ACTIONS(1731),
    [sym_struct_keyword] = ACTIONS(1731),
    [sym_structure_keyword] = ACTIONS(1731),
    [sym_button_event_keyword] = ACTIONS(1731),
    [sym_channel_event_keyword] = ACTIONS(1731),
    [sym_level_event_keyword] = ACTIONS(1731),
    [sym_data_event_keyword] = ACTIONS(1731),
    [sym_timeline_event_keyword] = ACTIONS(1731),
    [sym_custom_event_keyword] = ACTIONS(1731),
    [sym_push_keyword] = ACTIONS(1731),
    [sym_release_keyword] = ACTIONS(1731),
    [aux_sym_on_keyword_token1] = ACTIONS(1731),
    [aux_sym_off_keyword_token1] = ACTIONS(1731),
    [sym_send_level_keyword] = ACTIONS(1731),
    [sym_send_string_keyword] = ACTIONS(1731),
    [sym_send_command_keyword] = ACTIONS(1731),
    [sym_clear_buffer_keyword] = ACTIONS(1731),
    [sym_create_buffer_keyword] = ACTIONS(1731),
    [sym_create_multi_buffer_keyword] = ACTIONS(1731),
    [sym_call_keyword] = ACTIONS(1731),
    [sym_system_call_keyword] = ACTIONS(1731),
    [sym_devchan_to_keyword] = ACTIONS(1731),
    [sym_devchan_min_to_keyword] = ACTIONS(1731),
    [sym_devchan_total_off_keyword] = ACTIONS(1731),
    [sym_devchan_pulse_keyword] = ACTIONS(1731),
    [sym_wait_keyword] = ACTIONS(1731),
    [sym_cancel_wait_keyword] = ACTIONS(1731),
    [sym_wait_until_keyword] = ACTIONS(1731),
    [sym_cancel_wait_until_keyword] = ACTIONS(1731),
    [sym_cancel_all_wait_keyword] = ACTIONS(1731),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1731),
    [sym_not] = ACTIONS(1731),
    [sym_bnot] = ACTIONS(1731),
    [sym___file__] = ACTIONS(1731),
    [sym___date__] = ACTIONS(1731),
    [sym___ldate__] = ACTIONS(1731),
    [sym___time__] = ACTIONS(1731),
    [sym___line__] = ACTIONS(1731),
    [sym___name__] = ACTIONS(1731),
    [sym___netlinx__] = ACTIONS(1731),
    [sym_netlinx_axi_version] = ACTIONS(1731),
    [sym_url_flg_tcp] = ACTIONS(1731),
    [sym_url_flg_accinfopresent] = ACTIONS(1731),
    [sym_url_flg_temp] = ACTIONS(1731),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1731),
    [sym_url_flg_stat_mask] = ACTIONS(1731),
    [sym_url_flg_stat_lookup] = ACTIONS(1731),
    [sym_url_flg_stat_connecting] = ACTIONS(1731),
    [sym_url_flg_stat_waiting] = ACTIONS(1731),
    [sym_url_flg_stat_connected] = ACTIONS(1731),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1731),
    [sym_first_virtual_device] = ACTIONS(1731),
    [sym_first_local_port] = ACTIONS(1731),
    [sym_dynamic_virtual_device] = ACTIONS(1731),
    [sym_timeline_once] = ACTIONS(1731),
    [sym_timeline_repeat] = ACTIONS(1731),
    [sym_timeline_absolute] = ACTIONS(1731),
    [sym_timeline_relative] = ACTIONS(1731),
    [sym_do_push_timed_infinite] = ACTIONS(1731),
    [sym_source_type_no_address] = ACTIONS(1731),
    [sym_source_type_neuron_id] = ACTIONS(1731),
    [sym_source_type_ip_address] = ACTIONS(1731),
    [sym_source_type_axlink] = ACTIONS(1731),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1731),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1731),
    [sym_source_type_ip_socket_address] = ACTIONS(1731),
    [sym_source_type_rs232] = ACTIONS(1731),
    [sym_source_type_internal] = ACTIONS(1731),
    [sym_source_type_mac_address] = ACTIONS(1731),
    [sym_source_type_ipv4_port] = ACTIONS(1731),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1731),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1731),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1731),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1731),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1731),
    [sym_true] = ACTIONS(1731),
    [sym_false] = ACTIONS(1731),
    [sym_file_read_only] = ACTIONS(1731),
    [sym_file_rw_new] = ACTIONS(1731),
    [sym_file_rw_append] = ACTIONS(1731),
    [sym_ip_tcp] = ACTIONS(1731),
    [sym_ip_udp] = ACTIONS(1731),
    [sym_ip_udp_2way] = ACTIONS(1731),
    [sym_xml_encode_types] = ACTIONS(1731),
    [sym_xml_encode_char_as_list] = ACTIONS(1731),
    [sym_xml_encode_le] = ACTIONS(1731),
    [sym_xml_decode_types] = ACTIONS(1731),
    [sym_xml_decode_no_preserve] = ACTIONS(1731),
    [sym_duet_dev_type_display_device] = ACTIONS(1731),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1731),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1731),
    [sym_duet_dev_type_io_device] = ACTIONS(1731),
    [sym_duet_dev_type_relay_device] = ACTIONS(1731),
    [sym_duet_dev_type_ups] = ACTIONS(1731),
    [sym_duet_dev_type_amplifier] = ACTIONS(1731),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1731),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1731),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1731),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1731),
    [sym_duet_dev_type_camera] = ACTIONS(1731),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1731),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1731),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1731),
    [sym_duet_dev_type_dss] = ACTIONS(1731),
    [sym_duet_dev_type_dvr] = ACTIONS(1731),
    [sym_duet_dev_type_disc_device] = ACTIONS(1731),
    [sym_duet_dev_type_document_camera] = ACTIONS(1731),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1731),
    [sym_duet_dev_type_hvac] = ACTIONS(1731),
    [sym_duet_dev_type_keypad] = ACTIONS(1731),
    [sym_duet_dev_type_light] = ACTIONS(1731),
    [sym_duet_dev_type_monitor] = ACTIONS(1731),
    [sym_duet_dev_type_motor] = ACTIONS(1731),
    [sym_duet_dev_type_multi_window] = ACTIONS(1731),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1731),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1731),
    [sym_duet_dev_type_receiver] = ACTIONS(1731),
    [sym_duet_dev_type_security_system] = ACTIONS(1731),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1731),
    [sym_duet_dev_type_settop_box] = ACTIONS(1731),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1731),
    [sym_duet_dev_type_switcher] = ACTIONS(1731),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1731),
    [sym_duet_dev_type_tv] = ACTIONS(1731),
    [sym_duet_dev_type_utility] = ACTIONS(1731),
    [sym_duet_dev_type_vcr] = ACTIONS(1731),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1731),
    [sym_duet_dev_type_video_processor] = ACTIONS(1731),
    [sym_duet_dev_type_video_projector] = ACTIONS(1731),
    [sym_duet_dev_type_video_wall] = ACTIONS(1731),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1731),
    [sym_duet_dev_type_weather] = ACTIONS(1731),
    [sym_duet_dev_not_polled] = ACTIONS(1731),
    [sym_duet_dev_polled] = ACTIONS(1731),
    [sym_clkmgr_mode_network] = ACTIONS(1731),
    [sym_clkmgr_mode_standalone] = ACTIONS(1731),
    [sym_normal_standby] = ACTIONS(1731),
    [sym_normal_wake] = ACTIONS(1731),
    [sym_ip_multicast_ttl_option] = ACTIONS(1731),
    [sym_ip_ttl_subnet] = ACTIONS(1731),
    [sym_ip_ttl_site] = ACTIONS(1731),
    [sym_ip_ttl_region] = ACTIONS(1731),
    [sym_ip_ttl_continent] = ACTIONS(1731),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1731),
    [sym_ip_nodelay_on] = ACTIONS(1731),
    [sym_ip_nodelay_off] = ACTIONS(1731),
    [sym_amx_error] = ACTIONS(1731),
    [sym_amx_warning] = ACTIONS(1731),
    [sym_amx_info] = ACTIONS(1731),
    [sym_amx_debug] = ACTIONS(1731),
    [sym_netlinx_login_success] = ACTIONS(1731),
    [sym_netlinx_login_fail] = ACTIONS(1731),
    [sym_netlinx_logout] = ACTIONS(1731),
    [sym_null_str] = ACTIONS(1731),
    [sym_smtp_address] = ACTIONS(1731),
    [sym_smtp_port_number] = ACTIONS(1731),
    [sym_smtp_username] = ACTIONS(1731),
    [sym_smtp_password] = ACTIONS(1731),
    [sym_smtp_require_tls] = ACTIONS(1731),
    [sym_smtp_from] = ACTIONS(1731),
    [sym_smtp_tls_true] = ACTIONS(1731),
    [sym_smtp_tls_false] = ACTIONS(1731),
    [sym_listview_on_row_select_event] = ACTIONS(1731),
    [sym_data_structure_unknown] = ACTIONS(1731),
    [sym_data_structure_datafeed] = ACTIONS(1731),
    [sym_data_structure_datarecordset] = ACTIONS(1731),
    [sym_data_structure_datarecord] = ACTIONS(1731),
    [sym_data_structure_datafield] = ACTIONS(1731),
    [sym_data_max_content_field_count] = ACTIONS(1731),
    [sym_data_max_metadata_field_count] = ACTIONS(1731),
    [sym_data_max_id_length] = ACTIONS(1731),
    [sym_data_max_name_length] = ACTIONS(1731),
    [sym_data_max_description_length] = ACTIONS(1731),
    [sym_data_max_type_length] = ACTIONS(1731),
    [sym_data_max_format_length] = ACTIONS(1731),
    [sym_data_max_source_length] = ACTIONS(1731),
    [sym_data_max_value_length] = ACTIONS(1731),
    [sym_data_max_label_length] = ACTIONS(1731),
    [sym_data_max_url_length] = ACTIONS(1731),
    [sym_max_last_login_info_length] = ACTIONS(1731),
    [sym_data_type_unknown] = ACTIONS(1731),
    [sym_data_type_string] = ACTIONS(1731),
    [sym_data_type_datetime] = ACTIONS(1731),
    [sym_data_type_date] = ACTIONS(1731),
    [sym_data_type_time] = ACTIONS(1731),
    [sym_data_type_image] = ACTIONS(1731),
    [sym_data_format_url] = ACTIONS(1731),
    [sym_data_format_phone] = ACTIONS(1731),
    [sym_data_format_email] = ACTIONS(1731),
    [sym_data_format_iso8601] = ACTIONS(1731),
    [sym_auditlog_priv] = ACTIONS(1731),
    [sym_configuration_priv] = ACTIONS(1731),
    [sym_deviceconfig_priv] = ACTIONS(1731),
    [sym_ftp_priv] = ACTIONS(1731),
    [sym_http_priv] = ACTIONS(1731),
    [sym_networkconfig_priv] = ACTIONS(1731),
    [sym_programport_priv] = ACTIONS(1731),
    [sym_remoteui_priv] = ACTIONS(1731),
    [sym_securitycontrol_priv] = ACTIONS(1731),
    [sym_softwaremanagement_priv] = ACTIONS(1731),
    [sym_terminal_priv] = ACTIONS(1731),
    [sym_tpadmin_priv] = ACTIONS(1731),
    [sym_user1_priv] = ACTIONS(1731),
    [sym_user2_priv] = ACTIONS(1731),
    [sym_user3_priv] = ACTIONS(1731),
    [sym_user4_priv] = ACTIONS(1731),
    [sym_usermanagement_priv] = ACTIONS(1731),
    [sym_valid_account] = ACTIONS(1731),
    [sym_err_name_invalid] = ACTIONS(1731),
    [sym_err_password_invalid] = ACTIONS(1731),
    [sym_err_invalid_account] = ACTIONS(1731),
    [sym_err_no_authorization] = ACTIONS(1731),
    [sym_err_invalid_parameter] = ACTIONS(1731),
    [sym_err_user_locked_out] = ACTIONS(1731),
    [sym_err_user_account_expired] = ACTIONS(1731),
    [sym_err_general_error] = ACTIONS(1731),
    [sym_err_authentication_error] = ACTIONS(1731),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1731),
    [sym_tls_validate_certificate] = ACTIONS(1731),
    [sym_tdata] = ACTIONS(1731),
    [sym_tchannel] = ACTIONS(1731),
    [sym_tlevel] = ACTIONS(1731),
    [sym_tbutton] = ACTIONS(1731),
    [sym_ttimeline] = ACTIONS(1731),
    [sym_tcustom] = ACTIONS(1731),
    [sym_url_struct] = ACTIONS(1731),
    [sym_dns_struct] = ACTIONS(1731),
    [sym_ip_address_struct] = ACTIONS(1731),
    [sym_dev_info_struct] = ACTIONS(1731),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1731),
    [sym_data_feed] = ACTIONS(1731),
    [sym_data_field] = ACTIONS(1731),
    [sym_data_record] = ACTIONS(1731),
    [sym_wc_data_feed] = ACTIONS(1731),
    [sym_wc_data_field] = ACTIONS(1731),
    [sym_wc_data_record] = ACTIONS(1731),
    [sym_last_login_info] = ACTIONS(1731),
    [sym____reserved___] = ACTIONS(1731),
    [sym_date] = ACTIONS(1731),
    [sym_day] = ACTIONS(1731),
    [sym_time] = ACTIONS(1731),
    [sym_ldate] = ACTIONS(1731),
    [sym_push_device] = ACTIONS(1731),
    [sym_push_channel] = ACTIONS(1731),
    [sym_push_devchan] = ACTIONS(1731),
    [sym_release_device] = ACTIONS(1731),
    [sym_release_channel] = ACTIONS(1731),
    [sym_release_devchan] = ACTIONS(1731),
    [sym_master_slot] = ACTIONS(1731),
    [sym_get_pulse_time] = ACTIONS(1731),
    [sym_get_timer] = ACTIONS(1731),
    [sym_master_sn] = ACTIONS(1731),
    [sym_system_number] = ACTIONS(1731),
    [sym_button] = ACTIONS(1731),
    [sym_dv_channel] = ACTIONS(1731),
    [sym_data] = ACTIONS(1731),
    [sym_level] = ACTIONS(1731),
    [sym_timeline] = ACTIONS(1731),
    [sym_channel] = ACTIONS(1731),
    [sym_custom] = ACTIONS(1731),
    [sym_atoi] = ACTIONS(1731),
    [sym_atol] = ACTIONS(1731),
    [sym_atof] = ACTIONS(1731),
    [sym_itoa] = ACTIONS(1731),
    [sym_itohex] = ACTIONS(1731),
    [sym_hextoi] = ACTIONS(1731),
    [sym_ftoa] = ACTIONS(1731),
    [sym_find_string] = ACTIONS(1731),
    [sym_left_string] = ACTIONS(1731),
    [sym_right_string] = ACTIONS(1731),
    [sym_mid_string] = ACTIONS(1731),
    [sym_remove_string] = ACTIONS(1731),
    [sym_get_buffer_string] = ACTIONS(1731),
    [sym_get_buffer_char] = ACTIONS(1731),
    [sym_get_multi_buffer_string] = ACTIONS(1731),
    [sym_length_string] = ACTIONS(1731),
    [sym_max_length_string] = ACTIONS(1731),
    [sym_set_length_string] = ACTIONS(1731),
    [sym_compare_string] = ACTIONS(1731),
    [sym_upper_string] = ACTIONS(1731),
    [sym_lower_string] = ACTIONS(1731),
    [sym_get_last] = ACTIONS(1731),
    [sym_date_to_day] = ACTIONS(1731),
    [sym_date_to_month] = ACTIONS(1731),
    [sym_date_to_year] = ACTIONS(1731),
    [sym_time_to_hour] = ACTIONS(1731),
    [sym_time_to_minute] = ACTIONS(1731),
    [sym_time_to_second] = ACTIONS(1731),
    [sym_day_of_week] = ACTIONS(1731),
    [sym_set_pulse_time] = ACTIONS(1731),
    [sym_set_timer] = ACTIONS(1731),
    [sym_file_open] = ACTIONS(1731),
    [sym_file_close] = ACTIONS(1731),
    [sym_file_read] = ACTIONS(1731),
    [sym_file_read_line] = ACTIONS(1731),
    [sym_file_write] = ACTIONS(1731),
    [sym_file_write_line] = ACTIONS(1731),
    [sym_file_copy] = ACTIONS(1731),
    [sym_file_delete] = ACTIONS(1731),
    [sym_file_rename] = ACTIONS(1731),
    [sym_file_dir] = ACTIONS(1731),
    [sym_file_getdir] = ACTIONS(1731),
    [sym_file_setdir] = ACTIONS(1731),
    [sym_file_create_dir] = ACTIONS(1731),
    [sym_file_remove_dir] = ACTIONS(1731),
    [sym_file_seek] = ACTIONS(1731),
    [sym_length_array] = ACTIONS(1731),
    [sym_max_length_array] = ACTIONS(1731),
    [sym_set_length_array] = ACTIONS(1731),
    [sym_ip_client_open] = ACTIONS(1731),
    [sym_ip_client_close] = ACTIONS(1731),
    [sym_ip_server_open] = ACTIONS(1731),
    [sym_ip_server_close] = ACTIONS(1731),
    [sym_ip_bound_client_open] = ACTIONS(1731),
    [sym_ip_mc_server_open] = ACTIONS(1731),
    [sym_ip_set_option] = ACTIONS(1731),
    [sym_ssh_client_open] = ACTIONS(1731),
    [sym_ssh_client_close] = ACTIONS(1731),
    [sym_tls_client_open] = ACTIONS(1731),
    [sym_tls_client_close] = ACTIONS(1731),
    [sym_do_push] = ACTIONS(1731),
    [sym_do_push_timed] = ACTIONS(1731),
    [sym_do_release] = ACTIONS(1731),
    [sym_device_id] = ACTIONS(1731),
    [sym_device_info] = ACTIONS(1731),
    [sym_device_id_string] = ACTIONS(1731),
    [sym_device_standby] = ACTIONS(1731),
    [sym_device_wake] = ACTIONS(1731),
    [sym_timeline_create] = ACTIONS(1731),
    [sym_timeline_kill] = ACTIONS(1731),
    [sym_timeline_reload] = ACTIONS(1731),
    [sym_timeline_pause] = ACTIONS(1731),
    [sym_timeline_restart] = ACTIONS(1731),
    [sym_timeline_set] = ACTIONS(1731),
    [sym_timeline_get] = ACTIONS(1731),
    [sym_timeline_active] = ACTIONS(1731),
    [sym_timeline_dynamic_id] = ACTIONS(1731),
    [sym_variable_to_string] = ACTIONS(1731),
    [sym_string_to_variable] = ACTIONS(1731),
    [sym_length_variable_to_string] = ACTIONS(1731),
    [sym_type_cast] = ACTIONS(1731),
    [sym_variable_to_xml] = ACTIONS(1731),
    [sym_xml_to_variable] = ACTIONS(1731),
    [sym_length_variable_to_xml] = ACTIONS(1731),
    [sym_redirect_string] = ACTIONS(1731),
    [sym_random_number] = ACTIONS(1731),
    [sym_get_unique_id] = ACTIONS(1731),
    [sym_get_system_number] = ACTIONS(1731),
    [sym_set_system_number] = ACTIONS(1731),
    [sym_get_serial_number] = ACTIONS(1731),
    [sym_get_master_build] = ACTIONS(1731),
    [sym_set_validation_code] = ACTIONS(1731),
    [sym_reboot] = ACTIONS(1731),
    [sym_load_duet_module] = ACTIONS(1731),
    [sym_unload_duet_module] = ACTIONS(1731),
    [sym_duet_mem_size_set] = ACTIONS(1731),
    [sym_duet_mem_size_get] = ACTIONS(1731),
    [sym_dynamic_application_device] = ACTIONS(1731),
    [sym_dynamic_polled_port] = ACTIONS(1731),
    [sym_static_port_binding] = ACTIONS(1731),
    [sym_static_ip_binding] = ACTIONS(1731),
    [sym_get_dns_list] = ACTIONS(1731),
    [sym_set_dns_list] = ACTIONS(1731),
    [sym_get_ip_address] = ACTIONS(1731),
    [sym_set_ip_address] = ACTIONS(1731),
    [sym_get_url_list] = ACTIONS(1731),
    [sym_add_url_entry] = ACTIONS(1731),
    [sym_delete_url_entry] = ACTIONS(1731),
    [sym_abs_value] = ACTIONS(1731),
    [sym_max_value] = ACTIONS(1731),
    [sym_min_value] = ACTIONS(1731),
    [sym_exp_value] = ACTIONS(1731),
    [sym_log_value] = ACTIONS(1731),
    [sym_log10_value] = ACTIONS(1731),
    [sym_power_value] = ACTIONS(1731),
    [sym_sqrt_value] = ACTIONS(1731),
    [sym_raw_be] = ACTIONS(1731),
    [sym_raw_le] = ACTIONS(1731),
    [sym_internal_queue_size_set] = ACTIONS(1731),
    [sym_internal_queue_size_get] = ACTIONS(1731),
    [sym_internal_threshold_set] = ACTIONS(1731),
    [sym_internal_threshold_get] = ACTIONS(1731),
    [sym_rebuilt_event] = ACTIONS(1731),
    [sym_do_custom_event] = ACTIONS(1731),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1731),
    [sym_clkmgr_set_clk_source] = ACTIONS(1731),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1731),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1731),
    [sym_clkmgr_get_timezone] = ACTIONS(1731),
    [sym_clkmgr_set_timezone] = ACTIONS(1731),
    [sym_clkmgr_get_resync_period] = ACTIONS(1731),
    [sym_clkmgr_set_resync_period] = ACTIONS(1731),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1731),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1731),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1731),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1731),
    [sym_clkmgr_get_timeservers] = ACTIONS(1731),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1731),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1731),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1731),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1731),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1731),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1731),
    [sym_set_log_level] = ACTIONS(1731),
    [sym_get_log_level] = ACTIONS(1731),
    [sym_amx_log] = ACTIONS(1731),
    [sym_smtp_server_config_set] = ACTIONS(1731),
    [sym_smtp_server_config_get] = ACTIONS(1731),
    [sym_smtp_send] = ACTIONS(1731),
    [sym_validate_netlinx_account] = ACTIONS(1731),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1731),
    [sym_authenticate_certificate] = ACTIONS(1731),
    [sym_audit_netlinx_session_event] = ACTIONS(1731),
    [sym_audit_netlinx_generic_event] = ACTIONS(1731),
    [sym_get_available_flash_disk_space] = ACTIONS(1731),
    [sym_get_max_flash_disk_space] = ACTIONS(1731),
    [sym_data_create_feed] = ACTIONS(1731),
    [sym_data_delete_feed] = ACTIONS(1731),
    [sym_data_publish_feed] = ACTIONS(1731),
    [sym_data_get_published_feed] = ACTIONS(1731),
    [sym_data_add_record] = ACTIONS(1731),
    [sym_data_get_event_record] = ACTIONS(1731),
    [sym__wc_data_create_feed] = ACTIONS(1731),
    [sym__wc_data_add_record] = ACTIONS(1731),
    [sym__wc_data_get_event_record] = ACTIONS(1731),
    [sym_format] = ACTIONS(1731),
    [sym_astro_clock] = ACTIONS(1731),
    [sym_set_outdoor_temperature] = ACTIONS(1731),
    [sym_set_virtual_level_count] = ACTIONS(1731),
    [sym_set_virtual_channel_count] = ACTIONS(1731),
    [sym_set_virtual_port_count] = ACTIONS(1731),
    [sym__wc_encode_int] = ACTIONS(1731),
    [sym__wc_decode_int] = ACTIONS(1731),
    [sym__wc_int] = ACTIONS(1731),
    [sym__wc_to_ch_int] = ACTIONS(1731),
    [sym__ch_to_wc_int] = ACTIONS(1731),
    [sym__wc_find_string_int] = ACTIONS(1731),
    [sym__wc_left_string_int] = ACTIONS(1731),
    [sym__wc_lower_string_int] = ACTIONS(1731),
    [sym__wc_mid_string_int] = ACTIONS(1731),
    [sym__wc_remove_string_int] = ACTIONS(1731),
    [sym__wc_right_string_int] = ACTIONS(1731),
    [sym__wc_upper_string_int] = ACTIONS(1731),
    [sym__wc_compare_string_int] = ACTIONS(1731),
    [sym__wc_get_buffer_char_int] = ACTIONS(1731),
    [sym__wc_get_buffer_string_int] = ACTIONS(1731),
    [sym__wc_concat_string_int] = ACTIONS(1731),
    [sym__wc_file_open_int] = ACTIONS(1731),
    [sym__wc_file_close_int] = ACTIONS(1731),
    [sym__wc_file_read_int] = ACTIONS(1731),
    [sym__wc_file_read_line_int] = ACTIONS(1731),
    [sym__wc_file_write_int] = ACTIONS(1731),
    [sym__wc_file_write_line_int] = ACTIONS(1731),
    [aux_sym_preproc_include_token1] = ACTIONS(1731),
    [aux_sym_preproc_define_token1] = ACTIONS(1733),
    [aux_sym_preproc_warn_token1] = ACTIONS(1733),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1733),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1733),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1733),
    [anon_sym_LBRACK] = ACTIONS(1733),
    [anon_sym_LBRACE] = ACTIONS(1733),
    [anon_sym_LPAREN] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [anon_sym_DASH] = ACTIONS(1731),
    [anon_sym_PLUS] = ACTIONS(1731),
    [anon_sym_DASH_DASH] = ACTIONS(1733),
    [anon_sym_PLUS_PLUS] = ACTIONS(1733),
    [anon_sym_DQUOTE] = ACTIONS(1733),
    [anon_sym_SQUOTE] = ACTIONS(1733),
    [sym_number_literal] = ACTIONS(1733),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(335)] = {
    [sym_identifier] = ACTIONS(2037),
    [sym_program_name_keyword] = ACTIONS(2037),
    [sym_module_name_keyword] = ACTIONS(2037),
    [sym_define_device_keyword] = ACTIONS(2037),
    [sym_define_combine_keyword] = ACTIONS(2037),
    [sym_define_constant_keyword] = ACTIONS(2037),
    [sym_define_type_keyword] = ACTIONS(2037),
    [sym_define_variable_keyword] = ACTIONS(2037),
    [sym_define_system_variable_keyword] = ACTIONS(2037),
    [sym_define_connect_level_keyword] = ACTIONS(2037),
    [sym_define_latching_keyword] = ACTIONS(2037),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2037),
    [sym_define_toggling_keyword] = ACTIONS(2037),
    [sym_define_call_keyword] = ACTIONS(2037),
    [sym_define_function_keyword] = ACTIONS(2037),
    [sym_define_library_function_keyword] = ACTIONS(2037),
    [sym_define_start_keyword] = ACTIONS(2037),
    [sym_define_event_keyword] = ACTIONS(2037),
    [sym_define_program_keyword] = ACTIONS(2037),
    [sym_define_module_keyword] = ACTIONS(2037),
    [sym_char_keyword] = ACTIONS(2037),
    [sym_widechar_keyword] = ACTIONS(2037),
    [sym_integer_keyword] = ACTIONS(2037),
    [sym_sinteger_keyword] = ACTIONS(2037),
    [sym_long_keyword] = ACTIONS(2037),
    [sym_slong_keyword] = ACTIONS(2037),
    [sym_float_keyword] = ACTIONS(2037),
    [sym_double_keyword] = ACTIONS(2037),
    [sym_variant_keyword] = ACTIONS(2037),
    [sym_variantarray_keyword] = ACTIONS(2037),
    [sym_local_var_keyword] = ACTIONS(2037),
    [sym_stack_var_keyword] = ACTIONS(2037),
    [sym_constant_keyword] = ACTIONS(2037),
    [sym_volatile_keyword] = ACTIONS(2037),
    [sym_non_volatile_keyword] = ACTIONS(2037),
    [sym_persistent_keyword] = ACTIONS(2037),
    [sym_dev_keyword] = ACTIONS(2037),
    [sym_devlev_keyword] = ACTIONS(2037),
    [sym_devchan_keyword] = ACTIONS(2037),
    [sym_if_keyword] = ACTIONS(2037),
    [sym_switch_keyword] = ACTIONS(2037),
    [sym_case_keyword] = ACTIONS(2037),
    [sym_default_keyword] = ACTIONS(2037),
    [sym_while_keyword] = ACTIONS(2037),
    [sym_for_keyword] = ACTIONS(2037),
    [sym_break_keyword] = ACTIONS(2037),
    [sym_continue_keyword] = ACTIONS(2037),
    [sym_return_keyword] = ACTIONS(2037),
    [sym_select_keyword] = ACTIONS(2037),
    [sym_active_keyword] = ACTIONS(2037),
    [sym_struct_keyword] = ACTIONS(2037),
    [sym_structure_keyword] = ACTIONS(2037),
    [sym_button_event_keyword] = ACTIONS(2037),
    [sym_channel_event_keyword] = ACTIONS(2037),
    [sym_level_event_keyword] = ACTIONS(2037),
    [sym_data_event_keyword] = ACTIONS(2037),
    [sym_timeline_event_keyword] = ACTIONS(2037),
    [sym_custom_event_keyword] = ACTIONS(2037),
    [sym_push_keyword] = ACTIONS(2037),
    [sym_release_keyword] = ACTIONS(2037),
    [aux_sym_on_keyword_token1] = ACTIONS(2037),
    [aux_sym_off_keyword_token1] = ACTIONS(2037),
    [sym_send_level_keyword] = ACTIONS(2037),
    [sym_send_string_keyword] = ACTIONS(2037),
    [sym_send_command_keyword] = ACTIONS(2037),
    [sym_clear_buffer_keyword] = ACTIONS(2037),
    [sym_create_buffer_keyword] = ACTIONS(2037),
    [sym_create_multi_buffer_keyword] = ACTIONS(2037),
    [sym_call_keyword] = ACTIONS(2037),
    [sym_system_call_keyword] = ACTIONS(2037),
    [sym_devchan_to_keyword] = ACTIONS(2037),
    [sym_devchan_min_to_keyword] = ACTIONS(2037),
    [sym_devchan_total_off_keyword] = ACTIONS(2037),
    [sym_devchan_pulse_keyword] = ACTIONS(2037),
    [sym_wait_keyword] = ACTIONS(2037),
    [sym_cancel_wait_keyword] = ACTIONS(2037),
    [sym_wait_until_keyword] = ACTIONS(2037),
    [sym_cancel_wait_until_keyword] = ACTIONS(2037),
    [sym_cancel_all_wait_keyword] = ACTIONS(2037),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2037),
    [sym_not] = ACTIONS(2037),
    [sym_bnot] = ACTIONS(2037),
    [sym___file__] = ACTIONS(2037),
    [sym___date__] = ACTIONS(2037),
    [sym___ldate__] = ACTIONS(2037),
    [sym___time__] = ACTIONS(2037),
    [sym___line__] = ACTIONS(2037),
    [sym___name__] = ACTIONS(2037),
    [sym___netlinx__] = ACTIONS(2037),
    [sym_netlinx_axi_version] = ACTIONS(2037),
    [sym_url_flg_tcp] = ACTIONS(2037),
    [sym_url_flg_accinfopresent] = ACTIONS(2037),
    [sym_url_flg_temp] = ACTIONS(2037),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2037),
    [sym_url_flg_stat_mask] = ACTIONS(2037),
    [sym_url_flg_stat_lookup] = ACTIONS(2037),
    [sym_url_flg_stat_connecting] = ACTIONS(2037),
    [sym_url_flg_stat_waiting] = ACTIONS(2037),
    [sym_url_flg_stat_connected] = ACTIONS(2037),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2037),
    [sym_first_virtual_device] = ACTIONS(2037),
    [sym_first_local_port] = ACTIONS(2037),
    [sym_dynamic_virtual_device] = ACTIONS(2037),
    [sym_timeline_once] = ACTIONS(2037),
    [sym_timeline_repeat] = ACTIONS(2037),
    [sym_timeline_absolute] = ACTIONS(2037),
    [sym_timeline_relative] = ACTIONS(2037),
    [sym_do_push_timed_infinite] = ACTIONS(2037),
    [sym_source_type_no_address] = ACTIONS(2037),
    [sym_source_type_neuron_id] = ACTIONS(2037),
    [sym_source_type_ip_address] = ACTIONS(2037),
    [sym_source_type_axlink] = ACTIONS(2037),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2037),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2037),
    [sym_source_type_ip_socket_address] = ACTIONS(2037),
    [sym_source_type_rs232] = ACTIONS(2037),
    [sym_source_type_internal] = ACTIONS(2037),
    [sym_source_type_mac_address] = ACTIONS(2037),
    [sym_source_type_ipv4_port] = ACTIONS(2037),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2037),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2037),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2037),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2037),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2037),
    [sym_true] = ACTIONS(2037),
    [sym_false] = ACTIONS(2037),
    [sym_file_read_only] = ACTIONS(2037),
    [sym_file_rw_new] = ACTIONS(2037),
    [sym_file_rw_append] = ACTIONS(2037),
    [sym_ip_tcp] = ACTIONS(2037),
    [sym_ip_udp] = ACTIONS(2037),
    [sym_ip_udp_2way] = ACTIONS(2037),
    [sym_xml_encode_types] = ACTIONS(2037),
    [sym_xml_encode_char_as_list] = ACTIONS(2037),
    [sym_xml_encode_le] = ACTIONS(2037),
    [sym_xml_decode_types] = ACTIONS(2037),
    [sym_xml_decode_no_preserve] = ACTIONS(2037),
    [sym_duet_dev_type_display_device] = ACTIONS(2037),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2037),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2037),
    [sym_duet_dev_type_io_device] = ACTIONS(2037),
    [sym_duet_dev_type_relay_device] = ACTIONS(2037),
    [sym_duet_dev_type_ups] = ACTIONS(2037),
    [sym_duet_dev_type_amplifier] = ACTIONS(2037),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2037),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2037),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2037),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2037),
    [sym_duet_dev_type_camera] = ACTIONS(2037),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2037),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2037),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2037),
    [sym_duet_dev_type_dss] = ACTIONS(2037),
    [sym_duet_dev_type_dvr] = ACTIONS(2037),
    [sym_duet_dev_type_disc_device] = ACTIONS(2037),
    [sym_duet_dev_type_document_camera] = ACTIONS(2037),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2037),
    [sym_duet_dev_type_hvac] = ACTIONS(2037),
    [sym_duet_dev_type_keypad] = ACTIONS(2037),
    [sym_duet_dev_type_light] = ACTIONS(2037),
    [sym_duet_dev_type_monitor] = ACTIONS(2037),
    [sym_duet_dev_type_motor] = ACTIONS(2037),
    [sym_duet_dev_type_multi_window] = ACTIONS(2037),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2037),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2037),
    [sym_duet_dev_type_receiver] = ACTIONS(2037),
    [sym_duet_dev_type_security_system] = ACTIONS(2037),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2037),
    [sym_duet_dev_type_settop_box] = ACTIONS(2037),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2037),
    [sym_duet_dev_type_switcher] = ACTIONS(2037),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2037),
    [sym_duet_dev_type_tv] = ACTIONS(2037),
    [sym_duet_dev_type_utility] = ACTIONS(2037),
    [sym_duet_dev_type_vcr] = ACTIONS(2037),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2037),
    [sym_duet_dev_type_video_processor] = ACTIONS(2037),
    [sym_duet_dev_type_video_projector] = ACTIONS(2037),
    [sym_duet_dev_type_video_wall] = ACTIONS(2037),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2037),
    [sym_duet_dev_type_weather] = ACTIONS(2037),
    [sym_duet_dev_not_polled] = ACTIONS(2037),
    [sym_duet_dev_polled] = ACTIONS(2037),
    [sym_clkmgr_mode_network] = ACTIONS(2037),
    [sym_clkmgr_mode_standalone] = ACTIONS(2037),
    [sym_normal_standby] = ACTIONS(2037),
    [sym_normal_wake] = ACTIONS(2037),
    [sym_ip_multicast_ttl_option] = ACTIONS(2037),
    [sym_ip_ttl_subnet] = ACTIONS(2037),
    [sym_ip_ttl_site] = ACTIONS(2037),
    [sym_ip_ttl_region] = ACTIONS(2037),
    [sym_ip_ttl_continent] = ACTIONS(2037),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2037),
    [sym_ip_nodelay_on] = ACTIONS(2037),
    [sym_ip_nodelay_off] = ACTIONS(2037),
    [sym_amx_error] = ACTIONS(2037),
    [sym_amx_warning] = ACTIONS(2037),
    [sym_amx_info] = ACTIONS(2037),
    [sym_amx_debug] = ACTIONS(2037),
    [sym_netlinx_login_success] = ACTIONS(2037),
    [sym_netlinx_login_fail] = ACTIONS(2037),
    [sym_netlinx_logout] = ACTIONS(2037),
    [sym_null_str] = ACTIONS(2037),
    [sym_smtp_address] = ACTIONS(2037),
    [sym_smtp_port_number] = ACTIONS(2037),
    [sym_smtp_username] = ACTIONS(2037),
    [sym_smtp_password] = ACTIONS(2037),
    [sym_smtp_require_tls] = ACTIONS(2037),
    [sym_smtp_from] = ACTIONS(2037),
    [sym_smtp_tls_true] = ACTIONS(2037),
    [sym_smtp_tls_false] = ACTIONS(2037),
    [sym_listview_on_row_select_event] = ACTIONS(2037),
    [sym_data_structure_unknown] = ACTIONS(2037),
    [sym_data_structure_datafeed] = ACTIONS(2037),
    [sym_data_structure_datarecordset] = ACTIONS(2037),
    [sym_data_structure_datarecord] = ACTIONS(2037),
    [sym_data_structure_datafield] = ACTIONS(2037),
    [sym_data_max_content_field_count] = ACTIONS(2037),
    [sym_data_max_metadata_field_count] = ACTIONS(2037),
    [sym_data_max_id_length] = ACTIONS(2037),
    [sym_data_max_name_length] = ACTIONS(2037),
    [sym_data_max_description_length] = ACTIONS(2037),
    [sym_data_max_type_length] = ACTIONS(2037),
    [sym_data_max_format_length] = ACTIONS(2037),
    [sym_data_max_source_length] = ACTIONS(2037),
    [sym_data_max_value_length] = ACTIONS(2037),
    [sym_data_max_label_length] = ACTIONS(2037),
    [sym_data_max_url_length] = ACTIONS(2037),
    [sym_max_last_login_info_length] = ACTIONS(2037),
    [sym_data_type_unknown] = ACTIONS(2037),
    [sym_data_type_string] = ACTIONS(2037),
    [sym_data_type_datetime] = ACTIONS(2037),
    [sym_data_type_date] = ACTIONS(2037),
    [sym_data_type_time] = ACTIONS(2037),
    [sym_data_type_image] = ACTIONS(2037),
    [sym_data_format_url] = ACTIONS(2037),
    [sym_data_format_phone] = ACTIONS(2037),
    [sym_data_format_email] = ACTIONS(2037),
    [sym_data_format_iso8601] = ACTIONS(2037),
    [sym_auditlog_priv] = ACTIONS(2037),
    [sym_configuration_priv] = ACTIONS(2037),
    [sym_deviceconfig_priv] = ACTIONS(2037),
    [sym_ftp_priv] = ACTIONS(2037),
    [sym_http_priv] = ACTIONS(2037),
    [sym_networkconfig_priv] = ACTIONS(2037),
    [sym_programport_priv] = ACTIONS(2037),
    [sym_remoteui_priv] = ACTIONS(2037),
    [sym_securitycontrol_priv] = ACTIONS(2037),
    [sym_softwaremanagement_priv] = ACTIONS(2037),
    [sym_terminal_priv] = ACTIONS(2037),
    [sym_tpadmin_priv] = ACTIONS(2037),
    [sym_user1_priv] = ACTIONS(2037),
    [sym_user2_priv] = ACTIONS(2037),
    [sym_user3_priv] = ACTIONS(2037),
    [sym_user4_priv] = ACTIONS(2037),
    [sym_usermanagement_priv] = ACTIONS(2037),
    [sym_valid_account] = ACTIONS(2037),
    [sym_err_name_invalid] = ACTIONS(2037),
    [sym_err_password_invalid] = ACTIONS(2037),
    [sym_err_invalid_account] = ACTIONS(2037),
    [sym_err_no_authorization] = ACTIONS(2037),
    [sym_err_invalid_parameter] = ACTIONS(2037),
    [sym_err_user_locked_out] = ACTIONS(2037),
    [sym_err_user_account_expired] = ACTIONS(2037),
    [sym_err_general_error] = ACTIONS(2037),
    [sym_err_authentication_error] = ACTIONS(2037),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2037),
    [sym_tls_validate_certificate] = ACTIONS(2037),
    [sym_tdata] = ACTIONS(2037),
    [sym_tchannel] = ACTIONS(2037),
    [sym_tlevel] = ACTIONS(2037),
    [sym_tbutton] = ACTIONS(2037),
    [sym_ttimeline] = ACTIONS(2037),
    [sym_tcustom] = ACTIONS(2037),
    [sym_url_struct] = ACTIONS(2037),
    [sym_dns_struct] = ACTIONS(2037),
    [sym_ip_address_struct] = ACTIONS(2037),
    [sym_dev_info_struct] = ACTIONS(2037),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2037),
    [sym_data_feed] = ACTIONS(2037),
    [sym_data_field] = ACTIONS(2037),
    [sym_data_record] = ACTIONS(2037),
    [sym_wc_data_feed] = ACTIONS(2037),
    [sym_wc_data_field] = ACTIONS(2037),
    [sym_wc_data_record] = ACTIONS(2037),
    [sym_last_login_info] = ACTIONS(2037),
    [sym____reserved___] = ACTIONS(2037),
    [sym_date] = ACTIONS(2037),
    [sym_day] = ACTIONS(2037),
    [sym_time] = ACTIONS(2037),
    [sym_ldate] = ACTIONS(2037),
    [sym_push_device] = ACTIONS(2037),
    [sym_push_channel] = ACTIONS(2037),
    [sym_push_devchan] = ACTIONS(2037),
    [sym_release_device] = ACTIONS(2037),
    [sym_release_channel] = ACTIONS(2037),
    [sym_release_devchan] = ACTIONS(2037),
    [sym_master_slot] = ACTIONS(2037),
    [sym_get_pulse_time] = ACTIONS(2037),
    [sym_get_timer] = ACTIONS(2037),
    [sym_master_sn] = ACTIONS(2037),
    [sym_system_number] = ACTIONS(2037),
    [sym_button] = ACTIONS(2037),
    [sym_dv_channel] = ACTIONS(2037),
    [sym_data] = ACTIONS(2037),
    [sym_level] = ACTIONS(2037),
    [sym_timeline] = ACTIONS(2037),
    [sym_channel] = ACTIONS(2037),
    [sym_custom] = ACTIONS(2037),
    [sym_atoi] = ACTIONS(2037),
    [sym_atol] = ACTIONS(2037),
    [sym_atof] = ACTIONS(2037),
    [sym_itoa] = ACTIONS(2037),
    [sym_itohex] = ACTIONS(2037),
    [sym_hextoi] = ACTIONS(2037),
    [sym_ftoa] = ACTIONS(2037),
    [sym_find_string] = ACTIONS(2037),
    [sym_left_string] = ACTIONS(2037),
    [sym_right_string] = ACTIONS(2037),
    [sym_mid_string] = ACTIONS(2037),
    [sym_remove_string] = ACTIONS(2037),
    [sym_get_buffer_string] = ACTIONS(2037),
    [sym_get_buffer_char] = ACTIONS(2037),
    [sym_get_multi_buffer_string] = ACTIONS(2037),
    [sym_length_string] = ACTIONS(2037),
    [sym_max_length_string] = ACTIONS(2037),
    [sym_set_length_string] = ACTIONS(2037),
    [sym_compare_string] = ACTIONS(2037),
    [sym_upper_string] = ACTIONS(2037),
    [sym_lower_string] = ACTIONS(2037),
    [sym_get_last] = ACTIONS(2037),
    [sym_date_to_day] = ACTIONS(2037),
    [sym_date_to_month] = ACTIONS(2037),
    [sym_date_to_year] = ACTIONS(2037),
    [sym_time_to_hour] = ACTIONS(2037),
    [sym_time_to_minute] = ACTIONS(2037),
    [sym_time_to_second] = ACTIONS(2037),
    [sym_day_of_week] = ACTIONS(2037),
    [sym_set_pulse_time] = ACTIONS(2037),
    [sym_set_timer] = ACTIONS(2037),
    [sym_file_open] = ACTIONS(2037),
    [sym_file_close] = ACTIONS(2037),
    [sym_file_read] = ACTIONS(2037),
    [sym_file_read_line] = ACTIONS(2037),
    [sym_file_write] = ACTIONS(2037),
    [sym_file_write_line] = ACTIONS(2037),
    [sym_file_copy] = ACTIONS(2037),
    [sym_file_delete] = ACTIONS(2037),
    [sym_file_rename] = ACTIONS(2037),
    [sym_file_dir] = ACTIONS(2037),
    [sym_file_getdir] = ACTIONS(2037),
    [sym_file_setdir] = ACTIONS(2037),
    [sym_file_create_dir] = ACTIONS(2037),
    [sym_file_remove_dir] = ACTIONS(2037),
    [sym_file_seek] = ACTIONS(2037),
    [sym_length_array] = ACTIONS(2037),
    [sym_max_length_array] = ACTIONS(2037),
    [sym_set_length_array] = ACTIONS(2037),
    [sym_ip_client_open] = ACTIONS(2037),
    [sym_ip_client_close] = ACTIONS(2037),
    [sym_ip_server_open] = ACTIONS(2037),
    [sym_ip_server_close] = ACTIONS(2037),
    [sym_ip_bound_client_open] = ACTIONS(2037),
    [sym_ip_mc_server_open] = ACTIONS(2037),
    [sym_ip_set_option] = ACTIONS(2037),
    [sym_ssh_client_open] = ACTIONS(2037),
    [sym_ssh_client_close] = ACTIONS(2037),
    [sym_tls_client_open] = ACTIONS(2037),
    [sym_tls_client_close] = ACTIONS(2037),
    [sym_do_push] = ACTIONS(2037),
    [sym_do_push_timed] = ACTIONS(2037),
    [sym_do_release] = ACTIONS(2037),
    [sym_device_id] = ACTIONS(2037),
    [sym_device_info] = ACTIONS(2037),
    [sym_device_id_string] = ACTIONS(2037),
    [sym_device_standby] = ACTIONS(2037),
    [sym_device_wake] = ACTIONS(2037),
    [sym_timeline_create] = ACTIONS(2037),
    [sym_timeline_kill] = ACTIONS(2037),
    [sym_timeline_reload] = ACTIONS(2037),
    [sym_timeline_pause] = ACTIONS(2037),
    [sym_timeline_restart] = ACTIONS(2037),
    [sym_timeline_set] = ACTIONS(2037),
    [sym_timeline_get] = ACTIONS(2037),
    [sym_timeline_active] = ACTIONS(2037),
    [sym_timeline_dynamic_id] = ACTIONS(2037),
    [sym_variable_to_string] = ACTIONS(2037),
    [sym_string_to_variable] = ACTIONS(2037),
    [sym_length_variable_to_string] = ACTIONS(2037),
    [sym_type_cast] = ACTIONS(2037),
    [sym_variable_to_xml] = ACTIONS(2037),
    [sym_xml_to_variable] = ACTIONS(2037),
    [sym_length_variable_to_xml] = ACTIONS(2037),
    [sym_redirect_string] = ACTIONS(2037),
    [sym_random_number] = ACTIONS(2037),
    [sym_get_unique_id] = ACTIONS(2037),
    [sym_get_system_number] = ACTIONS(2037),
    [sym_set_system_number] = ACTIONS(2037),
    [sym_get_serial_number] = ACTIONS(2037),
    [sym_get_master_build] = ACTIONS(2037),
    [sym_set_validation_code] = ACTIONS(2037),
    [sym_reboot] = ACTIONS(2037),
    [sym_load_duet_module] = ACTIONS(2037),
    [sym_unload_duet_module] = ACTIONS(2037),
    [sym_duet_mem_size_set] = ACTIONS(2037),
    [sym_duet_mem_size_get] = ACTIONS(2037),
    [sym_dynamic_application_device] = ACTIONS(2037),
    [sym_dynamic_polled_port] = ACTIONS(2037),
    [sym_static_port_binding] = ACTIONS(2037),
    [sym_static_ip_binding] = ACTIONS(2037),
    [sym_get_dns_list] = ACTIONS(2037),
    [sym_set_dns_list] = ACTIONS(2037),
    [sym_get_ip_address] = ACTIONS(2037),
    [sym_set_ip_address] = ACTIONS(2037),
    [sym_get_url_list] = ACTIONS(2037),
    [sym_add_url_entry] = ACTIONS(2037),
    [sym_delete_url_entry] = ACTIONS(2037),
    [sym_abs_value] = ACTIONS(2037),
    [sym_max_value] = ACTIONS(2037),
    [sym_min_value] = ACTIONS(2037),
    [sym_exp_value] = ACTIONS(2037),
    [sym_log_value] = ACTIONS(2037),
    [sym_log10_value] = ACTIONS(2037),
    [sym_power_value] = ACTIONS(2037),
    [sym_sqrt_value] = ACTIONS(2037),
    [sym_raw_be] = ACTIONS(2037),
    [sym_raw_le] = ACTIONS(2037),
    [sym_internal_queue_size_set] = ACTIONS(2037),
    [sym_internal_queue_size_get] = ACTIONS(2037),
    [sym_internal_threshold_set] = ACTIONS(2037),
    [sym_internal_threshold_get] = ACTIONS(2037),
    [sym_rebuilt_event] = ACTIONS(2037),
    [sym_do_custom_event] = ACTIONS(2037),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2037),
    [sym_clkmgr_set_clk_source] = ACTIONS(2037),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2037),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2037),
    [sym_clkmgr_get_timezone] = ACTIONS(2037),
    [sym_clkmgr_set_timezone] = ACTIONS(2037),
    [sym_clkmgr_get_resync_period] = ACTIONS(2037),
    [sym_clkmgr_set_resync_period] = ACTIONS(2037),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2037),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2037),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2037),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2037),
    [sym_clkmgr_get_timeservers] = ACTIONS(2037),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2037),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2037),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2037),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2037),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2037),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2037),
    [sym_set_log_level] = ACTIONS(2037),
    [sym_get_log_level] = ACTIONS(2037),
    [sym_amx_log] = ACTIONS(2037),
    [sym_smtp_server_config_set] = ACTIONS(2037),
    [sym_smtp_server_config_get] = ACTIONS(2037),
    [sym_smtp_send] = ACTIONS(2037),
    [sym_validate_netlinx_account] = ACTIONS(2037),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2037),
    [sym_authenticate_certificate] = ACTIONS(2037),
    [sym_audit_netlinx_session_event] = ACTIONS(2037),
    [sym_audit_netlinx_generic_event] = ACTIONS(2037),
    [sym_get_available_flash_disk_space] = ACTIONS(2037),
    [sym_get_max_flash_disk_space] = ACTIONS(2037),
    [sym_data_create_feed] = ACTIONS(2037),
    [sym_data_delete_feed] = ACTIONS(2037),
    [sym_data_publish_feed] = ACTIONS(2037),
    [sym_data_get_published_feed] = ACTIONS(2037),
    [sym_data_add_record] = ACTIONS(2037),
    [sym_data_get_event_record] = ACTIONS(2037),
    [sym__wc_data_create_feed] = ACTIONS(2037),
    [sym__wc_data_add_record] = ACTIONS(2037),
    [sym__wc_data_get_event_record] = ACTIONS(2037),
    [sym_format] = ACTIONS(2037),
    [sym_astro_clock] = ACTIONS(2037),
    [sym_set_outdoor_temperature] = ACTIONS(2037),
    [sym_set_virtual_level_count] = ACTIONS(2037),
    [sym_set_virtual_channel_count] = ACTIONS(2037),
    [sym_set_virtual_port_count] = ACTIONS(2037),
    [sym__wc_encode_int] = ACTIONS(2037),
    [sym__wc_decode_int] = ACTIONS(2037),
    [sym__wc_int] = ACTIONS(2037),
    [sym__wc_to_ch_int] = ACTIONS(2037),
    [sym__ch_to_wc_int] = ACTIONS(2037),
    [sym__wc_find_string_int] = ACTIONS(2037),
    [sym__wc_left_string_int] = ACTIONS(2037),
    [sym__wc_lower_string_int] = ACTIONS(2037),
    [sym__wc_mid_string_int] = ACTIONS(2037),
    [sym__wc_remove_string_int] = ACTIONS(2037),
    [sym__wc_right_string_int] = ACTIONS(2037),
    [sym__wc_upper_string_int] = ACTIONS(2037),
    [sym__wc_compare_string_int] = ACTIONS(2037),
    [sym__wc_get_buffer_char_int] = ACTIONS(2037),
    [sym__wc_get_buffer_string_int] = ACTIONS(2037),
    [sym__wc_concat_string_int] = ACTIONS(2037),
    [sym__wc_file_open_int] = ACTIONS(2037),
    [sym__wc_file_close_int] = ACTIONS(2037),
    [sym__wc_file_read_int] = ACTIONS(2037),
    [sym__wc_file_read_line_int] = ACTIONS(2037),
    [sym__wc_file_write_int] = ACTIONS(2037),
    [sym__wc_file_write_line_int] = ACTIONS(2037),
    [aux_sym_preproc_include_token1] = ACTIONS(2037),
    [aux_sym_preproc_define_token1] = ACTIONS(2039),
    [aux_sym_preproc_warn_token1] = ACTIONS(2039),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2039),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2039),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2039),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2039),
    [aux_sym_preproc_else_token1] = ACTIONS(2039),
    [anon_sym_LBRACK] = ACTIONS(2039),
    [anon_sym_LBRACE] = ACTIONS(2039),
    [anon_sym_LPAREN] = ACTIONS(2037),
    [anon_sym_BANG] = ACTIONS(2039),
    [anon_sym_TILDE] = ACTIONS(2039),
    [anon_sym_DASH] = ACTIONS(2037),
    [anon_sym_PLUS] = ACTIONS(2037),
    [anon_sym_DASH_DASH] = ACTIONS(2039),
    [anon_sym_PLUS_PLUS] = ACTIONS(2039),
    [anon_sym_DQUOTE] = ACTIONS(2039),
    [anon_sym_SQUOTE] = ACTIONS(2039),
    [sym_number_literal] = ACTIONS(2039),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(336)] = {
    [sym_identifier] = ACTIONS(2041),
    [sym_program_name_keyword] = ACTIONS(2041),
    [sym_module_name_keyword] = ACTIONS(2041),
    [sym_define_device_keyword] = ACTIONS(2041),
    [sym_define_combine_keyword] = ACTIONS(2041),
    [sym_define_constant_keyword] = ACTIONS(2041),
    [sym_define_type_keyword] = ACTIONS(2041),
    [sym_define_variable_keyword] = ACTIONS(2041),
    [sym_define_system_variable_keyword] = ACTIONS(2041),
    [sym_define_connect_level_keyword] = ACTIONS(2041),
    [sym_define_latching_keyword] = ACTIONS(2041),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2041),
    [sym_define_toggling_keyword] = ACTIONS(2041),
    [sym_define_call_keyword] = ACTIONS(2041),
    [sym_define_function_keyword] = ACTIONS(2041),
    [sym_define_library_function_keyword] = ACTIONS(2041),
    [sym_define_start_keyword] = ACTIONS(2041),
    [sym_define_event_keyword] = ACTIONS(2041),
    [sym_define_program_keyword] = ACTIONS(2041),
    [sym_define_module_keyword] = ACTIONS(2041),
    [sym_char_keyword] = ACTIONS(2041),
    [sym_widechar_keyword] = ACTIONS(2041),
    [sym_integer_keyword] = ACTIONS(2041),
    [sym_sinteger_keyword] = ACTIONS(2041),
    [sym_long_keyword] = ACTIONS(2041),
    [sym_slong_keyword] = ACTIONS(2041),
    [sym_float_keyword] = ACTIONS(2041),
    [sym_double_keyword] = ACTIONS(2041),
    [sym_variant_keyword] = ACTIONS(2041),
    [sym_variantarray_keyword] = ACTIONS(2041),
    [sym_local_var_keyword] = ACTIONS(2041),
    [sym_stack_var_keyword] = ACTIONS(2041),
    [sym_constant_keyword] = ACTIONS(2041),
    [sym_volatile_keyword] = ACTIONS(2041),
    [sym_non_volatile_keyword] = ACTIONS(2041),
    [sym_persistent_keyword] = ACTIONS(2041),
    [sym_dev_keyword] = ACTIONS(2041),
    [sym_devlev_keyword] = ACTIONS(2041),
    [sym_devchan_keyword] = ACTIONS(2041),
    [sym_if_keyword] = ACTIONS(2041),
    [sym_switch_keyword] = ACTIONS(2041),
    [sym_case_keyword] = ACTIONS(2041),
    [sym_default_keyword] = ACTIONS(2041),
    [sym_while_keyword] = ACTIONS(2041),
    [sym_for_keyword] = ACTIONS(2041),
    [sym_break_keyword] = ACTIONS(2041),
    [sym_continue_keyword] = ACTIONS(2041),
    [sym_return_keyword] = ACTIONS(2041),
    [sym_select_keyword] = ACTIONS(2041),
    [sym_active_keyword] = ACTIONS(2041),
    [sym_struct_keyword] = ACTIONS(2041),
    [sym_structure_keyword] = ACTIONS(2041),
    [sym_button_event_keyword] = ACTIONS(2041),
    [sym_channel_event_keyword] = ACTIONS(2041),
    [sym_level_event_keyword] = ACTIONS(2041),
    [sym_data_event_keyword] = ACTIONS(2041),
    [sym_timeline_event_keyword] = ACTIONS(2041),
    [sym_custom_event_keyword] = ACTIONS(2041),
    [sym_push_keyword] = ACTIONS(2041),
    [sym_release_keyword] = ACTIONS(2041),
    [aux_sym_on_keyword_token1] = ACTIONS(2041),
    [aux_sym_off_keyword_token1] = ACTIONS(2041),
    [sym_send_level_keyword] = ACTIONS(2041),
    [sym_send_string_keyword] = ACTIONS(2041),
    [sym_send_command_keyword] = ACTIONS(2041),
    [sym_clear_buffer_keyword] = ACTIONS(2041),
    [sym_create_buffer_keyword] = ACTIONS(2041),
    [sym_create_multi_buffer_keyword] = ACTIONS(2041),
    [sym_call_keyword] = ACTIONS(2041),
    [sym_system_call_keyword] = ACTIONS(2041),
    [sym_devchan_to_keyword] = ACTIONS(2041),
    [sym_devchan_min_to_keyword] = ACTIONS(2041),
    [sym_devchan_total_off_keyword] = ACTIONS(2041),
    [sym_devchan_pulse_keyword] = ACTIONS(2041),
    [sym_wait_keyword] = ACTIONS(2041),
    [sym_cancel_wait_keyword] = ACTIONS(2041),
    [sym_wait_until_keyword] = ACTIONS(2041),
    [sym_cancel_wait_until_keyword] = ACTIONS(2041),
    [sym_cancel_all_wait_keyword] = ACTIONS(2041),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2041),
    [sym_not] = ACTIONS(2041),
    [sym_bnot] = ACTIONS(2041),
    [sym___file__] = ACTIONS(2041),
    [sym___date__] = ACTIONS(2041),
    [sym___ldate__] = ACTIONS(2041),
    [sym___time__] = ACTIONS(2041),
    [sym___line__] = ACTIONS(2041),
    [sym___name__] = ACTIONS(2041),
    [sym___netlinx__] = ACTIONS(2041),
    [sym_netlinx_axi_version] = ACTIONS(2041),
    [sym_url_flg_tcp] = ACTIONS(2041),
    [sym_url_flg_accinfopresent] = ACTIONS(2041),
    [sym_url_flg_temp] = ACTIONS(2041),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2041),
    [sym_url_flg_stat_mask] = ACTIONS(2041),
    [sym_url_flg_stat_lookup] = ACTIONS(2041),
    [sym_url_flg_stat_connecting] = ACTIONS(2041),
    [sym_url_flg_stat_waiting] = ACTIONS(2041),
    [sym_url_flg_stat_connected] = ACTIONS(2041),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2041),
    [sym_first_virtual_device] = ACTIONS(2041),
    [sym_first_local_port] = ACTIONS(2041),
    [sym_dynamic_virtual_device] = ACTIONS(2041),
    [sym_timeline_once] = ACTIONS(2041),
    [sym_timeline_repeat] = ACTIONS(2041),
    [sym_timeline_absolute] = ACTIONS(2041),
    [sym_timeline_relative] = ACTIONS(2041),
    [sym_do_push_timed_infinite] = ACTIONS(2041),
    [sym_source_type_no_address] = ACTIONS(2041),
    [sym_source_type_neuron_id] = ACTIONS(2041),
    [sym_source_type_ip_address] = ACTIONS(2041),
    [sym_source_type_axlink] = ACTIONS(2041),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2041),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2041),
    [sym_source_type_ip_socket_address] = ACTIONS(2041),
    [sym_source_type_rs232] = ACTIONS(2041),
    [sym_source_type_internal] = ACTIONS(2041),
    [sym_source_type_mac_address] = ACTIONS(2041),
    [sym_source_type_ipv4_port] = ACTIONS(2041),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2041),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2041),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2041),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2041),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2041),
    [sym_true] = ACTIONS(2041),
    [sym_false] = ACTIONS(2041),
    [sym_file_read_only] = ACTIONS(2041),
    [sym_file_rw_new] = ACTIONS(2041),
    [sym_file_rw_append] = ACTIONS(2041),
    [sym_ip_tcp] = ACTIONS(2041),
    [sym_ip_udp] = ACTIONS(2041),
    [sym_ip_udp_2way] = ACTIONS(2041),
    [sym_xml_encode_types] = ACTIONS(2041),
    [sym_xml_encode_char_as_list] = ACTIONS(2041),
    [sym_xml_encode_le] = ACTIONS(2041),
    [sym_xml_decode_types] = ACTIONS(2041),
    [sym_xml_decode_no_preserve] = ACTIONS(2041),
    [sym_duet_dev_type_display_device] = ACTIONS(2041),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2041),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2041),
    [sym_duet_dev_type_io_device] = ACTIONS(2041),
    [sym_duet_dev_type_relay_device] = ACTIONS(2041),
    [sym_duet_dev_type_ups] = ACTIONS(2041),
    [sym_duet_dev_type_amplifier] = ACTIONS(2041),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2041),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2041),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2041),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2041),
    [sym_duet_dev_type_camera] = ACTIONS(2041),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2041),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2041),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2041),
    [sym_duet_dev_type_dss] = ACTIONS(2041),
    [sym_duet_dev_type_dvr] = ACTIONS(2041),
    [sym_duet_dev_type_disc_device] = ACTIONS(2041),
    [sym_duet_dev_type_document_camera] = ACTIONS(2041),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2041),
    [sym_duet_dev_type_hvac] = ACTIONS(2041),
    [sym_duet_dev_type_keypad] = ACTIONS(2041),
    [sym_duet_dev_type_light] = ACTIONS(2041),
    [sym_duet_dev_type_monitor] = ACTIONS(2041),
    [sym_duet_dev_type_motor] = ACTIONS(2041),
    [sym_duet_dev_type_multi_window] = ACTIONS(2041),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2041),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2041),
    [sym_duet_dev_type_receiver] = ACTIONS(2041),
    [sym_duet_dev_type_security_system] = ACTIONS(2041),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2041),
    [sym_duet_dev_type_settop_box] = ACTIONS(2041),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2041),
    [sym_duet_dev_type_switcher] = ACTIONS(2041),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2041),
    [sym_duet_dev_type_tv] = ACTIONS(2041),
    [sym_duet_dev_type_utility] = ACTIONS(2041),
    [sym_duet_dev_type_vcr] = ACTIONS(2041),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2041),
    [sym_duet_dev_type_video_processor] = ACTIONS(2041),
    [sym_duet_dev_type_video_projector] = ACTIONS(2041),
    [sym_duet_dev_type_video_wall] = ACTIONS(2041),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2041),
    [sym_duet_dev_type_weather] = ACTIONS(2041),
    [sym_duet_dev_not_polled] = ACTIONS(2041),
    [sym_duet_dev_polled] = ACTIONS(2041),
    [sym_clkmgr_mode_network] = ACTIONS(2041),
    [sym_clkmgr_mode_standalone] = ACTIONS(2041),
    [sym_normal_standby] = ACTIONS(2041),
    [sym_normal_wake] = ACTIONS(2041),
    [sym_ip_multicast_ttl_option] = ACTIONS(2041),
    [sym_ip_ttl_subnet] = ACTIONS(2041),
    [sym_ip_ttl_site] = ACTIONS(2041),
    [sym_ip_ttl_region] = ACTIONS(2041),
    [sym_ip_ttl_continent] = ACTIONS(2041),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2041),
    [sym_ip_nodelay_on] = ACTIONS(2041),
    [sym_ip_nodelay_off] = ACTIONS(2041),
    [sym_amx_error] = ACTIONS(2041),
    [sym_amx_warning] = ACTIONS(2041),
    [sym_amx_info] = ACTIONS(2041),
    [sym_amx_debug] = ACTIONS(2041),
    [sym_netlinx_login_success] = ACTIONS(2041),
    [sym_netlinx_login_fail] = ACTIONS(2041),
    [sym_netlinx_logout] = ACTIONS(2041),
    [sym_null_str] = ACTIONS(2041),
    [sym_smtp_address] = ACTIONS(2041),
    [sym_smtp_port_number] = ACTIONS(2041),
    [sym_smtp_username] = ACTIONS(2041),
    [sym_smtp_password] = ACTIONS(2041),
    [sym_smtp_require_tls] = ACTIONS(2041),
    [sym_smtp_from] = ACTIONS(2041),
    [sym_smtp_tls_true] = ACTIONS(2041),
    [sym_smtp_tls_false] = ACTIONS(2041),
    [sym_listview_on_row_select_event] = ACTIONS(2041),
    [sym_data_structure_unknown] = ACTIONS(2041),
    [sym_data_structure_datafeed] = ACTIONS(2041),
    [sym_data_structure_datarecordset] = ACTIONS(2041),
    [sym_data_structure_datarecord] = ACTIONS(2041),
    [sym_data_structure_datafield] = ACTIONS(2041),
    [sym_data_max_content_field_count] = ACTIONS(2041),
    [sym_data_max_metadata_field_count] = ACTIONS(2041),
    [sym_data_max_id_length] = ACTIONS(2041),
    [sym_data_max_name_length] = ACTIONS(2041),
    [sym_data_max_description_length] = ACTIONS(2041),
    [sym_data_max_type_length] = ACTIONS(2041),
    [sym_data_max_format_length] = ACTIONS(2041),
    [sym_data_max_source_length] = ACTIONS(2041),
    [sym_data_max_value_length] = ACTIONS(2041),
    [sym_data_max_label_length] = ACTIONS(2041),
    [sym_data_max_url_length] = ACTIONS(2041),
    [sym_max_last_login_info_length] = ACTIONS(2041),
    [sym_data_type_unknown] = ACTIONS(2041),
    [sym_data_type_string] = ACTIONS(2041),
    [sym_data_type_datetime] = ACTIONS(2041),
    [sym_data_type_date] = ACTIONS(2041),
    [sym_data_type_time] = ACTIONS(2041),
    [sym_data_type_image] = ACTIONS(2041),
    [sym_data_format_url] = ACTIONS(2041),
    [sym_data_format_phone] = ACTIONS(2041),
    [sym_data_format_email] = ACTIONS(2041),
    [sym_data_format_iso8601] = ACTIONS(2041),
    [sym_auditlog_priv] = ACTIONS(2041),
    [sym_configuration_priv] = ACTIONS(2041),
    [sym_deviceconfig_priv] = ACTIONS(2041),
    [sym_ftp_priv] = ACTIONS(2041),
    [sym_http_priv] = ACTIONS(2041),
    [sym_networkconfig_priv] = ACTIONS(2041),
    [sym_programport_priv] = ACTIONS(2041),
    [sym_remoteui_priv] = ACTIONS(2041),
    [sym_securitycontrol_priv] = ACTIONS(2041),
    [sym_softwaremanagement_priv] = ACTIONS(2041),
    [sym_terminal_priv] = ACTIONS(2041),
    [sym_tpadmin_priv] = ACTIONS(2041),
    [sym_user1_priv] = ACTIONS(2041),
    [sym_user2_priv] = ACTIONS(2041),
    [sym_user3_priv] = ACTIONS(2041),
    [sym_user4_priv] = ACTIONS(2041),
    [sym_usermanagement_priv] = ACTIONS(2041),
    [sym_valid_account] = ACTIONS(2041),
    [sym_err_name_invalid] = ACTIONS(2041),
    [sym_err_password_invalid] = ACTIONS(2041),
    [sym_err_invalid_account] = ACTIONS(2041),
    [sym_err_no_authorization] = ACTIONS(2041),
    [sym_err_invalid_parameter] = ACTIONS(2041),
    [sym_err_user_locked_out] = ACTIONS(2041),
    [sym_err_user_account_expired] = ACTIONS(2041),
    [sym_err_general_error] = ACTIONS(2041),
    [sym_err_authentication_error] = ACTIONS(2041),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2041),
    [sym_tls_validate_certificate] = ACTIONS(2041),
    [sym_tdata] = ACTIONS(2041),
    [sym_tchannel] = ACTIONS(2041),
    [sym_tlevel] = ACTIONS(2041),
    [sym_tbutton] = ACTIONS(2041),
    [sym_ttimeline] = ACTIONS(2041),
    [sym_tcustom] = ACTIONS(2041),
    [sym_url_struct] = ACTIONS(2041),
    [sym_dns_struct] = ACTIONS(2041),
    [sym_ip_address_struct] = ACTIONS(2041),
    [sym_dev_info_struct] = ACTIONS(2041),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2041),
    [sym_data_feed] = ACTIONS(2041),
    [sym_data_field] = ACTIONS(2041),
    [sym_data_record] = ACTIONS(2041),
    [sym_wc_data_feed] = ACTIONS(2041),
    [sym_wc_data_field] = ACTIONS(2041),
    [sym_wc_data_record] = ACTIONS(2041),
    [sym_last_login_info] = ACTIONS(2041),
    [sym____reserved___] = ACTIONS(2041),
    [sym_date] = ACTIONS(2041),
    [sym_day] = ACTIONS(2041),
    [sym_time] = ACTIONS(2041),
    [sym_ldate] = ACTIONS(2041),
    [sym_push_device] = ACTIONS(2041),
    [sym_push_channel] = ACTIONS(2041),
    [sym_push_devchan] = ACTIONS(2041),
    [sym_release_device] = ACTIONS(2041),
    [sym_release_channel] = ACTIONS(2041),
    [sym_release_devchan] = ACTIONS(2041),
    [sym_master_slot] = ACTIONS(2041),
    [sym_get_pulse_time] = ACTIONS(2041),
    [sym_get_timer] = ACTIONS(2041),
    [sym_master_sn] = ACTIONS(2041),
    [sym_system_number] = ACTIONS(2041),
    [sym_button] = ACTIONS(2041),
    [sym_dv_channel] = ACTIONS(2041),
    [sym_data] = ACTIONS(2041),
    [sym_level] = ACTIONS(2041),
    [sym_timeline] = ACTIONS(2041),
    [sym_channel] = ACTIONS(2041),
    [sym_custom] = ACTIONS(2041),
    [sym_atoi] = ACTIONS(2041),
    [sym_atol] = ACTIONS(2041),
    [sym_atof] = ACTIONS(2041),
    [sym_itoa] = ACTIONS(2041),
    [sym_itohex] = ACTIONS(2041),
    [sym_hextoi] = ACTIONS(2041),
    [sym_ftoa] = ACTIONS(2041),
    [sym_find_string] = ACTIONS(2041),
    [sym_left_string] = ACTIONS(2041),
    [sym_right_string] = ACTIONS(2041),
    [sym_mid_string] = ACTIONS(2041),
    [sym_remove_string] = ACTIONS(2041),
    [sym_get_buffer_string] = ACTIONS(2041),
    [sym_get_buffer_char] = ACTIONS(2041),
    [sym_get_multi_buffer_string] = ACTIONS(2041),
    [sym_length_string] = ACTIONS(2041),
    [sym_max_length_string] = ACTIONS(2041),
    [sym_set_length_string] = ACTIONS(2041),
    [sym_compare_string] = ACTIONS(2041),
    [sym_upper_string] = ACTIONS(2041),
    [sym_lower_string] = ACTIONS(2041),
    [sym_get_last] = ACTIONS(2041),
    [sym_date_to_day] = ACTIONS(2041),
    [sym_date_to_month] = ACTIONS(2041),
    [sym_date_to_year] = ACTIONS(2041),
    [sym_time_to_hour] = ACTIONS(2041),
    [sym_time_to_minute] = ACTIONS(2041),
    [sym_time_to_second] = ACTIONS(2041),
    [sym_day_of_week] = ACTIONS(2041),
    [sym_set_pulse_time] = ACTIONS(2041),
    [sym_set_timer] = ACTIONS(2041),
    [sym_file_open] = ACTIONS(2041),
    [sym_file_close] = ACTIONS(2041),
    [sym_file_read] = ACTIONS(2041),
    [sym_file_read_line] = ACTIONS(2041),
    [sym_file_write] = ACTIONS(2041),
    [sym_file_write_line] = ACTIONS(2041),
    [sym_file_copy] = ACTIONS(2041),
    [sym_file_delete] = ACTIONS(2041),
    [sym_file_rename] = ACTIONS(2041),
    [sym_file_dir] = ACTIONS(2041),
    [sym_file_getdir] = ACTIONS(2041),
    [sym_file_setdir] = ACTIONS(2041),
    [sym_file_create_dir] = ACTIONS(2041),
    [sym_file_remove_dir] = ACTIONS(2041),
    [sym_file_seek] = ACTIONS(2041),
    [sym_length_array] = ACTIONS(2041),
    [sym_max_length_array] = ACTIONS(2041),
    [sym_set_length_array] = ACTIONS(2041),
    [sym_ip_client_open] = ACTIONS(2041),
    [sym_ip_client_close] = ACTIONS(2041),
    [sym_ip_server_open] = ACTIONS(2041),
    [sym_ip_server_close] = ACTIONS(2041),
    [sym_ip_bound_client_open] = ACTIONS(2041),
    [sym_ip_mc_server_open] = ACTIONS(2041),
    [sym_ip_set_option] = ACTIONS(2041),
    [sym_ssh_client_open] = ACTIONS(2041),
    [sym_ssh_client_close] = ACTIONS(2041),
    [sym_tls_client_open] = ACTIONS(2041),
    [sym_tls_client_close] = ACTIONS(2041),
    [sym_do_push] = ACTIONS(2041),
    [sym_do_push_timed] = ACTIONS(2041),
    [sym_do_release] = ACTIONS(2041),
    [sym_device_id] = ACTIONS(2041),
    [sym_device_info] = ACTIONS(2041),
    [sym_device_id_string] = ACTIONS(2041),
    [sym_device_standby] = ACTIONS(2041),
    [sym_device_wake] = ACTIONS(2041),
    [sym_timeline_create] = ACTIONS(2041),
    [sym_timeline_kill] = ACTIONS(2041),
    [sym_timeline_reload] = ACTIONS(2041),
    [sym_timeline_pause] = ACTIONS(2041),
    [sym_timeline_restart] = ACTIONS(2041),
    [sym_timeline_set] = ACTIONS(2041),
    [sym_timeline_get] = ACTIONS(2041),
    [sym_timeline_active] = ACTIONS(2041),
    [sym_timeline_dynamic_id] = ACTIONS(2041),
    [sym_variable_to_string] = ACTIONS(2041),
    [sym_string_to_variable] = ACTIONS(2041),
    [sym_length_variable_to_string] = ACTIONS(2041),
    [sym_type_cast] = ACTIONS(2041),
    [sym_variable_to_xml] = ACTIONS(2041),
    [sym_xml_to_variable] = ACTIONS(2041),
    [sym_length_variable_to_xml] = ACTIONS(2041),
    [sym_redirect_string] = ACTIONS(2041),
    [sym_random_number] = ACTIONS(2041),
    [sym_get_unique_id] = ACTIONS(2041),
    [sym_get_system_number] = ACTIONS(2041),
    [sym_set_system_number] = ACTIONS(2041),
    [sym_get_serial_number] = ACTIONS(2041),
    [sym_get_master_build] = ACTIONS(2041),
    [sym_set_validation_code] = ACTIONS(2041),
    [sym_reboot] = ACTIONS(2041),
    [sym_load_duet_module] = ACTIONS(2041),
    [sym_unload_duet_module] = ACTIONS(2041),
    [sym_duet_mem_size_set] = ACTIONS(2041),
    [sym_duet_mem_size_get] = ACTIONS(2041),
    [sym_dynamic_application_device] = ACTIONS(2041),
    [sym_dynamic_polled_port] = ACTIONS(2041),
    [sym_static_port_binding] = ACTIONS(2041),
    [sym_static_ip_binding] = ACTIONS(2041),
    [sym_get_dns_list] = ACTIONS(2041),
    [sym_set_dns_list] = ACTIONS(2041),
    [sym_get_ip_address] = ACTIONS(2041),
    [sym_set_ip_address] = ACTIONS(2041),
    [sym_get_url_list] = ACTIONS(2041),
    [sym_add_url_entry] = ACTIONS(2041),
    [sym_delete_url_entry] = ACTIONS(2041),
    [sym_abs_value] = ACTIONS(2041),
    [sym_max_value] = ACTIONS(2041),
    [sym_min_value] = ACTIONS(2041),
    [sym_exp_value] = ACTIONS(2041),
    [sym_log_value] = ACTIONS(2041),
    [sym_log10_value] = ACTIONS(2041),
    [sym_power_value] = ACTIONS(2041),
    [sym_sqrt_value] = ACTIONS(2041),
    [sym_raw_be] = ACTIONS(2041),
    [sym_raw_le] = ACTIONS(2041),
    [sym_internal_queue_size_set] = ACTIONS(2041),
    [sym_internal_queue_size_get] = ACTIONS(2041),
    [sym_internal_threshold_set] = ACTIONS(2041),
    [sym_internal_threshold_get] = ACTIONS(2041),
    [sym_rebuilt_event] = ACTIONS(2041),
    [sym_do_custom_event] = ACTIONS(2041),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2041),
    [sym_clkmgr_set_clk_source] = ACTIONS(2041),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2041),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2041),
    [sym_clkmgr_get_timezone] = ACTIONS(2041),
    [sym_clkmgr_set_timezone] = ACTIONS(2041),
    [sym_clkmgr_get_resync_period] = ACTIONS(2041),
    [sym_clkmgr_set_resync_period] = ACTIONS(2041),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2041),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2041),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2041),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2041),
    [sym_clkmgr_get_timeservers] = ACTIONS(2041),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2041),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2041),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2041),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2041),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2041),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2041),
    [sym_set_log_level] = ACTIONS(2041),
    [sym_get_log_level] = ACTIONS(2041),
    [sym_amx_log] = ACTIONS(2041),
    [sym_smtp_server_config_set] = ACTIONS(2041),
    [sym_smtp_server_config_get] = ACTIONS(2041),
    [sym_smtp_send] = ACTIONS(2041),
    [sym_validate_netlinx_account] = ACTIONS(2041),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2041),
    [sym_authenticate_certificate] = ACTIONS(2041),
    [sym_audit_netlinx_session_event] = ACTIONS(2041),
    [sym_audit_netlinx_generic_event] = ACTIONS(2041),
    [sym_get_available_flash_disk_space] = ACTIONS(2041),
    [sym_get_max_flash_disk_space] = ACTIONS(2041),
    [sym_data_create_feed] = ACTIONS(2041),
    [sym_data_delete_feed] = ACTIONS(2041),
    [sym_data_publish_feed] = ACTIONS(2041),
    [sym_data_get_published_feed] = ACTIONS(2041),
    [sym_data_add_record] = ACTIONS(2041),
    [sym_data_get_event_record] = ACTIONS(2041),
    [sym__wc_data_create_feed] = ACTIONS(2041),
    [sym__wc_data_add_record] = ACTIONS(2041),
    [sym__wc_data_get_event_record] = ACTIONS(2041),
    [sym_format] = ACTIONS(2041),
    [sym_astro_clock] = ACTIONS(2041),
    [sym_set_outdoor_temperature] = ACTIONS(2041),
    [sym_set_virtual_level_count] = ACTIONS(2041),
    [sym_set_virtual_channel_count] = ACTIONS(2041),
    [sym_set_virtual_port_count] = ACTIONS(2041),
    [sym__wc_encode_int] = ACTIONS(2041),
    [sym__wc_decode_int] = ACTIONS(2041),
    [sym__wc_int] = ACTIONS(2041),
    [sym__wc_to_ch_int] = ACTIONS(2041),
    [sym__ch_to_wc_int] = ACTIONS(2041),
    [sym__wc_find_string_int] = ACTIONS(2041),
    [sym__wc_left_string_int] = ACTIONS(2041),
    [sym__wc_lower_string_int] = ACTIONS(2041),
    [sym__wc_mid_string_int] = ACTIONS(2041),
    [sym__wc_remove_string_int] = ACTIONS(2041),
    [sym__wc_right_string_int] = ACTIONS(2041),
    [sym__wc_upper_string_int] = ACTIONS(2041),
    [sym__wc_compare_string_int] = ACTIONS(2041),
    [sym__wc_get_buffer_char_int] = ACTIONS(2041),
    [sym__wc_get_buffer_string_int] = ACTIONS(2041),
    [sym__wc_concat_string_int] = ACTIONS(2041),
    [sym__wc_file_open_int] = ACTIONS(2041),
    [sym__wc_file_close_int] = ACTIONS(2041),
    [sym__wc_file_read_int] = ACTIONS(2041),
    [sym__wc_file_read_line_int] = ACTIONS(2041),
    [sym__wc_file_write_int] = ACTIONS(2041),
    [sym__wc_file_write_line_int] = ACTIONS(2041),
    [aux_sym_preproc_include_token1] = ACTIONS(2041),
    [aux_sym_preproc_define_token1] = ACTIONS(2043),
    [aux_sym_preproc_warn_token1] = ACTIONS(2043),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2043),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2043),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2043),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2043),
    [aux_sym_preproc_else_token1] = ACTIONS(2043),
    [anon_sym_LBRACK] = ACTIONS(2043),
    [anon_sym_LBRACE] = ACTIONS(2043),
    [anon_sym_LPAREN] = ACTIONS(2041),
    [anon_sym_BANG] = ACTIONS(2043),
    [anon_sym_TILDE] = ACTIONS(2043),
    [anon_sym_DASH] = ACTIONS(2041),
    [anon_sym_PLUS] = ACTIONS(2041),
    [anon_sym_DASH_DASH] = ACTIONS(2043),
    [anon_sym_PLUS_PLUS] = ACTIONS(2043),
    [anon_sym_DQUOTE] = ACTIONS(2043),
    [anon_sym_SQUOTE] = ACTIONS(2043),
    [sym_number_literal] = ACTIONS(2043),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(337)] = {
    [sym_identifier] = ACTIONS(2045),
    [sym_program_name_keyword] = ACTIONS(2045),
    [sym_module_name_keyword] = ACTIONS(2045),
    [sym_define_device_keyword] = ACTIONS(2045),
    [sym_define_combine_keyword] = ACTIONS(2045),
    [sym_define_constant_keyword] = ACTIONS(2045),
    [sym_define_type_keyword] = ACTIONS(2045),
    [sym_define_variable_keyword] = ACTIONS(2045),
    [sym_define_system_variable_keyword] = ACTIONS(2045),
    [sym_define_connect_level_keyword] = ACTIONS(2045),
    [sym_define_latching_keyword] = ACTIONS(2045),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2045),
    [sym_define_toggling_keyword] = ACTIONS(2045),
    [sym_define_call_keyword] = ACTIONS(2045),
    [sym_define_function_keyword] = ACTIONS(2045),
    [sym_define_library_function_keyword] = ACTIONS(2045),
    [sym_define_start_keyword] = ACTIONS(2045),
    [sym_define_event_keyword] = ACTIONS(2045),
    [sym_define_program_keyword] = ACTIONS(2045),
    [sym_define_module_keyword] = ACTIONS(2045),
    [sym_char_keyword] = ACTIONS(2045),
    [sym_widechar_keyword] = ACTIONS(2045),
    [sym_integer_keyword] = ACTIONS(2045),
    [sym_sinteger_keyword] = ACTIONS(2045),
    [sym_long_keyword] = ACTIONS(2045),
    [sym_slong_keyword] = ACTIONS(2045),
    [sym_float_keyword] = ACTIONS(2045),
    [sym_double_keyword] = ACTIONS(2045),
    [sym_variant_keyword] = ACTIONS(2045),
    [sym_variantarray_keyword] = ACTIONS(2045),
    [sym_local_var_keyword] = ACTIONS(2045),
    [sym_stack_var_keyword] = ACTIONS(2045),
    [sym_constant_keyword] = ACTIONS(2045),
    [sym_volatile_keyword] = ACTIONS(2045),
    [sym_non_volatile_keyword] = ACTIONS(2045),
    [sym_persistent_keyword] = ACTIONS(2045),
    [sym_dev_keyword] = ACTIONS(2045),
    [sym_devlev_keyword] = ACTIONS(2045),
    [sym_devchan_keyword] = ACTIONS(2045),
    [sym_if_keyword] = ACTIONS(2045),
    [sym_switch_keyword] = ACTIONS(2045),
    [sym_case_keyword] = ACTIONS(2045),
    [sym_default_keyword] = ACTIONS(2045),
    [sym_while_keyword] = ACTIONS(2045),
    [sym_for_keyword] = ACTIONS(2045),
    [sym_break_keyword] = ACTIONS(2045),
    [sym_continue_keyword] = ACTIONS(2045),
    [sym_return_keyword] = ACTIONS(2045),
    [sym_select_keyword] = ACTIONS(2045),
    [sym_active_keyword] = ACTIONS(2045),
    [sym_struct_keyword] = ACTIONS(2045),
    [sym_structure_keyword] = ACTIONS(2045),
    [sym_button_event_keyword] = ACTIONS(2045),
    [sym_channel_event_keyword] = ACTIONS(2045),
    [sym_level_event_keyword] = ACTIONS(2045),
    [sym_data_event_keyword] = ACTIONS(2045),
    [sym_timeline_event_keyword] = ACTIONS(2045),
    [sym_custom_event_keyword] = ACTIONS(2045),
    [sym_push_keyword] = ACTIONS(2045),
    [sym_release_keyword] = ACTIONS(2045),
    [aux_sym_on_keyword_token1] = ACTIONS(2045),
    [aux_sym_off_keyword_token1] = ACTIONS(2045),
    [sym_send_level_keyword] = ACTIONS(2045),
    [sym_send_string_keyword] = ACTIONS(2045),
    [sym_send_command_keyword] = ACTIONS(2045),
    [sym_clear_buffer_keyword] = ACTIONS(2045),
    [sym_create_buffer_keyword] = ACTIONS(2045),
    [sym_create_multi_buffer_keyword] = ACTIONS(2045),
    [sym_call_keyword] = ACTIONS(2045),
    [sym_system_call_keyword] = ACTIONS(2045),
    [sym_devchan_to_keyword] = ACTIONS(2045),
    [sym_devchan_min_to_keyword] = ACTIONS(2045),
    [sym_devchan_total_off_keyword] = ACTIONS(2045),
    [sym_devchan_pulse_keyword] = ACTIONS(2045),
    [sym_wait_keyword] = ACTIONS(2045),
    [sym_cancel_wait_keyword] = ACTIONS(2045),
    [sym_wait_until_keyword] = ACTIONS(2045),
    [sym_cancel_wait_until_keyword] = ACTIONS(2045),
    [sym_cancel_all_wait_keyword] = ACTIONS(2045),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2045),
    [sym_not] = ACTIONS(2045),
    [sym_bnot] = ACTIONS(2045),
    [sym___file__] = ACTIONS(2045),
    [sym___date__] = ACTIONS(2045),
    [sym___ldate__] = ACTIONS(2045),
    [sym___time__] = ACTIONS(2045),
    [sym___line__] = ACTIONS(2045),
    [sym___name__] = ACTIONS(2045),
    [sym___netlinx__] = ACTIONS(2045),
    [sym_netlinx_axi_version] = ACTIONS(2045),
    [sym_url_flg_tcp] = ACTIONS(2045),
    [sym_url_flg_accinfopresent] = ACTIONS(2045),
    [sym_url_flg_temp] = ACTIONS(2045),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2045),
    [sym_url_flg_stat_mask] = ACTIONS(2045),
    [sym_url_flg_stat_lookup] = ACTIONS(2045),
    [sym_url_flg_stat_connecting] = ACTIONS(2045),
    [sym_url_flg_stat_waiting] = ACTIONS(2045),
    [sym_url_flg_stat_connected] = ACTIONS(2045),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2045),
    [sym_first_virtual_device] = ACTIONS(2045),
    [sym_first_local_port] = ACTIONS(2045),
    [sym_dynamic_virtual_device] = ACTIONS(2045),
    [sym_timeline_once] = ACTIONS(2045),
    [sym_timeline_repeat] = ACTIONS(2045),
    [sym_timeline_absolute] = ACTIONS(2045),
    [sym_timeline_relative] = ACTIONS(2045),
    [sym_do_push_timed_infinite] = ACTIONS(2045),
    [sym_source_type_no_address] = ACTIONS(2045),
    [sym_source_type_neuron_id] = ACTIONS(2045),
    [sym_source_type_ip_address] = ACTIONS(2045),
    [sym_source_type_axlink] = ACTIONS(2045),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2045),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2045),
    [sym_source_type_ip_socket_address] = ACTIONS(2045),
    [sym_source_type_rs232] = ACTIONS(2045),
    [sym_source_type_internal] = ACTIONS(2045),
    [sym_source_type_mac_address] = ACTIONS(2045),
    [sym_source_type_ipv4_port] = ACTIONS(2045),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2045),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2045),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2045),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2045),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2045),
    [sym_true] = ACTIONS(2045),
    [sym_false] = ACTIONS(2045),
    [sym_file_read_only] = ACTIONS(2045),
    [sym_file_rw_new] = ACTIONS(2045),
    [sym_file_rw_append] = ACTIONS(2045),
    [sym_ip_tcp] = ACTIONS(2045),
    [sym_ip_udp] = ACTIONS(2045),
    [sym_ip_udp_2way] = ACTIONS(2045),
    [sym_xml_encode_types] = ACTIONS(2045),
    [sym_xml_encode_char_as_list] = ACTIONS(2045),
    [sym_xml_encode_le] = ACTIONS(2045),
    [sym_xml_decode_types] = ACTIONS(2045),
    [sym_xml_decode_no_preserve] = ACTIONS(2045),
    [sym_duet_dev_type_display_device] = ACTIONS(2045),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2045),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2045),
    [sym_duet_dev_type_io_device] = ACTIONS(2045),
    [sym_duet_dev_type_relay_device] = ACTIONS(2045),
    [sym_duet_dev_type_ups] = ACTIONS(2045),
    [sym_duet_dev_type_amplifier] = ACTIONS(2045),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2045),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2045),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2045),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2045),
    [sym_duet_dev_type_camera] = ACTIONS(2045),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2045),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2045),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2045),
    [sym_duet_dev_type_dss] = ACTIONS(2045),
    [sym_duet_dev_type_dvr] = ACTIONS(2045),
    [sym_duet_dev_type_disc_device] = ACTIONS(2045),
    [sym_duet_dev_type_document_camera] = ACTIONS(2045),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2045),
    [sym_duet_dev_type_hvac] = ACTIONS(2045),
    [sym_duet_dev_type_keypad] = ACTIONS(2045),
    [sym_duet_dev_type_light] = ACTIONS(2045),
    [sym_duet_dev_type_monitor] = ACTIONS(2045),
    [sym_duet_dev_type_motor] = ACTIONS(2045),
    [sym_duet_dev_type_multi_window] = ACTIONS(2045),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2045),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2045),
    [sym_duet_dev_type_receiver] = ACTIONS(2045),
    [sym_duet_dev_type_security_system] = ACTIONS(2045),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2045),
    [sym_duet_dev_type_settop_box] = ACTIONS(2045),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2045),
    [sym_duet_dev_type_switcher] = ACTIONS(2045),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2045),
    [sym_duet_dev_type_tv] = ACTIONS(2045),
    [sym_duet_dev_type_utility] = ACTIONS(2045),
    [sym_duet_dev_type_vcr] = ACTIONS(2045),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2045),
    [sym_duet_dev_type_video_processor] = ACTIONS(2045),
    [sym_duet_dev_type_video_projector] = ACTIONS(2045),
    [sym_duet_dev_type_video_wall] = ACTIONS(2045),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2045),
    [sym_duet_dev_type_weather] = ACTIONS(2045),
    [sym_duet_dev_not_polled] = ACTIONS(2045),
    [sym_duet_dev_polled] = ACTIONS(2045),
    [sym_clkmgr_mode_network] = ACTIONS(2045),
    [sym_clkmgr_mode_standalone] = ACTIONS(2045),
    [sym_normal_standby] = ACTIONS(2045),
    [sym_normal_wake] = ACTIONS(2045),
    [sym_ip_multicast_ttl_option] = ACTIONS(2045),
    [sym_ip_ttl_subnet] = ACTIONS(2045),
    [sym_ip_ttl_site] = ACTIONS(2045),
    [sym_ip_ttl_region] = ACTIONS(2045),
    [sym_ip_ttl_continent] = ACTIONS(2045),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2045),
    [sym_ip_nodelay_on] = ACTIONS(2045),
    [sym_ip_nodelay_off] = ACTIONS(2045),
    [sym_amx_error] = ACTIONS(2045),
    [sym_amx_warning] = ACTIONS(2045),
    [sym_amx_info] = ACTIONS(2045),
    [sym_amx_debug] = ACTIONS(2045),
    [sym_netlinx_login_success] = ACTIONS(2045),
    [sym_netlinx_login_fail] = ACTIONS(2045),
    [sym_netlinx_logout] = ACTIONS(2045),
    [sym_null_str] = ACTIONS(2045),
    [sym_smtp_address] = ACTIONS(2045),
    [sym_smtp_port_number] = ACTIONS(2045),
    [sym_smtp_username] = ACTIONS(2045),
    [sym_smtp_password] = ACTIONS(2045),
    [sym_smtp_require_tls] = ACTIONS(2045),
    [sym_smtp_from] = ACTIONS(2045),
    [sym_smtp_tls_true] = ACTIONS(2045),
    [sym_smtp_tls_false] = ACTIONS(2045),
    [sym_listview_on_row_select_event] = ACTIONS(2045),
    [sym_data_structure_unknown] = ACTIONS(2045),
    [sym_data_structure_datafeed] = ACTIONS(2045),
    [sym_data_structure_datarecordset] = ACTIONS(2045),
    [sym_data_structure_datarecord] = ACTIONS(2045),
    [sym_data_structure_datafield] = ACTIONS(2045),
    [sym_data_max_content_field_count] = ACTIONS(2045),
    [sym_data_max_metadata_field_count] = ACTIONS(2045),
    [sym_data_max_id_length] = ACTIONS(2045),
    [sym_data_max_name_length] = ACTIONS(2045),
    [sym_data_max_description_length] = ACTIONS(2045),
    [sym_data_max_type_length] = ACTIONS(2045),
    [sym_data_max_format_length] = ACTIONS(2045),
    [sym_data_max_source_length] = ACTIONS(2045),
    [sym_data_max_value_length] = ACTIONS(2045),
    [sym_data_max_label_length] = ACTIONS(2045),
    [sym_data_max_url_length] = ACTIONS(2045),
    [sym_max_last_login_info_length] = ACTIONS(2045),
    [sym_data_type_unknown] = ACTIONS(2045),
    [sym_data_type_string] = ACTIONS(2045),
    [sym_data_type_datetime] = ACTIONS(2045),
    [sym_data_type_date] = ACTIONS(2045),
    [sym_data_type_time] = ACTIONS(2045),
    [sym_data_type_image] = ACTIONS(2045),
    [sym_data_format_url] = ACTIONS(2045),
    [sym_data_format_phone] = ACTIONS(2045),
    [sym_data_format_email] = ACTIONS(2045),
    [sym_data_format_iso8601] = ACTIONS(2045),
    [sym_auditlog_priv] = ACTIONS(2045),
    [sym_configuration_priv] = ACTIONS(2045),
    [sym_deviceconfig_priv] = ACTIONS(2045),
    [sym_ftp_priv] = ACTIONS(2045),
    [sym_http_priv] = ACTIONS(2045),
    [sym_networkconfig_priv] = ACTIONS(2045),
    [sym_programport_priv] = ACTIONS(2045),
    [sym_remoteui_priv] = ACTIONS(2045),
    [sym_securitycontrol_priv] = ACTIONS(2045),
    [sym_softwaremanagement_priv] = ACTIONS(2045),
    [sym_terminal_priv] = ACTIONS(2045),
    [sym_tpadmin_priv] = ACTIONS(2045),
    [sym_user1_priv] = ACTIONS(2045),
    [sym_user2_priv] = ACTIONS(2045),
    [sym_user3_priv] = ACTIONS(2045),
    [sym_user4_priv] = ACTIONS(2045),
    [sym_usermanagement_priv] = ACTIONS(2045),
    [sym_valid_account] = ACTIONS(2045),
    [sym_err_name_invalid] = ACTIONS(2045),
    [sym_err_password_invalid] = ACTIONS(2045),
    [sym_err_invalid_account] = ACTIONS(2045),
    [sym_err_no_authorization] = ACTIONS(2045),
    [sym_err_invalid_parameter] = ACTIONS(2045),
    [sym_err_user_locked_out] = ACTIONS(2045),
    [sym_err_user_account_expired] = ACTIONS(2045),
    [sym_err_general_error] = ACTIONS(2045),
    [sym_err_authentication_error] = ACTIONS(2045),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2045),
    [sym_tls_validate_certificate] = ACTIONS(2045),
    [sym_tdata] = ACTIONS(2045),
    [sym_tchannel] = ACTIONS(2045),
    [sym_tlevel] = ACTIONS(2045),
    [sym_tbutton] = ACTIONS(2045),
    [sym_ttimeline] = ACTIONS(2045),
    [sym_tcustom] = ACTIONS(2045),
    [sym_url_struct] = ACTIONS(2045),
    [sym_dns_struct] = ACTIONS(2045),
    [sym_ip_address_struct] = ACTIONS(2045),
    [sym_dev_info_struct] = ACTIONS(2045),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2045),
    [sym_data_feed] = ACTIONS(2045),
    [sym_data_field] = ACTIONS(2045),
    [sym_data_record] = ACTIONS(2045),
    [sym_wc_data_feed] = ACTIONS(2045),
    [sym_wc_data_field] = ACTIONS(2045),
    [sym_wc_data_record] = ACTIONS(2045),
    [sym_last_login_info] = ACTIONS(2045),
    [sym____reserved___] = ACTIONS(2045),
    [sym_date] = ACTIONS(2045),
    [sym_day] = ACTIONS(2045),
    [sym_time] = ACTIONS(2045),
    [sym_ldate] = ACTIONS(2045),
    [sym_push_device] = ACTIONS(2045),
    [sym_push_channel] = ACTIONS(2045),
    [sym_push_devchan] = ACTIONS(2045),
    [sym_release_device] = ACTIONS(2045),
    [sym_release_channel] = ACTIONS(2045),
    [sym_release_devchan] = ACTIONS(2045),
    [sym_master_slot] = ACTIONS(2045),
    [sym_get_pulse_time] = ACTIONS(2045),
    [sym_get_timer] = ACTIONS(2045),
    [sym_master_sn] = ACTIONS(2045),
    [sym_system_number] = ACTIONS(2045),
    [sym_button] = ACTIONS(2045),
    [sym_dv_channel] = ACTIONS(2045),
    [sym_data] = ACTIONS(2045),
    [sym_level] = ACTIONS(2045),
    [sym_timeline] = ACTIONS(2045),
    [sym_channel] = ACTIONS(2045),
    [sym_custom] = ACTIONS(2045),
    [sym_atoi] = ACTIONS(2045),
    [sym_atol] = ACTIONS(2045),
    [sym_atof] = ACTIONS(2045),
    [sym_itoa] = ACTIONS(2045),
    [sym_itohex] = ACTIONS(2045),
    [sym_hextoi] = ACTIONS(2045),
    [sym_ftoa] = ACTIONS(2045),
    [sym_find_string] = ACTIONS(2045),
    [sym_left_string] = ACTIONS(2045),
    [sym_right_string] = ACTIONS(2045),
    [sym_mid_string] = ACTIONS(2045),
    [sym_remove_string] = ACTIONS(2045),
    [sym_get_buffer_string] = ACTIONS(2045),
    [sym_get_buffer_char] = ACTIONS(2045),
    [sym_get_multi_buffer_string] = ACTIONS(2045),
    [sym_length_string] = ACTIONS(2045),
    [sym_max_length_string] = ACTIONS(2045),
    [sym_set_length_string] = ACTIONS(2045),
    [sym_compare_string] = ACTIONS(2045),
    [sym_upper_string] = ACTIONS(2045),
    [sym_lower_string] = ACTIONS(2045),
    [sym_get_last] = ACTIONS(2045),
    [sym_date_to_day] = ACTIONS(2045),
    [sym_date_to_month] = ACTIONS(2045),
    [sym_date_to_year] = ACTIONS(2045),
    [sym_time_to_hour] = ACTIONS(2045),
    [sym_time_to_minute] = ACTIONS(2045),
    [sym_time_to_second] = ACTIONS(2045),
    [sym_day_of_week] = ACTIONS(2045),
    [sym_set_pulse_time] = ACTIONS(2045),
    [sym_set_timer] = ACTIONS(2045),
    [sym_file_open] = ACTIONS(2045),
    [sym_file_close] = ACTIONS(2045),
    [sym_file_read] = ACTIONS(2045),
    [sym_file_read_line] = ACTIONS(2045),
    [sym_file_write] = ACTIONS(2045),
    [sym_file_write_line] = ACTIONS(2045),
    [sym_file_copy] = ACTIONS(2045),
    [sym_file_delete] = ACTIONS(2045),
    [sym_file_rename] = ACTIONS(2045),
    [sym_file_dir] = ACTIONS(2045),
    [sym_file_getdir] = ACTIONS(2045),
    [sym_file_setdir] = ACTIONS(2045),
    [sym_file_create_dir] = ACTIONS(2045),
    [sym_file_remove_dir] = ACTIONS(2045),
    [sym_file_seek] = ACTIONS(2045),
    [sym_length_array] = ACTIONS(2045),
    [sym_max_length_array] = ACTIONS(2045),
    [sym_set_length_array] = ACTIONS(2045),
    [sym_ip_client_open] = ACTIONS(2045),
    [sym_ip_client_close] = ACTIONS(2045),
    [sym_ip_server_open] = ACTIONS(2045),
    [sym_ip_server_close] = ACTIONS(2045),
    [sym_ip_bound_client_open] = ACTIONS(2045),
    [sym_ip_mc_server_open] = ACTIONS(2045),
    [sym_ip_set_option] = ACTIONS(2045),
    [sym_ssh_client_open] = ACTIONS(2045),
    [sym_ssh_client_close] = ACTIONS(2045),
    [sym_tls_client_open] = ACTIONS(2045),
    [sym_tls_client_close] = ACTIONS(2045),
    [sym_do_push] = ACTIONS(2045),
    [sym_do_push_timed] = ACTIONS(2045),
    [sym_do_release] = ACTIONS(2045),
    [sym_device_id] = ACTIONS(2045),
    [sym_device_info] = ACTIONS(2045),
    [sym_device_id_string] = ACTIONS(2045),
    [sym_device_standby] = ACTIONS(2045),
    [sym_device_wake] = ACTIONS(2045),
    [sym_timeline_create] = ACTIONS(2045),
    [sym_timeline_kill] = ACTIONS(2045),
    [sym_timeline_reload] = ACTIONS(2045),
    [sym_timeline_pause] = ACTIONS(2045),
    [sym_timeline_restart] = ACTIONS(2045),
    [sym_timeline_set] = ACTIONS(2045),
    [sym_timeline_get] = ACTIONS(2045),
    [sym_timeline_active] = ACTIONS(2045),
    [sym_timeline_dynamic_id] = ACTIONS(2045),
    [sym_variable_to_string] = ACTIONS(2045),
    [sym_string_to_variable] = ACTIONS(2045),
    [sym_length_variable_to_string] = ACTIONS(2045),
    [sym_type_cast] = ACTIONS(2045),
    [sym_variable_to_xml] = ACTIONS(2045),
    [sym_xml_to_variable] = ACTIONS(2045),
    [sym_length_variable_to_xml] = ACTIONS(2045),
    [sym_redirect_string] = ACTIONS(2045),
    [sym_random_number] = ACTIONS(2045),
    [sym_get_unique_id] = ACTIONS(2045),
    [sym_get_system_number] = ACTIONS(2045),
    [sym_set_system_number] = ACTIONS(2045),
    [sym_get_serial_number] = ACTIONS(2045),
    [sym_get_master_build] = ACTIONS(2045),
    [sym_set_validation_code] = ACTIONS(2045),
    [sym_reboot] = ACTIONS(2045),
    [sym_load_duet_module] = ACTIONS(2045),
    [sym_unload_duet_module] = ACTIONS(2045),
    [sym_duet_mem_size_set] = ACTIONS(2045),
    [sym_duet_mem_size_get] = ACTIONS(2045),
    [sym_dynamic_application_device] = ACTIONS(2045),
    [sym_dynamic_polled_port] = ACTIONS(2045),
    [sym_static_port_binding] = ACTIONS(2045),
    [sym_static_ip_binding] = ACTIONS(2045),
    [sym_get_dns_list] = ACTIONS(2045),
    [sym_set_dns_list] = ACTIONS(2045),
    [sym_get_ip_address] = ACTIONS(2045),
    [sym_set_ip_address] = ACTIONS(2045),
    [sym_get_url_list] = ACTIONS(2045),
    [sym_add_url_entry] = ACTIONS(2045),
    [sym_delete_url_entry] = ACTIONS(2045),
    [sym_abs_value] = ACTIONS(2045),
    [sym_max_value] = ACTIONS(2045),
    [sym_min_value] = ACTIONS(2045),
    [sym_exp_value] = ACTIONS(2045),
    [sym_log_value] = ACTIONS(2045),
    [sym_log10_value] = ACTIONS(2045),
    [sym_power_value] = ACTIONS(2045),
    [sym_sqrt_value] = ACTIONS(2045),
    [sym_raw_be] = ACTIONS(2045),
    [sym_raw_le] = ACTIONS(2045),
    [sym_internal_queue_size_set] = ACTIONS(2045),
    [sym_internal_queue_size_get] = ACTIONS(2045),
    [sym_internal_threshold_set] = ACTIONS(2045),
    [sym_internal_threshold_get] = ACTIONS(2045),
    [sym_rebuilt_event] = ACTIONS(2045),
    [sym_do_custom_event] = ACTIONS(2045),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2045),
    [sym_clkmgr_set_clk_source] = ACTIONS(2045),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2045),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2045),
    [sym_clkmgr_get_timezone] = ACTIONS(2045),
    [sym_clkmgr_set_timezone] = ACTIONS(2045),
    [sym_clkmgr_get_resync_period] = ACTIONS(2045),
    [sym_clkmgr_set_resync_period] = ACTIONS(2045),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2045),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2045),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2045),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2045),
    [sym_clkmgr_get_timeservers] = ACTIONS(2045),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2045),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2045),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2045),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2045),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2045),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2045),
    [sym_set_log_level] = ACTIONS(2045),
    [sym_get_log_level] = ACTIONS(2045),
    [sym_amx_log] = ACTIONS(2045),
    [sym_smtp_server_config_set] = ACTIONS(2045),
    [sym_smtp_server_config_get] = ACTIONS(2045),
    [sym_smtp_send] = ACTIONS(2045),
    [sym_validate_netlinx_account] = ACTIONS(2045),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2045),
    [sym_authenticate_certificate] = ACTIONS(2045),
    [sym_audit_netlinx_session_event] = ACTIONS(2045),
    [sym_audit_netlinx_generic_event] = ACTIONS(2045),
    [sym_get_available_flash_disk_space] = ACTIONS(2045),
    [sym_get_max_flash_disk_space] = ACTIONS(2045),
    [sym_data_create_feed] = ACTIONS(2045),
    [sym_data_delete_feed] = ACTIONS(2045),
    [sym_data_publish_feed] = ACTIONS(2045),
    [sym_data_get_published_feed] = ACTIONS(2045),
    [sym_data_add_record] = ACTIONS(2045),
    [sym_data_get_event_record] = ACTIONS(2045),
    [sym__wc_data_create_feed] = ACTIONS(2045),
    [sym__wc_data_add_record] = ACTIONS(2045),
    [sym__wc_data_get_event_record] = ACTIONS(2045),
    [sym_format] = ACTIONS(2045),
    [sym_astro_clock] = ACTIONS(2045),
    [sym_set_outdoor_temperature] = ACTIONS(2045),
    [sym_set_virtual_level_count] = ACTIONS(2045),
    [sym_set_virtual_channel_count] = ACTIONS(2045),
    [sym_set_virtual_port_count] = ACTIONS(2045),
    [sym__wc_encode_int] = ACTIONS(2045),
    [sym__wc_decode_int] = ACTIONS(2045),
    [sym__wc_int] = ACTIONS(2045),
    [sym__wc_to_ch_int] = ACTIONS(2045),
    [sym__ch_to_wc_int] = ACTIONS(2045),
    [sym__wc_find_string_int] = ACTIONS(2045),
    [sym__wc_left_string_int] = ACTIONS(2045),
    [sym__wc_lower_string_int] = ACTIONS(2045),
    [sym__wc_mid_string_int] = ACTIONS(2045),
    [sym__wc_remove_string_int] = ACTIONS(2045),
    [sym__wc_right_string_int] = ACTIONS(2045),
    [sym__wc_upper_string_int] = ACTIONS(2045),
    [sym__wc_compare_string_int] = ACTIONS(2045),
    [sym__wc_get_buffer_char_int] = ACTIONS(2045),
    [sym__wc_get_buffer_string_int] = ACTIONS(2045),
    [sym__wc_concat_string_int] = ACTIONS(2045),
    [sym__wc_file_open_int] = ACTIONS(2045),
    [sym__wc_file_close_int] = ACTIONS(2045),
    [sym__wc_file_read_int] = ACTIONS(2045),
    [sym__wc_file_read_line_int] = ACTIONS(2045),
    [sym__wc_file_write_int] = ACTIONS(2045),
    [sym__wc_file_write_line_int] = ACTIONS(2045),
    [aux_sym_preproc_include_token1] = ACTIONS(2045),
    [aux_sym_preproc_define_token1] = ACTIONS(2047),
    [aux_sym_preproc_warn_token1] = ACTIONS(2047),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2047),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2047),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2047),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2047),
    [aux_sym_preproc_else_token1] = ACTIONS(2047),
    [anon_sym_LBRACK] = ACTIONS(2047),
    [anon_sym_LBRACE] = ACTIONS(2047),
    [anon_sym_LPAREN] = ACTIONS(2045),
    [anon_sym_BANG] = ACTIONS(2047),
    [anon_sym_TILDE] = ACTIONS(2047),
    [anon_sym_DASH] = ACTIONS(2045),
    [anon_sym_PLUS] = ACTIONS(2045),
    [anon_sym_DASH_DASH] = ACTIONS(2047),
    [anon_sym_PLUS_PLUS] = ACTIONS(2047),
    [anon_sym_DQUOTE] = ACTIONS(2047),
    [anon_sym_SQUOTE] = ACTIONS(2047),
    [sym_number_literal] = ACTIONS(2047),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(338)] = {
    [sym_identifier] = ACTIONS(2049),
    [sym_program_name_keyword] = ACTIONS(2049),
    [sym_module_name_keyword] = ACTIONS(2049),
    [sym_define_device_keyword] = ACTIONS(2049),
    [sym_define_combine_keyword] = ACTIONS(2049),
    [sym_define_constant_keyword] = ACTIONS(2049),
    [sym_define_type_keyword] = ACTIONS(2049),
    [sym_define_variable_keyword] = ACTIONS(2049),
    [sym_define_system_variable_keyword] = ACTIONS(2049),
    [sym_define_connect_level_keyword] = ACTIONS(2049),
    [sym_define_latching_keyword] = ACTIONS(2049),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2049),
    [sym_define_toggling_keyword] = ACTIONS(2049),
    [sym_define_call_keyword] = ACTIONS(2049),
    [sym_define_function_keyword] = ACTIONS(2049),
    [sym_define_library_function_keyword] = ACTIONS(2049),
    [sym_define_start_keyword] = ACTIONS(2049),
    [sym_define_event_keyword] = ACTIONS(2049),
    [sym_define_program_keyword] = ACTIONS(2049),
    [sym_define_module_keyword] = ACTIONS(2049),
    [sym_char_keyword] = ACTIONS(2049),
    [sym_widechar_keyword] = ACTIONS(2049),
    [sym_integer_keyword] = ACTIONS(2049),
    [sym_sinteger_keyword] = ACTIONS(2049),
    [sym_long_keyword] = ACTIONS(2049),
    [sym_slong_keyword] = ACTIONS(2049),
    [sym_float_keyword] = ACTIONS(2049),
    [sym_double_keyword] = ACTIONS(2049),
    [sym_variant_keyword] = ACTIONS(2049),
    [sym_variantarray_keyword] = ACTIONS(2049),
    [sym_local_var_keyword] = ACTIONS(2049),
    [sym_stack_var_keyword] = ACTIONS(2049),
    [sym_constant_keyword] = ACTIONS(2049),
    [sym_volatile_keyword] = ACTIONS(2049),
    [sym_non_volatile_keyword] = ACTIONS(2049),
    [sym_persistent_keyword] = ACTIONS(2049),
    [sym_dev_keyword] = ACTIONS(2049),
    [sym_devlev_keyword] = ACTIONS(2049),
    [sym_devchan_keyword] = ACTIONS(2049),
    [sym_if_keyword] = ACTIONS(2049),
    [sym_switch_keyword] = ACTIONS(2049),
    [sym_case_keyword] = ACTIONS(2049),
    [sym_default_keyword] = ACTIONS(2049),
    [sym_while_keyword] = ACTIONS(2049),
    [sym_for_keyword] = ACTIONS(2049),
    [sym_break_keyword] = ACTIONS(2049),
    [sym_continue_keyword] = ACTIONS(2049),
    [sym_return_keyword] = ACTIONS(2049),
    [sym_select_keyword] = ACTIONS(2049),
    [sym_active_keyword] = ACTIONS(2049),
    [sym_struct_keyword] = ACTIONS(2049),
    [sym_structure_keyword] = ACTIONS(2049),
    [sym_button_event_keyword] = ACTIONS(2049),
    [sym_channel_event_keyword] = ACTIONS(2049),
    [sym_level_event_keyword] = ACTIONS(2049),
    [sym_data_event_keyword] = ACTIONS(2049),
    [sym_timeline_event_keyword] = ACTIONS(2049),
    [sym_custom_event_keyword] = ACTIONS(2049),
    [sym_push_keyword] = ACTIONS(2049),
    [sym_release_keyword] = ACTIONS(2049),
    [aux_sym_on_keyword_token1] = ACTIONS(2049),
    [aux_sym_off_keyword_token1] = ACTIONS(2049),
    [sym_send_level_keyword] = ACTIONS(2049),
    [sym_send_string_keyword] = ACTIONS(2049),
    [sym_send_command_keyword] = ACTIONS(2049),
    [sym_clear_buffer_keyword] = ACTIONS(2049),
    [sym_create_buffer_keyword] = ACTIONS(2049),
    [sym_create_multi_buffer_keyword] = ACTIONS(2049),
    [sym_call_keyword] = ACTIONS(2049),
    [sym_system_call_keyword] = ACTIONS(2049),
    [sym_devchan_to_keyword] = ACTIONS(2049),
    [sym_devchan_min_to_keyword] = ACTIONS(2049),
    [sym_devchan_total_off_keyword] = ACTIONS(2049),
    [sym_devchan_pulse_keyword] = ACTIONS(2049),
    [sym_wait_keyword] = ACTIONS(2049),
    [sym_cancel_wait_keyword] = ACTIONS(2049),
    [sym_wait_until_keyword] = ACTIONS(2049),
    [sym_cancel_wait_until_keyword] = ACTIONS(2049),
    [sym_cancel_all_wait_keyword] = ACTIONS(2049),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2049),
    [sym_not] = ACTIONS(2049),
    [sym_bnot] = ACTIONS(2049),
    [sym___file__] = ACTIONS(2049),
    [sym___date__] = ACTIONS(2049),
    [sym___ldate__] = ACTIONS(2049),
    [sym___time__] = ACTIONS(2049),
    [sym___line__] = ACTIONS(2049),
    [sym___name__] = ACTIONS(2049),
    [sym___netlinx__] = ACTIONS(2049),
    [sym_netlinx_axi_version] = ACTIONS(2049),
    [sym_url_flg_tcp] = ACTIONS(2049),
    [sym_url_flg_accinfopresent] = ACTIONS(2049),
    [sym_url_flg_temp] = ACTIONS(2049),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2049),
    [sym_url_flg_stat_mask] = ACTIONS(2049),
    [sym_url_flg_stat_lookup] = ACTIONS(2049),
    [sym_url_flg_stat_connecting] = ACTIONS(2049),
    [sym_url_flg_stat_waiting] = ACTIONS(2049),
    [sym_url_flg_stat_connected] = ACTIONS(2049),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2049),
    [sym_first_virtual_device] = ACTIONS(2049),
    [sym_first_local_port] = ACTIONS(2049),
    [sym_dynamic_virtual_device] = ACTIONS(2049),
    [sym_timeline_once] = ACTIONS(2049),
    [sym_timeline_repeat] = ACTIONS(2049),
    [sym_timeline_absolute] = ACTIONS(2049),
    [sym_timeline_relative] = ACTIONS(2049),
    [sym_do_push_timed_infinite] = ACTIONS(2049),
    [sym_source_type_no_address] = ACTIONS(2049),
    [sym_source_type_neuron_id] = ACTIONS(2049),
    [sym_source_type_ip_address] = ACTIONS(2049),
    [sym_source_type_axlink] = ACTIONS(2049),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2049),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2049),
    [sym_source_type_ip_socket_address] = ACTIONS(2049),
    [sym_source_type_rs232] = ACTIONS(2049),
    [sym_source_type_internal] = ACTIONS(2049),
    [sym_source_type_mac_address] = ACTIONS(2049),
    [sym_source_type_ipv4_port] = ACTIONS(2049),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2049),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2049),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2049),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2049),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2049),
    [sym_true] = ACTIONS(2049),
    [sym_false] = ACTIONS(2049),
    [sym_file_read_only] = ACTIONS(2049),
    [sym_file_rw_new] = ACTIONS(2049),
    [sym_file_rw_append] = ACTIONS(2049),
    [sym_ip_tcp] = ACTIONS(2049),
    [sym_ip_udp] = ACTIONS(2049),
    [sym_ip_udp_2way] = ACTIONS(2049),
    [sym_xml_encode_types] = ACTIONS(2049),
    [sym_xml_encode_char_as_list] = ACTIONS(2049),
    [sym_xml_encode_le] = ACTIONS(2049),
    [sym_xml_decode_types] = ACTIONS(2049),
    [sym_xml_decode_no_preserve] = ACTIONS(2049),
    [sym_duet_dev_type_display_device] = ACTIONS(2049),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2049),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2049),
    [sym_duet_dev_type_io_device] = ACTIONS(2049),
    [sym_duet_dev_type_relay_device] = ACTIONS(2049),
    [sym_duet_dev_type_ups] = ACTIONS(2049),
    [sym_duet_dev_type_amplifier] = ACTIONS(2049),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2049),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2049),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2049),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2049),
    [sym_duet_dev_type_camera] = ACTIONS(2049),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2049),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2049),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2049),
    [sym_duet_dev_type_dss] = ACTIONS(2049),
    [sym_duet_dev_type_dvr] = ACTIONS(2049),
    [sym_duet_dev_type_disc_device] = ACTIONS(2049),
    [sym_duet_dev_type_document_camera] = ACTIONS(2049),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2049),
    [sym_duet_dev_type_hvac] = ACTIONS(2049),
    [sym_duet_dev_type_keypad] = ACTIONS(2049),
    [sym_duet_dev_type_light] = ACTIONS(2049),
    [sym_duet_dev_type_monitor] = ACTIONS(2049),
    [sym_duet_dev_type_motor] = ACTIONS(2049),
    [sym_duet_dev_type_multi_window] = ACTIONS(2049),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2049),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2049),
    [sym_duet_dev_type_receiver] = ACTIONS(2049),
    [sym_duet_dev_type_security_system] = ACTIONS(2049),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2049),
    [sym_duet_dev_type_settop_box] = ACTIONS(2049),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2049),
    [sym_duet_dev_type_switcher] = ACTIONS(2049),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2049),
    [sym_duet_dev_type_tv] = ACTIONS(2049),
    [sym_duet_dev_type_utility] = ACTIONS(2049),
    [sym_duet_dev_type_vcr] = ACTIONS(2049),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2049),
    [sym_duet_dev_type_video_processor] = ACTIONS(2049),
    [sym_duet_dev_type_video_projector] = ACTIONS(2049),
    [sym_duet_dev_type_video_wall] = ACTIONS(2049),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2049),
    [sym_duet_dev_type_weather] = ACTIONS(2049),
    [sym_duet_dev_not_polled] = ACTIONS(2049),
    [sym_duet_dev_polled] = ACTIONS(2049),
    [sym_clkmgr_mode_network] = ACTIONS(2049),
    [sym_clkmgr_mode_standalone] = ACTIONS(2049),
    [sym_normal_standby] = ACTIONS(2049),
    [sym_normal_wake] = ACTIONS(2049),
    [sym_ip_multicast_ttl_option] = ACTIONS(2049),
    [sym_ip_ttl_subnet] = ACTIONS(2049),
    [sym_ip_ttl_site] = ACTIONS(2049),
    [sym_ip_ttl_region] = ACTIONS(2049),
    [sym_ip_ttl_continent] = ACTIONS(2049),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2049),
    [sym_ip_nodelay_on] = ACTIONS(2049),
    [sym_ip_nodelay_off] = ACTIONS(2049),
    [sym_amx_error] = ACTIONS(2049),
    [sym_amx_warning] = ACTIONS(2049),
    [sym_amx_info] = ACTIONS(2049),
    [sym_amx_debug] = ACTIONS(2049),
    [sym_netlinx_login_success] = ACTIONS(2049),
    [sym_netlinx_login_fail] = ACTIONS(2049),
    [sym_netlinx_logout] = ACTIONS(2049),
    [sym_null_str] = ACTIONS(2049),
    [sym_smtp_address] = ACTIONS(2049),
    [sym_smtp_port_number] = ACTIONS(2049),
    [sym_smtp_username] = ACTIONS(2049),
    [sym_smtp_password] = ACTIONS(2049),
    [sym_smtp_require_tls] = ACTIONS(2049),
    [sym_smtp_from] = ACTIONS(2049),
    [sym_smtp_tls_true] = ACTIONS(2049),
    [sym_smtp_tls_false] = ACTIONS(2049),
    [sym_listview_on_row_select_event] = ACTIONS(2049),
    [sym_data_structure_unknown] = ACTIONS(2049),
    [sym_data_structure_datafeed] = ACTIONS(2049),
    [sym_data_structure_datarecordset] = ACTIONS(2049),
    [sym_data_structure_datarecord] = ACTIONS(2049),
    [sym_data_structure_datafield] = ACTIONS(2049),
    [sym_data_max_content_field_count] = ACTIONS(2049),
    [sym_data_max_metadata_field_count] = ACTIONS(2049),
    [sym_data_max_id_length] = ACTIONS(2049),
    [sym_data_max_name_length] = ACTIONS(2049),
    [sym_data_max_description_length] = ACTIONS(2049),
    [sym_data_max_type_length] = ACTIONS(2049),
    [sym_data_max_format_length] = ACTIONS(2049),
    [sym_data_max_source_length] = ACTIONS(2049),
    [sym_data_max_value_length] = ACTIONS(2049),
    [sym_data_max_label_length] = ACTIONS(2049),
    [sym_data_max_url_length] = ACTIONS(2049),
    [sym_max_last_login_info_length] = ACTIONS(2049),
    [sym_data_type_unknown] = ACTIONS(2049),
    [sym_data_type_string] = ACTIONS(2049),
    [sym_data_type_datetime] = ACTIONS(2049),
    [sym_data_type_date] = ACTIONS(2049),
    [sym_data_type_time] = ACTIONS(2049),
    [sym_data_type_image] = ACTIONS(2049),
    [sym_data_format_url] = ACTIONS(2049),
    [sym_data_format_phone] = ACTIONS(2049),
    [sym_data_format_email] = ACTIONS(2049),
    [sym_data_format_iso8601] = ACTIONS(2049),
    [sym_auditlog_priv] = ACTIONS(2049),
    [sym_configuration_priv] = ACTIONS(2049),
    [sym_deviceconfig_priv] = ACTIONS(2049),
    [sym_ftp_priv] = ACTIONS(2049),
    [sym_http_priv] = ACTIONS(2049),
    [sym_networkconfig_priv] = ACTIONS(2049),
    [sym_programport_priv] = ACTIONS(2049),
    [sym_remoteui_priv] = ACTIONS(2049),
    [sym_securitycontrol_priv] = ACTIONS(2049),
    [sym_softwaremanagement_priv] = ACTIONS(2049),
    [sym_terminal_priv] = ACTIONS(2049),
    [sym_tpadmin_priv] = ACTIONS(2049),
    [sym_user1_priv] = ACTIONS(2049),
    [sym_user2_priv] = ACTIONS(2049),
    [sym_user3_priv] = ACTIONS(2049),
    [sym_user4_priv] = ACTIONS(2049),
    [sym_usermanagement_priv] = ACTIONS(2049),
    [sym_valid_account] = ACTIONS(2049),
    [sym_err_name_invalid] = ACTIONS(2049),
    [sym_err_password_invalid] = ACTIONS(2049),
    [sym_err_invalid_account] = ACTIONS(2049),
    [sym_err_no_authorization] = ACTIONS(2049),
    [sym_err_invalid_parameter] = ACTIONS(2049),
    [sym_err_user_locked_out] = ACTIONS(2049),
    [sym_err_user_account_expired] = ACTIONS(2049),
    [sym_err_general_error] = ACTIONS(2049),
    [sym_err_authentication_error] = ACTIONS(2049),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2049),
    [sym_tls_validate_certificate] = ACTIONS(2049),
    [sym_tdata] = ACTIONS(2049),
    [sym_tchannel] = ACTIONS(2049),
    [sym_tlevel] = ACTIONS(2049),
    [sym_tbutton] = ACTIONS(2049),
    [sym_ttimeline] = ACTIONS(2049),
    [sym_tcustom] = ACTIONS(2049),
    [sym_url_struct] = ACTIONS(2049),
    [sym_dns_struct] = ACTIONS(2049),
    [sym_ip_address_struct] = ACTIONS(2049),
    [sym_dev_info_struct] = ACTIONS(2049),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2049),
    [sym_data_feed] = ACTIONS(2049),
    [sym_data_field] = ACTIONS(2049),
    [sym_data_record] = ACTIONS(2049),
    [sym_wc_data_feed] = ACTIONS(2049),
    [sym_wc_data_field] = ACTIONS(2049),
    [sym_wc_data_record] = ACTIONS(2049),
    [sym_last_login_info] = ACTIONS(2049),
    [sym____reserved___] = ACTIONS(2049),
    [sym_date] = ACTIONS(2049),
    [sym_day] = ACTIONS(2049),
    [sym_time] = ACTIONS(2049),
    [sym_ldate] = ACTIONS(2049),
    [sym_push_device] = ACTIONS(2049),
    [sym_push_channel] = ACTIONS(2049),
    [sym_push_devchan] = ACTIONS(2049),
    [sym_release_device] = ACTIONS(2049),
    [sym_release_channel] = ACTIONS(2049),
    [sym_release_devchan] = ACTIONS(2049),
    [sym_master_slot] = ACTIONS(2049),
    [sym_get_pulse_time] = ACTIONS(2049),
    [sym_get_timer] = ACTIONS(2049),
    [sym_master_sn] = ACTIONS(2049),
    [sym_system_number] = ACTIONS(2049),
    [sym_button] = ACTIONS(2049),
    [sym_dv_channel] = ACTIONS(2049),
    [sym_data] = ACTIONS(2049),
    [sym_level] = ACTIONS(2049),
    [sym_timeline] = ACTIONS(2049),
    [sym_channel] = ACTIONS(2049),
    [sym_custom] = ACTIONS(2049),
    [sym_atoi] = ACTIONS(2049),
    [sym_atol] = ACTIONS(2049),
    [sym_atof] = ACTIONS(2049),
    [sym_itoa] = ACTIONS(2049),
    [sym_itohex] = ACTIONS(2049),
    [sym_hextoi] = ACTIONS(2049),
    [sym_ftoa] = ACTIONS(2049),
    [sym_find_string] = ACTIONS(2049),
    [sym_left_string] = ACTIONS(2049),
    [sym_right_string] = ACTIONS(2049),
    [sym_mid_string] = ACTIONS(2049),
    [sym_remove_string] = ACTIONS(2049),
    [sym_get_buffer_string] = ACTIONS(2049),
    [sym_get_buffer_char] = ACTIONS(2049),
    [sym_get_multi_buffer_string] = ACTIONS(2049),
    [sym_length_string] = ACTIONS(2049),
    [sym_max_length_string] = ACTIONS(2049),
    [sym_set_length_string] = ACTIONS(2049),
    [sym_compare_string] = ACTIONS(2049),
    [sym_upper_string] = ACTIONS(2049),
    [sym_lower_string] = ACTIONS(2049),
    [sym_get_last] = ACTIONS(2049),
    [sym_date_to_day] = ACTIONS(2049),
    [sym_date_to_month] = ACTIONS(2049),
    [sym_date_to_year] = ACTIONS(2049),
    [sym_time_to_hour] = ACTIONS(2049),
    [sym_time_to_minute] = ACTIONS(2049),
    [sym_time_to_second] = ACTIONS(2049),
    [sym_day_of_week] = ACTIONS(2049),
    [sym_set_pulse_time] = ACTIONS(2049),
    [sym_set_timer] = ACTIONS(2049),
    [sym_file_open] = ACTIONS(2049),
    [sym_file_close] = ACTIONS(2049),
    [sym_file_read] = ACTIONS(2049),
    [sym_file_read_line] = ACTIONS(2049),
    [sym_file_write] = ACTIONS(2049),
    [sym_file_write_line] = ACTIONS(2049),
    [sym_file_copy] = ACTIONS(2049),
    [sym_file_delete] = ACTIONS(2049),
    [sym_file_rename] = ACTIONS(2049),
    [sym_file_dir] = ACTIONS(2049),
    [sym_file_getdir] = ACTIONS(2049),
    [sym_file_setdir] = ACTIONS(2049),
    [sym_file_create_dir] = ACTIONS(2049),
    [sym_file_remove_dir] = ACTIONS(2049),
    [sym_file_seek] = ACTIONS(2049),
    [sym_length_array] = ACTIONS(2049),
    [sym_max_length_array] = ACTIONS(2049),
    [sym_set_length_array] = ACTIONS(2049),
    [sym_ip_client_open] = ACTIONS(2049),
    [sym_ip_client_close] = ACTIONS(2049),
    [sym_ip_server_open] = ACTIONS(2049),
    [sym_ip_server_close] = ACTIONS(2049),
    [sym_ip_bound_client_open] = ACTIONS(2049),
    [sym_ip_mc_server_open] = ACTIONS(2049),
    [sym_ip_set_option] = ACTIONS(2049),
    [sym_ssh_client_open] = ACTIONS(2049),
    [sym_ssh_client_close] = ACTIONS(2049),
    [sym_tls_client_open] = ACTIONS(2049),
    [sym_tls_client_close] = ACTIONS(2049),
    [sym_do_push] = ACTIONS(2049),
    [sym_do_push_timed] = ACTIONS(2049),
    [sym_do_release] = ACTIONS(2049),
    [sym_device_id] = ACTIONS(2049),
    [sym_device_info] = ACTIONS(2049),
    [sym_device_id_string] = ACTIONS(2049),
    [sym_device_standby] = ACTIONS(2049),
    [sym_device_wake] = ACTIONS(2049),
    [sym_timeline_create] = ACTIONS(2049),
    [sym_timeline_kill] = ACTIONS(2049),
    [sym_timeline_reload] = ACTIONS(2049),
    [sym_timeline_pause] = ACTIONS(2049),
    [sym_timeline_restart] = ACTIONS(2049),
    [sym_timeline_set] = ACTIONS(2049),
    [sym_timeline_get] = ACTIONS(2049),
    [sym_timeline_active] = ACTIONS(2049),
    [sym_timeline_dynamic_id] = ACTIONS(2049),
    [sym_variable_to_string] = ACTIONS(2049),
    [sym_string_to_variable] = ACTIONS(2049),
    [sym_length_variable_to_string] = ACTIONS(2049),
    [sym_type_cast] = ACTIONS(2049),
    [sym_variable_to_xml] = ACTIONS(2049),
    [sym_xml_to_variable] = ACTIONS(2049),
    [sym_length_variable_to_xml] = ACTIONS(2049),
    [sym_redirect_string] = ACTIONS(2049),
    [sym_random_number] = ACTIONS(2049),
    [sym_get_unique_id] = ACTIONS(2049),
    [sym_get_system_number] = ACTIONS(2049),
    [sym_set_system_number] = ACTIONS(2049),
    [sym_get_serial_number] = ACTIONS(2049),
    [sym_get_master_build] = ACTIONS(2049),
    [sym_set_validation_code] = ACTIONS(2049),
    [sym_reboot] = ACTIONS(2049),
    [sym_load_duet_module] = ACTIONS(2049),
    [sym_unload_duet_module] = ACTIONS(2049),
    [sym_duet_mem_size_set] = ACTIONS(2049),
    [sym_duet_mem_size_get] = ACTIONS(2049),
    [sym_dynamic_application_device] = ACTIONS(2049),
    [sym_dynamic_polled_port] = ACTIONS(2049),
    [sym_static_port_binding] = ACTIONS(2049),
    [sym_static_ip_binding] = ACTIONS(2049),
    [sym_get_dns_list] = ACTIONS(2049),
    [sym_set_dns_list] = ACTIONS(2049),
    [sym_get_ip_address] = ACTIONS(2049),
    [sym_set_ip_address] = ACTIONS(2049),
    [sym_get_url_list] = ACTIONS(2049),
    [sym_add_url_entry] = ACTIONS(2049),
    [sym_delete_url_entry] = ACTIONS(2049),
    [sym_abs_value] = ACTIONS(2049),
    [sym_max_value] = ACTIONS(2049),
    [sym_min_value] = ACTIONS(2049),
    [sym_exp_value] = ACTIONS(2049),
    [sym_log_value] = ACTIONS(2049),
    [sym_log10_value] = ACTIONS(2049),
    [sym_power_value] = ACTIONS(2049),
    [sym_sqrt_value] = ACTIONS(2049),
    [sym_raw_be] = ACTIONS(2049),
    [sym_raw_le] = ACTIONS(2049),
    [sym_internal_queue_size_set] = ACTIONS(2049),
    [sym_internal_queue_size_get] = ACTIONS(2049),
    [sym_internal_threshold_set] = ACTIONS(2049),
    [sym_internal_threshold_get] = ACTIONS(2049),
    [sym_rebuilt_event] = ACTIONS(2049),
    [sym_do_custom_event] = ACTIONS(2049),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2049),
    [sym_clkmgr_set_clk_source] = ACTIONS(2049),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2049),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2049),
    [sym_clkmgr_get_timezone] = ACTIONS(2049),
    [sym_clkmgr_set_timezone] = ACTIONS(2049),
    [sym_clkmgr_get_resync_period] = ACTIONS(2049),
    [sym_clkmgr_set_resync_period] = ACTIONS(2049),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2049),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2049),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2049),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2049),
    [sym_clkmgr_get_timeservers] = ACTIONS(2049),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2049),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2049),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2049),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2049),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2049),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2049),
    [sym_set_log_level] = ACTIONS(2049),
    [sym_get_log_level] = ACTIONS(2049),
    [sym_amx_log] = ACTIONS(2049),
    [sym_smtp_server_config_set] = ACTIONS(2049),
    [sym_smtp_server_config_get] = ACTIONS(2049),
    [sym_smtp_send] = ACTIONS(2049),
    [sym_validate_netlinx_account] = ACTIONS(2049),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2049),
    [sym_authenticate_certificate] = ACTIONS(2049),
    [sym_audit_netlinx_session_event] = ACTIONS(2049),
    [sym_audit_netlinx_generic_event] = ACTIONS(2049),
    [sym_get_available_flash_disk_space] = ACTIONS(2049),
    [sym_get_max_flash_disk_space] = ACTIONS(2049),
    [sym_data_create_feed] = ACTIONS(2049),
    [sym_data_delete_feed] = ACTIONS(2049),
    [sym_data_publish_feed] = ACTIONS(2049),
    [sym_data_get_published_feed] = ACTIONS(2049),
    [sym_data_add_record] = ACTIONS(2049),
    [sym_data_get_event_record] = ACTIONS(2049),
    [sym__wc_data_create_feed] = ACTIONS(2049),
    [sym__wc_data_add_record] = ACTIONS(2049),
    [sym__wc_data_get_event_record] = ACTIONS(2049),
    [sym_format] = ACTIONS(2049),
    [sym_astro_clock] = ACTIONS(2049),
    [sym_set_outdoor_temperature] = ACTIONS(2049),
    [sym_set_virtual_level_count] = ACTIONS(2049),
    [sym_set_virtual_channel_count] = ACTIONS(2049),
    [sym_set_virtual_port_count] = ACTIONS(2049),
    [sym__wc_encode_int] = ACTIONS(2049),
    [sym__wc_decode_int] = ACTIONS(2049),
    [sym__wc_int] = ACTIONS(2049),
    [sym__wc_to_ch_int] = ACTIONS(2049),
    [sym__ch_to_wc_int] = ACTIONS(2049),
    [sym__wc_find_string_int] = ACTIONS(2049),
    [sym__wc_left_string_int] = ACTIONS(2049),
    [sym__wc_lower_string_int] = ACTIONS(2049),
    [sym__wc_mid_string_int] = ACTIONS(2049),
    [sym__wc_remove_string_int] = ACTIONS(2049),
    [sym__wc_right_string_int] = ACTIONS(2049),
    [sym__wc_upper_string_int] = ACTIONS(2049),
    [sym__wc_compare_string_int] = ACTIONS(2049),
    [sym__wc_get_buffer_char_int] = ACTIONS(2049),
    [sym__wc_get_buffer_string_int] = ACTIONS(2049),
    [sym__wc_concat_string_int] = ACTIONS(2049),
    [sym__wc_file_open_int] = ACTIONS(2049),
    [sym__wc_file_close_int] = ACTIONS(2049),
    [sym__wc_file_read_int] = ACTIONS(2049),
    [sym__wc_file_read_line_int] = ACTIONS(2049),
    [sym__wc_file_write_int] = ACTIONS(2049),
    [sym__wc_file_write_line_int] = ACTIONS(2049),
    [aux_sym_preproc_include_token1] = ACTIONS(2049),
    [aux_sym_preproc_define_token1] = ACTIONS(2051),
    [aux_sym_preproc_warn_token1] = ACTIONS(2051),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2051),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2051),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2051),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2051),
    [aux_sym_preproc_else_token1] = ACTIONS(2051),
    [anon_sym_LBRACK] = ACTIONS(2051),
    [anon_sym_LBRACE] = ACTIONS(2051),
    [anon_sym_LPAREN] = ACTIONS(2049),
    [anon_sym_BANG] = ACTIONS(2051),
    [anon_sym_TILDE] = ACTIONS(2051),
    [anon_sym_DASH] = ACTIONS(2049),
    [anon_sym_PLUS] = ACTIONS(2049),
    [anon_sym_DASH_DASH] = ACTIONS(2051),
    [anon_sym_PLUS_PLUS] = ACTIONS(2051),
    [anon_sym_DQUOTE] = ACTIONS(2051),
    [anon_sym_SQUOTE] = ACTIONS(2051),
    [sym_number_literal] = ACTIONS(2051),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(339)] = {
    [sym_identifier] = ACTIONS(2053),
    [sym_program_name_keyword] = ACTIONS(2053),
    [sym_module_name_keyword] = ACTIONS(2053),
    [sym_define_device_keyword] = ACTIONS(2053),
    [sym_define_combine_keyword] = ACTIONS(2053),
    [sym_define_constant_keyword] = ACTIONS(2053),
    [sym_define_type_keyword] = ACTIONS(2053),
    [sym_define_variable_keyword] = ACTIONS(2053),
    [sym_define_system_variable_keyword] = ACTIONS(2053),
    [sym_define_connect_level_keyword] = ACTIONS(2053),
    [sym_define_latching_keyword] = ACTIONS(2053),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2053),
    [sym_define_toggling_keyword] = ACTIONS(2053),
    [sym_define_call_keyword] = ACTIONS(2053),
    [sym_define_function_keyword] = ACTIONS(2053),
    [sym_define_library_function_keyword] = ACTIONS(2053),
    [sym_define_start_keyword] = ACTIONS(2053),
    [sym_define_event_keyword] = ACTIONS(2053),
    [sym_define_program_keyword] = ACTIONS(2053),
    [sym_define_module_keyword] = ACTIONS(2053),
    [sym_char_keyword] = ACTIONS(2053),
    [sym_widechar_keyword] = ACTIONS(2053),
    [sym_integer_keyword] = ACTIONS(2053),
    [sym_sinteger_keyword] = ACTIONS(2053),
    [sym_long_keyword] = ACTIONS(2053),
    [sym_slong_keyword] = ACTIONS(2053),
    [sym_float_keyword] = ACTIONS(2053),
    [sym_double_keyword] = ACTIONS(2053),
    [sym_variant_keyword] = ACTIONS(2053),
    [sym_variantarray_keyword] = ACTIONS(2053),
    [sym_local_var_keyword] = ACTIONS(2053),
    [sym_stack_var_keyword] = ACTIONS(2053),
    [sym_constant_keyword] = ACTIONS(2053),
    [sym_volatile_keyword] = ACTIONS(2053),
    [sym_non_volatile_keyword] = ACTIONS(2053),
    [sym_persistent_keyword] = ACTIONS(2053),
    [sym_dev_keyword] = ACTIONS(2053),
    [sym_devlev_keyword] = ACTIONS(2053),
    [sym_devchan_keyword] = ACTIONS(2053),
    [sym_if_keyword] = ACTIONS(2053),
    [sym_switch_keyword] = ACTIONS(2053),
    [sym_case_keyword] = ACTIONS(2053),
    [sym_default_keyword] = ACTIONS(2053),
    [sym_while_keyword] = ACTIONS(2053),
    [sym_for_keyword] = ACTIONS(2053),
    [sym_break_keyword] = ACTIONS(2053),
    [sym_continue_keyword] = ACTIONS(2053),
    [sym_return_keyword] = ACTIONS(2053),
    [sym_select_keyword] = ACTIONS(2053),
    [sym_active_keyword] = ACTIONS(2053),
    [sym_struct_keyword] = ACTIONS(2053),
    [sym_structure_keyword] = ACTIONS(2053),
    [sym_button_event_keyword] = ACTIONS(2053),
    [sym_channel_event_keyword] = ACTIONS(2053),
    [sym_level_event_keyword] = ACTIONS(2053),
    [sym_data_event_keyword] = ACTIONS(2053),
    [sym_timeline_event_keyword] = ACTIONS(2053),
    [sym_custom_event_keyword] = ACTIONS(2053),
    [sym_push_keyword] = ACTIONS(2053),
    [sym_release_keyword] = ACTIONS(2053),
    [aux_sym_on_keyword_token1] = ACTIONS(2053),
    [aux_sym_off_keyword_token1] = ACTIONS(2053),
    [sym_send_level_keyword] = ACTIONS(2053),
    [sym_send_string_keyword] = ACTIONS(2053),
    [sym_send_command_keyword] = ACTIONS(2053),
    [sym_clear_buffer_keyword] = ACTIONS(2053),
    [sym_create_buffer_keyword] = ACTIONS(2053),
    [sym_create_multi_buffer_keyword] = ACTIONS(2053),
    [sym_call_keyword] = ACTIONS(2053),
    [sym_system_call_keyword] = ACTIONS(2053),
    [sym_devchan_to_keyword] = ACTIONS(2053),
    [sym_devchan_min_to_keyword] = ACTIONS(2053),
    [sym_devchan_total_off_keyword] = ACTIONS(2053),
    [sym_devchan_pulse_keyword] = ACTIONS(2053),
    [sym_wait_keyword] = ACTIONS(2053),
    [sym_cancel_wait_keyword] = ACTIONS(2053),
    [sym_wait_until_keyword] = ACTIONS(2053),
    [sym_cancel_wait_until_keyword] = ACTIONS(2053),
    [sym_cancel_all_wait_keyword] = ACTIONS(2053),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2053),
    [sym_not] = ACTIONS(2053),
    [sym_bnot] = ACTIONS(2053),
    [sym___file__] = ACTIONS(2053),
    [sym___date__] = ACTIONS(2053),
    [sym___ldate__] = ACTIONS(2053),
    [sym___time__] = ACTIONS(2053),
    [sym___line__] = ACTIONS(2053),
    [sym___name__] = ACTIONS(2053),
    [sym___netlinx__] = ACTIONS(2053),
    [sym_netlinx_axi_version] = ACTIONS(2053),
    [sym_url_flg_tcp] = ACTIONS(2053),
    [sym_url_flg_accinfopresent] = ACTIONS(2053),
    [sym_url_flg_temp] = ACTIONS(2053),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2053),
    [sym_url_flg_stat_mask] = ACTIONS(2053),
    [sym_url_flg_stat_lookup] = ACTIONS(2053),
    [sym_url_flg_stat_connecting] = ACTIONS(2053),
    [sym_url_flg_stat_waiting] = ACTIONS(2053),
    [sym_url_flg_stat_connected] = ACTIONS(2053),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2053),
    [sym_first_virtual_device] = ACTIONS(2053),
    [sym_first_local_port] = ACTIONS(2053),
    [sym_dynamic_virtual_device] = ACTIONS(2053),
    [sym_timeline_once] = ACTIONS(2053),
    [sym_timeline_repeat] = ACTIONS(2053),
    [sym_timeline_absolute] = ACTIONS(2053),
    [sym_timeline_relative] = ACTIONS(2053),
    [sym_do_push_timed_infinite] = ACTIONS(2053),
    [sym_source_type_no_address] = ACTIONS(2053),
    [sym_source_type_neuron_id] = ACTIONS(2053),
    [sym_source_type_ip_address] = ACTIONS(2053),
    [sym_source_type_axlink] = ACTIONS(2053),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2053),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2053),
    [sym_source_type_ip_socket_address] = ACTIONS(2053),
    [sym_source_type_rs232] = ACTIONS(2053),
    [sym_source_type_internal] = ACTIONS(2053),
    [sym_source_type_mac_address] = ACTIONS(2053),
    [sym_source_type_ipv4_port] = ACTIONS(2053),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2053),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2053),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2053),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2053),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2053),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2053),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2053),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2053),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2053),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2053),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2053),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2053),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2053),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2053),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2053),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2053),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2053),
    [sym_true] = ACTIONS(2053),
    [sym_false] = ACTIONS(2053),
    [sym_file_read_only] = ACTIONS(2053),
    [sym_file_rw_new] = ACTIONS(2053),
    [sym_file_rw_append] = ACTIONS(2053),
    [sym_ip_tcp] = ACTIONS(2053),
    [sym_ip_udp] = ACTIONS(2053),
    [sym_ip_udp_2way] = ACTIONS(2053),
    [sym_xml_encode_types] = ACTIONS(2053),
    [sym_xml_encode_char_as_list] = ACTIONS(2053),
    [sym_xml_encode_le] = ACTIONS(2053),
    [sym_xml_decode_types] = ACTIONS(2053),
    [sym_xml_decode_no_preserve] = ACTIONS(2053),
    [sym_duet_dev_type_display_device] = ACTIONS(2053),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2053),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2053),
    [sym_duet_dev_type_io_device] = ACTIONS(2053),
    [sym_duet_dev_type_relay_device] = ACTIONS(2053),
    [sym_duet_dev_type_ups] = ACTIONS(2053),
    [sym_duet_dev_type_amplifier] = ACTIONS(2053),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2053),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2053),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2053),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2053),
    [sym_duet_dev_type_camera] = ACTIONS(2053),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2053),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2053),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2053),
    [sym_duet_dev_type_dss] = ACTIONS(2053),
    [sym_duet_dev_type_dvr] = ACTIONS(2053),
    [sym_duet_dev_type_disc_device] = ACTIONS(2053),
    [sym_duet_dev_type_document_camera] = ACTIONS(2053),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2053),
    [sym_duet_dev_type_hvac] = ACTIONS(2053),
    [sym_duet_dev_type_keypad] = ACTIONS(2053),
    [sym_duet_dev_type_light] = ACTIONS(2053),
    [sym_duet_dev_type_monitor] = ACTIONS(2053),
    [sym_duet_dev_type_motor] = ACTIONS(2053),
    [sym_duet_dev_type_multi_window] = ACTIONS(2053),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2053),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2053),
    [sym_duet_dev_type_receiver] = ACTIONS(2053),
    [sym_duet_dev_type_security_system] = ACTIONS(2053),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2053),
    [sym_duet_dev_type_settop_box] = ACTIONS(2053),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2053),
    [sym_duet_dev_type_switcher] = ACTIONS(2053),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2053),
    [sym_duet_dev_type_tv] = ACTIONS(2053),
    [sym_duet_dev_type_utility] = ACTIONS(2053),
    [sym_duet_dev_type_vcr] = ACTIONS(2053),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2053),
    [sym_duet_dev_type_video_processor] = ACTIONS(2053),
    [sym_duet_dev_type_video_projector] = ACTIONS(2053),
    [sym_duet_dev_type_video_wall] = ACTIONS(2053),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2053),
    [sym_duet_dev_type_weather] = ACTIONS(2053),
    [sym_duet_dev_not_polled] = ACTIONS(2053),
    [sym_duet_dev_polled] = ACTIONS(2053),
    [sym_clkmgr_mode_network] = ACTIONS(2053),
    [sym_clkmgr_mode_standalone] = ACTIONS(2053),
    [sym_normal_standby] = ACTIONS(2053),
    [sym_normal_wake] = ACTIONS(2053),
    [sym_ip_multicast_ttl_option] = ACTIONS(2053),
    [sym_ip_ttl_subnet] = ACTIONS(2053),
    [sym_ip_ttl_site] = ACTIONS(2053),
    [sym_ip_ttl_region] = ACTIONS(2053),
    [sym_ip_ttl_continent] = ACTIONS(2053),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2053),
    [sym_ip_nodelay_on] = ACTIONS(2053),
    [sym_ip_nodelay_off] = ACTIONS(2053),
    [sym_amx_error] = ACTIONS(2053),
    [sym_amx_warning] = ACTIONS(2053),
    [sym_amx_info] = ACTIONS(2053),
    [sym_amx_debug] = ACTIONS(2053),
    [sym_netlinx_login_success] = ACTIONS(2053),
    [sym_netlinx_login_fail] = ACTIONS(2053),
    [sym_netlinx_logout] = ACTIONS(2053),
    [sym_null_str] = ACTIONS(2053),
    [sym_smtp_address] = ACTIONS(2053),
    [sym_smtp_port_number] = ACTIONS(2053),
    [sym_smtp_username] = ACTIONS(2053),
    [sym_smtp_password] = ACTIONS(2053),
    [sym_smtp_require_tls] = ACTIONS(2053),
    [sym_smtp_from] = ACTIONS(2053),
    [sym_smtp_tls_true] = ACTIONS(2053),
    [sym_smtp_tls_false] = ACTIONS(2053),
    [sym_listview_on_row_select_event] = ACTIONS(2053),
    [sym_data_structure_unknown] = ACTIONS(2053),
    [sym_data_structure_datafeed] = ACTIONS(2053),
    [sym_data_structure_datarecordset] = ACTIONS(2053),
    [sym_data_structure_datarecord] = ACTIONS(2053),
    [sym_data_structure_datafield] = ACTIONS(2053),
    [sym_data_max_content_field_count] = ACTIONS(2053),
    [sym_data_max_metadata_field_count] = ACTIONS(2053),
    [sym_data_max_id_length] = ACTIONS(2053),
    [sym_data_max_name_length] = ACTIONS(2053),
    [sym_data_max_description_length] = ACTIONS(2053),
    [sym_data_max_type_length] = ACTIONS(2053),
    [sym_data_max_format_length] = ACTIONS(2053),
    [sym_data_max_source_length] = ACTIONS(2053),
    [sym_data_max_value_length] = ACTIONS(2053),
    [sym_data_max_label_length] = ACTIONS(2053),
    [sym_data_max_url_length] = ACTIONS(2053),
    [sym_max_last_login_info_length] = ACTIONS(2053),
    [sym_data_type_unknown] = ACTIONS(2053),
    [sym_data_type_string] = ACTIONS(2053),
    [sym_data_type_datetime] = ACTIONS(2053),
    [sym_data_type_date] = ACTIONS(2053),
    [sym_data_type_time] = ACTIONS(2053),
    [sym_data_type_image] = ACTIONS(2053),
    [sym_data_format_url] = ACTIONS(2053),
    [sym_data_format_phone] = ACTIONS(2053),
    [sym_data_format_email] = ACTIONS(2053),
    [sym_data_format_iso8601] = ACTIONS(2053),
    [sym_auditlog_priv] = ACTIONS(2053),
    [sym_configuration_priv] = ACTIONS(2053),
    [sym_deviceconfig_priv] = ACTIONS(2053),
    [sym_ftp_priv] = ACTIONS(2053),
    [sym_http_priv] = ACTIONS(2053),
    [sym_networkconfig_priv] = ACTIONS(2053),
    [sym_programport_priv] = ACTIONS(2053),
    [sym_remoteui_priv] = ACTIONS(2053),
    [sym_securitycontrol_priv] = ACTIONS(2053),
    [sym_softwaremanagement_priv] = ACTIONS(2053),
    [sym_terminal_priv] = ACTIONS(2053),
    [sym_tpadmin_priv] = ACTIONS(2053),
    [sym_user1_priv] = ACTIONS(2053),
    [sym_user2_priv] = ACTIONS(2053),
    [sym_user3_priv] = ACTIONS(2053),
    [sym_user4_priv] = ACTIONS(2053),
    [sym_usermanagement_priv] = ACTIONS(2053),
    [sym_valid_account] = ACTIONS(2053),
    [sym_err_name_invalid] = ACTIONS(2053),
    [sym_err_password_invalid] = ACTIONS(2053),
    [sym_err_invalid_account] = ACTIONS(2053),
    [sym_err_no_authorization] = ACTIONS(2053),
    [sym_err_invalid_parameter] = ACTIONS(2053),
    [sym_err_user_locked_out] = ACTIONS(2053),
    [sym_err_user_account_expired] = ACTIONS(2053),
    [sym_err_general_error] = ACTIONS(2053),
    [sym_err_authentication_error] = ACTIONS(2053),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2053),
    [sym_tls_validate_certificate] = ACTIONS(2053),
    [sym_tdata] = ACTIONS(2053),
    [sym_tchannel] = ACTIONS(2053),
    [sym_tlevel] = ACTIONS(2053),
    [sym_tbutton] = ACTIONS(2053),
    [sym_ttimeline] = ACTIONS(2053),
    [sym_tcustom] = ACTIONS(2053),
    [sym_url_struct] = ACTIONS(2053),
    [sym_dns_struct] = ACTIONS(2053),
    [sym_ip_address_struct] = ACTIONS(2053),
    [sym_dev_info_struct] = ACTIONS(2053),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2053),
    [sym_data_feed] = ACTIONS(2053),
    [sym_data_field] = ACTIONS(2053),
    [sym_data_record] = ACTIONS(2053),
    [sym_wc_data_feed] = ACTIONS(2053),
    [sym_wc_data_field] = ACTIONS(2053),
    [sym_wc_data_record] = ACTIONS(2053),
    [sym_last_login_info] = ACTIONS(2053),
    [sym____reserved___] = ACTIONS(2053),
    [sym_date] = ACTIONS(2053),
    [sym_day] = ACTIONS(2053),
    [sym_time] = ACTIONS(2053),
    [sym_ldate] = ACTIONS(2053),
    [sym_push_device] = ACTIONS(2053),
    [sym_push_channel] = ACTIONS(2053),
    [sym_push_devchan] = ACTIONS(2053),
    [sym_release_device] = ACTIONS(2053),
    [sym_release_channel] = ACTIONS(2053),
    [sym_release_devchan] = ACTIONS(2053),
    [sym_master_slot] = ACTIONS(2053),
    [sym_get_pulse_time] = ACTIONS(2053),
    [sym_get_timer] = ACTIONS(2053),
    [sym_master_sn] = ACTIONS(2053),
    [sym_system_number] = ACTIONS(2053),
    [sym_button] = ACTIONS(2053),
    [sym_dv_channel] = ACTIONS(2053),
    [sym_data] = ACTIONS(2053),
    [sym_level] = ACTIONS(2053),
    [sym_timeline] = ACTIONS(2053),
    [sym_channel] = ACTIONS(2053),
    [sym_custom] = ACTIONS(2053),
    [sym_atoi] = ACTIONS(2053),
    [sym_atol] = ACTIONS(2053),
    [sym_atof] = ACTIONS(2053),
    [sym_itoa] = ACTIONS(2053),
    [sym_itohex] = ACTIONS(2053),
    [sym_hextoi] = ACTIONS(2053),
    [sym_ftoa] = ACTIONS(2053),
    [sym_find_string] = ACTIONS(2053),
    [sym_left_string] = ACTIONS(2053),
    [sym_right_string] = ACTIONS(2053),
    [sym_mid_string] = ACTIONS(2053),
    [sym_remove_string] = ACTIONS(2053),
    [sym_get_buffer_string] = ACTIONS(2053),
    [sym_get_buffer_char] = ACTIONS(2053),
    [sym_get_multi_buffer_string] = ACTIONS(2053),
    [sym_length_string] = ACTIONS(2053),
    [sym_max_length_string] = ACTIONS(2053),
    [sym_set_length_string] = ACTIONS(2053),
    [sym_compare_string] = ACTIONS(2053),
    [sym_upper_string] = ACTIONS(2053),
    [sym_lower_string] = ACTIONS(2053),
    [sym_get_last] = ACTIONS(2053),
    [sym_date_to_day] = ACTIONS(2053),
    [sym_date_to_month] = ACTIONS(2053),
    [sym_date_to_year] = ACTIONS(2053),
    [sym_time_to_hour] = ACTIONS(2053),
    [sym_time_to_minute] = ACTIONS(2053),
    [sym_time_to_second] = ACTIONS(2053),
    [sym_day_of_week] = ACTIONS(2053),
    [sym_set_pulse_time] = ACTIONS(2053),
    [sym_set_timer] = ACTIONS(2053),
    [sym_file_open] = ACTIONS(2053),
    [sym_file_close] = ACTIONS(2053),
    [sym_file_read] = ACTIONS(2053),
    [sym_file_read_line] = ACTIONS(2053),
    [sym_file_write] = ACTIONS(2053),
    [sym_file_write_line] = ACTIONS(2053),
    [sym_file_copy] = ACTIONS(2053),
    [sym_file_delete] = ACTIONS(2053),
    [sym_file_rename] = ACTIONS(2053),
    [sym_file_dir] = ACTIONS(2053),
    [sym_file_getdir] = ACTIONS(2053),
    [sym_file_setdir] = ACTIONS(2053),
    [sym_file_create_dir] = ACTIONS(2053),
    [sym_file_remove_dir] = ACTIONS(2053),
    [sym_file_seek] = ACTIONS(2053),
    [sym_length_array] = ACTIONS(2053),
    [sym_max_length_array] = ACTIONS(2053),
    [sym_set_length_array] = ACTIONS(2053),
    [sym_ip_client_open] = ACTIONS(2053),
    [sym_ip_client_close] = ACTIONS(2053),
    [sym_ip_server_open] = ACTIONS(2053),
    [sym_ip_server_close] = ACTIONS(2053),
    [sym_ip_bound_client_open] = ACTIONS(2053),
    [sym_ip_mc_server_open] = ACTIONS(2053),
    [sym_ip_set_option] = ACTIONS(2053),
    [sym_ssh_client_open] = ACTIONS(2053),
    [sym_ssh_client_close] = ACTIONS(2053),
    [sym_tls_client_open] = ACTIONS(2053),
    [sym_tls_client_close] = ACTIONS(2053),
    [sym_do_push] = ACTIONS(2053),
    [sym_do_push_timed] = ACTIONS(2053),
    [sym_do_release] = ACTIONS(2053),
    [sym_device_id] = ACTIONS(2053),
    [sym_device_info] = ACTIONS(2053),
    [sym_device_id_string] = ACTIONS(2053),
    [sym_device_standby] = ACTIONS(2053),
    [sym_device_wake] = ACTIONS(2053),
    [sym_timeline_create] = ACTIONS(2053),
    [sym_timeline_kill] = ACTIONS(2053),
    [sym_timeline_reload] = ACTIONS(2053),
    [sym_timeline_pause] = ACTIONS(2053),
    [sym_timeline_restart] = ACTIONS(2053),
    [sym_timeline_set] = ACTIONS(2053),
    [sym_timeline_get] = ACTIONS(2053),
    [sym_timeline_active] = ACTIONS(2053),
    [sym_timeline_dynamic_id] = ACTIONS(2053),
    [sym_variable_to_string] = ACTIONS(2053),
    [sym_string_to_variable] = ACTIONS(2053),
    [sym_length_variable_to_string] = ACTIONS(2053),
    [sym_type_cast] = ACTIONS(2053),
    [sym_variable_to_xml] = ACTIONS(2053),
    [sym_xml_to_variable] = ACTIONS(2053),
    [sym_length_variable_to_xml] = ACTIONS(2053),
    [sym_redirect_string] = ACTIONS(2053),
    [sym_random_number] = ACTIONS(2053),
    [sym_get_unique_id] = ACTIONS(2053),
    [sym_get_system_number] = ACTIONS(2053),
    [sym_set_system_number] = ACTIONS(2053),
    [sym_get_serial_number] = ACTIONS(2053),
    [sym_get_master_build] = ACTIONS(2053),
    [sym_set_validation_code] = ACTIONS(2053),
    [sym_reboot] = ACTIONS(2053),
    [sym_load_duet_module] = ACTIONS(2053),
    [sym_unload_duet_module] = ACTIONS(2053),
    [sym_duet_mem_size_set] = ACTIONS(2053),
    [sym_duet_mem_size_get] = ACTIONS(2053),
    [sym_dynamic_application_device] = ACTIONS(2053),
    [sym_dynamic_polled_port] = ACTIONS(2053),
    [sym_static_port_binding] = ACTIONS(2053),
    [sym_static_ip_binding] = ACTIONS(2053),
    [sym_get_dns_list] = ACTIONS(2053),
    [sym_set_dns_list] = ACTIONS(2053),
    [sym_get_ip_address] = ACTIONS(2053),
    [sym_set_ip_address] = ACTIONS(2053),
    [sym_get_url_list] = ACTIONS(2053),
    [sym_add_url_entry] = ACTIONS(2053),
    [sym_delete_url_entry] = ACTIONS(2053),
    [sym_abs_value] = ACTIONS(2053),
    [sym_max_value] = ACTIONS(2053),
    [sym_min_value] = ACTIONS(2053),
    [sym_exp_value] = ACTIONS(2053),
    [sym_log_value] = ACTIONS(2053),
    [sym_log10_value] = ACTIONS(2053),
    [sym_power_value] = ACTIONS(2053),
    [sym_sqrt_value] = ACTIONS(2053),
    [sym_raw_be] = ACTIONS(2053),
    [sym_raw_le] = ACTIONS(2053),
    [sym_internal_queue_size_set] = ACTIONS(2053),
    [sym_internal_queue_size_get] = ACTIONS(2053),
    [sym_internal_threshold_set] = ACTIONS(2053),
    [sym_internal_threshold_get] = ACTIONS(2053),
    [sym_rebuilt_event] = ACTIONS(2053),
    [sym_do_custom_event] = ACTIONS(2053),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2053),
    [sym_clkmgr_set_clk_source] = ACTIONS(2053),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2053),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2053),
    [sym_clkmgr_get_timezone] = ACTIONS(2053),
    [sym_clkmgr_set_timezone] = ACTIONS(2053),
    [sym_clkmgr_get_resync_period] = ACTIONS(2053),
    [sym_clkmgr_set_resync_period] = ACTIONS(2053),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2053),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2053),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2053),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2053),
    [sym_clkmgr_get_timeservers] = ACTIONS(2053),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2053),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2053),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2053),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2053),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2053),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2053),
    [sym_set_log_level] = ACTIONS(2053),
    [sym_get_log_level] = ACTIONS(2053),
    [sym_amx_log] = ACTIONS(2053),
    [sym_smtp_server_config_set] = ACTIONS(2053),
    [sym_smtp_server_config_get] = ACTIONS(2053),
    [sym_smtp_send] = ACTIONS(2053),
    [sym_validate_netlinx_account] = ACTIONS(2053),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2053),
    [sym_authenticate_certificate] = ACTIONS(2053),
    [sym_audit_netlinx_session_event] = ACTIONS(2053),
    [sym_audit_netlinx_generic_event] = ACTIONS(2053),
    [sym_get_available_flash_disk_space] = ACTIONS(2053),
    [sym_get_max_flash_disk_space] = ACTIONS(2053),
    [sym_data_create_feed] = ACTIONS(2053),
    [sym_data_delete_feed] = ACTIONS(2053),
    [sym_data_publish_feed] = ACTIONS(2053),
    [sym_data_get_published_feed] = ACTIONS(2053),
    [sym_data_add_record] = ACTIONS(2053),
    [sym_data_get_event_record] = ACTIONS(2053),
    [sym__wc_data_create_feed] = ACTIONS(2053),
    [sym__wc_data_add_record] = ACTIONS(2053),
    [sym__wc_data_get_event_record] = ACTIONS(2053),
    [sym_format] = ACTIONS(2053),
    [sym_astro_clock] = ACTIONS(2053),
    [sym_set_outdoor_temperature] = ACTIONS(2053),
    [sym_set_virtual_level_count] = ACTIONS(2053),
    [sym_set_virtual_channel_count] = ACTIONS(2053),
    [sym_set_virtual_port_count] = ACTIONS(2053),
    [sym__wc_encode_int] = ACTIONS(2053),
    [sym__wc_decode_int] = ACTIONS(2053),
    [sym__wc_int] = ACTIONS(2053),
    [sym__wc_to_ch_int] = ACTIONS(2053),
    [sym__ch_to_wc_int] = ACTIONS(2053),
    [sym__wc_find_string_int] = ACTIONS(2053),
    [sym__wc_left_string_int] = ACTIONS(2053),
    [sym__wc_lower_string_int] = ACTIONS(2053),
    [sym__wc_mid_string_int] = ACTIONS(2053),
    [sym__wc_remove_string_int] = ACTIONS(2053),
    [sym__wc_right_string_int] = ACTIONS(2053),
    [sym__wc_upper_string_int] = ACTIONS(2053),
    [sym__wc_compare_string_int] = ACTIONS(2053),
    [sym__wc_get_buffer_char_int] = ACTIONS(2053),
    [sym__wc_get_buffer_string_int] = ACTIONS(2053),
    [sym__wc_concat_string_int] = ACTIONS(2053),
    [sym__wc_file_open_int] = ACTIONS(2053),
    [sym__wc_file_close_int] = ACTIONS(2053),
    [sym__wc_file_read_int] = ACTIONS(2053),
    [sym__wc_file_read_line_int] = ACTIONS(2053),
    [sym__wc_file_write_int] = ACTIONS(2053),
    [sym__wc_file_write_line_int] = ACTIONS(2053),
    [aux_sym_preproc_include_token1] = ACTIONS(2053),
    [aux_sym_preproc_define_token1] = ACTIONS(2055),
    [aux_sym_preproc_warn_token1] = ACTIONS(2055),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2055),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2055),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2055),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2055),
    [aux_sym_preproc_else_token1] = ACTIONS(2055),
    [anon_sym_LBRACK] = ACTIONS(2055),
    [anon_sym_LBRACE] = ACTIONS(2055),
    [anon_sym_LPAREN] = ACTIONS(2053),
    [anon_sym_BANG] = ACTIONS(2055),
    [anon_sym_TILDE] = ACTIONS(2055),
    [anon_sym_DASH] = ACTIONS(2053),
    [anon_sym_PLUS] = ACTIONS(2053),
    [anon_sym_DASH_DASH] = ACTIONS(2055),
    [anon_sym_PLUS_PLUS] = ACTIONS(2055),
    [anon_sym_DQUOTE] = ACTIONS(2055),
    [anon_sym_SQUOTE] = ACTIONS(2055),
    [sym_number_literal] = ACTIONS(2055),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(340)] = {
    [ts_builtin_sym_end] = ACTIONS(1745),
    [sym_identifier] = ACTIONS(1743),
    [sym_program_name_keyword] = ACTIONS(1743),
    [sym_module_name_keyword] = ACTIONS(1743),
    [sym_define_device_keyword] = ACTIONS(1743),
    [sym_define_combine_keyword] = ACTIONS(1743),
    [sym_define_constant_keyword] = ACTIONS(1743),
    [sym_define_type_keyword] = ACTIONS(1743),
    [sym_define_variable_keyword] = ACTIONS(1743),
    [sym_define_system_variable_keyword] = ACTIONS(1743),
    [sym_define_connect_level_keyword] = ACTIONS(1743),
    [sym_define_latching_keyword] = ACTIONS(1743),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1743),
    [sym_define_toggling_keyword] = ACTIONS(1743),
    [sym_define_call_keyword] = ACTIONS(1743),
    [sym_define_function_keyword] = ACTIONS(1743),
    [sym_define_library_function_keyword] = ACTIONS(1743),
    [sym_define_start_keyword] = ACTIONS(1743),
    [sym_define_event_keyword] = ACTIONS(1743),
    [sym_define_program_keyword] = ACTIONS(1743),
    [sym_define_module_keyword] = ACTIONS(1743),
    [sym_char_keyword] = ACTIONS(1743),
    [sym_widechar_keyword] = ACTIONS(1743),
    [sym_integer_keyword] = ACTIONS(1743),
    [sym_sinteger_keyword] = ACTIONS(1743),
    [sym_long_keyword] = ACTIONS(1743),
    [sym_slong_keyword] = ACTIONS(1743),
    [sym_float_keyword] = ACTIONS(1743),
    [sym_double_keyword] = ACTIONS(1743),
    [sym_variant_keyword] = ACTIONS(1743),
    [sym_variantarray_keyword] = ACTIONS(1743),
    [sym_local_var_keyword] = ACTIONS(1743),
    [sym_stack_var_keyword] = ACTIONS(1743),
    [sym_constant_keyword] = ACTIONS(1743),
    [sym_volatile_keyword] = ACTIONS(1743),
    [sym_non_volatile_keyword] = ACTIONS(1743),
    [sym_persistent_keyword] = ACTIONS(1743),
    [sym_dev_keyword] = ACTIONS(1743),
    [sym_devlev_keyword] = ACTIONS(1743),
    [sym_devchan_keyword] = ACTIONS(1743),
    [sym_if_keyword] = ACTIONS(1743),
    [sym_else_keyword] = ACTIONS(1743),
    [sym_switch_keyword] = ACTIONS(1743),
    [sym_case_keyword] = ACTIONS(1743),
    [sym_default_keyword] = ACTIONS(1743),
    [sym_while_keyword] = ACTIONS(1743),
    [sym_for_keyword] = ACTIONS(1743),
    [sym_break_keyword] = ACTIONS(1743),
    [sym_continue_keyword] = ACTIONS(1743),
    [sym_return_keyword] = ACTIONS(1743),
    [sym_select_keyword] = ACTIONS(1743),
    [sym_active_keyword] = ACTIONS(1743),
    [sym_struct_keyword] = ACTIONS(1743),
    [sym_structure_keyword] = ACTIONS(1743),
    [sym_button_event_keyword] = ACTIONS(1743),
    [sym_channel_event_keyword] = ACTIONS(1743),
    [sym_level_event_keyword] = ACTIONS(1743),
    [sym_data_event_keyword] = ACTIONS(1743),
    [sym_timeline_event_keyword] = ACTIONS(1743),
    [sym_custom_event_keyword] = ACTIONS(1743),
    [sym_push_keyword] = ACTIONS(1743),
    [sym_release_keyword] = ACTIONS(1743),
    [aux_sym_on_keyword_token1] = ACTIONS(1743),
    [aux_sym_off_keyword_token1] = ACTIONS(1743),
    [sym_send_level_keyword] = ACTIONS(1743),
    [sym_send_string_keyword] = ACTIONS(1743),
    [sym_send_command_keyword] = ACTIONS(1743),
    [sym_clear_buffer_keyword] = ACTIONS(1743),
    [sym_create_buffer_keyword] = ACTIONS(1743),
    [sym_create_multi_buffer_keyword] = ACTIONS(1743),
    [sym_call_keyword] = ACTIONS(1743),
    [sym_system_call_keyword] = ACTIONS(1743),
    [sym_devchan_to_keyword] = ACTIONS(1743),
    [sym_devchan_min_to_keyword] = ACTIONS(1743),
    [sym_devchan_total_off_keyword] = ACTIONS(1743),
    [sym_devchan_pulse_keyword] = ACTIONS(1743),
    [sym_wait_keyword] = ACTIONS(1743),
    [sym_cancel_wait_keyword] = ACTIONS(1743),
    [sym_wait_until_keyword] = ACTIONS(1743),
    [sym_cancel_wait_until_keyword] = ACTIONS(1743),
    [sym_cancel_all_wait_keyword] = ACTIONS(1743),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1743),
    [sym_not] = ACTIONS(1743),
    [sym_bnot] = ACTIONS(1743),
    [sym___file__] = ACTIONS(1743),
    [sym___date__] = ACTIONS(1743),
    [sym___ldate__] = ACTIONS(1743),
    [sym___time__] = ACTIONS(1743),
    [sym___line__] = ACTIONS(1743),
    [sym___name__] = ACTIONS(1743),
    [sym___netlinx__] = ACTIONS(1743),
    [sym_netlinx_axi_version] = ACTIONS(1743),
    [sym_url_flg_tcp] = ACTIONS(1743),
    [sym_url_flg_accinfopresent] = ACTIONS(1743),
    [sym_url_flg_temp] = ACTIONS(1743),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1743),
    [sym_url_flg_stat_mask] = ACTIONS(1743),
    [sym_url_flg_stat_lookup] = ACTIONS(1743),
    [sym_url_flg_stat_connecting] = ACTIONS(1743),
    [sym_url_flg_stat_waiting] = ACTIONS(1743),
    [sym_url_flg_stat_connected] = ACTIONS(1743),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1743),
    [sym_first_virtual_device] = ACTIONS(1743),
    [sym_first_local_port] = ACTIONS(1743),
    [sym_dynamic_virtual_device] = ACTIONS(1743),
    [sym_timeline_once] = ACTIONS(1743),
    [sym_timeline_repeat] = ACTIONS(1743),
    [sym_timeline_absolute] = ACTIONS(1743),
    [sym_timeline_relative] = ACTIONS(1743),
    [sym_do_push_timed_infinite] = ACTIONS(1743),
    [sym_source_type_no_address] = ACTIONS(1743),
    [sym_source_type_neuron_id] = ACTIONS(1743),
    [sym_source_type_ip_address] = ACTIONS(1743),
    [sym_source_type_axlink] = ACTIONS(1743),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1743),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1743),
    [sym_source_type_ip_socket_address] = ACTIONS(1743),
    [sym_source_type_rs232] = ACTIONS(1743),
    [sym_source_type_internal] = ACTIONS(1743),
    [sym_source_type_mac_address] = ACTIONS(1743),
    [sym_source_type_ipv4_port] = ACTIONS(1743),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1743),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1743),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1743),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1743),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1743),
    [sym_true] = ACTIONS(1743),
    [sym_false] = ACTIONS(1743),
    [sym_file_read_only] = ACTIONS(1743),
    [sym_file_rw_new] = ACTIONS(1743),
    [sym_file_rw_append] = ACTIONS(1743),
    [sym_ip_tcp] = ACTIONS(1743),
    [sym_ip_udp] = ACTIONS(1743),
    [sym_ip_udp_2way] = ACTIONS(1743),
    [sym_xml_encode_types] = ACTIONS(1743),
    [sym_xml_encode_char_as_list] = ACTIONS(1743),
    [sym_xml_encode_le] = ACTIONS(1743),
    [sym_xml_decode_types] = ACTIONS(1743),
    [sym_xml_decode_no_preserve] = ACTIONS(1743),
    [sym_duet_dev_type_display_device] = ACTIONS(1743),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1743),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1743),
    [sym_duet_dev_type_io_device] = ACTIONS(1743),
    [sym_duet_dev_type_relay_device] = ACTIONS(1743),
    [sym_duet_dev_type_ups] = ACTIONS(1743),
    [sym_duet_dev_type_amplifier] = ACTIONS(1743),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1743),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1743),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1743),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1743),
    [sym_duet_dev_type_camera] = ACTIONS(1743),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1743),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1743),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1743),
    [sym_duet_dev_type_dss] = ACTIONS(1743),
    [sym_duet_dev_type_dvr] = ACTIONS(1743),
    [sym_duet_dev_type_disc_device] = ACTIONS(1743),
    [sym_duet_dev_type_document_camera] = ACTIONS(1743),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1743),
    [sym_duet_dev_type_hvac] = ACTIONS(1743),
    [sym_duet_dev_type_keypad] = ACTIONS(1743),
    [sym_duet_dev_type_light] = ACTIONS(1743),
    [sym_duet_dev_type_monitor] = ACTIONS(1743),
    [sym_duet_dev_type_motor] = ACTIONS(1743),
    [sym_duet_dev_type_multi_window] = ACTIONS(1743),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1743),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1743),
    [sym_duet_dev_type_receiver] = ACTIONS(1743),
    [sym_duet_dev_type_security_system] = ACTIONS(1743),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1743),
    [sym_duet_dev_type_settop_box] = ACTIONS(1743),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1743),
    [sym_duet_dev_type_switcher] = ACTIONS(1743),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1743),
    [sym_duet_dev_type_tv] = ACTIONS(1743),
    [sym_duet_dev_type_utility] = ACTIONS(1743),
    [sym_duet_dev_type_vcr] = ACTIONS(1743),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1743),
    [sym_duet_dev_type_video_processor] = ACTIONS(1743),
    [sym_duet_dev_type_video_projector] = ACTIONS(1743),
    [sym_duet_dev_type_video_wall] = ACTIONS(1743),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1743),
    [sym_duet_dev_type_weather] = ACTIONS(1743),
    [sym_duet_dev_not_polled] = ACTIONS(1743),
    [sym_duet_dev_polled] = ACTIONS(1743),
    [sym_clkmgr_mode_network] = ACTIONS(1743),
    [sym_clkmgr_mode_standalone] = ACTIONS(1743),
    [sym_normal_standby] = ACTIONS(1743),
    [sym_normal_wake] = ACTIONS(1743),
    [sym_ip_multicast_ttl_option] = ACTIONS(1743),
    [sym_ip_ttl_subnet] = ACTIONS(1743),
    [sym_ip_ttl_site] = ACTIONS(1743),
    [sym_ip_ttl_region] = ACTIONS(1743),
    [sym_ip_ttl_continent] = ACTIONS(1743),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1743),
    [sym_ip_nodelay_on] = ACTIONS(1743),
    [sym_ip_nodelay_off] = ACTIONS(1743),
    [sym_amx_error] = ACTIONS(1743),
    [sym_amx_warning] = ACTIONS(1743),
    [sym_amx_info] = ACTIONS(1743),
    [sym_amx_debug] = ACTIONS(1743),
    [sym_netlinx_login_success] = ACTIONS(1743),
    [sym_netlinx_login_fail] = ACTIONS(1743),
    [sym_netlinx_logout] = ACTIONS(1743),
    [sym_null_str] = ACTIONS(1743),
    [sym_smtp_address] = ACTIONS(1743),
    [sym_smtp_port_number] = ACTIONS(1743),
    [sym_smtp_username] = ACTIONS(1743),
    [sym_smtp_password] = ACTIONS(1743),
    [sym_smtp_require_tls] = ACTIONS(1743),
    [sym_smtp_from] = ACTIONS(1743),
    [sym_smtp_tls_true] = ACTIONS(1743),
    [sym_smtp_tls_false] = ACTIONS(1743),
    [sym_listview_on_row_select_event] = ACTIONS(1743),
    [sym_data_structure_unknown] = ACTIONS(1743),
    [sym_data_structure_datafeed] = ACTIONS(1743),
    [sym_data_structure_datarecordset] = ACTIONS(1743),
    [sym_data_structure_datarecord] = ACTIONS(1743),
    [sym_data_structure_datafield] = ACTIONS(1743),
    [sym_data_max_content_field_count] = ACTIONS(1743),
    [sym_data_max_metadata_field_count] = ACTIONS(1743),
    [sym_data_max_id_length] = ACTIONS(1743),
    [sym_data_max_name_length] = ACTIONS(1743),
    [sym_data_max_description_length] = ACTIONS(1743),
    [sym_data_max_type_length] = ACTIONS(1743),
    [sym_data_max_format_length] = ACTIONS(1743),
    [sym_data_max_source_length] = ACTIONS(1743),
    [sym_data_max_value_length] = ACTIONS(1743),
    [sym_data_max_label_length] = ACTIONS(1743),
    [sym_data_max_url_length] = ACTIONS(1743),
    [sym_max_last_login_info_length] = ACTIONS(1743),
    [sym_data_type_unknown] = ACTIONS(1743),
    [sym_data_type_string] = ACTIONS(1743),
    [sym_data_type_datetime] = ACTIONS(1743),
    [sym_data_type_date] = ACTIONS(1743),
    [sym_data_type_time] = ACTIONS(1743),
    [sym_data_type_image] = ACTIONS(1743),
    [sym_data_format_url] = ACTIONS(1743),
    [sym_data_format_phone] = ACTIONS(1743),
    [sym_data_format_email] = ACTIONS(1743),
    [sym_data_format_iso8601] = ACTIONS(1743),
    [sym_auditlog_priv] = ACTIONS(1743),
    [sym_configuration_priv] = ACTIONS(1743),
    [sym_deviceconfig_priv] = ACTIONS(1743),
    [sym_ftp_priv] = ACTIONS(1743),
    [sym_http_priv] = ACTIONS(1743),
    [sym_networkconfig_priv] = ACTIONS(1743),
    [sym_programport_priv] = ACTIONS(1743),
    [sym_remoteui_priv] = ACTIONS(1743),
    [sym_securitycontrol_priv] = ACTIONS(1743),
    [sym_softwaremanagement_priv] = ACTIONS(1743),
    [sym_terminal_priv] = ACTIONS(1743),
    [sym_tpadmin_priv] = ACTIONS(1743),
    [sym_user1_priv] = ACTIONS(1743),
    [sym_user2_priv] = ACTIONS(1743),
    [sym_user3_priv] = ACTIONS(1743),
    [sym_user4_priv] = ACTIONS(1743),
    [sym_usermanagement_priv] = ACTIONS(1743),
    [sym_valid_account] = ACTIONS(1743),
    [sym_err_name_invalid] = ACTIONS(1743),
    [sym_err_password_invalid] = ACTIONS(1743),
    [sym_err_invalid_account] = ACTIONS(1743),
    [sym_err_no_authorization] = ACTIONS(1743),
    [sym_err_invalid_parameter] = ACTIONS(1743),
    [sym_err_user_locked_out] = ACTIONS(1743),
    [sym_err_user_account_expired] = ACTIONS(1743),
    [sym_err_general_error] = ACTIONS(1743),
    [sym_err_authentication_error] = ACTIONS(1743),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1743),
    [sym_tls_validate_certificate] = ACTIONS(1743),
    [sym_tdata] = ACTIONS(1743),
    [sym_tchannel] = ACTIONS(1743),
    [sym_tlevel] = ACTIONS(1743),
    [sym_tbutton] = ACTIONS(1743),
    [sym_ttimeline] = ACTIONS(1743),
    [sym_tcustom] = ACTIONS(1743),
    [sym_url_struct] = ACTIONS(1743),
    [sym_dns_struct] = ACTIONS(1743),
    [sym_ip_address_struct] = ACTIONS(1743),
    [sym_dev_info_struct] = ACTIONS(1743),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1743),
    [sym_data_feed] = ACTIONS(1743),
    [sym_data_field] = ACTIONS(1743),
    [sym_data_record] = ACTIONS(1743),
    [sym_wc_data_feed] = ACTIONS(1743),
    [sym_wc_data_field] = ACTIONS(1743),
    [sym_wc_data_record] = ACTIONS(1743),
    [sym_last_login_info] = ACTIONS(1743),
    [sym____reserved___] = ACTIONS(1743),
    [sym_date] = ACTIONS(1743),
    [sym_day] = ACTIONS(1743),
    [sym_time] = ACTIONS(1743),
    [sym_ldate] = ACTIONS(1743),
    [sym_push_device] = ACTIONS(1743),
    [sym_push_channel] = ACTIONS(1743),
    [sym_push_devchan] = ACTIONS(1743),
    [sym_release_device] = ACTIONS(1743),
    [sym_release_channel] = ACTIONS(1743),
    [sym_release_devchan] = ACTIONS(1743),
    [sym_master_slot] = ACTIONS(1743),
    [sym_get_pulse_time] = ACTIONS(1743),
    [sym_get_timer] = ACTIONS(1743),
    [sym_master_sn] = ACTIONS(1743),
    [sym_system_number] = ACTIONS(1743),
    [sym_button] = ACTIONS(1743),
    [sym_dv_channel] = ACTIONS(1743),
    [sym_data] = ACTIONS(1743),
    [sym_level] = ACTIONS(1743),
    [sym_timeline] = ACTIONS(1743),
    [sym_channel] = ACTIONS(1743),
    [sym_custom] = ACTIONS(1743),
    [sym_atoi] = ACTIONS(1743),
    [sym_atol] = ACTIONS(1743),
    [sym_atof] = ACTIONS(1743),
    [sym_itoa] = ACTIONS(1743),
    [sym_itohex] = ACTIONS(1743),
    [sym_hextoi] = ACTIONS(1743),
    [sym_ftoa] = ACTIONS(1743),
    [sym_find_string] = ACTIONS(1743),
    [sym_left_string] = ACTIONS(1743),
    [sym_right_string] = ACTIONS(1743),
    [sym_mid_string] = ACTIONS(1743),
    [sym_remove_string] = ACTIONS(1743),
    [sym_get_buffer_string] = ACTIONS(1743),
    [sym_get_buffer_char] = ACTIONS(1743),
    [sym_get_multi_buffer_string] = ACTIONS(1743),
    [sym_length_string] = ACTIONS(1743),
    [sym_max_length_string] = ACTIONS(1743),
    [sym_set_length_string] = ACTIONS(1743),
    [sym_compare_string] = ACTIONS(1743),
    [sym_upper_string] = ACTIONS(1743),
    [sym_lower_string] = ACTIONS(1743),
    [sym_get_last] = ACTIONS(1743),
    [sym_date_to_day] = ACTIONS(1743),
    [sym_date_to_month] = ACTIONS(1743),
    [sym_date_to_year] = ACTIONS(1743),
    [sym_time_to_hour] = ACTIONS(1743),
    [sym_time_to_minute] = ACTIONS(1743),
    [sym_time_to_second] = ACTIONS(1743),
    [sym_day_of_week] = ACTIONS(1743),
    [sym_set_pulse_time] = ACTIONS(1743),
    [sym_set_timer] = ACTIONS(1743),
    [sym_file_open] = ACTIONS(1743),
    [sym_file_close] = ACTIONS(1743),
    [sym_file_read] = ACTIONS(1743),
    [sym_file_read_line] = ACTIONS(1743),
    [sym_file_write] = ACTIONS(1743),
    [sym_file_write_line] = ACTIONS(1743),
    [sym_file_copy] = ACTIONS(1743),
    [sym_file_delete] = ACTIONS(1743),
    [sym_file_rename] = ACTIONS(1743),
    [sym_file_dir] = ACTIONS(1743),
    [sym_file_getdir] = ACTIONS(1743),
    [sym_file_setdir] = ACTIONS(1743),
    [sym_file_create_dir] = ACTIONS(1743),
    [sym_file_remove_dir] = ACTIONS(1743),
    [sym_file_seek] = ACTIONS(1743),
    [sym_length_array] = ACTIONS(1743),
    [sym_max_length_array] = ACTIONS(1743),
    [sym_set_length_array] = ACTIONS(1743),
    [sym_ip_client_open] = ACTIONS(1743),
    [sym_ip_client_close] = ACTIONS(1743),
    [sym_ip_server_open] = ACTIONS(1743),
    [sym_ip_server_close] = ACTIONS(1743),
    [sym_ip_bound_client_open] = ACTIONS(1743),
    [sym_ip_mc_server_open] = ACTIONS(1743),
    [sym_ip_set_option] = ACTIONS(1743),
    [sym_ssh_client_open] = ACTIONS(1743),
    [sym_ssh_client_close] = ACTIONS(1743),
    [sym_tls_client_open] = ACTIONS(1743),
    [sym_tls_client_close] = ACTIONS(1743),
    [sym_do_push] = ACTIONS(1743),
    [sym_do_push_timed] = ACTIONS(1743),
    [sym_do_release] = ACTIONS(1743),
    [sym_device_id] = ACTIONS(1743),
    [sym_device_info] = ACTIONS(1743),
    [sym_device_id_string] = ACTIONS(1743),
    [sym_device_standby] = ACTIONS(1743),
    [sym_device_wake] = ACTIONS(1743),
    [sym_timeline_create] = ACTIONS(1743),
    [sym_timeline_kill] = ACTIONS(1743),
    [sym_timeline_reload] = ACTIONS(1743),
    [sym_timeline_pause] = ACTIONS(1743),
    [sym_timeline_restart] = ACTIONS(1743),
    [sym_timeline_set] = ACTIONS(1743),
    [sym_timeline_get] = ACTIONS(1743),
    [sym_timeline_active] = ACTIONS(1743),
    [sym_timeline_dynamic_id] = ACTIONS(1743),
    [sym_variable_to_string] = ACTIONS(1743),
    [sym_string_to_variable] = ACTIONS(1743),
    [sym_length_variable_to_string] = ACTIONS(1743),
    [sym_type_cast] = ACTIONS(1743),
    [sym_variable_to_xml] = ACTIONS(1743),
    [sym_xml_to_variable] = ACTIONS(1743),
    [sym_length_variable_to_xml] = ACTIONS(1743),
    [sym_redirect_string] = ACTIONS(1743),
    [sym_random_number] = ACTIONS(1743),
    [sym_get_unique_id] = ACTIONS(1743),
    [sym_get_system_number] = ACTIONS(1743),
    [sym_set_system_number] = ACTIONS(1743),
    [sym_get_serial_number] = ACTIONS(1743),
    [sym_get_master_build] = ACTIONS(1743),
    [sym_set_validation_code] = ACTIONS(1743),
    [sym_reboot] = ACTIONS(1743),
    [sym_load_duet_module] = ACTIONS(1743),
    [sym_unload_duet_module] = ACTIONS(1743),
    [sym_duet_mem_size_set] = ACTIONS(1743),
    [sym_duet_mem_size_get] = ACTIONS(1743),
    [sym_dynamic_application_device] = ACTIONS(1743),
    [sym_dynamic_polled_port] = ACTIONS(1743),
    [sym_static_port_binding] = ACTIONS(1743),
    [sym_static_ip_binding] = ACTIONS(1743),
    [sym_get_dns_list] = ACTIONS(1743),
    [sym_set_dns_list] = ACTIONS(1743),
    [sym_get_ip_address] = ACTIONS(1743),
    [sym_set_ip_address] = ACTIONS(1743),
    [sym_get_url_list] = ACTIONS(1743),
    [sym_add_url_entry] = ACTIONS(1743),
    [sym_delete_url_entry] = ACTIONS(1743),
    [sym_abs_value] = ACTIONS(1743),
    [sym_max_value] = ACTIONS(1743),
    [sym_min_value] = ACTIONS(1743),
    [sym_exp_value] = ACTIONS(1743),
    [sym_log_value] = ACTIONS(1743),
    [sym_log10_value] = ACTIONS(1743),
    [sym_power_value] = ACTIONS(1743),
    [sym_sqrt_value] = ACTIONS(1743),
    [sym_raw_be] = ACTIONS(1743),
    [sym_raw_le] = ACTIONS(1743),
    [sym_internal_queue_size_set] = ACTIONS(1743),
    [sym_internal_queue_size_get] = ACTIONS(1743),
    [sym_internal_threshold_set] = ACTIONS(1743),
    [sym_internal_threshold_get] = ACTIONS(1743),
    [sym_rebuilt_event] = ACTIONS(1743),
    [sym_do_custom_event] = ACTIONS(1743),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1743),
    [sym_clkmgr_set_clk_source] = ACTIONS(1743),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1743),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1743),
    [sym_clkmgr_get_timezone] = ACTIONS(1743),
    [sym_clkmgr_set_timezone] = ACTIONS(1743),
    [sym_clkmgr_get_resync_period] = ACTIONS(1743),
    [sym_clkmgr_set_resync_period] = ACTIONS(1743),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1743),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1743),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1743),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1743),
    [sym_clkmgr_get_timeservers] = ACTIONS(1743),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1743),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1743),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1743),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1743),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1743),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1743),
    [sym_set_log_level] = ACTIONS(1743),
    [sym_get_log_level] = ACTIONS(1743),
    [sym_amx_log] = ACTIONS(1743),
    [sym_smtp_server_config_set] = ACTIONS(1743),
    [sym_smtp_server_config_get] = ACTIONS(1743),
    [sym_smtp_send] = ACTIONS(1743),
    [sym_validate_netlinx_account] = ACTIONS(1743),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1743),
    [sym_authenticate_certificate] = ACTIONS(1743),
    [sym_audit_netlinx_session_event] = ACTIONS(1743),
    [sym_audit_netlinx_generic_event] = ACTIONS(1743),
    [sym_get_available_flash_disk_space] = ACTIONS(1743),
    [sym_get_max_flash_disk_space] = ACTIONS(1743),
    [sym_data_create_feed] = ACTIONS(1743),
    [sym_data_delete_feed] = ACTIONS(1743),
    [sym_data_publish_feed] = ACTIONS(1743),
    [sym_data_get_published_feed] = ACTIONS(1743),
    [sym_data_add_record] = ACTIONS(1743),
    [sym_data_get_event_record] = ACTIONS(1743),
    [sym__wc_data_create_feed] = ACTIONS(1743),
    [sym__wc_data_add_record] = ACTIONS(1743),
    [sym__wc_data_get_event_record] = ACTIONS(1743),
    [sym_format] = ACTIONS(1743),
    [sym_astro_clock] = ACTIONS(1743),
    [sym_set_outdoor_temperature] = ACTIONS(1743),
    [sym_set_virtual_level_count] = ACTIONS(1743),
    [sym_set_virtual_channel_count] = ACTIONS(1743),
    [sym_set_virtual_port_count] = ACTIONS(1743),
    [sym__wc_encode_int] = ACTIONS(1743),
    [sym__wc_decode_int] = ACTIONS(1743),
    [sym__wc_int] = ACTIONS(1743),
    [sym__wc_to_ch_int] = ACTIONS(1743),
    [sym__ch_to_wc_int] = ACTIONS(1743),
    [sym__wc_find_string_int] = ACTIONS(1743),
    [sym__wc_left_string_int] = ACTIONS(1743),
    [sym__wc_lower_string_int] = ACTIONS(1743),
    [sym__wc_mid_string_int] = ACTIONS(1743),
    [sym__wc_remove_string_int] = ACTIONS(1743),
    [sym__wc_right_string_int] = ACTIONS(1743),
    [sym__wc_upper_string_int] = ACTIONS(1743),
    [sym__wc_compare_string_int] = ACTIONS(1743),
    [sym__wc_get_buffer_char_int] = ACTIONS(1743),
    [sym__wc_get_buffer_string_int] = ACTIONS(1743),
    [sym__wc_concat_string_int] = ACTIONS(1743),
    [sym__wc_file_open_int] = ACTIONS(1743),
    [sym__wc_file_close_int] = ACTIONS(1743),
    [sym__wc_file_read_int] = ACTIONS(1743),
    [sym__wc_file_read_line_int] = ACTIONS(1743),
    [sym__wc_file_write_int] = ACTIONS(1743),
    [sym__wc_file_write_line_int] = ACTIONS(1743),
    [aux_sym_preproc_include_token1] = ACTIONS(1743),
    [aux_sym_preproc_define_token1] = ACTIONS(1745),
    [aux_sym_preproc_warn_token1] = ACTIONS(1745),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1745),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1745),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1745),
    [anon_sym_LBRACK] = ACTIONS(1745),
    [anon_sym_LBRACE] = ACTIONS(1745),
    [anon_sym_LPAREN] = ACTIONS(1743),
    [anon_sym_BANG] = ACTIONS(1745),
    [anon_sym_TILDE] = ACTIONS(1745),
    [anon_sym_DASH] = ACTIONS(1743),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_DASH_DASH] = ACTIONS(1745),
    [anon_sym_PLUS_PLUS] = ACTIONS(1745),
    [anon_sym_DQUOTE] = ACTIONS(1745),
    [anon_sym_SQUOTE] = ACTIONS(1745),
    [sym_number_literal] = ACTIONS(1745),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(341)] = {
    [ts_builtin_sym_end] = ACTIONS(1877),
    [sym_identifier] = ACTIONS(1875),
    [sym_program_name_keyword] = ACTIONS(1875),
    [sym_module_name_keyword] = ACTIONS(1875),
    [sym_define_device_keyword] = ACTIONS(1875),
    [sym_define_combine_keyword] = ACTIONS(1875),
    [sym_define_constant_keyword] = ACTIONS(1875),
    [sym_define_type_keyword] = ACTIONS(1875),
    [sym_define_variable_keyword] = ACTIONS(1875),
    [sym_define_system_variable_keyword] = ACTIONS(1875),
    [sym_define_connect_level_keyword] = ACTIONS(1875),
    [sym_define_latching_keyword] = ACTIONS(1875),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1875),
    [sym_define_toggling_keyword] = ACTIONS(1875),
    [sym_define_call_keyword] = ACTIONS(1875),
    [sym_define_function_keyword] = ACTIONS(1875),
    [sym_define_library_function_keyword] = ACTIONS(1875),
    [sym_define_start_keyword] = ACTIONS(1875),
    [sym_define_event_keyword] = ACTIONS(1875),
    [sym_define_program_keyword] = ACTIONS(1875),
    [sym_define_module_keyword] = ACTIONS(1875),
    [sym_char_keyword] = ACTIONS(1875),
    [sym_widechar_keyword] = ACTIONS(1875),
    [sym_integer_keyword] = ACTIONS(1875),
    [sym_sinteger_keyword] = ACTIONS(1875),
    [sym_long_keyword] = ACTIONS(1875),
    [sym_slong_keyword] = ACTIONS(1875),
    [sym_float_keyword] = ACTIONS(1875),
    [sym_double_keyword] = ACTIONS(1875),
    [sym_variant_keyword] = ACTIONS(1875),
    [sym_variantarray_keyword] = ACTIONS(1875),
    [sym_local_var_keyword] = ACTIONS(1875),
    [sym_stack_var_keyword] = ACTIONS(1875),
    [sym_constant_keyword] = ACTIONS(1875),
    [sym_volatile_keyword] = ACTIONS(1875),
    [sym_non_volatile_keyword] = ACTIONS(1875),
    [sym_persistent_keyword] = ACTIONS(1875),
    [sym_dev_keyword] = ACTIONS(1875),
    [sym_devlev_keyword] = ACTIONS(1875),
    [sym_devchan_keyword] = ACTIONS(1875),
    [sym_if_keyword] = ACTIONS(1875),
    [sym_else_keyword] = ACTIONS(1875),
    [sym_switch_keyword] = ACTIONS(1875),
    [sym_case_keyword] = ACTIONS(1875),
    [sym_default_keyword] = ACTIONS(1875),
    [sym_while_keyword] = ACTIONS(1875),
    [sym_for_keyword] = ACTIONS(1875),
    [sym_break_keyword] = ACTIONS(1875),
    [sym_continue_keyword] = ACTIONS(1875),
    [sym_return_keyword] = ACTIONS(1875),
    [sym_select_keyword] = ACTIONS(1875),
    [sym_active_keyword] = ACTIONS(1875),
    [sym_struct_keyword] = ACTIONS(1875),
    [sym_structure_keyword] = ACTIONS(1875),
    [sym_button_event_keyword] = ACTIONS(1875),
    [sym_channel_event_keyword] = ACTIONS(1875),
    [sym_level_event_keyword] = ACTIONS(1875),
    [sym_data_event_keyword] = ACTIONS(1875),
    [sym_timeline_event_keyword] = ACTIONS(1875),
    [sym_custom_event_keyword] = ACTIONS(1875),
    [sym_push_keyword] = ACTIONS(1875),
    [sym_release_keyword] = ACTIONS(1875),
    [aux_sym_on_keyword_token1] = ACTIONS(1875),
    [aux_sym_off_keyword_token1] = ACTIONS(1875),
    [sym_send_level_keyword] = ACTIONS(1875),
    [sym_send_string_keyword] = ACTIONS(1875),
    [sym_send_command_keyword] = ACTIONS(1875),
    [sym_clear_buffer_keyword] = ACTIONS(1875),
    [sym_create_buffer_keyword] = ACTIONS(1875),
    [sym_create_multi_buffer_keyword] = ACTIONS(1875),
    [sym_call_keyword] = ACTIONS(1875),
    [sym_system_call_keyword] = ACTIONS(1875),
    [sym_devchan_to_keyword] = ACTIONS(1875),
    [sym_devchan_min_to_keyword] = ACTIONS(1875),
    [sym_devchan_total_off_keyword] = ACTIONS(1875),
    [sym_devchan_pulse_keyword] = ACTIONS(1875),
    [sym_wait_keyword] = ACTIONS(1875),
    [sym_cancel_wait_keyword] = ACTIONS(1875),
    [sym_wait_until_keyword] = ACTIONS(1875),
    [sym_cancel_wait_until_keyword] = ACTIONS(1875),
    [sym_cancel_all_wait_keyword] = ACTIONS(1875),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1875),
    [sym_not] = ACTIONS(1875),
    [sym_bnot] = ACTIONS(1875),
    [sym___file__] = ACTIONS(1875),
    [sym___date__] = ACTIONS(1875),
    [sym___ldate__] = ACTIONS(1875),
    [sym___time__] = ACTIONS(1875),
    [sym___line__] = ACTIONS(1875),
    [sym___name__] = ACTIONS(1875),
    [sym___netlinx__] = ACTIONS(1875),
    [sym_netlinx_axi_version] = ACTIONS(1875),
    [sym_url_flg_tcp] = ACTIONS(1875),
    [sym_url_flg_accinfopresent] = ACTIONS(1875),
    [sym_url_flg_temp] = ACTIONS(1875),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1875),
    [sym_url_flg_stat_mask] = ACTIONS(1875),
    [sym_url_flg_stat_lookup] = ACTIONS(1875),
    [sym_url_flg_stat_connecting] = ACTIONS(1875),
    [sym_url_flg_stat_waiting] = ACTIONS(1875),
    [sym_url_flg_stat_connected] = ACTIONS(1875),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1875),
    [sym_first_virtual_device] = ACTIONS(1875),
    [sym_first_local_port] = ACTIONS(1875),
    [sym_dynamic_virtual_device] = ACTIONS(1875),
    [sym_timeline_once] = ACTIONS(1875),
    [sym_timeline_repeat] = ACTIONS(1875),
    [sym_timeline_absolute] = ACTIONS(1875),
    [sym_timeline_relative] = ACTIONS(1875),
    [sym_do_push_timed_infinite] = ACTIONS(1875),
    [sym_source_type_no_address] = ACTIONS(1875),
    [sym_source_type_neuron_id] = ACTIONS(1875),
    [sym_source_type_ip_address] = ACTIONS(1875),
    [sym_source_type_axlink] = ACTIONS(1875),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1875),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1875),
    [sym_source_type_ip_socket_address] = ACTIONS(1875),
    [sym_source_type_rs232] = ACTIONS(1875),
    [sym_source_type_internal] = ACTIONS(1875),
    [sym_source_type_mac_address] = ACTIONS(1875),
    [sym_source_type_ipv4_port] = ACTIONS(1875),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1875),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1875),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1875),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1875),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1875),
    [sym_true] = ACTIONS(1875),
    [sym_false] = ACTIONS(1875),
    [sym_file_read_only] = ACTIONS(1875),
    [sym_file_rw_new] = ACTIONS(1875),
    [sym_file_rw_append] = ACTIONS(1875),
    [sym_ip_tcp] = ACTIONS(1875),
    [sym_ip_udp] = ACTIONS(1875),
    [sym_ip_udp_2way] = ACTIONS(1875),
    [sym_xml_encode_types] = ACTIONS(1875),
    [sym_xml_encode_char_as_list] = ACTIONS(1875),
    [sym_xml_encode_le] = ACTIONS(1875),
    [sym_xml_decode_types] = ACTIONS(1875),
    [sym_xml_decode_no_preserve] = ACTIONS(1875),
    [sym_duet_dev_type_display_device] = ACTIONS(1875),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1875),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1875),
    [sym_duet_dev_type_io_device] = ACTIONS(1875),
    [sym_duet_dev_type_relay_device] = ACTIONS(1875),
    [sym_duet_dev_type_ups] = ACTIONS(1875),
    [sym_duet_dev_type_amplifier] = ACTIONS(1875),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1875),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1875),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1875),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1875),
    [sym_duet_dev_type_camera] = ACTIONS(1875),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1875),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1875),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1875),
    [sym_duet_dev_type_dss] = ACTIONS(1875),
    [sym_duet_dev_type_dvr] = ACTIONS(1875),
    [sym_duet_dev_type_disc_device] = ACTIONS(1875),
    [sym_duet_dev_type_document_camera] = ACTIONS(1875),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1875),
    [sym_duet_dev_type_hvac] = ACTIONS(1875),
    [sym_duet_dev_type_keypad] = ACTIONS(1875),
    [sym_duet_dev_type_light] = ACTIONS(1875),
    [sym_duet_dev_type_monitor] = ACTIONS(1875),
    [sym_duet_dev_type_motor] = ACTIONS(1875),
    [sym_duet_dev_type_multi_window] = ACTIONS(1875),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1875),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1875),
    [sym_duet_dev_type_receiver] = ACTIONS(1875),
    [sym_duet_dev_type_security_system] = ACTIONS(1875),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1875),
    [sym_duet_dev_type_settop_box] = ACTIONS(1875),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1875),
    [sym_duet_dev_type_switcher] = ACTIONS(1875),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1875),
    [sym_duet_dev_type_tv] = ACTIONS(1875),
    [sym_duet_dev_type_utility] = ACTIONS(1875),
    [sym_duet_dev_type_vcr] = ACTIONS(1875),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1875),
    [sym_duet_dev_type_video_processor] = ACTIONS(1875),
    [sym_duet_dev_type_video_projector] = ACTIONS(1875),
    [sym_duet_dev_type_video_wall] = ACTIONS(1875),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1875),
    [sym_duet_dev_type_weather] = ACTIONS(1875),
    [sym_duet_dev_not_polled] = ACTIONS(1875),
    [sym_duet_dev_polled] = ACTIONS(1875),
    [sym_clkmgr_mode_network] = ACTIONS(1875),
    [sym_clkmgr_mode_standalone] = ACTIONS(1875),
    [sym_normal_standby] = ACTIONS(1875),
    [sym_normal_wake] = ACTIONS(1875),
    [sym_ip_multicast_ttl_option] = ACTIONS(1875),
    [sym_ip_ttl_subnet] = ACTIONS(1875),
    [sym_ip_ttl_site] = ACTIONS(1875),
    [sym_ip_ttl_region] = ACTIONS(1875),
    [sym_ip_ttl_continent] = ACTIONS(1875),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1875),
    [sym_ip_nodelay_on] = ACTIONS(1875),
    [sym_ip_nodelay_off] = ACTIONS(1875),
    [sym_amx_error] = ACTIONS(1875),
    [sym_amx_warning] = ACTIONS(1875),
    [sym_amx_info] = ACTIONS(1875),
    [sym_amx_debug] = ACTIONS(1875),
    [sym_netlinx_login_success] = ACTIONS(1875),
    [sym_netlinx_login_fail] = ACTIONS(1875),
    [sym_netlinx_logout] = ACTIONS(1875),
    [sym_null_str] = ACTIONS(1875),
    [sym_smtp_address] = ACTIONS(1875),
    [sym_smtp_port_number] = ACTIONS(1875),
    [sym_smtp_username] = ACTIONS(1875),
    [sym_smtp_password] = ACTIONS(1875),
    [sym_smtp_require_tls] = ACTIONS(1875),
    [sym_smtp_from] = ACTIONS(1875),
    [sym_smtp_tls_true] = ACTIONS(1875),
    [sym_smtp_tls_false] = ACTIONS(1875),
    [sym_listview_on_row_select_event] = ACTIONS(1875),
    [sym_data_structure_unknown] = ACTIONS(1875),
    [sym_data_structure_datafeed] = ACTIONS(1875),
    [sym_data_structure_datarecordset] = ACTIONS(1875),
    [sym_data_structure_datarecord] = ACTIONS(1875),
    [sym_data_structure_datafield] = ACTIONS(1875),
    [sym_data_max_content_field_count] = ACTIONS(1875),
    [sym_data_max_metadata_field_count] = ACTIONS(1875),
    [sym_data_max_id_length] = ACTIONS(1875),
    [sym_data_max_name_length] = ACTIONS(1875),
    [sym_data_max_description_length] = ACTIONS(1875),
    [sym_data_max_type_length] = ACTIONS(1875),
    [sym_data_max_format_length] = ACTIONS(1875),
    [sym_data_max_source_length] = ACTIONS(1875),
    [sym_data_max_value_length] = ACTIONS(1875),
    [sym_data_max_label_length] = ACTIONS(1875),
    [sym_data_max_url_length] = ACTIONS(1875),
    [sym_max_last_login_info_length] = ACTIONS(1875),
    [sym_data_type_unknown] = ACTIONS(1875),
    [sym_data_type_string] = ACTIONS(1875),
    [sym_data_type_datetime] = ACTIONS(1875),
    [sym_data_type_date] = ACTIONS(1875),
    [sym_data_type_time] = ACTIONS(1875),
    [sym_data_type_image] = ACTIONS(1875),
    [sym_data_format_url] = ACTIONS(1875),
    [sym_data_format_phone] = ACTIONS(1875),
    [sym_data_format_email] = ACTIONS(1875),
    [sym_data_format_iso8601] = ACTIONS(1875),
    [sym_auditlog_priv] = ACTIONS(1875),
    [sym_configuration_priv] = ACTIONS(1875),
    [sym_deviceconfig_priv] = ACTIONS(1875),
    [sym_ftp_priv] = ACTIONS(1875),
    [sym_http_priv] = ACTIONS(1875),
    [sym_networkconfig_priv] = ACTIONS(1875),
    [sym_programport_priv] = ACTIONS(1875),
    [sym_remoteui_priv] = ACTIONS(1875),
    [sym_securitycontrol_priv] = ACTIONS(1875),
    [sym_softwaremanagement_priv] = ACTIONS(1875),
    [sym_terminal_priv] = ACTIONS(1875),
    [sym_tpadmin_priv] = ACTIONS(1875),
    [sym_user1_priv] = ACTIONS(1875),
    [sym_user2_priv] = ACTIONS(1875),
    [sym_user3_priv] = ACTIONS(1875),
    [sym_user4_priv] = ACTIONS(1875),
    [sym_usermanagement_priv] = ACTIONS(1875),
    [sym_valid_account] = ACTIONS(1875),
    [sym_err_name_invalid] = ACTIONS(1875),
    [sym_err_password_invalid] = ACTIONS(1875),
    [sym_err_invalid_account] = ACTIONS(1875),
    [sym_err_no_authorization] = ACTIONS(1875),
    [sym_err_invalid_parameter] = ACTIONS(1875),
    [sym_err_user_locked_out] = ACTIONS(1875),
    [sym_err_user_account_expired] = ACTIONS(1875),
    [sym_err_general_error] = ACTIONS(1875),
    [sym_err_authentication_error] = ACTIONS(1875),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1875),
    [sym_tls_validate_certificate] = ACTIONS(1875),
    [sym_tdata] = ACTIONS(1875),
    [sym_tchannel] = ACTIONS(1875),
    [sym_tlevel] = ACTIONS(1875),
    [sym_tbutton] = ACTIONS(1875),
    [sym_ttimeline] = ACTIONS(1875),
    [sym_tcustom] = ACTIONS(1875),
    [sym_url_struct] = ACTIONS(1875),
    [sym_dns_struct] = ACTIONS(1875),
    [sym_ip_address_struct] = ACTIONS(1875),
    [sym_dev_info_struct] = ACTIONS(1875),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1875),
    [sym_data_feed] = ACTIONS(1875),
    [sym_data_field] = ACTIONS(1875),
    [sym_data_record] = ACTIONS(1875),
    [sym_wc_data_feed] = ACTIONS(1875),
    [sym_wc_data_field] = ACTIONS(1875),
    [sym_wc_data_record] = ACTIONS(1875),
    [sym_last_login_info] = ACTIONS(1875),
    [sym____reserved___] = ACTIONS(1875),
    [sym_date] = ACTIONS(1875),
    [sym_day] = ACTIONS(1875),
    [sym_time] = ACTIONS(1875),
    [sym_ldate] = ACTIONS(1875),
    [sym_push_device] = ACTIONS(1875),
    [sym_push_channel] = ACTIONS(1875),
    [sym_push_devchan] = ACTIONS(1875),
    [sym_release_device] = ACTIONS(1875),
    [sym_release_channel] = ACTIONS(1875),
    [sym_release_devchan] = ACTIONS(1875),
    [sym_master_slot] = ACTIONS(1875),
    [sym_get_pulse_time] = ACTIONS(1875),
    [sym_get_timer] = ACTIONS(1875),
    [sym_master_sn] = ACTIONS(1875),
    [sym_system_number] = ACTIONS(1875),
    [sym_button] = ACTIONS(1875),
    [sym_dv_channel] = ACTIONS(1875),
    [sym_data] = ACTIONS(1875),
    [sym_level] = ACTIONS(1875),
    [sym_timeline] = ACTIONS(1875),
    [sym_channel] = ACTIONS(1875),
    [sym_custom] = ACTIONS(1875),
    [sym_atoi] = ACTIONS(1875),
    [sym_atol] = ACTIONS(1875),
    [sym_atof] = ACTIONS(1875),
    [sym_itoa] = ACTIONS(1875),
    [sym_itohex] = ACTIONS(1875),
    [sym_hextoi] = ACTIONS(1875),
    [sym_ftoa] = ACTIONS(1875),
    [sym_find_string] = ACTIONS(1875),
    [sym_left_string] = ACTIONS(1875),
    [sym_right_string] = ACTIONS(1875),
    [sym_mid_string] = ACTIONS(1875),
    [sym_remove_string] = ACTIONS(1875),
    [sym_get_buffer_string] = ACTIONS(1875),
    [sym_get_buffer_char] = ACTIONS(1875),
    [sym_get_multi_buffer_string] = ACTIONS(1875),
    [sym_length_string] = ACTIONS(1875),
    [sym_max_length_string] = ACTIONS(1875),
    [sym_set_length_string] = ACTIONS(1875),
    [sym_compare_string] = ACTIONS(1875),
    [sym_upper_string] = ACTIONS(1875),
    [sym_lower_string] = ACTIONS(1875),
    [sym_get_last] = ACTIONS(1875),
    [sym_date_to_day] = ACTIONS(1875),
    [sym_date_to_month] = ACTIONS(1875),
    [sym_date_to_year] = ACTIONS(1875),
    [sym_time_to_hour] = ACTIONS(1875),
    [sym_time_to_minute] = ACTIONS(1875),
    [sym_time_to_second] = ACTIONS(1875),
    [sym_day_of_week] = ACTIONS(1875),
    [sym_set_pulse_time] = ACTIONS(1875),
    [sym_set_timer] = ACTIONS(1875),
    [sym_file_open] = ACTIONS(1875),
    [sym_file_close] = ACTIONS(1875),
    [sym_file_read] = ACTIONS(1875),
    [sym_file_read_line] = ACTIONS(1875),
    [sym_file_write] = ACTIONS(1875),
    [sym_file_write_line] = ACTIONS(1875),
    [sym_file_copy] = ACTIONS(1875),
    [sym_file_delete] = ACTIONS(1875),
    [sym_file_rename] = ACTIONS(1875),
    [sym_file_dir] = ACTIONS(1875),
    [sym_file_getdir] = ACTIONS(1875),
    [sym_file_setdir] = ACTIONS(1875),
    [sym_file_create_dir] = ACTIONS(1875),
    [sym_file_remove_dir] = ACTIONS(1875),
    [sym_file_seek] = ACTIONS(1875),
    [sym_length_array] = ACTIONS(1875),
    [sym_max_length_array] = ACTIONS(1875),
    [sym_set_length_array] = ACTIONS(1875),
    [sym_ip_client_open] = ACTIONS(1875),
    [sym_ip_client_close] = ACTIONS(1875),
    [sym_ip_server_open] = ACTIONS(1875),
    [sym_ip_server_close] = ACTIONS(1875),
    [sym_ip_bound_client_open] = ACTIONS(1875),
    [sym_ip_mc_server_open] = ACTIONS(1875),
    [sym_ip_set_option] = ACTIONS(1875),
    [sym_ssh_client_open] = ACTIONS(1875),
    [sym_ssh_client_close] = ACTIONS(1875),
    [sym_tls_client_open] = ACTIONS(1875),
    [sym_tls_client_close] = ACTIONS(1875),
    [sym_do_push] = ACTIONS(1875),
    [sym_do_push_timed] = ACTIONS(1875),
    [sym_do_release] = ACTIONS(1875),
    [sym_device_id] = ACTIONS(1875),
    [sym_device_info] = ACTIONS(1875),
    [sym_device_id_string] = ACTIONS(1875),
    [sym_device_standby] = ACTIONS(1875),
    [sym_device_wake] = ACTIONS(1875),
    [sym_timeline_create] = ACTIONS(1875),
    [sym_timeline_kill] = ACTIONS(1875),
    [sym_timeline_reload] = ACTIONS(1875),
    [sym_timeline_pause] = ACTIONS(1875),
    [sym_timeline_restart] = ACTIONS(1875),
    [sym_timeline_set] = ACTIONS(1875),
    [sym_timeline_get] = ACTIONS(1875),
    [sym_timeline_active] = ACTIONS(1875),
    [sym_timeline_dynamic_id] = ACTIONS(1875),
    [sym_variable_to_string] = ACTIONS(1875),
    [sym_string_to_variable] = ACTIONS(1875),
    [sym_length_variable_to_string] = ACTIONS(1875),
    [sym_type_cast] = ACTIONS(1875),
    [sym_variable_to_xml] = ACTIONS(1875),
    [sym_xml_to_variable] = ACTIONS(1875),
    [sym_length_variable_to_xml] = ACTIONS(1875),
    [sym_redirect_string] = ACTIONS(1875),
    [sym_random_number] = ACTIONS(1875),
    [sym_get_unique_id] = ACTIONS(1875),
    [sym_get_system_number] = ACTIONS(1875),
    [sym_set_system_number] = ACTIONS(1875),
    [sym_get_serial_number] = ACTIONS(1875),
    [sym_get_master_build] = ACTIONS(1875),
    [sym_set_validation_code] = ACTIONS(1875),
    [sym_reboot] = ACTIONS(1875),
    [sym_load_duet_module] = ACTIONS(1875),
    [sym_unload_duet_module] = ACTIONS(1875),
    [sym_duet_mem_size_set] = ACTIONS(1875),
    [sym_duet_mem_size_get] = ACTIONS(1875),
    [sym_dynamic_application_device] = ACTIONS(1875),
    [sym_dynamic_polled_port] = ACTIONS(1875),
    [sym_static_port_binding] = ACTIONS(1875),
    [sym_static_ip_binding] = ACTIONS(1875),
    [sym_get_dns_list] = ACTIONS(1875),
    [sym_set_dns_list] = ACTIONS(1875),
    [sym_get_ip_address] = ACTIONS(1875),
    [sym_set_ip_address] = ACTIONS(1875),
    [sym_get_url_list] = ACTIONS(1875),
    [sym_add_url_entry] = ACTIONS(1875),
    [sym_delete_url_entry] = ACTIONS(1875),
    [sym_abs_value] = ACTIONS(1875),
    [sym_max_value] = ACTIONS(1875),
    [sym_min_value] = ACTIONS(1875),
    [sym_exp_value] = ACTIONS(1875),
    [sym_log_value] = ACTIONS(1875),
    [sym_log10_value] = ACTIONS(1875),
    [sym_power_value] = ACTIONS(1875),
    [sym_sqrt_value] = ACTIONS(1875),
    [sym_raw_be] = ACTIONS(1875),
    [sym_raw_le] = ACTIONS(1875),
    [sym_internal_queue_size_set] = ACTIONS(1875),
    [sym_internal_queue_size_get] = ACTIONS(1875),
    [sym_internal_threshold_set] = ACTIONS(1875),
    [sym_internal_threshold_get] = ACTIONS(1875),
    [sym_rebuilt_event] = ACTIONS(1875),
    [sym_do_custom_event] = ACTIONS(1875),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1875),
    [sym_clkmgr_set_clk_source] = ACTIONS(1875),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1875),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1875),
    [sym_clkmgr_get_timezone] = ACTIONS(1875),
    [sym_clkmgr_set_timezone] = ACTIONS(1875),
    [sym_clkmgr_get_resync_period] = ACTIONS(1875),
    [sym_clkmgr_set_resync_period] = ACTIONS(1875),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1875),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1875),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1875),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1875),
    [sym_clkmgr_get_timeservers] = ACTIONS(1875),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1875),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1875),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1875),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1875),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1875),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1875),
    [sym_set_log_level] = ACTIONS(1875),
    [sym_get_log_level] = ACTIONS(1875),
    [sym_amx_log] = ACTIONS(1875),
    [sym_smtp_server_config_set] = ACTIONS(1875),
    [sym_smtp_server_config_get] = ACTIONS(1875),
    [sym_smtp_send] = ACTIONS(1875),
    [sym_validate_netlinx_account] = ACTIONS(1875),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1875),
    [sym_authenticate_certificate] = ACTIONS(1875),
    [sym_audit_netlinx_session_event] = ACTIONS(1875),
    [sym_audit_netlinx_generic_event] = ACTIONS(1875),
    [sym_get_available_flash_disk_space] = ACTIONS(1875),
    [sym_get_max_flash_disk_space] = ACTIONS(1875),
    [sym_data_create_feed] = ACTIONS(1875),
    [sym_data_delete_feed] = ACTIONS(1875),
    [sym_data_publish_feed] = ACTIONS(1875),
    [sym_data_get_published_feed] = ACTIONS(1875),
    [sym_data_add_record] = ACTIONS(1875),
    [sym_data_get_event_record] = ACTIONS(1875),
    [sym__wc_data_create_feed] = ACTIONS(1875),
    [sym__wc_data_add_record] = ACTIONS(1875),
    [sym__wc_data_get_event_record] = ACTIONS(1875),
    [sym_format] = ACTIONS(1875),
    [sym_astro_clock] = ACTIONS(1875),
    [sym_set_outdoor_temperature] = ACTIONS(1875),
    [sym_set_virtual_level_count] = ACTIONS(1875),
    [sym_set_virtual_channel_count] = ACTIONS(1875),
    [sym_set_virtual_port_count] = ACTIONS(1875),
    [sym__wc_encode_int] = ACTIONS(1875),
    [sym__wc_decode_int] = ACTIONS(1875),
    [sym__wc_int] = ACTIONS(1875),
    [sym__wc_to_ch_int] = ACTIONS(1875),
    [sym__ch_to_wc_int] = ACTIONS(1875),
    [sym__wc_find_string_int] = ACTIONS(1875),
    [sym__wc_left_string_int] = ACTIONS(1875),
    [sym__wc_lower_string_int] = ACTIONS(1875),
    [sym__wc_mid_string_int] = ACTIONS(1875),
    [sym__wc_remove_string_int] = ACTIONS(1875),
    [sym__wc_right_string_int] = ACTIONS(1875),
    [sym__wc_upper_string_int] = ACTIONS(1875),
    [sym__wc_compare_string_int] = ACTIONS(1875),
    [sym__wc_get_buffer_char_int] = ACTIONS(1875),
    [sym__wc_get_buffer_string_int] = ACTIONS(1875),
    [sym__wc_concat_string_int] = ACTIONS(1875),
    [sym__wc_file_open_int] = ACTIONS(1875),
    [sym__wc_file_close_int] = ACTIONS(1875),
    [sym__wc_file_read_int] = ACTIONS(1875),
    [sym__wc_file_read_line_int] = ACTIONS(1875),
    [sym__wc_file_write_int] = ACTIONS(1875),
    [sym__wc_file_write_line_int] = ACTIONS(1875),
    [aux_sym_preproc_include_token1] = ACTIONS(1875),
    [aux_sym_preproc_define_token1] = ACTIONS(1877),
    [aux_sym_preproc_warn_token1] = ACTIONS(1877),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1877),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1877),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1877),
    [anon_sym_LBRACK] = ACTIONS(1877),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_LPAREN] = ACTIONS(1875),
    [anon_sym_BANG] = ACTIONS(1877),
    [anon_sym_TILDE] = ACTIONS(1877),
    [anon_sym_DASH] = ACTIONS(1875),
    [anon_sym_PLUS] = ACTIONS(1875),
    [anon_sym_DASH_DASH] = ACTIONS(1877),
    [anon_sym_PLUS_PLUS] = ACTIONS(1877),
    [anon_sym_DQUOTE] = ACTIONS(1877),
    [anon_sym_SQUOTE] = ACTIONS(1877),
    [sym_number_literal] = ACTIONS(1877),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(342)] = {
    [ts_builtin_sym_end] = ACTIONS(1881),
    [sym_identifier] = ACTIONS(1879),
    [sym_program_name_keyword] = ACTIONS(1879),
    [sym_module_name_keyword] = ACTIONS(1879),
    [sym_define_device_keyword] = ACTIONS(1879),
    [sym_define_combine_keyword] = ACTIONS(1879),
    [sym_define_constant_keyword] = ACTIONS(1879),
    [sym_define_type_keyword] = ACTIONS(1879),
    [sym_define_variable_keyword] = ACTIONS(1879),
    [sym_define_system_variable_keyword] = ACTIONS(1879),
    [sym_define_connect_level_keyword] = ACTIONS(1879),
    [sym_define_latching_keyword] = ACTIONS(1879),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1879),
    [sym_define_toggling_keyword] = ACTIONS(1879),
    [sym_define_call_keyword] = ACTIONS(1879),
    [sym_define_function_keyword] = ACTIONS(1879),
    [sym_define_library_function_keyword] = ACTIONS(1879),
    [sym_define_start_keyword] = ACTIONS(1879),
    [sym_define_event_keyword] = ACTIONS(1879),
    [sym_define_program_keyword] = ACTIONS(1879),
    [sym_define_module_keyword] = ACTIONS(1879),
    [sym_char_keyword] = ACTIONS(1879),
    [sym_widechar_keyword] = ACTIONS(1879),
    [sym_integer_keyword] = ACTIONS(1879),
    [sym_sinteger_keyword] = ACTIONS(1879),
    [sym_long_keyword] = ACTIONS(1879),
    [sym_slong_keyword] = ACTIONS(1879),
    [sym_float_keyword] = ACTIONS(1879),
    [sym_double_keyword] = ACTIONS(1879),
    [sym_variant_keyword] = ACTIONS(1879),
    [sym_variantarray_keyword] = ACTIONS(1879),
    [sym_local_var_keyword] = ACTIONS(1879),
    [sym_stack_var_keyword] = ACTIONS(1879),
    [sym_constant_keyword] = ACTIONS(1879),
    [sym_volatile_keyword] = ACTIONS(1879),
    [sym_non_volatile_keyword] = ACTIONS(1879),
    [sym_persistent_keyword] = ACTIONS(1879),
    [sym_dev_keyword] = ACTIONS(1879),
    [sym_devlev_keyword] = ACTIONS(1879),
    [sym_devchan_keyword] = ACTIONS(1879),
    [sym_if_keyword] = ACTIONS(1879),
    [sym_else_keyword] = ACTIONS(1879),
    [sym_switch_keyword] = ACTIONS(1879),
    [sym_case_keyword] = ACTIONS(1879),
    [sym_default_keyword] = ACTIONS(1879),
    [sym_while_keyword] = ACTIONS(1879),
    [sym_for_keyword] = ACTIONS(1879),
    [sym_break_keyword] = ACTIONS(1879),
    [sym_continue_keyword] = ACTIONS(1879),
    [sym_return_keyword] = ACTIONS(1879),
    [sym_select_keyword] = ACTIONS(1879),
    [sym_active_keyword] = ACTIONS(1879),
    [sym_struct_keyword] = ACTIONS(1879),
    [sym_structure_keyword] = ACTIONS(1879),
    [sym_button_event_keyword] = ACTIONS(1879),
    [sym_channel_event_keyword] = ACTIONS(1879),
    [sym_level_event_keyword] = ACTIONS(1879),
    [sym_data_event_keyword] = ACTIONS(1879),
    [sym_timeline_event_keyword] = ACTIONS(1879),
    [sym_custom_event_keyword] = ACTIONS(1879),
    [sym_push_keyword] = ACTIONS(1879),
    [sym_release_keyword] = ACTIONS(1879),
    [aux_sym_on_keyword_token1] = ACTIONS(1879),
    [aux_sym_off_keyword_token1] = ACTIONS(1879),
    [sym_send_level_keyword] = ACTIONS(1879),
    [sym_send_string_keyword] = ACTIONS(1879),
    [sym_send_command_keyword] = ACTIONS(1879),
    [sym_clear_buffer_keyword] = ACTIONS(1879),
    [sym_create_buffer_keyword] = ACTIONS(1879),
    [sym_create_multi_buffer_keyword] = ACTIONS(1879),
    [sym_call_keyword] = ACTIONS(1879),
    [sym_system_call_keyword] = ACTIONS(1879),
    [sym_devchan_to_keyword] = ACTIONS(1879),
    [sym_devchan_min_to_keyword] = ACTIONS(1879),
    [sym_devchan_total_off_keyword] = ACTIONS(1879),
    [sym_devchan_pulse_keyword] = ACTIONS(1879),
    [sym_wait_keyword] = ACTIONS(1879),
    [sym_cancel_wait_keyword] = ACTIONS(1879),
    [sym_wait_until_keyword] = ACTIONS(1879),
    [sym_cancel_wait_until_keyword] = ACTIONS(1879),
    [sym_cancel_all_wait_keyword] = ACTIONS(1879),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1879),
    [sym_not] = ACTIONS(1879),
    [sym_bnot] = ACTIONS(1879),
    [sym___file__] = ACTIONS(1879),
    [sym___date__] = ACTIONS(1879),
    [sym___ldate__] = ACTIONS(1879),
    [sym___time__] = ACTIONS(1879),
    [sym___line__] = ACTIONS(1879),
    [sym___name__] = ACTIONS(1879),
    [sym___netlinx__] = ACTIONS(1879),
    [sym_netlinx_axi_version] = ACTIONS(1879),
    [sym_url_flg_tcp] = ACTIONS(1879),
    [sym_url_flg_accinfopresent] = ACTIONS(1879),
    [sym_url_flg_temp] = ACTIONS(1879),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1879),
    [sym_url_flg_stat_mask] = ACTIONS(1879),
    [sym_url_flg_stat_lookup] = ACTIONS(1879),
    [sym_url_flg_stat_connecting] = ACTIONS(1879),
    [sym_url_flg_stat_waiting] = ACTIONS(1879),
    [sym_url_flg_stat_connected] = ACTIONS(1879),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1879),
    [sym_first_virtual_device] = ACTIONS(1879),
    [sym_first_local_port] = ACTIONS(1879),
    [sym_dynamic_virtual_device] = ACTIONS(1879),
    [sym_timeline_once] = ACTIONS(1879),
    [sym_timeline_repeat] = ACTIONS(1879),
    [sym_timeline_absolute] = ACTIONS(1879),
    [sym_timeline_relative] = ACTIONS(1879),
    [sym_do_push_timed_infinite] = ACTIONS(1879),
    [sym_source_type_no_address] = ACTIONS(1879),
    [sym_source_type_neuron_id] = ACTIONS(1879),
    [sym_source_type_ip_address] = ACTIONS(1879),
    [sym_source_type_axlink] = ACTIONS(1879),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1879),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1879),
    [sym_source_type_ip_socket_address] = ACTIONS(1879),
    [sym_source_type_rs232] = ACTIONS(1879),
    [sym_source_type_internal] = ACTIONS(1879),
    [sym_source_type_mac_address] = ACTIONS(1879),
    [sym_source_type_ipv4_port] = ACTIONS(1879),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1879),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1879),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1879),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1879),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1879),
    [sym_true] = ACTIONS(1879),
    [sym_false] = ACTIONS(1879),
    [sym_file_read_only] = ACTIONS(1879),
    [sym_file_rw_new] = ACTIONS(1879),
    [sym_file_rw_append] = ACTIONS(1879),
    [sym_ip_tcp] = ACTIONS(1879),
    [sym_ip_udp] = ACTIONS(1879),
    [sym_ip_udp_2way] = ACTIONS(1879),
    [sym_xml_encode_types] = ACTIONS(1879),
    [sym_xml_encode_char_as_list] = ACTIONS(1879),
    [sym_xml_encode_le] = ACTIONS(1879),
    [sym_xml_decode_types] = ACTIONS(1879),
    [sym_xml_decode_no_preserve] = ACTIONS(1879),
    [sym_duet_dev_type_display_device] = ACTIONS(1879),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1879),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1879),
    [sym_duet_dev_type_io_device] = ACTIONS(1879),
    [sym_duet_dev_type_relay_device] = ACTIONS(1879),
    [sym_duet_dev_type_ups] = ACTIONS(1879),
    [sym_duet_dev_type_amplifier] = ACTIONS(1879),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1879),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1879),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1879),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1879),
    [sym_duet_dev_type_camera] = ACTIONS(1879),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1879),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1879),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1879),
    [sym_duet_dev_type_dss] = ACTIONS(1879),
    [sym_duet_dev_type_dvr] = ACTIONS(1879),
    [sym_duet_dev_type_disc_device] = ACTIONS(1879),
    [sym_duet_dev_type_document_camera] = ACTIONS(1879),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1879),
    [sym_duet_dev_type_hvac] = ACTIONS(1879),
    [sym_duet_dev_type_keypad] = ACTIONS(1879),
    [sym_duet_dev_type_light] = ACTIONS(1879),
    [sym_duet_dev_type_monitor] = ACTIONS(1879),
    [sym_duet_dev_type_motor] = ACTIONS(1879),
    [sym_duet_dev_type_multi_window] = ACTIONS(1879),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1879),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1879),
    [sym_duet_dev_type_receiver] = ACTIONS(1879),
    [sym_duet_dev_type_security_system] = ACTIONS(1879),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1879),
    [sym_duet_dev_type_settop_box] = ACTIONS(1879),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1879),
    [sym_duet_dev_type_switcher] = ACTIONS(1879),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1879),
    [sym_duet_dev_type_tv] = ACTIONS(1879),
    [sym_duet_dev_type_utility] = ACTIONS(1879),
    [sym_duet_dev_type_vcr] = ACTIONS(1879),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1879),
    [sym_duet_dev_type_video_processor] = ACTIONS(1879),
    [sym_duet_dev_type_video_projector] = ACTIONS(1879),
    [sym_duet_dev_type_video_wall] = ACTIONS(1879),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1879),
    [sym_duet_dev_type_weather] = ACTIONS(1879),
    [sym_duet_dev_not_polled] = ACTIONS(1879),
    [sym_duet_dev_polled] = ACTIONS(1879),
    [sym_clkmgr_mode_network] = ACTIONS(1879),
    [sym_clkmgr_mode_standalone] = ACTIONS(1879),
    [sym_normal_standby] = ACTIONS(1879),
    [sym_normal_wake] = ACTIONS(1879),
    [sym_ip_multicast_ttl_option] = ACTIONS(1879),
    [sym_ip_ttl_subnet] = ACTIONS(1879),
    [sym_ip_ttl_site] = ACTIONS(1879),
    [sym_ip_ttl_region] = ACTIONS(1879),
    [sym_ip_ttl_continent] = ACTIONS(1879),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1879),
    [sym_ip_nodelay_on] = ACTIONS(1879),
    [sym_ip_nodelay_off] = ACTIONS(1879),
    [sym_amx_error] = ACTIONS(1879),
    [sym_amx_warning] = ACTIONS(1879),
    [sym_amx_info] = ACTIONS(1879),
    [sym_amx_debug] = ACTIONS(1879),
    [sym_netlinx_login_success] = ACTIONS(1879),
    [sym_netlinx_login_fail] = ACTIONS(1879),
    [sym_netlinx_logout] = ACTIONS(1879),
    [sym_null_str] = ACTIONS(1879),
    [sym_smtp_address] = ACTIONS(1879),
    [sym_smtp_port_number] = ACTIONS(1879),
    [sym_smtp_username] = ACTIONS(1879),
    [sym_smtp_password] = ACTIONS(1879),
    [sym_smtp_require_tls] = ACTIONS(1879),
    [sym_smtp_from] = ACTIONS(1879),
    [sym_smtp_tls_true] = ACTIONS(1879),
    [sym_smtp_tls_false] = ACTIONS(1879),
    [sym_listview_on_row_select_event] = ACTIONS(1879),
    [sym_data_structure_unknown] = ACTIONS(1879),
    [sym_data_structure_datafeed] = ACTIONS(1879),
    [sym_data_structure_datarecordset] = ACTIONS(1879),
    [sym_data_structure_datarecord] = ACTIONS(1879),
    [sym_data_structure_datafield] = ACTIONS(1879),
    [sym_data_max_content_field_count] = ACTIONS(1879),
    [sym_data_max_metadata_field_count] = ACTIONS(1879),
    [sym_data_max_id_length] = ACTIONS(1879),
    [sym_data_max_name_length] = ACTIONS(1879),
    [sym_data_max_description_length] = ACTIONS(1879),
    [sym_data_max_type_length] = ACTIONS(1879),
    [sym_data_max_format_length] = ACTIONS(1879),
    [sym_data_max_source_length] = ACTIONS(1879),
    [sym_data_max_value_length] = ACTIONS(1879),
    [sym_data_max_label_length] = ACTIONS(1879),
    [sym_data_max_url_length] = ACTIONS(1879),
    [sym_max_last_login_info_length] = ACTIONS(1879),
    [sym_data_type_unknown] = ACTIONS(1879),
    [sym_data_type_string] = ACTIONS(1879),
    [sym_data_type_datetime] = ACTIONS(1879),
    [sym_data_type_date] = ACTIONS(1879),
    [sym_data_type_time] = ACTIONS(1879),
    [sym_data_type_image] = ACTIONS(1879),
    [sym_data_format_url] = ACTIONS(1879),
    [sym_data_format_phone] = ACTIONS(1879),
    [sym_data_format_email] = ACTIONS(1879),
    [sym_data_format_iso8601] = ACTIONS(1879),
    [sym_auditlog_priv] = ACTIONS(1879),
    [sym_configuration_priv] = ACTIONS(1879),
    [sym_deviceconfig_priv] = ACTIONS(1879),
    [sym_ftp_priv] = ACTIONS(1879),
    [sym_http_priv] = ACTIONS(1879),
    [sym_networkconfig_priv] = ACTIONS(1879),
    [sym_programport_priv] = ACTIONS(1879),
    [sym_remoteui_priv] = ACTIONS(1879),
    [sym_securitycontrol_priv] = ACTIONS(1879),
    [sym_softwaremanagement_priv] = ACTIONS(1879),
    [sym_terminal_priv] = ACTIONS(1879),
    [sym_tpadmin_priv] = ACTIONS(1879),
    [sym_user1_priv] = ACTIONS(1879),
    [sym_user2_priv] = ACTIONS(1879),
    [sym_user3_priv] = ACTIONS(1879),
    [sym_user4_priv] = ACTIONS(1879),
    [sym_usermanagement_priv] = ACTIONS(1879),
    [sym_valid_account] = ACTIONS(1879),
    [sym_err_name_invalid] = ACTIONS(1879),
    [sym_err_password_invalid] = ACTIONS(1879),
    [sym_err_invalid_account] = ACTIONS(1879),
    [sym_err_no_authorization] = ACTIONS(1879),
    [sym_err_invalid_parameter] = ACTIONS(1879),
    [sym_err_user_locked_out] = ACTIONS(1879),
    [sym_err_user_account_expired] = ACTIONS(1879),
    [sym_err_general_error] = ACTIONS(1879),
    [sym_err_authentication_error] = ACTIONS(1879),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1879),
    [sym_tls_validate_certificate] = ACTIONS(1879),
    [sym_tdata] = ACTIONS(1879),
    [sym_tchannel] = ACTIONS(1879),
    [sym_tlevel] = ACTIONS(1879),
    [sym_tbutton] = ACTIONS(1879),
    [sym_ttimeline] = ACTIONS(1879),
    [sym_tcustom] = ACTIONS(1879),
    [sym_url_struct] = ACTIONS(1879),
    [sym_dns_struct] = ACTIONS(1879),
    [sym_ip_address_struct] = ACTIONS(1879),
    [sym_dev_info_struct] = ACTIONS(1879),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1879),
    [sym_data_feed] = ACTIONS(1879),
    [sym_data_field] = ACTIONS(1879),
    [sym_data_record] = ACTIONS(1879),
    [sym_wc_data_feed] = ACTIONS(1879),
    [sym_wc_data_field] = ACTIONS(1879),
    [sym_wc_data_record] = ACTIONS(1879),
    [sym_last_login_info] = ACTIONS(1879),
    [sym____reserved___] = ACTIONS(1879),
    [sym_date] = ACTIONS(1879),
    [sym_day] = ACTIONS(1879),
    [sym_time] = ACTIONS(1879),
    [sym_ldate] = ACTIONS(1879),
    [sym_push_device] = ACTIONS(1879),
    [sym_push_channel] = ACTIONS(1879),
    [sym_push_devchan] = ACTIONS(1879),
    [sym_release_device] = ACTIONS(1879),
    [sym_release_channel] = ACTIONS(1879),
    [sym_release_devchan] = ACTIONS(1879),
    [sym_master_slot] = ACTIONS(1879),
    [sym_get_pulse_time] = ACTIONS(1879),
    [sym_get_timer] = ACTIONS(1879),
    [sym_master_sn] = ACTIONS(1879),
    [sym_system_number] = ACTIONS(1879),
    [sym_button] = ACTIONS(1879),
    [sym_dv_channel] = ACTIONS(1879),
    [sym_data] = ACTIONS(1879),
    [sym_level] = ACTIONS(1879),
    [sym_timeline] = ACTIONS(1879),
    [sym_channel] = ACTIONS(1879),
    [sym_custom] = ACTIONS(1879),
    [sym_atoi] = ACTIONS(1879),
    [sym_atol] = ACTIONS(1879),
    [sym_atof] = ACTIONS(1879),
    [sym_itoa] = ACTIONS(1879),
    [sym_itohex] = ACTIONS(1879),
    [sym_hextoi] = ACTIONS(1879),
    [sym_ftoa] = ACTIONS(1879),
    [sym_find_string] = ACTIONS(1879),
    [sym_left_string] = ACTIONS(1879),
    [sym_right_string] = ACTIONS(1879),
    [sym_mid_string] = ACTIONS(1879),
    [sym_remove_string] = ACTIONS(1879),
    [sym_get_buffer_string] = ACTIONS(1879),
    [sym_get_buffer_char] = ACTIONS(1879),
    [sym_get_multi_buffer_string] = ACTIONS(1879),
    [sym_length_string] = ACTIONS(1879),
    [sym_max_length_string] = ACTIONS(1879),
    [sym_set_length_string] = ACTIONS(1879),
    [sym_compare_string] = ACTIONS(1879),
    [sym_upper_string] = ACTIONS(1879),
    [sym_lower_string] = ACTIONS(1879),
    [sym_get_last] = ACTIONS(1879),
    [sym_date_to_day] = ACTIONS(1879),
    [sym_date_to_month] = ACTIONS(1879),
    [sym_date_to_year] = ACTIONS(1879),
    [sym_time_to_hour] = ACTIONS(1879),
    [sym_time_to_minute] = ACTIONS(1879),
    [sym_time_to_second] = ACTIONS(1879),
    [sym_day_of_week] = ACTIONS(1879),
    [sym_set_pulse_time] = ACTIONS(1879),
    [sym_set_timer] = ACTIONS(1879),
    [sym_file_open] = ACTIONS(1879),
    [sym_file_close] = ACTIONS(1879),
    [sym_file_read] = ACTIONS(1879),
    [sym_file_read_line] = ACTIONS(1879),
    [sym_file_write] = ACTIONS(1879),
    [sym_file_write_line] = ACTIONS(1879),
    [sym_file_copy] = ACTIONS(1879),
    [sym_file_delete] = ACTIONS(1879),
    [sym_file_rename] = ACTIONS(1879),
    [sym_file_dir] = ACTIONS(1879),
    [sym_file_getdir] = ACTIONS(1879),
    [sym_file_setdir] = ACTIONS(1879),
    [sym_file_create_dir] = ACTIONS(1879),
    [sym_file_remove_dir] = ACTIONS(1879),
    [sym_file_seek] = ACTIONS(1879),
    [sym_length_array] = ACTIONS(1879),
    [sym_max_length_array] = ACTIONS(1879),
    [sym_set_length_array] = ACTIONS(1879),
    [sym_ip_client_open] = ACTIONS(1879),
    [sym_ip_client_close] = ACTIONS(1879),
    [sym_ip_server_open] = ACTIONS(1879),
    [sym_ip_server_close] = ACTIONS(1879),
    [sym_ip_bound_client_open] = ACTIONS(1879),
    [sym_ip_mc_server_open] = ACTIONS(1879),
    [sym_ip_set_option] = ACTIONS(1879),
    [sym_ssh_client_open] = ACTIONS(1879),
    [sym_ssh_client_close] = ACTIONS(1879),
    [sym_tls_client_open] = ACTIONS(1879),
    [sym_tls_client_close] = ACTIONS(1879),
    [sym_do_push] = ACTIONS(1879),
    [sym_do_push_timed] = ACTIONS(1879),
    [sym_do_release] = ACTIONS(1879),
    [sym_device_id] = ACTIONS(1879),
    [sym_device_info] = ACTIONS(1879),
    [sym_device_id_string] = ACTIONS(1879),
    [sym_device_standby] = ACTIONS(1879),
    [sym_device_wake] = ACTIONS(1879),
    [sym_timeline_create] = ACTIONS(1879),
    [sym_timeline_kill] = ACTIONS(1879),
    [sym_timeline_reload] = ACTIONS(1879),
    [sym_timeline_pause] = ACTIONS(1879),
    [sym_timeline_restart] = ACTIONS(1879),
    [sym_timeline_set] = ACTIONS(1879),
    [sym_timeline_get] = ACTIONS(1879),
    [sym_timeline_active] = ACTIONS(1879),
    [sym_timeline_dynamic_id] = ACTIONS(1879),
    [sym_variable_to_string] = ACTIONS(1879),
    [sym_string_to_variable] = ACTIONS(1879),
    [sym_length_variable_to_string] = ACTIONS(1879),
    [sym_type_cast] = ACTIONS(1879),
    [sym_variable_to_xml] = ACTIONS(1879),
    [sym_xml_to_variable] = ACTIONS(1879),
    [sym_length_variable_to_xml] = ACTIONS(1879),
    [sym_redirect_string] = ACTIONS(1879),
    [sym_random_number] = ACTIONS(1879),
    [sym_get_unique_id] = ACTIONS(1879),
    [sym_get_system_number] = ACTIONS(1879),
    [sym_set_system_number] = ACTIONS(1879),
    [sym_get_serial_number] = ACTIONS(1879),
    [sym_get_master_build] = ACTIONS(1879),
    [sym_set_validation_code] = ACTIONS(1879),
    [sym_reboot] = ACTIONS(1879),
    [sym_load_duet_module] = ACTIONS(1879),
    [sym_unload_duet_module] = ACTIONS(1879),
    [sym_duet_mem_size_set] = ACTIONS(1879),
    [sym_duet_mem_size_get] = ACTIONS(1879),
    [sym_dynamic_application_device] = ACTIONS(1879),
    [sym_dynamic_polled_port] = ACTIONS(1879),
    [sym_static_port_binding] = ACTIONS(1879),
    [sym_static_ip_binding] = ACTIONS(1879),
    [sym_get_dns_list] = ACTIONS(1879),
    [sym_set_dns_list] = ACTIONS(1879),
    [sym_get_ip_address] = ACTIONS(1879),
    [sym_set_ip_address] = ACTIONS(1879),
    [sym_get_url_list] = ACTIONS(1879),
    [sym_add_url_entry] = ACTIONS(1879),
    [sym_delete_url_entry] = ACTIONS(1879),
    [sym_abs_value] = ACTIONS(1879),
    [sym_max_value] = ACTIONS(1879),
    [sym_min_value] = ACTIONS(1879),
    [sym_exp_value] = ACTIONS(1879),
    [sym_log_value] = ACTIONS(1879),
    [sym_log10_value] = ACTIONS(1879),
    [sym_power_value] = ACTIONS(1879),
    [sym_sqrt_value] = ACTIONS(1879),
    [sym_raw_be] = ACTIONS(1879),
    [sym_raw_le] = ACTIONS(1879),
    [sym_internal_queue_size_set] = ACTIONS(1879),
    [sym_internal_queue_size_get] = ACTIONS(1879),
    [sym_internal_threshold_set] = ACTIONS(1879),
    [sym_internal_threshold_get] = ACTIONS(1879),
    [sym_rebuilt_event] = ACTIONS(1879),
    [sym_do_custom_event] = ACTIONS(1879),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1879),
    [sym_clkmgr_set_clk_source] = ACTIONS(1879),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1879),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1879),
    [sym_clkmgr_get_timezone] = ACTIONS(1879),
    [sym_clkmgr_set_timezone] = ACTIONS(1879),
    [sym_clkmgr_get_resync_period] = ACTIONS(1879),
    [sym_clkmgr_set_resync_period] = ACTIONS(1879),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1879),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1879),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1879),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1879),
    [sym_clkmgr_get_timeservers] = ACTIONS(1879),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1879),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1879),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1879),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1879),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1879),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1879),
    [sym_set_log_level] = ACTIONS(1879),
    [sym_get_log_level] = ACTIONS(1879),
    [sym_amx_log] = ACTIONS(1879),
    [sym_smtp_server_config_set] = ACTIONS(1879),
    [sym_smtp_server_config_get] = ACTIONS(1879),
    [sym_smtp_send] = ACTIONS(1879),
    [sym_validate_netlinx_account] = ACTIONS(1879),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1879),
    [sym_authenticate_certificate] = ACTIONS(1879),
    [sym_audit_netlinx_session_event] = ACTIONS(1879),
    [sym_audit_netlinx_generic_event] = ACTIONS(1879),
    [sym_get_available_flash_disk_space] = ACTIONS(1879),
    [sym_get_max_flash_disk_space] = ACTIONS(1879),
    [sym_data_create_feed] = ACTIONS(1879),
    [sym_data_delete_feed] = ACTIONS(1879),
    [sym_data_publish_feed] = ACTIONS(1879),
    [sym_data_get_published_feed] = ACTIONS(1879),
    [sym_data_add_record] = ACTIONS(1879),
    [sym_data_get_event_record] = ACTIONS(1879),
    [sym__wc_data_create_feed] = ACTIONS(1879),
    [sym__wc_data_add_record] = ACTIONS(1879),
    [sym__wc_data_get_event_record] = ACTIONS(1879),
    [sym_format] = ACTIONS(1879),
    [sym_astro_clock] = ACTIONS(1879),
    [sym_set_outdoor_temperature] = ACTIONS(1879),
    [sym_set_virtual_level_count] = ACTIONS(1879),
    [sym_set_virtual_channel_count] = ACTIONS(1879),
    [sym_set_virtual_port_count] = ACTIONS(1879),
    [sym__wc_encode_int] = ACTIONS(1879),
    [sym__wc_decode_int] = ACTIONS(1879),
    [sym__wc_int] = ACTIONS(1879),
    [sym__wc_to_ch_int] = ACTIONS(1879),
    [sym__ch_to_wc_int] = ACTIONS(1879),
    [sym__wc_find_string_int] = ACTIONS(1879),
    [sym__wc_left_string_int] = ACTIONS(1879),
    [sym__wc_lower_string_int] = ACTIONS(1879),
    [sym__wc_mid_string_int] = ACTIONS(1879),
    [sym__wc_remove_string_int] = ACTIONS(1879),
    [sym__wc_right_string_int] = ACTIONS(1879),
    [sym__wc_upper_string_int] = ACTIONS(1879),
    [sym__wc_compare_string_int] = ACTIONS(1879),
    [sym__wc_get_buffer_char_int] = ACTIONS(1879),
    [sym__wc_get_buffer_string_int] = ACTIONS(1879),
    [sym__wc_concat_string_int] = ACTIONS(1879),
    [sym__wc_file_open_int] = ACTIONS(1879),
    [sym__wc_file_close_int] = ACTIONS(1879),
    [sym__wc_file_read_int] = ACTIONS(1879),
    [sym__wc_file_read_line_int] = ACTIONS(1879),
    [sym__wc_file_write_int] = ACTIONS(1879),
    [sym__wc_file_write_line_int] = ACTIONS(1879),
    [aux_sym_preproc_include_token1] = ACTIONS(1879),
    [aux_sym_preproc_define_token1] = ACTIONS(1881),
    [aux_sym_preproc_warn_token1] = ACTIONS(1881),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1881),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1881),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1881),
    [anon_sym_LBRACK] = ACTIONS(1881),
    [anon_sym_LBRACE] = ACTIONS(1881),
    [anon_sym_LPAREN] = ACTIONS(1879),
    [anon_sym_BANG] = ACTIONS(1881),
    [anon_sym_TILDE] = ACTIONS(1881),
    [anon_sym_DASH] = ACTIONS(1879),
    [anon_sym_PLUS] = ACTIONS(1879),
    [anon_sym_DASH_DASH] = ACTIONS(1881),
    [anon_sym_PLUS_PLUS] = ACTIONS(1881),
    [anon_sym_DQUOTE] = ACTIONS(1881),
    [anon_sym_SQUOTE] = ACTIONS(1881),
    [sym_number_literal] = ACTIONS(1881),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(343)] = {
    [sym_identifier] = ACTIONS(1697),
    [sym_program_name_keyword] = ACTIONS(1697),
    [sym_module_name_keyword] = ACTIONS(1697),
    [sym_define_device_keyword] = ACTIONS(1697),
    [sym_define_combine_keyword] = ACTIONS(1697),
    [sym_define_constant_keyword] = ACTIONS(1697),
    [sym_define_type_keyword] = ACTIONS(1697),
    [sym_define_variable_keyword] = ACTIONS(1697),
    [sym_define_system_variable_keyword] = ACTIONS(1697),
    [sym_define_connect_level_keyword] = ACTIONS(1697),
    [sym_define_latching_keyword] = ACTIONS(1697),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1697),
    [sym_define_toggling_keyword] = ACTIONS(1697),
    [sym_define_call_keyword] = ACTIONS(1697),
    [sym_define_function_keyword] = ACTIONS(1697),
    [sym_define_library_function_keyword] = ACTIONS(1697),
    [sym_define_start_keyword] = ACTIONS(1697),
    [sym_define_event_keyword] = ACTIONS(1697),
    [sym_define_program_keyword] = ACTIONS(1697),
    [sym_define_module_keyword] = ACTIONS(1697),
    [sym_char_keyword] = ACTIONS(1697),
    [sym_widechar_keyword] = ACTIONS(1697),
    [sym_integer_keyword] = ACTIONS(1697),
    [sym_sinteger_keyword] = ACTIONS(1697),
    [sym_long_keyword] = ACTIONS(1697),
    [sym_slong_keyword] = ACTIONS(1697),
    [sym_float_keyword] = ACTIONS(1697),
    [sym_double_keyword] = ACTIONS(1697),
    [sym_variant_keyword] = ACTIONS(1697),
    [sym_variantarray_keyword] = ACTIONS(1697),
    [sym_local_var_keyword] = ACTIONS(1697),
    [sym_stack_var_keyword] = ACTIONS(1697),
    [sym_constant_keyword] = ACTIONS(1697),
    [sym_volatile_keyword] = ACTIONS(1697),
    [sym_non_volatile_keyword] = ACTIONS(1697),
    [sym_persistent_keyword] = ACTIONS(1697),
    [sym_dev_keyword] = ACTIONS(1697),
    [sym_devlev_keyword] = ACTIONS(1697),
    [sym_devchan_keyword] = ACTIONS(1697),
    [sym_if_keyword] = ACTIONS(1697),
    [sym_switch_keyword] = ACTIONS(1697),
    [sym_case_keyword] = ACTIONS(1697),
    [sym_default_keyword] = ACTIONS(1697),
    [sym_while_keyword] = ACTIONS(1697),
    [sym_for_keyword] = ACTIONS(1697),
    [sym_break_keyword] = ACTIONS(1697),
    [sym_continue_keyword] = ACTIONS(1697),
    [sym_return_keyword] = ACTIONS(1697),
    [sym_select_keyword] = ACTIONS(1697),
    [sym_active_keyword] = ACTIONS(1697),
    [sym_struct_keyword] = ACTIONS(1697),
    [sym_structure_keyword] = ACTIONS(1697),
    [sym_button_event_keyword] = ACTIONS(1697),
    [sym_channel_event_keyword] = ACTIONS(1697),
    [sym_level_event_keyword] = ACTIONS(1697),
    [sym_data_event_keyword] = ACTIONS(1697),
    [sym_timeline_event_keyword] = ACTIONS(1697),
    [sym_custom_event_keyword] = ACTIONS(1697),
    [sym_push_keyword] = ACTIONS(1697),
    [sym_release_keyword] = ACTIONS(1697),
    [aux_sym_on_keyword_token1] = ACTIONS(1697),
    [aux_sym_off_keyword_token1] = ACTIONS(1697),
    [sym_send_level_keyword] = ACTIONS(1697),
    [sym_send_string_keyword] = ACTIONS(1697),
    [sym_send_command_keyword] = ACTIONS(1697),
    [sym_clear_buffer_keyword] = ACTIONS(1697),
    [sym_create_buffer_keyword] = ACTIONS(1697),
    [sym_create_multi_buffer_keyword] = ACTIONS(1697),
    [sym_call_keyword] = ACTIONS(1697),
    [sym_system_call_keyword] = ACTIONS(1697),
    [sym_devchan_to_keyword] = ACTIONS(1697),
    [sym_devchan_min_to_keyword] = ACTIONS(1697),
    [sym_devchan_total_off_keyword] = ACTIONS(1697),
    [sym_devchan_pulse_keyword] = ACTIONS(1697),
    [sym_wait_keyword] = ACTIONS(1697),
    [sym_cancel_wait_keyword] = ACTIONS(1697),
    [sym_wait_until_keyword] = ACTIONS(1697),
    [sym_cancel_wait_until_keyword] = ACTIONS(1697),
    [sym_cancel_all_wait_keyword] = ACTIONS(1697),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1697),
    [sym_not] = ACTIONS(1697),
    [sym_bnot] = ACTIONS(1697),
    [sym___file__] = ACTIONS(1697),
    [sym___date__] = ACTIONS(1697),
    [sym___ldate__] = ACTIONS(1697),
    [sym___time__] = ACTIONS(1697),
    [sym___line__] = ACTIONS(1697),
    [sym___name__] = ACTIONS(1697),
    [sym___netlinx__] = ACTIONS(1697),
    [sym_netlinx_axi_version] = ACTIONS(1697),
    [sym_url_flg_tcp] = ACTIONS(1697),
    [sym_url_flg_accinfopresent] = ACTIONS(1697),
    [sym_url_flg_temp] = ACTIONS(1697),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1697),
    [sym_url_flg_stat_mask] = ACTIONS(1697),
    [sym_url_flg_stat_lookup] = ACTIONS(1697),
    [sym_url_flg_stat_connecting] = ACTIONS(1697),
    [sym_url_flg_stat_waiting] = ACTIONS(1697),
    [sym_url_flg_stat_connected] = ACTIONS(1697),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1697),
    [sym_first_virtual_device] = ACTIONS(1697),
    [sym_first_local_port] = ACTIONS(1697),
    [sym_dynamic_virtual_device] = ACTIONS(1697),
    [sym_timeline_once] = ACTIONS(1697),
    [sym_timeline_repeat] = ACTIONS(1697),
    [sym_timeline_absolute] = ACTIONS(1697),
    [sym_timeline_relative] = ACTIONS(1697),
    [sym_do_push_timed_infinite] = ACTIONS(1697),
    [sym_source_type_no_address] = ACTIONS(1697),
    [sym_source_type_neuron_id] = ACTIONS(1697),
    [sym_source_type_ip_address] = ACTIONS(1697),
    [sym_source_type_axlink] = ACTIONS(1697),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1697),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1697),
    [sym_source_type_ip_socket_address] = ACTIONS(1697),
    [sym_source_type_rs232] = ACTIONS(1697),
    [sym_source_type_internal] = ACTIONS(1697),
    [sym_source_type_mac_address] = ACTIONS(1697),
    [sym_source_type_ipv4_port] = ACTIONS(1697),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1697),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1697),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1697),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1697),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1697),
    [sym_true] = ACTIONS(1697),
    [sym_false] = ACTIONS(1697),
    [sym_file_read_only] = ACTIONS(1697),
    [sym_file_rw_new] = ACTIONS(1697),
    [sym_file_rw_append] = ACTIONS(1697),
    [sym_ip_tcp] = ACTIONS(1697),
    [sym_ip_udp] = ACTIONS(1697),
    [sym_ip_udp_2way] = ACTIONS(1697),
    [sym_xml_encode_types] = ACTIONS(1697),
    [sym_xml_encode_char_as_list] = ACTIONS(1697),
    [sym_xml_encode_le] = ACTIONS(1697),
    [sym_xml_decode_types] = ACTIONS(1697),
    [sym_xml_decode_no_preserve] = ACTIONS(1697),
    [sym_duet_dev_type_display_device] = ACTIONS(1697),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1697),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1697),
    [sym_duet_dev_type_io_device] = ACTIONS(1697),
    [sym_duet_dev_type_relay_device] = ACTIONS(1697),
    [sym_duet_dev_type_ups] = ACTIONS(1697),
    [sym_duet_dev_type_amplifier] = ACTIONS(1697),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1697),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1697),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1697),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1697),
    [sym_duet_dev_type_camera] = ACTIONS(1697),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1697),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1697),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1697),
    [sym_duet_dev_type_dss] = ACTIONS(1697),
    [sym_duet_dev_type_dvr] = ACTIONS(1697),
    [sym_duet_dev_type_disc_device] = ACTIONS(1697),
    [sym_duet_dev_type_document_camera] = ACTIONS(1697),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1697),
    [sym_duet_dev_type_hvac] = ACTIONS(1697),
    [sym_duet_dev_type_keypad] = ACTIONS(1697),
    [sym_duet_dev_type_light] = ACTIONS(1697),
    [sym_duet_dev_type_monitor] = ACTIONS(1697),
    [sym_duet_dev_type_motor] = ACTIONS(1697),
    [sym_duet_dev_type_multi_window] = ACTIONS(1697),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1697),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1697),
    [sym_duet_dev_type_receiver] = ACTIONS(1697),
    [sym_duet_dev_type_security_system] = ACTIONS(1697),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1697),
    [sym_duet_dev_type_settop_box] = ACTIONS(1697),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1697),
    [sym_duet_dev_type_switcher] = ACTIONS(1697),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1697),
    [sym_duet_dev_type_tv] = ACTIONS(1697),
    [sym_duet_dev_type_utility] = ACTIONS(1697),
    [sym_duet_dev_type_vcr] = ACTIONS(1697),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1697),
    [sym_duet_dev_type_video_processor] = ACTIONS(1697),
    [sym_duet_dev_type_video_projector] = ACTIONS(1697),
    [sym_duet_dev_type_video_wall] = ACTIONS(1697),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1697),
    [sym_duet_dev_type_weather] = ACTIONS(1697),
    [sym_duet_dev_not_polled] = ACTIONS(1697),
    [sym_duet_dev_polled] = ACTIONS(1697),
    [sym_clkmgr_mode_network] = ACTIONS(1697),
    [sym_clkmgr_mode_standalone] = ACTIONS(1697),
    [sym_normal_standby] = ACTIONS(1697),
    [sym_normal_wake] = ACTIONS(1697),
    [sym_ip_multicast_ttl_option] = ACTIONS(1697),
    [sym_ip_ttl_subnet] = ACTIONS(1697),
    [sym_ip_ttl_site] = ACTIONS(1697),
    [sym_ip_ttl_region] = ACTIONS(1697),
    [sym_ip_ttl_continent] = ACTIONS(1697),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1697),
    [sym_ip_nodelay_on] = ACTIONS(1697),
    [sym_ip_nodelay_off] = ACTIONS(1697),
    [sym_amx_error] = ACTIONS(1697),
    [sym_amx_warning] = ACTIONS(1697),
    [sym_amx_info] = ACTIONS(1697),
    [sym_amx_debug] = ACTIONS(1697),
    [sym_netlinx_login_success] = ACTIONS(1697),
    [sym_netlinx_login_fail] = ACTIONS(1697),
    [sym_netlinx_logout] = ACTIONS(1697),
    [sym_null_str] = ACTIONS(1697),
    [sym_smtp_address] = ACTIONS(1697),
    [sym_smtp_port_number] = ACTIONS(1697),
    [sym_smtp_username] = ACTIONS(1697),
    [sym_smtp_password] = ACTIONS(1697),
    [sym_smtp_require_tls] = ACTIONS(1697),
    [sym_smtp_from] = ACTIONS(1697),
    [sym_smtp_tls_true] = ACTIONS(1697),
    [sym_smtp_tls_false] = ACTIONS(1697),
    [sym_listview_on_row_select_event] = ACTIONS(1697),
    [sym_data_structure_unknown] = ACTIONS(1697),
    [sym_data_structure_datafeed] = ACTIONS(1697),
    [sym_data_structure_datarecordset] = ACTIONS(1697),
    [sym_data_structure_datarecord] = ACTIONS(1697),
    [sym_data_structure_datafield] = ACTIONS(1697),
    [sym_data_max_content_field_count] = ACTIONS(1697),
    [sym_data_max_metadata_field_count] = ACTIONS(1697),
    [sym_data_max_id_length] = ACTIONS(1697),
    [sym_data_max_name_length] = ACTIONS(1697),
    [sym_data_max_description_length] = ACTIONS(1697),
    [sym_data_max_type_length] = ACTIONS(1697),
    [sym_data_max_format_length] = ACTIONS(1697),
    [sym_data_max_source_length] = ACTIONS(1697),
    [sym_data_max_value_length] = ACTIONS(1697),
    [sym_data_max_label_length] = ACTIONS(1697),
    [sym_data_max_url_length] = ACTIONS(1697),
    [sym_max_last_login_info_length] = ACTIONS(1697),
    [sym_data_type_unknown] = ACTIONS(1697),
    [sym_data_type_string] = ACTIONS(1697),
    [sym_data_type_datetime] = ACTIONS(1697),
    [sym_data_type_date] = ACTIONS(1697),
    [sym_data_type_time] = ACTIONS(1697),
    [sym_data_type_image] = ACTIONS(1697),
    [sym_data_format_url] = ACTIONS(1697),
    [sym_data_format_phone] = ACTIONS(1697),
    [sym_data_format_email] = ACTIONS(1697),
    [sym_data_format_iso8601] = ACTIONS(1697),
    [sym_auditlog_priv] = ACTIONS(1697),
    [sym_configuration_priv] = ACTIONS(1697),
    [sym_deviceconfig_priv] = ACTIONS(1697),
    [sym_ftp_priv] = ACTIONS(1697),
    [sym_http_priv] = ACTIONS(1697),
    [sym_networkconfig_priv] = ACTIONS(1697),
    [sym_programport_priv] = ACTIONS(1697),
    [sym_remoteui_priv] = ACTIONS(1697),
    [sym_securitycontrol_priv] = ACTIONS(1697),
    [sym_softwaremanagement_priv] = ACTIONS(1697),
    [sym_terminal_priv] = ACTIONS(1697),
    [sym_tpadmin_priv] = ACTIONS(1697),
    [sym_user1_priv] = ACTIONS(1697),
    [sym_user2_priv] = ACTIONS(1697),
    [sym_user3_priv] = ACTIONS(1697),
    [sym_user4_priv] = ACTIONS(1697),
    [sym_usermanagement_priv] = ACTIONS(1697),
    [sym_valid_account] = ACTIONS(1697),
    [sym_err_name_invalid] = ACTIONS(1697),
    [sym_err_password_invalid] = ACTIONS(1697),
    [sym_err_invalid_account] = ACTIONS(1697),
    [sym_err_no_authorization] = ACTIONS(1697),
    [sym_err_invalid_parameter] = ACTIONS(1697),
    [sym_err_user_locked_out] = ACTIONS(1697),
    [sym_err_user_account_expired] = ACTIONS(1697),
    [sym_err_general_error] = ACTIONS(1697),
    [sym_err_authentication_error] = ACTIONS(1697),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1697),
    [sym_tls_validate_certificate] = ACTIONS(1697),
    [sym_tdata] = ACTIONS(1697),
    [sym_tchannel] = ACTIONS(1697),
    [sym_tlevel] = ACTIONS(1697),
    [sym_tbutton] = ACTIONS(1697),
    [sym_ttimeline] = ACTIONS(1697),
    [sym_tcustom] = ACTIONS(1697),
    [sym_url_struct] = ACTIONS(1697),
    [sym_dns_struct] = ACTIONS(1697),
    [sym_ip_address_struct] = ACTIONS(1697),
    [sym_dev_info_struct] = ACTIONS(1697),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1697),
    [sym_data_feed] = ACTIONS(1697),
    [sym_data_field] = ACTIONS(1697),
    [sym_data_record] = ACTIONS(1697),
    [sym_wc_data_feed] = ACTIONS(1697),
    [sym_wc_data_field] = ACTIONS(1697),
    [sym_wc_data_record] = ACTIONS(1697),
    [sym_last_login_info] = ACTIONS(1697),
    [sym____reserved___] = ACTIONS(1697),
    [sym_date] = ACTIONS(1697),
    [sym_day] = ACTIONS(1697),
    [sym_time] = ACTIONS(1697),
    [sym_ldate] = ACTIONS(1697),
    [sym_push_device] = ACTIONS(1697),
    [sym_push_channel] = ACTIONS(1697),
    [sym_push_devchan] = ACTIONS(1697),
    [sym_release_device] = ACTIONS(1697),
    [sym_release_channel] = ACTIONS(1697),
    [sym_release_devchan] = ACTIONS(1697),
    [sym_master_slot] = ACTIONS(1697),
    [sym_get_pulse_time] = ACTIONS(1697),
    [sym_get_timer] = ACTIONS(1697),
    [sym_master_sn] = ACTIONS(1697),
    [sym_system_number] = ACTIONS(1697),
    [sym_button] = ACTIONS(1697),
    [sym_dv_channel] = ACTIONS(1697),
    [sym_data] = ACTIONS(1697),
    [sym_level] = ACTIONS(1697),
    [sym_timeline] = ACTIONS(1697),
    [sym_channel] = ACTIONS(1697),
    [sym_custom] = ACTIONS(1697),
    [sym_atoi] = ACTIONS(1697),
    [sym_atol] = ACTIONS(1697),
    [sym_atof] = ACTIONS(1697),
    [sym_itoa] = ACTIONS(1697),
    [sym_itohex] = ACTIONS(1697),
    [sym_hextoi] = ACTIONS(1697),
    [sym_ftoa] = ACTIONS(1697),
    [sym_find_string] = ACTIONS(1697),
    [sym_left_string] = ACTIONS(1697),
    [sym_right_string] = ACTIONS(1697),
    [sym_mid_string] = ACTIONS(1697),
    [sym_remove_string] = ACTIONS(1697),
    [sym_get_buffer_string] = ACTIONS(1697),
    [sym_get_buffer_char] = ACTIONS(1697),
    [sym_get_multi_buffer_string] = ACTIONS(1697),
    [sym_length_string] = ACTIONS(1697),
    [sym_max_length_string] = ACTIONS(1697),
    [sym_set_length_string] = ACTIONS(1697),
    [sym_compare_string] = ACTIONS(1697),
    [sym_upper_string] = ACTIONS(1697),
    [sym_lower_string] = ACTIONS(1697),
    [sym_get_last] = ACTIONS(1697),
    [sym_date_to_day] = ACTIONS(1697),
    [sym_date_to_month] = ACTIONS(1697),
    [sym_date_to_year] = ACTIONS(1697),
    [sym_time_to_hour] = ACTIONS(1697),
    [sym_time_to_minute] = ACTIONS(1697),
    [sym_time_to_second] = ACTIONS(1697),
    [sym_day_of_week] = ACTIONS(1697),
    [sym_set_pulse_time] = ACTIONS(1697),
    [sym_set_timer] = ACTIONS(1697),
    [sym_file_open] = ACTIONS(1697),
    [sym_file_close] = ACTIONS(1697),
    [sym_file_read] = ACTIONS(1697),
    [sym_file_read_line] = ACTIONS(1697),
    [sym_file_write] = ACTIONS(1697),
    [sym_file_write_line] = ACTIONS(1697),
    [sym_file_copy] = ACTIONS(1697),
    [sym_file_delete] = ACTIONS(1697),
    [sym_file_rename] = ACTIONS(1697),
    [sym_file_dir] = ACTIONS(1697),
    [sym_file_getdir] = ACTIONS(1697),
    [sym_file_setdir] = ACTIONS(1697),
    [sym_file_create_dir] = ACTIONS(1697),
    [sym_file_remove_dir] = ACTIONS(1697),
    [sym_file_seek] = ACTIONS(1697),
    [sym_length_array] = ACTIONS(1697),
    [sym_max_length_array] = ACTIONS(1697),
    [sym_set_length_array] = ACTIONS(1697),
    [sym_ip_client_open] = ACTIONS(1697),
    [sym_ip_client_close] = ACTIONS(1697),
    [sym_ip_server_open] = ACTIONS(1697),
    [sym_ip_server_close] = ACTIONS(1697),
    [sym_ip_bound_client_open] = ACTIONS(1697),
    [sym_ip_mc_server_open] = ACTIONS(1697),
    [sym_ip_set_option] = ACTIONS(1697),
    [sym_ssh_client_open] = ACTIONS(1697),
    [sym_ssh_client_close] = ACTIONS(1697),
    [sym_tls_client_open] = ACTIONS(1697),
    [sym_tls_client_close] = ACTIONS(1697),
    [sym_do_push] = ACTIONS(1697),
    [sym_do_push_timed] = ACTIONS(1697),
    [sym_do_release] = ACTIONS(1697),
    [sym_device_id] = ACTIONS(1697),
    [sym_device_info] = ACTIONS(1697),
    [sym_device_id_string] = ACTIONS(1697),
    [sym_device_standby] = ACTIONS(1697),
    [sym_device_wake] = ACTIONS(1697),
    [sym_timeline_create] = ACTIONS(1697),
    [sym_timeline_kill] = ACTIONS(1697),
    [sym_timeline_reload] = ACTIONS(1697),
    [sym_timeline_pause] = ACTIONS(1697),
    [sym_timeline_restart] = ACTIONS(1697),
    [sym_timeline_set] = ACTIONS(1697),
    [sym_timeline_get] = ACTIONS(1697),
    [sym_timeline_active] = ACTIONS(1697),
    [sym_timeline_dynamic_id] = ACTIONS(1697),
    [sym_variable_to_string] = ACTIONS(1697),
    [sym_string_to_variable] = ACTIONS(1697),
    [sym_length_variable_to_string] = ACTIONS(1697),
    [sym_type_cast] = ACTIONS(1697),
    [sym_variable_to_xml] = ACTIONS(1697),
    [sym_xml_to_variable] = ACTIONS(1697),
    [sym_length_variable_to_xml] = ACTIONS(1697),
    [sym_redirect_string] = ACTIONS(1697),
    [sym_random_number] = ACTIONS(1697),
    [sym_get_unique_id] = ACTIONS(1697),
    [sym_get_system_number] = ACTIONS(1697),
    [sym_set_system_number] = ACTIONS(1697),
    [sym_get_serial_number] = ACTIONS(1697),
    [sym_get_master_build] = ACTIONS(1697),
    [sym_set_validation_code] = ACTIONS(1697),
    [sym_reboot] = ACTIONS(1697),
    [sym_load_duet_module] = ACTIONS(1697),
    [sym_unload_duet_module] = ACTIONS(1697),
    [sym_duet_mem_size_set] = ACTIONS(1697),
    [sym_duet_mem_size_get] = ACTIONS(1697),
    [sym_dynamic_application_device] = ACTIONS(1697),
    [sym_dynamic_polled_port] = ACTIONS(1697),
    [sym_static_port_binding] = ACTIONS(1697),
    [sym_static_ip_binding] = ACTIONS(1697),
    [sym_get_dns_list] = ACTIONS(1697),
    [sym_set_dns_list] = ACTIONS(1697),
    [sym_get_ip_address] = ACTIONS(1697),
    [sym_set_ip_address] = ACTIONS(1697),
    [sym_get_url_list] = ACTIONS(1697),
    [sym_add_url_entry] = ACTIONS(1697),
    [sym_delete_url_entry] = ACTIONS(1697),
    [sym_abs_value] = ACTIONS(1697),
    [sym_max_value] = ACTIONS(1697),
    [sym_min_value] = ACTIONS(1697),
    [sym_exp_value] = ACTIONS(1697),
    [sym_log_value] = ACTIONS(1697),
    [sym_log10_value] = ACTIONS(1697),
    [sym_power_value] = ACTIONS(1697),
    [sym_sqrt_value] = ACTIONS(1697),
    [sym_raw_be] = ACTIONS(1697),
    [sym_raw_le] = ACTIONS(1697),
    [sym_internal_queue_size_set] = ACTIONS(1697),
    [sym_internal_queue_size_get] = ACTIONS(1697),
    [sym_internal_threshold_set] = ACTIONS(1697),
    [sym_internal_threshold_get] = ACTIONS(1697),
    [sym_rebuilt_event] = ACTIONS(1697),
    [sym_do_custom_event] = ACTIONS(1697),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1697),
    [sym_clkmgr_set_clk_source] = ACTIONS(1697),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1697),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1697),
    [sym_clkmgr_get_timezone] = ACTIONS(1697),
    [sym_clkmgr_set_timezone] = ACTIONS(1697),
    [sym_clkmgr_get_resync_period] = ACTIONS(1697),
    [sym_clkmgr_set_resync_period] = ACTIONS(1697),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1697),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1697),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1697),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1697),
    [sym_clkmgr_get_timeservers] = ACTIONS(1697),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1697),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1697),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1697),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1697),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1697),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1697),
    [sym_set_log_level] = ACTIONS(1697),
    [sym_get_log_level] = ACTIONS(1697),
    [sym_amx_log] = ACTIONS(1697),
    [sym_smtp_server_config_set] = ACTIONS(1697),
    [sym_smtp_server_config_get] = ACTIONS(1697),
    [sym_smtp_send] = ACTIONS(1697),
    [sym_validate_netlinx_account] = ACTIONS(1697),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1697),
    [sym_authenticate_certificate] = ACTIONS(1697),
    [sym_audit_netlinx_session_event] = ACTIONS(1697),
    [sym_audit_netlinx_generic_event] = ACTIONS(1697),
    [sym_get_available_flash_disk_space] = ACTIONS(1697),
    [sym_get_max_flash_disk_space] = ACTIONS(1697),
    [sym_data_create_feed] = ACTIONS(1697),
    [sym_data_delete_feed] = ACTIONS(1697),
    [sym_data_publish_feed] = ACTIONS(1697),
    [sym_data_get_published_feed] = ACTIONS(1697),
    [sym_data_add_record] = ACTIONS(1697),
    [sym_data_get_event_record] = ACTIONS(1697),
    [sym__wc_data_create_feed] = ACTIONS(1697),
    [sym__wc_data_add_record] = ACTIONS(1697),
    [sym__wc_data_get_event_record] = ACTIONS(1697),
    [sym_format] = ACTIONS(1697),
    [sym_astro_clock] = ACTIONS(1697),
    [sym_set_outdoor_temperature] = ACTIONS(1697),
    [sym_set_virtual_level_count] = ACTIONS(1697),
    [sym_set_virtual_channel_count] = ACTIONS(1697),
    [sym_set_virtual_port_count] = ACTIONS(1697),
    [sym__wc_encode_int] = ACTIONS(1697),
    [sym__wc_decode_int] = ACTIONS(1697),
    [sym__wc_int] = ACTIONS(1697),
    [sym__wc_to_ch_int] = ACTIONS(1697),
    [sym__ch_to_wc_int] = ACTIONS(1697),
    [sym__wc_find_string_int] = ACTIONS(1697),
    [sym__wc_left_string_int] = ACTIONS(1697),
    [sym__wc_lower_string_int] = ACTIONS(1697),
    [sym__wc_mid_string_int] = ACTIONS(1697),
    [sym__wc_remove_string_int] = ACTIONS(1697),
    [sym__wc_right_string_int] = ACTIONS(1697),
    [sym__wc_upper_string_int] = ACTIONS(1697),
    [sym__wc_compare_string_int] = ACTIONS(1697),
    [sym__wc_get_buffer_char_int] = ACTIONS(1697),
    [sym__wc_get_buffer_string_int] = ACTIONS(1697),
    [sym__wc_concat_string_int] = ACTIONS(1697),
    [sym__wc_file_open_int] = ACTIONS(1697),
    [sym__wc_file_close_int] = ACTIONS(1697),
    [sym__wc_file_read_int] = ACTIONS(1697),
    [sym__wc_file_read_line_int] = ACTIONS(1697),
    [sym__wc_file_write_int] = ACTIONS(1697),
    [sym__wc_file_write_line_int] = ACTIONS(1697),
    [aux_sym_preproc_include_token1] = ACTIONS(1697),
    [aux_sym_preproc_define_token1] = ACTIONS(1695),
    [aux_sym_preproc_warn_token1] = ACTIONS(1695),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1695),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1695),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1695),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1695),
    [aux_sym_preproc_else_token1] = ACTIONS(1695),
    [anon_sym_LBRACK] = ACTIONS(1695),
    [anon_sym_LBRACE] = ACTIONS(1695),
    [anon_sym_LPAREN] = ACTIONS(1697),
    [anon_sym_BANG] = ACTIONS(1695),
    [anon_sym_TILDE] = ACTIONS(1695),
    [anon_sym_DASH] = ACTIONS(1697),
    [anon_sym_PLUS] = ACTIONS(1697),
    [anon_sym_DASH_DASH] = ACTIONS(1695),
    [anon_sym_PLUS_PLUS] = ACTIONS(1695),
    [anon_sym_DQUOTE] = ACTIONS(1695),
    [anon_sym_SQUOTE] = ACTIONS(1695),
    [sym_number_literal] = ACTIONS(1695),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(344)] = {
    [ts_builtin_sym_end] = ACTIONS(1749),
    [sym_identifier] = ACTIONS(1747),
    [sym_program_name_keyword] = ACTIONS(1747),
    [sym_module_name_keyword] = ACTIONS(1747),
    [sym_define_device_keyword] = ACTIONS(1747),
    [sym_define_combine_keyword] = ACTIONS(1747),
    [sym_define_constant_keyword] = ACTIONS(1747),
    [sym_define_type_keyword] = ACTIONS(1747),
    [sym_define_variable_keyword] = ACTIONS(1747),
    [sym_define_system_variable_keyword] = ACTIONS(1747),
    [sym_define_connect_level_keyword] = ACTIONS(1747),
    [sym_define_latching_keyword] = ACTIONS(1747),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1747),
    [sym_define_toggling_keyword] = ACTIONS(1747),
    [sym_define_call_keyword] = ACTIONS(1747),
    [sym_define_function_keyword] = ACTIONS(1747),
    [sym_define_library_function_keyword] = ACTIONS(1747),
    [sym_define_start_keyword] = ACTIONS(1747),
    [sym_define_event_keyword] = ACTIONS(1747),
    [sym_define_program_keyword] = ACTIONS(1747),
    [sym_define_module_keyword] = ACTIONS(1747),
    [sym_char_keyword] = ACTIONS(1747),
    [sym_widechar_keyword] = ACTIONS(1747),
    [sym_integer_keyword] = ACTIONS(1747),
    [sym_sinteger_keyword] = ACTIONS(1747),
    [sym_long_keyword] = ACTIONS(1747),
    [sym_slong_keyword] = ACTIONS(1747),
    [sym_float_keyword] = ACTIONS(1747),
    [sym_double_keyword] = ACTIONS(1747),
    [sym_variant_keyword] = ACTIONS(1747),
    [sym_variantarray_keyword] = ACTIONS(1747),
    [sym_local_var_keyword] = ACTIONS(1747),
    [sym_stack_var_keyword] = ACTIONS(1747),
    [sym_constant_keyword] = ACTIONS(1747),
    [sym_volatile_keyword] = ACTIONS(1747),
    [sym_non_volatile_keyword] = ACTIONS(1747),
    [sym_persistent_keyword] = ACTIONS(1747),
    [sym_dev_keyword] = ACTIONS(1747),
    [sym_devlev_keyword] = ACTIONS(1747),
    [sym_devchan_keyword] = ACTIONS(1747),
    [sym_if_keyword] = ACTIONS(1747),
    [sym_else_keyword] = ACTIONS(1747),
    [sym_switch_keyword] = ACTIONS(1747),
    [sym_case_keyword] = ACTIONS(1747),
    [sym_default_keyword] = ACTIONS(1747),
    [sym_while_keyword] = ACTIONS(1747),
    [sym_for_keyword] = ACTIONS(1747),
    [sym_break_keyword] = ACTIONS(1747),
    [sym_continue_keyword] = ACTIONS(1747),
    [sym_return_keyword] = ACTIONS(1747),
    [sym_select_keyword] = ACTIONS(1747),
    [sym_active_keyword] = ACTIONS(1747),
    [sym_struct_keyword] = ACTIONS(1747),
    [sym_structure_keyword] = ACTIONS(1747),
    [sym_button_event_keyword] = ACTIONS(1747),
    [sym_channel_event_keyword] = ACTIONS(1747),
    [sym_level_event_keyword] = ACTIONS(1747),
    [sym_data_event_keyword] = ACTIONS(1747),
    [sym_timeline_event_keyword] = ACTIONS(1747),
    [sym_custom_event_keyword] = ACTIONS(1747),
    [sym_push_keyword] = ACTIONS(1747),
    [sym_release_keyword] = ACTIONS(1747),
    [aux_sym_on_keyword_token1] = ACTIONS(1747),
    [aux_sym_off_keyword_token1] = ACTIONS(1747),
    [sym_send_level_keyword] = ACTIONS(1747),
    [sym_send_string_keyword] = ACTIONS(1747),
    [sym_send_command_keyword] = ACTIONS(1747),
    [sym_clear_buffer_keyword] = ACTIONS(1747),
    [sym_create_buffer_keyword] = ACTIONS(1747),
    [sym_create_multi_buffer_keyword] = ACTIONS(1747),
    [sym_call_keyword] = ACTIONS(1747),
    [sym_system_call_keyword] = ACTIONS(1747),
    [sym_devchan_to_keyword] = ACTIONS(1747),
    [sym_devchan_min_to_keyword] = ACTIONS(1747),
    [sym_devchan_total_off_keyword] = ACTIONS(1747),
    [sym_devchan_pulse_keyword] = ACTIONS(1747),
    [sym_wait_keyword] = ACTIONS(1747),
    [sym_cancel_wait_keyword] = ACTIONS(1747),
    [sym_wait_until_keyword] = ACTIONS(1747),
    [sym_cancel_wait_until_keyword] = ACTIONS(1747),
    [sym_cancel_all_wait_keyword] = ACTIONS(1747),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1747),
    [sym_not] = ACTIONS(1747),
    [sym_bnot] = ACTIONS(1747),
    [sym___file__] = ACTIONS(1747),
    [sym___date__] = ACTIONS(1747),
    [sym___ldate__] = ACTIONS(1747),
    [sym___time__] = ACTIONS(1747),
    [sym___line__] = ACTIONS(1747),
    [sym___name__] = ACTIONS(1747),
    [sym___netlinx__] = ACTIONS(1747),
    [sym_netlinx_axi_version] = ACTIONS(1747),
    [sym_url_flg_tcp] = ACTIONS(1747),
    [sym_url_flg_accinfopresent] = ACTIONS(1747),
    [sym_url_flg_temp] = ACTIONS(1747),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1747),
    [sym_url_flg_stat_mask] = ACTIONS(1747),
    [sym_url_flg_stat_lookup] = ACTIONS(1747),
    [sym_url_flg_stat_connecting] = ACTIONS(1747),
    [sym_url_flg_stat_waiting] = ACTIONS(1747),
    [sym_url_flg_stat_connected] = ACTIONS(1747),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1747),
    [sym_first_virtual_device] = ACTIONS(1747),
    [sym_first_local_port] = ACTIONS(1747),
    [sym_dynamic_virtual_device] = ACTIONS(1747),
    [sym_timeline_once] = ACTIONS(1747),
    [sym_timeline_repeat] = ACTIONS(1747),
    [sym_timeline_absolute] = ACTIONS(1747),
    [sym_timeline_relative] = ACTIONS(1747),
    [sym_do_push_timed_infinite] = ACTIONS(1747),
    [sym_source_type_no_address] = ACTIONS(1747),
    [sym_source_type_neuron_id] = ACTIONS(1747),
    [sym_source_type_ip_address] = ACTIONS(1747),
    [sym_source_type_axlink] = ACTIONS(1747),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1747),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1747),
    [sym_source_type_ip_socket_address] = ACTIONS(1747),
    [sym_source_type_rs232] = ACTIONS(1747),
    [sym_source_type_internal] = ACTIONS(1747),
    [sym_source_type_mac_address] = ACTIONS(1747),
    [sym_source_type_ipv4_port] = ACTIONS(1747),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1747),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1747),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1747),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1747),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1747),
    [sym_true] = ACTIONS(1747),
    [sym_false] = ACTIONS(1747),
    [sym_file_read_only] = ACTIONS(1747),
    [sym_file_rw_new] = ACTIONS(1747),
    [sym_file_rw_append] = ACTIONS(1747),
    [sym_ip_tcp] = ACTIONS(1747),
    [sym_ip_udp] = ACTIONS(1747),
    [sym_ip_udp_2way] = ACTIONS(1747),
    [sym_xml_encode_types] = ACTIONS(1747),
    [sym_xml_encode_char_as_list] = ACTIONS(1747),
    [sym_xml_encode_le] = ACTIONS(1747),
    [sym_xml_decode_types] = ACTIONS(1747),
    [sym_xml_decode_no_preserve] = ACTIONS(1747),
    [sym_duet_dev_type_display_device] = ACTIONS(1747),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1747),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1747),
    [sym_duet_dev_type_io_device] = ACTIONS(1747),
    [sym_duet_dev_type_relay_device] = ACTIONS(1747),
    [sym_duet_dev_type_ups] = ACTIONS(1747),
    [sym_duet_dev_type_amplifier] = ACTIONS(1747),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1747),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1747),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1747),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1747),
    [sym_duet_dev_type_camera] = ACTIONS(1747),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1747),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1747),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1747),
    [sym_duet_dev_type_dss] = ACTIONS(1747),
    [sym_duet_dev_type_dvr] = ACTIONS(1747),
    [sym_duet_dev_type_disc_device] = ACTIONS(1747),
    [sym_duet_dev_type_document_camera] = ACTIONS(1747),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1747),
    [sym_duet_dev_type_hvac] = ACTIONS(1747),
    [sym_duet_dev_type_keypad] = ACTIONS(1747),
    [sym_duet_dev_type_light] = ACTIONS(1747),
    [sym_duet_dev_type_monitor] = ACTIONS(1747),
    [sym_duet_dev_type_motor] = ACTIONS(1747),
    [sym_duet_dev_type_multi_window] = ACTIONS(1747),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1747),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1747),
    [sym_duet_dev_type_receiver] = ACTIONS(1747),
    [sym_duet_dev_type_security_system] = ACTIONS(1747),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1747),
    [sym_duet_dev_type_settop_box] = ACTIONS(1747),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1747),
    [sym_duet_dev_type_switcher] = ACTIONS(1747),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1747),
    [sym_duet_dev_type_tv] = ACTIONS(1747),
    [sym_duet_dev_type_utility] = ACTIONS(1747),
    [sym_duet_dev_type_vcr] = ACTIONS(1747),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1747),
    [sym_duet_dev_type_video_processor] = ACTIONS(1747),
    [sym_duet_dev_type_video_projector] = ACTIONS(1747),
    [sym_duet_dev_type_video_wall] = ACTIONS(1747),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1747),
    [sym_duet_dev_type_weather] = ACTIONS(1747),
    [sym_duet_dev_not_polled] = ACTIONS(1747),
    [sym_duet_dev_polled] = ACTIONS(1747),
    [sym_clkmgr_mode_network] = ACTIONS(1747),
    [sym_clkmgr_mode_standalone] = ACTIONS(1747),
    [sym_normal_standby] = ACTIONS(1747),
    [sym_normal_wake] = ACTIONS(1747),
    [sym_ip_multicast_ttl_option] = ACTIONS(1747),
    [sym_ip_ttl_subnet] = ACTIONS(1747),
    [sym_ip_ttl_site] = ACTIONS(1747),
    [sym_ip_ttl_region] = ACTIONS(1747),
    [sym_ip_ttl_continent] = ACTIONS(1747),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1747),
    [sym_ip_nodelay_on] = ACTIONS(1747),
    [sym_ip_nodelay_off] = ACTIONS(1747),
    [sym_amx_error] = ACTIONS(1747),
    [sym_amx_warning] = ACTIONS(1747),
    [sym_amx_info] = ACTIONS(1747),
    [sym_amx_debug] = ACTIONS(1747),
    [sym_netlinx_login_success] = ACTIONS(1747),
    [sym_netlinx_login_fail] = ACTIONS(1747),
    [sym_netlinx_logout] = ACTIONS(1747),
    [sym_null_str] = ACTIONS(1747),
    [sym_smtp_address] = ACTIONS(1747),
    [sym_smtp_port_number] = ACTIONS(1747),
    [sym_smtp_username] = ACTIONS(1747),
    [sym_smtp_password] = ACTIONS(1747),
    [sym_smtp_require_tls] = ACTIONS(1747),
    [sym_smtp_from] = ACTIONS(1747),
    [sym_smtp_tls_true] = ACTIONS(1747),
    [sym_smtp_tls_false] = ACTIONS(1747),
    [sym_listview_on_row_select_event] = ACTIONS(1747),
    [sym_data_structure_unknown] = ACTIONS(1747),
    [sym_data_structure_datafeed] = ACTIONS(1747),
    [sym_data_structure_datarecordset] = ACTIONS(1747),
    [sym_data_structure_datarecord] = ACTIONS(1747),
    [sym_data_structure_datafield] = ACTIONS(1747),
    [sym_data_max_content_field_count] = ACTIONS(1747),
    [sym_data_max_metadata_field_count] = ACTIONS(1747),
    [sym_data_max_id_length] = ACTIONS(1747),
    [sym_data_max_name_length] = ACTIONS(1747),
    [sym_data_max_description_length] = ACTIONS(1747),
    [sym_data_max_type_length] = ACTIONS(1747),
    [sym_data_max_format_length] = ACTIONS(1747),
    [sym_data_max_source_length] = ACTIONS(1747),
    [sym_data_max_value_length] = ACTIONS(1747),
    [sym_data_max_label_length] = ACTIONS(1747),
    [sym_data_max_url_length] = ACTIONS(1747),
    [sym_max_last_login_info_length] = ACTIONS(1747),
    [sym_data_type_unknown] = ACTIONS(1747),
    [sym_data_type_string] = ACTIONS(1747),
    [sym_data_type_datetime] = ACTIONS(1747),
    [sym_data_type_date] = ACTIONS(1747),
    [sym_data_type_time] = ACTIONS(1747),
    [sym_data_type_image] = ACTIONS(1747),
    [sym_data_format_url] = ACTIONS(1747),
    [sym_data_format_phone] = ACTIONS(1747),
    [sym_data_format_email] = ACTIONS(1747),
    [sym_data_format_iso8601] = ACTIONS(1747),
    [sym_auditlog_priv] = ACTIONS(1747),
    [sym_configuration_priv] = ACTIONS(1747),
    [sym_deviceconfig_priv] = ACTIONS(1747),
    [sym_ftp_priv] = ACTIONS(1747),
    [sym_http_priv] = ACTIONS(1747),
    [sym_networkconfig_priv] = ACTIONS(1747),
    [sym_programport_priv] = ACTIONS(1747),
    [sym_remoteui_priv] = ACTIONS(1747),
    [sym_securitycontrol_priv] = ACTIONS(1747),
    [sym_softwaremanagement_priv] = ACTIONS(1747),
    [sym_terminal_priv] = ACTIONS(1747),
    [sym_tpadmin_priv] = ACTIONS(1747),
    [sym_user1_priv] = ACTIONS(1747),
    [sym_user2_priv] = ACTIONS(1747),
    [sym_user3_priv] = ACTIONS(1747),
    [sym_user4_priv] = ACTIONS(1747),
    [sym_usermanagement_priv] = ACTIONS(1747),
    [sym_valid_account] = ACTIONS(1747),
    [sym_err_name_invalid] = ACTIONS(1747),
    [sym_err_password_invalid] = ACTIONS(1747),
    [sym_err_invalid_account] = ACTIONS(1747),
    [sym_err_no_authorization] = ACTIONS(1747),
    [sym_err_invalid_parameter] = ACTIONS(1747),
    [sym_err_user_locked_out] = ACTIONS(1747),
    [sym_err_user_account_expired] = ACTIONS(1747),
    [sym_err_general_error] = ACTIONS(1747),
    [sym_err_authentication_error] = ACTIONS(1747),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1747),
    [sym_tls_validate_certificate] = ACTIONS(1747),
    [sym_tdata] = ACTIONS(1747),
    [sym_tchannel] = ACTIONS(1747),
    [sym_tlevel] = ACTIONS(1747),
    [sym_tbutton] = ACTIONS(1747),
    [sym_ttimeline] = ACTIONS(1747),
    [sym_tcustom] = ACTIONS(1747),
    [sym_url_struct] = ACTIONS(1747),
    [sym_dns_struct] = ACTIONS(1747),
    [sym_ip_address_struct] = ACTIONS(1747),
    [sym_dev_info_struct] = ACTIONS(1747),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1747),
    [sym_data_feed] = ACTIONS(1747),
    [sym_data_field] = ACTIONS(1747),
    [sym_data_record] = ACTIONS(1747),
    [sym_wc_data_feed] = ACTIONS(1747),
    [sym_wc_data_field] = ACTIONS(1747),
    [sym_wc_data_record] = ACTIONS(1747),
    [sym_last_login_info] = ACTIONS(1747),
    [sym____reserved___] = ACTIONS(1747),
    [sym_date] = ACTIONS(1747),
    [sym_day] = ACTIONS(1747),
    [sym_time] = ACTIONS(1747),
    [sym_ldate] = ACTIONS(1747),
    [sym_push_device] = ACTIONS(1747),
    [sym_push_channel] = ACTIONS(1747),
    [sym_push_devchan] = ACTIONS(1747),
    [sym_release_device] = ACTIONS(1747),
    [sym_release_channel] = ACTIONS(1747),
    [sym_release_devchan] = ACTIONS(1747),
    [sym_master_slot] = ACTIONS(1747),
    [sym_get_pulse_time] = ACTIONS(1747),
    [sym_get_timer] = ACTIONS(1747),
    [sym_master_sn] = ACTIONS(1747),
    [sym_system_number] = ACTIONS(1747),
    [sym_button] = ACTIONS(1747),
    [sym_dv_channel] = ACTIONS(1747),
    [sym_data] = ACTIONS(1747),
    [sym_level] = ACTIONS(1747),
    [sym_timeline] = ACTIONS(1747),
    [sym_channel] = ACTIONS(1747),
    [sym_custom] = ACTIONS(1747),
    [sym_atoi] = ACTIONS(1747),
    [sym_atol] = ACTIONS(1747),
    [sym_atof] = ACTIONS(1747),
    [sym_itoa] = ACTIONS(1747),
    [sym_itohex] = ACTIONS(1747),
    [sym_hextoi] = ACTIONS(1747),
    [sym_ftoa] = ACTIONS(1747),
    [sym_find_string] = ACTIONS(1747),
    [sym_left_string] = ACTIONS(1747),
    [sym_right_string] = ACTIONS(1747),
    [sym_mid_string] = ACTIONS(1747),
    [sym_remove_string] = ACTIONS(1747),
    [sym_get_buffer_string] = ACTIONS(1747),
    [sym_get_buffer_char] = ACTIONS(1747),
    [sym_get_multi_buffer_string] = ACTIONS(1747),
    [sym_length_string] = ACTIONS(1747),
    [sym_max_length_string] = ACTIONS(1747),
    [sym_set_length_string] = ACTIONS(1747),
    [sym_compare_string] = ACTIONS(1747),
    [sym_upper_string] = ACTIONS(1747),
    [sym_lower_string] = ACTIONS(1747),
    [sym_get_last] = ACTIONS(1747),
    [sym_date_to_day] = ACTIONS(1747),
    [sym_date_to_month] = ACTIONS(1747),
    [sym_date_to_year] = ACTIONS(1747),
    [sym_time_to_hour] = ACTIONS(1747),
    [sym_time_to_minute] = ACTIONS(1747),
    [sym_time_to_second] = ACTIONS(1747),
    [sym_day_of_week] = ACTIONS(1747),
    [sym_set_pulse_time] = ACTIONS(1747),
    [sym_set_timer] = ACTIONS(1747),
    [sym_file_open] = ACTIONS(1747),
    [sym_file_close] = ACTIONS(1747),
    [sym_file_read] = ACTIONS(1747),
    [sym_file_read_line] = ACTIONS(1747),
    [sym_file_write] = ACTIONS(1747),
    [sym_file_write_line] = ACTIONS(1747),
    [sym_file_copy] = ACTIONS(1747),
    [sym_file_delete] = ACTIONS(1747),
    [sym_file_rename] = ACTIONS(1747),
    [sym_file_dir] = ACTIONS(1747),
    [sym_file_getdir] = ACTIONS(1747),
    [sym_file_setdir] = ACTIONS(1747),
    [sym_file_create_dir] = ACTIONS(1747),
    [sym_file_remove_dir] = ACTIONS(1747),
    [sym_file_seek] = ACTIONS(1747),
    [sym_length_array] = ACTIONS(1747),
    [sym_max_length_array] = ACTIONS(1747),
    [sym_set_length_array] = ACTIONS(1747),
    [sym_ip_client_open] = ACTIONS(1747),
    [sym_ip_client_close] = ACTIONS(1747),
    [sym_ip_server_open] = ACTIONS(1747),
    [sym_ip_server_close] = ACTIONS(1747),
    [sym_ip_bound_client_open] = ACTIONS(1747),
    [sym_ip_mc_server_open] = ACTIONS(1747),
    [sym_ip_set_option] = ACTIONS(1747),
    [sym_ssh_client_open] = ACTIONS(1747),
    [sym_ssh_client_close] = ACTIONS(1747),
    [sym_tls_client_open] = ACTIONS(1747),
    [sym_tls_client_close] = ACTIONS(1747),
    [sym_do_push] = ACTIONS(1747),
    [sym_do_push_timed] = ACTIONS(1747),
    [sym_do_release] = ACTIONS(1747),
    [sym_device_id] = ACTIONS(1747),
    [sym_device_info] = ACTIONS(1747),
    [sym_device_id_string] = ACTIONS(1747),
    [sym_device_standby] = ACTIONS(1747),
    [sym_device_wake] = ACTIONS(1747),
    [sym_timeline_create] = ACTIONS(1747),
    [sym_timeline_kill] = ACTIONS(1747),
    [sym_timeline_reload] = ACTIONS(1747),
    [sym_timeline_pause] = ACTIONS(1747),
    [sym_timeline_restart] = ACTIONS(1747),
    [sym_timeline_set] = ACTIONS(1747),
    [sym_timeline_get] = ACTIONS(1747),
    [sym_timeline_active] = ACTIONS(1747),
    [sym_timeline_dynamic_id] = ACTIONS(1747),
    [sym_variable_to_string] = ACTIONS(1747),
    [sym_string_to_variable] = ACTIONS(1747),
    [sym_length_variable_to_string] = ACTIONS(1747),
    [sym_type_cast] = ACTIONS(1747),
    [sym_variable_to_xml] = ACTIONS(1747),
    [sym_xml_to_variable] = ACTIONS(1747),
    [sym_length_variable_to_xml] = ACTIONS(1747),
    [sym_redirect_string] = ACTIONS(1747),
    [sym_random_number] = ACTIONS(1747),
    [sym_get_unique_id] = ACTIONS(1747),
    [sym_get_system_number] = ACTIONS(1747),
    [sym_set_system_number] = ACTIONS(1747),
    [sym_get_serial_number] = ACTIONS(1747),
    [sym_get_master_build] = ACTIONS(1747),
    [sym_set_validation_code] = ACTIONS(1747),
    [sym_reboot] = ACTIONS(1747),
    [sym_load_duet_module] = ACTIONS(1747),
    [sym_unload_duet_module] = ACTIONS(1747),
    [sym_duet_mem_size_set] = ACTIONS(1747),
    [sym_duet_mem_size_get] = ACTIONS(1747),
    [sym_dynamic_application_device] = ACTIONS(1747),
    [sym_dynamic_polled_port] = ACTIONS(1747),
    [sym_static_port_binding] = ACTIONS(1747),
    [sym_static_ip_binding] = ACTIONS(1747),
    [sym_get_dns_list] = ACTIONS(1747),
    [sym_set_dns_list] = ACTIONS(1747),
    [sym_get_ip_address] = ACTIONS(1747),
    [sym_set_ip_address] = ACTIONS(1747),
    [sym_get_url_list] = ACTIONS(1747),
    [sym_add_url_entry] = ACTIONS(1747),
    [sym_delete_url_entry] = ACTIONS(1747),
    [sym_abs_value] = ACTIONS(1747),
    [sym_max_value] = ACTIONS(1747),
    [sym_min_value] = ACTIONS(1747),
    [sym_exp_value] = ACTIONS(1747),
    [sym_log_value] = ACTIONS(1747),
    [sym_log10_value] = ACTIONS(1747),
    [sym_power_value] = ACTIONS(1747),
    [sym_sqrt_value] = ACTIONS(1747),
    [sym_raw_be] = ACTIONS(1747),
    [sym_raw_le] = ACTIONS(1747),
    [sym_internal_queue_size_set] = ACTIONS(1747),
    [sym_internal_queue_size_get] = ACTIONS(1747),
    [sym_internal_threshold_set] = ACTIONS(1747),
    [sym_internal_threshold_get] = ACTIONS(1747),
    [sym_rebuilt_event] = ACTIONS(1747),
    [sym_do_custom_event] = ACTIONS(1747),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1747),
    [sym_clkmgr_set_clk_source] = ACTIONS(1747),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1747),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1747),
    [sym_clkmgr_get_timezone] = ACTIONS(1747),
    [sym_clkmgr_set_timezone] = ACTIONS(1747),
    [sym_clkmgr_get_resync_period] = ACTIONS(1747),
    [sym_clkmgr_set_resync_period] = ACTIONS(1747),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1747),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1747),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1747),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1747),
    [sym_clkmgr_get_timeservers] = ACTIONS(1747),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1747),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1747),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1747),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1747),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1747),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1747),
    [sym_set_log_level] = ACTIONS(1747),
    [sym_get_log_level] = ACTIONS(1747),
    [sym_amx_log] = ACTIONS(1747),
    [sym_smtp_server_config_set] = ACTIONS(1747),
    [sym_smtp_server_config_get] = ACTIONS(1747),
    [sym_smtp_send] = ACTIONS(1747),
    [sym_validate_netlinx_account] = ACTIONS(1747),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1747),
    [sym_authenticate_certificate] = ACTIONS(1747),
    [sym_audit_netlinx_session_event] = ACTIONS(1747),
    [sym_audit_netlinx_generic_event] = ACTIONS(1747),
    [sym_get_available_flash_disk_space] = ACTIONS(1747),
    [sym_get_max_flash_disk_space] = ACTIONS(1747),
    [sym_data_create_feed] = ACTIONS(1747),
    [sym_data_delete_feed] = ACTIONS(1747),
    [sym_data_publish_feed] = ACTIONS(1747),
    [sym_data_get_published_feed] = ACTIONS(1747),
    [sym_data_add_record] = ACTIONS(1747),
    [sym_data_get_event_record] = ACTIONS(1747),
    [sym__wc_data_create_feed] = ACTIONS(1747),
    [sym__wc_data_add_record] = ACTIONS(1747),
    [sym__wc_data_get_event_record] = ACTIONS(1747),
    [sym_format] = ACTIONS(1747),
    [sym_astro_clock] = ACTIONS(1747),
    [sym_set_outdoor_temperature] = ACTIONS(1747),
    [sym_set_virtual_level_count] = ACTIONS(1747),
    [sym_set_virtual_channel_count] = ACTIONS(1747),
    [sym_set_virtual_port_count] = ACTIONS(1747),
    [sym__wc_encode_int] = ACTIONS(1747),
    [sym__wc_decode_int] = ACTIONS(1747),
    [sym__wc_int] = ACTIONS(1747),
    [sym__wc_to_ch_int] = ACTIONS(1747),
    [sym__ch_to_wc_int] = ACTIONS(1747),
    [sym__wc_find_string_int] = ACTIONS(1747),
    [sym__wc_left_string_int] = ACTIONS(1747),
    [sym__wc_lower_string_int] = ACTIONS(1747),
    [sym__wc_mid_string_int] = ACTIONS(1747),
    [sym__wc_remove_string_int] = ACTIONS(1747),
    [sym__wc_right_string_int] = ACTIONS(1747),
    [sym__wc_upper_string_int] = ACTIONS(1747),
    [sym__wc_compare_string_int] = ACTIONS(1747),
    [sym__wc_get_buffer_char_int] = ACTIONS(1747),
    [sym__wc_get_buffer_string_int] = ACTIONS(1747),
    [sym__wc_concat_string_int] = ACTIONS(1747),
    [sym__wc_file_open_int] = ACTIONS(1747),
    [sym__wc_file_close_int] = ACTIONS(1747),
    [sym__wc_file_read_int] = ACTIONS(1747),
    [sym__wc_file_read_line_int] = ACTIONS(1747),
    [sym__wc_file_write_int] = ACTIONS(1747),
    [sym__wc_file_write_line_int] = ACTIONS(1747),
    [aux_sym_preproc_include_token1] = ACTIONS(1747),
    [aux_sym_preproc_define_token1] = ACTIONS(1749),
    [aux_sym_preproc_warn_token1] = ACTIONS(1749),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1749),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1749),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1749),
    [anon_sym_LBRACK] = ACTIONS(1749),
    [anon_sym_LBRACE] = ACTIONS(1749),
    [anon_sym_LPAREN] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [anon_sym_DASH] = ACTIONS(1747),
    [anon_sym_PLUS] = ACTIONS(1747),
    [anon_sym_DASH_DASH] = ACTIONS(1749),
    [anon_sym_PLUS_PLUS] = ACTIONS(1749),
    [anon_sym_DQUOTE] = ACTIONS(1749),
    [anon_sym_SQUOTE] = ACTIONS(1749),
    [sym_number_literal] = ACTIONS(1749),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(345)] = {
    [sym_identifier] = ACTIONS(2057),
    [sym_program_name_keyword] = ACTIONS(2057),
    [sym_module_name_keyword] = ACTIONS(2057),
    [sym_define_device_keyword] = ACTIONS(2057),
    [sym_define_combine_keyword] = ACTIONS(2057),
    [sym_define_constant_keyword] = ACTIONS(2057),
    [sym_define_type_keyword] = ACTIONS(2057),
    [sym_define_variable_keyword] = ACTIONS(2057),
    [sym_define_system_variable_keyword] = ACTIONS(2057),
    [sym_define_connect_level_keyword] = ACTIONS(2057),
    [sym_define_latching_keyword] = ACTIONS(2057),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2057),
    [sym_define_toggling_keyword] = ACTIONS(2057),
    [sym_define_call_keyword] = ACTIONS(2057),
    [sym_define_function_keyword] = ACTIONS(2057),
    [sym_define_library_function_keyword] = ACTIONS(2057),
    [sym_define_start_keyword] = ACTIONS(2057),
    [sym_define_event_keyword] = ACTIONS(2057),
    [sym_define_program_keyword] = ACTIONS(2057),
    [sym_define_module_keyword] = ACTIONS(2057),
    [sym_char_keyword] = ACTIONS(2057),
    [sym_widechar_keyword] = ACTIONS(2057),
    [sym_integer_keyword] = ACTIONS(2057),
    [sym_sinteger_keyword] = ACTIONS(2057),
    [sym_long_keyword] = ACTIONS(2057),
    [sym_slong_keyword] = ACTIONS(2057),
    [sym_float_keyword] = ACTIONS(2057),
    [sym_double_keyword] = ACTIONS(2057),
    [sym_variant_keyword] = ACTIONS(2057),
    [sym_variantarray_keyword] = ACTIONS(2057),
    [sym_local_var_keyword] = ACTIONS(2057),
    [sym_stack_var_keyword] = ACTIONS(2057),
    [sym_constant_keyword] = ACTIONS(2057),
    [sym_volatile_keyword] = ACTIONS(2057),
    [sym_non_volatile_keyword] = ACTIONS(2057),
    [sym_persistent_keyword] = ACTIONS(2057),
    [sym_dev_keyword] = ACTIONS(2057),
    [sym_devlev_keyword] = ACTIONS(2057),
    [sym_devchan_keyword] = ACTIONS(2057),
    [sym_if_keyword] = ACTIONS(2057),
    [sym_switch_keyword] = ACTIONS(2057),
    [sym_case_keyword] = ACTIONS(2057),
    [sym_default_keyword] = ACTIONS(2057),
    [sym_while_keyword] = ACTIONS(2057),
    [sym_for_keyword] = ACTIONS(2057),
    [sym_break_keyword] = ACTIONS(2057),
    [sym_continue_keyword] = ACTIONS(2057),
    [sym_return_keyword] = ACTIONS(2057),
    [sym_select_keyword] = ACTIONS(2057),
    [sym_active_keyword] = ACTIONS(2057),
    [sym_struct_keyword] = ACTIONS(2057),
    [sym_structure_keyword] = ACTIONS(2057),
    [sym_button_event_keyword] = ACTIONS(2057),
    [sym_channel_event_keyword] = ACTIONS(2057),
    [sym_level_event_keyword] = ACTIONS(2057),
    [sym_data_event_keyword] = ACTIONS(2057),
    [sym_timeline_event_keyword] = ACTIONS(2057),
    [sym_custom_event_keyword] = ACTIONS(2057),
    [sym_push_keyword] = ACTIONS(2057),
    [sym_release_keyword] = ACTIONS(2057),
    [aux_sym_on_keyword_token1] = ACTIONS(2057),
    [aux_sym_off_keyword_token1] = ACTIONS(2057),
    [sym_send_level_keyword] = ACTIONS(2057),
    [sym_send_string_keyword] = ACTIONS(2057),
    [sym_send_command_keyword] = ACTIONS(2057),
    [sym_clear_buffer_keyword] = ACTIONS(2057),
    [sym_create_buffer_keyword] = ACTIONS(2057),
    [sym_create_multi_buffer_keyword] = ACTIONS(2057),
    [sym_call_keyword] = ACTIONS(2057),
    [sym_system_call_keyword] = ACTIONS(2057),
    [sym_devchan_to_keyword] = ACTIONS(2057),
    [sym_devchan_min_to_keyword] = ACTIONS(2057),
    [sym_devchan_total_off_keyword] = ACTIONS(2057),
    [sym_devchan_pulse_keyword] = ACTIONS(2057),
    [sym_wait_keyword] = ACTIONS(2057),
    [sym_cancel_wait_keyword] = ACTIONS(2057),
    [sym_wait_until_keyword] = ACTIONS(2057),
    [sym_cancel_wait_until_keyword] = ACTIONS(2057),
    [sym_cancel_all_wait_keyword] = ACTIONS(2057),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2057),
    [sym_not] = ACTIONS(2057),
    [sym_bnot] = ACTIONS(2057),
    [sym___file__] = ACTIONS(2057),
    [sym___date__] = ACTIONS(2057),
    [sym___ldate__] = ACTIONS(2057),
    [sym___time__] = ACTIONS(2057),
    [sym___line__] = ACTIONS(2057),
    [sym___name__] = ACTIONS(2057),
    [sym___netlinx__] = ACTIONS(2057),
    [sym_netlinx_axi_version] = ACTIONS(2057),
    [sym_url_flg_tcp] = ACTIONS(2057),
    [sym_url_flg_accinfopresent] = ACTIONS(2057),
    [sym_url_flg_temp] = ACTIONS(2057),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2057),
    [sym_url_flg_stat_mask] = ACTIONS(2057),
    [sym_url_flg_stat_lookup] = ACTIONS(2057),
    [sym_url_flg_stat_connecting] = ACTIONS(2057),
    [sym_url_flg_stat_waiting] = ACTIONS(2057),
    [sym_url_flg_stat_connected] = ACTIONS(2057),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2057),
    [sym_first_virtual_device] = ACTIONS(2057),
    [sym_first_local_port] = ACTIONS(2057),
    [sym_dynamic_virtual_device] = ACTIONS(2057),
    [sym_timeline_once] = ACTIONS(2057),
    [sym_timeline_repeat] = ACTIONS(2057),
    [sym_timeline_absolute] = ACTIONS(2057),
    [sym_timeline_relative] = ACTIONS(2057),
    [sym_do_push_timed_infinite] = ACTIONS(2057),
    [sym_source_type_no_address] = ACTIONS(2057),
    [sym_source_type_neuron_id] = ACTIONS(2057),
    [sym_source_type_ip_address] = ACTIONS(2057),
    [sym_source_type_axlink] = ACTIONS(2057),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2057),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2057),
    [sym_source_type_ip_socket_address] = ACTIONS(2057),
    [sym_source_type_rs232] = ACTIONS(2057),
    [sym_source_type_internal] = ACTIONS(2057),
    [sym_source_type_mac_address] = ACTIONS(2057),
    [sym_source_type_ipv4_port] = ACTIONS(2057),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2057),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2057),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2057),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2057),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2057),
    [sym_true] = ACTIONS(2057),
    [sym_false] = ACTIONS(2057),
    [sym_file_read_only] = ACTIONS(2057),
    [sym_file_rw_new] = ACTIONS(2057),
    [sym_file_rw_append] = ACTIONS(2057),
    [sym_ip_tcp] = ACTIONS(2057),
    [sym_ip_udp] = ACTIONS(2057),
    [sym_ip_udp_2way] = ACTIONS(2057),
    [sym_xml_encode_types] = ACTIONS(2057),
    [sym_xml_encode_char_as_list] = ACTIONS(2057),
    [sym_xml_encode_le] = ACTIONS(2057),
    [sym_xml_decode_types] = ACTIONS(2057),
    [sym_xml_decode_no_preserve] = ACTIONS(2057),
    [sym_duet_dev_type_display_device] = ACTIONS(2057),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2057),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2057),
    [sym_duet_dev_type_io_device] = ACTIONS(2057),
    [sym_duet_dev_type_relay_device] = ACTIONS(2057),
    [sym_duet_dev_type_ups] = ACTIONS(2057),
    [sym_duet_dev_type_amplifier] = ACTIONS(2057),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2057),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2057),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2057),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2057),
    [sym_duet_dev_type_camera] = ACTIONS(2057),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2057),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2057),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2057),
    [sym_duet_dev_type_dss] = ACTIONS(2057),
    [sym_duet_dev_type_dvr] = ACTIONS(2057),
    [sym_duet_dev_type_disc_device] = ACTIONS(2057),
    [sym_duet_dev_type_document_camera] = ACTIONS(2057),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2057),
    [sym_duet_dev_type_hvac] = ACTIONS(2057),
    [sym_duet_dev_type_keypad] = ACTIONS(2057),
    [sym_duet_dev_type_light] = ACTIONS(2057),
    [sym_duet_dev_type_monitor] = ACTIONS(2057),
    [sym_duet_dev_type_motor] = ACTIONS(2057),
    [sym_duet_dev_type_multi_window] = ACTIONS(2057),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2057),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2057),
    [sym_duet_dev_type_receiver] = ACTIONS(2057),
    [sym_duet_dev_type_security_system] = ACTIONS(2057),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2057),
    [sym_duet_dev_type_settop_box] = ACTIONS(2057),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2057),
    [sym_duet_dev_type_switcher] = ACTIONS(2057),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2057),
    [sym_duet_dev_type_tv] = ACTIONS(2057),
    [sym_duet_dev_type_utility] = ACTIONS(2057),
    [sym_duet_dev_type_vcr] = ACTIONS(2057),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2057),
    [sym_duet_dev_type_video_processor] = ACTIONS(2057),
    [sym_duet_dev_type_video_projector] = ACTIONS(2057),
    [sym_duet_dev_type_video_wall] = ACTIONS(2057),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2057),
    [sym_duet_dev_type_weather] = ACTIONS(2057),
    [sym_duet_dev_not_polled] = ACTIONS(2057),
    [sym_duet_dev_polled] = ACTIONS(2057),
    [sym_clkmgr_mode_network] = ACTIONS(2057),
    [sym_clkmgr_mode_standalone] = ACTIONS(2057),
    [sym_normal_standby] = ACTIONS(2057),
    [sym_normal_wake] = ACTIONS(2057),
    [sym_ip_multicast_ttl_option] = ACTIONS(2057),
    [sym_ip_ttl_subnet] = ACTIONS(2057),
    [sym_ip_ttl_site] = ACTIONS(2057),
    [sym_ip_ttl_region] = ACTIONS(2057),
    [sym_ip_ttl_continent] = ACTIONS(2057),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2057),
    [sym_ip_nodelay_on] = ACTIONS(2057),
    [sym_ip_nodelay_off] = ACTIONS(2057),
    [sym_amx_error] = ACTIONS(2057),
    [sym_amx_warning] = ACTIONS(2057),
    [sym_amx_info] = ACTIONS(2057),
    [sym_amx_debug] = ACTIONS(2057),
    [sym_netlinx_login_success] = ACTIONS(2057),
    [sym_netlinx_login_fail] = ACTIONS(2057),
    [sym_netlinx_logout] = ACTIONS(2057),
    [sym_null_str] = ACTIONS(2057),
    [sym_smtp_address] = ACTIONS(2057),
    [sym_smtp_port_number] = ACTIONS(2057),
    [sym_smtp_username] = ACTIONS(2057),
    [sym_smtp_password] = ACTIONS(2057),
    [sym_smtp_require_tls] = ACTIONS(2057),
    [sym_smtp_from] = ACTIONS(2057),
    [sym_smtp_tls_true] = ACTIONS(2057),
    [sym_smtp_tls_false] = ACTIONS(2057),
    [sym_listview_on_row_select_event] = ACTIONS(2057),
    [sym_data_structure_unknown] = ACTIONS(2057),
    [sym_data_structure_datafeed] = ACTIONS(2057),
    [sym_data_structure_datarecordset] = ACTIONS(2057),
    [sym_data_structure_datarecord] = ACTIONS(2057),
    [sym_data_structure_datafield] = ACTIONS(2057),
    [sym_data_max_content_field_count] = ACTIONS(2057),
    [sym_data_max_metadata_field_count] = ACTIONS(2057),
    [sym_data_max_id_length] = ACTIONS(2057),
    [sym_data_max_name_length] = ACTIONS(2057),
    [sym_data_max_description_length] = ACTIONS(2057),
    [sym_data_max_type_length] = ACTIONS(2057),
    [sym_data_max_format_length] = ACTIONS(2057),
    [sym_data_max_source_length] = ACTIONS(2057),
    [sym_data_max_value_length] = ACTIONS(2057),
    [sym_data_max_label_length] = ACTIONS(2057),
    [sym_data_max_url_length] = ACTIONS(2057),
    [sym_max_last_login_info_length] = ACTIONS(2057),
    [sym_data_type_unknown] = ACTIONS(2057),
    [sym_data_type_string] = ACTIONS(2057),
    [sym_data_type_datetime] = ACTIONS(2057),
    [sym_data_type_date] = ACTIONS(2057),
    [sym_data_type_time] = ACTIONS(2057),
    [sym_data_type_image] = ACTIONS(2057),
    [sym_data_format_url] = ACTIONS(2057),
    [sym_data_format_phone] = ACTIONS(2057),
    [sym_data_format_email] = ACTIONS(2057),
    [sym_data_format_iso8601] = ACTIONS(2057),
    [sym_auditlog_priv] = ACTIONS(2057),
    [sym_configuration_priv] = ACTIONS(2057),
    [sym_deviceconfig_priv] = ACTIONS(2057),
    [sym_ftp_priv] = ACTIONS(2057),
    [sym_http_priv] = ACTIONS(2057),
    [sym_networkconfig_priv] = ACTIONS(2057),
    [sym_programport_priv] = ACTIONS(2057),
    [sym_remoteui_priv] = ACTIONS(2057),
    [sym_securitycontrol_priv] = ACTIONS(2057),
    [sym_softwaremanagement_priv] = ACTIONS(2057),
    [sym_terminal_priv] = ACTIONS(2057),
    [sym_tpadmin_priv] = ACTIONS(2057),
    [sym_user1_priv] = ACTIONS(2057),
    [sym_user2_priv] = ACTIONS(2057),
    [sym_user3_priv] = ACTIONS(2057),
    [sym_user4_priv] = ACTIONS(2057),
    [sym_usermanagement_priv] = ACTIONS(2057),
    [sym_valid_account] = ACTIONS(2057),
    [sym_err_name_invalid] = ACTIONS(2057),
    [sym_err_password_invalid] = ACTIONS(2057),
    [sym_err_invalid_account] = ACTIONS(2057),
    [sym_err_no_authorization] = ACTIONS(2057),
    [sym_err_invalid_parameter] = ACTIONS(2057),
    [sym_err_user_locked_out] = ACTIONS(2057),
    [sym_err_user_account_expired] = ACTIONS(2057),
    [sym_err_general_error] = ACTIONS(2057),
    [sym_err_authentication_error] = ACTIONS(2057),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2057),
    [sym_tls_validate_certificate] = ACTIONS(2057),
    [sym_tdata] = ACTIONS(2057),
    [sym_tchannel] = ACTIONS(2057),
    [sym_tlevel] = ACTIONS(2057),
    [sym_tbutton] = ACTIONS(2057),
    [sym_ttimeline] = ACTIONS(2057),
    [sym_tcustom] = ACTIONS(2057),
    [sym_url_struct] = ACTIONS(2057),
    [sym_dns_struct] = ACTIONS(2057),
    [sym_ip_address_struct] = ACTIONS(2057),
    [sym_dev_info_struct] = ACTIONS(2057),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2057),
    [sym_data_feed] = ACTIONS(2057),
    [sym_data_field] = ACTIONS(2057),
    [sym_data_record] = ACTIONS(2057),
    [sym_wc_data_feed] = ACTIONS(2057),
    [sym_wc_data_field] = ACTIONS(2057),
    [sym_wc_data_record] = ACTIONS(2057),
    [sym_last_login_info] = ACTIONS(2057),
    [sym____reserved___] = ACTIONS(2057),
    [sym_date] = ACTIONS(2057),
    [sym_day] = ACTIONS(2057),
    [sym_time] = ACTIONS(2057),
    [sym_ldate] = ACTIONS(2057),
    [sym_push_device] = ACTIONS(2057),
    [sym_push_channel] = ACTIONS(2057),
    [sym_push_devchan] = ACTIONS(2057),
    [sym_release_device] = ACTIONS(2057),
    [sym_release_channel] = ACTIONS(2057),
    [sym_release_devchan] = ACTIONS(2057),
    [sym_master_slot] = ACTIONS(2057),
    [sym_get_pulse_time] = ACTIONS(2057),
    [sym_get_timer] = ACTIONS(2057),
    [sym_master_sn] = ACTIONS(2057),
    [sym_system_number] = ACTIONS(2057),
    [sym_button] = ACTIONS(2057),
    [sym_dv_channel] = ACTIONS(2057),
    [sym_data] = ACTIONS(2057),
    [sym_level] = ACTIONS(2057),
    [sym_timeline] = ACTIONS(2057),
    [sym_channel] = ACTIONS(2057),
    [sym_custom] = ACTIONS(2057),
    [sym_atoi] = ACTIONS(2057),
    [sym_atol] = ACTIONS(2057),
    [sym_atof] = ACTIONS(2057),
    [sym_itoa] = ACTIONS(2057),
    [sym_itohex] = ACTIONS(2057),
    [sym_hextoi] = ACTIONS(2057),
    [sym_ftoa] = ACTIONS(2057),
    [sym_find_string] = ACTIONS(2057),
    [sym_left_string] = ACTIONS(2057),
    [sym_right_string] = ACTIONS(2057),
    [sym_mid_string] = ACTIONS(2057),
    [sym_remove_string] = ACTIONS(2057),
    [sym_get_buffer_string] = ACTIONS(2057),
    [sym_get_buffer_char] = ACTIONS(2057),
    [sym_get_multi_buffer_string] = ACTIONS(2057),
    [sym_length_string] = ACTIONS(2057),
    [sym_max_length_string] = ACTIONS(2057),
    [sym_set_length_string] = ACTIONS(2057),
    [sym_compare_string] = ACTIONS(2057),
    [sym_upper_string] = ACTIONS(2057),
    [sym_lower_string] = ACTIONS(2057),
    [sym_get_last] = ACTIONS(2057),
    [sym_date_to_day] = ACTIONS(2057),
    [sym_date_to_month] = ACTIONS(2057),
    [sym_date_to_year] = ACTIONS(2057),
    [sym_time_to_hour] = ACTIONS(2057),
    [sym_time_to_minute] = ACTIONS(2057),
    [sym_time_to_second] = ACTIONS(2057),
    [sym_day_of_week] = ACTIONS(2057),
    [sym_set_pulse_time] = ACTIONS(2057),
    [sym_set_timer] = ACTIONS(2057),
    [sym_file_open] = ACTIONS(2057),
    [sym_file_close] = ACTIONS(2057),
    [sym_file_read] = ACTIONS(2057),
    [sym_file_read_line] = ACTIONS(2057),
    [sym_file_write] = ACTIONS(2057),
    [sym_file_write_line] = ACTIONS(2057),
    [sym_file_copy] = ACTIONS(2057),
    [sym_file_delete] = ACTIONS(2057),
    [sym_file_rename] = ACTIONS(2057),
    [sym_file_dir] = ACTIONS(2057),
    [sym_file_getdir] = ACTIONS(2057),
    [sym_file_setdir] = ACTIONS(2057),
    [sym_file_create_dir] = ACTIONS(2057),
    [sym_file_remove_dir] = ACTIONS(2057),
    [sym_file_seek] = ACTIONS(2057),
    [sym_length_array] = ACTIONS(2057),
    [sym_max_length_array] = ACTIONS(2057),
    [sym_set_length_array] = ACTIONS(2057),
    [sym_ip_client_open] = ACTIONS(2057),
    [sym_ip_client_close] = ACTIONS(2057),
    [sym_ip_server_open] = ACTIONS(2057),
    [sym_ip_server_close] = ACTIONS(2057),
    [sym_ip_bound_client_open] = ACTIONS(2057),
    [sym_ip_mc_server_open] = ACTIONS(2057),
    [sym_ip_set_option] = ACTIONS(2057),
    [sym_ssh_client_open] = ACTIONS(2057),
    [sym_ssh_client_close] = ACTIONS(2057),
    [sym_tls_client_open] = ACTIONS(2057),
    [sym_tls_client_close] = ACTIONS(2057),
    [sym_do_push] = ACTIONS(2057),
    [sym_do_push_timed] = ACTIONS(2057),
    [sym_do_release] = ACTIONS(2057),
    [sym_device_id] = ACTIONS(2057),
    [sym_device_info] = ACTIONS(2057),
    [sym_device_id_string] = ACTIONS(2057),
    [sym_device_standby] = ACTIONS(2057),
    [sym_device_wake] = ACTIONS(2057),
    [sym_timeline_create] = ACTIONS(2057),
    [sym_timeline_kill] = ACTIONS(2057),
    [sym_timeline_reload] = ACTIONS(2057),
    [sym_timeline_pause] = ACTIONS(2057),
    [sym_timeline_restart] = ACTIONS(2057),
    [sym_timeline_set] = ACTIONS(2057),
    [sym_timeline_get] = ACTIONS(2057),
    [sym_timeline_active] = ACTIONS(2057),
    [sym_timeline_dynamic_id] = ACTIONS(2057),
    [sym_variable_to_string] = ACTIONS(2057),
    [sym_string_to_variable] = ACTIONS(2057),
    [sym_length_variable_to_string] = ACTIONS(2057),
    [sym_type_cast] = ACTIONS(2057),
    [sym_variable_to_xml] = ACTIONS(2057),
    [sym_xml_to_variable] = ACTIONS(2057),
    [sym_length_variable_to_xml] = ACTIONS(2057),
    [sym_redirect_string] = ACTIONS(2057),
    [sym_random_number] = ACTIONS(2057),
    [sym_get_unique_id] = ACTIONS(2057),
    [sym_get_system_number] = ACTIONS(2057),
    [sym_set_system_number] = ACTIONS(2057),
    [sym_get_serial_number] = ACTIONS(2057),
    [sym_get_master_build] = ACTIONS(2057),
    [sym_set_validation_code] = ACTIONS(2057),
    [sym_reboot] = ACTIONS(2057),
    [sym_load_duet_module] = ACTIONS(2057),
    [sym_unload_duet_module] = ACTIONS(2057),
    [sym_duet_mem_size_set] = ACTIONS(2057),
    [sym_duet_mem_size_get] = ACTIONS(2057),
    [sym_dynamic_application_device] = ACTIONS(2057),
    [sym_dynamic_polled_port] = ACTIONS(2057),
    [sym_static_port_binding] = ACTIONS(2057),
    [sym_static_ip_binding] = ACTIONS(2057),
    [sym_get_dns_list] = ACTIONS(2057),
    [sym_set_dns_list] = ACTIONS(2057),
    [sym_get_ip_address] = ACTIONS(2057),
    [sym_set_ip_address] = ACTIONS(2057),
    [sym_get_url_list] = ACTIONS(2057),
    [sym_add_url_entry] = ACTIONS(2057),
    [sym_delete_url_entry] = ACTIONS(2057),
    [sym_abs_value] = ACTIONS(2057),
    [sym_max_value] = ACTIONS(2057),
    [sym_min_value] = ACTIONS(2057),
    [sym_exp_value] = ACTIONS(2057),
    [sym_log_value] = ACTIONS(2057),
    [sym_log10_value] = ACTIONS(2057),
    [sym_power_value] = ACTIONS(2057),
    [sym_sqrt_value] = ACTIONS(2057),
    [sym_raw_be] = ACTIONS(2057),
    [sym_raw_le] = ACTIONS(2057),
    [sym_internal_queue_size_set] = ACTIONS(2057),
    [sym_internal_queue_size_get] = ACTIONS(2057),
    [sym_internal_threshold_set] = ACTIONS(2057),
    [sym_internal_threshold_get] = ACTIONS(2057),
    [sym_rebuilt_event] = ACTIONS(2057),
    [sym_do_custom_event] = ACTIONS(2057),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2057),
    [sym_clkmgr_set_clk_source] = ACTIONS(2057),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2057),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2057),
    [sym_clkmgr_get_timezone] = ACTIONS(2057),
    [sym_clkmgr_set_timezone] = ACTIONS(2057),
    [sym_clkmgr_get_resync_period] = ACTIONS(2057),
    [sym_clkmgr_set_resync_period] = ACTIONS(2057),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2057),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2057),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2057),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2057),
    [sym_clkmgr_get_timeservers] = ACTIONS(2057),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2057),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2057),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2057),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2057),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2057),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2057),
    [sym_set_log_level] = ACTIONS(2057),
    [sym_get_log_level] = ACTIONS(2057),
    [sym_amx_log] = ACTIONS(2057),
    [sym_smtp_server_config_set] = ACTIONS(2057),
    [sym_smtp_server_config_get] = ACTIONS(2057),
    [sym_smtp_send] = ACTIONS(2057),
    [sym_validate_netlinx_account] = ACTIONS(2057),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2057),
    [sym_authenticate_certificate] = ACTIONS(2057),
    [sym_audit_netlinx_session_event] = ACTIONS(2057),
    [sym_audit_netlinx_generic_event] = ACTIONS(2057),
    [sym_get_available_flash_disk_space] = ACTIONS(2057),
    [sym_get_max_flash_disk_space] = ACTIONS(2057),
    [sym_data_create_feed] = ACTIONS(2057),
    [sym_data_delete_feed] = ACTIONS(2057),
    [sym_data_publish_feed] = ACTIONS(2057),
    [sym_data_get_published_feed] = ACTIONS(2057),
    [sym_data_add_record] = ACTIONS(2057),
    [sym_data_get_event_record] = ACTIONS(2057),
    [sym__wc_data_create_feed] = ACTIONS(2057),
    [sym__wc_data_add_record] = ACTIONS(2057),
    [sym__wc_data_get_event_record] = ACTIONS(2057),
    [sym_format] = ACTIONS(2057),
    [sym_astro_clock] = ACTIONS(2057),
    [sym_set_outdoor_temperature] = ACTIONS(2057),
    [sym_set_virtual_level_count] = ACTIONS(2057),
    [sym_set_virtual_channel_count] = ACTIONS(2057),
    [sym_set_virtual_port_count] = ACTIONS(2057),
    [sym__wc_encode_int] = ACTIONS(2057),
    [sym__wc_decode_int] = ACTIONS(2057),
    [sym__wc_int] = ACTIONS(2057),
    [sym__wc_to_ch_int] = ACTIONS(2057),
    [sym__ch_to_wc_int] = ACTIONS(2057),
    [sym__wc_find_string_int] = ACTIONS(2057),
    [sym__wc_left_string_int] = ACTIONS(2057),
    [sym__wc_lower_string_int] = ACTIONS(2057),
    [sym__wc_mid_string_int] = ACTIONS(2057),
    [sym__wc_remove_string_int] = ACTIONS(2057),
    [sym__wc_right_string_int] = ACTIONS(2057),
    [sym__wc_upper_string_int] = ACTIONS(2057),
    [sym__wc_compare_string_int] = ACTIONS(2057),
    [sym__wc_get_buffer_char_int] = ACTIONS(2057),
    [sym__wc_get_buffer_string_int] = ACTIONS(2057),
    [sym__wc_concat_string_int] = ACTIONS(2057),
    [sym__wc_file_open_int] = ACTIONS(2057),
    [sym__wc_file_close_int] = ACTIONS(2057),
    [sym__wc_file_read_int] = ACTIONS(2057),
    [sym__wc_file_read_line_int] = ACTIONS(2057),
    [sym__wc_file_write_int] = ACTIONS(2057),
    [sym__wc_file_write_line_int] = ACTIONS(2057),
    [aux_sym_preproc_include_token1] = ACTIONS(2057),
    [aux_sym_preproc_define_token1] = ACTIONS(2059),
    [aux_sym_preproc_warn_token1] = ACTIONS(2059),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2059),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2059),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2059),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2059),
    [aux_sym_preproc_else_token1] = ACTIONS(2059),
    [anon_sym_LBRACK] = ACTIONS(2059),
    [anon_sym_LBRACE] = ACTIONS(2059),
    [anon_sym_LPAREN] = ACTIONS(2057),
    [anon_sym_BANG] = ACTIONS(2059),
    [anon_sym_TILDE] = ACTIONS(2059),
    [anon_sym_DASH] = ACTIONS(2057),
    [anon_sym_PLUS] = ACTIONS(2057),
    [anon_sym_DASH_DASH] = ACTIONS(2059),
    [anon_sym_PLUS_PLUS] = ACTIONS(2059),
    [anon_sym_DQUOTE] = ACTIONS(2059),
    [anon_sym_SQUOTE] = ACTIONS(2059),
    [sym_number_literal] = ACTIONS(2059),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(346)] = {
    [sym_identifier] = ACTIONS(1701),
    [sym_program_name_keyword] = ACTIONS(1701),
    [sym_module_name_keyword] = ACTIONS(1701),
    [sym_define_device_keyword] = ACTIONS(1701),
    [sym_define_combine_keyword] = ACTIONS(1701),
    [sym_define_constant_keyword] = ACTIONS(1701),
    [sym_define_type_keyword] = ACTIONS(1701),
    [sym_define_variable_keyword] = ACTIONS(1701),
    [sym_define_system_variable_keyword] = ACTIONS(1701),
    [sym_define_connect_level_keyword] = ACTIONS(1701),
    [sym_define_latching_keyword] = ACTIONS(1701),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1701),
    [sym_define_toggling_keyword] = ACTIONS(1701),
    [sym_define_call_keyword] = ACTIONS(1701),
    [sym_define_function_keyword] = ACTIONS(1701),
    [sym_define_library_function_keyword] = ACTIONS(1701),
    [sym_define_start_keyword] = ACTIONS(1701),
    [sym_define_event_keyword] = ACTIONS(1701),
    [sym_define_program_keyword] = ACTIONS(1701),
    [sym_define_module_keyword] = ACTIONS(1701),
    [sym_char_keyword] = ACTIONS(1701),
    [sym_widechar_keyword] = ACTIONS(1701),
    [sym_integer_keyword] = ACTIONS(1701),
    [sym_sinteger_keyword] = ACTIONS(1701),
    [sym_long_keyword] = ACTIONS(1701),
    [sym_slong_keyword] = ACTIONS(1701),
    [sym_float_keyword] = ACTIONS(1701),
    [sym_double_keyword] = ACTIONS(1701),
    [sym_variant_keyword] = ACTIONS(1701),
    [sym_variantarray_keyword] = ACTIONS(1701),
    [sym_local_var_keyword] = ACTIONS(1701),
    [sym_stack_var_keyword] = ACTIONS(1701),
    [sym_constant_keyword] = ACTIONS(1701),
    [sym_volatile_keyword] = ACTIONS(1701),
    [sym_non_volatile_keyword] = ACTIONS(1701),
    [sym_persistent_keyword] = ACTIONS(1701),
    [sym_dev_keyword] = ACTIONS(1701),
    [sym_devlev_keyword] = ACTIONS(1701),
    [sym_devchan_keyword] = ACTIONS(1701),
    [sym_if_keyword] = ACTIONS(1701),
    [sym_switch_keyword] = ACTIONS(1701),
    [sym_case_keyword] = ACTIONS(1701),
    [sym_default_keyword] = ACTIONS(1701),
    [sym_while_keyword] = ACTIONS(1701),
    [sym_for_keyword] = ACTIONS(1701),
    [sym_break_keyword] = ACTIONS(1701),
    [sym_continue_keyword] = ACTIONS(1701),
    [sym_return_keyword] = ACTIONS(1701),
    [sym_select_keyword] = ACTIONS(1701),
    [sym_active_keyword] = ACTIONS(1701),
    [sym_struct_keyword] = ACTIONS(1701),
    [sym_structure_keyword] = ACTIONS(1701),
    [sym_button_event_keyword] = ACTIONS(1701),
    [sym_channel_event_keyword] = ACTIONS(1701),
    [sym_level_event_keyword] = ACTIONS(1701),
    [sym_data_event_keyword] = ACTIONS(1701),
    [sym_timeline_event_keyword] = ACTIONS(1701),
    [sym_custom_event_keyword] = ACTIONS(1701),
    [sym_push_keyword] = ACTIONS(1701),
    [sym_release_keyword] = ACTIONS(1701),
    [aux_sym_on_keyword_token1] = ACTIONS(1701),
    [aux_sym_off_keyword_token1] = ACTIONS(1701),
    [sym_send_level_keyword] = ACTIONS(1701),
    [sym_send_string_keyword] = ACTIONS(1701),
    [sym_send_command_keyword] = ACTIONS(1701),
    [sym_clear_buffer_keyword] = ACTIONS(1701),
    [sym_create_buffer_keyword] = ACTIONS(1701),
    [sym_create_multi_buffer_keyword] = ACTIONS(1701),
    [sym_call_keyword] = ACTIONS(1701),
    [sym_system_call_keyword] = ACTIONS(1701),
    [sym_devchan_to_keyword] = ACTIONS(1701),
    [sym_devchan_min_to_keyword] = ACTIONS(1701),
    [sym_devchan_total_off_keyword] = ACTIONS(1701),
    [sym_devchan_pulse_keyword] = ACTIONS(1701),
    [sym_wait_keyword] = ACTIONS(1701),
    [sym_cancel_wait_keyword] = ACTIONS(1701),
    [sym_wait_until_keyword] = ACTIONS(1701),
    [sym_cancel_wait_until_keyword] = ACTIONS(1701),
    [sym_cancel_all_wait_keyword] = ACTIONS(1701),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1701),
    [sym_not] = ACTIONS(1701),
    [sym_bnot] = ACTIONS(1701),
    [sym___file__] = ACTIONS(1701),
    [sym___date__] = ACTIONS(1701),
    [sym___ldate__] = ACTIONS(1701),
    [sym___time__] = ACTIONS(1701),
    [sym___line__] = ACTIONS(1701),
    [sym___name__] = ACTIONS(1701),
    [sym___netlinx__] = ACTIONS(1701),
    [sym_netlinx_axi_version] = ACTIONS(1701),
    [sym_url_flg_tcp] = ACTIONS(1701),
    [sym_url_flg_accinfopresent] = ACTIONS(1701),
    [sym_url_flg_temp] = ACTIONS(1701),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1701),
    [sym_url_flg_stat_mask] = ACTIONS(1701),
    [sym_url_flg_stat_lookup] = ACTIONS(1701),
    [sym_url_flg_stat_connecting] = ACTIONS(1701),
    [sym_url_flg_stat_waiting] = ACTIONS(1701),
    [sym_url_flg_stat_connected] = ACTIONS(1701),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1701),
    [sym_first_virtual_device] = ACTIONS(1701),
    [sym_first_local_port] = ACTIONS(1701),
    [sym_dynamic_virtual_device] = ACTIONS(1701),
    [sym_timeline_once] = ACTIONS(1701),
    [sym_timeline_repeat] = ACTIONS(1701),
    [sym_timeline_absolute] = ACTIONS(1701),
    [sym_timeline_relative] = ACTIONS(1701),
    [sym_do_push_timed_infinite] = ACTIONS(1701),
    [sym_source_type_no_address] = ACTIONS(1701),
    [sym_source_type_neuron_id] = ACTIONS(1701),
    [sym_source_type_ip_address] = ACTIONS(1701),
    [sym_source_type_axlink] = ACTIONS(1701),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1701),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1701),
    [sym_source_type_ip_socket_address] = ACTIONS(1701),
    [sym_source_type_rs232] = ACTIONS(1701),
    [sym_source_type_internal] = ACTIONS(1701),
    [sym_source_type_mac_address] = ACTIONS(1701),
    [sym_source_type_ipv4_port] = ACTIONS(1701),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1701),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1701),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1701),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1701),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1701),
    [sym_true] = ACTIONS(1701),
    [sym_false] = ACTIONS(1701),
    [sym_file_read_only] = ACTIONS(1701),
    [sym_file_rw_new] = ACTIONS(1701),
    [sym_file_rw_append] = ACTIONS(1701),
    [sym_ip_tcp] = ACTIONS(1701),
    [sym_ip_udp] = ACTIONS(1701),
    [sym_ip_udp_2way] = ACTIONS(1701),
    [sym_xml_encode_types] = ACTIONS(1701),
    [sym_xml_encode_char_as_list] = ACTIONS(1701),
    [sym_xml_encode_le] = ACTIONS(1701),
    [sym_xml_decode_types] = ACTIONS(1701),
    [sym_xml_decode_no_preserve] = ACTIONS(1701),
    [sym_duet_dev_type_display_device] = ACTIONS(1701),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1701),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1701),
    [sym_duet_dev_type_io_device] = ACTIONS(1701),
    [sym_duet_dev_type_relay_device] = ACTIONS(1701),
    [sym_duet_dev_type_ups] = ACTIONS(1701),
    [sym_duet_dev_type_amplifier] = ACTIONS(1701),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1701),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1701),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1701),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1701),
    [sym_duet_dev_type_camera] = ACTIONS(1701),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1701),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1701),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1701),
    [sym_duet_dev_type_dss] = ACTIONS(1701),
    [sym_duet_dev_type_dvr] = ACTIONS(1701),
    [sym_duet_dev_type_disc_device] = ACTIONS(1701),
    [sym_duet_dev_type_document_camera] = ACTIONS(1701),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1701),
    [sym_duet_dev_type_hvac] = ACTIONS(1701),
    [sym_duet_dev_type_keypad] = ACTIONS(1701),
    [sym_duet_dev_type_light] = ACTIONS(1701),
    [sym_duet_dev_type_monitor] = ACTIONS(1701),
    [sym_duet_dev_type_motor] = ACTIONS(1701),
    [sym_duet_dev_type_multi_window] = ACTIONS(1701),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1701),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1701),
    [sym_duet_dev_type_receiver] = ACTIONS(1701),
    [sym_duet_dev_type_security_system] = ACTIONS(1701),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1701),
    [sym_duet_dev_type_settop_box] = ACTIONS(1701),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1701),
    [sym_duet_dev_type_switcher] = ACTIONS(1701),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1701),
    [sym_duet_dev_type_tv] = ACTIONS(1701),
    [sym_duet_dev_type_utility] = ACTIONS(1701),
    [sym_duet_dev_type_vcr] = ACTIONS(1701),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1701),
    [sym_duet_dev_type_video_processor] = ACTIONS(1701),
    [sym_duet_dev_type_video_projector] = ACTIONS(1701),
    [sym_duet_dev_type_video_wall] = ACTIONS(1701),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1701),
    [sym_duet_dev_type_weather] = ACTIONS(1701),
    [sym_duet_dev_not_polled] = ACTIONS(1701),
    [sym_duet_dev_polled] = ACTIONS(1701),
    [sym_clkmgr_mode_network] = ACTIONS(1701),
    [sym_clkmgr_mode_standalone] = ACTIONS(1701),
    [sym_normal_standby] = ACTIONS(1701),
    [sym_normal_wake] = ACTIONS(1701),
    [sym_ip_multicast_ttl_option] = ACTIONS(1701),
    [sym_ip_ttl_subnet] = ACTIONS(1701),
    [sym_ip_ttl_site] = ACTIONS(1701),
    [sym_ip_ttl_region] = ACTIONS(1701),
    [sym_ip_ttl_continent] = ACTIONS(1701),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1701),
    [sym_ip_nodelay_on] = ACTIONS(1701),
    [sym_ip_nodelay_off] = ACTIONS(1701),
    [sym_amx_error] = ACTIONS(1701),
    [sym_amx_warning] = ACTIONS(1701),
    [sym_amx_info] = ACTIONS(1701),
    [sym_amx_debug] = ACTIONS(1701),
    [sym_netlinx_login_success] = ACTIONS(1701),
    [sym_netlinx_login_fail] = ACTIONS(1701),
    [sym_netlinx_logout] = ACTIONS(1701),
    [sym_null_str] = ACTIONS(1701),
    [sym_smtp_address] = ACTIONS(1701),
    [sym_smtp_port_number] = ACTIONS(1701),
    [sym_smtp_username] = ACTIONS(1701),
    [sym_smtp_password] = ACTIONS(1701),
    [sym_smtp_require_tls] = ACTIONS(1701),
    [sym_smtp_from] = ACTIONS(1701),
    [sym_smtp_tls_true] = ACTIONS(1701),
    [sym_smtp_tls_false] = ACTIONS(1701),
    [sym_listview_on_row_select_event] = ACTIONS(1701),
    [sym_data_structure_unknown] = ACTIONS(1701),
    [sym_data_structure_datafeed] = ACTIONS(1701),
    [sym_data_structure_datarecordset] = ACTIONS(1701),
    [sym_data_structure_datarecord] = ACTIONS(1701),
    [sym_data_structure_datafield] = ACTIONS(1701),
    [sym_data_max_content_field_count] = ACTIONS(1701),
    [sym_data_max_metadata_field_count] = ACTIONS(1701),
    [sym_data_max_id_length] = ACTIONS(1701),
    [sym_data_max_name_length] = ACTIONS(1701),
    [sym_data_max_description_length] = ACTIONS(1701),
    [sym_data_max_type_length] = ACTIONS(1701),
    [sym_data_max_format_length] = ACTIONS(1701),
    [sym_data_max_source_length] = ACTIONS(1701),
    [sym_data_max_value_length] = ACTIONS(1701),
    [sym_data_max_label_length] = ACTIONS(1701),
    [sym_data_max_url_length] = ACTIONS(1701),
    [sym_max_last_login_info_length] = ACTIONS(1701),
    [sym_data_type_unknown] = ACTIONS(1701),
    [sym_data_type_string] = ACTIONS(1701),
    [sym_data_type_datetime] = ACTIONS(1701),
    [sym_data_type_date] = ACTIONS(1701),
    [sym_data_type_time] = ACTIONS(1701),
    [sym_data_type_image] = ACTIONS(1701),
    [sym_data_format_url] = ACTIONS(1701),
    [sym_data_format_phone] = ACTIONS(1701),
    [sym_data_format_email] = ACTIONS(1701),
    [sym_data_format_iso8601] = ACTIONS(1701),
    [sym_auditlog_priv] = ACTIONS(1701),
    [sym_configuration_priv] = ACTIONS(1701),
    [sym_deviceconfig_priv] = ACTIONS(1701),
    [sym_ftp_priv] = ACTIONS(1701),
    [sym_http_priv] = ACTIONS(1701),
    [sym_networkconfig_priv] = ACTIONS(1701),
    [sym_programport_priv] = ACTIONS(1701),
    [sym_remoteui_priv] = ACTIONS(1701),
    [sym_securitycontrol_priv] = ACTIONS(1701),
    [sym_softwaremanagement_priv] = ACTIONS(1701),
    [sym_terminal_priv] = ACTIONS(1701),
    [sym_tpadmin_priv] = ACTIONS(1701),
    [sym_user1_priv] = ACTIONS(1701),
    [sym_user2_priv] = ACTIONS(1701),
    [sym_user3_priv] = ACTIONS(1701),
    [sym_user4_priv] = ACTIONS(1701),
    [sym_usermanagement_priv] = ACTIONS(1701),
    [sym_valid_account] = ACTIONS(1701),
    [sym_err_name_invalid] = ACTIONS(1701),
    [sym_err_password_invalid] = ACTIONS(1701),
    [sym_err_invalid_account] = ACTIONS(1701),
    [sym_err_no_authorization] = ACTIONS(1701),
    [sym_err_invalid_parameter] = ACTIONS(1701),
    [sym_err_user_locked_out] = ACTIONS(1701),
    [sym_err_user_account_expired] = ACTIONS(1701),
    [sym_err_general_error] = ACTIONS(1701),
    [sym_err_authentication_error] = ACTIONS(1701),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1701),
    [sym_tls_validate_certificate] = ACTIONS(1701),
    [sym_tdata] = ACTIONS(1701),
    [sym_tchannel] = ACTIONS(1701),
    [sym_tlevel] = ACTIONS(1701),
    [sym_tbutton] = ACTIONS(1701),
    [sym_ttimeline] = ACTIONS(1701),
    [sym_tcustom] = ACTIONS(1701),
    [sym_url_struct] = ACTIONS(1701),
    [sym_dns_struct] = ACTIONS(1701),
    [sym_ip_address_struct] = ACTIONS(1701),
    [sym_dev_info_struct] = ACTIONS(1701),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1701),
    [sym_data_feed] = ACTIONS(1701),
    [sym_data_field] = ACTIONS(1701),
    [sym_data_record] = ACTIONS(1701),
    [sym_wc_data_feed] = ACTIONS(1701),
    [sym_wc_data_field] = ACTIONS(1701),
    [sym_wc_data_record] = ACTIONS(1701),
    [sym_last_login_info] = ACTIONS(1701),
    [sym____reserved___] = ACTIONS(1701),
    [sym_date] = ACTIONS(1701),
    [sym_day] = ACTIONS(1701),
    [sym_time] = ACTIONS(1701),
    [sym_ldate] = ACTIONS(1701),
    [sym_push_device] = ACTIONS(1701),
    [sym_push_channel] = ACTIONS(1701),
    [sym_push_devchan] = ACTIONS(1701),
    [sym_release_device] = ACTIONS(1701),
    [sym_release_channel] = ACTIONS(1701),
    [sym_release_devchan] = ACTIONS(1701),
    [sym_master_slot] = ACTIONS(1701),
    [sym_get_pulse_time] = ACTIONS(1701),
    [sym_get_timer] = ACTIONS(1701),
    [sym_master_sn] = ACTIONS(1701),
    [sym_system_number] = ACTIONS(1701),
    [sym_button] = ACTIONS(1701),
    [sym_dv_channel] = ACTIONS(1701),
    [sym_data] = ACTIONS(1701),
    [sym_level] = ACTIONS(1701),
    [sym_timeline] = ACTIONS(1701),
    [sym_channel] = ACTIONS(1701),
    [sym_custom] = ACTIONS(1701),
    [sym_atoi] = ACTIONS(1701),
    [sym_atol] = ACTIONS(1701),
    [sym_atof] = ACTIONS(1701),
    [sym_itoa] = ACTIONS(1701),
    [sym_itohex] = ACTIONS(1701),
    [sym_hextoi] = ACTIONS(1701),
    [sym_ftoa] = ACTIONS(1701),
    [sym_find_string] = ACTIONS(1701),
    [sym_left_string] = ACTIONS(1701),
    [sym_right_string] = ACTIONS(1701),
    [sym_mid_string] = ACTIONS(1701),
    [sym_remove_string] = ACTIONS(1701),
    [sym_get_buffer_string] = ACTIONS(1701),
    [sym_get_buffer_char] = ACTIONS(1701),
    [sym_get_multi_buffer_string] = ACTIONS(1701),
    [sym_length_string] = ACTIONS(1701),
    [sym_max_length_string] = ACTIONS(1701),
    [sym_set_length_string] = ACTIONS(1701),
    [sym_compare_string] = ACTIONS(1701),
    [sym_upper_string] = ACTIONS(1701),
    [sym_lower_string] = ACTIONS(1701),
    [sym_get_last] = ACTIONS(1701),
    [sym_date_to_day] = ACTIONS(1701),
    [sym_date_to_month] = ACTIONS(1701),
    [sym_date_to_year] = ACTIONS(1701),
    [sym_time_to_hour] = ACTIONS(1701),
    [sym_time_to_minute] = ACTIONS(1701),
    [sym_time_to_second] = ACTIONS(1701),
    [sym_day_of_week] = ACTIONS(1701),
    [sym_set_pulse_time] = ACTIONS(1701),
    [sym_set_timer] = ACTIONS(1701),
    [sym_file_open] = ACTIONS(1701),
    [sym_file_close] = ACTIONS(1701),
    [sym_file_read] = ACTIONS(1701),
    [sym_file_read_line] = ACTIONS(1701),
    [sym_file_write] = ACTIONS(1701),
    [sym_file_write_line] = ACTIONS(1701),
    [sym_file_copy] = ACTIONS(1701),
    [sym_file_delete] = ACTIONS(1701),
    [sym_file_rename] = ACTIONS(1701),
    [sym_file_dir] = ACTIONS(1701),
    [sym_file_getdir] = ACTIONS(1701),
    [sym_file_setdir] = ACTIONS(1701),
    [sym_file_create_dir] = ACTIONS(1701),
    [sym_file_remove_dir] = ACTIONS(1701),
    [sym_file_seek] = ACTIONS(1701),
    [sym_length_array] = ACTIONS(1701),
    [sym_max_length_array] = ACTIONS(1701),
    [sym_set_length_array] = ACTIONS(1701),
    [sym_ip_client_open] = ACTIONS(1701),
    [sym_ip_client_close] = ACTIONS(1701),
    [sym_ip_server_open] = ACTIONS(1701),
    [sym_ip_server_close] = ACTIONS(1701),
    [sym_ip_bound_client_open] = ACTIONS(1701),
    [sym_ip_mc_server_open] = ACTIONS(1701),
    [sym_ip_set_option] = ACTIONS(1701),
    [sym_ssh_client_open] = ACTIONS(1701),
    [sym_ssh_client_close] = ACTIONS(1701),
    [sym_tls_client_open] = ACTIONS(1701),
    [sym_tls_client_close] = ACTIONS(1701),
    [sym_do_push] = ACTIONS(1701),
    [sym_do_push_timed] = ACTIONS(1701),
    [sym_do_release] = ACTIONS(1701),
    [sym_device_id] = ACTIONS(1701),
    [sym_device_info] = ACTIONS(1701),
    [sym_device_id_string] = ACTIONS(1701),
    [sym_device_standby] = ACTIONS(1701),
    [sym_device_wake] = ACTIONS(1701),
    [sym_timeline_create] = ACTIONS(1701),
    [sym_timeline_kill] = ACTIONS(1701),
    [sym_timeline_reload] = ACTIONS(1701),
    [sym_timeline_pause] = ACTIONS(1701),
    [sym_timeline_restart] = ACTIONS(1701),
    [sym_timeline_set] = ACTIONS(1701),
    [sym_timeline_get] = ACTIONS(1701),
    [sym_timeline_active] = ACTIONS(1701),
    [sym_timeline_dynamic_id] = ACTIONS(1701),
    [sym_variable_to_string] = ACTIONS(1701),
    [sym_string_to_variable] = ACTIONS(1701),
    [sym_length_variable_to_string] = ACTIONS(1701),
    [sym_type_cast] = ACTIONS(1701),
    [sym_variable_to_xml] = ACTIONS(1701),
    [sym_xml_to_variable] = ACTIONS(1701),
    [sym_length_variable_to_xml] = ACTIONS(1701),
    [sym_redirect_string] = ACTIONS(1701),
    [sym_random_number] = ACTIONS(1701),
    [sym_get_unique_id] = ACTIONS(1701),
    [sym_get_system_number] = ACTIONS(1701),
    [sym_set_system_number] = ACTIONS(1701),
    [sym_get_serial_number] = ACTIONS(1701),
    [sym_get_master_build] = ACTIONS(1701),
    [sym_set_validation_code] = ACTIONS(1701),
    [sym_reboot] = ACTIONS(1701),
    [sym_load_duet_module] = ACTIONS(1701),
    [sym_unload_duet_module] = ACTIONS(1701),
    [sym_duet_mem_size_set] = ACTIONS(1701),
    [sym_duet_mem_size_get] = ACTIONS(1701),
    [sym_dynamic_application_device] = ACTIONS(1701),
    [sym_dynamic_polled_port] = ACTIONS(1701),
    [sym_static_port_binding] = ACTIONS(1701),
    [sym_static_ip_binding] = ACTIONS(1701),
    [sym_get_dns_list] = ACTIONS(1701),
    [sym_set_dns_list] = ACTIONS(1701),
    [sym_get_ip_address] = ACTIONS(1701),
    [sym_set_ip_address] = ACTIONS(1701),
    [sym_get_url_list] = ACTIONS(1701),
    [sym_add_url_entry] = ACTIONS(1701),
    [sym_delete_url_entry] = ACTIONS(1701),
    [sym_abs_value] = ACTIONS(1701),
    [sym_max_value] = ACTIONS(1701),
    [sym_min_value] = ACTIONS(1701),
    [sym_exp_value] = ACTIONS(1701),
    [sym_log_value] = ACTIONS(1701),
    [sym_log10_value] = ACTIONS(1701),
    [sym_power_value] = ACTIONS(1701),
    [sym_sqrt_value] = ACTIONS(1701),
    [sym_raw_be] = ACTIONS(1701),
    [sym_raw_le] = ACTIONS(1701),
    [sym_internal_queue_size_set] = ACTIONS(1701),
    [sym_internal_queue_size_get] = ACTIONS(1701),
    [sym_internal_threshold_set] = ACTIONS(1701),
    [sym_internal_threshold_get] = ACTIONS(1701),
    [sym_rebuilt_event] = ACTIONS(1701),
    [sym_do_custom_event] = ACTIONS(1701),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1701),
    [sym_clkmgr_set_clk_source] = ACTIONS(1701),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1701),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1701),
    [sym_clkmgr_get_timezone] = ACTIONS(1701),
    [sym_clkmgr_set_timezone] = ACTIONS(1701),
    [sym_clkmgr_get_resync_period] = ACTIONS(1701),
    [sym_clkmgr_set_resync_period] = ACTIONS(1701),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1701),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1701),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1701),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1701),
    [sym_clkmgr_get_timeservers] = ACTIONS(1701),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1701),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1701),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1701),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1701),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1701),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1701),
    [sym_set_log_level] = ACTIONS(1701),
    [sym_get_log_level] = ACTIONS(1701),
    [sym_amx_log] = ACTIONS(1701),
    [sym_smtp_server_config_set] = ACTIONS(1701),
    [sym_smtp_server_config_get] = ACTIONS(1701),
    [sym_smtp_send] = ACTIONS(1701),
    [sym_validate_netlinx_account] = ACTIONS(1701),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1701),
    [sym_authenticate_certificate] = ACTIONS(1701),
    [sym_audit_netlinx_session_event] = ACTIONS(1701),
    [sym_audit_netlinx_generic_event] = ACTIONS(1701),
    [sym_get_available_flash_disk_space] = ACTIONS(1701),
    [sym_get_max_flash_disk_space] = ACTIONS(1701),
    [sym_data_create_feed] = ACTIONS(1701),
    [sym_data_delete_feed] = ACTIONS(1701),
    [sym_data_publish_feed] = ACTIONS(1701),
    [sym_data_get_published_feed] = ACTIONS(1701),
    [sym_data_add_record] = ACTIONS(1701),
    [sym_data_get_event_record] = ACTIONS(1701),
    [sym__wc_data_create_feed] = ACTIONS(1701),
    [sym__wc_data_add_record] = ACTIONS(1701),
    [sym__wc_data_get_event_record] = ACTIONS(1701),
    [sym_format] = ACTIONS(1701),
    [sym_astro_clock] = ACTIONS(1701),
    [sym_set_outdoor_temperature] = ACTIONS(1701),
    [sym_set_virtual_level_count] = ACTIONS(1701),
    [sym_set_virtual_channel_count] = ACTIONS(1701),
    [sym_set_virtual_port_count] = ACTIONS(1701),
    [sym__wc_encode_int] = ACTIONS(1701),
    [sym__wc_decode_int] = ACTIONS(1701),
    [sym__wc_int] = ACTIONS(1701),
    [sym__wc_to_ch_int] = ACTIONS(1701),
    [sym__ch_to_wc_int] = ACTIONS(1701),
    [sym__wc_find_string_int] = ACTIONS(1701),
    [sym__wc_left_string_int] = ACTIONS(1701),
    [sym__wc_lower_string_int] = ACTIONS(1701),
    [sym__wc_mid_string_int] = ACTIONS(1701),
    [sym__wc_remove_string_int] = ACTIONS(1701),
    [sym__wc_right_string_int] = ACTIONS(1701),
    [sym__wc_upper_string_int] = ACTIONS(1701),
    [sym__wc_compare_string_int] = ACTIONS(1701),
    [sym__wc_get_buffer_char_int] = ACTIONS(1701),
    [sym__wc_get_buffer_string_int] = ACTIONS(1701),
    [sym__wc_concat_string_int] = ACTIONS(1701),
    [sym__wc_file_open_int] = ACTIONS(1701),
    [sym__wc_file_close_int] = ACTIONS(1701),
    [sym__wc_file_read_int] = ACTIONS(1701),
    [sym__wc_file_read_line_int] = ACTIONS(1701),
    [sym__wc_file_write_int] = ACTIONS(1701),
    [sym__wc_file_write_line_int] = ACTIONS(1701),
    [aux_sym_preproc_include_token1] = ACTIONS(1701),
    [aux_sym_preproc_define_token1] = ACTIONS(1699),
    [aux_sym_preproc_warn_token1] = ACTIONS(1699),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1699),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1699),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1699),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1699),
    [aux_sym_preproc_else_token1] = ACTIONS(1699),
    [anon_sym_LBRACK] = ACTIONS(1699),
    [anon_sym_LBRACE] = ACTIONS(1699),
    [anon_sym_LPAREN] = ACTIONS(1701),
    [anon_sym_BANG] = ACTIONS(1699),
    [anon_sym_TILDE] = ACTIONS(1699),
    [anon_sym_DASH] = ACTIONS(1701),
    [anon_sym_PLUS] = ACTIONS(1701),
    [anon_sym_DASH_DASH] = ACTIONS(1699),
    [anon_sym_PLUS_PLUS] = ACTIONS(1699),
    [anon_sym_DQUOTE] = ACTIONS(1699),
    [anon_sym_SQUOTE] = ACTIONS(1699),
    [sym_number_literal] = ACTIONS(1699),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(347)] = {
    [sym_identifier] = ACTIONS(1709),
    [sym_program_name_keyword] = ACTIONS(1709),
    [sym_module_name_keyword] = ACTIONS(1709),
    [sym_define_device_keyword] = ACTIONS(1709),
    [sym_define_combine_keyword] = ACTIONS(1709),
    [sym_define_constant_keyword] = ACTIONS(1709),
    [sym_define_type_keyword] = ACTIONS(1709),
    [sym_define_variable_keyword] = ACTIONS(1709),
    [sym_define_system_variable_keyword] = ACTIONS(1709),
    [sym_define_connect_level_keyword] = ACTIONS(1709),
    [sym_define_latching_keyword] = ACTIONS(1709),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1709),
    [sym_define_toggling_keyword] = ACTIONS(1709),
    [sym_define_call_keyword] = ACTIONS(1709),
    [sym_define_function_keyword] = ACTIONS(1709),
    [sym_define_library_function_keyword] = ACTIONS(1709),
    [sym_define_start_keyword] = ACTIONS(1709),
    [sym_define_event_keyword] = ACTIONS(1709),
    [sym_define_program_keyword] = ACTIONS(1709),
    [sym_define_module_keyword] = ACTIONS(1709),
    [sym_char_keyword] = ACTIONS(1709),
    [sym_widechar_keyword] = ACTIONS(1709),
    [sym_integer_keyword] = ACTIONS(1709),
    [sym_sinteger_keyword] = ACTIONS(1709),
    [sym_long_keyword] = ACTIONS(1709),
    [sym_slong_keyword] = ACTIONS(1709),
    [sym_float_keyword] = ACTIONS(1709),
    [sym_double_keyword] = ACTIONS(1709),
    [sym_variant_keyword] = ACTIONS(1709),
    [sym_variantarray_keyword] = ACTIONS(1709),
    [sym_local_var_keyword] = ACTIONS(1709),
    [sym_stack_var_keyword] = ACTIONS(1709),
    [sym_constant_keyword] = ACTIONS(1709),
    [sym_volatile_keyword] = ACTIONS(1709),
    [sym_non_volatile_keyword] = ACTIONS(1709),
    [sym_persistent_keyword] = ACTIONS(1709),
    [sym_dev_keyword] = ACTIONS(1709),
    [sym_devlev_keyword] = ACTIONS(1709),
    [sym_devchan_keyword] = ACTIONS(1709),
    [sym_if_keyword] = ACTIONS(1709),
    [sym_switch_keyword] = ACTIONS(1709),
    [sym_case_keyword] = ACTIONS(1709),
    [sym_default_keyword] = ACTIONS(1709),
    [sym_while_keyword] = ACTIONS(1709),
    [sym_for_keyword] = ACTIONS(1709),
    [sym_break_keyword] = ACTIONS(1709),
    [sym_continue_keyword] = ACTIONS(1709),
    [sym_return_keyword] = ACTIONS(1709),
    [sym_select_keyword] = ACTIONS(1709),
    [sym_active_keyword] = ACTIONS(1709),
    [sym_struct_keyword] = ACTIONS(1709),
    [sym_structure_keyword] = ACTIONS(1709),
    [sym_button_event_keyword] = ACTIONS(1709),
    [sym_channel_event_keyword] = ACTIONS(1709),
    [sym_level_event_keyword] = ACTIONS(1709),
    [sym_data_event_keyword] = ACTIONS(1709),
    [sym_timeline_event_keyword] = ACTIONS(1709),
    [sym_custom_event_keyword] = ACTIONS(1709),
    [sym_push_keyword] = ACTIONS(1709),
    [sym_release_keyword] = ACTIONS(1709),
    [aux_sym_on_keyword_token1] = ACTIONS(1709),
    [aux_sym_off_keyword_token1] = ACTIONS(1709),
    [sym_send_level_keyword] = ACTIONS(1709),
    [sym_send_string_keyword] = ACTIONS(1709),
    [sym_send_command_keyword] = ACTIONS(1709),
    [sym_clear_buffer_keyword] = ACTIONS(1709),
    [sym_create_buffer_keyword] = ACTIONS(1709),
    [sym_create_multi_buffer_keyword] = ACTIONS(1709),
    [sym_call_keyword] = ACTIONS(1709),
    [sym_system_call_keyword] = ACTIONS(1709),
    [sym_devchan_to_keyword] = ACTIONS(1709),
    [sym_devchan_min_to_keyword] = ACTIONS(1709),
    [sym_devchan_total_off_keyword] = ACTIONS(1709),
    [sym_devchan_pulse_keyword] = ACTIONS(1709),
    [sym_wait_keyword] = ACTIONS(1709),
    [sym_cancel_wait_keyword] = ACTIONS(1709),
    [sym_wait_until_keyword] = ACTIONS(1709),
    [sym_cancel_wait_until_keyword] = ACTIONS(1709),
    [sym_cancel_all_wait_keyword] = ACTIONS(1709),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1709),
    [sym_not] = ACTIONS(1709),
    [sym_bnot] = ACTIONS(1709),
    [sym___file__] = ACTIONS(1709),
    [sym___date__] = ACTIONS(1709),
    [sym___ldate__] = ACTIONS(1709),
    [sym___time__] = ACTIONS(1709),
    [sym___line__] = ACTIONS(1709),
    [sym___name__] = ACTIONS(1709),
    [sym___netlinx__] = ACTIONS(1709),
    [sym_netlinx_axi_version] = ACTIONS(1709),
    [sym_url_flg_tcp] = ACTIONS(1709),
    [sym_url_flg_accinfopresent] = ACTIONS(1709),
    [sym_url_flg_temp] = ACTIONS(1709),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1709),
    [sym_url_flg_stat_mask] = ACTIONS(1709),
    [sym_url_flg_stat_lookup] = ACTIONS(1709),
    [sym_url_flg_stat_connecting] = ACTIONS(1709),
    [sym_url_flg_stat_waiting] = ACTIONS(1709),
    [sym_url_flg_stat_connected] = ACTIONS(1709),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1709),
    [sym_first_virtual_device] = ACTIONS(1709),
    [sym_first_local_port] = ACTIONS(1709),
    [sym_dynamic_virtual_device] = ACTIONS(1709),
    [sym_timeline_once] = ACTIONS(1709),
    [sym_timeline_repeat] = ACTIONS(1709),
    [sym_timeline_absolute] = ACTIONS(1709),
    [sym_timeline_relative] = ACTIONS(1709),
    [sym_do_push_timed_infinite] = ACTIONS(1709),
    [sym_source_type_no_address] = ACTIONS(1709),
    [sym_source_type_neuron_id] = ACTIONS(1709),
    [sym_source_type_ip_address] = ACTIONS(1709),
    [sym_source_type_axlink] = ACTIONS(1709),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1709),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1709),
    [sym_source_type_ip_socket_address] = ACTIONS(1709),
    [sym_source_type_rs232] = ACTIONS(1709),
    [sym_source_type_internal] = ACTIONS(1709),
    [sym_source_type_mac_address] = ACTIONS(1709),
    [sym_source_type_ipv4_port] = ACTIONS(1709),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1709),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1709),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1709),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1709),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1709),
    [sym_true] = ACTIONS(1709),
    [sym_false] = ACTIONS(1709),
    [sym_file_read_only] = ACTIONS(1709),
    [sym_file_rw_new] = ACTIONS(1709),
    [sym_file_rw_append] = ACTIONS(1709),
    [sym_ip_tcp] = ACTIONS(1709),
    [sym_ip_udp] = ACTIONS(1709),
    [sym_ip_udp_2way] = ACTIONS(1709),
    [sym_xml_encode_types] = ACTIONS(1709),
    [sym_xml_encode_char_as_list] = ACTIONS(1709),
    [sym_xml_encode_le] = ACTIONS(1709),
    [sym_xml_decode_types] = ACTIONS(1709),
    [sym_xml_decode_no_preserve] = ACTIONS(1709),
    [sym_duet_dev_type_display_device] = ACTIONS(1709),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1709),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1709),
    [sym_duet_dev_type_io_device] = ACTIONS(1709),
    [sym_duet_dev_type_relay_device] = ACTIONS(1709),
    [sym_duet_dev_type_ups] = ACTIONS(1709),
    [sym_duet_dev_type_amplifier] = ACTIONS(1709),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1709),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1709),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1709),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1709),
    [sym_duet_dev_type_camera] = ACTIONS(1709),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1709),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1709),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1709),
    [sym_duet_dev_type_dss] = ACTIONS(1709),
    [sym_duet_dev_type_dvr] = ACTIONS(1709),
    [sym_duet_dev_type_disc_device] = ACTIONS(1709),
    [sym_duet_dev_type_document_camera] = ACTIONS(1709),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1709),
    [sym_duet_dev_type_hvac] = ACTIONS(1709),
    [sym_duet_dev_type_keypad] = ACTIONS(1709),
    [sym_duet_dev_type_light] = ACTIONS(1709),
    [sym_duet_dev_type_monitor] = ACTIONS(1709),
    [sym_duet_dev_type_motor] = ACTIONS(1709),
    [sym_duet_dev_type_multi_window] = ACTIONS(1709),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1709),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1709),
    [sym_duet_dev_type_receiver] = ACTIONS(1709),
    [sym_duet_dev_type_security_system] = ACTIONS(1709),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1709),
    [sym_duet_dev_type_settop_box] = ACTIONS(1709),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1709),
    [sym_duet_dev_type_switcher] = ACTIONS(1709),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1709),
    [sym_duet_dev_type_tv] = ACTIONS(1709),
    [sym_duet_dev_type_utility] = ACTIONS(1709),
    [sym_duet_dev_type_vcr] = ACTIONS(1709),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1709),
    [sym_duet_dev_type_video_processor] = ACTIONS(1709),
    [sym_duet_dev_type_video_projector] = ACTIONS(1709),
    [sym_duet_dev_type_video_wall] = ACTIONS(1709),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1709),
    [sym_duet_dev_type_weather] = ACTIONS(1709),
    [sym_duet_dev_not_polled] = ACTIONS(1709),
    [sym_duet_dev_polled] = ACTIONS(1709),
    [sym_clkmgr_mode_network] = ACTIONS(1709),
    [sym_clkmgr_mode_standalone] = ACTIONS(1709),
    [sym_normal_standby] = ACTIONS(1709),
    [sym_normal_wake] = ACTIONS(1709),
    [sym_ip_multicast_ttl_option] = ACTIONS(1709),
    [sym_ip_ttl_subnet] = ACTIONS(1709),
    [sym_ip_ttl_site] = ACTIONS(1709),
    [sym_ip_ttl_region] = ACTIONS(1709),
    [sym_ip_ttl_continent] = ACTIONS(1709),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1709),
    [sym_ip_nodelay_on] = ACTIONS(1709),
    [sym_ip_nodelay_off] = ACTIONS(1709),
    [sym_amx_error] = ACTIONS(1709),
    [sym_amx_warning] = ACTIONS(1709),
    [sym_amx_info] = ACTIONS(1709),
    [sym_amx_debug] = ACTIONS(1709),
    [sym_netlinx_login_success] = ACTIONS(1709),
    [sym_netlinx_login_fail] = ACTIONS(1709),
    [sym_netlinx_logout] = ACTIONS(1709),
    [sym_null_str] = ACTIONS(1709),
    [sym_smtp_address] = ACTIONS(1709),
    [sym_smtp_port_number] = ACTIONS(1709),
    [sym_smtp_username] = ACTIONS(1709),
    [sym_smtp_password] = ACTIONS(1709),
    [sym_smtp_require_tls] = ACTIONS(1709),
    [sym_smtp_from] = ACTIONS(1709),
    [sym_smtp_tls_true] = ACTIONS(1709),
    [sym_smtp_tls_false] = ACTIONS(1709),
    [sym_listview_on_row_select_event] = ACTIONS(1709),
    [sym_data_structure_unknown] = ACTIONS(1709),
    [sym_data_structure_datafeed] = ACTIONS(1709),
    [sym_data_structure_datarecordset] = ACTIONS(1709),
    [sym_data_structure_datarecord] = ACTIONS(1709),
    [sym_data_structure_datafield] = ACTIONS(1709),
    [sym_data_max_content_field_count] = ACTIONS(1709),
    [sym_data_max_metadata_field_count] = ACTIONS(1709),
    [sym_data_max_id_length] = ACTIONS(1709),
    [sym_data_max_name_length] = ACTIONS(1709),
    [sym_data_max_description_length] = ACTIONS(1709),
    [sym_data_max_type_length] = ACTIONS(1709),
    [sym_data_max_format_length] = ACTIONS(1709),
    [sym_data_max_source_length] = ACTIONS(1709),
    [sym_data_max_value_length] = ACTIONS(1709),
    [sym_data_max_label_length] = ACTIONS(1709),
    [sym_data_max_url_length] = ACTIONS(1709),
    [sym_max_last_login_info_length] = ACTIONS(1709),
    [sym_data_type_unknown] = ACTIONS(1709),
    [sym_data_type_string] = ACTIONS(1709),
    [sym_data_type_datetime] = ACTIONS(1709),
    [sym_data_type_date] = ACTIONS(1709),
    [sym_data_type_time] = ACTIONS(1709),
    [sym_data_type_image] = ACTIONS(1709),
    [sym_data_format_url] = ACTIONS(1709),
    [sym_data_format_phone] = ACTIONS(1709),
    [sym_data_format_email] = ACTIONS(1709),
    [sym_data_format_iso8601] = ACTIONS(1709),
    [sym_auditlog_priv] = ACTIONS(1709),
    [sym_configuration_priv] = ACTIONS(1709),
    [sym_deviceconfig_priv] = ACTIONS(1709),
    [sym_ftp_priv] = ACTIONS(1709),
    [sym_http_priv] = ACTIONS(1709),
    [sym_networkconfig_priv] = ACTIONS(1709),
    [sym_programport_priv] = ACTIONS(1709),
    [sym_remoteui_priv] = ACTIONS(1709),
    [sym_securitycontrol_priv] = ACTIONS(1709),
    [sym_softwaremanagement_priv] = ACTIONS(1709),
    [sym_terminal_priv] = ACTIONS(1709),
    [sym_tpadmin_priv] = ACTIONS(1709),
    [sym_user1_priv] = ACTIONS(1709),
    [sym_user2_priv] = ACTIONS(1709),
    [sym_user3_priv] = ACTIONS(1709),
    [sym_user4_priv] = ACTIONS(1709),
    [sym_usermanagement_priv] = ACTIONS(1709),
    [sym_valid_account] = ACTIONS(1709),
    [sym_err_name_invalid] = ACTIONS(1709),
    [sym_err_password_invalid] = ACTIONS(1709),
    [sym_err_invalid_account] = ACTIONS(1709),
    [sym_err_no_authorization] = ACTIONS(1709),
    [sym_err_invalid_parameter] = ACTIONS(1709),
    [sym_err_user_locked_out] = ACTIONS(1709),
    [sym_err_user_account_expired] = ACTIONS(1709),
    [sym_err_general_error] = ACTIONS(1709),
    [sym_err_authentication_error] = ACTIONS(1709),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1709),
    [sym_tls_validate_certificate] = ACTIONS(1709),
    [sym_tdata] = ACTIONS(1709),
    [sym_tchannel] = ACTIONS(1709),
    [sym_tlevel] = ACTIONS(1709),
    [sym_tbutton] = ACTIONS(1709),
    [sym_ttimeline] = ACTIONS(1709),
    [sym_tcustom] = ACTIONS(1709),
    [sym_url_struct] = ACTIONS(1709),
    [sym_dns_struct] = ACTIONS(1709),
    [sym_ip_address_struct] = ACTIONS(1709),
    [sym_dev_info_struct] = ACTIONS(1709),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1709),
    [sym_data_feed] = ACTIONS(1709),
    [sym_data_field] = ACTIONS(1709),
    [sym_data_record] = ACTIONS(1709),
    [sym_wc_data_feed] = ACTIONS(1709),
    [sym_wc_data_field] = ACTIONS(1709),
    [sym_wc_data_record] = ACTIONS(1709),
    [sym_last_login_info] = ACTIONS(1709),
    [sym____reserved___] = ACTIONS(1709),
    [sym_date] = ACTIONS(1709),
    [sym_day] = ACTIONS(1709),
    [sym_time] = ACTIONS(1709),
    [sym_ldate] = ACTIONS(1709),
    [sym_push_device] = ACTIONS(1709),
    [sym_push_channel] = ACTIONS(1709),
    [sym_push_devchan] = ACTIONS(1709),
    [sym_release_device] = ACTIONS(1709),
    [sym_release_channel] = ACTIONS(1709),
    [sym_release_devchan] = ACTIONS(1709),
    [sym_master_slot] = ACTIONS(1709),
    [sym_get_pulse_time] = ACTIONS(1709),
    [sym_get_timer] = ACTIONS(1709),
    [sym_master_sn] = ACTIONS(1709),
    [sym_system_number] = ACTIONS(1709),
    [sym_button] = ACTIONS(1709),
    [sym_dv_channel] = ACTIONS(1709),
    [sym_data] = ACTIONS(1709),
    [sym_level] = ACTIONS(1709),
    [sym_timeline] = ACTIONS(1709),
    [sym_channel] = ACTIONS(1709),
    [sym_custom] = ACTIONS(1709),
    [sym_atoi] = ACTIONS(1709),
    [sym_atol] = ACTIONS(1709),
    [sym_atof] = ACTIONS(1709),
    [sym_itoa] = ACTIONS(1709),
    [sym_itohex] = ACTIONS(1709),
    [sym_hextoi] = ACTIONS(1709),
    [sym_ftoa] = ACTIONS(1709),
    [sym_find_string] = ACTIONS(1709),
    [sym_left_string] = ACTIONS(1709),
    [sym_right_string] = ACTIONS(1709),
    [sym_mid_string] = ACTIONS(1709),
    [sym_remove_string] = ACTIONS(1709),
    [sym_get_buffer_string] = ACTIONS(1709),
    [sym_get_buffer_char] = ACTIONS(1709),
    [sym_get_multi_buffer_string] = ACTIONS(1709),
    [sym_length_string] = ACTIONS(1709),
    [sym_max_length_string] = ACTIONS(1709),
    [sym_set_length_string] = ACTIONS(1709),
    [sym_compare_string] = ACTIONS(1709),
    [sym_upper_string] = ACTIONS(1709),
    [sym_lower_string] = ACTIONS(1709),
    [sym_get_last] = ACTIONS(1709),
    [sym_date_to_day] = ACTIONS(1709),
    [sym_date_to_month] = ACTIONS(1709),
    [sym_date_to_year] = ACTIONS(1709),
    [sym_time_to_hour] = ACTIONS(1709),
    [sym_time_to_minute] = ACTIONS(1709),
    [sym_time_to_second] = ACTIONS(1709),
    [sym_day_of_week] = ACTIONS(1709),
    [sym_set_pulse_time] = ACTIONS(1709),
    [sym_set_timer] = ACTIONS(1709),
    [sym_file_open] = ACTIONS(1709),
    [sym_file_close] = ACTIONS(1709),
    [sym_file_read] = ACTIONS(1709),
    [sym_file_read_line] = ACTIONS(1709),
    [sym_file_write] = ACTIONS(1709),
    [sym_file_write_line] = ACTIONS(1709),
    [sym_file_copy] = ACTIONS(1709),
    [sym_file_delete] = ACTIONS(1709),
    [sym_file_rename] = ACTIONS(1709),
    [sym_file_dir] = ACTIONS(1709),
    [sym_file_getdir] = ACTIONS(1709),
    [sym_file_setdir] = ACTIONS(1709),
    [sym_file_create_dir] = ACTIONS(1709),
    [sym_file_remove_dir] = ACTIONS(1709),
    [sym_file_seek] = ACTIONS(1709),
    [sym_length_array] = ACTIONS(1709),
    [sym_max_length_array] = ACTIONS(1709),
    [sym_set_length_array] = ACTIONS(1709),
    [sym_ip_client_open] = ACTIONS(1709),
    [sym_ip_client_close] = ACTIONS(1709),
    [sym_ip_server_open] = ACTIONS(1709),
    [sym_ip_server_close] = ACTIONS(1709),
    [sym_ip_bound_client_open] = ACTIONS(1709),
    [sym_ip_mc_server_open] = ACTIONS(1709),
    [sym_ip_set_option] = ACTIONS(1709),
    [sym_ssh_client_open] = ACTIONS(1709),
    [sym_ssh_client_close] = ACTIONS(1709),
    [sym_tls_client_open] = ACTIONS(1709),
    [sym_tls_client_close] = ACTIONS(1709),
    [sym_do_push] = ACTIONS(1709),
    [sym_do_push_timed] = ACTIONS(1709),
    [sym_do_release] = ACTIONS(1709),
    [sym_device_id] = ACTIONS(1709),
    [sym_device_info] = ACTIONS(1709),
    [sym_device_id_string] = ACTIONS(1709),
    [sym_device_standby] = ACTIONS(1709),
    [sym_device_wake] = ACTIONS(1709),
    [sym_timeline_create] = ACTIONS(1709),
    [sym_timeline_kill] = ACTIONS(1709),
    [sym_timeline_reload] = ACTIONS(1709),
    [sym_timeline_pause] = ACTIONS(1709),
    [sym_timeline_restart] = ACTIONS(1709),
    [sym_timeline_set] = ACTIONS(1709),
    [sym_timeline_get] = ACTIONS(1709),
    [sym_timeline_active] = ACTIONS(1709),
    [sym_timeline_dynamic_id] = ACTIONS(1709),
    [sym_variable_to_string] = ACTIONS(1709),
    [sym_string_to_variable] = ACTIONS(1709),
    [sym_length_variable_to_string] = ACTIONS(1709),
    [sym_type_cast] = ACTIONS(1709),
    [sym_variable_to_xml] = ACTIONS(1709),
    [sym_xml_to_variable] = ACTIONS(1709),
    [sym_length_variable_to_xml] = ACTIONS(1709),
    [sym_redirect_string] = ACTIONS(1709),
    [sym_random_number] = ACTIONS(1709),
    [sym_get_unique_id] = ACTIONS(1709),
    [sym_get_system_number] = ACTIONS(1709),
    [sym_set_system_number] = ACTIONS(1709),
    [sym_get_serial_number] = ACTIONS(1709),
    [sym_get_master_build] = ACTIONS(1709),
    [sym_set_validation_code] = ACTIONS(1709),
    [sym_reboot] = ACTIONS(1709),
    [sym_load_duet_module] = ACTIONS(1709),
    [sym_unload_duet_module] = ACTIONS(1709),
    [sym_duet_mem_size_set] = ACTIONS(1709),
    [sym_duet_mem_size_get] = ACTIONS(1709),
    [sym_dynamic_application_device] = ACTIONS(1709),
    [sym_dynamic_polled_port] = ACTIONS(1709),
    [sym_static_port_binding] = ACTIONS(1709),
    [sym_static_ip_binding] = ACTIONS(1709),
    [sym_get_dns_list] = ACTIONS(1709),
    [sym_set_dns_list] = ACTIONS(1709),
    [sym_get_ip_address] = ACTIONS(1709),
    [sym_set_ip_address] = ACTIONS(1709),
    [sym_get_url_list] = ACTIONS(1709),
    [sym_add_url_entry] = ACTIONS(1709),
    [sym_delete_url_entry] = ACTIONS(1709),
    [sym_abs_value] = ACTIONS(1709),
    [sym_max_value] = ACTIONS(1709),
    [sym_min_value] = ACTIONS(1709),
    [sym_exp_value] = ACTIONS(1709),
    [sym_log_value] = ACTIONS(1709),
    [sym_log10_value] = ACTIONS(1709),
    [sym_power_value] = ACTIONS(1709),
    [sym_sqrt_value] = ACTIONS(1709),
    [sym_raw_be] = ACTIONS(1709),
    [sym_raw_le] = ACTIONS(1709),
    [sym_internal_queue_size_set] = ACTIONS(1709),
    [sym_internal_queue_size_get] = ACTIONS(1709),
    [sym_internal_threshold_set] = ACTIONS(1709),
    [sym_internal_threshold_get] = ACTIONS(1709),
    [sym_rebuilt_event] = ACTIONS(1709),
    [sym_do_custom_event] = ACTIONS(1709),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1709),
    [sym_clkmgr_set_clk_source] = ACTIONS(1709),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1709),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1709),
    [sym_clkmgr_get_timezone] = ACTIONS(1709),
    [sym_clkmgr_set_timezone] = ACTIONS(1709),
    [sym_clkmgr_get_resync_period] = ACTIONS(1709),
    [sym_clkmgr_set_resync_period] = ACTIONS(1709),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1709),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1709),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1709),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1709),
    [sym_clkmgr_get_timeservers] = ACTIONS(1709),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1709),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1709),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1709),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1709),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1709),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1709),
    [sym_set_log_level] = ACTIONS(1709),
    [sym_get_log_level] = ACTIONS(1709),
    [sym_amx_log] = ACTIONS(1709),
    [sym_smtp_server_config_set] = ACTIONS(1709),
    [sym_smtp_server_config_get] = ACTIONS(1709),
    [sym_smtp_send] = ACTIONS(1709),
    [sym_validate_netlinx_account] = ACTIONS(1709),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1709),
    [sym_authenticate_certificate] = ACTIONS(1709),
    [sym_audit_netlinx_session_event] = ACTIONS(1709),
    [sym_audit_netlinx_generic_event] = ACTIONS(1709),
    [sym_get_available_flash_disk_space] = ACTIONS(1709),
    [sym_get_max_flash_disk_space] = ACTIONS(1709),
    [sym_data_create_feed] = ACTIONS(1709),
    [sym_data_delete_feed] = ACTIONS(1709),
    [sym_data_publish_feed] = ACTIONS(1709),
    [sym_data_get_published_feed] = ACTIONS(1709),
    [sym_data_add_record] = ACTIONS(1709),
    [sym_data_get_event_record] = ACTIONS(1709),
    [sym__wc_data_create_feed] = ACTIONS(1709),
    [sym__wc_data_add_record] = ACTIONS(1709),
    [sym__wc_data_get_event_record] = ACTIONS(1709),
    [sym_format] = ACTIONS(1709),
    [sym_astro_clock] = ACTIONS(1709),
    [sym_set_outdoor_temperature] = ACTIONS(1709),
    [sym_set_virtual_level_count] = ACTIONS(1709),
    [sym_set_virtual_channel_count] = ACTIONS(1709),
    [sym_set_virtual_port_count] = ACTIONS(1709),
    [sym__wc_encode_int] = ACTIONS(1709),
    [sym__wc_decode_int] = ACTIONS(1709),
    [sym__wc_int] = ACTIONS(1709),
    [sym__wc_to_ch_int] = ACTIONS(1709),
    [sym__ch_to_wc_int] = ACTIONS(1709),
    [sym__wc_find_string_int] = ACTIONS(1709),
    [sym__wc_left_string_int] = ACTIONS(1709),
    [sym__wc_lower_string_int] = ACTIONS(1709),
    [sym__wc_mid_string_int] = ACTIONS(1709),
    [sym__wc_remove_string_int] = ACTIONS(1709),
    [sym__wc_right_string_int] = ACTIONS(1709),
    [sym__wc_upper_string_int] = ACTIONS(1709),
    [sym__wc_compare_string_int] = ACTIONS(1709),
    [sym__wc_get_buffer_char_int] = ACTIONS(1709),
    [sym__wc_get_buffer_string_int] = ACTIONS(1709),
    [sym__wc_concat_string_int] = ACTIONS(1709),
    [sym__wc_file_open_int] = ACTIONS(1709),
    [sym__wc_file_close_int] = ACTIONS(1709),
    [sym__wc_file_read_int] = ACTIONS(1709),
    [sym__wc_file_read_line_int] = ACTIONS(1709),
    [sym__wc_file_write_int] = ACTIONS(1709),
    [sym__wc_file_write_line_int] = ACTIONS(1709),
    [aux_sym_preproc_include_token1] = ACTIONS(1709),
    [aux_sym_preproc_define_token1] = ACTIONS(1707),
    [aux_sym_preproc_warn_token1] = ACTIONS(1707),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1707),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1707),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1707),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1707),
    [aux_sym_preproc_else_token1] = ACTIONS(1707),
    [anon_sym_LBRACK] = ACTIONS(1707),
    [anon_sym_LBRACE] = ACTIONS(1707),
    [anon_sym_LPAREN] = ACTIONS(1709),
    [anon_sym_BANG] = ACTIONS(1707),
    [anon_sym_TILDE] = ACTIONS(1707),
    [anon_sym_DASH] = ACTIONS(1709),
    [anon_sym_PLUS] = ACTIONS(1709),
    [anon_sym_DASH_DASH] = ACTIONS(1707),
    [anon_sym_PLUS_PLUS] = ACTIONS(1707),
    [anon_sym_DQUOTE] = ACTIONS(1707),
    [anon_sym_SQUOTE] = ACTIONS(1707),
    [sym_number_literal] = ACTIONS(1707),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(348)] = {
    [sym_identifier] = ACTIONS(1705),
    [sym_program_name_keyword] = ACTIONS(1705),
    [sym_module_name_keyword] = ACTIONS(1705),
    [sym_define_device_keyword] = ACTIONS(1705),
    [sym_define_combine_keyword] = ACTIONS(1705),
    [sym_define_constant_keyword] = ACTIONS(1705),
    [sym_define_type_keyword] = ACTIONS(1705),
    [sym_define_variable_keyword] = ACTIONS(1705),
    [sym_define_system_variable_keyword] = ACTIONS(1705),
    [sym_define_connect_level_keyword] = ACTIONS(1705),
    [sym_define_latching_keyword] = ACTIONS(1705),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1705),
    [sym_define_toggling_keyword] = ACTIONS(1705),
    [sym_define_call_keyword] = ACTIONS(1705),
    [sym_define_function_keyword] = ACTIONS(1705),
    [sym_define_library_function_keyword] = ACTIONS(1705),
    [sym_define_start_keyword] = ACTIONS(1705),
    [sym_define_event_keyword] = ACTIONS(1705),
    [sym_define_program_keyword] = ACTIONS(1705),
    [sym_define_module_keyword] = ACTIONS(1705),
    [sym_char_keyword] = ACTIONS(1705),
    [sym_widechar_keyword] = ACTIONS(1705),
    [sym_integer_keyword] = ACTIONS(1705),
    [sym_sinteger_keyword] = ACTIONS(1705),
    [sym_long_keyword] = ACTIONS(1705),
    [sym_slong_keyword] = ACTIONS(1705),
    [sym_float_keyword] = ACTIONS(1705),
    [sym_double_keyword] = ACTIONS(1705),
    [sym_variant_keyword] = ACTIONS(1705),
    [sym_variantarray_keyword] = ACTIONS(1705),
    [sym_local_var_keyword] = ACTIONS(1705),
    [sym_stack_var_keyword] = ACTIONS(1705),
    [sym_constant_keyword] = ACTIONS(1705),
    [sym_volatile_keyword] = ACTIONS(1705),
    [sym_non_volatile_keyword] = ACTIONS(1705),
    [sym_persistent_keyword] = ACTIONS(1705),
    [sym_dev_keyword] = ACTIONS(1705),
    [sym_devlev_keyword] = ACTIONS(1705),
    [sym_devchan_keyword] = ACTIONS(1705),
    [sym_if_keyword] = ACTIONS(1705),
    [sym_switch_keyword] = ACTIONS(1705),
    [sym_case_keyword] = ACTIONS(1705),
    [sym_default_keyword] = ACTIONS(1705),
    [sym_while_keyword] = ACTIONS(1705),
    [sym_for_keyword] = ACTIONS(1705),
    [sym_break_keyword] = ACTIONS(1705),
    [sym_continue_keyword] = ACTIONS(1705),
    [sym_return_keyword] = ACTIONS(1705),
    [sym_select_keyword] = ACTIONS(1705),
    [sym_active_keyword] = ACTIONS(1705),
    [sym_struct_keyword] = ACTIONS(1705),
    [sym_structure_keyword] = ACTIONS(1705),
    [sym_button_event_keyword] = ACTIONS(1705),
    [sym_channel_event_keyword] = ACTIONS(1705),
    [sym_level_event_keyword] = ACTIONS(1705),
    [sym_data_event_keyword] = ACTIONS(1705),
    [sym_timeline_event_keyword] = ACTIONS(1705),
    [sym_custom_event_keyword] = ACTIONS(1705),
    [sym_push_keyword] = ACTIONS(1705),
    [sym_release_keyword] = ACTIONS(1705),
    [aux_sym_on_keyword_token1] = ACTIONS(1705),
    [aux_sym_off_keyword_token1] = ACTIONS(1705),
    [sym_send_level_keyword] = ACTIONS(1705),
    [sym_send_string_keyword] = ACTIONS(1705),
    [sym_send_command_keyword] = ACTIONS(1705),
    [sym_clear_buffer_keyword] = ACTIONS(1705),
    [sym_create_buffer_keyword] = ACTIONS(1705),
    [sym_create_multi_buffer_keyword] = ACTIONS(1705),
    [sym_call_keyword] = ACTIONS(1705),
    [sym_system_call_keyword] = ACTIONS(1705),
    [sym_devchan_to_keyword] = ACTIONS(1705),
    [sym_devchan_min_to_keyword] = ACTIONS(1705),
    [sym_devchan_total_off_keyword] = ACTIONS(1705),
    [sym_devchan_pulse_keyword] = ACTIONS(1705),
    [sym_wait_keyword] = ACTIONS(1705),
    [sym_cancel_wait_keyword] = ACTIONS(1705),
    [sym_wait_until_keyword] = ACTIONS(1705),
    [sym_cancel_wait_until_keyword] = ACTIONS(1705),
    [sym_cancel_all_wait_keyword] = ACTIONS(1705),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1705),
    [sym_not] = ACTIONS(1705),
    [sym_bnot] = ACTIONS(1705),
    [sym___file__] = ACTIONS(1705),
    [sym___date__] = ACTIONS(1705),
    [sym___ldate__] = ACTIONS(1705),
    [sym___time__] = ACTIONS(1705),
    [sym___line__] = ACTIONS(1705),
    [sym___name__] = ACTIONS(1705),
    [sym___netlinx__] = ACTIONS(1705),
    [sym_netlinx_axi_version] = ACTIONS(1705),
    [sym_url_flg_tcp] = ACTIONS(1705),
    [sym_url_flg_accinfopresent] = ACTIONS(1705),
    [sym_url_flg_temp] = ACTIONS(1705),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1705),
    [sym_url_flg_stat_mask] = ACTIONS(1705),
    [sym_url_flg_stat_lookup] = ACTIONS(1705),
    [sym_url_flg_stat_connecting] = ACTIONS(1705),
    [sym_url_flg_stat_waiting] = ACTIONS(1705),
    [sym_url_flg_stat_connected] = ACTIONS(1705),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1705),
    [sym_first_virtual_device] = ACTIONS(1705),
    [sym_first_local_port] = ACTIONS(1705),
    [sym_dynamic_virtual_device] = ACTIONS(1705),
    [sym_timeline_once] = ACTIONS(1705),
    [sym_timeline_repeat] = ACTIONS(1705),
    [sym_timeline_absolute] = ACTIONS(1705),
    [sym_timeline_relative] = ACTIONS(1705),
    [sym_do_push_timed_infinite] = ACTIONS(1705),
    [sym_source_type_no_address] = ACTIONS(1705),
    [sym_source_type_neuron_id] = ACTIONS(1705),
    [sym_source_type_ip_address] = ACTIONS(1705),
    [sym_source_type_axlink] = ACTIONS(1705),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1705),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1705),
    [sym_source_type_ip_socket_address] = ACTIONS(1705),
    [sym_source_type_rs232] = ACTIONS(1705),
    [sym_source_type_internal] = ACTIONS(1705),
    [sym_source_type_mac_address] = ACTIONS(1705),
    [sym_source_type_ipv4_port] = ACTIONS(1705),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1705),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1705),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1705),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1705),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1705),
    [sym_true] = ACTIONS(1705),
    [sym_false] = ACTIONS(1705),
    [sym_file_read_only] = ACTIONS(1705),
    [sym_file_rw_new] = ACTIONS(1705),
    [sym_file_rw_append] = ACTIONS(1705),
    [sym_ip_tcp] = ACTIONS(1705),
    [sym_ip_udp] = ACTIONS(1705),
    [sym_ip_udp_2way] = ACTIONS(1705),
    [sym_xml_encode_types] = ACTIONS(1705),
    [sym_xml_encode_char_as_list] = ACTIONS(1705),
    [sym_xml_encode_le] = ACTIONS(1705),
    [sym_xml_decode_types] = ACTIONS(1705),
    [sym_xml_decode_no_preserve] = ACTIONS(1705),
    [sym_duet_dev_type_display_device] = ACTIONS(1705),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1705),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1705),
    [sym_duet_dev_type_io_device] = ACTIONS(1705),
    [sym_duet_dev_type_relay_device] = ACTIONS(1705),
    [sym_duet_dev_type_ups] = ACTIONS(1705),
    [sym_duet_dev_type_amplifier] = ACTIONS(1705),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1705),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1705),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1705),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1705),
    [sym_duet_dev_type_camera] = ACTIONS(1705),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1705),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1705),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1705),
    [sym_duet_dev_type_dss] = ACTIONS(1705),
    [sym_duet_dev_type_dvr] = ACTIONS(1705),
    [sym_duet_dev_type_disc_device] = ACTIONS(1705),
    [sym_duet_dev_type_document_camera] = ACTIONS(1705),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1705),
    [sym_duet_dev_type_hvac] = ACTIONS(1705),
    [sym_duet_dev_type_keypad] = ACTIONS(1705),
    [sym_duet_dev_type_light] = ACTIONS(1705),
    [sym_duet_dev_type_monitor] = ACTIONS(1705),
    [sym_duet_dev_type_motor] = ACTIONS(1705),
    [sym_duet_dev_type_multi_window] = ACTIONS(1705),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1705),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1705),
    [sym_duet_dev_type_receiver] = ACTIONS(1705),
    [sym_duet_dev_type_security_system] = ACTIONS(1705),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1705),
    [sym_duet_dev_type_settop_box] = ACTIONS(1705),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1705),
    [sym_duet_dev_type_switcher] = ACTIONS(1705),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1705),
    [sym_duet_dev_type_tv] = ACTIONS(1705),
    [sym_duet_dev_type_utility] = ACTIONS(1705),
    [sym_duet_dev_type_vcr] = ACTIONS(1705),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1705),
    [sym_duet_dev_type_video_processor] = ACTIONS(1705),
    [sym_duet_dev_type_video_projector] = ACTIONS(1705),
    [sym_duet_dev_type_video_wall] = ACTIONS(1705),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1705),
    [sym_duet_dev_type_weather] = ACTIONS(1705),
    [sym_duet_dev_not_polled] = ACTIONS(1705),
    [sym_duet_dev_polled] = ACTIONS(1705),
    [sym_clkmgr_mode_network] = ACTIONS(1705),
    [sym_clkmgr_mode_standalone] = ACTIONS(1705),
    [sym_normal_standby] = ACTIONS(1705),
    [sym_normal_wake] = ACTIONS(1705),
    [sym_ip_multicast_ttl_option] = ACTIONS(1705),
    [sym_ip_ttl_subnet] = ACTIONS(1705),
    [sym_ip_ttl_site] = ACTIONS(1705),
    [sym_ip_ttl_region] = ACTIONS(1705),
    [sym_ip_ttl_continent] = ACTIONS(1705),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1705),
    [sym_ip_nodelay_on] = ACTIONS(1705),
    [sym_ip_nodelay_off] = ACTIONS(1705),
    [sym_amx_error] = ACTIONS(1705),
    [sym_amx_warning] = ACTIONS(1705),
    [sym_amx_info] = ACTIONS(1705),
    [sym_amx_debug] = ACTIONS(1705),
    [sym_netlinx_login_success] = ACTIONS(1705),
    [sym_netlinx_login_fail] = ACTIONS(1705),
    [sym_netlinx_logout] = ACTIONS(1705),
    [sym_null_str] = ACTIONS(1705),
    [sym_smtp_address] = ACTIONS(1705),
    [sym_smtp_port_number] = ACTIONS(1705),
    [sym_smtp_username] = ACTIONS(1705),
    [sym_smtp_password] = ACTIONS(1705),
    [sym_smtp_require_tls] = ACTIONS(1705),
    [sym_smtp_from] = ACTIONS(1705),
    [sym_smtp_tls_true] = ACTIONS(1705),
    [sym_smtp_tls_false] = ACTIONS(1705),
    [sym_listview_on_row_select_event] = ACTIONS(1705),
    [sym_data_structure_unknown] = ACTIONS(1705),
    [sym_data_structure_datafeed] = ACTIONS(1705),
    [sym_data_structure_datarecordset] = ACTIONS(1705),
    [sym_data_structure_datarecord] = ACTIONS(1705),
    [sym_data_structure_datafield] = ACTIONS(1705),
    [sym_data_max_content_field_count] = ACTIONS(1705),
    [sym_data_max_metadata_field_count] = ACTIONS(1705),
    [sym_data_max_id_length] = ACTIONS(1705),
    [sym_data_max_name_length] = ACTIONS(1705),
    [sym_data_max_description_length] = ACTIONS(1705),
    [sym_data_max_type_length] = ACTIONS(1705),
    [sym_data_max_format_length] = ACTIONS(1705),
    [sym_data_max_source_length] = ACTIONS(1705),
    [sym_data_max_value_length] = ACTIONS(1705),
    [sym_data_max_label_length] = ACTIONS(1705),
    [sym_data_max_url_length] = ACTIONS(1705),
    [sym_max_last_login_info_length] = ACTIONS(1705),
    [sym_data_type_unknown] = ACTIONS(1705),
    [sym_data_type_string] = ACTIONS(1705),
    [sym_data_type_datetime] = ACTIONS(1705),
    [sym_data_type_date] = ACTIONS(1705),
    [sym_data_type_time] = ACTIONS(1705),
    [sym_data_type_image] = ACTIONS(1705),
    [sym_data_format_url] = ACTIONS(1705),
    [sym_data_format_phone] = ACTIONS(1705),
    [sym_data_format_email] = ACTIONS(1705),
    [sym_data_format_iso8601] = ACTIONS(1705),
    [sym_auditlog_priv] = ACTIONS(1705),
    [sym_configuration_priv] = ACTIONS(1705),
    [sym_deviceconfig_priv] = ACTIONS(1705),
    [sym_ftp_priv] = ACTIONS(1705),
    [sym_http_priv] = ACTIONS(1705),
    [sym_networkconfig_priv] = ACTIONS(1705),
    [sym_programport_priv] = ACTIONS(1705),
    [sym_remoteui_priv] = ACTIONS(1705),
    [sym_securitycontrol_priv] = ACTIONS(1705),
    [sym_softwaremanagement_priv] = ACTIONS(1705),
    [sym_terminal_priv] = ACTIONS(1705),
    [sym_tpadmin_priv] = ACTIONS(1705),
    [sym_user1_priv] = ACTIONS(1705),
    [sym_user2_priv] = ACTIONS(1705),
    [sym_user3_priv] = ACTIONS(1705),
    [sym_user4_priv] = ACTIONS(1705),
    [sym_usermanagement_priv] = ACTIONS(1705),
    [sym_valid_account] = ACTIONS(1705),
    [sym_err_name_invalid] = ACTIONS(1705),
    [sym_err_password_invalid] = ACTIONS(1705),
    [sym_err_invalid_account] = ACTIONS(1705),
    [sym_err_no_authorization] = ACTIONS(1705),
    [sym_err_invalid_parameter] = ACTIONS(1705),
    [sym_err_user_locked_out] = ACTIONS(1705),
    [sym_err_user_account_expired] = ACTIONS(1705),
    [sym_err_general_error] = ACTIONS(1705),
    [sym_err_authentication_error] = ACTIONS(1705),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1705),
    [sym_tls_validate_certificate] = ACTIONS(1705),
    [sym_tdata] = ACTIONS(1705),
    [sym_tchannel] = ACTIONS(1705),
    [sym_tlevel] = ACTIONS(1705),
    [sym_tbutton] = ACTIONS(1705),
    [sym_ttimeline] = ACTIONS(1705),
    [sym_tcustom] = ACTIONS(1705),
    [sym_url_struct] = ACTIONS(1705),
    [sym_dns_struct] = ACTIONS(1705),
    [sym_ip_address_struct] = ACTIONS(1705),
    [sym_dev_info_struct] = ACTIONS(1705),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1705),
    [sym_data_feed] = ACTIONS(1705),
    [sym_data_field] = ACTIONS(1705),
    [sym_data_record] = ACTIONS(1705),
    [sym_wc_data_feed] = ACTIONS(1705),
    [sym_wc_data_field] = ACTIONS(1705),
    [sym_wc_data_record] = ACTIONS(1705),
    [sym_last_login_info] = ACTIONS(1705),
    [sym____reserved___] = ACTIONS(1705),
    [sym_date] = ACTIONS(1705),
    [sym_day] = ACTIONS(1705),
    [sym_time] = ACTIONS(1705),
    [sym_ldate] = ACTIONS(1705),
    [sym_push_device] = ACTIONS(1705),
    [sym_push_channel] = ACTIONS(1705),
    [sym_push_devchan] = ACTIONS(1705),
    [sym_release_device] = ACTIONS(1705),
    [sym_release_channel] = ACTIONS(1705),
    [sym_release_devchan] = ACTIONS(1705),
    [sym_master_slot] = ACTIONS(1705),
    [sym_get_pulse_time] = ACTIONS(1705),
    [sym_get_timer] = ACTIONS(1705),
    [sym_master_sn] = ACTIONS(1705),
    [sym_system_number] = ACTIONS(1705),
    [sym_button] = ACTIONS(1705),
    [sym_dv_channel] = ACTIONS(1705),
    [sym_data] = ACTIONS(1705),
    [sym_level] = ACTIONS(1705),
    [sym_timeline] = ACTIONS(1705),
    [sym_channel] = ACTIONS(1705),
    [sym_custom] = ACTIONS(1705),
    [sym_atoi] = ACTIONS(1705),
    [sym_atol] = ACTIONS(1705),
    [sym_atof] = ACTIONS(1705),
    [sym_itoa] = ACTIONS(1705),
    [sym_itohex] = ACTIONS(1705),
    [sym_hextoi] = ACTIONS(1705),
    [sym_ftoa] = ACTIONS(1705),
    [sym_find_string] = ACTIONS(1705),
    [sym_left_string] = ACTIONS(1705),
    [sym_right_string] = ACTIONS(1705),
    [sym_mid_string] = ACTIONS(1705),
    [sym_remove_string] = ACTIONS(1705),
    [sym_get_buffer_string] = ACTIONS(1705),
    [sym_get_buffer_char] = ACTIONS(1705),
    [sym_get_multi_buffer_string] = ACTIONS(1705),
    [sym_length_string] = ACTIONS(1705),
    [sym_max_length_string] = ACTIONS(1705),
    [sym_set_length_string] = ACTIONS(1705),
    [sym_compare_string] = ACTIONS(1705),
    [sym_upper_string] = ACTIONS(1705),
    [sym_lower_string] = ACTIONS(1705),
    [sym_get_last] = ACTIONS(1705),
    [sym_date_to_day] = ACTIONS(1705),
    [sym_date_to_month] = ACTIONS(1705),
    [sym_date_to_year] = ACTIONS(1705),
    [sym_time_to_hour] = ACTIONS(1705),
    [sym_time_to_minute] = ACTIONS(1705),
    [sym_time_to_second] = ACTIONS(1705),
    [sym_day_of_week] = ACTIONS(1705),
    [sym_set_pulse_time] = ACTIONS(1705),
    [sym_set_timer] = ACTIONS(1705),
    [sym_file_open] = ACTIONS(1705),
    [sym_file_close] = ACTIONS(1705),
    [sym_file_read] = ACTIONS(1705),
    [sym_file_read_line] = ACTIONS(1705),
    [sym_file_write] = ACTIONS(1705),
    [sym_file_write_line] = ACTIONS(1705),
    [sym_file_copy] = ACTIONS(1705),
    [sym_file_delete] = ACTIONS(1705),
    [sym_file_rename] = ACTIONS(1705),
    [sym_file_dir] = ACTIONS(1705),
    [sym_file_getdir] = ACTIONS(1705),
    [sym_file_setdir] = ACTIONS(1705),
    [sym_file_create_dir] = ACTIONS(1705),
    [sym_file_remove_dir] = ACTIONS(1705),
    [sym_file_seek] = ACTIONS(1705),
    [sym_length_array] = ACTIONS(1705),
    [sym_max_length_array] = ACTIONS(1705),
    [sym_set_length_array] = ACTIONS(1705),
    [sym_ip_client_open] = ACTIONS(1705),
    [sym_ip_client_close] = ACTIONS(1705),
    [sym_ip_server_open] = ACTIONS(1705),
    [sym_ip_server_close] = ACTIONS(1705),
    [sym_ip_bound_client_open] = ACTIONS(1705),
    [sym_ip_mc_server_open] = ACTIONS(1705),
    [sym_ip_set_option] = ACTIONS(1705),
    [sym_ssh_client_open] = ACTIONS(1705),
    [sym_ssh_client_close] = ACTIONS(1705),
    [sym_tls_client_open] = ACTIONS(1705),
    [sym_tls_client_close] = ACTIONS(1705),
    [sym_do_push] = ACTIONS(1705),
    [sym_do_push_timed] = ACTIONS(1705),
    [sym_do_release] = ACTIONS(1705),
    [sym_device_id] = ACTIONS(1705),
    [sym_device_info] = ACTIONS(1705),
    [sym_device_id_string] = ACTIONS(1705),
    [sym_device_standby] = ACTIONS(1705),
    [sym_device_wake] = ACTIONS(1705),
    [sym_timeline_create] = ACTIONS(1705),
    [sym_timeline_kill] = ACTIONS(1705),
    [sym_timeline_reload] = ACTIONS(1705),
    [sym_timeline_pause] = ACTIONS(1705),
    [sym_timeline_restart] = ACTIONS(1705),
    [sym_timeline_set] = ACTIONS(1705),
    [sym_timeline_get] = ACTIONS(1705),
    [sym_timeline_active] = ACTIONS(1705),
    [sym_timeline_dynamic_id] = ACTIONS(1705),
    [sym_variable_to_string] = ACTIONS(1705),
    [sym_string_to_variable] = ACTIONS(1705),
    [sym_length_variable_to_string] = ACTIONS(1705),
    [sym_type_cast] = ACTIONS(1705),
    [sym_variable_to_xml] = ACTIONS(1705),
    [sym_xml_to_variable] = ACTIONS(1705),
    [sym_length_variable_to_xml] = ACTIONS(1705),
    [sym_redirect_string] = ACTIONS(1705),
    [sym_random_number] = ACTIONS(1705),
    [sym_get_unique_id] = ACTIONS(1705),
    [sym_get_system_number] = ACTIONS(1705),
    [sym_set_system_number] = ACTIONS(1705),
    [sym_get_serial_number] = ACTIONS(1705),
    [sym_get_master_build] = ACTIONS(1705),
    [sym_set_validation_code] = ACTIONS(1705),
    [sym_reboot] = ACTIONS(1705),
    [sym_load_duet_module] = ACTIONS(1705),
    [sym_unload_duet_module] = ACTIONS(1705),
    [sym_duet_mem_size_set] = ACTIONS(1705),
    [sym_duet_mem_size_get] = ACTIONS(1705),
    [sym_dynamic_application_device] = ACTIONS(1705),
    [sym_dynamic_polled_port] = ACTIONS(1705),
    [sym_static_port_binding] = ACTIONS(1705),
    [sym_static_ip_binding] = ACTIONS(1705),
    [sym_get_dns_list] = ACTIONS(1705),
    [sym_set_dns_list] = ACTIONS(1705),
    [sym_get_ip_address] = ACTIONS(1705),
    [sym_set_ip_address] = ACTIONS(1705),
    [sym_get_url_list] = ACTIONS(1705),
    [sym_add_url_entry] = ACTIONS(1705),
    [sym_delete_url_entry] = ACTIONS(1705),
    [sym_abs_value] = ACTIONS(1705),
    [sym_max_value] = ACTIONS(1705),
    [sym_min_value] = ACTIONS(1705),
    [sym_exp_value] = ACTIONS(1705),
    [sym_log_value] = ACTIONS(1705),
    [sym_log10_value] = ACTIONS(1705),
    [sym_power_value] = ACTIONS(1705),
    [sym_sqrt_value] = ACTIONS(1705),
    [sym_raw_be] = ACTIONS(1705),
    [sym_raw_le] = ACTIONS(1705),
    [sym_internal_queue_size_set] = ACTIONS(1705),
    [sym_internal_queue_size_get] = ACTIONS(1705),
    [sym_internal_threshold_set] = ACTIONS(1705),
    [sym_internal_threshold_get] = ACTIONS(1705),
    [sym_rebuilt_event] = ACTIONS(1705),
    [sym_do_custom_event] = ACTIONS(1705),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1705),
    [sym_clkmgr_set_clk_source] = ACTIONS(1705),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1705),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1705),
    [sym_clkmgr_get_timezone] = ACTIONS(1705),
    [sym_clkmgr_set_timezone] = ACTIONS(1705),
    [sym_clkmgr_get_resync_period] = ACTIONS(1705),
    [sym_clkmgr_set_resync_period] = ACTIONS(1705),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1705),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1705),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1705),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1705),
    [sym_clkmgr_get_timeservers] = ACTIONS(1705),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1705),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1705),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1705),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1705),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1705),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1705),
    [sym_set_log_level] = ACTIONS(1705),
    [sym_get_log_level] = ACTIONS(1705),
    [sym_amx_log] = ACTIONS(1705),
    [sym_smtp_server_config_set] = ACTIONS(1705),
    [sym_smtp_server_config_get] = ACTIONS(1705),
    [sym_smtp_send] = ACTIONS(1705),
    [sym_validate_netlinx_account] = ACTIONS(1705),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1705),
    [sym_authenticate_certificate] = ACTIONS(1705),
    [sym_audit_netlinx_session_event] = ACTIONS(1705),
    [sym_audit_netlinx_generic_event] = ACTIONS(1705),
    [sym_get_available_flash_disk_space] = ACTIONS(1705),
    [sym_get_max_flash_disk_space] = ACTIONS(1705),
    [sym_data_create_feed] = ACTIONS(1705),
    [sym_data_delete_feed] = ACTIONS(1705),
    [sym_data_publish_feed] = ACTIONS(1705),
    [sym_data_get_published_feed] = ACTIONS(1705),
    [sym_data_add_record] = ACTIONS(1705),
    [sym_data_get_event_record] = ACTIONS(1705),
    [sym__wc_data_create_feed] = ACTIONS(1705),
    [sym__wc_data_add_record] = ACTIONS(1705),
    [sym__wc_data_get_event_record] = ACTIONS(1705),
    [sym_format] = ACTIONS(1705),
    [sym_astro_clock] = ACTIONS(1705),
    [sym_set_outdoor_temperature] = ACTIONS(1705),
    [sym_set_virtual_level_count] = ACTIONS(1705),
    [sym_set_virtual_channel_count] = ACTIONS(1705),
    [sym_set_virtual_port_count] = ACTIONS(1705),
    [sym__wc_encode_int] = ACTIONS(1705),
    [sym__wc_decode_int] = ACTIONS(1705),
    [sym__wc_int] = ACTIONS(1705),
    [sym__wc_to_ch_int] = ACTIONS(1705),
    [sym__ch_to_wc_int] = ACTIONS(1705),
    [sym__wc_find_string_int] = ACTIONS(1705),
    [sym__wc_left_string_int] = ACTIONS(1705),
    [sym__wc_lower_string_int] = ACTIONS(1705),
    [sym__wc_mid_string_int] = ACTIONS(1705),
    [sym__wc_remove_string_int] = ACTIONS(1705),
    [sym__wc_right_string_int] = ACTIONS(1705),
    [sym__wc_upper_string_int] = ACTIONS(1705),
    [sym__wc_compare_string_int] = ACTIONS(1705),
    [sym__wc_get_buffer_char_int] = ACTIONS(1705),
    [sym__wc_get_buffer_string_int] = ACTIONS(1705),
    [sym__wc_concat_string_int] = ACTIONS(1705),
    [sym__wc_file_open_int] = ACTIONS(1705),
    [sym__wc_file_close_int] = ACTIONS(1705),
    [sym__wc_file_read_int] = ACTIONS(1705),
    [sym__wc_file_read_line_int] = ACTIONS(1705),
    [sym__wc_file_write_int] = ACTIONS(1705),
    [sym__wc_file_write_line_int] = ACTIONS(1705),
    [aux_sym_preproc_include_token1] = ACTIONS(1705),
    [aux_sym_preproc_define_token1] = ACTIONS(1703),
    [aux_sym_preproc_warn_token1] = ACTIONS(1703),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1703),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1703),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1703),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1703),
    [aux_sym_preproc_else_token1] = ACTIONS(1703),
    [anon_sym_LBRACK] = ACTIONS(1703),
    [anon_sym_LBRACE] = ACTIONS(1703),
    [anon_sym_LPAREN] = ACTIONS(1705),
    [anon_sym_BANG] = ACTIONS(1703),
    [anon_sym_TILDE] = ACTIONS(1703),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_DASH_DASH] = ACTIONS(1703),
    [anon_sym_PLUS_PLUS] = ACTIONS(1703),
    [anon_sym_DQUOTE] = ACTIONS(1703),
    [anon_sym_SQUOTE] = ACTIONS(1703),
    [sym_number_literal] = ACTIONS(1703),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(349)] = {
    [sym_identifier] = ACTIONS(1713),
    [sym_program_name_keyword] = ACTIONS(1713),
    [sym_module_name_keyword] = ACTIONS(1713),
    [sym_define_device_keyword] = ACTIONS(1713),
    [sym_define_combine_keyword] = ACTIONS(1713),
    [sym_define_constant_keyword] = ACTIONS(1713),
    [sym_define_type_keyword] = ACTIONS(1713),
    [sym_define_variable_keyword] = ACTIONS(1713),
    [sym_define_system_variable_keyword] = ACTIONS(1713),
    [sym_define_connect_level_keyword] = ACTIONS(1713),
    [sym_define_latching_keyword] = ACTIONS(1713),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1713),
    [sym_define_toggling_keyword] = ACTIONS(1713),
    [sym_define_call_keyword] = ACTIONS(1713),
    [sym_define_function_keyword] = ACTIONS(1713),
    [sym_define_library_function_keyword] = ACTIONS(1713),
    [sym_define_start_keyword] = ACTIONS(1713),
    [sym_define_event_keyword] = ACTIONS(1713),
    [sym_define_program_keyword] = ACTIONS(1713),
    [sym_define_module_keyword] = ACTIONS(1713),
    [sym_char_keyword] = ACTIONS(1713),
    [sym_widechar_keyword] = ACTIONS(1713),
    [sym_integer_keyword] = ACTIONS(1713),
    [sym_sinteger_keyword] = ACTIONS(1713),
    [sym_long_keyword] = ACTIONS(1713),
    [sym_slong_keyword] = ACTIONS(1713),
    [sym_float_keyword] = ACTIONS(1713),
    [sym_double_keyword] = ACTIONS(1713),
    [sym_variant_keyword] = ACTIONS(1713),
    [sym_variantarray_keyword] = ACTIONS(1713),
    [sym_local_var_keyword] = ACTIONS(1713),
    [sym_stack_var_keyword] = ACTIONS(1713),
    [sym_constant_keyword] = ACTIONS(1713),
    [sym_volatile_keyword] = ACTIONS(1713),
    [sym_non_volatile_keyword] = ACTIONS(1713),
    [sym_persistent_keyword] = ACTIONS(1713),
    [sym_dev_keyword] = ACTIONS(1713),
    [sym_devlev_keyword] = ACTIONS(1713),
    [sym_devchan_keyword] = ACTIONS(1713),
    [sym_if_keyword] = ACTIONS(1713),
    [sym_switch_keyword] = ACTIONS(1713),
    [sym_case_keyword] = ACTIONS(1713),
    [sym_default_keyword] = ACTIONS(1713),
    [sym_while_keyword] = ACTIONS(1713),
    [sym_for_keyword] = ACTIONS(1713),
    [sym_break_keyword] = ACTIONS(1713),
    [sym_continue_keyword] = ACTIONS(1713),
    [sym_return_keyword] = ACTIONS(1713),
    [sym_select_keyword] = ACTIONS(1713),
    [sym_active_keyword] = ACTIONS(1713),
    [sym_struct_keyword] = ACTIONS(1713),
    [sym_structure_keyword] = ACTIONS(1713),
    [sym_button_event_keyword] = ACTIONS(1713),
    [sym_channel_event_keyword] = ACTIONS(1713),
    [sym_level_event_keyword] = ACTIONS(1713),
    [sym_data_event_keyword] = ACTIONS(1713),
    [sym_timeline_event_keyword] = ACTIONS(1713),
    [sym_custom_event_keyword] = ACTIONS(1713),
    [sym_push_keyword] = ACTIONS(1713),
    [sym_release_keyword] = ACTIONS(1713),
    [aux_sym_on_keyword_token1] = ACTIONS(1713),
    [aux_sym_off_keyword_token1] = ACTIONS(1713),
    [sym_send_level_keyword] = ACTIONS(1713),
    [sym_send_string_keyword] = ACTIONS(1713),
    [sym_send_command_keyword] = ACTIONS(1713),
    [sym_clear_buffer_keyword] = ACTIONS(1713),
    [sym_create_buffer_keyword] = ACTIONS(1713),
    [sym_create_multi_buffer_keyword] = ACTIONS(1713),
    [sym_call_keyword] = ACTIONS(1713),
    [sym_system_call_keyword] = ACTIONS(1713),
    [sym_devchan_to_keyword] = ACTIONS(1713),
    [sym_devchan_min_to_keyword] = ACTIONS(1713),
    [sym_devchan_total_off_keyword] = ACTIONS(1713),
    [sym_devchan_pulse_keyword] = ACTIONS(1713),
    [sym_wait_keyword] = ACTIONS(1713),
    [sym_cancel_wait_keyword] = ACTIONS(1713),
    [sym_wait_until_keyword] = ACTIONS(1713),
    [sym_cancel_wait_until_keyword] = ACTIONS(1713),
    [sym_cancel_all_wait_keyword] = ACTIONS(1713),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1713),
    [sym_not] = ACTIONS(1713),
    [sym_bnot] = ACTIONS(1713),
    [sym___file__] = ACTIONS(1713),
    [sym___date__] = ACTIONS(1713),
    [sym___ldate__] = ACTIONS(1713),
    [sym___time__] = ACTIONS(1713),
    [sym___line__] = ACTIONS(1713),
    [sym___name__] = ACTIONS(1713),
    [sym___netlinx__] = ACTIONS(1713),
    [sym_netlinx_axi_version] = ACTIONS(1713),
    [sym_url_flg_tcp] = ACTIONS(1713),
    [sym_url_flg_accinfopresent] = ACTIONS(1713),
    [sym_url_flg_temp] = ACTIONS(1713),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1713),
    [sym_url_flg_stat_mask] = ACTIONS(1713),
    [sym_url_flg_stat_lookup] = ACTIONS(1713),
    [sym_url_flg_stat_connecting] = ACTIONS(1713),
    [sym_url_flg_stat_waiting] = ACTIONS(1713),
    [sym_url_flg_stat_connected] = ACTIONS(1713),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1713),
    [sym_first_virtual_device] = ACTIONS(1713),
    [sym_first_local_port] = ACTIONS(1713),
    [sym_dynamic_virtual_device] = ACTIONS(1713),
    [sym_timeline_once] = ACTIONS(1713),
    [sym_timeline_repeat] = ACTIONS(1713),
    [sym_timeline_absolute] = ACTIONS(1713),
    [sym_timeline_relative] = ACTIONS(1713),
    [sym_do_push_timed_infinite] = ACTIONS(1713),
    [sym_source_type_no_address] = ACTIONS(1713),
    [sym_source_type_neuron_id] = ACTIONS(1713),
    [sym_source_type_ip_address] = ACTIONS(1713),
    [sym_source_type_axlink] = ACTIONS(1713),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1713),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1713),
    [sym_source_type_ip_socket_address] = ACTIONS(1713),
    [sym_source_type_rs232] = ACTIONS(1713),
    [sym_source_type_internal] = ACTIONS(1713),
    [sym_source_type_mac_address] = ACTIONS(1713),
    [sym_source_type_ipv4_port] = ACTIONS(1713),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1713),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1713),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1713),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1713),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1713),
    [sym_true] = ACTIONS(1713),
    [sym_false] = ACTIONS(1713),
    [sym_file_read_only] = ACTIONS(1713),
    [sym_file_rw_new] = ACTIONS(1713),
    [sym_file_rw_append] = ACTIONS(1713),
    [sym_ip_tcp] = ACTIONS(1713),
    [sym_ip_udp] = ACTIONS(1713),
    [sym_ip_udp_2way] = ACTIONS(1713),
    [sym_xml_encode_types] = ACTIONS(1713),
    [sym_xml_encode_char_as_list] = ACTIONS(1713),
    [sym_xml_encode_le] = ACTIONS(1713),
    [sym_xml_decode_types] = ACTIONS(1713),
    [sym_xml_decode_no_preserve] = ACTIONS(1713),
    [sym_duet_dev_type_display_device] = ACTIONS(1713),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1713),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1713),
    [sym_duet_dev_type_io_device] = ACTIONS(1713),
    [sym_duet_dev_type_relay_device] = ACTIONS(1713),
    [sym_duet_dev_type_ups] = ACTIONS(1713),
    [sym_duet_dev_type_amplifier] = ACTIONS(1713),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1713),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1713),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1713),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1713),
    [sym_duet_dev_type_camera] = ACTIONS(1713),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1713),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1713),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1713),
    [sym_duet_dev_type_dss] = ACTIONS(1713),
    [sym_duet_dev_type_dvr] = ACTIONS(1713),
    [sym_duet_dev_type_disc_device] = ACTIONS(1713),
    [sym_duet_dev_type_document_camera] = ACTIONS(1713),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1713),
    [sym_duet_dev_type_hvac] = ACTIONS(1713),
    [sym_duet_dev_type_keypad] = ACTIONS(1713),
    [sym_duet_dev_type_light] = ACTIONS(1713),
    [sym_duet_dev_type_monitor] = ACTIONS(1713),
    [sym_duet_dev_type_motor] = ACTIONS(1713),
    [sym_duet_dev_type_multi_window] = ACTIONS(1713),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1713),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1713),
    [sym_duet_dev_type_receiver] = ACTIONS(1713),
    [sym_duet_dev_type_security_system] = ACTIONS(1713),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1713),
    [sym_duet_dev_type_settop_box] = ACTIONS(1713),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1713),
    [sym_duet_dev_type_switcher] = ACTIONS(1713),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1713),
    [sym_duet_dev_type_tv] = ACTIONS(1713),
    [sym_duet_dev_type_utility] = ACTIONS(1713),
    [sym_duet_dev_type_vcr] = ACTIONS(1713),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1713),
    [sym_duet_dev_type_video_processor] = ACTIONS(1713),
    [sym_duet_dev_type_video_projector] = ACTIONS(1713),
    [sym_duet_dev_type_video_wall] = ACTIONS(1713),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1713),
    [sym_duet_dev_type_weather] = ACTIONS(1713),
    [sym_duet_dev_not_polled] = ACTIONS(1713),
    [sym_duet_dev_polled] = ACTIONS(1713),
    [sym_clkmgr_mode_network] = ACTIONS(1713),
    [sym_clkmgr_mode_standalone] = ACTIONS(1713),
    [sym_normal_standby] = ACTIONS(1713),
    [sym_normal_wake] = ACTIONS(1713),
    [sym_ip_multicast_ttl_option] = ACTIONS(1713),
    [sym_ip_ttl_subnet] = ACTIONS(1713),
    [sym_ip_ttl_site] = ACTIONS(1713),
    [sym_ip_ttl_region] = ACTIONS(1713),
    [sym_ip_ttl_continent] = ACTIONS(1713),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1713),
    [sym_ip_nodelay_on] = ACTIONS(1713),
    [sym_ip_nodelay_off] = ACTIONS(1713),
    [sym_amx_error] = ACTIONS(1713),
    [sym_amx_warning] = ACTIONS(1713),
    [sym_amx_info] = ACTIONS(1713),
    [sym_amx_debug] = ACTIONS(1713),
    [sym_netlinx_login_success] = ACTIONS(1713),
    [sym_netlinx_login_fail] = ACTIONS(1713),
    [sym_netlinx_logout] = ACTIONS(1713),
    [sym_null_str] = ACTIONS(1713),
    [sym_smtp_address] = ACTIONS(1713),
    [sym_smtp_port_number] = ACTIONS(1713),
    [sym_smtp_username] = ACTIONS(1713),
    [sym_smtp_password] = ACTIONS(1713),
    [sym_smtp_require_tls] = ACTIONS(1713),
    [sym_smtp_from] = ACTIONS(1713),
    [sym_smtp_tls_true] = ACTIONS(1713),
    [sym_smtp_tls_false] = ACTIONS(1713),
    [sym_listview_on_row_select_event] = ACTIONS(1713),
    [sym_data_structure_unknown] = ACTIONS(1713),
    [sym_data_structure_datafeed] = ACTIONS(1713),
    [sym_data_structure_datarecordset] = ACTIONS(1713),
    [sym_data_structure_datarecord] = ACTIONS(1713),
    [sym_data_structure_datafield] = ACTIONS(1713),
    [sym_data_max_content_field_count] = ACTIONS(1713),
    [sym_data_max_metadata_field_count] = ACTIONS(1713),
    [sym_data_max_id_length] = ACTIONS(1713),
    [sym_data_max_name_length] = ACTIONS(1713),
    [sym_data_max_description_length] = ACTIONS(1713),
    [sym_data_max_type_length] = ACTIONS(1713),
    [sym_data_max_format_length] = ACTIONS(1713),
    [sym_data_max_source_length] = ACTIONS(1713),
    [sym_data_max_value_length] = ACTIONS(1713),
    [sym_data_max_label_length] = ACTIONS(1713),
    [sym_data_max_url_length] = ACTIONS(1713),
    [sym_max_last_login_info_length] = ACTIONS(1713),
    [sym_data_type_unknown] = ACTIONS(1713),
    [sym_data_type_string] = ACTIONS(1713),
    [sym_data_type_datetime] = ACTIONS(1713),
    [sym_data_type_date] = ACTIONS(1713),
    [sym_data_type_time] = ACTIONS(1713),
    [sym_data_type_image] = ACTIONS(1713),
    [sym_data_format_url] = ACTIONS(1713),
    [sym_data_format_phone] = ACTIONS(1713),
    [sym_data_format_email] = ACTIONS(1713),
    [sym_data_format_iso8601] = ACTIONS(1713),
    [sym_auditlog_priv] = ACTIONS(1713),
    [sym_configuration_priv] = ACTIONS(1713),
    [sym_deviceconfig_priv] = ACTIONS(1713),
    [sym_ftp_priv] = ACTIONS(1713),
    [sym_http_priv] = ACTIONS(1713),
    [sym_networkconfig_priv] = ACTIONS(1713),
    [sym_programport_priv] = ACTIONS(1713),
    [sym_remoteui_priv] = ACTIONS(1713),
    [sym_securitycontrol_priv] = ACTIONS(1713),
    [sym_softwaremanagement_priv] = ACTIONS(1713),
    [sym_terminal_priv] = ACTIONS(1713),
    [sym_tpadmin_priv] = ACTIONS(1713),
    [sym_user1_priv] = ACTIONS(1713),
    [sym_user2_priv] = ACTIONS(1713),
    [sym_user3_priv] = ACTIONS(1713),
    [sym_user4_priv] = ACTIONS(1713),
    [sym_usermanagement_priv] = ACTIONS(1713),
    [sym_valid_account] = ACTIONS(1713),
    [sym_err_name_invalid] = ACTIONS(1713),
    [sym_err_password_invalid] = ACTIONS(1713),
    [sym_err_invalid_account] = ACTIONS(1713),
    [sym_err_no_authorization] = ACTIONS(1713),
    [sym_err_invalid_parameter] = ACTIONS(1713),
    [sym_err_user_locked_out] = ACTIONS(1713),
    [sym_err_user_account_expired] = ACTIONS(1713),
    [sym_err_general_error] = ACTIONS(1713),
    [sym_err_authentication_error] = ACTIONS(1713),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1713),
    [sym_tls_validate_certificate] = ACTIONS(1713),
    [sym_tdata] = ACTIONS(1713),
    [sym_tchannel] = ACTIONS(1713),
    [sym_tlevel] = ACTIONS(1713),
    [sym_tbutton] = ACTIONS(1713),
    [sym_ttimeline] = ACTIONS(1713),
    [sym_tcustom] = ACTIONS(1713),
    [sym_url_struct] = ACTIONS(1713),
    [sym_dns_struct] = ACTIONS(1713),
    [sym_ip_address_struct] = ACTIONS(1713),
    [sym_dev_info_struct] = ACTIONS(1713),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1713),
    [sym_data_feed] = ACTIONS(1713),
    [sym_data_field] = ACTIONS(1713),
    [sym_data_record] = ACTIONS(1713),
    [sym_wc_data_feed] = ACTIONS(1713),
    [sym_wc_data_field] = ACTIONS(1713),
    [sym_wc_data_record] = ACTIONS(1713),
    [sym_last_login_info] = ACTIONS(1713),
    [sym____reserved___] = ACTIONS(1713),
    [sym_date] = ACTIONS(1713),
    [sym_day] = ACTIONS(1713),
    [sym_time] = ACTIONS(1713),
    [sym_ldate] = ACTIONS(1713),
    [sym_push_device] = ACTIONS(1713),
    [sym_push_channel] = ACTIONS(1713),
    [sym_push_devchan] = ACTIONS(1713),
    [sym_release_device] = ACTIONS(1713),
    [sym_release_channel] = ACTIONS(1713),
    [sym_release_devchan] = ACTIONS(1713),
    [sym_master_slot] = ACTIONS(1713),
    [sym_get_pulse_time] = ACTIONS(1713),
    [sym_get_timer] = ACTIONS(1713),
    [sym_master_sn] = ACTIONS(1713),
    [sym_system_number] = ACTIONS(1713),
    [sym_button] = ACTIONS(1713),
    [sym_dv_channel] = ACTIONS(1713),
    [sym_data] = ACTIONS(1713),
    [sym_level] = ACTIONS(1713),
    [sym_timeline] = ACTIONS(1713),
    [sym_channel] = ACTIONS(1713),
    [sym_custom] = ACTIONS(1713),
    [sym_atoi] = ACTIONS(1713),
    [sym_atol] = ACTIONS(1713),
    [sym_atof] = ACTIONS(1713),
    [sym_itoa] = ACTIONS(1713),
    [sym_itohex] = ACTIONS(1713),
    [sym_hextoi] = ACTIONS(1713),
    [sym_ftoa] = ACTIONS(1713),
    [sym_find_string] = ACTIONS(1713),
    [sym_left_string] = ACTIONS(1713),
    [sym_right_string] = ACTIONS(1713),
    [sym_mid_string] = ACTIONS(1713),
    [sym_remove_string] = ACTIONS(1713),
    [sym_get_buffer_string] = ACTIONS(1713),
    [sym_get_buffer_char] = ACTIONS(1713),
    [sym_get_multi_buffer_string] = ACTIONS(1713),
    [sym_length_string] = ACTIONS(1713),
    [sym_max_length_string] = ACTIONS(1713),
    [sym_set_length_string] = ACTIONS(1713),
    [sym_compare_string] = ACTIONS(1713),
    [sym_upper_string] = ACTIONS(1713),
    [sym_lower_string] = ACTIONS(1713),
    [sym_get_last] = ACTIONS(1713),
    [sym_date_to_day] = ACTIONS(1713),
    [sym_date_to_month] = ACTIONS(1713),
    [sym_date_to_year] = ACTIONS(1713),
    [sym_time_to_hour] = ACTIONS(1713),
    [sym_time_to_minute] = ACTIONS(1713),
    [sym_time_to_second] = ACTIONS(1713),
    [sym_day_of_week] = ACTIONS(1713),
    [sym_set_pulse_time] = ACTIONS(1713),
    [sym_set_timer] = ACTIONS(1713),
    [sym_file_open] = ACTIONS(1713),
    [sym_file_close] = ACTIONS(1713),
    [sym_file_read] = ACTIONS(1713),
    [sym_file_read_line] = ACTIONS(1713),
    [sym_file_write] = ACTIONS(1713),
    [sym_file_write_line] = ACTIONS(1713),
    [sym_file_copy] = ACTIONS(1713),
    [sym_file_delete] = ACTIONS(1713),
    [sym_file_rename] = ACTIONS(1713),
    [sym_file_dir] = ACTIONS(1713),
    [sym_file_getdir] = ACTIONS(1713),
    [sym_file_setdir] = ACTIONS(1713),
    [sym_file_create_dir] = ACTIONS(1713),
    [sym_file_remove_dir] = ACTIONS(1713),
    [sym_file_seek] = ACTIONS(1713),
    [sym_length_array] = ACTIONS(1713),
    [sym_max_length_array] = ACTIONS(1713),
    [sym_set_length_array] = ACTIONS(1713),
    [sym_ip_client_open] = ACTIONS(1713),
    [sym_ip_client_close] = ACTIONS(1713),
    [sym_ip_server_open] = ACTIONS(1713),
    [sym_ip_server_close] = ACTIONS(1713),
    [sym_ip_bound_client_open] = ACTIONS(1713),
    [sym_ip_mc_server_open] = ACTIONS(1713),
    [sym_ip_set_option] = ACTIONS(1713),
    [sym_ssh_client_open] = ACTIONS(1713),
    [sym_ssh_client_close] = ACTIONS(1713),
    [sym_tls_client_open] = ACTIONS(1713),
    [sym_tls_client_close] = ACTIONS(1713),
    [sym_do_push] = ACTIONS(1713),
    [sym_do_push_timed] = ACTIONS(1713),
    [sym_do_release] = ACTIONS(1713),
    [sym_device_id] = ACTIONS(1713),
    [sym_device_info] = ACTIONS(1713),
    [sym_device_id_string] = ACTIONS(1713),
    [sym_device_standby] = ACTIONS(1713),
    [sym_device_wake] = ACTIONS(1713),
    [sym_timeline_create] = ACTIONS(1713),
    [sym_timeline_kill] = ACTIONS(1713),
    [sym_timeline_reload] = ACTIONS(1713),
    [sym_timeline_pause] = ACTIONS(1713),
    [sym_timeline_restart] = ACTIONS(1713),
    [sym_timeline_set] = ACTIONS(1713),
    [sym_timeline_get] = ACTIONS(1713),
    [sym_timeline_active] = ACTIONS(1713),
    [sym_timeline_dynamic_id] = ACTIONS(1713),
    [sym_variable_to_string] = ACTIONS(1713),
    [sym_string_to_variable] = ACTIONS(1713),
    [sym_length_variable_to_string] = ACTIONS(1713),
    [sym_type_cast] = ACTIONS(1713),
    [sym_variable_to_xml] = ACTIONS(1713),
    [sym_xml_to_variable] = ACTIONS(1713),
    [sym_length_variable_to_xml] = ACTIONS(1713),
    [sym_redirect_string] = ACTIONS(1713),
    [sym_random_number] = ACTIONS(1713),
    [sym_get_unique_id] = ACTIONS(1713),
    [sym_get_system_number] = ACTIONS(1713),
    [sym_set_system_number] = ACTIONS(1713),
    [sym_get_serial_number] = ACTIONS(1713),
    [sym_get_master_build] = ACTIONS(1713),
    [sym_set_validation_code] = ACTIONS(1713),
    [sym_reboot] = ACTIONS(1713),
    [sym_load_duet_module] = ACTIONS(1713),
    [sym_unload_duet_module] = ACTIONS(1713),
    [sym_duet_mem_size_set] = ACTIONS(1713),
    [sym_duet_mem_size_get] = ACTIONS(1713),
    [sym_dynamic_application_device] = ACTIONS(1713),
    [sym_dynamic_polled_port] = ACTIONS(1713),
    [sym_static_port_binding] = ACTIONS(1713),
    [sym_static_ip_binding] = ACTIONS(1713),
    [sym_get_dns_list] = ACTIONS(1713),
    [sym_set_dns_list] = ACTIONS(1713),
    [sym_get_ip_address] = ACTIONS(1713),
    [sym_set_ip_address] = ACTIONS(1713),
    [sym_get_url_list] = ACTIONS(1713),
    [sym_add_url_entry] = ACTIONS(1713),
    [sym_delete_url_entry] = ACTIONS(1713),
    [sym_abs_value] = ACTIONS(1713),
    [sym_max_value] = ACTIONS(1713),
    [sym_min_value] = ACTIONS(1713),
    [sym_exp_value] = ACTIONS(1713),
    [sym_log_value] = ACTIONS(1713),
    [sym_log10_value] = ACTIONS(1713),
    [sym_power_value] = ACTIONS(1713),
    [sym_sqrt_value] = ACTIONS(1713),
    [sym_raw_be] = ACTIONS(1713),
    [sym_raw_le] = ACTIONS(1713),
    [sym_internal_queue_size_set] = ACTIONS(1713),
    [sym_internal_queue_size_get] = ACTIONS(1713),
    [sym_internal_threshold_set] = ACTIONS(1713),
    [sym_internal_threshold_get] = ACTIONS(1713),
    [sym_rebuilt_event] = ACTIONS(1713),
    [sym_do_custom_event] = ACTIONS(1713),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1713),
    [sym_clkmgr_set_clk_source] = ACTIONS(1713),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1713),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1713),
    [sym_clkmgr_get_timezone] = ACTIONS(1713),
    [sym_clkmgr_set_timezone] = ACTIONS(1713),
    [sym_clkmgr_get_resync_period] = ACTIONS(1713),
    [sym_clkmgr_set_resync_period] = ACTIONS(1713),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1713),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1713),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1713),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1713),
    [sym_clkmgr_get_timeservers] = ACTIONS(1713),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1713),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1713),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1713),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1713),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1713),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1713),
    [sym_set_log_level] = ACTIONS(1713),
    [sym_get_log_level] = ACTIONS(1713),
    [sym_amx_log] = ACTIONS(1713),
    [sym_smtp_server_config_set] = ACTIONS(1713),
    [sym_smtp_server_config_get] = ACTIONS(1713),
    [sym_smtp_send] = ACTIONS(1713),
    [sym_validate_netlinx_account] = ACTIONS(1713),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1713),
    [sym_authenticate_certificate] = ACTIONS(1713),
    [sym_audit_netlinx_session_event] = ACTIONS(1713),
    [sym_audit_netlinx_generic_event] = ACTIONS(1713),
    [sym_get_available_flash_disk_space] = ACTIONS(1713),
    [sym_get_max_flash_disk_space] = ACTIONS(1713),
    [sym_data_create_feed] = ACTIONS(1713),
    [sym_data_delete_feed] = ACTIONS(1713),
    [sym_data_publish_feed] = ACTIONS(1713),
    [sym_data_get_published_feed] = ACTIONS(1713),
    [sym_data_add_record] = ACTIONS(1713),
    [sym_data_get_event_record] = ACTIONS(1713),
    [sym__wc_data_create_feed] = ACTIONS(1713),
    [sym__wc_data_add_record] = ACTIONS(1713),
    [sym__wc_data_get_event_record] = ACTIONS(1713),
    [sym_format] = ACTIONS(1713),
    [sym_astro_clock] = ACTIONS(1713),
    [sym_set_outdoor_temperature] = ACTIONS(1713),
    [sym_set_virtual_level_count] = ACTIONS(1713),
    [sym_set_virtual_channel_count] = ACTIONS(1713),
    [sym_set_virtual_port_count] = ACTIONS(1713),
    [sym__wc_encode_int] = ACTIONS(1713),
    [sym__wc_decode_int] = ACTIONS(1713),
    [sym__wc_int] = ACTIONS(1713),
    [sym__wc_to_ch_int] = ACTIONS(1713),
    [sym__ch_to_wc_int] = ACTIONS(1713),
    [sym__wc_find_string_int] = ACTIONS(1713),
    [sym__wc_left_string_int] = ACTIONS(1713),
    [sym__wc_lower_string_int] = ACTIONS(1713),
    [sym__wc_mid_string_int] = ACTIONS(1713),
    [sym__wc_remove_string_int] = ACTIONS(1713),
    [sym__wc_right_string_int] = ACTIONS(1713),
    [sym__wc_upper_string_int] = ACTIONS(1713),
    [sym__wc_compare_string_int] = ACTIONS(1713),
    [sym__wc_get_buffer_char_int] = ACTIONS(1713),
    [sym__wc_get_buffer_string_int] = ACTIONS(1713),
    [sym__wc_concat_string_int] = ACTIONS(1713),
    [sym__wc_file_open_int] = ACTIONS(1713),
    [sym__wc_file_close_int] = ACTIONS(1713),
    [sym__wc_file_read_int] = ACTIONS(1713),
    [sym__wc_file_read_line_int] = ACTIONS(1713),
    [sym__wc_file_write_int] = ACTIONS(1713),
    [sym__wc_file_write_line_int] = ACTIONS(1713),
    [aux_sym_preproc_include_token1] = ACTIONS(1713),
    [aux_sym_preproc_define_token1] = ACTIONS(1711),
    [aux_sym_preproc_warn_token1] = ACTIONS(1711),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1711),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1711),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1711),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1711),
    [aux_sym_preproc_else_token1] = ACTIONS(1711),
    [anon_sym_LBRACK] = ACTIONS(1711),
    [anon_sym_LBRACE] = ACTIONS(1711),
    [anon_sym_LPAREN] = ACTIONS(1713),
    [anon_sym_BANG] = ACTIONS(1711),
    [anon_sym_TILDE] = ACTIONS(1711),
    [anon_sym_DASH] = ACTIONS(1713),
    [anon_sym_PLUS] = ACTIONS(1713),
    [anon_sym_DASH_DASH] = ACTIONS(1711),
    [anon_sym_PLUS_PLUS] = ACTIONS(1711),
    [anon_sym_DQUOTE] = ACTIONS(1711),
    [anon_sym_SQUOTE] = ACTIONS(1711),
    [sym_number_literal] = ACTIONS(1711),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(350)] = {
    [sym_identifier] = ACTIONS(2061),
    [sym_program_name_keyword] = ACTIONS(2061),
    [sym_module_name_keyword] = ACTIONS(2061),
    [sym_define_device_keyword] = ACTIONS(2061),
    [sym_define_combine_keyword] = ACTIONS(2061),
    [sym_define_constant_keyword] = ACTIONS(2061),
    [sym_define_type_keyword] = ACTIONS(2061),
    [sym_define_variable_keyword] = ACTIONS(2061),
    [sym_define_system_variable_keyword] = ACTIONS(2061),
    [sym_define_connect_level_keyword] = ACTIONS(2061),
    [sym_define_latching_keyword] = ACTIONS(2061),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2061),
    [sym_define_toggling_keyword] = ACTIONS(2061),
    [sym_define_call_keyword] = ACTIONS(2061),
    [sym_define_function_keyword] = ACTIONS(2061),
    [sym_define_library_function_keyword] = ACTIONS(2061),
    [sym_define_start_keyword] = ACTIONS(2061),
    [sym_define_event_keyword] = ACTIONS(2061),
    [sym_define_program_keyword] = ACTIONS(2061),
    [sym_define_module_keyword] = ACTIONS(2061),
    [sym_char_keyword] = ACTIONS(2061),
    [sym_widechar_keyword] = ACTIONS(2061),
    [sym_integer_keyword] = ACTIONS(2061),
    [sym_sinteger_keyword] = ACTIONS(2061),
    [sym_long_keyword] = ACTIONS(2061),
    [sym_slong_keyword] = ACTIONS(2061),
    [sym_float_keyword] = ACTIONS(2061),
    [sym_double_keyword] = ACTIONS(2061),
    [sym_variant_keyword] = ACTIONS(2061),
    [sym_variantarray_keyword] = ACTIONS(2061),
    [sym_local_var_keyword] = ACTIONS(2061),
    [sym_stack_var_keyword] = ACTIONS(2061),
    [sym_constant_keyword] = ACTIONS(2061),
    [sym_volatile_keyword] = ACTIONS(2061),
    [sym_non_volatile_keyword] = ACTIONS(2061),
    [sym_persistent_keyword] = ACTIONS(2061),
    [sym_dev_keyword] = ACTIONS(2061),
    [sym_devlev_keyword] = ACTIONS(2061),
    [sym_devchan_keyword] = ACTIONS(2061),
    [sym_if_keyword] = ACTIONS(2061),
    [sym_switch_keyword] = ACTIONS(2061),
    [sym_case_keyword] = ACTIONS(2061),
    [sym_default_keyword] = ACTIONS(2061),
    [sym_while_keyword] = ACTIONS(2061),
    [sym_for_keyword] = ACTIONS(2061),
    [sym_break_keyword] = ACTIONS(2061),
    [sym_continue_keyword] = ACTIONS(2061),
    [sym_return_keyword] = ACTIONS(2061),
    [sym_select_keyword] = ACTIONS(2061),
    [sym_active_keyword] = ACTIONS(2061),
    [sym_struct_keyword] = ACTIONS(2061),
    [sym_structure_keyword] = ACTIONS(2061),
    [sym_button_event_keyword] = ACTIONS(2061),
    [sym_channel_event_keyword] = ACTIONS(2061),
    [sym_level_event_keyword] = ACTIONS(2061),
    [sym_data_event_keyword] = ACTIONS(2061),
    [sym_timeline_event_keyword] = ACTIONS(2061),
    [sym_custom_event_keyword] = ACTIONS(2061),
    [sym_push_keyword] = ACTIONS(2061),
    [sym_release_keyword] = ACTIONS(2061),
    [aux_sym_on_keyword_token1] = ACTIONS(2061),
    [aux_sym_off_keyword_token1] = ACTIONS(2061),
    [sym_send_level_keyword] = ACTIONS(2061),
    [sym_send_string_keyword] = ACTIONS(2061),
    [sym_send_command_keyword] = ACTIONS(2061),
    [sym_clear_buffer_keyword] = ACTIONS(2061),
    [sym_create_buffer_keyword] = ACTIONS(2061),
    [sym_create_multi_buffer_keyword] = ACTIONS(2061),
    [sym_call_keyword] = ACTIONS(2061),
    [sym_system_call_keyword] = ACTIONS(2061),
    [sym_devchan_to_keyword] = ACTIONS(2061),
    [sym_devchan_min_to_keyword] = ACTIONS(2061),
    [sym_devchan_total_off_keyword] = ACTIONS(2061),
    [sym_devchan_pulse_keyword] = ACTIONS(2061),
    [sym_wait_keyword] = ACTIONS(2061),
    [sym_cancel_wait_keyword] = ACTIONS(2061),
    [sym_wait_until_keyword] = ACTIONS(2061),
    [sym_cancel_wait_until_keyword] = ACTIONS(2061),
    [sym_cancel_all_wait_keyword] = ACTIONS(2061),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2061),
    [sym_not] = ACTIONS(2061),
    [sym_bnot] = ACTIONS(2061),
    [sym___file__] = ACTIONS(2061),
    [sym___date__] = ACTIONS(2061),
    [sym___ldate__] = ACTIONS(2061),
    [sym___time__] = ACTIONS(2061),
    [sym___line__] = ACTIONS(2061),
    [sym___name__] = ACTIONS(2061),
    [sym___netlinx__] = ACTIONS(2061),
    [sym_netlinx_axi_version] = ACTIONS(2061),
    [sym_url_flg_tcp] = ACTIONS(2061),
    [sym_url_flg_accinfopresent] = ACTIONS(2061),
    [sym_url_flg_temp] = ACTIONS(2061),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2061),
    [sym_url_flg_stat_mask] = ACTIONS(2061),
    [sym_url_flg_stat_lookup] = ACTIONS(2061),
    [sym_url_flg_stat_connecting] = ACTIONS(2061),
    [sym_url_flg_stat_waiting] = ACTIONS(2061),
    [sym_url_flg_stat_connected] = ACTIONS(2061),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2061),
    [sym_first_virtual_device] = ACTIONS(2061),
    [sym_first_local_port] = ACTIONS(2061),
    [sym_dynamic_virtual_device] = ACTIONS(2061),
    [sym_timeline_once] = ACTIONS(2061),
    [sym_timeline_repeat] = ACTIONS(2061),
    [sym_timeline_absolute] = ACTIONS(2061),
    [sym_timeline_relative] = ACTIONS(2061),
    [sym_do_push_timed_infinite] = ACTIONS(2061),
    [sym_source_type_no_address] = ACTIONS(2061),
    [sym_source_type_neuron_id] = ACTIONS(2061),
    [sym_source_type_ip_address] = ACTIONS(2061),
    [sym_source_type_axlink] = ACTIONS(2061),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2061),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2061),
    [sym_source_type_ip_socket_address] = ACTIONS(2061),
    [sym_source_type_rs232] = ACTIONS(2061),
    [sym_source_type_internal] = ACTIONS(2061),
    [sym_source_type_mac_address] = ACTIONS(2061),
    [sym_source_type_ipv4_port] = ACTIONS(2061),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2061),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2061),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2061),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2061),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2061),
    [sym_true] = ACTIONS(2061),
    [sym_false] = ACTIONS(2061),
    [sym_file_read_only] = ACTIONS(2061),
    [sym_file_rw_new] = ACTIONS(2061),
    [sym_file_rw_append] = ACTIONS(2061),
    [sym_ip_tcp] = ACTIONS(2061),
    [sym_ip_udp] = ACTIONS(2061),
    [sym_ip_udp_2way] = ACTIONS(2061),
    [sym_xml_encode_types] = ACTIONS(2061),
    [sym_xml_encode_char_as_list] = ACTIONS(2061),
    [sym_xml_encode_le] = ACTIONS(2061),
    [sym_xml_decode_types] = ACTIONS(2061),
    [sym_xml_decode_no_preserve] = ACTIONS(2061),
    [sym_duet_dev_type_display_device] = ACTIONS(2061),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2061),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2061),
    [sym_duet_dev_type_io_device] = ACTIONS(2061),
    [sym_duet_dev_type_relay_device] = ACTIONS(2061),
    [sym_duet_dev_type_ups] = ACTIONS(2061),
    [sym_duet_dev_type_amplifier] = ACTIONS(2061),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2061),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2061),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2061),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2061),
    [sym_duet_dev_type_camera] = ACTIONS(2061),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2061),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2061),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2061),
    [sym_duet_dev_type_dss] = ACTIONS(2061),
    [sym_duet_dev_type_dvr] = ACTIONS(2061),
    [sym_duet_dev_type_disc_device] = ACTIONS(2061),
    [sym_duet_dev_type_document_camera] = ACTIONS(2061),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2061),
    [sym_duet_dev_type_hvac] = ACTIONS(2061),
    [sym_duet_dev_type_keypad] = ACTIONS(2061),
    [sym_duet_dev_type_light] = ACTIONS(2061),
    [sym_duet_dev_type_monitor] = ACTIONS(2061),
    [sym_duet_dev_type_motor] = ACTIONS(2061),
    [sym_duet_dev_type_multi_window] = ACTIONS(2061),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2061),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2061),
    [sym_duet_dev_type_receiver] = ACTIONS(2061),
    [sym_duet_dev_type_security_system] = ACTIONS(2061),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2061),
    [sym_duet_dev_type_settop_box] = ACTIONS(2061),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2061),
    [sym_duet_dev_type_switcher] = ACTIONS(2061),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2061),
    [sym_duet_dev_type_tv] = ACTIONS(2061),
    [sym_duet_dev_type_utility] = ACTIONS(2061),
    [sym_duet_dev_type_vcr] = ACTIONS(2061),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2061),
    [sym_duet_dev_type_video_processor] = ACTIONS(2061),
    [sym_duet_dev_type_video_projector] = ACTIONS(2061),
    [sym_duet_dev_type_video_wall] = ACTIONS(2061),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2061),
    [sym_duet_dev_type_weather] = ACTIONS(2061),
    [sym_duet_dev_not_polled] = ACTIONS(2061),
    [sym_duet_dev_polled] = ACTIONS(2061),
    [sym_clkmgr_mode_network] = ACTIONS(2061),
    [sym_clkmgr_mode_standalone] = ACTIONS(2061),
    [sym_normal_standby] = ACTIONS(2061),
    [sym_normal_wake] = ACTIONS(2061),
    [sym_ip_multicast_ttl_option] = ACTIONS(2061),
    [sym_ip_ttl_subnet] = ACTIONS(2061),
    [sym_ip_ttl_site] = ACTIONS(2061),
    [sym_ip_ttl_region] = ACTIONS(2061),
    [sym_ip_ttl_continent] = ACTIONS(2061),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2061),
    [sym_ip_nodelay_on] = ACTIONS(2061),
    [sym_ip_nodelay_off] = ACTIONS(2061),
    [sym_amx_error] = ACTIONS(2061),
    [sym_amx_warning] = ACTIONS(2061),
    [sym_amx_info] = ACTIONS(2061),
    [sym_amx_debug] = ACTIONS(2061),
    [sym_netlinx_login_success] = ACTIONS(2061),
    [sym_netlinx_login_fail] = ACTIONS(2061),
    [sym_netlinx_logout] = ACTIONS(2061),
    [sym_null_str] = ACTIONS(2061),
    [sym_smtp_address] = ACTIONS(2061),
    [sym_smtp_port_number] = ACTIONS(2061),
    [sym_smtp_username] = ACTIONS(2061),
    [sym_smtp_password] = ACTIONS(2061),
    [sym_smtp_require_tls] = ACTIONS(2061),
    [sym_smtp_from] = ACTIONS(2061),
    [sym_smtp_tls_true] = ACTIONS(2061),
    [sym_smtp_tls_false] = ACTIONS(2061),
    [sym_listview_on_row_select_event] = ACTIONS(2061),
    [sym_data_structure_unknown] = ACTIONS(2061),
    [sym_data_structure_datafeed] = ACTIONS(2061),
    [sym_data_structure_datarecordset] = ACTIONS(2061),
    [sym_data_structure_datarecord] = ACTIONS(2061),
    [sym_data_structure_datafield] = ACTIONS(2061),
    [sym_data_max_content_field_count] = ACTIONS(2061),
    [sym_data_max_metadata_field_count] = ACTIONS(2061),
    [sym_data_max_id_length] = ACTIONS(2061),
    [sym_data_max_name_length] = ACTIONS(2061),
    [sym_data_max_description_length] = ACTIONS(2061),
    [sym_data_max_type_length] = ACTIONS(2061),
    [sym_data_max_format_length] = ACTIONS(2061),
    [sym_data_max_source_length] = ACTIONS(2061),
    [sym_data_max_value_length] = ACTIONS(2061),
    [sym_data_max_label_length] = ACTIONS(2061),
    [sym_data_max_url_length] = ACTIONS(2061),
    [sym_max_last_login_info_length] = ACTIONS(2061),
    [sym_data_type_unknown] = ACTIONS(2061),
    [sym_data_type_string] = ACTIONS(2061),
    [sym_data_type_datetime] = ACTIONS(2061),
    [sym_data_type_date] = ACTIONS(2061),
    [sym_data_type_time] = ACTIONS(2061),
    [sym_data_type_image] = ACTIONS(2061),
    [sym_data_format_url] = ACTIONS(2061),
    [sym_data_format_phone] = ACTIONS(2061),
    [sym_data_format_email] = ACTIONS(2061),
    [sym_data_format_iso8601] = ACTIONS(2061),
    [sym_auditlog_priv] = ACTIONS(2061),
    [sym_configuration_priv] = ACTIONS(2061),
    [sym_deviceconfig_priv] = ACTIONS(2061),
    [sym_ftp_priv] = ACTIONS(2061),
    [sym_http_priv] = ACTIONS(2061),
    [sym_networkconfig_priv] = ACTIONS(2061),
    [sym_programport_priv] = ACTIONS(2061),
    [sym_remoteui_priv] = ACTIONS(2061),
    [sym_securitycontrol_priv] = ACTIONS(2061),
    [sym_softwaremanagement_priv] = ACTIONS(2061),
    [sym_terminal_priv] = ACTIONS(2061),
    [sym_tpadmin_priv] = ACTIONS(2061),
    [sym_user1_priv] = ACTIONS(2061),
    [sym_user2_priv] = ACTIONS(2061),
    [sym_user3_priv] = ACTIONS(2061),
    [sym_user4_priv] = ACTIONS(2061),
    [sym_usermanagement_priv] = ACTIONS(2061),
    [sym_valid_account] = ACTIONS(2061),
    [sym_err_name_invalid] = ACTIONS(2061),
    [sym_err_password_invalid] = ACTIONS(2061),
    [sym_err_invalid_account] = ACTIONS(2061),
    [sym_err_no_authorization] = ACTIONS(2061),
    [sym_err_invalid_parameter] = ACTIONS(2061),
    [sym_err_user_locked_out] = ACTIONS(2061),
    [sym_err_user_account_expired] = ACTIONS(2061),
    [sym_err_general_error] = ACTIONS(2061),
    [sym_err_authentication_error] = ACTIONS(2061),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2061),
    [sym_tls_validate_certificate] = ACTIONS(2061),
    [sym_tdata] = ACTIONS(2061),
    [sym_tchannel] = ACTIONS(2061),
    [sym_tlevel] = ACTIONS(2061),
    [sym_tbutton] = ACTIONS(2061),
    [sym_ttimeline] = ACTIONS(2061),
    [sym_tcustom] = ACTIONS(2061),
    [sym_url_struct] = ACTIONS(2061),
    [sym_dns_struct] = ACTIONS(2061),
    [sym_ip_address_struct] = ACTIONS(2061),
    [sym_dev_info_struct] = ACTIONS(2061),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2061),
    [sym_data_feed] = ACTIONS(2061),
    [sym_data_field] = ACTIONS(2061),
    [sym_data_record] = ACTIONS(2061),
    [sym_wc_data_feed] = ACTIONS(2061),
    [sym_wc_data_field] = ACTIONS(2061),
    [sym_wc_data_record] = ACTIONS(2061),
    [sym_last_login_info] = ACTIONS(2061),
    [sym____reserved___] = ACTIONS(2061),
    [sym_date] = ACTIONS(2061),
    [sym_day] = ACTIONS(2061),
    [sym_time] = ACTIONS(2061),
    [sym_ldate] = ACTIONS(2061),
    [sym_push_device] = ACTIONS(2061),
    [sym_push_channel] = ACTIONS(2061),
    [sym_push_devchan] = ACTIONS(2061),
    [sym_release_device] = ACTIONS(2061),
    [sym_release_channel] = ACTIONS(2061),
    [sym_release_devchan] = ACTIONS(2061),
    [sym_master_slot] = ACTIONS(2061),
    [sym_get_pulse_time] = ACTIONS(2061),
    [sym_get_timer] = ACTIONS(2061),
    [sym_master_sn] = ACTIONS(2061),
    [sym_system_number] = ACTIONS(2061),
    [sym_button] = ACTIONS(2061),
    [sym_dv_channel] = ACTIONS(2061),
    [sym_data] = ACTIONS(2061),
    [sym_level] = ACTIONS(2061),
    [sym_timeline] = ACTIONS(2061),
    [sym_channel] = ACTIONS(2061),
    [sym_custom] = ACTIONS(2061),
    [sym_atoi] = ACTIONS(2061),
    [sym_atol] = ACTIONS(2061),
    [sym_atof] = ACTIONS(2061),
    [sym_itoa] = ACTIONS(2061),
    [sym_itohex] = ACTIONS(2061),
    [sym_hextoi] = ACTIONS(2061),
    [sym_ftoa] = ACTIONS(2061),
    [sym_find_string] = ACTIONS(2061),
    [sym_left_string] = ACTIONS(2061),
    [sym_right_string] = ACTIONS(2061),
    [sym_mid_string] = ACTIONS(2061),
    [sym_remove_string] = ACTIONS(2061),
    [sym_get_buffer_string] = ACTIONS(2061),
    [sym_get_buffer_char] = ACTIONS(2061),
    [sym_get_multi_buffer_string] = ACTIONS(2061),
    [sym_length_string] = ACTIONS(2061),
    [sym_max_length_string] = ACTIONS(2061),
    [sym_set_length_string] = ACTIONS(2061),
    [sym_compare_string] = ACTIONS(2061),
    [sym_upper_string] = ACTIONS(2061),
    [sym_lower_string] = ACTIONS(2061),
    [sym_get_last] = ACTIONS(2061),
    [sym_date_to_day] = ACTIONS(2061),
    [sym_date_to_month] = ACTIONS(2061),
    [sym_date_to_year] = ACTIONS(2061),
    [sym_time_to_hour] = ACTIONS(2061),
    [sym_time_to_minute] = ACTIONS(2061),
    [sym_time_to_second] = ACTIONS(2061),
    [sym_day_of_week] = ACTIONS(2061),
    [sym_set_pulse_time] = ACTIONS(2061),
    [sym_set_timer] = ACTIONS(2061),
    [sym_file_open] = ACTIONS(2061),
    [sym_file_close] = ACTIONS(2061),
    [sym_file_read] = ACTIONS(2061),
    [sym_file_read_line] = ACTIONS(2061),
    [sym_file_write] = ACTIONS(2061),
    [sym_file_write_line] = ACTIONS(2061),
    [sym_file_copy] = ACTIONS(2061),
    [sym_file_delete] = ACTIONS(2061),
    [sym_file_rename] = ACTIONS(2061),
    [sym_file_dir] = ACTIONS(2061),
    [sym_file_getdir] = ACTIONS(2061),
    [sym_file_setdir] = ACTIONS(2061),
    [sym_file_create_dir] = ACTIONS(2061),
    [sym_file_remove_dir] = ACTIONS(2061),
    [sym_file_seek] = ACTIONS(2061),
    [sym_length_array] = ACTIONS(2061),
    [sym_max_length_array] = ACTIONS(2061),
    [sym_set_length_array] = ACTIONS(2061),
    [sym_ip_client_open] = ACTIONS(2061),
    [sym_ip_client_close] = ACTIONS(2061),
    [sym_ip_server_open] = ACTIONS(2061),
    [sym_ip_server_close] = ACTIONS(2061),
    [sym_ip_bound_client_open] = ACTIONS(2061),
    [sym_ip_mc_server_open] = ACTIONS(2061),
    [sym_ip_set_option] = ACTIONS(2061),
    [sym_ssh_client_open] = ACTIONS(2061),
    [sym_ssh_client_close] = ACTIONS(2061),
    [sym_tls_client_open] = ACTIONS(2061),
    [sym_tls_client_close] = ACTIONS(2061),
    [sym_do_push] = ACTIONS(2061),
    [sym_do_push_timed] = ACTIONS(2061),
    [sym_do_release] = ACTIONS(2061),
    [sym_device_id] = ACTIONS(2061),
    [sym_device_info] = ACTIONS(2061),
    [sym_device_id_string] = ACTIONS(2061),
    [sym_device_standby] = ACTIONS(2061),
    [sym_device_wake] = ACTIONS(2061),
    [sym_timeline_create] = ACTIONS(2061),
    [sym_timeline_kill] = ACTIONS(2061),
    [sym_timeline_reload] = ACTIONS(2061),
    [sym_timeline_pause] = ACTIONS(2061),
    [sym_timeline_restart] = ACTIONS(2061),
    [sym_timeline_set] = ACTIONS(2061),
    [sym_timeline_get] = ACTIONS(2061),
    [sym_timeline_active] = ACTIONS(2061),
    [sym_timeline_dynamic_id] = ACTIONS(2061),
    [sym_variable_to_string] = ACTIONS(2061),
    [sym_string_to_variable] = ACTIONS(2061),
    [sym_length_variable_to_string] = ACTIONS(2061),
    [sym_type_cast] = ACTIONS(2061),
    [sym_variable_to_xml] = ACTIONS(2061),
    [sym_xml_to_variable] = ACTIONS(2061),
    [sym_length_variable_to_xml] = ACTIONS(2061),
    [sym_redirect_string] = ACTIONS(2061),
    [sym_random_number] = ACTIONS(2061),
    [sym_get_unique_id] = ACTIONS(2061),
    [sym_get_system_number] = ACTIONS(2061),
    [sym_set_system_number] = ACTIONS(2061),
    [sym_get_serial_number] = ACTIONS(2061),
    [sym_get_master_build] = ACTIONS(2061),
    [sym_set_validation_code] = ACTIONS(2061),
    [sym_reboot] = ACTIONS(2061),
    [sym_load_duet_module] = ACTIONS(2061),
    [sym_unload_duet_module] = ACTIONS(2061),
    [sym_duet_mem_size_set] = ACTIONS(2061),
    [sym_duet_mem_size_get] = ACTIONS(2061),
    [sym_dynamic_application_device] = ACTIONS(2061),
    [sym_dynamic_polled_port] = ACTIONS(2061),
    [sym_static_port_binding] = ACTIONS(2061),
    [sym_static_ip_binding] = ACTIONS(2061),
    [sym_get_dns_list] = ACTIONS(2061),
    [sym_set_dns_list] = ACTIONS(2061),
    [sym_get_ip_address] = ACTIONS(2061),
    [sym_set_ip_address] = ACTIONS(2061),
    [sym_get_url_list] = ACTIONS(2061),
    [sym_add_url_entry] = ACTIONS(2061),
    [sym_delete_url_entry] = ACTIONS(2061),
    [sym_abs_value] = ACTIONS(2061),
    [sym_max_value] = ACTIONS(2061),
    [sym_min_value] = ACTIONS(2061),
    [sym_exp_value] = ACTIONS(2061),
    [sym_log_value] = ACTIONS(2061),
    [sym_log10_value] = ACTIONS(2061),
    [sym_power_value] = ACTIONS(2061),
    [sym_sqrt_value] = ACTIONS(2061),
    [sym_raw_be] = ACTIONS(2061),
    [sym_raw_le] = ACTIONS(2061),
    [sym_internal_queue_size_set] = ACTIONS(2061),
    [sym_internal_queue_size_get] = ACTIONS(2061),
    [sym_internal_threshold_set] = ACTIONS(2061),
    [sym_internal_threshold_get] = ACTIONS(2061),
    [sym_rebuilt_event] = ACTIONS(2061),
    [sym_do_custom_event] = ACTIONS(2061),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2061),
    [sym_clkmgr_set_clk_source] = ACTIONS(2061),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2061),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2061),
    [sym_clkmgr_get_timezone] = ACTIONS(2061),
    [sym_clkmgr_set_timezone] = ACTIONS(2061),
    [sym_clkmgr_get_resync_period] = ACTIONS(2061),
    [sym_clkmgr_set_resync_period] = ACTIONS(2061),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2061),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2061),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2061),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2061),
    [sym_clkmgr_get_timeservers] = ACTIONS(2061),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2061),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2061),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2061),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2061),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2061),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2061),
    [sym_set_log_level] = ACTIONS(2061),
    [sym_get_log_level] = ACTIONS(2061),
    [sym_amx_log] = ACTIONS(2061),
    [sym_smtp_server_config_set] = ACTIONS(2061),
    [sym_smtp_server_config_get] = ACTIONS(2061),
    [sym_smtp_send] = ACTIONS(2061),
    [sym_validate_netlinx_account] = ACTIONS(2061),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2061),
    [sym_authenticate_certificate] = ACTIONS(2061),
    [sym_audit_netlinx_session_event] = ACTIONS(2061),
    [sym_audit_netlinx_generic_event] = ACTIONS(2061),
    [sym_get_available_flash_disk_space] = ACTIONS(2061),
    [sym_get_max_flash_disk_space] = ACTIONS(2061),
    [sym_data_create_feed] = ACTIONS(2061),
    [sym_data_delete_feed] = ACTIONS(2061),
    [sym_data_publish_feed] = ACTIONS(2061),
    [sym_data_get_published_feed] = ACTIONS(2061),
    [sym_data_add_record] = ACTIONS(2061),
    [sym_data_get_event_record] = ACTIONS(2061),
    [sym__wc_data_create_feed] = ACTIONS(2061),
    [sym__wc_data_add_record] = ACTIONS(2061),
    [sym__wc_data_get_event_record] = ACTIONS(2061),
    [sym_format] = ACTIONS(2061),
    [sym_astro_clock] = ACTIONS(2061),
    [sym_set_outdoor_temperature] = ACTIONS(2061),
    [sym_set_virtual_level_count] = ACTIONS(2061),
    [sym_set_virtual_channel_count] = ACTIONS(2061),
    [sym_set_virtual_port_count] = ACTIONS(2061),
    [sym__wc_encode_int] = ACTIONS(2061),
    [sym__wc_decode_int] = ACTIONS(2061),
    [sym__wc_int] = ACTIONS(2061),
    [sym__wc_to_ch_int] = ACTIONS(2061),
    [sym__ch_to_wc_int] = ACTIONS(2061),
    [sym__wc_find_string_int] = ACTIONS(2061),
    [sym__wc_left_string_int] = ACTIONS(2061),
    [sym__wc_lower_string_int] = ACTIONS(2061),
    [sym__wc_mid_string_int] = ACTIONS(2061),
    [sym__wc_remove_string_int] = ACTIONS(2061),
    [sym__wc_right_string_int] = ACTIONS(2061),
    [sym__wc_upper_string_int] = ACTIONS(2061),
    [sym__wc_compare_string_int] = ACTIONS(2061),
    [sym__wc_get_buffer_char_int] = ACTIONS(2061),
    [sym__wc_get_buffer_string_int] = ACTIONS(2061),
    [sym__wc_concat_string_int] = ACTIONS(2061),
    [sym__wc_file_open_int] = ACTIONS(2061),
    [sym__wc_file_close_int] = ACTIONS(2061),
    [sym__wc_file_read_int] = ACTIONS(2061),
    [sym__wc_file_read_line_int] = ACTIONS(2061),
    [sym__wc_file_write_int] = ACTIONS(2061),
    [sym__wc_file_write_line_int] = ACTIONS(2061),
    [aux_sym_preproc_include_token1] = ACTIONS(2061),
    [aux_sym_preproc_define_token1] = ACTIONS(2063),
    [aux_sym_preproc_warn_token1] = ACTIONS(2063),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2063),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2063),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2063),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2063),
    [aux_sym_preproc_else_token1] = ACTIONS(2063),
    [anon_sym_LBRACK] = ACTIONS(2063),
    [anon_sym_LBRACE] = ACTIONS(2063),
    [anon_sym_LPAREN] = ACTIONS(2061),
    [anon_sym_BANG] = ACTIONS(2063),
    [anon_sym_TILDE] = ACTIONS(2063),
    [anon_sym_DASH] = ACTIONS(2061),
    [anon_sym_PLUS] = ACTIONS(2061),
    [anon_sym_DASH_DASH] = ACTIONS(2063),
    [anon_sym_PLUS_PLUS] = ACTIONS(2063),
    [anon_sym_DQUOTE] = ACTIONS(2063),
    [anon_sym_SQUOTE] = ACTIONS(2063),
    [sym_number_literal] = ACTIONS(2063),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(351)] = {
    [sym_identifier] = ACTIONS(2065),
    [sym_program_name_keyword] = ACTIONS(2065),
    [sym_module_name_keyword] = ACTIONS(2065),
    [sym_define_device_keyword] = ACTIONS(2065),
    [sym_define_combine_keyword] = ACTIONS(2065),
    [sym_define_constant_keyword] = ACTIONS(2065),
    [sym_define_type_keyword] = ACTIONS(2065),
    [sym_define_variable_keyword] = ACTIONS(2065),
    [sym_define_system_variable_keyword] = ACTIONS(2065),
    [sym_define_connect_level_keyword] = ACTIONS(2065),
    [sym_define_latching_keyword] = ACTIONS(2065),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2065),
    [sym_define_toggling_keyword] = ACTIONS(2065),
    [sym_define_call_keyword] = ACTIONS(2065),
    [sym_define_function_keyword] = ACTIONS(2065),
    [sym_define_library_function_keyword] = ACTIONS(2065),
    [sym_define_start_keyword] = ACTIONS(2065),
    [sym_define_event_keyword] = ACTIONS(2065),
    [sym_define_program_keyword] = ACTIONS(2065),
    [sym_define_module_keyword] = ACTIONS(2065),
    [sym_char_keyword] = ACTIONS(2065),
    [sym_widechar_keyword] = ACTIONS(2065),
    [sym_integer_keyword] = ACTIONS(2065),
    [sym_sinteger_keyword] = ACTIONS(2065),
    [sym_long_keyword] = ACTIONS(2065),
    [sym_slong_keyword] = ACTIONS(2065),
    [sym_float_keyword] = ACTIONS(2065),
    [sym_double_keyword] = ACTIONS(2065),
    [sym_variant_keyword] = ACTIONS(2065),
    [sym_variantarray_keyword] = ACTIONS(2065),
    [sym_local_var_keyword] = ACTIONS(2065),
    [sym_stack_var_keyword] = ACTIONS(2065),
    [sym_constant_keyword] = ACTIONS(2065),
    [sym_volatile_keyword] = ACTIONS(2065),
    [sym_non_volatile_keyword] = ACTIONS(2065),
    [sym_persistent_keyword] = ACTIONS(2065),
    [sym_dev_keyword] = ACTIONS(2065),
    [sym_devlev_keyword] = ACTIONS(2065),
    [sym_devchan_keyword] = ACTIONS(2065),
    [sym_if_keyword] = ACTIONS(2065),
    [sym_switch_keyword] = ACTIONS(2065),
    [sym_case_keyword] = ACTIONS(2065),
    [sym_default_keyword] = ACTIONS(2065),
    [sym_while_keyword] = ACTIONS(2065),
    [sym_for_keyword] = ACTIONS(2065),
    [sym_break_keyword] = ACTIONS(2065),
    [sym_continue_keyword] = ACTIONS(2065),
    [sym_return_keyword] = ACTIONS(2065),
    [sym_select_keyword] = ACTIONS(2065),
    [sym_active_keyword] = ACTIONS(2065),
    [sym_struct_keyword] = ACTIONS(2065),
    [sym_structure_keyword] = ACTIONS(2065),
    [sym_button_event_keyword] = ACTIONS(2065),
    [sym_channel_event_keyword] = ACTIONS(2065),
    [sym_level_event_keyword] = ACTIONS(2065),
    [sym_data_event_keyword] = ACTIONS(2065),
    [sym_timeline_event_keyword] = ACTIONS(2065),
    [sym_custom_event_keyword] = ACTIONS(2065),
    [sym_push_keyword] = ACTIONS(2065),
    [sym_release_keyword] = ACTIONS(2065),
    [aux_sym_on_keyword_token1] = ACTIONS(2065),
    [aux_sym_off_keyword_token1] = ACTIONS(2065),
    [sym_send_level_keyword] = ACTIONS(2065),
    [sym_send_string_keyword] = ACTIONS(2065),
    [sym_send_command_keyword] = ACTIONS(2065),
    [sym_clear_buffer_keyword] = ACTIONS(2065),
    [sym_create_buffer_keyword] = ACTIONS(2065),
    [sym_create_multi_buffer_keyword] = ACTIONS(2065),
    [sym_call_keyword] = ACTIONS(2065),
    [sym_system_call_keyword] = ACTIONS(2065),
    [sym_devchan_to_keyword] = ACTIONS(2065),
    [sym_devchan_min_to_keyword] = ACTIONS(2065),
    [sym_devchan_total_off_keyword] = ACTIONS(2065),
    [sym_devchan_pulse_keyword] = ACTIONS(2065),
    [sym_wait_keyword] = ACTIONS(2065),
    [sym_cancel_wait_keyword] = ACTIONS(2065),
    [sym_wait_until_keyword] = ACTIONS(2065),
    [sym_cancel_wait_until_keyword] = ACTIONS(2065),
    [sym_cancel_all_wait_keyword] = ACTIONS(2065),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2065),
    [sym_not] = ACTIONS(2065),
    [sym_bnot] = ACTIONS(2065),
    [sym___file__] = ACTIONS(2065),
    [sym___date__] = ACTIONS(2065),
    [sym___ldate__] = ACTIONS(2065),
    [sym___time__] = ACTIONS(2065),
    [sym___line__] = ACTIONS(2065),
    [sym___name__] = ACTIONS(2065),
    [sym___netlinx__] = ACTIONS(2065),
    [sym_netlinx_axi_version] = ACTIONS(2065),
    [sym_url_flg_tcp] = ACTIONS(2065),
    [sym_url_flg_accinfopresent] = ACTIONS(2065),
    [sym_url_flg_temp] = ACTIONS(2065),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2065),
    [sym_url_flg_stat_mask] = ACTIONS(2065),
    [sym_url_flg_stat_lookup] = ACTIONS(2065),
    [sym_url_flg_stat_connecting] = ACTIONS(2065),
    [sym_url_flg_stat_waiting] = ACTIONS(2065),
    [sym_url_flg_stat_connected] = ACTIONS(2065),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2065),
    [sym_first_virtual_device] = ACTIONS(2065),
    [sym_first_local_port] = ACTIONS(2065),
    [sym_dynamic_virtual_device] = ACTIONS(2065),
    [sym_timeline_once] = ACTIONS(2065),
    [sym_timeline_repeat] = ACTIONS(2065),
    [sym_timeline_absolute] = ACTIONS(2065),
    [sym_timeline_relative] = ACTIONS(2065),
    [sym_do_push_timed_infinite] = ACTIONS(2065),
    [sym_source_type_no_address] = ACTIONS(2065),
    [sym_source_type_neuron_id] = ACTIONS(2065),
    [sym_source_type_ip_address] = ACTIONS(2065),
    [sym_source_type_axlink] = ACTIONS(2065),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2065),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2065),
    [sym_source_type_ip_socket_address] = ACTIONS(2065),
    [sym_source_type_rs232] = ACTIONS(2065),
    [sym_source_type_internal] = ACTIONS(2065),
    [sym_source_type_mac_address] = ACTIONS(2065),
    [sym_source_type_ipv4_port] = ACTIONS(2065),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2065),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2065),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2065),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2065),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2065),
    [sym_true] = ACTIONS(2065),
    [sym_false] = ACTIONS(2065),
    [sym_file_read_only] = ACTIONS(2065),
    [sym_file_rw_new] = ACTIONS(2065),
    [sym_file_rw_append] = ACTIONS(2065),
    [sym_ip_tcp] = ACTIONS(2065),
    [sym_ip_udp] = ACTIONS(2065),
    [sym_ip_udp_2way] = ACTIONS(2065),
    [sym_xml_encode_types] = ACTIONS(2065),
    [sym_xml_encode_char_as_list] = ACTIONS(2065),
    [sym_xml_encode_le] = ACTIONS(2065),
    [sym_xml_decode_types] = ACTIONS(2065),
    [sym_xml_decode_no_preserve] = ACTIONS(2065),
    [sym_duet_dev_type_display_device] = ACTIONS(2065),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2065),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2065),
    [sym_duet_dev_type_io_device] = ACTIONS(2065),
    [sym_duet_dev_type_relay_device] = ACTIONS(2065),
    [sym_duet_dev_type_ups] = ACTIONS(2065),
    [sym_duet_dev_type_amplifier] = ACTIONS(2065),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2065),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2065),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2065),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2065),
    [sym_duet_dev_type_camera] = ACTIONS(2065),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2065),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2065),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2065),
    [sym_duet_dev_type_dss] = ACTIONS(2065),
    [sym_duet_dev_type_dvr] = ACTIONS(2065),
    [sym_duet_dev_type_disc_device] = ACTIONS(2065),
    [sym_duet_dev_type_document_camera] = ACTIONS(2065),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2065),
    [sym_duet_dev_type_hvac] = ACTIONS(2065),
    [sym_duet_dev_type_keypad] = ACTIONS(2065),
    [sym_duet_dev_type_light] = ACTIONS(2065),
    [sym_duet_dev_type_monitor] = ACTIONS(2065),
    [sym_duet_dev_type_motor] = ACTIONS(2065),
    [sym_duet_dev_type_multi_window] = ACTIONS(2065),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2065),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2065),
    [sym_duet_dev_type_receiver] = ACTIONS(2065),
    [sym_duet_dev_type_security_system] = ACTIONS(2065),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2065),
    [sym_duet_dev_type_settop_box] = ACTIONS(2065),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2065),
    [sym_duet_dev_type_switcher] = ACTIONS(2065),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2065),
    [sym_duet_dev_type_tv] = ACTIONS(2065),
    [sym_duet_dev_type_utility] = ACTIONS(2065),
    [sym_duet_dev_type_vcr] = ACTIONS(2065),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2065),
    [sym_duet_dev_type_video_processor] = ACTIONS(2065),
    [sym_duet_dev_type_video_projector] = ACTIONS(2065),
    [sym_duet_dev_type_video_wall] = ACTIONS(2065),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2065),
    [sym_duet_dev_type_weather] = ACTIONS(2065),
    [sym_duet_dev_not_polled] = ACTIONS(2065),
    [sym_duet_dev_polled] = ACTIONS(2065),
    [sym_clkmgr_mode_network] = ACTIONS(2065),
    [sym_clkmgr_mode_standalone] = ACTIONS(2065),
    [sym_normal_standby] = ACTIONS(2065),
    [sym_normal_wake] = ACTIONS(2065),
    [sym_ip_multicast_ttl_option] = ACTIONS(2065),
    [sym_ip_ttl_subnet] = ACTIONS(2065),
    [sym_ip_ttl_site] = ACTIONS(2065),
    [sym_ip_ttl_region] = ACTIONS(2065),
    [sym_ip_ttl_continent] = ACTIONS(2065),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2065),
    [sym_ip_nodelay_on] = ACTIONS(2065),
    [sym_ip_nodelay_off] = ACTIONS(2065),
    [sym_amx_error] = ACTIONS(2065),
    [sym_amx_warning] = ACTIONS(2065),
    [sym_amx_info] = ACTIONS(2065),
    [sym_amx_debug] = ACTIONS(2065),
    [sym_netlinx_login_success] = ACTIONS(2065),
    [sym_netlinx_login_fail] = ACTIONS(2065),
    [sym_netlinx_logout] = ACTIONS(2065),
    [sym_null_str] = ACTIONS(2065),
    [sym_smtp_address] = ACTIONS(2065),
    [sym_smtp_port_number] = ACTIONS(2065),
    [sym_smtp_username] = ACTIONS(2065),
    [sym_smtp_password] = ACTIONS(2065),
    [sym_smtp_require_tls] = ACTIONS(2065),
    [sym_smtp_from] = ACTIONS(2065),
    [sym_smtp_tls_true] = ACTIONS(2065),
    [sym_smtp_tls_false] = ACTIONS(2065),
    [sym_listview_on_row_select_event] = ACTIONS(2065),
    [sym_data_structure_unknown] = ACTIONS(2065),
    [sym_data_structure_datafeed] = ACTIONS(2065),
    [sym_data_structure_datarecordset] = ACTIONS(2065),
    [sym_data_structure_datarecord] = ACTIONS(2065),
    [sym_data_structure_datafield] = ACTIONS(2065),
    [sym_data_max_content_field_count] = ACTIONS(2065),
    [sym_data_max_metadata_field_count] = ACTIONS(2065),
    [sym_data_max_id_length] = ACTIONS(2065),
    [sym_data_max_name_length] = ACTIONS(2065),
    [sym_data_max_description_length] = ACTIONS(2065),
    [sym_data_max_type_length] = ACTIONS(2065),
    [sym_data_max_format_length] = ACTIONS(2065),
    [sym_data_max_source_length] = ACTIONS(2065),
    [sym_data_max_value_length] = ACTIONS(2065),
    [sym_data_max_label_length] = ACTIONS(2065),
    [sym_data_max_url_length] = ACTIONS(2065),
    [sym_max_last_login_info_length] = ACTIONS(2065),
    [sym_data_type_unknown] = ACTIONS(2065),
    [sym_data_type_string] = ACTIONS(2065),
    [sym_data_type_datetime] = ACTIONS(2065),
    [sym_data_type_date] = ACTIONS(2065),
    [sym_data_type_time] = ACTIONS(2065),
    [sym_data_type_image] = ACTIONS(2065),
    [sym_data_format_url] = ACTIONS(2065),
    [sym_data_format_phone] = ACTIONS(2065),
    [sym_data_format_email] = ACTIONS(2065),
    [sym_data_format_iso8601] = ACTIONS(2065),
    [sym_auditlog_priv] = ACTIONS(2065),
    [sym_configuration_priv] = ACTIONS(2065),
    [sym_deviceconfig_priv] = ACTIONS(2065),
    [sym_ftp_priv] = ACTIONS(2065),
    [sym_http_priv] = ACTIONS(2065),
    [sym_networkconfig_priv] = ACTIONS(2065),
    [sym_programport_priv] = ACTIONS(2065),
    [sym_remoteui_priv] = ACTIONS(2065),
    [sym_securitycontrol_priv] = ACTIONS(2065),
    [sym_softwaremanagement_priv] = ACTIONS(2065),
    [sym_terminal_priv] = ACTIONS(2065),
    [sym_tpadmin_priv] = ACTIONS(2065),
    [sym_user1_priv] = ACTIONS(2065),
    [sym_user2_priv] = ACTIONS(2065),
    [sym_user3_priv] = ACTIONS(2065),
    [sym_user4_priv] = ACTIONS(2065),
    [sym_usermanagement_priv] = ACTIONS(2065),
    [sym_valid_account] = ACTIONS(2065),
    [sym_err_name_invalid] = ACTIONS(2065),
    [sym_err_password_invalid] = ACTIONS(2065),
    [sym_err_invalid_account] = ACTIONS(2065),
    [sym_err_no_authorization] = ACTIONS(2065),
    [sym_err_invalid_parameter] = ACTIONS(2065),
    [sym_err_user_locked_out] = ACTIONS(2065),
    [sym_err_user_account_expired] = ACTIONS(2065),
    [sym_err_general_error] = ACTIONS(2065),
    [sym_err_authentication_error] = ACTIONS(2065),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2065),
    [sym_tls_validate_certificate] = ACTIONS(2065),
    [sym_tdata] = ACTIONS(2065),
    [sym_tchannel] = ACTIONS(2065),
    [sym_tlevel] = ACTIONS(2065),
    [sym_tbutton] = ACTIONS(2065),
    [sym_ttimeline] = ACTIONS(2065),
    [sym_tcustom] = ACTIONS(2065),
    [sym_url_struct] = ACTIONS(2065),
    [sym_dns_struct] = ACTIONS(2065),
    [sym_ip_address_struct] = ACTIONS(2065),
    [sym_dev_info_struct] = ACTIONS(2065),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2065),
    [sym_data_feed] = ACTIONS(2065),
    [sym_data_field] = ACTIONS(2065),
    [sym_data_record] = ACTIONS(2065),
    [sym_wc_data_feed] = ACTIONS(2065),
    [sym_wc_data_field] = ACTIONS(2065),
    [sym_wc_data_record] = ACTIONS(2065),
    [sym_last_login_info] = ACTIONS(2065),
    [sym____reserved___] = ACTIONS(2065),
    [sym_date] = ACTIONS(2065),
    [sym_day] = ACTIONS(2065),
    [sym_time] = ACTIONS(2065),
    [sym_ldate] = ACTIONS(2065),
    [sym_push_device] = ACTIONS(2065),
    [sym_push_channel] = ACTIONS(2065),
    [sym_push_devchan] = ACTIONS(2065),
    [sym_release_device] = ACTIONS(2065),
    [sym_release_channel] = ACTIONS(2065),
    [sym_release_devchan] = ACTIONS(2065),
    [sym_master_slot] = ACTIONS(2065),
    [sym_get_pulse_time] = ACTIONS(2065),
    [sym_get_timer] = ACTIONS(2065),
    [sym_master_sn] = ACTIONS(2065),
    [sym_system_number] = ACTIONS(2065),
    [sym_button] = ACTIONS(2065),
    [sym_dv_channel] = ACTIONS(2065),
    [sym_data] = ACTIONS(2065),
    [sym_level] = ACTIONS(2065),
    [sym_timeline] = ACTIONS(2065),
    [sym_channel] = ACTIONS(2065),
    [sym_custom] = ACTIONS(2065),
    [sym_atoi] = ACTIONS(2065),
    [sym_atol] = ACTIONS(2065),
    [sym_atof] = ACTIONS(2065),
    [sym_itoa] = ACTIONS(2065),
    [sym_itohex] = ACTIONS(2065),
    [sym_hextoi] = ACTIONS(2065),
    [sym_ftoa] = ACTIONS(2065),
    [sym_find_string] = ACTIONS(2065),
    [sym_left_string] = ACTIONS(2065),
    [sym_right_string] = ACTIONS(2065),
    [sym_mid_string] = ACTIONS(2065),
    [sym_remove_string] = ACTIONS(2065),
    [sym_get_buffer_string] = ACTIONS(2065),
    [sym_get_buffer_char] = ACTIONS(2065),
    [sym_get_multi_buffer_string] = ACTIONS(2065),
    [sym_length_string] = ACTIONS(2065),
    [sym_max_length_string] = ACTIONS(2065),
    [sym_set_length_string] = ACTIONS(2065),
    [sym_compare_string] = ACTIONS(2065),
    [sym_upper_string] = ACTIONS(2065),
    [sym_lower_string] = ACTIONS(2065),
    [sym_get_last] = ACTIONS(2065),
    [sym_date_to_day] = ACTIONS(2065),
    [sym_date_to_month] = ACTIONS(2065),
    [sym_date_to_year] = ACTIONS(2065),
    [sym_time_to_hour] = ACTIONS(2065),
    [sym_time_to_minute] = ACTIONS(2065),
    [sym_time_to_second] = ACTIONS(2065),
    [sym_day_of_week] = ACTIONS(2065),
    [sym_set_pulse_time] = ACTIONS(2065),
    [sym_set_timer] = ACTIONS(2065),
    [sym_file_open] = ACTIONS(2065),
    [sym_file_close] = ACTIONS(2065),
    [sym_file_read] = ACTIONS(2065),
    [sym_file_read_line] = ACTIONS(2065),
    [sym_file_write] = ACTIONS(2065),
    [sym_file_write_line] = ACTIONS(2065),
    [sym_file_copy] = ACTIONS(2065),
    [sym_file_delete] = ACTIONS(2065),
    [sym_file_rename] = ACTIONS(2065),
    [sym_file_dir] = ACTIONS(2065),
    [sym_file_getdir] = ACTIONS(2065),
    [sym_file_setdir] = ACTIONS(2065),
    [sym_file_create_dir] = ACTIONS(2065),
    [sym_file_remove_dir] = ACTIONS(2065),
    [sym_file_seek] = ACTIONS(2065),
    [sym_length_array] = ACTIONS(2065),
    [sym_max_length_array] = ACTIONS(2065),
    [sym_set_length_array] = ACTIONS(2065),
    [sym_ip_client_open] = ACTIONS(2065),
    [sym_ip_client_close] = ACTIONS(2065),
    [sym_ip_server_open] = ACTIONS(2065),
    [sym_ip_server_close] = ACTIONS(2065),
    [sym_ip_bound_client_open] = ACTIONS(2065),
    [sym_ip_mc_server_open] = ACTIONS(2065),
    [sym_ip_set_option] = ACTIONS(2065),
    [sym_ssh_client_open] = ACTIONS(2065),
    [sym_ssh_client_close] = ACTIONS(2065),
    [sym_tls_client_open] = ACTIONS(2065),
    [sym_tls_client_close] = ACTIONS(2065),
    [sym_do_push] = ACTIONS(2065),
    [sym_do_push_timed] = ACTIONS(2065),
    [sym_do_release] = ACTIONS(2065),
    [sym_device_id] = ACTIONS(2065),
    [sym_device_info] = ACTIONS(2065),
    [sym_device_id_string] = ACTIONS(2065),
    [sym_device_standby] = ACTIONS(2065),
    [sym_device_wake] = ACTIONS(2065),
    [sym_timeline_create] = ACTIONS(2065),
    [sym_timeline_kill] = ACTIONS(2065),
    [sym_timeline_reload] = ACTIONS(2065),
    [sym_timeline_pause] = ACTIONS(2065),
    [sym_timeline_restart] = ACTIONS(2065),
    [sym_timeline_set] = ACTIONS(2065),
    [sym_timeline_get] = ACTIONS(2065),
    [sym_timeline_active] = ACTIONS(2065),
    [sym_timeline_dynamic_id] = ACTIONS(2065),
    [sym_variable_to_string] = ACTIONS(2065),
    [sym_string_to_variable] = ACTIONS(2065),
    [sym_length_variable_to_string] = ACTIONS(2065),
    [sym_type_cast] = ACTIONS(2065),
    [sym_variable_to_xml] = ACTIONS(2065),
    [sym_xml_to_variable] = ACTIONS(2065),
    [sym_length_variable_to_xml] = ACTIONS(2065),
    [sym_redirect_string] = ACTIONS(2065),
    [sym_random_number] = ACTIONS(2065),
    [sym_get_unique_id] = ACTIONS(2065),
    [sym_get_system_number] = ACTIONS(2065),
    [sym_set_system_number] = ACTIONS(2065),
    [sym_get_serial_number] = ACTIONS(2065),
    [sym_get_master_build] = ACTIONS(2065),
    [sym_set_validation_code] = ACTIONS(2065),
    [sym_reboot] = ACTIONS(2065),
    [sym_load_duet_module] = ACTIONS(2065),
    [sym_unload_duet_module] = ACTIONS(2065),
    [sym_duet_mem_size_set] = ACTIONS(2065),
    [sym_duet_mem_size_get] = ACTIONS(2065),
    [sym_dynamic_application_device] = ACTIONS(2065),
    [sym_dynamic_polled_port] = ACTIONS(2065),
    [sym_static_port_binding] = ACTIONS(2065),
    [sym_static_ip_binding] = ACTIONS(2065),
    [sym_get_dns_list] = ACTIONS(2065),
    [sym_set_dns_list] = ACTIONS(2065),
    [sym_get_ip_address] = ACTIONS(2065),
    [sym_set_ip_address] = ACTIONS(2065),
    [sym_get_url_list] = ACTIONS(2065),
    [sym_add_url_entry] = ACTIONS(2065),
    [sym_delete_url_entry] = ACTIONS(2065),
    [sym_abs_value] = ACTIONS(2065),
    [sym_max_value] = ACTIONS(2065),
    [sym_min_value] = ACTIONS(2065),
    [sym_exp_value] = ACTIONS(2065),
    [sym_log_value] = ACTIONS(2065),
    [sym_log10_value] = ACTIONS(2065),
    [sym_power_value] = ACTIONS(2065),
    [sym_sqrt_value] = ACTIONS(2065),
    [sym_raw_be] = ACTIONS(2065),
    [sym_raw_le] = ACTIONS(2065),
    [sym_internal_queue_size_set] = ACTIONS(2065),
    [sym_internal_queue_size_get] = ACTIONS(2065),
    [sym_internal_threshold_set] = ACTIONS(2065),
    [sym_internal_threshold_get] = ACTIONS(2065),
    [sym_rebuilt_event] = ACTIONS(2065),
    [sym_do_custom_event] = ACTIONS(2065),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2065),
    [sym_clkmgr_set_clk_source] = ACTIONS(2065),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2065),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2065),
    [sym_clkmgr_get_timezone] = ACTIONS(2065),
    [sym_clkmgr_set_timezone] = ACTIONS(2065),
    [sym_clkmgr_get_resync_period] = ACTIONS(2065),
    [sym_clkmgr_set_resync_period] = ACTIONS(2065),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2065),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2065),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2065),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2065),
    [sym_clkmgr_get_timeservers] = ACTIONS(2065),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2065),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2065),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2065),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2065),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2065),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2065),
    [sym_set_log_level] = ACTIONS(2065),
    [sym_get_log_level] = ACTIONS(2065),
    [sym_amx_log] = ACTIONS(2065),
    [sym_smtp_server_config_set] = ACTIONS(2065),
    [sym_smtp_server_config_get] = ACTIONS(2065),
    [sym_smtp_send] = ACTIONS(2065),
    [sym_validate_netlinx_account] = ACTIONS(2065),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2065),
    [sym_authenticate_certificate] = ACTIONS(2065),
    [sym_audit_netlinx_session_event] = ACTIONS(2065),
    [sym_audit_netlinx_generic_event] = ACTIONS(2065),
    [sym_get_available_flash_disk_space] = ACTIONS(2065),
    [sym_get_max_flash_disk_space] = ACTIONS(2065),
    [sym_data_create_feed] = ACTIONS(2065),
    [sym_data_delete_feed] = ACTIONS(2065),
    [sym_data_publish_feed] = ACTIONS(2065),
    [sym_data_get_published_feed] = ACTIONS(2065),
    [sym_data_add_record] = ACTIONS(2065),
    [sym_data_get_event_record] = ACTIONS(2065),
    [sym__wc_data_create_feed] = ACTIONS(2065),
    [sym__wc_data_add_record] = ACTIONS(2065),
    [sym__wc_data_get_event_record] = ACTIONS(2065),
    [sym_format] = ACTIONS(2065),
    [sym_astro_clock] = ACTIONS(2065),
    [sym_set_outdoor_temperature] = ACTIONS(2065),
    [sym_set_virtual_level_count] = ACTIONS(2065),
    [sym_set_virtual_channel_count] = ACTIONS(2065),
    [sym_set_virtual_port_count] = ACTIONS(2065),
    [sym__wc_encode_int] = ACTIONS(2065),
    [sym__wc_decode_int] = ACTIONS(2065),
    [sym__wc_int] = ACTIONS(2065),
    [sym__wc_to_ch_int] = ACTIONS(2065),
    [sym__ch_to_wc_int] = ACTIONS(2065),
    [sym__wc_find_string_int] = ACTIONS(2065),
    [sym__wc_left_string_int] = ACTIONS(2065),
    [sym__wc_lower_string_int] = ACTIONS(2065),
    [sym__wc_mid_string_int] = ACTIONS(2065),
    [sym__wc_remove_string_int] = ACTIONS(2065),
    [sym__wc_right_string_int] = ACTIONS(2065),
    [sym__wc_upper_string_int] = ACTIONS(2065),
    [sym__wc_compare_string_int] = ACTIONS(2065),
    [sym__wc_get_buffer_char_int] = ACTIONS(2065),
    [sym__wc_get_buffer_string_int] = ACTIONS(2065),
    [sym__wc_concat_string_int] = ACTIONS(2065),
    [sym__wc_file_open_int] = ACTIONS(2065),
    [sym__wc_file_close_int] = ACTIONS(2065),
    [sym__wc_file_read_int] = ACTIONS(2065),
    [sym__wc_file_read_line_int] = ACTIONS(2065),
    [sym__wc_file_write_int] = ACTIONS(2065),
    [sym__wc_file_write_line_int] = ACTIONS(2065),
    [aux_sym_preproc_include_token1] = ACTIONS(2065),
    [aux_sym_preproc_define_token1] = ACTIONS(2067),
    [aux_sym_preproc_warn_token1] = ACTIONS(2067),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2067),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2067),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2067),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2067),
    [aux_sym_preproc_else_token1] = ACTIONS(2067),
    [anon_sym_LBRACK] = ACTIONS(2067),
    [anon_sym_LBRACE] = ACTIONS(2067),
    [anon_sym_LPAREN] = ACTIONS(2065),
    [anon_sym_BANG] = ACTIONS(2067),
    [anon_sym_TILDE] = ACTIONS(2067),
    [anon_sym_DASH] = ACTIONS(2065),
    [anon_sym_PLUS] = ACTIONS(2065),
    [anon_sym_DASH_DASH] = ACTIONS(2067),
    [anon_sym_PLUS_PLUS] = ACTIONS(2067),
    [anon_sym_DQUOTE] = ACTIONS(2067),
    [anon_sym_SQUOTE] = ACTIONS(2067),
    [sym_number_literal] = ACTIONS(2067),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(352)] = {
    [sym_identifier] = ACTIONS(2069),
    [sym_program_name_keyword] = ACTIONS(2069),
    [sym_module_name_keyword] = ACTIONS(2069),
    [sym_define_device_keyword] = ACTIONS(2069),
    [sym_define_combine_keyword] = ACTIONS(2069),
    [sym_define_constant_keyword] = ACTIONS(2069),
    [sym_define_type_keyword] = ACTIONS(2069),
    [sym_define_variable_keyword] = ACTIONS(2069),
    [sym_define_system_variable_keyword] = ACTIONS(2069),
    [sym_define_connect_level_keyword] = ACTIONS(2069),
    [sym_define_latching_keyword] = ACTIONS(2069),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2069),
    [sym_define_toggling_keyword] = ACTIONS(2069),
    [sym_define_call_keyword] = ACTIONS(2069),
    [sym_define_function_keyword] = ACTIONS(2069),
    [sym_define_library_function_keyword] = ACTIONS(2069),
    [sym_define_start_keyword] = ACTIONS(2069),
    [sym_define_event_keyword] = ACTIONS(2069),
    [sym_define_program_keyword] = ACTIONS(2069),
    [sym_define_module_keyword] = ACTIONS(2069),
    [sym_char_keyword] = ACTIONS(2069),
    [sym_widechar_keyword] = ACTIONS(2069),
    [sym_integer_keyword] = ACTIONS(2069),
    [sym_sinteger_keyword] = ACTIONS(2069),
    [sym_long_keyword] = ACTIONS(2069),
    [sym_slong_keyword] = ACTIONS(2069),
    [sym_float_keyword] = ACTIONS(2069),
    [sym_double_keyword] = ACTIONS(2069),
    [sym_variant_keyword] = ACTIONS(2069),
    [sym_variantarray_keyword] = ACTIONS(2069),
    [sym_local_var_keyword] = ACTIONS(2069),
    [sym_stack_var_keyword] = ACTIONS(2069),
    [sym_constant_keyword] = ACTIONS(2069),
    [sym_volatile_keyword] = ACTIONS(2069),
    [sym_non_volatile_keyword] = ACTIONS(2069),
    [sym_persistent_keyword] = ACTIONS(2069),
    [sym_dev_keyword] = ACTIONS(2069),
    [sym_devlev_keyword] = ACTIONS(2069),
    [sym_devchan_keyword] = ACTIONS(2069),
    [sym_if_keyword] = ACTIONS(2069),
    [sym_switch_keyword] = ACTIONS(2069),
    [sym_case_keyword] = ACTIONS(2069),
    [sym_default_keyword] = ACTIONS(2069),
    [sym_while_keyword] = ACTIONS(2069),
    [sym_for_keyword] = ACTIONS(2069),
    [sym_break_keyword] = ACTIONS(2069),
    [sym_continue_keyword] = ACTIONS(2069),
    [sym_return_keyword] = ACTIONS(2069),
    [sym_select_keyword] = ACTIONS(2069),
    [sym_active_keyword] = ACTIONS(2069),
    [sym_struct_keyword] = ACTIONS(2069),
    [sym_structure_keyword] = ACTIONS(2069),
    [sym_button_event_keyword] = ACTIONS(2069),
    [sym_channel_event_keyword] = ACTIONS(2069),
    [sym_level_event_keyword] = ACTIONS(2069),
    [sym_data_event_keyword] = ACTIONS(2069),
    [sym_timeline_event_keyword] = ACTIONS(2069),
    [sym_custom_event_keyword] = ACTIONS(2069),
    [sym_push_keyword] = ACTIONS(2069),
    [sym_release_keyword] = ACTIONS(2069),
    [aux_sym_on_keyword_token1] = ACTIONS(2069),
    [aux_sym_off_keyword_token1] = ACTIONS(2069),
    [sym_send_level_keyword] = ACTIONS(2069),
    [sym_send_string_keyword] = ACTIONS(2069),
    [sym_send_command_keyword] = ACTIONS(2069),
    [sym_clear_buffer_keyword] = ACTIONS(2069),
    [sym_create_buffer_keyword] = ACTIONS(2069),
    [sym_create_multi_buffer_keyword] = ACTIONS(2069),
    [sym_call_keyword] = ACTIONS(2069),
    [sym_system_call_keyword] = ACTIONS(2069),
    [sym_devchan_to_keyword] = ACTIONS(2069),
    [sym_devchan_min_to_keyword] = ACTIONS(2069),
    [sym_devchan_total_off_keyword] = ACTIONS(2069),
    [sym_devchan_pulse_keyword] = ACTIONS(2069),
    [sym_wait_keyword] = ACTIONS(2069),
    [sym_cancel_wait_keyword] = ACTIONS(2069),
    [sym_wait_until_keyword] = ACTIONS(2069),
    [sym_cancel_wait_until_keyword] = ACTIONS(2069),
    [sym_cancel_all_wait_keyword] = ACTIONS(2069),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2069),
    [sym_not] = ACTIONS(2069),
    [sym_bnot] = ACTIONS(2069),
    [sym___file__] = ACTIONS(2069),
    [sym___date__] = ACTIONS(2069),
    [sym___ldate__] = ACTIONS(2069),
    [sym___time__] = ACTIONS(2069),
    [sym___line__] = ACTIONS(2069),
    [sym___name__] = ACTIONS(2069),
    [sym___netlinx__] = ACTIONS(2069),
    [sym_netlinx_axi_version] = ACTIONS(2069),
    [sym_url_flg_tcp] = ACTIONS(2069),
    [sym_url_flg_accinfopresent] = ACTIONS(2069),
    [sym_url_flg_temp] = ACTIONS(2069),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2069),
    [sym_url_flg_stat_mask] = ACTIONS(2069),
    [sym_url_flg_stat_lookup] = ACTIONS(2069),
    [sym_url_flg_stat_connecting] = ACTIONS(2069),
    [sym_url_flg_stat_waiting] = ACTIONS(2069),
    [sym_url_flg_stat_connected] = ACTIONS(2069),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2069),
    [sym_first_virtual_device] = ACTIONS(2069),
    [sym_first_local_port] = ACTIONS(2069),
    [sym_dynamic_virtual_device] = ACTIONS(2069),
    [sym_timeline_once] = ACTIONS(2069),
    [sym_timeline_repeat] = ACTIONS(2069),
    [sym_timeline_absolute] = ACTIONS(2069),
    [sym_timeline_relative] = ACTIONS(2069),
    [sym_do_push_timed_infinite] = ACTIONS(2069),
    [sym_source_type_no_address] = ACTIONS(2069),
    [sym_source_type_neuron_id] = ACTIONS(2069),
    [sym_source_type_ip_address] = ACTIONS(2069),
    [sym_source_type_axlink] = ACTIONS(2069),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2069),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2069),
    [sym_source_type_ip_socket_address] = ACTIONS(2069),
    [sym_source_type_rs232] = ACTIONS(2069),
    [sym_source_type_internal] = ACTIONS(2069),
    [sym_source_type_mac_address] = ACTIONS(2069),
    [sym_source_type_ipv4_port] = ACTIONS(2069),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2069),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2069),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2069),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2069),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2069),
    [sym_true] = ACTIONS(2069),
    [sym_false] = ACTIONS(2069),
    [sym_file_read_only] = ACTIONS(2069),
    [sym_file_rw_new] = ACTIONS(2069),
    [sym_file_rw_append] = ACTIONS(2069),
    [sym_ip_tcp] = ACTIONS(2069),
    [sym_ip_udp] = ACTIONS(2069),
    [sym_ip_udp_2way] = ACTIONS(2069),
    [sym_xml_encode_types] = ACTIONS(2069),
    [sym_xml_encode_char_as_list] = ACTIONS(2069),
    [sym_xml_encode_le] = ACTIONS(2069),
    [sym_xml_decode_types] = ACTIONS(2069),
    [sym_xml_decode_no_preserve] = ACTIONS(2069),
    [sym_duet_dev_type_display_device] = ACTIONS(2069),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2069),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2069),
    [sym_duet_dev_type_io_device] = ACTIONS(2069),
    [sym_duet_dev_type_relay_device] = ACTIONS(2069),
    [sym_duet_dev_type_ups] = ACTIONS(2069),
    [sym_duet_dev_type_amplifier] = ACTIONS(2069),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2069),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2069),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2069),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2069),
    [sym_duet_dev_type_camera] = ACTIONS(2069),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2069),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2069),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2069),
    [sym_duet_dev_type_dss] = ACTIONS(2069),
    [sym_duet_dev_type_dvr] = ACTIONS(2069),
    [sym_duet_dev_type_disc_device] = ACTIONS(2069),
    [sym_duet_dev_type_document_camera] = ACTIONS(2069),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2069),
    [sym_duet_dev_type_hvac] = ACTIONS(2069),
    [sym_duet_dev_type_keypad] = ACTIONS(2069),
    [sym_duet_dev_type_light] = ACTIONS(2069),
    [sym_duet_dev_type_monitor] = ACTIONS(2069),
    [sym_duet_dev_type_motor] = ACTIONS(2069),
    [sym_duet_dev_type_multi_window] = ACTIONS(2069),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2069),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2069),
    [sym_duet_dev_type_receiver] = ACTIONS(2069),
    [sym_duet_dev_type_security_system] = ACTIONS(2069),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2069),
    [sym_duet_dev_type_settop_box] = ACTIONS(2069),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2069),
    [sym_duet_dev_type_switcher] = ACTIONS(2069),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2069),
    [sym_duet_dev_type_tv] = ACTIONS(2069),
    [sym_duet_dev_type_utility] = ACTIONS(2069),
    [sym_duet_dev_type_vcr] = ACTIONS(2069),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2069),
    [sym_duet_dev_type_video_processor] = ACTIONS(2069),
    [sym_duet_dev_type_video_projector] = ACTIONS(2069),
    [sym_duet_dev_type_video_wall] = ACTIONS(2069),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2069),
    [sym_duet_dev_type_weather] = ACTIONS(2069),
    [sym_duet_dev_not_polled] = ACTIONS(2069),
    [sym_duet_dev_polled] = ACTIONS(2069),
    [sym_clkmgr_mode_network] = ACTIONS(2069),
    [sym_clkmgr_mode_standalone] = ACTIONS(2069),
    [sym_normal_standby] = ACTIONS(2069),
    [sym_normal_wake] = ACTIONS(2069),
    [sym_ip_multicast_ttl_option] = ACTIONS(2069),
    [sym_ip_ttl_subnet] = ACTIONS(2069),
    [sym_ip_ttl_site] = ACTIONS(2069),
    [sym_ip_ttl_region] = ACTIONS(2069),
    [sym_ip_ttl_continent] = ACTIONS(2069),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2069),
    [sym_ip_nodelay_on] = ACTIONS(2069),
    [sym_ip_nodelay_off] = ACTIONS(2069),
    [sym_amx_error] = ACTIONS(2069),
    [sym_amx_warning] = ACTIONS(2069),
    [sym_amx_info] = ACTIONS(2069),
    [sym_amx_debug] = ACTIONS(2069),
    [sym_netlinx_login_success] = ACTIONS(2069),
    [sym_netlinx_login_fail] = ACTIONS(2069),
    [sym_netlinx_logout] = ACTIONS(2069),
    [sym_null_str] = ACTIONS(2069),
    [sym_smtp_address] = ACTIONS(2069),
    [sym_smtp_port_number] = ACTIONS(2069),
    [sym_smtp_username] = ACTIONS(2069),
    [sym_smtp_password] = ACTIONS(2069),
    [sym_smtp_require_tls] = ACTIONS(2069),
    [sym_smtp_from] = ACTIONS(2069),
    [sym_smtp_tls_true] = ACTIONS(2069),
    [sym_smtp_tls_false] = ACTIONS(2069),
    [sym_listview_on_row_select_event] = ACTIONS(2069),
    [sym_data_structure_unknown] = ACTIONS(2069),
    [sym_data_structure_datafeed] = ACTIONS(2069),
    [sym_data_structure_datarecordset] = ACTIONS(2069),
    [sym_data_structure_datarecord] = ACTIONS(2069),
    [sym_data_structure_datafield] = ACTIONS(2069),
    [sym_data_max_content_field_count] = ACTIONS(2069),
    [sym_data_max_metadata_field_count] = ACTIONS(2069),
    [sym_data_max_id_length] = ACTIONS(2069),
    [sym_data_max_name_length] = ACTIONS(2069),
    [sym_data_max_description_length] = ACTIONS(2069),
    [sym_data_max_type_length] = ACTIONS(2069),
    [sym_data_max_format_length] = ACTIONS(2069),
    [sym_data_max_source_length] = ACTIONS(2069),
    [sym_data_max_value_length] = ACTIONS(2069),
    [sym_data_max_label_length] = ACTIONS(2069),
    [sym_data_max_url_length] = ACTIONS(2069),
    [sym_max_last_login_info_length] = ACTIONS(2069),
    [sym_data_type_unknown] = ACTIONS(2069),
    [sym_data_type_string] = ACTIONS(2069),
    [sym_data_type_datetime] = ACTIONS(2069),
    [sym_data_type_date] = ACTIONS(2069),
    [sym_data_type_time] = ACTIONS(2069),
    [sym_data_type_image] = ACTIONS(2069),
    [sym_data_format_url] = ACTIONS(2069),
    [sym_data_format_phone] = ACTIONS(2069),
    [sym_data_format_email] = ACTIONS(2069),
    [sym_data_format_iso8601] = ACTIONS(2069),
    [sym_auditlog_priv] = ACTIONS(2069),
    [sym_configuration_priv] = ACTIONS(2069),
    [sym_deviceconfig_priv] = ACTIONS(2069),
    [sym_ftp_priv] = ACTIONS(2069),
    [sym_http_priv] = ACTIONS(2069),
    [sym_networkconfig_priv] = ACTIONS(2069),
    [sym_programport_priv] = ACTIONS(2069),
    [sym_remoteui_priv] = ACTIONS(2069),
    [sym_securitycontrol_priv] = ACTIONS(2069),
    [sym_softwaremanagement_priv] = ACTIONS(2069),
    [sym_terminal_priv] = ACTIONS(2069),
    [sym_tpadmin_priv] = ACTIONS(2069),
    [sym_user1_priv] = ACTIONS(2069),
    [sym_user2_priv] = ACTIONS(2069),
    [sym_user3_priv] = ACTIONS(2069),
    [sym_user4_priv] = ACTIONS(2069),
    [sym_usermanagement_priv] = ACTIONS(2069),
    [sym_valid_account] = ACTIONS(2069),
    [sym_err_name_invalid] = ACTIONS(2069),
    [sym_err_password_invalid] = ACTIONS(2069),
    [sym_err_invalid_account] = ACTIONS(2069),
    [sym_err_no_authorization] = ACTIONS(2069),
    [sym_err_invalid_parameter] = ACTIONS(2069),
    [sym_err_user_locked_out] = ACTIONS(2069),
    [sym_err_user_account_expired] = ACTIONS(2069),
    [sym_err_general_error] = ACTIONS(2069),
    [sym_err_authentication_error] = ACTIONS(2069),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2069),
    [sym_tls_validate_certificate] = ACTIONS(2069),
    [sym_tdata] = ACTIONS(2069),
    [sym_tchannel] = ACTIONS(2069),
    [sym_tlevel] = ACTIONS(2069),
    [sym_tbutton] = ACTIONS(2069),
    [sym_ttimeline] = ACTIONS(2069),
    [sym_tcustom] = ACTIONS(2069),
    [sym_url_struct] = ACTIONS(2069),
    [sym_dns_struct] = ACTIONS(2069),
    [sym_ip_address_struct] = ACTIONS(2069),
    [sym_dev_info_struct] = ACTIONS(2069),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2069),
    [sym_data_feed] = ACTIONS(2069),
    [sym_data_field] = ACTIONS(2069),
    [sym_data_record] = ACTIONS(2069),
    [sym_wc_data_feed] = ACTIONS(2069),
    [sym_wc_data_field] = ACTIONS(2069),
    [sym_wc_data_record] = ACTIONS(2069),
    [sym_last_login_info] = ACTIONS(2069),
    [sym____reserved___] = ACTIONS(2069),
    [sym_date] = ACTIONS(2069),
    [sym_day] = ACTIONS(2069),
    [sym_time] = ACTIONS(2069),
    [sym_ldate] = ACTIONS(2069),
    [sym_push_device] = ACTIONS(2069),
    [sym_push_channel] = ACTIONS(2069),
    [sym_push_devchan] = ACTIONS(2069),
    [sym_release_device] = ACTIONS(2069),
    [sym_release_channel] = ACTIONS(2069),
    [sym_release_devchan] = ACTIONS(2069),
    [sym_master_slot] = ACTIONS(2069),
    [sym_get_pulse_time] = ACTIONS(2069),
    [sym_get_timer] = ACTIONS(2069),
    [sym_master_sn] = ACTIONS(2069),
    [sym_system_number] = ACTIONS(2069),
    [sym_button] = ACTIONS(2069),
    [sym_dv_channel] = ACTIONS(2069),
    [sym_data] = ACTIONS(2069),
    [sym_level] = ACTIONS(2069),
    [sym_timeline] = ACTIONS(2069),
    [sym_channel] = ACTIONS(2069),
    [sym_custom] = ACTIONS(2069),
    [sym_atoi] = ACTIONS(2069),
    [sym_atol] = ACTIONS(2069),
    [sym_atof] = ACTIONS(2069),
    [sym_itoa] = ACTIONS(2069),
    [sym_itohex] = ACTIONS(2069),
    [sym_hextoi] = ACTIONS(2069),
    [sym_ftoa] = ACTIONS(2069),
    [sym_find_string] = ACTIONS(2069),
    [sym_left_string] = ACTIONS(2069),
    [sym_right_string] = ACTIONS(2069),
    [sym_mid_string] = ACTIONS(2069),
    [sym_remove_string] = ACTIONS(2069),
    [sym_get_buffer_string] = ACTIONS(2069),
    [sym_get_buffer_char] = ACTIONS(2069),
    [sym_get_multi_buffer_string] = ACTIONS(2069),
    [sym_length_string] = ACTIONS(2069),
    [sym_max_length_string] = ACTIONS(2069),
    [sym_set_length_string] = ACTIONS(2069),
    [sym_compare_string] = ACTIONS(2069),
    [sym_upper_string] = ACTIONS(2069),
    [sym_lower_string] = ACTIONS(2069),
    [sym_get_last] = ACTIONS(2069),
    [sym_date_to_day] = ACTIONS(2069),
    [sym_date_to_month] = ACTIONS(2069),
    [sym_date_to_year] = ACTIONS(2069),
    [sym_time_to_hour] = ACTIONS(2069),
    [sym_time_to_minute] = ACTIONS(2069),
    [sym_time_to_second] = ACTIONS(2069),
    [sym_day_of_week] = ACTIONS(2069),
    [sym_set_pulse_time] = ACTIONS(2069),
    [sym_set_timer] = ACTIONS(2069),
    [sym_file_open] = ACTIONS(2069),
    [sym_file_close] = ACTIONS(2069),
    [sym_file_read] = ACTIONS(2069),
    [sym_file_read_line] = ACTIONS(2069),
    [sym_file_write] = ACTIONS(2069),
    [sym_file_write_line] = ACTIONS(2069),
    [sym_file_copy] = ACTIONS(2069),
    [sym_file_delete] = ACTIONS(2069),
    [sym_file_rename] = ACTIONS(2069),
    [sym_file_dir] = ACTIONS(2069),
    [sym_file_getdir] = ACTIONS(2069),
    [sym_file_setdir] = ACTIONS(2069),
    [sym_file_create_dir] = ACTIONS(2069),
    [sym_file_remove_dir] = ACTIONS(2069),
    [sym_file_seek] = ACTIONS(2069),
    [sym_length_array] = ACTIONS(2069),
    [sym_max_length_array] = ACTIONS(2069),
    [sym_set_length_array] = ACTIONS(2069),
    [sym_ip_client_open] = ACTIONS(2069),
    [sym_ip_client_close] = ACTIONS(2069),
    [sym_ip_server_open] = ACTIONS(2069),
    [sym_ip_server_close] = ACTIONS(2069),
    [sym_ip_bound_client_open] = ACTIONS(2069),
    [sym_ip_mc_server_open] = ACTIONS(2069),
    [sym_ip_set_option] = ACTIONS(2069),
    [sym_ssh_client_open] = ACTIONS(2069),
    [sym_ssh_client_close] = ACTIONS(2069),
    [sym_tls_client_open] = ACTIONS(2069),
    [sym_tls_client_close] = ACTIONS(2069),
    [sym_do_push] = ACTIONS(2069),
    [sym_do_push_timed] = ACTIONS(2069),
    [sym_do_release] = ACTIONS(2069),
    [sym_device_id] = ACTIONS(2069),
    [sym_device_info] = ACTIONS(2069),
    [sym_device_id_string] = ACTIONS(2069),
    [sym_device_standby] = ACTIONS(2069),
    [sym_device_wake] = ACTIONS(2069),
    [sym_timeline_create] = ACTIONS(2069),
    [sym_timeline_kill] = ACTIONS(2069),
    [sym_timeline_reload] = ACTIONS(2069),
    [sym_timeline_pause] = ACTIONS(2069),
    [sym_timeline_restart] = ACTIONS(2069),
    [sym_timeline_set] = ACTIONS(2069),
    [sym_timeline_get] = ACTIONS(2069),
    [sym_timeline_active] = ACTIONS(2069),
    [sym_timeline_dynamic_id] = ACTIONS(2069),
    [sym_variable_to_string] = ACTIONS(2069),
    [sym_string_to_variable] = ACTIONS(2069),
    [sym_length_variable_to_string] = ACTIONS(2069),
    [sym_type_cast] = ACTIONS(2069),
    [sym_variable_to_xml] = ACTIONS(2069),
    [sym_xml_to_variable] = ACTIONS(2069),
    [sym_length_variable_to_xml] = ACTIONS(2069),
    [sym_redirect_string] = ACTIONS(2069),
    [sym_random_number] = ACTIONS(2069),
    [sym_get_unique_id] = ACTIONS(2069),
    [sym_get_system_number] = ACTIONS(2069),
    [sym_set_system_number] = ACTIONS(2069),
    [sym_get_serial_number] = ACTIONS(2069),
    [sym_get_master_build] = ACTIONS(2069),
    [sym_set_validation_code] = ACTIONS(2069),
    [sym_reboot] = ACTIONS(2069),
    [sym_load_duet_module] = ACTIONS(2069),
    [sym_unload_duet_module] = ACTIONS(2069),
    [sym_duet_mem_size_set] = ACTIONS(2069),
    [sym_duet_mem_size_get] = ACTIONS(2069),
    [sym_dynamic_application_device] = ACTIONS(2069),
    [sym_dynamic_polled_port] = ACTIONS(2069),
    [sym_static_port_binding] = ACTIONS(2069),
    [sym_static_ip_binding] = ACTIONS(2069),
    [sym_get_dns_list] = ACTIONS(2069),
    [sym_set_dns_list] = ACTIONS(2069),
    [sym_get_ip_address] = ACTIONS(2069),
    [sym_set_ip_address] = ACTIONS(2069),
    [sym_get_url_list] = ACTIONS(2069),
    [sym_add_url_entry] = ACTIONS(2069),
    [sym_delete_url_entry] = ACTIONS(2069),
    [sym_abs_value] = ACTIONS(2069),
    [sym_max_value] = ACTIONS(2069),
    [sym_min_value] = ACTIONS(2069),
    [sym_exp_value] = ACTIONS(2069),
    [sym_log_value] = ACTIONS(2069),
    [sym_log10_value] = ACTIONS(2069),
    [sym_power_value] = ACTIONS(2069),
    [sym_sqrt_value] = ACTIONS(2069),
    [sym_raw_be] = ACTIONS(2069),
    [sym_raw_le] = ACTIONS(2069),
    [sym_internal_queue_size_set] = ACTIONS(2069),
    [sym_internal_queue_size_get] = ACTIONS(2069),
    [sym_internal_threshold_set] = ACTIONS(2069),
    [sym_internal_threshold_get] = ACTIONS(2069),
    [sym_rebuilt_event] = ACTIONS(2069),
    [sym_do_custom_event] = ACTIONS(2069),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2069),
    [sym_clkmgr_set_clk_source] = ACTIONS(2069),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2069),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2069),
    [sym_clkmgr_get_timezone] = ACTIONS(2069),
    [sym_clkmgr_set_timezone] = ACTIONS(2069),
    [sym_clkmgr_get_resync_period] = ACTIONS(2069),
    [sym_clkmgr_set_resync_period] = ACTIONS(2069),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2069),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2069),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2069),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2069),
    [sym_clkmgr_get_timeservers] = ACTIONS(2069),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2069),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2069),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2069),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2069),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2069),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2069),
    [sym_set_log_level] = ACTIONS(2069),
    [sym_get_log_level] = ACTIONS(2069),
    [sym_amx_log] = ACTIONS(2069),
    [sym_smtp_server_config_set] = ACTIONS(2069),
    [sym_smtp_server_config_get] = ACTIONS(2069),
    [sym_smtp_send] = ACTIONS(2069),
    [sym_validate_netlinx_account] = ACTIONS(2069),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2069),
    [sym_authenticate_certificate] = ACTIONS(2069),
    [sym_audit_netlinx_session_event] = ACTIONS(2069),
    [sym_audit_netlinx_generic_event] = ACTIONS(2069),
    [sym_get_available_flash_disk_space] = ACTIONS(2069),
    [sym_get_max_flash_disk_space] = ACTIONS(2069),
    [sym_data_create_feed] = ACTIONS(2069),
    [sym_data_delete_feed] = ACTIONS(2069),
    [sym_data_publish_feed] = ACTIONS(2069),
    [sym_data_get_published_feed] = ACTIONS(2069),
    [sym_data_add_record] = ACTIONS(2069),
    [sym_data_get_event_record] = ACTIONS(2069),
    [sym__wc_data_create_feed] = ACTIONS(2069),
    [sym__wc_data_add_record] = ACTIONS(2069),
    [sym__wc_data_get_event_record] = ACTIONS(2069),
    [sym_format] = ACTIONS(2069),
    [sym_astro_clock] = ACTIONS(2069),
    [sym_set_outdoor_temperature] = ACTIONS(2069),
    [sym_set_virtual_level_count] = ACTIONS(2069),
    [sym_set_virtual_channel_count] = ACTIONS(2069),
    [sym_set_virtual_port_count] = ACTIONS(2069),
    [sym__wc_encode_int] = ACTIONS(2069),
    [sym__wc_decode_int] = ACTIONS(2069),
    [sym__wc_int] = ACTIONS(2069),
    [sym__wc_to_ch_int] = ACTIONS(2069),
    [sym__ch_to_wc_int] = ACTIONS(2069),
    [sym__wc_find_string_int] = ACTIONS(2069),
    [sym__wc_left_string_int] = ACTIONS(2069),
    [sym__wc_lower_string_int] = ACTIONS(2069),
    [sym__wc_mid_string_int] = ACTIONS(2069),
    [sym__wc_remove_string_int] = ACTIONS(2069),
    [sym__wc_right_string_int] = ACTIONS(2069),
    [sym__wc_upper_string_int] = ACTIONS(2069),
    [sym__wc_compare_string_int] = ACTIONS(2069),
    [sym__wc_get_buffer_char_int] = ACTIONS(2069),
    [sym__wc_get_buffer_string_int] = ACTIONS(2069),
    [sym__wc_concat_string_int] = ACTIONS(2069),
    [sym__wc_file_open_int] = ACTIONS(2069),
    [sym__wc_file_close_int] = ACTIONS(2069),
    [sym__wc_file_read_int] = ACTIONS(2069),
    [sym__wc_file_read_line_int] = ACTIONS(2069),
    [sym__wc_file_write_int] = ACTIONS(2069),
    [sym__wc_file_write_line_int] = ACTIONS(2069),
    [aux_sym_preproc_include_token1] = ACTIONS(2069),
    [aux_sym_preproc_define_token1] = ACTIONS(2071),
    [aux_sym_preproc_warn_token1] = ACTIONS(2071),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2071),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2071),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2071),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2071),
    [aux_sym_preproc_else_token1] = ACTIONS(2071),
    [anon_sym_LBRACK] = ACTIONS(2071),
    [anon_sym_LBRACE] = ACTIONS(2071),
    [anon_sym_LPAREN] = ACTIONS(2069),
    [anon_sym_BANG] = ACTIONS(2071),
    [anon_sym_TILDE] = ACTIONS(2071),
    [anon_sym_DASH] = ACTIONS(2069),
    [anon_sym_PLUS] = ACTIONS(2069),
    [anon_sym_DASH_DASH] = ACTIONS(2071),
    [anon_sym_PLUS_PLUS] = ACTIONS(2071),
    [anon_sym_DQUOTE] = ACTIONS(2071),
    [anon_sym_SQUOTE] = ACTIONS(2071),
    [sym_number_literal] = ACTIONS(2071),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(353)] = {
    [ts_builtin_sym_end] = ACTIONS(1753),
    [sym_identifier] = ACTIONS(1751),
    [sym_program_name_keyword] = ACTIONS(1751),
    [sym_module_name_keyword] = ACTIONS(1751),
    [sym_define_device_keyword] = ACTIONS(1751),
    [sym_define_combine_keyword] = ACTIONS(1751),
    [sym_define_constant_keyword] = ACTIONS(1751),
    [sym_define_type_keyword] = ACTIONS(1751),
    [sym_define_variable_keyword] = ACTIONS(1751),
    [sym_define_system_variable_keyword] = ACTIONS(1751),
    [sym_define_connect_level_keyword] = ACTIONS(1751),
    [sym_define_latching_keyword] = ACTIONS(1751),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1751),
    [sym_define_toggling_keyword] = ACTIONS(1751),
    [sym_define_call_keyword] = ACTIONS(1751),
    [sym_define_function_keyword] = ACTIONS(1751),
    [sym_define_library_function_keyword] = ACTIONS(1751),
    [sym_define_start_keyword] = ACTIONS(1751),
    [sym_define_event_keyword] = ACTIONS(1751),
    [sym_define_program_keyword] = ACTIONS(1751),
    [sym_define_module_keyword] = ACTIONS(1751),
    [sym_char_keyword] = ACTIONS(1751),
    [sym_widechar_keyword] = ACTIONS(1751),
    [sym_integer_keyword] = ACTIONS(1751),
    [sym_sinteger_keyword] = ACTIONS(1751),
    [sym_long_keyword] = ACTIONS(1751),
    [sym_slong_keyword] = ACTIONS(1751),
    [sym_float_keyword] = ACTIONS(1751),
    [sym_double_keyword] = ACTIONS(1751),
    [sym_variant_keyword] = ACTIONS(1751),
    [sym_variantarray_keyword] = ACTIONS(1751),
    [sym_local_var_keyword] = ACTIONS(1751),
    [sym_stack_var_keyword] = ACTIONS(1751),
    [sym_constant_keyword] = ACTIONS(1751),
    [sym_volatile_keyword] = ACTIONS(1751),
    [sym_non_volatile_keyword] = ACTIONS(1751),
    [sym_persistent_keyword] = ACTIONS(1751),
    [sym_dev_keyword] = ACTIONS(1751),
    [sym_devlev_keyword] = ACTIONS(1751),
    [sym_devchan_keyword] = ACTIONS(1751),
    [sym_if_keyword] = ACTIONS(1751),
    [sym_else_keyword] = ACTIONS(1751),
    [sym_switch_keyword] = ACTIONS(1751),
    [sym_case_keyword] = ACTIONS(1751),
    [sym_default_keyword] = ACTIONS(1751),
    [sym_while_keyword] = ACTIONS(1751),
    [sym_for_keyword] = ACTIONS(1751),
    [sym_break_keyword] = ACTIONS(1751),
    [sym_continue_keyword] = ACTIONS(1751),
    [sym_return_keyword] = ACTIONS(1751),
    [sym_select_keyword] = ACTIONS(1751),
    [sym_active_keyword] = ACTIONS(1751),
    [sym_struct_keyword] = ACTIONS(1751),
    [sym_structure_keyword] = ACTIONS(1751),
    [sym_button_event_keyword] = ACTIONS(1751),
    [sym_channel_event_keyword] = ACTIONS(1751),
    [sym_level_event_keyword] = ACTIONS(1751),
    [sym_data_event_keyword] = ACTIONS(1751),
    [sym_timeline_event_keyword] = ACTIONS(1751),
    [sym_custom_event_keyword] = ACTIONS(1751),
    [sym_push_keyword] = ACTIONS(1751),
    [sym_release_keyword] = ACTIONS(1751),
    [aux_sym_on_keyword_token1] = ACTIONS(1751),
    [aux_sym_off_keyword_token1] = ACTIONS(1751),
    [sym_send_level_keyword] = ACTIONS(1751),
    [sym_send_string_keyword] = ACTIONS(1751),
    [sym_send_command_keyword] = ACTIONS(1751),
    [sym_clear_buffer_keyword] = ACTIONS(1751),
    [sym_create_buffer_keyword] = ACTIONS(1751),
    [sym_create_multi_buffer_keyword] = ACTIONS(1751),
    [sym_call_keyword] = ACTIONS(1751),
    [sym_system_call_keyword] = ACTIONS(1751),
    [sym_devchan_to_keyword] = ACTIONS(1751),
    [sym_devchan_min_to_keyword] = ACTIONS(1751),
    [sym_devchan_total_off_keyword] = ACTIONS(1751),
    [sym_devchan_pulse_keyword] = ACTIONS(1751),
    [sym_wait_keyword] = ACTIONS(1751),
    [sym_cancel_wait_keyword] = ACTIONS(1751),
    [sym_wait_until_keyword] = ACTIONS(1751),
    [sym_cancel_wait_until_keyword] = ACTIONS(1751),
    [sym_cancel_all_wait_keyword] = ACTIONS(1751),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1751),
    [sym_not] = ACTIONS(1751),
    [sym_bnot] = ACTIONS(1751),
    [sym___file__] = ACTIONS(1751),
    [sym___date__] = ACTIONS(1751),
    [sym___ldate__] = ACTIONS(1751),
    [sym___time__] = ACTIONS(1751),
    [sym___line__] = ACTIONS(1751),
    [sym___name__] = ACTIONS(1751),
    [sym___netlinx__] = ACTIONS(1751),
    [sym_netlinx_axi_version] = ACTIONS(1751),
    [sym_url_flg_tcp] = ACTIONS(1751),
    [sym_url_flg_accinfopresent] = ACTIONS(1751),
    [sym_url_flg_temp] = ACTIONS(1751),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1751),
    [sym_url_flg_stat_mask] = ACTIONS(1751),
    [sym_url_flg_stat_lookup] = ACTIONS(1751),
    [sym_url_flg_stat_connecting] = ACTIONS(1751),
    [sym_url_flg_stat_waiting] = ACTIONS(1751),
    [sym_url_flg_stat_connected] = ACTIONS(1751),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1751),
    [sym_first_virtual_device] = ACTIONS(1751),
    [sym_first_local_port] = ACTIONS(1751),
    [sym_dynamic_virtual_device] = ACTIONS(1751),
    [sym_timeline_once] = ACTIONS(1751),
    [sym_timeline_repeat] = ACTIONS(1751),
    [sym_timeline_absolute] = ACTIONS(1751),
    [sym_timeline_relative] = ACTIONS(1751),
    [sym_do_push_timed_infinite] = ACTIONS(1751),
    [sym_source_type_no_address] = ACTIONS(1751),
    [sym_source_type_neuron_id] = ACTIONS(1751),
    [sym_source_type_ip_address] = ACTIONS(1751),
    [sym_source_type_axlink] = ACTIONS(1751),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1751),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1751),
    [sym_source_type_ip_socket_address] = ACTIONS(1751),
    [sym_source_type_rs232] = ACTIONS(1751),
    [sym_source_type_internal] = ACTIONS(1751),
    [sym_source_type_mac_address] = ACTIONS(1751),
    [sym_source_type_ipv4_port] = ACTIONS(1751),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1751),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1751),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1751),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1751),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1751),
    [sym_true] = ACTIONS(1751),
    [sym_false] = ACTIONS(1751),
    [sym_file_read_only] = ACTIONS(1751),
    [sym_file_rw_new] = ACTIONS(1751),
    [sym_file_rw_append] = ACTIONS(1751),
    [sym_ip_tcp] = ACTIONS(1751),
    [sym_ip_udp] = ACTIONS(1751),
    [sym_ip_udp_2way] = ACTIONS(1751),
    [sym_xml_encode_types] = ACTIONS(1751),
    [sym_xml_encode_char_as_list] = ACTIONS(1751),
    [sym_xml_encode_le] = ACTIONS(1751),
    [sym_xml_decode_types] = ACTIONS(1751),
    [sym_xml_decode_no_preserve] = ACTIONS(1751),
    [sym_duet_dev_type_display_device] = ACTIONS(1751),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1751),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1751),
    [sym_duet_dev_type_io_device] = ACTIONS(1751),
    [sym_duet_dev_type_relay_device] = ACTIONS(1751),
    [sym_duet_dev_type_ups] = ACTIONS(1751),
    [sym_duet_dev_type_amplifier] = ACTIONS(1751),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1751),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1751),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1751),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1751),
    [sym_duet_dev_type_camera] = ACTIONS(1751),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1751),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1751),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1751),
    [sym_duet_dev_type_dss] = ACTIONS(1751),
    [sym_duet_dev_type_dvr] = ACTIONS(1751),
    [sym_duet_dev_type_disc_device] = ACTIONS(1751),
    [sym_duet_dev_type_document_camera] = ACTIONS(1751),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1751),
    [sym_duet_dev_type_hvac] = ACTIONS(1751),
    [sym_duet_dev_type_keypad] = ACTIONS(1751),
    [sym_duet_dev_type_light] = ACTIONS(1751),
    [sym_duet_dev_type_monitor] = ACTIONS(1751),
    [sym_duet_dev_type_motor] = ACTIONS(1751),
    [sym_duet_dev_type_multi_window] = ACTIONS(1751),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1751),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1751),
    [sym_duet_dev_type_receiver] = ACTIONS(1751),
    [sym_duet_dev_type_security_system] = ACTIONS(1751),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1751),
    [sym_duet_dev_type_settop_box] = ACTIONS(1751),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1751),
    [sym_duet_dev_type_switcher] = ACTIONS(1751),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1751),
    [sym_duet_dev_type_tv] = ACTIONS(1751),
    [sym_duet_dev_type_utility] = ACTIONS(1751),
    [sym_duet_dev_type_vcr] = ACTIONS(1751),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1751),
    [sym_duet_dev_type_video_processor] = ACTIONS(1751),
    [sym_duet_dev_type_video_projector] = ACTIONS(1751),
    [sym_duet_dev_type_video_wall] = ACTIONS(1751),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1751),
    [sym_duet_dev_type_weather] = ACTIONS(1751),
    [sym_duet_dev_not_polled] = ACTIONS(1751),
    [sym_duet_dev_polled] = ACTIONS(1751),
    [sym_clkmgr_mode_network] = ACTIONS(1751),
    [sym_clkmgr_mode_standalone] = ACTIONS(1751),
    [sym_normal_standby] = ACTIONS(1751),
    [sym_normal_wake] = ACTIONS(1751),
    [sym_ip_multicast_ttl_option] = ACTIONS(1751),
    [sym_ip_ttl_subnet] = ACTIONS(1751),
    [sym_ip_ttl_site] = ACTIONS(1751),
    [sym_ip_ttl_region] = ACTIONS(1751),
    [sym_ip_ttl_continent] = ACTIONS(1751),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1751),
    [sym_ip_nodelay_on] = ACTIONS(1751),
    [sym_ip_nodelay_off] = ACTIONS(1751),
    [sym_amx_error] = ACTIONS(1751),
    [sym_amx_warning] = ACTIONS(1751),
    [sym_amx_info] = ACTIONS(1751),
    [sym_amx_debug] = ACTIONS(1751),
    [sym_netlinx_login_success] = ACTIONS(1751),
    [sym_netlinx_login_fail] = ACTIONS(1751),
    [sym_netlinx_logout] = ACTIONS(1751),
    [sym_null_str] = ACTIONS(1751),
    [sym_smtp_address] = ACTIONS(1751),
    [sym_smtp_port_number] = ACTIONS(1751),
    [sym_smtp_username] = ACTIONS(1751),
    [sym_smtp_password] = ACTIONS(1751),
    [sym_smtp_require_tls] = ACTIONS(1751),
    [sym_smtp_from] = ACTIONS(1751),
    [sym_smtp_tls_true] = ACTIONS(1751),
    [sym_smtp_tls_false] = ACTIONS(1751),
    [sym_listview_on_row_select_event] = ACTIONS(1751),
    [sym_data_structure_unknown] = ACTIONS(1751),
    [sym_data_structure_datafeed] = ACTIONS(1751),
    [sym_data_structure_datarecordset] = ACTIONS(1751),
    [sym_data_structure_datarecord] = ACTIONS(1751),
    [sym_data_structure_datafield] = ACTIONS(1751),
    [sym_data_max_content_field_count] = ACTIONS(1751),
    [sym_data_max_metadata_field_count] = ACTIONS(1751),
    [sym_data_max_id_length] = ACTIONS(1751),
    [sym_data_max_name_length] = ACTIONS(1751),
    [sym_data_max_description_length] = ACTIONS(1751),
    [sym_data_max_type_length] = ACTIONS(1751),
    [sym_data_max_format_length] = ACTIONS(1751),
    [sym_data_max_source_length] = ACTIONS(1751),
    [sym_data_max_value_length] = ACTIONS(1751),
    [sym_data_max_label_length] = ACTIONS(1751),
    [sym_data_max_url_length] = ACTIONS(1751),
    [sym_max_last_login_info_length] = ACTIONS(1751),
    [sym_data_type_unknown] = ACTIONS(1751),
    [sym_data_type_string] = ACTIONS(1751),
    [sym_data_type_datetime] = ACTIONS(1751),
    [sym_data_type_date] = ACTIONS(1751),
    [sym_data_type_time] = ACTIONS(1751),
    [sym_data_type_image] = ACTIONS(1751),
    [sym_data_format_url] = ACTIONS(1751),
    [sym_data_format_phone] = ACTIONS(1751),
    [sym_data_format_email] = ACTIONS(1751),
    [sym_data_format_iso8601] = ACTIONS(1751),
    [sym_auditlog_priv] = ACTIONS(1751),
    [sym_configuration_priv] = ACTIONS(1751),
    [sym_deviceconfig_priv] = ACTIONS(1751),
    [sym_ftp_priv] = ACTIONS(1751),
    [sym_http_priv] = ACTIONS(1751),
    [sym_networkconfig_priv] = ACTIONS(1751),
    [sym_programport_priv] = ACTIONS(1751),
    [sym_remoteui_priv] = ACTIONS(1751),
    [sym_securitycontrol_priv] = ACTIONS(1751),
    [sym_softwaremanagement_priv] = ACTIONS(1751),
    [sym_terminal_priv] = ACTIONS(1751),
    [sym_tpadmin_priv] = ACTIONS(1751),
    [sym_user1_priv] = ACTIONS(1751),
    [sym_user2_priv] = ACTIONS(1751),
    [sym_user3_priv] = ACTIONS(1751),
    [sym_user4_priv] = ACTIONS(1751),
    [sym_usermanagement_priv] = ACTIONS(1751),
    [sym_valid_account] = ACTIONS(1751),
    [sym_err_name_invalid] = ACTIONS(1751),
    [sym_err_password_invalid] = ACTIONS(1751),
    [sym_err_invalid_account] = ACTIONS(1751),
    [sym_err_no_authorization] = ACTIONS(1751),
    [sym_err_invalid_parameter] = ACTIONS(1751),
    [sym_err_user_locked_out] = ACTIONS(1751),
    [sym_err_user_account_expired] = ACTIONS(1751),
    [sym_err_general_error] = ACTIONS(1751),
    [sym_err_authentication_error] = ACTIONS(1751),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1751),
    [sym_tls_validate_certificate] = ACTIONS(1751),
    [sym_tdata] = ACTIONS(1751),
    [sym_tchannel] = ACTIONS(1751),
    [sym_tlevel] = ACTIONS(1751),
    [sym_tbutton] = ACTIONS(1751),
    [sym_ttimeline] = ACTIONS(1751),
    [sym_tcustom] = ACTIONS(1751),
    [sym_url_struct] = ACTIONS(1751),
    [sym_dns_struct] = ACTIONS(1751),
    [sym_ip_address_struct] = ACTIONS(1751),
    [sym_dev_info_struct] = ACTIONS(1751),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1751),
    [sym_data_feed] = ACTIONS(1751),
    [sym_data_field] = ACTIONS(1751),
    [sym_data_record] = ACTIONS(1751),
    [sym_wc_data_feed] = ACTIONS(1751),
    [sym_wc_data_field] = ACTIONS(1751),
    [sym_wc_data_record] = ACTIONS(1751),
    [sym_last_login_info] = ACTIONS(1751),
    [sym____reserved___] = ACTIONS(1751),
    [sym_date] = ACTIONS(1751),
    [sym_day] = ACTIONS(1751),
    [sym_time] = ACTIONS(1751),
    [sym_ldate] = ACTIONS(1751),
    [sym_push_device] = ACTIONS(1751),
    [sym_push_channel] = ACTIONS(1751),
    [sym_push_devchan] = ACTIONS(1751),
    [sym_release_device] = ACTIONS(1751),
    [sym_release_channel] = ACTIONS(1751),
    [sym_release_devchan] = ACTIONS(1751),
    [sym_master_slot] = ACTIONS(1751),
    [sym_get_pulse_time] = ACTIONS(1751),
    [sym_get_timer] = ACTIONS(1751),
    [sym_master_sn] = ACTIONS(1751),
    [sym_system_number] = ACTIONS(1751),
    [sym_button] = ACTIONS(1751),
    [sym_dv_channel] = ACTIONS(1751),
    [sym_data] = ACTIONS(1751),
    [sym_level] = ACTIONS(1751),
    [sym_timeline] = ACTIONS(1751),
    [sym_channel] = ACTIONS(1751),
    [sym_custom] = ACTIONS(1751),
    [sym_atoi] = ACTIONS(1751),
    [sym_atol] = ACTIONS(1751),
    [sym_atof] = ACTIONS(1751),
    [sym_itoa] = ACTIONS(1751),
    [sym_itohex] = ACTIONS(1751),
    [sym_hextoi] = ACTIONS(1751),
    [sym_ftoa] = ACTIONS(1751),
    [sym_find_string] = ACTIONS(1751),
    [sym_left_string] = ACTIONS(1751),
    [sym_right_string] = ACTIONS(1751),
    [sym_mid_string] = ACTIONS(1751),
    [sym_remove_string] = ACTIONS(1751),
    [sym_get_buffer_string] = ACTIONS(1751),
    [sym_get_buffer_char] = ACTIONS(1751),
    [sym_get_multi_buffer_string] = ACTIONS(1751),
    [sym_length_string] = ACTIONS(1751),
    [sym_max_length_string] = ACTIONS(1751),
    [sym_set_length_string] = ACTIONS(1751),
    [sym_compare_string] = ACTIONS(1751),
    [sym_upper_string] = ACTIONS(1751),
    [sym_lower_string] = ACTIONS(1751),
    [sym_get_last] = ACTIONS(1751),
    [sym_date_to_day] = ACTIONS(1751),
    [sym_date_to_month] = ACTIONS(1751),
    [sym_date_to_year] = ACTIONS(1751),
    [sym_time_to_hour] = ACTIONS(1751),
    [sym_time_to_minute] = ACTIONS(1751),
    [sym_time_to_second] = ACTIONS(1751),
    [sym_day_of_week] = ACTIONS(1751),
    [sym_set_pulse_time] = ACTIONS(1751),
    [sym_set_timer] = ACTIONS(1751),
    [sym_file_open] = ACTIONS(1751),
    [sym_file_close] = ACTIONS(1751),
    [sym_file_read] = ACTIONS(1751),
    [sym_file_read_line] = ACTIONS(1751),
    [sym_file_write] = ACTIONS(1751),
    [sym_file_write_line] = ACTIONS(1751),
    [sym_file_copy] = ACTIONS(1751),
    [sym_file_delete] = ACTIONS(1751),
    [sym_file_rename] = ACTIONS(1751),
    [sym_file_dir] = ACTIONS(1751),
    [sym_file_getdir] = ACTIONS(1751),
    [sym_file_setdir] = ACTIONS(1751),
    [sym_file_create_dir] = ACTIONS(1751),
    [sym_file_remove_dir] = ACTIONS(1751),
    [sym_file_seek] = ACTIONS(1751),
    [sym_length_array] = ACTIONS(1751),
    [sym_max_length_array] = ACTIONS(1751),
    [sym_set_length_array] = ACTIONS(1751),
    [sym_ip_client_open] = ACTIONS(1751),
    [sym_ip_client_close] = ACTIONS(1751),
    [sym_ip_server_open] = ACTIONS(1751),
    [sym_ip_server_close] = ACTIONS(1751),
    [sym_ip_bound_client_open] = ACTIONS(1751),
    [sym_ip_mc_server_open] = ACTIONS(1751),
    [sym_ip_set_option] = ACTIONS(1751),
    [sym_ssh_client_open] = ACTIONS(1751),
    [sym_ssh_client_close] = ACTIONS(1751),
    [sym_tls_client_open] = ACTIONS(1751),
    [sym_tls_client_close] = ACTIONS(1751),
    [sym_do_push] = ACTIONS(1751),
    [sym_do_push_timed] = ACTIONS(1751),
    [sym_do_release] = ACTIONS(1751),
    [sym_device_id] = ACTIONS(1751),
    [sym_device_info] = ACTIONS(1751),
    [sym_device_id_string] = ACTIONS(1751),
    [sym_device_standby] = ACTIONS(1751),
    [sym_device_wake] = ACTIONS(1751),
    [sym_timeline_create] = ACTIONS(1751),
    [sym_timeline_kill] = ACTIONS(1751),
    [sym_timeline_reload] = ACTIONS(1751),
    [sym_timeline_pause] = ACTIONS(1751),
    [sym_timeline_restart] = ACTIONS(1751),
    [sym_timeline_set] = ACTIONS(1751),
    [sym_timeline_get] = ACTIONS(1751),
    [sym_timeline_active] = ACTIONS(1751),
    [sym_timeline_dynamic_id] = ACTIONS(1751),
    [sym_variable_to_string] = ACTIONS(1751),
    [sym_string_to_variable] = ACTIONS(1751),
    [sym_length_variable_to_string] = ACTIONS(1751),
    [sym_type_cast] = ACTIONS(1751),
    [sym_variable_to_xml] = ACTIONS(1751),
    [sym_xml_to_variable] = ACTIONS(1751),
    [sym_length_variable_to_xml] = ACTIONS(1751),
    [sym_redirect_string] = ACTIONS(1751),
    [sym_random_number] = ACTIONS(1751),
    [sym_get_unique_id] = ACTIONS(1751),
    [sym_get_system_number] = ACTIONS(1751),
    [sym_set_system_number] = ACTIONS(1751),
    [sym_get_serial_number] = ACTIONS(1751),
    [sym_get_master_build] = ACTIONS(1751),
    [sym_set_validation_code] = ACTIONS(1751),
    [sym_reboot] = ACTIONS(1751),
    [sym_load_duet_module] = ACTIONS(1751),
    [sym_unload_duet_module] = ACTIONS(1751),
    [sym_duet_mem_size_set] = ACTIONS(1751),
    [sym_duet_mem_size_get] = ACTIONS(1751),
    [sym_dynamic_application_device] = ACTIONS(1751),
    [sym_dynamic_polled_port] = ACTIONS(1751),
    [sym_static_port_binding] = ACTIONS(1751),
    [sym_static_ip_binding] = ACTIONS(1751),
    [sym_get_dns_list] = ACTIONS(1751),
    [sym_set_dns_list] = ACTIONS(1751),
    [sym_get_ip_address] = ACTIONS(1751),
    [sym_set_ip_address] = ACTIONS(1751),
    [sym_get_url_list] = ACTIONS(1751),
    [sym_add_url_entry] = ACTIONS(1751),
    [sym_delete_url_entry] = ACTIONS(1751),
    [sym_abs_value] = ACTIONS(1751),
    [sym_max_value] = ACTIONS(1751),
    [sym_min_value] = ACTIONS(1751),
    [sym_exp_value] = ACTIONS(1751),
    [sym_log_value] = ACTIONS(1751),
    [sym_log10_value] = ACTIONS(1751),
    [sym_power_value] = ACTIONS(1751),
    [sym_sqrt_value] = ACTIONS(1751),
    [sym_raw_be] = ACTIONS(1751),
    [sym_raw_le] = ACTIONS(1751),
    [sym_internal_queue_size_set] = ACTIONS(1751),
    [sym_internal_queue_size_get] = ACTIONS(1751),
    [sym_internal_threshold_set] = ACTIONS(1751),
    [sym_internal_threshold_get] = ACTIONS(1751),
    [sym_rebuilt_event] = ACTIONS(1751),
    [sym_do_custom_event] = ACTIONS(1751),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1751),
    [sym_clkmgr_set_clk_source] = ACTIONS(1751),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1751),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1751),
    [sym_clkmgr_get_timezone] = ACTIONS(1751),
    [sym_clkmgr_set_timezone] = ACTIONS(1751),
    [sym_clkmgr_get_resync_period] = ACTIONS(1751),
    [sym_clkmgr_set_resync_period] = ACTIONS(1751),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1751),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1751),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1751),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1751),
    [sym_clkmgr_get_timeservers] = ACTIONS(1751),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1751),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1751),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1751),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1751),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1751),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1751),
    [sym_set_log_level] = ACTIONS(1751),
    [sym_get_log_level] = ACTIONS(1751),
    [sym_amx_log] = ACTIONS(1751),
    [sym_smtp_server_config_set] = ACTIONS(1751),
    [sym_smtp_server_config_get] = ACTIONS(1751),
    [sym_smtp_send] = ACTIONS(1751),
    [sym_validate_netlinx_account] = ACTIONS(1751),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1751),
    [sym_authenticate_certificate] = ACTIONS(1751),
    [sym_audit_netlinx_session_event] = ACTIONS(1751),
    [sym_audit_netlinx_generic_event] = ACTIONS(1751),
    [sym_get_available_flash_disk_space] = ACTIONS(1751),
    [sym_get_max_flash_disk_space] = ACTIONS(1751),
    [sym_data_create_feed] = ACTIONS(1751),
    [sym_data_delete_feed] = ACTIONS(1751),
    [sym_data_publish_feed] = ACTIONS(1751),
    [sym_data_get_published_feed] = ACTIONS(1751),
    [sym_data_add_record] = ACTIONS(1751),
    [sym_data_get_event_record] = ACTIONS(1751),
    [sym__wc_data_create_feed] = ACTIONS(1751),
    [sym__wc_data_add_record] = ACTIONS(1751),
    [sym__wc_data_get_event_record] = ACTIONS(1751),
    [sym_format] = ACTIONS(1751),
    [sym_astro_clock] = ACTIONS(1751),
    [sym_set_outdoor_temperature] = ACTIONS(1751),
    [sym_set_virtual_level_count] = ACTIONS(1751),
    [sym_set_virtual_channel_count] = ACTIONS(1751),
    [sym_set_virtual_port_count] = ACTIONS(1751),
    [sym__wc_encode_int] = ACTIONS(1751),
    [sym__wc_decode_int] = ACTIONS(1751),
    [sym__wc_int] = ACTIONS(1751),
    [sym__wc_to_ch_int] = ACTIONS(1751),
    [sym__ch_to_wc_int] = ACTIONS(1751),
    [sym__wc_find_string_int] = ACTIONS(1751),
    [sym__wc_left_string_int] = ACTIONS(1751),
    [sym__wc_lower_string_int] = ACTIONS(1751),
    [sym__wc_mid_string_int] = ACTIONS(1751),
    [sym__wc_remove_string_int] = ACTIONS(1751),
    [sym__wc_right_string_int] = ACTIONS(1751),
    [sym__wc_upper_string_int] = ACTIONS(1751),
    [sym__wc_compare_string_int] = ACTIONS(1751),
    [sym__wc_get_buffer_char_int] = ACTIONS(1751),
    [sym__wc_get_buffer_string_int] = ACTIONS(1751),
    [sym__wc_concat_string_int] = ACTIONS(1751),
    [sym__wc_file_open_int] = ACTIONS(1751),
    [sym__wc_file_close_int] = ACTIONS(1751),
    [sym__wc_file_read_int] = ACTIONS(1751),
    [sym__wc_file_read_line_int] = ACTIONS(1751),
    [sym__wc_file_write_int] = ACTIONS(1751),
    [sym__wc_file_write_line_int] = ACTIONS(1751),
    [aux_sym_preproc_include_token1] = ACTIONS(1751),
    [aux_sym_preproc_define_token1] = ACTIONS(1753),
    [aux_sym_preproc_warn_token1] = ACTIONS(1753),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1753),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1753),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1753),
    [anon_sym_LBRACK] = ACTIONS(1753),
    [anon_sym_LBRACE] = ACTIONS(1753),
    [anon_sym_LPAREN] = ACTIONS(1751),
    [anon_sym_BANG] = ACTIONS(1753),
    [anon_sym_TILDE] = ACTIONS(1753),
    [anon_sym_DASH] = ACTIONS(1751),
    [anon_sym_PLUS] = ACTIONS(1751),
    [anon_sym_DASH_DASH] = ACTIONS(1753),
    [anon_sym_PLUS_PLUS] = ACTIONS(1753),
    [anon_sym_DQUOTE] = ACTIONS(1753),
    [anon_sym_SQUOTE] = ACTIONS(1753),
    [sym_number_literal] = ACTIONS(1753),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(354)] = {
    [ts_builtin_sym_end] = ACTIONS(1757),
    [sym_identifier] = ACTIONS(1755),
    [sym_program_name_keyword] = ACTIONS(1755),
    [sym_module_name_keyword] = ACTIONS(1755),
    [sym_define_device_keyword] = ACTIONS(1755),
    [sym_define_combine_keyword] = ACTIONS(1755),
    [sym_define_constant_keyword] = ACTIONS(1755),
    [sym_define_type_keyword] = ACTIONS(1755),
    [sym_define_variable_keyword] = ACTIONS(1755),
    [sym_define_system_variable_keyword] = ACTIONS(1755),
    [sym_define_connect_level_keyword] = ACTIONS(1755),
    [sym_define_latching_keyword] = ACTIONS(1755),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1755),
    [sym_define_toggling_keyword] = ACTIONS(1755),
    [sym_define_call_keyword] = ACTIONS(1755),
    [sym_define_function_keyword] = ACTIONS(1755),
    [sym_define_library_function_keyword] = ACTIONS(1755),
    [sym_define_start_keyword] = ACTIONS(1755),
    [sym_define_event_keyword] = ACTIONS(1755),
    [sym_define_program_keyword] = ACTIONS(1755),
    [sym_define_module_keyword] = ACTIONS(1755),
    [sym_char_keyword] = ACTIONS(1755),
    [sym_widechar_keyword] = ACTIONS(1755),
    [sym_integer_keyword] = ACTIONS(1755),
    [sym_sinteger_keyword] = ACTIONS(1755),
    [sym_long_keyword] = ACTIONS(1755),
    [sym_slong_keyword] = ACTIONS(1755),
    [sym_float_keyword] = ACTIONS(1755),
    [sym_double_keyword] = ACTIONS(1755),
    [sym_variant_keyword] = ACTIONS(1755),
    [sym_variantarray_keyword] = ACTIONS(1755),
    [sym_local_var_keyword] = ACTIONS(1755),
    [sym_stack_var_keyword] = ACTIONS(1755),
    [sym_constant_keyword] = ACTIONS(1755),
    [sym_volatile_keyword] = ACTIONS(1755),
    [sym_non_volatile_keyword] = ACTIONS(1755),
    [sym_persistent_keyword] = ACTIONS(1755),
    [sym_dev_keyword] = ACTIONS(1755),
    [sym_devlev_keyword] = ACTIONS(1755),
    [sym_devchan_keyword] = ACTIONS(1755),
    [sym_if_keyword] = ACTIONS(1755),
    [sym_else_keyword] = ACTIONS(1755),
    [sym_switch_keyword] = ACTIONS(1755),
    [sym_case_keyword] = ACTIONS(1755),
    [sym_default_keyword] = ACTIONS(1755),
    [sym_while_keyword] = ACTIONS(1755),
    [sym_for_keyword] = ACTIONS(1755),
    [sym_break_keyword] = ACTIONS(1755),
    [sym_continue_keyword] = ACTIONS(1755),
    [sym_return_keyword] = ACTIONS(1755),
    [sym_select_keyword] = ACTIONS(1755),
    [sym_active_keyword] = ACTIONS(1755),
    [sym_struct_keyword] = ACTIONS(1755),
    [sym_structure_keyword] = ACTIONS(1755),
    [sym_button_event_keyword] = ACTIONS(1755),
    [sym_channel_event_keyword] = ACTIONS(1755),
    [sym_level_event_keyword] = ACTIONS(1755),
    [sym_data_event_keyword] = ACTIONS(1755),
    [sym_timeline_event_keyword] = ACTIONS(1755),
    [sym_custom_event_keyword] = ACTIONS(1755),
    [sym_push_keyword] = ACTIONS(1755),
    [sym_release_keyword] = ACTIONS(1755),
    [aux_sym_on_keyword_token1] = ACTIONS(1755),
    [aux_sym_off_keyword_token1] = ACTIONS(1755),
    [sym_send_level_keyword] = ACTIONS(1755),
    [sym_send_string_keyword] = ACTIONS(1755),
    [sym_send_command_keyword] = ACTIONS(1755),
    [sym_clear_buffer_keyword] = ACTIONS(1755),
    [sym_create_buffer_keyword] = ACTIONS(1755),
    [sym_create_multi_buffer_keyword] = ACTIONS(1755),
    [sym_call_keyword] = ACTIONS(1755),
    [sym_system_call_keyword] = ACTIONS(1755),
    [sym_devchan_to_keyword] = ACTIONS(1755),
    [sym_devchan_min_to_keyword] = ACTIONS(1755),
    [sym_devchan_total_off_keyword] = ACTIONS(1755),
    [sym_devchan_pulse_keyword] = ACTIONS(1755),
    [sym_wait_keyword] = ACTIONS(1755),
    [sym_cancel_wait_keyword] = ACTIONS(1755),
    [sym_wait_until_keyword] = ACTIONS(1755),
    [sym_cancel_wait_until_keyword] = ACTIONS(1755),
    [sym_cancel_all_wait_keyword] = ACTIONS(1755),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1755),
    [sym_not] = ACTIONS(1755),
    [sym_bnot] = ACTIONS(1755),
    [sym___file__] = ACTIONS(1755),
    [sym___date__] = ACTIONS(1755),
    [sym___ldate__] = ACTIONS(1755),
    [sym___time__] = ACTIONS(1755),
    [sym___line__] = ACTIONS(1755),
    [sym___name__] = ACTIONS(1755),
    [sym___netlinx__] = ACTIONS(1755),
    [sym_netlinx_axi_version] = ACTIONS(1755),
    [sym_url_flg_tcp] = ACTIONS(1755),
    [sym_url_flg_accinfopresent] = ACTIONS(1755),
    [sym_url_flg_temp] = ACTIONS(1755),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1755),
    [sym_url_flg_stat_mask] = ACTIONS(1755),
    [sym_url_flg_stat_lookup] = ACTIONS(1755),
    [sym_url_flg_stat_connecting] = ACTIONS(1755),
    [sym_url_flg_stat_waiting] = ACTIONS(1755),
    [sym_url_flg_stat_connected] = ACTIONS(1755),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1755),
    [sym_first_virtual_device] = ACTIONS(1755),
    [sym_first_local_port] = ACTIONS(1755),
    [sym_dynamic_virtual_device] = ACTIONS(1755),
    [sym_timeline_once] = ACTIONS(1755),
    [sym_timeline_repeat] = ACTIONS(1755),
    [sym_timeline_absolute] = ACTIONS(1755),
    [sym_timeline_relative] = ACTIONS(1755),
    [sym_do_push_timed_infinite] = ACTIONS(1755),
    [sym_source_type_no_address] = ACTIONS(1755),
    [sym_source_type_neuron_id] = ACTIONS(1755),
    [sym_source_type_ip_address] = ACTIONS(1755),
    [sym_source_type_axlink] = ACTIONS(1755),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1755),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1755),
    [sym_source_type_ip_socket_address] = ACTIONS(1755),
    [sym_source_type_rs232] = ACTIONS(1755),
    [sym_source_type_internal] = ACTIONS(1755),
    [sym_source_type_mac_address] = ACTIONS(1755),
    [sym_source_type_ipv4_port] = ACTIONS(1755),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1755),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1755),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1755),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1755),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1755),
    [sym_true] = ACTIONS(1755),
    [sym_false] = ACTIONS(1755),
    [sym_file_read_only] = ACTIONS(1755),
    [sym_file_rw_new] = ACTIONS(1755),
    [sym_file_rw_append] = ACTIONS(1755),
    [sym_ip_tcp] = ACTIONS(1755),
    [sym_ip_udp] = ACTIONS(1755),
    [sym_ip_udp_2way] = ACTIONS(1755),
    [sym_xml_encode_types] = ACTIONS(1755),
    [sym_xml_encode_char_as_list] = ACTIONS(1755),
    [sym_xml_encode_le] = ACTIONS(1755),
    [sym_xml_decode_types] = ACTIONS(1755),
    [sym_xml_decode_no_preserve] = ACTIONS(1755),
    [sym_duet_dev_type_display_device] = ACTIONS(1755),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1755),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1755),
    [sym_duet_dev_type_io_device] = ACTIONS(1755),
    [sym_duet_dev_type_relay_device] = ACTIONS(1755),
    [sym_duet_dev_type_ups] = ACTIONS(1755),
    [sym_duet_dev_type_amplifier] = ACTIONS(1755),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1755),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1755),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1755),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1755),
    [sym_duet_dev_type_camera] = ACTIONS(1755),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1755),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1755),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1755),
    [sym_duet_dev_type_dss] = ACTIONS(1755),
    [sym_duet_dev_type_dvr] = ACTIONS(1755),
    [sym_duet_dev_type_disc_device] = ACTIONS(1755),
    [sym_duet_dev_type_document_camera] = ACTIONS(1755),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1755),
    [sym_duet_dev_type_hvac] = ACTIONS(1755),
    [sym_duet_dev_type_keypad] = ACTIONS(1755),
    [sym_duet_dev_type_light] = ACTIONS(1755),
    [sym_duet_dev_type_monitor] = ACTIONS(1755),
    [sym_duet_dev_type_motor] = ACTIONS(1755),
    [sym_duet_dev_type_multi_window] = ACTIONS(1755),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1755),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1755),
    [sym_duet_dev_type_receiver] = ACTIONS(1755),
    [sym_duet_dev_type_security_system] = ACTIONS(1755),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1755),
    [sym_duet_dev_type_settop_box] = ACTIONS(1755),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1755),
    [sym_duet_dev_type_switcher] = ACTIONS(1755),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1755),
    [sym_duet_dev_type_tv] = ACTIONS(1755),
    [sym_duet_dev_type_utility] = ACTIONS(1755),
    [sym_duet_dev_type_vcr] = ACTIONS(1755),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1755),
    [sym_duet_dev_type_video_processor] = ACTIONS(1755),
    [sym_duet_dev_type_video_projector] = ACTIONS(1755),
    [sym_duet_dev_type_video_wall] = ACTIONS(1755),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1755),
    [sym_duet_dev_type_weather] = ACTIONS(1755),
    [sym_duet_dev_not_polled] = ACTIONS(1755),
    [sym_duet_dev_polled] = ACTIONS(1755),
    [sym_clkmgr_mode_network] = ACTIONS(1755),
    [sym_clkmgr_mode_standalone] = ACTIONS(1755),
    [sym_normal_standby] = ACTIONS(1755),
    [sym_normal_wake] = ACTIONS(1755),
    [sym_ip_multicast_ttl_option] = ACTIONS(1755),
    [sym_ip_ttl_subnet] = ACTIONS(1755),
    [sym_ip_ttl_site] = ACTIONS(1755),
    [sym_ip_ttl_region] = ACTIONS(1755),
    [sym_ip_ttl_continent] = ACTIONS(1755),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1755),
    [sym_ip_nodelay_on] = ACTIONS(1755),
    [sym_ip_nodelay_off] = ACTIONS(1755),
    [sym_amx_error] = ACTIONS(1755),
    [sym_amx_warning] = ACTIONS(1755),
    [sym_amx_info] = ACTIONS(1755),
    [sym_amx_debug] = ACTIONS(1755),
    [sym_netlinx_login_success] = ACTIONS(1755),
    [sym_netlinx_login_fail] = ACTIONS(1755),
    [sym_netlinx_logout] = ACTIONS(1755),
    [sym_null_str] = ACTIONS(1755),
    [sym_smtp_address] = ACTIONS(1755),
    [sym_smtp_port_number] = ACTIONS(1755),
    [sym_smtp_username] = ACTIONS(1755),
    [sym_smtp_password] = ACTIONS(1755),
    [sym_smtp_require_tls] = ACTIONS(1755),
    [sym_smtp_from] = ACTIONS(1755),
    [sym_smtp_tls_true] = ACTIONS(1755),
    [sym_smtp_tls_false] = ACTIONS(1755),
    [sym_listview_on_row_select_event] = ACTIONS(1755),
    [sym_data_structure_unknown] = ACTIONS(1755),
    [sym_data_structure_datafeed] = ACTIONS(1755),
    [sym_data_structure_datarecordset] = ACTIONS(1755),
    [sym_data_structure_datarecord] = ACTIONS(1755),
    [sym_data_structure_datafield] = ACTIONS(1755),
    [sym_data_max_content_field_count] = ACTIONS(1755),
    [sym_data_max_metadata_field_count] = ACTIONS(1755),
    [sym_data_max_id_length] = ACTIONS(1755),
    [sym_data_max_name_length] = ACTIONS(1755),
    [sym_data_max_description_length] = ACTIONS(1755),
    [sym_data_max_type_length] = ACTIONS(1755),
    [sym_data_max_format_length] = ACTIONS(1755),
    [sym_data_max_source_length] = ACTIONS(1755),
    [sym_data_max_value_length] = ACTIONS(1755),
    [sym_data_max_label_length] = ACTIONS(1755),
    [sym_data_max_url_length] = ACTIONS(1755),
    [sym_max_last_login_info_length] = ACTIONS(1755),
    [sym_data_type_unknown] = ACTIONS(1755),
    [sym_data_type_string] = ACTIONS(1755),
    [sym_data_type_datetime] = ACTIONS(1755),
    [sym_data_type_date] = ACTIONS(1755),
    [sym_data_type_time] = ACTIONS(1755),
    [sym_data_type_image] = ACTIONS(1755),
    [sym_data_format_url] = ACTIONS(1755),
    [sym_data_format_phone] = ACTIONS(1755),
    [sym_data_format_email] = ACTIONS(1755),
    [sym_data_format_iso8601] = ACTIONS(1755),
    [sym_auditlog_priv] = ACTIONS(1755),
    [sym_configuration_priv] = ACTIONS(1755),
    [sym_deviceconfig_priv] = ACTIONS(1755),
    [sym_ftp_priv] = ACTIONS(1755),
    [sym_http_priv] = ACTIONS(1755),
    [sym_networkconfig_priv] = ACTIONS(1755),
    [sym_programport_priv] = ACTIONS(1755),
    [sym_remoteui_priv] = ACTIONS(1755),
    [sym_securitycontrol_priv] = ACTIONS(1755),
    [sym_softwaremanagement_priv] = ACTIONS(1755),
    [sym_terminal_priv] = ACTIONS(1755),
    [sym_tpadmin_priv] = ACTIONS(1755),
    [sym_user1_priv] = ACTIONS(1755),
    [sym_user2_priv] = ACTIONS(1755),
    [sym_user3_priv] = ACTIONS(1755),
    [sym_user4_priv] = ACTIONS(1755),
    [sym_usermanagement_priv] = ACTIONS(1755),
    [sym_valid_account] = ACTIONS(1755),
    [sym_err_name_invalid] = ACTIONS(1755),
    [sym_err_password_invalid] = ACTIONS(1755),
    [sym_err_invalid_account] = ACTIONS(1755),
    [sym_err_no_authorization] = ACTIONS(1755),
    [sym_err_invalid_parameter] = ACTIONS(1755),
    [sym_err_user_locked_out] = ACTIONS(1755),
    [sym_err_user_account_expired] = ACTIONS(1755),
    [sym_err_general_error] = ACTIONS(1755),
    [sym_err_authentication_error] = ACTIONS(1755),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1755),
    [sym_tls_validate_certificate] = ACTIONS(1755),
    [sym_tdata] = ACTIONS(1755),
    [sym_tchannel] = ACTIONS(1755),
    [sym_tlevel] = ACTIONS(1755),
    [sym_tbutton] = ACTIONS(1755),
    [sym_ttimeline] = ACTIONS(1755),
    [sym_tcustom] = ACTIONS(1755),
    [sym_url_struct] = ACTIONS(1755),
    [sym_dns_struct] = ACTIONS(1755),
    [sym_ip_address_struct] = ACTIONS(1755),
    [sym_dev_info_struct] = ACTIONS(1755),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1755),
    [sym_data_feed] = ACTIONS(1755),
    [sym_data_field] = ACTIONS(1755),
    [sym_data_record] = ACTIONS(1755),
    [sym_wc_data_feed] = ACTIONS(1755),
    [sym_wc_data_field] = ACTIONS(1755),
    [sym_wc_data_record] = ACTIONS(1755),
    [sym_last_login_info] = ACTIONS(1755),
    [sym____reserved___] = ACTIONS(1755),
    [sym_date] = ACTIONS(1755),
    [sym_day] = ACTIONS(1755),
    [sym_time] = ACTIONS(1755),
    [sym_ldate] = ACTIONS(1755),
    [sym_push_device] = ACTIONS(1755),
    [sym_push_channel] = ACTIONS(1755),
    [sym_push_devchan] = ACTIONS(1755),
    [sym_release_device] = ACTIONS(1755),
    [sym_release_channel] = ACTIONS(1755),
    [sym_release_devchan] = ACTIONS(1755),
    [sym_master_slot] = ACTIONS(1755),
    [sym_get_pulse_time] = ACTIONS(1755),
    [sym_get_timer] = ACTIONS(1755),
    [sym_master_sn] = ACTIONS(1755),
    [sym_system_number] = ACTIONS(1755),
    [sym_button] = ACTIONS(1755),
    [sym_dv_channel] = ACTIONS(1755),
    [sym_data] = ACTIONS(1755),
    [sym_level] = ACTIONS(1755),
    [sym_timeline] = ACTIONS(1755),
    [sym_channel] = ACTIONS(1755),
    [sym_custom] = ACTIONS(1755),
    [sym_atoi] = ACTIONS(1755),
    [sym_atol] = ACTIONS(1755),
    [sym_atof] = ACTIONS(1755),
    [sym_itoa] = ACTIONS(1755),
    [sym_itohex] = ACTIONS(1755),
    [sym_hextoi] = ACTIONS(1755),
    [sym_ftoa] = ACTIONS(1755),
    [sym_find_string] = ACTIONS(1755),
    [sym_left_string] = ACTIONS(1755),
    [sym_right_string] = ACTIONS(1755),
    [sym_mid_string] = ACTIONS(1755),
    [sym_remove_string] = ACTIONS(1755),
    [sym_get_buffer_string] = ACTIONS(1755),
    [sym_get_buffer_char] = ACTIONS(1755),
    [sym_get_multi_buffer_string] = ACTIONS(1755),
    [sym_length_string] = ACTIONS(1755),
    [sym_max_length_string] = ACTIONS(1755),
    [sym_set_length_string] = ACTIONS(1755),
    [sym_compare_string] = ACTIONS(1755),
    [sym_upper_string] = ACTIONS(1755),
    [sym_lower_string] = ACTIONS(1755),
    [sym_get_last] = ACTIONS(1755),
    [sym_date_to_day] = ACTIONS(1755),
    [sym_date_to_month] = ACTIONS(1755),
    [sym_date_to_year] = ACTIONS(1755),
    [sym_time_to_hour] = ACTIONS(1755),
    [sym_time_to_minute] = ACTIONS(1755),
    [sym_time_to_second] = ACTIONS(1755),
    [sym_day_of_week] = ACTIONS(1755),
    [sym_set_pulse_time] = ACTIONS(1755),
    [sym_set_timer] = ACTIONS(1755),
    [sym_file_open] = ACTIONS(1755),
    [sym_file_close] = ACTIONS(1755),
    [sym_file_read] = ACTIONS(1755),
    [sym_file_read_line] = ACTIONS(1755),
    [sym_file_write] = ACTIONS(1755),
    [sym_file_write_line] = ACTIONS(1755),
    [sym_file_copy] = ACTIONS(1755),
    [sym_file_delete] = ACTIONS(1755),
    [sym_file_rename] = ACTIONS(1755),
    [sym_file_dir] = ACTIONS(1755),
    [sym_file_getdir] = ACTIONS(1755),
    [sym_file_setdir] = ACTIONS(1755),
    [sym_file_create_dir] = ACTIONS(1755),
    [sym_file_remove_dir] = ACTIONS(1755),
    [sym_file_seek] = ACTIONS(1755),
    [sym_length_array] = ACTIONS(1755),
    [sym_max_length_array] = ACTIONS(1755),
    [sym_set_length_array] = ACTIONS(1755),
    [sym_ip_client_open] = ACTIONS(1755),
    [sym_ip_client_close] = ACTIONS(1755),
    [sym_ip_server_open] = ACTIONS(1755),
    [sym_ip_server_close] = ACTIONS(1755),
    [sym_ip_bound_client_open] = ACTIONS(1755),
    [sym_ip_mc_server_open] = ACTIONS(1755),
    [sym_ip_set_option] = ACTIONS(1755),
    [sym_ssh_client_open] = ACTIONS(1755),
    [sym_ssh_client_close] = ACTIONS(1755),
    [sym_tls_client_open] = ACTIONS(1755),
    [sym_tls_client_close] = ACTIONS(1755),
    [sym_do_push] = ACTIONS(1755),
    [sym_do_push_timed] = ACTIONS(1755),
    [sym_do_release] = ACTIONS(1755),
    [sym_device_id] = ACTIONS(1755),
    [sym_device_info] = ACTIONS(1755),
    [sym_device_id_string] = ACTIONS(1755),
    [sym_device_standby] = ACTIONS(1755),
    [sym_device_wake] = ACTIONS(1755),
    [sym_timeline_create] = ACTIONS(1755),
    [sym_timeline_kill] = ACTIONS(1755),
    [sym_timeline_reload] = ACTIONS(1755),
    [sym_timeline_pause] = ACTIONS(1755),
    [sym_timeline_restart] = ACTIONS(1755),
    [sym_timeline_set] = ACTIONS(1755),
    [sym_timeline_get] = ACTIONS(1755),
    [sym_timeline_active] = ACTIONS(1755),
    [sym_timeline_dynamic_id] = ACTIONS(1755),
    [sym_variable_to_string] = ACTIONS(1755),
    [sym_string_to_variable] = ACTIONS(1755),
    [sym_length_variable_to_string] = ACTIONS(1755),
    [sym_type_cast] = ACTIONS(1755),
    [sym_variable_to_xml] = ACTIONS(1755),
    [sym_xml_to_variable] = ACTIONS(1755),
    [sym_length_variable_to_xml] = ACTIONS(1755),
    [sym_redirect_string] = ACTIONS(1755),
    [sym_random_number] = ACTIONS(1755),
    [sym_get_unique_id] = ACTIONS(1755),
    [sym_get_system_number] = ACTIONS(1755),
    [sym_set_system_number] = ACTIONS(1755),
    [sym_get_serial_number] = ACTIONS(1755),
    [sym_get_master_build] = ACTIONS(1755),
    [sym_set_validation_code] = ACTIONS(1755),
    [sym_reboot] = ACTIONS(1755),
    [sym_load_duet_module] = ACTIONS(1755),
    [sym_unload_duet_module] = ACTIONS(1755),
    [sym_duet_mem_size_set] = ACTIONS(1755),
    [sym_duet_mem_size_get] = ACTIONS(1755),
    [sym_dynamic_application_device] = ACTIONS(1755),
    [sym_dynamic_polled_port] = ACTIONS(1755),
    [sym_static_port_binding] = ACTIONS(1755),
    [sym_static_ip_binding] = ACTIONS(1755),
    [sym_get_dns_list] = ACTIONS(1755),
    [sym_set_dns_list] = ACTIONS(1755),
    [sym_get_ip_address] = ACTIONS(1755),
    [sym_set_ip_address] = ACTIONS(1755),
    [sym_get_url_list] = ACTIONS(1755),
    [sym_add_url_entry] = ACTIONS(1755),
    [sym_delete_url_entry] = ACTIONS(1755),
    [sym_abs_value] = ACTIONS(1755),
    [sym_max_value] = ACTIONS(1755),
    [sym_min_value] = ACTIONS(1755),
    [sym_exp_value] = ACTIONS(1755),
    [sym_log_value] = ACTIONS(1755),
    [sym_log10_value] = ACTIONS(1755),
    [sym_power_value] = ACTIONS(1755),
    [sym_sqrt_value] = ACTIONS(1755),
    [sym_raw_be] = ACTIONS(1755),
    [sym_raw_le] = ACTIONS(1755),
    [sym_internal_queue_size_set] = ACTIONS(1755),
    [sym_internal_queue_size_get] = ACTIONS(1755),
    [sym_internal_threshold_set] = ACTIONS(1755),
    [sym_internal_threshold_get] = ACTIONS(1755),
    [sym_rebuilt_event] = ACTIONS(1755),
    [sym_do_custom_event] = ACTIONS(1755),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1755),
    [sym_clkmgr_set_clk_source] = ACTIONS(1755),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1755),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1755),
    [sym_clkmgr_get_timezone] = ACTIONS(1755),
    [sym_clkmgr_set_timezone] = ACTIONS(1755),
    [sym_clkmgr_get_resync_period] = ACTIONS(1755),
    [sym_clkmgr_set_resync_period] = ACTIONS(1755),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1755),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1755),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1755),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1755),
    [sym_clkmgr_get_timeservers] = ACTIONS(1755),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1755),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1755),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1755),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1755),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1755),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1755),
    [sym_set_log_level] = ACTIONS(1755),
    [sym_get_log_level] = ACTIONS(1755),
    [sym_amx_log] = ACTIONS(1755),
    [sym_smtp_server_config_set] = ACTIONS(1755),
    [sym_smtp_server_config_get] = ACTIONS(1755),
    [sym_smtp_send] = ACTIONS(1755),
    [sym_validate_netlinx_account] = ACTIONS(1755),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1755),
    [sym_authenticate_certificate] = ACTIONS(1755),
    [sym_audit_netlinx_session_event] = ACTIONS(1755),
    [sym_audit_netlinx_generic_event] = ACTIONS(1755),
    [sym_get_available_flash_disk_space] = ACTIONS(1755),
    [sym_get_max_flash_disk_space] = ACTIONS(1755),
    [sym_data_create_feed] = ACTIONS(1755),
    [sym_data_delete_feed] = ACTIONS(1755),
    [sym_data_publish_feed] = ACTIONS(1755),
    [sym_data_get_published_feed] = ACTIONS(1755),
    [sym_data_add_record] = ACTIONS(1755),
    [sym_data_get_event_record] = ACTIONS(1755),
    [sym__wc_data_create_feed] = ACTIONS(1755),
    [sym__wc_data_add_record] = ACTIONS(1755),
    [sym__wc_data_get_event_record] = ACTIONS(1755),
    [sym_format] = ACTIONS(1755),
    [sym_astro_clock] = ACTIONS(1755),
    [sym_set_outdoor_temperature] = ACTIONS(1755),
    [sym_set_virtual_level_count] = ACTIONS(1755),
    [sym_set_virtual_channel_count] = ACTIONS(1755),
    [sym_set_virtual_port_count] = ACTIONS(1755),
    [sym__wc_encode_int] = ACTIONS(1755),
    [sym__wc_decode_int] = ACTIONS(1755),
    [sym__wc_int] = ACTIONS(1755),
    [sym__wc_to_ch_int] = ACTIONS(1755),
    [sym__ch_to_wc_int] = ACTIONS(1755),
    [sym__wc_find_string_int] = ACTIONS(1755),
    [sym__wc_left_string_int] = ACTIONS(1755),
    [sym__wc_lower_string_int] = ACTIONS(1755),
    [sym__wc_mid_string_int] = ACTIONS(1755),
    [sym__wc_remove_string_int] = ACTIONS(1755),
    [sym__wc_right_string_int] = ACTIONS(1755),
    [sym__wc_upper_string_int] = ACTIONS(1755),
    [sym__wc_compare_string_int] = ACTIONS(1755),
    [sym__wc_get_buffer_char_int] = ACTIONS(1755),
    [sym__wc_get_buffer_string_int] = ACTIONS(1755),
    [sym__wc_concat_string_int] = ACTIONS(1755),
    [sym__wc_file_open_int] = ACTIONS(1755),
    [sym__wc_file_close_int] = ACTIONS(1755),
    [sym__wc_file_read_int] = ACTIONS(1755),
    [sym__wc_file_read_line_int] = ACTIONS(1755),
    [sym__wc_file_write_int] = ACTIONS(1755),
    [sym__wc_file_write_line_int] = ACTIONS(1755),
    [aux_sym_preproc_include_token1] = ACTIONS(1755),
    [aux_sym_preproc_define_token1] = ACTIONS(1757),
    [aux_sym_preproc_warn_token1] = ACTIONS(1757),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1757),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1757),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1757),
    [anon_sym_LBRACK] = ACTIONS(1757),
    [anon_sym_LBRACE] = ACTIONS(1757),
    [anon_sym_LPAREN] = ACTIONS(1755),
    [anon_sym_BANG] = ACTIONS(1757),
    [anon_sym_TILDE] = ACTIONS(1757),
    [anon_sym_DASH] = ACTIONS(1755),
    [anon_sym_PLUS] = ACTIONS(1755),
    [anon_sym_DASH_DASH] = ACTIONS(1757),
    [anon_sym_PLUS_PLUS] = ACTIONS(1757),
    [anon_sym_DQUOTE] = ACTIONS(1757),
    [anon_sym_SQUOTE] = ACTIONS(1757),
    [sym_number_literal] = ACTIONS(1757),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(355)] = {
    [ts_builtin_sym_end] = ACTIONS(1791),
    [sym_identifier] = ACTIONS(1789),
    [sym_program_name_keyword] = ACTIONS(1789),
    [sym_module_name_keyword] = ACTIONS(1789),
    [sym_define_device_keyword] = ACTIONS(1789),
    [sym_define_combine_keyword] = ACTIONS(1789),
    [sym_define_constant_keyword] = ACTIONS(1789),
    [sym_define_type_keyword] = ACTIONS(1789),
    [sym_define_variable_keyword] = ACTIONS(1789),
    [sym_define_system_variable_keyword] = ACTIONS(1789),
    [sym_define_connect_level_keyword] = ACTIONS(1789),
    [sym_define_latching_keyword] = ACTIONS(1789),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1789),
    [sym_define_toggling_keyword] = ACTIONS(1789),
    [sym_define_call_keyword] = ACTIONS(1789),
    [sym_define_function_keyword] = ACTIONS(1789),
    [sym_define_library_function_keyword] = ACTIONS(1789),
    [sym_define_start_keyword] = ACTIONS(1789),
    [sym_define_event_keyword] = ACTIONS(1789),
    [sym_define_program_keyword] = ACTIONS(1789),
    [sym_define_module_keyword] = ACTIONS(1789),
    [sym_char_keyword] = ACTIONS(1789),
    [sym_widechar_keyword] = ACTIONS(1789),
    [sym_integer_keyword] = ACTIONS(1789),
    [sym_sinteger_keyword] = ACTIONS(1789),
    [sym_long_keyword] = ACTIONS(1789),
    [sym_slong_keyword] = ACTIONS(1789),
    [sym_float_keyword] = ACTIONS(1789),
    [sym_double_keyword] = ACTIONS(1789),
    [sym_variant_keyword] = ACTIONS(1789),
    [sym_variantarray_keyword] = ACTIONS(1789),
    [sym_local_var_keyword] = ACTIONS(1789),
    [sym_stack_var_keyword] = ACTIONS(1789),
    [sym_constant_keyword] = ACTIONS(1789),
    [sym_volatile_keyword] = ACTIONS(1789),
    [sym_non_volatile_keyword] = ACTIONS(1789),
    [sym_persistent_keyword] = ACTIONS(1789),
    [sym_dev_keyword] = ACTIONS(1789),
    [sym_devlev_keyword] = ACTIONS(1789),
    [sym_devchan_keyword] = ACTIONS(1789),
    [sym_if_keyword] = ACTIONS(1789),
    [sym_else_keyword] = ACTIONS(1789),
    [sym_switch_keyword] = ACTIONS(1789),
    [sym_case_keyword] = ACTIONS(1789),
    [sym_default_keyword] = ACTIONS(1789),
    [sym_while_keyword] = ACTIONS(1789),
    [sym_for_keyword] = ACTIONS(1789),
    [sym_break_keyword] = ACTIONS(1789),
    [sym_continue_keyword] = ACTIONS(1789),
    [sym_return_keyword] = ACTIONS(1789),
    [sym_select_keyword] = ACTIONS(1789),
    [sym_active_keyword] = ACTIONS(1789),
    [sym_struct_keyword] = ACTIONS(1789),
    [sym_structure_keyword] = ACTIONS(1789),
    [sym_button_event_keyword] = ACTIONS(1789),
    [sym_channel_event_keyword] = ACTIONS(1789),
    [sym_level_event_keyword] = ACTIONS(1789),
    [sym_data_event_keyword] = ACTIONS(1789),
    [sym_timeline_event_keyword] = ACTIONS(1789),
    [sym_custom_event_keyword] = ACTIONS(1789),
    [sym_push_keyword] = ACTIONS(1789),
    [sym_release_keyword] = ACTIONS(1789),
    [aux_sym_on_keyword_token1] = ACTIONS(1789),
    [aux_sym_off_keyword_token1] = ACTIONS(1789),
    [sym_send_level_keyword] = ACTIONS(1789),
    [sym_send_string_keyword] = ACTIONS(1789),
    [sym_send_command_keyword] = ACTIONS(1789),
    [sym_clear_buffer_keyword] = ACTIONS(1789),
    [sym_create_buffer_keyword] = ACTIONS(1789),
    [sym_create_multi_buffer_keyword] = ACTIONS(1789),
    [sym_call_keyword] = ACTIONS(1789),
    [sym_system_call_keyword] = ACTIONS(1789),
    [sym_devchan_to_keyword] = ACTIONS(1789),
    [sym_devchan_min_to_keyword] = ACTIONS(1789),
    [sym_devchan_total_off_keyword] = ACTIONS(1789),
    [sym_devchan_pulse_keyword] = ACTIONS(1789),
    [sym_wait_keyword] = ACTIONS(1789),
    [sym_cancel_wait_keyword] = ACTIONS(1789),
    [sym_wait_until_keyword] = ACTIONS(1789),
    [sym_cancel_wait_until_keyword] = ACTIONS(1789),
    [sym_cancel_all_wait_keyword] = ACTIONS(1789),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1789),
    [sym_not] = ACTIONS(1789),
    [sym_bnot] = ACTIONS(1789),
    [sym___file__] = ACTIONS(1789),
    [sym___date__] = ACTIONS(1789),
    [sym___ldate__] = ACTIONS(1789),
    [sym___time__] = ACTIONS(1789),
    [sym___line__] = ACTIONS(1789),
    [sym___name__] = ACTIONS(1789),
    [sym___netlinx__] = ACTIONS(1789),
    [sym_netlinx_axi_version] = ACTIONS(1789),
    [sym_url_flg_tcp] = ACTIONS(1789),
    [sym_url_flg_accinfopresent] = ACTIONS(1789),
    [sym_url_flg_temp] = ACTIONS(1789),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1789),
    [sym_url_flg_stat_mask] = ACTIONS(1789),
    [sym_url_flg_stat_lookup] = ACTIONS(1789),
    [sym_url_flg_stat_connecting] = ACTIONS(1789),
    [sym_url_flg_stat_waiting] = ACTIONS(1789),
    [sym_url_flg_stat_connected] = ACTIONS(1789),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1789),
    [sym_first_virtual_device] = ACTIONS(1789),
    [sym_first_local_port] = ACTIONS(1789),
    [sym_dynamic_virtual_device] = ACTIONS(1789),
    [sym_timeline_once] = ACTIONS(1789),
    [sym_timeline_repeat] = ACTIONS(1789),
    [sym_timeline_absolute] = ACTIONS(1789),
    [sym_timeline_relative] = ACTIONS(1789),
    [sym_do_push_timed_infinite] = ACTIONS(1789),
    [sym_source_type_no_address] = ACTIONS(1789),
    [sym_source_type_neuron_id] = ACTIONS(1789),
    [sym_source_type_ip_address] = ACTIONS(1789),
    [sym_source_type_axlink] = ACTIONS(1789),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1789),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1789),
    [sym_source_type_ip_socket_address] = ACTIONS(1789),
    [sym_source_type_rs232] = ACTIONS(1789),
    [sym_source_type_internal] = ACTIONS(1789),
    [sym_source_type_mac_address] = ACTIONS(1789),
    [sym_source_type_ipv4_port] = ACTIONS(1789),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1789),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1789),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1789),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1789),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1789),
    [sym_true] = ACTIONS(1789),
    [sym_false] = ACTIONS(1789),
    [sym_file_read_only] = ACTIONS(1789),
    [sym_file_rw_new] = ACTIONS(1789),
    [sym_file_rw_append] = ACTIONS(1789),
    [sym_ip_tcp] = ACTIONS(1789),
    [sym_ip_udp] = ACTIONS(1789),
    [sym_ip_udp_2way] = ACTIONS(1789),
    [sym_xml_encode_types] = ACTIONS(1789),
    [sym_xml_encode_char_as_list] = ACTIONS(1789),
    [sym_xml_encode_le] = ACTIONS(1789),
    [sym_xml_decode_types] = ACTIONS(1789),
    [sym_xml_decode_no_preserve] = ACTIONS(1789),
    [sym_duet_dev_type_display_device] = ACTIONS(1789),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1789),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1789),
    [sym_duet_dev_type_io_device] = ACTIONS(1789),
    [sym_duet_dev_type_relay_device] = ACTIONS(1789),
    [sym_duet_dev_type_ups] = ACTIONS(1789),
    [sym_duet_dev_type_amplifier] = ACTIONS(1789),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1789),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1789),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1789),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1789),
    [sym_duet_dev_type_camera] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1789),
    [sym_duet_dev_type_dss] = ACTIONS(1789),
    [sym_duet_dev_type_dvr] = ACTIONS(1789),
    [sym_duet_dev_type_disc_device] = ACTIONS(1789),
    [sym_duet_dev_type_document_camera] = ACTIONS(1789),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1789),
    [sym_duet_dev_type_hvac] = ACTIONS(1789),
    [sym_duet_dev_type_keypad] = ACTIONS(1789),
    [sym_duet_dev_type_light] = ACTIONS(1789),
    [sym_duet_dev_type_monitor] = ACTIONS(1789),
    [sym_duet_dev_type_motor] = ACTIONS(1789),
    [sym_duet_dev_type_multi_window] = ACTIONS(1789),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1789),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1789),
    [sym_duet_dev_type_receiver] = ACTIONS(1789),
    [sym_duet_dev_type_security_system] = ACTIONS(1789),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1789),
    [sym_duet_dev_type_settop_box] = ACTIONS(1789),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1789),
    [sym_duet_dev_type_switcher] = ACTIONS(1789),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1789),
    [sym_duet_dev_type_tv] = ACTIONS(1789),
    [sym_duet_dev_type_utility] = ACTIONS(1789),
    [sym_duet_dev_type_vcr] = ACTIONS(1789),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1789),
    [sym_duet_dev_type_video_processor] = ACTIONS(1789),
    [sym_duet_dev_type_video_projector] = ACTIONS(1789),
    [sym_duet_dev_type_video_wall] = ACTIONS(1789),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1789),
    [sym_duet_dev_type_weather] = ACTIONS(1789),
    [sym_duet_dev_not_polled] = ACTIONS(1789),
    [sym_duet_dev_polled] = ACTIONS(1789),
    [sym_clkmgr_mode_network] = ACTIONS(1789),
    [sym_clkmgr_mode_standalone] = ACTIONS(1789),
    [sym_normal_standby] = ACTIONS(1789),
    [sym_normal_wake] = ACTIONS(1789),
    [sym_ip_multicast_ttl_option] = ACTIONS(1789),
    [sym_ip_ttl_subnet] = ACTIONS(1789),
    [sym_ip_ttl_site] = ACTIONS(1789),
    [sym_ip_ttl_region] = ACTIONS(1789),
    [sym_ip_ttl_continent] = ACTIONS(1789),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1789),
    [sym_ip_nodelay_on] = ACTIONS(1789),
    [sym_ip_nodelay_off] = ACTIONS(1789),
    [sym_amx_error] = ACTIONS(1789),
    [sym_amx_warning] = ACTIONS(1789),
    [sym_amx_info] = ACTIONS(1789),
    [sym_amx_debug] = ACTIONS(1789),
    [sym_netlinx_login_success] = ACTIONS(1789),
    [sym_netlinx_login_fail] = ACTIONS(1789),
    [sym_netlinx_logout] = ACTIONS(1789),
    [sym_null_str] = ACTIONS(1789),
    [sym_smtp_address] = ACTIONS(1789),
    [sym_smtp_port_number] = ACTIONS(1789),
    [sym_smtp_username] = ACTIONS(1789),
    [sym_smtp_password] = ACTIONS(1789),
    [sym_smtp_require_tls] = ACTIONS(1789),
    [sym_smtp_from] = ACTIONS(1789),
    [sym_smtp_tls_true] = ACTIONS(1789),
    [sym_smtp_tls_false] = ACTIONS(1789),
    [sym_listview_on_row_select_event] = ACTIONS(1789),
    [sym_data_structure_unknown] = ACTIONS(1789),
    [sym_data_structure_datafeed] = ACTIONS(1789),
    [sym_data_structure_datarecordset] = ACTIONS(1789),
    [sym_data_structure_datarecord] = ACTIONS(1789),
    [sym_data_structure_datafield] = ACTIONS(1789),
    [sym_data_max_content_field_count] = ACTIONS(1789),
    [sym_data_max_metadata_field_count] = ACTIONS(1789),
    [sym_data_max_id_length] = ACTIONS(1789),
    [sym_data_max_name_length] = ACTIONS(1789),
    [sym_data_max_description_length] = ACTIONS(1789),
    [sym_data_max_type_length] = ACTIONS(1789),
    [sym_data_max_format_length] = ACTIONS(1789),
    [sym_data_max_source_length] = ACTIONS(1789),
    [sym_data_max_value_length] = ACTIONS(1789),
    [sym_data_max_label_length] = ACTIONS(1789),
    [sym_data_max_url_length] = ACTIONS(1789),
    [sym_max_last_login_info_length] = ACTIONS(1789),
    [sym_data_type_unknown] = ACTIONS(1789),
    [sym_data_type_string] = ACTIONS(1789),
    [sym_data_type_datetime] = ACTIONS(1789),
    [sym_data_type_date] = ACTIONS(1789),
    [sym_data_type_time] = ACTIONS(1789),
    [sym_data_type_image] = ACTIONS(1789),
    [sym_data_format_url] = ACTIONS(1789),
    [sym_data_format_phone] = ACTIONS(1789),
    [sym_data_format_email] = ACTIONS(1789),
    [sym_data_format_iso8601] = ACTIONS(1789),
    [sym_auditlog_priv] = ACTIONS(1789),
    [sym_configuration_priv] = ACTIONS(1789),
    [sym_deviceconfig_priv] = ACTIONS(1789),
    [sym_ftp_priv] = ACTIONS(1789),
    [sym_http_priv] = ACTIONS(1789),
    [sym_networkconfig_priv] = ACTIONS(1789),
    [sym_programport_priv] = ACTIONS(1789),
    [sym_remoteui_priv] = ACTIONS(1789),
    [sym_securitycontrol_priv] = ACTIONS(1789),
    [sym_softwaremanagement_priv] = ACTIONS(1789),
    [sym_terminal_priv] = ACTIONS(1789),
    [sym_tpadmin_priv] = ACTIONS(1789),
    [sym_user1_priv] = ACTIONS(1789),
    [sym_user2_priv] = ACTIONS(1789),
    [sym_user3_priv] = ACTIONS(1789),
    [sym_user4_priv] = ACTIONS(1789),
    [sym_usermanagement_priv] = ACTIONS(1789),
    [sym_valid_account] = ACTIONS(1789),
    [sym_err_name_invalid] = ACTIONS(1789),
    [sym_err_password_invalid] = ACTIONS(1789),
    [sym_err_invalid_account] = ACTIONS(1789),
    [sym_err_no_authorization] = ACTIONS(1789),
    [sym_err_invalid_parameter] = ACTIONS(1789),
    [sym_err_user_locked_out] = ACTIONS(1789),
    [sym_err_user_account_expired] = ACTIONS(1789),
    [sym_err_general_error] = ACTIONS(1789),
    [sym_err_authentication_error] = ACTIONS(1789),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1789),
    [sym_tls_validate_certificate] = ACTIONS(1789),
    [sym_tdata] = ACTIONS(1789),
    [sym_tchannel] = ACTIONS(1789),
    [sym_tlevel] = ACTIONS(1789),
    [sym_tbutton] = ACTIONS(1789),
    [sym_ttimeline] = ACTIONS(1789),
    [sym_tcustom] = ACTIONS(1789),
    [sym_url_struct] = ACTIONS(1789),
    [sym_dns_struct] = ACTIONS(1789),
    [sym_ip_address_struct] = ACTIONS(1789),
    [sym_dev_info_struct] = ACTIONS(1789),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1789),
    [sym_data_feed] = ACTIONS(1789),
    [sym_data_field] = ACTIONS(1789),
    [sym_data_record] = ACTIONS(1789),
    [sym_wc_data_feed] = ACTIONS(1789),
    [sym_wc_data_field] = ACTIONS(1789),
    [sym_wc_data_record] = ACTIONS(1789),
    [sym_last_login_info] = ACTIONS(1789),
    [sym____reserved___] = ACTIONS(1789),
    [sym_date] = ACTIONS(1789),
    [sym_day] = ACTIONS(1789),
    [sym_time] = ACTIONS(1789),
    [sym_ldate] = ACTIONS(1789),
    [sym_push_device] = ACTIONS(1789),
    [sym_push_channel] = ACTIONS(1789),
    [sym_push_devchan] = ACTIONS(1789),
    [sym_release_device] = ACTIONS(1789),
    [sym_release_channel] = ACTIONS(1789),
    [sym_release_devchan] = ACTIONS(1789),
    [sym_master_slot] = ACTIONS(1789),
    [sym_get_pulse_time] = ACTIONS(1789),
    [sym_get_timer] = ACTIONS(1789),
    [sym_master_sn] = ACTIONS(1789),
    [sym_system_number] = ACTIONS(1789),
    [sym_button] = ACTIONS(1789),
    [sym_dv_channel] = ACTIONS(1789),
    [sym_data] = ACTIONS(1789),
    [sym_level] = ACTIONS(1789),
    [sym_timeline] = ACTIONS(1789),
    [sym_channel] = ACTIONS(1789),
    [sym_custom] = ACTIONS(1789),
    [sym_atoi] = ACTIONS(1789),
    [sym_atol] = ACTIONS(1789),
    [sym_atof] = ACTIONS(1789),
    [sym_itoa] = ACTIONS(1789),
    [sym_itohex] = ACTIONS(1789),
    [sym_hextoi] = ACTIONS(1789),
    [sym_ftoa] = ACTIONS(1789),
    [sym_find_string] = ACTIONS(1789),
    [sym_left_string] = ACTIONS(1789),
    [sym_right_string] = ACTIONS(1789),
    [sym_mid_string] = ACTIONS(1789),
    [sym_remove_string] = ACTIONS(1789),
    [sym_get_buffer_string] = ACTIONS(1789),
    [sym_get_buffer_char] = ACTIONS(1789),
    [sym_get_multi_buffer_string] = ACTIONS(1789),
    [sym_length_string] = ACTIONS(1789),
    [sym_max_length_string] = ACTIONS(1789),
    [sym_set_length_string] = ACTIONS(1789),
    [sym_compare_string] = ACTIONS(1789),
    [sym_upper_string] = ACTIONS(1789),
    [sym_lower_string] = ACTIONS(1789),
    [sym_get_last] = ACTIONS(1789),
    [sym_date_to_day] = ACTIONS(1789),
    [sym_date_to_month] = ACTIONS(1789),
    [sym_date_to_year] = ACTIONS(1789),
    [sym_time_to_hour] = ACTIONS(1789),
    [sym_time_to_minute] = ACTIONS(1789),
    [sym_time_to_second] = ACTIONS(1789),
    [sym_day_of_week] = ACTIONS(1789),
    [sym_set_pulse_time] = ACTIONS(1789),
    [sym_set_timer] = ACTIONS(1789),
    [sym_file_open] = ACTIONS(1789),
    [sym_file_close] = ACTIONS(1789),
    [sym_file_read] = ACTIONS(1789),
    [sym_file_read_line] = ACTIONS(1789),
    [sym_file_write] = ACTIONS(1789),
    [sym_file_write_line] = ACTIONS(1789),
    [sym_file_copy] = ACTIONS(1789),
    [sym_file_delete] = ACTIONS(1789),
    [sym_file_rename] = ACTIONS(1789),
    [sym_file_dir] = ACTIONS(1789),
    [sym_file_getdir] = ACTIONS(1789),
    [sym_file_setdir] = ACTIONS(1789),
    [sym_file_create_dir] = ACTIONS(1789),
    [sym_file_remove_dir] = ACTIONS(1789),
    [sym_file_seek] = ACTIONS(1789),
    [sym_length_array] = ACTIONS(1789),
    [sym_max_length_array] = ACTIONS(1789),
    [sym_set_length_array] = ACTIONS(1789),
    [sym_ip_client_open] = ACTIONS(1789),
    [sym_ip_client_close] = ACTIONS(1789),
    [sym_ip_server_open] = ACTIONS(1789),
    [sym_ip_server_close] = ACTIONS(1789),
    [sym_ip_bound_client_open] = ACTIONS(1789),
    [sym_ip_mc_server_open] = ACTIONS(1789),
    [sym_ip_set_option] = ACTIONS(1789),
    [sym_ssh_client_open] = ACTIONS(1789),
    [sym_ssh_client_close] = ACTIONS(1789),
    [sym_tls_client_open] = ACTIONS(1789),
    [sym_tls_client_close] = ACTIONS(1789),
    [sym_do_push] = ACTIONS(1789),
    [sym_do_push_timed] = ACTIONS(1789),
    [sym_do_release] = ACTIONS(1789),
    [sym_device_id] = ACTIONS(1789),
    [sym_device_info] = ACTIONS(1789),
    [sym_device_id_string] = ACTIONS(1789),
    [sym_device_standby] = ACTIONS(1789),
    [sym_device_wake] = ACTIONS(1789),
    [sym_timeline_create] = ACTIONS(1789),
    [sym_timeline_kill] = ACTIONS(1789),
    [sym_timeline_reload] = ACTIONS(1789),
    [sym_timeline_pause] = ACTIONS(1789),
    [sym_timeline_restart] = ACTIONS(1789),
    [sym_timeline_set] = ACTIONS(1789),
    [sym_timeline_get] = ACTIONS(1789),
    [sym_timeline_active] = ACTIONS(1789),
    [sym_timeline_dynamic_id] = ACTIONS(1789),
    [sym_variable_to_string] = ACTIONS(1789),
    [sym_string_to_variable] = ACTIONS(1789),
    [sym_length_variable_to_string] = ACTIONS(1789),
    [sym_type_cast] = ACTIONS(1789),
    [sym_variable_to_xml] = ACTIONS(1789),
    [sym_xml_to_variable] = ACTIONS(1789),
    [sym_length_variable_to_xml] = ACTIONS(1789),
    [sym_redirect_string] = ACTIONS(1789),
    [sym_random_number] = ACTIONS(1789),
    [sym_get_unique_id] = ACTIONS(1789),
    [sym_get_system_number] = ACTIONS(1789),
    [sym_set_system_number] = ACTIONS(1789),
    [sym_get_serial_number] = ACTIONS(1789),
    [sym_get_master_build] = ACTIONS(1789),
    [sym_set_validation_code] = ACTIONS(1789),
    [sym_reboot] = ACTIONS(1789),
    [sym_load_duet_module] = ACTIONS(1789),
    [sym_unload_duet_module] = ACTIONS(1789),
    [sym_duet_mem_size_set] = ACTIONS(1789),
    [sym_duet_mem_size_get] = ACTIONS(1789),
    [sym_dynamic_application_device] = ACTIONS(1789),
    [sym_dynamic_polled_port] = ACTIONS(1789),
    [sym_static_port_binding] = ACTIONS(1789),
    [sym_static_ip_binding] = ACTIONS(1789),
    [sym_get_dns_list] = ACTIONS(1789),
    [sym_set_dns_list] = ACTIONS(1789),
    [sym_get_ip_address] = ACTIONS(1789),
    [sym_set_ip_address] = ACTIONS(1789),
    [sym_get_url_list] = ACTIONS(1789),
    [sym_add_url_entry] = ACTIONS(1789),
    [sym_delete_url_entry] = ACTIONS(1789),
    [sym_abs_value] = ACTIONS(1789),
    [sym_max_value] = ACTIONS(1789),
    [sym_min_value] = ACTIONS(1789),
    [sym_exp_value] = ACTIONS(1789),
    [sym_log_value] = ACTIONS(1789),
    [sym_log10_value] = ACTIONS(1789),
    [sym_power_value] = ACTIONS(1789),
    [sym_sqrt_value] = ACTIONS(1789),
    [sym_raw_be] = ACTIONS(1789),
    [sym_raw_le] = ACTIONS(1789),
    [sym_internal_queue_size_set] = ACTIONS(1789),
    [sym_internal_queue_size_get] = ACTIONS(1789),
    [sym_internal_threshold_set] = ACTIONS(1789),
    [sym_internal_threshold_get] = ACTIONS(1789),
    [sym_rebuilt_event] = ACTIONS(1789),
    [sym_do_custom_event] = ACTIONS(1789),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1789),
    [sym_clkmgr_set_clk_source] = ACTIONS(1789),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1789),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1789),
    [sym_clkmgr_get_timezone] = ACTIONS(1789),
    [sym_clkmgr_set_timezone] = ACTIONS(1789),
    [sym_clkmgr_get_resync_period] = ACTIONS(1789),
    [sym_clkmgr_set_resync_period] = ACTIONS(1789),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1789),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1789),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1789),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1789),
    [sym_clkmgr_get_timeservers] = ACTIONS(1789),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1789),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1789),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1789),
    [sym_set_log_level] = ACTIONS(1789),
    [sym_get_log_level] = ACTIONS(1789),
    [sym_amx_log] = ACTIONS(1789),
    [sym_smtp_server_config_set] = ACTIONS(1789),
    [sym_smtp_server_config_get] = ACTIONS(1789),
    [sym_smtp_send] = ACTIONS(1789),
    [sym_validate_netlinx_account] = ACTIONS(1789),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1789),
    [sym_authenticate_certificate] = ACTIONS(1789),
    [sym_audit_netlinx_session_event] = ACTIONS(1789),
    [sym_audit_netlinx_generic_event] = ACTIONS(1789),
    [sym_get_available_flash_disk_space] = ACTIONS(1789),
    [sym_get_max_flash_disk_space] = ACTIONS(1789),
    [sym_data_create_feed] = ACTIONS(1789),
    [sym_data_delete_feed] = ACTIONS(1789),
    [sym_data_publish_feed] = ACTIONS(1789),
    [sym_data_get_published_feed] = ACTIONS(1789),
    [sym_data_add_record] = ACTIONS(1789),
    [sym_data_get_event_record] = ACTIONS(1789),
    [sym__wc_data_create_feed] = ACTIONS(1789),
    [sym__wc_data_add_record] = ACTIONS(1789),
    [sym__wc_data_get_event_record] = ACTIONS(1789),
    [sym_format] = ACTIONS(1789),
    [sym_astro_clock] = ACTIONS(1789),
    [sym_set_outdoor_temperature] = ACTIONS(1789),
    [sym_set_virtual_level_count] = ACTIONS(1789),
    [sym_set_virtual_channel_count] = ACTIONS(1789),
    [sym_set_virtual_port_count] = ACTIONS(1789),
    [sym__wc_encode_int] = ACTIONS(1789),
    [sym__wc_decode_int] = ACTIONS(1789),
    [sym__wc_int] = ACTIONS(1789),
    [sym__wc_to_ch_int] = ACTIONS(1789),
    [sym__ch_to_wc_int] = ACTIONS(1789),
    [sym__wc_find_string_int] = ACTIONS(1789),
    [sym__wc_left_string_int] = ACTIONS(1789),
    [sym__wc_lower_string_int] = ACTIONS(1789),
    [sym__wc_mid_string_int] = ACTIONS(1789),
    [sym__wc_remove_string_int] = ACTIONS(1789),
    [sym__wc_right_string_int] = ACTIONS(1789),
    [sym__wc_upper_string_int] = ACTIONS(1789),
    [sym__wc_compare_string_int] = ACTIONS(1789),
    [sym__wc_get_buffer_char_int] = ACTIONS(1789),
    [sym__wc_get_buffer_string_int] = ACTIONS(1789),
    [sym__wc_concat_string_int] = ACTIONS(1789),
    [sym__wc_file_open_int] = ACTIONS(1789),
    [sym__wc_file_close_int] = ACTIONS(1789),
    [sym__wc_file_read_int] = ACTIONS(1789),
    [sym__wc_file_read_line_int] = ACTIONS(1789),
    [sym__wc_file_write_int] = ACTIONS(1789),
    [sym__wc_file_write_line_int] = ACTIONS(1789),
    [aux_sym_preproc_include_token1] = ACTIONS(1789),
    [aux_sym_preproc_define_token1] = ACTIONS(1791),
    [aux_sym_preproc_warn_token1] = ACTIONS(1791),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1791),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1791),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1791),
    [anon_sym_LBRACK] = ACTIONS(1791),
    [anon_sym_LBRACE] = ACTIONS(1791),
    [anon_sym_LPAREN] = ACTIONS(1789),
    [anon_sym_BANG] = ACTIONS(1791),
    [anon_sym_TILDE] = ACTIONS(1791),
    [anon_sym_DASH] = ACTIONS(1789),
    [anon_sym_PLUS] = ACTIONS(1789),
    [anon_sym_DASH_DASH] = ACTIONS(1791),
    [anon_sym_PLUS_PLUS] = ACTIONS(1791),
    [anon_sym_DQUOTE] = ACTIONS(1791),
    [anon_sym_SQUOTE] = ACTIONS(1791),
    [sym_number_literal] = ACTIONS(1791),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(356)] = {
    [sym_identifier] = ACTIONS(2073),
    [sym_program_name_keyword] = ACTIONS(2073),
    [sym_module_name_keyword] = ACTIONS(2073),
    [sym_define_device_keyword] = ACTIONS(2073),
    [sym_define_combine_keyword] = ACTIONS(2073),
    [sym_define_constant_keyword] = ACTIONS(2073),
    [sym_define_type_keyword] = ACTIONS(2073),
    [sym_define_variable_keyword] = ACTIONS(2073),
    [sym_define_system_variable_keyword] = ACTIONS(2073),
    [sym_define_connect_level_keyword] = ACTIONS(2073),
    [sym_define_latching_keyword] = ACTIONS(2073),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2073),
    [sym_define_toggling_keyword] = ACTIONS(2073),
    [sym_define_call_keyword] = ACTIONS(2073),
    [sym_define_function_keyword] = ACTIONS(2073),
    [sym_define_library_function_keyword] = ACTIONS(2073),
    [sym_define_start_keyword] = ACTIONS(2073),
    [sym_define_event_keyword] = ACTIONS(2073),
    [sym_define_program_keyword] = ACTIONS(2073),
    [sym_define_module_keyword] = ACTIONS(2073),
    [sym_char_keyword] = ACTIONS(2073),
    [sym_widechar_keyword] = ACTIONS(2073),
    [sym_integer_keyword] = ACTIONS(2073),
    [sym_sinteger_keyword] = ACTIONS(2073),
    [sym_long_keyword] = ACTIONS(2073),
    [sym_slong_keyword] = ACTIONS(2073),
    [sym_float_keyword] = ACTIONS(2073),
    [sym_double_keyword] = ACTIONS(2073),
    [sym_variant_keyword] = ACTIONS(2073),
    [sym_variantarray_keyword] = ACTIONS(2073),
    [sym_local_var_keyword] = ACTIONS(2073),
    [sym_stack_var_keyword] = ACTIONS(2073),
    [sym_constant_keyword] = ACTIONS(2073),
    [sym_volatile_keyword] = ACTIONS(2073),
    [sym_non_volatile_keyword] = ACTIONS(2073),
    [sym_persistent_keyword] = ACTIONS(2073),
    [sym_dev_keyword] = ACTIONS(2073),
    [sym_devlev_keyword] = ACTIONS(2073),
    [sym_devchan_keyword] = ACTIONS(2073),
    [sym_if_keyword] = ACTIONS(2073),
    [sym_switch_keyword] = ACTIONS(2073),
    [sym_case_keyword] = ACTIONS(2073),
    [sym_default_keyword] = ACTIONS(2073),
    [sym_while_keyword] = ACTIONS(2073),
    [sym_for_keyword] = ACTIONS(2073),
    [sym_break_keyword] = ACTIONS(2073),
    [sym_continue_keyword] = ACTIONS(2073),
    [sym_return_keyword] = ACTIONS(2073),
    [sym_select_keyword] = ACTIONS(2073),
    [sym_active_keyword] = ACTIONS(2073),
    [sym_struct_keyword] = ACTIONS(2073),
    [sym_structure_keyword] = ACTIONS(2073),
    [sym_button_event_keyword] = ACTIONS(2073),
    [sym_channel_event_keyword] = ACTIONS(2073),
    [sym_level_event_keyword] = ACTIONS(2073),
    [sym_data_event_keyword] = ACTIONS(2073),
    [sym_timeline_event_keyword] = ACTIONS(2073),
    [sym_custom_event_keyword] = ACTIONS(2073),
    [sym_push_keyword] = ACTIONS(2073),
    [sym_release_keyword] = ACTIONS(2073),
    [aux_sym_on_keyword_token1] = ACTIONS(2073),
    [aux_sym_off_keyword_token1] = ACTIONS(2073),
    [sym_send_level_keyword] = ACTIONS(2073),
    [sym_send_string_keyword] = ACTIONS(2073),
    [sym_send_command_keyword] = ACTIONS(2073),
    [sym_clear_buffer_keyword] = ACTIONS(2073),
    [sym_create_buffer_keyword] = ACTIONS(2073),
    [sym_create_multi_buffer_keyword] = ACTIONS(2073),
    [sym_call_keyword] = ACTIONS(2073),
    [sym_system_call_keyword] = ACTIONS(2073),
    [sym_devchan_to_keyword] = ACTIONS(2073),
    [sym_devchan_min_to_keyword] = ACTIONS(2073),
    [sym_devchan_total_off_keyword] = ACTIONS(2073),
    [sym_devchan_pulse_keyword] = ACTIONS(2073),
    [sym_wait_keyword] = ACTIONS(2073),
    [sym_cancel_wait_keyword] = ACTIONS(2073),
    [sym_wait_until_keyword] = ACTIONS(2073),
    [sym_cancel_wait_until_keyword] = ACTIONS(2073),
    [sym_cancel_all_wait_keyword] = ACTIONS(2073),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2073),
    [sym_not] = ACTIONS(2073),
    [sym_bnot] = ACTIONS(2073),
    [sym___file__] = ACTIONS(2073),
    [sym___date__] = ACTIONS(2073),
    [sym___ldate__] = ACTIONS(2073),
    [sym___time__] = ACTIONS(2073),
    [sym___line__] = ACTIONS(2073),
    [sym___name__] = ACTIONS(2073),
    [sym___netlinx__] = ACTIONS(2073),
    [sym_netlinx_axi_version] = ACTIONS(2073),
    [sym_url_flg_tcp] = ACTIONS(2073),
    [sym_url_flg_accinfopresent] = ACTIONS(2073),
    [sym_url_flg_temp] = ACTIONS(2073),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2073),
    [sym_url_flg_stat_mask] = ACTIONS(2073),
    [sym_url_flg_stat_lookup] = ACTIONS(2073),
    [sym_url_flg_stat_connecting] = ACTIONS(2073),
    [sym_url_flg_stat_waiting] = ACTIONS(2073),
    [sym_url_flg_stat_connected] = ACTIONS(2073),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2073),
    [sym_first_virtual_device] = ACTIONS(2073),
    [sym_first_local_port] = ACTIONS(2073),
    [sym_dynamic_virtual_device] = ACTIONS(2073),
    [sym_timeline_once] = ACTIONS(2073),
    [sym_timeline_repeat] = ACTIONS(2073),
    [sym_timeline_absolute] = ACTIONS(2073),
    [sym_timeline_relative] = ACTIONS(2073),
    [sym_do_push_timed_infinite] = ACTIONS(2073),
    [sym_source_type_no_address] = ACTIONS(2073),
    [sym_source_type_neuron_id] = ACTIONS(2073),
    [sym_source_type_ip_address] = ACTIONS(2073),
    [sym_source_type_axlink] = ACTIONS(2073),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2073),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2073),
    [sym_source_type_ip_socket_address] = ACTIONS(2073),
    [sym_source_type_rs232] = ACTIONS(2073),
    [sym_source_type_internal] = ACTIONS(2073),
    [sym_source_type_mac_address] = ACTIONS(2073),
    [sym_source_type_ipv4_port] = ACTIONS(2073),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2073),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2073),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2073),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2073),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2073),
    [sym_true] = ACTIONS(2073),
    [sym_false] = ACTIONS(2073),
    [sym_file_read_only] = ACTIONS(2073),
    [sym_file_rw_new] = ACTIONS(2073),
    [sym_file_rw_append] = ACTIONS(2073),
    [sym_ip_tcp] = ACTIONS(2073),
    [sym_ip_udp] = ACTIONS(2073),
    [sym_ip_udp_2way] = ACTIONS(2073),
    [sym_xml_encode_types] = ACTIONS(2073),
    [sym_xml_encode_char_as_list] = ACTIONS(2073),
    [sym_xml_encode_le] = ACTIONS(2073),
    [sym_xml_decode_types] = ACTIONS(2073),
    [sym_xml_decode_no_preserve] = ACTIONS(2073),
    [sym_duet_dev_type_display_device] = ACTIONS(2073),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2073),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2073),
    [sym_duet_dev_type_io_device] = ACTIONS(2073),
    [sym_duet_dev_type_relay_device] = ACTIONS(2073),
    [sym_duet_dev_type_ups] = ACTIONS(2073),
    [sym_duet_dev_type_amplifier] = ACTIONS(2073),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2073),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2073),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2073),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2073),
    [sym_duet_dev_type_camera] = ACTIONS(2073),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2073),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2073),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2073),
    [sym_duet_dev_type_dss] = ACTIONS(2073),
    [sym_duet_dev_type_dvr] = ACTIONS(2073),
    [sym_duet_dev_type_disc_device] = ACTIONS(2073),
    [sym_duet_dev_type_document_camera] = ACTIONS(2073),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2073),
    [sym_duet_dev_type_hvac] = ACTIONS(2073),
    [sym_duet_dev_type_keypad] = ACTIONS(2073),
    [sym_duet_dev_type_light] = ACTIONS(2073),
    [sym_duet_dev_type_monitor] = ACTIONS(2073),
    [sym_duet_dev_type_motor] = ACTIONS(2073),
    [sym_duet_dev_type_multi_window] = ACTIONS(2073),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2073),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2073),
    [sym_duet_dev_type_receiver] = ACTIONS(2073),
    [sym_duet_dev_type_security_system] = ACTIONS(2073),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2073),
    [sym_duet_dev_type_settop_box] = ACTIONS(2073),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2073),
    [sym_duet_dev_type_switcher] = ACTIONS(2073),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2073),
    [sym_duet_dev_type_tv] = ACTIONS(2073),
    [sym_duet_dev_type_utility] = ACTIONS(2073),
    [sym_duet_dev_type_vcr] = ACTIONS(2073),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2073),
    [sym_duet_dev_type_video_processor] = ACTIONS(2073),
    [sym_duet_dev_type_video_projector] = ACTIONS(2073),
    [sym_duet_dev_type_video_wall] = ACTIONS(2073),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2073),
    [sym_duet_dev_type_weather] = ACTIONS(2073),
    [sym_duet_dev_not_polled] = ACTIONS(2073),
    [sym_duet_dev_polled] = ACTIONS(2073),
    [sym_clkmgr_mode_network] = ACTIONS(2073),
    [sym_clkmgr_mode_standalone] = ACTIONS(2073),
    [sym_normal_standby] = ACTIONS(2073),
    [sym_normal_wake] = ACTIONS(2073),
    [sym_ip_multicast_ttl_option] = ACTIONS(2073),
    [sym_ip_ttl_subnet] = ACTIONS(2073),
    [sym_ip_ttl_site] = ACTIONS(2073),
    [sym_ip_ttl_region] = ACTIONS(2073),
    [sym_ip_ttl_continent] = ACTIONS(2073),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2073),
    [sym_ip_nodelay_on] = ACTIONS(2073),
    [sym_ip_nodelay_off] = ACTIONS(2073),
    [sym_amx_error] = ACTIONS(2073),
    [sym_amx_warning] = ACTIONS(2073),
    [sym_amx_info] = ACTIONS(2073),
    [sym_amx_debug] = ACTIONS(2073),
    [sym_netlinx_login_success] = ACTIONS(2073),
    [sym_netlinx_login_fail] = ACTIONS(2073),
    [sym_netlinx_logout] = ACTIONS(2073),
    [sym_null_str] = ACTIONS(2073),
    [sym_smtp_address] = ACTIONS(2073),
    [sym_smtp_port_number] = ACTIONS(2073),
    [sym_smtp_username] = ACTIONS(2073),
    [sym_smtp_password] = ACTIONS(2073),
    [sym_smtp_require_tls] = ACTIONS(2073),
    [sym_smtp_from] = ACTIONS(2073),
    [sym_smtp_tls_true] = ACTIONS(2073),
    [sym_smtp_tls_false] = ACTIONS(2073),
    [sym_listview_on_row_select_event] = ACTIONS(2073),
    [sym_data_structure_unknown] = ACTIONS(2073),
    [sym_data_structure_datafeed] = ACTIONS(2073),
    [sym_data_structure_datarecordset] = ACTIONS(2073),
    [sym_data_structure_datarecord] = ACTIONS(2073),
    [sym_data_structure_datafield] = ACTIONS(2073),
    [sym_data_max_content_field_count] = ACTIONS(2073),
    [sym_data_max_metadata_field_count] = ACTIONS(2073),
    [sym_data_max_id_length] = ACTIONS(2073),
    [sym_data_max_name_length] = ACTIONS(2073),
    [sym_data_max_description_length] = ACTIONS(2073),
    [sym_data_max_type_length] = ACTIONS(2073),
    [sym_data_max_format_length] = ACTIONS(2073),
    [sym_data_max_source_length] = ACTIONS(2073),
    [sym_data_max_value_length] = ACTIONS(2073),
    [sym_data_max_label_length] = ACTIONS(2073),
    [sym_data_max_url_length] = ACTIONS(2073),
    [sym_max_last_login_info_length] = ACTIONS(2073),
    [sym_data_type_unknown] = ACTIONS(2073),
    [sym_data_type_string] = ACTIONS(2073),
    [sym_data_type_datetime] = ACTIONS(2073),
    [sym_data_type_date] = ACTIONS(2073),
    [sym_data_type_time] = ACTIONS(2073),
    [sym_data_type_image] = ACTIONS(2073),
    [sym_data_format_url] = ACTIONS(2073),
    [sym_data_format_phone] = ACTIONS(2073),
    [sym_data_format_email] = ACTIONS(2073),
    [sym_data_format_iso8601] = ACTIONS(2073),
    [sym_auditlog_priv] = ACTIONS(2073),
    [sym_configuration_priv] = ACTIONS(2073),
    [sym_deviceconfig_priv] = ACTIONS(2073),
    [sym_ftp_priv] = ACTIONS(2073),
    [sym_http_priv] = ACTIONS(2073),
    [sym_networkconfig_priv] = ACTIONS(2073),
    [sym_programport_priv] = ACTIONS(2073),
    [sym_remoteui_priv] = ACTIONS(2073),
    [sym_securitycontrol_priv] = ACTIONS(2073),
    [sym_softwaremanagement_priv] = ACTIONS(2073),
    [sym_terminal_priv] = ACTIONS(2073),
    [sym_tpadmin_priv] = ACTIONS(2073),
    [sym_user1_priv] = ACTIONS(2073),
    [sym_user2_priv] = ACTIONS(2073),
    [sym_user3_priv] = ACTIONS(2073),
    [sym_user4_priv] = ACTIONS(2073),
    [sym_usermanagement_priv] = ACTIONS(2073),
    [sym_valid_account] = ACTIONS(2073),
    [sym_err_name_invalid] = ACTIONS(2073),
    [sym_err_password_invalid] = ACTIONS(2073),
    [sym_err_invalid_account] = ACTIONS(2073),
    [sym_err_no_authorization] = ACTIONS(2073),
    [sym_err_invalid_parameter] = ACTIONS(2073),
    [sym_err_user_locked_out] = ACTIONS(2073),
    [sym_err_user_account_expired] = ACTIONS(2073),
    [sym_err_general_error] = ACTIONS(2073),
    [sym_err_authentication_error] = ACTIONS(2073),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2073),
    [sym_tls_validate_certificate] = ACTIONS(2073),
    [sym_tdata] = ACTIONS(2073),
    [sym_tchannel] = ACTIONS(2073),
    [sym_tlevel] = ACTIONS(2073),
    [sym_tbutton] = ACTIONS(2073),
    [sym_ttimeline] = ACTIONS(2073),
    [sym_tcustom] = ACTIONS(2073),
    [sym_url_struct] = ACTIONS(2073),
    [sym_dns_struct] = ACTIONS(2073),
    [sym_ip_address_struct] = ACTIONS(2073),
    [sym_dev_info_struct] = ACTIONS(2073),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2073),
    [sym_data_feed] = ACTIONS(2073),
    [sym_data_field] = ACTIONS(2073),
    [sym_data_record] = ACTIONS(2073),
    [sym_wc_data_feed] = ACTIONS(2073),
    [sym_wc_data_field] = ACTIONS(2073),
    [sym_wc_data_record] = ACTIONS(2073),
    [sym_last_login_info] = ACTIONS(2073),
    [sym____reserved___] = ACTIONS(2073),
    [sym_date] = ACTIONS(2073),
    [sym_day] = ACTIONS(2073),
    [sym_time] = ACTIONS(2073),
    [sym_ldate] = ACTIONS(2073),
    [sym_push_device] = ACTIONS(2073),
    [sym_push_channel] = ACTIONS(2073),
    [sym_push_devchan] = ACTIONS(2073),
    [sym_release_device] = ACTIONS(2073),
    [sym_release_channel] = ACTIONS(2073),
    [sym_release_devchan] = ACTIONS(2073),
    [sym_master_slot] = ACTIONS(2073),
    [sym_get_pulse_time] = ACTIONS(2073),
    [sym_get_timer] = ACTIONS(2073),
    [sym_master_sn] = ACTIONS(2073),
    [sym_system_number] = ACTIONS(2073),
    [sym_button] = ACTIONS(2073),
    [sym_dv_channel] = ACTIONS(2073),
    [sym_data] = ACTIONS(2073),
    [sym_level] = ACTIONS(2073),
    [sym_timeline] = ACTIONS(2073),
    [sym_channel] = ACTIONS(2073),
    [sym_custom] = ACTIONS(2073),
    [sym_atoi] = ACTIONS(2073),
    [sym_atol] = ACTIONS(2073),
    [sym_atof] = ACTIONS(2073),
    [sym_itoa] = ACTIONS(2073),
    [sym_itohex] = ACTIONS(2073),
    [sym_hextoi] = ACTIONS(2073),
    [sym_ftoa] = ACTIONS(2073),
    [sym_find_string] = ACTIONS(2073),
    [sym_left_string] = ACTIONS(2073),
    [sym_right_string] = ACTIONS(2073),
    [sym_mid_string] = ACTIONS(2073),
    [sym_remove_string] = ACTIONS(2073),
    [sym_get_buffer_string] = ACTIONS(2073),
    [sym_get_buffer_char] = ACTIONS(2073),
    [sym_get_multi_buffer_string] = ACTIONS(2073),
    [sym_length_string] = ACTIONS(2073),
    [sym_max_length_string] = ACTIONS(2073),
    [sym_set_length_string] = ACTIONS(2073),
    [sym_compare_string] = ACTIONS(2073),
    [sym_upper_string] = ACTIONS(2073),
    [sym_lower_string] = ACTIONS(2073),
    [sym_get_last] = ACTIONS(2073),
    [sym_date_to_day] = ACTIONS(2073),
    [sym_date_to_month] = ACTIONS(2073),
    [sym_date_to_year] = ACTIONS(2073),
    [sym_time_to_hour] = ACTIONS(2073),
    [sym_time_to_minute] = ACTIONS(2073),
    [sym_time_to_second] = ACTIONS(2073),
    [sym_day_of_week] = ACTIONS(2073),
    [sym_set_pulse_time] = ACTIONS(2073),
    [sym_set_timer] = ACTIONS(2073),
    [sym_file_open] = ACTIONS(2073),
    [sym_file_close] = ACTIONS(2073),
    [sym_file_read] = ACTIONS(2073),
    [sym_file_read_line] = ACTIONS(2073),
    [sym_file_write] = ACTIONS(2073),
    [sym_file_write_line] = ACTIONS(2073),
    [sym_file_copy] = ACTIONS(2073),
    [sym_file_delete] = ACTIONS(2073),
    [sym_file_rename] = ACTIONS(2073),
    [sym_file_dir] = ACTIONS(2073),
    [sym_file_getdir] = ACTIONS(2073),
    [sym_file_setdir] = ACTIONS(2073),
    [sym_file_create_dir] = ACTIONS(2073),
    [sym_file_remove_dir] = ACTIONS(2073),
    [sym_file_seek] = ACTIONS(2073),
    [sym_length_array] = ACTIONS(2073),
    [sym_max_length_array] = ACTIONS(2073),
    [sym_set_length_array] = ACTIONS(2073),
    [sym_ip_client_open] = ACTIONS(2073),
    [sym_ip_client_close] = ACTIONS(2073),
    [sym_ip_server_open] = ACTIONS(2073),
    [sym_ip_server_close] = ACTIONS(2073),
    [sym_ip_bound_client_open] = ACTIONS(2073),
    [sym_ip_mc_server_open] = ACTIONS(2073),
    [sym_ip_set_option] = ACTIONS(2073),
    [sym_ssh_client_open] = ACTIONS(2073),
    [sym_ssh_client_close] = ACTIONS(2073),
    [sym_tls_client_open] = ACTIONS(2073),
    [sym_tls_client_close] = ACTIONS(2073),
    [sym_do_push] = ACTIONS(2073),
    [sym_do_push_timed] = ACTIONS(2073),
    [sym_do_release] = ACTIONS(2073),
    [sym_device_id] = ACTIONS(2073),
    [sym_device_info] = ACTIONS(2073),
    [sym_device_id_string] = ACTIONS(2073),
    [sym_device_standby] = ACTIONS(2073),
    [sym_device_wake] = ACTIONS(2073),
    [sym_timeline_create] = ACTIONS(2073),
    [sym_timeline_kill] = ACTIONS(2073),
    [sym_timeline_reload] = ACTIONS(2073),
    [sym_timeline_pause] = ACTIONS(2073),
    [sym_timeline_restart] = ACTIONS(2073),
    [sym_timeline_set] = ACTIONS(2073),
    [sym_timeline_get] = ACTIONS(2073),
    [sym_timeline_active] = ACTIONS(2073),
    [sym_timeline_dynamic_id] = ACTIONS(2073),
    [sym_variable_to_string] = ACTIONS(2073),
    [sym_string_to_variable] = ACTIONS(2073),
    [sym_length_variable_to_string] = ACTIONS(2073),
    [sym_type_cast] = ACTIONS(2073),
    [sym_variable_to_xml] = ACTIONS(2073),
    [sym_xml_to_variable] = ACTIONS(2073),
    [sym_length_variable_to_xml] = ACTIONS(2073),
    [sym_redirect_string] = ACTIONS(2073),
    [sym_random_number] = ACTIONS(2073),
    [sym_get_unique_id] = ACTIONS(2073),
    [sym_get_system_number] = ACTIONS(2073),
    [sym_set_system_number] = ACTIONS(2073),
    [sym_get_serial_number] = ACTIONS(2073),
    [sym_get_master_build] = ACTIONS(2073),
    [sym_set_validation_code] = ACTIONS(2073),
    [sym_reboot] = ACTIONS(2073),
    [sym_load_duet_module] = ACTIONS(2073),
    [sym_unload_duet_module] = ACTIONS(2073),
    [sym_duet_mem_size_set] = ACTIONS(2073),
    [sym_duet_mem_size_get] = ACTIONS(2073),
    [sym_dynamic_application_device] = ACTIONS(2073),
    [sym_dynamic_polled_port] = ACTIONS(2073),
    [sym_static_port_binding] = ACTIONS(2073),
    [sym_static_ip_binding] = ACTIONS(2073),
    [sym_get_dns_list] = ACTIONS(2073),
    [sym_set_dns_list] = ACTIONS(2073),
    [sym_get_ip_address] = ACTIONS(2073),
    [sym_set_ip_address] = ACTIONS(2073),
    [sym_get_url_list] = ACTIONS(2073),
    [sym_add_url_entry] = ACTIONS(2073),
    [sym_delete_url_entry] = ACTIONS(2073),
    [sym_abs_value] = ACTIONS(2073),
    [sym_max_value] = ACTIONS(2073),
    [sym_min_value] = ACTIONS(2073),
    [sym_exp_value] = ACTIONS(2073),
    [sym_log_value] = ACTIONS(2073),
    [sym_log10_value] = ACTIONS(2073),
    [sym_power_value] = ACTIONS(2073),
    [sym_sqrt_value] = ACTIONS(2073),
    [sym_raw_be] = ACTIONS(2073),
    [sym_raw_le] = ACTIONS(2073),
    [sym_internal_queue_size_set] = ACTIONS(2073),
    [sym_internal_queue_size_get] = ACTIONS(2073),
    [sym_internal_threshold_set] = ACTIONS(2073),
    [sym_internal_threshold_get] = ACTIONS(2073),
    [sym_rebuilt_event] = ACTIONS(2073),
    [sym_do_custom_event] = ACTIONS(2073),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2073),
    [sym_clkmgr_set_clk_source] = ACTIONS(2073),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2073),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2073),
    [sym_clkmgr_get_timezone] = ACTIONS(2073),
    [sym_clkmgr_set_timezone] = ACTIONS(2073),
    [sym_clkmgr_get_resync_period] = ACTIONS(2073),
    [sym_clkmgr_set_resync_period] = ACTIONS(2073),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2073),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2073),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2073),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2073),
    [sym_clkmgr_get_timeservers] = ACTIONS(2073),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2073),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2073),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2073),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2073),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2073),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2073),
    [sym_set_log_level] = ACTIONS(2073),
    [sym_get_log_level] = ACTIONS(2073),
    [sym_amx_log] = ACTIONS(2073),
    [sym_smtp_server_config_set] = ACTIONS(2073),
    [sym_smtp_server_config_get] = ACTIONS(2073),
    [sym_smtp_send] = ACTIONS(2073),
    [sym_validate_netlinx_account] = ACTIONS(2073),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2073),
    [sym_authenticate_certificate] = ACTIONS(2073),
    [sym_audit_netlinx_session_event] = ACTIONS(2073),
    [sym_audit_netlinx_generic_event] = ACTIONS(2073),
    [sym_get_available_flash_disk_space] = ACTIONS(2073),
    [sym_get_max_flash_disk_space] = ACTIONS(2073),
    [sym_data_create_feed] = ACTIONS(2073),
    [sym_data_delete_feed] = ACTIONS(2073),
    [sym_data_publish_feed] = ACTIONS(2073),
    [sym_data_get_published_feed] = ACTIONS(2073),
    [sym_data_add_record] = ACTIONS(2073),
    [sym_data_get_event_record] = ACTIONS(2073),
    [sym__wc_data_create_feed] = ACTIONS(2073),
    [sym__wc_data_add_record] = ACTIONS(2073),
    [sym__wc_data_get_event_record] = ACTIONS(2073),
    [sym_format] = ACTIONS(2073),
    [sym_astro_clock] = ACTIONS(2073),
    [sym_set_outdoor_temperature] = ACTIONS(2073),
    [sym_set_virtual_level_count] = ACTIONS(2073),
    [sym_set_virtual_channel_count] = ACTIONS(2073),
    [sym_set_virtual_port_count] = ACTIONS(2073),
    [sym__wc_encode_int] = ACTIONS(2073),
    [sym__wc_decode_int] = ACTIONS(2073),
    [sym__wc_int] = ACTIONS(2073),
    [sym__wc_to_ch_int] = ACTIONS(2073),
    [sym__ch_to_wc_int] = ACTIONS(2073),
    [sym__wc_find_string_int] = ACTIONS(2073),
    [sym__wc_left_string_int] = ACTIONS(2073),
    [sym__wc_lower_string_int] = ACTIONS(2073),
    [sym__wc_mid_string_int] = ACTIONS(2073),
    [sym__wc_remove_string_int] = ACTIONS(2073),
    [sym__wc_right_string_int] = ACTIONS(2073),
    [sym__wc_upper_string_int] = ACTIONS(2073),
    [sym__wc_compare_string_int] = ACTIONS(2073),
    [sym__wc_get_buffer_char_int] = ACTIONS(2073),
    [sym__wc_get_buffer_string_int] = ACTIONS(2073),
    [sym__wc_concat_string_int] = ACTIONS(2073),
    [sym__wc_file_open_int] = ACTIONS(2073),
    [sym__wc_file_close_int] = ACTIONS(2073),
    [sym__wc_file_read_int] = ACTIONS(2073),
    [sym__wc_file_read_line_int] = ACTIONS(2073),
    [sym__wc_file_write_int] = ACTIONS(2073),
    [sym__wc_file_write_line_int] = ACTIONS(2073),
    [aux_sym_preproc_include_token1] = ACTIONS(2073),
    [aux_sym_preproc_define_token1] = ACTIONS(2075),
    [aux_sym_preproc_warn_token1] = ACTIONS(2075),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2075),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2075),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2075),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2075),
    [aux_sym_preproc_else_token1] = ACTIONS(2075),
    [anon_sym_LBRACK] = ACTIONS(2075),
    [anon_sym_LBRACE] = ACTIONS(2075),
    [anon_sym_LPAREN] = ACTIONS(2073),
    [anon_sym_BANG] = ACTIONS(2075),
    [anon_sym_TILDE] = ACTIONS(2075),
    [anon_sym_DASH] = ACTIONS(2073),
    [anon_sym_PLUS] = ACTIONS(2073),
    [anon_sym_DASH_DASH] = ACTIONS(2075),
    [anon_sym_PLUS_PLUS] = ACTIONS(2075),
    [anon_sym_DQUOTE] = ACTIONS(2075),
    [anon_sym_SQUOTE] = ACTIONS(2075),
    [sym_number_literal] = ACTIONS(2075),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(357)] = {
    [ts_builtin_sym_end] = ACTIONS(1791),
    [sym_identifier] = ACTIONS(1789),
    [sym_program_name_keyword] = ACTIONS(1789),
    [sym_module_name_keyword] = ACTIONS(1789),
    [sym_define_device_keyword] = ACTIONS(1789),
    [sym_define_combine_keyword] = ACTIONS(1789),
    [sym_define_constant_keyword] = ACTIONS(1789),
    [sym_define_type_keyword] = ACTIONS(1789),
    [sym_define_variable_keyword] = ACTIONS(1789),
    [sym_define_system_variable_keyword] = ACTIONS(1789),
    [sym_define_connect_level_keyword] = ACTIONS(1789),
    [sym_define_latching_keyword] = ACTIONS(1789),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1789),
    [sym_define_toggling_keyword] = ACTIONS(1789),
    [sym_define_call_keyword] = ACTIONS(1789),
    [sym_define_function_keyword] = ACTIONS(1789),
    [sym_define_library_function_keyword] = ACTIONS(1789),
    [sym_define_start_keyword] = ACTIONS(1789),
    [sym_define_event_keyword] = ACTIONS(1789),
    [sym_define_program_keyword] = ACTIONS(1789),
    [sym_define_module_keyword] = ACTIONS(1789),
    [sym_char_keyword] = ACTIONS(1789),
    [sym_widechar_keyword] = ACTIONS(1789),
    [sym_integer_keyword] = ACTIONS(1789),
    [sym_sinteger_keyword] = ACTIONS(1789),
    [sym_long_keyword] = ACTIONS(1789),
    [sym_slong_keyword] = ACTIONS(1789),
    [sym_float_keyword] = ACTIONS(1789),
    [sym_double_keyword] = ACTIONS(1789),
    [sym_variant_keyword] = ACTIONS(1789),
    [sym_variantarray_keyword] = ACTIONS(1789),
    [sym_local_var_keyword] = ACTIONS(1789),
    [sym_stack_var_keyword] = ACTIONS(1789),
    [sym_constant_keyword] = ACTIONS(1789),
    [sym_volatile_keyword] = ACTIONS(1789),
    [sym_non_volatile_keyword] = ACTIONS(1789),
    [sym_persistent_keyword] = ACTIONS(1789),
    [sym_dev_keyword] = ACTIONS(1789),
    [sym_devlev_keyword] = ACTIONS(1789),
    [sym_devchan_keyword] = ACTIONS(1789),
    [sym_if_keyword] = ACTIONS(1789),
    [sym_else_keyword] = ACTIONS(1789),
    [sym_switch_keyword] = ACTIONS(1789),
    [sym_case_keyword] = ACTIONS(1789),
    [sym_default_keyword] = ACTIONS(1789),
    [sym_while_keyword] = ACTIONS(1789),
    [sym_for_keyword] = ACTIONS(1789),
    [sym_break_keyword] = ACTIONS(1789),
    [sym_continue_keyword] = ACTIONS(1789),
    [sym_return_keyword] = ACTIONS(1789),
    [sym_select_keyword] = ACTIONS(1789),
    [sym_active_keyword] = ACTIONS(1789),
    [sym_struct_keyword] = ACTIONS(1789),
    [sym_structure_keyword] = ACTIONS(1789),
    [sym_button_event_keyword] = ACTIONS(1789),
    [sym_channel_event_keyword] = ACTIONS(1789),
    [sym_level_event_keyword] = ACTIONS(1789),
    [sym_data_event_keyword] = ACTIONS(1789),
    [sym_timeline_event_keyword] = ACTIONS(1789),
    [sym_custom_event_keyword] = ACTIONS(1789),
    [sym_push_keyword] = ACTIONS(1789),
    [sym_release_keyword] = ACTIONS(1789),
    [aux_sym_on_keyword_token1] = ACTIONS(1789),
    [aux_sym_off_keyword_token1] = ACTIONS(1789),
    [sym_send_level_keyword] = ACTIONS(1789),
    [sym_send_string_keyword] = ACTIONS(1789),
    [sym_send_command_keyword] = ACTIONS(1789),
    [sym_clear_buffer_keyword] = ACTIONS(1789),
    [sym_create_buffer_keyword] = ACTIONS(1789),
    [sym_create_multi_buffer_keyword] = ACTIONS(1789),
    [sym_call_keyword] = ACTIONS(1789),
    [sym_system_call_keyword] = ACTIONS(1789),
    [sym_devchan_to_keyword] = ACTIONS(1789),
    [sym_devchan_min_to_keyword] = ACTIONS(1789),
    [sym_devchan_total_off_keyword] = ACTIONS(1789),
    [sym_devchan_pulse_keyword] = ACTIONS(1789),
    [sym_wait_keyword] = ACTIONS(1789),
    [sym_cancel_wait_keyword] = ACTIONS(1789),
    [sym_wait_until_keyword] = ACTIONS(1789),
    [sym_cancel_wait_until_keyword] = ACTIONS(1789),
    [sym_cancel_all_wait_keyword] = ACTIONS(1789),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1789),
    [sym_not] = ACTIONS(1789),
    [sym_bnot] = ACTIONS(1789),
    [sym___file__] = ACTIONS(1789),
    [sym___date__] = ACTIONS(1789),
    [sym___ldate__] = ACTIONS(1789),
    [sym___time__] = ACTIONS(1789),
    [sym___line__] = ACTIONS(1789),
    [sym___name__] = ACTIONS(1789),
    [sym___netlinx__] = ACTIONS(1789),
    [sym_netlinx_axi_version] = ACTIONS(1789),
    [sym_url_flg_tcp] = ACTIONS(1789),
    [sym_url_flg_accinfopresent] = ACTIONS(1789),
    [sym_url_flg_temp] = ACTIONS(1789),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1789),
    [sym_url_flg_stat_mask] = ACTIONS(1789),
    [sym_url_flg_stat_lookup] = ACTIONS(1789),
    [sym_url_flg_stat_connecting] = ACTIONS(1789),
    [sym_url_flg_stat_waiting] = ACTIONS(1789),
    [sym_url_flg_stat_connected] = ACTIONS(1789),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1789),
    [sym_first_virtual_device] = ACTIONS(1789),
    [sym_first_local_port] = ACTIONS(1789),
    [sym_dynamic_virtual_device] = ACTIONS(1789),
    [sym_timeline_once] = ACTIONS(1789),
    [sym_timeline_repeat] = ACTIONS(1789),
    [sym_timeline_absolute] = ACTIONS(1789),
    [sym_timeline_relative] = ACTIONS(1789),
    [sym_do_push_timed_infinite] = ACTIONS(1789),
    [sym_source_type_no_address] = ACTIONS(1789),
    [sym_source_type_neuron_id] = ACTIONS(1789),
    [sym_source_type_ip_address] = ACTIONS(1789),
    [sym_source_type_axlink] = ACTIONS(1789),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1789),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1789),
    [sym_source_type_ip_socket_address] = ACTIONS(1789),
    [sym_source_type_rs232] = ACTIONS(1789),
    [sym_source_type_internal] = ACTIONS(1789),
    [sym_source_type_mac_address] = ACTIONS(1789),
    [sym_source_type_ipv4_port] = ACTIONS(1789),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1789),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1789),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1789),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1789),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1789),
    [sym_true] = ACTIONS(1789),
    [sym_false] = ACTIONS(1789),
    [sym_file_read_only] = ACTIONS(1789),
    [sym_file_rw_new] = ACTIONS(1789),
    [sym_file_rw_append] = ACTIONS(1789),
    [sym_ip_tcp] = ACTIONS(1789),
    [sym_ip_udp] = ACTIONS(1789),
    [sym_ip_udp_2way] = ACTIONS(1789),
    [sym_xml_encode_types] = ACTIONS(1789),
    [sym_xml_encode_char_as_list] = ACTIONS(1789),
    [sym_xml_encode_le] = ACTIONS(1789),
    [sym_xml_decode_types] = ACTIONS(1789),
    [sym_xml_decode_no_preserve] = ACTIONS(1789),
    [sym_duet_dev_type_display_device] = ACTIONS(1789),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1789),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1789),
    [sym_duet_dev_type_io_device] = ACTIONS(1789),
    [sym_duet_dev_type_relay_device] = ACTIONS(1789),
    [sym_duet_dev_type_ups] = ACTIONS(1789),
    [sym_duet_dev_type_amplifier] = ACTIONS(1789),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1789),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1789),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1789),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1789),
    [sym_duet_dev_type_camera] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1789),
    [sym_duet_dev_type_dss] = ACTIONS(1789),
    [sym_duet_dev_type_dvr] = ACTIONS(1789),
    [sym_duet_dev_type_disc_device] = ACTIONS(1789),
    [sym_duet_dev_type_document_camera] = ACTIONS(1789),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1789),
    [sym_duet_dev_type_hvac] = ACTIONS(1789),
    [sym_duet_dev_type_keypad] = ACTIONS(1789),
    [sym_duet_dev_type_light] = ACTIONS(1789),
    [sym_duet_dev_type_monitor] = ACTIONS(1789),
    [sym_duet_dev_type_motor] = ACTIONS(1789),
    [sym_duet_dev_type_multi_window] = ACTIONS(1789),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1789),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1789),
    [sym_duet_dev_type_receiver] = ACTIONS(1789),
    [sym_duet_dev_type_security_system] = ACTIONS(1789),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1789),
    [sym_duet_dev_type_settop_box] = ACTIONS(1789),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1789),
    [sym_duet_dev_type_switcher] = ACTIONS(1789),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1789),
    [sym_duet_dev_type_tv] = ACTIONS(1789),
    [sym_duet_dev_type_utility] = ACTIONS(1789),
    [sym_duet_dev_type_vcr] = ACTIONS(1789),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1789),
    [sym_duet_dev_type_video_processor] = ACTIONS(1789),
    [sym_duet_dev_type_video_projector] = ACTIONS(1789),
    [sym_duet_dev_type_video_wall] = ACTIONS(1789),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1789),
    [sym_duet_dev_type_weather] = ACTIONS(1789),
    [sym_duet_dev_not_polled] = ACTIONS(1789),
    [sym_duet_dev_polled] = ACTIONS(1789),
    [sym_clkmgr_mode_network] = ACTIONS(1789),
    [sym_clkmgr_mode_standalone] = ACTIONS(1789),
    [sym_normal_standby] = ACTIONS(1789),
    [sym_normal_wake] = ACTIONS(1789),
    [sym_ip_multicast_ttl_option] = ACTIONS(1789),
    [sym_ip_ttl_subnet] = ACTIONS(1789),
    [sym_ip_ttl_site] = ACTIONS(1789),
    [sym_ip_ttl_region] = ACTIONS(1789),
    [sym_ip_ttl_continent] = ACTIONS(1789),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1789),
    [sym_ip_nodelay_on] = ACTIONS(1789),
    [sym_ip_nodelay_off] = ACTIONS(1789),
    [sym_amx_error] = ACTIONS(1789),
    [sym_amx_warning] = ACTIONS(1789),
    [sym_amx_info] = ACTIONS(1789),
    [sym_amx_debug] = ACTIONS(1789),
    [sym_netlinx_login_success] = ACTIONS(1789),
    [sym_netlinx_login_fail] = ACTIONS(1789),
    [sym_netlinx_logout] = ACTIONS(1789),
    [sym_null_str] = ACTIONS(1789),
    [sym_smtp_address] = ACTIONS(1789),
    [sym_smtp_port_number] = ACTIONS(1789),
    [sym_smtp_username] = ACTIONS(1789),
    [sym_smtp_password] = ACTIONS(1789),
    [sym_smtp_require_tls] = ACTIONS(1789),
    [sym_smtp_from] = ACTIONS(1789),
    [sym_smtp_tls_true] = ACTIONS(1789),
    [sym_smtp_tls_false] = ACTIONS(1789),
    [sym_listview_on_row_select_event] = ACTIONS(1789),
    [sym_data_structure_unknown] = ACTIONS(1789),
    [sym_data_structure_datafeed] = ACTIONS(1789),
    [sym_data_structure_datarecordset] = ACTIONS(1789),
    [sym_data_structure_datarecord] = ACTIONS(1789),
    [sym_data_structure_datafield] = ACTIONS(1789),
    [sym_data_max_content_field_count] = ACTIONS(1789),
    [sym_data_max_metadata_field_count] = ACTIONS(1789),
    [sym_data_max_id_length] = ACTIONS(1789),
    [sym_data_max_name_length] = ACTIONS(1789),
    [sym_data_max_description_length] = ACTIONS(1789),
    [sym_data_max_type_length] = ACTIONS(1789),
    [sym_data_max_format_length] = ACTIONS(1789),
    [sym_data_max_source_length] = ACTIONS(1789),
    [sym_data_max_value_length] = ACTIONS(1789),
    [sym_data_max_label_length] = ACTIONS(1789),
    [sym_data_max_url_length] = ACTIONS(1789),
    [sym_max_last_login_info_length] = ACTIONS(1789),
    [sym_data_type_unknown] = ACTIONS(1789),
    [sym_data_type_string] = ACTIONS(1789),
    [sym_data_type_datetime] = ACTIONS(1789),
    [sym_data_type_date] = ACTIONS(1789),
    [sym_data_type_time] = ACTIONS(1789),
    [sym_data_type_image] = ACTIONS(1789),
    [sym_data_format_url] = ACTIONS(1789),
    [sym_data_format_phone] = ACTIONS(1789),
    [sym_data_format_email] = ACTIONS(1789),
    [sym_data_format_iso8601] = ACTIONS(1789),
    [sym_auditlog_priv] = ACTIONS(1789),
    [sym_configuration_priv] = ACTIONS(1789),
    [sym_deviceconfig_priv] = ACTIONS(1789),
    [sym_ftp_priv] = ACTIONS(1789),
    [sym_http_priv] = ACTIONS(1789),
    [sym_networkconfig_priv] = ACTIONS(1789),
    [sym_programport_priv] = ACTIONS(1789),
    [sym_remoteui_priv] = ACTIONS(1789),
    [sym_securitycontrol_priv] = ACTIONS(1789),
    [sym_softwaremanagement_priv] = ACTIONS(1789),
    [sym_terminal_priv] = ACTIONS(1789),
    [sym_tpadmin_priv] = ACTIONS(1789),
    [sym_user1_priv] = ACTIONS(1789),
    [sym_user2_priv] = ACTIONS(1789),
    [sym_user3_priv] = ACTIONS(1789),
    [sym_user4_priv] = ACTIONS(1789),
    [sym_usermanagement_priv] = ACTIONS(1789),
    [sym_valid_account] = ACTIONS(1789),
    [sym_err_name_invalid] = ACTIONS(1789),
    [sym_err_password_invalid] = ACTIONS(1789),
    [sym_err_invalid_account] = ACTIONS(1789),
    [sym_err_no_authorization] = ACTIONS(1789),
    [sym_err_invalid_parameter] = ACTIONS(1789),
    [sym_err_user_locked_out] = ACTIONS(1789),
    [sym_err_user_account_expired] = ACTIONS(1789),
    [sym_err_general_error] = ACTIONS(1789),
    [sym_err_authentication_error] = ACTIONS(1789),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1789),
    [sym_tls_validate_certificate] = ACTIONS(1789),
    [sym_tdata] = ACTIONS(1789),
    [sym_tchannel] = ACTIONS(1789),
    [sym_tlevel] = ACTIONS(1789),
    [sym_tbutton] = ACTIONS(1789),
    [sym_ttimeline] = ACTIONS(1789),
    [sym_tcustom] = ACTIONS(1789),
    [sym_url_struct] = ACTIONS(1789),
    [sym_dns_struct] = ACTIONS(1789),
    [sym_ip_address_struct] = ACTIONS(1789),
    [sym_dev_info_struct] = ACTIONS(1789),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1789),
    [sym_data_feed] = ACTIONS(1789),
    [sym_data_field] = ACTIONS(1789),
    [sym_data_record] = ACTIONS(1789),
    [sym_wc_data_feed] = ACTIONS(1789),
    [sym_wc_data_field] = ACTIONS(1789),
    [sym_wc_data_record] = ACTIONS(1789),
    [sym_last_login_info] = ACTIONS(1789),
    [sym____reserved___] = ACTIONS(1789),
    [sym_date] = ACTIONS(1789),
    [sym_day] = ACTIONS(1789),
    [sym_time] = ACTIONS(1789),
    [sym_ldate] = ACTIONS(1789),
    [sym_push_device] = ACTIONS(1789),
    [sym_push_channel] = ACTIONS(1789),
    [sym_push_devchan] = ACTIONS(1789),
    [sym_release_device] = ACTIONS(1789),
    [sym_release_channel] = ACTIONS(1789),
    [sym_release_devchan] = ACTIONS(1789),
    [sym_master_slot] = ACTIONS(1789),
    [sym_get_pulse_time] = ACTIONS(1789),
    [sym_get_timer] = ACTIONS(1789),
    [sym_master_sn] = ACTIONS(1789),
    [sym_system_number] = ACTIONS(1789),
    [sym_button] = ACTIONS(1789),
    [sym_dv_channel] = ACTIONS(1789),
    [sym_data] = ACTIONS(1789),
    [sym_level] = ACTIONS(1789),
    [sym_timeline] = ACTIONS(1789),
    [sym_channel] = ACTIONS(1789),
    [sym_custom] = ACTIONS(1789),
    [sym_atoi] = ACTIONS(1789),
    [sym_atol] = ACTIONS(1789),
    [sym_atof] = ACTIONS(1789),
    [sym_itoa] = ACTIONS(1789),
    [sym_itohex] = ACTIONS(1789),
    [sym_hextoi] = ACTIONS(1789),
    [sym_ftoa] = ACTIONS(1789),
    [sym_find_string] = ACTIONS(1789),
    [sym_left_string] = ACTIONS(1789),
    [sym_right_string] = ACTIONS(1789),
    [sym_mid_string] = ACTIONS(1789),
    [sym_remove_string] = ACTIONS(1789),
    [sym_get_buffer_string] = ACTIONS(1789),
    [sym_get_buffer_char] = ACTIONS(1789),
    [sym_get_multi_buffer_string] = ACTIONS(1789),
    [sym_length_string] = ACTIONS(1789),
    [sym_max_length_string] = ACTIONS(1789),
    [sym_set_length_string] = ACTIONS(1789),
    [sym_compare_string] = ACTIONS(1789),
    [sym_upper_string] = ACTIONS(1789),
    [sym_lower_string] = ACTIONS(1789),
    [sym_get_last] = ACTIONS(1789),
    [sym_date_to_day] = ACTIONS(1789),
    [sym_date_to_month] = ACTIONS(1789),
    [sym_date_to_year] = ACTIONS(1789),
    [sym_time_to_hour] = ACTIONS(1789),
    [sym_time_to_minute] = ACTIONS(1789),
    [sym_time_to_second] = ACTIONS(1789),
    [sym_day_of_week] = ACTIONS(1789),
    [sym_set_pulse_time] = ACTIONS(1789),
    [sym_set_timer] = ACTIONS(1789),
    [sym_file_open] = ACTIONS(1789),
    [sym_file_close] = ACTIONS(1789),
    [sym_file_read] = ACTIONS(1789),
    [sym_file_read_line] = ACTIONS(1789),
    [sym_file_write] = ACTIONS(1789),
    [sym_file_write_line] = ACTIONS(1789),
    [sym_file_copy] = ACTIONS(1789),
    [sym_file_delete] = ACTIONS(1789),
    [sym_file_rename] = ACTIONS(1789),
    [sym_file_dir] = ACTIONS(1789),
    [sym_file_getdir] = ACTIONS(1789),
    [sym_file_setdir] = ACTIONS(1789),
    [sym_file_create_dir] = ACTIONS(1789),
    [sym_file_remove_dir] = ACTIONS(1789),
    [sym_file_seek] = ACTIONS(1789),
    [sym_length_array] = ACTIONS(1789),
    [sym_max_length_array] = ACTIONS(1789),
    [sym_set_length_array] = ACTIONS(1789),
    [sym_ip_client_open] = ACTIONS(1789),
    [sym_ip_client_close] = ACTIONS(1789),
    [sym_ip_server_open] = ACTIONS(1789),
    [sym_ip_server_close] = ACTIONS(1789),
    [sym_ip_bound_client_open] = ACTIONS(1789),
    [sym_ip_mc_server_open] = ACTIONS(1789),
    [sym_ip_set_option] = ACTIONS(1789),
    [sym_ssh_client_open] = ACTIONS(1789),
    [sym_ssh_client_close] = ACTIONS(1789),
    [sym_tls_client_open] = ACTIONS(1789),
    [sym_tls_client_close] = ACTIONS(1789),
    [sym_do_push] = ACTIONS(1789),
    [sym_do_push_timed] = ACTIONS(1789),
    [sym_do_release] = ACTIONS(1789),
    [sym_device_id] = ACTIONS(1789),
    [sym_device_info] = ACTIONS(1789),
    [sym_device_id_string] = ACTIONS(1789),
    [sym_device_standby] = ACTIONS(1789),
    [sym_device_wake] = ACTIONS(1789),
    [sym_timeline_create] = ACTIONS(1789),
    [sym_timeline_kill] = ACTIONS(1789),
    [sym_timeline_reload] = ACTIONS(1789),
    [sym_timeline_pause] = ACTIONS(1789),
    [sym_timeline_restart] = ACTIONS(1789),
    [sym_timeline_set] = ACTIONS(1789),
    [sym_timeline_get] = ACTIONS(1789),
    [sym_timeline_active] = ACTIONS(1789),
    [sym_timeline_dynamic_id] = ACTIONS(1789),
    [sym_variable_to_string] = ACTIONS(1789),
    [sym_string_to_variable] = ACTIONS(1789),
    [sym_length_variable_to_string] = ACTIONS(1789),
    [sym_type_cast] = ACTIONS(1789),
    [sym_variable_to_xml] = ACTIONS(1789),
    [sym_xml_to_variable] = ACTIONS(1789),
    [sym_length_variable_to_xml] = ACTIONS(1789),
    [sym_redirect_string] = ACTIONS(1789),
    [sym_random_number] = ACTIONS(1789),
    [sym_get_unique_id] = ACTIONS(1789),
    [sym_get_system_number] = ACTIONS(1789),
    [sym_set_system_number] = ACTIONS(1789),
    [sym_get_serial_number] = ACTIONS(1789),
    [sym_get_master_build] = ACTIONS(1789),
    [sym_set_validation_code] = ACTIONS(1789),
    [sym_reboot] = ACTIONS(1789),
    [sym_load_duet_module] = ACTIONS(1789),
    [sym_unload_duet_module] = ACTIONS(1789),
    [sym_duet_mem_size_set] = ACTIONS(1789),
    [sym_duet_mem_size_get] = ACTIONS(1789),
    [sym_dynamic_application_device] = ACTIONS(1789),
    [sym_dynamic_polled_port] = ACTIONS(1789),
    [sym_static_port_binding] = ACTIONS(1789),
    [sym_static_ip_binding] = ACTIONS(1789),
    [sym_get_dns_list] = ACTIONS(1789),
    [sym_set_dns_list] = ACTIONS(1789),
    [sym_get_ip_address] = ACTIONS(1789),
    [sym_set_ip_address] = ACTIONS(1789),
    [sym_get_url_list] = ACTIONS(1789),
    [sym_add_url_entry] = ACTIONS(1789),
    [sym_delete_url_entry] = ACTIONS(1789),
    [sym_abs_value] = ACTIONS(1789),
    [sym_max_value] = ACTIONS(1789),
    [sym_min_value] = ACTIONS(1789),
    [sym_exp_value] = ACTIONS(1789),
    [sym_log_value] = ACTIONS(1789),
    [sym_log10_value] = ACTIONS(1789),
    [sym_power_value] = ACTIONS(1789),
    [sym_sqrt_value] = ACTIONS(1789),
    [sym_raw_be] = ACTIONS(1789),
    [sym_raw_le] = ACTIONS(1789),
    [sym_internal_queue_size_set] = ACTIONS(1789),
    [sym_internal_queue_size_get] = ACTIONS(1789),
    [sym_internal_threshold_set] = ACTIONS(1789),
    [sym_internal_threshold_get] = ACTIONS(1789),
    [sym_rebuilt_event] = ACTIONS(1789),
    [sym_do_custom_event] = ACTIONS(1789),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1789),
    [sym_clkmgr_set_clk_source] = ACTIONS(1789),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1789),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1789),
    [sym_clkmgr_get_timezone] = ACTIONS(1789),
    [sym_clkmgr_set_timezone] = ACTIONS(1789),
    [sym_clkmgr_get_resync_period] = ACTIONS(1789),
    [sym_clkmgr_set_resync_period] = ACTIONS(1789),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1789),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1789),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1789),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1789),
    [sym_clkmgr_get_timeservers] = ACTIONS(1789),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1789),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1789),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1789),
    [sym_set_log_level] = ACTIONS(1789),
    [sym_get_log_level] = ACTIONS(1789),
    [sym_amx_log] = ACTIONS(1789),
    [sym_smtp_server_config_set] = ACTIONS(1789),
    [sym_smtp_server_config_get] = ACTIONS(1789),
    [sym_smtp_send] = ACTIONS(1789),
    [sym_validate_netlinx_account] = ACTIONS(1789),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1789),
    [sym_authenticate_certificate] = ACTIONS(1789),
    [sym_audit_netlinx_session_event] = ACTIONS(1789),
    [sym_audit_netlinx_generic_event] = ACTIONS(1789),
    [sym_get_available_flash_disk_space] = ACTIONS(1789),
    [sym_get_max_flash_disk_space] = ACTIONS(1789),
    [sym_data_create_feed] = ACTIONS(1789),
    [sym_data_delete_feed] = ACTIONS(1789),
    [sym_data_publish_feed] = ACTIONS(1789),
    [sym_data_get_published_feed] = ACTIONS(1789),
    [sym_data_add_record] = ACTIONS(1789),
    [sym_data_get_event_record] = ACTIONS(1789),
    [sym__wc_data_create_feed] = ACTIONS(1789),
    [sym__wc_data_add_record] = ACTIONS(1789),
    [sym__wc_data_get_event_record] = ACTIONS(1789),
    [sym_format] = ACTIONS(1789),
    [sym_astro_clock] = ACTIONS(1789),
    [sym_set_outdoor_temperature] = ACTIONS(1789),
    [sym_set_virtual_level_count] = ACTIONS(1789),
    [sym_set_virtual_channel_count] = ACTIONS(1789),
    [sym_set_virtual_port_count] = ACTIONS(1789),
    [sym__wc_encode_int] = ACTIONS(1789),
    [sym__wc_decode_int] = ACTIONS(1789),
    [sym__wc_int] = ACTIONS(1789),
    [sym__wc_to_ch_int] = ACTIONS(1789),
    [sym__ch_to_wc_int] = ACTIONS(1789),
    [sym__wc_find_string_int] = ACTIONS(1789),
    [sym__wc_left_string_int] = ACTIONS(1789),
    [sym__wc_lower_string_int] = ACTIONS(1789),
    [sym__wc_mid_string_int] = ACTIONS(1789),
    [sym__wc_remove_string_int] = ACTIONS(1789),
    [sym__wc_right_string_int] = ACTIONS(1789),
    [sym__wc_upper_string_int] = ACTIONS(1789),
    [sym__wc_compare_string_int] = ACTIONS(1789),
    [sym__wc_get_buffer_char_int] = ACTIONS(1789),
    [sym__wc_get_buffer_string_int] = ACTIONS(1789),
    [sym__wc_concat_string_int] = ACTIONS(1789),
    [sym__wc_file_open_int] = ACTIONS(1789),
    [sym__wc_file_close_int] = ACTIONS(1789),
    [sym__wc_file_read_int] = ACTIONS(1789),
    [sym__wc_file_read_line_int] = ACTIONS(1789),
    [sym__wc_file_write_int] = ACTIONS(1789),
    [sym__wc_file_write_line_int] = ACTIONS(1789),
    [aux_sym_preproc_include_token1] = ACTIONS(1789),
    [aux_sym_preproc_define_token1] = ACTIONS(1791),
    [aux_sym_preproc_warn_token1] = ACTIONS(1791),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1791),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1791),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1791),
    [anon_sym_LBRACK] = ACTIONS(1791),
    [anon_sym_LBRACE] = ACTIONS(1791),
    [anon_sym_LPAREN] = ACTIONS(1789),
    [anon_sym_BANG] = ACTIONS(1791),
    [anon_sym_TILDE] = ACTIONS(1791),
    [anon_sym_DASH] = ACTIONS(1789),
    [anon_sym_PLUS] = ACTIONS(1789),
    [anon_sym_DASH_DASH] = ACTIONS(1791),
    [anon_sym_PLUS_PLUS] = ACTIONS(1791),
    [anon_sym_DQUOTE] = ACTIONS(1791),
    [anon_sym_SQUOTE] = ACTIONS(1791),
    [sym_number_literal] = ACTIONS(1791),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(358)] = {
    [ts_builtin_sym_end] = ACTIONS(1795),
    [sym_identifier] = ACTIONS(1793),
    [sym_program_name_keyword] = ACTIONS(1793),
    [sym_module_name_keyword] = ACTIONS(1793),
    [sym_define_device_keyword] = ACTIONS(1793),
    [sym_define_combine_keyword] = ACTIONS(1793),
    [sym_define_constant_keyword] = ACTIONS(1793),
    [sym_define_type_keyword] = ACTIONS(1793),
    [sym_define_variable_keyword] = ACTIONS(1793),
    [sym_define_system_variable_keyword] = ACTIONS(1793),
    [sym_define_connect_level_keyword] = ACTIONS(1793),
    [sym_define_latching_keyword] = ACTIONS(1793),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1793),
    [sym_define_toggling_keyword] = ACTIONS(1793),
    [sym_define_call_keyword] = ACTIONS(1793),
    [sym_define_function_keyword] = ACTIONS(1793),
    [sym_define_library_function_keyword] = ACTIONS(1793),
    [sym_define_start_keyword] = ACTIONS(1793),
    [sym_define_event_keyword] = ACTIONS(1793),
    [sym_define_program_keyword] = ACTIONS(1793),
    [sym_define_module_keyword] = ACTIONS(1793),
    [sym_char_keyword] = ACTIONS(1793),
    [sym_widechar_keyword] = ACTIONS(1793),
    [sym_integer_keyword] = ACTIONS(1793),
    [sym_sinteger_keyword] = ACTIONS(1793),
    [sym_long_keyword] = ACTIONS(1793),
    [sym_slong_keyword] = ACTIONS(1793),
    [sym_float_keyword] = ACTIONS(1793),
    [sym_double_keyword] = ACTIONS(1793),
    [sym_variant_keyword] = ACTIONS(1793),
    [sym_variantarray_keyword] = ACTIONS(1793),
    [sym_local_var_keyword] = ACTIONS(1793),
    [sym_stack_var_keyword] = ACTIONS(1793),
    [sym_constant_keyword] = ACTIONS(1793),
    [sym_volatile_keyword] = ACTIONS(1793),
    [sym_non_volatile_keyword] = ACTIONS(1793),
    [sym_persistent_keyword] = ACTIONS(1793),
    [sym_dev_keyword] = ACTIONS(1793),
    [sym_devlev_keyword] = ACTIONS(1793),
    [sym_devchan_keyword] = ACTIONS(1793),
    [sym_if_keyword] = ACTIONS(1793),
    [sym_else_keyword] = ACTIONS(1793),
    [sym_switch_keyword] = ACTIONS(1793),
    [sym_case_keyword] = ACTIONS(1793),
    [sym_default_keyword] = ACTIONS(1793),
    [sym_while_keyword] = ACTIONS(1793),
    [sym_for_keyword] = ACTIONS(1793),
    [sym_break_keyword] = ACTIONS(1793),
    [sym_continue_keyword] = ACTIONS(1793),
    [sym_return_keyword] = ACTIONS(1793),
    [sym_select_keyword] = ACTIONS(1793),
    [sym_active_keyword] = ACTIONS(1793),
    [sym_struct_keyword] = ACTIONS(1793),
    [sym_structure_keyword] = ACTIONS(1793),
    [sym_button_event_keyword] = ACTIONS(1793),
    [sym_channel_event_keyword] = ACTIONS(1793),
    [sym_level_event_keyword] = ACTIONS(1793),
    [sym_data_event_keyword] = ACTIONS(1793),
    [sym_timeline_event_keyword] = ACTIONS(1793),
    [sym_custom_event_keyword] = ACTIONS(1793),
    [sym_push_keyword] = ACTIONS(1793),
    [sym_release_keyword] = ACTIONS(1793),
    [aux_sym_on_keyword_token1] = ACTIONS(1793),
    [aux_sym_off_keyword_token1] = ACTIONS(1793),
    [sym_send_level_keyword] = ACTIONS(1793),
    [sym_send_string_keyword] = ACTIONS(1793),
    [sym_send_command_keyword] = ACTIONS(1793),
    [sym_clear_buffer_keyword] = ACTIONS(1793),
    [sym_create_buffer_keyword] = ACTIONS(1793),
    [sym_create_multi_buffer_keyword] = ACTIONS(1793),
    [sym_call_keyword] = ACTIONS(1793),
    [sym_system_call_keyword] = ACTIONS(1793),
    [sym_devchan_to_keyword] = ACTIONS(1793),
    [sym_devchan_min_to_keyword] = ACTIONS(1793),
    [sym_devchan_total_off_keyword] = ACTIONS(1793),
    [sym_devchan_pulse_keyword] = ACTIONS(1793),
    [sym_wait_keyword] = ACTIONS(1793),
    [sym_cancel_wait_keyword] = ACTIONS(1793),
    [sym_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1793),
    [sym_not] = ACTIONS(1793),
    [sym_bnot] = ACTIONS(1793),
    [sym___file__] = ACTIONS(1793),
    [sym___date__] = ACTIONS(1793),
    [sym___ldate__] = ACTIONS(1793),
    [sym___time__] = ACTIONS(1793),
    [sym___line__] = ACTIONS(1793),
    [sym___name__] = ACTIONS(1793),
    [sym___netlinx__] = ACTIONS(1793),
    [sym_netlinx_axi_version] = ACTIONS(1793),
    [sym_url_flg_tcp] = ACTIONS(1793),
    [sym_url_flg_accinfopresent] = ACTIONS(1793),
    [sym_url_flg_temp] = ACTIONS(1793),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1793),
    [sym_url_flg_stat_mask] = ACTIONS(1793),
    [sym_url_flg_stat_lookup] = ACTIONS(1793),
    [sym_url_flg_stat_connecting] = ACTIONS(1793),
    [sym_url_flg_stat_waiting] = ACTIONS(1793),
    [sym_url_flg_stat_connected] = ACTIONS(1793),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1793),
    [sym_first_virtual_device] = ACTIONS(1793),
    [sym_first_local_port] = ACTIONS(1793),
    [sym_dynamic_virtual_device] = ACTIONS(1793),
    [sym_timeline_once] = ACTIONS(1793),
    [sym_timeline_repeat] = ACTIONS(1793),
    [sym_timeline_absolute] = ACTIONS(1793),
    [sym_timeline_relative] = ACTIONS(1793),
    [sym_do_push_timed_infinite] = ACTIONS(1793),
    [sym_source_type_no_address] = ACTIONS(1793),
    [sym_source_type_neuron_id] = ACTIONS(1793),
    [sym_source_type_ip_address] = ACTIONS(1793),
    [sym_source_type_axlink] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1793),
    [sym_source_type_ip_socket_address] = ACTIONS(1793),
    [sym_source_type_rs232] = ACTIONS(1793),
    [sym_source_type_internal] = ACTIONS(1793),
    [sym_source_type_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1793),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1793),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1793),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1793),
    [sym_true] = ACTIONS(1793),
    [sym_false] = ACTIONS(1793),
    [sym_file_read_only] = ACTIONS(1793),
    [sym_file_rw_new] = ACTIONS(1793),
    [sym_file_rw_append] = ACTIONS(1793),
    [sym_ip_tcp] = ACTIONS(1793),
    [sym_ip_udp] = ACTIONS(1793),
    [sym_ip_udp_2way] = ACTIONS(1793),
    [sym_xml_encode_types] = ACTIONS(1793),
    [sym_xml_encode_char_as_list] = ACTIONS(1793),
    [sym_xml_encode_le] = ACTIONS(1793),
    [sym_xml_decode_types] = ACTIONS(1793),
    [sym_xml_decode_no_preserve] = ACTIONS(1793),
    [sym_duet_dev_type_display_device] = ACTIONS(1793),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_io_device] = ACTIONS(1793),
    [sym_duet_dev_type_relay_device] = ACTIONS(1793),
    [sym_duet_dev_type_ups] = ACTIONS(1793),
    [sym_duet_dev_type_amplifier] = ACTIONS(1793),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1793),
    [sym_duet_dev_type_camera] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1793),
    [sym_duet_dev_type_dss] = ACTIONS(1793),
    [sym_duet_dev_type_dvr] = ACTIONS(1793),
    [sym_duet_dev_type_disc_device] = ACTIONS(1793),
    [sym_duet_dev_type_document_camera] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1793),
    [sym_duet_dev_type_hvac] = ACTIONS(1793),
    [sym_duet_dev_type_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_light] = ACTIONS(1793),
    [sym_duet_dev_type_monitor] = ACTIONS(1793),
    [sym_duet_dev_type_motor] = ACTIONS(1793),
    [sym_duet_dev_type_multi_window] = ACTIONS(1793),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1793),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1793),
    [sym_duet_dev_type_receiver] = ACTIONS(1793),
    [sym_duet_dev_type_security_system] = ACTIONS(1793),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1793),
    [sym_duet_dev_type_settop_box] = ACTIONS(1793),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1793),
    [sym_duet_dev_type_switcher] = ACTIONS(1793),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_tv] = ACTIONS(1793),
    [sym_duet_dev_type_utility] = ACTIONS(1793),
    [sym_duet_dev_type_vcr] = ACTIONS(1793),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_video_processor] = ACTIONS(1793),
    [sym_duet_dev_type_video_projector] = ACTIONS(1793),
    [sym_duet_dev_type_video_wall] = ACTIONS(1793),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1793),
    [sym_duet_dev_type_weather] = ACTIONS(1793),
    [sym_duet_dev_not_polled] = ACTIONS(1793),
    [sym_duet_dev_polled] = ACTIONS(1793),
    [sym_clkmgr_mode_network] = ACTIONS(1793),
    [sym_clkmgr_mode_standalone] = ACTIONS(1793),
    [sym_normal_standby] = ACTIONS(1793),
    [sym_normal_wake] = ACTIONS(1793),
    [sym_ip_multicast_ttl_option] = ACTIONS(1793),
    [sym_ip_ttl_subnet] = ACTIONS(1793),
    [sym_ip_ttl_site] = ACTIONS(1793),
    [sym_ip_ttl_region] = ACTIONS(1793),
    [sym_ip_ttl_continent] = ACTIONS(1793),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1793),
    [sym_ip_nodelay_on] = ACTIONS(1793),
    [sym_ip_nodelay_off] = ACTIONS(1793),
    [sym_amx_error] = ACTIONS(1793),
    [sym_amx_warning] = ACTIONS(1793),
    [sym_amx_info] = ACTIONS(1793),
    [sym_amx_debug] = ACTIONS(1793),
    [sym_netlinx_login_success] = ACTIONS(1793),
    [sym_netlinx_login_fail] = ACTIONS(1793),
    [sym_netlinx_logout] = ACTIONS(1793),
    [sym_null_str] = ACTIONS(1793),
    [sym_smtp_address] = ACTIONS(1793),
    [sym_smtp_port_number] = ACTIONS(1793),
    [sym_smtp_username] = ACTIONS(1793),
    [sym_smtp_password] = ACTIONS(1793),
    [sym_smtp_require_tls] = ACTIONS(1793),
    [sym_smtp_from] = ACTIONS(1793),
    [sym_smtp_tls_true] = ACTIONS(1793),
    [sym_smtp_tls_false] = ACTIONS(1793),
    [sym_listview_on_row_select_event] = ACTIONS(1793),
    [sym_data_structure_unknown] = ACTIONS(1793),
    [sym_data_structure_datafeed] = ACTIONS(1793),
    [sym_data_structure_datarecordset] = ACTIONS(1793),
    [sym_data_structure_datarecord] = ACTIONS(1793),
    [sym_data_structure_datafield] = ACTIONS(1793),
    [sym_data_max_content_field_count] = ACTIONS(1793),
    [sym_data_max_metadata_field_count] = ACTIONS(1793),
    [sym_data_max_id_length] = ACTIONS(1793),
    [sym_data_max_name_length] = ACTIONS(1793),
    [sym_data_max_description_length] = ACTIONS(1793),
    [sym_data_max_type_length] = ACTIONS(1793),
    [sym_data_max_format_length] = ACTIONS(1793),
    [sym_data_max_source_length] = ACTIONS(1793),
    [sym_data_max_value_length] = ACTIONS(1793),
    [sym_data_max_label_length] = ACTIONS(1793),
    [sym_data_max_url_length] = ACTIONS(1793),
    [sym_max_last_login_info_length] = ACTIONS(1793),
    [sym_data_type_unknown] = ACTIONS(1793),
    [sym_data_type_string] = ACTIONS(1793),
    [sym_data_type_datetime] = ACTIONS(1793),
    [sym_data_type_date] = ACTIONS(1793),
    [sym_data_type_time] = ACTIONS(1793),
    [sym_data_type_image] = ACTIONS(1793),
    [sym_data_format_url] = ACTIONS(1793),
    [sym_data_format_phone] = ACTIONS(1793),
    [sym_data_format_email] = ACTIONS(1793),
    [sym_data_format_iso8601] = ACTIONS(1793),
    [sym_auditlog_priv] = ACTIONS(1793),
    [sym_configuration_priv] = ACTIONS(1793),
    [sym_deviceconfig_priv] = ACTIONS(1793),
    [sym_ftp_priv] = ACTIONS(1793),
    [sym_http_priv] = ACTIONS(1793),
    [sym_networkconfig_priv] = ACTIONS(1793),
    [sym_programport_priv] = ACTIONS(1793),
    [sym_remoteui_priv] = ACTIONS(1793),
    [sym_securitycontrol_priv] = ACTIONS(1793),
    [sym_softwaremanagement_priv] = ACTIONS(1793),
    [sym_terminal_priv] = ACTIONS(1793),
    [sym_tpadmin_priv] = ACTIONS(1793),
    [sym_user1_priv] = ACTIONS(1793),
    [sym_user2_priv] = ACTIONS(1793),
    [sym_user3_priv] = ACTIONS(1793),
    [sym_user4_priv] = ACTIONS(1793),
    [sym_usermanagement_priv] = ACTIONS(1793),
    [sym_valid_account] = ACTIONS(1793),
    [sym_err_name_invalid] = ACTIONS(1793),
    [sym_err_password_invalid] = ACTIONS(1793),
    [sym_err_invalid_account] = ACTIONS(1793),
    [sym_err_no_authorization] = ACTIONS(1793),
    [sym_err_invalid_parameter] = ACTIONS(1793),
    [sym_err_user_locked_out] = ACTIONS(1793),
    [sym_err_user_account_expired] = ACTIONS(1793),
    [sym_err_general_error] = ACTIONS(1793),
    [sym_err_authentication_error] = ACTIONS(1793),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1793),
    [sym_tls_validate_certificate] = ACTIONS(1793),
    [sym_tdata] = ACTIONS(1793),
    [sym_tchannel] = ACTIONS(1793),
    [sym_tlevel] = ACTIONS(1793),
    [sym_tbutton] = ACTIONS(1793),
    [sym_ttimeline] = ACTIONS(1793),
    [sym_tcustom] = ACTIONS(1793),
    [sym_url_struct] = ACTIONS(1793),
    [sym_dns_struct] = ACTIONS(1793),
    [sym_ip_address_struct] = ACTIONS(1793),
    [sym_dev_info_struct] = ACTIONS(1793),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1793),
    [sym_data_feed] = ACTIONS(1793),
    [sym_data_field] = ACTIONS(1793),
    [sym_data_record] = ACTIONS(1793),
    [sym_wc_data_feed] = ACTIONS(1793),
    [sym_wc_data_field] = ACTIONS(1793),
    [sym_wc_data_record] = ACTIONS(1793),
    [sym_last_login_info] = ACTIONS(1793),
    [sym____reserved___] = ACTIONS(1793),
    [sym_date] = ACTIONS(1793),
    [sym_day] = ACTIONS(1793),
    [sym_time] = ACTIONS(1793),
    [sym_ldate] = ACTIONS(1793),
    [sym_push_device] = ACTIONS(1793),
    [sym_push_channel] = ACTIONS(1793),
    [sym_push_devchan] = ACTIONS(1793),
    [sym_release_device] = ACTIONS(1793),
    [sym_release_channel] = ACTIONS(1793),
    [sym_release_devchan] = ACTIONS(1793),
    [sym_master_slot] = ACTIONS(1793),
    [sym_get_pulse_time] = ACTIONS(1793),
    [sym_get_timer] = ACTIONS(1793),
    [sym_master_sn] = ACTIONS(1793),
    [sym_system_number] = ACTIONS(1793),
    [sym_button] = ACTIONS(1793),
    [sym_dv_channel] = ACTIONS(1793),
    [sym_data] = ACTIONS(1793),
    [sym_level] = ACTIONS(1793),
    [sym_timeline] = ACTIONS(1793),
    [sym_channel] = ACTIONS(1793),
    [sym_custom] = ACTIONS(1793),
    [sym_atoi] = ACTIONS(1793),
    [sym_atol] = ACTIONS(1793),
    [sym_atof] = ACTIONS(1793),
    [sym_itoa] = ACTIONS(1793),
    [sym_itohex] = ACTIONS(1793),
    [sym_hextoi] = ACTIONS(1793),
    [sym_ftoa] = ACTIONS(1793),
    [sym_find_string] = ACTIONS(1793),
    [sym_left_string] = ACTIONS(1793),
    [sym_right_string] = ACTIONS(1793),
    [sym_mid_string] = ACTIONS(1793),
    [sym_remove_string] = ACTIONS(1793),
    [sym_get_buffer_string] = ACTIONS(1793),
    [sym_get_buffer_char] = ACTIONS(1793),
    [sym_get_multi_buffer_string] = ACTIONS(1793),
    [sym_length_string] = ACTIONS(1793),
    [sym_max_length_string] = ACTIONS(1793),
    [sym_set_length_string] = ACTIONS(1793),
    [sym_compare_string] = ACTIONS(1793),
    [sym_upper_string] = ACTIONS(1793),
    [sym_lower_string] = ACTIONS(1793),
    [sym_get_last] = ACTIONS(1793),
    [sym_date_to_day] = ACTIONS(1793),
    [sym_date_to_month] = ACTIONS(1793),
    [sym_date_to_year] = ACTIONS(1793),
    [sym_time_to_hour] = ACTIONS(1793),
    [sym_time_to_minute] = ACTIONS(1793),
    [sym_time_to_second] = ACTIONS(1793),
    [sym_day_of_week] = ACTIONS(1793),
    [sym_set_pulse_time] = ACTIONS(1793),
    [sym_set_timer] = ACTIONS(1793),
    [sym_file_open] = ACTIONS(1793),
    [sym_file_close] = ACTIONS(1793),
    [sym_file_read] = ACTIONS(1793),
    [sym_file_read_line] = ACTIONS(1793),
    [sym_file_write] = ACTIONS(1793),
    [sym_file_write_line] = ACTIONS(1793),
    [sym_file_copy] = ACTIONS(1793),
    [sym_file_delete] = ACTIONS(1793),
    [sym_file_rename] = ACTIONS(1793),
    [sym_file_dir] = ACTIONS(1793),
    [sym_file_getdir] = ACTIONS(1793),
    [sym_file_setdir] = ACTIONS(1793),
    [sym_file_create_dir] = ACTIONS(1793),
    [sym_file_remove_dir] = ACTIONS(1793),
    [sym_file_seek] = ACTIONS(1793),
    [sym_length_array] = ACTIONS(1793),
    [sym_max_length_array] = ACTIONS(1793),
    [sym_set_length_array] = ACTIONS(1793),
    [sym_ip_client_open] = ACTIONS(1793),
    [sym_ip_client_close] = ACTIONS(1793),
    [sym_ip_server_open] = ACTIONS(1793),
    [sym_ip_server_close] = ACTIONS(1793),
    [sym_ip_bound_client_open] = ACTIONS(1793),
    [sym_ip_mc_server_open] = ACTIONS(1793),
    [sym_ip_set_option] = ACTIONS(1793),
    [sym_ssh_client_open] = ACTIONS(1793),
    [sym_ssh_client_close] = ACTIONS(1793),
    [sym_tls_client_open] = ACTIONS(1793),
    [sym_tls_client_close] = ACTIONS(1793),
    [sym_do_push] = ACTIONS(1793),
    [sym_do_push_timed] = ACTIONS(1793),
    [sym_do_release] = ACTIONS(1793),
    [sym_device_id] = ACTIONS(1793),
    [sym_device_info] = ACTIONS(1793),
    [sym_device_id_string] = ACTIONS(1793),
    [sym_device_standby] = ACTIONS(1793),
    [sym_device_wake] = ACTIONS(1793),
    [sym_timeline_create] = ACTIONS(1793),
    [sym_timeline_kill] = ACTIONS(1793),
    [sym_timeline_reload] = ACTIONS(1793),
    [sym_timeline_pause] = ACTIONS(1793),
    [sym_timeline_restart] = ACTIONS(1793),
    [sym_timeline_set] = ACTIONS(1793),
    [sym_timeline_get] = ACTIONS(1793),
    [sym_timeline_active] = ACTIONS(1793),
    [sym_timeline_dynamic_id] = ACTIONS(1793),
    [sym_variable_to_string] = ACTIONS(1793),
    [sym_string_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_string] = ACTIONS(1793),
    [sym_type_cast] = ACTIONS(1793),
    [sym_variable_to_xml] = ACTIONS(1793),
    [sym_xml_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_xml] = ACTIONS(1793),
    [sym_redirect_string] = ACTIONS(1793),
    [sym_random_number] = ACTIONS(1793),
    [sym_get_unique_id] = ACTIONS(1793),
    [sym_get_system_number] = ACTIONS(1793),
    [sym_set_system_number] = ACTIONS(1793),
    [sym_get_serial_number] = ACTIONS(1793),
    [sym_get_master_build] = ACTIONS(1793),
    [sym_set_validation_code] = ACTIONS(1793),
    [sym_reboot] = ACTIONS(1793),
    [sym_load_duet_module] = ACTIONS(1793),
    [sym_unload_duet_module] = ACTIONS(1793),
    [sym_duet_mem_size_set] = ACTIONS(1793),
    [sym_duet_mem_size_get] = ACTIONS(1793),
    [sym_dynamic_application_device] = ACTIONS(1793),
    [sym_dynamic_polled_port] = ACTIONS(1793),
    [sym_static_port_binding] = ACTIONS(1793),
    [sym_static_ip_binding] = ACTIONS(1793),
    [sym_get_dns_list] = ACTIONS(1793),
    [sym_set_dns_list] = ACTIONS(1793),
    [sym_get_ip_address] = ACTIONS(1793),
    [sym_set_ip_address] = ACTIONS(1793),
    [sym_get_url_list] = ACTIONS(1793),
    [sym_add_url_entry] = ACTIONS(1793),
    [sym_delete_url_entry] = ACTIONS(1793),
    [sym_abs_value] = ACTIONS(1793),
    [sym_max_value] = ACTIONS(1793),
    [sym_min_value] = ACTIONS(1793),
    [sym_exp_value] = ACTIONS(1793),
    [sym_log_value] = ACTIONS(1793),
    [sym_log10_value] = ACTIONS(1793),
    [sym_power_value] = ACTIONS(1793),
    [sym_sqrt_value] = ACTIONS(1793),
    [sym_raw_be] = ACTIONS(1793),
    [sym_raw_le] = ACTIONS(1793),
    [sym_internal_queue_size_set] = ACTIONS(1793),
    [sym_internal_queue_size_get] = ACTIONS(1793),
    [sym_internal_threshold_set] = ACTIONS(1793),
    [sym_internal_threshold_get] = ACTIONS(1793),
    [sym_rebuilt_event] = ACTIONS(1793),
    [sym_do_custom_event] = ACTIONS(1793),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1793),
    [sym_clkmgr_set_clk_source] = ACTIONS(1793),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1793),
    [sym_clkmgr_get_timezone] = ACTIONS(1793),
    [sym_clkmgr_set_timezone] = ACTIONS(1793),
    [sym_clkmgr_get_resync_period] = ACTIONS(1793),
    [sym_clkmgr_set_resync_period] = ACTIONS(1793),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_timeservers] = ACTIONS(1793),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_set_log_level] = ACTIONS(1793),
    [sym_get_log_level] = ACTIONS(1793),
    [sym_amx_log] = ACTIONS(1793),
    [sym_smtp_server_config_set] = ACTIONS(1793),
    [sym_smtp_server_config_get] = ACTIONS(1793),
    [sym_smtp_send] = ACTIONS(1793),
    [sym_validate_netlinx_account] = ACTIONS(1793),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1793),
    [sym_authenticate_certificate] = ACTIONS(1793),
    [sym_audit_netlinx_session_event] = ACTIONS(1793),
    [sym_audit_netlinx_generic_event] = ACTIONS(1793),
    [sym_get_available_flash_disk_space] = ACTIONS(1793),
    [sym_get_max_flash_disk_space] = ACTIONS(1793),
    [sym_data_create_feed] = ACTIONS(1793),
    [sym_data_delete_feed] = ACTIONS(1793),
    [sym_data_publish_feed] = ACTIONS(1793),
    [sym_data_get_published_feed] = ACTIONS(1793),
    [sym_data_add_record] = ACTIONS(1793),
    [sym_data_get_event_record] = ACTIONS(1793),
    [sym__wc_data_create_feed] = ACTIONS(1793),
    [sym__wc_data_add_record] = ACTIONS(1793),
    [sym__wc_data_get_event_record] = ACTIONS(1793),
    [sym_format] = ACTIONS(1793),
    [sym_astro_clock] = ACTIONS(1793),
    [sym_set_outdoor_temperature] = ACTIONS(1793),
    [sym_set_virtual_level_count] = ACTIONS(1793),
    [sym_set_virtual_channel_count] = ACTIONS(1793),
    [sym_set_virtual_port_count] = ACTIONS(1793),
    [sym__wc_encode_int] = ACTIONS(1793),
    [sym__wc_decode_int] = ACTIONS(1793),
    [sym__wc_int] = ACTIONS(1793),
    [sym__wc_to_ch_int] = ACTIONS(1793),
    [sym__ch_to_wc_int] = ACTIONS(1793),
    [sym__wc_find_string_int] = ACTIONS(1793),
    [sym__wc_left_string_int] = ACTIONS(1793),
    [sym__wc_lower_string_int] = ACTIONS(1793),
    [sym__wc_mid_string_int] = ACTIONS(1793),
    [sym__wc_remove_string_int] = ACTIONS(1793),
    [sym__wc_right_string_int] = ACTIONS(1793),
    [sym__wc_upper_string_int] = ACTIONS(1793),
    [sym__wc_compare_string_int] = ACTIONS(1793),
    [sym__wc_get_buffer_char_int] = ACTIONS(1793),
    [sym__wc_get_buffer_string_int] = ACTIONS(1793),
    [sym__wc_concat_string_int] = ACTIONS(1793),
    [sym__wc_file_open_int] = ACTIONS(1793),
    [sym__wc_file_close_int] = ACTIONS(1793),
    [sym__wc_file_read_int] = ACTIONS(1793),
    [sym__wc_file_read_line_int] = ACTIONS(1793),
    [sym__wc_file_write_int] = ACTIONS(1793),
    [sym__wc_file_write_line_int] = ACTIONS(1793),
    [aux_sym_preproc_include_token1] = ACTIONS(1793),
    [aux_sym_preproc_define_token1] = ACTIONS(1795),
    [aux_sym_preproc_warn_token1] = ACTIONS(1795),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1795),
    [anon_sym_LBRACK] = ACTIONS(1795),
    [anon_sym_LBRACE] = ACTIONS(1795),
    [anon_sym_LPAREN] = ACTIONS(1793),
    [anon_sym_BANG] = ACTIONS(1795),
    [anon_sym_TILDE] = ACTIONS(1795),
    [anon_sym_DASH] = ACTIONS(1793),
    [anon_sym_PLUS] = ACTIONS(1793),
    [anon_sym_DASH_DASH] = ACTIONS(1795),
    [anon_sym_PLUS_PLUS] = ACTIONS(1795),
    [anon_sym_DQUOTE] = ACTIONS(1795),
    [anon_sym_SQUOTE] = ACTIONS(1795),
    [sym_number_literal] = ACTIONS(1795),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(359)] = {
    [ts_builtin_sym_end] = ACTIONS(1795),
    [sym_identifier] = ACTIONS(1793),
    [sym_program_name_keyword] = ACTIONS(1793),
    [sym_module_name_keyword] = ACTIONS(1793),
    [sym_define_device_keyword] = ACTIONS(1793),
    [sym_define_combine_keyword] = ACTIONS(1793),
    [sym_define_constant_keyword] = ACTIONS(1793),
    [sym_define_type_keyword] = ACTIONS(1793),
    [sym_define_variable_keyword] = ACTIONS(1793),
    [sym_define_system_variable_keyword] = ACTIONS(1793),
    [sym_define_connect_level_keyword] = ACTIONS(1793),
    [sym_define_latching_keyword] = ACTIONS(1793),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1793),
    [sym_define_toggling_keyword] = ACTIONS(1793),
    [sym_define_call_keyword] = ACTIONS(1793),
    [sym_define_function_keyword] = ACTIONS(1793),
    [sym_define_library_function_keyword] = ACTIONS(1793),
    [sym_define_start_keyword] = ACTIONS(1793),
    [sym_define_event_keyword] = ACTIONS(1793),
    [sym_define_program_keyword] = ACTIONS(1793),
    [sym_define_module_keyword] = ACTIONS(1793),
    [sym_char_keyword] = ACTIONS(1793),
    [sym_widechar_keyword] = ACTIONS(1793),
    [sym_integer_keyword] = ACTIONS(1793),
    [sym_sinteger_keyword] = ACTIONS(1793),
    [sym_long_keyword] = ACTIONS(1793),
    [sym_slong_keyword] = ACTIONS(1793),
    [sym_float_keyword] = ACTIONS(1793),
    [sym_double_keyword] = ACTIONS(1793),
    [sym_variant_keyword] = ACTIONS(1793),
    [sym_variantarray_keyword] = ACTIONS(1793),
    [sym_local_var_keyword] = ACTIONS(1793),
    [sym_stack_var_keyword] = ACTIONS(1793),
    [sym_constant_keyword] = ACTIONS(1793),
    [sym_volatile_keyword] = ACTIONS(1793),
    [sym_non_volatile_keyword] = ACTIONS(1793),
    [sym_persistent_keyword] = ACTIONS(1793),
    [sym_dev_keyword] = ACTIONS(1793),
    [sym_devlev_keyword] = ACTIONS(1793),
    [sym_devchan_keyword] = ACTIONS(1793),
    [sym_if_keyword] = ACTIONS(1793),
    [sym_else_keyword] = ACTIONS(1793),
    [sym_switch_keyword] = ACTIONS(1793),
    [sym_case_keyword] = ACTIONS(1793),
    [sym_default_keyword] = ACTIONS(1793),
    [sym_while_keyword] = ACTIONS(1793),
    [sym_for_keyword] = ACTIONS(1793),
    [sym_break_keyword] = ACTIONS(1793),
    [sym_continue_keyword] = ACTIONS(1793),
    [sym_return_keyword] = ACTIONS(1793),
    [sym_select_keyword] = ACTIONS(1793),
    [sym_active_keyword] = ACTIONS(1793),
    [sym_struct_keyword] = ACTIONS(1793),
    [sym_structure_keyword] = ACTIONS(1793),
    [sym_button_event_keyword] = ACTIONS(1793),
    [sym_channel_event_keyword] = ACTIONS(1793),
    [sym_level_event_keyword] = ACTIONS(1793),
    [sym_data_event_keyword] = ACTIONS(1793),
    [sym_timeline_event_keyword] = ACTIONS(1793),
    [sym_custom_event_keyword] = ACTIONS(1793),
    [sym_push_keyword] = ACTIONS(1793),
    [sym_release_keyword] = ACTIONS(1793),
    [aux_sym_on_keyword_token1] = ACTIONS(1793),
    [aux_sym_off_keyword_token1] = ACTIONS(1793),
    [sym_send_level_keyword] = ACTIONS(1793),
    [sym_send_string_keyword] = ACTIONS(1793),
    [sym_send_command_keyword] = ACTIONS(1793),
    [sym_clear_buffer_keyword] = ACTIONS(1793),
    [sym_create_buffer_keyword] = ACTIONS(1793),
    [sym_create_multi_buffer_keyword] = ACTIONS(1793),
    [sym_call_keyword] = ACTIONS(1793),
    [sym_system_call_keyword] = ACTIONS(1793),
    [sym_devchan_to_keyword] = ACTIONS(1793),
    [sym_devchan_min_to_keyword] = ACTIONS(1793),
    [sym_devchan_total_off_keyword] = ACTIONS(1793),
    [sym_devchan_pulse_keyword] = ACTIONS(1793),
    [sym_wait_keyword] = ACTIONS(1793),
    [sym_cancel_wait_keyword] = ACTIONS(1793),
    [sym_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1793),
    [sym_not] = ACTIONS(1793),
    [sym_bnot] = ACTIONS(1793),
    [sym___file__] = ACTIONS(1793),
    [sym___date__] = ACTIONS(1793),
    [sym___ldate__] = ACTIONS(1793),
    [sym___time__] = ACTIONS(1793),
    [sym___line__] = ACTIONS(1793),
    [sym___name__] = ACTIONS(1793),
    [sym___netlinx__] = ACTIONS(1793),
    [sym_netlinx_axi_version] = ACTIONS(1793),
    [sym_url_flg_tcp] = ACTIONS(1793),
    [sym_url_flg_accinfopresent] = ACTIONS(1793),
    [sym_url_flg_temp] = ACTIONS(1793),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1793),
    [sym_url_flg_stat_mask] = ACTIONS(1793),
    [sym_url_flg_stat_lookup] = ACTIONS(1793),
    [sym_url_flg_stat_connecting] = ACTIONS(1793),
    [sym_url_flg_stat_waiting] = ACTIONS(1793),
    [sym_url_flg_stat_connected] = ACTIONS(1793),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1793),
    [sym_first_virtual_device] = ACTIONS(1793),
    [sym_first_local_port] = ACTIONS(1793),
    [sym_dynamic_virtual_device] = ACTIONS(1793),
    [sym_timeline_once] = ACTIONS(1793),
    [sym_timeline_repeat] = ACTIONS(1793),
    [sym_timeline_absolute] = ACTIONS(1793),
    [sym_timeline_relative] = ACTIONS(1793),
    [sym_do_push_timed_infinite] = ACTIONS(1793),
    [sym_source_type_no_address] = ACTIONS(1793),
    [sym_source_type_neuron_id] = ACTIONS(1793),
    [sym_source_type_ip_address] = ACTIONS(1793),
    [sym_source_type_axlink] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1793),
    [sym_source_type_ip_socket_address] = ACTIONS(1793),
    [sym_source_type_rs232] = ACTIONS(1793),
    [sym_source_type_internal] = ACTIONS(1793),
    [sym_source_type_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1793),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1793),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1793),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1793),
    [sym_true] = ACTIONS(1793),
    [sym_false] = ACTIONS(1793),
    [sym_file_read_only] = ACTIONS(1793),
    [sym_file_rw_new] = ACTIONS(1793),
    [sym_file_rw_append] = ACTIONS(1793),
    [sym_ip_tcp] = ACTIONS(1793),
    [sym_ip_udp] = ACTIONS(1793),
    [sym_ip_udp_2way] = ACTIONS(1793),
    [sym_xml_encode_types] = ACTIONS(1793),
    [sym_xml_encode_char_as_list] = ACTIONS(1793),
    [sym_xml_encode_le] = ACTIONS(1793),
    [sym_xml_decode_types] = ACTIONS(1793),
    [sym_xml_decode_no_preserve] = ACTIONS(1793),
    [sym_duet_dev_type_display_device] = ACTIONS(1793),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_io_device] = ACTIONS(1793),
    [sym_duet_dev_type_relay_device] = ACTIONS(1793),
    [sym_duet_dev_type_ups] = ACTIONS(1793),
    [sym_duet_dev_type_amplifier] = ACTIONS(1793),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1793),
    [sym_duet_dev_type_camera] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1793),
    [sym_duet_dev_type_dss] = ACTIONS(1793),
    [sym_duet_dev_type_dvr] = ACTIONS(1793),
    [sym_duet_dev_type_disc_device] = ACTIONS(1793),
    [sym_duet_dev_type_document_camera] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1793),
    [sym_duet_dev_type_hvac] = ACTIONS(1793),
    [sym_duet_dev_type_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_light] = ACTIONS(1793),
    [sym_duet_dev_type_monitor] = ACTIONS(1793),
    [sym_duet_dev_type_motor] = ACTIONS(1793),
    [sym_duet_dev_type_multi_window] = ACTIONS(1793),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1793),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1793),
    [sym_duet_dev_type_receiver] = ACTIONS(1793),
    [sym_duet_dev_type_security_system] = ACTIONS(1793),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1793),
    [sym_duet_dev_type_settop_box] = ACTIONS(1793),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1793),
    [sym_duet_dev_type_switcher] = ACTIONS(1793),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_tv] = ACTIONS(1793),
    [sym_duet_dev_type_utility] = ACTIONS(1793),
    [sym_duet_dev_type_vcr] = ACTIONS(1793),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_video_processor] = ACTIONS(1793),
    [sym_duet_dev_type_video_projector] = ACTIONS(1793),
    [sym_duet_dev_type_video_wall] = ACTIONS(1793),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1793),
    [sym_duet_dev_type_weather] = ACTIONS(1793),
    [sym_duet_dev_not_polled] = ACTIONS(1793),
    [sym_duet_dev_polled] = ACTIONS(1793),
    [sym_clkmgr_mode_network] = ACTIONS(1793),
    [sym_clkmgr_mode_standalone] = ACTIONS(1793),
    [sym_normal_standby] = ACTIONS(1793),
    [sym_normal_wake] = ACTIONS(1793),
    [sym_ip_multicast_ttl_option] = ACTIONS(1793),
    [sym_ip_ttl_subnet] = ACTIONS(1793),
    [sym_ip_ttl_site] = ACTIONS(1793),
    [sym_ip_ttl_region] = ACTIONS(1793),
    [sym_ip_ttl_continent] = ACTIONS(1793),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1793),
    [sym_ip_nodelay_on] = ACTIONS(1793),
    [sym_ip_nodelay_off] = ACTIONS(1793),
    [sym_amx_error] = ACTIONS(1793),
    [sym_amx_warning] = ACTIONS(1793),
    [sym_amx_info] = ACTIONS(1793),
    [sym_amx_debug] = ACTIONS(1793),
    [sym_netlinx_login_success] = ACTIONS(1793),
    [sym_netlinx_login_fail] = ACTIONS(1793),
    [sym_netlinx_logout] = ACTIONS(1793),
    [sym_null_str] = ACTIONS(1793),
    [sym_smtp_address] = ACTIONS(1793),
    [sym_smtp_port_number] = ACTIONS(1793),
    [sym_smtp_username] = ACTIONS(1793),
    [sym_smtp_password] = ACTIONS(1793),
    [sym_smtp_require_tls] = ACTIONS(1793),
    [sym_smtp_from] = ACTIONS(1793),
    [sym_smtp_tls_true] = ACTIONS(1793),
    [sym_smtp_tls_false] = ACTIONS(1793),
    [sym_listview_on_row_select_event] = ACTIONS(1793),
    [sym_data_structure_unknown] = ACTIONS(1793),
    [sym_data_structure_datafeed] = ACTIONS(1793),
    [sym_data_structure_datarecordset] = ACTIONS(1793),
    [sym_data_structure_datarecord] = ACTIONS(1793),
    [sym_data_structure_datafield] = ACTIONS(1793),
    [sym_data_max_content_field_count] = ACTIONS(1793),
    [sym_data_max_metadata_field_count] = ACTIONS(1793),
    [sym_data_max_id_length] = ACTIONS(1793),
    [sym_data_max_name_length] = ACTIONS(1793),
    [sym_data_max_description_length] = ACTIONS(1793),
    [sym_data_max_type_length] = ACTIONS(1793),
    [sym_data_max_format_length] = ACTIONS(1793),
    [sym_data_max_source_length] = ACTIONS(1793),
    [sym_data_max_value_length] = ACTIONS(1793),
    [sym_data_max_label_length] = ACTIONS(1793),
    [sym_data_max_url_length] = ACTIONS(1793),
    [sym_max_last_login_info_length] = ACTIONS(1793),
    [sym_data_type_unknown] = ACTIONS(1793),
    [sym_data_type_string] = ACTIONS(1793),
    [sym_data_type_datetime] = ACTIONS(1793),
    [sym_data_type_date] = ACTIONS(1793),
    [sym_data_type_time] = ACTIONS(1793),
    [sym_data_type_image] = ACTIONS(1793),
    [sym_data_format_url] = ACTIONS(1793),
    [sym_data_format_phone] = ACTIONS(1793),
    [sym_data_format_email] = ACTIONS(1793),
    [sym_data_format_iso8601] = ACTIONS(1793),
    [sym_auditlog_priv] = ACTIONS(1793),
    [sym_configuration_priv] = ACTIONS(1793),
    [sym_deviceconfig_priv] = ACTIONS(1793),
    [sym_ftp_priv] = ACTIONS(1793),
    [sym_http_priv] = ACTIONS(1793),
    [sym_networkconfig_priv] = ACTIONS(1793),
    [sym_programport_priv] = ACTIONS(1793),
    [sym_remoteui_priv] = ACTIONS(1793),
    [sym_securitycontrol_priv] = ACTIONS(1793),
    [sym_softwaremanagement_priv] = ACTIONS(1793),
    [sym_terminal_priv] = ACTIONS(1793),
    [sym_tpadmin_priv] = ACTIONS(1793),
    [sym_user1_priv] = ACTIONS(1793),
    [sym_user2_priv] = ACTIONS(1793),
    [sym_user3_priv] = ACTIONS(1793),
    [sym_user4_priv] = ACTIONS(1793),
    [sym_usermanagement_priv] = ACTIONS(1793),
    [sym_valid_account] = ACTIONS(1793),
    [sym_err_name_invalid] = ACTIONS(1793),
    [sym_err_password_invalid] = ACTIONS(1793),
    [sym_err_invalid_account] = ACTIONS(1793),
    [sym_err_no_authorization] = ACTIONS(1793),
    [sym_err_invalid_parameter] = ACTIONS(1793),
    [sym_err_user_locked_out] = ACTIONS(1793),
    [sym_err_user_account_expired] = ACTIONS(1793),
    [sym_err_general_error] = ACTIONS(1793),
    [sym_err_authentication_error] = ACTIONS(1793),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1793),
    [sym_tls_validate_certificate] = ACTIONS(1793),
    [sym_tdata] = ACTIONS(1793),
    [sym_tchannel] = ACTIONS(1793),
    [sym_tlevel] = ACTIONS(1793),
    [sym_tbutton] = ACTIONS(1793),
    [sym_ttimeline] = ACTIONS(1793),
    [sym_tcustom] = ACTIONS(1793),
    [sym_url_struct] = ACTIONS(1793),
    [sym_dns_struct] = ACTIONS(1793),
    [sym_ip_address_struct] = ACTIONS(1793),
    [sym_dev_info_struct] = ACTIONS(1793),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1793),
    [sym_data_feed] = ACTIONS(1793),
    [sym_data_field] = ACTIONS(1793),
    [sym_data_record] = ACTIONS(1793),
    [sym_wc_data_feed] = ACTIONS(1793),
    [sym_wc_data_field] = ACTIONS(1793),
    [sym_wc_data_record] = ACTIONS(1793),
    [sym_last_login_info] = ACTIONS(1793),
    [sym____reserved___] = ACTIONS(1793),
    [sym_date] = ACTIONS(1793),
    [sym_day] = ACTIONS(1793),
    [sym_time] = ACTIONS(1793),
    [sym_ldate] = ACTIONS(1793),
    [sym_push_device] = ACTIONS(1793),
    [sym_push_channel] = ACTIONS(1793),
    [sym_push_devchan] = ACTIONS(1793),
    [sym_release_device] = ACTIONS(1793),
    [sym_release_channel] = ACTIONS(1793),
    [sym_release_devchan] = ACTIONS(1793),
    [sym_master_slot] = ACTIONS(1793),
    [sym_get_pulse_time] = ACTIONS(1793),
    [sym_get_timer] = ACTIONS(1793),
    [sym_master_sn] = ACTIONS(1793),
    [sym_system_number] = ACTIONS(1793),
    [sym_button] = ACTIONS(1793),
    [sym_dv_channel] = ACTIONS(1793),
    [sym_data] = ACTIONS(1793),
    [sym_level] = ACTIONS(1793),
    [sym_timeline] = ACTIONS(1793),
    [sym_channel] = ACTIONS(1793),
    [sym_custom] = ACTIONS(1793),
    [sym_atoi] = ACTIONS(1793),
    [sym_atol] = ACTIONS(1793),
    [sym_atof] = ACTIONS(1793),
    [sym_itoa] = ACTIONS(1793),
    [sym_itohex] = ACTIONS(1793),
    [sym_hextoi] = ACTIONS(1793),
    [sym_ftoa] = ACTIONS(1793),
    [sym_find_string] = ACTIONS(1793),
    [sym_left_string] = ACTIONS(1793),
    [sym_right_string] = ACTIONS(1793),
    [sym_mid_string] = ACTIONS(1793),
    [sym_remove_string] = ACTIONS(1793),
    [sym_get_buffer_string] = ACTIONS(1793),
    [sym_get_buffer_char] = ACTIONS(1793),
    [sym_get_multi_buffer_string] = ACTIONS(1793),
    [sym_length_string] = ACTIONS(1793),
    [sym_max_length_string] = ACTIONS(1793),
    [sym_set_length_string] = ACTIONS(1793),
    [sym_compare_string] = ACTIONS(1793),
    [sym_upper_string] = ACTIONS(1793),
    [sym_lower_string] = ACTIONS(1793),
    [sym_get_last] = ACTIONS(1793),
    [sym_date_to_day] = ACTIONS(1793),
    [sym_date_to_month] = ACTIONS(1793),
    [sym_date_to_year] = ACTIONS(1793),
    [sym_time_to_hour] = ACTIONS(1793),
    [sym_time_to_minute] = ACTIONS(1793),
    [sym_time_to_second] = ACTIONS(1793),
    [sym_day_of_week] = ACTIONS(1793),
    [sym_set_pulse_time] = ACTIONS(1793),
    [sym_set_timer] = ACTIONS(1793),
    [sym_file_open] = ACTIONS(1793),
    [sym_file_close] = ACTIONS(1793),
    [sym_file_read] = ACTIONS(1793),
    [sym_file_read_line] = ACTIONS(1793),
    [sym_file_write] = ACTIONS(1793),
    [sym_file_write_line] = ACTIONS(1793),
    [sym_file_copy] = ACTIONS(1793),
    [sym_file_delete] = ACTIONS(1793),
    [sym_file_rename] = ACTIONS(1793),
    [sym_file_dir] = ACTIONS(1793),
    [sym_file_getdir] = ACTIONS(1793),
    [sym_file_setdir] = ACTIONS(1793),
    [sym_file_create_dir] = ACTIONS(1793),
    [sym_file_remove_dir] = ACTIONS(1793),
    [sym_file_seek] = ACTIONS(1793),
    [sym_length_array] = ACTIONS(1793),
    [sym_max_length_array] = ACTIONS(1793),
    [sym_set_length_array] = ACTIONS(1793),
    [sym_ip_client_open] = ACTIONS(1793),
    [sym_ip_client_close] = ACTIONS(1793),
    [sym_ip_server_open] = ACTIONS(1793),
    [sym_ip_server_close] = ACTIONS(1793),
    [sym_ip_bound_client_open] = ACTIONS(1793),
    [sym_ip_mc_server_open] = ACTIONS(1793),
    [sym_ip_set_option] = ACTIONS(1793),
    [sym_ssh_client_open] = ACTIONS(1793),
    [sym_ssh_client_close] = ACTIONS(1793),
    [sym_tls_client_open] = ACTIONS(1793),
    [sym_tls_client_close] = ACTIONS(1793),
    [sym_do_push] = ACTIONS(1793),
    [sym_do_push_timed] = ACTIONS(1793),
    [sym_do_release] = ACTIONS(1793),
    [sym_device_id] = ACTIONS(1793),
    [sym_device_info] = ACTIONS(1793),
    [sym_device_id_string] = ACTIONS(1793),
    [sym_device_standby] = ACTIONS(1793),
    [sym_device_wake] = ACTIONS(1793),
    [sym_timeline_create] = ACTIONS(1793),
    [sym_timeline_kill] = ACTIONS(1793),
    [sym_timeline_reload] = ACTIONS(1793),
    [sym_timeline_pause] = ACTIONS(1793),
    [sym_timeline_restart] = ACTIONS(1793),
    [sym_timeline_set] = ACTIONS(1793),
    [sym_timeline_get] = ACTIONS(1793),
    [sym_timeline_active] = ACTIONS(1793),
    [sym_timeline_dynamic_id] = ACTIONS(1793),
    [sym_variable_to_string] = ACTIONS(1793),
    [sym_string_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_string] = ACTIONS(1793),
    [sym_type_cast] = ACTIONS(1793),
    [sym_variable_to_xml] = ACTIONS(1793),
    [sym_xml_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_xml] = ACTIONS(1793),
    [sym_redirect_string] = ACTIONS(1793),
    [sym_random_number] = ACTIONS(1793),
    [sym_get_unique_id] = ACTIONS(1793),
    [sym_get_system_number] = ACTIONS(1793),
    [sym_set_system_number] = ACTIONS(1793),
    [sym_get_serial_number] = ACTIONS(1793),
    [sym_get_master_build] = ACTIONS(1793),
    [sym_set_validation_code] = ACTIONS(1793),
    [sym_reboot] = ACTIONS(1793),
    [sym_load_duet_module] = ACTIONS(1793),
    [sym_unload_duet_module] = ACTIONS(1793),
    [sym_duet_mem_size_set] = ACTIONS(1793),
    [sym_duet_mem_size_get] = ACTIONS(1793),
    [sym_dynamic_application_device] = ACTIONS(1793),
    [sym_dynamic_polled_port] = ACTIONS(1793),
    [sym_static_port_binding] = ACTIONS(1793),
    [sym_static_ip_binding] = ACTIONS(1793),
    [sym_get_dns_list] = ACTIONS(1793),
    [sym_set_dns_list] = ACTIONS(1793),
    [sym_get_ip_address] = ACTIONS(1793),
    [sym_set_ip_address] = ACTIONS(1793),
    [sym_get_url_list] = ACTIONS(1793),
    [sym_add_url_entry] = ACTIONS(1793),
    [sym_delete_url_entry] = ACTIONS(1793),
    [sym_abs_value] = ACTIONS(1793),
    [sym_max_value] = ACTIONS(1793),
    [sym_min_value] = ACTIONS(1793),
    [sym_exp_value] = ACTIONS(1793),
    [sym_log_value] = ACTIONS(1793),
    [sym_log10_value] = ACTIONS(1793),
    [sym_power_value] = ACTIONS(1793),
    [sym_sqrt_value] = ACTIONS(1793),
    [sym_raw_be] = ACTIONS(1793),
    [sym_raw_le] = ACTIONS(1793),
    [sym_internal_queue_size_set] = ACTIONS(1793),
    [sym_internal_queue_size_get] = ACTIONS(1793),
    [sym_internal_threshold_set] = ACTIONS(1793),
    [sym_internal_threshold_get] = ACTIONS(1793),
    [sym_rebuilt_event] = ACTIONS(1793),
    [sym_do_custom_event] = ACTIONS(1793),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1793),
    [sym_clkmgr_set_clk_source] = ACTIONS(1793),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1793),
    [sym_clkmgr_get_timezone] = ACTIONS(1793),
    [sym_clkmgr_set_timezone] = ACTIONS(1793),
    [sym_clkmgr_get_resync_period] = ACTIONS(1793),
    [sym_clkmgr_set_resync_period] = ACTIONS(1793),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_timeservers] = ACTIONS(1793),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_set_log_level] = ACTIONS(1793),
    [sym_get_log_level] = ACTIONS(1793),
    [sym_amx_log] = ACTIONS(1793),
    [sym_smtp_server_config_set] = ACTIONS(1793),
    [sym_smtp_server_config_get] = ACTIONS(1793),
    [sym_smtp_send] = ACTIONS(1793),
    [sym_validate_netlinx_account] = ACTIONS(1793),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1793),
    [sym_authenticate_certificate] = ACTIONS(1793),
    [sym_audit_netlinx_session_event] = ACTIONS(1793),
    [sym_audit_netlinx_generic_event] = ACTIONS(1793),
    [sym_get_available_flash_disk_space] = ACTIONS(1793),
    [sym_get_max_flash_disk_space] = ACTIONS(1793),
    [sym_data_create_feed] = ACTIONS(1793),
    [sym_data_delete_feed] = ACTIONS(1793),
    [sym_data_publish_feed] = ACTIONS(1793),
    [sym_data_get_published_feed] = ACTIONS(1793),
    [sym_data_add_record] = ACTIONS(1793),
    [sym_data_get_event_record] = ACTIONS(1793),
    [sym__wc_data_create_feed] = ACTIONS(1793),
    [sym__wc_data_add_record] = ACTIONS(1793),
    [sym__wc_data_get_event_record] = ACTIONS(1793),
    [sym_format] = ACTIONS(1793),
    [sym_astro_clock] = ACTIONS(1793),
    [sym_set_outdoor_temperature] = ACTIONS(1793),
    [sym_set_virtual_level_count] = ACTIONS(1793),
    [sym_set_virtual_channel_count] = ACTIONS(1793),
    [sym_set_virtual_port_count] = ACTIONS(1793),
    [sym__wc_encode_int] = ACTIONS(1793),
    [sym__wc_decode_int] = ACTIONS(1793),
    [sym__wc_int] = ACTIONS(1793),
    [sym__wc_to_ch_int] = ACTIONS(1793),
    [sym__ch_to_wc_int] = ACTIONS(1793),
    [sym__wc_find_string_int] = ACTIONS(1793),
    [sym__wc_left_string_int] = ACTIONS(1793),
    [sym__wc_lower_string_int] = ACTIONS(1793),
    [sym__wc_mid_string_int] = ACTIONS(1793),
    [sym__wc_remove_string_int] = ACTIONS(1793),
    [sym__wc_right_string_int] = ACTIONS(1793),
    [sym__wc_upper_string_int] = ACTIONS(1793),
    [sym__wc_compare_string_int] = ACTIONS(1793),
    [sym__wc_get_buffer_char_int] = ACTIONS(1793),
    [sym__wc_get_buffer_string_int] = ACTIONS(1793),
    [sym__wc_concat_string_int] = ACTIONS(1793),
    [sym__wc_file_open_int] = ACTIONS(1793),
    [sym__wc_file_close_int] = ACTIONS(1793),
    [sym__wc_file_read_int] = ACTIONS(1793),
    [sym__wc_file_read_line_int] = ACTIONS(1793),
    [sym__wc_file_write_int] = ACTIONS(1793),
    [sym__wc_file_write_line_int] = ACTIONS(1793),
    [aux_sym_preproc_include_token1] = ACTIONS(1793),
    [aux_sym_preproc_define_token1] = ACTIONS(1795),
    [aux_sym_preproc_warn_token1] = ACTIONS(1795),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1795),
    [anon_sym_LBRACK] = ACTIONS(1795),
    [anon_sym_LBRACE] = ACTIONS(1795),
    [anon_sym_LPAREN] = ACTIONS(1793),
    [anon_sym_BANG] = ACTIONS(1795),
    [anon_sym_TILDE] = ACTIONS(1795),
    [anon_sym_DASH] = ACTIONS(1793),
    [anon_sym_PLUS] = ACTIONS(1793),
    [anon_sym_DASH_DASH] = ACTIONS(1795),
    [anon_sym_PLUS_PLUS] = ACTIONS(1795),
    [anon_sym_DQUOTE] = ACTIONS(1795),
    [anon_sym_SQUOTE] = ACTIONS(1795),
    [sym_number_literal] = ACTIONS(1795),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(360)] = {
    [ts_builtin_sym_end] = ACTIONS(1795),
    [sym_identifier] = ACTIONS(1793),
    [sym_program_name_keyword] = ACTIONS(1793),
    [sym_module_name_keyword] = ACTIONS(1793),
    [sym_define_device_keyword] = ACTIONS(1793),
    [sym_define_combine_keyword] = ACTIONS(1793),
    [sym_define_constant_keyword] = ACTIONS(1793),
    [sym_define_type_keyword] = ACTIONS(1793),
    [sym_define_variable_keyword] = ACTIONS(1793),
    [sym_define_system_variable_keyword] = ACTIONS(1793),
    [sym_define_connect_level_keyword] = ACTIONS(1793),
    [sym_define_latching_keyword] = ACTIONS(1793),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1793),
    [sym_define_toggling_keyword] = ACTIONS(1793),
    [sym_define_call_keyword] = ACTIONS(1793),
    [sym_define_function_keyword] = ACTIONS(1793),
    [sym_define_library_function_keyword] = ACTIONS(1793),
    [sym_define_start_keyword] = ACTIONS(1793),
    [sym_define_event_keyword] = ACTIONS(1793),
    [sym_define_program_keyword] = ACTIONS(1793),
    [sym_define_module_keyword] = ACTIONS(1793),
    [sym_char_keyword] = ACTIONS(1793),
    [sym_widechar_keyword] = ACTIONS(1793),
    [sym_integer_keyword] = ACTIONS(1793),
    [sym_sinteger_keyword] = ACTIONS(1793),
    [sym_long_keyword] = ACTIONS(1793),
    [sym_slong_keyword] = ACTIONS(1793),
    [sym_float_keyword] = ACTIONS(1793),
    [sym_double_keyword] = ACTIONS(1793),
    [sym_variant_keyword] = ACTIONS(1793),
    [sym_variantarray_keyword] = ACTIONS(1793),
    [sym_local_var_keyword] = ACTIONS(1793),
    [sym_stack_var_keyword] = ACTIONS(1793),
    [sym_constant_keyword] = ACTIONS(1793),
    [sym_volatile_keyword] = ACTIONS(1793),
    [sym_non_volatile_keyword] = ACTIONS(1793),
    [sym_persistent_keyword] = ACTIONS(1793),
    [sym_dev_keyword] = ACTIONS(1793),
    [sym_devlev_keyword] = ACTIONS(1793),
    [sym_devchan_keyword] = ACTIONS(1793),
    [sym_if_keyword] = ACTIONS(1793),
    [sym_else_keyword] = ACTIONS(1793),
    [sym_switch_keyword] = ACTIONS(1793),
    [sym_case_keyword] = ACTIONS(1793),
    [sym_default_keyword] = ACTIONS(1793),
    [sym_while_keyword] = ACTIONS(1793),
    [sym_for_keyword] = ACTIONS(1793),
    [sym_break_keyword] = ACTIONS(1793),
    [sym_continue_keyword] = ACTIONS(1793),
    [sym_return_keyword] = ACTIONS(1793),
    [sym_select_keyword] = ACTIONS(1793),
    [sym_active_keyword] = ACTIONS(1793),
    [sym_struct_keyword] = ACTIONS(1793),
    [sym_structure_keyword] = ACTIONS(1793),
    [sym_button_event_keyword] = ACTIONS(1793),
    [sym_channel_event_keyword] = ACTIONS(1793),
    [sym_level_event_keyword] = ACTIONS(1793),
    [sym_data_event_keyword] = ACTIONS(1793),
    [sym_timeline_event_keyword] = ACTIONS(1793),
    [sym_custom_event_keyword] = ACTIONS(1793),
    [sym_push_keyword] = ACTIONS(1793),
    [sym_release_keyword] = ACTIONS(1793),
    [aux_sym_on_keyword_token1] = ACTIONS(1793),
    [aux_sym_off_keyword_token1] = ACTIONS(1793),
    [sym_send_level_keyword] = ACTIONS(1793),
    [sym_send_string_keyword] = ACTIONS(1793),
    [sym_send_command_keyword] = ACTIONS(1793),
    [sym_clear_buffer_keyword] = ACTIONS(1793),
    [sym_create_buffer_keyword] = ACTIONS(1793),
    [sym_create_multi_buffer_keyword] = ACTIONS(1793),
    [sym_call_keyword] = ACTIONS(1793),
    [sym_system_call_keyword] = ACTIONS(1793),
    [sym_devchan_to_keyword] = ACTIONS(1793),
    [sym_devchan_min_to_keyword] = ACTIONS(1793),
    [sym_devchan_total_off_keyword] = ACTIONS(1793),
    [sym_devchan_pulse_keyword] = ACTIONS(1793),
    [sym_wait_keyword] = ACTIONS(1793),
    [sym_cancel_wait_keyword] = ACTIONS(1793),
    [sym_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1793),
    [sym_not] = ACTIONS(1793),
    [sym_bnot] = ACTIONS(1793),
    [sym___file__] = ACTIONS(1793),
    [sym___date__] = ACTIONS(1793),
    [sym___ldate__] = ACTIONS(1793),
    [sym___time__] = ACTIONS(1793),
    [sym___line__] = ACTIONS(1793),
    [sym___name__] = ACTIONS(1793),
    [sym___netlinx__] = ACTIONS(1793),
    [sym_netlinx_axi_version] = ACTIONS(1793),
    [sym_url_flg_tcp] = ACTIONS(1793),
    [sym_url_flg_accinfopresent] = ACTIONS(1793),
    [sym_url_flg_temp] = ACTIONS(1793),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1793),
    [sym_url_flg_stat_mask] = ACTIONS(1793),
    [sym_url_flg_stat_lookup] = ACTIONS(1793),
    [sym_url_flg_stat_connecting] = ACTIONS(1793),
    [sym_url_flg_stat_waiting] = ACTIONS(1793),
    [sym_url_flg_stat_connected] = ACTIONS(1793),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1793),
    [sym_first_virtual_device] = ACTIONS(1793),
    [sym_first_local_port] = ACTIONS(1793),
    [sym_dynamic_virtual_device] = ACTIONS(1793),
    [sym_timeline_once] = ACTIONS(1793),
    [sym_timeline_repeat] = ACTIONS(1793),
    [sym_timeline_absolute] = ACTIONS(1793),
    [sym_timeline_relative] = ACTIONS(1793),
    [sym_do_push_timed_infinite] = ACTIONS(1793),
    [sym_source_type_no_address] = ACTIONS(1793),
    [sym_source_type_neuron_id] = ACTIONS(1793),
    [sym_source_type_ip_address] = ACTIONS(1793),
    [sym_source_type_axlink] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1793),
    [sym_source_type_ip_socket_address] = ACTIONS(1793),
    [sym_source_type_rs232] = ACTIONS(1793),
    [sym_source_type_internal] = ACTIONS(1793),
    [sym_source_type_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1793),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1793),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1793),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1793),
    [sym_true] = ACTIONS(1793),
    [sym_false] = ACTIONS(1793),
    [sym_file_read_only] = ACTIONS(1793),
    [sym_file_rw_new] = ACTIONS(1793),
    [sym_file_rw_append] = ACTIONS(1793),
    [sym_ip_tcp] = ACTIONS(1793),
    [sym_ip_udp] = ACTIONS(1793),
    [sym_ip_udp_2way] = ACTIONS(1793),
    [sym_xml_encode_types] = ACTIONS(1793),
    [sym_xml_encode_char_as_list] = ACTIONS(1793),
    [sym_xml_encode_le] = ACTIONS(1793),
    [sym_xml_decode_types] = ACTIONS(1793),
    [sym_xml_decode_no_preserve] = ACTIONS(1793),
    [sym_duet_dev_type_display_device] = ACTIONS(1793),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_io_device] = ACTIONS(1793),
    [sym_duet_dev_type_relay_device] = ACTIONS(1793),
    [sym_duet_dev_type_ups] = ACTIONS(1793),
    [sym_duet_dev_type_amplifier] = ACTIONS(1793),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1793),
    [sym_duet_dev_type_camera] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1793),
    [sym_duet_dev_type_dss] = ACTIONS(1793),
    [sym_duet_dev_type_dvr] = ACTIONS(1793),
    [sym_duet_dev_type_disc_device] = ACTIONS(1793),
    [sym_duet_dev_type_document_camera] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1793),
    [sym_duet_dev_type_hvac] = ACTIONS(1793),
    [sym_duet_dev_type_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_light] = ACTIONS(1793),
    [sym_duet_dev_type_monitor] = ACTIONS(1793),
    [sym_duet_dev_type_motor] = ACTIONS(1793),
    [sym_duet_dev_type_multi_window] = ACTIONS(1793),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1793),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1793),
    [sym_duet_dev_type_receiver] = ACTIONS(1793),
    [sym_duet_dev_type_security_system] = ACTIONS(1793),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1793),
    [sym_duet_dev_type_settop_box] = ACTIONS(1793),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1793),
    [sym_duet_dev_type_switcher] = ACTIONS(1793),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_tv] = ACTIONS(1793),
    [sym_duet_dev_type_utility] = ACTIONS(1793),
    [sym_duet_dev_type_vcr] = ACTIONS(1793),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_video_processor] = ACTIONS(1793),
    [sym_duet_dev_type_video_projector] = ACTIONS(1793),
    [sym_duet_dev_type_video_wall] = ACTIONS(1793),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1793),
    [sym_duet_dev_type_weather] = ACTIONS(1793),
    [sym_duet_dev_not_polled] = ACTIONS(1793),
    [sym_duet_dev_polled] = ACTIONS(1793),
    [sym_clkmgr_mode_network] = ACTIONS(1793),
    [sym_clkmgr_mode_standalone] = ACTIONS(1793),
    [sym_normal_standby] = ACTIONS(1793),
    [sym_normal_wake] = ACTIONS(1793),
    [sym_ip_multicast_ttl_option] = ACTIONS(1793),
    [sym_ip_ttl_subnet] = ACTIONS(1793),
    [sym_ip_ttl_site] = ACTIONS(1793),
    [sym_ip_ttl_region] = ACTIONS(1793),
    [sym_ip_ttl_continent] = ACTIONS(1793),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1793),
    [sym_ip_nodelay_on] = ACTIONS(1793),
    [sym_ip_nodelay_off] = ACTIONS(1793),
    [sym_amx_error] = ACTIONS(1793),
    [sym_amx_warning] = ACTIONS(1793),
    [sym_amx_info] = ACTIONS(1793),
    [sym_amx_debug] = ACTIONS(1793),
    [sym_netlinx_login_success] = ACTIONS(1793),
    [sym_netlinx_login_fail] = ACTIONS(1793),
    [sym_netlinx_logout] = ACTIONS(1793),
    [sym_null_str] = ACTIONS(1793),
    [sym_smtp_address] = ACTIONS(1793),
    [sym_smtp_port_number] = ACTIONS(1793),
    [sym_smtp_username] = ACTIONS(1793),
    [sym_smtp_password] = ACTIONS(1793),
    [sym_smtp_require_tls] = ACTIONS(1793),
    [sym_smtp_from] = ACTIONS(1793),
    [sym_smtp_tls_true] = ACTIONS(1793),
    [sym_smtp_tls_false] = ACTIONS(1793),
    [sym_listview_on_row_select_event] = ACTIONS(1793),
    [sym_data_structure_unknown] = ACTIONS(1793),
    [sym_data_structure_datafeed] = ACTIONS(1793),
    [sym_data_structure_datarecordset] = ACTIONS(1793),
    [sym_data_structure_datarecord] = ACTIONS(1793),
    [sym_data_structure_datafield] = ACTIONS(1793),
    [sym_data_max_content_field_count] = ACTIONS(1793),
    [sym_data_max_metadata_field_count] = ACTIONS(1793),
    [sym_data_max_id_length] = ACTIONS(1793),
    [sym_data_max_name_length] = ACTIONS(1793),
    [sym_data_max_description_length] = ACTIONS(1793),
    [sym_data_max_type_length] = ACTIONS(1793),
    [sym_data_max_format_length] = ACTIONS(1793),
    [sym_data_max_source_length] = ACTIONS(1793),
    [sym_data_max_value_length] = ACTIONS(1793),
    [sym_data_max_label_length] = ACTIONS(1793),
    [sym_data_max_url_length] = ACTIONS(1793),
    [sym_max_last_login_info_length] = ACTIONS(1793),
    [sym_data_type_unknown] = ACTIONS(1793),
    [sym_data_type_string] = ACTIONS(1793),
    [sym_data_type_datetime] = ACTIONS(1793),
    [sym_data_type_date] = ACTIONS(1793),
    [sym_data_type_time] = ACTIONS(1793),
    [sym_data_type_image] = ACTIONS(1793),
    [sym_data_format_url] = ACTIONS(1793),
    [sym_data_format_phone] = ACTIONS(1793),
    [sym_data_format_email] = ACTIONS(1793),
    [sym_data_format_iso8601] = ACTIONS(1793),
    [sym_auditlog_priv] = ACTIONS(1793),
    [sym_configuration_priv] = ACTIONS(1793),
    [sym_deviceconfig_priv] = ACTIONS(1793),
    [sym_ftp_priv] = ACTIONS(1793),
    [sym_http_priv] = ACTIONS(1793),
    [sym_networkconfig_priv] = ACTIONS(1793),
    [sym_programport_priv] = ACTIONS(1793),
    [sym_remoteui_priv] = ACTIONS(1793),
    [sym_securitycontrol_priv] = ACTIONS(1793),
    [sym_softwaremanagement_priv] = ACTIONS(1793),
    [sym_terminal_priv] = ACTIONS(1793),
    [sym_tpadmin_priv] = ACTIONS(1793),
    [sym_user1_priv] = ACTIONS(1793),
    [sym_user2_priv] = ACTIONS(1793),
    [sym_user3_priv] = ACTIONS(1793),
    [sym_user4_priv] = ACTIONS(1793),
    [sym_usermanagement_priv] = ACTIONS(1793),
    [sym_valid_account] = ACTIONS(1793),
    [sym_err_name_invalid] = ACTIONS(1793),
    [sym_err_password_invalid] = ACTIONS(1793),
    [sym_err_invalid_account] = ACTIONS(1793),
    [sym_err_no_authorization] = ACTIONS(1793),
    [sym_err_invalid_parameter] = ACTIONS(1793),
    [sym_err_user_locked_out] = ACTIONS(1793),
    [sym_err_user_account_expired] = ACTIONS(1793),
    [sym_err_general_error] = ACTIONS(1793),
    [sym_err_authentication_error] = ACTIONS(1793),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1793),
    [sym_tls_validate_certificate] = ACTIONS(1793),
    [sym_tdata] = ACTIONS(1793),
    [sym_tchannel] = ACTIONS(1793),
    [sym_tlevel] = ACTIONS(1793),
    [sym_tbutton] = ACTIONS(1793),
    [sym_ttimeline] = ACTIONS(1793),
    [sym_tcustom] = ACTIONS(1793),
    [sym_url_struct] = ACTIONS(1793),
    [sym_dns_struct] = ACTIONS(1793),
    [sym_ip_address_struct] = ACTIONS(1793),
    [sym_dev_info_struct] = ACTIONS(1793),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1793),
    [sym_data_feed] = ACTIONS(1793),
    [sym_data_field] = ACTIONS(1793),
    [sym_data_record] = ACTIONS(1793),
    [sym_wc_data_feed] = ACTIONS(1793),
    [sym_wc_data_field] = ACTIONS(1793),
    [sym_wc_data_record] = ACTIONS(1793),
    [sym_last_login_info] = ACTIONS(1793),
    [sym____reserved___] = ACTIONS(1793),
    [sym_date] = ACTIONS(1793),
    [sym_day] = ACTIONS(1793),
    [sym_time] = ACTIONS(1793),
    [sym_ldate] = ACTIONS(1793),
    [sym_push_device] = ACTIONS(1793),
    [sym_push_channel] = ACTIONS(1793),
    [sym_push_devchan] = ACTIONS(1793),
    [sym_release_device] = ACTIONS(1793),
    [sym_release_channel] = ACTIONS(1793),
    [sym_release_devchan] = ACTIONS(1793),
    [sym_master_slot] = ACTIONS(1793),
    [sym_get_pulse_time] = ACTIONS(1793),
    [sym_get_timer] = ACTIONS(1793),
    [sym_master_sn] = ACTIONS(1793),
    [sym_system_number] = ACTIONS(1793),
    [sym_button] = ACTIONS(1793),
    [sym_dv_channel] = ACTIONS(1793),
    [sym_data] = ACTIONS(1793),
    [sym_level] = ACTIONS(1793),
    [sym_timeline] = ACTIONS(1793),
    [sym_channel] = ACTIONS(1793),
    [sym_custom] = ACTIONS(1793),
    [sym_atoi] = ACTIONS(1793),
    [sym_atol] = ACTIONS(1793),
    [sym_atof] = ACTIONS(1793),
    [sym_itoa] = ACTIONS(1793),
    [sym_itohex] = ACTIONS(1793),
    [sym_hextoi] = ACTIONS(1793),
    [sym_ftoa] = ACTIONS(1793),
    [sym_find_string] = ACTIONS(1793),
    [sym_left_string] = ACTIONS(1793),
    [sym_right_string] = ACTIONS(1793),
    [sym_mid_string] = ACTIONS(1793),
    [sym_remove_string] = ACTIONS(1793),
    [sym_get_buffer_string] = ACTIONS(1793),
    [sym_get_buffer_char] = ACTIONS(1793),
    [sym_get_multi_buffer_string] = ACTIONS(1793),
    [sym_length_string] = ACTIONS(1793),
    [sym_max_length_string] = ACTIONS(1793),
    [sym_set_length_string] = ACTIONS(1793),
    [sym_compare_string] = ACTIONS(1793),
    [sym_upper_string] = ACTIONS(1793),
    [sym_lower_string] = ACTIONS(1793),
    [sym_get_last] = ACTIONS(1793),
    [sym_date_to_day] = ACTIONS(1793),
    [sym_date_to_month] = ACTIONS(1793),
    [sym_date_to_year] = ACTIONS(1793),
    [sym_time_to_hour] = ACTIONS(1793),
    [sym_time_to_minute] = ACTIONS(1793),
    [sym_time_to_second] = ACTIONS(1793),
    [sym_day_of_week] = ACTIONS(1793),
    [sym_set_pulse_time] = ACTIONS(1793),
    [sym_set_timer] = ACTIONS(1793),
    [sym_file_open] = ACTIONS(1793),
    [sym_file_close] = ACTIONS(1793),
    [sym_file_read] = ACTIONS(1793),
    [sym_file_read_line] = ACTIONS(1793),
    [sym_file_write] = ACTIONS(1793),
    [sym_file_write_line] = ACTIONS(1793),
    [sym_file_copy] = ACTIONS(1793),
    [sym_file_delete] = ACTIONS(1793),
    [sym_file_rename] = ACTIONS(1793),
    [sym_file_dir] = ACTIONS(1793),
    [sym_file_getdir] = ACTIONS(1793),
    [sym_file_setdir] = ACTIONS(1793),
    [sym_file_create_dir] = ACTIONS(1793),
    [sym_file_remove_dir] = ACTIONS(1793),
    [sym_file_seek] = ACTIONS(1793),
    [sym_length_array] = ACTIONS(1793),
    [sym_max_length_array] = ACTIONS(1793),
    [sym_set_length_array] = ACTIONS(1793),
    [sym_ip_client_open] = ACTIONS(1793),
    [sym_ip_client_close] = ACTIONS(1793),
    [sym_ip_server_open] = ACTIONS(1793),
    [sym_ip_server_close] = ACTIONS(1793),
    [sym_ip_bound_client_open] = ACTIONS(1793),
    [sym_ip_mc_server_open] = ACTIONS(1793),
    [sym_ip_set_option] = ACTIONS(1793),
    [sym_ssh_client_open] = ACTIONS(1793),
    [sym_ssh_client_close] = ACTIONS(1793),
    [sym_tls_client_open] = ACTIONS(1793),
    [sym_tls_client_close] = ACTIONS(1793),
    [sym_do_push] = ACTIONS(1793),
    [sym_do_push_timed] = ACTIONS(1793),
    [sym_do_release] = ACTIONS(1793),
    [sym_device_id] = ACTIONS(1793),
    [sym_device_info] = ACTIONS(1793),
    [sym_device_id_string] = ACTIONS(1793),
    [sym_device_standby] = ACTIONS(1793),
    [sym_device_wake] = ACTIONS(1793),
    [sym_timeline_create] = ACTIONS(1793),
    [sym_timeline_kill] = ACTIONS(1793),
    [sym_timeline_reload] = ACTIONS(1793),
    [sym_timeline_pause] = ACTIONS(1793),
    [sym_timeline_restart] = ACTIONS(1793),
    [sym_timeline_set] = ACTIONS(1793),
    [sym_timeline_get] = ACTIONS(1793),
    [sym_timeline_active] = ACTIONS(1793),
    [sym_timeline_dynamic_id] = ACTIONS(1793),
    [sym_variable_to_string] = ACTIONS(1793),
    [sym_string_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_string] = ACTIONS(1793),
    [sym_type_cast] = ACTIONS(1793),
    [sym_variable_to_xml] = ACTIONS(1793),
    [sym_xml_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_xml] = ACTIONS(1793),
    [sym_redirect_string] = ACTIONS(1793),
    [sym_random_number] = ACTIONS(1793),
    [sym_get_unique_id] = ACTIONS(1793),
    [sym_get_system_number] = ACTIONS(1793),
    [sym_set_system_number] = ACTIONS(1793),
    [sym_get_serial_number] = ACTIONS(1793),
    [sym_get_master_build] = ACTIONS(1793),
    [sym_set_validation_code] = ACTIONS(1793),
    [sym_reboot] = ACTIONS(1793),
    [sym_load_duet_module] = ACTIONS(1793),
    [sym_unload_duet_module] = ACTIONS(1793),
    [sym_duet_mem_size_set] = ACTIONS(1793),
    [sym_duet_mem_size_get] = ACTIONS(1793),
    [sym_dynamic_application_device] = ACTIONS(1793),
    [sym_dynamic_polled_port] = ACTIONS(1793),
    [sym_static_port_binding] = ACTIONS(1793),
    [sym_static_ip_binding] = ACTIONS(1793),
    [sym_get_dns_list] = ACTIONS(1793),
    [sym_set_dns_list] = ACTIONS(1793),
    [sym_get_ip_address] = ACTIONS(1793),
    [sym_set_ip_address] = ACTIONS(1793),
    [sym_get_url_list] = ACTIONS(1793),
    [sym_add_url_entry] = ACTIONS(1793),
    [sym_delete_url_entry] = ACTIONS(1793),
    [sym_abs_value] = ACTIONS(1793),
    [sym_max_value] = ACTIONS(1793),
    [sym_min_value] = ACTIONS(1793),
    [sym_exp_value] = ACTIONS(1793),
    [sym_log_value] = ACTIONS(1793),
    [sym_log10_value] = ACTIONS(1793),
    [sym_power_value] = ACTIONS(1793),
    [sym_sqrt_value] = ACTIONS(1793),
    [sym_raw_be] = ACTIONS(1793),
    [sym_raw_le] = ACTIONS(1793),
    [sym_internal_queue_size_set] = ACTIONS(1793),
    [sym_internal_queue_size_get] = ACTIONS(1793),
    [sym_internal_threshold_set] = ACTIONS(1793),
    [sym_internal_threshold_get] = ACTIONS(1793),
    [sym_rebuilt_event] = ACTIONS(1793),
    [sym_do_custom_event] = ACTIONS(1793),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1793),
    [sym_clkmgr_set_clk_source] = ACTIONS(1793),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1793),
    [sym_clkmgr_get_timezone] = ACTIONS(1793),
    [sym_clkmgr_set_timezone] = ACTIONS(1793),
    [sym_clkmgr_get_resync_period] = ACTIONS(1793),
    [sym_clkmgr_set_resync_period] = ACTIONS(1793),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_timeservers] = ACTIONS(1793),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_set_log_level] = ACTIONS(1793),
    [sym_get_log_level] = ACTIONS(1793),
    [sym_amx_log] = ACTIONS(1793),
    [sym_smtp_server_config_set] = ACTIONS(1793),
    [sym_smtp_server_config_get] = ACTIONS(1793),
    [sym_smtp_send] = ACTIONS(1793),
    [sym_validate_netlinx_account] = ACTIONS(1793),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1793),
    [sym_authenticate_certificate] = ACTIONS(1793),
    [sym_audit_netlinx_session_event] = ACTIONS(1793),
    [sym_audit_netlinx_generic_event] = ACTIONS(1793),
    [sym_get_available_flash_disk_space] = ACTIONS(1793),
    [sym_get_max_flash_disk_space] = ACTIONS(1793),
    [sym_data_create_feed] = ACTIONS(1793),
    [sym_data_delete_feed] = ACTIONS(1793),
    [sym_data_publish_feed] = ACTIONS(1793),
    [sym_data_get_published_feed] = ACTIONS(1793),
    [sym_data_add_record] = ACTIONS(1793),
    [sym_data_get_event_record] = ACTIONS(1793),
    [sym__wc_data_create_feed] = ACTIONS(1793),
    [sym__wc_data_add_record] = ACTIONS(1793),
    [sym__wc_data_get_event_record] = ACTIONS(1793),
    [sym_format] = ACTIONS(1793),
    [sym_astro_clock] = ACTIONS(1793),
    [sym_set_outdoor_temperature] = ACTIONS(1793),
    [sym_set_virtual_level_count] = ACTIONS(1793),
    [sym_set_virtual_channel_count] = ACTIONS(1793),
    [sym_set_virtual_port_count] = ACTIONS(1793),
    [sym__wc_encode_int] = ACTIONS(1793),
    [sym__wc_decode_int] = ACTIONS(1793),
    [sym__wc_int] = ACTIONS(1793),
    [sym__wc_to_ch_int] = ACTIONS(1793),
    [sym__ch_to_wc_int] = ACTIONS(1793),
    [sym__wc_find_string_int] = ACTIONS(1793),
    [sym__wc_left_string_int] = ACTIONS(1793),
    [sym__wc_lower_string_int] = ACTIONS(1793),
    [sym__wc_mid_string_int] = ACTIONS(1793),
    [sym__wc_remove_string_int] = ACTIONS(1793),
    [sym__wc_right_string_int] = ACTIONS(1793),
    [sym__wc_upper_string_int] = ACTIONS(1793),
    [sym__wc_compare_string_int] = ACTIONS(1793),
    [sym__wc_get_buffer_char_int] = ACTIONS(1793),
    [sym__wc_get_buffer_string_int] = ACTIONS(1793),
    [sym__wc_concat_string_int] = ACTIONS(1793),
    [sym__wc_file_open_int] = ACTIONS(1793),
    [sym__wc_file_close_int] = ACTIONS(1793),
    [sym__wc_file_read_int] = ACTIONS(1793),
    [sym__wc_file_read_line_int] = ACTIONS(1793),
    [sym__wc_file_write_int] = ACTIONS(1793),
    [sym__wc_file_write_line_int] = ACTIONS(1793),
    [aux_sym_preproc_include_token1] = ACTIONS(1793),
    [aux_sym_preproc_define_token1] = ACTIONS(1795),
    [aux_sym_preproc_warn_token1] = ACTIONS(1795),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1795),
    [anon_sym_LBRACK] = ACTIONS(1795),
    [anon_sym_LBRACE] = ACTIONS(1795),
    [anon_sym_LPAREN] = ACTIONS(1793),
    [anon_sym_BANG] = ACTIONS(1795),
    [anon_sym_TILDE] = ACTIONS(1795),
    [anon_sym_DASH] = ACTIONS(1793),
    [anon_sym_PLUS] = ACTIONS(1793),
    [anon_sym_DASH_DASH] = ACTIONS(1795),
    [anon_sym_PLUS_PLUS] = ACTIONS(1795),
    [anon_sym_DQUOTE] = ACTIONS(1795),
    [anon_sym_SQUOTE] = ACTIONS(1795),
    [sym_number_literal] = ACTIONS(1795),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(361)] = {
    [ts_builtin_sym_end] = ACTIONS(1795),
    [sym_identifier] = ACTIONS(1793),
    [sym_program_name_keyword] = ACTIONS(1793),
    [sym_module_name_keyword] = ACTIONS(1793),
    [sym_define_device_keyword] = ACTIONS(1793),
    [sym_define_combine_keyword] = ACTIONS(1793),
    [sym_define_constant_keyword] = ACTIONS(1793),
    [sym_define_type_keyword] = ACTIONS(1793),
    [sym_define_variable_keyword] = ACTIONS(1793),
    [sym_define_system_variable_keyword] = ACTIONS(1793),
    [sym_define_connect_level_keyword] = ACTIONS(1793),
    [sym_define_latching_keyword] = ACTIONS(1793),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1793),
    [sym_define_toggling_keyword] = ACTIONS(1793),
    [sym_define_call_keyword] = ACTIONS(1793),
    [sym_define_function_keyword] = ACTIONS(1793),
    [sym_define_library_function_keyword] = ACTIONS(1793),
    [sym_define_start_keyword] = ACTIONS(1793),
    [sym_define_event_keyword] = ACTIONS(1793),
    [sym_define_program_keyword] = ACTIONS(1793),
    [sym_define_module_keyword] = ACTIONS(1793),
    [sym_char_keyword] = ACTIONS(1793),
    [sym_widechar_keyword] = ACTIONS(1793),
    [sym_integer_keyword] = ACTIONS(1793),
    [sym_sinteger_keyword] = ACTIONS(1793),
    [sym_long_keyword] = ACTIONS(1793),
    [sym_slong_keyword] = ACTIONS(1793),
    [sym_float_keyword] = ACTIONS(1793),
    [sym_double_keyword] = ACTIONS(1793),
    [sym_variant_keyword] = ACTIONS(1793),
    [sym_variantarray_keyword] = ACTIONS(1793),
    [sym_local_var_keyword] = ACTIONS(1793),
    [sym_stack_var_keyword] = ACTIONS(1793),
    [sym_constant_keyword] = ACTIONS(1793),
    [sym_volatile_keyword] = ACTIONS(1793),
    [sym_non_volatile_keyword] = ACTIONS(1793),
    [sym_persistent_keyword] = ACTIONS(1793),
    [sym_dev_keyword] = ACTIONS(1793),
    [sym_devlev_keyword] = ACTIONS(1793),
    [sym_devchan_keyword] = ACTIONS(1793),
    [sym_if_keyword] = ACTIONS(1793),
    [sym_else_keyword] = ACTIONS(1793),
    [sym_switch_keyword] = ACTIONS(1793),
    [sym_case_keyword] = ACTIONS(1793),
    [sym_default_keyword] = ACTIONS(1793),
    [sym_while_keyword] = ACTIONS(1793),
    [sym_for_keyword] = ACTIONS(1793),
    [sym_break_keyword] = ACTIONS(1793),
    [sym_continue_keyword] = ACTIONS(1793),
    [sym_return_keyword] = ACTIONS(1793),
    [sym_select_keyword] = ACTIONS(1793),
    [sym_active_keyword] = ACTIONS(1793),
    [sym_struct_keyword] = ACTIONS(1793),
    [sym_structure_keyword] = ACTIONS(1793),
    [sym_button_event_keyword] = ACTIONS(1793),
    [sym_channel_event_keyword] = ACTIONS(1793),
    [sym_level_event_keyword] = ACTIONS(1793),
    [sym_data_event_keyword] = ACTIONS(1793),
    [sym_timeline_event_keyword] = ACTIONS(1793),
    [sym_custom_event_keyword] = ACTIONS(1793),
    [sym_push_keyword] = ACTIONS(1793),
    [sym_release_keyword] = ACTIONS(1793),
    [aux_sym_on_keyword_token1] = ACTIONS(1793),
    [aux_sym_off_keyword_token1] = ACTIONS(1793),
    [sym_send_level_keyword] = ACTIONS(1793),
    [sym_send_string_keyword] = ACTIONS(1793),
    [sym_send_command_keyword] = ACTIONS(1793),
    [sym_clear_buffer_keyword] = ACTIONS(1793),
    [sym_create_buffer_keyword] = ACTIONS(1793),
    [sym_create_multi_buffer_keyword] = ACTIONS(1793),
    [sym_call_keyword] = ACTIONS(1793),
    [sym_system_call_keyword] = ACTIONS(1793),
    [sym_devchan_to_keyword] = ACTIONS(1793),
    [sym_devchan_min_to_keyword] = ACTIONS(1793),
    [sym_devchan_total_off_keyword] = ACTIONS(1793),
    [sym_devchan_pulse_keyword] = ACTIONS(1793),
    [sym_wait_keyword] = ACTIONS(1793),
    [sym_cancel_wait_keyword] = ACTIONS(1793),
    [sym_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1793),
    [sym_not] = ACTIONS(1793),
    [sym_bnot] = ACTIONS(1793),
    [sym___file__] = ACTIONS(1793),
    [sym___date__] = ACTIONS(1793),
    [sym___ldate__] = ACTIONS(1793),
    [sym___time__] = ACTIONS(1793),
    [sym___line__] = ACTIONS(1793),
    [sym___name__] = ACTIONS(1793),
    [sym___netlinx__] = ACTIONS(1793),
    [sym_netlinx_axi_version] = ACTIONS(1793),
    [sym_url_flg_tcp] = ACTIONS(1793),
    [sym_url_flg_accinfopresent] = ACTIONS(1793),
    [sym_url_flg_temp] = ACTIONS(1793),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1793),
    [sym_url_flg_stat_mask] = ACTIONS(1793),
    [sym_url_flg_stat_lookup] = ACTIONS(1793),
    [sym_url_flg_stat_connecting] = ACTIONS(1793),
    [sym_url_flg_stat_waiting] = ACTIONS(1793),
    [sym_url_flg_stat_connected] = ACTIONS(1793),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1793),
    [sym_first_virtual_device] = ACTIONS(1793),
    [sym_first_local_port] = ACTIONS(1793),
    [sym_dynamic_virtual_device] = ACTIONS(1793),
    [sym_timeline_once] = ACTIONS(1793),
    [sym_timeline_repeat] = ACTIONS(1793),
    [sym_timeline_absolute] = ACTIONS(1793),
    [sym_timeline_relative] = ACTIONS(1793),
    [sym_do_push_timed_infinite] = ACTIONS(1793),
    [sym_source_type_no_address] = ACTIONS(1793),
    [sym_source_type_neuron_id] = ACTIONS(1793),
    [sym_source_type_ip_address] = ACTIONS(1793),
    [sym_source_type_axlink] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1793),
    [sym_source_type_ip_socket_address] = ACTIONS(1793),
    [sym_source_type_rs232] = ACTIONS(1793),
    [sym_source_type_internal] = ACTIONS(1793),
    [sym_source_type_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1793),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1793),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1793),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1793),
    [sym_true] = ACTIONS(1793),
    [sym_false] = ACTIONS(1793),
    [sym_file_read_only] = ACTIONS(1793),
    [sym_file_rw_new] = ACTIONS(1793),
    [sym_file_rw_append] = ACTIONS(1793),
    [sym_ip_tcp] = ACTIONS(1793),
    [sym_ip_udp] = ACTIONS(1793),
    [sym_ip_udp_2way] = ACTIONS(1793),
    [sym_xml_encode_types] = ACTIONS(1793),
    [sym_xml_encode_char_as_list] = ACTIONS(1793),
    [sym_xml_encode_le] = ACTIONS(1793),
    [sym_xml_decode_types] = ACTIONS(1793),
    [sym_xml_decode_no_preserve] = ACTIONS(1793),
    [sym_duet_dev_type_display_device] = ACTIONS(1793),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_io_device] = ACTIONS(1793),
    [sym_duet_dev_type_relay_device] = ACTIONS(1793),
    [sym_duet_dev_type_ups] = ACTIONS(1793),
    [sym_duet_dev_type_amplifier] = ACTIONS(1793),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1793),
    [sym_duet_dev_type_camera] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1793),
    [sym_duet_dev_type_dss] = ACTIONS(1793),
    [sym_duet_dev_type_dvr] = ACTIONS(1793),
    [sym_duet_dev_type_disc_device] = ACTIONS(1793),
    [sym_duet_dev_type_document_camera] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1793),
    [sym_duet_dev_type_hvac] = ACTIONS(1793),
    [sym_duet_dev_type_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_light] = ACTIONS(1793),
    [sym_duet_dev_type_monitor] = ACTIONS(1793),
    [sym_duet_dev_type_motor] = ACTIONS(1793),
    [sym_duet_dev_type_multi_window] = ACTIONS(1793),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1793),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1793),
    [sym_duet_dev_type_receiver] = ACTIONS(1793),
    [sym_duet_dev_type_security_system] = ACTIONS(1793),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1793),
    [sym_duet_dev_type_settop_box] = ACTIONS(1793),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1793),
    [sym_duet_dev_type_switcher] = ACTIONS(1793),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_tv] = ACTIONS(1793),
    [sym_duet_dev_type_utility] = ACTIONS(1793),
    [sym_duet_dev_type_vcr] = ACTIONS(1793),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_video_processor] = ACTIONS(1793),
    [sym_duet_dev_type_video_projector] = ACTIONS(1793),
    [sym_duet_dev_type_video_wall] = ACTIONS(1793),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1793),
    [sym_duet_dev_type_weather] = ACTIONS(1793),
    [sym_duet_dev_not_polled] = ACTIONS(1793),
    [sym_duet_dev_polled] = ACTIONS(1793),
    [sym_clkmgr_mode_network] = ACTIONS(1793),
    [sym_clkmgr_mode_standalone] = ACTIONS(1793),
    [sym_normal_standby] = ACTIONS(1793),
    [sym_normal_wake] = ACTIONS(1793),
    [sym_ip_multicast_ttl_option] = ACTIONS(1793),
    [sym_ip_ttl_subnet] = ACTIONS(1793),
    [sym_ip_ttl_site] = ACTIONS(1793),
    [sym_ip_ttl_region] = ACTIONS(1793),
    [sym_ip_ttl_continent] = ACTIONS(1793),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1793),
    [sym_ip_nodelay_on] = ACTIONS(1793),
    [sym_ip_nodelay_off] = ACTIONS(1793),
    [sym_amx_error] = ACTIONS(1793),
    [sym_amx_warning] = ACTIONS(1793),
    [sym_amx_info] = ACTIONS(1793),
    [sym_amx_debug] = ACTIONS(1793),
    [sym_netlinx_login_success] = ACTIONS(1793),
    [sym_netlinx_login_fail] = ACTIONS(1793),
    [sym_netlinx_logout] = ACTIONS(1793),
    [sym_null_str] = ACTIONS(1793),
    [sym_smtp_address] = ACTIONS(1793),
    [sym_smtp_port_number] = ACTIONS(1793),
    [sym_smtp_username] = ACTIONS(1793),
    [sym_smtp_password] = ACTIONS(1793),
    [sym_smtp_require_tls] = ACTIONS(1793),
    [sym_smtp_from] = ACTIONS(1793),
    [sym_smtp_tls_true] = ACTIONS(1793),
    [sym_smtp_tls_false] = ACTIONS(1793),
    [sym_listview_on_row_select_event] = ACTIONS(1793),
    [sym_data_structure_unknown] = ACTIONS(1793),
    [sym_data_structure_datafeed] = ACTIONS(1793),
    [sym_data_structure_datarecordset] = ACTIONS(1793),
    [sym_data_structure_datarecord] = ACTIONS(1793),
    [sym_data_structure_datafield] = ACTIONS(1793),
    [sym_data_max_content_field_count] = ACTIONS(1793),
    [sym_data_max_metadata_field_count] = ACTIONS(1793),
    [sym_data_max_id_length] = ACTIONS(1793),
    [sym_data_max_name_length] = ACTIONS(1793),
    [sym_data_max_description_length] = ACTIONS(1793),
    [sym_data_max_type_length] = ACTIONS(1793),
    [sym_data_max_format_length] = ACTIONS(1793),
    [sym_data_max_source_length] = ACTIONS(1793),
    [sym_data_max_value_length] = ACTIONS(1793),
    [sym_data_max_label_length] = ACTIONS(1793),
    [sym_data_max_url_length] = ACTIONS(1793),
    [sym_max_last_login_info_length] = ACTIONS(1793),
    [sym_data_type_unknown] = ACTIONS(1793),
    [sym_data_type_string] = ACTIONS(1793),
    [sym_data_type_datetime] = ACTIONS(1793),
    [sym_data_type_date] = ACTIONS(1793),
    [sym_data_type_time] = ACTIONS(1793),
    [sym_data_type_image] = ACTIONS(1793),
    [sym_data_format_url] = ACTIONS(1793),
    [sym_data_format_phone] = ACTIONS(1793),
    [sym_data_format_email] = ACTIONS(1793),
    [sym_data_format_iso8601] = ACTIONS(1793),
    [sym_auditlog_priv] = ACTIONS(1793),
    [sym_configuration_priv] = ACTIONS(1793),
    [sym_deviceconfig_priv] = ACTIONS(1793),
    [sym_ftp_priv] = ACTIONS(1793),
    [sym_http_priv] = ACTIONS(1793),
    [sym_networkconfig_priv] = ACTIONS(1793),
    [sym_programport_priv] = ACTIONS(1793),
    [sym_remoteui_priv] = ACTIONS(1793),
    [sym_securitycontrol_priv] = ACTIONS(1793),
    [sym_softwaremanagement_priv] = ACTIONS(1793),
    [sym_terminal_priv] = ACTIONS(1793),
    [sym_tpadmin_priv] = ACTIONS(1793),
    [sym_user1_priv] = ACTIONS(1793),
    [sym_user2_priv] = ACTIONS(1793),
    [sym_user3_priv] = ACTIONS(1793),
    [sym_user4_priv] = ACTIONS(1793),
    [sym_usermanagement_priv] = ACTIONS(1793),
    [sym_valid_account] = ACTIONS(1793),
    [sym_err_name_invalid] = ACTIONS(1793),
    [sym_err_password_invalid] = ACTIONS(1793),
    [sym_err_invalid_account] = ACTIONS(1793),
    [sym_err_no_authorization] = ACTIONS(1793),
    [sym_err_invalid_parameter] = ACTIONS(1793),
    [sym_err_user_locked_out] = ACTIONS(1793),
    [sym_err_user_account_expired] = ACTIONS(1793),
    [sym_err_general_error] = ACTIONS(1793),
    [sym_err_authentication_error] = ACTIONS(1793),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1793),
    [sym_tls_validate_certificate] = ACTIONS(1793),
    [sym_tdata] = ACTIONS(1793),
    [sym_tchannel] = ACTIONS(1793),
    [sym_tlevel] = ACTIONS(1793),
    [sym_tbutton] = ACTIONS(1793),
    [sym_ttimeline] = ACTIONS(1793),
    [sym_tcustom] = ACTIONS(1793),
    [sym_url_struct] = ACTIONS(1793),
    [sym_dns_struct] = ACTIONS(1793),
    [sym_ip_address_struct] = ACTIONS(1793),
    [sym_dev_info_struct] = ACTIONS(1793),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1793),
    [sym_data_feed] = ACTIONS(1793),
    [sym_data_field] = ACTIONS(1793),
    [sym_data_record] = ACTIONS(1793),
    [sym_wc_data_feed] = ACTIONS(1793),
    [sym_wc_data_field] = ACTIONS(1793),
    [sym_wc_data_record] = ACTIONS(1793),
    [sym_last_login_info] = ACTIONS(1793),
    [sym____reserved___] = ACTIONS(1793),
    [sym_date] = ACTIONS(1793),
    [sym_day] = ACTIONS(1793),
    [sym_time] = ACTIONS(1793),
    [sym_ldate] = ACTIONS(1793),
    [sym_push_device] = ACTIONS(1793),
    [sym_push_channel] = ACTIONS(1793),
    [sym_push_devchan] = ACTIONS(1793),
    [sym_release_device] = ACTIONS(1793),
    [sym_release_channel] = ACTIONS(1793),
    [sym_release_devchan] = ACTIONS(1793),
    [sym_master_slot] = ACTIONS(1793),
    [sym_get_pulse_time] = ACTIONS(1793),
    [sym_get_timer] = ACTIONS(1793),
    [sym_master_sn] = ACTIONS(1793),
    [sym_system_number] = ACTIONS(1793),
    [sym_button] = ACTIONS(1793),
    [sym_dv_channel] = ACTIONS(1793),
    [sym_data] = ACTIONS(1793),
    [sym_level] = ACTIONS(1793),
    [sym_timeline] = ACTIONS(1793),
    [sym_channel] = ACTIONS(1793),
    [sym_custom] = ACTIONS(1793),
    [sym_atoi] = ACTIONS(1793),
    [sym_atol] = ACTIONS(1793),
    [sym_atof] = ACTIONS(1793),
    [sym_itoa] = ACTIONS(1793),
    [sym_itohex] = ACTIONS(1793),
    [sym_hextoi] = ACTIONS(1793),
    [sym_ftoa] = ACTIONS(1793),
    [sym_find_string] = ACTIONS(1793),
    [sym_left_string] = ACTIONS(1793),
    [sym_right_string] = ACTIONS(1793),
    [sym_mid_string] = ACTIONS(1793),
    [sym_remove_string] = ACTIONS(1793),
    [sym_get_buffer_string] = ACTIONS(1793),
    [sym_get_buffer_char] = ACTIONS(1793),
    [sym_get_multi_buffer_string] = ACTIONS(1793),
    [sym_length_string] = ACTIONS(1793),
    [sym_max_length_string] = ACTIONS(1793),
    [sym_set_length_string] = ACTIONS(1793),
    [sym_compare_string] = ACTIONS(1793),
    [sym_upper_string] = ACTIONS(1793),
    [sym_lower_string] = ACTIONS(1793),
    [sym_get_last] = ACTIONS(1793),
    [sym_date_to_day] = ACTIONS(1793),
    [sym_date_to_month] = ACTIONS(1793),
    [sym_date_to_year] = ACTIONS(1793),
    [sym_time_to_hour] = ACTIONS(1793),
    [sym_time_to_minute] = ACTIONS(1793),
    [sym_time_to_second] = ACTIONS(1793),
    [sym_day_of_week] = ACTIONS(1793),
    [sym_set_pulse_time] = ACTIONS(1793),
    [sym_set_timer] = ACTIONS(1793),
    [sym_file_open] = ACTIONS(1793),
    [sym_file_close] = ACTIONS(1793),
    [sym_file_read] = ACTIONS(1793),
    [sym_file_read_line] = ACTIONS(1793),
    [sym_file_write] = ACTIONS(1793),
    [sym_file_write_line] = ACTIONS(1793),
    [sym_file_copy] = ACTIONS(1793),
    [sym_file_delete] = ACTIONS(1793),
    [sym_file_rename] = ACTIONS(1793),
    [sym_file_dir] = ACTIONS(1793),
    [sym_file_getdir] = ACTIONS(1793),
    [sym_file_setdir] = ACTIONS(1793),
    [sym_file_create_dir] = ACTIONS(1793),
    [sym_file_remove_dir] = ACTIONS(1793),
    [sym_file_seek] = ACTIONS(1793),
    [sym_length_array] = ACTIONS(1793),
    [sym_max_length_array] = ACTIONS(1793),
    [sym_set_length_array] = ACTIONS(1793),
    [sym_ip_client_open] = ACTIONS(1793),
    [sym_ip_client_close] = ACTIONS(1793),
    [sym_ip_server_open] = ACTIONS(1793),
    [sym_ip_server_close] = ACTIONS(1793),
    [sym_ip_bound_client_open] = ACTIONS(1793),
    [sym_ip_mc_server_open] = ACTIONS(1793),
    [sym_ip_set_option] = ACTIONS(1793),
    [sym_ssh_client_open] = ACTIONS(1793),
    [sym_ssh_client_close] = ACTIONS(1793),
    [sym_tls_client_open] = ACTIONS(1793),
    [sym_tls_client_close] = ACTIONS(1793),
    [sym_do_push] = ACTIONS(1793),
    [sym_do_push_timed] = ACTIONS(1793),
    [sym_do_release] = ACTIONS(1793),
    [sym_device_id] = ACTIONS(1793),
    [sym_device_info] = ACTIONS(1793),
    [sym_device_id_string] = ACTIONS(1793),
    [sym_device_standby] = ACTIONS(1793),
    [sym_device_wake] = ACTIONS(1793),
    [sym_timeline_create] = ACTIONS(1793),
    [sym_timeline_kill] = ACTIONS(1793),
    [sym_timeline_reload] = ACTIONS(1793),
    [sym_timeline_pause] = ACTIONS(1793),
    [sym_timeline_restart] = ACTIONS(1793),
    [sym_timeline_set] = ACTIONS(1793),
    [sym_timeline_get] = ACTIONS(1793),
    [sym_timeline_active] = ACTIONS(1793),
    [sym_timeline_dynamic_id] = ACTIONS(1793),
    [sym_variable_to_string] = ACTIONS(1793),
    [sym_string_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_string] = ACTIONS(1793),
    [sym_type_cast] = ACTIONS(1793),
    [sym_variable_to_xml] = ACTIONS(1793),
    [sym_xml_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_xml] = ACTIONS(1793),
    [sym_redirect_string] = ACTIONS(1793),
    [sym_random_number] = ACTIONS(1793),
    [sym_get_unique_id] = ACTIONS(1793),
    [sym_get_system_number] = ACTIONS(1793),
    [sym_set_system_number] = ACTIONS(1793),
    [sym_get_serial_number] = ACTIONS(1793),
    [sym_get_master_build] = ACTIONS(1793),
    [sym_set_validation_code] = ACTIONS(1793),
    [sym_reboot] = ACTIONS(1793),
    [sym_load_duet_module] = ACTIONS(1793),
    [sym_unload_duet_module] = ACTIONS(1793),
    [sym_duet_mem_size_set] = ACTIONS(1793),
    [sym_duet_mem_size_get] = ACTIONS(1793),
    [sym_dynamic_application_device] = ACTIONS(1793),
    [sym_dynamic_polled_port] = ACTIONS(1793),
    [sym_static_port_binding] = ACTIONS(1793),
    [sym_static_ip_binding] = ACTIONS(1793),
    [sym_get_dns_list] = ACTIONS(1793),
    [sym_set_dns_list] = ACTIONS(1793),
    [sym_get_ip_address] = ACTIONS(1793),
    [sym_set_ip_address] = ACTIONS(1793),
    [sym_get_url_list] = ACTIONS(1793),
    [sym_add_url_entry] = ACTIONS(1793),
    [sym_delete_url_entry] = ACTIONS(1793),
    [sym_abs_value] = ACTIONS(1793),
    [sym_max_value] = ACTIONS(1793),
    [sym_min_value] = ACTIONS(1793),
    [sym_exp_value] = ACTIONS(1793),
    [sym_log_value] = ACTIONS(1793),
    [sym_log10_value] = ACTIONS(1793),
    [sym_power_value] = ACTIONS(1793),
    [sym_sqrt_value] = ACTIONS(1793),
    [sym_raw_be] = ACTIONS(1793),
    [sym_raw_le] = ACTIONS(1793),
    [sym_internal_queue_size_set] = ACTIONS(1793),
    [sym_internal_queue_size_get] = ACTIONS(1793),
    [sym_internal_threshold_set] = ACTIONS(1793),
    [sym_internal_threshold_get] = ACTIONS(1793),
    [sym_rebuilt_event] = ACTIONS(1793),
    [sym_do_custom_event] = ACTIONS(1793),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1793),
    [sym_clkmgr_set_clk_source] = ACTIONS(1793),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1793),
    [sym_clkmgr_get_timezone] = ACTIONS(1793),
    [sym_clkmgr_set_timezone] = ACTIONS(1793),
    [sym_clkmgr_get_resync_period] = ACTIONS(1793),
    [sym_clkmgr_set_resync_period] = ACTIONS(1793),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_timeservers] = ACTIONS(1793),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_set_log_level] = ACTIONS(1793),
    [sym_get_log_level] = ACTIONS(1793),
    [sym_amx_log] = ACTIONS(1793),
    [sym_smtp_server_config_set] = ACTIONS(1793),
    [sym_smtp_server_config_get] = ACTIONS(1793),
    [sym_smtp_send] = ACTIONS(1793),
    [sym_validate_netlinx_account] = ACTIONS(1793),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1793),
    [sym_authenticate_certificate] = ACTIONS(1793),
    [sym_audit_netlinx_session_event] = ACTIONS(1793),
    [sym_audit_netlinx_generic_event] = ACTIONS(1793),
    [sym_get_available_flash_disk_space] = ACTIONS(1793),
    [sym_get_max_flash_disk_space] = ACTIONS(1793),
    [sym_data_create_feed] = ACTIONS(1793),
    [sym_data_delete_feed] = ACTIONS(1793),
    [sym_data_publish_feed] = ACTIONS(1793),
    [sym_data_get_published_feed] = ACTIONS(1793),
    [sym_data_add_record] = ACTIONS(1793),
    [sym_data_get_event_record] = ACTIONS(1793),
    [sym__wc_data_create_feed] = ACTIONS(1793),
    [sym__wc_data_add_record] = ACTIONS(1793),
    [sym__wc_data_get_event_record] = ACTIONS(1793),
    [sym_format] = ACTIONS(1793),
    [sym_astro_clock] = ACTIONS(1793),
    [sym_set_outdoor_temperature] = ACTIONS(1793),
    [sym_set_virtual_level_count] = ACTIONS(1793),
    [sym_set_virtual_channel_count] = ACTIONS(1793),
    [sym_set_virtual_port_count] = ACTIONS(1793),
    [sym__wc_encode_int] = ACTIONS(1793),
    [sym__wc_decode_int] = ACTIONS(1793),
    [sym__wc_int] = ACTIONS(1793),
    [sym__wc_to_ch_int] = ACTIONS(1793),
    [sym__ch_to_wc_int] = ACTIONS(1793),
    [sym__wc_find_string_int] = ACTIONS(1793),
    [sym__wc_left_string_int] = ACTIONS(1793),
    [sym__wc_lower_string_int] = ACTIONS(1793),
    [sym__wc_mid_string_int] = ACTIONS(1793),
    [sym__wc_remove_string_int] = ACTIONS(1793),
    [sym__wc_right_string_int] = ACTIONS(1793),
    [sym__wc_upper_string_int] = ACTIONS(1793),
    [sym__wc_compare_string_int] = ACTIONS(1793),
    [sym__wc_get_buffer_char_int] = ACTIONS(1793),
    [sym__wc_get_buffer_string_int] = ACTIONS(1793),
    [sym__wc_concat_string_int] = ACTIONS(1793),
    [sym__wc_file_open_int] = ACTIONS(1793),
    [sym__wc_file_close_int] = ACTIONS(1793),
    [sym__wc_file_read_int] = ACTIONS(1793),
    [sym__wc_file_read_line_int] = ACTIONS(1793),
    [sym__wc_file_write_int] = ACTIONS(1793),
    [sym__wc_file_write_line_int] = ACTIONS(1793),
    [aux_sym_preproc_include_token1] = ACTIONS(1793),
    [aux_sym_preproc_define_token1] = ACTIONS(1795),
    [aux_sym_preproc_warn_token1] = ACTIONS(1795),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1795),
    [anon_sym_LBRACK] = ACTIONS(1795),
    [anon_sym_LBRACE] = ACTIONS(1795),
    [anon_sym_LPAREN] = ACTIONS(1793),
    [anon_sym_BANG] = ACTIONS(1795),
    [anon_sym_TILDE] = ACTIONS(1795),
    [anon_sym_DASH] = ACTIONS(1793),
    [anon_sym_PLUS] = ACTIONS(1793),
    [anon_sym_DASH_DASH] = ACTIONS(1795),
    [anon_sym_PLUS_PLUS] = ACTIONS(1795),
    [anon_sym_DQUOTE] = ACTIONS(1795),
    [anon_sym_SQUOTE] = ACTIONS(1795),
    [sym_number_literal] = ACTIONS(1795),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(362)] = {
    [ts_builtin_sym_end] = ACTIONS(1761),
    [sym_identifier] = ACTIONS(1759),
    [sym_program_name_keyword] = ACTIONS(1759),
    [sym_module_name_keyword] = ACTIONS(1759),
    [sym_define_device_keyword] = ACTIONS(1759),
    [sym_define_combine_keyword] = ACTIONS(1759),
    [sym_define_constant_keyword] = ACTIONS(1759),
    [sym_define_type_keyword] = ACTIONS(1759),
    [sym_define_variable_keyword] = ACTIONS(1759),
    [sym_define_system_variable_keyword] = ACTIONS(1759),
    [sym_define_connect_level_keyword] = ACTIONS(1759),
    [sym_define_latching_keyword] = ACTIONS(1759),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1759),
    [sym_define_toggling_keyword] = ACTIONS(1759),
    [sym_define_call_keyword] = ACTIONS(1759),
    [sym_define_function_keyword] = ACTIONS(1759),
    [sym_define_library_function_keyword] = ACTIONS(1759),
    [sym_define_start_keyword] = ACTIONS(1759),
    [sym_define_event_keyword] = ACTIONS(1759),
    [sym_define_program_keyword] = ACTIONS(1759),
    [sym_define_module_keyword] = ACTIONS(1759),
    [sym_char_keyword] = ACTIONS(1759),
    [sym_widechar_keyword] = ACTIONS(1759),
    [sym_integer_keyword] = ACTIONS(1759),
    [sym_sinteger_keyword] = ACTIONS(1759),
    [sym_long_keyword] = ACTIONS(1759),
    [sym_slong_keyword] = ACTIONS(1759),
    [sym_float_keyword] = ACTIONS(1759),
    [sym_double_keyword] = ACTIONS(1759),
    [sym_variant_keyword] = ACTIONS(1759),
    [sym_variantarray_keyword] = ACTIONS(1759),
    [sym_local_var_keyword] = ACTIONS(1759),
    [sym_stack_var_keyword] = ACTIONS(1759),
    [sym_constant_keyword] = ACTIONS(1759),
    [sym_volatile_keyword] = ACTIONS(1759),
    [sym_non_volatile_keyword] = ACTIONS(1759),
    [sym_persistent_keyword] = ACTIONS(1759),
    [sym_dev_keyword] = ACTIONS(1759),
    [sym_devlev_keyword] = ACTIONS(1759),
    [sym_devchan_keyword] = ACTIONS(1759),
    [sym_if_keyword] = ACTIONS(1759),
    [sym_else_keyword] = ACTIONS(1759),
    [sym_switch_keyword] = ACTIONS(1759),
    [sym_case_keyword] = ACTIONS(1759),
    [sym_default_keyword] = ACTIONS(1759),
    [sym_while_keyword] = ACTIONS(1759),
    [sym_for_keyword] = ACTIONS(1759),
    [sym_break_keyword] = ACTIONS(1759),
    [sym_continue_keyword] = ACTIONS(1759),
    [sym_return_keyword] = ACTIONS(1759),
    [sym_select_keyword] = ACTIONS(1759),
    [sym_active_keyword] = ACTIONS(1759),
    [sym_struct_keyword] = ACTIONS(1759),
    [sym_structure_keyword] = ACTIONS(1759),
    [sym_button_event_keyword] = ACTIONS(1759),
    [sym_channel_event_keyword] = ACTIONS(1759),
    [sym_level_event_keyword] = ACTIONS(1759),
    [sym_data_event_keyword] = ACTIONS(1759),
    [sym_timeline_event_keyword] = ACTIONS(1759),
    [sym_custom_event_keyword] = ACTIONS(1759),
    [sym_push_keyword] = ACTIONS(1759),
    [sym_release_keyword] = ACTIONS(1759),
    [aux_sym_on_keyword_token1] = ACTIONS(1759),
    [aux_sym_off_keyword_token1] = ACTIONS(1759),
    [sym_send_level_keyword] = ACTIONS(1759),
    [sym_send_string_keyword] = ACTIONS(1759),
    [sym_send_command_keyword] = ACTIONS(1759),
    [sym_clear_buffer_keyword] = ACTIONS(1759),
    [sym_create_buffer_keyword] = ACTIONS(1759),
    [sym_create_multi_buffer_keyword] = ACTIONS(1759),
    [sym_call_keyword] = ACTIONS(1759),
    [sym_system_call_keyword] = ACTIONS(1759),
    [sym_devchan_to_keyword] = ACTIONS(1759),
    [sym_devchan_min_to_keyword] = ACTIONS(1759),
    [sym_devchan_total_off_keyword] = ACTIONS(1759),
    [sym_devchan_pulse_keyword] = ACTIONS(1759),
    [sym_wait_keyword] = ACTIONS(1759),
    [sym_cancel_wait_keyword] = ACTIONS(1759),
    [sym_wait_until_keyword] = ACTIONS(1759),
    [sym_cancel_wait_until_keyword] = ACTIONS(1759),
    [sym_cancel_all_wait_keyword] = ACTIONS(1759),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1759),
    [sym_not] = ACTIONS(1759),
    [sym_bnot] = ACTIONS(1759),
    [sym___file__] = ACTIONS(1759),
    [sym___date__] = ACTIONS(1759),
    [sym___ldate__] = ACTIONS(1759),
    [sym___time__] = ACTIONS(1759),
    [sym___line__] = ACTIONS(1759),
    [sym___name__] = ACTIONS(1759),
    [sym___netlinx__] = ACTIONS(1759),
    [sym_netlinx_axi_version] = ACTIONS(1759),
    [sym_url_flg_tcp] = ACTIONS(1759),
    [sym_url_flg_accinfopresent] = ACTIONS(1759),
    [sym_url_flg_temp] = ACTIONS(1759),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1759),
    [sym_url_flg_stat_mask] = ACTIONS(1759),
    [sym_url_flg_stat_lookup] = ACTIONS(1759),
    [sym_url_flg_stat_connecting] = ACTIONS(1759),
    [sym_url_flg_stat_waiting] = ACTIONS(1759),
    [sym_url_flg_stat_connected] = ACTIONS(1759),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1759),
    [sym_first_virtual_device] = ACTIONS(1759),
    [sym_first_local_port] = ACTIONS(1759),
    [sym_dynamic_virtual_device] = ACTIONS(1759),
    [sym_timeline_once] = ACTIONS(1759),
    [sym_timeline_repeat] = ACTIONS(1759),
    [sym_timeline_absolute] = ACTIONS(1759),
    [sym_timeline_relative] = ACTIONS(1759),
    [sym_do_push_timed_infinite] = ACTIONS(1759),
    [sym_source_type_no_address] = ACTIONS(1759),
    [sym_source_type_neuron_id] = ACTIONS(1759),
    [sym_source_type_ip_address] = ACTIONS(1759),
    [sym_source_type_axlink] = ACTIONS(1759),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1759),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1759),
    [sym_source_type_ip_socket_address] = ACTIONS(1759),
    [sym_source_type_rs232] = ACTIONS(1759),
    [sym_source_type_internal] = ACTIONS(1759),
    [sym_source_type_mac_address] = ACTIONS(1759),
    [sym_source_type_ipv4_port] = ACTIONS(1759),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1759),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1759),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1759),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1759),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1759),
    [sym_true] = ACTIONS(1759),
    [sym_false] = ACTIONS(1759),
    [sym_file_read_only] = ACTIONS(1759),
    [sym_file_rw_new] = ACTIONS(1759),
    [sym_file_rw_append] = ACTIONS(1759),
    [sym_ip_tcp] = ACTIONS(1759),
    [sym_ip_udp] = ACTIONS(1759),
    [sym_ip_udp_2way] = ACTIONS(1759),
    [sym_xml_encode_types] = ACTIONS(1759),
    [sym_xml_encode_char_as_list] = ACTIONS(1759),
    [sym_xml_encode_le] = ACTIONS(1759),
    [sym_xml_decode_types] = ACTIONS(1759),
    [sym_xml_decode_no_preserve] = ACTIONS(1759),
    [sym_duet_dev_type_display_device] = ACTIONS(1759),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1759),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1759),
    [sym_duet_dev_type_io_device] = ACTIONS(1759),
    [sym_duet_dev_type_relay_device] = ACTIONS(1759),
    [sym_duet_dev_type_ups] = ACTIONS(1759),
    [sym_duet_dev_type_amplifier] = ACTIONS(1759),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1759),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1759),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1759),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1759),
    [sym_duet_dev_type_camera] = ACTIONS(1759),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1759),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1759),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1759),
    [sym_duet_dev_type_dss] = ACTIONS(1759),
    [sym_duet_dev_type_dvr] = ACTIONS(1759),
    [sym_duet_dev_type_disc_device] = ACTIONS(1759),
    [sym_duet_dev_type_document_camera] = ACTIONS(1759),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1759),
    [sym_duet_dev_type_hvac] = ACTIONS(1759),
    [sym_duet_dev_type_keypad] = ACTIONS(1759),
    [sym_duet_dev_type_light] = ACTIONS(1759),
    [sym_duet_dev_type_monitor] = ACTIONS(1759),
    [sym_duet_dev_type_motor] = ACTIONS(1759),
    [sym_duet_dev_type_multi_window] = ACTIONS(1759),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1759),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1759),
    [sym_duet_dev_type_receiver] = ACTIONS(1759),
    [sym_duet_dev_type_security_system] = ACTIONS(1759),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1759),
    [sym_duet_dev_type_settop_box] = ACTIONS(1759),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1759),
    [sym_duet_dev_type_switcher] = ACTIONS(1759),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1759),
    [sym_duet_dev_type_tv] = ACTIONS(1759),
    [sym_duet_dev_type_utility] = ACTIONS(1759),
    [sym_duet_dev_type_vcr] = ACTIONS(1759),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1759),
    [sym_duet_dev_type_video_processor] = ACTIONS(1759),
    [sym_duet_dev_type_video_projector] = ACTIONS(1759),
    [sym_duet_dev_type_video_wall] = ACTIONS(1759),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1759),
    [sym_duet_dev_type_weather] = ACTIONS(1759),
    [sym_duet_dev_not_polled] = ACTIONS(1759),
    [sym_duet_dev_polled] = ACTIONS(1759),
    [sym_clkmgr_mode_network] = ACTIONS(1759),
    [sym_clkmgr_mode_standalone] = ACTIONS(1759),
    [sym_normal_standby] = ACTIONS(1759),
    [sym_normal_wake] = ACTIONS(1759),
    [sym_ip_multicast_ttl_option] = ACTIONS(1759),
    [sym_ip_ttl_subnet] = ACTIONS(1759),
    [sym_ip_ttl_site] = ACTIONS(1759),
    [sym_ip_ttl_region] = ACTIONS(1759),
    [sym_ip_ttl_continent] = ACTIONS(1759),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1759),
    [sym_ip_nodelay_on] = ACTIONS(1759),
    [sym_ip_nodelay_off] = ACTIONS(1759),
    [sym_amx_error] = ACTIONS(1759),
    [sym_amx_warning] = ACTIONS(1759),
    [sym_amx_info] = ACTIONS(1759),
    [sym_amx_debug] = ACTIONS(1759),
    [sym_netlinx_login_success] = ACTIONS(1759),
    [sym_netlinx_login_fail] = ACTIONS(1759),
    [sym_netlinx_logout] = ACTIONS(1759),
    [sym_null_str] = ACTIONS(1759),
    [sym_smtp_address] = ACTIONS(1759),
    [sym_smtp_port_number] = ACTIONS(1759),
    [sym_smtp_username] = ACTIONS(1759),
    [sym_smtp_password] = ACTIONS(1759),
    [sym_smtp_require_tls] = ACTIONS(1759),
    [sym_smtp_from] = ACTIONS(1759),
    [sym_smtp_tls_true] = ACTIONS(1759),
    [sym_smtp_tls_false] = ACTIONS(1759),
    [sym_listview_on_row_select_event] = ACTIONS(1759),
    [sym_data_structure_unknown] = ACTIONS(1759),
    [sym_data_structure_datafeed] = ACTIONS(1759),
    [sym_data_structure_datarecordset] = ACTIONS(1759),
    [sym_data_structure_datarecord] = ACTIONS(1759),
    [sym_data_structure_datafield] = ACTIONS(1759),
    [sym_data_max_content_field_count] = ACTIONS(1759),
    [sym_data_max_metadata_field_count] = ACTIONS(1759),
    [sym_data_max_id_length] = ACTIONS(1759),
    [sym_data_max_name_length] = ACTIONS(1759),
    [sym_data_max_description_length] = ACTIONS(1759),
    [sym_data_max_type_length] = ACTIONS(1759),
    [sym_data_max_format_length] = ACTIONS(1759),
    [sym_data_max_source_length] = ACTIONS(1759),
    [sym_data_max_value_length] = ACTIONS(1759),
    [sym_data_max_label_length] = ACTIONS(1759),
    [sym_data_max_url_length] = ACTIONS(1759),
    [sym_max_last_login_info_length] = ACTIONS(1759),
    [sym_data_type_unknown] = ACTIONS(1759),
    [sym_data_type_string] = ACTIONS(1759),
    [sym_data_type_datetime] = ACTIONS(1759),
    [sym_data_type_date] = ACTIONS(1759),
    [sym_data_type_time] = ACTIONS(1759),
    [sym_data_type_image] = ACTIONS(1759),
    [sym_data_format_url] = ACTIONS(1759),
    [sym_data_format_phone] = ACTIONS(1759),
    [sym_data_format_email] = ACTIONS(1759),
    [sym_data_format_iso8601] = ACTIONS(1759),
    [sym_auditlog_priv] = ACTIONS(1759),
    [sym_configuration_priv] = ACTIONS(1759),
    [sym_deviceconfig_priv] = ACTIONS(1759),
    [sym_ftp_priv] = ACTIONS(1759),
    [sym_http_priv] = ACTIONS(1759),
    [sym_networkconfig_priv] = ACTIONS(1759),
    [sym_programport_priv] = ACTIONS(1759),
    [sym_remoteui_priv] = ACTIONS(1759),
    [sym_securitycontrol_priv] = ACTIONS(1759),
    [sym_softwaremanagement_priv] = ACTIONS(1759),
    [sym_terminal_priv] = ACTIONS(1759),
    [sym_tpadmin_priv] = ACTIONS(1759),
    [sym_user1_priv] = ACTIONS(1759),
    [sym_user2_priv] = ACTIONS(1759),
    [sym_user3_priv] = ACTIONS(1759),
    [sym_user4_priv] = ACTIONS(1759),
    [sym_usermanagement_priv] = ACTIONS(1759),
    [sym_valid_account] = ACTIONS(1759),
    [sym_err_name_invalid] = ACTIONS(1759),
    [sym_err_password_invalid] = ACTIONS(1759),
    [sym_err_invalid_account] = ACTIONS(1759),
    [sym_err_no_authorization] = ACTIONS(1759),
    [sym_err_invalid_parameter] = ACTIONS(1759),
    [sym_err_user_locked_out] = ACTIONS(1759),
    [sym_err_user_account_expired] = ACTIONS(1759),
    [sym_err_general_error] = ACTIONS(1759),
    [sym_err_authentication_error] = ACTIONS(1759),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1759),
    [sym_tls_validate_certificate] = ACTIONS(1759),
    [sym_tdata] = ACTIONS(1759),
    [sym_tchannel] = ACTIONS(1759),
    [sym_tlevel] = ACTIONS(1759),
    [sym_tbutton] = ACTIONS(1759),
    [sym_ttimeline] = ACTIONS(1759),
    [sym_tcustom] = ACTIONS(1759),
    [sym_url_struct] = ACTIONS(1759),
    [sym_dns_struct] = ACTIONS(1759),
    [sym_ip_address_struct] = ACTIONS(1759),
    [sym_dev_info_struct] = ACTIONS(1759),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1759),
    [sym_data_feed] = ACTIONS(1759),
    [sym_data_field] = ACTIONS(1759),
    [sym_data_record] = ACTIONS(1759),
    [sym_wc_data_feed] = ACTIONS(1759),
    [sym_wc_data_field] = ACTIONS(1759),
    [sym_wc_data_record] = ACTIONS(1759),
    [sym_last_login_info] = ACTIONS(1759),
    [sym____reserved___] = ACTIONS(1759),
    [sym_date] = ACTIONS(1759),
    [sym_day] = ACTIONS(1759),
    [sym_time] = ACTIONS(1759),
    [sym_ldate] = ACTIONS(1759),
    [sym_push_device] = ACTIONS(1759),
    [sym_push_channel] = ACTIONS(1759),
    [sym_push_devchan] = ACTIONS(1759),
    [sym_release_device] = ACTIONS(1759),
    [sym_release_channel] = ACTIONS(1759),
    [sym_release_devchan] = ACTIONS(1759),
    [sym_master_slot] = ACTIONS(1759),
    [sym_get_pulse_time] = ACTIONS(1759),
    [sym_get_timer] = ACTIONS(1759),
    [sym_master_sn] = ACTIONS(1759),
    [sym_system_number] = ACTIONS(1759),
    [sym_button] = ACTIONS(1759),
    [sym_dv_channel] = ACTIONS(1759),
    [sym_data] = ACTIONS(1759),
    [sym_level] = ACTIONS(1759),
    [sym_timeline] = ACTIONS(1759),
    [sym_channel] = ACTIONS(1759),
    [sym_custom] = ACTIONS(1759),
    [sym_atoi] = ACTIONS(1759),
    [sym_atol] = ACTIONS(1759),
    [sym_atof] = ACTIONS(1759),
    [sym_itoa] = ACTIONS(1759),
    [sym_itohex] = ACTIONS(1759),
    [sym_hextoi] = ACTIONS(1759),
    [sym_ftoa] = ACTIONS(1759),
    [sym_find_string] = ACTIONS(1759),
    [sym_left_string] = ACTIONS(1759),
    [sym_right_string] = ACTIONS(1759),
    [sym_mid_string] = ACTIONS(1759),
    [sym_remove_string] = ACTIONS(1759),
    [sym_get_buffer_string] = ACTIONS(1759),
    [sym_get_buffer_char] = ACTIONS(1759),
    [sym_get_multi_buffer_string] = ACTIONS(1759),
    [sym_length_string] = ACTIONS(1759),
    [sym_max_length_string] = ACTIONS(1759),
    [sym_set_length_string] = ACTIONS(1759),
    [sym_compare_string] = ACTIONS(1759),
    [sym_upper_string] = ACTIONS(1759),
    [sym_lower_string] = ACTIONS(1759),
    [sym_get_last] = ACTIONS(1759),
    [sym_date_to_day] = ACTIONS(1759),
    [sym_date_to_month] = ACTIONS(1759),
    [sym_date_to_year] = ACTIONS(1759),
    [sym_time_to_hour] = ACTIONS(1759),
    [sym_time_to_minute] = ACTIONS(1759),
    [sym_time_to_second] = ACTIONS(1759),
    [sym_day_of_week] = ACTIONS(1759),
    [sym_set_pulse_time] = ACTIONS(1759),
    [sym_set_timer] = ACTIONS(1759),
    [sym_file_open] = ACTIONS(1759),
    [sym_file_close] = ACTIONS(1759),
    [sym_file_read] = ACTIONS(1759),
    [sym_file_read_line] = ACTIONS(1759),
    [sym_file_write] = ACTIONS(1759),
    [sym_file_write_line] = ACTIONS(1759),
    [sym_file_copy] = ACTIONS(1759),
    [sym_file_delete] = ACTIONS(1759),
    [sym_file_rename] = ACTIONS(1759),
    [sym_file_dir] = ACTIONS(1759),
    [sym_file_getdir] = ACTIONS(1759),
    [sym_file_setdir] = ACTIONS(1759),
    [sym_file_create_dir] = ACTIONS(1759),
    [sym_file_remove_dir] = ACTIONS(1759),
    [sym_file_seek] = ACTIONS(1759),
    [sym_length_array] = ACTIONS(1759),
    [sym_max_length_array] = ACTIONS(1759),
    [sym_set_length_array] = ACTIONS(1759),
    [sym_ip_client_open] = ACTIONS(1759),
    [sym_ip_client_close] = ACTIONS(1759),
    [sym_ip_server_open] = ACTIONS(1759),
    [sym_ip_server_close] = ACTIONS(1759),
    [sym_ip_bound_client_open] = ACTIONS(1759),
    [sym_ip_mc_server_open] = ACTIONS(1759),
    [sym_ip_set_option] = ACTIONS(1759),
    [sym_ssh_client_open] = ACTIONS(1759),
    [sym_ssh_client_close] = ACTIONS(1759),
    [sym_tls_client_open] = ACTIONS(1759),
    [sym_tls_client_close] = ACTIONS(1759),
    [sym_do_push] = ACTIONS(1759),
    [sym_do_push_timed] = ACTIONS(1759),
    [sym_do_release] = ACTIONS(1759),
    [sym_device_id] = ACTIONS(1759),
    [sym_device_info] = ACTIONS(1759),
    [sym_device_id_string] = ACTIONS(1759),
    [sym_device_standby] = ACTIONS(1759),
    [sym_device_wake] = ACTIONS(1759),
    [sym_timeline_create] = ACTIONS(1759),
    [sym_timeline_kill] = ACTIONS(1759),
    [sym_timeline_reload] = ACTIONS(1759),
    [sym_timeline_pause] = ACTIONS(1759),
    [sym_timeline_restart] = ACTIONS(1759),
    [sym_timeline_set] = ACTIONS(1759),
    [sym_timeline_get] = ACTIONS(1759),
    [sym_timeline_active] = ACTIONS(1759),
    [sym_timeline_dynamic_id] = ACTIONS(1759),
    [sym_variable_to_string] = ACTIONS(1759),
    [sym_string_to_variable] = ACTIONS(1759),
    [sym_length_variable_to_string] = ACTIONS(1759),
    [sym_type_cast] = ACTIONS(1759),
    [sym_variable_to_xml] = ACTIONS(1759),
    [sym_xml_to_variable] = ACTIONS(1759),
    [sym_length_variable_to_xml] = ACTIONS(1759),
    [sym_redirect_string] = ACTIONS(1759),
    [sym_random_number] = ACTIONS(1759),
    [sym_get_unique_id] = ACTIONS(1759),
    [sym_get_system_number] = ACTIONS(1759),
    [sym_set_system_number] = ACTIONS(1759),
    [sym_get_serial_number] = ACTIONS(1759),
    [sym_get_master_build] = ACTIONS(1759),
    [sym_set_validation_code] = ACTIONS(1759),
    [sym_reboot] = ACTIONS(1759),
    [sym_load_duet_module] = ACTIONS(1759),
    [sym_unload_duet_module] = ACTIONS(1759),
    [sym_duet_mem_size_set] = ACTIONS(1759),
    [sym_duet_mem_size_get] = ACTIONS(1759),
    [sym_dynamic_application_device] = ACTIONS(1759),
    [sym_dynamic_polled_port] = ACTIONS(1759),
    [sym_static_port_binding] = ACTIONS(1759),
    [sym_static_ip_binding] = ACTIONS(1759),
    [sym_get_dns_list] = ACTIONS(1759),
    [sym_set_dns_list] = ACTIONS(1759),
    [sym_get_ip_address] = ACTIONS(1759),
    [sym_set_ip_address] = ACTIONS(1759),
    [sym_get_url_list] = ACTIONS(1759),
    [sym_add_url_entry] = ACTIONS(1759),
    [sym_delete_url_entry] = ACTIONS(1759),
    [sym_abs_value] = ACTIONS(1759),
    [sym_max_value] = ACTIONS(1759),
    [sym_min_value] = ACTIONS(1759),
    [sym_exp_value] = ACTIONS(1759),
    [sym_log_value] = ACTIONS(1759),
    [sym_log10_value] = ACTIONS(1759),
    [sym_power_value] = ACTIONS(1759),
    [sym_sqrt_value] = ACTIONS(1759),
    [sym_raw_be] = ACTIONS(1759),
    [sym_raw_le] = ACTIONS(1759),
    [sym_internal_queue_size_set] = ACTIONS(1759),
    [sym_internal_queue_size_get] = ACTIONS(1759),
    [sym_internal_threshold_set] = ACTIONS(1759),
    [sym_internal_threshold_get] = ACTIONS(1759),
    [sym_rebuilt_event] = ACTIONS(1759),
    [sym_do_custom_event] = ACTIONS(1759),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1759),
    [sym_clkmgr_set_clk_source] = ACTIONS(1759),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1759),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1759),
    [sym_clkmgr_get_timezone] = ACTIONS(1759),
    [sym_clkmgr_set_timezone] = ACTIONS(1759),
    [sym_clkmgr_get_resync_period] = ACTIONS(1759),
    [sym_clkmgr_set_resync_period] = ACTIONS(1759),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1759),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1759),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1759),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1759),
    [sym_clkmgr_get_timeservers] = ACTIONS(1759),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1759),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1759),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1759),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1759),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1759),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1759),
    [sym_set_log_level] = ACTIONS(1759),
    [sym_get_log_level] = ACTIONS(1759),
    [sym_amx_log] = ACTIONS(1759),
    [sym_smtp_server_config_set] = ACTIONS(1759),
    [sym_smtp_server_config_get] = ACTIONS(1759),
    [sym_smtp_send] = ACTIONS(1759),
    [sym_validate_netlinx_account] = ACTIONS(1759),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1759),
    [sym_authenticate_certificate] = ACTIONS(1759),
    [sym_audit_netlinx_session_event] = ACTIONS(1759),
    [sym_audit_netlinx_generic_event] = ACTIONS(1759),
    [sym_get_available_flash_disk_space] = ACTIONS(1759),
    [sym_get_max_flash_disk_space] = ACTIONS(1759),
    [sym_data_create_feed] = ACTIONS(1759),
    [sym_data_delete_feed] = ACTIONS(1759),
    [sym_data_publish_feed] = ACTIONS(1759),
    [sym_data_get_published_feed] = ACTIONS(1759),
    [sym_data_add_record] = ACTIONS(1759),
    [sym_data_get_event_record] = ACTIONS(1759),
    [sym__wc_data_create_feed] = ACTIONS(1759),
    [sym__wc_data_add_record] = ACTIONS(1759),
    [sym__wc_data_get_event_record] = ACTIONS(1759),
    [sym_format] = ACTIONS(1759),
    [sym_astro_clock] = ACTIONS(1759),
    [sym_set_outdoor_temperature] = ACTIONS(1759),
    [sym_set_virtual_level_count] = ACTIONS(1759),
    [sym_set_virtual_channel_count] = ACTIONS(1759),
    [sym_set_virtual_port_count] = ACTIONS(1759),
    [sym__wc_encode_int] = ACTIONS(1759),
    [sym__wc_decode_int] = ACTIONS(1759),
    [sym__wc_int] = ACTIONS(1759),
    [sym__wc_to_ch_int] = ACTIONS(1759),
    [sym__ch_to_wc_int] = ACTIONS(1759),
    [sym__wc_find_string_int] = ACTIONS(1759),
    [sym__wc_left_string_int] = ACTIONS(1759),
    [sym__wc_lower_string_int] = ACTIONS(1759),
    [sym__wc_mid_string_int] = ACTIONS(1759),
    [sym__wc_remove_string_int] = ACTIONS(1759),
    [sym__wc_right_string_int] = ACTIONS(1759),
    [sym__wc_upper_string_int] = ACTIONS(1759),
    [sym__wc_compare_string_int] = ACTIONS(1759),
    [sym__wc_get_buffer_char_int] = ACTIONS(1759),
    [sym__wc_get_buffer_string_int] = ACTIONS(1759),
    [sym__wc_concat_string_int] = ACTIONS(1759),
    [sym__wc_file_open_int] = ACTIONS(1759),
    [sym__wc_file_close_int] = ACTIONS(1759),
    [sym__wc_file_read_int] = ACTIONS(1759),
    [sym__wc_file_read_line_int] = ACTIONS(1759),
    [sym__wc_file_write_int] = ACTIONS(1759),
    [sym__wc_file_write_line_int] = ACTIONS(1759),
    [aux_sym_preproc_include_token1] = ACTIONS(1759),
    [aux_sym_preproc_define_token1] = ACTIONS(1761),
    [aux_sym_preproc_warn_token1] = ACTIONS(1761),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1761),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1761),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1761),
    [anon_sym_LBRACK] = ACTIONS(1761),
    [anon_sym_LBRACE] = ACTIONS(1761),
    [anon_sym_LPAREN] = ACTIONS(1759),
    [anon_sym_BANG] = ACTIONS(1761),
    [anon_sym_TILDE] = ACTIONS(1761),
    [anon_sym_DASH] = ACTIONS(1759),
    [anon_sym_PLUS] = ACTIONS(1759),
    [anon_sym_DASH_DASH] = ACTIONS(1761),
    [anon_sym_PLUS_PLUS] = ACTIONS(1761),
    [anon_sym_DQUOTE] = ACTIONS(1761),
    [anon_sym_SQUOTE] = ACTIONS(1761),
    [sym_number_literal] = ACTIONS(1761),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(363)] = {
    [sym_identifier] = ACTIONS(2077),
    [sym_program_name_keyword] = ACTIONS(2077),
    [sym_module_name_keyword] = ACTIONS(2077),
    [sym_define_device_keyword] = ACTIONS(2077),
    [sym_define_combine_keyword] = ACTIONS(2077),
    [sym_define_constant_keyword] = ACTIONS(2077),
    [sym_define_type_keyword] = ACTIONS(2077),
    [sym_define_variable_keyword] = ACTIONS(2077),
    [sym_define_system_variable_keyword] = ACTIONS(2077),
    [sym_define_connect_level_keyword] = ACTIONS(2077),
    [sym_define_latching_keyword] = ACTIONS(2077),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2077),
    [sym_define_toggling_keyword] = ACTIONS(2077),
    [sym_define_call_keyword] = ACTIONS(2077),
    [sym_define_function_keyword] = ACTIONS(2077),
    [sym_define_library_function_keyword] = ACTIONS(2077),
    [sym_define_start_keyword] = ACTIONS(2077),
    [sym_define_event_keyword] = ACTIONS(2077),
    [sym_define_program_keyword] = ACTIONS(2077),
    [sym_define_module_keyword] = ACTIONS(2077),
    [sym_char_keyword] = ACTIONS(2077),
    [sym_widechar_keyword] = ACTIONS(2077),
    [sym_integer_keyword] = ACTIONS(2077),
    [sym_sinteger_keyword] = ACTIONS(2077),
    [sym_long_keyword] = ACTIONS(2077),
    [sym_slong_keyword] = ACTIONS(2077),
    [sym_float_keyword] = ACTIONS(2077),
    [sym_double_keyword] = ACTIONS(2077),
    [sym_variant_keyword] = ACTIONS(2077),
    [sym_variantarray_keyword] = ACTIONS(2077),
    [sym_local_var_keyword] = ACTIONS(2077),
    [sym_stack_var_keyword] = ACTIONS(2077),
    [sym_constant_keyword] = ACTIONS(2077),
    [sym_volatile_keyword] = ACTIONS(2077),
    [sym_non_volatile_keyword] = ACTIONS(2077),
    [sym_persistent_keyword] = ACTIONS(2077),
    [sym_dev_keyword] = ACTIONS(2077),
    [sym_devlev_keyword] = ACTIONS(2077),
    [sym_devchan_keyword] = ACTIONS(2077),
    [sym_if_keyword] = ACTIONS(2077),
    [sym_switch_keyword] = ACTIONS(2077),
    [sym_case_keyword] = ACTIONS(2077),
    [sym_default_keyword] = ACTIONS(2077),
    [sym_while_keyword] = ACTIONS(2077),
    [sym_for_keyword] = ACTIONS(2077),
    [sym_break_keyword] = ACTIONS(2077),
    [sym_continue_keyword] = ACTIONS(2077),
    [sym_return_keyword] = ACTIONS(2077),
    [sym_select_keyword] = ACTIONS(2077),
    [sym_active_keyword] = ACTIONS(2077),
    [sym_struct_keyword] = ACTIONS(2077),
    [sym_structure_keyword] = ACTIONS(2077),
    [sym_button_event_keyword] = ACTIONS(2077),
    [sym_channel_event_keyword] = ACTIONS(2077),
    [sym_level_event_keyword] = ACTIONS(2077),
    [sym_data_event_keyword] = ACTIONS(2077),
    [sym_timeline_event_keyword] = ACTIONS(2077),
    [sym_custom_event_keyword] = ACTIONS(2077),
    [sym_push_keyword] = ACTIONS(2077),
    [sym_release_keyword] = ACTIONS(2077),
    [aux_sym_on_keyword_token1] = ACTIONS(2077),
    [aux_sym_off_keyword_token1] = ACTIONS(2077),
    [sym_send_level_keyword] = ACTIONS(2077),
    [sym_send_string_keyword] = ACTIONS(2077),
    [sym_send_command_keyword] = ACTIONS(2077),
    [sym_clear_buffer_keyword] = ACTIONS(2077),
    [sym_create_buffer_keyword] = ACTIONS(2077),
    [sym_create_multi_buffer_keyword] = ACTIONS(2077),
    [sym_call_keyword] = ACTIONS(2077),
    [sym_system_call_keyword] = ACTIONS(2077),
    [sym_devchan_to_keyword] = ACTIONS(2077),
    [sym_devchan_min_to_keyword] = ACTIONS(2077),
    [sym_devchan_total_off_keyword] = ACTIONS(2077),
    [sym_devchan_pulse_keyword] = ACTIONS(2077),
    [sym_wait_keyword] = ACTIONS(2077),
    [sym_cancel_wait_keyword] = ACTIONS(2077),
    [sym_wait_until_keyword] = ACTIONS(2077),
    [sym_cancel_wait_until_keyword] = ACTIONS(2077),
    [sym_cancel_all_wait_keyword] = ACTIONS(2077),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2077),
    [sym_not] = ACTIONS(2077),
    [sym_bnot] = ACTIONS(2077),
    [sym___file__] = ACTIONS(2077),
    [sym___date__] = ACTIONS(2077),
    [sym___ldate__] = ACTIONS(2077),
    [sym___time__] = ACTIONS(2077),
    [sym___line__] = ACTIONS(2077),
    [sym___name__] = ACTIONS(2077),
    [sym___netlinx__] = ACTIONS(2077),
    [sym_netlinx_axi_version] = ACTIONS(2077),
    [sym_url_flg_tcp] = ACTIONS(2077),
    [sym_url_flg_accinfopresent] = ACTIONS(2077),
    [sym_url_flg_temp] = ACTIONS(2077),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2077),
    [sym_url_flg_stat_mask] = ACTIONS(2077),
    [sym_url_flg_stat_lookup] = ACTIONS(2077),
    [sym_url_flg_stat_connecting] = ACTIONS(2077),
    [sym_url_flg_stat_waiting] = ACTIONS(2077),
    [sym_url_flg_stat_connected] = ACTIONS(2077),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2077),
    [sym_first_virtual_device] = ACTIONS(2077),
    [sym_first_local_port] = ACTIONS(2077),
    [sym_dynamic_virtual_device] = ACTIONS(2077),
    [sym_timeline_once] = ACTIONS(2077),
    [sym_timeline_repeat] = ACTIONS(2077),
    [sym_timeline_absolute] = ACTIONS(2077),
    [sym_timeline_relative] = ACTIONS(2077),
    [sym_do_push_timed_infinite] = ACTIONS(2077),
    [sym_source_type_no_address] = ACTIONS(2077),
    [sym_source_type_neuron_id] = ACTIONS(2077),
    [sym_source_type_ip_address] = ACTIONS(2077),
    [sym_source_type_axlink] = ACTIONS(2077),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2077),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2077),
    [sym_source_type_ip_socket_address] = ACTIONS(2077),
    [sym_source_type_rs232] = ACTIONS(2077),
    [sym_source_type_internal] = ACTIONS(2077),
    [sym_source_type_mac_address] = ACTIONS(2077),
    [sym_source_type_ipv4_port] = ACTIONS(2077),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2077),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2077),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2077),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2077),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2077),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2077),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2077),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2077),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2077),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2077),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2077),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2077),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2077),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2077),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2077),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2077),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2077),
    [sym_true] = ACTIONS(2077),
    [sym_false] = ACTIONS(2077),
    [sym_file_read_only] = ACTIONS(2077),
    [sym_file_rw_new] = ACTIONS(2077),
    [sym_file_rw_append] = ACTIONS(2077),
    [sym_ip_tcp] = ACTIONS(2077),
    [sym_ip_udp] = ACTIONS(2077),
    [sym_ip_udp_2way] = ACTIONS(2077),
    [sym_xml_encode_types] = ACTIONS(2077),
    [sym_xml_encode_char_as_list] = ACTIONS(2077),
    [sym_xml_encode_le] = ACTIONS(2077),
    [sym_xml_decode_types] = ACTIONS(2077),
    [sym_xml_decode_no_preserve] = ACTIONS(2077),
    [sym_duet_dev_type_display_device] = ACTIONS(2077),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2077),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2077),
    [sym_duet_dev_type_io_device] = ACTIONS(2077),
    [sym_duet_dev_type_relay_device] = ACTIONS(2077),
    [sym_duet_dev_type_ups] = ACTIONS(2077),
    [sym_duet_dev_type_amplifier] = ACTIONS(2077),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2077),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2077),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2077),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2077),
    [sym_duet_dev_type_camera] = ACTIONS(2077),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2077),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2077),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2077),
    [sym_duet_dev_type_dss] = ACTIONS(2077),
    [sym_duet_dev_type_dvr] = ACTIONS(2077),
    [sym_duet_dev_type_disc_device] = ACTIONS(2077),
    [sym_duet_dev_type_document_camera] = ACTIONS(2077),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2077),
    [sym_duet_dev_type_hvac] = ACTIONS(2077),
    [sym_duet_dev_type_keypad] = ACTIONS(2077),
    [sym_duet_dev_type_light] = ACTIONS(2077),
    [sym_duet_dev_type_monitor] = ACTIONS(2077),
    [sym_duet_dev_type_motor] = ACTIONS(2077),
    [sym_duet_dev_type_multi_window] = ACTIONS(2077),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2077),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2077),
    [sym_duet_dev_type_receiver] = ACTIONS(2077),
    [sym_duet_dev_type_security_system] = ACTIONS(2077),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2077),
    [sym_duet_dev_type_settop_box] = ACTIONS(2077),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2077),
    [sym_duet_dev_type_switcher] = ACTIONS(2077),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2077),
    [sym_duet_dev_type_tv] = ACTIONS(2077),
    [sym_duet_dev_type_utility] = ACTIONS(2077),
    [sym_duet_dev_type_vcr] = ACTIONS(2077),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2077),
    [sym_duet_dev_type_video_processor] = ACTIONS(2077),
    [sym_duet_dev_type_video_projector] = ACTIONS(2077),
    [sym_duet_dev_type_video_wall] = ACTIONS(2077),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2077),
    [sym_duet_dev_type_weather] = ACTIONS(2077),
    [sym_duet_dev_not_polled] = ACTIONS(2077),
    [sym_duet_dev_polled] = ACTIONS(2077),
    [sym_clkmgr_mode_network] = ACTIONS(2077),
    [sym_clkmgr_mode_standalone] = ACTIONS(2077),
    [sym_normal_standby] = ACTIONS(2077),
    [sym_normal_wake] = ACTIONS(2077),
    [sym_ip_multicast_ttl_option] = ACTIONS(2077),
    [sym_ip_ttl_subnet] = ACTIONS(2077),
    [sym_ip_ttl_site] = ACTIONS(2077),
    [sym_ip_ttl_region] = ACTIONS(2077),
    [sym_ip_ttl_continent] = ACTIONS(2077),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2077),
    [sym_ip_nodelay_on] = ACTIONS(2077),
    [sym_ip_nodelay_off] = ACTIONS(2077),
    [sym_amx_error] = ACTIONS(2077),
    [sym_amx_warning] = ACTIONS(2077),
    [sym_amx_info] = ACTIONS(2077),
    [sym_amx_debug] = ACTIONS(2077),
    [sym_netlinx_login_success] = ACTIONS(2077),
    [sym_netlinx_login_fail] = ACTIONS(2077),
    [sym_netlinx_logout] = ACTIONS(2077),
    [sym_null_str] = ACTIONS(2077),
    [sym_smtp_address] = ACTIONS(2077),
    [sym_smtp_port_number] = ACTIONS(2077),
    [sym_smtp_username] = ACTIONS(2077),
    [sym_smtp_password] = ACTIONS(2077),
    [sym_smtp_require_tls] = ACTIONS(2077),
    [sym_smtp_from] = ACTIONS(2077),
    [sym_smtp_tls_true] = ACTIONS(2077),
    [sym_smtp_tls_false] = ACTIONS(2077),
    [sym_listview_on_row_select_event] = ACTIONS(2077),
    [sym_data_structure_unknown] = ACTIONS(2077),
    [sym_data_structure_datafeed] = ACTIONS(2077),
    [sym_data_structure_datarecordset] = ACTIONS(2077),
    [sym_data_structure_datarecord] = ACTIONS(2077),
    [sym_data_structure_datafield] = ACTIONS(2077),
    [sym_data_max_content_field_count] = ACTIONS(2077),
    [sym_data_max_metadata_field_count] = ACTIONS(2077),
    [sym_data_max_id_length] = ACTIONS(2077),
    [sym_data_max_name_length] = ACTIONS(2077),
    [sym_data_max_description_length] = ACTIONS(2077),
    [sym_data_max_type_length] = ACTIONS(2077),
    [sym_data_max_format_length] = ACTIONS(2077),
    [sym_data_max_source_length] = ACTIONS(2077),
    [sym_data_max_value_length] = ACTIONS(2077),
    [sym_data_max_label_length] = ACTIONS(2077),
    [sym_data_max_url_length] = ACTIONS(2077),
    [sym_max_last_login_info_length] = ACTIONS(2077),
    [sym_data_type_unknown] = ACTIONS(2077),
    [sym_data_type_string] = ACTIONS(2077),
    [sym_data_type_datetime] = ACTIONS(2077),
    [sym_data_type_date] = ACTIONS(2077),
    [sym_data_type_time] = ACTIONS(2077),
    [sym_data_type_image] = ACTIONS(2077),
    [sym_data_format_url] = ACTIONS(2077),
    [sym_data_format_phone] = ACTIONS(2077),
    [sym_data_format_email] = ACTIONS(2077),
    [sym_data_format_iso8601] = ACTIONS(2077),
    [sym_auditlog_priv] = ACTIONS(2077),
    [sym_configuration_priv] = ACTIONS(2077),
    [sym_deviceconfig_priv] = ACTIONS(2077),
    [sym_ftp_priv] = ACTIONS(2077),
    [sym_http_priv] = ACTIONS(2077),
    [sym_networkconfig_priv] = ACTIONS(2077),
    [sym_programport_priv] = ACTIONS(2077),
    [sym_remoteui_priv] = ACTIONS(2077),
    [sym_securitycontrol_priv] = ACTIONS(2077),
    [sym_softwaremanagement_priv] = ACTIONS(2077),
    [sym_terminal_priv] = ACTIONS(2077),
    [sym_tpadmin_priv] = ACTIONS(2077),
    [sym_user1_priv] = ACTIONS(2077),
    [sym_user2_priv] = ACTIONS(2077),
    [sym_user3_priv] = ACTIONS(2077),
    [sym_user4_priv] = ACTIONS(2077),
    [sym_usermanagement_priv] = ACTIONS(2077),
    [sym_valid_account] = ACTIONS(2077),
    [sym_err_name_invalid] = ACTIONS(2077),
    [sym_err_password_invalid] = ACTIONS(2077),
    [sym_err_invalid_account] = ACTIONS(2077),
    [sym_err_no_authorization] = ACTIONS(2077),
    [sym_err_invalid_parameter] = ACTIONS(2077),
    [sym_err_user_locked_out] = ACTIONS(2077),
    [sym_err_user_account_expired] = ACTIONS(2077),
    [sym_err_general_error] = ACTIONS(2077),
    [sym_err_authentication_error] = ACTIONS(2077),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2077),
    [sym_tls_validate_certificate] = ACTIONS(2077),
    [sym_tdata] = ACTIONS(2077),
    [sym_tchannel] = ACTIONS(2077),
    [sym_tlevel] = ACTIONS(2077),
    [sym_tbutton] = ACTIONS(2077),
    [sym_ttimeline] = ACTIONS(2077),
    [sym_tcustom] = ACTIONS(2077),
    [sym_url_struct] = ACTIONS(2077),
    [sym_dns_struct] = ACTIONS(2077),
    [sym_ip_address_struct] = ACTIONS(2077),
    [sym_dev_info_struct] = ACTIONS(2077),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2077),
    [sym_data_feed] = ACTIONS(2077),
    [sym_data_field] = ACTIONS(2077),
    [sym_data_record] = ACTIONS(2077),
    [sym_wc_data_feed] = ACTIONS(2077),
    [sym_wc_data_field] = ACTIONS(2077),
    [sym_wc_data_record] = ACTIONS(2077),
    [sym_last_login_info] = ACTIONS(2077),
    [sym____reserved___] = ACTIONS(2077),
    [sym_date] = ACTIONS(2077),
    [sym_day] = ACTIONS(2077),
    [sym_time] = ACTIONS(2077),
    [sym_ldate] = ACTIONS(2077),
    [sym_push_device] = ACTIONS(2077),
    [sym_push_channel] = ACTIONS(2077),
    [sym_push_devchan] = ACTIONS(2077),
    [sym_release_device] = ACTIONS(2077),
    [sym_release_channel] = ACTIONS(2077),
    [sym_release_devchan] = ACTIONS(2077),
    [sym_master_slot] = ACTIONS(2077),
    [sym_get_pulse_time] = ACTIONS(2077),
    [sym_get_timer] = ACTIONS(2077),
    [sym_master_sn] = ACTIONS(2077),
    [sym_system_number] = ACTIONS(2077),
    [sym_button] = ACTIONS(2077),
    [sym_dv_channel] = ACTIONS(2077),
    [sym_data] = ACTIONS(2077),
    [sym_level] = ACTIONS(2077),
    [sym_timeline] = ACTIONS(2077),
    [sym_channel] = ACTIONS(2077),
    [sym_custom] = ACTIONS(2077),
    [sym_atoi] = ACTIONS(2077),
    [sym_atol] = ACTIONS(2077),
    [sym_atof] = ACTIONS(2077),
    [sym_itoa] = ACTIONS(2077),
    [sym_itohex] = ACTIONS(2077),
    [sym_hextoi] = ACTIONS(2077),
    [sym_ftoa] = ACTIONS(2077),
    [sym_find_string] = ACTIONS(2077),
    [sym_left_string] = ACTIONS(2077),
    [sym_right_string] = ACTIONS(2077),
    [sym_mid_string] = ACTIONS(2077),
    [sym_remove_string] = ACTIONS(2077),
    [sym_get_buffer_string] = ACTIONS(2077),
    [sym_get_buffer_char] = ACTIONS(2077),
    [sym_get_multi_buffer_string] = ACTIONS(2077),
    [sym_length_string] = ACTIONS(2077),
    [sym_max_length_string] = ACTIONS(2077),
    [sym_set_length_string] = ACTIONS(2077),
    [sym_compare_string] = ACTIONS(2077),
    [sym_upper_string] = ACTIONS(2077),
    [sym_lower_string] = ACTIONS(2077),
    [sym_get_last] = ACTIONS(2077),
    [sym_date_to_day] = ACTIONS(2077),
    [sym_date_to_month] = ACTIONS(2077),
    [sym_date_to_year] = ACTIONS(2077),
    [sym_time_to_hour] = ACTIONS(2077),
    [sym_time_to_minute] = ACTIONS(2077),
    [sym_time_to_second] = ACTIONS(2077),
    [sym_day_of_week] = ACTIONS(2077),
    [sym_set_pulse_time] = ACTIONS(2077),
    [sym_set_timer] = ACTIONS(2077),
    [sym_file_open] = ACTIONS(2077),
    [sym_file_close] = ACTIONS(2077),
    [sym_file_read] = ACTIONS(2077),
    [sym_file_read_line] = ACTIONS(2077),
    [sym_file_write] = ACTIONS(2077),
    [sym_file_write_line] = ACTIONS(2077),
    [sym_file_copy] = ACTIONS(2077),
    [sym_file_delete] = ACTIONS(2077),
    [sym_file_rename] = ACTIONS(2077),
    [sym_file_dir] = ACTIONS(2077),
    [sym_file_getdir] = ACTIONS(2077),
    [sym_file_setdir] = ACTIONS(2077),
    [sym_file_create_dir] = ACTIONS(2077),
    [sym_file_remove_dir] = ACTIONS(2077),
    [sym_file_seek] = ACTIONS(2077),
    [sym_length_array] = ACTIONS(2077),
    [sym_max_length_array] = ACTIONS(2077),
    [sym_set_length_array] = ACTIONS(2077),
    [sym_ip_client_open] = ACTIONS(2077),
    [sym_ip_client_close] = ACTIONS(2077),
    [sym_ip_server_open] = ACTIONS(2077),
    [sym_ip_server_close] = ACTIONS(2077),
    [sym_ip_bound_client_open] = ACTIONS(2077),
    [sym_ip_mc_server_open] = ACTIONS(2077),
    [sym_ip_set_option] = ACTIONS(2077),
    [sym_ssh_client_open] = ACTIONS(2077),
    [sym_ssh_client_close] = ACTIONS(2077),
    [sym_tls_client_open] = ACTIONS(2077),
    [sym_tls_client_close] = ACTIONS(2077),
    [sym_do_push] = ACTIONS(2077),
    [sym_do_push_timed] = ACTIONS(2077),
    [sym_do_release] = ACTIONS(2077),
    [sym_device_id] = ACTIONS(2077),
    [sym_device_info] = ACTIONS(2077),
    [sym_device_id_string] = ACTIONS(2077),
    [sym_device_standby] = ACTIONS(2077),
    [sym_device_wake] = ACTIONS(2077),
    [sym_timeline_create] = ACTIONS(2077),
    [sym_timeline_kill] = ACTIONS(2077),
    [sym_timeline_reload] = ACTIONS(2077),
    [sym_timeline_pause] = ACTIONS(2077),
    [sym_timeline_restart] = ACTIONS(2077),
    [sym_timeline_set] = ACTIONS(2077),
    [sym_timeline_get] = ACTIONS(2077),
    [sym_timeline_active] = ACTIONS(2077),
    [sym_timeline_dynamic_id] = ACTIONS(2077),
    [sym_variable_to_string] = ACTIONS(2077),
    [sym_string_to_variable] = ACTIONS(2077),
    [sym_length_variable_to_string] = ACTIONS(2077),
    [sym_type_cast] = ACTIONS(2077),
    [sym_variable_to_xml] = ACTIONS(2077),
    [sym_xml_to_variable] = ACTIONS(2077),
    [sym_length_variable_to_xml] = ACTIONS(2077),
    [sym_redirect_string] = ACTIONS(2077),
    [sym_random_number] = ACTIONS(2077),
    [sym_get_unique_id] = ACTIONS(2077),
    [sym_get_system_number] = ACTIONS(2077),
    [sym_set_system_number] = ACTIONS(2077),
    [sym_get_serial_number] = ACTIONS(2077),
    [sym_get_master_build] = ACTIONS(2077),
    [sym_set_validation_code] = ACTIONS(2077),
    [sym_reboot] = ACTIONS(2077),
    [sym_load_duet_module] = ACTIONS(2077),
    [sym_unload_duet_module] = ACTIONS(2077),
    [sym_duet_mem_size_set] = ACTIONS(2077),
    [sym_duet_mem_size_get] = ACTIONS(2077),
    [sym_dynamic_application_device] = ACTIONS(2077),
    [sym_dynamic_polled_port] = ACTIONS(2077),
    [sym_static_port_binding] = ACTIONS(2077),
    [sym_static_ip_binding] = ACTIONS(2077),
    [sym_get_dns_list] = ACTIONS(2077),
    [sym_set_dns_list] = ACTIONS(2077),
    [sym_get_ip_address] = ACTIONS(2077),
    [sym_set_ip_address] = ACTIONS(2077),
    [sym_get_url_list] = ACTIONS(2077),
    [sym_add_url_entry] = ACTIONS(2077),
    [sym_delete_url_entry] = ACTIONS(2077),
    [sym_abs_value] = ACTIONS(2077),
    [sym_max_value] = ACTIONS(2077),
    [sym_min_value] = ACTIONS(2077),
    [sym_exp_value] = ACTIONS(2077),
    [sym_log_value] = ACTIONS(2077),
    [sym_log10_value] = ACTIONS(2077),
    [sym_power_value] = ACTIONS(2077),
    [sym_sqrt_value] = ACTIONS(2077),
    [sym_raw_be] = ACTIONS(2077),
    [sym_raw_le] = ACTIONS(2077),
    [sym_internal_queue_size_set] = ACTIONS(2077),
    [sym_internal_queue_size_get] = ACTIONS(2077),
    [sym_internal_threshold_set] = ACTIONS(2077),
    [sym_internal_threshold_get] = ACTIONS(2077),
    [sym_rebuilt_event] = ACTIONS(2077),
    [sym_do_custom_event] = ACTIONS(2077),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2077),
    [sym_clkmgr_set_clk_source] = ACTIONS(2077),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2077),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2077),
    [sym_clkmgr_get_timezone] = ACTIONS(2077),
    [sym_clkmgr_set_timezone] = ACTIONS(2077),
    [sym_clkmgr_get_resync_period] = ACTIONS(2077),
    [sym_clkmgr_set_resync_period] = ACTIONS(2077),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2077),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2077),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2077),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2077),
    [sym_clkmgr_get_timeservers] = ACTIONS(2077),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2077),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2077),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2077),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2077),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2077),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2077),
    [sym_set_log_level] = ACTIONS(2077),
    [sym_get_log_level] = ACTIONS(2077),
    [sym_amx_log] = ACTIONS(2077),
    [sym_smtp_server_config_set] = ACTIONS(2077),
    [sym_smtp_server_config_get] = ACTIONS(2077),
    [sym_smtp_send] = ACTIONS(2077),
    [sym_validate_netlinx_account] = ACTIONS(2077),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2077),
    [sym_authenticate_certificate] = ACTIONS(2077),
    [sym_audit_netlinx_session_event] = ACTIONS(2077),
    [sym_audit_netlinx_generic_event] = ACTIONS(2077),
    [sym_get_available_flash_disk_space] = ACTIONS(2077),
    [sym_get_max_flash_disk_space] = ACTIONS(2077),
    [sym_data_create_feed] = ACTIONS(2077),
    [sym_data_delete_feed] = ACTIONS(2077),
    [sym_data_publish_feed] = ACTIONS(2077),
    [sym_data_get_published_feed] = ACTIONS(2077),
    [sym_data_add_record] = ACTIONS(2077),
    [sym_data_get_event_record] = ACTIONS(2077),
    [sym__wc_data_create_feed] = ACTIONS(2077),
    [sym__wc_data_add_record] = ACTIONS(2077),
    [sym__wc_data_get_event_record] = ACTIONS(2077),
    [sym_format] = ACTIONS(2077),
    [sym_astro_clock] = ACTIONS(2077),
    [sym_set_outdoor_temperature] = ACTIONS(2077),
    [sym_set_virtual_level_count] = ACTIONS(2077),
    [sym_set_virtual_channel_count] = ACTIONS(2077),
    [sym_set_virtual_port_count] = ACTIONS(2077),
    [sym__wc_encode_int] = ACTIONS(2077),
    [sym__wc_decode_int] = ACTIONS(2077),
    [sym__wc_int] = ACTIONS(2077),
    [sym__wc_to_ch_int] = ACTIONS(2077),
    [sym__ch_to_wc_int] = ACTIONS(2077),
    [sym__wc_find_string_int] = ACTIONS(2077),
    [sym__wc_left_string_int] = ACTIONS(2077),
    [sym__wc_lower_string_int] = ACTIONS(2077),
    [sym__wc_mid_string_int] = ACTIONS(2077),
    [sym__wc_remove_string_int] = ACTIONS(2077),
    [sym__wc_right_string_int] = ACTIONS(2077),
    [sym__wc_upper_string_int] = ACTIONS(2077),
    [sym__wc_compare_string_int] = ACTIONS(2077),
    [sym__wc_get_buffer_char_int] = ACTIONS(2077),
    [sym__wc_get_buffer_string_int] = ACTIONS(2077),
    [sym__wc_concat_string_int] = ACTIONS(2077),
    [sym__wc_file_open_int] = ACTIONS(2077),
    [sym__wc_file_close_int] = ACTIONS(2077),
    [sym__wc_file_read_int] = ACTIONS(2077),
    [sym__wc_file_read_line_int] = ACTIONS(2077),
    [sym__wc_file_write_int] = ACTIONS(2077),
    [sym__wc_file_write_line_int] = ACTIONS(2077),
    [aux_sym_preproc_include_token1] = ACTIONS(2077),
    [aux_sym_preproc_define_token1] = ACTIONS(2079),
    [aux_sym_preproc_warn_token1] = ACTIONS(2079),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2079),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2079),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2079),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2079),
    [aux_sym_preproc_else_token1] = ACTIONS(2079),
    [anon_sym_LBRACK] = ACTIONS(2079),
    [anon_sym_LBRACE] = ACTIONS(2079),
    [anon_sym_LPAREN] = ACTIONS(2077),
    [anon_sym_BANG] = ACTIONS(2079),
    [anon_sym_TILDE] = ACTIONS(2079),
    [anon_sym_DASH] = ACTIONS(2077),
    [anon_sym_PLUS] = ACTIONS(2077),
    [anon_sym_DASH_DASH] = ACTIONS(2079),
    [anon_sym_PLUS_PLUS] = ACTIONS(2079),
    [anon_sym_DQUOTE] = ACTIONS(2079),
    [anon_sym_SQUOTE] = ACTIONS(2079),
    [sym_number_literal] = ACTIONS(2079),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(364)] = {
    [sym_identifier] = ACTIONS(2081),
    [sym_program_name_keyword] = ACTIONS(2081),
    [sym_module_name_keyword] = ACTIONS(2081),
    [sym_define_device_keyword] = ACTIONS(2081),
    [sym_define_combine_keyword] = ACTIONS(2081),
    [sym_define_constant_keyword] = ACTIONS(2081),
    [sym_define_type_keyword] = ACTIONS(2081),
    [sym_define_variable_keyword] = ACTIONS(2081),
    [sym_define_system_variable_keyword] = ACTIONS(2081),
    [sym_define_connect_level_keyword] = ACTIONS(2081),
    [sym_define_latching_keyword] = ACTIONS(2081),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2081),
    [sym_define_toggling_keyword] = ACTIONS(2081),
    [sym_define_call_keyword] = ACTIONS(2081),
    [sym_define_function_keyword] = ACTIONS(2081),
    [sym_define_library_function_keyword] = ACTIONS(2081),
    [sym_define_start_keyword] = ACTIONS(2081),
    [sym_define_event_keyword] = ACTIONS(2081),
    [sym_define_program_keyword] = ACTIONS(2081),
    [sym_define_module_keyword] = ACTIONS(2081),
    [sym_char_keyword] = ACTIONS(2081),
    [sym_widechar_keyword] = ACTIONS(2081),
    [sym_integer_keyword] = ACTIONS(2081),
    [sym_sinteger_keyword] = ACTIONS(2081),
    [sym_long_keyword] = ACTIONS(2081),
    [sym_slong_keyword] = ACTIONS(2081),
    [sym_float_keyword] = ACTIONS(2081),
    [sym_double_keyword] = ACTIONS(2081),
    [sym_variant_keyword] = ACTIONS(2081),
    [sym_variantarray_keyword] = ACTIONS(2081),
    [sym_local_var_keyword] = ACTIONS(2081),
    [sym_stack_var_keyword] = ACTIONS(2081),
    [sym_constant_keyword] = ACTIONS(2081),
    [sym_volatile_keyword] = ACTIONS(2081),
    [sym_non_volatile_keyword] = ACTIONS(2081),
    [sym_persistent_keyword] = ACTIONS(2081),
    [sym_dev_keyword] = ACTIONS(2081),
    [sym_devlev_keyword] = ACTIONS(2081),
    [sym_devchan_keyword] = ACTIONS(2081),
    [sym_if_keyword] = ACTIONS(2081),
    [sym_switch_keyword] = ACTIONS(2081),
    [sym_case_keyword] = ACTIONS(2081),
    [sym_default_keyword] = ACTIONS(2081),
    [sym_while_keyword] = ACTIONS(2081),
    [sym_for_keyword] = ACTIONS(2081),
    [sym_break_keyword] = ACTIONS(2081),
    [sym_continue_keyword] = ACTIONS(2081),
    [sym_return_keyword] = ACTIONS(2081),
    [sym_select_keyword] = ACTIONS(2081),
    [sym_active_keyword] = ACTIONS(2081),
    [sym_struct_keyword] = ACTIONS(2081),
    [sym_structure_keyword] = ACTIONS(2081),
    [sym_button_event_keyword] = ACTIONS(2081),
    [sym_channel_event_keyword] = ACTIONS(2081),
    [sym_level_event_keyword] = ACTIONS(2081),
    [sym_data_event_keyword] = ACTIONS(2081),
    [sym_timeline_event_keyword] = ACTIONS(2081),
    [sym_custom_event_keyword] = ACTIONS(2081),
    [sym_push_keyword] = ACTIONS(2081),
    [sym_release_keyword] = ACTIONS(2081),
    [aux_sym_on_keyword_token1] = ACTIONS(2081),
    [aux_sym_off_keyword_token1] = ACTIONS(2081),
    [sym_send_level_keyword] = ACTIONS(2081),
    [sym_send_string_keyword] = ACTIONS(2081),
    [sym_send_command_keyword] = ACTIONS(2081),
    [sym_clear_buffer_keyword] = ACTIONS(2081),
    [sym_create_buffer_keyword] = ACTIONS(2081),
    [sym_create_multi_buffer_keyword] = ACTIONS(2081),
    [sym_call_keyword] = ACTIONS(2081),
    [sym_system_call_keyword] = ACTIONS(2081),
    [sym_devchan_to_keyword] = ACTIONS(2081),
    [sym_devchan_min_to_keyword] = ACTIONS(2081),
    [sym_devchan_total_off_keyword] = ACTIONS(2081),
    [sym_devchan_pulse_keyword] = ACTIONS(2081),
    [sym_wait_keyword] = ACTIONS(2081),
    [sym_cancel_wait_keyword] = ACTIONS(2081),
    [sym_wait_until_keyword] = ACTIONS(2081),
    [sym_cancel_wait_until_keyword] = ACTIONS(2081),
    [sym_cancel_all_wait_keyword] = ACTIONS(2081),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2081),
    [sym_not] = ACTIONS(2081),
    [sym_bnot] = ACTIONS(2081),
    [sym___file__] = ACTIONS(2081),
    [sym___date__] = ACTIONS(2081),
    [sym___ldate__] = ACTIONS(2081),
    [sym___time__] = ACTIONS(2081),
    [sym___line__] = ACTIONS(2081),
    [sym___name__] = ACTIONS(2081),
    [sym___netlinx__] = ACTIONS(2081),
    [sym_netlinx_axi_version] = ACTIONS(2081),
    [sym_url_flg_tcp] = ACTIONS(2081),
    [sym_url_flg_accinfopresent] = ACTIONS(2081),
    [sym_url_flg_temp] = ACTIONS(2081),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2081),
    [sym_url_flg_stat_mask] = ACTIONS(2081),
    [sym_url_flg_stat_lookup] = ACTIONS(2081),
    [sym_url_flg_stat_connecting] = ACTIONS(2081),
    [sym_url_flg_stat_waiting] = ACTIONS(2081),
    [sym_url_flg_stat_connected] = ACTIONS(2081),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2081),
    [sym_first_virtual_device] = ACTIONS(2081),
    [sym_first_local_port] = ACTIONS(2081),
    [sym_dynamic_virtual_device] = ACTIONS(2081),
    [sym_timeline_once] = ACTIONS(2081),
    [sym_timeline_repeat] = ACTIONS(2081),
    [sym_timeline_absolute] = ACTIONS(2081),
    [sym_timeline_relative] = ACTIONS(2081),
    [sym_do_push_timed_infinite] = ACTIONS(2081),
    [sym_source_type_no_address] = ACTIONS(2081),
    [sym_source_type_neuron_id] = ACTIONS(2081),
    [sym_source_type_ip_address] = ACTIONS(2081),
    [sym_source_type_axlink] = ACTIONS(2081),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2081),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2081),
    [sym_source_type_ip_socket_address] = ACTIONS(2081),
    [sym_source_type_rs232] = ACTIONS(2081),
    [sym_source_type_internal] = ACTIONS(2081),
    [sym_source_type_mac_address] = ACTIONS(2081),
    [sym_source_type_ipv4_port] = ACTIONS(2081),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2081),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2081),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2081),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2081),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2081),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2081),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2081),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2081),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2081),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2081),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2081),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2081),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2081),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2081),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2081),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2081),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2081),
    [sym_true] = ACTIONS(2081),
    [sym_false] = ACTIONS(2081),
    [sym_file_read_only] = ACTIONS(2081),
    [sym_file_rw_new] = ACTIONS(2081),
    [sym_file_rw_append] = ACTIONS(2081),
    [sym_ip_tcp] = ACTIONS(2081),
    [sym_ip_udp] = ACTIONS(2081),
    [sym_ip_udp_2way] = ACTIONS(2081),
    [sym_xml_encode_types] = ACTIONS(2081),
    [sym_xml_encode_char_as_list] = ACTIONS(2081),
    [sym_xml_encode_le] = ACTIONS(2081),
    [sym_xml_decode_types] = ACTIONS(2081),
    [sym_xml_decode_no_preserve] = ACTIONS(2081),
    [sym_duet_dev_type_display_device] = ACTIONS(2081),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2081),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2081),
    [sym_duet_dev_type_io_device] = ACTIONS(2081),
    [sym_duet_dev_type_relay_device] = ACTIONS(2081),
    [sym_duet_dev_type_ups] = ACTIONS(2081),
    [sym_duet_dev_type_amplifier] = ACTIONS(2081),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2081),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2081),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2081),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2081),
    [sym_duet_dev_type_camera] = ACTIONS(2081),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2081),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2081),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2081),
    [sym_duet_dev_type_dss] = ACTIONS(2081),
    [sym_duet_dev_type_dvr] = ACTIONS(2081),
    [sym_duet_dev_type_disc_device] = ACTIONS(2081),
    [sym_duet_dev_type_document_camera] = ACTIONS(2081),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2081),
    [sym_duet_dev_type_hvac] = ACTIONS(2081),
    [sym_duet_dev_type_keypad] = ACTIONS(2081),
    [sym_duet_dev_type_light] = ACTIONS(2081),
    [sym_duet_dev_type_monitor] = ACTIONS(2081),
    [sym_duet_dev_type_motor] = ACTIONS(2081),
    [sym_duet_dev_type_multi_window] = ACTIONS(2081),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2081),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2081),
    [sym_duet_dev_type_receiver] = ACTIONS(2081),
    [sym_duet_dev_type_security_system] = ACTIONS(2081),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2081),
    [sym_duet_dev_type_settop_box] = ACTIONS(2081),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2081),
    [sym_duet_dev_type_switcher] = ACTIONS(2081),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2081),
    [sym_duet_dev_type_tv] = ACTIONS(2081),
    [sym_duet_dev_type_utility] = ACTIONS(2081),
    [sym_duet_dev_type_vcr] = ACTIONS(2081),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2081),
    [sym_duet_dev_type_video_processor] = ACTIONS(2081),
    [sym_duet_dev_type_video_projector] = ACTIONS(2081),
    [sym_duet_dev_type_video_wall] = ACTIONS(2081),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2081),
    [sym_duet_dev_type_weather] = ACTIONS(2081),
    [sym_duet_dev_not_polled] = ACTIONS(2081),
    [sym_duet_dev_polled] = ACTIONS(2081),
    [sym_clkmgr_mode_network] = ACTIONS(2081),
    [sym_clkmgr_mode_standalone] = ACTIONS(2081),
    [sym_normal_standby] = ACTIONS(2081),
    [sym_normal_wake] = ACTIONS(2081),
    [sym_ip_multicast_ttl_option] = ACTIONS(2081),
    [sym_ip_ttl_subnet] = ACTIONS(2081),
    [sym_ip_ttl_site] = ACTIONS(2081),
    [sym_ip_ttl_region] = ACTIONS(2081),
    [sym_ip_ttl_continent] = ACTIONS(2081),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2081),
    [sym_ip_nodelay_on] = ACTIONS(2081),
    [sym_ip_nodelay_off] = ACTIONS(2081),
    [sym_amx_error] = ACTIONS(2081),
    [sym_amx_warning] = ACTIONS(2081),
    [sym_amx_info] = ACTIONS(2081),
    [sym_amx_debug] = ACTIONS(2081),
    [sym_netlinx_login_success] = ACTIONS(2081),
    [sym_netlinx_login_fail] = ACTIONS(2081),
    [sym_netlinx_logout] = ACTIONS(2081),
    [sym_null_str] = ACTIONS(2081),
    [sym_smtp_address] = ACTIONS(2081),
    [sym_smtp_port_number] = ACTIONS(2081),
    [sym_smtp_username] = ACTIONS(2081),
    [sym_smtp_password] = ACTIONS(2081),
    [sym_smtp_require_tls] = ACTIONS(2081),
    [sym_smtp_from] = ACTIONS(2081),
    [sym_smtp_tls_true] = ACTIONS(2081),
    [sym_smtp_tls_false] = ACTIONS(2081),
    [sym_listview_on_row_select_event] = ACTIONS(2081),
    [sym_data_structure_unknown] = ACTIONS(2081),
    [sym_data_structure_datafeed] = ACTIONS(2081),
    [sym_data_structure_datarecordset] = ACTIONS(2081),
    [sym_data_structure_datarecord] = ACTIONS(2081),
    [sym_data_structure_datafield] = ACTIONS(2081),
    [sym_data_max_content_field_count] = ACTIONS(2081),
    [sym_data_max_metadata_field_count] = ACTIONS(2081),
    [sym_data_max_id_length] = ACTIONS(2081),
    [sym_data_max_name_length] = ACTIONS(2081),
    [sym_data_max_description_length] = ACTIONS(2081),
    [sym_data_max_type_length] = ACTIONS(2081),
    [sym_data_max_format_length] = ACTIONS(2081),
    [sym_data_max_source_length] = ACTIONS(2081),
    [sym_data_max_value_length] = ACTIONS(2081),
    [sym_data_max_label_length] = ACTIONS(2081),
    [sym_data_max_url_length] = ACTIONS(2081),
    [sym_max_last_login_info_length] = ACTIONS(2081),
    [sym_data_type_unknown] = ACTIONS(2081),
    [sym_data_type_string] = ACTIONS(2081),
    [sym_data_type_datetime] = ACTIONS(2081),
    [sym_data_type_date] = ACTIONS(2081),
    [sym_data_type_time] = ACTIONS(2081),
    [sym_data_type_image] = ACTIONS(2081),
    [sym_data_format_url] = ACTIONS(2081),
    [sym_data_format_phone] = ACTIONS(2081),
    [sym_data_format_email] = ACTIONS(2081),
    [sym_data_format_iso8601] = ACTIONS(2081),
    [sym_auditlog_priv] = ACTIONS(2081),
    [sym_configuration_priv] = ACTIONS(2081),
    [sym_deviceconfig_priv] = ACTIONS(2081),
    [sym_ftp_priv] = ACTIONS(2081),
    [sym_http_priv] = ACTIONS(2081),
    [sym_networkconfig_priv] = ACTIONS(2081),
    [sym_programport_priv] = ACTIONS(2081),
    [sym_remoteui_priv] = ACTIONS(2081),
    [sym_securitycontrol_priv] = ACTIONS(2081),
    [sym_softwaremanagement_priv] = ACTIONS(2081),
    [sym_terminal_priv] = ACTIONS(2081),
    [sym_tpadmin_priv] = ACTIONS(2081),
    [sym_user1_priv] = ACTIONS(2081),
    [sym_user2_priv] = ACTIONS(2081),
    [sym_user3_priv] = ACTIONS(2081),
    [sym_user4_priv] = ACTIONS(2081),
    [sym_usermanagement_priv] = ACTIONS(2081),
    [sym_valid_account] = ACTIONS(2081),
    [sym_err_name_invalid] = ACTIONS(2081),
    [sym_err_password_invalid] = ACTIONS(2081),
    [sym_err_invalid_account] = ACTIONS(2081),
    [sym_err_no_authorization] = ACTIONS(2081),
    [sym_err_invalid_parameter] = ACTIONS(2081),
    [sym_err_user_locked_out] = ACTIONS(2081),
    [sym_err_user_account_expired] = ACTIONS(2081),
    [sym_err_general_error] = ACTIONS(2081),
    [sym_err_authentication_error] = ACTIONS(2081),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2081),
    [sym_tls_validate_certificate] = ACTIONS(2081),
    [sym_tdata] = ACTIONS(2081),
    [sym_tchannel] = ACTIONS(2081),
    [sym_tlevel] = ACTIONS(2081),
    [sym_tbutton] = ACTIONS(2081),
    [sym_ttimeline] = ACTIONS(2081),
    [sym_tcustom] = ACTIONS(2081),
    [sym_url_struct] = ACTIONS(2081),
    [sym_dns_struct] = ACTIONS(2081),
    [sym_ip_address_struct] = ACTIONS(2081),
    [sym_dev_info_struct] = ACTIONS(2081),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2081),
    [sym_data_feed] = ACTIONS(2081),
    [sym_data_field] = ACTIONS(2081),
    [sym_data_record] = ACTIONS(2081),
    [sym_wc_data_feed] = ACTIONS(2081),
    [sym_wc_data_field] = ACTIONS(2081),
    [sym_wc_data_record] = ACTIONS(2081),
    [sym_last_login_info] = ACTIONS(2081),
    [sym____reserved___] = ACTIONS(2081),
    [sym_date] = ACTIONS(2081),
    [sym_day] = ACTIONS(2081),
    [sym_time] = ACTIONS(2081),
    [sym_ldate] = ACTIONS(2081),
    [sym_push_device] = ACTIONS(2081),
    [sym_push_channel] = ACTIONS(2081),
    [sym_push_devchan] = ACTIONS(2081),
    [sym_release_device] = ACTIONS(2081),
    [sym_release_channel] = ACTIONS(2081),
    [sym_release_devchan] = ACTIONS(2081),
    [sym_master_slot] = ACTIONS(2081),
    [sym_get_pulse_time] = ACTIONS(2081),
    [sym_get_timer] = ACTIONS(2081),
    [sym_master_sn] = ACTIONS(2081),
    [sym_system_number] = ACTIONS(2081),
    [sym_button] = ACTIONS(2081),
    [sym_dv_channel] = ACTIONS(2081),
    [sym_data] = ACTIONS(2081),
    [sym_level] = ACTIONS(2081),
    [sym_timeline] = ACTIONS(2081),
    [sym_channel] = ACTIONS(2081),
    [sym_custom] = ACTIONS(2081),
    [sym_atoi] = ACTIONS(2081),
    [sym_atol] = ACTIONS(2081),
    [sym_atof] = ACTIONS(2081),
    [sym_itoa] = ACTIONS(2081),
    [sym_itohex] = ACTIONS(2081),
    [sym_hextoi] = ACTIONS(2081),
    [sym_ftoa] = ACTIONS(2081),
    [sym_find_string] = ACTIONS(2081),
    [sym_left_string] = ACTIONS(2081),
    [sym_right_string] = ACTIONS(2081),
    [sym_mid_string] = ACTIONS(2081),
    [sym_remove_string] = ACTIONS(2081),
    [sym_get_buffer_string] = ACTIONS(2081),
    [sym_get_buffer_char] = ACTIONS(2081),
    [sym_get_multi_buffer_string] = ACTIONS(2081),
    [sym_length_string] = ACTIONS(2081),
    [sym_max_length_string] = ACTIONS(2081),
    [sym_set_length_string] = ACTIONS(2081),
    [sym_compare_string] = ACTIONS(2081),
    [sym_upper_string] = ACTIONS(2081),
    [sym_lower_string] = ACTIONS(2081),
    [sym_get_last] = ACTIONS(2081),
    [sym_date_to_day] = ACTIONS(2081),
    [sym_date_to_month] = ACTIONS(2081),
    [sym_date_to_year] = ACTIONS(2081),
    [sym_time_to_hour] = ACTIONS(2081),
    [sym_time_to_minute] = ACTIONS(2081),
    [sym_time_to_second] = ACTIONS(2081),
    [sym_day_of_week] = ACTIONS(2081),
    [sym_set_pulse_time] = ACTIONS(2081),
    [sym_set_timer] = ACTIONS(2081),
    [sym_file_open] = ACTIONS(2081),
    [sym_file_close] = ACTIONS(2081),
    [sym_file_read] = ACTIONS(2081),
    [sym_file_read_line] = ACTIONS(2081),
    [sym_file_write] = ACTIONS(2081),
    [sym_file_write_line] = ACTIONS(2081),
    [sym_file_copy] = ACTIONS(2081),
    [sym_file_delete] = ACTIONS(2081),
    [sym_file_rename] = ACTIONS(2081),
    [sym_file_dir] = ACTIONS(2081),
    [sym_file_getdir] = ACTIONS(2081),
    [sym_file_setdir] = ACTIONS(2081),
    [sym_file_create_dir] = ACTIONS(2081),
    [sym_file_remove_dir] = ACTIONS(2081),
    [sym_file_seek] = ACTIONS(2081),
    [sym_length_array] = ACTIONS(2081),
    [sym_max_length_array] = ACTIONS(2081),
    [sym_set_length_array] = ACTIONS(2081),
    [sym_ip_client_open] = ACTIONS(2081),
    [sym_ip_client_close] = ACTIONS(2081),
    [sym_ip_server_open] = ACTIONS(2081),
    [sym_ip_server_close] = ACTIONS(2081),
    [sym_ip_bound_client_open] = ACTIONS(2081),
    [sym_ip_mc_server_open] = ACTIONS(2081),
    [sym_ip_set_option] = ACTIONS(2081),
    [sym_ssh_client_open] = ACTIONS(2081),
    [sym_ssh_client_close] = ACTIONS(2081),
    [sym_tls_client_open] = ACTIONS(2081),
    [sym_tls_client_close] = ACTIONS(2081),
    [sym_do_push] = ACTIONS(2081),
    [sym_do_push_timed] = ACTIONS(2081),
    [sym_do_release] = ACTIONS(2081),
    [sym_device_id] = ACTIONS(2081),
    [sym_device_info] = ACTIONS(2081),
    [sym_device_id_string] = ACTIONS(2081),
    [sym_device_standby] = ACTIONS(2081),
    [sym_device_wake] = ACTIONS(2081),
    [sym_timeline_create] = ACTIONS(2081),
    [sym_timeline_kill] = ACTIONS(2081),
    [sym_timeline_reload] = ACTIONS(2081),
    [sym_timeline_pause] = ACTIONS(2081),
    [sym_timeline_restart] = ACTIONS(2081),
    [sym_timeline_set] = ACTIONS(2081),
    [sym_timeline_get] = ACTIONS(2081),
    [sym_timeline_active] = ACTIONS(2081),
    [sym_timeline_dynamic_id] = ACTIONS(2081),
    [sym_variable_to_string] = ACTIONS(2081),
    [sym_string_to_variable] = ACTIONS(2081),
    [sym_length_variable_to_string] = ACTIONS(2081),
    [sym_type_cast] = ACTIONS(2081),
    [sym_variable_to_xml] = ACTIONS(2081),
    [sym_xml_to_variable] = ACTIONS(2081),
    [sym_length_variable_to_xml] = ACTIONS(2081),
    [sym_redirect_string] = ACTIONS(2081),
    [sym_random_number] = ACTIONS(2081),
    [sym_get_unique_id] = ACTIONS(2081),
    [sym_get_system_number] = ACTIONS(2081),
    [sym_set_system_number] = ACTIONS(2081),
    [sym_get_serial_number] = ACTIONS(2081),
    [sym_get_master_build] = ACTIONS(2081),
    [sym_set_validation_code] = ACTIONS(2081),
    [sym_reboot] = ACTIONS(2081),
    [sym_load_duet_module] = ACTIONS(2081),
    [sym_unload_duet_module] = ACTIONS(2081),
    [sym_duet_mem_size_set] = ACTIONS(2081),
    [sym_duet_mem_size_get] = ACTIONS(2081),
    [sym_dynamic_application_device] = ACTIONS(2081),
    [sym_dynamic_polled_port] = ACTIONS(2081),
    [sym_static_port_binding] = ACTIONS(2081),
    [sym_static_ip_binding] = ACTIONS(2081),
    [sym_get_dns_list] = ACTIONS(2081),
    [sym_set_dns_list] = ACTIONS(2081),
    [sym_get_ip_address] = ACTIONS(2081),
    [sym_set_ip_address] = ACTIONS(2081),
    [sym_get_url_list] = ACTIONS(2081),
    [sym_add_url_entry] = ACTIONS(2081),
    [sym_delete_url_entry] = ACTIONS(2081),
    [sym_abs_value] = ACTIONS(2081),
    [sym_max_value] = ACTIONS(2081),
    [sym_min_value] = ACTIONS(2081),
    [sym_exp_value] = ACTIONS(2081),
    [sym_log_value] = ACTIONS(2081),
    [sym_log10_value] = ACTIONS(2081),
    [sym_power_value] = ACTIONS(2081),
    [sym_sqrt_value] = ACTIONS(2081),
    [sym_raw_be] = ACTIONS(2081),
    [sym_raw_le] = ACTIONS(2081),
    [sym_internal_queue_size_set] = ACTIONS(2081),
    [sym_internal_queue_size_get] = ACTIONS(2081),
    [sym_internal_threshold_set] = ACTIONS(2081),
    [sym_internal_threshold_get] = ACTIONS(2081),
    [sym_rebuilt_event] = ACTIONS(2081),
    [sym_do_custom_event] = ACTIONS(2081),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2081),
    [sym_clkmgr_set_clk_source] = ACTIONS(2081),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2081),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2081),
    [sym_clkmgr_get_timezone] = ACTIONS(2081),
    [sym_clkmgr_set_timezone] = ACTIONS(2081),
    [sym_clkmgr_get_resync_period] = ACTIONS(2081),
    [sym_clkmgr_set_resync_period] = ACTIONS(2081),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2081),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2081),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2081),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2081),
    [sym_clkmgr_get_timeservers] = ACTIONS(2081),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2081),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2081),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2081),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2081),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2081),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2081),
    [sym_set_log_level] = ACTIONS(2081),
    [sym_get_log_level] = ACTIONS(2081),
    [sym_amx_log] = ACTIONS(2081),
    [sym_smtp_server_config_set] = ACTIONS(2081),
    [sym_smtp_server_config_get] = ACTIONS(2081),
    [sym_smtp_send] = ACTIONS(2081),
    [sym_validate_netlinx_account] = ACTIONS(2081),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2081),
    [sym_authenticate_certificate] = ACTIONS(2081),
    [sym_audit_netlinx_session_event] = ACTIONS(2081),
    [sym_audit_netlinx_generic_event] = ACTIONS(2081),
    [sym_get_available_flash_disk_space] = ACTIONS(2081),
    [sym_get_max_flash_disk_space] = ACTIONS(2081),
    [sym_data_create_feed] = ACTIONS(2081),
    [sym_data_delete_feed] = ACTIONS(2081),
    [sym_data_publish_feed] = ACTIONS(2081),
    [sym_data_get_published_feed] = ACTIONS(2081),
    [sym_data_add_record] = ACTIONS(2081),
    [sym_data_get_event_record] = ACTIONS(2081),
    [sym__wc_data_create_feed] = ACTIONS(2081),
    [sym__wc_data_add_record] = ACTIONS(2081),
    [sym__wc_data_get_event_record] = ACTIONS(2081),
    [sym_format] = ACTIONS(2081),
    [sym_astro_clock] = ACTIONS(2081),
    [sym_set_outdoor_temperature] = ACTIONS(2081),
    [sym_set_virtual_level_count] = ACTIONS(2081),
    [sym_set_virtual_channel_count] = ACTIONS(2081),
    [sym_set_virtual_port_count] = ACTIONS(2081),
    [sym__wc_encode_int] = ACTIONS(2081),
    [sym__wc_decode_int] = ACTIONS(2081),
    [sym__wc_int] = ACTIONS(2081),
    [sym__wc_to_ch_int] = ACTIONS(2081),
    [sym__ch_to_wc_int] = ACTIONS(2081),
    [sym__wc_find_string_int] = ACTIONS(2081),
    [sym__wc_left_string_int] = ACTIONS(2081),
    [sym__wc_lower_string_int] = ACTIONS(2081),
    [sym__wc_mid_string_int] = ACTIONS(2081),
    [sym__wc_remove_string_int] = ACTIONS(2081),
    [sym__wc_right_string_int] = ACTIONS(2081),
    [sym__wc_upper_string_int] = ACTIONS(2081),
    [sym__wc_compare_string_int] = ACTIONS(2081),
    [sym__wc_get_buffer_char_int] = ACTIONS(2081),
    [sym__wc_get_buffer_string_int] = ACTIONS(2081),
    [sym__wc_concat_string_int] = ACTIONS(2081),
    [sym__wc_file_open_int] = ACTIONS(2081),
    [sym__wc_file_close_int] = ACTIONS(2081),
    [sym__wc_file_read_int] = ACTIONS(2081),
    [sym__wc_file_read_line_int] = ACTIONS(2081),
    [sym__wc_file_write_int] = ACTIONS(2081),
    [sym__wc_file_write_line_int] = ACTIONS(2081),
    [aux_sym_preproc_include_token1] = ACTIONS(2081),
    [aux_sym_preproc_define_token1] = ACTIONS(2083),
    [aux_sym_preproc_warn_token1] = ACTIONS(2083),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2083),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2083),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2083),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2083),
    [aux_sym_preproc_else_token1] = ACTIONS(2083),
    [anon_sym_LBRACK] = ACTIONS(2083),
    [anon_sym_LBRACE] = ACTIONS(2083),
    [anon_sym_LPAREN] = ACTIONS(2081),
    [anon_sym_BANG] = ACTIONS(2083),
    [anon_sym_TILDE] = ACTIONS(2083),
    [anon_sym_DASH] = ACTIONS(2081),
    [anon_sym_PLUS] = ACTIONS(2081),
    [anon_sym_DASH_DASH] = ACTIONS(2083),
    [anon_sym_PLUS_PLUS] = ACTIONS(2083),
    [anon_sym_DQUOTE] = ACTIONS(2083),
    [anon_sym_SQUOTE] = ACTIONS(2083),
    [sym_number_literal] = ACTIONS(2083),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(365)] = {
    [sym_identifier] = ACTIONS(2085),
    [sym_program_name_keyword] = ACTIONS(2085),
    [sym_module_name_keyword] = ACTIONS(2085),
    [sym_define_device_keyword] = ACTIONS(2085),
    [sym_define_combine_keyword] = ACTIONS(2085),
    [sym_define_constant_keyword] = ACTIONS(2085),
    [sym_define_type_keyword] = ACTIONS(2085),
    [sym_define_variable_keyword] = ACTIONS(2085),
    [sym_define_system_variable_keyword] = ACTIONS(2085),
    [sym_define_connect_level_keyword] = ACTIONS(2085),
    [sym_define_latching_keyword] = ACTIONS(2085),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2085),
    [sym_define_toggling_keyword] = ACTIONS(2085),
    [sym_define_call_keyword] = ACTIONS(2085),
    [sym_define_function_keyword] = ACTIONS(2085),
    [sym_define_library_function_keyword] = ACTIONS(2085),
    [sym_define_start_keyword] = ACTIONS(2085),
    [sym_define_event_keyword] = ACTIONS(2085),
    [sym_define_program_keyword] = ACTIONS(2085),
    [sym_define_module_keyword] = ACTIONS(2085),
    [sym_char_keyword] = ACTIONS(2085),
    [sym_widechar_keyword] = ACTIONS(2085),
    [sym_integer_keyword] = ACTIONS(2085),
    [sym_sinteger_keyword] = ACTIONS(2085),
    [sym_long_keyword] = ACTIONS(2085),
    [sym_slong_keyword] = ACTIONS(2085),
    [sym_float_keyword] = ACTIONS(2085),
    [sym_double_keyword] = ACTIONS(2085),
    [sym_variant_keyword] = ACTIONS(2085),
    [sym_variantarray_keyword] = ACTIONS(2085),
    [sym_local_var_keyword] = ACTIONS(2085),
    [sym_stack_var_keyword] = ACTIONS(2085),
    [sym_constant_keyword] = ACTIONS(2085),
    [sym_volatile_keyword] = ACTIONS(2085),
    [sym_non_volatile_keyword] = ACTIONS(2085),
    [sym_persistent_keyword] = ACTIONS(2085),
    [sym_dev_keyword] = ACTIONS(2085),
    [sym_devlev_keyword] = ACTIONS(2085),
    [sym_devchan_keyword] = ACTIONS(2085),
    [sym_if_keyword] = ACTIONS(2085),
    [sym_switch_keyword] = ACTIONS(2085),
    [sym_case_keyword] = ACTIONS(2085),
    [sym_default_keyword] = ACTIONS(2085),
    [sym_while_keyword] = ACTIONS(2085),
    [sym_for_keyword] = ACTIONS(2085),
    [sym_break_keyword] = ACTIONS(2085),
    [sym_continue_keyword] = ACTIONS(2085),
    [sym_return_keyword] = ACTIONS(2085),
    [sym_select_keyword] = ACTIONS(2085),
    [sym_active_keyword] = ACTIONS(2085),
    [sym_struct_keyword] = ACTIONS(2085),
    [sym_structure_keyword] = ACTIONS(2085),
    [sym_button_event_keyword] = ACTIONS(2085),
    [sym_channel_event_keyword] = ACTIONS(2085),
    [sym_level_event_keyword] = ACTIONS(2085),
    [sym_data_event_keyword] = ACTIONS(2085),
    [sym_timeline_event_keyword] = ACTIONS(2085),
    [sym_custom_event_keyword] = ACTIONS(2085),
    [sym_push_keyword] = ACTIONS(2085),
    [sym_release_keyword] = ACTIONS(2085),
    [aux_sym_on_keyword_token1] = ACTIONS(2085),
    [aux_sym_off_keyword_token1] = ACTIONS(2085),
    [sym_send_level_keyword] = ACTIONS(2085),
    [sym_send_string_keyword] = ACTIONS(2085),
    [sym_send_command_keyword] = ACTIONS(2085),
    [sym_clear_buffer_keyword] = ACTIONS(2085),
    [sym_create_buffer_keyword] = ACTIONS(2085),
    [sym_create_multi_buffer_keyword] = ACTIONS(2085),
    [sym_call_keyword] = ACTIONS(2085),
    [sym_system_call_keyword] = ACTIONS(2085),
    [sym_devchan_to_keyword] = ACTIONS(2085),
    [sym_devchan_min_to_keyword] = ACTIONS(2085),
    [sym_devchan_total_off_keyword] = ACTIONS(2085),
    [sym_devchan_pulse_keyword] = ACTIONS(2085),
    [sym_wait_keyword] = ACTIONS(2085),
    [sym_cancel_wait_keyword] = ACTIONS(2085),
    [sym_wait_until_keyword] = ACTIONS(2085),
    [sym_cancel_wait_until_keyword] = ACTIONS(2085),
    [sym_cancel_all_wait_keyword] = ACTIONS(2085),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2085),
    [sym_not] = ACTIONS(2085),
    [sym_bnot] = ACTIONS(2085),
    [sym___file__] = ACTIONS(2085),
    [sym___date__] = ACTIONS(2085),
    [sym___ldate__] = ACTIONS(2085),
    [sym___time__] = ACTIONS(2085),
    [sym___line__] = ACTIONS(2085),
    [sym___name__] = ACTIONS(2085),
    [sym___netlinx__] = ACTIONS(2085),
    [sym_netlinx_axi_version] = ACTIONS(2085),
    [sym_url_flg_tcp] = ACTIONS(2085),
    [sym_url_flg_accinfopresent] = ACTIONS(2085),
    [sym_url_flg_temp] = ACTIONS(2085),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2085),
    [sym_url_flg_stat_mask] = ACTIONS(2085),
    [sym_url_flg_stat_lookup] = ACTIONS(2085),
    [sym_url_flg_stat_connecting] = ACTIONS(2085),
    [sym_url_flg_stat_waiting] = ACTIONS(2085),
    [sym_url_flg_stat_connected] = ACTIONS(2085),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2085),
    [sym_first_virtual_device] = ACTIONS(2085),
    [sym_first_local_port] = ACTIONS(2085),
    [sym_dynamic_virtual_device] = ACTIONS(2085),
    [sym_timeline_once] = ACTIONS(2085),
    [sym_timeline_repeat] = ACTIONS(2085),
    [sym_timeline_absolute] = ACTIONS(2085),
    [sym_timeline_relative] = ACTIONS(2085),
    [sym_do_push_timed_infinite] = ACTIONS(2085),
    [sym_source_type_no_address] = ACTIONS(2085),
    [sym_source_type_neuron_id] = ACTIONS(2085),
    [sym_source_type_ip_address] = ACTIONS(2085),
    [sym_source_type_axlink] = ACTIONS(2085),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2085),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2085),
    [sym_source_type_ip_socket_address] = ACTIONS(2085),
    [sym_source_type_rs232] = ACTIONS(2085),
    [sym_source_type_internal] = ACTIONS(2085),
    [sym_source_type_mac_address] = ACTIONS(2085),
    [sym_source_type_ipv4_port] = ACTIONS(2085),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2085),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2085),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2085),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2085),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2085),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2085),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2085),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2085),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2085),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2085),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2085),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2085),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2085),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2085),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2085),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2085),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2085),
    [sym_true] = ACTIONS(2085),
    [sym_false] = ACTIONS(2085),
    [sym_file_read_only] = ACTIONS(2085),
    [sym_file_rw_new] = ACTIONS(2085),
    [sym_file_rw_append] = ACTIONS(2085),
    [sym_ip_tcp] = ACTIONS(2085),
    [sym_ip_udp] = ACTIONS(2085),
    [sym_ip_udp_2way] = ACTIONS(2085),
    [sym_xml_encode_types] = ACTIONS(2085),
    [sym_xml_encode_char_as_list] = ACTIONS(2085),
    [sym_xml_encode_le] = ACTIONS(2085),
    [sym_xml_decode_types] = ACTIONS(2085),
    [sym_xml_decode_no_preserve] = ACTIONS(2085),
    [sym_duet_dev_type_display_device] = ACTIONS(2085),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2085),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2085),
    [sym_duet_dev_type_io_device] = ACTIONS(2085),
    [sym_duet_dev_type_relay_device] = ACTIONS(2085),
    [sym_duet_dev_type_ups] = ACTIONS(2085),
    [sym_duet_dev_type_amplifier] = ACTIONS(2085),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2085),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2085),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2085),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2085),
    [sym_duet_dev_type_camera] = ACTIONS(2085),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2085),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2085),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2085),
    [sym_duet_dev_type_dss] = ACTIONS(2085),
    [sym_duet_dev_type_dvr] = ACTIONS(2085),
    [sym_duet_dev_type_disc_device] = ACTIONS(2085),
    [sym_duet_dev_type_document_camera] = ACTIONS(2085),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2085),
    [sym_duet_dev_type_hvac] = ACTIONS(2085),
    [sym_duet_dev_type_keypad] = ACTIONS(2085),
    [sym_duet_dev_type_light] = ACTIONS(2085),
    [sym_duet_dev_type_monitor] = ACTIONS(2085),
    [sym_duet_dev_type_motor] = ACTIONS(2085),
    [sym_duet_dev_type_multi_window] = ACTIONS(2085),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2085),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2085),
    [sym_duet_dev_type_receiver] = ACTIONS(2085),
    [sym_duet_dev_type_security_system] = ACTIONS(2085),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2085),
    [sym_duet_dev_type_settop_box] = ACTIONS(2085),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2085),
    [sym_duet_dev_type_switcher] = ACTIONS(2085),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2085),
    [sym_duet_dev_type_tv] = ACTIONS(2085),
    [sym_duet_dev_type_utility] = ACTIONS(2085),
    [sym_duet_dev_type_vcr] = ACTIONS(2085),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2085),
    [sym_duet_dev_type_video_processor] = ACTIONS(2085),
    [sym_duet_dev_type_video_projector] = ACTIONS(2085),
    [sym_duet_dev_type_video_wall] = ACTIONS(2085),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2085),
    [sym_duet_dev_type_weather] = ACTIONS(2085),
    [sym_duet_dev_not_polled] = ACTIONS(2085),
    [sym_duet_dev_polled] = ACTIONS(2085),
    [sym_clkmgr_mode_network] = ACTIONS(2085),
    [sym_clkmgr_mode_standalone] = ACTIONS(2085),
    [sym_normal_standby] = ACTIONS(2085),
    [sym_normal_wake] = ACTIONS(2085),
    [sym_ip_multicast_ttl_option] = ACTIONS(2085),
    [sym_ip_ttl_subnet] = ACTIONS(2085),
    [sym_ip_ttl_site] = ACTIONS(2085),
    [sym_ip_ttl_region] = ACTIONS(2085),
    [sym_ip_ttl_continent] = ACTIONS(2085),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2085),
    [sym_ip_nodelay_on] = ACTIONS(2085),
    [sym_ip_nodelay_off] = ACTIONS(2085),
    [sym_amx_error] = ACTIONS(2085),
    [sym_amx_warning] = ACTIONS(2085),
    [sym_amx_info] = ACTIONS(2085),
    [sym_amx_debug] = ACTIONS(2085),
    [sym_netlinx_login_success] = ACTIONS(2085),
    [sym_netlinx_login_fail] = ACTIONS(2085),
    [sym_netlinx_logout] = ACTIONS(2085),
    [sym_null_str] = ACTIONS(2085),
    [sym_smtp_address] = ACTIONS(2085),
    [sym_smtp_port_number] = ACTIONS(2085),
    [sym_smtp_username] = ACTIONS(2085),
    [sym_smtp_password] = ACTIONS(2085),
    [sym_smtp_require_tls] = ACTIONS(2085),
    [sym_smtp_from] = ACTIONS(2085),
    [sym_smtp_tls_true] = ACTIONS(2085),
    [sym_smtp_tls_false] = ACTIONS(2085),
    [sym_listview_on_row_select_event] = ACTIONS(2085),
    [sym_data_structure_unknown] = ACTIONS(2085),
    [sym_data_structure_datafeed] = ACTIONS(2085),
    [sym_data_structure_datarecordset] = ACTIONS(2085),
    [sym_data_structure_datarecord] = ACTIONS(2085),
    [sym_data_structure_datafield] = ACTIONS(2085),
    [sym_data_max_content_field_count] = ACTIONS(2085),
    [sym_data_max_metadata_field_count] = ACTIONS(2085),
    [sym_data_max_id_length] = ACTIONS(2085),
    [sym_data_max_name_length] = ACTIONS(2085),
    [sym_data_max_description_length] = ACTIONS(2085),
    [sym_data_max_type_length] = ACTIONS(2085),
    [sym_data_max_format_length] = ACTIONS(2085),
    [sym_data_max_source_length] = ACTIONS(2085),
    [sym_data_max_value_length] = ACTIONS(2085),
    [sym_data_max_label_length] = ACTIONS(2085),
    [sym_data_max_url_length] = ACTIONS(2085),
    [sym_max_last_login_info_length] = ACTIONS(2085),
    [sym_data_type_unknown] = ACTIONS(2085),
    [sym_data_type_string] = ACTIONS(2085),
    [sym_data_type_datetime] = ACTIONS(2085),
    [sym_data_type_date] = ACTIONS(2085),
    [sym_data_type_time] = ACTIONS(2085),
    [sym_data_type_image] = ACTIONS(2085),
    [sym_data_format_url] = ACTIONS(2085),
    [sym_data_format_phone] = ACTIONS(2085),
    [sym_data_format_email] = ACTIONS(2085),
    [sym_data_format_iso8601] = ACTIONS(2085),
    [sym_auditlog_priv] = ACTIONS(2085),
    [sym_configuration_priv] = ACTIONS(2085),
    [sym_deviceconfig_priv] = ACTIONS(2085),
    [sym_ftp_priv] = ACTIONS(2085),
    [sym_http_priv] = ACTIONS(2085),
    [sym_networkconfig_priv] = ACTIONS(2085),
    [sym_programport_priv] = ACTIONS(2085),
    [sym_remoteui_priv] = ACTIONS(2085),
    [sym_securitycontrol_priv] = ACTIONS(2085),
    [sym_softwaremanagement_priv] = ACTIONS(2085),
    [sym_terminal_priv] = ACTIONS(2085),
    [sym_tpadmin_priv] = ACTIONS(2085),
    [sym_user1_priv] = ACTIONS(2085),
    [sym_user2_priv] = ACTIONS(2085),
    [sym_user3_priv] = ACTIONS(2085),
    [sym_user4_priv] = ACTIONS(2085),
    [sym_usermanagement_priv] = ACTIONS(2085),
    [sym_valid_account] = ACTIONS(2085),
    [sym_err_name_invalid] = ACTIONS(2085),
    [sym_err_password_invalid] = ACTIONS(2085),
    [sym_err_invalid_account] = ACTIONS(2085),
    [sym_err_no_authorization] = ACTIONS(2085),
    [sym_err_invalid_parameter] = ACTIONS(2085),
    [sym_err_user_locked_out] = ACTIONS(2085),
    [sym_err_user_account_expired] = ACTIONS(2085),
    [sym_err_general_error] = ACTIONS(2085),
    [sym_err_authentication_error] = ACTIONS(2085),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2085),
    [sym_tls_validate_certificate] = ACTIONS(2085),
    [sym_tdata] = ACTIONS(2085),
    [sym_tchannel] = ACTIONS(2085),
    [sym_tlevel] = ACTIONS(2085),
    [sym_tbutton] = ACTIONS(2085),
    [sym_ttimeline] = ACTIONS(2085),
    [sym_tcustom] = ACTIONS(2085),
    [sym_url_struct] = ACTIONS(2085),
    [sym_dns_struct] = ACTIONS(2085),
    [sym_ip_address_struct] = ACTIONS(2085),
    [sym_dev_info_struct] = ACTIONS(2085),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2085),
    [sym_data_feed] = ACTIONS(2085),
    [sym_data_field] = ACTIONS(2085),
    [sym_data_record] = ACTIONS(2085),
    [sym_wc_data_feed] = ACTIONS(2085),
    [sym_wc_data_field] = ACTIONS(2085),
    [sym_wc_data_record] = ACTIONS(2085),
    [sym_last_login_info] = ACTIONS(2085),
    [sym____reserved___] = ACTIONS(2085),
    [sym_date] = ACTIONS(2085),
    [sym_day] = ACTIONS(2085),
    [sym_time] = ACTIONS(2085),
    [sym_ldate] = ACTIONS(2085),
    [sym_push_device] = ACTIONS(2085),
    [sym_push_channel] = ACTIONS(2085),
    [sym_push_devchan] = ACTIONS(2085),
    [sym_release_device] = ACTIONS(2085),
    [sym_release_channel] = ACTIONS(2085),
    [sym_release_devchan] = ACTIONS(2085),
    [sym_master_slot] = ACTIONS(2085),
    [sym_get_pulse_time] = ACTIONS(2085),
    [sym_get_timer] = ACTIONS(2085),
    [sym_master_sn] = ACTIONS(2085),
    [sym_system_number] = ACTIONS(2085),
    [sym_button] = ACTIONS(2085),
    [sym_dv_channel] = ACTIONS(2085),
    [sym_data] = ACTIONS(2085),
    [sym_level] = ACTIONS(2085),
    [sym_timeline] = ACTIONS(2085),
    [sym_channel] = ACTIONS(2085),
    [sym_custom] = ACTIONS(2085),
    [sym_atoi] = ACTIONS(2085),
    [sym_atol] = ACTIONS(2085),
    [sym_atof] = ACTIONS(2085),
    [sym_itoa] = ACTIONS(2085),
    [sym_itohex] = ACTIONS(2085),
    [sym_hextoi] = ACTIONS(2085),
    [sym_ftoa] = ACTIONS(2085),
    [sym_find_string] = ACTIONS(2085),
    [sym_left_string] = ACTIONS(2085),
    [sym_right_string] = ACTIONS(2085),
    [sym_mid_string] = ACTIONS(2085),
    [sym_remove_string] = ACTIONS(2085),
    [sym_get_buffer_string] = ACTIONS(2085),
    [sym_get_buffer_char] = ACTIONS(2085),
    [sym_get_multi_buffer_string] = ACTIONS(2085),
    [sym_length_string] = ACTIONS(2085),
    [sym_max_length_string] = ACTIONS(2085),
    [sym_set_length_string] = ACTIONS(2085),
    [sym_compare_string] = ACTIONS(2085),
    [sym_upper_string] = ACTIONS(2085),
    [sym_lower_string] = ACTIONS(2085),
    [sym_get_last] = ACTIONS(2085),
    [sym_date_to_day] = ACTIONS(2085),
    [sym_date_to_month] = ACTIONS(2085),
    [sym_date_to_year] = ACTIONS(2085),
    [sym_time_to_hour] = ACTIONS(2085),
    [sym_time_to_minute] = ACTIONS(2085),
    [sym_time_to_second] = ACTIONS(2085),
    [sym_day_of_week] = ACTIONS(2085),
    [sym_set_pulse_time] = ACTIONS(2085),
    [sym_set_timer] = ACTIONS(2085),
    [sym_file_open] = ACTIONS(2085),
    [sym_file_close] = ACTIONS(2085),
    [sym_file_read] = ACTIONS(2085),
    [sym_file_read_line] = ACTIONS(2085),
    [sym_file_write] = ACTIONS(2085),
    [sym_file_write_line] = ACTIONS(2085),
    [sym_file_copy] = ACTIONS(2085),
    [sym_file_delete] = ACTIONS(2085),
    [sym_file_rename] = ACTIONS(2085),
    [sym_file_dir] = ACTIONS(2085),
    [sym_file_getdir] = ACTIONS(2085),
    [sym_file_setdir] = ACTIONS(2085),
    [sym_file_create_dir] = ACTIONS(2085),
    [sym_file_remove_dir] = ACTIONS(2085),
    [sym_file_seek] = ACTIONS(2085),
    [sym_length_array] = ACTIONS(2085),
    [sym_max_length_array] = ACTIONS(2085),
    [sym_set_length_array] = ACTIONS(2085),
    [sym_ip_client_open] = ACTIONS(2085),
    [sym_ip_client_close] = ACTIONS(2085),
    [sym_ip_server_open] = ACTIONS(2085),
    [sym_ip_server_close] = ACTIONS(2085),
    [sym_ip_bound_client_open] = ACTIONS(2085),
    [sym_ip_mc_server_open] = ACTIONS(2085),
    [sym_ip_set_option] = ACTIONS(2085),
    [sym_ssh_client_open] = ACTIONS(2085),
    [sym_ssh_client_close] = ACTIONS(2085),
    [sym_tls_client_open] = ACTIONS(2085),
    [sym_tls_client_close] = ACTIONS(2085),
    [sym_do_push] = ACTIONS(2085),
    [sym_do_push_timed] = ACTIONS(2085),
    [sym_do_release] = ACTIONS(2085),
    [sym_device_id] = ACTIONS(2085),
    [sym_device_info] = ACTIONS(2085),
    [sym_device_id_string] = ACTIONS(2085),
    [sym_device_standby] = ACTIONS(2085),
    [sym_device_wake] = ACTIONS(2085),
    [sym_timeline_create] = ACTIONS(2085),
    [sym_timeline_kill] = ACTIONS(2085),
    [sym_timeline_reload] = ACTIONS(2085),
    [sym_timeline_pause] = ACTIONS(2085),
    [sym_timeline_restart] = ACTIONS(2085),
    [sym_timeline_set] = ACTIONS(2085),
    [sym_timeline_get] = ACTIONS(2085),
    [sym_timeline_active] = ACTIONS(2085),
    [sym_timeline_dynamic_id] = ACTIONS(2085),
    [sym_variable_to_string] = ACTIONS(2085),
    [sym_string_to_variable] = ACTIONS(2085),
    [sym_length_variable_to_string] = ACTIONS(2085),
    [sym_type_cast] = ACTIONS(2085),
    [sym_variable_to_xml] = ACTIONS(2085),
    [sym_xml_to_variable] = ACTIONS(2085),
    [sym_length_variable_to_xml] = ACTIONS(2085),
    [sym_redirect_string] = ACTIONS(2085),
    [sym_random_number] = ACTIONS(2085),
    [sym_get_unique_id] = ACTIONS(2085),
    [sym_get_system_number] = ACTIONS(2085),
    [sym_set_system_number] = ACTIONS(2085),
    [sym_get_serial_number] = ACTIONS(2085),
    [sym_get_master_build] = ACTIONS(2085),
    [sym_set_validation_code] = ACTIONS(2085),
    [sym_reboot] = ACTIONS(2085),
    [sym_load_duet_module] = ACTIONS(2085),
    [sym_unload_duet_module] = ACTIONS(2085),
    [sym_duet_mem_size_set] = ACTIONS(2085),
    [sym_duet_mem_size_get] = ACTIONS(2085),
    [sym_dynamic_application_device] = ACTIONS(2085),
    [sym_dynamic_polled_port] = ACTIONS(2085),
    [sym_static_port_binding] = ACTIONS(2085),
    [sym_static_ip_binding] = ACTIONS(2085),
    [sym_get_dns_list] = ACTIONS(2085),
    [sym_set_dns_list] = ACTIONS(2085),
    [sym_get_ip_address] = ACTIONS(2085),
    [sym_set_ip_address] = ACTIONS(2085),
    [sym_get_url_list] = ACTIONS(2085),
    [sym_add_url_entry] = ACTIONS(2085),
    [sym_delete_url_entry] = ACTIONS(2085),
    [sym_abs_value] = ACTIONS(2085),
    [sym_max_value] = ACTIONS(2085),
    [sym_min_value] = ACTIONS(2085),
    [sym_exp_value] = ACTIONS(2085),
    [sym_log_value] = ACTIONS(2085),
    [sym_log10_value] = ACTIONS(2085),
    [sym_power_value] = ACTIONS(2085),
    [sym_sqrt_value] = ACTIONS(2085),
    [sym_raw_be] = ACTIONS(2085),
    [sym_raw_le] = ACTIONS(2085),
    [sym_internal_queue_size_set] = ACTIONS(2085),
    [sym_internal_queue_size_get] = ACTIONS(2085),
    [sym_internal_threshold_set] = ACTIONS(2085),
    [sym_internal_threshold_get] = ACTIONS(2085),
    [sym_rebuilt_event] = ACTIONS(2085),
    [sym_do_custom_event] = ACTIONS(2085),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2085),
    [sym_clkmgr_set_clk_source] = ACTIONS(2085),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2085),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2085),
    [sym_clkmgr_get_timezone] = ACTIONS(2085),
    [sym_clkmgr_set_timezone] = ACTIONS(2085),
    [sym_clkmgr_get_resync_period] = ACTIONS(2085),
    [sym_clkmgr_set_resync_period] = ACTIONS(2085),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2085),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2085),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2085),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2085),
    [sym_clkmgr_get_timeservers] = ACTIONS(2085),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2085),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2085),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2085),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2085),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2085),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2085),
    [sym_set_log_level] = ACTIONS(2085),
    [sym_get_log_level] = ACTIONS(2085),
    [sym_amx_log] = ACTIONS(2085),
    [sym_smtp_server_config_set] = ACTIONS(2085),
    [sym_smtp_server_config_get] = ACTIONS(2085),
    [sym_smtp_send] = ACTIONS(2085),
    [sym_validate_netlinx_account] = ACTIONS(2085),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2085),
    [sym_authenticate_certificate] = ACTIONS(2085),
    [sym_audit_netlinx_session_event] = ACTIONS(2085),
    [sym_audit_netlinx_generic_event] = ACTIONS(2085),
    [sym_get_available_flash_disk_space] = ACTIONS(2085),
    [sym_get_max_flash_disk_space] = ACTIONS(2085),
    [sym_data_create_feed] = ACTIONS(2085),
    [sym_data_delete_feed] = ACTIONS(2085),
    [sym_data_publish_feed] = ACTIONS(2085),
    [sym_data_get_published_feed] = ACTIONS(2085),
    [sym_data_add_record] = ACTIONS(2085),
    [sym_data_get_event_record] = ACTIONS(2085),
    [sym__wc_data_create_feed] = ACTIONS(2085),
    [sym__wc_data_add_record] = ACTIONS(2085),
    [sym__wc_data_get_event_record] = ACTIONS(2085),
    [sym_format] = ACTIONS(2085),
    [sym_astro_clock] = ACTIONS(2085),
    [sym_set_outdoor_temperature] = ACTIONS(2085),
    [sym_set_virtual_level_count] = ACTIONS(2085),
    [sym_set_virtual_channel_count] = ACTIONS(2085),
    [sym_set_virtual_port_count] = ACTIONS(2085),
    [sym__wc_encode_int] = ACTIONS(2085),
    [sym__wc_decode_int] = ACTIONS(2085),
    [sym__wc_int] = ACTIONS(2085),
    [sym__wc_to_ch_int] = ACTIONS(2085),
    [sym__ch_to_wc_int] = ACTIONS(2085),
    [sym__wc_find_string_int] = ACTIONS(2085),
    [sym__wc_left_string_int] = ACTIONS(2085),
    [sym__wc_lower_string_int] = ACTIONS(2085),
    [sym__wc_mid_string_int] = ACTIONS(2085),
    [sym__wc_remove_string_int] = ACTIONS(2085),
    [sym__wc_right_string_int] = ACTIONS(2085),
    [sym__wc_upper_string_int] = ACTIONS(2085),
    [sym__wc_compare_string_int] = ACTIONS(2085),
    [sym__wc_get_buffer_char_int] = ACTIONS(2085),
    [sym__wc_get_buffer_string_int] = ACTIONS(2085),
    [sym__wc_concat_string_int] = ACTIONS(2085),
    [sym__wc_file_open_int] = ACTIONS(2085),
    [sym__wc_file_close_int] = ACTIONS(2085),
    [sym__wc_file_read_int] = ACTIONS(2085),
    [sym__wc_file_read_line_int] = ACTIONS(2085),
    [sym__wc_file_write_int] = ACTIONS(2085),
    [sym__wc_file_write_line_int] = ACTIONS(2085),
    [aux_sym_preproc_include_token1] = ACTIONS(2085),
    [aux_sym_preproc_define_token1] = ACTIONS(2087),
    [aux_sym_preproc_warn_token1] = ACTIONS(2087),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2087),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2087),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2087),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2087),
    [aux_sym_preproc_else_token1] = ACTIONS(2087),
    [anon_sym_LBRACK] = ACTIONS(2087),
    [anon_sym_LBRACE] = ACTIONS(2087),
    [anon_sym_LPAREN] = ACTIONS(2085),
    [anon_sym_BANG] = ACTIONS(2087),
    [anon_sym_TILDE] = ACTIONS(2087),
    [anon_sym_DASH] = ACTIONS(2085),
    [anon_sym_PLUS] = ACTIONS(2085),
    [anon_sym_DASH_DASH] = ACTIONS(2087),
    [anon_sym_PLUS_PLUS] = ACTIONS(2087),
    [anon_sym_DQUOTE] = ACTIONS(2087),
    [anon_sym_SQUOTE] = ACTIONS(2087),
    [sym_number_literal] = ACTIONS(2087),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(366)] = {
    [sym_identifier] = ACTIONS(2089),
    [sym_program_name_keyword] = ACTIONS(2089),
    [sym_module_name_keyword] = ACTIONS(2089),
    [sym_define_device_keyword] = ACTIONS(2089),
    [sym_define_combine_keyword] = ACTIONS(2089),
    [sym_define_constant_keyword] = ACTIONS(2089),
    [sym_define_type_keyword] = ACTIONS(2089),
    [sym_define_variable_keyword] = ACTIONS(2089),
    [sym_define_system_variable_keyword] = ACTIONS(2089),
    [sym_define_connect_level_keyword] = ACTIONS(2089),
    [sym_define_latching_keyword] = ACTIONS(2089),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2089),
    [sym_define_toggling_keyword] = ACTIONS(2089),
    [sym_define_call_keyword] = ACTIONS(2089),
    [sym_define_function_keyword] = ACTIONS(2089),
    [sym_define_library_function_keyword] = ACTIONS(2089),
    [sym_define_start_keyword] = ACTIONS(2089),
    [sym_define_event_keyword] = ACTIONS(2089),
    [sym_define_program_keyword] = ACTIONS(2089),
    [sym_define_module_keyword] = ACTIONS(2089),
    [sym_char_keyword] = ACTIONS(2089),
    [sym_widechar_keyword] = ACTIONS(2089),
    [sym_integer_keyword] = ACTIONS(2089),
    [sym_sinteger_keyword] = ACTIONS(2089),
    [sym_long_keyword] = ACTIONS(2089),
    [sym_slong_keyword] = ACTIONS(2089),
    [sym_float_keyword] = ACTIONS(2089),
    [sym_double_keyword] = ACTIONS(2089),
    [sym_variant_keyword] = ACTIONS(2089),
    [sym_variantarray_keyword] = ACTIONS(2089),
    [sym_local_var_keyword] = ACTIONS(2089),
    [sym_stack_var_keyword] = ACTIONS(2089),
    [sym_constant_keyword] = ACTIONS(2089),
    [sym_volatile_keyword] = ACTIONS(2089),
    [sym_non_volatile_keyword] = ACTIONS(2089),
    [sym_persistent_keyword] = ACTIONS(2089),
    [sym_dev_keyword] = ACTIONS(2089),
    [sym_devlev_keyword] = ACTIONS(2089),
    [sym_devchan_keyword] = ACTIONS(2089),
    [sym_if_keyword] = ACTIONS(2089),
    [sym_switch_keyword] = ACTIONS(2089),
    [sym_case_keyword] = ACTIONS(2089),
    [sym_default_keyword] = ACTIONS(2089),
    [sym_while_keyword] = ACTIONS(2089),
    [sym_for_keyword] = ACTIONS(2089),
    [sym_break_keyword] = ACTIONS(2089),
    [sym_continue_keyword] = ACTIONS(2089),
    [sym_return_keyword] = ACTIONS(2089),
    [sym_select_keyword] = ACTIONS(2089),
    [sym_active_keyword] = ACTIONS(2089),
    [sym_struct_keyword] = ACTIONS(2089),
    [sym_structure_keyword] = ACTIONS(2089),
    [sym_button_event_keyword] = ACTIONS(2089),
    [sym_channel_event_keyword] = ACTIONS(2089),
    [sym_level_event_keyword] = ACTIONS(2089),
    [sym_data_event_keyword] = ACTIONS(2089),
    [sym_timeline_event_keyword] = ACTIONS(2089),
    [sym_custom_event_keyword] = ACTIONS(2089),
    [sym_push_keyword] = ACTIONS(2089),
    [sym_release_keyword] = ACTIONS(2089),
    [aux_sym_on_keyword_token1] = ACTIONS(2089),
    [aux_sym_off_keyword_token1] = ACTIONS(2089),
    [sym_send_level_keyword] = ACTIONS(2089),
    [sym_send_string_keyword] = ACTIONS(2089),
    [sym_send_command_keyword] = ACTIONS(2089),
    [sym_clear_buffer_keyword] = ACTIONS(2089),
    [sym_create_buffer_keyword] = ACTIONS(2089),
    [sym_create_multi_buffer_keyword] = ACTIONS(2089),
    [sym_call_keyword] = ACTIONS(2089),
    [sym_system_call_keyword] = ACTIONS(2089),
    [sym_devchan_to_keyword] = ACTIONS(2089),
    [sym_devchan_min_to_keyword] = ACTIONS(2089),
    [sym_devchan_total_off_keyword] = ACTIONS(2089),
    [sym_devchan_pulse_keyword] = ACTIONS(2089),
    [sym_wait_keyword] = ACTIONS(2089),
    [sym_cancel_wait_keyword] = ACTIONS(2089),
    [sym_wait_until_keyword] = ACTIONS(2089),
    [sym_cancel_wait_until_keyword] = ACTIONS(2089),
    [sym_cancel_all_wait_keyword] = ACTIONS(2089),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2089),
    [sym_not] = ACTIONS(2089),
    [sym_bnot] = ACTIONS(2089),
    [sym___file__] = ACTIONS(2089),
    [sym___date__] = ACTIONS(2089),
    [sym___ldate__] = ACTIONS(2089),
    [sym___time__] = ACTIONS(2089),
    [sym___line__] = ACTIONS(2089),
    [sym___name__] = ACTIONS(2089),
    [sym___netlinx__] = ACTIONS(2089),
    [sym_netlinx_axi_version] = ACTIONS(2089),
    [sym_url_flg_tcp] = ACTIONS(2089),
    [sym_url_flg_accinfopresent] = ACTIONS(2089),
    [sym_url_flg_temp] = ACTIONS(2089),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2089),
    [sym_url_flg_stat_mask] = ACTIONS(2089),
    [sym_url_flg_stat_lookup] = ACTIONS(2089),
    [sym_url_flg_stat_connecting] = ACTIONS(2089),
    [sym_url_flg_stat_waiting] = ACTIONS(2089),
    [sym_url_flg_stat_connected] = ACTIONS(2089),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2089),
    [sym_first_virtual_device] = ACTIONS(2089),
    [sym_first_local_port] = ACTIONS(2089),
    [sym_dynamic_virtual_device] = ACTIONS(2089),
    [sym_timeline_once] = ACTIONS(2089),
    [sym_timeline_repeat] = ACTIONS(2089),
    [sym_timeline_absolute] = ACTIONS(2089),
    [sym_timeline_relative] = ACTIONS(2089),
    [sym_do_push_timed_infinite] = ACTIONS(2089),
    [sym_source_type_no_address] = ACTIONS(2089),
    [sym_source_type_neuron_id] = ACTIONS(2089),
    [sym_source_type_ip_address] = ACTIONS(2089),
    [sym_source_type_axlink] = ACTIONS(2089),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2089),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2089),
    [sym_source_type_ip_socket_address] = ACTIONS(2089),
    [sym_source_type_rs232] = ACTIONS(2089),
    [sym_source_type_internal] = ACTIONS(2089),
    [sym_source_type_mac_address] = ACTIONS(2089),
    [sym_source_type_ipv4_port] = ACTIONS(2089),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2089),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2089),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2089),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2089),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2089),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2089),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2089),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2089),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2089),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2089),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2089),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2089),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2089),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2089),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2089),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2089),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2089),
    [sym_true] = ACTIONS(2089),
    [sym_false] = ACTIONS(2089),
    [sym_file_read_only] = ACTIONS(2089),
    [sym_file_rw_new] = ACTIONS(2089),
    [sym_file_rw_append] = ACTIONS(2089),
    [sym_ip_tcp] = ACTIONS(2089),
    [sym_ip_udp] = ACTIONS(2089),
    [sym_ip_udp_2way] = ACTIONS(2089),
    [sym_xml_encode_types] = ACTIONS(2089),
    [sym_xml_encode_char_as_list] = ACTIONS(2089),
    [sym_xml_encode_le] = ACTIONS(2089),
    [sym_xml_decode_types] = ACTIONS(2089),
    [sym_xml_decode_no_preserve] = ACTIONS(2089),
    [sym_duet_dev_type_display_device] = ACTIONS(2089),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2089),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2089),
    [sym_duet_dev_type_io_device] = ACTIONS(2089),
    [sym_duet_dev_type_relay_device] = ACTIONS(2089),
    [sym_duet_dev_type_ups] = ACTIONS(2089),
    [sym_duet_dev_type_amplifier] = ACTIONS(2089),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2089),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2089),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2089),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2089),
    [sym_duet_dev_type_camera] = ACTIONS(2089),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2089),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2089),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2089),
    [sym_duet_dev_type_dss] = ACTIONS(2089),
    [sym_duet_dev_type_dvr] = ACTIONS(2089),
    [sym_duet_dev_type_disc_device] = ACTIONS(2089),
    [sym_duet_dev_type_document_camera] = ACTIONS(2089),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2089),
    [sym_duet_dev_type_hvac] = ACTIONS(2089),
    [sym_duet_dev_type_keypad] = ACTIONS(2089),
    [sym_duet_dev_type_light] = ACTIONS(2089),
    [sym_duet_dev_type_monitor] = ACTIONS(2089),
    [sym_duet_dev_type_motor] = ACTIONS(2089),
    [sym_duet_dev_type_multi_window] = ACTIONS(2089),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2089),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2089),
    [sym_duet_dev_type_receiver] = ACTIONS(2089),
    [sym_duet_dev_type_security_system] = ACTIONS(2089),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2089),
    [sym_duet_dev_type_settop_box] = ACTIONS(2089),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2089),
    [sym_duet_dev_type_switcher] = ACTIONS(2089),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2089),
    [sym_duet_dev_type_tv] = ACTIONS(2089),
    [sym_duet_dev_type_utility] = ACTIONS(2089),
    [sym_duet_dev_type_vcr] = ACTIONS(2089),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2089),
    [sym_duet_dev_type_video_processor] = ACTIONS(2089),
    [sym_duet_dev_type_video_projector] = ACTIONS(2089),
    [sym_duet_dev_type_video_wall] = ACTIONS(2089),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2089),
    [sym_duet_dev_type_weather] = ACTIONS(2089),
    [sym_duet_dev_not_polled] = ACTIONS(2089),
    [sym_duet_dev_polled] = ACTIONS(2089),
    [sym_clkmgr_mode_network] = ACTIONS(2089),
    [sym_clkmgr_mode_standalone] = ACTIONS(2089),
    [sym_normal_standby] = ACTIONS(2089),
    [sym_normal_wake] = ACTIONS(2089),
    [sym_ip_multicast_ttl_option] = ACTIONS(2089),
    [sym_ip_ttl_subnet] = ACTIONS(2089),
    [sym_ip_ttl_site] = ACTIONS(2089),
    [sym_ip_ttl_region] = ACTIONS(2089),
    [sym_ip_ttl_continent] = ACTIONS(2089),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2089),
    [sym_ip_nodelay_on] = ACTIONS(2089),
    [sym_ip_nodelay_off] = ACTIONS(2089),
    [sym_amx_error] = ACTIONS(2089),
    [sym_amx_warning] = ACTIONS(2089),
    [sym_amx_info] = ACTIONS(2089),
    [sym_amx_debug] = ACTIONS(2089),
    [sym_netlinx_login_success] = ACTIONS(2089),
    [sym_netlinx_login_fail] = ACTIONS(2089),
    [sym_netlinx_logout] = ACTIONS(2089),
    [sym_null_str] = ACTIONS(2089),
    [sym_smtp_address] = ACTIONS(2089),
    [sym_smtp_port_number] = ACTIONS(2089),
    [sym_smtp_username] = ACTIONS(2089),
    [sym_smtp_password] = ACTIONS(2089),
    [sym_smtp_require_tls] = ACTIONS(2089),
    [sym_smtp_from] = ACTIONS(2089),
    [sym_smtp_tls_true] = ACTIONS(2089),
    [sym_smtp_tls_false] = ACTIONS(2089),
    [sym_listview_on_row_select_event] = ACTIONS(2089),
    [sym_data_structure_unknown] = ACTIONS(2089),
    [sym_data_structure_datafeed] = ACTIONS(2089),
    [sym_data_structure_datarecordset] = ACTIONS(2089),
    [sym_data_structure_datarecord] = ACTIONS(2089),
    [sym_data_structure_datafield] = ACTIONS(2089),
    [sym_data_max_content_field_count] = ACTIONS(2089),
    [sym_data_max_metadata_field_count] = ACTIONS(2089),
    [sym_data_max_id_length] = ACTIONS(2089),
    [sym_data_max_name_length] = ACTIONS(2089),
    [sym_data_max_description_length] = ACTIONS(2089),
    [sym_data_max_type_length] = ACTIONS(2089),
    [sym_data_max_format_length] = ACTIONS(2089),
    [sym_data_max_source_length] = ACTIONS(2089),
    [sym_data_max_value_length] = ACTIONS(2089),
    [sym_data_max_label_length] = ACTIONS(2089),
    [sym_data_max_url_length] = ACTIONS(2089),
    [sym_max_last_login_info_length] = ACTIONS(2089),
    [sym_data_type_unknown] = ACTIONS(2089),
    [sym_data_type_string] = ACTIONS(2089),
    [sym_data_type_datetime] = ACTIONS(2089),
    [sym_data_type_date] = ACTIONS(2089),
    [sym_data_type_time] = ACTIONS(2089),
    [sym_data_type_image] = ACTIONS(2089),
    [sym_data_format_url] = ACTIONS(2089),
    [sym_data_format_phone] = ACTIONS(2089),
    [sym_data_format_email] = ACTIONS(2089),
    [sym_data_format_iso8601] = ACTIONS(2089),
    [sym_auditlog_priv] = ACTIONS(2089),
    [sym_configuration_priv] = ACTIONS(2089),
    [sym_deviceconfig_priv] = ACTIONS(2089),
    [sym_ftp_priv] = ACTIONS(2089),
    [sym_http_priv] = ACTIONS(2089),
    [sym_networkconfig_priv] = ACTIONS(2089),
    [sym_programport_priv] = ACTIONS(2089),
    [sym_remoteui_priv] = ACTIONS(2089),
    [sym_securitycontrol_priv] = ACTIONS(2089),
    [sym_softwaremanagement_priv] = ACTIONS(2089),
    [sym_terminal_priv] = ACTIONS(2089),
    [sym_tpadmin_priv] = ACTIONS(2089),
    [sym_user1_priv] = ACTIONS(2089),
    [sym_user2_priv] = ACTIONS(2089),
    [sym_user3_priv] = ACTIONS(2089),
    [sym_user4_priv] = ACTIONS(2089),
    [sym_usermanagement_priv] = ACTIONS(2089),
    [sym_valid_account] = ACTIONS(2089),
    [sym_err_name_invalid] = ACTIONS(2089),
    [sym_err_password_invalid] = ACTIONS(2089),
    [sym_err_invalid_account] = ACTIONS(2089),
    [sym_err_no_authorization] = ACTIONS(2089),
    [sym_err_invalid_parameter] = ACTIONS(2089),
    [sym_err_user_locked_out] = ACTIONS(2089),
    [sym_err_user_account_expired] = ACTIONS(2089),
    [sym_err_general_error] = ACTIONS(2089),
    [sym_err_authentication_error] = ACTIONS(2089),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2089),
    [sym_tls_validate_certificate] = ACTIONS(2089),
    [sym_tdata] = ACTIONS(2089),
    [sym_tchannel] = ACTIONS(2089),
    [sym_tlevel] = ACTIONS(2089),
    [sym_tbutton] = ACTIONS(2089),
    [sym_ttimeline] = ACTIONS(2089),
    [sym_tcustom] = ACTIONS(2089),
    [sym_url_struct] = ACTIONS(2089),
    [sym_dns_struct] = ACTIONS(2089),
    [sym_ip_address_struct] = ACTIONS(2089),
    [sym_dev_info_struct] = ACTIONS(2089),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2089),
    [sym_data_feed] = ACTIONS(2089),
    [sym_data_field] = ACTIONS(2089),
    [sym_data_record] = ACTIONS(2089),
    [sym_wc_data_feed] = ACTIONS(2089),
    [sym_wc_data_field] = ACTIONS(2089),
    [sym_wc_data_record] = ACTIONS(2089),
    [sym_last_login_info] = ACTIONS(2089),
    [sym____reserved___] = ACTIONS(2089),
    [sym_date] = ACTIONS(2089),
    [sym_day] = ACTIONS(2089),
    [sym_time] = ACTIONS(2089),
    [sym_ldate] = ACTIONS(2089),
    [sym_push_device] = ACTIONS(2089),
    [sym_push_channel] = ACTIONS(2089),
    [sym_push_devchan] = ACTIONS(2089),
    [sym_release_device] = ACTIONS(2089),
    [sym_release_channel] = ACTIONS(2089),
    [sym_release_devchan] = ACTIONS(2089),
    [sym_master_slot] = ACTIONS(2089),
    [sym_get_pulse_time] = ACTIONS(2089),
    [sym_get_timer] = ACTIONS(2089),
    [sym_master_sn] = ACTIONS(2089),
    [sym_system_number] = ACTIONS(2089),
    [sym_button] = ACTIONS(2089),
    [sym_dv_channel] = ACTIONS(2089),
    [sym_data] = ACTIONS(2089),
    [sym_level] = ACTIONS(2089),
    [sym_timeline] = ACTIONS(2089),
    [sym_channel] = ACTIONS(2089),
    [sym_custom] = ACTIONS(2089),
    [sym_atoi] = ACTIONS(2089),
    [sym_atol] = ACTIONS(2089),
    [sym_atof] = ACTIONS(2089),
    [sym_itoa] = ACTIONS(2089),
    [sym_itohex] = ACTIONS(2089),
    [sym_hextoi] = ACTIONS(2089),
    [sym_ftoa] = ACTIONS(2089),
    [sym_find_string] = ACTIONS(2089),
    [sym_left_string] = ACTIONS(2089),
    [sym_right_string] = ACTIONS(2089),
    [sym_mid_string] = ACTIONS(2089),
    [sym_remove_string] = ACTIONS(2089),
    [sym_get_buffer_string] = ACTIONS(2089),
    [sym_get_buffer_char] = ACTIONS(2089),
    [sym_get_multi_buffer_string] = ACTIONS(2089),
    [sym_length_string] = ACTIONS(2089),
    [sym_max_length_string] = ACTIONS(2089),
    [sym_set_length_string] = ACTIONS(2089),
    [sym_compare_string] = ACTIONS(2089),
    [sym_upper_string] = ACTIONS(2089),
    [sym_lower_string] = ACTIONS(2089),
    [sym_get_last] = ACTIONS(2089),
    [sym_date_to_day] = ACTIONS(2089),
    [sym_date_to_month] = ACTIONS(2089),
    [sym_date_to_year] = ACTIONS(2089),
    [sym_time_to_hour] = ACTIONS(2089),
    [sym_time_to_minute] = ACTIONS(2089),
    [sym_time_to_second] = ACTIONS(2089),
    [sym_day_of_week] = ACTIONS(2089),
    [sym_set_pulse_time] = ACTIONS(2089),
    [sym_set_timer] = ACTIONS(2089),
    [sym_file_open] = ACTIONS(2089),
    [sym_file_close] = ACTIONS(2089),
    [sym_file_read] = ACTIONS(2089),
    [sym_file_read_line] = ACTIONS(2089),
    [sym_file_write] = ACTIONS(2089),
    [sym_file_write_line] = ACTIONS(2089),
    [sym_file_copy] = ACTIONS(2089),
    [sym_file_delete] = ACTIONS(2089),
    [sym_file_rename] = ACTIONS(2089),
    [sym_file_dir] = ACTIONS(2089),
    [sym_file_getdir] = ACTIONS(2089),
    [sym_file_setdir] = ACTIONS(2089),
    [sym_file_create_dir] = ACTIONS(2089),
    [sym_file_remove_dir] = ACTIONS(2089),
    [sym_file_seek] = ACTIONS(2089),
    [sym_length_array] = ACTIONS(2089),
    [sym_max_length_array] = ACTIONS(2089),
    [sym_set_length_array] = ACTIONS(2089),
    [sym_ip_client_open] = ACTIONS(2089),
    [sym_ip_client_close] = ACTIONS(2089),
    [sym_ip_server_open] = ACTIONS(2089),
    [sym_ip_server_close] = ACTIONS(2089),
    [sym_ip_bound_client_open] = ACTIONS(2089),
    [sym_ip_mc_server_open] = ACTIONS(2089),
    [sym_ip_set_option] = ACTIONS(2089),
    [sym_ssh_client_open] = ACTIONS(2089),
    [sym_ssh_client_close] = ACTIONS(2089),
    [sym_tls_client_open] = ACTIONS(2089),
    [sym_tls_client_close] = ACTIONS(2089),
    [sym_do_push] = ACTIONS(2089),
    [sym_do_push_timed] = ACTIONS(2089),
    [sym_do_release] = ACTIONS(2089),
    [sym_device_id] = ACTIONS(2089),
    [sym_device_info] = ACTIONS(2089),
    [sym_device_id_string] = ACTIONS(2089),
    [sym_device_standby] = ACTIONS(2089),
    [sym_device_wake] = ACTIONS(2089),
    [sym_timeline_create] = ACTIONS(2089),
    [sym_timeline_kill] = ACTIONS(2089),
    [sym_timeline_reload] = ACTIONS(2089),
    [sym_timeline_pause] = ACTIONS(2089),
    [sym_timeline_restart] = ACTIONS(2089),
    [sym_timeline_set] = ACTIONS(2089),
    [sym_timeline_get] = ACTIONS(2089),
    [sym_timeline_active] = ACTIONS(2089),
    [sym_timeline_dynamic_id] = ACTIONS(2089),
    [sym_variable_to_string] = ACTIONS(2089),
    [sym_string_to_variable] = ACTIONS(2089),
    [sym_length_variable_to_string] = ACTIONS(2089),
    [sym_type_cast] = ACTIONS(2089),
    [sym_variable_to_xml] = ACTIONS(2089),
    [sym_xml_to_variable] = ACTIONS(2089),
    [sym_length_variable_to_xml] = ACTIONS(2089),
    [sym_redirect_string] = ACTIONS(2089),
    [sym_random_number] = ACTIONS(2089),
    [sym_get_unique_id] = ACTIONS(2089),
    [sym_get_system_number] = ACTIONS(2089),
    [sym_set_system_number] = ACTIONS(2089),
    [sym_get_serial_number] = ACTIONS(2089),
    [sym_get_master_build] = ACTIONS(2089),
    [sym_set_validation_code] = ACTIONS(2089),
    [sym_reboot] = ACTIONS(2089),
    [sym_load_duet_module] = ACTIONS(2089),
    [sym_unload_duet_module] = ACTIONS(2089),
    [sym_duet_mem_size_set] = ACTIONS(2089),
    [sym_duet_mem_size_get] = ACTIONS(2089),
    [sym_dynamic_application_device] = ACTIONS(2089),
    [sym_dynamic_polled_port] = ACTIONS(2089),
    [sym_static_port_binding] = ACTIONS(2089),
    [sym_static_ip_binding] = ACTIONS(2089),
    [sym_get_dns_list] = ACTIONS(2089),
    [sym_set_dns_list] = ACTIONS(2089),
    [sym_get_ip_address] = ACTIONS(2089),
    [sym_set_ip_address] = ACTIONS(2089),
    [sym_get_url_list] = ACTIONS(2089),
    [sym_add_url_entry] = ACTIONS(2089),
    [sym_delete_url_entry] = ACTIONS(2089),
    [sym_abs_value] = ACTIONS(2089),
    [sym_max_value] = ACTIONS(2089),
    [sym_min_value] = ACTIONS(2089),
    [sym_exp_value] = ACTIONS(2089),
    [sym_log_value] = ACTIONS(2089),
    [sym_log10_value] = ACTIONS(2089),
    [sym_power_value] = ACTIONS(2089),
    [sym_sqrt_value] = ACTIONS(2089),
    [sym_raw_be] = ACTIONS(2089),
    [sym_raw_le] = ACTIONS(2089),
    [sym_internal_queue_size_set] = ACTIONS(2089),
    [sym_internal_queue_size_get] = ACTIONS(2089),
    [sym_internal_threshold_set] = ACTIONS(2089),
    [sym_internal_threshold_get] = ACTIONS(2089),
    [sym_rebuilt_event] = ACTIONS(2089),
    [sym_do_custom_event] = ACTIONS(2089),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2089),
    [sym_clkmgr_set_clk_source] = ACTIONS(2089),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2089),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2089),
    [sym_clkmgr_get_timezone] = ACTIONS(2089),
    [sym_clkmgr_set_timezone] = ACTIONS(2089),
    [sym_clkmgr_get_resync_period] = ACTIONS(2089),
    [sym_clkmgr_set_resync_period] = ACTIONS(2089),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2089),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2089),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2089),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2089),
    [sym_clkmgr_get_timeservers] = ACTIONS(2089),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2089),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2089),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2089),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2089),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2089),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2089),
    [sym_set_log_level] = ACTIONS(2089),
    [sym_get_log_level] = ACTIONS(2089),
    [sym_amx_log] = ACTIONS(2089),
    [sym_smtp_server_config_set] = ACTIONS(2089),
    [sym_smtp_server_config_get] = ACTIONS(2089),
    [sym_smtp_send] = ACTIONS(2089),
    [sym_validate_netlinx_account] = ACTIONS(2089),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2089),
    [sym_authenticate_certificate] = ACTIONS(2089),
    [sym_audit_netlinx_session_event] = ACTIONS(2089),
    [sym_audit_netlinx_generic_event] = ACTIONS(2089),
    [sym_get_available_flash_disk_space] = ACTIONS(2089),
    [sym_get_max_flash_disk_space] = ACTIONS(2089),
    [sym_data_create_feed] = ACTIONS(2089),
    [sym_data_delete_feed] = ACTIONS(2089),
    [sym_data_publish_feed] = ACTIONS(2089),
    [sym_data_get_published_feed] = ACTIONS(2089),
    [sym_data_add_record] = ACTIONS(2089),
    [sym_data_get_event_record] = ACTIONS(2089),
    [sym__wc_data_create_feed] = ACTIONS(2089),
    [sym__wc_data_add_record] = ACTIONS(2089),
    [sym__wc_data_get_event_record] = ACTIONS(2089),
    [sym_format] = ACTIONS(2089),
    [sym_astro_clock] = ACTIONS(2089),
    [sym_set_outdoor_temperature] = ACTIONS(2089),
    [sym_set_virtual_level_count] = ACTIONS(2089),
    [sym_set_virtual_channel_count] = ACTIONS(2089),
    [sym_set_virtual_port_count] = ACTIONS(2089),
    [sym__wc_encode_int] = ACTIONS(2089),
    [sym__wc_decode_int] = ACTIONS(2089),
    [sym__wc_int] = ACTIONS(2089),
    [sym__wc_to_ch_int] = ACTIONS(2089),
    [sym__ch_to_wc_int] = ACTIONS(2089),
    [sym__wc_find_string_int] = ACTIONS(2089),
    [sym__wc_left_string_int] = ACTIONS(2089),
    [sym__wc_lower_string_int] = ACTIONS(2089),
    [sym__wc_mid_string_int] = ACTIONS(2089),
    [sym__wc_remove_string_int] = ACTIONS(2089),
    [sym__wc_right_string_int] = ACTIONS(2089),
    [sym__wc_upper_string_int] = ACTIONS(2089),
    [sym__wc_compare_string_int] = ACTIONS(2089),
    [sym__wc_get_buffer_char_int] = ACTIONS(2089),
    [sym__wc_get_buffer_string_int] = ACTIONS(2089),
    [sym__wc_concat_string_int] = ACTIONS(2089),
    [sym__wc_file_open_int] = ACTIONS(2089),
    [sym__wc_file_close_int] = ACTIONS(2089),
    [sym__wc_file_read_int] = ACTIONS(2089),
    [sym__wc_file_read_line_int] = ACTIONS(2089),
    [sym__wc_file_write_int] = ACTIONS(2089),
    [sym__wc_file_write_line_int] = ACTIONS(2089),
    [aux_sym_preproc_include_token1] = ACTIONS(2089),
    [aux_sym_preproc_define_token1] = ACTIONS(2091),
    [aux_sym_preproc_warn_token1] = ACTIONS(2091),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2091),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2091),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2091),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2091),
    [aux_sym_preproc_else_token1] = ACTIONS(2091),
    [anon_sym_LBRACK] = ACTIONS(2091),
    [anon_sym_LBRACE] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(2089),
    [anon_sym_BANG] = ACTIONS(2091),
    [anon_sym_TILDE] = ACTIONS(2091),
    [anon_sym_DASH] = ACTIONS(2089),
    [anon_sym_PLUS] = ACTIONS(2089),
    [anon_sym_DASH_DASH] = ACTIONS(2091),
    [anon_sym_PLUS_PLUS] = ACTIONS(2091),
    [anon_sym_DQUOTE] = ACTIONS(2091),
    [anon_sym_SQUOTE] = ACTIONS(2091),
    [sym_number_literal] = ACTIONS(2091),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(367)] = {
    [sym_identifier] = ACTIONS(2093),
    [sym_program_name_keyword] = ACTIONS(2093),
    [sym_module_name_keyword] = ACTIONS(2093),
    [sym_define_device_keyword] = ACTIONS(2093),
    [sym_define_combine_keyword] = ACTIONS(2093),
    [sym_define_constant_keyword] = ACTIONS(2093),
    [sym_define_type_keyword] = ACTIONS(2093),
    [sym_define_variable_keyword] = ACTIONS(2093),
    [sym_define_system_variable_keyword] = ACTIONS(2093),
    [sym_define_connect_level_keyword] = ACTIONS(2093),
    [sym_define_latching_keyword] = ACTIONS(2093),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2093),
    [sym_define_toggling_keyword] = ACTIONS(2093),
    [sym_define_call_keyword] = ACTIONS(2093),
    [sym_define_function_keyword] = ACTIONS(2093),
    [sym_define_library_function_keyword] = ACTIONS(2093),
    [sym_define_start_keyword] = ACTIONS(2093),
    [sym_define_event_keyword] = ACTIONS(2093),
    [sym_define_program_keyword] = ACTIONS(2093),
    [sym_define_module_keyword] = ACTIONS(2093),
    [sym_char_keyword] = ACTIONS(2093),
    [sym_widechar_keyword] = ACTIONS(2093),
    [sym_integer_keyword] = ACTIONS(2093),
    [sym_sinteger_keyword] = ACTIONS(2093),
    [sym_long_keyword] = ACTIONS(2093),
    [sym_slong_keyword] = ACTIONS(2093),
    [sym_float_keyword] = ACTIONS(2093),
    [sym_double_keyword] = ACTIONS(2093),
    [sym_variant_keyword] = ACTIONS(2093),
    [sym_variantarray_keyword] = ACTIONS(2093),
    [sym_local_var_keyword] = ACTIONS(2093),
    [sym_stack_var_keyword] = ACTIONS(2093),
    [sym_constant_keyword] = ACTIONS(2093),
    [sym_volatile_keyword] = ACTIONS(2093),
    [sym_non_volatile_keyword] = ACTIONS(2093),
    [sym_persistent_keyword] = ACTIONS(2093),
    [sym_dev_keyword] = ACTIONS(2093),
    [sym_devlev_keyword] = ACTIONS(2093),
    [sym_devchan_keyword] = ACTIONS(2093),
    [sym_if_keyword] = ACTIONS(2093),
    [sym_switch_keyword] = ACTIONS(2093),
    [sym_case_keyword] = ACTIONS(2093),
    [sym_default_keyword] = ACTIONS(2093),
    [sym_while_keyword] = ACTIONS(2093),
    [sym_for_keyword] = ACTIONS(2093),
    [sym_break_keyword] = ACTIONS(2093),
    [sym_continue_keyword] = ACTIONS(2093),
    [sym_return_keyword] = ACTIONS(2093),
    [sym_select_keyword] = ACTIONS(2093),
    [sym_active_keyword] = ACTIONS(2093),
    [sym_struct_keyword] = ACTIONS(2093),
    [sym_structure_keyword] = ACTIONS(2093),
    [sym_button_event_keyword] = ACTIONS(2093),
    [sym_channel_event_keyword] = ACTIONS(2093),
    [sym_level_event_keyword] = ACTIONS(2093),
    [sym_data_event_keyword] = ACTIONS(2093),
    [sym_timeline_event_keyword] = ACTIONS(2093),
    [sym_custom_event_keyword] = ACTIONS(2093),
    [sym_push_keyword] = ACTIONS(2093),
    [sym_release_keyword] = ACTIONS(2093),
    [aux_sym_on_keyword_token1] = ACTIONS(2093),
    [aux_sym_off_keyword_token1] = ACTIONS(2093),
    [sym_send_level_keyword] = ACTIONS(2093),
    [sym_send_string_keyword] = ACTIONS(2093),
    [sym_send_command_keyword] = ACTIONS(2093),
    [sym_clear_buffer_keyword] = ACTIONS(2093),
    [sym_create_buffer_keyword] = ACTIONS(2093),
    [sym_create_multi_buffer_keyword] = ACTIONS(2093),
    [sym_call_keyword] = ACTIONS(2093),
    [sym_system_call_keyword] = ACTIONS(2093),
    [sym_devchan_to_keyword] = ACTIONS(2093),
    [sym_devchan_min_to_keyword] = ACTIONS(2093),
    [sym_devchan_total_off_keyword] = ACTIONS(2093),
    [sym_devchan_pulse_keyword] = ACTIONS(2093),
    [sym_wait_keyword] = ACTIONS(2093),
    [sym_cancel_wait_keyword] = ACTIONS(2093),
    [sym_wait_until_keyword] = ACTIONS(2093),
    [sym_cancel_wait_until_keyword] = ACTIONS(2093),
    [sym_cancel_all_wait_keyword] = ACTIONS(2093),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2093),
    [sym_not] = ACTIONS(2093),
    [sym_bnot] = ACTIONS(2093),
    [sym___file__] = ACTIONS(2093),
    [sym___date__] = ACTIONS(2093),
    [sym___ldate__] = ACTIONS(2093),
    [sym___time__] = ACTIONS(2093),
    [sym___line__] = ACTIONS(2093),
    [sym___name__] = ACTIONS(2093),
    [sym___netlinx__] = ACTIONS(2093),
    [sym_netlinx_axi_version] = ACTIONS(2093),
    [sym_url_flg_tcp] = ACTIONS(2093),
    [sym_url_flg_accinfopresent] = ACTIONS(2093),
    [sym_url_flg_temp] = ACTIONS(2093),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2093),
    [sym_url_flg_stat_mask] = ACTIONS(2093),
    [sym_url_flg_stat_lookup] = ACTIONS(2093),
    [sym_url_flg_stat_connecting] = ACTIONS(2093),
    [sym_url_flg_stat_waiting] = ACTIONS(2093),
    [sym_url_flg_stat_connected] = ACTIONS(2093),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2093),
    [sym_first_virtual_device] = ACTIONS(2093),
    [sym_first_local_port] = ACTIONS(2093),
    [sym_dynamic_virtual_device] = ACTIONS(2093),
    [sym_timeline_once] = ACTIONS(2093),
    [sym_timeline_repeat] = ACTIONS(2093),
    [sym_timeline_absolute] = ACTIONS(2093),
    [sym_timeline_relative] = ACTIONS(2093),
    [sym_do_push_timed_infinite] = ACTIONS(2093),
    [sym_source_type_no_address] = ACTIONS(2093),
    [sym_source_type_neuron_id] = ACTIONS(2093),
    [sym_source_type_ip_address] = ACTIONS(2093),
    [sym_source_type_axlink] = ACTIONS(2093),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2093),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2093),
    [sym_source_type_ip_socket_address] = ACTIONS(2093),
    [sym_source_type_rs232] = ACTIONS(2093),
    [sym_source_type_internal] = ACTIONS(2093),
    [sym_source_type_mac_address] = ACTIONS(2093),
    [sym_source_type_ipv4_port] = ACTIONS(2093),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2093),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2093),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2093),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2093),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2093),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2093),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2093),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2093),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2093),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2093),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2093),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2093),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2093),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2093),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2093),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2093),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2093),
    [sym_true] = ACTIONS(2093),
    [sym_false] = ACTIONS(2093),
    [sym_file_read_only] = ACTIONS(2093),
    [sym_file_rw_new] = ACTIONS(2093),
    [sym_file_rw_append] = ACTIONS(2093),
    [sym_ip_tcp] = ACTIONS(2093),
    [sym_ip_udp] = ACTIONS(2093),
    [sym_ip_udp_2way] = ACTIONS(2093),
    [sym_xml_encode_types] = ACTIONS(2093),
    [sym_xml_encode_char_as_list] = ACTIONS(2093),
    [sym_xml_encode_le] = ACTIONS(2093),
    [sym_xml_decode_types] = ACTIONS(2093),
    [sym_xml_decode_no_preserve] = ACTIONS(2093),
    [sym_duet_dev_type_display_device] = ACTIONS(2093),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2093),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2093),
    [sym_duet_dev_type_io_device] = ACTIONS(2093),
    [sym_duet_dev_type_relay_device] = ACTIONS(2093),
    [sym_duet_dev_type_ups] = ACTIONS(2093),
    [sym_duet_dev_type_amplifier] = ACTIONS(2093),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2093),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2093),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2093),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2093),
    [sym_duet_dev_type_camera] = ACTIONS(2093),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2093),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2093),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2093),
    [sym_duet_dev_type_dss] = ACTIONS(2093),
    [sym_duet_dev_type_dvr] = ACTIONS(2093),
    [sym_duet_dev_type_disc_device] = ACTIONS(2093),
    [sym_duet_dev_type_document_camera] = ACTIONS(2093),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2093),
    [sym_duet_dev_type_hvac] = ACTIONS(2093),
    [sym_duet_dev_type_keypad] = ACTIONS(2093),
    [sym_duet_dev_type_light] = ACTIONS(2093),
    [sym_duet_dev_type_monitor] = ACTIONS(2093),
    [sym_duet_dev_type_motor] = ACTIONS(2093),
    [sym_duet_dev_type_multi_window] = ACTIONS(2093),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2093),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2093),
    [sym_duet_dev_type_receiver] = ACTIONS(2093),
    [sym_duet_dev_type_security_system] = ACTIONS(2093),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2093),
    [sym_duet_dev_type_settop_box] = ACTIONS(2093),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2093),
    [sym_duet_dev_type_switcher] = ACTIONS(2093),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2093),
    [sym_duet_dev_type_tv] = ACTIONS(2093),
    [sym_duet_dev_type_utility] = ACTIONS(2093),
    [sym_duet_dev_type_vcr] = ACTIONS(2093),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2093),
    [sym_duet_dev_type_video_processor] = ACTIONS(2093),
    [sym_duet_dev_type_video_projector] = ACTIONS(2093),
    [sym_duet_dev_type_video_wall] = ACTIONS(2093),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2093),
    [sym_duet_dev_type_weather] = ACTIONS(2093),
    [sym_duet_dev_not_polled] = ACTIONS(2093),
    [sym_duet_dev_polled] = ACTIONS(2093),
    [sym_clkmgr_mode_network] = ACTIONS(2093),
    [sym_clkmgr_mode_standalone] = ACTIONS(2093),
    [sym_normal_standby] = ACTIONS(2093),
    [sym_normal_wake] = ACTIONS(2093),
    [sym_ip_multicast_ttl_option] = ACTIONS(2093),
    [sym_ip_ttl_subnet] = ACTIONS(2093),
    [sym_ip_ttl_site] = ACTIONS(2093),
    [sym_ip_ttl_region] = ACTIONS(2093),
    [sym_ip_ttl_continent] = ACTIONS(2093),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2093),
    [sym_ip_nodelay_on] = ACTIONS(2093),
    [sym_ip_nodelay_off] = ACTIONS(2093),
    [sym_amx_error] = ACTIONS(2093),
    [sym_amx_warning] = ACTIONS(2093),
    [sym_amx_info] = ACTIONS(2093),
    [sym_amx_debug] = ACTIONS(2093),
    [sym_netlinx_login_success] = ACTIONS(2093),
    [sym_netlinx_login_fail] = ACTIONS(2093),
    [sym_netlinx_logout] = ACTIONS(2093),
    [sym_null_str] = ACTIONS(2093),
    [sym_smtp_address] = ACTIONS(2093),
    [sym_smtp_port_number] = ACTIONS(2093),
    [sym_smtp_username] = ACTIONS(2093),
    [sym_smtp_password] = ACTIONS(2093),
    [sym_smtp_require_tls] = ACTIONS(2093),
    [sym_smtp_from] = ACTIONS(2093),
    [sym_smtp_tls_true] = ACTIONS(2093),
    [sym_smtp_tls_false] = ACTIONS(2093),
    [sym_listview_on_row_select_event] = ACTIONS(2093),
    [sym_data_structure_unknown] = ACTIONS(2093),
    [sym_data_structure_datafeed] = ACTIONS(2093),
    [sym_data_structure_datarecordset] = ACTIONS(2093),
    [sym_data_structure_datarecord] = ACTIONS(2093),
    [sym_data_structure_datafield] = ACTIONS(2093),
    [sym_data_max_content_field_count] = ACTIONS(2093),
    [sym_data_max_metadata_field_count] = ACTIONS(2093),
    [sym_data_max_id_length] = ACTIONS(2093),
    [sym_data_max_name_length] = ACTIONS(2093),
    [sym_data_max_description_length] = ACTIONS(2093),
    [sym_data_max_type_length] = ACTIONS(2093),
    [sym_data_max_format_length] = ACTIONS(2093),
    [sym_data_max_source_length] = ACTIONS(2093),
    [sym_data_max_value_length] = ACTIONS(2093),
    [sym_data_max_label_length] = ACTIONS(2093),
    [sym_data_max_url_length] = ACTIONS(2093),
    [sym_max_last_login_info_length] = ACTIONS(2093),
    [sym_data_type_unknown] = ACTIONS(2093),
    [sym_data_type_string] = ACTIONS(2093),
    [sym_data_type_datetime] = ACTIONS(2093),
    [sym_data_type_date] = ACTIONS(2093),
    [sym_data_type_time] = ACTIONS(2093),
    [sym_data_type_image] = ACTIONS(2093),
    [sym_data_format_url] = ACTIONS(2093),
    [sym_data_format_phone] = ACTIONS(2093),
    [sym_data_format_email] = ACTIONS(2093),
    [sym_data_format_iso8601] = ACTIONS(2093),
    [sym_auditlog_priv] = ACTIONS(2093),
    [sym_configuration_priv] = ACTIONS(2093),
    [sym_deviceconfig_priv] = ACTIONS(2093),
    [sym_ftp_priv] = ACTIONS(2093),
    [sym_http_priv] = ACTIONS(2093),
    [sym_networkconfig_priv] = ACTIONS(2093),
    [sym_programport_priv] = ACTIONS(2093),
    [sym_remoteui_priv] = ACTIONS(2093),
    [sym_securitycontrol_priv] = ACTIONS(2093),
    [sym_softwaremanagement_priv] = ACTIONS(2093),
    [sym_terminal_priv] = ACTIONS(2093),
    [sym_tpadmin_priv] = ACTIONS(2093),
    [sym_user1_priv] = ACTIONS(2093),
    [sym_user2_priv] = ACTIONS(2093),
    [sym_user3_priv] = ACTIONS(2093),
    [sym_user4_priv] = ACTIONS(2093),
    [sym_usermanagement_priv] = ACTIONS(2093),
    [sym_valid_account] = ACTIONS(2093),
    [sym_err_name_invalid] = ACTIONS(2093),
    [sym_err_password_invalid] = ACTIONS(2093),
    [sym_err_invalid_account] = ACTIONS(2093),
    [sym_err_no_authorization] = ACTIONS(2093),
    [sym_err_invalid_parameter] = ACTIONS(2093),
    [sym_err_user_locked_out] = ACTIONS(2093),
    [sym_err_user_account_expired] = ACTIONS(2093),
    [sym_err_general_error] = ACTIONS(2093),
    [sym_err_authentication_error] = ACTIONS(2093),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2093),
    [sym_tls_validate_certificate] = ACTIONS(2093),
    [sym_tdata] = ACTIONS(2093),
    [sym_tchannel] = ACTIONS(2093),
    [sym_tlevel] = ACTIONS(2093),
    [sym_tbutton] = ACTIONS(2093),
    [sym_ttimeline] = ACTIONS(2093),
    [sym_tcustom] = ACTIONS(2093),
    [sym_url_struct] = ACTIONS(2093),
    [sym_dns_struct] = ACTIONS(2093),
    [sym_ip_address_struct] = ACTIONS(2093),
    [sym_dev_info_struct] = ACTIONS(2093),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2093),
    [sym_data_feed] = ACTIONS(2093),
    [sym_data_field] = ACTIONS(2093),
    [sym_data_record] = ACTIONS(2093),
    [sym_wc_data_feed] = ACTIONS(2093),
    [sym_wc_data_field] = ACTIONS(2093),
    [sym_wc_data_record] = ACTIONS(2093),
    [sym_last_login_info] = ACTIONS(2093),
    [sym____reserved___] = ACTIONS(2093),
    [sym_date] = ACTIONS(2093),
    [sym_day] = ACTIONS(2093),
    [sym_time] = ACTIONS(2093),
    [sym_ldate] = ACTIONS(2093),
    [sym_push_device] = ACTIONS(2093),
    [sym_push_channel] = ACTIONS(2093),
    [sym_push_devchan] = ACTIONS(2093),
    [sym_release_device] = ACTIONS(2093),
    [sym_release_channel] = ACTIONS(2093),
    [sym_release_devchan] = ACTIONS(2093),
    [sym_master_slot] = ACTIONS(2093),
    [sym_get_pulse_time] = ACTIONS(2093),
    [sym_get_timer] = ACTIONS(2093),
    [sym_master_sn] = ACTIONS(2093),
    [sym_system_number] = ACTIONS(2093),
    [sym_button] = ACTIONS(2093),
    [sym_dv_channel] = ACTIONS(2093),
    [sym_data] = ACTIONS(2093),
    [sym_level] = ACTIONS(2093),
    [sym_timeline] = ACTIONS(2093),
    [sym_channel] = ACTIONS(2093),
    [sym_custom] = ACTIONS(2093),
    [sym_atoi] = ACTIONS(2093),
    [sym_atol] = ACTIONS(2093),
    [sym_atof] = ACTIONS(2093),
    [sym_itoa] = ACTIONS(2093),
    [sym_itohex] = ACTIONS(2093),
    [sym_hextoi] = ACTIONS(2093),
    [sym_ftoa] = ACTIONS(2093),
    [sym_find_string] = ACTIONS(2093),
    [sym_left_string] = ACTIONS(2093),
    [sym_right_string] = ACTIONS(2093),
    [sym_mid_string] = ACTIONS(2093),
    [sym_remove_string] = ACTIONS(2093),
    [sym_get_buffer_string] = ACTIONS(2093),
    [sym_get_buffer_char] = ACTIONS(2093),
    [sym_get_multi_buffer_string] = ACTIONS(2093),
    [sym_length_string] = ACTIONS(2093),
    [sym_max_length_string] = ACTIONS(2093),
    [sym_set_length_string] = ACTIONS(2093),
    [sym_compare_string] = ACTIONS(2093),
    [sym_upper_string] = ACTIONS(2093),
    [sym_lower_string] = ACTIONS(2093),
    [sym_get_last] = ACTIONS(2093),
    [sym_date_to_day] = ACTIONS(2093),
    [sym_date_to_month] = ACTIONS(2093),
    [sym_date_to_year] = ACTIONS(2093),
    [sym_time_to_hour] = ACTIONS(2093),
    [sym_time_to_minute] = ACTIONS(2093),
    [sym_time_to_second] = ACTIONS(2093),
    [sym_day_of_week] = ACTIONS(2093),
    [sym_set_pulse_time] = ACTIONS(2093),
    [sym_set_timer] = ACTIONS(2093),
    [sym_file_open] = ACTIONS(2093),
    [sym_file_close] = ACTIONS(2093),
    [sym_file_read] = ACTIONS(2093),
    [sym_file_read_line] = ACTIONS(2093),
    [sym_file_write] = ACTIONS(2093),
    [sym_file_write_line] = ACTIONS(2093),
    [sym_file_copy] = ACTIONS(2093),
    [sym_file_delete] = ACTIONS(2093),
    [sym_file_rename] = ACTIONS(2093),
    [sym_file_dir] = ACTIONS(2093),
    [sym_file_getdir] = ACTIONS(2093),
    [sym_file_setdir] = ACTIONS(2093),
    [sym_file_create_dir] = ACTIONS(2093),
    [sym_file_remove_dir] = ACTIONS(2093),
    [sym_file_seek] = ACTIONS(2093),
    [sym_length_array] = ACTIONS(2093),
    [sym_max_length_array] = ACTIONS(2093),
    [sym_set_length_array] = ACTIONS(2093),
    [sym_ip_client_open] = ACTIONS(2093),
    [sym_ip_client_close] = ACTIONS(2093),
    [sym_ip_server_open] = ACTIONS(2093),
    [sym_ip_server_close] = ACTIONS(2093),
    [sym_ip_bound_client_open] = ACTIONS(2093),
    [sym_ip_mc_server_open] = ACTIONS(2093),
    [sym_ip_set_option] = ACTIONS(2093),
    [sym_ssh_client_open] = ACTIONS(2093),
    [sym_ssh_client_close] = ACTIONS(2093),
    [sym_tls_client_open] = ACTIONS(2093),
    [sym_tls_client_close] = ACTIONS(2093),
    [sym_do_push] = ACTIONS(2093),
    [sym_do_push_timed] = ACTIONS(2093),
    [sym_do_release] = ACTIONS(2093),
    [sym_device_id] = ACTIONS(2093),
    [sym_device_info] = ACTIONS(2093),
    [sym_device_id_string] = ACTIONS(2093),
    [sym_device_standby] = ACTIONS(2093),
    [sym_device_wake] = ACTIONS(2093),
    [sym_timeline_create] = ACTIONS(2093),
    [sym_timeline_kill] = ACTIONS(2093),
    [sym_timeline_reload] = ACTIONS(2093),
    [sym_timeline_pause] = ACTIONS(2093),
    [sym_timeline_restart] = ACTIONS(2093),
    [sym_timeline_set] = ACTIONS(2093),
    [sym_timeline_get] = ACTIONS(2093),
    [sym_timeline_active] = ACTIONS(2093),
    [sym_timeline_dynamic_id] = ACTIONS(2093),
    [sym_variable_to_string] = ACTIONS(2093),
    [sym_string_to_variable] = ACTIONS(2093),
    [sym_length_variable_to_string] = ACTIONS(2093),
    [sym_type_cast] = ACTIONS(2093),
    [sym_variable_to_xml] = ACTIONS(2093),
    [sym_xml_to_variable] = ACTIONS(2093),
    [sym_length_variable_to_xml] = ACTIONS(2093),
    [sym_redirect_string] = ACTIONS(2093),
    [sym_random_number] = ACTIONS(2093),
    [sym_get_unique_id] = ACTIONS(2093),
    [sym_get_system_number] = ACTIONS(2093),
    [sym_set_system_number] = ACTIONS(2093),
    [sym_get_serial_number] = ACTIONS(2093),
    [sym_get_master_build] = ACTIONS(2093),
    [sym_set_validation_code] = ACTIONS(2093),
    [sym_reboot] = ACTIONS(2093),
    [sym_load_duet_module] = ACTIONS(2093),
    [sym_unload_duet_module] = ACTIONS(2093),
    [sym_duet_mem_size_set] = ACTIONS(2093),
    [sym_duet_mem_size_get] = ACTIONS(2093),
    [sym_dynamic_application_device] = ACTIONS(2093),
    [sym_dynamic_polled_port] = ACTIONS(2093),
    [sym_static_port_binding] = ACTIONS(2093),
    [sym_static_ip_binding] = ACTIONS(2093),
    [sym_get_dns_list] = ACTIONS(2093),
    [sym_set_dns_list] = ACTIONS(2093),
    [sym_get_ip_address] = ACTIONS(2093),
    [sym_set_ip_address] = ACTIONS(2093),
    [sym_get_url_list] = ACTIONS(2093),
    [sym_add_url_entry] = ACTIONS(2093),
    [sym_delete_url_entry] = ACTIONS(2093),
    [sym_abs_value] = ACTIONS(2093),
    [sym_max_value] = ACTIONS(2093),
    [sym_min_value] = ACTIONS(2093),
    [sym_exp_value] = ACTIONS(2093),
    [sym_log_value] = ACTIONS(2093),
    [sym_log10_value] = ACTIONS(2093),
    [sym_power_value] = ACTIONS(2093),
    [sym_sqrt_value] = ACTIONS(2093),
    [sym_raw_be] = ACTIONS(2093),
    [sym_raw_le] = ACTIONS(2093),
    [sym_internal_queue_size_set] = ACTIONS(2093),
    [sym_internal_queue_size_get] = ACTIONS(2093),
    [sym_internal_threshold_set] = ACTIONS(2093),
    [sym_internal_threshold_get] = ACTIONS(2093),
    [sym_rebuilt_event] = ACTIONS(2093),
    [sym_do_custom_event] = ACTIONS(2093),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2093),
    [sym_clkmgr_set_clk_source] = ACTIONS(2093),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2093),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2093),
    [sym_clkmgr_get_timezone] = ACTIONS(2093),
    [sym_clkmgr_set_timezone] = ACTIONS(2093),
    [sym_clkmgr_get_resync_period] = ACTIONS(2093),
    [sym_clkmgr_set_resync_period] = ACTIONS(2093),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2093),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2093),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2093),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2093),
    [sym_clkmgr_get_timeservers] = ACTIONS(2093),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2093),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2093),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2093),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2093),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2093),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2093),
    [sym_set_log_level] = ACTIONS(2093),
    [sym_get_log_level] = ACTIONS(2093),
    [sym_amx_log] = ACTIONS(2093),
    [sym_smtp_server_config_set] = ACTIONS(2093),
    [sym_smtp_server_config_get] = ACTIONS(2093),
    [sym_smtp_send] = ACTIONS(2093),
    [sym_validate_netlinx_account] = ACTIONS(2093),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2093),
    [sym_authenticate_certificate] = ACTIONS(2093),
    [sym_audit_netlinx_session_event] = ACTIONS(2093),
    [sym_audit_netlinx_generic_event] = ACTIONS(2093),
    [sym_get_available_flash_disk_space] = ACTIONS(2093),
    [sym_get_max_flash_disk_space] = ACTIONS(2093),
    [sym_data_create_feed] = ACTIONS(2093),
    [sym_data_delete_feed] = ACTIONS(2093),
    [sym_data_publish_feed] = ACTIONS(2093),
    [sym_data_get_published_feed] = ACTIONS(2093),
    [sym_data_add_record] = ACTIONS(2093),
    [sym_data_get_event_record] = ACTIONS(2093),
    [sym__wc_data_create_feed] = ACTIONS(2093),
    [sym__wc_data_add_record] = ACTIONS(2093),
    [sym__wc_data_get_event_record] = ACTIONS(2093),
    [sym_format] = ACTIONS(2093),
    [sym_astro_clock] = ACTIONS(2093),
    [sym_set_outdoor_temperature] = ACTIONS(2093),
    [sym_set_virtual_level_count] = ACTIONS(2093),
    [sym_set_virtual_channel_count] = ACTIONS(2093),
    [sym_set_virtual_port_count] = ACTIONS(2093),
    [sym__wc_encode_int] = ACTIONS(2093),
    [sym__wc_decode_int] = ACTIONS(2093),
    [sym__wc_int] = ACTIONS(2093),
    [sym__wc_to_ch_int] = ACTIONS(2093),
    [sym__ch_to_wc_int] = ACTIONS(2093),
    [sym__wc_find_string_int] = ACTIONS(2093),
    [sym__wc_left_string_int] = ACTIONS(2093),
    [sym__wc_lower_string_int] = ACTIONS(2093),
    [sym__wc_mid_string_int] = ACTIONS(2093),
    [sym__wc_remove_string_int] = ACTIONS(2093),
    [sym__wc_right_string_int] = ACTIONS(2093),
    [sym__wc_upper_string_int] = ACTIONS(2093),
    [sym__wc_compare_string_int] = ACTIONS(2093),
    [sym__wc_get_buffer_char_int] = ACTIONS(2093),
    [sym__wc_get_buffer_string_int] = ACTIONS(2093),
    [sym__wc_concat_string_int] = ACTIONS(2093),
    [sym__wc_file_open_int] = ACTIONS(2093),
    [sym__wc_file_close_int] = ACTIONS(2093),
    [sym__wc_file_read_int] = ACTIONS(2093),
    [sym__wc_file_read_line_int] = ACTIONS(2093),
    [sym__wc_file_write_int] = ACTIONS(2093),
    [sym__wc_file_write_line_int] = ACTIONS(2093),
    [aux_sym_preproc_include_token1] = ACTIONS(2093),
    [aux_sym_preproc_define_token1] = ACTIONS(2095),
    [aux_sym_preproc_warn_token1] = ACTIONS(2095),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2095),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2095),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2095),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2095),
    [aux_sym_preproc_else_token1] = ACTIONS(2095),
    [anon_sym_LBRACK] = ACTIONS(2095),
    [anon_sym_LBRACE] = ACTIONS(2095),
    [anon_sym_LPAREN] = ACTIONS(2093),
    [anon_sym_BANG] = ACTIONS(2095),
    [anon_sym_TILDE] = ACTIONS(2095),
    [anon_sym_DASH] = ACTIONS(2093),
    [anon_sym_PLUS] = ACTIONS(2093),
    [anon_sym_DASH_DASH] = ACTIONS(2095),
    [anon_sym_PLUS_PLUS] = ACTIONS(2095),
    [anon_sym_DQUOTE] = ACTIONS(2095),
    [anon_sym_SQUOTE] = ACTIONS(2095),
    [sym_number_literal] = ACTIONS(2095),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(368)] = {
    [sym_identifier] = ACTIONS(2097),
    [sym_program_name_keyword] = ACTIONS(2097),
    [sym_module_name_keyword] = ACTIONS(2097),
    [sym_define_device_keyword] = ACTIONS(2097),
    [sym_define_combine_keyword] = ACTIONS(2097),
    [sym_define_constant_keyword] = ACTIONS(2097),
    [sym_define_type_keyword] = ACTIONS(2097),
    [sym_define_variable_keyword] = ACTIONS(2097),
    [sym_define_system_variable_keyword] = ACTIONS(2097),
    [sym_define_connect_level_keyword] = ACTIONS(2097),
    [sym_define_latching_keyword] = ACTIONS(2097),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2097),
    [sym_define_toggling_keyword] = ACTIONS(2097),
    [sym_define_call_keyword] = ACTIONS(2097),
    [sym_define_function_keyword] = ACTIONS(2097),
    [sym_define_library_function_keyword] = ACTIONS(2097),
    [sym_define_start_keyword] = ACTIONS(2097),
    [sym_define_event_keyword] = ACTIONS(2097),
    [sym_define_program_keyword] = ACTIONS(2097),
    [sym_define_module_keyword] = ACTIONS(2097),
    [sym_char_keyword] = ACTIONS(2097),
    [sym_widechar_keyword] = ACTIONS(2097),
    [sym_integer_keyword] = ACTIONS(2097),
    [sym_sinteger_keyword] = ACTIONS(2097),
    [sym_long_keyword] = ACTIONS(2097),
    [sym_slong_keyword] = ACTIONS(2097),
    [sym_float_keyword] = ACTIONS(2097),
    [sym_double_keyword] = ACTIONS(2097),
    [sym_variant_keyword] = ACTIONS(2097),
    [sym_variantarray_keyword] = ACTIONS(2097),
    [sym_local_var_keyword] = ACTIONS(2097),
    [sym_stack_var_keyword] = ACTIONS(2097),
    [sym_constant_keyword] = ACTIONS(2097),
    [sym_volatile_keyword] = ACTIONS(2097),
    [sym_non_volatile_keyword] = ACTIONS(2097),
    [sym_persistent_keyword] = ACTIONS(2097),
    [sym_dev_keyword] = ACTIONS(2097),
    [sym_devlev_keyword] = ACTIONS(2097),
    [sym_devchan_keyword] = ACTIONS(2097),
    [sym_if_keyword] = ACTIONS(2097),
    [sym_switch_keyword] = ACTIONS(2097),
    [sym_case_keyword] = ACTIONS(2097),
    [sym_default_keyword] = ACTIONS(2097),
    [sym_while_keyword] = ACTIONS(2097),
    [sym_for_keyword] = ACTIONS(2097),
    [sym_break_keyword] = ACTIONS(2097),
    [sym_continue_keyword] = ACTIONS(2097),
    [sym_return_keyword] = ACTIONS(2097),
    [sym_select_keyword] = ACTIONS(2097),
    [sym_active_keyword] = ACTIONS(2097),
    [sym_struct_keyword] = ACTIONS(2097),
    [sym_structure_keyword] = ACTIONS(2097),
    [sym_button_event_keyword] = ACTIONS(2097),
    [sym_channel_event_keyword] = ACTIONS(2097),
    [sym_level_event_keyword] = ACTIONS(2097),
    [sym_data_event_keyword] = ACTIONS(2097),
    [sym_timeline_event_keyword] = ACTIONS(2097),
    [sym_custom_event_keyword] = ACTIONS(2097),
    [sym_push_keyword] = ACTIONS(2097),
    [sym_release_keyword] = ACTIONS(2097),
    [aux_sym_on_keyword_token1] = ACTIONS(2097),
    [aux_sym_off_keyword_token1] = ACTIONS(2097),
    [sym_send_level_keyword] = ACTIONS(2097),
    [sym_send_string_keyword] = ACTIONS(2097),
    [sym_send_command_keyword] = ACTIONS(2097),
    [sym_clear_buffer_keyword] = ACTIONS(2097),
    [sym_create_buffer_keyword] = ACTIONS(2097),
    [sym_create_multi_buffer_keyword] = ACTIONS(2097),
    [sym_call_keyword] = ACTIONS(2097),
    [sym_system_call_keyword] = ACTIONS(2097),
    [sym_devchan_to_keyword] = ACTIONS(2097),
    [sym_devchan_min_to_keyword] = ACTIONS(2097),
    [sym_devchan_total_off_keyword] = ACTIONS(2097),
    [sym_devchan_pulse_keyword] = ACTIONS(2097),
    [sym_wait_keyword] = ACTIONS(2097),
    [sym_cancel_wait_keyword] = ACTIONS(2097),
    [sym_wait_until_keyword] = ACTIONS(2097),
    [sym_cancel_wait_until_keyword] = ACTIONS(2097),
    [sym_cancel_all_wait_keyword] = ACTIONS(2097),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2097),
    [sym_not] = ACTIONS(2097),
    [sym_bnot] = ACTIONS(2097),
    [sym___file__] = ACTIONS(2097),
    [sym___date__] = ACTIONS(2097),
    [sym___ldate__] = ACTIONS(2097),
    [sym___time__] = ACTIONS(2097),
    [sym___line__] = ACTIONS(2097),
    [sym___name__] = ACTIONS(2097),
    [sym___netlinx__] = ACTIONS(2097),
    [sym_netlinx_axi_version] = ACTIONS(2097),
    [sym_url_flg_tcp] = ACTIONS(2097),
    [sym_url_flg_accinfopresent] = ACTIONS(2097),
    [sym_url_flg_temp] = ACTIONS(2097),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2097),
    [sym_url_flg_stat_mask] = ACTIONS(2097),
    [sym_url_flg_stat_lookup] = ACTIONS(2097),
    [sym_url_flg_stat_connecting] = ACTIONS(2097),
    [sym_url_flg_stat_waiting] = ACTIONS(2097),
    [sym_url_flg_stat_connected] = ACTIONS(2097),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2097),
    [sym_first_virtual_device] = ACTIONS(2097),
    [sym_first_local_port] = ACTIONS(2097),
    [sym_dynamic_virtual_device] = ACTIONS(2097),
    [sym_timeline_once] = ACTIONS(2097),
    [sym_timeline_repeat] = ACTIONS(2097),
    [sym_timeline_absolute] = ACTIONS(2097),
    [sym_timeline_relative] = ACTIONS(2097),
    [sym_do_push_timed_infinite] = ACTIONS(2097),
    [sym_source_type_no_address] = ACTIONS(2097),
    [sym_source_type_neuron_id] = ACTIONS(2097),
    [sym_source_type_ip_address] = ACTIONS(2097),
    [sym_source_type_axlink] = ACTIONS(2097),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2097),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2097),
    [sym_source_type_ip_socket_address] = ACTIONS(2097),
    [sym_source_type_rs232] = ACTIONS(2097),
    [sym_source_type_internal] = ACTIONS(2097),
    [sym_source_type_mac_address] = ACTIONS(2097),
    [sym_source_type_ipv4_port] = ACTIONS(2097),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2097),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2097),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2097),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2097),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2097),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2097),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2097),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2097),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2097),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2097),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2097),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2097),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2097),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2097),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2097),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2097),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2097),
    [sym_true] = ACTIONS(2097),
    [sym_false] = ACTIONS(2097),
    [sym_file_read_only] = ACTIONS(2097),
    [sym_file_rw_new] = ACTIONS(2097),
    [sym_file_rw_append] = ACTIONS(2097),
    [sym_ip_tcp] = ACTIONS(2097),
    [sym_ip_udp] = ACTIONS(2097),
    [sym_ip_udp_2way] = ACTIONS(2097),
    [sym_xml_encode_types] = ACTIONS(2097),
    [sym_xml_encode_char_as_list] = ACTIONS(2097),
    [sym_xml_encode_le] = ACTIONS(2097),
    [sym_xml_decode_types] = ACTIONS(2097),
    [sym_xml_decode_no_preserve] = ACTIONS(2097),
    [sym_duet_dev_type_display_device] = ACTIONS(2097),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2097),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2097),
    [sym_duet_dev_type_io_device] = ACTIONS(2097),
    [sym_duet_dev_type_relay_device] = ACTIONS(2097),
    [sym_duet_dev_type_ups] = ACTIONS(2097),
    [sym_duet_dev_type_amplifier] = ACTIONS(2097),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2097),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2097),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2097),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2097),
    [sym_duet_dev_type_camera] = ACTIONS(2097),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2097),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2097),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2097),
    [sym_duet_dev_type_dss] = ACTIONS(2097),
    [sym_duet_dev_type_dvr] = ACTIONS(2097),
    [sym_duet_dev_type_disc_device] = ACTIONS(2097),
    [sym_duet_dev_type_document_camera] = ACTIONS(2097),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2097),
    [sym_duet_dev_type_hvac] = ACTIONS(2097),
    [sym_duet_dev_type_keypad] = ACTIONS(2097),
    [sym_duet_dev_type_light] = ACTIONS(2097),
    [sym_duet_dev_type_monitor] = ACTIONS(2097),
    [sym_duet_dev_type_motor] = ACTIONS(2097),
    [sym_duet_dev_type_multi_window] = ACTIONS(2097),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2097),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2097),
    [sym_duet_dev_type_receiver] = ACTIONS(2097),
    [sym_duet_dev_type_security_system] = ACTIONS(2097),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2097),
    [sym_duet_dev_type_settop_box] = ACTIONS(2097),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2097),
    [sym_duet_dev_type_switcher] = ACTIONS(2097),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2097),
    [sym_duet_dev_type_tv] = ACTIONS(2097),
    [sym_duet_dev_type_utility] = ACTIONS(2097),
    [sym_duet_dev_type_vcr] = ACTIONS(2097),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2097),
    [sym_duet_dev_type_video_processor] = ACTIONS(2097),
    [sym_duet_dev_type_video_projector] = ACTIONS(2097),
    [sym_duet_dev_type_video_wall] = ACTIONS(2097),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2097),
    [sym_duet_dev_type_weather] = ACTIONS(2097),
    [sym_duet_dev_not_polled] = ACTIONS(2097),
    [sym_duet_dev_polled] = ACTIONS(2097),
    [sym_clkmgr_mode_network] = ACTIONS(2097),
    [sym_clkmgr_mode_standalone] = ACTIONS(2097),
    [sym_normal_standby] = ACTIONS(2097),
    [sym_normal_wake] = ACTIONS(2097),
    [sym_ip_multicast_ttl_option] = ACTIONS(2097),
    [sym_ip_ttl_subnet] = ACTIONS(2097),
    [sym_ip_ttl_site] = ACTIONS(2097),
    [sym_ip_ttl_region] = ACTIONS(2097),
    [sym_ip_ttl_continent] = ACTIONS(2097),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2097),
    [sym_ip_nodelay_on] = ACTIONS(2097),
    [sym_ip_nodelay_off] = ACTIONS(2097),
    [sym_amx_error] = ACTIONS(2097),
    [sym_amx_warning] = ACTIONS(2097),
    [sym_amx_info] = ACTIONS(2097),
    [sym_amx_debug] = ACTIONS(2097),
    [sym_netlinx_login_success] = ACTIONS(2097),
    [sym_netlinx_login_fail] = ACTIONS(2097),
    [sym_netlinx_logout] = ACTIONS(2097),
    [sym_null_str] = ACTIONS(2097),
    [sym_smtp_address] = ACTIONS(2097),
    [sym_smtp_port_number] = ACTIONS(2097),
    [sym_smtp_username] = ACTIONS(2097),
    [sym_smtp_password] = ACTIONS(2097),
    [sym_smtp_require_tls] = ACTIONS(2097),
    [sym_smtp_from] = ACTIONS(2097),
    [sym_smtp_tls_true] = ACTIONS(2097),
    [sym_smtp_tls_false] = ACTIONS(2097),
    [sym_listview_on_row_select_event] = ACTIONS(2097),
    [sym_data_structure_unknown] = ACTIONS(2097),
    [sym_data_structure_datafeed] = ACTIONS(2097),
    [sym_data_structure_datarecordset] = ACTIONS(2097),
    [sym_data_structure_datarecord] = ACTIONS(2097),
    [sym_data_structure_datafield] = ACTIONS(2097),
    [sym_data_max_content_field_count] = ACTIONS(2097),
    [sym_data_max_metadata_field_count] = ACTIONS(2097),
    [sym_data_max_id_length] = ACTIONS(2097),
    [sym_data_max_name_length] = ACTIONS(2097),
    [sym_data_max_description_length] = ACTIONS(2097),
    [sym_data_max_type_length] = ACTIONS(2097),
    [sym_data_max_format_length] = ACTIONS(2097),
    [sym_data_max_source_length] = ACTIONS(2097),
    [sym_data_max_value_length] = ACTIONS(2097),
    [sym_data_max_label_length] = ACTIONS(2097),
    [sym_data_max_url_length] = ACTIONS(2097),
    [sym_max_last_login_info_length] = ACTIONS(2097),
    [sym_data_type_unknown] = ACTIONS(2097),
    [sym_data_type_string] = ACTIONS(2097),
    [sym_data_type_datetime] = ACTIONS(2097),
    [sym_data_type_date] = ACTIONS(2097),
    [sym_data_type_time] = ACTIONS(2097),
    [sym_data_type_image] = ACTIONS(2097),
    [sym_data_format_url] = ACTIONS(2097),
    [sym_data_format_phone] = ACTIONS(2097),
    [sym_data_format_email] = ACTIONS(2097),
    [sym_data_format_iso8601] = ACTIONS(2097),
    [sym_auditlog_priv] = ACTIONS(2097),
    [sym_configuration_priv] = ACTIONS(2097),
    [sym_deviceconfig_priv] = ACTIONS(2097),
    [sym_ftp_priv] = ACTIONS(2097),
    [sym_http_priv] = ACTIONS(2097),
    [sym_networkconfig_priv] = ACTIONS(2097),
    [sym_programport_priv] = ACTIONS(2097),
    [sym_remoteui_priv] = ACTIONS(2097),
    [sym_securitycontrol_priv] = ACTIONS(2097),
    [sym_softwaremanagement_priv] = ACTIONS(2097),
    [sym_terminal_priv] = ACTIONS(2097),
    [sym_tpadmin_priv] = ACTIONS(2097),
    [sym_user1_priv] = ACTIONS(2097),
    [sym_user2_priv] = ACTIONS(2097),
    [sym_user3_priv] = ACTIONS(2097),
    [sym_user4_priv] = ACTIONS(2097),
    [sym_usermanagement_priv] = ACTIONS(2097),
    [sym_valid_account] = ACTIONS(2097),
    [sym_err_name_invalid] = ACTIONS(2097),
    [sym_err_password_invalid] = ACTIONS(2097),
    [sym_err_invalid_account] = ACTIONS(2097),
    [sym_err_no_authorization] = ACTIONS(2097),
    [sym_err_invalid_parameter] = ACTIONS(2097),
    [sym_err_user_locked_out] = ACTIONS(2097),
    [sym_err_user_account_expired] = ACTIONS(2097),
    [sym_err_general_error] = ACTIONS(2097),
    [sym_err_authentication_error] = ACTIONS(2097),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2097),
    [sym_tls_validate_certificate] = ACTIONS(2097),
    [sym_tdata] = ACTIONS(2097),
    [sym_tchannel] = ACTIONS(2097),
    [sym_tlevel] = ACTIONS(2097),
    [sym_tbutton] = ACTIONS(2097),
    [sym_ttimeline] = ACTIONS(2097),
    [sym_tcustom] = ACTIONS(2097),
    [sym_url_struct] = ACTIONS(2097),
    [sym_dns_struct] = ACTIONS(2097),
    [sym_ip_address_struct] = ACTIONS(2097),
    [sym_dev_info_struct] = ACTIONS(2097),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2097),
    [sym_data_feed] = ACTIONS(2097),
    [sym_data_field] = ACTIONS(2097),
    [sym_data_record] = ACTIONS(2097),
    [sym_wc_data_feed] = ACTIONS(2097),
    [sym_wc_data_field] = ACTIONS(2097),
    [sym_wc_data_record] = ACTIONS(2097),
    [sym_last_login_info] = ACTIONS(2097),
    [sym____reserved___] = ACTIONS(2097),
    [sym_date] = ACTIONS(2097),
    [sym_day] = ACTIONS(2097),
    [sym_time] = ACTIONS(2097),
    [sym_ldate] = ACTIONS(2097),
    [sym_push_device] = ACTIONS(2097),
    [sym_push_channel] = ACTIONS(2097),
    [sym_push_devchan] = ACTIONS(2097),
    [sym_release_device] = ACTIONS(2097),
    [sym_release_channel] = ACTIONS(2097),
    [sym_release_devchan] = ACTIONS(2097),
    [sym_master_slot] = ACTIONS(2097),
    [sym_get_pulse_time] = ACTIONS(2097),
    [sym_get_timer] = ACTIONS(2097),
    [sym_master_sn] = ACTIONS(2097),
    [sym_system_number] = ACTIONS(2097),
    [sym_button] = ACTIONS(2097),
    [sym_dv_channel] = ACTIONS(2097),
    [sym_data] = ACTIONS(2097),
    [sym_level] = ACTIONS(2097),
    [sym_timeline] = ACTIONS(2097),
    [sym_channel] = ACTIONS(2097),
    [sym_custom] = ACTIONS(2097),
    [sym_atoi] = ACTIONS(2097),
    [sym_atol] = ACTIONS(2097),
    [sym_atof] = ACTIONS(2097),
    [sym_itoa] = ACTIONS(2097),
    [sym_itohex] = ACTIONS(2097),
    [sym_hextoi] = ACTIONS(2097),
    [sym_ftoa] = ACTIONS(2097),
    [sym_find_string] = ACTIONS(2097),
    [sym_left_string] = ACTIONS(2097),
    [sym_right_string] = ACTIONS(2097),
    [sym_mid_string] = ACTIONS(2097),
    [sym_remove_string] = ACTIONS(2097),
    [sym_get_buffer_string] = ACTIONS(2097),
    [sym_get_buffer_char] = ACTIONS(2097),
    [sym_get_multi_buffer_string] = ACTIONS(2097),
    [sym_length_string] = ACTIONS(2097),
    [sym_max_length_string] = ACTIONS(2097),
    [sym_set_length_string] = ACTIONS(2097),
    [sym_compare_string] = ACTIONS(2097),
    [sym_upper_string] = ACTIONS(2097),
    [sym_lower_string] = ACTIONS(2097),
    [sym_get_last] = ACTIONS(2097),
    [sym_date_to_day] = ACTIONS(2097),
    [sym_date_to_month] = ACTIONS(2097),
    [sym_date_to_year] = ACTIONS(2097),
    [sym_time_to_hour] = ACTIONS(2097),
    [sym_time_to_minute] = ACTIONS(2097),
    [sym_time_to_second] = ACTIONS(2097),
    [sym_day_of_week] = ACTIONS(2097),
    [sym_set_pulse_time] = ACTIONS(2097),
    [sym_set_timer] = ACTIONS(2097),
    [sym_file_open] = ACTIONS(2097),
    [sym_file_close] = ACTIONS(2097),
    [sym_file_read] = ACTIONS(2097),
    [sym_file_read_line] = ACTIONS(2097),
    [sym_file_write] = ACTIONS(2097),
    [sym_file_write_line] = ACTIONS(2097),
    [sym_file_copy] = ACTIONS(2097),
    [sym_file_delete] = ACTIONS(2097),
    [sym_file_rename] = ACTIONS(2097),
    [sym_file_dir] = ACTIONS(2097),
    [sym_file_getdir] = ACTIONS(2097),
    [sym_file_setdir] = ACTIONS(2097),
    [sym_file_create_dir] = ACTIONS(2097),
    [sym_file_remove_dir] = ACTIONS(2097),
    [sym_file_seek] = ACTIONS(2097),
    [sym_length_array] = ACTIONS(2097),
    [sym_max_length_array] = ACTIONS(2097),
    [sym_set_length_array] = ACTIONS(2097),
    [sym_ip_client_open] = ACTIONS(2097),
    [sym_ip_client_close] = ACTIONS(2097),
    [sym_ip_server_open] = ACTIONS(2097),
    [sym_ip_server_close] = ACTIONS(2097),
    [sym_ip_bound_client_open] = ACTIONS(2097),
    [sym_ip_mc_server_open] = ACTIONS(2097),
    [sym_ip_set_option] = ACTIONS(2097),
    [sym_ssh_client_open] = ACTIONS(2097),
    [sym_ssh_client_close] = ACTIONS(2097),
    [sym_tls_client_open] = ACTIONS(2097),
    [sym_tls_client_close] = ACTIONS(2097),
    [sym_do_push] = ACTIONS(2097),
    [sym_do_push_timed] = ACTIONS(2097),
    [sym_do_release] = ACTIONS(2097),
    [sym_device_id] = ACTIONS(2097),
    [sym_device_info] = ACTIONS(2097),
    [sym_device_id_string] = ACTIONS(2097),
    [sym_device_standby] = ACTIONS(2097),
    [sym_device_wake] = ACTIONS(2097),
    [sym_timeline_create] = ACTIONS(2097),
    [sym_timeline_kill] = ACTIONS(2097),
    [sym_timeline_reload] = ACTIONS(2097),
    [sym_timeline_pause] = ACTIONS(2097),
    [sym_timeline_restart] = ACTIONS(2097),
    [sym_timeline_set] = ACTIONS(2097),
    [sym_timeline_get] = ACTIONS(2097),
    [sym_timeline_active] = ACTIONS(2097),
    [sym_timeline_dynamic_id] = ACTIONS(2097),
    [sym_variable_to_string] = ACTIONS(2097),
    [sym_string_to_variable] = ACTIONS(2097),
    [sym_length_variable_to_string] = ACTIONS(2097),
    [sym_type_cast] = ACTIONS(2097),
    [sym_variable_to_xml] = ACTIONS(2097),
    [sym_xml_to_variable] = ACTIONS(2097),
    [sym_length_variable_to_xml] = ACTIONS(2097),
    [sym_redirect_string] = ACTIONS(2097),
    [sym_random_number] = ACTIONS(2097),
    [sym_get_unique_id] = ACTIONS(2097),
    [sym_get_system_number] = ACTIONS(2097),
    [sym_set_system_number] = ACTIONS(2097),
    [sym_get_serial_number] = ACTIONS(2097),
    [sym_get_master_build] = ACTIONS(2097),
    [sym_set_validation_code] = ACTIONS(2097),
    [sym_reboot] = ACTIONS(2097),
    [sym_load_duet_module] = ACTIONS(2097),
    [sym_unload_duet_module] = ACTIONS(2097),
    [sym_duet_mem_size_set] = ACTIONS(2097),
    [sym_duet_mem_size_get] = ACTIONS(2097),
    [sym_dynamic_application_device] = ACTIONS(2097),
    [sym_dynamic_polled_port] = ACTIONS(2097),
    [sym_static_port_binding] = ACTIONS(2097),
    [sym_static_ip_binding] = ACTIONS(2097),
    [sym_get_dns_list] = ACTIONS(2097),
    [sym_set_dns_list] = ACTIONS(2097),
    [sym_get_ip_address] = ACTIONS(2097),
    [sym_set_ip_address] = ACTIONS(2097),
    [sym_get_url_list] = ACTIONS(2097),
    [sym_add_url_entry] = ACTIONS(2097),
    [sym_delete_url_entry] = ACTIONS(2097),
    [sym_abs_value] = ACTIONS(2097),
    [sym_max_value] = ACTIONS(2097),
    [sym_min_value] = ACTIONS(2097),
    [sym_exp_value] = ACTIONS(2097),
    [sym_log_value] = ACTIONS(2097),
    [sym_log10_value] = ACTIONS(2097),
    [sym_power_value] = ACTIONS(2097),
    [sym_sqrt_value] = ACTIONS(2097),
    [sym_raw_be] = ACTIONS(2097),
    [sym_raw_le] = ACTIONS(2097),
    [sym_internal_queue_size_set] = ACTIONS(2097),
    [sym_internal_queue_size_get] = ACTIONS(2097),
    [sym_internal_threshold_set] = ACTIONS(2097),
    [sym_internal_threshold_get] = ACTIONS(2097),
    [sym_rebuilt_event] = ACTIONS(2097),
    [sym_do_custom_event] = ACTIONS(2097),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2097),
    [sym_clkmgr_set_clk_source] = ACTIONS(2097),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2097),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2097),
    [sym_clkmgr_get_timezone] = ACTIONS(2097),
    [sym_clkmgr_set_timezone] = ACTIONS(2097),
    [sym_clkmgr_get_resync_period] = ACTIONS(2097),
    [sym_clkmgr_set_resync_period] = ACTIONS(2097),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2097),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2097),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2097),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2097),
    [sym_clkmgr_get_timeservers] = ACTIONS(2097),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2097),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2097),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2097),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2097),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2097),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2097),
    [sym_set_log_level] = ACTIONS(2097),
    [sym_get_log_level] = ACTIONS(2097),
    [sym_amx_log] = ACTIONS(2097),
    [sym_smtp_server_config_set] = ACTIONS(2097),
    [sym_smtp_server_config_get] = ACTIONS(2097),
    [sym_smtp_send] = ACTIONS(2097),
    [sym_validate_netlinx_account] = ACTIONS(2097),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2097),
    [sym_authenticate_certificate] = ACTIONS(2097),
    [sym_audit_netlinx_session_event] = ACTIONS(2097),
    [sym_audit_netlinx_generic_event] = ACTIONS(2097),
    [sym_get_available_flash_disk_space] = ACTIONS(2097),
    [sym_get_max_flash_disk_space] = ACTIONS(2097),
    [sym_data_create_feed] = ACTIONS(2097),
    [sym_data_delete_feed] = ACTIONS(2097),
    [sym_data_publish_feed] = ACTIONS(2097),
    [sym_data_get_published_feed] = ACTIONS(2097),
    [sym_data_add_record] = ACTIONS(2097),
    [sym_data_get_event_record] = ACTIONS(2097),
    [sym__wc_data_create_feed] = ACTIONS(2097),
    [sym__wc_data_add_record] = ACTIONS(2097),
    [sym__wc_data_get_event_record] = ACTIONS(2097),
    [sym_format] = ACTIONS(2097),
    [sym_astro_clock] = ACTIONS(2097),
    [sym_set_outdoor_temperature] = ACTIONS(2097),
    [sym_set_virtual_level_count] = ACTIONS(2097),
    [sym_set_virtual_channel_count] = ACTIONS(2097),
    [sym_set_virtual_port_count] = ACTIONS(2097),
    [sym__wc_encode_int] = ACTIONS(2097),
    [sym__wc_decode_int] = ACTIONS(2097),
    [sym__wc_int] = ACTIONS(2097),
    [sym__wc_to_ch_int] = ACTIONS(2097),
    [sym__ch_to_wc_int] = ACTIONS(2097),
    [sym__wc_find_string_int] = ACTIONS(2097),
    [sym__wc_left_string_int] = ACTIONS(2097),
    [sym__wc_lower_string_int] = ACTIONS(2097),
    [sym__wc_mid_string_int] = ACTIONS(2097),
    [sym__wc_remove_string_int] = ACTIONS(2097),
    [sym__wc_right_string_int] = ACTIONS(2097),
    [sym__wc_upper_string_int] = ACTIONS(2097),
    [sym__wc_compare_string_int] = ACTIONS(2097),
    [sym__wc_get_buffer_char_int] = ACTIONS(2097),
    [sym__wc_get_buffer_string_int] = ACTIONS(2097),
    [sym__wc_concat_string_int] = ACTIONS(2097),
    [sym__wc_file_open_int] = ACTIONS(2097),
    [sym__wc_file_close_int] = ACTIONS(2097),
    [sym__wc_file_read_int] = ACTIONS(2097),
    [sym__wc_file_read_line_int] = ACTIONS(2097),
    [sym__wc_file_write_int] = ACTIONS(2097),
    [sym__wc_file_write_line_int] = ACTIONS(2097),
    [aux_sym_preproc_include_token1] = ACTIONS(2097),
    [aux_sym_preproc_define_token1] = ACTIONS(2099),
    [aux_sym_preproc_warn_token1] = ACTIONS(2099),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2099),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2099),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2099),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2099),
    [aux_sym_preproc_else_token1] = ACTIONS(2099),
    [anon_sym_LBRACK] = ACTIONS(2099),
    [anon_sym_LBRACE] = ACTIONS(2099),
    [anon_sym_LPAREN] = ACTIONS(2097),
    [anon_sym_BANG] = ACTIONS(2099),
    [anon_sym_TILDE] = ACTIONS(2099),
    [anon_sym_DASH] = ACTIONS(2097),
    [anon_sym_PLUS] = ACTIONS(2097),
    [anon_sym_DASH_DASH] = ACTIONS(2099),
    [anon_sym_PLUS_PLUS] = ACTIONS(2099),
    [anon_sym_DQUOTE] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2099),
    [sym_number_literal] = ACTIONS(2099),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(369)] = {
    [sym_identifier] = ACTIONS(2101),
    [sym_program_name_keyword] = ACTIONS(2101),
    [sym_module_name_keyword] = ACTIONS(2101),
    [sym_define_device_keyword] = ACTIONS(2101),
    [sym_define_combine_keyword] = ACTIONS(2101),
    [sym_define_constant_keyword] = ACTIONS(2101),
    [sym_define_type_keyword] = ACTIONS(2101),
    [sym_define_variable_keyword] = ACTIONS(2101),
    [sym_define_system_variable_keyword] = ACTIONS(2101),
    [sym_define_connect_level_keyword] = ACTIONS(2101),
    [sym_define_latching_keyword] = ACTIONS(2101),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2101),
    [sym_define_toggling_keyword] = ACTIONS(2101),
    [sym_define_call_keyword] = ACTIONS(2101),
    [sym_define_function_keyword] = ACTIONS(2101),
    [sym_define_library_function_keyword] = ACTIONS(2101),
    [sym_define_start_keyword] = ACTIONS(2101),
    [sym_define_event_keyword] = ACTIONS(2101),
    [sym_define_program_keyword] = ACTIONS(2101),
    [sym_define_module_keyword] = ACTIONS(2101),
    [sym_char_keyword] = ACTIONS(2101),
    [sym_widechar_keyword] = ACTIONS(2101),
    [sym_integer_keyword] = ACTIONS(2101),
    [sym_sinteger_keyword] = ACTIONS(2101),
    [sym_long_keyword] = ACTIONS(2101),
    [sym_slong_keyword] = ACTIONS(2101),
    [sym_float_keyword] = ACTIONS(2101),
    [sym_double_keyword] = ACTIONS(2101),
    [sym_variant_keyword] = ACTIONS(2101),
    [sym_variantarray_keyword] = ACTIONS(2101),
    [sym_local_var_keyword] = ACTIONS(2101),
    [sym_stack_var_keyword] = ACTIONS(2101),
    [sym_constant_keyword] = ACTIONS(2101),
    [sym_volatile_keyword] = ACTIONS(2101),
    [sym_non_volatile_keyword] = ACTIONS(2101),
    [sym_persistent_keyword] = ACTIONS(2101),
    [sym_dev_keyword] = ACTIONS(2101),
    [sym_devlev_keyword] = ACTIONS(2101),
    [sym_devchan_keyword] = ACTIONS(2101),
    [sym_if_keyword] = ACTIONS(2101),
    [sym_switch_keyword] = ACTIONS(2101),
    [sym_case_keyword] = ACTIONS(2101),
    [sym_default_keyword] = ACTIONS(2101),
    [sym_while_keyword] = ACTIONS(2101),
    [sym_for_keyword] = ACTIONS(2101),
    [sym_break_keyword] = ACTIONS(2101),
    [sym_continue_keyword] = ACTIONS(2101),
    [sym_return_keyword] = ACTIONS(2101),
    [sym_select_keyword] = ACTIONS(2101),
    [sym_active_keyword] = ACTIONS(2101),
    [sym_struct_keyword] = ACTIONS(2101),
    [sym_structure_keyword] = ACTIONS(2101),
    [sym_button_event_keyword] = ACTIONS(2101),
    [sym_channel_event_keyword] = ACTIONS(2101),
    [sym_level_event_keyword] = ACTIONS(2101),
    [sym_data_event_keyword] = ACTIONS(2101),
    [sym_timeline_event_keyword] = ACTIONS(2101),
    [sym_custom_event_keyword] = ACTIONS(2101),
    [sym_push_keyword] = ACTIONS(2101),
    [sym_release_keyword] = ACTIONS(2101),
    [aux_sym_on_keyword_token1] = ACTIONS(2101),
    [aux_sym_off_keyword_token1] = ACTIONS(2101),
    [sym_send_level_keyword] = ACTIONS(2101),
    [sym_send_string_keyword] = ACTIONS(2101),
    [sym_send_command_keyword] = ACTIONS(2101),
    [sym_clear_buffer_keyword] = ACTIONS(2101),
    [sym_create_buffer_keyword] = ACTIONS(2101),
    [sym_create_multi_buffer_keyword] = ACTIONS(2101),
    [sym_call_keyword] = ACTIONS(2101),
    [sym_system_call_keyword] = ACTIONS(2101),
    [sym_devchan_to_keyword] = ACTIONS(2101),
    [sym_devchan_min_to_keyword] = ACTIONS(2101),
    [sym_devchan_total_off_keyword] = ACTIONS(2101),
    [sym_devchan_pulse_keyword] = ACTIONS(2101),
    [sym_wait_keyword] = ACTIONS(2101),
    [sym_cancel_wait_keyword] = ACTIONS(2101),
    [sym_wait_until_keyword] = ACTIONS(2101),
    [sym_cancel_wait_until_keyword] = ACTIONS(2101),
    [sym_cancel_all_wait_keyword] = ACTIONS(2101),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2101),
    [sym_not] = ACTIONS(2101),
    [sym_bnot] = ACTIONS(2101),
    [sym___file__] = ACTIONS(2101),
    [sym___date__] = ACTIONS(2101),
    [sym___ldate__] = ACTIONS(2101),
    [sym___time__] = ACTIONS(2101),
    [sym___line__] = ACTIONS(2101),
    [sym___name__] = ACTIONS(2101),
    [sym___netlinx__] = ACTIONS(2101),
    [sym_netlinx_axi_version] = ACTIONS(2101),
    [sym_url_flg_tcp] = ACTIONS(2101),
    [sym_url_flg_accinfopresent] = ACTIONS(2101),
    [sym_url_flg_temp] = ACTIONS(2101),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2101),
    [sym_url_flg_stat_mask] = ACTIONS(2101),
    [sym_url_flg_stat_lookup] = ACTIONS(2101),
    [sym_url_flg_stat_connecting] = ACTIONS(2101),
    [sym_url_flg_stat_waiting] = ACTIONS(2101),
    [sym_url_flg_stat_connected] = ACTIONS(2101),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2101),
    [sym_first_virtual_device] = ACTIONS(2101),
    [sym_first_local_port] = ACTIONS(2101),
    [sym_dynamic_virtual_device] = ACTIONS(2101),
    [sym_timeline_once] = ACTIONS(2101),
    [sym_timeline_repeat] = ACTIONS(2101),
    [sym_timeline_absolute] = ACTIONS(2101),
    [sym_timeline_relative] = ACTIONS(2101),
    [sym_do_push_timed_infinite] = ACTIONS(2101),
    [sym_source_type_no_address] = ACTIONS(2101),
    [sym_source_type_neuron_id] = ACTIONS(2101),
    [sym_source_type_ip_address] = ACTIONS(2101),
    [sym_source_type_axlink] = ACTIONS(2101),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2101),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2101),
    [sym_source_type_ip_socket_address] = ACTIONS(2101),
    [sym_source_type_rs232] = ACTIONS(2101),
    [sym_source_type_internal] = ACTIONS(2101),
    [sym_source_type_mac_address] = ACTIONS(2101),
    [sym_source_type_ipv4_port] = ACTIONS(2101),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2101),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2101),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2101),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2101),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2101),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2101),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2101),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2101),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2101),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2101),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2101),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2101),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2101),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2101),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2101),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2101),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2101),
    [sym_true] = ACTIONS(2101),
    [sym_false] = ACTIONS(2101),
    [sym_file_read_only] = ACTIONS(2101),
    [sym_file_rw_new] = ACTIONS(2101),
    [sym_file_rw_append] = ACTIONS(2101),
    [sym_ip_tcp] = ACTIONS(2101),
    [sym_ip_udp] = ACTIONS(2101),
    [sym_ip_udp_2way] = ACTIONS(2101),
    [sym_xml_encode_types] = ACTIONS(2101),
    [sym_xml_encode_char_as_list] = ACTIONS(2101),
    [sym_xml_encode_le] = ACTIONS(2101),
    [sym_xml_decode_types] = ACTIONS(2101),
    [sym_xml_decode_no_preserve] = ACTIONS(2101),
    [sym_duet_dev_type_display_device] = ACTIONS(2101),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2101),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2101),
    [sym_duet_dev_type_io_device] = ACTIONS(2101),
    [sym_duet_dev_type_relay_device] = ACTIONS(2101),
    [sym_duet_dev_type_ups] = ACTIONS(2101),
    [sym_duet_dev_type_amplifier] = ACTIONS(2101),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2101),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2101),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2101),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2101),
    [sym_duet_dev_type_camera] = ACTIONS(2101),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2101),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2101),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2101),
    [sym_duet_dev_type_dss] = ACTIONS(2101),
    [sym_duet_dev_type_dvr] = ACTIONS(2101),
    [sym_duet_dev_type_disc_device] = ACTIONS(2101),
    [sym_duet_dev_type_document_camera] = ACTIONS(2101),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2101),
    [sym_duet_dev_type_hvac] = ACTIONS(2101),
    [sym_duet_dev_type_keypad] = ACTIONS(2101),
    [sym_duet_dev_type_light] = ACTIONS(2101),
    [sym_duet_dev_type_monitor] = ACTIONS(2101),
    [sym_duet_dev_type_motor] = ACTIONS(2101),
    [sym_duet_dev_type_multi_window] = ACTIONS(2101),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2101),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2101),
    [sym_duet_dev_type_receiver] = ACTIONS(2101),
    [sym_duet_dev_type_security_system] = ACTIONS(2101),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2101),
    [sym_duet_dev_type_settop_box] = ACTIONS(2101),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2101),
    [sym_duet_dev_type_switcher] = ACTIONS(2101),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2101),
    [sym_duet_dev_type_tv] = ACTIONS(2101),
    [sym_duet_dev_type_utility] = ACTIONS(2101),
    [sym_duet_dev_type_vcr] = ACTIONS(2101),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2101),
    [sym_duet_dev_type_video_processor] = ACTIONS(2101),
    [sym_duet_dev_type_video_projector] = ACTIONS(2101),
    [sym_duet_dev_type_video_wall] = ACTIONS(2101),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2101),
    [sym_duet_dev_type_weather] = ACTIONS(2101),
    [sym_duet_dev_not_polled] = ACTIONS(2101),
    [sym_duet_dev_polled] = ACTIONS(2101),
    [sym_clkmgr_mode_network] = ACTIONS(2101),
    [sym_clkmgr_mode_standalone] = ACTIONS(2101),
    [sym_normal_standby] = ACTIONS(2101),
    [sym_normal_wake] = ACTIONS(2101),
    [sym_ip_multicast_ttl_option] = ACTIONS(2101),
    [sym_ip_ttl_subnet] = ACTIONS(2101),
    [sym_ip_ttl_site] = ACTIONS(2101),
    [sym_ip_ttl_region] = ACTIONS(2101),
    [sym_ip_ttl_continent] = ACTIONS(2101),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2101),
    [sym_ip_nodelay_on] = ACTIONS(2101),
    [sym_ip_nodelay_off] = ACTIONS(2101),
    [sym_amx_error] = ACTIONS(2101),
    [sym_amx_warning] = ACTIONS(2101),
    [sym_amx_info] = ACTIONS(2101),
    [sym_amx_debug] = ACTIONS(2101),
    [sym_netlinx_login_success] = ACTIONS(2101),
    [sym_netlinx_login_fail] = ACTIONS(2101),
    [sym_netlinx_logout] = ACTIONS(2101),
    [sym_null_str] = ACTIONS(2101),
    [sym_smtp_address] = ACTIONS(2101),
    [sym_smtp_port_number] = ACTIONS(2101),
    [sym_smtp_username] = ACTIONS(2101),
    [sym_smtp_password] = ACTIONS(2101),
    [sym_smtp_require_tls] = ACTIONS(2101),
    [sym_smtp_from] = ACTIONS(2101),
    [sym_smtp_tls_true] = ACTIONS(2101),
    [sym_smtp_tls_false] = ACTIONS(2101),
    [sym_listview_on_row_select_event] = ACTIONS(2101),
    [sym_data_structure_unknown] = ACTIONS(2101),
    [sym_data_structure_datafeed] = ACTIONS(2101),
    [sym_data_structure_datarecordset] = ACTIONS(2101),
    [sym_data_structure_datarecord] = ACTIONS(2101),
    [sym_data_structure_datafield] = ACTIONS(2101),
    [sym_data_max_content_field_count] = ACTIONS(2101),
    [sym_data_max_metadata_field_count] = ACTIONS(2101),
    [sym_data_max_id_length] = ACTIONS(2101),
    [sym_data_max_name_length] = ACTIONS(2101),
    [sym_data_max_description_length] = ACTIONS(2101),
    [sym_data_max_type_length] = ACTIONS(2101),
    [sym_data_max_format_length] = ACTIONS(2101),
    [sym_data_max_source_length] = ACTIONS(2101),
    [sym_data_max_value_length] = ACTIONS(2101),
    [sym_data_max_label_length] = ACTIONS(2101),
    [sym_data_max_url_length] = ACTIONS(2101),
    [sym_max_last_login_info_length] = ACTIONS(2101),
    [sym_data_type_unknown] = ACTIONS(2101),
    [sym_data_type_string] = ACTIONS(2101),
    [sym_data_type_datetime] = ACTIONS(2101),
    [sym_data_type_date] = ACTIONS(2101),
    [sym_data_type_time] = ACTIONS(2101),
    [sym_data_type_image] = ACTIONS(2101),
    [sym_data_format_url] = ACTIONS(2101),
    [sym_data_format_phone] = ACTIONS(2101),
    [sym_data_format_email] = ACTIONS(2101),
    [sym_data_format_iso8601] = ACTIONS(2101),
    [sym_auditlog_priv] = ACTIONS(2101),
    [sym_configuration_priv] = ACTIONS(2101),
    [sym_deviceconfig_priv] = ACTIONS(2101),
    [sym_ftp_priv] = ACTIONS(2101),
    [sym_http_priv] = ACTIONS(2101),
    [sym_networkconfig_priv] = ACTIONS(2101),
    [sym_programport_priv] = ACTIONS(2101),
    [sym_remoteui_priv] = ACTIONS(2101),
    [sym_securitycontrol_priv] = ACTIONS(2101),
    [sym_softwaremanagement_priv] = ACTIONS(2101),
    [sym_terminal_priv] = ACTIONS(2101),
    [sym_tpadmin_priv] = ACTIONS(2101),
    [sym_user1_priv] = ACTIONS(2101),
    [sym_user2_priv] = ACTIONS(2101),
    [sym_user3_priv] = ACTIONS(2101),
    [sym_user4_priv] = ACTIONS(2101),
    [sym_usermanagement_priv] = ACTIONS(2101),
    [sym_valid_account] = ACTIONS(2101),
    [sym_err_name_invalid] = ACTIONS(2101),
    [sym_err_password_invalid] = ACTIONS(2101),
    [sym_err_invalid_account] = ACTIONS(2101),
    [sym_err_no_authorization] = ACTIONS(2101),
    [sym_err_invalid_parameter] = ACTIONS(2101),
    [sym_err_user_locked_out] = ACTIONS(2101),
    [sym_err_user_account_expired] = ACTIONS(2101),
    [sym_err_general_error] = ACTIONS(2101),
    [sym_err_authentication_error] = ACTIONS(2101),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2101),
    [sym_tls_validate_certificate] = ACTIONS(2101),
    [sym_tdata] = ACTIONS(2101),
    [sym_tchannel] = ACTIONS(2101),
    [sym_tlevel] = ACTIONS(2101),
    [sym_tbutton] = ACTIONS(2101),
    [sym_ttimeline] = ACTIONS(2101),
    [sym_tcustom] = ACTIONS(2101),
    [sym_url_struct] = ACTIONS(2101),
    [sym_dns_struct] = ACTIONS(2101),
    [sym_ip_address_struct] = ACTIONS(2101),
    [sym_dev_info_struct] = ACTIONS(2101),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2101),
    [sym_data_feed] = ACTIONS(2101),
    [sym_data_field] = ACTIONS(2101),
    [sym_data_record] = ACTIONS(2101),
    [sym_wc_data_feed] = ACTIONS(2101),
    [sym_wc_data_field] = ACTIONS(2101),
    [sym_wc_data_record] = ACTIONS(2101),
    [sym_last_login_info] = ACTIONS(2101),
    [sym____reserved___] = ACTIONS(2101),
    [sym_date] = ACTIONS(2101),
    [sym_day] = ACTIONS(2101),
    [sym_time] = ACTIONS(2101),
    [sym_ldate] = ACTIONS(2101),
    [sym_push_device] = ACTIONS(2101),
    [sym_push_channel] = ACTIONS(2101),
    [sym_push_devchan] = ACTIONS(2101),
    [sym_release_device] = ACTIONS(2101),
    [sym_release_channel] = ACTIONS(2101),
    [sym_release_devchan] = ACTIONS(2101),
    [sym_master_slot] = ACTIONS(2101),
    [sym_get_pulse_time] = ACTIONS(2101),
    [sym_get_timer] = ACTIONS(2101),
    [sym_master_sn] = ACTIONS(2101),
    [sym_system_number] = ACTIONS(2101),
    [sym_button] = ACTIONS(2101),
    [sym_dv_channel] = ACTIONS(2101),
    [sym_data] = ACTIONS(2101),
    [sym_level] = ACTIONS(2101),
    [sym_timeline] = ACTIONS(2101),
    [sym_channel] = ACTIONS(2101),
    [sym_custom] = ACTIONS(2101),
    [sym_atoi] = ACTIONS(2101),
    [sym_atol] = ACTIONS(2101),
    [sym_atof] = ACTIONS(2101),
    [sym_itoa] = ACTIONS(2101),
    [sym_itohex] = ACTIONS(2101),
    [sym_hextoi] = ACTIONS(2101),
    [sym_ftoa] = ACTIONS(2101),
    [sym_find_string] = ACTIONS(2101),
    [sym_left_string] = ACTIONS(2101),
    [sym_right_string] = ACTIONS(2101),
    [sym_mid_string] = ACTIONS(2101),
    [sym_remove_string] = ACTIONS(2101),
    [sym_get_buffer_string] = ACTIONS(2101),
    [sym_get_buffer_char] = ACTIONS(2101),
    [sym_get_multi_buffer_string] = ACTIONS(2101),
    [sym_length_string] = ACTIONS(2101),
    [sym_max_length_string] = ACTIONS(2101),
    [sym_set_length_string] = ACTIONS(2101),
    [sym_compare_string] = ACTIONS(2101),
    [sym_upper_string] = ACTIONS(2101),
    [sym_lower_string] = ACTIONS(2101),
    [sym_get_last] = ACTIONS(2101),
    [sym_date_to_day] = ACTIONS(2101),
    [sym_date_to_month] = ACTIONS(2101),
    [sym_date_to_year] = ACTIONS(2101),
    [sym_time_to_hour] = ACTIONS(2101),
    [sym_time_to_minute] = ACTIONS(2101),
    [sym_time_to_second] = ACTIONS(2101),
    [sym_day_of_week] = ACTIONS(2101),
    [sym_set_pulse_time] = ACTIONS(2101),
    [sym_set_timer] = ACTIONS(2101),
    [sym_file_open] = ACTIONS(2101),
    [sym_file_close] = ACTIONS(2101),
    [sym_file_read] = ACTIONS(2101),
    [sym_file_read_line] = ACTIONS(2101),
    [sym_file_write] = ACTIONS(2101),
    [sym_file_write_line] = ACTIONS(2101),
    [sym_file_copy] = ACTIONS(2101),
    [sym_file_delete] = ACTIONS(2101),
    [sym_file_rename] = ACTIONS(2101),
    [sym_file_dir] = ACTIONS(2101),
    [sym_file_getdir] = ACTIONS(2101),
    [sym_file_setdir] = ACTIONS(2101),
    [sym_file_create_dir] = ACTIONS(2101),
    [sym_file_remove_dir] = ACTIONS(2101),
    [sym_file_seek] = ACTIONS(2101),
    [sym_length_array] = ACTIONS(2101),
    [sym_max_length_array] = ACTIONS(2101),
    [sym_set_length_array] = ACTIONS(2101),
    [sym_ip_client_open] = ACTIONS(2101),
    [sym_ip_client_close] = ACTIONS(2101),
    [sym_ip_server_open] = ACTIONS(2101),
    [sym_ip_server_close] = ACTIONS(2101),
    [sym_ip_bound_client_open] = ACTIONS(2101),
    [sym_ip_mc_server_open] = ACTIONS(2101),
    [sym_ip_set_option] = ACTIONS(2101),
    [sym_ssh_client_open] = ACTIONS(2101),
    [sym_ssh_client_close] = ACTIONS(2101),
    [sym_tls_client_open] = ACTIONS(2101),
    [sym_tls_client_close] = ACTIONS(2101),
    [sym_do_push] = ACTIONS(2101),
    [sym_do_push_timed] = ACTIONS(2101),
    [sym_do_release] = ACTIONS(2101),
    [sym_device_id] = ACTIONS(2101),
    [sym_device_info] = ACTIONS(2101),
    [sym_device_id_string] = ACTIONS(2101),
    [sym_device_standby] = ACTIONS(2101),
    [sym_device_wake] = ACTIONS(2101),
    [sym_timeline_create] = ACTIONS(2101),
    [sym_timeline_kill] = ACTIONS(2101),
    [sym_timeline_reload] = ACTIONS(2101),
    [sym_timeline_pause] = ACTIONS(2101),
    [sym_timeline_restart] = ACTIONS(2101),
    [sym_timeline_set] = ACTIONS(2101),
    [sym_timeline_get] = ACTIONS(2101),
    [sym_timeline_active] = ACTIONS(2101),
    [sym_timeline_dynamic_id] = ACTIONS(2101),
    [sym_variable_to_string] = ACTIONS(2101),
    [sym_string_to_variable] = ACTIONS(2101),
    [sym_length_variable_to_string] = ACTIONS(2101),
    [sym_type_cast] = ACTIONS(2101),
    [sym_variable_to_xml] = ACTIONS(2101),
    [sym_xml_to_variable] = ACTIONS(2101),
    [sym_length_variable_to_xml] = ACTIONS(2101),
    [sym_redirect_string] = ACTIONS(2101),
    [sym_random_number] = ACTIONS(2101),
    [sym_get_unique_id] = ACTIONS(2101),
    [sym_get_system_number] = ACTIONS(2101),
    [sym_set_system_number] = ACTIONS(2101),
    [sym_get_serial_number] = ACTIONS(2101),
    [sym_get_master_build] = ACTIONS(2101),
    [sym_set_validation_code] = ACTIONS(2101),
    [sym_reboot] = ACTIONS(2101),
    [sym_load_duet_module] = ACTIONS(2101),
    [sym_unload_duet_module] = ACTIONS(2101),
    [sym_duet_mem_size_set] = ACTIONS(2101),
    [sym_duet_mem_size_get] = ACTIONS(2101),
    [sym_dynamic_application_device] = ACTIONS(2101),
    [sym_dynamic_polled_port] = ACTIONS(2101),
    [sym_static_port_binding] = ACTIONS(2101),
    [sym_static_ip_binding] = ACTIONS(2101),
    [sym_get_dns_list] = ACTIONS(2101),
    [sym_set_dns_list] = ACTIONS(2101),
    [sym_get_ip_address] = ACTIONS(2101),
    [sym_set_ip_address] = ACTIONS(2101),
    [sym_get_url_list] = ACTIONS(2101),
    [sym_add_url_entry] = ACTIONS(2101),
    [sym_delete_url_entry] = ACTIONS(2101),
    [sym_abs_value] = ACTIONS(2101),
    [sym_max_value] = ACTIONS(2101),
    [sym_min_value] = ACTIONS(2101),
    [sym_exp_value] = ACTIONS(2101),
    [sym_log_value] = ACTIONS(2101),
    [sym_log10_value] = ACTIONS(2101),
    [sym_power_value] = ACTIONS(2101),
    [sym_sqrt_value] = ACTIONS(2101),
    [sym_raw_be] = ACTIONS(2101),
    [sym_raw_le] = ACTIONS(2101),
    [sym_internal_queue_size_set] = ACTIONS(2101),
    [sym_internal_queue_size_get] = ACTIONS(2101),
    [sym_internal_threshold_set] = ACTIONS(2101),
    [sym_internal_threshold_get] = ACTIONS(2101),
    [sym_rebuilt_event] = ACTIONS(2101),
    [sym_do_custom_event] = ACTIONS(2101),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2101),
    [sym_clkmgr_set_clk_source] = ACTIONS(2101),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2101),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2101),
    [sym_clkmgr_get_timezone] = ACTIONS(2101),
    [sym_clkmgr_set_timezone] = ACTIONS(2101),
    [sym_clkmgr_get_resync_period] = ACTIONS(2101),
    [sym_clkmgr_set_resync_period] = ACTIONS(2101),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2101),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2101),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2101),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2101),
    [sym_clkmgr_get_timeservers] = ACTIONS(2101),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2101),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2101),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2101),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2101),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2101),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2101),
    [sym_set_log_level] = ACTIONS(2101),
    [sym_get_log_level] = ACTIONS(2101),
    [sym_amx_log] = ACTIONS(2101),
    [sym_smtp_server_config_set] = ACTIONS(2101),
    [sym_smtp_server_config_get] = ACTIONS(2101),
    [sym_smtp_send] = ACTIONS(2101),
    [sym_validate_netlinx_account] = ACTIONS(2101),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2101),
    [sym_authenticate_certificate] = ACTIONS(2101),
    [sym_audit_netlinx_session_event] = ACTIONS(2101),
    [sym_audit_netlinx_generic_event] = ACTIONS(2101),
    [sym_get_available_flash_disk_space] = ACTIONS(2101),
    [sym_get_max_flash_disk_space] = ACTIONS(2101),
    [sym_data_create_feed] = ACTIONS(2101),
    [sym_data_delete_feed] = ACTIONS(2101),
    [sym_data_publish_feed] = ACTIONS(2101),
    [sym_data_get_published_feed] = ACTIONS(2101),
    [sym_data_add_record] = ACTIONS(2101),
    [sym_data_get_event_record] = ACTIONS(2101),
    [sym__wc_data_create_feed] = ACTIONS(2101),
    [sym__wc_data_add_record] = ACTIONS(2101),
    [sym__wc_data_get_event_record] = ACTIONS(2101),
    [sym_format] = ACTIONS(2101),
    [sym_astro_clock] = ACTIONS(2101),
    [sym_set_outdoor_temperature] = ACTIONS(2101),
    [sym_set_virtual_level_count] = ACTIONS(2101),
    [sym_set_virtual_channel_count] = ACTIONS(2101),
    [sym_set_virtual_port_count] = ACTIONS(2101),
    [sym__wc_encode_int] = ACTIONS(2101),
    [sym__wc_decode_int] = ACTIONS(2101),
    [sym__wc_int] = ACTIONS(2101),
    [sym__wc_to_ch_int] = ACTIONS(2101),
    [sym__ch_to_wc_int] = ACTIONS(2101),
    [sym__wc_find_string_int] = ACTIONS(2101),
    [sym__wc_left_string_int] = ACTIONS(2101),
    [sym__wc_lower_string_int] = ACTIONS(2101),
    [sym__wc_mid_string_int] = ACTIONS(2101),
    [sym__wc_remove_string_int] = ACTIONS(2101),
    [sym__wc_right_string_int] = ACTIONS(2101),
    [sym__wc_upper_string_int] = ACTIONS(2101),
    [sym__wc_compare_string_int] = ACTIONS(2101),
    [sym__wc_get_buffer_char_int] = ACTIONS(2101),
    [sym__wc_get_buffer_string_int] = ACTIONS(2101),
    [sym__wc_concat_string_int] = ACTIONS(2101),
    [sym__wc_file_open_int] = ACTIONS(2101),
    [sym__wc_file_close_int] = ACTIONS(2101),
    [sym__wc_file_read_int] = ACTIONS(2101),
    [sym__wc_file_read_line_int] = ACTIONS(2101),
    [sym__wc_file_write_int] = ACTIONS(2101),
    [sym__wc_file_write_line_int] = ACTIONS(2101),
    [aux_sym_preproc_include_token1] = ACTIONS(2101),
    [aux_sym_preproc_define_token1] = ACTIONS(2103),
    [aux_sym_preproc_warn_token1] = ACTIONS(2103),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2103),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2103),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2103),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2103),
    [aux_sym_preproc_else_token1] = ACTIONS(2103),
    [anon_sym_LBRACK] = ACTIONS(2103),
    [anon_sym_LBRACE] = ACTIONS(2103),
    [anon_sym_LPAREN] = ACTIONS(2101),
    [anon_sym_BANG] = ACTIONS(2103),
    [anon_sym_TILDE] = ACTIONS(2103),
    [anon_sym_DASH] = ACTIONS(2101),
    [anon_sym_PLUS] = ACTIONS(2101),
    [anon_sym_DASH_DASH] = ACTIONS(2103),
    [anon_sym_PLUS_PLUS] = ACTIONS(2103),
    [anon_sym_DQUOTE] = ACTIONS(2103),
    [anon_sym_SQUOTE] = ACTIONS(2103),
    [sym_number_literal] = ACTIONS(2103),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(370)] = {
    [sym_identifier] = ACTIONS(2105),
    [sym_program_name_keyword] = ACTIONS(2105),
    [sym_module_name_keyword] = ACTIONS(2105),
    [sym_define_device_keyword] = ACTIONS(2105),
    [sym_define_combine_keyword] = ACTIONS(2105),
    [sym_define_constant_keyword] = ACTIONS(2105),
    [sym_define_type_keyword] = ACTIONS(2105),
    [sym_define_variable_keyword] = ACTIONS(2105),
    [sym_define_system_variable_keyword] = ACTIONS(2105),
    [sym_define_connect_level_keyword] = ACTIONS(2105),
    [sym_define_latching_keyword] = ACTIONS(2105),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2105),
    [sym_define_toggling_keyword] = ACTIONS(2105),
    [sym_define_call_keyword] = ACTIONS(2105),
    [sym_define_function_keyword] = ACTIONS(2105),
    [sym_define_library_function_keyword] = ACTIONS(2105),
    [sym_define_start_keyword] = ACTIONS(2105),
    [sym_define_event_keyword] = ACTIONS(2105),
    [sym_define_program_keyword] = ACTIONS(2105),
    [sym_define_module_keyword] = ACTIONS(2105),
    [sym_char_keyword] = ACTIONS(2105),
    [sym_widechar_keyword] = ACTIONS(2105),
    [sym_integer_keyword] = ACTIONS(2105),
    [sym_sinteger_keyword] = ACTIONS(2105),
    [sym_long_keyword] = ACTIONS(2105),
    [sym_slong_keyword] = ACTIONS(2105),
    [sym_float_keyword] = ACTIONS(2105),
    [sym_double_keyword] = ACTIONS(2105),
    [sym_variant_keyword] = ACTIONS(2105),
    [sym_variantarray_keyword] = ACTIONS(2105),
    [sym_local_var_keyword] = ACTIONS(2105),
    [sym_stack_var_keyword] = ACTIONS(2105),
    [sym_constant_keyword] = ACTIONS(2105),
    [sym_volatile_keyword] = ACTIONS(2105),
    [sym_non_volatile_keyword] = ACTIONS(2105),
    [sym_persistent_keyword] = ACTIONS(2105),
    [sym_dev_keyword] = ACTIONS(2105),
    [sym_devlev_keyword] = ACTIONS(2105),
    [sym_devchan_keyword] = ACTIONS(2105),
    [sym_if_keyword] = ACTIONS(2105),
    [sym_switch_keyword] = ACTIONS(2105),
    [sym_case_keyword] = ACTIONS(2105),
    [sym_default_keyword] = ACTIONS(2105),
    [sym_while_keyword] = ACTIONS(2105),
    [sym_for_keyword] = ACTIONS(2105),
    [sym_break_keyword] = ACTIONS(2105),
    [sym_continue_keyword] = ACTIONS(2105),
    [sym_return_keyword] = ACTIONS(2105),
    [sym_select_keyword] = ACTIONS(2105),
    [sym_active_keyword] = ACTIONS(2105),
    [sym_struct_keyword] = ACTIONS(2105),
    [sym_structure_keyword] = ACTIONS(2105),
    [sym_button_event_keyword] = ACTIONS(2105),
    [sym_channel_event_keyword] = ACTIONS(2105),
    [sym_level_event_keyword] = ACTIONS(2105),
    [sym_data_event_keyword] = ACTIONS(2105),
    [sym_timeline_event_keyword] = ACTIONS(2105),
    [sym_custom_event_keyword] = ACTIONS(2105),
    [sym_push_keyword] = ACTIONS(2105),
    [sym_release_keyword] = ACTIONS(2105),
    [aux_sym_on_keyword_token1] = ACTIONS(2105),
    [aux_sym_off_keyword_token1] = ACTIONS(2105),
    [sym_send_level_keyword] = ACTIONS(2105),
    [sym_send_string_keyword] = ACTIONS(2105),
    [sym_send_command_keyword] = ACTIONS(2105),
    [sym_clear_buffer_keyword] = ACTIONS(2105),
    [sym_create_buffer_keyword] = ACTIONS(2105),
    [sym_create_multi_buffer_keyword] = ACTIONS(2105),
    [sym_call_keyword] = ACTIONS(2105),
    [sym_system_call_keyword] = ACTIONS(2105),
    [sym_devchan_to_keyword] = ACTIONS(2105),
    [sym_devchan_min_to_keyword] = ACTIONS(2105),
    [sym_devchan_total_off_keyword] = ACTIONS(2105),
    [sym_devchan_pulse_keyword] = ACTIONS(2105),
    [sym_wait_keyword] = ACTIONS(2105),
    [sym_cancel_wait_keyword] = ACTIONS(2105),
    [sym_wait_until_keyword] = ACTIONS(2105),
    [sym_cancel_wait_until_keyword] = ACTIONS(2105),
    [sym_cancel_all_wait_keyword] = ACTIONS(2105),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2105),
    [sym_not] = ACTIONS(2105),
    [sym_bnot] = ACTIONS(2105),
    [sym___file__] = ACTIONS(2105),
    [sym___date__] = ACTIONS(2105),
    [sym___ldate__] = ACTIONS(2105),
    [sym___time__] = ACTIONS(2105),
    [sym___line__] = ACTIONS(2105),
    [sym___name__] = ACTIONS(2105),
    [sym___netlinx__] = ACTIONS(2105),
    [sym_netlinx_axi_version] = ACTIONS(2105),
    [sym_url_flg_tcp] = ACTIONS(2105),
    [sym_url_flg_accinfopresent] = ACTIONS(2105),
    [sym_url_flg_temp] = ACTIONS(2105),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2105),
    [sym_url_flg_stat_mask] = ACTIONS(2105),
    [sym_url_flg_stat_lookup] = ACTIONS(2105),
    [sym_url_flg_stat_connecting] = ACTIONS(2105),
    [sym_url_flg_stat_waiting] = ACTIONS(2105),
    [sym_url_flg_stat_connected] = ACTIONS(2105),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2105),
    [sym_first_virtual_device] = ACTIONS(2105),
    [sym_first_local_port] = ACTIONS(2105),
    [sym_dynamic_virtual_device] = ACTIONS(2105),
    [sym_timeline_once] = ACTIONS(2105),
    [sym_timeline_repeat] = ACTIONS(2105),
    [sym_timeline_absolute] = ACTIONS(2105),
    [sym_timeline_relative] = ACTIONS(2105),
    [sym_do_push_timed_infinite] = ACTIONS(2105),
    [sym_source_type_no_address] = ACTIONS(2105),
    [sym_source_type_neuron_id] = ACTIONS(2105),
    [sym_source_type_ip_address] = ACTIONS(2105),
    [sym_source_type_axlink] = ACTIONS(2105),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2105),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2105),
    [sym_source_type_ip_socket_address] = ACTIONS(2105),
    [sym_source_type_rs232] = ACTIONS(2105),
    [sym_source_type_internal] = ACTIONS(2105),
    [sym_source_type_mac_address] = ACTIONS(2105),
    [sym_source_type_ipv4_port] = ACTIONS(2105),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2105),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2105),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2105),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2105),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2105),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2105),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2105),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2105),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2105),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2105),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2105),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2105),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2105),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2105),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2105),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2105),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2105),
    [sym_true] = ACTIONS(2105),
    [sym_false] = ACTIONS(2105),
    [sym_file_read_only] = ACTIONS(2105),
    [sym_file_rw_new] = ACTIONS(2105),
    [sym_file_rw_append] = ACTIONS(2105),
    [sym_ip_tcp] = ACTIONS(2105),
    [sym_ip_udp] = ACTIONS(2105),
    [sym_ip_udp_2way] = ACTIONS(2105),
    [sym_xml_encode_types] = ACTIONS(2105),
    [sym_xml_encode_char_as_list] = ACTIONS(2105),
    [sym_xml_encode_le] = ACTIONS(2105),
    [sym_xml_decode_types] = ACTIONS(2105),
    [sym_xml_decode_no_preserve] = ACTIONS(2105),
    [sym_duet_dev_type_display_device] = ACTIONS(2105),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2105),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2105),
    [sym_duet_dev_type_io_device] = ACTIONS(2105),
    [sym_duet_dev_type_relay_device] = ACTIONS(2105),
    [sym_duet_dev_type_ups] = ACTIONS(2105),
    [sym_duet_dev_type_amplifier] = ACTIONS(2105),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2105),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2105),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2105),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2105),
    [sym_duet_dev_type_camera] = ACTIONS(2105),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2105),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2105),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2105),
    [sym_duet_dev_type_dss] = ACTIONS(2105),
    [sym_duet_dev_type_dvr] = ACTIONS(2105),
    [sym_duet_dev_type_disc_device] = ACTIONS(2105),
    [sym_duet_dev_type_document_camera] = ACTIONS(2105),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2105),
    [sym_duet_dev_type_hvac] = ACTIONS(2105),
    [sym_duet_dev_type_keypad] = ACTIONS(2105),
    [sym_duet_dev_type_light] = ACTIONS(2105),
    [sym_duet_dev_type_monitor] = ACTIONS(2105),
    [sym_duet_dev_type_motor] = ACTIONS(2105),
    [sym_duet_dev_type_multi_window] = ACTIONS(2105),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2105),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2105),
    [sym_duet_dev_type_receiver] = ACTIONS(2105),
    [sym_duet_dev_type_security_system] = ACTIONS(2105),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2105),
    [sym_duet_dev_type_settop_box] = ACTIONS(2105),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2105),
    [sym_duet_dev_type_switcher] = ACTIONS(2105),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2105),
    [sym_duet_dev_type_tv] = ACTIONS(2105),
    [sym_duet_dev_type_utility] = ACTIONS(2105),
    [sym_duet_dev_type_vcr] = ACTIONS(2105),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2105),
    [sym_duet_dev_type_video_processor] = ACTIONS(2105),
    [sym_duet_dev_type_video_projector] = ACTIONS(2105),
    [sym_duet_dev_type_video_wall] = ACTIONS(2105),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2105),
    [sym_duet_dev_type_weather] = ACTIONS(2105),
    [sym_duet_dev_not_polled] = ACTIONS(2105),
    [sym_duet_dev_polled] = ACTIONS(2105),
    [sym_clkmgr_mode_network] = ACTIONS(2105),
    [sym_clkmgr_mode_standalone] = ACTIONS(2105),
    [sym_normal_standby] = ACTIONS(2105),
    [sym_normal_wake] = ACTIONS(2105),
    [sym_ip_multicast_ttl_option] = ACTIONS(2105),
    [sym_ip_ttl_subnet] = ACTIONS(2105),
    [sym_ip_ttl_site] = ACTIONS(2105),
    [sym_ip_ttl_region] = ACTIONS(2105),
    [sym_ip_ttl_continent] = ACTIONS(2105),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2105),
    [sym_ip_nodelay_on] = ACTIONS(2105),
    [sym_ip_nodelay_off] = ACTIONS(2105),
    [sym_amx_error] = ACTIONS(2105),
    [sym_amx_warning] = ACTIONS(2105),
    [sym_amx_info] = ACTIONS(2105),
    [sym_amx_debug] = ACTIONS(2105),
    [sym_netlinx_login_success] = ACTIONS(2105),
    [sym_netlinx_login_fail] = ACTIONS(2105),
    [sym_netlinx_logout] = ACTIONS(2105),
    [sym_null_str] = ACTIONS(2105),
    [sym_smtp_address] = ACTIONS(2105),
    [sym_smtp_port_number] = ACTIONS(2105),
    [sym_smtp_username] = ACTIONS(2105),
    [sym_smtp_password] = ACTIONS(2105),
    [sym_smtp_require_tls] = ACTIONS(2105),
    [sym_smtp_from] = ACTIONS(2105),
    [sym_smtp_tls_true] = ACTIONS(2105),
    [sym_smtp_tls_false] = ACTIONS(2105),
    [sym_listview_on_row_select_event] = ACTIONS(2105),
    [sym_data_structure_unknown] = ACTIONS(2105),
    [sym_data_structure_datafeed] = ACTIONS(2105),
    [sym_data_structure_datarecordset] = ACTIONS(2105),
    [sym_data_structure_datarecord] = ACTIONS(2105),
    [sym_data_structure_datafield] = ACTIONS(2105),
    [sym_data_max_content_field_count] = ACTIONS(2105),
    [sym_data_max_metadata_field_count] = ACTIONS(2105),
    [sym_data_max_id_length] = ACTIONS(2105),
    [sym_data_max_name_length] = ACTIONS(2105),
    [sym_data_max_description_length] = ACTIONS(2105),
    [sym_data_max_type_length] = ACTIONS(2105),
    [sym_data_max_format_length] = ACTIONS(2105),
    [sym_data_max_source_length] = ACTIONS(2105),
    [sym_data_max_value_length] = ACTIONS(2105),
    [sym_data_max_label_length] = ACTIONS(2105),
    [sym_data_max_url_length] = ACTIONS(2105),
    [sym_max_last_login_info_length] = ACTIONS(2105),
    [sym_data_type_unknown] = ACTIONS(2105),
    [sym_data_type_string] = ACTIONS(2105),
    [sym_data_type_datetime] = ACTIONS(2105),
    [sym_data_type_date] = ACTIONS(2105),
    [sym_data_type_time] = ACTIONS(2105),
    [sym_data_type_image] = ACTIONS(2105),
    [sym_data_format_url] = ACTIONS(2105),
    [sym_data_format_phone] = ACTIONS(2105),
    [sym_data_format_email] = ACTIONS(2105),
    [sym_data_format_iso8601] = ACTIONS(2105),
    [sym_auditlog_priv] = ACTIONS(2105),
    [sym_configuration_priv] = ACTIONS(2105),
    [sym_deviceconfig_priv] = ACTIONS(2105),
    [sym_ftp_priv] = ACTIONS(2105),
    [sym_http_priv] = ACTIONS(2105),
    [sym_networkconfig_priv] = ACTIONS(2105),
    [sym_programport_priv] = ACTIONS(2105),
    [sym_remoteui_priv] = ACTIONS(2105),
    [sym_securitycontrol_priv] = ACTIONS(2105),
    [sym_softwaremanagement_priv] = ACTIONS(2105),
    [sym_terminal_priv] = ACTIONS(2105),
    [sym_tpadmin_priv] = ACTIONS(2105),
    [sym_user1_priv] = ACTIONS(2105),
    [sym_user2_priv] = ACTIONS(2105),
    [sym_user3_priv] = ACTIONS(2105),
    [sym_user4_priv] = ACTIONS(2105),
    [sym_usermanagement_priv] = ACTIONS(2105),
    [sym_valid_account] = ACTIONS(2105),
    [sym_err_name_invalid] = ACTIONS(2105),
    [sym_err_password_invalid] = ACTIONS(2105),
    [sym_err_invalid_account] = ACTIONS(2105),
    [sym_err_no_authorization] = ACTIONS(2105),
    [sym_err_invalid_parameter] = ACTIONS(2105),
    [sym_err_user_locked_out] = ACTIONS(2105),
    [sym_err_user_account_expired] = ACTIONS(2105),
    [sym_err_general_error] = ACTIONS(2105),
    [sym_err_authentication_error] = ACTIONS(2105),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2105),
    [sym_tls_validate_certificate] = ACTIONS(2105),
    [sym_tdata] = ACTIONS(2105),
    [sym_tchannel] = ACTIONS(2105),
    [sym_tlevel] = ACTIONS(2105),
    [sym_tbutton] = ACTIONS(2105),
    [sym_ttimeline] = ACTIONS(2105),
    [sym_tcustom] = ACTIONS(2105),
    [sym_url_struct] = ACTIONS(2105),
    [sym_dns_struct] = ACTIONS(2105),
    [sym_ip_address_struct] = ACTIONS(2105),
    [sym_dev_info_struct] = ACTIONS(2105),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2105),
    [sym_data_feed] = ACTIONS(2105),
    [sym_data_field] = ACTIONS(2105),
    [sym_data_record] = ACTIONS(2105),
    [sym_wc_data_feed] = ACTIONS(2105),
    [sym_wc_data_field] = ACTIONS(2105),
    [sym_wc_data_record] = ACTIONS(2105),
    [sym_last_login_info] = ACTIONS(2105),
    [sym____reserved___] = ACTIONS(2105),
    [sym_date] = ACTIONS(2105),
    [sym_day] = ACTIONS(2105),
    [sym_time] = ACTIONS(2105),
    [sym_ldate] = ACTIONS(2105),
    [sym_push_device] = ACTIONS(2105),
    [sym_push_channel] = ACTIONS(2105),
    [sym_push_devchan] = ACTIONS(2105),
    [sym_release_device] = ACTIONS(2105),
    [sym_release_channel] = ACTIONS(2105),
    [sym_release_devchan] = ACTIONS(2105),
    [sym_master_slot] = ACTIONS(2105),
    [sym_get_pulse_time] = ACTIONS(2105),
    [sym_get_timer] = ACTIONS(2105),
    [sym_master_sn] = ACTIONS(2105),
    [sym_system_number] = ACTIONS(2105),
    [sym_button] = ACTIONS(2105),
    [sym_dv_channel] = ACTIONS(2105),
    [sym_data] = ACTIONS(2105),
    [sym_level] = ACTIONS(2105),
    [sym_timeline] = ACTIONS(2105),
    [sym_channel] = ACTIONS(2105),
    [sym_custom] = ACTIONS(2105),
    [sym_atoi] = ACTIONS(2105),
    [sym_atol] = ACTIONS(2105),
    [sym_atof] = ACTIONS(2105),
    [sym_itoa] = ACTIONS(2105),
    [sym_itohex] = ACTIONS(2105),
    [sym_hextoi] = ACTIONS(2105),
    [sym_ftoa] = ACTIONS(2105),
    [sym_find_string] = ACTIONS(2105),
    [sym_left_string] = ACTIONS(2105),
    [sym_right_string] = ACTIONS(2105),
    [sym_mid_string] = ACTIONS(2105),
    [sym_remove_string] = ACTIONS(2105),
    [sym_get_buffer_string] = ACTIONS(2105),
    [sym_get_buffer_char] = ACTIONS(2105),
    [sym_get_multi_buffer_string] = ACTIONS(2105),
    [sym_length_string] = ACTIONS(2105),
    [sym_max_length_string] = ACTIONS(2105),
    [sym_set_length_string] = ACTIONS(2105),
    [sym_compare_string] = ACTIONS(2105),
    [sym_upper_string] = ACTIONS(2105),
    [sym_lower_string] = ACTIONS(2105),
    [sym_get_last] = ACTIONS(2105),
    [sym_date_to_day] = ACTIONS(2105),
    [sym_date_to_month] = ACTIONS(2105),
    [sym_date_to_year] = ACTIONS(2105),
    [sym_time_to_hour] = ACTIONS(2105),
    [sym_time_to_minute] = ACTIONS(2105),
    [sym_time_to_second] = ACTIONS(2105),
    [sym_day_of_week] = ACTIONS(2105),
    [sym_set_pulse_time] = ACTIONS(2105),
    [sym_set_timer] = ACTIONS(2105),
    [sym_file_open] = ACTIONS(2105),
    [sym_file_close] = ACTIONS(2105),
    [sym_file_read] = ACTIONS(2105),
    [sym_file_read_line] = ACTIONS(2105),
    [sym_file_write] = ACTIONS(2105),
    [sym_file_write_line] = ACTIONS(2105),
    [sym_file_copy] = ACTIONS(2105),
    [sym_file_delete] = ACTIONS(2105),
    [sym_file_rename] = ACTIONS(2105),
    [sym_file_dir] = ACTIONS(2105),
    [sym_file_getdir] = ACTIONS(2105),
    [sym_file_setdir] = ACTIONS(2105),
    [sym_file_create_dir] = ACTIONS(2105),
    [sym_file_remove_dir] = ACTIONS(2105),
    [sym_file_seek] = ACTIONS(2105),
    [sym_length_array] = ACTIONS(2105),
    [sym_max_length_array] = ACTIONS(2105),
    [sym_set_length_array] = ACTIONS(2105),
    [sym_ip_client_open] = ACTIONS(2105),
    [sym_ip_client_close] = ACTIONS(2105),
    [sym_ip_server_open] = ACTIONS(2105),
    [sym_ip_server_close] = ACTIONS(2105),
    [sym_ip_bound_client_open] = ACTIONS(2105),
    [sym_ip_mc_server_open] = ACTIONS(2105),
    [sym_ip_set_option] = ACTIONS(2105),
    [sym_ssh_client_open] = ACTIONS(2105),
    [sym_ssh_client_close] = ACTIONS(2105),
    [sym_tls_client_open] = ACTIONS(2105),
    [sym_tls_client_close] = ACTIONS(2105),
    [sym_do_push] = ACTIONS(2105),
    [sym_do_push_timed] = ACTIONS(2105),
    [sym_do_release] = ACTIONS(2105),
    [sym_device_id] = ACTIONS(2105),
    [sym_device_info] = ACTIONS(2105),
    [sym_device_id_string] = ACTIONS(2105),
    [sym_device_standby] = ACTIONS(2105),
    [sym_device_wake] = ACTIONS(2105),
    [sym_timeline_create] = ACTIONS(2105),
    [sym_timeline_kill] = ACTIONS(2105),
    [sym_timeline_reload] = ACTIONS(2105),
    [sym_timeline_pause] = ACTIONS(2105),
    [sym_timeline_restart] = ACTIONS(2105),
    [sym_timeline_set] = ACTIONS(2105),
    [sym_timeline_get] = ACTIONS(2105),
    [sym_timeline_active] = ACTIONS(2105),
    [sym_timeline_dynamic_id] = ACTIONS(2105),
    [sym_variable_to_string] = ACTIONS(2105),
    [sym_string_to_variable] = ACTIONS(2105),
    [sym_length_variable_to_string] = ACTIONS(2105),
    [sym_type_cast] = ACTIONS(2105),
    [sym_variable_to_xml] = ACTIONS(2105),
    [sym_xml_to_variable] = ACTIONS(2105),
    [sym_length_variable_to_xml] = ACTIONS(2105),
    [sym_redirect_string] = ACTIONS(2105),
    [sym_random_number] = ACTIONS(2105),
    [sym_get_unique_id] = ACTIONS(2105),
    [sym_get_system_number] = ACTIONS(2105),
    [sym_set_system_number] = ACTIONS(2105),
    [sym_get_serial_number] = ACTIONS(2105),
    [sym_get_master_build] = ACTIONS(2105),
    [sym_set_validation_code] = ACTIONS(2105),
    [sym_reboot] = ACTIONS(2105),
    [sym_load_duet_module] = ACTIONS(2105),
    [sym_unload_duet_module] = ACTIONS(2105),
    [sym_duet_mem_size_set] = ACTIONS(2105),
    [sym_duet_mem_size_get] = ACTIONS(2105),
    [sym_dynamic_application_device] = ACTIONS(2105),
    [sym_dynamic_polled_port] = ACTIONS(2105),
    [sym_static_port_binding] = ACTIONS(2105),
    [sym_static_ip_binding] = ACTIONS(2105),
    [sym_get_dns_list] = ACTIONS(2105),
    [sym_set_dns_list] = ACTIONS(2105),
    [sym_get_ip_address] = ACTIONS(2105),
    [sym_set_ip_address] = ACTIONS(2105),
    [sym_get_url_list] = ACTIONS(2105),
    [sym_add_url_entry] = ACTIONS(2105),
    [sym_delete_url_entry] = ACTIONS(2105),
    [sym_abs_value] = ACTIONS(2105),
    [sym_max_value] = ACTIONS(2105),
    [sym_min_value] = ACTIONS(2105),
    [sym_exp_value] = ACTIONS(2105),
    [sym_log_value] = ACTIONS(2105),
    [sym_log10_value] = ACTIONS(2105),
    [sym_power_value] = ACTIONS(2105),
    [sym_sqrt_value] = ACTIONS(2105),
    [sym_raw_be] = ACTIONS(2105),
    [sym_raw_le] = ACTIONS(2105),
    [sym_internal_queue_size_set] = ACTIONS(2105),
    [sym_internal_queue_size_get] = ACTIONS(2105),
    [sym_internal_threshold_set] = ACTIONS(2105),
    [sym_internal_threshold_get] = ACTIONS(2105),
    [sym_rebuilt_event] = ACTIONS(2105),
    [sym_do_custom_event] = ACTIONS(2105),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2105),
    [sym_clkmgr_set_clk_source] = ACTIONS(2105),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2105),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2105),
    [sym_clkmgr_get_timezone] = ACTIONS(2105),
    [sym_clkmgr_set_timezone] = ACTIONS(2105),
    [sym_clkmgr_get_resync_period] = ACTIONS(2105),
    [sym_clkmgr_set_resync_period] = ACTIONS(2105),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2105),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2105),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2105),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2105),
    [sym_clkmgr_get_timeservers] = ACTIONS(2105),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2105),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2105),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2105),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2105),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2105),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2105),
    [sym_set_log_level] = ACTIONS(2105),
    [sym_get_log_level] = ACTIONS(2105),
    [sym_amx_log] = ACTIONS(2105),
    [sym_smtp_server_config_set] = ACTIONS(2105),
    [sym_smtp_server_config_get] = ACTIONS(2105),
    [sym_smtp_send] = ACTIONS(2105),
    [sym_validate_netlinx_account] = ACTIONS(2105),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2105),
    [sym_authenticate_certificate] = ACTIONS(2105),
    [sym_audit_netlinx_session_event] = ACTIONS(2105),
    [sym_audit_netlinx_generic_event] = ACTIONS(2105),
    [sym_get_available_flash_disk_space] = ACTIONS(2105),
    [sym_get_max_flash_disk_space] = ACTIONS(2105),
    [sym_data_create_feed] = ACTIONS(2105),
    [sym_data_delete_feed] = ACTIONS(2105),
    [sym_data_publish_feed] = ACTIONS(2105),
    [sym_data_get_published_feed] = ACTIONS(2105),
    [sym_data_add_record] = ACTIONS(2105),
    [sym_data_get_event_record] = ACTIONS(2105),
    [sym__wc_data_create_feed] = ACTIONS(2105),
    [sym__wc_data_add_record] = ACTIONS(2105),
    [sym__wc_data_get_event_record] = ACTIONS(2105),
    [sym_format] = ACTIONS(2105),
    [sym_astro_clock] = ACTIONS(2105),
    [sym_set_outdoor_temperature] = ACTIONS(2105),
    [sym_set_virtual_level_count] = ACTIONS(2105),
    [sym_set_virtual_channel_count] = ACTIONS(2105),
    [sym_set_virtual_port_count] = ACTIONS(2105),
    [sym__wc_encode_int] = ACTIONS(2105),
    [sym__wc_decode_int] = ACTIONS(2105),
    [sym__wc_int] = ACTIONS(2105),
    [sym__wc_to_ch_int] = ACTIONS(2105),
    [sym__ch_to_wc_int] = ACTIONS(2105),
    [sym__wc_find_string_int] = ACTIONS(2105),
    [sym__wc_left_string_int] = ACTIONS(2105),
    [sym__wc_lower_string_int] = ACTIONS(2105),
    [sym__wc_mid_string_int] = ACTIONS(2105),
    [sym__wc_remove_string_int] = ACTIONS(2105),
    [sym__wc_right_string_int] = ACTIONS(2105),
    [sym__wc_upper_string_int] = ACTIONS(2105),
    [sym__wc_compare_string_int] = ACTIONS(2105),
    [sym__wc_get_buffer_char_int] = ACTIONS(2105),
    [sym__wc_get_buffer_string_int] = ACTIONS(2105),
    [sym__wc_concat_string_int] = ACTIONS(2105),
    [sym__wc_file_open_int] = ACTIONS(2105),
    [sym__wc_file_close_int] = ACTIONS(2105),
    [sym__wc_file_read_int] = ACTIONS(2105),
    [sym__wc_file_read_line_int] = ACTIONS(2105),
    [sym__wc_file_write_int] = ACTIONS(2105),
    [sym__wc_file_write_line_int] = ACTIONS(2105),
    [aux_sym_preproc_include_token1] = ACTIONS(2105),
    [aux_sym_preproc_define_token1] = ACTIONS(2107),
    [aux_sym_preproc_warn_token1] = ACTIONS(2107),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2107),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2107),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2107),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2107),
    [aux_sym_preproc_else_token1] = ACTIONS(2107),
    [anon_sym_LBRACK] = ACTIONS(2107),
    [anon_sym_LBRACE] = ACTIONS(2107),
    [anon_sym_LPAREN] = ACTIONS(2105),
    [anon_sym_BANG] = ACTIONS(2107),
    [anon_sym_TILDE] = ACTIONS(2107),
    [anon_sym_DASH] = ACTIONS(2105),
    [anon_sym_PLUS] = ACTIONS(2105),
    [anon_sym_DASH_DASH] = ACTIONS(2107),
    [anon_sym_PLUS_PLUS] = ACTIONS(2107),
    [anon_sym_DQUOTE] = ACTIONS(2107),
    [anon_sym_SQUOTE] = ACTIONS(2107),
    [sym_number_literal] = ACTIONS(2107),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(371)] = {
    [sym_identifier] = ACTIONS(1597),
    [sym_program_name_keyword] = ACTIONS(1597),
    [sym_module_name_keyword] = ACTIONS(1597),
    [sym_define_device_keyword] = ACTIONS(1597),
    [sym_define_combine_keyword] = ACTIONS(1597),
    [sym_define_constant_keyword] = ACTIONS(1597),
    [sym_define_type_keyword] = ACTIONS(1597),
    [sym_define_variable_keyword] = ACTIONS(1597),
    [sym_define_system_variable_keyword] = ACTIONS(1597),
    [sym_define_connect_level_keyword] = ACTIONS(1597),
    [sym_define_latching_keyword] = ACTIONS(1597),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1597),
    [sym_define_toggling_keyword] = ACTIONS(1597),
    [sym_define_call_keyword] = ACTIONS(1597),
    [sym_define_function_keyword] = ACTIONS(1597),
    [sym_define_library_function_keyword] = ACTIONS(1597),
    [sym_define_start_keyword] = ACTIONS(1597),
    [sym_define_event_keyword] = ACTIONS(1597),
    [sym_define_program_keyword] = ACTIONS(1597),
    [sym_define_module_keyword] = ACTIONS(1597),
    [sym_char_keyword] = ACTIONS(1597),
    [sym_widechar_keyword] = ACTIONS(1597),
    [sym_integer_keyword] = ACTIONS(1597),
    [sym_sinteger_keyword] = ACTIONS(1597),
    [sym_long_keyword] = ACTIONS(1597),
    [sym_slong_keyword] = ACTIONS(1597),
    [sym_float_keyword] = ACTIONS(1597),
    [sym_double_keyword] = ACTIONS(1597),
    [sym_variant_keyword] = ACTIONS(1597),
    [sym_variantarray_keyword] = ACTIONS(1597),
    [sym_local_var_keyword] = ACTIONS(1597),
    [sym_stack_var_keyword] = ACTIONS(1597),
    [sym_constant_keyword] = ACTIONS(1597),
    [sym_volatile_keyword] = ACTIONS(1597),
    [sym_non_volatile_keyword] = ACTIONS(1597),
    [sym_persistent_keyword] = ACTIONS(1597),
    [sym_dev_keyword] = ACTIONS(1597),
    [sym_devlev_keyword] = ACTIONS(1597),
    [sym_devchan_keyword] = ACTIONS(1597),
    [sym_if_keyword] = ACTIONS(1597),
    [sym_switch_keyword] = ACTIONS(1597),
    [sym_case_keyword] = ACTIONS(1597),
    [sym_default_keyword] = ACTIONS(1597),
    [sym_while_keyword] = ACTIONS(1597),
    [sym_for_keyword] = ACTIONS(1597),
    [sym_break_keyword] = ACTIONS(1597),
    [sym_continue_keyword] = ACTIONS(1597),
    [sym_return_keyword] = ACTIONS(1597),
    [sym_select_keyword] = ACTIONS(1597),
    [sym_active_keyword] = ACTIONS(1597),
    [sym_struct_keyword] = ACTIONS(1597),
    [sym_structure_keyword] = ACTIONS(1597),
    [sym_button_event_keyword] = ACTIONS(1597),
    [sym_channel_event_keyword] = ACTIONS(1597),
    [sym_level_event_keyword] = ACTIONS(1597),
    [sym_data_event_keyword] = ACTIONS(1597),
    [sym_timeline_event_keyword] = ACTIONS(1597),
    [sym_custom_event_keyword] = ACTIONS(1597),
    [sym_push_keyword] = ACTIONS(1597),
    [sym_release_keyword] = ACTIONS(1597),
    [aux_sym_on_keyword_token1] = ACTIONS(1597),
    [aux_sym_off_keyword_token1] = ACTIONS(1597),
    [sym_send_level_keyword] = ACTIONS(1597),
    [sym_send_string_keyword] = ACTIONS(1597),
    [sym_send_command_keyword] = ACTIONS(1597),
    [sym_clear_buffer_keyword] = ACTIONS(1597),
    [sym_create_buffer_keyword] = ACTIONS(1597),
    [sym_create_multi_buffer_keyword] = ACTIONS(1597),
    [sym_call_keyword] = ACTIONS(1597),
    [sym_system_call_keyword] = ACTIONS(1597),
    [sym_devchan_to_keyword] = ACTIONS(1597),
    [sym_devchan_min_to_keyword] = ACTIONS(1597),
    [sym_devchan_total_off_keyword] = ACTIONS(1597),
    [sym_devchan_pulse_keyword] = ACTIONS(1597),
    [sym_wait_keyword] = ACTIONS(1597),
    [sym_cancel_wait_keyword] = ACTIONS(1597),
    [sym_wait_until_keyword] = ACTIONS(1597),
    [sym_cancel_wait_until_keyword] = ACTIONS(1597),
    [sym_cancel_all_wait_keyword] = ACTIONS(1597),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1597),
    [sym_not] = ACTIONS(1597),
    [sym_bnot] = ACTIONS(1597),
    [sym___file__] = ACTIONS(1597),
    [sym___date__] = ACTIONS(1597),
    [sym___ldate__] = ACTIONS(1597),
    [sym___time__] = ACTIONS(1597),
    [sym___line__] = ACTIONS(1597),
    [sym___name__] = ACTIONS(1597),
    [sym___netlinx__] = ACTIONS(1597),
    [sym_netlinx_axi_version] = ACTIONS(1597),
    [sym_url_flg_tcp] = ACTIONS(1597),
    [sym_url_flg_accinfopresent] = ACTIONS(1597),
    [sym_url_flg_temp] = ACTIONS(1597),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1597),
    [sym_url_flg_stat_mask] = ACTIONS(1597),
    [sym_url_flg_stat_lookup] = ACTIONS(1597),
    [sym_url_flg_stat_connecting] = ACTIONS(1597),
    [sym_url_flg_stat_waiting] = ACTIONS(1597),
    [sym_url_flg_stat_connected] = ACTIONS(1597),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1597),
    [sym_first_virtual_device] = ACTIONS(1597),
    [sym_first_local_port] = ACTIONS(1597),
    [sym_dynamic_virtual_device] = ACTIONS(1597),
    [sym_timeline_once] = ACTIONS(1597),
    [sym_timeline_repeat] = ACTIONS(1597),
    [sym_timeline_absolute] = ACTIONS(1597),
    [sym_timeline_relative] = ACTIONS(1597),
    [sym_do_push_timed_infinite] = ACTIONS(1597),
    [sym_source_type_no_address] = ACTIONS(1597),
    [sym_source_type_neuron_id] = ACTIONS(1597),
    [sym_source_type_ip_address] = ACTIONS(1597),
    [sym_source_type_axlink] = ACTIONS(1597),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1597),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1597),
    [sym_source_type_ip_socket_address] = ACTIONS(1597),
    [sym_source_type_rs232] = ACTIONS(1597),
    [sym_source_type_internal] = ACTIONS(1597),
    [sym_source_type_mac_address] = ACTIONS(1597),
    [sym_source_type_ipv4_port] = ACTIONS(1597),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1597),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1597),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1597),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1597),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1597),
    [sym_true] = ACTIONS(1597),
    [sym_false] = ACTIONS(1597),
    [sym_file_read_only] = ACTIONS(1597),
    [sym_file_rw_new] = ACTIONS(1597),
    [sym_file_rw_append] = ACTIONS(1597),
    [sym_ip_tcp] = ACTIONS(1597),
    [sym_ip_udp] = ACTIONS(1597),
    [sym_ip_udp_2way] = ACTIONS(1597),
    [sym_xml_encode_types] = ACTIONS(1597),
    [sym_xml_encode_char_as_list] = ACTIONS(1597),
    [sym_xml_encode_le] = ACTIONS(1597),
    [sym_xml_decode_types] = ACTIONS(1597),
    [sym_xml_decode_no_preserve] = ACTIONS(1597),
    [sym_duet_dev_type_display_device] = ACTIONS(1597),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1597),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1597),
    [sym_duet_dev_type_io_device] = ACTIONS(1597),
    [sym_duet_dev_type_relay_device] = ACTIONS(1597),
    [sym_duet_dev_type_ups] = ACTIONS(1597),
    [sym_duet_dev_type_amplifier] = ACTIONS(1597),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1597),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1597),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1597),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1597),
    [sym_duet_dev_type_camera] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1597),
    [sym_duet_dev_type_dss] = ACTIONS(1597),
    [sym_duet_dev_type_dvr] = ACTIONS(1597),
    [sym_duet_dev_type_disc_device] = ACTIONS(1597),
    [sym_duet_dev_type_document_camera] = ACTIONS(1597),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1597),
    [sym_duet_dev_type_hvac] = ACTIONS(1597),
    [sym_duet_dev_type_keypad] = ACTIONS(1597),
    [sym_duet_dev_type_light] = ACTIONS(1597),
    [sym_duet_dev_type_monitor] = ACTIONS(1597),
    [sym_duet_dev_type_motor] = ACTIONS(1597),
    [sym_duet_dev_type_multi_window] = ACTIONS(1597),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1597),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1597),
    [sym_duet_dev_type_receiver] = ACTIONS(1597),
    [sym_duet_dev_type_security_system] = ACTIONS(1597),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1597),
    [sym_duet_dev_type_settop_box] = ACTIONS(1597),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1597),
    [sym_duet_dev_type_switcher] = ACTIONS(1597),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1597),
    [sym_duet_dev_type_tv] = ACTIONS(1597),
    [sym_duet_dev_type_utility] = ACTIONS(1597),
    [sym_duet_dev_type_vcr] = ACTIONS(1597),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1597),
    [sym_duet_dev_type_video_processor] = ACTIONS(1597),
    [sym_duet_dev_type_video_projector] = ACTIONS(1597),
    [sym_duet_dev_type_video_wall] = ACTIONS(1597),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1597),
    [sym_duet_dev_type_weather] = ACTIONS(1597),
    [sym_duet_dev_not_polled] = ACTIONS(1597),
    [sym_duet_dev_polled] = ACTIONS(1597),
    [sym_clkmgr_mode_network] = ACTIONS(1597),
    [sym_clkmgr_mode_standalone] = ACTIONS(1597),
    [sym_normal_standby] = ACTIONS(1597),
    [sym_normal_wake] = ACTIONS(1597),
    [sym_ip_multicast_ttl_option] = ACTIONS(1597),
    [sym_ip_ttl_subnet] = ACTIONS(1597),
    [sym_ip_ttl_site] = ACTIONS(1597),
    [sym_ip_ttl_region] = ACTIONS(1597),
    [sym_ip_ttl_continent] = ACTIONS(1597),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1597),
    [sym_ip_nodelay_on] = ACTIONS(1597),
    [sym_ip_nodelay_off] = ACTIONS(1597),
    [sym_amx_error] = ACTIONS(1597),
    [sym_amx_warning] = ACTIONS(1597),
    [sym_amx_info] = ACTIONS(1597),
    [sym_amx_debug] = ACTIONS(1597),
    [sym_netlinx_login_success] = ACTIONS(1597),
    [sym_netlinx_login_fail] = ACTIONS(1597),
    [sym_netlinx_logout] = ACTIONS(1597),
    [sym_null_str] = ACTIONS(1597),
    [sym_smtp_address] = ACTIONS(1597),
    [sym_smtp_port_number] = ACTIONS(1597),
    [sym_smtp_username] = ACTIONS(1597),
    [sym_smtp_password] = ACTIONS(1597),
    [sym_smtp_require_tls] = ACTIONS(1597),
    [sym_smtp_from] = ACTIONS(1597),
    [sym_smtp_tls_true] = ACTIONS(1597),
    [sym_smtp_tls_false] = ACTIONS(1597),
    [sym_listview_on_row_select_event] = ACTIONS(1597),
    [sym_data_structure_unknown] = ACTIONS(1597),
    [sym_data_structure_datafeed] = ACTIONS(1597),
    [sym_data_structure_datarecordset] = ACTIONS(1597),
    [sym_data_structure_datarecord] = ACTIONS(1597),
    [sym_data_structure_datafield] = ACTIONS(1597),
    [sym_data_max_content_field_count] = ACTIONS(1597),
    [sym_data_max_metadata_field_count] = ACTIONS(1597),
    [sym_data_max_id_length] = ACTIONS(1597),
    [sym_data_max_name_length] = ACTIONS(1597),
    [sym_data_max_description_length] = ACTIONS(1597),
    [sym_data_max_type_length] = ACTIONS(1597),
    [sym_data_max_format_length] = ACTIONS(1597),
    [sym_data_max_source_length] = ACTIONS(1597),
    [sym_data_max_value_length] = ACTIONS(1597),
    [sym_data_max_label_length] = ACTIONS(1597),
    [sym_data_max_url_length] = ACTIONS(1597),
    [sym_max_last_login_info_length] = ACTIONS(1597),
    [sym_data_type_unknown] = ACTIONS(1597),
    [sym_data_type_string] = ACTIONS(1597),
    [sym_data_type_datetime] = ACTIONS(1597),
    [sym_data_type_date] = ACTIONS(1597),
    [sym_data_type_time] = ACTIONS(1597),
    [sym_data_type_image] = ACTIONS(1597),
    [sym_data_format_url] = ACTIONS(1597),
    [sym_data_format_phone] = ACTIONS(1597),
    [sym_data_format_email] = ACTIONS(1597),
    [sym_data_format_iso8601] = ACTIONS(1597),
    [sym_auditlog_priv] = ACTIONS(1597),
    [sym_configuration_priv] = ACTIONS(1597),
    [sym_deviceconfig_priv] = ACTIONS(1597),
    [sym_ftp_priv] = ACTIONS(1597),
    [sym_http_priv] = ACTIONS(1597),
    [sym_networkconfig_priv] = ACTIONS(1597),
    [sym_programport_priv] = ACTIONS(1597),
    [sym_remoteui_priv] = ACTIONS(1597),
    [sym_securitycontrol_priv] = ACTIONS(1597),
    [sym_softwaremanagement_priv] = ACTIONS(1597),
    [sym_terminal_priv] = ACTIONS(1597),
    [sym_tpadmin_priv] = ACTIONS(1597),
    [sym_user1_priv] = ACTIONS(1597),
    [sym_user2_priv] = ACTIONS(1597),
    [sym_user3_priv] = ACTIONS(1597),
    [sym_user4_priv] = ACTIONS(1597),
    [sym_usermanagement_priv] = ACTIONS(1597),
    [sym_valid_account] = ACTIONS(1597),
    [sym_err_name_invalid] = ACTIONS(1597),
    [sym_err_password_invalid] = ACTIONS(1597),
    [sym_err_invalid_account] = ACTIONS(1597),
    [sym_err_no_authorization] = ACTIONS(1597),
    [sym_err_invalid_parameter] = ACTIONS(1597),
    [sym_err_user_locked_out] = ACTIONS(1597),
    [sym_err_user_account_expired] = ACTIONS(1597),
    [sym_err_general_error] = ACTIONS(1597),
    [sym_err_authentication_error] = ACTIONS(1597),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1597),
    [sym_tls_validate_certificate] = ACTIONS(1597),
    [sym_tdata] = ACTIONS(1597),
    [sym_tchannel] = ACTIONS(1597),
    [sym_tlevel] = ACTIONS(1597),
    [sym_tbutton] = ACTIONS(1597),
    [sym_ttimeline] = ACTIONS(1597),
    [sym_tcustom] = ACTIONS(1597),
    [sym_url_struct] = ACTIONS(1597),
    [sym_dns_struct] = ACTIONS(1597),
    [sym_ip_address_struct] = ACTIONS(1597),
    [sym_dev_info_struct] = ACTIONS(1597),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1597),
    [sym_data_feed] = ACTIONS(1597),
    [sym_data_field] = ACTIONS(1597),
    [sym_data_record] = ACTIONS(1597),
    [sym_wc_data_feed] = ACTIONS(1597),
    [sym_wc_data_field] = ACTIONS(1597),
    [sym_wc_data_record] = ACTIONS(1597),
    [sym_last_login_info] = ACTIONS(1597),
    [sym____reserved___] = ACTIONS(1597),
    [sym_date] = ACTIONS(1597),
    [sym_day] = ACTIONS(1597),
    [sym_time] = ACTIONS(1597),
    [sym_ldate] = ACTIONS(1597),
    [sym_push_device] = ACTIONS(1597),
    [sym_push_channel] = ACTIONS(1597),
    [sym_push_devchan] = ACTIONS(1597),
    [sym_release_device] = ACTIONS(1597),
    [sym_release_channel] = ACTIONS(1597),
    [sym_release_devchan] = ACTIONS(1597),
    [sym_master_slot] = ACTIONS(1597),
    [sym_get_pulse_time] = ACTIONS(1597),
    [sym_get_timer] = ACTIONS(1597),
    [sym_master_sn] = ACTIONS(1597),
    [sym_system_number] = ACTIONS(1597),
    [sym_button] = ACTIONS(1597),
    [sym_dv_channel] = ACTIONS(1597),
    [sym_data] = ACTIONS(1597),
    [sym_level] = ACTIONS(1597),
    [sym_timeline] = ACTIONS(1597),
    [sym_channel] = ACTIONS(1597),
    [sym_custom] = ACTIONS(1597),
    [sym_atoi] = ACTIONS(1597),
    [sym_atol] = ACTIONS(1597),
    [sym_atof] = ACTIONS(1597),
    [sym_itoa] = ACTIONS(1597),
    [sym_itohex] = ACTIONS(1597),
    [sym_hextoi] = ACTIONS(1597),
    [sym_ftoa] = ACTIONS(1597),
    [sym_find_string] = ACTIONS(1597),
    [sym_left_string] = ACTIONS(1597),
    [sym_right_string] = ACTIONS(1597),
    [sym_mid_string] = ACTIONS(1597),
    [sym_remove_string] = ACTIONS(1597),
    [sym_get_buffer_string] = ACTIONS(1597),
    [sym_get_buffer_char] = ACTIONS(1597),
    [sym_get_multi_buffer_string] = ACTIONS(1597),
    [sym_length_string] = ACTIONS(1597),
    [sym_max_length_string] = ACTIONS(1597),
    [sym_set_length_string] = ACTIONS(1597),
    [sym_compare_string] = ACTIONS(1597),
    [sym_upper_string] = ACTIONS(1597),
    [sym_lower_string] = ACTIONS(1597),
    [sym_get_last] = ACTIONS(1597),
    [sym_date_to_day] = ACTIONS(1597),
    [sym_date_to_month] = ACTIONS(1597),
    [sym_date_to_year] = ACTIONS(1597),
    [sym_time_to_hour] = ACTIONS(1597),
    [sym_time_to_minute] = ACTIONS(1597),
    [sym_time_to_second] = ACTIONS(1597),
    [sym_day_of_week] = ACTIONS(1597),
    [sym_set_pulse_time] = ACTIONS(1597),
    [sym_set_timer] = ACTIONS(1597),
    [sym_file_open] = ACTIONS(1597),
    [sym_file_close] = ACTIONS(1597),
    [sym_file_read] = ACTIONS(1597),
    [sym_file_read_line] = ACTIONS(1597),
    [sym_file_write] = ACTIONS(1597),
    [sym_file_write_line] = ACTIONS(1597),
    [sym_file_copy] = ACTIONS(1597),
    [sym_file_delete] = ACTIONS(1597),
    [sym_file_rename] = ACTIONS(1597),
    [sym_file_dir] = ACTIONS(1597),
    [sym_file_getdir] = ACTIONS(1597),
    [sym_file_setdir] = ACTIONS(1597),
    [sym_file_create_dir] = ACTIONS(1597),
    [sym_file_remove_dir] = ACTIONS(1597),
    [sym_file_seek] = ACTIONS(1597),
    [sym_length_array] = ACTIONS(1597),
    [sym_max_length_array] = ACTIONS(1597),
    [sym_set_length_array] = ACTIONS(1597),
    [sym_ip_client_open] = ACTIONS(1597),
    [sym_ip_client_close] = ACTIONS(1597),
    [sym_ip_server_open] = ACTIONS(1597),
    [sym_ip_server_close] = ACTIONS(1597),
    [sym_ip_bound_client_open] = ACTIONS(1597),
    [sym_ip_mc_server_open] = ACTIONS(1597),
    [sym_ip_set_option] = ACTIONS(1597),
    [sym_ssh_client_open] = ACTIONS(1597),
    [sym_ssh_client_close] = ACTIONS(1597),
    [sym_tls_client_open] = ACTIONS(1597),
    [sym_tls_client_close] = ACTIONS(1597),
    [sym_do_push] = ACTIONS(1597),
    [sym_do_push_timed] = ACTIONS(1597),
    [sym_do_release] = ACTIONS(1597),
    [sym_device_id] = ACTIONS(1597),
    [sym_device_info] = ACTIONS(1597),
    [sym_device_id_string] = ACTIONS(1597),
    [sym_device_standby] = ACTIONS(1597),
    [sym_device_wake] = ACTIONS(1597),
    [sym_timeline_create] = ACTIONS(1597),
    [sym_timeline_kill] = ACTIONS(1597),
    [sym_timeline_reload] = ACTIONS(1597),
    [sym_timeline_pause] = ACTIONS(1597),
    [sym_timeline_restart] = ACTIONS(1597),
    [sym_timeline_set] = ACTIONS(1597),
    [sym_timeline_get] = ACTIONS(1597),
    [sym_timeline_active] = ACTIONS(1597),
    [sym_timeline_dynamic_id] = ACTIONS(1597),
    [sym_variable_to_string] = ACTIONS(1597),
    [sym_string_to_variable] = ACTIONS(1597),
    [sym_length_variable_to_string] = ACTIONS(1597),
    [sym_type_cast] = ACTIONS(1597),
    [sym_variable_to_xml] = ACTIONS(1597),
    [sym_xml_to_variable] = ACTIONS(1597),
    [sym_length_variable_to_xml] = ACTIONS(1597),
    [sym_redirect_string] = ACTIONS(1597),
    [sym_random_number] = ACTIONS(1597),
    [sym_get_unique_id] = ACTIONS(1597),
    [sym_get_system_number] = ACTIONS(1597),
    [sym_set_system_number] = ACTIONS(1597),
    [sym_get_serial_number] = ACTIONS(1597),
    [sym_get_master_build] = ACTIONS(1597),
    [sym_set_validation_code] = ACTIONS(1597),
    [sym_reboot] = ACTIONS(1597),
    [sym_load_duet_module] = ACTIONS(1597),
    [sym_unload_duet_module] = ACTIONS(1597),
    [sym_duet_mem_size_set] = ACTIONS(1597),
    [sym_duet_mem_size_get] = ACTIONS(1597),
    [sym_dynamic_application_device] = ACTIONS(1597),
    [sym_dynamic_polled_port] = ACTIONS(1597),
    [sym_static_port_binding] = ACTIONS(1597),
    [sym_static_ip_binding] = ACTIONS(1597),
    [sym_get_dns_list] = ACTIONS(1597),
    [sym_set_dns_list] = ACTIONS(1597),
    [sym_get_ip_address] = ACTIONS(1597),
    [sym_set_ip_address] = ACTIONS(1597),
    [sym_get_url_list] = ACTIONS(1597),
    [sym_add_url_entry] = ACTIONS(1597),
    [sym_delete_url_entry] = ACTIONS(1597),
    [sym_abs_value] = ACTIONS(1597),
    [sym_max_value] = ACTIONS(1597),
    [sym_min_value] = ACTIONS(1597),
    [sym_exp_value] = ACTIONS(1597),
    [sym_log_value] = ACTIONS(1597),
    [sym_log10_value] = ACTIONS(1597),
    [sym_power_value] = ACTIONS(1597),
    [sym_sqrt_value] = ACTIONS(1597),
    [sym_raw_be] = ACTIONS(1597),
    [sym_raw_le] = ACTIONS(1597),
    [sym_internal_queue_size_set] = ACTIONS(1597),
    [sym_internal_queue_size_get] = ACTIONS(1597),
    [sym_internal_threshold_set] = ACTIONS(1597),
    [sym_internal_threshold_get] = ACTIONS(1597),
    [sym_rebuilt_event] = ACTIONS(1597),
    [sym_do_custom_event] = ACTIONS(1597),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1597),
    [sym_clkmgr_set_clk_source] = ACTIONS(1597),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1597),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1597),
    [sym_clkmgr_get_timezone] = ACTIONS(1597),
    [sym_clkmgr_set_timezone] = ACTIONS(1597),
    [sym_clkmgr_get_resync_period] = ACTIONS(1597),
    [sym_clkmgr_set_resync_period] = ACTIONS(1597),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1597),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1597),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1597),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1597),
    [sym_clkmgr_get_timeservers] = ACTIONS(1597),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1597),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1597),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1597),
    [sym_set_log_level] = ACTIONS(1597),
    [sym_get_log_level] = ACTIONS(1597),
    [sym_amx_log] = ACTIONS(1597),
    [sym_smtp_server_config_set] = ACTIONS(1597),
    [sym_smtp_server_config_get] = ACTIONS(1597),
    [sym_smtp_send] = ACTIONS(1597),
    [sym_validate_netlinx_account] = ACTIONS(1597),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1597),
    [sym_authenticate_certificate] = ACTIONS(1597),
    [sym_audit_netlinx_session_event] = ACTIONS(1597),
    [sym_audit_netlinx_generic_event] = ACTIONS(1597),
    [sym_get_available_flash_disk_space] = ACTIONS(1597),
    [sym_get_max_flash_disk_space] = ACTIONS(1597),
    [sym_data_create_feed] = ACTIONS(1597),
    [sym_data_delete_feed] = ACTIONS(1597),
    [sym_data_publish_feed] = ACTIONS(1597),
    [sym_data_get_published_feed] = ACTIONS(1597),
    [sym_data_add_record] = ACTIONS(1597),
    [sym_data_get_event_record] = ACTIONS(1597),
    [sym__wc_data_create_feed] = ACTIONS(1597),
    [sym__wc_data_add_record] = ACTIONS(1597),
    [sym__wc_data_get_event_record] = ACTIONS(1597),
    [sym_format] = ACTIONS(1597),
    [sym_astro_clock] = ACTIONS(1597),
    [sym_set_outdoor_temperature] = ACTIONS(1597),
    [sym_set_virtual_level_count] = ACTIONS(1597),
    [sym_set_virtual_channel_count] = ACTIONS(1597),
    [sym_set_virtual_port_count] = ACTIONS(1597),
    [sym__wc_encode_int] = ACTIONS(1597),
    [sym__wc_decode_int] = ACTIONS(1597),
    [sym__wc_int] = ACTIONS(1597),
    [sym__wc_to_ch_int] = ACTIONS(1597),
    [sym__ch_to_wc_int] = ACTIONS(1597),
    [sym__wc_find_string_int] = ACTIONS(1597),
    [sym__wc_left_string_int] = ACTIONS(1597),
    [sym__wc_lower_string_int] = ACTIONS(1597),
    [sym__wc_mid_string_int] = ACTIONS(1597),
    [sym__wc_remove_string_int] = ACTIONS(1597),
    [sym__wc_right_string_int] = ACTIONS(1597),
    [sym__wc_upper_string_int] = ACTIONS(1597),
    [sym__wc_compare_string_int] = ACTIONS(1597),
    [sym__wc_get_buffer_char_int] = ACTIONS(1597),
    [sym__wc_get_buffer_string_int] = ACTIONS(1597),
    [sym__wc_concat_string_int] = ACTIONS(1597),
    [sym__wc_file_open_int] = ACTIONS(1597),
    [sym__wc_file_close_int] = ACTIONS(1597),
    [sym__wc_file_read_int] = ACTIONS(1597),
    [sym__wc_file_read_line_int] = ACTIONS(1597),
    [sym__wc_file_write_int] = ACTIONS(1597),
    [sym__wc_file_write_line_int] = ACTIONS(1597),
    [aux_sym_preproc_include_token1] = ACTIONS(1597),
    [aux_sym_preproc_define_token1] = ACTIONS(1599),
    [aux_sym_preproc_warn_token1] = ACTIONS(1599),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1599),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1599),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1599),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1599),
    [aux_sym_preproc_else_token1] = ACTIONS(1599),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1599),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_BANG] = ACTIONS(1599),
    [anon_sym_TILDE] = ACTIONS(1599),
    [anon_sym_DASH] = ACTIONS(1597),
    [anon_sym_PLUS] = ACTIONS(1597),
    [anon_sym_DASH_DASH] = ACTIONS(1599),
    [anon_sym_PLUS_PLUS] = ACTIONS(1599),
    [anon_sym_DQUOTE] = ACTIONS(1599),
    [anon_sym_SQUOTE] = ACTIONS(1599),
    [sym_number_literal] = ACTIONS(1599),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(372)] = {
    [sym_identifier] = ACTIONS(1601),
    [sym_program_name_keyword] = ACTIONS(1601),
    [sym_module_name_keyword] = ACTIONS(1601),
    [sym_define_device_keyword] = ACTIONS(1601),
    [sym_define_combine_keyword] = ACTIONS(1601),
    [sym_define_constant_keyword] = ACTIONS(1601),
    [sym_define_type_keyword] = ACTIONS(1601),
    [sym_define_variable_keyword] = ACTIONS(1601),
    [sym_define_system_variable_keyword] = ACTIONS(1601),
    [sym_define_connect_level_keyword] = ACTIONS(1601),
    [sym_define_latching_keyword] = ACTIONS(1601),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1601),
    [sym_define_toggling_keyword] = ACTIONS(1601),
    [sym_define_call_keyword] = ACTIONS(1601),
    [sym_define_function_keyword] = ACTIONS(1601),
    [sym_define_library_function_keyword] = ACTIONS(1601),
    [sym_define_start_keyword] = ACTIONS(1601),
    [sym_define_event_keyword] = ACTIONS(1601),
    [sym_define_program_keyword] = ACTIONS(1601),
    [sym_define_module_keyword] = ACTIONS(1601),
    [sym_char_keyword] = ACTIONS(1601),
    [sym_widechar_keyword] = ACTIONS(1601),
    [sym_integer_keyword] = ACTIONS(1601),
    [sym_sinteger_keyword] = ACTIONS(1601),
    [sym_long_keyword] = ACTIONS(1601),
    [sym_slong_keyword] = ACTIONS(1601),
    [sym_float_keyword] = ACTIONS(1601),
    [sym_double_keyword] = ACTIONS(1601),
    [sym_variant_keyword] = ACTIONS(1601),
    [sym_variantarray_keyword] = ACTIONS(1601),
    [sym_local_var_keyword] = ACTIONS(1601),
    [sym_stack_var_keyword] = ACTIONS(1601),
    [sym_constant_keyword] = ACTIONS(1601),
    [sym_volatile_keyword] = ACTIONS(1601),
    [sym_non_volatile_keyword] = ACTIONS(1601),
    [sym_persistent_keyword] = ACTIONS(1601),
    [sym_dev_keyword] = ACTIONS(1601),
    [sym_devlev_keyword] = ACTIONS(1601),
    [sym_devchan_keyword] = ACTIONS(1601),
    [sym_if_keyword] = ACTIONS(1601),
    [sym_switch_keyword] = ACTIONS(1601),
    [sym_case_keyword] = ACTIONS(1601),
    [sym_default_keyword] = ACTIONS(1601),
    [sym_while_keyword] = ACTIONS(1601),
    [sym_for_keyword] = ACTIONS(1601),
    [sym_break_keyword] = ACTIONS(1601),
    [sym_continue_keyword] = ACTIONS(1601),
    [sym_return_keyword] = ACTIONS(1601),
    [sym_select_keyword] = ACTIONS(1601),
    [sym_active_keyword] = ACTIONS(1601),
    [sym_struct_keyword] = ACTIONS(1601),
    [sym_structure_keyword] = ACTIONS(1601),
    [sym_button_event_keyword] = ACTIONS(1601),
    [sym_channel_event_keyword] = ACTIONS(1601),
    [sym_level_event_keyword] = ACTIONS(1601),
    [sym_data_event_keyword] = ACTIONS(1601),
    [sym_timeline_event_keyword] = ACTIONS(1601),
    [sym_custom_event_keyword] = ACTIONS(1601),
    [sym_push_keyword] = ACTIONS(1601),
    [sym_release_keyword] = ACTIONS(1601),
    [aux_sym_on_keyword_token1] = ACTIONS(1601),
    [aux_sym_off_keyword_token1] = ACTIONS(1601),
    [sym_send_level_keyword] = ACTIONS(1601),
    [sym_send_string_keyword] = ACTIONS(1601),
    [sym_send_command_keyword] = ACTIONS(1601),
    [sym_clear_buffer_keyword] = ACTIONS(1601),
    [sym_create_buffer_keyword] = ACTIONS(1601),
    [sym_create_multi_buffer_keyword] = ACTIONS(1601),
    [sym_call_keyword] = ACTIONS(1601),
    [sym_system_call_keyword] = ACTIONS(1601),
    [sym_devchan_to_keyword] = ACTIONS(1601),
    [sym_devchan_min_to_keyword] = ACTIONS(1601),
    [sym_devchan_total_off_keyword] = ACTIONS(1601),
    [sym_devchan_pulse_keyword] = ACTIONS(1601),
    [sym_wait_keyword] = ACTIONS(1601),
    [sym_cancel_wait_keyword] = ACTIONS(1601),
    [sym_wait_until_keyword] = ACTIONS(1601),
    [sym_cancel_wait_until_keyword] = ACTIONS(1601),
    [sym_cancel_all_wait_keyword] = ACTIONS(1601),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1601),
    [sym_not] = ACTIONS(1601),
    [sym_bnot] = ACTIONS(1601),
    [sym___file__] = ACTIONS(1601),
    [sym___date__] = ACTIONS(1601),
    [sym___ldate__] = ACTIONS(1601),
    [sym___time__] = ACTIONS(1601),
    [sym___line__] = ACTIONS(1601),
    [sym___name__] = ACTIONS(1601),
    [sym___netlinx__] = ACTIONS(1601),
    [sym_netlinx_axi_version] = ACTIONS(1601),
    [sym_url_flg_tcp] = ACTIONS(1601),
    [sym_url_flg_accinfopresent] = ACTIONS(1601),
    [sym_url_flg_temp] = ACTIONS(1601),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1601),
    [sym_url_flg_stat_mask] = ACTIONS(1601),
    [sym_url_flg_stat_lookup] = ACTIONS(1601),
    [sym_url_flg_stat_connecting] = ACTIONS(1601),
    [sym_url_flg_stat_waiting] = ACTIONS(1601),
    [sym_url_flg_stat_connected] = ACTIONS(1601),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1601),
    [sym_first_virtual_device] = ACTIONS(1601),
    [sym_first_local_port] = ACTIONS(1601),
    [sym_dynamic_virtual_device] = ACTIONS(1601),
    [sym_timeline_once] = ACTIONS(1601),
    [sym_timeline_repeat] = ACTIONS(1601),
    [sym_timeline_absolute] = ACTIONS(1601),
    [sym_timeline_relative] = ACTIONS(1601),
    [sym_do_push_timed_infinite] = ACTIONS(1601),
    [sym_source_type_no_address] = ACTIONS(1601),
    [sym_source_type_neuron_id] = ACTIONS(1601),
    [sym_source_type_ip_address] = ACTIONS(1601),
    [sym_source_type_axlink] = ACTIONS(1601),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1601),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1601),
    [sym_source_type_ip_socket_address] = ACTIONS(1601),
    [sym_source_type_rs232] = ACTIONS(1601),
    [sym_source_type_internal] = ACTIONS(1601),
    [sym_source_type_mac_address] = ACTIONS(1601),
    [sym_source_type_ipv4_port] = ACTIONS(1601),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1601),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1601),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1601),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1601),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1601),
    [sym_true] = ACTIONS(1601),
    [sym_false] = ACTIONS(1601),
    [sym_file_read_only] = ACTIONS(1601),
    [sym_file_rw_new] = ACTIONS(1601),
    [sym_file_rw_append] = ACTIONS(1601),
    [sym_ip_tcp] = ACTIONS(1601),
    [sym_ip_udp] = ACTIONS(1601),
    [sym_ip_udp_2way] = ACTIONS(1601),
    [sym_xml_encode_types] = ACTIONS(1601),
    [sym_xml_encode_char_as_list] = ACTIONS(1601),
    [sym_xml_encode_le] = ACTIONS(1601),
    [sym_xml_decode_types] = ACTIONS(1601),
    [sym_xml_decode_no_preserve] = ACTIONS(1601),
    [sym_duet_dev_type_display_device] = ACTIONS(1601),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1601),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1601),
    [sym_duet_dev_type_io_device] = ACTIONS(1601),
    [sym_duet_dev_type_relay_device] = ACTIONS(1601),
    [sym_duet_dev_type_ups] = ACTIONS(1601),
    [sym_duet_dev_type_amplifier] = ACTIONS(1601),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1601),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1601),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1601),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1601),
    [sym_duet_dev_type_camera] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1601),
    [sym_duet_dev_type_dss] = ACTIONS(1601),
    [sym_duet_dev_type_dvr] = ACTIONS(1601),
    [sym_duet_dev_type_disc_device] = ACTIONS(1601),
    [sym_duet_dev_type_document_camera] = ACTIONS(1601),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1601),
    [sym_duet_dev_type_hvac] = ACTIONS(1601),
    [sym_duet_dev_type_keypad] = ACTIONS(1601),
    [sym_duet_dev_type_light] = ACTIONS(1601),
    [sym_duet_dev_type_monitor] = ACTIONS(1601),
    [sym_duet_dev_type_motor] = ACTIONS(1601),
    [sym_duet_dev_type_multi_window] = ACTIONS(1601),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1601),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1601),
    [sym_duet_dev_type_receiver] = ACTIONS(1601),
    [sym_duet_dev_type_security_system] = ACTIONS(1601),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1601),
    [sym_duet_dev_type_settop_box] = ACTIONS(1601),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1601),
    [sym_duet_dev_type_switcher] = ACTIONS(1601),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1601),
    [sym_duet_dev_type_tv] = ACTIONS(1601),
    [sym_duet_dev_type_utility] = ACTIONS(1601),
    [sym_duet_dev_type_vcr] = ACTIONS(1601),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1601),
    [sym_duet_dev_type_video_processor] = ACTIONS(1601),
    [sym_duet_dev_type_video_projector] = ACTIONS(1601),
    [sym_duet_dev_type_video_wall] = ACTIONS(1601),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1601),
    [sym_duet_dev_type_weather] = ACTIONS(1601),
    [sym_duet_dev_not_polled] = ACTIONS(1601),
    [sym_duet_dev_polled] = ACTIONS(1601),
    [sym_clkmgr_mode_network] = ACTIONS(1601),
    [sym_clkmgr_mode_standalone] = ACTIONS(1601),
    [sym_normal_standby] = ACTIONS(1601),
    [sym_normal_wake] = ACTIONS(1601),
    [sym_ip_multicast_ttl_option] = ACTIONS(1601),
    [sym_ip_ttl_subnet] = ACTIONS(1601),
    [sym_ip_ttl_site] = ACTIONS(1601),
    [sym_ip_ttl_region] = ACTIONS(1601),
    [sym_ip_ttl_continent] = ACTIONS(1601),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1601),
    [sym_ip_nodelay_on] = ACTIONS(1601),
    [sym_ip_nodelay_off] = ACTIONS(1601),
    [sym_amx_error] = ACTIONS(1601),
    [sym_amx_warning] = ACTIONS(1601),
    [sym_amx_info] = ACTIONS(1601),
    [sym_amx_debug] = ACTIONS(1601),
    [sym_netlinx_login_success] = ACTIONS(1601),
    [sym_netlinx_login_fail] = ACTIONS(1601),
    [sym_netlinx_logout] = ACTIONS(1601),
    [sym_null_str] = ACTIONS(1601),
    [sym_smtp_address] = ACTIONS(1601),
    [sym_smtp_port_number] = ACTIONS(1601),
    [sym_smtp_username] = ACTIONS(1601),
    [sym_smtp_password] = ACTIONS(1601),
    [sym_smtp_require_tls] = ACTIONS(1601),
    [sym_smtp_from] = ACTIONS(1601),
    [sym_smtp_tls_true] = ACTIONS(1601),
    [sym_smtp_tls_false] = ACTIONS(1601),
    [sym_listview_on_row_select_event] = ACTIONS(1601),
    [sym_data_structure_unknown] = ACTIONS(1601),
    [sym_data_structure_datafeed] = ACTIONS(1601),
    [sym_data_structure_datarecordset] = ACTIONS(1601),
    [sym_data_structure_datarecord] = ACTIONS(1601),
    [sym_data_structure_datafield] = ACTIONS(1601),
    [sym_data_max_content_field_count] = ACTIONS(1601),
    [sym_data_max_metadata_field_count] = ACTIONS(1601),
    [sym_data_max_id_length] = ACTIONS(1601),
    [sym_data_max_name_length] = ACTIONS(1601),
    [sym_data_max_description_length] = ACTIONS(1601),
    [sym_data_max_type_length] = ACTIONS(1601),
    [sym_data_max_format_length] = ACTIONS(1601),
    [sym_data_max_source_length] = ACTIONS(1601),
    [sym_data_max_value_length] = ACTIONS(1601),
    [sym_data_max_label_length] = ACTIONS(1601),
    [sym_data_max_url_length] = ACTIONS(1601),
    [sym_max_last_login_info_length] = ACTIONS(1601),
    [sym_data_type_unknown] = ACTIONS(1601),
    [sym_data_type_string] = ACTIONS(1601),
    [sym_data_type_datetime] = ACTIONS(1601),
    [sym_data_type_date] = ACTIONS(1601),
    [sym_data_type_time] = ACTIONS(1601),
    [sym_data_type_image] = ACTIONS(1601),
    [sym_data_format_url] = ACTIONS(1601),
    [sym_data_format_phone] = ACTIONS(1601),
    [sym_data_format_email] = ACTIONS(1601),
    [sym_data_format_iso8601] = ACTIONS(1601),
    [sym_auditlog_priv] = ACTIONS(1601),
    [sym_configuration_priv] = ACTIONS(1601),
    [sym_deviceconfig_priv] = ACTIONS(1601),
    [sym_ftp_priv] = ACTIONS(1601),
    [sym_http_priv] = ACTIONS(1601),
    [sym_networkconfig_priv] = ACTIONS(1601),
    [sym_programport_priv] = ACTIONS(1601),
    [sym_remoteui_priv] = ACTIONS(1601),
    [sym_securitycontrol_priv] = ACTIONS(1601),
    [sym_softwaremanagement_priv] = ACTIONS(1601),
    [sym_terminal_priv] = ACTIONS(1601),
    [sym_tpadmin_priv] = ACTIONS(1601),
    [sym_user1_priv] = ACTIONS(1601),
    [sym_user2_priv] = ACTIONS(1601),
    [sym_user3_priv] = ACTIONS(1601),
    [sym_user4_priv] = ACTIONS(1601),
    [sym_usermanagement_priv] = ACTIONS(1601),
    [sym_valid_account] = ACTIONS(1601),
    [sym_err_name_invalid] = ACTIONS(1601),
    [sym_err_password_invalid] = ACTIONS(1601),
    [sym_err_invalid_account] = ACTIONS(1601),
    [sym_err_no_authorization] = ACTIONS(1601),
    [sym_err_invalid_parameter] = ACTIONS(1601),
    [sym_err_user_locked_out] = ACTIONS(1601),
    [sym_err_user_account_expired] = ACTIONS(1601),
    [sym_err_general_error] = ACTIONS(1601),
    [sym_err_authentication_error] = ACTIONS(1601),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1601),
    [sym_tls_validate_certificate] = ACTIONS(1601),
    [sym_tdata] = ACTIONS(1601),
    [sym_tchannel] = ACTIONS(1601),
    [sym_tlevel] = ACTIONS(1601),
    [sym_tbutton] = ACTIONS(1601),
    [sym_ttimeline] = ACTIONS(1601),
    [sym_tcustom] = ACTIONS(1601),
    [sym_url_struct] = ACTIONS(1601),
    [sym_dns_struct] = ACTIONS(1601),
    [sym_ip_address_struct] = ACTIONS(1601),
    [sym_dev_info_struct] = ACTIONS(1601),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1601),
    [sym_data_feed] = ACTIONS(1601),
    [sym_data_field] = ACTIONS(1601),
    [sym_data_record] = ACTIONS(1601),
    [sym_wc_data_feed] = ACTIONS(1601),
    [sym_wc_data_field] = ACTIONS(1601),
    [sym_wc_data_record] = ACTIONS(1601),
    [sym_last_login_info] = ACTIONS(1601),
    [sym____reserved___] = ACTIONS(1601),
    [sym_date] = ACTIONS(1601),
    [sym_day] = ACTIONS(1601),
    [sym_time] = ACTIONS(1601),
    [sym_ldate] = ACTIONS(1601),
    [sym_push_device] = ACTIONS(1601),
    [sym_push_channel] = ACTIONS(1601),
    [sym_push_devchan] = ACTIONS(1601),
    [sym_release_device] = ACTIONS(1601),
    [sym_release_channel] = ACTIONS(1601),
    [sym_release_devchan] = ACTIONS(1601),
    [sym_master_slot] = ACTIONS(1601),
    [sym_get_pulse_time] = ACTIONS(1601),
    [sym_get_timer] = ACTIONS(1601),
    [sym_master_sn] = ACTIONS(1601),
    [sym_system_number] = ACTIONS(1601),
    [sym_button] = ACTIONS(1601),
    [sym_dv_channel] = ACTIONS(1601),
    [sym_data] = ACTIONS(1601),
    [sym_level] = ACTIONS(1601),
    [sym_timeline] = ACTIONS(1601),
    [sym_channel] = ACTIONS(1601),
    [sym_custom] = ACTIONS(1601),
    [sym_atoi] = ACTIONS(1601),
    [sym_atol] = ACTIONS(1601),
    [sym_atof] = ACTIONS(1601),
    [sym_itoa] = ACTIONS(1601),
    [sym_itohex] = ACTIONS(1601),
    [sym_hextoi] = ACTIONS(1601),
    [sym_ftoa] = ACTIONS(1601),
    [sym_find_string] = ACTIONS(1601),
    [sym_left_string] = ACTIONS(1601),
    [sym_right_string] = ACTIONS(1601),
    [sym_mid_string] = ACTIONS(1601),
    [sym_remove_string] = ACTIONS(1601),
    [sym_get_buffer_string] = ACTIONS(1601),
    [sym_get_buffer_char] = ACTIONS(1601),
    [sym_get_multi_buffer_string] = ACTIONS(1601),
    [sym_length_string] = ACTIONS(1601),
    [sym_max_length_string] = ACTIONS(1601),
    [sym_set_length_string] = ACTIONS(1601),
    [sym_compare_string] = ACTIONS(1601),
    [sym_upper_string] = ACTIONS(1601),
    [sym_lower_string] = ACTIONS(1601),
    [sym_get_last] = ACTIONS(1601),
    [sym_date_to_day] = ACTIONS(1601),
    [sym_date_to_month] = ACTIONS(1601),
    [sym_date_to_year] = ACTIONS(1601),
    [sym_time_to_hour] = ACTIONS(1601),
    [sym_time_to_minute] = ACTIONS(1601),
    [sym_time_to_second] = ACTIONS(1601),
    [sym_day_of_week] = ACTIONS(1601),
    [sym_set_pulse_time] = ACTIONS(1601),
    [sym_set_timer] = ACTIONS(1601),
    [sym_file_open] = ACTIONS(1601),
    [sym_file_close] = ACTIONS(1601),
    [sym_file_read] = ACTIONS(1601),
    [sym_file_read_line] = ACTIONS(1601),
    [sym_file_write] = ACTIONS(1601),
    [sym_file_write_line] = ACTIONS(1601),
    [sym_file_copy] = ACTIONS(1601),
    [sym_file_delete] = ACTIONS(1601),
    [sym_file_rename] = ACTIONS(1601),
    [sym_file_dir] = ACTIONS(1601),
    [sym_file_getdir] = ACTIONS(1601),
    [sym_file_setdir] = ACTIONS(1601),
    [sym_file_create_dir] = ACTIONS(1601),
    [sym_file_remove_dir] = ACTIONS(1601),
    [sym_file_seek] = ACTIONS(1601),
    [sym_length_array] = ACTIONS(1601),
    [sym_max_length_array] = ACTIONS(1601),
    [sym_set_length_array] = ACTIONS(1601),
    [sym_ip_client_open] = ACTIONS(1601),
    [sym_ip_client_close] = ACTIONS(1601),
    [sym_ip_server_open] = ACTIONS(1601),
    [sym_ip_server_close] = ACTIONS(1601),
    [sym_ip_bound_client_open] = ACTIONS(1601),
    [sym_ip_mc_server_open] = ACTIONS(1601),
    [sym_ip_set_option] = ACTIONS(1601),
    [sym_ssh_client_open] = ACTIONS(1601),
    [sym_ssh_client_close] = ACTIONS(1601),
    [sym_tls_client_open] = ACTIONS(1601),
    [sym_tls_client_close] = ACTIONS(1601),
    [sym_do_push] = ACTIONS(1601),
    [sym_do_push_timed] = ACTIONS(1601),
    [sym_do_release] = ACTIONS(1601),
    [sym_device_id] = ACTIONS(1601),
    [sym_device_info] = ACTIONS(1601),
    [sym_device_id_string] = ACTIONS(1601),
    [sym_device_standby] = ACTIONS(1601),
    [sym_device_wake] = ACTIONS(1601),
    [sym_timeline_create] = ACTIONS(1601),
    [sym_timeline_kill] = ACTIONS(1601),
    [sym_timeline_reload] = ACTIONS(1601),
    [sym_timeline_pause] = ACTIONS(1601),
    [sym_timeline_restart] = ACTIONS(1601),
    [sym_timeline_set] = ACTIONS(1601),
    [sym_timeline_get] = ACTIONS(1601),
    [sym_timeline_active] = ACTIONS(1601),
    [sym_timeline_dynamic_id] = ACTIONS(1601),
    [sym_variable_to_string] = ACTIONS(1601),
    [sym_string_to_variable] = ACTIONS(1601),
    [sym_length_variable_to_string] = ACTIONS(1601),
    [sym_type_cast] = ACTIONS(1601),
    [sym_variable_to_xml] = ACTIONS(1601),
    [sym_xml_to_variable] = ACTIONS(1601),
    [sym_length_variable_to_xml] = ACTIONS(1601),
    [sym_redirect_string] = ACTIONS(1601),
    [sym_random_number] = ACTIONS(1601),
    [sym_get_unique_id] = ACTIONS(1601),
    [sym_get_system_number] = ACTIONS(1601),
    [sym_set_system_number] = ACTIONS(1601),
    [sym_get_serial_number] = ACTIONS(1601),
    [sym_get_master_build] = ACTIONS(1601),
    [sym_set_validation_code] = ACTIONS(1601),
    [sym_reboot] = ACTIONS(1601),
    [sym_load_duet_module] = ACTIONS(1601),
    [sym_unload_duet_module] = ACTIONS(1601),
    [sym_duet_mem_size_set] = ACTIONS(1601),
    [sym_duet_mem_size_get] = ACTIONS(1601),
    [sym_dynamic_application_device] = ACTIONS(1601),
    [sym_dynamic_polled_port] = ACTIONS(1601),
    [sym_static_port_binding] = ACTIONS(1601),
    [sym_static_ip_binding] = ACTIONS(1601),
    [sym_get_dns_list] = ACTIONS(1601),
    [sym_set_dns_list] = ACTIONS(1601),
    [sym_get_ip_address] = ACTIONS(1601),
    [sym_set_ip_address] = ACTIONS(1601),
    [sym_get_url_list] = ACTIONS(1601),
    [sym_add_url_entry] = ACTIONS(1601),
    [sym_delete_url_entry] = ACTIONS(1601),
    [sym_abs_value] = ACTIONS(1601),
    [sym_max_value] = ACTIONS(1601),
    [sym_min_value] = ACTIONS(1601),
    [sym_exp_value] = ACTIONS(1601),
    [sym_log_value] = ACTIONS(1601),
    [sym_log10_value] = ACTIONS(1601),
    [sym_power_value] = ACTIONS(1601),
    [sym_sqrt_value] = ACTIONS(1601),
    [sym_raw_be] = ACTIONS(1601),
    [sym_raw_le] = ACTIONS(1601),
    [sym_internal_queue_size_set] = ACTIONS(1601),
    [sym_internal_queue_size_get] = ACTIONS(1601),
    [sym_internal_threshold_set] = ACTIONS(1601),
    [sym_internal_threshold_get] = ACTIONS(1601),
    [sym_rebuilt_event] = ACTIONS(1601),
    [sym_do_custom_event] = ACTIONS(1601),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1601),
    [sym_clkmgr_set_clk_source] = ACTIONS(1601),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1601),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1601),
    [sym_clkmgr_get_timezone] = ACTIONS(1601),
    [sym_clkmgr_set_timezone] = ACTIONS(1601),
    [sym_clkmgr_get_resync_period] = ACTIONS(1601),
    [sym_clkmgr_set_resync_period] = ACTIONS(1601),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1601),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1601),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1601),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1601),
    [sym_clkmgr_get_timeservers] = ACTIONS(1601),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1601),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1601),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1601),
    [sym_set_log_level] = ACTIONS(1601),
    [sym_get_log_level] = ACTIONS(1601),
    [sym_amx_log] = ACTIONS(1601),
    [sym_smtp_server_config_set] = ACTIONS(1601),
    [sym_smtp_server_config_get] = ACTIONS(1601),
    [sym_smtp_send] = ACTIONS(1601),
    [sym_validate_netlinx_account] = ACTIONS(1601),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1601),
    [sym_authenticate_certificate] = ACTIONS(1601),
    [sym_audit_netlinx_session_event] = ACTIONS(1601),
    [sym_audit_netlinx_generic_event] = ACTIONS(1601),
    [sym_get_available_flash_disk_space] = ACTIONS(1601),
    [sym_get_max_flash_disk_space] = ACTIONS(1601),
    [sym_data_create_feed] = ACTIONS(1601),
    [sym_data_delete_feed] = ACTIONS(1601),
    [sym_data_publish_feed] = ACTIONS(1601),
    [sym_data_get_published_feed] = ACTIONS(1601),
    [sym_data_add_record] = ACTIONS(1601),
    [sym_data_get_event_record] = ACTIONS(1601),
    [sym__wc_data_create_feed] = ACTIONS(1601),
    [sym__wc_data_add_record] = ACTIONS(1601),
    [sym__wc_data_get_event_record] = ACTIONS(1601),
    [sym_format] = ACTIONS(1601),
    [sym_astro_clock] = ACTIONS(1601),
    [sym_set_outdoor_temperature] = ACTIONS(1601),
    [sym_set_virtual_level_count] = ACTIONS(1601),
    [sym_set_virtual_channel_count] = ACTIONS(1601),
    [sym_set_virtual_port_count] = ACTIONS(1601),
    [sym__wc_encode_int] = ACTIONS(1601),
    [sym__wc_decode_int] = ACTIONS(1601),
    [sym__wc_int] = ACTIONS(1601),
    [sym__wc_to_ch_int] = ACTIONS(1601),
    [sym__ch_to_wc_int] = ACTIONS(1601),
    [sym__wc_find_string_int] = ACTIONS(1601),
    [sym__wc_left_string_int] = ACTIONS(1601),
    [sym__wc_lower_string_int] = ACTIONS(1601),
    [sym__wc_mid_string_int] = ACTIONS(1601),
    [sym__wc_remove_string_int] = ACTIONS(1601),
    [sym__wc_right_string_int] = ACTIONS(1601),
    [sym__wc_upper_string_int] = ACTIONS(1601),
    [sym__wc_compare_string_int] = ACTIONS(1601),
    [sym__wc_get_buffer_char_int] = ACTIONS(1601),
    [sym__wc_get_buffer_string_int] = ACTIONS(1601),
    [sym__wc_concat_string_int] = ACTIONS(1601),
    [sym__wc_file_open_int] = ACTIONS(1601),
    [sym__wc_file_close_int] = ACTIONS(1601),
    [sym__wc_file_read_int] = ACTIONS(1601),
    [sym__wc_file_read_line_int] = ACTIONS(1601),
    [sym__wc_file_write_int] = ACTIONS(1601),
    [sym__wc_file_write_line_int] = ACTIONS(1601),
    [aux_sym_preproc_include_token1] = ACTIONS(1601),
    [aux_sym_preproc_define_token1] = ACTIONS(1603),
    [aux_sym_preproc_warn_token1] = ACTIONS(1603),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1603),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1603),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1603),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1603),
    [aux_sym_preproc_else_token1] = ACTIONS(1603),
    [anon_sym_LBRACK] = ACTIONS(1603),
    [anon_sym_LBRACE] = ACTIONS(1603),
    [anon_sym_LPAREN] = ACTIONS(1601),
    [anon_sym_BANG] = ACTIONS(1603),
    [anon_sym_TILDE] = ACTIONS(1603),
    [anon_sym_DASH] = ACTIONS(1601),
    [anon_sym_PLUS] = ACTIONS(1601),
    [anon_sym_DASH_DASH] = ACTIONS(1603),
    [anon_sym_PLUS_PLUS] = ACTIONS(1603),
    [anon_sym_DQUOTE] = ACTIONS(1603),
    [anon_sym_SQUOTE] = ACTIONS(1603),
    [sym_number_literal] = ACTIONS(1603),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(373)] = {
    [sym_identifier] = ACTIONS(2109),
    [sym_program_name_keyword] = ACTIONS(2109),
    [sym_module_name_keyword] = ACTIONS(2109),
    [sym_define_device_keyword] = ACTIONS(2109),
    [sym_define_combine_keyword] = ACTIONS(2109),
    [sym_define_constant_keyword] = ACTIONS(2109),
    [sym_define_type_keyword] = ACTIONS(2109),
    [sym_define_variable_keyword] = ACTIONS(2109),
    [sym_define_system_variable_keyword] = ACTIONS(2109),
    [sym_define_connect_level_keyword] = ACTIONS(2109),
    [sym_define_latching_keyword] = ACTIONS(2109),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2109),
    [sym_define_toggling_keyword] = ACTIONS(2109),
    [sym_define_call_keyword] = ACTIONS(2109),
    [sym_define_function_keyword] = ACTIONS(2109),
    [sym_define_library_function_keyword] = ACTIONS(2109),
    [sym_define_start_keyword] = ACTIONS(2109),
    [sym_define_event_keyword] = ACTIONS(2109),
    [sym_define_program_keyword] = ACTIONS(2109),
    [sym_define_module_keyword] = ACTIONS(2109),
    [sym_char_keyword] = ACTIONS(2109),
    [sym_widechar_keyword] = ACTIONS(2109),
    [sym_integer_keyword] = ACTIONS(2109),
    [sym_sinteger_keyword] = ACTIONS(2109),
    [sym_long_keyword] = ACTIONS(2109),
    [sym_slong_keyword] = ACTIONS(2109),
    [sym_float_keyword] = ACTIONS(2109),
    [sym_double_keyword] = ACTIONS(2109),
    [sym_variant_keyword] = ACTIONS(2109),
    [sym_variantarray_keyword] = ACTIONS(2109),
    [sym_local_var_keyword] = ACTIONS(2109),
    [sym_stack_var_keyword] = ACTIONS(2109),
    [sym_constant_keyword] = ACTIONS(2109),
    [sym_volatile_keyword] = ACTIONS(2109),
    [sym_non_volatile_keyword] = ACTIONS(2109),
    [sym_persistent_keyword] = ACTIONS(2109),
    [sym_dev_keyword] = ACTIONS(2109),
    [sym_devlev_keyword] = ACTIONS(2109),
    [sym_devchan_keyword] = ACTIONS(2109),
    [sym_if_keyword] = ACTIONS(2109),
    [sym_switch_keyword] = ACTIONS(2109),
    [sym_case_keyword] = ACTIONS(2109),
    [sym_default_keyword] = ACTIONS(2109),
    [sym_while_keyword] = ACTIONS(2109),
    [sym_for_keyword] = ACTIONS(2109),
    [sym_break_keyword] = ACTIONS(2109),
    [sym_continue_keyword] = ACTIONS(2109),
    [sym_return_keyword] = ACTIONS(2109),
    [sym_select_keyword] = ACTIONS(2109),
    [sym_active_keyword] = ACTIONS(2109),
    [sym_struct_keyword] = ACTIONS(2109),
    [sym_structure_keyword] = ACTIONS(2109),
    [sym_button_event_keyword] = ACTIONS(2109),
    [sym_channel_event_keyword] = ACTIONS(2109),
    [sym_level_event_keyword] = ACTIONS(2109),
    [sym_data_event_keyword] = ACTIONS(2109),
    [sym_timeline_event_keyword] = ACTIONS(2109),
    [sym_custom_event_keyword] = ACTIONS(2109),
    [sym_push_keyword] = ACTIONS(2109),
    [sym_release_keyword] = ACTIONS(2109),
    [aux_sym_on_keyword_token1] = ACTIONS(2109),
    [aux_sym_off_keyword_token1] = ACTIONS(2109),
    [sym_send_level_keyword] = ACTIONS(2109),
    [sym_send_string_keyword] = ACTIONS(2109),
    [sym_send_command_keyword] = ACTIONS(2109),
    [sym_clear_buffer_keyword] = ACTIONS(2109),
    [sym_create_buffer_keyword] = ACTIONS(2109),
    [sym_create_multi_buffer_keyword] = ACTIONS(2109),
    [sym_call_keyword] = ACTIONS(2109),
    [sym_system_call_keyword] = ACTIONS(2109),
    [sym_devchan_to_keyword] = ACTIONS(2109),
    [sym_devchan_min_to_keyword] = ACTIONS(2109),
    [sym_devchan_total_off_keyword] = ACTIONS(2109),
    [sym_devchan_pulse_keyword] = ACTIONS(2109),
    [sym_wait_keyword] = ACTIONS(2109),
    [sym_cancel_wait_keyword] = ACTIONS(2109),
    [sym_wait_until_keyword] = ACTIONS(2109),
    [sym_cancel_wait_until_keyword] = ACTIONS(2109),
    [sym_cancel_all_wait_keyword] = ACTIONS(2109),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2109),
    [sym_not] = ACTIONS(2109),
    [sym_bnot] = ACTIONS(2109),
    [sym___file__] = ACTIONS(2109),
    [sym___date__] = ACTIONS(2109),
    [sym___ldate__] = ACTIONS(2109),
    [sym___time__] = ACTIONS(2109),
    [sym___line__] = ACTIONS(2109),
    [sym___name__] = ACTIONS(2109),
    [sym___netlinx__] = ACTIONS(2109),
    [sym_netlinx_axi_version] = ACTIONS(2109),
    [sym_url_flg_tcp] = ACTIONS(2109),
    [sym_url_flg_accinfopresent] = ACTIONS(2109),
    [sym_url_flg_temp] = ACTIONS(2109),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2109),
    [sym_url_flg_stat_mask] = ACTIONS(2109),
    [sym_url_flg_stat_lookup] = ACTIONS(2109),
    [sym_url_flg_stat_connecting] = ACTIONS(2109),
    [sym_url_flg_stat_waiting] = ACTIONS(2109),
    [sym_url_flg_stat_connected] = ACTIONS(2109),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2109),
    [sym_first_virtual_device] = ACTIONS(2109),
    [sym_first_local_port] = ACTIONS(2109),
    [sym_dynamic_virtual_device] = ACTIONS(2109),
    [sym_timeline_once] = ACTIONS(2109),
    [sym_timeline_repeat] = ACTIONS(2109),
    [sym_timeline_absolute] = ACTIONS(2109),
    [sym_timeline_relative] = ACTIONS(2109),
    [sym_do_push_timed_infinite] = ACTIONS(2109),
    [sym_source_type_no_address] = ACTIONS(2109),
    [sym_source_type_neuron_id] = ACTIONS(2109),
    [sym_source_type_ip_address] = ACTIONS(2109),
    [sym_source_type_axlink] = ACTIONS(2109),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2109),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2109),
    [sym_source_type_ip_socket_address] = ACTIONS(2109),
    [sym_source_type_rs232] = ACTIONS(2109),
    [sym_source_type_internal] = ACTIONS(2109),
    [sym_source_type_mac_address] = ACTIONS(2109),
    [sym_source_type_ipv4_port] = ACTIONS(2109),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2109),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2109),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2109),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2109),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2109),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2109),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2109),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2109),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2109),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2109),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2109),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2109),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2109),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2109),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2109),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2109),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2109),
    [sym_true] = ACTIONS(2109),
    [sym_false] = ACTIONS(2109),
    [sym_file_read_only] = ACTIONS(2109),
    [sym_file_rw_new] = ACTIONS(2109),
    [sym_file_rw_append] = ACTIONS(2109),
    [sym_ip_tcp] = ACTIONS(2109),
    [sym_ip_udp] = ACTIONS(2109),
    [sym_ip_udp_2way] = ACTIONS(2109),
    [sym_xml_encode_types] = ACTIONS(2109),
    [sym_xml_encode_char_as_list] = ACTIONS(2109),
    [sym_xml_encode_le] = ACTIONS(2109),
    [sym_xml_decode_types] = ACTIONS(2109),
    [sym_xml_decode_no_preserve] = ACTIONS(2109),
    [sym_duet_dev_type_display_device] = ACTIONS(2109),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2109),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2109),
    [sym_duet_dev_type_io_device] = ACTIONS(2109),
    [sym_duet_dev_type_relay_device] = ACTIONS(2109),
    [sym_duet_dev_type_ups] = ACTIONS(2109),
    [sym_duet_dev_type_amplifier] = ACTIONS(2109),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2109),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2109),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2109),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2109),
    [sym_duet_dev_type_camera] = ACTIONS(2109),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2109),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2109),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2109),
    [sym_duet_dev_type_dss] = ACTIONS(2109),
    [sym_duet_dev_type_dvr] = ACTIONS(2109),
    [sym_duet_dev_type_disc_device] = ACTIONS(2109),
    [sym_duet_dev_type_document_camera] = ACTIONS(2109),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2109),
    [sym_duet_dev_type_hvac] = ACTIONS(2109),
    [sym_duet_dev_type_keypad] = ACTIONS(2109),
    [sym_duet_dev_type_light] = ACTIONS(2109),
    [sym_duet_dev_type_monitor] = ACTIONS(2109),
    [sym_duet_dev_type_motor] = ACTIONS(2109),
    [sym_duet_dev_type_multi_window] = ACTIONS(2109),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2109),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2109),
    [sym_duet_dev_type_receiver] = ACTIONS(2109),
    [sym_duet_dev_type_security_system] = ACTIONS(2109),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2109),
    [sym_duet_dev_type_settop_box] = ACTIONS(2109),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2109),
    [sym_duet_dev_type_switcher] = ACTIONS(2109),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2109),
    [sym_duet_dev_type_tv] = ACTIONS(2109),
    [sym_duet_dev_type_utility] = ACTIONS(2109),
    [sym_duet_dev_type_vcr] = ACTIONS(2109),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2109),
    [sym_duet_dev_type_video_processor] = ACTIONS(2109),
    [sym_duet_dev_type_video_projector] = ACTIONS(2109),
    [sym_duet_dev_type_video_wall] = ACTIONS(2109),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2109),
    [sym_duet_dev_type_weather] = ACTIONS(2109),
    [sym_duet_dev_not_polled] = ACTIONS(2109),
    [sym_duet_dev_polled] = ACTIONS(2109),
    [sym_clkmgr_mode_network] = ACTIONS(2109),
    [sym_clkmgr_mode_standalone] = ACTIONS(2109),
    [sym_normal_standby] = ACTIONS(2109),
    [sym_normal_wake] = ACTIONS(2109),
    [sym_ip_multicast_ttl_option] = ACTIONS(2109),
    [sym_ip_ttl_subnet] = ACTIONS(2109),
    [sym_ip_ttl_site] = ACTIONS(2109),
    [sym_ip_ttl_region] = ACTIONS(2109),
    [sym_ip_ttl_continent] = ACTIONS(2109),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2109),
    [sym_ip_nodelay_on] = ACTIONS(2109),
    [sym_ip_nodelay_off] = ACTIONS(2109),
    [sym_amx_error] = ACTIONS(2109),
    [sym_amx_warning] = ACTIONS(2109),
    [sym_amx_info] = ACTIONS(2109),
    [sym_amx_debug] = ACTIONS(2109),
    [sym_netlinx_login_success] = ACTIONS(2109),
    [sym_netlinx_login_fail] = ACTIONS(2109),
    [sym_netlinx_logout] = ACTIONS(2109),
    [sym_null_str] = ACTIONS(2109),
    [sym_smtp_address] = ACTIONS(2109),
    [sym_smtp_port_number] = ACTIONS(2109),
    [sym_smtp_username] = ACTIONS(2109),
    [sym_smtp_password] = ACTIONS(2109),
    [sym_smtp_require_tls] = ACTIONS(2109),
    [sym_smtp_from] = ACTIONS(2109),
    [sym_smtp_tls_true] = ACTIONS(2109),
    [sym_smtp_tls_false] = ACTIONS(2109),
    [sym_listview_on_row_select_event] = ACTIONS(2109),
    [sym_data_structure_unknown] = ACTIONS(2109),
    [sym_data_structure_datafeed] = ACTIONS(2109),
    [sym_data_structure_datarecordset] = ACTIONS(2109),
    [sym_data_structure_datarecord] = ACTIONS(2109),
    [sym_data_structure_datafield] = ACTIONS(2109),
    [sym_data_max_content_field_count] = ACTIONS(2109),
    [sym_data_max_metadata_field_count] = ACTIONS(2109),
    [sym_data_max_id_length] = ACTIONS(2109),
    [sym_data_max_name_length] = ACTIONS(2109),
    [sym_data_max_description_length] = ACTIONS(2109),
    [sym_data_max_type_length] = ACTIONS(2109),
    [sym_data_max_format_length] = ACTIONS(2109),
    [sym_data_max_source_length] = ACTIONS(2109),
    [sym_data_max_value_length] = ACTIONS(2109),
    [sym_data_max_label_length] = ACTIONS(2109),
    [sym_data_max_url_length] = ACTIONS(2109),
    [sym_max_last_login_info_length] = ACTIONS(2109),
    [sym_data_type_unknown] = ACTIONS(2109),
    [sym_data_type_string] = ACTIONS(2109),
    [sym_data_type_datetime] = ACTIONS(2109),
    [sym_data_type_date] = ACTIONS(2109),
    [sym_data_type_time] = ACTIONS(2109),
    [sym_data_type_image] = ACTIONS(2109),
    [sym_data_format_url] = ACTIONS(2109),
    [sym_data_format_phone] = ACTIONS(2109),
    [sym_data_format_email] = ACTIONS(2109),
    [sym_data_format_iso8601] = ACTIONS(2109),
    [sym_auditlog_priv] = ACTIONS(2109),
    [sym_configuration_priv] = ACTIONS(2109),
    [sym_deviceconfig_priv] = ACTIONS(2109),
    [sym_ftp_priv] = ACTIONS(2109),
    [sym_http_priv] = ACTIONS(2109),
    [sym_networkconfig_priv] = ACTIONS(2109),
    [sym_programport_priv] = ACTIONS(2109),
    [sym_remoteui_priv] = ACTIONS(2109),
    [sym_securitycontrol_priv] = ACTIONS(2109),
    [sym_softwaremanagement_priv] = ACTIONS(2109),
    [sym_terminal_priv] = ACTIONS(2109),
    [sym_tpadmin_priv] = ACTIONS(2109),
    [sym_user1_priv] = ACTIONS(2109),
    [sym_user2_priv] = ACTIONS(2109),
    [sym_user3_priv] = ACTIONS(2109),
    [sym_user4_priv] = ACTIONS(2109),
    [sym_usermanagement_priv] = ACTIONS(2109),
    [sym_valid_account] = ACTIONS(2109),
    [sym_err_name_invalid] = ACTIONS(2109),
    [sym_err_password_invalid] = ACTIONS(2109),
    [sym_err_invalid_account] = ACTIONS(2109),
    [sym_err_no_authorization] = ACTIONS(2109),
    [sym_err_invalid_parameter] = ACTIONS(2109),
    [sym_err_user_locked_out] = ACTIONS(2109),
    [sym_err_user_account_expired] = ACTIONS(2109),
    [sym_err_general_error] = ACTIONS(2109),
    [sym_err_authentication_error] = ACTIONS(2109),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2109),
    [sym_tls_validate_certificate] = ACTIONS(2109),
    [sym_tdata] = ACTIONS(2109),
    [sym_tchannel] = ACTIONS(2109),
    [sym_tlevel] = ACTIONS(2109),
    [sym_tbutton] = ACTIONS(2109),
    [sym_ttimeline] = ACTIONS(2109),
    [sym_tcustom] = ACTIONS(2109),
    [sym_url_struct] = ACTIONS(2109),
    [sym_dns_struct] = ACTIONS(2109),
    [sym_ip_address_struct] = ACTIONS(2109),
    [sym_dev_info_struct] = ACTIONS(2109),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2109),
    [sym_data_feed] = ACTIONS(2109),
    [sym_data_field] = ACTIONS(2109),
    [sym_data_record] = ACTIONS(2109),
    [sym_wc_data_feed] = ACTIONS(2109),
    [sym_wc_data_field] = ACTIONS(2109),
    [sym_wc_data_record] = ACTIONS(2109),
    [sym_last_login_info] = ACTIONS(2109),
    [sym____reserved___] = ACTIONS(2109),
    [sym_date] = ACTIONS(2109),
    [sym_day] = ACTIONS(2109),
    [sym_time] = ACTIONS(2109),
    [sym_ldate] = ACTIONS(2109),
    [sym_push_device] = ACTIONS(2109),
    [sym_push_channel] = ACTIONS(2109),
    [sym_push_devchan] = ACTIONS(2109),
    [sym_release_device] = ACTIONS(2109),
    [sym_release_channel] = ACTIONS(2109),
    [sym_release_devchan] = ACTIONS(2109),
    [sym_master_slot] = ACTIONS(2109),
    [sym_get_pulse_time] = ACTIONS(2109),
    [sym_get_timer] = ACTIONS(2109),
    [sym_master_sn] = ACTIONS(2109),
    [sym_system_number] = ACTIONS(2109),
    [sym_button] = ACTIONS(2109),
    [sym_dv_channel] = ACTIONS(2109),
    [sym_data] = ACTIONS(2109),
    [sym_level] = ACTIONS(2109),
    [sym_timeline] = ACTIONS(2109),
    [sym_channel] = ACTIONS(2109),
    [sym_custom] = ACTIONS(2109),
    [sym_atoi] = ACTIONS(2109),
    [sym_atol] = ACTIONS(2109),
    [sym_atof] = ACTIONS(2109),
    [sym_itoa] = ACTIONS(2109),
    [sym_itohex] = ACTIONS(2109),
    [sym_hextoi] = ACTIONS(2109),
    [sym_ftoa] = ACTIONS(2109),
    [sym_find_string] = ACTIONS(2109),
    [sym_left_string] = ACTIONS(2109),
    [sym_right_string] = ACTIONS(2109),
    [sym_mid_string] = ACTIONS(2109),
    [sym_remove_string] = ACTIONS(2109),
    [sym_get_buffer_string] = ACTIONS(2109),
    [sym_get_buffer_char] = ACTIONS(2109),
    [sym_get_multi_buffer_string] = ACTIONS(2109),
    [sym_length_string] = ACTIONS(2109),
    [sym_max_length_string] = ACTIONS(2109),
    [sym_set_length_string] = ACTIONS(2109),
    [sym_compare_string] = ACTIONS(2109),
    [sym_upper_string] = ACTIONS(2109),
    [sym_lower_string] = ACTIONS(2109),
    [sym_get_last] = ACTIONS(2109),
    [sym_date_to_day] = ACTIONS(2109),
    [sym_date_to_month] = ACTIONS(2109),
    [sym_date_to_year] = ACTIONS(2109),
    [sym_time_to_hour] = ACTIONS(2109),
    [sym_time_to_minute] = ACTIONS(2109),
    [sym_time_to_second] = ACTIONS(2109),
    [sym_day_of_week] = ACTIONS(2109),
    [sym_set_pulse_time] = ACTIONS(2109),
    [sym_set_timer] = ACTIONS(2109),
    [sym_file_open] = ACTIONS(2109),
    [sym_file_close] = ACTIONS(2109),
    [sym_file_read] = ACTIONS(2109),
    [sym_file_read_line] = ACTIONS(2109),
    [sym_file_write] = ACTIONS(2109),
    [sym_file_write_line] = ACTIONS(2109),
    [sym_file_copy] = ACTIONS(2109),
    [sym_file_delete] = ACTIONS(2109),
    [sym_file_rename] = ACTIONS(2109),
    [sym_file_dir] = ACTIONS(2109),
    [sym_file_getdir] = ACTIONS(2109),
    [sym_file_setdir] = ACTIONS(2109),
    [sym_file_create_dir] = ACTIONS(2109),
    [sym_file_remove_dir] = ACTIONS(2109),
    [sym_file_seek] = ACTIONS(2109),
    [sym_length_array] = ACTIONS(2109),
    [sym_max_length_array] = ACTIONS(2109),
    [sym_set_length_array] = ACTIONS(2109),
    [sym_ip_client_open] = ACTIONS(2109),
    [sym_ip_client_close] = ACTIONS(2109),
    [sym_ip_server_open] = ACTIONS(2109),
    [sym_ip_server_close] = ACTIONS(2109),
    [sym_ip_bound_client_open] = ACTIONS(2109),
    [sym_ip_mc_server_open] = ACTIONS(2109),
    [sym_ip_set_option] = ACTIONS(2109),
    [sym_ssh_client_open] = ACTIONS(2109),
    [sym_ssh_client_close] = ACTIONS(2109),
    [sym_tls_client_open] = ACTIONS(2109),
    [sym_tls_client_close] = ACTIONS(2109),
    [sym_do_push] = ACTIONS(2109),
    [sym_do_push_timed] = ACTIONS(2109),
    [sym_do_release] = ACTIONS(2109),
    [sym_device_id] = ACTIONS(2109),
    [sym_device_info] = ACTIONS(2109),
    [sym_device_id_string] = ACTIONS(2109),
    [sym_device_standby] = ACTIONS(2109),
    [sym_device_wake] = ACTIONS(2109),
    [sym_timeline_create] = ACTIONS(2109),
    [sym_timeline_kill] = ACTIONS(2109),
    [sym_timeline_reload] = ACTIONS(2109),
    [sym_timeline_pause] = ACTIONS(2109),
    [sym_timeline_restart] = ACTIONS(2109),
    [sym_timeline_set] = ACTIONS(2109),
    [sym_timeline_get] = ACTIONS(2109),
    [sym_timeline_active] = ACTIONS(2109),
    [sym_timeline_dynamic_id] = ACTIONS(2109),
    [sym_variable_to_string] = ACTIONS(2109),
    [sym_string_to_variable] = ACTIONS(2109),
    [sym_length_variable_to_string] = ACTIONS(2109),
    [sym_type_cast] = ACTIONS(2109),
    [sym_variable_to_xml] = ACTIONS(2109),
    [sym_xml_to_variable] = ACTIONS(2109),
    [sym_length_variable_to_xml] = ACTIONS(2109),
    [sym_redirect_string] = ACTIONS(2109),
    [sym_random_number] = ACTIONS(2109),
    [sym_get_unique_id] = ACTIONS(2109),
    [sym_get_system_number] = ACTIONS(2109),
    [sym_set_system_number] = ACTIONS(2109),
    [sym_get_serial_number] = ACTIONS(2109),
    [sym_get_master_build] = ACTIONS(2109),
    [sym_set_validation_code] = ACTIONS(2109),
    [sym_reboot] = ACTIONS(2109),
    [sym_load_duet_module] = ACTIONS(2109),
    [sym_unload_duet_module] = ACTIONS(2109),
    [sym_duet_mem_size_set] = ACTIONS(2109),
    [sym_duet_mem_size_get] = ACTIONS(2109),
    [sym_dynamic_application_device] = ACTIONS(2109),
    [sym_dynamic_polled_port] = ACTIONS(2109),
    [sym_static_port_binding] = ACTIONS(2109),
    [sym_static_ip_binding] = ACTIONS(2109),
    [sym_get_dns_list] = ACTIONS(2109),
    [sym_set_dns_list] = ACTIONS(2109),
    [sym_get_ip_address] = ACTIONS(2109),
    [sym_set_ip_address] = ACTIONS(2109),
    [sym_get_url_list] = ACTIONS(2109),
    [sym_add_url_entry] = ACTIONS(2109),
    [sym_delete_url_entry] = ACTIONS(2109),
    [sym_abs_value] = ACTIONS(2109),
    [sym_max_value] = ACTIONS(2109),
    [sym_min_value] = ACTIONS(2109),
    [sym_exp_value] = ACTIONS(2109),
    [sym_log_value] = ACTIONS(2109),
    [sym_log10_value] = ACTIONS(2109),
    [sym_power_value] = ACTIONS(2109),
    [sym_sqrt_value] = ACTIONS(2109),
    [sym_raw_be] = ACTIONS(2109),
    [sym_raw_le] = ACTIONS(2109),
    [sym_internal_queue_size_set] = ACTIONS(2109),
    [sym_internal_queue_size_get] = ACTIONS(2109),
    [sym_internal_threshold_set] = ACTIONS(2109),
    [sym_internal_threshold_get] = ACTIONS(2109),
    [sym_rebuilt_event] = ACTIONS(2109),
    [sym_do_custom_event] = ACTIONS(2109),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2109),
    [sym_clkmgr_set_clk_source] = ACTIONS(2109),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2109),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2109),
    [sym_clkmgr_get_timezone] = ACTIONS(2109),
    [sym_clkmgr_set_timezone] = ACTIONS(2109),
    [sym_clkmgr_get_resync_period] = ACTIONS(2109),
    [sym_clkmgr_set_resync_period] = ACTIONS(2109),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2109),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2109),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2109),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2109),
    [sym_clkmgr_get_timeservers] = ACTIONS(2109),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2109),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2109),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2109),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2109),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2109),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2109),
    [sym_set_log_level] = ACTIONS(2109),
    [sym_get_log_level] = ACTIONS(2109),
    [sym_amx_log] = ACTIONS(2109),
    [sym_smtp_server_config_set] = ACTIONS(2109),
    [sym_smtp_server_config_get] = ACTIONS(2109),
    [sym_smtp_send] = ACTIONS(2109),
    [sym_validate_netlinx_account] = ACTIONS(2109),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2109),
    [sym_authenticate_certificate] = ACTIONS(2109),
    [sym_audit_netlinx_session_event] = ACTIONS(2109),
    [sym_audit_netlinx_generic_event] = ACTIONS(2109),
    [sym_get_available_flash_disk_space] = ACTIONS(2109),
    [sym_get_max_flash_disk_space] = ACTIONS(2109),
    [sym_data_create_feed] = ACTIONS(2109),
    [sym_data_delete_feed] = ACTIONS(2109),
    [sym_data_publish_feed] = ACTIONS(2109),
    [sym_data_get_published_feed] = ACTIONS(2109),
    [sym_data_add_record] = ACTIONS(2109),
    [sym_data_get_event_record] = ACTIONS(2109),
    [sym__wc_data_create_feed] = ACTIONS(2109),
    [sym__wc_data_add_record] = ACTIONS(2109),
    [sym__wc_data_get_event_record] = ACTIONS(2109),
    [sym_format] = ACTIONS(2109),
    [sym_astro_clock] = ACTIONS(2109),
    [sym_set_outdoor_temperature] = ACTIONS(2109),
    [sym_set_virtual_level_count] = ACTIONS(2109),
    [sym_set_virtual_channel_count] = ACTIONS(2109),
    [sym_set_virtual_port_count] = ACTIONS(2109),
    [sym__wc_encode_int] = ACTIONS(2109),
    [sym__wc_decode_int] = ACTIONS(2109),
    [sym__wc_int] = ACTIONS(2109),
    [sym__wc_to_ch_int] = ACTIONS(2109),
    [sym__ch_to_wc_int] = ACTIONS(2109),
    [sym__wc_find_string_int] = ACTIONS(2109),
    [sym__wc_left_string_int] = ACTIONS(2109),
    [sym__wc_lower_string_int] = ACTIONS(2109),
    [sym__wc_mid_string_int] = ACTIONS(2109),
    [sym__wc_remove_string_int] = ACTIONS(2109),
    [sym__wc_right_string_int] = ACTIONS(2109),
    [sym__wc_upper_string_int] = ACTIONS(2109),
    [sym__wc_compare_string_int] = ACTIONS(2109),
    [sym__wc_get_buffer_char_int] = ACTIONS(2109),
    [sym__wc_get_buffer_string_int] = ACTIONS(2109),
    [sym__wc_concat_string_int] = ACTIONS(2109),
    [sym__wc_file_open_int] = ACTIONS(2109),
    [sym__wc_file_close_int] = ACTIONS(2109),
    [sym__wc_file_read_int] = ACTIONS(2109),
    [sym__wc_file_read_line_int] = ACTIONS(2109),
    [sym__wc_file_write_int] = ACTIONS(2109),
    [sym__wc_file_write_line_int] = ACTIONS(2109),
    [aux_sym_preproc_include_token1] = ACTIONS(2109),
    [aux_sym_preproc_define_token1] = ACTIONS(2111),
    [aux_sym_preproc_warn_token1] = ACTIONS(2111),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2111),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2111),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2111),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2111),
    [aux_sym_preproc_else_token1] = ACTIONS(2111),
    [anon_sym_LBRACK] = ACTIONS(2111),
    [anon_sym_LBRACE] = ACTIONS(2111),
    [anon_sym_LPAREN] = ACTIONS(2109),
    [anon_sym_BANG] = ACTIONS(2111),
    [anon_sym_TILDE] = ACTIONS(2111),
    [anon_sym_DASH] = ACTIONS(2109),
    [anon_sym_PLUS] = ACTIONS(2109),
    [anon_sym_DASH_DASH] = ACTIONS(2111),
    [anon_sym_PLUS_PLUS] = ACTIONS(2111),
    [anon_sym_DQUOTE] = ACTIONS(2111),
    [anon_sym_SQUOTE] = ACTIONS(2111),
    [sym_number_literal] = ACTIONS(2111),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(374)] = {
    [sym_identifier] = ACTIONS(2113),
    [sym_program_name_keyword] = ACTIONS(2113),
    [sym_module_name_keyword] = ACTIONS(2113),
    [sym_define_device_keyword] = ACTIONS(2113),
    [sym_define_combine_keyword] = ACTIONS(2113),
    [sym_define_constant_keyword] = ACTIONS(2113),
    [sym_define_type_keyword] = ACTIONS(2113),
    [sym_define_variable_keyword] = ACTIONS(2113),
    [sym_define_system_variable_keyword] = ACTIONS(2113),
    [sym_define_connect_level_keyword] = ACTIONS(2113),
    [sym_define_latching_keyword] = ACTIONS(2113),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2113),
    [sym_define_toggling_keyword] = ACTIONS(2113),
    [sym_define_call_keyword] = ACTIONS(2113),
    [sym_define_function_keyword] = ACTIONS(2113),
    [sym_define_library_function_keyword] = ACTIONS(2113),
    [sym_define_start_keyword] = ACTIONS(2113),
    [sym_define_event_keyword] = ACTIONS(2113),
    [sym_define_program_keyword] = ACTIONS(2113),
    [sym_define_module_keyword] = ACTIONS(2113),
    [sym_char_keyword] = ACTIONS(2113),
    [sym_widechar_keyword] = ACTIONS(2113),
    [sym_integer_keyword] = ACTIONS(2113),
    [sym_sinteger_keyword] = ACTIONS(2113),
    [sym_long_keyword] = ACTIONS(2113),
    [sym_slong_keyword] = ACTIONS(2113),
    [sym_float_keyword] = ACTIONS(2113),
    [sym_double_keyword] = ACTIONS(2113),
    [sym_variant_keyword] = ACTIONS(2113),
    [sym_variantarray_keyword] = ACTIONS(2113),
    [sym_local_var_keyword] = ACTIONS(2113),
    [sym_stack_var_keyword] = ACTIONS(2113),
    [sym_constant_keyword] = ACTIONS(2113),
    [sym_volatile_keyword] = ACTIONS(2113),
    [sym_non_volatile_keyword] = ACTIONS(2113),
    [sym_persistent_keyword] = ACTIONS(2113),
    [sym_dev_keyword] = ACTIONS(2113),
    [sym_devlev_keyword] = ACTIONS(2113),
    [sym_devchan_keyword] = ACTIONS(2113),
    [sym_if_keyword] = ACTIONS(2113),
    [sym_switch_keyword] = ACTIONS(2113),
    [sym_case_keyword] = ACTIONS(2113),
    [sym_default_keyword] = ACTIONS(2113),
    [sym_while_keyword] = ACTIONS(2113),
    [sym_for_keyword] = ACTIONS(2113),
    [sym_break_keyword] = ACTIONS(2113),
    [sym_continue_keyword] = ACTIONS(2113),
    [sym_return_keyword] = ACTIONS(2113),
    [sym_select_keyword] = ACTIONS(2113),
    [sym_active_keyword] = ACTIONS(2113),
    [sym_struct_keyword] = ACTIONS(2113),
    [sym_structure_keyword] = ACTIONS(2113),
    [sym_button_event_keyword] = ACTIONS(2113),
    [sym_channel_event_keyword] = ACTIONS(2113),
    [sym_level_event_keyword] = ACTIONS(2113),
    [sym_data_event_keyword] = ACTIONS(2113),
    [sym_timeline_event_keyword] = ACTIONS(2113),
    [sym_custom_event_keyword] = ACTIONS(2113),
    [sym_push_keyword] = ACTIONS(2113),
    [sym_release_keyword] = ACTIONS(2113),
    [aux_sym_on_keyword_token1] = ACTIONS(2113),
    [aux_sym_off_keyword_token1] = ACTIONS(2113),
    [sym_send_level_keyword] = ACTIONS(2113),
    [sym_send_string_keyword] = ACTIONS(2113),
    [sym_send_command_keyword] = ACTIONS(2113),
    [sym_clear_buffer_keyword] = ACTIONS(2113),
    [sym_create_buffer_keyword] = ACTIONS(2113),
    [sym_create_multi_buffer_keyword] = ACTIONS(2113),
    [sym_call_keyword] = ACTIONS(2113),
    [sym_system_call_keyword] = ACTIONS(2113),
    [sym_devchan_to_keyword] = ACTIONS(2113),
    [sym_devchan_min_to_keyword] = ACTIONS(2113),
    [sym_devchan_total_off_keyword] = ACTIONS(2113),
    [sym_devchan_pulse_keyword] = ACTIONS(2113),
    [sym_wait_keyword] = ACTIONS(2113),
    [sym_cancel_wait_keyword] = ACTIONS(2113),
    [sym_wait_until_keyword] = ACTIONS(2113),
    [sym_cancel_wait_until_keyword] = ACTIONS(2113),
    [sym_cancel_all_wait_keyword] = ACTIONS(2113),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2113),
    [sym_not] = ACTIONS(2113),
    [sym_bnot] = ACTIONS(2113),
    [sym___file__] = ACTIONS(2113),
    [sym___date__] = ACTIONS(2113),
    [sym___ldate__] = ACTIONS(2113),
    [sym___time__] = ACTIONS(2113),
    [sym___line__] = ACTIONS(2113),
    [sym___name__] = ACTIONS(2113),
    [sym___netlinx__] = ACTIONS(2113),
    [sym_netlinx_axi_version] = ACTIONS(2113),
    [sym_url_flg_tcp] = ACTIONS(2113),
    [sym_url_flg_accinfopresent] = ACTIONS(2113),
    [sym_url_flg_temp] = ACTIONS(2113),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2113),
    [sym_url_flg_stat_mask] = ACTIONS(2113),
    [sym_url_flg_stat_lookup] = ACTIONS(2113),
    [sym_url_flg_stat_connecting] = ACTIONS(2113),
    [sym_url_flg_stat_waiting] = ACTIONS(2113),
    [sym_url_flg_stat_connected] = ACTIONS(2113),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2113),
    [sym_first_virtual_device] = ACTIONS(2113),
    [sym_first_local_port] = ACTIONS(2113),
    [sym_dynamic_virtual_device] = ACTIONS(2113),
    [sym_timeline_once] = ACTIONS(2113),
    [sym_timeline_repeat] = ACTIONS(2113),
    [sym_timeline_absolute] = ACTIONS(2113),
    [sym_timeline_relative] = ACTIONS(2113),
    [sym_do_push_timed_infinite] = ACTIONS(2113),
    [sym_source_type_no_address] = ACTIONS(2113),
    [sym_source_type_neuron_id] = ACTIONS(2113),
    [sym_source_type_ip_address] = ACTIONS(2113),
    [sym_source_type_axlink] = ACTIONS(2113),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2113),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2113),
    [sym_source_type_ip_socket_address] = ACTIONS(2113),
    [sym_source_type_rs232] = ACTIONS(2113),
    [sym_source_type_internal] = ACTIONS(2113),
    [sym_source_type_mac_address] = ACTIONS(2113),
    [sym_source_type_ipv4_port] = ACTIONS(2113),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2113),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2113),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2113),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2113),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2113),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2113),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2113),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2113),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2113),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2113),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2113),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2113),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2113),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2113),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2113),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2113),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2113),
    [sym_true] = ACTIONS(2113),
    [sym_false] = ACTIONS(2113),
    [sym_file_read_only] = ACTIONS(2113),
    [sym_file_rw_new] = ACTIONS(2113),
    [sym_file_rw_append] = ACTIONS(2113),
    [sym_ip_tcp] = ACTIONS(2113),
    [sym_ip_udp] = ACTIONS(2113),
    [sym_ip_udp_2way] = ACTIONS(2113),
    [sym_xml_encode_types] = ACTIONS(2113),
    [sym_xml_encode_char_as_list] = ACTIONS(2113),
    [sym_xml_encode_le] = ACTIONS(2113),
    [sym_xml_decode_types] = ACTIONS(2113),
    [sym_xml_decode_no_preserve] = ACTIONS(2113),
    [sym_duet_dev_type_display_device] = ACTIONS(2113),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2113),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2113),
    [sym_duet_dev_type_io_device] = ACTIONS(2113),
    [sym_duet_dev_type_relay_device] = ACTIONS(2113),
    [sym_duet_dev_type_ups] = ACTIONS(2113),
    [sym_duet_dev_type_amplifier] = ACTIONS(2113),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2113),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2113),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2113),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2113),
    [sym_duet_dev_type_camera] = ACTIONS(2113),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2113),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2113),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2113),
    [sym_duet_dev_type_dss] = ACTIONS(2113),
    [sym_duet_dev_type_dvr] = ACTIONS(2113),
    [sym_duet_dev_type_disc_device] = ACTIONS(2113),
    [sym_duet_dev_type_document_camera] = ACTIONS(2113),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2113),
    [sym_duet_dev_type_hvac] = ACTIONS(2113),
    [sym_duet_dev_type_keypad] = ACTIONS(2113),
    [sym_duet_dev_type_light] = ACTIONS(2113),
    [sym_duet_dev_type_monitor] = ACTIONS(2113),
    [sym_duet_dev_type_motor] = ACTIONS(2113),
    [sym_duet_dev_type_multi_window] = ACTIONS(2113),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2113),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2113),
    [sym_duet_dev_type_receiver] = ACTIONS(2113),
    [sym_duet_dev_type_security_system] = ACTIONS(2113),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2113),
    [sym_duet_dev_type_settop_box] = ACTIONS(2113),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2113),
    [sym_duet_dev_type_switcher] = ACTIONS(2113),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2113),
    [sym_duet_dev_type_tv] = ACTIONS(2113),
    [sym_duet_dev_type_utility] = ACTIONS(2113),
    [sym_duet_dev_type_vcr] = ACTIONS(2113),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2113),
    [sym_duet_dev_type_video_processor] = ACTIONS(2113),
    [sym_duet_dev_type_video_projector] = ACTIONS(2113),
    [sym_duet_dev_type_video_wall] = ACTIONS(2113),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2113),
    [sym_duet_dev_type_weather] = ACTIONS(2113),
    [sym_duet_dev_not_polled] = ACTIONS(2113),
    [sym_duet_dev_polled] = ACTIONS(2113),
    [sym_clkmgr_mode_network] = ACTIONS(2113),
    [sym_clkmgr_mode_standalone] = ACTIONS(2113),
    [sym_normal_standby] = ACTIONS(2113),
    [sym_normal_wake] = ACTIONS(2113),
    [sym_ip_multicast_ttl_option] = ACTIONS(2113),
    [sym_ip_ttl_subnet] = ACTIONS(2113),
    [sym_ip_ttl_site] = ACTIONS(2113),
    [sym_ip_ttl_region] = ACTIONS(2113),
    [sym_ip_ttl_continent] = ACTIONS(2113),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2113),
    [sym_ip_nodelay_on] = ACTIONS(2113),
    [sym_ip_nodelay_off] = ACTIONS(2113),
    [sym_amx_error] = ACTIONS(2113),
    [sym_amx_warning] = ACTIONS(2113),
    [sym_amx_info] = ACTIONS(2113),
    [sym_amx_debug] = ACTIONS(2113),
    [sym_netlinx_login_success] = ACTIONS(2113),
    [sym_netlinx_login_fail] = ACTIONS(2113),
    [sym_netlinx_logout] = ACTIONS(2113),
    [sym_null_str] = ACTIONS(2113),
    [sym_smtp_address] = ACTIONS(2113),
    [sym_smtp_port_number] = ACTIONS(2113),
    [sym_smtp_username] = ACTIONS(2113),
    [sym_smtp_password] = ACTIONS(2113),
    [sym_smtp_require_tls] = ACTIONS(2113),
    [sym_smtp_from] = ACTIONS(2113),
    [sym_smtp_tls_true] = ACTIONS(2113),
    [sym_smtp_tls_false] = ACTIONS(2113),
    [sym_listview_on_row_select_event] = ACTIONS(2113),
    [sym_data_structure_unknown] = ACTIONS(2113),
    [sym_data_structure_datafeed] = ACTIONS(2113),
    [sym_data_structure_datarecordset] = ACTIONS(2113),
    [sym_data_structure_datarecord] = ACTIONS(2113),
    [sym_data_structure_datafield] = ACTIONS(2113),
    [sym_data_max_content_field_count] = ACTIONS(2113),
    [sym_data_max_metadata_field_count] = ACTIONS(2113),
    [sym_data_max_id_length] = ACTIONS(2113),
    [sym_data_max_name_length] = ACTIONS(2113),
    [sym_data_max_description_length] = ACTIONS(2113),
    [sym_data_max_type_length] = ACTIONS(2113),
    [sym_data_max_format_length] = ACTIONS(2113),
    [sym_data_max_source_length] = ACTIONS(2113),
    [sym_data_max_value_length] = ACTIONS(2113),
    [sym_data_max_label_length] = ACTIONS(2113),
    [sym_data_max_url_length] = ACTIONS(2113),
    [sym_max_last_login_info_length] = ACTIONS(2113),
    [sym_data_type_unknown] = ACTIONS(2113),
    [sym_data_type_string] = ACTIONS(2113),
    [sym_data_type_datetime] = ACTIONS(2113),
    [sym_data_type_date] = ACTIONS(2113),
    [sym_data_type_time] = ACTIONS(2113),
    [sym_data_type_image] = ACTIONS(2113),
    [sym_data_format_url] = ACTIONS(2113),
    [sym_data_format_phone] = ACTIONS(2113),
    [sym_data_format_email] = ACTIONS(2113),
    [sym_data_format_iso8601] = ACTIONS(2113),
    [sym_auditlog_priv] = ACTIONS(2113),
    [sym_configuration_priv] = ACTIONS(2113),
    [sym_deviceconfig_priv] = ACTIONS(2113),
    [sym_ftp_priv] = ACTIONS(2113),
    [sym_http_priv] = ACTIONS(2113),
    [sym_networkconfig_priv] = ACTIONS(2113),
    [sym_programport_priv] = ACTIONS(2113),
    [sym_remoteui_priv] = ACTIONS(2113),
    [sym_securitycontrol_priv] = ACTIONS(2113),
    [sym_softwaremanagement_priv] = ACTIONS(2113),
    [sym_terminal_priv] = ACTIONS(2113),
    [sym_tpadmin_priv] = ACTIONS(2113),
    [sym_user1_priv] = ACTIONS(2113),
    [sym_user2_priv] = ACTIONS(2113),
    [sym_user3_priv] = ACTIONS(2113),
    [sym_user4_priv] = ACTIONS(2113),
    [sym_usermanagement_priv] = ACTIONS(2113),
    [sym_valid_account] = ACTIONS(2113),
    [sym_err_name_invalid] = ACTIONS(2113),
    [sym_err_password_invalid] = ACTIONS(2113),
    [sym_err_invalid_account] = ACTIONS(2113),
    [sym_err_no_authorization] = ACTIONS(2113),
    [sym_err_invalid_parameter] = ACTIONS(2113),
    [sym_err_user_locked_out] = ACTIONS(2113),
    [sym_err_user_account_expired] = ACTIONS(2113),
    [sym_err_general_error] = ACTIONS(2113),
    [sym_err_authentication_error] = ACTIONS(2113),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2113),
    [sym_tls_validate_certificate] = ACTIONS(2113),
    [sym_tdata] = ACTIONS(2113),
    [sym_tchannel] = ACTIONS(2113),
    [sym_tlevel] = ACTIONS(2113),
    [sym_tbutton] = ACTIONS(2113),
    [sym_ttimeline] = ACTIONS(2113),
    [sym_tcustom] = ACTIONS(2113),
    [sym_url_struct] = ACTIONS(2113),
    [sym_dns_struct] = ACTIONS(2113),
    [sym_ip_address_struct] = ACTIONS(2113),
    [sym_dev_info_struct] = ACTIONS(2113),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2113),
    [sym_data_feed] = ACTIONS(2113),
    [sym_data_field] = ACTIONS(2113),
    [sym_data_record] = ACTIONS(2113),
    [sym_wc_data_feed] = ACTIONS(2113),
    [sym_wc_data_field] = ACTIONS(2113),
    [sym_wc_data_record] = ACTIONS(2113),
    [sym_last_login_info] = ACTIONS(2113),
    [sym____reserved___] = ACTIONS(2113),
    [sym_date] = ACTIONS(2113),
    [sym_day] = ACTIONS(2113),
    [sym_time] = ACTIONS(2113),
    [sym_ldate] = ACTIONS(2113),
    [sym_push_device] = ACTIONS(2113),
    [sym_push_channel] = ACTIONS(2113),
    [sym_push_devchan] = ACTIONS(2113),
    [sym_release_device] = ACTIONS(2113),
    [sym_release_channel] = ACTIONS(2113),
    [sym_release_devchan] = ACTIONS(2113),
    [sym_master_slot] = ACTIONS(2113),
    [sym_get_pulse_time] = ACTIONS(2113),
    [sym_get_timer] = ACTIONS(2113),
    [sym_master_sn] = ACTIONS(2113),
    [sym_system_number] = ACTIONS(2113),
    [sym_button] = ACTIONS(2113),
    [sym_dv_channel] = ACTIONS(2113),
    [sym_data] = ACTIONS(2113),
    [sym_level] = ACTIONS(2113),
    [sym_timeline] = ACTIONS(2113),
    [sym_channel] = ACTIONS(2113),
    [sym_custom] = ACTIONS(2113),
    [sym_atoi] = ACTIONS(2113),
    [sym_atol] = ACTIONS(2113),
    [sym_atof] = ACTIONS(2113),
    [sym_itoa] = ACTIONS(2113),
    [sym_itohex] = ACTIONS(2113),
    [sym_hextoi] = ACTIONS(2113),
    [sym_ftoa] = ACTIONS(2113),
    [sym_find_string] = ACTIONS(2113),
    [sym_left_string] = ACTIONS(2113),
    [sym_right_string] = ACTIONS(2113),
    [sym_mid_string] = ACTIONS(2113),
    [sym_remove_string] = ACTIONS(2113),
    [sym_get_buffer_string] = ACTIONS(2113),
    [sym_get_buffer_char] = ACTIONS(2113),
    [sym_get_multi_buffer_string] = ACTIONS(2113),
    [sym_length_string] = ACTIONS(2113),
    [sym_max_length_string] = ACTIONS(2113),
    [sym_set_length_string] = ACTIONS(2113),
    [sym_compare_string] = ACTIONS(2113),
    [sym_upper_string] = ACTIONS(2113),
    [sym_lower_string] = ACTIONS(2113),
    [sym_get_last] = ACTIONS(2113),
    [sym_date_to_day] = ACTIONS(2113),
    [sym_date_to_month] = ACTIONS(2113),
    [sym_date_to_year] = ACTIONS(2113),
    [sym_time_to_hour] = ACTIONS(2113),
    [sym_time_to_minute] = ACTIONS(2113),
    [sym_time_to_second] = ACTIONS(2113),
    [sym_day_of_week] = ACTIONS(2113),
    [sym_set_pulse_time] = ACTIONS(2113),
    [sym_set_timer] = ACTIONS(2113),
    [sym_file_open] = ACTIONS(2113),
    [sym_file_close] = ACTIONS(2113),
    [sym_file_read] = ACTIONS(2113),
    [sym_file_read_line] = ACTIONS(2113),
    [sym_file_write] = ACTIONS(2113),
    [sym_file_write_line] = ACTIONS(2113),
    [sym_file_copy] = ACTIONS(2113),
    [sym_file_delete] = ACTIONS(2113),
    [sym_file_rename] = ACTIONS(2113),
    [sym_file_dir] = ACTIONS(2113),
    [sym_file_getdir] = ACTIONS(2113),
    [sym_file_setdir] = ACTIONS(2113),
    [sym_file_create_dir] = ACTIONS(2113),
    [sym_file_remove_dir] = ACTIONS(2113),
    [sym_file_seek] = ACTIONS(2113),
    [sym_length_array] = ACTIONS(2113),
    [sym_max_length_array] = ACTIONS(2113),
    [sym_set_length_array] = ACTIONS(2113),
    [sym_ip_client_open] = ACTIONS(2113),
    [sym_ip_client_close] = ACTIONS(2113),
    [sym_ip_server_open] = ACTIONS(2113),
    [sym_ip_server_close] = ACTIONS(2113),
    [sym_ip_bound_client_open] = ACTIONS(2113),
    [sym_ip_mc_server_open] = ACTIONS(2113),
    [sym_ip_set_option] = ACTIONS(2113),
    [sym_ssh_client_open] = ACTIONS(2113),
    [sym_ssh_client_close] = ACTIONS(2113),
    [sym_tls_client_open] = ACTIONS(2113),
    [sym_tls_client_close] = ACTIONS(2113),
    [sym_do_push] = ACTIONS(2113),
    [sym_do_push_timed] = ACTIONS(2113),
    [sym_do_release] = ACTIONS(2113),
    [sym_device_id] = ACTIONS(2113),
    [sym_device_info] = ACTIONS(2113),
    [sym_device_id_string] = ACTIONS(2113),
    [sym_device_standby] = ACTIONS(2113),
    [sym_device_wake] = ACTIONS(2113),
    [sym_timeline_create] = ACTIONS(2113),
    [sym_timeline_kill] = ACTIONS(2113),
    [sym_timeline_reload] = ACTIONS(2113),
    [sym_timeline_pause] = ACTIONS(2113),
    [sym_timeline_restart] = ACTIONS(2113),
    [sym_timeline_set] = ACTIONS(2113),
    [sym_timeline_get] = ACTIONS(2113),
    [sym_timeline_active] = ACTIONS(2113),
    [sym_timeline_dynamic_id] = ACTIONS(2113),
    [sym_variable_to_string] = ACTIONS(2113),
    [sym_string_to_variable] = ACTIONS(2113),
    [sym_length_variable_to_string] = ACTIONS(2113),
    [sym_type_cast] = ACTIONS(2113),
    [sym_variable_to_xml] = ACTIONS(2113),
    [sym_xml_to_variable] = ACTIONS(2113),
    [sym_length_variable_to_xml] = ACTIONS(2113),
    [sym_redirect_string] = ACTIONS(2113),
    [sym_random_number] = ACTIONS(2113),
    [sym_get_unique_id] = ACTIONS(2113),
    [sym_get_system_number] = ACTIONS(2113),
    [sym_set_system_number] = ACTIONS(2113),
    [sym_get_serial_number] = ACTIONS(2113),
    [sym_get_master_build] = ACTIONS(2113),
    [sym_set_validation_code] = ACTIONS(2113),
    [sym_reboot] = ACTIONS(2113),
    [sym_load_duet_module] = ACTIONS(2113),
    [sym_unload_duet_module] = ACTIONS(2113),
    [sym_duet_mem_size_set] = ACTIONS(2113),
    [sym_duet_mem_size_get] = ACTIONS(2113),
    [sym_dynamic_application_device] = ACTIONS(2113),
    [sym_dynamic_polled_port] = ACTIONS(2113),
    [sym_static_port_binding] = ACTIONS(2113),
    [sym_static_ip_binding] = ACTIONS(2113),
    [sym_get_dns_list] = ACTIONS(2113),
    [sym_set_dns_list] = ACTIONS(2113),
    [sym_get_ip_address] = ACTIONS(2113),
    [sym_set_ip_address] = ACTIONS(2113),
    [sym_get_url_list] = ACTIONS(2113),
    [sym_add_url_entry] = ACTIONS(2113),
    [sym_delete_url_entry] = ACTIONS(2113),
    [sym_abs_value] = ACTIONS(2113),
    [sym_max_value] = ACTIONS(2113),
    [sym_min_value] = ACTIONS(2113),
    [sym_exp_value] = ACTIONS(2113),
    [sym_log_value] = ACTIONS(2113),
    [sym_log10_value] = ACTIONS(2113),
    [sym_power_value] = ACTIONS(2113),
    [sym_sqrt_value] = ACTIONS(2113),
    [sym_raw_be] = ACTIONS(2113),
    [sym_raw_le] = ACTIONS(2113),
    [sym_internal_queue_size_set] = ACTIONS(2113),
    [sym_internal_queue_size_get] = ACTIONS(2113),
    [sym_internal_threshold_set] = ACTIONS(2113),
    [sym_internal_threshold_get] = ACTIONS(2113),
    [sym_rebuilt_event] = ACTIONS(2113),
    [sym_do_custom_event] = ACTIONS(2113),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2113),
    [sym_clkmgr_set_clk_source] = ACTIONS(2113),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2113),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2113),
    [sym_clkmgr_get_timezone] = ACTIONS(2113),
    [sym_clkmgr_set_timezone] = ACTIONS(2113),
    [sym_clkmgr_get_resync_period] = ACTIONS(2113),
    [sym_clkmgr_set_resync_period] = ACTIONS(2113),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2113),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2113),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2113),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2113),
    [sym_clkmgr_get_timeservers] = ACTIONS(2113),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2113),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2113),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2113),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2113),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2113),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2113),
    [sym_set_log_level] = ACTIONS(2113),
    [sym_get_log_level] = ACTIONS(2113),
    [sym_amx_log] = ACTIONS(2113),
    [sym_smtp_server_config_set] = ACTIONS(2113),
    [sym_smtp_server_config_get] = ACTIONS(2113),
    [sym_smtp_send] = ACTIONS(2113),
    [sym_validate_netlinx_account] = ACTIONS(2113),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2113),
    [sym_authenticate_certificate] = ACTIONS(2113),
    [sym_audit_netlinx_session_event] = ACTIONS(2113),
    [sym_audit_netlinx_generic_event] = ACTIONS(2113),
    [sym_get_available_flash_disk_space] = ACTIONS(2113),
    [sym_get_max_flash_disk_space] = ACTIONS(2113),
    [sym_data_create_feed] = ACTIONS(2113),
    [sym_data_delete_feed] = ACTIONS(2113),
    [sym_data_publish_feed] = ACTIONS(2113),
    [sym_data_get_published_feed] = ACTIONS(2113),
    [sym_data_add_record] = ACTIONS(2113),
    [sym_data_get_event_record] = ACTIONS(2113),
    [sym__wc_data_create_feed] = ACTIONS(2113),
    [sym__wc_data_add_record] = ACTIONS(2113),
    [sym__wc_data_get_event_record] = ACTIONS(2113),
    [sym_format] = ACTIONS(2113),
    [sym_astro_clock] = ACTIONS(2113),
    [sym_set_outdoor_temperature] = ACTIONS(2113),
    [sym_set_virtual_level_count] = ACTIONS(2113),
    [sym_set_virtual_channel_count] = ACTIONS(2113),
    [sym_set_virtual_port_count] = ACTIONS(2113),
    [sym__wc_encode_int] = ACTIONS(2113),
    [sym__wc_decode_int] = ACTIONS(2113),
    [sym__wc_int] = ACTIONS(2113),
    [sym__wc_to_ch_int] = ACTIONS(2113),
    [sym__ch_to_wc_int] = ACTIONS(2113),
    [sym__wc_find_string_int] = ACTIONS(2113),
    [sym__wc_left_string_int] = ACTIONS(2113),
    [sym__wc_lower_string_int] = ACTIONS(2113),
    [sym__wc_mid_string_int] = ACTIONS(2113),
    [sym__wc_remove_string_int] = ACTIONS(2113),
    [sym__wc_right_string_int] = ACTIONS(2113),
    [sym__wc_upper_string_int] = ACTIONS(2113),
    [sym__wc_compare_string_int] = ACTIONS(2113),
    [sym__wc_get_buffer_char_int] = ACTIONS(2113),
    [sym__wc_get_buffer_string_int] = ACTIONS(2113),
    [sym__wc_concat_string_int] = ACTIONS(2113),
    [sym__wc_file_open_int] = ACTIONS(2113),
    [sym__wc_file_close_int] = ACTIONS(2113),
    [sym__wc_file_read_int] = ACTIONS(2113),
    [sym__wc_file_read_line_int] = ACTIONS(2113),
    [sym__wc_file_write_int] = ACTIONS(2113),
    [sym__wc_file_write_line_int] = ACTIONS(2113),
    [aux_sym_preproc_include_token1] = ACTIONS(2113),
    [aux_sym_preproc_define_token1] = ACTIONS(2115),
    [aux_sym_preproc_warn_token1] = ACTIONS(2115),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2115),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2115),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2115),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2115),
    [aux_sym_preproc_else_token1] = ACTIONS(2115),
    [anon_sym_LBRACK] = ACTIONS(2115),
    [anon_sym_LBRACE] = ACTIONS(2115),
    [anon_sym_LPAREN] = ACTIONS(2113),
    [anon_sym_BANG] = ACTIONS(2115),
    [anon_sym_TILDE] = ACTIONS(2115),
    [anon_sym_DASH] = ACTIONS(2113),
    [anon_sym_PLUS] = ACTIONS(2113),
    [anon_sym_DASH_DASH] = ACTIONS(2115),
    [anon_sym_PLUS_PLUS] = ACTIONS(2115),
    [anon_sym_DQUOTE] = ACTIONS(2115),
    [anon_sym_SQUOTE] = ACTIONS(2115),
    [sym_number_literal] = ACTIONS(2115),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(375)] = {
    [sym_identifier] = ACTIONS(2117),
    [sym_program_name_keyword] = ACTIONS(2117),
    [sym_module_name_keyword] = ACTIONS(2117),
    [sym_define_device_keyword] = ACTIONS(2117),
    [sym_define_combine_keyword] = ACTIONS(2117),
    [sym_define_constant_keyword] = ACTIONS(2117),
    [sym_define_type_keyword] = ACTIONS(2117),
    [sym_define_variable_keyword] = ACTIONS(2117),
    [sym_define_system_variable_keyword] = ACTIONS(2117),
    [sym_define_connect_level_keyword] = ACTIONS(2117),
    [sym_define_latching_keyword] = ACTIONS(2117),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2117),
    [sym_define_toggling_keyword] = ACTIONS(2117),
    [sym_define_call_keyword] = ACTIONS(2117),
    [sym_define_function_keyword] = ACTIONS(2117),
    [sym_define_library_function_keyword] = ACTIONS(2117),
    [sym_define_start_keyword] = ACTIONS(2117),
    [sym_define_event_keyword] = ACTIONS(2117),
    [sym_define_program_keyword] = ACTIONS(2117),
    [sym_define_module_keyword] = ACTIONS(2117),
    [sym_char_keyword] = ACTIONS(2117),
    [sym_widechar_keyword] = ACTIONS(2117),
    [sym_integer_keyword] = ACTIONS(2117),
    [sym_sinteger_keyword] = ACTIONS(2117),
    [sym_long_keyword] = ACTIONS(2117),
    [sym_slong_keyword] = ACTIONS(2117),
    [sym_float_keyword] = ACTIONS(2117),
    [sym_double_keyword] = ACTIONS(2117),
    [sym_variant_keyword] = ACTIONS(2117),
    [sym_variantarray_keyword] = ACTIONS(2117),
    [sym_local_var_keyword] = ACTIONS(2117),
    [sym_stack_var_keyword] = ACTIONS(2117),
    [sym_constant_keyword] = ACTIONS(2117),
    [sym_volatile_keyword] = ACTIONS(2117),
    [sym_non_volatile_keyword] = ACTIONS(2117),
    [sym_persistent_keyword] = ACTIONS(2117),
    [sym_dev_keyword] = ACTIONS(2117),
    [sym_devlev_keyword] = ACTIONS(2117),
    [sym_devchan_keyword] = ACTIONS(2117),
    [sym_if_keyword] = ACTIONS(2117),
    [sym_switch_keyword] = ACTIONS(2117),
    [sym_case_keyword] = ACTIONS(2117),
    [sym_default_keyword] = ACTIONS(2117),
    [sym_while_keyword] = ACTIONS(2117),
    [sym_for_keyword] = ACTIONS(2117),
    [sym_break_keyword] = ACTIONS(2117),
    [sym_continue_keyword] = ACTIONS(2117),
    [sym_return_keyword] = ACTIONS(2117),
    [sym_select_keyword] = ACTIONS(2117),
    [sym_active_keyword] = ACTIONS(2117),
    [sym_struct_keyword] = ACTIONS(2117),
    [sym_structure_keyword] = ACTIONS(2117),
    [sym_button_event_keyword] = ACTIONS(2117),
    [sym_channel_event_keyword] = ACTIONS(2117),
    [sym_level_event_keyword] = ACTIONS(2117),
    [sym_data_event_keyword] = ACTIONS(2117),
    [sym_timeline_event_keyword] = ACTIONS(2117),
    [sym_custom_event_keyword] = ACTIONS(2117),
    [sym_push_keyword] = ACTIONS(2117),
    [sym_release_keyword] = ACTIONS(2117),
    [aux_sym_on_keyword_token1] = ACTIONS(2117),
    [aux_sym_off_keyword_token1] = ACTIONS(2117),
    [sym_send_level_keyword] = ACTIONS(2117),
    [sym_send_string_keyword] = ACTIONS(2117),
    [sym_send_command_keyword] = ACTIONS(2117),
    [sym_clear_buffer_keyword] = ACTIONS(2117),
    [sym_create_buffer_keyword] = ACTIONS(2117),
    [sym_create_multi_buffer_keyword] = ACTIONS(2117),
    [sym_call_keyword] = ACTIONS(2117),
    [sym_system_call_keyword] = ACTIONS(2117),
    [sym_devchan_to_keyword] = ACTIONS(2117),
    [sym_devchan_min_to_keyword] = ACTIONS(2117),
    [sym_devchan_total_off_keyword] = ACTIONS(2117),
    [sym_devchan_pulse_keyword] = ACTIONS(2117),
    [sym_wait_keyword] = ACTIONS(2117),
    [sym_cancel_wait_keyword] = ACTIONS(2117),
    [sym_wait_until_keyword] = ACTIONS(2117),
    [sym_cancel_wait_until_keyword] = ACTIONS(2117),
    [sym_cancel_all_wait_keyword] = ACTIONS(2117),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2117),
    [sym_not] = ACTIONS(2117),
    [sym_bnot] = ACTIONS(2117),
    [sym___file__] = ACTIONS(2117),
    [sym___date__] = ACTIONS(2117),
    [sym___ldate__] = ACTIONS(2117),
    [sym___time__] = ACTIONS(2117),
    [sym___line__] = ACTIONS(2117),
    [sym___name__] = ACTIONS(2117),
    [sym___netlinx__] = ACTIONS(2117),
    [sym_netlinx_axi_version] = ACTIONS(2117),
    [sym_url_flg_tcp] = ACTIONS(2117),
    [sym_url_flg_accinfopresent] = ACTIONS(2117),
    [sym_url_flg_temp] = ACTIONS(2117),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2117),
    [sym_url_flg_stat_mask] = ACTIONS(2117),
    [sym_url_flg_stat_lookup] = ACTIONS(2117),
    [sym_url_flg_stat_connecting] = ACTIONS(2117),
    [sym_url_flg_stat_waiting] = ACTIONS(2117),
    [sym_url_flg_stat_connected] = ACTIONS(2117),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2117),
    [sym_first_virtual_device] = ACTIONS(2117),
    [sym_first_local_port] = ACTIONS(2117),
    [sym_dynamic_virtual_device] = ACTIONS(2117),
    [sym_timeline_once] = ACTIONS(2117),
    [sym_timeline_repeat] = ACTIONS(2117),
    [sym_timeline_absolute] = ACTIONS(2117),
    [sym_timeline_relative] = ACTIONS(2117),
    [sym_do_push_timed_infinite] = ACTIONS(2117),
    [sym_source_type_no_address] = ACTIONS(2117),
    [sym_source_type_neuron_id] = ACTIONS(2117),
    [sym_source_type_ip_address] = ACTIONS(2117),
    [sym_source_type_axlink] = ACTIONS(2117),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2117),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2117),
    [sym_source_type_ip_socket_address] = ACTIONS(2117),
    [sym_source_type_rs232] = ACTIONS(2117),
    [sym_source_type_internal] = ACTIONS(2117),
    [sym_source_type_mac_address] = ACTIONS(2117),
    [sym_source_type_ipv4_port] = ACTIONS(2117),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2117),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2117),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2117),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2117),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2117),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2117),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2117),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2117),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2117),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2117),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2117),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2117),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2117),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2117),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2117),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2117),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2117),
    [sym_true] = ACTIONS(2117),
    [sym_false] = ACTIONS(2117),
    [sym_file_read_only] = ACTIONS(2117),
    [sym_file_rw_new] = ACTIONS(2117),
    [sym_file_rw_append] = ACTIONS(2117),
    [sym_ip_tcp] = ACTIONS(2117),
    [sym_ip_udp] = ACTIONS(2117),
    [sym_ip_udp_2way] = ACTIONS(2117),
    [sym_xml_encode_types] = ACTIONS(2117),
    [sym_xml_encode_char_as_list] = ACTIONS(2117),
    [sym_xml_encode_le] = ACTIONS(2117),
    [sym_xml_decode_types] = ACTIONS(2117),
    [sym_xml_decode_no_preserve] = ACTIONS(2117),
    [sym_duet_dev_type_display_device] = ACTIONS(2117),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2117),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2117),
    [sym_duet_dev_type_io_device] = ACTIONS(2117),
    [sym_duet_dev_type_relay_device] = ACTIONS(2117),
    [sym_duet_dev_type_ups] = ACTIONS(2117),
    [sym_duet_dev_type_amplifier] = ACTIONS(2117),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2117),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2117),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2117),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2117),
    [sym_duet_dev_type_camera] = ACTIONS(2117),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2117),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2117),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2117),
    [sym_duet_dev_type_dss] = ACTIONS(2117),
    [sym_duet_dev_type_dvr] = ACTIONS(2117),
    [sym_duet_dev_type_disc_device] = ACTIONS(2117),
    [sym_duet_dev_type_document_camera] = ACTIONS(2117),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2117),
    [sym_duet_dev_type_hvac] = ACTIONS(2117),
    [sym_duet_dev_type_keypad] = ACTIONS(2117),
    [sym_duet_dev_type_light] = ACTIONS(2117),
    [sym_duet_dev_type_monitor] = ACTIONS(2117),
    [sym_duet_dev_type_motor] = ACTIONS(2117),
    [sym_duet_dev_type_multi_window] = ACTIONS(2117),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2117),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2117),
    [sym_duet_dev_type_receiver] = ACTIONS(2117),
    [sym_duet_dev_type_security_system] = ACTIONS(2117),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2117),
    [sym_duet_dev_type_settop_box] = ACTIONS(2117),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2117),
    [sym_duet_dev_type_switcher] = ACTIONS(2117),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2117),
    [sym_duet_dev_type_tv] = ACTIONS(2117),
    [sym_duet_dev_type_utility] = ACTIONS(2117),
    [sym_duet_dev_type_vcr] = ACTIONS(2117),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2117),
    [sym_duet_dev_type_video_processor] = ACTIONS(2117),
    [sym_duet_dev_type_video_projector] = ACTIONS(2117),
    [sym_duet_dev_type_video_wall] = ACTIONS(2117),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2117),
    [sym_duet_dev_type_weather] = ACTIONS(2117),
    [sym_duet_dev_not_polled] = ACTIONS(2117),
    [sym_duet_dev_polled] = ACTIONS(2117),
    [sym_clkmgr_mode_network] = ACTIONS(2117),
    [sym_clkmgr_mode_standalone] = ACTIONS(2117),
    [sym_normal_standby] = ACTIONS(2117),
    [sym_normal_wake] = ACTIONS(2117),
    [sym_ip_multicast_ttl_option] = ACTIONS(2117),
    [sym_ip_ttl_subnet] = ACTIONS(2117),
    [sym_ip_ttl_site] = ACTIONS(2117),
    [sym_ip_ttl_region] = ACTIONS(2117),
    [sym_ip_ttl_continent] = ACTIONS(2117),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2117),
    [sym_ip_nodelay_on] = ACTIONS(2117),
    [sym_ip_nodelay_off] = ACTIONS(2117),
    [sym_amx_error] = ACTIONS(2117),
    [sym_amx_warning] = ACTIONS(2117),
    [sym_amx_info] = ACTIONS(2117),
    [sym_amx_debug] = ACTIONS(2117),
    [sym_netlinx_login_success] = ACTIONS(2117),
    [sym_netlinx_login_fail] = ACTIONS(2117),
    [sym_netlinx_logout] = ACTIONS(2117),
    [sym_null_str] = ACTIONS(2117),
    [sym_smtp_address] = ACTIONS(2117),
    [sym_smtp_port_number] = ACTIONS(2117),
    [sym_smtp_username] = ACTIONS(2117),
    [sym_smtp_password] = ACTIONS(2117),
    [sym_smtp_require_tls] = ACTIONS(2117),
    [sym_smtp_from] = ACTIONS(2117),
    [sym_smtp_tls_true] = ACTIONS(2117),
    [sym_smtp_tls_false] = ACTIONS(2117),
    [sym_listview_on_row_select_event] = ACTIONS(2117),
    [sym_data_structure_unknown] = ACTIONS(2117),
    [sym_data_structure_datafeed] = ACTIONS(2117),
    [sym_data_structure_datarecordset] = ACTIONS(2117),
    [sym_data_structure_datarecord] = ACTIONS(2117),
    [sym_data_structure_datafield] = ACTIONS(2117),
    [sym_data_max_content_field_count] = ACTIONS(2117),
    [sym_data_max_metadata_field_count] = ACTIONS(2117),
    [sym_data_max_id_length] = ACTIONS(2117),
    [sym_data_max_name_length] = ACTIONS(2117),
    [sym_data_max_description_length] = ACTIONS(2117),
    [sym_data_max_type_length] = ACTIONS(2117),
    [sym_data_max_format_length] = ACTIONS(2117),
    [sym_data_max_source_length] = ACTIONS(2117),
    [sym_data_max_value_length] = ACTIONS(2117),
    [sym_data_max_label_length] = ACTIONS(2117),
    [sym_data_max_url_length] = ACTIONS(2117),
    [sym_max_last_login_info_length] = ACTIONS(2117),
    [sym_data_type_unknown] = ACTIONS(2117),
    [sym_data_type_string] = ACTIONS(2117),
    [sym_data_type_datetime] = ACTIONS(2117),
    [sym_data_type_date] = ACTIONS(2117),
    [sym_data_type_time] = ACTIONS(2117),
    [sym_data_type_image] = ACTIONS(2117),
    [sym_data_format_url] = ACTIONS(2117),
    [sym_data_format_phone] = ACTIONS(2117),
    [sym_data_format_email] = ACTIONS(2117),
    [sym_data_format_iso8601] = ACTIONS(2117),
    [sym_auditlog_priv] = ACTIONS(2117),
    [sym_configuration_priv] = ACTIONS(2117),
    [sym_deviceconfig_priv] = ACTIONS(2117),
    [sym_ftp_priv] = ACTIONS(2117),
    [sym_http_priv] = ACTIONS(2117),
    [sym_networkconfig_priv] = ACTIONS(2117),
    [sym_programport_priv] = ACTIONS(2117),
    [sym_remoteui_priv] = ACTIONS(2117),
    [sym_securitycontrol_priv] = ACTIONS(2117),
    [sym_softwaremanagement_priv] = ACTIONS(2117),
    [sym_terminal_priv] = ACTIONS(2117),
    [sym_tpadmin_priv] = ACTIONS(2117),
    [sym_user1_priv] = ACTIONS(2117),
    [sym_user2_priv] = ACTIONS(2117),
    [sym_user3_priv] = ACTIONS(2117),
    [sym_user4_priv] = ACTIONS(2117),
    [sym_usermanagement_priv] = ACTIONS(2117),
    [sym_valid_account] = ACTIONS(2117),
    [sym_err_name_invalid] = ACTIONS(2117),
    [sym_err_password_invalid] = ACTIONS(2117),
    [sym_err_invalid_account] = ACTIONS(2117),
    [sym_err_no_authorization] = ACTIONS(2117),
    [sym_err_invalid_parameter] = ACTIONS(2117),
    [sym_err_user_locked_out] = ACTIONS(2117),
    [sym_err_user_account_expired] = ACTIONS(2117),
    [sym_err_general_error] = ACTIONS(2117),
    [sym_err_authentication_error] = ACTIONS(2117),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2117),
    [sym_tls_validate_certificate] = ACTIONS(2117),
    [sym_tdata] = ACTIONS(2117),
    [sym_tchannel] = ACTIONS(2117),
    [sym_tlevel] = ACTIONS(2117),
    [sym_tbutton] = ACTIONS(2117),
    [sym_ttimeline] = ACTIONS(2117),
    [sym_tcustom] = ACTIONS(2117),
    [sym_url_struct] = ACTIONS(2117),
    [sym_dns_struct] = ACTIONS(2117),
    [sym_ip_address_struct] = ACTIONS(2117),
    [sym_dev_info_struct] = ACTIONS(2117),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2117),
    [sym_data_feed] = ACTIONS(2117),
    [sym_data_field] = ACTIONS(2117),
    [sym_data_record] = ACTIONS(2117),
    [sym_wc_data_feed] = ACTIONS(2117),
    [sym_wc_data_field] = ACTIONS(2117),
    [sym_wc_data_record] = ACTIONS(2117),
    [sym_last_login_info] = ACTIONS(2117),
    [sym____reserved___] = ACTIONS(2117),
    [sym_date] = ACTIONS(2117),
    [sym_day] = ACTIONS(2117),
    [sym_time] = ACTIONS(2117),
    [sym_ldate] = ACTIONS(2117),
    [sym_push_device] = ACTIONS(2117),
    [sym_push_channel] = ACTIONS(2117),
    [sym_push_devchan] = ACTIONS(2117),
    [sym_release_device] = ACTIONS(2117),
    [sym_release_channel] = ACTIONS(2117),
    [sym_release_devchan] = ACTIONS(2117),
    [sym_master_slot] = ACTIONS(2117),
    [sym_get_pulse_time] = ACTIONS(2117),
    [sym_get_timer] = ACTIONS(2117),
    [sym_master_sn] = ACTIONS(2117),
    [sym_system_number] = ACTIONS(2117),
    [sym_button] = ACTIONS(2117),
    [sym_dv_channel] = ACTIONS(2117),
    [sym_data] = ACTIONS(2117),
    [sym_level] = ACTIONS(2117),
    [sym_timeline] = ACTIONS(2117),
    [sym_channel] = ACTIONS(2117),
    [sym_custom] = ACTIONS(2117),
    [sym_atoi] = ACTIONS(2117),
    [sym_atol] = ACTIONS(2117),
    [sym_atof] = ACTIONS(2117),
    [sym_itoa] = ACTIONS(2117),
    [sym_itohex] = ACTIONS(2117),
    [sym_hextoi] = ACTIONS(2117),
    [sym_ftoa] = ACTIONS(2117),
    [sym_find_string] = ACTIONS(2117),
    [sym_left_string] = ACTIONS(2117),
    [sym_right_string] = ACTIONS(2117),
    [sym_mid_string] = ACTIONS(2117),
    [sym_remove_string] = ACTIONS(2117),
    [sym_get_buffer_string] = ACTIONS(2117),
    [sym_get_buffer_char] = ACTIONS(2117),
    [sym_get_multi_buffer_string] = ACTIONS(2117),
    [sym_length_string] = ACTIONS(2117),
    [sym_max_length_string] = ACTIONS(2117),
    [sym_set_length_string] = ACTIONS(2117),
    [sym_compare_string] = ACTIONS(2117),
    [sym_upper_string] = ACTIONS(2117),
    [sym_lower_string] = ACTIONS(2117),
    [sym_get_last] = ACTIONS(2117),
    [sym_date_to_day] = ACTIONS(2117),
    [sym_date_to_month] = ACTIONS(2117),
    [sym_date_to_year] = ACTIONS(2117),
    [sym_time_to_hour] = ACTIONS(2117),
    [sym_time_to_minute] = ACTIONS(2117),
    [sym_time_to_second] = ACTIONS(2117),
    [sym_day_of_week] = ACTIONS(2117),
    [sym_set_pulse_time] = ACTIONS(2117),
    [sym_set_timer] = ACTIONS(2117),
    [sym_file_open] = ACTIONS(2117),
    [sym_file_close] = ACTIONS(2117),
    [sym_file_read] = ACTIONS(2117),
    [sym_file_read_line] = ACTIONS(2117),
    [sym_file_write] = ACTIONS(2117),
    [sym_file_write_line] = ACTIONS(2117),
    [sym_file_copy] = ACTIONS(2117),
    [sym_file_delete] = ACTIONS(2117),
    [sym_file_rename] = ACTIONS(2117),
    [sym_file_dir] = ACTIONS(2117),
    [sym_file_getdir] = ACTIONS(2117),
    [sym_file_setdir] = ACTIONS(2117),
    [sym_file_create_dir] = ACTIONS(2117),
    [sym_file_remove_dir] = ACTIONS(2117),
    [sym_file_seek] = ACTIONS(2117),
    [sym_length_array] = ACTIONS(2117),
    [sym_max_length_array] = ACTIONS(2117),
    [sym_set_length_array] = ACTIONS(2117),
    [sym_ip_client_open] = ACTIONS(2117),
    [sym_ip_client_close] = ACTIONS(2117),
    [sym_ip_server_open] = ACTIONS(2117),
    [sym_ip_server_close] = ACTIONS(2117),
    [sym_ip_bound_client_open] = ACTIONS(2117),
    [sym_ip_mc_server_open] = ACTIONS(2117),
    [sym_ip_set_option] = ACTIONS(2117),
    [sym_ssh_client_open] = ACTIONS(2117),
    [sym_ssh_client_close] = ACTIONS(2117),
    [sym_tls_client_open] = ACTIONS(2117),
    [sym_tls_client_close] = ACTIONS(2117),
    [sym_do_push] = ACTIONS(2117),
    [sym_do_push_timed] = ACTIONS(2117),
    [sym_do_release] = ACTIONS(2117),
    [sym_device_id] = ACTIONS(2117),
    [sym_device_info] = ACTIONS(2117),
    [sym_device_id_string] = ACTIONS(2117),
    [sym_device_standby] = ACTIONS(2117),
    [sym_device_wake] = ACTIONS(2117),
    [sym_timeline_create] = ACTIONS(2117),
    [sym_timeline_kill] = ACTIONS(2117),
    [sym_timeline_reload] = ACTIONS(2117),
    [sym_timeline_pause] = ACTIONS(2117),
    [sym_timeline_restart] = ACTIONS(2117),
    [sym_timeline_set] = ACTIONS(2117),
    [sym_timeline_get] = ACTIONS(2117),
    [sym_timeline_active] = ACTIONS(2117),
    [sym_timeline_dynamic_id] = ACTIONS(2117),
    [sym_variable_to_string] = ACTIONS(2117),
    [sym_string_to_variable] = ACTIONS(2117),
    [sym_length_variable_to_string] = ACTIONS(2117),
    [sym_type_cast] = ACTIONS(2117),
    [sym_variable_to_xml] = ACTIONS(2117),
    [sym_xml_to_variable] = ACTIONS(2117),
    [sym_length_variable_to_xml] = ACTIONS(2117),
    [sym_redirect_string] = ACTIONS(2117),
    [sym_random_number] = ACTIONS(2117),
    [sym_get_unique_id] = ACTIONS(2117),
    [sym_get_system_number] = ACTIONS(2117),
    [sym_set_system_number] = ACTIONS(2117),
    [sym_get_serial_number] = ACTIONS(2117),
    [sym_get_master_build] = ACTIONS(2117),
    [sym_set_validation_code] = ACTIONS(2117),
    [sym_reboot] = ACTIONS(2117),
    [sym_load_duet_module] = ACTIONS(2117),
    [sym_unload_duet_module] = ACTIONS(2117),
    [sym_duet_mem_size_set] = ACTIONS(2117),
    [sym_duet_mem_size_get] = ACTIONS(2117),
    [sym_dynamic_application_device] = ACTIONS(2117),
    [sym_dynamic_polled_port] = ACTIONS(2117),
    [sym_static_port_binding] = ACTIONS(2117),
    [sym_static_ip_binding] = ACTIONS(2117),
    [sym_get_dns_list] = ACTIONS(2117),
    [sym_set_dns_list] = ACTIONS(2117),
    [sym_get_ip_address] = ACTIONS(2117),
    [sym_set_ip_address] = ACTIONS(2117),
    [sym_get_url_list] = ACTIONS(2117),
    [sym_add_url_entry] = ACTIONS(2117),
    [sym_delete_url_entry] = ACTIONS(2117),
    [sym_abs_value] = ACTIONS(2117),
    [sym_max_value] = ACTIONS(2117),
    [sym_min_value] = ACTIONS(2117),
    [sym_exp_value] = ACTIONS(2117),
    [sym_log_value] = ACTIONS(2117),
    [sym_log10_value] = ACTIONS(2117),
    [sym_power_value] = ACTIONS(2117),
    [sym_sqrt_value] = ACTIONS(2117),
    [sym_raw_be] = ACTIONS(2117),
    [sym_raw_le] = ACTIONS(2117),
    [sym_internal_queue_size_set] = ACTIONS(2117),
    [sym_internal_queue_size_get] = ACTIONS(2117),
    [sym_internal_threshold_set] = ACTIONS(2117),
    [sym_internal_threshold_get] = ACTIONS(2117),
    [sym_rebuilt_event] = ACTIONS(2117),
    [sym_do_custom_event] = ACTIONS(2117),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2117),
    [sym_clkmgr_set_clk_source] = ACTIONS(2117),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2117),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2117),
    [sym_clkmgr_get_timezone] = ACTIONS(2117),
    [sym_clkmgr_set_timezone] = ACTIONS(2117),
    [sym_clkmgr_get_resync_period] = ACTIONS(2117),
    [sym_clkmgr_set_resync_period] = ACTIONS(2117),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2117),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2117),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2117),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2117),
    [sym_clkmgr_get_timeservers] = ACTIONS(2117),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2117),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2117),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2117),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2117),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2117),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2117),
    [sym_set_log_level] = ACTIONS(2117),
    [sym_get_log_level] = ACTIONS(2117),
    [sym_amx_log] = ACTIONS(2117),
    [sym_smtp_server_config_set] = ACTIONS(2117),
    [sym_smtp_server_config_get] = ACTIONS(2117),
    [sym_smtp_send] = ACTIONS(2117),
    [sym_validate_netlinx_account] = ACTIONS(2117),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2117),
    [sym_authenticate_certificate] = ACTIONS(2117),
    [sym_audit_netlinx_session_event] = ACTIONS(2117),
    [sym_audit_netlinx_generic_event] = ACTIONS(2117),
    [sym_get_available_flash_disk_space] = ACTIONS(2117),
    [sym_get_max_flash_disk_space] = ACTIONS(2117),
    [sym_data_create_feed] = ACTIONS(2117),
    [sym_data_delete_feed] = ACTIONS(2117),
    [sym_data_publish_feed] = ACTIONS(2117),
    [sym_data_get_published_feed] = ACTIONS(2117),
    [sym_data_add_record] = ACTIONS(2117),
    [sym_data_get_event_record] = ACTIONS(2117),
    [sym__wc_data_create_feed] = ACTIONS(2117),
    [sym__wc_data_add_record] = ACTIONS(2117),
    [sym__wc_data_get_event_record] = ACTIONS(2117),
    [sym_format] = ACTIONS(2117),
    [sym_astro_clock] = ACTIONS(2117),
    [sym_set_outdoor_temperature] = ACTIONS(2117),
    [sym_set_virtual_level_count] = ACTIONS(2117),
    [sym_set_virtual_channel_count] = ACTIONS(2117),
    [sym_set_virtual_port_count] = ACTIONS(2117),
    [sym__wc_encode_int] = ACTIONS(2117),
    [sym__wc_decode_int] = ACTIONS(2117),
    [sym__wc_int] = ACTIONS(2117),
    [sym__wc_to_ch_int] = ACTIONS(2117),
    [sym__ch_to_wc_int] = ACTIONS(2117),
    [sym__wc_find_string_int] = ACTIONS(2117),
    [sym__wc_left_string_int] = ACTIONS(2117),
    [sym__wc_lower_string_int] = ACTIONS(2117),
    [sym__wc_mid_string_int] = ACTIONS(2117),
    [sym__wc_remove_string_int] = ACTIONS(2117),
    [sym__wc_right_string_int] = ACTIONS(2117),
    [sym__wc_upper_string_int] = ACTIONS(2117),
    [sym__wc_compare_string_int] = ACTIONS(2117),
    [sym__wc_get_buffer_char_int] = ACTIONS(2117),
    [sym__wc_get_buffer_string_int] = ACTIONS(2117),
    [sym__wc_concat_string_int] = ACTIONS(2117),
    [sym__wc_file_open_int] = ACTIONS(2117),
    [sym__wc_file_close_int] = ACTIONS(2117),
    [sym__wc_file_read_int] = ACTIONS(2117),
    [sym__wc_file_read_line_int] = ACTIONS(2117),
    [sym__wc_file_write_int] = ACTIONS(2117),
    [sym__wc_file_write_line_int] = ACTIONS(2117),
    [aux_sym_preproc_include_token1] = ACTIONS(2117),
    [aux_sym_preproc_define_token1] = ACTIONS(2119),
    [aux_sym_preproc_warn_token1] = ACTIONS(2119),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2119),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2119),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2119),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2119),
    [aux_sym_preproc_else_token1] = ACTIONS(2119),
    [anon_sym_LBRACK] = ACTIONS(2119),
    [anon_sym_LBRACE] = ACTIONS(2119),
    [anon_sym_LPAREN] = ACTIONS(2117),
    [anon_sym_BANG] = ACTIONS(2119),
    [anon_sym_TILDE] = ACTIONS(2119),
    [anon_sym_DASH] = ACTIONS(2117),
    [anon_sym_PLUS] = ACTIONS(2117),
    [anon_sym_DASH_DASH] = ACTIONS(2119),
    [anon_sym_PLUS_PLUS] = ACTIONS(2119),
    [anon_sym_DQUOTE] = ACTIONS(2119),
    [anon_sym_SQUOTE] = ACTIONS(2119),
    [sym_number_literal] = ACTIONS(2119),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(376)] = {
    [ts_builtin_sym_end] = ACTIONS(1825),
    [sym_identifier] = ACTIONS(1823),
    [sym_program_name_keyword] = ACTIONS(1823),
    [sym_module_name_keyword] = ACTIONS(1823),
    [sym_define_device_keyword] = ACTIONS(1823),
    [sym_define_combine_keyword] = ACTIONS(1823),
    [sym_define_constant_keyword] = ACTIONS(1823),
    [sym_define_type_keyword] = ACTIONS(1823),
    [sym_define_variable_keyword] = ACTIONS(1823),
    [sym_define_system_variable_keyword] = ACTIONS(1823),
    [sym_define_connect_level_keyword] = ACTIONS(1823),
    [sym_define_latching_keyword] = ACTIONS(1823),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1823),
    [sym_define_toggling_keyword] = ACTIONS(1823),
    [sym_define_call_keyword] = ACTIONS(1823),
    [sym_define_function_keyword] = ACTIONS(1823),
    [sym_define_library_function_keyword] = ACTIONS(1823),
    [sym_define_start_keyword] = ACTIONS(1823),
    [sym_define_event_keyword] = ACTIONS(1823),
    [sym_define_program_keyword] = ACTIONS(1823),
    [sym_define_module_keyword] = ACTIONS(1823),
    [sym_char_keyword] = ACTIONS(1823),
    [sym_widechar_keyword] = ACTIONS(1823),
    [sym_integer_keyword] = ACTIONS(1823),
    [sym_sinteger_keyword] = ACTIONS(1823),
    [sym_long_keyword] = ACTIONS(1823),
    [sym_slong_keyword] = ACTIONS(1823),
    [sym_float_keyword] = ACTIONS(1823),
    [sym_double_keyword] = ACTIONS(1823),
    [sym_variant_keyword] = ACTIONS(1823),
    [sym_variantarray_keyword] = ACTIONS(1823),
    [sym_local_var_keyword] = ACTIONS(1823),
    [sym_stack_var_keyword] = ACTIONS(1823),
    [sym_constant_keyword] = ACTIONS(1823),
    [sym_volatile_keyword] = ACTIONS(1823),
    [sym_non_volatile_keyword] = ACTIONS(1823),
    [sym_persistent_keyword] = ACTIONS(1823),
    [sym_dev_keyword] = ACTIONS(1823),
    [sym_devlev_keyword] = ACTIONS(1823),
    [sym_devchan_keyword] = ACTIONS(1823),
    [sym_if_keyword] = ACTIONS(1823),
    [sym_else_keyword] = ACTIONS(1823),
    [sym_switch_keyword] = ACTIONS(1823),
    [sym_case_keyword] = ACTIONS(1823),
    [sym_default_keyword] = ACTIONS(1823),
    [sym_while_keyword] = ACTIONS(1823),
    [sym_for_keyword] = ACTIONS(1823),
    [sym_break_keyword] = ACTIONS(1823),
    [sym_continue_keyword] = ACTIONS(1823),
    [sym_return_keyword] = ACTIONS(1823),
    [sym_select_keyword] = ACTIONS(1823),
    [sym_active_keyword] = ACTIONS(1823),
    [sym_struct_keyword] = ACTIONS(1823),
    [sym_structure_keyword] = ACTIONS(1823),
    [sym_button_event_keyword] = ACTIONS(1823),
    [sym_channel_event_keyword] = ACTIONS(1823),
    [sym_level_event_keyword] = ACTIONS(1823),
    [sym_data_event_keyword] = ACTIONS(1823),
    [sym_timeline_event_keyword] = ACTIONS(1823),
    [sym_custom_event_keyword] = ACTIONS(1823),
    [sym_push_keyword] = ACTIONS(1823),
    [sym_release_keyword] = ACTIONS(1823),
    [aux_sym_on_keyword_token1] = ACTIONS(1823),
    [aux_sym_off_keyword_token1] = ACTIONS(1823),
    [sym_send_level_keyword] = ACTIONS(1823),
    [sym_send_string_keyword] = ACTIONS(1823),
    [sym_send_command_keyword] = ACTIONS(1823),
    [sym_clear_buffer_keyword] = ACTIONS(1823),
    [sym_create_buffer_keyword] = ACTIONS(1823),
    [sym_create_multi_buffer_keyword] = ACTIONS(1823),
    [sym_call_keyword] = ACTIONS(1823),
    [sym_system_call_keyword] = ACTIONS(1823),
    [sym_devchan_to_keyword] = ACTIONS(1823),
    [sym_devchan_min_to_keyword] = ACTIONS(1823),
    [sym_devchan_total_off_keyword] = ACTIONS(1823),
    [sym_devchan_pulse_keyword] = ACTIONS(1823),
    [sym_wait_keyword] = ACTIONS(1823),
    [sym_cancel_wait_keyword] = ACTIONS(1823),
    [sym_wait_until_keyword] = ACTIONS(1823),
    [sym_cancel_wait_until_keyword] = ACTIONS(1823),
    [sym_cancel_all_wait_keyword] = ACTIONS(1823),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1823),
    [sym_not] = ACTIONS(1823),
    [sym_bnot] = ACTIONS(1823),
    [sym___file__] = ACTIONS(1823),
    [sym___date__] = ACTIONS(1823),
    [sym___ldate__] = ACTIONS(1823),
    [sym___time__] = ACTIONS(1823),
    [sym___line__] = ACTIONS(1823),
    [sym___name__] = ACTIONS(1823),
    [sym___netlinx__] = ACTIONS(1823),
    [sym_netlinx_axi_version] = ACTIONS(1823),
    [sym_url_flg_tcp] = ACTIONS(1823),
    [sym_url_flg_accinfopresent] = ACTIONS(1823),
    [sym_url_flg_temp] = ACTIONS(1823),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1823),
    [sym_url_flg_stat_mask] = ACTIONS(1823),
    [sym_url_flg_stat_lookup] = ACTIONS(1823),
    [sym_url_flg_stat_connecting] = ACTIONS(1823),
    [sym_url_flg_stat_waiting] = ACTIONS(1823),
    [sym_url_flg_stat_connected] = ACTIONS(1823),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1823),
    [sym_first_virtual_device] = ACTIONS(1823),
    [sym_first_local_port] = ACTIONS(1823),
    [sym_dynamic_virtual_device] = ACTIONS(1823),
    [sym_timeline_once] = ACTIONS(1823),
    [sym_timeline_repeat] = ACTIONS(1823),
    [sym_timeline_absolute] = ACTIONS(1823),
    [sym_timeline_relative] = ACTIONS(1823),
    [sym_do_push_timed_infinite] = ACTIONS(1823),
    [sym_source_type_no_address] = ACTIONS(1823),
    [sym_source_type_neuron_id] = ACTIONS(1823),
    [sym_source_type_ip_address] = ACTIONS(1823),
    [sym_source_type_axlink] = ACTIONS(1823),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1823),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1823),
    [sym_source_type_ip_socket_address] = ACTIONS(1823),
    [sym_source_type_rs232] = ACTIONS(1823),
    [sym_source_type_internal] = ACTIONS(1823),
    [sym_source_type_mac_address] = ACTIONS(1823),
    [sym_source_type_ipv4_port] = ACTIONS(1823),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1823),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1823),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1823),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1823),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1823),
    [sym_true] = ACTIONS(1823),
    [sym_false] = ACTIONS(1823),
    [sym_file_read_only] = ACTIONS(1823),
    [sym_file_rw_new] = ACTIONS(1823),
    [sym_file_rw_append] = ACTIONS(1823),
    [sym_ip_tcp] = ACTIONS(1823),
    [sym_ip_udp] = ACTIONS(1823),
    [sym_ip_udp_2way] = ACTIONS(1823),
    [sym_xml_encode_types] = ACTIONS(1823),
    [sym_xml_encode_char_as_list] = ACTIONS(1823),
    [sym_xml_encode_le] = ACTIONS(1823),
    [sym_xml_decode_types] = ACTIONS(1823),
    [sym_xml_decode_no_preserve] = ACTIONS(1823),
    [sym_duet_dev_type_display_device] = ACTIONS(1823),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1823),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1823),
    [sym_duet_dev_type_io_device] = ACTIONS(1823),
    [sym_duet_dev_type_relay_device] = ACTIONS(1823),
    [sym_duet_dev_type_ups] = ACTIONS(1823),
    [sym_duet_dev_type_amplifier] = ACTIONS(1823),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1823),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1823),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1823),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1823),
    [sym_duet_dev_type_camera] = ACTIONS(1823),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1823),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1823),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1823),
    [sym_duet_dev_type_dss] = ACTIONS(1823),
    [sym_duet_dev_type_dvr] = ACTIONS(1823),
    [sym_duet_dev_type_disc_device] = ACTIONS(1823),
    [sym_duet_dev_type_document_camera] = ACTIONS(1823),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1823),
    [sym_duet_dev_type_hvac] = ACTIONS(1823),
    [sym_duet_dev_type_keypad] = ACTIONS(1823),
    [sym_duet_dev_type_light] = ACTIONS(1823),
    [sym_duet_dev_type_monitor] = ACTIONS(1823),
    [sym_duet_dev_type_motor] = ACTIONS(1823),
    [sym_duet_dev_type_multi_window] = ACTIONS(1823),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1823),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1823),
    [sym_duet_dev_type_receiver] = ACTIONS(1823),
    [sym_duet_dev_type_security_system] = ACTIONS(1823),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1823),
    [sym_duet_dev_type_settop_box] = ACTIONS(1823),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1823),
    [sym_duet_dev_type_switcher] = ACTIONS(1823),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1823),
    [sym_duet_dev_type_tv] = ACTIONS(1823),
    [sym_duet_dev_type_utility] = ACTIONS(1823),
    [sym_duet_dev_type_vcr] = ACTIONS(1823),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1823),
    [sym_duet_dev_type_video_processor] = ACTIONS(1823),
    [sym_duet_dev_type_video_projector] = ACTIONS(1823),
    [sym_duet_dev_type_video_wall] = ACTIONS(1823),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1823),
    [sym_duet_dev_type_weather] = ACTIONS(1823),
    [sym_duet_dev_not_polled] = ACTIONS(1823),
    [sym_duet_dev_polled] = ACTIONS(1823),
    [sym_clkmgr_mode_network] = ACTIONS(1823),
    [sym_clkmgr_mode_standalone] = ACTIONS(1823),
    [sym_normal_standby] = ACTIONS(1823),
    [sym_normal_wake] = ACTIONS(1823),
    [sym_ip_multicast_ttl_option] = ACTIONS(1823),
    [sym_ip_ttl_subnet] = ACTIONS(1823),
    [sym_ip_ttl_site] = ACTIONS(1823),
    [sym_ip_ttl_region] = ACTIONS(1823),
    [sym_ip_ttl_continent] = ACTIONS(1823),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1823),
    [sym_ip_nodelay_on] = ACTIONS(1823),
    [sym_ip_nodelay_off] = ACTIONS(1823),
    [sym_amx_error] = ACTIONS(1823),
    [sym_amx_warning] = ACTIONS(1823),
    [sym_amx_info] = ACTIONS(1823),
    [sym_amx_debug] = ACTIONS(1823),
    [sym_netlinx_login_success] = ACTIONS(1823),
    [sym_netlinx_login_fail] = ACTIONS(1823),
    [sym_netlinx_logout] = ACTIONS(1823),
    [sym_null_str] = ACTIONS(1823),
    [sym_smtp_address] = ACTIONS(1823),
    [sym_smtp_port_number] = ACTIONS(1823),
    [sym_smtp_username] = ACTIONS(1823),
    [sym_smtp_password] = ACTIONS(1823),
    [sym_smtp_require_tls] = ACTIONS(1823),
    [sym_smtp_from] = ACTIONS(1823),
    [sym_smtp_tls_true] = ACTIONS(1823),
    [sym_smtp_tls_false] = ACTIONS(1823),
    [sym_listview_on_row_select_event] = ACTIONS(1823),
    [sym_data_structure_unknown] = ACTIONS(1823),
    [sym_data_structure_datafeed] = ACTIONS(1823),
    [sym_data_structure_datarecordset] = ACTIONS(1823),
    [sym_data_structure_datarecord] = ACTIONS(1823),
    [sym_data_structure_datafield] = ACTIONS(1823),
    [sym_data_max_content_field_count] = ACTIONS(1823),
    [sym_data_max_metadata_field_count] = ACTIONS(1823),
    [sym_data_max_id_length] = ACTIONS(1823),
    [sym_data_max_name_length] = ACTIONS(1823),
    [sym_data_max_description_length] = ACTIONS(1823),
    [sym_data_max_type_length] = ACTIONS(1823),
    [sym_data_max_format_length] = ACTIONS(1823),
    [sym_data_max_source_length] = ACTIONS(1823),
    [sym_data_max_value_length] = ACTIONS(1823),
    [sym_data_max_label_length] = ACTIONS(1823),
    [sym_data_max_url_length] = ACTIONS(1823),
    [sym_max_last_login_info_length] = ACTIONS(1823),
    [sym_data_type_unknown] = ACTIONS(1823),
    [sym_data_type_string] = ACTIONS(1823),
    [sym_data_type_datetime] = ACTIONS(1823),
    [sym_data_type_date] = ACTIONS(1823),
    [sym_data_type_time] = ACTIONS(1823),
    [sym_data_type_image] = ACTIONS(1823),
    [sym_data_format_url] = ACTIONS(1823),
    [sym_data_format_phone] = ACTIONS(1823),
    [sym_data_format_email] = ACTIONS(1823),
    [sym_data_format_iso8601] = ACTIONS(1823),
    [sym_auditlog_priv] = ACTIONS(1823),
    [sym_configuration_priv] = ACTIONS(1823),
    [sym_deviceconfig_priv] = ACTIONS(1823),
    [sym_ftp_priv] = ACTIONS(1823),
    [sym_http_priv] = ACTIONS(1823),
    [sym_networkconfig_priv] = ACTIONS(1823),
    [sym_programport_priv] = ACTIONS(1823),
    [sym_remoteui_priv] = ACTIONS(1823),
    [sym_securitycontrol_priv] = ACTIONS(1823),
    [sym_softwaremanagement_priv] = ACTIONS(1823),
    [sym_terminal_priv] = ACTIONS(1823),
    [sym_tpadmin_priv] = ACTIONS(1823),
    [sym_user1_priv] = ACTIONS(1823),
    [sym_user2_priv] = ACTIONS(1823),
    [sym_user3_priv] = ACTIONS(1823),
    [sym_user4_priv] = ACTIONS(1823),
    [sym_usermanagement_priv] = ACTIONS(1823),
    [sym_valid_account] = ACTIONS(1823),
    [sym_err_name_invalid] = ACTIONS(1823),
    [sym_err_password_invalid] = ACTIONS(1823),
    [sym_err_invalid_account] = ACTIONS(1823),
    [sym_err_no_authorization] = ACTIONS(1823),
    [sym_err_invalid_parameter] = ACTIONS(1823),
    [sym_err_user_locked_out] = ACTIONS(1823),
    [sym_err_user_account_expired] = ACTIONS(1823),
    [sym_err_general_error] = ACTIONS(1823),
    [sym_err_authentication_error] = ACTIONS(1823),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1823),
    [sym_tls_validate_certificate] = ACTIONS(1823),
    [sym_tdata] = ACTIONS(1823),
    [sym_tchannel] = ACTIONS(1823),
    [sym_tlevel] = ACTIONS(1823),
    [sym_tbutton] = ACTIONS(1823),
    [sym_ttimeline] = ACTIONS(1823),
    [sym_tcustom] = ACTIONS(1823),
    [sym_url_struct] = ACTIONS(1823),
    [sym_dns_struct] = ACTIONS(1823),
    [sym_ip_address_struct] = ACTIONS(1823),
    [sym_dev_info_struct] = ACTIONS(1823),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1823),
    [sym_data_feed] = ACTIONS(1823),
    [sym_data_field] = ACTIONS(1823),
    [sym_data_record] = ACTIONS(1823),
    [sym_wc_data_feed] = ACTIONS(1823),
    [sym_wc_data_field] = ACTIONS(1823),
    [sym_wc_data_record] = ACTIONS(1823),
    [sym_last_login_info] = ACTIONS(1823),
    [sym____reserved___] = ACTIONS(1823),
    [sym_date] = ACTIONS(1823),
    [sym_day] = ACTIONS(1823),
    [sym_time] = ACTIONS(1823),
    [sym_ldate] = ACTIONS(1823),
    [sym_push_device] = ACTIONS(1823),
    [sym_push_channel] = ACTIONS(1823),
    [sym_push_devchan] = ACTIONS(1823),
    [sym_release_device] = ACTIONS(1823),
    [sym_release_channel] = ACTIONS(1823),
    [sym_release_devchan] = ACTIONS(1823),
    [sym_master_slot] = ACTIONS(1823),
    [sym_get_pulse_time] = ACTIONS(1823),
    [sym_get_timer] = ACTIONS(1823),
    [sym_master_sn] = ACTIONS(1823),
    [sym_system_number] = ACTIONS(1823),
    [sym_button] = ACTIONS(1823),
    [sym_dv_channel] = ACTIONS(1823),
    [sym_data] = ACTIONS(1823),
    [sym_level] = ACTIONS(1823),
    [sym_timeline] = ACTIONS(1823),
    [sym_channel] = ACTIONS(1823),
    [sym_custom] = ACTIONS(1823),
    [sym_atoi] = ACTIONS(1823),
    [sym_atol] = ACTIONS(1823),
    [sym_atof] = ACTIONS(1823),
    [sym_itoa] = ACTIONS(1823),
    [sym_itohex] = ACTIONS(1823),
    [sym_hextoi] = ACTIONS(1823),
    [sym_ftoa] = ACTIONS(1823),
    [sym_find_string] = ACTIONS(1823),
    [sym_left_string] = ACTIONS(1823),
    [sym_right_string] = ACTIONS(1823),
    [sym_mid_string] = ACTIONS(1823),
    [sym_remove_string] = ACTIONS(1823),
    [sym_get_buffer_string] = ACTIONS(1823),
    [sym_get_buffer_char] = ACTIONS(1823),
    [sym_get_multi_buffer_string] = ACTIONS(1823),
    [sym_length_string] = ACTIONS(1823),
    [sym_max_length_string] = ACTIONS(1823),
    [sym_set_length_string] = ACTIONS(1823),
    [sym_compare_string] = ACTIONS(1823),
    [sym_upper_string] = ACTIONS(1823),
    [sym_lower_string] = ACTIONS(1823),
    [sym_get_last] = ACTIONS(1823),
    [sym_date_to_day] = ACTIONS(1823),
    [sym_date_to_month] = ACTIONS(1823),
    [sym_date_to_year] = ACTIONS(1823),
    [sym_time_to_hour] = ACTIONS(1823),
    [sym_time_to_minute] = ACTIONS(1823),
    [sym_time_to_second] = ACTIONS(1823),
    [sym_day_of_week] = ACTIONS(1823),
    [sym_set_pulse_time] = ACTIONS(1823),
    [sym_set_timer] = ACTIONS(1823),
    [sym_file_open] = ACTIONS(1823),
    [sym_file_close] = ACTIONS(1823),
    [sym_file_read] = ACTIONS(1823),
    [sym_file_read_line] = ACTIONS(1823),
    [sym_file_write] = ACTIONS(1823),
    [sym_file_write_line] = ACTIONS(1823),
    [sym_file_copy] = ACTIONS(1823),
    [sym_file_delete] = ACTIONS(1823),
    [sym_file_rename] = ACTIONS(1823),
    [sym_file_dir] = ACTIONS(1823),
    [sym_file_getdir] = ACTIONS(1823),
    [sym_file_setdir] = ACTIONS(1823),
    [sym_file_create_dir] = ACTIONS(1823),
    [sym_file_remove_dir] = ACTIONS(1823),
    [sym_file_seek] = ACTIONS(1823),
    [sym_length_array] = ACTIONS(1823),
    [sym_max_length_array] = ACTIONS(1823),
    [sym_set_length_array] = ACTIONS(1823),
    [sym_ip_client_open] = ACTIONS(1823),
    [sym_ip_client_close] = ACTIONS(1823),
    [sym_ip_server_open] = ACTIONS(1823),
    [sym_ip_server_close] = ACTIONS(1823),
    [sym_ip_bound_client_open] = ACTIONS(1823),
    [sym_ip_mc_server_open] = ACTIONS(1823),
    [sym_ip_set_option] = ACTIONS(1823),
    [sym_ssh_client_open] = ACTIONS(1823),
    [sym_ssh_client_close] = ACTIONS(1823),
    [sym_tls_client_open] = ACTIONS(1823),
    [sym_tls_client_close] = ACTIONS(1823),
    [sym_do_push] = ACTIONS(1823),
    [sym_do_push_timed] = ACTIONS(1823),
    [sym_do_release] = ACTIONS(1823),
    [sym_device_id] = ACTIONS(1823),
    [sym_device_info] = ACTIONS(1823),
    [sym_device_id_string] = ACTIONS(1823),
    [sym_device_standby] = ACTIONS(1823),
    [sym_device_wake] = ACTIONS(1823),
    [sym_timeline_create] = ACTIONS(1823),
    [sym_timeline_kill] = ACTIONS(1823),
    [sym_timeline_reload] = ACTIONS(1823),
    [sym_timeline_pause] = ACTIONS(1823),
    [sym_timeline_restart] = ACTIONS(1823),
    [sym_timeline_set] = ACTIONS(1823),
    [sym_timeline_get] = ACTIONS(1823),
    [sym_timeline_active] = ACTIONS(1823),
    [sym_timeline_dynamic_id] = ACTIONS(1823),
    [sym_variable_to_string] = ACTIONS(1823),
    [sym_string_to_variable] = ACTIONS(1823),
    [sym_length_variable_to_string] = ACTIONS(1823),
    [sym_type_cast] = ACTIONS(1823),
    [sym_variable_to_xml] = ACTIONS(1823),
    [sym_xml_to_variable] = ACTIONS(1823),
    [sym_length_variable_to_xml] = ACTIONS(1823),
    [sym_redirect_string] = ACTIONS(1823),
    [sym_random_number] = ACTIONS(1823),
    [sym_get_unique_id] = ACTIONS(1823),
    [sym_get_system_number] = ACTIONS(1823),
    [sym_set_system_number] = ACTIONS(1823),
    [sym_get_serial_number] = ACTIONS(1823),
    [sym_get_master_build] = ACTIONS(1823),
    [sym_set_validation_code] = ACTIONS(1823),
    [sym_reboot] = ACTIONS(1823),
    [sym_load_duet_module] = ACTIONS(1823),
    [sym_unload_duet_module] = ACTIONS(1823),
    [sym_duet_mem_size_set] = ACTIONS(1823),
    [sym_duet_mem_size_get] = ACTIONS(1823),
    [sym_dynamic_application_device] = ACTIONS(1823),
    [sym_dynamic_polled_port] = ACTIONS(1823),
    [sym_static_port_binding] = ACTIONS(1823),
    [sym_static_ip_binding] = ACTIONS(1823),
    [sym_get_dns_list] = ACTIONS(1823),
    [sym_set_dns_list] = ACTIONS(1823),
    [sym_get_ip_address] = ACTIONS(1823),
    [sym_set_ip_address] = ACTIONS(1823),
    [sym_get_url_list] = ACTIONS(1823),
    [sym_add_url_entry] = ACTIONS(1823),
    [sym_delete_url_entry] = ACTIONS(1823),
    [sym_abs_value] = ACTIONS(1823),
    [sym_max_value] = ACTIONS(1823),
    [sym_min_value] = ACTIONS(1823),
    [sym_exp_value] = ACTIONS(1823),
    [sym_log_value] = ACTIONS(1823),
    [sym_log10_value] = ACTIONS(1823),
    [sym_power_value] = ACTIONS(1823),
    [sym_sqrt_value] = ACTIONS(1823),
    [sym_raw_be] = ACTIONS(1823),
    [sym_raw_le] = ACTIONS(1823),
    [sym_internal_queue_size_set] = ACTIONS(1823),
    [sym_internal_queue_size_get] = ACTIONS(1823),
    [sym_internal_threshold_set] = ACTIONS(1823),
    [sym_internal_threshold_get] = ACTIONS(1823),
    [sym_rebuilt_event] = ACTIONS(1823),
    [sym_do_custom_event] = ACTIONS(1823),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1823),
    [sym_clkmgr_set_clk_source] = ACTIONS(1823),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1823),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1823),
    [sym_clkmgr_get_timezone] = ACTIONS(1823),
    [sym_clkmgr_set_timezone] = ACTIONS(1823),
    [sym_clkmgr_get_resync_period] = ACTIONS(1823),
    [sym_clkmgr_set_resync_period] = ACTIONS(1823),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1823),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1823),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1823),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1823),
    [sym_clkmgr_get_timeservers] = ACTIONS(1823),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1823),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1823),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1823),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1823),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1823),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1823),
    [sym_set_log_level] = ACTIONS(1823),
    [sym_get_log_level] = ACTIONS(1823),
    [sym_amx_log] = ACTIONS(1823),
    [sym_smtp_server_config_set] = ACTIONS(1823),
    [sym_smtp_server_config_get] = ACTIONS(1823),
    [sym_smtp_send] = ACTIONS(1823),
    [sym_validate_netlinx_account] = ACTIONS(1823),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1823),
    [sym_authenticate_certificate] = ACTIONS(1823),
    [sym_audit_netlinx_session_event] = ACTIONS(1823),
    [sym_audit_netlinx_generic_event] = ACTIONS(1823),
    [sym_get_available_flash_disk_space] = ACTIONS(1823),
    [sym_get_max_flash_disk_space] = ACTIONS(1823),
    [sym_data_create_feed] = ACTIONS(1823),
    [sym_data_delete_feed] = ACTIONS(1823),
    [sym_data_publish_feed] = ACTIONS(1823),
    [sym_data_get_published_feed] = ACTIONS(1823),
    [sym_data_add_record] = ACTIONS(1823),
    [sym_data_get_event_record] = ACTIONS(1823),
    [sym__wc_data_create_feed] = ACTIONS(1823),
    [sym__wc_data_add_record] = ACTIONS(1823),
    [sym__wc_data_get_event_record] = ACTIONS(1823),
    [sym_format] = ACTIONS(1823),
    [sym_astro_clock] = ACTIONS(1823),
    [sym_set_outdoor_temperature] = ACTIONS(1823),
    [sym_set_virtual_level_count] = ACTIONS(1823),
    [sym_set_virtual_channel_count] = ACTIONS(1823),
    [sym_set_virtual_port_count] = ACTIONS(1823),
    [sym__wc_encode_int] = ACTIONS(1823),
    [sym__wc_decode_int] = ACTIONS(1823),
    [sym__wc_int] = ACTIONS(1823),
    [sym__wc_to_ch_int] = ACTIONS(1823),
    [sym__ch_to_wc_int] = ACTIONS(1823),
    [sym__wc_find_string_int] = ACTIONS(1823),
    [sym__wc_left_string_int] = ACTIONS(1823),
    [sym__wc_lower_string_int] = ACTIONS(1823),
    [sym__wc_mid_string_int] = ACTIONS(1823),
    [sym__wc_remove_string_int] = ACTIONS(1823),
    [sym__wc_right_string_int] = ACTIONS(1823),
    [sym__wc_upper_string_int] = ACTIONS(1823),
    [sym__wc_compare_string_int] = ACTIONS(1823),
    [sym__wc_get_buffer_char_int] = ACTIONS(1823),
    [sym__wc_get_buffer_string_int] = ACTIONS(1823),
    [sym__wc_concat_string_int] = ACTIONS(1823),
    [sym__wc_file_open_int] = ACTIONS(1823),
    [sym__wc_file_close_int] = ACTIONS(1823),
    [sym__wc_file_read_int] = ACTIONS(1823),
    [sym__wc_file_read_line_int] = ACTIONS(1823),
    [sym__wc_file_write_int] = ACTIONS(1823),
    [sym__wc_file_write_line_int] = ACTIONS(1823),
    [aux_sym_preproc_include_token1] = ACTIONS(1823),
    [aux_sym_preproc_define_token1] = ACTIONS(1825),
    [aux_sym_preproc_warn_token1] = ACTIONS(1825),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1825),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1825),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1825),
    [anon_sym_LBRACK] = ACTIONS(1825),
    [anon_sym_LBRACE] = ACTIONS(1825),
    [anon_sym_LPAREN] = ACTIONS(1823),
    [anon_sym_BANG] = ACTIONS(1825),
    [anon_sym_TILDE] = ACTIONS(1825),
    [anon_sym_DASH] = ACTIONS(1823),
    [anon_sym_PLUS] = ACTIONS(1823),
    [anon_sym_DASH_DASH] = ACTIONS(1825),
    [anon_sym_PLUS_PLUS] = ACTIONS(1825),
    [anon_sym_DQUOTE] = ACTIONS(1825),
    [anon_sym_SQUOTE] = ACTIONS(1825),
    [sym_number_literal] = ACTIONS(1825),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(377)] = {
    [ts_builtin_sym_end] = ACTIONS(1809),
    [sym_identifier] = ACTIONS(1807),
    [sym_program_name_keyword] = ACTIONS(1807),
    [sym_module_name_keyword] = ACTIONS(1807),
    [sym_define_device_keyword] = ACTIONS(1807),
    [sym_define_combine_keyword] = ACTIONS(1807),
    [sym_define_constant_keyword] = ACTIONS(1807),
    [sym_define_type_keyword] = ACTIONS(1807),
    [sym_define_variable_keyword] = ACTIONS(1807),
    [sym_define_system_variable_keyword] = ACTIONS(1807),
    [sym_define_connect_level_keyword] = ACTIONS(1807),
    [sym_define_latching_keyword] = ACTIONS(1807),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1807),
    [sym_define_toggling_keyword] = ACTIONS(1807),
    [sym_define_call_keyword] = ACTIONS(1807),
    [sym_define_function_keyword] = ACTIONS(1807),
    [sym_define_library_function_keyword] = ACTIONS(1807),
    [sym_define_start_keyword] = ACTIONS(1807),
    [sym_define_event_keyword] = ACTIONS(1807),
    [sym_define_program_keyword] = ACTIONS(1807),
    [sym_define_module_keyword] = ACTIONS(1807),
    [sym_char_keyword] = ACTIONS(1807),
    [sym_widechar_keyword] = ACTIONS(1807),
    [sym_integer_keyword] = ACTIONS(1807),
    [sym_sinteger_keyword] = ACTIONS(1807),
    [sym_long_keyword] = ACTIONS(1807),
    [sym_slong_keyword] = ACTIONS(1807),
    [sym_float_keyword] = ACTIONS(1807),
    [sym_double_keyword] = ACTIONS(1807),
    [sym_variant_keyword] = ACTIONS(1807),
    [sym_variantarray_keyword] = ACTIONS(1807),
    [sym_local_var_keyword] = ACTIONS(1807),
    [sym_stack_var_keyword] = ACTIONS(1807),
    [sym_constant_keyword] = ACTIONS(1807),
    [sym_volatile_keyword] = ACTIONS(1807),
    [sym_non_volatile_keyword] = ACTIONS(1807),
    [sym_persistent_keyword] = ACTIONS(1807),
    [sym_dev_keyword] = ACTIONS(1807),
    [sym_devlev_keyword] = ACTIONS(1807),
    [sym_devchan_keyword] = ACTIONS(1807),
    [sym_if_keyword] = ACTIONS(1807),
    [sym_else_keyword] = ACTIONS(1807),
    [sym_switch_keyword] = ACTIONS(1807),
    [sym_case_keyword] = ACTIONS(1807),
    [sym_default_keyword] = ACTIONS(1807),
    [sym_while_keyword] = ACTIONS(1807),
    [sym_for_keyword] = ACTIONS(1807),
    [sym_break_keyword] = ACTIONS(1807),
    [sym_continue_keyword] = ACTIONS(1807),
    [sym_return_keyword] = ACTIONS(1807),
    [sym_select_keyword] = ACTIONS(1807),
    [sym_active_keyword] = ACTIONS(1807),
    [sym_struct_keyword] = ACTIONS(1807),
    [sym_structure_keyword] = ACTIONS(1807),
    [sym_button_event_keyword] = ACTIONS(1807),
    [sym_channel_event_keyword] = ACTIONS(1807),
    [sym_level_event_keyword] = ACTIONS(1807),
    [sym_data_event_keyword] = ACTIONS(1807),
    [sym_timeline_event_keyword] = ACTIONS(1807),
    [sym_custom_event_keyword] = ACTIONS(1807),
    [sym_push_keyword] = ACTIONS(1807),
    [sym_release_keyword] = ACTIONS(1807),
    [aux_sym_on_keyword_token1] = ACTIONS(1807),
    [aux_sym_off_keyword_token1] = ACTIONS(1807),
    [sym_send_level_keyword] = ACTIONS(1807),
    [sym_send_string_keyword] = ACTIONS(1807),
    [sym_send_command_keyword] = ACTIONS(1807),
    [sym_clear_buffer_keyword] = ACTIONS(1807),
    [sym_create_buffer_keyword] = ACTIONS(1807),
    [sym_create_multi_buffer_keyword] = ACTIONS(1807),
    [sym_call_keyword] = ACTIONS(1807),
    [sym_system_call_keyword] = ACTIONS(1807),
    [sym_devchan_to_keyword] = ACTIONS(1807),
    [sym_devchan_min_to_keyword] = ACTIONS(1807),
    [sym_devchan_total_off_keyword] = ACTIONS(1807),
    [sym_devchan_pulse_keyword] = ACTIONS(1807),
    [sym_wait_keyword] = ACTIONS(1807),
    [sym_cancel_wait_keyword] = ACTIONS(1807),
    [sym_wait_until_keyword] = ACTIONS(1807),
    [sym_cancel_wait_until_keyword] = ACTIONS(1807),
    [sym_cancel_all_wait_keyword] = ACTIONS(1807),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1807),
    [sym_not] = ACTIONS(1807),
    [sym_bnot] = ACTIONS(1807),
    [sym___file__] = ACTIONS(1807),
    [sym___date__] = ACTIONS(1807),
    [sym___ldate__] = ACTIONS(1807),
    [sym___time__] = ACTIONS(1807),
    [sym___line__] = ACTIONS(1807),
    [sym___name__] = ACTIONS(1807),
    [sym___netlinx__] = ACTIONS(1807),
    [sym_netlinx_axi_version] = ACTIONS(1807),
    [sym_url_flg_tcp] = ACTIONS(1807),
    [sym_url_flg_accinfopresent] = ACTIONS(1807),
    [sym_url_flg_temp] = ACTIONS(1807),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1807),
    [sym_url_flg_stat_mask] = ACTIONS(1807),
    [sym_url_flg_stat_lookup] = ACTIONS(1807),
    [sym_url_flg_stat_connecting] = ACTIONS(1807),
    [sym_url_flg_stat_waiting] = ACTIONS(1807),
    [sym_url_flg_stat_connected] = ACTIONS(1807),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1807),
    [sym_first_virtual_device] = ACTIONS(1807),
    [sym_first_local_port] = ACTIONS(1807),
    [sym_dynamic_virtual_device] = ACTIONS(1807),
    [sym_timeline_once] = ACTIONS(1807),
    [sym_timeline_repeat] = ACTIONS(1807),
    [sym_timeline_absolute] = ACTIONS(1807),
    [sym_timeline_relative] = ACTIONS(1807),
    [sym_do_push_timed_infinite] = ACTIONS(1807),
    [sym_source_type_no_address] = ACTIONS(1807),
    [sym_source_type_neuron_id] = ACTIONS(1807),
    [sym_source_type_ip_address] = ACTIONS(1807),
    [sym_source_type_axlink] = ACTIONS(1807),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1807),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1807),
    [sym_source_type_ip_socket_address] = ACTIONS(1807),
    [sym_source_type_rs232] = ACTIONS(1807),
    [sym_source_type_internal] = ACTIONS(1807),
    [sym_source_type_mac_address] = ACTIONS(1807),
    [sym_source_type_ipv4_port] = ACTIONS(1807),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1807),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1807),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1807),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1807),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1807),
    [sym_true] = ACTIONS(1807),
    [sym_false] = ACTIONS(1807),
    [sym_file_read_only] = ACTIONS(1807),
    [sym_file_rw_new] = ACTIONS(1807),
    [sym_file_rw_append] = ACTIONS(1807),
    [sym_ip_tcp] = ACTIONS(1807),
    [sym_ip_udp] = ACTIONS(1807),
    [sym_ip_udp_2way] = ACTIONS(1807),
    [sym_xml_encode_types] = ACTIONS(1807),
    [sym_xml_encode_char_as_list] = ACTIONS(1807),
    [sym_xml_encode_le] = ACTIONS(1807),
    [sym_xml_decode_types] = ACTIONS(1807),
    [sym_xml_decode_no_preserve] = ACTIONS(1807),
    [sym_duet_dev_type_display_device] = ACTIONS(1807),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1807),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1807),
    [sym_duet_dev_type_io_device] = ACTIONS(1807),
    [sym_duet_dev_type_relay_device] = ACTIONS(1807),
    [sym_duet_dev_type_ups] = ACTIONS(1807),
    [sym_duet_dev_type_amplifier] = ACTIONS(1807),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1807),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1807),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1807),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1807),
    [sym_duet_dev_type_camera] = ACTIONS(1807),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1807),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1807),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1807),
    [sym_duet_dev_type_dss] = ACTIONS(1807),
    [sym_duet_dev_type_dvr] = ACTIONS(1807),
    [sym_duet_dev_type_disc_device] = ACTIONS(1807),
    [sym_duet_dev_type_document_camera] = ACTIONS(1807),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1807),
    [sym_duet_dev_type_hvac] = ACTIONS(1807),
    [sym_duet_dev_type_keypad] = ACTIONS(1807),
    [sym_duet_dev_type_light] = ACTIONS(1807),
    [sym_duet_dev_type_monitor] = ACTIONS(1807),
    [sym_duet_dev_type_motor] = ACTIONS(1807),
    [sym_duet_dev_type_multi_window] = ACTIONS(1807),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1807),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1807),
    [sym_duet_dev_type_receiver] = ACTIONS(1807),
    [sym_duet_dev_type_security_system] = ACTIONS(1807),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1807),
    [sym_duet_dev_type_settop_box] = ACTIONS(1807),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1807),
    [sym_duet_dev_type_switcher] = ACTIONS(1807),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1807),
    [sym_duet_dev_type_tv] = ACTIONS(1807),
    [sym_duet_dev_type_utility] = ACTIONS(1807),
    [sym_duet_dev_type_vcr] = ACTIONS(1807),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1807),
    [sym_duet_dev_type_video_processor] = ACTIONS(1807),
    [sym_duet_dev_type_video_projector] = ACTIONS(1807),
    [sym_duet_dev_type_video_wall] = ACTIONS(1807),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1807),
    [sym_duet_dev_type_weather] = ACTIONS(1807),
    [sym_duet_dev_not_polled] = ACTIONS(1807),
    [sym_duet_dev_polled] = ACTIONS(1807),
    [sym_clkmgr_mode_network] = ACTIONS(1807),
    [sym_clkmgr_mode_standalone] = ACTIONS(1807),
    [sym_normal_standby] = ACTIONS(1807),
    [sym_normal_wake] = ACTIONS(1807),
    [sym_ip_multicast_ttl_option] = ACTIONS(1807),
    [sym_ip_ttl_subnet] = ACTIONS(1807),
    [sym_ip_ttl_site] = ACTIONS(1807),
    [sym_ip_ttl_region] = ACTIONS(1807),
    [sym_ip_ttl_continent] = ACTIONS(1807),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1807),
    [sym_ip_nodelay_on] = ACTIONS(1807),
    [sym_ip_nodelay_off] = ACTIONS(1807),
    [sym_amx_error] = ACTIONS(1807),
    [sym_amx_warning] = ACTIONS(1807),
    [sym_amx_info] = ACTIONS(1807),
    [sym_amx_debug] = ACTIONS(1807),
    [sym_netlinx_login_success] = ACTIONS(1807),
    [sym_netlinx_login_fail] = ACTIONS(1807),
    [sym_netlinx_logout] = ACTIONS(1807),
    [sym_null_str] = ACTIONS(1807),
    [sym_smtp_address] = ACTIONS(1807),
    [sym_smtp_port_number] = ACTIONS(1807),
    [sym_smtp_username] = ACTIONS(1807),
    [sym_smtp_password] = ACTIONS(1807),
    [sym_smtp_require_tls] = ACTIONS(1807),
    [sym_smtp_from] = ACTIONS(1807),
    [sym_smtp_tls_true] = ACTIONS(1807),
    [sym_smtp_tls_false] = ACTIONS(1807),
    [sym_listview_on_row_select_event] = ACTIONS(1807),
    [sym_data_structure_unknown] = ACTIONS(1807),
    [sym_data_structure_datafeed] = ACTIONS(1807),
    [sym_data_structure_datarecordset] = ACTIONS(1807),
    [sym_data_structure_datarecord] = ACTIONS(1807),
    [sym_data_structure_datafield] = ACTIONS(1807),
    [sym_data_max_content_field_count] = ACTIONS(1807),
    [sym_data_max_metadata_field_count] = ACTIONS(1807),
    [sym_data_max_id_length] = ACTIONS(1807),
    [sym_data_max_name_length] = ACTIONS(1807),
    [sym_data_max_description_length] = ACTIONS(1807),
    [sym_data_max_type_length] = ACTIONS(1807),
    [sym_data_max_format_length] = ACTIONS(1807),
    [sym_data_max_source_length] = ACTIONS(1807),
    [sym_data_max_value_length] = ACTIONS(1807),
    [sym_data_max_label_length] = ACTIONS(1807),
    [sym_data_max_url_length] = ACTIONS(1807),
    [sym_max_last_login_info_length] = ACTIONS(1807),
    [sym_data_type_unknown] = ACTIONS(1807),
    [sym_data_type_string] = ACTIONS(1807),
    [sym_data_type_datetime] = ACTIONS(1807),
    [sym_data_type_date] = ACTIONS(1807),
    [sym_data_type_time] = ACTIONS(1807),
    [sym_data_type_image] = ACTIONS(1807),
    [sym_data_format_url] = ACTIONS(1807),
    [sym_data_format_phone] = ACTIONS(1807),
    [sym_data_format_email] = ACTIONS(1807),
    [sym_data_format_iso8601] = ACTIONS(1807),
    [sym_auditlog_priv] = ACTIONS(1807),
    [sym_configuration_priv] = ACTIONS(1807),
    [sym_deviceconfig_priv] = ACTIONS(1807),
    [sym_ftp_priv] = ACTIONS(1807),
    [sym_http_priv] = ACTIONS(1807),
    [sym_networkconfig_priv] = ACTIONS(1807),
    [sym_programport_priv] = ACTIONS(1807),
    [sym_remoteui_priv] = ACTIONS(1807),
    [sym_securitycontrol_priv] = ACTIONS(1807),
    [sym_softwaremanagement_priv] = ACTIONS(1807),
    [sym_terminal_priv] = ACTIONS(1807),
    [sym_tpadmin_priv] = ACTIONS(1807),
    [sym_user1_priv] = ACTIONS(1807),
    [sym_user2_priv] = ACTIONS(1807),
    [sym_user3_priv] = ACTIONS(1807),
    [sym_user4_priv] = ACTIONS(1807),
    [sym_usermanagement_priv] = ACTIONS(1807),
    [sym_valid_account] = ACTIONS(1807),
    [sym_err_name_invalid] = ACTIONS(1807),
    [sym_err_password_invalid] = ACTIONS(1807),
    [sym_err_invalid_account] = ACTIONS(1807),
    [sym_err_no_authorization] = ACTIONS(1807),
    [sym_err_invalid_parameter] = ACTIONS(1807),
    [sym_err_user_locked_out] = ACTIONS(1807),
    [sym_err_user_account_expired] = ACTIONS(1807),
    [sym_err_general_error] = ACTIONS(1807),
    [sym_err_authentication_error] = ACTIONS(1807),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1807),
    [sym_tls_validate_certificate] = ACTIONS(1807),
    [sym_tdata] = ACTIONS(1807),
    [sym_tchannel] = ACTIONS(1807),
    [sym_tlevel] = ACTIONS(1807),
    [sym_tbutton] = ACTIONS(1807),
    [sym_ttimeline] = ACTIONS(1807),
    [sym_tcustom] = ACTIONS(1807),
    [sym_url_struct] = ACTIONS(1807),
    [sym_dns_struct] = ACTIONS(1807),
    [sym_ip_address_struct] = ACTIONS(1807),
    [sym_dev_info_struct] = ACTIONS(1807),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1807),
    [sym_data_feed] = ACTIONS(1807),
    [sym_data_field] = ACTIONS(1807),
    [sym_data_record] = ACTIONS(1807),
    [sym_wc_data_feed] = ACTIONS(1807),
    [sym_wc_data_field] = ACTIONS(1807),
    [sym_wc_data_record] = ACTIONS(1807),
    [sym_last_login_info] = ACTIONS(1807),
    [sym____reserved___] = ACTIONS(1807),
    [sym_date] = ACTIONS(1807),
    [sym_day] = ACTIONS(1807),
    [sym_time] = ACTIONS(1807),
    [sym_ldate] = ACTIONS(1807),
    [sym_push_device] = ACTIONS(1807),
    [sym_push_channel] = ACTIONS(1807),
    [sym_push_devchan] = ACTIONS(1807),
    [sym_release_device] = ACTIONS(1807),
    [sym_release_channel] = ACTIONS(1807),
    [sym_release_devchan] = ACTIONS(1807),
    [sym_master_slot] = ACTIONS(1807),
    [sym_get_pulse_time] = ACTIONS(1807),
    [sym_get_timer] = ACTIONS(1807),
    [sym_master_sn] = ACTIONS(1807),
    [sym_system_number] = ACTIONS(1807),
    [sym_button] = ACTIONS(1807),
    [sym_dv_channel] = ACTIONS(1807),
    [sym_data] = ACTIONS(1807),
    [sym_level] = ACTIONS(1807),
    [sym_timeline] = ACTIONS(1807),
    [sym_channel] = ACTIONS(1807),
    [sym_custom] = ACTIONS(1807),
    [sym_atoi] = ACTIONS(1807),
    [sym_atol] = ACTIONS(1807),
    [sym_atof] = ACTIONS(1807),
    [sym_itoa] = ACTIONS(1807),
    [sym_itohex] = ACTIONS(1807),
    [sym_hextoi] = ACTIONS(1807),
    [sym_ftoa] = ACTIONS(1807),
    [sym_find_string] = ACTIONS(1807),
    [sym_left_string] = ACTIONS(1807),
    [sym_right_string] = ACTIONS(1807),
    [sym_mid_string] = ACTIONS(1807),
    [sym_remove_string] = ACTIONS(1807),
    [sym_get_buffer_string] = ACTIONS(1807),
    [sym_get_buffer_char] = ACTIONS(1807),
    [sym_get_multi_buffer_string] = ACTIONS(1807),
    [sym_length_string] = ACTIONS(1807),
    [sym_max_length_string] = ACTIONS(1807),
    [sym_set_length_string] = ACTIONS(1807),
    [sym_compare_string] = ACTIONS(1807),
    [sym_upper_string] = ACTIONS(1807),
    [sym_lower_string] = ACTIONS(1807),
    [sym_get_last] = ACTIONS(1807),
    [sym_date_to_day] = ACTIONS(1807),
    [sym_date_to_month] = ACTIONS(1807),
    [sym_date_to_year] = ACTIONS(1807),
    [sym_time_to_hour] = ACTIONS(1807),
    [sym_time_to_minute] = ACTIONS(1807),
    [sym_time_to_second] = ACTIONS(1807),
    [sym_day_of_week] = ACTIONS(1807),
    [sym_set_pulse_time] = ACTIONS(1807),
    [sym_set_timer] = ACTIONS(1807),
    [sym_file_open] = ACTIONS(1807),
    [sym_file_close] = ACTIONS(1807),
    [sym_file_read] = ACTIONS(1807),
    [sym_file_read_line] = ACTIONS(1807),
    [sym_file_write] = ACTIONS(1807),
    [sym_file_write_line] = ACTIONS(1807),
    [sym_file_copy] = ACTIONS(1807),
    [sym_file_delete] = ACTIONS(1807),
    [sym_file_rename] = ACTIONS(1807),
    [sym_file_dir] = ACTIONS(1807),
    [sym_file_getdir] = ACTIONS(1807),
    [sym_file_setdir] = ACTIONS(1807),
    [sym_file_create_dir] = ACTIONS(1807),
    [sym_file_remove_dir] = ACTIONS(1807),
    [sym_file_seek] = ACTIONS(1807),
    [sym_length_array] = ACTIONS(1807),
    [sym_max_length_array] = ACTIONS(1807),
    [sym_set_length_array] = ACTIONS(1807),
    [sym_ip_client_open] = ACTIONS(1807),
    [sym_ip_client_close] = ACTIONS(1807),
    [sym_ip_server_open] = ACTIONS(1807),
    [sym_ip_server_close] = ACTIONS(1807),
    [sym_ip_bound_client_open] = ACTIONS(1807),
    [sym_ip_mc_server_open] = ACTIONS(1807),
    [sym_ip_set_option] = ACTIONS(1807),
    [sym_ssh_client_open] = ACTIONS(1807),
    [sym_ssh_client_close] = ACTIONS(1807),
    [sym_tls_client_open] = ACTIONS(1807),
    [sym_tls_client_close] = ACTIONS(1807),
    [sym_do_push] = ACTIONS(1807),
    [sym_do_push_timed] = ACTIONS(1807),
    [sym_do_release] = ACTIONS(1807),
    [sym_device_id] = ACTIONS(1807),
    [sym_device_info] = ACTIONS(1807),
    [sym_device_id_string] = ACTIONS(1807),
    [sym_device_standby] = ACTIONS(1807),
    [sym_device_wake] = ACTIONS(1807),
    [sym_timeline_create] = ACTIONS(1807),
    [sym_timeline_kill] = ACTIONS(1807),
    [sym_timeline_reload] = ACTIONS(1807),
    [sym_timeline_pause] = ACTIONS(1807),
    [sym_timeline_restart] = ACTIONS(1807),
    [sym_timeline_set] = ACTIONS(1807),
    [sym_timeline_get] = ACTIONS(1807),
    [sym_timeline_active] = ACTIONS(1807),
    [sym_timeline_dynamic_id] = ACTIONS(1807),
    [sym_variable_to_string] = ACTIONS(1807),
    [sym_string_to_variable] = ACTIONS(1807),
    [sym_length_variable_to_string] = ACTIONS(1807),
    [sym_type_cast] = ACTIONS(1807),
    [sym_variable_to_xml] = ACTIONS(1807),
    [sym_xml_to_variable] = ACTIONS(1807),
    [sym_length_variable_to_xml] = ACTIONS(1807),
    [sym_redirect_string] = ACTIONS(1807),
    [sym_random_number] = ACTIONS(1807),
    [sym_get_unique_id] = ACTIONS(1807),
    [sym_get_system_number] = ACTIONS(1807),
    [sym_set_system_number] = ACTIONS(1807),
    [sym_get_serial_number] = ACTIONS(1807),
    [sym_get_master_build] = ACTIONS(1807),
    [sym_set_validation_code] = ACTIONS(1807),
    [sym_reboot] = ACTIONS(1807),
    [sym_load_duet_module] = ACTIONS(1807),
    [sym_unload_duet_module] = ACTIONS(1807),
    [sym_duet_mem_size_set] = ACTIONS(1807),
    [sym_duet_mem_size_get] = ACTIONS(1807),
    [sym_dynamic_application_device] = ACTIONS(1807),
    [sym_dynamic_polled_port] = ACTIONS(1807),
    [sym_static_port_binding] = ACTIONS(1807),
    [sym_static_ip_binding] = ACTIONS(1807),
    [sym_get_dns_list] = ACTIONS(1807),
    [sym_set_dns_list] = ACTIONS(1807),
    [sym_get_ip_address] = ACTIONS(1807),
    [sym_set_ip_address] = ACTIONS(1807),
    [sym_get_url_list] = ACTIONS(1807),
    [sym_add_url_entry] = ACTIONS(1807),
    [sym_delete_url_entry] = ACTIONS(1807),
    [sym_abs_value] = ACTIONS(1807),
    [sym_max_value] = ACTIONS(1807),
    [sym_min_value] = ACTIONS(1807),
    [sym_exp_value] = ACTIONS(1807),
    [sym_log_value] = ACTIONS(1807),
    [sym_log10_value] = ACTIONS(1807),
    [sym_power_value] = ACTIONS(1807),
    [sym_sqrt_value] = ACTIONS(1807),
    [sym_raw_be] = ACTIONS(1807),
    [sym_raw_le] = ACTIONS(1807),
    [sym_internal_queue_size_set] = ACTIONS(1807),
    [sym_internal_queue_size_get] = ACTIONS(1807),
    [sym_internal_threshold_set] = ACTIONS(1807),
    [sym_internal_threshold_get] = ACTIONS(1807),
    [sym_rebuilt_event] = ACTIONS(1807),
    [sym_do_custom_event] = ACTIONS(1807),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1807),
    [sym_clkmgr_set_clk_source] = ACTIONS(1807),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1807),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1807),
    [sym_clkmgr_get_timezone] = ACTIONS(1807),
    [sym_clkmgr_set_timezone] = ACTIONS(1807),
    [sym_clkmgr_get_resync_period] = ACTIONS(1807),
    [sym_clkmgr_set_resync_period] = ACTIONS(1807),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1807),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1807),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1807),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1807),
    [sym_clkmgr_get_timeservers] = ACTIONS(1807),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1807),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1807),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1807),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1807),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1807),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1807),
    [sym_set_log_level] = ACTIONS(1807),
    [sym_get_log_level] = ACTIONS(1807),
    [sym_amx_log] = ACTIONS(1807),
    [sym_smtp_server_config_set] = ACTIONS(1807),
    [sym_smtp_server_config_get] = ACTIONS(1807),
    [sym_smtp_send] = ACTIONS(1807),
    [sym_validate_netlinx_account] = ACTIONS(1807),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1807),
    [sym_authenticate_certificate] = ACTIONS(1807),
    [sym_audit_netlinx_session_event] = ACTIONS(1807),
    [sym_audit_netlinx_generic_event] = ACTIONS(1807),
    [sym_get_available_flash_disk_space] = ACTIONS(1807),
    [sym_get_max_flash_disk_space] = ACTIONS(1807),
    [sym_data_create_feed] = ACTIONS(1807),
    [sym_data_delete_feed] = ACTIONS(1807),
    [sym_data_publish_feed] = ACTIONS(1807),
    [sym_data_get_published_feed] = ACTIONS(1807),
    [sym_data_add_record] = ACTIONS(1807),
    [sym_data_get_event_record] = ACTIONS(1807),
    [sym__wc_data_create_feed] = ACTIONS(1807),
    [sym__wc_data_add_record] = ACTIONS(1807),
    [sym__wc_data_get_event_record] = ACTIONS(1807),
    [sym_format] = ACTIONS(1807),
    [sym_astro_clock] = ACTIONS(1807),
    [sym_set_outdoor_temperature] = ACTIONS(1807),
    [sym_set_virtual_level_count] = ACTIONS(1807),
    [sym_set_virtual_channel_count] = ACTIONS(1807),
    [sym_set_virtual_port_count] = ACTIONS(1807),
    [sym__wc_encode_int] = ACTIONS(1807),
    [sym__wc_decode_int] = ACTIONS(1807),
    [sym__wc_int] = ACTIONS(1807),
    [sym__wc_to_ch_int] = ACTIONS(1807),
    [sym__ch_to_wc_int] = ACTIONS(1807),
    [sym__wc_find_string_int] = ACTIONS(1807),
    [sym__wc_left_string_int] = ACTIONS(1807),
    [sym__wc_lower_string_int] = ACTIONS(1807),
    [sym__wc_mid_string_int] = ACTIONS(1807),
    [sym__wc_remove_string_int] = ACTIONS(1807),
    [sym__wc_right_string_int] = ACTIONS(1807),
    [sym__wc_upper_string_int] = ACTIONS(1807),
    [sym__wc_compare_string_int] = ACTIONS(1807),
    [sym__wc_get_buffer_char_int] = ACTIONS(1807),
    [sym__wc_get_buffer_string_int] = ACTIONS(1807),
    [sym__wc_concat_string_int] = ACTIONS(1807),
    [sym__wc_file_open_int] = ACTIONS(1807),
    [sym__wc_file_close_int] = ACTIONS(1807),
    [sym__wc_file_read_int] = ACTIONS(1807),
    [sym__wc_file_read_line_int] = ACTIONS(1807),
    [sym__wc_file_write_int] = ACTIONS(1807),
    [sym__wc_file_write_line_int] = ACTIONS(1807),
    [aux_sym_preproc_include_token1] = ACTIONS(1807),
    [aux_sym_preproc_define_token1] = ACTIONS(1809),
    [aux_sym_preproc_warn_token1] = ACTIONS(1809),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1809),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1809),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1809),
    [anon_sym_LBRACK] = ACTIONS(1809),
    [anon_sym_LBRACE] = ACTIONS(1809),
    [anon_sym_LPAREN] = ACTIONS(1807),
    [anon_sym_BANG] = ACTIONS(1809),
    [anon_sym_TILDE] = ACTIONS(1809),
    [anon_sym_DASH] = ACTIONS(1807),
    [anon_sym_PLUS] = ACTIONS(1807),
    [anon_sym_DASH_DASH] = ACTIONS(1809),
    [anon_sym_PLUS_PLUS] = ACTIONS(1809),
    [anon_sym_DQUOTE] = ACTIONS(1809),
    [anon_sym_SQUOTE] = ACTIONS(1809),
    [sym_number_literal] = ACTIONS(1809),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(378)] = {
    [ts_builtin_sym_end] = ACTIONS(1813),
    [sym_identifier] = ACTIONS(1811),
    [sym_program_name_keyword] = ACTIONS(1811),
    [sym_module_name_keyword] = ACTIONS(1811),
    [sym_define_device_keyword] = ACTIONS(1811),
    [sym_define_combine_keyword] = ACTIONS(1811),
    [sym_define_constant_keyword] = ACTIONS(1811),
    [sym_define_type_keyword] = ACTIONS(1811),
    [sym_define_variable_keyword] = ACTIONS(1811),
    [sym_define_system_variable_keyword] = ACTIONS(1811),
    [sym_define_connect_level_keyword] = ACTIONS(1811),
    [sym_define_latching_keyword] = ACTIONS(1811),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1811),
    [sym_define_toggling_keyword] = ACTIONS(1811),
    [sym_define_call_keyword] = ACTIONS(1811),
    [sym_define_function_keyword] = ACTIONS(1811),
    [sym_define_library_function_keyword] = ACTIONS(1811),
    [sym_define_start_keyword] = ACTIONS(1811),
    [sym_define_event_keyword] = ACTIONS(1811),
    [sym_define_program_keyword] = ACTIONS(1811),
    [sym_define_module_keyword] = ACTIONS(1811),
    [sym_char_keyword] = ACTIONS(1811),
    [sym_widechar_keyword] = ACTIONS(1811),
    [sym_integer_keyword] = ACTIONS(1811),
    [sym_sinteger_keyword] = ACTIONS(1811),
    [sym_long_keyword] = ACTIONS(1811),
    [sym_slong_keyword] = ACTIONS(1811),
    [sym_float_keyword] = ACTIONS(1811),
    [sym_double_keyword] = ACTIONS(1811),
    [sym_variant_keyword] = ACTIONS(1811),
    [sym_variantarray_keyword] = ACTIONS(1811),
    [sym_local_var_keyword] = ACTIONS(1811),
    [sym_stack_var_keyword] = ACTIONS(1811),
    [sym_constant_keyword] = ACTIONS(1811),
    [sym_volatile_keyword] = ACTIONS(1811),
    [sym_non_volatile_keyword] = ACTIONS(1811),
    [sym_persistent_keyword] = ACTIONS(1811),
    [sym_dev_keyword] = ACTIONS(1811),
    [sym_devlev_keyword] = ACTIONS(1811),
    [sym_devchan_keyword] = ACTIONS(1811),
    [sym_if_keyword] = ACTIONS(1811),
    [sym_else_keyword] = ACTIONS(1811),
    [sym_switch_keyword] = ACTIONS(1811),
    [sym_case_keyword] = ACTIONS(1811),
    [sym_default_keyword] = ACTIONS(1811),
    [sym_while_keyword] = ACTIONS(1811),
    [sym_for_keyword] = ACTIONS(1811),
    [sym_break_keyword] = ACTIONS(1811),
    [sym_continue_keyword] = ACTIONS(1811),
    [sym_return_keyword] = ACTIONS(1811),
    [sym_select_keyword] = ACTIONS(1811),
    [sym_active_keyword] = ACTIONS(1811),
    [sym_struct_keyword] = ACTIONS(1811),
    [sym_structure_keyword] = ACTIONS(1811),
    [sym_button_event_keyword] = ACTIONS(1811),
    [sym_channel_event_keyword] = ACTIONS(1811),
    [sym_level_event_keyword] = ACTIONS(1811),
    [sym_data_event_keyword] = ACTIONS(1811),
    [sym_timeline_event_keyword] = ACTIONS(1811),
    [sym_custom_event_keyword] = ACTIONS(1811),
    [sym_push_keyword] = ACTIONS(1811),
    [sym_release_keyword] = ACTIONS(1811),
    [aux_sym_on_keyword_token1] = ACTIONS(1811),
    [aux_sym_off_keyword_token1] = ACTIONS(1811),
    [sym_send_level_keyword] = ACTIONS(1811),
    [sym_send_string_keyword] = ACTIONS(1811),
    [sym_send_command_keyword] = ACTIONS(1811),
    [sym_clear_buffer_keyword] = ACTIONS(1811),
    [sym_create_buffer_keyword] = ACTIONS(1811),
    [sym_create_multi_buffer_keyword] = ACTIONS(1811),
    [sym_call_keyword] = ACTIONS(1811),
    [sym_system_call_keyword] = ACTIONS(1811),
    [sym_devchan_to_keyword] = ACTIONS(1811),
    [sym_devchan_min_to_keyword] = ACTIONS(1811),
    [sym_devchan_total_off_keyword] = ACTIONS(1811),
    [sym_devchan_pulse_keyword] = ACTIONS(1811),
    [sym_wait_keyword] = ACTIONS(1811),
    [sym_cancel_wait_keyword] = ACTIONS(1811),
    [sym_wait_until_keyword] = ACTIONS(1811),
    [sym_cancel_wait_until_keyword] = ACTIONS(1811),
    [sym_cancel_all_wait_keyword] = ACTIONS(1811),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1811),
    [sym_not] = ACTIONS(1811),
    [sym_bnot] = ACTIONS(1811),
    [sym___file__] = ACTIONS(1811),
    [sym___date__] = ACTIONS(1811),
    [sym___ldate__] = ACTIONS(1811),
    [sym___time__] = ACTIONS(1811),
    [sym___line__] = ACTIONS(1811),
    [sym___name__] = ACTIONS(1811),
    [sym___netlinx__] = ACTIONS(1811),
    [sym_netlinx_axi_version] = ACTIONS(1811),
    [sym_url_flg_tcp] = ACTIONS(1811),
    [sym_url_flg_accinfopresent] = ACTIONS(1811),
    [sym_url_flg_temp] = ACTIONS(1811),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1811),
    [sym_url_flg_stat_mask] = ACTIONS(1811),
    [sym_url_flg_stat_lookup] = ACTIONS(1811),
    [sym_url_flg_stat_connecting] = ACTIONS(1811),
    [sym_url_flg_stat_waiting] = ACTIONS(1811),
    [sym_url_flg_stat_connected] = ACTIONS(1811),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1811),
    [sym_first_virtual_device] = ACTIONS(1811),
    [sym_first_local_port] = ACTIONS(1811),
    [sym_dynamic_virtual_device] = ACTIONS(1811),
    [sym_timeline_once] = ACTIONS(1811),
    [sym_timeline_repeat] = ACTIONS(1811),
    [sym_timeline_absolute] = ACTIONS(1811),
    [sym_timeline_relative] = ACTIONS(1811),
    [sym_do_push_timed_infinite] = ACTIONS(1811),
    [sym_source_type_no_address] = ACTIONS(1811),
    [sym_source_type_neuron_id] = ACTIONS(1811),
    [sym_source_type_ip_address] = ACTIONS(1811),
    [sym_source_type_axlink] = ACTIONS(1811),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1811),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1811),
    [sym_source_type_ip_socket_address] = ACTIONS(1811),
    [sym_source_type_rs232] = ACTIONS(1811),
    [sym_source_type_internal] = ACTIONS(1811),
    [sym_source_type_mac_address] = ACTIONS(1811),
    [sym_source_type_ipv4_port] = ACTIONS(1811),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1811),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1811),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1811),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1811),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1811),
    [sym_true] = ACTIONS(1811),
    [sym_false] = ACTIONS(1811),
    [sym_file_read_only] = ACTIONS(1811),
    [sym_file_rw_new] = ACTIONS(1811),
    [sym_file_rw_append] = ACTIONS(1811),
    [sym_ip_tcp] = ACTIONS(1811),
    [sym_ip_udp] = ACTIONS(1811),
    [sym_ip_udp_2way] = ACTIONS(1811),
    [sym_xml_encode_types] = ACTIONS(1811),
    [sym_xml_encode_char_as_list] = ACTIONS(1811),
    [sym_xml_encode_le] = ACTIONS(1811),
    [sym_xml_decode_types] = ACTIONS(1811),
    [sym_xml_decode_no_preserve] = ACTIONS(1811),
    [sym_duet_dev_type_display_device] = ACTIONS(1811),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1811),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1811),
    [sym_duet_dev_type_io_device] = ACTIONS(1811),
    [sym_duet_dev_type_relay_device] = ACTIONS(1811),
    [sym_duet_dev_type_ups] = ACTIONS(1811),
    [sym_duet_dev_type_amplifier] = ACTIONS(1811),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1811),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1811),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1811),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1811),
    [sym_duet_dev_type_camera] = ACTIONS(1811),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1811),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1811),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1811),
    [sym_duet_dev_type_dss] = ACTIONS(1811),
    [sym_duet_dev_type_dvr] = ACTIONS(1811),
    [sym_duet_dev_type_disc_device] = ACTIONS(1811),
    [sym_duet_dev_type_document_camera] = ACTIONS(1811),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1811),
    [sym_duet_dev_type_hvac] = ACTIONS(1811),
    [sym_duet_dev_type_keypad] = ACTIONS(1811),
    [sym_duet_dev_type_light] = ACTIONS(1811),
    [sym_duet_dev_type_monitor] = ACTIONS(1811),
    [sym_duet_dev_type_motor] = ACTIONS(1811),
    [sym_duet_dev_type_multi_window] = ACTIONS(1811),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1811),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1811),
    [sym_duet_dev_type_receiver] = ACTIONS(1811),
    [sym_duet_dev_type_security_system] = ACTIONS(1811),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1811),
    [sym_duet_dev_type_settop_box] = ACTIONS(1811),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1811),
    [sym_duet_dev_type_switcher] = ACTIONS(1811),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1811),
    [sym_duet_dev_type_tv] = ACTIONS(1811),
    [sym_duet_dev_type_utility] = ACTIONS(1811),
    [sym_duet_dev_type_vcr] = ACTIONS(1811),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1811),
    [sym_duet_dev_type_video_processor] = ACTIONS(1811),
    [sym_duet_dev_type_video_projector] = ACTIONS(1811),
    [sym_duet_dev_type_video_wall] = ACTIONS(1811),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1811),
    [sym_duet_dev_type_weather] = ACTIONS(1811),
    [sym_duet_dev_not_polled] = ACTIONS(1811),
    [sym_duet_dev_polled] = ACTIONS(1811),
    [sym_clkmgr_mode_network] = ACTIONS(1811),
    [sym_clkmgr_mode_standalone] = ACTIONS(1811),
    [sym_normal_standby] = ACTIONS(1811),
    [sym_normal_wake] = ACTIONS(1811),
    [sym_ip_multicast_ttl_option] = ACTIONS(1811),
    [sym_ip_ttl_subnet] = ACTIONS(1811),
    [sym_ip_ttl_site] = ACTIONS(1811),
    [sym_ip_ttl_region] = ACTIONS(1811),
    [sym_ip_ttl_continent] = ACTIONS(1811),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1811),
    [sym_ip_nodelay_on] = ACTIONS(1811),
    [sym_ip_nodelay_off] = ACTIONS(1811),
    [sym_amx_error] = ACTIONS(1811),
    [sym_amx_warning] = ACTIONS(1811),
    [sym_amx_info] = ACTIONS(1811),
    [sym_amx_debug] = ACTIONS(1811),
    [sym_netlinx_login_success] = ACTIONS(1811),
    [sym_netlinx_login_fail] = ACTIONS(1811),
    [sym_netlinx_logout] = ACTIONS(1811),
    [sym_null_str] = ACTIONS(1811),
    [sym_smtp_address] = ACTIONS(1811),
    [sym_smtp_port_number] = ACTIONS(1811),
    [sym_smtp_username] = ACTIONS(1811),
    [sym_smtp_password] = ACTIONS(1811),
    [sym_smtp_require_tls] = ACTIONS(1811),
    [sym_smtp_from] = ACTIONS(1811),
    [sym_smtp_tls_true] = ACTIONS(1811),
    [sym_smtp_tls_false] = ACTIONS(1811),
    [sym_listview_on_row_select_event] = ACTIONS(1811),
    [sym_data_structure_unknown] = ACTIONS(1811),
    [sym_data_structure_datafeed] = ACTIONS(1811),
    [sym_data_structure_datarecordset] = ACTIONS(1811),
    [sym_data_structure_datarecord] = ACTIONS(1811),
    [sym_data_structure_datafield] = ACTIONS(1811),
    [sym_data_max_content_field_count] = ACTIONS(1811),
    [sym_data_max_metadata_field_count] = ACTIONS(1811),
    [sym_data_max_id_length] = ACTIONS(1811),
    [sym_data_max_name_length] = ACTIONS(1811),
    [sym_data_max_description_length] = ACTIONS(1811),
    [sym_data_max_type_length] = ACTIONS(1811),
    [sym_data_max_format_length] = ACTIONS(1811),
    [sym_data_max_source_length] = ACTIONS(1811),
    [sym_data_max_value_length] = ACTIONS(1811),
    [sym_data_max_label_length] = ACTIONS(1811),
    [sym_data_max_url_length] = ACTIONS(1811),
    [sym_max_last_login_info_length] = ACTIONS(1811),
    [sym_data_type_unknown] = ACTIONS(1811),
    [sym_data_type_string] = ACTIONS(1811),
    [sym_data_type_datetime] = ACTIONS(1811),
    [sym_data_type_date] = ACTIONS(1811),
    [sym_data_type_time] = ACTIONS(1811),
    [sym_data_type_image] = ACTIONS(1811),
    [sym_data_format_url] = ACTIONS(1811),
    [sym_data_format_phone] = ACTIONS(1811),
    [sym_data_format_email] = ACTIONS(1811),
    [sym_data_format_iso8601] = ACTIONS(1811),
    [sym_auditlog_priv] = ACTIONS(1811),
    [sym_configuration_priv] = ACTIONS(1811),
    [sym_deviceconfig_priv] = ACTIONS(1811),
    [sym_ftp_priv] = ACTIONS(1811),
    [sym_http_priv] = ACTIONS(1811),
    [sym_networkconfig_priv] = ACTIONS(1811),
    [sym_programport_priv] = ACTIONS(1811),
    [sym_remoteui_priv] = ACTIONS(1811),
    [sym_securitycontrol_priv] = ACTIONS(1811),
    [sym_softwaremanagement_priv] = ACTIONS(1811),
    [sym_terminal_priv] = ACTIONS(1811),
    [sym_tpadmin_priv] = ACTIONS(1811),
    [sym_user1_priv] = ACTIONS(1811),
    [sym_user2_priv] = ACTIONS(1811),
    [sym_user3_priv] = ACTIONS(1811),
    [sym_user4_priv] = ACTIONS(1811),
    [sym_usermanagement_priv] = ACTIONS(1811),
    [sym_valid_account] = ACTIONS(1811),
    [sym_err_name_invalid] = ACTIONS(1811),
    [sym_err_password_invalid] = ACTIONS(1811),
    [sym_err_invalid_account] = ACTIONS(1811),
    [sym_err_no_authorization] = ACTIONS(1811),
    [sym_err_invalid_parameter] = ACTIONS(1811),
    [sym_err_user_locked_out] = ACTIONS(1811),
    [sym_err_user_account_expired] = ACTIONS(1811),
    [sym_err_general_error] = ACTIONS(1811),
    [sym_err_authentication_error] = ACTIONS(1811),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1811),
    [sym_tls_validate_certificate] = ACTIONS(1811),
    [sym_tdata] = ACTIONS(1811),
    [sym_tchannel] = ACTIONS(1811),
    [sym_tlevel] = ACTIONS(1811),
    [sym_tbutton] = ACTIONS(1811),
    [sym_ttimeline] = ACTIONS(1811),
    [sym_tcustom] = ACTIONS(1811),
    [sym_url_struct] = ACTIONS(1811),
    [sym_dns_struct] = ACTIONS(1811),
    [sym_ip_address_struct] = ACTIONS(1811),
    [sym_dev_info_struct] = ACTIONS(1811),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1811),
    [sym_data_feed] = ACTIONS(1811),
    [sym_data_field] = ACTIONS(1811),
    [sym_data_record] = ACTIONS(1811),
    [sym_wc_data_feed] = ACTIONS(1811),
    [sym_wc_data_field] = ACTIONS(1811),
    [sym_wc_data_record] = ACTIONS(1811),
    [sym_last_login_info] = ACTIONS(1811),
    [sym____reserved___] = ACTIONS(1811),
    [sym_date] = ACTIONS(1811),
    [sym_day] = ACTIONS(1811),
    [sym_time] = ACTIONS(1811),
    [sym_ldate] = ACTIONS(1811),
    [sym_push_device] = ACTIONS(1811),
    [sym_push_channel] = ACTIONS(1811),
    [sym_push_devchan] = ACTIONS(1811),
    [sym_release_device] = ACTIONS(1811),
    [sym_release_channel] = ACTIONS(1811),
    [sym_release_devchan] = ACTIONS(1811),
    [sym_master_slot] = ACTIONS(1811),
    [sym_get_pulse_time] = ACTIONS(1811),
    [sym_get_timer] = ACTIONS(1811),
    [sym_master_sn] = ACTIONS(1811),
    [sym_system_number] = ACTIONS(1811),
    [sym_button] = ACTIONS(1811),
    [sym_dv_channel] = ACTIONS(1811),
    [sym_data] = ACTIONS(1811),
    [sym_level] = ACTIONS(1811),
    [sym_timeline] = ACTIONS(1811),
    [sym_channel] = ACTIONS(1811),
    [sym_custom] = ACTIONS(1811),
    [sym_atoi] = ACTIONS(1811),
    [sym_atol] = ACTIONS(1811),
    [sym_atof] = ACTIONS(1811),
    [sym_itoa] = ACTIONS(1811),
    [sym_itohex] = ACTIONS(1811),
    [sym_hextoi] = ACTIONS(1811),
    [sym_ftoa] = ACTIONS(1811),
    [sym_find_string] = ACTIONS(1811),
    [sym_left_string] = ACTIONS(1811),
    [sym_right_string] = ACTIONS(1811),
    [sym_mid_string] = ACTIONS(1811),
    [sym_remove_string] = ACTIONS(1811),
    [sym_get_buffer_string] = ACTIONS(1811),
    [sym_get_buffer_char] = ACTIONS(1811),
    [sym_get_multi_buffer_string] = ACTIONS(1811),
    [sym_length_string] = ACTIONS(1811),
    [sym_max_length_string] = ACTIONS(1811),
    [sym_set_length_string] = ACTIONS(1811),
    [sym_compare_string] = ACTIONS(1811),
    [sym_upper_string] = ACTIONS(1811),
    [sym_lower_string] = ACTIONS(1811),
    [sym_get_last] = ACTIONS(1811),
    [sym_date_to_day] = ACTIONS(1811),
    [sym_date_to_month] = ACTIONS(1811),
    [sym_date_to_year] = ACTIONS(1811),
    [sym_time_to_hour] = ACTIONS(1811),
    [sym_time_to_minute] = ACTIONS(1811),
    [sym_time_to_second] = ACTIONS(1811),
    [sym_day_of_week] = ACTIONS(1811),
    [sym_set_pulse_time] = ACTIONS(1811),
    [sym_set_timer] = ACTIONS(1811),
    [sym_file_open] = ACTIONS(1811),
    [sym_file_close] = ACTIONS(1811),
    [sym_file_read] = ACTIONS(1811),
    [sym_file_read_line] = ACTIONS(1811),
    [sym_file_write] = ACTIONS(1811),
    [sym_file_write_line] = ACTIONS(1811),
    [sym_file_copy] = ACTIONS(1811),
    [sym_file_delete] = ACTIONS(1811),
    [sym_file_rename] = ACTIONS(1811),
    [sym_file_dir] = ACTIONS(1811),
    [sym_file_getdir] = ACTIONS(1811),
    [sym_file_setdir] = ACTIONS(1811),
    [sym_file_create_dir] = ACTIONS(1811),
    [sym_file_remove_dir] = ACTIONS(1811),
    [sym_file_seek] = ACTIONS(1811),
    [sym_length_array] = ACTIONS(1811),
    [sym_max_length_array] = ACTIONS(1811),
    [sym_set_length_array] = ACTIONS(1811),
    [sym_ip_client_open] = ACTIONS(1811),
    [sym_ip_client_close] = ACTIONS(1811),
    [sym_ip_server_open] = ACTIONS(1811),
    [sym_ip_server_close] = ACTIONS(1811),
    [sym_ip_bound_client_open] = ACTIONS(1811),
    [sym_ip_mc_server_open] = ACTIONS(1811),
    [sym_ip_set_option] = ACTIONS(1811),
    [sym_ssh_client_open] = ACTIONS(1811),
    [sym_ssh_client_close] = ACTIONS(1811),
    [sym_tls_client_open] = ACTIONS(1811),
    [sym_tls_client_close] = ACTIONS(1811),
    [sym_do_push] = ACTIONS(1811),
    [sym_do_push_timed] = ACTIONS(1811),
    [sym_do_release] = ACTIONS(1811),
    [sym_device_id] = ACTIONS(1811),
    [sym_device_info] = ACTIONS(1811),
    [sym_device_id_string] = ACTIONS(1811),
    [sym_device_standby] = ACTIONS(1811),
    [sym_device_wake] = ACTIONS(1811),
    [sym_timeline_create] = ACTIONS(1811),
    [sym_timeline_kill] = ACTIONS(1811),
    [sym_timeline_reload] = ACTIONS(1811),
    [sym_timeline_pause] = ACTIONS(1811),
    [sym_timeline_restart] = ACTIONS(1811),
    [sym_timeline_set] = ACTIONS(1811),
    [sym_timeline_get] = ACTIONS(1811),
    [sym_timeline_active] = ACTIONS(1811),
    [sym_timeline_dynamic_id] = ACTIONS(1811),
    [sym_variable_to_string] = ACTIONS(1811),
    [sym_string_to_variable] = ACTIONS(1811),
    [sym_length_variable_to_string] = ACTIONS(1811),
    [sym_type_cast] = ACTIONS(1811),
    [sym_variable_to_xml] = ACTIONS(1811),
    [sym_xml_to_variable] = ACTIONS(1811),
    [sym_length_variable_to_xml] = ACTIONS(1811),
    [sym_redirect_string] = ACTIONS(1811),
    [sym_random_number] = ACTIONS(1811),
    [sym_get_unique_id] = ACTIONS(1811),
    [sym_get_system_number] = ACTIONS(1811),
    [sym_set_system_number] = ACTIONS(1811),
    [sym_get_serial_number] = ACTIONS(1811),
    [sym_get_master_build] = ACTIONS(1811),
    [sym_set_validation_code] = ACTIONS(1811),
    [sym_reboot] = ACTIONS(1811),
    [sym_load_duet_module] = ACTIONS(1811),
    [sym_unload_duet_module] = ACTIONS(1811),
    [sym_duet_mem_size_set] = ACTIONS(1811),
    [sym_duet_mem_size_get] = ACTIONS(1811),
    [sym_dynamic_application_device] = ACTIONS(1811),
    [sym_dynamic_polled_port] = ACTIONS(1811),
    [sym_static_port_binding] = ACTIONS(1811),
    [sym_static_ip_binding] = ACTIONS(1811),
    [sym_get_dns_list] = ACTIONS(1811),
    [sym_set_dns_list] = ACTIONS(1811),
    [sym_get_ip_address] = ACTIONS(1811),
    [sym_set_ip_address] = ACTIONS(1811),
    [sym_get_url_list] = ACTIONS(1811),
    [sym_add_url_entry] = ACTIONS(1811),
    [sym_delete_url_entry] = ACTIONS(1811),
    [sym_abs_value] = ACTIONS(1811),
    [sym_max_value] = ACTIONS(1811),
    [sym_min_value] = ACTIONS(1811),
    [sym_exp_value] = ACTIONS(1811),
    [sym_log_value] = ACTIONS(1811),
    [sym_log10_value] = ACTIONS(1811),
    [sym_power_value] = ACTIONS(1811),
    [sym_sqrt_value] = ACTIONS(1811),
    [sym_raw_be] = ACTIONS(1811),
    [sym_raw_le] = ACTIONS(1811),
    [sym_internal_queue_size_set] = ACTIONS(1811),
    [sym_internal_queue_size_get] = ACTIONS(1811),
    [sym_internal_threshold_set] = ACTIONS(1811),
    [sym_internal_threshold_get] = ACTIONS(1811),
    [sym_rebuilt_event] = ACTIONS(1811),
    [sym_do_custom_event] = ACTIONS(1811),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1811),
    [sym_clkmgr_set_clk_source] = ACTIONS(1811),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1811),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1811),
    [sym_clkmgr_get_timezone] = ACTIONS(1811),
    [sym_clkmgr_set_timezone] = ACTIONS(1811),
    [sym_clkmgr_get_resync_period] = ACTIONS(1811),
    [sym_clkmgr_set_resync_period] = ACTIONS(1811),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1811),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1811),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1811),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1811),
    [sym_clkmgr_get_timeservers] = ACTIONS(1811),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1811),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1811),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1811),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1811),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1811),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1811),
    [sym_set_log_level] = ACTIONS(1811),
    [sym_get_log_level] = ACTIONS(1811),
    [sym_amx_log] = ACTIONS(1811),
    [sym_smtp_server_config_set] = ACTIONS(1811),
    [sym_smtp_server_config_get] = ACTIONS(1811),
    [sym_smtp_send] = ACTIONS(1811),
    [sym_validate_netlinx_account] = ACTIONS(1811),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1811),
    [sym_authenticate_certificate] = ACTIONS(1811),
    [sym_audit_netlinx_session_event] = ACTIONS(1811),
    [sym_audit_netlinx_generic_event] = ACTIONS(1811),
    [sym_get_available_flash_disk_space] = ACTIONS(1811),
    [sym_get_max_flash_disk_space] = ACTIONS(1811),
    [sym_data_create_feed] = ACTIONS(1811),
    [sym_data_delete_feed] = ACTIONS(1811),
    [sym_data_publish_feed] = ACTIONS(1811),
    [sym_data_get_published_feed] = ACTIONS(1811),
    [sym_data_add_record] = ACTIONS(1811),
    [sym_data_get_event_record] = ACTIONS(1811),
    [sym__wc_data_create_feed] = ACTIONS(1811),
    [sym__wc_data_add_record] = ACTIONS(1811),
    [sym__wc_data_get_event_record] = ACTIONS(1811),
    [sym_format] = ACTIONS(1811),
    [sym_astro_clock] = ACTIONS(1811),
    [sym_set_outdoor_temperature] = ACTIONS(1811),
    [sym_set_virtual_level_count] = ACTIONS(1811),
    [sym_set_virtual_channel_count] = ACTIONS(1811),
    [sym_set_virtual_port_count] = ACTIONS(1811),
    [sym__wc_encode_int] = ACTIONS(1811),
    [sym__wc_decode_int] = ACTIONS(1811),
    [sym__wc_int] = ACTIONS(1811),
    [sym__wc_to_ch_int] = ACTIONS(1811),
    [sym__ch_to_wc_int] = ACTIONS(1811),
    [sym__wc_find_string_int] = ACTIONS(1811),
    [sym__wc_left_string_int] = ACTIONS(1811),
    [sym__wc_lower_string_int] = ACTIONS(1811),
    [sym__wc_mid_string_int] = ACTIONS(1811),
    [sym__wc_remove_string_int] = ACTIONS(1811),
    [sym__wc_right_string_int] = ACTIONS(1811),
    [sym__wc_upper_string_int] = ACTIONS(1811),
    [sym__wc_compare_string_int] = ACTIONS(1811),
    [sym__wc_get_buffer_char_int] = ACTIONS(1811),
    [sym__wc_get_buffer_string_int] = ACTIONS(1811),
    [sym__wc_concat_string_int] = ACTIONS(1811),
    [sym__wc_file_open_int] = ACTIONS(1811),
    [sym__wc_file_close_int] = ACTIONS(1811),
    [sym__wc_file_read_int] = ACTIONS(1811),
    [sym__wc_file_read_line_int] = ACTIONS(1811),
    [sym__wc_file_write_int] = ACTIONS(1811),
    [sym__wc_file_write_line_int] = ACTIONS(1811),
    [aux_sym_preproc_include_token1] = ACTIONS(1811),
    [aux_sym_preproc_define_token1] = ACTIONS(1813),
    [aux_sym_preproc_warn_token1] = ACTIONS(1813),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1813),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1813),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1813),
    [anon_sym_LBRACK] = ACTIONS(1813),
    [anon_sym_LBRACE] = ACTIONS(1813),
    [anon_sym_LPAREN] = ACTIONS(1811),
    [anon_sym_BANG] = ACTIONS(1813),
    [anon_sym_TILDE] = ACTIONS(1813),
    [anon_sym_DASH] = ACTIONS(1811),
    [anon_sym_PLUS] = ACTIONS(1811),
    [anon_sym_DASH_DASH] = ACTIONS(1813),
    [anon_sym_PLUS_PLUS] = ACTIONS(1813),
    [anon_sym_DQUOTE] = ACTIONS(1813),
    [anon_sym_SQUOTE] = ACTIONS(1813),
    [sym_number_literal] = ACTIONS(1813),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(379)] = {
    [sym_identifier] = ACTIONS(2121),
    [sym_program_name_keyword] = ACTIONS(2121),
    [sym_module_name_keyword] = ACTIONS(2121),
    [sym_define_device_keyword] = ACTIONS(2121),
    [sym_define_combine_keyword] = ACTIONS(2121),
    [sym_define_constant_keyword] = ACTIONS(2121),
    [sym_define_type_keyword] = ACTIONS(2121),
    [sym_define_variable_keyword] = ACTIONS(2121),
    [sym_define_system_variable_keyword] = ACTIONS(2121),
    [sym_define_connect_level_keyword] = ACTIONS(2121),
    [sym_define_latching_keyword] = ACTIONS(2121),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2121),
    [sym_define_toggling_keyword] = ACTIONS(2121),
    [sym_define_call_keyword] = ACTIONS(2121),
    [sym_define_function_keyword] = ACTIONS(2121),
    [sym_define_library_function_keyword] = ACTIONS(2121),
    [sym_define_start_keyword] = ACTIONS(2121),
    [sym_define_event_keyword] = ACTIONS(2121),
    [sym_define_program_keyword] = ACTIONS(2121),
    [sym_define_module_keyword] = ACTIONS(2121),
    [sym_char_keyword] = ACTIONS(2121),
    [sym_widechar_keyword] = ACTIONS(2121),
    [sym_integer_keyword] = ACTIONS(2121),
    [sym_sinteger_keyword] = ACTIONS(2121),
    [sym_long_keyword] = ACTIONS(2121),
    [sym_slong_keyword] = ACTIONS(2121),
    [sym_float_keyword] = ACTIONS(2121),
    [sym_double_keyword] = ACTIONS(2121),
    [sym_variant_keyword] = ACTIONS(2121),
    [sym_variantarray_keyword] = ACTIONS(2121),
    [sym_local_var_keyword] = ACTIONS(2121),
    [sym_stack_var_keyword] = ACTIONS(2121),
    [sym_constant_keyword] = ACTIONS(2121),
    [sym_volatile_keyword] = ACTIONS(2121),
    [sym_non_volatile_keyword] = ACTIONS(2121),
    [sym_persistent_keyword] = ACTIONS(2121),
    [sym_dev_keyword] = ACTIONS(2121),
    [sym_devlev_keyword] = ACTIONS(2121),
    [sym_devchan_keyword] = ACTIONS(2121),
    [sym_if_keyword] = ACTIONS(2121),
    [sym_switch_keyword] = ACTIONS(2121),
    [sym_case_keyword] = ACTIONS(2121),
    [sym_default_keyword] = ACTIONS(2121),
    [sym_while_keyword] = ACTIONS(2121),
    [sym_for_keyword] = ACTIONS(2121),
    [sym_break_keyword] = ACTIONS(2121),
    [sym_continue_keyword] = ACTIONS(2121),
    [sym_return_keyword] = ACTIONS(2121),
    [sym_select_keyword] = ACTIONS(2121),
    [sym_active_keyword] = ACTIONS(2121),
    [sym_struct_keyword] = ACTIONS(2121),
    [sym_structure_keyword] = ACTIONS(2121),
    [sym_button_event_keyword] = ACTIONS(2121),
    [sym_channel_event_keyword] = ACTIONS(2121),
    [sym_level_event_keyword] = ACTIONS(2121),
    [sym_data_event_keyword] = ACTIONS(2121),
    [sym_timeline_event_keyword] = ACTIONS(2121),
    [sym_custom_event_keyword] = ACTIONS(2121),
    [sym_push_keyword] = ACTIONS(2121),
    [sym_release_keyword] = ACTIONS(2121),
    [aux_sym_on_keyword_token1] = ACTIONS(2121),
    [aux_sym_off_keyword_token1] = ACTIONS(2121),
    [sym_send_level_keyword] = ACTIONS(2121),
    [sym_send_string_keyword] = ACTIONS(2121),
    [sym_send_command_keyword] = ACTIONS(2121),
    [sym_clear_buffer_keyword] = ACTIONS(2121),
    [sym_create_buffer_keyword] = ACTIONS(2121),
    [sym_create_multi_buffer_keyword] = ACTIONS(2121),
    [sym_call_keyword] = ACTIONS(2121),
    [sym_system_call_keyword] = ACTIONS(2121),
    [sym_devchan_to_keyword] = ACTIONS(2121),
    [sym_devchan_min_to_keyword] = ACTIONS(2121),
    [sym_devchan_total_off_keyword] = ACTIONS(2121),
    [sym_devchan_pulse_keyword] = ACTIONS(2121),
    [sym_wait_keyword] = ACTIONS(2121),
    [sym_cancel_wait_keyword] = ACTIONS(2121),
    [sym_wait_until_keyword] = ACTIONS(2121),
    [sym_cancel_wait_until_keyword] = ACTIONS(2121),
    [sym_cancel_all_wait_keyword] = ACTIONS(2121),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2121),
    [sym_not] = ACTIONS(2121),
    [sym_bnot] = ACTIONS(2121),
    [sym___file__] = ACTIONS(2121),
    [sym___date__] = ACTIONS(2121),
    [sym___ldate__] = ACTIONS(2121),
    [sym___time__] = ACTIONS(2121),
    [sym___line__] = ACTIONS(2121),
    [sym___name__] = ACTIONS(2121),
    [sym___netlinx__] = ACTIONS(2121),
    [sym_netlinx_axi_version] = ACTIONS(2121),
    [sym_url_flg_tcp] = ACTIONS(2121),
    [sym_url_flg_accinfopresent] = ACTIONS(2121),
    [sym_url_flg_temp] = ACTIONS(2121),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2121),
    [sym_url_flg_stat_mask] = ACTIONS(2121),
    [sym_url_flg_stat_lookup] = ACTIONS(2121),
    [sym_url_flg_stat_connecting] = ACTIONS(2121),
    [sym_url_flg_stat_waiting] = ACTIONS(2121),
    [sym_url_flg_stat_connected] = ACTIONS(2121),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2121),
    [sym_first_virtual_device] = ACTIONS(2121),
    [sym_first_local_port] = ACTIONS(2121),
    [sym_dynamic_virtual_device] = ACTIONS(2121),
    [sym_timeline_once] = ACTIONS(2121),
    [sym_timeline_repeat] = ACTIONS(2121),
    [sym_timeline_absolute] = ACTIONS(2121),
    [sym_timeline_relative] = ACTIONS(2121),
    [sym_do_push_timed_infinite] = ACTIONS(2121),
    [sym_source_type_no_address] = ACTIONS(2121),
    [sym_source_type_neuron_id] = ACTIONS(2121),
    [sym_source_type_ip_address] = ACTIONS(2121),
    [sym_source_type_axlink] = ACTIONS(2121),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2121),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2121),
    [sym_source_type_ip_socket_address] = ACTIONS(2121),
    [sym_source_type_rs232] = ACTIONS(2121),
    [sym_source_type_internal] = ACTIONS(2121),
    [sym_source_type_mac_address] = ACTIONS(2121),
    [sym_source_type_ipv4_port] = ACTIONS(2121),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2121),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2121),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2121),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2121),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2121),
    [sym_true] = ACTIONS(2121),
    [sym_false] = ACTIONS(2121),
    [sym_file_read_only] = ACTIONS(2121),
    [sym_file_rw_new] = ACTIONS(2121),
    [sym_file_rw_append] = ACTIONS(2121),
    [sym_ip_tcp] = ACTIONS(2121),
    [sym_ip_udp] = ACTIONS(2121),
    [sym_ip_udp_2way] = ACTIONS(2121),
    [sym_xml_encode_types] = ACTIONS(2121),
    [sym_xml_encode_char_as_list] = ACTIONS(2121),
    [sym_xml_encode_le] = ACTIONS(2121),
    [sym_xml_decode_types] = ACTIONS(2121),
    [sym_xml_decode_no_preserve] = ACTIONS(2121),
    [sym_duet_dev_type_display_device] = ACTIONS(2121),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2121),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2121),
    [sym_duet_dev_type_io_device] = ACTIONS(2121),
    [sym_duet_dev_type_relay_device] = ACTIONS(2121),
    [sym_duet_dev_type_ups] = ACTIONS(2121),
    [sym_duet_dev_type_amplifier] = ACTIONS(2121),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2121),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2121),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2121),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2121),
    [sym_duet_dev_type_camera] = ACTIONS(2121),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2121),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2121),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2121),
    [sym_duet_dev_type_dss] = ACTIONS(2121),
    [sym_duet_dev_type_dvr] = ACTIONS(2121),
    [sym_duet_dev_type_disc_device] = ACTIONS(2121),
    [sym_duet_dev_type_document_camera] = ACTIONS(2121),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2121),
    [sym_duet_dev_type_hvac] = ACTIONS(2121),
    [sym_duet_dev_type_keypad] = ACTIONS(2121),
    [sym_duet_dev_type_light] = ACTIONS(2121),
    [sym_duet_dev_type_monitor] = ACTIONS(2121),
    [sym_duet_dev_type_motor] = ACTIONS(2121),
    [sym_duet_dev_type_multi_window] = ACTIONS(2121),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2121),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2121),
    [sym_duet_dev_type_receiver] = ACTIONS(2121),
    [sym_duet_dev_type_security_system] = ACTIONS(2121),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2121),
    [sym_duet_dev_type_settop_box] = ACTIONS(2121),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2121),
    [sym_duet_dev_type_switcher] = ACTIONS(2121),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2121),
    [sym_duet_dev_type_tv] = ACTIONS(2121),
    [sym_duet_dev_type_utility] = ACTIONS(2121),
    [sym_duet_dev_type_vcr] = ACTIONS(2121),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2121),
    [sym_duet_dev_type_video_processor] = ACTIONS(2121),
    [sym_duet_dev_type_video_projector] = ACTIONS(2121),
    [sym_duet_dev_type_video_wall] = ACTIONS(2121),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2121),
    [sym_duet_dev_type_weather] = ACTIONS(2121),
    [sym_duet_dev_not_polled] = ACTIONS(2121),
    [sym_duet_dev_polled] = ACTIONS(2121),
    [sym_clkmgr_mode_network] = ACTIONS(2121),
    [sym_clkmgr_mode_standalone] = ACTIONS(2121),
    [sym_normal_standby] = ACTIONS(2121),
    [sym_normal_wake] = ACTIONS(2121),
    [sym_ip_multicast_ttl_option] = ACTIONS(2121),
    [sym_ip_ttl_subnet] = ACTIONS(2121),
    [sym_ip_ttl_site] = ACTIONS(2121),
    [sym_ip_ttl_region] = ACTIONS(2121),
    [sym_ip_ttl_continent] = ACTIONS(2121),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2121),
    [sym_ip_nodelay_on] = ACTIONS(2121),
    [sym_ip_nodelay_off] = ACTIONS(2121),
    [sym_amx_error] = ACTIONS(2121),
    [sym_amx_warning] = ACTIONS(2121),
    [sym_amx_info] = ACTIONS(2121),
    [sym_amx_debug] = ACTIONS(2121),
    [sym_netlinx_login_success] = ACTIONS(2121),
    [sym_netlinx_login_fail] = ACTIONS(2121),
    [sym_netlinx_logout] = ACTIONS(2121),
    [sym_null_str] = ACTIONS(2121),
    [sym_smtp_address] = ACTIONS(2121),
    [sym_smtp_port_number] = ACTIONS(2121),
    [sym_smtp_username] = ACTIONS(2121),
    [sym_smtp_password] = ACTIONS(2121),
    [sym_smtp_require_tls] = ACTIONS(2121),
    [sym_smtp_from] = ACTIONS(2121),
    [sym_smtp_tls_true] = ACTIONS(2121),
    [sym_smtp_tls_false] = ACTIONS(2121),
    [sym_listview_on_row_select_event] = ACTIONS(2121),
    [sym_data_structure_unknown] = ACTIONS(2121),
    [sym_data_structure_datafeed] = ACTIONS(2121),
    [sym_data_structure_datarecordset] = ACTIONS(2121),
    [sym_data_structure_datarecord] = ACTIONS(2121),
    [sym_data_structure_datafield] = ACTIONS(2121),
    [sym_data_max_content_field_count] = ACTIONS(2121),
    [sym_data_max_metadata_field_count] = ACTIONS(2121),
    [sym_data_max_id_length] = ACTIONS(2121),
    [sym_data_max_name_length] = ACTIONS(2121),
    [sym_data_max_description_length] = ACTIONS(2121),
    [sym_data_max_type_length] = ACTIONS(2121),
    [sym_data_max_format_length] = ACTIONS(2121),
    [sym_data_max_source_length] = ACTIONS(2121),
    [sym_data_max_value_length] = ACTIONS(2121),
    [sym_data_max_label_length] = ACTIONS(2121),
    [sym_data_max_url_length] = ACTIONS(2121),
    [sym_max_last_login_info_length] = ACTIONS(2121),
    [sym_data_type_unknown] = ACTIONS(2121),
    [sym_data_type_string] = ACTIONS(2121),
    [sym_data_type_datetime] = ACTIONS(2121),
    [sym_data_type_date] = ACTIONS(2121),
    [sym_data_type_time] = ACTIONS(2121),
    [sym_data_type_image] = ACTIONS(2121),
    [sym_data_format_url] = ACTIONS(2121),
    [sym_data_format_phone] = ACTIONS(2121),
    [sym_data_format_email] = ACTIONS(2121),
    [sym_data_format_iso8601] = ACTIONS(2121),
    [sym_auditlog_priv] = ACTIONS(2121),
    [sym_configuration_priv] = ACTIONS(2121),
    [sym_deviceconfig_priv] = ACTIONS(2121),
    [sym_ftp_priv] = ACTIONS(2121),
    [sym_http_priv] = ACTIONS(2121),
    [sym_networkconfig_priv] = ACTIONS(2121),
    [sym_programport_priv] = ACTIONS(2121),
    [sym_remoteui_priv] = ACTIONS(2121),
    [sym_securitycontrol_priv] = ACTIONS(2121),
    [sym_softwaremanagement_priv] = ACTIONS(2121),
    [sym_terminal_priv] = ACTIONS(2121),
    [sym_tpadmin_priv] = ACTIONS(2121),
    [sym_user1_priv] = ACTIONS(2121),
    [sym_user2_priv] = ACTIONS(2121),
    [sym_user3_priv] = ACTIONS(2121),
    [sym_user4_priv] = ACTIONS(2121),
    [sym_usermanagement_priv] = ACTIONS(2121),
    [sym_valid_account] = ACTIONS(2121),
    [sym_err_name_invalid] = ACTIONS(2121),
    [sym_err_password_invalid] = ACTIONS(2121),
    [sym_err_invalid_account] = ACTIONS(2121),
    [sym_err_no_authorization] = ACTIONS(2121),
    [sym_err_invalid_parameter] = ACTIONS(2121),
    [sym_err_user_locked_out] = ACTIONS(2121),
    [sym_err_user_account_expired] = ACTIONS(2121),
    [sym_err_general_error] = ACTIONS(2121),
    [sym_err_authentication_error] = ACTIONS(2121),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2121),
    [sym_tls_validate_certificate] = ACTIONS(2121),
    [sym_tdata] = ACTIONS(2121),
    [sym_tchannel] = ACTIONS(2121),
    [sym_tlevel] = ACTIONS(2121),
    [sym_tbutton] = ACTIONS(2121),
    [sym_ttimeline] = ACTIONS(2121),
    [sym_tcustom] = ACTIONS(2121),
    [sym_url_struct] = ACTIONS(2121),
    [sym_dns_struct] = ACTIONS(2121),
    [sym_ip_address_struct] = ACTIONS(2121),
    [sym_dev_info_struct] = ACTIONS(2121),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2121),
    [sym_data_feed] = ACTIONS(2121),
    [sym_data_field] = ACTIONS(2121),
    [sym_data_record] = ACTIONS(2121),
    [sym_wc_data_feed] = ACTIONS(2121),
    [sym_wc_data_field] = ACTIONS(2121),
    [sym_wc_data_record] = ACTIONS(2121),
    [sym_last_login_info] = ACTIONS(2121),
    [sym____reserved___] = ACTIONS(2121),
    [sym_date] = ACTIONS(2121),
    [sym_day] = ACTIONS(2121),
    [sym_time] = ACTIONS(2121),
    [sym_ldate] = ACTIONS(2121),
    [sym_push_device] = ACTIONS(2121),
    [sym_push_channel] = ACTIONS(2121),
    [sym_push_devchan] = ACTIONS(2121),
    [sym_release_device] = ACTIONS(2121),
    [sym_release_channel] = ACTIONS(2121),
    [sym_release_devchan] = ACTIONS(2121),
    [sym_master_slot] = ACTIONS(2121),
    [sym_get_pulse_time] = ACTIONS(2121),
    [sym_get_timer] = ACTIONS(2121),
    [sym_master_sn] = ACTIONS(2121),
    [sym_system_number] = ACTIONS(2121),
    [sym_button] = ACTIONS(2121),
    [sym_dv_channel] = ACTIONS(2121),
    [sym_data] = ACTIONS(2121),
    [sym_level] = ACTIONS(2121),
    [sym_timeline] = ACTIONS(2121),
    [sym_channel] = ACTIONS(2121),
    [sym_custom] = ACTIONS(2121),
    [sym_atoi] = ACTIONS(2121),
    [sym_atol] = ACTIONS(2121),
    [sym_atof] = ACTIONS(2121),
    [sym_itoa] = ACTIONS(2121),
    [sym_itohex] = ACTIONS(2121),
    [sym_hextoi] = ACTIONS(2121),
    [sym_ftoa] = ACTIONS(2121),
    [sym_find_string] = ACTIONS(2121),
    [sym_left_string] = ACTIONS(2121),
    [sym_right_string] = ACTIONS(2121),
    [sym_mid_string] = ACTIONS(2121),
    [sym_remove_string] = ACTIONS(2121),
    [sym_get_buffer_string] = ACTIONS(2121),
    [sym_get_buffer_char] = ACTIONS(2121),
    [sym_get_multi_buffer_string] = ACTIONS(2121),
    [sym_length_string] = ACTIONS(2121),
    [sym_max_length_string] = ACTIONS(2121),
    [sym_set_length_string] = ACTIONS(2121),
    [sym_compare_string] = ACTIONS(2121),
    [sym_upper_string] = ACTIONS(2121),
    [sym_lower_string] = ACTIONS(2121),
    [sym_get_last] = ACTIONS(2121),
    [sym_date_to_day] = ACTIONS(2121),
    [sym_date_to_month] = ACTIONS(2121),
    [sym_date_to_year] = ACTIONS(2121),
    [sym_time_to_hour] = ACTIONS(2121),
    [sym_time_to_minute] = ACTIONS(2121),
    [sym_time_to_second] = ACTIONS(2121),
    [sym_day_of_week] = ACTIONS(2121),
    [sym_set_pulse_time] = ACTIONS(2121),
    [sym_set_timer] = ACTIONS(2121),
    [sym_file_open] = ACTIONS(2121),
    [sym_file_close] = ACTIONS(2121),
    [sym_file_read] = ACTIONS(2121),
    [sym_file_read_line] = ACTIONS(2121),
    [sym_file_write] = ACTIONS(2121),
    [sym_file_write_line] = ACTIONS(2121),
    [sym_file_copy] = ACTIONS(2121),
    [sym_file_delete] = ACTIONS(2121),
    [sym_file_rename] = ACTIONS(2121),
    [sym_file_dir] = ACTIONS(2121),
    [sym_file_getdir] = ACTIONS(2121),
    [sym_file_setdir] = ACTIONS(2121),
    [sym_file_create_dir] = ACTIONS(2121),
    [sym_file_remove_dir] = ACTIONS(2121),
    [sym_file_seek] = ACTIONS(2121),
    [sym_length_array] = ACTIONS(2121),
    [sym_max_length_array] = ACTIONS(2121),
    [sym_set_length_array] = ACTIONS(2121),
    [sym_ip_client_open] = ACTIONS(2121),
    [sym_ip_client_close] = ACTIONS(2121),
    [sym_ip_server_open] = ACTIONS(2121),
    [sym_ip_server_close] = ACTIONS(2121),
    [sym_ip_bound_client_open] = ACTIONS(2121),
    [sym_ip_mc_server_open] = ACTIONS(2121),
    [sym_ip_set_option] = ACTIONS(2121),
    [sym_ssh_client_open] = ACTIONS(2121),
    [sym_ssh_client_close] = ACTIONS(2121),
    [sym_tls_client_open] = ACTIONS(2121),
    [sym_tls_client_close] = ACTIONS(2121),
    [sym_do_push] = ACTIONS(2121),
    [sym_do_push_timed] = ACTIONS(2121),
    [sym_do_release] = ACTIONS(2121),
    [sym_device_id] = ACTIONS(2121),
    [sym_device_info] = ACTIONS(2121),
    [sym_device_id_string] = ACTIONS(2121),
    [sym_device_standby] = ACTIONS(2121),
    [sym_device_wake] = ACTIONS(2121),
    [sym_timeline_create] = ACTIONS(2121),
    [sym_timeline_kill] = ACTIONS(2121),
    [sym_timeline_reload] = ACTIONS(2121),
    [sym_timeline_pause] = ACTIONS(2121),
    [sym_timeline_restart] = ACTIONS(2121),
    [sym_timeline_set] = ACTIONS(2121),
    [sym_timeline_get] = ACTIONS(2121),
    [sym_timeline_active] = ACTIONS(2121),
    [sym_timeline_dynamic_id] = ACTIONS(2121),
    [sym_variable_to_string] = ACTIONS(2121),
    [sym_string_to_variable] = ACTIONS(2121),
    [sym_length_variable_to_string] = ACTIONS(2121),
    [sym_type_cast] = ACTIONS(2121),
    [sym_variable_to_xml] = ACTIONS(2121),
    [sym_xml_to_variable] = ACTIONS(2121),
    [sym_length_variable_to_xml] = ACTIONS(2121),
    [sym_redirect_string] = ACTIONS(2121),
    [sym_random_number] = ACTIONS(2121),
    [sym_get_unique_id] = ACTIONS(2121),
    [sym_get_system_number] = ACTIONS(2121),
    [sym_set_system_number] = ACTIONS(2121),
    [sym_get_serial_number] = ACTIONS(2121),
    [sym_get_master_build] = ACTIONS(2121),
    [sym_set_validation_code] = ACTIONS(2121),
    [sym_reboot] = ACTIONS(2121),
    [sym_load_duet_module] = ACTIONS(2121),
    [sym_unload_duet_module] = ACTIONS(2121),
    [sym_duet_mem_size_set] = ACTIONS(2121),
    [sym_duet_mem_size_get] = ACTIONS(2121),
    [sym_dynamic_application_device] = ACTIONS(2121),
    [sym_dynamic_polled_port] = ACTIONS(2121),
    [sym_static_port_binding] = ACTIONS(2121),
    [sym_static_ip_binding] = ACTIONS(2121),
    [sym_get_dns_list] = ACTIONS(2121),
    [sym_set_dns_list] = ACTIONS(2121),
    [sym_get_ip_address] = ACTIONS(2121),
    [sym_set_ip_address] = ACTIONS(2121),
    [sym_get_url_list] = ACTIONS(2121),
    [sym_add_url_entry] = ACTIONS(2121),
    [sym_delete_url_entry] = ACTIONS(2121),
    [sym_abs_value] = ACTIONS(2121),
    [sym_max_value] = ACTIONS(2121),
    [sym_min_value] = ACTIONS(2121),
    [sym_exp_value] = ACTIONS(2121),
    [sym_log_value] = ACTIONS(2121),
    [sym_log10_value] = ACTIONS(2121),
    [sym_power_value] = ACTIONS(2121),
    [sym_sqrt_value] = ACTIONS(2121),
    [sym_raw_be] = ACTIONS(2121),
    [sym_raw_le] = ACTIONS(2121),
    [sym_internal_queue_size_set] = ACTIONS(2121),
    [sym_internal_queue_size_get] = ACTIONS(2121),
    [sym_internal_threshold_set] = ACTIONS(2121),
    [sym_internal_threshold_get] = ACTIONS(2121),
    [sym_rebuilt_event] = ACTIONS(2121),
    [sym_do_custom_event] = ACTIONS(2121),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2121),
    [sym_clkmgr_set_clk_source] = ACTIONS(2121),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2121),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2121),
    [sym_clkmgr_get_timezone] = ACTIONS(2121),
    [sym_clkmgr_set_timezone] = ACTIONS(2121),
    [sym_clkmgr_get_resync_period] = ACTIONS(2121),
    [sym_clkmgr_set_resync_period] = ACTIONS(2121),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2121),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2121),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2121),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2121),
    [sym_clkmgr_get_timeservers] = ACTIONS(2121),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2121),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2121),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2121),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2121),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2121),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2121),
    [sym_set_log_level] = ACTIONS(2121),
    [sym_get_log_level] = ACTIONS(2121),
    [sym_amx_log] = ACTIONS(2121),
    [sym_smtp_server_config_set] = ACTIONS(2121),
    [sym_smtp_server_config_get] = ACTIONS(2121),
    [sym_smtp_send] = ACTIONS(2121),
    [sym_validate_netlinx_account] = ACTIONS(2121),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2121),
    [sym_authenticate_certificate] = ACTIONS(2121),
    [sym_audit_netlinx_session_event] = ACTIONS(2121),
    [sym_audit_netlinx_generic_event] = ACTIONS(2121),
    [sym_get_available_flash_disk_space] = ACTIONS(2121),
    [sym_get_max_flash_disk_space] = ACTIONS(2121),
    [sym_data_create_feed] = ACTIONS(2121),
    [sym_data_delete_feed] = ACTIONS(2121),
    [sym_data_publish_feed] = ACTIONS(2121),
    [sym_data_get_published_feed] = ACTIONS(2121),
    [sym_data_add_record] = ACTIONS(2121),
    [sym_data_get_event_record] = ACTIONS(2121),
    [sym__wc_data_create_feed] = ACTIONS(2121),
    [sym__wc_data_add_record] = ACTIONS(2121),
    [sym__wc_data_get_event_record] = ACTIONS(2121),
    [sym_format] = ACTIONS(2121),
    [sym_astro_clock] = ACTIONS(2121),
    [sym_set_outdoor_temperature] = ACTIONS(2121),
    [sym_set_virtual_level_count] = ACTIONS(2121),
    [sym_set_virtual_channel_count] = ACTIONS(2121),
    [sym_set_virtual_port_count] = ACTIONS(2121),
    [sym__wc_encode_int] = ACTIONS(2121),
    [sym__wc_decode_int] = ACTIONS(2121),
    [sym__wc_int] = ACTIONS(2121),
    [sym__wc_to_ch_int] = ACTIONS(2121),
    [sym__ch_to_wc_int] = ACTIONS(2121),
    [sym__wc_find_string_int] = ACTIONS(2121),
    [sym__wc_left_string_int] = ACTIONS(2121),
    [sym__wc_lower_string_int] = ACTIONS(2121),
    [sym__wc_mid_string_int] = ACTIONS(2121),
    [sym__wc_remove_string_int] = ACTIONS(2121),
    [sym__wc_right_string_int] = ACTIONS(2121),
    [sym__wc_upper_string_int] = ACTIONS(2121),
    [sym__wc_compare_string_int] = ACTIONS(2121),
    [sym__wc_get_buffer_char_int] = ACTIONS(2121),
    [sym__wc_get_buffer_string_int] = ACTIONS(2121),
    [sym__wc_concat_string_int] = ACTIONS(2121),
    [sym__wc_file_open_int] = ACTIONS(2121),
    [sym__wc_file_close_int] = ACTIONS(2121),
    [sym__wc_file_read_int] = ACTIONS(2121),
    [sym__wc_file_read_line_int] = ACTIONS(2121),
    [sym__wc_file_write_int] = ACTIONS(2121),
    [sym__wc_file_write_line_int] = ACTIONS(2121),
    [aux_sym_preproc_include_token1] = ACTIONS(2121),
    [aux_sym_preproc_define_token1] = ACTIONS(2123),
    [aux_sym_preproc_warn_token1] = ACTIONS(2123),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2123),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2123),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2123),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2123),
    [aux_sym_preproc_else_token1] = ACTIONS(2123),
    [anon_sym_LBRACK] = ACTIONS(2123),
    [anon_sym_LBRACE] = ACTIONS(2123),
    [anon_sym_LPAREN] = ACTIONS(2121),
    [anon_sym_BANG] = ACTIONS(2123),
    [anon_sym_TILDE] = ACTIONS(2123),
    [anon_sym_DASH] = ACTIONS(2121),
    [anon_sym_PLUS] = ACTIONS(2121),
    [anon_sym_DASH_DASH] = ACTIONS(2123),
    [anon_sym_PLUS_PLUS] = ACTIONS(2123),
    [anon_sym_DQUOTE] = ACTIONS(2123),
    [anon_sym_SQUOTE] = ACTIONS(2123),
    [sym_number_literal] = ACTIONS(2123),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(380)] = {
    [ts_builtin_sym_end] = ACTIONS(1805),
    [sym_identifier] = ACTIONS(1803),
    [sym_program_name_keyword] = ACTIONS(1803),
    [sym_module_name_keyword] = ACTIONS(1803),
    [sym_define_device_keyword] = ACTIONS(1803),
    [sym_define_combine_keyword] = ACTIONS(1803),
    [sym_define_constant_keyword] = ACTIONS(1803),
    [sym_define_type_keyword] = ACTIONS(1803),
    [sym_define_variable_keyword] = ACTIONS(1803),
    [sym_define_system_variable_keyword] = ACTIONS(1803),
    [sym_define_connect_level_keyword] = ACTIONS(1803),
    [sym_define_latching_keyword] = ACTIONS(1803),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1803),
    [sym_define_toggling_keyword] = ACTIONS(1803),
    [sym_define_call_keyword] = ACTIONS(1803),
    [sym_define_function_keyword] = ACTIONS(1803),
    [sym_define_library_function_keyword] = ACTIONS(1803),
    [sym_define_start_keyword] = ACTIONS(1803),
    [sym_define_event_keyword] = ACTIONS(1803),
    [sym_define_program_keyword] = ACTIONS(1803),
    [sym_define_module_keyword] = ACTIONS(1803),
    [sym_char_keyword] = ACTIONS(1803),
    [sym_widechar_keyword] = ACTIONS(1803),
    [sym_integer_keyword] = ACTIONS(1803),
    [sym_sinteger_keyword] = ACTIONS(1803),
    [sym_long_keyword] = ACTIONS(1803),
    [sym_slong_keyword] = ACTIONS(1803),
    [sym_float_keyword] = ACTIONS(1803),
    [sym_double_keyword] = ACTIONS(1803),
    [sym_variant_keyword] = ACTIONS(1803),
    [sym_variantarray_keyword] = ACTIONS(1803),
    [sym_local_var_keyword] = ACTIONS(1803),
    [sym_stack_var_keyword] = ACTIONS(1803),
    [sym_constant_keyword] = ACTIONS(1803),
    [sym_volatile_keyword] = ACTIONS(1803),
    [sym_non_volatile_keyword] = ACTIONS(1803),
    [sym_persistent_keyword] = ACTIONS(1803),
    [sym_dev_keyword] = ACTIONS(1803),
    [sym_devlev_keyword] = ACTIONS(1803),
    [sym_devchan_keyword] = ACTIONS(1803),
    [sym_if_keyword] = ACTIONS(1803),
    [sym_else_keyword] = ACTIONS(1803),
    [sym_switch_keyword] = ACTIONS(1803),
    [sym_case_keyword] = ACTIONS(1803),
    [sym_default_keyword] = ACTIONS(1803),
    [sym_while_keyword] = ACTIONS(1803),
    [sym_for_keyword] = ACTIONS(1803),
    [sym_break_keyword] = ACTIONS(1803),
    [sym_continue_keyword] = ACTIONS(1803),
    [sym_return_keyword] = ACTIONS(1803),
    [sym_select_keyword] = ACTIONS(1803),
    [sym_active_keyword] = ACTIONS(1803),
    [sym_struct_keyword] = ACTIONS(1803),
    [sym_structure_keyword] = ACTIONS(1803),
    [sym_button_event_keyword] = ACTIONS(1803),
    [sym_channel_event_keyword] = ACTIONS(1803),
    [sym_level_event_keyword] = ACTIONS(1803),
    [sym_data_event_keyword] = ACTIONS(1803),
    [sym_timeline_event_keyword] = ACTIONS(1803),
    [sym_custom_event_keyword] = ACTIONS(1803),
    [sym_push_keyword] = ACTIONS(1803),
    [sym_release_keyword] = ACTIONS(1803),
    [aux_sym_on_keyword_token1] = ACTIONS(1803),
    [aux_sym_off_keyword_token1] = ACTIONS(1803),
    [sym_send_level_keyword] = ACTIONS(1803),
    [sym_send_string_keyword] = ACTIONS(1803),
    [sym_send_command_keyword] = ACTIONS(1803),
    [sym_clear_buffer_keyword] = ACTIONS(1803),
    [sym_create_buffer_keyword] = ACTIONS(1803),
    [sym_create_multi_buffer_keyword] = ACTIONS(1803),
    [sym_call_keyword] = ACTIONS(1803),
    [sym_system_call_keyword] = ACTIONS(1803),
    [sym_devchan_to_keyword] = ACTIONS(1803),
    [sym_devchan_min_to_keyword] = ACTIONS(1803),
    [sym_devchan_total_off_keyword] = ACTIONS(1803),
    [sym_devchan_pulse_keyword] = ACTIONS(1803),
    [sym_wait_keyword] = ACTIONS(1803),
    [sym_cancel_wait_keyword] = ACTIONS(1803),
    [sym_wait_until_keyword] = ACTIONS(1803),
    [sym_cancel_wait_until_keyword] = ACTIONS(1803),
    [sym_cancel_all_wait_keyword] = ACTIONS(1803),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1803),
    [sym_not] = ACTIONS(1803),
    [sym_bnot] = ACTIONS(1803),
    [sym___file__] = ACTIONS(1803),
    [sym___date__] = ACTIONS(1803),
    [sym___ldate__] = ACTIONS(1803),
    [sym___time__] = ACTIONS(1803),
    [sym___line__] = ACTIONS(1803),
    [sym___name__] = ACTIONS(1803),
    [sym___netlinx__] = ACTIONS(1803),
    [sym_netlinx_axi_version] = ACTIONS(1803),
    [sym_url_flg_tcp] = ACTIONS(1803),
    [sym_url_flg_accinfopresent] = ACTIONS(1803),
    [sym_url_flg_temp] = ACTIONS(1803),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1803),
    [sym_url_flg_stat_mask] = ACTIONS(1803),
    [sym_url_flg_stat_lookup] = ACTIONS(1803),
    [sym_url_flg_stat_connecting] = ACTIONS(1803),
    [sym_url_flg_stat_waiting] = ACTIONS(1803),
    [sym_url_flg_stat_connected] = ACTIONS(1803),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1803),
    [sym_first_virtual_device] = ACTIONS(1803),
    [sym_first_local_port] = ACTIONS(1803),
    [sym_dynamic_virtual_device] = ACTIONS(1803),
    [sym_timeline_once] = ACTIONS(1803),
    [sym_timeline_repeat] = ACTIONS(1803),
    [sym_timeline_absolute] = ACTIONS(1803),
    [sym_timeline_relative] = ACTIONS(1803),
    [sym_do_push_timed_infinite] = ACTIONS(1803),
    [sym_source_type_no_address] = ACTIONS(1803),
    [sym_source_type_neuron_id] = ACTIONS(1803),
    [sym_source_type_ip_address] = ACTIONS(1803),
    [sym_source_type_axlink] = ACTIONS(1803),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1803),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1803),
    [sym_source_type_ip_socket_address] = ACTIONS(1803),
    [sym_source_type_rs232] = ACTIONS(1803),
    [sym_source_type_internal] = ACTIONS(1803),
    [sym_source_type_mac_address] = ACTIONS(1803),
    [sym_source_type_ipv4_port] = ACTIONS(1803),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1803),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1803),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1803),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1803),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1803),
    [sym_true] = ACTIONS(1803),
    [sym_false] = ACTIONS(1803),
    [sym_file_read_only] = ACTIONS(1803),
    [sym_file_rw_new] = ACTIONS(1803),
    [sym_file_rw_append] = ACTIONS(1803),
    [sym_ip_tcp] = ACTIONS(1803),
    [sym_ip_udp] = ACTIONS(1803),
    [sym_ip_udp_2way] = ACTIONS(1803),
    [sym_xml_encode_types] = ACTIONS(1803),
    [sym_xml_encode_char_as_list] = ACTIONS(1803),
    [sym_xml_encode_le] = ACTIONS(1803),
    [sym_xml_decode_types] = ACTIONS(1803),
    [sym_xml_decode_no_preserve] = ACTIONS(1803),
    [sym_duet_dev_type_display_device] = ACTIONS(1803),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1803),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1803),
    [sym_duet_dev_type_io_device] = ACTIONS(1803),
    [sym_duet_dev_type_relay_device] = ACTIONS(1803),
    [sym_duet_dev_type_ups] = ACTIONS(1803),
    [sym_duet_dev_type_amplifier] = ACTIONS(1803),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1803),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1803),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1803),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1803),
    [sym_duet_dev_type_camera] = ACTIONS(1803),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1803),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1803),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1803),
    [sym_duet_dev_type_dss] = ACTIONS(1803),
    [sym_duet_dev_type_dvr] = ACTIONS(1803),
    [sym_duet_dev_type_disc_device] = ACTIONS(1803),
    [sym_duet_dev_type_document_camera] = ACTIONS(1803),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1803),
    [sym_duet_dev_type_hvac] = ACTIONS(1803),
    [sym_duet_dev_type_keypad] = ACTIONS(1803),
    [sym_duet_dev_type_light] = ACTIONS(1803),
    [sym_duet_dev_type_monitor] = ACTIONS(1803),
    [sym_duet_dev_type_motor] = ACTIONS(1803),
    [sym_duet_dev_type_multi_window] = ACTIONS(1803),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1803),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1803),
    [sym_duet_dev_type_receiver] = ACTIONS(1803),
    [sym_duet_dev_type_security_system] = ACTIONS(1803),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1803),
    [sym_duet_dev_type_settop_box] = ACTIONS(1803),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1803),
    [sym_duet_dev_type_switcher] = ACTIONS(1803),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1803),
    [sym_duet_dev_type_tv] = ACTIONS(1803),
    [sym_duet_dev_type_utility] = ACTIONS(1803),
    [sym_duet_dev_type_vcr] = ACTIONS(1803),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1803),
    [sym_duet_dev_type_video_processor] = ACTIONS(1803),
    [sym_duet_dev_type_video_projector] = ACTIONS(1803),
    [sym_duet_dev_type_video_wall] = ACTIONS(1803),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1803),
    [sym_duet_dev_type_weather] = ACTIONS(1803),
    [sym_duet_dev_not_polled] = ACTIONS(1803),
    [sym_duet_dev_polled] = ACTIONS(1803),
    [sym_clkmgr_mode_network] = ACTIONS(1803),
    [sym_clkmgr_mode_standalone] = ACTIONS(1803),
    [sym_normal_standby] = ACTIONS(1803),
    [sym_normal_wake] = ACTIONS(1803),
    [sym_ip_multicast_ttl_option] = ACTIONS(1803),
    [sym_ip_ttl_subnet] = ACTIONS(1803),
    [sym_ip_ttl_site] = ACTIONS(1803),
    [sym_ip_ttl_region] = ACTIONS(1803),
    [sym_ip_ttl_continent] = ACTIONS(1803),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1803),
    [sym_ip_nodelay_on] = ACTIONS(1803),
    [sym_ip_nodelay_off] = ACTIONS(1803),
    [sym_amx_error] = ACTIONS(1803),
    [sym_amx_warning] = ACTIONS(1803),
    [sym_amx_info] = ACTIONS(1803),
    [sym_amx_debug] = ACTIONS(1803),
    [sym_netlinx_login_success] = ACTIONS(1803),
    [sym_netlinx_login_fail] = ACTIONS(1803),
    [sym_netlinx_logout] = ACTIONS(1803),
    [sym_null_str] = ACTIONS(1803),
    [sym_smtp_address] = ACTIONS(1803),
    [sym_smtp_port_number] = ACTIONS(1803),
    [sym_smtp_username] = ACTIONS(1803),
    [sym_smtp_password] = ACTIONS(1803),
    [sym_smtp_require_tls] = ACTIONS(1803),
    [sym_smtp_from] = ACTIONS(1803),
    [sym_smtp_tls_true] = ACTIONS(1803),
    [sym_smtp_tls_false] = ACTIONS(1803),
    [sym_listview_on_row_select_event] = ACTIONS(1803),
    [sym_data_structure_unknown] = ACTIONS(1803),
    [sym_data_structure_datafeed] = ACTIONS(1803),
    [sym_data_structure_datarecordset] = ACTIONS(1803),
    [sym_data_structure_datarecord] = ACTIONS(1803),
    [sym_data_structure_datafield] = ACTIONS(1803),
    [sym_data_max_content_field_count] = ACTIONS(1803),
    [sym_data_max_metadata_field_count] = ACTIONS(1803),
    [sym_data_max_id_length] = ACTIONS(1803),
    [sym_data_max_name_length] = ACTIONS(1803),
    [sym_data_max_description_length] = ACTIONS(1803),
    [sym_data_max_type_length] = ACTIONS(1803),
    [sym_data_max_format_length] = ACTIONS(1803),
    [sym_data_max_source_length] = ACTIONS(1803),
    [sym_data_max_value_length] = ACTIONS(1803),
    [sym_data_max_label_length] = ACTIONS(1803),
    [sym_data_max_url_length] = ACTIONS(1803),
    [sym_max_last_login_info_length] = ACTIONS(1803),
    [sym_data_type_unknown] = ACTIONS(1803),
    [sym_data_type_string] = ACTIONS(1803),
    [sym_data_type_datetime] = ACTIONS(1803),
    [sym_data_type_date] = ACTIONS(1803),
    [sym_data_type_time] = ACTIONS(1803),
    [sym_data_type_image] = ACTIONS(1803),
    [sym_data_format_url] = ACTIONS(1803),
    [sym_data_format_phone] = ACTIONS(1803),
    [sym_data_format_email] = ACTIONS(1803),
    [sym_data_format_iso8601] = ACTIONS(1803),
    [sym_auditlog_priv] = ACTIONS(1803),
    [sym_configuration_priv] = ACTIONS(1803),
    [sym_deviceconfig_priv] = ACTIONS(1803),
    [sym_ftp_priv] = ACTIONS(1803),
    [sym_http_priv] = ACTIONS(1803),
    [sym_networkconfig_priv] = ACTIONS(1803),
    [sym_programport_priv] = ACTIONS(1803),
    [sym_remoteui_priv] = ACTIONS(1803),
    [sym_securitycontrol_priv] = ACTIONS(1803),
    [sym_softwaremanagement_priv] = ACTIONS(1803),
    [sym_terminal_priv] = ACTIONS(1803),
    [sym_tpadmin_priv] = ACTIONS(1803),
    [sym_user1_priv] = ACTIONS(1803),
    [sym_user2_priv] = ACTIONS(1803),
    [sym_user3_priv] = ACTIONS(1803),
    [sym_user4_priv] = ACTIONS(1803),
    [sym_usermanagement_priv] = ACTIONS(1803),
    [sym_valid_account] = ACTIONS(1803),
    [sym_err_name_invalid] = ACTIONS(1803),
    [sym_err_password_invalid] = ACTIONS(1803),
    [sym_err_invalid_account] = ACTIONS(1803),
    [sym_err_no_authorization] = ACTIONS(1803),
    [sym_err_invalid_parameter] = ACTIONS(1803),
    [sym_err_user_locked_out] = ACTIONS(1803),
    [sym_err_user_account_expired] = ACTIONS(1803),
    [sym_err_general_error] = ACTIONS(1803),
    [sym_err_authentication_error] = ACTIONS(1803),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1803),
    [sym_tls_validate_certificate] = ACTIONS(1803),
    [sym_tdata] = ACTIONS(1803),
    [sym_tchannel] = ACTIONS(1803),
    [sym_tlevel] = ACTIONS(1803),
    [sym_tbutton] = ACTIONS(1803),
    [sym_ttimeline] = ACTIONS(1803),
    [sym_tcustom] = ACTIONS(1803),
    [sym_url_struct] = ACTIONS(1803),
    [sym_dns_struct] = ACTIONS(1803),
    [sym_ip_address_struct] = ACTIONS(1803),
    [sym_dev_info_struct] = ACTIONS(1803),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1803),
    [sym_data_feed] = ACTIONS(1803),
    [sym_data_field] = ACTIONS(1803),
    [sym_data_record] = ACTIONS(1803),
    [sym_wc_data_feed] = ACTIONS(1803),
    [sym_wc_data_field] = ACTIONS(1803),
    [sym_wc_data_record] = ACTIONS(1803),
    [sym_last_login_info] = ACTIONS(1803),
    [sym____reserved___] = ACTIONS(1803),
    [sym_date] = ACTIONS(1803),
    [sym_day] = ACTIONS(1803),
    [sym_time] = ACTIONS(1803),
    [sym_ldate] = ACTIONS(1803),
    [sym_push_device] = ACTIONS(1803),
    [sym_push_channel] = ACTIONS(1803),
    [sym_push_devchan] = ACTIONS(1803),
    [sym_release_device] = ACTIONS(1803),
    [sym_release_channel] = ACTIONS(1803),
    [sym_release_devchan] = ACTIONS(1803),
    [sym_master_slot] = ACTIONS(1803),
    [sym_get_pulse_time] = ACTIONS(1803),
    [sym_get_timer] = ACTIONS(1803),
    [sym_master_sn] = ACTIONS(1803),
    [sym_system_number] = ACTIONS(1803),
    [sym_button] = ACTIONS(1803),
    [sym_dv_channel] = ACTIONS(1803),
    [sym_data] = ACTIONS(1803),
    [sym_level] = ACTIONS(1803),
    [sym_timeline] = ACTIONS(1803),
    [sym_channel] = ACTIONS(1803),
    [sym_custom] = ACTIONS(1803),
    [sym_atoi] = ACTIONS(1803),
    [sym_atol] = ACTIONS(1803),
    [sym_atof] = ACTIONS(1803),
    [sym_itoa] = ACTIONS(1803),
    [sym_itohex] = ACTIONS(1803),
    [sym_hextoi] = ACTIONS(1803),
    [sym_ftoa] = ACTIONS(1803),
    [sym_find_string] = ACTIONS(1803),
    [sym_left_string] = ACTIONS(1803),
    [sym_right_string] = ACTIONS(1803),
    [sym_mid_string] = ACTIONS(1803),
    [sym_remove_string] = ACTIONS(1803),
    [sym_get_buffer_string] = ACTIONS(1803),
    [sym_get_buffer_char] = ACTIONS(1803),
    [sym_get_multi_buffer_string] = ACTIONS(1803),
    [sym_length_string] = ACTIONS(1803),
    [sym_max_length_string] = ACTIONS(1803),
    [sym_set_length_string] = ACTIONS(1803),
    [sym_compare_string] = ACTIONS(1803),
    [sym_upper_string] = ACTIONS(1803),
    [sym_lower_string] = ACTIONS(1803),
    [sym_get_last] = ACTIONS(1803),
    [sym_date_to_day] = ACTIONS(1803),
    [sym_date_to_month] = ACTIONS(1803),
    [sym_date_to_year] = ACTIONS(1803),
    [sym_time_to_hour] = ACTIONS(1803),
    [sym_time_to_minute] = ACTIONS(1803),
    [sym_time_to_second] = ACTIONS(1803),
    [sym_day_of_week] = ACTIONS(1803),
    [sym_set_pulse_time] = ACTIONS(1803),
    [sym_set_timer] = ACTIONS(1803),
    [sym_file_open] = ACTIONS(1803),
    [sym_file_close] = ACTIONS(1803),
    [sym_file_read] = ACTIONS(1803),
    [sym_file_read_line] = ACTIONS(1803),
    [sym_file_write] = ACTIONS(1803),
    [sym_file_write_line] = ACTIONS(1803),
    [sym_file_copy] = ACTIONS(1803),
    [sym_file_delete] = ACTIONS(1803),
    [sym_file_rename] = ACTIONS(1803),
    [sym_file_dir] = ACTIONS(1803),
    [sym_file_getdir] = ACTIONS(1803),
    [sym_file_setdir] = ACTIONS(1803),
    [sym_file_create_dir] = ACTIONS(1803),
    [sym_file_remove_dir] = ACTIONS(1803),
    [sym_file_seek] = ACTIONS(1803),
    [sym_length_array] = ACTIONS(1803),
    [sym_max_length_array] = ACTIONS(1803),
    [sym_set_length_array] = ACTIONS(1803),
    [sym_ip_client_open] = ACTIONS(1803),
    [sym_ip_client_close] = ACTIONS(1803),
    [sym_ip_server_open] = ACTIONS(1803),
    [sym_ip_server_close] = ACTIONS(1803),
    [sym_ip_bound_client_open] = ACTIONS(1803),
    [sym_ip_mc_server_open] = ACTIONS(1803),
    [sym_ip_set_option] = ACTIONS(1803),
    [sym_ssh_client_open] = ACTIONS(1803),
    [sym_ssh_client_close] = ACTIONS(1803),
    [sym_tls_client_open] = ACTIONS(1803),
    [sym_tls_client_close] = ACTIONS(1803),
    [sym_do_push] = ACTIONS(1803),
    [sym_do_push_timed] = ACTIONS(1803),
    [sym_do_release] = ACTIONS(1803),
    [sym_device_id] = ACTIONS(1803),
    [sym_device_info] = ACTIONS(1803),
    [sym_device_id_string] = ACTIONS(1803),
    [sym_device_standby] = ACTIONS(1803),
    [sym_device_wake] = ACTIONS(1803),
    [sym_timeline_create] = ACTIONS(1803),
    [sym_timeline_kill] = ACTIONS(1803),
    [sym_timeline_reload] = ACTIONS(1803),
    [sym_timeline_pause] = ACTIONS(1803),
    [sym_timeline_restart] = ACTIONS(1803),
    [sym_timeline_set] = ACTIONS(1803),
    [sym_timeline_get] = ACTIONS(1803),
    [sym_timeline_active] = ACTIONS(1803),
    [sym_timeline_dynamic_id] = ACTIONS(1803),
    [sym_variable_to_string] = ACTIONS(1803),
    [sym_string_to_variable] = ACTIONS(1803),
    [sym_length_variable_to_string] = ACTIONS(1803),
    [sym_type_cast] = ACTIONS(1803),
    [sym_variable_to_xml] = ACTIONS(1803),
    [sym_xml_to_variable] = ACTIONS(1803),
    [sym_length_variable_to_xml] = ACTIONS(1803),
    [sym_redirect_string] = ACTIONS(1803),
    [sym_random_number] = ACTIONS(1803),
    [sym_get_unique_id] = ACTIONS(1803),
    [sym_get_system_number] = ACTIONS(1803),
    [sym_set_system_number] = ACTIONS(1803),
    [sym_get_serial_number] = ACTIONS(1803),
    [sym_get_master_build] = ACTIONS(1803),
    [sym_set_validation_code] = ACTIONS(1803),
    [sym_reboot] = ACTIONS(1803),
    [sym_load_duet_module] = ACTIONS(1803),
    [sym_unload_duet_module] = ACTIONS(1803),
    [sym_duet_mem_size_set] = ACTIONS(1803),
    [sym_duet_mem_size_get] = ACTIONS(1803),
    [sym_dynamic_application_device] = ACTIONS(1803),
    [sym_dynamic_polled_port] = ACTIONS(1803),
    [sym_static_port_binding] = ACTIONS(1803),
    [sym_static_ip_binding] = ACTIONS(1803),
    [sym_get_dns_list] = ACTIONS(1803),
    [sym_set_dns_list] = ACTIONS(1803),
    [sym_get_ip_address] = ACTIONS(1803),
    [sym_set_ip_address] = ACTIONS(1803),
    [sym_get_url_list] = ACTIONS(1803),
    [sym_add_url_entry] = ACTIONS(1803),
    [sym_delete_url_entry] = ACTIONS(1803),
    [sym_abs_value] = ACTIONS(1803),
    [sym_max_value] = ACTIONS(1803),
    [sym_min_value] = ACTIONS(1803),
    [sym_exp_value] = ACTIONS(1803),
    [sym_log_value] = ACTIONS(1803),
    [sym_log10_value] = ACTIONS(1803),
    [sym_power_value] = ACTIONS(1803),
    [sym_sqrt_value] = ACTIONS(1803),
    [sym_raw_be] = ACTIONS(1803),
    [sym_raw_le] = ACTIONS(1803),
    [sym_internal_queue_size_set] = ACTIONS(1803),
    [sym_internal_queue_size_get] = ACTIONS(1803),
    [sym_internal_threshold_set] = ACTIONS(1803),
    [sym_internal_threshold_get] = ACTIONS(1803),
    [sym_rebuilt_event] = ACTIONS(1803),
    [sym_do_custom_event] = ACTIONS(1803),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1803),
    [sym_clkmgr_set_clk_source] = ACTIONS(1803),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1803),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1803),
    [sym_clkmgr_get_timezone] = ACTIONS(1803),
    [sym_clkmgr_set_timezone] = ACTIONS(1803),
    [sym_clkmgr_get_resync_period] = ACTIONS(1803),
    [sym_clkmgr_set_resync_period] = ACTIONS(1803),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1803),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1803),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1803),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1803),
    [sym_clkmgr_get_timeservers] = ACTIONS(1803),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1803),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1803),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1803),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1803),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1803),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1803),
    [sym_set_log_level] = ACTIONS(1803),
    [sym_get_log_level] = ACTIONS(1803),
    [sym_amx_log] = ACTIONS(1803),
    [sym_smtp_server_config_set] = ACTIONS(1803),
    [sym_smtp_server_config_get] = ACTIONS(1803),
    [sym_smtp_send] = ACTIONS(1803),
    [sym_validate_netlinx_account] = ACTIONS(1803),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1803),
    [sym_authenticate_certificate] = ACTIONS(1803),
    [sym_audit_netlinx_session_event] = ACTIONS(1803),
    [sym_audit_netlinx_generic_event] = ACTIONS(1803),
    [sym_get_available_flash_disk_space] = ACTIONS(1803),
    [sym_get_max_flash_disk_space] = ACTIONS(1803),
    [sym_data_create_feed] = ACTIONS(1803),
    [sym_data_delete_feed] = ACTIONS(1803),
    [sym_data_publish_feed] = ACTIONS(1803),
    [sym_data_get_published_feed] = ACTIONS(1803),
    [sym_data_add_record] = ACTIONS(1803),
    [sym_data_get_event_record] = ACTIONS(1803),
    [sym__wc_data_create_feed] = ACTIONS(1803),
    [sym__wc_data_add_record] = ACTIONS(1803),
    [sym__wc_data_get_event_record] = ACTIONS(1803),
    [sym_format] = ACTIONS(1803),
    [sym_astro_clock] = ACTIONS(1803),
    [sym_set_outdoor_temperature] = ACTIONS(1803),
    [sym_set_virtual_level_count] = ACTIONS(1803),
    [sym_set_virtual_channel_count] = ACTIONS(1803),
    [sym_set_virtual_port_count] = ACTIONS(1803),
    [sym__wc_encode_int] = ACTIONS(1803),
    [sym__wc_decode_int] = ACTIONS(1803),
    [sym__wc_int] = ACTIONS(1803),
    [sym__wc_to_ch_int] = ACTIONS(1803),
    [sym__ch_to_wc_int] = ACTIONS(1803),
    [sym__wc_find_string_int] = ACTIONS(1803),
    [sym__wc_left_string_int] = ACTIONS(1803),
    [sym__wc_lower_string_int] = ACTIONS(1803),
    [sym__wc_mid_string_int] = ACTIONS(1803),
    [sym__wc_remove_string_int] = ACTIONS(1803),
    [sym__wc_right_string_int] = ACTIONS(1803),
    [sym__wc_upper_string_int] = ACTIONS(1803),
    [sym__wc_compare_string_int] = ACTIONS(1803),
    [sym__wc_get_buffer_char_int] = ACTIONS(1803),
    [sym__wc_get_buffer_string_int] = ACTIONS(1803),
    [sym__wc_concat_string_int] = ACTIONS(1803),
    [sym__wc_file_open_int] = ACTIONS(1803),
    [sym__wc_file_close_int] = ACTIONS(1803),
    [sym__wc_file_read_int] = ACTIONS(1803),
    [sym__wc_file_read_line_int] = ACTIONS(1803),
    [sym__wc_file_write_int] = ACTIONS(1803),
    [sym__wc_file_write_line_int] = ACTIONS(1803),
    [aux_sym_preproc_include_token1] = ACTIONS(1803),
    [aux_sym_preproc_define_token1] = ACTIONS(1805),
    [aux_sym_preproc_warn_token1] = ACTIONS(1805),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1805),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1805),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1805),
    [anon_sym_LBRACK] = ACTIONS(1805),
    [anon_sym_LBRACE] = ACTIONS(1805),
    [anon_sym_LPAREN] = ACTIONS(1803),
    [anon_sym_BANG] = ACTIONS(1805),
    [anon_sym_TILDE] = ACTIONS(1805),
    [anon_sym_DASH] = ACTIONS(1803),
    [anon_sym_PLUS] = ACTIONS(1803),
    [anon_sym_DASH_DASH] = ACTIONS(1805),
    [anon_sym_PLUS_PLUS] = ACTIONS(1805),
    [anon_sym_DQUOTE] = ACTIONS(1805),
    [anon_sym_SQUOTE] = ACTIONS(1805),
    [sym_number_literal] = ACTIONS(1805),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(381)] = {
    [sym_identifier] = ACTIONS(2125),
    [sym_program_name_keyword] = ACTIONS(2125),
    [sym_module_name_keyword] = ACTIONS(2125),
    [sym_define_device_keyword] = ACTIONS(2125),
    [sym_define_combine_keyword] = ACTIONS(2125),
    [sym_define_constant_keyword] = ACTIONS(2125),
    [sym_define_type_keyword] = ACTIONS(2125),
    [sym_define_variable_keyword] = ACTIONS(2125),
    [sym_define_system_variable_keyword] = ACTIONS(2125),
    [sym_define_connect_level_keyword] = ACTIONS(2125),
    [sym_define_latching_keyword] = ACTIONS(2125),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2125),
    [sym_define_toggling_keyword] = ACTIONS(2125),
    [sym_define_call_keyword] = ACTIONS(2125),
    [sym_define_function_keyword] = ACTIONS(2125),
    [sym_define_library_function_keyword] = ACTIONS(2125),
    [sym_define_start_keyword] = ACTIONS(2125),
    [sym_define_event_keyword] = ACTIONS(2125),
    [sym_define_program_keyword] = ACTIONS(2125),
    [sym_define_module_keyword] = ACTIONS(2125),
    [sym_char_keyword] = ACTIONS(2125),
    [sym_widechar_keyword] = ACTIONS(2125),
    [sym_integer_keyword] = ACTIONS(2125),
    [sym_sinteger_keyword] = ACTIONS(2125),
    [sym_long_keyword] = ACTIONS(2125),
    [sym_slong_keyword] = ACTIONS(2125),
    [sym_float_keyword] = ACTIONS(2125),
    [sym_double_keyword] = ACTIONS(2125),
    [sym_variant_keyword] = ACTIONS(2125),
    [sym_variantarray_keyword] = ACTIONS(2125),
    [sym_local_var_keyword] = ACTIONS(2125),
    [sym_stack_var_keyword] = ACTIONS(2125),
    [sym_constant_keyword] = ACTIONS(2125),
    [sym_volatile_keyword] = ACTIONS(2125),
    [sym_non_volatile_keyword] = ACTIONS(2125),
    [sym_persistent_keyword] = ACTIONS(2125),
    [sym_dev_keyword] = ACTIONS(2125),
    [sym_devlev_keyword] = ACTIONS(2125),
    [sym_devchan_keyword] = ACTIONS(2125),
    [sym_if_keyword] = ACTIONS(2125),
    [sym_switch_keyword] = ACTIONS(2125),
    [sym_case_keyword] = ACTIONS(2125),
    [sym_default_keyword] = ACTIONS(2125),
    [sym_while_keyword] = ACTIONS(2125),
    [sym_for_keyword] = ACTIONS(2125),
    [sym_break_keyword] = ACTIONS(2125),
    [sym_continue_keyword] = ACTIONS(2125),
    [sym_return_keyword] = ACTIONS(2125),
    [sym_select_keyword] = ACTIONS(2125),
    [sym_active_keyword] = ACTIONS(2125),
    [sym_struct_keyword] = ACTIONS(2125),
    [sym_structure_keyword] = ACTIONS(2125),
    [sym_button_event_keyword] = ACTIONS(2125),
    [sym_channel_event_keyword] = ACTIONS(2125),
    [sym_level_event_keyword] = ACTIONS(2125),
    [sym_data_event_keyword] = ACTIONS(2125),
    [sym_timeline_event_keyword] = ACTIONS(2125),
    [sym_custom_event_keyword] = ACTIONS(2125),
    [sym_push_keyword] = ACTIONS(2125),
    [sym_release_keyword] = ACTIONS(2125),
    [aux_sym_on_keyword_token1] = ACTIONS(2125),
    [aux_sym_off_keyword_token1] = ACTIONS(2125),
    [sym_send_level_keyword] = ACTIONS(2125),
    [sym_send_string_keyword] = ACTIONS(2125),
    [sym_send_command_keyword] = ACTIONS(2125),
    [sym_clear_buffer_keyword] = ACTIONS(2125),
    [sym_create_buffer_keyword] = ACTIONS(2125),
    [sym_create_multi_buffer_keyword] = ACTIONS(2125),
    [sym_call_keyword] = ACTIONS(2125),
    [sym_system_call_keyword] = ACTIONS(2125),
    [sym_devchan_to_keyword] = ACTIONS(2125),
    [sym_devchan_min_to_keyword] = ACTIONS(2125),
    [sym_devchan_total_off_keyword] = ACTIONS(2125),
    [sym_devchan_pulse_keyword] = ACTIONS(2125),
    [sym_wait_keyword] = ACTIONS(2125),
    [sym_cancel_wait_keyword] = ACTIONS(2125),
    [sym_wait_until_keyword] = ACTIONS(2125),
    [sym_cancel_wait_until_keyword] = ACTIONS(2125),
    [sym_cancel_all_wait_keyword] = ACTIONS(2125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2125),
    [sym_not] = ACTIONS(2125),
    [sym_bnot] = ACTIONS(2125),
    [sym___file__] = ACTIONS(2125),
    [sym___date__] = ACTIONS(2125),
    [sym___ldate__] = ACTIONS(2125),
    [sym___time__] = ACTIONS(2125),
    [sym___line__] = ACTIONS(2125),
    [sym___name__] = ACTIONS(2125),
    [sym___netlinx__] = ACTIONS(2125),
    [sym_netlinx_axi_version] = ACTIONS(2125),
    [sym_url_flg_tcp] = ACTIONS(2125),
    [sym_url_flg_accinfopresent] = ACTIONS(2125),
    [sym_url_flg_temp] = ACTIONS(2125),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2125),
    [sym_url_flg_stat_mask] = ACTIONS(2125),
    [sym_url_flg_stat_lookup] = ACTIONS(2125),
    [sym_url_flg_stat_connecting] = ACTIONS(2125),
    [sym_url_flg_stat_waiting] = ACTIONS(2125),
    [sym_url_flg_stat_connected] = ACTIONS(2125),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2125),
    [sym_first_virtual_device] = ACTIONS(2125),
    [sym_first_local_port] = ACTIONS(2125),
    [sym_dynamic_virtual_device] = ACTIONS(2125),
    [sym_timeline_once] = ACTIONS(2125),
    [sym_timeline_repeat] = ACTIONS(2125),
    [sym_timeline_absolute] = ACTIONS(2125),
    [sym_timeline_relative] = ACTIONS(2125),
    [sym_do_push_timed_infinite] = ACTIONS(2125),
    [sym_source_type_no_address] = ACTIONS(2125),
    [sym_source_type_neuron_id] = ACTIONS(2125),
    [sym_source_type_ip_address] = ACTIONS(2125),
    [sym_source_type_axlink] = ACTIONS(2125),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2125),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2125),
    [sym_source_type_ip_socket_address] = ACTIONS(2125),
    [sym_source_type_rs232] = ACTIONS(2125),
    [sym_source_type_internal] = ACTIONS(2125),
    [sym_source_type_mac_address] = ACTIONS(2125),
    [sym_source_type_ipv4_port] = ACTIONS(2125),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2125),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2125),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2125),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2125),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2125),
    [sym_true] = ACTIONS(2125),
    [sym_false] = ACTIONS(2125),
    [sym_file_read_only] = ACTIONS(2125),
    [sym_file_rw_new] = ACTIONS(2125),
    [sym_file_rw_append] = ACTIONS(2125),
    [sym_ip_tcp] = ACTIONS(2125),
    [sym_ip_udp] = ACTIONS(2125),
    [sym_ip_udp_2way] = ACTIONS(2125),
    [sym_xml_encode_types] = ACTIONS(2125),
    [sym_xml_encode_char_as_list] = ACTIONS(2125),
    [sym_xml_encode_le] = ACTIONS(2125),
    [sym_xml_decode_types] = ACTIONS(2125),
    [sym_xml_decode_no_preserve] = ACTIONS(2125),
    [sym_duet_dev_type_display_device] = ACTIONS(2125),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2125),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2125),
    [sym_duet_dev_type_io_device] = ACTIONS(2125),
    [sym_duet_dev_type_relay_device] = ACTIONS(2125),
    [sym_duet_dev_type_ups] = ACTIONS(2125),
    [sym_duet_dev_type_amplifier] = ACTIONS(2125),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2125),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2125),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2125),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2125),
    [sym_duet_dev_type_camera] = ACTIONS(2125),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2125),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2125),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2125),
    [sym_duet_dev_type_dss] = ACTIONS(2125),
    [sym_duet_dev_type_dvr] = ACTIONS(2125),
    [sym_duet_dev_type_disc_device] = ACTIONS(2125),
    [sym_duet_dev_type_document_camera] = ACTIONS(2125),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2125),
    [sym_duet_dev_type_hvac] = ACTIONS(2125),
    [sym_duet_dev_type_keypad] = ACTIONS(2125),
    [sym_duet_dev_type_light] = ACTIONS(2125),
    [sym_duet_dev_type_monitor] = ACTIONS(2125),
    [sym_duet_dev_type_motor] = ACTIONS(2125),
    [sym_duet_dev_type_multi_window] = ACTIONS(2125),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2125),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2125),
    [sym_duet_dev_type_receiver] = ACTIONS(2125),
    [sym_duet_dev_type_security_system] = ACTIONS(2125),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2125),
    [sym_duet_dev_type_settop_box] = ACTIONS(2125),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2125),
    [sym_duet_dev_type_switcher] = ACTIONS(2125),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2125),
    [sym_duet_dev_type_tv] = ACTIONS(2125),
    [sym_duet_dev_type_utility] = ACTIONS(2125),
    [sym_duet_dev_type_vcr] = ACTIONS(2125),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2125),
    [sym_duet_dev_type_video_processor] = ACTIONS(2125),
    [sym_duet_dev_type_video_projector] = ACTIONS(2125),
    [sym_duet_dev_type_video_wall] = ACTIONS(2125),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2125),
    [sym_duet_dev_type_weather] = ACTIONS(2125),
    [sym_duet_dev_not_polled] = ACTIONS(2125),
    [sym_duet_dev_polled] = ACTIONS(2125),
    [sym_clkmgr_mode_network] = ACTIONS(2125),
    [sym_clkmgr_mode_standalone] = ACTIONS(2125),
    [sym_normal_standby] = ACTIONS(2125),
    [sym_normal_wake] = ACTIONS(2125),
    [sym_ip_multicast_ttl_option] = ACTIONS(2125),
    [sym_ip_ttl_subnet] = ACTIONS(2125),
    [sym_ip_ttl_site] = ACTIONS(2125),
    [sym_ip_ttl_region] = ACTIONS(2125),
    [sym_ip_ttl_continent] = ACTIONS(2125),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2125),
    [sym_ip_nodelay_on] = ACTIONS(2125),
    [sym_ip_nodelay_off] = ACTIONS(2125),
    [sym_amx_error] = ACTIONS(2125),
    [sym_amx_warning] = ACTIONS(2125),
    [sym_amx_info] = ACTIONS(2125),
    [sym_amx_debug] = ACTIONS(2125),
    [sym_netlinx_login_success] = ACTIONS(2125),
    [sym_netlinx_login_fail] = ACTIONS(2125),
    [sym_netlinx_logout] = ACTIONS(2125),
    [sym_null_str] = ACTIONS(2125),
    [sym_smtp_address] = ACTIONS(2125),
    [sym_smtp_port_number] = ACTIONS(2125),
    [sym_smtp_username] = ACTIONS(2125),
    [sym_smtp_password] = ACTIONS(2125),
    [sym_smtp_require_tls] = ACTIONS(2125),
    [sym_smtp_from] = ACTIONS(2125),
    [sym_smtp_tls_true] = ACTIONS(2125),
    [sym_smtp_tls_false] = ACTIONS(2125),
    [sym_listview_on_row_select_event] = ACTIONS(2125),
    [sym_data_structure_unknown] = ACTIONS(2125),
    [sym_data_structure_datafeed] = ACTIONS(2125),
    [sym_data_structure_datarecordset] = ACTIONS(2125),
    [sym_data_structure_datarecord] = ACTIONS(2125),
    [sym_data_structure_datafield] = ACTIONS(2125),
    [sym_data_max_content_field_count] = ACTIONS(2125),
    [sym_data_max_metadata_field_count] = ACTIONS(2125),
    [sym_data_max_id_length] = ACTIONS(2125),
    [sym_data_max_name_length] = ACTIONS(2125),
    [sym_data_max_description_length] = ACTIONS(2125),
    [sym_data_max_type_length] = ACTIONS(2125),
    [sym_data_max_format_length] = ACTIONS(2125),
    [sym_data_max_source_length] = ACTIONS(2125),
    [sym_data_max_value_length] = ACTIONS(2125),
    [sym_data_max_label_length] = ACTIONS(2125),
    [sym_data_max_url_length] = ACTIONS(2125),
    [sym_max_last_login_info_length] = ACTIONS(2125),
    [sym_data_type_unknown] = ACTIONS(2125),
    [sym_data_type_string] = ACTIONS(2125),
    [sym_data_type_datetime] = ACTIONS(2125),
    [sym_data_type_date] = ACTIONS(2125),
    [sym_data_type_time] = ACTIONS(2125),
    [sym_data_type_image] = ACTIONS(2125),
    [sym_data_format_url] = ACTIONS(2125),
    [sym_data_format_phone] = ACTIONS(2125),
    [sym_data_format_email] = ACTIONS(2125),
    [sym_data_format_iso8601] = ACTIONS(2125),
    [sym_auditlog_priv] = ACTIONS(2125),
    [sym_configuration_priv] = ACTIONS(2125),
    [sym_deviceconfig_priv] = ACTIONS(2125),
    [sym_ftp_priv] = ACTIONS(2125),
    [sym_http_priv] = ACTIONS(2125),
    [sym_networkconfig_priv] = ACTIONS(2125),
    [sym_programport_priv] = ACTIONS(2125),
    [sym_remoteui_priv] = ACTIONS(2125),
    [sym_securitycontrol_priv] = ACTIONS(2125),
    [sym_softwaremanagement_priv] = ACTIONS(2125),
    [sym_terminal_priv] = ACTIONS(2125),
    [sym_tpadmin_priv] = ACTIONS(2125),
    [sym_user1_priv] = ACTIONS(2125),
    [sym_user2_priv] = ACTIONS(2125),
    [sym_user3_priv] = ACTIONS(2125),
    [sym_user4_priv] = ACTIONS(2125),
    [sym_usermanagement_priv] = ACTIONS(2125),
    [sym_valid_account] = ACTIONS(2125),
    [sym_err_name_invalid] = ACTIONS(2125),
    [sym_err_password_invalid] = ACTIONS(2125),
    [sym_err_invalid_account] = ACTIONS(2125),
    [sym_err_no_authorization] = ACTIONS(2125),
    [sym_err_invalid_parameter] = ACTIONS(2125),
    [sym_err_user_locked_out] = ACTIONS(2125),
    [sym_err_user_account_expired] = ACTIONS(2125),
    [sym_err_general_error] = ACTIONS(2125),
    [sym_err_authentication_error] = ACTIONS(2125),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2125),
    [sym_tls_validate_certificate] = ACTIONS(2125),
    [sym_tdata] = ACTIONS(2125),
    [sym_tchannel] = ACTIONS(2125),
    [sym_tlevel] = ACTIONS(2125),
    [sym_tbutton] = ACTIONS(2125),
    [sym_ttimeline] = ACTIONS(2125),
    [sym_tcustom] = ACTIONS(2125),
    [sym_url_struct] = ACTIONS(2125),
    [sym_dns_struct] = ACTIONS(2125),
    [sym_ip_address_struct] = ACTIONS(2125),
    [sym_dev_info_struct] = ACTIONS(2125),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2125),
    [sym_data_feed] = ACTIONS(2125),
    [sym_data_field] = ACTIONS(2125),
    [sym_data_record] = ACTIONS(2125),
    [sym_wc_data_feed] = ACTIONS(2125),
    [sym_wc_data_field] = ACTIONS(2125),
    [sym_wc_data_record] = ACTIONS(2125),
    [sym_last_login_info] = ACTIONS(2125),
    [sym____reserved___] = ACTIONS(2125),
    [sym_date] = ACTIONS(2125),
    [sym_day] = ACTIONS(2125),
    [sym_time] = ACTIONS(2125),
    [sym_ldate] = ACTIONS(2125),
    [sym_push_device] = ACTIONS(2125),
    [sym_push_channel] = ACTIONS(2125),
    [sym_push_devchan] = ACTIONS(2125),
    [sym_release_device] = ACTIONS(2125),
    [sym_release_channel] = ACTIONS(2125),
    [sym_release_devchan] = ACTIONS(2125),
    [sym_master_slot] = ACTIONS(2125),
    [sym_get_pulse_time] = ACTIONS(2125),
    [sym_get_timer] = ACTIONS(2125),
    [sym_master_sn] = ACTIONS(2125),
    [sym_system_number] = ACTIONS(2125),
    [sym_button] = ACTIONS(2125),
    [sym_dv_channel] = ACTIONS(2125),
    [sym_data] = ACTIONS(2125),
    [sym_level] = ACTIONS(2125),
    [sym_timeline] = ACTIONS(2125),
    [sym_channel] = ACTIONS(2125),
    [sym_custom] = ACTIONS(2125),
    [sym_atoi] = ACTIONS(2125),
    [sym_atol] = ACTIONS(2125),
    [sym_atof] = ACTIONS(2125),
    [sym_itoa] = ACTIONS(2125),
    [sym_itohex] = ACTIONS(2125),
    [sym_hextoi] = ACTIONS(2125),
    [sym_ftoa] = ACTIONS(2125),
    [sym_find_string] = ACTIONS(2125),
    [sym_left_string] = ACTIONS(2125),
    [sym_right_string] = ACTIONS(2125),
    [sym_mid_string] = ACTIONS(2125),
    [sym_remove_string] = ACTIONS(2125),
    [sym_get_buffer_string] = ACTIONS(2125),
    [sym_get_buffer_char] = ACTIONS(2125),
    [sym_get_multi_buffer_string] = ACTIONS(2125),
    [sym_length_string] = ACTIONS(2125),
    [sym_max_length_string] = ACTIONS(2125),
    [sym_set_length_string] = ACTIONS(2125),
    [sym_compare_string] = ACTIONS(2125),
    [sym_upper_string] = ACTIONS(2125),
    [sym_lower_string] = ACTIONS(2125),
    [sym_get_last] = ACTIONS(2125),
    [sym_date_to_day] = ACTIONS(2125),
    [sym_date_to_month] = ACTIONS(2125),
    [sym_date_to_year] = ACTIONS(2125),
    [sym_time_to_hour] = ACTIONS(2125),
    [sym_time_to_minute] = ACTIONS(2125),
    [sym_time_to_second] = ACTIONS(2125),
    [sym_day_of_week] = ACTIONS(2125),
    [sym_set_pulse_time] = ACTIONS(2125),
    [sym_set_timer] = ACTIONS(2125),
    [sym_file_open] = ACTIONS(2125),
    [sym_file_close] = ACTIONS(2125),
    [sym_file_read] = ACTIONS(2125),
    [sym_file_read_line] = ACTIONS(2125),
    [sym_file_write] = ACTIONS(2125),
    [sym_file_write_line] = ACTIONS(2125),
    [sym_file_copy] = ACTIONS(2125),
    [sym_file_delete] = ACTIONS(2125),
    [sym_file_rename] = ACTIONS(2125),
    [sym_file_dir] = ACTIONS(2125),
    [sym_file_getdir] = ACTIONS(2125),
    [sym_file_setdir] = ACTIONS(2125),
    [sym_file_create_dir] = ACTIONS(2125),
    [sym_file_remove_dir] = ACTIONS(2125),
    [sym_file_seek] = ACTIONS(2125),
    [sym_length_array] = ACTIONS(2125),
    [sym_max_length_array] = ACTIONS(2125),
    [sym_set_length_array] = ACTIONS(2125),
    [sym_ip_client_open] = ACTIONS(2125),
    [sym_ip_client_close] = ACTIONS(2125),
    [sym_ip_server_open] = ACTIONS(2125),
    [sym_ip_server_close] = ACTIONS(2125),
    [sym_ip_bound_client_open] = ACTIONS(2125),
    [sym_ip_mc_server_open] = ACTIONS(2125),
    [sym_ip_set_option] = ACTIONS(2125),
    [sym_ssh_client_open] = ACTIONS(2125),
    [sym_ssh_client_close] = ACTIONS(2125),
    [sym_tls_client_open] = ACTIONS(2125),
    [sym_tls_client_close] = ACTIONS(2125),
    [sym_do_push] = ACTIONS(2125),
    [sym_do_push_timed] = ACTIONS(2125),
    [sym_do_release] = ACTIONS(2125),
    [sym_device_id] = ACTIONS(2125),
    [sym_device_info] = ACTIONS(2125),
    [sym_device_id_string] = ACTIONS(2125),
    [sym_device_standby] = ACTIONS(2125),
    [sym_device_wake] = ACTIONS(2125),
    [sym_timeline_create] = ACTIONS(2125),
    [sym_timeline_kill] = ACTIONS(2125),
    [sym_timeline_reload] = ACTIONS(2125),
    [sym_timeline_pause] = ACTIONS(2125),
    [sym_timeline_restart] = ACTIONS(2125),
    [sym_timeline_set] = ACTIONS(2125),
    [sym_timeline_get] = ACTIONS(2125),
    [sym_timeline_active] = ACTIONS(2125),
    [sym_timeline_dynamic_id] = ACTIONS(2125),
    [sym_variable_to_string] = ACTIONS(2125),
    [sym_string_to_variable] = ACTIONS(2125),
    [sym_length_variable_to_string] = ACTIONS(2125),
    [sym_type_cast] = ACTIONS(2125),
    [sym_variable_to_xml] = ACTIONS(2125),
    [sym_xml_to_variable] = ACTIONS(2125),
    [sym_length_variable_to_xml] = ACTIONS(2125),
    [sym_redirect_string] = ACTIONS(2125),
    [sym_random_number] = ACTIONS(2125),
    [sym_get_unique_id] = ACTIONS(2125),
    [sym_get_system_number] = ACTIONS(2125),
    [sym_set_system_number] = ACTIONS(2125),
    [sym_get_serial_number] = ACTIONS(2125),
    [sym_get_master_build] = ACTIONS(2125),
    [sym_set_validation_code] = ACTIONS(2125),
    [sym_reboot] = ACTIONS(2125),
    [sym_load_duet_module] = ACTIONS(2125),
    [sym_unload_duet_module] = ACTIONS(2125),
    [sym_duet_mem_size_set] = ACTIONS(2125),
    [sym_duet_mem_size_get] = ACTIONS(2125),
    [sym_dynamic_application_device] = ACTIONS(2125),
    [sym_dynamic_polled_port] = ACTIONS(2125),
    [sym_static_port_binding] = ACTIONS(2125),
    [sym_static_ip_binding] = ACTIONS(2125),
    [sym_get_dns_list] = ACTIONS(2125),
    [sym_set_dns_list] = ACTIONS(2125),
    [sym_get_ip_address] = ACTIONS(2125),
    [sym_set_ip_address] = ACTIONS(2125),
    [sym_get_url_list] = ACTIONS(2125),
    [sym_add_url_entry] = ACTIONS(2125),
    [sym_delete_url_entry] = ACTIONS(2125),
    [sym_abs_value] = ACTIONS(2125),
    [sym_max_value] = ACTIONS(2125),
    [sym_min_value] = ACTIONS(2125),
    [sym_exp_value] = ACTIONS(2125),
    [sym_log_value] = ACTIONS(2125),
    [sym_log10_value] = ACTIONS(2125),
    [sym_power_value] = ACTIONS(2125),
    [sym_sqrt_value] = ACTIONS(2125),
    [sym_raw_be] = ACTIONS(2125),
    [sym_raw_le] = ACTIONS(2125),
    [sym_internal_queue_size_set] = ACTIONS(2125),
    [sym_internal_queue_size_get] = ACTIONS(2125),
    [sym_internal_threshold_set] = ACTIONS(2125),
    [sym_internal_threshold_get] = ACTIONS(2125),
    [sym_rebuilt_event] = ACTIONS(2125),
    [sym_do_custom_event] = ACTIONS(2125),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2125),
    [sym_clkmgr_set_clk_source] = ACTIONS(2125),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2125),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2125),
    [sym_clkmgr_get_timezone] = ACTIONS(2125),
    [sym_clkmgr_set_timezone] = ACTIONS(2125),
    [sym_clkmgr_get_resync_period] = ACTIONS(2125),
    [sym_clkmgr_set_resync_period] = ACTIONS(2125),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2125),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2125),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2125),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2125),
    [sym_clkmgr_get_timeservers] = ACTIONS(2125),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2125),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2125),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2125),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2125),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2125),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2125),
    [sym_set_log_level] = ACTIONS(2125),
    [sym_get_log_level] = ACTIONS(2125),
    [sym_amx_log] = ACTIONS(2125),
    [sym_smtp_server_config_set] = ACTIONS(2125),
    [sym_smtp_server_config_get] = ACTIONS(2125),
    [sym_smtp_send] = ACTIONS(2125),
    [sym_validate_netlinx_account] = ACTIONS(2125),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2125),
    [sym_authenticate_certificate] = ACTIONS(2125),
    [sym_audit_netlinx_session_event] = ACTIONS(2125),
    [sym_audit_netlinx_generic_event] = ACTIONS(2125),
    [sym_get_available_flash_disk_space] = ACTIONS(2125),
    [sym_get_max_flash_disk_space] = ACTIONS(2125),
    [sym_data_create_feed] = ACTIONS(2125),
    [sym_data_delete_feed] = ACTIONS(2125),
    [sym_data_publish_feed] = ACTIONS(2125),
    [sym_data_get_published_feed] = ACTIONS(2125),
    [sym_data_add_record] = ACTIONS(2125),
    [sym_data_get_event_record] = ACTIONS(2125),
    [sym__wc_data_create_feed] = ACTIONS(2125),
    [sym__wc_data_add_record] = ACTIONS(2125),
    [sym__wc_data_get_event_record] = ACTIONS(2125),
    [sym_format] = ACTIONS(2125),
    [sym_astro_clock] = ACTIONS(2125),
    [sym_set_outdoor_temperature] = ACTIONS(2125),
    [sym_set_virtual_level_count] = ACTIONS(2125),
    [sym_set_virtual_channel_count] = ACTIONS(2125),
    [sym_set_virtual_port_count] = ACTIONS(2125),
    [sym__wc_encode_int] = ACTIONS(2125),
    [sym__wc_decode_int] = ACTIONS(2125),
    [sym__wc_int] = ACTIONS(2125),
    [sym__wc_to_ch_int] = ACTIONS(2125),
    [sym__ch_to_wc_int] = ACTIONS(2125),
    [sym__wc_find_string_int] = ACTIONS(2125),
    [sym__wc_left_string_int] = ACTIONS(2125),
    [sym__wc_lower_string_int] = ACTIONS(2125),
    [sym__wc_mid_string_int] = ACTIONS(2125),
    [sym__wc_remove_string_int] = ACTIONS(2125),
    [sym__wc_right_string_int] = ACTIONS(2125),
    [sym__wc_upper_string_int] = ACTIONS(2125),
    [sym__wc_compare_string_int] = ACTIONS(2125),
    [sym__wc_get_buffer_char_int] = ACTIONS(2125),
    [sym__wc_get_buffer_string_int] = ACTIONS(2125),
    [sym__wc_concat_string_int] = ACTIONS(2125),
    [sym__wc_file_open_int] = ACTIONS(2125),
    [sym__wc_file_close_int] = ACTIONS(2125),
    [sym__wc_file_read_int] = ACTIONS(2125),
    [sym__wc_file_read_line_int] = ACTIONS(2125),
    [sym__wc_file_write_int] = ACTIONS(2125),
    [sym__wc_file_write_line_int] = ACTIONS(2125),
    [aux_sym_preproc_include_token1] = ACTIONS(2125),
    [aux_sym_preproc_define_token1] = ACTIONS(2127),
    [aux_sym_preproc_warn_token1] = ACTIONS(2127),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2127),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2127),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2127),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2127),
    [aux_sym_preproc_else_token1] = ACTIONS(2127),
    [anon_sym_LBRACK] = ACTIONS(2127),
    [anon_sym_LBRACE] = ACTIONS(2127),
    [anon_sym_LPAREN] = ACTIONS(2125),
    [anon_sym_BANG] = ACTIONS(2127),
    [anon_sym_TILDE] = ACTIONS(2127),
    [anon_sym_DASH] = ACTIONS(2125),
    [anon_sym_PLUS] = ACTIONS(2125),
    [anon_sym_DASH_DASH] = ACTIONS(2127),
    [anon_sym_PLUS_PLUS] = ACTIONS(2127),
    [anon_sym_DQUOTE] = ACTIONS(2127),
    [anon_sym_SQUOTE] = ACTIONS(2127),
    [sym_number_literal] = ACTIONS(2127),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(382)] = {
    [ts_builtin_sym_end] = ACTIONS(1817),
    [sym_identifier] = ACTIONS(1815),
    [sym_program_name_keyword] = ACTIONS(1815),
    [sym_module_name_keyword] = ACTIONS(1815),
    [sym_define_device_keyword] = ACTIONS(1815),
    [sym_define_combine_keyword] = ACTIONS(1815),
    [sym_define_constant_keyword] = ACTIONS(1815),
    [sym_define_type_keyword] = ACTIONS(1815),
    [sym_define_variable_keyword] = ACTIONS(1815),
    [sym_define_system_variable_keyword] = ACTIONS(1815),
    [sym_define_connect_level_keyword] = ACTIONS(1815),
    [sym_define_latching_keyword] = ACTIONS(1815),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1815),
    [sym_define_toggling_keyword] = ACTIONS(1815),
    [sym_define_call_keyword] = ACTIONS(1815),
    [sym_define_function_keyword] = ACTIONS(1815),
    [sym_define_library_function_keyword] = ACTIONS(1815),
    [sym_define_start_keyword] = ACTIONS(1815),
    [sym_define_event_keyword] = ACTIONS(1815),
    [sym_define_program_keyword] = ACTIONS(1815),
    [sym_define_module_keyword] = ACTIONS(1815),
    [sym_char_keyword] = ACTIONS(1815),
    [sym_widechar_keyword] = ACTIONS(1815),
    [sym_integer_keyword] = ACTIONS(1815),
    [sym_sinteger_keyword] = ACTIONS(1815),
    [sym_long_keyword] = ACTIONS(1815),
    [sym_slong_keyword] = ACTIONS(1815),
    [sym_float_keyword] = ACTIONS(1815),
    [sym_double_keyword] = ACTIONS(1815),
    [sym_variant_keyword] = ACTIONS(1815),
    [sym_variantarray_keyword] = ACTIONS(1815),
    [sym_local_var_keyword] = ACTIONS(1815),
    [sym_stack_var_keyword] = ACTIONS(1815),
    [sym_constant_keyword] = ACTIONS(1815),
    [sym_volatile_keyword] = ACTIONS(1815),
    [sym_non_volatile_keyword] = ACTIONS(1815),
    [sym_persistent_keyword] = ACTIONS(1815),
    [sym_dev_keyword] = ACTIONS(1815),
    [sym_devlev_keyword] = ACTIONS(1815),
    [sym_devchan_keyword] = ACTIONS(1815),
    [sym_if_keyword] = ACTIONS(1815),
    [sym_else_keyword] = ACTIONS(1815),
    [sym_switch_keyword] = ACTIONS(1815),
    [sym_case_keyword] = ACTIONS(1815),
    [sym_default_keyword] = ACTIONS(1815),
    [sym_while_keyword] = ACTIONS(1815),
    [sym_for_keyword] = ACTIONS(1815),
    [sym_break_keyword] = ACTIONS(1815),
    [sym_continue_keyword] = ACTIONS(1815),
    [sym_return_keyword] = ACTIONS(1815),
    [sym_select_keyword] = ACTIONS(1815),
    [sym_active_keyword] = ACTIONS(1815),
    [sym_struct_keyword] = ACTIONS(1815),
    [sym_structure_keyword] = ACTIONS(1815),
    [sym_button_event_keyword] = ACTIONS(1815),
    [sym_channel_event_keyword] = ACTIONS(1815),
    [sym_level_event_keyword] = ACTIONS(1815),
    [sym_data_event_keyword] = ACTIONS(1815),
    [sym_timeline_event_keyword] = ACTIONS(1815),
    [sym_custom_event_keyword] = ACTIONS(1815),
    [sym_push_keyword] = ACTIONS(1815),
    [sym_release_keyword] = ACTIONS(1815),
    [aux_sym_on_keyword_token1] = ACTIONS(1815),
    [aux_sym_off_keyword_token1] = ACTIONS(1815),
    [sym_send_level_keyword] = ACTIONS(1815),
    [sym_send_string_keyword] = ACTIONS(1815),
    [sym_send_command_keyword] = ACTIONS(1815),
    [sym_clear_buffer_keyword] = ACTIONS(1815),
    [sym_create_buffer_keyword] = ACTIONS(1815),
    [sym_create_multi_buffer_keyword] = ACTIONS(1815),
    [sym_call_keyword] = ACTIONS(1815),
    [sym_system_call_keyword] = ACTIONS(1815),
    [sym_devchan_to_keyword] = ACTIONS(1815),
    [sym_devchan_min_to_keyword] = ACTIONS(1815),
    [sym_devchan_total_off_keyword] = ACTIONS(1815),
    [sym_devchan_pulse_keyword] = ACTIONS(1815),
    [sym_wait_keyword] = ACTIONS(1815),
    [sym_cancel_wait_keyword] = ACTIONS(1815),
    [sym_wait_until_keyword] = ACTIONS(1815),
    [sym_cancel_wait_until_keyword] = ACTIONS(1815),
    [sym_cancel_all_wait_keyword] = ACTIONS(1815),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1815),
    [sym_not] = ACTIONS(1815),
    [sym_bnot] = ACTIONS(1815),
    [sym___file__] = ACTIONS(1815),
    [sym___date__] = ACTIONS(1815),
    [sym___ldate__] = ACTIONS(1815),
    [sym___time__] = ACTIONS(1815),
    [sym___line__] = ACTIONS(1815),
    [sym___name__] = ACTIONS(1815),
    [sym___netlinx__] = ACTIONS(1815),
    [sym_netlinx_axi_version] = ACTIONS(1815),
    [sym_url_flg_tcp] = ACTIONS(1815),
    [sym_url_flg_accinfopresent] = ACTIONS(1815),
    [sym_url_flg_temp] = ACTIONS(1815),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1815),
    [sym_url_flg_stat_mask] = ACTIONS(1815),
    [sym_url_flg_stat_lookup] = ACTIONS(1815),
    [sym_url_flg_stat_connecting] = ACTIONS(1815),
    [sym_url_flg_stat_waiting] = ACTIONS(1815),
    [sym_url_flg_stat_connected] = ACTIONS(1815),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1815),
    [sym_first_virtual_device] = ACTIONS(1815),
    [sym_first_local_port] = ACTIONS(1815),
    [sym_dynamic_virtual_device] = ACTIONS(1815),
    [sym_timeline_once] = ACTIONS(1815),
    [sym_timeline_repeat] = ACTIONS(1815),
    [sym_timeline_absolute] = ACTIONS(1815),
    [sym_timeline_relative] = ACTIONS(1815),
    [sym_do_push_timed_infinite] = ACTIONS(1815),
    [sym_source_type_no_address] = ACTIONS(1815),
    [sym_source_type_neuron_id] = ACTIONS(1815),
    [sym_source_type_ip_address] = ACTIONS(1815),
    [sym_source_type_axlink] = ACTIONS(1815),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1815),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1815),
    [sym_source_type_ip_socket_address] = ACTIONS(1815),
    [sym_source_type_rs232] = ACTIONS(1815),
    [sym_source_type_internal] = ACTIONS(1815),
    [sym_source_type_mac_address] = ACTIONS(1815),
    [sym_source_type_ipv4_port] = ACTIONS(1815),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1815),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1815),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1815),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1815),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1815),
    [sym_true] = ACTIONS(1815),
    [sym_false] = ACTIONS(1815),
    [sym_file_read_only] = ACTIONS(1815),
    [sym_file_rw_new] = ACTIONS(1815),
    [sym_file_rw_append] = ACTIONS(1815),
    [sym_ip_tcp] = ACTIONS(1815),
    [sym_ip_udp] = ACTIONS(1815),
    [sym_ip_udp_2way] = ACTIONS(1815),
    [sym_xml_encode_types] = ACTIONS(1815),
    [sym_xml_encode_char_as_list] = ACTIONS(1815),
    [sym_xml_encode_le] = ACTIONS(1815),
    [sym_xml_decode_types] = ACTIONS(1815),
    [sym_xml_decode_no_preserve] = ACTIONS(1815),
    [sym_duet_dev_type_display_device] = ACTIONS(1815),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1815),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1815),
    [sym_duet_dev_type_io_device] = ACTIONS(1815),
    [sym_duet_dev_type_relay_device] = ACTIONS(1815),
    [sym_duet_dev_type_ups] = ACTIONS(1815),
    [sym_duet_dev_type_amplifier] = ACTIONS(1815),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1815),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1815),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1815),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1815),
    [sym_duet_dev_type_camera] = ACTIONS(1815),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1815),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1815),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1815),
    [sym_duet_dev_type_dss] = ACTIONS(1815),
    [sym_duet_dev_type_dvr] = ACTIONS(1815),
    [sym_duet_dev_type_disc_device] = ACTIONS(1815),
    [sym_duet_dev_type_document_camera] = ACTIONS(1815),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1815),
    [sym_duet_dev_type_hvac] = ACTIONS(1815),
    [sym_duet_dev_type_keypad] = ACTIONS(1815),
    [sym_duet_dev_type_light] = ACTIONS(1815),
    [sym_duet_dev_type_monitor] = ACTIONS(1815),
    [sym_duet_dev_type_motor] = ACTIONS(1815),
    [sym_duet_dev_type_multi_window] = ACTIONS(1815),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1815),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1815),
    [sym_duet_dev_type_receiver] = ACTIONS(1815),
    [sym_duet_dev_type_security_system] = ACTIONS(1815),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1815),
    [sym_duet_dev_type_settop_box] = ACTIONS(1815),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1815),
    [sym_duet_dev_type_switcher] = ACTIONS(1815),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1815),
    [sym_duet_dev_type_tv] = ACTIONS(1815),
    [sym_duet_dev_type_utility] = ACTIONS(1815),
    [sym_duet_dev_type_vcr] = ACTIONS(1815),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1815),
    [sym_duet_dev_type_video_processor] = ACTIONS(1815),
    [sym_duet_dev_type_video_projector] = ACTIONS(1815),
    [sym_duet_dev_type_video_wall] = ACTIONS(1815),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1815),
    [sym_duet_dev_type_weather] = ACTIONS(1815),
    [sym_duet_dev_not_polled] = ACTIONS(1815),
    [sym_duet_dev_polled] = ACTIONS(1815),
    [sym_clkmgr_mode_network] = ACTIONS(1815),
    [sym_clkmgr_mode_standalone] = ACTIONS(1815),
    [sym_normal_standby] = ACTIONS(1815),
    [sym_normal_wake] = ACTIONS(1815),
    [sym_ip_multicast_ttl_option] = ACTIONS(1815),
    [sym_ip_ttl_subnet] = ACTIONS(1815),
    [sym_ip_ttl_site] = ACTIONS(1815),
    [sym_ip_ttl_region] = ACTIONS(1815),
    [sym_ip_ttl_continent] = ACTIONS(1815),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1815),
    [sym_ip_nodelay_on] = ACTIONS(1815),
    [sym_ip_nodelay_off] = ACTIONS(1815),
    [sym_amx_error] = ACTIONS(1815),
    [sym_amx_warning] = ACTIONS(1815),
    [sym_amx_info] = ACTIONS(1815),
    [sym_amx_debug] = ACTIONS(1815),
    [sym_netlinx_login_success] = ACTIONS(1815),
    [sym_netlinx_login_fail] = ACTIONS(1815),
    [sym_netlinx_logout] = ACTIONS(1815),
    [sym_null_str] = ACTIONS(1815),
    [sym_smtp_address] = ACTIONS(1815),
    [sym_smtp_port_number] = ACTIONS(1815),
    [sym_smtp_username] = ACTIONS(1815),
    [sym_smtp_password] = ACTIONS(1815),
    [sym_smtp_require_tls] = ACTIONS(1815),
    [sym_smtp_from] = ACTIONS(1815),
    [sym_smtp_tls_true] = ACTIONS(1815),
    [sym_smtp_tls_false] = ACTIONS(1815),
    [sym_listview_on_row_select_event] = ACTIONS(1815),
    [sym_data_structure_unknown] = ACTIONS(1815),
    [sym_data_structure_datafeed] = ACTIONS(1815),
    [sym_data_structure_datarecordset] = ACTIONS(1815),
    [sym_data_structure_datarecord] = ACTIONS(1815),
    [sym_data_structure_datafield] = ACTIONS(1815),
    [sym_data_max_content_field_count] = ACTIONS(1815),
    [sym_data_max_metadata_field_count] = ACTIONS(1815),
    [sym_data_max_id_length] = ACTIONS(1815),
    [sym_data_max_name_length] = ACTIONS(1815),
    [sym_data_max_description_length] = ACTIONS(1815),
    [sym_data_max_type_length] = ACTIONS(1815),
    [sym_data_max_format_length] = ACTIONS(1815),
    [sym_data_max_source_length] = ACTIONS(1815),
    [sym_data_max_value_length] = ACTIONS(1815),
    [sym_data_max_label_length] = ACTIONS(1815),
    [sym_data_max_url_length] = ACTIONS(1815),
    [sym_max_last_login_info_length] = ACTIONS(1815),
    [sym_data_type_unknown] = ACTIONS(1815),
    [sym_data_type_string] = ACTIONS(1815),
    [sym_data_type_datetime] = ACTIONS(1815),
    [sym_data_type_date] = ACTIONS(1815),
    [sym_data_type_time] = ACTIONS(1815),
    [sym_data_type_image] = ACTIONS(1815),
    [sym_data_format_url] = ACTIONS(1815),
    [sym_data_format_phone] = ACTIONS(1815),
    [sym_data_format_email] = ACTIONS(1815),
    [sym_data_format_iso8601] = ACTIONS(1815),
    [sym_auditlog_priv] = ACTIONS(1815),
    [sym_configuration_priv] = ACTIONS(1815),
    [sym_deviceconfig_priv] = ACTIONS(1815),
    [sym_ftp_priv] = ACTIONS(1815),
    [sym_http_priv] = ACTIONS(1815),
    [sym_networkconfig_priv] = ACTIONS(1815),
    [sym_programport_priv] = ACTIONS(1815),
    [sym_remoteui_priv] = ACTIONS(1815),
    [sym_securitycontrol_priv] = ACTIONS(1815),
    [sym_softwaremanagement_priv] = ACTIONS(1815),
    [sym_terminal_priv] = ACTIONS(1815),
    [sym_tpadmin_priv] = ACTIONS(1815),
    [sym_user1_priv] = ACTIONS(1815),
    [sym_user2_priv] = ACTIONS(1815),
    [sym_user3_priv] = ACTIONS(1815),
    [sym_user4_priv] = ACTIONS(1815),
    [sym_usermanagement_priv] = ACTIONS(1815),
    [sym_valid_account] = ACTIONS(1815),
    [sym_err_name_invalid] = ACTIONS(1815),
    [sym_err_password_invalid] = ACTIONS(1815),
    [sym_err_invalid_account] = ACTIONS(1815),
    [sym_err_no_authorization] = ACTIONS(1815),
    [sym_err_invalid_parameter] = ACTIONS(1815),
    [sym_err_user_locked_out] = ACTIONS(1815),
    [sym_err_user_account_expired] = ACTIONS(1815),
    [sym_err_general_error] = ACTIONS(1815),
    [sym_err_authentication_error] = ACTIONS(1815),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1815),
    [sym_tls_validate_certificate] = ACTIONS(1815),
    [sym_tdata] = ACTIONS(1815),
    [sym_tchannel] = ACTIONS(1815),
    [sym_tlevel] = ACTIONS(1815),
    [sym_tbutton] = ACTIONS(1815),
    [sym_ttimeline] = ACTIONS(1815),
    [sym_tcustom] = ACTIONS(1815),
    [sym_url_struct] = ACTIONS(1815),
    [sym_dns_struct] = ACTIONS(1815),
    [sym_ip_address_struct] = ACTIONS(1815),
    [sym_dev_info_struct] = ACTIONS(1815),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1815),
    [sym_data_feed] = ACTIONS(1815),
    [sym_data_field] = ACTIONS(1815),
    [sym_data_record] = ACTIONS(1815),
    [sym_wc_data_feed] = ACTIONS(1815),
    [sym_wc_data_field] = ACTIONS(1815),
    [sym_wc_data_record] = ACTIONS(1815),
    [sym_last_login_info] = ACTIONS(1815),
    [sym____reserved___] = ACTIONS(1815),
    [sym_date] = ACTIONS(1815),
    [sym_day] = ACTIONS(1815),
    [sym_time] = ACTIONS(1815),
    [sym_ldate] = ACTIONS(1815),
    [sym_push_device] = ACTIONS(1815),
    [sym_push_channel] = ACTIONS(1815),
    [sym_push_devchan] = ACTIONS(1815),
    [sym_release_device] = ACTIONS(1815),
    [sym_release_channel] = ACTIONS(1815),
    [sym_release_devchan] = ACTIONS(1815),
    [sym_master_slot] = ACTIONS(1815),
    [sym_get_pulse_time] = ACTIONS(1815),
    [sym_get_timer] = ACTIONS(1815),
    [sym_master_sn] = ACTIONS(1815),
    [sym_system_number] = ACTIONS(1815),
    [sym_button] = ACTIONS(1815),
    [sym_dv_channel] = ACTIONS(1815),
    [sym_data] = ACTIONS(1815),
    [sym_level] = ACTIONS(1815),
    [sym_timeline] = ACTIONS(1815),
    [sym_channel] = ACTIONS(1815),
    [sym_custom] = ACTIONS(1815),
    [sym_atoi] = ACTIONS(1815),
    [sym_atol] = ACTIONS(1815),
    [sym_atof] = ACTIONS(1815),
    [sym_itoa] = ACTIONS(1815),
    [sym_itohex] = ACTIONS(1815),
    [sym_hextoi] = ACTIONS(1815),
    [sym_ftoa] = ACTIONS(1815),
    [sym_find_string] = ACTIONS(1815),
    [sym_left_string] = ACTIONS(1815),
    [sym_right_string] = ACTIONS(1815),
    [sym_mid_string] = ACTIONS(1815),
    [sym_remove_string] = ACTIONS(1815),
    [sym_get_buffer_string] = ACTIONS(1815),
    [sym_get_buffer_char] = ACTIONS(1815),
    [sym_get_multi_buffer_string] = ACTIONS(1815),
    [sym_length_string] = ACTIONS(1815),
    [sym_max_length_string] = ACTIONS(1815),
    [sym_set_length_string] = ACTIONS(1815),
    [sym_compare_string] = ACTIONS(1815),
    [sym_upper_string] = ACTIONS(1815),
    [sym_lower_string] = ACTIONS(1815),
    [sym_get_last] = ACTIONS(1815),
    [sym_date_to_day] = ACTIONS(1815),
    [sym_date_to_month] = ACTIONS(1815),
    [sym_date_to_year] = ACTIONS(1815),
    [sym_time_to_hour] = ACTIONS(1815),
    [sym_time_to_minute] = ACTIONS(1815),
    [sym_time_to_second] = ACTIONS(1815),
    [sym_day_of_week] = ACTIONS(1815),
    [sym_set_pulse_time] = ACTIONS(1815),
    [sym_set_timer] = ACTIONS(1815),
    [sym_file_open] = ACTIONS(1815),
    [sym_file_close] = ACTIONS(1815),
    [sym_file_read] = ACTIONS(1815),
    [sym_file_read_line] = ACTIONS(1815),
    [sym_file_write] = ACTIONS(1815),
    [sym_file_write_line] = ACTIONS(1815),
    [sym_file_copy] = ACTIONS(1815),
    [sym_file_delete] = ACTIONS(1815),
    [sym_file_rename] = ACTIONS(1815),
    [sym_file_dir] = ACTIONS(1815),
    [sym_file_getdir] = ACTIONS(1815),
    [sym_file_setdir] = ACTIONS(1815),
    [sym_file_create_dir] = ACTIONS(1815),
    [sym_file_remove_dir] = ACTIONS(1815),
    [sym_file_seek] = ACTIONS(1815),
    [sym_length_array] = ACTIONS(1815),
    [sym_max_length_array] = ACTIONS(1815),
    [sym_set_length_array] = ACTIONS(1815),
    [sym_ip_client_open] = ACTIONS(1815),
    [sym_ip_client_close] = ACTIONS(1815),
    [sym_ip_server_open] = ACTIONS(1815),
    [sym_ip_server_close] = ACTIONS(1815),
    [sym_ip_bound_client_open] = ACTIONS(1815),
    [sym_ip_mc_server_open] = ACTIONS(1815),
    [sym_ip_set_option] = ACTIONS(1815),
    [sym_ssh_client_open] = ACTIONS(1815),
    [sym_ssh_client_close] = ACTIONS(1815),
    [sym_tls_client_open] = ACTIONS(1815),
    [sym_tls_client_close] = ACTIONS(1815),
    [sym_do_push] = ACTIONS(1815),
    [sym_do_push_timed] = ACTIONS(1815),
    [sym_do_release] = ACTIONS(1815),
    [sym_device_id] = ACTIONS(1815),
    [sym_device_info] = ACTIONS(1815),
    [sym_device_id_string] = ACTIONS(1815),
    [sym_device_standby] = ACTIONS(1815),
    [sym_device_wake] = ACTIONS(1815),
    [sym_timeline_create] = ACTIONS(1815),
    [sym_timeline_kill] = ACTIONS(1815),
    [sym_timeline_reload] = ACTIONS(1815),
    [sym_timeline_pause] = ACTIONS(1815),
    [sym_timeline_restart] = ACTIONS(1815),
    [sym_timeline_set] = ACTIONS(1815),
    [sym_timeline_get] = ACTIONS(1815),
    [sym_timeline_active] = ACTIONS(1815),
    [sym_timeline_dynamic_id] = ACTIONS(1815),
    [sym_variable_to_string] = ACTIONS(1815),
    [sym_string_to_variable] = ACTIONS(1815),
    [sym_length_variable_to_string] = ACTIONS(1815),
    [sym_type_cast] = ACTIONS(1815),
    [sym_variable_to_xml] = ACTIONS(1815),
    [sym_xml_to_variable] = ACTIONS(1815),
    [sym_length_variable_to_xml] = ACTIONS(1815),
    [sym_redirect_string] = ACTIONS(1815),
    [sym_random_number] = ACTIONS(1815),
    [sym_get_unique_id] = ACTIONS(1815),
    [sym_get_system_number] = ACTIONS(1815),
    [sym_set_system_number] = ACTIONS(1815),
    [sym_get_serial_number] = ACTIONS(1815),
    [sym_get_master_build] = ACTIONS(1815),
    [sym_set_validation_code] = ACTIONS(1815),
    [sym_reboot] = ACTIONS(1815),
    [sym_load_duet_module] = ACTIONS(1815),
    [sym_unload_duet_module] = ACTIONS(1815),
    [sym_duet_mem_size_set] = ACTIONS(1815),
    [sym_duet_mem_size_get] = ACTIONS(1815),
    [sym_dynamic_application_device] = ACTIONS(1815),
    [sym_dynamic_polled_port] = ACTIONS(1815),
    [sym_static_port_binding] = ACTIONS(1815),
    [sym_static_ip_binding] = ACTIONS(1815),
    [sym_get_dns_list] = ACTIONS(1815),
    [sym_set_dns_list] = ACTIONS(1815),
    [sym_get_ip_address] = ACTIONS(1815),
    [sym_set_ip_address] = ACTIONS(1815),
    [sym_get_url_list] = ACTIONS(1815),
    [sym_add_url_entry] = ACTIONS(1815),
    [sym_delete_url_entry] = ACTIONS(1815),
    [sym_abs_value] = ACTIONS(1815),
    [sym_max_value] = ACTIONS(1815),
    [sym_min_value] = ACTIONS(1815),
    [sym_exp_value] = ACTIONS(1815),
    [sym_log_value] = ACTIONS(1815),
    [sym_log10_value] = ACTIONS(1815),
    [sym_power_value] = ACTIONS(1815),
    [sym_sqrt_value] = ACTIONS(1815),
    [sym_raw_be] = ACTIONS(1815),
    [sym_raw_le] = ACTIONS(1815),
    [sym_internal_queue_size_set] = ACTIONS(1815),
    [sym_internal_queue_size_get] = ACTIONS(1815),
    [sym_internal_threshold_set] = ACTIONS(1815),
    [sym_internal_threshold_get] = ACTIONS(1815),
    [sym_rebuilt_event] = ACTIONS(1815),
    [sym_do_custom_event] = ACTIONS(1815),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1815),
    [sym_clkmgr_set_clk_source] = ACTIONS(1815),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1815),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1815),
    [sym_clkmgr_get_timezone] = ACTIONS(1815),
    [sym_clkmgr_set_timezone] = ACTIONS(1815),
    [sym_clkmgr_get_resync_period] = ACTIONS(1815),
    [sym_clkmgr_set_resync_period] = ACTIONS(1815),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1815),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1815),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1815),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1815),
    [sym_clkmgr_get_timeservers] = ACTIONS(1815),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1815),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1815),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1815),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1815),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1815),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1815),
    [sym_set_log_level] = ACTIONS(1815),
    [sym_get_log_level] = ACTIONS(1815),
    [sym_amx_log] = ACTIONS(1815),
    [sym_smtp_server_config_set] = ACTIONS(1815),
    [sym_smtp_server_config_get] = ACTIONS(1815),
    [sym_smtp_send] = ACTIONS(1815),
    [sym_validate_netlinx_account] = ACTIONS(1815),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1815),
    [sym_authenticate_certificate] = ACTIONS(1815),
    [sym_audit_netlinx_session_event] = ACTIONS(1815),
    [sym_audit_netlinx_generic_event] = ACTIONS(1815),
    [sym_get_available_flash_disk_space] = ACTIONS(1815),
    [sym_get_max_flash_disk_space] = ACTIONS(1815),
    [sym_data_create_feed] = ACTIONS(1815),
    [sym_data_delete_feed] = ACTIONS(1815),
    [sym_data_publish_feed] = ACTIONS(1815),
    [sym_data_get_published_feed] = ACTIONS(1815),
    [sym_data_add_record] = ACTIONS(1815),
    [sym_data_get_event_record] = ACTIONS(1815),
    [sym__wc_data_create_feed] = ACTIONS(1815),
    [sym__wc_data_add_record] = ACTIONS(1815),
    [sym__wc_data_get_event_record] = ACTIONS(1815),
    [sym_format] = ACTIONS(1815),
    [sym_astro_clock] = ACTIONS(1815),
    [sym_set_outdoor_temperature] = ACTIONS(1815),
    [sym_set_virtual_level_count] = ACTIONS(1815),
    [sym_set_virtual_channel_count] = ACTIONS(1815),
    [sym_set_virtual_port_count] = ACTIONS(1815),
    [sym__wc_encode_int] = ACTIONS(1815),
    [sym__wc_decode_int] = ACTIONS(1815),
    [sym__wc_int] = ACTIONS(1815),
    [sym__wc_to_ch_int] = ACTIONS(1815),
    [sym__ch_to_wc_int] = ACTIONS(1815),
    [sym__wc_find_string_int] = ACTIONS(1815),
    [sym__wc_left_string_int] = ACTIONS(1815),
    [sym__wc_lower_string_int] = ACTIONS(1815),
    [sym__wc_mid_string_int] = ACTIONS(1815),
    [sym__wc_remove_string_int] = ACTIONS(1815),
    [sym__wc_right_string_int] = ACTIONS(1815),
    [sym__wc_upper_string_int] = ACTIONS(1815),
    [sym__wc_compare_string_int] = ACTIONS(1815),
    [sym__wc_get_buffer_char_int] = ACTIONS(1815),
    [sym__wc_get_buffer_string_int] = ACTIONS(1815),
    [sym__wc_concat_string_int] = ACTIONS(1815),
    [sym__wc_file_open_int] = ACTIONS(1815),
    [sym__wc_file_close_int] = ACTIONS(1815),
    [sym__wc_file_read_int] = ACTIONS(1815),
    [sym__wc_file_read_line_int] = ACTIONS(1815),
    [sym__wc_file_write_int] = ACTIONS(1815),
    [sym__wc_file_write_line_int] = ACTIONS(1815),
    [aux_sym_preproc_include_token1] = ACTIONS(1815),
    [aux_sym_preproc_define_token1] = ACTIONS(1817),
    [aux_sym_preproc_warn_token1] = ACTIONS(1817),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1817),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1817),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1817),
    [anon_sym_LBRACK] = ACTIONS(1817),
    [anon_sym_LBRACE] = ACTIONS(1817),
    [anon_sym_LPAREN] = ACTIONS(1815),
    [anon_sym_BANG] = ACTIONS(1817),
    [anon_sym_TILDE] = ACTIONS(1817),
    [anon_sym_DASH] = ACTIONS(1815),
    [anon_sym_PLUS] = ACTIONS(1815),
    [anon_sym_DASH_DASH] = ACTIONS(1817),
    [anon_sym_PLUS_PLUS] = ACTIONS(1817),
    [anon_sym_DQUOTE] = ACTIONS(1817),
    [anon_sym_SQUOTE] = ACTIONS(1817),
    [sym_number_literal] = ACTIONS(1817),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(383)] = {
    [ts_builtin_sym_end] = ACTIONS(1767),
    [sym_identifier] = ACTIONS(1765),
    [sym_program_name_keyword] = ACTIONS(1765),
    [sym_module_name_keyword] = ACTIONS(1765),
    [sym_define_device_keyword] = ACTIONS(1765),
    [sym_define_combine_keyword] = ACTIONS(1765),
    [sym_define_constant_keyword] = ACTIONS(1765),
    [sym_define_type_keyword] = ACTIONS(1765),
    [sym_define_variable_keyword] = ACTIONS(1765),
    [sym_define_system_variable_keyword] = ACTIONS(1765),
    [sym_define_connect_level_keyword] = ACTIONS(1765),
    [sym_define_latching_keyword] = ACTIONS(1765),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1765),
    [sym_define_toggling_keyword] = ACTIONS(1765),
    [sym_define_call_keyword] = ACTIONS(1765),
    [sym_define_function_keyword] = ACTIONS(1765),
    [sym_define_library_function_keyword] = ACTIONS(1765),
    [sym_define_start_keyword] = ACTIONS(1765),
    [sym_define_event_keyword] = ACTIONS(1765),
    [sym_define_program_keyword] = ACTIONS(1765),
    [sym_define_module_keyword] = ACTIONS(1765),
    [sym_char_keyword] = ACTIONS(1765),
    [sym_widechar_keyword] = ACTIONS(1765),
    [sym_integer_keyword] = ACTIONS(1765),
    [sym_sinteger_keyword] = ACTIONS(1765),
    [sym_long_keyword] = ACTIONS(1765),
    [sym_slong_keyword] = ACTIONS(1765),
    [sym_float_keyword] = ACTIONS(1765),
    [sym_double_keyword] = ACTIONS(1765),
    [sym_variant_keyword] = ACTIONS(1765),
    [sym_variantarray_keyword] = ACTIONS(1765),
    [sym_local_var_keyword] = ACTIONS(1765),
    [sym_stack_var_keyword] = ACTIONS(1765),
    [sym_constant_keyword] = ACTIONS(1765),
    [sym_volatile_keyword] = ACTIONS(1765),
    [sym_non_volatile_keyword] = ACTIONS(1765),
    [sym_persistent_keyword] = ACTIONS(1765),
    [sym_dev_keyword] = ACTIONS(1765),
    [sym_devlev_keyword] = ACTIONS(1765),
    [sym_devchan_keyword] = ACTIONS(1765),
    [sym_if_keyword] = ACTIONS(1765),
    [sym_else_keyword] = ACTIONS(1765),
    [sym_switch_keyword] = ACTIONS(1765),
    [sym_case_keyword] = ACTIONS(1765),
    [sym_default_keyword] = ACTIONS(1765),
    [sym_while_keyword] = ACTIONS(1765),
    [sym_for_keyword] = ACTIONS(1765),
    [sym_break_keyword] = ACTIONS(1765),
    [sym_continue_keyword] = ACTIONS(1765),
    [sym_return_keyword] = ACTIONS(1765),
    [sym_select_keyword] = ACTIONS(1765),
    [sym_active_keyword] = ACTIONS(1765),
    [sym_struct_keyword] = ACTIONS(1765),
    [sym_structure_keyword] = ACTIONS(1765),
    [sym_button_event_keyword] = ACTIONS(1765),
    [sym_channel_event_keyword] = ACTIONS(1765),
    [sym_level_event_keyword] = ACTIONS(1765),
    [sym_data_event_keyword] = ACTIONS(1765),
    [sym_timeline_event_keyword] = ACTIONS(1765),
    [sym_custom_event_keyword] = ACTIONS(1765),
    [sym_push_keyword] = ACTIONS(1765),
    [sym_release_keyword] = ACTIONS(1765),
    [aux_sym_on_keyword_token1] = ACTIONS(1765),
    [aux_sym_off_keyword_token1] = ACTIONS(1765),
    [sym_send_level_keyword] = ACTIONS(1765),
    [sym_send_string_keyword] = ACTIONS(1765),
    [sym_send_command_keyword] = ACTIONS(1765),
    [sym_clear_buffer_keyword] = ACTIONS(1765),
    [sym_create_buffer_keyword] = ACTIONS(1765),
    [sym_create_multi_buffer_keyword] = ACTIONS(1765),
    [sym_call_keyword] = ACTIONS(1765),
    [sym_system_call_keyword] = ACTIONS(1765),
    [sym_devchan_to_keyword] = ACTIONS(1765),
    [sym_devchan_min_to_keyword] = ACTIONS(1765),
    [sym_devchan_total_off_keyword] = ACTIONS(1765),
    [sym_devchan_pulse_keyword] = ACTIONS(1765),
    [sym_wait_keyword] = ACTIONS(1765),
    [sym_cancel_wait_keyword] = ACTIONS(1765),
    [sym_wait_until_keyword] = ACTIONS(1765),
    [sym_cancel_wait_until_keyword] = ACTIONS(1765),
    [sym_cancel_all_wait_keyword] = ACTIONS(1765),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1765),
    [sym_not] = ACTIONS(1765),
    [sym_bnot] = ACTIONS(1765),
    [sym___file__] = ACTIONS(1765),
    [sym___date__] = ACTIONS(1765),
    [sym___ldate__] = ACTIONS(1765),
    [sym___time__] = ACTIONS(1765),
    [sym___line__] = ACTIONS(1765),
    [sym___name__] = ACTIONS(1765),
    [sym___netlinx__] = ACTIONS(1765),
    [sym_netlinx_axi_version] = ACTIONS(1765),
    [sym_url_flg_tcp] = ACTIONS(1765),
    [sym_url_flg_accinfopresent] = ACTIONS(1765),
    [sym_url_flg_temp] = ACTIONS(1765),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1765),
    [sym_url_flg_stat_mask] = ACTIONS(1765),
    [sym_url_flg_stat_lookup] = ACTIONS(1765),
    [sym_url_flg_stat_connecting] = ACTIONS(1765),
    [sym_url_flg_stat_waiting] = ACTIONS(1765),
    [sym_url_flg_stat_connected] = ACTIONS(1765),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1765),
    [sym_first_virtual_device] = ACTIONS(1765),
    [sym_first_local_port] = ACTIONS(1765),
    [sym_dynamic_virtual_device] = ACTIONS(1765),
    [sym_timeline_once] = ACTIONS(1765),
    [sym_timeline_repeat] = ACTIONS(1765),
    [sym_timeline_absolute] = ACTIONS(1765),
    [sym_timeline_relative] = ACTIONS(1765),
    [sym_do_push_timed_infinite] = ACTIONS(1765),
    [sym_source_type_no_address] = ACTIONS(1765),
    [sym_source_type_neuron_id] = ACTIONS(1765),
    [sym_source_type_ip_address] = ACTIONS(1765),
    [sym_source_type_axlink] = ACTIONS(1765),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1765),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1765),
    [sym_source_type_ip_socket_address] = ACTIONS(1765),
    [sym_source_type_rs232] = ACTIONS(1765),
    [sym_source_type_internal] = ACTIONS(1765),
    [sym_source_type_mac_address] = ACTIONS(1765),
    [sym_source_type_ipv4_port] = ACTIONS(1765),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1765),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1765),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1765),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1765),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1765),
    [sym_true] = ACTIONS(1765),
    [sym_false] = ACTIONS(1765),
    [sym_file_read_only] = ACTIONS(1765),
    [sym_file_rw_new] = ACTIONS(1765),
    [sym_file_rw_append] = ACTIONS(1765),
    [sym_ip_tcp] = ACTIONS(1765),
    [sym_ip_udp] = ACTIONS(1765),
    [sym_ip_udp_2way] = ACTIONS(1765),
    [sym_xml_encode_types] = ACTIONS(1765),
    [sym_xml_encode_char_as_list] = ACTIONS(1765),
    [sym_xml_encode_le] = ACTIONS(1765),
    [sym_xml_decode_types] = ACTIONS(1765),
    [sym_xml_decode_no_preserve] = ACTIONS(1765),
    [sym_duet_dev_type_display_device] = ACTIONS(1765),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1765),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1765),
    [sym_duet_dev_type_io_device] = ACTIONS(1765),
    [sym_duet_dev_type_relay_device] = ACTIONS(1765),
    [sym_duet_dev_type_ups] = ACTIONS(1765),
    [sym_duet_dev_type_amplifier] = ACTIONS(1765),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1765),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1765),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1765),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1765),
    [sym_duet_dev_type_camera] = ACTIONS(1765),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1765),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1765),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1765),
    [sym_duet_dev_type_dss] = ACTIONS(1765),
    [sym_duet_dev_type_dvr] = ACTIONS(1765),
    [sym_duet_dev_type_disc_device] = ACTIONS(1765),
    [sym_duet_dev_type_document_camera] = ACTIONS(1765),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1765),
    [sym_duet_dev_type_hvac] = ACTIONS(1765),
    [sym_duet_dev_type_keypad] = ACTIONS(1765),
    [sym_duet_dev_type_light] = ACTIONS(1765),
    [sym_duet_dev_type_monitor] = ACTIONS(1765),
    [sym_duet_dev_type_motor] = ACTIONS(1765),
    [sym_duet_dev_type_multi_window] = ACTIONS(1765),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1765),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1765),
    [sym_duet_dev_type_receiver] = ACTIONS(1765),
    [sym_duet_dev_type_security_system] = ACTIONS(1765),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1765),
    [sym_duet_dev_type_settop_box] = ACTIONS(1765),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1765),
    [sym_duet_dev_type_switcher] = ACTIONS(1765),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1765),
    [sym_duet_dev_type_tv] = ACTIONS(1765),
    [sym_duet_dev_type_utility] = ACTIONS(1765),
    [sym_duet_dev_type_vcr] = ACTIONS(1765),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1765),
    [sym_duet_dev_type_video_processor] = ACTIONS(1765),
    [sym_duet_dev_type_video_projector] = ACTIONS(1765),
    [sym_duet_dev_type_video_wall] = ACTIONS(1765),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1765),
    [sym_duet_dev_type_weather] = ACTIONS(1765),
    [sym_duet_dev_not_polled] = ACTIONS(1765),
    [sym_duet_dev_polled] = ACTIONS(1765),
    [sym_clkmgr_mode_network] = ACTIONS(1765),
    [sym_clkmgr_mode_standalone] = ACTIONS(1765),
    [sym_normal_standby] = ACTIONS(1765),
    [sym_normal_wake] = ACTIONS(1765),
    [sym_ip_multicast_ttl_option] = ACTIONS(1765),
    [sym_ip_ttl_subnet] = ACTIONS(1765),
    [sym_ip_ttl_site] = ACTIONS(1765),
    [sym_ip_ttl_region] = ACTIONS(1765),
    [sym_ip_ttl_continent] = ACTIONS(1765),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1765),
    [sym_ip_nodelay_on] = ACTIONS(1765),
    [sym_ip_nodelay_off] = ACTIONS(1765),
    [sym_amx_error] = ACTIONS(1765),
    [sym_amx_warning] = ACTIONS(1765),
    [sym_amx_info] = ACTIONS(1765),
    [sym_amx_debug] = ACTIONS(1765),
    [sym_netlinx_login_success] = ACTIONS(1765),
    [sym_netlinx_login_fail] = ACTIONS(1765),
    [sym_netlinx_logout] = ACTIONS(1765),
    [sym_null_str] = ACTIONS(1765),
    [sym_smtp_address] = ACTIONS(1765),
    [sym_smtp_port_number] = ACTIONS(1765),
    [sym_smtp_username] = ACTIONS(1765),
    [sym_smtp_password] = ACTIONS(1765),
    [sym_smtp_require_tls] = ACTIONS(1765),
    [sym_smtp_from] = ACTIONS(1765),
    [sym_smtp_tls_true] = ACTIONS(1765),
    [sym_smtp_tls_false] = ACTIONS(1765),
    [sym_listview_on_row_select_event] = ACTIONS(1765),
    [sym_data_structure_unknown] = ACTIONS(1765),
    [sym_data_structure_datafeed] = ACTIONS(1765),
    [sym_data_structure_datarecordset] = ACTIONS(1765),
    [sym_data_structure_datarecord] = ACTIONS(1765),
    [sym_data_structure_datafield] = ACTIONS(1765),
    [sym_data_max_content_field_count] = ACTIONS(1765),
    [sym_data_max_metadata_field_count] = ACTIONS(1765),
    [sym_data_max_id_length] = ACTIONS(1765),
    [sym_data_max_name_length] = ACTIONS(1765),
    [sym_data_max_description_length] = ACTIONS(1765),
    [sym_data_max_type_length] = ACTIONS(1765),
    [sym_data_max_format_length] = ACTIONS(1765),
    [sym_data_max_source_length] = ACTIONS(1765),
    [sym_data_max_value_length] = ACTIONS(1765),
    [sym_data_max_label_length] = ACTIONS(1765),
    [sym_data_max_url_length] = ACTIONS(1765),
    [sym_max_last_login_info_length] = ACTIONS(1765),
    [sym_data_type_unknown] = ACTIONS(1765),
    [sym_data_type_string] = ACTIONS(1765),
    [sym_data_type_datetime] = ACTIONS(1765),
    [sym_data_type_date] = ACTIONS(1765),
    [sym_data_type_time] = ACTIONS(1765),
    [sym_data_type_image] = ACTIONS(1765),
    [sym_data_format_url] = ACTIONS(1765),
    [sym_data_format_phone] = ACTIONS(1765),
    [sym_data_format_email] = ACTIONS(1765),
    [sym_data_format_iso8601] = ACTIONS(1765),
    [sym_auditlog_priv] = ACTIONS(1765),
    [sym_configuration_priv] = ACTIONS(1765),
    [sym_deviceconfig_priv] = ACTIONS(1765),
    [sym_ftp_priv] = ACTIONS(1765),
    [sym_http_priv] = ACTIONS(1765),
    [sym_networkconfig_priv] = ACTIONS(1765),
    [sym_programport_priv] = ACTIONS(1765),
    [sym_remoteui_priv] = ACTIONS(1765),
    [sym_securitycontrol_priv] = ACTIONS(1765),
    [sym_softwaremanagement_priv] = ACTIONS(1765),
    [sym_terminal_priv] = ACTIONS(1765),
    [sym_tpadmin_priv] = ACTIONS(1765),
    [sym_user1_priv] = ACTIONS(1765),
    [sym_user2_priv] = ACTIONS(1765),
    [sym_user3_priv] = ACTIONS(1765),
    [sym_user4_priv] = ACTIONS(1765),
    [sym_usermanagement_priv] = ACTIONS(1765),
    [sym_valid_account] = ACTIONS(1765),
    [sym_err_name_invalid] = ACTIONS(1765),
    [sym_err_password_invalid] = ACTIONS(1765),
    [sym_err_invalid_account] = ACTIONS(1765),
    [sym_err_no_authorization] = ACTIONS(1765),
    [sym_err_invalid_parameter] = ACTIONS(1765),
    [sym_err_user_locked_out] = ACTIONS(1765),
    [sym_err_user_account_expired] = ACTIONS(1765),
    [sym_err_general_error] = ACTIONS(1765),
    [sym_err_authentication_error] = ACTIONS(1765),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1765),
    [sym_tls_validate_certificate] = ACTIONS(1765),
    [sym_tdata] = ACTIONS(1765),
    [sym_tchannel] = ACTIONS(1765),
    [sym_tlevel] = ACTIONS(1765),
    [sym_tbutton] = ACTIONS(1765),
    [sym_ttimeline] = ACTIONS(1765),
    [sym_tcustom] = ACTIONS(1765),
    [sym_url_struct] = ACTIONS(1765),
    [sym_dns_struct] = ACTIONS(1765),
    [sym_ip_address_struct] = ACTIONS(1765),
    [sym_dev_info_struct] = ACTIONS(1765),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1765),
    [sym_data_feed] = ACTIONS(1765),
    [sym_data_field] = ACTIONS(1765),
    [sym_data_record] = ACTIONS(1765),
    [sym_wc_data_feed] = ACTIONS(1765),
    [sym_wc_data_field] = ACTIONS(1765),
    [sym_wc_data_record] = ACTIONS(1765),
    [sym_last_login_info] = ACTIONS(1765),
    [sym____reserved___] = ACTIONS(1765),
    [sym_date] = ACTIONS(1765),
    [sym_day] = ACTIONS(1765),
    [sym_time] = ACTIONS(1765),
    [sym_ldate] = ACTIONS(1765),
    [sym_push_device] = ACTIONS(1765),
    [sym_push_channel] = ACTIONS(1765),
    [sym_push_devchan] = ACTIONS(1765),
    [sym_release_device] = ACTIONS(1765),
    [sym_release_channel] = ACTIONS(1765),
    [sym_release_devchan] = ACTIONS(1765),
    [sym_master_slot] = ACTIONS(1765),
    [sym_get_pulse_time] = ACTIONS(1765),
    [sym_get_timer] = ACTIONS(1765),
    [sym_master_sn] = ACTIONS(1765),
    [sym_system_number] = ACTIONS(1765),
    [sym_button] = ACTIONS(1765),
    [sym_dv_channel] = ACTIONS(1765),
    [sym_data] = ACTIONS(1765),
    [sym_level] = ACTIONS(1765),
    [sym_timeline] = ACTIONS(1765),
    [sym_channel] = ACTIONS(1765),
    [sym_custom] = ACTIONS(1765),
    [sym_atoi] = ACTIONS(1765),
    [sym_atol] = ACTIONS(1765),
    [sym_atof] = ACTIONS(1765),
    [sym_itoa] = ACTIONS(1765),
    [sym_itohex] = ACTIONS(1765),
    [sym_hextoi] = ACTIONS(1765),
    [sym_ftoa] = ACTIONS(1765),
    [sym_find_string] = ACTIONS(1765),
    [sym_left_string] = ACTIONS(1765),
    [sym_right_string] = ACTIONS(1765),
    [sym_mid_string] = ACTIONS(1765),
    [sym_remove_string] = ACTIONS(1765),
    [sym_get_buffer_string] = ACTIONS(1765),
    [sym_get_buffer_char] = ACTIONS(1765),
    [sym_get_multi_buffer_string] = ACTIONS(1765),
    [sym_length_string] = ACTIONS(1765),
    [sym_max_length_string] = ACTIONS(1765),
    [sym_set_length_string] = ACTIONS(1765),
    [sym_compare_string] = ACTIONS(1765),
    [sym_upper_string] = ACTIONS(1765),
    [sym_lower_string] = ACTIONS(1765),
    [sym_get_last] = ACTIONS(1765),
    [sym_date_to_day] = ACTIONS(1765),
    [sym_date_to_month] = ACTIONS(1765),
    [sym_date_to_year] = ACTIONS(1765),
    [sym_time_to_hour] = ACTIONS(1765),
    [sym_time_to_minute] = ACTIONS(1765),
    [sym_time_to_second] = ACTIONS(1765),
    [sym_day_of_week] = ACTIONS(1765),
    [sym_set_pulse_time] = ACTIONS(1765),
    [sym_set_timer] = ACTIONS(1765),
    [sym_file_open] = ACTIONS(1765),
    [sym_file_close] = ACTIONS(1765),
    [sym_file_read] = ACTIONS(1765),
    [sym_file_read_line] = ACTIONS(1765),
    [sym_file_write] = ACTIONS(1765),
    [sym_file_write_line] = ACTIONS(1765),
    [sym_file_copy] = ACTIONS(1765),
    [sym_file_delete] = ACTIONS(1765),
    [sym_file_rename] = ACTIONS(1765),
    [sym_file_dir] = ACTIONS(1765),
    [sym_file_getdir] = ACTIONS(1765),
    [sym_file_setdir] = ACTIONS(1765),
    [sym_file_create_dir] = ACTIONS(1765),
    [sym_file_remove_dir] = ACTIONS(1765),
    [sym_file_seek] = ACTIONS(1765),
    [sym_length_array] = ACTIONS(1765),
    [sym_max_length_array] = ACTIONS(1765),
    [sym_set_length_array] = ACTIONS(1765),
    [sym_ip_client_open] = ACTIONS(1765),
    [sym_ip_client_close] = ACTIONS(1765),
    [sym_ip_server_open] = ACTIONS(1765),
    [sym_ip_server_close] = ACTIONS(1765),
    [sym_ip_bound_client_open] = ACTIONS(1765),
    [sym_ip_mc_server_open] = ACTIONS(1765),
    [sym_ip_set_option] = ACTIONS(1765),
    [sym_ssh_client_open] = ACTIONS(1765),
    [sym_ssh_client_close] = ACTIONS(1765),
    [sym_tls_client_open] = ACTIONS(1765),
    [sym_tls_client_close] = ACTIONS(1765),
    [sym_do_push] = ACTIONS(1765),
    [sym_do_push_timed] = ACTIONS(1765),
    [sym_do_release] = ACTIONS(1765),
    [sym_device_id] = ACTIONS(1765),
    [sym_device_info] = ACTIONS(1765),
    [sym_device_id_string] = ACTIONS(1765),
    [sym_device_standby] = ACTIONS(1765),
    [sym_device_wake] = ACTIONS(1765),
    [sym_timeline_create] = ACTIONS(1765),
    [sym_timeline_kill] = ACTIONS(1765),
    [sym_timeline_reload] = ACTIONS(1765),
    [sym_timeline_pause] = ACTIONS(1765),
    [sym_timeline_restart] = ACTIONS(1765),
    [sym_timeline_set] = ACTIONS(1765),
    [sym_timeline_get] = ACTIONS(1765),
    [sym_timeline_active] = ACTIONS(1765),
    [sym_timeline_dynamic_id] = ACTIONS(1765),
    [sym_variable_to_string] = ACTIONS(1765),
    [sym_string_to_variable] = ACTIONS(1765),
    [sym_length_variable_to_string] = ACTIONS(1765),
    [sym_type_cast] = ACTIONS(1765),
    [sym_variable_to_xml] = ACTIONS(1765),
    [sym_xml_to_variable] = ACTIONS(1765),
    [sym_length_variable_to_xml] = ACTIONS(1765),
    [sym_redirect_string] = ACTIONS(1765),
    [sym_random_number] = ACTIONS(1765),
    [sym_get_unique_id] = ACTIONS(1765),
    [sym_get_system_number] = ACTIONS(1765),
    [sym_set_system_number] = ACTIONS(1765),
    [sym_get_serial_number] = ACTIONS(1765),
    [sym_get_master_build] = ACTIONS(1765),
    [sym_set_validation_code] = ACTIONS(1765),
    [sym_reboot] = ACTIONS(1765),
    [sym_load_duet_module] = ACTIONS(1765),
    [sym_unload_duet_module] = ACTIONS(1765),
    [sym_duet_mem_size_set] = ACTIONS(1765),
    [sym_duet_mem_size_get] = ACTIONS(1765),
    [sym_dynamic_application_device] = ACTIONS(1765),
    [sym_dynamic_polled_port] = ACTIONS(1765),
    [sym_static_port_binding] = ACTIONS(1765),
    [sym_static_ip_binding] = ACTIONS(1765),
    [sym_get_dns_list] = ACTIONS(1765),
    [sym_set_dns_list] = ACTIONS(1765),
    [sym_get_ip_address] = ACTIONS(1765),
    [sym_set_ip_address] = ACTIONS(1765),
    [sym_get_url_list] = ACTIONS(1765),
    [sym_add_url_entry] = ACTIONS(1765),
    [sym_delete_url_entry] = ACTIONS(1765),
    [sym_abs_value] = ACTIONS(1765),
    [sym_max_value] = ACTIONS(1765),
    [sym_min_value] = ACTIONS(1765),
    [sym_exp_value] = ACTIONS(1765),
    [sym_log_value] = ACTIONS(1765),
    [sym_log10_value] = ACTIONS(1765),
    [sym_power_value] = ACTIONS(1765),
    [sym_sqrt_value] = ACTIONS(1765),
    [sym_raw_be] = ACTIONS(1765),
    [sym_raw_le] = ACTIONS(1765),
    [sym_internal_queue_size_set] = ACTIONS(1765),
    [sym_internal_queue_size_get] = ACTIONS(1765),
    [sym_internal_threshold_set] = ACTIONS(1765),
    [sym_internal_threshold_get] = ACTIONS(1765),
    [sym_rebuilt_event] = ACTIONS(1765),
    [sym_do_custom_event] = ACTIONS(1765),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1765),
    [sym_clkmgr_set_clk_source] = ACTIONS(1765),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1765),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1765),
    [sym_clkmgr_get_timezone] = ACTIONS(1765),
    [sym_clkmgr_set_timezone] = ACTIONS(1765),
    [sym_clkmgr_get_resync_period] = ACTIONS(1765),
    [sym_clkmgr_set_resync_period] = ACTIONS(1765),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1765),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1765),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1765),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1765),
    [sym_clkmgr_get_timeservers] = ACTIONS(1765),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1765),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1765),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1765),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1765),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1765),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1765),
    [sym_set_log_level] = ACTIONS(1765),
    [sym_get_log_level] = ACTIONS(1765),
    [sym_amx_log] = ACTIONS(1765),
    [sym_smtp_server_config_set] = ACTIONS(1765),
    [sym_smtp_server_config_get] = ACTIONS(1765),
    [sym_smtp_send] = ACTIONS(1765),
    [sym_validate_netlinx_account] = ACTIONS(1765),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1765),
    [sym_authenticate_certificate] = ACTIONS(1765),
    [sym_audit_netlinx_session_event] = ACTIONS(1765),
    [sym_audit_netlinx_generic_event] = ACTIONS(1765),
    [sym_get_available_flash_disk_space] = ACTIONS(1765),
    [sym_get_max_flash_disk_space] = ACTIONS(1765),
    [sym_data_create_feed] = ACTIONS(1765),
    [sym_data_delete_feed] = ACTIONS(1765),
    [sym_data_publish_feed] = ACTIONS(1765),
    [sym_data_get_published_feed] = ACTIONS(1765),
    [sym_data_add_record] = ACTIONS(1765),
    [sym_data_get_event_record] = ACTIONS(1765),
    [sym__wc_data_create_feed] = ACTIONS(1765),
    [sym__wc_data_add_record] = ACTIONS(1765),
    [sym__wc_data_get_event_record] = ACTIONS(1765),
    [sym_format] = ACTIONS(1765),
    [sym_astro_clock] = ACTIONS(1765),
    [sym_set_outdoor_temperature] = ACTIONS(1765),
    [sym_set_virtual_level_count] = ACTIONS(1765),
    [sym_set_virtual_channel_count] = ACTIONS(1765),
    [sym_set_virtual_port_count] = ACTIONS(1765),
    [sym__wc_encode_int] = ACTIONS(1765),
    [sym__wc_decode_int] = ACTIONS(1765),
    [sym__wc_int] = ACTIONS(1765),
    [sym__wc_to_ch_int] = ACTIONS(1765),
    [sym__ch_to_wc_int] = ACTIONS(1765),
    [sym__wc_find_string_int] = ACTIONS(1765),
    [sym__wc_left_string_int] = ACTIONS(1765),
    [sym__wc_lower_string_int] = ACTIONS(1765),
    [sym__wc_mid_string_int] = ACTIONS(1765),
    [sym__wc_remove_string_int] = ACTIONS(1765),
    [sym__wc_right_string_int] = ACTIONS(1765),
    [sym__wc_upper_string_int] = ACTIONS(1765),
    [sym__wc_compare_string_int] = ACTIONS(1765),
    [sym__wc_get_buffer_char_int] = ACTIONS(1765),
    [sym__wc_get_buffer_string_int] = ACTIONS(1765),
    [sym__wc_concat_string_int] = ACTIONS(1765),
    [sym__wc_file_open_int] = ACTIONS(1765),
    [sym__wc_file_close_int] = ACTIONS(1765),
    [sym__wc_file_read_int] = ACTIONS(1765),
    [sym__wc_file_read_line_int] = ACTIONS(1765),
    [sym__wc_file_write_int] = ACTIONS(1765),
    [sym__wc_file_write_line_int] = ACTIONS(1765),
    [aux_sym_preproc_include_token1] = ACTIONS(1765),
    [aux_sym_preproc_define_token1] = ACTIONS(1767),
    [aux_sym_preproc_warn_token1] = ACTIONS(1767),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1767),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1767),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1767),
    [anon_sym_LBRACK] = ACTIONS(1767),
    [anon_sym_LBRACE] = ACTIONS(1767),
    [anon_sym_LPAREN] = ACTIONS(1765),
    [anon_sym_BANG] = ACTIONS(1767),
    [anon_sym_TILDE] = ACTIONS(1767),
    [anon_sym_DASH] = ACTIONS(1765),
    [anon_sym_PLUS] = ACTIONS(1765),
    [anon_sym_DASH_DASH] = ACTIONS(1767),
    [anon_sym_PLUS_PLUS] = ACTIONS(1767),
    [anon_sym_DQUOTE] = ACTIONS(1767),
    [anon_sym_SQUOTE] = ACTIONS(1767),
    [sym_number_literal] = ACTIONS(1767),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(384)] = {
    [ts_builtin_sym_end] = ACTIONS(1771),
    [sym_identifier] = ACTIONS(1769),
    [sym_program_name_keyword] = ACTIONS(1769),
    [sym_module_name_keyword] = ACTIONS(1769),
    [sym_define_device_keyword] = ACTIONS(1769),
    [sym_define_combine_keyword] = ACTIONS(1769),
    [sym_define_constant_keyword] = ACTIONS(1769),
    [sym_define_type_keyword] = ACTIONS(1769),
    [sym_define_variable_keyword] = ACTIONS(1769),
    [sym_define_system_variable_keyword] = ACTIONS(1769),
    [sym_define_connect_level_keyword] = ACTIONS(1769),
    [sym_define_latching_keyword] = ACTIONS(1769),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1769),
    [sym_define_toggling_keyword] = ACTIONS(1769),
    [sym_define_call_keyword] = ACTIONS(1769),
    [sym_define_function_keyword] = ACTIONS(1769),
    [sym_define_library_function_keyword] = ACTIONS(1769),
    [sym_define_start_keyword] = ACTIONS(1769),
    [sym_define_event_keyword] = ACTIONS(1769),
    [sym_define_program_keyword] = ACTIONS(1769),
    [sym_define_module_keyword] = ACTIONS(1769),
    [sym_char_keyword] = ACTIONS(1769),
    [sym_widechar_keyword] = ACTIONS(1769),
    [sym_integer_keyword] = ACTIONS(1769),
    [sym_sinteger_keyword] = ACTIONS(1769),
    [sym_long_keyword] = ACTIONS(1769),
    [sym_slong_keyword] = ACTIONS(1769),
    [sym_float_keyword] = ACTIONS(1769),
    [sym_double_keyword] = ACTIONS(1769),
    [sym_variant_keyword] = ACTIONS(1769),
    [sym_variantarray_keyword] = ACTIONS(1769),
    [sym_local_var_keyword] = ACTIONS(1769),
    [sym_stack_var_keyword] = ACTIONS(1769),
    [sym_constant_keyword] = ACTIONS(1769),
    [sym_volatile_keyword] = ACTIONS(1769),
    [sym_non_volatile_keyword] = ACTIONS(1769),
    [sym_persistent_keyword] = ACTIONS(1769),
    [sym_dev_keyword] = ACTIONS(1769),
    [sym_devlev_keyword] = ACTIONS(1769),
    [sym_devchan_keyword] = ACTIONS(1769),
    [sym_if_keyword] = ACTIONS(1769),
    [sym_else_keyword] = ACTIONS(1769),
    [sym_switch_keyword] = ACTIONS(1769),
    [sym_case_keyword] = ACTIONS(1769),
    [sym_default_keyword] = ACTIONS(1769),
    [sym_while_keyword] = ACTIONS(1769),
    [sym_for_keyword] = ACTIONS(1769),
    [sym_break_keyword] = ACTIONS(1769),
    [sym_continue_keyword] = ACTIONS(1769),
    [sym_return_keyword] = ACTIONS(1769),
    [sym_select_keyword] = ACTIONS(1769),
    [sym_active_keyword] = ACTIONS(1769),
    [sym_struct_keyword] = ACTIONS(1769),
    [sym_structure_keyword] = ACTIONS(1769),
    [sym_button_event_keyword] = ACTIONS(1769),
    [sym_channel_event_keyword] = ACTIONS(1769),
    [sym_level_event_keyword] = ACTIONS(1769),
    [sym_data_event_keyword] = ACTIONS(1769),
    [sym_timeline_event_keyword] = ACTIONS(1769),
    [sym_custom_event_keyword] = ACTIONS(1769),
    [sym_push_keyword] = ACTIONS(1769),
    [sym_release_keyword] = ACTIONS(1769),
    [aux_sym_on_keyword_token1] = ACTIONS(1769),
    [aux_sym_off_keyword_token1] = ACTIONS(1769),
    [sym_send_level_keyword] = ACTIONS(1769),
    [sym_send_string_keyword] = ACTIONS(1769),
    [sym_send_command_keyword] = ACTIONS(1769),
    [sym_clear_buffer_keyword] = ACTIONS(1769),
    [sym_create_buffer_keyword] = ACTIONS(1769),
    [sym_create_multi_buffer_keyword] = ACTIONS(1769),
    [sym_call_keyword] = ACTIONS(1769),
    [sym_system_call_keyword] = ACTIONS(1769),
    [sym_devchan_to_keyword] = ACTIONS(1769),
    [sym_devchan_min_to_keyword] = ACTIONS(1769),
    [sym_devchan_total_off_keyword] = ACTIONS(1769),
    [sym_devchan_pulse_keyword] = ACTIONS(1769),
    [sym_wait_keyword] = ACTIONS(1769),
    [sym_cancel_wait_keyword] = ACTIONS(1769),
    [sym_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1769),
    [sym_not] = ACTIONS(1769),
    [sym_bnot] = ACTIONS(1769),
    [sym___file__] = ACTIONS(1769),
    [sym___date__] = ACTIONS(1769),
    [sym___ldate__] = ACTIONS(1769),
    [sym___time__] = ACTIONS(1769),
    [sym___line__] = ACTIONS(1769),
    [sym___name__] = ACTIONS(1769),
    [sym___netlinx__] = ACTIONS(1769),
    [sym_netlinx_axi_version] = ACTIONS(1769),
    [sym_url_flg_tcp] = ACTIONS(1769),
    [sym_url_flg_accinfopresent] = ACTIONS(1769),
    [sym_url_flg_temp] = ACTIONS(1769),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1769),
    [sym_url_flg_stat_mask] = ACTIONS(1769),
    [sym_url_flg_stat_lookup] = ACTIONS(1769),
    [sym_url_flg_stat_connecting] = ACTIONS(1769),
    [sym_url_flg_stat_waiting] = ACTIONS(1769),
    [sym_url_flg_stat_connected] = ACTIONS(1769),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1769),
    [sym_first_virtual_device] = ACTIONS(1769),
    [sym_first_local_port] = ACTIONS(1769),
    [sym_dynamic_virtual_device] = ACTIONS(1769),
    [sym_timeline_once] = ACTIONS(1769),
    [sym_timeline_repeat] = ACTIONS(1769),
    [sym_timeline_absolute] = ACTIONS(1769),
    [sym_timeline_relative] = ACTIONS(1769),
    [sym_do_push_timed_infinite] = ACTIONS(1769),
    [sym_source_type_no_address] = ACTIONS(1769),
    [sym_source_type_neuron_id] = ACTIONS(1769),
    [sym_source_type_ip_address] = ACTIONS(1769),
    [sym_source_type_axlink] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1769),
    [sym_source_type_ip_socket_address] = ACTIONS(1769),
    [sym_source_type_rs232] = ACTIONS(1769),
    [sym_source_type_internal] = ACTIONS(1769),
    [sym_source_type_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1769),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1769),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1769),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1769),
    [sym_true] = ACTIONS(1769),
    [sym_false] = ACTIONS(1769),
    [sym_file_read_only] = ACTIONS(1769),
    [sym_file_rw_new] = ACTIONS(1769),
    [sym_file_rw_append] = ACTIONS(1769),
    [sym_ip_tcp] = ACTIONS(1769),
    [sym_ip_udp] = ACTIONS(1769),
    [sym_ip_udp_2way] = ACTIONS(1769),
    [sym_xml_encode_types] = ACTIONS(1769),
    [sym_xml_encode_char_as_list] = ACTIONS(1769),
    [sym_xml_encode_le] = ACTIONS(1769),
    [sym_xml_decode_types] = ACTIONS(1769),
    [sym_xml_decode_no_preserve] = ACTIONS(1769),
    [sym_duet_dev_type_display_device] = ACTIONS(1769),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_io_device] = ACTIONS(1769),
    [sym_duet_dev_type_relay_device] = ACTIONS(1769),
    [sym_duet_dev_type_ups] = ACTIONS(1769),
    [sym_duet_dev_type_amplifier] = ACTIONS(1769),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1769),
    [sym_duet_dev_type_camera] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1769),
    [sym_duet_dev_type_dss] = ACTIONS(1769),
    [sym_duet_dev_type_dvr] = ACTIONS(1769),
    [sym_duet_dev_type_disc_device] = ACTIONS(1769),
    [sym_duet_dev_type_document_camera] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1769),
    [sym_duet_dev_type_hvac] = ACTIONS(1769),
    [sym_duet_dev_type_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_light] = ACTIONS(1769),
    [sym_duet_dev_type_monitor] = ACTIONS(1769),
    [sym_duet_dev_type_motor] = ACTIONS(1769),
    [sym_duet_dev_type_multi_window] = ACTIONS(1769),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1769),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1769),
    [sym_duet_dev_type_receiver] = ACTIONS(1769),
    [sym_duet_dev_type_security_system] = ACTIONS(1769),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1769),
    [sym_duet_dev_type_settop_box] = ACTIONS(1769),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1769),
    [sym_duet_dev_type_switcher] = ACTIONS(1769),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_tv] = ACTIONS(1769),
    [sym_duet_dev_type_utility] = ACTIONS(1769),
    [sym_duet_dev_type_vcr] = ACTIONS(1769),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_video_processor] = ACTIONS(1769),
    [sym_duet_dev_type_video_projector] = ACTIONS(1769),
    [sym_duet_dev_type_video_wall] = ACTIONS(1769),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1769),
    [sym_duet_dev_type_weather] = ACTIONS(1769),
    [sym_duet_dev_not_polled] = ACTIONS(1769),
    [sym_duet_dev_polled] = ACTIONS(1769),
    [sym_clkmgr_mode_network] = ACTIONS(1769),
    [sym_clkmgr_mode_standalone] = ACTIONS(1769),
    [sym_normal_standby] = ACTIONS(1769),
    [sym_normal_wake] = ACTIONS(1769),
    [sym_ip_multicast_ttl_option] = ACTIONS(1769),
    [sym_ip_ttl_subnet] = ACTIONS(1769),
    [sym_ip_ttl_site] = ACTIONS(1769),
    [sym_ip_ttl_region] = ACTIONS(1769),
    [sym_ip_ttl_continent] = ACTIONS(1769),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1769),
    [sym_ip_nodelay_on] = ACTIONS(1769),
    [sym_ip_nodelay_off] = ACTIONS(1769),
    [sym_amx_error] = ACTIONS(1769),
    [sym_amx_warning] = ACTIONS(1769),
    [sym_amx_info] = ACTIONS(1769),
    [sym_amx_debug] = ACTIONS(1769),
    [sym_netlinx_login_success] = ACTIONS(1769),
    [sym_netlinx_login_fail] = ACTIONS(1769),
    [sym_netlinx_logout] = ACTIONS(1769),
    [sym_null_str] = ACTIONS(1769),
    [sym_smtp_address] = ACTIONS(1769),
    [sym_smtp_port_number] = ACTIONS(1769),
    [sym_smtp_username] = ACTIONS(1769),
    [sym_smtp_password] = ACTIONS(1769),
    [sym_smtp_require_tls] = ACTIONS(1769),
    [sym_smtp_from] = ACTIONS(1769),
    [sym_smtp_tls_true] = ACTIONS(1769),
    [sym_smtp_tls_false] = ACTIONS(1769),
    [sym_listview_on_row_select_event] = ACTIONS(1769),
    [sym_data_structure_unknown] = ACTIONS(1769),
    [sym_data_structure_datafeed] = ACTIONS(1769),
    [sym_data_structure_datarecordset] = ACTIONS(1769),
    [sym_data_structure_datarecord] = ACTIONS(1769),
    [sym_data_structure_datafield] = ACTIONS(1769),
    [sym_data_max_content_field_count] = ACTIONS(1769),
    [sym_data_max_metadata_field_count] = ACTIONS(1769),
    [sym_data_max_id_length] = ACTIONS(1769),
    [sym_data_max_name_length] = ACTIONS(1769),
    [sym_data_max_description_length] = ACTIONS(1769),
    [sym_data_max_type_length] = ACTIONS(1769),
    [sym_data_max_format_length] = ACTIONS(1769),
    [sym_data_max_source_length] = ACTIONS(1769),
    [sym_data_max_value_length] = ACTIONS(1769),
    [sym_data_max_label_length] = ACTIONS(1769),
    [sym_data_max_url_length] = ACTIONS(1769),
    [sym_max_last_login_info_length] = ACTIONS(1769),
    [sym_data_type_unknown] = ACTIONS(1769),
    [sym_data_type_string] = ACTIONS(1769),
    [sym_data_type_datetime] = ACTIONS(1769),
    [sym_data_type_date] = ACTIONS(1769),
    [sym_data_type_time] = ACTIONS(1769),
    [sym_data_type_image] = ACTIONS(1769),
    [sym_data_format_url] = ACTIONS(1769),
    [sym_data_format_phone] = ACTIONS(1769),
    [sym_data_format_email] = ACTIONS(1769),
    [sym_data_format_iso8601] = ACTIONS(1769),
    [sym_auditlog_priv] = ACTIONS(1769),
    [sym_configuration_priv] = ACTIONS(1769),
    [sym_deviceconfig_priv] = ACTIONS(1769),
    [sym_ftp_priv] = ACTIONS(1769),
    [sym_http_priv] = ACTIONS(1769),
    [sym_networkconfig_priv] = ACTIONS(1769),
    [sym_programport_priv] = ACTIONS(1769),
    [sym_remoteui_priv] = ACTIONS(1769),
    [sym_securitycontrol_priv] = ACTIONS(1769),
    [sym_softwaremanagement_priv] = ACTIONS(1769),
    [sym_terminal_priv] = ACTIONS(1769),
    [sym_tpadmin_priv] = ACTIONS(1769),
    [sym_user1_priv] = ACTIONS(1769),
    [sym_user2_priv] = ACTIONS(1769),
    [sym_user3_priv] = ACTIONS(1769),
    [sym_user4_priv] = ACTIONS(1769),
    [sym_usermanagement_priv] = ACTIONS(1769),
    [sym_valid_account] = ACTIONS(1769),
    [sym_err_name_invalid] = ACTIONS(1769),
    [sym_err_password_invalid] = ACTIONS(1769),
    [sym_err_invalid_account] = ACTIONS(1769),
    [sym_err_no_authorization] = ACTIONS(1769),
    [sym_err_invalid_parameter] = ACTIONS(1769),
    [sym_err_user_locked_out] = ACTIONS(1769),
    [sym_err_user_account_expired] = ACTIONS(1769),
    [sym_err_general_error] = ACTIONS(1769),
    [sym_err_authentication_error] = ACTIONS(1769),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1769),
    [sym_tls_validate_certificate] = ACTIONS(1769),
    [sym_tdata] = ACTIONS(1769),
    [sym_tchannel] = ACTIONS(1769),
    [sym_tlevel] = ACTIONS(1769),
    [sym_tbutton] = ACTIONS(1769),
    [sym_ttimeline] = ACTIONS(1769),
    [sym_tcustom] = ACTIONS(1769),
    [sym_url_struct] = ACTIONS(1769),
    [sym_dns_struct] = ACTIONS(1769),
    [sym_ip_address_struct] = ACTIONS(1769),
    [sym_dev_info_struct] = ACTIONS(1769),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1769),
    [sym_data_feed] = ACTIONS(1769),
    [sym_data_field] = ACTIONS(1769),
    [sym_data_record] = ACTIONS(1769),
    [sym_wc_data_feed] = ACTIONS(1769),
    [sym_wc_data_field] = ACTIONS(1769),
    [sym_wc_data_record] = ACTIONS(1769),
    [sym_last_login_info] = ACTIONS(1769),
    [sym____reserved___] = ACTIONS(1769),
    [sym_date] = ACTIONS(1769),
    [sym_day] = ACTIONS(1769),
    [sym_time] = ACTIONS(1769),
    [sym_ldate] = ACTIONS(1769),
    [sym_push_device] = ACTIONS(1769),
    [sym_push_channel] = ACTIONS(1769),
    [sym_push_devchan] = ACTIONS(1769),
    [sym_release_device] = ACTIONS(1769),
    [sym_release_channel] = ACTIONS(1769),
    [sym_release_devchan] = ACTIONS(1769),
    [sym_master_slot] = ACTIONS(1769),
    [sym_get_pulse_time] = ACTIONS(1769),
    [sym_get_timer] = ACTIONS(1769),
    [sym_master_sn] = ACTIONS(1769),
    [sym_system_number] = ACTIONS(1769),
    [sym_button] = ACTIONS(1769),
    [sym_dv_channel] = ACTIONS(1769),
    [sym_data] = ACTIONS(1769),
    [sym_level] = ACTIONS(1769),
    [sym_timeline] = ACTIONS(1769),
    [sym_channel] = ACTIONS(1769),
    [sym_custom] = ACTIONS(1769),
    [sym_atoi] = ACTIONS(1769),
    [sym_atol] = ACTIONS(1769),
    [sym_atof] = ACTIONS(1769),
    [sym_itoa] = ACTIONS(1769),
    [sym_itohex] = ACTIONS(1769),
    [sym_hextoi] = ACTIONS(1769),
    [sym_ftoa] = ACTIONS(1769),
    [sym_find_string] = ACTIONS(1769),
    [sym_left_string] = ACTIONS(1769),
    [sym_right_string] = ACTIONS(1769),
    [sym_mid_string] = ACTIONS(1769),
    [sym_remove_string] = ACTIONS(1769),
    [sym_get_buffer_string] = ACTIONS(1769),
    [sym_get_buffer_char] = ACTIONS(1769),
    [sym_get_multi_buffer_string] = ACTIONS(1769),
    [sym_length_string] = ACTIONS(1769),
    [sym_max_length_string] = ACTIONS(1769),
    [sym_set_length_string] = ACTIONS(1769),
    [sym_compare_string] = ACTIONS(1769),
    [sym_upper_string] = ACTIONS(1769),
    [sym_lower_string] = ACTIONS(1769),
    [sym_get_last] = ACTIONS(1769),
    [sym_date_to_day] = ACTIONS(1769),
    [sym_date_to_month] = ACTIONS(1769),
    [sym_date_to_year] = ACTIONS(1769),
    [sym_time_to_hour] = ACTIONS(1769),
    [sym_time_to_minute] = ACTIONS(1769),
    [sym_time_to_second] = ACTIONS(1769),
    [sym_day_of_week] = ACTIONS(1769),
    [sym_set_pulse_time] = ACTIONS(1769),
    [sym_set_timer] = ACTIONS(1769),
    [sym_file_open] = ACTIONS(1769),
    [sym_file_close] = ACTIONS(1769),
    [sym_file_read] = ACTIONS(1769),
    [sym_file_read_line] = ACTIONS(1769),
    [sym_file_write] = ACTIONS(1769),
    [sym_file_write_line] = ACTIONS(1769),
    [sym_file_copy] = ACTIONS(1769),
    [sym_file_delete] = ACTIONS(1769),
    [sym_file_rename] = ACTIONS(1769),
    [sym_file_dir] = ACTIONS(1769),
    [sym_file_getdir] = ACTIONS(1769),
    [sym_file_setdir] = ACTIONS(1769),
    [sym_file_create_dir] = ACTIONS(1769),
    [sym_file_remove_dir] = ACTIONS(1769),
    [sym_file_seek] = ACTIONS(1769),
    [sym_length_array] = ACTIONS(1769),
    [sym_max_length_array] = ACTIONS(1769),
    [sym_set_length_array] = ACTIONS(1769),
    [sym_ip_client_open] = ACTIONS(1769),
    [sym_ip_client_close] = ACTIONS(1769),
    [sym_ip_server_open] = ACTIONS(1769),
    [sym_ip_server_close] = ACTIONS(1769),
    [sym_ip_bound_client_open] = ACTIONS(1769),
    [sym_ip_mc_server_open] = ACTIONS(1769),
    [sym_ip_set_option] = ACTIONS(1769),
    [sym_ssh_client_open] = ACTIONS(1769),
    [sym_ssh_client_close] = ACTIONS(1769),
    [sym_tls_client_open] = ACTIONS(1769),
    [sym_tls_client_close] = ACTIONS(1769),
    [sym_do_push] = ACTIONS(1769),
    [sym_do_push_timed] = ACTIONS(1769),
    [sym_do_release] = ACTIONS(1769),
    [sym_device_id] = ACTIONS(1769),
    [sym_device_info] = ACTIONS(1769),
    [sym_device_id_string] = ACTIONS(1769),
    [sym_device_standby] = ACTIONS(1769),
    [sym_device_wake] = ACTIONS(1769),
    [sym_timeline_create] = ACTIONS(1769),
    [sym_timeline_kill] = ACTIONS(1769),
    [sym_timeline_reload] = ACTIONS(1769),
    [sym_timeline_pause] = ACTIONS(1769),
    [sym_timeline_restart] = ACTIONS(1769),
    [sym_timeline_set] = ACTIONS(1769),
    [sym_timeline_get] = ACTIONS(1769),
    [sym_timeline_active] = ACTIONS(1769),
    [sym_timeline_dynamic_id] = ACTIONS(1769),
    [sym_variable_to_string] = ACTIONS(1769),
    [sym_string_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_string] = ACTIONS(1769),
    [sym_type_cast] = ACTIONS(1769),
    [sym_variable_to_xml] = ACTIONS(1769),
    [sym_xml_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_xml] = ACTIONS(1769),
    [sym_redirect_string] = ACTIONS(1769),
    [sym_random_number] = ACTIONS(1769),
    [sym_get_unique_id] = ACTIONS(1769),
    [sym_get_system_number] = ACTIONS(1769),
    [sym_set_system_number] = ACTIONS(1769),
    [sym_get_serial_number] = ACTIONS(1769),
    [sym_get_master_build] = ACTIONS(1769),
    [sym_set_validation_code] = ACTIONS(1769),
    [sym_reboot] = ACTIONS(1769),
    [sym_load_duet_module] = ACTIONS(1769),
    [sym_unload_duet_module] = ACTIONS(1769),
    [sym_duet_mem_size_set] = ACTIONS(1769),
    [sym_duet_mem_size_get] = ACTIONS(1769),
    [sym_dynamic_application_device] = ACTIONS(1769),
    [sym_dynamic_polled_port] = ACTIONS(1769),
    [sym_static_port_binding] = ACTIONS(1769),
    [sym_static_ip_binding] = ACTIONS(1769),
    [sym_get_dns_list] = ACTIONS(1769),
    [sym_set_dns_list] = ACTIONS(1769),
    [sym_get_ip_address] = ACTIONS(1769),
    [sym_set_ip_address] = ACTIONS(1769),
    [sym_get_url_list] = ACTIONS(1769),
    [sym_add_url_entry] = ACTIONS(1769),
    [sym_delete_url_entry] = ACTIONS(1769),
    [sym_abs_value] = ACTIONS(1769),
    [sym_max_value] = ACTIONS(1769),
    [sym_min_value] = ACTIONS(1769),
    [sym_exp_value] = ACTIONS(1769),
    [sym_log_value] = ACTIONS(1769),
    [sym_log10_value] = ACTIONS(1769),
    [sym_power_value] = ACTIONS(1769),
    [sym_sqrt_value] = ACTIONS(1769),
    [sym_raw_be] = ACTIONS(1769),
    [sym_raw_le] = ACTIONS(1769),
    [sym_internal_queue_size_set] = ACTIONS(1769),
    [sym_internal_queue_size_get] = ACTIONS(1769),
    [sym_internal_threshold_set] = ACTIONS(1769),
    [sym_internal_threshold_get] = ACTIONS(1769),
    [sym_rebuilt_event] = ACTIONS(1769),
    [sym_do_custom_event] = ACTIONS(1769),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1769),
    [sym_clkmgr_set_clk_source] = ACTIONS(1769),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1769),
    [sym_clkmgr_get_timezone] = ACTIONS(1769),
    [sym_clkmgr_set_timezone] = ACTIONS(1769),
    [sym_clkmgr_get_resync_period] = ACTIONS(1769),
    [sym_clkmgr_set_resync_period] = ACTIONS(1769),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_timeservers] = ACTIONS(1769),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_set_log_level] = ACTIONS(1769),
    [sym_get_log_level] = ACTIONS(1769),
    [sym_amx_log] = ACTIONS(1769),
    [sym_smtp_server_config_set] = ACTIONS(1769),
    [sym_smtp_server_config_get] = ACTIONS(1769),
    [sym_smtp_send] = ACTIONS(1769),
    [sym_validate_netlinx_account] = ACTIONS(1769),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1769),
    [sym_authenticate_certificate] = ACTIONS(1769),
    [sym_audit_netlinx_session_event] = ACTIONS(1769),
    [sym_audit_netlinx_generic_event] = ACTIONS(1769),
    [sym_get_available_flash_disk_space] = ACTIONS(1769),
    [sym_get_max_flash_disk_space] = ACTIONS(1769),
    [sym_data_create_feed] = ACTIONS(1769),
    [sym_data_delete_feed] = ACTIONS(1769),
    [sym_data_publish_feed] = ACTIONS(1769),
    [sym_data_get_published_feed] = ACTIONS(1769),
    [sym_data_add_record] = ACTIONS(1769),
    [sym_data_get_event_record] = ACTIONS(1769),
    [sym__wc_data_create_feed] = ACTIONS(1769),
    [sym__wc_data_add_record] = ACTIONS(1769),
    [sym__wc_data_get_event_record] = ACTIONS(1769),
    [sym_format] = ACTIONS(1769),
    [sym_astro_clock] = ACTIONS(1769),
    [sym_set_outdoor_temperature] = ACTIONS(1769),
    [sym_set_virtual_level_count] = ACTIONS(1769),
    [sym_set_virtual_channel_count] = ACTIONS(1769),
    [sym_set_virtual_port_count] = ACTIONS(1769),
    [sym__wc_encode_int] = ACTIONS(1769),
    [sym__wc_decode_int] = ACTIONS(1769),
    [sym__wc_int] = ACTIONS(1769),
    [sym__wc_to_ch_int] = ACTIONS(1769),
    [sym__ch_to_wc_int] = ACTIONS(1769),
    [sym__wc_find_string_int] = ACTIONS(1769),
    [sym__wc_left_string_int] = ACTIONS(1769),
    [sym__wc_lower_string_int] = ACTIONS(1769),
    [sym__wc_mid_string_int] = ACTIONS(1769),
    [sym__wc_remove_string_int] = ACTIONS(1769),
    [sym__wc_right_string_int] = ACTIONS(1769),
    [sym__wc_upper_string_int] = ACTIONS(1769),
    [sym__wc_compare_string_int] = ACTIONS(1769),
    [sym__wc_get_buffer_char_int] = ACTIONS(1769),
    [sym__wc_get_buffer_string_int] = ACTIONS(1769),
    [sym__wc_concat_string_int] = ACTIONS(1769),
    [sym__wc_file_open_int] = ACTIONS(1769),
    [sym__wc_file_close_int] = ACTIONS(1769),
    [sym__wc_file_read_int] = ACTIONS(1769),
    [sym__wc_file_read_line_int] = ACTIONS(1769),
    [sym__wc_file_write_int] = ACTIONS(1769),
    [sym__wc_file_write_line_int] = ACTIONS(1769),
    [aux_sym_preproc_include_token1] = ACTIONS(1769),
    [aux_sym_preproc_define_token1] = ACTIONS(1771),
    [aux_sym_preproc_warn_token1] = ACTIONS(1771),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1771),
    [anon_sym_LBRACK] = ACTIONS(1771),
    [anon_sym_LBRACE] = ACTIONS(1771),
    [anon_sym_LPAREN] = ACTIONS(1769),
    [anon_sym_BANG] = ACTIONS(1771),
    [anon_sym_TILDE] = ACTIONS(1771),
    [anon_sym_DASH] = ACTIONS(1769),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_DASH_DASH] = ACTIONS(1771),
    [anon_sym_PLUS_PLUS] = ACTIONS(1771),
    [anon_sym_DQUOTE] = ACTIONS(1771),
    [anon_sym_SQUOTE] = ACTIONS(1771),
    [sym_number_literal] = ACTIONS(1771),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(385)] = {
    [ts_builtin_sym_end] = ACTIONS(1771),
    [sym_identifier] = ACTIONS(1769),
    [sym_program_name_keyword] = ACTIONS(1769),
    [sym_module_name_keyword] = ACTIONS(1769),
    [sym_define_device_keyword] = ACTIONS(1769),
    [sym_define_combine_keyword] = ACTIONS(1769),
    [sym_define_constant_keyword] = ACTIONS(1769),
    [sym_define_type_keyword] = ACTIONS(1769),
    [sym_define_variable_keyword] = ACTIONS(1769),
    [sym_define_system_variable_keyword] = ACTIONS(1769),
    [sym_define_connect_level_keyword] = ACTIONS(1769),
    [sym_define_latching_keyword] = ACTIONS(1769),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1769),
    [sym_define_toggling_keyword] = ACTIONS(1769),
    [sym_define_call_keyword] = ACTIONS(1769),
    [sym_define_function_keyword] = ACTIONS(1769),
    [sym_define_library_function_keyword] = ACTIONS(1769),
    [sym_define_start_keyword] = ACTIONS(1769),
    [sym_define_event_keyword] = ACTIONS(1769),
    [sym_define_program_keyword] = ACTIONS(1769),
    [sym_define_module_keyword] = ACTIONS(1769),
    [sym_char_keyword] = ACTIONS(1769),
    [sym_widechar_keyword] = ACTIONS(1769),
    [sym_integer_keyword] = ACTIONS(1769),
    [sym_sinteger_keyword] = ACTIONS(1769),
    [sym_long_keyword] = ACTIONS(1769),
    [sym_slong_keyword] = ACTIONS(1769),
    [sym_float_keyword] = ACTIONS(1769),
    [sym_double_keyword] = ACTIONS(1769),
    [sym_variant_keyword] = ACTIONS(1769),
    [sym_variantarray_keyword] = ACTIONS(1769),
    [sym_local_var_keyword] = ACTIONS(1769),
    [sym_stack_var_keyword] = ACTIONS(1769),
    [sym_constant_keyword] = ACTIONS(1769),
    [sym_volatile_keyword] = ACTIONS(1769),
    [sym_non_volatile_keyword] = ACTIONS(1769),
    [sym_persistent_keyword] = ACTIONS(1769),
    [sym_dev_keyword] = ACTIONS(1769),
    [sym_devlev_keyword] = ACTIONS(1769),
    [sym_devchan_keyword] = ACTIONS(1769),
    [sym_if_keyword] = ACTIONS(1769),
    [sym_else_keyword] = ACTIONS(1769),
    [sym_switch_keyword] = ACTIONS(1769),
    [sym_case_keyword] = ACTIONS(1769),
    [sym_default_keyword] = ACTIONS(1769),
    [sym_while_keyword] = ACTIONS(1769),
    [sym_for_keyword] = ACTIONS(1769),
    [sym_break_keyword] = ACTIONS(1769),
    [sym_continue_keyword] = ACTIONS(1769),
    [sym_return_keyword] = ACTIONS(1769),
    [sym_select_keyword] = ACTIONS(1769),
    [sym_active_keyword] = ACTIONS(1769),
    [sym_struct_keyword] = ACTIONS(1769),
    [sym_structure_keyword] = ACTIONS(1769),
    [sym_button_event_keyword] = ACTIONS(1769),
    [sym_channel_event_keyword] = ACTIONS(1769),
    [sym_level_event_keyword] = ACTIONS(1769),
    [sym_data_event_keyword] = ACTIONS(1769),
    [sym_timeline_event_keyword] = ACTIONS(1769),
    [sym_custom_event_keyword] = ACTIONS(1769),
    [sym_push_keyword] = ACTIONS(1769),
    [sym_release_keyword] = ACTIONS(1769),
    [aux_sym_on_keyword_token1] = ACTIONS(1769),
    [aux_sym_off_keyword_token1] = ACTIONS(1769),
    [sym_send_level_keyword] = ACTIONS(1769),
    [sym_send_string_keyword] = ACTIONS(1769),
    [sym_send_command_keyword] = ACTIONS(1769),
    [sym_clear_buffer_keyword] = ACTIONS(1769),
    [sym_create_buffer_keyword] = ACTIONS(1769),
    [sym_create_multi_buffer_keyword] = ACTIONS(1769),
    [sym_call_keyword] = ACTIONS(1769),
    [sym_system_call_keyword] = ACTIONS(1769),
    [sym_devchan_to_keyword] = ACTIONS(1769),
    [sym_devchan_min_to_keyword] = ACTIONS(1769),
    [sym_devchan_total_off_keyword] = ACTIONS(1769),
    [sym_devchan_pulse_keyword] = ACTIONS(1769),
    [sym_wait_keyword] = ACTIONS(1769),
    [sym_cancel_wait_keyword] = ACTIONS(1769),
    [sym_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1769),
    [sym_not] = ACTIONS(1769),
    [sym_bnot] = ACTIONS(1769),
    [sym___file__] = ACTIONS(1769),
    [sym___date__] = ACTIONS(1769),
    [sym___ldate__] = ACTIONS(1769),
    [sym___time__] = ACTIONS(1769),
    [sym___line__] = ACTIONS(1769),
    [sym___name__] = ACTIONS(1769),
    [sym___netlinx__] = ACTIONS(1769),
    [sym_netlinx_axi_version] = ACTIONS(1769),
    [sym_url_flg_tcp] = ACTIONS(1769),
    [sym_url_flg_accinfopresent] = ACTIONS(1769),
    [sym_url_flg_temp] = ACTIONS(1769),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1769),
    [sym_url_flg_stat_mask] = ACTIONS(1769),
    [sym_url_flg_stat_lookup] = ACTIONS(1769),
    [sym_url_flg_stat_connecting] = ACTIONS(1769),
    [sym_url_flg_stat_waiting] = ACTIONS(1769),
    [sym_url_flg_stat_connected] = ACTIONS(1769),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1769),
    [sym_first_virtual_device] = ACTIONS(1769),
    [sym_first_local_port] = ACTIONS(1769),
    [sym_dynamic_virtual_device] = ACTIONS(1769),
    [sym_timeline_once] = ACTIONS(1769),
    [sym_timeline_repeat] = ACTIONS(1769),
    [sym_timeline_absolute] = ACTIONS(1769),
    [sym_timeline_relative] = ACTIONS(1769),
    [sym_do_push_timed_infinite] = ACTIONS(1769),
    [sym_source_type_no_address] = ACTIONS(1769),
    [sym_source_type_neuron_id] = ACTIONS(1769),
    [sym_source_type_ip_address] = ACTIONS(1769),
    [sym_source_type_axlink] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1769),
    [sym_source_type_ip_socket_address] = ACTIONS(1769),
    [sym_source_type_rs232] = ACTIONS(1769),
    [sym_source_type_internal] = ACTIONS(1769),
    [sym_source_type_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1769),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1769),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1769),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1769),
    [sym_true] = ACTIONS(1769),
    [sym_false] = ACTIONS(1769),
    [sym_file_read_only] = ACTIONS(1769),
    [sym_file_rw_new] = ACTIONS(1769),
    [sym_file_rw_append] = ACTIONS(1769),
    [sym_ip_tcp] = ACTIONS(1769),
    [sym_ip_udp] = ACTIONS(1769),
    [sym_ip_udp_2way] = ACTIONS(1769),
    [sym_xml_encode_types] = ACTIONS(1769),
    [sym_xml_encode_char_as_list] = ACTIONS(1769),
    [sym_xml_encode_le] = ACTIONS(1769),
    [sym_xml_decode_types] = ACTIONS(1769),
    [sym_xml_decode_no_preserve] = ACTIONS(1769),
    [sym_duet_dev_type_display_device] = ACTIONS(1769),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_io_device] = ACTIONS(1769),
    [sym_duet_dev_type_relay_device] = ACTIONS(1769),
    [sym_duet_dev_type_ups] = ACTIONS(1769),
    [sym_duet_dev_type_amplifier] = ACTIONS(1769),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1769),
    [sym_duet_dev_type_camera] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1769),
    [sym_duet_dev_type_dss] = ACTIONS(1769),
    [sym_duet_dev_type_dvr] = ACTIONS(1769),
    [sym_duet_dev_type_disc_device] = ACTIONS(1769),
    [sym_duet_dev_type_document_camera] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1769),
    [sym_duet_dev_type_hvac] = ACTIONS(1769),
    [sym_duet_dev_type_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_light] = ACTIONS(1769),
    [sym_duet_dev_type_monitor] = ACTIONS(1769),
    [sym_duet_dev_type_motor] = ACTIONS(1769),
    [sym_duet_dev_type_multi_window] = ACTIONS(1769),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1769),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1769),
    [sym_duet_dev_type_receiver] = ACTIONS(1769),
    [sym_duet_dev_type_security_system] = ACTIONS(1769),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1769),
    [sym_duet_dev_type_settop_box] = ACTIONS(1769),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1769),
    [sym_duet_dev_type_switcher] = ACTIONS(1769),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_tv] = ACTIONS(1769),
    [sym_duet_dev_type_utility] = ACTIONS(1769),
    [sym_duet_dev_type_vcr] = ACTIONS(1769),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_video_processor] = ACTIONS(1769),
    [sym_duet_dev_type_video_projector] = ACTIONS(1769),
    [sym_duet_dev_type_video_wall] = ACTIONS(1769),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1769),
    [sym_duet_dev_type_weather] = ACTIONS(1769),
    [sym_duet_dev_not_polled] = ACTIONS(1769),
    [sym_duet_dev_polled] = ACTIONS(1769),
    [sym_clkmgr_mode_network] = ACTIONS(1769),
    [sym_clkmgr_mode_standalone] = ACTIONS(1769),
    [sym_normal_standby] = ACTIONS(1769),
    [sym_normal_wake] = ACTIONS(1769),
    [sym_ip_multicast_ttl_option] = ACTIONS(1769),
    [sym_ip_ttl_subnet] = ACTIONS(1769),
    [sym_ip_ttl_site] = ACTIONS(1769),
    [sym_ip_ttl_region] = ACTIONS(1769),
    [sym_ip_ttl_continent] = ACTIONS(1769),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1769),
    [sym_ip_nodelay_on] = ACTIONS(1769),
    [sym_ip_nodelay_off] = ACTIONS(1769),
    [sym_amx_error] = ACTIONS(1769),
    [sym_amx_warning] = ACTIONS(1769),
    [sym_amx_info] = ACTIONS(1769),
    [sym_amx_debug] = ACTIONS(1769),
    [sym_netlinx_login_success] = ACTIONS(1769),
    [sym_netlinx_login_fail] = ACTIONS(1769),
    [sym_netlinx_logout] = ACTIONS(1769),
    [sym_null_str] = ACTIONS(1769),
    [sym_smtp_address] = ACTIONS(1769),
    [sym_smtp_port_number] = ACTIONS(1769),
    [sym_smtp_username] = ACTIONS(1769),
    [sym_smtp_password] = ACTIONS(1769),
    [sym_smtp_require_tls] = ACTIONS(1769),
    [sym_smtp_from] = ACTIONS(1769),
    [sym_smtp_tls_true] = ACTIONS(1769),
    [sym_smtp_tls_false] = ACTIONS(1769),
    [sym_listview_on_row_select_event] = ACTIONS(1769),
    [sym_data_structure_unknown] = ACTIONS(1769),
    [sym_data_structure_datafeed] = ACTIONS(1769),
    [sym_data_structure_datarecordset] = ACTIONS(1769),
    [sym_data_structure_datarecord] = ACTIONS(1769),
    [sym_data_structure_datafield] = ACTIONS(1769),
    [sym_data_max_content_field_count] = ACTIONS(1769),
    [sym_data_max_metadata_field_count] = ACTIONS(1769),
    [sym_data_max_id_length] = ACTIONS(1769),
    [sym_data_max_name_length] = ACTIONS(1769),
    [sym_data_max_description_length] = ACTIONS(1769),
    [sym_data_max_type_length] = ACTIONS(1769),
    [sym_data_max_format_length] = ACTIONS(1769),
    [sym_data_max_source_length] = ACTIONS(1769),
    [sym_data_max_value_length] = ACTIONS(1769),
    [sym_data_max_label_length] = ACTIONS(1769),
    [sym_data_max_url_length] = ACTIONS(1769),
    [sym_max_last_login_info_length] = ACTIONS(1769),
    [sym_data_type_unknown] = ACTIONS(1769),
    [sym_data_type_string] = ACTIONS(1769),
    [sym_data_type_datetime] = ACTIONS(1769),
    [sym_data_type_date] = ACTIONS(1769),
    [sym_data_type_time] = ACTIONS(1769),
    [sym_data_type_image] = ACTIONS(1769),
    [sym_data_format_url] = ACTIONS(1769),
    [sym_data_format_phone] = ACTIONS(1769),
    [sym_data_format_email] = ACTIONS(1769),
    [sym_data_format_iso8601] = ACTIONS(1769),
    [sym_auditlog_priv] = ACTIONS(1769),
    [sym_configuration_priv] = ACTIONS(1769),
    [sym_deviceconfig_priv] = ACTIONS(1769),
    [sym_ftp_priv] = ACTIONS(1769),
    [sym_http_priv] = ACTIONS(1769),
    [sym_networkconfig_priv] = ACTIONS(1769),
    [sym_programport_priv] = ACTIONS(1769),
    [sym_remoteui_priv] = ACTIONS(1769),
    [sym_securitycontrol_priv] = ACTIONS(1769),
    [sym_softwaremanagement_priv] = ACTIONS(1769),
    [sym_terminal_priv] = ACTIONS(1769),
    [sym_tpadmin_priv] = ACTIONS(1769),
    [sym_user1_priv] = ACTIONS(1769),
    [sym_user2_priv] = ACTIONS(1769),
    [sym_user3_priv] = ACTIONS(1769),
    [sym_user4_priv] = ACTIONS(1769),
    [sym_usermanagement_priv] = ACTIONS(1769),
    [sym_valid_account] = ACTIONS(1769),
    [sym_err_name_invalid] = ACTIONS(1769),
    [sym_err_password_invalid] = ACTIONS(1769),
    [sym_err_invalid_account] = ACTIONS(1769),
    [sym_err_no_authorization] = ACTIONS(1769),
    [sym_err_invalid_parameter] = ACTIONS(1769),
    [sym_err_user_locked_out] = ACTIONS(1769),
    [sym_err_user_account_expired] = ACTIONS(1769),
    [sym_err_general_error] = ACTIONS(1769),
    [sym_err_authentication_error] = ACTIONS(1769),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1769),
    [sym_tls_validate_certificate] = ACTIONS(1769),
    [sym_tdata] = ACTIONS(1769),
    [sym_tchannel] = ACTIONS(1769),
    [sym_tlevel] = ACTIONS(1769),
    [sym_tbutton] = ACTIONS(1769),
    [sym_ttimeline] = ACTIONS(1769),
    [sym_tcustom] = ACTIONS(1769),
    [sym_url_struct] = ACTIONS(1769),
    [sym_dns_struct] = ACTIONS(1769),
    [sym_ip_address_struct] = ACTIONS(1769),
    [sym_dev_info_struct] = ACTIONS(1769),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1769),
    [sym_data_feed] = ACTIONS(1769),
    [sym_data_field] = ACTIONS(1769),
    [sym_data_record] = ACTIONS(1769),
    [sym_wc_data_feed] = ACTIONS(1769),
    [sym_wc_data_field] = ACTIONS(1769),
    [sym_wc_data_record] = ACTIONS(1769),
    [sym_last_login_info] = ACTIONS(1769),
    [sym____reserved___] = ACTIONS(1769),
    [sym_date] = ACTIONS(1769),
    [sym_day] = ACTIONS(1769),
    [sym_time] = ACTIONS(1769),
    [sym_ldate] = ACTIONS(1769),
    [sym_push_device] = ACTIONS(1769),
    [sym_push_channel] = ACTIONS(1769),
    [sym_push_devchan] = ACTIONS(1769),
    [sym_release_device] = ACTIONS(1769),
    [sym_release_channel] = ACTIONS(1769),
    [sym_release_devchan] = ACTIONS(1769),
    [sym_master_slot] = ACTIONS(1769),
    [sym_get_pulse_time] = ACTIONS(1769),
    [sym_get_timer] = ACTIONS(1769),
    [sym_master_sn] = ACTIONS(1769),
    [sym_system_number] = ACTIONS(1769),
    [sym_button] = ACTIONS(1769),
    [sym_dv_channel] = ACTIONS(1769),
    [sym_data] = ACTIONS(1769),
    [sym_level] = ACTIONS(1769),
    [sym_timeline] = ACTIONS(1769),
    [sym_channel] = ACTIONS(1769),
    [sym_custom] = ACTIONS(1769),
    [sym_atoi] = ACTIONS(1769),
    [sym_atol] = ACTIONS(1769),
    [sym_atof] = ACTIONS(1769),
    [sym_itoa] = ACTIONS(1769),
    [sym_itohex] = ACTIONS(1769),
    [sym_hextoi] = ACTIONS(1769),
    [sym_ftoa] = ACTIONS(1769),
    [sym_find_string] = ACTIONS(1769),
    [sym_left_string] = ACTIONS(1769),
    [sym_right_string] = ACTIONS(1769),
    [sym_mid_string] = ACTIONS(1769),
    [sym_remove_string] = ACTIONS(1769),
    [sym_get_buffer_string] = ACTIONS(1769),
    [sym_get_buffer_char] = ACTIONS(1769),
    [sym_get_multi_buffer_string] = ACTIONS(1769),
    [sym_length_string] = ACTIONS(1769),
    [sym_max_length_string] = ACTIONS(1769),
    [sym_set_length_string] = ACTIONS(1769),
    [sym_compare_string] = ACTIONS(1769),
    [sym_upper_string] = ACTIONS(1769),
    [sym_lower_string] = ACTIONS(1769),
    [sym_get_last] = ACTIONS(1769),
    [sym_date_to_day] = ACTIONS(1769),
    [sym_date_to_month] = ACTIONS(1769),
    [sym_date_to_year] = ACTIONS(1769),
    [sym_time_to_hour] = ACTIONS(1769),
    [sym_time_to_minute] = ACTIONS(1769),
    [sym_time_to_second] = ACTIONS(1769),
    [sym_day_of_week] = ACTIONS(1769),
    [sym_set_pulse_time] = ACTIONS(1769),
    [sym_set_timer] = ACTIONS(1769),
    [sym_file_open] = ACTIONS(1769),
    [sym_file_close] = ACTIONS(1769),
    [sym_file_read] = ACTIONS(1769),
    [sym_file_read_line] = ACTIONS(1769),
    [sym_file_write] = ACTIONS(1769),
    [sym_file_write_line] = ACTIONS(1769),
    [sym_file_copy] = ACTIONS(1769),
    [sym_file_delete] = ACTIONS(1769),
    [sym_file_rename] = ACTIONS(1769),
    [sym_file_dir] = ACTIONS(1769),
    [sym_file_getdir] = ACTIONS(1769),
    [sym_file_setdir] = ACTIONS(1769),
    [sym_file_create_dir] = ACTIONS(1769),
    [sym_file_remove_dir] = ACTIONS(1769),
    [sym_file_seek] = ACTIONS(1769),
    [sym_length_array] = ACTIONS(1769),
    [sym_max_length_array] = ACTIONS(1769),
    [sym_set_length_array] = ACTIONS(1769),
    [sym_ip_client_open] = ACTIONS(1769),
    [sym_ip_client_close] = ACTIONS(1769),
    [sym_ip_server_open] = ACTIONS(1769),
    [sym_ip_server_close] = ACTIONS(1769),
    [sym_ip_bound_client_open] = ACTIONS(1769),
    [sym_ip_mc_server_open] = ACTIONS(1769),
    [sym_ip_set_option] = ACTIONS(1769),
    [sym_ssh_client_open] = ACTIONS(1769),
    [sym_ssh_client_close] = ACTIONS(1769),
    [sym_tls_client_open] = ACTIONS(1769),
    [sym_tls_client_close] = ACTIONS(1769),
    [sym_do_push] = ACTIONS(1769),
    [sym_do_push_timed] = ACTIONS(1769),
    [sym_do_release] = ACTIONS(1769),
    [sym_device_id] = ACTIONS(1769),
    [sym_device_info] = ACTIONS(1769),
    [sym_device_id_string] = ACTIONS(1769),
    [sym_device_standby] = ACTIONS(1769),
    [sym_device_wake] = ACTIONS(1769),
    [sym_timeline_create] = ACTIONS(1769),
    [sym_timeline_kill] = ACTIONS(1769),
    [sym_timeline_reload] = ACTIONS(1769),
    [sym_timeline_pause] = ACTIONS(1769),
    [sym_timeline_restart] = ACTIONS(1769),
    [sym_timeline_set] = ACTIONS(1769),
    [sym_timeline_get] = ACTIONS(1769),
    [sym_timeline_active] = ACTIONS(1769),
    [sym_timeline_dynamic_id] = ACTIONS(1769),
    [sym_variable_to_string] = ACTIONS(1769),
    [sym_string_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_string] = ACTIONS(1769),
    [sym_type_cast] = ACTIONS(1769),
    [sym_variable_to_xml] = ACTIONS(1769),
    [sym_xml_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_xml] = ACTIONS(1769),
    [sym_redirect_string] = ACTIONS(1769),
    [sym_random_number] = ACTIONS(1769),
    [sym_get_unique_id] = ACTIONS(1769),
    [sym_get_system_number] = ACTIONS(1769),
    [sym_set_system_number] = ACTIONS(1769),
    [sym_get_serial_number] = ACTIONS(1769),
    [sym_get_master_build] = ACTIONS(1769),
    [sym_set_validation_code] = ACTIONS(1769),
    [sym_reboot] = ACTIONS(1769),
    [sym_load_duet_module] = ACTIONS(1769),
    [sym_unload_duet_module] = ACTIONS(1769),
    [sym_duet_mem_size_set] = ACTIONS(1769),
    [sym_duet_mem_size_get] = ACTIONS(1769),
    [sym_dynamic_application_device] = ACTIONS(1769),
    [sym_dynamic_polled_port] = ACTIONS(1769),
    [sym_static_port_binding] = ACTIONS(1769),
    [sym_static_ip_binding] = ACTIONS(1769),
    [sym_get_dns_list] = ACTIONS(1769),
    [sym_set_dns_list] = ACTIONS(1769),
    [sym_get_ip_address] = ACTIONS(1769),
    [sym_set_ip_address] = ACTIONS(1769),
    [sym_get_url_list] = ACTIONS(1769),
    [sym_add_url_entry] = ACTIONS(1769),
    [sym_delete_url_entry] = ACTIONS(1769),
    [sym_abs_value] = ACTIONS(1769),
    [sym_max_value] = ACTIONS(1769),
    [sym_min_value] = ACTIONS(1769),
    [sym_exp_value] = ACTIONS(1769),
    [sym_log_value] = ACTIONS(1769),
    [sym_log10_value] = ACTIONS(1769),
    [sym_power_value] = ACTIONS(1769),
    [sym_sqrt_value] = ACTIONS(1769),
    [sym_raw_be] = ACTIONS(1769),
    [sym_raw_le] = ACTIONS(1769),
    [sym_internal_queue_size_set] = ACTIONS(1769),
    [sym_internal_queue_size_get] = ACTIONS(1769),
    [sym_internal_threshold_set] = ACTIONS(1769),
    [sym_internal_threshold_get] = ACTIONS(1769),
    [sym_rebuilt_event] = ACTIONS(1769),
    [sym_do_custom_event] = ACTIONS(1769),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1769),
    [sym_clkmgr_set_clk_source] = ACTIONS(1769),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1769),
    [sym_clkmgr_get_timezone] = ACTIONS(1769),
    [sym_clkmgr_set_timezone] = ACTIONS(1769),
    [sym_clkmgr_get_resync_period] = ACTIONS(1769),
    [sym_clkmgr_set_resync_period] = ACTIONS(1769),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_timeservers] = ACTIONS(1769),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_set_log_level] = ACTIONS(1769),
    [sym_get_log_level] = ACTIONS(1769),
    [sym_amx_log] = ACTIONS(1769),
    [sym_smtp_server_config_set] = ACTIONS(1769),
    [sym_smtp_server_config_get] = ACTIONS(1769),
    [sym_smtp_send] = ACTIONS(1769),
    [sym_validate_netlinx_account] = ACTIONS(1769),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1769),
    [sym_authenticate_certificate] = ACTIONS(1769),
    [sym_audit_netlinx_session_event] = ACTIONS(1769),
    [sym_audit_netlinx_generic_event] = ACTIONS(1769),
    [sym_get_available_flash_disk_space] = ACTIONS(1769),
    [sym_get_max_flash_disk_space] = ACTIONS(1769),
    [sym_data_create_feed] = ACTIONS(1769),
    [sym_data_delete_feed] = ACTIONS(1769),
    [sym_data_publish_feed] = ACTIONS(1769),
    [sym_data_get_published_feed] = ACTIONS(1769),
    [sym_data_add_record] = ACTIONS(1769),
    [sym_data_get_event_record] = ACTIONS(1769),
    [sym__wc_data_create_feed] = ACTIONS(1769),
    [sym__wc_data_add_record] = ACTIONS(1769),
    [sym__wc_data_get_event_record] = ACTIONS(1769),
    [sym_format] = ACTIONS(1769),
    [sym_astro_clock] = ACTIONS(1769),
    [sym_set_outdoor_temperature] = ACTIONS(1769),
    [sym_set_virtual_level_count] = ACTIONS(1769),
    [sym_set_virtual_channel_count] = ACTIONS(1769),
    [sym_set_virtual_port_count] = ACTIONS(1769),
    [sym__wc_encode_int] = ACTIONS(1769),
    [sym__wc_decode_int] = ACTIONS(1769),
    [sym__wc_int] = ACTIONS(1769),
    [sym__wc_to_ch_int] = ACTIONS(1769),
    [sym__ch_to_wc_int] = ACTIONS(1769),
    [sym__wc_find_string_int] = ACTIONS(1769),
    [sym__wc_left_string_int] = ACTIONS(1769),
    [sym__wc_lower_string_int] = ACTIONS(1769),
    [sym__wc_mid_string_int] = ACTIONS(1769),
    [sym__wc_remove_string_int] = ACTIONS(1769),
    [sym__wc_right_string_int] = ACTIONS(1769),
    [sym__wc_upper_string_int] = ACTIONS(1769),
    [sym__wc_compare_string_int] = ACTIONS(1769),
    [sym__wc_get_buffer_char_int] = ACTIONS(1769),
    [sym__wc_get_buffer_string_int] = ACTIONS(1769),
    [sym__wc_concat_string_int] = ACTIONS(1769),
    [sym__wc_file_open_int] = ACTIONS(1769),
    [sym__wc_file_close_int] = ACTIONS(1769),
    [sym__wc_file_read_int] = ACTIONS(1769),
    [sym__wc_file_read_line_int] = ACTIONS(1769),
    [sym__wc_file_write_int] = ACTIONS(1769),
    [sym__wc_file_write_line_int] = ACTIONS(1769),
    [aux_sym_preproc_include_token1] = ACTIONS(1769),
    [aux_sym_preproc_define_token1] = ACTIONS(1771),
    [aux_sym_preproc_warn_token1] = ACTIONS(1771),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1771),
    [anon_sym_LBRACK] = ACTIONS(1771),
    [anon_sym_LBRACE] = ACTIONS(1771),
    [anon_sym_LPAREN] = ACTIONS(1769),
    [anon_sym_BANG] = ACTIONS(1771),
    [anon_sym_TILDE] = ACTIONS(1771),
    [anon_sym_DASH] = ACTIONS(1769),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_DASH_DASH] = ACTIONS(1771),
    [anon_sym_PLUS_PLUS] = ACTIONS(1771),
    [anon_sym_DQUOTE] = ACTIONS(1771),
    [anon_sym_SQUOTE] = ACTIONS(1771),
    [sym_number_literal] = ACTIONS(1771),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(386)] = {
    [ts_builtin_sym_end] = ACTIONS(1771),
    [sym_identifier] = ACTIONS(1769),
    [sym_program_name_keyword] = ACTIONS(1769),
    [sym_module_name_keyword] = ACTIONS(1769),
    [sym_define_device_keyword] = ACTIONS(1769),
    [sym_define_combine_keyword] = ACTIONS(1769),
    [sym_define_constant_keyword] = ACTIONS(1769),
    [sym_define_type_keyword] = ACTIONS(1769),
    [sym_define_variable_keyword] = ACTIONS(1769),
    [sym_define_system_variable_keyword] = ACTIONS(1769),
    [sym_define_connect_level_keyword] = ACTIONS(1769),
    [sym_define_latching_keyword] = ACTIONS(1769),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1769),
    [sym_define_toggling_keyword] = ACTIONS(1769),
    [sym_define_call_keyword] = ACTIONS(1769),
    [sym_define_function_keyword] = ACTIONS(1769),
    [sym_define_library_function_keyword] = ACTIONS(1769),
    [sym_define_start_keyword] = ACTIONS(1769),
    [sym_define_event_keyword] = ACTIONS(1769),
    [sym_define_program_keyword] = ACTIONS(1769),
    [sym_define_module_keyword] = ACTIONS(1769),
    [sym_char_keyword] = ACTIONS(1769),
    [sym_widechar_keyword] = ACTIONS(1769),
    [sym_integer_keyword] = ACTIONS(1769),
    [sym_sinteger_keyword] = ACTIONS(1769),
    [sym_long_keyword] = ACTIONS(1769),
    [sym_slong_keyword] = ACTIONS(1769),
    [sym_float_keyword] = ACTIONS(1769),
    [sym_double_keyword] = ACTIONS(1769),
    [sym_variant_keyword] = ACTIONS(1769),
    [sym_variantarray_keyword] = ACTIONS(1769),
    [sym_local_var_keyword] = ACTIONS(1769),
    [sym_stack_var_keyword] = ACTIONS(1769),
    [sym_constant_keyword] = ACTIONS(1769),
    [sym_volatile_keyword] = ACTIONS(1769),
    [sym_non_volatile_keyword] = ACTIONS(1769),
    [sym_persistent_keyword] = ACTIONS(1769),
    [sym_dev_keyword] = ACTIONS(1769),
    [sym_devlev_keyword] = ACTIONS(1769),
    [sym_devchan_keyword] = ACTIONS(1769),
    [sym_if_keyword] = ACTIONS(1769),
    [sym_else_keyword] = ACTIONS(1769),
    [sym_switch_keyword] = ACTIONS(1769),
    [sym_case_keyword] = ACTIONS(1769),
    [sym_default_keyword] = ACTIONS(1769),
    [sym_while_keyword] = ACTIONS(1769),
    [sym_for_keyword] = ACTIONS(1769),
    [sym_break_keyword] = ACTIONS(1769),
    [sym_continue_keyword] = ACTIONS(1769),
    [sym_return_keyword] = ACTIONS(1769),
    [sym_select_keyword] = ACTIONS(1769),
    [sym_active_keyword] = ACTIONS(1769),
    [sym_struct_keyword] = ACTIONS(1769),
    [sym_structure_keyword] = ACTIONS(1769),
    [sym_button_event_keyword] = ACTIONS(1769),
    [sym_channel_event_keyword] = ACTIONS(1769),
    [sym_level_event_keyword] = ACTIONS(1769),
    [sym_data_event_keyword] = ACTIONS(1769),
    [sym_timeline_event_keyword] = ACTIONS(1769),
    [sym_custom_event_keyword] = ACTIONS(1769),
    [sym_push_keyword] = ACTIONS(1769),
    [sym_release_keyword] = ACTIONS(1769),
    [aux_sym_on_keyword_token1] = ACTIONS(1769),
    [aux_sym_off_keyword_token1] = ACTIONS(1769),
    [sym_send_level_keyword] = ACTIONS(1769),
    [sym_send_string_keyword] = ACTIONS(1769),
    [sym_send_command_keyword] = ACTIONS(1769),
    [sym_clear_buffer_keyword] = ACTIONS(1769),
    [sym_create_buffer_keyword] = ACTIONS(1769),
    [sym_create_multi_buffer_keyword] = ACTIONS(1769),
    [sym_call_keyword] = ACTIONS(1769),
    [sym_system_call_keyword] = ACTIONS(1769),
    [sym_devchan_to_keyword] = ACTIONS(1769),
    [sym_devchan_min_to_keyword] = ACTIONS(1769),
    [sym_devchan_total_off_keyword] = ACTIONS(1769),
    [sym_devchan_pulse_keyword] = ACTIONS(1769),
    [sym_wait_keyword] = ACTIONS(1769),
    [sym_cancel_wait_keyword] = ACTIONS(1769),
    [sym_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1769),
    [sym_not] = ACTIONS(1769),
    [sym_bnot] = ACTIONS(1769),
    [sym___file__] = ACTIONS(1769),
    [sym___date__] = ACTIONS(1769),
    [sym___ldate__] = ACTIONS(1769),
    [sym___time__] = ACTIONS(1769),
    [sym___line__] = ACTIONS(1769),
    [sym___name__] = ACTIONS(1769),
    [sym___netlinx__] = ACTIONS(1769),
    [sym_netlinx_axi_version] = ACTIONS(1769),
    [sym_url_flg_tcp] = ACTIONS(1769),
    [sym_url_flg_accinfopresent] = ACTIONS(1769),
    [sym_url_flg_temp] = ACTIONS(1769),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1769),
    [sym_url_flg_stat_mask] = ACTIONS(1769),
    [sym_url_flg_stat_lookup] = ACTIONS(1769),
    [sym_url_flg_stat_connecting] = ACTIONS(1769),
    [sym_url_flg_stat_waiting] = ACTIONS(1769),
    [sym_url_flg_stat_connected] = ACTIONS(1769),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1769),
    [sym_first_virtual_device] = ACTIONS(1769),
    [sym_first_local_port] = ACTIONS(1769),
    [sym_dynamic_virtual_device] = ACTIONS(1769),
    [sym_timeline_once] = ACTIONS(1769),
    [sym_timeline_repeat] = ACTIONS(1769),
    [sym_timeline_absolute] = ACTIONS(1769),
    [sym_timeline_relative] = ACTIONS(1769),
    [sym_do_push_timed_infinite] = ACTIONS(1769),
    [sym_source_type_no_address] = ACTIONS(1769),
    [sym_source_type_neuron_id] = ACTIONS(1769),
    [sym_source_type_ip_address] = ACTIONS(1769),
    [sym_source_type_axlink] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1769),
    [sym_source_type_ip_socket_address] = ACTIONS(1769),
    [sym_source_type_rs232] = ACTIONS(1769),
    [sym_source_type_internal] = ACTIONS(1769),
    [sym_source_type_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1769),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1769),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1769),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1769),
    [sym_true] = ACTIONS(1769),
    [sym_false] = ACTIONS(1769),
    [sym_file_read_only] = ACTIONS(1769),
    [sym_file_rw_new] = ACTIONS(1769),
    [sym_file_rw_append] = ACTIONS(1769),
    [sym_ip_tcp] = ACTIONS(1769),
    [sym_ip_udp] = ACTIONS(1769),
    [sym_ip_udp_2way] = ACTIONS(1769),
    [sym_xml_encode_types] = ACTIONS(1769),
    [sym_xml_encode_char_as_list] = ACTIONS(1769),
    [sym_xml_encode_le] = ACTIONS(1769),
    [sym_xml_decode_types] = ACTIONS(1769),
    [sym_xml_decode_no_preserve] = ACTIONS(1769),
    [sym_duet_dev_type_display_device] = ACTIONS(1769),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_io_device] = ACTIONS(1769),
    [sym_duet_dev_type_relay_device] = ACTIONS(1769),
    [sym_duet_dev_type_ups] = ACTIONS(1769),
    [sym_duet_dev_type_amplifier] = ACTIONS(1769),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1769),
    [sym_duet_dev_type_camera] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1769),
    [sym_duet_dev_type_dss] = ACTIONS(1769),
    [sym_duet_dev_type_dvr] = ACTIONS(1769),
    [sym_duet_dev_type_disc_device] = ACTIONS(1769),
    [sym_duet_dev_type_document_camera] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1769),
    [sym_duet_dev_type_hvac] = ACTIONS(1769),
    [sym_duet_dev_type_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_light] = ACTIONS(1769),
    [sym_duet_dev_type_monitor] = ACTIONS(1769),
    [sym_duet_dev_type_motor] = ACTIONS(1769),
    [sym_duet_dev_type_multi_window] = ACTIONS(1769),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1769),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1769),
    [sym_duet_dev_type_receiver] = ACTIONS(1769),
    [sym_duet_dev_type_security_system] = ACTIONS(1769),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1769),
    [sym_duet_dev_type_settop_box] = ACTIONS(1769),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1769),
    [sym_duet_dev_type_switcher] = ACTIONS(1769),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_tv] = ACTIONS(1769),
    [sym_duet_dev_type_utility] = ACTIONS(1769),
    [sym_duet_dev_type_vcr] = ACTIONS(1769),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_video_processor] = ACTIONS(1769),
    [sym_duet_dev_type_video_projector] = ACTIONS(1769),
    [sym_duet_dev_type_video_wall] = ACTIONS(1769),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1769),
    [sym_duet_dev_type_weather] = ACTIONS(1769),
    [sym_duet_dev_not_polled] = ACTIONS(1769),
    [sym_duet_dev_polled] = ACTIONS(1769),
    [sym_clkmgr_mode_network] = ACTIONS(1769),
    [sym_clkmgr_mode_standalone] = ACTIONS(1769),
    [sym_normal_standby] = ACTIONS(1769),
    [sym_normal_wake] = ACTIONS(1769),
    [sym_ip_multicast_ttl_option] = ACTIONS(1769),
    [sym_ip_ttl_subnet] = ACTIONS(1769),
    [sym_ip_ttl_site] = ACTIONS(1769),
    [sym_ip_ttl_region] = ACTIONS(1769),
    [sym_ip_ttl_continent] = ACTIONS(1769),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1769),
    [sym_ip_nodelay_on] = ACTIONS(1769),
    [sym_ip_nodelay_off] = ACTIONS(1769),
    [sym_amx_error] = ACTIONS(1769),
    [sym_amx_warning] = ACTIONS(1769),
    [sym_amx_info] = ACTIONS(1769),
    [sym_amx_debug] = ACTIONS(1769),
    [sym_netlinx_login_success] = ACTIONS(1769),
    [sym_netlinx_login_fail] = ACTIONS(1769),
    [sym_netlinx_logout] = ACTIONS(1769),
    [sym_null_str] = ACTIONS(1769),
    [sym_smtp_address] = ACTIONS(1769),
    [sym_smtp_port_number] = ACTIONS(1769),
    [sym_smtp_username] = ACTIONS(1769),
    [sym_smtp_password] = ACTIONS(1769),
    [sym_smtp_require_tls] = ACTIONS(1769),
    [sym_smtp_from] = ACTIONS(1769),
    [sym_smtp_tls_true] = ACTIONS(1769),
    [sym_smtp_tls_false] = ACTIONS(1769),
    [sym_listview_on_row_select_event] = ACTIONS(1769),
    [sym_data_structure_unknown] = ACTIONS(1769),
    [sym_data_structure_datafeed] = ACTIONS(1769),
    [sym_data_structure_datarecordset] = ACTIONS(1769),
    [sym_data_structure_datarecord] = ACTIONS(1769),
    [sym_data_structure_datafield] = ACTIONS(1769),
    [sym_data_max_content_field_count] = ACTIONS(1769),
    [sym_data_max_metadata_field_count] = ACTIONS(1769),
    [sym_data_max_id_length] = ACTIONS(1769),
    [sym_data_max_name_length] = ACTIONS(1769),
    [sym_data_max_description_length] = ACTIONS(1769),
    [sym_data_max_type_length] = ACTIONS(1769),
    [sym_data_max_format_length] = ACTIONS(1769),
    [sym_data_max_source_length] = ACTIONS(1769),
    [sym_data_max_value_length] = ACTIONS(1769),
    [sym_data_max_label_length] = ACTIONS(1769),
    [sym_data_max_url_length] = ACTIONS(1769),
    [sym_max_last_login_info_length] = ACTIONS(1769),
    [sym_data_type_unknown] = ACTIONS(1769),
    [sym_data_type_string] = ACTIONS(1769),
    [sym_data_type_datetime] = ACTIONS(1769),
    [sym_data_type_date] = ACTIONS(1769),
    [sym_data_type_time] = ACTIONS(1769),
    [sym_data_type_image] = ACTIONS(1769),
    [sym_data_format_url] = ACTIONS(1769),
    [sym_data_format_phone] = ACTIONS(1769),
    [sym_data_format_email] = ACTIONS(1769),
    [sym_data_format_iso8601] = ACTIONS(1769),
    [sym_auditlog_priv] = ACTIONS(1769),
    [sym_configuration_priv] = ACTIONS(1769),
    [sym_deviceconfig_priv] = ACTIONS(1769),
    [sym_ftp_priv] = ACTIONS(1769),
    [sym_http_priv] = ACTIONS(1769),
    [sym_networkconfig_priv] = ACTIONS(1769),
    [sym_programport_priv] = ACTIONS(1769),
    [sym_remoteui_priv] = ACTIONS(1769),
    [sym_securitycontrol_priv] = ACTIONS(1769),
    [sym_softwaremanagement_priv] = ACTIONS(1769),
    [sym_terminal_priv] = ACTIONS(1769),
    [sym_tpadmin_priv] = ACTIONS(1769),
    [sym_user1_priv] = ACTIONS(1769),
    [sym_user2_priv] = ACTIONS(1769),
    [sym_user3_priv] = ACTIONS(1769),
    [sym_user4_priv] = ACTIONS(1769),
    [sym_usermanagement_priv] = ACTIONS(1769),
    [sym_valid_account] = ACTIONS(1769),
    [sym_err_name_invalid] = ACTIONS(1769),
    [sym_err_password_invalid] = ACTIONS(1769),
    [sym_err_invalid_account] = ACTIONS(1769),
    [sym_err_no_authorization] = ACTIONS(1769),
    [sym_err_invalid_parameter] = ACTIONS(1769),
    [sym_err_user_locked_out] = ACTIONS(1769),
    [sym_err_user_account_expired] = ACTIONS(1769),
    [sym_err_general_error] = ACTIONS(1769),
    [sym_err_authentication_error] = ACTIONS(1769),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1769),
    [sym_tls_validate_certificate] = ACTIONS(1769),
    [sym_tdata] = ACTIONS(1769),
    [sym_tchannel] = ACTIONS(1769),
    [sym_tlevel] = ACTIONS(1769),
    [sym_tbutton] = ACTIONS(1769),
    [sym_ttimeline] = ACTIONS(1769),
    [sym_tcustom] = ACTIONS(1769),
    [sym_url_struct] = ACTIONS(1769),
    [sym_dns_struct] = ACTIONS(1769),
    [sym_ip_address_struct] = ACTIONS(1769),
    [sym_dev_info_struct] = ACTIONS(1769),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1769),
    [sym_data_feed] = ACTIONS(1769),
    [sym_data_field] = ACTIONS(1769),
    [sym_data_record] = ACTIONS(1769),
    [sym_wc_data_feed] = ACTIONS(1769),
    [sym_wc_data_field] = ACTIONS(1769),
    [sym_wc_data_record] = ACTIONS(1769),
    [sym_last_login_info] = ACTIONS(1769),
    [sym____reserved___] = ACTIONS(1769),
    [sym_date] = ACTIONS(1769),
    [sym_day] = ACTIONS(1769),
    [sym_time] = ACTIONS(1769),
    [sym_ldate] = ACTIONS(1769),
    [sym_push_device] = ACTIONS(1769),
    [sym_push_channel] = ACTIONS(1769),
    [sym_push_devchan] = ACTIONS(1769),
    [sym_release_device] = ACTIONS(1769),
    [sym_release_channel] = ACTIONS(1769),
    [sym_release_devchan] = ACTIONS(1769),
    [sym_master_slot] = ACTIONS(1769),
    [sym_get_pulse_time] = ACTIONS(1769),
    [sym_get_timer] = ACTIONS(1769),
    [sym_master_sn] = ACTIONS(1769),
    [sym_system_number] = ACTIONS(1769),
    [sym_button] = ACTIONS(1769),
    [sym_dv_channel] = ACTIONS(1769),
    [sym_data] = ACTIONS(1769),
    [sym_level] = ACTIONS(1769),
    [sym_timeline] = ACTIONS(1769),
    [sym_channel] = ACTIONS(1769),
    [sym_custom] = ACTIONS(1769),
    [sym_atoi] = ACTIONS(1769),
    [sym_atol] = ACTIONS(1769),
    [sym_atof] = ACTIONS(1769),
    [sym_itoa] = ACTIONS(1769),
    [sym_itohex] = ACTIONS(1769),
    [sym_hextoi] = ACTIONS(1769),
    [sym_ftoa] = ACTIONS(1769),
    [sym_find_string] = ACTIONS(1769),
    [sym_left_string] = ACTIONS(1769),
    [sym_right_string] = ACTIONS(1769),
    [sym_mid_string] = ACTIONS(1769),
    [sym_remove_string] = ACTIONS(1769),
    [sym_get_buffer_string] = ACTIONS(1769),
    [sym_get_buffer_char] = ACTIONS(1769),
    [sym_get_multi_buffer_string] = ACTIONS(1769),
    [sym_length_string] = ACTIONS(1769),
    [sym_max_length_string] = ACTIONS(1769),
    [sym_set_length_string] = ACTIONS(1769),
    [sym_compare_string] = ACTIONS(1769),
    [sym_upper_string] = ACTIONS(1769),
    [sym_lower_string] = ACTIONS(1769),
    [sym_get_last] = ACTIONS(1769),
    [sym_date_to_day] = ACTIONS(1769),
    [sym_date_to_month] = ACTIONS(1769),
    [sym_date_to_year] = ACTIONS(1769),
    [sym_time_to_hour] = ACTIONS(1769),
    [sym_time_to_minute] = ACTIONS(1769),
    [sym_time_to_second] = ACTIONS(1769),
    [sym_day_of_week] = ACTIONS(1769),
    [sym_set_pulse_time] = ACTIONS(1769),
    [sym_set_timer] = ACTIONS(1769),
    [sym_file_open] = ACTIONS(1769),
    [sym_file_close] = ACTIONS(1769),
    [sym_file_read] = ACTIONS(1769),
    [sym_file_read_line] = ACTIONS(1769),
    [sym_file_write] = ACTIONS(1769),
    [sym_file_write_line] = ACTIONS(1769),
    [sym_file_copy] = ACTIONS(1769),
    [sym_file_delete] = ACTIONS(1769),
    [sym_file_rename] = ACTIONS(1769),
    [sym_file_dir] = ACTIONS(1769),
    [sym_file_getdir] = ACTIONS(1769),
    [sym_file_setdir] = ACTIONS(1769),
    [sym_file_create_dir] = ACTIONS(1769),
    [sym_file_remove_dir] = ACTIONS(1769),
    [sym_file_seek] = ACTIONS(1769),
    [sym_length_array] = ACTIONS(1769),
    [sym_max_length_array] = ACTIONS(1769),
    [sym_set_length_array] = ACTIONS(1769),
    [sym_ip_client_open] = ACTIONS(1769),
    [sym_ip_client_close] = ACTIONS(1769),
    [sym_ip_server_open] = ACTIONS(1769),
    [sym_ip_server_close] = ACTIONS(1769),
    [sym_ip_bound_client_open] = ACTIONS(1769),
    [sym_ip_mc_server_open] = ACTIONS(1769),
    [sym_ip_set_option] = ACTIONS(1769),
    [sym_ssh_client_open] = ACTIONS(1769),
    [sym_ssh_client_close] = ACTIONS(1769),
    [sym_tls_client_open] = ACTIONS(1769),
    [sym_tls_client_close] = ACTIONS(1769),
    [sym_do_push] = ACTIONS(1769),
    [sym_do_push_timed] = ACTIONS(1769),
    [sym_do_release] = ACTIONS(1769),
    [sym_device_id] = ACTIONS(1769),
    [sym_device_info] = ACTIONS(1769),
    [sym_device_id_string] = ACTIONS(1769),
    [sym_device_standby] = ACTIONS(1769),
    [sym_device_wake] = ACTIONS(1769),
    [sym_timeline_create] = ACTIONS(1769),
    [sym_timeline_kill] = ACTIONS(1769),
    [sym_timeline_reload] = ACTIONS(1769),
    [sym_timeline_pause] = ACTIONS(1769),
    [sym_timeline_restart] = ACTIONS(1769),
    [sym_timeline_set] = ACTIONS(1769),
    [sym_timeline_get] = ACTIONS(1769),
    [sym_timeline_active] = ACTIONS(1769),
    [sym_timeline_dynamic_id] = ACTIONS(1769),
    [sym_variable_to_string] = ACTIONS(1769),
    [sym_string_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_string] = ACTIONS(1769),
    [sym_type_cast] = ACTIONS(1769),
    [sym_variable_to_xml] = ACTIONS(1769),
    [sym_xml_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_xml] = ACTIONS(1769),
    [sym_redirect_string] = ACTIONS(1769),
    [sym_random_number] = ACTIONS(1769),
    [sym_get_unique_id] = ACTIONS(1769),
    [sym_get_system_number] = ACTIONS(1769),
    [sym_set_system_number] = ACTIONS(1769),
    [sym_get_serial_number] = ACTIONS(1769),
    [sym_get_master_build] = ACTIONS(1769),
    [sym_set_validation_code] = ACTIONS(1769),
    [sym_reboot] = ACTIONS(1769),
    [sym_load_duet_module] = ACTIONS(1769),
    [sym_unload_duet_module] = ACTIONS(1769),
    [sym_duet_mem_size_set] = ACTIONS(1769),
    [sym_duet_mem_size_get] = ACTIONS(1769),
    [sym_dynamic_application_device] = ACTIONS(1769),
    [sym_dynamic_polled_port] = ACTIONS(1769),
    [sym_static_port_binding] = ACTIONS(1769),
    [sym_static_ip_binding] = ACTIONS(1769),
    [sym_get_dns_list] = ACTIONS(1769),
    [sym_set_dns_list] = ACTIONS(1769),
    [sym_get_ip_address] = ACTIONS(1769),
    [sym_set_ip_address] = ACTIONS(1769),
    [sym_get_url_list] = ACTIONS(1769),
    [sym_add_url_entry] = ACTIONS(1769),
    [sym_delete_url_entry] = ACTIONS(1769),
    [sym_abs_value] = ACTIONS(1769),
    [sym_max_value] = ACTIONS(1769),
    [sym_min_value] = ACTIONS(1769),
    [sym_exp_value] = ACTIONS(1769),
    [sym_log_value] = ACTIONS(1769),
    [sym_log10_value] = ACTIONS(1769),
    [sym_power_value] = ACTIONS(1769),
    [sym_sqrt_value] = ACTIONS(1769),
    [sym_raw_be] = ACTIONS(1769),
    [sym_raw_le] = ACTIONS(1769),
    [sym_internal_queue_size_set] = ACTIONS(1769),
    [sym_internal_queue_size_get] = ACTIONS(1769),
    [sym_internal_threshold_set] = ACTIONS(1769),
    [sym_internal_threshold_get] = ACTIONS(1769),
    [sym_rebuilt_event] = ACTIONS(1769),
    [sym_do_custom_event] = ACTIONS(1769),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1769),
    [sym_clkmgr_set_clk_source] = ACTIONS(1769),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1769),
    [sym_clkmgr_get_timezone] = ACTIONS(1769),
    [sym_clkmgr_set_timezone] = ACTIONS(1769),
    [sym_clkmgr_get_resync_period] = ACTIONS(1769),
    [sym_clkmgr_set_resync_period] = ACTIONS(1769),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_timeservers] = ACTIONS(1769),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_set_log_level] = ACTIONS(1769),
    [sym_get_log_level] = ACTIONS(1769),
    [sym_amx_log] = ACTIONS(1769),
    [sym_smtp_server_config_set] = ACTIONS(1769),
    [sym_smtp_server_config_get] = ACTIONS(1769),
    [sym_smtp_send] = ACTIONS(1769),
    [sym_validate_netlinx_account] = ACTIONS(1769),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1769),
    [sym_authenticate_certificate] = ACTIONS(1769),
    [sym_audit_netlinx_session_event] = ACTIONS(1769),
    [sym_audit_netlinx_generic_event] = ACTIONS(1769),
    [sym_get_available_flash_disk_space] = ACTIONS(1769),
    [sym_get_max_flash_disk_space] = ACTIONS(1769),
    [sym_data_create_feed] = ACTIONS(1769),
    [sym_data_delete_feed] = ACTIONS(1769),
    [sym_data_publish_feed] = ACTIONS(1769),
    [sym_data_get_published_feed] = ACTIONS(1769),
    [sym_data_add_record] = ACTIONS(1769),
    [sym_data_get_event_record] = ACTIONS(1769),
    [sym__wc_data_create_feed] = ACTIONS(1769),
    [sym__wc_data_add_record] = ACTIONS(1769),
    [sym__wc_data_get_event_record] = ACTIONS(1769),
    [sym_format] = ACTIONS(1769),
    [sym_astro_clock] = ACTIONS(1769),
    [sym_set_outdoor_temperature] = ACTIONS(1769),
    [sym_set_virtual_level_count] = ACTIONS(1769),
    [sym_set_virtual_channel_count] = ACTIONS(1769),
    [sym_set_virtual_port_count] = ACTIONS(1769),
    [sym__wc_encode_int] = ACTIONS(1769),
    [sym__wc_decode_int] = ACTIONS(1769),
    [sym__wc_int] = ACTIONS(1769),
    [sym__wc_to_ch_int] = ACTIONS(1769),
    [sym__ch_to_wc_int] = ACTIONS(1769),
    [sym__wc_find_string_int] = ACTIONS(1769),
    [sym__wc_left_string_int] = ACTIONS(1769),
    [sym__wc_lower_string_int] = ACTIONS(1769),
    [sym__wc_mid_string_int] = ACTIONS(1769),
    [sym__wc_remove_string_int] = ACTIONS(1769),
    [sym__wc_right_string_int] = ACTIONS(1769),
    [sym__wc_upper_string_int] = ACTIONS(1769),
    [sym__wc_compare_string_int] = ACTIONS(1769),
    [sym__wc_get_buffer_char_int] = ACTIONS(1769),
    [sym__wc_get_buffer_string_int] = ACTIONS(1769),
    [sym__wc_concat_string_int] = ACTIONS(1769),
    [sym__wc_file_open_int] = ACTIONS(1769),
    [sym__wc_file_close_int] = ACTIONS(1769),
    [sym__wc_file_read_int] = ACTIONS(1769),
    [sym__wc_file_read_line_int] = ACTIONS(1769),
    [sym__wc_file_write_int] = ACTIONS(1769),
    [sym__wc_file_write_line_int] = ACTIONS(1769),
    [aux_sym_preproc_include_token1] = ACTIONS(1769),
    [aux_sym_preproc_define_token1] = ACTIONS(1771),
    [aux_sym_preproc_warn_token1] = ACTIONS(1771),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1771),
    [anon_sym_LBRACK] = ACTIONS(1771),
    [anon_sym_LBRACE] = ACTIONS(1771),
    [anon_sym_LPAREN] = ACTIONS(1769),
    [anon_sym_BANG] = ACTIONS(1771),
    [anon_sym_TILDE] = ACTIONS(1771),
    [anon_sym_DASH] = ACTIONS(1769),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_DASH_DASH] = ACTIONS(1771),
    [anon_sym_PLUS_PLUS] = ACTIONS(1771),
    [anon_sym_DQUOTE] = ACTIONS(1771),
    [anon_sym_SQUOTE] = ACTIONS(1771),
    [sym_number_literal] = ACTIONS(1771),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(387)] = {
    [ts_builtin_sym_end] = ACTIONS(1771),
    [sym_identifier] = ACTIONS(1769),
    [sym_program_name_keyword] = ACTIONS(1769),
    [sym_module_name_keyword] = ACTIONS(1769),
    [sym_define_device_keyword] = ACTIONS(1769),
    [sym_define_combine_keyword] = ACTIONS(1769),
    [sym_define_constant_keyword] = ACTIONS(1769),
    [sym_define_type_keyword] = ACTIONS(1769),
    [sym_define_variable_keyword] = ACTIONS(1769),
    [sym_define_system_variable_keyword] = ACTIONS(1769),
    [sym_define_connect_level_keyword] = ACTIONS(1769),
    [sym_define_latching_keyword] = ACTIONS(1769),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1769),
    [sym_define_toggling_keyword] = ACTIONS(1769),
    [sym_define_call_keyword] = ACTIONS(1769),
    [sym_define_function_keyword] = ACTIONS(1769),
    [sym_define_library_function_keyword] = ACTIONS(1769),
    [sym_define_start_keyword] = ACTIONS(1769),
    [sym_define_event_keyword] = ACTIONS(1769),
    [sym_define_program_keyword] = ACTIONS(1769),
    [sym_define_module_keyword] = ACTIONS(1769),
    [sym_char_keyword] = ACTIONS(1769),
    [sym_widechar_keyword] = ACTIONS(1769),
    [sym_integer_keyword] = ACTIONS(1769),
    [sym_sinteger_keyword] = ACTIONS(1769),
    [sym_long_keyword] = ACTIONS(1769),
    [sym_slong_keyword] = ACTIONS(1769),
    [sym_float_keyword] = ACTIONS(1769),
    [sym_double_keyword] = ACTIONS(1769),
    [sym_variant_keyword] = ACTIONS(1769),
    [sym_variantarray_keyword] = ACTIONS(1769),
    [sym_local_var_keyword] = ACTIONS(1769),
    [sym_stack_var_keyword] = ACTIONS(1769),
    [sym_constant_keyword] = ACTIONS(1769),
    [sym_volatile_keyword] = ACTIONS(1769),
    [sym_non_volatile_keyword] = ACTIONS(1769),
    [sym_persistent_keyword] = ACTIONS(1769),
    [sym_dev_keyword] = ACTIONS(1769),
    [sym_devlev_keyword] = ACTIONS(1769),
    [sym_devchan_keyword] = ACTIONS(1769),
    [sym_if_keyword] = ACTIONS(1769),
    [sym_else_keyword] = ACTIONS(1769),
    [sym_switch_keyword] = ACTIONS(1769),
    [sym_case_keyword] = ACTIONS(1769),
    [sym_default_keyword] = ACTIONS(1769),
    [sym_while_keyword] = ACTIONS(1769),
    [sym_for_keyword] = ACTIONS(1769),
    [sym_break_keyword] = ACTIONS(1769),
    [sym_continue_keyword] = ACTIONS(1769),
    [sym_return_keyword] = ACTIONS(1769),
    [sym_select_keyword] = ACTIONS(1769),
    [sym_active_keyword] = ACTIONS(1769),
    [sym_struct_keyword] = ACTIONS(1769),
    [sym_structure_keyword] = ACTIONS(1769),
    [sym_button_event_keyword] = ACTIONS(1769),
    [sym_channel_event_keyword] = ACTIONS(1769),
    [sym_level_event_keyword] = ACTIONS(1769),
    [sym_data_event_keyword] = ACTIONS(1769),
    [sym_timeline_event_keyword] = ACTIONS(1769),
    [sym_custom_event_keyword] = ACTIONS(1769),
    [sym_push_keyword] = ACTIONS(1769),
    [sym_release_keyword] = ACTIONS(1769),
    [aux_sym_on_keyword_token1] = ACTIONS(1769),
    [aux_sym_off_keyword_token1] = ACTIONS(1769),
    [sym_send_level_keyword] = ACTIONS(1769),
    [sym_send_string_keyword] = ACTIONS(1769),
    [sym_send_command_keyword] = ACTIONS(1769),
    [sym_clear_buffer_keyword] = ACTIONS(1769),
    [sym_create_buffer_keyword] = ACTIONS(1769),
    [sym_create_multi_buffer_keyword] = ACTIONS(1769),
    [sym_call_keyword] = ACTIONS(1769),
    [sym_system_call_keyword] = ACTIONS(1769),
    [sym_devchan_to_keyword] = ACTIONS(1769),
    [sym_devchan_min_to_keyword] = ACTIONS(1769),
    [sym_devchan_total_off_keyword] = ACTIONS(1769),
    [sym_devchan_pulse_keyword] = ACTIONS(1769),
    [sym_wait_keyword] = ACTIONS(1769),
    [sym_cancel_wait_keyword] = ACTIONS(1769),
    [sym_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1769),
    [sym_not] = ACTIONS(1769),
    [sym_bnot] = ACTIONS(1769),
    [sym___file__] = ACTIONS(1769),
    [sym___date__] = ACTIONS(1769),
    [sym___ldate__] = ACTIONS(1769),
    [sym___time__] = ACTIONS(1769),
    [sym___line__] = ACTIONS(1769),
    [sym___name__] = ACTIONS(1769),
    [sym___netlinx__] = ACTIONS(1769),
    [sym_netlinx_axi_version] = ACTIONS(1769),
    [sym_url_flg_tcp] = ACTIONS(1769),
    [sym_url_flg_accinfopresent] = ACTIONS(1769),
    [sym_url_flg_temp] = ACTIONS(1769),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1769),
    [sym_url_flg_stat_mask] = ACTIONS(1769),
    [sym_url_flg_stat_lookup] = ACTIONS(1769),
    [sym_url_flg_stat_connecting] = ACTIONS(1769),
    [sym_url_flg_stat_waiting] = ACTIONS(1769),
    [sym_url_flg_stat_connected] = ACTIONS(1769),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1769),
    [sym_first_virtual_device] = ACTIONS(1769),
    [sym_first_local_port] = ACTIONS(1769),
    [sym_dynamic_virtual_device] = ACTIONS(1769),
    [sym_timeline_once] = ACTIONS(1769),
    [sym_timeline_repeat] = ACTIONS(1769),
    [sym_timeline_absolute] = ACTIONS(1769),
    [sym_timeline_relative] = ACTIONS(1769),
    [sym_do_push_timed_infinite] = ACTIONS(1769),
    [sym_source_type_no_address] = ACTIONS(1769),
    [sym_source_type_neuron_id] = ACTIONS(1769),
    [sym_source_type_ip_address] = ACTIONS(1769),
    [sym_source_type_axlink] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1769),
    [sym_source_type_ip_socket_address] = ACTIONS(1769),
    [sym_source_type_rs232] = ACTIONS(1769),
    [sym_source_type_internal] = ACTIONS(1769),
    [sym_source_type_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1769),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1769),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1769),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1769),
    [sym_true] = ACTIONS(1769),
    [sym_false] = ACTIONS(1769),
    [sym_file_read_only] = ACTIONS(1769),
    [sym_file_rw_new] = ACTIONS(1769),
    [sym_file_rw_append] = ACTIONS(1769),
    [sym_ip_tcp] = ACTIONS(1769),
    [sym_ip_udp] = ACTIONS(1769),
    [sym_ip_udp_2way] = ACTIONS(1769),
    [sym_xml_encode_types] = ACTIONS(1769),
    [sym_xml_encode_char_as_list] = ACTIONS(1769),
    [sym_xml_encode_le] = ACTIONS(1769),
    [sym_xml_decode_types] = ACTIONS(1769),
    [sym_xml_decode_no_preserve] = ACTIONS(1769),
    [sym_duet_dev_type_display_device] = ACTIONS(1769),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_io_device] = ACTIONS(1769),
    [sym_duet_dev_type_relay_device] = ACTIONS(1769),
    [sym_duet_dev_type_ups] = ACTIONS(1769),
    [sym_duet_dev_type_amplifier] = ACTIONS(1769),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1769),
    [sym_duet_dev_type_camera] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1769),
    [sym_duet_dev_type_dss] = ACTIONS(1769),
    [sym_duet_dev_type_dvr] = ACTIONS(1769),
    [sym_duet_dev_type_disc_device] = ACTIONS(1769),
    [sym_duet_dev_type_document_camera] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1769),
    [sym_duet_dev_type_hvac] = ACTIONS(1769),
    [sym_duet_dev_type_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_light] = ACTIONS(1769),
    [sym_duet_dev_type_monitor] = ACTIONS(1769),
    [sym_duet_dev_type_motor] = ACTIONS(1769),
    [sym_duet_dev_type_multi_window] = ACTIONS(1769),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1769),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1769),
    [sym_duet_dev_type_receiver] = ACTIONS(1769),
    [sym_duet_dev_type_security_system] = ACTIONS(1769),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1769),
    [sym_duet_dev_type_settop_box] = ACTIONS(1769),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1769),
    [sym_duet_dev_type_switcher] = ACTIONS(1769),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_tv] = ACTIONS(1769),
    [sym_duet_dev_type_utility] = ACTIONS(1769),
    [sym_duet_dev_type_vcr] = ACTIONS(1769),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_video_processor] = ACTIONS(1769),
    [sym_duet_dev_type_video_projector] = ACTIONS(1769),
    [sym_duet_dev_type_video_wall] = ACTIONS(1769),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1769),
    [sym_duet_dev_type_weather] = ACTIONS(1769),
    [sym_duet_dev_not_polled] = ACTIONS(1769),
    [sym_duet_dev_polled] = ACTIONS(1769),
    [sym_clkmgr_mode_network] = ACTIONS(1769),
    [sym_clkmgr_mode_standalone] = ACTIONS(1769),
    [sym_normal_standby] = ACTIONS(1769),
    [sym_normal_wake] = ACTIONS(1769),
    [sym_ip_multicast_ttl_option] = ACTIONS(1769),
    [sym_ip_ttl_subnet] = ACTIONS(1769),
    [sym_ip_ttl_site] = ACTIONS(1769),
    [sym_ip_ttl_region] = ACTIONS(1769),
    [sym_ip_ttl_continent] = ACTIONS(1769),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1769),
    [sym_ip_nodelay_on] = ACTIONS(1769),
    [sym_ip_nodelay_off] = ACTIONS(1769),
    [sym_amx_error] = ACTIONS(1769),
    [sym_amx_warning] = ACTIONS(1769),
    [sym_amx_info] = ACTIONS(1769),
    [sym_amx_debug] = ACTIONS(1769),
    [sym_netlinx_login_success] = ACTIONS(1769),
    [sym_netlinx_login_fail] = ACTIONS(1769),
    [sym_netlinx_logout] = ACTIONS(1769),
    [sym_null_str] = ACTIONS(1769),
    [sym_smtp_address] = ACTIONS(1769),
    [sym_smtp_port_number] = ACTIONS(1769),
    [sym_smtp_username] = ACTIONS(1769),
    [sym_smtp_password] = ACTIONS(1769),
    [sym_smtp_require_tls] = ACTIONS(1769),
    [sym_smtp_from] = ACTIONS(1769),
    [sym_smtp_tls_true] = ACTIONS(1769),
    [sym_smtp_tls_false] = ACTIONS(1769),
    [sym_listview_on_row_select_event] = ACTIONS(1769),
    [sym_data_structure_unknown] = ACTIONS(1769),
    [sym_data_structure_datafeed] = ACTIONS(1769),
    [sym_data_structure_datarecordset] = ACTIONS(1769),
    [sym_data_structure_datarecord] = ACTIONS(1769),
    [sym_data_structure_datafield] = ACTIONS(1769),
    [sym_data_max_content_field_count] = ACTIONS(1769),
    [sym_data_max_metadata_field_count] = ACTIONS(1769),
    [sym_data_max_id_length] = ACTIONS(1769),
    [sym_data_max_name_length] = ACTIONS(1769),
    [sym_data_max_description_length] = ACTIONS(1769),
    [sym_data_max_type_length] = ACTIONS(1769),
    [sym_data_max_format_length] = ACTIONS(1769),
    [sym_data_max_source_length] = ACTIONS(1769),
    [sym_data_max_value_length] = ACTIONS(1769),
    [sym_data_max_label_length] = ACTIONS(1769),
    [sym_data_max_url_length] = ACTIONS(1769),
    [sym_max_last_login_info_length] = ACTIONS(1769),
    [sym_data_type_unknown] = ACTIONS(1769),
    [sym_data_type_string] = ACTIONS(1769),
    [sym_data_type_datetime] = ACTIONS(1769),
    [sym_data_type_date] = ACTIONS(1769),
    [sym_data_type_time] = ACTIONS(1769),
    [sym_data_type_image] = ACTIONS(1769),
    [sym_data_format_url] = ACTIONS(1769),
    [sym_data_format_phone] = ACTIONS(1769),
    [sym_data_format_email] = ACTIONS(1769),
    [sym_data_format_iso8601] = ACTIONS(1769),
    [sym_auditlog_priv] = ACTIONS(1769),
    [sym_configuration_priv] = ACTIONS(1769),
    [sym_deviceconfig_priv] = ACTIONS(1769),
    [sym_ftp_priv] = ACTIONS(1769),
    [sym_http_priv] = ACTIONS(1769),
    [sym_networkconfig_priv] = ACTIONS(1769),
    [sym_programport_priv] = ACTIONS(1769),
    [sym_remoteui_priv] = ACTIONS(1769),
    [sym_securitycontrol_priv] = ACTIONS(1769),
    [sym_softwaremanagement_priv] = ACTIONS(1769),
    [sym_terminal_priv] = ACTIONS(1769),
    [sym_tpadmin_priv] = ACTIONS(1769),
    [sym_user1_priv] = ACTIONS(1769),
    [sym_user2_priv] = ACTIONS(1769),
    [sym_user3_priv] = ACTIONS(1769),
    [sym_user4_priv] = ACTIONS(1769),
    [sym_usermanagement_priv] = ACTIONS(1769),
    [sym_valid_account] = ACTIONS(1769),
    [sym_err_name_invalid] = ACTIONS(1769),
    [sym_err_password_invalid] = ACTIONS(1769),
    [sym_err_invalid_account] = ACTIONS(1769),
    [sym_err_no_authorization] = ACTIONS(1769),
    [sym_err_invalid_parameter] = ACTIONS(1769),
    [sym_err_user_locked_out] = ACTIONS(1769),
    [sym_err_user_account_expired] = ACTIONS(1769),
    [sym_err_general_error] = ACTIONS(1769),
    [sym_err_authentication_error] = ACTIONS(1769),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1769),
    [sym_tls_validate_certificate] = ACTIONS(1769),
    [sym_tdata] = ACTIONS(1769),
    [sym_tchannel] = ACTIONS(1769),
    [sym_tlevel] = ACTIONS(1769),
    [sym_tbutton] = ACTIONS(1769),
    [sym_ttimeline] = ACTIONS(1769),
    [sym_tcustom] = ACTIONS(1769),
    [sym_url_struct] = ACTIONS(1769),
    [sym_dns_struct] = ACTIONS(1769),
    [sym_ip_address_struct] = ACTIONS(1769),
    [sym_dev_info_struct] = ACTIONS(1769),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1769),
    [sym_data_feed] = ACTIONS(1769),
    [sym_data_field] = ACTIONS(1769),
    [sym_data_record] = ACTIONS(1769),
    [sym_wc_data_feed] = ACTIONS(1769),
    [sym_wc_data_field] = ACTIONS(1769),
    [sym_wc_data_record] = ACTIONS(1769),
    [sym_last_login_info] = ACTIONS(1769),
    [sym____reserved___] = ACTIONS(1769),
    [sym_date] = ACTIONS(1769),
    [sym_day] = ACTIONS(1769),
    [sym_time] = ACTIONS(1769),
    [sym_ldate] = ACTIONS(1769),
    [sym_push_device] = ACTIONS(1769),
    [sym_push_channel] = ACTIONS(1769),
    [sym_push_devchan] = ACTIONS(1769),
    [sym_release_device] = ACTIONS(1769),
    [sym_release_channel] = ACTIONS(1769),
    [sym_release_devchan] = ACTIONS(1769),
    [sym_master_slot] = ACTIONS(1769),
    [sym_get_pulse_time] = ACTIONS(1769),
    [sym_get_timer] = ACTIONS(1769),
    [sym_master_sn] = ACTIONS(1769),
    [sym_system_number] = ACTIONS(1769),
    [sym_button] = ACTIONS(1769),
    [sym_dv_channel] = ACTIONS(1769),
    [sym_data] = ACTIONS(1769),
    [sym_level] = ACTIONS(1769),
    [sym_timeline] = ACTIONS(1769),
    [sym_channel] = ACTIONS(1769),
    [sym_custom] = ACTIONS(1769),
    [sym_atoi] = ACTIONS(1769),
    [sym_atol] = ACTIONS(1769),
    [sym_atof] = ACTIONS(1769),
    [sym_itoa] = ACTIONS(1769),
    [sym_itohex] = ACTIONS(1769),
    [sym_hextoi] = ACTIONS(1769),
    [sym_ftoa] = ACTIONS(1769),
    [sym_find_string] = ACTIONS(1769),
    [sym_left_string] = ACTIONS(1769),
    [sym_right_string] = ACTIONS(1769),
    [sym_mid_string] = ACTIONS(1769),
    [sym_remove_string] = ACTIONS(1769),
    [sym_get_buffer_string] = ACTIONS(1769),
    [sym_get_buffer_char] = ACTIONS(1769),
    [sym_get_multi_buffer_string] = ACTIONS(1769),
    [sym_length_string] = ACTIONS(1769),
    [sym_max_length_string] = ACTIONS(1769),
    [sym_set_length_string] = ACTIONS(1769),
    [sym_compare_string] = ACTIONS(1769),
    [sym_upper_string] = ACTIONS(1769),
    [sym_lower_string] = ACTIONS(1769),
    [sym_get_last] = ACTIONS(1769),
    [sym_date_to_day] = ACTIONS(1769),
    [sym_date_to_month] = ACTIONS(1769),
    [sym_date_to_year] = ACTIONS(1769),
    [sym_time_to_hour] = ACTIONS(1769),
    [sym_time_to_minute] = ACTIONS(1769),
    [sym_time_to_second] = ACTIONS(1769),
    [sym_day_of_week] = ACTIONS(1769),
    [sym_set_pulse_time] = ACTIONS(1769),
    [sym_set_timer] = ACTIONS(1769),
    [sym_file_open] = ACTIONS(1769),
    [sym_file_close] = ACTIONS(1769),
    [sym_file_read] = ACTIONS(1769),
    [sym_file_read_line] = ACTIONS(1769),
    [sym_file_write] = ACTIONS(1769),
    [sym_file_write_line] = ACTIONS(1769),
    [sym_file_copy] = ACTIONS(1769),
    [sym_file_delete] = ACTIONS(1769),
    [sym_file_rename] = ACTIONS(1769),
    [sym_file_dir] = ACTIONS(1769),
    [sym_file_getdir] = ACTIONS(1769),
    [sym_file_setdir] = ACTIONS(1769),
    [sym_file_create_dir] = ACTIONS(1769),
    [sym_file_remove_dir] = ACTIONS(1769),
    [sym_file_seek] = ACTIONS(1769),
    [sym_length_array] = ACTIONS(1769),
    [sym_max_length_array] = ACTIONS(1769),
    [sym_set_length_array] = ACTIONS(1769),
    [sym_ip_client_open] = ACTIONS(1769),
    [sym_ip_client_close] = ACTIONS(1769),
    [sym_ip_server_open] = ACTIONS(1769),
    [sym_ip_server_close] = ACTIONS(1769),
    [sym_ip_bound_client_open] = ACTIONS(1769),
    [sym_ip_mc_server_open] = ACTIONS(1769),
    [sym_ip_set_option] = ACTIONS(1769),
    [sym_ssh_client_open] = ACTIONS(1769),
    [sym_ssh_client_close] = ACTIONS(1769),
    [sym_tls_client_open] = ACTIONS(1769),
    [sym_tls_client_close] = ACTIONS(1769),
    [sym_do_push] = ACTIONS(1769),
    [sym_do_push_timed] = ACTIONS(1769),
    [sym_do_release] = ACTIONS(1769),
    [sym_device_id] = ACTIONS(1769),
    [sym_device_info] = ACTIONS(1769),
    [sym_device_id_string] = ACTIONS(1769),
    [sym_device_standby] = ACTIONS(1769),
    [sym_device_wake] = ACTIONS(1769),
    [sym_timeline_create] = ACTIONS(1769),
    [sym_timeline_kill] = ACTIONS(1769),
    [sym_timeline_reload] = ACTIONS(1769),
    [sym_timeline_pause] = ACTIONS(1769),
    [sym_timeline_restart] = ACTIONS(1769),
    [sym_timeline_set] = ACTIONS(1769),
    [sym_timeline_get] = ACTIONS(1769),
    [sym_timeline_active] = ACTIONS(1769),
    [sym_timeline_dynamic_id] = ACTIONS(1769),
    [sym_variable_to_string] = ACTIONS(1769),
    [sym_string_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_string] = ACTIONS(1769),
    [sym_type_cast] = ACTIONS(1769),
    [sym_variable_to_xml] = ACTIONS(1769),
    [sym_xml_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_xml] = ACTIONS(1769),
    [sym_redirect_string] = ACTIONS(1769),
    [sym_random_number] = ACTIONS(1769),
    [sym_get_unique_id] = ACTIONS(1769),
    [sym_get_system_number] = ACTIONS(1769),
    [sym_set_system_number] = ACTIONS(1769),
    [sym_get_serial_number] = ACTIONS(1769),
    [sym_get_master_build] = ACTIONS(1769),
    [sym_set_validation_code] = ACTIONS(1769),
    [sym_reboot] = ACTIONS(1769),
    [sym_load_duet_module] = ACTIONS(1769),
    [sym_unload_duet_module] = ACTIONS(1769),
    [sym_duet_mem_size_set] = ACTIONS(1769),
    [sym_duet_mem_size_get] = ACTIONS(1769),
    [sym_dynamic_application_device] = ACTIONS(1769),
    [sym_dynamic_polled_port] = ACTIONS(1769),
    [sym_static_port_binding] = ACTIONS(1769),
    [sym_static_ip_binding] = ACTIONS(1769),
    [sym_get_dns_list] = ACTIONS(1769),
    [sym_set_dns_list] = ACTIONS(1769),
    [sym_get_ip_address] = ACTIONS(1769),
    [sym_set_ip_address] = ACTIONS(1769),
    [sym_get_url_list] = ACTIONS(1769),
    [sym_add_url_entry] = ACTIONS(1769),
    [sym_delete_url_entry] = ACTIONS(1769),
    [sym_abs_value] = ACTIONS(1769),
    [sym_max_value] = ACTIONS(1769),
    [sym_min_value] = ACTIONS(1769),
    [sym_exp_value] = ACTIONS(1769),
    [sym_log_value] = ACTIONS(1769),
    [sym_log10_value] = ACTIONS(1769),
    [sym_power_value] = ACTIONS(1769),
    [sym_sqrt_value] = ACTIONS(1769),
    [sym_raw_be] = ACTIONS(1769),
    [sym_raw_le] = ACTIONS(1769),
    [sym_internal_queue_size_set] = ACTIONS(1769),
    [sym_internal_queue_size_get] = ACTIONS(1769),
    [sym_internal_threshold_set] = ACTIONS(1769),
    [sym_internal_threshold_get] = ACTIONS(1769),
    [sym_rebuilt_event] = ACTIONS(1769),
    [sym_do_custom_event] = ACTIONS(1769),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1769),
    [sym_clkmgr_set_clk_source] = ACTIONS(1769),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1769),
    [sym_clkmgr_get_timezone] = ACTIONS(1769),
    [sym_clkmgr_set_timezone] = ACTIONS(1769),
    [sym_clkmgr_get_resync_period] = ACTIONS(1769),
    [sym_clkmgr_set_resync_period] = ACTIONS(1769),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_timeservers] = ACTIONS(1769),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_set_log_level] = ACTIONS(1769),
    [sym_get_log_level] = ACTIONS(1769),
    [sym_amx_log] = ACTIONS(1769),
    [sym_smtp_server_config_set] = ACTIONS(1769),
    [sym_smtp_server_config_get] = ACTIONS(1769),
    [sym_smtp_send] = ACTIONS(1769),
    [sym_validate_netlinx_account] = ACTIONS(1769),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1769),
    [sym_authenticate_certificate] = ACTIONS(1769),
    [sym_audit_netlinx_session_event] = ACTIONS(1769),
    [sym_audit_netlinx_generic_event] = ACTIONS(1769),
    [sym_get_available_flash_disk_space] = ACTIONS(1769),
    [sym_get_max_flash_disk_space] = ACTIONS(1769),
    [sym_data_create_feed] = ACTIONS(1769),
    [sym_data_delete_feed] = ACTIONS(1769),
    [sym_data_publish_feed] = ACTIONS(1769),
    [sym_data_get_published_feed] = ACTIONS(1769),
    [sym_data_add_record] = ACTIONS(1769),
    [sym_data_get_event_record] = ACTIONS(1769),
    [sym__wc_data_create_feed] = ACTIONS(1769),
    [sym__wc_data_add_record] = ACTIONS(1769),
    [sym__wc_data_get_event_record] = ACTIONS(1769),
    [sym_format] = ACTIONS(1769),
    [sym_astro_clock] = ACTIONS(1769),
    [sym_set_outdoor_temperature] = ACTIONS(1769),
    [sym_set_virtual_level_count] = ACTIONS(1769),
    [sym_set_virtual_channel_count] = ACTIONS(1769),
    [sym_set_virtual_port_count] = ACTIONS(1769),
    [sym__wc_encode_int] = ACTIONS(1769),
    [sym__wc_decode_int] = ACTIONS(1769),
    [sym__wc_int] = ACTIONS(1769),
    [sym__wc_to_ch_int] = ACTIONS(1769),
    [sym__ch_to_wc_int] = ACTIONS(1769),
    [sym__wc_find_string_int] = ACTIONS(1769),
    [sym__wc_left_string_int] = ACTIONS(1769),
    [sym__wc_lower_string_int] = ACTIONS(1769),
    [sym__wc_mid_string_int] = ACTIONS(1769),
    [sym__wc_remove_string_int] = ACTIONS(1769),
    [sym__wc_right_string_int] = ACTIONS(1769),
    [sym__wc_upper_string_int] = ACTIONS(1769),
    [sym__wc_compare_string_int] = ACTIONS(1769),
    [sym__wc_get_buffer_char_int] = ACTIONS(1769),
    [sym__wc_get_buffer_string_int] = ACTIONS(1769),
    [sym__wc_concat_string_int] = ACTIONS(1769),
    [sym__wc_file_open_int] = ACTIONS(1769),
    [sym__wc_file_close_int] = ACTIONS(1769),
    [sym__wc_file_read_int] = ACTIONS(1769),
    [sym__wc_file_read_line_int] = ACTIONS(1769),
    [sym__wc_file_write_int] = ACTIONS(1769),
    [sym__wc_file_write_line_int] = ACTIONS(1769),
    [aux_sym_preproc_include_token1] = ACTIONS(1769),
    [aux_sym_preproc_define_token1] = ACTIONS(1771),
    [aux_sym_preproc_warn_token1] = ACTIONS(1771),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1771),
    [anon_sym_LBRACK] = ACTIONS(1771),
    [anon_sym_LBRACE] = ACTIONS(1771),
    [anon_sym_LPAREN] = ACTIONS(1769),
    [anon_sym_BANG] = ACTIONS(1771),
    [anon_sym_TILDE] = ACTIONS(1771),
    [anon_sym_DASH] = ACTIONS(1769),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_DASH_DASH] = ACTIONS(1771),
    [anon_sym_PLUS_PLUS] = ACTIONS(1771),
    [anon_sym_DQUOTE] = ACTIONS(1771),
    [anon_sym_SQUOTE] = ACTIONS(1771),
    [sym_number_literal] = ACTIONS(1771),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(388)] = {
    [ts_builtin_sym_end] = ACTIONS(1787),
    [sym_identifier] = ACTIONS(1785),
    [sym_program_name_keyword] = ACTIONS(1785),
    [sym_module_name_keyword] = ACTIONS(1785),
    [sym_define_device_keyword] = ACTIONS(1785),
    [sym_define_combine_keyword] = ACTIONS(1785),
    [sym_define_constant_keyword] = ACTIONS(1785),
    [sym_define_type_keyword] = ACTIONS(1785),
    [sym_define_variable_keyword] = ACTIONS(1785),
    [sym_define_system_variable_keyword] = ACTIONS(1785),
    [sym_define_connect_level_keyword] = ACTIONS(1785),
    [sym_define_latching_keyword] = ACTIONS(1785),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1785),
    [sym_define_toggling_keyword] = ACTIONS(1785),
    [sym_define_call_keyword] = ACTIONS(1785),
    [sym_define_function_keyword] = ACTIONS(1785),
    [sym_define_library_function_keyword] = ACTIONS(1785),
    [sym_define_start_keyword] = ACTIONS(1785),
    [sym_define_event_keyword] = ACTIONS(1785),
    [sym_define_program_keyword] = ACTIONS(1785),
    [sym_define_module_keyword] = ACTIONS(1785),
    [sym_char_keyword] = ACTIONS(1785),
    [sym_widechar_keyword] = ACTIONS(1785),
    [sym_integer_keyword] = ACTIONS(1785),
    [sym_sinteger_keyword] = ACTIONS(1785),
    [sym_long_keyword] = ACTIONS(1785),
    [sym_slong_keyword] = ACTIONS(1785),
    [sym_float_keyword] = ACTIONS(1785),
    [sym_double_keyword] = ACTIONS(1785),
    [sym_variant_keyword] = ACTIONS(1785),
    [sym_variantarray_keyword] = ACTIONS(1785),
    [sym_local_var_keyword] = ACTIONS(1785),
    [sym_stack_var_keyword] = ACTIONS(1785),
    [sym_constant_keyword] = ACTIONS(1785),
    [sym_volatile_keyword] = ACTIONS(1785),
    [sym_non_volatile_keyword] = ACTIONS(1785),
    [sym_persistent_keyword] = ACTIONS(1785),
    [sym_dev_keyword] = ACTIONS(1785),
    [sym_devlev_keyword] = ACTIONS(1785),
    [sym_devchan_keyword] = ACTIONS(1785),
    [sym_if_keyword] = ACTIONS(1785),
    [sym_else_keyword] = ACTIONS(1785),
    [sym_switch_keyword] = ACTIONS(1785),
    [sym_case_keyword] = ACTIONS(1785),
    [sym_default_keyword] = ACTIONS(1785),
    [sym_while_keyword] = ACTIONS(1785),
    [sym_for_keyword] = ACTIONS(1785),
    [sym_break_keyword] = ACTIONS(1785),
    [sym_continue_keyword] = ACTIONS(1785),
    [sym_return_keyword] = ACTIONS(1785),
    [sym_select_keyword] = ACTIONS(1785),
    [sym_active_keyword] = ACTIONS(1785),
    [sym_struct_keyword] = ACTIONS(1785),
    [sym_structure_keyword] = ACTIONS(1785),
    [sym_button_event_keyword] = ACTIONS(1785),
    [sym_channel_event_keyword] = ACTIONS(1785),
    [sym_level_event_keyword] = ACTIONS(1785),
    [sym_data_event_keyword] = ACTIONS(1785),
    [sym_timeline_event_keyword] = ACTIONS(1785),
    [sym_custom_event_keyword] = ACTIONS(1785),
    [sym_push_keyword] = ACTIONS(1785),
    [sym_release_keyword] = ACTIONS(1785),
    [aux_sym_on_keyword_token1] = ACTIONS(1785),
    [aux_sym_off_keyword_token1] = ACTIONS(1785),
    [sym_send_level_keyword] = ACTIONS(1785),
    [sym_send_string_keyword] = ACTIONS(1785),
    [sym_send_command_keyword] = ACTIONS(1785),
    [sym_clear_buffer_keyword] = ACTIONS(1785),
    [sym_create_buffer_keyword] = ACTIONS(1785),
    [sym_create_multi_buffer_keyword] = ACTIONS(1785),
    [sym_call_keyword] = ACTIONS(1785),
    [sym_system_call_keyword] = ACTIONS(1785),
    [sym_devchan_to_keyword] = ACTIONS(1785),
    [sym_devchan_min_to_keyword] = ACTIONS(1785),
    [sym_devchan_total_off_keyword] = ACTIONS(1785),
    [sym_devchan_pulse_keyword] = ACTIONS(1785),
    [sym_wait_keyword] = ACTIONS(1785),
    [sym_cancel_wait_keyword] = ACTIONS(1785),
    [sym_wait_until_keyword] = ACTIONS(1785),
    [sym_cancel_wait_until_keyword] = ACTIONS(1785),
    [sym_cancel_all_wait_keyword] = ACTIONS(1785),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1785),
    [sym_not] = ACTIONS(1785),
    [sym_bnot] = ACTIONS(1785),
    [sym___file__] = ACTIONS(1785),
    [sym___date__] = ACTIONS(1785),
    [sym___ldate__] = ACTIONS(1785),
    [sym___time__] = ACTIONS(1785),
    [sym___line__] = ACTIONS(1785),
    [sym___name__] = ACTIONS(1785),
    [sym___netlinx__] = ACTIONS(1785),
    [sym_netlinx_axi_version] = ACTIONS(1785),
    [sym_url_flg_tcp] = ACTIONS(1785),
    [sym_url_flg_accinfopresent] = ACTIONS(1785),
    [sym_url_flg_temp] = ACTIONS(1785),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1785),
    [sym_url_flg_stat_mask] = ACTIONS(1785),
    [sym_url_flg_stat_lookup] = ACTIONS(1785),
    [sym_url_flg_stat_connecting] = ACTIONS(1785),
    [sym_url_flg_stat_waiting] = ACTIONS(1785),
    [sym_url_flg_stat_connected] = ACTIONS(1785),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1785),
    [sym_first_virtual_device] = ACTIONS(1785),
    [sym_first_local_port] = ACTIONS(1785),
    [sym_dynamic_virtual_device] = ACTIONS(1785),
    [sym_timeline_once] = ACTIONS(1785),
    [sym_timeline_repeat] = ACTIONS(1785),
    [sym_timeline_absolute] = ACTIONS(1785),
    [sym_timeline_relative] = ACTIONS(1785),
    [sym_do_push_timed_infinite] = ACTIONS(1785),
    [sym_source_type_no_address] = ACTIONS(1785),
    [sym_source_type_neuron_id] = ACTIONS(1785),
    [sym_source_type_ip_address] = ACTIONS(1785),
    [sym_source_type_axlink] = ACTIONS(1785),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1785),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1785),
    [sym_source_type_ip_socket_address] = ACTIONS(1785),
    [sym_source_type_rs232] = ACTIONS(1785),
    [sym_source_type_internal] = ACTIONS(1785),
    [sym_source_type_mac_address] = ACTIONS(1785),
    [sym_source_type_ipv4_port] = ACTIONS(1785),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1785),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1785),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1785),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1785),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1785),
    [sym_true] = ACTIONS(1785),
    [sym_false] = ACTIONS(1785),
    [sym_file_read_only] = ACTIONS(1785),
    [sym_file_rw_new] = ACTIONS(1785),
    [sym_file_rw_append] = ACTIONS(1785),
    [sym_ip_tcp] = ACTIONS(1785),
    [sym_ip_udp] = ACTIONS(1785),
    [sym_ip_udp_2way] = ACTIONS(1785),
    [sym_xml_encode_types] = ACTIONS(1785),
    [sym_xml_encode_char_as_list] = ACTIONS(1785),
    [sym_xml_encode_le] = ACTIONS(1785),
    [sym_xml_decode_types] = ACTIONS(1785),
    [sym_xml_decode_no_preserve] = ACTIONS(1785),
    [sym_duet_dev_type_display_device] = ACTIONS(1785),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1785),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1785),
    [sym_duet_dev_type_io_device] = ACTIONS(1785),
    [sym_duet_dev_type_relay_device] = ACTIONS(1785),
    [sym_duet_dev_type_ups] = ACTIONS(1785),
    [sym_duet_dev_type_amplifier] = ACTIONS(1785),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1785),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1785),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1785),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1785),
    [sym_duet_dev_type_camera] = ACTIONS(1785),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1785),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1785),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1785),
    [sym_duet_dev_type_dss] = ACTIONS(1785),
    [sym_duet_dev_type_dvr] = ACTIONS(1785),
    [sym_duet_dev_type_disc_device] = ACTIONS(1785),
    [sym_duet_dev_type_document_camera] = ACTIONS(1785),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1785),
    [sym_duet_dev_type_hvac] = ACTIONS(1785),
    [sym_duet_dev_type_keypad] = ACTIONS(1785),
    [sym_duet_dev_type_light] = ACTIONS(1785),
    [sym_duet_dev_type_monitor] = ACTIONS(1785),
    [sym_duet_dev_type_motor] = ACTIONS(1785),
    [sym_duet_dev_type_multi_window] = ACTIONS(1785),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1785),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1785),
    [sym_duet_dev_type_receiver] = ACTIONS(1785),
    [sym_duet_dev_type_security_system] = ACTIONS(1785),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1785),
    [sym_duet_dev_type_settop_box] = ACTIONS(1785),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1785),
    [sym_duet_dev_type_switcher] = ACTIONS(1785),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1785),
    [sym_duet_dev_type_tv] = ACTIONS(1785),
    [sym_duet_dev_type_utility] = ACTIONS(1785),
    [sym_duet_dev_type_vcr] = ACTIONS(1785),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1785),
    [sym_duet_dev_type_video_processor] = ACTIONS(1785),
    [sym_duet_dev_type_video_projector] = ACTIONS(1785),
    [sym_duet_dev_type_video_wall] = ACTIONS(1785),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1785),
    [sym_duet_dev_type_weather] = ACTIONS(1785),
    [sym_duet_dev_not_polled] = ACTIONS(1785),
    [sym_duet_dev_polled] = ACTIONS(1785),
    [sym_clkmgr_mode_network] = ACTIONS(1785),
    [sym_clkmgr_mode_standalone] = ACTIONS(1785),
    [sym_normal_standby] = ACTIONS(1785),
    [sym_normal_wake] = ACTIONS(1785),
    [sym_ip_multicast_ttl_option] = ACTIONS(1785),
    [sym_ip_ttl_subnet] = ACTIONS(1785),
    [sym_ip_ttl_site] = ACTIONS(1785),
    [sym_ip_ttl_region] = ACTIONS(1785),
    [sym_ip_ttl_continent] = ACTIONS(1785),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1785),
    [sym_ip_nodelay_on] = ACTIONS(1785),
    [sym_ip_nodelay_off] = ACTIONS(1785),
    [sym_amx_error] = ACTIONS(1785),
    [sym_amx_warning] = ACTIONS(1785),
    [sym_amx_info] = ACTIONS(1785),
    [sym_amx_debug] = ACTIONS(1785),
    [sym_netlinx_login_success] = ACTIONS(1785),
    [sym_netlinx_login_fail] = ACTIONS(1785),
    [sym_netlinx_logout] = ACTIONS(1785),
    [sym_null_str] = ACTIONS(1785),
    [sym_smtp_address] = ACTIONS(1785),
    [sym_smtp_port_number] = ACTIONS(1785),
    [sym_smtp_username] = ACTIONS(1785),
    [sym_smtp_password] = ACTIONS(1785),
    [sym_smtp_require_tls] = ACTIONS(1785),
    [sym_smtp_from] = ACTIONS(1785),
    [sym_smtp_tls_true] = ACTIONS(1785),
    [sym_smtp_tls_false] = ACTIONS(1785),
    [sym_listview_on_row_select_event] = ACTIONS(1785),
    [sym_data_structure_unknown] = ACTIONS(1785),
    [sym_data_structure_datafeed] = ACTIONS(1785),
    [sym_data_structure_datarecordset] = ACTIONS(1785),
    [sym_data_structure_datarecord] = ACTIONS(1785),
    [sym_data_structure_datafield] = ACTIONS(1785),
    [sym_data_max_content_field_count] = ACTIONS(1785),
    [sym_data_max_metadata_field_count] = ACTIONS(1785),
    [sym_data_max_id_length] = ACTIONS(1785),
    [sym_data_max_name_length] = ACTIONS(1785),
    [sym_data_max_description_length] = ACTIONS(1785),
    [sym_data_max_type_length] = ACTIONS(1785),
    [sym_data_max_format_length] = ACTIONS(1785),
    [sym_data_max_source_length] = ACTIONS(1785),
    [sym_data_max_value_length] = ACTIONS(1785),
    [sym_data_max_label_length] = ACTIONS(1785),
    [sym_data_max_url_length] = ACTIONS(1785),
    [sym_max_last_login_info_length] = ACTIONS(1785),
    [sym_data_type_unknown] = ACTIONS(1785),
    [sym_data_type_string] = ACTIONS(1785),
    [sym_data_type_datetime] = ACTIONS(1785),
    [sym_data_type_date] = ACTIONS(1785),
    [sym_data_type_time] = ACTIONS(1785),
    [sym_data_type_image] = ACTIONS(1785),
    [sym_data_format_url] = ACTIONS(1785),
    [sym_data_format_phone] = ACTIONS(1785),
    [sym_data_format_email] = ACTIONS(1785),
    [sym_data_format_iso8601] = ACTIONS(1785),
    [sym_auditlog_priv] = ACTIONS(1785),
    [sym_configuration_priv] = ACTIONS(1785),
    [sym_deviceconfig_priv] = ACTIONS(1785),
    [sym_ftp_priv] = ACTIONS(1785),
    [sym_http_priv] = ACTIONS(1785),
    [sym_networkconfig_priv] = ACTIONS(1785),
    [sym_programport_priv] = ACTIONS(1785),
    [sym_remoteui_priv] = ACTIONS(1785),
    [sym_securitycontrol_priv] = ACTIONS(1785),
    [sym_softwaremanagement_priv] = ACTIONS(1785),
    [sym_terminal_priv] = ACTIONS(1785),
    [sym_tpadmin_priv] = ACTIONS(1785),
    [sym_user1_priv] = ACTIONS(1785),
    [sym_user2_priv] = ACTIONS(1785),
    [sym_user3_priv] = ACTIONS(1785),
    [sym_user4_priv] = ACTIONS(1785),
    [sym_usermanagement_priv] = ACTIONS(1785),
    [sym_valid_account] = ACTIONS(1785),
    [sym_err_name_invalid] = ACTIONS(1785),
    [sym_err_password_invalid] = ACTIONS(1785),
    [sym_err_invalid_account] = ACTIONS(1785),
    [sym_err_no_authorization] = ACTIONS(1785),
    [sym_err_invalid_parameter] = ACTIONS(1785),
    [sym_err_user_locked_out] = ACTIONS(1785),
    [sym_err_user_account_expired] = ACTIONS(1785),
    [sym_err_general_error] = ACTIONS(1785),
    [sym_err_authentication_error] = ACTIONS(1785),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1785),
    [sym_tls_validate_certificate] = ACTIONS(1785),
    [sym_tdata] = ACTIONS(1785),
    [sym_tchannel] = ACTIONS(1785),
    [sym_tlevel] = ACTIONS(1785),
    [sym_tbutton] = ACTIONS(1785),
    [sym_ttimeline] = ACTIONS(1785),
    [sym_tcustom] = ACTIONS(1785),
    [sym_url_struct] = ACTIONS(1785),
    [sym_dns_struct] = ACTIONS(1785),
    [sym_ip_address_struct] = ACTIONS(1785),
    [sym_dev_info_struct] = ACTIONS(1785),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1785),
    [sym_data_feed] = ACTIONS(1785),
    [sym_data_field] = ACTIONS(1785),
    [sym_data_record] = ACTIONS(1785),
    [sym_wc_data_feed] = ACTIONS(1785),
    [sym_wc_data_field] = ACTIONS(1785),
    [sym_wc_data_record] = ACTIONS(1785),
    [sym_last_login_info] = ACTIONS(1785),
    [sym____reserved___] = ACTIONS(1785),
    [sym_date] = ACTIONS(1785),
    [sym_day] = ACTIONS(1785),
    [sym_time] = ACTIONS(1785),
    [sym_ldate] = ACTIONS(1785),
    [sym_push_device] = ACTIONS(1785),
    [sym_push_channel] = ACTIONS(1785),
    [sym_push_devchan] = ACTIONS(1785),
    [sym_release_device] = ACTIONS(1785),
    [sym_release_channel] = ACTIONS(1785),
    [sym_release_devchan] = ACTIONS(1785),
    [sym_master_slot] = ACTIONS(1785),
    [sym_get_pulse_time] = ACTIONS(1785),
    [sym_get_timer] = ACTIONS(1785),
    [sym_master_sn] = ACTIONS(1785),
    [sym_system_number] = ACTIONS(1785),
    [sym_button] = ACTIONS(1785),
    [sym_dv_channel] = ACTIONS(1785),
    [sym_data] = ACTIONS(1785),
    [sym_level] = ACTIONS(1785),
    [sym_timeline] = ACTIONS(1785),
    [sym_channel] = ACTIONS(1785),
    [sym_custom] = ACTIONS(1785),
    [sym_atoi] = ACTIONS(1785),
    [sym_atol] = ACTIONS(1785),
    [sym_atof] = ACTIONS(1785),
    [sym_itoa] = ACTIONS(1785),
    [sym_itohex] = ACTIONS(1785),
    [sym_hextoi] = ACTIONS(1785),
    [sym_ftoa] = ACTIONS(1785),
    [sym_find_string] = ACTIONS(1785),
    [sym_left_string] = ACTIONS(1785),
    [sym_right_string] = ACTIONS(1785),
    [sym_mid_string] = ACTIONS(1785),
    [sym_remove_string] = ACTIONS(1785),
    [sym_get_buffer_string] = ACTIONS(1785),
    [sym_get_buffer_char] = ACTIONS(1785),
    [sym_get_multi_buffer_string] = ACTIONS(1785),
    [sym_length_string] = ACTIONS(1785),
    [sym_max_length_string] = ACTIONS(1785),
    [sym_set_length_string] = ACTIONS(1785),
    [sym_compare_string] = ACTIONS(1785),
    [sym_upper_string] = ACTIONS(1785),
    [sym_lower_string] = ACTIONS(1785),
    [sym_get_last] = ACTIONS(1785),
    [sym_date_to_day] = ACTIONS(1785),
    [sym_date_to_month] = ACTIONS(1785),
    [sym_date_to_year] = ACTIONS(1785),
    [sym_time_to_hour] = ACTIONS(1785),
    [sym_time_to_minute] = ACTIONS(1785),
    [sym_time_to_second] = ACTIONS(1785),
    [sym_day_of_week] = ACTIONS(1785),
    [sym_set_pulse_time] = ACTIONS(1785),
    [sym_set_timer] = ACTIONS(1785),
    [sym_file_open] = ACTIONS(1785),
    [sym_file_close] = ACTIONS(1785),
    [sym_file_read] = ACTIONS(1785),
    [sym_file_read_line] = ACTIONS(1785),
    [sym_file_write] = ACTIONS(1785),
    [sym_file_write_line] = ACTIONS(1785),
    [sym_file_copy] = ACTIONS(1785),
    [sym_file_delete] = ACTIONS(1785),
    [sym_file_rename] = ACTIONS(1785),
    [sym_file_dir] = ACTIONS(1785),
    [sym_file_getdir] = ACTIONS(1785),
    [sym_file_setdir] = ACTIONS(1785),
    [sym_file_create_dir] = ACTIONS(1785),
    [sym_file_remove_dir] = ACTIONS(1785),
    [sym_file_seek] = ACTIONS(1785),
    [sym_length_array] = ACTIONS(1785),
    [sym_max_length_array] = ACTIONS(1785),
    [sym_set_length_array] = ACTIONS(1785),
    [sym_ip_client_open] = ACTIONS(1785),
    [sym_ip_client_close] = ACTIONS(1785),
    [sym_ip_server_open] = ACTIONS(1785),
    [sym_ip_server_close] = ACTIONS(1785),
    [sym_ip_bound_client_open] = ACTIONS(1785),
    [sym_ip_mc_server_open] = ACTIONS(1785),
    [sym_ip_set_option] = ACTIONS(1785),
    [sym_ssh_client_open] = ACTIONS(1785),
    [sym_ssh_client_close] = ACTIONS(1785),
    [sym_tls_client_open] = ACTIONS(1785),
    [sym_tls_client_close] = ACTIONS(1785),
    [sym_do_push] = ACTIONS(1785),
    [sym_do_push_timed] = ACTIONS(1785),
    [sym_do_release] = ACTIONS(1785),
    [sym_device_id] = ACTIONS(1785),
    [sym_device_info] = ACTIONS(1785),
    [sym_device_id_string] = ACTIONS(1785),
    [sym_device_standby] = ACTIONS(1785),
    [sym_device_wake] = ACTIONS(1785),
    [sym_timeline_create] = ACTIONS(1785),
    [sym_timeline_kill] = ACTIONS(1785),
    [sym_timeline_reload] = ACTIONS(1785),
    [sym_timeline_pause] = ACTIONS(1785),
    [sym_timeline_restart] = ACTIONS(1785),
    [sym_timeline_set] = ACTIONS(1785),
    [sym_timeline_get] = ACTIONS(1785),
    [sym_timeline_active] = ACTIONS(1785),
    [sym_timeline_dynamic_id] = ACTIONS(1785),
    [sym_variable_to_string] = ACTIONS(1785),
    [sym_string_to_variable] = ACTIONS(1785),
    [sym_length_variable_to_string] = ACTIONS(1785),
    [sym_type_cast] = ACTIONS(1785),
    [sym_variable_to_xml] = ACTIONS(1785),
    [sym_xml_to_variable] = ACTIONS(1785),
    [sym_length_variable_to_xml] = ACTIONS(1785),
    [sym_redirect_string] = ACTIONS(1785),
    [sym_random_number] = ACTIONS(1785),
    [sym_get_unique_id] = ACTIONS(1785),
    [sym_get_system_number] = ACTIONS(1785),
    [sym_set_system_number] = ACTIONS(1785),
    [sym_get_serial_number] = ACTIONS(1785),
    [sym_get_master_build] = ACTIONS(1785),
    [sym_set_validation_code] = ACTIONS(1785),
    [sym_reboot] = ACTIONS(1785),
    [sym_load_duet_module] = ACTIONS(1785),
    [sym_unload_duet_module] = ACTIONS(1785),
    [sym_duet_mem_size_set] = ACTIONS(1785),
    [sym_duet_mem_size_get] = ACTIONS(1785),
    [sym_dynamic_application_device] = ACTIONS(1785),
    [sym_dynamic_polled_port] = ACTIONS(1785),
    [sym_static_port_binding] = ACTIONS(1785),
    [sym_static_ip_binding] = ACTIONS(1785),
    [sym_get_dns_list] = ACTIONS(1785),
    [sym_set_dns_list] = ACTIONS(1785),
    [sym_get_ip_address] = ACTIONS(1785),
    [sym_set_ip_address] = ACTIONS(1785),
    [sym_get_url_list] = ACTIONS(1785),
    [sym_add_url_entry] = ACTIONS(1785),
    [sym_delete_url_entry] = ACTIONS(1785),
    [sym_abs_value] = ACTIONS(1785),
    [sym_max_value] = ACTIONS(1785),
    [sym_min_value] = ACTIONS(1785),
    [sym_exp_value] = ACTIONS(1785),
    [sym_log_value] = ACTIONS(1785),
    [sym_log10_value] = ACTIONS(1785),
    [sym_power_value] = ACTIONS(1785),
    [sym_sqrt_value] = ACTIONS(1785),
    [sym_raw_be] = ACTIONS(1785),
    [sym_raw_le] = ACTIONS(1785),
    [sym_internal_queue_size_set] = ACTIONS(1785),
    [sym_internal_queue_size_get] = ACTIONS(1785),
    [sym_internal_threshold_set] = ACTIONS(1785),
    [sym_internal_threshold_get] = ACTIONS(1785),
    [sym_rebuilt_event] = ACTIONS(1785),
    [sym_do_custom_event] = ACTIONS(1785),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1785),
    [sym_clkmgr_set_clk_source] = ACTIONS(1785),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1785),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1785),
    [sym_clkmgr_get_timezone] = ACTIONS(1785),
    [sym_clkmgr_set_timezone] = ACTIONS(1785),
    [sym_clkmgr_get_resync_period] = ACTIONS(1785),
    [sym_clkmgr_set_resync_period] = ACTIONS(1785),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1785),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1785),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1785),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1785),
    [sym_clkmgr_get_timeservers] = ACTIONS(1785),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1785),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1785),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1785),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1785),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1785),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1785),
    [sym_set_log_level] = ACTIONS(1785),
    [sym_get_log_level] = ACTIONS(1785),
    [sym_amx_log] = ACTIONS(1785),
    [sym_smtp_server_config_set] = ACTIONS(1785),
    [sym_smtp_server_config_get] = ACTIONS(1785),
    [sym_smtp_send] = ACTIONS(1785),
    [sym_validate_netlinx_account] = ACTIONS(1785),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1785),
    [sym_authenticate_certificate] = ACTIONS(1785),
    [sym_audit_netlinx_session_event] = ACTIONS(1785),
    [sym_audit_netlinx_generic_event] = ACTIONS(1785),
    [sym_get_available_flash_disk_space] = ACTIONS(1785),
    [sym_get_max_flash_disk_space] = ACTIONS(1785),
    [sym_data_create_feed] = ACTIONS(1785),
    [sym_data_delete_feed] = ACTIONS(1785),
    [sym_data_publish_feed] = ACTIONS(1785),
    [sym_data_get_published_feed] = ACTIONS(1785),
    [sym_data_add_record] = ACTIONS(1785),
    [sym_data_get_event_record] = ACTIONS(1785),
    [sym__wc_data_create_feed] = ACTIONS(1785),
    [sym__wc_data_add_record] = ACTIONS(1785),
    [sym__wc_data_get_event_record] = ACTIONS(1785),
    [sym_format] = ACTIONS(1785),
    [sym_astro_clock] = ACTIONS(1785),
    [sym_set_outdoor_temperature] = ACTIONS(1785),
    [sym_set_virtual_level_count] = ACTIONS(1785),
    [sym_set_virtual_channel_count] = ACTIONS(1785),
    [sym_set_virtual_port_count] = ACTIONS(1785),
    [sym__wc_encode_int] = ACTIONS(1785),
    [sym__wc_decode_int] = ACTIONS(1785),
    [sym__wc_int] = ACTIONS(1785),
    [sym__wc_to_ch_int] = ACTIONS(1785),
    [sym__ch_to_wc_int] = ACTIONS(1785),
    [sym__wc_find_string_int] = ACTIONS(1785),
    [sym__wc_left_string_int] = ACTIONS(1785),
    [sym__wc_lower_string_int] = ACTIONS(1785),
    [sym__wc_mid_string_int] = ACTIONS(1785),
    [sym__wc_remove_string_int] = ACTIONS(1785),
    [sym__wc_right_string_int] = ACTIONS(1785),
    [sym__wc_upper_string_int] = ACTIONS(1785),
    [sym__wc_compare_string_int] = ACTIONS(1785),
    [sym__wc_get_buffer_char_int] = ACTIONS(1785),
    [sym__wc_get_buffer_string_int] = ACTIONS(1785),
    [sym__wc_concat_string_int] = ACTIONS(1785),
    [sym__wc_file_open_int] = ACTIONS(1785),
    [sym__wc_file_close_int] = ACTIONS(1785),
    [sym__wc_file_read_int] = ACTIONS(1785),
    [sym__wc_file_read_line_int] = ACTIONS(1785),
    [sym__wc_file_write_int] = ACTIONS(1785),
    [sym__wc_file_write_line_int] = ACTIONS(1785),
    [aux_sym_preproc_include_token1] = ACTIONS(1785),
    [aux_sym_preproc_define_token1] = ACTIONS(1787),
    [aux_sym_preproc_warn_token1] = ACTIONS(1787),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1787),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1787),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1787),
    [anon_sym_LBRACK] = ACTIONS(1787),
    [anon_sym_LBRACE] = ACTIONS(1787),
    [anon_sym_LPAREN] = ACTIONS(1785),
    [anon_sym_BANG] = ACTIONS(1787),
    [anon_sym_TILDE] = ACTIONS(1787),
    [anon_sym_DASH] = ACTIONS(1785),
    [anon_sym_PLUS] = ACTIONS(1785),
    [anon_sym_DASH_DASH] = ACTIONS(1787),
    [anon_sym_PLUS_PLUS] = ACTIONS(1787),
    [anon_sym_DQUOTE] = ACTIONS(1787),
    [anon_sym_SQUOTE] = ACTIONS(1787),
    [sym_number_literal] = ACTIONS(1787),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(389)] = {
    [ts_builtin_sym_end] = ACTIONS(1721),
    [sym_identifier] = ACTIONS(1719),
    [sym_program_name_keyword] = ACTIONS(1719),
    [sym_module_name_keyword] = ACTIONS(1719),
    [sym_define_device_keyword] = ACTIONS(1719),
    [sym_define_combine_keyword] = ACTIONS(1719),
    [sym_define_constant_keyword] = ACTIONS(1719),
    [sym_define_type_keyword] = ACTIONS(1719),
    [sym_define_variable_keyword] = ACTIONS(1719),
    [sym_define_system_variable_keyword] = ACTIONS(1719),
    [sym_define_connect_level_keyword] = ACTIONS(1719),
    [sym_define_latching_keyword] = ACTIONS(1719),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1719),
    [sym_define_toggling_keyword] = ACTIONS(1719),
    [sym_define_call_keyword] = ACTIONS(1719),
    [sym_define_function_keyword] = ACTIONS(1719),
    [sym_define_library_function_keyword] = ACTIONS(1719),
    [sym_define_start_keyword] = ACTIONS(1719),
    [sym_define_event_keyword] = ACTIONS(1719),
    [sym_define_program_keyword] = ACTIONS(1719),
    [sym_define_module_keyword] = ACTIONS(1719),
    [sym_char_keyword] = ACTIONS(1719),
    [sym_widechar_keyword] = ACTIONS(1719),
    [sym_integer_keyword] = ACTIONS(1719),
    [sym_sinteger_keyword] = ACTIONS(1719),
    [sym_long_keyword] = ACTIONS(1719),
    [sym_slong_keyword] = ACTIONS(1719),
    [sym_float_keyword] = ACTIONS(1719),
    [sym_double_keyword] = ACTIONS(1719),
    [sym_variant_keyword] = ACTIONS(1719),
    [sym_variantarray_keyword] = ACTIONS(1719),
    [sym_local_var_keyword] = ACTIONS(1719),
    [sym_stack_var_keyword] = ACTIONS(1719),
    [sym_constant_keyword] = ACTIONS(1719),
    [sym_volatile_keyword] = ACTIONS(1719),
    [sym_non_volatile_keyword] = ACTIONS(1719),
    [sym_persistent_keyword] = ACTIONS(1719),
    [sym_dev_keyword] = ACTIONS(1719),
    [sym_devlev_keyword] = ACTIONS(1719),
    [sym_devchan_keyword] = ACTIONS(1719),
    [sym_if_keyword] = ACTIONS(1719),
    [sym_else_keyword] = ACTIONS(1719),
    [sym_switch_keyword] = ACTIONS(1719),
    [sym_case_keyword] = ACTIONS(1719),
    [sym_default_keyword] = ACTIONS(1719),
    [sym_while_keyword] = ACTIONS(1719),
    [sym_for_keyword] = ACTIONS(1719),
    [sym_break_keyword] = ACTIONS(1719),
    [sym_continue_keyword] = ACTIONS(1719),
    [sym_return_keyword] = ACTIONS(1719),
    [sym_select_keyword] = ACTIONS(1719),
    [sym_active_keyword] = ACTIONS(1719),
    [sym_struct_keyword] = ACTIONS(1719),
    [sym_structure_keyword] = ACTIONS(1719),
    [sym_button_event_keyword] = ACTIONS(1719),
    [sym_channel_event_keyword] = ACTIONS(1719),
    [sym_level_event_keyword] = ACTIONS(1719),
    [sym_data_event_keyword] = ACTIONS(1719),
    [sym_timeline_event_keyword] = ACTIONS(1719),
    [sym_custom_event_keyword] = ACTIONS(1719),
    [sym_push_keyword] = ACTIONS(1719),
    [sym_release_keyword] = ACTIONS(1719),
    [aux_sym_on_keyword_token1] = ACTIONS(1719),
    [aux_sym_off_keyword_token1] = ACTIONS(1719),
    [sym_send_level_keyword] = ACTIONS(1719),
    [sym_send_string_keyword] = ACTIONS(1719),
    [sym_send_command_keyword] = ACTIONS(1719),
    [sym_clear_buffer_keyword] = ACTIONS(1719),
    [sym_create_buffer_keyword] = ACTIONS(1719),
    [sym_create_multi_buffer_keyword] = ACTIONS(1719),
    [sym_call_keyword] = ACTIONS(1719),
    [sym_system_call_keyword] = ACTIONS(1719),
    [sym_devchan_to_keyword] = ACTIONS(1719),
    [sym_devchan_min_to_keyword] = ACTIONS(1719),
    [sym_devchan_total_off_keyword] = ACTIONS(1719),
    [sym_devchan_pulse_keyword] = ACTIONS(1719),
    [sym_wait_keyword] = ACTIONS(1719),
    [sym_cancel_wait_keyword] = ACTIONS(1719),
    [sym_wait_until_keyword] = ACTIONS(1719),
    [sym_cancel_wait_until_keyword] = ACTIONS(1719),
    [sym_cancel_all_wait_keyword] = ACTIONS(1719),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1719),
    [sym_not] = ACTIONS(1719),
    [sym_bnot] = ACTIONS(1719),
    [sym___file__] = ACTIONS(1719),
    [sym___date__] = ACTIONS(1719),
    [sym___ldate__] = ACTIONS(1719),
    [sym___time__] = ACTIONS(1719),
    [sym___line__] = ACTIONS(1719),
    [sym___name__] = ACTIONS(1719),
    [sym___netlinx__] = ACTIONS(1719),
    [sym_netlinx_axi_version] = ACTIONS(1719),
    [sym_url_flg_tcp] = ACTIONS(1719),
    [sym_url_flg_accinfopresent] = ACTIONS(1719),
    [sym_url_flg_temp] = ACTIONS(1719),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1719),
    [sym_url_flg_stat_mask] = ACTIONS(1719),
    [sym_url_flg_stat_lookup] = ACTIONS(1719),
    [sym_url_flg_stat_connecting] = ACTIONS(1719),
    [sym_url_flg_stat_waiting] = ACTIONS(1719),
    [sym_url_flg_stat_connected] = ACTIONS(1719),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1719),
    [sym_first_virtual_device] = ACTIONS(1719),
    [sym_first_local_port] = ACTIONS(1719),
    [sym_dynamic_virtual_device] = ACTIONS(1719),
    [sym_timeline_once] = ACTIONS(1719),
    [sym_timeline_repeat] = ACTIONS(1719),
    [sym_timeline_absolute] = ACTIONS(1719),
    [sym_timeline_relative] = ACTIONS(1719),
    [sym_do_push_timed_infinite] = ACTIONS(1719),
    [sym_source_type_no_address] = ACTIONS(1719),
    [sym_source_type_neuron_id] = ACTIONS(1719),
    [sym_source_type_ip_address] = ACTIONS(1719),
    [sym_source_type_axlink] = ACTIONS(1719),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1719),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1719),
    [sym_source_type_ip_socket_address] = ACTIONS(1719),
    [sym_source_type_rs232] = ACTIONS(1719),
    [sym_source_type_internal] = ACTIONS(1719),
    [sym_source_type_mac_address] = ACTIONS(1719),
    [sym_source_type_ipv4_port] = ACTIONS(1719),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1719),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1719),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1719),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1719),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1719),
    [sym_true] = ACTIONS(1719),
    [sym_false] = ACTIONS(1719),
    [sym_file_read_only] = ACTIONS(1719),
    [sym_file_rw_new] = ACTIONS(1719),
    [sym_file_rw_append] = ACTIONS(1719),
    [sym_ip_tcp] = ACTIONS(1719),
    [sym_ip_udp] = ACTIONS(1719),
    [sym_ip_udp_2way] = ACTIONS(1719),
    [sym_xml_encode_types] = ACTIONS(1719),
    [sym_xml_encode_char_as_list] = ACTIONS(1719),
    [sym_xml_encode_le] = ACTIONS(1719),
    [sym_xml_decode_types] = ACTIONS(1719),
    [sym_xml_decode_no_preserve] = ACTIONS(1719),
    [sym_duet_dev_type_display_device] = ACTIONS(1719),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1719),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1719),
    [sym_duet_dev_type_io_device] = ACTIONS(1719),
    [sym_duet_dev_type_relay_device] = ACTIONS(1719),
    [sym_duet_dev_type_ups] = ACTIONS(1719),
    [sym_duet_dev_type_amplifier] = ACTIONS(1719),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1719),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1719),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1719),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1719),
    [sym_duet_dev_type_camera] = ACTIONS(1719),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1719),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1719),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1719),
    [sym_duet_dev_type_dss] = ACTIONS(1719),
    [sym_duet_dev_type_dvr] = ACTIONS(1719),
    [sym_duet_dev_type_disc_device] = ACTIONS(1719),
    [sym_duet_dev_type_document_camera] = ACTIONS(1719),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1719),
    [sym_duet_dev_type_hvac] = ACTIONS(1719),
    [sym_duet_dev_type_keypad] = ACTIONS(1719),
    [sym_duet_dev_type_light] = ACTIONS(1719),
    [sym_duet_dev_type_monitor] = ACTIONS(1719),
    [sym_duet_dev_type_motor] = ACTIONS(1719),
    [sym_duet_dev_type_multi_window] = ACTIONS(1719),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1719),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1719),
    [sym_duet_dev_type_receiver] = ACTIONS(1719),
    [sym_duet_dev_type_security_system] = ACTIONS(1719),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1719),
    [sym_duet_dev_type_settop_box] = ACTIONS(1719),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1719),
    [sym_duet_dev_type_switcher] = ACTIONS(1719),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1719),
    [sym_duet_dev_type_tv] = ACTIONS(1719),
    [sym_duet_dev_type_utility] = ACTIONS(1719),
    [sym_duet_dev_type_vcr] = ACTIONS(1719),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1719),
    [sym_duet_dev_type_video_processor] = ACTIONS(1719),
    [sym_duet_dev_type_video_projector] = ACTIONS(1719),
    [sym_duet_dev_type_video_wall] = ACTIONS(1719),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1719),
    [sym_duet_dev_type_weather] = ACTIONS(1719),
    [sym_duet_dev_not_polled] = ACTIONS(1719),
    [sym_duet_dev_polled] = ACTIONS(1719),
    [sym_clkmgr_mode_network] = ACTIONS(1719),
    [sym_clkmgr_mode_standalone] = ACTIONS(1719),
    [sym_normal_standby] = ACTIONS(1719),
    [sym_normal_wake] = ACTIONS(1719),
    [sym_ip_multicast_ttl_option] = ACTIONS(1719),
    [sym_ip_ttl_subnet] = ACTIONS(1719),
    [sym_ip_ttl_site] = ACTIONS(1719),
    [sym_ip_ttl_region] = ACTIONS(1719),
    [sym_ip_ttl_continent] = ACTIONS(1719),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1719),
    [sym_ip_nodelay_on] = ACTIONS(1719),
    [sym_ip_nodelay_off] = ACTIONS(1719),
    [sym_amx_error] = ACTIONS(1719),
    [sym_amx_warning] = ACTIONS(1719),
    [sym_amx_info] = ACTIONS(1719),
    [sym_amx_debug] = ACTIONS(1719),
    [sym_netlinx_login_success] = ACTIONS(1719),
    [sym_netlinx_login_fail] = ACTIONS(1719),
    [sym_netlinx_logout] = ACTIONS(1719),
    [sym_null_str] = ACTIONS(1719),
    [sym_smtp_address] = ACTIONS(1719),
    [sym_smtp_port_number] = ACTIONS(1719),
    [sym_smtp_username] = ACTIONS(1719),
    [sym_smtp_password] = ACTIONS(1719),
    [sym_smtp_require_tls] = ACTIONS(1719),
    [sym_smtp_from] = ACTIONS(1719),
    [sym_smtp_tls_true] = ACTIONS(1719),
    [sym_smtp_tls_false] = ACTIONS(1719),
    [sym_listview_on_row_select_event] = ACTIONS(1719),
    [sym_data_structure_unknown] = ACTIONS(1719),
    [sym_data_structure_datafeed] = ACTIONS(1719),
    [sym_data_structure_datarecordset] = ACTIONS(1719),
    [sym_data_structure_datarecord] = ACTIONS(1719),
    [sym_data_structure_datafield] = ACTIONS(1719),
    [sym_data_max_content_field_count] = ACTIONS(1719),
    [sym_data_max_metadata_field_count] = ACTIONS(1719),
    [sym_data_max_id_length] = ACTIONS(1719),
    [sym_data_max_name_length] = ACTIONS(1719),
    [sym_data_max_description_length] = ACTIONS(1719),
    [sym_data_max_type_length] = ACTIONS(1719),
    [sym_data_max_format_length] = ACTIONS(1719),
    [sym_data_max_source_length] = ACTIONS(1719),
    [sym_data_max_value_length] = ACTIONS(1719),
    [sym_data_max_label_length] = ACTIONS(1719),
    [sym_data_max_url_length] = ACTIONS(1719),
    [sym_max_last_login_info_length] = ACTIONS(1719),
    [sym_data_type_unknown] = ACTIONS(1719),
    [sym_data_type_string] = ACTIONS(1719),
    [sym_data_type_datetime] = ACTIONS(1719),
    [sym_data_type_date] = ACTIONS(1719),
    [sym_data_type_time] = ACTIONS(1719),
    [sym_data_type_image] = ACTIONS(1719),
    [sym_data_format_url] = ACTIONS(1719),
    [sym_data_format_phone] = ACTIONS(1719),
    [sym_data_format_email] = ACTIONS(1719),
    [sym_data_format_iso8601] = ACTIONS(1719),
    [sym_auditlog_priv] = ACTIONS(1719),
    [sym_configuration_priv] = ACTIONS(1719),
    [sym_deviceconfig_priv] = ACTIONS(1719),
    [sym_ftp_priv] = ACTIONS(1719),
    [sym_http_priv] = ACTIONS(1719),
    [sym_networkconfig_priv] = ACTIONS(1719),
    [sym_programport_priv] = ACTIONS(1719),
    [sym_remoteui_priv] = ACTIONS(1719),
    [sym_securitycontrol_priv] = ACTIONS(1719),
    [sym_softwaremanagement_priv] = ACTIONS(1719),
    [sym_terminal_priv] = ACTIONS(1719),
    [sym_tpadmin_priv] = ACTIONS(1719),
    [sym_user1_priv] = ACTIONS(1719),
    [sym_user2_priv] = ACTIONS(1719),
    [sym_user3_priv] = ACTIONS(1719),
    [sym_user4_priv] = ACTIONS(1719),
    [sym_usermanagement_priv] = ACTIONS(1719),
    [sym_valid_account] = ACTIONS(1719),
    [sym_err_name_invalid] = ACTIONS(1719),
    [sym_err_password_invalid] = ACTIONS(1719),
    [sym_err_invalid_account] = ACTIONS(1719),
    [sym_err_no_authorization] = ACTIONS(1719),
    [sym_err_invalid_parameter] = ACTIONS(1719),
    [sym_err_user_locked_out] = ACTIONS(1719),
    [sym_err_user_account_expired] = ACTIONS(1719),
    [sym_err_general_error] = ACTIONS(1719),
    [sym_err_authentication_error] = ACTIONS(1719),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1719),
    [sym_tls_validate_certificate] = ACTIONS(1719),
    [sym_tdata] = ACTIONS(1719),
    [sym_tchannel] = ACTIONS(1719),
    [sym_tlevel] = ACTIONS(1719),
    [sym_tbutton] = ACTIONS(1719),
    [sym_ttimeline] = ACTIONS(1719),
    [sym_tcustom] = ACTIONS(1719),
    [sym_url_struct] = ACTIONS(1719),
    [sym_dns_struct] = ACTIONS(1719),
    [sym_ip_address_struct] = ACTIONS(1719),
    [sym_dev_info_struct] = ACTIONS(1719),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1719),
    [sym_data_feed] = ACTIONS(1719),
    [sym_data_field] = ACTIONS(1719),
    [sym_data_record] = ACTIONS(1719),
    [sym_wc_data_feed] = ACTIONS(1719),
    [sym_wc_data_field] = ACTIONS(1719),
    [sym_wc_data_record] = ACTIONS(1719),
    [sym_last_login_info] = ACTIONS(1719),
    [sym____reserved___] = ACTIONS(1719),
    [sym_date] = ACTIONS(1719),
    [sym_day] = ACTIONS(1719),
    [sym_time] = ACTIONS(1719),
    [sym_ldate] = ACTIONS(1719),
    [sym_push_device] = ACTIONS(1719),
    [sym_push_channel] = ACTIONS(1719),
    [sym_push_devchan] = ACTIONS(1719),
    [sym_release_device] = ACTIONS(1719),
    [sym_release_channel] = ACTIONS(1719),
    [sym_release_devchan] = ACTIONS(1719),
    [sym_master_slot] = ACTIONS(1719),
    [sym_get_pulse_time] = ACTIONS(1719),
    [sym_get_timer] = ACTIONS(1719),
    [sym_master_sn] = ACTIONS(1719),
    [sym_system_number] = ACTIONS(1719),
    [sym_button] = ACTIONS(1719),
    [sym_dv_channel] = ACTIONS(1719),
    [sym_data] = ACTIONS(1719),
    [sym_level] = ACTIONS(1719),
    [sym_timeline] = ACTIONS(1719),
    [sym_channel] = ACTIONS(1719),
    [sym_custom] = ACTIONS(1719),
    [sym_atoi] = ACTIONS(1719),
    [sym_atol] = ACTIONS(1719),
    [sym_atof] = ACTIONS(1719),
    [sym_itoa] = ACTIONS(1719),
    [sym_itohex] = ACTIONS(1719),
    [sym_hextoi] = ACTIONS(1719),
    [sym_ftoa] = ACTIONS(1719),
    [sym_find_string] = ACTIONS(1719),
    [sym_left_string] = ACTIONS(1719),
    [sym_right_string] = ACTIONS(1719),
    [sym_mid_string] = ACTIONS(1719),
    [sym_remove_string] = ACTIONS(1719),
    [sym_get_buffer_string] = ACTIONS(1719),
    [sym_get_buffer_char] = ACTIONS(1719),
    [sym_get_multi_buffer_string] = ACTIONS(1719),
    [sym_length_string] = ACTIONS(1719),
    [sym_max_length_string] = ACTIONS(1719),
    [sym_set_length_string] = ACTIONS(1719),
    [sym_compare_string] = ACTIONS(1719),
    [sym_upper_string] = ACTIONS(1719),
    [sym_lower_string] = ACTIONS(1719),
    [sym_get_last] = ACTIONS(1719),
    [sym_date_to_day] = ACTIONS(1719),
    [sym_date_to_month] = ACTIONS(1719),
    [sym_date_to_year] = ACTIONS(1719),
    [sym_time_to_hour] = ACTIONS(1719),
    [sym_time_to_minute] = ACTIONS(1719),
    [sym_time_to_second] = ACTIONS(1719),
    [sym_day_of_week] = ACTIONS(1719),
    [sym_set_pulse_time] = ACTIONS(1719),
    [sym_set_timer] = ACTIONS(1719),
    [sym_file_open] = ACTIONS(1719),
    [sym_file_close] = ACTIONS(1719),
    [sym_file_read] = ACTIONS(1719),
    [sym_file_read_line] = ACTIONS(1719),
    [sym_file_write] = ACTIONS(1719),
    [sym_file_write_line] = ACTIONS(1719),
    [sym_file_copy] = ACTIONS(1719),
    [sym_file_delete] = ACTIONS(1719),
    [sym_file_rename] = ACTIONS(1719),
    [sym_file_dir] = ACTIONS(1719),
    [sym_file_getdir] = ACTIONS(1719),
    [sym_file_setdir] = ACTIONS(1719),
    [sym_file_create_dir] = ACTIONS(1719),
    [sym_file_remove_dir] = ACTIONS(1719),
    [sym_file_seek] = ACTIONS(1719),
    [sym_length_array] = ACTIONS(1719),
    [sym_max_length_array] = ACTIONS(1719),
    [sym_set_length_array] = ACTIONS(1719),
    [sym_ip_client_open] = ACTIONS(1719),
    [sym_ip_client_close] = ACTIONS(1719),
    [sym_ip_server_open] = ACTIONS(1719),
    [sym_ip_server_close] = ACTIONS(1719),
    [sym_ip_bound_client_open] = ACTIONS(1719),
    [sym_ip_mc_server_open] = ACTIONS(1719),
    [sym_ip_set_option] = ACTIONS(1719),
    [sym_ssh_client_open] = ACTIONS(1719),
    [sym_ssh_client_close] = ACTIONS(1719),
    [sym_tls_client_open] = ACTIONS(1719),
    [sym_tls_client_close] = ACTIONS(1719),
    [sym_do_push] = ACTIONS(1719),
    [sym_do_push_timed] = ACTIONS(1719),
    [sym_do_release] = ACTIONS(1719),
    [sym_device_id] = ACTIONS(1719),
    [sym_device_info] = ACTIONS(1719),
    [sym_device_id_string] = ACTIONS(1719),
    [sym_device_standby] = ACTIONS(1719),
    [sym_device_wake] = ACTIONS(1719),
    [sym_timeline_create] = ACTIONS(1719),
    [sym_timeline_kill] = ACTIONS(1719),
    [sym_timeline_reload] = ACTIONS(1719),
    [sym_timeline_pause] = ACTIONS(1719),
    [sym_timeline_restart] = ACTIONS(1719),
    [sym_timeline_set] = ACTIONS(1719),
    [sym_timeline_get] = ACTIONS(1719),
    [sym_timeline_active] = ACTIONS(1719),
    [sym_timeline_dynamic_id] = ACTIONS(1719),
    [sym_variable_to_string] = ACTIONS(1719),
    [sym_string_to_variable] = ACTIONS(1719),
    [sym_length_variable_to_string] = ACTIONS(1719),
    [sym_type_cast] = ACTIONS(1719),
    [sym_variable_to_xml] = ACTIONS(1719),
    [sym_xml_to_variable] = ACTIONS(1719),
    [sym_length_variable_to_xml] = ACTIONS(1719),
    [sym_redirect_string] = ACTIONS(1719),
    [sym_random_number] = ACTIONS(1719),
    [sym_get_unique_id] = ACTIONS(1719),
    [sym_get_system_number] = ACTIONS(1719),
    [sym_set_system_number] = ACTIONS(1719),
    [sym_get_serial_number] = ACTIONS(1719),
    [sym_get_master_build] = ACTIONS(1719),
    [sym_set_validation_code] = ACTIONS(1719),
    [sym_reboot] = ACTIONS(1719),
    [sym_load_duet_module] = ACTIONS(1719),
    [sym_unload_duet_module] = ACTIONS(1719),
    [sym_duet_mem_size_set] = ACTIONS(1719),
    [sym_duet_mem_size_get] = ACTIONS(1719),
    [sym_dynamic_application_device] = ACTIONS(1719),
    [sym_dynamic_polled_port] = ACTIONS(1719),
    [sym_static_port_binding] = ACTIONS(1719),
    [sym_static_ip_binding] = ACTIONS(1719),
    [sym_get_dns_list] = ACTIONS(1719),
    [sym_set_dns_list] = ACTIONS(1719),
    [sym_get_ip_address] = ACTIONS(1719),
    [sym_set_ip_address] = ACTIONS(1719),
    [sym_get_url_list] = ACTIONS(1719),
    [sym_add_url_entry] = ACTIONS(1719),
    [sym_delete_url_entry] = ACTIONS(1719),
    [sym_abs_value] = ACTIONS(1719),
    [sym_max_value] = ACTIONS(1719),
    [sym_min_value] = ACTIONS(1719),
    [sym_exp_value] = ACTIONS(1719),
    [sym_log_value] = ACTIONS(1719),
    [sym_log10_value] = ACTIONS(1719),
    [sym_power_value] = ACTIONS(1719),
    [sym_sqrt_value] = ACTIONS(1719),
    [sym_raw_be] = ACTIONS(1719),
    [sym_raw_le] = ACTIONS(1719),
    [sym_internal_queue_size_set] = ACTIONS(1719),
    [sym_internal_queue_size_get] = ACTIONS(1719),
    [sym_internal_threshold_set] = ACTIONS(1719),
    [sym_internal_threshold_get] = ACTIONS(1719),
    [sym_rebuilt_event] = ACTIONS(1719),
    [sym_do_custom_event] = ACTIONS(1719),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1719),
    [sym_clkmgr_set_clk_source] = ACTIONS(1719),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1719),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1719),
    [sym_clkmgr_get_timezone] = ACTIONS(1719),
    [sym_clkmgr_set_timezone] = ACTIONS(1719),
    [sym_clkmgr_get_resync_period] = ACTIONS(1719),
    [sym_clkmgr_set_resync_period] = ACTIONS(1719),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1719),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1719),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1719),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1719),
    [sym_clkmgr_get_timeservers] = ACTIONS(1719),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1719),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1719),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1719),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1719),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1719),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1719),
    [sym_set_log_level] = ACTIONS(1719),
    [sym_get_log_level] = ACTIONS(1719),
    [sym_amx_log] = ACTIONS(1719),
    [sym_smtp_server_config_set] = ACTIONS(1719),
    [sym_smtp_server_config_get] = ACTIONS(1719),
    [sym_smtp_send] = ACTIONS(1719),
    [sym_validate_netlinx_account] = ACTIONS(1719),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1719),
    [sym_authenticate_certificate] = ACTIONS(1719),
    [sym_audit_netlinx_session_event] = ACTIONS(1719),
    [sym_audit_netlinx_generic_event] = ACTIONS(1719),
    [sym_get_available_flash_disk_space] = ACTIONS(1719),
    [sym_get_max_flash_disk_space] = ACTIONS(1719),
    [sym_data_create_feed] = ACTIONS(1719),
    [sym_data_delete_feed] = ACTIONS(1719),
    [sym_data_publish_feed] = ACTIONS(1719),
    [sym_data_get_published_feed] = ACTIONS(1719),
    [sym_data_add_record] = ACTIONS(1719),
    [sym_data_get_event_record] = ACTIONS(1719),
    [sym__wc_data_create_feed] = ACTIONS(1719),
    [sym__wc_data_add_record] = ACTIONS(1719),
    [sym__wc_data_get_event_record] = ACTIONS(1719),
    [sym_format] = ACTIONS(1719),
    [sym_astro_clock] = ACTIONS(1719),
    [sym_set_outdoor_temperature] = ACTIONS(1719),
    [sym_set_virtual_level_count] = ACTIONS(1719),
    [sym_set_virtual_channel_count] = ACTIONS(1719),
    [sym_set_virtual_port_count] = ACTIONS(1719),
    [sym__wc_encode_int] = ACTIONS(1719),
    [sym__wc_decode_int] = ACTIONS(1719),
    [sym__wc_int] = ACTIONS(1719),
    [sym__wc_to_ch_int] = ACTIONS(1719),
    [sym__ch_to_wc_int] = ACTIONS(1719),
    [sym__wc_find_string_int] = ACTIONS(1719),
    [sym__wc_left_string_int] = ACTIONS(1719),
    [sym__wc_lower_string_int] = ACTIONS(1719),
    [sym__wc_mid_string_int] = ACTIONS(1719),
    [sym__wc_remove_string_int] = ACTIONS(1719),
    [sym__wc_right_string_int] = ACTIONS(1719),
    [sym__wc_upper_string_int] = ACTIONS(1719),
    [sym__wc_compare_string_int] = ACTIONS(1719),
    [sym__wc_get_buffer_char_int] = ACTIONS(1719),
    [sym__wc_get_buffer_string_int] = ACTIONS(1719),
    [sym__wc_concat_string_int] = ACTIONS(1719),
    [sym__wc_file_open_int] = ACTIONS(1719),
    [sym__wc_file_close_int] = ACTIONS(1719),
    [sym__wc_file_read_int] = ACTIONS(1719),
    [sym__wc_file_read_line_int] = ACTIONS(1719),
    [sym__wc_file_write_int] = ACTIONS(1719),
    [sym__wc_file_write_line_int] = ACTIONS(1719),
    [aux_sym_preproc_include_token1] = ACTIONS(1719),
    [aux_sym_preproc_define_token1] = ACTIONS(1721),
    [aux_sym_preproc_warn_token1] = ACTIONS(1721),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1721),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1721),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1721),
    [anon_sym_LBRACK] = ACTIONS(1721),
    [anon_sym_LBRACE] = ACTIONS(1721),
    [anon_sym_LPAREN] = ACTIONS(1719),
    [anon_sym_BANG] = ACTIONS(1721),
    [anon_sym_TILDE] = ACTIONS(1721),
    [anon_sym_DASH] = ACTIONS(1719),
    [anon_sym_PLUS] = ACTIONS(1719),
    [anon_sym_DASH_DASH] = ACTIONS(1721),
    [anon_sym_PLUS_PLUS] = ACTIONS(1721),
    [anon_sym_DQUOTE] = ACTIONS(1721),
    [anon_sym_SQUOTE] = ACTIONS(1721),
    [sym_number_literal] = ACTIONS(1721),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(390)] = {
    [ts_builtin_sym_end] = ACTIONS(1833),
    [sym_identifier] = ACTIONS(1831),
    [sym_program_name_keyword] = ACTIONS(1831),
    [sym_module_name_keyword] = ACTIONS(1831),
    [sym_define_device_keyword] = ACTIONS(1831),
    [sym_define_combine_keyword] = ACTIONS(1831),
    [sym_define_constant_keyword] = ACTIONS(1831),
    [sym_define_type_keyword] = ACTIONS(1831),
    [sym_define_variable_keyword] = ACTIONS(1831),
    [sym_define_system_variable_keyword] = ACTIONS(1831),
    [sym_define_connect_level_keyword] = ACTIONS(1831),
    [sym_define_latching_keyword] = ACTIONS(1831),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1831),
    [sym_define_toggling_keyword] = ACTIONS(1831),
    [sym_define_call_keyword] = ACTIONS(1831),
    [sym_define_function_keyword] = ACTIONS(1831),
    [sym_define_library_function_keyword] = ACTIONS(1831),
    [sym_define_start_keyword] = ACTIONS(1831),
    [sym_define_event_keyword] = ACTIONS(1831),
    [sym_define_program_keyword] = ACTIONS(1831),
    [sym_define_module_keyword] = ACTIONS(1831),
    [sym_char_keyword] = ACTIONS(1831),
    [sym_widechar_keyword] = ACTIONS(1831),
    [sym_integer_keyword] = ACTIONS(1831),
    [sym_sinteger_keyword] = ACTIONS(1831),
    [sym_long_keyword] = ACTIONS(1831),
    [sym_slong_keyword] = ACTIONS(1831),
    [sym_float_keyword] = ACTIONS(1831),
    [sym_double_keyword] = ACTIONS(1831),
    [sym_variant_keyword] = ACTIONS(1831),
    [sym_variantarray_keyword] = ACTIONS(1831),
    [sym_local_var_keyword] = ACTIONS(1831),
    [sym_stack_var_keyword] = ACTIONS(1831),
    [sym_constant_keyword] = ACTIONS(1831),
    [sym_volatile_keyword] = ACTIONS(1831),
    [sym_non_volatile_keyword] = ACTIONS(1831),
    [sym_persistent_keyword] = ACTIONS(1831),
    [sym_dev_keyword] = ACTIONS(1831),
    [sym_devlev_keyword] = ACTIONS(1831),
    [sym_devchan_keyword] = ACTIONS(1831),
    [sym_if_keyword] = ACTIONS(1831),
    [sym_else_keyword] = ACTIONS(1831),
    [sym_switch_keyword] = ACTIONS(1831),
    [sym_case_keyword] = ACTIONS(1831),
    [sym_default_keyword] = ACTIONS(1831),
    [sym_while_keyword] = ACTIONS(1831),
    [sym_for_keyword] = ACTIONS(1831),
    [sym_break_keyword] = ACTIONS(1831),
    [sym_continue_keyword] = ACTIONS(1831),
    [sym_return_keyword] = ACTIONS(1831),
    [sym_select_keyword] = ACTIONS(1831),
    [sym_active_keyword] = ACTIONS(1831),
    [sym_struct_keyword] = ACTIONS(1831),
    [sym_structure_keyword] = ACTIONS(1831),
    [sym_button_event_keyword] = ACTIONS(1831),
    [sym_channel_event_keyword] = ACTIONS(1831),
    [sym_level_event_keyword] = ACTIONS(1831),
    [sym_data_event_keyword] = ACTIONS(1831),
    [sym_timeline_event_keyword] = ACTIONS(1831),
    [sym_custom_event_keyword] = ACTIONS(1831),
    [sym_push_keyword] = ACTIONS(1831),
    [sym_release_keyword] = ACTIONS(1831),
    [aux_sym_on_keyword_token1] = ACTIONS(1831),
    [aux_sym_off_keyword_token1] = ACTIONS(1831),
    [sym_send_level_keyword] = ACTIONS(1831),
    [sym_send_string_keyword] = ACTIONS(1831),
    [sym_send_command_keyword] = ACTIONS(1831),
    [sym_clear_buffer_keyword] = ACTIONS(1831),
    [sym_create_buffer_keyword] = ACTIONS(1831),
    [sym_create_multi_buffer_keyword] = ACTIONS(1831),
    [sym_call_keyword] = ACTIONS(1831),
    [sym_system_call_keyword] = ACTIONS(1831),
    [sym_devchan_to_keyword] = ACTIONS(1831),
    [sym_devchan_min_to_keyword] = ACTIONS(1831),
    [sym_devchan_total_off_keyword] = ACTIONS(1831),
    [sym_devchan_pulse_keyword] = ACTIONS(1831),
    [sym_wait_keyword] = ACTIONS(1831),
    [sym_cancel_wait_keyword] = ACTIONS(1831),
    [sym_wait_until_keyword] = ACTIONS(1831),
    [sym_cancel_wait_until_keyword] = ACTIONS(1831),
    [sym_cancel_all_wait_keyword] = ACTIONS(1831),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1831),
    [sym_not] = ACTIONS(1831),
    [sym_bnot] = ACTIONS(1831),
    [sym___file__] = ACTIONS(1831),
    [sym___date__] = ACTIONS(1831),
    [sym___ldate__] = ACTIONS(1831),
    [sym___time__] = ACTIONS(1831),
    [sym___line__] = ACTIONS(1831),
    [sym___name__] = ACTIONS(1831),
    [sym___netlinx__] = ACTIONS(1831),
    [sym_netlinx_axi_version] = ACTIONS(1831),
    [sym_url_flg_tcp] = ACTIONS(1831),
    [sym_url_flg_accinfopresent] = ACTIONS(1831),
    [sym_url_flg_temp] = ACTIONS(1831),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1831),
    [sym_url_flg_stat_mask] = ACTIONS(1831),
    [sym_url_flg_stat_lookup] = ACTIONS(1831),
    [sym_url_flg_stat_connecting] = ACTIONS(1831),
    [sym_url_flg_stat_waiting] = ACTIONS(1831),
    [sym_url_flg_stat_connected] = ACTIONS(1831),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1831),
    [sym_first_virtual_device] = ACTIONS(1831),
    [sym_first_local_port] = ACTIONS(1831),
    [sym_dynamic_virtual_device] = ACTIONS(1831),
    [sym_timeline_once] = ACTIONS(1831),
    [sym_timeline_repeat] = ACTIONS(1831),
    [sym_timeline_absolute] = ACTIONS(1831),
    [sym_timeline_relative] = ACTIONS(1831),
    [sym_do_push_timed_infinite] = ACTIONS(1831),
    [sym_source_type_no_address] = ACTIONS(1831),
    [sym_source_type_neuron_id] = ACTIONS(1831),
    [sym_source_type_ip_address] = ACTIONS(1831),
    [sym_source_type_axlink] = ACTIONS(1831),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1831),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1831),
    [sym_source_type_ip_socket_address] = ACTIONS(1831),
    [sym_source_type_rs232] = ACTIONS(1831),
    [sym_source_type_internal] = ACTIONS(1831),
    [sym_source_type_mac_address] = ACTIONS(1831),
    [sym_source_type_ipv4_port] = ACTIONS(1831),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1831),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1831),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1831),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1831),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1831),
    [sym_true] = ACTIONS(1831),
    [sym_false] = ACTIONS(1831),
    [sym_file_read_only] = ACTIONS(1831),
    [sym_file_rw_new] = ACTIONS(1831),
    [sym_file_rw_append] = ACTIONS(1831),
    [sym_ip_tcp] = ACTIONS(1831),
    [sym_ip_udp] = ACTIONS(1831),
    [sym_ip_udp_2way] = ACTIONS(1831),
    [sym_xml_encode_types] = ACTIONS(1831),
    [sym_xml_encode_char_as_list] = ACTIONS(1831),
    [sym_xml_encode_le] = ACTIONS(1831),
    [sym_xml_decode_types] = ACTIONS(1831),
    [sym_xml_decode_no_preserve] = ACTIONS(1831),
    [sym_duet_dev_type_display_device] = ACTIONS(1831),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1831),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1831),
    [sym_duet_dev_type_io_device] = ACTIONS(1831),
    [sym_duet_dev_type_relay_device] = ACTIONS(1831),
    [sym_duet_dev_type_ups] = ACTIONS(1831),
    [sym_duet_dev_type_amplifier] = ACTIONS(1831),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1831),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1831),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1831),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1831),
    [sym_duet_dev_type_camera] = ACTIONS(1831),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1831),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1831),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1831),
    [sym_duet_dev_type_dss] = ACTIONS(1831),
    [sym_duet_dev_type_dvr] = ACTIONS(1831),
    [sym_duet_dev_type_disc_device] = ACTIONS(1831),
    [sym_duet_dev_type_document_camera] = ACTIONS(1831),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1831),
    [sym_duet_dev_type_hvac] = ACTIONS(1831),
    [sym_duet_dev_type_keypad] = ACTIONS(1831),
    [sym_duet_dev_type_light] = ACTIONS(1831),
    [sym_duet_dev_type_monitor] = ACTIONS(1831),
    [sym_duet_dev_type_motor] = ACTIONS(1831),
    [sym_duet_dev_type_multi_window] = ACTIONS(1831),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1831),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1831),
    [sym_duet_dev_type_receiver] = ACTIONS(1831),
    [sym_duet_dev_type_security_system] = ACTIONS(1831),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1831),
    [sym_duet_dev_type_settop_box] = ACTIONS(1831),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1831),
    [sym_duet_dev_type_switcher] = ACTIONS(1831),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1831),
    [sym_duet_dev_type_tv] = ACTIONS(1831),
    [sym_duet_dev_type_utility] = ACTIONS(1831),
    [sym_duet_dev_type_vcr] = ACTIONS(1831),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1831),
    [sym_duet_dev_type_video_processor] = ACTIONS(1831),
    [sym_duet_dev_type_video_projector] = ACTIONS(1831),
    [sym_duet_dev_type_video_wall] = ACTIONS(1831),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1831),
    [sym_duet_dev_type_weather] = ACTIONS(1831),
    [sym_duet_dev_not_polled] = ACTIONS(1831),
    [sym_duet_dev_polled] = ACTIONS(1831),
    [sym_clkmgr_mode_network] = ACTIONS(1831),
    [sym_clkmgr_mode_standalone] = ACTIONS(1831),
    [sym_normal_standby] = ACTIONS(1831),
    [sym_normal_wake] = ACTIONS(1831),
    [sym_ip_multicast_ttl_option] = ACTIONS(1831),
    [sym_ip_ttl_subnet] = ACTIONS(1831),
    [sym_ip_ttl_site] = ACTIONS(1831),
    [sym_ip_ttl_region] = ACTIONS(1831),
    [sym_ip_ttl_continent] = ACTIONS(1831),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1831),
    [sym_ip_nodelay_on] = ACTIONS(1831),
    [sym_ip_nodelay_off] = ACTIONS(1831),
    [sym_amx_error] = ACTIONS(1831),
    [sym_amx_warning] = ACTIONS(1831),
    [sym_amx_info] = ACTIONS(1831),
    [sym_amx_debug] = ACTIONS(1831),
    [sym_netlinx_login_success] = ACTIONS(1831),
    [sym_netlinx_login_fail] = ACTIONS(1831),
    [sym_netlinx_logout] = ACTIONS(1831),
    [sym_null_str] = ACTIONS(1831),
    [sym_smtp_address] = ACTIONS(1831),
    [sym_smtp_port_number] = ACTIONS(1831),
    [sym_smtp_username] = ACTIONS(1831),
    [sym_smtp_password] = ACTIONS(1831),
    [sym_smtp_require_tls] = ACTIONS(1831),
    [sym_smtp_from] = ACTIONS(1831),
    [sym_smtp_tls_true] = ACTIONS(1831),
    [sym_smtp_tls_false] = ACTIONS(1831),
    [sym_listview_on_row_select_event] = ACTIONS(1831),
    [sym_data_structure_unknown] = ACTIONS(1831),
    [sym_data_structure_datafeed] = ACTIONS(1831),
    [sym_data_structure_datarecordset] = ACTIONS(1831),
    [sym_data_structure_datarecord] = ACTIONS(1831),
    [sym_data_structure_datafield] = ACTIONS(1831),
    [sym_data_max_content_field_count] = ACTIONS(1831),
    [sym_data_max_metadata_field_count] = ACTIONS(1831),
    [sym_data_max_id_length] = ACTIONS(1831),
    [sym_data_max_name_length] = ACTIONS(1831),
    [sym_data_max_description_length] = ACTIONS(1831),
    [sym_data_max_type_length] = ACTIONS(1831),
    [sym_data_max_format_length] = ACTIONS(1831),
    [sym_data_max_source_length] = ACTIONS(1831),
    [sym_data_max_value_length] = ACTIONS(1831),
    [sym_data_max_label_length] = ACTIONS(1831),
    [sym_data_max_url_length] = ACTIONS(1831),
    [sym_max_last_login_info_length] = ACTIONS(1831),
    [sym_data_type_unknown] = ACTIONS(1831),
    [sym_data_type_string] = ACTIONS(1831),
    [sym_data_type_datetime] = ACTIONS(1831),
    [sym_data_type_date] = ACTIONS(1831),
    [sym_data_type_time] = ACTIONS(1831),
    [sym_data_type_image] = ACTIONS(1831),
    [sym_data_format_url] = ACTIONS(1831),
    [sym_data_format_phone] = ACTIONS(1831),
    [sym_data_format_email] = ACTIONS(1831),
    [sym_data_format_iso8601] = ACTIONS(1831),
    [sym_auditlog_priv] = ACTIONS(1831),
    [sym_configuration_priv] = ACTIONS(1831),
    [sym_deviceconfig_priv] = ACTIONS(1831),
    [sym_ftp_priv] = ACTIONS(1831),
    [sym_http_priv] = ACTIONS(1831),
    [sym_networkconfig_priv] = ACTIONS(1831),
    [sym_programport_priv] = ACTIONS(1831),
    [sym_remoteui_priv] = ACTIONS(1831),
    [sym_securitycontrol_priv] = ACTIONS(1831),
    [sym_softwaremanagement_priv] = ACTIONS(1831),
    [sym_terminal_priv] = ACTIONS(1831),
    [sym_tpadmin_priv] = ACTIONS(1831),
    [sym_user1_priv] = ACTIONS(1831),
    [sym_user2_priv] = ACTIONS(1831),
    [sym_user3_priv] = ACTIONS(1831),
    [sym_user4_priv] = ACTIONS(1831),
    [sym_usermanagement_priv] = ACTIONS(1831),
    [sym_valid_account] = ACTIONS(1831),
    [sym_err_name_invalid] = ACTIONS(1831),
    [sym_err_password_invalid] = ACTIONS(1831),
    [sym_err_invalid_account] = ACTIONS(1831),
    [sym_err_no_authorization] = ACTIONS(1831),
    [sym_err_invalid_parameter] = ACTIONS(1831),
    [sym_err_user_locked_out] = ACTIONS(1831),
    [sym_err_user_account_expired] = ACTIONS(1831),
    [sym_err_general_error] = ACTIONS(1831),
    [sym_err_authentication_error] = ACTIONS(1831),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1831),
    [sym_tls_validate_certificate] = ACTIONS(1831),
    [sym_tdata] = ACTIONS(1831),
    [sym_tchannel] = ACTIONS(1831),
    [sym_tlevel] = ACTIONS(1831),
    [sym_tbutton] = ACTIONS(1831),
    [sym_ttimeline] = ACTIONS(1831),
    [sym_tcustom] = ACTIONS(1831),
    [sym_url_struct] = ACTIONS(1831),
    [sym_dns_struct] = ACTIONS(1831),
    [sym_ip_address_struct] = ACTIONS(1831),
    [sym_dev_info_struct] = ACTIONS(1831),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1831),
    [sym_data_feed] = ACTIONS(1831),
    [sym_data_field] = ACTIONS(1831),
    [sym_data_record] = ACTIONS(1831),
    [sym_wc_data_feed] = ACTIONS(1831),
    [sym_wc_data_field] = ACTIONS(1831),
    [sym_wc_data_record] = ACTIONS(1831),
    [sym_last_login_info] = ACTIONS(1831),
    [sym____reserved___] = ACTIONS(1831),
    [sym_date] = ACTIONS(1831),
    [sym_day] = ACTIONS(1831),
    [sym_time] = ACTIONS(1831),
    [sym_ldate] = ACTIONS(1831),
    [sym_push_device] = ACTIONS(1831),
    [sym_push_channel] = ACTIONS(1831),
    [sym_push_devchan] = ACTIONS(1831),
    [sym_release_device] = ACTIONS(1831),
    [sym_release_channel] = ACTIONS(1831),
    [sym_release_devchan] = ACTIONS(1831),
    [sym_master_slot] = ACTIONS(1831),
    [sym_get_pulse_time] = ACTIONS(1831),
    [sym_get_timer] = ACTIONS(1831),
    [sym_master_sn] = ACTIONS(1831),
    [sym_system_number] = ACTIONS(1831),
    [sym_button] = ACTIONS(1831),
    [sym_dv_channel] = ACTIONS(1831),
    [sym_data] = ACTIONS(1831),
    [sym_level] = ACTIONS(1831),
    [sym_timeline] = ACTIONS(1831),
    [sym_channel] = ACTIONS(1831),
    [sym_custom] = ACTIONS(1831),
    [sym_atoi] = ACTIONS(1831),
    [sym_atol] = ACTIONS(1831),
    [sym_atof] = ACTIONS(1831),
    [sym_itoa] = ACTIONS(1831),
    [sym_itohex] = ACTIONS(1831),
    [sym_hextoi] = ACTIONS(1831),
    [sym_ftoa] = ACTIONS(1831),
    [sym_find_string] = ACTIONS(1831),
    [sym_left_string] = ACTIONS(1831),
    [sym_right_string] = ACTIONS(1831),
    [sym_mid_string] = ACTIONS(1831),
    [sym_remove_string] = ACTIONS(1831),
    [sym_get_buffer_string] = ACTIONS(1831),
    [sym_get_buffer_char] = ACTIONS(1831),
    [sym_get_multi_buffer_string] = ACTIONS(1831),
    [sym_length_string] = ACTIONS(1831),
    [sym_max_length_string] = ACTIONS(1831),
    [sym_set_length_string] = ACTIONS(1831),
    [sym_compare_string] = ACTIONS(1831),
    [sym_upper_string] = ACTIONS(1831),
    [sym_lower_string] = ACTIONS(1831),
    [sym_get_last] = ACTIONS(1831),
    [sym_date_to_day] = ACTIONS(1831),
    [sym_date_to_month] = ACTIONS(1831),
    [sym_date_to_year] = ACTIONS(1831),
    [sym_time_to_hour] = ACTIONS(1831),
    [sym_time_to_minute] = ACTIONS(1831),
    [sym_time_to_second] = ACTIONS(1831),
    [sym_day_of_week] = ACTIONS(1831),
    [sym_set_pulse_time] = ACTIONS(1831),
    [sym_set_timer] = ACTIONS(1831),
    [sym_file_open] = ACTIONS(1831),
    [sym_file_close] = ACTIONS(1831),
    [sym_file_read] = ACTIONS(1831),
    [sym_file_read_line] = ACTIONS(1831),
    [sym_file_write] = ACTIONS(1831),
    [sym_file_write_line] = ACTIONS(1831),
    [sym_file_copy] = ACTIONS(1831),
    [sym_file_delete] = ACTIONS(1831),
    [sym_file_rename] = ACTIONS(1831),
    [sym_file_dir] = ACTIONS(1831),
    [sym_file_getdir] = ACTIONS(1831),
    [sym_file_setdir] = ACTIONS(1831),
    [sym_file_create_dir] = ACTIONS(1831),
    [sym_file_remove_dir] = ACTIONS(1831),
    [sym_file_seek] = ACTIONS(1831),
    [sym_length_array] = ACTIONS(1831),
    [sym_max_length_array] = ACTIONS(1831),
    [sym_set_length_array] = ACTIONS(1831),
    [sym_ip_client_open] = ACTIONS(1831),
    [sym_ip_client_close] = ACTIONS(1831),
    [sym_ip_server_open] = ACTIONS(1831),
    [sym_ip_server_close] = ACTIONS(1831),
    [sym_ip_bound_client_open] = ACTIONS(1831),
    [sym_ip_mc_server_open] = ACTIONS(1831),
    [sym_ip_set_option] = ACTIONS(1831),
    [sym_ssh_client_open] = ACTIONS(1831),
    [sym_ssh_client_close] = ACTIONS(1831),
    [sym_tls_client_open] = ACTIONS(1831),
    [sym_tls_client_close] = ACTIONS(1831),
    [sym_do_push] = ACTIONS(1831),
    [sym_do_push_timed] = ACTIONS(1831),
    [sym_do_release] = ACTIONS(1831),
    [sym_device_id] = ACTIONS(1831),
    [sym_device_info] = ACTIONS(1831),
    [sym_device_id_string] = ACTIONS(1831),
    [sym_device_standby] = ACTIONS(1831),
    [sym_device_wake] = ACTIONS(1831),
    [sym_timeline_create] = ACTIONS(1831),
    [sym_timeline_kill] = ACTIONS(1831),
    [sym_timeline_reload] = ACTIONS(1831),
    [sym_timeline_pause] = ACTIONS(1831),
    [sym_timeline_restart] = ACTIONS(1831),
    [sym_timeline_set] = ACTIONS(1831),
    [sym_timeline_get] = ACTIONS(1831),
    [sym_timeline_active] = ACTIONS(1831),
    [sym_timeline_dynamic_id] = ACTIONS(1831),
    [sym_variable_to_string] = ACTIONS(1831),
    [sym_string_to_variable] = ACTIONS(1831),
    [sym_length_variable_to_string] = ACTIONS(1831),
    [sym_type_cast] = ACTIONS(1831),
    [sym_variable_to_xml] = ACTIONS(1831),
    [sym_xml_to_variable] = ACTIONS(1831),
    [sym_length_variable_to_xml] = ACTIONS(1831),
    [sym_redirect_string] = ACTIONS(1831),
    [sym_random_number] = ACTIONS(1831),
    [sym_get_unique_id] = ACTIONS(1831),
    [sym_get_system_number] = ACTIONS(1831),
    [sym_set_system_number] = ACTIONS(1831),
    [sym_get_serial_number] = ACTIONS(1831),
    [sym_get_master_build] = ACTIONS(1831),
    [sym_set_validation_code] = ACTIONS(1831),
    [sym_reboot] = ACTIONS(1831),
    [sym_load_duet_module] = ACTIONS(1831),
    [sym_unload_duet_module] = ACTIONS(1831),
    [sym_duet_mem_size_set] = ACTIONS(1831),
    [sym_duet_mem_size_get] = ACTIONS(1831),
    [sym_dynamic_application_device] = ACTIONS(1831),
    [sym_dynamic_polled_port] = ACTIONS(1831),
    [sym_static_port_binding] = ACTIONS(1831),
    [sym_static_ip_binding] = ACTIONS(1831),
    [sym_get_dns_list] = ACTIONS(1831),
    [sym_set_dns_list] = ACTIONS(1831),
    [sym_get_ip_address] = ACTIONS(1831),
    [sym_set_ip_address] = ACTIONS(1831),
    [sym_get_url_list] = ACTIONS(1831),
    [sym_add_url_entry] = ACTIONS(1831),
    [sym_delete_url_entry] = ACTIONS(1831),
    [sym_abs_value] = ACTIONS(1831),
    [sym_max_value] = ACTIONS(1831),
    [sym_min_value] = ACTIONS(1831),
    [sym_exp_value] = ACTIONS(1831),
    [sym_log_value] = ACTIONS(1831),
    [sym_log10_value] = ACTIONS(1831),
    [sym_power_value] = ACTIONS(1831),
    [sym_sqrt_value] = ACTIONS(1831),
    [sym_raw_be] = ACTIONS(1831),
    [sym_raw_le] = ACTIONS(1831),
    [sym_internal_queue_size_set] = ACTIONS(1831),
    [sym_internal_queue_size_get] = ACTIONS(1831),
    [sym_internal_threshold_set] = ACTIONS(1831),
    [sym_internal_threshold_get] = ACTIONS(1831),
    [sym_rebuilt_event] = ACTIONS(1831),
    [sym_do_custom_event] = ACTIONS(1831),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1831),
    [sym_clkmgr_set_clk_source] = ACTIONS(1831),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1831),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1831),
    [sym_clkmgr_get_timezone] = ACTIONS(1831),
    [sym_clkmgr_set_timezone] = ACTIONS(1831),
    [sym_clkmgr_get_resync_period] = ACTIONS(1831),
    [sym_clkmgr_set_resync_period] = ACTIONS(1831),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1831),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1831),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1831),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1831),
    [sym_clkmgr_get_timeservers] = ACTIONS(1831),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1831),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1831),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1831),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1831),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1831),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1831),
    [sym_set_log_level] = ACTIONS(1831),
    [sym_get_log_level] = ACTIONS(1831),
    [sym_amx_log] = ACTIONS(1831),
    [sym_smtp_server_config_set] = ACTIONS(1831),
    [sym_smtp_server_config_get] = ACTIONS(1831),
    [sym_smtp_send] = ACTIONS(1831),
    [sym_validate_netlinx_account] = ACTIONS(1831),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1831),
    [sym_authenticate_certificate] = ACTIONS(1831),
    [sym_audit_netlinx_session_event] = ACTIONS(1831),
    [sym_audit_netlinx_generic_event] = ACTIONS(1831),
    [sym_get_available_flash_disk_space] = ACTIONS(1831),
    [sym_get_max_flash_disk_space] = ACTIONS(1831),
    [sym_data_create_feed] = ACTIONS(1831),
    [sym_data_delete_feed] = ACTIONS(1831),
    [sym_data_publish_feed] = ACTIONS(1831),
    [sym_data_get_published_feed] = ACTIONS(1831),
    [sym_data_add_record] = ACTIONS(1831),
    [sym_data_get_event_record] = ACTIONS(1831),
    [sym__wc_data_create_feed] = ACTIONS(1831),
    [sym__wc_data_add_record] = ACTIONS(1831),
    [sym__wc_data_get_event_record] = ACTIONS(1831),
    [sym_format] = ACTIONS(1831),
    [sym_astro_clock] = ACTIONS(1831),
    [sym_set_outdoor_temperature] = ACTIONS(1831),
    [sym_set_virtual_level_count] = ACTIONS(1831),
    [sym_set_virtual_channel_count] = ACTIONS(1831),
    [sym_set_virtual_port_count] = ACTIONS(1831),
    [sym__wc_encode_int] = ACTIONS(1831),
    [sym__wc_decode_int] = ACTIONS(1831),
    [sym__wc_int] = ACTIONS(1831),
    [sym__wc_to_ch_int] = ACTIONS(1831),
    [sym__ch_to_wc_int] = ACTIONS(1831),
    [sym__wc_find_string_int] = ACTIONS(1831),
    [sym__wc_left_string_int] = ACTIONS(1831),
    [sym__wc_lower_string_int] = ACTIONS(1831),
    [sym__wc_mid_string_int] = ACTIONS(1831),
    [sym__wc_remove_string_int] = ACTIONS(1831),
    [sym__wc_right_string_int] = ACTIONS(1831),
    [sym__wc_upper_string_int] = ACTIONS(1831),
    [sym__wc_compare_string_int] = ACTIONS(1831),
    [sym__wc_get_buffer_char_int] = ACTIONS(1831),
    [sym__wc_get_buffer_string_int] = ACTIONS(1831),
    [sym__wc_concat_string_int] = ACTIONS(1831),
    [sym__wc_file_open_int] = ACTIONS(1831),
    [sym__wc_file_close_int] = ACTIONS(1831),
    [sym__wc_file_read_int] = ACTIONS(1831),
    [sym__wc_file_read_line_int] = ACTIONS(1831),
    [sym__wc_file_write_int] = ACTIONS(1831),
    [sym__wc_file_write_line_int] = ACTIONS(1831),
    [aux_sym_preproc_include_token1] = ACTIONS(1831),
    [aux_sym_preproc_define_token1] = ACTIONS(1833),
    [aux_sym_preproc_warn_token1] = ACTIONS(1833),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1833),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1833),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1833),
    [anon_sym_LBRACK] = ACTIONS(1833),
    [anon_sym_LBRACE] = ACTIONS(1833),
    [anon_sym_LPAREN] = ACTIONS(1831),
    [anon_sym_BANG] = ACTIONS(1833),
    [anon_sym_TILDE] = ACTIONS(1833),
    [anon_sym_DASH] = ACTIONS(1831),
    [anon_sym_PLUS] = ACTIONS(1831),
    [anon_sym_DASH_DASH] = ACTIONS(1833),
    [anon_sym_PLUS_PLUS] = ACTIONS(1833),
    [anon_sym_DQUOTE] = ACTIONS(1833),
    [anon_sym_SQUOTE] = ACTIONS(1833),
    [sym_number_literal] = ACTIONS(1833),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(391)] = {
    [sym_identifier] = ACTIONS(2129),
    [sym_program_name_keyword] = ACTIONS(2129),
    [sym_module_name_keyword] = ACTIONS(2129),
    [sym_define_device_keyword] = ACTIONS(2129),
    [sym_define_combine_keyword] = ACTIONS(2129),
    [sym_define_constant_keyword] = ACTIONS(2129),
    [sym_define_type_keyword] = ACTIONS(2129),
    [sym_define_variable_keyword] = ACTIONS(2129),
    [sym_define_system_variable_keyword] = ACTIONS(2129),
    [sym_define_connect_level_keyword] = ACTIONS(2129),
    [sym_define_latching_keyword] = ACTIONS(2129),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2129),
    [sym_define_toggling_keyword] = ACTIONS(2129),
    [sym_define_call_keyword] = ACTIONS(2129),
    [sym_define_function_keyword] = ACTIONS(2129),
    [sym_define_library_function_keyword] = ACTIONS(2129),
    [sym_define_start_keyword] = ACTIONS(2129),
    [sym_define_event_keyword] = ACTIONS(2129),
    [sym_define_program_keyword] = ACTIONS(2129),
    [sym_define_module_keyword] = ACTIONS(2129),
    [sym_char_keyword] = ACTIONS(2129),
    [sym_widechar_keyword] = ACTIONS(2129),
    [sym_integer_keyword] = ACTIONS(2129),
    [sym_sinteger_keyword] = ACTIONS(2129),
    [sym_long_keyword] = ACTIONS(2129),
    [sym_slong_keyword] = ACTIONS(2129),
    [sym_float_keyword] = ACTIONS(2129),
    [sym_double_keyword] = ACTIONS(2129),
    [sym_variant_keyword] = ACTIONS(2129),
    [sym_variantarray_keyword] = ACTIONS(2129),
    [sym_local_var_keyword] = ACTIONS(2129),
    [sym_stack_var_keyword] = ACTIONS(2129),
    [sym_constant_keyword] = ACTIONS(2129),
    [sym_volatile_keyword] = ACTIONS(2129),
    [sym_non_volatile_keyword] = ACTIONS(2129),
    [sym_persistent_keyword] = ACTIONS(2129),
    [sym_dev_keyword] = ACTIONS(2129),
    [sym_devlev_keyword] = ACTIONS(2129),
    [sym_devchan_keyword] = ACTIONS(2129),
    [sym_if_keyword] = ACTIONS(2129),
    [sym_switch_keyword] = ACTIONS(2129),
    [sym_case_keyword] = ACTIONS(2129),
    [sym_default_keyword] = ACTIONS(2129),
    [sym_while_keyword] = ACTIONS(2129),
    [sym_for_keyword] = ACTIONS(2129),
    [sym_break_keyword] = ACTIONS(2129),
    [sym_continue_keyword] = ACTIONS(2129),
    [sym_return_keyword] = ACTIONS(2129),
    [sym_select_keyword] = ACTIONS(2129),
    [sym_active_keyword] = ACTIONS(2129),
    [sym_struct_keyword] = ACTIONS(2129),
    [sym_structure_keyword] = ACTIONS(2129),
    [sym_button_event_keyword] = ACTIONS(2129),
    [sym_channel_event_keyword] = ACTIONS(2129),
    [sym_level_event_keyword] = ACTIONS(2129),
    [sym_data_event_keyword] = ACTIONS(2129),
    [sym_timeline_event_keyword] = ACTIONS(2129),
    [sym_custom_event_keyword] = ACTIONS(2129),
    [sym_push_keyword] = ACTIONS(2129),
    [sym_release_keyword] = ACTIONS(2129),
    [aux_sym_on_keyword_token1] = ACTIONS(2129),
    [aux_sym_off_keyword_token1] = ACTIONS(2129),
    [sym_send_level_keyword] = ACTIONS(2129),
    [sym_send_string_keyword] = ACTIONS(2129),
    [sym_send_command_keyword] = ACTIONS(2129),
    [sym_clear_buffer_keyword] = ACTIONS(2129),
    [sym_create_buffer_keyword] = ACTIONS(2129),
    [sym_create_multi_buffer_keyword] = ACTIONS(2129),
    [sym_call_keyword] = ACTIONS(2129),
    [sym_system_call_keyword] = ACTIONS(2129),
    [sym_devchan_to_keyword] = ACTIONS(2129),
    [sym_devchan_min_to_keyword] = ACTIONS(2129),
    [sym_devchan_total_off_keyword] = ACTIONS(2129),
    [sym_devchan_pulse_keyword] = ACTIONS(2129),
    [sym_wait_keyword] = ACTIONS(2129),
    [sym_cancel_wait_keyword] = ACTIONS(2129),
    [sym_wait_until_keyword] = ACTIONS(2129),
    [sym_cancel_wait_until_keyword] = ACTIONS(2129),
    [sym_cancel_all_wait_keyword] = ACTIONS(2129),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2129),
    [sym_not] = ACTIONS(2129),
    [sym_bnot] = ACTIONS(2129),
    [sym___file__] = ACTIONS(2129),
    [sym___date__] = ACTIONS(2129),
    [sym___ldate__] = ACTIONS(2129),
    [sym___time__] = ACTIONS(2129),
    [sym___line__] = ACTIONS(2129),
    [sym___name__] = ACTIONS(2129),
    [sym___netlinx__] = ACTIONS(2129),
    [sym_netlinx_axi_version] = ACTIONS(2129),
    [sym_url_flg_tcp] = ACTIONS(2129),
    [sym_url_flg_accinfopresent] = ACTIONS(2129),
    [sym_url_flg_temp] = ACTIONS(2129),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2129),
    [sym_url_flg_stat_mask] = ACTIONS(2129),
    [sym_url_flg_stat_lookup] = ACTIONS(2129),
    [sym_url_flg_stat_connecting] = ACTIONS(2129),
    [sym_url_flg_stat_waiting] = ACTIONS(2129),
    [sym_url_flg_stat_connected] = ACTIONS(2129),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2129),
    [sym_first_virtual_device] = ACTIONS(2129),
    [sym_first_local_port] = ACTIONS(2129),
    [sym_dynamic_virtual_device] = ACTIONS(2129),
    [sym_timeline_once] = ACTIONS(2129),
    [sym_timeline_repeat] = ACTIONS(2129),
    [sym_timeline_absolute] = ACTIONS(2129),
    [sym_timeline_relative] = ACTIONS(2129),
    [sym_do_push_timed_infinite] = ACTIONS(2129),
    [sym_source_type_no_address] = ACTIONS(2129),
    [sym_source_type_neuron_id] = ACTIONS(2129),
    [sym_source_type_ip_address] = ACTIONS(2129),
    [sym_source_type_axlink] = ACTIONS(2129),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2129),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2129),
    [sym_source_type_ip_socket_address] = ACTIONS(2129),
    [sym_source_type_rs232] = ACTIONS(2129),
    [sym_source_type_internal] = ACTIONS(2129),
    [sym_source_type_mac_address] = ACTIONS(2129),
    [sym_source_type_ipv4_port] = ACTIONS(2129),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2129),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2129),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2129),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2129),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2129),
    [sym_true] = ACTIONS(2129),
    [sym_false] = ACTIONS(2129),
    [sym_file_read_only] = ACTIONS(2129),
    [sym_file_rw_new] = ACTIONS(2129),
    [sym_file_rw_append] = ACTIONS(2129),
    [sym_ip_tcp] = ACTIONS(2129),
    [sym_ip_udp] = ACTIONS(2129),
    [sym_ip_udp_2way] = ACTIONS(2129),
    [sym_xml_encode_types] = ACTIONS(2129),
    [sym_xml_encode_char_as_list] = ACTIONS(2129),
    [sym_xml_encode_le] = ACTIONS(2129),
    [sym_xml_decode_types] = ACTIONS(2129),
    [sym_xml_decode_no_preserve] = ACTIONS(2129),
    [sym_duet_dev_type_display_device] = ACTIONS(2129),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2129),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2129),
    [sym_duet_dev_type_io_device] = ACTIONS(2129),
    [sym_duet_dev_type_relay_device] = ACTIONS(2129),
    [sym_duet_dev_type_ups] = ACTIONS(2129),
    [sym_duet_dev_type_amplifier] = ACTIONS(2129),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2129),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2129),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2129),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2129),
    [sym_duet_dev_type_camera] = ACTIONS(2129),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2129),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2129),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2129),
    [sym_duet_dev_type_dss] = ACTIONS(2129),
    [sym_duet_dev_type_dvr] = ACTIONS(2129),
    [sym_duet_dev_type_disc_device] = ACTIONS(2129),
    [sym_duet_dev_type_document_camera] = ACTIONS(2129),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2129),
    [sym_duet_dev_type_hvac] = ACTIONS(2129),
    [sym_duet_dev_type_keypad] = ACTIONS(2129),
    [sym_duet_dev_type_light] = ACTIONS(2129),
    [sym_duet_dev_type_monitor] = ACTIONS(2129),
    [sym_duet_dev_type_motor] = ACTIONS(2129),
    [sym_duet_dev_type_multi_window] = ACTIONS(2129),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2129),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2129),
    [sym_duet_dev_type_receiver] = ACTIONS(2129),
    [sym_duet_dev_type_security_system] = ACTIONS(2129),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2129),
    [sym_duet_dev_type_settop_box] = ACTIONS(2129),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2129),
    [sym_duet_dev_type_switcher] = ACTIONS(2129),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2129),
    [sym_duet_dev_type_tv] = ACTIONS(2129),
    [sym_duet_dev_type_utility] = ACTIONS(2129),
    [sym_duet_dev_type_vcr] = ACTIONS(2129),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2129),
    [sym_duet_dev_type_video_processor] = ACTIONS(2129),
    [sym_duet_dev_type_video_projector] = ACTIONS(2129),
    [sym_duet_dev_type_video_wall] = ACTIONS(2129),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2129),
    [sym_duet_dev_type_weather] = ACTIONS(2129),
    [sym_duet_dev_not_polled] = ACTIONS(2129),
    [sym_duet_dev_polled] = ACTIONS(2129),
    [sym_clkmgr_mode_network] = ACTIONS(2129),
    [sym_clkmgr_mode_standalone] = ACTIONS(2129),
    [sym_normal_standby] = ACTIONS(2129),
    [sym_normal_wake] = ACTIONS(2129),
    [sym_ip_multicast_ttl_option] = ACTIONS(2129),
    [sym_ip_ttl_subnet] = ACTIONS(2129),
    [sym_ip_ttl_site] = ACTIONS(2129),
    [sym_ip_ttl_region] = ACTIONS(2129),
    [sym_ip_ttl_continent] = ACTIONS(2129),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2129),
    [sym_ip_nodelay_on] = ACTIONS(2129),
    [sym_ip_nodelay_off] = ACTIONS(2129),
    [sym_amx_error] = ACTIONS(2129),
    [sym_amx_warning] = ACTIONS(2129),
    [sym_amx_info] = ACTIONS(2129),
    [sym_amx_debug] = ACTIONS(2129),
    [sym_netlinx_login_success] = ACTIONS(2129),
    [sym_netlinx_login_fail] = ACTIONS(2129),
    [sym_netlinx_logout] = ACTIONS(2129),
    [sym_null_str] = ACTIONS(2129),
    [sym_smtp_address] = ACTIONS(2129),
    [sym_smtp_port_number] = ACTIONS(2129),
    [sym_smtp_username] = ACTIONS(2129),
    [sym_smtp_password] = ACTIONS(2129),
    [sym_smtp_require_tls] = ACTIONS(2129),
    [sym_smtp_from] = ACTIONS(2129),
    [sym_smtp_tls_true] = ACTIONS(2129),
    [sym_smtp_tls_false] = ACTIONS(2129),
    [sym_listview_on_row_select_event] = ACTIONS(2129),
    [sym_data_structure_unknown] = ACTIONS(2129),
    [sym_data_structure_datafeed] = ACTIONS(2129),
    [sym_data_structure_datarecordset] = ACTIONS(2129),
    [sym_data_structure_datarecord] = ACTIONS(2129),
    [sym_data_structure_datafield] = ACTIONS(2129),
    [sym_data_max_content_field_count] = ACTIONS(2129),
    [sym_data_max_metadata_field_count] = ACTIONS(2129),
    [sym_data_max_id_length] = ACTIONS(2129),
    [sym_data_max_name_length] = ACTIONS(2129),
    [sym_data_max_description_length] = ACTIONS(2129),
    [sym_data_max_type_length] = ACTIONS(2129),
    [sym_data_max_format_length] = ACTIONS(2129),
    [sym_data_max_source_length] = ACTIONS(2129),
    [sym_data_max_value_length] = ACTIONS(2129),
    [sym_data_max_label_length] = ACTIONS(2129),
    [sym_data_max_url_length] = ACTIONS(2129),
    [sym_max_last_login_info_length] = ACTIONS(2129),
    [sym_data_type_unknown] = ACTIONS(2129),
    [sym_data_type_string] = ACTIONS(2129),
    [sym_data_type_datetime] = ACTIONS(2129),
    [sym_data_type_date] = ACTIONS(2129),
    [sym_data_type_time] = ACTIONS(2129),
    [sym_data_type_image] = ACTIONS(2129),
    [sym_data_format_url] = ACTIONS(2129),
    [sym_data_format_phone] = ACTIONS(2129),
    [sym_data_format_email] = ACTIONS(2129),
    [sym_data_format_iso8601] = ACTIONS(2129),
    [sym_auditlog_priv] = ACTIONS(2129),
    [sym_configuration_priv] = ACTIONS(2129),
    [sym_deviceconfig_priv] = ACTIONS(2129),
    [sym_ftp_priv] = ACTIONS(2129),
    [sym_http_priv] = ACTIONS(2129),
    [sym_networkconfig_priv] = ACTIONS(2129),
    [sym_programport_priv] = ACTIONS(2129),
    [sym_remoteui_priv] = ACTIONS(2129),
    [sym_securitycontrol_priv] = ACTIONS(2129),
    [sym_softwaremanagement_priv] = ACTIONS(2129),
    [sym_terminal_priv] = ACTIONS(2129),
    [sym_tpadmin_priv] = ACTIONS(2129),
    [sym_user1_priv] = ACTIONS(2129),
    [sym_user2_priv] = ACTIONS(2129),
    [sym_user3_priv] = ACTIONS(2129),
    [sym_user4_priv] = ACTIONS(2129),
    [sym_usermanagement_priv] = ACTIONS(2129),
    [sym_valid_account] = ACTIONS(2129),
    [sym_err_name_invalid] = ACTIONS(2129),
    [sym_err_password_invalid] = ACTIONS(2129),
    [sym_err_invalid_account] = ACTIONS(2129),
    [sym_err_no_authorization] = ACTIONS(2129),
    [sym_err_invalid_parameter] = ACTIONS(2129),
    [sym_err_user_locked_out] = ACTIONS(2129),
    [sym_err_user_account_expired] = ACTIONS(2129),
    [sym_err_general_error] = ACTIONS(2129),
    [sym_err_authentication_error] = ACTIONS(2129),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2129),
    [sym_tls_validate_certificate] = ACTIONS(2129),
    [sym_tdata] = ACTIONS(2129),
    [sym_tchannel] = ACTIONS(2129),
    [sym_tlevel] = ACTIONS(2129),
    [sym_tbutton] = ACTIONS(2129),
    [sym_ttimeline] = ACTIONS(2129),
    [sym_tcustom] = ACTIONS(2129),
    [sym_url_struct] = ACTIONS(2129),
    [sym_dns_struct] = ACTIONS(2129),
    [sym_ip_address_struct] = ACTIONS(2129),
    [sym_dev_info_struct] = ACTIONS(2129),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2129),
    [sym_data_feed] = ACTIONS(2129),
    [sym_data_field] = ACTIONS(2129),
    [sym_data_record] = ACTIONS(2129),
    [sym_wc_data_feed] = ACTIONS(2129),
    [sym_wc_data_field] = ACTIONS(2129),
    [sym_wc_data_record] = ACTIONS(2129),
    [sym_last_login_info] = ACTIONS(2129),
    [sym____reserved___] = ACTIONS(2129),
    [sym_date] = ACTIONS(2129),
    [sym_day] = ACTIONS(2129),
    [sym_time] = ACTIONS(2129),
    [sym_ldate] = ACTIONS(2129),
    [sym_push_device] = ACTIONS(2129),
    [sym_push_channel] = ACTIONS(2129),
    [sym_push_devchan] = ACTIONS(2129),
    [sym_release_device] = ACTIONS(2129),
    [sym_release_channel] = ACTIONS(2129),
    [sym_release_devchan] = ACTIONS(2129),
    [sym_master_slot] = ACTIONS(2129),
    [sym_get_pulse_time] = ACTIONS(2129),
    [sym_get_timer] = ACTIONS(2129),
    [sym_master_sn] = ACTIONS(2129),
    [sym_system_number] = ACTIONS(2129),
    [sym_button] = ACTIONS(2129),
    [sym_dv_channel] = ACTIONS(2129),
    [sym_data] = ACTIONS(2129),
    [sym_level] = ACTIONS(2129),
    [sym_timeline] = ACTIONS(2129),
    [sym_channel] = ACTIONS(2129),
    [sym_custom] = ACTIONS(2129),
    [sym_atoi] = ACTIONS(2129),
    [sym_atol] = ACTIONS(2129),
    [sym_atof] = ACTIONS(2129),
    [sym_itoa] = ACTIONS(2129),
    [sym_itohex] = ACTIONS(2129),
    [sym_hextoi] = ACTIONS(2129),
    [sym_ftoa] = ACTIONS(2129),
    [sym_find_string] = ACTIONS(2129),
    [sym_left_string] = ACTIONS(2129),
    [sym_right_string] = ACTIONS(2129),
    [sym_mid_string] = ACTIONS(2129),
    [sym_remove_string] = ACTIONS(2129),
    [sym_get_buffer_string] = ACTIONS(2129),
    [sym_get_buffer_char] = ACTIONS(2129),
    [sym_get_multi_buffer_string] = ACTIONS(2129),
    [sym_length_string] = ACTIONS(2129),
    [sym_max_length_string] = ACTIONS(2129),
    [sym_set_length_string] = ACTIONS(2129),
    [sym_compare_string] = ACTIONS(2129),
    [sym_upper_string] = ACTIONS(2129),
    [sym_lower_string] = ACTIONS(2129),
    [sym_get_last] = ACTIONS(2129),
    [sym_date_to_day] = ACTIONS(2129),
    [sym_date_to_month] = ACTIONS(2129),
    [sym_date_to_year] = ACTIONS(2129),
    [sym_time_to_hour] = ACTIONS(2129),
    [sym_time_to_minute] = ACTIONS(2129),
    [sym_time_to_second] = ACTIONS(2129),
    [sym_day_of_week] = ACTIONS(2129),
    [sym_set_pulse_time] = ACTIONS(2129),
    [sym_set_timer] = ACTIONS(2129),
    [sym_file_open] = ACTIONS(2129),
    [sym_file_close] = ACTIONS(2129),
    [sym_file_read] = ACTIONS(2129),
    [sym_file_read_line] = ACTIONS(2129),
    [sym_file_write] = ACTIONS(2129),
    [sym_file_write_line] = ACTIONS(2129),
    [sym_file_copy] = ACTIONS(2129),
    [sym_file_delete] = ACTIONS(2129),
    [sym_file_rename] = ACTIONS(2129),
    [sym_file_dir] = ACTIONS(2129),
    [sym_file_getdir] = ACTIONS(2129),
    [sym_file_setdir] = ACTIONS(2129),
    [sym_file_create_dir] = ACTIONS(2129),
    [sym_file_remove_dir] = ACTIONS(2129),
    [sym_file_seek] = ACTIONS(2129),
    [sym_length_array] = ACTIONS(2129),
    [sym_max_length_array] = ACTIONS(2129),
    [sym_set_length_array] = ACTIONS(2129),
    [sym_ip_client_open] = ACTIONS(2129),
    [sym_ip_client_close] = ACTIONS(2129),
    [sym_ip_server_open] = ACTIONS(2129),
    [sym_ip_server_close] = ACTIONS(2129),
    [sym_ip_bound_client_open] = ACTIONS(2129),
    [sym_ip_mc_server_open] = ACTIONS(2129),
    [sym_ip_set_option] = ACTIONS(2129),
    [sym_ssh_client_open] = ACTIONS(2129),
    [sym_ssh_client_close] = ACTIONS(2129),
    [sym_tls_client_open] = ACTIONS(2129),
    [sym_tls_client_close] = ACTIONS(2129),
    [sym_do_push] = ACTIONS(2129),
    [sym_do_push_timed] = ACTIONS(2129),
    [sym_do_release] = ACTIONS(2129),
    [sym_device_id] = ACTIONS(2129),
    [sym_device_info] = ACTIONS(2129),
    [sym_device_id_string] = ACTIONS(2129),
    [sym_device_standby] = ACTIONS(2129),
    [sym_device_wake] = ACTIONS(2129),
    [sym_timeline_create] = ACTIONS(2129),
    [sym_timeline_kill] = ACTIONS(2129),
    [sym_timeline_reload] = ACTIONS(2129),
    [sym_timeline_pause] = ACTIONS(2129),
    [sym_timeline_restart] = ACTIONS(2129),
    [sym_timeline_set] = ACTIONS(2129),
    [sym_timeline_get] = ACTIONS(2129),
    [sym_timeline_active] = ACTIONS(2129),
    [sym_timeline_dynamic_id] = ACTIONS(2129),
    [sym_variable_to_string] = ACTIONS(2129),
    [sym_string_to_variable] = ACTIONS(2129),
    [sym_length_variable_to_string] = ACTIONS(2129),
    [sym_type_cast] = ACTIONS(2129),
    [sym_variable_to_xml] = ACTIONS(2129),
    [sym_xml_to_variable] = ACTIONS(2129),
    [sym_length_variable_to_xml] = ACTIONS(2129),
    [sym_redirect_string] = ACTIONS(2129),
    [sym_random_number] = ACTIONS(2129),
    [sym_get_unique_id] = ACTIONS(2129),
    [sym_get_system_number] = ACTIONS(2129),
    [sym_set_system_number] = ACTIONS(2129),
    [sym_get_serial_number] = ACTIONS(2129),
    [sym_get_master_build] = ACTIONS(2129),
    [sym_set_validation_code] = ACTIONS(2129),
    [sym_reboot] = ACTIONS(2129),
    [sym_load_duet_module] = ACTIONS(2129),
    [sym_unload_duet_module] = ACTIONS(2129),
    [sym_duet_mem_size_set] = ACTIONS(2129),
    [sym_duet_mem_size_get] = ACTIONS(2129),
    [sym_dynamic_application_device] = ACTIONS(2129),
    [sym_dynamic_polled_port] = ACTIONS(2129),
    [sym_static_port_binding] = ACTIONS(2129),
    [sym_static_ip_binding] = ACTIONS(2129),
    [sym_get_dns_list] = ACTIONS(2129),
    [sym_set_dns_list] = ACTIONS(2129),
    [sym_get_ip_address] = ACTIONS(2129),
    [sym_set_ip_address] = ACTIONS(2129),
    [sym_get_url_list] = ACTIONS(2129),
    [sym_add_url_entry] = ACTIONS(2129),
    [sym_delete_url_entry] = ACTIONS(2129),
    [sym_abs_value] = ACTIONS(2129),
    [sym_max_value] = ACTIONS(2129),
    [sym_min_value] = ACTIONS(2129),
    [sym_exp_value] = ACTIONS(2129),
    [sym_log_value] = ACTIONS(2129),
    [sym_log10_value] = ACTIONS(2129),
    [sym_power_value] = ACTIONS(2129),
    [sym_sqrt_value] = ACTIONS(2129),
    [sym_raw_be] = ACTIONS(2129),
    [sym_raw_le] = ACTIONS(2129),
    [sym_internal_queue_size_set] = ACTIONS(2129),
    [sym_internal_queue_size_get] = ACTIONS(2129),
    [sym_internal_threshold_set] = ACTIONS(2129),
    [sym_internal_threshold_get] = ACTIONS(2129),
    [sym_rebuilt_event] = ACTIONS(2129),
    [sym_do_custom_event] = ACTIONS(2129),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2129),
    [sym_clkmgr_set_clk_source] = ACTIONS(2129),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2129),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2129),
    [sym_clkmgr_get_timezone] = ACTIONS(2129),
    [sym_clkmgr_set_timezone] = ACTIONS(2129),
    [sym_clkmgr_get_resync_period] = ACTIONS(2129),
    [sym_clkmgr_set_resync_period] = ACTIONS(2129),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2129),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2129),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2129),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2129),
    [sym_clkmgr_get_timeservers] = ACTIONS(2129),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2129),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2129),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2129),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2129),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2129),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2129),
    [sym_set_log_level] = ACTIONS(2129),
    [sym_get_log_level] = ACTIONS(2129),
    [sym_amx_log] = ACTIONS(2129),
    [sym_smtp_server_config_set] = ACTIONS(2129),
    [sym_smtp_server_config_get] = ACTIONS(2129),
    [sym_smtp_send] = ACTIONS(2129),
    [sym_validate_netlinx_account] = ACTIONS(2129),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2129),
    [sym_authenticate_certificate] = ACTIONS(2129),
    [sym_audit_netlinx_session_event] = ACTIONS(2129),
    [sym_audit_netlinx_generic_event] = ACTIONS(2129),
    [sym_get_available_flash_disk_space] = ACTIONS(2129),
    [sym_get_max_flash_disk_space] = ACTIONS(2129),
    [sym_data_create_feed] = ACTIONS(2129),
    [sym_data_delete_feed] = ACTIONS(2129),
    [sym_data_publish_feed] = ACTIONS(2129),
    [sym_data_get_published_feed] = ACTIONS(2129),
    [sym_data_add_record] = ACTIONS(2129),
    [sym_data_get_event_record] = ACTIONS(2129),
    [sym__wc_data_create_feed] = ACTIONS(2129),
    [sym__wc_data_add_record] = ACTIONS(2129),
    [sym__wc_data_get_event_record] = ACTIONS(2129),
    [sym_format] = ACTIONS(2129),
    [sym_astro_clock] = ACTIONS(2129),
    [sym_set_outdoor_temperature] = ACTIONS(2129),
    [sym_set_virtual_level_count] = ACTIONS(2129),
    [sym_set_virtual_channel_count] = ACTIONS(2129),
    [sym_set_virtual_port_count] = ACTIONS(2129),
    [sym__wc_encode_int] = ACTIONS(2129),
    [sym__wc_decode_int] = ACTIONS(2129),
    [sym__wc_int] = ACTIONS(2129),
    [sym__wc_to_ch_int] = ACTIONS(2129),
    [sym__ch_to_wc_int] = ACTIONS(2129),
    [sym__wc_find_string_int] = ACTIONS(2129),
    [sym__wc_left_string_int] = ACTIONS(2129),
    [sym__wc_lower_string_int] = ACTIONS(2129),
    [sym__wc_mid_string_int] = ACTIONS(2129),
    [sym__wc_remove_string_int] = ACTIONS(2129),
    [sym__wc_right_string_int] = ACTIONS(2129),
    [sym__wc_upper_string_int] = ACTIONS(2129),
    [sym__wc_compare_string_int] = ACTIONS(2129),
    [sym__wc_get_buffer_char_int] = ACTIONS(2129),
    [sym__wc_get_buffer_string_int] = ACTIONS(2129),
    [sym__wc_concat_string_int] = ACTIONS(2129),
    [sym__wc_file_open_int] = ACTIONS(2129),
    [sym__wc_file_close_int] = ACTIONS(2129),
    [sym__wc_file_read_int] = ACTIONS(2129),
    [sym__wc_file_read_line_int] = ACTIONS(2129),
    [sym__wc_file_write_int] = ACTIONS(2129),
    [sym__wc_file_write_line_int] = ACTIONS(2129),
    [aux_sym_preproc_include_token1] = ACTIONS(2129),
    [aux_sym_preproc_define_token1] = ACTIONS(2131),
    [aux_sym_preproc_warn_token1] = ACTIONS(2131),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2131),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2131),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2131),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2131),
    [aux_sym_preproc_else_token1] = ACTIONS(2131),
    [anon_sym_LBRACK] = ACTIONS(2131),
    [anon_sym_LBRACE] = ACTIONS(2131),
    [anon_sym_LPAREN] = ACTIONS(2129),
    [anon_sym_BANG] = ACTIONS(2131),
    [anon_sym_TILDE] = ACTIONS(2131),
    [anon_sym_DASH] = ACTIONS(2129),
    [anon_sym_PLUS] = ACTIONS(2129),
    [anon_sym_DASH_DASH] = ACTIONS(2131),
    [anon_sym_PLUS_PLUS] = ACTIONS(2131),
    [anon_sym_DQUOTE] = ACTIONS(2131),
    [anon_sym_SQUOTE] = ACTIONS(2131),
    [sym_number_literal] = ACTIONS(2131),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(392)] = {
    [ts_builtin_sym_end] = ACTIONS(1725),
    [sym_identifier] = ACTIONS(1723),
    [sym_program_name_keyword] = ACTIONS(1723),
    [sym_module_name_keyword] = ACTIONS(1723),
    [sym_define_device_keyword] = ACTIONS(1723),
    [sym_define_combine_keyword] = ACTIONS(1723),
    [sym_define_constant_keyword] = ACTIONS(1723),
    [sym_define_type_keyword] = ACTIONS(1723),
    [sym_define_variable_keyword] = ACTIONS(1723),
    [sym_define_system_variable_keyword] = ACTIONS(1723),
    [sym_define_connect_level_keyword] = ACTIONS(1723),
    [sym_define_latching_keyword] = ACTIONS(1723),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1723),
    [sym_define_toggling_keyword] = ACTIONS(1723),
    [sym_define_call_keyword] = ACTIONS(1723),
    [sym_define_function_keyword] = ACTIONS(1723),
    [sym_define_library_function_keyword] = ACTIONS(1723),
    [sym_define_start_keyword] = ACTIONS(1723),
    [sym_define_event_keyword] = ACTIONS(1723),
    [sym_define_program_keyword] = ACTIONS(1723),
    [sym_define_module_keyword] = ACTIONS(1723),
    [sym_char_keyword] = ACTIONS(1723),
    [sym_widechar_keyword] = ACTIONS(1723),
    [sym_integer_keyword] = ACTIONS(1723),
    [sym_sinteger_keyword] = ACTIONS(1723),
    [sym_long_keyword] = ACTIONS(1723),
    [sym_slong_keyword] = ACTIONS(1723),
    [sym_float_keyword] = ACTIONS(1723),
    [sym_double_keyword] = ACTIONS(1723),
    [sym_variant_keyword] = ACTIONS(1723),
    [sym_variantarray_keyword] = ACTIONS(1723),
    [sym_local_var_keyword] = ACTIONS(1723),
    [sym_stack_var_keyword] = ACTIONS(1723),
    [sym_constant_keyword] = ACTIONS(1723),
    [sym_volatile_keyword] = ACTIONS(1723),
    [sym_non_volatile_keyword] = ACTIONS(1723),
    [sym_persistent_keyword] = ACTIONS(1723),
    [sym_dev_keyword] = ACTIONS(1723),
    [sym_devlev_keyword] = ACTIONS(1723),
    [sym_devchan_keyword] = ACTIONS(1723),
    [sym_if_keyword] = ACTIONS(1723),
    [sym_else_keyword] = ACTIONS(1723),
    [sym_switch_keyword] = ACTIONS(1723),
    [sym_case_keyword] = ACTIONS(1723),
    [sym_default_keyword] = ACTIONS(1723),
    [sym_while_keyword] = ACTIONS(1723),
    [sym_for_keyword] = ACTIONS(1723),
    [sym_break_keyword] = ACTIONS(1723),
    [sym_continue_keyword] = ACTIONS(1723),
    [sym_return_keyword] = ACTIONS(1723),
    [sym_select_keyword] = ACTIONS(1723),
    [sym_active_keyword] = ACTIONS(1723),
    [sym_struct_keyword] = ACTIONS(1723),
    [sym_structure_keyword] = ACTIONS(1723),
    [sym_button_event_keyword] = ACTIONS(1723),
    [sym_channel_event_keyword] = ACTIONS(1723),
    [sym_level_event_keyword] = ACTIONS(1723),
    [sym_data_event_keyword] = ACTIONS(1723),
    [sym_timeline_event_keyword] = ACTIONS(1723),
    [sym_custom_event_keyword] = ACTIONS(1723),
    [sym_push_keyword] = ACTIONS(1723),
    [sym_release_keyword] = ACTIONS(1723),
    [aux_sym_on_keyword_token1] = ACTIONS(1723),
    [aux_sym_off_keyword_token1] = ACTIONS(1723),
    [sym_send_level_keyword] = ACTIONS(1723),
    [sym_send_string_keyword] = ACTIONS(1723),
    [sym_send_command_keyword] = ACTIONS(1723),
    [sym_clear_buffer_keyword] = ACTIONS(1723),
    [sym_create_buffer_keyword] = ACTIONS(1723),
    [sym_create_multi_buffer_keyword] = ACTIONS(1723),
    [sym_call_keyword] = ACTIONS(1723),
    [sym_system_call_keyword] = ACTIONS(1723),
    [sym_devchan_to_keyword] = ACTIONS(1723),
    [sym_devchan_min_to_keyword] = ACTIONS(1723),
    [sym_devchan_total_off_keyword] = ACTIONS(1723),
    [sym_devchan_pulse_keyword] = ACTIONS(1723),
    [sym_wait_keyword] = ACTIONS(1723),
    [sym_cancel_wait_keyword] = ACTIONS(1723),
    [sym_wait_until_keyword] = ACTIONS(1723),
    [sym_cancel_wait_until_keyword] = ACTIONS(1723),
    [sym_cancel_all_wait_keyword] = ACTIONS(1723),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1723),
    [sym_not] = ACTIONS(1723),
    [sym_bnot] = ACTIONS(1723),
    [sym___file__] = ACTIONS(1723),
    [sym___date__] = ACTIONS(1723),
    [sym___ldate__] = ACTIONS(1723),
    [sym___time__] = ACTIONS(1723),
    [sym___line__] = ACTIONS(1723),
    [sym___name__] = ACTIONS(1723),
    [sym___netlinx__] = ACTIONS(1723),
    [sym_netlinx_axi_version] = ACTIONS(1723),
    [sym_url_flg_tcp] = ACTIONS(1723),
    [sym_url_flg_accinfopresent] = ACTIONS(1723),
    [sym_url_flg_temp] = ACTIONS(1723),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1723),
    [sym_url_flg_stat_mask] = ACTIONS(1723),
    [sym_url_flg_stat_lookup] = ACTIONS(1723),
    [sym_url_flg_stat_connecting] = ACTIONS(1723),
    [sym_url_flg_stat_waiting] = ACTIONS(1723),
    [sym_url_flg_stat_connected] = ACTIONS(1723),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1723),
    [sym_first_virtual_device] = ACTIONS(1723),
    [sym_first_local_port] = ACTIONS(1723),
    [sym_dynamic_virtual_device] = ACTIONS(1723),
    [sym_timeline_once] = ACTIONS(1723),
    [sym_timeline_repeat] = ACTIONS(1723),
    [sym_timeline_absolute] = ACTIONS(1723),
    [sym_timeline_relative] = ACTIONS(1723),
    [sym_do_push_timed_infinite] = ACTIONS(1723),
    [sym_source_type_no_address] = ACTIONS(1723),
    [sym_source_type_neuron_id] = ACTIONS(1723),
    [sym_source_type_ip_address] = ACTIONS(1723),
    [sym_source_type_axlink] = ACTIONS(1723),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1723),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1723),
    [sym_source_type_ip_socket_address] = ACTIONS(1723),
    [sym_source_type_rs232] = ACTIONS(1723),
    [sym_source_type_internal] = ACTIONS(1723),
    [sym_source_type_mac_address] = ACTIONS(1723),
    [sym_source_type_ipv4_port] = ACTIONS(1723),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1723),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1723),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1723),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1723),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1723),
    [sym_true] = ACTIONS(1723),
    [sym_false] = ACTIONS(1723),
    [sym_file_read_only] = ACTIONS(1723),
    [sym_file_rw_new] = ACTIONS(1723),
    [sym_file_rw_append] = ACTIONS(1723),
    [sym_ip_tcp] = ACTIONS(1723),
    [sym_ip_udp] = ACTIONS(1723),
    [sym_ip_udp_2way] = ACTIONS(1723),
    [sym_xml_encode_types] = ACTIONS(1723),
    [sym_xml_encode_char_as_list] = ACTIONS(1723),
    [sym_xml_encode_le] = ACTIONS(1723),
    [sym_xml_decode_types] = ACTIONS(1723),
    [sym_xml_decode_no_preserve] = ACTIONS(1723),
    [sym_duet_dev_type_display_device] = ACTIONS(1723),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1723),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1723),
    [sym_duet_dev_type_io_device] = ACTIONS(1723),
    [sym_duet_dev_type_relay_device] = ACTIONS(1723),
    [sym_duet_dev_type_ups] = ACTIONS(1723),
    [sym_duet_dev_type_amplifier] = ACTIONS(1723),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1723),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1723),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1723),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1723),
    [sym_duet_dev_type_camera] = ACTIONS(1723),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1723),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1723),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1723),
    [sym_duet_dev_type_dss] = ACTIONS(1723),
    [sym_duet_dev_type_dvr] = ACTIONS(1723),
    [sym_duet_dev_type_disc_device] = ACTIONS(1723),
    [sym_duet_dev_type_document_camera] = ACTIONS(1723),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1723),
    [sym_duet_dev_type_hvac] = ACTIONS(1723),
    [sym_duet_dev_type_keypad] = ACTIONS(1723),
    [sym_duet_dev_type_light] = ACTIONS(1723),
    [sym_duet_dev_type_monitor] = ACTIONS(1723),
    [sym_duet_dev_type_motor] = ACTIONS(1723),
    [sym_duet_dev_type_multi_window] = ACTIONS(1723),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1723),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1723),
    [sym_duet_dev_type_receiver] = ACTIONS(1723),
    [sym_duet_dev_type_security_system] = ACTIONS(1723),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1723),
    [sym_duet_dev_type_settop_box] = ACTIONS(1723),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1723),
    [sym_duet_dev_type_switcher] = ACTIONS(1723),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1723),
    [sym_duet_dev_type_tv] = ACTIONS(1723),
    [sym_duet_dev_type_utility] = ACTIONS(1723),
    [sym_duet_dev_type_vcr] = ACTIONS(1723),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1723),
    [sym_duet_dev_type_video_processor] = ACTIONS(1723),
    [sym_duet_dev_type_video_projector] = ACTIONS(1723),
    [sym_duet_dev_type_video_wall] = ACTIONS(1723),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1723),
    [sym_duet_dev_type_weather] = ACTIONS(1723),
    [sym_duet_dev_not_polled] = ACTIONS(1723),
    [sym_duet_dev_polled] = ACTIONS(1723),
    [sym_clkmgr_mode_network] = ACTIONS(1723),
    [sym_clkmgr_mode_standalone] = ACTIONS(1723),
    [sym_normal_standby] = ACTIONS(1723),
    [sym_normal_wake] = ACTIONS(1723),
    [sym_ip_multicast_ttl_option] = ACTIONS(1723),
    [sym_ip_ttl_subnet] = ACTIONS(1723),
    [sym_ip_ttl_site] = ACTIONS(1723),
    [sym_ip_ttl_region] = ACTIONS(1723),
    [sym_ip_ttl_continent] = ACTIONS(1723),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1723),
    [sym_ip_nodelay_on] = ACTIONS(1723),
    [sym_ip_nodelay_off] = ACTIONS(1723),
    [sym_amx_error] = ACTIONS(1723),
    [sym_amx_warning] = ACTIONS(1723),
    [sym_amx_info] = ACTIONS(1723),
    [sym_amx_debug] = ACTIONS(1723),
    [sym_netlinx_login_success] = ACTIONS(1723),
    [sym_netlinx_login_fail] = ACTIONS(1723),
    [sym_netlinx_logout] = ACTIONS(1723),
    [sym_null_str] = ACTIONS(1723),
    [sym_smtp_address] = ACTIONS(1723),
    [sym_smtp_port_number] = ACTIONS(1723),
    [sym_smtp_username] = ACTIONS(1723),
    [sym_smtp_password] = ACTIONS(1723),
    [sym_smtp_require_tls] = ACTIONS(1723),
    [sym_smtp_from] = ACTIONS(1723),
    [sym_smtp_tls_true] = ACTIONS(1723),
    [sym_smtp_tls_false] = ACTIONS(1723),
    [sym_listview_on_row_select_event] = ACTIONS(1723),
    [sym_data_structure_unknown] = ACTIONS(1723),
    [sym_data_structure_datafeed] = ACTIONS(1723),
    [sym_data_structure_datarecordset] = ACTIONS(1723),
    [sym_data_structure_datarecord] = ACTIONS(1723),
    [sym_data_structure_datafield] = ACTIONS(1723),
    [sym_data_max_content_field_count] = ACTIONS(1723),
    [sym_data_max_metadata_field_count] = ACTIONS(1723),
    [sym_data_max_id_length] = ACTIONS(1723),
    [sym_data_max_name_length] = ACTIONS(1723),
    [sym_data_max_description_length] = ACTIONS(1723),
    [sym_data_max_type_length] = ACTIONS(1723),
    [sym_data_max_format_length] = ACTIONS(1723),
    [sym_data_max_source_length] = ACTIONS(1723),
    [sym_data_max_value_length] = ACTIONS(1723),
    [sym_data_max_label_length] = ACTIONS(1723),
    [sym_data_max_url_length] = ACTIONS(1723),
    [sym_max_last_login_info_length] = ACTIONS(1723),
    [sym_data_type_unknown] = ACTIONS(1723),
    [sym_data_type_string] = ACTIONS(1723),
    [sym_data_type_datetime] = ACTIONS(1723),
    [sym_data_type_date] = ACTIONS(1723),
    [sym_data_type_time] = ACTIONS(1723),
    [sym_data_type_image] = ACTIONS(1723),
    [sym_data_format_url] = ACTIONS(1723),
    [sym_data_format_phone] = ACTIONS(1723),
    [sym_data_format_email] = ACTIONS(1723),
    [sym_data_format_iso8601] = ACTIONS(1723),
    [sym_auditlog_priv] = ACTIONS(1723),
    [sym_configuration_priv] = ACTIONS(1723),
    [sym_deviceconfig_priv] = ACTIONS(1723),
    [sym_ftp_priv] = ACTIONS(1723),
    [sym_http_priv] = ACTIONS(1723),
    [sym_networkconfig_priv] = ACTIONS(1723),
    [sym_programport_priv] = ACTIONS(1723),
    [sym_remoteui_priv] = ACTIONS(1723),
    [sym_securitycontrol_priv] = ACTIONS(1723),
    [sym_softwaremanagement_priv] = ACTIONS(1723),
    [sym_terminal_priv] = ACTIONS(1723),
    [sym_tpadmin_priv] = ACTIONS(1723),
    [sym_user1_priv] = ACTIONS(1723),
    [sym_user2_priv] = ACTIONS(1723),
    [sym_user3_priv] = ACTIONS(1723),
    [sym_user4_priv] = ACTIONS(1723),
    [sym_usermanagement_priv] = ACTIONS(1723),
    [sym_valid_account] = ACTIONS(1723),
    [sym_err_name_invalid] = ACTIONS(1723),
    [sym_err_password_invalid] = ACTIONS(1723),
    [sym_err_invalid_account] = ACTIONS(1723),
    [sym_err_no_authorization] = ACTIONS(1723),
    [sym_err_invalid_parameter] = ACTIONS(1723),
    [sym_err_user_locked_out] = ACTIONS(1723),
    [sym_err_user_account_expired] = ACTIONS(1723),
    [sym_err_general_error] = ACTIONS(1723),
    [sym_err_authentication_error] = ACTIONS(1723),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1723),
    [sym_tls_validate_certificate] = ACTIONS(1723),
    [sym_tdata] = ACTIONS(1723),
    [sym_tchannel] = ACTIONS(1723),
    [sym_tlevel] = ACTIONS(1723),
    [sym_tbutton] = ACTIONS(1723),
    [sym_ttimeline] = ACTIONS(1723),
    [sym_tcustom] = ACTIONS(1723),
    [sym_url_struct] = ACTIONS(1723),
    [sym_dns_struct] = ACTIONS(1723),
    [sym_ip_address_struct] = ACTIONS(1723),
    [sym_dev_info_struct] = ACTIONS(1723),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1723),
    [sym_data_feed] = ACTIONS(1723),
    [sym_data_field] = ACTIONS(1723),
    [sym_data_record] = ACTIONS(1723),
    [sym_wc_data_feed] = ACTIONS(1723),
    [sym_wc_data_field] = ACTIONS(1723),
    [sym_wc_data_record] = ACTIONS(1723),
    [sym_last_login_info] = ACTIONS(1723),
    [sym____reserved___] = ACTIONS(1723),
    [sym_date] = ACTIONS(1723),
    [sym_day] = ACTIONS(1723),
    [sym_time] = ACTIONS(1723),
    [sym_ldate] = ACTIONS(1723),
    [sym_push_device] = ACTIONS(1723),
    [sym_push_channel] = ACTIONS(1723),
    [sym_push_devchan] = ACTIONS(1723),
    [sym_release_device] = ACTIONS(1723),
    [sym_release_channel] = ACTIONS(1723),
    [sym_release_devchan] = ACTIONS(1723),
    [sym_master_slot] = ACTIONS(1723),
    [sym_get_pulse_time] = ACTIONS(1723),
    [sym_get_timer] = ACTIONS(1723),
    [sym_master_sn] = ACTIONS(1723),
    [sym_system_number] = ACTIONS(1723),
    [sym_button] = ACTIONS(1723),
    [sym_dv_channel] = ACTIONS(1723),
    [sym_data] = ACTIONS(1723),
    [sym_level] = ACTIONS(1723),
    [sym_timeline] = ACTIONS(1723),
    [sym_channel] = ACTIONS(1723),
    [sym_custom] = ACTIONS(1723),
    [sym_atoi] = ACTIONS(1723),
    [sym_atol] = ACTIONS(1723),
    [sym_atof] = ACTIONS(1723),
    [sym_itoa] = ACTIONS(1723),
    [sym_itohex] = ACTIONS(1723),
    [sym_hextoi] = ACTIONS(1723),
    [sym_ftoa] = ACTIONS(1723),
    [sym_find_string] = ACTIONS(1723),
    [sym_left_string] = ACTIONS(1723),
    [sym_right_string] = ACTIONS(1723),
    [sym_mid_string] = ACTIONS(1723),
    [sym_remove_string] = ACTIONS(1723),
    [sym_get_buffer_string] = ACTIONS(1723),
    [sym_get_buffer_char] = ACTIONS(1723),
    [sym_get_multi_buffer_string] = ACTIONS(1723),
    [sym_length_string] = ACTIONS(1723),
    [sym_max_length_string] = ACTIONS(1723),
    [sym_set_length_string] = ACTIONS(1723),
    [sym_compare_string] = ACTIONS(1723),
    [sym_upper_string] = ACTIONS(1723),
    [sym_lower_string] = ACTIONS(1723),
    [sym_get_last] = ACTIONS(1723),
    [sym_date_to_day] = ACTIONS(1723),
    [sym_date_to_month] = ACTIONS(1723),
    [sym_date_to_year] = ACTIONS(1723),
    [sym_time_to_hour] = ACTIONS(1723),
    [sym_time_to_minute] = ACTIONS(1723),
    [sym_time_to_second] = ACTIONS(1723),
    [sym_day_of_week] = ACTIONS(1723),
    [sym_set_pulse_time] = ACTIONS(1723),
    [sym_set_timer] = ACTIONS(1723),
    [sym_file_open] = ACTIONS(1723),
    [sym_file_close] = ACTIONS(1723),
    [sym_file_read] = ACTIONS(1723),
    [sym_file_read_line] = ACTIONS(1723),
    [sym_file_write] = ACTIONS(1723),
    [sym_file_write_line] = ACTIONS(1723),
    [sym_file_copy] = ACTIONS(1723),
    [sym_file_delete] = ACTIONS(1723),
    [sym_file_rename] = ACTIONS(1723),
    [sym_file_dir] = ACTIONS(1723),
    [sym_file_getdir] = ACTIONS(1723),
    [sym_file_setdir] = ACTIONS(1723),
    [sym_file_create_dir] = ACTIONS(1723),
    [sym_file_remove_dir] = ACTIONS(1723),
    [sym_file_seek] = ACTIONS(1723),
    [sym_length_array] = ACTIONS(1723),
    [sym_max_length_array] = ACTIONS(1723),
    [sym_set_length_array] = ACTIONS(1723),
    [sym_ip_client_open] = ACTIONS(1723),
    [sym_ip_client_close] = ACTIONS(1723),
    [sym_ip_server_open] = ACTIONS(1723),
    [sym_ip_server_close] = ACTIONS(1723),
    [sym_ip_bound_client_open] = ACTIONS(1723),
    [sym_ip_mc_server_open] = ACTIONS(1723),
    [sym_ip_set_option] = ACTIONS(1723),
    [sym_ssh_client_open] = ACTIONS(1723),
    [sym_ssh_client_close] = ACTIONS(1723),
    [sym_tls_client_open] = ACTIONS(1723),
    [sym_tls_client_close] = ACTIONS(1723),
    [sym_do_push] = ACTIONS(1723),
    [sym_do_push_timed] = ACTIONS(1723),
    [sym_do_release] = ACTIONS(1723),
    [sym_device_id] = ACTIONS(1723),
    [sym_device_info] = ACTIONS(1723),
    [sym_device_id_string] = ACTIONS(1723),
    [sym_device_standby] = ACTIONS(1723),
    [sym_device_wake] = ACTIONS(1723),
    [sym_timeline_create] = ACTIONS(1723),
    [sym_timeline_kill] = ACTIONS(1723),
    [sym_timeline_reload] = ACTIONS(1723),
    [sym_timeline_pause] = ACTIONS(1723),
    [sym_timeline_restart] = ACTIONS(1723),
    [sym_timeline_set] = ACTIONS(1723),
    [sym_timeline_get] = ACTIONS(1723),
    [sym_timeline_active] = ACTIONS(1723),
    [sym_timeline_dynamic_id] = ACTIONS(1723),
    [sym_variable_to_string] = ACTIONS(1723),
    [sym_string_to_variable] = ACTIONS(1723),
    [sym_length_variable_to_string] = ACTIONS(1723),
    [sym_type_cast] = ACTIONS(1723),
    [sym_variable_to_xml] = ACTIONS(1723),
    [sym_xml_to_variable] = ACTIONS(1723),
    [sym_length_variable_to_xml] = ACTIONS(1723),
    [sym_redirect_string] = ACTIONS(1723),
    [sym_random_number] = ACTIONS(1723),
    [sym_get_unique_id] = ACTIONS(1723),
    [sym_get_system_number] = ACTIONS(1723),
    [sym_set_system_number] = ACTIONS(1723),
    [sym_get_serial_number] = ACTIONS(1723),
    [sym_get_master_build] = ACTIONS(1723),
    [sym_set_validation_code] = ACTIONS(1723),
    [sym_reboot] = ACTIONS(1723),
    [sym_load_duet_module] = ACTIONS(1723),
    [sym_unload_duet_module] = ACTIONS(1723),
    [sym_duet_mem_size_set] = ACTIONS(1723),
    [sym_duet_mem_size_get] = ACTIONS(1723),
    [sym_dynamic_application_device] = ACTIONS(1723),
    [sym_dynamic_polled_port] = ACTIONS(1723),
    [sym_static_port_binding] = ACTIONS(1723),
    [sym_static_ip_binding] = ACTIONS(1723),
    [sym_get_dns_list] = ACTIONS(1723),
    [sym_set_dns_list] = ACTIONS(1723),
    [sym_get_ip_address] = ACTIONS(1723),
    [sym_set_ip_address] = ACTIONS(1723),
    [sym_get_url_list] = ACTIONS(1723),
    [sym_add_url_entry] = ACTIONS(1723),
    [sym_delete_url_entry] = ACTIONS(1723),
    [sym_abs_value] = ACTIONS(1723),
    [sym_max_value] = ACTIONS(1723),
    [sym_min_value] = ACTIONS(1723),
    [sym_exp_value] = ACTIONS(1723),
    [sym_log_value] = ACTIONS(1723),
    [sym_log10_value] = ACTIONS(1723),
    [sym_power_value] = ACTIONS(1723),
    [sym_sqrt_value] = ACTIONS(1723),
    [sym_raw_be] = ACTIONS(1723),
    [sym_raw_le] = ACTIONS(1723),
    [sym_internal_queue_size_set] = ACTIONS(1723),
    [sym_internal_queue_size_get] = ACTIONS(1723),
    [sym_internal_threshold_set] = ACTIONS(1723),
    [sym_internal_threshold_get] = ACTIONS(1723),
    [sym_rebuilt_event] = ACTIONS(1723),
    [sym_do_custom_event] = ACTIONS(1723),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1723),
    [sym_clkmgr_set_clk_source] = ACTIONS(1723),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1723),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1723),
    [sym_clkmgr_get_timezone] = ACTIONS(1723),
    [sym_clkmgr_set_timezone] = ACTIONS(1723),
    [sym_clkmgr_get_resync_period] = ACTIONS(1723),
    [sym_clkmgr_set_resync_period] = ACTIONS(1723),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1723),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1723),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1723),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1723),
    [sym_clkmgr_get_timeservers] = ACTIONS(1723),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1723),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1723),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1723),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1723),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1723),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1723),
    [sym_set_log_level] = ACTIONS(1723),
    [sym_get_log_level] = ACTIONS(1723),
    [sym_amx_log] = ACTIONS(1723),
    [sym_smtp_server_config_set] = ACTIONS(1723),
    [sym_smtp_server_config_get] = ACTIONS(1723),
    [sym_smtp_send] = ACTIONS(1723),
    [sym_validate_netlinx_account] = ACTIONS(1723),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1723),
    [sym_authenticate_certificate] = ACTIONS(1723),
    [sym_audit_netlinx_session_event] = ACTIONS(1723),
    [sym_audit_netlinx_generic_event] = ACTIONS(1723),
    [sym_get_available_flash_disk_space] = ACTIONS(1723),
    [sym_get_max_flash_disk_space] = ACTIONS(1723),
    [sym_data_create_feed] = ACTIONS(1723),
    [sym_data_delete_feed] = ACTIONS(1723),
    [sym_data_publish_feed] = ACTIONS(1723),
    [sym_data_get_published_feed] = ACTIONS(1723),
    [sym_data_add_record] = ACTIONS(1723),
    [sym_data_get_event_record] = ACTIONS(1723),
    [sym__wc_data_create_feed] = ACTIONS(1723),
    [sym__wc_data_add_record] = ACTIONS(1723),
    [sym__wc_data_get_event_record] = ACTIONS(1723),
    [sym_format] = ACTIONS(1723),
    [sym_astro_clock] = ACTIONS(1723),
    [sym_set_outdoor_temperature] = ACTIONS(1723),
    [sym_set_virtual_level_count] = ACTIONS(1723),
    [sym_set_virtual_channel_count] = ACTIONS(1723),
    [sym_set_virtual_port_count] = ACTIONS(1723),
    [sym__wc_encode_int] = ACTIONS(1723),
    [sym__wc_decode_int] = ACTIONS(1723),
    [sym__wc_int] = ACTIONS(1723),
    [sym__wc_to_ch_int] = ACTIONS(1723),
    [sym__ch_to_wc_int] = ACTIONS(1723),
    [sym__wc_find_string_int] = ACTIONS(1723),
    [sym__wc_left_string_int] = ACTIONS(1723),
    [sym__wc_lower_string_int] = ACTIONS(1723),
    [sym__wc_mid_string_int] = ACTIONS(1723),
    [sym__wc_remove_string_int] = ACTIONS(1723),
    [sym__wc_right_string_int] = ACTIONS(1723),
    [sym__wc_upper_string_int] = ACTIONS(1723),
    [sym__wc_compare_string_int] = ACTIONS(1723),
    [sym__wc_get_buffer_char_int] = ACTIONS(1723),
    [sym__wc_get_buffer_string_int] = ACTIONS(1723),
    [sym__wc_concat_string_int] = ACTIONS(1723),
    [sym__wc_file_open_int] = ACTIONS(1723),
    [sym__wc_file_close_int] = ACTIONS(1723),
    [sym__wc_file_read_int] = ACTIONS(1723),
    [sym__wc_file_read_line_int] = ACTIONS(1723),
    [sym__wc_file_write_int] = ACTIONS(1723),
    [sym__wc_file_write_line_int] = ACTIONS(1723),
    [aux_sym_preproc_include_token1] = ACTIONS(1723),
    [aux_sym_preproc_define_token1] = ACTIONS(1725),
    [aux_sym_preproc_warn_token1] = ACTIONS(1725),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1725),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1725),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1725),
    [anon_sym_LBRACK] = ACTIONS(1725),
    [anon_sym_LBRACE] = ACTIONS(1725),
    [anon_sym_LPAREN] = ACTIONS(1723),
    [anon_sym_BANG] = ACTIONS(1725),
    [anon_sym_TILDE] = ACTIONS(1725),
    [anon_sym_DASH] = ACTIONS(1723),
    [anon_sym_PLUS] = ACTIONS(1723),
    [anon_sym_DASH_DASH] = ACTIONS(1725),
    [anon_sym_PLUS_PLUS] = ACTIONS(1725),
    [anon_sym_DQUOTE] = ACTIONS(1725),
    [anon_sym_SQUOTE] = ACTIONS(1725),
    [sym_number_literal] = ACTIONS(1725),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(393)] = {
    [ts_builtin_sym_end] = ACTIONS(1729),
    [sym_identifier] = ACTIONS(1727),
    [sym_program_name_keyword] = ACTIONS(1727),
    [sym_module_name_keyword] = ACTIONS(1727),
    [sym_define_device_keyword] = ACTIONS(1727),
    [sym_define_combine_keyword] = ACTIONS(1727),
    [sym_define_constant_keyword] = ACTIONS(1727),
    [sym_define_type_keyword] = ACTIONS(1727),
    [sym_define_variable_keyword] = ACTIONS(1727),
    [sym_define_system_variable_keyword] = ACTIONS(1727),
    [sym_define_connect_level_keyword] = ACTIONS(1727),
    [sym_define_latching_keyword] = ACTIONS(1727),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1727),
    [sym_define_toggling_keyword] = ACTIONS(1727),
    [sym_define_call_keyword] = ACTIONS(1727),
    [sym_define_function_keyword] = ACTIONS(1727),
    [sym_define_library_function_keyword] = ACTIONS(1727),
    [sym_define_start_keyword] = ACTIONS(1727),
    [sym_define_event_keyword] = ACTIONS(1727),
    [sym_define_program_keyword] = ACTIONS(1727),
    [sym_define_module_keyword] = ACTIONS(1727),
    [sym_char_keyword] = ACTIONS(1727),
    [sym_widechar_keyword] = ACTIONS(1727),
    [sym_integer_keyword] = ACTIONS(1727),
    [sym_sinteger_keyword] = ACTIONS(1727),
    [sym_long_keyword] = ACTIONS(1727),
    [sym_slong_keyword] = ACTIONS(1727),
    [sym_float_keyword] = ACTIONS(1727),
    [sym_double_keyword] = ACTIONS(1727),
    [sym_variant_keyword] = ACTIONS(1727),
    [sym_variantarray_keyword] = ACTIONS(1727),
    [sym_local_var_keyword] = ACTIONS(1727),
    [sym_stack_var_keyword] = ACTIONS(1727),
    [sym_constant_keyword] = ACTIONS(1727),
    [sym_volatile_keyword] = ACTIONS(1727),
    [sym_non_volatile_keyword] = ACTIONS(1727),
    [sym_persistent_keyword] = ACTIONS(1727),
    [sym_dev_keyword] = ACTIONS(1727),
    [sym_devlev_keyword] = ACTIONS(1727),
    [sym_devchan_keyword] = ACTIONS(1727),
    [sym_if_keyword] = ACTIONS(1727),
    [sym_else_keyword] = ACTIONS(1727),
    [sym_switch_keyword] = ACTIONS(1727),
    [sym_case_keyword] = ACTIONS(1727),
    [sym_default_keyword] = ACTIONS(1727),
    [sym_while_keyword] = ACTIONS(1727),
    [sym_for_keyword] = ACTIONS(1727),
    [sym_break_keyword] = ACTIONS(1727),
    [sym_continue_keyword] = ACTIONS(1727),
    [sym_return_keyword] = ACTIONS(1727),
    [sym_select_keyword] = ACTIONS(1727),
    [sym_active_keyword] = ACTIONS(1727),
    [sym_struct_keyword] = ACTIONS(1727),
    [sym_structure_keyword] = ACTIONS(1727),
    [sym_button_event_keyword] = ACTIONS(1727),
    [sym_channel_event_keyword] = ACTIONS(1727),
    [sym_level_event_keyword] = ACTIONS(1727),
    [sym_data_event_keyword] = ACTIONS(1727),
    [sym_timeline_event_keyword] = ACTIONS(1727),
    [sym_custom_event_keyword] = ACTIONS(1727),
    [sym_push_keyword] = ACTIONS(1727),
    [sym_release_keyword] = ACTIONS(1727),
    [aux_sym_on_keyword_token1] = ACTIONS(1727),
    [aux_sym_off_keyword_token1] = ACTIONS(1727),
    [sym_send_level_keyword] = ACTIONS(1727),
    [sym_send_string_keyword] = ACTIONS(1727),
    [sym_send_command_keyword] = ACTIONS(1727),
    [sym_clear_buffer_keyword] = ACTIONS(1727),
    [sym_create_buffer_keyword] = ACTIONS(1727),
    [sym_create_multi_buffer_keyword] = ACTIONS(1727),
    [sym_call_keyword] = ACTIONS(1727),
    [sym_system_call_keyword] = ACTIONS(1727),
    [sym_devchan_to_keyword] = ACTIONS(1727),
    [sym_devchan_min_to_keyword] = ACTIONS(1727),
    [sym_devchan_total_off_keyword] = ACTIONS(1727),
    [sym_devchan_pulse_keyword] = ACTIONS(1727),
    [sym_wait_keyword] = ACTIONS(1727),
    [sym_cancel_wait_keyword] = ACTIONS(1727),
    [sym_wait_until_keyword] = ACTIONS(1727),
    [sym_cancel_wait_until_keyword] = ACTIONS(1727),
    [sym_cancel_all_wait_keyword] = ACTIONS(1727),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1727),
    [sym_not] = ACTIONS(1727),
    [sym_bnot] = ACTIONS(1727),
    [sym___file__] = ACTIONS(1727),
    [sym___date__] = ACTIONS(1727),
    [sym___ldate__] = ACTIONS(1727),
    [sym___time__] = ACTIONS(1727),
    [sym___line__] = ACTIONS(1727),
    [sym___name__] = ACTIONS(1727),
    [sym___netlinx__] = ACTIONS(1727),
    [sym_netlinx_axi_version] = ACTIONS(1727),
    [sym_url_flg_tcp] = ACTIONS(1727),
    [sym_url_flg_accinfopresent] = ACTIONS(1727),
    [sym_url_flg_temp] = ACTIONS(1727),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1727),
    [sym_url_flg_stat_mask] = ACTIONS(1727),
    [sym_url_flg_stat_lookup] = ACTIONS(1727),
    [sym_url_flg_stat_connecting] = ACTIONS(1727),
    [sym_url_flg_stat_waiting] = ACTIONS(1727),
    [sym_url_flg_stat_connected] = ACTIONS(1727),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1727),
    [sym_first_virtual_device] = ACTIONS(1727),
    [sym_first_local_port] = ACTIONS(1727),
    [sym_dynamic_virtual_device] = ACTIONS(1727),
    [sym_timeline_once] = ACTIONS(1727),
    [sym_timeline_repeat] = ACTIONS(1727),
    [sym_timeline_absolute] = ACTIONS(1727),
    [sym_timeline_relative] = ACTIONS(1727),
    [sym_do_push_timed_infinite] = ACTIONS(1727),
    [sym_source_type_no_address] = ACTIONS(1727),
    [sym_source_type_neuron_id] = ACTIONS(1727),
    [sym_source_type_ip_address] = ACTIONS(1727),
    [sym_source_type_axlink] = ACTIONS(1727),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1727),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1727),
    [sym_source_type_ip_socket_address] = ACTIONS(1727),
    [sym_source_type_rs232] = ACTIONS(1727),
    [sym_source_type_internal] = ACTIONS(1727),
    [sym_source_type_mac_address] = ACTIONS(1727),
    [sym_source_type_ipv4_port] = ACTIONS(1727),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1727),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1727),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1727),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1727),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1727),
    [sym_true] = ACTIONS(1727),
    [sym_false] = ACTIONS(1727),
    [sym_file_read_only] = ACTIONS(1727),
    [sym_file_rw_new] = ACTIONS(1727),
    [sym_file_rw_append] = ACTIONS(1727),
    [sym_ip_tcp] = ACTIONS(1727),
    [sym_ip_udp] = ACTIONS(1727),
    [sym_ip_udp_2way] = ACTIONS(1727),
    [sym_xml_encode_types] = ACTIONS(1727),
    [sym_xml_encode_char_as_list] = ACTIONS(1727),
    [sym_xml_encode_le] = ACTIONS(1727),
    [sym_xml_decode_types] = ACTIONS(1727),
    [sym_xml_decode_no_preserve] = ACTIONS(1727),
    [sym_duet_dev_type_display_device] = ACTIONS(1727),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1727),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1727),
    [sym_duet_dev_type_io_device] = ACTIONS(1727),
    [sym_duet_dev_type_relay_device] = ACTIONS(1727),
    [sym_duet_dev_type_ups] = ACTIONS(1727),
    [sym_duet_dev_type_amplifier] = ACTIONS(1727),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1727),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1727),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1727),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1727),
    [sym_duet_dev_type_camera] = ACTIONS(1727),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1727),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1727),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1727),
    [sym_duet_dev_type_dss] = ACTIONS(1727),
    [sym_duet_dev_type_dvr] = ACTIONS(1727),
    [sym_duet_dev_type_disc_device] = ACTIONS(1727),
    [sym_duet_dev_type_document_camera] = ACTIONS(1727),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1727),
    [sym_duet_dev_type_hvac] = ACTIONS(1727),
    [sym_duet_dev_type_keypad] = ACTIONS(1727),
    [sym_duet_dev_type_light] = ACTIONS(1727),
    [sym_duet_dev_type_monitor] = ACTIONS(1727),
    [sym_duet_dev_type_motor] = ACTIONS(1727),
    [sym_duet_dev_type_multi_window] = ACTIONS(1727),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1727),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1727),
    [sym_duet_dev_type_receiver] = ACTIONS(1727),
    [sym_duet_dev_type_security_system] = ACTIONS(1727),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1727),
    [sym_duet_dev_type_settop_box] = ACTIONS(1727),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1727),
    [sym_duet_dev_type_switcher] = ACTIONS(1727),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1727),
    [sym_duet_dev_type_tv] = ACTIONS(1727),
    [sym_duet_dev_type_utility] = ACTIONS(1727),
    [sym_duet_dev_type_vcr] = ACTIONS(1727),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1727),
    [sym_duet_dev_type_video_processor] = ACTIONS(1727),
    [sym_duet_dev_type_video_projector] = ACTIONS(1727),
    [sym_duet_dev_type_video_wall] = ACTIONS(1727),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1727),
    [sym_duet_dev_type_weather] = ACTIONS(1727),
    [sym_duet_dev_not_polled] = ACTIONS(1727),
    [sym_duet_dev_polled] = ACTIONS(1727),
    [sym_clkmgr_mode_network] = ACTIONS(1727),
    [sym_clkmgr_mode_standalone] = ACTIONS(1727),
    [sym_normal_standby] = ACTIONS(1727),
    [sym_normal_wake] = ACTIONS(1727),
    [sym_ip_multicast_ttl_option] = ACTIONS(1727),
    [sym_ip_ttl_subnet] = ACTIONS(1727),
    [sym_ip_ttl_site] = ACTIONS(1727),
    [sym_ip_ttl_region] = ACTIONS(1727),
    [sym_ip_ttl_continent] = ACTIONS(1727),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1727),
    [sym_ip_nodelay_on] = ACTIONS(1727),
    [sym_ip_nodelay_off] = ACTIONS(1727),
    [sym_amx_error] = ACTIONS(1727),
    [sym_amx_warning] = ACTIONS(1727),
    [sym_amx_info] = ACTIONS(1727),
    [sym_amx_debug] = ACTIONS(1727),
    [sym_netlinx_login_success] = ACTIONS(1727),
    [sym_netlinx_login_fail] = ACTIONS(1727),
    [sym_netlinx_logout] = ACTIONS(1727),
    [sym_null_str] = ACTIONS(1727),
    [sym_smtp_address] = ACTIONS(1727),
    [sym_smtp_port_number] = ACTIONS(1727),
    [sym_smtp_username] = ACTIONS(1727),
    [sym_smtp_password] = ACTIONS(1727),
    [sym_smtp_require_tls] = ACTIONS(1727),
    [sym_smtp_from] = ACTIONS(1727),
    [sym_smtp_tls_true] = ACTIONS(1727),
    [sym_smtp_tls_false] = ACTIONS(1727),
    [sym_listview_on_row_select_event] = ACTIONS(1727),
    [sym_data_structure_unknown] = ACTIONS(1727),
    [sym_data_structure_datafeed] = ACTIONS(1727),
    [sym_data_structure_datarecordset] = ACTIONS(1727),
    [sym_data_structure_datarecord] = ACTIONS(1727),
    [sym_data_structure_datafield] = ACTIONS(1727),
    [sym_data_max_content_field_count] = ACTIONS(1727),
    [sym_data_max_metadata_field_count] = ACTIONS(1727),
    [sym_data_max_id_length] = ACTIONS(1727),
    [sym_data_max_name_length] = ACTIONS(1727),
    [sym_data_max_description_length] = ACTIONS(1727),
    [sym_data_max_type_length] = ACTIONS(1727),
    [sym_data_max_format_length] = ACTIONS(1727),
    [sym_data_max_source_length] = ACTIONS(1727),
    [sym_data_max_value_length] = ACTIONS(1727),
    [sym_data_max_label_length] = ACTIONS(1727),
    [sym_data_max_url_length] = ACTIONS(1727),
    [sym_max_last_login_info_length] = ACTIONS(1727),
    [sym_data_type_unknown] = ACTIONS(1727),
    [sym_data_type_string] = ACTIONS(1727),
    [sym_data_type_datetime] = ACTIONS(1727),
    [sym_data_type_date] = ACTIONS(1727),
    [sym_data_type_time] = ACTIONS(1727),
    [sym_data_type_image] = ACTIONS(1727),
    [sym_data_format_url] = ACTIONS(1727),
    [sym_data_format_phone] = ACTIONS(1727),
    [sym_data_format_email] = ACTIONS(1727),
    [sym_data_format_iso8601] = ACTIONS(1727),
    [sym_auditlog_priv] = ACTIONS(1727),
    [sym_configuration_priv] = ACTIONS(1727),
    [sym_deviceconfig_priv] = ACTIONS(1727),
    [sym_ftp_priv] = ACTIONS(1727),
    [sym_http_priv] = ACTIONS(1727),
    [sym_networkconfig_priv] = ACTIONS(1727),
    [sym_programport_priv] = ACTIONS(1727),
    [sym_remoteui_priv] = ACTIONS(1727),
    [sym_securitycontrol_priv] = ACTIONS(1727),
    [sym_softwaremanagement_priv] = ACTIONS(1727),
    [sym_terminal_priv] = ACTIONS(1727),
    [sym_tpadmin_priv] = ACTIONS(1727),
    [sym_user1_priv] = ACTIONS(1727),
    [sym_user2_priv] = ACTIONS(1727),
    [sym_user3_priv] = ACTIONS(1727),
    [sym_user4_priv] = ACTIONS(1727),
    [sym_usermanagement_priv] = ACTIONS(1727),
    [sym_valid_account] = ACTIONS(1727),
    [sym_err_name_invalid] = ACTIONS(1727),
    [sym_err_password_invalid] = ACTIONS(1727),
    [sym_err_invalid_account] = ACTIONS(1727),
    [sym_err_no_authorization] = ACTIONS(1727),
    [sym_err_invalid_parameter] = ACTIONS(1727),
    [sym_err_user_locked_out] = ACTIONS(1727),
    [sym_err_user_account_expired] = ACTIONS(1727),
    [sym_err_general_error] = ACTIONS(1727),
    [sym_err_authentication_error] = ACTIONS(1727),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1727),
    [sym_tls_validate_certificate] = ACTIONS(1727),
    [sym_tdata] = ACTIONS(1727),
    [sym_tchannel] = ACTIONS(1727),
    [sym_tlevel] = ACTIONS(1727),
    [sym_tbutton] = ACTIONS(1727),
    [sym_ttimeline] = ACTIONS(1727),
    [sym_tcustom] = ACTIONS(1727),
    [sym_url_struct] = ACTIONS(1727),
    [sym_dns_struct] = ACTIONS(1727),
    [sym_ip_address_struct] = ACTIONS(1727),
    [sym_dev_info_struct] = ACTIONS(1727),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1727),
    [sym_data_feed] = ACTIONS(1727),
    [sym_data_field] = ACTIONS(1727),
    [sym_data_record] = ACTIONS(1727),
    [sym_wc_data_feed] = ACTIONS(1727),
    [sym_wc_data_field] = ACTIONS(1727),
    [sym_wc_data_record] = ACTIONS(1727),
    [sym_last_login_info] = ACTIONS(1727),
    [sym____reserved___] = ACTIONS(1727),
    [sym_date] = ACTIONS(1727),
    [sym_day] = ACTIONS(1727),
    [sym_time] = ACTIONS(1727),
    [sym_ldate] = ACTIONS(1727),
    [sym_push_device] = ACTIONS(1727),
    [sym_push_channel] = ACTIONS(1727),
    [sym_push_devchan] = ACTIONS(1727),
    [sym_release_device] = ACTIONS(1727),
    [sym_release_channel] = ACTIONS(1727),
    [sym_release_devchan] = ACTIONS(1727),
    [sym_master_slot] = ACTIONS(1727),
    [sym_get_pulse_time] = ACTIONS(1727),
    [sym_get_timer] = ACTIONS(1727),
    [sym_master_sn] = ACTIONS(1727),
    [sym_system_number] = ACTIONS(1727),
    [sym_button] = ACTIONS(1727),
    [sym_dv_channel] = ACTIONS(1727),
    [sym_data] = ACTIONS(1727),
    [sym_level] = ACTIONS(1727),
    [sym_timeline] = ACTIONS(1727),
    [sym_channel] = ACTIONS(1727),
    [sym_custom] = ACTIONS(1727),
    [sym_atoi] = ACTIONS(1727),
    [sym_atol] = ACTIONS(1727),
    [sym_atof] = ACTIONS(1727),
    [sym_itoa] = ACTIONS(1727),
    [sym_itohex] = ACTIONS(1727),
    [sym_hextoi] = ACTIONS(1727),
    [sym_ftoa] = ACTIONS(1727),
    [sym_find_string] = ACTIONS(1727),
    [sym_left_string] = ACTIONS(1727),
    [sym_right_string] = ACTIONS(1727),
    [sym_mid_string] = ACTIONS(1727),
    [sym_remove_string] = ACTIONS(1727),
    [sym_get_buffer_string] = ACTIONS(1727),
    [sym_get_buffer_char] = ACTIONS(1727),
    [sym_get_multi_buffer_string] = ACTIONS(1727),
    [sym_length_string] = ACTIONS(1727),
    [sym_max_length_string] = ACTIONS(1727),
    [sym_set_length_string] = ACTIONS(1727),
    [sym_compare_string] = ACTIONS(1727),
    [sym_upper_string] = ACTIONS(1727),
    [sym_lower_string] = ACTIONS(1727),
    [sym_get_last] = ACTIONS(1727),
    [sym_date_to_day] = ACTIONS(1727),
    [sym_date_to_month] = ACTIONS(1727),
    [sym_date_to_year] = ACTIONS(1727),
    [sym_time_to_hour] = ACTIONS(1727),
    [sym_time_to_minute] = ACTIONS(1727),
    [sym_time_to_second] = ACTIONS(1727),
    [sym_day_of_week] = ACTIONS(1727),
    [sym_set_pulse_time] = ACTIONS(1727),
    [sym_set_timer] = ACTIONS(1727),
    [sym_file_open] = ACTIONS(1727),
    [sym_file_close] = ACTIONS(1727),
    [sym_file_read] = ACTIONS(1727),
    [sym_file_read_line] = ACTIONS(1727),
    [sym_file_write] = ACTIONS(1727),
    [sym_file_write_line] = ACTIONS(1727),
    [sym_file_copy] = ACTIONS(1727),
    [sym_file_delete] = ACTIONS(1727),
    [sym_file_rename] = ACTIONS(1727),
    [sym_file_dir] = ACTIONS(1727),
    [sym_file_getdir] = ACTIONS(1727),
    [sym_file_setdir] = ACTIONS(1727),
    [sym_file_create_dir] = ACTIONS(1727),
    [sym_file_remove_dir] = ACTIONS(1727),
    [sym_file_seek] = ACTIONS(1727),
    [sym_length_array] = ACTIONS(1727),
    [sym_max_length_array] = ACTIONS(1727),
    [sym_set_length_array] = ACTIONS(1727),
    [sym_ip_client_open] = ACTIONS(1727),
    [sym_ip_client_close] = ACTIONS(1727),
    [sym_ip_server_open] = ACTIONS(1727),
    [sym_ip_server_close] = ACTIONS(1727),
    [sym_ip_bound_client_open] = ACTIONS(1727),
    [sym_ip_mc_server_open] = ACTIONS(1727),
    [sym_ip_set_option] = ACTIONS(1727),
    [sym_ssh_client_open] = ACTIONS(1727),
    [sym_ssh_client_close] = ACTIONS(1727),
    [sym_tls_client_open] = ACTIONS(1727),
    [sym_tls_client_close] = ACTIONS(1727),
    [sym_do_push] = ACTIONS(1727),
    [sym_do_push_timed] = ACTIONS(1727),
    [sym_do_release] = ACTIONS(1727),
    [sym_device_id] = ACTIONS(1727),
    [sym_device_info] = ACTIONS(1727),
    [sym_device_id_string] = ACTIONS(1727),
    [sym_device_standby] = ACTIONS(1727),
    [sym_device_wake] = ACTIONS(1727),
    [sym_timeline_create] = ACTIONS(1727),
    [sym_timeline_kill] = ACTIONS(1727),
    [sym_timeline_reload] = ACTIONS(1727),
    [sym_timeline_pause] = ACTIONS(1727),
    [sym_timeline_restart] = ACTIONS(1727),
    [sym_timeline_set] = ACTIONS(1727),
    [sym_timeline_get] = ACTIONS(1727),
    [sym_timeline_active] = ACTIONS(1727),
    [sym_timeline_dynamic_id] = ACTIONS(1727),
    [sym_variable_to_string] = ACTIONS(1727),
    [sym_string_to_variable] = ACTIONS(1727),
    [sym_length_variable_to_string] = ACTIONS(1727),
    [sym_type_cast] = ACTIONS(1727),
    [sym_variable_to_xml] = ACTIONS(1727),
    [sym_xml_to_variable] = ACTIONS(1727),
    [sym_length_variable_to_xml] = ACTIONS(1727),
    [sym_redirect_string] = ACTIONS(1727),
    [sym_random_number] = ACTIONS(1727),
    [sym_get_unique_id] = ACTIONS(1727),
    [sym_get_system_number] = ACTIONS(1727),
    [sym_set_system_number] = ACTIONS(1727),
    [sym_get_serial_number] = ACTIONS(1727),
    [sym_get_master_build] = ACTIONS(1727),
    [sym_set_validation_code] = ACTIONS(1727),
    [sym_reboot] = ACTIONS(1727),
    [sym_load_duet_module] = ACTIONS(1727),
    [sym_unload_duet_module] = ACTIONS(1727),
    [sym_duet_mem_size_set] = ACTIONS(1727),
    [sym_duet_mem_size_get] = ACTIONS(1727),
    [sym_dynamic_application_device] = ACTIONS(1727),
    [sym_dynamic_polled_port] = ACTIONS(1727),
    [sym_static_port_binding] = ACTIONS(1727),
    [sym_static_ip_binding] = ACTIONS(1727),
    [sym_get_dns_list] = ACTIONS(1727),
    [sym_set_dns_list] = ACTIONS(1727),
    [sym_get_ip_address] = ACTIONS(1727),
    [sym_set_ip_address] = ACTIONS(1727),
    [sym_get_url_list] = ACTIONS(1727),
    [sym_add_url_entry] = ACTIONS(1727),
    [sym_delete_url_entry] = ACTIONS(1727),
    [sym_abs_value] = ACTIONS(1727),
    [sym_max_value] = ACTIONS(1727),
    [sym_min_value] = ACTIONS(1727),
    [sym_exp_value] = ACTIONS(1727),
    [sym_log_value] = ACTIONS(1727),
    [sym_log10_value] = ACTIONS(1727),
    [sym_power_value] = ACTIONS(1727),
    [sym_sqrt_value] = ACTIONS(1727),
    [sym_raw_be] = ACTIONS(1727),
    [sym_raw_le] = ACTIONS(1727),
    [sym_internal_queue_size_set] = ACTIONS(1727),
    [sym_internal_queue_size_get] = ACTIONS(1727),
    [sym_internal_threshold_set] = ACTIONS(1727),
    [sym_internal_threshold_get] = ACTIONS(1727),
    [sym_rebuilt_event] = ACTIONS(1727),
    [sym_do_custom_event] = ACTIONS(1727),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1727),
    [sym_clkmgr_set_clk_source] = ACTIONS(1727),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1727),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1727),
    [sym_clkmgr_get_timezone] = ACTIONS(1727),
    [sym_clkmgr_set_timezone] = ACTIONS(1727),
    [sym_clkmgr_get_resync_period] = ACTIONS(1727),
    [sym_clkmgr_set_resync_period] = ACTIONS(1727),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1727),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1727),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1727),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1727),
    [sym_clkmgr_get_timeservers] = ACTIONS(1727),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1727),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1727),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1727),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1727),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1727),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1727),
    [sym_set_log_level] = ACTIONS(1727),
    [sym_get_log_level] = ACTIONS(1727),
    [sym_amx_log] = ACTIONS(1727),
    [sym_smtp_server_config_set] = ACTIONS(1727),
    [sym_smtp_server_config_get] = ACTIONS(1727),
    [sym_smtp_send] = ACTIONS(1727),
    [sym_validate_netlinx_account] = ACTIONS(1727),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1727),
    [sym_authenticate_certificate] = ACTIONS(1727),
    [sym_audit_netlinx_session_event] = ACTIONS(1727),
    [sym_audit_netlinx_generic_event] = ACTIONS(1727),
    [sym_get_available_flash_disk_space] = ACTIONS(1727),
    [sym_get_max_flash_disk_space] = ACTIONS(1727),
    [sym_data_create_feed] = ACTIONS(1727),
    [sym_data_delete_feed] = ACTIONS(1727),
    [sym_data_publish_feed] = ACTIONS(1727),
    [sym_data_get_published_feed] = ACTIONS(1727),
    [sym_data_add_record] = ACTIONS(1727),
    [sym_data_get_event_record] = ACTIONS(1727),
    [sym__wc_data_create_feed] = ACTIONS(1727),
    [sym__wc_data_add_record] = ACTIONS(1727),
    [sym__wc_data_get_event_record] = ACTIONS(1727),
    [sym_format] = ACTIONS(1727),
    [sym_astro_clock] = ACTIONS(1727),
    [sym_set_outdoor_temperature] = ACTIONS(1727),
    [sym_set_virtual_level_count] = ACTIONS(1727),
    [sym_set_virtual_channel_count] = ACTIONS(1727),
    [sym_set_virtual_port_count] = ACTIONS(1727),
    [sym__wc_encode_int] = ACTIONS(1727),
    [sym__wc_decode_int] = ACTIONS(1727),
    [sym__wc_int] = ACTIONS(1727),
    [sym__wc_to_ch_int] = ACTIONS(1727),
    [sym__ch_to_wc_int] = ACTIONS(1727),
    [sym__wc_find_string_int] = ACTIONS(1727),
    [sym__wc_left_string_int] = ACTIONS(1727),
    [sym__wc_lower_string_int] = ACTIONS(1727),
    [sym__wc_mid_string_int] = ACTIONS(1727),
    [sym__wc_remove_string_int] = ACTIONS(1727),
    [sym__wc_right_string_int] = ACTIONS(1727),
    [sym__wc_upper_string_int] = ACTIONS(1727),
    [sym__wc_compare_string_int] = ACTIONS(1727),
    [sym__wc_get_buffer_char_int] = ACTIONS(1727),
    [sym__wc_get_buffer_string_int] = ACTIONS(1727),
    [sym__wc_concat_string_int] = ACTIONS(1727),
    [sym__wc_file_open_int] = ACTIONS(1727),
    [sym__wc_file_close_int] = ACTIONS(1727),
    [sym__wc_file_read_int] = ACTIONS(1727),
    [sym__wc_file_read_line_int] = ACTIONS(1727),
    [sym__wc_file_write_int] = ACTIONS(1727),
    [sym__wc_file_write_line_int] = ACTIONS(1727),
    [aux_sym_preproc_include_token1] = ACTIONS(1727),
    [aux_sym_preproc_define_token1] = ACTIONS(1729),
    [aux_sym_preproc_warn_token1] = ACTIONS(1729),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1729),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1729),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1729),
    [anon_sym_LBRACK] = ACTIONS(1729),
    [anon_sym_LBRACE] = ACTIONS(1729),
    [anon_sym_LPAREN] = ACTIONS(1727),
    [anon_sym_BANG] = ACTIONS(1729),
    [anon_sym_TILDE] = ACTIONS(1729),
    [anon_sym_DASH] = ACTIONS(1727),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_DASH_DASH] = ACTIONS(1729),
    [anon_sym_PLUS_PLUS] = ACTIONS(1729),
    [anon_sym_DQUOTE] = ACTIONS(1729),
    [anon_sym_SQUOTE] = ACTIONS(1729),
    [sym_number_literal] = ACTIONS(1729),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(394)] = {
    [sym_identifier] = ACTIONS(2133),
    [sym_program_name_keyword] = ACTIONS(2133),
    [sym_module_name_keyword] = ACTIONS(2133),
    [sym_define_device_keyword] = ACTIONS(2133),
    [sym_define_combine_keyword] = ACTIONS(2133),
    [sym_define_constant_keyword] = ACTIONS(2133),
    [sym_define_type_keyword] = ACTIONS(2133),
    [sym_define_variable_keyword] = ACTIONS(2133),
    [sym_define_system_variable_keyword] = ACTIONS(2133),
    [sym_define_connect_level_keyword] = ACTIONS(2133),
    [sym_define_latching_keyword] = ACTIONS(2133),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2133),
    [sym_define_toggling_keyword] = ACTIONS(2133),
    [sym_define_call_keyword] = ACTIONS(2133),
    [sym_define_function_keyword] = ACTIONS(2133),
    [sym_define_library_function_keyword] = ACTIONS(2133),
    [sym_define_start_keyword] = ACTIONS(2133),
    [sym_define_event_keyword] = ACTIONS(2133),
    [sym_define_program_keyword] = ACTIONS(2133),
    [sym_define_module_keyword] = ACTIONS(2133),
    [sym_char_keyword] = ACTIONS(2133),
    [sym_widechar_keyword] = ACTIONS(2133),
    [sym_integer_keyword] = ACTIONS(2133),
    [sym_sinteger_keyword] = ACTIONS(2133),
    [sym_long_keyword] = ACTIONS(2133),
    [sym_slong_keyword] = ACTIONS(2133),
    [sym_float_keyword] = ACTIONS(2133),
    [sym_double_keyword] = ACTIONS(2133),
    [sym_variant_keyword] = ACTIONS(2133),
    [sym_variantarray_keyword] = ACTIONS(2133),
    [sym_local_var_keyword] = ACTIONS(2133),
    [sym_stack_var_keyword] = ACTIONS(2133),
    [sym_constant_keyword] = ACTIONS(2133),
    [sym_volatile_keyword] = ACTIONS(2133),
    [sym_non_volatile_keyword] = ACTIONS(2133),
    [sym_persistent_keyword] = ACTIONS(2133),
    [sym_dev_keyword] = ACTIONS(2133),
    [sym_devlev_keyword] = ACTIONS(2133),
    [sym_devchan_keyword] = ACTIONS(2133),
    [sym_if_keyword] = ACTIONS(2133),
    [sym_switch_keyword] = ACTIONS(2133),
    [sym_case_keyword] = ACTIONS(2133),
    [sym_default_keyword] = ACTIONS(2133),
    [sym_while_keyword] = ACTIONS(2133),
    [sym_for_keyword] = ACTIONS(2133),
    [sym_break_keyword] = ACTIONS(2133),
    [sym_continue_keyword] = ACTIONS(2133),
    [sym_return_keyword] = ACTIONS(2133),
    [sym_select_keyword] = ACTIONS(2133),
    [sym_active_keyword] = ACTIONS(2133),
    [sym_struct_keyword] = ACTIONS(2133),
    [sym_structure_keyword] = ACTIONS(2133),
    [sym_button_event_keyword] = ACTIONS(2133),
    [sym_channel_event_keyword] = ACTIONS(2133),
    [sym_level_event_keyword] = ACTIONS(2133),
    [sym_data_event_keyword] = ACTIONS(2133),
    [sym_timeline_event_keyword] = ACTIONS(2133),
    [sym_custom_event_keyword] = ACTIONS(2133),
    [sym_push_keyword] = ACTIONS(2133),
    [sym_release_keyword] = ACTIONS(2133),
    [aux_sym_on_keyword_token1] = ACTIONS(2133),
    [aux_sym_off_keyword_token1] = ACTIONS(2133),
    [sym_send_level_keyword] = ACTIONS(2133),
    [sym_send_string_keyword] = ACTIONS(2133),
    [sym_send_command_keyword] = ACTIONS(2133),
    [sym_clear_buffer_keyword] = ACTIONS(2133),
    [sym_create_buffer_keyword] = ACTIONS(2133),
    [sym_create_multi_buffer_keyword] = ACTIONS(2133),
    [sym_call_keyword] = ACTIONS(2133),
    [sym_system_call_keyword] = ACTIONS(2133),
    [sym_devchan_to_keyword] = ACTIONS(2133),
    [sym_devchan_min_to_keyword] = ACTIONS(2133),
    [sym_devchan_total_off_keyword] = ACTIONS(2133),
    [sym_devchan_pulse_keyword] = ACTIONS(2133),
    [sym_wait_keyword] = ACTIONS(2133),
    [sym_cancel_wait_keyword] = ACTIONS(2133),
    [sym_wait_until_keyword] = ACTIONS(2133),
    [sym_cancel_wait_until_keyword] = ACTIONS(2133),
    [sym_cancel_all_wait_keyword] = ACTIONS(2133),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2133),
    [sym_not] = ACTIONS(2133),
    [sym_bnot] = ACTIONS(2133),
    [sym___file__] = ACTIONS(2133),
    [sym___date__] = ACTIONS(2133),
    [sym___ldate__] = ACTIONS(2133),
    [sym___time__] = ACTIONS(2133),
    [sym___line__] = ACTIONS(2133),
    [sym___name__] = ACTIONS(2133),
    [sym___netlinx__] = ACTIONS(2133),
    [sym_netlinx_axi_version] = ACTIONS(2133),
    [sym_url_flg_tcp] = ACTIONS(2133),
    [sym_url_flg_accinfopresent] = ACTIONS(2133),
    [sym_url_flg_temp] = ACTIONS(2133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2133),
    [sym_url_flg_stat_mask] = ACTIONS(2133),
    [sym_url_flg_stat_lookup] = ACTIONS(2133),
    [sym_url_flg_stat_connecting] = ACTIONS(2133),
    [sym_url_flg_stat_waiting] = ACTIONS(2133),
    [sym_url_flg_stat_connected] = ACTIONS(2133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2133),
    [sym_first_virtual_device] = ACTIONS(2133),
    [sym_first_local_port] = ACTIONS(2133),
    [sym_dynamic_virtual_device] = ACTIONS(2133),
    [sym_timeline_once] = ACTIONS(2133),
    [sym_timeline_repeat] = ACTIONS(2133),
    [sym_timeline_absolute] = ACTIONS(2133),
    [sym_timeline_relative] = ACTIONS(2133),
    [sym_do_push_timed_infinite] = ACTIONS(2133),
    [sym_source_type_no_address] = ACTIONS(2133),
    [sym_source_type_neuron_id] = ACTIONS(2133),
    [sym_source_type_ip_address] = ACTIONS(2133),
    [sym_source_type_axlink] = ACTIONS(2133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2133),
    [sym_source_type_ip_socket_address] = ACTIONS(2133),
    [sym_source_type_rs232] = ACTIONS(2133),
    [sym_source_type_internal] = ACTIONS(2133),
    [sym_source_type_mac_address] = ACTIONS(2133),
    [sym_source_type_ipv4_port] = ACTIONS(2133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2133),
    [sym_true] = ACTIONS(2133),
    [sym_false] = ACTIONS(2133),
    [sym_file_read_only] = ACTIONS(2133),
    [sym_file_rw_new] = ACTIONS(2133),
    [sym_file_rw_append] = ACTIONS(2133),
    [sym_ip_tcp] = ACTIONS(2133),
    [sym_ip_udp] = ACTIONS(2133),
    [sym_ip_udp_2way] = ACTIONS(2133),
    [sym_xml_encode_types] = ACTIONS(2133),
    [sym_xml_encode_char_as_list] = ACTIONS(2133),
    [sym_xml_encode_le] = ACTIONS(2133),
    [sym_xml_decode_types] = ACTIONS(2133),
    [sym_xml_decode_no_preserve] = ACTIONS(2133),
    [sym_duet_dev_type_display_device] = ACTIONS(2133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2133),
    [sym_duet_dev_type_io_device] = ACTIONS(2133),
    [sym_duet_dev_type_relay_device] = ACTIONS(2133),
    [sym_duet_dev_type_ups] = ACTIONS(2133),
    [sym_duet_dev_type_amplifier] = ACTIONS(2133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2133),
    [sym_duet_dev_type_camera] = ACTIONS(2133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2133),
    [sym_duet_dev_type_dss] = ACTIONS(2133),
    [sym_duet_dev_type_dvr] = ACTIONS(2133),
    [sym_duet_dev_type_disc_device] = ACTIONS(2133),
    [sym_duet_dev_type_document_camera] = ACTIONS(2133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2133),
    [sym_duet_dev_type_hvac] = ACTIONS(2133),
    [sym_duet_dev_type_keypad] = ACTIONS(2133),
    [sym_duet_dev_type_light] = ACTIONS(2133),
    [sym_duet_dev_type_monitor] = ACTIONS(2133),
    [sym_duet_dev_type_motor] = ACTIONS(2133),
    [sym_duet_dev_type_multi_window] = ACTIONS(2133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2133),
    [sym_duet_dev_type_receiver] = ACTIONS(2133),
    [sym_duet_dev_type_security_system] = ACTIONS(2133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2133),
    [sym_duet_dev_type_settop_box] = ACTIONS(2133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2133),
    [sym_duet_dev_type_switcher] = ACTIONS(2133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2133),
    [sym_duet_dev_type_tv] = ACTIONS(2133),
    [sym_duet_dev_type_utility] = ACTIONS(2133),
    [sym_duet_dev_type_vcr] = ACTIONS(2133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2133),
    [sym_duet_dev_type_video_processor] = ACTIONS(2133),
    [sym_duet_dev_type_video_projector] = ACTIONS(2133),
    [sym_duet_dev_type_video_wall] = ACTIONS(2133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2133),
    [sym_duet_dev_type_weather] = ACTIONS(2133),
    [sym_duet_dev_not_polled] = ACTIONS(2133),
    [sym_duet_dev_polled] = ACTIONS(2133),
    [sym_clkmgr_mode_network] = ACTIONS(2133),
    [sym_clkmgr_mode_standalone] = ACTIONS(2133),
    [sym_normal_standby] = ACTIONS(2133),
    [sym_normal_wake] = ACTIONS(2133),
    [sym_ip_multicast_ttl_option] = ACTIONS(2133),
    [sym_ip_ttl_subnet] = ACTIONS(2133),
    [sym_ip_ttl_site] = ACTIONS(2133),
    [sym_ip_ttl_region] = ACTIONS(2133),
    [sym_ip_ttl_continent] = ACTIONS(2133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2133),
    [sym_ip_nodelay_on] = ACTIONS(2133),
    [sym_ip_nodelay_off] = ACTIONS(2133),
    [sym_amx_error] = ACTIONS(2133),
    [sym_amx_warning] = ACTIONS(2133),
    [sym_amx_info] = ACTIONS(2133),
    [sym_amx_debug] = ACTIONS(2133),
    [sym_netlinx_login_success] = ACTIONS(2133),
    [sym_netlinx_login_fail] = ACTIONS(2133),
    [sym_netlinx_logout] = ACTIONS(2133),
    [sym_null_str] = ACTIONS(2133),
    [sym_smtp_address] = ACTIONS(2133),
    [sym_smtp_port_number] = ACTIONS(2133),
    [sym_smtp_username] = ACTIONS(2133),
    [sym_smtp_password] = ACTIONS(2133),
    [sym_smtp_require_tls] = ACTIONS(2133),
    [sym_smtp_from] = ACTIONS(2133),
    [sym_smtp_tls_true] = ACTIONS(2133),
    [sym_smtp_tls_false] = ACTIONS(2133),
    [sym_listview_on_row_select_event] = ACTIONS(2133),
    [sym_data_structure_unknown] = ACTIONS(2133),
    [sym_data_structure_datafeed] = ACTIONS(2133),
    [sym_data_structure_datarecordset] = ACTIONS(2133),
    [sym_data_structure_datarecord] = ACTIONS(2133),
    [sym_data_structure_datafield] = ACTIONS(2133),
    [sym_data_max_content_field_count] = ACTIONS(2133),
    [sym_data_max_metadata_field_count] = ACTIONS(2133),
    [sym_data_max_id_length] = ACTIONS(2133),
    [sym_data_max_name_length] = ACTIONS(2133),
    [sym_data_max_description_length] = ACTIONS(2133),
    [sym_data_max_type_length] = ACTIONS(2133),
    [sym_data_max_format_length] = ACTIONS(2133),
    [sym_data_max_source_length] = ACTIONS(2133),
    [sym_data_max_value_length] = ACTIONS(2133),
    [sym_data_max_label_length] = ACTIONS(2133),
    [sym_data_max_url_length] = ACTIONS(2133),
    [sym_max_last_login_info_length] = ACTIONS(2133),
    [sym_data_type_unknown] = ACTIONS(2133),
    [sym_data_type_string] = ACTIONS(2133),
    [sym_data_type_datetime] = ACTIONS(2133),
    [sym_data_type_date] = ACTIONS(2133),
    [sym_data_type_time] = ACTIONS(2133),
    [sym_data_type_image] = ACTIONS(2133),
    [sym_data_format_url] = ACTIONS(2133),
    [sym_data_format_phone] = ACTIONS(2133),
    [sym_data_format_email] = ACTIONS(2133),
    [sym_data_format_iso8601] = ACTIONS(2133),
    [sym_auditlog_priv] = ACTIONS(2133),
    [sym_configuration_priv] = ACTIONS(2133),
    [sym_deviceconfig_priv] = ACTIONS(2133),
    [sym_ftp_priv] = ACTIONS(2133),
    [sym_http_priv] = ACTIONS(2133),
    [sym_networkconfig_priv] = ACTIONS(2133),
    [sym_programport_priv] = ACTIONS(2133),
    [sym_remoteui_priv] = ACTIONS(2133),
    [sym_securitycontrol_priv] = ACTIONS(2133),
    [sym_softwaremanagement_priv] = ACTIONS(2133),
    [sym_terminal_priv] = ACTIONS(2133),
    [sym_tpadmin_priv] = ACTIONS(2133),
    [sym_user1_priv] = ACTIONS(2133),
    [sym_user2_priv] = ACTIONS(2133),
    [sym_user3_priv] = ACTIONS(2133),
    [sym_user4_priv] = ACTIONS(2133),
    [sym_usermanagement_priv] = ACTIONS(2133),
    [sym_valid_account] = ACTIONS(2133),
    [sym_err_name_invalid] = ACTIONS(2133),
    [sym_err_password_invalid] = ACTIONS(2133),
    [sym_err_invalid_account] = ACTIONS(2133),
    [sym_err_no_authorization] = ACTIONS(2133),
    [sym_err_invalid_parameter] = ACTIONS(2133),
    [sym_err_user_locked_out] = ACTIONS(2133),
    [sym_err_user_account_expired] = ACTIONS(2133),
    [sym_err_general_error] = ACTIONS(2133),
    [sym_err_authentication_error] = ACTIONS(2133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2133),
    [sym_tls_validate_certificate] = ACTIONS(2133),
    [sym_tdata] = ACTIONS(2133),
    [sym_tchannel] = ACTIONS(2133),
    [sym_tlevel] = ACTIONS(2133),
    [sym_tbutton] = ACTIONS(2133),
    [sym_ttimeline] = ACTIONS(2133),
    [sym_tcustom] = ACTIONS(2133),
    [sym_url_struct] = ACTIONS(2133),
    [sym_dns_struct] = ACTIONS(2133),
    [sym_ip_address_struct] = ACTIONS(2133),
    [sym_dev_info_struct] = ACTIONS(2133),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2133),
    [sym_data_feed] = ACTIONS(2133),
    [sym_data_field] = ACTIONS(2133),
    [sym_data_record] = ACTIONS(2133),
    [sym_wc_data_feed] = ACTIONS(2133),
    [sym_wc_data_field] = ACTIONS(2133),
    [sym_wc_data_record] = ACTIONS(2133),
    [sym_last_login_info] = ACTIONS(2133),
    [sym____reserved___] = ACTIONS(2133),
    [sym_date] = ACTIONS(2133),
    [sym_day] = ACTIONS(2133),
    [sym_time] = ACTIONS(2133),
    [sym_ldate] = ACTIONS(2133),
    [sym_push_device] = ACTIONS(2133),
    [sym_push_channel] = ACTIONS(2133),
    [sym_push_devchan] = ACTIONS(2133),
    [sym_release_device] = ACTIONS(2133),
    [sym_release_channel] = ACTIONS(2133),
    [sym_release_devchan] = ACTIONS(2133),
    [sym_master_slot] = ACTIONS(2133),
    [sym_get_pulse_time] = ACTIONS(2133),
    [sym_get_timer] = ACTIONS(2133),
    [sym_master_sn] = ACTIONS(2133),
    [sym_system_number] = ACTIONS(2133),
    [sym_button] = ACTIONS(2133),
    [sym_dv_channel] = ACTIONS(2133),
    [sym_data] = ACTIONS(2133),
    [sym_level] = ACTIONS(2133),
    [sym_timeline] = ACTIONS(2133),
    [sym_channel] = ACTIONS(2133),
    [sym_custom] = ACTIONS(2133),
    [sym_atoi] = ACTIONS(2133),
    [sym_atol] = ACTIONS(2133),
    [sym_atof] = ACTIONS(2133),
    [sym_itoa] = ACTIONS(2133),
    [sym_itohex] = ACTIONS(2133),
    [sym_hextoi] = ACTIONS(2133),
    [sym_ftoa] = ACTIONS(2133),
    [sym_find_string] = ACTIONS(2133),
    [sym_left_string] = ACTIONS(2133),
    [sym_right_string] = ACTIONS(2133),
    [sym_mid_string] = ACTIONS(2133),
    [sym_remove_string] = ACTIONS(2133),
    [sym_get_buffer_string] = ACTIONS(2133),
    [sym_get_buffer_char] = ACTIONS(2133),
    [sym_get_multi_buffer_string] = ACTIONS(2133),
    [sym_length_string] = ACTIONS(2133),
    [sym_max_length_string] = ACTIONS(2133),
    [sym_set_length_string] = ACTIONS(2133),
    [sym_compare_string] = ACTIONS(2133),
    [sym_upper_string] = ACTIONS(2133),
    [sym_lower_string] = ACTIONS(2133),
    [sym_get_last] = ACTIONS(2133),
    [sym_date_to_day] = ACTIONS(2133),
    [sym_date_to_month] = ACTIONS(2133),
    [sym_date_to_year] = ACTIONS(2133),
    [sym_time_to_hour] = ACTIONS(2133),
    [sym_time_to_minute] = ACTIONS(2133),
    [sym_time_to_second] = ACTIONS(2133),
    [sym_day_of_week] = ACTIONS(2133),
    [sym_set_pulse_time] = ACTIONS(2133),
    [sym_set_timer] = ACTIONS(2133),
    [sym_file_open] = ACTIONS(2133),
    [sym_file_close] = ACTIONS(2133),
    [sym_file_read] = ACTIONS(2133),
    [sym_file_read_line] = ACTIONS(2133),
    [sym_file_write] = ACTIONS(2133),
    [sym_file_write_line] = ACTIONS(2133),
    [sym_file_copy] = ACTIONS(2133),
    [sym_file_delete] = ACTIONS(2133),
    [sym_file_rename] = ACTIONS(2133),
    [sym_file_dir] = ACTIONS(2133),
    [sym_file_getdir] = ACTIONS(2133),
    [sym_file_setdir] = ACTIONS(2133),
    [sym_file_create_dir] = ACTIONS(2133),
    [sym_file_remove_dir] = ACTIONS(2133),
    [sym_file_seek] = ACTIONS(2133),
    [sym_length_array] = ACTIONS(2133),
    [sym_max_length_array] = ACTIONS(2133),
    [sym_set_length_array] = ACTIONS(2133),
    [sym_ip_client_open] = ACTIONS(2133),
    [sym_ip_client_close] = ACTIONS(2133),
    [sym_ip_server_open] = ACTIONS(2133),
    [sym_ip_server_close] = ACTIONS(2133),
    [sym_ip_bound_client_open] = ACTIONS(2133),
    [sym_ip_mc_server_open] = ACTIONS(2133),
    [sym_ip_set_option] = ACTIONS(2133),
    [sym_ssh_client_open] = ACTIONS(2133),
    [sym_ssh_client_close] = ACTIONS(2133),
    [sym_tls_client_open] = ACTIONS(2133),
    [sym_tls_client_close] = ACTIONS(2133),
    [sym_do_push] = ACTIONS(2133),
    [sym_do_push_timed] = ACTIONS(2133),
    [sym_do_release] = ACTIONS(2133),
    [sym_device_id] = ACTIONS(2133),
    [sym_device_info] = ACTIONS(2133),
    [sym_device_id_string] = ACTIONS(2133),
    [sym_device_standby] = ACTIONS(2133),
    [sym_device_wake] = ACTIONS(2133),
    [sym_timeline_create] = ACTIONS(2133),
    [sym_timeline_kill] = ACTIONS(2133),
    [sym_timeline_reload] = ACTIONS(2133),
    [sym_timeline_pause] = ACTIONS(2133),
    [sym_timeline_restart] = ACTIONS(2133),
    [sym_timeline_set] = ACTIONS(2133),
    [sym_timeline_get] = ACTIONS(2133),
    [sym_timeline_active] = ACTIONS(2133),
    [sym_timeline_dynamic_id] = ACTIONS(2133),
    [sym_variable_to_string] = ACTIONS(2133),
    [sym_string_to_variable] = ACTIONS(2133),
    [sym_length_variable_to_string] = ACTIONS(2133),
    [sym_type_cast] = ACTIONS(2133),
    [sym_variable_to_xml] = ACTIONS(2133),
    [sym_xml_to_variable] = ACTIONS(2133),
    [sym_length_variable_to_xml] = ACTIONS(2133),
    [sym_redirect_string] = ACTIONS(2133),
    [sym_random_number] = ACTIONS(2133),
    [sym_get_unique_id] = ACTIONS(2133),
    [sym_get_system_number] = ACTIONS(2133),
    [sym_set_system_number] = ACTIONS(2133),
    [sym_get_serial_number] = ACTIONS(2133),
    [sym_get_master_build] = ACTIONS(2133),
    [sym_set_validation_code] = ACTIONS(2133),
    [sym_reboot] = ACTIONS(2133),
    [sym_load_duet_module] = ACTIONS(2133),
    [sym_unload_duet_module] = ACTIONS(2133),
    [sym_duet_mem_size_set] = ACTIONS(2133),
    [sym_duet_mem_size_get] = ACTIONS(2133),
    [sym_dynamic_application_device] = ACTIONS(2133),
    [sym_dynamic_polled_port] = ACTIONS(2133),
    [sym_static_port_binding] = ACTIONS(2133),
    [sym_static_ip_binding] = ACTIONS(2133),
    [sym_get_dns_list] = ACTIONS(2133),
    [sym_set_dns_list] = ACTIONS(2133),
    [sym_get_ip_address] = ACTIONS(2133),
    [sym_set_ip_address] = ACTIONS(2133),
    [sym_get_url_list] = ACTIONS(2133),
    [sym_add_url_entry] = ACTIONS(2133),
    [sym_delete_url_entry] = ACTIONS(2133),
    [sym_abs_value] = ACTIONS(2133),
    [sym_max_value] = ACTIONS(2133),
    [sym_min_value] = ACTIONS(2133),
    [sym_exp_value] = ACTIONS(2133),
    [sym_log_value] = ACTIONS(2133),
    [sym_log10_value] = ACTIONS(2133),
    [sym_power_value] = ACTIONS(2133),
    [sym_sqrt_value] = ACTIONS(2133),
    [sym_raw_be] = ACTIONS(2133),
    [sym_raw_le] = ACTIONS(2133),
    [sym_internal_queue_size_set] = ACTIONS(2133),
    [sym_internal_queue_size_get] = ACTIONS(2133),
    [sym_internal_threshold_set] = ACTIONS(2133),
    [sym_internal_threshold_get] = ACTIONS(2133),
    [sym_rebuilt_event] = ACTIONS(2133),
    [sym_do_custom_event] = ACTIONS(2133),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2133),
    [sym_clkmgr_set_clk_source] = ACTIONS(2133),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2133),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2133),
    [sym_clkmgr_get_timezone] = ACTIONS(2133),
    [sym_clkmgr_set_timezone] = ACTIONS(2133),
    [sym_clkmgr_get_resync_period] = ACTIONS(2133),
    [sym_clkmgr_set_resync_period] = ACTIONS(2133),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2133),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2133),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2133),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2133),
    [sym_clkmgr_get_timeservers] = ACTIONS(2133),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2133),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2133),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2133),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2133),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2133),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2133),
    [sym_set_log_level] = ACTIONS(2133),
    [sym_get_log_level] = ACTIONS(2133),
    [sym_amx_log] = ACTIONS(2133),
    [sym_smtp_server_config_set] = ACTIONS(2133),
    [sym_smtp_server_config_get] = ACTIONS(2133),
    [sym_smtp_send] = ACTIONS(2133),
    [sym_validate_netlinx_account] = ACTIONS(2133),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2133),
    [sym_authenticate_certificate] = ACTIONS(2133),
    [sym_audit_netlinx_session_event] = ACTIONS(2133),
    [sym_audit_netlinx_generic_event] = ACTIONS(2133),
    [sym_get_available_flash_disk_space] = ACTIONS(2133),
    [sym_get_max_flash_disk_space] = ACTIONS(2133),
    [sym_data_create_feed] = ACTIONS(2133),
    [sym_data_delete_feed] = ACTIONS(2133),
    [sym_data_publish_feed] = ACTIONS(2133),
    [sym_data_get_published_feed] = ACTIONS(2133),
    [sym_data_add_record] = ACTIONS(2133),
    [sym_data_get_event_record] = ACTIONS(2133),
    [sym__wc_data_create_feed] = ACTIONS(2133),
    [sym__wc_data_add_record] = ACTIONS(2133),
    [sym__wc_data_get_event_record] = ACTIONS(2133),
    [sym_format] = ACTIONS(2133),
    [sym_astro_clock] = ACTIONS(2133),
    [sym_set_outdoor_temperature] = ACTIONS(2133),
    [sym_set_virtual_level_count] = ACTIONS(2133),
    [sym_set_virtual_channel_count] = ACTIONS(2133),
    [sym_set_virtual_port_count] = ACTIONS(2133),
    [sym__wc_encode_int] = ACTIONS(2133),
    [sym__wc_decode_int] = ACTIONS(2133),
    [sym__wc_int] = ACTIONS(2133),
    [sym__wc_to_ch_int] = ACTIONS(2133),
    [sym__ch_to_wc_int] = ACTIONS(2133),
    [sym__wc_find_string_int] = ACTIONS(2133),
    [sym__wc_left_string_int] = ACTIONS(2133),
    [sym__wc_lower_string_int] = ACTIONS(2133),
    [sym__wc_mid_string_int] = ACTIONS(2133),
    [sym__wc_remove_string_int] = ACTIONS(2133),
    [sym__wc_right_string_int] = ACTIONS(2133),
    [sym__wc_upper_string_int] = ACTIONS(2133),
    [sym__wc_compare_string_int] = ACTIONS(2133),
    [sym__wc_get_buffer_char_int] = ACTIONS(2133),
    [sym__wc_get_buffer_string_int] = ACTIONS(2133),
    [sym__wc_concat_string_int] = ACTIONS(2133),
    [sym__wc_file_open_int] = ACTIONS(2133),
    [sym__wc_file_close_int] = ACTIONS(2133),
    [sym__wc_file_read_int] = ACTIONS(2133),
    [sym__wc_file_read_line_int] = ACTIONS(2133),
    [sym__wc_file_write_int] = ACTIONS(2133),
    [sym__wc_file_write_line_int] = ACTIONS(2133),
    [aux_sym_preproc_include_token1] = ACTIONS(2133),
    [aux_sym_preproc_define_token1] = ACTIONS(2135),
    [aux_sym_preproc_warn_token1] = ACTIONS(2135),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2135),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2135),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2135),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2135),
    [aux_sym_preproc_else_token1] = ACTIONS(2135),
    [anon_sym_LBRACK] = ACTIONS(2135),
    [anon_sym_LBRACE] = ACTIONS(2135),
    [anon_sym_LPAREN] = ACTIONS(2133),
    [anon_sym_BANG] = ACTIONS(2135),
    [anon_sym_TILDE] = ACTIONS(2135),
    [anon_sym_DASH] = ACTIONS(2133),
    [anon_sym_PLUS] = ACTIONS(2133),
    [anon_sym_DASH_DASH] = ACTIONS(2135),
    [anon_sym_PLUS_PLUS] = ACTIONS(2135),
    [anon_sym_DQUOTE] = ACTIONS(2135),
    [anon_sym_SQUOTE] = ACTIONS(2135),
    [sym_number_literal] = ACTIONS(2135),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(395)] = {
    [ts_builtin_sym_end] = ACTIONS(2015),
    [sym_identifier] = ACTIONS(2013),
    [sym_program_name_keyword] = ACTIONS(2013),
    [sym_module_name_keyword] = ACTIONS(2013),
    [sym_define_device_keyword] = ACTIONS(2013),
    [sym_define_combine_keyword] = ACTIONS(2013),
    [sym_define_constant_keyword] = ACTIONS(2013),
    [sym_define_type_keyword] = ACTIONS(2013),
    [sym_define_variable_keyword] = ACTIONS(2013),
    [sym_define_system_variable_keyword] = ACTIONS(2013),
    [sym_define_connect_level_keyword] = ACTIONS(2013),
    [sym_define_latching_keyword] = ACTIONS(2013),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2013),
    [sym_define_toggling_keyword] = ACTIONS(2013),
    [sym_define_call_keyword] = ACTIONS(2013),
    [sym_define_function_keyword] = ACTIONS(2013),
    [sym_define_library_function_keyword] = ACTIONS(2013),
    [sym_define_start_keyword] = ACTIONS(2013),
    [sym_define_event_keyword] = ACTIONS(2013),
    [sym_define_program_keyword] = ACTIONS(2013),
    [sym_define_module_keyword] = ACTIONS(2013),
    [sym_char_keyword] = ACTIONS(2013),
    [sym_widechar_keyword] = ACTIONS(2013),
    [sym_integer_keyword] = ACTIONS(2013),
    [sym_sinteger_keyword] = ACTIONS(2013),
    [sym_long_keyword] = ACTIONS(2013),
    [sym_slong_keyword] = ACTIONS(2013),
    [sym_float_keyword] = ACTIONS(2013),
    [sym_double_keyword] = ACTIONS(2013),
    [sym_variant_keyword] = ACTIONS(2013),
    [sym_variantarray_keyword] = ACTIONS(2013),
    [sym_local_var_keyword] = ACTIONS(2013),
    [sym_stack_var_keyword] = ACTIONS(2013),
    [sym_constant_keyword] = ACTIONS(2013),
    [sym_volatile_keyword] = ACTIONS(2013),
    [sym_non_volatile_keyword] = ACTIONS(2013),
    [sym_persistent_keyword] = ACTIONS(2013),
    [sym_dev_keyword] = ACTIONS(2013),
    [sym_devlev_keyword] = ACTIONS(2013),
    [sym_devchan_keyword] = ACTIONS(2013),
    [sym_if_keyword] = ACTIONS(2013),
    [sym_switch_keyword] = ACTIONS(2013),
    [sym_case_keyword] = ACTIONS(2013),
    [sym_default_keyword] = ACTIONS(2013),
    [sym_while_keyword] = ACTIONS(2013),
    [sym_for_keyword] = ACTIONS(2013),
    [sym_break_keyword] = ACTIONS(2013),
    [sym_continue_keyword] = ACTIONS(2013),
    [sym_return_keyword] = ACTIONS(2013),
    [sym_select_keyword] = ACTIONS(2013),
    [sym_active_keyword] = ACTIONS(2013),
    [sym_struct_keyword] = ACTIONS(2013),
    [sym_structure_keyword] = ACTIONS(2013),
    [sym_button_event_keyword] = ACTIONS(2013),
    [sym_channel_event_keyword] = ACTIONS(2013),
    [sym_level_event_keyword] = ACTIONS(2013),
    [sym_data_event_keyword] = ACTIONS(2013),
    [sym_timeline_event_keyword] = ACTIONS(2013),
    [sym_custom_event_keyword] = ACTIONS(2013),
    [sym_push_keyword] = ACTIONS(2013),
    [sym_release_keyword] = ACTIONS(2013),
    [aux_sym_on_keyword_token1] = ACTIONS(2013),
    [aux_sym_off_keyword_token1] = ACTIONS(2013),
    [sym_send_level_keyword] = ACTIONS(2013),
    [sym_send_string_keyword] = ACTIONS(2013),
    [sym_send_command_keyword] = ACTIONS(2013),
    [sym_clear_buffer_keyword] = ACTIONS(2013),
    [sym_create_buffer_keyword] = ACTIONS(2013),
    [sym_create_multi_buffer_keyword] = ACTIONS(2013),
    [sym_call_keyword] = ACTIONS(2013),
    [sym_system_call_keyword] = ACTIONS(2013),
    [sym_devchan_to_keyword] = ACTIONS(2013),
    [sym_devchan_min_to_keyword] = ACTIONS(2013),
    [sym_devchan_total_off_keyword] = ACTIONS(2013),
    [sym_devchan_pulse_keyword] = ACTIONS(2013),
    [sym_wait_keyword] = ACTIONS(2013),
    [sym_cancel_wait_keyword] = ACTIONS(2013),
    [sym_wait_until_keyword] = ACTIONS(2013),
    [sym_cancel_wait_until_keyword] = ACTIONS(2013),
    [sym_cancel_all_wait_keyword] = ACTIONS(2013),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2013),
    [sym_not] = ACTIONS(2013),
    [sym_bnot] = ACTIONS(2013),
    [sym___file__] = ACTIONS(2013),
    [sym___date__] = ACTIONS(2013),
    [sym___ldate__] = ACTIONS(2013),
    [sym___time__] = ACTIONS(2013),
    [sym___line__] = ACTIONS(2013),
    [sym___name__] = ACTIONS(2013),
    [sym___netlinx__] = ACTIONS(2013),
    [sym_netlinx_axi_version] = ACTIONS(2013),
    [sym_url_flg_tcp] = ACTIONS(2013),
    [sym_url_flg_accinfopresent] = ACTIONS(2013),
    [sym_url_flg_temp] = ACTIONS(2013),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2013),
    [sym_url_flg_stat_mask] = ACTIONS(2013),
    [sym_url_flg_stat_lookup] = ACTIONS(2013),
    [sym_url_flg_stat_connecting] = ACTIONS(2013),
    [sym_url_flg_stat_waiting] = ACTIONS(2013),
    [sym_url_flg_stat_connected] = ACTIONS(2013),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2013),
    [sym_first_virtual_device] = ACTIONS(2013),
    [sym_first_local_port] = ACTIONS(2013),
    [sym_dynamic_virtual_device] = ACTIONS(2013),
    [sym_timeline_once] = ACTIONS(2013),
    [sym_timeline_repeat] = ACTIONS(2013),
    [sym_timeline_absolute] = ACTIONS(2013),
    [sym_timeline_relative] = ACTIONS(2013),
    [sym_do_push_timed_infinite] = ACTIONS(2013),
    [sym_source_type_no_address] = ACTIONS(2013),
    [sym_source_type_neuron_id] = ACTIONS(2013),
    [sym_source_type_ip_address] = ACTIONS(2013),
    [sym_source_type_axlink] = ACTIONS(2013),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2013),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2013),
    [sym_source_type_ip_socket_address] = ACTIONS(2013),
    [sym_source_type_rs232] = ACTIONS(2013),
    [sym_source_type_internal] = ACTIONS(2013),
    [sym_source_type_mac_address] = ACTIONS(2013),
    [sym_source_type_ipv4_port] = ACTIONS(2013),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2013),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2013),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2013),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2013),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2013),
    [sym_true] = ACTIONS(2013),
    [sym_false] = ACTIONS(2013),
    [sym_file_read_only] = ACTIONS(2013),
    [sym_file_rw_new] = ACTIONS(2013),
    [sym_file_rw_append] = ACTIONS(2013),
    [sym_ip_tcp] = ACTIONS(2013),
    [sym_ip_udp] = ACTIONS(2013),
    [sym_ip_udp_2way] = ACTIONS(2013),
    [sym_xml_encode_types] = ACTIONS(2013),
    [sym_xml_encode_char_as_list] = ACTIONS(2013),
    [sym_xml_encode_le] = ACTIONS(2013),
    [sym_xml_decode_types] = ACTIONS(2013),
    [sym_xml_decode_no_preserve] = ACTIONS(2013),
    [sym_duet_dev_type_display_device] = ACTIONS(2013),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2013),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2013),
    [sym_duet_dev_type_io_device] = ACTIONS(2013),
    [sym_duet_dev_type_relay_device] = ACTIONS(2013),
    [sym_duet_dev_type_ups] = ACTIONS(2013),
    [sym_duet_dev_type_amplifier] = ACTIONS(2013),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2013),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2013),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2013),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2013),
    [sym_duet_dev_type_camera] = ACTIONS(2013),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2013),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2013),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2013),
    [sym_duet_dev_type_dss] = ACTIONS(2013),
    [sym_duet_dev_type_dvr] = ACTIONS(2013),
    [sym_duet_dev_type_disc_device] = ACTIONS(2013),
    [sym_duet_dev_type_document_camera] = ACTIONS(2013),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2013),
    [sym_duet_dev_type_hvac] = ACTIONS(2013),
    [sym_duet_dev_type_keypad] = ACTIONS(2013),
    [sym_duet_dev_type_light] = ACTIONS(2013),
    [sym_duet_dev_type_monitor] = ACTIONS(2013),
    [sym_duet_dev_type_motor] = ACTIONS(2013),
    [sym_duet_dev_type_multi_window] = ACTIONS(2013),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2013),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2013),
    [sym_duet_dev_type_receiver] = ACTIONS(2013),
    [sym_duet_dev_type_security_system] = ACTIONS(2013),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2013),
    [sym_duet_dev_type_settop_box] = ACTIONS(2013),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2013),
    [sym_duet_dev_type_switcher] = ACTIONS(2013),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2013),
    [sym_duet_dev_type_tv] = ACTIONS(2013),
    [sym_duet_dev_type_utility] = ACTIONS(2013),
    [sym_duet_dev_type_vcr] = ACTIONS(2013),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2013),
    [sym_duet_dev_type_video_processor] = ACTIONS(2013),
    [sym_duet_dev_type_video_projector] = ACTIONS(2013),
    [sym_duet_dev_type_video_wall] = ACTIONS(2013),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2013),
    [sym_duet_dev_type_weather] = ACTIONS(2013),
    [sym_duet_dev_not_polled] = ACTIONS(2013),
    [sym_duet_dev_polled] = ACTIONS(2013),
    [sym_clkmgr_mode_network] = ACTIONS(2013),
    [sym_clkmgr_mode_standalone] = ACTIONS(2013),
    [sym_normal_standby] = ACTIONS(2013),
    [sym_normal_wake] = ACTIONS(2013),
    [sym_ip_multicast_ttl_option] = ACTIONS(2013),
    [sym_ip_ttl_subnet] = ACTIONS(2013),
    [sym_ip_ttl_site] = ACTIONS(2013),
    [sym_ip_ttl_region] = ACTIONS(2013),
    [sym_ip_ttl_continent] = ACTIONS(2013),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2013),
    [sym_ip_nodelay_on] = ACTIONS(2013),
    [sym_ip_nodelay_off] = ACTIONS(2013),
    [sym_amx_error] = ACTIONS(2013),
    [sym_amx_warning] = ACTIONS(2013),
    [sym_amx_info] = ACTIONS(2013),
    [sym_amx_debug] = ACTIONS(2013),
    [sym_netlinx_login_success] = ACTIONS(2013),
    [sym_netlinx_login_fail] = ACTIONS(2013),
    [sym_netlinx_logout] = ACTIONS(2013),
    [sym_null_str] = ACTIONS(2013),
    [sym_smtp_address] = ACTIONS(2013),
    [sym_smtp_port_number] = ACTIONS(2013),
    [sym_smtp_username] = ACTIONS(2013),
    [sym_smtp_password] = ACTIONS(2013),
    [sym_smtp_require_tls] = ACTIONS(2013),
    [sym_smtp_from] = ACTIONS(2013),
    [sym_smtp_tls_true] = ACTIONS(2013),
    [sym_smtp_tls_false] = ACTIONS(2013),
    [sym_listview_on_row_select_event] = ACTIONS(2013),
    [sym_data_structure_unknown] = ACTIONS(2013),
    [sym_data_structure_datafeed] = ACTIONS(2013),
    [sym_data_structure_datarecordset] = ACTIONS(2013),
    [sym_data_structure_datarecord] = ACTIONS(2013),
    [sym_data_structure_datafield] = ACTIONS(2013),
    [sym_data_max_content_field_count] = ACTIONS(2013),
    [sym_data_max_metadata_field_count] = ACTIONS(2013),
    [sym_data_max_id_length] = ACTIONS(2013),
    [sym_data_max_name_length] = ACTIONS(2013),
    [sym_data_max_description_length] = ACTIONS(2013),
    [sym_data_max_type_length] = ACTIONS(2013),
    [sym_data_max_format_length] = ACTIONS(2013),
    [sym_data_max_source_length] = ACTIONS(2013),
    [sym_data_max_value_length] = ACTIONS(2013),
    [sym_data_max_label_length] = ACTIONS(2013),
    [sym_data_max_url_length] = ACTIONS(2013),
    [sym_max_last_login_info_length] = ACTIONS(2013),
    [sym_data_type_unknown] = ACTIONS(2013),
    [sym_data_type_string] = ACTIONS(2013),
    [sym_data_type_datetime] = ACTIONS(2013),
    [sym_data_type_date] = ACTIONS(2013),
    [sym_data_type_time] = ACTIONS(2013),
    [sym_data_type_image] = ACTIONS(2013),
    [sym_data_format_url] = ACTIONS(2013),
    [sym_data_format_phone] = ACTIONS(2013),
    [sym_data_format_email] = ACTIONS(2013),
    [sym_data_format_iso8601] = ACTIONS(2013),
    [sym_auditlog_priv] = ACTIONS(2013),
    [sym_configuration_priv] = ACTIONS(2013),
    [sym_deviceconfig_priv] = ACTIONS(2013),
    [sym_ftp_priv] = ACTIONS(2013),
    [sym_http_priv] = ACTIONS(2013),
    [sym_networkconfig_priv] = ACTIONS(2013),
    [sym_programport_priv] = ACTIONS(2013),
    [sym_remoteui_priv] = ACTIONS(2013),
    [sym_securitycontrol_priv] = ACTIONS(2013),
    [sym_softwaremanagement_priv] = ACTIONS(2013),
    [sym_terminal_priv] = ACTIONS(2013),
    [sym_tpadmin_priv] = ACTIONS(2013),
    [sym_user1_priv] = ACTIONS(2013),
    [sym_user2_priv] = ACTIONS(2013),
    [sym_user3_priv] = ACTIONS(2013),
    [sym_user4_priv] = ACTIONS(2013),
    [sym_usermanagement_priv] = ACTIONS(2013),
    [sym_valid_account] = ACTIONS(2013),
    [sym_err_name_invalid] = ACTIONS(2013),
    [sym_err_password_invalid] = ACTIONS(2013),
    [sym_err_invalid_account] = ACTIONS(2013),
    [sym_err_no_authorization] = ACTIONS(2013),
    [sym_err_invalid_parameter] = ACTIONS(2013),
    [sym_err_user_locked_out] = ACTIONS(2013),
    [sym_err_user_account_expired] = ACTIONS(2013),
    [sym_err_general_error] = ACTIONS(2013),
    [sym_err_authentication_error] = ACTIONS(2013),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2013),
    [sym_tls_validate_certificate] = ACTIONS(2013),
    [sym_tdata] = ACTIONS(2013),
    [sym_tchannel] = ACTIONS(2013),
    [sym_tlevel] = ACTIONS(2013),
    [sym_tbutton] = ACTIONS(2013),
    [sym_ttimeline] = ACTIONS(2013),
    [sym_tcustom] = ACTIONS(2013),
    [sym_url_struct] = ACTIONS(2013),
    [sym_dns_struct] = ACTIONS(2013),
    [sym_ip_address_struct] = ACTIONS(2013),
    [sym_dev_info_struct] = ACTIONS(2013),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2013),
    [sym_data_feed] = ACTIONS(2013),
    [sym_data_field] = ACTIONS(2013),
    [sym_data_record] = ACTIONS(2013),
    [sym_wc_data_feed] = ACTIONS(2013),
    [sym_wc_data_field] = ACTIONS(2013),
    [sym_wc_data_record] = ACTIONS(2013),
    [sym_last_login_info] = ACTIONS(2013),
    [sym____reserved___] = ACTIONS(2013),
    [sym_date] = ACTIONS(2013),
    [sym_day] = ACTIONS(2013),
    [sym_time] = ACTIONS(2013),
    [sym_ldate] = ACTIONS(2013),
    [sym_push_device] = ACTIONS(2013),
    [sym_push_channel] = ACTIONS(2013),
    [sym_push_devchan] = ACTIONS(2013),
    [sym_release_device] = ACTIONS(2013),
    [sym_release_channel] = ACTIONS(2013),
    [sym_release_devchan] = ACTIONS(2013),
    [sym_master_slot] = ACTIONS(2013),
    [sym_get_pulse_time] = ACTIONS(2013),
    [sym_get_timer] = ACTIONS(2013),
    [sym_master_sn] = ACTIONS(2013),
    [sym_system_number] = ACTIONS(2013),
    [sym_button] = ACTIONS(2013),
    [sym_dv_channel] = ACTIONS(2013),
    [sym_data] = ACTIONS(2013),
    [sym_level] = ACTIONS(2013),
    [sym_timeline] = ACTIONS(2013),
    [sym_channel] = ACTIONS(2013),
    [sym_custom] = ACTIONS(2013),
    [sym_atoi] = ACTIONS(2013),
    [sym_atol] = ACTIONS(2013),
    [sym_atof] = ACTIONS(2013),
    [sym_itoa] = ACTIONS(2013),
    [sym_itohex] = ACTIONS(2013),
    [sym_hextoi] = ACTIONS(2013),
    [sym_ftoa] = ACTIONS(2013),
    [sym_find_string] = ACTIONS(2013),
    [sym_left_string] = ACTIONS(2013),
    [sym_right_string] = ACTIONS(2013),
    [sym_mid_string] = ACTIONS(2013),
    [sym_remove_string] = ACTIONS(2013),
    [sym_get_buffer_string] = ACTIONS(2013),
    [sym_get_buffer_char] = ACTIONS(2013),
    [sym_get_multi_buffer_string] = ACTIONS(2013),
    [sym_length_string] = ACTIONS(2013),
    [sym_max_length_string] = ACTIONS(2013),
    [sym_set_length_string] = ACTIONS(2013),
    [sym_compare_string] = ACTIONS(2013),
    [sym_upper_string] = ACTIONS(2013),
    [sym_lower_string] = ACTIONS(2013),
    [sym_get_last] = ACTIONS(2013),
    [sym_date_to_day] = ACTIONS(2013),
    [sym_date_to_month] = ACTIONS(2013),
    [sym_date_to_year] = ACTIONS(2013),
    [sym_time_to_hour] = ACTIONS(2013),
    [sym_time_to_minute] = ACTIONS(2013),
    [sym_time_to_second] = ACTIONS(2013),
    [sym_day_of_week] = ACTIONS(2013),
    [sym_set_pulse_time] = ACTIONS(2013),
    [sym_set_timer] = ACTIONS(2013),
    [sym_file_open] = ACTIONS(2013),
    [sym_file_close] = ACTIONS(2013),
    [sym_file_read] = ACTIONS(2013),
    [sym_file_read_line] = ACTIONS(2013),
    [sym_file_write] = ACTIONS(2013),
    [sym_file_write_line] = ACTIONS(2013),
    [sym_file_copy] = ACTIONS(2013),
    [sym_file_delete] = ACTIONS(2013),
    [sym_file_rename] = ACTIONS(2013),
    [sym_file_dir] = ACTIONS(2013),
    [sym_file_getdir] = ACTIONS(2013),
    [sym_file_setdir] = ACTIONS(2013),
    [sym_file_create_dir] = ACTIONS(2013),
    [sym_file_remove_dir] = ACTIONS(2013),
    [sym_file_seek] = ACTIONS(2013),
    [sym_length_array] = ACTIONS(2013),
    [sym_max_length_array] = ACTIONS(2013),
    [sym_set_length_array] = ACTIONS(2013),
    [sym_ip_client_open] = ACTIONS(2013),
    [sym_ip_client_close] = ACTIONS(2013),
    [sym_ip_server_open] = ACTIONS(2013),
    [sym_ip_server_close] = ACTIONS(2013),
    [sym_ip_bound_client_open] = ACTIONS(2013),
    [sym_ip_mc_server_open] = ACTIONS(2013),
    [sym_ip_set_option] = ACTIONS(2013),
    [sym_ssh_client_open] = ACTIONS(2013),
    [sym_ssh_client_close] = ACTIONS(2013),
    [sym_tls_client_open] = ACTIONS(2013),
    [sym_tls_client_close] = ACTIONS(2013),
    [sym_do_push] = ACTIONS(2013),
    [sym_do_push_timed] = ACTIONS(2013),
    [sym_do_release] = ACTIONS(2013),
    [sym_device_id] = ACTIONS(2013),
    [sym_device_info] = ACTIONS(2013),
    [sym_device_id_string] = ACTIONS(2013),
    [sym_device_standby] = ACTIONS(2013),
    [sym_device_wake] = ACTIONS(2013),
    [sym_timeline_create] = ACTIONS(2013),
    [sym_timeline_kill] = ACTIONS(2013),
    [sym_timeline_reload] = ACTIONS(2013),
    [sym_timeline_pause] = ACTIONS(2013),
    [sym_timeline_restart] = ACTIONS(2013),
    [sym_timeline_set] = ACTIONS(2013),
    [sym_timeline_get] = ACTIONS(2013),
    [sym_timeline_active] = ACTIONS(2013),
    [sym_timeline_dynamic_id] = ACTIONS(2013),
    [sym_variable_to_string] = ACTIONS(2013),
    [sym_string_to_variable] = ACTIONS(2013),
    [sym_length_variable_to_string] = ACTIONS(2013),
    [sym_type_cast] = ACTIONS(2013),
    [sym_variable_to_xml] = ACTIONS(2013),
    [sym_xml_to_variable] = ACTIONS(2013),
    [sym_length_variable_to_xml] = ACTIONS(2013),
    [sym_redirect_string] = ACTIONS(2013),
    [sym_random_number] = ACTIONS(2013),
    [sym_get_unique_id] = ACTIONS(2013),
    [sym_get_system_number] = ACTIONS(2013),
    [sym_set_system_number] = ACTIONS(2013),
    [sym_get_serial_number] = ACTIONS(2013),
    [sym_get_master_build] = ACTIONS(2013),
    [sym_set_validation_code] = ACTIONS(2013),
    [sym_reboot] = ACTIONS(2013),
    [sym_load_duet_module] = ACTIONS(2013),
    [sym_unload_duet_module] = ACTIONS(2013),
    [sym_duet_mem_size_set] = ACTIONS(2013),
    [sym_duet_mem_size_get] = ACTIONS(2013),
    [sym_dynamic_application_device] = ACTIONS(2013),
    [sym_dynamic_polled_port] = ACTIONS(2013),
    [sym_static_port_binding] = ACTIONS(2013),
    [sym_static_ip_binding] = ACTIONS(2013),
    [sym_get_dns_list] = ACTIONS(2013),
    [sym_set_dns_list] = ACTIONS(2013),
    [sym_get_ip_address] = ACTIONS(2013),
    [sym_set_ip_address] = ACTIONS(2013),
    [sym_get_url_list] = ACTIONS(2013),
    [sym_add_url_entry] = ACTIONS(2013),
    [sym_delete_url_entry] = ACTIONS(2013),
    [sym_abs_value] = ACTIONS(2013),
    [sym_max_value] = ACTIONS(2013),
    [sym_min_value] = ACTIONS(2013),
    [sym_exp_value] = ACTIONS(2013),
    [sym_log_value] = ACTIONS(2013),
    [sym_log10_value] = ACTIONS(2013),
    [sym_power_value] = ACTIONS(2013),
    [sym_sqrt_value] = ACTIONS(2013),
    [sym_raw_be] = ACTIONS(2013),
    [sym_raw_le] = ACTIONS(2013),
    [sym_internal_queue_size_set] = ACTIONS(2013),
    [sym_internal_queue_size_get] = ACTIONS(2013),
    [sym_internal_threshold_set] = ACTIONS(2013),
    [sym_internal_threshold_get] = ACTIONS(2013),
    [sym_rebuilt_event] = ACTIONS(2013),
    [sym_do_custom_event] = ACTIONS(2013),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2013),
    [sym_clkmgr_set_clk_source] = ACTIONS(2013),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2013),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2013),
    [sym_clkmgr_get_timezone] = ACTIONS(2013),
    [sym_clkmgr_set_timezone] = ACTIONS(2013),
    [sym_clkmgr_get_resync_period] = ACTIONS(2013),
    [sym_clkmgr_set_resync_period] = ACTIONS(2013),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2013),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2013),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2013),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2013),
    [sym_clkmgr_get_timeservers] = ACTIONS(2013),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2013),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2013),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2013),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2013),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2013),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2013),
    [sym_set_log_level] = ACTIONS(2013),
    [sym_get_log_level] = ACTIONS(2013),
    [sym_amx_log] = ACTIONS(2013),
    [sym_smtp_server_config_set] = ACTIONS(2013),
    [sym_smtp_server_config_get] = ACTIONS(2013),
    [sym_smtp_send] = ACTIONS(2013),
    [sym_validate_netlinx_account] = ACTIONS(2013),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2013),
    [sym_authenticate_certificate] = ACTIONS(2013),
    [sym_audit_netlinx_session_event] = ACTIONS(2013),
    [sym_audit_netlinx_generic_event] = ACTIONS(2013),
    [sym_get_available_flash_disk_space] = ACTIONS(2013),
    [sym_get_max_flash_disk_space] = ACTIONS(2013),
    [sym_data_create_feed] = ACTIONS(2013),
    [sym_data_delete_feed] = ACTIONS(2013),
    [sym_data_publish_feed] = ACTIONS(2013),
    [sym_data_get_published_feed] = ACTIONS(2013),
    [sym_data_add_record] = ACTIONS(2013),
    [sym_data_get_event_record] = ACTIONS(2013),
    [sym__wc_data_create_feed] = ACTIONS(2013),
    [sym__wc_data_add_record] = ACTIONS(2013),
    [sym__wc_data_get_event_record] = ACTIONS(2013),
    [sym_format] = ACTIONS(2013),
    [sym_astro_clock] = ACTIONS(2013),
    [sym_set_outdoor_temperature] = ACTIONS(2013),
    [sym_set_virtual_level_count] = ACTIONS(2013),
    [sym_set_virtual_channel_count] = ACTIONS(2013),
    [sym_set_virtual_port_count] = ACTIONS(2013),
    [sym__wc_encode_int] = ACTIONS(2013),
    [sym__wc_decode_int] = ACTIONS(2013),
    [sym__wc_int] = ACTIONS(2013),
    [sym__wc_to_ch_int] = ACTIONS(2013),
    [sym__ch_to_wc_int] = ACTIONS(2013),
    [sym__wc_find_string_int] = ACTIONS(2013),
    [sym__wc_left_string_int] = ACTIONS(2013),
    [sym__wc_lower_string_int] = ACTIONS(2013),
    [sym__wc_mid_string_int] = ACTIONS(2013),
    [sym__wc_remove_string_int] = ACTIONS(2013),
    [sym__wc_right_string_int] = ACTIONS(2013),
    [sym__wc_upper_string_int] = ACTIONS(2013),
    [sym__wc_compare_string_int] = ACTIONS(2013),
    [sym__wc_get_buffer_char_int] = ACTIONS(2013),
    [sym__wc_get_buffer_string_int] = ACTIONS(2013),
    [sym__wc_concat_string_int] = ACTIONS(2013),
    [sym__wc_file_open_int] = ACTIONS(2013),
    [sym__wc_file_close_int] = ACTIONS(2013),
    [sym__wc_file_read_int] = ACTIONS(2013),
    [sym__wc_file_read_line_int] = ACTIONS(2013),
    [sym__wc_file_write_int] = ACTIONS(2013),
    [sym__wc_file_write_line_int] = ACTIONS(2013),
    [aux_sym_preproc_include_token1] = ACTIONS(2013),
    [aux_sym_preproc_define_token1] = ACTIONS(2015),
    [aux_sym_preproc_warn_token1] = ACTIONS(2015),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2015),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2015),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2015),
    [anon_sym_LBRACK] = ACTIONS(2015),
    [anon_sym_LBRACE] = ACTIONS(2015),
    [anon_sym_LPAREN] = ACTIONS(2013),
    [anon_sym_BANG] = ACTIONS(2015),
    [anon_sym_TILDE] = ACTIONS(2015),
    [anon_sym_DASH] = ACTIONS(2013),
    [anon_sym_PLUS] = ACTIONS(2013),
    [anon_sym_DASH_DASH] = ACTIONS(2015),
    [anon_sym_PLUS_PLUS] = ACTIONS(2015),
    [anon_sym_DQUOTE] = ACTIONS(2015),
    [anon_sym_SQUOTE] = ACTIONS(2015),
    [sym_number_literal] = ACTIONS(2015),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(396)] = {
    [ts_builtin_sym_end] = ACTIONS(2023),
    [sym_identifier] = ACTIONS(2021),
    [sym_program_name_keyword] = ACTIONS(2021),
    [sym_module_name_keyword] = ACTIONS(2021),
    [sym_define_device_keyword] = ACTIONS(2021),
    [sym_define_combine_keyword] = ACTIONS(2021),
    [sym_define_constant_keyword] = ACTIONS(2021),
    [sym_define_type_keyword] = ACTIONS(2021),
    [sym_define_variable_keyword] = ACTIONS(2021),
    [sym_define_system_variable_keyword] = ACTIONS(2021),
    [sym_define_connect_level_keyword] = ACTIONS(2021),
    [sym_define_latching_keyword] = ACTIONS(2021),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2021),
    [sym_define_toggling_keyword] = ACTIONS(2021),
    [sym_define_call_keyword] = ACTIONS(2021),
    [sym_define_function_keyword] = ACTIONS(2021),
    [sym_define_library_function_keyword] = ACTIONS(2021),
    [sym_define_start_keyword] = ACTIONS(2021),
    [sym_define_event_keyword] = ACTIONS(2021),
    [sym_define_program_keyword] = ACTIONS(2021),
    [sym_define_module_keyword] = ACTIONS(2021),
    [sym_char_keyword] = ACTIONS(2021),
    [sym_widechar_keyword] = ACTIONS(2021),
    [sym_integer_keyword] = ACTIONS(2021),
    [sym_sinteger_keyword] = ACTIONS(2021),
    [sym_long_keyword] = ACTIONS(2021),
    [sym_slong_keyword] = ACTIONS(2021),
    [sym_float_keyword] = ACTIONS(2021),
    [sym_double_keyword] = ACTIONS(2021),
    [sym_variant_keyword] = ACTIONS(2021),
    [sym_variantarray_keyword] = ACTIONS(2021),
    [sym_local_var_keyword] = ACTIONS(2021),
    [sym_stack_var_keyword] = ACTIONS(2021),
    [sym_constant_keyword] = ACTIONS(2021),
    [sym_volatile_keyword] = ACTIONS(2021),
    [sym_non_volatile_keyword] = ACTIONS(2021),
    [sym_persistent_keyword] = ACTIONS(2021),
    [sym_dev_keyword] = ACTIONS(2021),
    [sym_devlev_keyword] = ACTIONS(2021),
    [sym_devchan_keyword] = ACTIONS(2021),
    [sym_if_keyword] = ACTIONS(2021),
    [sym_switch_keyword] = ACTIONS(2021),
    [sym_case_keyword] = ACTIONS(2021),
    [sym_default_keyword] = ACTIONS(2021),
    [sym_while_keyword] = ACTIONS(2021),
    [sym_for_keyword] = ACTIONS(2021),
    [sym_break_keyword] = ACTIONS(2021),
    [sym_continue_keyword] = ACTIONS(2021),
    [sym_return_keyword] = ACTIONS(2021),
    [sym_select_keyword] = ACTIONS(2021),
    [sym_active_keyword] = ACTIONS(2021),
    [sym_struct_keyword] = ACTIONS(2021),
    [sym_structure_keyword] = ACTIONS(2021),
    [sym_button_event_keyword] = ACTIONS(2021),
    [sym_channel_event_keyword] = ACTIONS(2021),
    [sym_level_event_keyword] = ACTIONS(2021),
    [sym_data_event_keyword] = ACTIONS(2021),
    [sym_timeline_event_keyword] = ACTIONS(2021),
    [sym_custom_event_keyword] = ACTIONS(2021),
    [sym_push_keyword] = ACTIONS(2021),
    [sym_release_keyword] = ACTIONS(2021),
    [aux_sym_on_keyword_token1] = ACTIONS(2021),
    [aux_sym_off_keyword_token1] = ACTIONS(2021),
    [sym_send_level_keyword] = ACTIONS(2021),
    [sym_send_string_keyword] = ACTIONS(2021),
    [sym_send_command_keyword] = ACTIONS(2021),
    [sym_clear_buffer_keyword] = ACTIONS(2021),
    [sym_create_buffer_keyword] = ACTIONS(2021),
    [sym_create_multi_buffer_keyword] = ACTIONS(2021),
    [sym_call_keyword] = ACTIONS(2021),
    [sym_system_call_keyword] = ACTIONS(2021),
    [sym_devchan_to_keyword] = ACTIONS(2021),
    [sym_devchan_min_to_keyword] = ACTIONS(2021),
    [sym_devchan_total_off_keyword] = ACTIONS(2021),
    [sym_devchan_pulse_keyword] = ACTIONS(2021),
    [sym_wait_keyword] = ACTIONS(2021),
    [sym_cancel_wait_keyword] = ACTIONS(2021),
    [sym_wait_until_keyword] = ACTIONS(2021),
    [sym_cancel_wait_until_keyword] = ACTIONS(2021),
    [sym_cancel_all_wait_keyword] = ACTIONS(2021),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2021),
    [sym_not] = ACTIONS(2021),
    [sym_bnot] = ACTIONS(2021),
    [sym___file__] = ACTIONS(2021),
    [sym___date__] = ACTIONS(2021),
    [sym___ldate__] = ACTIONS(2021),
    [sym___time__] = ACTIONS(2021),
    [sym___line__] = ACTIONS(2021),
    [sym___name__] = ACTIONS(2021),
    [sym___netlinx__] = ACTIONS(2021),
    [sym_netlinx_axi_version] = ACTIONS(2021),
    [sym_url_flg_tcp] = ACTIONS(2021),
    [sym_url_flg_accinfopresent] = ACTIONS(2021),
    [sym_url_flg_temp] = ACTIONS(2021),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2021),
    [sym_url_flg_stat_mask] = ACTIONS(2021),
    [sym_url_flg_stat_lookup] = ACTIONS(2021),
    [sym_url_flg_stat_connecting] = ACTIONS(2021),
    [sym_url_flg_stat_waiting] = ACTIONS(2021),
    [sym_url_flg_stat_connected] = ACTIONS(2021),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2021),
    [sym_first_virtual_device] = ACTIONS(2021),
    [sym_first_local_port] = ACTIONS(2021),
    [sym_dynamic_virtual_device] = ACTIONS(2021),
    [sym_timeline_once] = ACTIONS(2021),
    [sym_timeline_repeat] = ACTIONS(2021),
    [sym_timeline_absolute] = ACTIONS(2021),
    [sym_timeline_relative] = ACTIONS(2021),
    [sym_do_push_timed_infinite] = ACTIONS(2021),
    [sym_source_type_no_address] = ACTIONS(2021),
    [sym_source_type_neuron_id] = ACTIONS(2021),
    [sym_source_type_ip_address] = ACTIONS(2021),
    [sym_source_type_axlink] = ACTIONS(2021),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2021),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2021),
    [sym_source_type_ip_socket_address] = ACTIONS(2021),
    [sym_source_type_rs232] = ACTIONS(2021),
    [sym_source_type_internal] = ACTIONS(2021),
    [sym_source_type_mac_address] = ACTIONS(2021),
    [sym_source_type_ipv4_port] = ACTIONS(2021),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2021),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2021),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2021),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2021),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2021),
    [sym_true] = ACTIONS(2021),
    [sym_false] = ACTIONS(2021),
    [sym_file_read_only] = ACTIONS(2021),
    [sym_file_rw_new] = ACTIONS(2021),
    [sym_file_rw_append] = ACTIONS(2021),
    [sym_ip_tcp] = ACTIONS(2021),
    [sym_ip_udp] = ACTIONS(2021),
    [sym_ip_udp_2way] = ACTIONS(2021),
    [sym_xml_encode_types] = ACTIONS(2021),
    [sym_xml_encode_char_as_list] = ACTIONS(2021),
    [sym_xml_encode_le] = ACTIONS(2021),
    [sym_xml_decode_types] = ACTIONS(2021),
    [sym_xml_decode_no_preserve] = ACTIONS(2021),
    [sym_duet_dev_type_display_device] = ACTIONS(2021),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2021),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2021),
    [sym_duet_dev_type_io_device] = ACTIONS(2021),
    [sym_duet_dev_type_relay_device] = ACTIONS(2021),
    [sym_duet_dev_type_ups] = ACTIONS(2021),
    [sym_duet_dev_type_amplifier] = ACTIONS(2021),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2021),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2021),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2021),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2021),
    [sym_duet_dev_type_camera] = ACTIONS(2021),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2021),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2021),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2021),
    [sym_duet_dev_type_dss] = ACTIONS(2021),
    [sym_duet_dev_type_dvr] = ACTIONS(2021),
    [sym_duet_dev_type_disc_device] = ACTIONS(2021),
    [sym_duet_dev_type_document_camera] = ACTIONS(2021),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2021),
    [sym_duet_dev_type_hvac] = ACTIONS(2021),
    [sym_duet_dev_type_keypad] = ACTIONS(2021),
    [sym_duet_dev_type_light] = ACTIONS(2021),
    [sym_duet_dev_type_monitor] = ACTIONS(2021),
    [sym_duet_dev_type_motor] = ACTIONS(2021),
    [sym_duet_dev_type_multi_window] = ACTIONS(2021),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2021),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2021),
    [sym_duet_dev_type_receiver] = ACTIONS(2021),
    [sym_duet_dev_type_security_system] = ACTIONS(2021),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2021),
    [sym_duet_dev_type_settop_box] = ACTIONS(2021),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2021),
    [sym_duet_dev_type_switcher] = ACTIONS(2021),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2021),
    [sym_duet_dev_type_tv] = ACTIONS(2021),
    [sym_duet_dev_type_utility] = ACTIONS(2021),
    [sym_duet_dev_type_vcr] = ACTIONS(2021),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2021),
    [sym_duet_dev_type_video_processor] = ACTIONS(2021),
    [sym_duet_dev_type_video_projector] = ACTIONS(2021),
    [sym_duet_dev_type_video_wall] = ACTIONS(2021),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2021),
    [sym_duet_dev_type_weather] = ACTIONS(2021),
    [sym_duet_dev_not_polled] = ACTIONS(2021),
    [sym_duet_dev_polled] = ACTIONS(2021),
    [sym_clkmgr_mode_network] = ACTIONS(2021),
    [sym_clkmgr_mode_standalone] = ACTIONS(2021),
    [sym_normal_standby] = ACTIONS(2021),
    [sym_normal_wake] = ACTIONS(2021),
    [sym_ip_multicast_ttl_option] = ACTIONS(2021),
    [sym_ip_ttl_subnet] = ACTIONS(2021),
    [sym_ip_ttl_site] = ACTIONS(2021),
    [sym_ip_ttl_region] = ACTIONS(2021),
    [sym_ip_ttl_continent] = ACTIONS(2021),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2021),
    [sym_ip_nodelay_on] = ACTIONS(2021),
    [sym_ip_nodelay_off] = ACTIONS(2021),
    [sym_amx_error] = ACTIONS(2021),
    [sym_amx_warning] = ACTIONS(2021),
    [sym_amx_info] = ACTIONS(2021),
    [sym_amx_debug] = ACTIONS(2021),
    [sym_netlinx_login_success] = ACTIONS(2021),
    [sym_netlinx_login_fail] = ACTIONS(2021),
    [sym_netlinx_logout] = ACTIONS(2021),
    [sym_null_str] = ACTIONS(2021),
    [sym_smtp_address] = ACTIONS(2021),
    [sym_smtp_port_number] = ACTIONS(2021),
    [sym_smtp_username] = ACTIONS(2021),
    [sym_smtp_password] = ACTIONS(2021),
    [sym_smtp_require_tls] = ACTIONS(2021),
    [sym_smtp_from] = ACTIONS(2021),
    [sym_smtp_tls_true] = ACTIONS(2021),
    [sym_smtp_tls_false] = ACTIONS(2021),
    [sym_listview_on_row_select_event] = ACTIONS(2021),
    [sym_data_structure_unknown] = ACTIONS(2021),
    [sym_data_structure_datafeed] = ACTIONS(2021),
    [sym_data_structure_datarecordset] = ACTIONS(2021),
    [sym_data_structure_datarecord] = ACTIONS(2021),
    [sym_data_structure_datafield] = ACTIONS(2021),
    [sym_data_max_content_field_count] = ACTIONS(2021),
    [sym_data_max_metadata_field_count] = ACTIONS(2021),
    [sym_data_max_id_length] = ACTIONS(2021),
    [sym_data_max_name_length] = ACTIONS(2021),
    [sym_data_max_description_length] = ACTIONS(2021),
    [sym_data_max_type_length] = ACTIONS(2021),
    [sym_data_max_format_length] = ACTIONS(2021),
    [sym_data_max_source_length] = ACTIONS(2021),
    [sym_data_max_value_length] = ACTIONS(2021),
    [sym_data_max_label_length] = ACTIONS(2021),
    [sym_data_max_url_length] = ACTIONS(2021),
    [sym_max_last_login_info_length] = ACTIONS(2021),
    [sym_data_type_unknown] = ACTIONS(2021),
    [sym_data_type_string] = ACTIONS(2021),
    [sym_data_type_datetime] = ACTIONS(2021),
    [sym_data_type_date] = ACTIONS(2021),
    [sym_data_type_time] = ACTIONS(2021),
    [sym_data_type_image] = ACTIONS(2021),
    [sym_data_format_url] = ACTIONS(2021),
    [sym_data_format_phone] = ACTIONS(2021),
    [sym_data_format_email] = ACTIONS(2021),
    [sym_data_format_iso8601] = ACTIONS(2021),
    [sym_auditlog_priv] = ACTIONS(2021),
    [sym_configuration_priv] = ACTIONS(2021),
    [sym_deviceconfig_priv] = ACTIONS(2021),
    [sym_ftp_priv] = ACTIONS(2021),
    [sym_http_priv] = ACTIONS(2021),
    [sym_networkconfig_priv] = ACTIONS(2021),
    [sym_programport_priv] = ACTIONS(2021),
    [sym_remoteui_priv] = ACTIONS(2021),
    [sym_securitycontrol_priv] = ACTIONS(2021),
    [sym_softwaremanagement_priv] = ACTIONS(2021),
    [sym_terminal_priv] = ACTIONS(2021),
    [sym_tpadmin_priv] = ACTIONS(2021),
    [sym_user1_priv] = ACTIONS(2021),
    [sym_user2_priv] = ACTIONS(2021),
    [sym_user3_priv] = ACTIONS(2021),
    [sym_user4_priv] = ACTIONS(2021),
    [sym_usermanagement_priv] = ACTIONS(2021),
    [sym_valid_account] = ACTIONS(2021),
    [sym_err_name_invalid] = ACTIONS(2021),
    [sym_err_password_invalid] = ACTIONS(2021),
    [sym_err_invalid_account] = ACTIONS(2021),
    [sym_err_no_authorization] = ACTIONS(2021),
    [sym_err_invalid_parameter] = ACTIONS(2021),
    [sym_err_user_locked_out] = ACTIONS(2021),
    [sym_err_user_account_expired] = ACTIONS(2021),
    [sym_err_general_error] = ACTIONS(2021),
    [sym_err_authentication_error] = ACTIONS(2021),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2021),
    [sym_tls_validate_certificate] = ACTIONS(2021),
    [sym_tdata] = ACTIONS(2021),
    [sym_tchannel] = ACTIONS(2021),
    [sym_tlevel] = ACTIONS(2021),
    [sym_tbutton] = ACTIONS(2021),
    [sym_ttimeline] = ACTIONS(2021),
    [sym_tcustom] = ACTIONS(2021),
    [sym_url_struct] = ACTIONS(2021),
    [sym_dns_struct] = ACTIONS(2021),
    [sym_ip_address_struct] = ACTIONS(2021),
    [sym_dev_info_struct] = ACTIONS(2021),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2021),
    [sym_data_feed] = ACTIONS(2021),
    [sym_data_field] = ACTIONS(2021),
    [sym_data_record] = ACTIONS(2021),
    [sym_wc_data_feed] = ACTIONS(2021),
    [sym_wc_data_field] = ACTIONS(2021),
    [sym_wc_data_record] = ACTIONS(2021),
    [sym_last_login_info] = ACTIONS(2021),
    [sym____reserved___] = ACTIONS(2021),
    [sym_date] = ACTIONS(2021),
    [sym_day] = ACTIONS(2021),
    [sym_time] = ACTIONS(2021),
    [sym_ldate] = ACTIONS(2021),
    [sym_push_device] = ACTIONS(2021),
    [sym_push_channel] = ACTIONS(2021),
    [sym_push_devchan] = ACTIONS(2021),
    [sym_release_device] = ACTIONS(2021),
    [sym_release_channel] = ACTIONS(2021),
    [sym_release_devchan] = ACTIONS(2021),
    [sym_master_slot] = ACTIONS(2021),
    [sym_get_pulse_time] = ACTIONS(2021),
    [sym_get_timer] = ACTIONS(2021),
    [sym_master_sn] = ACTIONS(2021),
    [sym_system_number] = ACTIONS(2021),
    [sym_button] = ACTIONS(2021),
    [sym_dv_channel] = ACTIONS(2021),
    [sym_data] = ACTIONS(2021),
    [sym_level] = ACTIONS(2021),
    [sym_timeline] = ACTIONS(2021),
    [sym_channel] = ACTIONS(2021),
    [sym_custom] = ACTIONS(2021),
    [sym_atoi] = ACTIONS(2021),
    [sym_atol] = ACTIONS(2021),
    [sym_atof] = ACTIONS(2021),
    [sym_itoa] = ACTIONS(2021),
    [sym_itohex] = ACTIONS(2021),
    [sym_hextoi] = ACTIONS(2021),
    [sym_ftoa] = ACTIONS(2021),
    [sym_find_string] = ACTIONS(2021),
    [sym_left_string] = ACTIONS(2021),
    [sym_right_string] = ACTIONS(2021),
    [sym_mid_string] = ACTIONS(2021),
    [sym_remove_string] = ACTIONS(2021),
    [sym_get_buffer_string] = ACTIONS(2021),
    [sym_get_buffer_char] = ACTIONS(2021),
    [sym_get_multi_buffer_string] = ACTIONS(2021),
    [sym_length_string] = ACTIONS(2021),
    [sym_max_length_string] = ACTIONS(2021),
    [sym_set_length_string] = ACTIONS(2021),
    [sym_compare_string] = ACTIONS(2021),
    [sym_upper_string] = ACTIONS(2021),
    [sym_lower_string] = ACTIONS(2021),
    [sym_get_last] = ACTIONS(2021),
    [sym_date_to_day] = ACTIONS(2021),
    [sym_date_to_month] = ACTIONS(2021),
    [sym_date_to_year] = ACTIONS(2021),
    [sym_time_to_hour] = ACTIONS(2021),
    [sym_time_to_minute] = ACTIONS(2021),
    [sym_time_to_second] = ACTIONS(2021),
    [sym_day_of_week] = ACTIONS(2021),
    [sym_set_pulse_time] = ACTIONS(2021),
    [sym_set_timer] = ACTIONS(2021),
    [sym_file_open] = ACTIONS(2021),
    [sym_file_close] = ACTIONS(2021),
    [sym_file_read] = ACTIONS(2021),
    [sym_file_read_line] = ACTIONS(2021),
    [sym_file_write] = ACTIONS(2021),
    [sym_file_write_line] = ACTIONS(2021),
    [sym_file_copy] = ACTIONS(2021),
    [sym_file_delete] = ACTIONS(2021),
    [sym_file_rename] = ACTIONS(2021),
    [sym_file_dir] = ACTIONS(2021),
    [sym_file_getdir] = ACTIONS(2021),
    [sym_file_setdir] = ACTIONS(2021),
    [sym_file_create_dir] = ACTIONS(2021),
    [sym_file_remove_dir] = ACTIONS(2021),
    [sym_file_seek] = ACTIONS(2021),
    [sym_length_array] = ACTIONS(2021),
    [sym_max_length_array] = ACTIONS(2021),
    [sym_set_length_array] = ACTIONS(2021),
    [sym_ip_client_open] = ACTIONS(2021),
    [sym_ip_client_close] = ACTIONS(2021),
    [sym_ip_server_open] = ACTIONS(2021),
    [sym_ip_server_close] = ACTIONS(2021),
    [sym_ip_bound_client_open] = ACTIONS(2021),
    [sym_ip_mc_server_open] = ACTIONS(2021),
    [sym_ip_set_option] = ACTIONS(2021),
    [sym_ssh_client_open] = ACTIONS(2021),
    [sym_ssh_client_close] = ACTIONS(2021),
    [sym_tls_client_open] = ACTIONS(2021),
    [sym_tls_client_close] = ACTIONS(2021),
    [sym_do_push] = ACTIONS(2021),
    [sym_do_push_timed] = ACTIONS(2021),
    [sym_do_release] = ACTIONS(2021),
    [sym_device_id] = ACTIONS(2021),
    [sym_device_info] = ACTIONS(2021),
    [sym_device_id_string] = ACTIONS(2021),
    [sym_device_standby] = ACTIONS(2021),
    [sym_device_wake] = ACTIONS(2021),
    [sym_timeline_create] = ACTIONS(2021),
    [sym_timeline_kill] = ACTIONS(2021),
    [sym_timeline_reload] = ACTIONS(2021),
    [sym_timeline_pause] = ACTIONS(2021),
    [sym_timeline_restart] = ACTIONS(2021),
    [sym_timeline_set] = ACTIONS(2021),
    [sym_timeline_get] = ACTIONS(2021),
    [sym_timeline_active] = ACTIONS(2021),
    [sym_timeline_dynamic_id] = ACTIONS(2021),
    [sym_variable_to_string] = ACTIONS(2021),
    [sym_string_to_variable] = ACTIONS(2021),
    [sym_length_variable_to_string] = ACTIONS(2021),
    [sym_type_cast] = ACTIONS(2021),
    [sym_variable_to_xml] = ACTIONS(2021),
    [sym_xml_to_variable] = ACTIONS(2021),
    [sym_length_variable_to_xml] = ACTIONS(2021),
    [sym_redirect_string] = ACTIONS(2021),
    [sym_random_number] = ACTIONS(2021),
    [sym_get_unique_id] = ACTIONS(2021),
    [sym_get_system_number] = ACTIONS(2021),
    [sym_set_system_number] = ACTIONS(2021),
    [sym_get_serial_number] = ACTIONS(2021),
    [sym_get_master_build] = ACTIONS(2021),
    [sym_set_validation_code] = ACTIONS(2021),
    [sym_reboot] = ACTIONS(2021),
    [sym_load_duet_module] = ACTIONS(2021),
    [sym_unload_duet_module] = ACTIONS(2021),
    [sym_duet_mem_size_set] = ACTIONS(2021),
    [sym_duet_mem_size_get] = ACTIONS(2021),
    [sym_dynamic_application_device] = ACTIONS(2021),
    [sym_dynamic_polled_port] = ACTIONS(2021),
    [sym_static_port_binding] = ACTIONS(2021),
    [sym_static_ip_binding] = ACTIONS(2021),
    [sym_get_dns_list] = ACTIONS(2021),
    [sym_set_dns_list] = ACTIONS(2021),
    [sym_get_ip_address] = ACTIONS(2021),
    [sym_set_ip_address] = ACTIONS(2021),
    [sym_get_url_list] = ACTIONS(2021),
    [sym_add_url_entry] = ACTIONS(2021),
    [sym_delete_url_entry] = ACTIONS(2021),
    [sym_abs_value] = ACTIONS(2021),
    [sym_max_value] = ACTIONS(2021),
    [sym_min_value] = ACTIONS(2021),
    [sym_exp_value] = ACTIONS(2021),
    [sym_log_value] = ACTIONS(2021),
    [sym_log10_value] = ACTIONS(2021),
    [sym_power_value] = ACTIONS(2021),
    [sym_sqrt_value] = ACTIONS(2021),
    [sym_raw_be] = ACTIONS(2021),
    [sym_raw_le] = ACTIONS(2021),
    [sym_internal_queue_size_set] = ACTIONS(2021),
    [sym_internal_queue_size_get] = ACTIONS(2021),
    [sym_internal_threshold_set] = ACTIONS(2021),
    [sym_internal_threshold_get] = ACTIONS(2021),
    [sym_rebuilt_event] = ACTIONS(2021),
    [sym_do_custom_event] = ACTIONS(2021),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2021),
    [sym_clkmgr_set_clk_source] = ACTIONS(2021),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2021),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2021),
    [sym_clkmgr_get_timezone] = ACTIONS(2021),
    [sym_clkmgr_set_timezone] = ACTIONS(2021),
    [sym_clkmgr_get_resync_period] = ACTIONS(2021),
    [sym_clkmgr_set_resync_period] = ACTIONS(2021),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2021),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2021),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2021),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2021),
    [sym_clkmgr_get_timeservers] = ACTIONS(2021),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2021),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2021),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2021),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2021),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2021),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2021),
    [sym_set_log_level] = ACTIONS(2021),
    [sym_get_log_level] = ACTIONS(2021),
    [sym_amx_log] = ACTIONS(2021),
    [sym_smtp_server_config_set] = ACTIONS(2021),
    [sym_smtp_server_config_get] = ACTIONS(2021),
    [sym_smtp_send] = ACTIONS(2021),
    [sym_validate_netlinx_account] = ACTIONS(2021),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2021),
    [sym_authenticate_certificate] = ACTIONS(2021),
    [sym_audit_netlinx_session_event] = ACTIONS(2021),
    [sym_audit_netlinx_generic_event] = ACTIONS(2021),
    [sym_get_available_flash_disk_space] = ACTIONS(2021),
    [sym_get_max_flash_disk_space] = ACTIONS(2021),
    [sym_data_create_feed] = ACTIONS(2021),
    [sym_data_delete_feed] = ACTIONS(2021),
    [sym_data_publish_feed] = ACTIONS(2021),
    [sym_data_get_published_feed] = ACTIONS(2021),
    [sym_data_add_record] = ACTIONS(2021),
    [sym_data_get_event_record] = ACTIONS(2021),
    [sym__wc_data_create_feed] = ACTIONS(2021),
    [sym__wc_data_add_record] = ACTIONS(2021),
    [sym__wc_data_get_event_record] = ACTIONS(2021),
    [sym_format] = ACTIONS(2021),
    [sym_astro_clock] = ACTIONS(2021),
    [sym_set_outdoor_temperature] = ACTIONS(2021),
    [sym_set_virtual_level_count] = ACTIONS(2021),
    [sym_set_virtual_channel_count] = ACTIONS(2021),
    [sym_set_virtual_port_count] = ACTIONS(2021),
    [sym__wc_encode_int] = ACTIONS(2021),
    [sym__wc_decode_int] = ACTIONS(2021),
    [sym__wc_int] = ACTIONS(2021),
    [sym__wc_to_ch_int] = ACTIONS(2021),
    [sym__ch_to_wc_int] = ACTIONS(2021),
    [sym__wc_find_string_int] = ACTIONS(2021),
    [sym__wc_left_string_int] = ACTIONS(2021),
    [sym__wc_lower_string_int] = ACTIONS(2021),
    [sym__wc_mid_string_int] = ACTIONS(2021),
    [sym__wc_remove_string_int] = ACTIONS(2021),
    [sym__wc_right_string_int] = ACTIONS(2021),
    [sym__wc_upper_string_int] = ACTIONS(2021),
    [sym__wc_compare_string_int] = ACTIONS(2021),
    [sym__wc_get_buffer_char_int] = ACTIONS(2021),
    [sym__wc_get_buffer_string_int] = ACTIONS(2021),
    [sym__wc_concat_string_int] = ACTIONS(2021),
    [sym__wc_file_open_int] = ACTIONS(2021),
    [sym__wc_file_close_int] = ACTIONS(2021),
    [sym__wc_file_read_int] = ACTIONS(2021),
    [sym__wc_file_read_line_int] = ACTIONS(2021),
    [sym__wc_file_write_int] = ACTIONS(2021),
    [sym__wc_file_write_line_int] = ACTIONS(2021),
    [aux_sym_preproc_include_token1] = ACTIONS(2021),
    [aux_sym_preproc_define_token1] = ACTIONS(2023),
    [aux_sym_preproc_warn_token1] = ACTIONS(2023),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2023),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2023),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2023),
    [anon_sym_LBRACK] = ACTIONS(2023),
    [anon_sym_LBRACE] = ACTIONS(2023),
    [anon_sym_LPAREN] = ACTIONS(2021),
    [anon_sym_BANG] = ACTIONS(2023),
    [anon_sym_TILDE] = ACTIONS(2023),
    [anon_sym_DASH] = ACTIONS(2021),
    [anon_sym_PLUS] = ACTIONS(2021),
    [anon_sym_DASH_DASH] = ACTIONS(2023),
    [anon_sym_PLUS_PLUS] = ACTIONS(2023),
    [anon_sym_DQUOTE] = ACTIONS(2023),
    [anon_sym_SQUOTE] = ACTIONS(2023),
    [sym_number_literal] = ACTIONS(2023),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(397)] = {
    [ts_builtin_sym_end] = ACTIONS(2083),
    [sym_identifier] = ACTIONS(2081),
    [sym_program_name_keyword] = ACTIONS(2081),
    [sym_module_name_keyword] = ACTIONS(2081),
    [sym_define_device_keyword] = ACTIONS(2081),
    [sym_define_combine_keyword] = ACTIONS(2081),
    [sym_define_constant_keyword] = ACTIONS(2081),
    [sym_define_type_keyword] = ACTIONS(2081),
    [sym_define_variable_keyword] = ACTIONS(2081),
    [sym_define_system_variable_keyword] = ACTIONS(2081),
    [sym_define_connect_level_keyword] = ACTIONS(2081),
    [sym_define_latching_keyword] = ACTIONS(2081),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2081),
    [sym_define_toggling_keyword] = ACTIONS(2081),
    [sym_define_call_keyword] = ACTIONS(2081),
    [sym_define_function_keyword] = ACTIONS(2081),
    [sym_define_library_function_keyword] = ACTIONS(2081),
    [sym_define_start_keyword] = ACTIONS(2081),
    [sym_define_event_keyword] = ACTIONS(2081),
    [sym_define_program_keyword] = ACTIONS(2081),
    [sym_define_module_keyword] = ACTIONS(2081),
    [sym_char_keyword] = ACTIONS(2081),
    [sym_widechar_keyword] = ACTIONS(2081),
    [sym_integer_keyword] = ACTIONS(2081),
    [sym_sinteger_keyword] = ACTIONS(2081),
    [sym_long_keyword] = ACTIONS(2081),
    [sym_slong_keyword] = ACTIONS(2081),
    [sym_float_keyword] = ACTIONS(2081),
    [sym_double_keyword] = ACTIONS(2081),
    [sym_variant_keyword] = ACTIONS(2081),
    [sym_variantarray_keyword] = ACTIONS(2081),
    [sym_local_var_keyword] = ACTIONS(2081),
    [sym_stack_var_keyword] = ACTIONS(2081),
    [sym_constant_keyword] = ACTIONS(2081),
    [sym_volatile_keyword] = ACTIONS(2081),
    [sym_non_volatile_keyword] = ACTIONS(2081),
    [sym_persistent_keyword] = ACTIONS(2081),
    [sym_dev_keyword] = ACTIONS(2081),
    [sym_devlev_keyword] = ACTIONS(2081),
    [sym_devchan_keyword] = ACTIONS(2081),
    [sym_if_keyword] = ACTIONS(2081),
    [sym_switch_keyword] = ACTIONS(2081),
    [sym_case_keyword] = ACTIONS(2081),
    [sym_default_keyword] = ACTIONS(2081),
    [sym_while_keyword] = ACTIONS(2081),
    [sym_for_keyword] = ACTIONS(2081),
    [sym_break_keyword] = ACTIONS(2081),
    [sym_continue_keyword] = ACTIONS(2081),
    [sym_return_keyword] = ACTIONS(2081),
    [sym_select_keyword] = ACTIONS(2081),
    [sym_active_keyword] = ACTIONS(2081),
    [sym_struct_keyword] = ACTIONS(2081),
    [sym_structure_keyword] = ACTIONS(2081),
    [sym_button_event_keyword] = ACTIONS(2081),
    [sym_channel_event_keyword] = ACTIONS(2081),
    [sym_level_event_keyword] = ACTIONS(2081),
    [sym_data_event_keyword] = ACTIONS(2081),
    [sym_timeline_event_keyword] = ACTIONS(2081),
    [sym_custom_event_keyword] = ACTIONS(2081),
    [sym_push_keyword] = ACTIONS(2081),
    [sym_release_keyword] = ACTIONS(2081),
    [aux_sym_on_keyword_token1] = ACTIONS(2081),
    [aux_sym_off_keyword_token1] = ACTIONS(2081),
    [sym_send_level_keyword] = ACTIONS(2081),
    [sym_send_string_keyword] = ACTIONS(2081),
    [sym_send_command_keyword] = ACTIONS(2081),
    [sym_clear_buffer_keyword] = ACTIONS(2081),
    [sym_create_buffer_keyword] = ACTIONS(2081),
    [sym_create_multi_buffer_keyword] = ACTIONS(2081),
    [sym_call_keyword] = ACTIONS(2081),
    [sym_system_call_keyword] = ACTIONS(2081),
    [sym_devchan_to_keyword] = ACTIONS(2081),
    [sym_devchan_min_to_keyword] = ACTIONS(2081),
    [sym_devchan_total_off_keyword] = ACTIONS(2081),
    [sym_devchan_pulse_keyword] = ACTIONS(2081),
    [sym_wait_keyword] = ACTIONS(2081),
    [sym_cancel_wait_keyword] = ACTIONS(2081),
    [sym_wait_until_keyword] = ACTIONS(2081),
    [sym_cancel_wait_until_keyword] = ACTIONS(2081),
    [sym_cancel_all_wait_keyword] = ACTIONS(2081),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2081),
    [sym_not] = ACTIONS(2081),
    [sym_bnot] = ACTIONS(2081),
    [sym___file__] = ACTIONS(2081),
    [sym___date__] = ACTIONS(2081),
    [sym___ldate__] = ACTIONS(2081),
    [sym___time__] = ACTIONS(2081),
    [sym___line__] = ACTIONS(2081),
    [sym___name__] = ACTIONS(2081),
    [sym___netlinx__] = ACTIONS(2081),
    [sym_netlinx_axi_version] = ACTIONS(2081),
    [sym_url_flg_tcp] = ACTIONS(2081),
    [sym_url_flg_accinfopresent] = ACTIONS(2081),
    [sym_url_flg_temp] = ACTIONS(2081),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2081),
    [sym_url_flg_stat_mask] = ACTIONS(2081),
    [sym_url_flg_stat_lookup] = ACTIONS(2081),
    [sym_url_flg_stat_connecting] = ACTIONS(2081),
    [sym_url_flg_stat_waiting] = ACTIONS(2081),
    [sym_url_flg_stat_connected] = ACTIONS(2081),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2081),
    [sym_first_virtual_device] = ACTIONS(2081),
    [sym_first_local_port] = ACTIONS(2081),
    [sym_dynamic_virtual_device] = ACTIONS(2081),
    [sym_timeline_once] = ACTIONS(2081),
    [sym_timeline_repeat] = ACTIONS(2081),
    [sym_timeline_absolute] = ACTIONS(2081),
    [sym_timeline_relative] = ACTIONS(2081),
    [sym_do_push_timed_infinite] = ACTIONS(2081),
    [sym_source_type_no_address] = ACTIONS(2081),
    [sym_source_type_neuron_id] = ACTIONS(2081),
    [sym_source_type_ip_address] = ACTIONS(2081),
    [sym_source_type_axlink] = ACTIONS(2081),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2081),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2081),
    [sym_source_type_ip_socket_address] = ACTIONS(2081),
    [sym_source_type_rs232] = ACTIONS(2081),
    [sym_source_type_internal] = ACTIONS(2081),
    [sym_source_type_mac_address] = ACTIONS(2081),
    [sym_source_type_ipv4_port] = ACTIONS(2081),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2081),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2081),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2081),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2081),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2081),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2081),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2081),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2081),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2081),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2081),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2081),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2081),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2081),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2081),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2081),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2081),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2081),
    [sym_true] = ACTIONS(2081),
    [sym_false] = ACTIONS(2081),
    [sym_file_read_only] = ACTIONS(2081),
    [sym_file_rw_new] = ACTIONS(2081),
    [sym_file_rw_append] = ACTIONS(2081),
    [sym_ip_tcp] = ACTIONS(2081),
    [sym_ip_udp] = ACTIONS(2081),
    [sym_ip_udp_2way] = ACTIONS(2081),
    [sym_xml_encode_types] = ACTIONS(2081),
    [sym_xml_encode_char_as_list] = ACTIONS(2081),
    [sym_xml_encode_le] = ACTIONS(2081),
    [sym_xml_decode_types] = ACTIONS(2081),
    [sym_xml_decode_no_preserve] = ACTIONS(2081),
    [sym_duet_dev_type_display_device] = ACTIONS(2081),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2081),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2081),
    [sym_duet_dev_type_io_device] = ACTIONS(2081),
    [sym_duet_dev_type_relay_device] = ACTIONS(2081),
    [sym_duet_dev_type_ups] = ACTIONS(2081),
    [sym_duet_dev_type_amplifier] = ACTIONS(2081),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2081),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2081),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2081),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2081),
    [sym_duet_dev_type_camera] = ACTIONS(2081),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2081),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2081),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2081),
    [sym_duet_dev_type_dss] = ACTIONS(2081),
    [sym_duet_dev_type_dvr] = ACTIONS(2081),
    [sym_duet_dev_type_disc_device] = ACTIONS(2081),
    [sym_duet_dev_type_document_camera] = ACTIONS(2081),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2081),
    [sym_duet_dev_type_hvac] = ACTIONS(2081),
    [sym_duet_dev_type_keypad] = ACTIONS(2081),
    [sym_duet_dev_type_light] = ACTIONS(2081),
    [sym_duet_dev_type_monitor] = ACTIONS(2081),
    [sym_duet_dev_type_motor] = ACTIONS(2081),
    [sym_duet_dev_type_multi_window] = ACTIONS(2081),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2081),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2081),
    [sym_duet_dev_type_receiver] = ACTIONS(2081),
    [sym_duet_dev_type_security_system] = ACTIONS(2081),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2081),
    [sym_duet_dev_type_settop_box] = ACTIONS(2081),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2081),
    [sym_duet_dev_type_switcher] = ACTIONS(2081),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2081),
    [sym_duet_dev_type_tv] = ACTIONS(2081),
    [sym_duet_dev_type_utility] = ACTIONS(2081),
    [sym_duet_dev_type_vcr] = ACTIONS(2081),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2081),
    [sym_duet_dev_type_video_processor] = ACTIONS(2081),
    [sym_duet_dev_type_video_projector] = ACTIONS(2081),
    [sym_duet_dev_type_video_wall] = ACTIONS(2081),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2081),
    [sym_duet_dev_type_weather] = ACTIONS(2081),
    [sym_duet_dev_not_polled] = ACTIONS(2081),
    [sym_duet_dev_polled] = ACTIONS(2081),
    [sym_clkmgr_mode_network] = ACTIONS(2081),
    [sym_clkmgr_mode_standalone] = ACTIONS(2081),
    [sym_normal_standby] = ACTIONS(2081),
    [sym_normal_wake] = ACTIONS(2081),
    [sym_ip_multicast_ttl_option] = ACTIONS(2081),
    [sym_ip_ttl_subnet] = ACTIONS(2081),
    [sym_ip_ttl_site] = ACTIONS(2081),
    [sym_ip_ttl_region] = ACTIONS(2081),
    [sym_ip_ttl_continent] = ACTIONS(2081),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2081),
    [sym_ip_nodelay_on] = ACTIONS(2081),
    [sym_ip_nodelay_off] = ACTIONS(2081),
    [sym_amx_error] = ACTIONS(2081),
    [sym_amx_warning] = ACTIONS(2081),
    [sym_amx_info] = ACTIONS(2081),
    [sym_amx_debug] = ACTIONS(2081),
    [sym_netlinx_login_success] = ACTIONS(2081),
    [sym_netlinx_login_fail] = ACTIONS(2081),
    [sym_netlinx_logout] = ACTIONS(2081),
    [sym_null_str] = ACTIONS(2081),
    [sym_smtp_address] = ACTIONS(2081),
    [sym_smtp_port_number] = ACTIONS(2081),
    [sym_smtp_username] = ACTIONS(2081),
    [sym_smtp_password] = ACTIONS(2081),
    [sym_smtp_require_tls] = ACTIONS(2081),
    [sym_smtp_from] = ACTIONS(2081),
    [sym_smtp_tls_true] = ACTIONS(2081),
    [sym_smtp_tls_false] = ACTIONS(2081),
    [sym_listview_on_row_select_event] = ACTIONS(2081),
    [sym_data_structure_unknown] = ACTIONS(2081),
    [sym_data_structure_datafeed] = ACTIONS(2081),
    [sym_data_structure_datarecordset] = ACTIONS(2081),
    [sym_data_structure_datarecord] = ACTIONS(2081),
    [sym_data_structure_datafield] = ACTIONS(2081),
    [sym_data_max_content_field_count] = ACTIONS(2081),
    [sym_data_max_metadata_field_count] = ACTIONS(2081),
    [sym_data_max_id_length] = ACTIONS(2081),
    [sym_data_max_name_length] = ACTIONS(2081),
    [sym_data_max_description_length] = ACTIONS(2081),
    [sym_data_max_type_length] = ACTIONS(2081),
    [sym_data_max_format_length] = ACTIONS(2081),
    [sym_data_max_source_length] = ACTIONS(2081),
    [sym_data_max_value_length] = ACTIONS(2081),
    [sym_data_max_label_length] = ACTIONS(2081),
    [sym_data_max_url_length] = ACTIONS(2081),
    [sym_max_last_login_info_length] = ACTIONS(2081),
    [sym_data_type_unknown] = ACTIONS(2081),
    [sym_data_type_string] = ACTIONS(2081),
    [sym_data_type_datetime] = ACTIONS(2081),
    [sym_data_type_date] = ACTIONS(2081),
    [sym_data_type_time] = ACTIONS(2081),
    [sym_data_type_image] = ACTIONS(2081),
    [sym_data_format_url] = ACTIONS(2081),
    [sym_data_format_phone] = ACTIONS(2081),
    [sym_data_format_email] = ACTIONS(2081),
    [sym_data_format_iso8601] = ACTIONS(2081),
    [sym_auditlog_priv] = ACTIONS(2081),
    [sym_configuration_priv] = ACTIONS(2081),
    [sym_deviceconfig_priv] = ACTIONS(2081),
    [sym_ftp_priv] = ACTIONS(2081),
    [sym_http_priv] = ACTIONS(2081),
    [sym_networkconfig_priv] = ACTIONS(2081),
    [sym_programport_priv] = ACTIONS(2081),
    [sym_remoteui_priv] = ACTIONS(2081),
    [sym_securitycontrol_priv] = ACTIONS(2081),
    [sym_softwaremanagement_priv] = ACTIONS(2081),
    [sym_terminal_priv] = ACTIONS(2081),
    [sym_tpadmin_priv] = ACTIONS(2081),
    [sym_user1_priv] = ACTIONS(2081),
    [sym_user2_priv] = ACTIONS(2081),
    [sym_user3_priv] = ACTIONS(2081),
    [sym_user4_priv] = ACTIONS(2081),
    [sym_usermanagement_priv] = ACTIONS(2081),
    [sym_valid_account] = ACTIONS(2081),
    [sym_err_name_invalid] = ACTIONS(2081),
    [sym_err_password_invalid] = ACTIONS(2081),
    [sym_err_invalid_account] = ACTIONS(2081),
    [sym_err_no_authorization] = ACTIONS(2081),
    [sym_err_invalid_parameter] = ACTIONS(2081),
    [sym_err_user_locked_out] = ACTIONS(2081),
    [sym_err_user_account_expired] = ACTIONS(2081),
    [sym_err_general_error] = ACTIONS(2081),
    [sym_err_authentication_error] = ACTIONS(2081),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2081),
    [sym_tls_validate_certificate] = ACTIONS(2081),
    [sym_tdata] = ACTIONS(2081),
    [sym_tchannel] = ACTIONS(2081),
    [sym_tlevel] = ACTIONS(2081),
    [sym_tbutton] = ACTIONS(2081),
    [sym_ttimeline] = ACTIONS(2081),
    [sym_tcustom] = ACTIONS(2081),
    [sym_url_struct] = ACTIONS(2081),
    [sym_dns_struct] = ACTIONS(2081),
    [sym_ip_address_struct] = ACTIONS(2081),
    [sym_dev_info_struct] = ACTIONS(2081),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2081),
    [sym_data_feed] = ACTIONS(2081),
    [sym_data_field] = ACTIONS(2081),
    [sym_data_record] = ACTIONS(2081),
    [sym_wc_data_feed] = ACTIONS(2081),
    [sym_wc_data_field] = ACTIONS(2081),
    [sym_wc_data_record] = ACTIONS(2081),
    [sym_last_login_info] = ACTIONS(2081),
    [sym____reserved___] = ACTIONS(2081),
    [sym_date] = ACTIONS(2081),
    [sym_day] = ACTIONS(2081),
    [sym_time] = ACTIONS(2081),
    [sym_ldate] = ACTIONS(2081),
    [sym_push_device] = ACTIONS(2081),
    [sym_push_channel] = ACTIONS(2081),
    [sym_push_devchan] = ACTIONS(2081),
    [sym_release_device] = ACTIONS(2081),
    [sym_release_channel] = ACTIONS(2081),
    [sym_release_devchan] = ACTIONS(2081),
    [sym_master_slot] = ACTIONS(2081),
    [sym_get_pulse_time] = ACTIONS(2081),
    [sym_get_timer] = ACTIONS(2081),
    [sym_master_sn] = ACTIONS(2081),
    [sym_system_number] = ACTIONS(2081),
    [sym_button] = ACTIONS(2081),
    [sym_dv_channel] = ACTIONS(2081),
    [sym_data] = ACTIONS(2081),
    [sym_level] = ACTIONS(2081),
    [sym_timeline] = ACTIONS(2081),
    [sym_channel] = ACTIONS(2081),
    [sym_custom] = ACTIONS(2081),
    [sym_atoi] = ACTIONS(2081),
    [sym_atol] = ACTIONS(2081),
    [sym_atof] = ACTIONS(2081),
    [sym_itoa] = ACTIONS(2081),
    [sym_itohex] = ACTIONS(2081),
    [sym_hextoi] = ACTIONS(2081),
    [sym_ftoa] = ACTIONS(2081),
    [sym_find_string] = ACTIONS(2081),
    [sym_left_string] = ACTIONS(2081),
    [sym_right_string] = ACTIONS(2081),
    [sym_mid_string] = ACTIONS(2081),
    [sym_remove_string] = ACTIONS(2081),
    [sym_get_buffer_string] = ACTIONS(2081),
    [sym_get_buffer_char] = ACTIONS(2081),
    [sym_get_multi_buffer_string] = ACTIONS(2081),
    [sym_length_string] = ACTIONS(2081),
    [sym_max_length_string] = ACTIONS(2081),
    [sym_set_length_string] = ACTIONS(2081),
    [sym_compare_string] = ACTIONS(2081),
    [sym_upper_string] = ACTIONS(2081),
    [sym_lower_string] = ACTIONS(2081),
    [sym_get_last] = ACTIONS(2081),
    [sym_date_to_day] = ACTIONS(2081),
    [sym_date_to_month] = ACTIONS(2081),
    [sym_date_to_year] = ACTIONS(2081),
    [sym_time_to_hour] = ACTIONS(2081),
    [sym_time_to_minute] = ACTIONS(2081),
    [sym_time_to_second] = ACTIONS(2081),
    [sym_day_of_week] = ACTIONS(2081),
    [sym_set_pulse_time] = ACTIONS(2081),
    [sym_set_timer] = ACTIONS(2081),
    [sym_file_open] = ACTIONS(2081),
    [sym_file_close] = ACTIONS(2081),
    [sym_file_read] = ACTIONS(2081),
    [sym_file_read_line] = ACTIONS(2081),
    [sym_file_write] = ACTIONS(2081),
    [sym_file_write_line] = ACTIONS(2081),
    [sym_file_copy] = ACTIONS(2081),
    [sym_file_delete] = ACTIONS(2081),
    [sym_file_rename] = ACTIONS(2081),
    [sym_file_dir] = ACTIONS(2081),
    [sym_file_getdir] = ACTIONS(2081),
    [sym_file_setdir] = ACTIONS(2081),
    [sym_file_create_dir] = ACTIONS(2081),
    [sym_file_remove_dir] = ACTIONS(2081),
    [sym_file_seek] = ACTIONS(2081),
    [sym_length_array] = ACTIONS(2081),
    [sym_max_length_array] = ACTIONS(2081),
    [sym_set_length_array] = ACTIONS(2081),
    [sym_ip_client_open] = ACTIONS(2081),
    [sym_ip_client_close] = ACTIONS(2081),
    [sym_ip_server_open] = ACTIONS(2081),
    [sym_ip_server_close] = ACTIONS(2081),
    [sym_ip_bound_client_open] = ACTIONS(2081),
    [sym_ip_mc_server_open] = ACTIONS(2081),
    [sym_ip_set_option] = ACTIONS(2081),
    [sym_ssh_client_open] = ACTIONS(2081),
    [sym_ssh_client_close] = ACTIONS(2081),
    [sym_tls_client_open] = ACTIONS(2081),
    [sym_tls_client_close] = ACTIONS(2081),
    [sym_do_push] = ACTIONS(2081),
    [sym_do_push_timed] = ACTIONS(2081),
    [sym_do_release] = ACTIONS(2081),
    [sym_device_id] = ACTIONS(2081),
    [sym_device_info] = ACTIONS(2081),
    [sym_device_id_string] = ACTIONS(2081),
    [sym_device_standby] = ACTIONS(2081),
    [sym_device_wake] = ACTIONS(2081),
    [sym_timeline_create] = ACTIONS(2081),
    [sym_timeline_kill] = ACTIONS(2081),
    [sym_timeline_reload] = ACTIONS(2081),
    [sym_timeline_pause] = ACTIONS(2081),
    [sym_timeline_restart] = ACTIONS(2081),
    [sym_timeline_set] = ACTIONS(2081),
    [sym_timeline_get] = ACTIONS(2081),
    [sym_timeline_active] = ACTIONS(2081),
    [sym_timeline_dynamic_id] = ACTIONS(2081),
    [sym_variable_to_string] = ACTIONS(2081),
    [sym_string_to_variable] = ACTIONS(2081),
    [sym_length_variable_to_string] = ACTIONS(2081),
    [sym_type_cast] = ACTIONS(2081),
    [sym_variable_to_xml] = ACTIONS(2081),
    [sym_xml_to_variable] = ACTIONS(2081),
    [sym_length_variable_to_xml] = ACTIONS(2081),
    [sym_redirect_string] = ACTIONS(2081),
    [sym_random_number] = ACTIONS(2081),
    [sym_get_unique_id] = ACTIONS(2081),
    [sym_get_system_number] = ACTIONS(2081),
    [sym_set_system_number] = ACTIONS(2081),
    [sym_get_serial_number] = ACTIONS(2081),
    [sym_get_master_build] = ACTIONS(2081),
    [sym_set_validation_code] = ACTIONS(2081),
    [sym_reboot] = ACTIONS(2081),
    [sym_load_duet_module] = ACTIONS(2081),
    [sym_unload_duet_module] = ACTIONS(2081),
    [sym_duet_mem_size_set] = ACTIONS(2081),
    [sym_duet_mem_size_get] = ACTIONS(2081),
    [sym_dynamic_application_device] = ACTIONS(2081),
    [sym_dynamic_polled_port] = ACTIONS(2081),
    [sym_static_port_binding] = ACTIONS(2081),
    [sym_static_ip_binding] = ACTIONS(2081),
    [sym_get_dns_list] = ACTIONS(2081),
    [sym_set_dns_list] = ACTIONS(2081),
    [sym_get_ip_address] = ACTIONS(2081),
    [sym_set_ip_address] = ACTIONS(2081),
    [sym_get_url_list] = ACTIONS(2081),
    [sym_add_url_entry] = ACTIONS(2081),
    [sym_delete_url_entry] = ACTIONS(2081),
    [sym_abs_value] = ACTIONS(2081),
    [sym_max_value] = ACTIONS(2081),
    [sym_min_value] = ACTIONS(2081),
    [sym_exp_value] = ACTIONS(2081),
    [sym_log_value] = ACTIONS(2081),
    [sym_log10_value] = ACTIONS(2081),
    [sym_power_value] = ACTIONS(2081),
    [sym_sqrt_value] = ACTIONS(2081),
    [sym_raw_be] = ACTIONS(2081),
    [sym_raw_le] = ACTIONS(2081),
    [sym_internal_queue_size_set] = ACTIONS(2081),
    [sym_internal_queue_size_get] = ACTIONS(2081),
    [sym_internal_threshold_set] = ACTIONS(2081),
    [sym_internal_threshold_get] = ACTIONS(2081),
    [sym_rebuilt_event] = ACTIONS(2081),
    [sym_do_custom_event] = ACTIONS(2081),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2081),
    [sym_clkmgr_set_clk_source] = ACTIONS(2081),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2081),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2081),
    [sym_clkmgr_get_timezone] = ACTIONS(2081),
    [sym_clkmgr_set_timezone] = ACTIONS(2081),
    [sym_clkmgr_get_resync_period] = ACTIONS(2081),
    [sym_clkmgr_set_resync_period] = ACTIONS(2081),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2081),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2081),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2081),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2081),
    [sym_clkmgr_get_timeservers] = ACTIONS(2081),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2081),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2081),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2081),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2081),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2081),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2081),
    [sym_set_log_level] = ACTIONS(2081),
    [sym_get_log_level] = ACTIONS(2081),
    [sym_amx_log] = ACTIONS(2081),
    [sym_smtp_server_config_set] = ACTIONS(2081),
    [sym_smtp_server_config_get] = ACTIONS(2081),
    [sym_smtp_send] = ACTIONS(2081),
    [sym_validate_netlinx_account] = ACTIONS(2081),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2081),
    [sym_authenticate_certificate] = ACTIONS(2081),
    [sym_audit_netlinx_session_event] = ACTIONS(2081),
    [sym_audit_netlinx_generic_event] = ACTIONS(2081),
    [sym_get_available_flash_disk_space] = ACTIONS(2081),
    [sym_get_max_flash_disk_space] = ACTIONS(2081),
    [sym_data_create_feed] = ACTIONS(2081),
    [sym_data_delete_feed] = ACTIONS(2081),
    [sym_data_publish_feed] = ACTIONS(2081),
    [sym_data_get_published_feed] = ACTIONS(2081),
    [sym_data_add_record] = ACTIONS(2081),
    [sym_data_get_event_record] = ACTIONS(2081),
    [sym__wc_data_create_feed] = ACTIONS(2081),
    [sym__wc_data_add_record] = ACTIONS(2081),
    [sym__wc_data_get_event_record] = ACTIONS(2081),
    [sym_format] = ACTIONS(2081),
    [sym_astro_clock] = ACTIONS(2081),
    [sym_set_outdoor_temperature] = ACTIONS(2081),
    [sym_set_virtual_level_count] = ACTIONS(2081),
    [sym_set_virtual_channel_count] = ACTIONS(2081),
    [sym_set_virtual_port_count] = ACTIONS(2081),
    [sym__wc_encode_int] = ACTIONS(2081),
    [sym__wc_decode_int] = ACTIONS(2081),
    [sym__wc_int] = ACTIONS(2081),
    [sym__wc_to_ch_int] = ACTIONS(2081),
    [sym__ch_to_wc_int] = ACTIONS(2081),
    [sym__wc_find_string_int] = ACTIONS(2081),
    [sym__wc_left_string_int] = ACTIONS(2081),
    [sym__wc_lower_string_int] = ACTIONS(2081),
    [sym__wc_mid_string_int] = ACTIONS(2081),
    [sym__wc_remove_string_int] = ACTIONS(2081),
    [sym__wc_right_string_int] = ACTIONS(2081),
    [sym__wc_upper_string_int] = ACTIONS(2081),
    [sym__wc_compare_string_int] = ACTIONS(2081),
    [sym__wc_get_buffer_char_int] = ACTIONS(2081),
    [sym__wc_get_buffer_string_int] = ACTIONS(2081),
    [sym__wc_concat_string_int] = ACTIONS(2081),
    [sym__wc_file_open_int] = ACTIONS(2081),
    [sym__wc_file_close_int] = ACTIONS(2081),
    [sym__wc_file_read_int] = ACTIONS(2081),
    [sym__wc_file_read_line_int] = ACTIONS(2081),
    [sym__wc_file_write_int] = ACTIONS(2081),
    [sym__wc_file_write_line_int] = ACTIONS(2081),
    [aux_sym_preproc_include_token1] = ACTIONS(2081),
    [aux_sym_preproc_define_token1] = ACTIONS(2083),
    [aux_sym_preproc_warn_token1] = ACTIONS(2083),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2083),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2083),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2083),
    [anon_sym_LBRACK] = ACTIONS(2083),
    [anon_sym_LBRACE] = ACTIONS(2083),
    [anon_sym_LPAREN] = ACTIONS(2081),
    [anon_sym_BANG] = ACTIONS(2083),
    [anon_sym_TILDE] = ACTIONS(2083),
    [anon_sym_DASH] = ACTIONS(2081),
    [anon_sym_PLUS] = ACTIONS(2081),
    [anon_sym_DASH_DASH] = ACTIONS(2083),
    [anon_sym_PLUS_PLUS] = ACTIONS(2083),
    [anon_sym_DQUOTE] = ACTIONS(2083),
    [anon_sym_SQUOTE] = ACTIONS(2083),
    [sym_number_literal] = ACTIONS(2083),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(398)] = {
    [ts_builtin_sym_end] = ACTIONS(2011),
    [sym_identifier] = ACTIONS(2009),
    [sym_program_name_keyword] = ACTIONS(2009),
    [sym_module_name_keyword] = ACTIONS(2009),
    [sym_define_device_keyword] = ACTIONS(2009),
    [sym_define_combine_keyword] = ACTIONS(2009),
    [sym_define_constant_keyword] = ACTIONS(2009),
    [sym_define_type_keyword] = ACTIONS(2009),
    [sym_define_variable_keyword] = ACTIONS(2009),
    [sym_define_system_variable_keyword] = ACTIONS(2009),
    [sym_define_connect_level_keyword] = ACTIONS(2009),
    [sym_define_latching_keyword] = ACTIONS(2009),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2009),
    [sym_define_toggling_keyword] = ACTIONS(2009),
    [sym_define_call_keyword] = ACTIONS(2009),
    [sym_define_function_keyword] = ACTIONS(2009),
    [sym_define_library_function_keyword] = ACTIONS(2009),
    [sym_define_start_keyword] = ACTIONS(2009),
    [sym_define_event_keyword] = ACTIONS(2009),
    [sym_define_program_keyword] = ACTIONS(2009),
    [sym_define_module_keyword] = ACTIONS(2009),
    [sym_char_keyword] = ACTIONS(2009),
    [sym_widechar_keyword] = ACTIONS(2009),
    [sym_integer_keyword] = ACTIONS(2009),
    [sym_sinteger_keyword] = ACTIONS(2009),
    [sym_long_keyword] = ACTIONS(2009),
    [sym_slong_keyword] = ACTIONS(2009),
    [sym_float_keyword] = ACTIONS(2009),
    [sym_double_keyword] = ACTIONS(2009),
    [sym_variant_keyword] = ACTIONS(2009),
    [sym_variantarray_keyword] = ACTIONS(2009),
    [sym_local_var_keyword] = ACTIONS(2009),
    [sym_stack_var_keyword] = ACTIONS(2009),
    [sym_constant_keyword] = ACTIONS(2009),
    [sym_volatile_keyword] = ACTIONS(2009),
    [sym_non_volatile_keyword] = ACTIONS(2009),
    [sym_persistent_keyword] = ACTIONS(2009),
    [sym_dev_keyword] = ACTIONS(2009),
    [sym_devlev_keyword] = ACTIONS(2009),
    [sym_devchan_keyword] = ACTIONS(2009),
    [sym_if_keyword] = ACTIONS(2009),
    [sym_switch_keyword] = ACTIONS(2009),
    [sym_case_keyword] = ACTIONS(2009),
    [sym_default_keyword] = ACTIONS(2009),
    [sym_while_keyword] = ACTIONS(2009),
    [sym_for_keyword] = ACTIONS(2009),
    [sym_break_keyword] = ACTIONS(2009),
    [sym_continue_keyword] = ACTIONS(2009),
    [sym_return_keyword] = ACTIONS(2009),
    [sym_select_keyword] = ACTIONS(2009),
    [sym_active_keyword] = ACTIONS(2009),
    [sym_struct_keyword] = ACTIONS(2009),
    [sym_structure_keyword] = ACTIONS(2009),
    [sym_button_event_keyword] = ACTIONS(2009),
    [sym_channel_event_keyword] = ACTIONS(2009),
    [sym_level_event_keyword] = ACTIONS(2009),
    [sym_data_event_keyword] = ACTIONS(2009),
    [sym_timeline_event_keyword] = ACTIONS(2009),
    [sym_custom_event_keyword] = ACTIONS(2009),
    [sym_push_keyword] = ACTIONS(2009),
    [sym_release_keyword] = ACTIONS(2009),
    [aux_sym_on_keyword_token1] = ACTIONS(2009),
    [aux_sym_off_keyword_token1] = ACTIONS(2009),
    [sym_send_level_keyword] = ACTIONS(2009),
    [sym_send_string_keyword] = ACTIONS(2009),
    [sym_send_command_keyword] = ACTIONS(2009),
    [sym_clear_buffer_keyword] = ACTIONS(2009),
    [sym_create_buffer_keyword] = ACTIONS(2009),
    [sym_create_multi_buffer_keyword] = ACTIONS(2009),
    [sym_call_keyword] = ACTIONS(2009),
    [sym_system_call_keyword] = ACTIONS(2009),
    [sym_devchan_to_keyword] = ACTIONS(2009),
    [sym_devchan_min_to_keyword] = ACTIONS(2009),
    [sym_devchan_total_off_keyword] = ACTIONS(2009),
    [sym_devchan_pulse_keyword] = ACTIONS(2009),
    [sym_wait_keyword] = ACTIONS(2009),
    [sym_cancel_wait_keyword] = ACTIONS(2009),
    [sym_wait_until_keyword] = ACTIONS(2009),
    [sym_cancel_wait_until_keyword] = ACTIONS(2009),
    [sym_cancel_all_wait_keyword] = ACTIONS(2009),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2009),
    [sym_not] = ACTIONS(2009),
    [sym_bnot] = ACTIONS(2009),
    [sym___file__] = ACTIONS(2009),
    [sym___date__] = ACTIONS(2009),
    [sym___ldate__] = ACTIONS(2009),
    [sym___time__] = ACTIONS(2009),
    [sym___line__] = ACTIONS(2009),
    [sym___name__] = ACTIONS(2009),
    [sym___netlinx__] = ACTIONS(2009),
    [sym_netlinx_axi_version] = ACTIONS(2009),
    [sym_url_flg_tcp] = ACTIONS(2009),
    [sym_url_flg_accinfopresent] = ACTIONS(2009),
    [sym_url_flg_temp] = ACTIONS(2009),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2009),
    [sym_url_flg_stat_mask] = ACTIONS(2009),
    [sym_url_flg_stat_lookup] = ACTIONS(2009),
    [sym_url_flg_stat_connecting] = ACTIONS(2009),
    [sym_url_flg_stat_waiting] = ACTIONS(2009),
    [sym_url_flg_stat_connected] = ACTIONS(2009),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2009),
    [sym_first_virtual_device] = ACTIONS(2009),
    [sym_first_local_port] = ACTIONS(2009),
    [sym_dynamic_virtual_device] = ACTIONS(2009),
    [sym_timeline_once] = ACTIONS(2009),
    [sym_timeline_repeat] = ACTIONS(2009),
    [sym_timeline_absolute] = ACTIONS(2009),
    [sym_timeline_relative] = ACTIONS(2009),
    [sym_do_push_timed_infinite] = ACTIONS(2009),
    [sym_source_type_no_address] = ACTIONS(2009),
    [sym_source_type_neuron_id] = ACTIONS(2009),
    [sym_source_type_ip_address] = ACTIONS(2009),
    [sym_source_type_axlink] = ACTIONS(2009),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2009),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2009),
    [sym_source_type_ip_socket_address] = ACTIONS(2009),
    [sym_source_type_rs232] = ACTIONS(2009),
    [sym_source_type_internal] = ACTIONS(2009),
    [sym_source_type_mac_address] = ACTIONS(2009),
    [sym_source_type_ipv4_port] = ACTIONS(2009),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2009),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2009),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2009),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2009),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2009),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2009),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2009),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2009),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2009),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2009),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2009),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2009),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2009),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2009),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2009),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2009),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2009),
    [sym_true] = ACTIONS(2009),
    [sym_false] = ACTIONS(2009),
    [sym_file_read_only] = ACTIONS(2009),
    [sym_file_rw_new] = ACTIONS(2009),
    [sym_file_rw_append] = ACTIONS(2009),
    [sym_ip_tcp] = ACTIONS(2009),
    [sym_ip_udp] = ACTIONS(2009),
    [sym_ip_udp_2way] = ACTIONS(2009),
    [sym_xml_encode_types] = ACTIONS(2009),
    [sym_xml_encode_char_as_list] = ACTIONS(2009),
    [sym_xml_encode_le] = ACTIONS(2009),
    [sym_xml_decode_types] = ACTIONS(2009),
    [sym_xml_decode_no_preserve] = ACTIONS(2009),
    [sym_duet_dev_type_display_device] = ACTIONS(2009),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2009),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2009),
    [sym_duet_dev_type_io_device] = ACTIONS(2009),
    [sym_duet_dev_type_relay_device] = ACTIONS(2009),
    [sym_duet_dev_type_ups] = ACTIONS(2009),
    [sym_duet_dev_type_amplifier] = ACTIONS(2009),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2009),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2009),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2009),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2009),
    [sym_duet_dev_type_camera] = ACTIONS(2009),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2009),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2009),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2009),
    [sym_duet_dev_type_dss] = ACTIONS(2009),
    [sym_duet_dev_type_dvr] = ACTIONS(2009),
    [sym_duet_dev_type_disc_device] = ACTIONS(2009),
    [sym_duet_dev_type_document_camera] = ACTIONS(2009),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2009),
    [sym_duet_dev_type_hvac] = ACTIONS(2009),
    [sym_duet_dev_type_keypad] = ACTIONS(2009),
    [sym_duet_dev_type_light] = ACTIONS(2009),
    [sym_duet_dev_type_monitor] = ACTIONS(2009),
    [sym_duet_dev_type_motor] = ACTIONS(2009),
    [sym_duet_dev_type_multi_window] = ACTIONS(2009),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2009),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2009),
    [sym_duet_dev_type_receiver] = ACTIONS(2009),
    [sym_duet_dev_type_security_system] = ACTIONS(2009),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2009),
    [sym_duet_dev_type_settop_box] = ACTIONS(2009),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2009),
    [sym_duet_dev_type_switcher] = ACTIONS(2009),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2009),
    [sym_duet_dev_type_tv] = ACTIONS(2009),
    [sym_duet_dev_type_utility] = ACTIONS(2009),
    [sym_duet_dev_type_vcr] = ACTIONS(2009),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2009),
    [sym_duet_dev_type_video_processor] = ACTIONS(2009),
    [sym_duet_dev_type_video_projector] = ACTIONS(2009),
    [sym_duet_dev_type_video_wall] = ACTIONS(2009),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2009),
    [sym_duet_dev_type_weather] = ACTIONS(2009),
    [sym_duet_dev_not_polled] = ACTIONS(2009),
    [sym_duet_dev_polled] = ACTIONS(2009),
    [sym_clkmgr_mode_network] = ACTIONS(2009),
    [sym_clkmgr_mode_standalone] = ACTIONS(2009),
    [sym_normal_standby] = ACTIONS(2009),
    [sym_normal_wake] = ACTIONS(2009),
    [sym_ip_multicast_ttl_option] = ACTIONS(2009),
    [sym_ip_ttl_subnet] = ACTIONS(2009),
    [sym_ip_ttl_site] = ACTIONS(2009),
    [sym_ip_ttl_region] = ACTIONS(2009),
    [sym_ip_ttl_continent] = ACTIONS(2009),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2009),
    [sym_ip_nodelay_on] = ACTIONS(2009),
    [sym_ip_nodelay_off] = ACTIONS(2009),
    [sym_amx_error] = ACTIONS(2009),
    [sym_amx_warning] = ACTIONS(2009),
    [sym_amx_info] = ACTIONS(2009),
    [sym_amx_debug] = ACTIONS(2009),
    [sym_netlinx_login_success] = ACTIONS(2009),
    [sym_netlinx_login_fail] = ACTIONS(2009),
    [sym_netlinx_logout] = ACTIONS(2009),
    [sym_null_str] = ACTIONS(2009),
    [sym_smtp_address] = ACTIONS(2009),
    [sym_smtp_port_number] = ACTIONS(2009),
    [sym_smtp_username] = ACTIONS(2009),
    [sym_smtp_password] = ACTIONS(2009),
    [sym_smtp_require_tls] = ACTIONS(2009),
    [sym_smtp_from] = ACTIONS(2009),
    [sym_smtp_tls_true] = ACTIONS(2009),
    [sym_smtp_tls_false] = ACTIONS(2009),
    [sym_listview_on_row_select_event] = ACTIONS(2009),
    [sym_data_structure_unknown] = ACTIONS(2009),
    [sym_data_structure_datafeed] = ACTIONS(2009),
    [sym_data_structure_datarecordset] = ACTIONS(2009),
    [sym_data_structure_datarecord] = ACTIONS(2009),
    [sym_data_structure_datafield] = ACTIONS(2009),
    [sym_data_max_content_field_count] = ACTIONS(2009),
    [sym_data_max_metadata_field_count] = ACTIONS(2009),
    [sym_data_max_id_length] = ACTIONS(2009),
    [sym_data_max_name_length] = ACTIONS(2009),
    [sym_data_max_description_length] = ACTIONS(2009),
    [sym_data_max_type_length] = ACTIONS(2009),
    [sym_data_max_format_length] = ACTIONS(2009),
    [sym_data_max_source_length] = ACTIONS(2009),
    [sym_data_max_value_length] = ACTIONS(2009),
    [sym_data_max_label_length] = ACTIONS(2009),
    [sym_data_max_url_length] = ACTIONS(2009),
    [sym_max_last_login_info_length] = ACTIONS(2009),
    [sym_data_type_unknown] = ACTIONS(2009),
    [sym_data_type_string] = ACTIONS(2009),
    [sym_data_type_datetime] = ACTIONS(2009),
    [sym_data_type_date] = ACTIONS(2009),
    [sym_data_type_time] = ACTIONS(2009),
    [sym_data_type_image] = ACTIONS(2009),
    [sym_data_format_url] = ACTIONS(2009),
    [sym_data_format_phone] = ACTIONS(2009),
    [sym_data_format_email] = ACTIONS(2009),
    [sym_data_format_iso8601] = ACTIONS(2009),
    [sym_auditlog_priv] = ACTIONS(2009),
    [sym_configuration_priv] = ACTIONS(2009),
    [sym_deviceconfig_priv] = ACTIONS(2009),
    [sym_ftp_priv] = ACTIONS(2009),
    [sym_http_priv] = ACTIONS(2009),
    [sym_networkconfig_priv] = ACTIONS(2009),
    [sym_programport_priv] = ACTIONS(2009),
    [sym_remoteui_priv] = ACTIONS(2009),
    [sym_securitycontrol_priv] = ACTIONS(2009),
    [sym_softwaremanagement_priv] = ACTIONS(2009),
    [sym_terminal_priv] = ACTIONS(2009),
    [sym_tpadmin_priv] = ACTIONS(2009),
    [sym_user1_priv] = ACTIONS(2009),
    [sym_user2_priv] = ACTIONS(2009),
    [sym_user3_priv] = ACTIONS(2009),
    [sym_user4_priv] = ACTIONS(2009),
    [sym_usermanagement_priv] = ACTIONS(2009),
    [sym_valid_account] = ACTIONS(2009),
    [sym_err_name_invalid] = ACTIONS(2009),
    [sym_err_password_invalid] = ACTIONS(2009),
    [sym_err_invalid_account] = ACTIONS(2009),
    [sym_err_no_authorization] = ACTIONS(2009),
    [sym_err_invalid_parameter] = ACTIONS(2009),
    [sym_err_user_locked_out] = ACTIONS(2009),
    [sym_err_user_account_expired] = ACTIONS(2009),
    [sym_err_general_error] = ACTIONS(2009),
    [sym_err_authentication_error] = ACTIONS(2009),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2009),
    [sym_tls_validate_certificate] = ACTIONS(2009),
    [sym_tdata] = ACTIONS(2009),
    [sym_tchannel] = ACTIONS(2009),
    [sym_tlevel] = ACTIONS(2009),
    [sym_tbutton] = ACTIONS(2009),
    [sym_ttimeline] = ACTIONS(2009),
    [sym_tcustom] = ACTIONS(2009),
    [sym_url_struct] = ACTIONS(2009),
    [sym_dns_struct] = ACTIONS(2009),
    [sym_ip_address_struct] = ACTIONS(2009),
    [sym_dev_info_struct] = ACTIONS(2009),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2009),
    [sym_data_feed] = ACTIONS(2009),
    [sym_data_field] = ACTIONS(2009),
    [sym_data_record] = ACTIONS(2009),
    [sym_wc_data_feed] = ACTIONS(2009),
    [sym_wc_data_field] = ACTIONS(2009),
    [sym_wc_data_record] = ACTIONS(2009),
    [sym_last_login_info] = ACTIONS(2009),
    [sym____reserved___] = ACTIONS(2009),
    [sym_date] = ACTIONS(2009),
    [sym_day] = ACTIONS(2009),
    [sym_time] = ACTIONS(2009),
    [sym_ldate] = ACTIONS(2009),
    [sym_push_device] = ACTIONS(2009),
    [sym_push_channel] = ACTIONS(2009),
    [sym_push_devchan] = ACTIONS(2009),
    [sym_release_device] = ACTIONS(2009),
    [sym_release_channel] = ACTIONS(2009),
    [sym_release_devchan] = ACTIONS(2009),
    [sym_master_slot] = ACTIONS(2009),
    [sym_get_pulse_time] = ACTIONS(2009),
    [sym_get_timer] = ACTIONS(2009),
    [sym_master_sn] = ACTIONS(2009),
    [sym_system_number] = ACTIONS(2009),
    [sym_button] = ACTIONS(2009),
    [sym_dv_channel] = ACTIONS(2009),
    [sym_data] = ACTIONS(2009),
    [sym_level] = ACTIONS(2009),
    [sym_timeline] = ACTIONS(2009),
    [sym_channel] = ACTIONS(2009),
    [sym_custom] = ACTIONS(2009),
    [sym_atoi] = ACTIONS(2009),
    [sym_atol] = ACTIONS(2009),
    [sym_atof] = ACTIONS(2009),
    [sym_itoa] = ACTIONS(2009),
    [sym_itohex] = ACTIONS(2009),
    [sym_hextoi] = ACTIONS(2009),
    [sym_ftoa] = ACTIONS(2009),
    [sym_find_string] = ACTIONS(2009),
    [sym_left_string] = ACTIONS(2009),
    [sym_right_string] = ACTIONS(2009),
    [sym_mid_string] = ACTIONS(2009),
    [sym_remove_string] = ACTIONS(2009),
    [sym_get_buffer_string] = ACTIONS(2009),
    [sym_get_buffer_char] = ACTIONS(2009),
    [sym_get_multi_buffer_string] = ACTIONS(2009),
    [sym_length_string] = ACTIONS(2009),
    [sym_max_length_string] = ACTIONS(2009),
    [sym_set_length_string] = ACTIONS(2009),
    [sym_compare_string] = ACTIONS(2009),
    [sym_upper_string] = ACTIONS(2009),
    [sym_lower_string] = ACTIONS(2009),
    [sym_get_last] = ACTIONS(2009),
    [sym_date_to_day] = ACTIONS(2009),
    [sym_date_to_month] = ACTIONS(2009),
    [sym_date_to_year] = ACTIONS(2009),
    [sym_time_to_hour] = ACTIONS(2009),
    [sym_time_to_minute] = ACTIONS(2009),
    [sym_time_to_second] = ACTIONS(2009),
    [sym_day_of_week] = ACTIONS(2009),
    [sym_set_pulse_time] = ACTIONS(2009),
    [sym_set_timer] = ACTIONS(2009),
    [sym_file_open] = ACTIONS(2009),
    [sym_file_close] = ACTIONS(2009),
    [sym_file_read] = ACTIONS(2009),
    [sym_file_read_line] = ACTIONS(2009),
    [sym_file_write] = ACTIONS(2009),
    [sym_file_write_line] = ACTIONS(2009),
    [sym_file_copy] = ACTIONS(2009),
    [sym_file_delete] = ACTIONS(2009),
    [sym_file_rename] = ACTIONS(2009),
    [sym_file_dir] = ACTIONS(2009),
    [sym_file_getdir] = ACTIONS(2009),
    [sym_file_setdir] = ACTIONS(2009),
    [sym_file_create_dir] = ACTIONS(2009),
    [sym_file_remove_dir] = ACTIONS(2009),
    [sym_file_seek] = ACTIONS(2009),
    [sym_length_array] = ACTIONS(2009),
    [sym_max_length_array] = ACTIONS(2009),
    [sym_set_length_array] = ACTIONS(2009),
    [sym_ip_client_open] = ACTIONS(2009),
    [sym_ip_client_close] = ACTIONS(2009),
    [sym_ip_server_open] = ACTIONS(2009),
    [sym_ip_server_close] = ACTIONS(2009),
    [sym_ip_bound_client_open] = ACTIONS(2009),
    [sym_ip_mc_server_open] = ACTIONS(2009),
    [sym_ip_set_option] = ACTIONS(2009),
    [sym_ssh_client_open] = ACTIONS(2009),
    [sym_ssh_client_close] = ACTIONS(2009),
    [sym_tls_client_open] = ACTIONS(2009),
    [sym_tls_client_close] = ACTIONS(2009),
    [sym_do_push] = ACTIONS(2009),
    [sym_do_push_timed] = ACTIONS(2009),
    [sym_do_release] = ACTIONS(2009),
    [sym_device_id] = ACTIONS(2009),
    [sym_device_info] = ACTIONS(2009),
    [sym_device_id_string] = ACTIONS(2009),
    [sym_device_standby] = ACTIONS(2009),
    [sym_device_wake] = ACTIONS(2009),
    [sym_timeline_create] = ACTIONS(2009),
    [sym_timeline_kill] = ACTIONS(2009),
    [sym_timeline_reload] = ACTIONS(2009),
    [sym_timeline_pause] = ACTIONS(2009),
    [sym_timeline_restart] = ACTIONS(2009),
    [sym_timeline_set] = ACTIONS(2009),
    [sym_timeline_get] = ACTIONS(2009),
    [sym_timeline_active] = ACTIONS(2009),
    [sym_timeline_dynamic_id] = ACTIONS(2009),
    [sym_variable_to_string] = ACTIONS(2009),
    [sym_string_to_variable] = ACTIONS(2009),
    [sym_length_variable_to_string] = ACTIONS(2009),
    [sym_type_cast] = ACTIONS(2009),
    [sym_variable_to_xml] = ACTIONS(2009),
    [sym_xml_to_variable] = ACTIONS(2009),
    [sym_length_variable_to_xml] = ACTIONS(2009),
    [sym_redirect_string] = ACTIONS(2009),
    [sym_random_number] = ACTIONS(2009),
    [sym_get_unique_id] = ACTIONS(2009),
    [sym_get_system_number] = ACTIONS(2009),
    [sym_set_system_number] = ACTIONS(2009),
    [sym_get_serial_number] = ACTIONS(2009),
    [sym_get_master_build] = ACTIONS(2009),
    [sym_set_validation_code] = ACTIONS(2009),
    [sym_reboot] = ACTIONS(2009),
    [sym_load_duet_module] = ACTIONS(2009),
    [sym_unload_duet_module] = ACTIONS(2009),
    [sym_duet_mem_size_set] = ACTIONS(2009),
    [sym_duet_mem_size_get] = ACTIONS(2009),
    [sym_dynamic_application_device] = ACTIONS(2009),
    [sym_dynamic_polled_port] = ACTIONS(2009),
    [sym_static_port_binding] = ACTIONS(2009),
    [sym_static_ip_binding] = ACTIONS(2009),
    [sym_get_dns_list] = ACTIONS(2009),
    [sym_set_dns_list] = ACTIONS(2009),
    [sym_get_ip_address] = ACTIONS(2009),
    [sym_set_ip_address] = ACTIONS(2009),
    [sym_get_url_list] = ACTIONS(2009),
    [sym_add_url_entry] = ACTIONS(2009),
    [sym_delete_url_entry] = ACTIONS(2009),
    [sym_abs_value] = ACTIONS(2009),
    [sym_max_value] = ACTIONS(2009),
    [sym_min_value] = ACTIONS(2009),
    [sym_exp_value] = ACTIONS(2009),
    [sym_log_value] = ACTIONS(2009),
    [sym_log10_value] = ACTIONS(2009),
    [sym_power_value] = ACTIONS(2009),
    [sym_sqrt_value] = ACTIONS(2009),
    [sym_raw_be] = ACTIONS(2009),
    [sym_raw_le] = ACTIONS(2009),
    [sym_internal_queue_size_set] = ACTIONS(2009),
    [sym_internal_queue_size_get] = ACTIONS(2009),
    [sym_internal_threshold_set] = ACTIONS(2009),
    [sym_internal_threshold_get] = ACTIONS(2009),
    [sym_rebuilt_event] = ACTIONS(2009),
    [sym_do_custom_event] = ACTIONS(2009),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2009),
    [sym_clkmgr_set_clk_source] = ACTIONS(2009),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2009),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2009),
    [sym_clkmgr_get_timezone] = ACTIONS(2009),
    [sym_clkmgr_set_timezone] = ACTIONS(2009),
    [sym_clkmgr_get_resync_period] = ACTIONS(2009),
    [sym_clkmgr_set_resync_period] = ACTIONS(2009),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2009),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2009),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2009),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2009),
    [sym_clkmgr_get_timeservers] = ACTIONS(2009),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2009),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2009),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2009),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2009),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2009),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2009),
    [sym_set_log_level] = ACTIONS(2009),
    [sym_get_log_level] = ACTIONS(2009),
    [sym_amx_log] = ACTIONS(2009),
    [sym_smtp_server_config_set] = ACTIONS(2009),
    [sym_smtp_server_config_get] = ACTIONS(2009),
    [sym_smtp_send] = ACTIONS(2009),
    [sym_validate_netlinx_account] = ACTIONS(2009),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2009),
    [sym_authenticate_certificate] = ACTIONS(2009),
    [sym_audit_netlinx_session_event] = ACTIONS(2009),
    [sym_audit_netlinx_generic_event] = ACTIONS(2009),
    [sym_get_available_flash_disk_space] = ACTIONS(2009),
    [sym_get_max_flash_disk_space] = ACTIONS(2009),
    [sym_data_create_feed] = ACTIONS(2009),
    [sym_data_delete_feed] = ACTIONS(2009),
    [sym_data_publish_feed] = ACTIONS(2009),
    [sym_data_get_published_feed] = ACTIONS(2009),
    [sym_data_add_record] = ACTIONS(2009),
    [sym_data_get_event_record] = ACTIONS(2009),
    [sym__wc_data_create_feed] = ACTIONS(2009),
    [sym__wc_data_add_record] = ACTIONS(2009),
    [sym__wc_data_get_event_record] = ACTIONS(2009),
    [sym_format] = ACTIONS(2009),
    [sym_astro_clock] = ACTIONS(2009),
    [sym_set_outdoor_temperature] = ACTIONS(2009),
    [sym_set_virtual_level_count] = ACTIONS(2009),
    [sym_set_virtual_channel_count] = ACTIONS(2009),
    [sym_set_virtual_port_count] = ACTIONS(2009),
    [sym__wc_encode_int] = ACTIONS(2009),
    [sym__wc_decode_int] = ACTIONS(2009),
    [sym__wc_int] = ACTIONS(2009),
    [sym__wc_to_ch_int] = ACTIONS(2009),
    [sym__ch_to_wc_int] = ACTIONS(2009),
    [sym__wc_find_string_int] = ACTIONS(2009),
    [sym__wc_left_string_int] = ACTIONS(2009),
    [sym__wc_lower_string_int] = ACTIONS(2009),
    [sym__wc_mid_string_int] = ACTIONS(2009),
    [sym__wc_remove_string_int] = ACTIONS(2009),
    [sym__wc_right_string_int] = ACTIONS(2009),
    [sym__wc_upper_string_int] = ACTIONS(2009),
    [sym__wc_compare_string_int] = ACTIONS(2009),
    [sym__wc_get_buffer_char_int] = ACTIONS(2009),
    [sym__wc_get_buffer_string_int] = ACTIONS(2009),
    [sym__wc_concat_string_int] = ACTIONS(2009),
    [sym__wc_file_open_int] = ACTIONS(2009),
    [sym__wc_file_close_int] = ACTIONS(2009),
    [sym__wc_file_read_int] = ACTIONS(2009),
    [sym__wc_file_read_line_int] = ACTIONS(2009),
    [sym__wc_file_write_int] = ACTIONS(2009),
    [sym__wc_file_write_line_int] = ACTIONS(2009),
    [aux_sym_preproc_include_token1] = ACTIONS(2009),
    [aux_sym_preproc_define_token1] = ACTIONS(2011),
    [aux_sym_preproc_warn_token1] = ACTIONS(2011),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2011),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2011),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2011),
    [anon_sym_LBRACK] = ACTIONS(2011),
    [anon_sym_LBRACE] = ACTIONS(2011),
    [anon_sym_LPAREN] = ACTIONS(2009),
    [anon_sym_BANG] = ACTIONS(2011),
    [anon_sym_TILDE] = ACTIONS(2011),
    [anon_sym_DASH] = ACTIONS(2009),
    [anon_sym_PLUS] = ACTIONS(2009),
    [anon_sym_DASH_DASH] = ACTIONS(2011),
    [anon_sym_PLUS_PLUS] = ACTIONS(2011),
    [anon_sym_DQUOTE] = ACTIONS(2011),
    [anon_sym_SQUOTE] = ACTIONS(2011),
    [sym_number_literal] = ACTIONS(2011),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(399)] = {
    [ts_builtin_sym_end] = ACTIONS(2019),
    [sym_identifier] = ACTIONS(2017),
    [sym_program_name_keyword] = ACTIONS(2017),
    [sym_module_name_keyword] = ACTIONS(2017),
    [sym_define_device_keyword] = ACTIONS(2017),
    [sym_define_combine_keyword] = ACTIONS(2017),
    [sym_define_constant_keyword] = ACTIONS(2017),
    [sym_define_type_keyword] = ACTIONS(2017),
    [sym_define_variable_keyword] = ACTIONS(2017),
    [sym_define_system_variable_keyword] = ACTIONS(2017),
    [sym_define_connect_level_keyword] = ACTIONS(2017),
    [sym_define_latching_keyword] = ACTIONS(2017),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2017),
    [sym_define_toggling_keyword] = ACTIONS(2017),
    [sym_define_call_keyword] = ACTIONS(2017),
    [sym_define_function_keyword] = ACTIONS(2017),
    [sym_define_library_function_keyword] = ACTIONS(2017),
    [sym_define_start_keyword] = ACTIONS(2017),
    [sym_define_event_keyword] = ACTIONS(2017),
    [sym_define_program_keyword] = ACTIONS(2017),
    [sym_define_module_keyword] = ACTIONS(2017),
    [sym_char_keyword] = ACTIONS(2017),
    [sym_widechar_keyword] = ACTIONS(2017),
    [sym_integer_keyword] = ACTIONS(2017),
    [sym_sinteger_keyword] = ACTIONS(2017),
    [sym_long_keyword] = ACTIONS(2017),
    [sym_slong_keyword] = ACTIONS(2017),
    [sym_float_keyword] = ACTIONS(2017),
    [sym_double_keyword] = ACTIONS(2017),
    [sym_variant_keyword] = ACTIONS(2017),
    [sym_variantarray_keyword] = ACTIONS(2017),
    [sym_local_var_keyword] = ACTIONS(2017),
    [sym_stack_var_keyword] = ACTIONS(2017),
    [sym_constant_keyword] = ACTIONS(2017),
    [sym_volatile_keyword] = ACTIONS(2017),
    [sym_non_volatile_keyword] = ACTIONS(2017),
    [sym_persistent_keyword] = ACTIONS(2017),
    [sym_dev_keyword] = ACTIONS(2017),
    [sym_devlev_keyword] = ACTIONS(2017),
    [sym_devchan_keyword] = ACTIONS(2017),
    [sym_if_keyword] = ACTIONS(2017),
    [sym_switch_keyword] = ACTIONS(2017),
    [sym_case_keyword] = ACTIONS(2017),
    [sym_default_keyword] = ACTIONS(2017),
    [sym_while_keyword] = ACTIONS(2017),
    [sym_for_keyword] = ACTIONS(2017),
    [sym_break_keyword] = ACTIONS(2017),
    [sym_continue_keyword] = ACTIONS(2017),
    [sym_return_keyword] = ACTIONS(2017),
    [sym_select_keyword] = ACTIONS(2017),
    [sym_active_keyword] = ACTIONS(2017),
    [sym_struct_keyword] = ACTIONS(2017),
    [sym_structure_keyword] = ACTIONS(2017),
    [sym_button_event_keyword] = ACTIONS(2017),
    [sym_channel_event_keyword] = ACTIONS(2017),
    [sym_level_event_keyword] = ACTIONS(2017),
    [sym_data_event_keyword] = ACTIONS(2017),
    [sym_timeline_event_keyword] = ACTIONS(2017),
    [sym_custom_event_keyword] = ACTIONS(2017),
    [sym_push_keyword] = ACTIONS(2017),
    [sym_release_keyword] = ACTIONS(2017),
    [aux_sym_on_keyword_token1] = ACTIONS(2017),
    [aux_sym_off_keyword_token1] = ACTIONS(2017),
    [sym_send_level_keyword] = ACTIONS(2017),
    [sym_send_string_keyword] = ACTIONS(2017),
    [sym_send_command_keyword] = ACTIONS(2017),
    [sym_clear_buffer_keyword] = ACTIONS(2017),
    [sym_create_buffer_keyword] = ACTIONS(2017),
    [sym_create_multi_buffer_keyword] = ACTIONS(2017),
    [sym_call_keyword] = ACTIONS(2017),
    [sym_system_call_keyword] = ACTIONS(2017),
    [sym_devchan_to_keyword] = ACTIONS(2017),
    [sym_devchan_min_to_keyword] = ACTIONS(2017),
    [sym_devchan_total_off_keyword] = ACTIONS(2017),
    [sym_devchan_pulse_keyword] = ACTIONS(2017),
    [sym_wait_keyword] = ACTIONS(2017),
    [sym_cancel_wait_keyword] = ACTIONS(2017),
    [sym_wait_until_keyword] = ACTIONS(2017),
    [sym_cancel_wait_until_keyword] = ACTIONS(2017),
    [sym_cancel_all_wait_keyword] = ACTIONS(2017),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2017),
    [sym_not] = ACTIONS(2017),
    [sym_bnot] = ACTIONS(2017),
    [sym___file__] = ACTIONS(2017),
    [sym___date__] = ACTIONS(2017),
    [sym___ldate__] = ACTIONS(2017),
    [sym___time__] = ACTIONS(2017),
    [sym___line__] = ACTIONS(2017),
    [sym___name__] = ACTIONS(2017),
    [sym___netlinx__] = ACTIONS(2017),
    [sym_netlinx_axi_version] = ACTIONS(2017),
    [sym_url_flg_tcp] = ACTIONS(2017),
    [sym_url_flg_accinfopresent] = ACTIONS(2017),
    [sym_url_flg_temp] = ACTIONS(2017),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2017),
    [sym_url_flg_stat_mask] = ACTIONS(2017),
    [sym_url_flg_stat_lookup] = ACTIONS(2017),
    [sym_url_flg_stat_connecting] = ACTIONS(2017),
    [sym_url_flg_stat_waiting] = ACTIONS(2017),
    [sym_url_flg_stat_connected] = ACTIONS(2017),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2017),
    [sym_first_virtual_device] = ACTIONS(2017),
    [sym_first_local_port] = ACTIONS(2017),
    [sym_dynamic_virtual_device] = ACTIONS(2017),
    [sym_timeline_once] = ACTIONS(2017),
    [sym_timeline_repeat] = ACTIONS(2017),
    [sym_timeline_absolute] = ACTIONS(2017),
    [sym_timeline_relative] = ACTIONS(2017),
    [sym_do_push_timed_infinite] = ACTIONS(2017),
    [sym_source_type_no_address] = ACTIONS(2017),
    [sym_source_type_neuron_id] = ACTIONS(2017),
    [sym_source_type_ip_address] = ACTIONS(2017),
    [sym_source_type_axlink] = ACTIONS(2017),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2017),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2017),
    [sym_source_type_ip_socket_address] = ACTIONS(2017),
    [sym_source_type_rs232] = ACTIONS(2017),
    [sym_source_type_internal] = ACTIONS(2017),
    [sym_source_type_mac_address] = ACTIONS(2017),
    [sym_source_type_ipv4_port] = ACTIONS(2017),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2017),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2017),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2017),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2017),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2017),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2017),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2017),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2017),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2017),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2017),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2017),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2017),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2017),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2017),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2017),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2017),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2017),
    [sym_true] = ACTIONS(2017),
    [sym_false] = ACTIONS(2017),
    [sym_file_read_only] = ACTIONS(2017),
    [sym_file_rw_new] = ACTIONS(2017),
    [sym_file_rw_append] = ACTIONS(2017),
    [sym_ip_tcp] = ACTIONS(2017),
    [sym_ip_udp] = ACTIONS(2017),
    [sym_ip_udp_2way] = ACTIONS(2017),
    [sym_xml_encode_types] = ACTIONS(2017),
    [sym_xml_encode_char_as_list] = ACTIONS(2017),
    [sym_xml_encode_le] = ACTIONS(2017),
    [sym_xml_decode_types] = ACTIONS(2017),
    [sym_xml_decode_no_preserve] = ACTIONS(2017),
    [sym_duet_dev_type_display_device] = ACTIONS(2017),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2017),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2017),
    [sym_duet_dev_type_io_device] = ACTIONS(2017),
    [sym_duet_dev_type_relay_device] = ACTIONS(2017),
    [sym_duet_dev_type_ups] = ACTIONS(2017),
    [sym_duet_dev_type_amplifier] = ACTIONS(2017),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2017),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2017),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2017),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2017),
    [sym_duet_dev_type_camera] = ACTIONS(2017),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2017),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2017),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2017),
    [sym_duet_dev_type_dss] = ACTIONS(2017),
    [sym_duet_dev_type_dvr] = ACTIONS(2017),
    [sym_duet_dev_type_disc_device] = ACTIONS(2017),
    [sym_duet_dev_type_document_camera] = ACTIONS(2017),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2017),
    [sym_duet_dev_type_hvac] = ACTIONS(2017),
    [sym_duet_dev_type_keypad] = ACTIONS(2017),
    [sym_duet_dev_type_light] = ACTIONS(2017),
    [sym_duet_dev_type_monitor] = ACTIONS(2017),
    [sym_duet_dev_type_motor] = ACTIONS(2017),
    [sym_duet_dev_type_multi_window] = ACTIONS(2017),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2017),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2017),
    [sym_duet_dev_type_receiver] = ACTIONS(2017),
    [sym_duet_dev_type_security_system] = ACTIONS(2017),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2017),
    [sym_duet_dev_type_settop_box] = ACTIONS(2017),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2017),
    [sym_duet_dev_type_switcher] = ACTIONS(2017),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2017),
    [sym_duet_dev_type_tv] = ACTIONS(2017),
    [sym_duet_dev_type_utility] = ACTIONS(2017),
    [sym_duet_dev_type_vcr] = ACTIONS(2017),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2017),
    [sym_duet_dev_type_video_processor] = ACTIONS(2017),
    [sym_duet_dev_type_video_projector] = ACTIONS(2017),
    [sym_duet_dev_type_video_wall] = ACTIONS(2017),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2017),
    [sym_duet_dev_type_weather] = ACTIONS(2017),
    [sym_duet_dev_not_polled] = ACTIONS(2017),
    [sym_duet_dev_polled] = ACTIONS(2017),
    [sym_clkmgr_mode_network] = ACTIONS(2017),
    [sym_clkmgr_mode_standalone] = ACTIONS(2017),
    [sym_normal_standby] = ACTIONS(2017),
    [sym_normal_wake] = ACTIONS(2017),
    [sym_ip_multicast_ttl_option] = ACTIONS(2017),
    [sym_ip_ttl_subnet] = ACTIONS(2017),
    [sym_ip_ttl_site] = ACTIONS(2017),
    [sym_ip_ttl_region] = ACTIONS(2017),
    [sym_ip_ttl_continent] = ACTIONS(2017),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2017),
    [sym_ip_nodelay_on] = ACTIONS(2017),
    [sym_ip_nodelay_off] = ACTIONS(2017),
    [sym_amx_error] = ACTIONS(2017),
    [sym_amx_warning] = ACTIONS(2017),
    [sym_amx_info] = ACTIONS(2017),
    [sym_amx_debug] = ACTIONS(2017),
    [sym_netlinx_login_success] = ACTIONS(2017),
    [sym_netlinx_login_fail] = ACTIONS(2017),
    [sym_netlinx_logout] = ACTIONS(2017),
    [sym_null_str] = ACTIONS(2017),
    [sym_smtp_address] = ACTIONS(2017),
    [sym_smtp_port_number] = ACTIONS(2017),
    [sym_smtp_username] = ACTIONS(2017),
    [sym_smtp_password] = ACTIONS(2017),
    [sym_smtp_require_tls] = ACTIONS(2017),
    [sym_smtp_from] = ACTIONS(2017),
    [sym_smtp_tls_true] = ACTIONS(2017),
    [sym_smtp_tls_false] = ACTIONS(2017),
    [sym_listview_on_row_select_event] = ACTIONS(2017),
    [sym_data_structure_unknown] = ACTIONS(2017),
    [sym_data_structure_datafeed] = ACTIONS(2017),
    [sym_data_structure_datarecordset] = ACTIONS(2017),
    [sym_data_structure_datarecord] = ACTIONS(2017),
    [sym_data_structure_datafield] = ACTIONS(2017),
    [sym_data_max_content_field_count] = ACTIONS(2017),
    [sym_data_max_metadata_field_count] = ACTIONS(2017),
    [sym_data_max_id_length] = ACTIONS(2017),
    [sym_data_max_name_length] = ACTIONS(2017),
    [sym_data_max_description_length] = ACTIONS(2017),
    [sym_data_max_type_length] = ACTIONS(2017),
    [sym_data_max_format_length] = ACTIONS(2017),
    [sym_data_max_source_length] = ACTIONS(2017),
    [sym_data_max_value_length] = ACTIONS(2017),
    [sym_data_max_label_length] = ACTIONS(2017),
    [sym_data_max_url_length] = ACTIONS(2017),
    [sym_max_last_login_info_length] = ACTIONS(2017),
    [sym_data_type_unknown] = ACTIONS(2017),
    [sym_data_type_string] = ACTIONS(2017),
    [sym_data_type_datetime] = ACTIONS(2017),
    [sym_data_type_date] = ACTIONS(2017),
    [sym_data_type_time] = ACTIONS(2017),
    [sym_data_type_image] = ACTIONS(2017),
    [sym_data_format_url] = ACTIONS(2017),
    [sym_data_format_phone] = ACTIONS(2017),
    [sym_data_format_email] = ACTIONS(2017),
    [sym_data_format_iso8601] = ACTIONS(2017),
    [sym_auditlog_priv] = ACTIONS(2017),
    [sym_configuration_priv] = ACTIONS(2017),
    [sym_deviceconfig_priv] = ACTIONS(2017),
    [sym_ftp_priv] = ACTIONS(2017),
    [sym_http_priv] = ACTIONS(2017),
    [sym_networkconfig_priv] = ACTIONS(2017),
    [sym_programport_priv] = ACTIONS(2017),
    [sym_remoteui_priv] = ACTIONS(2017),
    [sym_securitycontrol_priv] = ACTIONS(2017),
    [sym_softwaremanagement_priv] = ACTIONS(2017),
    [sym_terminal_priv] = ACTIONS(2017),
    [sym_tpadmin_priv] = ACTIONS(2017),
    [sym_user1_priv] = ACTIONS(2017),
    [sym_user2_priv] = ACTIONS(2017),
    [sym_user3_priv] = ACTIONS(2017),
    [sym_user4_priv] = ACTIONS(2017),
    [sym_usermanagement_priv] = ACTIONS(2017),
    [sym_valid_account] = ACTIONS(2017),
    [sym_err_name_invalid] = ACTIONS(2017),
    [sym_err_password_invalid] = ACTIONS(2017),
    [sym_err_invalid_account] = ACTIONS(2017),
    [sym_err_no_authorization] = ACTIONS(2017),
    [sym_err_invalid_parameter] = ACTIONS(2017),
    [sym_err_user_locked_out] = ACTIONS(2017),
    [sym_err_user_account_expired] = ACTIONS(2017),
    [sym_err_general_error] = ACTIONS(2017),
    [sym_err_authentication_error] = ACTIONS(2017),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2017),
    [sym_tls_validate_certificate] = ACTIONS(2017),
    [sym_tdata] = ACTIONS(2017),
    [sym_tchannel] = ACTIONS(2017),
    [sym_tlevel] = ACTIONS(2017),
    [sym_tbutton] = ACTIONS(2017),
    [sym_ttimeline] = ACTIONS(2017),
    [sym_tcustom] = ACTIONS(2017),
    [sym_url_struct] = ACTIONS(2017),
    [sym_dns_struct] = ACTIONS(2017),
    [sym_ip_address_struct] = ACTIONS(2017),
    [sym_dev_info_struct] = ACTIONS(2017),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2017),
    [sym_data_feed] = ACTIONS(2017),
    [sym_data_field] = ACTIONS(2017),
    [sym_data_record] = ACTIONS(2017),
    [sym_wc_data_feed] = ACTIONS(2017),
    [sym_wc_data_field] = ACTIONS(2017),
    [sym_wc_data_record] = ACTIONS(2017),
    [sym_last_login_info] = ACTIONS(2017),
    [sym____reserved___] = ACTIONS(2017),
    [sym_date] = ACTIONS(2017),
    [sym_day] = ACTIONS(2017),
    [sym_time] = ACTIONS(2017),
    [sym_ldate] = ACTIONS(2017),
    [sym_push_device] = ACTIONS(2017),
    [sym_push_channel] = ACTIONS(2017),
    [sym_push_devchan] = ACTIONS(2017),
    [sym_release_device] = ACTIONS(2017),
    [sym_release_channel] = ACTIONS(2017),
    [sym_release_devchan] = ACTIONS(2017),
    [sym_master_slot] = ACTIONS(2017),
    [sym_get_pulse_time] = ACTIONS(2017),
    [sym_get_timer] = ACTIONS(2017),
    [sym_master_sn] = ACTIONS(2017),
    [sym_system_number] = ACTIONS(2017),
    [sym_button] = ACTIONS(2017),
    [sym_dv_channel] = ACTIONS(2017),
    [sym_data] = ACTIONS(2017),
    [sym_level] = ACTIONS(2017),
    [sym_timeline] = ACTIONS(2017),
    [sym_channel] = ACTIONS(2017),
    [sym_custom] = ACTIONS(2017),
    [sym_atoi] = ACTIONS(2017),
    [sym_atol] = ACTIONS(2017),
    [sym_atof] = ACTIONS(2017),
    [sym_itoa] = ACTIONS(2017),
    [sym_itohex] = ACTIONS(2017),
    [sym_hextoi] = ACTIONS(2017),
    [sym_ftoa] = ACTIONS(2017),
    [sym_find_string] = ACTIONS(2017),
    [sym_left_string] = ACTIONS(2017),
    [sym_right_string] = ACTIONS(2017),
    [sym_mid_string] = ACTIONS(2017),
    [sym_remove_string] = ACTIONS(2017),
    [sym_get_buffer_string] = ACTIONS(2017),
    [sym_get_buffer_char] = ACTIONS(2017),
    [sym_get_multi_buffer_string] = ACTIONS(2017),
    [sym_length_string] = ACTIONS(2017),
    [sym_max_length_string] = ACTIONS(2017),
    [sym_set_length_string] = ACTIONS(2017),
    [sym_compare_string] = ACTIONS(2017),
    [sym_upper_string] = ACTIONS(2017),
    [sym_lower_string] = ACTIONS(2017),
    [sym_get_last] = ACTIONS(2017),
    [sym_date_to_day] = ACTIONS(2017),
    [sym_date_to_month] = ACTIONS(2017),
    [sym_date_to_year] = ACTIONS(2017),
    [sym_time_to_hour] = ACTIONS(2017),
    [sym_time_to_minute] = ACTIONS(2017),
    [sym_time_to_second] = ACTIONS(2017),
    [sym_day_of_week] = ACTIONS(2017),
    [sym_set_pulse_time] = ACTIONS(2017),
    [sym_set_timer] = ACTIONS(2017),
    [sym_file_open] = ACTIONS(2017),
    [sym_file_close] = ACTIONS(2017),
    [sym_file_read] = ACTIONS(2017),
    [sym_file_read_line] = ACTIONS(2017),
    [sym_file_write] = ACTIONS(2017),
    [sym_file_write_line] = ACTIONS(2017),
    [sym_file_copy] = ACTIONS(2017),
    [sym_file_delete] = ACTIONS(2017),
    [sym_file_rename] = ACTIONS(2017),
    [sym_file_dir] = ACTIONS(2017),
    [sym_file_getdir] = ACTIONS(2017),
    [sym_file_setdir] = ACTIONS(2017),
    [sym_file_create_dir] = ACTIONS(2017),
    [sym_file_remove_dir] = ACTIONS(2017),
    [sym_file_seek] = ACTIONS(2017),
    [sym_length_array] = ACTIONS(2017),
    [sym_max_length_array] = ACTIONS(2017),
    [sym_set_length_array] = ACTIONS(2017),
    [sym_ip_client_open] = ACTIONS(2017),
    [sym_ip_client_close] = ACTIONS(2017),
    [sym_ip_server_open] = ACTIONS(2017),
    [sym_ip_server_close] = ACTIONS(2017),
    [sym_ip_bound_client_open] = ACTIONS(2017),
    [sym_ip_mc_server_open] = ACTIONS(2017),
    [sym_ip_set_option] = ACTIONS(2017),
    [sym_ssh_client_open] = ACTIONS(2017),
    [sym_ssh_client_close] = ACTIONS(2017),
    [sym_tls_client_open] = ACTIONS(2017),
    [sym_tls_client_close] = ACTIONS(2017),
    [sym_do_push] = ACTIONS(2017),
    [sym_do_push_timed] = ACTIONS(2017),
    [sym_do_release] = ACTIONS(2017),
    [sym_device_id] = ACTIONS(2017),
    [sym_device_info] = ACTIONS(2017),
    [sym_device_id_string] = ACTIONS(2017),
    [sym_device_standby] = ACTIONS(2017),
    [sym_device_wake] = ACTIONS(2017),
    [sym_timeline_create] = ACTIONS(2017),
    [sym_timeline_kill] = ACTIONS(2017),
    [sym_timeline_reload] = ACTIONS(2017),
    [sym_timeline_pause] = ACTIONS(2017),
    [sym_timeline_restart] = ACTIONS(2017),
    [sym_timeline_set] = ACTIONS(2017),
    [sym_timeline_get] = ACTIONS(2017),
    [sym_timeline_active] = ACTIONS(2017),
    [sym_timeline_dynamic_id] = ACTIONS(2017),
    [sym_variable_to_string] = ACTIONS(2017),
    [sym_string_to_variable] = ACTIONS(2017),
    [sym_length_variable_to_string] = ACTIONS(2017),
    [sym_type_cast] = ACTIONS(2017),
    [sym_variable_to_xml] = ACTIONS(2017),
    [sym_xml_to_variable] = ACTIONS(2017),
    [sym_length_variable_to_xml] = ACTIONS(2017),
    [sym_redirect_string] = ACTIONS(2017),
    [sym_random_number] = ACTIONS(2017),
    [sym_get_unique_id] = ACTIONS(2017),
    [sym_get_system_number] = ACTIONS(2017),
    [sym_set_system_number] = ACTIONS(2017),
    [sym_get_serial_number] = ACTIONS(2017),
    [sym_get_master_build] = ACTIONS(2017),
    [sym_set_validation_code] = ACTIONS(2017),
    [sym_reboot] = ACTIONS(2017),
    [sym_load_duet_module] = ACTIONS(2017),
    [sym_unload_duet_module] = ACTIONS(2017),
    [sym_duet_mem_size_set] = ACTIONS(2017),
    [sym_duet_mem_size_get] = ACTIONS(2017),
    [sym_dynamic_application_device] = ACTIONS(2017),
    [sym_dynamic_polled_port] = ACTIONS(2017),
    [sym_static_port_binding] = ACTIONS(2017),
    [sym_static_ip_binding] = ACTIONS(2017),
    [sym_get_dns_list] = ACTIONS(2017),
    [sym_set_dns_list] = ACTIONS(2017),
    [sym_get_ip_address] = ACTIONS(2017),
    [sym_set_ip_address] = ACTIONS(2017),
    [sym_get_url_list] = ACTIONS(2017),
    [sym_add_url_entry] = ACTIONS(2017),
    [sym_delete_url_entry] = ACTIONS(2017),
    [sym_abs_value] = ACTIONS(2017),
    [sym_max_value] = ACTIONS(2017),
    [sym_min_value] = ACTIONS(2017),
    [sym_exp_value] = ACTIONS(2017),
    [sym_log_value] = ACTIONS(2017),
    [sym_log10_value] = ACTIONS(2017),
    [sym_power_value] = ACTIONS(2017),
    [sym_sqrt_value] = ACTIONS(2017),
    [sym_raw_be] = ACTIONS(2017),
    [sym_raw_le] = ACTIONS(2017),
    [sym_internal_queue_size_set] = ACTIONS(2017),
    [sym_internal_queue_size_get] = ACTIONS(2017),
    [sym_internal_threshold_set] = ACTIONS(2017),
    [sym_internal_threshold_get] = ACTIONS(2017),
    [sym_rebuilt_event] = ACTIONS(2017),
    [sym_do_custom_event] = ACTIONS(2017),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2017),
    [sym_clkmgr_set_clk_source] = ACTIONS(2017),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2017),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2017),
    [sym_clkmgr_get_timezone] = ACTIONS(2017),
    [sym_clkmgr_set_timezone] = ACTIONS(2017),
    [sym_clkmgr_get_resync_period] = ACTIONS(2017),
    [sym_clkmgr_set_resync_period] = ACTIONS(2017),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2017),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2017),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2017),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2017),
    [sym_clkmgr_get_timeservers] = ACTIONS(2017),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2017),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2017),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2017),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2017),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2017),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2017),
    [sym_set_log_level] = ACTIONS(2017),
    [sym_get_log_level] = ACTIONS(2017),
    [sym_amx_log] = ACTIONS(2017),
    [sym_smtp_server_config_set] = ACTIONS(2017),
    [sym_smtp_server_config_get] = ACTIONS(2017),
    [sym_smtp_send] = ACTIONS(2017),
    [sym_validate_netlinx_account] = ACTIONS(2017),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2017),
    [sym_authenticate_certificate] = ACTIONS(2017),
    [sym_audit_netlinx_session_event] = ACTIONS(2017),
    [sym_audit_netlinx_generic_event] = ACTIONS(2017),
    [sym_get_available_flash_disk_space] = ACTIONS(2017),
    [sym_get_max_flash_disk_space] = ACTIONS(2017),
    [sym_data_create_feed] = ACTIONS(2017),
    [sym_data_delete_feed] = ACTIONS(2017),
    [sym_data_publish_feed] = ACTIONS(2017),
    [sym_data_get_published_feed] = ACTIONS(2017),
    [sym_data_add_record] = ACTIONS(2017),
    [sym_data_get_event_record] = ACTIONS(2017),
    [sym__wc_data_create_feed] = ACTIONS(2017),
    [sym__wc_data_add_record] = ACTIONS(2017),
    [sym__wc_data_get_event_record] = ACTIONS(2017),
    [sym_format] = ACTIONS(2017),
    [sym_astro_clock] = ACTIONS(2017),
    [sym_set_outdoor_temperature] = ACTIONS(2017),
    [sym_set_virtual_level_count] = ACTIONS(2017),
    [sym_set_virtual_channel_count] = ACTIONS(2017),
    [sym_set_virtual_port_count] = ACTIONS(2017),
    [sym__wc_encode_int] = ACTIONS(2017),
    [sym__wc_decode_int] = ACTIONS(2017),
    [sym__wc_int] = ACTIONS(2017),
    [sym__wc_to_ch_int] = ACTIONS(2017),
    [sym__ch_to_wc_int] = ACTIONS(2017),
    [sym__wc_find_string_int] = ACTIONS(2017),
    [sym__wc_left_string_int] = ACTIONS(2017),
    [sym__wc_lower_string_int] = ACTIONS(2017),
    [sym__wc_mid_string_int] = ACTIONS(2017),
    [sym__wc_remove_string_int] = ACTIONS(2017),
    [sym__wc_right_string_int] = ACTIONS(2017),
    [sym__wc_upper_string_int] = ACTIONS(2017),
    [sym__wc_compare_string_int] = ACTIONS(2017),
    [sym__wc_get_buffer_char_int] = ACTIONS(2017),
    [sym__wc_get_buffer_string_int] = ACTIONS(2017),
    [sym__wc_concat_string_int] = ACTIONS(2017),
    [sym__wc_file_open_int] = ACTIONS(2017),
    [sym__wc_file_close_int] = ACTIONS(2017),
    [sym__wc_file_read_int] = ACTIONS(2017),
    [sym__wc_file_read_line_int] = ACTIONS(2017),
    [sym__wc_file_write_int] = ACTIONS(2017),
    [sym__wc_file_write_line_int] = ACTIONS(2017),
    [aux_sym_preproc_include_token1] = ACTIONS(2017),
    [aux_sym_preproc_define_token1] = ACTIONS(2019),
    [aux_sym_preproc_warn_token1] = ACTIONS(2019),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2019),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2019),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2019),
    [anon_sym_LBRACK] = ACTIONS(2019),
    [anon_sym_LBRACE] = ACTIONS(2019),
    [anon_sym_LPAREN] = ACTIONS(2017),
    [anon_sym_BANG] = ACTIONS(2019),
    [anon_sym_TILDE] = ACTIONS(2019),
    [anon_sym_DASH] = ACTIONS(2017),
    [anon_sym_PLUS] = ACTIONS(2017),
    [anon_sym_DASH_DASH] = ACTIONS(2019),
    [anon_sym_PLUS_PLUS] = ACTIONS(2019),
    [anon_sym_DQUOTE] = ACTIONS(2019),
    [anon_sym_SQUOTE] = ACTIONS(2019),
    [sym_number_literal] = ACTIONS(2019),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(400)] = {
    [ts_builtin_sym_end] = ACTIONS(2087),
    [sym_identifier] = ACTIONS(2085),
    [sym_program_name_keyword] = ACTIONS(2085),
    [sym_module_name_keyword] = ACTIONS(2085),
    [sym_define_device_keyword] = ACTIONS(2085),
    [sym_define_combine_keyword] = ACTIONS(2085),
    [sym_define_constant_keyword] = ACTIONS(2085),
    [sym_define_type_keyword] = ACTIONS(2085),
    [sym_define_variable_keyword] = ACTIONS(2085),
    [sym_define_system_variable_keyword] = ACTIONS(2085),
    [sym_define_connect_level_keyword] = ACTIONS(2085),
    [sym_define_latching_keyword] = ACTIONS(2085),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2085),
    [sym_define_toggling_keyword] = ACTIONS(2085),
    [sym_define_call_keyword] = ACTIONS(2085),
    [sym_define_function_keyword] = ACTIONS(2085),
    [sym_define_library_function_keyword] = ACTIONS(2085),
    [sym_define_start_keyword] = ACTIONS(2085),
    [sym_define_event_keyword] = ACTIONS(2085),
    [sym_define_program_keyword] = ACTIONS(2085),
    [sym_define_module_keyword] = ACTIONS(2085),
    [sym_char_keyword] = ACTIONS(2085),
    [sym_widechar_keyword] = ACTIONS(2085),
    [sym_integer_keyword] = ACTIONS(2085),
    [sym_sinteger_keyword] = ACTIONS(2085),
    [sym_long_keyword] = ACTIONS(2085),
    [sym_slong_keyword] = ACTIONS(2085),
    [sym_float_keyword] = ACTIONS(2085),
    [sym_double_keyword] = ACTIONS(2085),
    [sym_variant_keyword] = ACTIONS(2085),
    [sym_variantarray_keyword] = ACTIONS(2085),
    [sym_local_var_keyword] = ACTIONS(2085),
    [sym_stack_var_keyword] = ACTIONS(2085),
    [sym_constant_keyword] = ACTIONS(2085),
    [sym_volatile_keyword] = ACTIONS(2085),
    [sym_non_volatile_keyword] = ACTIONS(2085),
    [sym_persistent_keyword] = ACTIONS(2085),
    [sym_dev_keyword] = ACTIONS(2085),
    [sym_devlev_keyword] = ACTIONS(2085),
    [sym_devchan_keyword] = ACTIONS(2085),
    [sym_if_keyword] = ACTIONS(2085),
    [sym_switch_keyword] = ACTIONS(2085),
    [sym_case_keyword] = ACTIONS(2085),
    [sym_default_keyword] = ACTIONS(2085),
    [sym_while_keyword] = ACTIONS(2085),
    [sym_for_keyword] = ACTIONS(2085),
    [sym_break_keyword] = ACTIONS(2085),
    [sym_continue_keyword] = ACTIONS(2085),
    [sym_return_keyword] = ACTIONS(2085),
    [sym_select_keyword] = ACTIONS(2085),
    [sym_active_keyword] = ACTIONS(2085),
    [sym_struct_keyword] = ACTIONS(2085),
    [sym_structure_keyword] = ACTIONS(2085),
    [sym_button_event_keyword] = ACTIONS(2085),
    [sym_channel_event_keyword] = ACTIONS(2085),
    [sym_level_event_keyword] = ACTIONS(2085),
    [sym_data_event_keyword] = ACTIONS(2085),
    [sym_timeline_event_keyword] = ACTIONS(2085),
    [sym_custom_event_keyword] = ACTIONS(2085),
    [sym_push_keyword] = ACTIONS(2085),
    [sym_release_keyword] = ACTIONS(2085),
    [aux_sym_on_keyword_token1] = ACTIONS(2085),
    [aux_sym_off_keyword_token1] = ACTIONS(2085),
    [sym_send_level_keyword] = ACTIONS(2085),
    [sym_send_string_keyword] = ACTIONS(2085),
    [sym_send_command_keyword] = ACTIONS(2085),
    [sym_clear_buffer_keyword] = ACTIONS(2085),
    [sym_create_buffer_keyword] = ACTIONS(2085),
    [sym_create_multi_buffer_keyword] = ACTIONS(2085),
    [sym_call_keyword] = ACTIONS(2085),
    [sym_system_call_keyword] = ACTIONS(2085),
    [sym_devchan_to_keyword] = ACTIONS(2085),
    [sym_devchan_min_to_keyword] = ACTIONS(2085),
    [sym_devchan_total_off_keyword] = ACTIONS(2085),
    [sym_devchan_pulse_keyword] = ACTIONS(2085),
    [sym_wait_keyword] = ACTIONS(2085),
    [sym_cancel_wait_keyword] = ACTIONS(2085),
    [sym_wait_until_keyword] = ACTIONS(2085),
    [sym_cancel_wait_until_keyword] = ACTIONS(2085),
    [sym_cancel_all_wait_keyword] = ACTIONS(2085),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2085),
    [sym_not] = ACTIONS(2085),
    [sym_bnot] = ACTIONS(2085),
    [sym___file__] = ACTIONS(2085),
    [sym___date__] = ACTIONS(2085),
    [sym___ldate__] = ACTIONS(2085),
    [sym___time__] = ACTIONS(2085),
    [sym___line__] = ACTIONS(2085),
    [sym___name__] = ACTIONS(2085),
    [sym___netlinx__] = ACTIONS(2085),
    [sym_netlinx_axi_version] = ACTIONS(2085),
    [sym_url_flg_tcp] = ACTIONS(2085),
    [sym_url_flg_accinfopresent] = ACTIONS(2085),
    [sym_url_flg_temp] = ACTIONS(2085),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2085),
    [sym_url_flg_stat_mask] = ACTIONS(2085),
    [sym_url_flg_stat_lookup] = ACTIONS(2085),
    [sym_url_flg_stat_connecting] = ACTIONS(2085),
    [sym_url_flg_stat_waiting] = ACTIONS(2085),
    [sym_url_flg_stat_connected] = ACTIONS(2085),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2085),
    [sym_first_virtual_device] = ACTIONS(2085),
    [sym_first_local_port] = ACTIONS(2085),
    [sym_dynamic_virtual_device] = ACTIONS(2085),
    [sym_timeline_once] = ACTIONS(2085),
    [sym_timeline_repeat] = ACTIONS(2085),
    [sym_timeline_absolute] = ACTIONS(2085),
    [sym_timeline_relative] = ACTIONS(2085),
    [sym_do_push_timed_infinite] = ACTIONS(2085),
    [sym_source_type_no_address] = ACTIONS(2085),
    [sym_source_type_neuron_id] = ACTIONS(2085),
    [sym_source_type_ip_address] = ACTIONS(2085),
    [sym_source_type_axlink] = ACTIONS(2085),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2085),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2085),
    [sym_source_type_ip_socket_address] = ACTIONS(2085),
    [sym_source_type_rs232] = ACTIONS(2085),
    [sym_source_type_internal] = ACTIONS(2085),
    [sym_source_type_mac_address] = ACTIONS(2085),
    [sym_source_type_ipv4_port] = ACTIONS(2085),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2085),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2085),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2085),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2085),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2085),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2085),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2085),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2085),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2085),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2085),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2085),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2085),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2085),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2085),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2085),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2085),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2085),
    [sym_true] = ACTIONS(2085),
    [sym_false] = ACTIONS(2085),
    [sym_file_read_only] = ACTIONS(2085),
    [sym_file_rw_new] = ACTIONS(2085),
    [sym_file_rw_append] = ACTIONS(2085),
    [sym_ip_tcp] = ACTIONS(2085),
    [sym_ip_udp] = ACTIONS(2085),
    [sym_ip_udp_2way] = ACTIONS(2085),
    [sym_xml_encode_types] = ACTIONS(2085),
    [sym_xml_encode_char_as_list] = ACTIONS(2085),
    [sym_xml_encode_le] = ACTIONS(2085),
    [sym_xml_decode_types] = ACTIONS(2085),
    [sym_xml_decode_no_preserve] = ACTIONS(2085),
    [sym_duet_dev_type_display_device] = ACTIONS(2085),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2085),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2085),
    [sym_duet_dev_type_io_device] = ACTIONS(2085),
    [sym_duet_dev_type_relay_device] = ACTIONS(2085),
    [sym_duet_dev_type_ups] = ACTIONS(2085),
    [sym_duet_dev_type_amplifier] = ACTIONS(2085),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2085),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2085),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2085),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2085),
    [sym_duet_dev_type_camera] = ACTIONS(2085),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2085),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2085),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2085),
    [sym_duet_dev_type_dss] = ACTIONS(2085),
    [sym_duet_dev_type_dvr] = ACTIONS(2085),
    [sym_duet_dev_type_disc_device] = ACTIONS(2085),
    [sym_duet_dev_type_document_camera] = ACTIONS(2085),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2085),
    [sym_duet_dev_type_hvac] = ACTIONS(2085),
    [sym_duet_dev_type_keypad] = ACTIONS(2085),
    [sym_duet_dev_type_light] = ACTIONS(2085),
    [sym_duet_dev_type_monitor] = ACTIONS(2085),
    [sym_duet_dev_type_motor] = ACTIONS(2085),
    [sym_duet_dev_type_multi_window] = ACTIONS(2085),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2085),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2085),
    [sym_duet_dev_type_receiver] = ACTIONS(2085),
    [sym_duet_dev_type_security_system] = ACTIONS(2085),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2085),
    [sym_duet_dev_type_settop_box] = ACTIONS(2085),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2085),
    [sym_duet_dev_type_switcher] = ACTIONS(2085),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2085),
    [sym_duet_dev_type_tv] = ACTIONS(2085),
    [sym_duet_dev_type_utility] = ACTIONS(2085),
    [sym_duet_dev_type_vcr] = ACTIONS(2085),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2085),
    [sym_duet_dev_type_video_processor] = ACTIONS(2085),
    [sym_duet_dev_type_video_projector] = ACTIONS(2085),
    [sym_duet_dev_type_video_wall] = ACTIONS(2085),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2085),
    [sym_duet_dev_type_weather] = ACTIONS(2085),
    [sym_duet_dev_not_polled] = ACTIONS(2085),
    [sym_duet_dev_polled] = ACTIONS(2085),
    [sym_clkmgr_mode_network] = ACTIONS(2085),
    [sym_clkmgr_mode_standalone] = ACTIONS(2085),
    [sym_normal_standby] = ACTIONS(2085),
    [sym_normal_wake] = ACTIONS(2085),
    [sym_ip_multicast_ttl_option] = ACTIONS(2085),
    [sym_ip_ttl_subnet] = ACTIONS(2085),
    [sym_ip_ttl_site] = ACTIONS(2085),
    [sym_ip_ttl_region] = ACTIONS(2085),
    [sym_ip_ttl_continent] = ACTIONS(2085),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2085),
    [sym_ip_nodelay_on] = ACTIONS(2085),
    [sym_ip_nodelay_off] = ACTIONS(2085),
    [sym_amx_error] = ACTIONS(2085),
    [sym_amx_warning] = ACTIONS(2085),
    [sym_amx_info] = ACTIONS(2085),
    [sym_amx_debug] = ACTIONS(2085),
    [sym_netlinx_login_success] = ACTIONS(2085),
    [sym_netlinx_login_fail] = ACTIONS(2085),
    [sym_netlinx_logout] = ACTIONS(2085),
    [sym_null_str] = ACTIONS(2085),
    [sym_smtp_address] = ACTIONS(2085),
    [sym_smtp_port_number] = ACTIONS(2085),
    [sym_smtp_username] = ACTIONS(2085),
    [sym_smtp_password] = ACTIONS(2085),
    [sym_smtp_require_tls] = ACTIONS(2085),
    [sym_smtp_from] = ACTIONS(2085),
    [sym_smtp_tls_true] = ACTIONS(2085),
    [sym_smtp_tls_false] = ACTIONS(2085),
    [sym_listview_on_row_select_event] = ACTIONS(2085),
    [sym_data_structure_unknown] = ACTIONS(2085),
    [sym_data_structure_datafeed] = ACTIONS(2085),
    [sym_data_structure_datarecordset] = ACTIONS(2085),
    [sym_data_structure_datarecord] = ACTIONS(2085),
    [sym_data_structure_datafield] = ACTIONS(2085),
    [sym_data_max_content_field_count] = ACTIONS(2085),
    [sym_data_max_metadata_field_count] = ACTIONS(2085),
    [sym_data_max_id_length] = ACTIONS(2085),
    [sym_data_max_name_length] = ACTIONS(2085),
    [sym_data_max_description_length] = ACTIONS(2085),
    [sym_data_max_type_length] = ACTIONS(2085),
    [sym_data_max_format_length] = ACTIONS(2085),
    [sym_data_max_source_length] = ACTIONS(2085),
    [sym_data_max_value_length] = ACTIONS(2085),
    [sym_data_max_label_length] = ACTIONS(2085),
    [sym_data_max_url_length] = ACTIONS(2085),
    [sym_max_last_login_info_length] = ACTIONS(2085),
    [sym_data_type_unknown] = ACTIONS(2085),
    [sym_data_type_string] = ACTIONS(2085),
    [sym_data_type_datetime] = ACTIONS(2085),
    [sym_data_type_date] = ACTIONS(2085),
    [sym_data_type_time] = ACTIONS(2085),
    [sym_data_type_image] = ACTIONS(2085),
    [sym_data_format_url] = ACTIONS(2085),
    [sym_data_format_phone] = ACTIONS(2085),
    [sym_data_format_email] = ACTIONS(2085),
    [sym_data_format_iso8601] = ACTIONS(2085),
    [sym_auditlog_priv] = ACTIONS(2085),
    [sym_configuration_priv] = ACTIONS(2085),
    [sym_deviceconfig_priv] = ACTIONS(2085),
    [sym_ftp_priv] = ACTIONS(2085),
    [sym_http_priv] = ACTIONS(2085),
    [sym_networkconfig_priv] = ACTIONS(2085),
    [sym_programport_priv] = ACTIONS(2085),
    [sym_remoteui_priv] = ACTIONS(2085),
    [sym_securitycontrol_priv] = ACTIONS(2085),
    [sym_softwaremanagement_priv] = ACTIONS(2085),
    [sym_terminal_priv] = ACTIONS(2085),
    [sym_tpadmin_priv] = ACTIONS(2085),
    [sym_user1_priv] = ACTIONS(2085),
    [sym_user2_priv] = ACTIONS(2085),
    [sym_user3_priv] = ACTIONS(2085),
    [sym_user4_priv] = ACTIONS(2085),
    [sym_usermanagement_priv] = ACTIONS(2085),
    [sym_valid_account] = ACTIONS(2085),
    [sym_err_name_invalid] = ACTIONS(2085),
    [sym_err_password_invalid] = ACTIONS(2085),
    [sym_err_invalid_account] = ACTIONS(2085),
    [sym_err_no_authorization] = ACTIONS(2085),
    [sym_err_invalid_parameter] = ACTIONS(2085),
    [sym_err_user_locked_out] = ACTIONS(2085),
    [sym_err_user_account_expired] = ACTIONS(2085),
    [sym_err_general_error] = ACTIONS(2085),
    [sym_err_authentication_error] = ACTIONS(2085),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2085),
    [sym_tls_validate_certificate] = ACTIONS(2085),
    [sym_tdata] = ACTIONS(2085),
    [sym_tchannel] = ACTIONS(2085),
    [sym_tlevel] = ACTIONS(2085),
    [sym_tbutton] = ACTIONS(2085),
    [sym_ttimeline] = ACTIONS(2085),
    [sym_tcustom] = ACTIONS(2085),
    [sym_url_struct] = ACTIONS(2085),
    [sym_dns_struct] = ACTIONS(2085),
    [sym_ip_address_struct] = ACTIONS(2085),
    [sym_dev_info_struct] = ACTIONS(2085),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2085),
    [sym_data_feed] = ACTIONS(2085),
    [sym_data_field] = ACTIONS(2085),
    [sym_data_record] = ACTIONS(2085),
    [sym_wc_data_feed] = ACTIONS(2085),
    [sym_wc_data_field] = ACTIONS(2085),
    [sym_wc_data_record] = ACTIONS(2085),
    [sym_last_login_info] = ACTIONS(2085),
    [sym____reserved___] = ACTIONS(2085),
    [sym_date] = ACTIONS(2085),
    [sym_day] = ACTIONS(2085),
    [sym_time] = ACTIONS(2085),
    [sym_ldate] = ACTIONS(2085),
    [sym_push_device] = ACTIONS(2085),
    [sym_push_channel] = ACTIONS(2085),
    [sym_push_devchan] = ACTIONS(2085),
    [sym_release_device] = ACTIONS(2085),
    [sym_release_channel] = ACTIONS(2085),
    [sym_release_devchan] = ACTIONS(2085),
    [sym_master_slot] = ACTIONS(2085),
    [sym_get_pulse_time] = ACTIONS(2085),
    [sym_get_timer] = ACTIONS(2085),
    [sym_master_sn] = ACTIONS(2085),
    [sym_system_number] = ACTIONS(2085),
    [sym_button] = ACTIONS(2085),
    [sym_dv_channel] = ACTIONS(2085),
    [sym_data] = ACTIONS(2085),
    [sym_level] = ACTIONS(2085),
    [sym_timeline] = ACTIONS(2085),
    [sym_channel] = ACTIONS(2085),
    [sym_custom] = ACTIONS(2085),
    [sym_atoi] = ACTIONS(2085),
    [sym_atol] = ACTIONS(2085),
    [sym_atof] = ACTIONS(2085),
    [sym_itoa] = ACTIONS(2085),
    [sym_itohex] = ACTIONS(2085),
    [sym_hextoi] = ACTIONS(2085),
    [sym_ftoa] = ACTIONS(2085),
    [sym_find_string] = ACTIONS(2085),
    [sym_left_string] = ACTIONS(2085),
    [sym_right_string] = ACTIONS(2085),
    [sym_mid_string] = ACTIONS(2085),
    [sym_remove_string] = ACTIONS(2085),
    [sym_get_buffer_string] = ACTIONS(2085),
    [sym_get_buffer_char] = ACTIONS(2085),
    [sym_get_multi_buffer_string] = ACTIONS(2085),
    [sym_length_string] = ACTIONS(2085),
    [sym_max_length_string] = ACTIONS(2085),
    [sym_set_length_string] = ACTIONS(2085),
    [sym_compare_string] = ACTIONS(2085),
    [sym_upper_string] = ACTIONS(2085),
    [sym_lower_string] = ACTIONS(2085),
    [sym_get_last] = ACTIONS(2085),
    [sym_date_to_day] = ACTIONS(2085),
    [sym_date_to_month] = ACTIONS(2085),
    [sym_date_to_year] = ACTIONS(2085),
    [sym_time_to_hour] = ACTIONS(2085),
    [sym_time_to_minute] = ACTIONS(2085),
    [sym_time_to_second] = ACTIONS(2085),
    [sym_day_of_week] = ACTIONS(2085),
    [sym_set_pulse_time] = ACTIONS(2085),
    [sym_set_timer] = ACTIONS(2085),
    [sym_file_open] = ACTIONS(2085),
    [sym_file_close] = ACTIONS(2085),
    [sym_file_read] = ACTIONS(2085),
    [sym_file_read_line] = ACTIONS(2085),
    [sym_file_write] = ACTIONS(2085),
    [sym_file_write_line] = ACTIONS(2085),
    [sym_file_copy] = ACTIONS(2085),
    [sym_file_delete] = ACTIONS(2085),
    [sym_file_rename] = ACTIONS(2085),
    [sym_file_dir] = ACTIONS(2085),
    [sym_file_getdir] = ACTIONS(2085),
    [sym_file_setdir] = ACTIONS(2085),
    [sym_file_create_dir] = ACTIONS(2085),
    [sym_file_remove_dir] = ACTIONS(2085),
    [sym_file_seek] = ACTIONS(2085),
    [sym_length_array] = ACTIONS(2085),
    [sym_max_length_array] = ACTIONS(2085),
    [sym_set_length_array] = ACTIONS(2085),
    [sym_ip_client_open] = ACTIONS(2085),
    [sym_ip_client_close] = ACTIONS(2085),
    [sym_ip_server_open] = ACTIONS(2085),
    [sym_ip_server_close] = ACTIONS(2085),
    [sym_ip_bound_client_open] = ACTIONS(2085),
    [sym_ip_mc_server_open] = ACTIONS(2085),
    [sym_ip_set_option] = ACTIONS(2085),
    [sym_ssh_client_open] = ACTIONS(2085),
    [sym_ssh_client_close] = ACTIONS(2085),
    [sym_tls_client_open] = ACTIONS(2085),
    [sym_tls_client_close] = ACTIONS(2085),
    [sym_do_push] = ACTIONS(2085),
    [sym_do_push_timed] = ACTIONS(2085),
    [sym_do_release] = ACTIONS(2085),
    [sym_device_id] = ACTIONS(2085),
    [sym_device_info] = ACTIONS(2085),
    [sym_device_id_string] = ACTIONS(2085),
    [sym_device_standby] = ACTIONS(2085),
    [sym_device_wake] = ACTIONS(2085),
    [sym_timeline_create] = ACTIONS(2085),
    [sym_timeline_kill] = ACTIONS(2085),
    [sym_timeline_reload] = ACTIONS(2085),
    [sym_timeline_pause] = ACTIONS(2085),
    [sym_timeline_restart] = ACTIONS(2085),
    [sym_timeline_set] = ACTIONS(2085),
    [sym_timeline_get] = ACTIONS(2085),
    [sym_timeline_active] = ACTIONS(2085),
    [sym_timeline_dynamic_id] = ACTIONS(2085),
    [sym_variable_to_string] = ACTIONS(2085),
    [sym_string_to_variable] = ACTIONS(2085),
    [sym_length_variable_to_string] = ACTIONS(2085),
    [sym_type_cast] = ACTIONS(2085),
    [sym_variable_to_xml] = ACTIONS(2085),
    [sym_xml_to_variable] = ACTIONS(2085),
    [sym_length_variable_to_xml] = ACTIONS(2085),
    [sym_redirect_string] = ACTIONS(2085),
    [sym_random_number] = ACTIONS(2085),
    [sym_get_unique_id] = ACTIONS(2085),
    [sym_get_system_number] = ACTIONS(2085),
    [sym_set_system_number] = ACTIONS(2085),
    [sym_get_serial_number] = ACTIONS(2085),
    [sym_get_master_build] = ACTIONS(2085),
    [sym_set_validation_code] = ACTIONS(2085),
    [sym_reboot] = ACTIONS(2085),
    [sym_load_duet_module] = ACTIONS(2085),
    [sym_unload_duet_module] = ACTIONS(2085),
    [sym_duet_mem_size_set] = ACTIONS(2085),
    [sym_duet_mem_size_get] = ACTIONS(2085),
    [sym_dynamic_application_device] = ACTIONS(2085),
    [sym_dynamic_polled_port] = ACTIONS(2085),
    [sym_static_port_binding] = ACTIONS(2085),
    [sym_static_ip_binding] = ACTIONS(2085),
    [sym_get_dns_list] = ACTIONS(2085),
    [sym_set_dns_list] = ACTIONS(2085),
    [sym_get_ip_address] = ACTIONS(2085),
    [sym_set_ip_address] = ACTIONS(2085),
    [sym_get_url_list] = ACTIONS(2085),
    [sym_add_url_entry] = ACTIONS(2085),
    [sym_delete_url_entry] = ACTIONS(2085),
    [sym_abs_value] = ACTIONS(2085),
    [sym_max_value] = ACTIONS(2085),
    [sym_min_value] = ACTIONS(2085),
    [sym_exp_value] = ACTIONS(2085),
    [sym_log_value] = ACTIONS(2085),
    [sym_log10_value] = ACTIONS(2085),
    [sym_power_value] = ACTIONS(2085),
    [sym_sqrt_value] = ACTIONS(2085),
    [sym_raw_be] = ACTIONS(2085),
    [sym_raw_le] = ACTIONS(2085),
    [sym_internal_queue_size_set] = ACTIONS(2085),
    [sym_internal_queue_size_get] = ACTIONS(2085),
    [sym_internal_threshold_set] = ACTIONS(2085),
    [sym_internal_threshold_get] = ACTIONS(2085),
    [sym_rebuilt_event] = ACTIONS(2085),
    [sym_do_custom_event] = ACTIONS(2085),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2085),
    [sym_clkmgr_set_clk_source] = ACTIONS(2085),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2085),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2085),
    [sym_clkmgr_get_timezone] = ACTIONS(2085),
    [sym_clkmgr_set_timezone] = ACTIONS(2085),
    [sym_clkmgr_get_resync_period] = ACTIONS(2085),
    [sym_clkmgr_set_resync_period] = ACTIONS(2085),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2085),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2085),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2085),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2085),
    [sym_clkmgr_get_timeservers] = ACTIONS(2085),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2085),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2085),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2085),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2085),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2085),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2085),
    [sym_set_log_level] = ACTIONS(2085),
    [sym_get_log_level] = ACTIONS(2085),
    [sym_amx_log] = ACTIONS(2085),
    [sym_smtp_server_config_set] = ACTIONS(2085),
    [sym_smtp_server_config_get] = ACTIONS(2085),
    [sym_smtp_send] = ACTIONS(2085),
    [sym_validate_netlinx_account] = ACTIONS(2085),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2085),
    [sym_authenticate_certificate] = ACTIONS(2085),
    [sym_audit_netlinx_session_event] = ACTIONS(2085),
    [sym_audit_netlinx_generic_event] = ACTIONS(2085),
    [sym_get_available_flash_disk_space] = ACTIONS(2085),
    [sym_get_max_flash_disk_space] = ACTIONS(2085),
    [sym_data_create_feed] = ACTIONS(2085),
    [sym_data_delete_feed] = ACTIONS(2085),
    [sym_data_publish_feed] = ACTIONS(2085),
    [sym_data_get_published_feed] = ACTIONS(2085),
    [sym_data_add_record] = ACTIONS(2085),
    [sym_data_get_event_record] = ACTIONS(2085),
    [sym__wc_data_create_feed] = ACTIONS(2085),
    [sym__wc_data_add_record] = ACTIONS(2085),
    [sym__wc_data_get_event_record] = ACTIONS(2085),
    [sym_format] = ACTIONS(2085),
    [sym_astro_clock] = ACTIONS(2085),
    [sym_set_outdoor_temperature] = ACTIONS(2085),
    [sym_set_virtual_level_count] = ACTIONS(2085),
    [sym_set_virtual_channel_count] = ACTIONS(2085),
    [sym_set_virtual_port_count] = ACTIONS(2085),
    [sym__wc_encode_int] = ACTIONS(2085),
    [sym__wc_decode_int] = ACTIONS(2085),
    [sym__wc_int] = ACTIONS(2085),
    [sym__wc_to_ch_int] = ACTIONS(2085),
    [sym__ch_to_wc_int] = ACTIONS(2085),
    [sym__wc_find_string_int] = ACTIONS(2085),
    [sym__wc_left_string_int] = ACTIONS(2085),
    [sym__wc_lower_string_int] = ACTIONS(2085),
    [sym__wc_mid_string_int] = ACTIONS(2085),
    [sym__wc_remove_string_int] = ACTIONS(2085),
    [sym__wc_right_string_int] = ACTIONS(2085),
    [sym__wc_upper_string_int] = ACTIONS(2085),
    [sym__wc_compare_string_int] = ACTIONS(2085),
    [sym__wc_get_buffer_char_int] = ACTIONS(2085),
    [sym__wc_get_buffer_string_int] = ACTIONS(2085),
    [sym__wc_concat_string_int] = ACTIONS(2085),
    [sym__wc_file_open_int] = ACTIONS(2085),
    [sym__wc_file_close_int] = ACTIONS(2085),
    [sym__wc_file_read_int] = ACTIONS(2085),
    [sym__wc_file_read_line_int] = ACTIONS(2085),
    [sym__wc_file_write_int] = ACTIONS(2085),
    [sym__wc_file_write_line_int] = ACTIONS(2085),
    [aux_sym_preproc_include_token1] = ACTIONS(2085),
    [aux_sym_preproc_define_token1] = ACTIONS(2087),
    [aux_sym_preproc_warn_token1] = ACTIONS(2087),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2087),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2087),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2087),
    [anon_sym_LBRACK] = ACTIONS(2087),
    [anon_sym_LBRACE] = ACTIONS(2087),
    [anon_sym_LPAREN] = ACTIONS(2085),
    [anon_sym_BANG] = ACTIONS(2087),
    [anon_sym_TILDE] = ACTIONS(2087),
    [anon_sym_DASH] = ACTIONS(2085),
    [anon_sym_PLUS] = ACTIONS(2085),
    [anon_sym_DASH_DASH] = ACTIONS(2087),
    [anon_sym_PLUS_PLUS] = ACTIONS(2087),
    [anon_sym_DQUOTE] = ACTIONS(2087),
    [anon_sym_SQUOTE] = ACTIONS(2087),
    [sym_number_literal] = ACTIONS(2087),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(401)] = {
    [ts_builtin_sym_end] = ACTIONS(1975),
    [sym_identifier] = ACTIONS(1973),
    [sym_program_name_keyword] = ACTIONS(1973),
    [sym_module_name_keyword] = ACTIONS(1973),
    [sym_define_device_keyword] = ACTIONS(1973),
    [sym_define_combine_keyword] = ACTIONS(1973),
    [sym_define_constant_keyword] = ACTIONS(1973),
    [sym_define_type_keyword] = ACTIONS(1973),
    [sym_define_variable_keyword] = ACTIONS(1973),
    [sym_define_system_variable_keyword] = ACTIONS(1973),
    [sym_define_connect_level_keyword] = ACTIONS(1973),
    [sym_define_latching_keyword] = ACTIONS(1973),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1973),
    [sym_define_toggling_keyword] = ACTIONS(1973),
    [sym_define_call_keyword] = ACTIONS(1973),
    [sym_define_function_keyword] = ACTIONS(1973),
    [sym_define_library_function_keyword] = ACTIONS(1973),
    [sym_define_start_keyword] = ACTIONS(1973),
    [sym_define_event_keyword] = ACTIONS(1973),
    [sym_define_program_keyword] = ACTIONS(1973),
    [sym_define_module_keyword] = ACTIONS(1973),
    [sym_char_keyword] = ACTIONS(1973),
    [sym_widechar_keyword] = ACTIONS(1973),
    [sym_integer_keyword] = ACTIONS(1973),
    [sym_sinteger_keyword] = ACTIONS(1973),
    [sym_long_keyword] = ACTIONS(1973),
    [sym_slong_keyword] = ACTIONS(1973),
    [sym_float_keyword] = ACTIONS(1973),
    [sym_double_keyword] = ACTIONS(1973),
    [sym_variant_keyword] = ACTIONS(1973),
    [sym_variantarray_keyword] = ACTIONS(1973),
    [sym_local_var_keyword] = ACTIONS(1973),
    [sym_stack_var_keyword] = ACTIONS(1973),
    [sym_constant_keyword] = ACTIONS(1973),
    [sym_volatile_keyword] = ACTIONS(1973),
    [sym_non_volatile_keyword] = ACTIONS(1973),
    [sym_persistent_keyword] = ACTIONS(1973),
    [sym_dev_keyword] = ACTIONS(1973),
    [sym_devlev_keyword] = ACTIONS(1973),
    [sym_devchan_keyword] = ACTIONS(1973),
    [sym_if_keyword] = ACTIONS(1973),
    [sym_switch_keyword] = ACTIONS(1973),
    [sym_case_keyword] = ACTIONS(1973),
    [sym_default_keyword] = ACTIONS(1973),
    [sym_while_keyword] = ACTIONS(1973),
    [sym_for_keyword] = ACTIONS(1973),
    [sym_break_keyword] = ACTIONS(1973),
    [sym_continue_keyword] = ACTIONS(1973),
    [sym_return_keyword] = ACTIONS(1973),
    [sym_select_keyword] = ACTIONS(1973),
    [sym_active_keyword] = ACTIONS(1973),
    [sym_struct_keyword] = ACTIONS(1973),
    [sym_structure_keyword] = ACTIONS(1973),
    [sym_button_event_keyword] = ACTIONS(1973),
    [sym_channel_event_keyword] = ACTIONS(1973),
    [sym_level_event_keyword] = ACTIONS(1973),
    [sym_data_event_keyword] = ACTIONS(1973),
    [sym_timeline_event_keyword] = ACTIONS(1973),
    [sym_custom_event_keyword] = ACTIONS(1973),
    [sym_push_keyword] = ACTIONS(1973),
    [sym_release_keyword] = ACTIONS(1973),
    [aux_sym_on_keyword_token1] = ACTIONS(1973),
    [aux_sym_off_keyword_token1] = ACTIONS(1973),
    [sym_send_level_keyword] = ACTIONS(1973),
    [sym_send_string_keyword] = ACTIONS(1973),
    [sym_send_command_keyword] = ACTIONS(1973),
    [sym_clear_buffer_keyword] = ACTIONS(1973),
    [sym_create_buffer_keyword] = ACTIONS(1973),
    [sym_create_multi_buffer_keyword] = ACTIONS(1973),
    [sym_call_keyword] = ACTIONS(1973),
    [sym_system_call_keyword] = ACTIONS(1973),
    [sym_devchan_to_keyword] = ACTIONS(1973),
    [sym_devchan_min_to_keyword] = ACTIONS(1973),
    [sym_devchan_total_off_keyword] = ACTIONS(1973),
    [sym_devchan_pulse_keyword] = ACTIONS(1973),
    [sym_wait_keyword] = ACTIONS(1973),
    [sym_cancel_wait_keyword] = ACTIONS(1973),
    [sym_wait_until_keyword] = ACTIONS(1973),
    [sym_cancel_wait_until_keyword] = ACTIONS(1973),
    [sym_cancel_all_wait_keyword] = ACTIONS(1973),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1973),
    [sym_not] = ACTIONS(1973),
    [sym_bnot] = ACTIONS(1973),
    [sym___file__] = ACTIONS(1973),
    [sym___date__] = ACTIONS(1973),
    [sym___ldate__] = ACTIONS(1973),
    [sym___time__] = ACTIONS(1973),
    [sym___line__] = ACTIONS(1973),
    [sym___name__] = ACTIONS(1973),
    [sym___netlinx__] = ACTIONS(1973),
    [sym_netlinx_axi_version] = ACTIONS(1973),
    [sym_url_flg_tcp] = ACTIONS(1973),
    [sym_url_flg_accinfopresent] = ACTIONS(1973),
    [sym_url_flg_temp] = ACTIONS(1973),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1973),
    [sym_url_flg_stat_mask] = ACTIONS(1973),
    [sym_url_flg_stat_lookup] = ACTIONS(1973),
    [sym_url_flg_stat_connecting] = ACTIONS(1973),
    [sym_url_flg_stat_waiting] = ACTIONS(1973),
    [sym_url_flg_stat_connected] = ACTIONS(1973),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1973),
    [sym_first_virtual_device] = ACTIONS(1973),
    [sym_first_local_port] = ACTIONS(1973),
    [sym_dynamic_virtual_device] = ACTIONS(1973),
    [sym_timeline_once] = ACTIONS(1973),
    [sym_timeline_repeat] = ACTIONS(1973),
    [sym_timeline_absolute] = ACTIONS(1973),
    [sym_timeline_relative] = ACTIONS(1973),
    [sym_do_push_timed_infinite] = ACTIONS(1973),
    [sym_source_type_no_address] = ACTIONS(1973),
    [sym_source_type_neuron_id] = ACTIONS(1973),
    [sym_source_type_ip_address] = ACTIONS(1973),
    [sym_source_type_axlink] = ACTIONS(1973),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1973),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1973),
    [sym_source_type_ip_socket_address] = ACTIONS(1973),
    [sym_source_type_rs232] = ACTIONS(1973),
    [sym_source_type_internal] = ACTIONS(1973),
    [sym_source_type_mac_address] = ACTIONS(1973),
    [sym_source_type_ipv4_port] = ACTIONS(1973),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1973),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1973),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1973),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1973),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1973),
    [sym_true] = ACTIONS(1973),
    [sym_false] = ACTIONS(1973),
    [sym_file_read_only] = ACTIONS(1973),
    [sym_file_rw_new] = ACTIONS(1973),
    [sym_file_rw_append] = ACTIONS(1973),
    [sym_ip_tcp] = ACTIONS(1973),
    [sym_ip_udp] = ACTIONS(1973),
    [sym_ip_udp_2way] = ACTIONS(1973),
    [sym_xml_encode_types] = ACTIONS(1973),
    [sym_xml_encode_char_as_list] = ACTIONS(1973),
    [sym_xml_encode_le] = ACTIONS(1973),
    [sym_xml_decode_types] = ACTIONS(1973),
    [sym_xml_decode_no_preserve] = ACTIONS(1973),
    [sym_duet_dev_type_display_device] = ACTIONS(1973),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1973),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1973),
    [sym_duet_dev_type_io_device] = ACTIONS(1973),
    [sym_duet_dev_type_relay_device] = ACTIONS(1973),
    [sym_duet_dev_type_ups] = ACTIONS(1973),
    [sym_duet_dev_type_amplifier] = ACTIONS(1973),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1973),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1973),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1973),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1973),
    [sym_duet_dev_type_camera] = ACTIONS(1973),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1973),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1973),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1973),
    [sym_duet_dev_type_dss] = ACTIONS(1973),
    [sym_duet_dev_type_dvr] = ACTIONS(1973),
    [sym_duet_dev_type_disc_device] = ACTIONS(1973),
    [sym_duet_dev_type_document_camera] = ACTIONS(1973),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1973),
    [sym_duet_dev_type_hvac] = ACTIONS(1973),
    [sym_duet_dev_type_keypad] = ACTIONS(1973),
    [sym_duet_dev_type_light] = ACTIONS(1973),
    [sym_duet_dev_type_monitor] = ACTIONS(1973),
    [sym_duet_dev_type_motor] = ACTIONS(1973),
    [sym_duet_dev_type_multi_window] = ACTIONS(1973),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1973),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1973),
    [sym_duet_dev_type_receiver] = ACTIONS(1973),
    [sym_duet_dev_type_security_system] = ACTIONS(1973),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1973),
    [sym_duet_dev_type_settop_box] = ACTIONS(1973),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1973),
    [sym_duet_dev_type_switcher] = ACTIONS(1973),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1973),
    [sym_duet_dev_type_tv] = ACTIONS(1973),
    [sym_duet_dev_type_utility] = ACTIONS(1973),
    [sym_duet_dev_type_vcr] = ACTIONS(1973),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1973),
    [sym_duet_dev_type_video_processor] = ACTIONS(1973),
    [sym_duet_dev_type_video_projector] = ACTIONS(1973),
    [sym_duet_dev_type_video_wall] = ACTIONS(1973),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1973),
    [sym_duet_dev_type_weather] = ACTIONS(1973),
    [sym_duet_dev_not_polled] = ACTIONS(1973),
    [sym_duet_dev_polled] = ACTIONS(1973),
    [sym_clkmgr_mode_network] = ACTIONS(1973),
    [sym_clkmgr_mode_standalone] = ACTIONS(1973),
    [sym_normal_standby] = ACTIONS(1973),
    [sym_normal_wake] = ACTIONS(1973),
    [sym_ip_multicast_ttl_option] = ACTIONS(1973),
    [sym_ip_ttl_subnet] = ACTIONS(1973),
    [sym_ip_ttl_site] = ACTIONS(1973),
    [sym_ip_ttl_region] = ACTIONS(1973),
    [sym_ip_ttl_continent] = ACTIONS(1973),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1973),
    [sym_ip_nodelay_on] = ACTIONS(1973),
    [sym_ip_nodelay_off] = ACTIONS(1973),
    [sym_amx_error] = ACTIONS(1973),
    [sym_amx_warning] = ACTIONS(1973),
    [sym_amx_info] = ACTIONS(1973),
    [sym_amx_debug] = ACTIONS(1973),
    [sym_netlinx_login_success] = ACTIONS(1973),
    [sym_netlinx_login_fail] = ACTIONS(1973),
    [sym_netlinx_logout] = ACTIONS(1973),
    [sym_null_str] = ACTIONS(1973),
    [sym_smtp_address] = ACTIONS(1973),
    [sym_smtp_port_number] = ACTIONS(1973),
    [sym_smtp_username] = ACTIONS(1973),
    [sym_smtp_password] = ACTIONS(1973),
    [sym_smtp_require_tls] = ACTIONS(1973),
    [sym_smtp_from] = ACTIONS(1973),
    [sym_smtp_tls_true] = ACTIONS(1973),
    [sym_smtp_tls_false] = ACTIONS(1973),
    [sym_listview_on_row_select_event] = ACTIONS(1973),
    [sym_data_structure_unknown] = ACTIONS(1973),
    [sym_data_structure_datafeed] = ACTIONS(1973),
    [sym_data_structure_datarecordset] = ACTIONS(1973),
    [sym_data_structure_datarecord] = ACTIONS(1973),
    [sym_data_structure_datafield] = ACTIONS(1973),
    [sym_data_max_content_field_count] = ACTIONS(1973),
    [sym_data_max_metadata_field_count] = ACTIONS(1973),
    [sym_data_max_id_length] = ACTIONS(1973),
    [sym_data_max_name_length] = ACTIONS(1973),
    [sym_data_max_description_length] = ACTIONS(1973),
    [sym_data_max_type_length] = ACTIONS(1973),
    [sym_data_max_format_length] = ACTIONS(1973),
    [sym_data_max_source_length] = ACTIONS(1973),
    [sym_data_max_value_length] = ACTIONS(1973),
    [sym_data_max_label_length] = ACTIONS(1973),
    [sym_data_max_url_length] = ACTIONS(1973),
    [sym_max_last_login_info_length] = ACTIONS(1973),
    [sym_data_type_unknown] = ACTIONS(1973),
    [sym_data_type_string] = ACTIONS(1973),
    [sym_data_type_datetime] = ACTIONS(1973),
    [sym_data_type_date] = ACTIONS(1973),
    [sym_data_type_time] = ACTIONS(1973),
    [sym_data_type_image] = ACTIONS(1973),
    [sym_data_format_url] = ACTIONS(1973),
    [sym_data_format_phone] = ACTIONS(1973),
    [sym_data_format_email] = ACTIONS(1973),
    [sym_data_format_iso8601] = ACTIONS(1973),
    [sym_auditlog_priv] = ACTIONS(1973),
    [sym_configuration_priv] = ACTIONS(1973),
    [sym_deviceconfig_priv] = ACTIONS(1973),
    [sym_ftp_priv] = ACTIONS(1973),
    [sym_http_priv] = ACTIONS(1973),
    [sym_networkconfig_priv] = ACTIONS(1973),
    [sym_programport_priv] = ACTIONS(1973),
    [sym_remoteui_priv] = ACTIONS(1973),
    [sym_securitycontrol_priv] = ACTIONS(1973),
    [sym_softwaremanagement_priv] = ACTIONS(1973),
    [sym_terminal_priv] = ACTIONS(1973),
    [sym_tpadmin_priv] = ACTIONS(1973),
    [sym_user1_priv] = ACTIONS(1973),
    [sym_user2_priv] = ACTIONS(1973),
    [sym_user3_priv] = ACTIONS(1973),
    [sym_user4_priv] = ACTIONS(1973),
    [sym_usermanagement_priv] = ACTIONS(1973),
    [sym_valid_account] = ACTIONS(1973),
    [sym_err_name_invalid] = ACTIONS(1973),
    [sym_err_password_invalid] = ACTIONS(1973),
    [sym_err_invalid_account] = ACTIONS(1973),
    [sym_err_no_authorization] = ACTIONS(1973),
    [sym_err_invalid_parameter] = ACTIONS(1973),
    [sym_err_user_locked_out] = ACTIONS(1973),
    [sym_err_user_account_expired] = ACTIONS(1973),
    [sym_err_general_error] = ACTIONS(1973),
    [sym_err_authentication_error] = ACTIONS(1973),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1973),
    [sym_tls_validate_certificate] = ACTIONS(1973),
    [sym_tdata] = ACTIONS(1973),
    [sym_tchannel] = ACTIONS(1973),
    [sym_tlevel] = ACTIONS(1973),
    [sym_tbutton] = ACTIONS(1973),
    [sym_ttimeline] = ACTIONS(1973),
    [sym_tcustom] = ACTIONS(1973),
    [sym_url_struct] = ACTIONS(1973),
    [sym_dns_struct] = ACTIONS(1973),
    [sym_ip_address_struct] = ACTIONS(1973),
    [sym_dev_info_struct] = ACTIONS(1973),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1973),
    [sym_data_feed] = ACTIONS(1973),
    [sym_data_field] = ACTIONS(1973),
    [sym_data_record] = ACTIONS(1973),
    [sym_wc_data_feed] = ACTIONS(1973),
    [sym_wc_data_field] = ACTIONS(1973),
    [sym_wc_data_record] = ACTIONS(1973),
    [sym_last_login_info] = ACTIONS(1973),
    [sym____reserved___] = ACTIONS(1973),
    [sym_date] = ACTIONS(1973),
    [sym_day] = ACTIONS(1973),
    [sym_time] = ACTIONS(1973),
    [sym_ldate] = ACTIONS(1973),
    [sym_push_device] = ACTIONS(1973),
    [sym_push_channel] = ACTIONS(1973),
    [sym_push_devchan] = ACTIONS(1973),
    [sym_release_device] = ACTIONS(1973),
    [sym_release_channel] = ACTIONS(1973),
    [sym_release_devchan] = ACTIONS(1973),
    [sym_master_slot] = ACTIONS(1973),
    [sym_get_pulse_time] = ACTIONS(1973),
    [sym_get_timer] = ACTIONS(1973),
    [sym_master_sn] = ACTIONS(1973),
    [sym_system_number] = ACTIONS(1973),
    [sym_button] = ACTIONS(1973),
    [sym_dv_channel] = ACTIONS(1973),
    [sym_data] = ACTIONS(1973),
    [sym_level] = ACTIONS(1973),
    [sym_timeline] = ACTIONS(1973),
    [sym_channel] = ACTIONS(1973),
    [sym_custom] = ACTIONS(1973),
    [sym_atoi] = ACTIONS(1973),
    [sym_atol] = ACTIONS(1973),
    [sym_atof] = ACTIONS(1973),
    [sym_itoa] = ACTIONS(1973),
    [sym_itohex] = ACTIONS(1973),
    [sym_hextoi] = ACTIONS(1973),
    [sym_ftoa] = ACTIONS(1973),
    [sym_find_string] = ACTIONS(1973),
    [sym_left_string] = ACTIONS(1973),
    [sym_right_string] = ACTIONS(1973),
    [sym_mid_string] = ACTIONS(1973),
    [sym_remove_string] = ACTIONS(1973),
    [sym_get_buffer_string] = ACTIONS(1973),
    [sym_get_buffer_char] = ACTIONS(1973),
    [sym_get_multi_buffer_string] = ACTIONS(1973),
    [sym_length_string] = ACTIONS(1973),
    [sym_max_length_string] = ACTIONS(1973),
    [sym_set_length_string] = ACTIONS(1973),
    [sym_compare_string] = ACTIONS(1973),
    [sym_upper_string] = ACTIONS(1973),
    [sym_lower_string] = ACTIONS(1973),
    [sym_get_last] = ACTIONS(1973),
    [sym_date_to_day] = ACTIONS(1973),
    [sym_date_to_month] = ACTIONS(1973),
    [sym_date_to_year] = ACTIONS(1973),
    [sym_time_to_hour] = ACTIONS(1973),
    [sym_time_to_minute] = ACTIONS(1973),
    [sym_time_to_second] = ACTIONS(1973),
    [sym_day_of_week] = ACTIONS(1973),
    [sym_set_pulse_time] = ACTIONS(1973),
    [sym_set_timer] = ACTIONS(1973),
    [sym_file_open] = ACTIONS(1973),
    [sym_file_close] = ACTIONS(1973),
    [sym_file_read] = ACTIONS(1973),
    [sym_file_read_line] = ACTIONS(1973),
    [sym_file_write] = ACTIONS(1973),
    [sym_file_write_line] = ACTIONS(1973),
    [sym_file_copy] = ACTIONS(1973),
    [sym_file_delete] = ACTIONS(1973),
    [sym_file_rename] = ACTIONS(1973),
    [sym_file_dir] = ACTIONS(1973),
    [sym_file_getdir] = ACTIONS(1973),
    [sym_file_setdir] = ACTIONS(1973),
    [sym_file_create_dir] = ACTIONS(1973),
    [sym_file_remove_dir] = ACTIONS(1973),
    [sym_file_seek] = ACTIONS(1973),
    [sym_length_array] = ACTIONS(1973),
    [sym_max_length_array] = ACTIONS(1973),
    [sym_set_length_array] = ACTIONS(1973),
    [sym_ip_client_open] = ACTIONS(1973),
    [sym_ip_client_close] = ACTIONS(1973),
    [sym_ip_server_open] = ACTIONS(1973),
    [sym_ip_server_close] = ACTIONS(1973),
    [sym_ip_bound_client_open] = ACTIONS(1973),
    [sym_ip_mc_server_open] = ACTIONS(1973),
    [sym_ip_set_option] = ACTIONS(1973),
    [sym_ssh_client_open] = ACTIONS(1973),
    [sym_ssh_client_close] = ACTIONS(1973),
    [sym_tls_client_open] = ACTIONS(1973),
    [sym_tls_client_close] = ACTIONS(1973),
    [sym_do_push] = ACTIONS(1973),
    [sym_do_push_timed] = ACTIONS(1973),
    [sym_do_release] = ACTIONS(1973),
    [sym_device_id] = ACTIONS(1973),
    [sym_device_info] = ACTIONS(1973),
    [sym_device_id_string] = ACTIONS(1973),
    [sym_device_standby] = ACTIONS(1973),
    [sym_device_wake] = ACTIONS(1973),
    [sym_timeline_create] = ACTIONS(1973),
    [sym_timeline_kill] = ACTIONS(1973),
    [sym_timeline_reload] = ACTIONS(1973),
    [sym_timeline_pause] = ACTIONS(1973),
    [sym_timeline_restart] = ACTIONS(1973),
    [sym_timeline_set] = ACTIONS(1973),
    [sym_timeline_get] = ACTIONS(1973),
    [sym_timeline_active] = ACTIONS(1973),
    [sym_timeline_dynamic_id] = ACTIONS(1973),
    [sym_variable_to_string] = ACTIONS(1973),
    [sym_string_to_variable] = ACTIONS(1973),
    [sym_length_variable_to_string] = ACTIONS(1973),
    [sym_type_cast] = ACTIONS(1973),
    [sym_variable_to_xml] = ACTIONS(1973),
    [sym_xml_to_variable] = ACTIONS(1973),
    [sym_length_variable_to_xml] = ACTIONS(1973),
    [sym_redirect_string] = ACTIONS(1973),
    [sym_random_number] = ACTIONS(1973),
    [sym_get_unique_id] = ACTIONS(1973),
    [sym_get_system_number] = ACTIONS(1973),
    [sym_set_system_number] = ACTIONS(1973),
    [sym_get_serial_number] = ACTIONS(1973),
    [sym_get_master_build] = ACTIONS(1973),
    [sym_set_validation_code] = ACTIONS(1973),
    [sym_reboot] = ACTIONS(1973),
    [sym_load_duet_module] = ACTIONS(1973),
    [sym_unload_duet_module] = ACTIONS(1973),
    [sym_duet_mem_size_set] = ACTIONS(1973),
    [sym_duet_mem_size_get] = ACTIONS(1973),
    [sym_dynamic_application_device] = ACTIONS(1973),
    [sym_dynamic_polled_port] = ACTIONS(1973),
    [sym_static_port_binding] = ACTIONS(1973),
    [sym_static_ip_binding] = ACTIONS(1973),
    [sym_get_dns_list] = ACTIONS(1973),
    [sym_set_dns_list] = ACTIONS(1973),
    [sym_get_ip_address] = ACTIONS(1973),
    [sym_set_ip_address] = ACTIONS(1973),
    [sym_get_url_list] = ACTIONS(1973),
    [sym_add_url_entry] = ACTIONS(1973),
    [sym_delete_url_entry] = ACTIONS(1973),
    [sym_abs_value] = ACTIONS(1973),
    [sym_max_value] = ACTIONS(1973),
    [sym_min_value] = ACTIONS(1973),
    [sym_exp_value] = ACTIONS(1973),
    [sym_log_value] = ACTIONS(1973),
    [sym_log10_value] = ACTIONS(1973),
    [sym_power_value] = ACTIONS(1973),
    [sym_sqrt_value] = ACTIONS(1973),
    [sym_raw_be] = ACTIONS(1973),
    [sym_raw_le] = ACTIONS(1973),
    [sym_internal_queue_size_set] = ACTIONS(1973),
    [sym_internal_queue_size_get] = ACTIONS(1973),
    [sym_internal_threshold_set] = ACTIONS(1973),
    [sym_internal_threshold_get] = ACTIONS(1973),
    [sym_rebuilt_event] = ACTIONS(1973),
    [sym_do_custom_event] = ACTIONS(1973),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1973),
    [sym_clkmgr_set_clk_source] = ACTIONS(1973),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1973),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1973),
    [sym_clkmgr_get_timezone] = ACTIONS(1973),
    [sym_clkmgr_set_timezone] = ACTIONS(1973),
    [sym_clkmgr_get_resync_period] = ACTIONS(1973),
    [sym_clkmgr_set_resync_period] = ACTIONS(1973),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1973),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1973),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1973),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1973),
    [sym_clkmgr_get_timeservers] = ACTIONS(1973),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1973),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1973),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1973),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1973),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1973),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1973),
    [sym_set_log_level] = ACTIONS(1973),
    [sym_get_log_level] = ACTIONS(1973),
    [sym_amx_log] = ACTIONS(1973),
    [sym_smtp_server_config_set] = ACTIONS(1973),
    [sym_smtp_server_config_get] = ACTIONS(1973),
    [sym_smtp_send] = ACTIONS(1973),
    [sym_validate_netlinx_account] = ACTIONS(1973),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1973),
    [sym_authenticate_certificate] = ACTIONS(1973),
    [sym_audit_netlinx_session_event] = ACTIONS(1973),
    [sym_audit_netlinx_generic_event] = ACTIONS(1973),
    [sym_get_available_flash_disk_space] = ACTIONS(1973),
    [sym_get_max_flash_disk_space] = ACTIONS(1973),
    [sym_data_create_feed] = ACTIONS(1973),
    [sym_data_delete_feed] = ACTIONS(1973),
    [sym_data_publish_feed] = ACTIONS(1973),
    [sym_data_get_published_feed] = ACTIONS(1973),
    [sym_data_add_record] = ACTIONS(1973),
    [sym_data_get_event_record] = ACTIONS(1973),
    [sym__wc_data_create_feed] = ACTIONS(1973),
    [sym__wc_data_add_record] = ACTIONS(1973),
    [sym__wc_data_get_event_record] = ACTIONS(1973),
    [sym_format] = ACTIONS(1973),
    [sym_astro_clock] = ACTIONS(1973),
    [sym_set_outdoor_temperature] = ACTIONS(1973),
    [sym_set_virtual_level_count] = ACTIONS(1973),
    [sym_set_virtual_channel_count] = ACTIONS(1973),
    [sym_set_virtual_port_count] = ACTIONS(1973),
    [sym__wc_encode_int] = ACTIONS(1973),
    [sym__wc_decode_int] = ACTIONS(1973),
    [sym__wc_int] = ACTIONS(1973),
    [sym__wc_to_ch_int] = ACTIONS(1973),
    [sym__ch_to_wc_int] = ACTIONS(1973),
    [sym__wc_find_string_int] = ACTIONS(1973),
    [sym__wc_left_string_int] = ACTIONS(1973),
    [sym__wc_lower_string_int] = ACTIONS(1973),
    [sym__wc_mid_string_int] = ACTIONS(1973),
    [sym__wc_remove_string_int] = ACTIONS(1973),
    [sym__wc_right_string_int] = ACTIONS(1973),
    [sym__wc_upper_string_int] = ACTIONS(1973),
    [sym__wc_compare_string_int] = ACTIONS(1973),
    [sym__wc_get_buffer_char_int] = ACTIONS(1973),
    [sym__wc_get_buffer_string_int] = ACTIONS(1973),
    [sym__wc_concat_string_int] = ACTIONS(1973),
    [sym__wc_file_open_int] = ACTIONS(1973),
    [sym__wc_file_close_int] = ACTIONS(1973),
    [sym__wc_file_read_int] = ACTIONS(1973),
    [sym__wc_file_read_line_int] = ACTIONS(1973),
    [sym__wc_file_write_int] = ACTIONS(1973),
    [sym__wc_file_write_line_int] = ACTIONS(1973),
    [aux_sym_preproc_include_token1] = ACTIONS(1973),
    [aux_sym_preproc_define_token1] = ACTIONS(1975),
    [aux_sym_preproc_warn_token1] = ACTIONS(1975),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1975),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1975),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1975),
    [anon_sym_LBRACK] = ACTIONS(1975),
    [anon_sym_LBRACE] = ACTIONS(1975),
    [anon_sym_LPAREN] = ACTIONS(1973),
    [anon_sym_BANG] = ACTIONS(1975),
    [anon_sym_TILDE] = ACTIONS(1975),
    [anon_sym_DASH] = ACTIONS(1973),
    [anon_sym_PLUS] = ACTIONS(1973),
    [anon_sym_DASH_DASH] = ACTIONS(1975),
    [anon_sym_PLUS_PLUS] = ACTIONS(1975),
    [anon_sym_DQUOTE] = ACTIONS(1975),
    [anon_sym_SQUOTE] = ACTIONS(1975),
    [sym_number_literal] = ACTIONS(1975),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(402)] = {
    [ts_builtin_sym_end] = ACTIONS(1979),
    [sym_identifier] = ACTIONS(1977),
    [sym_program_name_keyword] = ACTIONS(1977),
    [sym_module_name_keyword] = ACTIONS(1977),
    [sym_define_device_keyword] = ACTIONS(1977),
    [sym_define_combine_keyword] = ACTIONS(1977),
    [sym_define_constant_keyword] = ACTIONS(1977),
    [sym_define_type_keyword] = ACTIONS(1977),
    [sym_define_variable_keyword] = ACTIONS(1977),
    [sym_define_system_variable_keyword] = ACTIONS(1977),
    [sym_define_connect_level_keyword] = ACTIONS(1977),
    [sym_define_latching_keyword] = ACTIONS(1977),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1977),
    [sym_define_toggling_keyword] = ACTIONS(1977),
    [sym_define_call_keyword] = ACTIONS(1977),
    [sym_define_function_keyword] = ACTIONS(1977),
    [sym_define_library_function_keyword] = ACTIONS(1977),
    [sym_define_start_keyword] = ACTIONS(1977),
    [sym_define_event_keyword] = ACTIONS(1977),
    [sym_define_program_keyword] = ACTIONS(1977),
    [sym_define_module_keyword] = ACTIONS(1977),
    [sym_char_keyword] = ACTIONS(1977),
    [sym_widechar_keyword] = ACTIONS(1977),
    [sym_integer_keyword] = ACTIONS(1977),
    [sym_sinteger_keyword] = ACTIONS(1977),
    [sym_long_keyword] = ACTIONS(1977),
    [sym_slong_keyword] = ACTIONS(1977),
    [sym_float_keyword] = ACTIONS(1977),
    [sym_double_keyword] = ACTIONS(1977),
    [sym_variant_keyword] = ACTIONS(1977),
    [sym_variantarray_keyword] = ACTIONS(1977),
    [sym_local_var_keyword] = ACTIONS(1977),
    [sym_stack_var_keyword] = ACTIONS(1977),
    [sym_constant_keyword] = ACTIONS(1977),
    [sym_volatile_keyword] = ACTIONS(1977),
    [sym_non_volatile_keyword] = ACTIONS(1977),
    [sym_persistent_keyword] = ACTIONS(1977),
    [sym_dev_keyword] = ACTIONS(1977),
    [sym_devlev_keyword] = ACTIONS(1977),
    [sym_devchan_keyword] = ACTIONS(1977),
    [sym_if_keyword] = ACTIONS(1977),
    [sym_switch_keyword] = ACTIONS(1977),
    [sym_case_keyword] = ACTIONS(1977),
    [sym_default_keyword] = ACTIONS(1977),
    [sym_while_keyword] = ACTIONS(1977),
    [sym_for_keyword] = ACTIONS(1977),
    [sym_break_keyword] = ACTIONS(1977),
    [sym_continue_keyword] = ACTIONS(1977),
    [sym_return_keyword] = ACTIONS(1977),
    [sym_select_keyword] = ACTIONS(1977),
    [sym_active_keyword] = ACTIONS(1977),
    [sym_struct_keyword] = ACTIONS(1977),
    [sym_structure_keyword] = ACTIONS(1977),
    [sym_button_event_keyword] = ACTIONS(1977),
    [sym_channel_event_keyword] = ACTIONS(1977),
    [sym_level_event_keyword] = ACTIONS(1977),
    [sym_data_event_keyword] = ACTIONS(1977),
    [sym_timeline_event_keyword] = ACTIONS(1977),
    [sym_custom_event_keyword] = ACTIONS(1977),
    [sym_push_keyword] = ACTIONS(1977),
    [sym_release_keyword] = ACTIONS(1977),
    [aux_sym_on_keyword_token1] = ACTIONS(1977),
    [aux_sym_off_keyword_token1] = ACTIONS(1977),
    [sym_send_level_keyword] = ACTIONS(1977),
    [sym_send_string_keyword] = ACTIONS(1977),
    [sym_send_command_keyword] = ACTIONS(1977),
    [sym_clear_buffer_keyword] = ACTIONS(1977),
    [sym_create_buffer_keyword] = ACTIONS(1977),
    [sym_create_multi_buffer_keyword] = ACTIONS(1977),
    [sym_call_keyword] = ACTIONS(1977),
    [sym_system_call_keyword] = ACTIONS(1977),
    [sym_devchan_to_keyword] = ACTIONS(1977),
    [sym_devchan_min_to_keyword] = ACTIONS(1977),
    [sym_devchan_total_off_keyword] = ACTIONS(1977),
    [sym_devchan_pulse_keyword] = ACTIONS(1977),
    [sym_wait_keyword] = ACTIONS(1977),
    [sym_cancel_wait_keyword] = ACTIONS(1977),
    [sym_wait_until_keyword] = ACTIONS(1977),
    [sym_cancel_wait_until_keyword] = ACTIONS(1977),
    [sym_cancel_all_wait_keyword] = ACTIONS(1977),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1977),
    [sym_not] = ACTIONS(1977),
    [sym_bnot] = ACTIONS(1977),
    [sym___file__] = ACTIONS(1977),
    [sym___date__] = ACTIONS(1977),
    [sym___ldate__] = ACTIONS(1977),
    [sym___time__] = ACTIONS(1977),
    [sym___line__] = ACTIONS(1977),
    [sym___name__] = ACTIONS(1977),
    [sym___netlinx__] = ACTIONS(1977),
    [sym_netlinx_axi_version] = ACTIONS(1977),
    [sym_url_flg_tcp] = ACTIONS(1977),
    [sym_url_flg_accinfopresent] = ACTIONS(1977),
    [sym_url_flg_temp] = ACTIONS(1977),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1977),
    [sym_url_flg_stat_mask] = ACTIONS(1977),
    [sym_url_flg_stat_lookup] = ACTIONS(1977),
    [sym_url_flg_stat_connecting] = ACTIONS(1977),
    [sym_url_flg_stat_waiting] = ACTIONS(1977),
    [sym_url_flg_stat_connected] = ACTIONS(1977),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1977),
    [sym_first_virtual_device] = ACTIONS(1977),
    [sym_first_local_port] = ACTIONS(1977),
    [sym_dynamic_virtual_device] = ACTIONS(1977),
    [sym_timeline_once] = ACTIONS(1977),
    [sym_timeline_repeat] = ACTIONS(1977),
    [sym_timeline_absolute] = ACTIONS(1977),
    [sym_timeline_relative] = ACTIONS(1977),
    [sym_do_push_timed_infinite] = ACTIONS(1977),
    [sym_source_type_no_address] = ACTIONS(1977),
    [sym_source_type_neuron_id] = ACTIONS(1977),
    [sym_source_type_ip_address] = ACTIONS(1977),
    [sym_source_type_axlink] = ACTIONS(1977),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1977),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1977),
    [sym_source_type_ip_socket_address] = ACTIONS(1977),
    [sym_source_type_rs232] = ACTIONS(1977),
    [sym_source_type_internal] = ACTIONS(1977),
    [sym_source_type_mac_address] = ACTIONS(1977),
    [sym_source_type_ipv4_port] = ACTIONS(1977),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1977),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1977),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1977),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1977),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1977),
    [sym_true] = ACTIONS(1977),
    [sym_false] = ACTIONS(1977),
    [sym_file_read_only] = ACTIONS(1977),
    [sym_file_rw_new] = ACTIONS(1977),
    [sym_file_rw_append] = ACTIONS(1977),
    [sym_ip_tcp] = ACTIONS(1977),
    [sym_ip_udp] = ACTIONS(1977),
    [sym_ip_udp_2way] = ACTIONS(1977),
    [sym_xml_encode_types] = ACTIONS(1977),
    [sym_xml_encode_char_as_list] = ACTIONS(1977),
    [sym_xml_encode_le] = ACTIONS(1977),
    [sym_xml_decode_types] = ACTIONS(1977),
    [sym_xml_decode_no_preserve] = ACTIONS(1977),
    [sym_duet_dev_type_display_device] = ACTIONS(1977),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1977),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1977),
    [sym_duet_dev_type_io_device] = ACTIONS(1977),
    [sym_duet_dev_type_relay_device] = ACTIONS(1977),
    [sym_duet_dev_type_ups] = ACTIONS(1977),
    [sym_duet_dev_type_amplifier] = ACTIONS(1977),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1977),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1977),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1977),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1977),
    [sym_duet_dev_type_camera] = ACTIONS(1977),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1977),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1977),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1977),
    [sym_duet_dev_type_dss] = ACTIONS(1977),
    [sym_duet_dev_type_dvr] = ACTIONS(1977),
    [sym_duet_dev_type_disc_device] = ACTIONS(1977),
    [sym_duet_dev_type_document_camera] = ACTIONS(1977),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1977),
    [sym_duet_dev_type_hvac] = ACTIONS(1977),
    [sym_duet_dev_type_keypad] = ACTIONS(1977),
    [sym_duet_dev_type_light] = ACTIONS(1977),
    [sym_duet_dev_type_monitor] = ACTIONS(1977),
    [sym_duet_dev_type_motor] = ACTIONS(1977),
    [sym_duet_dev_type_multi_window] = ACTIONS(1977),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1977),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1977),
    [sym_duet_dev_type_receiver] = ACTIONS(1977),
    [sym_duet_dev_type_security_system] = ACTIONS(1977),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1977),
    [sym_duet_dev_type_settop_box] = ACTIONS(1977),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1977),
    [sym_duet_dev_type_switcher] = ACTIONS(1977),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1977),
    [sym_duet_dev_type_tv] = ACTIONS(1977),
    [sym_duet_dev_type_utility] = ACTIONS(1977),
    [sym_duet_dev_type_vcr] = ACTIONS(1977),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1977),
    [sym_duet_dev_type_video_processor] = ACTIONS(1977),
    [sym_duet_dev_type_video_projector] = ACTIONS(1977),
    [sym_duet_dev_type_video_wall] = ACTIONS(1977),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1977),
    [sym_duet_dev_type_weather] = ACTIONS(1977),
    [sym_duet_dev_not_polled] = ACTIONS(1977),
    [sym_duet_dev_polled] = ACTIONS(1977),
    [sym_clkmgr_mode_network] = ACTIONS(1977),
    [sym_clkmgr_mode_standalone] = ACTIONS(1977),
    [sym_normal_standby] = ACTIONS(1977),
    [sym_normal_wake] = ACTIONS(1977),
    [sym_ip_multicast_ttl_option] = ACTIONS(1977),
    [sym_ip_ttl_subnet] = ACTIONS(1977),
    [sym_ip_ttl_site] = ACTIONS(1977),
    [sym_ip_ttl_region] = ACTIONS(1977),
    [sym_ip_ttl_continent] = ACTIONS(1977),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1977),
    [sym_ip_nodelay_on] = ACTIONS(1977),
    [sym_ip_nodelay_off] = ACTIONS(1977),
    [sym_amx_error] = ACTIONS(1977),
    [sym_amx_warning] = ACTIONS(1977),
    [sym_amx_info] = ACTIONS(1977),
    [sym_amx_debug] = ACTIONS(1977),
    [sym_netlinx_login_success] = ACTIONS(1977),
    [sym_netlinx_login_fail] = ACTIONS(1977),
    [sym_netlinx_logout] = ACTIONS(1977),
    [sym_null_str] = ACTIONS(1977),
    [sym_smtp_address] = ACTIONS(1977),
    [sym_smtp_port_number] = ACTIONS(1977),
    [sym_smtp_username] = ACTIONS(1977),
    [sym_smtp_password] = ACTIONS(1977),
    [sym_smtp_require_tls] = ACTIONS(1977),
    [sym_smtp_from] = ACTIONS(1977),
    [sym_smtp_tls_true] = ACTIONS(1977),
    [sym_smtp_tls_false] = ACTIONS(1977),
    [sym_listview_on_row_select_event] = ACTIONS(1977),
    [sym_data_structure_unknown] = ACTIONS(1977),
    [sym_data_structure_datafeed] = ACTIONS(1977),
    [sym_data_structure_datarecordset] = ACTIONS(1977),
    [sym_data_structure_datarecord] = ACTIONS(1977),
    [sym_data_structure_datafield] = ACTIONS(1977),
    [sym_data_max_content_field_count] = ACTIONS(1977),
    [sym_data_max_metadata_field_count] = ACTIONS(1977),
    [sym_data_max_id_length] = ACTIONS(1977),
    [sym_data_max_name_length] = ACTIONS(1977),
    [sym_data_max_description_length] = ACTIONS(1977),
    [sym_data_max_type_length] = ACTIONS(1977),
    [sym_data_max_format_length] = ACTIONS(1977),
    [sym_data_max_source_length] = ACTIONS(1977),
    [sym_data_max_value_length] = ACTIONS(1977),
    [sym_data_max_label_length] = ACTIONS(1977),
    [sym_data_max_url_length] = ACTIONS(1977),
    [sym_max_last_login_info_length] = ACTIONS(1977),
    [sym_data_type_unknown] = ACTIONS(1977),
    [sym_data_type_string] = ACTIONS(1977),
    [sym_data_type_datetime] = ACTIONS(1977),
    [sym_data_type_date] = ACTIONS(1977),
    [sym_data_type_time] = ACTIONS(1977),
    [sym_data_type_image] = ACTIONS(1977),
    [sym_data_format_url] = ACTIONS(1977),
    [sym_data_format_phone] = ACTIONS(1977),
    [sym_data_format_email] = ACTIONS(1977),
    [sym_data_format_iso8601] = ACTIONS(1977),
    [sym_auditlog_priv] = ACTIONS(1977),
    [sym_configuration_priv] = ACTIONS(1977),
    [sym_deviceconfig_priv] = ACTIONS(1977),
    [sym_ftp_priv] = ACTIONS(1977),
    [sym_http_priv] = ACTIONS(1977),
    [sym_networkconfig_priv] = ACTIONS(1977),
    [sym_programport_priv] = ACTIONS(1977),
    [sym_remoteui_priv] = ACTIONS(1977),
    [sym_securitycontrol_priv] = ACTIONS(1977),
    [sym_softwaremanagement_priv] = ACTIONS(1977),
    [sym_terminal_priv] = ACTIONS(1977),
    [sym_tpadmin_priv] = ACTIONS(1977),
    [sym_user1_priv] = ACTIONS(1977),
    [sym_user2_priv] = ACTIONS(1977),
    [sym_user3_priv] = ACTIONS(1977),
    [sym_user4_priv] = ACTIONS(1977),
    [sym_usermanagement_priv] = ACTIONS(1977),
    [sym_valid_account] = ACTIONS(1977),
    [sym_err_name_invalid] = ACTIONS(1977),
    [sym_err_password_invalid] = ACTIONS(1977),
    [sym_err_invalid_account] = ACTIONS(1977),
    [sym_err_no_authorization] = ACTIONS(1977),
    [sym_err_invalid_parameter] = ACTIONS(1977),
    [sym_err_user_locked_out] = ACTIONS(1977),
    [sym_err_user_account_expired] = ACTIONS(1977),
    [sym_err_general_error] = ACTIONS(1977),
    [sym_err_authentication_error] = ACTIONS(1977),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1977),
    [sym_tls_validate_certificate] = ACTIONS(1977),
    [sym_tdata] = ACTIONS(1977),
    [sym_tchannel] = ACTIONS(1977),
    [sym_tlevel] = ACTIONS(1977),
    [sym_tbutton] = ACTIONS(1977),
    [sym_ttimeline] = ACTIONS(1977),
    [sym_tcustom] = ACTIONS(1977),
    [sym_url_struct] = ACTIONS(1977),
    [sym_dns_struct] = ACTIONS(1977),
    [sym_ip_address_struct] = ACTIONS(1977),
    [sym_dev_info_struct] = ACTIONS(1977),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1977),
    [sym_data_feed] = ACTIONS(1977),
    [sym_data_field] = ACTIONS(1977),
    [sym_data_record] = ACTIONS(1977),
    [sym_wc_data_feed] = ACTIONS(1977),
    [sym_wc_data_field] = ACTIONS(1977),
    [sym_wc_data_record] = ACTIONS(1977),
    [sym_last_login_info] = ACTIONS(1977),
    [sym____reserved___] = ACTIONS(1977),
    [sym_date] = ACTIONS(1977),
    [sym_day] = ACTIONS(1977),
    [sym_time] = ACTIONS(1977),
    [sym_ldate] = ACTIONS(1977),
    [sym_push_device] = ACTIONS(1977),
    [sym_push_channel] = ACTIONS(1977),
    [sym_push_devchan] = ACTIONS(1977),
    [sym_release_device] = ACTIONS(1977),
    [sym_release_channel] = ACTIONS(1977),
    [sym_release_devchan] = ACTIONS(1977),
    [sym_master_slot] = ACTIONS(1977),
    [sym_get_pulse_time] = ACTIONS(1977),
    [sym_get_timer] = ACTIONS(1977),
    [sym_master_sn] = ACTIONS(1977),
    [sym_system_number] = ACTIONS(1977),
    [sym_button] = ACTIONS(1977),
    [sym_dv_channel] = ACTIONS(1977),
    [sym_data] = ACTIONS(1977),
    [sym_level] = ACTIONS(1977),
    [sym_timeline] = ACTIONS(1977),
    [sym_channel] = ACTIONS(1977),
    [sym_custom] = ACTIONS(1977),
    [sym_atoi] = ACTIONS(1977),
    [sym_atol] = ACTIONS(1977),
    [sym_atof] = ACTIONS(1977),
    [sym_itoa] = ACTIONS(1977),
    [sym_itohex] = ACTIONS(1977),
    [sym_hextoi] = ACTIONS(1977),
    [sym_ftoa] = ACTIONS(1977),
    [sym_find_string] = ACTIONS(1977),
    [sym_left_string] = ACTIONS(1977),
    [sym_right_string] = ACTIONS(1977),
    [sym_mid_string] = ACTIONS(1977),
    [sym_remove_string] = ACTIONS(1977),
    [sym_get_buffer_string] = ACTIONS(1977),
    [sym_get_buffer_char] = ACTIONS(1977),
    [sym_get_multi_buffer_string] = ACTIONS(1977),
    [sym_length_string] = ACTIONS(1977),
    [sym_max_length_string] = ACTIONS(1977),
    [sym_set_length_string] = ACTIONS(1977),
    [sym_compare_string] = ACTIONS(1977),
    [sym_upper_string] = ACTIONS(1977),
    [sym_lower_string] = ACTIONS(1977),
    [sym_get_last] = ACTIONS(1977),
    [sym_date_to_day] = ACTIONS(1977),
    [sym_date_to_month] = ACTIONS(1977),
    [sym_date_to_year] = ACTIONS(1977),
    [sym_time_to_hour] = ACTIONS(1977),
    [sym_time_to_minute] = ACTIONS(1977),
    [sym_time_to_second] = ACTIONS(1977),
    [sym_day_of_week] = ACTIONS(1977),
    [sym_set_pulse_time] = ACTIONS(1977),
    [sym_set_timer] = ACTIONS(1977),
    [sym_file_open] = ACTIONS(1977),
    [sym_file_close] = ACTIONS(1977),
    [sym_file_read] = ACTIONS(1977),
    [sym_file_read_line] = ACTIONS(1977),
    [sym_file_write] = ACTIONS(1977),
    [sym_file_write_line] = ACTIONS(1977),
    [sym_file_copy] = ACTIONS(1977),
    [sym_file_delete] = ACTIONS(1977),
    [sym_file_rename] = ACTIONS(1977),
    [sym_file_dir] = ACTIONS(1977),
    [sym_file_getdir] = ACTIONS(1977),
    [sym_file_setdir] = ACTIONS(1977),
    [sym_file_create_dir] = ACTIONS(1977),
    [sym_file_remove_dir] = ACTIONS(1977),
    [sym_file_seek] = ACTIONS(1977),
    [sym_length_array] = ACTIONS(1977),
    [sym_max_length_array] = ACTIONS(1977),
    [sym_set_length_array] = ACTIONS(1977),
    [sym_ip_client_open] = ACTIONS(1977),
    [sym_ip_client_close] = ACTIONS(1977),
    [sym_ip_server_open] = ACTIONS(1977),
    [sym_ip_server_close] = ACTIONS(1977),
    [sym_ip_bound_client_open] = ACTIONS(1977),
    [sym_ip_mc_server_open] = ACTIONS(1977),
    [sym_ip_set_option] = ACTIONS(1977),
    [sym_ssh_client_open] = ACTIONS(1977),
    [sym_ssh_client_close] = ACTIONS(1977),
    [sym_tls_client_open] = ACTIONS(1977),
    [sym_tls_client_close] = ACTIONS(1977),
    [sym_do_push] = ACTIONS(1977),
    [sym_do_push_timed] = ACTIONS(1977),
    [sym_do_release] = ACTIONS(1977),
    [sym_device_id] = ACTIONS(1977),
    [sym_device_info] = ACTIONS(1977),
    [sym_device_id_string] = ACTIONS(1977),
    [sym_device_standby] = ACTIONS(1977),
    [sym_device_wake] = ACTIONS(1977),
    [sym_timeline_create] = ACTIONS(1977),
    [sym_timeline_kill] = ACTIONS(1977),
    [sym_timeline_reload] = ACTIONS(1977),
    [sym_timeline_pause] = ACTIONS(1977),
    [sym_timeline_restart] = ACTIONS(1977),
    [sym_timeline_set] = ACTIONS(1977),
    [sym_timeline_get] = ACTIONS(1977),
    [sym_timeline_active] = ACTIONS(1977),
    [sym_timeline_dynamic_id] = ACTIONS(1977),
    [sym_variable_to_string] = ACTIONS(1977),
    [sym_string_to_variable] = ACTIONS(1977),
    [sym_length_variable_to_string] = ACTIONS(1977),
    [sym_type_cast] = ACTIONS(1977),
    [sym_variable_to_xml] = ACTIONS(1977),
    [sym_xml_to_variable] = ACTIONS(1977),
    [sym_length_variable_to_xml] = ACTIONS(1977),
    [sym_redirect_string] = ACTIONS(1977),
    [sym_random_number] = ACTIONS(1977),
    [sym_get_unique_id] = ACTIONS(1977),
    [sym_get_system_number] = ACTIONS(1977),
    [sym_set_system_number] = ACTIONS(1977),
    [sym_get_serial_number] = ACTIONS(1977),
    [sym_get_master_build] = ACTIONS(1977),
    [sym_set_validation_code] = ACTIONS(1977),
    [sym_reboot] = ACTIONS(1977),
    [sym_load_duet_module] = ACTIONS(1977),
    [sym_unload_duet_module] = ACTIONS(1977),
    [sym_duet_mem_size_set] = ACTIONS(1977),
    [sym_duet_mem_size_get] = ACTIONS(1977),
    [sym_dynamic_application_device] = ACTIONS(1977),
    [sym_dynamic_polled_port] = ACTIONS(1977),
    [sym_static_port_binding] = ACTIONS(1977),
    [sym_static_ip_binding] = ACTIONS(1977),
    [sym_get_dns_list] = ACTIONS(1977),
    [sym_set_dns_list] = ACTIONS(1977),
    [sym_get_ip_address] = ACTIONS(1977),
    [sym_set_ip_address] = ACTIONS(1977),
    [sym_get_url_list] = ACTIONS(1977),
    [sym_add_url_entry] = ACTIONS(1977),
    [sym_delete_url_entry] = ACTIONS(1977),
    [sym_abs_value] = ACTIONS(1977),
    [sym_max_value] = ACTIONS(1977),
    [sym_min_value] = ACTIONS(1977),
    [sym_exp_value] = ACTIONS(1977),
    [sym_log_value] = ACTIONS(1977),
    [sym_log10_value] = ACTIONS(1977),
    [sym_power_value] = ACTIONS(1977),
    [sym_sqrt_value] = ACTIONS(1977),
    [sym_raw_be] = ACTIONS(1977),
    [sym_raw_le] = ACTIONS(1977),
    [sym_internal_queue_size_set] = ACTIONS(1977),
    [sym_internal_queue_size_get] = ACTIONS(1977),
    [sym_internal_threshold_set] = ACTIONS(1977),
    [sym_internal_threshold_get] = ACTIONS(1977),
    [sym_rebuilt_event] = ACTIONS(1977),
    [sym_do_custom_event] = ACTIONS(1977),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1977),
    [sym_clkmgr_set_clk_source] = ACTIONS(1977),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1977),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1977),
    [sym_clkmgr_get_timezone] = ACTIONS(1977),
    [sym_clkmgr_set_timezone] = ACTIONS(1977),
    [sym_clkmgr_get_resync_period] = ACTIONS(1977),
    [sym_clkmgr_set_resync_period] = ACTIONS(1977),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1977),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1977),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1977),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1977),
    [sym_clkmgr_get_timeservers] = ACTIONS(1977),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1977),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1977),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1977),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1977),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1977),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1977),
    [sym_set_log_level] = ACTIONS(1977),
    [sym_get_log_level] = ACTIONS(1977),
    [sym_amx_log] = ACTIONS(1977),
    [sym_smtp_server_config_set] = ACTIONS(1977),
    [sym_smtp_server_config_get] = ACTIONS(1977),
    [sym_smtp_send] = ACTIONS(1977),
    [sym_validate_netlinx_account] = ACTIONS(1977),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1977),
    [sym_authenticate_certificate] = ACTIONS(1977),
    [sym_audit_netlinx_session_event] = ACTIONS(1977),
    [sym_audit_netlinx_generic_event] = ACTIONS(1977),
    [sym_get_available_flash_disk_space] = ACTIONS(1977),
    [sym_get_max_flash_disk_space] = ACTIONS(1977),
    [sym_data_create_feed] = ACTIONS(1977),
    [sym_data_delete_feed] = ACTIONS(1977),
    [sym_data_publish_feed] = ACTIONS(1977),
    [sym_data_get_published_feed] = ACTIONS(1977),
    [sym_data_add_record] = ACTIONS(1977),
    [sym_data_get_event_record] = ACTIONS(1977),
    [sym__wc_data_create_feed] = ACTIONS(1977),
    [sym__wc_data_add_record] = ACTIONS(1977),
    [sym__wc_data_get_event_record] = ACTIONS(1977),
    [sym_format] = ACTIONS(1977),
    [sym_astro_clock] = ACTIONS(1977),
    [sym_set_outdoor_temperature] = ACTIONS(1977),
    [sym_set_virtual_level_count] = ACTIONS(1977),
    [sym_set_virtual_channel_count] = ACTIONS(1977),
    [sym_set_virtual_port_count] = ACTIONS(1977),
    [sym__wc_encode_int] = ACTIONS(1977),
    [sym__wc_decode_int] = ACTIONS(1977),
    [sym__wc_int] = ACTIONS(1977),
    [sym__wc_to_ch_int] = ACTIONS(1977),
    [sym__ch_to_wc_int] = ACTIONS(1977),
    [sym__wc_find_string_int] = ACTIONS(1977),
    [sym__wc_left_string_int] = ACTIONS(1977),
    [sym__wc_lower_string_int] = ACTIONS(1977),
    [sym__wc_mid_string_int] = ACTIONS(1977),
    [sym__wc_remove_string_int] = ACTIONS(1977),
    [sym__wc_right_string_int] = ACTIONS(1977),
    [sym__wc_upper_string_int] = ACTIONS(1977),
    [sym__wc_compare_string_int] = ACTIONS(1977),
    [sym__wc_get_buffer_char_int] = ACTIONS(1977),
    [sym__wc_get_buffer_string_int] = ACTIONS(1977),
    [sym__wc_concat_string_int] = ACTIONS(1977),
    [sym__wc_file_open_int] = ACTIONS(1977),
    [sym__wc_file_close_int] = ACTIONS(1977),
    [sym__wc_file_read_int] = ACTIONS(1977),
    [sym__wc_file_read_line_int] = ACTIONS(1977),
    [sym__wc_file_write_int] = ACTIONS(1977),
    [sym__wc_file_write_line_int] = ACTIONS(1977),
    [aux_sym_preproc_include_token1] = ACTIONS(1977),
    [aux_sym_preproc_define_token1] = ACTIONS(1979),
    [aux_sym_preproc_warn_token1] = ACTIONS(1979),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1979),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1979),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1979),
    [anon_sym_LBRACK] = ACTIONS(1979),
    [anon_sym_LBRACE] = ACTIONS(1979),
    [anon_sym_LPAREN] = ACTIONS(1977),
    [anon_sym_BANG] = ACTIONS(1979),
    [anon_sym_TILDE] = ACTIONS(1979),
    [anon_sym_DASH] = ACTIONS(1977),
    [anon_sym_PLUS] = ACTIONS(1977),
    [anon_sym_DASH_DASH] = ACTIONS(1979),
    [anon_sym_PLUS_PLUS] = ACTIONS(1979),
    [anon_sym_DQUOTE] = ACTIONS(1979),
    [anon_sym_SQUOTE] = ACTIONS(1979),
    [sym_number_literal] = ACTIONS(1979),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(403)] = {
    [ts_builtin_sym_end] = ACTIONS(2063),
    [sym_identifier] = ACTIONS(2061),
    [sym_program_name_keyword] = ACTIONS(2061),
    [sym_module_name_keyword] = ACTIONS(2061),
    [sym_define_device_keyword] = ACTIONS(2061),
    [sym_define_combine_keyword] = ACTIONS(2061),
    [sym_define_constant_keyword] = ACTIONS(2061),
    [sym_define_type_keyword] = ACTIONS(2061),
    [sym_define_variable_keyword] = ACTIONS(2061),
    [sym_define_system_variable_keyword] = ACTIONS(2061),
    [sym_define_connect_level_keyword] = ACTIONS(2061),
    [sym_define_latching_keyword] = ACTIONS(2061),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2061),
    [sym_define_toggling_keyword] = ACTIONS(2061),
    [sym_define_call_keyword] = ACTIONS(2061),
    [sym_define_function_keyword] = ACTIONS(2061),
    [sym_define_library_function_keyword] = ACTIONS(2061),
    [sym_define_start_keyword] = ACTIONS(2061),
    [sym_define_event_keyword] = ACTIONS(2061),
    [sym_define_program_keyword] = ACTIONS(2061),
    [sym_define_module_keyword] = ACTIONS(2061),
    [sym_char_keyword] = ACTIONS(2061),
    [sym_widechar_keyword] = ACTIONS(2061),
    [sym_integer_keyword] = ACTIONS(2061),
    [sym_sinteger_keyword] = ACTIONS(2061),
    [sym_long_keyword] = ACTIONS(2061),
    [sym_slong_keyword] = ACTIONS(2061),
    [sym_float_keyword] = ACTIONS(2061),
    [sym_double_keyword] = ACTIONS(2061),
    [sym_variant_keyword] = ACTIONS(2061),
    [sym_variantarray_keyword] = ACTIONS(2061),
    [sym_local_var_keyword] = ACTIONS(2061),
    [sym_stack_var_keyword] = ACTIONS(2061),
    [sym_constant_keyword] = ACTIONS(2061),
    [sym_volatile_keyword] = ACTIONS(2061),
    [sym_non_volatile_keyword] = ACTIONS(2061),
    [sym_persistent_keyword] = ACTIONS(2061),
    [sym_dev_keyword] = ACTIONS(2061),
    [sym_devlev_keyword] = ACTIONS(2061),
    [sym_devchan_keyword] = ACTIONS(2061),
    [sym_if_keyword] = ACTIONS(2061),
    [sym_switch_keyword] = ACTIONS(2061),
    [sym_case_keyword] = ACTIONS(2061),
    [sym_default_keyword] = ACTIONS(2061),
    [sym_while_keyword] = ACTIONS(2061),
    [sym_for_keyword] = ACTIONS(2061),
    [sym_break_keyword] = ACTIONS(2061),
    [sym_continue_keyword] = ACTIONS(2061),
    [sym_return_keyword] = ACTIONS(2061),
    [sym_select_keyword] = ACTIONS(2061),
    [sym_active_keyword] = ACTIONS(2061),
    [sym_struct_keyword] = ACTIONS(2061),
    [sym_structure_keyword] = ACTIONS(2061),
    [sym_button_event_keyword] = ACTIONS(2061),
    [sym_channel_event_keyword] = ACTIONS(2061),
    [sym_level_event_keyword] = ACTIONS(2061),
    [sym_data_event_keyword] = ACTIONS(2061),
    [sym_timeline_event_keyword] = ACTIONS(2061),
    [sym_custom_event_keyword] = ACTIONS(2061),
    [sym_push_keyword] = ACTIONS(2061),
    [sym_release_keyword] = ACTIONS(2061),
    [aux_sym_on_keyword_token1] = ACTIONS(2061),
    [aux_sym_off_keyword_token1] = ACTIONS(2061),
    [sym_send_level_keyword] = ACTIONS(2061),
    [sym_send_string_keyword] = ACTIONS(2061),
    [sym_send_command_keyword] = ACTIONS(2061),
    [sym_clear_buffer_keyword] = ACTIONS(2061),
    [sym_create_buffer_keyword] = ACTIONS(2061),
    [sym_create_multi_buffer_keyword] = ACTIONS(2061),
    [sym_call_keyword] = ACTIONS(2061),
    [sym_system_call_keyword] = ACTIONS(2061),
    [sym_devchan_to_keyword] = ACTIONS(2061),
    [sym_devchan_min_to_keyword] = ACTIONS(2061),
    [sym_devchan_total_off_keyword] = ACTIONS(2061),
    [sym_devchan_pulse_keyword] = ACTIONS(2061),
    [sym_wait_keyword] = ACTIONS(2061),
    [sym_cancel_wait_keyword] = ACTIONS(2061),
    [sym_wait_until_keyword] = ACTIONS(2061),
    [sym_cancel_wait_until_keyword] = ACTIONS(2061),
    [sym_cancel_all_wait_keyword] = ACTIONS(2061),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2061),
    [sym_not] = ACTIONS(2061),
    [sym_bnot] = ACTIONS(2061),
    [sym___file__] = ACTIONS(2061),
    [sym___date__] = ACTIONS(2061),
    [sym___ldate__] = ACTIONS(2061),
    [sym___time__] = ACTIONS(2061),
    [sym___line__] = ACTIONS(2061),
    [sym___name__] = ACTIONS(2061),
    [sym___netlinx__] = ACTIONS(2061),
    [sym_netlinx_axi_version] = ACTIONS(2061),
    [sym_url_flg_tcp] = ACTIONS(2061),
    [sym_url_flg_accinfopresent] = ACTIONS(2061),
    [sym_url_flg_temp] = ACTIONS(2061),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2061),
    [sym_url_flg_stat_mask] = ACTIONS(2061),
    [sym_url_flg_stat_lookup] = ACTIONS(2061),
    [sym_url_flg_stat_connecting] = ACTIONS(2061),
    [sym_url_flg_stat_waiting] = ACTIONS(2061),
    [sym_url_flg_stat_connected] = ACTIONS(2061),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2061),
    [sym_first_virtual_device] = ACTIONS(2061),
    [sym_first_local_port] = ACTIONS(2061),
    [sym_dynamic_virtual_device] = ACTIONS(2061),
    [sym_timeline_once] = ACTIONS(2061),
    [sym_timeline_repeat] = ACTIONS(2061),
    [sym_timeline_absolute] = ACTIONS(2061),
    [sym_timeline_relative] = ACTIONS(2061),
    [sym_do_push_timed_infinite] = ACTIONS(2061),
    [sym_source_type_no_address] = ACTIONS(2061),
    [sym_source_type_neuron_id] = ACTIONS(2061),
    [sym_source_type_ip_address] = ACTIONS(2061),
    [sym_source_type_axlink] = ACTIONS(2061),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2061),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2061),
    [sym_source_type_ip_socket_address] = ACTIONS(2061),
    [sym_source_type_rs232] = ACTIONS(2061),
    [sym_source_type_internal] = ACTIONS(2061),
    [sym_source_type_mac_address] = ACTIONS(2061),
    [sym_source_type_ipv4_port] = ACTIONS(2061),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2061),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2061),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2061),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2061),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2061),
    [sym_true] = ACTIONS(2061),
    [sym_false] = ACTIONS(2061),
    [sym_file_read_only] = ACTIONS(2061),
    [sym_file_rw_new] = ACTIONS(2061),
    [sym_file_rw_append] = ACTIONS(2061),
    [sym_ip_tcp] = ACTIONS(2061),
    [sym_ip_udp] = ACTIONS(2061),
    [sym_ip_udp_2way] = ACTIONS(2061),
    [sym_xml_encode_types] = ACTIONS(2061),
    [sym_xml_encode_char_as_list] = ACTIONS(2061),
    [sym_xml_encode_le] = ACTIONS(2061),
    [sym_xml_decode_types] = ACTIONS(2061),
    [sym_xml_decode_no_preserve] = ACTIONS(2061),
    [sym_duet_dev_type_display_device] = ACTIONS(2061),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2061),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2061),
    [sym_duet_dev_type_io_device] = ACTIONS(2061),
    [sym_duet_dev_type_relay_device] = ACTIONS(2061),
    [sym_duet_dev_type_ups] = ACTIONS(2061),
    [sym_duet_dev_type_amplifier] = ACTIONS(2061),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2061),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2061),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2061),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2061),
    [sym_duet_dev_type_camera] = ACTIONS(2061),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2061),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2061),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2061),
    [sym_duet_dev_type_dss] = ACTIONS(2061),
    [sym_duet_dev_type_dvr] = ACTIONS(2061),
    [sym_duet_dev_type_disc_device] = ACTIONS(2061),
    [sym_duet_dev_type_document_camera] = ACTIONS(2061),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2061),
    [sym_duet_dev_type_hvac] = ACTIONS(2061),
    [sym_duet_dev_type_keypad] = ACTIONS(2061),
    [sym_duet_dev_type_light] = ACTIONS(2061),
    [sym_duet_dev_type_monitor] = ACTIONS(2061),
    [sym_duet_dev_type_motor] = ACTIONS(2061),
    [sym_duet_dev_type_multi_window] = ACTIONS(2061),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2061),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2061),
    [sym_duet_dev_type_receiver] = ACTIONS(2061),
    [sym_duet_dev_type_security_system] = ACTIONS(2061),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2061),
    [sym_duet_dev_type_settop_box] = ACTIONS(2061),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2061),
    [sym_duet_dev_type_switcher] = ACTIONS(2061),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2061),
    [sym_duet_dev_type_tv] = ACTIONS(2061),
    [sym_duet_dev_type_utility] = ACTIONS(2061),
    [sym_duet_dev_type_vcr] = ACTIONS(2061),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2061),
    [sym_duet_dev_type_video_processor] = ACTIONS(2061),
    [sym_duet_dev_type_video_projector] = ACTIONS(2061),
    [sym_duet_dev_type_video_wall] = ACTIONS(2061),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2061),
    [sym_duet_dev_type_weather] = ACTIONS(2061),
    [sym_duet_dev_not_polled] = ACTIONS(2061),
    [sym_duet_dev_polled] = ACTIONS(2061),
    [sym_clkmgr_mode_network] = ACTIONS(2061),
    [sym_clkmgr_mode_standalone] = ACTIONS(2061),
    [sym_normal_standby] = ACTIONS(2061),
    [sym_normal_wake] = ACTIONS(2061),
    [sym_ip_multicast_ttl_option] = ACTIONS(2061),
    [sym_ip_ttl_subnet] = ACTIONS(2061),
    [sym_ip_ttl_site] = ACTIONS(2061),
    [sym_ip_ttl_region] = ACTIONS(2061),
    [sym_ip_ttl_continent] = ACTIONS(2061),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2061),
    [sym_ip_nodelay_on] = ACTIONS(2061),
    [sym_ip_nodelay_off] = ACTIONS(2061),
    [sym_amx_error] = ACTIONS(2061),
    [sym_amx_warning] = ACTIONS(2061),
    [sym_amx_info] = ACTIONS(2061),
    [sym_amx_debug] = ACTIONS(2061),
    [sym_netlinx_login_success] = ACTIONS(2061),
    [sym_netlinx_login_fail] = ACTIONS(2061),
    [sym_netlinx_logout] = ACTIONS(2061),
    [sym_null_str] = ACTIONS(2061),
    [sym_smtp_address] = ACTIONS(2061),
    [sym_smtp_port_number] = ACTIONS(2061),
    [sym_smtp_username] = ACTIONS(2061),
    [sym_smtp_password] = ACTIONS(2061),
    [sym_smtp_require_tls] = ACTIONS(2061),
    [sym_smtp_from] = ACTIONS(2061),
    [sym_smtp_tls_true] = ACTIONS(2061),
    [sym_smtp_tls_false] = ACTIONS(2061),
    [sym_listview_on_row_select_event] = ACTIONS(2061),
    [sym_data_structure_unknown] = ACTIONS(2061),
    [sym_data_structure_datafeed] = ACTIONS(2061),
    [sym_data_structure_datarecordset] = ACTIONS(2061),
    [sym_data_structure_datarecord] = ACTIONS(2061),
    [sym_data_structure_datafield] = ACTIONS(2061),
    [sym_data_max_content_field_count] = ACTIONS(2061),
    [sym_data_max_metadata_field_count] = ACTIONS(2061),
    [sym_data_max_id_length] = ACTIONS(2061),
    [sym_data_max_name_length] = ACTIONS(2061),
    [sym_data_max_description_length] = ACTIONS(2061),
    [sym_data_max_type_length] = ACTIONS(2061),
    [sym_data_max_format_length] = ACTIONS(2061),
    [sym_data_max_source_length] = ACTIONS(2061),
    [sym_data_max_value_length] = ACTIONS(2061),
    [sym_data_max_label_length] = ACTIONS(2061),
    [sym_data_max_url_length] = ACTIONS(2061),
    [sym_max_last_login_info_length] = ACTIONS(2061),
    [sym_data_type_unknown] = ACTIONS(2061),
    [sym_data_type_string] = ACTIONS(2061),
    [sym_data_type_datetime] = ACTIONS(2061),
    [sym_data_type_date] = ACTIONS(2061),
    [sym_data_type_time] = ACTIONS(2061),
    [sym_data_type_image] = ACTIONS(2061),
    [sym_data_format_url] = ACTIONS(2061),
    [sym_data_format_phone] = ACTIONS(2061),
    [sym_data_format_email] = ACTIONS(2061),
    [sym_data_format_iso8601] = ACTIONS(2061),
    [sym_auditlog_priv] = ACTIONS(2061),
    [sym_configuration_priv] = ACTIONS(2061),
    [sym_deviceconfig_priv] = ACTIONS(2061),
    [sym_ftp_priv] = ACTIONS(2061),
    [sym_http_priv] = ACTIONS(2061),
    [sym_networkconfig_priv] = ACTIONS(2061),
    [sym_programport_priv] = ACTIONS(2061),
    [sym_remoteui_priv] = ACTIONS(2061),
    [sym_securitycontrol_priv] = ACTIONS(2061),
    [sym_softwaremanagement_priv] = ACTIONS(2061),
    [sym_terminal_priv] = ACTIONS(2061),
    [sym_tpadmin_priv] = ACTIONS(2061),
    [sym_user1_priv] = ACTIONS(2061),
    [sym_user2_priv] = ACTIONS(2061),
    [sym_user3_priv] = ACTIONS(2061),
    [sym_user4_priv] = ACTIONS(2061),
    [sym_usermanagement_priv] = ACTIONS(2061),
    [sym_valid_account] = ACTIONS(2061),
    [sym_err_name_invalid] = ACTIONS(2061),
    [sym_err_password_invalid] = ACTIONS(2061),
    [sym_err_invalid_account] = ACTIONS(2061),
    [sym_err_no_authorization] = ACTIONS(2061),
    [sym_err_invalid_parameter] = ACTIONS(2061),
    [sym_err_user_locked_out] = ACTIONS(2061),
    [sym_err_user_account_expired] = ACTIONS(2061),
    [sym_err_general_error] = ACTIONS(2061),
    [sym_err_authentication_error] = ACTIONS(2061),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2061),
    [sym_tls_validate_certificate] = ACTIONS(2061),
    [sym_tdata] = ACTIONS(2061),
    [sym_tchannel] = ACTIONS(2061),
    [sym_tlevel] = ACTIONS(2061),
    [sym_tbutton] = ACTIONS(2061),
    [sym_ttimeline] = ACTIONS(2061),
    [sym_tcustom] = ACTIONS(2061),
    [sym_url_struct] = ACTIONS(2061),
    [sym_dns_struct] = ACTIONS(2061),
    [sym_ip_address_struct] = ACTIONS(2061),
    [sym_dev_info_struct] = ACTIONS(2061),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2061),
    [sym_data_feed] = ACTIONS(2061),
    [sym_data_field] = ACTIONS(2061),
    [sym_data_record] = ACTIONS(2061),
    [sym_wc_data_feed] = ACTIONS(2061),
    [sym_wc_data_field] = ACTIONS(2061),
    [sym_wc_data_record] = ACTIONS(2061),
    [sym_last_login_info] = ACTIONS(2061),
    [sym____reserved___] = ACTIONS(2061),
    [sym_date] = ACTIONS(2061),
    [sym_day] = ACTIONS(2061),
    [sym_time] = ACTIONS(2061),
    [sym_ldate] = ACTIONS(2061),
    [sym_push_device] = ACTIONS(2061),
    [sym_push_channel] = ACTIONS(2061),
    [sym_push_devchan] = ACTIONS(2061),
    [sym_release_device] = ACTIONS(2061),
    [sym_release_channel] = ACTIONS(2061),
    [sym_release_devchan] = ACTIONS(2061),
    [sym_master_slot] = ACTIONS(2061),
    [sym_get_pulse_time] = ACTIONS(2061),
    [sym_get_timer] = ACTIONS(2061),
    [sym_master_sn] = ACTIONS(2061),
    [sym_system_number] = ACTIONS(2061),
    [sym_button] = ACTIONS(2061),
    [sym_dv_channel] = ACTIONS(2061),
    [sym_data] = ACTIONS(2061),
    [sym_level] = ACTIONS(2061),
    [sym_timeline] = ACTIONS(2061),
    [sym_channel] = ACTIONS(2061),
    [sym_custom] = ACTIONS(2061),
    [sym_atoi] = ACTIONS(2061),
    [sym_atol] = ACTIONS(2061),
    [sym_atof] = ACTIONS(2061),
    [sym_itoa] = ACTIONS(2061),
    [sym_itohex] = ACTIONS(2061),
    [sym_hextoi] = ACTIONS(2061),
    [sym_ftoa] = ACTIONS(2061),
    [sym_find_string] = ACTIONS(2061),
    [sym_left_string] = ACTIONS(2061),
    [sym_right_string] = ACTIONS(2061),
    [sym_mid_string] = ACTIONS(2061),
    [sym_remove_string] = ACTIONS(2061),
    [sym_get_buffer_string] = ACTIONS(2061),
    [sym_get_buffer_char] = ACTIONS(2061),
    [sym_get_multi_buffer_string] = ACTIONS(2061),
    [sym_length_string] = ACTIONS(2061),
    [sym_max_length_string] = ACTIONS(2061),
    [sym_set_length_string] = ACTIONS(2061),
    [sym_compare_string] = ACTIONS(2061),
    [sym_upper_string] = ACTIONS(2061),
    [sym_lower_string] = ACTIONS(2061),
    [sym_get_last] = ACTIONS(2061),
    [sym_date_to_day] = ACTIONS(2061),
    [sym_date_to_month] = ACTIONS(2061),
    [sym_date_to_year] = ACTIONS(2061),
    [sym_time_to_hour] = ACTIONS(2061),
    [sym_time_to_minute] = ACTIONS(2061),
    [sym_time_to_second] = ACTIONS(2061),
    [sym_day_of_week] = ACTIONS(2061),
    [sym_set_pulse_time] = ACTIONS(2061),
    [sym_set_timer] = ACTIONS(2061),
    [sym_file_open] = ACTIONS(2061),
    [sym_file_close] = ACTIONS(2061),
    [sym_file_read] = ACTIONS(2061),
    [sym_file_read_line] = ACTIONS(2061),
    [sym_file_write] = ACTIONS(2061),
    [sym_file_write_line] = ACTIONS(2061),
    [sym_file_copy] = ACTIONS(2061),
    [sym_file_delete] = ACTIONS(2061),
    [sym_file_rename] = ACTIONS(2061),
    [sym_file_dir] = ACTIONS(2061),
    [sym_file_getdir] = ACTIONS(2061),
    [sym_file_setdir] = ACTIONS(2061),
    [sym_file_create_dir] = ACTIONS(2061),
    [sym_file_remove_dir] = ACTIONS(2061),
    [sym_file_seek] = ACTIONS(2061),
    [sym_length_array] = ACTIONS(2061),
    [sym_max_length_array] = ACTIONS(2061),
    [sym_set_length_array] = ACTIONS(2061),
    [sym_ip_client_open] = ACTIONS(2061),
    [sym_ip_client_close] = ACTIONS(2061),
    [sym_ip_server_open] = ACTIONS(2061),
    [sym_ip_server_close] = ACTIONS(2061),
    [sym_ip_bound_client_open] = ACTIONS(2061),
    [sym_ip_mc_server_open] = ACTIONS(2061),
    [sym_ip_set_option] = ACTIONS(2061),
    [sym_ssh_client_open] = ACTIONS(2061),
    [sym_ssh_client_close] = ACTIONS(2061),
    [sym_tls_client_open] = ACTIONS(2061),
    [sym_tls_client_close] = ACTIONS(2061),
    [sym_do_push] = ACTIONS(2061),
    [sym_do_push_timed] = ACTIONS(2061),
    [sym_do_release] = ACTIONS(2061),
    [sym_device_id] = ACTIONS(2061),
    [sym_device_info] = ACTIONS(2061),
    [sym_device_id_string] = ACTIONS(2061),
    [sym_device_standby] = ACTIONS(2061),
    [sym_device_wake] = ACTIONS(2061),
    [sym_timeline_create] = ACTIONS(2061),
    [sym_timeline_kill] = ACTIONS(2061),
    [sym_timeline_reload] = ACTIONS(2061),
    [sym_timeline_pause] = ACTIONS(2061),
    [sym_timeline_restart] = ACTIONS(2061),
    [sym_timeline_set] = ACTIONS(2061),
    [sym_timeline_get] = ACTIONS(2061),
    [sym_timeline_active] = ACTIONS(2061),
    [sym_timeline_dynamic_id] = ACTIONS(2061),
    [sym_variable_to_string] = ACTIONS(2061),
    [sym_string_to_variable] = ACTIONS(2061),
    [sym_length_variable_to_string] = ACTIONS(2061),
    [sym_type_cast] = ACTIONS(2061),
    [sym_variable_to_xml] = ACTIONS(2061),
    [sym_xml_to_variable] = ACTIONS(2061),
    [sym_length_variable_to_xml] = ACTIONS(2061),
    [sym_redirect_string] = ACTIONS(2061),
    [sym_random_number] = ACTIONS(2061),
    [sym_get_unique_id] = ACTIONS(2061),
    [sym_get_system_number] = ACTIONS(2061),
    [sym_set_system_number] = ACTIONS(2061),
    [sym_get_serial_number] = ACTIONS(2061),
    [sym_get_master_build] = ACTIONS(2061),
    [sym_set_validation_code] = ACTIONS(2061),
    [sym_reboot] = ACTIONS(2061),
    [sym_load_duet_module] = ACTIONS(2061),
    [sym_unload_duet_module] = ACTIONS(2061),
    [sym_duet_mem_size_set] = ACTIONS(2061),
    [sym_duet_mem_size_get] = ACTIONS(2061),
    [sym_dynamic_application_device] = ACTIONS(2061),
    [sym_dynamic_polled_port] = ACTIONS(2061),
    [sym_static_port_binding] = ACTIONS(2061),
    [sym_static_ip_binding] = ACTIONS(2061),
    [sym_get_dns_list] = ACTIONS(2061),
    [sym_set_dns_list] = ACTIONS(2061),
    [sym_get_ip_address] = ACTIONS(2061),
    [sym_set_ip_address] = ACTIONS(2061),
    [sym_get_url_list] = ACTIONS(2061),
    [sym_add_url_entry] = ACTIONS(2061),
    [sym_delete_url_entry] = ACTIONS(2061),
    [sym_abs_value] = ACTIONS(2061),
    [sym_max_value] = ACTIONS(2061),
    [sym_min_value] = ACTIONS(2061),
    [sym_exp_value] = ACTIONS(2061),
    [sym_log_value] = ACTIONS(2061),
    [sym_log10_value] = ACTIONS(2061),
    [sym_power_value] = ACTIONS(2061),
    [sym_sqrt_value] = ACTIONS(2061),
    [sym_raw_be] = ACTIONS(2061),
    [sym_raw_le] = ACTIONS(2061),
    [sym_internal_queue_size_set] = ACTIONS(2061),
    [sym_internal_queue_size_get] = ACTIONS(2061),
    [sym_internal_threshold_set] = ACTIONS(2061),
    [sym_internal_threshold_get] = ACTIONS(2061),
    [sym_rebuilt_event] = ACTIONS(2061),
    [sym_do_custom_event] = ACTIONS(2061),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2061),
    [sym_clkmgr_set_clk_source] = ACTIONS(2061),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2061),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2061),
    [sym_clkmgr_get_timezone] = ACTIONS(2061),
    [sym_clkmgr_set_timezone] = ACTIONS(2061),
    [sym_clkmgr_get_resync_period] = ACTIONS(2061),
    [sym_clkmgr_set_resync_period] = ACTIONS(2061),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2061),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2061),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2061),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2061),
    [sym_clkmgr_get_timeservers] = ACTIONS(2061),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2061),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2061),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2061),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2061),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2061),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2061),
    [sym_set_log_level] = ACTIONS(2061),
    [sym_get_log_level] = ACTIONS(2061),
    [sym_amx_log] = ACTIONS(2061),
    [sym_smtp_server_config_set] = ACTIONS(2061),
    [sym_smtp_server_config_get] = ACTIONS(2061),
    [sym_smtp_send] = ACTIONS(2061),
    [sym_validate_netlinx_account] = ACTIONS(2061),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2061),
    [sym_authenticate_certificate] = ACTIONS(2061),
    [sym_audit_netlinx_session_event] = ACTIONS(2061),
    [sym_audit_netlinx_generic_event] = ACTIONS(2061),
    [sym_get_available_flash_disk_space] = ACTIONS(2061),
    [sym_get_max_flash_disk_space] = ACTIONS(2061),
    [sym_data_create_feed] = ACTIONS(2061),
    [sym_data_delete_feed] = ACTIONS(2061),
    [sym_data_publish_feed] = ACTIONS(2061),
    [sym_data_get_published_feed] = ACTIONS(2061),
    [sym_data_add_record] = ACTIONS(2061),
    [sym_data_get_event_record] = ACTIONS(2061),
    [sym__wc_data_create_feed] = ACTIONS(2061),
    [sym__wc_data_add_record] = ACTIONS(2061),
    [sym__wc_data_get_event_record] = ACTIONS(2061),
    [sym_format] = ACTIONS(2061),
    [sym_astro_clock] = ACTIONS(2061),
    [sym_set_outdoor_temperature] = ACTIONS(2061),
    [sym_set_virtual_level_count] = ACTIONS(2061),
    [sym_set_virtual_channel_count] = ACTIONS(2061),
    [sym_set_virtual_port_count] = ACTIONS(2061),
    [sym__wc_encode_int] = ACTIONS(2061),
    [sym__wc_decode_int] = ACTIONS(2061),
    [sym__wc_int] = ACTIONS(2061),
    [sym__wc_to_ch_int] = ACTIONS(2061),
    [sym__ch_to_wc_int] = ACTIONS(2061),
    [sym__wc_find_string_int] = ACTIONS(2061),
    [sym__wc_left_string_int] = ACTIONS(2061),
    [sym__wc_lower_string_int] = ACTIONS(2061),
    [sym__wc_mid_string_int] = ACTIONS(2061),
    [sym__wc_remove_string_int] = ACTIONS(2061),
    [sym__wc_right_string_int] = ACTIONS(2061),
    [sym__wc_upper_string_int] = ACTIONS(2061),
    [sym__wc_compare_string_int] = ACTIONS(2061),
    [sym__wc_get_buffer_char_int] = ACTIONS(2061),
    [sym__wc_get_buffer_string_int] = ACTIONS(2061),
    [sym__wc_concat_string_int] = ACTIONS(2061),
    [sym__wc_file_open_int] = ACTIONS(2061),
    [sym__wc_file_close_int] = ACTIONS(2061),
    [sym__wc_file_read_int] = ACTIONS(2061),
    [sym__wc_file_read_line_int] = ACTIONS(2061),
    [sym__wc_file_write_int] = ACTIONS(2061),
    [sym__wc_file_write_line_int] = ACTIONS(2061),
    [aux_sym_preproc_include_token1] = ACTIONS(2061),
    [aux_sym_preproc_define_token1] = ACTIONS(2063),
    [aux_sym_preproc_warn_token1] = ACTIONS(2063),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2063),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2063),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2063),
    [anon_sym_LBRACK] = ACTIONS(2063),
    [anon_sym_LBRACE] = ACTIONS(2063),
    [anon_sym_LPAREN] = ACTIONS(2061),
    [anon_sym_BANG] = ACTIONS(2063),
    [anon_sym_TILDE] = ACTIONS(2063),
    [anon_sym_DASH] = ACTIONS(2061),
    [anon_sym_PLUS] = ACTIONS(2061),
    [anon_sym_DASH_DASH] = ACTIONS(2063),
    [anon_sym_PLUS_PLUS] = ACTIONS(2063),
    [anon_sym_DQUOTE] = ACTIONS(2063),
    [anon_sym_SQUOTE] = ACTIONS(2063),
    [sym_number_literal] = ACTIONS(2063),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(404)] = {
    [ts_builtin_sym_end] = ACTIONS(1983),
    [sym_identifier] = ACTIONS(1981),
    [sym_program_name_keyword] = ACTIONS(1981),
    [sym_module_name_keyword] = ACTIONS(1981),
    [sym_define_device_keyword] = ACTIONS(1981),
    [sym_define_combine_keyword] = ACTIONS(1981),
    [sym_define_constant_keyword] = ACTIONS(1981),
    [sym_define_type_keyword] = ACTIONS(1981),
    [sym_define_variable_keyword] = ACTIONS(1981),
    [sym_define_system_variable_keyword] = ACTIONS(1981),
    [sym_define_connect_level_keyword] = ACTIONS(1981),
    [sym_define_latching_keyword] = ACTIONS(1981),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1981),
    [sym_define_toggling_keyword] = ACTIONS(1981),
    [sym_define_call_keyword] = ACTIONS(1981),
    [sym_define_function_keyword] = ACTIONS(1981),
    [sym_define_library_function_keyword] = ACTIONS(1981),
    [sym_define_start_keyword] = ACTIONS(1981),
    [sym_define_event_keyword] = ACTIONS(1981),
    [sym_define_program_keyword] = ACTIONS(1981),
    [sym_define_module_keyword] = ACTIONS(1981),
    [sym_char_keyword] = ACTIONS(1981),
    [sym_widechar_keyword] = ACTIONS(1981),
    [sym_integer_keyword] = ACTIONS(1981),
    [sym_sinteger_keyword] = ACTIONS(1981),
    [sym_long_keyword] = ACTIONS(1981),
    [sym_slong_keyword] = ACTIONS(1981),
    [sym_float_keyword] = ACTIONS(1981),
    [sym_double_keyword] = ACTIONS(1981),
    [sym_variant_keyword] = ACTIONS(1981),
    [sym_variantarray_keyword] = ACTIONS(1981),
    [sym_local_var_keyword] = ACTIONS(1981),
    [sym_stack_var_keyword] = ACTIONS(1981),
    [sym_constant_keyword] = ACTIONS(1981),
    [sym_volatile_keyword] = ACTIONS(1981),
    [sym_non_volatile_keyword] = ACTIONS(1981),
    [sym_persistent_keyword] = ACTIONS(1981),
    [sym_dev_keyword] = ACTIONS(1981),
    [sym_devlev_keyword] = ACTIONS(1981),
    [sym_devchan_keyword] = ACTIONS(1981),
    [sym_if_keyword] = ACTIONS(1981),
    [sym_switch_keyword] = ACTIONS(1981),
    [sym_case_keyword] = ACTIONS(1981),
    [sym_default_keyword] = ACTIONS(1981),
    [sym_while_keyword] = ACTIONS(1981),
    [sym_for_keyword] = ACTIONS(1981),
    [sym_break_keyword] = ACTIONS(1981),
    [sym_continue_keyword] = ACTIONS(1981),
    [sym_return_keyword] = ACTIONS(1981),
    [sym_select_keyword] = ACTIONS(1981),
    [sym_active_keyword] = ACTIONS(1981),
    [sym_struct_keyword] = ACTIONS(1981),
    [sym_structure_keyword] = ACTIONS(1981),
    [sym_button_event_keyword] = ACTIONS(1981),
    [sym_channel_event_keyword] = ACTIONS(1981),
    [sym_level_event_keyword] = ACTIONS(1981),
    [sym_data_event_keyword] = ACTIONS(1981),
    [sym_timeline_event_keyword] = ACTIONS(1981),
    [sym_custom_event_keyword] = ACTIONS(1981),
    [sym_push_keyword] = ACTIONS(1981),
    [sym_release_keyword] = ACTIONS(1981),
    [aux_sym_on_keyword_token1] = ACTIONS(1981),
    [aux_sym_off_keyword_token1] = ACTIONS(1981),
    [sym_send_level_keyword] = ACTIONS(1981),
    [sym_send_string_keyword] = ACTIONS(1981),
    [sym_send_command_keyword] = ACTIONS(1981),
    [sym_clear_buffer_keyword] = ACTIONS(1981),
    [sym_create_buffer_keyword] = ACTIONS(1981),
    [sym_create_multi_buffer_keyword] = ACTIONS(1981),
    [sym_call_keyword] = ACTIONS(1981),
    [sym_system_call_keyword] = ACTIONS(1981),
    [sym_devchan_to_keyword] = ACTIONS(1981),
    [sym_devchan_min_to_keyword] = ACTIONS(1981),
    [sym_devchan_total_off_keyword] = ACTIONS(1981),
    [sym_devchan_pulse_keyword] = ACTIONS(1981),
    [sym_wait_keyword] = ACTIONS(1981),
    [sym_cancel_wait_keyword] = ACTIONS(1981),
    [sym_wait_until_keyword] = ACTIONS(1981),
    [sym_cancel_wait_until_keyword] = ACTIONS(1981),
    [sym_cancel_all_wait_keyword] = ACTIONS(1981),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1981),
    [sym_not] = ACTIONS(1981),
    [sym_bnot] = ACTIONS(1981),
    [sym___file__] = ACTIONS(1981),
    [sym___date__] = ACTIONS(1981),
    [sym___ldate__] = ACTIONS(1981),
    [sym___time__] = ACTIONS(1981),
    [sym___line__] = ACTIONS(1981),
    [sym___name__] = ACTIONS(1981),
    [sym___netlinx__] = ACTIONS(1981),
    [sym_netlinx_axi_version] = ACTIONS(1981),
    [sym_url_flg_tcp] = ACTIONS(1981),
    [sym_url_flg_accinfopresent] = ACTIONS(1981),
    [sym_url_flg_temp] = ACTIONS(1981),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1981),
    [sym_url_flg_stat_mask] = ACTIONS(1981),
    [sym_url_flg_stat_lookup] = ACTIONS(1981),
    [sym_url_flg_stat_connecting] = ACTIONS(1981),
    [sym_url_flg_stat_waiting] = ACTIONS(1981),
    [sym_url_flg_stat_connected] = ACTIONS(1981),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1981),
    [sym_first_virtual_device] = ACTIONS(1981),
    [sym_first_local_port] = ACTIONS(1981),
    [sym_dynamic_virtual_device] = ACTIONS(1981),
    [sym_timeline_once] = ACTIONS(1981),
    [sym_timeline_repeat] = ACTIONS(1981),
    [sym_timeline_absolute] = ACTIONS(1981),
    [sym_timeline_relative] = ACTIONS(1981),
    [sym_do_push_timed_infinite] = ACTIONS(1981),
    [sym_source_type_no_address] = ACTIONS(1981),
    [sym_source_type_neuron_id] = ACTIONS(1981),
    [sym_source_type_ip_address] = ACTIONS(1981),
    [sym_source_type_axlink] = ACTIONS(1981),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1981),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1981),
    [sym_source_type_ip_socket_address] = ACTIONS(1981),
    [sym_source_type_rs232] = ACTIONS(1981),
    [sym_source_type_internal] = ACTIONS(1981),
    [sym_source_type_mac_address] = ACTIONS(1981),
    [sym_source_type_ipv4_port] = ACTIONS(1981),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1981),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1981),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1981),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1981),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1981),
    [sym_true] = ACTIONS(1981),
    [sym_false] = ACTIONS(1981),
    [sym_file_read_only] = ACTIONS(1981),
    [sym_file_rw_new] = ACTIONS(1981),
    [sym_file_rw_append] = ACTIONS(1981),
    [sym_ip_tcp] = ACTIONS(1981),
    [sym_ip_udp] = ACTIONS(1981),
    [sym_ip_udp_2way] = ACTIONS(1981),
    [sym_xml_encode_types] = ACTIONS(1981),
    [sym_xml_encode_char_as_list] = ACTIONS(1981),
    [sym_xml_encode_le] = ACTIONS(1981),
    [sym_xml_decode_types] = ACTIONS(1981),
    [sym_xml_decode_no_preserve] = ACTIONS(1981),
    [sym_duet_dev_type_display_device] = ACTIONS(1981),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1981),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1981),
    [sym_duet_dev_type_io_device] = ACTIONS(1981),
    [sym_duet_dev_type_relay_device] = ACTIONS(1981),
    [sym_duet_dev_type_ups] = ACTIONS(1981),
    [sym_duet_dev_type_amplifier] = ACTIONS(1981),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1981),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1981),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1981),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1981),
    [sym_duet_dev_type_camera] = ACTIONS(1981),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1981),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1981),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1981),
    [sym_duet_dev_type_dss] = ACTIONS(1981),
    [sym_duet_dev_type_dvr] = ACTIONS(1981),
    [sym_duet_dev_type_disc_device] = ACTIONS(1981),
    [sym_duet_dev_type_document_camera] = ACTIONS(1981),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1981),
    [sym_duet_dev_type_hvac] = ACTIONS(1981),
    [sym_duet_dev_type_keypad] = ACTIONS(1981),
    [sym_duet_dev_type_light] = ACTIONS(1981),
    [sym_duet_dev_type_monitor] = ACTIONS(1981),
    [sym_duet_dev_type_motor] = ACTIONS(1981),
    [sym_duet_dev_type_multi_window] = ACTIONS(1981),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1981),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1981),
    [sym_duet_dev_type_receiver] = ACTIONS(1981),
    [sym_duet_dev_type_security_system] = ACTIONS(1981),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1981),
    [sym_duet_dev_type_settop_box] = ACTIONS(1981),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1981),
    [sym_duet_dev_type_switcher] = ACTIONS(1981),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1981),
    [sym_duet_dev_type_tv] = ACTIONS(1981),
    [sym_duet_dev_type_utility] = ACTIONS(1981),
    [sym_duet_dev_type_vcr] = ACTIONS(1981),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1981),
    [sym_duet_dev_type_video_processor] = ACTIONS(1981),
    [sym_duet_dev_type_video_projector] = ACTIONS(1981),
    [sym_duet_dev_type_video_wall] = ACTIONS(1981),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1981),
    [sym_duet_dev_type_weather] = ACTIONS(1981),
    [sym_duet_dev_not_polled] = ACTIONS(1981),
    [sym_duet_dev_polled] = ACTIONS(1981),
    [sym_clkmgr_mode_network] = ACTIONS(1981),
    [sym_clkmgr_mode_standalone] = ACTIONS(1981),
    [sym_normal_standby] = ACTIONS(1981),
    [sym_normal_wake] = ACTIONS(1981),
    [sym_ip_multicast_ttl_option] = ACTIONS(1981),
    [sym_ip_ttl_subnet] = ACTIONS(1981),
    [sym_ip_ttl_site] = ACTIONS(1981),
    [sym_ip_ttl_region] = ACTIONS(1981),
    [sym_ip_ttl_continent] = ACTIONS(1981),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1981),
    [sym_ip_nodelay_on] = ACTIONS(1981),
    [sym_ip_nodelay_off] = ACTIONS(1981),
    [sym_amx_error] = ACTIONS(1981),
    [sym_amx_warning] = ACTIONS(1981),
    [sym_amx_info] = ACTIONS(1981),
    [sym_amx_debug] = ACTIONS(1981),
    [sym_netlinx_login_success] = ACTIONS(1981),
    [sym_netlinx_login_fail] = ACTIONS(1981),
    [sym_netlinx_logout] = ACTIONS(1981),
    [sym_null_str] = ACTIONS(1981),
    [sym_smtp_address] = ACTIONS(1981),
    [sym_smtp_port_number] = ACTIONS(1981),
    [sym_smtp_username] = ACTIONS(1981),
    [sym_smtp_password] = ACTIONS(1981),
    [sym_smtp_require_tls] = ACTIONS(1981),
    [sym_smtp_from] = ACTIONS(1981),
    [sym_smtp_tls_true] = ACTIONS(1981),
    [sym_smtp_tls_false] = ACTIONS(1981),
    [sym_listview_on_row_select_event] = ACTIONS(1981),
    [sym_data_structure_unknown] = ACTIONS(1981),
    [sym_data_structure_datafeed] = ACTIONS(1981),
    [sym_data_structure_datarecordset] = ACTIONS(1981),
    [sym_data_structure_datarecord] = ACTIONS(1981),
    [sym_data_structure_datafield] = ACTIONS(1981),
    [sym_data_max_content_field_count] = ACTIONS(1981),
    [sym_data_max_metadata_field_count] = ACTIONS(1981),
    [sym_data_max_id_length] = ACTIONS(1981),
    [sym_data_max_name_length] = ACTIONS(1981),
    [sym_data_max_description_length] = ACTIONS(1981),
    [sym_data_max_type_length] = ACTIONS(1981),
    [sym_data_max_format_length] = ACTIONS(1981),
    [sym_data_max_source_length] = ACTIONS(1981),
    [sym_data_max_value_length] = ACTIONS(1981),
    [sym_data_max_label_length] = ACTIONS(1981),
    [sym_data_max_url_length] = ACTIONS(1981),
    [sym_max_last_login_info_length] = ACTIONS(1981),
    [sym_data_type_unknown] = ACTIONS(1981),
    [sym_data_type_string] = ACTIONS(1981),
    [sym_data_type_datetime] = ACTIONS(1981),
    [sym_data_type_date] = ACTIONS(1981),
    [sym_data_type_time] = ACTIONS(1981),
    [sym_data_type_image] = ACTIONS(1981),
    [sym_data_format_url] = ACTIONS(1981),
    [sym_data_format_phone] = ACTIONS(1981),
    [sym_data_format_email] = ACTIONS(1981),
    [sym_data_format_iso8601] = ACTIONS(1981),
    [sym_auditlog_priv] = ACTIONS(1981),
    [sym_configuration_priv] = ACTIONS(1981),
    [sym_deviceconfig_priv] = ACTIONS(1981),
    [sym_ftp_priv] = ACTIONS(1981),
    [sym_http_priv] = ACTIONS(1981),
    [sym_networkconfig_priv] = ACTIONS(1981),
    [sym_programport_priv] = ACTIONS(1981),
    [sym_remoteui_priv] = ACTIONS(1981),
    [sym_securitycontrol_priv] = ACTIONS(1981),
    [sym_softwaremanagement_priv] = ACTIONS(1981),
    [sym_terminal_priv] = ACTIONS(1981),
    [sym_tpadmin_priv] = ACTIONS(1981),
    [sym_user1_priv] = ACTIONS(1981),
    [sym_user2_priv] = ACTIONS(1981),
    [sym_user3_priv] = ACTIONS(1981),
    [sym_user4_priv] = ACTIONS(1981),
    [sym_usermanagement_priv] = ACTIONS(1981),
    [sym_valid_account] = ACTIONS(1981),
    [sym_err_name_invalid] = ACTIONS(1981),
    [sym_err_password_invalid] = ACTIONS(1981),
    [sym_err_invalid_account] = ACTIONS(1981),
    [sym_err_no_authorization] = ACTIONS(1981),
    [sym_err_invalid_parameter] = ACTIONS(1981),
    [sym_err_user_locked_out] = ACTIONS(1981),
    [sym_err_user_account_expired] = ACTIONS(1981),
    [sym_err_general_error] = ACTIONS(1981),
    [sym_err_authentication_error] = ACTIONS(1981),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1981),
    [sym_tls_validate_certificate] = ACTIONS(1981),
    [sym_tdata] = ACTIONS(1981),
    [sym_tchannel] = ACTIONS(1981),
    [sym_tlevel] = ACTIONS(1981),
    [sym_tbutton] = ACTIONS(1981),
    [sym_ttimeline] = ACTIONS(1981),
    [sym_tcustom] = ACTIONS(1981),
    [sym_url_struct] = ACTIONS(1981),
    [sym_dns_struct] = ACTIONS(1981),
    [sym_ip_address_struct] = ACTIONS(1981),
    [sym_dev_info_struct] = ACTIONS(1981),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1981),
    [sym_data_feed] = ACTIONS(1981),
    [sym_data_field] = ACTIONS(1981),
    [sym_data_record] = ACTIONS(1981),
    [sym_wc_data_feed] = ACTIONS(1981),
    [sym_wc_data_field] = ACTIONS(1981),
    [sym_wc_data_record] = ACTIONS(1981),
    [sym_last_login_info] = ACTIONS(1981),
    [sym____reserved___] = ACTIONS(1981),
    [sym_date] = ACTIONS(1981),
    [sym_day] = ACTIONS(1981),
    [sym_time] = ACTIONS(1981),
    [sym_ldate] = ACTIONS(1981),
    [sym_push_device] = ACTIONS(1981),
    [sym_push_channel] = ACTIONS(1981),
    [sym_push_devchan] = ACTIONS(1981),
    [sym_release_device] = ACTIONS(1981),
    [sym_release_channel] = ACTIONS(1981),
    [sym_release_devchan] = ACTIONS(1981),
    [sym_master_slot] = ACTIONS(1981),
    [sym_get_pulse_time] = ACTIONS(1981),
    [sym_get_timer] = ACTIONS(1981),
    [sym_master_sn] = ACTIONS(1981),
    [sym_system_number] = ACTIONS(1981),
    [sym_button] = ACTIONS(1981),
    [sym_dv_channel] = ACTIONS(1981),
    [sym_data] = ACTIONS(1981),
    [sym_level] = ACTIONS(1981),
    [sym_timeline] = ACTIONS(1981),
    [sym_channel] = ACTIONS(1981),
    [sym_custom] = ACTIONS(1981),
    [sym_atoi] = ACTIONS(1981),
    [sym_atol] = ACTIONS(1981),
    [sym_atof] = ACTIONS(1981),
    [sym_itoa] = ACTIONS(1981),
    [sym_itohex] = ACTIONS(1981),
    [sym_hextoi] = ACTIONS(1981),
    [sym_ftoa] = ACTIONS(1981),
    [sym_find_string] = ACTIONS(1981),
    [sym_left_string] = ACTIONS(1981),
    [sym_right_string] = ACTIONS(1981),
    [sym_mid_string] = ACTIONS(1981),
    [sym_remove_string] = ACTIONS(1981),
    [sym_get_buffer_string] = ACTIONS(1981),
    [sym_get_buffer_char] = ACTIONS(1981),
    [sym_get_multi_buffer_string] = ACTIONS(1981),
    [sym_length_string] = ACTIONS(1981),
    [sym_max_length_string] = ACTIONS(1981),
    [sym_set_length_string] = ACTIONS(1981),
    [sym_compare_string] = ACTIONS(1981),
    [sym_upper_string] = ACTIONS(1981),
    [sym_lower_string] = ACTIONS(1981),
    [sym_get_last] = ACTIONS(1981),
    [sym_date_to_day] = ACTIONS(1981),
    [sym_date_to_month] = ACTIONS(1981),
    [sym_date_to_year] = ACTIONS(1981),
    [sym_time_to_hour] = ACTIONS(1981),
    [sym_time_to_minute] = ACTIONS(1981),
    [sym_time_to_second] = ACTIONS(1981),
    [sym_day_of_week] = ACTIONS(1981),
    [sym_set_pulse_time] = ACTIONS(1981),
    [sym_set_timer] = ACTIONS(1981),
    [sym_file_open] = ACTIONS(1981),
    [sym_file_close] = ACTIONS(1981),
    [sym_file_read] = ACTIONS(1981),
    [sym_file_read_line] = ACTIONS(1981),
    [sym_file_write] = ACTIONS(1981),
    [sym_file_write_line] = ACTIONS(1981),
    [sym_file_copy] = ACTIONS(1981),
    [sym_file_delete] = ACTIONS(1981),
    [sym_file_rename] = ACTIONS(1981),
    [sym_file_dir] = ACTIONS(1981),
    [sym_file_getdir] = ACTIONS(1981),
    [sym_file_setdir] = ACTIONS(1981),
    [sym_file_create_dir] = ACTIONS(1981),
    [sym_file_remove_dir] = ACTIONS(1981),
    [sym_file_seek] = ACTIONS(1981),
    [sym_length_array] = ACTIONS(1981),
    [sym_max_length_array] = ACTIONS(1981),
    [sym_set_length_array] = ACTIONS(1981),
    [sym_ip_client_open] = ACTIONS(1981),
    [sym_ip_client_close] = ACTIONS(1981),
    [sym_ip_server_open] = ACTIONS(1981),
    [sym_ip_server_close] = ACTIONS(1981),
    [sym_ip_bound_client_open] = ACTIONS(1981),
    [sym_ip_mc_server_open] = ACTIONS(1981),
    [sym_ip_set_option] = ACTIONS(1981),
    [sym_ssh_client_open] = ACTIONS(1981),
    [sym_ssh_client_close] = ACTIONS(1981),
    [sym_tls_client_open] = ACTIONS(1981),
    [sym_tls_client_close] = ACTIONS(1981),
    [sym_do_push] = ACTIONS(1981),
    [sym_do_push_timed] = ACTIONS(1981),
    [sym_do_release] = ACTIONS(1981),
    [sym_device_id] = ACTIONS(1981),
    [sym_device_info] = ACTIONS(1981),
    [sym_device_id_string] = ACTIONS(1981),
    [sym_device_standby] = ACTIONS(1981),
    [sym_device_wake] = ACTIONS(1981),
    [sym_timeline_create] = ACTIONS(1981),
    [sym_timeline_kill] = ACTIONS(1981),
    [sym_timeline_reload] = ACTIONS(1981),
    [sym_timeline_pause] = ACTIONS(1981),
    [sym_timeline_restart] = ACTIONS(1981),
    [sym_timeline_set] = ACTIONS(1981),
    [sym_timeline_get] = ACTIONS(1981),
    [sym_timeline_active] = ACTIONS(1981),
    [sym_timeline_dynamic_id] = ACTIONS(1981),
    [sym_variable_to_string] = ACTIONS(1981),
    [sym_string_to_variable] = ACTIONS(1981),
    [sym_length_variable_to_string] = ACTIONS(1981),
    [sym_type_cast] = ACTIONS(1981),
    [sym_variable_to_xml] = ACTIONS(1981),
    [sym_xml_to_variable] = ACTIONS(1981),
    [sym_length_variable_to_xml] = ACTIONS(1981),
    [sym_redirect_string] = ACTIONS(1981),
    [sym_random_number] = ACTIONS(1981),
    [sym_get_unique_id] = ACTIONS(1981),
    [sym_get_system_number] = ACTIONS(1981),
    [sym_set_system_number] = ACTIONS(1981),
    [sym_get_serial_number] = ACTIONS(1981),
    [sym_get_master_build] = ACTIONS(1981),
    [sym_set_validation_code] = ACTIONS(1981),
    [sym_reboot] = ACTIONS(1981),
    [sym_load_duet_module] = ACTIONS(1981),
    [sym_unload_duet_module] = ACTIONS(1981),
    [sym_duet_mem_size_set] = ACTIONS(1981),
    [sym_duet_mem_size_get] = ACTIONS(1981),
    [sym_dynamic_application_device] = ACTIONS(1981),
    [sym_dynamic_polled_port] = ACTIONS(1981),
    [sym_static_port_binding] = ACTIONS(1981),
    [sym_static_ip_binding] = ACTIONS(1981),
    [sym_get_dns_list] = ACTIONS(1981),
    [sym_set_dns_list] = ACTIONS(1981),
    [sym_get_ip_address] = ACTIONS(1981),
    [sym_set_ip_address] = ACTIONS(1981),
    [sym_get_url_list] = ACTIONS(1981),
    [sym_add_url_entry] = ACTIONS(1981),
    [sym_delete_url_entry] = ACTIONS(1981),
    [sym_abs_value] = ACTIONS(1981),
    [sym_max_value] = ACTIONS(1981),
    [sym_min_value] = ACTIONS(1981),
    [sym_exp_value] = ACTIONS(1981),
    [sym_log_value] = ACTIONS(1981),
    [sym_log10_value] = ACTIONS(1981),
    [sym_power_value] = ACTIONS(1981),
    [sym_sqrt_value] = ACTIONS(1981),
    [sym_raw_be] = ACTIONS(1981),
    [sym_raw_le] = ACTIONS(1981),
    [sym_internal_queue_size_set] = ACTIONS(1981),
    [sym_internal_queue_size_get] = ACTIONS(1981),
    [sym_internal_threshold_set] = ACTIONS(1981),
    [sym_internal_threshold_get] = ACTIONS(1981),
    [sym_rebuilt_event] = ACTIONS(1981),
    [sym_do_custom_event] = ACTIONS(1981),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1981),
    [sym_clkmgr_set_clk_source] = ACTIONS(1981),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1981),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1981),
    [sym_clkmgr_get_timezone] = ACTIONS(1981),
    [sym_clkmgr_set_timezone] = ACTIONS(1981),
    [sym_clkmgr_get_resync_period] = ACTIONS(1981),
    [sym_clkmgr_set_resync_period] = ACTIONS(1981),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1981),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1981),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1981),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1981),
    [sym_clkmgr_get_timeservers] = ACTIONS(1981),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1981),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1981),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1981),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1981),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1981),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1981),
    [sym_set_log_level] = ACTIONS(1981),
    [sym_get_log_level] = ACTIONS(1981),
    [sym_amx_log] = ACTIONS(1981),
    [sym_smtp_server_config_set] = ACTIONS(1981),
    [sym_smtp_server_config_get] = ACTIONS(1981),
    [sym_smtp_send] = ACTIONS(1981),
    [sym_validate_netlinx_account] = ACTIONS(1981),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1981),
    [sym_authenticate_certificate] = ACTIONS(1981),
    [sym_audit_netlinx_session_event] = ACTIONS(1981),
    [sym_audit_netlinx_generic_event] = ACTIONS(1981),
    [sym_get_available_flash_disk_space] = ACTIONS(1981),
    [sym_get_max_flash_disk_space] = ACTIONS(1981),
    [sym_data_create_feed] = ACTIONS(1981),
    [sym_data_delete_feed] = ACTIONS(1981),
    [sym_data_publish_feed] = ACTIONS(1981),
    [sym_data_get_published_feed] = ACTIONS(1981),
    [sym_data_add_record] = ACTIONS(1981),
    [sym_data_get_event_record] = ACTIONS(1981),
    [sym__wc_data_create_feed] = ACTIONS(1981),
    [sym__wc_data_add_record] = ACTIONS(1981),
    [sym__wc_data_get_event_record] = ACTIONS(1981),
    [sym_format] = ACTIONS(1981),
    [sym_astro_clock] = ACTIONS(1981),
    [sym_set_outdoor_temperature] = ACTIONS(1981),
    [sym_set_virtual_level_count] = ACTIONS(1981),
    [sym_set_virtual_channel_count] = ACTIONS(1981),
    [sym_set_virtual_port_count] = ACTIONS(1981),
    [sym__wc_encode_int] = ACTIONS(1981),
    [sym__wc_decode_int] = ACTIONS(1981),
    [sym__wc_int] = ACTIONS(1981),
    [sym__wc_to_ch_int] = ACTIONS(1981),
    [sym__ch_to_wc_int] = ACTIONS(1981),
    [sym__wc_find_string_int] = ACTIONS(1981),
    [sym__wc_left_string_int] = ACTIONS(1981),
    [sym__wc_lower_string_int] = ACTIONS(1981),
    [sym__wc_mid_string_int] = ACTIONS(1981),
    [sym__wc_remove_string_int] = ACTIONS(1981),
    [sym__wc_right_string_int] = ACTIONS(1981),
    [sym__wc_upper_string_int] = ACTIONS(1981),
    [sym__wc_compare_string_int] = ACTIONS(1981),
    [sym__wc_get_buffer_char_int] = ACTIONS(1981),
    [sym__wc_get_buffer_string_int] = ACTIONS(1981),
    [sym__wc_concat_string_int] = ACTIONS(1981),
    [sym__wc_file_open_int] = ACTIONS(1981),
    [sym__wc_file_close_int] = ACTIONS(1981),
    [sym__wc_file_read_int] = ACTIONS(1981),
    [sym__wc_file_read_line_int] = ACTIONS(1981),
    [sym__wc_file_write_int] = ACTIONS(1981),
    [sym__wc_file_write_line_int] = ACTIONS(1981),
    [aux_sym_preproc_include_token1] = ACTIONS(1981),
    [aux_sym_preproc_define_token1] = ACTIONS(1983),
    [aux_sym_preproc_warn_token1] = ACTIONS(1983),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1983),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1983),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1983),
    [anon_sym_LBRACK] = ACTIONS(1983),
    [anon_sym_LBRACE] = ACTIONS(1983),
    [anon_sym_LPAREN] = ACTIONS(1981),
    [anon_sym_BANG] = ACTIONS(1983),
    [anon_sym_TILDE] = ACTIONS(1983),
    [anon_sym_DASH] = ACTIONS(1981),
    [anon_sym_PLUS] = ACTIONS(1981),
    [anon_sym_DASH_DASH] = ACTIONS(1983),
    [anon_sym_PLUS_PLUS] = ACTIONS(1983),
    [anon_sym_DQUOTE] = ACTIONS(1983),
    [anon_sym_SQUOTE] = ACTIONS(1983),
    [sym_number_literal] = ACTIONS(1983),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(405)] = {
    [ts_builtin_sym_end] = ACTIONS(2091),
    [sym_identifier] = ACTIONS(2089),
    [sym_program_name_keyword] = ACTIONS(2089),
    [sym_module_name_keyword] = ACTIONS(2089),
    [sym_define_device_keyword] = ACTIONS(2089),
    [sym_define_combine_keyword] = ACTIONS(2089),
    [sym_define_constant_keyword] = ACTIONS(2089),
    [sym_define_type_keyword] = ACTIONS(2089),
    [sym_define_variable_keyword] = ACTIONS(2089),
    [sym_define_system_variable_keyword] = ACTIONS(2089),
    [sym_define_connect_level_keyword] = ACTIONS(2089),
    [sym_define_latching_keyword] = ACTIONS(2089),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2089),
    [sym_define_toggling_keyword] = ACTIONS(2089),
    [sym_define_call_keyword] = ACTIONS(2089),
    [sym_define_function_keyword] = ACTIONS(2089),
    [sym_define_library_function_keyword] = ACTIONS(2089),
    [sym_define_start_keyword] = ACTIONS(2089),
    [sym_define_event_keyword] = ACTIONS(2089),
    [sym_define_program_keyword] = ACTIONS(2089),
    [sym_define_module_keyword] = ACTIONS(2089),
    [sym_char_keyword] = ACTIONS(2089),
    [sym_widechar_keyword] = ACTIONS(2089),
    [sym_integer_keyword] = ACTIONS(2089),
    [sym_sinteger_keyword] = ACTIONS(2089),
    [sym_long_keyword] = ACTIONS(2089),
    [sym_slong_keyword] = ACTIONS(2089),
    [sym_float_keyword] = ACTIONS(2089),
    [sym_double_keyword] = ACTIONS(2089),
    [sym_variant_keyword] = ACTIONS(2089),
    [sym_variantarray_keyword] = ACTIONS(2089),
    [sym_local_var_keyword] = ACTIONS(2089),
    [sym_stack_var_keyword] = ACTIONS(2089),
    [sym_constant_keyword] = ACTIONS(2089),
    [sym_volatile_keyword] = ACTIONS(2089),
    [sym_non_volatile_keyword] = ACTIONS(2089),
    [sym_persistent_keyword] = ACTIONS(2089),
    [sym_dev_keyword] = ACTIONS(2089),
    [sym_devlev_keyword] = ACTIONS(2089),
    [sym_devchan_keyword] = ACTIONS(2089),
    [sym_if_keyword] = ACTIONS(2089),
    [sym_switch_keyword] = ACTIONS(2089),
    [sym_case_keyword] = ACTIONS(2089),
    [sym_default_keyword] = ACTIONS(2089),
    [sym_while_keyword] = ACTIONS(2089),
    [sym_for_keyword] = ACTIONS(2089),
    [sym_break_keyword] = ACTIONS(2089),
    [sym_continue_keyword] = ACTIONS(2089),
    [sym_return_keyword] = ACTIONS(2089),
    [sym_select_keyword] = ACTIONS(2089),
    [sym_active_keyword] = ACTIONS(2089),
    [sym_struct_keyword] = ACTIONS(2089),
    [sym_structure_keyword] = ACTIONS(2089),
    [sym_button_event_keyword] = ACTIONS(2089),
    [sym_channel_event_keyword] = ACTIONS(2089),
    [sym_level_event_keyword] = ACTIONS(2089),
    [sym_data_event_keyword] = ACTIONS(2089),
    [sym_timeline_event_keyword] = ACTIONS(2089),
    [sym_custom_event_keyword] = ACTIONS(2089),
    [sym_push_keyword] = ACTIONS(2089),
    [sym_release_keyword] = ACTIONS(2089),
    [aux_sym_on_keyword_token1] = ACTIONS(2089),
    [aux_sym_off_keyword_token1] = ACTIONS(2089),
    [sym_send_level_keyword] = ACTIONS(2089),
    [sym_send_string_keyword] = ACTIONS(2089),
    [sym_send_command_keyword] = ACTIONS(2089),
    [sym_clear_buffer_keyword] = ACTIONS(2089),
    [sym_create_buffer_keyword] = ACTIONS(2089),
    [sym_create_multi_buffer_keyword] = ACTIONS(2089),
    [sym_call_keyword] = ACTIONS(2089),
    [sym_system_call_keyword] = ACTIONS(2089),
    [sym_devchan_to_keyword] = ACTIONS(2089),
    [sym_devchan_min_to_keyword] = ACTIONS(2089),
    [sym_devchan_total_off_keyword] = ACTIONS(2089),
    [sym_devchan_pulse_keyword] = ACTIONS(2089),
    [sym_wait_keyword] = ACTIONS(2089),
    [sym_cancel_wait_keyword] = ACTIONS(2089),
    [sym_wait_until_keyword] = ACTIONS(2089),
    [sym_cancel_wait_until_keyword] = ACTIONS(2089),
    [sym_cancel_all_wait_keyword] = ACTIONS(2089),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2089),
    [sym_not] = ACTIONS(2089),
    [sym_bnot] = ACTIONS(2089),
    [sym___file__] = ACTIONS(2089),
    [sym___date__] = ACTIONS(2089),
    [sym___ldate__] = ACTIONS(2089),
    [sym___time__] = ACTIONS(2089),
    [sym___line__] = ACTIONS(2089),
    [sym___name__] = ACTIONS(2089),
    [sym___netlinx__] = ACTIONS(2089),
    [sym_netlinx_axi_version] = ACTIONS(2089),
    [sym_url_flg_tcp] = ACTIONS(2089),
    [sym_url_flg_accinfopresent] = ACTIONS(2089),
    [sym_url_flg_temp] = ACTIONS(2089),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2089),
    [sym_url_flg_stat_mask] = ACTIONS(2089),
    [sym_url_flg_stat_lookup] = ACTIONS(2089),
    [sym_url_flg_stat_connecting] = ACTIONS(2089),
    [sym_url_flg_stat_waiting] = ACTIONS(2089),
    [sym_url_flg_stat_connected] = ACTIONS(2089),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2089),
    [sym_first_virtual_device] = ACTIONS(2089),
    [sym_first_local_port] = ACTIONS(2089),
    [sym_dynamic_virtual_device] = ACTIONS(2089),
    [sym_timeline_once] = ACTIONS(2089),
    [sym_timeline_repeat] = ACTIONS(2089),
    [sym_timeline_absolute] = ACTIONS(2089),
    [sym_timeline_relative] = ACTIONS(2089),
    [sym_do_push_timed_infinite] = ACTIONS(2089),
    [sym_source_type_no_address] = ACTIONS(2089),
    [sym_source_type_neuron_id] = ACTIONS(2089),
    [sym_source_type_ip_address] = ACTIONS(2089),
    [sym_source_type_axlink] = ACTIONS(2089),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2089),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2089),
    [sym_source_type_ip_socket_address] = ACTIONS(2089),
    [sym_source_type_rs232] = ACTIONS(2089),
    [sym_source_type_internal] = ACTIONS(2089),
    [sym_source_type_mac_address] = ACTIONS(2089),
    [sym_source_type_ipv4_port] = ACTIONS(2089),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2089),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2089),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2089),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2089),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2089),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2089),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2089),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2089),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2089),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2089),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2089),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2089),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2089),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2089),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2089),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2089),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2089),
    [sym_true] = ACTIONS(2089),
    [sym_false] = ACTIONS(2089),
    [sym_file_read_only] = ACTIONS(2089),
    [sym_file_rw_new] = ACTIONS(2089),
    [sym_file_rw_append] = ACTIONS(2089),
    [sym_ip_tcp] = ACTIONS(2089),
    [sym_ip_udp] = ACTIONS(2089),
    [sym_ip_udp_2way] = ACTIONS(2089),
    [sym_xml_encode_types] = ACTIONS(2089),
    [sym_xml_encode_char_as_list] = ACTIONS(2089),
    [sym_xml_encode_le] = ACTIONS(2089),
    [sym_xml_decode_types] = ACTIONS(2089),
    [sym_xml_decode_no_preserve] = ACTIONS(2089),
    [sym_duet_dev_type_display_device] = ACTIONS(2089),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2089),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2089),
    [sym_duet_dev_type_io_device] = ACTIONS(2089),
    [sym_duet_dev_type_relay_device] = ACTIONS(2089),
    [sym_duet_dev_type_ups] = ACTIONS(2089),
    [sym_duet_dev_type_amplifier] = ACTIONS(2089),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2089),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2089),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2089),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2089),
    [sym_duet_dev_type_camera] = ACTIONS(2089),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2089),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2089),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2089),
    [sym_duet_dev_type_dss] = ACTIONS(2089),
    [sym_duet_dev_type_dvr] = ACTIONS(2089),
    [sym_duet_dev_type_disc_device] = ACTIONS(2089),
    [sym_duet_dev_type_document_camera] = ACTIONS(2089),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2089),
    [sym_duet_dev_type_hvac] = ACTIONS(2089),
    [sym_duet_dev_type_keypad] = ACTIONS(2089),
    [sym_duet_dev_type_light] = ACTIONS(2089),
    [sym_duet_dev_type_monitor] = ACTIONS(2089),
    [sym_duet_dev_type_motor] = ACTIONS(2089),
    [sym_duet_dev_type_multi_window] = ACTIONS(2089),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2089),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2089),
    [sym_duet_dev_type_receiver] = ACTIONS(2089),
    [sym_duet_dev_type_security_system] = ACTIONS(2089),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2089),
    [sym_duet_dev_type_settop_box] = ACTIONS(2089),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2089),
    [sym_duet_dev_type_switcher] = ACTIONS(2089),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2089),
    [sym_duet_dev_type_tv] = ACTIONS(2089),
    [sym_duet_dev_type_utility] = ACTIONS(2089),
    [sym_duet_dev_type_vcr] = ACTIONS(2089),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2089),
    [sym_duet_dev_type_video_processor] = ACTIONS(2089),
    [sym_duet_dev_type_video_projector] = ACTIONS(2089),
    [sym_duet_dev_type_video_wall] = ACTIONS(2089),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2089),
    [sym_duet_dev_type_weather] = ACTIONS(2089),
    [sym_duet_dev_not_polled] = ACTIONS(2089),
    [sym_duet_dev_polled] = ACTIONS(2089),
    [sym_clkmgr_mode_network] = ACTIONS(2089),
    [sym_clkmgr_mode_standalone] = ACTIONS(2089),
    [sym_normal_standby] = ACTIONS(2089),
    [sym_normal_wake] = ACTIONS(2089),
    [sym_ip_multicast_ttl_option] = ACTIONS(2089),
    [sym_ip_ttl_subnet] = ACTIONS(2089),
    [sym_ip_ttl_site] = ACTIONS(2089),
    [sym_ip_ttl_region] = ACTIONS(2089),
    [sym_ip_ttl_continent] = ACTIONS(2089),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2089),
    [sym_ip_nodelay_on] = ACTIONS(2089),
    [sym_ip_nodelay_off] = ACTIONS(2089),
    [sym_amx_error] = ACTIONS(2089),
    [sym_amx_warning] = ACTIONS(2089),
    [sym_amx_info] = ACTIONS(2089),
    [sym_amx_debug] = ACTIONS(2089),
    [sym_netlinx_login_success] = ACTIONS(2089),
    [sym_netlinx_login_fail] = ACTIONS(2089),
    [sym_netlinx_logout] = ACTIONS(2089),
    [sym_null_str] = ACTIONS(2089),
    [sym_smtp_address] = ACTIONS(2089),
    [sym_smtp_port_number] = ACTIONS(2089),
    [sym_smtp_username] = ACTIONS(2089),
    [sym_smtp_password] = ACTIONS(2089),
    [sym_smtp_require_tls] = ACTIONS(2089),
    [sym_smtp_from] = ACTIONS(2089),
    [sym_smtp_tls_true] = ACTIONS(2089),
    [sym_smtp_tls_false] = ACTIONS(2089),
    [sym_listview_on_row_select_event] = ACTIONS(2089),
    [sym_data_structure_unknown] = ACTIONS(2089),
    [sym_data_structure_datafeed] = ACTIONS(2089),
    [sym_data_structure_datarecordset] = ACTIONS(2089),
    [sym_data_structure_datarecord] = ACTIONS(2089),
    [sym_data_structure_datafield] = ACTIONS(2089),
    [sym_data_max_content_field_count] = ACTIONS(2089),
    [sym_data_max_metadata_field_count] = ACTIONS(2089),
    [sym_data_max_id_length] = ACTIONS(2089),
    [sym_data_max_name_length] = ACTIONS(2089),
    [sym_data_max_description_length] = ACTIONS(2089),
    [sym_data_max_type_length] = ACTIONS(2089),
    [sym_data_max_format_length] = ACTIONS(2089),
    [sym_data_max_source_length] = ACTIONS(2089),
    [sym_data_max_value_length] = ACTIONS(2089),
    [sym_data_max_label_length] = ACTIONS(2089),
    [sym_data_max_url_length] = ACTIONS(2089),
    [sym_max_last_login_info_length] = ACTIONS(2089),
    [sym_data_type_unknown] = ACTIONS(2089),
    [sym_data_type_string] = ACTIONS(2089),
    [sym_data_type_datetime] = ACTIONS(2089),
    [sym_data_type_date] = ACTIONS(2089),
    [sym_data_type_time] = ACTIONS(2089),
    [sym_data_type_image] = ACTIONS(2089),
    [sym_data_format_url] = ACTIONS(2089),
    [sym_data_format_phone] = ACTIONS(2089),
    [sym_data_format_email] = ACTIONS(2089),
    [sym_data_format_iso8601] = ACTIONS(2089),
    [sym_auditlog_priv] = ACTIONS(2089),
    [sym_configuration_priv] = ACTIONS(2089),
    [sym_deviceconfig_priv] = ACTIONS(2089),
    [sym_ftp_priv] = ACTIONS(2089),
    [sym_http_priv] = ACTIONS(2089),
    [sym_networkconfig_priv] = ACTIONS(2089),
    [sym_programport_priv] = ACTIONS(2089),
    [sym_remoteui_priv] = ACTIONS(2089),
    [sym_securitycontrol_priv] = ACTIONS(2089),
    [sym_softwaremanagement_priv] = ACTIONS(2089),
    [sym_terminal_priv] = ACTIONS(2089),
    [sym_tpadmin_priv] = ACTIONS(2089),
    [sym_user1_priv] = ACTIONS(2089),
    [sym_user2_priv] = ACTIONS(2089),
    [sym_user3_priv] = ACTIONS(2089),
    [sym_user4_priv] = ACTIONS(2089),
    [sym_usermanagement_priv] = ACTIONS(2089),
    [sym_valid_account] = ACTIONS(2089),
    [sym_err_name_invalid] = ACTIONS(2089),
    [sym_err_password_invalid] = ACTIONS(2089),
    [sym_err_invalid_account] = ACTIONS(2089),
    [sym_err_no_authorization] = ACTIONS(2089),
    [sym_err_invalid_parameter] = ACTIONS(2089),
    [sym_err_user_locked_out] = ACTIONS(2089),
    [sym_err_user_account_expired] = ACTIONS(2089),
    [sym_err_general_error] = ACTIONS(2089),
    [sym_err_authentication_error] = ACTIONS(2089),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2089),
    [sym_tls_validate_certificate] = ACTIONS(2089),
    [sym_tdata] = ACTIONS(2089),
    [sym_tchannel] = ACTIONS(2089),
    [sym_tlevel] = ACTIONS(2089),
    [sym_tbutton] = ACTIONS(2089),
    [sym_ttimeline] = ACTIONS(2089),
    [sym_tcustom] = ACTIONS(2089),
    [sym_url_struct] = ACTIONS(2089),
    [sym_dns_struct] = ACTIONS(2089),
    [sym_ip_address_struct] = ACTIONS(2089),
    [sym_dev_info_struct] = ACTIONS(2089),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2089),
    [sym_data_feed] = ACTIONS(2089),
    [sym_data_field] = ACTIONS(2089),
    [sym_data_record] = ACTIONS(2089),
    [sym_wc_data_feed] = ACTIONS(2089),
    [sym_wc_data_field] = ACTIONS(2089),
    [sym_wc_data_record] = ACTIONS(2089),
    [sym_last_login_info] = ACTIONS(2089),
    [sym____reserved___] = ACTIONS(2089),
    [sym_date] = ACTIONS(2089),
    [sym_day] = ACTIONS(2089),
    [sym_time] = ACTIONS(2089),
    [sym_ldate] = ACTIONS(2089),
    [sym_push_device] = ACTIONS(2089),
    [sym_push_channel] = ACTIONS(2089),
    [sym_push_devchan] = ACTIONS(2089),
    [sym_release_device] = ACTIONS(2089),
    [sym_release_channel] = ACTIONS(2089),
    [sym_release_devchan] = ACTIONS(2089),
    [sym_master_slot] = ACTIONS(2089),
    [sym_get_pulse_time] = ACTIONS(2089),
    [sym_get_timer] = ACTIONS(2089),
    [sym_master_sn] = ACTIONS(2089),
    [sym_system_number] = ACTIONS(2089),
    [sym_button] = ACTIONS(2089),
    [sym_dv_channel] = ACTIONS(2089),
    [sym_data] = ACTIONS(2089),
    [sym_level] = ACTIONS(2089),
    [sym_timeline] = ACTIONS(2089),
    [sym_channel] = ACTIONS(2089),
    [sym_custom] = ACTIONS(2089),
    [sym_atoi] = ACTIONS(2089),
    [sym_atol] = ACTIONS(2089),
    [sym_atof] = ACTIONS(2089),
    [sym_itoa] = ACTIONS(2089),
    [sym_itohex] = ACTIONS(2089),
    [sym_hextoi] = ACTIONS(2089),
    [sym_ftoa] = ACTIONS(2089),
    [sym_find_string] = ACTIONS(2089),
    [sym_left_string] = ACTIONS(2089),
    [sym_right_string] = ACTIONS(2089),
    [sym_mid_string] = ACTIONS(2089),
    [sym_remove_string] = ACTIONS(2089),
    [sym_get_buffer_string] = ACTIONS(2089),
    [sym_get_buffer_char] = ACTIONS(2089),
    [sym_get_multi_buffer_string] = ACTIONS(2089),
    [sym_length_string] = ACTIONS(2089),
    [sym_max_length_string] = ACTIONS(2089),
    [sym_set_length_string] = ACTIONS(2089),
    [sym_compare_string] = ACTIONS(2089),
    [sym_upper_string] = ACTIONS(2089),
    [sym_lower_string] = ACTIONS(2089),
    [sym_get_last] = ACTIONS(2089),
    [sym_date_to_day] = ACTIONS(2089),
    [sym_date_to_month] = ACTIONS(2089),
    [sym_date_to_year] = ACTIONS(2089),
    [sym_time_to_hour] = ACTIONS(2089),
    [sym_time_to_minute] = ACTIONS(2089),
    [sym_time_to_second] = ACTIONS(2089),
    [sym_day_of_week] = ACTIONS(2089),
    [sym_set_pulse_time] = ACTIONS(2089),
    [sym_set_timer] = ACTIONS(2089),
    [sym_file_open] = ACTIONS(2089),
    [sym_file_close] = ACTIONS(2089),
    [sym_file_read] = ACTIONS(2089),
    [sym_file_read_line] = ACTIONS(2089),
    [sym_file_write] = ACTIONS(2089),
    [sym_file_write_line] = ACTIONS(2089),
    [sym_file_copy] = ACTIONS(2089),
    [sym_file_delete] = ACTIONS(2089),
    [sym_file_rename] = ACTIONS(2089),
    [sym_file_dir] = ACTIONS(2089),
    [sym_file_getdir] = ACTIONS(2089),
    [sym_file_setdir] = ACTIONS(2089),
    [sym_file_create_dir] = ACTIONS(2089),
    [sym_file_remove_dir] = ACTIONS(2089),
    [sym_file_seek] = ACTIONS(2089),
    [sym_length_array] = ACTIONS(2089),
    [sym_max_length_array] = ACTIONS(2089),
    [sym_set_length_array] = ACTIONS(2089),
    [sym_ip_client_open] = ACTIONS(2089),
    [sym_ip_client_close] = ACTIONS(2089),
    [sym_ip_server_open] = ACTIONS(2089),
    [sym_ip_server_close] = ACTIONS(2089),
    [sym_ip_bound_client_open] = ACTIONS(2089),
    [sym_ip_mc_server_open] = ACTIONS(2089),
    [sym_ip_set_option] = ACTIONS(2089),
    [sym_ssh_client_open] = ACTIONS(2089),
    [sym_ssh_client_close] = ACTIONS(2089),
    [sym_tls_client_open] = ACTIONS(2089),
    [sym_tls_client_close] = ACTIONS(2089),
    [sym_do_push] = ACTIONS(2089),
    [sym_do_push_timed] = ACTIONS(2089),
    [sym_do_release] = ACTIONS(2089),
    [sym_device_id] = ACTIONS(2089),
    [sym_device_info] = ACTIONS(2089),
    [sym_device_id_string] = ACTIONS(2089),
    [sym_device_standby] = ACTIONS(2089),
    [sym_device_wake] = ACTIONS(2089),
    [sym_timeline_create] = ACTIONS(2089),
    [sym_timeline_kill] = ACTIONS(2089),
    [sym_timeline_reload] = ACTIONS(2089),
    [sym_timeline_pause] = ACTIONS(2089),
    [sym_timeline_restart] = ACTIONS(2089),
    [sym_timeline_set] = ACTIONS(2089),
    [sym_timeline_get] = ACTIONS(2089),
    [sym_timeline_active] = ACTIONS(2089),
    [sym_timeline_dynamic_id] = ACTIONS(2089),
    [sym_variable_to_string] = ACTIONS(2089),
    [sym_string_to_variable] = ACTIONS(2089),
    [sym_length_variable_to_string] = ACTIONS(2089),
    [sym_type_cast] = ACTIONS(2089),
    [sym_variable_to_xml] = ACTIONS(2089),
    [sym_xml_to_variable] = ACTIONS(2089),
    [sym_length_variable_to_xml] = ACTIONS(2089),
    [sym_redirect_string] = ACTIONS(2089),
    [sym_random_number] = ACTIONS(2089),
    [sym_get_unique_id] = ACTIONS(2089),
    [sym_get_system_number] = ACTIONS(2089),
    [sym_set_system_number] = ACTIONS(2089),
    [sym_get_serial_number] = ACTIONS(2089),
    [sym_get_master_build] = ACTIONS(2089),
    [sym_set_validation_code] = ACTIONS(2089),
    [sym_reboot] = ACTIONS(2089),
    [sym_load_duet_module] = ACTIONS(2089),
    [sym_unload_duet_module] = ACTIONS(2089),
    [sym_duet_mem_size_set] = ACTIONS(2089),
    [sym_duet_mem_size_get] = ACTIONS(2089),
    [sym_dynamic_application_device] = ACTIONS(2089),
    [sym_dynamic_polled_port] = ACTIONS(2089),
    [sym_static_port_binding] = ACTIONS(2089),
    [sym_static_ip_binding] = ACTIONS(2089),
    [sym_get_dns_list] = ACTIONS(2089),
    [sym_set_dns_list] = ACTIONS(2089),
    [sym_get_ip_address] = ACTIONS(2089),
    [sym_set_ip_address] = ACTIONS(2089),
    [sym_get_url_list] = ACTIONS(2089),
    [sym_add_url_entry] = ACTIONS(2089),
    [sym_delete_url_entry] = ACTIONS(2089),
    [sym_abs_value] = ACTIONS(2089),
    [sym_max_value] = ACTIONS(2089),
    [sym_min_value] = ACTIONS(2089),
    [sym_exp_value] = ACTIONS(2089),
    [sym_log_value] = ACTIONS(2089),
    [sym_log10_value] = ACTIONS(2089),
    [sym_power_value] = ACTIONS(2089),
    [sym_sqrt_value] = ACTIONS(2089),
    [sym_raw_be] = ACTIONS(2089),
    [sym_raw_le] = ACTIONS(2089),
    [sym_internal_queue_size_set] = ACTIONS(2089),
    [sym_internal_queue_size_get] = ACTIONS(2089),
    [sym_internal_threshold_set] = ACTIONS(2089),
    [sym_internal_threshold_get] = ACTIONS(2089),
    [sym_rebuilt_event] = ACTIONS(2089),
    [sym_do_custom_event] = ACTIONS(2089),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2089),
    [sym_clkmgr_set_clk_source] = ACTIONS(2089),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2089),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2089),
    [sym_clkmgr_get_timezone] = ACTIONS(2089),
    [sym_clkmgr_set_timezone] = ACTIONS(2089),
    [sym_clkmgr_get_resync_period] = ACTIONS(2089),
    [sym_clkmgr_set_resync_period] = ACTIONS(2089),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2089),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2089),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2089),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2089),
    [sym_clkmgr_get_timeservers] = ACTIONS(2089),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2089),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2089),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2089),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2089),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2089),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2089),
    [sym_set_log_level] = ACTIONS(2089),
    [sym_get_log_level] = ACTIONS(2089),
    [sym_amx_log] = ACTIONS(2089),
    [sym_smtp_server_config_set] = ACTIONS(2089),
    [sym_smtp_server_config_get] = ACTIONS(2089),
    [sym_smtp_send] = ACTIONS(2089),
    [sym_validate_netlinx_account] = ACTIONS(2089),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2089),
    [sym_authenticate_certificate] = ACTIONS(2089),
    [sym_audit_netlinx_session_event] = ACTIONS(2089),
    [sym_audit_netlinx_generic_event] = ACTIONS(2089),
    [sym_get_available_flash_disk_space] = ACTIONS(2089),
    [sym_get_max_flash_disk_space] = ACTIONS(2089),
    [sym_data_create_feed] = ACTIONS(2089),
    [sym_data_delete_feed] = ACTIONS(2089),
    [sym_data_publish_feed] = ACTIONS(2089),
    [sym_data_get_published_feed] = ACTIONS(2089),
    [sym_data_add_record] = ACTIONS(2089),
    [sym_data_get_event_record] = ACTIONS(2089),
    [sym__wc_data_create_feed] = ACTIONS(2089),
    [sym__wc_data_add_record] = ACTIONS(2089),
    [sym__wc_data_get_event_record] = ACTIONS(2089),
    [sym_format] = ACTIONS(2089),
    [sym_astro_clock] = ACTIONS(2089),
    [sym_set_outdoor_temperature] = ACTIONS(2089),
    [sym_set_virtual_level_count] = ACTIONS(2089),
    [sym_set_virtual_channel_count] = ACTIONS(2089),
    [sym_set_virtual_port_count] = ACTIONS(2089),
    [sym__wc_encode_int] = ACTIONS(2089),
    [sym__wc_decode_int] = ACTIONS(2089),
    [sym__wc_int] = ACTIONS(2089),
    [sym__wc_to_ch_int] = ACTIONS(2089),
    [sym__ch_to_wc_int] = ACTIONS(2089),
    [sym__wc_find_string_int] = ACTIONS(2089),
    [sym__wc_left_string_int] = ACTIONS(2089),
    [sym__wc_lower_string_int] = ACTIONS(2089),
    [sym__wc_mid_string_int] = ACTIONS(2089),
    [sym__wc_remove_string_int] = ACTIONS(2089),
    [sym__wc_right_string_int] = ACTIONS(2089),
    [sym__wc_upper_string_int] = ACTIONS(2089),
    [sym__wc_compare_string_int] = ACTIONS(2089),
    [sym__wc_get_buffer_char_int] = ACTIONS(2089),
    [sym__wc_get_buffer_string_int] = ACTIONS(2089),
    [sym__wc_concat_string_int] = ACTIONS(2089),
    [sym__wc_file_open_int] = ACTIONS(2089),
    [sym__wc_file_close_int] = ACTIONS(2089),
    [sym__wc_file_read_int] = ACTIONS(2089),
    [sym__wc_file_read_line_int] = ACTIONS(2089),
    [sym__wc_file_write_int] = ACTIONS(2089),
    [sym__wc_file_write_line_int] = ACTIONS(2089),
    [aux_sym_preproc_include_token1] = ACTIONS(2089),
    [aux_sym_preproc_define_token1] = ACTIONS(2091),
    [aux_sym_preproc_warn_token1] = ACTIONS(2091),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2091),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2091),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2091),
    [anon_sym_LBRACK] = ACTIONS(2091),
    [anon_sym_LBRACE] = ACTIONS(2091),
    [anon_sym_LPAREN] = ACTIONS(2089),
    [anon_sym_BANG] = ACTIONS(2091),
    [anon_sym_TILDE] = ACTIONS(2091),
    [anon_sym_DASH] = ACTIONS(2089),
    [anon_sym_PLUS] = ACTIONS(2089),
    [anon_sym_DASH_DASH] = ACTIONS(2091),
    [anon_sym_PLUS_PLUS] = ACTIONS(2091),
    [anon_sym_DQUOTE] = ACTIONS(2091),
    [anon_sym_SQUOTE] = ACTIONS(2091),
    [sym_number_literal] = ACTIONS(2091),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(406)] = {
    [ts_builtin_sym_end] = ACTIONS(2067),
    [sym_identifier] = ACTIONS(2065),
    [sym_program_name_keyword] = ACTIONS(2065),
    [sym_module_name_keyword] = ACTIONS(2065),
    [sym_define_device_keyword] = ACTIONS(2065),
    [sym_define_combine_keyword] = ACTIONS(2065),
    [sym_define_constant_keyword] = ACTIONS(2065),
    [sym_define_type_keyword] = ACTIONS(2065),
    [sym_define_variable_keyword] = ACTIONS(2065),
    [sym_define_system_variable_keyword] = ACTIONS(2065),
    [sym_define_connect_level_keyword] = ACTIONS(2065),
    [sym_define_latching_keyword] = ACTIONS(2065),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2065),
    [sym_define_toggling_keyword] = ACTIONS(2065),
    [sym_define_call_keyword] = ACTIONS(2065),
    [sym_define_function_keyword] = ACTIONS(2065),
    [sym_define_library_function_keyword] = ACTIONS(2065),
    [sym_define_start_keyword] = ACTIONS(2065),
    [sym_define_event_keyword] = ACTIONS(2065),
    [sym_define_program_keyword] = ACTIONS(2065),
    [sym_define_module_keyword] = ACTIONS(2065),
    [sym_char_keyword] = ACTIONS(2065),
    [sym_widechar_keyword] = ACTIONS(2065),
    [sym_integer_keyword] = ACTIONS(2065),
    [sym_sinteger_keyword] = ACTIONS(2065),
    [sym_long_keyword] = ACTIONS(2065),
    [sym_slong_keyword] = ACTIONS(2065),
    [sym_float_keyword] = ACTIONS(2065),
    [sym_double_keyword] = ACTIONS(2065),
    [sym_variant_keyword] = ACTIONS(2065),
    [sym_variantarray_keyword] = ACTIONS(2065),
    [sym_local_var_keyword] = ACTIONS(2065),
    [sym_stack_var_keyword] = ACTIONS(2065),
    [sym_constant_keyword] = ACTIONS(2065),
    [sym_volatile_keyword] = ACTIONS(2065),
    [sym_non_volatile_keyword] = ACTIONS(2065),
    [sym_persistent_keyword] = ACTIONS(2065),
    [sym_dev_keyword] = ACTIONS(2065),
    [sym_devlev_keyword] = ACTIONS(2065),
    [sym_devchan_keyword] = ACTIONS(2065),
    [sym_if_keyword] = ACTIONS(2065),
    [sym_switch_keyword] = ACTIONS(2065),
    [sym_case_keyword] = ACTIONS(2065),
    [sym_default_keyword] = ACTIONS(2065),
    [sym_while_keyword] = ACTIONS(2065),
    [sym_for_keyword] = ACTIONS(2065),
    [sym_break_keyword] = ACTIONS(2065),
    [sym_continue_keyword] = ACTIONS(2065),
    [sym_return_keyword] = ACTIONS(2065),
    [sym_select_keyword] = ACTIONS(2065),
    [sym_active_keyword] = ACTIONS(2065),
    [sym_struct_keyword] = ACTIONS(2065),
    [sym_structure_keyword] = ACTIONS(2065),
    [sym_button_event_keyword] = ACTIONS(2065),
    [sym_channel_event_keyword] = ACTIONS(2065),
    [sym_level_event_keyword] = ACTIONS(2065),
    [sym_data_event_keyword] = ACTIONS(2065),
    [sym_timeline_event_keyword] = ACTIONS(2065),
    [sym_custom_event_keyword] = ACTIONS(2065),
    [sym_push_keyword] = ACTIONS(2065),
    [sym_release_keyword] = ACTIONS(2065),
    [aux_sym_on_keyword_token1] = ACTIONS(2065),
    [aux_sym_off_keyword_token1] = ACTIONS(2065),
    [sym_send_level_keyword] = ACTIONS(2065),
    [sym_send_string_keyword] = ACTIONS(2065),
    [sym_send_command_keyword] = ACTIONS(2065),
    [sym_clear_buffer_keyword] = ACTIONS(2065),
    [sym_create_buffer_keyword] = ACTIONS(2065),
    [sym_create_multi_buffer_keyword] = ACTIONS(2065),
    [sym_call_keyword] = ACTIONS(2065),
    [sym_system_call_keyword] = ACTIONS(2065),
    [sym_devchan_to_keyword] = ACTIONS(2065),
    [sym_devchan_min_to_keyword] = ACTIONS(2065),
    [sym_devchan_total_off_keyword] = ACTIONS(2065),
    [sym_devchan_pulse_keyword] = ACTIONS(2065),
    [sym_wait_keyword] = ACTIONS(2065),
    [sym_cancel_wait_keyword] = ACTIONS(2065),
    [sym_wait_until_keyword] = ACTIONS(2065),
    [sym_cancel_wait_until_keyword] = ACTIONS(2065),
    [sym_cancel_all_wait_keyword] = ACTIONS(2065),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2065),
    [sym_not] = ACTIONS(2065),
    [sym_bnot] = ACTIONS(2065),
    [sym___file__] = ACTIONS(2065),
    [sym___date__] = ACTIONS(2065),
    [sym___ldate__] = ACTIONS(2065),
    [sym___time__] = ACTIONS(2065),
    [sym___line__] = ACTIONS(2065),
    [sym___name__] = ACTIONS(2065),
    [sym___netlinx__] = ACTIONS(2065),
    [sym_netlinx_axi_version] = ACTIONS(2065),
    [sym_url_flg_tcp] = ACTIONS(2065),
    [sym_url_flg_accinfopresent] = ACTIONS(2065),
    [sym_url_flg_temp] = ACTIONS(2065),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2065),
    [sym_url_flg_stat_mask] = ACTIONS(2065),
    [sym_url_flg_stat_lookup] = ACTIONS(2065),
    [sym_url_flg_stat_connecting] = ACTIONS(2065),
    [sym_url_flg_stat_waiting] = ACTIONS(2065),
    [sym_url_flg_stat_connected] = ACTIONS(2065),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2065),
    [sym_first_virtual_device] = ACTIONS(2065),
    [sym_first_local_port] = ACTIONS(2065),
    [sym_dynamic_virtual_device] = ACTIONS(2065),
    [sym_timeline_once] = ACTIONS(2065),
    [sym_timeline_repeat] = ACTIONS(2065),
    [sym_timeline_absolute] = ACTIONS(2065),
    [sym_timeline_relative] = ACTIONS(2065),
    [sym_do_push_timed_infinite] = ACTIONS(2065),
    [sym_source_type_no_address] = ACTIONS(2065),
    [sym_source_type_neuron_id] = ACTIONS(2065),
    [sym_source_type_ip_address] = ACTIONS(2065),
    [sym_source_type_axlink] = ACTIONS(2065),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2065),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2065),
    [sym_source_type_ip_socket_address] = ACTIONS(2065),
    [sym_source_type_rs232] = ACTIONS(2065),
    [sym_source_type_internal] = ACTIONS(2065),
    [sym_source_type_mac_address] = ACTIONS(2065),
    [sym_source_type_ipv4_port] = ACTIONS(2065),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2065),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2065),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2065),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2065),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2065),
    [sym_true] = ACTIONS(2065),
    [sym_false] = ACTIONS(2065),
    [sym_file_read_only] = ACTIONS(2065),
    [sym_file_rw_new] = ACTIONS(2065),
    [sym_file_rw_append] = ACTIONS(2065),
    [sym_ip_tcp] = ACTIONS(2065),
    [sym_ip_udp] = ACTIONS(2065),
    [sym_ip_udp_2way] = ACTIONS(2065),
    [sym_xml_encode_types] = ACTIONS(2065),
    [sym_xml_encode_char_as_list] = ACTIONS(2065),
    [sym_xml_encode_le] = ACTIONS(2065),
    [sym_xml_decode_types] = ACTIONS(2065),
    [sym_xml_decode_no_preserve] = ACTIONS(2065),
    [sym_duet_dev_type_display_device] = ACTIONS(2065),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2065),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2065),
    [sym_duet_dev_type_io_device] = ACTIONS(2065),
    [sym_duet_dev_type_relay_device] = ACTIONS(2065),
    [sym_duet_dev_type_ups] = ACTIONS(2065),
    [sym_duet_dev_type_amplifier] = ACTIONS(2065),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2065),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2065),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2065),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2065),
    [sym_duet_dev_type_camera] = ACTIONS(2065),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2065),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2065),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2065),
    [sym_duet_dev_type_dss] = ACTIONS(2065),
    [sym_duet_dev_type_dvr] = ACTIONS(2065),
    [sym_duet_dev_type_disc_device] = ACTIONS(2065),
    [sym_duet_dev_type_document_camera] = ACTIONS(2065),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2065),
    [sym_duet_dev_type_hvac] = ACTIONS(2065),
    [sym_duet_dev_type_keypad] = ACTIONS(2065),
    [sym_duet_dev_type_light] = ACTIONS(2065),
    [sym_duet_dev_type_monitor] = ACTIONS(2065),
    [sym_duet_dev_type_motor] = ACTIONS(2065),
    [sym_duet_dev_type_multi_window] = ACTIONS(2065),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2065),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2065),
    [sym_duet_dev_type_receiver] = ACTIONS(2065),
    [sym_duet_dev_type_security_system] = ACTIONS(2065),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2065),
    [sym_duet_dev_type_settop_box] = ACTIONS(2065),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2065),
    [sym_duet_dev_type_switcher] = ACTIONS(2065),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2065),
    [sym_duet_dev_type_tv] = ACTIONS(2065),
    [sym_duet_dev_type_utility] = ACTIONS(2065),
    [sym_duet_dev_type_vcr] = ACTIONS(2065),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2065),
    [sym_duet_dev_type_video_processor] = ACTIONS(2065),
    [sym_duet_dev_type_video_projector] = ACTIONS(2065),
    [sym_duet_dev_type_video_wall] = ACTIONS(2065),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2065),
    [sym_duet_dev_type_weather] = ACTIONS(2065),
    [sym_duet_dev_not_polled] = ACTIONS(2065),
    [sym_duet_dev_polled] = ACTIONS(2065),
    [sym_clkmgr_mode_network] = ACTIONS(2065),
    [sym_clkmgr_mode_standalone] = ACTIONS(2065),
    [sym_normal_standby] = ACTIONS(2065),
    [sym_normal_wake] = ACTIONS(2065),
    [sym_ip_multicast_ttl_option] = ACTIONS(2065),
    [sym_ip_ttl_subnet] = ACTIONS(2065),
    [sym_ip_ttl_site] = ACTIONS(2065),
    [sym_ip_ttl_region] = ACTIONS(2065),
    [sym_ip_ttl_continent] = ACTIONS(2065),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2065),
    [sym_ip_nodelay_on] = ACTIONS(2065),
    [sym_ip_nodelay_off] = ACTIONS(2065),
    [sym_amx_error] = ACTIONS(2065),
    [sym_amx_warning] = ACTIONS(2065),
    [sym_amx_info] = ACTIONS(2065),
    [sym_amx_debug] = ACTIONS(2065),
    [sym_netlinx_login_success] = ACTIONS(2065),
    [sym_netlinx_login_fail] = ACTIONS(2065),
    [sym_netlinx_logout] = ACTIONS(2065),
    [sym_null_str] = ACTIONS(2065),
    [sym_smtp_address] = ACTIONS(2065),
    [sym_smtp_port_number] = ACTIONS(2065),
    [sym_smtp_username] = ACTIONS(2065),
    [sym_smtp_password] = ACTIONS(2065),
    [sym_smtp_require_tls] = ACTIONS(2065),
    [sym_smtp_from] = ACTIONS(2065),
    [sym_smtp_tls_true] = ACTIONS(2065),
    [sym_smtp_tls_false] = ACTIONS(2065),
    [sym_listview_on_row_select_event] = ACTIONS(2065),
    [sym_data_structure_unknown] = ACTIONS(2065),
    [sym_data_structure_datafeed] = ACTIONS(2065),
    [sym_data_structure_datarecordset] = ACTIONS(2065),
    [sym_data_structure_datarecord] = ACTIONS(2065),
    [sym_data_structure_datafield] = ACTIONS(2065),
    [sym_data_max_content_field_count] = ACTIONS(2065),
    [sym_data_max_metadata_field_count] = ACTIONS(2065),
    [sym_data_max_id_length] = ACTIONS(2065),
    [sym_data_max_name_length] = ACTIONS(2065),
    [sym_data_max_description_length] = ACTIONS(2065),
    [sym_data_max_type_length] = ACTIONS(2065),
    [sym_data_max_format_length] = ACTIONS(2065),
    [sym_data_max_source_length] = ACTIONS(2065),
    [sym_data_max_value_length] = ACTIONS(2065),
    [sym_data_max_label_length] = ACTIONS(2065),
    [sym_data_max_url_length] = ACTIONS(2065),
    [sym_max_last_login_info_length] = ACTIONS(2065),
    [sym_data_type_unknown] = ACTIONS(2065),
    [sym_data_type_string] = ACTIONS(2065),
    [sym_data_type_datetime] = ACTIONS(2065),
    [sym_data_type_date] = ACTIONS(2065),
    [sym_data_type_time] = ACTIONS(2065),
    [sym_data_type_image] = ACTIONS(2065),
    [sym_data_format_url] = ACTIONS(2065),
    [sym_data_format_phone] = ACTIONS(2065),
    [sym_data_format_email] = ACTIONS(2065),
    [sym_data_format_iso8601] = ACTIONS(2065),
    [sym_auditlog_priv] = ACTIONS(2065),
    [sym_configuration_priv] = ACTIONS(2065),
    [sym_deviceconfig_priv] = ACTIONS(2065),
    [sym_ftp_priv] = ACTIONS(2065),
    [sym_http_priv] = ACTIONS(2065),
    [sym_networkconfig_priv] = ACTIONS(2065),
    [sym_programport_priv] = ACTIONS(2065),
    [sym_remoteui_priv] = ACTIONS(2065),
    [sym_securitycontrol_priv] = ACTIONS(2065),
    [sym_softwaremanagement_priv] = ACTIONS(2065),
    [sym_terminal_priv] = ACTIONS(2065),
    [sym_tpadmin_priv] = ACTIONS(2065),
    [sym_user1_priv] = ACTIONS(2065),
    [sym_user2_priv] = ACTIONS(2065),
    [sym_user3_priv] = ACTIONS(2065),
    [sym_user4_priv] = ACTIONS(2065),
    [sym_usermanagement_priv] = ACTIONS(2065),
    [sym_valid_account] = ACTIONS(2065),
    [sym_err_name_invalid] = ACTIONS(2065),
    [sym_err_password_invalid] = ACTIONS(2065),
    [sym_err_invalid_account] = ACTIONS(2065),
    [sym_err_no_authorization] = ACTIONS(2065),
    [sym_err_invalid_parameter] = ACTIONS(2065),
    [sym_err_user_locked_out] = ACTIONS(2065),
    [sym_err_user_account_expired] = ACTIONS(2065),
    [sym_err_general_error] = ACTIONS(2065),
    [sym_err_authentication_error] = ACTIONS(2065),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2065),
    [sym_tls_validate_certificate] = ACTIONS(2065),
    [sym_tdata] = ACTIONS(2065),
    [sym_tchannel] = ACTIONS(2065),
    [sym_tlevel] = ACTIONS(2065),
    [sym_tbutton] = ACTIONS(2065),
    [sym_ttimeline] = ACTIONS(2065),
    [sym_tcustom] = ACTIONS(2065),
    [sym_url_struct] = ACTIONS(2065),
    [sym_dns_struct] = ACTIONS(2065),
    [sym_ip_address_struct] = ACTIONS(2065),
    [sym_dev_info_struct] = ACTIONS(2065),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2065),
    [sym_data_feed] = ACTIONS(2065),
    [sym_data_field] = ACTIONS(2065),
    [sym_data_record] = ACTIONS(2065),
    [sym_wc_data_feed] = ACTIONS(2065),
    [sym_wc_data_field] = ACTIONS(2065),
    [sym_wc_data_record] = ACTIONS(2065),
    [sym_last_login_info] = ACTIONS(2065),
    [sym____reserved___] = ACTIONS(2065),
    [sym_date] = ACTIONS(2065),
    [sym_day] = ACTIONS(2065),
    [sym_time] = ACTIONS(2065),
    [sym_ldate] = ACTIONS(2065),
    [sym_push_device] = ACTIONS(2065),
    [sym_push_channel] = ACTIONS(2065),
    [sym_push_devchan] = ACTIONS(2065),
    [sym_release_device] = ACTIONS(2065),
    [sym_release_channel] = ACTIONS(2065),
    [sym_release_devchan] = ACTIONS(2065),
    [sym_master_slot] = ACTIONS(2065),
    [sym_get_pulse_time] = ACTIONS(2065),
    [sym_get_timer] = ACTIONS(2065),
    [sym_master_sn] = ACTIONS(2065),
    [sym_system_number] = ACTIONS(2065),
    [sym_button] = ACTIONS(2065),
    [sym_dv_channel] = ACTIONS(2065),
    [sym_data] = ACTIONS(2065),
    [sym_level] = ACTIONS(2065),
    [sym_timeline] = ACTIONS(2065),
    [sym_channel] = ACTIONS(2065),
    [sym_custom] = ACTIONS(2065),
    [sym_atoi] = ACTIONS(2065),
    [sym_atol] = ACTIONS(2065),
    [sym_atof] = ACTIONS(2065),
    [sym_itoa] = ACTIONS(2065),
    [sym_itohex] = ACTIONS(2065),
    [sym_hextoi] = ACTIONS(2065),
    [sym_ftoa] = ACTIONS(2065),
    [sym_find_string] = ACTIONS(2065),
    [sym_left_string] = ACTIONS(2065),
    [sym_right_string] = ACTIONS(2065),
    [sym_mid_string] = ACTIONS(2065),
    [sym_remove_string] = ACTIONS(2065),
    [sym_get_buffer_string] = ACTIONS(2065),
    [sym_get_buffer_char] = ACTIONS(2065),
    [sym_get_multi_buffer_string] = ACTIONS(2065),
    [sym_length_string] = ACTIONS(2065),
    [sym_max_length_string] = ACTIONS(2065),
    [sym_set_length_string] = ACTIONS(2065),
    [sym_compare_string] = ACTIONS(2065),
    [sym_upper_string] = ACTIONS(2065),
    [sym_lower_string] = ACTIONS(2065),
    [sym_get_last] = ACTIONS(2065),
    [sym_date_to_day] = ACTIONS(2065),
    [sym_date_to_month] = ACTIONS(2065),
    [sym_date_to_year] = ACTIONS(2065),
    [sym_time_to_hour] = ACTIONS(2065),
    [sym_time_to_minute] = ACTIONS(2065),
    [sym_time_to_second] = ACTIONS(2065),
    [sym_day_of_week] = ACTIONS(2065),
    [sym_set_pulse_time] = ACTIONS(2065),
    [sym_set_timer] = ACTIONS(2065),
    [sym_file_open] = ACTIONS(2065),
    [sym_file_close] = ACTIONS(2065),
    [sym_file_read] = ACTIONS(2065),
    [sym_file_read_line] = ACTIONS(2065),
    [sym_file_write] = ACTIONS(2065),
    [sym_file_write_line] = ACTIONS(2065),
    [sym_file_copy] = ACTIONS(2065),
    [sym_file_delete] = ACTIONS(2065),
    [sym_file_rename] = ACTIONS(2065),
    [sym_file_dir] = ACTIONS(2065),
    [sym_file_getdir] = ACTIONS(2065),
    [sym_file_setdir] = ACTIONS(2065),
    [sym_file_create_dir] = ACTIONS(2065),
    [sym_file_remove_dir] = ACTIONS(2065),
    [sym_file_seek] = ACTIONS(2065),
    [sym_length_array] = ACTIONS(2065),
    [sym_max_length_array] = ACTIONS(2065),
    [sym_set_length_array] = ACTIONS(2065),
    [sym_ip_client_open] = ACTIONS(2065),
    [sym_ip_client_close] = ACTIONS(2065),
    [sym_ip_server_open] = ACTIONS(2065),
    [sym_ip_server_close] = ACTIONS(2065),
    [sym_ip_bound_client_open] = ACTIONS(2065),
    [sym_ip_mc_server_open] = ACTIONS(2065),
    [sym_ip_set_option] = ACTIONS(2065),
    [sym_ssh_client_open] = ACTIONS(2065),
    [sym_ssh_client_close] = ACTIONS(2065),
    [sym_tls_client_open] = ACTIONS(2065),
    [sym_tls_client_close] = ACTIONS(2065),
    [sym_do_push] = ACTIONS(2065),
    [sym_do_push_timed] = ACTIONS(2065),
    [sym_do_release] = ACTIONS(2065),
    [sym_device_id] = ACTIONS(2065),
    [sym_device_info] = ACTIONS(2065),
    [sym_device_id_string] = ACTIONS(2065),
    [sym_device_standby] = ACTIONS(2065),
    [sym_device_wake] = ACTIONS(2065),
    [sym_timeline_create] = ACTIONS(2065),
    [sym_timeline_kill] = ACTIONS(2065),
    [sym_timeline_reload] = ACTIONS(2065),
    [sym_timeline_pause] = ACTIONS(2065),
    [sym_timeline_restart] = ACTIONS(2065),
    [sym_timeline_set] = ACTIONS(2065),
    [sym_timeline_get] = ACTIONS(2065),
    [sym_timeline_active] = ACTIONS(2065),
    [sym_timeline_dynamic_id] = ACTIONS(2065),
    [sym_variable_to_string] = ACTIONS(2065),
    [sym_string_to_variable] = ACTIONS(2065),
    [sym_length_variable_to_string] = ACTIONS(2065),
    [sym_type_cast] = ACTIONS(2065),
    [sym_variable_to_xml] = ACTIONS(2065),
    [sym_xml_to_variable] = ACTIONS(2065),
    [sym_length_variable_to_xml] = ACTIONS(2065),
    [sym_redirect_string] = ACTIONS(2065),
    [sym_random_number] = ACTIONS(2065),
    [sym_get_unique_id] = ACTIONS(2065),
    [sym_get_system_number] = ACTIONS(2065),
    [sym_set_system_number] = ACTIONS(2065),
    [sym_get_serial_number] = ACTIONS(2065),
    [sym_get_master_build] = ACTIONS(2065),
    [sym_set_validation_code] = ACTIONS(2065),
    [sym_reboot] = ACTIONS(2065),
    [sym_load_duet_module] = ACTIONS(2065),
    [sym_unload_duet_module] = ACTIONS(2065),
    [sym_duet_mem_size_set] = ACTIONS(2065),
    [sym_duet_mem_size_get] = ACTIONS(2065),
    [sym_dynamic_application_device] = ACTIONS(2065),
    [sym_dynamic_polled_port] = ACTIONS(2065),
    [sym_static_port_binding] = ACTIONS(2065),
    [sym_static_ip_binding] = ACTIONS(2065),
    [sym_get_dns_list] = ACTIONS(2065),
    [sym_set_dns_list] = ACTIONS(2065),
    [sym_get_ip_address] = ACTIONS(2065),
    [sym_set_ip_address] = ACTIONS(2065),
    [sym_get_url_list] = ACTIONS(2065),
    [sym_add_url_entry] = ACTIONS(2065),
    [sym_delete_url_entry] = ACTIONS(2065),
    [sym_abs_value] = ACTIONS(2065),
    [sym_max_value] = ACTIONS(2065),
    [sym_min_value] = ACTIONS(2065),
    [sym_exp_value] = ACTIONS(2065),
    [sym_log_value] = ACTIONS(2065),
    [sym_log10_value] = ACTIONS(2065),
    [sym_power_value] = ACTIONS(2065),
    [sym_sqrt_value] = ACTIONS(2065),
    [sym_raw_be] = ACTIONS(2065),
    [sym_raw_le] = ACTIONS(2065),
    [sym_internal_queue_size_set] = ACTIONS(2065),
    [sym_internal_queue_size_get] = ACTIONS(2065),
    [sym_internal_threshold_set] = ACTIONS(2065),
    [sym_internal_threshold_get] = ACTIONS(2065),
    [sym_rebuilt_event] = ACTIONS(2065),
    [sym_do_custom_event] = ACTIONS(2065),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2065),
    [sym_clkmgr_set_clk_source] = ACTIONS(2065),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2065),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2065),
    [sym_clkmgr_get_timezone] = ACTIONS(2065),
    [sym_clkmgr_set_timezone] = ACTIONS(2065),
    [sym_clkmgr_get_resync_period] = ACTIONS(2065),
    [sym_clkmgr_set_resync_period] = ACTIONS(2065),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2065),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2065),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2065),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2065),
    [sym_clkmgr_get_timeservers] = ACTIONS(2065),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2065),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2065),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2065),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2065),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2065),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2065),
    [sym_set_log_level] = ACTIONS(2065),
    [sym_get_log_level] = ACTIONS(2065),
    [sym_amx_log] = ACTIONS(2065),
    [sym_smtp_server_config_set] = ACTIONS(2065),
    [sym_smtp_server_config_get] = ACTIONS(2065),
    [sym_smtp_send] = ACTIONS(2065),
    [sym_validate_netlinx_account] = ACTIONS(2065),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2065),
    [sym_authenticate_certificate] = ACTIONS(2065),
    [sym_audit_netlinx_session_event] = ACTIONS(2065),
    [sym_audit_netlinx_generic_event] = ACTIONS(2065),
    [sym_get_available_flash_disk_space] = ACTIONS(2065),
    [sym_get_max_flash_disk_space] = ACTIONS(2065),
    [sym_data_create_feed] = ACTIONS(2065),
    [sym_data_delete_feed] = ACTIONS(2065),
    [sym_data_publish_feed] = ACTIONS(2065),
    [sym_data_get_published_feed] = ACTIONS(2065),
    [sym_data_add_record] = ACTIONS(2065),
    [sym_data_get_event_record] = ACTIONS(2065),
    [sym__wc_data_create_feed] = ACTIONS(2065),
    [sym__wc_data_add_record] = ACTIONS(2065),
    [sym__wc_data_get_event_record] = ACTIONS(2065),
    [sym_format] = ACTIONS(2065),
    [sym_astro_clock] = ACTIONS(2065),
    [sym_set_outdoor_temperature] = ACTIONS(2065),
    [sym_set_virtual_level_count] = ACTIONS(2065),
    [sym_set_virtual_channel_count] = ACTIONS(2065),
    [sym_set_virtual_port_count] = ACTIONS(2065),
    [sym__wc_encode_int] = ACTIONS(2065),
    [sym__wc_decode_int] = ACTIONS(2065),
    [sym__wc_int] = ACTIONS(2065),
    [sym__wc_to_ch_int] = ACTIONS(2065),
    [sym__ch_to_wc_int] = ACTIONS(2065),
    [sym__wc_find_string_int] = ACTIONS(2065),
    [sym__wc_left_string_int] = ACTIONS(2065),
    [sym__wc_lower_string_int] = ACTIONS(2065),
    [sym__wc_mid_string_int] = ACTIONS(2065),
    [sym__wc_remove_string_int] = ACTIONS(2065),
    [sym__wc_right_string_int] = ACTIONS(2065),
    [sym__wc_upper_string_int] = ACTIONS(2065),
    [sym__wc_compare_string_int] = ACTIONS(2065),
    [sym__wc_get_buffer_char_int] = ACTIONS(2065),
    [sym__wc_get_buffer_string_int] = ACTIONS(2065),
    [sym__wc_concat_string_int] = ACTIONS(2065),
    [sym__wc_file_open_int] = ACTIONS(2065),
    [sym__wc_file_close_int] = ACTIONS(2065),
    [sym__wc_file_read_int] = ACTIONS(2065),
    [sym__wc_file_read_line_int] = ACTIONS(2065),
    [sym__wc_file_write_int] = ACTIONS(2065),
    [sym__wc_file_write_line_int] = ACTIONS(2065),
    [aux_sym_preproc_include_token1] = ACTIONS(2065),
    [aux_sym_preproc_define_token1] = ACTIONS(2067),
    [aux_sym_preproc_warn_token1] = ACTIONS(2067),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2067),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2067),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2067),
    [anon_sym_LBRACK] = ACTIONS(2067),
    [anon_sym_LBRACE] = ACTIONS(2067),
    [anon_sym_LPAREN] = ACTIONS(2065),
    [anon_sym_BANG] = ACTIONS(2067),
    [anon_sym_TILDE] = ACTIONS(2067),
    [anon_sym_DASH] = ACTIONS(2065),
    [anon_sym_PLUS] = ACTIONS(2065),
    [anon_sym_DASH_DASH] = ACTIONS(2067),
    [anon_sym_PLUS_PLUS] = ACTIONS(2067),
    [anon_sym_DQUOTE] = ACTIONS(2067),
    [anon_sym_SQUOTE] = ACTIONS(2067),
    [sym_number_literal] = ACTIONS(2067),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(407)] = {
    [ts_builtin_sym_end] = ACTIONS(2095),
    [sym_identifier] = ACTIONS(2093),
    [sym_program_name_keyword] = ACTIONS(2093),
    [sym_module_name_keyword] = ACTIONS(2093),
    [sym_define_device_keyword] = ACTIONS(2093),
    [sym_define_combine_keyword] = ACTIONS(2093),
    [sym_define_constant_keyword] = ACTIONS(2093),
    [sym_define_type_keyword] = ACTIONS(2093),
    [sym_define_variable_keyword] = ACTIONS(2093),
    [sym_define_system_variable_keyword] = ACTIONS(2093),
    [sym_define_connect_level_keyword] = ACTIONS(2093),
    [sym_define_latching_keyword] = ACTIONS(2093),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2093),
    [sym_define_toggling_keyword] = ACTIONS(2093),
    [sym_define_call_keyword] = ACTIONS(2093),
    [sym_define_function_keyword] = ACTIONS(2093),
    [sym_define_library_function_keyword] = ACTIONS(2093),
    [sym_define_start_keyword] = ACTIONS(2093),
    [sym_define_event_keyword] = ACTIONS(2093),
    [sym_define_program_keyword] = ACTIONS(2093),
    [sym_define_module_keyword] = ACTIONS(2093),
    [sym_char_keyword] = ACTIONS(2093),
    [sym_widechar_keyword] = ACTIONS(2093),
    [sym_integer_keyword] = ACTIONS(2093),
    [sym_sinteger_keyword] = ACTIONS(2093),
    [sym_long_keyword] = ACTIONS(2093),
    [sym_slong_keyword] = ACTIONS(2093),
    [sym_float_keyword] = ACTIONS(2093),
    [sym_double_keyword] = ACTIONS(2093),
    [sym_variant_keyword] = ACTIONS(2093),
    [sym_variantarray_keyword] = ACTIONS(2093),
    [sym_local_var_keyword] = ACTIONS(2093),
    [sym_stack_var_keyword] = ACTIONS(2093),
    [sym_constant_keyword] = ACTIONS(2093),
    [sym_volatile_keyword] = ACTIONS(2093),
    [sym_non_volatile_keyword] = ACTIONS(2093),
    [sym_persistent_keyword] = ACTIONS(2093),
    [sym_dev_keyword] = ACTIONS(2093),
    [sym_devlev_keyword] = ACTIONS(2093),
    [sym_devchan_keyword] = ACTIONS(2093),
    [sym_if_keyword] = ACTIONS(2093),
    [sym_switch_keyword] = ACTIONS(2093),
    [sym_case_keyword] = ACTIONS(2093),
    [sym_default_keyword] = ACTIONS(2093),
    [sym_while_keyword] = ACTIONS(2093),
    [sym_for_keyword] = ACTIONS(2093),
    [sym_break_keyword] = ACTIONS(2093),
    [sym_continue_keyword] = ACTIONS(2093),
    [sym_return_keyword] = ACTIONS(2093),
    [sym_select_keyword] = ACTIONS(2093),
    [sym_active_keyword] = ACTIONS(2093),
    [sym_struct_keyword] = ACTIONS(2093),
    [sym_structure_keyword] = ACTIONS(2093),
    [sym_button_event_keyword] = ACTIONS(2093),
    [sym_channel_event_keyword] = ACTIONS(2093),
    [sym_level_event_keyword] = ACTIONS(2093),
    [sym_data_event_keyword] = ACTIONS(2093),
    [sym_timeline_event_keyword] = ACTIONS(2093),
    [sym_custom_event_keyword] = ACTIONS(2093),
    [sym_push_keyword] = ACTIONS(2093),
    [sym_release_keyword] = ACTIONS(2093),
    [aux_sym_on_keyword_token1] = ACTIONS(2093),
    [aux_sym_off_keyword_token1] = ACTIONS(2093),
    [sym_send_level_keyword] = ACTIONS(2093),
    [sym_send_string_keyword] = ACTIONS(2093),
    [sym_send_command_keyword] = ACTIONS(2093),
    [sym_clear_buffer_keyword] = ACTIONS(2093),
    [sym_create_buffer_keyword] = ACTIONS(2093),
    [sym_create_multi_buffer_keyword] = ACTIONS(2093),
    [sym_call_keyword] = ACTIONS(2093),
    [sym_system_call_keyword] = ACTIONS(2093),
    [sym_devchan_to_keyword] = ACTIONS(2093),
    [sym_devchan_min_to_keyword] = ACTIONS(2093),
    [sym_devchan_total_off_keyword] = ACTIONS(2093),
    [sym_devchan_pulse_keyword] = ACTIONS(2093),
    [sym_wait_keyword] = ACTIONS(2093),
    [sym_cancel_wait_keyword] = ACTIONS(2093),
    [sym_wait_until_keyword] = ACTIONS(2093),
    [sym_cancel_wait_until_keyword] = ACTIONS(2093),
    [sym_cancel_all_wait_keyword] = ACTIONS(2093),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2093),
    [sym_not] = ACTIONS(2093),
    [sym_bnot] = ACTIONS(2093),
    [sym___file__] = ACTIONS(2093),
    [sym___date__] = ACTIONS(2093),
    [sym___ldate__] = ACTIONS(2093),
    [sym___time__] = ACTIONS(2093),
    [sym___line__] = ACTIONS(2093),
    [sym___name__] = ACTIONS(2093),
    [sym___netlinx__] = ACTIONS(2093),
    [sym_netlinx_axi_version] = ACTIONS(2093),
    [sym_url_flg_tcp] = ACTIONS(2093),
    [sym_url_flg_accinfopresent] = ACTIONS(2093),
    [sym_url_flg_temp] = ACTIONS(2093),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2093),
    [sym_url_flg_stat_mask] = ACTIONS(2093),
    [sym_url_flg_stat_lookup] = ACTIONS(2093),
    [sym_url_flg_stat_connecting] = ACTIONS(2093),
    [sym_url_flg_stat_waiting] = ACTIONS(2093),
    [sym_url_flg_stat_connected] = ACTIONS(2093),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2093),
    [sym_first_virtual_device] = ACTIONS(2093),
    [sym_first_local_port] = ACTIONS(2093),
    [sym_dynamic_virtual_device] = ACTIONS(2093),
    [sym_timeline_once] = ACTIONS(2093),
    [sym_timeline_repeat] = ACTIONS(2093),
    [sym_timeline_absolute] = ACTIONS(2093),
    [sym_timeline_relative] = ACTIONS(2093),
    [sym_do_push_timed_infinite] = ACTIONS(2093),
    [sym_source_type_no_address] = ACTIONS(2093),
    [sym_source_type_neuron_id] = ACTIONS(2093),
    [sym_source_type_ip_address] = ACTIONS(2093),
    [sym_source_type_axlink] = ACTIONS(2093),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2093),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2093),
    [sym_source_type_ip_socket_address] = ACTIONS(2093),
    [sym_source_type_rs232] = ACTIONS(2093),
    [sym_source_type_internal] = ACTIONS(2093),
    [sym_source_type_mac_address] = ACTIONS(2093),
    [sym_source_type_ipv4_port] = ACTIONS(2093),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2093),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2093),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2093),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2093),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2093),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2093),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2093),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2093),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2093),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2093),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2093),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2093),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2093),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2093),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2093),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2093),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2093),
    [sym_true] = ACTIONS(2093),
    [sym_false] = ACTIONS(2093),
    [sym_file_read_only] = ACTIONS(2093),
    [sym_file_rw_new] = ACTIONS(2093),
    [sym_file_rw_append] = ACTIONS(2093),
    [sym_ip_tcp] = ACTIONS(2093),
    [sym_ip_udp] = ACTIONS(2093),
    [sym_ip_udp_2way] = ACTIONS(2093),
    [sym_xml_encode_types] = ACTIONS(2093),
    [sym_xml_encode_char_as_list] = ACTIONS(2093),
    [sym_xml_encode_le] = ACTIONS(2093),
    [sym_xml_decode_types] = ACTIONS(2093),
    [sym_xml_decode_no_preserve] = ACTIONS(2093),
    [sym_duet_dev_type_display_device] = ACTIONS(2093),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2093),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2093),
    [sym_duet_dev_type_io_device] = ACTIONS(2093),
    [sym_duet_dev_type_relay_device] = ACTIONS(2093),
    [sym_duet_dev_type_ups] = ACTIONS(2093),
    [sym_duet_dev_type_amplifier] = ACTIONS(2093),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2093),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2093),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2093),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2093),
    [sym_duet_dev_type_camera] = ACTIONS(2093),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2093),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2093),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2093),
    [sym_duet_dev_type_dss] = ACTIONS(2093),
    [sym_duet_dev_type_dvr] = ACTIONS(2093),
    [sym_duet_dev_type_disc_device] = ACTIONS(2093),
    [sym_duet_dev_type_document_camera] = ACTIONS(2093),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2093),
    [sym_duet_dev_type_hvac] = ACTIONS(2093),
    [sym_duet_dev_type_keypad] = ACTIONS(2093),
    [sym_duet_dev_type_light] = ACTIONS(2093),
    [sym_duet_dev_type_monitor] = ACTIONS(2093),
    [sym_duet_dev_type_motor] = ACTIONS(2093),
    [sym_duet_dev_type_multi_window] = ACTIONS(2093),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2093),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2093),
    [sym_duet_dev_type_receiver] = ACTIONS(2093),
    [sym_duet_dev_type_security_system] = ACTIONS(2093),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2093),
    [sym_duet_dev_type_settop_box] = ACTIONS(2093),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2093),
    [sym_duet_dev_type_switcher] = ACTIONS(2093),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2093),
    [sym_duet_dev_type_tv] = ACTIONS(2093),
    [sym_duet_dev_type_utility] = ACTIONS(2093),
    [sym_duet_dev_type_vcr] = ACTIONS(2093),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2093),
    [sym_duet_dev_type_video_processor] = ACTIONS(2093),
    [sym_duet_dev_type_video_projector] = ACTIONS(2093),
    [sym_duet_dev_type_video_wall] = ACTIONS(2093),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2093),
    [sym_duet_dev_type_weather] = ACTIONS(2093),
    [sym_duet_dev_not_polled] = ACTIONS(2093),
    [sym_duet_dev_polled] = ACTIONS(2093),
    [sym_clkmgr_mode_network] = ACTIONS(2093),
    [sym_clkmgr_mode_standalone] = ACTIONS(2093),
    [sym_normal_standby] = ACTIONS(2093),
    [sym_normal_wake] = ACTIONS(2093),
    [sym_ip_multicast_ttl_option] = ACTIONS(2093),
    [sym_ip_ttl_subnet] = ACTIONS(2093),
    [sym_ip_ttl_site] = ACTIONS(2093),
    [sym_ip_ttl_region] = ACTIONS(2093),
    [sym_ip_ttl_continent] = ACTIONS(2093),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2093),
    [sym_ip_nodelay_on] = ACTIONS(2093),
    [sym_ip_nodelay_off] = ACTIONS(2093),
    [sym_amx_error] = ACTIONS(2093),
    [sym_amx_warning] = ACTIONS(2093),
    [sym_amx_info] = ACTIONS(2093),
    [sym_amx_debug] = ACTIONS(2093),
    [sym_netlinx_login_success] = ACTIONS(2093),
    [sym_netlinx_login_fail] = ACTIONS(2093),
    [sym_netlinx_logout] = ACTIONS(2093),
    [sym_null_str] = ACTIONS(2093),
    [sym_smtp_address] = ACTIONS(2093),
    [sym_smtp_port_number] = ACTIONS(2093),
    [sym_smtp_username] = ACTIONS(2093),
    [sym_smtp_password] = ACTIONS(2093),
    [sym_smtp_require_tls] = ACTIONS(2093),
    [sym_smtp_from] = ACTIONS(2093),
    [sym_smtp_tls_true] = ACTIONS(2093),
    [sym_smtp_tls_false] = ACTIONS(2093),
    [sym_listview_on_row_select_event] = ACTIONS(2093),
    [sym_data_structure_unknown] = ACTIONS(2093),
    [sym_data_structure_datafeed] = ACTIONS(2093),
    [sym_data_structure_datarecordset] = ACTIONS(2093),
    [sym_data_structure_datarecord] = ACTIONS(2093),
    [sym_data_structure_datafield] = ACTIONS(2093),
    [sym_data_max_content_field_count] = ACTIONS(2093),
    [sym_data_max_metadata_field_count] = ACTIONS(2093),
    [sym_data_max_id_length] = ACTIONS(2093),
    [sym_data_max_name_length] = ACTIONS(2093),
    [sym_data_max_description_length] = ACTIONS(2093),
    [sym_data_max_type_length] = ACTIONS(2093),
    [sym_data_max_format_length] = ACTIONS(2093),
    [sym_data_max_source_length] = ACTIONS(2093),
    [sym_data_max_value_length] = ACTIONS(2093),
    [sym_data_max_label_length] = ACTIONS(2093),
    [sym_data_max_url_length] = ACTIONS(2093),
    [sym_max_last_login_info_length] = ACTIONS(2093),
    [sym_data_type_unknown] = ACTIONS(2093),
    [sym_data_type_string] = ACTIONS(2093),
    [sym_data_type_datetime] = ACTIONS(2093),
    [sym_data_type_date] = ACTIONS(2093),
    [sym_data_type_time] = ACTIONS(2093),
    [sym_data_type_image] = ACTIONS(2093),
    [sym_data_format_url] = ACTIONS(2093),
    [sym_data_format_phone] = ACTIONS(2093),
    [sym_data_format_email] = ACTIONS(2093),
    [sym_data_format_iso8601] = ACTIONS(2093),
    [sym_auditlog_priv] = ACTIONS(2093),
    [sym_configuration_priv] = ACTIONS(2093),
    [sym_deviceconfig_priv] = ACTIONS(2093),
    [sym_ftp_priv] = ACTIONS(2093),
    [sym_http_priv] = ACTIONS(2093),
    [sym_networkconfig_priv] = ACTIONS(2093),
    [sym_programport_priv] = ACTIONS(2093),
    [sym_remoteui_priv] = ACTIONS(2093),
    [sym_securitycontrol_priv] = ACTIONS(2093),
    [sym_softwaremanagement_priv] = ACTIONS(2093),
    [sym_terminal_priv] = ACTIONS(2093),
    [sym_tpadmin_priv] = ACTIONS(2093),
    [sym_user1_priv] = ACTIONS(2093),
    [sym_user2_priv] = ACTIONS(2093),
    [sym_user3_priv] = ACTIONS(2093),
    [sym_user4_priv] = ACTIONS(2093),
    [sym_usermanagement_priv] = ACTIONS(2093),
    [sym_valid_account] = ACTIONS(2093),
    [sym_err_name_invalid] = ACTIONS(2093),
    [sym_err_password_invalid] = ACTIONS(2093),
    [sym_err_invalid_account] = ACTIONS(2093),
    [sym_err_no_authorization] = ACTIONS(2093),
    [sym_err_invalid_parameter] = ACTIONS(2093),
    [sym_err_user_locked_out] = ACTIONS(2093),
    [sym_err_user_account_expired] = ACTIONS(2093),
    [sym_err_general_error] = ACTIONS(2093),
    [sym_err_authentication_error] = ACTIONS(2093),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2093),
    [sym_tls_validate_certificate] = ACTIONS(2093),
    [sym_tdata] = ACTIONS(2093),
    [sym_tchannel] = ACTIONS(2093),
    [sym_tlevel] = ACTIONS(2093),
    [sym_tbutton] = ACTIONS(2093),
    [sym_ttimeline] = ACTIONS(2093),
    [sym_tcustom] = ACTIONS(2093),
    [sym_url_struct] = ACTIONS(2093),
    [sym_dns_struct] = ACTIONS(2093),
    [sym_ip_address_struct] = ACTIONS(2093),
    [sym_dev_info_struct] = ACTIONS(2093),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2093),
    [sym_data_feed] = ACTIONS(2093),
    [sym_data_field] = ACTIONS(2093),
    [sym_data_record] = ACTIONS(2093),
    [sym_wc_data_feed] = ACTIONS(2093),
    [sym_wc_data_field] = ACTIONS(2093),
    [sym_wc_data_record] = ACTIONS(2093),
    [sym_last_login_info] = ACTIONS(2093),
    [sym____reserved___] = ACTIONS(2093),
    [sym_date] = ACTIONS(2093),
    [sym_day] = ACTIONS(2093),
    [sym_time] = ACTIONS(2093),
    [sym_ldate] = ACTIONS(2093),
    [sym_push_device] = ACTIONS(2093),
    [sym_push_channel] = ACTIONS(2093),
    [sym_push_devchan] = ACTIONS(2093),
    [sym_release_device] = ACTIONS(2093),
    [sym_release_channel] = ACTIONS(2093),
    [sym_release_devchan] = ACTIONS(2093),
    [sym_master_slot] = ACTIONS(2093),
    [sym_get_pulse_time] = ACTIONS(2093),
    [sym_get_timer] = ACTIONS(2093),
    [sym_master_sn] = ACTIONS(2093),
    [sym_system_number] = ACTIONS(2093),
    [sym_button] = ACTIONS(2093),
    [sym_dv_channel] = ACTIONS(2093),
    [sym_data] = ACTIONS(2093),
    [sym_level] = ACTIONS(2093),
    [sym_timeline] = ACTIONS(2093),
    [sym_channel] = ACTIONS(2093),
    [sym_custom] = ACTIONS(2093),
    [sym_atoi] = ACTIONS(2093),
    [sym_atol] = ACTIONS(2093),
    [sym_atof] = ACTIONS(2093),
    [sym_itoa] = ACTIONS(2093),
    [sym_itohex] = ACTIONS(2093),
    [sym_hextoi] = ACTIONS(2093),
    [sym_ftoa] = ACTIONS(2093),
    [sym_find_string] = ACTIONS(2093),
    [sym_left_string] = ACTIONS(2093),
    [sym_right_string] = ACTIONS(2093),
    [sym_mid_string] = ACTIONS(2093),
    [sym_remove_string] = ACTIONS(2093),
    [sym_get_buffer_string] = ACTIONS(2093),
    [sym_get_buffer_char] = ACTIONS(2093),
    [sym_get_multi_buffer_string] = ACTIONS(2093),
    [sym_length_string] = ACTIONS(2093),
    [sym_max_length_string] = ACTIONS(2093),
    [sym_set_length_string] = ACTIONS(2093),
    [sym_compare_string] = ACTIONS(2093),
    [sym_upper_string] = ACTIONS(2093),
    [sym_lower_string] = ACTIONS(2093),
    [sym_get_last] = ACTIONS(2093),
    [sym_date_to_day] = ACTIONS(2093),
    [sym_date_to_month] = ACTIONS(2093),
    [sym_date_to_year] = ACTIONS(2093),
    [sym_time_to_hour] = ACTIONS(2093),
    [sym_time_to_minute] = ACTIONS(2093),
    [sym_time_to_second] = ACTIONS(2093),
    [sym_day_of_week] = ACTIONS(2093),
    [sym_set_pulse_time] = ACTIONS(2093),
    [sym_set_timer] = ACTIONS(2093),
    [sym_file_open] = ACTIONS(2093),
    [sym_file_close] = ACTIONS(2093),
    [sym_file_read] = ACTIONS(2093),
    [sym_file_read_line] = ACTIONS(2093),
    [sym_file_write] = ACTIONS(2093),
    [sym_file_write_line] = ACTIONS(2093),
    [sym_file_copy] = ACTIONS(2093),
    [sym_file_delete] = ACTIONS(2093),
    [sym_file_rename] = ACTIONS(2093),
    [sym_file_dir] = ACTIONS(2093),
    [sym_file_getdir] = ACTIONS(2093),
    [sym_file_setdir] = ACTIONS(2093),
    [sym_file_create_dir] = ACTIONS(2093),
    [sym_file_remove_dir] = ACTIONS(2093),
    [sym_file_seek] = ACTIONS(2093),
    [sym_length_array] = ACTIONS(2093),
    [sym_max_length_array] = ACTIONS(2093),
    [sym_set_length_array] = ACTIONS(2093),
    [sym_ip_client_open] = ACTIONS(2093),
    [sym_ip_client_close] = ACTIONS(2093),
    [sym_ip_server_open] = ACTIONS(2093),
    [sym_ip_server_close] = ACTIONS(2093),
    [sym_ip_bound_client_open] = ACTIONS(2093),
    [sym_ip_mc_server_open] = ACTIONS(2093),
    [sym_ip_set_option] = ACTIONS(2093),
    [sym_ssh_client_open] = ACTIONS(2093),
    [sym_ssh_client_close] = ACTIONS(2093),
    [sym_tls_client_open] = ACTIONS(2093),
    [sym_tls_client_close] = ACTIONS(2093),
    [sym_do_push] = ACTIONS(2093),
    [sym_do_push_timed] = ACTIONS(2093),
    [sym_do_release] = ACTIONS(2093),
    [sym_device_id] = ACTIONS(2093),
    [sym_device_info] = ACTIONS(2093),
    [sym_device_id_string] = ACTIONS(2093),
    [sym_device_standby] = ACTIONS(2093),
    [sym_device_wake] = ACTIONS(2093),
    [sym_timeline_create] = ACTIONS(2093),
    [sym_timeline_kill] = ACTIONS(2093),
    [sym_timeline_reload] = ACTIONS(2093),
    [sym_timeline_pause] = ACTIONS(2093),
    [sym_timeline_restart] = ACTIONS(2093),
    [sym_timeline_set] = ACTIONS(2093),
    [sym_timeline_get] = ACTIONS(2093),
    [sym_timeline_active] = ACTIONS(2093),
    [sym_timeline_dynamic_id] = ACTIONS(2093),
    [sym_variable_to_string] = ACTIONS(2093),
    [sym_string_to_variable] = ACTIONS(2093),
    [sym_length_variable_to_string] = ACTIONS(2093),
    [sym_type_cast] = ACTIONS(2093),
    [sym_variable_to_xml] = ACTIONS(2093),
    [sym_xml_to_variable] = ACTIONS(2093),
    [sym_length_variable_to_xml] = ACTIONS(2093),
    [sym_redirect_string] = ACTIONS(2093),
    [sym_random_number] = ACTIONS(2093),
    [sym_get_unique_id] = ACTIONS(2093),
    [sym_get_system_number] = ACTIONS(2093),
    [sym_set_system_number] = ACTIONS(2093),
    [sym_get_serial_number] = ACTIONS(2093),
    [sym_get_master_build] = ACTIONS(2093),
    [sym_set_validation_code] = ACTIONS(2093),
    [sym_reboot] = ACTIONS(2093),
    [sym_load_duet_module] = ACTIONS(2093),
    [sym_unload_duet_module] = ACTIONS(2093),
    [sym_duet_mem_size_set] = ACTIONS(2093),
    [sym_duet_mem_size_get] = ACTIONS(2093),
    [sym_dynamic_application_device] = ACTIONS(2093),
    [sym_dynamic_polled_port] = ACTIONS(2093),
    [sym_static_port_binding] = ACTIONS(2093),
    [sym_static_ip_binding] = ACTIONS(2093),
    [sym_get_dns_list] = ACTIONS(2093),
    [sym_set_dns_list] = ACTIONS(2093),
    [sym_get_ip_address] = ACTIONS(2093),
    [sym_set_ip_address] = ACTIONS(2093),
    [sym_get_url_list] = ACTIONS(2093),
    [sym_add_url_entry] = ACTIONS(2093),
    [sym_delete_url_entry] = ACTIONS(2093),
    [sym_abs_value] = ACTIONS(2093),
    [sym_max_value] = ACTIONS(2093),
    [sym_min_value] = ACTIONS(2093),
    [sym_exp_value] = ACTIONS(2093),
    [sym_log_value] = ACTIONS(2093),
    [sym_log10_value] = ACTIONS(2093),
    [sym_power_value] = ACTIONS(2093),
    [sym_sqrt_value] = ACTIONS(2093),
    [sym_raw_be] = ACTIONS(2093),
    [sym_raw_le] = ACTIONS(2093),
    [sym_internal_queue_size_set] = ACTIONS(2093),
    [sym_internal_queue_size_get] = ACTIONS(2093),
    [sym_internal_threshold_set] = ACTIONS(2093),
    [sym_internal_threshold_get] = ACTIONS(2093),
    [sym_rebuilt_event] = ACTIONS(2093),
    [sym_do_custom_event] = ACTIONS(2093),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2093),
    [sym_clkmgr_set_clk_source] = ACTIONS(2093),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2093),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2093),
    [sym_clkmgr_get_timezone] = ACTIONS(2093),
    [sym_clkmgr_set_timezone] = ACTIONS(2093),
    [sym_clkmgr_get_resync_period] = ACTIONS(2093),
    [sym_clkmgr_set_resync_period] = ACTIONS(2093),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2093),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2093),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2093),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2093),
    [sym_clkmgr_get_timeservers] = ACTIONS(2093),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2093),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2093),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2093),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2093),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2093),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2093),
    [sym_set_log_level] = ACTIONS(2093),
    [sym_get_log_level] = ACTIONS(2093),
    [sym_amx_log] = ACTIONS(2093),
    [sym_smtp_server_config_set] = ACTIONS(2093),
    [sym_smtp_server_config_get] = ACTIONS(2093),
    [sym_smtp_send] = ACTIONS(2093),
    [sym_validate_netlinx_account] = ACTIONS(2093),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2093),
    [sym_authenticate_certificate] = ACTIONS(2093),
    [sym_audit_netlinx_session_event] = ACTIONS(2093),
    [sym_audit_netlinx_generic_event] = ACTIONS(2093),
    [sym_get_available_flash_disk_space] = ACTIONS(2093),
    [sym_get_max_flash_disk_space] = ACTIONS(2093),
    [sym_data_create_feed] = ACTIONS(2093),
    [sym_data_delete_feed] = ACTIONS(2093),
    [sym_data_publish_feed] = ACTIONS(2093),
    [sym_data_get_published_feed] = ACTIONS(2093),
    [sym_data_add_record] = ACTIONS(2093),
    [sym_data_get_event_record] = ACTIONS(2093),
    [sym__wc_data_create_feed] = ACTIONS(2093),
    [sym__wc_data_add_record] = ACTIONS(2093),
    [sym__wc_data_get_event_record] = ACTIONS(2093),
    [sym_format] = ACTIONS(2093),
    [sym_astro_clock] = ACTIONS(2093),
    [sym_set_outdoor_temperature] = ACTIONS(2093),
    [sym_set_virtual_level_count] = ACTIONS(2093),
    [sym_set_virtual_channel_count] = ACTIONS(2093),
    [sym_set_virtual_port_count] = ACTIONS(2093),
    [sym__wc_encode_int] = ACTIONS(2093),
    [sym__wc_decode_int] = ACTIONS(2093),
    [sym__wc_int] = ACTIONS(2093),
    [sym__wc_to_ch_int] = ACTIONS(2093),
    [sym__ch_to_wc_int] = ACTIONS(2093),
    [sym__wc_find_string_int] = ACTIONS(2093),
    [sym__wc_left_string_int] = ACTIONS(2093),
    [sym__wc_lower_string_int] = ACTIONS(2093),
    [sym__wc_mid_string_int] = ACTIONS(2093),
    [sym__wc_remove_string_int] = ACTIONS(2093),
    [sym__wc_right_string_int] = ACTIONS(2093),
    [sym__wc_upper_string_int] = ACTIONS(2093),
    [sym__wc_compare_string_int] = ACTIONS(2093),
    [sym__wc_get_buffer_char_int] = ACTIONS(2093),
    [sym__wc_get_buffer_string_int] = ACTIONS(2093),
    [sym__wc_concat_string_int] = ACTIONS(2093),
    [sym__wc_file_open_int] = ACTIONS(2093),
    [sym__wc_file_close_int] = ACTIONS(2093),
    [sym__wc_file_read_int] = ACTIONS(2093),
    [sym__wc_file_read_line_int] = ACTIONS(2093),
    [sym__wc_file_write_int] = ACTIONS(2093),
    [sym__wc_file_write_line_int] = ACTIONS(2093),
    [aux_sym_preproc_include_token1] = ACTIONS(2093),
    [aux_sym_preproc_define_token1] = ACTIONS(2095),
    [aux_sym_preproc_warn_token1] = ACTIONS(2095),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2095),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2095),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2095),
    [anon_sym_LBRACK] = ACTIONS(2095),
    [anon_sym_LBRACE] = ACTIONS(2095),
    [anon_sym_LPAREN] = ACTIONS(2093),
    [anon_sym_BANG] = ACTIONS(2095),
    [anon_sym_TILDE] = ACTIONS(2095),
    [anon_sym_DASH] = ACTIONS(2093),
    [anon_sym_PLUS] = ACTIONS(2093),
    [anon_sym_DASH_DASH] = ACTIONS(2095),
    [anon_sym_PLUS_PLUS] = ACTIONS(2095),
    [anon_sym_DQUOTE] = ACTIONS(2095),
    [anon_sym_SQUOTE] = ACTIONS(2095),
    [sym_number_literal] = ACTIONS(2095),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(408)] = {
    [ts_builtin_sym_end] = ACTIONS(1947),
    [sym_identifier] = ACTIONS(1945),
    [sym_program_name_keyword] = ACTIONS(1945),
    [sym_module_name_keyword] = ACTIONS(1945),
    [sym_define_device_keyword] = ACTIONS(1945),
    [sym_define_combine_keyword] = ACTIONS(1945),
    [sym_define_constant_keyword] = ACTIONS(1945),
    [sym_define_type_keyword] = ACTIONS(1945),
    [sym_define_variable_keyword] = ACTIONS(1945),
    [sym_define_system_variable_keyword] = ACTIONS(1945),
    [sym_define_connect_level_keyword] = ACTIONS(1945),
    [sym_define_latching_keyword] = ACTIONS(1945),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1945),
    [sym_define_toggling_keyword] = ACTIONS(1945),
    [sym_define_call_keyword] = ACTIONS(1945),
    [sym_define_function_keyword] = ACTIONS(1945),
    [sym_define_library_function_keyword] = ACTIONS(1945),
    [sym_define_start_keyword] = ACTIONS(1945),
    [sym_define_event_keyword] = ACTIONS(1945),
    [sym_define_program_keyword] = ACTIONS(1945),
    [sym_define_module_keyword] = ACTIONS(1945),
    [sym_char_keyword] = ACTIONS(1945),
    [sym_widechar_keyword] = ACTIONS(1945),
    [sym_integer_keyword] = ACTIONS(1945),
    [sym_sinteger_keyword] = ACTIONS(1945),
    [sym_long_keyword] = ACTIONS(1945),
    [sym_slong_keyword] = ACTIONS(1945),
    [sym_float_keyword] = ACTIONS(1945),
    [sym_double_keyword] = ACTIONS(1945),
    [sym_variant_keyword] = ACTIONS(1945),
    [sym_variantarray_keyword] = ACTIONS(1945),
    [sym_local_var_keyword] = ACTIONS(1945),
    [sym_stack_var_keyword] = ACTIONS(1945),
    [sym_constant_keyword] = ACTIONS(1945),
    [sym_volatile_keyword] = ACTIONS(1945),
    [sym_non_volatile_keyword] = ACTIONS(1945),
    [sym_persistent_keyword] = ACTIONS(1945),
    [sym_dev_keyword] = ACTIONS(1945),
    [sym_devlev_keyword] = ACTIONS(1945),
    [sym_devchan_keyword] = ACTIONS(1945),
    [sym_if_keyword] = ACTIONS(1945),
    [sym_switch_keyword] = ACTIONS(1945),
    [sym_case_keyword] = ACTIONS(1945),
    [sym_default_keyword] = ACTIONS(1945),
    [sym_while_keyword] = ACTIONS(1945),
    [sym_for_keyword] = ACTIONS(1945),
    [sym_break_keyword] = ACTIONS(1945),
    [sym_continue_keyword] = ACTIONS(1945),
    [sym_return_keyword] = ACTIONS(1945),
    [sym_select_keyword] = ACTIONS(1945),
    [sym_active_keyword] = ACTIONS(1945),
    [sym_struct_keyword] = ACTIONS(1945),
    [sym_structure_keyword] = ACTIONS(1945),
    [sym_button_event_keyword] = ACTIONS(1945),
    [sym_channel_event_keyword] = ACTIONS(1945),
    [sym_level_event_keyword] = ACTIONS(1945),
    [sym_data_event_keyword] = ACTIONS(1945),
    [sym_timeline_event_keyword] = ACTIONS(1945),
    [sym_custom_event_keyword] = ACTIONS(1945),
    [sym_push_keyword] = ACTIONS(1945),
    [sym_release_keyword] = ACTIONS(1945),
    [aux_sym_on_keyword_token1] = ACTIONS(1945),
    [aux_sym_off_keyword_token1] = ACTIONS(1945),
    [sym_send_level_keyword] = ACTIONS(1945),
    [sym_send_string_keyword] = ACTIONS(1945),
    [sym_send_command_keyword] = ACTIONS(1945),
    [sym_clear_buffer_keyword] = ACTIONS(1945),
    [sym_create_buffer_keyword] = ACTIONS(1945),
    [sym_create_multi_buffer_keyword] = ACTIONS(1945),
    [sym_call_keyword] = ACTIONS(1945),
    [sym_system_call_keyword] = ACTIONS(1945),
    [sym_devchan_to_keyword] = ACTIONS(1945),
    [sym_devchan_min_to_keyword] = ACTIONS(1945),
    [sym_devchan_total_off_keyword] = ACTIONS(1945),
    [sym_devchan_pulse_keyword] = ACTIONS(1945),
    [sym_wait_keyword] = ACTIONS(1945),
    [sym_cancel_wait_keyword] = ACTIONS(1945),
    [sym_wait_until_keyword] = ACTIONS(1945),
    [sym_cancel_wait_until_keyword] = ACTIONS(1945),
    [sym_cancel_all_wait_keyword] = ACTIONS(1945),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1945),
    [sym_not] = ACTIONS(1945),
    [sym_bnot] = ACTIONS(1945),
    [sym___file__] = ACTIONS(1945),
    [sym___date__] = ACTIONS(1945),
    [sym___ldate__] = ACTIONS(1945),
    [sym___time__] = ACTIONS(1945),
    [sym___line__] = ACTIONS(1945),
    [sym___name__] = ACTIONS(1945),
    [sym___netlinx__] = ACTIONS(1945),
    [sym_netlinx_axi_version] = ACTIONS(1945),
    [sym_url_flg_tcp] = ACTIONS(1945),
    [sym_url_flg_accinfopresent] = ACTIONS(1945),
    [sym_url_flg_temp] = ACTIONS(1945),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1945),
    [sym_url_flg_stat_mask] = ACTIONS(1945),
    [sym_url_flg_stat_lookup] = ACTIONS(1945),
    [sym_url_flg_stat_connecting] = ACTIONS(1945),
    [sym_url_flg_stat_waiting] = ACTIONS(1945),
    [sym_url_flg_stat_connected] = ACTIONS(1945),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1945),
    [sym_first_virtual_device] = ACTIONS(1945),
    [sym_first_local_port] = ACTIONS(1945),
    [sym_dynamic_virtual_device] = ACTIONS(1945),
    [sym_timeline_once] = ACTIONS(1945),
    [sym_timeline_repeat] = ACTIONS(1945),
    [sym_timeline_absolute] = ACTIONS(1945),
    [sym_timeline_relative] = ACTIONS(1945),
    [sym_do_push_timed_infinite] = ACTIONS(1945),
    [sym_source_type_no_address] = ACTIONS(1945),
    [sym_source_type_neuron_id] = ACTIONS(1945),
    [sym_source_type_ip_address] = ACTIONS(1945),
    [sym_source_type_axlink] = ACTIONS(1945),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1945),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1945),
    [sym_source_type_ip_socket_address] = ACTIONS(1945),
    [sym_source_type_rs232] = ACTIONS(1945),
    [sym_source_type_internal] = ACTIONS(1945),
    [sym_source_type_mac_address] = ACTIONS(1945),
    [sym_source_type_ipv4_port] = ACTIONS(1945),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1945),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1945),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1945),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1945),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1945),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1945),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1945),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1945),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1945),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1945),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1945),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1945),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1945),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1945),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1945),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1945),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1945),
    [sym_true] = ACTIONS(1945),
    [sym_false] = ACTIONS(1945),
    [sym_file_read_only] = ACTIONS(1945),
    [sym_file_rw_new] = ACTIONS(1945),
    [sym_file_rw_append] = ACTIONS(1945),
    [sym_ip_tcp] = ACTIONS(1945),
    [sym_ip_udp] = ACTIONS(1945),
    [sym_ip_udp_2way] = ACTIONS(1945),
    [sym_xml_encode_types] = ACTIONS(1945),
    [sym_xml_encode_char_as_list] = ACTIONS(1945),
    [sym_xml_encode_le] = ACTIONS(1945),
    [sym_xml_decode_types] = ACTIONS(1945),
    [sym_xml_decode_no_preserve] = ACTIONS(1945),
    [sym_duet_dev_type_display_device] = ACTIONS(1945),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1945),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1945),
    [sym_duet_dev_type_io_device] = ACTIONS(1945),
    [sym_duet_dev_type_relay_device] = ACTIONS(1945),
    [sym_duet_dev_type_ups] = ACTIONS(1945),
    [sym_duet_dev_type_amplifier] = ACTIONS(1945),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1945),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1945),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1945),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1945),
    [sym_duet_dev_type_camera] = ACTIONS(1945),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1945),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1945),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1945),
    [sym_duet_dev_type_dss] = ACTIONS(1945),
    [sym_duet_dev_type_dvr] = ACTIONS(1945),
    [sym_duet_dev_type_disc_device] = ACTIONS(1945),
    [sym_duet_dev_type_document_camera] = ACTIONS(1945),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1945),
    [sym_duet_dev_type_hvac] = ACTIONS(1945),
    [sym_duet_dev_type_keypad] = ACTIONS(1945),
    [sym_duet_dev_type_light] = ACTIONS(1945),
    [sym_duet_dev_type_monitor] = ACTIONS(1945),
    [sym_duet_dev_type_motor] = ACTIONS(1945),
    [sym_duet_dev_type_multi_window] = ACTIONS(1945),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1945),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1945),
    [sym_duet_dev_type_receiver] = ACTIONS(1945),
    [sym_duet_dev_type_security_system] = ACTIONS(1945),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1945),
    [sym_duet_dev_type_settop_box] = ACTIONS(1945),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1945),
    [sym_duet_dev_type_switcher] = ACTIONS(1945),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1945),
    [sym_duet_dev_type_tv] = ACTIONS(1945),
    [sym_duet_dev_type_utility] = ACTIONS(1945),
    [sym_duet_dev_type_vcr] = ACTIONS(1945),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1945),
    [sym_duet_dev_type_video_processor] = ACTIONS(1945),
    [sym_duet_dev_type_video_projector] = ACTIONS(1945),
    [sym_duet_dev_type_video_wall] = ACTIONS(1945),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1945),
    [sym_duet_dev_type_weather] = ACTIONS(1945),
    [sym_duet_dev_not_polled] = ACTIONS(1945),
    [sym_duet_dev_polled] = ACTIONS(1945),
    [sym_clkmgr_mode_network] = ACTIONS(1945),
    [sym_clkmgr_mode_standalone] = ACTIONS(1945),
    [sym_normal_standby] = ACTIONS(1945),
    [sym_normal_wake] = ACTIONS(1945),
    [sym_ip_multicast_ttl_option] = ACTIONS(1945),
    [sym_ip_ttl_subnet] = ACTIONS(1945),
    [sym_ip_ttl_site] = ACTIONS(1945),
    [sym_ip_ttl_region] = ACTIONS(1945),
    [sym_ip_ttl_continent] = ACTIONS(1945),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1945),
    [sym_ip_nodelay_on] = ACTIONS(1945),
    [sym_ip_nodelay_off] = ACTIONS(1945),
    [sym_amx_error] = ACTIONS(1945),
    [sym_amx_warning] = ACTIONS(1945),
    [sym_amx_info] = ACTIONS(1945),
    [sym_amx_debug] = ACTIONS(1945),
    [sym_netlinx_login_success] = ACTIONS(1945),
    [sym_netlinx_login_fail] = ACTIONS(1945),
    [sym_netlinx_logout] = ACTIONS(1945),
    [sym_null_str] = ACTIONS(1945),
    [sym_smtp_address] = ACTIONS(1945),
    [sym_smtp_port_number] = ACTIONS(1945),
    [sym_smtp_username] = ACTIONS(1945),
    [sym_smtp_password] = ACTIONS(1945),
    [sym_smtp_require_tls] = ACTIONS(1945),
    [sym_smtp_from] = ACTIONS(1945),
    [sym_smtp_tls_true] = ACTIONS(1945),
    [sym_smtp_tls_false] = ACTIONS(1945),
    [sym_listview_on_row_select_event] = ACTIONS(1945),
    [sym_data_structure_unknown] = ACTIONS(1945),
    [sym_data_structure_datafeed] = ACTIONS(1945),
    [sym_data_structure_datarecordset] = ACTIONS(1945),
    [sym_data_structure_datarecord] = ACTIONS(1945),
    [sym_data_structure_datafield] = ACTIONS(1945),
    [sym_data_max_content_field_count] = ACTIONS(1945),
    [sym_data_max_metadata_field_count] = ACTIONS(1945),
    [sym_data_max_id_length] = ACTIONS(1945),
    [sym_data_max_name_length] = ACTIONS(1945),
    [sym_data_max_description_length] = ACTIONS(1945),
    [sym_data_max_type_length] = ACTIONS(1945),
    [sym_data_max_format_length] = ACTIONS(1945),
    [sym_data_max_source_length] = ACTIONS(1945),
    [sym_data_max_value_length] = ACTIONS(1945),
    [sym_data_max_label_length] = ACTIONS(1945),
    [sym_data_max_url_length] = ACTIONS(1945),
    [sym_max_last_login_info_length] = ACTIONS(1945),
    [sym_data_type_unknown] = ACTIONS(1945),
    [sym_data_type_string] = ACTIONS(1945),
    [sym_data_type_datetime] = ACTIONS(1945),
    [sym_data_type_date] = ACTIONS(1945),
    [sym_data_type_time] = ACTIONS(1945),
    [sym_data_type_image] = ACTIONS(1945),
    [sym_data_format_url] = ACTIONS(1945),
    [sym_data_format_phone] = ACTIONS(1945),
    [sym_data_format_email] = ACTIONS(1945),
    [sym_data_format_iso8601] = ACTIONS(1945),
    [sym_auditlog_priv] = ACTIONS(1945),
    [sym_configuration_priv] = ACTIONS(1945),
    [sym_deviceconfig_priv] = ACTIONS(1945),
    [sym_ftp_priv] = ACTIONS(1945),
    [sym_http_priv] = ACTIONS(1945),
    [sym_networkconfig_priv] = ACTIONS(1945),
    [sym_programport_priv] = ACTIONS(1945),
    [sym_remoteui_priv] = ACTIONS(1945),
    [sym_securitycontrol_priv] = ACTIONS(1945),
    [sym_softwaremanagement_priv] = ACTIONS(1945),
    [sym_terminal_priv] = ACTIONS(1945),
    [sym_tpadmin_priv] = ACTIONS(1945),
    [sym_user1_priv] = ACTIONS(1945),
    [sym_user2_priv] = ACTIONS(1945),
    [sym_user3_priv] = ACTIONS(1945),
    [sym_user4_priv] = ACTIONS(1945),
    [sym_usermanagement_priv] = ACTIONS(1945),
    [sym_valid_account] = ACTIONS(1945),
    [sym_err_name_invalid] = ACTIONS(1945),
    [sym_err_password_invalid] = ACTIONS(1945),
    [sym_err_invalid_account] = ACTIONS(1945),
    [sym_err_no_authorization] = ACTIONS(1945),
    [sym_err_invalid_parameter] = ACTIONS(1945),
    [sym_err_user_locked_out] = ACTIONS(1945),
    [sym_err_user_account_expired] = ACTIONS(1945),
    [sym_err_general_error] = ACTIONS(1945),
    [sym_err_authentication_error] = ACTIONS(1945),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1945),
    [sym_tls_validate_certificate] = ACTIONS(1945),
    [sym_tdata] = ACTIONS(1945),
    [sym_tchannel] = ACTIONS(1945),
    [sym_tlevel] = ACTIONS(1945),
    [sym_tbutton] = ACTIONS(1945),
    [sym_ttimeline] = ACTIONS(1945),
    [sym_tcustom] = ACTIONS(1945),
    [sym_url_struct] = ACTIONS(1945),
    [sym_dns_struct] = ACTIONS(1945),
    [sym_ip_address_struct] = ACTIONS(1945),
    [sym_dev_info_struct] = ACTIONS(1945),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1945),
    [sym_data_feed] = ACTIONS(1945),
    [sym_data_field] = ACTIONS(1945),
    [sym_data_record] = ACTIONS(1945),
    [sym_wc_data_feed] = ACTIONS(1945),
    [sym_wc_data_field] = ACTIONS(1945),
    [sym_wc_data_record] = ACTIONS(1945),
    [sym_last_login_info] = ACTIONS(1945),
    [sym____reserved___] = ACTIONS(1945),
    [sym_date] = ACTIONS(1945),
    [sym_day] = ACTIONS(1945),
    [sym_time] = ACTIONS(1945),
    [sym_ldate] = ACTIONS(1945),
    [sym_push_device] = ACTIONS(1945),
    [sym_push_channel] = ACTIONS(1945),
    [sym_push_devchan] = ACTIONS(1945),
    [sym_release_device] = ACTIONS(1945),
    [sym_release_channel] = ACTIONS(1945),
    [sym_release_devchan] = ACTIONS(1945),
    [sym_master_slot] = ACTIONS(1945),
    [sym_get_pulse_time] = ACTIONS(1945),
    [sym_get_timer] = ACTIONS(1945),
    [sym_master_sn] = ACTIONS(1945),
    [sym_system_number] = ACTIONS(1945),
    [sym_button] = ACTIONS(1945),
    [sym_dv_channel] = ACTIONS(1945),
    [sym_data] = ACTIONS(1945),
    [sym_level] = ACTIONS(1945),
    [sym_timeline] = ACTIONS(1945),
    [sym_channel] = ACTIONS(1945),
    [sym_custom] = ACTIONS(1945),
    [sym_atoi] = ACTIONS(1945),
    [sym_atol] = ACTIONS(1945),
    [sym_atof] = ACTIONS(1945),
    [sym_itoa] = ACTIONS(1945),
    [sym_itohex] = ACTIONS(1945),
    [sym_hextoi] = ACTIONS(1945),
    [sym_ftoa] = ACTIONS(1945),
    [sym_find_string] = ACTIONS(1945),
    [sym_left_string] = ACTIONS(1945),
    [sym_right_string] = ACTIONS(1945),
    [sym_mid_string] = ACTIONS(1945),
    [sym_remove_string] = ACTIONS(1945),
    [sym_get_buffer_string] = ACTIONS(1945),
    [sym_get_buffer_char] = ACTIONS(1945),
    [sym_get_multi_buffer_string] = ACTIONS(1945),
    [sym_length_string] = ACTIONS(1945),
    [sym_max_length_string] = ACTIONS(1945),
    [sym_set_length_string] = ACTIONS(1945),
    [sym_compare_string] = ACTIONS(1945),
    [sym_upper_string] = ACTIONS(1945),
    [sym_lower_string] = ACTIONS(1945),
    [sym_get_last] = ACTIONS(1945),
    [sym_date_to_day] = ACTIONS(1945),
    [sym_date_to_month] = ACTIONS(1945),
    [sym_date_to_year] = ACTIONS(1945),
    [sym_time_to_hour] = ACTIONS(1945),
    [sym_time_to_minute] = ACTIONS(1945),
    [sym_time_to_second] = ACTIONS(1945),
    [sym_day_of_week] = ACTIONS(1945),
    [sym_set_pulse_time] = ACTIONS(1945),
    [sym_set_timer] = ACTIONS(1945),
    [sym_file_open] = ACTIONS(1945),
    [sym_file_close] = ACTIONS(1945),
    [sym_file_read] = ACTIONS(1945),
    [sym_file_read_line] = ACTIONS(1945),
    [sym_file_write] = ACTIONS(1945),
    [sym_file_write_line] = ACTIONS(1945),
    [sym_file_copy] = ACTIONS(1945),
    [sym_file_delete] = ACTIONS(1945),
    [sym_file_rename] = ACTIONS(1945),
    [sym_file_dir] = ACTIONS(1945),
    [sym_file_getdir] = ACTIONS(1945),
    [sym_file_setdir] = ACTIONS(1945),
    [sym_file_create_dir] = ACTIONS(1945),
    [sym_file_remove_dir] = ACTIONS(1945),
    [sym_file_seek] = ACTIONS(1945),
    [sym_length_array] = ACTIONS(1945),
    [sym_max_length_array] = ACTIONS(1945),
    [sym_set_length_array] = ACTIONS(1945),
    [sym_ip_client_open] = ACTIONS(1945),
    [sym_ip_client_close] = ACTIONS(1945),
    [sym_ip_server_open] = ACTIONS(1945),
    [sym_ip_server_close] = ACTIONS(1945),
    [sym_ip_bound_client_open] = ACTIONS(1945),
    [sym_ip_mc_server_open] = ACTIONS(1945),
    [sym_ip_set_option] = ACTIONS(1945),
    [sym_ssh_client_open] = ACTIONS(1945),
    [sym_ssh_client_close] = ACTIONS(1945),
    [sym_tls_client_open] = ACTIONS(1945),
    [sym_tls_client_close] = ACTIONS(1945),
    [sym_do_push] = ACTIONS(1945),
    [sym_do_push_timed] = ACTIONS(1945),
    [sym_do_release] = ACTIONS(1945),
    [sym_device_id] = ACTIONS(1945),
    [sym_device_info] = ACTIONS(1945),
    [sym_device_id_string] = ACTIONS(1945),
    [sym_device_standby] = ACTIONS(1945),
    [sym_device_wake] = ACTIONS(1945),
    [sym_timeline_create] = ACTIONS(1945),
    [sym_timeline_kill] = ACTIONS(1945),
    [sym_timeline_reload] = ACTIONS(1945),
    [sym_timeline_pause] = ACTIONS(1945),
    [sym_timeline_restart] = ACTIONS(1945),
    [sym_timeline_set] = ACTIONS(1945),
    [sym_timeline_get] = ACTIONS(1945),
    [sym_timeline_active] = ACTIONS(1945),
    [sym_timeline_dynamic_id] = ACTIONS(1945),
    [sym_variable_to_string] = ACTIONS(1945),
    [sym_string_to_variable] = ACTIONS(1945),
    [sym_length_variable_to_string] = ACTIONS(1945),
    [sym_type_cast] = ACTIONS(1945),
    [sym_variable_to_xml] = ACTIONS(1945),
    [sym_xml_to_variable] = ACTIONS(1945),
    [sym_length_variable_to_xml] = ACTIONS(1945),
    [sym_redirect_string] = ACTIONS(1945),
    [sym_random_number] = ACTIONS(1945),
    [sym_get_unique_id] = ACTIONS(1945),
    [sym_get_system_number] = ACTIONS(1945),
    [sym_set_system_number] = ACTIONS(1945),
    [sym_get_serial_number] = ACTIONS(1945),
    [sym_get_master_build] = ACTIONS(1945),
    [sym_set_validation_code] = ACTIONS(1945),
    [sym_reboot] = ACTIONS(1945),
    [sym_load_duet_module] = ACTIONS(1945),
    [sym_unload_duet_module] = ACTIONS(1945),
    [sym_duet_mem_size_set] = ACTIONS(1945),
    [sym_duet_mem_size_get] = ACTIONS(1945),
    [sym_dynamic_application_device] = ACTIONS(1945),
    [sym_dynamic_polled_port] = ACTIONS(1945),
    [sym_static_port_binding] = ACTIONS(1945),
    [sym_static_ip_binding] = ACTIONS(1945),
    [sym_get_dns_list] = ACTIONS(1945),
    [sym_set_dns_list] = ACTIONS(1945),
    [sym_get_ip_address] = ACTIONS(1945),
    [sym_set_ip_address] = ACTIONS(1945),
    [sym_get_url_list] = ACTIONS(1945),
    [sym_add_url_entry] = ACTIONS(1945),
    [sym_delete_url_entry] = ACTIONS(1945),
    [sym_abs_value] = ACTIONS(1945),
    [sym_max_value] = ACTIONS(1945),
    [sym_min_value] = ACTIONS(1945),
    [sym_exp_value] = ACTIONS(1945),
    [sym_log_value] = ACTIONS(1945),
    [sym_log10_value] = ACTIONS(1945),
    [sym_power_value] = ACTIONS(1945),
    [sym_sqrt_value] = ACTIONS(1945),
    [sym_raw_be] = ACTIONS(1945),
    [sym_raw_le] = ACTIONS(1945),
    [sym_internal_queue_size_set] = ACTIONS(1945),
    [sym_internal_queue_size_get] = ACTIONS(1945),
    [sym_internal_threshold_set] = ACTIONS(1945),
    [sym_internal_threshold_get] = ACTIONS(1945),
    [sym_rebuilt_event] = ACTIONS(1945),
    [sym_do_custom_event] = ACTIONS(1945),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1945),
    [sym_clkmgr_set_clk_source] = ACTIONS(1945),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1945),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1945),
    [sym_clkmgr_get_timezone] = ACTIONS(1945),
    [sym_clkmgr_set_timezone] = ACTIONS(1945),
    [sym_clkmgr_get_resync_period] = ACTIONS(1945),
    [sym_clkmgr_set_resync_period] = ACTIONS(1945),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1945),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1945),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1945),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1945),
    [sym_clkmgr_get_timeservers] = ACTIONS(1945),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1945),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1945),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1945),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1945),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1945),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1945),
    [sym_set_log_level] = ACTIONS(1945),
    [sym_get_log_level] = ACTIONS(1945),
    [sym_amx_log] = ACTIONS(1945),
    [sym_smtp_server_config_set] = ACTIONS(1945),
    [sym_smtp_server_config_get] = ACTIONS(1945),
    [sym_smtp_send] = ACTIONS(1945),
    [sym_validate_netlinx_account] = ACTIONS(1945),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1945),
    [sym_authenticate_certificate] = ACTIONS(1945),
    [sym_audit_netlinx_session_event] = ACTIONS(1945),
    [sym_audit_netlinx_generic_event] = ACTIONS(1945),
    [sym_get_available_flash_disk_space] = ACTIONS(1945),
    [sym_get_max_flash_disk_space] = ACTIONS(1945),
    [sym_data_create_feed] = ACTIONS(1945),
    [sym_data_delete_feed] = ACTIONS(1945),
    [sym_data_publish_feed] = ACTIONS(1945),
    [sym_data_get_published_feed] = ACTIONS(1945),
    [sym_data_add_record] = ACTIONS(1945),
    [sym_data_get_event_record] = ACTIONS(1945),
    [sym__wc_data_create_feed] = ACTIONS(1945),
    [sym__wc_data_add_record] = ACTIONS(1945),
    [sym__wc_data_get_event_record] = ACTIONS(1945),
    [sym_format] = ACTIONS(1945),
    [sym_astro_clock] = ACTIONS(1945),
    [sym_set_outdoor_temperature] = ACTIONS(1945),
    [sym_set_virtual_level_count] = ACTIONS(1945),
    [sym_set_virtual_channel_count] = ACTIONS(1945),
    [sym_set_virtual_port_count] = ACTIONS(1945),
    [sym__wc_encode_int] = ACTIONS(1945),
    [sym__wc_decode_int] = ACTIONS(1945),
    [sym__wc_int] = ACTIONS(1945),
    [sym__wc_to_ch_int] = ACTIONS(1945),
    [sym__ch_to_wc_int] = ACTIONS(1945),
    [sym__wc_find_string_int] = ACTIONS(1945),
    [sym__wc_left_string_int] = ACTIONS(1945),
    [sym__wc_lower_string_int] = ACTIONS(1945),
    [sym__wc_mid_string_int] = ACTIONS(1945),
    [sym__wc_remove_string_int] = ACTIONS(1945),
    [sym__wc_right_string_int] = ACTIONS(1945),
    [sym__wc_upper_string_int] = ACTIONS(1945),
    [sym__wc_compare_string_int] = ACTIONS(1945),
    [sym__wc_get_buffer_char_int] = ACTIONS(1945),
    [sym__wc_get_buffer_string_int] = ACTIONS(1945),
    [sym__wc_concat_string_int] = ACTIONS(1945),
    [sym__wc_file_open_int] = ACTIONS(1945),
    [sym__wc_file_close_int] = ACTIONS(1945),
    [sym__wc_file_read_int] = ACTIONS(1945),
    [sym__wc_file_read_line_int] = ACTIONS(1945),
    [sym__wc_file_write_int] = ACTIONS(1945),
    [sym__wc_file_write_line_int] = ACTIONS(1945),
    [aux_sym_preproc_include_token1] = ACTIONS(1945),
    [aux_sym_preproc_define_token1] = ACTIONS(1947),
    [aux_sym_preproc_warn_token1] = ACTIONS(1947),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1947),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1947),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1947),
    [anon_sym_LBRACK] = ACTIONS(1947),
    [anon_sym_LBRACE] = ACTIONS(1947),
    [anon_sym_LPAREN] = ACTIONS(1945),
    [anon_sym_BANG] = ACTIONS(1947),
    [anon_sym_TILDE] = ACTIONS(1947),
    [anon_sym_DASH] = ACTIONS(1945),
    [anon_sym_PLUS] = ACTIONS(1945),
    [anon_sym_DASH_DASH] = ACTIONS(1947),
    [anon_sym_PLUS_PLUS] = ACTIONS(1947),
    [anon_sym_DQUOTE] = ACTIONS(1947),
    [anon_sym_SQUOTE] = ACTIONS(1947),
    [sym_number_literal] = ACTIONS(1947),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(409)] = {
    [ts_builtin_sym_end] = ACTIONS(1951),
    [sym_identifier] = ACTIONS(1949),
    [sym_program_name_keyword] = ACTIONS(1949),
    [sym_module_name_keyword] = ACTIONS(1949),
    [sym_define_device_keyword] = ACTIONS(1949),
    [sym_define_combine_keyword] = ACTIONS(1949),
    [sym_define_constant_keyword] = ACTIONS(1949),
    [sym_define_type_keyword] = ACTIONS(1949),
    [sym_define_variable_keyword] = ACTIONS(1949),
    [sym_define_system_variable_keyword] = ACTIONS(1949),
    [sym_define_connect_level_keyword] = ACTIONS(1949),
    [sym_define_latching_keyword] = ACTIONS(1949),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1949),
    [sym_define_toggling_keyword] = ACTIONS(1949),
    [sym_define_call_keyword] = ACTIONS(1949),
    [sym_define_function_keyword] = ACTIONS(1949),
    [sym_define_library_function_keyword] = ACTIONS(1949),
    [sym_define_start_keyword] = ACTIONS(1949),
    [sym_define_event_keyword] = ACTIONS(1949),
    [sym_define_program_keyword] = ACTIONS(1949),
    [sym_define_module_keyword] = ACTIONS(1949),
    [sym_char_keyword] = ACTIONS(1949),
    [sym_widechar_keyword] = ACTIONS(1949),
    [sym_integer_keyword] = ACTIONS(1949),
    [sym_sinteger_keyword] = ACTIONS(1949),
    [sym_long_keyword] = ACTIONS(1949),
    [sym_slong_keyword] = ACTIONS(1949),
    [sym_float_keyword] = ACTIONS(1949),
    [sym_double_keyword] = ACTIONS(1949),
    [sym_variant_keyword] = ACTIONS(1949),
    [sym_variantarray_keyword] = ACTIONS(1949),
    [sym_local_var_keyword] = ACTIONS(1949),
    [sym_stack_var_keyword] = ACTIONS(1949),
    [sym_constant_keyword] = ACTIONS(1949),
    [sym_volatile_keyword] = ACTIONS(1949),
    [sym_non_volatile_keyword] = ACTIONS(1949),
    [sym_persistent_keyword] = ACTIONS(1949),
    [sym_dev_keyword] = ACTIONS(1949),
    [sym_devlev_keyword] = ACTIONS(1949),
    [sym_devchan_keyword] = ACTIONS(1949),
    [sym_if_keyword] = ACTIONS(1949),
    [sym_switch_keyword] = ACTIONS(1949),
    [sym_case_keyword] = ACTIONS(1949),
    [sym_default_keyword] = ACTIONS(1949),
    [sym_while_keyword] = ACTIONS(1949),
    [sym_for_keyword] = ACTIONS(1949),
    [sym_break_keyword] = ACTIONS(1949),
    [sym_continue_keyword] = ACTIONS(1949),
    [sym_return_keyword] = ACTIONS(1949),
    [sym_select_keyword] = ACTIONS(1949),
    [sym_active_keyword] = ACTIONS(1949),
    [sym_struct_keyword] = ACTIONS(1949),
    [sym_structure_keyword] = ACTIONS(1949),
    [sym_button_event_keyword] = ACTIONS(1949),
    [sym_channel_event_keyword] = ACTIONS(1949),
    [sym_level_event_keyword] = ACTIONS(1949),
    [sym_data_event_keyword] = ACTIONS(1949),
    [sym_timeline_event_keyword] = ACTIONS(1949),
    [sym_custom_event_keyword] = ACTIONS(1949),
    [sym_push_keyword] = ACTIONS(1949),
    [sym_release_keyword] = ACTIONS(1949),
    [aux_sym_on_keyword_token1] = ACTIONS(1949),
    [aux_sym_off_keyword_token1] = ACTIONS(1949),
    [sym_send_level_keyword] = ACTIONS(1949),
    [sym_send_string_keyword] = ACTIONS(1949),
    [sym_send_command_keyword] = ACTIONS(1949),
    [sym_clear_buffer_keyword] = ACTIONS(1949),
    [sym_create_buffer_keyword] = ACTIONS(1949),
    [sym_create_multi_buffer_keyword] = ACTIONS(1949),
    [sym_call_keyword] = ACTIONS(1949),
    [sym_system_call_keyword] = ACTIONS(1949),
    [sym_devchan_to_keyword] = ACTIONS(1949),
    [sym_devchan_min_to_keyword] = ACTIONS(1949),
    [sym_devchan_total_off_keyword] = ACTIONS(1949),
    [sym_devchan_pulse_keyword] = ACTIONS(1949),
    [sym_wait_keyword] = ACTIONS(1949),
    [sym_cancel_wait_keyword] = ACTIONS(1949),
    [sym_wait_until_keyword] = ACTIONS(1949),
    [sym_cancel_wait_until_keyword] = ACTIONS(1949),
    [sym_cancel_all_wait_keyword] = ACTIONS(1949),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1949),
    [sym_not] = ACTIONS(1949),
    [sym_bnot] = ACTIONS(1949),
    [sym___file__] = ACTIONS(1949),
    [sym___date__] = ACTIONS(1949),
    [sym___ldate__] = ACTIONS(1949),
    [sym___time__] = ACTIONS(1949),
    [sym___line__] = ACTIONS(1949),
    [sym___name__] = ACTIONS(1949),
    [sym___netlinx__] = ACTIONS(1949),
    [sym_netlinx_axi_version] = ACTIONS(1949),
    [sym_url_flg_tcp] = ACTIONS(1949),
    [sym_url_flg_accinfopresent] = ACTIONS(1949),
    [sym_url_flg_temp] = ACTIONS(1949),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1949),
    [sym_url_flg_stat_mask] = ACTIONS(1949),
    [sym_url_flg_stat_lookup] = ACTIONS(1949),
    [sym_url_flg_stat_connecting] = ACTIONS(1949),
    [sym_url_flg_stat_waiting] = ACTIONS(1949),
    [sym_url_flg_stat_connected] = ACTIONS(1949),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1949),
    [sym_first_virtual_device] = ACTIONS(1949),
    [sym_first_local_port] = ACTIONS(1949),
    [sym_dynamic_virtual_device] = ACTIONS(1949),
    [sym_timeline_once] = ACTIONS(1949),
    [sym_timeline_repeat] = ACTIONS(1949),
    [sym_timeline_absolute] = ACTIONS(1949),
    [sym_timeline_relative] = ACTIONS(1949),
    [sym_do_push_timed_infinite] = ACTIONS(1949),
    [sym_source_type_no_address] = ACTIONS(1949),
    [sym_source_type_neuron_id] = ACTIONS(1949),
    [sym_source_type_ip_address] = ACTIONS(1949),
    [sym_source_type_axlink] = ACTIONS(1949),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1949),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1949),
    [sym_source_type_ip_socket_address] = ACTIONS(1949),
    [sym_source_type_rs232] = ACTIONS(1949),
    [sym_source_type_internal] = ACTIONS(1949),
    [sym_source_type_mac_address] = ACTIONS(1949),
    [sym_source_type_ipv4_port] = ACTIONS(1949),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1949),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1949),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1949),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1949),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1949),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1949),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1949),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1949),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1949),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1949),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1949),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1949),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1949),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1949),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1949),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1949),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1949),
    [sym_true] = ACTIONS(1949),
    [sym_false] = ACTIONS(1949),
    [sym_file_read_only] = ACTIONS(1949),
    [sym_file_rw_new] = ACTIONS(1949),
    [sym_file_rw_append] = ACTIONS(1949),
    [sym_ip_tcp] = ACTIONS(1949),
    [sym_ip_udp] = ACTIONS(1949),
    [sym_ip_udp_2way] = ACTIONS(1949),
    [sym_xml_encode_types] = ACTIONS(1949),
    [sym_xml_encode_char_as_list] = ACTIONS(1949),
    [sym_xml_encode_le] = ACTIONS(1949),
    [sym_xml_decode_types] = ACTIONS(1949),
    [sym_xml_decode_no_preserve] = ACTIONS(1949),
    [sym_duet_dev_type_display_device] = ACTIONS(1949),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1949),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1949),
    [sym_duet_dev_type_io_device] = ACTIONS(1949),
    [sym_duet_dev_type_relay_device] = ACTIONS(1949),
    [sym_duet_dev_type_ups] = ACTIONS(1949),
    [sym_duet_dev_type_amplifier] = ACTIONS(1949),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1949),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1949),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1949),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1949),
    [sym_duet_dev_type_camera] = ACTIONS(1949),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1949),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1949),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1949),
    [sym_duet_dev_type_dss] = ACTIONS(1949),
    [sym_duet_dev_type_dvr] = ACTIONS(1949),
    [sym_duet_dev_type_disc_device] = ACTIONS(1949),
    [sym_duet_dev_type_document_camera] = ACTIONS(1949),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1949),
    [sym_duet_dev_type_hvac] = ACTIONS(1949),
    [sym_duet_dev_type_keypad] = ACTIONS(1949),
    [sym_duet_dev_type_light] = ACTIONS(1949),
    [sym_duet_dev_type_monitor] = ACTIONS(1949),
    [sym_duet_dev_type_motor] = ACTIONS(1949),
    [sym_duet_dev_type_multi_window] = ACTIONS(1949),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1949),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1949),
    [sym_duet_dev_type_receiver] = ACTIONS(1949),
    [sym_duet_dev_type_security_system] = ACTIONS(1949),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1949),
    [sym_duet_dev_type_settop_box] = ACTIONS(1949),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1949),
    [sym_duet_dev_type_switcher] = ACTIONS(1949),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1949),
    [sym_duet_dev_type_tv] = ACTIONS(1949),
    [sym_duet_dev_type_utility] = ACTIONS(1949),
    [sym_duet_dev_type_vcr] = ACTIONS(1949),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1949),
    [sym_duet_dev_type_video_processor] = ACTIONS(1949),
    [sym_duet_dev_type_video_projector] = ACTIONS(1949),
    [sym_duet_dev_type_video_wall] = ACTIONS(1949),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1949),
    [sym_duet_dev_type_weather] = ACTIONS(1949),
    [sym_duet_dev_not_polled] = ACTIONS(1949),
    [sym_duet_dev_polled] = ACTIONS(1949),
    [sym_clkmgr_mode_network] = ACTIONS(1949),
    [sym_clkmgr_mode_standalone] = ACTIONS(1949),
    [sym_normal_standby] = ACTIONS(1949),
    [sym_normal_wake] = ACTIONS(1949),
    [sym_ip_multicast_ttl_option] = ACTIONS(1949),
    [sym_ip_ttl_subnet] = ACTIONS(1949),
    [sym_ip_ttl_site] = ACTIONS(1949),
    [sym_ip_ttl_region] = ACTIONS(1949),
    [sym_ip_ttl_continent] = ACTIONS(1949),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1949),
    [sym_ip_nodelay_on] = ACTIONS(1949),
    [sym_ip_nodelay_off] = ACTIONS(1949),
    [sym_amx_error] = ACTIONS(1949),
    [sym_amx_warning] = ACTIONS(1949),
    [sym_amx_info] = ACTIONS(1949),
    [sym_amx_debug] = ACTIONS(1949),
    [sym_netlinx_login_success] = ACTIONS(1949),
    [sym_netlinx_login_fail] = ACTIONS(1949),
    [sym_netlinx_logout] = ACTIONS(1949),
    [sym_null_str] = ACTIONS(1949),
    [sym_smtp_address] = ACTIONS(1949),
    [sym_smtp_port_number] = ACTIONS(1949),
    [sym_smtp_username] = ACTIONS(1949),
    [sym_smtp_password] = ACTIONS(1949),
    [sym_smtp_require_tls] = ACTIONS(1949),
    [sym_smtp_from] = ACTIONS(1949),
    [sym_smtp_tls_true] = ACTIONS(1949),
    [sym_smtp_tls_false] = ACTIONS(1949),
    [sym_listview_on_row_select_event] = ACTIONS(1949),
    [sym_data_structure_unknown] = ACTIONS(1949),
    [sym_data_structure_datafeed] = ACTIONS(1949),
    [sym_data_structure_datarecordset] = ACTIONS(1949),
    [sym_data_structure_datarecord] = ACTIONS(1949),
    [sym_data_structure_datafield] = ACTIONS(1949),
    [sym_data_max_content_field_count] = ACTIONS(1949),
    [sym_data_max_metadata_field_count] = ACTIONS(1949),
    [sym_data_max_id_length] = ACTIONS(1949),
    [sym_data_max_name_length] = ACTIONS(1949),
    [sym_data_max_description_length] = ACTIONS(1949),
    [sym_data_max_type_length] = ACTIONS(1949),
    [sym_data_max_format_length] = ACTIONS(1949),
    [sym_data_max_source_length] = ACTIONS(1949),
    [sym_data_max_value_length] = ACTIONS(1949),
    [sym_data_max_label_length] = ACTIONS(1949),
    [sym_data_max_url_length] = ACTIONS(1949),
    [sym_max_last_login_info_length] = ACTIONS(1949),
    [sym_data_type_unknown] = ACTIONS(1949),
    [sym_data_type_string] = ACTIONS(1949),
    [sym_data_type_datetime] = ACTIONS(1949),
    [sym_data_type_date] = ACTIONS(1949),
    [sym_data_type_time] = ACTIONS(1949),
    [sym_data_type_image] = ACTIONS(1949),
    [sym_data_format_url] = ACTIONS(1949),
    [sym_data_format_phone] = ACTIONS(1949),
    [sym_data_format_email] = ACTIONS(1949),
    [sym_data_format_iso8601] = ACTIONS(1949),
    [sym_auditlog_priv] = ACTIONS(1949),
    [sym_configuration_priv] = ACTIONS(1949),
    [sym_deviceconfig_priv] = ACTIONS(1949),
    [sym_ftp_priv] = ACTIONS(1949),
    [sym_http_priv] = ACTIONS(1949),
    [sym_networkconfig_priv] = ACTIONS(1949),
    [sym_programport_priv] = ACTIONS(1949),
    [sym_remoteui_priv] = ACTIONS(1949),
    [sym_securitycontrol_priv] = ACTIONS(1949),
    [sym_softwaremanagement_priv] = ACTIONS(1949),
    [sym_terminal_priv] = ACTIONS(1949),
    [sym_tpadmin_priv] = ACTIONS(1949),
    [sym_user1_priv] = ACTIONS(1949),
    [sym_user2_priv] = ACTIONS(1949),
    [sym_user3_priv] = ACTIONS(1949),
    [sym_user4_priv] = ACTIONS(1949),
    [sym_usermanagement_priv] = ACTIONS(1949),
    [sym_valid_account] = ACTIONS(1949),
    [sym_err_name_invalid] = ACTIONS(1949),
    [sym_err_password_invalid] = ACTIONS(1949),
    [sym_err_invalid_account] = ACTIONS(1949),
    [sym_err_no_authorization] = ACTIONS(1949),
    [sym_err_invalid_parameter] = ACTIONS(1949),
    [sym_err_user_locked_out] = ACTIONS(1949),
    [sym_err_user_account_expired] = ACTIONS(1949),
    [sym_err_general_error] = ACTIONS(1949),
    [sym_err_authentication_error] = ACTIONS(1949),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1949),
    [sym_tls_validate_certificate] = ACTIONS(1949),
    [sym_tdata] = ACTIONS(1949),
    [sym_tchannel] = ACTIONS(1949),
    [sym_tlevel] = ACTIONS(1949),
    [sym_tbutton] = ACTIONS(1949),
    [sym_ttimeline] = ACTIONS(1949),
    [sym_tcustom] = ACTIONS(1949),
    [sym_url_struct] = ACTIONS(1949),
    [sym_dns_struct] = ACTIONS(1949),
    [sym_ip_address_struct] = ACTIONS(1949),
    [sym_dev_info_struct] = ACTIONS(1949),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1949),
    [sym_data_feed] = ACTIONS(1949),
    [sym_data_field] = ACTIONS(1949),
    [sym_data_record] = ACTIONS(1949),
    [sym_wc_data_feed] = ACTIONS(1949),
    [sym_wc_data_field] = ACTIONS(1949),
    [sym_wc_data_record] = ACTIONS(1949),
    [sym_last_login_info] = ACTIONS(1949),
    [sym____reserved___] = ACTIONS(1949),
    [sym_date] = ACTIONS(1949),
    [sym_day] = ACTIONS(1949),
    [sym_time] = ACTIONS(1949),
    [sym_ldate] = ACTIONS(1949),
    [sym_push_device] = ACTIONS(1949),
    [sym_push_channel] = ACTIONS(1949),
    [sym_push_devchan] = ACTIONS(1949),
    [sym_release_device] = ACTIONS(1949),
    [sym_release_channel] = ACTIONS(1949),
    [sym_release_devchan] = ACTIONS(1949),
    [sym_master_slot] = ACTIONS(1949),
    [sym_get_pulse_time] = ACTIONS(1949),
    [sym_get_timer] = ACTIONS(1949),
    [sym_master_sn] = ACTIONS(1949),
    [sym_system_number] = ACTIONS(1949),
    [sym_button] = ACTIONS(1949),
    [sym_dv_channel] = ACTIONS(1949),
    [sym_data] = ACTIONS(1949),
    [sym_level] = ACTIONS(1949),
    [sym_timeline] = ACTIONS(1949),
    [sym_channel] = ACTIONS(1949),
    [sym_custom] = ACTIONS(1949),
    [sym_atoi] = ACTIONS(1949),
    [sym_atol] = ACTIONS(1949),
    [sym_atof] = ACTIONS(1949),
    [sym_itoa] = ACTIONS(1949),
    [sym_itohex] = ACTIONS(1949),
    [sym_hextoi] = ACTIONS(1949),
    [sym_ftoa] = ACTIONS(1949),
    [sym_find_string] = ACTIONS(1949),
    [sym_left_string] = ACTIONS(1949),
    [sym_right_string] = ACTIONS(1949),
    [sym_mid_string] = ACTIONS(1949),
    [sym_remove_string] = ACTIONS(1949),
    [sym_get_buffer_string] = ACTIONS(1949),
    [sym_get_buffer_char] = ACTIONS(1949),
    [sym_get_multi_buffer_string] = ACTIONS(1949),
    [sym_length_string] = ACTIONS(1949),
    [sym_max_length_string] = ACTIONS(1949),
    [sym_set_length_string] = ACTIONS(1949),
    [sym_compare_string] = ACTIONS(1949),
    [sym_upper_string] = ACTIONS(1949),
    [sym_lower_string] = ACTIONS(1949),
    [sym_get_last] = ACTIONS(1949),
    [sym_date_to_day] = ACTIONS(1949),
    [sym_date_to_month] = ACTIONS(1949),
    [sym_date_to_year] = ACTIONS(1949),
    [sym_time_to_hour] = ACTIONS(1949),
    [sym_time_to_minute] = ACTIONS(1949),
    [sym_time_to_second] = ACTIONS(1949),
    [sym_day_of_week] = ACTIONS(1949),
    [sym_set_pulse_time] = ACTIONS(1949),
    [sym_set_timer] = ACTIONS(1949),
    [sym_file_open] = ACTIONS(1949),
    [sym_file_close] = ACTIONS(1949),
    [sym_file_read] = ACTIONS(1949),
    [sym_file_read_line] = ACTIONS(1949),
    [sym_file_write] = ACTIONS(1949),
    [sym_file_write_line] = ACTIONS(1949),
    [sym_file_copy] = ACTIONS(1949),
    [sym_file_delete] = ACTIONS(1949),
    [sym_file_rename] = ACTIONS(1949),
    [sym_file_dir] = ACTIONS(1949),
    [sym_file_getdir] = ACTIONS(1949),
    [sym_file_setdir] = ACTIONS(1949),
    [sym_file_create_dir] = ACTIONS(1949),
    [sym_file_remove_dir] = ACTIONS(1949),
    [sym_file_seek] = ACTIONS(1949),
    [sym_length_array] = ACTIONS(1949),
    [sym_max_length_array] = ACTIONS(1949),
    [sym_set_length_array] = ACTIONS(1949),
    [sym_ip_client_open] = ACTIONS(1949),
    [sym_ip_client_close] = ACTIONS(1949),
    [sym_ip_server_open] = ACTIONS(1949),
    [sym_ip_server_close] = ACTIONS(1949),
    [sym_ip_bound_client_open] = ACTIONS(1949),
    [sym_ip_mc_server_open] = ACTIONS(1949),
    [sym_ip_set_option] = ACTIONS(1949),
    [sym_ssh_client_open] = ACTIONS(1949),
    [sym_ssh_client_close] = ACTIONS(1949),
    [sym_tls_client_open] = ACTIONS(1949),
    [sym_tls_client_close] = ACTIONS(1949),
    [sym_do_push] = ACTIONS(1949),
    [sym_do_push_timed] = ACTIONS(1949),
    [sym_do_release] = ACTIONS(1949),
    [sym_device_id] = ACTIONS(1949),
    [sym_device_info] = ACTIONS(1949),
    [sym_device_id_string] = ACTIONS(1949),
    [sym_device_standby] = ACTIONS(1949),
    [sym_device_wake] = ACTIONS(1949),
    [sym_timeline_create] = ACTIONS(1949),
    [sym_timeline_kill] = ACTIONS(1949),
    [sym_timeline_reload] = ACTIONS(1949),
    [sym_timeline_pause] = ACTIONS(1949),
    [sym_timeline_restart] = ACTIONS(1949),
    [sym_timeline_set] = ACTIONS(1949),
    [sym_timeline_get] = ACTIONS(1949),
    [sym_timeline_active] = ACTIONS(1949),
    [sym_timeline_dynamic_id] = ACTIONS(1949),
    [sym_variable_to_string] = ACTIONS(1949),
    [sym_string_to_variable] = ACTIONS(1949),
    [sym_length_variable_to_string] = ACTIONS(1949),
    [sym_type_cast] = ACTIONS(1949),
    [sym_variable_to_xml] = ACTIONS(1949),
    [sym_xml_to_variable] = ACTIONS(1949),
    [sym_length_variable_to_xml] = ACTIONS(1949),
    [sym_redirect_string] = ACTIONS(1949),
    [sym_random_number] = ACTIONS(1949),
    [sym_get_unique_id] = ACTIONS(1949),
    [sym_get_system_number] = ACTIONS(1949),
    [sym_set_system_number] = ACTIONS(1949),
    [sym_get_serial_number] = ACTIONS(1949),
    [sym_get_master_build] = ACTIONS(1949),
    [sym_set_validation_code] = ACTIONS(1949),
    [sym_reboot] = ACTIONS(1949),
    [sym_load_duet_module] = ACTIONS(1949),
    [sym_unload_duet_module] = ACTIONS(1949),
    [sym_duet_mem_size_set] = ACTIONS(1949),
    [sym_duet_mem_size_get] = ACTIONS(1949),
    [sym_dynamic_application_device] = ACTIONS(1949),
    [sym_dynamic_polled_port] = ACTIONS(1949),
    [sym_static_port_binding] = ACTIONS(1949),
    [sym_static_ip_binding] = ACTIONS(1949),
    [sym_get_dns_list] = ACTIONS(1949),
    [sym_set_dns_list] = ACTIONS(1949),
    [sym_get_ip_address] = ACTIONS(1949),
    [sym_set_ip_address] = ACTIONS(1949),
    [sym_get_url_list] = ACTIONS(1949),
    [sym_add_url_entry] = ACTIONS(1949),
    [sym_delete_url_entry] = ACTIONS(1949),
    [sym_abs_value] = ACTIONS(1949),
    [sym_max_value] = ACTIONS(1949),
    [sym_min_value] = ACTIONS(1949),
    [sym_exp_value] = ACTIONS(1949),
    [sym_log_value] = ACTIONS(1949),
    [sym_log10_value] = ACTIONS(1949),
    [sym_power_value] = ACTIONS(1949),
    [sym_sqrt_value] = ACTIONS(1949),
    [sym_raw_be] = ACTIONS(1949),
    [sym_raw_le] = ACTIONS(1949),
    [sym_internal_queue_size_set] = ACTIONS(1949),
    [sym_internal_queue_size_get] = ACTIONS(1949),
    [sym_internal_threshold_set] = ACTIONS(1949),
    [sym_internal_threshold_get] = ACTIONS(1949),
    [sym_rebuilt_event] = ACTIONS(1949),
    [sym_do_custom_event] = ACTIONS(1949),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1949),
    [sym_clkmgr_set_clk_source] = ACTIONS(1949),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1949),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1949),
    [sym_clkmgr_get_timezone] = ACTIONS(1949),
    [sym_clkmgr_set_timezone] = ACTIONS(1949),
    [sym_clkmgr_get_resync_period] = ACTIONS(1949),
    [sym_clkmgr_set_resync_period] = ACTIONS(1949),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1949),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1949),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1949),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1949),
    [sym_clkmgr_get_timeservers] = ACTIONS(1949),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1949),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1949),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1949),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1949),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1949),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1949),
    [sym_set_log_level] = ACTIONS(1949),
    [sym_get_log_level] = ACTIONS(1949),
    [sym_amx_log] = ACTIONS(1949),
    [sym_smtp_server_config_set] = ACTIONS(1949),
    [sym_smtp_server_config_get] = ACTIONS(1949),
    [sym_smtp_send] = ACTIONS(1949),
    [sym_validate_netlinx_account] = ACTIONS(1949),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1949),
    [sym_authenticate_certificate] = ACTIONS(1949),
    [sym_audit_netlinx_session_event] = ACTIONS(1949),
    [sym_audit_netlinx_generic_event] = ACTIONS(1949),
    [sym_get_available_flash_disk_space] = ACTIONS(1949),
    [sym_get_max_flash_disk_space] = ACTIONS(1949),
    [sym_data_create_feed] = ACTIONS(1949),
    [sym_data_delete_feed] = ACTIONS(1949),
    [sym_data_publish_feed] = ACTIONS(1949),
    [sym_data_get_published_feed] = ACTIONS(1949),
    [sym_data_add_record] = ACTIONS(1949),
    [sym_data_get_event_record] = ACTIONS(1949),
    [sym__wc_data_create_feed] = ACTIONS(1949),
    [sym__wc_data_add_record] = ACTIONS(1949),
    [sym__wc_data_get_event_record] = ACTIONS(1949),
    [sym_format] = ACTIONS(1949),
    [sym_astro_clock] = ACTIONS(1949),
    [sym_set_outdoor_temperature] = ACTIONS(1949),
    [sym_set_virtual_level_count] = ACTIONS(1949),
    [sym_set_virtual_channel_count] = ACTIONS(1949),
    [sym_set_virtual_port_count] = ACTIONS(1949),
    [sym__wc_encode_int] = ACTIONS(1949),
    [sym__wc_decode_int] = ACTIONS(1949),
    [sym__wc_int] = ACTIONS(1949),
    [sym__wc_to_ch_int] = ACTIONS(1949),
    [sym__ch_to_wc_int] = ACTIONS(1949),
    [sym__wc_find_string_int] = ACTIONS(1949),
    [sym__wc_left_string_int] = ACTIONS(1949),
    [sym__wc_lower_string_int] = ACTIONS(1949),
    [sym__wc_mid_string_int] = ACTIONS(1949),
    [sym__wc_remove_string_int] = ACTIONS(1949),
    [sym__wc_right_string_int] = ACTIONS(1949),
    [sym__wc_upper_string_int] = ACTIONS(1949),
    [sym__wc_compare_string_int] = ACTIONS(1949),
    [sym__wc_get_buffer_char_int] = ACTIONS(1949),
    [sym__wc_get_buffer_string_int] = ACTIONS(1949),
    [sym__wc_concat_string_int] = ACTIONS(1949),
    [sym__wc_file_open_int] = ACTIONS(1949),
    [sym__wc_file_close_int] = ACTIONS(1949),
    [sym__wc_file_read_int] = ACTIONS(1949),
    [sym__wc_file_read_line_int] = ACTIONS(1949),
    [sym__wc_file_write_int] = ACTIONS(1949),
    [sym__wc_file_write_line_int] = ACTIONS(1949),
    [aux_sym_preproc_include_token1] = ACTIONS(1949),
    [aux_sym_preproc_define_token1] = ACTIONS(1951),
    [aux_sym_preproc_warn_token1] = ACTIONS(1951),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1951),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1951),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1951),
    [anon_sym_LBRACK] = ACTIONS(1951),
    [anon_sym_LBRACE] = ACTIONS(1951),
    [anon_sym_LPAREN] = ACTIONS(1949),
    [anon_sym_BANG] = ACTIONS(1951),
    [anon_sym_TILDE] = ACTIONS(1951),
    [anon_sym_DASH] = ACTIONS(1949),
    [anon_sym_PLUS] = ACTIONS(1949),
    [anon_sym_DASH_DASH] = ACTIONS(1951),
    [anon_sym_PLUS_PLUS] = ACTIONS(1951),
    [anon_sym_DQUOTE] = ACTIONS(1951),
    [anon_sym_SQUOTE] = ACTIONS(1951),
    [sym_number_literal] = ACTIONS(1951),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(410)] = {
    [ts_builtin_sym_end] = ACTIONS(1955),
    [sym_identifier] = ACTIONS(1953),
    [sym_program_name_keyword] = ACTIONS(1953),
    [sym_module_name_keyword] = ACTIONS(1953),
    [sym_define_device_keyword] = ACTIONS(1953),
    [sym_define_combine_keyword] = ACTIONS(1953),
    [sym_define_constant_keyword] = ACTIONS(1953),
    [sym_define_type_keyword] = ACTIONS(1953),
    [sym_define_variable_keyword] = ACTIONS(1953),
    [sym_define_system_variable_keyword] = ACTIONS(1953),
    [sym_define_connect_level_keyword] = ACTIONS(1953),
    [sym_define_latching_keyword] = ACTIONS(1953),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1953),
    [sym_define_toggling_keyword] = ACTIONS(1953),
    [sym_define_call_keyword] = ACTIONS(1953),
    [sym_define_function_keyword] = ACTIONS(1953),
    [sym_define_library_function_keyword] = ACTIONS(1953),
    [sym_define_start_keyword] = ACTIONS(1953),
    [sym_define_event_keyword] = ACTIONS(1953),
    [sym_define_program_keyword] = ACTIONS(1953),
    [sym_define_module_keyword] = ACTIONS(1953),
    [sym_char_keyword] = ACTIONS(1953),
    [sym_widechar_keyword] = ACTIONS(1953),
    [sym_integer_keyword] = ACTIONS(1953),
    [sym_sinteger_keyword] = ACTIONS(1953),
    [sym_long_keyword] = ACTIONS(1953),
    [sym_slong_keyword] = ACTIONS(1953),
    [sym_float_keyword] = ACTIONS(1953),
    [sym_double_keyword] = ACTIONS(1953),
    [sym_variant_keyword] = ACTIONS(1953),
    [sym_variantarray_keyword] = ACTIONS(1953),
    [sym_local_var_keyword] = ACTIONS(1953),
    [sym_stack_var_keyword] = ACTIONS(1953),
    [sym_constant_keyword] = ACTIONS(1953),
    [sym_volatile_keyword] = ACTIONS(1953),
    [sym_non_volatile_keyword] = ACTIONS(1953),
    [sym_persistent_keyword] = ACTIONS(1953),
    [sym_dev_keyword] = ACTIONS(1953),
    [sym_devlev_keyword] = ACTIONS(1953),
    [sym_devchan_keyword] = ACTIONS(1953),
    [sym_if_keyword] = ACTIONS(1953),
    [sym_switch_keyword] = ACTIONS(1953),
    [sym_case_keyword] = ACTIONS(1953),
    [sym_default_keyword] = ACTIONS(1953),
    [sym_while_keyword] = ACTIONS(1953),
    [sym_for_keyword] = ACTIONS(1953),
    [sym_break_keyword] = ACTIONS(1953),
    [sym_continue_keyword] = ACTIONS(1953),
    [sym_return_keyword] = ACTIONS(1953),
    [sym_select_keyword] = ACTIONS(1953),
    [sym_active_keyword] = ACTIONS(1953),
    [sym_struct_keyword] = ACTIONS(1953),
    [sym_structure_keyword] = ACTIONS(1953),
    [sym_button_event_keyword] = ACTIONS(1953),
    [sym_channel_event_keyword] = ACTIONS(1953),
    [sym_level_event_keyword] = ACTIONS(1953),
    [sym_data_event_keyword] = ACTIONS(1953),
    [sym_timeline_event_keyword] = ACTIONS(1953),
    [sym_custom_event_keyword] = ACTIONS(1953),
    [sym_push_keyword] = ACTIONS(1953),
    [sym_release_keyword] = ACTIONS(1953),
    [aux_sym_on_keyword_token1] = ACTIONS(1953),
    [aux_sym_off_keyword_token1] = ACTIONS(1953),
    [sym_send_level_keyword] = ACTIONS(1953),
    [sym_send_string_keyword] = ACTIONS(1953),
    [sym_send_command_keyword] = ACTIONS(1953),
    [sym_clear_buffer_keyword] = ACTIONS(1953),
    [sym_create_buffer_keyword] = ACTIONS(1953),
    [sym_create_multi_buffer_keyword] = ACTIONS(1953),
    [sym_call_keyword] = ACTIONS(1953),
    [sym_system_call_keyword] = ACTIONS(1953),
    [sym_devchan_to_keyword] = ACTIONS(1953),
    [sym_devchan_min_to_keyword] = ACTIONS(1953),
    [sym_devchan_total_off_keyword] = ACTIONS(1953),
    [sym_devchan_pulse_keyword] = ACTIONS(1953),
    [sym_wait_keyword] = ACTIONS(1953),
    [sym_cancel_wait_keyword] = ACTIONS(1953),
    [sym_wait_until_keyword] = ACTIONS(1953),
    [sym_cancel_wait_until_keyword] = ACTIONS(1953),
    [sym_cancel_all_wait_keyword] = ACTIONS(1953),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1953),
    [sym_not] = ACTIONS(1953),
    [sym_bnot] = ACTIONS(1953),
    [sym___file__] = ACTIONS(1953),
    [sym___date__] = ACTIONS(1953),
    [sym___ldate__] = ACTIONS(1953),
    [sym___time__] = ACTIONS(1953),
    [sym___line__] = ACTIONS(1953),
    [sym___name__] = ACTIONS(1953),
    [sym___netlinx__] = ACTIONS(1953),
    [sym_netlinx_axi_version] = ACTIONS(1953),
    [sym_url_flg_tcp] = ACTIONS(1953),
    [sym_url_flg_accinfopresent] = ACTIONS(1953),
    [sym_url_flg_temp] = ACTIONS(1953),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1953),
    [sym_url_flg_stat_mask] = ACTIONS(1953),
    [sym_url_flg_stat_lookup] = ACTIONS(1953),
    [sym_url_flg_stat_connecting] = ACTIONS(1953),
    [sym_url_flg_stat_waiting] = ACTIONS(1953),
    [sym_url_flg_stat_connected] = ACTIONS(1953),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1953),
    [sym_first_virtual_device] = ACTIONS(1953),
    [sym_first_local_port] = ACTIONS(1953),
    [sym_dynamic_virtual_device] = ACTIONS(1953),
    [sym_timeline_once] = ACTIONS(1953),
    [sym_timeline_repeat] = ACTIONS(1953),
    [sym_timeline_absolute] = ACTIONS(1953),
    [sym_timeline_relative] = ACTIONS(1953),
    [sym_do_push_timed_infinite] = ACTIONS(1953),
    [sym_source_type_no_address] = ACTIONS(1953),
    [sym_source_type_neuron_id] = ACTIONS(1953),
    [sym_source_type_ip_address] = ACTIONS(1953),
    [sym_source_type_axlink] = ACTIONS(1953),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1953),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1953),
    [sym_source_type_ip_socket_address] = ACTIONS(1953),
    [sym_source_type_rs232] = ACTIONS(1953),
    [sym_source_type_internal] = ACTIONS(1953),
    [sym_source_type_mac_address] = ACTIONS(1953),
    [sym_source_type_ipv4_port] = ACTIONS(1953),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1953),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1953),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1953),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1953),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1953),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1953),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1953),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1953),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1953),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1953),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1953),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1953),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1953),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1953),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1953),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1953),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1953),
    [sym_true] = ACTIONS(1953),
    [sym_false] = ACTIONS(1953),
    [sym_file_read_only] = ACTIONS(1953),
    [sym_file_rw_new] = ACTIONS(1953),
    [sym_file_rw_append] = ACTIONS(1953),
    [sym_ip_tcp] = ACTIONS(1953),
    [sym_ip_udp] = ACTIONS(1953),
    [sym_ip_udp_2way] = ACTIONS(1953),
    [sym_xml_encode_types] = ACTIONS(1953),
    [sym_xml_encode_char_as_list] = ACTIONS(1953),
    [sym_xml_encode_le] = ACTIONS(1953),
    [sym_xml_decode_types] = ACTIONS(1953),
    [sym_xml_decode_no_preserve] = ACTIONS(1953),
    [sym_duet_dev_type_display_device] = ACTIONS(1953),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1953),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1953),
    [sym_duet_dev_type_io_device] = ACTIONS(1953),
    [sym_duet_dev_type_relay_device] = ACTIONS(1953),
    [sym_duet_dev_type_ups] = ACTIONS(1953),
    [sym_duet_dev_type_amplifier] = ACTIONS(1953),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1953),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1953),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1953),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1953),
    [sym_duet_dev_type_camera] = ACTIONS(1953),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1953),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1953),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1953),
    [sym_duet_dev_type_dss] = ACTIONS(1953),
    [sym_duet_dev_type_dvr] = ACTIONS(1953),
    [sym_duet_dev_type_disc_device] = ACTIONS(1953),
    [sym_duet_dev_type_document_camera] = ACTIONS(1953),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1953),
    [sym_duet_dev_type_hvac] = ACTIONS(1953),
    [sym_duet_dev_type_keypad] = ACTIONS(1953),
    [sym_duet_dev_type_light] = ACTIONS(1953),
    [sym_duet_dev_type_monitor] = ACTIONS(1953),
    [sym_duet_dev_type_motor] = ACTIONS(1953),
    [sym_duet_dev_type_multi_window] = ACTIONS(1953),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1953),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1953),
    [sym_duet_dev_type_receiver] = ACTIONS(1953),
    [sym_duet_dev_type_security_system] = ACTIONS(1953),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1953),
    [sym_duet_dev_type_settop_box] = ACTIONS(1953),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1953),
    [sym_duet_dev_type_switcher] = ACTIONS(1953),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1953),
    [sym_duet_dev_type_tv] = ACTIONS(1953),
    [sym_duet_dev_type_utility] = ACTIONS(1953),
    [sym_duet_dev_type_vcr] = ACTIONS(1953),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1953),
    [sym_duet_dev_type_video_processor] = ACTIONS(1953),
    [sym_duet_dev_type_video_projector] = ACTIONS(1953),
    [sym_duet_dev_type_video_wall] = ACTIONS(1953),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1953),
    [sym_duet_dev_type_weather] = ACTIONS(1953),
    [sym_duet_dev_not_polled] = ACTIONS(1953),
    [sym_duet_dev_polled] = ACTIONS(1953),
    [sym_clkmgr_mode_network] = ACTIONS(1953),
    [sym_clkmgr_mode_standalone] = ACTIONS(1953),
    [sym_normal_standby] = ACTIONS(1953),
    [sym_normal_wake] = ACTIONS(1953),
    [sym_ip_multicast_ttl_option] = ACTIONS(1953),
    [sym_ip_ttl_subnet] = ACTIONS(1953),
    [sym_ip_ttl_site] = ACTIONS(1953),
    [sym_ip_ttl_region] = ACTIONS(1953),
    [sym_ip_ttl_continent] = ACTIONS(1953),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1953),
    [sym_ip_nodelay_on] = ACTIONS(1953),
    [sym_ip_nodelay_off] = ACTIONS(1953),
    [sym_amx_error] = ACTIONS(1953),
    [sym_amx_warning] = ACTIONS(1953),
    [sym_amx_info] = ACTIONS(1953),
    [sym_amx_debug] = ACTIONS(1953),
    [sym_netlinx_login_success] = ACTIONS(1953),
    [sym_netlinx_login_fail] = ACTIONS(1953),
    [sym_netlinx_logout] = ACTIONS(1953),
    [sym_null_str] = ACTIONS(1953),
    [sym_smtp_address] = ACTIONS(1953),
    [sym_smtp_port_number] = ACTIONS(1953),
    [sym_smtp_username] = ACTIONS(1953),
    [sym_smtp_password] = ACTIONS(1953),
    [sym_smtp_require_tls] = ACTIONS(1953),
    [sym_smtp_from] = ACTIONS(1953),
    [sym_smtp_tls_true] = ACTIONS(1953),
    [sym_smtp_tls_false] = ACTIONS(1953),
    [sym_listview_on_row_select_event] = ACTIONS(1953),
    [sym_data_structure_unknown] = ACTIONS(1953),
    [sym_data_structure_datafeed] = ACTIONS(1953),
    [sym_data_structure_datarecordset] = ACTIONS(1953),
    [sym_data_structure_datarecord] = ACTIONS(1953),
    [sym_data_structure_datafield] = ACTIONS(1953),
    [sym_data_max_content_field_count] = ACTIONS(1953),
    [sym_data_max_metadata_field_count] = ACTIONS(1953),
    [sym_data_max_id_length] = ACTIONS(1953),
    [sym_data_max_name_length] = ACTIONS(1953),
    [sym_data_max_description_length] = ACTIONS(1953),
    [sym_data_max_type_length] = ACTIONS(1953),
    [sym_data_max_format_length] = ACTIONS(1953),
    [sym_data_max_source_length] = ACTIONS(1953),
    [sym_data_max_value_length] = ACTIONS(1953),
    [sym_data_max_label_length] = ACTIONS(1953),
    [sym_data_max_url_length] = ACTIONS(1953),
    [sym_max_last_login_info_length] = ACTIONS(1953),
    [sym_data_type_unknown] = ACTIONS(1953),
    [sym_data_type_string] = ACTIONS(1953),
    [sym_data_type_datetime] = ACTIONS(1953),
    [sym_data_type_date] = ACTIONS(1953),
    [sym_data_type_time] = ACTIONS(1953),
    [sym_data_type_image] = ACTIONS(1953),
    [sym_data_format_url] = ACTIONS(1953),
    [sym_data_format_phone] = ACTIONS(1953),
    [sym_data_format_email] = ACTIONS(1953),
    [sym_data_format_iso8601] = ACTIONS(1953),
    [sym_auditlog_priv] = ACTIONS(1953),
    [sym_configuration_priv] = ACTIONS(1953),
    [sym_deviceconfig_priv] = ACTIONS(1953),
    [sym_ftp_priv] = ACTIONS(1953),
    [sym_http_priv] = ACTIONS(1953),
    [sym_networkconfig_priv] = ACTIONS(1953),
    [sym_programport_priv] = ACTIONS(1953),
    [sym_remoteui_priv] = ACTIONS(1953),
    [sym_securitycontrol_priv] = ACTIONS(1953),
    [sym_softwaremanagement_priv] = ACTIONS(1953),
    [sym_terminal_priv] = ACTIONS(1953),
    [sym_tpadmin_priv] = ACTIONS(1953),
    [sym_user1_priv] = ACTIONS(1953),
    [sym_user2_priv] = ACTIONS(1953),
    [sym_user3_priv] = ACTIONS(1953),
    [sym_user4_priv] = ACTIONS(1953),
    [sym_usermanagement_priv] = ACTIONS(1953),
    [sym_valid_account] = ACTIONS(1953),
    [sym_err_name_invalid] = ACTIONS(1953),
    [sym_err_password_invalid] = ACTIONS(1953),
    [sym_err_invalid_account] = ACTIONS(1953),
    [sym_err_no_authorization] = ACTIONS(1953),
    [sym_err_invalid_parameter] = ACTIONS(1953),
    [sym_err_user_locked_out] = ACTIONS(1953),
    [sym_err_user_account_expired] = ACTIONS(1953),
    [sym_err_general_error] = ACTIONS(1953),
    [sym_err_authentication_error] = ACTIONS(1953),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1953),
    [sym_tls_validate_certificate] = ACTIONS(1953),
    [sym_tdata] = ACTIONS(1953),
    [sym_tchannel] = ACTIONS(1953),
    [sym_tlevel] = ACTIONS(1953),
    [sym_tbutton] = ACTIONS(1953),
    [sym_ttimeline] = ACTIONS(1953),
    [sym_tcustom] = ACTIONS(1953),
    [sym_url_struct] = ACTIONS(1953),
    [sym_dns_struct] = ACTIONS(1953),
    [sym_ip_address_struct] = ACTIONS(1953),
    [sym_dev_info_struct] = ACTIONS(1953),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1953),
    [sym_data_feed] = ACTIONS(1953),
    [sym_data_field] = ACTIONS(1953),
    [sym_data_record] = ACTIONS(1953),
    [sym_wc_data_feed] = ACTIONS(1953),
    [sym_wc_data_field] = ACTIONS(1953),
    [sym_wc_data_record] = ACTIONS(1953),
    [sym_last_login_info] = ACTIONS(1953),
    [sym____reserved___] = ACTIONS(1953),
    [sym_date] = ACTIONS(1953),
    [sym_day] = ACTIONS(1953),
    [sym_time] = ACTIONS(1953),
    [sym_ldate] = ACTIONS(1953),
    [sym_push_device] = ACTIONS(1953),
    [sym_push_channel] = ACTIONS(1953),
    [sym_push_devchan] = ACTIONS(1953),
    [sym_release_device] = ACTIONS(1953),
    [sym_release_channel] = ACTIONS(1953),
    [sym_release_devchan] = ACTIONS(1953),
    [sym_master_slot] = ACTIONS(1953),
    [sym_get_pulse_time] = ACTIONS(1953),
    [sym_get_timer] = ACTIONS(1953),
    [sym_master_sn] = ACTIONS(1953),
    [sym_system_number] = ACTIONS(1953),
    [sym_button] = ACTIONS(1953),
    [sym_dv_channel] = ACTIONS(1953),
    [sym_data] = ACTIONS(1953),
    [sym_level] = ACTIONS(1953),
    [sym_timeline] = ACTIONS(1953),
    [sym_channel] = ACTIONS(1953),
    [sym_custom] = ACTIONS(1953),
    [sym_atoi] = ACTIONS(1953),
    [sym_atol] = ACTIONS(1953),
    [sym_atof] = ACTIONS(1953),
    [sym_itoa] = ACTIONS(1953),
    [sym_itohex] = ACTIONS(1953),
    [sym_hextoi] = ACTIONS(1953),
    [sym_ftoa] = ACTIONS(1953),
    [sym_find_string] = ACTIONS(1953),
    [sym_left_string] = ACTIONS(1953),
    [sym_right_string] = ACTIONS(1953),
    [sym_mid_string] = ACTIONS(1953),
    [sym_remove_string] = ACTIONS(1953),
    [sym_get_buffer_string] = ACTIONS(1953),
    [sym_get_buffer_char] = ACTIONS(1953),
    [sym_get_multi_buffer_string] = ACTIONS(1953),
    [sym_length_string] = ACTIONS(1953),
    [sym_max_length_string] = ACTIONS(1953),
    [sym_set_length_string] = ACTIONS(1953),
    [sym_compare_string] = ACTIONS(1953),
    [sym_upper_string] = ACTIONS(1953),
    [sym_lower_string] = ACTIONS(1953),
    [sym_get_last] = ACTIONS(1953),
    [sym_date_to_day] = ACTIONS(1953),
    [sym_date_to_month] = ACTIONS(1953),
    [sym_date_to_year] = ACTIONS(1953),
    [sym_time_to_hour] = ACTIONS(1953),
    [sym_time_to_minute] = ACTIONS(1953),
    [sym_time_to_second] = ACTIONS(1953),
    [sym_day_of_week] = ACTIONS(1953),
    [sym_set_pulse_time] = ACTIONS(1953),
    [sym_set_timer] = ACTIONS(1953),
    [sym_file_open] = ACTIONS(1953),
    [sym_file_close] = ACTIONS(1953),
    [sym_file_read] = ACTIONS(1953),
    [sym_file_read_line] = ACTIONS(1953),
    [sym_file_write] = ACTIONS(1953),
    [sym_file_write_line] = ACTIONS(1953),
    [sym_file_copy] = ACTIONS(1953),
    [sym_file_delete] = ACTIONS(1953),
    [sym_file_rename] = ACTIONS(1953),
    [sym_file_dir] = ACTIONS(1953),
    [sym_file_getdir] = ACTIONS(1953),
    [sym_file_setdir] = ACTIONS(1953),
    [sym_file_create_dir] = ACTIONS(1953),
    [sym_file_remove_dir] = ACTIONS(1953),
    [sym_file_seek] = ACTIONS(1953),
    [sym_length_array] = ACTIONS(1953),
    [sym_max_length_array] = ACTIONS(1953),
    [sym_set_length_array] = ACTIONS(1953),
    [sym_ip_client_open] = ACTIONS(1953),
    [sym_ip_client_close] = ACTIONS(1953),
    [sym_ip_server_open] = ACTIONS(1953),
    [sym_ip_server_close] = ACTIONS(1953),
    [sym_ip_bound_client_open] = ACTIONS(1953),
    [sym_ip_mc_server_open] = ACTIONS(1953),
    [sym_ip_set_option] = ACTIONS(1953),
    [sym_ssh_client_open] = ACTIONS(1953),
    [sym_ssh_client_close] = ACTIONS(1953),
    [sym_tls_client_open] = ACTIONS(1953),
    [sym_tls_client_close] = ACTIONS(1953),
    [sym_do_push] = ACTIONS(1953),
    [sym_do_push_timed] = ACTIONS(1953),
    [sym_do_release] = ACTIONS(1953),
    [sym_device_id] = ACTIONS(1953),
    [sym_device_info] = ACTIONS(1953),
    [sym_device_id_string] = ACTIONS(1953),
    [sym_device_standby] = ACTIONS(1953),
    [sym_device_wake] = ACTIONS(1953),
    [sym_timeline_create] = ACTIONS(1953),
    [sym_timeline_kill] = ACTIONS(1953),
    [sym_timeline_reload] = ACTIONS(1953),
    [sym_timeline_pause] = ACTIONS(1953),
    [sym_timeline_restart] = ACTIONS(1953),
    [sym_timeline_set] = ACTIONS(1953),
    [sym_timeline_get] = ACTIONS(1953),
    [sym_timeline_active] = ACTIONS(1953),
    [sym_timeline_dynamic_id] = ACTIONS(1953),
    [sym_variable_to_string] = ACTIONS(1953),
    [sym_string_to_variable] = ACTIONS(1953),
    [sym_length_variable_to_string] = ACTIONS(1953),
    [sym_type_cast] = ACTIONS(1953),
    [sym_variable_to_xml] = ACTIONS(1953),
    [sym_xml_to_variable] = ACTIONS(1953),
    [sym_length_variable_to_xml] = ACTIONS(1953),
    [sym_redirect_string] = ACTIONS(1953),
    [sym_random_number] = ACTIONS(1953),
    [sym_get_unique_id] = ACTIONS(1953),
    [sym_get_system_number] = ACTIONS(1953),
    [sym_set_system_number] = ACTIONS(1953),
    [sym_get_serial_number] = ACTIONS(1953),
    [sym_get_master_build] = ACTIONS(1953),
    [sym_set_validation_code] = ACTIONS(1953),
    [sym_reboot] = ACTIONS(1953),
    [sym_load_duet_module] = ACTIONS(1953),
    [sym_unload_duet_module] = ACTIONS(1953),
    [sym_duet_mem_size_set] = ACTIONS(1953),
    [sym_duet_mem_size_get] = ACTIONS(1953),
    [sym_dynamic_application_device] = ACTIONS(1953),
    [sym_dynamic_polled_port] = ACTIONS(1953),
    [sym_static_port_binding] = ACTIONS(1953),
    [sym_static_ip_binding] = ACTIONS(1953),
    [sym_get_dns_list] = ACTIONS(1953),
    [sym_set_dns_list] = ACTIONS(1953),
    [sym_get_ip_address] = ACTIONS(1953),
    [sym_set_ip_address] = ACTIONS(1953),
    [sym_get_url_list] = ACTIONS(1953),
    [sym_add_url_entry] = ACTIONS(1953),
    [sym_delete_url_entry] = ACTIONS(1953),
    [sym_abs_value] = ACTIONS(1953),
    [sym_max_value] = ACTIONS(1953),
    [sym_min_value] = ACTIONS(1953),
    [sym_exp_value] = ACTIONS(1953),
    [sym_log_value] = ACTIONS(1953),
    [sym_log10_value] = ACTIONS(1953),
    [sym_power_value] = ACTIONS(1953),
    [sym_sqrt_value] = ACTIONS(1953),
    [sym_raw_be] = ACTIONS(1953),
    [sym_raw_le] = ACTIONS(1953),
    [sym_internal_queue_size_set] = ACTIONS(1953),
    [sym_internal_queue_size_get] = ACTIONS(1953),
    [sym_internal_threshold_set] = ACTIONS(1953),
    [sym_internal_threshold_get] = ACTIONS(1953),
    [sym_rebuilt_event] = ACTIONS(1953),
    [sym_do_custom_event] = ACTIONS(1953),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1953),
    [sym_clkmgr_set_clk_source] = ACTIONS(1953),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1953),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1953),
    [sym_clkmgr_get_timezone] = ACTIONS(1953),
    [sym_clkmgr_set_timezone] = ACTIONS(1953),
    [sym_clkmgr_get_resync_period] = ACTIONS(1953),
    [sym_clkmgr_set_resync_period] = ACTIONS(1953),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1953),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1953),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1953),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1953),
    [sym_clkmgr_get_timeservers] = ACTIONS(1953),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1953),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1953),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1953),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1953),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1953),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1953),
    [sym_set_log_level] = ACTIONS(1953),
    [sym_get_log_level] = ACTIONS(1953),
    [sym_amx_log] = ACTIONS(1953),
    [sym_smtp_server_config_set] = ACTIONS(1953),
    [sym_smtp_server_config_get] = ACTIONS(1953),
    [sym_smtp_send] = ACTIONS(1953),
    [sym_validate_netlinx_account] = ACTIONS(1953),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1953),
    [sym_authenticate_certificate] = ACTIONS(1953),
    [sym_audit_netlinx_session_event] = ACTIONS(1953),
    [sym_audit_netlinx_generic_event] = ACTIONS(1953),
    [sym_get_available_flash_disk_space] = ACTIONS(1953),
    [sym_get_max_flash_disk_space] = ACTIONS(1953),
    [sym_data_create_feed] = ACTIONS(1953),
    [sym_data_delete_feed] = ACTIONS(1953),
    [sym_data_publish_feed] = ACTIONS(1953),
    [sym_data_get_published_feed] = ACTIONS(1953),
    [sym_data_add_record] = ACTIONS(1953),
    [sym_data_get_event_record] = ACTIONS(1953),
    [sym__wc_data_create_feed] = ACTIONS(1953),
    [sym__wc_data_add_record] = ACTIONS(1953),
    [sym__wc_data_get_event_record] = ACTIONS(1953),
    [sym_format] = ACTIONS(1953),
    [sym_astro_clock] = ACTIONS(1953),
    [sym_set_outdoor_temperature] = ACTIONS(1953),
    [sym_set_virtual_level_count] = ACTIONS(1953),
    [sym_set_virtual_channel_count] = ACTIONS(1953),
    [sym_set_virtual_port_count] = ACTIONS(1953),
    [sym__wc_encode_int] = ACTIONS(1953),
    [sym__wc_decode_int] = ACTIONS(1953),
    [sym__wc_int] = ACTIONS(1953),
    [sym__wc_to_ch_int] = ACTIONS(1953),
    [sym__ch_to_wc_int] = ACTIONS(1953),
    [sym__wc_find_string_int] = ACTIONS(1953),
    [sym__wc_left_string_int] = ACTIONS(1953),
    [sym__wc_lower_string_int] = ACTIONS(1953),
    [sym__wc_mid_string_int] = ACTIONS(1953),
    [sym__wc_remove_string_int] = ACTIONS(1953),
    [sym__wc_right_string_int] = ACTIONS(1953),
    [sym__wc_upper_string_int] = ACTIONS(1953),
    [sym__wc_compare_string_int] = ACTIONS(1953),
    [sym__wc_get_buffer_char_int] = ACTIONS(1953),
    [sym__wc_get_buffer_string_int] = ACTIONS(1953),
    [sym__wc_concat_string_int] = ACTIONS(1953),
    [sym__wc_file_open_int] = ACTIONS(1953),
    [sym__wc_file_close_int] = ACTIONS(1953),
    [sym__wc_file_read_int] = ACTIONS(1953),
    [sym__wc_file_read_line_int] = ACTIONS(1953),
    [sym__wc_file_write_int] = ACTIONS(1953),
    [sym__wc_file_write_line_int] = ACTIONS(1953),
    [aux_sym_preproc_include_token1] = ACTIONS(1953),
    [aux_sym_preproc_define_token1] = ACTIONS(1955),
    [aux_sym_preproc_warn_token1] = ACTIONS(1955),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1955),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1955),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1955),
    [anon_sym_LBRACK] = ACTIONS(1955),
    [anon_sym_LBRACE] = ACTIONS(1955),
    [anon_sym_LPAREN] = ACTIONS(1953),
    [anon_sym_BANG] = ACTIONS(1955),
    [anon_sym_TILDE] = ACTIONS(1955),
    [anon_sym_DASH] = ACTIONS(1953),
    [anon_sym_PLUS] = ACTIONS(1953),
    [anon_sym_DASH_DASH] = ACTIONS(1955),
    [anon_sym_PLUS_PLUS] = ACTIONS(1955),
    [anon_sym_DQUOTE] = ACTIONS(1955),
    [anon_sym_SQUOTE] = ACTIONS(1955),
    [sym_number_literal] = ACTIONS(1955),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(411)] = {
    [ts_builtin_sym_end] = ACTIONS(2099),
    [sym_identifier] = ACTIONS(2097),
    [sym_program_name_keyword] = ACTIONS(2097),
    [sym_module_name_keyword] = ACTIONS(2097),
    [sym_define_device_keyword] = ACTIONS(2097),
    [sym_define_combine_keyword] = ACTIONS(2097),
    [sym_define_constant_keyword] = ACTIONS(2097),
    [sym_define_type_keyword] = ACTIONS(2097),
    [sym_define_variable_keyword] = ACTIONS(2097),
    [sym_define_system_variable_keyword] = ACTIONS(2097),
    [sym_define_connect_level_keyword] = ACTIONS(2097),
    [sym_define_latching_keyword] = ACTIONS(2097),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2097),
    [sym_define_toggling_keyword] = ACTIONS(2097),
    [sym_define_call_keyword] = ACTIONS(2097),
    [sym_define_function_keyword] = ACTIONS(2097),
    [sym_define_library_function_keyword] = ACTIONS(2097),
    [sym_define_start_keyword] = ACTIONS(2097),
    [sym_define_event_keyword] = ACTIONS(2097),
    [sym_define_program_keyword] = ACTIONS(2097),
    [sym_define_module_keyword] = ACTIONS(2097),
    [sym_char_keyword] = ACTIONS(2097),
    [sym_widechar_keyword] = ACTIONS(2097),
    [sym_integer_keyword] = ACTIONS(2097),
    [sym_sinteger_keyword] = ACTIONS(2097),
    [sym_long_keyword] = ACTIONS(2097),
    [sym_slong_keyword] = ACTIONS(2097),
    [sym_float_keyword] = ACTIONS(2097),
    [sym_double_keyword] = ACTIONS(2097),
    [sym_variant_keyword] = ACTIONS(2097),
    [sym_variantarray_keyword] = ACTIONS(2097),
    [sym_local_var_keyword] = ACTIONS(2097),
    [sym_stack_var_keyword] = ACTIONS(2097),
    [sym_constant_keyword] = ACTIONS(2097),
    [sym_volatile_keyword] = ACTIONS(2097),
    [sym_non_volatile_keyword] = ACTIONS(2097),
    [sym_persistent_keyword] = ACTIONS(2097),
    [sym_dev_keyword] = ACTIONS(2097),
    [sym_devlev_keyword] = ACTIONS(2097),
    [sym_devchan_keyword] = ACTIONS(2097),
    [sym_if_keyword] = ACTIONS(2097),
    [sym_switch_keyword] = ACTIONS(2097),
    [sym_case_keyword] = ACTIONS(2097),
    [sym_default_keyword] = ACTIONS(2097),
    [sym_while_keyword] = ACTIONS(2097),
    [sym_for_keyword] = ACTIONS(2097),
    [sym_break_keyword] = ACTIONS(2097),
    [sym_continue_keyword] = ACTIONS(2097),
    [sym_return_keyword] = ACTIONS(2097),
    [sym_select_keyword] = ACTIONS(2097),
    [sym_active_keyword] = ACTIONS(2097),
    [sym_struct_keyword] = ACTIONS(2097),
    [sym_structure_keyword] = ACTIONS(2097),
    [sym_button_event_keyword] = ACTIONS(2097),
    [sym_channel_event_keyword] = ACTIONS(2097),
    [sym_level_event_keyword] = ACTIONS(2097),
    [sym_data_event_keyword] = ACTIONS(2097),
    [sym_timeline_event_keyword] = ACTIONS(2097),
    [sym_custom_event_keyword] = ACTIONS(2097),
    [sym_push_keyword] = ACTIONS(2097),
    [sym_release_keyword] = ACTIONS(2097),
    [aux_sym_on_keyword_token1] = ACTIONS(2097),
    [aux_sym_off_keyword_token1] = ACTIONS(2097),
    [sym_send_level_keyword] = ACTIONS(2097),
    [sym_send_string_keyword] = ACTIONS(2097),
    [sym_send_command_keyword] = ACTIONS(2097),
    [sym_clear_buffer_keyword] = ACTIONS(2097),
    [sym_create_buffer_keyword] = ACTIONS(2097),
    [sym_create_multi_buffer_keyword] = ACTIONS(2097),
    [sym_call_keyword] = ACTIONS(2097),
    [sym_system_call_keyword] = ACTIONS(2097),
    [sym_devchan_to_keyword] = ACTIONS(2097),
    [sym_devchan_min_to_keyword] = ACTIONS(2097),
    [sym_devchan_total_off_keyword] = ACTIONS(2097),
    [sym_devchan_pulse_keyword] = ACTIONS(2097),
    [sym_wait_keyword] = ACTIONS(2097),
    [sym_cancel_wait_keyword] = ACTIONS(2097),
    [sym_wait_until_keyword] = ACTIONS(2097),
    [sym_cancel_wait_until_keyword] = ACTIONS(2097),
    [sym_cancel_all_wait_keyword] = ACTIONS(2097),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2097),
    [sym_not] = ACTIONS(2097),
    [sym_bnot] = ACTIONS(2097),
    [sym___file__] = ACTIONS(2097),
    [sym___date__] = ACTIONS(2097),
    [sym___ldate__] = ACTIONS(2097),
    [sym___time__] = ACTIONS(2097),
    [sym___line__] = ACTIONS(2097),
    [sym___name__] = ACTIONS(2097),
    [sym___netlinx__] = ACTIONS(2097),
    [sym_netlinx_axi_version] = ACTIONS(2097),
    [sym_url_flg_tcp] = ACTIONS(2097),
    [sym_url_flg_accinfopresent] = ACTIONS(2097),
    [sym_url_flg_temp] = ACTIONS(2097),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2097),
    [sym_url_flg_stat_mask] = ACTIONS(2097),
    [sym_url_flg_stat_lookup] = ACTIONS(2097),
    [sym_url_flg_stat_connecting] = ACTIONS(2097),
    [sym_url_flg_stat_waiting] = ACTIONS(2097),
    [sym_url_flg_stat_connected] = ACTIONS(2097),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2097),
    [sym_first_virtual_device] = ACTIONS(2097),
    [sym_first_local_port] = ACTIONS(2097),
    [sym_dynamic_virtual_device] = ACTIONS(2097),
    [sym_timeline_once] = ACTIONS(2097),
    [sym_timeline_repeat] = ACTIONS(2097),
    [sym_timeline_absolute] = ACTIONS(2097),
    [sym_timeline_relative] = ACTIONS(2097),
    [sym_do_push_timed_infinite] = ACTIONS(2097),
    [sym_source_type_no_address] = ACTIONS(2097),
    [sym_source_type_neuron_id] = ACTIONS(2097),
    [sym_source_type_ip_address] = ACTIONS(2097),
    [sym_source_type_axlink] = ACTIONS(2097),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2097),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2097),
    [sym_source_type_ip_socket_address] = ACTIONS(2097),
    [sym_source_type_rs232] = ACTIONS(2097),
    [sym_source_type_internal] = ACTIONS(2097),
    [sym_source_type_mac_address] = ACTIONS(2097),
    [sym_source_type_ipv4_port] = ACTIONS(2097),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2097),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2097),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2097),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2097),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2097),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2097),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2097),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2097),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2097),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2097),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2097),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2097),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2097),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2097),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2097),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2097),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2097),
    [sym_true] = ACTIONS(2097),
    [sym_false] = ACTIONS(2097),
    [sym_file_read_only] = ACTIONS(2097),
    [sym_file_rw_new] = ACTIONS(2097),
    [sym_file_rw_append] = ACTIONS(2097),
    [sym_ip_tcp] = ACTIONS(2097),
    [sym_ip_udp] = ACTIONS(2097),
    [sym_ip_udp_2way] = ACTIONS(2097),
    [sym_xml_encode_types] = ACTIONS(2097),
    [sym_xml_encode_char_as_list] = ACTIONS(2097),
    [sym_xml_encode_le] = ACTIONS(2097),
    [sym_xml_decode_types] = ACTIONS(2097),
    [sym_xml_decode_no_preserve] = ACTIONS(2097),
    [sym_duet_dev_type_display_device] = ACTIONS(2097),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2097),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2097),
    [sym_duet_dev_type_io_device] = ACTIONS(2097),
    [sym_duet_dev_type_relay_device] = ACTIONS(2097),
    [sym_duet_dev_type_ups] = ACTIONS(2097),
    [sym_duet_dev_type_amplifier] = ACTIONS(2097),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2097),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2097),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2097),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2097),
    [sym_duet_dev_type_camera] = ACTIONS(2097),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2097),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2097),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2097),
    [sym_duet_dev_type_dss] = ACTIONS(2097),
    [sym_duet_dev_type_dvr] = ACTIONS(2097),
    [sym_duet_dev_type_disc_device] = ACTIONS(2097),
    [sym_duet_dev_type_document_camera] = ACTIONS(2097),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2097),
    [sym_duet_dev_type_hvac] = ACTIONS(2097),
    [sym_duet_dev_type_keypad] = ACTIONS(2097),
    [sym_duet_dev_type_light] = ACTIONS(2097),
    [sym_duet_dev_type_monitor] = ACTIONS(2097),
    [sym_duet_dev_type_motor] = ACTIONS(2097),
    [sym_duet_dev_type_multi_window] = ACTIONS(2097),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2097),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2097),
    [sym_duet_dev_type_receiver] = ACTIONS(2097),
    [sym_duet_dev_type_security_system] = ACTIONS(2097),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2097),
    [sym_duet_dev_type_settop_box] = ACTIONS(2097),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2097),
    [sym_duet_dev_type_switcher] = ACTIONS(2097),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2097),
    [sym_duet_dev_type_tv] = ACTIONS(2097),
    [sym_duet_dev_type_utility] = ACTIONS(2097),
    [sym_duet_dev_type_vcr] = ACTIONS(2097),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2097),
    [sym_duet_dev_type_video_processor] = ACTIONS(2097),
    [sym_duet_dev_type_video_projector] = ACTIONS(2097),
    [sym_duet_dev_type_video_wall] = ACTIONS(2097),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2097),
    [sym_duet_dev_type_weather] = ACTIONS(2097),
    [sym_duet_dev_not_polled] = ACTIONS(2097),
    [sym_duet_dev_polled] = ACTIONS(2097),
    [sym_clkmgr_mode_network] = ACTIONS(2097),
    [sym_clkmgr_mode_standalone] = ACTIONS(2097),
    [sym_normal_standby] = ACTIONS(2097),
    [sym_normal_wake] = ACTIONS(2097),
    [sym_ip_multicast_ttl_option] = ACTIONS(2097),
    [sym_ip_ttl_subnet] = ACTIONS(2097),
    [sym_ip_ttl_site] = ACTIONS(2097),
    [sym_ip_ttl_region] = ACTIONS(2097),
    [sym_ip_ttl_continent] = ACTIONS(2097),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2097),
    [sym_ip_nodelay_on] = ACTIONS(2097),
    [sym_ip_nodelay_off] = ACTIONS(2097),
    [sym_amx_error] = ACTIONS(2097),
    [sym_amx_warning] = ACTIONS(2097),
    [sym_amx_info] = ACTIONS(2097),
    [sym_amx_debug] = ACTIONS(2097),
    [sym_netlinx_login_success] = ACTIONS(2097),
    [sym_netlinx_login_fail] = ACTIONS(2097),
    [sym_netlinx_logout] = ACTIONS(2097),
    [sym_null_str] = ACTIONS(2097),
    [sym_smtp_address] = ACTIONS(2097),
    [sym_smtp_port_number] = ACTIONS(2097),
    [sym_smtp_username] = ACTIONS(2097),
    [sym_smtp_password] = ACTIONS(2097),
    [sym_smtp_require_tls] = ACTIONS(2097),
    [sym_smtp_from] = ACTIONS(2097),
    [sym_smtp_tls_true] = ACTIONS(2097),
    [sym_smtp_tls_false] = ACTIONS(2097),
    [sym_listview_on_row_select_event] = ACTIONS(2097),
    [sym_data_structure_unknown] = ACTIONS(2097),
    [sym_data_structure_datafeed] = ACTIONS(2097),
    [sym_data_structure_datarecordset] = ACTIONS(2097),
    [sym_data_structure_datarecord] = ACTIONS(2097),
    [sym_data_structure_datafield] = ACTIONS(2097),
    [sym_data_max_content_field_count] = ACTIONS(2097),
    [sym_data_max_metadata_field_count] = ACTIONS(2097),
    [sym_data_max_id_length] = ACTIONS(2097),
    [sym_data_max_name_length] = ACTIONS(2097),
    [sym_data_max_description_length] = ACTIONS(2097),
    [sym_data_max_type_length] = ACTIONS(2097),
    [sym_data_max_format_length] = ACTIONS(2097),
    [sym_data_max_source_length] = ACTIONS(2097),
    [sym_data_max_value_length] = ACTIONS(2097),
    [sym_data_max_label_length] = ACTIONS(2097),
    [sym_data_max_url_length] = ACTIONS(2097),
    [sym_max_last_login_info_length] = ACTIONS(2097),
    [sym_data_type_unknown] = ACTIONS(2097),
    [sym_data_type_string] = ACTIONS(2097),
    [sym_data_type_datetime] = ACTIONS(2097),
    [sym_data_type_date] = ACTIONS(2097),
    [sym_data_type_time] = ACTIONS(2097),
    [sym_data_type_image] = ACTIONS(2097),
    [sym_data_format_url] = ACTIONS(2097),
    [sym_data_format_phone] = ACTIONS(2097),
    [sym_data_format_email] = ACTIONS(2097),
    [sym_data_format_iso8601] = ACTIONS(2097),
    [sym_auditlog_priv] = ACTIONS(2097),
    [sym_configuration_priv] = ACTIONS(2097),
    [sym_deviceconfig_priv] = ACTIONS(2097),
    [sym_ftp_priv] = ACTIONS(2097),
    [sym_http_priv] = ACTIONS(2097),
    [sym_networkconfig_priv] = ACTIONS(2097),
    [sym_programport_priv] = ACTIONS(2097),
    [sym_remoteui_priv] = ACTIONS(2097),
    [sym_securitycontrol_priv] = ACTIONS(2097),
    [sym_softwaremanagement_priv] = ACTIONS(2097),
    [sym_terminal_priv] = ACTIONS(2097),
    [sym_tpadmin_priv] = ACTIONS(2097),
    [sym_user1_priv] = ACTIONS(2097),
    [sym_user2_priv] = ACTIONS(2097),
    [sym_user3_priv] = ACTIONS(2097),
    [sym_user4_priv] = ACTIONS(2097),
    [sym_usermanagement_priv] = ACTIONS(2097),
    [sym_valid_account] = ACTIONS(2097),
    [sym_err_name_invalid] = ACTIONS(2097),
    [sym_err_password_invalid] = ACTIONS(2097),
    [sym_err_invalid_account] = ACTIONS(2097),
    [sym_err_no_authorization] = ACTIONS(2097),
    [sym_err_invalid_parameter] = ACTIONS(2097),
    [sym_err_user_locked_out] = ACTIONS(2097),
    [sym_err_user_account_expired] = ACTIONS(2097),
    [sym_err_general_error] = ACTIONS(2097),
    [sym_err_authentication_error] = ACTIONS(2097),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2097),
    [sym_tls_validate_certificate] = ACTIONS(2097),
    [sym_tdata] = ACTIONS(2097),
    [sym_tchannel] = ACTIONS(2097),
    [sym_tlevel] = ACTIONS(2097),
    [sym_tbutton] = ACTIONS(2097),
    [sym_ttimeline] = ACTIONS(2097),
    [sym_tcustom] = ACTIONS(2097),
    [sym_url_struct] = ACTIONS(2097),
    [sym_dns_struct] = ACTIONS(2097),
    [sym_ip_address_struct] = ACTIONS(2097),
    [sym_dev_info_struct] = ACTIONS(2097),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2097),
    [sym_data_feed] = ACTIONS(2097),
    [sym_data_field] = ACTIONS(2097),
    [sym_data_record] = ACTIONS(2097),
    [sym_wc_data_feed] = ACTIONS(2097),
    [sym_wc_data_field] = ACTIONS(2097),
    [sym_wc_data_record] = ACTIONS(2097),
    [sym_last_login_info] = ACTIONS(2097),
    [sym____reserved___] = ACTIONS(2097),
    [sym_date] = ACTIONS(2097),
    [sym_day] = ACTIONS(2097),
    [sym_time] = ACTIONS(2097),
    [sym_ldate] = ACTIONS(2097),
    [sym_push_device] = ACTIONS(2097),
    [sym_push_channel] = ACTIONS(2097),
    [sym_push_devchan] = ACTIONS(2097),
    [sym_release_device] = ACTIONS(2097),
    [sym_release_channel] = ACTIONS(2097),
    [sym_release_devchan] = ACTIONS(2097),
    [sym_master_slot] = ACTIONS(2097),
    [sym_get_pulse_time] = ACTIONS(2097),
    [sym_get_timer] = ACTIONS(2097),
    [sym_master_sn] = ACTIONS(2097),
    [sym_system_number] = ACTIONS(2097),
    [sym_button] = ACTIONS(2097),
    [sym_dv_channel] = ACTIONS(2097),
    [sym_data] = ACTIONS(2097),
    [sym_level] = ACTIONS(2097),
    [sym_timeline] = ACTIONS(2097),
    [sym_channel] = ACTIONS(2097),
    [sym_custom] = ACTIONS(2097),
    [sym_atoi] = ACTIONS(2097),
    [sym_atol] = ACTIONS(2097),
    [sym_atof] = ACTIONS(2097),
    [sym_itoa] = ACTIONS(2097),
    [sym_itohex] = ACTIONS(2097),
    [sym_hextoi] = ACTIONS(2097),
    [sym_ftoa] = ACTIONS(2097),
    [sym_find_string] = ACTIONS(2097),
    [sym_left_string] = ACTIONS(2097),
    [sym_right_string] = ACTIONS(2097),
    [sym_mid_string] = ACTIONS(2097),
    [sym_remove_string] = ACTIONS(2097),
    [sym_get_buffer_string] = ACTIONS(2097),
    [sym_get_buffer_char] = ACTIONS(2097),
    [sym_get_multi_buffer_string] = ACTIONS(2097),
    [sym_length_string] = ACTIONS(2097),
    [sym_max_length_string] = ACTIONS(2097),
    [sym_set_length_string] = ACTIONS(2097),
    [sym_compare_string] = ACTIONS(2097),
    [sym_upper_string] = ACTIONS(2097),
    [sym_lower_string] = ACTIONS(2097),
    [sym_get_last] = ACTIONS(2097),
    [sym_date_to_day] = ACTIONS(2097),
    [sym_date_to_month] = ACTIONS(2097),
    [sym_date_to_year] = ACTIONS(2097),
    [sym_time_to_hour] = ACTIONS(2097),
    [sym_time_to_minute] = ACTIONS(2097),
    [sym_time_to_second] = ACTIONS(2097),
    [sym_day_of_week] = ACTIONS(2097),
    [sym_set_pulse_time] = ACTIONS(2097),
    [sym_set_timer] = ACTIONS(2097),
    [sym_file_open] = ACTIONS(2097),
    [sym_file_close] = ACTIONS(2097),
    [sym_file_read] = ACTIONS(2097),
    [sym_file_read_line] = ACTIONS(2097),
    [sym_file_write] = ACTIONS(2097),
    [sym_file_write_line] = ACTIONS(2097),
    [sym_file_copy] = ACTIONS(2097),
    [sym_file_delete] = ACTIONS(2097),
    [sym_file_rename] = ACTIONS(2097),
    [sym_file_dir] = ACTIONS(2097),
    [sym_file_getdir] = ACTIONS(2097),
    [sym_file_setdir] = ACTIONS(2097),
    [sym_file_create_dir] = ACTIONS(2097),
    [sym_file_remove_dir] = ACTIONS(2097),
    [sym_file_seek] = ACTIONS(2097),
    [sym_length_array] = ACTIONS(2097),
    [sym_max_length_array] = ACTIONS(2097),
    [sym_set_length_array] = ACTIONS(2097),
    [sym_ip_client_open] = ACTIONS(2097),
    [sym_ip_client_close] = ACTIONS(2097),
    [sym_ip_server_open] = ACTIONS(2097),
    [sym_ip_server_close] = ACTIONS(2097),
    [sym_ip_bound_client_open] = ACTIONS(2097),
    [sym_ip_mc_server_open] = ACTIONS(2097),
    [sym_ip_set_option] = ACTIONS(2097),
    [sym_ssh_client_open] = ACTIONS(2097),
    [sym_ssh_client_close] = ACTIONS(2097),
    [sym_tls_client_open] = ACTIONS(2097),
    [sym_tls_client_close] = ACTIONS(2097),
    [sym_do_push] = ACTIONS(2097),
    [sym_do_push_timed] = ACTIONS(2097),
    [sym_do_release] = ACTIONS(2097),
    [sym_device_id] = ACTIONS(2097),
    [sym_device_info] = ACTIONS(2097),
    [sym_device_id_string] = ACTIONS(2097),
    [sym_device_standby] = ACTIONS(2097),
    [sym_device_wake] = ACTIONS(2097),
    [sym_timeline_create] = ACTIONS(2097),
    [sym_timeline_kill] = ACTIONS(2097),
    [sym_timeline_reload] = ACTIONS(2097),
    [sym_timeline_pause] = ACTIONS(2097),
    [sym_timeline_restart] = ACTIONS(2097),
    [sym_timeline_set] = ACTIONS(2097),
    [sym_timeline_get] = ACTIONS(2097),
    [sym_timeline_active] = ACTIONS(2097),
    [sym_timeline_dynamic_id] = ACTIONS(2097),
    [sym_variable_to_string] = ACTIONS(2097),
    [sym_string_to_variable] = ACTIONS(2097),
    [sym_length_variable_to_string] = ACTIONS(2097),
    [sym_type_cast] = ACTIONS(2097),
    [sym_variable_to_xml] = ACTIONS(2097),
    [sym_xml_to_variable] = ACTIONS(2097),
    [sym_length_variable_to_xml] = ACTIONS(2097),
    [sym_redirect_string] = ACTIONS(2097),
    [sym_random_number] = ACTIONS(2097),
    [sym_get_unique_id] = ACTIONS(2097),
    [sym_get_system_number] = ACTIONS(2097),
    [sym_set_system_number] = ACTIONS(2097),
    [sym_get_serial_number] = ACTIONS(2097),
    [sym_get_master_build] = ACTIONS(2097),
    [sym_set_validation_code] = ACTIONS(2097),
    [sym_reboot] = ACTIONS(2097),
    [sym_load_duet_module] = ACTIONS(2097),
    [sym_unload_duet_module] = ACTIONS(2097),
    [sym_duet_mem_size_set] = ACTIONS(2097),
    [sym_duet_mem_size_get] = ACTIONS(2097),
    [sym_dynamic_application_device] = ACTIONS(2097),
    [sym_dynamic_polled_port] = ACTIONS(2097),
    [sym_static_port_binding] = ACTIONS(2097),
    [sym_static_ip_binding] = ACTIONS(2097),
    [sym_get_dns_list] = ACTIONS(2097),
    [sym_set_dns_list] = ACTIONS(2097),
    [sym_get_ip_address] = ACTIONS(2097),
    [sym_set_ip_address] = ACTIONS(2097),
    [sym_get_url_list] = ACTIONS(2097),
    [sym_add_url_entry] = ACTIONS(2097),
    [sym_delete_url_entry] = ACTIONS(2097),
    [sym_abs_value] = ACTIONS(2097),
    [sym_max_value] = ACTIONS(2097),
    [sym_min_value] = ACTIONS(2097),
    [sym_exp_value] = ACTIONS(2097),
    [sym_log_value] = ACTIONS(2097),
    [sym_log10_value] = ACTIONS(2097),
    [sym_power_value] = ACTIONS(2097),
    [sym_sqrt_value] = ACTIONS(2097),
    [sym_raw_be] = ACTIONS(2097),
    [sym_raw_le] = ACTIONS(2097),
    [sym_internal_queue_size_set] = ACTIONS(2097),
    [sym_internal_queue_size_get] = ACTIONS(2097),
    [sym_internal_threshold_set] = ACTIONS(2097),
    [sym_internal_threshold_get] = ACTIONS(2097),
    [sym_rebuilt_event] = ACTIONS(2097),
    [sym_do_custom_event] = ACTIONS(2097),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2097),
    [sym_clkmgr_set_clk_source] = ACTIONS(2097),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2097),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2097),
    [sym_clkmgr_get_timezone] = ACTIONS(2097),
    [sym_clkmgr_set_timezone] = ACTIONS(2097),
    [sym_clkmgr_get_resync_period] = ACTIONS(2097),
    [sym_clkmgr_set_resync_period] = ACTIONS(2097),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2097),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2097),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2097),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2097),
    [sym_clkmgr_get_timeservers] = ACTIONS(2097),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2097),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2097),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2097),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2097),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2097),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2097),
    [sym_set_log_level] = ACTIONS(2097),
    [sym_get_log_level] = ACTIONS(2097),
    [sym_amx_log] = ACTIONS(2097),
    [sym_smtp_server_config_set] = ACTIONS(2097),
    [sym_smtp_server_config_get] = ACTIONS(2097),
    [sym_smtp_send] = ACTIONS(2097),
    [sym_validate_netlinx_account] = ACTIONS(2097),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2097),
    [sym_authenticate_certificate] = ACTIONS(2097),
    [sym_audit_netlinx_session_event] = ACTIONS(2097),
    [sym_audit_netlinx_generic_event] = ACTIONS(2097),
    [sym_get_available_flash_disk_space] = ACTIONS(2097),
    [sym_get_max_flash_disk_space] = ACTIONS(2097),
    [sym_data_create_feed] = ACTIONS(2097),
    [sym_data_delete_feed] = ACTIONS(2097),
    [sym_data_publish_feed] = ACTIONS(2097),
    [sym_data_get_published_feed] = ACTIONS(2097),
    [sym_data_add_record] = ACTIONS(2097),
    [sym_data_get_event_record] = ACTIONS(2097),
    [sym__wc_data_create_feed] = ACTIONS(2097),
    [sym__wc_data_add_record] = ACTIONS(2097),
    [sym__wc_data_get_event_record] = ACTIONS(2097),
    [sym_format] = ACTIONS(2097),
    [sym_astro_clock] = ACTIONS(2097),
    [sym_set_outdoor_temperature] = ACTIONS(2097),
    [sym_set_virtual_level_count] = ACTIONS(2097),
    [sym_set_virtual_channel_count] = ACTIONS(2097),
    [sym_set_virtual_port_count] = ACTIONS(2097),
    [sym__wc_encode_int] = ACTIONS(2097),
    [sym__wc_decode_int] = ACTIONS(2097),
    [sym__wc_int] = ACTIONS(2097),
    [sym__wc_to_ch_int] = ACTIONS(2097),
    [sym__ch_to_wc_int] = ACTIONS(2097),
    [sym__wc_find_string_int] = ACTIONS(2097),
    [sym__wc_left_string_int] = ACTIONS(2097),
    [sym__wc_lower_string_int] = ACTIONS(2097),
    [sym__wc_mid_string_int] = ACTIONS(2097),
    [sym__wc_remove_string_int] = ACTIONS(2097),
    [sym__wc_right_string_int] = ACTIONS(2097),
    [sym__wc_upper_string_int] = ACTIONS(2097),
    [sym__wc_compare_string_int] = ACTIONS(2097),
    [sym__wc_get_buffer_char_int] = ACTIONS(2097),
    [sym__wc_get_buffer_string_int] = ACTIONS(2097),
    [sym__wc_concat_string_int] = ACTIONS(2097),
    [sym__wc_file_open_int] = ACTIONS(2097),
    [sym__wc_file_close_int] = ACTIONS(2097),
    [sym__wc_file_read_int] = ACTIONS(2097),
    [sym__wc_file_read_line_int] = ACTIONS(2097),
    [sym__wc_file_write_int] = ACTIONS(2097),
    [sym__wc_file_write_line_int] = ACTIONS(2097),
    [aux_sym_preproc_include_token1] = ACTIONS(2097),
    [aux_sym_preproc_define_token1] = ACTIONS(2099),
    [aux_sym_preproc_warn_token1] = ACTIONS(2099),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2099),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2099),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2099),
    [anon_sym_LBRACK] = ACTIONS(2099),
    [anon_sym_LBRACE] = ACTIONS(2099),
    [anon_sym_LPAREN] = ACTIONS(2097),
    [anon_sym_BANG] = ACTIONS(2099),
    [anon_sym_TILDE] = ACTIONS(2099),
    [anon_sym_DASH] = ACTIONS(2097),
    [anon_sym_PLUS] = ACTIONS(2097),
    [anon_sym_DASH_DASH] = ACTIONS(2099),
    [anon_sym_PLUS_PLUS] = ACTIONS(2099),
    [anon_sym_DQUOTE] = ACTIONS(2099),
    [anon_sym_SQUOTE] = ACTIONS(2099),
    [sym_number_literal] = ACTIONS(2099),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(412)] = {
    [ts_builtin_sym_end] = ACTIONS(2103),
    [sym_identifier] = ACTIONS(2101),
    [sym_program_name_keyword] = ACTIONS(2101),
    [sym_module_name_keyword] = ACTIONS(2101),
    [sym_define_device_keyword] = ACTIONS(2101),
    [sym_define_combine_keyword] = ACTIONS(2101),
    [sym_define_constant_keyword] = ACTIONS(2101),
    [sym_define_type_keyword] = ACTIONS(2101),
    [sym_define_variable_keyword] = ACTIONS(2101),
    [sym_define_system_variable_keyword] = ACTIONS(2101),
    [sym_define_connect_level_keyword] = ACTIONS(2101),
    [sym_define_latching_keyword] = ACTIONS(2101),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2101),
    [sym_define_toggling_keyword] = ACTIONS(2101),
    [sym_define_call_keyword] = ACTIONS(2101),
    [sym_define_function_keyword] = ACTIONS(2101),
    [sym_define_library_function_keyword] = ACTIONS(2101),
    [sym_define_start_keyword] = ACTIONS(2101),
    [sym_define_event_keyword] = ACTIONS(2101),
    [sym_define_program_keyword] = ACTIONS(2101),
    [sym_define_module_keyword] = ACTIONS(2101),
    [sym_char_keyword] = ACTIONS(2101),
    [sym_widechar_keyword] = ACTIONS(2101),
    [sym_integer_keyword] = ACTIONS(2101),
    [sym_sinteger_keyword] = ACTIONS(2101),
    [sym_long_keyword] = ACTIONS(2101),
    [sym_slong_keyword] = ACTIONS(2101),
    [sym_float_keyword] = ACTIONS(2101),
    [sym_double_keyword] = ACTIONS(2101),
    [sym_variant_keyword] = ACTIONS(2101),
    [sym_variantarray_keyword] = ACTIONS(2101),
    [sym_local_var_keyword] = ACTIONS(2101),
    [sym_stack_var_keyword] = ACTIONS(2101),
    [sym_constant_keyword] = ACTIONS(2101),
    [sym_volatile_keyword] = ACTIONS(2101),
    [sym_non_volatile_keyword] = ACTIONS(2101),
    [sym_persistent_keyword] = ACTIONS(2101),
    [sym_dev_keyword] = ACTIONS(2101),
    [sym_devlev_keyword] = ACTIONS(2101),
    [sym_devchan_keyword] = ACTIONS(2101),
    [sym_if_keyword] = ACTIONS(2101),
    [sym_switch_keyword] = ACTIONS(2101),
    [sym_case_keyword] = ACTIONS(2101),
    [sym_default_keyword] = ACTIONS(2101),
    [sym_while_keyword] = ACTIONS(2101),
    [sym_for_keyword] = ACTIONS(2101),
    [sym_break_keyword] = ACTIONS(2101),
    [sym_continue_keyword] = ACTIONS(2101),
    [sym_return_keyword] = ACTIONS(2101),
    [sym_select_keyword] = ACTIONS(2101),
    [sym_active_keyword] = ACTIONS(2101),
    [sym_struct_keyword] = ACTIONS(2101),
    [sym_structure_keyword] = ACTIONS(2101),
    [sym_button_event_keyword] = ACTIONS(2101),
    [sym_channel_event_keyword] = ACTIONS(2101),
    [sym_level_event_keyword] = ACTIONS(2101),
    [sym_data_event_keyword] = ACTIONS(2101),
    [sym_timeline_event_keyword] = ACTIONS(2101),
    [sym_custom_event_keyword] = ACTIONS(2101),
    [sym_push_keyword] = ACTIONS(2101),
    [sym_release_keyword] = ACTIONS(2101),
    [aux_sym_on_keyword_token1] = ACTIONS(2101),
    [aux_sym_off_keyword_token1] = ACTIONS(2101),
    [sym_send_level_keyword] = ACTIONS(2101),
    [sym_send_string_keyword] = ACTIONS(2101),
    [sym_send_command_keyword] = ACTIONS(2101),
    [sym_clear_buffer_keyword] = ACTIONS(2101),
    [sym_create_buffer_keyword] = ACTIONS(2101),
    [sym_create_multi_buffer_keyword] = ACTIONS(2101),
    [sym_call_keyword] = ACTIONS(2101),
    [sym_system_call_keyword] = ACTIONS(2101),
    [sym_devchan_to_keyword] = ACTIONS(2101),
    [sym_devchan_min_to_keyword] = ACTIONS(2101),
    [sym_devchan_total_off_keyword] = ACTIONS(2101),
    [sym_devchan_pulse_keyword] = ACTIONS(2101),
    [sym_wait_keyword] = ACTIONS(2101),
    [sym_cancel_wait_keyword] = ACTIONS(2101),
    [sym_wait_until_keyword] = ACTIONS(2101),
    [sym_cancel_wait_until_keyword] = ACTIONS(2101),
    [sym_cancel_all_wait_keyword] = ACTIONS(2101),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2101),
    [sym_not] = ACTIONS(2101),
    [sym_bnot] = ACTIONS(2101),
    [sym___file__] = ACTIONS(2101),
    [sym___date__] = ACTIONS(2101),
    [sym___ldate__] = ACTIONS(2101),
    [sym___time__] = ACTIONS(2101),
    [sym___line__] = ACTIONS(2101),
    [sym___name__] = ACTIONS(2101),
    [sym___netlinx__] = ACTIONS(2101),
    [sym_netlinx_axi_version] = ACTIONS(2101),
    [sym_url_flg_tcp] = ACTIONS(2101),
    [sym_url_flg_accinfopresent] = ACTIONS(2101),
    [sym_url_flg_temp] = ACTIONS(2101),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2101),
    [sym_url_flg_stat_mask] = ACTIONS(2101),
    [sym_url_flg_stat_lookup] = ACTIONS(2101),
    [sym_url_flg_stat_connecting] = ACTIONS(2101),
    [sym_url_flg_stat_waiting] = ACTIONS(2101),
    [sym_url_flg_stat_connected] = ACTIONS(2101),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2101),
    [sym_first_virtual_device] = ACTIONS(2101),
    [sym_first_local_port] = ACTIONS(2101),
    [sym_dynamic_virtual_device] = ACTIONS(2101),
    [sym_timeline_once] = ACTIONS(2101),
    [sym_timeline_repeat] = ACTIONS(2101),
    [sym_timeline_absolute] = ACTIONS(2101),
    [sym_timeline_relative] = ACTIONS(2101),
    [sym_do_push_timed_infinite] = ACTIONS(2101),
    [sym_source_type_no_address] = ACTIONS(2101),
    [sym_source_type_neuron_id] = ACTIONS(2101),
    [sym_source_type_ip_address] = ACTIONS(2101),
    [sym_source_type_axlink] = ACTIONS(2101),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2101),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2101),
    [sym_source_type_ip_socket_address] = ACTIONS(2101),
    [sym_source_type_rs232] = ACTIONS(2101),
    [sym_source_type_internal] = ACTIONS(2101),
    [sym_source_type_mac_address] = ACTIONS(2101),
    [sym_source_type_ipv4_port] = ACTIONS(2101),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2101),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2101),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2101),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2101),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2101),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2101),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2101),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2101),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2101),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2101),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2101),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2101),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2101),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2101),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2101),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2101),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2101),
    [sym_true] = ACTIONS(2101),
    [sym_false] = ACTIONS(2101),
    [sym_file_read_only] = ACTIONS(2101),
    [sym_file_rw_new] = ACTIONS(2101),
    [sym_file_rw_append] = ACTIONS(2101),
    [sym_ip_tcp] = ACTIONS(2101),
    [sym_ip_udp] = ACTIONS(2101),
    [sym_ip_udp_2way] = ACTIONS(2101),
    [sym_xml_encode_types] = ACTIONS(2101),
    [sym_xml_encode_char_as_list] = ACTIONS(2101),
    [sym_xml_encode_le] = ACTIONS(2101),
    [sym_xml_decode_types] = ACTIONS(2101),
    [sym_xml_decode_no_preserve] = ACTIONS(2101),
    [sym_duet_dev_type_display_device] = ACTIONS(2101),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2101),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2101),
    [sym_duet_dev_type_io_device] = ACTIONS(2101),
    [sym_duet_dev_type_relay_device] = ACTIONS(2101),
    [sym_duet_dev_type_ups] = ACTIONS(2101),
    [sym_duet_dev_type_amplifier] = ACTIONS(2101),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2101),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2101),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2101),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2101),
    [sym_duet_dev_type_camera] = ACTIONS(2101),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2101),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2101),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2101),
    [sym_duet_dev_type_dss] = ACTIONS(2101),
    [sym_duet_dev_type_dvr] = ACTIONS(2101),
    [sym_duet_dev_type_disc_device] = ACTIONS(2101),
    [sym_duet_dev_type_document_camera] = ACTIONS(2101),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2101),
    [sym_duet_dev_type_hvac] = ACTIONS(2101),
    [sym_duet_dev_type_keypad] = ACTIONS(2101),
    [sym_duet_dev_type_light] = ACTIONS(2101),
    [sym_duet_dev_type_monitor] = ACTIONS(2101),
    [sym_duet_dev_type_motor] = ACTIONS(2101),
    [sym_duet_dev_type_multi_window] = ACTIONS(2101),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2101),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2101),
    [sym_duet_dev_type_receiver] = ACTIONS(2101),
    [sym_duet_dev_type_security_system] = ACTIONS(2101),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2101),
    [sym_duet_dev_type_settop_box] = ACTIONS(2101),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2101),
    [sym_duet_dev_type_switcher] = ACTIONS(2101),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2101),
    [sym_duet_dev_type_tv] = ACTIONS(2101),
    [sym_duet_dev_type_utility] = ACTIONS(2101),
    [sym_duet_dev_type_vcr] = ACTIONS(2101),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2101),
    [sym_duet_dev_type_video_processor] = ACTIONS(2101),
    [sym_duet_dev_type_video_projector] = ACTIONS(2101),
    [sym_duet_dev_type_video_wall] = ACTIONS(2101),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2101),
    [sym_duet_dev_type_weather] = ACTIONS(2101),
    [sym_duet_dev_not_polled] = ACTIONS(2101),
    [sym_duet_dev_polled] = ACTIONS(2101),
    [sym_clkmgr_mode_network] = ACTIONS(2101),
    [sym_clkmgr_mode_standalone] = ACTIONS(2101),
    [sym_normal_standby] = ACTIONS(2101),
    [sym_normal_wake] = ACTIONS(2101),
    [sym_ip_multicast_ttl_option] = ACTIONS(2101),
    [sym_ip_ttl_subnet] = ACTIONS(2101),
    [sym_ip_ttl_site] = ACTIONS(2101),
    [sym_ip_ttl_region] = ACTIONS(2101),
    [sym_ip_ttl_continent] = ACTIONS(2101),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2101),
    [sym_ip_nodelay_on] = ACTIONS(2101),
    [sym_ip_nodelay_off] = ACTIONS(2101),
    [sym_amx_error] = ACTIONS(2101),
    [sym_amx_warning] = ACTIONS(2101),
    [sym_amx_info] = ACTIONS(2101),
    [sym_amx_debug] = ACTIONS(2101),
    [sym_netlinx_login_success] = ACTIONS(2101),
    [sym_netlinx_login_fail] = ACTIONS(2101),
    [sym_netlinx_logout] = ACTIONS(2101),
    [sym_null_str] = ACTIONS(2101),
    [sym_smtp_address] = ACTIONS(2101),
    [sym_smtp_port_number] = ACTIONS(2101),
    [sym_smtp_username] = ACTIONS(2101),
    [sym_smtp_password] = ACTIONS(2101),
    [sym_smtp_require_tls] = ACTIONS(2101),
    [sym_smtp_from] = ACTIONS(2101),
    [sym_smtp_tls_true] = ACTIONS(2101),
    [sym_smtp_tls_false] = ACTIONS(2101),
    [sym_listview_on_row_select_event] = ACTIONS(2101),
    [sym_data_structure_unknown] = ACTIONS(2101),
    [sym_data_structure_datafeed] = ACTIONS(2101),
    [sym_data_structure_datarecordset] = ACTIONS(2101),
    [sym_data_structure_datarecord] = ACTIONS(2101),
    [sym_data_structure_datafield] = ACTIONS(2101),
    [sym_data_max_content_field_count] = ACTIONS(2101),
    [sym_data_max_metadata_field_count] = ACTIONS(2101),
    [sym_data_max_id_length] = ACTIONS(2101),
    [sym_data_max_name_length] = ACTIONS(2101),
    [sym_data_max_description_length] = ACTIONS(2101),
    [sym_data_max_type_length] = ACTIONS(2101),
    [sym_data_max_format_length] = ACTIONS(2101),
    [sym_data_max_source_length] = ACTIONS(2101),
    [sym_data_max_value_length] = ACTIONS(2101),
    [sym_data_max_label_length] = ACTIONS(2101),
    [sym_data_max_url_length] = ACTIONS(2101),
    [sym_max_last_login_info_length] = ACTIONS(2101),
    [sym_data_type_unknown] = ACTIONS(2101),
    [sym_data_type_string] = ACTIONS(2101),
    [sym_data_type_datetime] = ACTIONS(2101),
    [sym_data_type_date] = ACTIONS(2101),
    [sym_data_type_time] = ACTIONS(2101),
    [sym_data_type_image] = ACTIONS(2101),
    [sym_data_format_url] = ACTIONS(2101),
    [sym_data_format_phone] = ACTIONS(2101),
    [sym_data_format_email] = ACTIONS(2101),
    [sym_data_format_iso8601] = ACTIONS(2101),
    [sym_auditlog_priv] = ACTIONS(2101),
    [sym_configuration_priv] = ACTIONS(2101),
    [sym_deviceconfig_priv] = ACTIONS(2101),
    [sym_ftp_priv] = ACTIONS(2101),
    [sym_http_priv] = ACTIONS(2101),
    [sym_networkconfig_priv] = ACTIONS(2101),
    [sym_programport_priv] = ACTIONS(2101),
    [sym_remoteui_priv] = ACTIONS(2101),
    [sym_securitycontrol_priv] = ACTIONS(2101),
    [sym_softwaremanagement_priv] = ACTIONS(2101),
    [sym_terminal_priv] = ACTIONS(2101),
    [sym_tpadmin_priv] = ACTIONS(2101),
    [sym_user1_priv] = ACTIONS(2101),
    [sym_user2_priv] = ACTIONS(2101),
    [sym_user3_priv] = ACTIONS(2101),
    [sym_user4_priv] = ACTIONS(2101),
    [sym_usermanagement_priv] = ACTIONS(2101),
    [sym_valid_account] = ACTIONS(2101),
    [sym_err_name_invalid] = ACTIONS(2101),
    [sym_err_password_invalid] = ACTIONS(2101),
    [sym_err_invalid_account] = ACTIONS(2101),
    [sym_err_no_authorization] = ACTIONS(2101),
    [sym_err_invalid_parameter] = ACTIONS(2101),
    [sym_err_user_locked_out] = ACTIONS(2101),
    [sym_err_user_account_expired] = ACTIONS(2101),
    [sym_err_general_error] = ACTIONS(2101),
    [sym_err_authentication_error] = ACTIONS(2101),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2101),
    [sym_tls_validate_certificate] = ACTIONS(2101),
    [sym_tdata] = ACTIONS(2101),
    [sym_tchannel] = ACTIONS(2101),
    [sym_tlevel] = ACTIONS(2101),
    [sym_tbutton] = ACTIONS(2101),
    [sym_ttimeline] = ACTIONS(2101),
    [sym_tcustom] = ACTIONS(2101),
    [sym_url_struct] = ACTIONS(2101),
    [sym_dns_struct] = ACTIONS(2101),
    [sym_ip_address_struct] = ACTIONS(2101),
    [sym_dev_info_struct] = ACTIONS(2101),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2101),
    [sym_data_feed] = ACTIONS(2101),
    [sym_data_field] = ACTIONS(2101),
    [sym_data_record] = ACTIONS(2101),
    [sym_wc_data_feed] = ACTIONS(2101),
    [sym_wc_data_field] = ACTIONS(2101),
    [sym_wc_data_record] = ACTIONS(2101),
    [sym_last_login_info] = ACTIONS(2101),
    [sym____reserved___] = ACTIONS(2101),
    [sym_date] = ACTIONS(2101),
    [sym_day] = ACTIONS(2101),
    [sym_time] = ACTIONS(2101),
    [sym_ldate] = ACTIONS(2101),
    [sym_push_device] = ACTIONS(2101),
    [sym_push_channel] = ACTIONS(2101),
    [sym_push_devchan] = ACTIONS(2101),
    [sym_release_device] = ACTIONS(2101),
    [sym_release_channel] = ACTIONS(2101),
    [sym_release_devchan] = ACTIONS(2101),
    [sym_master_slot] = ACTIONS(2101),
    [sym_get_pulse_time] = ACTIONS(2101),
    [sym_get_timer] = ACTIONS(2101),
    [sym_master_sn] = ACTIONS(2101),
    [sym_system_number] = ACTIONS(2101),
    [sym_button] = ACTIONS(2101),
    [sym_dv_channel] = ACTIONS(2101),
    [sym_data] = ACTIONS(2101),
    [sym_level] = ACTIONS(2101),
    [sym_timeline] = ACTIONS(2101),
    [sym_channel] = ACTIONS(2101),
    [sym_custom] = ACTIONS(2101),
    [sym_atoi] = ACTIONS(2101),
    [sym_atol] = ACTIONS(2101),
    [sym_atof] = ACTIONS(2101),
    [sym_itoa] = ACTIONS(2101),
    [sym_itohex] = ACTIONS(2101),
    [sym_hextoi] = ACTIONS(2101),
    [sym_ftoa] = ACTIONS(2101),
    [sym_find_string] = ACTIONS(2101),
    [sym_left_string] = ACTIONS(2101),
    [sym_right_string] = ACTIONS(2101),
    [sym_mid_string] = ACTIONS(2101),
    [sym_remove_string] = ACTIONS(2101),
    [sym_get_buffer_string] = ACTIONS(2101),
    [sym_get_buffer_char] = ACTIONS(2101),
    [sym_get_multi_buffer_string] = ACTIONS(2101),
    [sym_length_string] = ACTIONS(2101),
    [sym_max_length_string] = ACTIONS(2101),
    [sym_set_length_string] = ACTIONS(2101),
    [sym_compare_string] = ACTIONS(2101),
    [sym_upper_string] = ACTIONS(2101),
    [sym_lower_string] = ACTIONS(2101),
    [sym_get_last] = ACTIONS(2101),
    [sym_date_to_day] = ACTIONS(2101),
    [sym_date_to_month] = ACTIONS(2101),
    [sym_date_to_year] = ACTIONS(2101),
    [sym_time_to_hour] = ACTIONS(2101),
    [sym_time_to_minute] = ACTIONS(2101),
    [sym_time_to_second] = ACTIONS(2101),
    [sym_day_of_week] = ACTIONS(2101),
    [sym_set_pulse_time] = ACTIONS(2101),
    [sym_set_timer] = ACTIONS(2101),
    [sym_file_open] = ACTIONS(2101),
    [sym_file_close] = ACTIONS(2101),
    [sym_file_read] = ACTIONS(2101),
    [sym_file_read_line] = ACTIONS(2101),
    [sym_file_write] = ACTIONS(2101),
    [sym_file_write_line] = ACTIONS(2101),
    [sym_file_copy] = ACTIONS(2101),
    [sym_file_delete] = ACTIONS(2101),
    [sym_file_rename] = ACTIONS(2101),
    [sym_file_dir] = ACTIONS(2101),
    [sym_file_getdir] = ACTIONS(2101),
    [sym_file_setdir] = ACTIONS(2101),
    [sym_file_create_dir] = ACTIONS(2101),
    [sym_file_remove_dir] = ACTIONS(2101),
    [sym_file_seek] = ACTIONS(2101),
    [sym_length_array] = ACTIONS(2101),
    [sym_max_length_array] = ACTIONS(2101),
    [sym_set_length_array] = ACTIONS(2101),
    [sym_ip_client_open] = ACTIONS(2101),
    [sym_ip_client_close] = ACTIONS(2101),
    [sym_ip_server_open] = ACTIONS(2101),
    [sym_ip_server_close] = ACTIONS(2101),
    [sym_ip_bound_client_open] = ACTIONS(2101),
    [sym_ip_mc_server_open] = ACTIONS(2101),
    [sym_ip_set_option] = ACTIONS(2101),
    [sym_ssh_client_open] = ACTIONS(2101),
    [sym_ssh_client_close] = ACTIONS(2101),
    [sym_tls_client_open] = ACTIONS(2101),
    [sym_tls_client_close] = ACTIONS(2101),
    [sym_do_push] = ACTIONS(2101),
    [sym_do_push_timed] = ACTIONS(2101),
    [sym_do_release] = ACTIONS(2101),
    [sym_device_id] = ACTIONS(2101),
    [sym_device_info] = ACTIONS(2101),
    [sym_device_id_string] = ACTIONS(2101),
    [sym_device_standby] = ACTIONS(2101),
    [sym_device_wake] = ACTIONS(2101),
    [sym_timeline_create] = ACTIONS(2101),
    [sym_timeline_kill] = ACTIONS(2101),
    [sym_timeline_reload] = ACTIONS(2101),
    [sym_timeline_pause] = ACTIONS(2101),
    [sym_timeline_restart] = ACTIONS(2101),
    [sym_timeline_set] = ACTIONS(2101),
    [sym_timeline_get] = ACTIONS(2101),
    [sym_timeline_active] = ACTIONS(2101),
    [sym_timeline_dynamic_id] = ACTIONS(2101),
    [sym_variable_to_string] = ACTIONS(2101),
    [sym_string_to_variable] = ACTIONS(2101),
    [sym_length_variable_to_string] = ACTIONS(2101),
    [sym_type_cast] = ACTIONS(2101),
    [sym_variable_to_xml] = ACTIONS(2101),
    [sym_xml_to_variable] = ACTIONS(2101),
    [sym_length_variable_to_xml] = ACTIONS(2101),
    [sym_redirect_string] = ACTIONS(2101),
    [sym_random_number] = ACTIONS(2101),
    [sym_get_unique_id] = ACTIONS(2101),
    [sym_get_system_number] = ACTIONS(2101),
    [sym_set_system_number] = ACTIONS(2101),
    [sym_get_serial_number] = ACTIONS(2101),
    [sym_get_master_build] = ACTIONS(2101),
    [sym_set_validation_code] = ACTIONS(2101),
    [sym_reboot] = ACTIONS(2101),
    [sym_load_duet_module] = ACTIONS(2101),
    [sym_unload_duet_module] = ACTIONS(2101),
    [sym_duet_mem_size_set] = ACTIONS(2101),
    [sym_duet_mem_size_get] = ACTIONS(2101),
    [sym_dynamic_application_device] = ACTIONS(2101),
    [sym_dynamic_polled_port] = ACTIONS(2101),
    [sym_static_port_binding] = ACTIONS(2101),
    [sym_static_ip_binding] = ACTIONS(2101),
    [sym_get_dns_list] = ACTIONS(2101),
    [sym_set_dns_list] = ACTIONS(2101),
    [sym_get_ip_address] = ACTIONS(2101),
    [sym_set_ip_address] = ACTIONS(2101),
    [sym_get_url_list] = ACTIONS(2101),
    [sym_add_url_entry] = ACTIONS(2101),
    [sym_delete_url_entry] = ACTIONS(2101),
    [sym_abs_value] = ACTIONS(2101),
    [sym_max_value] = ACTIONS(2101),
    [sym_min_value] = ACTIONS(2101),
    [sym_exp_value] = ACTIONS(2101),
    [sym_log_value] = ACTIONS(2101),
    [sym_log10_value] = ACTIONS(2101),
    [sym_power_value] = ACTIONS(2101),
    [sym_sqrt_value] = ACTIONS(2101),
    [sym_raw_be] = ACTIONS(2101),
    [sym_raw_le] = ACTIONS(2101),
    [sym_internal_queue_size_set] = ACTIONS(2101),
    [sym_internal_queue_size_get] = ACTIONS(2101),
    [sym_internal_threshold_set] = ACTIONS(2101),
    [sym_internal_threshold_get] = ACTIONS(2101),
    [sym_rebuilt_event] = ACTIONS(2101),
    [sym_do_custom_event] = ACTIONS(2101),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2101),
    [sym_clkmgr_set_clk_source] = ACTIONS(2101),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2101),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2101),
    [sym_clkmgr_get_timezone] = ACTIONS(2101),
    [sym_clkmgr_set_timezone] = ACTIONS(2101),
    [sym_clkmgr_get_resync_period] = ACTIONS(2101),
    [sym_clkmgr_set_resync_period] = ACTIONS(2101),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2101),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2101),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2101),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2101),
    [sym_clkmgr_get_timeservers] = ACTIONS(2101),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2101),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2101),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2101),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2101),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2101),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2101),
    [sym_set_log_level] = ACTIONS(2101),
    [sym_get_log_level] = ACTIONS(2101),
    [sym_amx_log] = ACTIONS(2101),
    [sym_smtp_server_config_set] = ACTIONS(2101),
    [sym_smtp_server_config_get] = ACTIONS(2101),
    [sym_smtp_send] = ACTIONS(2101),
    [sym_validate_netlinx_account] = ACTIONS(2101),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2101),
    [sym_authenticate_certificate] = ACTIONS(2101),
    [sym_audit_netlinx_session_event] = ACTIONS(2101),
    [sym_audit_netlinx_generic_event] = ACTIONS(2101),
    [sym_get_available_flash_disk_space] = ACTIONS(2101),
    [sym_get_max_flash_disk_space] = ACTIONS(2101),
    [sym_data_create_feed] = ACTIONS(2101),
    [sym_data_delete_feed] = ACTIONS(2101),
    [sym_data_publish_feed] = ACTIONS(2101),
    [sym_data_get_published_feed] = ACTIONS(2101),
    [sym_data_add_record] = ACTIONS(2101),
    [sym_data_get_event_record] = ACTIONS(2101),
    [sym__wc_data_create_feed] = ACTIONS(2101),
    [sym__wc_data_add_record] = ACTIONS(2101),
    [sym__wc_data_get_event_record] = ACTIONS(2101),
    [sym_format] = ACTIONS(2101),
    [sym_astro_clock] = ACTIONS(2101),
    [sym_set_outdoor_temperature] = ACTIONS(2101),
    [sym_set_virtual_level_count] = ACTIONS(2101),
    [sym_set_virtual_channel_count] = ACTIONS(2101),
    [sym_set_virtual_port_count] = ACTIONS(2101),
    [sym__wc_encode_int] = ACTIONS(2101),
    [sym__wc_decode_int] = ACTIONS(2101),
    [sym__wc_int] = ACTIONS(2101),
    [sym__wc_to_ch_int] = ACTIONS(2101),
    [sym__ch_to_wc_int] = ACTIONS(2101),
    [sym__wc_find_string_int] = ACTIONS(2101),
    [sym__wc_left_string_int] = ACTIONS(2101),
    [sym__wc_lower_string_int] = ACTIONS(2101),
    [sym__wc_mid_string_int] = ACTIONS(2101),
    [sym__wc_remove_string_int] = ACTIONS(2101),
    [sym__wc_right_string_int] = ACTIONS(2101),
    [sym__wc_upper_string_int] = ACTIONS(2101),
    [sym__wc_compare_string_int] = ACTIONS(2101),
    [sym__wc_get_buffer_char_int] = ACTIONS(2101),
    [sym__wc_get_buffer_string_int] = ACTIONS(2101),
    [sym__wc_concat_string_int] = ACTIONS(2101),
    [sym__wc_file_open_int] = ACTIONS(2101),
    [sym__wc_file_close_int] = ACTIONS(2101),
    [sym__wc_file_read_int] = ACTIONS(2101),
    [sym__wc_file_read_line_int] = ACTIONS(2101),
    [sym__wc_file_write_int] = ACTIONS(2101),
    [sym__wc_file_write_line_int] = ACTIONS(2101),
    [aux_sym_preproc_include_token1] = ACTIONS(2101),
    [aux_sym_preproc_define_token1] = ACTIONS(2103),
    [aux_sym_preproc_warn_token1] = ACTIONS(2103),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2103),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2103),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2103),
    [anon_sym_LBRACK] = ACTIONS(2103),
    [anon_sym_LBRACE] = ACTIONS(2103),
    [anon_sym_LPAREN] = ACTIONS(2101),
    [anon_sym_BANG] = ACTIONS(2103),
    [anon_sym_TILDE] = ACTIONS(2103),
    [anon_sym_DASH] = ACTIONS(2101),
    [anon_sym_PLUS] = ACTIONS(2101),
    [anon_sym_DASH_DASH] = ACTIONS(2103),
    [anon_sym_PLUS_PLUS] = ACTIONS(2103),
    [anon_sym_DQUOTE] = ACTIONS(2103),
    [anon_sym_SQUOTE] = ACTIONS(2103),
    [sym_number_literal] = ACTIONS(2103),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(413)] = {
    [ts_builtin_sym_end] = ACTIONS(2107),
    [sym_identifier] = ACTIONS(2105),
    [sym_program_name_keyword] = ACTIONS(2105),
    [sym_module_name_keyword] = ACTIONS(2105),
    [sym_define_device_keyword] = ACTIONS(2105),
    [sym_define_combine_keyword] = ACTIONS(2105),
    [sym_define_constant_keyword] = ACTIONS(2105),
    [sym_define_type_keyword] = ACTIONS(2105),
    [sym_define_variable_keyword] = ACTIONS(2105),
    [sym_define_system_variable_keyword] = ACTIONS(2105),
    [sym_define_connect_level_keyword] = ACTIONS(2105),
    [sym_define_latching_keyword] = ACTIONS(2105),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2105),
    [sym_define_toggling_keyword] = ACTIONS(2105),
    [sym_define_call_keyword] = ACTIONS(2105),
    [sym_define_function_keyword] = ACTIONS(2105),
    [sym_define_library_function_keyword] = ACTIONS(2105),
    [sym_define_start_keyword] = ACTIONS(2105),
    [sym_define_event_keyword] = ACTIONS(2105),
    [sym_define_program_keyword] = ACTIONS(2105),
    [sym_define_module_keyword] = ACTIONS(2105),
    [sym_char_keyword] = ACTIONS(2105),
    [sym_widechar_keyword] = ACTIONS(2105),
    [sym_integer_keyword] = ACTIONS(2105),
    [sym_sinteger_keyword] = ACTIONS(2105),
    [sym_long_keyword] = ACTIONS(2105),
    [sym_slong_keyword] = ACTIONS(2105),
    [sym_float_keyword] = ACTIONS(2105),
    [sym_double_keyword] = ACTIONS(2105),
    [sym_variant_keyword] = ACTIONS(2105),
    [sym_variantarray_keyword] = ACTIONS(2105),
    [sym_local_var_keyword] = ACTIONS(2105),
    [sym_stack_var_keyword] = ACTIONS(2105),
    [sym_constant_keyword] = ACTIONS(2105),
    [sym_volatile_keyword] = ACTIONS(2105),
    [sym_non_volatile_keyword] = ACTIONS(2105),
    [sym_persistent_keyword] = ACTIONS(2105),
    [sym_dev_keyword] = ACTIONS(2105),
    [sym_devlev_keyword] = ACTIONS(2105),
    [sym_devchan_keyword] = ACTIONS(2105),
    [sym_if_keyword] = ACTIONS(2105),
    [sym_switch_keyword] = ACTIONS(2105),
    [sym_case_keyword] = ACTIONS(2105),
    [sym_default_keyword] = ACTIONS(2105),
    [sym_while_keyword] = ACTIONS(2105),
    [sym_for_keyword] = ACTIONS(2105),
    [sym_break_keyword] = ACTIONS(2105),
    [sym_continue_keyword] = ACTIONS(2105),
    [sym_return_keyword] = ACTIONS(2105),
    [sym_select_keyword] = ACTIONS(2105),
    [sym_active_keyword] = ACTIONS(2105),
    [sym_struct_keyword] = ACTIONS(2105),
    [sym_structure_keyword] = ACTIONS(2105),
    [sym_button_event_keyword] = ACTIONS(2105),
    [sym_channel_event_keyword] = ACTIONS(2105),
    [sym_level_event_keyword] = ACTIONS(2105),
    [sym_data_event_keyword] = ACTIONS(2105),
    [sym_timeline_event_keyword] = ACTIONS(2105),
    [sym_custom_event_keyword] = ACTIONS(2105),
    [sym_push_keyword] = ACTIONS(2105),
    [sym_release_keyword] = ACTIONS(2105),
    [aux_sym_on_keyword_token1] = ACTIONS(2105),
    [aux_sym_off_keyword_token1] = ACTIONS(2105),
    [sym_send_level_keyword] = ACTIONS(2105),
    [sym_send_string_keyword] = ACTIONS(2105),
    [sym_send_command_keyword] = ACTIONS(2105),
    [sym_clear_buffer_keyword] = ACTIONS(2105),
    [sym_create_buffer_keyword] = ACTIONS(2105),
    [sym_create_multi_buffer_keyword] = ACTIONS(2105),
    [sym_call_keyword] = ACTIONS(2105),
    [sym_system_call_keyword] = ACTIONS(2105),
    [sym_devchan_to_keyword] = ACTIONS(2105),
    [sym_devchan_min_to_keyword] = ACTIONS(2105),
    [sym_devchan_total_off_keyword] = ACTIONS(2105),
    [sym_devchan_pulse_keyword] = ACTIONS(2105),
    [sym_wait_keyword] = ACTIONS(2105),
    [sym_cancel_wait_keyword] = ACTIONS(2105),
    [sym_wait_until_keyword] = ACTIONS(2105),
    [sym_cancel_wait_until_keyword] = ACTIONS(2105),
    [sym_cancel_all_wait_keyword] = ACTIONS(2105),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2105),
    [sym_not] = ACTIONS(2105),
    [sym_bnot] = ACTIONS(2105),
    [sym___file__] = ACTIONS(2105),
    [sym___date__] = ACTIONS(2105),
    [sym___ldate__] = ACTIONS(2105),
    [sym___time__] = ACTIONS(2105),
    [sym___line__] = ACTIONS(2105),
    [sym___name__] = ACTIONS(2105),
    [sym___netlinx__] = ACTIONS(2105),
    [sym_netlinx_axi_version] = ACTIONS(2105),
    [sym_url_flg_tcp] = ACTIONS(2105),
    [sym_url_flg_accinfopresent] = ACTIONS(2105),
    [sym_url_flg_temp] = ACTIONS(2105),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2105),
    [sym_url_flg_stat_mask] = ACTIONS(2105),
    [sym_url_flg_stat_lookup] = ACTIONS(2105),
    [sym_url_flg_stat_connecting] = ACTIONS(2105),
    [sym_url_flg_stat_waiting] = ACTIONS(2105),
    [sym_url_flg_stat_connected] = ACTIONS(2105),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2105),
    [sym_first_virtual_device] = ACTIONS(2105),
    [sym_first_local_port] = ACTIONS(2105),
    [sym_dynamic_virtual_device] = ACTIONS(2105),
    [sym_timeline_once] = ACTIONS(2105),
    [sym_timeline_repeat] = ACTIONS(2105),
    [sym_timeline_absolute] = ACTIONS(2105),
    [sym_timeline_relative] = ACTIONS(2105),
    [sym_do_push_timed_infinite] = ACTIONS(2105),
    [sym_source_type_no_address] = ACTIONS(2105),
    [sym_source_type_neuron_id] = ACTIONS(2105),
    [sym_source_type_ip_address] = ACTIONS(2105),
    [sym_source_type_axlink] = ACTIONS(2105),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2105),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2105),
    [sym_source_type_ip_socket_address] = ACTIONS(2105),
    [sym_source_type_rs232] = ACTIONS(2105),
    [sym_source_type_internal] = ACTIONS(2105),
    [sym_source_type_mac_address] = ACTIONS(2105),
    [sym_source_type_ipv4_port] = ACTIONS(2105),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2105),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2105),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2105),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2105),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2105),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2105),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2105),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2105),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2105),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2105),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2105),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2105),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2105),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2105),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2105),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2105),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2105),
    [sym_true] = ACTIONS(2105),
    [sym_false] = ACTIONS(2105),
    [sym_file_read_only] = ACTIONS(2105),
    [sym_file_rw_new] = ACTIONS(2105),
    [sym_file_rw_append] = ACTIONS(2105),
    [sym_ip_tcp] = ACTIONS(2105),
    [sym_ip_udp] = ACTIONS(2105),
    [sym_ip_udp_2way] = ACTIONS(2105),
    [sym_xml_encode_types] = ACTIONS(2105),
    [sym_xml_encode_char_as_list] = ACTIONS(2105),
    [sym_xml_encode_le] = ACTIONS(2105),
    [sym_xml_decode_types] = ACTIONS(2105),
    [sym_xml_decode_no_preserve] = ACTIONS(2105),
    [sym_duet_dev_type_display_device] = ACTIONS(2105),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2105),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2105),
    [sym_duet_dev_type_io_device] = ACTIONS(2105),
    [sym_duet_dev_type_relay_device] = ACTIONS(2105),
    [sym_duet_dev_type_ups] = ACTIONS(2105),
    [sym_duet_dev_type_amplifier] = ACTIONS(2105),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2105),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2105),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2105),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2105),
    [sym_duet_dev_type_camera] = ACTIONS(2105),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2105),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2105),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2105),
    [sym_duet_dev_type_dss] = ACTIONS(2105),
    [sym_duet_dev_type_dvr] = ACTIONS(2105),
    [sym_duet_dev_type_disc_device] = ACTIONS(2105),
    [sym_duet_dev_type_document_camera] = ACTIONS(2105),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2105),
    [sym_duet_dev_type_hvac] = ACTIONS(2105),
    [sym_duet_dev_type_keypad] = ACTIONS(2105),
    [sym_duet_dev_type_light] = ACTIONS(2105),
    [sym_duet_dev_type_monitor] = ACTIONS(2105),
    [sym_duet_dev_type_motor] = ACTIONS(2105),
    [sym_duet_dev_type_multi_window] = ACTIONS(2105),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2105),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2105),
    [sym_duet_dev_type_receiver] = ACTIONS(2105),
    [sym_duet_dev_type_security_system] = ACTIONS(2105),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2105),
    [sym_duet_dev_type_settop_box] = ACTIONS(2105),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2105),
    [sym_duet_dev_type_switcher] = ACTIONS(2105),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2105),
    [sym_duet_dev_type_tv] = ACTIONS(2105),
    [sym_duet_dev_type_utility] = ACTIONS(2105),
    [sym_duet_dev_type_vcr] = ACTIONS(2105),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2105),
    [sym_duet_dev_type_video_processor] = ACTIONS(2105),
    [sym_duet_dev_type_video_projector] = ACTIONS(2105),
    [sym_duet_dev_type_video_wall] = ACTIONS(2105),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2105),
    [sym_duet_dev_type_weather] = ACTIONS(2105),
    [sym_duet_dev_not_polled] = ACTIONS(2105),
    [sym_duet_dev_polled] = ACTIONS(2105),
    [sym_clkmgr_mode_network] = ACTIONS(2105),
    [sym_clkmgr_mode_standalone] = ACTIONS(2105),
    [sym_normal_standby] = ACTIONS(2105),
    [sym_normal_wake] = ACTIONS(2105),
    [sym_ip_multicast_ttl_option] = ACTIONS(2105),
    [sym_ip_ttl_subnet] = ACTIONS(2105),
    [sym_ip_ttl_site] = ACTIONS(2105),
    [sym_ip_ttl_region] = ACTIONS(2105),
    [sym_ip_ttl_continent] = ACTIONS(2105),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2105),
    [sym_ip_nodelay_on] = ACTIONS(2105),
    [sym_ip_nodelay_off] = ACTIONS(2105),
    [sym_amx_error] = ACTIONS(2105),
    [sym_amx_warning] = ACTIONS(2105),
    [sym_amx_info] = ACTIONS(2105),
    [sym_amx_debug] = ACTIONS(2105),
    [sym_netlinx_login_success] = ACTIONS(2105),
    [sym_netlinx_login_fail] = ACTIONS(2105),
    [sym_netlinx_logout] = ACTIONS(2105),
    [sym_null_str] = ACTIONS(2105),
    [sym_smtp_address] = ACTIONS(2105),
    [sym_smtp_port_number] = ACTIONS(2105),
    [sym_smtp_username] = ACTIONS(2105),
    [sym_smtp_password] = ACTIONS(2105),
    [sym_smtp_require_tls] = ACTIONS(2105),
    [sym_smtp_from] = ACTIONS(2105),
    [sym_smtp_tls_true] = ACTIONS(2105),
    [sym_smtp_tls_false] = ACTIONS(2105),
    [sym_listview_on_row_select_event] = ACTIONS(2105),
    [sym_data_structure_unknown] = ACTIONS(2105),
    [sym_data_structure_datafeed] = ACTIONS(2105),
    [sym_data_structure_datarecordset] = ACTIONS(2105),
    [sym_data_structure_datarecord] = ACTIONS(2105),
    [sym_data_structure_datafield] = ACTIONS(2105),
    [sym_data_max_content_field_count] = ACTIONS(2105),
    [sym_data_max_metadata_field_count] = ACTIONS(2105),
    [sym_data_max_id_length] = ACTIONS(2105),
    [sym_data_max_name_length] = ACTIONS(2105),
    [sym_data_max_description_length] = ACTIONS(2105),
    [sym_data_max_type_length] = ACTIONS(2105),
    [sym_data_max_format_length] = ACTIONS(2105),
    [sym_data_max_source_length] = ACTIONS(2105),
    [sym_data_max_value_length] = ACTIONS(2105),
    [sym_data_max_label_length] = ACTIONS(2105),
    [sym_data_max_url_length] = ACTIONS(2105),
    [sym_max_last_login_info_length] = ACTIONS(2105),
    [sym_data_type_unknown] = ACTIONS(2105),
    [sym_data_type_string] = ACTIONS(2105),
    [sym_data_type_datetime] = ACTIONS(2105),
    [sym_data_type_date] = ACTIONS(2105),
    [sym_data_type_time] = ACTIONS(2105),
    [sym_data_type_image] = ACTIONS(2105),
    [sym_data_format_url] = ACTIONS(2105),
    [sym_data_format_phone] = ACTIONS(2105),
    [sym_data_format_email] = ACTIONS(2105),
    [sym_data_format_iso8601] = ACTIONS(2105),
    [sym_auditlog_priv] = ACTIONS(2105),
    [sym_configuration_priv] = ACTIONS(2105),
    [sym_deviceconfig_priv] = ACTIONS(2105),
    [sym_ftp_priv] = ACTIONS(2105),
    [sym_http_priv] = ACTIONS(2105),
    [sym_networkconfig_priv] = ACTIONS(2105),
    [sym_programport_priv] = ACTIONS(2105),
    [sym_remoteui_priv] = ACTIONS(2105),
    [sym_securitycontrol_priv] = ACTIONS(2105),
    [sym_softwaremanagement_priv] = ACTIONS(2105),
    [sym_terminal_priv] = ACTIONS(2105),
    [sym_tpadmin_priv] = ACTIONS(2105),
    [sym_user1_priv] = ACTIONS(2105),
    [sym_user2_priv] = ACTIONS(2105),
    [sym_user3_priv] = ACTIONS(2105),
    [sym_user4_priv] = ACTIONS(2105),
    [sym_usermanagement_priv] = ACTIONS(2105),
    [sym_valid_account] = ACTIONS(2105),
    [sym_err_name_invalid] = ACTIONS(2105),
    [sym_err_password_invalid] = ACTIONS(2105),
    [sym_err_invalid_account] = ACTIONS(2105),
    [sym_err_no_authorization] = ACTIONS(2105),
    [sym_err_invalid_parameter] = ACTIONS(2105),
    [sym_err_user_locked_out] = ACTIONS(2105),
    [sym_err_user_account_expired] = ACTIONS(2105),
    [sym_err_general_error] = ACTIONS(2105),
    [sym_err_authentication_error] = ACTIONS(2105),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2105),
    [sym_tls_validate_certificate] = ACTIONS(2105),
    [sym_tdata] = ACTIONS(2105),
    [sym_tchannel] = ACTIONS(2105),
    [sym_tlevel] = ACTIONS(2105),
    [sym_tbutton] = ACTIONS(2105),
    [sym_ttimeline] = ACTIONS(2105),
    [sym_tcustom] = ACTIONS(2105),
    [sym_url_struct] = ACTIONS(2105),
    [sym_dns_struct] = ACTIONS(2105),
    [sym_ip_address_struct] = ACTIONS(2105),
    [sym_dev_info_struct] = ACTIONS(2105),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2105),
    [sym_data_feed] = ACTIONS(2105),
    [sym_data_field] = ACTIONS(2105),
    [sym_data_record] = ACTIONS(2105),
    [sym_wc_data_feed] = ACTIONS(2105),
    [sym_wc_data_field] = ACTIONS(2105),
    [sym_wc_data_record] = ACTIONS(2105),
    [sym_last_login_info] = ACTIONS(2105),
    [sym____reserved___] = ACTIONS(2105),
    [sym_date] = ACTIONS(2105),
    [sym_day] = ACTIONS(2105),
    [sym_time] = ACTIONS(2105),
    [sym_ldate] = ACTIONS(2105),
    [sym_push_device] = ACTIONS(2105),
    [sym_push_channel] = ACTIONS(2105),
    [sym_push_devchan] = ACTIONS(2105),
    [sym_release_device] = ACTIONS(2105),
    [sym_release_channel] = ACTIONS(2105),
    [sym_release_devchan] = ACTIONS(2105),
    [sym_master_slot] = ACTIONS(2105),
    [sym_get_pulse_time] = ACTIONS(2105),
    [sym_get_timer] = ACTIONS(2105),
    [sym_master_sn] = ACTIONS(2105),
    [sym_system_number] = ACTIONS(2105),
    [sym_button] = ACTIONS(2105),
    [sym_dv_channel] = ACTIONS(2105),
    [sym_data] = ACTIONS(2105),
    [sym_level] = ACTIONS(2105),
    [sym_timeline] = ACTIONS(2105),
    [sym_channel] = ACTIONS(2105),
    [sym_custom] = ACTIONS(2105),
    [sym_atoi] = ACTIONS(2105),
    [sym_atol] = ACTIONS(2105),
    [sym_atof] = ACTIONS(2105),
    [sym_itoa] = ACTIONS(2105),
    [sym_itohex] = ACTIONS(2105),
    [sym_hextoi] = ACTIONS(2105),
    [sym_ftoa] = ACTIONS(2105),
    [sym_find_string] = ACTIONS(2105),
    [sym_left_string] = ACTIONS(2105),
    [sym_right_string] = ACTIONS(2105),
    [sym_mid_string] = ACTIONS(2105),
    [sym_remove_string] = ACTIONS(2105),
    [sym_get_buffer_string] = ACTIONS(2105),
    [sym_get_buffer_char] = ACTIONS(2105),
    [sym_get_multi_buffer_string] = ACTIONS(2105),
    [sym_length_string] = ACTIONS(2105),
    [sym_max_length_string] = ACTIONS(2105),
    [sym_set_length_string] = ACTIONS(2105),
    [sym_compare_string] = ACTIONS(2105),
    [sym_upper_string] = ACTIONS(2105),
    [sym_lower_string] = ACTIONS(2105),
    [sym_get_last] = ACTIONS(2105),
    [sym_date_to_day] = ACTIONS(2105),
    [sym_date_to_month] = ACTIONS(2105),
    [sym_date_to_year] = ACTIONS(2105),
    [sym_time_to_hour] = ACTIONS(2105),
    [sym_time_to_minute] = ACTIONS(2105),
    [sym_time_to_second] = ACTIONS(2105),
    [sym_day_of_week] = ACTIONS(2105),
    [sym_set_pulse_time] = ACTIONS(2105),
    [sym_set_timer] = ACTIONS(2105),
    [sym_file_open] = ACTIONS(2105),
    [sym_file_close] = ACTIONS(2105),
    [sym_file_read] = ACTIONS(2105),
    [sym_file_read_line] = ACTIONS(2105),
    [sym_file_write] = ACTIONS(2105),
    [sym_file_write_line] = ACTIONS(2105),
    [sym_file_copy] = ACTIONS(2105),
    [sym_file_delete] = ACTIONS(2105),
    [sym_file_rename] = ACTIONS(2105),
    [sym_file_dir] = ACTIONS(2105),
    [sym_file_getdir] = ACTIONS(2105),
    [sym_file_setdir] = ACTIONS(2105),
    [sym_file_create_dir] = ACTIONS(2105),
    [sym_file_remove_dir] = ACTIONS(2105),
    [sym_file_seek] = ACTIONS(2105),
    [sym_length_array] = ACTIONS(2105),
    [sym_max_length_array] = ACTIONS(2105),
    [sym_set_length_array] = ACTIONS(2105),
    [sym_ip_client_open] = ACTIONS(2105),
    [sym_ip_client_close] = ACTIONS(2105),
    [sym_ip_server_open] = ACTIONS(2105),
    [sym_ip_server_close] = ACTIONS(2105),
    [sym_ip_bound_client_open] = ACTIONS(2105),
    [sym_ip_mc_server_open] = ACTIONS(2105),
    [sym_ip_set_option] = ACTIONS(2105),
    [sym_ssh_client_open] = ACTIONS(2105),
    [sym_ssh_client_close] = ACTIONS(2105),
    [sym_tls_client_open] = ACTIONS(2105),
    [sym_tls_client_close] = ACTIONS(2105),
    [sym_do_push] = ACTIONS(2105),
    [sym_do_push_timed] = ACTIONS(2105),
    [sym_do_release] = ACTIONS(2105),
    [sym_device_id] = ACTIONS(2105),
    [sym_device_info] = ACTIONS(2105),
    [sym_device_id_string] = ACTIONS(2105),
    [sym_device_standby] = ACTIONS(2105),
    [sym_device_wake] = ACTIONS(2105),
    [sym_timeline_create] = ACTIONS(2105),
    [sym_timeline_kill] = ACTIONS(2105),
    [sym_timeline_reload] = ACTIONS(2105),
    [sym_timeline_pause] = ACTIONS(2105),
    [sym_timeline_restart] = ACTIONS(2105),
    [sym_timeline_set] = ACTIONS(2105),
    [sym_timeline_get] = ACTIONS(2105),
    [sym_timeline_active] = ACTIONS(2105),
    [sym_timeline_dynamic_id] = ACTIONS(2105),
    [sym_variable_to_string] = ACTIONS(2105),
    [sym_string_to_variable] = ACTIONS(2105),
    [sym_length_variable_to_string] = ACTIONS(2105),
    [sym_type_cast] = ACTIONS(2105),
    [sym_variable_to_xml] = ACTIONS(2105),
    [sym_xml_to_variable] = ACTIONS(2105),
    [sym_length_variable_to_xml] = ACTIONS(2105),
    [sym_redirect_string] = ACTIONS(2105),
    [sym_random_number] = ACTIONS(2105),
    [sym_get_unique_id] = ACTIONS(2105),
    [sym_get_system_number] = ACTIONS(2105),
    [sym_set_system_number] = ACTIONS(2105),
    [sym_get_serial_number] = ACTIONS(2105),
    [sym_get_master_build] = ACTIONS(2105),
    [sym_set_validation_code] = ACTIONS(2105),
    [sym_reboot] = ACTIONS(2105),
    [sym_load_duet_module] = ACTIONS(2105),
    [sym_unload_duet_module] = ACTIONS(2105),
    [sym_duet_mem_size_set] = ACTIONS(2105),
    [sym_duet_mem_size_get] = ACTIONS(2105),
    [sym_dynamic_application_device] = ACTIONS(2105),
    [sym_dynamic_polled_port] = ACTIONS(2105),
    [sym_static_port_binding] = ACTIONS(2105),
    [sym_static_ip_binding] = ACTIONS(2105),
    [sym_get_dns_list] = ACTIONS(2105),
    [sym_set_dns_list] = ACTIONS(2105),
    [sym_get_ip_address] = ACTIONS(2105),
    [sym_set_ip_address] = ACTIONS(2105),
    [sym_get_url_list] = ACTIONS(2105),
    [sym_add_url_entry] = ACTIONS(2105),
    [sym_delete_url_entry] = ACTIONS(2105),
    [sym_abs_value] = ACTIONS(2105),
    [sym_max_value] = ACTIONS(2105),
    [sym_min_value] = ACTIONS(2105),
    [sym_exp_value] = ACTIONS(2105),
    [sym_log_value] = ACTIONS(2105),
    [sym_log10_value] = ACTIONS(2105),
    [sym_power_value] = ACTIONS(2105),
    [sym_sqrt_value] = ACTIONS(2105),
    [sym_raw_be] = ACTIONS(2105),
    [sym_raw_le] = ACTIONS(2105),
    [sym_internal_queue_size_set] = ACTIONS(2105),
    [sym_internal_queue_size_get] = ACTIONS(2105),
    [sym_internal_threshold_set] = ACTIONS(2105),
    [sym_internal_threshold_get] = ACTIONS(2105),
    [sym_rebuilt_event] = ACTIONS(2105),
    [sym_do_custom_event] = ACTIONS(2105),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2105),
    [sym_clkmgr_set_clk_source] = ACTIONS(2105),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2105),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2105),
    [sym_clkmgr_get_timezone] = ACTIONS(2105),
    [sym_clkmgr_set_timezone] = ACTIONS(2105),
    [sym_clkmgr_get_resync_period] = ACTIONS(2105),
    [sym_clkmgr_set_resync_period] = ACTIONS(2105),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2105),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2105),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2105),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2105),
    [sym_clkmgr_get_timeservers] = ACTIONS(2105),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2105),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2105),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2105),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2105),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2105),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2105),
    [sym_set_log_level] = ACTIONS(2105),
    [sym_get_log_level] = ACTIONS(2105),
    [sym_amx_log] = ACTIONS(2105),
    [sym_smtp_server_config_set] = ACTIONS(2105),
    [sym_smtp_server_config_get] = ACTIONS(2105),
    [sym_smtp_send] = ACTIONS(2105),
    [sym_validate_netlinx_account] = ACTIONS(2105),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2105),
    [sym_authenticate_certificate] = ACTIONS(2105),
    [sym_audit_netlinx_session_event] = ACTIONS(2105),
    [sym_audit_netlinx_generic_event] = ACTIONS(2105),
    [sym_get_available_flash_disk_space] = ACTIONS(2105),
    [sym_get_max_flash_disk_space] = ACTIONS(2105),
    [sym_data_create_feed] = ACTIONS(2105),
    [sym_data_delete_feed] = ACTIONS(2105),
    [sym_data_publish_feed] = ACTIONS(2105),
    [sym_data_get_published_feed] = ACTIONS(2105),
    [sym_data_add_record] = ACTIONS(2105),
    [sym_data_get_event_record] = ACTIONS(2105),
    [sym__wc_data_create_feed] = ACTIONS(2105),
    [sym__wc_data_add_record] = ACTIONS(2105),
    [sym__wc_data_get_event_record] = ACTIONS(2105),
    [sym_format] = ACTIONS(2105),
    [sym_astro_clock] = ACTIONS(2105),
    [sym_set_outdoor_temperature] = ACTIONS(2105),
    [sym_set_virtual_level_count] = ACTIONS(2105),
    [sym_set_virtual_channel_count] = ACTIONS(2105),
    [sym_set_virtual_port_count] = ACTIONS(2105),
    [sym__wc_encode_int] = ACTIONS(2105),
    [sym__wc_decode_int] = ACTIONS(2105),
    [sym__wc_int] = ACTIONS(2105),
    [sym__wc_to_ch_int] = ACTIONS(2105),
    [sym__ch_to_wc_int] = ACTIONS(2105),
    [sym__wc_find_string_int] = ACTIONS(2105),
    [sym__wc_left_string_int] = ACTIONS(2105),
    [sym__wc_lower_string_int] = ACTIONS(2105),
    [sym__wc_mid_string_int] = ACTIONS(2105),
    [sym__wc_remove_string_int] = ACTIONS(2105),
    [sym__wc_right_string_int] = ACTIONS(2105),
    [sym__wc_upper_string_int] = ACTIONS(2105),
    [sym__wc_compare_string_int] = ACTIONS(2105),
    [sym__wc_get_buffer_char_int] = ACTIONS(2105),
    [sym__wc_get_buffer_string_int] = ACTIONS(2105),
    [sym__wc_concat_string_int] = ACTIONS(2105),
    [sym__wc_file_open_int] = ACTIONS(2105),
    [sym__wc_file_close_int] = ACTIONS(2105),
    [sym__wc_file_read_int] = ACTIONS(2105),
    [sym__wc_file_read_line_int] = ACTIONS(2105),
    [sym__wc_file_write_int] = ACTIONS(2105),
    [sym__wc_file_write_line_int] = ACTIONS(2105),
    [aux_sym_preproc_include_token1] = ACTIONS(2105),
    [aux_sym_preproc_define_token1] = ACTIONS(2107),
    [aux_sym_preproc_warn_token1] = ACTIONS(2107),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2107),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2107),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2107),
    [anon_sym_LBRACK] = ACTIONS(2107),
    [anon_sym_LBRACE] = ACTIONS(2107),
    [anon_sym_LPAREN] = ACTIONS(2105),
    [anon_sym_BANG] = ACTIONS(2107),
    [anon_sym_TILDE] = ACTIONS(2107),
    [anon_sym_DASH] = ACTIONS(2105),
    [anon_sym_PLUS] = ACTIONS(2105),
    [anon_sym_DASH_DASH] = ACTIONS(2107),
    [anon_sym_PLUS_PLUS] = ACTIONS(2107),
    [anon_sym_DQUOTE] = ACTIONS(2107),
    [anon_sym_SQUOTE] = ACTIONS(2107),
    [sym_number_literal] = ACTIONS(2107),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(414)] = {
    [ts_builtin_sym_end] = ACTIONS(2111),
    [sym_identifier] = ACTIONS(2109),
    [sym_program_name_keyword] = ACTIONS(2109),
    [sym_module_name_keyword] = ACTIONS(2109),
    [sym_define_device_keyword] = ACTIONS(2109),
    [sym_define_combine_keyword] = ACTIONS(2109),
    [sym_define_constant_keyword] = ACTIONS(2109),
    [sym_define_type_keyword] = ACTIONS(2109),
    [sym_define_variable_keyword] = ACTIONS(2109),
    [sym_define_system_variable_keyword] = ACTIONS(2109),
    [sym_define_connect_level_keyword] = ACTIONS(2109),
    [sym_define_latching_keyword] = ACTIONS(2109),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2109),
    [sym_define_toggling_keyword] = ACTIONS(2109),
    [sym_define_call_keyword] = ACTIONS(2109),
    [sym_define_function_keyword] = ACTIONS(2109),
    [sym_define_library_function_keyword] = ACTIONS(2109),
    [sym_define_start_keyword] = ACTIONS(2109),
    [sym_define_event_keyword] = ACTIONS(2109),
    [sym_define_program_keyword] = ACTIONS(2109),
    [sym_define_module_keyword] = ACTIONS(2109),
    [sym_char_keyword] = ACTIONS(2109),
    [sym_widechar_keyword] = ACTIONS(2109),
    [sym_integer_keyword] = ACTIONS(2109),
    [sym_sinteger_keyword] = ACTIONS(2109),
    [sym_long_keyword] = ACTIONS(2109),
    [sym_slong_keyword] = ACTIONS(2109),
    [sym_float_keyword] = ACTIONS(2109),
    [sym_double_keyword] = ACTIONS(2109),
    [sym_variant_keyword] = ACTIONS(2109),
    [sym_variantarray_keyword] = ACTIONS(2109),
    [sym_local_var_keyword] = ACTIONS(2109),
    [sym_stack_var_keyword] = ACTIONS(2109),
    [sym_constant_keyword] = ACTIONS(2109),
    [sym_volatile_keyword] = ACTIONS(2109),
    [sym_non_volatile_keyword] = ACTIONS(2109),
    [sym_persistent_keyword] = ACTIONS(2109),
    [sym_dev_keyword] = ACTIONS(2109),
    [sym_devlev_keyword] = ACTIONS(2109),
    [sym_devchan_keyword] = ACTIONS(2109),
    [sym_if_keyword] = ACTIONS(2109),
    [sym_switch_keyword] = ACTIONS(2109),
    [sym_case_keyword] = ACTIONS(2109),
    [sym_default_keyword] = ACTIONS(2109),
    [sym_while_keyword] = ACTIONS(2109),
    [sym_for_keyword] = ACTIONS(2109),
    [sym_break_keyword] = ACTIONS(2109),
    [sym_continue_keyword] = ACTIONS(2109),
    [sym_return_keyword] = ACTIONS(2109),
    [sym_select_keyword] = ACTIONS(2109),
    [sym_active_keyword] = ACTIONS(2109),
    [sym_struct_keyword] = ACTIONS(2109),
    [sym_structure_keyword] = ACTIONS(2109),
    [sym_button_event_keyword] = ACTIONS(2109),
    [sym_channel_event_keyword] = ACTIONS(2109),
    [sym_level_event_keyword] = ACTIONS(2109),
    [sym_data_event_keyword] = ACTIONS(2109),
    [sym_timeline_event_keyword] = ACTIONS(2109),
    [sym_custom_event_keyword] = ACTIONS(2109),
    [sym_push_keyword] = ACTIONS(2109),
    [sym_release_keyword] = ACTIONS(2109),
    [aux_sym_on_keyword_token1] = ACTIONS(2109),
    [aux_sym_off_keyword_token1] = ACTIONS(2109),
    [sym_send_level_keyword] = ACTIONS(2109),
    [sym_send_string_keyword] = ACTIONS(2109),
    [sym_send_command_keyword] = ACTIONS(2109),
    [sym_clear_buffer_keyword] = ACTIONS(2109),
    [sym_create_buffer_keyword] = ACTIONS(2109),
    [sym_create_multi_buffer_keyword] = ACTIONS(2109),
    [sym_call_keyword] = ACTIONS(2109),
    [sym_system_call_keyword] = ACTIONS(2109),
    [sym_devchan_to_keyword] = ACTIONS(2109),
    [sym_devchan_min_to_keyword] = ACTIONS(2109),
    [sym_devchan_total_off_keyword] = ACTIONS(2109),
    [sym_devchan_pulse_keyword] = ACTIONS(2109),
    [sym_wait_keyword] = ACTIONS(2109),
    [sym_cancel_wait_keyword] = ACTIONS(2109),
    [sym_wait_until_keyword] = ACTIONS(2109),
    [sym_cancel_wait_until_keyword] = ACTIONS(2109),
    [sym_cancel_all_wait_keyword] = ACTIONS(2109),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2109),
    [sym_not] = ACTIONS(2109),
    [sym_bnot] = ACTIONS(2109),
    [sym___file__] = ACTIONS(2109),
    [sym___date__] = ACTIONS(2109),
    [sym___ldate__] = ACTIONS(2109),
    [sym___time__] = ACTIONS(2109),
    [sym___line__] = ACTIONS(2109),
    [sym___name__] = ACTIONS(2109),
    [sym___netlinx__] = ACTIONS(2109),
    [sym_netlinx_axi_version] = ACTIONS(2109),
    [sym_url_flg_tcp] = ACTIONS(2109),
    [sym_url_flg_accinfopresent] = ACTIONS(2109),
    [sym_url_flg_temp] = ACTIONS(2109),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2109),
    [sym_url_flg_stat_mask] = ACTIONS(2109),
    [sym_url_flg_stat_lookup] = ACTIONS(2109),
    [sym_url_flg_stat_connecting] = ACTIONS(2109),
    [sym_url_flg_stat_waiting] = ACTIONS(2109),
    [sym_url_flg_stat_connected] = ACTIONS(2109),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2109),
    [sym_first_virtual_device] = ACTIONS(2109),
    [sym_first_local_port] = ACTIONS(2109),
    [sym_dynamic_virtual_device] = ACTIONS(2109),
    [sym_timeline_once] = ACTIONS(2109),
    [sym_timeline_repeat] = ACTIONS(2109),
    [sym_timeline_absolute] = ACTIONS(2109),
    [sym_timeline_relative] = ACTIONS(2109),
    [sym_do_push_timed_infinite] = ACTIONS(2109),
    [sym_source_type_no_address] = ACTIONS(2109),
    [sym_source_type_neuron_id] = ACTIONS(2109),
    [sym_source_type_ip_address] = ACTIONS(2109),
    [sym_source_type_axlink] = ACTIONS(2109),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2109),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2109),
    [sym_source_type_ip_socket_address] = ACTIONS(2109),
    [sym_source_type_rs232] = ACTIONS(2109),
    [sym_source_type_internal] = ACTIONS(2109),
    [sym_source_type_mac_address] = ACTIONS(2109),
    [sym_source_type_ipv4_port] = ACTIONS(2109),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2109),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2109),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2109),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2109),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2109),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2109),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2109),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2109),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2109),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2109),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2109),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2109),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2109),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2109),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2109),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2109),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2109),
    [sym_true] = ACTIONS(2109),
    [sym_false] = ACTIONS(2109),
    [sym_file_read_only] = ACTIONS(2109),
    [sym_file_rw_new] = ACTIONS(2109),
    [sym_file_rw_append] = ACTIONS(2109),
    [sym_ip_tcp] = ACTIONS(2109),
    [sym_ip_udp] = ACTIONS(2109),
    [sym_ip_udp_2way] = ACTIONS(2109),
    [sym_xml_encode_types] = ACTIONS(2109),
    [sym_xml_encode_char_as_list] = ACTIONS(2109),
    [sym_xml_encode_le] = ACTIONS(2109),
    [sym_xml_decode_types] = ACTIONS(2109),
    [sym_xml_decode_no_preserve] = ACTIONS(2109),
    [sym_duet_dev_type_display_device] = ACTIONS(2109),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2109),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2109),
    [sym_duet_dev_type_io_device] = ACTIONS(2109),
    [sym_duet_dev_type_relay_device] = ACTIONS(2109),
    [sym_duet_dev_type_ups] = ACTIONS(2109),
    [sym_duet_dev_type_amplifier] = ACTIONS(2109),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2109),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2109),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2109),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2109),
    [sym_duet_dev_type_camera] = ACTIONS(2109),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2109),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2109),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2109),
    [sym_duet_dev_type_dss] = ACTIONS(2109),
    [sym_duet_dev_type_dvr] = ACTIONS(2109),
    [sym_duet_dev_type_disc_device] = ACTIONS(2109),
    [sym_duet_dev_type_document_camera] = ACTIONS(2109),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2109),
    [sym_duet_dev_type_hvac] = ACTIONS(2109),
    [sym_duet_dev_type_keypad] = ACTIONS(2109),
    [sym_duet_dev_type_light] = ACTIONS(2109),
    [sym_duet_dev_type_monitor] = ACTIONS(2109),
    [sym_duet_dev_type_motor] = ACTIONS(2109),
    [sym_duet_dev_type_multi_window] = ACTIONS(2109),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2109),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2109),
    [sym_duet_dev_type_receiver] = ACTIONS(2109),
    [sym_duet_dev_type_security_system] = ACTIONS(2109),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2109),
    [sym_duet_dev_type_settop_box] = ACTIONS(2109),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2109),
    [sym_duet_dev_type_switcher] = ACTIONS(2109),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2109),
    [sym_duet_dev_type_tv] = ACTIONS(2109),
    [sym_duet_dev_type_utility] = ACTIONS(2109),
    [sym_duet_dev_type_vcr] = ACTIONS(2109),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2109),
    [sym_duet_dev_type_video_processor] = ACTIONS(2109),
    [sym_duet_dev_type_video_projector] = ACTIONS(2109),
    [sym_duet_dev_type_video_wall] = ACTIONS(2109),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2109),
    [sym_duet_dev_type_weather] = ACTIONS(2109),
    [sym_duet_dev_not_polled] = ACTIONS(2109),
    [sym_duet_dev_polled] = ACTIONS(2109),
    [sym_clkmgr_mode_network] = ACTIONS(2109),
    [sym_clkmgr_mode_standalone] = ACTIONS(2109),
    [sym_normal_standby] = ACTIONS(2109),
    [sym_normal_wake] = ACTIONS(2109),
    [sym_ip_multicast_ttl_option] = ACTIONS(2109),
    [sym_ip_ttl_subnet] = ACTIONS(2109),
    [sym_ip_ttl_site] = ACTIONS(2109),
    [sym_ip_ttl_region] = ACTIONS(2109),
    [sym_ip_ttl_continent] = ACTIONS(2109),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2109),
    [sym_ip_nodelay_on] = ACTIONS(2109),
    [sym_ip_nodelay_off] = ACTIONS(2109),
    [sym_amx_error] = ACTIONS(2109),
    [sym_amx_warning] = ACTIONS(2109),
    [sym_amx_info] = ACTIONS(2109),
    [sym_amx_debug] = ACTIONS(2109),
    [sym_netlinx_login_success] = ACTIONS(2109),
    [sym_netlinx_login_fail] = ACTIONS(2109),
    [sym_netlinx_logout] = ACTIONS(2109),
    [sym_null_str] = ACTIONS(2109),
    [sym_smtp_address] = ACTIONS(2109),
    [sym_smtp_port_number] = ACTIONS(2109),
    [sym_smtp_username] = ACTIONS(2109),
    [sym_smtp_password] = ACTIONS(2109),
    [sym_smtp_require_tls] = ACTIONS(2109),
    [sym_smtp_from] = ACTIONS(2109),
    [sym_smtp_tls_true] = ACTIONS(2109),
    [sym_smtp_tls_false] = ACTIONS(2109),
    [sym_listview_on_row_select_event] = ACTIONS(2109),
    [sym_data_structure_unknown] = ACTIONS(2109),
    [sym_data_structure_datafeed] = ACTIONS(2109),
    [sym_data_structure_datarecordset] = ACTIONS(2109),
    [sym_data_structure_datarecord] = ACTIONS(2109),
    [sym_data_structure_datafield] = ACTIONS(2109),
    [sym_data_max_content_field_count] = ACTIONS(2109),
    [sym_data_max_metadata_field_count] = ACTIONS(2109),
    [sym_data_max_id_length] = ACTIONS(2109),
    [sym_data_max_name_length] = ACTIONS(2109),
    [sym_data_max_description_length] = ACTIONS(2109),
    [sym_data_max_type_length] = ACTIONS(2109),
    [sym_data_max_format_length] = ACTIONS(2109),
    [sym_data_max_source_length] = ACTIONS(2109),
    [sym_data_max_value_length] = ACTIONS(2109),
    [sym_data_max_label_length] = ACTIONS(2109),
    [sym_data_max_url_length] = ACTIONS(2109),
    [sym_max_last_login_info_length] = ACTIONS(2109),
    [sym_data_type_unknown] = ACTIONS(2109),
    [sym_data_type_string] = ACTIONS(2109),
    [sym_data_type_datetime] = ACTIONS(2109),
    [sym_data_type_date] = ACTIONS(2109),
    [sym_data_type_time] = ACTIONS(2109),
    [sym_data_type_image] = ACTIONS(2109),
    [sym_data_format_url] = ACTIONS(2109),
    [sym_data_format_phone] = ACTIONS(2109),
    [sym_data_format_email] = ACTIONS(2109),
    [sym_data_format_iso8601] = ACTIONS(2109),
    [sym_auditlog_priv] = ACTIONS(2109),
    [sym_configuration_priv] = ACTIONS(2109),
    [sym_deviceconfig_priv] = ACTIONS(2109),
    [sym_ftp_priv] = ACTIONS(2109),
    [sym_http_priv] = ACTIONS(2109),
    [sym_networkconfig_priv] = ACTIONS(2109),
    [sym_programport_priv] = ACTIONS(2109),
    [sym_remoteui_priv] = ACTIONS(2109),
    [sym_securitycontrol_priv] = ACTIONS(2109),
    [sym_softwaremanagement_priv] = ACTIONS(2109),
    [sym_terminal_priv] = ACTIONS(2109),
    [sym_tpadmin_priv] = ACTIONS(2109),
    [sym_user1_priv] = ACTIONS(2109),
    [sym_user2_priv] = ACTIONS(2109),
    [sym_user3_priv] = ACTIONS(2109),
    [sym_user4_priv] = ACTIONS(2109),
    [sym_usermanagement_priv] = ACTIONS(2109),
    [sym_valid_account] = ACTIONS(2109),
    [sym_err_name_invalid] = ACTIONS(2109),
    [sym_err_password_invalid] = ACTIONS(2109),
    [sym_err_invalid_account] = ACTIONS(2109),
    [sym_err_no_authorization] = ACTIONS(2109),
    [sym_err_invalid_parameter] = ACTIONS(2109),
    [sym_err_user_locked_out] = ACTIONS(2109),
    [sym_err_user_account_expired] = ACTIONS(2109),
    [sym_err_general_error] = ACTIONS(2109),
    [sym_err_authentication_error] = ACTIONS(2109),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2109),
    [sym_tls_validate_certificate] = ACTIONS(2109),
    [sym_tdata] = ACTIONS(2109),
    [sym_tchannel] = ACTIONS(2109),
    [sym_tlevel] = ACTIONS(2109),
    [sym_tbutton] = ACTIONS(2109),
    [sym_ttimeline] = ACTIONS(2109),
    [sym_tcustom] = ACTIONS(2109),
    [sym_url_struct] = ACTIONS(2109),
    [sym_dns_struct] = ACTIONS(2109),
    [sym_ip_address_struct] = ACTIONS(2109),
    [sym_dev_info_struct] = ACTIONS(2109),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2109),
    [sym_data_feed] = ACTIONS(2109),
    [sym_data_field] = ACTIONS(2109),
    [sym_data_record] = ACTIONS(2109),
    [sym_wc_data_feed] = ACTIONS(2109),
    [sym_wc_data_field] = ACTIONS(2109),
    [sym_wc_data_record] = ACTIONS(2109),
    [sym_last_login_info] = ACTIONS(2109),
    [sym____reserved___] = ACTIONS(2109),
    [sym_date] = ACTIONS(2109),
    [sym_day] = ACTIONS(2109),
    [sym_time] = ACTIONS(2109),
    [sym_ldate] = ACTIONS(2109),
    [sym_push_device] = ACTIONS(2109),
    [sym_push_channel] = ACTIONS(2109),
    [sym_push_devchan] = ACTIONS(2109),
    [sym_release_device] = ACTIONS(2109),
    [sym_release_channel] = ACTIONS(2109),
    [sym_release_devchan] = ACTIONS(2109),
    [sym_master_slot] = ACTIONS(2109),
    [sym_get_pulse_time] = ACTIONS(2109),
    [sym_get_timer] = ACTIONS(2109),
    [sym_master_sn] = ACTIONS(2109),
    [sym_system_number] = ACTIONS(2109),
    [sym_button] = ACTIONS(2109),
    [sym_dv_channel] = ACTIONS(2109),
    [sym_data] = ACTIONS(2109),
    [sym_level] = ACTIONS(2109),
    [sym_timeline] = ACTIONS(2109),
    [sym_channel] = ACTIONS(2109),
    [sym_custom] = ACTIONS(2109),
    [sym_atoi] = ACTIONS(2109),
    [sym_atol] = ACTIONS(2109),
    [sym_atof] = ACTIONS(2109),
    [sym_itoa] = ACTIONS(2109),
    [sym_itohex] = ACTIONS(2109),
    [sym_hextoi] = ACTIONS(2109),
    [sym_ftoa] = ACTIONS(2109),
    [sym_find_string] = ACTIONS(2109),
    [sym_left_string] = ACTIONS(2109),
    [sym_right_string] = ACTIONS(2109),
    [sym_mid_string] = ACTIONS(2109),
    [sym_remove_string] = ACTIONS(2109),
    [sym_get_buffer_string] = ACTIONS(2109),
    [sym_get_buffer_char] = ACTIONS(2109),
    [sym_get_multi_buffer_string] = ACTIONS(2109),
    [sym_length_string] = ACTIONS(2109),
    [sym_max_length_string] = ACTIONS(2109),
    [sym_set_length_string] = ACTIONS(2109),
    [sym_compare_string] = ACTIONS(2109),
    [sym_upper_string] = ACTIONS(2109),
    [sym_lower_string] = ACTIONS(2109),
    [sym_get_last] = ACTIONS(2109),
    [sym_date_to_day] = ACTIONS(2109),
    [sym_date_to_month] = ACTIONS(2109),
    [sym_date_to_year] = ACTIONS(2109),
    [sym_time_to_hour] = ACTIONS(2109),
    [sym_time_to_minute] = ACTIONS(2109),
    [sym_time_to_second] = ACTIONS(2109),
    [sym_day_of_week] = ACTIONS(2109),
    [sym_set_pulse_time] = ACTIONS(2109),
    [sym_set_timer] = ACTIONS(2109),
    [sym_file_open] = ACTIONS(2109),
    [sym_file_close] = ACTIONS(2109),
    [sym_file_read] = ACTIONS(2109),
    [sym_file_read_line] = ACTIONS(2109),
    [sym_file_write] = ACTIONS(2109),
    [sym_file_write_line] = ACTIONS(2109),
    [sym_file_copy] = ACTIONS(2109),
    [sym_file_delete] = ACTIONS(2109),
    [sym_file_rename] = ACTIONS(2109),
    [sym_file_dir] = ACTIONS(2109),
    [sym_file_getdir] = ACTIONS(2109),
    [sym_file_setdir] = ACTIONS(2109),
    [sym_file_create_dir] = ACTIONS(2109),
    [sym_file_remove_dir] = ACTIONS(2109),
    [sym_file_seek] = ACTIONS(2109),
    [sym_length_array] = ACTIONS(2109),
    [sym_max_length_array] = ACTIONS(2109),
    [sym_set_length_array] = ACTIONS(2109),
    [sym_ip_client_open] = ACTIONS(2109),
    [sym_ip_client_close] = ACTIONS(2109),
    [sym_ip_server_open] = ACTIONS(2109),
    [sym_ip_server_close] = ACTIONS(2109),
    [sym_ip_bound_client_open] = ACTIONS(2109),
    [sym_ip_mc_server_open] = ACTIONS(2109),
    [sym_ip_set_option] = ACTIONS(2109),
    [sym_ssh_client_open] = ACTIONS(2109),
    [sym_ssh_client_close] = ACTIONS(2109),
    [sym_tls_client_open] = ACTIONS(2109),
    [sym_tls_client_close] = ACTIONS(2109),
    [sym_do_push] = ACTIONS(2109),
    [sym_do_push_timed] = ACTIONS(2109),
    [sym_do_release] = ACTIONS(2109),
    [sym_device_id] = ACTIONS(2109),
    [sym_device_info] = ACTIONS(2109),
    [sym_device_id_string] = ACTIONS(2109),
    [sym_device_standby] = ACTIONS(2109),
    [sym_device_wake] = ACTIONS(2109),
    [sym_timeline_create] = ACTIONS(2109),
    [sym_timeline_kill] = ACTIONS(2109),
    [sym_timeline_reload] = ACTIONS(2109),
    [sym_timeline_pause] = ACTIONS(2109),
    [sym_timeline_restart] = ACTIONS(2109),
    [sym_timeline_set] = ACTIONS(2109),
    [sym_timeline_get] = ACTIONS(2109),
    [sym_timeline_active] = ACTIONS(2109),
    [sym_timeline_dynamic_id] = ACTIONS(2109),
    [sym_variable_to_string] = ACTIONS(2109),
    [sym_string_to_variable] = ACTIONS(2109),
    [sym_length_variable_to_string] = ACTIONS(2109),
    [sym_type_cast] = ACTIONS(2109),
    [sym_variable_to_xml] = ACTIONS(2109),
    [sym_xml_to_variable] = ACTIONS(2109),
    [sym_length_variable_to_xml] = ACTIONS(2109),
    [sym_redirect_string] = ACTIONS(2109),
    [sym_random_number] = ACTIONS(2109),
    [sym_get_unique_id] = ACTIONS(2109),
    [sym_get_system_number] = ACTIONS(2109),
    [sym_set_system_number] = ACTIONS(2109),
    [sym_get_serial_number] = ACTIONS(2109),
    [sym_get_master_build] = ACTIONS(2109),
    [sym_set_validation_code] = ACTIONS(2109),
    [sym_reboot] = ACTIONS(2109),
    [sym_load_duet_module] = ACTIONS(2109),
    [sym_unload_duet_module] = ACTIONS(2109),
    [sym_duet_mem_size_set] = ACTIONS(2109),
    [sym_duet_mem_size_get] = ACTIONS(2109),
    [sym_dynamic_application_device] = ACTIONS(2109),
    [sym_dynamic_polled_port] = ACTIONS(2109),
    [sym_static_port_binding] = ACTIONS(2109),
    [sym_static_ip_binding] = ACTIONS(2109),
    [sym_get_dns_list] = ACTIONS(2109),
    [sym_set_dns_list] = ACTIONS(2109),
    [sym_get_ip_address] = ACTIONS(2109),
    [sym_set_ip_address] = ACTIONS(2109),
    [sym_get_url_list] = ACTIONS(2109),
    [sym_add_url_entry] = ACTIONS(2109),
    [sym_delete_url_entry] = ACTIONS(2109),
    [sym_abs_value] = ACTIONS(2109),
    [sym_max_value] = ACTIONS(2109),
    [sym_min_value] = ACTIONS(2109),
    [sym_exp_value] = ACTIONS(2109),
    [sym_log_value] = ACTIONS(2109),
    [sym_log10_value] = ACTIONS(2109),
    [sym_power_value] = ACTIONS(2109),
    [sym_sqrt_value] = ACTIONS(2109),
    [sym_raw_be] = ACTIONS(2109),
    [sym_raw_le] = ACTIONS(2109),
    [sym_internal_queue_size_set] = ACTIONS(2109),
    [sym_internal_queue_size_get] = ACTIONS(2109),
    [sym_internal_threshold_set] = ACTIONS(2109),
    [sym_internal_threshold_get] = ACTIONS(2109),
    [sym_rebuilt_event] = ACTIONS(2109),
    [sym_do_custom_event] = ACTIONS(2109),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2109),
    [sym_clkmgr_set_clk_source] = ACTIONS(2109),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2109),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2109),
    [sym_clkmgr_get_timezone] = ACTIONS(2109),
    [sym_clkmgr_set_timezone] = ACTIONS(2109),
    [sym_clkmgr_get_resync_period] = ACTIONS(2109),
    [sym_clkmgr_set_resync_period] = ACTIONS(2109),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2109),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2109),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2109),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2109),
    [sym_clkmgr_get_timeservers] = ACTIONS(2109),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2109),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2109),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2109),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2109),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2109),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2109),
    [sym_set_log_level] = ACTIONS(2109),
    [sym_get_log_level] = ACTIONS(2109),
    [sym_amx_log] = ACTIONS(2109),
    [sym_smtp_server_config_set] = ACTIONS(2109),
    [sym_smtp_server_config_get] = ACTIONS(2109),
    [sym_smtp_send] = ACTIONS(2109),
    [sym_validate_netlinx_account] = ACTIONS(2109),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2109),
    [sym_authenticate_certificate] = ACTIONS(2109),
    [sym_audit_netlinx_session_event] = ACTIONS(2109),
    [sym_audit_netlinx_generic_event] = ACTIONS(2109),
    [sym_get_available_flash_disk_space] = ACTIONS(2109),
    [sym_get_max_flash_disk_space] = ACTIONS(2109),
    [sym_data_create_feed] = ACTIONS(2109),
    [sym_data_delete_feed] = ACTIONS(2109),
    [sym_data_publish_feed] = ACTIONS(2109),
    [sym_data_get_published_feed] = ACTIONS(2109),
    [sym_data_add_record] = ACTIONS(2109),
    [sym_data_get_event_record] = ACTIONS(2109),
    [sym__wc_data_create_feed] = ACTIONS(2109),
    [sym__wc_data_add_record] = ACTIONS(2109),
    [sym__wc_data_get_event_record] = ACTIONS(2109),
    [sym_format] = ACTIONS(2109),
    [sym_astro_clock] = ACTIONS(2109),
    [sym_set_outdoor_temperature] = ACTIONS(2109),
    [sym_set_virtual_level_count] = ACTIONS(2109),
    [sym_set_virtual_channel_count] = ACTIONS(2109),
    [sym_set_virtual_port_count] = ACTIONS(2109),
    [sym__wc_encode_int] = ACTIONS(2109),
    [sym__wc_decode_int] = ACTIONS(2109),
    [sym__wc_int] = ACTIONS(2109),
    [sym__wc_to_ch_int] = ACTIONS(2109),
    [sym__ch_to_wc_int] = ACTIONS(2109),
    [sym__wc_find_string_int] = ACTIONS(2109),
    [sym__wc_left_string_int] = ACTIONS(2109),
    [sym__wc_lower_string_int] = ACTIONS(2109),
    [sym__wc_mid_string_int] = ACTIONS(2109),
    [sym__wc_remove_string_int] = ACTIONS(2109),
    [sym__wc_right_string_int] = ACTIONS(2109),
    [sym__wc_upper_string_int] = ACTIONS(2109),
    [sym__wc_compare_string_int] = ACTIONS(2109),
    [sym__wc_get_buffer_char_int] = ACTIONS(2109),
    [sym__wc_get_buffer_string_int] = ACTIONS(2109),
    [sym__wc_concat_string_int] = ACTIONS(2109),
    [sym__wc_file_open_int] = ACTIONS(2109),
    [sym__wc_file_close_int] = ACTIONS(2109),
    [sym__wc_file_read_int] = ACTIONS(2109),
    [sym__wc_file_read_line_int] = ACTIONS(2109),
    [sym__wc_file_write_int] = ACTIONS(2109),
    [sym__wc_file_write_line_int] = ACTIONS(2109),
    [aux_sym_preproc_include_token1] = ACTIONS(2109),
    [aux_sym_preproc_define_token1] = ACTIONS(2111),
    [aux_sym_preproc_warn_token1] = ACTIONS(2111),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2111),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2111),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2111),
    [anon_sym_LBRACK] = ACTIONS(2111),
    [anon_sym_LBRACE] = ACTIONS(2111),
    [anon_sym_LPAREN] = ACTIONS(2109),
    [anon_sym_BANG] = ACTIONS(2111),
    [anon_sym_TILDE] = ACTIONS(2111),
    [anon_sym_DASH] = ACTIONS(2109),
    [anon_sym_PLUS] = ACTIONS(2109),
    [anon_sym_DASH_DASH] = ACTIONS(2111),
    [anon_sym_PLUS_PLUS] = ACTIONS(2111),
    [anon_sym_DQUOTE] = ACTIONS(2111),
    [anon_sym_SQUOTE] = ACTIONS(2111),
    [sym_number_literal] = ACTIONS(2111),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(415)] = {
    [ts_builtin_sym_end] = ACTIONS(1959),
    [sym_identifier] = ACTIONS(1957),
    [sym_program_name_keyword] = ACTIONS(1957),
    [sym_module_name_keyword] = ACTIONS(1957),
    [sym_define_device_keyword] = ACTIONS(1957),
    [sym_define_combine_keyword] = ACTIONS(1957),
    [sym_define_constant_keyword] = ACTIONS(1957),
    [sym_define_type_keyword] = ACTIONS(1957),
    [sym_define_variable_keyword] = ACTIONS(1957),
    [sym_define_system_variable_keyword] = ACTIONS(1957),
    [sym_define_connect_level_keyword] = ACTIONS(1957),
    [sym_define_latching_keyword] = ACTIONS(1957),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1957),
    [sym_define_toggling_keyword] = ACTIONS(1957),
    [sym_define_call_keyword] = ACTIONS(1957),
    [sym_define_function_keyword] = ACTIONS(1957),
    [sym_define_library_function_keyword] = ACTIONS(1957),
    [sym_define_start_keyword] = ACTIONS(1957),
    [sym_define_event_keyword] = ACTIONS(1957),
    [sym_define_program_keyword] = ACTIONS(1957),
    [sym_define_module_keyword] = ACTIONS(1957),
    [sym_char_keyword] = ACTIONS(1957),
    [sym_widechar_keyword] = ACTIONS(1957),
    [sym_integer_keyword] = ACTIONS(1957),
    [sym_sinteger_keyword] = ACTIONS(1957),
    [sym_long_keyword] = ACTIONS(1957),
    [sym_slong_keyword] = ACTIONS(1957),
    [sym_float_keyword] = ACTIONS(1957),
    [sym_double_keyword] = ACTIONS(1957),
    [sym_variant_keyword] = ACTIONS(1957),
    [sym_variantarray_keyword] = ACTIONS(1957),
    [sym_local_var_keyword] = ACTIONS(1957),
    [sym_stack_var_keyword] = ACTIONS(1957),
    [sym_constant_keyword] = ACTIONS(1957),
    [sym_volatile_keyword] = ACTIONS(1957),
    [sym_non_volatile_keyword] = ACTIONS(1957),
    [sym_persistent_keyword] = ACTIONS(1957),
    [sym_dev_keyword] = ACTIONS(1957),
    [sym_devlev_keyword] = ACTIONS(1957),
    [sym_devchan_keyword] = ACTIONS(1957),
    [sym_if_keyword] = ACTIONS(1957),
    [sym_switch_keyword] = ACTIONS(1957),
    [sym_case_keyword] = ACTIONS(1957),
    [sym_default_keyword] = ACTIONS(1957),
    [sym_while_keyword] = ACTIONS(1957),
    [sym_for_keyword] = ACTIONS(1957),
    [sym_break_keyword] = ACTIONS(1957),
    [sym_continue_keyword] = ACTIONS(1957),
    [sym_return_keyword] = ACTIONS(1957),
    [sym_select_keyword] = ACTIONS(1957),
    [sym_active_keyword] = ACTIONS(1957),
    [sym_struct_keyword] = ACTIONS(1957),
    [sym_structure_keyword] = ACTIONS(1957),
    [sym_button_event_keyword] = ACTIONS(1957),
    [sym_channel_event_keyword] = ACTIONS(1957),
    [sym_level_event_keyword] = ACTIONS(1957),
    [sym_data_event_keyword] = ACTIONS(1957),
    [sym_timeline_event_keyword] = ACTIONS(1957),
    [sym_custom_event_keyword] = ACTIONS(1957),
    [sym_push_keyword] = ACTIONS(1957),
    [sym_release_keyword] = ACTIONS(1957),
    [aux_sym_on_keyword_token1] = ACTIONS(1957),
    [aux_sym_off_keyword_token1] = ACTIONS(1957),
    [sym_send_level_keyword] = ACTIONS(1957),
    [sym_send_string_keyword] = ACTIONS(1957),
    [sym_send_command_keyword] = ACTIONS(1957),
    [sym_clear_buffer_keyword] = ACTIONS(1957),
    [sym_create_buffer_keyword] = ACTIONS(1957),
    [sym_create_multi_buffer_keyword] = ACTIONS(1957),
    [sym_call_keyword] = ACTIONS(1957),
    [sym_system_call_keyword] = ACTIONS(1957),
    [sym_devchan_to_keyword] = ACTIONS(1957),
    [sym_devchan_min_to_keyword] = ACTIONS(1957),
    [sym_devchan_total_off_keyword] = ACTIONS(1957),
    [sym_devchan_pulse_keyword] = ACTIONS(1957),
    [sym_wait_keyword] = ACTIONS(1957),
    [sym_cancel_wait_keyword] = ACTIONS(1957),
    [sym_wait_until_keyword] = ACTIONS(1957),
    [sym_cancel_wait_until_keyword] = ACTIONS(1957),
    [sym_cancel_all_wait_keyword] = ACTIONS(1957),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1957),
    [sym_not] = ACTIONS(1957),
    [sym_bnot] = ACTIONS(1957),
    [sym___file__] = ACTIONS(1957),
    [sym___date__] = ACTIONS(1957),
    [sym___ldate__] = ACTIONS(1957),
    [sym___time__] = ACTIONS(1957),
    [sym___line__] = ACTIONS(1957),
    [sym___name__] = ACTIONS(1957),
    [sym___netlinx__] = ACTIONS(1957),
    [sym_netlinx_axi_version] = ACTIONS(1957),
    [sym_url_flg_tcp] = ACTIONS(1957),
    [sym_url_flg_accinfopresent] = ACTIONS(1957),
    [sym_url_flg_temp] = ACTIONS(1957),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1957),
    [sym_url_flg_stat_mask] = ACTIONS(1957),
    [sym_url_flg_stat_lookup] = ACTIONS(1957),
    [sym_url_flg_stat_connecting] = ACTIONS(1957),
    [sym_url_flg_stat_waiting] = ACTIONS(1957),
    [sym_url_flg_stat_connected] = ACTIONS(1957),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1957),
    [sym_first_virtual_device] = ACTIONS(1957),
    [sym_first_local_port] = ACTIONS(1957),
    [sym_dynamic_virtual_device] = ACTIONS(1957),
    [sym_timeline_once] = ACTIONS(1957),
    [sym_timeline_repeat] = ACTIONS(1957),
    [sym_timeline_absolute] = ACTIONS(1957),
    [sym_timeline_relative] = ACTIONS(1957),
    [sym_do_push_timed_infinite] = ACTIONS(1957),
    [sym_source_type_no_address] = ACTIONS(1957),
    [sym_source_type_neuron_id] = ACTIONS(1957),
    [sym_source_type_ip_address] = ACTIONS(1957),
    [sym_source_type_axlink] = ACTIONS(1957),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1957),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1957),
    [sym_source_type_ip_socket_address] = ACTIONS(1957),
    [sym_source_type_rs232] = ACTIONS(1957),
    [sym_source_type_internal] = ACTIONS(1957),
    [sym_source_type_mac_address] = ACTIONS(1957),
    [sym_source_type_ipv4_port] = ACTIONS(1957),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1957),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1957),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1957),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1957),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1957),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1957),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1957),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1957),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1957),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1957),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1957),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1957),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1957),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1957),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1957),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1957),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1957),
    [sym_true] = ACTIONS(1957),
    [sym_false] = ACTIONS(1957),
    [sym_file_read_only] = ACTIONS(1957),
    [sym_file_rw_new] = ACTIONS(1957),
    [sym_file_rw_append] = ACTIONS(1957),
    [sym_ip_tcp] = ACTIONS(1957),
    [sym_ip_udp] = ACTIONS(1957),
    [sym_ip_udp_2way] = ACTIONS(1957),
    [sym_xml_encode_types] = ACTIONS(1957),
    [sym_xml_encode_char_as_list] = ACTIONS(1957),
    [sym_xml_encode_le] = ACTIONS(1957),
    [sym_xml_decode_types] = ACTIONS(1957),
    [sym_xml_decode_no_preserve] = ACTIONS(1957),
    [sym_duet_dev_type_display_device] = ACTIONS(1957),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1957),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1957),
    [sym_duet_dev_type_io_device] = ACTIONS(1957),
    [sym_duet_dev_type_relay_device] = ACTIONS(1957),
    [sym_duet_dev_type_ups] = ACTIONS(1957),
    [sym_duet_dev_type_amplifier] = ACTIONS(1957),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1957),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1957),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1957),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1957),
    [sym_duet_dev_type_camera] = ACTIONS(1957),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1957),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1957),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1957),
    [sym_duet_dev_type_dss] = ACTIONS(1957),
    [sym_duet_dev_type_dvr] = ACTIONS(1957),
    [sym_duet_dev_type_disc_device] = ACTIONS(1957),
    [sym_duet_dev_type_document_camera] = ACTIONS(1957),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1957),
    [sym_duet_dev_type_hvac] = ACTIONS(1957),
    [sym_duet_dev_type_keypad] = ACTIONS(1957),
    [sym_duet_dev_type_light] = ACTIONS(1957),
    [sym_duet_dev_type_monitor] = ACTIONS(1957),
    [sym_duet_dev_type_motor] = ACTIONS(1957),
    [sym_duet_dev_type_multi_window] = ACTIONS(1957),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1957),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1957),
    [sym_duet_dev_type_receiver] = ACTIONS(1957),
    [sym_duet_dev_type_security_system] = ACTIONS(1957),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1957),
    [sym_duet_dev_type_settop_box] = ACTIONS(1957),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1957),
    [sym_duet_dev_type_switcher] = ACTIONS(1957),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1957),
    [sym_duet_dev_type_tv] = ACTIONS(1957),
    [sym_duet_dev_type_utility] = ACTIONS(1957),
    [sym_duet_dev_type_vcr] = ACTIONS(1957),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1957),
    [sym_duet_dev_type_video_processor] = ACTIONS(1957),
    [sym_duet_dev_type_video_projector] = ACTIONS(1957),
    [sym_duet_dev_type_video_wall] = ACTIONS(1957),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1957),
    [sym_duet_dev_type_weather] = ACTIONS(1957),
    [sym_duet_dev_not_polled] = ACTIONS(1957),
    [sym_duet_dev_polled] = ACTIONS(1957),
    [sym_clkmgr_mode_network] = ACTIONS(1957),
    [sym_clkmgr_mode_standalone] = ACTIONS(1957),
    [sym_normal_standby] = ACTIONS(1957),
    [sym_normal_wake] = ACTIONS(1957),
    [sym_ip_multicast_ttl_option] = ACTIONS(1957),
    [sym_ip_ttl_subnet] = ACTIONS(1957),
    [sym_ip_ttl_site] = ACTIONS(1957),
    [sym_ip_ttl_region] = ACTIONS(1957),
    [sym_ip_ttl_continent] = ACTIONS(1957),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1957),
    [sym_ip_nodelay_on] = ACTIONS(1957),
    [sym_ip_nodelay_off] = ACTIONS(1957),
    [sym_amx_error] = ACTIONS(1957),
    [sym_amx_warning] = ACTIONS(1957),
    [sym_amx_info] = ACTIONS(1957),
    [sym_amx_debug] = ACTIONS(1957),
    [sym_netlinx_login_success] = ACTIONS(1957),
    [sym_netlinx_login_fail] = ACTIONS(1957),
    [sym_netlinx_logout] = ACTIONS(1957),
    [sym_null_str] = ACTIONS(1957),
    [sym_smtp_address] = ACTIONS(1957),
    [sym_smtp_port_number] = ACTIONS(1957),
    [sym_smtp_username] = ACTIONS(1957),
    [sym_smtp_password] = ACTIONS(1957),
    [sym_smtp_require_tls] = ACTIONS(1957),
    [sym_smtp_from] = ACTIONS(1957),
    [sym_smtp_tls_true] = ACTIONS(1957),
    [sym_smtp_tls_false] = ACTIONS(1957),
    [sym_listview_on_row_select_event] = ACTIONS(1957),
    [sym_data_structure_unknown] = ACTIONS(1957),
    [sym_data_structure_datafeed] = ACTIONS(1957),
    [sym_data_structure_datarecordset] = ACTIONS(1957),
    [sym_data_structure_datarecord] = ACTIONS(1957),
    [sym_data_structure_datafield] = ACTIONS(1957),
    [sym_data_max_content_field_count] = ACTIONS(1957),
    [sym_data_max_metadata_field_count] = ACTIONS(1957),
    [sym_data_max_id_length] = ACTIONS(1957),
    [sym_data_max_name_length] = ACTIONS(1957),
    [sym_data_max_description_length] = ACTIONS(1957),
    [sym_data_max_type_length] = ACTIONS(1957),
    [sym_data_max_format_length] = ACTIONS(1957),
    [sym_data_max_source_length] = ACTIONS(1957),
    [sym_data_max_value_length] = ACTIONS(1957),
    [sym_data_max_label_length] = ACTIONS(1957),
    [sym_data_max_url_length] = ACTIONS(1957),
    [sym_max_last_login_info_length] = ACTIONS(1957),
    [sym_data_type_unknown] = ACTIONS(1957),
    [sym_data_type_string] = ACTIONS(1957),
    [sym_data_type_datetime] = ACTIONS(1957),
    [sym_data_type_date] = ACTIONS(1957),
    [sym_data_type_time] = ACTIONS(1957),
    [sym_data_type_image] = ACTIONS(1957),
    [sym_data_format_url] = ACTIONS(1957),
    [sym_data_format_phone] = ACTIONS(1957),
    [sym_data_format_email] = ACTIONS(1957),
    [sym_data_format_iso8601] = ACTIONS(1957),
    [sym_auditlog_priv] = ACTIONS(1957),
    [sym_configuration_priv] = ACTIONS(1957),
    [sym_deviceconfig_priv] = ACTIONS(1957),
    [sym_ftp_priv] = ACTIONS(1957),
    [sym_http_priv] = ACTIONS(1957),
    [sym_networkconfig_priv] = ACTIONS(1957),
    [sym_programport_priv] = ACTIONS(1957),
    [sym_remoteui_priv] = ACTIONS(1957),
    [sym_securitycontrol_priv] = ACTIONS(1957),
    [sym_softwaremanagement_priv] = ACTIONS(1957),
    [sym_terminal_priv] = ACTIONS(1957),
    [sym_tpadmin_priv] = ACTIONS(1957),
    [sym_user1_priv] = ACTIONS(1957),
    [sym_user2_priv] = ACTIONS(1957),
    [sym_user3_priv] = ACTIONS(1957),
    [sym_user4_priv] = ACTIONS(1957),
    [sym_usermanagement_priv] = ACTIONS(1957),
    [sym_valid_account] = ACTIONS(1957),
    [sym_err_name_invalid] = ACTIONS(1957),
    [sym_err_password_invalid] = ACTIONS(1957),
    [sym_err_invalid_account] = ACTIONS(1957),
    [sym_err_no_authorization] = ACTIONS(1957),
    [sym_err_invalid_parameter] = ACTIONS(1957),
    [sym_err_user_locked_out] = ACTIONS(1957),
    [sym_err_user_account_expired] = ACTIONS(1957),
    [sym_err_general_error] = ACTIONS(1957),
    [sym_err_authentication_error] = ACTIONS(1957),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1957),
    [sym_tls_validate_certificate] = ACTIONS(1957),
    [sym_tdata] = ACTIONS(1957),
    [sym_tchannel] = ACTIONS(1957),
    [sym_tlevel] = ACTIONS(1957),
    [sym_tbutton] = ACTIONS(1957),
    [sym_ttimeline] = ACTIONS(1957),
    [sym_tcustom] = ACTIONS(1957),
    [sym_url_struct] = ACTIONS(1957),
    [sym_dns_struct] = ACTIONS(1957),
    [sym_ip_address_struct] = ACTIONS(1957),
    [sym_dev_info_struct] = ACTIONS(1957),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1957),
    [sym_data_feed] = ACTIONS(1957),
    [sym_data_field] = ACTIONS(1957),
    [sym_data_record] = ACTIONS(1957),
    [sym_wc_data_feed] = ACTIONS(1957),
    [sym_wc_data_field] = ACTIONS(1957),
    [sym_wc_data_record] = ACTIONS(1957),
    [sym_last_login_info] = ACTIONS(1957),
    [sym____reserved___] = ACTIONS(1957),
    [sym_date] = ACTIONS(1957),
    [sym_day] = ACTIONS(1957),
    [sym_time] = ACTIONS(1957),
    [sym_ldate] = ACTIONS(1957),
    [sym_push_device] = ACTIONS(1957),
    [sym_push_channel] = ACTIONS(1957),
    [sym_push_devchan] = ACTIONS(1957),
    [sym_release_device] = ACTIONS(1957),
    [sym_release_channel] = ACTIONS(1957),
    [sym_release_devchan] = ACTIONS(1957),
    [sym_master_slot] = ACTIONS(1957),
    [sym_get_pulse_time] = ACTIONS(1957),
    [sym_get_timer] = ACTIONS(1957),
    [sym_master_sn] = ACTIONS(1957),
    [sym_system_number] = ACTIONS(1957),
    [sym_button] = ACTIONS(1957),
    [sym_dv_channel] = ACTIONS(1957),
    [sym_data] = ACTIONS(1957),
    [sym_level] = ACTIONS(1957),
    [sym_timeline] = ACTIONS(1957),
    [sym_channel] = ACTIONS(1957),
    [sym_custom] = ACTIONS(1957),
    [sym_atoi] = ACTIONS(1957),
    [sym_atol] = ACTIONS(1957),
    [sym_atof] = ACTIONS(1957),
    [sym_itoa] = ACTIONS(1957),
    [sym_itohex] = ACTIONS(1957),
    [sym_hextoi] = ACTIONS(1957),
    [sym_ftoa] = ACTIONS(1957),
    [sym_find_string] = ACTIONS(1957),
    [sym_left_string] = ACTIONS(1957),
    [sym_right_string] = ACTIONS(1957),
    [sym_mid_string] = ACTIONS(1957),
    [sym_remove_string] = ACTIONS(1957),
    [sym_get_buffer_string] = ACTIONS(1957),
    [sym_get_buffer_char] = ACTIONS(1957),
    [sym_get_multi_buffer_string] = ACTIONS(1957),
    [sym_length_string] = ACTIONS(1957),
    [sym_max_length_string] = ACTIONS(1957),
    [sym_set_length_string] = ACTIONS(1957),
    [sym_compare_string] = ACTIONS(1957),
    [sym_upper_string] = ACTIONS(1957),
    [sym_lower_string] = ACTIONS(1957),
    [sym_get_last] = ACTIONS(1957),
    [sym_date_to_day] = ACTIONS(1957),
    [sym_date_to_month] = ACTIONS(1957),
    [sym_date_to_year] = ACTIONS(1957),
    [sym_time_to_hour] = ACTIONS(1957),
    [sym_time_to_minute] = ACTIONS(1957),
    [sym_time_to_second] = ACTIONS(1957),
    [sym_day_of_week] = ACTIONS(1957),
    [sym_set_pulse_time] = ACTIONS(1957),
    [sym_set_timer] = ACTIONS(1957),
    [sym_file_open] = ACTIONS(1957),
    [sym_file_close] = ACTIONS(1957),
    [sym_file_read] = ACTIONS(1957),
    [sym_file_read_line] = ACTIONS(1957),
    [sym_file_write] = ACTIONS(1957),
    [sym_file_write_line] = ACTIONS(1957),
    [sym_file_copy] = ACTIONS(1957),
    [sym_file_delete] = ACTIONS(1957),
    [sym_file_rename] = ACTIONS(1957),
    [sym_file_dir] = ACTIONS(1957),
    [sym_file_getdir] = ACTIONS(1957),
    [sym_file_setdir] = ACTIONS(1957),
    [sym_file_create_dir] = ACTIONS(1957),
    [sym_file_remove_dir] = ACTIONS(1957),
    [sym_file_seek] = ACTIONS(1957),
    [sym_length_array] = ACTIONS(1957),
    [sym_max_length_array] = ACTIONS(1957),
    [sym_set_length_array] = ACTIONS(1957),
    [sym_ip_client_open] = ACTIONS(1957),
    [sym_ip_client_close] = ACTIONS(1957),
    [sym_ip_server_open] = ACTIONS(1957),
    [sym_ip_server_close] = ACTIONS(1957),
    [sym_ip_bound_client_open] = ACTIONS(1957),
    [sym_ip_mc_server_open] = ACTIONS(1957),
    [sym_ip_set_option] = ACTIONS(1957),
    [sym_ssh_client_open] = ACTIONS(1957),
    [sym_ssh_client_close] = ACTIONS(1957),
    [sym_tls_client_open] = ACTIONS(1957),
    [sym_tls_client_close] = ACTIONS(1957),
    [sym_do_push] = ACTIONS(1957),
    [sym_do_push_timed] = ACTIONS(1957),
    [sym_do_release] = ACTIONS(1957),
    [sym_device_id] = ACTIONS(1957),
    [sym_device_info] = ACTIONS(1957),
    [sym_device_id_string] = ACTIONS(1957),
    [sym_device_standby] = ACTIONS(1957),
    [sym_device_wake] = ACTIONS(1957),
    [sym_timeline_create] = ACTIONS(1957),
    [sym_timeline_kill] = ACTIONS(1957),
    [sym_timeline_reload] = ACTIONS(1957),
    [sym_timeline_pause] = ACTIONS(1957),
    [sym_timeline_restart] = ACTIONS(1957),
    [sym_timeline_set] = ACTIONS(1957),
    [sym_timeline_get] = ACTIONS(1957),
    [sym_timeline_active] = ACTIONS(1957),
    [sym_timeline_dynamic_id] = ACTIONS(1957),
    [sym_variable_to_string] = ACTIONS(1957),
    [sym_string_to_variable] = ACTIONS(1957),
    [sym_length_variable_to_string] = ACTIONS(1957),
    [sym_type_cast] = ACTIONS(1957),
    [sym_variable_to_xml] = ACTIONS(1957),
    [sym_xml_to_variable] = ACTIONS(1957),
    [sym_length_variable_to_xml] = ACTIONS(1957),
    [sym_redirect_string] = ACTIONS(1957),
    [sym_random_number] = ACTIONS(1957),
    [sym_get_unique_id] = ACTIONS(1957),
    [sym_get_system_number] = ACTIONS(1957),
    [sym_set_system_number] = ACTIONS(1957),
    [sym_get_serial_number] = ACTIONS(1957),
    [sym_get_master_build] = ACTIONS(1957),
    [sym_set_validation_code] = ACTIONS(1957),
    [sym_reboot] = ACTIONS(1957),
    [sym_load_duet_module] = ACTIONS(1957),
    [sym_unload_duet_module] = ACTIONS(1957),
    [sym_duet_mem_size_set] = ACTIONS(1957),
    [sym_duet_mem_size_get] = ACTIONS(1957),
    [sym_dynamic_application_device] = ACTIONS(1957),
    [sym_dynamic_polled_port] = ACTIONS(1957),
    [sym_static_port_binding] = ACTIONS(1957),
    [sym_static_ip_binding] = ACTIONS(1957),
    [sym_get_dns_list] = ACTIONS(1957),
    [sym_set_dns_list] = ACTIONS(1957),
    [sym_get_ip_address] = ACTIONS(1957),
    [sym_set_ip_address] = ACTIONS(1957),
    [sym_get_url_list] = ACTIONS(1957),
    [sym_add_url_entry] = ACTIONS(1957),
    [sym_delete_url_entry] = ACTIONS(1957),
    [sym_abs_value] = ACTIONS(1957),
    [sym_max_value] = ACTIONS(1957),
    [sym_min_value] = ACTIONS(1957),
    [sym_exp_value] = ACTIONS(1957),
    [sym_log_value] = ACTIONS(1957),
    [sym_log10_value] = ACTIONS(1957),
    [sym_power_value] = ACTIONS(1957),
    [sym_sqrt_value] = ACTIONS(1957),
    [sym_raw_be] = ACTIONS(1957),
    [sym_raw_le] = ACTIONS(1957),
    [sym_internal_queue_size_set] = ACTIONS(1957),
    [sym_internal_queue_size_get] = ACTIONS(1957),
    [sym_internal_threshold_set] = ACTIONS(1957),
    [sym_internal_threshold_get] = ACTIONS(1957),
    [sym_rebuilt_event] = ACTIONS(1957),
    [sym_do_custom_event] = ACTIONS(1957),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1957),
    [sym_clkmgr_set_clk_source] = ACTIONS(1957),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1957),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1957),
    [sym_clkmgr_get_timezone] = ACTIONS(1957),
    [sym_clkmgr_set_timezone] = ACTIONS(1957),
    [sym_clkmgr_get_resync_period] = ACTIONS(1957),
    [sym_clkmgr_set_resync_period] = ACTIONS(1957),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1957),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1957),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1957),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1957),
    [sym_clkmgr_get_timeservers] = ACTIONS(1957),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1957),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1957),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1957),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1957),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1957),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1957),
    [sym_set_log_level] = ACTIONS(1957),
    [sym_get_log_level] = ACTIONS(1957),
    [sym_amx_log] = ACTIONS(1957),
    [sym_smtp_server_config_set] = ACTIONS(1957),
    [sym_smtp_server_config_get] = ACTIONS(1957),
    [sym_smtp_send] = ACTIONS(1957),
    [sym_validate_netlinx_account] = ACTIONS(1957),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1957),
    [sym_authenticate_certificate] = ACTIONS(1957),
    [sym_audit_netlinx_session_event] = ACTIONS(1957),
    [sym_audit_netlinx_generic_event] = ACTIONS(1957),
    [sym_get_available_flash_disk_space] = ACTIONS(1957),
    [sym_get_max_flash_disk_space] = ACTIONS(1957),
    [sym_data_create_feed] = ACTIONS(1957),
    [sym_data_delete_feed] = ACTIONS(1957),
    [sym_data_publish_feed] = ACTIONS(1957),
    [sym_data_get_published_feed] = ACTIONS(1957),
    [sym_data_add_record] = ACTIONS(1957),
    [sym_data_get_event_record] = ACTIONS(1957),
    [sym__wc_data_create_feed] = ACTIONS(1957),
    [sym__wc_data_add_record] = ACTIONS(1957),
    [sym__wc_data_get_event_record] = ACTIONS(1957),
    [sym_format] = ACTIONS(1957),
    [sym_astro_clock] = ACTIONS(1957),
    [sym_set_outdoor_temperature] = ACTIONS(1957),
    [sym_set_virtual_level_count] = ACTIONS(1957),
    [sym_set_virtual_channel_count] = ACTIONS(1957),
    [sym_set_virtual_port_count] = ACTIONS(1957),
    [sym__wc_encode_int] = ACTIONS(1957),
    [sym__wc_decode_int] = ACTIONS(1957),
    [sym__wc_int] = ACTIONS(1957),
    [sym__wc_to_ch_int] = ACTIONS(1957),
    [sym__ch_to_wc_int] = ACTIONS(1957),
    [sym__wc_find_string_int] = ACTIONS(1957),
    [sym__wc_left_string_int] = ACTIONS(1957),
    [sym__wc_lower_string_int] = ACTIONS(1957),
    [sym__wc_mid_string_int] = ACTIONS(1957),
    [sym__wc_remove_string_int] = ACTIONS(1957),
    [sym__wc_right_string_int] = ACTIONS(1957),
    [sym__wc_upper_string_int] = ACTIONS(1957),
    [sym__wc_compare_string_int] = ACTIONS(1957),
    [sym__wc_get_buffer_char_int] = ACTIONS(1957),
    [sym__wc_get_buffer_string_int] = ACTIONS(1957),
    [sym__wc_concat_string_int] = ACTIONS(1957),
    [sym__wc_file_open_int] = ACTIONS(1957),
    [sym__wc_file_close_int] = ACTIONS(1957),
    [sym__wc_file_read_int] = ACTIONS(1957),
    [sym__wc_file_read_line_int] = ACTIONS(1957),
    [sym__wc_file_write_int] = ACTIONS(1957),
    [sym__wc_file_write_line_int] = ACTIONS(1957),
    [aux_sym_preproc_include_token1] = ACTIONS(1957),
    [aux_sym_preproc_define_token1] = ACTIONS(1959),
    [aux_sym_preproc_warn_token1] = ACTIONS(1959),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1959),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1959),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1959),
    [anon_sym_LBRACK] = ACTIONS(1959),
    [anon_sym_LBRACE] = ACTIONS(1959),
    [anon_sym_LPAREN] = ACTIONS(1957),
    [anon_sym_BANG] = ACTIONS(1959),
    [anon_sym_TILDE] = ACTIONS(1959),
    [anon_sym_DASH] = ACTIONS(1957),
    [anon_sym_PLUS] = ACTIONS(1957),
    [anon_sym_DASH_DASH] = ACTIONS(1959),
    [anon_sym_PLUS_PLUS] = ACTIONS(1959),
    [anon_sym_DQUOTE] = ACTIONS(1959),
    [anon_sym_SQUOTE] = ACTIONS(1959),
    [sym_number_literal] = ACTIONS(1959),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(416)] = {
    [ts_builtin_sym_end] = ACTIONS(2123),
    [sym_identifier] = ACTIONS(2121),
    [sym_program_name_keyword] = ACTIONS(2121),
    [sym_module_name_keyword] = ACTIONS(2121),
    [sym_define_device_keyword] = ACTIONS(2121),
    [sym_define_combine_keyword] = ACTIONS(2121),
    [sym_define_constant_keyword] = ACTIONS(2121),
    [sym_define_type_keyword] = ACTIONS(2121),
    [sym_define_variable_keyword] = ACTIONS(2121),
    [sym_define_system_variable_keyword] = ACTIONS(2121),
    [sym_define_connect_level_keyword] = ACTIONS(2121),
    [sym_define_latching_keyword] = ACTIONS(2121),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2121),
    [sym_define_toggling_keyword] = ACTIONS(2121),
    [sym_define_call_keyword] = ACTIONS(2121),
    [sym_define_function_keyword] = ACTIONS(2121),
    [sym_define_library_function_keyword] = ACTIONS(2121),
    [sym_define_start_keyword] = ACTIONS(2121),
    [sym_define_event_keyword] = ACTIONS(2121),
    [sym_define_program_keyword] = ACTIONS(2121),
    [sym_define_module_keyword] = ACTIONS(2121),
    [sym_char_keyword] = ACTIONS(2121),
    [sym_widechar_keyword] = ACTIONS(2121),
    [sym_integer_keyword] = ACTIONS(2121),
    [sym_sinteger_keyword] = ACTIONS(2121),
    [sym_long_keyword] = ACTIONS(2121),
    [sym_slong_keyword] = ACTIONS(2121),
    [sym_float_keyword] = ACTIONS(2121),
    [sym_double_keyword] = ACTIONS(2121),
    [sym_variant_keyword] = ACTIONS(2121),
    [sym_variantarray_keyword] = ACTIONS(2121),
    [sym_local_var_keyword] = ACTIONS(2121),
    [sym_stack_var_keyword] = ACTIONS(2121),
    [sym_constant_keyword] = ACTIONS(2121),
    [sym_volatile_keyword] = ACTIONS(2121),
    [sym_non_volatile_keyword] = ACTIONS(2121),
    [sym_persistent_keyword] = ACTIONS(2121),
    [sym_dev_keyword] = ACTIONS(2121),
    [sym_devlev_keyword] = ACTIONS(2121),
    [sym_devchan_keyword] = ACTIONS(2121),
    [sym_if_keyword] = ACTIONS(2121),
    [sym_switch_keyword] = ACTIONS(2121),
    [sym_case_keyword] = ACTIONS(2121),
    [sym_default_keyword] = ACTIONS(2121),
    [sym_while_keyword] = ACTIONS(2121),
    [sym_for_keyword] = ACTIONS(2121),
    [sym_break_keyword] = ACTIONS(2121),
    [sym_continue_keyword] = ACTIONS(2121),
    [sym_return_keyword] = ACTIONS(2121),
    [sym_select_keyword] = ACTIONS(2121),
    [sym_active_keyword] = ACTIONS(2121),
    [sym_struct_keyword] = ACTIONS(2121),
    [sym_structure_keyword] = ACTIONS(2121),
    [sym_button_event_keyword] = ACTIONS(2121),
    [sym_channel_event_keyword] = ACTIONS(2121),
    [sym_level_event_keyword] = ACTIONS(2121),
    [sym_data_event_keyword] = ACTIONS(2121),
    [sym_timeline_event_keyword] = ACTIONS(2121),
    [sym_custom_event_keyword] = ACTIONS(2121),
    [sym_push_keyword] = ACTIONS(2121),
    [sym_release_keyword] = ACTIONS(2121),
    [aux_sym_on_keyword_token1] = ACTIONS(2121),
    [aux_sym_off_keyword_token1] = ACTIONS(2121),
    [sym_send_level_keyword] = ACTIONS(2121),
    [sym_send_string_keyword] = ACTIONS(2121),
    [sym_send_command_keyword] = ACTIONS(2121),
    [sym_clear_buffer_keyword] = ACTIONS(2121),
    [sym_create_buffer_keyword] = ACTIONS(2121),
    [sym_create_multi_buffer_keyword] = ACTIONS(2121),
    [sym_call_keyword] = ACTIONS(2121),
    [sym_system_call_keyword] = ACTIONS(2121),
    [sym_devchan_to_keyword] = ACTIONS(2121),
    [sym_devchan_min_to_keyword] = ACTIONS(2121),
    [sym_devchan_total_off_keyword] = ACTIONS(2121),
    [sym_devchan_pulse_keyword] = ACTIONS(2121),
    [sym_wait_keyword] = ACTIONS(2121),
    [sym_cancel_wait_keyword] = ACTIONS(2121),
    [sym_wait_until_keyword] = ACTIONS(2121),
    [sym_cancel_wait_until_keyword] = ACTIONS(2121),
    [sym_cancel_all_wait_keyword] = ACTIONS(2121),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2121),
    [sym_not] = ACTIONS(2121),
    [sym_bnot] = ACTIONS(2121),
    [sym___file__] = ACTIONS(2121),
    [sym___date__] = ACTIONS(2121),
    [sym___ldate__] = ACTIONS(2121),
    [sym___time__] = ACTIONS(2121),
    [sym___line__] = ACTIONS(2121),
    [sym___name__] = ACTIONS(2121),
    [sym___netlinx__] = ACTIONS(2121),
    [sym_netlinx_axi_version] = ACTIONS(2121),
    [sym_url_flg_tcp] = ACTIONS(2121),
    [sym_url_flg_accinfopresent] = ACTIONS(2121),
    [sym_url_flg_temp] = ACTIONS(2121),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2121),
    [sym_url_flg_stat_mask] = ACTIONS(2121),
    [sym_url_flg_stat_lookup] = ACTIONS(2121),
    [sym_url_flg_stat_connecting] = ACTIONS(2121),
    [sym_url_flg_stat_waiting] = ACTIONS(2121),
    [sym_url_flg_stat_connected] = ACTIONS(2121),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2121),
    [sym_first_virtual_device] = ACTIONS(2121),
    [sym_first_local_port] = ACTIONS(2121),
    [sym_dynamic_virtual_device] = ACTIONS(2121),
    [sym_timeline_once] = ACTIONS(2121),
    [sym_timeline_repeat] = ACTIONS(2121),
    [sym_timeline_absolute] = ACTIONS(2121),
    [sym_timeline_relative] = ACTIONS(2121),
    [sym_do_push_timed_infinite] = ACTIONS(2121),
    [sym_source_type_no_address] = ACTIONS(2121),
    [sym_source_type_neuron_id] = ACTIONS(2121),
    [sym_source_type_ip_address] = ACTIONS(2121),
    [sym_source_type_axlink] = ACTIONS(2121),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2121),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2121),
    [sym_source_type_ip_socket_address] = ACTIONS(2121),
    [sym_source_type_rs232] = ACTIONS(2121),
    [sym_source_type_internal] = ACTIONS(2121),
    [sym_source_type_mac_address] = ACTIONS(2121),
    [sym_source_type_ipv4_port] = ACTIONS(2121),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2121),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2121),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2121),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2121),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2121),
    [sym_true] = ACTIONS(2121),
    [sym_false] = ACTIONS(2121),
    [sym_file_read_only] = ACTIONS(2121),
    [sym_file_rw_new] = ACTIONS(2121),
    [sym_file_rw_append] = ACTIONS(2121),
    [sym_ip_tcp] = ACTIONS(2121),
    [sym_ip_udp] = ACTIONS(2121),
    [sym_ip_udp_2way] = ACTIONS(2121),
    [sym_xml_encode_types] = ACTIONS(2121),
    [sym_xml_encode_char_as_list] = ACTIONS(2121),
    [sym_xml_encode_le] = ACTIONS(2121),
    [sym_xml_decode_types] = ACTIONS(2121),
    [sym_xml_decode_no_preserve] = ACTIONS(2121),
    [sym_duet_dev_type_display_device] = ACTIONS(2121),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2121),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2121),
    [sym_duet_dev_type_io_device] = ACTIONS(2121),
    [sym_duet_dev_type_relay_device] = ACTIONS(2121),
    [sym_duet_dev_type_ups] = ACTIONS(2121),
    [sym_duet_dev_type_amplifier] = ACTIONS(2121),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2121),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2121),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2121),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2121),
    [sym_duet_dev_type_camera] = ACTIONS(2121),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2121),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2121),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2121),
    [sym_duet_dev_type_dss] = ACTIONS(2121),
    [sym_duet_dev_type_dvr] = ACTIONS(2121),
    [sym_duet_dev_type_disc_device] = ACTIONS(2121),
    [sym_duet_dev_type_document_camera] = ACTIONS(2121),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2121),
    [sym_duet_dev_type_hvac] = ACTIONS(2121),
    [sym_duet_dev_type_keypad] = ACTIONS(2121),
    [sym_duet_dev_type_light] = ACTIONS(2121),
    [sym_duet_dev_type_monitor] = ACTIONS(2121),
    [sym_duet_dev_type_motor] = ACTIONS(2121),
    [sym_duet_dev_type_multi_window] = ACTIONS(2121),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2121),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2121),
    [sym_duet_dev_type_receiver] = ACTIONS(2121),
    [sym_duet_dev_type_security_system] = ACTIONS(2121),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2121),
    [sym_duet_dev_type_settop_box] = ACTIONS(2121),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2121),
    [sym_duet_dev_type_switcher] = ACTIONS(2121),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2121),
    [sym_duet_dev_type_tv] = ACTIONS(2121),
    [sym_duet_dev_type_utility] = ACTIONS(2121),
    [sym_duet_dev_type_vcr] = ACTIONS(2121),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2121),
    [sym_duet_dev_type_video_processor] = ACTIONS(2121),
    [sym_duet_dev_type_video_projector] = ACTIONS(2121),
    [sym_duet_dev_type_video_wall] = ACTIONS(2121),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2121),
    [sym_duet_dev_type_weather] = ACTIONS(2121),
    [sym_duet_dev_not_polled] = ACTIONS(2121),
    [sym_duet_dev_polled] = ACTIONS(2121),
    [sym_clkmgr_mode_network] = ACTIONS(2121),
    [sym_clkmgr_mode_standalone] = ACTIONS(2121),
    [sym_normal_standby] = ACTIONS(2121),
    [sym_normal_wake] = ACTIONS(2121),
    [sym_ip_multicast_ttl_option] = ACTIONS(2121),
    [sym_ip_ttl_subnet] = ACTIONS(2121),
    [sym_ip_ttl_site] = ACTIONS(2121),
    [sym_ip_ttl_region] = ACTIONS(2121),
    [sym_ip_ttl_continent] = ACTIONS(2121),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2121),
    [sym_ip_nodelay_on] = ACTIONS(2121),
    [sym_ip_nodelay_off] = ACTIONS(2121),
    [sym_amx_error] = ACTIONS(2121),
    [sym_amx_warning] = ACTIONS(2121),
    [sym_amx_info] = ACTIONS(2121),
    [sym_amx_debug] = ACTIONS(2121),
    [sym_netlinx_login_success] = ACTIONS(2121),
    [sym_netlinx_login_fail] = ACTIONS(2121),
    [sym_netlinx_logout] = ACTIONS(2121),
    [sym_null_str] = ACTIONS(2121),
    [sym_smtp_address] = ACTIONS(2121),
    [sym_smtp_port_number] = ACTIONS(2121),
    [sym_smtp_username] = ACTIONS(2121),
    [sym_smtp_password] = ACTIONS(2121),
    [sym_smtp_require_tls] = ACTIONS(2121),
    [sym_smtp_from] = ACTIONS(2121),
    [sym_smtp_tls_true] = ACTIONS(2121),
    [sym_smtp_tls_false] = ACTIONS(2121),
    [sym_listview_on_row_select_event] = ACTIONS(2121),
    [sym_data_structure_unknown] = ACTIONS(2121),
    [sym_data_structure_datafeed] = ACTIONS(2121),
    [sym_data_structure_datarecordset] = ACTIONS(2121),
    [sym_data_structure_datarecord] = ACTIONS(2121),
    [sym_data_structure_datafield] = ACTIONS(2121),
    [sym_data_max_content_field_count] = ACTIONS(2121),
    [sym_data_max_metadata_field_count] = ACTIONS(2121),
    [sym_data_max_id_length] = ACTIONS(2121),
    [sym_data_max_name_length] = ACTIONS(2121),
    [sym_data_max_description_length] = ACTIONS(2121),
    [sym_data_max_type_length] = ACTIONS(2121),
    [sym_data_max_format_length] = ACTIONS(2121),
    [sym_data_max_source_length] = ACTIONS(2121),
    [sym_data_max_value_length] = ACTIONS(2121),
    [sym_data_max_label_length] = ACTIONS(2121),
    [sym_data_max_url_length] = ACTIONS(2121),
    [sym_max_last_login_info_length] = ACTIONS(2121),
    [sym_data_type_unknown] = ACTIONS(2121),
    [sym_data_type_string] = ACTIONS(2121),
    [sym_data_type_datetime] = ACTIONS(2121),
    [sym_data_type_date] = ACTIONS(2121),
    [sym_data_type_time] = ACTIONS(2121),
    [sym_data_type_image] = ACTIONS(2121),
    [sym_data_format_url] = ACTIONS(2121),
    [sym_data_format_phone] = ACTIONS(2121),
    [sym_data_format_email] = ACTIONS(2121),
    [sym_data_format_iso8601] = ACTIONS(2121),
    [sym_auditlog_priv] = ACTIONS(2121),
    [sym_configuration_priv] = ACTIONS(2121),
    [sym_deviceconfig_priv] = ACTIONS(2121),
    [sym_ftp_priv] = ACTIONS(2121),
    [sym_http_priv] = ACTIONS(2121),
    [sym_networkconfig_priv] = ACTIONS(2121),
    [sym_programport_priv] = ACTIONS(2121),
    [sym_remoteui_priv] = ACTIONS(2121),
    [sym_securitycontrol_priv] = ACTIONS(2121),
    [sym_softwaremanagement_priv] = ACTIONS(2121),
    [sym_terminal_priv] = ACTIONS(2121),
    [sym_tpadmin_priv] = ACTIONS(2121),
    [sym_user1_priv] = ACTIONS(2121),
    [sym_user2_priv] = ACTIONS(2121),
    [sym_user3_priv] = ACTIONS(2121),
    [sym_user4_priv] = ACTIONS(2121),
    [sym_usermanagement_priv] = ACTIONS(2121),
    [sym_valid_account] = ACTIONS(2121),
    [sym_err_name_invalid] = ACTIONS(2121),
    [sym_err_password_invalid] = ACTIONS(2121),
    [sym_err_invalid_account] = ACTIONS(2121),
    [sym_err_no_authorization] = ACTIONS(2121),
    [sym_err_invalid_parameter] = ACTIONS(2121),
    [sym_err_user_locked_out] = ACTIONS(2121),
    [sym_err_user_account_expired] = ACTIONS(2121),
    [sym_err_general_error] = ACTIONS(2121),
    [sym_err_authentication_error] = ACTIONS(2121),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2121),
    [sym_tls_validate_certificate] = ACTIONS(2121),
    [sym_tdata] = ACTIONS(2121),
    [sym_tchannel] = ACTIONS(2121),
    [sym_tlevel] = ACTIONS(2121),
    [sym_tbutton] = ACTIONS(2121),
    [sym_ttimeline] = ACTIONS(2121),
    [sym_tcustom] = ACTIONS(2121),
    [sym_url_struct] = ACTIONS(2121),
    [sym_dns_struct] = ACTIONS(2121),
    [sym_ip_address_struct] = ACTIONS(2121),
    [sym_dev_info_struct] = ACTIONS(2121),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2121),
    [sym_data_feed] = ACTIONS(2121),
    [sym_data_field] = ACTIONS(2121),
    [sym_data_record] = ACTIONS(2121),
    [sym_wc_data_feed] = ACTIONS(2121),
    [sym_wc_data_field] = ACTIONS(2121),
    [sym_wc_data_record] = ACTIONS(2121),
    [sym_last_login_info] = ACTIONS(2121),
    [sym____reserved___] = ACTIONS(2121),
    [sym_date] = ACTIONS(2121),
    [sym_day] = ACTIONS(2121),
    [sym_time] = ACTIONS(2121),
    [sym_ldate] = ACTIONS(2121),
    [sym_push_device] = ACTIONS(2121),
    [sym_push_channel] = ACTIONS(2121),
    [sym_push_devchan] = ACTIONS(2121),
    [sym_release_device] = ACTIONS(2121),
    [sym_release_channel] = ACTIONS(2121),
    [sym_release_devchan] = ACTIONS(2121),
    [sym_master_slot] = ACTIONS(2121),
    [sym_get_pulse_time] = ACTIONS(2121),
    [sym_get_timer] = ACTIONS(2121),
    [sym_master_sn] = ACTIONS(2121),
    [sym_system_number] = ACTIONS(2121),
    [sym_button] = ACTIONS(2121),
    [sym_dv_channel] = ACTIONS(2121),
    [sym_data] = ACTIONS(2121),
    [sym_level] = ACTIONS(2121),
    [sym_timeline] = ACTIONS(2121),
    [sym_channel] = ACTIONS(2121),
    [sym_custom] = ACTIONS(2121),
    [sym_atoi] = ACTIONS(2121),
    [sym_atol] = ACTIONS(2121),
    [sym_atof] = ACTIONS(2121),
    [sym_itoa] = ACTIONS(2121),
    [sym_itohex] = ACTIONS(2121),
    [sym_hextoi] = ACTIONS(2121),
    [sym_ftoa] = ACTIONS(2121),
    [sym_find_string] = ACTIONS(2121),
    [sym_left_string] = ACTIONS(2121),
    [sym_right_string] = ACTIONS(2121),
    [sym_mid_string] = ACTIONS(2121),
    [sym_remove_string] = ACTIONS(2121),
    [sym_get_buffer_string] = ACTIONS(2121),
    [sym_get_buffer_char] = ACTIONS(2121),
    [sym_get_multi_buffer_string] = ACTIONS(2121),
    [sym_length_string] = ACTIONS(2121),
    [sym_max_length_string] = ACTIONS(2121),
    [sym_set_length_string] = ACTIONS(2121),
    [sym_compare_string] = ACTIONS(2121),
    [sym_upper_string] = ACTIONS(2121),
    [sym_lower_string] = ACTIONS(2121),
    [sym_get_last] = ACTIONS(2121),
    [sym_date_to_day] = ACTIONS(2121),
    [sym_date_to_month] = ACTIONS(2121),
    [sym_date_to_year] = ACTIONS(2121),
    [sym_time_to_hour] = ACTIONS(2121),
    [sym_time_to_minute] = ACTIONS(2121),
    [sym_time_to_second] = ACTIONS(2121),
    [sym_day_of_week] = ACTIONS(2121),
    [sym_set_pulse_time] = ACTIONS(2121),
    [sym_set_timer] = ACTIONS(2121),
    [sym_file_open] = ACTIONS(2121),
    [sym_file_close] = ACTIONS(2121),
    [sym_file_read] = ACTIONS(2121),
    [sym_file_read_line] = ACTIONS(2121),
    [sym_file_write] = ACTIONS(2121),
    [sym_file_write_line] = ACTIONS(2121),
    [sym_file_copy] = ACTIONS(2121),
    [sym_file_delete] = ACTIONS(2121),
    [sym_file_rename] = ACTIONS(2121),
    [sym_file_dir] = ACTIONS(2121),
    [sym_file_getdir] = ACTIONS(2121),
    [sym_file_setdir] = ACTIONS(2121),
    [sym_file_create_dir] = ACTIONS(2121),
    [sym_file_remove_dir] = ACTIONS(2121),
    [sym_file_seek] = ACTIONS(2121),
    [sym_length_array] = ACTIONS(2121),
    [sym_max_length_array] = ACTIONS(2121),
    [sym_set_length_array] = ACTIONS(2121),
    [sym_ip_client_open] = ACTIONS(2121),
    [sym_ip_client_close] = ACTIONS(2121),
    [sym_ip_server_open] = ACTIONS(2121),
    [sym_ip_server_close] = ACTIONS(2121),
    [sym_ip_bound_client_open] = ACTIONS(2121),
    [sym_ip_mc_server_open] = ACTIONS(2121),
    [sym_ip_set_option] = ACTIONS(2121),
    [sym_ssh_client_open] = ACTIONS(2121),
    [sym_ssh_client_close] = ACTIONS(2121),
    [sym_tls_client_open] = ACTIONS(2121),
    [sym_tls_client_close] = ACTIONS(2121),
    [sym_do_push] = ACTIONS(2121),
    [sym_do_push_timed] = ACTIONS(2121),
    [sym_do_release] = ACTIONS(2121),
    [sym_device_id] = ACTIONS(2121),
    [sym_device_info] = ACTIONS(2121),
    [sym_device_id_string] = ACTIONS(2121),
    [sym_device_standby] = ACTIONS(2121),
    [sym_device_wake] = ACTIONS(2121),
    [sym_timeline_create] = ACTIONS(2121),
    [sym_timeline_kill] = ACTIONS(2121),
    [sym_timeline_reload] = ACTIONS(2121),
    [sym_timeline_pause] = ACTIONS(2121),
    [sym_timeline_restart] = ACTIONS(2121),
    [sym_timeline_set] = ACTIONS(2121),
    [sym_timeline_get] = ACTIONS(2121),
    [sym_timeline_active] = ACTIONS(2121),
    [sym_timeline_dynamic_id] = ACTIONS(2121),
    [sym_variable_to_string] = ACTIONS(2121),
    [sym_string_to_variable] = ACTIONS(2121),
    [sym_length_variable_to_string] = ACTIONS(2121),
    [sym_type_cast] = ACTIONS(2121),
    [sym_variable_to_xml] = ACTIONS(2121),
    [sym_xml_to_variable] = ACTIONS(2121),
    [sym_length_variable_to_xml] = ACTIONS(2121),
    [sym_redirect_string] = ACTIONS(2121),
    [sym_random_number] = ACTIONS(2121),
    [sym_get_unique_id] = ACTIONS(2121),
    [sym_get_system_number] = ACTIONS(2121),
    [sym_set_system_number] = ACTIONS(2121),
    [sym_get_serial_number] = ACTIONS(2121),
    [sym_get_master_build] = ACTIONS(2121),
    [sym_set_validation_code] = ACTIONS(2121),
    [sym_reboot] = ACTIONS(2121),
    [sym_load_duet_module] = ACTIONS(2121),
    [sym_unload_duet_module] = ACTIONS(2121),
    [sym_duet_mem_size_set] = ACTIONS(2121),
    [sym_duet_mem_size_get] = ACTIONS(2121),
    [sym_dynamic_application_device] = ACTIONS(2121),
    [sym_dynamic_polled_port] = ACTIONS(2121),
    [sym_static_port_binding] = ACTIONS(2121),
    [sym_static_ip_binding] = ACTIONS(2121),
    [sym_get_dns_list] = ACTIONS(2121),
    [sym_set_dns_list] = ACTIONS(2121),
    [sym_get_ip_address] = ACTIONS(2121),
    [sym_set_ip_address] = ACTIONS(2121),
    [sym_get_url_list] = ACTIONS(2121),
    [sym_add_url_entry] = ACTIONS(2121),
    [sym_delete_url_entry] = ACTIONS(2121),
    [sym_abs_value] = ACTIONS(2121),
    [sym_max_value] = ACTIONS(2121),
    [sym_min_value] = ACTIONS(2121),
    [sym_exp_value] = ACTIONS(2121),
    [sym_log_value] = ACTIONS(2121),
    [sym_log10_value] = ACTIONS(2121),
    [sym_power_value] = ACTIONS(2121),
    [sym_sqrt_value] = ACTIONS(2121),
    [sym_raw_be] = ACTIONS(2121),
    [sym_raw_le] = ACTIONS(2121),
    [sym_internal_queue_size_set] = ACTIONS(2121),
    [sym_internal_queue_size_get] = ACTIONS(2121),
    [sym_internal_threshold_set] = ACTIONS(2121),
    [sym_internal_threshold_get] = ACTIONS(2121),
    [sym_rebuilt_event] = ACTIONS(2121),
    [sym_do_custom_event] = ACTIONS(2121),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2121),
    [sym_clkmgr_set_clk_source] = ACTIONS(2121),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2121),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2121),
    [sym_clkmgr_get_timezone] = ACTIONS(2121),
    [sym_clkmgr_set_timezone] = ACTIONS(2121),
    [sym_clkmgr_get_resync_period] = ACTIONS(2121),
    [sym_clkmgr_set_resync_period] = ACTIONS(2121),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2121),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2121),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2121),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2121),
    [sym_clkmgr_get_timeservers] = ACTIONS(2121),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2121),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2121),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2121),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2121),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2121),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2121),
    [sym_set_log_level] = ACTIONS(2121),
    [sym_get_log_level] = ACTIONS(2121),
    [sym_amx_log] = ACTIONS(2121),
    [sym_smtp_server_config_set] = ACTIONS(2121),
    [sym_smtp_server_config_get] = ACTIONS(2121),
    [sym_smtp_send] = ACTIONS(2121),
    [sym_validate_netlinx_account] = ACTIONS(2121),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2121),
    [sym_authenticate_certificate] = ACTIONS(2121),
    [sym_audit_netlinx_session_event] = ACTIONS(2121),
    [sym_audit_netlinx_generic_event] = ACTIONS(2121),
    [sym_get_available_flash_disk_space] = ACTIONS(2121),
    [sym_get_max_flash_disk_space] = ACTIONS(2121),
    [sym_data_create_feed] = ACTIONS(2121),
    [sym_data_delete_feed] = ACTIONS(2121),
    [sym_data_publish_feed] = ACTIONS(2121),
    [sym_data_get_published_feed] = ACTIONS(2121),
    [sym_data_add_record] = ACTIONS(2121),
    [sym_data_get_event_record] = ACTIONS(2121),
    [sym__wc_data_create_feed] = ACTIONS(2121),
    [sym__wc_data_add_record] = ACTIONS(2121),
    [sym__wc_data_get_event_record] = ACTIONS(2121),
    [sym_format] = ACTIONS(2121),
    [sym_astro_clock] = ACTIONS(2121),
    [sym_set_outdoor_temperature] = ACTIONS(2121),
    [sym_set_virtual_level_count] = ACTIONS(2121),
    [sym_set_virtual_channel_count] = ACTIONS(2121),
    [sym_set_virtual_port_count] = ACTIONS(2121),
    [sym__wc_encode_int] = ACTIONS(2121),
    [sym__wc_decode_int] = ACTIONS(2121),
    [sym__wc_int] = ACTIONS(2121),
    [sym__wc_to_ch_int] = ACTIONS(2121),
    [sym__ch_to_wc_int] = ACTIONS(2121),
    [sym__wc_find_string_int] = ACTIONS(2121),
    [sym__wc_left_string_int] = ACTIONS(2121),
    [sym__wc_lower_string_int] = ACTIONS(2121),
    [sym__wc_mid_string_int] = ACTIONS(2121),
    [sym__wc_remove_string_int] = ACTIONS(2121),
    [sym__wc_right_string_int] = ACTIONS(2121),
    [sym__wc_upper_string_int] = ACTIONS(2121),
    [sym__wc_compare_string_int] = ACTIONS(2121),
    [sym__wc_get_buffer_char_int] = ACTIONS(2121),
    [sym__wc_get_buffer_string_int] = ACTIONS(2121),
    [sym__wc_concat_string_int] = ACTIONS(2121),
    [sym__wc_file_open_int] = ACTIONS(2121),
    [sym__wc_file_close_int] = ACTIONS(2121),
    [sym__wc_file_read_int] = ACTIONS(2121),
    [sym__wc_file_read_line_int] = ACTIONS(2121),
    [sym__wc_file_write_int] = ACTIONS(2121),
    [sym__wc_file_write_line_int] = ACTIONS(2121),
    [aux_sym_preproc_include_token1] = ACTIONS(2121),
    [aux_sym_preproc_define_token1] = ACTIONS(2123),
    [aux_sym_preproc_warn_token1] = ACTIONS(2123),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2123),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2123),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2123),
    [anon_sym_LBRACK] = ACTIONS(2123),
    [anon_sym_LBRACE] = ACTIONS(2123),
    [anon_sym_LPAREN] = ACTIONS(2121),
    [anon_sym_BANG] = ACTIONS(2123),
    [anon_sym_TILDE] = ACTIONS(2123),
    [anon_sym_DASH] = ACTIONS(2121),
    [anon_sym_PLUS] = ACTIONS(2121),
    [anon_sym_DASH_DASH] = ACTIONS(2123),
    [anon_sym_PLUS_PLUS] = ACTIONS(2123),
    [anon_sym_DQUOTE] = ACTIONS(2123),
    [anon_sym_SQUOTE] = ACTIONS(2123),
    [sym_number_literal] = ACTIONS(2123),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(417)] = {
    [ts_builtin_sym_end] = ACTIONS(2127),
    [sym_identifier] = ACTIONS(2125),
    [sym_program_name_keyword] = ACTIONS(2125),
    [sym_module_name_keyword] = ACTIONS(2125),
    [sym_define_device_keyword] = ACTIONS(2125),
    [sym_define_combine_keyword] = ACTIONS(2125),
    [sym_define_constant_keyword] = ACTIONS(2125),
    [sym_define_type_keyword] = ACTIONS(2125),
    [sym_define_variable_keyword] = ACTIONS(2125),
    [sym_define_system_variable_keyword] = ACTIONS(2125),
    [sym_define_connect_level_keyword] = ACTIONS(2125),
    [sym_define_latching_keyword] = ACTIONS(2125),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2125),
    [sym_define_toggling_keyword] = ACTIONS(2125),
    [sym_define_call_keyword] = ACTIONS(2125),
    [sym_define_function_keyword] = ACTIONS(2125),
    [sym_define_library_function_keyword] = ACTIONS(2125),
    [sym_define_start_keyword] = ACTIONS(2125),
    [sym_define_event_keyword] = ACTIONS(2125),
    [sym_define_program_keyword] = ACTIONS(2125),
    [sym_define_module_keyword] = ACTIONS(2125),
    [sym_char_keyword] = ACTIONS(2125),
    [sym_widechar_keyword] = ACTIONS(2125),
    [sym_integer_keyword] = ACTIONS(2125),
    [sym_sinteger_keyword] = ACTIONS(2125),
    [sym_long_keyword] = ACTIONS(2125),
    [sym_slong_keyword] = ACTIONS(2125),
    [sym_float_keyword] = ACTIONS(2125),
    [sym_double_keyword] = ACTIONS(2125),
    [sym_variant_keyword] = ACTIONS(2125),
    [sym_variantarray_keyword] = ACTIONS(2125),
    [sym_local_var_keyword] = ACTIONS(2125),
    [sym_stack_var_keyword] = ACTIONS(2125),
    [sym_constant_keyword] = ACTIONS(2125),
    [sym_volatile_keyword] = ACTIONS(2125),
    [sym_non_volatile_keyword] = ACTIONS(2125),
    [sym_persistent_keyword] = ACTIONS(2125),
    [sym_dev_keyword] = ACTIONS(2125),
    [sym_devlev_keyword] = ACTIONS(2125),
    [sym_devchan_keyword] = ACTIONS(2125),
    [sym_if_keyword] = ACTIONS(2125),
    [sym_switch_keyword] = ACTIONS(2125),
    [sym_case_keyword] = ACTIONS(2125),
    [sym_default_keyword] = ACTIONS(2125),
    [sym_while_keyword] = ACTIONS(2125),
    [sym_for_keyword] = ACTIONS(2125),
    [sym_break_keyword] = ACTIONS(2125),
    [sym_continue_keyword] = ACTIONS(2125),
    [sym_return_keyword] = ACTIONS(2125),
    [sym_select_keyword] = ACTIONS(2125),
    [sym_active_keyword] = ACTIONS(2125),
    [sym_struct_keyword] = ACTIONS(2125),
    [sym_structure_keyword] = ACTIONS(2125),
    [sym_button_event_keyword] = ACTIONS(2125),
    [sym_channel_event_keyword] = ACTIONS(2125),
    [sym_level_event_keyword] = ACTIONS(2125),
    [sym_data_event_keyword] = ACTIONS(2125),
    [sym_timeline_event_keyword] = ACTIONS(2125),
    [sym_custom_event_keyword] = ACTIONS(2125),
    [sym_push_keyword] = ACTIONS(2125),
    [sym_release_keyword] = ACTIONS(2125),
    [aux_sym_on_keyword_token1] = ACTIONS(2125),
    [aux_sym_off_keyword_token1] = ACTIONS(2125),
    [sym_send_level_keyword] = ACTIONS(2125),
    [sym_send_string_keyword] = ACTIONS(2125),
    [sym_send_command_keyword] = ACTIONS(2125),
    [sym_clear_buffer_keyword] = ACTIONS(2125),
    [sym_create_buffer_keyword] = ACTIONS(2125),
    [sym_create_multi_buffer_keyword] = ACTIONS(2125),
    [sym_call_keyword] = ACTIONS(2125),
    [sym_system_call_keyword] = ACTIONS(2125),
    [sym_devchan_to_keyword] = ACTIONS(2125),
    [sym_devchan_min_to_keyword] = ACTIONS(2125),
    [sym_devchan_total_off_keyword] = ACTIONS(2125),
    [sym_devchan_pulse_keyword] = ACTIONS(2125),
    [sym_wait_keyword] = ACTIONS(2125),
    [sym_cancel_wait_keyword] = ACTIONS(2125),
    [sym_wait_until_keyword] = ACTIONS(2125),
    [sym_cancel_wait_until_keyword] = ACTIONS(2125),
    [sym_cancel_all_wait_keyword] = ACTIONS(2125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2125),
    [sym_not] = ACTIONS(2125),
    [sym_bnot] = ACTIONS(2125),
    [sym___file__] = ACTIONS(2125),
    [sym___date__] = ACTIONS(2125),
    [sym___ldate__] = ACTIONS(2125),
    [sym___time__] = ACTIONS(2125),
    [sym___line__] = ACTIONS(2125),
    [sym___name__] = ACTIONS(2125),
    [sym___netlinx__] = ACTIONS(2125),
    [sym_netlinx_axi_version] = ACTIONS(2125),
    [sym_url_flg_tcp] = ACTIONS(2125),
    [sym_url_flg_accinfopresent] = ACTIONS(2125),
    [sym_url_flg_temp] = ACTIONS(2125),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2125),
    [sym_url_flg_stat_mask] = ACTIONS(2125),
    [sym_url_flg_stat_lookup] = ACTIONS(2125),
    [sym_url_flg_stat_connecting] = ACTIONS(2125),
    [sym_url_flg_stat_waiting] = ACTIONS(2125),
    [sym_url_flg_stat_connected] = ACTIONS(2125),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2125),
    [sym_first_virtual_device] = ACTIONS(2125),
    [sym_first_local_port] = ACTIONS(2125),
    [sym_dynamic_virtual_device] = ACTIONS(2125),
    [sym_timeline_once] = ACTIONS(2125),
    [sym_timeline_repeat] = ACTIONS(2125),
    [sym_timeline_absolute] = ACTIONS(2125),
    [sym_timeline_relative] = ACTIONS(2125),
    [sym_do_push_timed_infinite] = ACTIONS(2125),
    [sym_source_type_no_address] = ACTIONS(2125),
    [sym_source_type_neuron_id] = ACTIONS(2125),
    [sym_source_type_ip_address] = ACTIONS(2125),
    [sym_source_type_axlink] = ACTIONS(2125),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2125),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2125),
    [sym_source_type_ip_socket_address] = ACTIONS(2125),
    [sym_source_type_rs232] = ACTIONS(2125),
    [sym_source_type_internal] = ACTIONS(2125),
    [sym_source_type_mac_address] = ACTIONS(2125),
    [sym_source_type_ipv4_port] = ACTIONS(2125),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2125),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2125),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2125),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2125),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2125),
    [sym_true] = ACTIONS(2125),
    [sym_false] = ACTIONS(2125),
    [sym_file_read_only] = ACTIONS(2125),
    [sym_file_rw_new] = ACTIONS(2125),
    [sym_file_rw_append] = ACTIONS(2125),
    [sym_ip_tcp] = ACTIONS(2125),
    [sym_ip_udp] = ACTIONS(2125),
    [sym_ip_udp_2way] = ACTIONS(2125),
    [sym_xml_encode_types] = ACTIONS(2125),
    [sym_xml_encode_char_as_list] = ACTIONS(2125),
    [sym_xml_encode_le] = ACTIONS(2125),
    [sym_xml_decode_types] = ACTIONS(2125),
    [sym_xml_decode_no_preserve] = ACTIONS(2125),
    [sym_duet_dev_type_display_device] = ACTIONS(2125),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2125),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2125),
    [sym_duet_dev_type_io_device] = ACTIONS(2125),
    [sym_duet_dev_type_relay_device] = ACTIONS(2125),
    [sym_duet_dev_type_ups] = ACTIONS(2125),
    [sym_duet_dev_type_amplifier] = ACTIONS(2125),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2125),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2125),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2125),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2125),
    [sym_duet_dev_type_camera] = ACTIONS(2125),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2125),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2125),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2125),
    [sym_duet_dev_type_dss] = ACTIONS(2125),
    [sym_duet_dev_type_dvr] = ACTIONS(2125),
    [sym_duet_dev_type_disc_device] = ACTIONS(2125),
    [sym_duet_dev_type_document_camera] = ACTIONS(2125),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2125),
    [sym_duet_dev_type_hvac] = ACTIONS(2125),
    [sym_duet_dev_type_keypad] = ACTIONS(2125),
    [sym_duet_dev_type_light] = ACTIONS(2125),
    [sym_duet_dev_type_monitor] = ACTIONS(2125),
    [sym_duet_dev_type_motor] = ACTIONS(2125),
    [sym_duet_dev_type_multi_window] = ACTIONS(2125),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2125),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2125),
    [sym_duet_dev_type_receiver] = ACTIONS(2125),
    [sym_duet_dev_type_security_system] = ACTIONS(2125),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2125),
    [sym_duet_dev_type_settop_box] = ACTIONS(2125),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2125),
    [sym_duet_dev_type_switcher] = ACTIONS(2125),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2125),
    [sym_duet_dev_type_tv] = ACTIONS(2125),
    [sym_duet_dev_type_utility] = ACTIONS(2125),
    [sym_duet_dev_type_vcr] = ACTIONS(2125),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2125),
    [sym_duet_dev_type_video_processor] = ACTIONS(2125),
    [sym_duet_dev_type_video_projector] = ACTIONS(2125),
    [sym_duet_dev_type_video_wall] = ACTIONS(2125),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2125),
    [sym_duet_dev_type_weather] = ACTIONS(2125),
    [sym_duet_dev_not_polled] = ACTIONS(2125),
    [sym_duet_dev_polled] = ACTIONS(2125),
    [sym_clkmgr_mode_network] = ACTIONS(2125),
    [sym_clkmgr_mode_standalone] = ACTIONS(2125),
    [sym_normal_standby] = ACTIONS(2125),
    [sym_normal_wake] = ACTIONS(2125),
    [sym_ip_multicast_ttl_option] = ACTIONS(2125),
    [sym_ip_ttl_subnet] = ACTIONS(2125),
    [sym_ip_ttl_site] = ACTIONS(2125),
    [sym_ip_ttl_region] = ACTIONS(2125),
    [sym_ip_ttl_continent] = ACTIONS(2125),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2125),
    [sym_ip_nodelay_on] = ACTIONS(2125),
    [sym_ip_nodelay_off] = ACTIONS(2125),
    [sym_amx_error] = ACTIONS(2125),
    [sym_amx_warning] = ACTIONS(2125),
    [sym_amx_info] = ACTIONS(2125),
    [sym_amx_debug] = ACTIONS(2125),
    [sym_netlinx_login_success] = ACTIONS(2125),
    [sym_netlinx_login_fail] = ACTIONS(2125),
    [sym_netlinx_logout] = ACTIONS(2125),
    [sym_null_str] = ACTIONS(2125),
    [sym_smtp_address] = ACTIONS(2125),
    [sym_smtp_port_number] = ACTIONS(2125),
    [sym_smtp_username] = ACTIONS(2125),
    [sym_smtp_password] = ACTIONS(2125),
    [sym_smtp_require_tls] = ACTIONS(2125),
    [sym_smtp_from] = ACTIONS(2125),
    [sym_smtp_tls_true] = ACTIONS(2125),
    [sym_smtp_tls_false] = ACTIONS(2125),
    [sym_listview_on_row_select_event] = ACTIONS(2125),
    [sym_data_structure_unknown] = ACTIONS(2125),
    [sym_data_structure_datafeed] = ACTIONS(2125),
    [sym_data_structure_datarecordset] = ACTIONS(2125),
    [sym_data_structure_datarecord] = ACTIONS(2125),
    [sym_data_structure_datafield] = ACTIONS(2125),
    [sym_data_max_content_field_count] = ACTIONS(2125),
    [sym_data_max_metadata_field_count] = ACTIONS(2125),
    [sym_data_max_id_length] = ACTIONS(2125),
    [sym_data_max_name_length] = ACTIONS(2125),
    [sym_data_max_description_length] = ACTIONS(2125),
    [sym_data_max_type_length] = ACTIONS(2125),
    [sym_data_max_format_length] = ACTIONS(2125),
    [sym_data_max_source_length] = ACTIONS(2125),
    [sym_data_max_value_length] = ACTIONS(2125),
    [sym_data_max_label_length] = ACTIONS(2125),
    [sym_data_max_url_length] = ACTIONS(2125),
    [sym_max_last_login_info_length] = ACTIONS(2125),
    [sym_data_type_unknown] = ACTIONS(2125),
    [sym_data_type_string] = ACTIONS(2125),
    [sym_data_type_datetime] = ACTIONS(2125),
    [sym_data_type_date] = ACTIONS(2125),
    [sym_data_type_time] = ACTIONS(2125),
    [sym_data_type_image] = ACTIONS(2125),
    [sym_data_format_url] = ACTIONS(2125),
    [sym_data_format_phone] = ACTIONS(2125),
    [sym_data_format_email] = ACTIONS(2125),
    [sym_data_format_iso8601] = ACTIONS(2125),
    [sym_auditlog_priv] = ACTIONS(2125),
    [sym_configuration_priv] = ACTIONS(2125),
    [sym_deviceconfig_priv] = ACTIONS(2125),
    [sym_ftp_priv] = ACTIONS(2125),
    [sym_http_priv] = ACTIONS(2125),
    [sym_networkconfig_priv] = ACTIONS(2125),
    [sym_programport_priv] = ACTIONS(2125),
    [sym_remoteui_priv] = ACTIONS(2125),
    [sym_securitycontrol_priv] = ACTIONS(2125),
    [sym_softwaremanagement_priv] = ACTIONS(2125),
    [sym_terminal_priv] = ACTIONS(2125),
    [sym_tpadmin_priv] = ACTIONS(2125),
    [sym_user1_priv] = ACTIONS(2125),
    [sym_user2_priv] = ACTIONS(2125),
    [sym_user3_priv] = ACTIONS(2125),
    [sym_user4_priv] = ACTIONS(2125),
    [sym_usermanagement_priv] = ACTIONS(2125),
    [sym_valid_account] = ACTIONS(2125),
    [sym_err_name_invalid] = ACTIONS(2125),
    [sym_err_password_invalid] = ACTIONS(2125),
    [sym_err_invalid_account] = ACTIONS(2125),
    [sym_err_no_authorization] = ACTIONS(2125),
    [sym_err_invalid_parameter] = ACTIONS(2125),
    [sym_err_user_locked_out] = ACTIONS(2125),
    [sym_err_user_account_expired] = ACTIONS(2125),
    [sym_err_general_error] = ACTIONS(2125),
    [sym_err_authentication_error] = ACTIONS(2125),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2125),
    [sym_tls_validate_certificate] = ACTIONS(2125),
    [sym_tdata] = ACTIONS(2125),
    [sym_tchannel] = ACTIONS(2125),
    [sym_tlevel] = ACTIONS(2125),
    [sym_tbutton] = ACTIONS(2125),
    [sym_ttimeline] = ACTIONS(2125),
    [sym_tcustom] = ACTIONS(2125),
    [sym_url_struct] = ACTIONS(2125),
    [sym_dns_struct] = ACTIONS(2125),
    [sym_ip_address_struct] = ACTIONS(2125),
    [sym_dev_info_struct] = ACTIONS(2125),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2125),
    [sym_data_feed] = ACTIONS(2125),
    [sym_data_field] = ACTIONS(2125),
    [sym_data_record] = ACTIONS(2125),
    [sym_wc_data_feed] = ACTIONS(2125),
    [sym_wc_data_field] = ACTIONS(2125),
    [sym_wc_data_record] = ACTIONS(2125),
    [sym_last_login_info] = ACTIONS(2125),
    [sym____reserved___] = ACTIONS(2125),
    [sym_date] = ACTIONS(2125),
    [sym_day] = ACTIONS(2125),
    [sym_time] = ACTIONS(2125),
    [sym_ldate] = ACTIONS(2125),
    [sym_push_device] = ACTIONS(2125),
    [sym_push_channel] = ACTIONS(2125),
    [sym_push_devchan] = ACTIONS(2125),
    [sym_release_device] = ACTIONS(2125),
    [sym_release_channel] = ACTIONS(2125),
    [sym_release_devchan] = ACTIONS(2125),
    [sym_master_slot] = ACTIONS(2125),
    [sym_get_pulse_time] = ACTIONS(2125),
    [sym_get_timer] = ACTIONS(2125),
    [sym_master_sn] = ACTIONS(2125),
    [sym_system_number] = ACTIONS(2125),
    [sym_button] = ACTIONS(2125),
    [sym_dv_channel] = ACTIONS(2125),
    [sym_data] = ACTIONS(2125),
    [sym_level] = ACTIONS(2125),
    [sym_timeline] = ACTIONS(2125),
    [sym_channel] = ACTIONS(2125),
    [sym_custom] = ACTIONS(2125),
    [sym_atoi] = ACTIONS(2125),
    [sym_atol] = ACTIONS(2125),
    [sym_atof] = ACTIONS(2125),
    [sym_itoa] = ACTIONS(2125),
    [sym_itohex] = ACTIONS(2125),
    [sym_hextoi] = ACTIONS(2125),
    [sym_ftoa] = ACTIONS(2125),
    [sym_find_string] = ACTIONS(2125),
    [sym_left_string] = ACTIONS(2125),
    [sym_right_string] = ACTIONS(2125),
    [sym_mid_string] = ACTIONS(2125),
    [sym_remove_string] = ACTIONS(2125),
    [sym_get_buffer_string] = ACTIONS(2125),
    [sym_get_buffer_char] = ACTIONS(2125),
    [sym_get_multi_buffer_string] = ACTIONS(2125),
    [sym_length_string] = ACTIONS(2125),
    [sym_max_length_string] = ACTIONS(2125),
    [sym_set_length_string] = ACTIONS(2125),
    [sym_compare_string] = ACTIONS(2125),
    [sym_upper_string] = ACTIONS(2125),
    [sym_lower_string] = ACTIONS(2125),
    [sym_get_last] = ACTIONS(2125),
    [sym_date_to_day] = ACTIONS(2125),
    [sym_date_to_month] = ACTIONS(2125),
    [sym_date_to_year] = ACTIONS(2125),
    [sym_time_to_hour] = ACTIONS(2125),
    [sym_time_to_minute] = ACTIONS(2125),
    [sym_time_to_second] = ACTIONS(2125),
    [sym_day_of_week] = ACTIONS(2125),
    [sym_set_pulse_time] = ACTIONS(2125),
    [sym_set_timer] = ACTIONS(2125),
    [sym_file_open] = ACTIONS(2125),
    [sym_file_close] = ACTIONS(2125),
    [sym_file_read] = ACTIONS(2125),
    [sym_file_read_line] = ACTIONS(2125),
    [sym_file_write] = ACTIONS(2125),
    [sym_file_write_line] = ACTIONS(2125),
    [sym_file_copy] = ACTIONS(2125),
    [sym_file_delete] = ACTIONS(2125),
    [sym_file_rename] = ACTIONS(2125),
    [sym_file_dir] = ACTIONS(2125),
    [sym_file_getdir] = ACTIONS(2125),
    [sym_file_setdir] = ACTIONS(2125),
    [sym_file_create_dir] = ACTIONS(2125),
    [sym_file_remove_dir] = ACTIONS(2125),
    [sym_file_seek] = ACTIONS(2125),
    [sym_length_array] = ACTIONS(2125),
    [sym_max_length_array] = ACTIONS(2125),
    [sym_set_length_array] = ACTIONS(2125),
    [sym_ip_client_open] = ACTIONS(2125),
    [sym_ip_client_close] = ACTIONS(2125),
    [sym_ip_server_open] = ACTIONS(2125),
    [sym_ip_server_close] = ACTIONS(2125),
    [sym_ip_bound_client_open] = ACTIONS(2125),
    [sym_ip_mc_server_open] = ACTIONS(2125),
    [sym_ip_set_option] = ACTIONS(2125),
    [sym_ssh_client_open] = ACTIONS(2125),
    [sym_ssh_client_close] = ACTIONS(2125),
    [sym_tls_client_open] = ACTIONS(2125),
    [sym_tls_client_close] = ACTIONS(2125),
    [sym_do_push] = ACTIONS(2125),
    [sym_do_push_timed] = ACTIONS(2125),
    [sym_do_release] = ACTIONS(2125),
    [sym_device_id] = ACTIONS(2125),
    [sym_device_info] = ACTIONS(2125),
    [sym_device_id_string] = ACTIONS(2125),
    [sym_device_standby] = ACTIONS(2125),
    [sym_device_wake] = ACTIONS(2125),
    [sym_timeline_create] = ACTIONS(2125),
    [sym_timeline_kill] = ACTIONS(2125),
    [sym_timeline_reload] = ACTIONS(2125),
    [sym_timeline_pause] = ACTIONS(2125),
    [sym_timeline_restart] = ACTIONS(2125),
    [sym_timeline_set] = ACTIONS(2125),
    [sym_timeline_get] = ACTIONS(2125),
    [sym_timeline_active] = ACTIONS(2125),
    [sym_timeline_dynamic_id] = ACTIONS(2125),
    [sym_variable_to_string] = ACTIONS(2125),
    [sym_string_to_variable] = ACTIONS(2125),
    [sym_length_variable_to_string] = ACTIONS(2125),
    [sym_type_cast] = ACTIONS(2125),
    [sym_variable_to_xml] = ACTIONS(2125),
    [sym_xml_to_variable] = ACTIONS(2125),
    [sym_length_variable_to_xml] = ACTIONS(2125),
    [sym_redirect_string] = ACTIONS(2125),
    [sym_random_number] = ACTIONS(2125),
    [sym_get_unique_id] = ACTIONS(2125),
    [sym_get_system_number] = ACTIONS(2125),
    [sym_set_system_number] = ACTIONS(2125),
    [sym_get_serial_number] = ACTIONS(2125),
    [sym_get_master_build] = ACTIONS(2125),
    [sym_set_validation_code] = ACTIONS(2125),
    [sym_reboot] = ACTIONS(2125),
    [sym_load_duet_module] = ACTIONS(2125),
    [sym_unload_duet_module] = ACTIONS(2125),
    [sym_duet_mem_size_set] = ACTIONS(2125),
    [sym_duet_mem_size_get] = ACTIONS(2125),
    [sym_dynamic_application_device] = ACTIONS(2125),
    [sym_dynamic_polled_port] = ACTIONS(2125),
    [sym_static_port_binding] = ACTIONS(2125),
    [sym_static_ip_binding] = ACTIONS(2125),
    [sym_get_dns_list] = ACTIONS(2125),
    [sym_set_dns_list] = ACTIONS(2125),
    [sym_get_ip_address] = ACTIONS(2125),
    [sym_set_ip_address] = ACTIONS(2125),
    [sym_get_url_list] = ACTIONS(2125),
    [sym_add_url_entry] = ACTIONS(2125),
    [sym_delete_url_entry] = ACTIONS(2125),
    [sym_abs_value] = ACTIONS(2125),
    [sym_max_value] = ACTIONS(2125),
    [sym_min_value] = ACTIONS(2125),
    [sym_exp_value] = ACTIONS(2125),
    [sym_log_value] = ACTIONS(2125),
    [sym_log10_value] = ACTIONS(2125),
    [sym_power_value] = ACTIONS(2125),
    [sym_sqrt_value] = ACTIONS(2125),
    [sym_raw_be] = ACTIONS(2125),
    [sym_raw_le] = ACTIONS(2125),
    [sym_internal_queue_size_set] = ACTIONS(2125),
    [sym_internal_queue_size_get] = ACTIONS(2125),
    [sym_internal_threshold_set] = ACTIONS(2125),
    [sym_internal_threshold_get] = ACTIONS(2125),
    [sym_rebuilt_event] = ACTIONS(2125),
    [sym_do_custom_event] = ACTIONS(2125),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2125),
    [sym_clkmgr_set_clk_source] = ACTIONS(2125),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2125),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2125),
    [sym_clkmgr_get_timezone] = ACTIONS(2125),
    [sym_clkmgr_set_timezone] = ACTIONS(2125),
    [sym_clkmgr_get_resync_period] = ACTIONS(2125),
    [sym_clkmgr_set_resync_period] = ACTIONS(2125),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2125),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2125),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2125),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2125),
    [sym_clkmgr_get_timeservers] = ACTIONS(2125),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2125),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2125),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2125),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2125),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2125),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2125),
    [sym_set_log_level] = ACTIONS(2125),
    [sym_get_log_level] = ACTIONS(2125),
    [sym_amx_log] = ACTIONS(2125),
    [sym_smtp_server_config_set] = ACTIONS(2125),
    [sym_smtp_server_config_get] = ACTIONS(2125),
    [sym_smtp_send] = ACTIONS(2125),
    [sym_validate_netlinx_account] = ACTIONS(2125),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2125),
    [sym_authenticate_certificate] = ACTIONS(2125),
    [sym_audit_netlinx_session_event] = ACTIONS(2125),
    [sym_audit_netlinx_generic_event] = ACTIONS(2125),
    [sym_get_available_flash_disk_space] = ACTIONS(2125),
    [sym_get_max_flash_disk_space] = ACTIONS(2125),
    [sym_data_create_feed] = ACTIONS(2125),
    [sym_data_delete_feed] = ACTIONS(2125),
    [sym_data_publish_feed] = ACTIONS(2125),
    [sym_data_get_published_feed] = ACTIONS(2125),
    [sym_data_add_record] = ACTIONS(2125),
    [sym_data_get_event_record] = ACTIONS(2125),
    [sym__wc_data_create_feed] = ACTIONS(2125),
    [sym__wc_data_add_record] = ACTIONS(2125),
    [sym__wc_data_get_event_record] = ACTIONS(2125),
    [sym_format] = ACTIONS(2125),
    [sym_astro_clock] = ACTIONS(2125),
    [sym_set_outdoor_temperature] = ACTIONS(2125),
    [sym_set_virtual_level_count] = ACTIONS(2125),
    [sym_set_virtual_channel_count] = ACTIONS(2125),
    [sym_set_virtual_port_count] = ACTIONS(2125),
    [sym__wc_encode_int] = ACTIONS(2125),
    [sym__wc_decode_int] = ACTIONS(2125),
    [sym__wc_int] = ACTIONS(2125),
    [sym__wc_to_ch_int] = ACTIONS(2125),
    [sym__ch_to_wc_int] = ACTIONS(2125),
    [sym__wc_find_string_int] = ACTIONS(2125),
    [sym__wc_left_string_int] = ACTIONS(2125),
    [sym__wc_lower_string_int] = ACTIONS(2125),
    [sym__wc_mid_string_int] = ACTIONS(2125),
    [sym__wc_remove_string_int] = ACTIONS(2125),
    [sym__wc_right_string_int] = ACTIONS(2125),
    [sym__wc_upper_string_int] = ACTIONS(2125),
    [sym__wc_compare_string_int] = ACTIONS(2125),
    [sym__wc_get_buffer_char_int] = ACTIONS(2125),
    [sym__wc_get_buffer_string_int] = ACTIONS(2125),
    [sym__wc_concat_string_int] = ACTIONS(2125),
    [sym__wc_file_open_int] = ACTIONS(2125),
    [sym__wc_file_close_int] = ACTIONS(2125),
    [sym__wc_file_read_int] = ACTIONS(2125),
    [sym__wc_file_read_line_int] = ACTIONS(2125),
    [sym__wc_file_write_int] = ACTIONS(2125),
    [sym__wc_file_write_line_int] = ACTIONS(2125),
    [aux_sym_preproc_include_token1] = ACTIONS(2125),
    [aux_sym_preproc_define_token1] = ACTIONS(2127),
    [aux_sym_preproc_warn_token1] = ACTIONS(2127),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2127),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2127),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2127),
    [anon_sym_LBRACK] = ACTIONS(2127),
    [anon_sym_LBRACE] = ACTIONS(2127),
    [anon_sym_LPAREN] = ACTIONS(2125),
    [anon_sym_BANG] = ACTIONS(2127),
    [anon_sym_TILDE] = ACTIONS(2127),
    [anon_sym_DASH] = ACTIONS(2125),
    [anon_sym_PLUS] = ACTIONS(2125),
    [anon_sym_DASH_DASH] = ACTIONS(2127),
    [anon_sym_PLUS_PLUS] = ACTIONS(2127),
    [anon_sym_DQUOTE] = ACTIONS(2127),
    [anon_sym_SQUOTE] = ACTIONS(2127),
    [sym_number_literal] = ACTIONS(2127),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(418)] = {
    [ts_builtin_sym_end] = ACTIONS(2131),
    [sym_identifier] = ACTIONS(2129),
    [sym_program_name_keyword] = ACTIONS(2129),
    [sym_module_name_keyword] = ACTIONS(2129),
    [sym_define_device_keyword] = ACTIONS(2129),
    [sym_define_combine_keyword] = ACTIONS(2129),
    [sym_define_constant_keyword] = ACTIONS(2129),
    [sym_define_type_keyword] = ACTIONS(2129),
    [sym_define_variable_keyword] = ACTIONS(2129),
    [sym_define_system_variable_keyword] = ACTIONS(2129),
    [sym_define_connect_level_keyword] = ACTIONS(2129),
    [sym_define_latching_keyword] = ACTIONS(2129),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2129),
    [sym_define_toggling_keyword] = ACTIONS(2129),
    [sym_define_call_keyword] = ACTIONS(2129),
    [sym_define_function_keyword] = ACTIONS(2129),
    [sym_define_library_function_keyword] = ACTIONS(2129),
    [sym_define_start_keyword] = ACTIONS(2129),
    [sym_define_event_keyword] = ACTIONS(2129),
    [sym_define_program_keyword] = ACTIONS(2129),
    [sym_define_module_keyword] = ACTIONS(2129),
    [sym_char_keyword] = ACTIONS(2129),
    [sym_widechar_keyword] = ACTIONS(2129),
    [sym_integer_keyword] = ACTIONS(2129),
    [sym_sinteger_keyword] = ACTIONS(2129),
    [sym_long_keyword] = ACTIONS(2129),
    [sym_slong_keyword] = ACTIONS(2129),
    [sym_float_keyword] = ACTIONS(2129),
    [sym_double_keyword] = ACTIONS(2129),
    [sym_variant_keyword] = ACTIONS(2129),
    [sym_variantarray_keyword] = ACTIONS(2129),
    [sym_local_var_keyword] = ACTIONS(2129),
    [sym_stack_var_keyword] = ACTIONS(2129),
    [sym_constant_keyword] = ACTIONS(2129),
    [sym_volatile_keyword] = ACTIONS(2129),
    [sym_non_volatile_keyword] = ACTIONS(2129),
    [sym_persistent_keyword] = ACTIONS(2129),
    [sym_dev_keyword] = ACTIONS(2129),
    [sym_devlev_keyword] = ACTIONS(2129),
    [sym_devchan_keyword] = ACTIONS(2129),
    [sym_if_keyword] = ACTIONS(2129),
    [sym_switch_keyword] = ACTIONS(2129),
    [sym_case_keyword] = ACTIONS(2129),
    [sym_default_keyword] = ACTIONS(2129),
    [sym_while_keyword] = ACTIONS(2129),
    [sym_for_keyword] = ACTIONS(2129),
    [sym_break_keyword] = ACTIONS(2129),
    [sym_continue_keyword] = ACTIONS(2129),
    [sym_return_keyword] = ACTIONS(2129),
    [sym_select_keyword] = ACTIONS(2129),
    [sym_active_keyword] = ACTIONS(2129),
    [sym_struct_keyword] = ACTIONS(2129),
    [sym_structure_keyword] = ACTIONS(2129),
    [sym_button_event_keyword] = ACTIONS(2129),
    [sym_channel_event_keyword] = ACTIONS(2129),
    [sym_level_event_keyword] = ACTIONS(2129),
    [sym_data_event_keyword] = ACTIONS(2129),
    [sym_timeline_event_keyword] = ACTIONS(2129),
    [sym_custom_event_keyword] = ACTIONS(2129),
    [sym_push_keyword] = ACTIONS(2129),
    [sym_release_keyword] = ACTIONS(2129),
    [aux_sym_on_keyword_token1] = ACTIONS(2129),
    [aux_sym_off_keyword_token1] = ACTIONS(2129),
    [sym_send_level_keyword] = ACTIONS(2129),
    [sym_send_string_keyword] = ACTIONS(2129),
    [sym_send_command_keyword] = ACTIONS(2129),
    [sym_clear_buffer_keyword] = ACTIONS(2129),
    [sym_create_buffer_keyword] = ACTIONS(2129),
    [sym_create_multi_buffer_keyword] = ACTIONS(2129),
    [sym_call_keyword] = ACTIONS(2129),
    [sym_system_call_keyword] = ACTIONS(2129),
    [sym_devchan_to_keyword] = ACTIONS(2129),
    [sym_devchan_min_to_keyword] = ACTIONS(2129),
    [sym_devchan_total_off_keyword] = ACTIONS(2129),
    [sym_devchan_pulse_keyword] = ACTIONS(2129),
    [sym_wait_keyword] = ACTIONS(2129),
    [sym_cancel_wait_keyword] = ACTIONS(2129),
    [sym_wait_until_keyword] = ACTIONS(2129),
    [sym_cancel_wait_until_keyword] = ACTIONS(2129),
    [sym_cancel_all_wait_keyword] = ACTIONS(2129),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2129),
    [sym_not] = ACTIONS(2129),
    [sym_bnot] = ACTIONS(2129),
    [sym___file__] = ACTIONS(2129),
    [sym___date__] = ACTIONS(2129),
    [sym___ldate__] = ACTIONS(2129),
    [sym___time__] = ACTIONS(2129),
    [sym___line__] = ACTIONS(2129),
    [sym___name__] = ACTIONS(2129),
    [sym___netlinx__] = ACTIONS(2129),
    [sym_netlinx_axi_version] = ACTIONS(2129),
    [sym_url_flg_tcp] = ACTIONS(2129),
    [sym_url_flg_accinfopresent] = ACTIONS(2129),
    [sym_url_flg_temp] = ACTIONS(2129),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2129),
    [sym_url_flg_stat_mask] = ACTIONS(2129),
    [sym_url_flg_stat_lookup] = ACTIONS(2129),
    [sym_url_flg_stat_connecting] = ACTIONS(2129),
    [sym_url_flg_stat_waiting] = ACTIONS(2129),
    [sym_url_flg_stat_connected] = ACTIONS(2129),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2129),
    [sym_first_virtual_device] = ACTIONS(2129),
    [sym_first_local_port] = ACTIONS(2129),
    [sym_dynamic_virtual_device] = ACTIONS(2129),
    [sym_timeline_once] = ACTIONS(2129),
    [sym_timeline_repeat] = ACTIONS(2129),
    [sym_timeline_absolute] = ACTIONS(2129),
    [sym_timeline_relative] = ACTIONS(2129),
    [sym_do_push_timed_infinite] = ACTIONS(2129),
    [sym_source_type_no_address] = ACTIONS(2129),
    [sym_source_type_neuron_id] = ACTIONS(2129),
    [sym_source_type_ip_address] = ACTIONS(2129),
    [sym_source_type_axlink] = ACTIONS(2129),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2129),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2129),
    [sym_source_type_ip_socket_address] = ACTIONS(2129),
    [sym_source_type_rs232] = ACTIONS(2129),
    [sym_source_type_internal] = ACTIONS(2129),
    [sym_source_type_mac_address] = ACTIONS(2129),
    [sym_source_type_ipv4_port] = ACTIONS(2129),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2129),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2129),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2129),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2129),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2129),
    [sym_true] = ACTIONS(2129),
    [sym_false] = ACTIONS(2129),
    [sym_file_read_only] = ACTIONS(2129),
    [sym_file_rw_new] = ACTIONS(2129),
    [sym_file_rw_append] = ACTIONS(2129),
    [sym_ip_tcp] = ACTIONS(2129),
    [sym_ip_udp] = ACTIONS(2129),
    [sym_ip_udp_2way] = ACTIONS(2129),
    [sym_xml_encode_types] = ACTIONS(2129),
    [sym_xml_encode_char_as_list] = ACTIONS(2129),
    [sym_xml_encode_le] = ACTIONS(2129),
    [sym_xml_decode_types] = ACTIONS(2129),
    [sym_xml_decode_no_preserve] = ACTIONS(2129),
    [sym_duet_dev_type_display_device] = ACTIONS(2129),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2129),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2129),
    [sym_duet_dev_type_io_device] = ACTIONS(2129),
    [sym_duet_dev_type_relay_device] = ACTIONS(2129),
    [sym_duet_dev_type_ups] = ACTIONS(2129),
    [sym_duet_dev_type_amplifier] = ACTIONS(2129),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2129),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2129),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2129),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2129),
    [sym_duet_dev_type_camera] = ACTIONS(2129),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2129),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2129),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2129),
    [sym_duet_dev_type_dss] = ACTIONS(2129),
    [sym_duet_dev_type_dvr] = ACTIONS(2129),
    [sym_duet_dev_type_disc_device] = ACTIONS(2129),
    [sym_duet_dev_type_document_camera] = ACTIONS(2129),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2129),
    [sym_duet_dev_type_hvac] = ACTIONS(2129),
    [sym_duet_dev_type_keypad] = ACTIONS(2129),
    [sym_duet_dev_type_light] = ACTIONS(2129),
    [sym_duet_dev_type_monitor] = ACTIONS(2129),
    [sym_duet_dev_type_motor] = ACTIONS(2129),
    [sym_duet_dev_type_multi_window] = ACTIONS(2129),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2129),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2129),
    [sym_duet_dev_type_receiver] = ACTIONS(2129),
    [sym_duet_dev_type_security_system] = ACTIONS(2129),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2129),
    [sym_duet_dev_type_settop_box] = ACTIONS(2129),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2129),
    [sym_duet_dev_type_switcher] = ACTIONS(2129),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2129),
    [sym_duet_dev_type_tv] = ACTIONS(2129),
    [sym_duet_dev_type_utility] = ACTIONS(2129),
    [sym_duet_dev_type_vcr] = ACTIONS(2129),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2129),
    [sym_duet_dev_type_video_processor] = ACTIONS(2129),
    [sym_duet_dev_type_video_projector] = ACTIONS(2129),
    [sym_duet_dev_type_video_wall] = ACTIONS(2129),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2129),
    [sym_duet_dev_type_weather] = ACTIONS(2129),
    [sym_duet_dev_not_polled] = ACTIONS(2129),
    [sym_duet_dev_polled] = ACTIONS(2129),
    [sym_clkmgr_mode_network] = ACTIONS(2129),
    [sym_clkmgr_mode_standalone] = ACTIONS(2129),
    [sym_normal_standby] = ACTIONS(2129),
    [sym_normal_wake] = ACTIONS(2129),
    [sym_ip_multicast_ttl_option] = ACTIONS(2129),
    [sym_ip_ttl_subnet] = ACTIONS(2129),
    [sym_ip_ttl_site] = ACTIONS(2129),
    [sym_ip_ttl_region] = ACTIONS(2129),
    [sym_ip_ttl_continent] = ACTIONS(2129),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2129),
    [sym_ip_nodelay_on] = ACTIONS(2129),
    [sym_ip_nodelay_off] = ACTIONS(2129),
    [sym_amx_error] = ACTIONS(2129),
    [sym_amx_warning] = ACTIONS(2129),
    [sym_amx_info] = ACTIONS(2129),
    [sym_amx_debug] = ACTIONS(2129),
    [sym_netlinx_login_success] = ACTIONS(2129),
    [sym_netlinx_login_fail] = ACTIONS(2129),
    [sym_netlinx_logout] = ACTIONS(2129),
    [sym_null_str] = ACTIONS(2129),
    [sym_smtp_address] = ACTIONS(2129),
    [sym_smtp_port_number] = ACTIONS(2129),
    [sym_smtp_username] = ACTIONS(2129),
    [sym_smtp_password] = ACTIONS(2129),
    [sym_smtp_require_tls] = ACTIONS(2129),
    [sym_smtp_from] = ACTIONS(2129),
    [sym_smtp_tls_true] = ACTIONS(2129),
    [sym_smtp_tls_false] = ACTIONS(2129),
    [sym_listview_on_row_select_event] = ACTIONS(2129),
    [sym_data_structure_unknown] = ACTIONS(2129),
    [sym_data_structure_datafeed] = ACTIONS(2129),
    [sym_data_structure_datarecordset] = ACTIONS(2129),
    [sym_data_structure_datarecord] = ACTIONS(2129),
    [sym_data_structure_datafield] = ACTIONS(2129),
    [sym_data_max_content_field_count] = ACTIONS(2129),
    [sym_data_max_metadata_field_count] = ACTIONS(2129),
    [sym_data_max_id_length] = ACTIONS(2129),
    [sym_data_max_name_length] = ACTIONS(2129),
    [sym_data_max_description_length] = ACTIONS(2129),
    [sym_data_max_type_length] = ACTIONS(2129),
    [sym_data_max_format_length] = ACTIONS(2129),
    [sym_data_max_source_length] = ACTIONS(2129),
    [sym_data_max_value_length] = ACTIONS(2129),
    [sym_data_max_label_length] = ACTIONS(2129),
    [sym_data_max_url_length] = ACTIONS(2129),
    [sym_max_last_login_info_length] = ACTIONS(2129),
    [sym_data_type_unknown] = ACTIONS(2129),
    [sym_data_type_string] = ACTIONS(2129),
    [sym_data_type_datetime] = ACTIONS(2129),
    [sym_data_type_date] = ACTIONS(2129),
    [sym_data_type_time] = ACTIONS(2129),
    [sym_data_type_image] = ACTIONS(2129),
    [sym_data_format_url] = ACTIONS(2129),
    [sym_data_format_phone] = ACTIONS(2129),
    [sym_data_format_email] = ACTIONS(2129),
    [sym_data_format_iso8601] = ACTIONS(2129),
    [sym_auditlog_priv] = ACTIONS(2129),
    [sym_configuration_priv] = ACTIONS(2129),
    [sym_deviceconfig_priv] = ACTIONS(2129),
    [sym_ftp_priv] = ACTIONS(2129),
    [sym_http_priv] = ACTIONS(2129),
    [sym_networkconfig_priv] = ACTIONS(2129),
    [sym_programport_priv] = ACTIONS(2129),
    [sym_remoteui_priv] = ACTIONS(2129),
    [sym_securitycontrol_priv] = ACTIONS(2129),
    [sym_softwaremanagement_priv] = ACTIONS(2129),
    [sym_terminal_priv] = ACTIONS(2129),
    [sym_tpadmin_priv] = ACTIONS(2129),
    [sym_user1_priv] = ACTIONS(2129),
    [sym_user2_priv] = ACTIONS(2129),
    [sym_user3_priv] = ACTIONS(2129),
    [sym_user4_priv] = ACTIONS(2129),
    [sym_usermanagement_priv] = ACTIONS(2129),
    [sym_valid_account] = ACTIONS(2129),
    [sym_err_name_invalid] = ACTIONS(2129),
    [sym_err_password_invalid] = ACTIONS(2129),
    [sym_err_invalid_account] = ACTIONS(2129),
    [sym_err_no_authorization] = ACTIONS(2129),
    [sym_err_invalid_parameter] = ACTIONS(2129),
    [sym_err_user_locked_out] = ACTIONS(2129),
    [sym_err_user_account_expired] = ACTIONS(2129),
    [sym_err_general_error] = ACTIONS(2129),
    [sym_err_authentication_error] = ACTIONS(2129),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2129),
    [sym_tls_validate_certificate] = ACTIONS(2129),
    [sym_tdata] = ACTIONS(2129),
    [sym_tchannel] = ACTIONS(2129),
    [sym_tlevel] = ACTIONS(2129),
    [sym_tbutton] = ACTIONS(2129),
    [sym_ttimeline] = ACTIONS(2129),
    [sym_tcustom] = ACTIONS(2129),
    [sym_url_struct] = ACTIONS(2129),
    [sym_dns_struct] = ACTIONS(2129),
    [sym_ip_address_struct] = ACTIONS(2129),
    [sym_dev_info_struct] = ACTIONS(2129),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2129),
    [sym_data_feed] = ACTIONS(2129),
    [sym_data_field] = ACTIONS(2129),
    [sym_data_record] = ACTIONS(2129),
    [sym_wc_data_feed] = ACTIONS(2129),
    [sym_wc_data_field] = ACTIONS(2129),
    [sym_wc_data_record] = ACTIONS(2129),
    [sym_last_login_info] = ACTIONS(2129),
    [sym____reserved___] = ACTIONS(2129),
    [sym_date] = ACTIONS(2129),
    [sym_day] = ACTIONS(2129),
    [sym_time] = ACTIONS(2129),
    [sym_ldate] = ACTIONS(2129),
    [sym_push_device] = ACTIONS(2129),
    [sym_push_channel] = ACTIONS(2129),
    [sym_push_devchan] = ACTIONS(2129),
    [sym_release_device] = ACTIONS(2129),
    [sym_release_channel] = ACTIONS(2129),
    [sym_release_devchan] = ACTIONS(2129),
    [sym_master_slot] = ACTIONS(2129),
    [sym_get_pulse_time] = ACTIONS(2129),
    [sym_get_timer] = ACTIONS(2129),
    [sym_master_sn] = ACTIONS(2129),
    [sym_system_number] = ACTIONS(2129),
    [sym_button] = ACTIONS(2129),
    [sym_dv_channel] = ACTIONS(2129),
    [sym_data] = ACTIONS(2129),
    [sym_level] = ACTIONS(2129),
    [sym_timeline] = ACTIONS(2129),
    [sym_channel] = ACTIONS(2129),
    [sym_custom] = ACTIONS(2129),
    [sym_atoi] = ACTIONS(2129),
    [sym_atol] = ACTIONS(2129),
    [sym_atof] = ACTIONS(2129),
    [sym_itoa] = ACTIONS(2129),
    [sym_itohex] = ACTIONS(2129),
    [sym_hextoi] = ACTIONS(2129),
    [sym_ftoa] = ACTIONS(2129),
    [sym_find_string] = ACTIONS(2129),
    [sym_left_string] = ACTIONS(2129),
    [sym_right_string] = ACTIONS(2129),
    [sym_mid_string] = ACTIONS(2129),
    [sym_remove_string] = ACTIONS(2129),
    [sym_get_buffer_string] = ACTIONS(2129),
    [sym_get_buffer_char] = ACTIONS(2129),
    [sym_get_multi_buffer_string] = ACTIONS(2129),
    [sym_length_string] = ACTIONS(2129),
    [sym_max_length_string] = ACTIONS(2129),
    [sym_set_length_string] = ACTIONS(2129),
    [sym_compare_string] = ACTIONS(2129),
    [sym_upper_string] = ACTIONS(2129),
    [sym_lower_string] = ACTIONS(2129),
    [sym_get_last] = ACTIONS(2129),
    [sym_date_to_day] = ACTIONS(2129),
    [sym_date_to_month] = ACTIONS(2129),
    [sym_date_to_year] = ACTIONS(2129),
    [sym_time_to_hour] = ACTIONS(2129),
    [sym_time_to_minute] = ACTIONS(2129),
    [sym_time_to_second] = ACTIONS(2129),
    [sym_day_of_week] = ACTIONS(2129),
    [sym_set_pulse_time] = ACTIONS(2129),
    [sym_set_timer] = ACTIONS(2129),
    [sym_file_open] = ACTIONS(2129),
    [sym_file_close] = ACTIONS(2129),
    [sym_file_read] = ACTIONS(2129),
    [sym_file_read_line] = ACTIONS(2129),
    [sym_file_write] = ACTIONS(2129),
    [sym_file_write_line] = ACTIONS(2129),
    [sym_file_copy] = ACTIONS(2129),
    [sym_file_delete] = ACTIONS(2129),
    [sym_file_rename] = ACTIONS(2129),
    [sym_file_dir] = ACTIONS(2129),
    [sym_file_getdir] = ACTIONS(2129),
    [sym_file_setdir] = ACTIONS(2129),
    [sym_file_create_dir] = ACTIONS(2129),
    [sym_file_remove_dir] = ACTIONS(2129),
    [sym_file_seek] = ACTIONS(2129),
    [sym_length_array] = ACTIONS(2129),
    [sym_max_length_array] = ACTIONS(2129),
    [sym_set_length_array] = ACTIONS(2129),
    [sym_ip_client_open] = ACTIONS(2129),
    [sym_ip_client_close] = ACTIONS(2129),
    [sym_ip_server_open] = ACTIONS(2129),
    [sym_ip_server_close] = ACTIONS(2129),
    [sym_ip_bound_client_open] = ACTIONS(2129),
    [sym_ip_mc_server_open] = ACTIONS(2129),
    [sym_ip_set_option] = ACTIONS(2129),
    [sym_ssh_client_open] = ACTIONS(2129),
    [sym_ssh_client_close] = ACTIONS(2129),
    [sym_tls_client_open] = ACTIONS(2129),
    [sym_tls_client_close] = ACTIONS(2129),
    [sym_do_push] = ACTIONS(2129),
    [sym_do_push_timed] = ACTIONS(2129),
    [sym_do_release] = ACTIONS(2129),
    [sym_device_id] = ACTIONS(2129),
    [sym_device_info] = ACTIONS(2129),
    [sym_device_id_string] = ACTIONS(2129),
    [sym_device_standby] = ACTIONS(2129),
    [sym_device_wake] = ACTIONS(2129),
    [sym_timeline_create] = ACTIONS(2129),
    [sym_timeline_kill] = ACTIONS(2129),
    [sym_timeline_reload] = ACTIONS(2129),
    [sym_timeline_pause] = ACTIONS(2129),
    [sym_timeline_restart] = ACTIONS(2129),
    [sym_timeline_set] = ACTIONS(2129),
    [sym_timeline_get] = ACTIONS(2129),
    [sym_timeline_active] = ACTIONS(2129),
    [sym_timeline_dynamic_id] = ACTIONS(2129),
    [sym_variable_to_string] = ACTIONS(2129),
    [sym_string_to_variable] = ACTIONS(2129),
    [sym_length_variable_to_string] = ACTIONS(2129),
    [sym_type_cast] = ACTIONS(2129),
    [sym_variable_to_xml] = ACTIONS(2129),
    [sym_xml_to_variable] = ACTIONS(2129),
    [sym_length_variable_to_xml] = ACTIONS(2129),
    [sym_redirect_string] = ACTIONS(2129),
    [sym_random_number] = ACTIONS(2129),
    [sym_get_unique_id] = ACTIONS(2129),
    [sym_get_system_number] = ACTIONS(2129),
    [sym_set_system_number] = ACTIONS(2129),
    [sym_get_serial_number] = ACTIONS(2129),
    [sym_get_master_build] = ACTIONS(2129),
    [sym_set_validation_code] = ACTIONS(2129),
    [sym_reboot] = ACTIONS(2129),
    [sym_load_duet_module] = ACTIONS(2129),
    [sym_unload_duet_module] = ACTIONS(2129),
    [sym_duet_mem_size_set] = ACTIONS(2129),
    [sym_duet_mem_size_get] = ACTIONS(2129),
    [sym_dynamic_application_device] = ACTIONS(2129),
    [sym_dynamic_polled_port] = ACTIONS(2129),
    [sym_static_port_binding] = ACTIONS(2129),
    [sym_static_ip_binding] = ACTIONS(2129),
    [sym_get_dns_list] = ACTIONS(2129),
    [sym_set_dns_list] = ACTIONS(2129),
    [sym_get_ip_address] = ACTIONS(2129),
    [sym_set_ip_address] = ACTIONS(2129),
    [sym_get_url_list] = ACTIONS(2129),
    [sym_add_url_entry] = ACTIONS(2129),
    [sym_delete_url_entry] = ACTIONS(2129),
    [sym_abs_value] = ACTIONS(2129),
    [sym_max_value] = ACTIONS(2129),
    [sym_min_value] = ACTIONS(2129),
    [sym_exp_value] = ACTIONS(2129),
    [sym_log_value] = ACTIONS(2129),
    [sym_log10_value] = ACTIONS(2129),
    [sym_power_value] = ACTIONS(2129),
    [sym_sqrt_value] = ACTIONS(2129),
    [sym_raw_be] = ACTIONS(2129),
    [sym_raw_le] = ACTIONS(2129),
    [sym_internal_queue_size_set] = ACTIONS(2129),
    [sym_internal_queue_size_get] = ACTIONS(2129),
    [sym_internal_threshold_set] = ACTIONS(2129),
    [sym_internal_threshold_get] = ACTIONS(2129),
    [sym_rebuilt_event] = ACTIONS(2129),
    [sym_do_custom_event] = ACTIONS(2129),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2129),
    [sym_clkmgr_set_clk_source] = ACTIONS(2129),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2129),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2129),
    [sym_clkmgr_get_timezone] = ACTIONS(2129),
    [sym_clkmgr_set_timezone] = ACTIONS(2129),
    [sym_clkmgr_get_resync_period] = ACTIONS(2129),
    [sym_clkmgr_set_resync_period] = ACTIONS(2129),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2129),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2129),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2129),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2129),
    [sym_clkmgr_get_timeservers] = ACTIONS(2129),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2129),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2129),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2129),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2129),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2129),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2129),
    [sym_set_log_level] = ACTIONS(2129),
    [sym_get_log_level] = ACTIONS(2129),
    [sym_amx_log] = ACTIONS(2129),
    [sym_smtp_server_config_set] = ACTIONS(2129),
    [sym_smtp_server_config_get] = ACTIONS(2129),
    [sym_smtp_send] = ACTIONS(2129),
    [sym_validate_netlinx_account] = ACTIONS(2129),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2129),
    [sym_authenticate_certificate] = ACTIONS(2129),
    [sym_audit_netlinx_session_event] = ACTIONS(2129),
    [sym_audit_netlinx_generic_event] = ACTIONS(2129),
    [sym_get_available_flash_disk_space] = ACTIONS(2129),
    [sym_get_max_flash_disk_space] = ACTIONS(2129),
    [sym_data_create_feed] = ACTIONS(2129),
    [sym_data_delete_feed] = ACTIONS(2129),
    [sym_data_publish_feed] = ACTIONS(2129),
    [sym_data_get_published_feed] = ACTIONS(2129),
    [sym_data_add_record] = ACTIONS(2129),
    [sym_data_get_event_record] = ACTIONS(2129),
    [sym__wc_data_create_feed] = ACTIONS(2129),
    [sym__wc_data_add_record] = ACTIONS(2129),
    [sym__wc_data_get_event_record] = ACTIONS(2129),
    [sym_format] = ACTIONS(2129),
    [sym_astro_clock] = ACTIONS(2129),
    [sym_set_outdoor_temperature] = ACTIONS(2129),
    [sym_set_virtual_level_count] = ACTIONS(2129),
    [sym_set_virtual_channel_count] = ACTIONS(2129),
    [sym_set_virtual_port_count] = ACTIONS(2129),
    [sym__wc_encode_int] = ACTIONS(2129),
    [sym__wc_decode_int] = ACTIONS(2129),
    [sym__wc_int] = ACTIONS(2129),
    [sym__wc_to_ch_int] = ACTIONS(2129),
    [sym__ch_to_wc_int] = ACTIONS(2129),
    [sym__wc_find_string_int] = ACTIONS(2129),
    [sym__wc_left_string_int] = ACTIONS(2129),
    [sym__wc_lower_string_int] = ACTIONS(2129),
    [sym__wc_mid_string_int] = ACTIONS(2129),
    [sym__wc_remove_string_int] = ACTIONS(2129),
    [sym__wc_right_string_int] = ACTIONS(2129),
    [sym__wc_upper_string_int] = ACTIONS(2129),
    [sym__wc_compare_string_int] = ACTIONS(2129),
    [sym__wc_get_buffer_char_int] = ACTIONS(2129),
    [sym__wc_get_buffer_string_int] = ACTIONS(2129),
    [sym__wc_concat_string_int] = ACTIONS(2129),
    [sym__wc_file_open_int] = ACTIONS(2129),
    [sym__wc_file_close_int] = ACTIONS(2129),
    [sym__wc_file_read_int] = ACTIONS(2129),
    [sym__wc_file_read_line_int] = ACTIONS(2129),
    [sym__wc_file_write_int] = ACTIONS(2129),
    [sym__wc_file_write_line_int] = ACTIONS(2129),
    [aux_sym_preproc_include_token1] = ACTIONS(2129),
    [aux_sym_preproc_define_token1] = ACTIONS(2131),
    [aux_sym_preproc_warn_token1] = ACTIONS(2131),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2131),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2131),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2131),
    [anon_sym_LBRACK] = ACTIONS(2131),
    [anon_sym_LBRACE] = ACTIONS(2131),
    [anon_sym_LPAREN] = ACTIONS(2129),
    [anon_sym_BANG] = ACTIONS(2131),
    [anon_sym_TILDE] = ACTIONS(2131),
    [anon_sym_DASH] = ACTIONS(2129),
    [anon_sym_PLUS] = ACTIONS(2129),
    [anon_sym_DASH_DASH] = ACTIONS(2131),
    [anon_sym_PLUS_PLUS] = ACTIONS(2131),
    [anon_sym_DQUOTE] = ACTIONS(2131),
    [anon_sym_SQUOTE] = ACTIONS(2131),
    [sym_number_literal] = ACTIONS(2131),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(419)] = {
    [ts_builtin_sym_end] = ACTIONS(2031),
    [sym_identifier] = ACTIONS(2029),
    [sym_program_name_keyword] = ACTIONS(2029),
    [sym_module_name_keyword] = ACTIONS(2029),
    [sym_define_device_keyword] = ACTIONS(2029),
    [sym_define_combine_keyword] = ACTIONS(2029),
    [sym_define_constant_keyword] = ACTIONS(2029),
    [sym_define_type_keyword] = ACTIONS(2029),
    [sym_define_variable_keyword] = ACTIONS(2029),
    [sym_define_system_variable_keyword] = ACTIONS(2029),
    [sym_define_connect_level_keyword] = ACTIONS(2029),
    [sym_define_latching_keyword] = ACTIONS(2029),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2029),
    [sym_define_toggling_keyword] = ACTIONS(2029),
    [sym_define_call_keyword] = ACTIONS(2029),
    [sym_define_function_keyword] = ACTIONS(2029),
    [sym_define_library_function_keyword] = ACTIONS(2029),
    [sym_define_start_keyword] = ACTIONS(2029),
    [sym_define_event_keyword] = ACTIONS(2029),
    [sym_define_program_keyword] = ACTIONS(2029),
    [sym_define_module_keyword] = ACTIONS(2029),
    [sym_char_keyword] = ACTIONS(2029),
    [sym_widechar_keyword] = ACTIONS(2029),
    [sym_integer_keyword] = ACTIONS(2029),
    [sym_sinteger_keyword] = ACTIONS(2029),
    [sym_long_keyword] = ACTIONS(2029),
    [sym_slong_keyword] = ACTIONS(2029),
    [sym_float_keyword] = ACTIONS(2029),
    [sym_double_keyword] = ACTIONS(2029),
    [sym_variant_keyword] = ACTIONS(2029),
    [sym_variantarray_keyword] = ACTIONS(2029),
    [sym_local_var_keyword] = ACTIONS(2029),
    [sym_stack_var_keyword] = ACTIONS(2029),
    [sym_constant_keyword] = ACTIONS(2029),
    [sym_volatile_keyword] = ACTIONS(2029),
    [sym_non_volatile_keyword] = ACTIONS(2029),
    [sym_persistent_keyword] = ACTIONS(2029),
    [sym_dev_keyword] = ACTIONS(2029),
    [sym_devlev_keyword] = ACTIONS(2029),
    [sym_devchan_keyword] = ACTIONS(2029),
    [sym_if_keyword] = ACTIONS(2029),
    [sym_switch_keyword] = ACTIONS(2029),
    [sym_case_keyword] = ACTIONS(2029),
    [sym_default_keyword] = ACTIONS(2029),
    [sym_while_keyword] = ACTIONS(2029),
    [sym_for_keyword] = ACTIONS(2029),
    [sym_break_keyword] = ACTIONS(2029),
    [sym_continue_keyword] = ACTIONS(2029),
    [sym_return_keyword] = ACTIONS(2029),
    [sym_select_keyword] = ACTIONS(2029),
    [sym_active_keyword] = ACTIONS(2029),
    [sym_struct_keyword] = ACTIONS(2029),
    [sym_structure_keyword] = ACTIONS(2029),
    [sym_button_event_keyword] = ACTIONS(2029),
    [sym_channel_event_keyword] = ACTIONS(2029),
    [sym_level_event_keyword] = ACTIONS(2029),
    [sym_data_event_keyword] = ACTIONS(2029),
    [sym_timeline_event_keyword] = ACTIONS(2029),
    [sym_custom_event_keyword] = ACTIONS(2029),
    [sym_push_keyword] = ACTIONS(2029),
    [sym_release_keyword] = ACTIONS(2029),
    [aux_sym_on_keyword_token1] = ACTIONS(2029),
    [aux_sym_off_keyword_token1] = ACTIONS(2029),
    [sym_send_level_keyword] = ACTIONS(2029),
    [sym_send_string_keyword] = ACTIONS(2029),
    [sym_send_command_keyword] = ACTIONS(2029),
    [sym_clear_buffer_keyword] = ACTIONS(2029),
    [sym_create_buffer_keyword] = ACTIONS(2029),
    [sym_create_multi_buffer_keyword] = ACTIONS(2029),
    [sym_call_keyword] = ACTIONS(2029),
    [sym_system_call_keyword] = ACTIONS(2029),
    [sym_devchan_to_keyword] = ACTIONS(2029),
    [sym_devchan_min_to_keyword] = ACTIONS(2029),
    [sym_devchan_total_off_keyword] = ACTIONS(2029),
    [sym_devchan_pulse_keyword] = ACTIONS(2029),
    [sym_wait_keyword] = ACTIONS(2029),
    [sym_cancel_wait_keyword] = ACTIONS(2029),
    [sym_wait_until_keyword] = ACTIONS(2029),
    [sym_cancel_wait_until_keyword] = ACTIONS(2029),
    [sym_cancel_all_wait_keyword] = ACTIONS(2029),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2029),
    [sym_not] = ACTIONS(2029),
    [sym_bnot] = ACTIONS(2029),
    [sym___file__] = ACTIONS(2029),
    [sym___date__] = ACTIONS(2029),
    [sym___ldate__] = ACTIONS(2029),
    [sym___time__] = ACTIONS(2029),
    [sym___line__] = ACTIONS(2029),
    [sym___name__] = ACTIONS(2029),
    [sym___netlinx__] = ACTIONS(2029),
    [sym_netlinx_axi_version] = ACTIONS(2029),
    [sym_url_flg_tcp] = ACTIONS(2029),
    [sym_url_flg_accinfopresent] = ACTIONS(2029),
    [sym_url_flg_temp] = ACTIONS(2029),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2029),
    [sym_url_flg_stat_mask] = ACTIONS(2029),
    [sym_url_flg_stat_lookup] = ACTIONS(2029),
    [sym_url_flg_stat_connecting] = ACTIONS(2029),
    [sym_url_flg_stat_waiting] = ACTIONS(2029),
    [sym_url_flg_stat_connected] = ACTIONS(2029),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2029),
    [sym_first_virtual_device] = ACTIONS(2029),
    [sym_first_local_port] = ACTIONS(2029),
    [sym_dynamic_virtual_device] = ACTIONS(2029),
    [sym_timeline_once] = ACTIONS(2029),
    [sym_timeline_repeat] = ACTIONS(2029),
    [sym_timeline_absolute] = ACTIONS(2029),
    [sym_timeline_relative] = ACTIONS(2029),
    [sym_do_push_timed_infinite] = ACTIONS(2029),
    [sym_source_type_no_address] = ACTIONS(2029),
    [sym_source_type_neuron_id] = ACTIONS(2029),
    [sym_source_type_ip_address] = ACTIONS(2029),
    [sym_source_type_axlink] = ACTIONS(2029),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2029),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2029),
    [sym_source_type_ip_socket_address] = ACTIONS(2029),
    [sym_source_type_rs232] = ACTIONS(2029),
    [sym_source_type_internal] = ACTIONS(2029),
    [sym_source_type_mac_address] = ACTIONS(2029),
    [sym_source_type_ipv4_port] = ACTIONS(2029),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2029),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2029),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2029),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2029),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2029),
    [sym_true] = ACTIONS(2029),
    [sym_false] = ACTIONS(2029),
    [sym_file_read_only] = ACTIONS(2029),
    [sym_file_rw_new] = ACTIONS(2029),
    [sym_file_rw_append] = ACTIONS(2029),
    [sym_ip_tcp] = ACTIONS(2029),
    [sym_ip_udp] = ACTIONS(2029),
    [sym_ip_udp_2way] = ACTIONS(2029),
    [sym_xml_encode_types] = ACTIONS(2029),
    [sym_xml_encode_char_as_list] = ACTIONS(2029),
    [sym_xml_encode_le] = ACTIONS(2029),
    [sym_xml_decode_types] = ACTIONS(2029),
    [sym_xml_decode_no_preserve] = ACTIONS(2029),
    [sym_duet_dev_type_display_device] = ACTIONS(2029),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2029),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2029),
    [sym_duet_dev_type_io_device] = ACTIONS(2029),
    [sym_duet_dev_type_relay_device] = ACTIONS(2029),
    [sym_duet_dev_type_ups] = ACTIONS(2029),
    [sym_duet_dev_type_amplifier] = ACTIONS(2029),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2029),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2029),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2029),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2029),
    [sym_duet_dev_type_camera] = ACTIONS(2029),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2029),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2029),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2029),
    [sym_duet_dev_type_dss] = ACTIONS(2029),
    [sym_duet_dev_type_dvr] = ACTIONS(2029),
    [sym_duet_dev_type_disc_device] = ACTIONS(2029),
    [sym_duet_dev_type_document_camera] = ACTIONS(2029),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2029),
    [sym_duet_dev_type_hvac] = ACTIONS(2029),
    [sym_duet_dev_type_keypad] = ACTIONS(2029),
    [sym_duet_dev_type_light] = ACTIONS(2029),
    [sym_duet_dev_type_monitor] = ACTIONS(2029),
    [sym_duet_dev_type_motor] = ACTIONS(2029),
    [sym_duet_dev_type_multi_window] = ACTIONS(2029),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2029),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2029),
    [sym_duet_dev_type_receiver] = ACTIONS(2029),
    [sym_duet_dev_type_security_system] = ACTIONS(2029),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2029),
    [sym_duet_dev_type_settop_box] = ACTIONS(2029),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2029),
    [sym_duet_dev_type_switcher] = ACTIONS(2029),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2029),
    [sym_duet_dev_type_tv] = ACTIONS(2029),
    [sym_duet_dev_type_utility] = ACTIONS(2029),
    [sym_duet_dev_type_vcr] = ACTIONS(2029),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2029),
    [sym_duet_dev_type_video_processor] = ACTIONS(2029),
    [sym_duet_dev_type_video_projector] = ACTIONS(2029),
    [sym_duet_dev_type_video_wall] = ACTIONS(2029),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2029),
    [sym_duet_dev_type_weather] = ACTIONS(2029),
    [sym_duet_dev_not_polled] = ACTIONS(2029),
    [sym_duet_dev_polled] = ACTIONS(2029),
    [sym_clkmgr_mode_network] = ACTIONS(2029),
    [sym_clkmgr_mode_standalone] = ACTIONS(2029),
    [sym_normal_standby] = ACTIONS(2029),
    [sym_normal_wake] = ACTIONS(2029),
    [sym_ip_multicast_ttl_option] = ACTIONS(2029),
    [sym_ip_ttl_subnet] = ACTIONS(2029),
    [sym_ip_ttl_site] = ACTIONS(2029),
    [sym_ip_ttl_region] = ACTIONS(2029),
    [sym_ip_ttl_continent] = ACTIONS(2029),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2029),
    [sym_ip_nodelay_on] = ACTIONS(2029),
    [sym_ip_nodelay_off] = ACTIONS(2029),
    [sym_amx_error] = ACTIONS(2029),
    [sym_amx_warning] = ACTIONS(2029),
    [sym_amx_info] = ACTIONS(2029),
    [sym_amx_debug] = ACTIONS(2029),
    [sym_netlinx_login_success] = ACTIONS(2029),
    [sym_netlinx_login_fail] = ACTIONS(2029),
    [sym_netlinx_logout] = ACTIONS(2029),
    [sym_null_str] = ACTIONS(2029),
    [sym_smtp_address] = ACTIONS(2029),
    [sym_smtp_port_number] = ACTIONS(2029),
    [sym_smtp_username] = ACTIONS(2029),
    [sym_smtp_password] = ACTIONS(2029),
    [sym_smtp_require_tls] = ACTIONS(2029),
    [sym_smtp_from] = ACTIONS(2029),
    [sym_smtp_tls_true] = ACTIONS(2029),
    [sym_smtp_tls_false] = ACTIONS(2029),
    [sym_listview_on_row_select_event] = ACTIONS(2029),
    [sym_data_structure_unknown] = ACTIONS(2029),
    [sym_data_structure_datafeed] = ACTIONS(2029),
    [sym_data_structure_datarecordset] = ACTIONS(2029),
    [sym_data_structure_datarecord] = ACTIONS(2029),
    [sym_data_structure_datafield] = ACTIONS(2029),
    [sym_data_max_content_field_count] = ACTIONS(2029),
    [sym_data_max_metadata_field_count] = ACTIONS(2029),
    [sym_data_max_id_length] = ACTIONS(2029),
    [sym_data_max_name_length] = ACTIONS(2029),
    [sym_data_max_description_length] = ACTIONS(2029),
    [sym_data_max_type_length] = ACTIONS(2029),
    [sym_data_max_format_length] = ACTIONS(2029),
    [sym_data_max_source_length] = ACTIONS(2029),
    [sym_data_max_value_length] = ACTIONS(2029),
    [sym_data_max_label_length] = ACTIONS(2029),
    [sym_data_max_url_length] = ACTIONS(2029),
    [sym_max_last_login_info_length] = ACTIONS(2029),
    [sym_data_type_unknown] = ACTIONS(2029),
    [sym_data_type_string] = ACTIONS(2029),
    [sym_data_type_datetime] = ACTIONS(2029),
    [sym_data_type_date] = ACTIONS(2029),
    [sym_data_type_time] = ACTIONS(2029),
    [sym_data_type_image] = ACTIONS(2029),
    [sym_data_format_url] = ACTIONS(2029),
    [sym_data_format_phone] = ACTIONS(2029),
    [sym_data_format_email] = ACTIONS(2029),
    [sym_data_format_iso8601] = ACTIONS(2029),
    [sym_auditlog_priv] = ACTIONS(2029),
    [sym_configuration_priv] = ACTIONS(2029),
    [sym_deviceconfig_priv] = ACTIONS(2029),
    [sym_ftp_priv] = ACTIONS(2029),
    [sym_http_priv] = ACTIONS(2029),
    [sym_networkconfig_priv] = ACTIONS(2029),
    [sym_programport_priv] = ACTIONS(2029),
    [sym_remoteui_priv] = ACTIONS(2029),
    [sym_securitycontrol_priv] = ACTIONS(2029),
    [sym_softwaremanagement_priv] = ACTIONS(2029),
    [sym_terminal_priv] = ACTIONS(2029),
    [sym_tpadmin_priv] = ACTIONS(2029),
    [sym_user1_priv] = ACTIONS(2029),
    [sym_user2_priv] = ACTIONS(2029),
    [sym_user3_priv] = ACTIONS(2029),
    [sym_user4_priv] = ACTIONS(2029),
    [sym_usermanagement_priv] = ACTIONS(2029),
    [sym_valid_account] = ACTIONS(2029),
    [sym_err_name_invalid] = ACTIONS(2029),
    [sym_err_password_invalid] = ACTIONS(2029),
    [sym_err_invalid_account] = ACTIONS(2029),
    [sym_err_no_authorization] = ACTIONS(2029),
    [sym_err_invalid_parameter] = ACTIONS(2029),
    [sym_err_user_locked_out] = ACTIONS(2029),
    [sym_err_user_account_expired] = ACTIONS(2029),
    [sym_err_general_error] = ACTIONS(2029),
    [sym_err_authentication_error] = ACTIONS(2029),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2029),
    [sym_tls_validate_certificate] = ACTIONS(2029),
    [sym_tdata] = ACTIONS(2029),
    [sym_tchannel] = ACTIONS(2029),
    [sym_tlevel] = ACTIONS(2029),
    [sym_tbutton] = ACTIONS(2029),
    [sym_ttimeline] = ACTIONS(2029),
    [sym_tcustom] = ACTIONS(2029),
    [sym_url_struct] = ACTIONS(2029),
    [sym_dns_struct] = ACTIONS(2029),
    [sym_ip_address_struct] = ACTIONS(2029),
    [sym_dev_info_struct] = ACTIONS(2029),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2029),
    [sym_data_feed] = ACTIONS(2029),
    [sym_data_field] = ACTIONS(2029),
    [sym_data_record] = ACTIONS(2029),
    [sym_wc_data_feed] = ACTIONS(2029),
    [sym_wc_data_field] = ACTIONS(2029),
    [sym_wc_data_record] = ACTIONS(2029),
    [sym_last_login_info] = ACTIONS(2029),
    [sym____reserved___] = ACTIONS(2029),
    [sym_date] = ACTIONS(2029),
    [sym_day] = ACTIONS(2029),
    [sym_time] = ACTIONS(2029),
    [sym_ldate] = ACTIONS(2029),
    [sym_push_device] = ACTIONS(2029),
    [sym_push_channel] = ACTIONS(2029),
    [sym_push_devchan] = ACTIONS(2029),
    [sym_release_device] = ACTIONS(2029),
    [sym_release_channel] = ACTIONS(2029),
    [sym_release_devchan] = ACTIONS(2029),
    [sym_master_slot] = ACTIONS(2029),
    [sym_get_pulse_time] = ACTIONS(2029),
    [sym_get_timer] = ACTIONS(2029),
    [sym_master_sn] = ACTIONS(2029),
    [sym_system_number] = ACTIONS(2029),
    [sym_button] = ACTIONS(2029),
    [sym_dv_channel] = ACTIONS(2029),
    [sym_data] = ACTIONS(2029),
    [sym_level] = ACTIONS(2029),
    [sym_timeline] = ACTIONS(2029),
    [sym_channel] = ACTIONS(2029),
    [sym_custom] = ACTIONS(2029),
    [sym_atoi] = ACTIONS(2029),
    [sym_atol] = ACTIONS(2029),
    [sym_atof] = ACTIONS(2029),
    [sym_itoa] = ACTIONS(2029),
    [sym_itohex] = ACTIONS(2029),
    [sym_hextoi] = ACTIONS(2029),
    [sym_ftoa] = ACTIONS(2029),
    [sym_find_string] = ACTIONS(2029),
    [sym_left_string] = ACTIONS(2029),
    [sym_right_string] = ACTIONS(2029),
    [sym_mid_string] = ACTIONS(2029),
    [sym_remove_string] = ACTIONS(2029),
    [sym_get_buffer_string] = ACTIONS(2029),
    [sym_get_buffer_char] = ACTIONS(2029),
    [sym_get_multi_buffer_string] = ACTIONS(2029),
    [sym_length_string] = ACTIONS(2029),
    [sym_max_length_string] = ACTIONS(2029),
    [sym_set_length_string] = ACTIONS(2029),
    [sym_compare_string] = ACTIONS(2029),
    [sym_upper_string] = ACTIONS(2029),
    [sym_lower_string] = ACTIONS(2029),
    [sym_get_last] = ACTIONS(2029),
    [sym_date_to_day] = ACTIONS(2029),
    [sym_date_to_month] = ACTIONS(2029),
    [sym_date_to_year] = ACTIONS(2029),
    [sym_time_to_hour] = ACTIONS(2029),
    [sym_time_to_minute] = ACTIONS(2029),
    [sym_time_to_second] = ACTIONS(2029),
    [sym_day_of_week] = ACTIONS(2029),
    [sym_set_pulse_time] = ACTIONS(2029),
    [sym_set_timer] = ACTIONS(2029),
    [sym_file_open] = ACTIONS(2029),
    [sym_file_close] = ACTIONS(2029),
    [sym_file_read] = ACTIONS(2029),
    [sym_file_read_line] = ACTIONS(2029),
    [sym_file_write] = ACTIONS(2029),
    [sym_file_write_line] = ACTIONS(2029),
    [sym_file_copy] = ACTIONS(2029),
    [sym_file_delete] = ACTIONS(2029),
    [sym_file_rename] = ACTIONS(2029),
    [sym_file_dir] = ACTIONS(2029),
    [sym_file_getdir] = ACTIONS(2029),
    [sym_file_setdir] = ACTIONS(2029),
    [sym_file_create_dir] = ACTIONS(2029),
    [sym_file_remove_dir] = ACTIONS(2029),
    [sym_file_seek] = ACTIONS(2029),
    [sym_length_array] = ACTIONS(2029),
    [sym_max_length_array] = ACTIONS(2029),
    [sym_set_length_array] = ACTIONS(2029),
    [sym_ip_client_open] = ACTIONS(2029),
    [sym_ip_client_close] = ACTIONS(2029),
    [sym_ip_server_open] = ACTIONS(2029),
    [sym_ip_server_close] = ACTIONS(2029),
    [sym_ip_bound_client_open] = ACTIONS(2029),
    [sym_ip_mc_server_open] = ACTIONS(2029),
    [sym_ip_set_option] = ACTIONS(2029),
    [sym_ssh_client_open] = ACTIONS(2029),
    [sym_ssh_client_close] = ACTIONS(2029),
    [sym_tls_client_open] = ACTIONS(2029),
    [sym_tls_client_close] = ACTIONS(2029),
    [sym_do_push] = ACTIONS(2029),
    [sym_do_push_timed] = ACTIONS(2029),
    [sym_do_release] = ACTIONS(2029),
    [sym_device_id] = ACTIONS(2029),
    [sym_device_info] = ACTIONS(2029),
    [sym_device_id_string] = ACTIONS(2029),
    [sym_device_standby] = ACTIONS(2029),
    [sym_device_wake] = ACTIONS(2029),
    [sym_timeline_create] = ACTIONS(2029),
    [sym_timeline_kill] = ACTIONS(2029),
    [sym_timeline_reload] = ACTIONS(2029),
    [sym_timeline_pause] = ACTIONS(2029),
    [sym_timeline_restart] = ACTIONS(2029),
    [sym_timeline_set] = ACTIONS(2029),
    [sym_timeline_get] = ACTIONS(2029),
    [sym_timeline_active] = ACTIONS(2029),
    [sym_timeline_dynamic_id] = ACTIONS(2029),
    [sym_variable_to_string] = ACTIONS(2029),
    [sym_string_to_variable] = ACTIONS(2029),
    [sym_length_variable_to_string] = ACTIONS(2029),
    [sym_type_cast] = ACTIONS(2029),
    [sym_variable_to_xml] = ACTIONS(2029),
    [sym_xml_to_variable] = ACTIONS(2029),
    [sym_length_variable_to_xml] = ACTIONS(2029),
    [sym_redirect_string] = ACTIONS(2029),
    [sym_random_number] = ACTIONS(2029),
    [sym_get_unique_id] = ACTIONS(2029),
    [sym_get_system_number] = ACTIONS(2029),
    [sym_set_system_number] = ACTIONS(2029),
    [sym_get_serial_number] = ACTIONS(2029),
    [sym_get_master_build] = ACTIONS(2029),
    [sym_set_validation_code] = ACTIONS(2029),
    [sym_reboot] = ACTIONS(2029),
    [sym_load_duet_module] = ACTIONS(2029),
    [sym_unload_duet_module] = ACTIONS(2029),
    [sym_duet_mem_size_set] = ACTIONS(2029),
    [sym_duet_mem_size_get] = ACTIONS(2029),
    [sym_dynamic_application_device] = ACTIONS(2029),
    [sym_dynamic_polled_port] = ACTIONS(2029),
    [sym_static_port_binding] = ACTIONS(2029),
    [sym_static_ip_binding] = ACTIONS(2029),
    [sym_get_dns_list] = ACTIONS(2029),
    [sym_set_dns_list] = ACTIONS(2029),
    [sym_get_ip_address] = ACTIONS(2029),
    [sym_set_ip_address] = ACTIONS(2029),
    [sym_get_url_list] = ACTIONS(2029),
    [sym_add_url_entry] = ACTIONS(2029),
    [sym_delete_url_entry] = ACTIONS(2029),
    [sym_abs_value] = ACTIONS(2029),
    [sym_max_value] = ACTIONS(2029),
    [sym_min_value] = ACTIONS(2029),
    [sym_exp_value] = ACTIONS(2029),
    [sym_log_value] = ACTIONS(2029),
    [sym_log10_value] = ACTIONS(2029),
    [sym_power_value] = ACTIONS(2029),
    [sym_sqrt_value] = ACTIONS(2029),
    [sym_raw_be] = ACTIONS(2029),
    [sym_raw_le] = ACTIONS(2029),
    [sym_internal_queue_size_set] = ACTIONS(2029),
    [sym_internal_queue_size_get] = ACTIONS(2029),
    [sym_internal_threshold_set] = ACTIONS(2029),
    [sym_internal_threshold_get] = ACTIONS(2029),
    [sym_rebuilt_event] = ACTIONS(2029),
    [sym_do_custom_event] = ACTIONS(2029),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2029),
    [sym_clkmgr_set_clk_source] = ACTIONS(2029),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2029),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2029),
    [sym_clkmgr_get_timezone] = ACTIONS(2029),
    [sym_clkmgr_set_timezone] = ACTIONS(2029),
    [sym_clkmgr_get_resync_period] = ACTIONS(2029),
    [sym_clkmgr_set_resync_period] = ACTIONS(2029),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2029),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2029),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2029),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2029),
    [sym_clkmgr_get_timeservers] = ACTIONS(2029),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2029),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2029),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2029),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2029),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2029),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2029),
    [sym_set_log_level] = ACTIONS(2029),
    [sym_get_log_level] = ACTIONS(2029),
    [sym_amx_log] = ACTIONS(2029),
    [sym_smtp_server_config_set] = ACTIONS(2029),
    [sym_smtp_server_config_get] = ACTIONS(2029),
    [sym_smtp_send] = ACTIONS(2029),
    [sym_validate_netlinx_account] = ACTIONS(2029),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2029),
    [sym_authenticate_certificate] = ACTIONS(2029),
    [sym_audit_netlinx_session_event] = ACTIONS(2029),
    [sym_audit_netlinx_generic_event] = ACTIONS(2029),
    [sym_get_available_flash_disk_space] = ACTIONS(2029),
    [sym_get_max_flash_disk_space] = ACTIONS(2029),
    [sym_data_create_feed] = ACTIONS(2029),
    [sym_data_delete_feed] = ACTIONS(2029),
    [sym_data_publish_feed] = ACTIONS(2029),
    [sym_data_get_published_feed] = ACTIONS(2029),
    [sym_data_add_record] = ACTIONS(2029),
    [sym_data_get_event_record] = ACTIONS(2029),
    [sym__wc_data_create_feed] = ACTIONS(2029),
    [sym__wc_data_add_record] = ACTIONS(2029),
    [sym__wc_data_get_event_record] = ACTIONS(2029),
    [sym_format] = ACTIONS(2029),
    [sym_astro_clock] = ACTIONS(2029),
    [sym_set_outdoor_temperature] = ACTIONS(2029),
    [sym_set_virtual_level_count] = ACTIONS(2029),
    [sym_set_virtual_channel_count] = ACTIONS(2029),
    [sym_set_virtual_port_count] = ACTIONS(2029),
    [sym__wc_encode_int] = ACTIONS(2029),
    [sym__wc_decode_int] = ACTIONS(2029),
    [sym__wc_int] = ACTIONS(2029),
    [sym__wc_to_ch_int] = ACTIONS(2029),
    [sym__ch_to_wc_int] = ACTIONS(2029),
    [sym__wc_find_string_int] = ACTIONS(2029),
    [sym__wc_left_string_int] = ACTIONS(2029),
    [sym__wc_lower_string_int] = ACTIONS(2029),
    [sym__wc_mid_string_int] = ACTIONS(2029),
    [sym__wc_remove_string_int] = ACTIONS(2029),
    [sym__wc_right_string_int] = ACTIONS(2029),
    [sym__wc_upper_string_int] = ACTIONS(2029),
    [sym__wc_compare_string_int] = ACTIONS(2029),
    [sym__wc_get_buffer_char_int] = ACTIONS(2029),
    [sym__wc_get_buffer_string_int] = ACTIONS(2029),
    [sym__wc_concat_string_int] = ACTIONS(2029),
    [sym__wc_file_open_int] = ACTIONS(2029),
    [sym__wc_file_close_int] = ACTIONS(2029),
    [sym__wc_file_read_int] = ACTIONS(2029),
    [sym__wc_file_read_line_int] = ACTIONS(2029),
    [sym__wc_file_write_int] = ACTIONS(2029),
    [sym__wc_file_write_line_int] = ACTIONS(2029),
    [aux_sym_preproc_include_token1] = ACTIONS(2029),
    [aux_sym_preproc_define_token1] = ACTIONS(2031),
    [aux_sym_preproc_warn_token1] = ACTIONS(2031),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2031),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2031),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2031),
    [anon_sym_LBRACK] = ACTIONS(2031),
    [anon_sym_LBRACE] = ACTIONS(2031),
    [anon_sym_LPAREN] = ACTIONS(2029),
    [anon_sym_BANG] = ACTIONS(2031),
    [anon_sym_TILDE] = ACTIONS(2031),
    [anon_sym_DASH] = ACTIONS(2029),
    [anon_sym_PLUS] = ACTIONS(2029),
    [anon_sym_DASH_DASH] = ACTIONS(2031),
    [anon_sym_PLUS_PLUS] = ACTIONS(2031),
    [anon_sym_DQUOTE] = ACTIONS(2031),
    [anon_sym_SQUOTE] = ACTIONS(2031),
    [sym_number_literal] = ACTIONS(2031),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(420)] = {
    [ts_builtin_sym_end] = ACTIONS(2115),
    [sym_identifier] = ACTIONS(2113),
    [sym_program_name_keyword] = ACTIONS(2113),
    [sym_module_name_keyword] = ACTIONS(2113),
    [sym_define_device_keyword] = ACTIONS(2113),
    [sym_define_combine_keyword] = ACTIONS(2113),
    [sym_define_constant_keyword] = ACTIONS(2113),
    [sym_define_type_keyword] = ACTIONS(2113),
    [sym_define_variable_keyword] = ACTIONS(2113),
    [sym_define_system_variable_keyword] = ACTIONS(2113),
    [sym_define_connect_level_keyword] = ACTIONS(2113),
    [sym_define_latching_keyword] = ACTIONS(2113),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2113),
    [sym_define_toggling_keyword] = ACTIONS(2113),
    [sym_define_call_keyword] = ACTIONS(2113),
    [sym_define_function_keyword] = ACTIONS(2113),
    [sym_define_library_function_keyword] = ACTIONS(2113),
    [sym_define_start_keyword] = ACTIONS(2113),
    [sym_define_event_keyword] = ACTIONS(2113),
    [sym_define_program_keyword] = ACTIONS(2113),
    [sym_define_module_keyword] = ACTIONS(2113),
    [sym_char_keyword] = ACTIONS(2113),
    [sym_widechar_keyword] = ACTIONS(2113),
    [sym_integer_keyword] = ACTIONS(2113),
    [sym_sinteger_keyword] = ACTIONS(2113),
    [sym_long_keyword] = ACTIONS(2113),
    [sym_slong_keyword] = ACTIONS(2113),
    [sym_float_keyword] = ACTIONS(2113),
    [sym_double_keyword] = ACTIONS(2113),
    [sym_variant_keyword] = ACTIONS(2113),
    [sym_variantarray_keyword] = ACTIONS(2113),
    [sym_local_var_keyword] = ACTIONS(2113),
    [sym_stack_var_keyword] = ACTIONS(2113),
    [sym_constant_keyword] = ACTIONS(2113),
    [sym_volatile_keyword] = ACTIONS(2113),
    [sym_non_volatile_keyword] = ACTIONS(2113),
    [sym_persistent_keyword] = ACTIONS(2113),
    [sym_dev_keyword] = ACTIONS(2113),
    [sym_devlev_keyword] = ACTIONS(2113),
    [sym_devchan_keyword] = ACTIONS(2113),
    [sym_if_keyword] = ACTIONS(2113),
    [sym_switch_keyword] = ACTIONS(2113),
    [sym_case_keyword] = ACTIONS(2113),
    [sym_default_keyword] = ACTIONS(2113),
    [sym_while_keyword] = ACTIONS(2113),
    [sym_for_keyword] = ACTIONS(2113),
    [sym_break_keyword] = ACTIONS(2113),
    [sym_continue_keyword] = ACTIONS(2113),
    [sym_return_keyword] = ACTIONS(2113),
    [sym_select_keyword] = ACTIONS(2113),
    [sym_active_keyword] = ACTIONS(2113),
    [sym_struct_keyword] = ACTIONS(2113),
    [sym_structure_keyword] = ACTIONS(2113),
    [sym_button_event_keyword] = ACTIONS(2113),
    [sym_channel_event_keyword] = ACTIONS(2113),
    [sym_level_event_keyword] = ACTIONS(2113),
    [sym_data_event_keyword] = ACTIONS(2113),
    [sym_timeline_event_keyword] = ACTIONS(2113),
    [sym_custom_event_keyword] = ACTIONS(2113),
    [sym_push_keyword] = ACTIONS(2113),
    [sym_release_keyword] = ACTIONS(2113),
    [aux_sym_on_keyword_token1] = ACTIONS(2113),
    [aux_sym_off_keyword_token1] = ACTIONS(2113),
    [sym_send_level_keyword] = ACTIONS(2113),
    [sym_send_string_keyword] = ACTIONS(2113),
    [sym_send_command_keyword] = ACTIONS(2113),
    [sym_clear_buffer_keyword] = ACTIONS(2113),
    [sym_create_buffer_keyword] = ACTIONS(2113),
    [sym_create_multi_buffer_keyword] = ACTIONS(2113),
    [sym_call_keyword] = ACTIONS(2113),
    [sym_system_call_keyword] = ACTIONS(2113),
    [sym_devchan_to_keyword] = ACTIONS(2113),
    [sym_devchan_min_to_keyword] = ACTIONS(2113),
    [sym_devchan_total_off_keyword] = ACTIONS(2113),
    [sym_devchan_pulse_keyword] = ACTIONS(2113),
    [sym_wait_keyword] = ACTIONS(2113),
    [sym_cancel_wait_keyword] = ACTIONS(2113),
    [sym_wait_until_keyword] = ACTIONS(2113),
    [sym_cancel_wait_until_keyword] = ACTIONS(2113),
    [sym_cancel_all_wait_keyword] = ACTIONS(2113),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2113),
    [sym_not] = ACTIONS(2113),
    [sym_bnot] = ACTIONS(2113),
    [sym___file__] = ACTIONS(2113),
    [sym___date__] = ACTIONS(2113),
    [sym___ldate__] = ACTIONS(2113),
    [sym___time__] = ACTIONS(2113),
    [sym___line__] = ACTIONS(2113),
    [sym___name__] = ACTIONS(2113),
    [sym___netlinx__] = ACTIONS(2113),
    [sym_netlinx_axi_version] = ACTIONS(2113),
    [sym_url_flg_tcp] = ACTIONS(2113),
    [sym_url_flg_accinfopresent] = ACTIONS(2113),
    [sym_url_flg_temp] = ACTIONS(2113),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2113),
    [sym_url_flg_stat_mask] = ACTIONS(2113),
    [sym_url_flg_stat_lookup] = ACTIONS(2113),
    [sym_url_flg_stat_connecting] = ACTIONS(2113),
    [sym_url_flg_stat_waiting] = ACTIONS(2113),
    [sym_url_flg_stat_connected] = ACTIONS(2113),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2113),
    [sym_first_virtual_device] = ACTIONS(2113),
    [sym_first_local_port] = ACTIONS(2113),
    [sym_dynamic_virtual_device] = ACTIONS(2113),
    [sym_timeline_once] = ACTIONS(2113),
    [sym_timeline_repeat] = ACTIONS(2113),
    [sym_timeline_absolute] = ACTIONS(2113),
    [sym_timeline_relative] = ACTIONS(2113),
    [sym_do_push_timed_infinite] = ACTIONS(2113),
    [sym_source_type_no_address] = ACTIONS(2113),
    [sym_source_type_neuron_id] = ACTIONS(2113),
    [sym_source_type_ip_address] = ACTIONS(2113),
    [sym_source_type_axlink] = ACTIONS(2113),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2113),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2113),
    [sym_source_type_ip_socket_address] = ACTIONS(2113),
    [sym_source_type_rs232] = ACTIONS(2113),
    [sym_source_type_internal] = ACTIONS(2113),
    [sym_source_type_mac_address] = ACTIONS(2113),
    [sym_source_type_ipv4_port] = ACTIONS(2113),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2113),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2113),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2113),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2113),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2113),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2113),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2113),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2113),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2113),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2113),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2113),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2113),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2113),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2113),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2113),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2113),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2113),
    [sym_true] = ACTIONS(2113),
    [sym_false] = ACTIONS(2113),
    [sym_file_read_only] = ACTIONS(2113),
    [sym_file_rw_new] = ACTIONS(2113),
    [sym_file_rw_append] = ACTIONS(2113),
    [sym_ip_tcp] = ACTIONS(2113),
    [sym_ip_udp] = ACTIONS(2113),
    [sym_ip_udp_2way] = ACTIONS(2113),
    [sym_xml_encode_types] = ACTIONS(2113),
    [sym_xml_encode_char_as_list] = ACTIONS(2113),
    [sym_xml_encode_le] = ACTIONS(2113),
    [sym_xml_decode_types] = ACTIONS(2113),
    [sym_xml_decode_no_preserve] = ACTIONS(2113),
    [sym_duet_dev_type_display_device] = ACTIONS(2113),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2113),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2113),
    [sym_duet_dev_type_io_device] = ACTIONS(2113),
    [sym_duet_dev_type_relay_device] = ACTIONS(2113),
    [sym_duet_dev_type_ups] = ACTIONS(2113),
    [sym_duet_dev_type_amplifier] = ACTIONS(2113),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2113),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2113),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2113),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2113),
    [sym_duet_dev_type_camera] = ACTIONS(2113),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2113),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2113),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2113),
    [sym_duet_dev_type_dss] = ACTIONS(2113),
    [sym_duet_dev_type_dvr] = ACTIONS(2113),
    [sym_duet_dev_type_disc_device] = ACTIONS(2113),
    [sym_duet_dev_type_document_camera] = ACTIONS(2113),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2113),
    [sym_duet_dev_type_hvac] = ACTIONS(2113),
    [sym_duet_dev_type_keypad] = ACTIONS(2113),
    [sym_duet_dev_type_light] = ACTIONS(2113),
    [sym_duet_dev_type_monitor] = ACTIONS(2113),
    [sym_duet_dev_type_motor] = ACTIONS(2113),
    [sym_duet_dev_type_multi_window] = ACTIONS(2113),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2113),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2113),
    [sym_duet_dev_type_receiver] = ACTIONS(2113),
    [sym_duet_dev_type_security_system] = ACTIONS(2113),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2113),
    [sym_duet_dev_type_settop_box] = ACTIONS(2113),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2113),
    [sym_duet_dev_type_switcher] = ACTIONS(2113),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2113),
    [sym_duet_dev_type_tv] = ACTIONS(2113),
    [sym_duet_dev_type_utility] = ACTIONS(2113),
    [sym_duet_dev_type_vcr] = ACTIONS(2113),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2113),
    [sym_duet_dev_type_video_processor] = ACTIONS(2113),
    [sym_duet_dev_type_video_projector] = ACTIONS(2113),
    [sym_duet_dev_type_video_wall] = ACTIONS(2113),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2113),
    [sym_duet_dev_type_weather] = ACTIONS(2113),
    [sym_duet_dev_not_polled] = ACTIONS(2113),
    [sym_duet_dev_polled] = ACTIONS(2113),
    [sym_clkmgr_mode_network] = ACTIONS(2113),
    [sym_clkmgr_mode_standalone] = ACTIONS(2113),
    [sym_normal_standby] = ACTIONS(2113),
    [sym_normal_wake] = ACTIONS(2113),
    [sym_ip_multicast_ttl_option] = ACTIONS(2113),
    [sym_ip_ttl_subnet] = ACTIONS(2113),
    [sym_ip_ttl_site] = ACTIONS(2113),
    [sym_ip_ttl_region] = ACTIONS(2113),
    [sym_ip_ttl_continent] = ACTIONS(2113),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2113),
    [sym_ip_nodelay_on] = ACTIONS(2113),
    [sym_ip_nodelay_off] = ACTIONS(2113),
    [sym_amx_error] = ACTIONS(2113),
    [sym_amx_warning] = ACTIONS(2113),
    [sym_amx_info] = ACTIONS(2113),
    [sym_amx_debug] = ACTIONS(2113),
    [sym_netlinx_login_success] = ACTIONS(2113),
    [sym_netlinx_login_fail] = ACTIONS(2113),
    [sym_netlinx_logout] = ACTIONS(2113),
    [sym_null_str] = ACTIONS(2113),
    [sym_smtp_address] = ACTIONS(2113),
    [sym_smtp_port_number] = ACTIONS(2113),
    [sym_smtp_username] = ACTIONS(2113),
    [sym_smtp_password] = ACTIONS(2113),
    [sym_smtp_require_tls] = ACTIONS(2113),
    [sym_smtp_from] = ACTIONS(2113),
    [sym_smtp_tls_true] = ACTIONS(2113),
    [sym_smtp_tls_false] = ACTIONS(2113),
    [sym_listview_on_row_select_event] = ACTIONS(2113),
    [sym_data_structure_unknown] = ACTIONS(2113),
    [sym_data_structure_datafeed] = ACTIONS(2113),
    [sym_data_structure_datarecordset] = ACTIONS(2113),
    [sym_data_structure_datarecord] = ACTIONS(2113),
    [sym_data_structure_datafield] = ACTIONS(2113),
    [sym_data_max_content_field_count] = ACTIONS(2113),
    [sym_data_max_metadata_field_count] = ACTIONS(2113),
    [sym_data_max_id_length] = ACTIONS(2113),
    [sym_data_max_name_length] = ACTIONS(2113),
    [sym_data_max_description_length] = ACTIONS(2113),
    [sym_data_max_type_length] = ACTIONS(2113),
    [sym_data_max_format_length] = ACTIONS(2113),
    [sym_data_max_source_length] = ACTIONS(2113),
    [sym_data_max_value_length] = ACTIONS(2113),
    [sym_data_max_label_length] = ACTIONS(2113),
    [sym_data_max_url_length] = ACTIONS(2113),
    [sym_max_last_login_info_length] = ACTIONS(2113),
    [sym_data_type_unknown] = ACTIONS(2113),
    [sym_data_type_string] = ACTIONS(2113),
    [sym_data_type_datetime] = ACTIONS(2113),
    [sym_data_type_date] = ACTIONS(2113),
    [sym_data_type_time] = ACTIONS(2113),
    [sym_data_type_image] = ACTIONS(2113),
    [sym_data_format_url] = ACTIONS(2113),
    [sym_data_format_phone] = ACTIONS(2113),
    [sym_data_format_email] = ACTIONS(2113),
    [sym_data_format_iso8601] = ACTIONS(2113),
    [sym_auditlog_priv] = ACTIONS(2113),
    [sym_configuration_priv] = ACTIONS(2113),
    [sym_deviceconfig_priv] = ACTIONS(2113),
    [sym_ftp_priv] = ACTIONS(2113),
    [sym_http_priv] = ACTIONS(2113),
    [sym_networkconfig_priv] = ACTIONS(2113),
    [sym_programport_priv] = ACTIONS(2113),
    [sym_remoteui_priv] = ACTIONS(2113),
    [sym_securitycontrol_priv] = ACTIONS(2113),
    [sym_softwaremanagement_priv] = ACTIONS(2113),
    [sym_terminal_priv] = ACTIONS(2113),
    [sym_tpadmin_priv] = ACTIONS(2113),
    [sym_user1_priv] = ACTIONS(2113),
    [sym_user2_priv] = ACTIONS(2113),
    [sym_user3_priv] = ACTIONS(2113),
    [sym_user4_priv] = ACTIONS(2113),
    [sym_usermanagement_priv] = ACTIONS(2113),
    [sym_valid_account] = ACTIONS(2113),
    [sym_err_name_invalid] = ACTIONS(2113),
    [sym_err_password_invalid] = ACTIONS(2113),
    [sym_err_invalid_account] = ACTIONS(2113),
    [sym_err_no_authorization] = ACTIONS(2113),
    [sym_err_invalid_parameter] = ACTIONS(2113),
    [sym_err_user_locked_out] = ACTIONS(2113),
    [sym_err_user_account_expired] = ACTIONS(2113),
    [sym_err_general_error] = ACTIONS(2113),
    [sym_err_authentication_error] = ACTIONS(2113),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2113),
    [sym_tls_validate_certificate] = ACTIONS(2113),
    [sym_tdata] = ACTIONS(2113),
    [sym_tchannel] = ACTIONS(2113),
    [sym_tlevel] = ACTIONS(2113),
    [sym_tbutton] = ACTIONS(2113),
    [sym_ttimeline] = ACTIONS(2113),
    [sym_tcustom] = ACTIONS(2113),
    [sym_url_struct] = ACTIONS(2113),
    [sym_dns_struct] = ACTIONS(2113),
    [sym_ip_address_struct] = ACTIONS(2113),
    [sym_dev_info_struct] = ACTIONS(2113),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2113),
    [sym_data_feed] = ACTIONS(2113),
    [sym_data_field] = ACTIONS(2113),
    [sym_data_record] = ACTIONS(2113),
    [sym_wc_data_feed] = ACTIONS(2113),
    [sym_wc_data_field] = ACTIONS(2113),
    [sym_wc_data_record] = ACTIONS(2113),
    [sym_last_login_info] = ACTIONS(2113),
    [sym____reserved___] = ACTIONS(2113),
    [sym_date] = ACTIONS(2113),
    [sym_day] = ACTIONS(2113),
    [sym_time] = ACTIONS(2113),
    [sym_ldate] = ACTIONS(2113),
    [sym_push_device] = ACTIONS(2113),
    [sym_push_channel] = ACTIONS(2113),
    [sym_push_devchan] = ACTIONS(2113),
    [sym_release_device] = ACTIONS(2113),
    [sym_release_channel] = ACTIONS(2113),
    [sym_release_devchan] = ACTIONS(2113),
    [sym_master_slot] = ACTIONS(2113),
    [sym_get_pulse_time] = ACTIONS(2113),
    [sym_get_timer] = ACTIONS(2113),
    [sym_master_sn] = ACTIONS(2113),
    [sym_system_number] = ACTIONS(2113),
    [sym_button] = ACTIONS(2113),
    [sym_dv_channel] = ACTIONS(2113),
    [sym_data] = ACTIONS(2113),
    [sym_level] = ACTIONS(2113),
    [sym_timeline] = ACTIONS(2113),
    [sym_channel] = ACTIONS(2113),
    [sym_custom] = ACTIONS(2113),
    [sym_atoi] = ACTIONS(2113),
    [sym_atol] = ACTIONS(2113),
    [sym_atof] = ACTIONS(2113),
    [sym_itoa] = ACTIONS(2113),
    [sym_itohex] = ACTIONS(2113),
    [sym_hextoi] = ACTIONS(2113),
    [sym_ftoa] = ACTIONS(2113),
    [sym_find_string] = ACTIONS(2113),
    [sym_left_string] = ACTIONS(2113),
    [sym_right_string] = ACTIONS(2113),
    [sym_mid_string] = ACTIONS(2113),
    [sym_remove_string] = ACTIONS(2113),
    [sym_get_buffer_string] = ACTIONS(2113),
    [sym_get_buffer_char] = ACTIONS(2113),
    [sym_get_multi_buffer_string] = ACTIONS(2113),
    [sym_length_string] = ACTIONS(2113),
    [sym_max_length_string] = ACTIONS(2113),
    [sym_set_length_string] = ACTIONS(2113),
    [sym_compare_string] = ACTIONS(2113),
    [sym_upper_string] = ACTIONS(2113),
    [sym_lower_string] = ACTIONS(2113),
    [sym_get_last] = ACTIONS(2113),
    [sym_date_to_day] = ACTIONS(2113),
    [sym_date_to_month] = ACTIONS(2113),
    [sym_date_to_year] = ACTIONS(2113),
    [sym_time_to_hour] = ACTIONS(2113),
    [sym_time_to_minute] = ACTIONS(2113),
    [sym_time_to_second] = ACTIONS(2113),
    [sym_day_of_week] = ACTIONS(2113),
    [sym_set_pulse_time] = ACTIONS(2113),
    [sym_set_timer] = ACTIONS(2113),
    [sym_file_open] = ACTIONS(2113),
    [sym_file_close] = ACTIONS(2113),
    [sym_file_read] = ACTIONS(2113),
    [sym_file_read_line] = ACTIONS(2113),
    [sym_file_write] = ACTIONS(2113),
    [sym_file_write_line] = ACTIONS(2113),
    [sym_file_copy] = ACTIONS(2113),
    [sym_file_delete] = ACTIONS(2113),
    [sym_file_rename] = ACTIONS(2113),
    [sym_file_dir] = ACTIONS(2113),
    [sym_file_getdir] = ACTIONS(2113),
    [sym_file_setdir] = ACTIONS(2113),
    [sym_file_create_dir] = ACTIONS(2113),
    [sym_file_remove_dir] = ACTIONS(2113),
    [sym_file_seek] = ACTIONS(2113),
    [sym_length_array] = ACTIONS(2113),
    [sym_max_length_array] = ACTIONS(2113),
    [sym_set_length_array] = ACTIONS(2113),
    [sym_ip_client_open] = ACTIONS(2113),
    [sym_ip_client_close] = ACTIONS(2113),
    [sym_ip_server_open] = ACTIONS(2113),
    [sym_ip_server_close] = ACTIONS(2113),
    [sym_ip_bound_client_open] = ACTIONS(2113),
    [sym_ip_mc_server_open] = ACTIONS(2113),
    [sym_ip_set_option] = ACTIONS(2113),
    [sym_ssh_client_open] = ACTIONS(2113),
    [sym_ssh_client_close] = ACTIONS(2113),
    [sym_tls_client_open] = ACTIONS(2113),
    [sym_tls_client_close] = ACTIONS(2113),
    [sym_do_push] = ACTIONS(2113),
    [sym_do_push_timed] = ACTIONS(2113),
    [sym_do_release] = ACTIONS(2113),
    [sym_device_id] = ACTIONS(2113),
    [sym_device_info] = ACTIONS(2113),
    [sym_device_id_string] = ACTIONS(2113),
    [sym_device_standby] = ACTIONS(2113),
    [sym_device_wake] = ACTIONS(2113),
    [sym_timeline_create] = ACTIONS(2113),
    [sym_timeline_kill] = ACTIONS(2113),
    [sym_timeline_reload] = ACTIONS(2113),
    [sym_timeline_pause] = ACTIONS(2113),
    [sym_timeline_restart] = ACTIONS(2113),
    [sym_timeline_set] = ACTIONS(2113),
    [sym_timeline_get] = ACTIONS(2113),
    [sym_timeline_active] = ACTIONS(2113),
    [sym_timeline_dynamic_id] = ACTIONS(2113),
    [sym_variable_to_string] = ACTIONS(2113),
    [sym_string_to_variable] = ACTIONS(2113),
    [sym_length_variable_to_string] = ACTIONS(2113),
    [sym_type_cast] = ACTIONS(2113),
    [sym_variable_to_xml] = ACTIONS(2113),
    [sym_xml_to_variable] = ACTIONS(2113),
    [sym_length_variable_to_xml] = ACTIONS(2113),
    [sym_redirect_string] = ACTIONS(2113),
    [sym_random_number] = ACTIONS(2113),
    [sym_get_unique_id] = ACTIONS(2113),
    [sym_get_system_number] = ACTIONS(2113),
    [sym_set_system_number] = ACTIONS(2113),
    [sym_get_serial_number] = ACTIONS(2113),
    [sym_get_master_build] = ACTIONS(2113),
    [sym_set_validation_code] = ACTIONS(2113),
    [sym_reboot] = ACTIONS(2113),
    [sym_load_duet_module] = ACTIONS(2113),
    [sym_unload_duet_module] = ACTIONS(2113),
    [sym_duet_mem_size_set] = ACTIONS(2113),
    [sym_duet_mem_size_get] = ACTIONS(2113),
    [sym_dynamic_application_device] = ACTIONS(2113),
    [sym_dynamic_polled_port] = ACTIONS(2113),
    [sym_static_port_binding] = ACTIONS(2113),
    [sym_static_ip_binding] = ACTIONS(2113),
    [sym_get_dns_list] = ACTIONS(2113),
    [sym_set_dns_list] = ACTIONS(2113),
    [sym_get_ip_address] = ACTIONS(2113),
    [sym_set_ip_address] = ACTIONS(2113),
    [sym_get_url_list] = ACTIONS(2113),
    [sym_add_url_entry] = ACTIONS(2113),
    [sym_delete_url_entry] = ACTIONS(2113),
    [sym_abs_value] = ACTIONS(2113),
    [sym_max_value] = ACTIONS(2113),
    [sym_min_value] = ACTIONS(2113),
    [sym_exp_value] = ACTIONS(2113),
    [sym_log_value] = ACTIONS(2113),
    [sym_log10_value] = ACTIONS(2113),
    [sym_power_value] = ACTIONS(2113),
    [sym_sqrt_value] = ACTIONS(2113),
    [sym_raw_be] = ACTIONS(2113),
    [sym_raw_le] = ACTIONS(2113),
    [sym_internal_queue_size_set] = ACTIONS(2113),
    [sym_internal_queue_size_get] = ACTIONS(2113),
    [sym_internal_threshold_set] = ACTIONS(2113),
    [sym_internal_threshold_get] = ACTIONS(2113),
    [sym_rebuilt_event] = ACTIONS(2113),
    [sym_do_custom_event] = ACTIONS(2113),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2113),
    [sym_clkmgr_set_clk_source] = ACTIONS(2113),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2113),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2113),
    [sym_clkmgr_get_timezone] = ACTIONS(2113),
    [sym_clkmgr_set_timezone] = ACTIONS(2113),
    [sym_clkmgr_get_resync_period] = ACTIONS(2113),
    [sym_clkmgr_set_resync_period] = ACTIONS(2113),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2113),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2113),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2113),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2113),
    [sym_clkmgr_get_timeservers] = ACTIONS(2113),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2113),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2113),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2113),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2113),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2113),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2113),
    [sym_set_log_level] = ACTIONS(2113),
    [sym_get_log_level] = ACTIONS(2113),
    [sym_amx_log] = ACTIONS(2113),
    [sym_smtp_server_config_set] = ACTIONS(2113),
    [sym_smtp_server_config_get] = ACTIONS(2113),
    [sym_smtp_send] = ACTIONS(2113),
    [sym_validate_netlinx_account] = ACTIONS(2113),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2113),
    [sym_authenticate_certificate] = ACTIONS(2113),
    [sym_audit_netlinx_session_event] = ACTIONS(2113),
    [sym_audit_netlinx_generic_event] = ACTIONS(2113),
    [sym_get_available_flash_disk_space] = ACTIONS(2113),
    [sym_get_max_flash_disk_space] = ACTIONS(2113),
    [sym_data_create_feed] = ACTIONS(2113),
    [sym_data_delete_feed] = ACTIONS(2113),
    [sym_data_publish_feed] = ACTIONS(2113),
    [sym_data_get_published_feed] = ACTIONS(2113),
    [sym_data_add_record] = ACTIONS(2113),
    [sym_data_get_event_record] = ACTIONS(2113),
    [sym__wc_data_create_feed] = ACTIONS(2113),
    [sym__wc_data_add_record] = ACTIONS(2113),
    [sym__wc_data_get_event_record] = ACTIONS(2113),
    [sym_format] = ACTIONS(2113),
    [sym_astro_clock] = ACTIONS(2113),
    [sym_set_outdoor_temperature] = ACTIONS(2113),
    [sym_set_virtual_level_count] = ACTIONS(2113),
    [sym_set_virtual_channel_count] = ACTIONS(2113),
    [sym_set_virtual_port_count] = ACTIONS(2113),
    [sym__wc_encode_int] = ACTIONS(2113),
    [sym__wc_decode_int] = ACTIONS(2113),
    [sym__wc_int] = ACTIONS(2113),
    [sym__wc_to_ch_int] = ACTIONS(2113),
    [sym__ch_to_wc_int] = ACTIONS(2113),
    [sym__wc_find_string_int] = ACTIONS(2113),
    [sym__wc_left_string_int] = ACTIONS(2113),
    [sym__wc_lower_string_int] = ACTIONS(2113),
    [sym__wc_mid_string_int] = ACTIONS(2113),
    [sym__wc_remove_string_int] = ACTIONS(2113),
    [sym__wc_right_string_int] = ACTIONS(2113),
    [sym__wc_upper_string_int] = ACTIONS(2113),
    [sym__wc_compare_string_int] = ACTIONS(2113),
    [sym__wc_get_buffer_char_int] = ACTIONS(2113),
    [sym__wc_get_buffer_string_int] = ACTIONS(2113),
    [sym__wc_concat_string_int] = ACTIONS(2113),
    [sym__wc_file_open_int] = ACTIONS(2113),
    [sym__wc_file_close_int] = ACTIONS(2113),
    [sym__wc_file_read_int] = ACTIONS(2113),
    [sym__wc_file_read_line_int] = ACTIONS(2113),
    [sym__wc_file_write_int] = ACTIONS(2113),
    [sym__wc_file_write_line_int] = ACTIONS(2113),
    [aux_sym_preproc_include_token1] = ACTIONS(2113),
    [aux_sym_preproc_define_token1] = ACTIONS(2115),
    [aux_sym_preproc_warn_token1] = ACTIONS(2115),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2115),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2115),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2115),
    [anon_sym_LBRACK] = ACTIONS(2115),
    [anon_sym_LBRACE] = ACTIONS(2115),
    [anon_sym_LPAREN] = ACTIONS(2113),
    [anon_sym_BANG] = ACTIONS(2115),
    [anon_sym_TILDE] = ACTIONS(2115),
    [anon_sym_DASH] = ACTIONS(2113),
    [anon_sym_PLUS] = ACTIONS(2113),
    [anon_sym_DASH_DASH] = ACTIONS(2115),
    [anon_sym_PLUS_PLUS] = ACTIONS(2115),
    [anon_sym_DQUOTE] = ACTIONS(2115),
    [anon_sym_SQUOTE] = ACTIONS(2115),
    [sym_number_literal] = ACTIONS(2115),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(421)] = {
    [ts_builtin_sym_end] = ACTIONS(2059),
    [sym_identifier] = ACTIONS(2057),
    [sym_program_name_keyword] = ACTIONS(2057),
    [sym_module_name_keyword] = ACTIONS(2057),
    [sym_define_device_keyword] = ACTIONS(2057),
    [sym_define_combine_keyword] = ACTIONS(2057),
    [sym_define_constant_keyword] = ACTIONS(2057),
    [sym_define_type_keyword] = ACTIONS(2057),
    [sym_define_variable_keyword] = ACTIONS(2057),
    [sym_define_system_variable_keyword] = ACTIONS(2057),
    [sym_define_connect_level_keyword] = ACTIONS(2057),
    [sym_define_latching_keyword] = ACTIONS(2057),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2057),
    [sym_define_toggling_keyword] = ACTIONS(2057),
    [sym_define_call_keyword] = ACTIONS(2057),
    [sym_define_function_keyword] = ACTIONS(2057),
    [sym_define_library_function_keyword] = ACTIONS(2057),
    [sym_define_start_keyword] = ACTIONS(2057),
    [sym_define_event_keyword] = ACTIONS(2057),
    [sym_define_program_keyword] = ACTIONS(2057),
    [sym_define_module_keyword] = ACTIONS(2057),
    [sym_char_keyword] = ACTIONS(2057),
    [sym_widechar_keyword] = ACTIONS(2057),
    [sym_integer_keyword] = ACTIONS(2057),
    [sym_sinteger_keyword] = ACTIONS(2057),
    [sym_long_keyword] = ACTIONS(2057),
    [sym_slong_keyword] = ACTIONS(2057),
    [sym_float_keyword] = ACTIONS(2057),
    [sym_double_keyword] = ACTIONS(2057),
    [sym_variant_keyword] = ACTIONS(2057),
    [sym_variantarray_keyword] = ACTIONS(2057),
    [sym_local_var_keyword] = ACTIONS(2057),
    [sym_stack_var_keyword] = ACTIONS(2057),
    [sym_constant_keyword] = ACTIONS(2057),
    [sym_volatile_keyword] = ACTIONS(2057),
    [sym_non_volatile_keyword] = ACTIONS(2057),
    [sym_persistent_keyword] = ACTIONS(2057),
    [sym_dev_keyword] = ACTIONS(2057),
    [sym_devlev_keyword] = ACTIONS(2057),
    [sym_devchan_keyword] = ACTIONS(2057),
    [sym_if_keyword] = ACTIONS(2057),
    [sym_switch_keyword] = ACTIONS(2057),
    [sym_case_keyword] = ACTIONS(2057),
    [sym_default_keyword] = ACTIONS(2057),
    [sym_while_keyword] = ACTIONS(2057),
    [sym_for_keyword] = ACTIONS(2057),
    [sym_break_keyword] = ACTIONS(2057),
    [sym_continue_keyword] = ACTIONS(2057),
    [sym_return_keyword] = ACTIONS(2057),
    [sym_select_keyword] = ACTIONS(2057),
    [sym_active_keyword] = ACTIONS(2057),
    [sym_struct_keyword] = ACTIONS(2057),
    [sym_structure_keyword] = ACTIONS(2057),
    [sym_button_event_keyword] = ACTIONS(2057),
    [sym_channel_event_keyword] = ACTIONS(2057),
    [sym_level_event_keyword] = ACTIONS(2057),
    [sym_data_event_keyword] = ACTIONS(2057),
    [sym_timeline_event_keyword] = ACTIONS(2057),
    [sym_custom_event_keyword] = ACTIONS(2057),
    [sym_push_keyword] = ACTIONS(2057),
    [sym_release_keyword] = ACTIONS(2057),
    [aux_sym_on_keyword_token1] = ACTIONS(2057),
    [aux_sym_off_keyword_token1] = ACTIONS(2057),
    [sym_send_level_keyword] = ACTIONS(2057),
    [sym_send_string_keyword] = ACTIONS(2057),
    [sym_send_command_keyword] = ACTIONS(2057),
    [sym_clear_buffer_keyword] = ACTIONS(2057),
    [sym_create_buffer_keyword] = ACTIONS(2057),
    [sym_create_multi_buffer_keyword] = ACTIONS(2057),
    [sym_call_keyword] = ACTIONS(2057),
    [sym_system_call_keyword] = ACTIONS(2057),
    [sym_devchan_to_keyword] = ACTIONS(2057),
    [sym_devchan_min_to_keyword] = ACTIONS(2057),
    [sym_devchan_total_off_keyword] = ACTIONS(2057),
    [sym_devchan_pulse_keyword] = ACTIONS(2057),
    [sym_wait_keyword] = ACTIONS(2057),
    [sym_cancel_wait_keyword] = ACTIONS(2057),
    [sym_wait_until_keyword] = ACTIONS(2057),
    [sym_cancel_wait_until_keyword] = ACTIONS(2057),
    [sym_cancel_all_wait_keyword] = ACTIONS(2057),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2057),
    [sym_not] = ACTIONS(2057),
    [sym_bnot] = ACTIONS(2057),
    [sym___file__] = ACTIONS(2057),
    [sym___date__] = ACTIONS(2057),
    [sym___ldate__] = ACTIONS(2057),
    [sym___time__] = ACTIONS(2057),
    [sym___line__] = ACTIONS(2057),
    [sym___name__] = ACTIONS(2057),
    [sym___netlinx__] = ACTIONS(2057),
    [sym_netlinx_axi_version] = ACTIONS(2057),
    [sym_url_flg_tcp] = ACTIONS(2057),
    [sym_url_flg_accinfopresent] = ACTIONS(2057),
    [sym_url_flg_temp] = ACTIONS(2057),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2057),
    [sym_url_flg_stat_mask] = ACTIONS(2057),
    [sym_url_flg_stat_lookup] = ACTIONS(2057),
    [sym_url_flg_stat_connecting] = ACTIONS(2057),
    [sym_url_flg_stat_waiting] = ACTIONS(2057),
    [sym_url_flg_stat_connected] = ACTIONS(2057),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2057),
    [sym_first_virtual_device] = ACTIONS(2057),
    [sym_first_local_port] = ACTIONS(2057),
    [sym_dynamic_virtual_device] = ACTIONS(2057),
    [sym_timeline_once] = ACTIONS(2057),
    [sym_timeline_repeat] = ACTIONS(2057),
    [sym_timeline_absolute] = ACTIONS(2057),
    [sym_timeline_relative] = ACTIONS(2057),
    [sym_do_push_timed_infinite] = ACTIONS(2057),
    [sym_source_type_no_address] = ACTIONS(2057),
    [sym_source_type_neuron_id] = ACTIONS(2057),
    [sym_source_type_ip_address] = ACTIONS(2057),
    [sym_source_type_axlink] = ACTIONS(2057),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2057),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2057),
    [sym_source_type_ip_socket_address] = ACTIONS(2057),
    [sym_source_type_rs232] = ACTIONS(2057),
    [sym_source_type_internal] = ACTIONS(2057),
    [sym_source_type_mac_address] = ACTIONS(2057),
    [sym_source_type_ipv4_port] = ACTIONS(2057),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2057),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2057),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2057),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2057),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2057),
    [sym_true] = ACTIONS(2057),
    [sym_false] = ACTIONS(2057),
    [sym_file_read_only] = ACTIONS(2057),
    [sym_file_rw_new] = ACTIONS(2057),
    [sym_file_rw_append] = ACTIONS(2057),
    [sym_ip_tcp] = ACTIONS(2057),
    [sym_ip_udp] = ACTIONS(2057),
    [sym_ip_udp_2way] = ACTIONS(2057),
    [sym_xml_encode_types] = ACTIONS(2057),
    [sym_xml_encode_char_as_list] = ACTIONS(2057),
    [sym_xml_encode_le] = ACTIONS(2057),
    [sym_xml_decode_types] = ACTIONS(2057),
    [sym_xml_decode_no_preserve] = ACTIONS(2057),
    [sym_duet_dev_type_display_device] = ACTIONS(2057),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2057),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2057),
    [sym_duet_dev_type_io_device] = ACTIONS(2057),
    [sym_duet_dev_type_relay_device] = ACTIONS(2057),
    [sym_duet_dev_type_ups] = ACTIONS(2057),
    [sym_duet_dev_type_amplifier] = ACTIONS(2057),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2057),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2057),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2057),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2057),
    [sym_duet_dev_type_camera] = ACTIONS(2057),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2057),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2057),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2057),
    [sym_duet_dev_type_dss] = ACTIONS(2057),
    [sym_duet_dev_type_dvr] = ACTIONS(2057),
    [sym_duet_dev_type_disc_device] = ACTIONS(2057),
    [sym_duet_dev_type_document_camera] = ACTIONS(2057),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2057),
    [sym_duet_dev_type_hvac] = ACTIONS(2057),
    [sym_duet_dev_type_keypad] = ACTIONS(2057),
    [sym_duet_dev_type_light] = ACTIONS(2057),
    [sym_duet_dev_type_monitor] = ACTIONS(2057),
    [sym_duet_dev_type_motor] = ACTIONS(2057),
    [sym_duet_dev_type_multi_window] = ACTIONS(2057),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2057),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2057),
    [sym_duet_dev_type_receiver] = ACTIONS(2057),
    [sym_duet_dev_type_security_system] = ACTIONS(2057),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2057),
    [sym_duet_dev_type_settop_box] = ACTIONS(2057),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2057),
    [sym_duet_dev_type_switcher] = ACTIONS(2057),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2057),
    [sym_duet_dev_type_tv] = ACTIONS(2057),
    [sym_duet_dev_type_utility] = ACTIONS(2057),
    [sym_duet_dev_type_vcr] = ACTIONS(2057),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2057),
    [sym_duet_dev_type_video_processor] = ACTIONS(2057),
    [sym_duet_dev_type_video_projector] = ACTIONS(2057),
    [sym_duet_dev_type_video_wall] = ACTIONS(2057),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2057),
    [sym_duet_dev_type_weather] = ACTIONS(2057),
    [sym_duet_dev_not_polled] = ACTIONS(2057),
    [sym_duet_dev_polled] = ACTIONS(2057),
    [sym_clkmgr_mode_network] = ACTIONS(2057),
    [sym_clkmgr_mode_standalone] = ACTIONS(2057),
    [sym_normal_standby] = ACTIONS(2057),
    [sym_normal_wake] = ACTIONS(2057),
    [sym_ip_multicast_ttl_option] = ACTIONS(2057),
    [sym_ip_ttl_subnet] = ACTIONS(2057),
    [sym_ip_ttl_site] = ACTIONS(2057),
    [sym_ip_ttl_region] = ACTIONS(2057),
    [sym_ip_ttl_continent] = ACTIONS(2057),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2057),
    [sym_ip_nodelay_on] = ACTIONS(2057),
    [sym_ip_nodelay_off] = ACTIONS(2057),
    [sym_amx_error] = ACTIONS(2057),
    [sym_amx_warning] = ACTIONS(2057),
    [sym_amx_info] = ACTIONS(2057),
    [sym_amx_debug] = ACTIONS(2057),
    [sym_netlinx_login_success] = ACTIONS(2057),
    [sym_netlinx_login_fail] = ACTIONS(2057),
    [sym_netlinx_logout] = ACTIONS(2057),
    [sym_null_str] = ACTIONS(2057),
    [sym_smtp_address] = ACTIONS(2057),
    [sym_smtp_port_number] = ACTIONS(2057),
    [sym_smtp_username] = ACTIONS(2057),
    [sym_smtp_password] = ACTIONS(2057),
    [sym_smtp_require_tls] = ACTIONS(2057),
    [sym_smtp_from] = ACTIONS(2057),
    [sym_smtp_tls_true] = ACTIONS(2057),
    [sym_smtp_tls_false] = ACTIONS(2057),
    [sym_listview_on_row_select_event] = ACTIONS(2057),
    [sym_data_structure_unknown] = ACTIONS(2057),
    [sym_data_structure_datafeed] = ACTIONS(2057),
    [sym_data_structure_datarecordset] = ACTIONS(2057),
    [sym_data_structure_datarecord] = ACTIONS(2057),
    [sym_data_structure_datafield] = ACTIONS(2057),
    [sym_data_max_content_field_count] = ACTIONS(2057),
    [sym_data_max_metadata_field_count] = ACTIONS(2057),
    [sym_data_max_id_length] = ACTIONS(2057),
    [sym_data_max_name_length] = ACTIONS(2057),
    [sym_data_max_description_length] = ACTIONS(2057),
    [sym_data_max_type_length] = ACTIONS(2057),
    [sym_data_max_format_length] = ACTIONS(2057),
    [sym_data_max_source_length] = ACTIONS(2057),
    [sym_data_max_value_length] = ACTIONS(2057),
    [sym_data_max_label_length] = ACTIONS(2057),
    [sym_data_max_url_length] = ACTIONS(2057),
    [sym_max_last_login_info_length] = ACTIONS(2057),
    [sym_data_type_unknown] = ACTIONS(2057),
    [sym_data_type_string] = ACTIONS(2057),
    [sym_data_type_datetime] = ACTIONS(2057),
    [sym_data_type_date] = ACTIONS(2057),
    [sym_data_type_time] = ACTIONS(2057),
    [sym_data_type_image] = ACTIONS(2057),
    [sym_data_format_url] = ACTIONS(2057),
    [sym_data_format_phone] = ACTIONS(2057),
    [sym_data_format_email] = ACTIONS(2057),
    [sym_data_format_iso8601] = ACTIONS(2057),
    [sym_auditlog_priv] = ACTIONS(2057),
    [sym_configuration_priv] = ACTIONS(2057),
    [sym_deviceconfig_priv] = ACTIONS(2057),
    [sym_ftp_priv] = ACTIONS(2057),
    [sym_http_priv] = ACTIONS(2057),
    [sym_networkconfig_priv] = ACTIONS(2057),
    [sym_programport_priv] = ACTIONS(2057),
    [sym_remoteui_priv] = ACTIONS(2057),
    [sym_securitycontrol_priv] = ACTIONS(2057),
    [sym_softwaremanagement_priv] = ACTIONS(2057),
    [sym_terminal_priv] = ACTIONS(2057),
    [sym_tpadmin_priv] = ACTIONS(2057),
    [sym_user1_priv] = ACTIONS(2057),
    [sym_user2_priv] = ACTIONS(2057),
    [sym_user3_priv] = ACTIONS(2057),
    [sym_user4_priv] = ACTIONS(2057),
    [sym_usermanagement_priv] = ACTIONS(2057),
    [sym_valid_account] = ACTIONS(2057),
    [sym_err_name_invalid] = ACTIONS(2057),
    [sym_err_password_invalid] = ACTIONS(2057),
    [sym_err_invalid_account] = ACTIONS(2057),
    [sym_err_no_authorization] = ACTIONS(2057),
    [sym_err_invalid_parameter] = ACTIONS(2057),
    [sym_err_user_locked_out] = ACTIONS(2057),
    [sym_err_user_account_expired] = ACTIONS(2057),
    [sym_err_general_error] = ACTIONS(2057),
    [sym_err_authentication_error] = ACTIONS(2057),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2057),
    [sym_tls_validate_certificate] = ACTIONS(2057),
    [sym_tdata] = ACTIONS(2057),
    [sym_tchannel] = ACTIONS(2057),
    [sym_tlevel] = ACTIONS(2057),
    [sym_tbutton] = ACTIONS(2057),
    [sym_ttimeline] = ACTIONS(2057),
    [sym_tcustom] = ACTIONS(2057),
    [sym_url_struct] = ACTIONS(2057),
    [sym_dns_struct] = ACTIONS(2057),
    [sym_ip_address_struct] = ACTIONS(2057),
    [sym_dev_info_struct] = ACTIONS(2057),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2057),
    [sym_data_feed] = ACTIONS(2057),
    [sym_data_field] = ACTIONS(2057),
    [sym_data_record] = ACTIONS(2057),
    [sym_wc_data_feed] = ACTIONS(2057),
    [sym_wc_data_field] = ACTIONS(2057),
    [sym_wc_data_record] = ACTIONS(2057),
    [sym_last_login_info] = ACTIONS(2057),
    [sym____reserved___] = ACTIONS(2057),
    [sym_date] = ACTIONS(2057),
    [sym_day] = ACTIONS(2057),
    [sym_time] = ACTIONS(2057),
    [sym_ldate] = ACTIONS(2057),
    [sym_push_device] = ACTIONS(2057),
    [sym_push_channel] = ACTIONS(2057),
    [sym_push_devchan] = ACTIONS(2057),
    [sym_release_device] = ACTIONS(2057),
    [sym_release_channel] = ACTIONS(2057),
    [sym_release_devchan] = ACTIONS(2057),
    [sym_master_slot] = ACTIONS(2057),
    [sym_get_pulse_time] = ACTIONS(2057),
    [sym_get_timer] = ACTIONS(2057),
    [sym_master_sn] = ACTIONS(2057),
    [sym_system_number] = ACTIONS(2057),
    [sym_button] = ACTIONS(2057),
    [sym_dv_channel] = ACTIONS(2057),
    [sym_data] = ACTIONS(2057),
    [sym_level] = ACTIONS(2057),
    [sym_timeline] = ACTIONS(2057),
    [sym_channel] = ACTIONS(2057),
    [sym_custom] = ACTIONS(2057),
    [sym_atoi] = ACTIONS(2057),
    [sym_atol] = ACTIONS(2057),
    [sym_atof] = ACTIONS(2057),
    [sym_itoa] = ACTIONS(2057),
    [sym_itohex] = ACTIONS(2057),
    [sym_hextoi] = ACTIONS(2057),
    [sym_ftoa] = ACTIONS(2057),
    [sym_find_string] = ACTIONS(2057),
    [sym_left_string] = ACTIONS(2057),
    [sym_right_string] = ACTIONS(2057),
    [sym_mid_string] = ACTIONS(2057),
    [sym_remove_string] = ACTIONS(2057),
    [sym_get_buffer_string] = ACTIONS(2057),
    [sym_get_buffer_char] = ACTIONS(2057),
    [sym_get_multi_buffer_string] = ACTIONS(2057),
    [sym_length_string] = ACTIONS(2057),
    [sym_max_length_string] = ACTIONS(2057),
    [sym_set_length_string] = ACTIONS(2057),
    [sym_compare_string] = ACTIONS(2057),
    [sym_upper_string] = ACTIONS(2057),
    [sym_lower_string] = ACTIONS(2057),
    [sym_get_last] = ACTIONS(2057),
    [sym_date_to_day] = ACTIONS(2057),
    [sym_date_to_month] = ACTIONS(2057),
    [sym_date_to_year] = ACTIONS(2057),
    [sym_time_to_hour] = ACTIONS(2057),
    [sym_time_to_minute] = ACTIONS(2057),
    [sym_time_to_second] = ACTIONS(2057),
    [sym_day_of_week] = ACTIONS(2057),
    [sym_set_pulse_time] = ACTIONS(2057),
    [sym_set_timer] = ACTIONS(2057),
    [sym_file_open] = ACTIONS(2057),
    [sym_file_close] = ACTIONS(2057),
    [sym_file_read] = ACTIONS(2057),
    [sym_file_read_line] = ACTIONS(2057),
    [sym_file_write] = ACTIONS(2057),
    [sym_file_write_line] = ACTIONS(2057),
    [sym_file_copy] = ACTIONS(2057),
    [sym_file_delete] = ACTIONS(2057),
    [sym_file_rename] = ACTIONS(2057),
    [sym_file_dir] = ACTIONS(2057),
    [sym_file_getdir] = ACTIONS(2057),
    [sym_file_setdir] = ACTIONS(2057),
    [sym_file_create_dir] = ACTIONS(2057),
    [sym_file_remove_dir] = ACTIONS(2057),
    [sym_file_seek] = ACTIONS(2057),
    [sym_length_array] = ACTIONS(2057),
    [sym_max_length_array] = ACTIONS(2057),
    [sym_set_length_array] = ACTIONS(2057),
    [sym_ip_client_open] = ACTIONS(2057),
    [sym_ip_client_close] = ACTIONS(2057),
    [sym_ip_server_open] = ACTIONS(2057),
    [sym_ip_server_close] = ACTIONS(2057),
    [sym_ip_bound_client_open] = ACTIONS(2057),
    [sym_ip_mc_server_open] = ACTIONS(2057),
    [sym_ip_set_option] = ACTIONS(2057),
    [sym_ssh_client_open] = ACTIONS(2057),
    [sym_ssh_client_close] = ACTIONS(2057),
    [sym_tls_client_open] = ACTIONS(2057),
    [sym_tls_client_close] = ACTIONS(2057),
    [sym_do_push] = ACTIONS(2057),
    [sym_do_push_timed] = ACTIONS(2057),
    [sym_do_release] = ACTIONS(2057),
    [sym_device_id] = ACTIONS(2057),
    [sym_device_info] = ACTIONS(2057),
    [sym_device_id_string] = ACTIONS(2057),
    [sym_device_standby] = ACTIONS(2057),
    [sym_device_wake] = ACTIONS(2057),
    [sym_timeline_create] = ACTIONS(2057),
    [sym_timeline_kill] = ACTIONS(2057),
    [sym_timeline_reload] = ACTIONS(2057),
    [sym_timeline_pause] = ACTIONS(2057),
    [sym_timeline_restart] = ACTIONS(2057),
    [sym_timeline_set] = ACTIONS(2057),
    [sym_timeline_get] = ACTIONS(2057),
    [sym_timeline_active] = ACTIONS(2057),
    [sym_timeline_dynamic_id] = ACTIONS(2057),
    [sym_variable_to_string] = ACTIONS(2057),
    [sym_string_to_variable] = ACTIONS(2057),
    [sym_length_variable_to_string] = ACTIONS(2057),
    [sym_type_cast] = ACTIONS(2057),
    [sym_variable_to_xml] = ACTIONS(2057),
    [sym_xml_to_variable] = ACTIONS(2057),
    [sym_length_variable_to_xml] = ACTIONS(2057),
    [sym_redirect_string] = ACTIONS(2057),
    [sym_random_number] = ACTIONS(2057),
    [sym_get_unique_id] = ACTIONS(2057),
    [sym_get_system_number] = ACTIONS(2057),
    [sym_set_system_number] = ACTIONS(2057),
    [sym_get_serial_number] = ACTIONS(2057),
    [sym_get_master_build] = ACTIONS(2057),
    [sym_set_validation_code] = ACTIONS(2057),
    [sym_reboot] = ACTIONS(2057),
    [sym_load_duet_module] = ACTIONS(2057),
    [sym_unload_duet_module] = ACTIONS(2057),
    [sym_duet_mem_size_set] = ACTIONS(2057),
    [sym_duet_mem_size_get] = ACTIONS(2057),
    [sym_dynamic_application_device] = ACTIONS(2057),
    [sym_dynamic_polled_port] = ACTIONS(2057),
    [sym_static_port_binding] = ACTIONS(2057),
    [sym_static_ip_binding] = ACTIONS(2057),
    [sym_get_dns_list] = ACTIONS(2057),
    [sym_set_dns_list] = ACTIONS(2057),
    [sym_get_ip_address] = ACTIONS(2057),
    [sym_set_ip_address] = ACTIONS(2057),
    [sym_get_url_list] = ACTIONS(2057),
    [sym_add_url_entry] = ACTIONS(2057),
    [sym_delete_url_entry] = ACTIONS(2057),
    [sym_abs_value] = ACTIONS(2057),
    [sym_max_value] = ACTIONS(2057),
    [sym_min_value] = ACTIONS(2057),
    [sym_exp_value] = ACTIONS(2057),
    [sym_log_value] = ACTIONS(2057),
    [sym_log10_value] = ACTIONS(2057),
    [sym_power_value] = ACTIONS(2057),
    [sym_sqrt_value] = ACTIONS(2057),
    [sym_raw_be] = ACTIONS(2057),
    [sym_raw_le] = ACTIONS(2057),
    [sym_internal_queue_size_set] = ACTIONS(2057),
    [sym_internal_queue_size_get] = ACTIONS(2057),
    [sym_internal_threshold_set] = ACTIONS(2057),
    [sym_internal_threshold_get] = ACTIONS(2057),
    [sym_rebuilt_event] = ACTIONS(2057),
    [sym_do_custom_event] = ACTIONS(2057),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2057),
    [sym_clkmgr_set_clk_source] = ACTIONS(2057),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2057),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2057),
    [sym_clkmgr_get_timezone] = ACTIONS(2057),
    [sym_clkmgr_set_timezone] = ACTIONS(2057),
    [sym_clkmgr_get_resync_period] = ACTIONS(2057),
    [sym_clkmgr_set_resync_period] = ACTIONS(2057),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2057),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2057),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2057),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2057),
    [sym_clkmgr_get_timeservers] = ACTIONS(2057),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2057),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2057),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2057),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2057),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2057),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2057),
    [sym_set_log_level] = ACTIONS(2057),
    [sym_get_log_level] = ACTIONS(2057),
    [sym_amx_log] = ACTIONS(2057),
    [sym_smtp_server_config_set] = ACTIONS(2057),
    [sym_smtp_server_config_get] = ACTIONS(2057),
    [sym_smtp_send] = ACTIONS(2057),
    [sym_validate_netlinx_account] = ACTIONS(2057),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2057),
    [sym_authenticate_certificate] = ACTIONS(2057),
    [sym_audit_netlinx_session_event] = ACTIONS(2057),
    [sym_audit_netlinx_generic_event] = ACTIONS(2057),
    [sym_get_available_flash_disk_space] = ACTIONS(2057),
    [sym_get_max_flash_disk_space] = ACTIONS(2057),
    [sym_data_create_feed] = ACTIONS(2057),
    [sym_data_delete_feed] = ACTIONS(2057),
    [sym_data_publish_feed] = ACTIONS(2057),
    [sym_data_get_published_feed] = ACTIONS(2057),
    [sym_data_add_record] = ACTIONS(2057),
    [sym_data_get_event_record] = ACTIONS(2057),
    [sym__wc_data_create_feed] = ACTIONS(2057),
    [sym__wc_data_add_record] = ACTIONS(2057),
    [sym__wc_data_get_event_record] = ACTIONS(2057),
    [sym_format] = ACTIONS(2057),
    [sym_astro_clock] = ACTIONS(2057),
    [sym_set_outdoor_temperature] = ACTIONS(2057),
    [sym_set_virtual_level_count] = ACTIONS(2057),
    [sym_set_virtual_channel_count] = ACTIONS(2057),
    [sym_set_virtual_port_count] = ACTIONS(2057),
    [sym__wc_encode_int] = ACTIONS(2057),
    [sym__wc_decode_int] = ACTIONS(2057),
    [sym__wc_int] = ACTIONS(2057),
    [sym__wc_to_ch_int] = ACTIONS(2057),
    [sym__ch_to_wc_int] = ACTIONS(2057),
    [sym__wc_find_string_int] = ACTIONS(2057),
    [sym__wc_left_string_int] = ACTIONS(2057),
    [sym__wc_lower_string_int] = ACTIONS(2057),
    [sym__wc_mid_string_int] = ACTIONS(2057),
    [sym__wc_remove_string_int] = ACTIONS(2057),
    [sym__wc_right_string_int] = ACTIONS(2057),
    [sym__wc_upper_string_int] = ACTIONS(2057),
    [sym__wc_compare_string_int] = ACTIONS(2057),
    [sym__wc_get_buffer_char_int] = ACTIONS(2057),
    [sym__wc_get_buffer_string_int] = ACTIONS(2057),
    [sym__wc_concat_string_int] = ACTIONS(2057),
    [sym__wc_file_open_int] = ACTIONS(2057),
    [sym__wc_file_close_int] = ACTIONS(2057),
    [sym__wc_file_read_int] = ACTIONS(2057),
    [sym__wc_file_read_line_int] = ACTIONS(2057),
    [sym__wc_file_write_int] = ACTIONS(2057),
    [sym__wc_file_write_line_int] = ACTIONS(2057),
    [aux_sym_preproc_include_token1] = ACTIONS(2057),
    [aux_sym_preproc_define_token1] = ACTIONS(2059),
    [aux_sym_preproc_warn_token1] = ACTIONS(2059),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2059),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2059),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2059),
    [anon_sym_LBRACK] = ACTIONS(2059),
    [anon_sym_LBRACE] = ACTIONS(2059),
    [anon_sym_LPAREN] = ACTIONS(2057),
    [anon_sym_BANG] = ACTIONS(2059),
    [anon_sym_TILDE] = ACTIONS(2059),
    [anon_sym_DASH] = ACTIONS(2057),
    [anon_sym_PLUS] = ACTIONS(2057),
    [anon_sym_DASH_DASH] = ACTIONS(2059),
    [anon_sym_PLUS_PLUS] = ACTIONS(2059),
    [anon_sym_DQUOTE] = ACTIONS(2059),
    [anon_sym_SQUOTE] = ACTIONS(2059),
    [sym_number_literal] = ACTIONS(2059),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(422)] = {
    [ts_builtin_sym_end] = ACTIONS(2119),
    [sym_identifier] = ACTIONS(2117),
    [sym_program_name_keyword] = ACTIONS(2117),
    [sym_module_name_keyword] = ACTIONS(2117),
    [sym_define_device_keyword] = ACTIONS(2117),
    [sym_define_combine_keyword] = ACTIONS(2117),
    [sym_define_constant_keyword] = ACTIONS(2117),
    [sym_define_type_keyword] = ACTIONS(2117),
    [sym_define_variable_keyword] = ACTIONS(2117),
    [sym_define_system_variable_keyword] = ACTIONS(2117),
    [sym_define_connect_level_keyword] = ACTIONS(2117),
    [sym_define_latching_keyword] = ACTIONS(2117),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2117),
    [sym_define_toggling_keyword] = ACTIONS(2117),
    [sym_define_call_keyword] = ACTIONS(2117),
    [sym_define_function_keyword] = ACTIONS(2117),
    [sym_define_library_function_keyword] = ACTIONS(2117),
    [sym_define_start_keyword] = ACTIONS(2117),
    [sym_define_event_keyword] = ACTIONS(2117),
    [sym_define_program_keyword] = ACTIONS(2117),
    [sym_define_module_keyword] = ACTIONS(2117),
    [sym_char_keyword] = ACTIONS(2117),
    [sym_widechar_keyword] = ACTIONS(2117),
    [sym_integer_keyword] = ACTIONS(2117),
    [sym_sinteger_keyword] = ACTIONS(2117),
    [sym_long_keyword] = ACTIONS(2117),
    [sym_slong_keyword] = ACTIONS(2117),
    [sym_float_keyword] = ACTIONS(2117),
    [sym_double_keyword] = ACTIONS(2117),
    [sym_variant_keyword] = ACTIONS(2117),
    [sym_variantarray_keyword] = ACTIONS(2117),
    [sym_local_var_keyword] = ACTIONS(2117),
    [sym_stack_var_keyword] = ACTIONS(2117),
    [sym_constant_keyword] = ACTIONS(2117),
    [sym_volatile_keyword] = ACTIONS(2117),
    [sym_non_volatile_keyword] = ACTIONS(2117),
    [sym_persistent_keyword] = ACTIONS(2117),
    [sym_dev_keyword] = ACTIONS(2117),
    [sym_devlev_keyword] = ACTIONS(2117),
    [sym_devchan_keyword] = ACTIONS(2117),
    [sym_if_keyword] = ACTIONS(2117),
    [sym_switch_keyword] = ACTIONS(2117),
    [sym_case_keyword] = ACTIONS(2117),
    [sym_default_keyword] = ACTIONS(2117),
    [sym_while_keyword] = ACTIONS(2117),
    [sym_for_keyword] = ACTIONS(2117),
    [sym_break_keyword] = ACTIONS(2117),
    [sym_continue_keyword] = ACTIONS(2117),
    [sym_return_keyword] = ACTIONS(2117),
    [sym_select_keyword] = ACTIONS(2117),
    [sym_active_keyword] = ACTIONS(2117),
    [sym_struct_keyword] = ACTIONS(2117),
    [sym_structure_keyword] = ACTIONS(2117),
    [sym_button_event_keyword] = ACTIONS(2117),
    [sym_channel_event_keyword] = ACTIONS(2117),
    [sym_level_event_keyword] = ACTIONS(2117),
    [sym_data_event_keyword] = ACTIONS(2117),
    [sym_timeline_event_keyword] = ACTIONS(2117),
    [sym_custom_event_keyword] = ACTIONS(2117),
    [sym_push_keyword] = ACTIONS(2117),
    [sym_release_keyword] = ACTIONS(2117),
    [aux_sym_on_keyword_token1] = ACTIONS(2117),
    [aux_sym_off_keyword_token1] = ACTIONS(2117),
    [sym_send_level_keyword] = ACTIONS(2117),
    [sym_send_string_keyword] = ACTIONS(2117),
    [sym_send_command_keyword] = ACTIONS(2117),
    [sym_clear_buffer_keyword] = ACTIONS(2117),
    [sym_create_buffer_keyword] = ACTIONS(2117),
    [sym_create_multi_buffer_keyword] = ACTIONS(2117),
    [sym_call_keyword] = ACTIONS(2117),
    [sym_system_call_keyword] = ACTIONS(2117),
    [sym_devchan_to_keyword] = ACTIONS(2117),
    [sym_devchan_min_to_keyword] = ACTIONS(2117),
    [sym_devchan_total_off_keyword] = ACTIONS(2117),
    [sym_devchan_pulse_keyword] = ACTIONS(2117),
    [sym_wait_keyword] = ACTIONS(2117),
    [sym_cancel_wait_keyword] = ACTIONS(2117),
    [sym_wait_until_keyword] = ACTIONS(2117),
    [sym_cancel_wait_until_keyword] = ACTIONS(2117),
    [sym_cancel_all_wait_keyword] = ACTIONS(2117),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2117),
    [sym_not] = ACTIONS(2117),
    [sym_bnot] = ACTIONS(2117),
    [sym___file__] = ACTIONS(2117),
    [sym___date__] = ACTIONS(2117),
    [sym___ldate__] = ACTIONS(2117),
    [sym___time__] = ACTIONS(2117),
    [sym___line__] = ACTIONS(2117),
    [sym___name__] = ACTIONS(2117),
    [sym___netlinx__] = ACTIONS(2117),
    [sym_netlinx_axi_version] = ACTIONS(2117),
    [sym_url_flg_tcp] = ACTIONS(2117),
    [sym_url_flg_accinfopresent] = ACTIONS(2117),
    [sym_url_flg_temp] = ACTIONS(2117),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2117),
    [sym_url_flg_stat_mask] = ACTIONS(2117),
    [sym_url_flg_stat_lookup] = ACTIONS(2117),
    [sym_url_flg_stat_connecting] = ACTIONS(2117),
    [sym_url_flg_stat_waiting] = ACTIONS(2117),
    [sym_url_flg_stat_connected] = ACTIONS(2117),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2117),
    [sym_first_virtual_device] = ACTIONS(2117),
    [sym_first_local_port] = ACTIONS(2117),
    [sym_dynamic_virtual_device] = ACTIONS(2117),
    [sym_timeline_once] = ACTIONS(2117),
    [sym_timeline_repeat] = ACTIONS(2117),
    [sym_timeline_absolute] = ACTIONS(2117),
    [sym_timeline_relative] = ACTIONS(2117),
    [sym_do_push_timed_infinite] = ACTIONS(2117),
    [sym_source_type_no_address] = ACTIONS(2117),
    [sym_source_type_neuron_id] = ACTIONS(2117),
    [sym_source_type_ip_address] = ACTIONS(2117),
    [sym_source_type_axlink] = ACTIONS(2117),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2117),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2117),
    [sym_source_type_ip_socket_address] = ACTIONS(2117),
    [sym_source_type_rs232] = ACTIONS(2117),
    [sym_source_type_internal] = ACTIONS(2117),
    [sym_source_type_mac_address] = ACTIONS(2117),
    [sym_source_type_ipv4_port] = ACTIONS(2117),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2117),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2117),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2117),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2117),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2117),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2117),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2117),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2117),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2117),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2117),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2117),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2117),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2117),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2117),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2117),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2117),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2117),
    [sym_true] = ACTIONS(2117),
    [sym_false] = ACTIONS(2117),
    [sym_file_read_only] = ACTIONS(2117),
    [sym_file_rw_new] = ACTIONS(2117),
    [sym_file_rw_append] = ACTIONS(2117),
    [sym_ip_tcp] = ACTIONS(2117),
    [sym_ip_udp] = ACTIONS(2117),
    [sym_ip_udp_2way] = ACTIONS(2117),
    [sym_xml_encode_types] = ACTIONS(2117),
    [sym_xml_encode_char_as_list] = ACTIONS(2117),
    [sym_xml_encode_le] = ACTIONS(2117),
    [sym_xml_decode_types] = ACTIONS(2117),
    [sym_xml_decode_no_preserve] = ACTIONS(2117),
    [sym_duet_dev_type_display_device] = ACTIONS(2117),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2117),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2117),
    [sym_duet_dev_type_io_device] = ACTIONS(2117),
    [sym_duet_dev_type_relay_device] = ACTIONS(2117),
    [sym_duet_dev_type_ups] = ACTIONS(2117),
    [sym_duet_dev_type_amplifier] = ACTIONS(2117),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2117),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2117),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2117),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2117),
    [sym_duet_dev_type_camera] = ACTIONS(2117),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2117),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2117),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2117),
    [sym_duet_dev_type_dss] = ACTIONS(2117),
    [sym_duet_dev_type_dvr] = ACTIONS(2117),
    [sym_duet_dev_type_disc_device] = ACTIONS(2117),
    [sym_duet_dev_type_document_camera] = ACTIONS(2117),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2117),
    [sym_duet_dev_type_hvac] = ACTIONS(2117),
    [sym_duet_dev_type_keypad] = ACTIONS(2117),
    [sym_duet_dev_type_light] = ACTIONS(2117),
    [sym_duet_dev_type_monitor] = ACTIONS(2117),
    [sym_duet_dev_type_motor] = ACTIONS(2117),
    [sym_duet_dev_type_multi_window] = ACTIONS(2117),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2117),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2117),
    [sym_duet_dev_type_receiver] = ACTIONS(2117),
    [sym_duet_dev_type_security_system] = ACTIONS(2117),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2117),
    [sym_duet_dev_type_settop_box] = ACTIONS(2117),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2117),
    [sym_duet_dev_type_switcher] = ACTIONS(2117),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2117),
    [sym_duet_dev_type_tv] = ACTIONS(2117),
    [sym_duet_dev_type_utility] = ACTIONS(2117),
    [sym_duet_dev_type_vcr] = ACTIONS(2117),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2117),
    [sym_duet_dev_type_video_processor] = ACTIONS(2117),
    [sym_duet_dev_type_video_projector] = ACTIONS(2117),
    [sym_duet_dev_type_video_wall] = ACTIONS(2117),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2117),
    [sym_duet_dev_type_weather] = ACTIONS(2117),
    [sym_duet_dev_not_polled] = ACTIONS(2117),
    [sym_duet_dev_polled] = ACTIONS(2117),
    [sym_clkmgr_mode_network] = ACTIONS(2117),
    [sym_clkmgr_mode_standalone] = ACTIONS(2117),
    [sym_normal_standby] = ACTIONS(2117),
    [sym_normal_wake] = ACTIONS(2117),
    [sym_ip_multicast_ttl_option] = ACTIONS(2117),
    [sym_ip_ttl_subnet] = ACTIONS(2117),
    [sym_ip_ttl_site] = ACTIONS(2117),
    [sym_ip_ttl_region] = ACTIONS(2117),
    [sym_ip_ttl_continent] = ACTIONS(2117),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2117),
    [sym_ip_nodelay_on] = ACTIONS(2117),
    [sym_ip_nodelay_off] = ACTIONS(2117),
    [sym_amx_error] = ACTIONS(2117),
    [sym_amx_warning] = ACTIONS(2117),
    [sym_amx_info] = ACTIONS(2117),
    [sym_amx_debug] = ACTIONS(2117),
    [sym_netlinx_login_success] = ACTIONS(2117),
    [sym_netlinx_login_fail] = ACTIONS(2117),
    [sym_netlinx_logout] = ACTIONS(2117),
    [sym_null_str] = ACTIONS(2117),
    [sym_smtp_address] = ACTIONS(2117),
    [sym_smtp_port_number] = ACTIONS(2117),
    [sym_smtp_username] = ACTIONS(2117),
    [sym_smtp_password] = ACTIONS(2117),
    [sym_smtp_require_tls] = ACTIONS(2117),
    [sym_smtp_from] = ACTIONS(2117),
    [sym_smtp_tls_true] = ACTIONS(2117),
    [sym_smtp_tls_false] = ACTIONS(2117),
    [sym_listview_on_row_select_event] = ACTIONS(2117),
    [sym_data_structure_unknown] = ACTIONS(2117),
    [sym_data_structure_datafeed] = ACTIONS(2117),
    [sym_data_structure_datarecordset] = ACTIONS(2117),
    [sym_data_structure_datarecord] = ACTIONS(2117),
    [sym_data_structure_datafield] = ACTIONS(2117),
    [sym_data_max_content_field_count] = ACTIONS(2117),
    [sym_data_max_metadata_field_count] = ACTIONS(2117),
    [sym_data_max_id_length] = ACTIONS(2117),
    [sym_data_max_name_length] = ACTIONS(2117),
    [sym_data_max_description_length] = ACTIONS(2117),
    [sym_data_max_type_length] = ACTIONS(2117),
    [sym_data_max_format_length] = ACTIONS(2117),
    [sym_data_max_source_length] = ACTIONS(2117),
    [sym_data_max_value_length] = ACTIONS(2117),
    [sym_data_max_label_length] = ACTIONS(2117),
    [sym_data_max_url_length] = ACTIONS(2117),
    [sym_max_last_login_info_length] = ACTIONS(2117),
    [sym_data_type_unknown] = ACTIONS(2117),
    [sym_data_type_string] = ACTIONS(2117),
    [sym_data_type_datetime] = ACTIONS(2117),
    [sym_data_type_date] = ACTIONS(2117),
    [sym_data_type_time] = ACTIONS(2117),
    [sym_data_type_image] = ACTIONS(2117),
    [sym_data_format_url] = ACTIONS(2117),
    [sym_data_format_phone] = ACTIONS(2117),
    [sym_data_format_email] = ACTIONS(2117),
    [sym_data_format_iso8601] = ACTIONS(2117),
    [sym_auditlog_priv] = ACTIONS(2117),
    [sym_configuration_priv] = ACTIONS(2117),
    [sym_deviceconfig_priv] = ACTIONS(2117),
    [sym_ftp_priv] = ACTIONS(2117),
    [sym_http_priv] = ACTIONS(2117),
    [sym_networkconfig_priv] = ACTIONS(2117),
    [sym_programport_priv] = ACTIONS(2117),
    [sym_remoteui_priv] = ACTIONS(2117),
    [sym_securitycontrol_priv] = ACTIONS(2117),
    [sym_softwaremanagement_priv] = ACTIONS(2117),
    [sym_terminal_priv] = ACTIONS(2117),
    [sym_tpadmin_priv] = ACTIONS(2117),
    [sym_user1_priv] = ACTIONS(2117),
    [sym_user2_priv] = ACTIONS(2117),
    [sym_user3_priv] = ACTIONS(2117),
    [sym_user4_priv] = ACTIONS(2117),
    [sym_usermanagement_priv] = ACTIONS(2117),
    [sym_valid_account] = ACTIONS(2117),
    [sym_err_name_invalid] = ACTIONS(2117),
    [sym_err_password_invalid] = ACTIONS(2117),
    [sym_err_invalid_account] = ACTIONS(2117),
    [sym_err_no_authorization] = ACTIONS(2117),
    [sym_err_invalid_parameter] = ACTIONS(2117),
    [sym_err_user_locked_out] = ACTIONS(2117),
    [sym_err_user_account_expired] = ACTIONS(2117),
    [sym_err_general_error] = ACTIONS(2117),
    [sym_err_authentication_error] = ACTIONS(2117),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2117),
    [sym_tls_validate_certificate] = ACTIONS(2117),
    [sym_tdata] = ACTIONS(2117),
    [sym_tchannel] = ACTIONS(2117),
    [sym_tlevel] = ACTIONS(2117),
    [sym_tbutton] = ACTIONS(2117),
    [sym_ttimeline] = ACTIONS(2117),
    [sym_tcustom] = ACTIONS(2117),
    [sym_url_struct] = ACTIONS(2117),
    [sym_dns_struct] = ACTIONS(2117),
    [sym_ip_address_struct] = ACTIONS(2117),
    [sym_dev_info_struct] = ACTIONS(2117),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2117),
    [sym_data_feed] = ACTIONS(2117),
    [sym_data_field] = ACTIONS(2117),
    [sym_data_record] = ACTIONS(2117),
    [sym_wc_data_feed] = ACTIONS(2117),
    [sym_wc_data_field] = ACTIONS(2117),
    [sym_wc_data_record] = ACTIONS(2117),
    [sym_last_login_info] = ACTIONS(2117),
    [sym____reserved___] = ACTIONS(2117),
    [sym_date] = ACTIONS(2117),
    [sym_day] = ACTIONS(2117),
    [sym_time] = ACTIONS(2117),
    [sym_ldate] = ACTIONS(2117),
    [sym_push_device] = ACTIONS(2117),
    [sym_push_channel] = ACTIONS(2117),
    [sym_push_devchan] = ACTIONS(2117),
    [sym_release_device] = ACTIONS(2117),
    [sym_release_channel] = ACTIONS(2117),
    [sym_release_devchan] = ACTIONS(2117),
    [sym_master_slot] = ACTIONS(2117),
    [sym_get_pulse_time] = ACTIONS(2117),
    [sym_get_timer] = ACTIONS(2117),
    [sym_master_sn] = ACTIONS(2117),
    [sym_system_number] = ACTIONS(2117),
    [sym_button] = ACTIONS(2117),
    [sym_dv_channel] = ACTIONS(2117),
    [sym_data] = ACTIONS(2117),
    [sym_level] = ACTIONS(2117),
    [sym_timeline] = ACTIONS(2117),
    [sym_channel] = ACTIONS(2117),
    [sym_custom] = ACTIONS(2117),
    [sym_atoi] = ACTIONS(2117),
    [sym_atol] = ACTIONS(2117),
    [sym_atof] = ACTIONS(2117),
    [sym_itoa] = ACTIONS(2117),
    [sym_itohex] = ACTIONS(2117),
    [sym_hextoi] = ACTIONS(2117),
    [sym_ftoa] = ACTIONS(2117),
    [sym_find_string] = ACTIONS(2117),
    [sym_left_string] = ACTIONS(2117),
    [sym_right_string] = ACTIONS(2117),
    [sym_mid_string] = ACTIONS(2117),
    [sym_remove_string] = ACTIONS(2117),
    [sym_get_buffer_string] = ACTIONS(2117),
    [sym_get_buffer_char] = ACTIONS(2117),
    [sym_get_multi_buffer_string] = ACTIONS(2117),
    [sym_length_string] = ACTIONS(2117),
    [sym_max_length_string] = ACTIONS(2117),
    [sym_set_length_string] = ACTIONS(2117),
    [sym_compare_string] = ACTIONS(2117),
    [sym_upper_string] = ACTIONS(2117),
    [sym_lower_string] = ACTIONS(2117),
    [sym_get_last] = ACTIONS(2117),
    [sym_date_to_day] = ACTIONS(2117),
    [sym_date_to_month] = ACTIONS(2117),
    [sym_date_to_year] = ACTIONS(2117),
    [sym_time_to_hour] = ACTIONS(2117),
    [sym_time_to_minute] = ACTIONS(2117),
    [sym_time_to_second] = ACTIONS(2117),
    [sym_day_of_week] = ACTIONS(2117),
    [sym_set_pulse_time] = ACTIONS(2117),
    [sym_set_timer] = ACTIONS(2117),
    [sym_file_open] = ACTIONS(2117),
    [sym_file_close] = ACTIONS(2117),
    [sym_file_read] = ACTIONS(2117),
    [sym_file_read_line] = ACTIONS(2117),
    [sym_file_write] = ACTIONS(2117),
    [sym_file_write_line] = ACTIONS(2117),
    [sym_file_copy] = ACTIONS(2117),
    [sym_file_delete] = ACTIONS(2117),
    [sym_file_rename] = ACTIONS(2117),
    [sym_file_dir] = ACTIONS(2117),
    [sym_file_getdir] = ACTIONS(2117),
    [sym_file_setdir] = ACTIONS(2117),
    [sym_file_create_dir] = ACTIONS(2117),
    [sym_file_remove_dir] = ACTIONS(2117),
    [sym_file_seek] = ACTIONS(2117),
    [sym_length_array] = ACTIONS(2117),
    [sym_max_length_array] = ACTIONS(2117),
    [sym_set_length_array] = ACTIONS(2117),
    [sym_ip_client_open] = ACTIONS(2117),
    [sym_ip_client_close] = ACTIONS(2117),
    [sym_ip_server_open] = ACTIONS(2117),
    [sym_ip_server_close] = ACTIONS(2117),
    [sym_ip_bound_client_open] = ACTIONS(2117),
    [sym_ip_mc_server_open] = ACTIONS(2117),
    [sym_ip_set_option] = ACTIONS(2117),
    [sym_ssh_client_open] = ACTIONS(2117),
    [sym_ssh_client_close] = ACTIONS(2117),
    [sym_tls_client_open] = ACTIONS(2117),
    [sym_tls_client_close] = ACTIONS(2117),
    [sym_do_push] = ACTIONS(2117),
    [sym_do_push_timed] = ACTIONS(2117),
    [sym_do_release] = ACTIONS(2117),
    [sym_device_id] = ACTIONS(2117),
    [sym_device_info] = ACTIONS(2117),
    [sym_device_id_string] = ACTIONS(2117),
    [sym_device_standby] = ACTIONS(2117),
    [sym_device_wake] = ACTIONS(2117),
    [sym_timeline_create] = ACTIONS(2117),
    [sym_timeline_kill] = ACTIONS(2117),
    [sym_timeline_reload] = ACTIONS(2117),
    [sym_timeline_pause] = ACTIONS(2117),
    [sym_timeline_restart] = ACTIONS(2117),
    [sym_timeline_set] = ACTIONS(2117),
    [sym_timeline_get] = ACTIONS(2117),
    [sym_timeline_active] = ACTIONS(2117),
    [sym_timeline_dynamic_id] = ACTIONS(2117),
    [sym_variable_to_string] = ACTIONS(2117),
    [sym_string_to_variable] = ACTIONS(2117),
    [sym_length_variable_to_string] = ACTIONS(2117),
    [sym_type_cast] = ACTIONS(2117),
    [sym_variable_to_xml] = ACTIONS(2117),
    [sym_xml_to_variable] = ACTIONS(2117),
    [sym_length_variable_to_xml] = ACTIONS(2117),
    [sym_redirect_string] = ACTIONS(2117),
    [sym_random_number] = ACTIONS(2117),
    [sym_get_unique_id] = ACTIONS(2117),
    [sym_get_system_number] = ACTIONS(2117),
    [sym_set_system_number] = ACTIONS(2117),
    [sym_get_serial_number] = ACTIONS(2117),
    [sym_get_master_build] = ACTIONS(2117),
    [sym_set_validation_code] = ACTIONS(2117),
    [sym_reboot] = ACTIONS(2117),
    [sym_load_duet_module] = ACTIONS(2117),
    [sym_unload_duet_module] = ACTIONS(2117),
    [sym_duet_mem_size_set] = ACTIONS(2117),
    [sym_duet_mem_size_get] = ACTIONS(2117),
    [sym_dynamic_application_device] = ACTIONS(2117),
    [sym_dynamic_polled_port] = ACTIONS(2117),
    [sym_static_port_binding] = ACTIONS(2117),
    [sym_static_ip_binding] = ACTIONS(2117),
    [sym_get_dns_list] = ACTIONS(2117),
    [sym_set_dns_list] = ACTIONS(2117),
    [sym_get_ip_address] = ACTIONS(2117),
    [sym_set_ip_address] = ACTIONS(2117),
    [sym_get_url_list] = ACTIONS(2117),
    [sym_add_url_entry] = ACTIONS(2117),
    [sym_delete_url_entry] = ACTIONS(2117),
    [sym_abs_value] = ACTIONS(2117),
    [sym_max_value] = ACTIONS(2117),
    [sym_min_value] = ACTIONS(2117),
    [sym_exp_value] = ACTIONS(2117),
    [sym_log_value] = ACTIONS(2117),
    [sym_log10_value] = ACTIONS(2117),
    [sym_power_value] = ACTIONS(2117),
    [sym_sqrt_value] = ACTIONS(2117),
    [sym_raw_be] = ACTIONS(2117),
    [sym_raw_le] = ACTIONS(2117),
    [sym_internal_queue_size_set] = ACTIONS(2117),
    [sym_internal_queue_size_get] = ACTIONS(2117),
    [sym_internal_threshold_set] = ACTIONS(2117),
    [sym_internal_threshold_get] = ACTIONS(2117),
    [sym_rebuilt_event] = ACTIONS(2117),
    [sym_do_custom_event] = ACTIONS(2117),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2117),
    [sym_clkmgr_set_clk_source] = ACTIONS(2117),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2117),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2117),
    [sym_clkmgr_get_timezone] = ACTIONS(2117),
    [sym_clkmgr_set_timezone] = ACTIONS(2117),
    [sym_clkmgr_get_resync_period] = ACTIONS(2117),
    [sym_clkmgr_set_resync_period] = ACTIONS(2117),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2117),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2117),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2117),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2117),
    [sym_clkmgr_get_timeservers] = ACTIONS(2117),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2117),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2117),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2117),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2117),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2117),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2117),
    [sym_set_log_level] = ACTIONS(2117),
    [sym_get_log_level] = ACTIONS(2117),
    [sym_amx_log] = ACTIONS(2117),
    [sym_smtp_server_config_set] = ACTIONS(2117),
    [sym_smtp_server_config_get] = ACTIONS(2117),
    [sym_smtp_send] = ACTIONS(2117),
    [sym_validate_netlinx_account] = ACTIONS(2117),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2117),
    [sym_authenticate_certificate] = ACTIONS(2117),
    [sym_audit_netlinx_session_event] = ACTIONS(2117),
    [sym_audit_netlinx_generic_event] = ACTIONS(2117),
    [sym_get_available_flash_disk_space] = ACTIONS(2117),
    [sym_get_max_flash_disk_space] = ACTIONS(2117),
    [sym_data_create_feed] = ACTIONS(2117),
    [sym_data_delete_feed] = ACTIONS(2117),
    [sym_data_publish_feed] = ACTIONS(2117),
    [sym_data_get_published_feed] = ACTIONS(2117),
    [sym_data_add_record] = ACTIONS(2117),
    [sym_data_get_event_record] = ACTIONS(2117),
    [sym__wc_data_create_feed] = ACTIONS(2117),
    [sym__wc_data_add_record] = ACTIONS(2117),
    [sym__wc_data_get_event_record] = ACTIONS(2117),
    [sym_format] = ACTIONS(2117),
    [sym_astro_clock] = ACTIONS(2117),
    [sym_set_outdoor_temperature] = ACTIONS(2117),
    [sym_set_virtual_level_count] = ACTIONS(2117),
    [sym_set_virtual_channel_count] = ACTIONS(2117),
    [sym_set_virtual_port_count] = ACTIONS(2117),
    [sym__wc_encode_int] = ACTIONS(2117),
    [sym__wc_decode_int] = ACTIONS(2117),
    [sym__wc_int] = ACTIONS(2117),
    [sym__wc_to_ch_int] = ACTIONS(2117),
    [sym__ch_to_wc_int] = ACTIONS(2117),
    [sym__wc_find_string_int] = ACTIONS(2117),
    [sym__wc_left_string_int] = ACTIONS(2117),
    [sym__wc_lower_string_int] = ACTIONS(2117),
    [sym__wc_mid_string_int] = ACTIONS(2117),
    [sym__wc_remove_string_int] = ACTIONS(2117),
    [sym__wc_right_string_int] = ACTIONS(2117),
    [sym__wc_upper_string_int] = ACTIONS(2117),
    [sym__wc_compare_string_int] = ACTIONS(2117),
    [sym__wc_get_buffer_char_int] = ACTIONS(2117),
    [sym__wc_get_buffer_string_int] = ACTIONS(2117),
    [sym__wc_concat_string_int] = ACTIONS(2117),
    [sym__wc_file_open_int] = ACTIONS(2117),
    [sym__wc_file_close_int] = ACTIONS(2117),
    [sym__wc_file_read_int] = ACTIONS(2117),
    [sym__wc_file_read_line_int] = ACTIONS(2117),
    [sym__wc_file_write_int] = ACTIONS(2117),
    [sym__wc_file_write_line_int] = ACTIONS(2117),
    [aux_sym_preproc_include_token1] = ACTIONS(2117),
    [aux_sym_preproc_define_token1] = ACTIONS(2119),
    [aux_sym_preproc_warn_token1] = ACTIONS(2119),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2119),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2119),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2119),
    [anon_sym_LBRACK] = ACTIONS(2119),
    [anon_sym_LBRACE] = ACTIONS(2119),
    [anon_sym_LPAREN] = ACTIONS(2117),
    [anon_sym_BANG] = ACTIONS(2119),
    [anon_sym_TILDE] = ACTIONS(2119),
    [anon_sym_DASH] = ACTIONS(2117),
    [anon_sym_PLUS] = ACTIONS(2117),
    [anon_sym_DASH_DASH] = ACTIONS(2119),
    [anon_sym_PLUS_PLUS] = ACTIONS(2119),
    [anon_sym_DQUOTE] = ACTIONS(2119),
    [anon_sym_SQUOTE] = ACTIONS(2119),
    [sym_number_literal] = ACTIONS(2119),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(423)] = {
    [ts_builtin_sym_end] = ACTIONS(1987),
    [sym_identifier] = ACTIONS(1985),
    [sym_program_name_keyword] = ACTIONS(1985),
    [sym_module_name_keyword] = ACTIONS(1985),
    [sym_define_device_keyword] = ACTIONS(1985),
    [sym_define_combine_keyword] = ACTIONS(1985),
    [sym_define_constant_keyword] = ACTIONS(1985),
    [sym_define_type_keyword] = ACTIONS(1985),
    [sym_define_variable_keyword] = ACTIONS(1985),
    [sym_define_system_variable_keyword] = ACTIONS(1985),
    [sym_define_connect_level_keyword] = ACTIONS(1985),
    [sym_define_latching_keyword] = ACTIONS(1985),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1985),
    [sym_define_toggling_keyword] = ACTIONS(1985),
    [sym_define_call_keyword] = ACTIONS(1985),
    [sym_define_function_keyword] = ACTIONS(1985),
    [sym_define_library_function_keyword] = ACTIONS(1985),
    [sym_define_start_keyword] = ACTIONS(1985),
    [sym_define_event_keyword] = ACTIONS(1985),
    [sym_define_program_keyword] = ACTIONS(1985),
    [sym_define_module_keyword] = ACTIONS(1985),
    [sym_char_keyword] = ACTIONS(1985),
    [sym_widechar_keyword] = ACTIONS(1985),
    [sym_integer_keyword] = ACTIONS(1985),
    [sym_sinteger_keyword] = ACTIONS(1985),
    [sym_long_keyword] = ACTIONS(1985),
    [sym_slong_keyword] = ACTIONS(1985),
    [sym_float_keyword] = ACTIONS(1985),
    [sym_double_keyword] = ACTIONS(1985),
    [sym_variant_keyword] = ACTIONS(1985),
    [sym_variantarray_keyword] = ACTIONS(1985),
    [sym_local_var_keyword] = ACTIONS(1985),
    [sym_stack_var_keyword] = ACTIONS(1985),
    [sym_constant_keyword] = ACTIONS(1985),
    [sym_volatile_keyword] = ACTIONS(1985),
    [sym_non_volatile_keyword] = ACTIONS(1985),
    [sym_persistent_keyword] = ACTIONS(1985),
    [sym_dev_keyword] = ACTIONS(1985),
    [sym_devlev_keyword] = ACTIONS(1985),
    [sym_devchan_keyword] = ACTIONS(1985),
    [sym_if_keyword] = ACTIONS(1985),
    [sym_switch_keyword] = ACTIONS(1985),
    [sym_case_keyword] = ACTIONS(1985),
    [sym_default_keyword] = ACTIONS(1985),
    [sym_while_keyword] = ACTIONS(1985),
    [sym_for_keyword] = ACTIONS(1985),
    [sym_break_keyword] = ACTIONS(1985),
    [sym_continue_keyword] = ACTIONS(1985),
    [sym_return_keyword] = ACTIONS(1985),
    [sym_select_keyword] = ACTIONS(1985),
    [sym_active_keyword] = ACTIONS(1985),
    [sym_struct_keyword] = ACTIONS(1985),
    [sym_structure_keyword] = ACTIONS(1985),
    [sym_button_event_keyword] = ACTIONS(1985),
    [sym_channel_event_keyword] = ACTIONS(1985),
    [sym_level_event_keyword] = ACTIONS(1985),
    [sym_data_event_keyword] = ACTIONS(1985),
    [sym_timeline_event_keyword] = ACTIONS(1985),
    [sym_custom_event_keyword] = ACTIONS(1985),
    [sym_push_keyword] = ACTIONS(1985),
    [sym_release_keyword] = ACTIONS(1985),
    [aux_sym_on_keyword_token1] = ACTIONS(1985),
    [aux_sym_off_keyword_token1] = ACTIONS(1985),
    [sym_send_level_keyword] = ACTIONS(1985),
    [sym_send_string_keyword] = ACTIONS(1985),
    [sym_send_command_keyword] = ACTIONS(1985),
    [sym_clear_buffer_keyword] = ACTIONS(1985),
    [sym_create_buffer_keyword] = ACTIONS(1985),
    [sym_create_multi_buffer_keyword] = ACTIONS(1985),
    [sym_call_keyword] = ACTIONS(1985),
    [sym_system_call_keyword] = ACTIONS(1985),
    [sym_devchan_to_keyword] = ACTIONS(1985),
    [sym_devchan_min_to_keyword] = ACTIONS(1985),
    [sym_devchan_total_off_keyword] = ACTIONS(1985),
    [sym_devchan_pulse_keyword] = ACTIONS(1985),
    [sym_wait_keyword] = ACTIONS(1985),
    [sym_cancel_wait_keyword] = ACTIONS(1985),
    [sym_wait_until_keyword] = ACTIONS(1985),
    [sym_cancel_wait_until_keyword] = ACTIONS(1985),
    [sym_cancel_all_wait_keyword] = ACTIONS(1985),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1985),
    [sym_not] = ACTIONS(1985),
    [sym_bnot] = ACTIONS(1985),
    [sym___file__] = ACTIONS(1985),
    [sym___date__] = ACTIONS(1985),
    [sym___ldate__] = ACTIONS(1985),
    [sym___time__] = ACTIONS(1985),
    [sym___line__] = ACTIONS(1985),
    [sym___name__] = ACTIONS(1985),
    [sym___netlinx__] = ACTIONS(1985),
    [sym_netlinx_axi_version] = ACTIONS(1985),
    [sym_url_flg_tcp] = ACTIONS(1985),
    [sym_url_flg_accinfopresent] = ACTIONS(1985),
    [sym_url_flg_temp] = ACTIONS(1985),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1985),
    [sym_url_flg_stat_mask] = ACTIONS(1985),
    [sym_url_flg_stat_lookup] = ACTIONS(1985),
    [sym_url_flg_stat_connecting] = ACTIONS(1985),
    [sym_url_flg_stat_waiting] = ACTIONS(1985),
    [sym_url_flg_stat_connected] = ACTIONS(1985),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1985),
    [sym_first_virtual_device] = ACTIONS(1985),
    [sym_first_local_port] = ACTIONS(1985),
    [sym_dynamic_virtual_device] = ACTIONS(1985),
    [sym_timeline_once] = ACTIONS(1985),
    [sym_timeline_repeat] = ACTIONS(1985),
    [sym_timeline_absolute] = ACTIONS(1985),
    [sym_timeline_relative] = ACTIONS(1985),
    [sym_do_push_timed_infinite] = ACTIONS(1985),
    [sym_source_type_no_address] = ACTIONS(1985),
    [sym_source_type_neuron_id] = ACTIONS(1985),
    [sym_source_type_ip_address] = ACTIONS(1985),
    [sym_source_type_axlink] = ACTIONS(1985),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1985),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1985),
    [sym_source_type_ip_socket_address] = ACTIONS(1985),
    [sym_source_type_rs232] = ACTIONS(1985),
    [sym_source_type_internal] = ACTIONS(1985),
    [sym_source_type_mac_address] = ACTIONS(1985),
    [sym_source_type_ipv4_port] = ACTIONS(1985),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1985),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1985),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1985),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1985),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1985),
    [sym_true] = ACTIONS(1985),
    [sym_false] = ACTIONS(1985),
    [sym_file_read_only] = ACTIONS(1985),
    [sym_file_rw_new] = ACTIONS(1985),
    [sym_file_rw_append] = ACTIONS(1985),
    [sym_ip_tcp] = ACTIONS(1985),
    [sym_ip_udp] = ACTIONS(1985),
    [sym_ip_udp_2way] = ACTIONS(1985),
    [sym_xml_encode_types] = ACTIONS(1985),
    [sym_xml_encode_char_as_list] = ACTIONS(1985),
    [sym_xml_encode_le] = ACTIONS(1985),
    [sym_xml_decode_types] = ACTIONS(1985),
    [sym_xml_decode_no_preserve] = ACTIONS(1985),
    [sym_duet_dev_type_display_device] = ACTIONS(1985),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1985),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1985),
    [sym_duet_dev_type_io_device] = ACTIONS(1985),
    [sym_duet_dev_type_relay_device] = ACTIONS(1985),
    [sym_duet_dev_type_ups] = ACTIONS(1985),
    [sym_duet_dev_type_amplifier] = ACTIONS(1985),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1985),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1985),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1985),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1985),
    [sym_duet_dev_type_camera] = ACTIONS(1985),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1985),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1985),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1985),
    [sym_duet_dev_type_dss] = ACTIONS(1985),
    [sym_duet_dev_type_dvr] = ACTIONS(1985),
    [sym_duet_dev_type_disc_device] = ACTIONS(1985),
    [sym_duet_dev_type_document_camera] = ACTIONS(1985),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1985),
    [sym_duet_dev_type_hvac] = ACTIONS(1985),
    [sym_duet_dev_type_keypad] = ACTIONS(1985),
    [sym_duet_dev_type_light] = ACTIONS(1985),
    [sym_duet_dev_type_monitor] = ACTIONS(1985),
    [sym_duet_dev_type_motor] = ACTIONS(1985),
    [sym_duet_dev_type_multi_window] = ACTIONS(1985),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1985),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1985),
    [sym_duet_dev_type_receiver] = ACTIONS(1985),
    [sym_duet_dev_type_security_system] = ACTIONS(1985),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1985),
    [sym_duet_dev_type_settop_box] = ACTIONS(1985),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1985),
    [sym_duet_dev_type_switcher] = ACTIONS(1985),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1985),
    [sym_duet_dev_type_tv] = ACTIONS(1985),
    [sym_duet_dev_type_utility] = ACTIONS(1985),
    [sym_duet_dev_type_vcr] = ACTIONS(1985),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1985),
    [sym_duet_dev_type_video_processor] = ACTIONS(1985),
    [sym_duet_dev_type_video_projector] = ACTIONS(1985),
    [sym_duet_dev_type_video_wall] = ACTIONS(1985),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1985),
    [sym_duet_dev_type_weather] = ACTIONS(1985),
    [sym_duet_dev_not_polled] = ACTIONS(1985),
    [sym_duet_dev_polled] = ACTIONS(1985),
    [sym_clkmgr_mode_network] = ACTIONS(1985),
    [sym_clkmgr_mode_standalone] = ACTIONS(1985),
    [sym_normal_standby] = ACTIONS(1985),
    [sym_normal_wake] = ACTIONS(1985),
    [sym_ip_multicast_ttl_option] = ACTIONS(1985),
    [sym_ip_ttl_subnet] = ACTIONS(1985),
    [sym_ip_ttl_site] = ACTIONS(1985),
    [sym_ip_ttl_region] = ACTIONS(1985),
    [sym_ip_ttl_continent] = ACTIONS(1985),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1985),
    [sym_ip_nodelay_on] = ACTIONS(1985),
    [sym_ip_nodelay_off] = ACTIONS(1985),
    [sym_amx_error] = ACTIONS(1985),
    [sym_amx_warning] = ACTIONS(1985),
    [sym_amx_info] = ACTIONS(1985),
    [sym_amx_debug] = ACTIONS(1985),
    [sym_netlinx_login_success] = ACTIONS(1985),
    [sym_netlinx_login_fail] = ACTIONS(1985),
    [sym_netlinx_logout] = ACTIONS(1985),
    [sym_null_str] = ACTIONS(1985),
    [sym_smtp_address] = ACTIONS(1985),
    [sym_smtp_port_number] = ACTIONS(1985),
    [sym_smtp_username] = ACTIONS(1985),
    [sym_smtp_password] = ACTIONS(1985),
    [sym_smtp_require_tls] = ACTIONS(1985),
    [sym_smtp_from] = ACTIONS(1985),
    [sym_smtp_tls_true] = ACTIONS(1985),
    [sym_smtp_tls_false] = ACTIONS(1985),
    [sym_listview_on_row_select_event] = ACTIONS(1985),
    [sym_data_structure_unknown] = ACTIONS(1985),
    [sym_data_structure_datafeed] = ACTIONS(1985),
    [sym_data_structure_datarecordset] = ACTIONS(1985),
    [sym_data_structure_datarecord] = ACTIONS(1985),
    [sym_data_structure_datafield] = ACTIONS(1985),
    [sym_data_max_content_field_count] = ACTIONS(1985),
    [sym_data_max_metadata_field_count] = ACTIONS(1985),
    [sym_data_max_id_length] = ACTIONS(1985),
    [sym_data_max_name_length] = ACTIONS(1985),
    [sym_data_max_description_length] = ACTIONS(1985),
    [sym_data_max_type_length] = ACTIONS(1985),
    [sym_data_max_format_length] = ACTIONS(1985),
    [sym_data_max_source_length] = ACTIONS(1985),
    [sym_data_max_value_length] = ACTIONS(1985),
    [sym_data_max_label_length] = ACTIONS(1985),
    [sym_data_max_url_length] = ACTIONS(1985),
    [sym_max_last_login_info_length] = ACTIONS(1985),
    [sym_data_type_unknown] = ACTIONS(1985),
    [sym_data_type_string] = ACTIONS(1985),
    [sym_data_type_datetime] = ACTIONS(1985),
    [sym_data_type_date] = ACTIONS(1985),
    [sym_data_type_time] = ACTIONS(1985),
    [sym_data_type_image] = ACTIONS(1985),
    [sym_data_format_url] = ACTIONS(1985),
    [sym_data_format_phone] = ACTIONS(1985),
    [sym_data_format_email] = ACTIONS(1985),
    [sym_data_format_iso8601] = ACTIONS(1985),
    [sym_auditlog_priv] = ACTIONS(1985),
    [sym_configuration_priv] = ACTIONS(1985),
    [sym_deviceconfig_priv] = ACTIONS(1985),
    [sym_ftp_priv] = ACTIONS(1985),
    [sym_http_priv] = ACTIONS(1985),
    [sym_networkconfig_priv] = ACTIONS(1985),
    [sym_programport_priv] = ACTIONS(1985),
    [sym_remoteui_priv] = ACTIONS(1985),
    [sym_securitycontrol_priv] = ACTIONS(1985),
    [sym_softwaremanagement_priv] = ACTIONS(1985),
    [sym_terminal_priv] = ACTIONS(1985),
    [sym_tpadmin_priv] = ACTIONS(1985),
    [sym_user1_priv] = ACTIONS(1985),
    [sym_user2_priv] = ACTIONS(1985),
    [sym_user3_priv] = ACTIONS(1985),
    [sym_user4_priv] = ACTIONS(1985),
    [sym_usermanagement_priv] = ACTIONS(1985),
    [sym_valid_account] = ACTIONS(1985),
    [sym_err_name_invalid] = ACTIONS(1985),
    [sym_err_password_invalid] = ACTIONS(1985),
    [sym_err_invalid_account] = ACTIONS(1985),
    [sym_err_no_authorization] = ACTIONS(1985),
    [sym_err_invalid_parameter] = ACTIONS(1985),
    [sym_err_user_locked_out] = ACTIONS(1985),
    [sym_err_user_account_expired] = ACTIONS(1985),
    [sym_err_general_error] = ACTIONS(1985),
    [sym_err_authentication_error] = ACTIONS(1985),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1985),
    [sym_tls_validate_certificate] = ACTIONS(1985),
    [sym_tdata] = ACTIONS(1985),
    [sym_tchannel] = ACTIONS(1985),
    [sym_tlevel] = ACTIONS(1985),
    [sym_tbutton] = ACTIONS(1985),
    [sym_ttimeline] = ACTIONS(1985),
    [sym_tcustom] = ACTIONS(1985),
    [sym_url_struct] = ACTIONS(1985),
    [sym_dns_struct] = ACTIONS(1985),
    [sym_ip_address_struct] = ACTIONS(1985),
    [sym_dev_info_struct] = ACTIONS(1985),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1985),
    [sym_data_feed] = ACTIONS(1985),
    [sym_data_field] = ACTIONS(1985),
    [sym_data_record] = ACTIONS(1985),
    [sym_wc_data_feed] = ACTIONS(1985),
    [sym_wc_data_field] = ACTIONS(1985),
    [sym_wc_data_record] = ACTIONS(1985),
    [sym_last_login_info] = ACTIONS(1985),
    [sym____reserved___] = ACTIONS(1985),
    [sym_date] = ACTIONS(1985),
    [sym_day] = ACTIONS(1985),
    [sym_time] = ACTIONS(1985),
    [sym_ldate] = ACTIONS(1985),
    [sym_push_device] = ACTIONS(1985),
    [sym_push_channel] = ACTIONS(1985),
    [sym_push_devchan] = ACTIONS(1985),
    [sym_release_device] = ACTIONS(1985),
    [sym_release_channel] = ACTIONS(1985),
    [sym_release_devchan] = ACTIONS(1985),
    [sym_master_slot] = ACTIONS(1985),
    [sym_get_pulse_time] = ACTIONS(1985),
    [sym_get_timer] = ACTIONS(1985),
    [sym_master_sn] = ACTIONS(1985),
    [sym_system_number] = ACTIONS(1985),
    [sym_button] = ACTIONS(1985),
    [sym_dv_channel] = ACTIONS(1985),
    [sym_data] = ACTIONS(1985),
    [sym_level] = ACTIONS(1985),
    [sym_timeline] = ACTIONS(1985),
    [sym_channel] = ACTIONS(1985),
    [sym_custom] = ACTIONS(1985),
    [sym_atoi] = ACTIONS(1985),
    [sym_atol] = ACTIONS(1985),
    [sym_atof] = ACTIONS(1985),
    [sym_itoa] = ACTIONS(1985),
    [sym_itohex] = ACTIONS(1985),
    [sym_hextoi] = ACTIONS(1985),
    [sym_ftoa] = ACTIONS(1985),
    [sym_find_string] = ACTIONS(1985),
    [sym_left_string] = ACTIONS(1985),
    [sym_right_string] = ACTIONS(1985),
    [sym_mid_string] = ACTIONS(1985),
    [sym_remove_string] = ACTIONS(1985),
    [sym_get_buffer_string] = ACTIONS(1985),
    [sym_get_buffer_char] = ACTIONS(1985),
    [sym_get_multi_buffer_string] = ACTIONS(1985),
    [sym_length_string] = ACTIONS(1985),
    [sym_max_length_string] = ACTIONS(1985),
    [sym_set_length_string] = ACTIONS(1985),
    [sym_compare_string] = ACTIONS(1985),
    [sym_upper_string] = ACTIONS(1985),
    [sym_lower_string] = ACTIONS(1985),
    [sym_get_last] = ACTIONS(1985),
    [sym_date_to_day] = ACTIONS(1985),
    [sym_date_to_month] = ACTIONS(1985),
    [sym_date_to_year] = ACTIONS(1985),
    [sym_time_to_hour] = ACTIONS(1985),
    [sym_time_to_minute] = ACTIONS(1985),
    [sym_time_to_second] = ACTIONS(1985),
    [sym_day_of_week] = ACTIONS(1985),
    [sym_set_pulse_time] = ACTIONS(1985),
    [sym_set_timer] = ACTIONS(1985),
    [sym_file_open] = ACTIONS(1985),
    [sym_file_close] = ACTIONS(1985),
    [sym_file_read] = ACTIONS(1985),
    [sym_file_read_line] = ACTIONS(1985),
    [sym_file_write] = ACTIONS(1985),
    [sym_file_write_line] = ACTIONS(1985),
    [sym_file_copy] = ACTIONS(1985),
    [sym_file_delete] = ACTIONS(1985),
    [sym_file_rename] = ACTIONS(1985),
    [sym_file_dir] = ACTIONS(1985),
    [sym_file_getdir] = ACTIONS(1985),
    [sym_file_setdir] = ACTIONS(1985),
    [sym_file_create_dir] = ACTIONS(1985),
    [sym_file_remove_dir] = ACTIONS(1985),
    [sym_file_seek] = ACTIONS(1985),
    [sym_length_array] = ACTIONS(1985),
    [sym_max_length_array] = ACTIONS(1985),
    [sym_set_length_array] = ACTIONS(1985),
    [sym_ip_client_open] = ACTIONS(1985),
    [sym_ip_client_close] = ACTIONS(1985),
    [sym_ip_server_open] = ACTIONS(1985),
    [sym_ip_server_close] = ACTIONS(1985),
    [sym_ip_bound_client_open] = ACTIONS(1985),
    [sym_ip_mc_server_open] = ACTIONS(1985),
    [sym_ip_set_option] = ACTIONS(1985),
    [sym_ssh_client_open] = ACTIONS(1985),
    [sym_ssh_client_close] = ACTIONS(1985),
    [sym_tls_client_open] = ACTIONS(1985),
    [sym_tls_client_close] = ACTIONS(1985),
    [sym_do_push] = ACTIONS(1985),
    [sym_do_push_timed] = ACTIONS(1985),
    [sym_do_release] = ACTIONS(1985),
    [sym_device_id] = ACTIONS(1985),
    [sym_device_info] = ACTIONS(1985),
    [sym_device_id_string] = ACTIONS(1985),
    [sym_device_standby] = ACTIONS(1985),
    [sym_device_wake] = ACTIONS(1985),
    [sym_timeline_create] = ACTIONS(1985),
    [sym_timeline_kill] = ACTIONS(1985),
    [sym_timeline_reload] = ACTIONS(1985),
    [sym_timeline_pause] = ACTIONS(1985),
    [sym_timeline_restart] = ACTIONS(1985),
    [sym_timeline_set] = ACTIONS(1985),
    [sym_timeline_get] = ACTIONS(1985),
    [sym_timeline_active] = ACTIONS(1985),
    [sym_timeline_dynamic_id] = ACTIONS(1985),
    [sym_variable_to_string] = ACTIONS(1985),
    [sym_string_to_variable] = ACTIONS(1985),
    [sym_length_variable_to_string] = ACTIONS(1985),
    [sym_type_cast] = ACTIONS(1985),
    [sym_variable_to_xml] = ACTIONS(1985),
    [sym_xml_to_variable] = ACTIONS(1985),
    [sym_length_variable_to_xml] = ACTIONS(1985),
    [sym_redirect_string] = ACTIONS(1985),
    [sym_random_number] = ACTIONS(1985),
    [sym_get_unique_id] = ACTIONS(1985),
    [sym_get_system_number] = ACTIONS(1985),
    [sym_set_system_number] = ACTIONS(1985),
    [sym_get_serial_number] = ACTIONS(1985),
    [sym_get_master_build] = ACTIONS(1985),
    [sym_set_validation_code] = ACTIONS(1985),
    [sym_reboot] = ACTIONS(1985),
    [sym_load_duet_module] = ACTIONS(1985),
    [sym_unload_duet_module] = ACTIONS(1985),
    [sym_duet_mem_size_set] = ACTIONS(1985),
    [sym_duet_mem_size_get] = ACTIONS(1985),
    [sym_dynamic_application_device] = ACTIONS(1985),
    [sym_dynamic_polled_port] = ACTIONS(1985),
    [sym_static_port_binding] = ACTIONS(1985),
    [sym_static_ip_binding] = ACTIONS(1985),
    [sym_get_dns_list] = ACTIONS(1985),
    [sym_set_dns_list] = ACTIONS(1985),
    [sym_get_ip_address] = ACTIONS(1985),
    [sym_set_ip_address] = ACTIONS(1985),
    [sym_get_url_list] = ACTIONS(1985),
    [sym_add_url_entry] = ACTIONS(1985),
    [sym_delete_url_entry] = ACTIONS(1985),
    [sym_abs_value] = ACTIONS(1985),
    [sym_max_value] = ACTIONS(1985),
    [sym_min_value] = ACTIONS(1985),
    [sym_exp_value] = ACTIONS(1985),
    [sym_log_value] = ACTIONS(1985),
    [sym_log10_value] = ACTIONS(1985),
    [sym_power_value] = ACTIONS(1985),
    [sym_sqrt_value] = ACTIONS(1985),
    [sym_raw_be] = ACTIONS(1985),
    [sym_raw_le] = ACTIONS(1985),
    [sym_internal_queue_size_set] = ACTIONS(1985),
    [sym_internal_queue_size_get] = ACTIONS(1985),
    [sym_internal_threshold_set] = ACTIONS(1985),
    [sym_internal_threshold_get] = ACTIONS(1985),
    [sym_rebuilt_event] = ACTIONS(1985),
    [sym_do_custom_event] = ACTIONS(1985),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1985),
    [sym_clkmgr_set_clk_source] = ACTIONS(1985),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1985),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1985),
    [sym_clkmgr_get_timezone] = ACTIONS(1985),
    [sym_clkmgr_set_timezone] = ACTIONS(1985),
    [sym_clkmgr_get_resync_period] = ACTIONS(1985),
    [sym_clkmgr_set_resync_period] = ACTIONS(1985),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1985),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1985),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1985),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1985),
    [sym_clkmgr_get_timeservers] = ACTIONS(1985),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1985),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1985),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1985),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1985),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1985),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1985),
    [sym_set_log_level] = ACTIONS(1985),
    [sym_get_log_level] = ACTIONS(1985),
    [sym_amx_log] = ACTIONS(1985),
    [sym_smtp_server_config_set] = ACTIONS(1985),
    [sym_smtp_server_config_get] = ACTIONS(1985),
    [sym_smtp_send] = ACTIONS(1985),
    [sym_validate_netlinx_account] = ACTIONS(1985),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1985),
    [sym_authenticate_certificate] = ACTIONS(1985),
    [sym_audit_netlinx_session_event] = ACTIONS(1985),
    [sym_audit_netlinx_generic_event] = ACTIONS(1985),
    [sym_get_available_flash_disk_space] = ACTIONS(1985),
    [sym_get_max_flash_disk_space] = ACTIONS(1985),
    [sym_data_create_feed] = ACTIONS(1985),
    [sym_data_delete_feed] = ACTIONS(1985),
    [sym_data_publish_feed] = ACTIONS(1985),
    [sym_data_get_published_feed] = ACTIONS(1985),
    [sym_data_add_record] = ACTIONS(1985),
    [sym_data_get_event_record] = ACTIONS(1985),
    [sym__wc_data_create_feed] = ACTIONS(1985),
    [sym__wc_data_add_record] = ACTIONS(1985),
    [sym__wc_data_get_event_record] = ACTIONS(1985),
    [sym_format] = ACTIONS(1985),
    [sym_astro_clock] = ACTIONS(1985),
    [sym_set_outdoor_temperature] = ACTIONS(1985),
    [sym_set_virtual_level_count] = ACTIONS(1985),
    [sym_set_virtual_channel_count] = ACTIONS(1985),
    [sym_set_virtual_port_count] = ACTIONS(1985),
    [sym__wc_encode_int] = ACTIONS(1985),
    [sym__wc_decode_int] = ACTIONS(1985),
    [sym__wc_int] = ACTIONS(1985),
    [sym__wc_to_ch_int] = ACTIONS(1985),
    [sym__ch_to_wc_int] = ACTIONS(1985),
    [sym__wc_find_string_int] = ACTIONS(1985),
    [sym__wc_left_string_int] = ACTIONS(1985),
    [sym__wc_lower_string_int] = ACTIONS(1985),
    [sym__wc_mid_string_int] = ACTIONS(1985),
    [sym__wc_remove_string_int] = ACTIONS(1985),
    [sym__wc_right_string_int] = ACTIONS(1985),
    [sym__wc_upper_string_int] = ACTIONS(1985),
    [sym__wc_compare_string_int] = ACTIONS(1985),
    [sym__wc_get_buffer_char_int] = ACTIONS(1985),
    [sym__wc_get_buffer_string_int] = ACTIONS(1985),
    [sym__wc_concat_string_int] = ACTIONS(1985),
    [sym__wc_file_open_int] = ACTIONS(1985),
    [sym__wc_file_close_int] = ACTIONS(1985),
    [sym__wc_file_read_int] = ACTIONS(1985),
    [sym__wc_file_read_line_int] = ACTIONS(1985),
    [sym__wc_file_write_int] = ACTIONS(1985),
    [sym__wc_file_write_line_int] = ACTIONS(1985),
    [aux_sym_preproc_include_token1] = ACTIONS(1985),
    [aux_sym_preproc_define_token1] = ACTIONS(1987),
    [aux_sym_preproc_warn_token1] = ACTIONS(1987),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1987),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1987),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1987),
    [anon_sym_LBRACK] = ACTIONS(1987),
    [anon_sym_LBRACE] = ACTIONS(1987),
    [anon_sym_LPAREN] = ACTIONS(1985),
    [anon_sym_BANG] = ACTIONS(1987),
    [anon_sym_TILDE] = ACTIONS(1987),
    [anon_sym_DASH] = ACTIONS(1985),
    [anon_sym_PLUS] = ACTIONS(1985),
    [anon_sym_DASH_DASH] = ACTIONS(1987),
    [anon_sym_PLUS_PLUS] = ACTIONS(1987),
    [anon_sym_DQUOTE] = ACTIONS(1987),
    [anon_sym_SQUOTE] = ACTIONS(1987),
    [sym_number_literal] = ACTIONS(1987),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(424)] = {
    [ts_builtin_sym_end] = ACTIONS(1991),
    [sym_identifier] = ACTIONS(1989),
    [sym_program_name_keyword] = ACTIONS(1989),
    [sym_module_name_keyword] = ACTIONS(1989),
    [sym_define_device_keyword] = ACTIONS(1989),
    [sym_define_combine_keyword] = ACTIONS(1989),
    [sym_define_constant_keyword] = ACTIONS(1989),
    [sym_define_type_keyword] = ACTIONS(1989),
    [sym_define_variable_keyword] = ACTIONS(1989),
    [sym_define_system_variable_keyword] = ACTIONS(1989),
    [sym_define_connect_level_keyword] = ACTIONS(1989),
    [sym_define_latching_keyword] = ACTIONS(1989),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1989),
    [sym_define_toggling_keyword] = ACTIONS(1989),
    [sym_define_call_keyword] = ACTIONS(1989),
    [sym_define_function_keyword] = ACTIONS(1989),
    [sym_define_library_function_keyword] = ACTIONS(1989),
    [sym_define_start_keyword] = ACTIONS(1989),
    [sym_define_event_keyword] = ACTIONS(1989),
    [sym_define_program_keyword] = ACTIONS(1989),
    [sym_define_module_keyword] = ACTIONS(1989),
    [sym_char_keyword] = ACTIONS(1989),
    [sym_widechar_keyword] = ACTIONS(1989),
    [sym_integer_keyword] = ACTIONS(1989),
    [sym_sinteger_keyword] = ACTIONS(1989),
    [sym_long_keyword] = ACTIONS(1989),
    [sym_slong_keyword] = ACTIONS(1989),
    [sym_float_keyword] = ACTIONS(1989),
    [sym_double_keyword] = ACTIONS(1989),
    [sym_variant_keyword] = ACTIONS(1989),
    [sym_variantarray_keyword] = ACTIONS(1989),
    [sym_local_var_keyword] = ACTIONS(1989),
    [sym_stack_var_keyword] = ACTIONS(1989),
    [sym_constant_keyword] = ACTIONS(1989),
    [sym_volatile_keyword] = ACTIONS(1989),
    [sym_non_volatile_keyword] = ACTIONS(1989),
    [sym_persistent_keyword] = ACTIONS(1989),
    [sym_dev_keyword] = ACTIONS(1989),
    [sym_devlev_keyword] = ACTIONS(1989),
    [sym_devchan_keyword] = ACTIONS(1989),
    [sym_if_keyword] = ACTIONS(1989),
    [sym_switch_keyword] = ACTIONS(1989),
    [sym_case_keyword] = ACTIONS(1989),
    [sym_default_keyword] = ACTIONS(1989),
    [sym_while_keyword] = ACTIONS(1989),
    [sym_for_keyword] = ACTIONS(1989),
    [sym_break_keyword] = ACTIONS(1989),
    [sym_continue_keyword] = ACTIONS(1989),
    [sym_return_keyword] = ACTIONS(1989),
    [sym_select_keyword] = ACTIONS(1989),
    [sym_active_keyword] = ACTIONS(1989),
    [sym_struct_keyword] = ACTIONS(1989),
    [sym_structure_keyword] = ACTIONS(1989),
    [sym_button_event_keyword] = ACTIONS(1989),
    [sym_channel_event_keyword] = ACTIONS(1989),
    [sym_level_event_keyword] = ACTIONS(1989),
    [sym_data_event_keyword] = ACTIONS(1989),
    [sym_timeline_event_keyword] = ACTIONS(1989),
    [sym_custom_event_keyword] = ACTIONS(1989),
    [sym_push_keyword] = ACTIONS(1989),
    [sym_release_keyword] = ACTIONS(1989),
    [aux_sym_on_keyword_token1] = ACTIONS(1989),
    [aux_sym_off_keyword_token1] = ACTIONS(1989),
    [sym_send_level_keyword] = ACTIONS(1989),
    [sym_send_string_keyword] = ACTIONS(1989),
    [sym_send_command_keyword] = ACTIONS(1989),
    [sym_clear_buffer_keyword] = ACTIONS(1989),
    [sym_create_buffer_keyword] = ACTIONS(1989),
    [sym_create_multi_buffer_keyword] = ACTIONS(1989),
    [sym_call_keyword] = ACTIONS(1989),
    [sym_system_call_keyword] = ACTIONS(1989),
    [sym_devchan_to_keyword] = ACTIONS(1989),
    [sym_devchan_min_to_keyword] = ACTIONS(1989),
    [sym_devchan_total_off_keyword] = ACTIONS(1989),
    [sym_devchan_pulse_keyword] = ACTIONS(1989),
    [sym_wait_keyword] = ACTIONS(1989),
    [sym_cancel_wait_keyword] = ACTIONS(1989),
    [sym_wait_until_keyword] = ACTIONS(1989),
    [sym_cancel_wait_until_keyword] = ACTIONS(1989),
    [sym_cancel_all_wait_keyword] = ACTIONS(1989),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1989),
    [sym_not] = ACTIONS(1989),
    [sym_bnot] = ACTIONS(1989),
    [sym___file__] = ACTIONS(1989),
    [sym___date__] = ACTIONS(1989),
    [sym___ldate__] = ACTIONS(1989),
    [sym___time__] = ACTIONS(1989),
    [sym___line__] = ACTIONS(1989),
    [sym___name__] = ACTIONS(1989),
    [sym___netlinx__] = ACTIONS(1989),
    [sym_netlinx_axi_version] = ACTIONS(1989),
    [sym_url_flg_tcp] = ACTIONS(1989),
    [sym_url_flg_accinfopresent] = ACTIONS(1989),
    [sym_url_flg_temp] = ACTIONS(1989),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1989),
    [sym_url_flg_stat_mask] = ACTIONS(1989),
    [sym_url_flg_stat_lookup] = ACTIONS(1989),
    [sym_url_flg_stat_connecting] = ACTIONS(1989),
    [sym_url_flg_stat_waiting] = ACTIONS(1989),
    [sym_url_flg_stat_connected] = ACTIONS(1989),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1989),
    [sym_first_virtual_device] = ACTIONS(1989),
    [sym_first_local_port] = ACTIONS(1989),
    [sym_dynamic_virtual_device] = ACTIONS(1989),
    [sym_timeline_once] = ACTIONS(1989),
    [sym_timeline_repeat] = ACTIONS(1989),
    [sym_timeline_absolute] = ACTIONS(1989),
    [sym_timeline_relative] = ACTIONS(1989),
    [sym_do_push_timed_infinite] = ACTIONS(1989),
    [sym_source_type_no_address] = ACTIONS(1989),
    [sym_source_type_neuron_id] = ACTIONS(1989),
    [sym_source_type_ip_address] = ACTIONS(1989),
    [sym_source_type_axlink] = ACTIONS(1989),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1989),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1989),
    [sym_source_type_ip_socket_address] = ACTIONS(1989),
    [sym_source_type_rs232] = ACTIONS(1989),
    [sym_source_type_internal] = ACTIONS(1989),
    [sym_source_type_mac_address] = ACTIONS(1989),
    [sym_source_type_ipv4_port] = ACTIONS(1989),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1989),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1989),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1989),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1989),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1989),
    [sym_true] = ACTIONS(1989),
    [sym_false] = ACTIONS(1989),
    [sym_file_read_only] = ACTIONS(1989),
    [sym_file_rw_new] = ACTIONS(1989),
    [sym_file_rw_append] = ACTIONS(1989),
    [sym_ip_tcp] = ACTIONS(1989),
    [sym_ip_udp] = ACTIONS(1989),
    [sym_ip_udp_2way] = ACTIONS(1989),
    [sym_xml_encode_types] = ACTIONS(1989),
    [sym_xml_encode_char_as_list] = ACTIONS(1989),
    [sym_xml_encode_le] = ACTIONS(1989),
    [sym_xml_decode_types] = ACTIONS(1989),
    [sym_xml_decode_no_preserve] = ACTIONS(1989),
    [sym_duet_dev_type_display_device] = ACTIONS(1989),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1989),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1989),
    [sym_duet_dev_type_io_device] = ACTIONS(1989),
    [sym_duet_dev_type_relay_device] = ACTIONS(1989),
    [sym_duet_dev_type_ups] = ACTIONS(1989),
    [sym_duet_dev_type_amplifier] = ACTIONS(1989),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1989),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1989),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1989),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1989),
    [sym_duet_dev_type_camera] = ACTIONS(1989),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1989),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1989),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1989),
    [sym_duet_dev_type_dss] = ACTIONS(1989),
    [sym_duet_dev_type_dvr] = ACTIONS(1989),
    [sym_duet_dev_type_disc_device] = ACTIONS(1989),
    [sym_duet_dev_type_document_camera] = ACTIONS(1989),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1989),
    [sym_duet_dev_type_hvac] = ACTIONS(1989),
    [sym_duet_dev_type_keypad] = ACTIONS(1989),
    [sym_duet_dev_type_light] = ACTIONS(1989),
    [sym_duet_dev_type_monitor] = ACTIONS(1989),
    [sym_duet_dev_type_motor] = ACTIONS(1989),
    [sym_duet_dev_type_multi_window] = ACTIONS(1989),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1989),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1989),
    [sym_duet_dev_type_receiver] = ACTIONS(1989),
    [sym_duet_dev_type_security_system] = ACTIONS(1989),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1989),
    [sym_duet_dev_type_settop_box] = ACTIONS(1989),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1989),
    [sym_duet_dev_type_switcher] = ACTIONS(1989),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1989),
    [sym_duet_dev_type_tv] = ACTIONS(1989),
    [sym_duet_dev_type_utility] = ACTIONS(1989),
    [sym_duet_dev_type_vcr] = ACTIONS(1989),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1989),
    [sym_duet_dev_type_video_processor] = ACTIONS(1989),
    [sym_duet_dev_type_video_projector] = ACTIONS(1989),
    [sym_duet_dev_type_video_wall] = ACTIONS(1989),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1989),
    [sym_duet_dev_type_weather] = ACTIONS(1989),
    [sym_duet_dev_not_polled] = ACTIONS(1989),
    [sym_duet_dev_polled] = ACTIONS(1989),
    [sym_clkmgr_mode_network] = ACTIONS(1989),
    [sym_clkmgr_mode_standalone] = ACTIONS(1989),
    [sym_normal_standby] = ACTIONS(1989),
    [sym_normal_wake] = ACTIONS(1989),
    [sym_ip_multicast_ttl_option] = ACTIONS(1989),
    [sym_ip_ttl_subnet] = ACTIONS(1989),
    [sym_ip_ttl_site] = ACTIONS(1989),
    [sym_ip_ttl_region] = ACTIONS(1989),
    [sym_ip_ttl_continent] = ACTIONS(1989),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1989),
    [sym_ip_nodelay_on] = ACTIONS(1989),
    [sym_ip_nodelay_off] = ACTIONS(1989),
    [sym_amx_error] = ACTIONS(1989),
    [sym_amx_warning] = ACTIONS(1989),
    [sym_amx_info] = ACTIONS(1989),
    [sym_amx_debug] = ACTIONS(1989),
    [sym_netlinx_login_success] = ACTIONS(1989),
    [sym_netlinx_login_fail] = ACTIONS(1989),
    [sym_netlinx_logout] = ACTIONS(1989),
    [sym_null_str] = ACTIONS(1989),
    [sym_smtp_address] = ACTIONS(1989),
    [sym_smtp_port_number] = ACTIONS(1989),
    [sym_smtp_username] = ACTIONS(1989),
    [sym_smtp_password] = ACTIONS(1989),
    [sym_smtp_require_tls] = ACTIONS(1989),
    [sym_smtp_from] = ACTIONS(1989),
    [sym_smtp_tls_true] = ACTIONS(1989),
    [sym_smtp_tls_false] = ACTIONS(1989),
    [sym_listview_on_row_select_event] = ACTIONS(1989),
    [sym_data_structure_unknown] = ACTIONS(1989),
    [sym_data_structure_datafeed] = ACTIONS(1989),
    [sym_data_structure_datarecordset] = ACTIONS(1989),
    [sym_data_structure_datarecord] = ACTIONS(1989),
    [sym_data_structure_datafield] = ACTIONS(1989),
    [sym_data_max_content_field_count] = ACTIONS(1989),
    [sym_data_max_metadata_field_count] = ACTIONS(1989),
    [sym_data_max_id_length] = ACTIONS(1989),
    [sym_data_max_name_length] = ACTIONS(1989),
    [sym_data_max_description_length] = ACTIONS(1989),
    [sym_data_max_type_length] = ACTIONS(1989),
    [sym_data_max_format_length] = ACTIONS(1989),
    [sym_data_max_source_length] = ACTIONS(1989),
    [sym_data_max_value_length] = ACTIONS(1989),
    [sym_data_max_label_length] = ACTIONS(1989),
    [sym_data_max_url_length] = ACTIONS(1989),
    [sym_max_last_login_info_length] = ACTIONS(1989),
    [sym_data_type_unknown] = ACTIONS(1989),
    [sym_data_type_string] = ACTIONS(1989),
    [sym_data_type_datetime] = ACTIONS(1989),
    [sym_data_type_date] = ACTIONS(1989),
    [sym_data_type_time] = ACTIONS(1989),
    [sym_data_type_image] = ACTIONS(1989),
    [sym_data_format_url] = ACTIONS(1989),
    [sym_data_format_phone] = ACTIONS(1989),
    [sym_data_format_email] = ACTIONS(1989),
    [sym_data_format_iso8601] = ACTIONS(1989),
    [sym_auditlog_priv] = ACTIONS(1989),
    [sym_configuration_priv] = ACTIONS(1989),
    [sym_deviceconfig_priv] = ACTIONS(1989),
    [sym_ftp_priv] = ACTIONS(1989),
    [sym_http_priv] = ACTIONS(1989),
    [sym_networkconfig_priv] = ACTIONS(1989),
    [sym_programport_priv] = ACTIONS(1989),
    [sym_remoteui_priv] = ACTIONS(1989),
    [sym_securitycontrol_priv] = ACTIONS(1989),
    [sym_softwaremanagement_priv] = ACTIONS(1989),
    [sym_terminal_priv] = ACTIONS(1989),
    [sym_tpadmin_priv] = ACTIONS(1989),
    [sym_user1_priv] = ACTIONS(1989),
    [sym_user2_priv] = ACTIONS(1989),
    [sym_user3_priv] = ACTIONS(1989),
    [sym_user4_priv] = ACTIONS(1989),
    [sym_usermanagement_priv] = ACTIONS(1989),
    [sym_valid_account] = ACTIONS(1989),
    [sym_err_name_invalid] = ACTIONS(1989),
    [sym_err_password_invalid] = ACTIONS(1989),
    [sym_err_invalid_account] = ACTIONS(1989),
    [sym_err_no_authorization] = ACTIONS(1989),
    [sym_err_invalid_parameter] = ACTIONS(1989),
    [sym_err_user_locked_out] = ACTIONS(1989),
    [sym_err_user_account_expired] = ACTIONS(1989),
    [sym_err_general_error] = ACTIONS(1989),
    [sym_err_authentication_error] = ACTIONS(1989),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1989),
    [sym_tls_validate_certificate] = ACTIONS(1989),
    [sym_tdata] = ACTIONS(1989),
    [sym_tchannel] = ACTIONS(1989),
    [sym_tlevel] = ACTIONS(1989),
    [sym_tbutton] = ACTIONS(1989),
    [sym_ttimeline] = ACTIONS(1989),
    [sym_tcustom] = ACTIONS(1989),
    [sym_url_struct] = ACTIONS(1989),
    [sym_dns_struct] = ACTIONS(1989),
    [sym_ip_address_struct] = ACTIONS(1989),
    [sym_dev_info_struct] = ACTIONS(1989),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1989),
    [sym_data_feed] = ACTIONS(1989),
    [sym_data_field] = ACTIONS(1989),
    [sym_data_record] = ACTIONS(1989),
    [sym_wc_data_feed] = ACTIONS(1989),
    [sym_wc_data_field] = ACTIONS(1989),
    [sym_wc_data_record] = ACTIONS(1989),
    [sym_last_login_info] = ACTIONS(1989),
    [sym____reserved___] = ACTIONS(1989),
    [sym_date] = ACTIONS(1989),
    [sym_day] = ACTIONS(1989),
    [sym_time] = ACTIONS(1989),
    [sym_ldate] = ACTIONS(1989),
    [sym_push_device] = ACTIONS(1989),
    [sym_push_channel] = ACTIONS(1989),
    [sym_push_devchan] = ACTIONS(1989),
    [sym_release_device] = ACTIONS(1989),
    [sym_release_channel] = ACTIONS(1989),
    [sym_release_devchan] = ACTIONS(1989),
    [sym_master_slot] = ACTIONS(1989),
    [sym_get_pulse_time] = ACTIONS(1989),
    [sym_get_timer] = ACTIONS(1989),
    [sym_master_sn] = ACTIONS(1989),
    [sym_system_number] = ACTIONS(1989),
    [sym_button] = ACTIONS(1989),
    [sym_dv_channel] = ACTIONS(1989),
    [sym_data] = ACTIONS(1989),
    [sym_level] = ACTIONS(1989),
    [sym_timeline] = ACTIONS(1989),
    [sym_channel] = ACTIONS(1989),
    [sym_custom] = ACTIONS(1989),
    [sym_atoi] = ACTIONS(1989),
    [sym_atol] = ACTIONS(1989),
    [sym_atof] = ACTIONS(1989),
    [sym_itoa] = ACTIONS(1989),
    [sym_itohex] = ACTIONS(1989),
    [sym_hextoi] = ACTIONS(1989),
    [sym_ftoa] = ACTIONS(1989),
    [sym_find_string] = ACTIONS(1989),
    [sym_left_string] = ACTIONS(1989),
    [sym_right_string] = ACTIONS(1989),
    [sym_mid_string] = ACTIONS(1989),
    [sym_remove_string] = ACTIONS(1989),
    [sym_get_buffer_string] = ACTIONS(1989),
    [sym_get_buffer_char] = ACTIONS(1989),
    [sym_get_multi_buffer_string] = ACTIONS(1989),
    [sym_length_string] = ACTIONS(1989),
    [sym_max_length_string] = ACTIONS(1989),
    [sym_set_length_string] = ACTIONS(1989),
    [sym_compare_string] = ACTIONS(1989),
    [sym_upper_string] = ACTIONS(1989),
    [sym_lower_string] = ACTIONS(1989),
    [sym_get_last] = ACTIONS(1989),
    [sym_date_to_day] = ACTIONS(1989),
    [sym_date_to_month] = ACTIONS(1989),
    [sym_date_to_year] = ACTIONS(1989),
    [sym_time_to_hour] = ACTIONS(1989),
    [sym_time_to_minute] = ACTIONS(1989),
    [sym_time_to_second] = ACTIONS(1989),
    [sym_day_of_week] = ACTIONS(1989),
    [sym_set_pulse_time] = ACTIONS(1989),
    [sym_set_timer] = ACTIONS(1989),
    [sym_file_open] = ACTIONS(1989),
    [sym_file_close] = ACTIONS(1989),
    [sym_file_read] = ACTIONS(1989),
    [sym_file_read_line] = ACTIONS(1989),
    [sym_file_write] = ACTIONS(1989),
    [sym_file_write_line] = ACTIONS(1989),
    [sym_file_copy] = ACTIONS(1989),
    [sym_file_delete] = ACTIONS(1989),
    [sym_file_rename] = ACTIONS(1989),
    [sym_file_dir] = ACTIONS(1989),
    [sym_file_getdir] = ACTIONS(1989),
    [sym_file_setdir] = ACTIONS(1989),
    [sym_file_create_dir] = ACTIONS(1989),
    [sym_file_remove_dir] = ACTIONS(1989),
    [sym_file_seek] = ACTIONS(1989),
    [sym_length_array] = ACTIONS(1989),
    [sym_max_length_array] = ACTIONS(1989),
    [sym_set_length_array] = ACTIONS(1989),
    [sym_ip_client_open] = ACTIONS(1989),
    [sym_ip_client_close] = ACTIONS(1989),
    [sym_ip_server_open] = ACTIONS(1989),
    [sym_ip_server_close] = ACTIONS(1989),
    [sym_ip_bound_client_open] = ACTIONS(1989),
    [sym_ip_mc_server_open] = ACTIONS(1989),
    [sym_ip_set_option] = ACTIONS(1989),
    [sym_ssh_client_open] = ACTIONS(1989),
    [sym_ssh_client_close] = ACTIONS(1989),
    [sym_tls_client_open] = ACTIONS(1989),
    [sym_tls_client_close] = ACTIONS(1989),
    [sym_do_push] = ACTIONS(1989),
    [sym_do_push_timed] = ACTIONS(1989),
    [sym_do_release] = ACTIONS(1989),
    [sym_device_id] = ACTIONS(1989),
    [sym_device_info] = ACTIONS(1989),
    [sym_device_id_string] = ACTIONS(1989),
    [sym_device_standby] = ACTIONS(1989),
    [sym_device_wake] = ACTIONS(1989),
    [sym_timeline_create] = ACTIONS(1989),
    [sym_timeline_kill] = ACTIONS(1989),
    [sym_timeline_reload] = ACTIONS(1989),
    [sym_timeline_pause] = ACTIONS(1989),
    [sym_timeline_restart] = ACTIONS(1989),
    [sym_timeline_set] = ACTIONS(1989),
    [sym_timeline_get] = ACTIONS(1989),
    [sym_timeline_active] = ACTIONS(1989),
    [sym_timeline_dynamic_id] = ACTIONS(1989),
    [sym_variable_to_string] = ACTIONS(1989),
    [sym_string_to_variable] = ACTIONS(1989),
    [sym_length_variable_to_string] = ACTIONS(1989),
    [sym_type_cast] = ACTIONS(1989),
    [sym_variable_to_xml] = ACTIONS(1989),
    [sym_xml_to_variable] = ACTIONS(1989),
    [sym_length_variable_to_xml] = ACTIONS(1989),
    [sym_redirect_string] = ACTIONS(1989),
    [sym_random_number] = ACTIONS(1989),
    [sym_get_unique_id] = ACTIONS(1989),
    [sym_get_system_number] = ACTIONS(1989),
    [sym_set_system_number] = ACTIONS(1989),
    [sym_get_serial_number] = ACTIONS(1989),
    [sym_get_master_build] = ACTIONS(1989),
    [sym_set_validation_code] = ACTIONS(1989),
    [sym_reboot] = ACTIONS(1989),
    [sym_load_duet_module] = ACTIONS(1989),
    [sym_unload_duet_module] = ACTIONS(1989),
    [sym_duet_mem_size_set] = ACTIONS(1989),
    [sym_duet_mem_size_get] = ACTIONS(1989),
    [sym_dynamic_application_device] = ACTIONS(1989),
    [sym_dynamic_polled_port] = ACTIONS(1989),
    [sym_static_port_binding] = ACTIONS(1989),
    [sym_static_ip_binding] = ACTIONS(1989),
    [sym_get_dns_list] = ACTIONS(1989),
    [sym_set_dns_list] = ACTIONS(1989),
    [sym_get_ip_address] = ACTIONS(1989),
    [sym_set_ip_address] = ACTIONS(1989),
    [sym_get_url_list] = ACTIONS(1989),
    [sym_add_url_entry] = ACTIONS(1989),
    [sym_delete_url_entry] = ACTIONS(1989),
    [sym_abs_value] = ACTIONS(1989),
    [sym_max_value] = ACTIONS(1989),
    [sym_min_value] = ACTIONS(1989),
    [sym_exp_value] = ACTIONS(1989),
    [sym_log_value] = ACTIONS(1989),
    [sym_log10_value] = ACTIONS(1989),
    [sym_power_value] = ACTIONS(1989),
    [sym_sqrt_value] = ACTIONS(1989),
    [sym_raw_be] = ACTIONS(1989),
    [sym_raw_le] = ACTIONS(1989),
    [sym_internal_queue_size_set] = ACTIONS(1989),
    [sym_internal_queue_size_get] = ACTIONS(1989),
    [sym_internal_threshold_set] = ACTIONS(1989),
    [sym_internal_threshold_get] = ACTIONS(1989),
    [sym_rebuilt_event] = ACTIONS(1989),
    [sym_do_custom_event] = ACTIONS(1989),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1989),
    [sym_clkmgr_set_clk_source] = ACTIONS(1989),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1989),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1989),
    [sym_clkmgr_get_timezone] = ACTIONS(1989),
    [sym_clkmgr_set_timezone] = ACTIONS(1989),
    [sym_clkmgr_get_resync_period] = ACTIONS(1989),
    [sym_clkmgr_set_resync_period] = ACTIONS(1989),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1989),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1989),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1989),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1989),
    [sym_clkmgr_get_timeservers] = ACTIONS(1989),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1989),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1989),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1989),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1989),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1989),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1989),
    [sym_set_log_level] = ACTIONS(1989),
    [sym_get_log_level] = ACTIONS(1989),
    [sym_amx_log] = ACTIONS(1989),
    [sym_smtp_server_config_set] = ACTIONS(1989),
    [sym_smtp_server_config_get] = ACTIONS(1989),
    [sym_smtp_send] = ACTIONS(1989),
    [sym_validate_netlinx_account] = ACTIONS(1989),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1989),
    [sym_authenticate_certificate] = ACTIONS(1989),
    [sym_audit_netlinx_session_event] = ACTIONS(1989),
    [sym_audit_netlinx_generic_event] = ACTIONS(1989),
    [sym_get_available_flash_disk_space] = ACTIONS(1989),
    [sym_get_max_flash_disk_space] = ACTIONS(1989),
    [sym_data_create_feed] = ACTIONS(1989),
    [sym_data_delete_feed] = ACTIONS(1989),
    [sym_data_publish_feed] = ACTIONS(1989),
    [sym_data_get_published_feed] = ACTIONS(1989),
    [sym_data_add_record] = ACTIONS(1989),
    [sym_data_get_event_record] = ACTIONS(1989),
    [sym__wc_data_create_feed] = ACTIONS(1989),
    [sym__wc_data_add_record] = ACTIONS(1989),
    [sym__wc_data_get_event_record] = ACTIONS(1989),
    [sym_format] = ACTIONS(1989),
    [sym_astro_clock] = ACTIONS(1989),
    [sym_set_outdoor_temperature] = ACTIONS(1989),
    [sym_set_virtual_level_count] = ACTIONS(1989),
    [sym_set_virtual_channel_count] = ACTIONS(1989),
    [sym_set_virtual_port_count] = ACTIONS(1989),
    [sym__wc_encode_int] = ACTIONS(1989),
    [sym__wc_decode_int] = ACTIONS(1989),
    [sym__wc_int] = ACTIONS(1989),
    [sym__wc_to_ch_int] = ACTIONS(1989),
    [sym__ch_to_wc_int] = ACTIONS(1989),
    [sym__wc_find_string_int] = ACTIONS(1989),
    [sym__wc_left_string_int] = ACTIONS(1989),
    [sym__wc_lower_string_int] = ACTIONS(1989),
    [sym__wc_mid_string_int] = ACTIONS(1989),
    [sym__wc_remove_string_int] = ACTIONS(1989),
    [sym__wc_right_string_int] = ACTIONS(1989),
    [sym__wc_upper_string_int] = ACTIONS(1989),
    [sym__wc_compare_string_int] = ACTIONS(1989),
    [sym__wc_get_buffer_char_int] = ACTIONS(1989),
    [sym__wc_get_buffer_string_int] = ACTIONS(1989),
    [sym__wc_concat_string_int] = ACTIONS(1989),
    [sym__wc_file_open_int] = ACTIONS(1989),
    [sym__wc_file_close_int] = ACTIONS(1989),
    [sym__wc_file_read_int] = ACTIONS(1989),
    [sym__wc_file_read_line_int] = ACTIONS(1989),
    [sym__wc_file_write_int] = ACTIONS(1989),
    [sym__wc_file_write_line_int] = ACTIONS(1989),
    [aux_sym_preproc_include_token1] = ACTIONS(1989),
    [aux_sym_preproc_define_token1] = ACTIONS(1991),
    [aux_sym_preproc_warn_token1] = ACTIONS(1991),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1991),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1991),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1991),
    [anon_sym_LBRACK] = ACTIONS(1991),
    [anon_sym_LBRACE] = ACTIONS(1991),
    [anon_sym_LPAREN] = ACTIONS(1989),
    [anon_sym_BANG] = ACTIONS(1991),
    [anon_sym_TILDE] = ACTIONS(1991),
    [anon_sym_DASH] = ACTIONS(1989),
    [anon_sym_PLUS] = ACTIONS(1989),
    [anon_sym_DASH_DASH] = ACTIONS(1991),
    [anon_sym_PLUS_PLUS] = ACTIONS(1991),
    [anon_sym_DQUOTE] = ACTIONS(1991),
    [anon_sym_SQUOTE] = ACTIONS(1991),
    [sym_number_literal] = ACTIONS(1991),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(425)] = {
    [ts_builtin_sym_end] = ACTIONS(2055),
    [sym_identifier] = ACTIONS(2053),
    [sym_program_name_keyword] = ACTIONS(2053),
    [sym_module_name_keyword] = ACTIONS(2053),
    [sym_define_device_keyword] = ACTIONS(2053),
    [sym_define_combine_keyword] = ACTIONS(2053),
    [sym_define_constant_keyword] = ACTIONS(2053),
    [sym_define_type_keyword] = ACTIONS(2053),
    [sym_define_variable_keyword] = ACTIONS(2053),
    [sym_define_system_variable_keyword] = ACTIONS(2053),
    [sym_define_connect_level_keyword] = ACTIONS(2053),
    [sym_define_latching_keyword] = ACTIONS(2053),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2053),
    [sym_define_toggling_keyword] = ACTIONS(2053),
    [sym_define_call_keyword] = ACTIONS(2053),
    [sym_define_function_keyword] = ACTIONS(2053),
    [sym_define_library_function_keyword] = ACTIONS(2053),
    [sym_define_start_keyword] = ACTIONS(2053),
    [sym_define_event_keyword] = ACTIONS(2053),
    [sym_define_program_keyword] = ACTIONS(2053),
    [sym_define_module_keyword] = ACTIONS(2053),
    [sym_char_keyword] = ACTIONS(2053),
    [sym_widechar_keyword] = ACTIONS(2053),
    [sym_integer_keyword] = ACTIONS(2053),
    [sym_sinteger_keyword] = ACTIONS(2053),
    [sym_long_keyword] = ACTIONS(2053),
    [sym_slong_keyword] = ACTIONS(2053),
    [sym_float_keyword] = ACTIONS(2053),
    [sym_double_keyword] = ACTIONS(2053),
    [sym_variant_keyword] = ACTIONS(2053),
    [sym_variantarray_keyword] = ACTIONS(2053),
    [sym_local_var_keyword] = ACTIONS(2053),
    [sym_stack_var_keyword] = ACTIONS(2053),
    [sym_constant_keyword] = ACTIONS(2053),
    [sym_volatile_keyword] = ACTIONS(2053),
    [sym_non_volatile_keyword] = ACTIONS(2053),
    [sym_persistent_keyword] = ACTIONS(2053),
    [sym_dev_keyword] = ACTIONS(2053),
    [sym_devlev_keyword] = ACTIONS(2053),
    [sym_devchan_keyword] = ACTIONS(2053),
    [sym_if_keyword] = ACTIONS(2053),
    [sym_switch_keyword] = ACTIONS(2053),
    [sym_case_keyword] = ACTIONS(2053),
    [sym_default_keyword] = ACTIONS(2053),
    [sym_while_keyword] = ACTIONS(2053),
    [sym_for_keyword] = ACTIONS(2053),
    [sym_break_keyword] = ACTIONS(2053),
    [sym_continue_keyword] = ACTIONS(2053),
    [sym_return_keyword] = ACTIONS(2053),
    [sym_select_keyword] = ACTIONS(2053),
    [sym_active_keyword] = ACTIONS(2053),
    [sym_struct_keyword] = ACTIONS(2053),
    [sym_structure_keyword] = ACTIONS(2053),
    [sym_button_event_keyword] = ACTIONS(2053),
    [sym_channel_event_keyword] = ACTIONS(2053),
    [sym_level_event_keyword] = ACTIONS(2053),
    [sym_data_event_keyword] = ACTIONS(2053),
    [sym_timeline_event_keyword] = ACTIONS(2053),
    [sym_custom_event_keyword] = ACTIONS(2053),
    [sym_push_keyword] = ACTIONS(2053),
    [sym_release_keyword] = ACTIONS(2053),
    [aux_sym_on_keyword_token1] = ACTIONS(2053),
    [aux_sym_off_keyword_token1] = ACTIONS(2053),
    [sym_send_level_keyword] = ACTIONS(2053),
    [sym_send_string_keyword] = ACTIONS(2053),
    [sym_send_command_keyword] = ACTIONS(2053),
    [sym_clear_buffer_keyword] = ACTIONS(2053),
    [sym_create_buffer_keyword] = ACTIONS(2053),
    [sym_create_multi_buffer_keyword] = ACTIONS(2053),
    [sym_call_keyword] = ACTIONS(2053),
    [sym_system_call_keyword] = ACTIONS(2053),
    [sym_devchan_to_keyword] = ACTIONS(2053),
    [sym_devchan_min_to_keyword] = ACTIONS(2053),
    [sym_devchan_total_off_keyword] = ACTIONS(2053),
    [sym_devchan_pulse_keyword] = ACTIONS(2053),
    [sym_wait_keyword] = ACTIONS(2053),
    [sym_cancel_wait_keyword] = ACTIONS(2053),
    [sym_wait_until_keyword] = ACTIONS(2053),
    [sym_cancel_wait_until_keyword] = ACTIONS(2053),
    [sym_cancel_all_wait_keyword] = ACTIONS(2053),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2053),
    [sym_not] = ACTIONS(2053),
    [sym_bnot] = ACTIONS(2053),
    [sym___file__] = ACTIONS(2053),
    [sym___date__] = ACTIONS(2053),
    [sym___ldate__] = ACTIONS(2053),
    [sym___time__] = ACTIONS(2053),
    [sym___line__] = ACTIONS(2053),
    [sym___name__] = ACTIONS(2053),
    [sym___netlinx__] = ACTIONS(2053),
    [sym_netlinx_axi_version] = ACTIONS(2053),
    [sym_url_flg_tcp] = ACTIONS(2053),
    [sym_url_flg_accinfopresent] = ACTIONS(2053),
    [sym_url_flg_temp] = ACTIONS(2053),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2053),
    [sym_url_flg_stat_mask] = ACTIONS(2053),
    [sym_url_flg_stat_lookup] = ACTIONS(2053),
    [sym_url_flg_stat_connecting] = ACTIONS(2053),
    [sym_url_flg_stat_waiting] = ACTIONS(2053),
    [sym_url_flg_stat_connected] = ACTIONS(2053),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2053),
    [sym_first_virtual_device] = ACTIONS(2053),
    [sym_first_local_port] = ACTIONS(2053),
    [sym_dynamic_virtual_device] = ACTIONS(2053),
    [sym_timeline_once] = ACTIONS(2053),
    [sym_timeline_repeat] = ACTIONS(2053),
    [sym_timeline_absolute] = ACTIONS(2053),
    [sym_timeline_relative] = ACTIONS(2053),
    [sym_do_push_timed_infinite] = ACTIONS(2053),
    [sym_source_type_no_address] = ACTIONS(2053),
    [sym_source_type_neuron_id] = ACTIONS(2053),
    [sym_source_type_ip_address] = ACTIONS(2053),
    [sym_source_type_axlink] = ACTIONS(2053),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2053),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2053),
    [sym_source_type_ip_socket_address] = ACTIONS(2053),
    [sym_source_type_rs232] = ACTIONS(2053),
    [sym_source_type_internal] = ACTIONS(2053),
    [sym_source_type_mac_address] = ACTIONS(2053),
    [sym_source_type_ipv4_port] = ACTIONS(2053),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2053),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2053),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2053),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2053),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2053),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2053),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2053),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2053),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2053),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2053),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2053),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2053),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2053),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2053),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2053),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2053),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2053),
    [sym_true] = ACTIONS(2053),
    [sym_false] = ACTIONS(2053),
    [sym_file_read_only] = ACTIONS(2053),
    [sym_file_rw_new] = ACTIONS(2053),
    [sym_file_rw_append] = ACTIONS(2053),
    [sym_ip_tcp] = ACTIONS(2053),
    [sym_ip_udp] = ACTIONS(2053),
    [sym_ip_udp_2way] = ACTIONS(2053),
    [sym_xml_encode_types] = ACTIONS(2053),
    [sym_xml_encode_char_as_list] = ACTIONS(2053),
    [sym_xml_encode_le] = ACTIONS(2053),
    [sym_xml_decode_types] = ACTIONS(2053),
    [sym_xml_decode_no_preserve] = ACTIONS(2053),
    [sym_duet_dev_type_display_device] = ACTIONS(2053),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2053),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2053),
    [sym_duet_dev_type_io_device] = ACTIONS(2053),
    [sym_duet_dev_type_relay_device] = ACTIONS(2053),
    [sym_duet_dev_type_ups] = ACTIONS(2053),
    [sym_duet_dev_type_amplifier] = ACTIONS(2053),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2053),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2053),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2053),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2053),
    [sym_duet_dev_type_camera] = ACTIONS(2053),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2053),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2053),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2053),
    [sym_duet_dev_type_dss] = ACTIONS(2053),
    [sym_duet_dev_type_dvr] = ACTIONS(2053),
    [sym_duet_dev_type_disc_device] = ACTIONS(2053),
    [sym_duet_dev_type_document_camera] = ACTIONS(2053),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2053),
    [sym_duet_dev_type_hvac] = ACTIONS(2053),
    [sym_duet_dev_type_keypad] = ACTIONS(2053),
    [sym_duet_dev_type_light] = ACTIONS(2053),
    [sym_duet_dev_type_monitor] = ACTIONS(2053),
    [sym_duet_dev_type_motor] = ACTIONS(2053),
    [sym_duet_dev_type_multi_window] = ACTIONS(2053),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2053),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2053),
    [sym_duet_dev_type_receiver] = ACTIONS(2053),
    [sym_duet_dev_type_security_system] = ACTIONS(2053),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2053),
    [sym_duet_dev_type_settop_box] = ACTIONS(2053),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2053),
    [sym_duet_dev_type_switcher] = ACTIONS(2053),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2053),
    [sym_duet_dev_type_tv] = ACTIONS(2053),
    [sym_duet_dev_type_utility] = ACTIONS(2053),
    [sym_duet_dev_type_vcr] = ACTIONS(2053),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2053),
    [sym_duet_dev_type_video_processor] = ACTIONS(2053),
    [sym_duet_dev_type_video_projector] = ACTIONS(2053),
    [sym_duet_dev_type_video_wall] = ACTIONS(2053),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2053),
    [sym_duet_dev_type_weather] = ACTIONS(2053),
    [sym_duet_dev_not_polled] = ACTIONS(2053),
    [sym_duet_dev_polled] = ACTIONS(2053),
    [sym_clkmgr_mode_network] = ACTIONS(2053),
    [sym_clkmgr_mode_standalone] = ACTIONS(2053),
    [sym_normal_standby] = ACTIONS(2053),
    [sym_normal_wake] = ACTIONS(2053),
    [sym_ip_multicast_ttl_option] = ACTIONS(2053),
    [sym_ip_ttl_subnet] = ACTIONS(2053),
    [sym_ip_ttl_site] = ACTIONS(2053),
    [sym_ip_ttl_region] = ACTIONS(2053),
    [sym_ip_ttl_continent] = ACTIONS(2053),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2053),
    [sym_ip_nodelay_on] = ACTIONS(2053),
    [sym_ip_nodelay_off] = ACTIONS(2053),
    [sym_amx_error] = ACTIONS(2053),
    [sym_amx_warning] = ACTIONS(2053),
    [sym_amx_info] = ACTIONS(2053),
    [sym_amx_debug] = ACTIONS(2053),
    [sym_netlinx_login_success] = ACTIONS(2053),
    [sym_netlinx_login_fail] = ACTIONS(2053),
    [sym_netlinx_logout] = ACTIONS(2053),
    [sym_null_str] = ACTIONS(2053),
    [sym_smtp_address] = ACTIONS(2053),
    [sym_smtp_port_number] = ACTIONS(2053),
    [sym_smtp_username] = ACTIONS(2053),
    [sym_smtp_password] = ACTIONS(2053),
    [sym_smtp_require_tls] = ACTIONS(2053),
    [sym_smtp_from] = ACTIONS(2053),
    [sym_smtp_tls_true] = ACTIONS(2053),
    [sym_smtp_tls_false] = ACTIONS(2053),
    [sym_listview_on_row_select_event] = ACTIONS(2053),
    [sym_data_structure_unknown] = ACTIONS(2053),
    [sym_data_structure_datafeed] = ACTIONS(2053),
    [sym_data_structure_datarecordset] = ACTIONS(2053),
    [sym_data_structure_datarecord] = ACTIONS(2053),
    [sym_data_structure_datafield] = ACTIONS(2053),
    [sym_data_max_content_field_count] = ACTIONS(2053),
    [sym_data_max_metadata_field_count] = ACTIONS(2053),
    [sym_data_max_id_length] = ACTIONS(2053),
    [sym_data_max_name_length] = ACTIONS(2053),
    [sym_data_max_description_length] = ACTIONS(2053),
    [sym_data_max_type_length] = ACTIONS(2053),
    [sym_data_max_format_length] = ACTIONS(2053),
    [sym_data_max_source_length] = ACTIONS(2053),
    [sym_data_max_value_length] = ACTIONS(2053),
    [sym_data_max_label_length] = ACTIONS(2053),
    [sym_data_max_url_length] = ACTIONS(2053),
    [sym_max_last_login_info_length] = ACTIONS(2053),
    [sym_data_type_unknown] = ACTIONS(2053),
    [sym_data_type_string] = ACTIONS(2053),
    [sym_data_type_datetime] = ACTIONS(2053),
    [sym_data_type_date] = ACTIONS(2053),
    [sym_data_type_time] = ACTIONS(2053),
    [sym_data_type_image] = ACTIONS(2053),
    [sym_data_format_url] = ACTIONS(2053),
    [sym_data_format_phone] = ACTIONS(2053),
    [sym_data_format_email] = ACTIONS(2053),
    [sym_data_format_iso8601] = ACTIONS(2053),
    [sym_auditlog_priv] = ACTIONS(2053),
    [sym_configuration_priv] = ACTIONS(2053),
    [sym_deviceconfig_priv] = ACTIONS(2053),
    [sym_ftp_priv] = ACTIONS(2053),
    [sym_http_priv] = ACTIONS(2053),
    [sym_networkconfig_priv] = ACTIONS(2053),
    [sym_programport_priv] = ACTIONS(2053),
    [sym_remoteui_priv] = ACTIONS(2053),
    [sym_securitycontrol_priv] = ACTIONS(2053),
    [sym_softwaremanagement_priv] = ACTIONS(2053),
    [sym_terminal_priv] = ACTIONS(2053),
    [sym_tpadmin_priv] = ACTIONS(2053),
    [sym_user1_priv] = ACTIONS(2053),
    [sym_user2_priv] = ACTIONS(2053),
    [sym_user3_priv] = ACTIONS(2053),
    [sym_user4_priv] = ACTIONS(2053),
    [sym_usermanagement_priv] = ACTIONS(2053),
    [sym_valid_account] = ACTIONS(2053),
    [sym_err_name_invalid] = ACTIONS(2053),
    [sym_err_password_invalid] = ACTIONS(2053),
    [sym_err_invalid_account] = ACTIONS(2053),
    [sym_err_no_authorization] = ACTIONS(2053),
    [sym_err_invalid_parameter] = ACTIONS(2053),
    [sym_err_user_locked_out] = ACTIONS(2053),
    [sym_err_user_account_expired] = ACTIONS(2053),
    [sym_err_general_error] = ACTIONS(2053),
    [sym_err_authentication_error] = ACTIONS(2053),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2053),
    [sym_tls_validate_certificate] = ACTIONS(2053),
    [sym_tdata] = ACTIONS(2053),
    [sym_tchannel] = ACTIONS(2053),
    [sym_tlevel] = ACTIONS(2053),
    [sym_tbutton] = ACTIONS(2053),
    [sym_ttimeline] = ACTIONS(2053),
    [sym_tcustom] = ACTIONS(2053),
    [sym_url_struct] = ACTIONS(2053),
    [sym_dns_struct] = ACTIONS(2053),
    [sym_ip_address_struct] = ACTIONS(2053),
    [sym_dev_info_struct] = ACTIONS(2053),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2053),
    [sym_data_feed] = ACTIONS(2053),
    [sym_data_field] = ACTIONS(2053),
    [sym_data_record] = ACTIONS(2053),
    [sym_wc_data_feed] = ACTIONS(2053),
    [sym_wc_data_field] = ACTIONS(2053),
    [sym_wc_data_record] = ACTIONS(2053),
    [sym_last_login_info] = ACTIONS(2053),
    [sym____reserved___] = ACTIONS(2053),
    [sym_date] = ACTIONS(2053),
    [sym_day] = ACTIONS(2053),
    [sym_time] = ACTIONS(2053),
    [sym_ldate] = ACTIONS(2053),
    [sym_push_device] = ACTIONS(2053),
    [sym_push_channel] = ACTIONS(2053),
    [sym_push_devchan] = ACTIONS(2053),
    [sym_release_device] = ACTIONS(2053),
    [sym_release_channel] = ACTIONS(2053),
    [sym_release_devchan] = ACTIONS(2053),
    [sym_master_slot] = ACTIONS(2053),
    [sym_get_pulse_time] = ACTIONS(2053),
    [sym_get_timer] = ACTIONS(2053),
    [sym_master_sn] = ACTIONS(2053),
    [sym_system_number] = ACTIONS(2053),
    [sym_button] = ACTIONS(2053),
    [sym_dv_channel] = ACTIONS(2053),
    [sym_data] = ACTIONS(2053),
    [sym_level] = ACTIONS(2053),
    [sym_timeline] = ACTIONS(2053),
    [sym_channel] = ACTIONS(2053),
    [sym_custom] = ACTIONS(2053),
    [sym_atoi] = ACTIONS(2053),
    [sym_atol] = ACTIONS(2053),
    [sym_atof] = ACTIONS(2053),
    [sym_itoa] = ACTIONS(2053),
    [sym_itohex] = ACTIONS(2053),
    [sym_hextoi] = ACTIONS(2053),
    [sym_ftoa] = ACTIONS(2053),
    [sym_find_string] = ACTIONS(2053),
    [sym_left_string] = ACTIONS(2053),
    [sym_right_string] = ACTIONS(2053),
    [sym_mid_string] = ACTIONS(2053),
    [sym_remove_string] = ACTIONS(2053),
    [sym_get_buffer_string] = ACTIONS(2053),
    [sym_get_buffer_char] = ACTIONS(2053),
    [sym_get_multi_buffer_string] = ACTIONS(2053),
    [sym_length_string] = ACTIONS(2053),
    [sym_max_length_string] = ACTIONS(2053),
    [sym_set_length_string] = ACTIONS(2053),
    [sym_compare_string] = ACTIONS(2053),
    [sym_upper_string] = ACTIONS(2053),
    [sym_lower_string] = ACTIONS(2053),
    [sym_get_last] = ACTIONS(2053),
    [sym_date_to_day] = ACTIONS(2053),
    [sym_date_to_month] = ACTIONS(2053),
    [sym_date_to_year] = ACTIONS(2053),
    [sym_time_to_hour] = ACTIONS(2053),
    [sym_time_to_minute] = ACTIONS(2053),
    [sym_time_to_second] = ACTIONS(2053),
    [sym_day_of_week] = ACTIONS(2053),
    [sym_set_pulse_time] = ACTIONS(2053),
    [sym_set_timer] = ACTIONS(2053),
    [sym_file_open] = ACTIONS(2053),
    [sym_file_close] = ACTIONS(2053),
    [sym_file_read] = ACTIONS(2053),
    [sym_file_read_line] = ACTIONS(2053),
    [sym_file_write] = ACTIONS(2053),
    [sym_file_write_line] = ACTIONS(2053),
    [sym_file_copy] = ACTIONS(2053),
    [sym_file_delete] = ACTIONS(2053),
    [sym_file_rename] = ACTIONS(2053),
    [sym_file_dir] = ACTIONS(2053),
    [sym_file_getdir] = ACTIONS(2053),
    [sym_file_setdir] = ACTIONS(2053),
    [sym_file_create_dir] = ACTIONS(2053),
    [sym_file_remove_dir] = ACTIONS(2053),
    [sym_file_seek] = ACTIONS(2053),
    [sym_length_array] = ACTIONS(2053),
    [sym_max_length_array] = ACTIONS(2053),
    [sym_set_length_array] = ACTIONS(2053),
    [sym_ip_client_open] = ACTIONS(2053),
    [sym_ip_client_close] = ACTIONS(2053),
    [sym_ip_server_open] = ACTIONS(2053),
    [sym_ip_server_close] = ACTIONS(2053),
    [sym_ip_bound_client_open] = ACTIONS(2053),
    [sym_ip_mc_server_open] = ACTIONS(2053),
    [sym_ip_set_option] = ACTIONS(2053),
    [sym_ssh_client_open] = ACTIONS(2053),
    [sym_ssh_client_close] = ACTIONS(2053),
    [sym_tls_client_open] = ACTIONS(2053),
    [sym_tls_client_close] = ACTIONS(2053),
    [sym_do_push] = ACTIONS(2053),
    [sym_do_push_timed] = ACTIONS(2053),
    [sym_do_release] = ACTIONS(2053),
    [sym_device_id] = ACTIONS(2053),
    [sym_device_info] = ACTIONS(2053),
    [sym_device_id_string] = ACTIONS(2053),
    [sym_device_standby] = ACTIONS(2053),
    [sym_device_wake] = ACTIONS(2053),
    [sym_timeline_create] = ACTIONS(2053),
    [sym_timeline_kill] = ACTIONS(2053),
    [sym_timeline_reload] = ACTIONS(2053),
    [sym_timeline_pause] = ACTIONS(2053),
    [sym_timeline_restart] = ACTIONS(2053),
    [sym_timeline_set] = ACTIONS(2053),
    [sym_timeline_get] = ACTIONS(2053),
    [sym_timeline_active] = ACTIONS(2053),
    [sym_timeline_dynamic_id] = ACTIONS(2053),
    [sym_variable_to_string] = ACTIONS(2053),
    [sym_string_to_variable] = ACTIONS(2053),
    [sym_length_variable_to_string] = ACTIONS(2053),
    [sym_type_cast] = ACTIONS(2053),
    [sym_variable_to_xml] = ACTIONS(2053),
    [sym_xml_to_variable] = ACTIONS(2053),
    [sym_length_variable_to_xml] = ACTIONS(2053),
    [sym_redirect_string] = ACTIONS(2053),
    [sym_random_number] = ACTIONS(2053),
    [sym_get_unique_id] = ACTIONS(2053),
    [sym_get_system_number] = ACTIONS(2053),
    [sym_set_system_number] = ACTIONS(2053),
    [sym_get_serial_number] = ACTIONS(2053),
    [sym_get_master_build] = ACTIONS(2053),
    [sym_set_validation_code] = ACTIONS(2053),
    [sym_reboot] = ACTIONS(2053),
    [sym_load_duet_module] = ACTIONS(2053),
    [sym_unload_duet_module] = ACTIONS(2053),
    [sym_duet_mem_size_set] = ACTIONS(2053),
    [sym_duet_mem_size_get] = ACTIONS(2053),
    [sym_dynamic_application_device] = ACTIONS(2053),
    [sym_dynamic_polled_port] = ACTIONS(2053),
    [sym_static_port_binding] = ACTIONS(2053),
    [sym_static_ip_binding] = ACTIONS(2053),
    [sym_get_dns_list] = ACTIONS(2053),
    [sym_set_dns_list] = ACTIONS(2053),
    [sym_get_ip_address] = ACTIONS(2053),
    [sym_set_ip_address] = ACTIONS(2053),
    [sym_get_url_list] = ACTIONS(2053),
    [sym_add_url_entry] = ACTIONS(2053),
    [sym_delete_url_entry] = ACTIONS(2053),
    [sym_abs_value] = ACTIONS(2053),
    [sym_max_value] = ACTIONS(2053),
    [sym_min_value] = ACTIONS(2053),
    [sym_exp_value] = ACTIONS(2053),
    [sym_log_value] = ACTIONS(2053),
    [sym_log10_value] = ACTIONS(2053),
    [sym_power_value] = ACTIONS(2053),
    [sym_sqrt_value] = ACTIONS(2053),
    [sym_raw_be] = ACTIONS(2053),
    [sym_raw_le] = ACTIONS(2053),
    [sym_internal_queue_size_set] = ACTIONS(2053),
    [sym_internal_queue_size_get] = ACTIONS(2053),
    [sym_internal_threshold_set] = ACTIONS(2053),
    [sym_internal_threshold_get] = ACTIONS(2053),
    [sym_rebuilt_event] = ACTIONS(2053),
    [sym_do_custom_event] = ACTIONS(2053),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2053),
    [sym_clkmgr_set_clk_source] = ACTIONS(2053),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2053),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2053),
    [sym_clkmgr_get_timezone] = ACTIONS(2053),
    [sym_clkmgr_set_timezone] = ACTIONS(2053),
    [sym_clkmgr_get_resync_period] = ACTIONS(2053),
    [sym_clkmgr_set_resync_period] = ACTIONS(2053),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2053),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2053),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2053),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2053),
    [sym_clkmgr_get_timeservers] = ACTIONS(2053),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2053),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2053),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2053),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2053),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2053),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2053),
    [sym_set_log_level] = ACTIONS(2053),
    [sym_get_log_level] = ACTIONS(2053),
    [sym_amx_log] = ACTIONS(2053),
    [sym_smtp_server_config_set] = ACTIONS(2053),
    [sym_smtp_server_config_get] = ACTIONS(2053),
    [sym_smtp_send] = ACTIONS(2053),
    [sym_validate_netlinx_account] = ACTIONS(2053),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2053),
    [sym_authenticate_certificate] = ACTIONS(2053),
    [sym_audit_netlinx_session_event] = ACTIONS(2053),
    [sym_audit_netlinx_generic_event] = ACTIONS(2053),
    [sym_get_available_flash_disk_space] = ACTIONS(2053),
    [sym_get_max_flash_disk_space] = ACTIONS(2053),
    [sym_data_create_feed] = ACTIONS(2053),
    [sym_data_delete_feed] = ACTIONS(2053),
    [sym_data_publish_feed] = ACTIONS(2053),
    [sym_data_get_published_feed] = ACTIONS(2053),
    [sym_data_add_record] = ACTIONS(2053),
    [sym_data_get_event_record] = ACTIONS(2053),
    [sym__wc_data_create_feed] = ACTIONS(2053),
    [sym__wc_data_add_record] = ACTIONS(2053),
    [sym__wc_data_get_event_record] = ACTIONS(2053),
    [sym_format] = ACTIONS(2053),
    [sym_astro_clock] = ACTIONS(2053),
    [sym_set_outdoor_temperature] = ACTIONS(2053),
    [sym_set_virtual_level_count] = ACTIONS(2053),
    [sym_set_virtual_channel_count] = ACTIONS(2053),
    [sym_set_virtual_port_count] = ACTIONS(2053),
    [sym__wc_encode_int] = ACTIONS(2053),
    [sym__wc_decode_int] = ACTIONS(2053),
    [sym__wc_int] = ACTIONS(2053),
    [sym__wc_to_ch_int] = ACTIONS(2053),
    [sym__ch_to_wc_int] = ACTIONS(2053),
    [sym__wc_find_string_int] = ACTIONS(2053),
    [sym__wc_left_string_int] = ACTIONS(2053),
    [sym__wc_lower_string_int] = ACTIONS(2053),
    [sym__wc_mid_string_int] = ACTIONS(2053),
    [sym__wc_remove_string_int] = ACTIONS(2053),
    [sym__wc_right_string_int] = ACTIONS(2053),
    [sym__wc_upper_string_int] = ACTIONS(2053),
    [sym__wc_compare_string_int] = ACTIONS(2053),
    [sym__wc_get_buffer_char_int] = ACTIONS(2053),
    [sym__wc_get_buffer_string_int] = ACTIONS(2053),
    [sym__wc_concat_string_int] = ACTIONS(2053),
    [sym__wc_file_open_int] = ACTIONS(2053),
    [sym__wc_file_close_int] = ACTIONS(2053),
    [sym__wc_file_read_int] = ACTIONS(2053),
    [sym__wc_file_read_line_int] = ACTIONS(2053),
    [sym__wc_file_write_int] = ACTIONS(2053),
    [sym__wc_file_write_line_int] = ACTIONS(2053),
    [aux_sym_preproc_include_token1] = ACTIONS(2053),
    [aux_sym_preproc_define_token1] = ACTIONS(2055),
    [aux_sym_preproc_warn_token1] = ACTIONS(2055),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2055),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2055),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2055),
    [anon_sym_LBRACK] = ACTIONS(2055),
    [anon_sym_LBRACE] = ACTIONS(2055),
    [anon_sym_LPAREN] = ACTIONS(2053),
    [anon_sym_BANG] = ACTIONS(2055),
    [anon_sym_TILDE] = ACTIONS(2055),
    [anon_sym_DASH] = ACTIONS(2053),
    [anon_sym_PLUS] = ACTIONS(2053),
    [anon_sym_DASH_DASH] = ACTIONS(2055),
    [anon_sym_PLUS_PLUS] = ACTIONS(2055),
    [anon_sym_DQUOTE] = ACTIONS(2055),
    [anon_sym_SQUOTE] = ACTIONS(2055),
    [sym_number_literal] = ACTIONS(2055),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(426)] = {
    [ts_builtin_sym_end] = ACTIONS(2003),
    [sym_identifier] = ACTIONS(2001),
    [sym_program_name_keyword] = ACTIONS(2001),
    [sym_module_name_keyword] = ACTIONS(2001),
    [sym_define_device_keyword] = ACTIONS(2001),
    [sym_define_combine_keyword] = ACTIONS(2001),
    [sym_define_constant_keyword] = ACTIONS(2001),
    [sym_define_type_keyword] = ACTIONS(2001),
    [sym_define_variable_keyword] = ACTIONS(2001),
    [sym_define_system_variable_keyword] = ACTIONS(2001),
    [sym_define_connect_level_keyword] = ACTIONS(2001),
    [sym_define_latching_keyword] = ACTIONS(2001),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2001),
    [sym_define_toggling_keyword] = ACTIONS(2001),
    [sym_define_call_keyword] = ACTIONS(2001),
    [sym_define_function_keyword] = ACTIONS(2001),
    [sym_define_library_function_keyword] = ACTIONS(2001),
    [sym_define_start_keyword] = ACTIONS(2001),
    [sym_define_event_keyword] = ACTIONS(2001),
    [sym_define_program_keyword] = ACTIONS(2001),
    [sym_define_module_keyword] = ACTIONS(2001),
    [sym_char_keyword] = ACTIONS(2001),
    [sym_widechar_keyword] = ACTIONS(2001),
    [sym_integer_keyword] = ACTIONS(2001),
    [sym_sinteger_keyword] = ACTIONS(2001),
    [sym_long_keyword] = ACTIONS(2001),
    [sym_slong_keyword] = ACTIONS(2001),
    [sym_float_keyword] = ACTIONS(2001),
    [sym_double_keyword] = ACTIONS(2001),
    [sym_variant_keyword] = ACTIONS(2001),
    [sym_variantarray_keyword] = ACTIONS(2001),
    [sym_local_var_keyword] = ACTIONS(2001),
    [sym_stack_var_keyword] = ACTIONS(2001),
    [sym_constant_keyword] = ACTIONS(2001),
    [sym_volatile_keyword] = ACTIONS(2001),
    [sym_non_volatile_keyword] = ACTIONS(2001),
    [sym_persistent_keyword] = ACTIONS(2001),
    [sym_dev_keyword] = ACTIONS(2001),
    [sym_devlev_keyword] = ACTIONS(2001),
    [sym_devchan_keyword] = ACTIONS(2001),
    [sym_if_keyword] = ACTIONS(2001),
    [sym_switch_keyword] = ACTIONS(2001),
    [sym_case_keyword] = ACTIONS(2001),
    [sym_default_keyword] = ACTIONS(2001),
    [sym_while_keyword] = ACTIONS(2001),
    [sym_for_keyword] = ACTIONS(2001),
    [sym_break_keyword] = ACTIONS(2001),
    [sym_continue_keyword] = ACTIONS(2001),
    [sym_return_keyword] = ACTIONS(2001),
    [sym_select_keyword] = ACTIONS(2001),
    [sym_active_keyword] = ACTIONS(2001),
    [sym_struct_keyword] = ACTIONS(2001),
    [sym_structure_keyword] = ACTIONS(2001),
    [sym_button_event_keyword] = ACTIONS(2001),
    [sym_channel_event_keyword] = ACTIONS(2001),
    [sym_level_event_keyword] = ACTIONS(2001),
    [sym_data_event_keyword] = ACTIONS(2001),
    [sym_timeline_event_keyword] = ACTIONS(2001),
    [sym_custom_event_keyword] = ACTIONS(2001),
    [sym_push_keyword] = ACTIONS(2001),
    [sym_release_keyword] = ACTIONS(2001),
    [aux_sym_on_keyword_token1] = ACTIONS(2001),
    [aux_sym_off_keyword_token1] = ACTIONS(2001),
    [sym_send_level_keyword] = ACTIONS(2001),
    [sym_send_string_keyword] = ACTIONS(2001),
    [sym_send_command_keyword] = ACTIONS(2001),
    [sym_clear_buffer_keyword] = ACTIONS(2001),
    [sym_create_buffer_keyword] = ACTIONS(2001),
    [sym_create_multi_buffer_keyword] = ACTIONS(2001),
    [sym_call_keyword] = ACTIONS(2001),
    [sym_system_call_keyword] = ACTIONS(2001),
    [sym_devchan_to_keyword] = ACTIONS(2001),
    [sym_devchan_min_to_keyword] = ACTIONS(2001),
    [sym_devchan_total_off_keyword] = ACTIONS(2001),
    [sym_devchan_pulse_keyword] = ACTIONS(2001),
    [sym_wait_keyword] = ACTIONS(2001),
    [sym_cancel_wait_keyword] = ACTIONS(2001),
    [sym_wait_until_keyword] = ACTIONS(2001),
    [sym_cancel_wait_until_keyword] = ACTIONS(2001),
    [sym_cancel_all_wait_keyword] = ACTIONS(2001),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2001),
    [sym_not] = ACTIONS(2001),
    [sym_bnot] = ACTIONS(2001),
    [sym___file__] = ACTIONS(2001),
    [sym___date__] = ACTIONS(2001),
    [sym___ldate__] = ACTIONS(2001),
    [sym___time__] = ACTIONS(2001),
    [sym___line__] = ACTIONS(2001),
    [sym___name__] = ACTIONS(2001),
    [sym___netlinx__] = ACTIONS(2001),
    [sym_netlinx_axi_version] = ACTIONS(2001),
    [sym_url_flg_tcp] = ACTIONS(2001),
    [sym_url_flg_accinfopresent] = ACTIONS(2001),
    [sym_url_flg_temp] = ACTIONS(2001),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2001),
    [sym_url_flg_stat_mask] = ACTIONS(2001),
    [sym_url_flg_stat_lookup] = ACTIONS(2001),
    [sym_url_flg_stat_connecting] = ACTIONS(2001),
    [sym_url_flg_stat_waiting] = ACTIONS(2001),
    [sym_url_flg_stat_connected] = ACTIONS(2001),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2001),
    [sym_first_virtual_device] = ACTIONS(2001),
    [sym_first_local_port] = ACTIONS(2001),
    [sym_dynamic_virtual_device] = ACTIONS(2001),
    [sym_timeline_once] = ACTIONS(2001),
    [sym_timeline_repeat] = ACTIONS(2001),
    [sym_timeline_absolute] = ACTIONS(2001),
    [sym_timeline_relative] = ACTIONS(2001),
    [sym_do_push_timed_infinite] = ACTIONS(2001),
    [sym_source_type_no_address] = ACTIONS(2001),
    [sym_source_type_neuron_id] = ACTIONS(2001),
    [sym_source_type_ip_address] = ACTIONS(2001),
    [sym_source_type_axlink] = ACTIONS(2001),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2001),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2001),
    [sym_source_type_ip_socket_address] = ACTIONS(2001),
    [sym_source_type_rs232] = ACTIONS(2001),
    [sym_source_type_internal] = ACTIONS(2001),
    [sym_source_type_mac_address] = ACTIONS(2001),
    [sym_source_type_ipv4_port] = ACTIONS(2001),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2001),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2001),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2001),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2001),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2001),
    [sym_true] = ACTIONS(2001),
    [sym_false] = ACTIONS(2001),
    [sym_file_read_only] = ACTIONS(2001),
    [sym_file_rw_new] = ACTIONS(2001),
    [sym_file_rw_append] = ACTIONS(2001),
    [sym_ip_tcp] = ACTIONS(2001),
    [sym_ip_udp] = ACTIONS(2001),
    [sym_ip_udp_2way] = ACTIONS(2001),
    [sym_xml_encode_types] = ACTIONS(2001),
    [sym_xml_encode_char_as_list] = ACTIONS(2001),
    [sym_xml_encode_le] = ACTIONS(2001),
    [sym_xml_decode_types] = ACTIONS(2001),
    [sym_xml_decode_no_preserve] = ACTIONS(2001),
    [sym_duet_dev_type_display_device] = ACTIONS(2001),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2001),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2001),
    [sym_duet_dev_type_io_device] = ACTIONS(2001),
    [sym_duet_dev_type_relay_device] = ACTIONS(2001),
    [sym_duet_dev_type_ups] = ACTIONS(2001),
    [sym_duet_dev_type_amplifier] = ACTIONS(2001),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2001),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2001),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2001),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2001),
    [sym_duet_dev_type_camera] = ACTIONS(2001),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2001),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2001),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2001),
    [sym_duet_dev_type_dss] = ACTIONS(2001),
    [sym_duet_dev_type_dvr] = ACTIONS(2001),
    [sym_duet_dev_type_disc_device] = ACTIONS(2001),
    [sym_duet_dev_type_document_camera] = ACTIONS(2001),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2001),
    [sym_duet_dev_type_hvac] = ACTIONS(2001),
    [sym_duet_dev_type_keypad] = ACTIONS(2001),
    [sym_duet_dev_type_light] = ACTIONS(2001),
    [sym_duet_dev_type_monitor] = ACTIONS(2001),
    [sym_duet_dev_type_motor] = ACTIONS(2001),
    [sym_duet_dev_type_multi_window] = ACTIONS(2001),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2001),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2001),
    [sym_duet_dev_type_receiver] = ACTIONS(2001),
    [sym_duet_dev_type_security_system] = ACTIONS(2001),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2001),
    [sym_duet_dev_type_settop_box] = ACTIONS(2001),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2001),
    [sym_duet_dev_type_switcher] = ACTIONS(2001),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2001),
    [sym_duet_dev_type_tv] = ACTIONS(2001),
    [sym_duet_dev_type_utility] = ACTIONS(2001),
    [sym_duet_dev_type_vcr] = ACTIONS(2001),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2001),
    [sym_duet_dev_type_video_processor] = ACTIONS(2001),
    [sym_duet_dev_type_video_projector] = ACTIONS(2001),
    [sym_duet_dev_type_video_wall] = ACTIONS(2001),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2001),
    [sym_duet_dev_type_weather] = ACTIONS(2001),
    [sym_duet_dev_not_polled] = ACTIONS(2001),
    [sym_duet_dev_polled] = ACTIONS(2001),
    [sym_clkmgr_mode_network] = ACTIONS(2001),
    [sym_clkmgr_mode_standalone] = ACTIONS(2001),
    [sym_normal_standby] = ACTIONS(2001),
    [sym_normal_wake] = ACTIONS(2001),
    [sym_ip_multicast_ttl_option] = ACTIONS(2001),
    [sym_ip_ttl_subnet] = ACTIONS(2001),
    [sym_ip_ttl_site] = ACTIONS(2001),
    [sym_ip_ttl_region] = ACTIONS(2001),
    [sym_ip_ttl_continent] = ACTIONS(2001),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2001),
    [sym_ip_nodelay_on] = ACTIONS(2001),
    [sym_ip_nodelay_off] = ACTIONS(2001),
    [sym_amx_error] = ACTIONS(2001),
    [sym_amx_warning] = ACTIONS(2001),
    [sym_amx_info] = ACTIONS(2001),
    [sym_amx_debug] = ACTIONS(2001),
    [sym_netlinx_login_success] = ACTIONS(2001),
    [sym_netlinx_login_fail] = ACTIONS(2001),
    [sym_netlinx_logout] = ACTIONS(2001),
    [sym_null_str] = ACTIONS(2001),
    [sym_smtp_address] = ACTIONS(2001),
    [sym_smtp_port_number] = ACTIONS(2001),
    [sym_smtp_username] = ACTIONS(2001),
    [sym_smtp_password] = ACTIONS(2001),
    [sym_smtp_require_tls] = ACTIONS(2001),
    [sym_smtp_from] = ACTIONS(2001),
    [sym_smtp_tls_true] = ACTIONS(2001),
    [sym_smtp_tls_false] = ACTIONS(2001),
    [sym_listview_on_row_select_event] = ACTIONS(2001),
    [sym_data_structure_unknown] = ACTIONS(2001),
    [sym_data_structure_datafeed] = ACTIONS(2001),
    [sym_data_structure_datarecordset] = ACTIONS(2001),
    [sym_data_structure_datarecord] = ACTIONS(2001),
    [sym_data_structure_datafield] = ACTIONS(2001),
    [sym_data_max_content_field_count] = ACTIONS(2001),
    [sym_data_max_metadata_field_count] = ACTIONS(2001),
    [sym_data_max_id_length] = ACTIONS(2001),
    [sym_data_max_name_length] = ACTIONS(2001),
    [sym_data_max_description_length] = ACTIONS(2001),
    [sym_data_max_type_length] = ACTIONS(2001),
    [sym_data_max_format_length] = ACTIONS(2001),
    [sym_data_max_source_length] = ACTIONS(2001),
    [sym_data_max_value_length] = ACTIONS(2001),
    [sym_data_max_label_length] = ACTIONS(2001),
    [sym_data_max_url_length] = ACTIONS(2001),
    [sym_max_last_login_info_length] = ACTIONS(2001),
    [sym_data_type_unknown] = ACTIONS(2001),
    [sym_data_type_string] = ACTIONS(2001),
    [sym_data_type_datetime] = ACTIONS(2001),
    [sym_data_type_date] = ACTIONS(2001),
    [sym_data_type_time] = ACTIONS(2001),
    [sym_data_type_image] = ACTIONS(2001),
    [sym_data_format_url] = ACTIONS(2001),
    [sym_data_format_phone] = ACTIONS(2001),
    [sym_data_format_email] = ACTIONS(2001),
    [sym_data_format_iso8601] = ACTIONS(2001),
    [sym_auditlog_priv] = ACTIONS(2001),
    [sym_configuration_priv] = ACTIONS(2001),
    [sym_deviceconfig_priv] = ACTIONS(2001),
    [sym_ftp_priv] = ACTIONS(2001),
    [sym_http_priv] = ACTIONS(2001),
    [sym_networkconfig_priv] = ACTIONS(2001),
    [sym_programport_priv] = ACTIONS(2001),
    [sym_remoteui_priv] = ACTIONS(2001),
    [sym_securitycontrol_priv] = ACTIONS(2001),
    [sym_softwaremanagement_priv] = ACTIONS(2001),
    [sym_terminal_priv] = ACTIONS(2001),
    [sym_tpadmin_priv] = ACTIONS(2001),
    [sym_user1_priv] = ACTIONS(2001),
    [sym_user2_priv] = ACTIONS(2001),
    [sym_user3_priv] = ACTIONS(2001),
    [sym_user4_priv] = ACTIONS(2001),
    [sym_usermanagement_priv] = ACTIONS(2001),
    [sym_valid_account] = ACTIONS(2001),
    [sym_err_name_invalid] = ACTIONS(2001),
    [sym_err_password_invalid] = ACTIONS(2001),
    [sym_err_invalid_account] = ACTIONS(2001),
    [sym_err_no_authorization] = ACTIONS(2001),
    [sym_err_invalid_parameter] = ACTIONS(2001),
    [sym_err_user_locked_out] = ACTIONS(2001),
    [sym_err_user_account_expired] = ACTIONS(2001),
    [sym_err_general_error] = ACTIONS(2001),
    [sym_err_authentication_error] = ACTIONS(2001),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2001),
    [sym_tls_validate_certificate] = ACTIONS(2001),
    [sym_tdata] = ACTIONS(2001),
    [sym_tchannel] = ACTIONS(2001),
    [sym_tlevel] = ACTIONS(2001),
    [sym_tbutton] = ACTIONS(2001),
    [sym_ttimeline] = ACTIONS(2001),
    [sym_tcustom] = ACTIONS(2001),
    [sym_url_struct] = ACTIONS(2001),
    [sym_dns_struct] = ACTIONS(2001),
    [sym_ip_address_struct] = ACTIONS(2001),
    [sym_dev_info_struct] = ACTIONS(2001),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2001),
    [sym_data_feed] = ACTIONS(2001),
    [sym_data_field] = ACTIONS(2001),
    [sym_data_record] = ACTIONS(2001),
    [sym_wc_data_feed] = ACTIONS(2001),
    [sym_wc_data_field] = ACTIONS(2001),
    [sym_wc_data_record] = ACTIONS(2001),
    [sym_last_login_info] = ACTIONS(2001),
    [sym____reserved___] = ACTIONS(2001),
    [sym_date] = ACTIONS(2001),
    [sym_day] = ACTIONS(2001),
    [sym_time] = ACTIONS(2001),
    [sym_ldate] = ACTIONS(2001),
    [sym_push_device] = ACTIONS(2001),
    [sym_push_channel] = ACTIONS(2001),
    [sym_push_devchan] = ACTIONS(2001),
    [sym_release_device] = ACTIONS(2001),
    [sym_release_channel] = ACTIONS(2001),
    [sym_release_devchan] = ACTIONS(2001),
    [sym_master_slot] = ACTIONS(2001),
    [sym_get_pulse_time] = ACTIONS(2001),
    [sym_get_timer] = ACTIONS(2001),
    [sym_master_sn] = ACTIONS(2001),
    [sym_system_number] = ACTIONS(2001),
    [sym_button] = ACTIONS(2001),
    [sym_dv_channel] = ACTIONS(2001),
    [sym_data] = ACTIONS(2001),
    [sym_level] = ACTIONS(2001),
    [sym_timeline] = ACTIONS(2001),
    [sym_channel] = ACTIONS(2001),
    [sym_custom] = ACTIONS(2001),
    [sym_atoi] = ACTIONS(2001),
    [sym_atol] = ACTIONS(2001),
    [sym_atof] = ACTIONS(2001),
    [sym_itoa] = ACTIONS(2001),
    [sym_itohex] = ACTIONS(2001),
    [sym_hextoi] = ACTIONS(2001),
    [sym_ftoa] = ACTIONS(2001),
    [sym_find_string] = ACTIONS(2001),
    [sym_left_string] = ACTIONS(2001),
    [sym_right_string] = ACTIONS(2001),
    [sym_mid_string] = ACTIONS(2001),
    [sym_remove_string] = ACTIONS(2001),
    [sym_get_buffer_string] = ACTIONS(2001),
    [sym_get_buffer_char] = ACTIONS(2001),
    [sym_get_multi_buffer_string] = ACTIONS(2001),
    [sym_length_string] = ACTIONS(2001),
    [sym_max_length_string] = ACTIONS(2001),
    [sym_set_length_string] = ACTIONS(2001),
    [sym_compare_string] = ACTIONS(2001),
    [sym_upper_string] = ACTIONS(2001),
    [sym_lower_string] = ACTIONS(2001),
    [sym_get_last] = ACTIONS(2001),
    [sym_date_to_day] = ACTIONS(2001),
    [sym_date_to_month] = ACTIONS(2001),
    [sym_date_to_year] = ACTIONS(2001),
    [sym_time_to_hour] = ACTIONS(2001),
    [sym_time_to_minute] = ACTIONS(2001),
    [sym_time_to_second] = ACTIONS(2001),
    [sym_day_of_week] = ACTIONS(2001),
    [sym_set_pulse_time] = ACTIONS(2001),
    [sym_set_timer] = ACTIONS(2001),
    [sym_file_open] = ACTIONS(2001),
    [sym_file_close] = ACTIONS(2001),
    [sym_file_read] = ACTIONS(2001),
    [sym_file_read_line] = ACTIONS(2001),
    [sym_file_write] = ACTIONS(2001),
    [sym_file_write_line] = ACTIONS(2001),
    [sym_file_copy] = ACTIONS(2001),
    [sym_file_delete] = ACTIONS(2001),
    [sym_file_rename] = ACTIONS(2001),
    [sym_file_dir] = ACTIONS(2001),
    [sym_file_getdir] = ACTIONS(2001),
    [sym_file_setdir] = ACTIONS(2001),
    [sym_file_create_dir] = ACTIONS(2001),
    [sym_file_remove_dir] = ACTIONS(2001),
    [sym_file_seek] = ACTIONS(2001),
    [sym_length_array] = ACTIONS(2001),
    [sym_max_length_array] = ACTIONS(2001),
    [sym_set_length_array] = ACTIONS(2001),
    [sym_ip_client_open] = ACTIONS(2001),
    [sym_ip_client_close] = ACTIONS(2001),
    [sym_ip_server_open] = ACTIONS(2001),
    [sym_ip_server_close] = ACTIONS(2001),
    [sym_ip_bound_client_open] = ACTIONS(2001),
    [sym_ip_mc_server_open] = ACTIONS(2001),
    [sym_ip_set_option] = ACTIONS(2001),
    [sym_ssh_client_open] = ACTIONS(2001),
    [sym_ssh_client_close] = ACTIONS(2001),
    [sym_tls_client_open] = ACTIONS(2001),
    [sym_tls_client_close] = ACTIONS(2001),
    [sym_do_push] = ACTIONS(2001),
    [sym_do_push_timed] = ACTIONS(2001),
    [sym_do_release] = ACTIONS(2001),
    [sym_device_id] = ACTIONS(2001),
    [sym_device_info] = ACTIONS(2001),
    [sym_device_id_string] = ACTIONS(2001),
    [sym_device_standby] = ACTIONS(2001),
    [sym_device_wake] = ACTIONS(2001),
    [sym_timeline_create] = ACTIONS(2001),
    [sym_timeline_kill] = ACTIONS(2001),
    [sym_timeline_reload] = ACTIONS(2001),
    [sym_timeline_pause] = ACTIONS(2001),
    [sym_timeline_restart] = ACTIONS(2001),
    [sym_timeline_set] = ACTIONS(2001),
    [sym_timeline_get] = ACTIONS(2001),
    [sym_timeline_active] = ACTIONS(2001),
    [sym_timeline_dynamic_id] = ACTIONS(2001),
    [sym_variable_to_string] = ACTIONS(2001),
    [sym_string_to_variable] = ACTIONS(2001),
    [sym_length_variable_to_string] = ACTIONS(2001),
    [sym_type_cast] = ACTIONS(2001),
    [sym_variable_to_xml] = ACTIONS(2001),
    [sym_xml_to_variable] = ACTIONS(2001),
    [sym_length_variable_to_xml] = ACTIONS(2001),
    [sym_redirect_string] = ACTIONS(2001),
    [sym_random_number] = ACTIONS(2001),
    [sym_get_unique_id] = ACTIONS(2001),
    [sym_get_system_number] = ACTIONS(2001),
    [sym_set_system_number] = ACTIONS(2001),
    [sym_get_serial_number] = ACTIONS(2001),
    [sym_get_master_build] = ACTIONS(2001),
    [sym_set_validation_code] = ACTIONS(2001),
    [sym_reboot] = ACTIONS(2001),
    [sym_load_duet_module] = ACTIONS(2001),
    [sym_unload_duet_module] = ACTIONS(2001),
    [sym_duet_mem_size_set] = ACTIONS(2001),
    [sym_duet_mem_size_get] = ACTIONS(2001),
    [sym_dynamic_application_device] = ACTIONS(2001),
    [sym_dynamic_polled_port] = ACTIONS(2001),
    [sym_static_port_binding] = ACTIONS(2001),
    [sym_static_ip_binding] = ACTIONS(2001),
    [sym_get_dns_list] = ACTIONS(2001),
    [sym_set_dns_list] = ACTIONS(2001),
    [sym_get_ip_address] = ACTIONS(2001),
    [sym_set_ip_address] = ACTIONS(2001),
    [sym_get_url_list] = ACTIONS(2001),
    [sym_add_url_entry] = ACTIONS(2001),
    [sym_delete_url_entry] = ACTIONS(2001),
    [sym_abs_value] = ACTIONS(2001),
    [sym_max_value] = ACTIONS(2001),
    [sym_min_value] = ACTIONS(2001),
    [sym_exp_value] = ACTIONS(2001),
    [sym_log_value] = ACTIONS(2001),
    [sym_log10_value] = ACTIONS(2001),
    [sym_power_value] = ACTIONS(2001),
    [sym_sqrt_value] = ACTIONS(2001),
    [sym_raw_be] = ACTIONS(2001),
    [sym_raw_le] = ACTIONS(2001),
    [sym_internal_queue_size_set] = ACTIONS(2001),
    [sym_internal_queue_size_get] = ACTIONS(2001),
    [sym_internal_threshold_set] = ACTIONS(2001),
    [sym_internal_threshold_get] = ACTIONS(2001),
    [sym_rebuilt_event] = ACTIONS(2001),
    [sym_do_custom_event] = ACTIONS(2001),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2001),
    [sym_clkmgr_set_clk_source] = ACTIONS(2001),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2001),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2001),
    [sym_clkmgr_get_timezone] = ACTIONS(2001),
    [sym_clkmgr_set_timezone] = ACTIONS(2001),
    [sym_clkmgr_get_resync_period] = ACTIONS(2001),
    [sym_clkmgr_set_resync_period] = ACTIONS(2001),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2001),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2001),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2001),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2001),
    [sym_clkmgr_get_timeservers] = ACTIONS(2001),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2001),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2001),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2001),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2001),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2001),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2001),
    [sym_set_log_level] = ACTIONS(2001),
    [sym_get_log_level] = ACTIONS(2001),
    [sym_amx_log] = ACTIONS(2001),
    [sym_smtp_server_config_set] = ACTIONS(2001),
    [sym_smtp_server_config_get] = ACTIONS(2001),
    [sym_smtp_send] = ACTIONS(2001),
    [sym_validate_netlinx_account] = ACTIONS(2001),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2001),
    [sym_authenticate_certificate] = ACTIONS(2001),
    [sym_audit_netlinx_session_event] = ACTIONS(2001),
    [sym_audit_netlinx_generic_event] = ACTIONS(2001),
    [sym_get_available_flash_disk_space] = ACTIONS(2001),
    [sym_get_max_flash_disk_space] = ACTIONS(2001),
    [sym_data_create_feed] = ACTIONS(2001),
    [sym_data_delete_feed] = ACTIONS(2001),
    [sym_data_publish_feed] = ACTIONS(2001),
    [sym_data_get_published_feed] = ACTIONS(2001),
    [sym_data_add_record] = ACTIONS(2001),
    [sym_data_get_event_record] = ACTIONS(2001),
    [sym__wc_data_create_feed] = ACTIONS(2001),
    [sym__wc_data_add_record] = ACTIONS(2001),
    [sym__wc_data_get_event_record] = ACTIONS(2001),
    [sym_format] = ACTIONS(2001),
    [sym_astro_clock] = ACTIONS(2001),
    [sym_set_outdoor_temperature] = ACTIONS(2001),
    [sym_set_virtual_level_count] = ACTIONS(2001),
    [sym_set_virtual_channel_count] = ACTIONS(2001),
    [sym_set_virtual_port_count] = ACTIONS(2001),
    [sym__wc_encode_int] = ACTIONS(2001),
    [sym__wc_decode_int] = ACTIONS(2001),
    [sym__wc_int] = ACTIONS(2001),
    [sym__wc_to_ch_int] = ACTIONS(2001),
    [sym__ch_to_wc_int] = ACTIONS(2001),
    [sym__wc_find_string_int] = ACTIONS(2001),
    [sym__wc_left_string_int] = ACTIONS(2001),
    [sym__wc_lower_string_int] = ACTIONS(2001),
    [sym__wc_mid_string_int] = ACTIONS(2001),
    [sym__wc_remove_string_int] = ACTIONS(2001),
    [sym__wc_right_string_int] = ACTIONS(2001),
    [sym__wc_upper_string_int] = ACTIONS(2001),
    [sym__wc_compare_string_int] = ACTIONS(2001),
    [sym__wc_get_buffer_char_int] = ACTIONS(2001),
    [sym__wc_get_buffer_string_int] = ACTIONS(2001),
    [sym__wc_concat_string_int] = ACTIONS(2001),
    [sym__wc_file_open_int] = ACTIONS(2001),
    [sym__wc_file_close_int] = ACTIONS(2001),
    [sym__wc_file_read_int] = ACTIONS(2001),
    [sym__wc_file_read_line_int] = ACTIONS(2001),
    [sym__wc_file_write_int] = ACTIONS(2001),
    [sym__wc_file_write_line_int] = ACTIONS(2001),
    [aux_sym_preproc_include_token1] = ACTIONS(2001),
    [aux_sym_preproc_define_token1] = ACTIONS(2003),
    [aux_sym_preproc_warn_token1] = ACTIONS(2003),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2003),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2003),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2003),
    [anon_sym_LBRACK] = ACTIONS(2003),
    [anon_sym_LBRACE] = ACTIONS(2003),
    [anon_sym_LPAREN] = ACTIONS(2001),
    [anon_sym_BANG] = ACTIONS(2003),
    [anon_sym_TILDE] = ACTIONS(2003),
    [anon_sym_DASH] = ACTIONS(2001),
    [anon_sym_PLUS] = ACTIONS(2001),
    [anon_sym_DASH_DASH] = ACTIONS(2003),
    [anon_sym_PLUS_PLUS] = ACTIONS(2003),
    [anon_sym_DQUOTE] = ACTIONS(2003),
    [anon_sym_SQUOTE] = ACTIONS(2003),
    [sym_number_literal] = ACTIONS(2003),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(427)] = {
    [ts_builtin_sym_end] = ACTIONS(1963),
    [sym_identifier] = ACTIONS(1961),
    [sym_program_name_keyword] = ACTIONS(1961),
    [sym_module_name_keyword] = ACTIONS(1961),
    [sym_define_device_keyword] = ACTIONS(1961),
    [sym_define_combine_keyword] = ACTIONS(1961),
    [sym_define_constant_keyword] = ACTIONS(1961),
    [sym_define_type_keyword] = ACTIONS(1961),
    [sym_define_variable_keyword] = ACTIONS(1961),
    [sym_define_system_variable_keyword] = ACTIONS(1961),
    [sym_define_connect_level_keyword] = ACTIONS(1961),
    [sym_define_latching_keyword] = ACTIONS(1961),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1961),
    [sym_define_toggling_keyword] = ACTIONS(1961),
    [sym_define_call_keyword] = ACTIONS(1961),
    [sym_define_function_keyword] = ACTIONS(1961),
    [sym_define_library_function_keyword] = ACTIONS(1961),
    [sym_define_start_keyword] = ACTIONS(1961),
    [sym_define_event_keyword] = ACTIONS(1961),
    [sym_define_program_keyword] = ACTIONS(1961),
    [sym_define_module_keyword] = ACTIONS(1961),
    [sym_char_keyword] = ACTIONS(1961),
    [sym_widechar_keyword] = ACTIONS(1961),
    [sym_integer_keyword] = ACTIONS(1961),
    [sym_sinteger_keyword] = ACTIONS(1961),
    [sym_long_keyword] = ACTIONS(1961),
    [sym_slong_keyword] = ACTIONS(1961),
    [sym_float_keyword] = ACTIONS(1961),
    [sym_double_keyword] = ACTIONS(1961),
    [sym_variant_keyword] = ACTIONS(1961),
    [sym_variantarray_keyword] = ACTIONS(1961),
    [sym_local_var_keyword] = ACTIONS(1961),
    [sym_stack_var_keyword] = ACTIONS(1961),
    [sym_constant_keyword] = ACTIONS(1961),
    [sym_volatile_keyword] = ACTIONS(1961),
    [sym_non_volatile_keyword] = ACTIONS(1961),
    [sym_persistent_keyword] = ACTIONS(1961),
    [sym_dev_keyword] = ACTIONS(1961),
    [sym_devlev_keyword] = ACTIONS(1961),
    [sym_devchan_keyword] = ACTIONS(1961),
    [sym_if_keyword] = ACTIONS(1961),
    [sym_switch_keyword] = ACTIONS(1961),
    [sym_case_keyword] = ACTIONS(1961),
    [sym_default_keyword] = ACTIONS(1961),
    [sym_while_keyword] = ACTIONS(1961),
    [sym_for_keyword] = ACTIONS(1961),
    [sym_break_keyword] = ACTIONS(1961),
    [sym_continue_keyword] = ACTIONS(1961),
    [sym_return_keyword] = ACTIONS(1961),
    [sym_select_keyword] = ACTIONS(1961),
    [sym_active_keyword] = ACTIONS(1961),
    [sym_struct_keyword] = ACTIONS(1961),
    [sym_structure_keyword] = ACTIONS(1961),
    [sym_button_event_keyword] = ACTIONS(1961),
    [sym_channel_event_keyword] = ACTIONS(1961),
    [sym_level_event_keyword] = ACTIONS(1961),
    [sym_data_event_keyword] = ACTIONS(1961),
    [sym_timeline_event_keyword] = ACTIONS(1961),
    [sym_custom_event_keyword] = ACTIONS(1961),
    [sym_push_keyword] = ACTIONS(1961),
    [sym_release_keyword] = ACTIONS(1961),
    [aux_sym_on_keyword_token1] = ACTIONS(1961),
    [aux_sym_off_keyword_token1] = ACTIONS(1961),
    [sym_send_level_keyword] = ACTIONS(1961),
    [sym_send_string_keyword] = ACTIONS(1961),
    [sym_send_command_keyword] = ACTIONS(1961),
    [sym_clear_buffer_keyword] = ACTIONS(1961),
    [sym_create_buffer_keyword] = ACTIONS(1961),
    [sym_create_multi_buffer_keyword] = ACTIONS(1961),
    [sym_call_keyword] = ACTIONS(1961),
    [sym_system_call_keyword] = ACTIONS(1961),
    [sym_devchan_to_keyword] = ACTIONS(1961),
    [sym_devchan_min_to_keyword] = ACTIONS(1961),
    [sym_devchan_total_off_keyword] = ACTIONS(1961),
    [sym_devchan_pulse_keyword] = ACTIONS(1961),
    [sym_wait_keyword] = ACTIONS(1961),
    [sym_cancel_wait_keyword] = ACTIONS(1961),
    [sym_wait_until_keyword] = ACTIONS(1961),
    [sym_cancel_wait_until_keyword] = ACTIONS(1961),
    [sym_cancel_all_wait_keyword] = ACTIONS(1961),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1961),
    [sym_not] = ACTIONS(1961),
    [sym_bnot] = ACTIONS(1961),
    [sym___file__] = ACTIONS(1961),
    [sym___date__] = ACTIONS(1961),
    [sym___ldate__] = ACTIONS(1961),
    [sym___time__] = ACTIONS(1961),
    [sym___line__] = ACTIONS(1961),
    [sym___name__] = ACTIONS(1961),
    [sym___netlinx__] = ACTIONS(1961),
    [sym_netlinx_axi_version] = ACTIONS(1961),
    [sym_url_flg_tcp] = ACTIONS(1961),
    [sym_url_flg_accinfopresent] = ACTIONS(1961),
    [sym_url_flg_temp] = ACTIONS(1961),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1961),
    [sym_url_flg_stat_mask] = ACTIONS(1961),
    [sym_url_flg_stat_lookup] = ACTIONS(1961),
    [sym_url_flg_stat_connecting] = ACTIONS(1961),
    [sym_url_flg_stat_waiting] = ACTIONS(1961),
    [sym_url_flg_stat_connected] = ACTIONS(1961),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1961),
    [sym_first_virtual_device] = ACTIONS(1961),
    [sym_first_local_port] = ACTIONS(1961),
    [sym_dynamic_virtual_device] = ACTIONS(1961),
    [sym_timeline_once] = ACTIONS(1961),
    [sym_timeline_repeat] = ACTIONS(1961),
    [sym_timeline_absolute] = ACTIONS(1961),
    [sym_timeline_relative] = ACTIONS(1961),
    [sym_do_push_timed_infinite] = ACTIONS(1961),
    [sym_source_type_no_address] = ACTIONS(1961),
    [sym_source_type_neuron_id] = ACTIONS(1961),
    [sym_source_type_ip_address] = ACTIONS(1961),
    [sym_source_type_axlink] = ACTIONS(1961),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1961),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1961),
    [sym_source_type_ip_socket_address] = ACTIONS(1961),
    [sym_source_type_rs232] = ACTIONS(1961),
    [sym_source_type_internal] = ACTIONS(1961),
    [sym_source_type_mac_address] = ACTIONS(1961),
    [sym_source_type_ipv4_port] = ACTIONS(1961),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1961),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1961),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1961),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1961),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1961),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1961),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1961),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1961),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1961),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1961),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1961),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1961),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1961),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1961),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1961),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1961),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1961),
    [sym_true] = ACTIONS(1961),
    [sym_false] = ACTIONS(1961),
    [sym_file_read_only] = ACTIONS(1961),
    [sym_file_rw_new] = ACTIONS(1961),
    [sym_file_rw_append] = ACTIONS(1961),
    [sym_ip_tcp] = ACTIONS(1961),
    [sym_ip_udp] = ACTIONS(1961),
    [sym_ip_udp_2way] = ACTIONS(1961),
    [sym_xml_encode_types] = ACTIONS(1961),
    [sym_xml_encode_char_as_list] = ACTIONS(1961),
    [sym_xml_encode_le] = ACTIONS(1961),
    [sym_xml_decode_types] = ACTIONS(1961),
    [sym_xml_decode_no_preserve] = ACTIONS(1961),
    [sym_duet_dev_type_display_device] = ACTIONS(1961),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1961),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1961),
    [sym_duet_dev_type_io_device] = ACTIONS(1961),
    [sym_duet_dev_type_relay_device] = ACTIONS(1961),
    [sym_duet_dev_type_ups] = ACTIONS(1961),
    [sym_duet_dev_type_amplifier] = ACTIONS(1961),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1961),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1961),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1961),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1961),
    [sym_duet_dev_type_camera] = ACTIONS(1961),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1961),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1961),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1961),
    [sym_duet_dev_type_dss] = ACTIONS(1961),
    [sym_duet_dev_type_dvr] = ACTIONS(1961),
    [sym_duet_dev_type_disc_device] = ACTIONS(1961),
    [sym_duet_dev_type_document_camera] = ACTIONS(1961),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1961),
    [sym_duet_dev_type_hvac] = ACTIONS(1961),
    [sym_duet_dev_type_keypad] = ACTIONS(1961),
    [sym_duet_dev_type_light] = ACTIONS(1961),
    [sym_duet_dev_type_monitor] = ACTIONS(1961),
    [sym_duet_dev_type_motor] = ACTIONS(1961),
    [sym_duet_dev_type_multi_window] = ACTIONS(1961),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1961),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1961),
    [sym_duet_dev_type_receiver] = ACTIONS(1961),
    [sym_duet_dev_type_security_system] = ACTIONS(1961),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1961),
    [sym_duet_dev_type_settop_box] = ACTIONS(1961),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1961),
    [sym_duet_dev_type_switcher] = ACTIONS(1961),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1961),
    [sym_duet_dev_type_tv] = ACTIONS(1961),
    [sym_duet_dev_type_utility] = ACTIONS(1961),
    [sym_duet_dev_type_vcr] = ACTIONS(1961),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1961),
    [sym_duet_dev_type_video_processor] = ACTIONS(1961),
    [sym_duet_dev_type_video_projector] = ACTIONS(1961),
    [sym_duet_dev_type_video_wall] = ACTIONS(1961),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1961),
    [sym_duet_dev_type_weather] = ACTIONS(1961),
    [sym_duet_dev_not_polled] = ACTIONS(1961),
    [sym_duet_dev_polled] = ACTIONS(1961),
    [sym_clkmgr_mode_network] = ACTIONS(1961),
    [sym_clkmgr_mode_standalone] = ACTIONS(1961),
    [sym_normal_standby] = ACTIONS(1961),
    [sym_normal_wake] = ACTIONS(1961),
    [sym_ip_multicast_ttl_option] = ACTIONS(1961),
    [sym_ip_ttl_subnet] = ACTIONS(1961),
    [sym_ip_ttl_site] = ACTIONS(1961),
    [sym_ip_ttl_region] = ACTIONS(1961),
    [sym_ip_ttl_continent] = ACTIONS(1961),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1961),
    [sym_ip_nodelay_on] = ACTIONS(1961),
    [sym_ip_nodelay_off] = ACTIONS(1961),
    [sym_amx_error] = ACTIONS(1961),
    [sym_amx_warning] = ACTIONS(1961),
    [sym_amx_info] = ACTIONS(1961),
    [sym_amx_debug] = ACTIONS(1961),
    [sym_netlinx_login_success] = ACTIONS(1961),
    [sym_netlinx_login_fail] = ACTIONS(1961),
    [sym_netlinx_logout] = ACTIONS(1961),
    [sym_null_str] = ACTIONS(1961),
    [sym_smtp_address] = ACTIONS(1961),
    [sym_smtp_port_number] = ACTIONS(1961),
    [sym_smtp_username] = ACTIONS(1961),
    [sym_smtp_password] = ACTIONS(1961),
    [sym_smtp_require_tls] = ACTIONS(1961),
    [sym_smtp_from] = ACTIONS(1961),
    [sym_smtp_tls_true] = ACTIONS(1961),
    [sym_smtp_tls_false] = ACTIONS(1961),
    [sym_listview_on_row_select_event] = ACTIONS(1961),
    [sym_data_structure_unknown] = ACTIONS(1961),
    [sym_data_structure_datafeed] = ACTIONS(1961),
    [sym_data_structure_datarecordset] = ACTIONS(1961),
    [sym_data_structure_datarecord] = ACTIONS(1961),
    [sym_data_structure_datafield] = ACTIONS(1961),
    [sym_data_max_content_field_count] = ACTIONS(1961),
    [sym_data_max_metadata_field_count] = ACTIONS(1961),
    [sym_data_max_id_length] = ACTIONS(1961),
    [sym_data_max_name_length] = ACTIONS(1961),
    [sym_data_max_description_length] = ACTIONS(1961),
    [sym_data_max_type_length] = ACTIONS(1961),
    [sym_data_max_format_length] = ACTIONS(1961),
    [sym_data_max_source_length] = ACTIONS(1961),
    [sym_data_max_value_length] = ACTIONS(1961),
    [sym_data_max_label_length] = ACTIONS(1961),
    [sym_data_max_url_length] = ACTIONS(1961),
    [sym_max_last_login_info_length] = ACTIONS(1961),
    [sym_data_type_unknown] = ACTIONS(1961),
    [sym_data_type_string] = ACTIONS(1961),
    [sym_data_type_datetime] = ACTIONS(1961),
    [sym_data_type_date] = ACTIONS(1961),
    [sym_data_type_time] = ACTIONS(1961),
    [sym_data_type_image] = ACTIONS(1961),
    [sym_data_format_url] = ACTIONS(1961),
    [sym_data_format_phone] = ACTIONS(1961),
    [sym_data_format_email] = ACTIONS(1961),
    [sym_data_format_iso8601] = ACTIONS(1961),
    [sym_auditlog_priv] = ACTIONS(1961),
    [sym_configuration_priv] = ACTIONS(1961),
    [sym_deviceconfig_priv] = ACTIONS(1961),
    [sym_ftp_priv] = ACTIONS(1961),
    [sym_http_priv] = ACTIONS(1961),
    [sym_networkconfig_priv] = ACTIONS(1961),
    [sym_programport_priv] = ACTIONS(1961),
    [sym_remoteui_priv] = ACTIONS(1961),
    [sym_securitycontrol_priv] = ACTIONS(1961),
    [sym_softwaremanagement_priv] = ACTIONS(1961),
    [sym_terminal_priv] = ACTIONS(1961),
    [sym_tpadmin_priv] = ACTIONS(1961),
    [sym_user1_priv] = ACTIONS(1961),
    [sym_user2_priv] = ACTIONS(1961),
    [sym_user3_priv] = ACTIONS(1961),
    [sym_user4_priv] = ACTIONS(1961),
    [sym_usermanagement_priv] = ACTIONS(1961),
    [sym_valid_account] = ACTIONS(1961),
    [sym_err_name_invalid] = ACTIONS(1961),
    [sym_err_password_invalid] = ACTIONS(1961),
    [sym_err_invalid_account] = ACTIONS(1961),
    [sym_err_no_authorization] = ACTIONS(1961),
    [sym_err_invalid_parameter] = ACTIONS(1961),
    [sym_err_user_locked_out] = ACTIONS(1961),
    [sym_err_user_account_expired] = ACTIONS(1961),
    [sym_err_general_error] = ACTIONS(1961),
    [sym_err_authentication_error] = ACTIONS(1961),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1961),
    [sym_tls_validate_certificate] = ACTIONS(1961),
    [sym_tdata] = ACTIONS(1961),
    [sym_tchannel] = ACTIONS(1961),
    [sym_tlevel] = ACTIONS(1961),
    [sym_tbutton] = ACTIONS(1961),
    [sym_ttimeline] = ACTIONS(1961),
    [sym_tcustom] = ACTIONS(1961),
    [sym_url_struct] = ACTIONS(1961),
    [sym_dns_struct] = ACTIONS(1961),
    [sym_ip_address_struct] = ACTIONS(1961),
    [sym_dev_info_struct] = ACTIONS(1961),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1961),
    [sym_data_feed] = ACTIONS(1961),
    [sym_data_field] = ACTIONS(1961),
    [sym_data_record] = ACTIONS(1961),
    [sym_wc_data_feed] = ACTIONS(1961),
    [sym_wc_data_field] = ACTIONS(1961),
    [sym_wc_data_record] = ACTIONS(1961),
    [sym_last_login_info] = ACTIONS(1961),
    [sym____reserved___] = ACTIONS(1961),
    [sym_date] = ACTIONS(1961),
    [sym_day] = ACTIONS(1961),
    [sym_time] = ACTIONS(1961),
    [sym_ldate] = ACTIONS(1961),
    [sym_push_device] = ACTIONS(1961),
    [sym_push_channel] = ACTIONS(1961),
    [sym_push_devchan] = ACTIONS(1961),
    [sym_release_device] = ACTIONS(1961),
    [sym_release_channel] = ACTIONS(1961),
    [sym_release_devchan] = ACTIONS(1961),
    [sym_master_slot] = ACTIONS(1961),
    [sym_get_pulse_time] = ACTIONS(1961),
    [sym_get_timer] = ACTIONS(1961),
    [sym_master_sn] = ACTIONS(1961),
    [sym_system_number] = ACTIONS(1961),
    [sym_button] = ACTIONS(1961),
    [sym_dv_channel] = ACTIONS(1961),
    [sym_data] = ACTIONS(1961),
    [sym_level] = ACTIONS(1961),
    [sym_timeline] = ACTIONS(1961),
    [sym_channel] = ACTIONS(1961),
    [sym_custom] = ACTIONS(1961),
    [sym_atoi] = ACTIONS(1961),
    [sym_atol] = ACTIONS(1961),
    [sym_atof] = ACTIONS(1961),
    [sym_itoa] = ACTIONS(1961),
    [sym_itohex] = ACTIONS(1961),
    [sym_hextoi] = ACTIONS(1961),
    [sym_ftoa] = ACTIONS(1961),
    [sym_find_string] = ACTIONS(1961),
    [sym_left_string] = ACTIONS(1961),
    [sym_right_string] = ACTIONS(1961),
    [sym_mid_string] = ACTIONS(1961),
    [sym_remove_string] = ACTIONS(1961),
    [sym_get_buffer_string] = ACTIONS(1961),
    [sym_get_buffer_char] = ACTIONS(1961),
    [sym_get_multi_buffer_string] = ACTIONS(1961),
    [sym_length_string] = ACTIONS(1961),
    [sym_max_length_string] = ACTIONS(1961),
    [sym_set_length_string] = ACTIONS(1961),
    [sym_compare_string] = ACTIONS(1961),
    [sym_upper_string] = ACTIONS(1961),
    [sym_lower_string] = ACTIONS(1961),
    [sym_get_last] = ACTIONS(1961),
    [sym_date_to_day] = ACTIONS(1961),
    [sym_date_to_month] = ACTIONS(1961),
    [sym_date_to_year] = ACTIONS(1961),
    [sym_time_to_hour] = ACTIONS(1961),
    [sym_time_to_minute] = ACTIONS(1961),
    [sym_time_to_second] = ACTIONS(1961),
    [sym_day_of_week] = ACTIONS(1961),
    [sym_set_pulse_time] = ACTIONS(1961),
    [sym_set_timer] = ACTIONS(1961),
    [sym_file_open] = ACTIONS(1961),
    [sym_file_close] = ACTIONS(1961),
    [sym_file_read] = ACTIONS(1961),
    [sym_file_read_line] = ACTIONS(1961),
    [sym_file_write] = ACTIONS(1961),
    [sym_file_write_line] = ACTIONS(1961),
    [sym_file_copy] = ACTIONS(1961),
    [sym_file_delete] = ACTIONS(1961),
    [sym_file_rename] = ACTIONS(1961),
    [sym_file_dir] = ACTIONS(1961),
    [sym_file_getdir] = ACTIONS(1961),
    [sym_file_setdir] = ACTIONS(1961),
    [sym_file_create_dir] = ACTIONS(1961),
    [sym_file_remove_dir] = ACTIONS(1961),
    [sym_file_seek] = ACTIONS(1961),
    [sym_length_array] = ACTIONS(1961),
    [sym_max_length_array] = ACTIONS(1961),
    [sym_set_length_array] = ACTIONS(1961),
    [sym_ip_client_open] = ACTIONS(1961),
    [sym_ip_client_close] = ACTIONS(1961),
    [sym_ip_server_open] = ACTIONS(1961),
    [sym_ip_server_close] = ACTIONS(1961),
    [sym_ip_bound_client_open] = ACTIONS(1961),
    [sym_ip_mc_server_open] = ACTIONS(1961),
    [sym_ip_set_option] = ACTIONS(1961),
    [sym_ssh_client_open] = ACTIONS(1961),
    [sym_ssh_client_close] = ACTIONS(1961),
    [sym_tls_client_open] = ACTIONS(1961),
    [sym_tls_client_close] = ACTIONS(1961),
    [sym_do_push] = ACTIONS(1961),
    [sym_do_push_timed] = ACTIONS(1961),
    [sym_do_release] = ACTIONS(1961),
    [sym_device_id] = ACTIONS(1961),
    [sym_device_info] = ACTIONS(1961),
    [sym_device_id_string] = ACTIONS(1961),
    [sym_device_standby] = ACTIONS(1961),
    [sym_device_wake] = ACTIONS(1961),
    [sym_timeline_create] = ACTIONS(1961),
    [sym_timeline_kill] = ACTIONS(1961),
    [sym_timeline_reload] = ACTIONS(1961),
    [sym_timeline_pause] = ACTIONS(1961),
    [sym_timeline_restart] = ACTIONS(1961),
    [sym_timeline_set] = ACTIONS(1961),
    [sym_timeline_get] = ACTIONS(1961),
    [sym_timeline_active] = ACTIONS(1961),
    [sym_timeline_dynamic_id] = ACTIONS(1961),
    [sym_variable_to_string] = ACTIONS(1961),
    [sym_string_to_variable] = ACTIONS(1961),
    [sym_length_variable_to_string] = ACTIONS(1961),
    [sym_type_cast] = ACTIONS(1961),
    [sym_variable_to_xml] = ACTIONS(1961),
    [sym_xml_to_variable] = ACTIONS(1961),
    [sym_length_variable_to_xml] = ACTIONS(1961),
    [sym_redirect_string] = ACTIONS(1961),
    [sym_random_number] = ACTIONS(1961),
    [sym_get_unique_id] = ACTIONS(1961),
    [sym_get_system_number] = ACTIONS(1961),
    [sym_set_system_number] = ACTIONS(1961),
    [sym_get_serial_number] = ACTIONS(1961),
    [sym_get_master_build] = ACTIONS(1961),
    [sym_set_validation_code] = ACTIONS(1961),
    [sym_reboot] = ACTIONS(1961),
    [sym_load_duet_module] = ACTIONS(1961),
    [sym_unload_duet_module] = ACTIONS(1961),
    [sym_duet_mem_size_set] = ACTIONS(1961),
    [sym_duet_mem_size_get] = ACTIONS(1961),
    [sym_dynamic_application_device] = ACTIONS(1961),
    [sym_dynamic_polled_port] = ACTIONS(1961),
    [sym_static_port_binding] = ACTIONS(1961),
    [sym_static_ip_binding] = ACTIONS(1961),
    [sym_get_dns_list] = ACTIONS(1961),
    [sym_set_dns_list] = ACTIONS(1961),
    [sym_get_ip_address] = ACTIONS(1961),
    [sym_set_ip_address] = ACTIONS(1961),
    [sym_get_url_list] = ACTIONS(1961),
    [sym_add_url_entry] = ACTIONS(1961),
    [sym_delete_url_entry] = ACTIONS(1961),
    [sym_abs_value] = ACTIONS(1961),
    [sym_max_value] = ACTIONS(1961),
    [sym_min_value] = ACTIONS(1961),
    [sym_exp_value] = ACTIONS(1961),
    [sym_log_value] = ACTIONS(1961),
    [sym_log10_value] = ACTIONS(1961),
    [sym_power_value] = ACTIONS(1961),
    [sym_sqrt_value] = ACTIONS(1961),
    [sym_raw_be] = ACTIONS(1961),
    [sym_raw_le] = ACTIONS(1961),
    [sym_internal_queue_size_set] = ACTIONS(1961),
    [sym_internal_queue_size_get] = ACTIONS(1961),
    [sym_internal_threshold_set] = ACTIONS(1961),
    [sym_internal_threshold_get] = ACTIONS(1961),
    [sym_rebuilt_event] = ACTIONS(1961),
    [sym_do_custom_event] = ACTIONS(1961),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1961),
    [sym_clkmgr_set_clk_source] = ACTIONS(1961),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1961),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1961),
    [sym_clkmgr_get_timezone] = ACTIONS(1961),
    [sym_clkmgr_set_timezone] = ACTIONS(1961),
    [sym_clkmgr_get_resync_period] = ACTIONS(1961),
    [sym_clkmgr_set_resync_period] = ACTIONS(1961),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1961),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1961),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1961),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1961),
    [sym_clkmgr_get_timeservers] = ACTIONS(1961),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1961),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1961),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1961),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1961),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1961),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1961),
    [sym_set_log_level] = ACTIONS(1961),
    [sym_get_log_level] = ACTIONS(1961),
    [sym_amx_log] = ACTIONS(1961),
    [sym_smtp_server_config_set] = ACTIONS(1961),
    [sym_smtp_server_config_get] = ACTIONS(1961),
    [sym_smtp_send] = ACTIONS(1961),
    [sym_validate_netlinx_account] = ACTIONS(1961),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1961),
    [sym_authenticate_certificate] = ACTIONS(1961),
    [sym_audit_netlinx_session_event] = ACTIONS(1961),
    [sym_audit_netlinx_generic_event] = ACTIONS(1961),
    [sym_get_available_flash_disk_space] = ACTIONS(1961),
    [sym_get_max_flash_disk_space] = ACTIONS(1961),
    [sym_data_create_feed] = ACTIONS(1961),
    [sym_data_delete_feed] = ACTIONS(1961),
    [sym_data_publish_feed] = ACTIONS(1961),
    [sym_data_get_published_feed] = ACTIONS(1961),
    [sym_data_add_record] = ACTIONS(1961),
    [sym_data_get_event_record] = ACTIONS(1961),
    [sym__wc_data_create_feed] = ACTIONS(1961),
    [sym__wc_data_add_record] = ACTIONS(1961),
    [sym__wc_data_get_event_record] = ACTIONS(1961),
    [sym_format] = ACTIONS(1961),
    [sym_astro_clock] = ACTIONS(1961),
    [sym_set_outdoor_temperature] = ACTIONS(1961),
    [sym_set_virtual_level_count] = ACTIONS(1961),
    [sym_set_virtual_channel_count] = ACTIONS(1961),
    [sym_set_virtual_port_count] = ACTIONS(1961),
    [sym__wc_encode_int] = ACTIONS(1961),
    [sym__wc_decode_int] = ACTIONS(1961),
    [sym__wc_int] = ACTIONS(1961),
    [sym__wc_to_ch_int] = ACTIONS(1961),
    [sym__ch_to_wc_int] = ACTIONS(1961),
    [sym__wc_find_string_int] = ACTIONS(1961),
    [sym__wc_left_string_int] = ACTIONS(1961),
    [sym__wc_lower_string_int] = ACTIONS(1961),
    [sym__wc_mid_string_int] = ACTIONS(1961),
    [sym__wc_remove_string_int] = ACTIONS(1961),
    [sym__wc_right_string_int] = ACTIONS(1961),
    [sym__wc_upper_string_int] = ACTIONS(1961),
    [sym__wc_compare_string_int] = ACTIONS(1961),
    [sym__wc_get_buffer_char_int] = ACTIONS(1961),
    [sym__wc_get_buffer_string_int] = ACTIONS(1961),
    [sym__wc_concat_string_int] = ACTIONS(1961),
    [sym__wc_file_open_int] = ACTIONS(1961),
    [sym__wc_file_close_int] = ACTIONS(1961),
    [sym__wc_file_read_int] = ACTIONS(1961),
    [sym__wc_file_read_line_int] = ACTIONS(1961),
    [sym__wc_file_write_int] = ACTIONS(1961),
    [sym__wc_file_write_line_int] = ACTIONS(1961),
    [aux_sym_preproc_include_token1] = ACTIONS(1961),
    [aux_sym_preproc_define_token1] = ACTIONS(1963),
    [aux_sym_preproc_warn_token1] = ACTIONS(1963),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1963),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1963),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1963),
    [anon_sym_LBRACK] = ACTIONS(1963),
    [anon_sym_LBRACE] = ACTIONS(1963),
    [anon_sym_LPAREN] = ACTIONS(1961),
    [anon_sym_BANG] = ACTIONS(1963),
    [anon_sym_TILDE] = ACTIONS(1963),
    [anon_sym_DASH] = ACTIONS(1961),
    [anon_sym_PLUS] = ACTIONS(1961),
    [anon_sym_DASH_DASH] = ACTIONS(1963),
    [anon_sym_PLUS_PLUS] = ACTIONS(1963),
    [anon_sym_DQUOTE] = ACTIONS(1963),
    [anon_sym_SQUOTE] = ACTIONS(1963),
    [sym_number_literal] = ACTIONS(1963),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(428)] = {
    [ts_builtin_sym_end] = ACTIONS(1967),
    [sym_identifier] = ACTIONS(1965),
    [sym_program_name_keyword] = ACTIONS(1965),
    [sym_module_name_keyword] = ACTIONS(1965),
    [sym_define_device_keyword] = ACTIONS(1965),
    [sym_define_combine_keyword] = ACTIONS(1965),
    [sym_define_constant_keyword] = ACTIONS(1965),
    [sym_define_type_keyword] = ACTIONS(1965),
    [sym_define_variable_keyword] = ACTIONS(1965),
    [sym_define_system_variable_keyword] = ACTIONS(1965),
    [sym_define_connect_level_keyword] = ACTIONS(1965),
    [sym_define_latching_keyword] = ACTIONS(1965),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1965),
    [sym_define_toggling_keyword] = ACTIONS(1965),
    [sym_define_call_keyword] = ACTIONS(1965),
    [sym_define_function_keyword] = ACTIONS(1965),
    [sym_define_library_function_keyword] = ACTIONS(1965),
    [sym_define_start_keyword] = ACTIONS(1965),
    [sym_define_event_keyword] = ACTIONS(1965),
    [sym_define_program_keyword] = ACTIONS(1965),
    [sym_define_module_keyword] = ACTIONS(1965),
    [sym_char_keyword] = ACTIONS(1965),
    [sym_widechar_keyword] = ACTIONS(1965),
    [sym_integer_keyword] = ACTIONS(1965),
    [sym_sinteger_keyword] = ACTIONS(1965),
    [sym_long_keyword] = ACTIONS(1965),
    [sym_slong_keyword] = ACTIONS(1965),
    [sym_float_keyword] = ACTIONS(1965),
    [sym_double_keyword] = ACTIONS(1965),
    [sym_variant_keyword] = ACTIONS(1965),
    [sym_variantarray_keyword] = ACTIONS(1965),
    [sym_local_var_keyword] = ACTIONS(1965),
    [sym_stack_var_keyword] = ACTIONS(1965),
    [sym_constant_keyword] = ACTIONS(1965),
    [sym_volatile_keyword] = ACTIONS(1965),
    [sym_non_volatile_keyword] = ACTIONS(1965),
    [sym_persistent_keyword] = ACTIONS(1965),
    [sym_dev_keyword] = ACTIONS(1965),
    [sym_devlev_keyword] = ACTIONS(1965),
    [sym_devchan_keyword] = ACTIONS(1965),
    [sym_if_keyword] = ACTIONS(1965),
    [sym_switch_keyword] = ACTIONS(1965),
    [sym_case_keyword] = ACTIONS(1965),
    [sym_default_keyword] = ACTIONS(1965),
    [sym_while_keyword] = ACTIONS(1965),
    [sym_for_keyword] = ACTIONS(1965),
    [sym_break_keyword] = ACTIONS(1965),
    [sym_continue_keyword] = ACTIONS(1965),
    [sym_return_keyword] = ACTIONS(1965),
    [sym_select_keyword] = ACTIONS(1965),
    [sym_active_keyword] = ACTIONS(1965),
    [sym_struct_keyword] = ACTIONS(1965),
    [sym_structure_keyword] = ACTIONS(1965),
    [sym_button_event_keyword] = ACTIONS(1965),
    [sym_channel_event_keyword] = ACTIONS(1965),
    [sym_level_event_keyword] = ACTIONS(1965),
    [sym_data_event_keyword] = ACTIONS(1965),
    [sym_timeline_event_keyword] = ACTIONS(1965),
    [sym_custom_event_keyword] = ACTIONS(1965),
    [sym_push_keyword] = ACTIONS(1965),
    [sym_release_keyword] = ACTIONS(1965),
    [aux_sym_on_keyword_token1] = ACTIONS(1965),
    [aux_sym_off_keyword_token1] = ACTIONS(1965),
    [sym_send_level_keyword] = ACTIONS(1965),
    [sym_send_string_keyword] = ACTIONS(1965),
    [sym_send_command_keyword] = ACTIONS(1965),
    [sym_clear_buffer_keyword] = ACTIONS(1965),
    [sym_create_buffer_keyword] = ACTIONS(1965),
    [sym_create_multi_buffer_keyword] = ACTIONS(1965),
    [sym_call_keyword] = ACTIONS(1965),
    [sym_system_call_keyword] = ACTIONS(1965),
    [sym_devchan_to_keyword] = ACTIONS(1965),
    [sym_devchan_min_to_keyword] = ACTIONS(1965),
    [sym_devchan_total_off_keyword] = ACTIONS(1965),
    [sym_devchan_pulse_keyword] = ACTIONS(1965),
    [sym_wait_keyword] = ACTIONS(1965),
    [sym_cancel_wait_keyword] = ACTIONS(1965),
    [sym_wait_until_keyword] = ACTIONS(1965),
    [sym_cancel_wait_until_keyword] = ACTIONS(1965),
    [sym_cancel_all_wait_keyword] = ACTIONS(1965),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1965),
    [sym_not] = ACTIONS(1965),
    [sym_bnot] = ACTIONS(1965),
    [sym___file__] = ACTIONS(1965),
    [sym___date__] = ACTIONS(1965),
    [sym___ldate__] = ACTIONS(1965),
    [sym___time__] = ACTIONS(1965),
    [sym___line__] = ACTIONS(1965),
    [sym___name__] = ACTIONS(1965),
    [sym___netlinx__] = ACTIONS(1965),
    [sym_netlinx_axi_version] = ACTIONS(1965),
    [sym_url_flg_tcp] = ACTIONS(1965),
    [sym_url_flg_accinfopresent] = ACTIONS(1965),
    [sym_url_flg_temp] = ACTIONS(1965),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1965),
    [sym_url_flg_stat_mask] = ACTIONS(1965),
    [sym_url_flg_stat_lookup] = ACTIONS(1965),
    [sym_url_flg_stat_connecting] = ACTIONS(1965),
    [sym_url_flg_stat_waiting] = ACTIONS(1965),
    [sym_url_flg_stat_connected] = ACTIONS(1965),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1965),
    [sym_first_virtual_device] = ACTIONS(1965),
    [sym_first_local_port] = ACTIONS(1965),
    [sym_dynamic_virtual_device] = ACTIONS(1965),
    [sym_timeline_once] = ACTIONS(1965),
    [sym_timeline_repeat] = ACTIONS(1965),
    [sym_timeline_absolute] = ACTIONS(1965),
    [sym_timeline_relative] = ACTIONS(1965),
    [sym_do_push_timed_infinite] = ACTIONS(1965),
    [sym_source_type_no_address] = ACTIONS(1965),
    [sym_source_type_neuron_id] = ACTIONS(1965),
    [sym_source_type_ip_address] = ACTIONS(1965),
    [sym_source_type_axlink] = ACTIONS(1965),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1965),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1965),
    [sym_source_type_ip_socket_address] = ACTIONS(1965),
    [sym_source_type_rs232] = ACTIONS(1965),
    [sym_source_type_internal] = ACTIONS(1965),
    [sym_source_type_mac_address] = ACTIONS(1965),
    [sym_source_type_ipv4_port] = ACTIONS(1965),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1965),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1965),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1965),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1965),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1965),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1965),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1965),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1965),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1965),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1965),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1965),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1965),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1965),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1965),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1965),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1965),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1965),
    [sym_true] = ACTIONS(1965),
    [sym_false] = ACTIONS(1965),
    [sym_file_read_only] = ACTIONS(1965),
    [sym_file_rw_new] = ACTIONS(1965),
    [sym_file_rw_append] = ACTIONS(1965),
    [sym_ip_tcp] = ACTIONS(1965),
    [sym_ip_udp] = ACTIONS(1965),
    [sym_ip_udp_2way] = ACTIONS(1965),
    [sym_xml_encode_types] = ACTIONS(1965),
    [sym_xml_encode_char_as_list] = ACTIONS(1965),
    [sym_xml_encode_le] = ACTIONS(1965),
    [sym_xml_decode_types] = ACTIONS(1965),
    [sym_xml_decode_no_preserve] = ACTIONS(1965),
    [sym_duet_dev_type_display_device] = ACTIONS(1965),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1965),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1965),
    [sym_duet_dev_type_io_device] = ACTIONS(1965),
    [sym_duet_dev_type_relay_device] = ACTIONS(1965),
    [sym_duet_dev_type_ups] = ACTIONS(1965),
    [sym_duet_dev_type_amplifier] = ACTIONS(1965),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1965),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1965),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1965),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1965),
    [sym_duet_dev_type_camera] = ACTIONS(1965),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1965),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1965),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1965),
    [sym_duet_dev_type_dss] = ACTIONS(1965),
    [sym_duet_dev_type_dvr] = ACTIONS(1965),
    [sym_duet_dev_type_disc_device] = ACTIONS(1965),
    [sym_duet_dev_type_document_camera] = ACTIONS(1965),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1965),
    [sym_duet_dev_type_hvac] = ACTIONS(1965),
    [sym_duet_dev_type_keypad] = ACTIONS(1965),
    [sym_duet_dev_type_light] = ACTIONS(1965),
    [sym_duet_dev_type_monitor] = ACTIONS(1965),
    [sym_duet_dev_type_motor] = ACTIONS(1965),
    [sym_duet_dev_type_multi_window] = ACTIONS(1965),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1965),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1965),
    [sym_duet_dev_type_receiver] = ACTIONS(1965),
    [sym_duet_dev_type_security_system] = ACTIONS(1965),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1965),
    [sym_duet_dev_type_settop_box] = ACTIONS(1965),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1965),
    [sym_duet_dev_type_switcher] = ACTIONS(1965),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1965),
    [sym_duet_dev_type_tv] = ACTIONS(1965),
    [sym_duet_dev_type_utility] = ACTIONS(1965),
    [sym_duet_dev_type_vcr] = ACTIONS(1965),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1965),
    [sym_duet_dev_type_video_processor] = ACTIONS(1965),
    [sym_duet_dev_type_video_projector] = ACTIONS(1965),
    [sym_duet_dev_type_video_wall] = ACTIONS(1965),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1965),
    [sym_duet_dev_type_weather] = ACTIONS(1965),
    [sym_duet_dev_not_polled] = ACTIONS(1965),
    [sym_duet_dev_polled] = ACTIONS(1965),
    [sym_clkmgr_mode_network] = ACTIONS(1965),
    [sym_clkmgr_mode_standalone] = ACTIONS(1965),
    [sym_normal_standby] = ACTIONS(1965),
    [sym_normal_wake] = ACTIONS(1965),
    [sym_ip_multicast_ttl_option] = ACTIONS(1965),
    [sym_ip_ttl_subnet] = ACTIONS(1965),
    [sym_ip_ttl_site] = ACTIONS(1965),
    [sym_ip_ttl_region] = ACTIONS(1965),
    [sym_ip_ttl_continent] = ACTIONS(1965),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1965),
    [sym_ip_nodelay_on] = ACTIONS(1965),
    [sym_ip_nodelay_off] = ACTIONS(1965),
    [sym_amx_error] = ACTIONS(1965),
    [sym_amx_warning] = ACTIONS(1965),
    [sym_amx_info] = ACTIONS(1965),
    [sym_amx_debug] = ACTIONS(1965),
    [sym_netlinx_login_success] = ACTIONS(1965),
    [sym_netlinx_login_fail] = ACTIONS(1965),
    [sym_netlinx_logout] = ACTIONS(1965),
    [sym_null_str] = ACTIONS(1965),
    [sym_smtp_address] = ACTIONS(1965),
    [sym_smtp_port_number] = ACTIONS(1965),
    [sym_smtp_username] = ACTIONS(1965),
    [sym_smtp_password] = ACTIONS(1965),
    [sym_smtp_require_tls] = ACTIONS(1965),
    [sym_smtp_from] = ACTIONS(1965),
    [sym_smtp_tls_true] = ACTIONS(1965),
    [sym_smtp_tls_false] = ACTIONS(1965),
    [sym_listview_on_row_select_event] = ACTIONS(1965),
    [sym_data_structure_unknown] = ACTIONS(1965),
    [sym_data_structure_datafeed] = ACTIONS(1965),
    [sym_data_structure_datarecordset] = ACTIONS(1965),
    [sym_data_structure_datarecord] = ACTIONS(1965),
    [sym_data_structure_datafield] = ACTIONS(1965),
    [sym_data_max_content_field_count] = ACTIONS(1965),
    [sym_data_max_metadata_field_count] = ACTIONS(1965),
    [sym_data_max_id_length] = ACTIONS(1965),
    [sym_data_max_name_length] = ACTIONS(1965),
    [sym_data_max_description_length] = ACTIONS(1965),
    [sym_data_max_type_length] = ACTIONS(1965),
    [sym_data_max_format_length] = ACTIONS(1965),
    [sym_data_max_source_length] = ACTIONS(1965),
    [sym_data_max_value_length] = ACTIONS(1965),
    [sym_data_max_label_length] = ACTIONS(1965),
    [sym_data_max_url_length] = ACTIONS(1965),
    [sym_max_last_login_info_length] = ACTIONS(1965),
    [sym_data_type_unknown] = ACTIONS(1965),
    [sym_data_type_string] = ACTIONS(1965),
    [sym_data_type_datetime] = ACTIONS(1965),
    [sym_data_type_date] = ACTIONS(1965),
    [sym_data_type_time] = ACTIONS(1965),
    [sym_data_type_image] = ACTIONS(1965),
    [sym_data_format_url] = ACTIONS(1965),
    [sym_data_format_phone] = ACTIONS(1965),
    [sym_data_format_email] = ACTIONS(1965),
    [sym_data_format_iso8601] = ACTIONS(1965),
    [sym_auditlog_priv] = ACTIONS(1965),
    [sym_configuration_priv] = ACTIONS(1965),
    [sym_deviceconfig_priv] = ACTIONS(1965),
    [sym_ftp_priv] = ACTIONS(1965),
    [sym_http_priv] = ACTIONS(1965),
    [sym_networkconfig_priv] = ACTIONS(1965),
    [sym_programport_priv] = ACTIONS(1965),
    [sym_remoteui_priv] = ACTIONS(1965),
    [sym_securitycontrol_priv] = ACTIONS(1965),
    [sym_softwaremanagement_priv] = ACTIONS(1965),
    [sym_terminal_priv] = ACTIONS(1965),
    [sym_tpadmin_priv] = ACTIONS(1965),
    [sym_user1_priv] = ACTIONS(1965),
    [sym_user2_priv] = ACTIONS(1965),
    [sym_user3_priv] = ACTIONS(1965),
    [sym_user4_priv] = ACTIONS(1965),
    [sym_usermanagement_priv] = ACTIONS(1965),
    [sym_valid_account] = ACTIONS(1965),
    [sym_err_name_invalid] = ACTIONS(1965),
    [sym_err_password_invalid] = ACTIONS(1965),
    [sym_err_invalid_account] = ACTIONS(1965),
    [sym_err_no_authorization] = ACTIONS(1965),
    [sym_err_invalid_parameter] = ACTIONS(1965),
    [sym_err_user_locked_out] = ACTIONS(1965),
    [sym_err_user_account_expired] = ACTIONS(1965),
    [sym_err_general_error] = ACTIONS(1965),
    [sym_err_authentication_error] = ACTIONS(1965),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1965),
    [sym_tls_validate_certificate] = ACTIONS(1965),
    [sym_tdata] = ACTIONS(1965),
    [sym_tchannel] = ACTIONS(1965),
    [sym_tlevel] = ACTIONS(1965),
    [sym_tbutton] = ACTIONS(1965),
    [sym_ttimeline] = ACTIONS(1965),
    [sym_tcustom] = ACTIONS(1965),
    [sym_url_struct] = ACTIONS(1965),
    [sym_dns_struct] = ACTIONS(1965),
    [sym_ip_address_struct] = ACTIONS(1965),
    [sym_dev_info_struct] = ACTIONS(1965),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1965),
    [sym_data_feed] = ACTIONS(1965),
    [sym_data_field] = ACTIONS(1965),
    [sym_data_record] = ACTIONS(1965),
    [sym_wc_data_feed] = ACTIONS(1965),
    [sym_wc_data_field] = ACTIONS(1965),
    [sym_wc_data_record] = ACTIONS(1965),
    [sym_last_login_info] = ACTIONS(1965),
    [sym____reserved___] = ACTIONS(1965),
    [sym_date] = ACTIONS(1965),
    [sym_day] = ACTIONS(1965),
    [sym_time] = ACTIONS(1965),
    [sym_ldate] = ACTIONS(1965),
    [sym_push_device] = ACTIONS(1965),
    [sym_push_channel] = ACTIONS(1965),
    [sym_push_devchan] = ACTIONS(1965),
    [sym_release_device] = ACTIONS(1965),
    [sym_release_channel] = ACTIONS(1965),
    [sym_release_devchan] = ACTIONS(1965),
    [sym_master_slot] = ACTIONS(1965),
    [sym_get_pulse_time] = ACTIONS(1965),
    [sym_get_timer] = ACTIONS(1965),
    [sym_master_sn] = ACTIONS(1965),
    [sym_system_number] = ACTIONS(1965),
    [sym_button] = ACTIONS(1965),
    [sym_dv_channel] = ACTIONS(1965),
    [sym_data] = ACTIONS(1965),
    [sym_level] = ACTIONS(1965),
    [sym_timeline] = ACTIONS(1965),
    [sym_channel] = ACTIONS(1965),
    [sym_custom] = ACTIONS(1965),
    [sym_atoi] = ACTIONS(1965),
    [sym_atol] = ACTIONS(1965),
    [sym_atof] = ACTIONS(1965),
    [sym_itoa] = ACTIONS(1965),
    [sym_itohex] = ACTIONS(1965),
    [sym_hextoi] = ACTIONS(1965),
    [sym_ftoa] = ACTIONS(1965),
    [sym_find_string] = ACTIONS(1965),
    [sym_left_string] = ACTIONS(1965),
    [sym_right_string] = ACTIONS(1965),
    [sym_mid_string] = ACTIONS(1965),
    [sym_remove_string] = ACTIONS(1965),
    [sym_get_buffer_string] = ACTIONS(1965),
    [sym_get_buffer_char] = ACTIONS(1965),
    [sym_get_multi_buffer_string] = ACTIONS(1965),
    [sym_length_string] = ACTIONS(1965),
    [sym_max_length_string] = ACTIONS(1965),
    [sym_set_length_string] = ACTIONS(1965),
    [sym_compare_string] = ACTIONS(1965),
    [sym_upper_string] = ACTIONS(1965),
    [sym_lower_string] = ACTIONS(1965),
    [sym_get_last] = ACTIONS(1965),
    [sym_date_to_day] = ACTIONS(1965),
    [sym_date_to_month] = ACTIONS(1965),
    [sym_date_to_year] = ACTIONS(1965),
    [sym_time_to_hour] = ACTIONS(1965),
    [sym_time_to_minute] = ACTIONS(1965),
    [sym_time_to_second] = ACTIONS(1965),
    [sym_day_of_week] = ACTIONS(1965),
    [sym_set_pulse_time] = ACTIONS(1965),
    [sym_set_timer] = ACTIONS(1965),
    [sym_file_open] = ACTIONS(1965),
    [sym_file_close] = ACTIONS(1965),
    [sym_file_read] = ACTIONS(1965),
    [sym_file_read_line] = ACTIONS(1965),
    [sym_file_write] = ACTIONS(1965),
    [sym_file_write_line] = ACTIONS(1965),
    [sym_file_copy] = ACTIONS(1965),
    [sym_file_delete] = ACTIONS(1965),
    [sym_file_rename] = ACTIONS(1965),
    [sym_file_dir] = ACTIONS(1965),
    [sym_file_getdir] = ACTIONS(1965),
    [sym_file_setdir] = ACTIONS(1965),
    [sym_file_create_dir] = ACTIONS(1965),
    [sym_file_remove_dir] = ACTIONS(1965),
    [sym_file_seek] = ACTIONS(1965),
    [sym_length_array] = ACTIONS(1965),
    [sym_max_length_array] = ACTIONS(1965),
    [sym_set_length_array] = ACTIONS(1965),
    [sym_ip_client_open] = ACTIONS(1965),
    [sym_ip_client_close] = ACTIONS(1965),
    [sym_ip_server_open] = ACTIONS(1965),
    [sym_ip_server_close] = ACTIONS(1965),
    [sym_ip_bound_client_open] = ACTIONS(1965),
    [sym_ip_mc_server_open] = ACTIONS(1965),
    [sym_ip_set_option] = ACTIONS(1965),
    [sym_ssh_client_open] = ACTIONS(1965),
    [sym_ssh_client_close] = ACTIONS(1965),
    [sym_tls_client_open] = ACTIONS(1965),
    [sym_tls_client_close] = ACTIONS(1965),
    [sym_do_push] = ACTIONS(1965),
    [sym_do_push_timed] = ACTIONS(1965),
    [sym_do_release] = ACTIONS(1965),
    [sym_device_id] = ACTIONS(1965),
    [sym_device_info] = ACTIONS(1965),
    [sym_device_id_string] = ACTIONS(1965),
    [sym_device_standby] = ACTIONS(1965),
    [sym_device_wake] = ACTIONS(1965),
    [sym_timeline_create] = ACTIONS(1965),
    [sym_timeline_kill] = ACTIONS(1965),
    [sym_timeline_reload] = ACTIONS(1965),
    [sym_timeline_pause] = ACTIONS(1965),
    [sym_timeline_restart] = ACTIONS(1965),
    [sym_timeline_set] = ACTIONS(1965),
    [sym_timeline_get] = ACTIONS(1965),
    [sym_timeline_active] = ACTIONS(1965),
    [sym_timeline_dynamic_id] = ACTIONS(1965),
    [sym_variable_to_string] = ACTIONS(1965),
    [sym_string_to_variable] = ACTIONS(1965),
    [sym_length_variable_to_string] = ACTIONS(1965),
    [sym_type_cast] = ACTIONS(1965),
    [sym_variable_to_xml] = ACTIONS(1965),
    [sym_xml_to_variable] = ACTIONS(1965),
    [sym_length_variable_to_xml] = ACTIONS(1965),
    [sym_redirect_string] = ACTIONS(1965),
    [sym_random_number] = ACTIONS(1965),
    [sym_get_unique_id] = ACTIONS(1965),
    [sym_get_system_number] = ACTIONS(1965),
    [sym_set_system_number] = ACTIONS(1965),
    [sym_get_serial_number] = ACTIONS(1965),
    [sym_get_master_build] = ACTIONS(1965),
    [sym_set_validation_code] = ACTIONS(1965),
    [sym_reboot] = ACTIONS(1965),
    [sym_load_duet_module] = ACTIONS(1965),
    [sym_unload_duet_module] = ACTIONS(1965),
    [sym_duet_mem_size_set] = ACTIONS(1965),
    [sym_duet_mem_size_get] = ACTIONS(1965),
    [sym_dynamic_application_device] = ACTIONS(1965),
    [sym_dynamic_polled_port] = ACTIONS(1965),
    [sym_static_port_binding] = ACTIONS(1965),
    [sym_static_ip_binding] = ACTIONS(1965),
    [sym_get_dns_list] = ACTIONS(1965),
    [sym_set_dns_list] = ACTIONS(1965),
    [sym_get_ip_address] = ACTIONS(1965),
    [sym_set_ip_address] = ACTIONS(1965),
    [sym_get_url_list] = ACTIONS(1965),
    [sym_add_url_entry] = ACTIONS(1965),
    [sym_delete_url_entry] = ACTIONS(1965),
    [sym_abs_value] = ACTIONS(1965),
    [sym_max_value] = ACTIONS(1965),
    [sym_min_value] = ACTIONS(1965),
    [sym_exp_value] = ACTIONS(1965),
    [sym_log_value] = ACTIONS(1965),
    [sym_log10_value] = ACTIONS(1965),
    [sym_power_value] = ACTIONS(1965),
    [sym_sqrt_value] = ACTIONS(1965),
    [sym_raw_be] = ACTIONS(1965),
    [sym_raw_le] = ACTIONS(1965),
    [sym_internal_queue_size_set] = ACTIONS(1965),
    [sym_internal_queue_size_get] = ACTIONS(1965),
    [sym_internal_threshold_set] = ACTIONS(1965),
    [sym_internal_threshold_get] = ACTIONS(1965),
    [sym_rebuilt_event] = ACTIONS(1965),
    [sym_do_custom_event] = ACTIONS(1965),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1965),
    [sym_clkmgr_set_clk_source] = ACTIONS(1965),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1965),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1965),
    [sym_clkmgr_get_timezone] = ACTIONS(1965),
    [sym_clkmgr_set_timezone] = ACTIONS(1965),
    [sym_clkmgr_get_resync_period] = ACTIONS(1965),
    [sym_clkmgr_set_resync_period] = ACTIONS(1965),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1965),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1965),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1965),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1965),
    [sym_clkmgr_get_timeservers] = ACTIONS(1965),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1965),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1965),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1965),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1965),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1965),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1965),
    [sym_set_log_level] = ACTIONS(1965),
    [sym_get_log_level] = ACTIONS(1965),
    [sym_amx_log] = ACTIONS(1965),
    [sym_smtp_server_config_set] = ACTIONS(1965),
    [sym_smtp_server_config_get] = ACTIONS(1965),
    [sym_smtp_send] = ACTIONS(1965),
    [sym_validate_netlinx_account] = ACTIONS(1965),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1965),
    [sym_authenticate_certificate] = ACTIONS(1965),
    [sym_audit_netlinx_session_event] = ACTIONS(1965),
    [sym_audit_netlinx_generic_event] = ACTIONS(1965),
    [sym_get_available_flash_disk_space] = ACTIONS(1965),
    [sym_get_max_flash_disk_space] = ACTIONS(1965),
    [sym_data_create_feed] = ACTIONS(1965),
    [sym_data_delete_feed] = ACTIONS(1965),
    [sym_data_publish_feed] = ACTIONS(1965),
    [sym_data_get_published_feed] = ACTIONS(1965),
    [sym_data_add_record] = ACTIONS(1965),
    [sym_data_get_event_record] = ACTIONS(1965),
    [sym__wc_data_create_feed] = ACTIONS(1965),
    [sym__wc_data_add_record] = ACTIONS(1965),
    [sym__wc_data_get_event_record] = ACTIONS(1965),
    [sym_format] = ACTIONS(1965),
    [sym_astro_clock] = ACTIONS(1965),
    [sym_set_outdoor_temperature] = ACTIONS(1965),
    [sym_set_virtual_level_count] = ACTIONS(1965),
    [sym_set_virtual_channel_count] = ACTIONS(1965),
    [sym_set_virtual_port_count] = ACTIONS(1965),
    [sym__wc_encode_int] = ACTIONS(1965),
    [sym__wc_decode_int] = ACTIONS(1965),
    [sym__wc_int] = ACTIONS(1965),
    [sym__wc_to_ch_int] = ACTIONS(1965),
    [sym__ch_to_wc_int] = ACTIONS(1965),
    [sym__wc_find_string_int] = ACTIONS(1965),
    [sym__wc_left_string_int] = ACTIONS(1965),
    [sym__wc_lower_string_int] = ACTIONS(1965),
    [sym__wc_mid_string_int] = ACTIONS(1965),
    [sym__wc_remove_string_int] = ACTIONS(1965),
    [sym__wc_right_string_int] = ACTIONS(1965),
    [sym__wc_upper_string_int] = ACTIONS(1965),
    [sym__wc_compare_string_int] = ACTIONS(1965),
    [sym__wc_get_buffer_char_int] = ACTIONS(1965),
    [sym__wc_get_buffer_string_int] = ACTIONS(1965),
    [sym__wc_concat_string_int] = ACTIONS(1965),
    [sym__wc_file_open_int] = ACTIONS(1965),
    [sym__wc_file_close_int] = ACTIONS(1965),
    [sym__wc_file_read_int] = ACTIONS(1965),
    [sym__wc_file_read_line_int] = ACTIONS(1965),
    [sym__wc_file_write_int] = ACTIONS(1965),
    [sym__wc_file_write_line_int] = ACTIONS(1965),
    [aux_sym_preproc_include_token1] = ACTIONS(1965),
    [aux_sym_preproc_define_token1] = ACTIONS(1967),
    [aux_sym_preproc_warn_token1] = ACTIONS(1967),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1967),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1967),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1967),
    [anon_sym_LBRACK] = ACTIONS(1967),
    [anon_sym_LBRACE] = ACTIONS(1967),
    [anon_sym_LPAREN] = ACTIONS(1965),
    [anon_sym_BANG] = ACTIONS(1967),
    [anon_sym_TILDE] = ACTIONS(1967),
    [anon_sym_DASH] = ACTIONS(1965),
    [anon_sym_PLUS] = ACTIONS(1965),
    [anon_sym_DASH_DASH] = ACTIONS(1967),
    [anon_sym_PLUS_PLUS] = ACTIONS(1967),
    [anon_sym_DQUOTE] = ACTIONS(1967),
    [anon_sym_SQUOTE] = ACTIONS(1967),
    [sym_number_literal] = ACTIONS(1967),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(429)] = {
    [ts_builtin_sym_end] = ACTIONS(2079),
    [sym_identifier] = ACTIONS(2077),
    [sym_program_name_keyword] = ACTIONS(2077),
    [sym_module_name_keyword] = ACTIONS(2077),
    [sym_define_device_keyword] = ACTIONS(2077),
    [sym_define_combine_keyword] = ACTIONS(2077),
    [sym_define_constant_keyword] = ACTIONS(2077),
    [sym_define_type_keyword] = ACTIONS(2077),
    [sym_define_variable_keyword] = ACTIONS(2077),
    [sym_define_system_variable_keyword] = ACTIONS(2077),
    [sym_define_connect_level_keyword] = ACTIONS(2077),
    [sym_define_latching_keyword] = ACTIONS(2077),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2077),
    [sym_define_toggling_keyword] = ACTIONS(2077),
    [sym_define_call_keyword] = ACTIONS(2077),
    [sym_define_function_keyword] = ACTIONS(2077),
    [sym_define_library_function_keyword] = ACTIONS(2077),
    [sym_define_start_keyword] = ACTIONS(2077),
    [sym_define_event_keyword] = ACTIONS(2077),
    [sym_define_program_keyword] = ACTIONS(2077),
    [sym_define_module_keyword] = ACTIONS(2077),
    [sym_char_keyword] = ACTIONS(2077),
    [sym_widechar_keyword] = ACTIONS(2077),
    [sym_integer_keyword] = ACTIONS(2077),
    [sym_sinteger_keyword] = ACTIONS(2077),
    [sym_long_keyword] = ACTIONS(2077),
    [sym_slong_keyword] = ACTIONS(2077),
    [sym_float_keyword] = ACTIONS(2077),
    [sym_double_keyword] = ACTIONS(2077),
    [sym_variant_keyword] = ACTIONS(2077),
    [sym_variantarray_keyword] = ACTIONS(2077),
    [sym_local_var_keyword] = ACTIONS(2077),
    [sym_stack_var_keyword] = ACTIONS(2077),
    [sym_constant_keyword] = ACTIONS(2077),
    [sym_volatile_keyword] = ACTIONS(2077),
    [sym_non_volatile_keyword] = ACTIONS(2077),
    [sym_persistent_keyword] = ACTIONS(2077),
    [sym_dev_keyword] = ACTIONS(2077),
    [sym_devlev_keyword] = ACTIONS(2077),
    [sym_devchan_keyword] = ACTIONS(2077),
    [sym_if_keyword] = ACTIONS(2077),
    [sym_switch_keyword] = ACTIONS(2077),
    [sym_case_keyword] = ACTIONS(2077),
    [sym_default_keyword] = ACTIONS(2077),
    [sym_while_keyword] = ACTIONS(2077),
    [sym_for_keyword] = ACTIONS(2077),
    [sym_break_keyword] = ACTIONS(2077),
    [sym_continue_keyword] = ACTIONS(2077),
    [sym_return_keyword] = ACTIONS(2077),
    [sym_select_keyword] = ACTIONS(2077),
    [sym_active_keyword] = ACTIONS(2077),
    [sym_struct_keyword] = ACTIONS(2077),
    [sym_structure_keyword] = ACTIONS(2077),
    [sym_button_event_keyword] = ACTIONS(2077),
    [sym_channel_event_keyword] = ACTIONS(2077),
    [sym_level_event_keyword] = ACTIONS(2077),
    [sym_data_event_keyword] = ACTIONS(2077),
    [sym_timeline_event_keyword] = ACTIONS(2077),
    [sym_custom_event_keyword] = ACTIONS(2077),
    [sym_push_keyword] = ACTIONS(2077),
    [sym_release_keyword] = ACTIONS(2077),
    [aux_sym_on_keyword_token1] = ACTIONS(2077),
    [aux_sym_off_keyword_token1] = ACTIONS(2077),
    [sym_send_level_keyword] = ACTIONS(2077),
    [sym_send_string_keyword] = ACTIONS(2077),
    [sym_send_command_keyword] = ACTIONS(2077),
    [sym_clear_buffer_keyword] = ACTIONS(2077),
    [sym_create_buffer_keyword] = ACTIONS(2077),
    [sym_create_multi_buffer_keyword] = ACTIONS(2077),
    [sym_call_keyword] = ACTIONS(2077),
    [sym_system_call_keyword] = ACTIONS(2077),
    [sym_devchan_to_keyword] = ACTIONS(2077),
    [sym_devchan_min_to_keyword] = ACTIONS(2077),
    [sym_devchan_total_off_keyword] = ACTIONS(2077),
    [sym_devchan_pulse_keyword] = ACTIONS(2077),
    [sym_wait_keyword] = ACTIONS(2077),
    [sym_cancel_wait_keyword] = ACTIONS(2077),
    [sym_wait_until_keyword] = ACTIONS(2077),
    [sym_cancel_wait_until_keyword] = ACTIONS(2077),
    [sym_cancel_all_wait_keyword] = ACTIONS(2077),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2077),
    [sym_not] = ACTIONS(2077),
    [sym_bnot] = ACTIONS(2077),
    [sym___file__] = ACTIONS(2077),
    [sym___date__] = ACTIONS(2077),
    [sym___ldate__] = ACTIONS(2077),
    [sym___time__] = ACTIONS(2077),
    [sym___line__] = ACTIONS(2077),
    [sym___name__] = ACTIONS(2077),
    [sym___netlinx__] = ACTIONS(2077),
    [sym_netlinx_axi_version] = ACTIONS(2077),
    [sym_url_flg_tcp] = ACTIONS(2077),
    [sym_url_flg_accinfopresent] = ACTIONS(2077),
    [sym_url_flg_temp] = ACTIONS(2077),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2077),
    [sym_url_flg_stat_mask] = ACTIONS(2077),
    [sym_url_flg_stat_lookup] = ACTIONS(2077),
    [sym_url_flg_stat_connecting] = ACTIONS(2077),
    [sym_url_flg_stat_waiting] = ACTIONS(2077),
    [sym_url_flg_stat_connected] = ACTIONS(2077),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2077),
    [sym_first_virtual_device] = ACTIONS(2077),
    [sym_first_local_port] = ACTIONS(2077),
    [sym_dynamic_virtual_device] = ACTIONS(2077),
    [sym_timeline_once] = ACTIONS(2077),
    [sym_timeline_repeat] = ACTIONS(2077),
    [sym_timeline_absolute] = ACTIONS(2077),
    [sym_timeline_relative] = ACTIONS(2077),
    [sym_do_push_timed_infinite] = ACTIONS(2077),
    [sym_source_type_no_address] = ACTIONS(2077),
    [sym_source_type_neuron_id] = ACTIONS(2077),
    [sym_source_type_ip_address] = ACTIONS(2077),
    [sym_source_type_axlink] = ACTIONS(2077),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2077),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2077),
    [sym_source_type_ip_socket_address] = ACTIONS(2077),
    [sym_source_type_rs232] = ACTIONS(2077),
    [sym_source_type_internal] = ACTIONS(2077),
    [sym_source_type_mac_address] = ACTIONS(2077),
    [sym_source_type_ipv4_port] = ACTIONS(2077),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2077),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2077),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2077),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2077),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2077),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2077),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2077),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2077),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2077),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2077),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2077),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2077),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2077),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2077),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2077),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2077),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2077),
    [sym_true] = ACTIONS(2077),
    [sym_false] = ACTIONS(2077),
    [sym_file_read_only] = ACTIONS(2077),
    [sym_file_rw_new] = ACTIONS(2077),
    [sym_file_rw_append] = ACTIONS(2077),
    [sym_ip_tcp] = ACTIONS(2077),
    [sym_ip_udp] = ACTIONS(2077),
    [sym_ip_udp_2way] = ACTIONS(2077),
    [sym_xml_encode_types] = ACTIONS(2077),
    [sym_xml_encode_char_as_list] = ACTIONS(2077),
    [sym_xml_encode_le] = ACTIONS(2077),
    [sym_xml_decode_types] = ACTIONS(2077),
    [sym_xml_decode_no_preserve] = ACTIONS(2077),
    [sym_duet_dev_type_display_device] = ACTIONS(2077),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2077),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2077),
    [sym_duet_dev_type_io_device] = ACTIONS(2077),
    [sym_duet_dev_type_relay_device] = ACTIONS(2077),
    [sym_duet_dev_type_ups] = ACTIONS(2077),
    [sym_duet_dev_type_amplifier] = ACTIONS(2077),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2077),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2077),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2077),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2077),
    [sym_duet_dev_type_camera] = ACTIONS(2077),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2077),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2077),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2077),
    [sym_duet_dev_type_dss] = ACTIONS(2077),
    [sym_duet_dev_type_dvr] = ACTIONS(2077),
    [sym_duet_dev_type_disc_device] = ACTIONS(2077),
    [sym_duet_dev_type_document_camera] = ACTIONS(2077),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2077),
    [sym_duet_dev_type_hvac] = ACTIONS(2077),
    [sym_duet_dev_type_keypad] = ACTIONS(2077),
    [sym_duet_dev_type_light] = ACTIONS(2077),
    [sym_duet_dev_type_monitor] = ACTIONS(2077),
    [sym_duet_dev_type_motor] = ACTIONS(2077),
    [sym_duet_dev_type_multi_window] = ACTIONS(2077),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2077),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2077),
    [sym_duet_dev_type_receiver] = ACTIONS(2077),
    [sym_duet_dev_type_security_system] = ACTIONS(2077),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2077),
    [sym_duet_dev_type_settop_box] = ACTIONS(2077),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2077),
    [sym_duet_dev_type_switcher] = ACTIONS(2077),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2077),
    [sym_duet_dev_type_tv] = ACTIONS(2077),
    [sym_duet_dev_type_utility] = ACTIONS(2077),
    [sym_duet_dev_type_vcr] = ACTIONS(2077),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2077),
    [sym_duet_dev_type_video_processor] = ACTIONS(2077),
    [sym_duet_dev_type_video_projector] = ACTIONS(2077),
    [sym_duet_dev_type_video_wall] = ACTIONS(2077),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2077),
    [sym_duet_dev_type_weather] = ACTIONS(2077),
    [sym_duet_dev_not_polled] = ACTIONS(2077),
    [sym_duet_dev_polled] = ACTIONS(2077),
    [sym_clkmgr_mode_network] = ACTIONS(2077),
    [sym_clkmgr_mode_standalone] = ACTIONS(2077),
    [sym_normal_standby] = ACTIONS(2077),
    [sym_normal_wake] = ACTIONS(2077),
    [sym_ip_multicast_ttl_option] = ACTIONS(2077),
    [sym_ip_ttl_subnet] = ACTIONS(2077),
    [sym_ip_ttl_site] = ACTIONS(2077),
    [sym_ip_ttl_region] = ACTIONS(2077),
    [sym_ip_ttl_continent] = ACTIONS(2077),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2077),
    [sym_ip_nodelay_on] = ACTIONS(2077),
    [sym_ip_nodelay_off] = ACTIONS(2077),
    [sym_amx_error] = ACTIONS(2077),
    [sym_amx_warning] = ACTIONS(2077),
    [sym_amx_info] = ACTIONS(2077),
    [sym_amx_debug] = ACTIONS(2077),
    [sym_netlinx_login_success] = ACTIONS(2077),
    [sym_netlinx_login_fail] = ACTIONS(2077),
    [sym_netlinx_logout] = ACTIONS(2077),
    [sym_null_str] = ACTIONS(2077),
    [sym_smtp_address] = ACTIONS(2077),
    [sym_smtp_port_number] = ACTIONS(2077),
    [sym_smtp_username] = ACTIONS(2077),
    [sym_smtp_password] = ACTIONS(2077),
    [sym_smtp_require_tls] = ACTIONS(2077),
    [sym_smtp_from] = ACTIONS(2077),
    [sym_smtp_tls_true] = ACTIONS(2077),
    [sym_smtp_tls_false] = ACTIONS(2077),
    [sym_listview_on_row_select_event] = ACTIONS(2077),
    [sym_data_structure_unknown] = ACTIONS(2077),
    [sym_data_structure_datafeed] = ACTIONS(2077),
    [sym_data_structure_datarecordset] = ACTIONS(2077),
    [sym_data_structure_datarecord] = ACTIONS(2077),
    [sym_data_structure_datafield] = ACTIONS(2077),
    [sym_data_max_content_field_count] = ACTIONS(2077),
    [sym_data_max_metadata_field_count] = ACTIONS(2077),
    [sym_data_max_id_length] = ACTIONS(2077),
    [sym_data_max_name_length] = ACTIONS(2077),
    [sym_data_max_description_length] = ACTIONS(2077),
    [sym_data_max_type_length] = ACTIONS(2077),
    [sym_data_max_format_length] = ACTIONS(2077),
    [sym_data_max_source_length] = ACTIONS(2077),
    [sym_data_max_value_length] = ACTIONS(2077),
    [sym_data_max_label_length] = ACTIONS(2077),
    [sym_data_max_url_length] = ACTIONS(2077),
    [sym_max_last_login_info_length] = ACTIONS(2077),
    [sym_data_type_unknown] = ACTIONS(2077),
    [sym_data_type_string] = ACTIONS(2077),
    [sym_data_type_datetime] = ACTIONS(2077),
    [sym_data_type_date] = ACTIONS(2077),
    [sym_data_type_time] = ACTIONS(2077),
    [sym_data_type_image] = ACTIONS(2077),
    [sym_data_format_url] = ACTIONS(2077),
    [sym_data_format_phone] = ACTIONS(2077),
    [sym_data_format_email] = ACTIONS(2077),
    [sym_data_format_iso8601] = ACTIONS(2077),
    [sym_auditlog_priv] = ACTIONS(2077),
    [sym_configuration_priv] = ACTIONS(2077),
    [sym_deviceconfig_priv] = ACTIONS(2077),
    [sym_ftp_priv] = ACTIONS(2077),
    [sym_http_priv] = ACTIONS(2077),
    [sym_networkconfig_priv] = ACTIONS(2077),
    [sym_programport_priv] = ACTIONS(2077),
    [sym_remoteui_priv] = ACTIONS(2077),
    [sym_securitycontrol_priv] = ACTIONS(2077),
    [sym_softwaremanagement_priv] = ACTIONS(2077),
    [sym_terminal_priv] = ACTIONS(2077),
    [sym_tpadmin_priv] = ACTIONS(2077),
    [sym_user1_priv] = ACTIONS(2077),
    [sym_user2_priv] = ACTIONS(2077),
    [sym_user3_priv] = ACTIONS(2077),
    [sym_user4_priv] = ACTIONS(2077),
    [sym_usermanagement_priv] = ACTIONS(2077),
    [sym_valid_account] = ACTIONS(2077),
    [sym_err_name_invalid] = ACTIONS(2077),
    [sym_err_password_invalid] = ACTIONS(2077),
    [sym_err_invalid_account] = ACTIONS(2077),
    [sym_err_no_authorization] = ACTIONS(2077),
    [sym_err_invalid_parameter] = ACTIONS(2077),
    [sym_err_user_locked_out] = ACTIONS(2077),
    [sym_err_user_account_expired] = ACTIONS(2077),
    [sym_err_general_error] = ACTIONS(2077),
    [sym_err_authentication_error] = ACTIONS(2077),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2077),
    [sym_tls_validate_certificate] = ACTIONS(2077),
    [sym_tdata] = ACTIONS(2077),
    [sym_tchannel] = ACTIONS(2077),
    [sym_tlevel] = ACTIONS(2077),
    [sym_tbutton] = ACTIONS(2077),
    [sym_ttimeline] = ACTIONS(2077),
    [sym_tcustom] = ACTIONS(2077),
    [sym_url_struct] = ACTIONS(2077),
    [sym_dns_struct] = ACTIONS(2077),
    [sym_ip_address_struct] = ACTIONS(2077),
    [sym_dev_info_struct] = ACTIONS(2077),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2077),
    [sym_data_feed] = ACTIONS(2077),
    [sym_data_field] = ACTIONS(2077),
    [sym_data_record] = ACTIONS(2077),
    [sym_wc_data_feed] = ACTIONS(2077),
    [sym_wc_data_field] = ACTIONS(2077),
    [sym_wc_data_record] = ACTIONS(2077),
    [sym_last_login_info] = ACTIONS(2077),
    [sym____reserved___] = ACTIONS(2077),
    [sym_date] = ACTIONS(2077),
    [sym_day] = ACTIONS(2077),
    [sym_time] = ACTIONS(2077),
    [sym_ldate] = ACTIONS(2077),
    [sym_push_device] = ACTIONS(2077),
    [sym_push_channel] = ACTIONS(2077),
    [sym_push_devchan] = ACTIONS(2077),
    [sym_release_device] = ACTIONS(2077),
    [sym_release_channel] = ACTIONS(2077),
    [sym_release_devchan] = ACTIONS(2077),
    [sym_master_slot] = ACTIONS(2077),
    [sym_get_pulse_time] = ACTIONS(2077),
    [sym_get_timer] = ACTIONS(2077),
    [sym_master_sn] = ACTIONS(2077),
    [sym_system_number] = ACTIONS(2077),
    [sym_button] = ACTIONS(2077),
    [sym_dv_channel] = ACTIONS(2077),
    [sym_data] = ACTIONS(2077),
    [sym_level] = ACTIONS(2077),
    [sym_timeline] = ACTIONS(2077),
    [sym_channel] = ACTIONS(2077),
    [sym_custom] = ACTIONS(2077),
    [sym_atoi] = ACTIONS(2077),
    [sym_atol] = ACTIONS(2077),
    [sym_atof] = ACTIONS(2077),
    [sym_itoa] = ACTIONS(2077),
    [sym_itohex] = ACTIONS(2077),
    [sym_hextoi] = ACTIONS(2077),
    [sym_ftoa] = ACTIONS(2077),
    [sym_find_string] = ACTIONS(2077),
    [sym_left_string] = ACTIONS(2077),
    [sym_right_string] = ACTIONS(2077),
    [sym_mid_string] = ACTIONS(2077),
    [sym_remove_string] = ACTIONS(2077),
    [sym_get_buffer_string] = ACTIONS(2077),
    [sym_get_buffer_char] = ACTIONS(2077),
    [sym_get_multi_buffer_string] = ACTIONS(2077),
    [sym_length_string] = ACTIONS(2077),
    [sym_max_length_string] = ACTIONS(2077),
    [sym_set_length_string] = ACTIONS(2077),
    [sym_compare_string] = ACTIONS(2077),
    [sym_upper_string] = ACTIONS(2077),
    [sym_lower_string] = ACTIONS(2077),
    [sym_get_last] = ACTIONS(2077),
    [sym_date_to_day] = ACTIONS(2077),
    [sym_date_to_month] = ACTIONS(2077),
    [sym_date_to_year] = ACTIONS(2077),
    [sym_time_to_hour] = ACTIONS(2077),
    [sym_time_to_minute] = ACTIONS(2077),
    [sym_time_to_second] = ACTIONS(2077),
    [sym_day_of_week] = ACTIONS(2077),
    [sym_set_pulse_time] = ACTIONS(2077),
    [sym_set_timer] = ACTIONS(2077),
    [sym_file_open] = ACTIONS(2077),
    [sym_file_close] = ACTIONS(2077),
    [sym_file_read] = ACTIONS(2077),
    [sym_file_read_line] = ACTIONS(2077),
    [sym_file_write] = ACTIONS(2077),
    [sym_file_write_line] = ACTIONS(2077),
    [sym_file_copy] = ACTIONS(2077),
    [sym_file_delete] = ACTIONS(2077),
    [sym_file_rename] = ACTIONS(2077),
    [sym_file_dir] = ACTIONS(2077),
    [sym_file_getdir] = ACTIONS(2077),
    [sym_file_setdir] = ACTIONS(2077),
    [sym_file_create_dir] = ACTIONS(2077),
    [sym_file_remove_dir] = ACTIONS(2077),
    [sym_file_seek] = ACTIONS(2077),
    [sym_length_array] = ACTIONS(2077),
    [sym_max_length_array] = ACTIONS(2077),
    [sym_set_length_array] = ACTIONS(2077),
    [sym_ip_client_open] = ACTIONS(2077),
    [sym_ip_client_close] = ACTIONS(2077),
    [sym_ip_server_open] = ACTIONS(2077),
    [sym_ip_server_close] = ACTIONS(2077),
    [sym_ip_bound_client_open] = ACTIONS(2077),
    [sym_ip_mc_server_open] = ACTIONS(2077),
    [sym_ip_set_option] = ACTIONS(2077),
    [sym_ssh_client_open] = ACTIONS(2077),
    [sym_ssh_client_close] = ACTIONS(2077),
    [sym_tls_client_open] = ACTIONS(2077),
    [sym_tls_client_close] = ACTIONS(2077),
    [sym_do_push] = ACTIONS(2077),
    [sym_do_push_timed] = ACTIONS(2077),
    [sym_do_release] = ACTIONS(2077),
    [sym_device_id] = ACTIONS(2077),
    [sym_device_info] = ACTIONS(2077),
    [sym_device_id_string] = ACTIONS(2077),
    [sym_device_standby] = ACTIONS(2077),
    [sym_device_wake] = ACTIONS(2077),
    [sym_timeline_create] = ACTIONS(2077),
    [sym_timeline_kill] = ACTIONS(2077),
    [sym_timeline_reload] = ACTIONS(2077),
    [sym_timeline_pause] = ACTIONS(2077),
    [sym_timeline_restart] = ACTIONS(2077),
    [sym_timeline_set] = ACTIONS(2077),
    [sym_timeline_get] = ACTIONS(2077),
    [sym_timeline_active] = ACTIONS(2077),
    [sym_timeline_dynamic_id] = ACTIONS(2077),
    [sym_variable_to_string] = ACTIONS(2077),
    [sym_string_to_variable] = ACTIONS(2077),
    [sym_length_variable_to_string] = ACTIONS(2077),
    [sym_type_cast] = ACTIONS(2077),
    [sym_variable_to_xml] = ACTIONS(2077),
    [sym_xml_to_variable] = ACTIONS(2077),
    [sym_length_variable_to_xml] = ACTIONS(2077),
    [sym_redirect_string] = ACTIONS(2077),
    [sym_random_number] = ACTIONS(2077),
    [sym_get_unique_id] = ACTIONS(2077),
    [sym_get_system_number] = ACTIONS(2077),
    [sym_set_system_number] = ACTIONS(2077),
    [sym_get_serial_number] = ACTIONS(2077),
    [sym_get_master_build] = ACTIONS(2077),
    [sym_set_validation_code] = ACTIONS(2077),
    [sym_reboot] = ACTIONS(2077),
    [sym_load_duet_module] = ACTIONS(2077),
    [sym_unload_duet_module] = ACTIONS(2077),
    [sym_duet_mem_size_set] = ACTIONS(2077),
    [sym_duet_mem_size_get] = ACTIONS(2077),
    [sym_dynamic_application_device] = ACTIONS(2077),
    [sym_dynamic_polled_port] = ACTIONS(2077),
    [sym_static_port_binding] = ACTIONS(2077),
    [sym_static_ip_binding] = ACTIONS(2077),
    [sym_get_dns_list] = ACTIONS(2077),
    [sym_set_dns_list] = ACTIONS(2077),
    [sym_get_ip_address] = ACTIONS(2077),
    [sym_set_ip_address] = ACTIONS(2077),
    [sym_get_url_list] = ACTIONS(2077),
    [sym_add_url_entry] = ACTIONS(2077),
    [sym_delete_url_entry] = ACTIONS(2077),
    [sym_abs_value] = ACTIONS(2077),
    [sym_max_value] = ACTIONS(2077),
    [sym_min_value] = ACTIONS(2077),
    [sym_exp_value] = ACTIONS(2077),
    [sym_log_value] = ACTIONS(2077),
    [sym_log10_value] = ACTIONS(2077),
    [sym_power_value] = ACTIONS(2077),
    [sym_sqrt_value] = ACTIONS(2077),
    [sym_raw_be] = ACTIONS(2077),
    [sym_raw_le] = ACTIONS(2077),
    [sym_internal_queue_size_set] = ACTIONS(2077),
    [sym_internal_queue_size_get] = ACTIONS(2077),
    [sym_internal_threshold_set] = ACTIONS(2077),
    [sym_internal_threshold_get] = ACTIONS(2077),
    [sym_rebuilt_event] = ACTIONS(2077),
    [sym_do_custom_event] = ACTIONS(2077),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2077),
    [sym_clkmgr_set_clk_source] = ACTIONS(2077),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2077),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2077),
    [sym_clkmgr_get_timezone] = ACTIONS(2077),
    [sym_clkmgr_set_timezone] = ACTIONS(2077),
    [sym_clkmgr_get_resync_period] = ACTIONS(2077),
    [sym_clkmgr_set_resync_period] = ACTIONS(2077),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2077),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2077),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2077),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2077),
    [sym_clkmgr_get_timeservers] = ACTIONS(2077),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2077),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2077),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2077),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2077),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2077),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2077),
    [sym_set_log_level] = ACTIONS(2077),
    [sym_get_log_level] = ACTIONS(2077),
    [sym_amx_log] = ACTIONS(2077),
    [sym_smtp_server_config_set] = ACTIONS(2077),
    [sym_smtp_server_config_get] = ACTIONS(2077),
    [sym_smtp_send] = ACTIONS(2077),
    [sym_validate_netlinx_account] = ACTIONS(2077),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2077),
    [sym_authenticate_certificate] = ACTIONS(2077),
    [sym_audit_netlinx_session_event] = ACTIONS(2077),
    [sym_audit_netlinx_generic_event] = ACTIONS(2077),
    [sym_get_available_flash_disk_space] = ACTIONS(2077),
    [sym_get_max_flash_disk_space] = ACTIONS(2077),
    [sym_data_create_feed] = ACTIONS(2077),
    [sym_data_delete_feed] = ACTIONS(2077),
    [sym_data_publish_feed] = ACTIONS(2077),
    [sym_data_get_published_feed] = ACTIONS(2077),
    [sym_data_add_record] = ACTIONS(2077),
    [sym_data_get_event_record] = ACTIONS(2077),
    [sym__wc_data_create_feed] = ACTIONS(2077),
    [sym__wc_data_add_record] = ACTIONS(2077),
    [sym__wc_data_get_event_record] = ACTIONS(2077),
    [sym_format] = ACTIONS(2077),
    [sym_astro_clock] = ACTIONS(2077),
    [sym_set_outdoor_temperature] = ACTIONS(2077),
    [sym_set_virtual_level_count] = ACTIONS(2077),
    [sym_set_virtual_channel_count] = ACTIONS(2077),
    [sym_set_virtual_port_count] = ACTIONS(2077),
    [sym__wc_encode_int] = ACTIONS(2077),
    [sym__wc_decode_int] = ACTIONS(2077),
    [sym__wc_int] = ACTIONS(2077),
    [sym__wc_to_ch_int] = ACTIONS(2077),
    [sym__ch_to_wc_int] = ACTIONS(2077),
    [sym__wc_find_string_int] = ACTIONS(2077),
    [sym__wc_left_string_int] = ACTIONS(2077),
    [sym__wc_lower_string_int] = ACTIONS(2077),
    [sym__wc_mid_string_int] = ACTIONS(2077),
    [sym__wc_remove_string_int] = ACTIONS(2077),
    [sym__wc_right_string_int] = ACTIONS(2077),
    [sym__wc_upper_string_int] = ACTIONS(2077),
    [sym__wc_compare_string_int] = ACTIONS(2077),
    [sym__wc_get_buffer_char_int] = ACTIONS(2077),
    [sym__wc_get_buffer_string_int] = ACTIONS(2077),
    [sym__wc_concat_string_int] = ACTIONS(2077),
    [sym__wc_file_open_int] = ACTIONS(2077),
    [sym__wc_file_close_int] = ACTIONS(2077),
    [sym__wc_file_read_int] = ACTIONS(2077),
    [sym__wc_file_read_line_int] = ACTIONS(2077),
    [sym__wc_file_write_int] = ACTIONS(2077),
    [sym__wc_file_write_line_int] = ACTIONS(2077),
    [aux_sym_preproc_include_token1] = ACTIONS(2077),
    [aux_sym_preproc_define_token1] = ACTIONS(2079),
    [aux_sym_preproc_warn_token1] = ACTIONS(2079),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2079),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2079),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2079),
    [anon_sym_LBRACK] = ACTIONS(2079),
    [anon_sym_LBRACE] = ACTIONS(2079),
    [anon_sym_LPAREN] = ACTIONS(2077),
    [anon_sym_BANG] = ACTIONS(2079),
    [anon_sym_TILDE] = ACTIONS(2079),
    [anon_sym_DASH] = ACTIONS(2077),
    [anon_sym_PLUS] = ACTIONS(2077),
    [anon_sym_DASH_DASH] = ACTIONS(2079),
    [anon_sym_PLUS_PLUS] = ACTIONS(2079),
    [anon_sym_DQUOTE] = ACTIONS(2079),
    [anon_sym_SQUOTE] = ACTIONS(2079),
    [sym_number_literal] = ACTIONS(2079),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(430)] = {
    [ts_builtin_sym_end] = ACTIONS(2007),
    [sym_identifier] = ACTIONS(2005),
    [sym_program_name_keyword] = ACTIONS(2005),
    [sym_module_name_keyword] = ACTIONS(2005),
    [sym_define_device_keyword] = ACTIONS(2005),
    [sym_define_combine_keyword] = ACTIONS(2005),
    [sym_define_constant_keyword] = ACTIONS(2005),
    [sym_define_type_keyword] = ACTIONS(2005),
    [sym_define_variable_keyword] = ACTIONS(2005),
    [sym_define_system_variable_keyword] = ACTIONS(2005),
    [sym_define_connect_level_keyword] = ACTIONS(2005),
    [sym_define_latching_keyword] = ACTIONS(2005),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2005),
    [sym_define_toggling_keyword] = ACTIONS(2005),
    [sym_define_call_keyword] = ACTIONS(2005),
    [sym_define_function_keyword] = ACTIONS(2005),
    [sym_define_library_function_keyword] = ACTIONS(2005),
    [sym_define_start_keyword] = ACTIONS(2005),
    [sym_define_event_keyword] = ACTIONS(2005),
    [sym_define_program_keyword] = ACTIONS(2005),
    [sym_define_module_keyword] = ACTIONS(2005),
    [sym_char_keyword] = ACTIONS(2005),
    [sym_widechar_keyword] = ACTIONS(2005),
    [sym_integer_keyword] = ACTIONS(2005),
    [sym_sinteger_keyword] = ACTIONS(2005),
    [sym_long_keyword] = ACTIONS(2005),
    [sym_slong_keyword] = ACTIONS(2005),
    [sym_float_keyword] = ACTIONS(2005),
    [sym_double_keyword] = ACTIONS(2005),
    [sym_variant_keyword] = ACTIONS(2005),
    [sym_variantarray_keyword] = ACTIONS(2005),
    [sym_local_var_keyword] = ACTIONS(2005),
    [sym_stack_var_keyword] = ACTIONS(2005),
    [sym_constant_keyword] = ACTIONS(2005),
    [sym_volatile_keyword] = ACTIONS(2005),
    [sym_non_volatile_keyword] = ACTIONS(2005),
    [sym_persistent_keyword] = ACTIONS(2005),
    [sym_dev_keyword] = ACTIONS(2005),
    [sym_devlev_keyword] = ACTIONS(2005),
    [sym_devchan_keyword] = ACTIONS(2005),
    [sym_if_keyword] = ACTIONS(2005),
    [sym_switch_keyword] = ACTIONS(2005),
    [sym_case_keyword] = ACTIONS(2005),
    [sym_default_keyword] = ACTIONS(2005),
    [sym_while_keyword] = ACTIONS(2005),
    [sym_for_keyword] = ACTIONS(2005),
    [sym_break_keyword] = ACTIONS(2005),
    [sym_continue_keyword] = ACTIONS(2005),
    [sym_return_keyword] = ACTIONS(2005),
    [sym_select_keyword] = ACTIONS(2005),
    [sym_active_keyword] = ACTIONS(2005),
    [sym_struct_keyword] = ACTIONS(2005),
    [sym_structure_keyword] = ACTIONS(2005),
    [sym_button_event_keyword] = ACTIONS(2005),
    [sym_channel_event_keyword] = ACTIONS(2005),
    [sym_level_event_keyword] = ACTIONS(2005),
    [sym_data_event_keyword] = ACTIONS(2005),
    [sym_timeline_event_keyword] = ACTIONS(2005),
    [sym_custom_event_keyword] = ACTIONS(2005),
    [sym_push_keyword] = ACTIONS(2005),
    [sym_release_keyword] = ACTIONS(2005),
    [aux_sym_on_keyword_token1] = ACTIONS(2005),
    [aux_sym_off_keyword_token1] = ACTIONS(2005),
    [sym_send_level_keyword] = ACTIONS(2005),
    [sym_send_string_keyword] = ACTIONS(2005),
    [sym_send_command_keyword] = ACTIONS(2005),
    [sym_clear_buffer_keyword] = ACTIONS(2005),
    [sym_create_buffer_keyword] = ACTIONS(2005),
    [sym_create_multi_buffer_keyword] = ACTIONS(2005),
    [sym_call_keyword] = ACTIONS(2005),
    [sym_system_call_keyword] = ACTIONS(2005),
    [sym_devchan_to_keyword] = ACTIONS(2005),
    [sym_devchan_min_to_keyword] = ACTIONS(2005),
    [sym_devchan_total_off_keyword] = ACTIONS(2005),
    [sym_devchan_pulse_keyword] = ACTIONS(2005),
    [sym_wait_keyword] = ACTIONS(2005),
    [sym_cancel_wait_keyword] = ACTIONS(2005),
    [sym_wait_until_keyword] = ACTIONS(2005),
    [sym_cancel_wait_until_keyword] = ACTIONS(2005),
    [sym_cancel_all_wait_keyword] = ACTIONS(2005),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2005),
    [sym_not] = ACTIONS(2005),
    [sym_bnot] = ACTIONS(2005),
    [sym___file__] = ACTIONS(2005),
    [sym___date__] = ACTIONS(2005),
    [sym___ldate__] = ACTIONS(2005),
    [sym___time__] = ACTIONS(2005),
    [sym___line__] = ACTIONS(2005),
    [sym___name__] = ACTIONS(2005),
    [sym___netlinx__] = ACTIONS(2005),
    [sym_netlinx_axi_version] = ACTIONS(2005),
    [sym_url_flg_tcp] = ACTIONS(2005),
    [sym_url_flg_accinfopresent] = ACTIONS(2005),
    [sym_url_flg_temp] = ACTIONS(2005),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2005),
    [sym_url_flg_stat_mask] = ACTIONS(2005),
    [sym_url_flg_stat_lookup] = ACTIONS(2005),
    [sym_url_flg_stat_connecting] = ACTIONS(2005),
    [sym_url_flg_stat_waiting] = ACTIONS(2005),
    [sym_url_flg_stat_connected] = ACTIONS(2005),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2005),
    [sym_first_virtual_device] = ACTIONS(2005),
    [sym_first_local_port] = ACTIONS(2005),
    [sym_dynamic_virtual_device] = ACTIONS(2005),
    [sym_timeline_once] = ACTIONS(2005),
    [sym_timeline_repeat] = ACTIONS(2005),
    [sym_timeline_absolute] = ACTIONS(2005),
    [sym_timeline_relative] = ACTIONS(2005),
    [sym_do_push_timed_infinite] = ACTIONS(2005),
    [sym_source_type_no_address] = ACTIONS(2005),
    [sym_source_type_neuron_id] = ACTIONS(2005),
    [sym_source_type_ip_address] = ACTIONS(2005),
    [sym_source_type_axlink] = ACTIONS(2005),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2005),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2005),
    [sym_source_type_ip_socket_address] = ACTIONS(2005),
    [sym_source_type_rs232] = ACTIONS(2005),
    [sym_source_type_internal] = ACTIONS(2005),
    [sym_source_type_mac_address] = ACTIONS(2005),
    [sym_source_type_ipv4_port] = ACTIONS(2005),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2005),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2005),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2005),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2005),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2005),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2005),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2005),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2005),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2005),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2005),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2005),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2005),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2005),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2005),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2005),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2005),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2005),
    [sym_true] = ACTIONS(2005),
    [sym_false] = ACTIONS(2005),
    [sym_file_read_only] = ACTIONS(2005),
    [sym_file_rw_new] = ACTIONS(2005),
    [sym_file_rw_append] = ACTIONS(2005),
    [sym_ip_tcp] = ACTIONS(2005),
    [sym_ip_udp] = ACTIONS(2005),
    [sym_ip_udp_2way] = ACTIONS(2005),
    [sym_xml_encode_types] = ACTIONS(2005),
    [sym_xml_encode_char_as_list] = ACTIONS(2005),
    [sym_xml_encode_le] = ACTIONS(2005),
    [sym_xml_decode_types] = ACTIONS(2005),
    [sym_xml_decode_no_preserve] = ACTIONS(2005),
    [sym_duet_dev_type_display_device] = ACTIONS(2005),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2005),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2005),
    [sym_duet_dev_type_io_device] = ACTIONS(2005),
    [sym_duet_dev_type_relay_device] = ACTIONS(2005),
    [sym_duet_dev_type_ups] = ACTIONS(2005),
    [sym_duet_dev_type_amplifier] = ACTIONS(2005),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2005),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2005),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2005),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2005),
    [sym_duet_dev_type_camera] = ACTIONS(2005),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2005),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2005),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2005),
    [sym_duet_dev_type_dss] = ACTIONS(2005),
    [sym_duet_dev_type_dvr] = ACTIONS(2005),
    [sym_duet_dev_type_disc_device] = ACTIONS(2005),
    [sym_duet_dev_type_document_camera] = ACTIONS(2005),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2005),
    [sym_duet_dev_type_hvac] = ACTIONS(2005),
    [sym_duet_dev_type_keypad] = ACTIONS(2005),
    [sym_duet_dev_type_light] = ACTIONS(2005),
    [sym_duet_dev_type_monitor] = ACTIONS(2005),
    [sym_duet_dev_type_motor] = ACTIONS(2005),
    [sym_duet_dev_type_multi_window] = ACTIONS(2005),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2005),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2005),
    [sym_duet_dev_type_receiver] = ACTIONS(2005),
    [sym_duet_dev_type_security_system] = ACTIONS(2005),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2005),
    [sym_duet_dev_type_settop_box] = ACTIONS(2005),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2005),
    [sym_duet_dev_type_switcher] = ACTIONS(2005),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2005),
    [sym_duet_dev_type_tv] = ACTIONS(2005),
    [sym_duet_dev_type_utility] = ACTIONS(2005),
    [sym_duet_dev_type_vcr] = ACTIONS(2005),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2005),
    [sym_duet_dev_type_video_processor] = ACTIONS(2005),
    [sym_duet_dev_type_video_projector] = ACTIONS(2005),
    [sym_duet_dev_type_video_wall] = ACTIONS(2005),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2005),
    [sym_duet_dev_type_weather] = ACTIONS(2005),
    [sym_duet_dev_not_polled] = ACTIONS(2005),
    [sym_duet_dev_polled] = ACTIONS(2005),
    [sym_clkmgr_mode_network] = ACTIONS(2005),
    [sym_clkmgr_mode_standalone] = ACTIONS(2005),
    [sym_normal_standby] = ACTIONS(2005),
    [sym_normal_wake] = ACTIONS(2005),
    [sym_ip_multicast_ttl_option] = ACTIONS(2005),
    [sym_ip_ttl_subnet] = ACTIONS(2005),
    [sym_ip_ttl_site] = ACTIONS(2005),
    [sym_ip_ttl_region] = ACTIONS(2005),
    [sym_ip_ttl_continent] = ACTIONS(2005),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2005),
    [sym_ip_nodelay_on] = ACTIONS(2005),
    [sym_ip_nodelay_off] = ACTIONS(2005),
    [sym_amx_error] = ACTIONS(2005),
    [sym_amx_warning] = ACTIONS(2005),
    [sym_amx_info] = ACTIONS(2005),
    [sym_amx_debug] = ACTIONS(2005),
    [sym_netlinx_login_success] = ACTIONS(2005),
    [sym_netlinx_login_fail] = ACTIONS(2005),
    [sym_netlinx_logout] = ACTIONS(2005),
    [sym_null_str] = ACTIONS(2005),
    [sym_smtp_address] = ACTIONS(2005),
    [sym_smtp_port_number] = ACTIONS(2005),
    [sym_smtp_username] = ACTIONS(2005),
    [sym_smtp_password] = ACTIONS(2005),
    [sym_smtp_require_tls] = ACTIONS(2005),
    [sym_smtp_from] = ACTIONS(2005),
    [sym_smtp_tls_true] = ACTIONS(2005),
    [sym_smtp_tls_false] = ACTIONS(2005),
    [sym_listview_on_row_select_event] = ACTIONS(2005),
    [sym_data_structure_unknown] = ACTIONS(2005),
    [sym_data_structure_datafeed] = ACTIONS(2005),
    [sym_data_structure_datarecordset] = ACTIONS(2005),
    [sym_data_structure_datarecord] = ACTIONS(2005),
    [sym_data_structure_datafield] = ACTIONS(2005),
    [sym_data_max_content_field_count] = ACTIONS(2005),
    [sym_data_max_metadata_field_count] = ACTIONS(2005),
    [sym_data_max_id_length] = ACTIONS(2005),
    [sym_data_max_name_length] = ACTIONS(2005),
    [sym_data_max_description_length] = ACTIONS(2005),
    [sym_data_max_type_length] = ACTIONS(2005),
    [sym_data_max_format_length] = ACTIONS(2005),
    [sym_data_max_source_length] = ACTIONS(2005),
    [sym_data_max_value_length] = ACTIONS(2005),
    [sym_data_max_label_length] = ACTIONS(2005),
    [sym_data_max_url_length] = ACTIONS(2005),
    [sym_max_last_login_info_length] = ACTIONS(2005),
    [sym_data_type_unknown] = ACTIONS(2005),
    [sym_data_type_string] = ACTIONS(2005),
    [sym_data_type_datetime] = ACTIONS(2005),
    [sym_data_type_date] = ACTIONS(2005),
    [sym_data_type_time] = ACTIONS(2005),
    [sym_data_type_image] = ACTIONS(2005),
    [sym_data_format_url] = ACTIONS(2005),
    [sym_data_format_phone] = ACTIONS(2005),
    [sym_data_format_email] = ACTIONS(2005),
    [sym_data_format_iso8601] = ACTIONS(2005),
    [sym_auditlog_priv] = ACTIONS(2005),
    [sym_configuration_priv] = ACTIONS(2005),
    [sym_deviceconfig_priv] = ACTIONS(2005),
    [sym_ftp_priv] = ACTIONS(2005),
    [sym_http_priv] = ACTIONS(2005),
    [sym_networkconfig_priv] = ACTIONS(2005),
    [sym_programport_priv] = ACTIONS(2005),
    [sym_remoteui_priv] = ACTIONS(2005),
    [sym_securitycontrol_priv] = ACTIONS(2005),
    [sym_softwaremanagement_priv] = ACTIONS(2005),
    [sym_terminal_priv] = ACTIONS(2005),
    [sym_tpadmin_priv] = ACTIONS(2005),
    [sym_user1_priv] = ACTIONS(2005),
    [sym_user2_priv] = ACTIONS(2005),
    [sym_user3_priv] = ACTIONS(2005),
    [sym_user4_priv] = ACTIONS(2005),
    [sym_usermanagement_priv] = ACTIONS(2005),
    [sym_valid_account] = ACTIONS(2005),
    [sym_err_name_invalid] = ACTIONS(2005),
    [sym_err_password_invalid] = ACTIONS(2005),
    [sym_err_invalid_account] = ACTIONS(2005),
    [sym_err_no_authorization] = ACTIONS(2005),
    [sym_err_invalid_parameter] = ACTIONS(2005),
    [sym_err_user_locked_out] = ACTIONS(2005),
    [sym_err_user_account_expired] = ACTIONS(2005),
    [sym_err_general_error] = ACTIONS(2005),
    [sym_err_authentication_error] = ACTIONS(2005),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2005),
    [sym_tls_validate_certificate] = ACTIONS(2005),
    [sym_tdata] = ACTIONS(2005),
    [sym_tchannel] = ACTIONS(2005),
    [sym_tlevel] = ACTIONS(2005),
    [sym_tbutton] = ACTIONS(2005),
    [sym_ttimeline] = ACTIONS(2005),
    [sym_tcustom] = ACTIONS(2005),
    [sym_url_struct] = ACTIONS(2005),
    [sym_dns_struct] = ACTIONS(2005),
    [sym_ip_address_struct] = ACTIONS(2005),
    [sym_dev_info_struct] = ACTIONS(2005),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2005),
    [sym_data_feed] = ACTIONS(2005),
    [sym_data_field] = ACTIONS(2005),
    [sym_data_record] = ACTIONS(2005),
    [sym_wc_data_feed] = ACTIONS(2005),
    [sym_wc_data_field] = ACTIONS(2005),
    [sym_wc_data_record] = ACTIONS(2005),
    [sym_last_login_info] = ACTIONS(2005),
    [sym____reserved___] = ACTIONS(2005),
    [sym_date] = ACTIONS(2005),
    [sym_day] = ACTIONS(2005),
    [sym_time] = ACTIONS(2005),
    [sym_ldate] = ACTIONS(2005),
    [sym_push_device] = ACTIONS(2005),
    [sym_push_channel] = ACTIONS(2005),
    [sym_push_devchan] = ACTIONS(2005),
    [sym_release_device] = ACTIONS(2005),
    [sym_release_channel] = ACTIONS(2005),
    [sym_release_devchan] = ACTIONS(2005),
    [sym_master_slot] = ACTIONS(2005),
    [sym_get_pulse_time] = ACTIONS(2005),
    [sym_get_timer] = ACTIONS(2005),
    [sym_master_sn] = ACTIONS(2005),
    [sym_system_number] = ACTIONS(2005),
    [sym_button] = ACTIONS(2005),
    [sym_dv_channel] = ACTIONS(2005),
    [sym_data] = ACTIONS(2005),
    [sym_level] = ACTIONS(2005),
    [sym_timeline] = ACTIONS(2005),
    [sym_channel] = ACTIONS(2005),
    [sym_custom] = ACTIONS(2005),
    [sym_atoi] = ACTIONS(2005),
    [sym_atol] = ACTIONS(2005),
    [sym_atof] = ACTIONS(2005),
    [sym_itoa] = ACTIONS(2005),
    [sym_itohex] = ACTIONS(2005),
    [sym_hextoi] = ACTIONS(2005),
    [sym_ftoa] = ACTIONS(2005),
    [sym_find_string] = ACTIONS(2005),
    [sym_left_string] = ACTIONS(2005),
    [sym_right_string] = ACTIONS(2005),
    [sym_mid_string] = ACTIONS(2005),
    [sym_remove_string] = ACTIONS(2005),
    [sym_get_buffer_string] = ACTIONS(2005),
    [sym_get_buffer_char] = ACTIONS(2005),
    [sym_get_multi_buffer_string] = ACTIONS(2005),
    [sym_length_string] = ACTIONS(2005),
    [sym_max_length_string] = ACTIONS(2005),
    [sym_set_length_string] = ACTIONS(2005),
    [sym_compare_string] = ACTIONS(2005),
    [sym_upper_string] = ACTIONS(2005),
    [sym_lower_string] = ACTIONS(2005),
    [sym_get_last] = ACTIONS(2005),
    [sym_date_to_day] = ACTIONS(2005),
    [sym_date_to_month] = ACTIONS(2005),
    [sym_date_to_year] = ACTIONS(2005),
    [sym_time_to_hour] = ACTIONS(2005),
    [sym_time_to_minute] = ACTIONS(2005),
    [sym_time_to_second] = ACTIONS(2005),
    [sym_day_of_week] = ACTIONS(2005),
    [sym_set_pulse_time] = ACTIONS(2005),
    [sym_set_timer] = ACTIONS(2005),
    [sym_file_open] = ACTIONS(2005),
    [sym_file_close] = ACTIONS(2005),
    [sym_file_read] = ACTIONS(2005),
    [sym_file_read_line] = ACTIONS(2005),
    [sym_file_write] = ACTIONS(2005),
    [sym_file_write_line] = ACTIONS(2005),
    [sym_file_copy] = ACTIONS(2005),
    [sym_file_delete] = ACTIONS(2005),
    [sym_file_rename] = ACTIONS(2005),
    [sym_file_dir] = ACTIONS(2005),
    [sym_file_getdir] = ACTIONS(2005),
    [sym_file_setdir] = ACTIONS(2005),
    [sym_file_create_dir] = ACTIONS(2005),
    [sym_file_remove_dir] = ACTIONS(2005),
    [sym_file_seek] = ACTIONS(2005),
    [sym_length_array] = ACTIONS(2005),
    [sym_max_length_array] = ACTIONS(2005),
    [sym_set_length_array] = ACTIONS(2005),
    [sym_ip_client_open] = ACTIONS(2005),
    [sym_ip_client_close] = ACTIONS(2005),
    [sym_ip_server_open] = ACTIONS(2005),
    [sym_ip_server_close] = ACTIONS(2005),
    [sym_ip_bound_client_open] = ACTIONS(2005),
    [sym_ip_mc_server_open] = ACTIONS(2005),
    [sym_ip_set_option] = ACTIONS(2005),
    [sym_ssh_client_open] = ACTIONS(2005),
    [sym_ssh_client_close] = ACTIONS(2005),
    [sym_tls_client_open] = ACTIONS(2005),
    [sym_tls_client_close] = ACTIONS(2005),
    [sym_do_push] = ACTIONS(2005),
    [sym_do_push_timed] = ACTIONS(2005),
    [sym_do_release] = ACTIONS(2005),
    [sym_device_id] = ACTIONS(2005),
    [sym_device_info] = ACTIONS(2005),
    [sym_device_id_string] = ACTIONS(2005),
    [sym_device_standby] = ACTIONS(2005),
    [sym_device_wake] = ACTIONS(2005),
    [sym_timeline_create] = ACTIONS(2005),
    [sym_timeline_kill] = ACTIONS(2005),
    [sym_timeline_reload] = ACTIONS(2005),
    [sym_timeline_pause] = ACTIONS(2005),
    [sym_timeline_restart] = ACTIONS(2005),
    [sym_timeline_set] = ACTIONS(2005),
    [sym_timeline_get] = ACTIONS(2005),
    [sym_timeline_active] = ACTIONS(2005),
    [sym_timeline_dynamic_id] = ACTIONS(2005),
    [sym_variable_to_string] = ACTIONS(2005),
    [sym_string_to_variable] = ACTIONS(2005),
    [sym_length_variable_to_string] = ACTIONS(2005),
    [sym_type_cast] = ACTIONS(2005),
    [sym_variable_to_xml] = ACTIONS(2005),
    [sym_xml_to_variable] = ACTIONS(2005),
    [sym_length_variable_to_xml] = ACTIONS(2005),
    [sym_redirect_string] = ACTIONS(2005),
    [sym_random_number] = ACTIONS(2005),
    [sym_get_unique_id] = ACTIONS(2005),
    [sym_get_system_number] = ACTIONS(2005),
    [sym_set_system_number] = ACTIONS(2005),
    [sym_get_serial_number] = ACTIONS(2005),
    [sym_get_master_build] = ACTIONS(2005),
    [sym_set_validation_code] = ACTIONS(2005),
    [sym_reboot] = ACTIONS(2005),
    [sym_load_duet_module] = ACTIONS(2005),
    [sym_unload_duet_module] = ACTIONS(2005),
    [sym_duet_mem_size_set] = ACTIONS(2005),
    [sym_duet_mem_size_get] = ACTIONS(2005),
    [sym_dynamic_application_device] = ACTIONS(2005),
    [sym_dynamic_polled_port] = ACTIONS(2005),
    [sym_static_port_binding] = ACTIONS(2005),
    [sym_static_ip_binding] = ACTIONS(2005),
    [sym_get_dns_list] = ACTIONS(2005),
    [sym_set_dns_list] = ACTIONS(2005),
    [sym_get_ip_address] = ACTIONS(2005),
    [sym_set_ip_address] = ACTIONS(2005),
    [sym_get_url_list] = ACTIONS(2005),
    [sym_add_url_entry] = ACTIONS(2005),
    [sym_delete_url_entry] = ACTIONS(2005),
    [sym_abs_value] = ACTIONS(2005),
    [sym_max_value] = ACTIONS(2005),
    [sym_min_value] = ACTIONS(2005),
    [sym_exp_value] = ACTIONS(2005),
    [sym_log_value] = ACTIONS(2005),
    [sym_log10_value] = ACTIONS(2005),
    [sym_power_value] = ACTIONS(2005),
    [sym_sqrt_value] = ACTIONS(2005),
    [sym_raw_be] = ACTIONS(2005),
    [sym_raw_le] = ACTIONS(2005),
    [sym_internal_queue_size_set] = ACTIONS(2005),
    [sym_internal_queue_size_get] = ACTIONS(2005),
    [sym_internal_threshold_set] = ACTIONS(2005),
    [sym_internal_threshold_get] = ACTIONS(2005),
    [sym_rebuilt_event] = ACTIONS(2005),
    [sym_do_custom_event] = ACTIONS(2005),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2005),
    [sym_clkmgr_set_clk_source] = ACTIONS(2005),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2005),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2005),
    [sym_clkmgr_get_timezone] = ACTIONS(2005),
    [sym_clkmgr_set_timezone] = ACTIONS(2005),
    [sym_clkmgr_get_resync_period] = ACTIONS(2005),
    [sym_clkmgr_set_resync_period] = ACTIONS(2005),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2005),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2005),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2005),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2005),
    [sym_clkmgr_get_timeservers] = ACTIONS(2005),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2005),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2005),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2005),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2005),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2005),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2005),
    [sym_set_log_level] = ACTIONS(2005),
    [sym_get_log_level] = ACTIONS(2005),
    [sym_amx_log] = ACTIONS(2005),
    [sym_smtp_server_config_set] = ACTIONS(2005),
    [sym_smtp_server_config_get] = ACTIONS(2005),
    [sym_smtp_send] = ACTIONS(2005),
    [sym_validate_netlinx_account] = ACTIONS(2005),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2005),
    [sym_authenticate_certificate] = ACTIONS(2005),
    [sym_audit_netlinx_session_event] = ACTIONS(2005),
    [sym_audit_netlinx_generic_event] = ACTIONS(2005),
    [sym_get_available_flash_disk_space] = ACTIONS(2005),
    [sym_get_max_flash_disk_space] = ACTIONS(2005),
    [sym_data_create_feed] = ACTIONS(2005),
    [sym_data_delete_feed] = ACTIONS(2005),
    [sym_data_publish_feed] = ACTIONS(2005),
    [sym_data_get_published_feed] = ACTIONS(2005),
    [sym_data_add_record] = ACTIONS(2005),
    [sym_data_get_event_record] = ACTIONS(2005),
    [sym__wc_data_create_feed] = ACTIONS(2005),
    [sym__wc_data_add_record] = ACTIONS(2005),
    [sym__wc_data_get_event_record] = ACTIONS(2005),
    [sym_format] = ACTIONS(2005),
    [sym_astro_clock] = ACTIONS(2005),
    [sym_set_outdoor_temperature] = ACTIONS(2005),
    [sym_set_virtual_level_count] = ACTIONS(2005),
    [sym_set_virtual_channel_count] = ACTIONS(2005),
    [sym_set_virtual_port_count] = ACTIONS(2005),
    [sym__wc_encode_int] = ACTIONS(2005),
    [sym__wc_decode_int] = ACTIONS(2005),
    [sym__wc_int] = ACTIONS(2005),
    [sym__wc_to_ch_int] = ACTIONS(2005),
    [sym__ch_to_wc_int] = ACTIONS(2005),
    [sym__wc_find_string_int] = ACTIONS(2005),
    [sym__wc_left_string_int] = ACTIONS(2005),
    [sym__wc_lower_string_int] = ACTIONS(2005),
    [sym__wc_mid_string_int] = ACTIONS(2005),
    [sym__wc_remove_string_int] = ACTIONS(2005),
    [sym__wc_right_string_int] = ACTIONS(2005),
    [sym__wc_upper_string_int] = ACTIONS(2005),
    [sym__wc_compare_string_int] = ACTIONS(2005),
    [sym__wc_get_buffer_char_int] = ACTIONS(2005),
    [sym__wc_get_buffer_string_int] = ACTIONS(2005),
    [sym__wc_concat_string_int] = ACTIONS(2005),
    [sym__wc_file_open_int] = ACTIONS(2005),
    [sym__wc_file_close_int] = ACTIONS(2005),
    [sym__wc_file_read_int] = ACTIONS(2005),
    [sym__wc_file_read_line_int] = ACTIONS(2005),
    [sym__wc_file_write_int] = ACTIONS(2005),
    [sym__wc_file_write_line_int] = ACTIONS(2005),
    [aux_sym_preproc_include_token1] = ACTIONS(2005),
    [aux_sym_preproc_define_token1] = ACTIONS(2007),
    [aux_sym_preproc_warn_token1] = ACTIONS(2007),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2007),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2007),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2007),
    [anon_sym_LBRACK] = ACTIONS(2007),
    [anon_sym_LBRACE] = ACTIONS(2007),
    [anon_sym_LPAREN] = ACTIONS(2005),
    [anon_sym_BANG] = ACTIONS(2007),
    [anon_sym_TILDE] = ACTIONS(2007),
    [anon_sym_DASH] = ACTIONS(2005),
    [anon_sym_PLUS] = ACTIONS(2005),
    [anon_sym_DASH_DASH] = ACTIONS(2007),
    [anon_sym_PLUS_PLUS] = ACTIONS(2007),
    [anon_sym_DQUOTE] = ACTIONS(2007),
    [anon_sym_SQUOTE] = ACTIONS(2007),
    [sym_number_literal] = ACTIONS(2007),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(431)] = {
    [ts_builtin_sym_end] = ACTIONS(2135),
    [sym_identifier] = ACTIONS(2133),
    [sym_program_name_keyword] = ACTIONS(2133),
    [sym_module_name_keyword] = ACTIONS(2133),
    [sym_define_device_keyword] = ACTIONS(2133),
    [sym_define_combine_keyword] = ACTIONS(2133),
    [sym_define_constant_keyword] = ACTIONS(2133),
    [sym_define_type_keyword] = ACTIONS(2133),
    [sym_define_variable_keyword] = ACTIONS(2133),
    [sym_define_system_variable_keyword] = ACTIONS(2133),
    [sym_define_connect_level_keyword] = ACTIONS(2133),
    [sym_define_latching_keyword] = ACTIONS(2133),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2133),
    [sym_define_toggling_keyword] = ACTIONS(2133),
    [sym_define_call_keyword] = ACTIONS(2133),
    [sym_define_function_keyword] = ACTIONS(2133),
    [sym_define_library_function_keyword] = ACTIONS(2133),
    [sym_define_start_keyword] = ACTIONS(2133),
    [sym_define_event_keyword] = ACTIONS(2133),
    [sym_define_program_keyword] = ACTIONS(2133),
    [sym_define_module_keyword] = ACTIONS(2133),
    [sym_char_keyword] = ACTIONS(2133),
    [sym_widechar_keyword] = ACTIONS(2133),
    [sym_integer_keyword] = ACTIONS(2133),
    [sym_sinteger_keyword] = ACTIONS(2133),
    [sym_long_keyword] = ACTIONS(2133),
    [sym_slong_keyword] = ACTIONS(2133),
    [sym_float_keyword] = ACTIONS(2133),
    [sym_double_keyword] = ACTIONS(2133),
    [sym_variant_keyword] = ACTIONS(2133),
    [sym_variantarray_keyword] = ACTIONS(2133),
    [sym_local_var_keyword] = ACTIONS(2133),
    [sym_stack_var_keyword] = ACTIONS(2133),
    [sym_constant_keyword] = ACTIONS(2133),
    [sym_volatile_keyword] = ACTIONS(2133),
    [sym_non_volatile_keyword] = ACTIONS(2133),
    [sym_persistent_keyword] = ACTIONS(2133),
    [sym_dev_keyword] = ACTIONS(2133),
    [sym_devlev_keyword] = ACTIONS(2133),
    [sym_devchan_keyword] = ACTIONS(2133),
    [sym_if_keyword] = ACTIONS(2133),
    [sym_switch_keyword] = ACTIONS(2133),
    [sym_case_keyword] = ACTIONS(2133),
    [sym_default_keyword] = ACTIONS(2133),
    [sym_while_keyword] = ACTIONS(2133),
    [sym_for_keyword] = ACTIONS(2133),
    [sym_break_keyword] = ACTIONS(2133),
    [sym_continue_keyword] = ACTIONS(2133),
    [sym_return_keyword] = ACTIONS(2133),
    [sym_select_keyword] = ACTIONS(2133),
    [sym_active_keyword] = ACTIONS(2133),
    [sym_struct_keyword] = ACTIONS(2133),
    [sym_structure_keyword] = ACTIONS(2133),
    [sym_button_event_keyword] = ACTIONS(2133),
    [sym_channel_event_keyword] = ACTIONS(2133),
    [sym_level_event_keyword] = ACTIONS(2133),
    [sym_data_event_keyword] = ACTIONS(2133),
    [sym_timeline_event_keyword] = ACTIONS(2133),
    [sym_custom_event_keyword] = ACTIONS(2133),
    [sym_push_keyword] = ACTIONS(2133),
    [sym_release_keyword] = ACTIONS(2133),
    [aux_sym_on_keyword_token1] = ACTIONS(2133),
    [aux_sym_off_keyword_token1] = ACTIONS(2133),
    [sym_send_level_keyword] = ACTIONS(2133),
    [sym_send_string_keyword] = ACTIONS(2133),
    [sym_send_command_keyword] = ACTIONS(2133),
    [sym_clear_buffer_keyword] = ACTIONS(2133),
    [sym_create_buffer_keyword] = ACTIONS(2133),
    [sym_create_multi_buffer_keyword] = ACTIONS(2133),
    [sym_call_keyword] = ACTIONS(2133),
    [sym_system_call_keyword] = ACTIONS(2133),
    [sym_devchan_to_keyword] = ACTIONS(2133),
    [sym_devchan_min_to_keyword] = ACTIONS(2133),
    [sym_devchan_total_off_keyword] = ACTIONS(2133),
    [sym_devchan_pulse_keyword] = ACTIONS(2133),
    [sym_wait_keyword] = ACTIONS(2133),
    [sym_cancel_wait_keyword] = ACTIONS(2133),
    [sym_wait_until_keyword] = ACTIONS(2133),
    [sym_cancel_wait_until_keyword] = ACTIONS(2133),
    [sym_cancel_all_wait_keyword] = ACTIONS(2133),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2133),
    [sym_not] = ACTIONS(2133),
    [sym_bnot] = ACTIONS(2133),
    [sym___file__] = ACTIONS(2133),
    [sym___date__] = ACTIONS(2133),
    [sym___ldate__] = ACTIONS(2133),
    [sym___time__] = ACTIONS(2133),
    [sym___line__] = ACTIONS(2133),
    [sym___name__] = ACTIONS(2133),
    [sym___netlinx__] = ACTIONS(2133),
    [sym_netlinx_axi_version] = ACTIONS(2133),
    [sym_url_flg_tcp] = ACTIONS(2133),
    [sym_url_flg_accinfopresent] = ACTIONS(2133),
    [sym_url_flg_temp] = ACTIONS(2133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2133),
    [sym_url_flg_stat_mask] = ACTIONS(2133),
    [sym_url_flg_stat_lookup] = ACTIONS(2133),
    [sym_url_flg_stat_connecting] = ACTIONS(2133),
    [sym_url_flg_stat_waiting] = ACTIONS(2133),
    [sym_url_flg_stat_connected] = ACTIONS(2133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2133),
    [sym_first_virtual_device] = ACTIONS(2133),
    [sym_first_local_port] = ACTIONS(2133),
    [sym_dynamic_virtual_device] = ACTIONS(2133),
    [sym_timeline_once] = ACTIONS(2133),
    [sym_timeline_repeat] = ACTIONS(2133),
    [sym_timeline_absolute] = ACTIONS(2133),
    [sym_timeline_relative] = ACTIONS(2133),
    [sym_do_push_timed_infinite] = ACTIONS(2133),
    [sym_source_type_no_address] = ACTIONS(2133),
    [sym_source_type_neuron_id] = ACTIONS(2133),
    [sym_source_type_ip_address] = ACTIONS(2133),
    [sym_source_type_axlink] = ACTIONS(2133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2133),
    [sym_source_type_ip_socket_address] = ACTIONS(2133),
    [sym_source_type_rs232] = ACTIONS(2133),
    [sym_source_type_internal] = ACTIONS(2133),
    [sym_source_type_mac_address] = ACTIONS(2133),
    [sym_source_type_ipv4_port] = ACTIONS(2133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2133),
    [sym_true] = ACTIONS(2133),
    [sym_false] = ACTIONS(2133),
    [sym_file_read_only] = ACTIONS(2133),
    [sym_file_rw_new] = ACTIONS(2133),
    [sym_file_rw_append] = ACTIONS(2133),
    [sym_ip_tcp] = ACTIONS(2133),
    [sym_ip_udp] = ACTIONS(2133),
    [sym_ip_udp_2way] = ACTIONS(2133),
    [sym_xml_encode_types] = ACTIONS(2133),
    [sym_xml_encode_char_as_list] = ACTIONS(2133),
    [sym_xml_encode_le] = ACTIONS(2133),
    [sym_xml_decode_types] = ACTIONS(2133),
    [sym_xml_decode_no_preserve] = ACTIONS(2133),
    [sym_duet_dev_type_display_device] = ACTIONS(2133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2133),
    [sym_duet_dev_type_io_device] = ACTIONS(2133),
    [sym_duet_dev_type_relay_device] = ACTIONS(2133),
    [sym_duet_dev_type_ups] = ACTIONS(2133),
    [sym_duet_dev_type_amplifier] = ACTIONS(2133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2133),
    [sym_duet_dev_type_camera] = ACTIONS(2133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2133),
    [sym_duet_dev_type_dss] = ACTIONS(2133),
    [sym_duet_dev_type_dvr] = ACTIONS(2133),
    [sym_duet_dev_type_disc_device] = ACTIONS(2133),
    [sym_duet_dev_type_document_camera] = ACTIONS(2133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2133),
    [sym_duet_dev_type_hvac] = ACTIONS(2133),
    [sym_duet_dev_type_keypad] = ACTIONS(2133),
    [sym_duet_dev_type_light] = ACTIONS(2133),
    [sym_duet_dev_type_monitor] = ACTIONS(2133),
    [sym_duet_dev_type_motor] = ACTIONS(2133),
    [sym_duet_dev_type_multi_window] = ACTIONS(2133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2133),
    [sym_duet_dev_type_receiver] = ACTIONS(2133),
    [sym_duet_dev_type_security_system] = ACTIONS(2133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2133),
    [sym_duet_dev_type_settop_box] = ACTIONS(2133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2133),
    [sym_duet_dev_type_switcher] = ACTIONS(2133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2133),
    [sym_duet_dev_type_tv] = ACTIONS(2133),
    [sym_duet_dev_type_utility] = ACTIONS(2133),
    [sym_duet_dev_type_vcr] = ACTIONS(2133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2133),
    [sym_duet_dev_type_video_processor] = ACTIONS(2133),
    [sym_duet_dev_type_video_projector] = ACTIONS(2133),
    [sym_duet_dev_type_video_wall] = ACTIONS(2133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2133),
    [sym_duet_dev_type_weather] = ACTIONS(2133),
    [sym_duet_dev_not_polled] = ACTIONS(2133),
    [sym_duet_dev_polled] = ACTIONS(2133),
    [sym_clkmgr_mode_network] = ACTIONS(2133),
    [sym_clkmgr_mode_standalone] = ACTIONS(2133),
    [sym_normal_standby] = ACTIONS(2133),
    [sym_normal_wake] = ACTIONS(2133),
    [sym_ip_multicast_ttl_option] = ACTIONS(2133),
    [sym_ip_ttl_subnet] = ACTIONS(2133),
    [sym_ip_ttl_site] = ACTIONS(2133),
    [sym_ip_ttl_region] = ACTIONS(2133),
    [sym_ip_ttl_continent] = ACTIONS(2133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2133),
    [sym_ip_nodelay_on] = ACTIONS(2133),
    [sym_ip_nodelay_off] = ACTIONS(2133),
    [sym_amx_error] = ACTIONS(2133),
    [sym_amx_warning] = ACTIONS(2133),
    [sym_amx_info] = ACTIONS(2133),
    [sym_amx_debug] = ACTIONS(2133),
    [sym_netlinx_login_success] = ACTIONS(2133),
    [sym_netlinx_login_fail] = ACTIONS(2133),
    [sym_netlinx_logout] = ACTIONS(2133),
    [sym_null_str] = ACTIONS(2133),
    [sym_smtp_address] = ACTIONS(2133),
    [sym_smtp_port_number] = ACTIONS(2133),
    [sym_smtp_username] = ACTIONS(2133),
    [sym_smtp_password] = ACTIONS(2133),
    [sym_smtp_require_tls] = ACTIONS(2133),
    [sym_smtp_from] = ACTIONS(2133),
    [sym_smtp_tls_true] = ACTIONS(2133),
    [sym_smtp_tls_false] = ACTIONS(2133),
    [sym_listview_on_row_select_event] = ACTIONS(2133),
    [sym_data_structure_unknown] = ACTIONS(2133),
    [sym_data_structure_datafeed] = ACTIONS(2133),
    [sym_data_structure_datarecordset] = ACTIONS(2133),
    [sym_data_structure_datarecord] = ACTIONS(2133),
    [sym_data_structure_datafield] = ACTIONS(2133),
    [sym_data_max_content_field_count] = ACTIONS(2133),
    [sym_data_max_metadata_field_count] = ACTIONS(2133),
    [sym_data_max_id_length] = ACTIONS(2133),
    [sym_data_max_name_length] = ACTIONS(2133),
    [sym_data_max_description_length] = ACTIONS(2133),
    [sym_data_max_type_length] = ACTIONS(2133),
    [sym_data_max_format_length] = ACTIONS(2133),
    [sym_data_max_source_length] = ACTIONS(2133),
    [sym_data_max_value_length] = ACTIONS(2133),
    [sym_data_max_label_length] = ACTIONS(2133),
    [sym_data_max_url_length] = ACTIONS(2133),
    [sym_max_last_login_info_length] = ACTIONS(2133),
    [sym_data_type_unknown] = ACTIONS(2133),
    [sym_data_type_string] = ACTIONS(2133),
    [sym_data_type_datetime] = ACTIONS(2133),
    [sym_data_type_date] = ACTIONS(2133),
    [sym_data_type_time] = ACTIONS(2133),
    [sym_data_type_image] = ACTIONS(2133),
    [sym_data_format_url] = ACTIONS(2133),
    [sym_data_format_phone] = ACTIONS(2133),
    [sym_data_format_email] = ACTIONS(2133),
    [sym_data_format_iso8601] = ACTIONS(2133),
    [sym_auditlog_priv] = ACTIONS(2133),
    [sym_configuration_priv] = ACTIONS(2133),
    [sym_deviceconfig_priv] = ACTIONS(2133),
    [sym_ftp_priv] = ACTIONS(2133),
    [sym_http_priv] = ACTIONS(2133),
    [sym_networkconfig_priv] = ACTIONS(2133),
    [sym_programport_priv] = ACTIONS(2133),
    [sym_remoteui_priv] = ACTIONS(2133),
    [sym_securitycontrol_priv] = ACTIONS(2133),
    [sym_softwaremanagement_priv] = ACTIONS(2133),
    [sym_terminal_priv] = ACTIONS(2133),
    [sym_tpadmin_priv] = ACTIONS(2133),
    [sym_user1_priv] = ACTIONS(2133),
    [sym_user2_priv] = ACTIONS(2133),
    [sym_user3_priv] = ACTIONS(2133),
    [sym_user4_priv] = ACTIONS(2133),
    [sym_usermanagement_priv] = ACTIONS(2133),
    [sym_valid_account] = ACTIONS(2133),
    [sym_err_name_invalid] = ACTIONS(2133),
    [sym_err_password_invalid] = ACTIONS(2133),
    [sym_err_invalid_account] = ACTIONS(2133),
    [sym_err_no_authorization] = ACTIONS(2133),
    [sym_err_invalid_parameter] = ACTIONS(2133),
    [sym_err_user_locked_out] = ACTIONS(2133),
    [sym_err_user_account_expired] = ACTIONS(2133),
    [sym_err_general_error] = ACTIONS(2133),
    [sym_err_authentication_error] = ACTIONS(2133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2133),
    [sym_tls_validate_certificate] = ACTIONS(2133),
    [sym_tdata] = ACTIONS(2133),
    [sym_tchannel] = ACTIONS(2133),
    [sym_tlevel] = ACTIONS(2133),
    [sym_tbutton] = ACTIONS(2133),
    [sym_ttimeline] = ACTIONS(2133),
    [sym_tcustom] = ACTIONS(2133),
    [sym_url_struct] = ACTIONS(2133),
    [sym_dns_struct] = ACTIONS(2133),
    [sym_ip_address_struct] = ACTIONS(2133),
    [sym_dev_info_struct] = ACTIONS(2133),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2133),
    [sym_data_feed] = ACTIONS(2133),
    [sym_data_field] = ACTIONS(2133),
    [sym_data_record] = ACTIONS(2133),
    [sym_wc_data_feed] = ACTIONS(2133),
    [sym_wc_data_field] = ACTIONS(2133),
    [sym_wc_data_record] = ACTIONS(2133),
    [sym_last_login_info] = ACTIONS(2133),
    [sym____reserved___] = ACTIONS(2133),
    [sym_date] = ACTIONS(2133),
    [sym_day] = ACTIONS(2133),
    [sym_time] = ACTIONS(2133),
    [sym_ldate] = ACTIONS(2133),
    [sym_push_device] = ACTIONS(2133),
    [sym_push_channel] = ACTIONS(2133),
    [sym_push_devchan] = ACTIONS(2133),
    [sym_release_device] = ACTIONS(2133),
    [sym_release_channel] = ACTIONS(2133),
    [sym_release_devchan] = ACTIONS(2133),
    [sym_master_slot] = ACTIONS(2133),
    [sym_get_pulse_time] = ACTIONS(2133),
    [sym_get_timer] = ACTIONS(2133),
    [sym_master_sn] = ACTIONS(2133),
    [sym_system_number] = ACTIONS(2133),
    [sym_button] = ACTIONS(2133),
    [sym_dv_channel] = ACTIONS(2133),
    [sym_data] = ACTIONS(2133),
    [sym_level] = ACTIONS(2133),
    [sym_timeline] = ACTIONS(2133),
    [sym_channel] = ACTIONS(2133),
    [sym_custom] = ACTIONS(2133),
    [sym_atoi] = ACTIONS(2133),
    [sym_atol] = ACTIONS(2133),
    [sym_atof] = ACTIONS(2133),
    [sym_itoa] = ACTIONS(2133),
    [sym_itohex] = ACTIONS(2133),
    [sym_hextoi] = ACTIONS(2133),
    [sym_ftoa] = ACTIONS(2133),
    [sym_find_string] = ACTIONS(2133),
    [sym_left_string] = ACTIONS(2133),
    [sym_right_string] = ACTIONS(2133),
    [sym_mid_string] = ACTIONS(2133),
    [sym_remove_string] = ACTIONS(2133),
    [sym_get_buffer_string] = ACTIONS(2133),
    [sym_get_buffer_char] = ACTIONS(2133),
    [sym_get_multi_buffer_string] = ACTIONS(2133),
    [sym_length_string] = ACTIONS(2133),
    [sym_max_length_string] = ACTIONS(2133),
    [sym_set_length_string] = ACTIONS(2133),
    [sym_compare_string] = ACTIONS(2133),
    [sym_upper_string] = ACTIONS(2133),
    [sym_lower_string] = ACTIONS(2133),
    [sym_get_last] = ACTIONS(2133),
    [sym_date_to_day] = ACTIONS(2133),
    [sym_date_to_month] = ACTIONS(2133),
    [sym_date_to_year] = ACTIONS(2133),
    [sym_time_to_hour] = ACTIONS(2133),
    [sym_time_to_minute] = ACTIONS(2133),
    [sym_time_to_second] = ACTIONS(2133),
    [sym_day_of_week] = ACTIONS(2133),
    [sym_set_pulse_time] = ACTIONS(2133),
    [sym_set_timer] = ACTIONS(2133),
    [sym_file_open] = ACTIONS(2133),
    [sym_file_close] = ACTIONS(2133),
    [sym_file_read] = ACTIONS(2133),
    [sym_file_read_line] = ACTIONS(2133),
    [sym_file_write] = ACTIONS(2133),
    [sym_file_write_line] = ACTIONS(2133),
    [sym_file_copy] = ACTIONS(2133),
    [sym_file_delete] = ACTIONS(2133),
    [sym_file_rename] = ACTIONS(2133),
    [sym_file_dir] = ACTIONS(2133),
    [sym_file_getdir] = ACTIONS(2133),
    [sym_file_setdir] = ACTIONS(2133),
    [sym_file_create_dir] = ACTIONS(2133),
    [sym_file_remove_dir] = ACTIONS(2133),
    [sym_file_seek] = ACTIONS(2133),
    [sym_length_array] = ACTIONS(2133),
    [sym_max_length_array] = ACTIONS(2133),
    [sym_set_length_array] = ACTIONS(2133),
    [sym_ip_client_open] = ACTIONS(2133),
    [sym_ip_client_close] = ACTIONS(2133),
    [sym_ip_server_open] = ACTIONS(2133),
    [sym_ip_server_close] = ACTIONS(2133),
    [sym_ip_bound_client_open] = ACTIONS(2133),
    [sym_ip_mc_server_open] = ACTIONS(2133),
    [sym_ip_set_option] = ACTIONS(2133),
    [sym_ssh_client_open] = ACTIONS(2133),
    [sym_ssh_client_close] = ACTIONS(2133),
    [sym_tls_client_open] = ACTIONS(2133),
    [sym_tls_client_close] = ACTIONS(2133),
    [sym_do_push] = ACTIONS(2133),
    [sym_do_push_timed] = ACTIONS(2133),
    [sym_do_release] = ACTIONS(2133),
    [sym_device_id] = ACTIONS(2133),
    [sym_device_info] = ACTIONS(2133),
    [sym_device_id_string] = ACTIONS(2133),
    [sym_device_standby] = ACTIONS(2133),
    [sym_device_wake] = ACTIONS(2133),
    [sym_timeline_create] = ACTIONS(2133),
    [sym_timeline_kill] = ACTIONS(2133),
    [sym_timeline_reload] = ACTIONS(2133),
    [sym_timeline_pause] = ACTIONS(2133),
    [sym_timeline_restart] = ACTIONS(2133),
    [sym_timeline_set] = ACTIONS(2133),
    [sym_timeline_get] = ACTIONS(2133),
    [sym_timeline_active] = ACTIONS(2133),
    [sym_timeline_dynamic_id] = ACTIONS(2133),
    [sym_variable_to_string] = ACTIONS(2133),
    [sym_string_to_variable] = ACTIONS(2133),
    [sym_length_variable_to_string] = ACTIONS(2133),
    [sym_type_cast] = ACTIONS(2133),
    [sym_variable_to_xml] = ACTIONS(2133),
    [sym_xml_to_variable] = ACTIONS(2133),
    [sym_length_variable_to_xml] = ACTIONS(2133),
    [sym_redirect_string] = ACTIONS(2133),
    [sym_random_number] = ACTIONS(2133),
    [sym_get_unique_id] = ACTIONS(2133),
    [sym_get_system_number] = ACTIONS(2133),
    [sym_set_system_number] = ACTIONS(2133),
    [sym_get_serial_number] = ACTIONS(2133),
    [sym_get_master_build] = ACTIONS(2133),
    [sym_set_validation_code] = ACTIONS(2133),
    [sym_reboot] = ACTIONS(2133),
    [sym_load_duet_module] = ACTIONS(2133),
    [sym_unload_duet_module] = ACTIONS(2133),
    [sym_duet_mem_size_set] = ACTIONS(2133),
    [sym_duet_mem_size_get] = ACTIONS(2133),
    [sym_dynamic_application_device] = ACTIONS(2133),
    [sym_dynamic_polled_port] = ACTIONS(2133),
    [sym_static_port_binding] = ACTIONS(2133),
    [sym_static_ip_binding] = ACTIONS(2133),
    [sym_get_dns_list] = ACTIONS(2133),
    [sym_set_dns_list] = ACTIONS(2133),
    [sym_get_ip_address] = ACTIONS(2133),
    [sym_set_ip_address] = ACTIONS(2133),
    [sym_get_url_list] = ACTIONS(2133),
    [sym_add_url_entry] = ACTIONS(2133),
    [sym_delete_url_entry] = ACTIONS(2133),
    [sym_abs_value] = ACTIONS(2133),
    [sym_max_value] = ACTIONS(2133),
    [sym_min_value] = ACTIONS(2133),
    [sym_exp_value] = ACTIONS(2133),
    [sym_log_value] = ACTIONS(2133),
    [sym_log10_value] = ACTIONS(2133),
    [sym_power_value] = ACTIONS(2133),
    [sym_sqrt_value] = ACTIONS(2133),
    [sym_raw_be] = ACTIONS(2133),
    [sym_raw_le] = ACTIONS(2133),
    [sym_internal_queue_size_set] = ACTIONS(2133),
    [sym_internal_queue_size_get] = ACTIONS(2133),
    [sym_internal_threshold_set] = ACTIONS(2133),
    [sym_internal_threshold_get] = ACTIONS(2133),
    [sym_rebuilt_event] = ACTIONS(2133),
    [sym_do_custom_event] = ACTIONS(2133),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2133),
    [sym_clkmgr_set_clk_source] = ACTIONS(2133),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2133),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2133),
    [sym_clkmgr_get_timezone] = ACTIONS(2133),
    [sym_clkmgr_set_timezone] = ACTIONS(2133),
    [sym_clkmgr_get_resync_period] = ACTIONS(2133),
    [sym_clkmgr_set_resync_period] = ACTIONS(2133),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2133),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2133),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2133),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2133),
    [sym_clkmgr_get_timeservers] = ACTIONS(2133),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2133),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2133),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2133),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2133),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2133),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2133),
    [sym_set_log_level] = ACTIONS(2133),
    [sym_get_log_level] = ACTIONS(2133),
    [sym_amx_log] = ACTIONS(2133),
    [sym_smtp_server_config_set] = ACTIONS(2133),
    [sym_smtp_server_config_get] = ACTIONS(2133),
    [sym_smtp_send] = ACTIONS(2133),
    [sym_validate_netlinx_account] = ACTIONS(2133),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2133),
    [sym_authenticate_certificate] = ACTIONS(2133),
    [sym_audit_netlinx_session_event] = ACTIONS(2133),
    [sym_audit_netlinx_generic_event] = ACTIONS(2133),
    [sym_get_available_flash_disk_space] = ACTIONS(2133),
    [sym_get_max_flash_disk_space] = ACTIONS(2133),
    [sym_data_create_feed] = ACTIONS(2133),
    [sym_data_delete_feed] = ACTIONS(2133),
    [sym_data_publish_feed] = ACTIONS(2133),
    [sym_data_get_published_feed] = ACTIONS(2133),
    [sym_data_add_record] = ACTIONS(2133),
    [sym_data_get_event_record] = ACTIONS(2133),
    [sym__wc_data_create_feed] = ACTIONS(2133),
    [sym__wc_data_add_record] = ACTIONS(2133),
    [sym__wc_data_get_event_record] = ACTIONS(2133),
    [sym_format] = ACTIONS(2133),
    [sym_astro_clock] = ACTIONS(2133),
    [sym_set_outdoor_temperature] = ACTIONS(2133),
    [sym_set_virtual_level_count] = ACTIONS(2133),
    [sym_set_virtual_channel_count] = ACTIONS(2133),
    [sym_set_virtual_port_count] = ACTIONS(2133),
    [sym__wc_encode_int] = ACTIONS(2133),
    [sym__wc_decode_int] = ACTIONS(2133),
    [sym__wc_int] = ACTIONS(2133),
    [sym__wc_to_ch_int] = ACTIONS(2133),
    [sym__ch_to_wc_int] = ACTIONS(2133),
    [sym__wc_find_string_int] = ACTIONS(2133),
    [sym__wc_left_string_int] = ACTIONS(2133),
    [sym__wc_lower_string_int] = ACTIONS(2133),
    [sym__wc_mid_string_int] = ACTIONS(2133),
    [sym__wc_remove_string_int] = ACTIONS(2133),
    [sym__wc_right_string_int] = ACTIONS(2133),
    [sym__wc_upper_string_int] = ACTIONS(2133),
    [sym__wc_compare_string_int] = ACTIONS(2133),
    [sym__wc_get_buffer_char_int] = ACTIONS(2133),
    [sym__wc_get_buffer_string_int] = ACTIONS(2133),
    [sym__wc_concat_string_int] = ACTIONS(2133),
    [sym__wc_file_open_int] = ACTIONS(2133),
    [sym__wc_file_close_int] = ACTIONS(2133),
    [sym__wc_file_read_int] = ACTIONS(2133),
    [sym__wc_file_read_line_int] = ACTIONS(2133),
    [sym__wc_file_write_int] = ACTIONS(2133),
    [sym__wc_file_write_line_int] = ACTIONS(2133),
    [aux_sym_preproc_include_token1] = ACTIONS(2133),
    [aux_sym_preproc_define_token1] = ACTIONS(2135),
    [aux_sym_preproc_warn_token1] = ACTIONS(2135),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2135),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2135),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2135),
    [anon_sym_LBRACK] = ACTIONS(2135),
    [anon_sym_LBRACE] = ACTIONS(2135),
    [anon_sym_LPAREN] = ACTIONS(2133),
    [anon_sym_BANG] = ACTIONS(2135),
    [anon_sym_TILDE] = ACTIONS(2135),
    [anon_sym_DASH] = ACTIONS(2133),
    [anon_sym_PLUS] = ACTIONS(2133),
    [anon_sym_DASH_DASH] = ACTIONS(2135),
    [anon_sym_PLUS_PLUS] = ACTIONS(2135),
    [anon_sym_DQUOTE] = ACTIONS(2135),
    [anon_sym_SQUOTE] = ACTIONS(2135),
    [sym_number_literal] = ACTIONS(2135),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(432)] = {
    [ts_builtin_sym_end] = ACTIONS(1995),
    [sym_identifier] = ACTIONS(1993),
    [sym_program_name_keyword] = ACTIONS(1993),
    [sym_module_name_keyword] = ACTIONS(1993),
    [sym_define_device_keyword] = ACTIONS(1993),
    [sym_define_combine_keyword] = ACTIONS(1993),
    [sym_define_constant_keyword] = ACTIONS(1993),
    [sym_define_type_keyword] = ACTIONS(1993),
    [sym_define_variable_keyword] = ACTIONS(1993),
    [sym_define_system_variable_keyword] = ACTIONS(1993),
    [sym_define_connect_level_keyword] = ACTIONS(1993),
    [sym_define_latching_keyword] = ACTIONS(1993),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1993),
    [sym_define_toggling_keyword] = ACTIONS(1993),
    [sym_define_call_keyword] = ACTIONS(1993),
    [sym_define_function_keyword] = ACTIONS(1993),
    [sym_define_library_function_keyword] = ACTIONS(1993),
    [sym_define_start_keyword] = ACTIONS(1993),
    [sym_define_event_keyword] = ACTIONS(1993),
    [sym_define_program_keyword] = ACTIONS(1993),
    [sym_define_module_keyword] = ACTIONS(1993),
    [sym_char_keyword] = ACTIONS(1993),
    [sym_widechar_keyword] = ACTIONS(1993),
    [sym_integer_keyword] = ACTIONS(1993),
    [sym_sinteger_keyword] = ACTIONS(1993),
    [sym_long_keyword] = ACTIONS(1993),
    [sym_slong_keyword] = ACTIONS(1993),
    [sym_float_keyword] = ACTIONS(1993),
    [sym_double_keyword] = ACTIONS(1993),
    [sym_variant_keyword] = ACTIONS(1993),
    [sym_variantarray_keyword] = ACTIONS(1993),
    [sym_local_var_keyword] = ACTIONS(1993),
    [sym_stack_var_keyword] = ACTIONS(1993),
    [sym_constant_keyword] = ACTIONS(1993),
    [sym_volatile_keyword] = ACTIONS(1993),
    [sym_non_volatile_keyword] = ACTIONS(1993),
    [sym_persistent_keyword] = ACTIONS(1993),
    [sym_dev_keyword] = ACTIONS(1993),
    [sym_devlev_keyword] = ACTIONS(1993),
    [sym_devchan_keyword] = ACTIONS(1993),
    [sym_if_keyword] = ACTIONS(1993),
    [sym_switch_keyword] = ACTIONS(1993),
    [sym_case_keyword] = ACTIONS(1993),
    [sym_default_keyword] = ACTIONS(1993),
    [sym_while_keyword] = ACTIONS(1993),
    [sym_for_keyword] = ACTIONS(1993),
    [sym_break_keyword] = ACTIONS(1993),
    [sym_continue_keyword] = ACTIONS(1993),
    [sym_return_keyword] = ACTIONS(1993),
    [sym_select_keyword] = ACTIONS(1993),
    [sym_active_keyword] = ACTIONS(1993),
    [sym_struct_keyword] = ACTIONS(1993),
    [sym_structure_keyword] = ACTIONS(1993),
    [sym_button_event_keyword] = ACTIONS(1993),
    [sym_channel_event_keyword] = ACTIONS(1993),
    [sym_level_event_keyword] = ACTIONS(1993),
    [sym_data_event_keyword] = ACTIONS(1993),
    [sym_timeline_event_keyword] = ACTIONS(1993),
    [sym_custom_event_keyword] = ACTIONS(1993),
    [sym_push_keyword] = ACTIONS(1993),
    [sym_release_keyword] = ACTIONS(1993),
    [aux_sym_on_keyword_token1] = ACTIONS(1993),
    [aux_sym_off_keyword_token1] = ACTIONS(1993),
    [sym_send_level_keyword] = ACTIONS(1993),
    [sym_send_string_keyword] = ACTIONS(1993),
    [sym_send_command_keyword] = ACTIONS(1993),
    [sym_clear_buffer_keyword] = ACTIONS(1993),
    [sym_create_buffer_keyword] = ACTIONS(1993),
    [sym_create_multi_buffer_keyword] = ACTIONS(1993),
    [sym_call_keyword] = ACTIONS(1993),
    [sym_system_call_keyword] = ACTIONS(1993),
    [sym_devchan_to_keyword] = ACTIONS(1993),
    [sym_devchan_min_to_keyword] = ACTIONS(1993),
    [sym_devchan_total_off_keyword] = ACTIONS(1993),
    [sym_devchan_pulse_keyword] = ACTIONS(1993),
    [sym_wait_keyword] = ACTIONS(1993),
    [sym_cancel_wait_keyword] = ACTIONS(1993),
    [sym_wait_until_keyword] = ACTIONS(1993),
    [sym_cancel_wait_until_keyword] = ACTIONS(1993),
    [sym_cancel_all_wait_keyword] = ACTIONS(1993),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1993),
    [sym_not] = ACTIONS(1993),
    [sym_bnot] = ACTIONS(1993),
    [sym___file__] = ACTIONS(1993),
    [sym___date__] = ACTIONS(1993),
    [sym___ldate__] = ACTIONS(1993),
    [sym___time__] = ACTIONS(1993),
    [sym___line__] = ACTIONS(1993),
    [sym___name__] = ACTIONS(1993),
    [sym___netlinx__] = ACTIONS(1993),
    [sym_netlinx_axi_version] = ACTIONS(1993),
    [sym_url_flg_tcp] = ACTIONS(1993),
    [sym_url_flg_accinfopresent] = ACTIONS(1993),
    [sym_url_flg_temp] = ACTIONS(1993),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1993),
    [sym_url_flg_stat_mask] = ACTIONS(1993),
    [sym_url_flg_stat_lookup] = ACTIONS(1993),
    [sym_url_flg_stat_connecting] = ACTIONS(1993),
    [sym_url_flg_stat_waiting] = ACTIONS(1993),
    [sym_url_flg_stat_connected] = ACTIONS(1993),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1993),
    [sym_first_virtual_device] = ACTIONS(1993),
    [sym_first_local_port] = ACTIONS(1993),
    [sym_dynamic_virtual_device] = ACTIONS(1993),
    [sym_timeline_once] = ACTIONS(1993),
    [sym_timeline_repeat] = ACTIONS(1993),
    [sym_timeline_absolute] = ACTIONS(1993),
    [sym_timeline_relative] = ACTIONS(1993),
    [sym_do_push_timed_infinite] = ACTIONS(1993),
    [sym_source_type_no_address] = ACTIONS(1993),
    [sym_source_type_neuron_id] = ACTIONS(1993),
    [sym_source_type_ip_address] = ACTIONS(1993),
    [sym_source_type_axlink] = ACTIONS(1993),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1993),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1993),
    [sym_source_type_ip_socket_address] = ACTIONS(1993),
    [sym_source_type_rs232] = ACTIONS(1993),
    [sym_source_type_internal] = ACTIONS(1993),
    [sym_source_type_mac_address] = ACTIONS(1993),
    [sym_source_type_ipv4_port] = ACTIONS(1993),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1993),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1993),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1993),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1993),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1993),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1993),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1993),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1993),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1993),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1993),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1993),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1993),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1993),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1993),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1993),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1993),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1993),
    [sym_true] = ACTIONS(1993),
    [sym_false] = ACTIONS(1993),
    [sym_file_read_only] = ACTIONS(1993),
    [sym_file_rw_new] = ACTIONS(1993),
    [sym_file_rw_append] = ACTIONS(1993),
    [sym_ip_tcp] = ACTIONS(1993),
    [sym_ip_udp] = ACTIONS(1993),
    [sym_ip_udp_2way] = ACTIONS(1993),
    [sym_xml_encode_types] = ACTIONS(1993),
    [sym_xml_encode_char_as_list] = ACTIONS(1993),
    [sym_xml_encode_le] = ACTIONS(1993),
    [sym_xml_decode_types] = ACTIONS(1993),
    [sym_xml_decode_no_preserve] = ACTIONS(1993),
    [sym_duet_dev_type_display_device] = ACTIONS(1993),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1993),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1993),
    [sym_duet_dev_type_io_device] = ACTIONS(1993),
    [sym_duet_dev_type_relay_device] = ACTIONS(1993),
    [sym_duet_dev_type_ups] = ACTIONS(1993),
    [sym_duet_dev_type_amplifier] = ACTIONS(1993),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1993),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1993),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1993),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1993),
    [sym_duet_dev_type_camera] = ACTIONS(1993),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1993),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1993),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1993),
    [sym_duet_dev_type_dss] = ACTIONS(1993),
    [sym_duet_dev_type_dvr] = ACTIONS(1993),
    [sym_duet_dev_type_disc_device] = ACTIONS(1993),
    [sym_duet_dev_type_document_camera] = ACTIONS(1993),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1993),
    [sym_duet_dev_type_hvac] = ACTIONS(1993),
    [sym_duet_dev_type_keypad] = ACTIONS(1993),
    [sym_duet_dev_type_light] = ACTIONS(1993),
    [sym_duet_dev_type_monitor] = ACTIONS(1993),
    [sym_duet_dev_type_motor] = ACTIONS(1993),
    [sym_duet_dev_type_multi_window] = ACTIONS(1993),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1993),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1993),
    [sym_duet_dev_type_receiver] = ACTIONS(1993),
    [sym_duet_dev_type_security_system] = ACTIONS(1993),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1993),
    [sym_duet_dev_type_settop_box] = ACTIONS(1993),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1993),
    [sym_duet_dev_type_switcher] = ACTIONS(1993),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1993),
    [sym_duet_dev_type_tv] = ACTIONS(1993),
    [sym_duet_dev_type_utility] = ACTIONS(1993),
    [sym_duet_dev_type_vcr] = ACTIONS(1993),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1993),
    [sym_duet_dev_type_video_processor] = ACTIONS(1993),
    [sym_duet_dev_type_video_projector] = ACTIONS(1993),
    [sym_duet_dev_type_video_wall] = ACTIONS(1993),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1993),
    [sym_duet_dev_type_weather] = ACTIONS(1993),
    [sym_duet_dev_not_polled] = ACTIONS(1993),
    [sym_duet_dev_polled] = ACTIONS(1993),
    [sym_clkmgr_mode_network] = ACTIONS(1993),
    [sym_clkmgr_mode_standalone] = ACTIONS(1993),
    [sym_normal_standby] = ACTIONS(1993),
    [sym_normal_wake] = ACTIONS(1993),
    [sym_ip_multicast_ttl_option] = ACTIONS(1993),
    [sym_ip_ttl_subnet] = ACTIONS(1993),
    [sym_ip_ttl_site] = ACTIONS(1993),
    [sym_ip_ttl_region] = ACTIONS(1993),
    [sym_ip_ttl_continent] = ACTIONS(1993),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1993),
    [sym_ip_nodelay_on] = ACTIONS(1993),
    [sym_ip_nodelay_off] = ACTIONS(1993),
    [sym_amx_error] = ACTIONS(1993),
    [sym_amx_warning] = ACTIONS(1993),
    [sym_amx_info] = ACTIONS(1993),
    [sym_amx_debug] = ACTIONS(1993),
    [sym_netlinx_login_success] = ACTIONS(1993),
    [sym_netlinx_login_fail] = ACTIONS(1993),
    [sym_netlinx_logout] = ACTIONS(1993),
    [sym_null_str] = ACTIONS(1993),
    [sym_smtp_address] = ACTIONS(1993),
    [sym_smtp_port_number] = ACTIONS(1993),
    [sym_smtp_username] = ACTIONS(1993),
    [sym_smtp_password] = ACTIONS(1993),
    [sym_smtp_require_tls] = ACTIONS(1993),
    [sym_smtp_from] = ACTIONS(1993),
    [sym_smtp_tls_true] = ACTIONS(1993),
    [sym_smtp_tls_false] = ACTIONS(1993),
    [sym_listview_on_row_select_event] = ACTIONS(1993),
    [sym_data_structure_unknown] = ACTIONS(1993),
    [sym_data_structure_datafeed] = ACTIONS(1993),
    [sym_data_structure_datarecordset] = ACTIONS(1993),
    [sym_data_structure_datarecord] = ACTIONS(1993),
    [sym_data_structure_datafield] = ACTIONS(1993),
    [sym_data_max_content_field_count] = ACTIONS(1993),
    [sym_data_max_metadata_field_count] = ACTIONS(1993),
    [sym_data_max_id_length] = ACTIONS(1993),
    [sym_data_max_name_length] = ACTIONS(1993),
    [sym_data_max_description_length] = ACTIONS(1993),
    [sym_data_max_type_length] = ACTIONS(1993),
    [sym_data_max_format_length] = ACTIONS(1993),
    [sym_data_max_source_length] = ACTIONS(1993),
    [sym_data_max_value_length] = ACTIONS(1993),
    [sym_data_max_label_length] = ACTIONS(1993),
    [sym_data_max_url_length] = ACTIONS(1993),
    [sym_max_last_login_info_length] = ACTIONS(1993),
    [sym_data_type_unknown] = ACTIONS(1993),
    [sym_data_type_string] = ACTIONS(1993),
    [sym_data_type_datetime] = ACTIONS(1993),
    [sym_data_type_date] = ACTIONS(1993),
    [sym_data_type_time] = ACTIONS(1993),
    [sym_data_type_image] = ACTIONS(1993),
    [sym_data_format_url] = ACTIONS(1993),
    [sym_data_format_phone] = ACTIONS(1993),
    [sym_data_format_email] = ACTIONS(1993),
    [sym_data_format_iso8601] = ACTIONS(1993),
    [sym_auditlog_priv] = ACTIONS(1993),
    [sym_configuration_priv] = ACTIONS(1993),
    [sym_deviceconfig_priv] = ACTIONS(1993),
    [sym_ftp_priv] = ACTIONS(1993),
    [sym_http_priv] = ACTIONS(1993),
    [sym_networkconfig_priv] = ACTIONS(1993),
    [sym_programport_priv] = ACTIONS(1993),
    [sym_remoteui_priv] = ACTIONS(1993),
    [sym_securitycontrol_priv] = ACTIONS(1993),
    [sym_softwaremanagement_priv] = ACTIONS(1993),
    [sym_terminal_priv] = ACTIONS(1993),
    [sym_tpadmin_priv] = ACTIONS(1993),
    [sym_user1_priv] = ACTIONS(1993),
    [sym_user2_priv] = ACTIONS(1993),
    [sym_user3_priv] = ACTIONS(1993),
    [sym_user4_priv] = ACTIONS(1993),
    [sym_usermanagement_priv] = ACTIONS(1993),
    [sym_valid_account] = ACTIONS(1993),
    [sym_err_name_invalid] = ACTIONS(1993),
    [sym_err_password_invalid] = ACTIONS(1993),
    [sym_err_invalid_account] = ACTIONS(1993),
    [sym_err_no_authorization] = ACTIONS(1993),
    [sym_err_invalid_parameter] = ACTIONS(1993),
    [sym_err_user_locked_out] = ACTIONS(1993),
    [sym_err_user_account_expired] = ACTIONS(1993),
    [sym_err_general_error] = ACTIONS(1993),
    [sym_err_authentication_error] = ACTIONS(1993),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1993),
    [sym_tls_validate_certificate] = ACTIONS(1993),
    [sym_tdata] = ACTIONS(1993),
    [sym_tchannel] = ACTIONS(1993),
    [sym_tlevel] = ACTIONS(1993),
    [sym_tbutton] = ACTIONS(1993),
    [sym_ttimeline] = ACTIONS(1993),
    [sym_tcustom] = ACTIONS(1993),
    [sym_url_struct] = ACTIONS(1993),
    [sym_dns_struct] = ACTIONS(1993),
    [sym_ip_address_struct] = ACTIONS(1993),
    [sym_dev_info_struct] = ACTIONS(1993),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1993),
    [sym_data_feed] = ACTIONS(1993),
    [sym_data_field] = ACTIONS(1993),
    [sym_data_record] = ACTIONS(1993),
    [sym_wc_data_feed] = ACTIONS(1993),
    [sym_wc_data_field] = ACTIONS(1993),
    [sym_wc_data_record] = ACTIONS(1993),
    [sym_last_login_info] = ACTIONS(1993),
    [sym____reserved___] = ACTIONS(1993),
    [sym_date] = ACTIONS(1993),
    [sym_day] = ACTIONS(1993),
    [sym_time] = ACTIONS(1993),
    [sym_ldate] = ACTIONS(1993),
    [sym_push_device] = ACTIONS(1993),
    [sym_push_channel] = ACTIONS(1993),
    [sym_push_devchan] = ACTIONS(1993),
    [sym_release_device] = ACTIONS(1993),
    [sym_release_channel] = ACTIONS(1993),
    [sym_release_devchan] = ACTIONS(1993),
    [sym_master_slot] = ACTIONS(1993),
    [sym_get_pulse_time] = ACTIONS(1993),
    [sym_get_timer] = ACTIONS(1993),
    [sym_master_sn] = ACTIONS(1993),
    [sym_system_number] = ACTIONS(1993),
    [sym_button] = ACTIONS(1993),
    [sym_dv_channel] = ACTIONS(1993),
    [sym_data] = ACTIONS(1993),
    [sym_level] = ACTIONS(1993),
    [sym_timeline] = ACTIONS(1993),
    [sym_channel] = ACTIONS(1993),
    [sym_custom] = ACTIONS(1993),
    [sym_atoi] = ACTIONS(1993),
    [sym_atol] = ACTIONS(1993),
    [sym_atof] = ACTIONS(1993),
    [sym_itoa] = ACTIONS(1993),
    [sym_itohex] = ACTIONS(1993),
    [sym_hextoi] = ACTIONS(1993),
    [sym_ftoa] = ACTIONS(1993),
    [sym_find_string] = ACTIONS(1993),
    [sym_left_string] = ACTIONS(1993),
    [sym_right_string] = ACTIONS(1993),
    [sym_mid_string] = ACTIONS(1993),
    [sym_remove_string] = ACTIONS(1993),
    [sym_get_buffer_string] = ACTIONS(1993),
    [sym_get_buffer_char] = ACTIONS(1993),
    [sym_get_multi_buffer_string] = ACTIONS(1993),
    [sym_length_string] = ACTIONS(1993),
    [sym_max_length_string] = ACTIONS(1993),
    [sym_set_length_string] = ACTIONS(1993),
    [sym_compare_string] = ACTIONS(1993),
    [sym_upper_string] = ACTIONS(1993),
    [sym_lower_string] = ACTIONS(1993),
    [sym_get_last] = ACTIONS(1993),
    [sym_date_to_day] = ACTIONS(1993),
    [sym_date_to_month] = ACTIONS(1993),
    [sym_date_to_year] = ACTIONS(1993),
    [sym_time_to_hour] = ACTIONS(1993),
    [sym_time_to_minute] = ACTIONS(1993),
    [sym_time_to_second] = ACTIONS(1993),
    [sym_day_of_week] = ACTIONS(1993),
    [sym_set_pulse_time] = ACTIONS(1993),
    [sym_set_timer] = ACTIONS(1993),
    [sym_file_open] = ACTIONS(1993),
    [sym_file_close] = ACTIONS(1993),
    [sym_file_read] = ACTIONS(1993),
    [sym_file_read_line] = ACTIONS(1993),
    [sym_file_write] = ACTIONS(1993),
    [sym_file_write_line] = ACTIONS(1993),
    [sym_file_copy] = ACTIONS(1993),
    [sym_file_delete] = ACTIONS(1993),
    [sym_file_rename] = ACTIONS(1993),
    [sym_file_dir] = ACTIONS(1993),
    [sym_file_getdir] = ACTIONS(1993),
    [sym_file_setdir] = ACTIONS(1993),
    [sym_file_create_dir] = ACTIONS(1993),
    [sym_file_remove_dir] = ACTIONS(1993),
    [sym_file_seek] = ACTIONS(1993),
    [sym_length_array] = ACTIONS(1993),
    [sym_max_length_array] = ACTIONS(1993),
    [sym_set_length_array] = ACTIONS(1993),
    [sym_ip_client_open] = ACTIONS(1993),
    [sym_ip_client_close] = ACTIONS(1993),
    [sym_ip_server_open] = ACTIONS(1993),
    [sym_ip_server_close] = ACTIONS(1993),
    [sym_ip_bound_client_open] = ACTIONS(1993),
    [sym_ip_mc_server_open] = ACTIONS(1993),
    [sym_ip_set_option] = ACTIONS(1993),
    [sym_ssh_client_open] = ACTIONS(1993),
    [sym_ssh_client_close] = ACTIONS(1993),
    [sym_tls_client_open] = ACTIONS(1993),
    [sym_tls_client_close] = ACTIONS(1993),
    [sym_do_push] = ACTIONS(1993),
    [sym_do_push_timed] = ACTIONS(1993),
    [sym_do_release] = ACTIONS(1993),
    [sym_device_id] = ACTIONS(1993),
    [sym_device_info] = ACTIONS(1993),
    [sym_device_id_string] = ACTIONS(1993),
    [sym_device_standby] = ACTIONS(1993),
    [sym_device_wake] = ACTIONS(1993),
    [sym_timeline_create] = ACTIONS(1993),
    [sym_timeline_kill] = ACTIONS(1993),
    [sym_timeline_reload] = ACTIONS(1993),
    [sym_timeline_pause] = ACTIONS(1993),
    [sym_timeline_restart] = ACTIONS(1993),
    [sym_timeline_set] = ACTIONS(1993),
    [sym_timeline_get] = ACTIONS(1993),
    [sym_timeline_active] = ACTIONS(1993),
    [sym_timeline_dynamic_id] = ACTIONS(1993),
    [sym_variable_to_string] = ACTIONS(1993),
    [sym_string_to_variable] = ACTIONS(1993),
    [sym_length_variable_to_string] = ACTIONS(1993),
    [sym_type_cast] = ACTIONS(1993),
    [sym_variable_to_xml] = ACTIONS(1993),
    [sym_xml_to_variable] = ACTIONS(1993),
    [sym_length_variable_to_xml] = ACTIONS(1993),
    [sym_redirect_string] = ACTIONS(1993),
    [sym_random_number] = ACTIONS(1993),
    [sym_get_unique_id] = ACTIONS(1993),
    [sym_get_system_number] = ACTIONS(1993),
    [sym_set_system_number] = ACTIONS(1993),
    [sym_get_serial_number] = ACTIONS(1993),
    [sym_get_master_build] = ACTIONS(1993),
    [sym_set_validation_code] = ACTIONS(1993),
    [sym_reboot] = ACTIONS(1993),
    [sym_load_duet_module] = ACTIONS(1993),
    [sym_unload_duet_module] = ACTIONS(1993),
    [sym_duet_mem_size_set] = ACTIONS(1993),
    [sym_duet_mem_size_get] = ACTIONS(1993),
    [sym_dynamic_application_device] = ACTIONS(1993),
    [sym_dynamic_polled_port] = ACTIONS(1993),
    [sym_static_port_binding] = ACTIONS(1993),
    [sym_static_ip_binding] = ACTIONS(1993),
    [sym_get_dns_list] = ACTIONS(1993),
    [sym_set_dns_list] = ACTIONS(1993),
    [sym_get_ip_address] = ACTIONS(1993),
    [sym_set_ip_address] = ACTIONS(1993),
    [sym_get_url_list] = ACTIONS(1993),
    [sym_add_url_entry] = ACTIONS(1993),
    [sym_delete_url_entry] = ACTIONS(1993),
    [sym_abs_value] = ACTIONS(1993),
    [sym_max_value] = ACTIONS(1993),
    [sym_min_value] = ACTIONS(1993),
    [sym_exp_value] = ACTIONS(1993),
    [sym_log_value] = ACTIONS(1993),
    [sym_log10_value] = ACTIONS(1993),
    [sym_power_value] = ACTIONS(1993),
    [sym_sqrt_value] = ACTIONS(1993),
    [sym_raw_be] = ACTIONS(1993),
    [sym_raw_le] = ACTIONS(1993),
    [sym_internal_queue_size_set] = ACTIONS(1993),
    [sym_internal_queue_size_get] = ACTIONS(1993),
    [sym_internal_threshold_set] = ACTIONS(1993),
    [sym_internal_threshold_get] = ACTIONS(1993),
    [sym_rebuilt_event] = ACTIONS(1993),
    [sym_do_custom_event] = ACTIONS(1993),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1993),
    [sym_clkmgr_set_clk_source] = ACTIONS(1993),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1993),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1993),
    [sym_clkmgr_get_timezone] = ACTIONS(1993),
    [sym_clkmgr_set_timezone] = ACTIONS(1993),
    [sym_clkmgr_get_resync_period] = ACTIONS(1993),
    [sym_clkmgr_set_resync_period] = ACTIONS(1993),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1993),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1993),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1993),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1993),
    [sym_clkmgr_get_timeservers] = ACTIONS(1993),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1993),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1993),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1993),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1993),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1993),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1993),
    [sym_set_log_level] = ACTIONS(1993),
    [sym_get_log_level] = ACTIONS(1993),
    [sym_amx_log] = ACTIONS(1993),
    [sym_smtp_server_config_set] = ACTIONS(1993),
    [sym_smtp_server_config_get] = ACTIONS(1993),
    [sym_smtp_send] = ACTIONS(1993),
    [sym_validate_netlinx_account] = ACTIONS(1993),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1993),
    [sym_authenticate_certificate] = ACTIONS(1993),
    [sym_audit_netlinx_session_event] = ACTIONS(1993),
    [sym_audit_netlinx_generic_event] = ACTIONS(1993),
    [sym_get_available_flash_disk_space] = ACTIONS(1993),
    [sym_get_max_flash_disk_space] = ACTIONS(1993),
    [sym_data_create_feed] = ACTIONS(1993),
    [sym_data_delete_feed] = ACTIONS(1993),
    [sym_data_publish_feed] = ACTIONS(1993),
    [sym_data_get_published_feed] = ACTIONS(1993),
    [sym_data_add_record] = ACTIONS(1993),
    [sym_data_get_event_record] = ACTIONS(1993),
    [sym__wc_data_create_feed] = ACTIONS(1993),
    [sym__wc_data_add_record] = ACTIONS(1993),
    [sym__wc_data_get_event_record] = ACTIONS(1993),
    [sym_format] = ACTIONS(1993),
    [sym_astro_clock] = ACTIONS(1993),
    [sym_set_outdoor_temperature] = ACTIONS(1993),
    [sym_set_virtual_level_count] = ACTIONS(1993),
    [sym_set_virtual_channel_count] = ACTIONS(1993),
    [sym_set_virtual_port_count] = ACTIONS(1993),
    [sym__wc_encode_int] = ACTIONS(1993),
    [sym__wc_decode_int] = ACTIONS(1993),
    [sym__wc_int] = ACTIONS(1993),
    [sym__wc_to_ch_int] = ACTIONS(1993),
    [sym__ch_to_wc_int] = ACTIONS(1993),
    [sym__wc_find_string_int] = ACTIONS(1993),
    [sym__wc_left_string_int] = ACTIONS(1993),
    [sym__wc_lower_string_int] = ACTIONS(1993),
    [sym__wc_mid_string_int] = ACTIONS(1993),
    [sym__wc_remove_string_int] = ACTIONS(1993),
    [sym__wc_right_string_int] = ACTIONS(1993),
    [sym__wc_upper_string_int] = ACTIONS(1993),
    [sym__wc_compare_string_int] = ACTIONS(1993),
    [sym__wc_get_buffer_char_int] = ACTIONS(1993),
    [sym__wc_get_buffer_string_int] = ACTIONS(1993),
    [sym__wc_concat_string_int] = ACTIONS(1993),
    [sym__wc_file_open_int] = ACTIONS(1993),
    [sym__wc_file_close_int] = ACTIONS(1993),
    [sym__wc_file_read_int] = ACTIONS(1993),
    [sym__wc_file_read_line_int] = ACTIONS(1993),
    [sym__wc_file_write_int] = ACTIONS(1993),
    [sym__wc_file_write_line_int] = ACTIONS(1993),
    [aux_sym_preproc_include_token1] = ACTIONS(1993),
    [aux_sym_preproc_define_token1] = ACTIONS(1995),
    [aux_sym_preproc_warn_token1] = ACTIONS(1995),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1995),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1995),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1995),
    [anon_sym_LBRACK] = ACTIONS(1995),
    [anon_sym_LBRACE] = ACTIONS(1995),
    [anon_sym_LPAREN] = ACTIONS(1993),
    [anon_sym_BANG] = ACTIONS(1995),
    [anon_sym_TILDE] = ACTIONS(1995),
    [anon_sym_DASH] = ACTIONS(1993),
    [anon_sym_PLUS] = ACTIONS(1993),
    [anon_sym_DASH_DASH] = ACTIONS(1995),
    [anon_sym_PLUS_PLUS] = ACTIONS(1995),
    [anon_sym_DQUOTE] = ACTIONS(1995),
    [anon_sym_SQUOTE] = ACTIONS(1995),
    [sym_number_literal] = ACTIONS(1995),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(433)] = {
    [ts_builtin_sym_end] = ACTIONS(1999),
    [sym_identifier] = ACTIONS(1997),
    [sym_program_name_keyword] = ACTIONS(1997),
    [sym_module_name_keyword] = ACTIONS(1997),
    [sym_define_device_keyword] = ACTIONS(1997),
    [sym_define_combine_keyword] = ACTIONS(1997),
    [sym_define_constant_keyword] = ACTIONS(1997),
    [sym_define_type_keyword] = ACTIONS(1997),
    [sym_define_variable_keyword] = ACTIONS(1997),
    [sym_define_system_variable_keyword] = ACTIONS(1997),
    [sym_define_connect_level_keyword] = ACTIONS(1997),
    [sym_define_latching_keyword] = ACTIONS(1997),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1997),
    [sym_define_toggling_keyword] = ACTIONS(1997),
    [sym_define_call_keyword] = ACTIONS(1997),
    [sym_define_function_keyword] = ACTIONS(1997),
    [sym_define_library_function_keyword] = ACTIONS(1997),
    [sym_define_start_keyword] = ACTIONS(1997),
    [sym_define_event_keyword] = ACTIONS(1997),
    [sym_define_program_keyword] = ACTIONS(1997),
    [sym_define_module_keyword] = ACTIONS(1997),
    [sym_char_keyword] = ACTIONS(1997),
    [sym_widechar_keyword] = ACTIONS(1997),
    [sym_integer_keyword] = ACTIONS(1997),
    [sym_sinteger_keyword] = ACTIONS(1997),
    [sym_long_keyword] = ACTIONS(1997),
    [sym_slong_keyword] = ACTIONS(1997),
    [sym_float_keyword] = ACTIONS(1997),
    [sym_double_keyword] = ACTIONS(1997),
    [sym_variant_keyword] = ACTIONS(1997),
    [sym_variantarray_keyword] = ACTIONS(1997),
    [sym_local_var_keyword] = ACTIONS(1997),
    [sym_stack_var_keyword] = ACTIONS(1997),
    [sym_constant_keyword] = ACTIONS(1997),
    [sym_volatile_keyword] = ACTIONS(1997),
    [sym_non_volatile_keyword] = ACTIONS(1997),
    [sym_persistent_keyword] = ACTIONS(1997),
    [sym_dev_keyword] = ACTIONS(1997),
    [sym_devlev_keyword] = ACTIONS(1997),
    [sym_devchan_keyword] = ACTIONS(1997),
    [sym_if_keyword] = ACTIONS(1997),
    [sym_switch_keyword] = ACTIONS(1997),
    [sym_case_keyword] = ACTIONS(1997),
    [sym_default_keyword] = ACTIONS(1997),
    [sym_while_keyword] = ACTIONS(1997),
    [sym_for_keyword] = ACTIONS(1997),
    [sym_break_keyword] = ACTIONS(1997),
    [sym_continue_keyword] = ACTIONS(1997),
    [sym_return_keyword] = ACTIONS(1997),
    [sym_select_keyword] = ACTIONS(1997),
    [sym_active_keyword] = ACTIONS(1997),
    [sym_struct_keyword] = ACTIONS(1997),
    [sym_structure_keyword] = ACTIONS(1997),
    [sym_button_event_keyword] = ACTIONS(1997),
    [sym_channel_event_keyword] = ACTIONS(1997),
    [sym_level_event_keyword] = ACTIONS(1997),
    [sym_data_event_keyword] = ACTIONS(1997),
    [sym_timeline_event_keyword] = ACTIONS(1997),
    [sym_custom_event_keyword] = ACTIONS(1997),
    [sym_push_keyword] = ACTIONS(1997),
    [sym_release_keyword] = ACTIONS(1997),
    [aux_sym_on_keyword_token1] = ACTIONS(1997),
    [aux_sym_off_keyword_token1] = ACTIONS(1997),
    [sym_send_level_keyword] = ACTIONS(1997),
    [sym_send_string_keyword] = ACTIONS(1997),
    [sym_send_command_keyword] = ACTIONS(1997),
    [sym_clear_buffer_keyword] = ACTIONS(1997),
    [sym_create_buffer_keyword] = ACTIONS(1997),
    [sym_create_multi_buffer_keyword] = ACTIONS(1997),
    [sym_call_keyword] = ACTIONS(1997),
    [sym_system_call_keyword] = ACTIONS(1997),
    [sym_devchan_to_keyword] = ACTIONS(1997),
    [sym_devchan_min_to_keyword] = ACTIONS(1997),
    [sym_devchan_total_off_keyword] = ACTIONS(1997),
    [sym_devchan_pulse_keyword] = ACTIONS(1997),
    [sym_wait_keyword] = ACTIONS(1997),
    [sym_cancel_wait_keyword] = ACTIONS(1997),
    [sym_wait_until_keyword] = ACTIONS(1997),
    [sym_cancel_wait_until_keyword] = ACTIONS(1997),
    [sym_cancel_all_wait_keyword] = ACTIONS(1997),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1997),
    [sym_not] = ACTIONS(1997),
    [sym_bnot] = ACTIONS(1997),
    [sym___file__] = ACTIONS(1997),
    [sym___date__] = ACTIONS(1997),
    [sym___ldate__] = ACTIONS(1997),
    [sym___time__] = ACTIONS(1997),
    [sym___line__] = ACTIONS(1997),
    [sym___name__] = ACTIONS(1997),
    [sym___netlinx__] = ACTIONS(1997),
    [sym_netlinx_axi_version] = ACTIONS(1997),
    [sym_url_flg_tcp] = ACTIONS(1997),
    [sym_url_flg_accinfopresent] = ACTIONS(1997),
    [sym_url_flg_temp] = ACTIONS(1997),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1997),
    [sym_url_flg_stat_mask] = ACTIONS(1997),
    [sym_url_flg_stat_lookup] = ACTIONS(1997),
    [sym_url_flg_stat_connecting] = ACTIONS(1997),
    [sym_url_flg_stat_waiting] = ACTIONS(1997),
    [sym_url_flg_stat_connected] = ACTIONS(1997),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1997),
    [sym_first_virtual_device] = ACTIONS(1997),
    [sym_first_local_port] = ACTIONS(1997),
    [sym_dynamic_virtual_device] = ACTIONS(1997),
    [sym_timeline_once] = ACTIONS(1997),
    [sym_timeline_repeat] = ACTIONS(1997),
    [sym_timeline_absolute] = ACTIONS(1997),
    [sym_timeline_relative] = ACTIONS(1997),
    [sym_do_push_timed_infinite] = ACTIONS(1997),
    [sym_source_type_no_address] = ACTIONS(1997),
    [sym_source_type_neuron_id] = ACTIONS(1997),
    [sym_source_type_ip_address] = ACTIONS(1997),
    [sym_source_type_axlink] = ACTIONS(1997),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1997),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1997),
    [sym_source_type_ip_socket_address] = ACTIONS(1997),
    [sym_source_type_rs232] = ACTIONS(1997),
    [sym_source_type_internal] = ACTIONS(1997),
    [sym_source_type_mac_address] = ACTIONS(1997),
    [sym_source_type_ipv4_port] = ACTIONS(1997),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1997),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1997),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1997),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1997),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1997),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1997),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1997),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1997),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1997),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1997),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1997),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1997),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1997),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1997),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1997),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1997),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1997),
    [sym_true] = ACTIONS(1997),
    [sym_false] = ACTIONS(1997),
    [sym_file_read_only] = ACTIONS(1997),
    [sym_file_rw_new] = ACTIONS(1997),
    [sym_file_rw_append] = ACTIONS(1997),
    [sym_ip_tcp] = ACTIONS(1997),
    [sym_ip_udp] = ACTIONS(1997),
    [sym_ip_udp_2way] = ACTIONS(1997),
    [sym_xml_encode_types] = ACTIONS(1997),
    [sym_xml_encode_char_as_list] = ACTIONS(1997),
    [sym_xml_encode_le] = ACTIONS(1997),
    [sym_xml_decode_types] = ACTIONS(1997),
    [sym_xml_decode_no_preserve] = ACTIONS(1997),
    [sym_duet_dev_type_display_device] = ACTIONS(1997),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1997),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1997),
    [sym_duet_dev_type_io_device] = ACTIONS(1997),
    [sym_duet_dev_type_relay_device] = ACTIONS(1997),
    [sym_duet_dev_type_ups] = ACTIONS(1997),
    [sym_duet_dev_type_amplifier] = ACTIONS(1997),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1997),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1997),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1997),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1997),
    [sym_duet_dev_type_camera] = ACTIONS(1997),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1997),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1997),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1997),
    [sym_duet_dev_type_dss] = ACTIONS(1997),
    [sym_duet_dev_type_dvr] = ACTIONS(1997),
    [sym_duet_dev_type_disc_device] = ACTIONS(1997),
    [sym_duet_dev_type_document_camera] = ACTIONS(1997),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1997),
    [sym_duet_dev_type_hvac] = ACTIONS(1997),
    [sym_duet_dev_type_keypad] = ACTIONS(1997),
    [sym_duet_dev_type_light] = ACTIONS(1997),
    [sym_duet_dev_type_monitor] = ACTIONS(1997),
    [sym_duet_dev_type_motor] = ACTIONS(1997),
    [sym_duet_dev_type_multi_window] = ACTIONS(1997),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1997),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1997),
    [sym_duet_dev_type_receiver] = ACTIONS(1997),
    [sym_duet_dev_type_security_system] = ACTIONS(1997),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1997),
    [sym_duet_dev_type_settop_box] = ACTIONS(1997),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1997),
    [sym_duet_dev_type_switcher] = ACTIONS(1997),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1997),
    [sym_duet_dev_type_tv] = ACTIONS(1997),
    [sym_duet_dev_type_utility] = ACTIONS(1997),
    [sym_duet_dev_type_vcr] = ACTIONS(1997),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1997),
    [sym_duet_dev_type_video_processor] = ACTIONS(1997),
    [sym_duet_dev_type_video_projector] = ACTIONS(1997),
    [sym_duet_dev_type_video_wall] = ACTIONS(1997),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1997),
    [sym_duet_dev_type_weather] = ACTIONS(1997),
    [sym_duet_dev_not_polled] = ACTIONS(1997),
    [sym_duet_dev_polled] = ACTIONS(1997),
    [sym_clkmgr_mode_network] = ACTIONS(1997),
    [sym_clkmgr_mode_standalone] = ACTIONS(1997),
    [sym_normal_standby] = ACTIONS(1997),
    [sym_normal_wake] = ACTIONS(1997),
    [sym_ip_multicast_ttl_option] = ACTIONS(1997),
    [sym_ip_ttl_subnet] = ACTIONS(1997),
    [sym_ip_ttl_site] = ACTIONS(1997),
    [sym_ip_ttl_region] = ACTIONS(1997),
    [sym_ip_ttl_continent] = ACTIONS(1997),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1997),
    [sym_ip_nodelay_on] = ACTIONS(1997),
    [sym_ip_nodelay_off] = ACTIONS(1997),
    [sym_amx_error] = ACTIONS(1997),
    [sym_amx_warning] = ACTIONS(1997),
    [sym_amx_info] = ACTIONS(1997),
    [sym_amx_debug] = ACTIONS(1997),
    [sym_netlinx_login_success] = ACTIONS(1997),
    [sym_netlinx_login_fail] = ACTIONS(1997),
    [sym_netlinx_logout] = ACTIONS(1997),
    [sym_null_str] = ACTIONS(1997),
    [sym_smtp_address] = ACTIONS(1997),
    [sym_smtp_port_number] = ACTIONS(1997),
    [sym_smtp_username] = ACTIONS(1997),
    [sym_smtp_password] = ACTIONS(1997),
    [sym_smtp_require_tls] = ACTIONS(1997),
    [sym_smtp_from] = ACTIONS(1997),
    [sym_smtp_tls_true] = ACTIONS(1997),
    [sym_smtp_tls_false] = ACTIONS(1997),
    [sym_listview_on_row_select_event] = ACTIONS(1997),
    [sym_data_structure_unknown] = ACTIONS(1997),
    [sym_data_structure_datafeed] = ACTIONS(1997),
    [sym_data_structure_datarecordset] = ACTIONS(1997),
    [sym_data_structure_datarecord] = ACTIONS(1997),
    [sym_data_structure_datafield] = ACTIONS(1997),
    [sym_data_max_content_field_count] = ACTIONS(1997),
    [sym_data_max_metadata_field_count] = ACTIONS(1997),
    [sym_data_max_id_length] = ACTIONS(1997),
    [sym_data_max_name_length] = ACTIONS(1997),
    [sym_data_max_description_length] = ACTIONS(1997),
    [sym_data_max_type_length] = ACTIONS(1997),
    [sym_data_max_format_length] = ACTIONS(1997),
    [sym_data_max_source_length] = ACTIONS(1997),
    [sym_data_max_value_length] = ACTIONS(1997),
    [sym_data_max_label_length] = ACTIONS(1997),
    [sym_data_max_url_length] = ACTIONS(1997),
    [sym_max_last_login_info_length] = ACTIONS(1997),
    [sym_data_type_unknown] = ACTIONS(1997),
    [sym_data_type_string] = ACTIONS(1997),
    [sym_data_type_datetime] = ACTIONS(1997),
    [sym_data_type_date] = ACTIONS(1997),
    [sym_data_type_time] = ACTIONS(1997),
    [sym_data_type_image] = ACTIONS(1997),
    [sym_data_format_url] = ACTIONS(1997),
    [sym_data_format_phone] = ACTIONS(1997),
    [sym_data_format_email] = ACTIONS(1997),
    [sym_data_format_iso8601] = ACTIONS(1997),
    [sym_auditlog_priv] = ACTIONS(1997),
    [sym_configuration_priv] = ACTIONS(1997),
    [sym_deviceconfig_priv] = ACTIONS(1997),
    [sym_ftp_priv] = ACTIONS(1997),
    [sym_http_priv] = ACTIONS(1997),
    [sym_networkconfig_priv] = ACTIONS(1997),
    [sym_programport_priv] = ACTIONS(1997),
    [sym_remoteui_priv] = ACTIONS(1997),
    [sym_securitycontrol_priv] = ACTIONS(1997),
    [sym_softwaremanagement_priv] = ACTIONS(1997),
    [sym_terminal_priv] = ACTIONS(1997),
    [sym_tpadmin_priv] = ACTIONS(1997),
    [sym_user1_priv] = ACTIONS(1997),
    [sym_user2_priv] = ACTIONS(1997),
    [sym_user3_priv] = ACTIONS(1997),
    [sym_user4_priv] = ACTIONS(1997),
    [sym_usermanagement_priv] = ACTIONS(1997),
    [sym_valid_account] = ACTIONS(1997),
    [sym_err_name_invalid] = ACTIONS(1997),
    [sym_err_password_invalid] = ACTIONS(1997),
    [sym_err_invalid_account] = ACTIONS(1997),
    [sym_err_no_authorization] = ACTIONS(1997),
    [sym_err_invalid_parameter] = ACTIONS(1997),
    [sym_err_user_locked_out] = ACTIONS(1997),
    [sym_err_user_account_expired] = ACTIONS(1997),
    [sym_err_general_error] = ACTIONS(1997),
    [sym_err_authentication_error] = ACTIONS(1997),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1997),
    [sym_tls_validate_certificate] = ACTIONS(1997),
    [sym_tdata] = ACTIONS(1997),
    [sym_tchannel] = ACTIONS(1997),
    [sym_tlevel] = ACTIONS(1997),
    [sym_tbutton] = ACTIONS(1997),
    [sym_ttimeline] = ACTIONS(1997),
    [sym_tcustom] = ACTIONS(1997),
    [sym_url_struct] = ACTIONS(1997),
    [sym_dns_struct] = ACTIONS(1997),
    [sym_ip_address_struct] = ACTIONS(1997),
    [sym_dev_info_struct] = ACTIONS(1997),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1997),
    [sym_data_feed] = ACTIONS(1997),
    [sym_data_field] = ACTIONS(1997),
    [sym_data_record] = ACTIONS(1997),
    [sym_wc_data_feed] = ACTIONS(1997),
    [sym_wc_data_field] = ACTIONS(1997),
    [sym_wc_data_record] = ACTIONS(1997),
    [sym_last_login_info] = ACTIONS(1997),
    [sym____reserved___] = ACTIONS(1997),
    [sym_date] = ACTIONS(1997),
    [sym_day] = ACTIONS(1997),
    [sym_time] = ACTIONS(1997),
    [sym_ldate] = ACTIONS(1997),
    [sym_push_device] = ACTIONS(1997),
    [sym_push_channel] = ACTIONS(1997),
    [sym_push_devchan] = ACTIONS(1997),
    [sym_release_device] = ACTIONS(1997),
    [sym_release_channel] = ACTIONS(1997),
    [sym_release_devchan] = ACTIONS(1997),
    [sym_master_slot] = ACTIONS(1997),
    [sym_get_pulse_time] = ACTIONS(1997),
    [sym_get_timer] = ACTIONS(1997),
    [sym_master_sn] = ACTIONS(1997),
    [sym_system_number] = ACTIONS(1997),
    [sym_button] = ACTIONS(1997),
    [sym_dv_channel] = ACTIONS(1997),
    [sym_data] = ACTIONS(1997),
    [sym_level] = ACTIONS(1997),
    [sym_timeline] = ACTIONS(1997),
    [sym_channel] = ACTIONS(1997),
    [sym_custom] = ACTIONS(1997),
    [sym_atoi] = ACTIONS(1997),
    [sym_atol] = ACTIONS(1997),
    [sym_atof] = ACTIONS(1997),
    [sym_itoa] = ACTIONS(1997),
    [sym_itohex] = ACTIONS(1997),
    [sym_hextoi] = ACTIONS(1997),
    [sym_ftoa] = ACTIONS(1997),
    [sym_find_string] = ACTIONS(1997),
    [sym_left_string] = ACTIONS(1997),
    [sym_right_string] = ACTIONS(1997),
    [sym_mid_string] = ACTIONS(1997),
    [sym_remove_string] = ACTIONS(1997),
    [sym_get_buffer_string] = ACTIONS(1997),
    [sym_get_buffer_char] = ACTIONS(1997),
    [sym_get_multi_buffer_string] = ACTIONS(1997),
    [sym_length_string] = ACTIONS(1997),
    [sym_max_length_string] = ACTIONS(1997),
    [sym_set_length_string] = ACTIONS(1997),
    [sym_compare_string] = ACTIONS(1997),
    [sym_upper_string] = ACTIONS(1997),
    [sym_lower_string] = ACTIONS(1997),
    [sym_get_last] = ACTIONS(1997),
    [sym_date_to_day] = ACTIONS(1997),
    [sym_date_to_month] = ACTIONS(1997),
    [sym_date_to_year] = ACTIONS(1997),
    [sym_time_to_hour] = ACTIONS(1997),
    [sym_time_to_minute] = ACTIONS(1997),
    [sym_time_to_second] = ACTIONS(1997),
    [sym_day_of_week] = ACTIONS(1997),
    [sym_set_pulse_time] = ACTIONS(1997),
    [sym_set_timer] = ACTIONS(1997),
    [sym_file_open] = ACTIONS(1997),
    [sym_file_close] = ACTIONS(1997),
    [sym_file_read] = ACTIONS(1997),
    [sym_file_read_line] = ACTIONS(1997),
    [sym_file_write] = ACTIONS(1997),
    [sym_file_write_line] = ACTIONS(1997),
    [sym_file_copy] = ACTIONS(1997),
    [sym_file_delete] = ACTIONS(1997),
    [sym_file_rename] = ACTIONS(1997),
    [sym_file_dir] = ACTIONS(1997),
    [sym_file_getdir] = ACTIONS(1997),
    [sym_file_setdir] = ACTIONS(1997),
    [sym_file_create_dir] = ACTIONS(1997),
    [sym_file_remove_dir] = ACTIONS(1997),
    [sym_file_seek] = ACTIONS(1997),
    [sym_length_array] = ACTIONS(1997),
    [sym_max_length_array] = ACTIONS(1997),
    [sym_set_length_array] = ACTIONS(1997),
    [sym_ip_client_open] = ACTIONS(1997),
    [sym_ip_client_close] = ACTIONS(1997),
    [sym_ip_server_open] = ACTIONS(1997),
    [sym_ip_server_close] = ACTIONS(1997),
    [sym_ip_bound_client_open] = ACTIONS(1997),
    [sym_ip_mc_server_open] = ACTIONS(1997),
    [sym_ip_set_option] = ACTIONS(1997),
    [sym_ssh_client_open] = ACTIONS(1997),
    [sym_ssh_client_close] = ACTIONS(1997),
    [sym_tls_client_open] = ACTIONS(1997),
    [sym_tls_client_close] = ACTIONS(1997),
    [sym_do_push] = ACTIONS(1997),
    [sym_do_push_timed] = ACTIONS(1997),
    [sym_do_release] = ACTIONS(1997),
    [sym_device_id] = ACTIONS(1997),
    [sym_device_info] = ACTIONS(1997),
    [sym_device_id_string] = ACTIONS(1997),
    [sym_device_standby] = ACTIONS(1997),
    [sym_device_wake] = ACTIONS(1997),
    [sym_timeline_create] = ACTIONS(1997),
    [sym_timeline_kill] = ACTIONS(1997),
    [sym_timeline_reload] = ACTIONS(1997),
    [sym_timeline_pause] = ACTIONS(1997),
    [sym_timeline_restart] = ACTIONS(1997),
    [sym_timeline_set] = ACTIONS(1997),
    [sym_timeline_get] = ACTIONS(1997),
    [sym_timeline_active] = ACTIONS(1997),
    [sym_timeline_dynamic_id] = ACTIONS(1997),
    [sym_variable_to_string] = ACTIONS(1997),
    [sym_string_to_variable] = ACTIONS(1997),
    [sym_length_variable_to_string] = ACTIONS(1997),
    [sym_type_cast] = ACTIONS(1997),
    [sym_variable_to_xml] = ACTIONS(1997),
    [sym_xml_to_variable] = ACTIONS(1997),
    [sym_length_variable_to_xml] = ACTIONS(1997),
    [sym_redirect_string] = ACTIONS(1997),
    [sym_random_number] = ACTIONS(1997),
    [sym_get_unique_id] = ACTIONS(1997),
    [sym_get_system_number] = ACTIONS(1997),
    [sym_set_system_number] = ACTIONS(1997),
    [sym_get_serial_number] = ACTIONS(1997),
    [sym_get_master_build] = ACTIONS(1997),
    [sym_set_validation_code] = ACTIONS(1997),
    [sym_reboot] = ACTIONS(1997),
    [sym_load_duet_module] = ACTIONS(1997),
    [sym_unload_duet_module] = ACTIONS(1997),
    [sym_duet_mem_size_set] = ACTIONS(1997),
    [sym_duet_mem_size_get] = ACTIONS(1997),
    [sym_dynamic_application_device] = ACTIONS(1997),
    [sym_dynamic_polled_port] = ACTIONS(1997),
    [sym_static_port_binding] = ACTIONS(1997),
    [sym_static_ip_binding] = ACTIONS(1997),
    [sym_get_dns_list] = ACTIONS(1997),
    [sym_set_dns_list] = ACTIONS(1997),
    [sym_get_ip_address] = ACTIONS(1997),
    [sym_set_ip_address] = ACTIONS(1997),
    [sym_get_url_list] = ACTIONS(1997),
    [sym_add_url_entry] = ACTIONS(1997),
    [sym_delete_url_entry] = ACTIONS(1997),
    [sym_abs_value] = ACTIONS(1997),
    [sym_max_value] = ACTIONS(1997),
    [sym_min_value] = ACTIONS(1997),
    [sym_exp_value] = ACTIONS(1997),
    [sym_log_value] = ACTIONS(1997),
    [sym_log10_value] = ACTIONS(1997),
    [sym_power_value] = ACTIONS(1997),
    [sym_sqrt_value] = ACTIONS(1997),
    [sym_raw_be] = ACTIONS(1997),
    [sym_raw_le] = ACTIONS(1997),
    [sym_internal_queue_size_set] = ACTIONS(1997),
    [sym_internal_queue_size_get] = ACTIONS(1997),
    [sym_internal_threshold_set] = ACTIONS(1997),
    [sym_internal_threshold_get] = ACTIONS(1997),
    [sym_rebuilt_event] = ACTIONS(1997),
    [sym_do_custom_event] = ACTIONS(1997),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1997),
    [sym_clkmgr_set_clk_source] = ACTIONS(1997),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1997),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1997),
    [sym_clkmgr_get_timezone] = ACTIONS(1997),
    [sym_clkmgr_set_timezone] = ACTIONS(1997),
    [sym_clkmgr_get_resync_period] = ACTIONS(1997),
    [sym_clkmgr_set_resync_period] = ACTIONS(1997),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1997),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1997),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1997),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1997),
    [sym_clkmgr_get_timeservers] = ACTIONS(1997),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1997),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1997),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1997),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1997),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1997),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1997),
    [sym_set_log_level] = ACTIONS(1997),
    [sym_get_log_level] = ACTIONS(1997),
    [sym_amx_log] = ACTIONS(1997),
    [sym_smtp_server_config_set] = ACTIONS(1997),
    [sym_smtp_server_config_get] = ACTIONS(1997),
    [sym_smtp_send] = ACTIONS(1997),
    [sym_validate_netlinx_account] = ACTIONS(1997),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1997),
    [sym_authenticate_certificate] = ACTIONS(1997),
    [sym_audit_netlinx_session_event] = ACTIONS(1997),
    [sym_audit_netlinx_generic_event] = ACTIONS(1997),
    [sym_get_available_flash_disk_space] = ACTIONS(1997),
    [sym_get_max_flash_disk_space] = ACTIONS(1997),
    [sym_data_create_feed] = ACTIONS(1997),
    [sym_data_delete_feed] = ACTIONS(1997),
    [sym_data_publish_feed] = ACTIONS(1997),
    [sym_data_get_published_feed] = ACTIONS(1997),
    [sym_data_add_record] = ACTIONS(1997),
    [sym_data_get_event_record] = ACTIONS(1997),
    [sym__wc_data_create_feed] = ACTIONS(1997),
    [sym__wc_data_add_record] = ACTIONS(1997),
    [sym__wc_data_get_event_record] = ACTIONS(1997),
    [sym_format] = ACTIONS(1997),
    [sym_astro_clock] = ACTIONS(1997),
    [sym_set_outdoor_temperature] = ACTIONS(1997),
    [sym_set_virtual_level_count] = ACTIONS(1997),
    [sym_set_virtual_channel_count] = ACTIONS(1997),
    [sym_set_virtual_port_count] = ACTIONS(1997),
    [sym__wc_encode_int] = ACTIONS(1997),
    [sym__wc_decode_int] = ACTIONS(1997),
    [sym__wc_int] = ACTIONS(1997),
    [sym__wc_to_ch_int] = ACTIONS(1997),
    [sym__ch_to_wc_int] = ACTIONS(1997),
    [sym__wc_find_string_int] = ACTIONS(1997),
    [sym__wc_left_string_int] = ACTIONS(1997),
    [sym__wc_lower_string_int] = ACTIONS(1997),
    [sym__wc_mid_string_int] = ACTIONS(1997),
    [sym__wc_remove_string_int] = ACTIONS(1997),
    [sym__wc_right_string_int] = ACTIONS(1997),
    [sym__wc_upper_string_int] = ACTIONS(1997),
    [sym__wc_compare_string_int] = ACTIONS(1997),
    [sym__wc_get_buffer_char_int] = ACTIONS(1997),
    [sym__wc_get_buffer_string_int] = ACTIONS(1997),
    [sym__wc_concat_string_int] = ACTIONS(1997),
    [sym__wc_file_open_int] = ACTIONS(1997),
    [sym__wc_file_close_int] = ACTIONS(1997),
    [sym__wc_file_read_int] = ACTIONS(1997),
    [sym__wc_file_read_line_int] = ACTIONS(1997),
    [sym__wc_file_write_int] = ACTIONS(1997),
    [sym__wc_file_write_line_int] = ACTIONS(1997),
    [aux_sym_preproc_include_token1] = ACTIONS(1997),
    [aux_sym_preproc_define_token1] = ACTIONS(1999),
    [aux_sym_preproc_warn_token1] = ACTIONS(1999),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1999),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1999),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1999),
    [anon_sym_LBRACK] = ACTIONS(1999),
    [anon_sym_LBRACE] = ACTIONS(1999),
    [anon_sym_LPAREN] = ACTIONS(1997),
    [anon_sym_BANG] = ACTIONS(1999),
    [anon_sym_TILDE] = ACTIONS(1999),
    [anon_sym_DASH] = ACTIONS(1997),
    [anon_sym_PLUS] = ACTIONS(1997),
    [anon_sym_DASH_DASH] = ACTIONS(1999),
    [anon_sym_PLUS_PLUS] = ACTIONS(1999),
    [anon_sym_DQUOTE] = ACTIONS(1999),
    [anon_sym_SQUOTE] = ACTIONS(1999),
    [sym_number_literal] = ACTIONS(1999),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(434)] = {
    [ts_builtin_sym_end] = ACTIONS(2035),
    [sym_identifier] = ACTIONS(2033),
    [sym_program_name_keyword] = ACTIONS(2033),
    [sym_module_name_keyword] = ACTIONS(2033),
    [sym_define_device_keyword] = ACTIONS(2033),
    [sym_define_combine_keyword] = ACTIONS(2033),
    [sym_define_constant_keyword] = ACTIONS(2033),
    [sym_define_type_keyword] = ACTIONS(2033),
    [sym_define_variable_keyword] = ACTIONS(2033),
    [sym_define_system_variable_keyword] = ACTIONS(2033),
    [sym_define_connect_level_keyword] = ACTIONS(2033),
    [sym_define_latching_keyword] = ACTIONS(2033),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2033),
    [sym_define_toggling_keyword] = ACTIONS(2033),
    [sym_define_call_keyword] = ACTIONS(2033),
    [sym_define_function_keyword] = ACTIONS(2033),
    [sym_define_library_function_keyword] = ACTIONS(2033),
    [sym_define_start_keyword] = ACTIONS(2033),
    [sym_define_event_keyword] = ACTIONS(2033),
    [sym_define_program_keyword] = ACTIONS(2033),
    [sym_define_module_keyword] = ACTIONS(2033),
    [sym_char_keyword] = ACTIONS(2033),
    [sym_widechar_keyword] = ACTIONS(2033),
    [sym_integer_keyword] = ACTIONS(2033),
    [sym_sinteger_keyword] = ACTIONS(2033),
    [sym_long_keyword] = ACTIONS(2033),
    [sym_slong_keyword] = ACTIONS(2033),
    [sym_float_keyword] = ACTIONS(2033),
    [sym_double_keyword] = ACTIONS(2033),
    [sym_variant_keyword] = ACTIONS(2033),
    [sym_variantarray_keyword] = ACTIONS(2033),
    [sym_local_var_keyword] = ACTIONS(2033),
    [sym_stack_var_keyword] = ACTIONS(2033),
    [sym_constant_keyword] = ACTIONS(2033),
    [sym_volatile_keyword] = ACTIONS(2033),
    [sym_non_volatile_keyword] = ACTIONS(2033),
    [sym_persistent_keyword] = ACTIONS(2033),
    [sym_dev_keyword] = ACTIONS(2033),
    [sym_devlev_keyword] = ACTIONS(2033),
    [sym_devchan_keyword] = ACTIONS(2033),
    [sym_if_keyword] = ACTIONS(2033),
    [sym_switch_keyword] = ACTIONS(2033),
    [sym_case_keyword] = ACTIONS(2033),
    [sym_default_keyword] = ACTIONS(2033),
    [sym_while_keyword] = ACTIONS(2033),
    [sym_for_keyword] = ACTIONS(2033),
    [sym_break_keyword] = ACTIONS(2033),
    [sym_continue_keyword] = ACTIONS(2033),
    [sym_return_keyword] = ACTIONS(2033),
    [sym_select_keyword] = ACTIONS(2033),
    [sym_active_keyword] = ACTIONS(2033),
    [sym_struct_keyword] = ACTIONS(2033),
    [sym_structure_keyword] = ACTIONS(2033),
    [sym_button_event_keyword] = ACTIONS(2033),
    [sym_channel_event_keyword] = ACTIONS(2033),
    [sym_level_event_keyword] = ACTIONS(2033),
    [sym_data_event_keyword] = ACTIONS(2033),
    [sym_timeline_event_keyword] = ACTIONS(2033),
    [sym_custom_event_keyword] = ACTIONS(2033),
    [sym_push_keyword] = ACTIONS(2033),
    [sym_release_keyword] = ACTIONS(2033),
    [aux_sym_on_keyword_token1] = ACTIONS(2033),
    [aux_sym_off_keyword_token1] = ACTIONS(2033),
    [sym_send_level_keyword] = ACTIONS(2033),
    [sym_send_string_keyword] = ACTIONS(2033),
    [sym_send_command_keyword] = ACTIONS(2033),
    [sym_clear_buffer_keyword] = ACTIONS(2033),
    [sym_create_buffer_keyword] = ACTIONS(2033),
    [sym_create_multi_buffer_keyword] = ACTIONS(2033),
    [sym_call_keyword] = ACTIONS(2033),
    [sym_system_call_keyword] = ACTIONS(2033),
    [sym_devchan_to_keyword] = ACTIONS(2033),
    [sym_devchan_min_to_keyword] = ACTIONS(2033),
    [sym_devchan_total_off_keyword] = ACTIONS(2033),
    [sym_devchan_pulse_keyword] = ACTIONS(2033),
    [sym_wait_keyword] = ACTIONS(2033),
    [sym_cancel_wait_keyword] = ACTIONS(2033),
    [sym_wait_until_keyword] = ACTIONS(2033),
    [sym_cancel_wait_until_keyword] = ACTIONS(2033),
    [sym_cancel_all_wait_keyword] = ACTIONS(2033),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2033),
    [sym_not] = ACTIONS(2033),
    [sym_bnot] = ACTIONS(2033),
    [sym___file__] = ACTIONS(2033),
    [sym___date__] = ACTIONS(2033),
    [sym___ldate__] = ACTIONS(2033),
    [sym___time__] = ACTIONS(2033),
    [sym___line__] = ACTIONS(2033),
    [sym___name__] = ACTIONS(2033),
    [sym___netlinx__] = ACTIONS(2033),
    [sym_netlinx_axi_version] = ACTIONS(2033),
    [sym_url_flg_tcp] = ACTIONS(2033),
    [sym_url_flg_accinfopresent] = ACTIONS(2033),
    [sym_url_flg_temp] = ACTIONS(2033),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2033),
    [sym_url_flg_stat_mask] = ACTIONS(2033),
    [sym_url_flg_stat_lookup] = ACTIONS(2033),
    [sym_url_flg_stat_connecting] = ACTIONS(2033),
    [sym_url_flg_stat_waiting] = ACTIONS(2033),
    [sym_url_flg_stat_connected] = ACTIONS(2033),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2033),
    [sym_first_virtual_device] = ACTIONS(2033),
    [sym_first_local_port] = ACTIONS(2033),
    [sym_dynamic_virtual_device] = ACTIONS(2033),
    [sym_timeline_once] = ACTIONS(2033),
    [sym_timeline_repeat] = ACTIONS(2033),
    [sym_timeline_absolute] = ACTIONS(2033),
    [sym_timeline_relative] = ACTIONS(2033),
    [sym_do_push_timed_infinite] = ACTIONS(2033),
    [sym_source_type_no_address] = ACTIONS(2033),
    [sym_source_type_neuron_id] = ACTIONS(2033),
    [sym_source_type_ip_address] = ACTIONS(2033),
    [sym_source_type_axlink] = ACTIONS(2033),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2033),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2033),
    [sym_source_type_ip_socket_address] = ACTIONS(2033),
    [sym_source_type_rs232] = ACTIONS(2033),
    [sym_source_type_internal] = ACTIONS(2033),
    [sym_source_type_mac_address] = ACTIONS(2033),
    [sym_source_type_ipv4_port] = ACTIONS(2033),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2033),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2033),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2033),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2033),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2033),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2033),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2033),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2033),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2033),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2033),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2033),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2033),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2033),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2033),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2033),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2033),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2033),
    [sym_true] = ACTIONS(2033),
    [sym_false] = ACTIONS(2033),
    [sym_file_read_only] = ACTIONS(2033),
    [sym_file_rw_new] = ACTIONS(2033),
    [sym_file_rw_append] = ACTIONS(2033),
    [sym_ip_tcp] = ACTIONS(2033),
    [sym_ip_udp] = ACTIONS(2033),
    [sym_ip_udp_2way] = ACTIONS(2033),
    [sym_xml_encode_types] = ACTIONS(2033),
    [sym_xml_encode_char_as_list] = ACTIONS(2033),
    [sym_xml_encode_le] = ACTIONS(2033),
    [sym_xml_decode_types] = ACTIONS(2033),
    [sym_xml_decode_no_preserve] = ACTIONS(2033),
    [sym_duet_dev_type_display_device] = ACTIONS(2033),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2033),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2033),
    [sym_duet_dev_type_io_device] = ACTIONS(2033),
    [sym_duet_dev_type_relay_device] = ACTIONS(2033),
    [sym_duet_dev_type_ups] = ACTIONS(2033),
    [sym_duet_dev_type_amplifier] = ACTIONS(2033),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2033),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2033),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2033),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2033),
    [sym_duet_dev_type_camera] = ACTIONS(2033),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2033),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2033),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2033),
    [sym_duet_dev_type_dss] = ACTIONS(2033),
    [sym_duet_dev_type_dvr] = ACTIONS(2033),
    [sym_duet_dev_type_disc_device] = ACTIONS(2033),
    [sym_duet_dev_type_document_camera] = ACTIONS(2033),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2033),
    [sym_duet_dev_type_hvac] = ACTIONS(2033),
    [sym_duet_dev_type_keypad] = ACTIONS(2033),
    [sym_duet_dev_type_light] = ACTIONS(2033),
    [sym_duet_dev_type_monitor] = ACTIONS(2033),
    [sym_duet_dev_type_motor] = ACTIONS(2033),
    [sym_duet_dev_type_multi_window] = ACTIONS(2033),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2033),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2033),
    [sym_duet_dev_type_receiver] = ACTIONS(2033),
    [sym_duet_dev_type_security_system] = ACTIONS(2033),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2033),
    [sym_duet_dev_type_settop_box] = ACTIONS(2033),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2033),
    [sym_duet_dev_type_switcher] = ACTIONS(2033),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2033),
    [sym_duet_dev_type_tv] = ACTIONS(2033),
    [sym_duet_dev_type_utility] = ACTIONS(2033),
    [sym_duet_dev_type_vcr] = ACTIONS(2033),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2033),
    [sym_duet_dev_type_video_processor] = ACTIONS(2033),
    [sym_duet_dev_type_video_projector] = ACTIONS(2033),
    [sym_duet_dev_type_video_wall] = ACTIONS(2033),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2033),
    [sym_duet_dev_type_weather] = ACTIONS(2033),
    [sym_duet_dev_not_polled] = ACTIONS(2033),
    [sym_duet_dev_polled] = ACTIONS(2033),
    [sym_clkmgr_mode_network] = ACTIONS(2033),
    [sym_clkmgr_mode_standalone] = ACTIONS(2033),
    [sym_normal_standby] = ACTIONS(2033),
    [sym_normal_wake] = ACTIONS(2033),
    [sym_ip_multicast_ttl_option] = ACTIONS(2033),
    [sym_ip_ttl_subnet] = ACTIONS(2033),
    [sym_ip_ttl_site] = ACTIONS(2033),
    [sym_ip_ttl_region] = ACTIONS(2033),
    [sym_ip_ttl_continent] = ACTIONS(2033),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2033),
    [sym_ip_nodelay_on] = ACTIONS(2033),
    [sym_ip_nodelay_off] = ACTIONS(2033),
    [sym_amx_error] = ACTIONS(2033),
    [sym_amx_warning] = ACTIONS(2033),
    [sym_amx_info] = ACTIONS(2033),
    [sym_amx_debug] = ACTIONS(2033),
    [sym_netlinx_login_success] = ACTIONS(2033),
    [sym_netlinx_login_fail] = ACTIONS(2033),
    [sym_netlinx_logout] = ACTIONS(2033),
    [sym_null_str] = ACTIONS(2033),
    [sym_smtp_address] = ACTIONS(2033),
    [sym_smtp_port_number] = ACTIONS(2033),
    [sym_smtp_username] = ACTIONS(2033),
    [sym_smtp_password] = ACTIONS(2033),
    [sym_smtp_require_tls] = ACTIONS(2033),
    [sym_smtp_from] = ACTIONS(2033),
    [sym_smtp_tls_true] = ACTIONS(2033),
    [sym_smtp_tls_false] = ACTIONS(2033),
    [sym_listview_on_row_select_event] = ACTIONS(2033),
    [sym_data_structure_unknown] = ACTIONS(2033),
    [sym_data_structure_datafeed] = ACTIONS(2033),
    [sym_data_structure_datarecordset] = ACTIONS(2033),
    [sym_data_structure_datarecord] = ACTIONS(2033),
    [sym_data_structure_datafield] = ACTIONS(2033),
    [sym_data_max_content_field_count] = ACTIONS(2033),
    [sym_data_max_metadata_field_count] = ACTIONS(2033),
    [sym_data_max_id_length] = ACTIONS(2033),
    [sym_data_max_name_length] = ACTIONS(2033),
    [sym_data_max_description_length] = ACTIONS(2033),
    [sym_data_max_type_length] = ACTIONS(2033),
    [sym_data_max_format_length] = ACTIONS(2033),
    [sym_data_max_source_length] = ACTIONS(2033),
    [sym_data_max_value_length] = ACTIONS(2033),
    [sym_data_max_label_length] = ACTIONS(2033),
    [sym_data_max_url_length] = ACTIONS(2033),
    [sym_max_last_login_info_length] = ACTIONS(2033),
    [sym_data_type_unknown] = ACTIONS(2033),
    [sym_data_type_string] = ACTIONS(2033),
    [sym_data_type_datetime] = ACTIONS(2033),
    [sym_data_type_date] = ACTIONS(2033),
    [sym_data_type_time] = ACTIONS(2033),
    [sym_data_type_image] = ACTIONS(2033),
    [sym_data_format_url] = ACTIONS(2033),
    [sym_data_format_phone] = ACTIONS(2033),
    [sym_data_format_email] = ACTIONS(2033),
    [sym_data_format_iso8601] = ACTIONS(2033),
    [sym_auditlog_priv] = ACTIONS(2033),
    [sym_configuration_priv] = ACTIONS(2033),
    [sym_deviceconfig_priv] = ACTIONS(2033),
    [sym_ftp_priv] = ACTIONS(2033),
    [sym_http_priv] = ACTIONS(2033),
    [sym_networkconfig_priv] = ACTIONS(2033),
    [sym_programport_priv] = ACTIONS(2033),
    [sym_remoteui_priv] = ACTIONS(2033),
    [sym_securitycontrol_priv] = ACTIONS(2033),
    [sym_softwaremanagement_priv] = ACTIONS(2033),
    [sym_terminal_priv] = ACTIONS(2033),
    [sym_tpadmin_priv] = ACTIONS(2033),
    [sym_user1_priv] = ACTIONS(2033),
    [sym_user2_priv] = ACTIONS(2033),
    [sym_user3_priv] = ACTIONS(2033),
    [sym_user4_priv] = ACTIONS(2033),
    [sym_usermanagement_priv] = ACTIONS(2033),
    [sym_valid_account] = ACTIONS(2033),
    [sym_err_name_invalid] = ACTIONS(2033),
    [sym_err_password_invalid] = ACTIONS(2033),
    [sym_err_invalid_account] = ACTIONS(2033),
    [sym_err_no_authorization] = ACTIONS(2033),
    [sym_err_invalid_parameter] = ACTIONS(2033),
    [sym_err_user_locked_out] = ACTIONS(2033),
    [sym_err_user_account_expired] = ACTIONS(2033),
    [sym_err_general_error] = ACTIONS(2033),
    [sym_err_authentication_error] = ACTIONS(2033),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2033),
    [sym_tls_validate_certificate] = ACTIONS(2033),
    [sym_tdata] = ACTIONS(2033),
    [sym_tchannel] = ACTIONS(2033),
    [sym_tlevel] = ACTIONS(2033),
    [sym_tbutton] = ACTIONS(2033),
    [sym_ttimeline] = ACTIONS(2033),
    [sym_tcustom] = ACTIONS(2033),
    [sym_url_struct] = ACTIONS(2033),
    [sym_dns_struct] = ACTIONS(2033),
    [sym_ip_address_struct] = ACTIONS(2033),
    [sym_dev_info_struct] = ACTIONS(2033),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2033),
    [sym_data_feed] = ACTIONS(2033),
    [sym_data_field] = ACTIONS(2033),
    [sym_data_record] = ACTIONS(2033),
    [sym_wc_data_feed] = ACTIONS(2033),
    [sym_wc_data_field] = ACTIONS(2033),
    [sym_wc_data_record] = ACTIONS(2033),
    [sym_last_login_info] = ACTIONS(2033),
    [sym____reserved___] = ACTIONS(2033),
    [sym_date] = ACTIONS(2033),
    [sym_day] = ACTIONS(2033),
    [sym_time] = ACTIONS(2033),
    [sym_ldate] = ACTIONS(2033),
    [sym_push_device] = ACTIONS(2033),
    [sym_push_channel] = ACTIONS(2033),
    [sym_push_devchan] = ACTIONS(2033),
    [sym_release_device] = ACTIONS(2033),
    [sym_release_channel] = ACTIONS(2033),
    [sym_release_devchan] = ACTIONS(2033),
    [sym_master_slot] = ACTIONS(2033),
    [sym_get_pulse_time] = ACTIONS(2033),
    [sym_get_timer] = ACTIONS(2033),
    [sym_master_sn] = ACTIONS(2033),
    [sym_system_number] = ACTIONS(2033),
    [sym_button] = ACTIONS(2033),
    [sym_dv_channel] = ACTIONS(2033),
    [sym_data] = ACTIONS(2033),
    [sym_level] = ACTIONS(2033),
    [sym_timeline] = ACTIONS(2033),
    [sym_channel] = ACTIONS(2033),
    [sym_custom] = ACTIONS(2033),
    [sym_atoi] = ACTIONS(2033),
    [sym_atol] = ACTIONS(2033),
    [sym_atof] = ACTIONS(2033),
    [sym_itoa] = ACTIONS(2033),
    [sym_itohex] = ACTIONS(2033),
    [sym_hextoi] = ACTIONS(2033),
    [sym_ftoa] = ACTIONS(2033),
    [sym_find_string] = ACTIONS(2033),
    [sym_left_string] = ACTIONS(2033),
    [sym_right_string] = ACTIONS(2033),
    [sym_mid_string] = ACTIONS(2033),
    [sym_remove_string] = ACTIONS(2033),
    [sym_get_buffer_string] = ACTIONS(2033),
    [sym_get_buffer_char] = ACTIONS(2033),
    [sym_get_multi_buffer_string] = ACTIONS(2033),
    [sym_length_string] = ACTIONS(2033),
    [sym_max_length_string] = ACTIONS(2033),
    [sym_set_length_string] = ACTIONS(2033),
    [sym_compare_string] = ACTIONS(2033),
    [sym_upper_string] = ACTIONS(2033),
    [sym_lower_string] = ACTIONS(2033),
    [sym_get_last] = ACTIONS(2033),
    [sym_date_to_day] = ACTIONS(2033),
    [sym_date_to_month] = ACTIONS(2033),
    [sym_date_to_year] = ACTIONS(2033),
    [sym_time_to_hour] = ACTIONS(2033),
    [sym_time_to_minute] = ACTIONS(2033),
    [sym_time_to_second] = ACTIONS(2033),
    [sym_day_of_week] = ACTIONS(2033),
    [sym_set_pulse_time] = ACTIONS(2033),
    [sym_set_timer] = ACTIONS(2033),
    [sym_file_open] = ACTIONS(2033),
    [sym_file_close] = ACTIONS(2033),
    [sym_file_read] = ACTIONS(2033),
    [sym_file_read_line] = ACTIONS(2033),
    [sym_file_write] = ACTIONS(2033),
    [sym_file_write_line] = ACTIONS(2033),
    [sym_file_copy] = ACTIONS(2033),
    [sym_file_delete] = ACTIONS(2033),
    [sym_file_rename] = ACTIONS(2033),
    [sym_file_dir] = ACTIONS(2033),
    [sym_file_getdir] = ACTIONS(2033),
    [sym_file_setdir] = ACTIONS(2033),
    [sym_file_create_dir] = ACTIONS(2033),
    [sym_file_remove_dir] = ACTIONS(2033),
    [sym_file_seek] = ACTIONS(2033),
    [sym_length_array] = ACTIONS(2033),
    [sym_max_length_array] = ACTIONS(2033),
    [sym_set_length_array] = ACTIONS(2033),
    [sym_ip_client_open] = ACTIONS(2033),
    [sym_ip_client_close] = ACTIONS(2033),
    [sym_ip_server_open] = ACTIONS(2033),
    [sym_ip_server_close] = ACTIONS(2033),
    [sym_ip_bound_client_open] = ACTIONS(2033),
    [sym_ip_mc_server_open] = ACTIONS(2033),
    [sym_ip_set_option] = ACTIONS(2033),
    [sym_ssh_client_open] = ACTIONS(2033),
    [sym_ssh_client_close] = ACTIONS(2033),
    [sym_tls_client_open] = ACTIONS(2033),
    [sym_tls_client_close] = ACTIONS(2033),
    [sym_do_push] = ACTIONS(2033),
    [sym_do_push_timed] = ACTIONS(2033),
    [sym_do_release] = ACTIONS(2033),
    [sym_device_id] = ACTIONS(2033),
    [sym_device_info] = ACTIONS(2033),
    [sym_device_id_string] = ACTIONS(2033),
    [sym_device_standby] = ACTIONS(2033),
    [sym_device_wake] = ACTIONS(2033),
    [sym_timeline_create] = ACTIONS(2033),
    [sym_timeline_kill] = ACTIONS(2033),
    [sym_timeline_reload] = ACTIONS(2033),
    [sym_timeline_pause] = ACTIONS(2033),
    [sym_timeline_restart] = ACTIONS(2033),
    [sym_timeline_set] = ACTIONS(2033),
    [sym_timeline_get] = ACTIONS(2033),
    [sym_timeline_active] = ACTIONS(2033),
    [sym_timeline_dynamic_id] = ACTIONS(2033),
    [sym_variable_to_string] = ACTIONS(2033),
    [sym_string_to_variable] = ACTIONS(2033),
    [sym_length_variable_to_string] = ACTIONS(2033),
    [sym_type_cast] = ACTIONS(2033),
    [sym_variable_to_xml] = ACTIONS(2033),
    [sym_xml_to_variable] = ACTIONS(2033),
    [sym_length_variable_to_xml] = ACTIONS(2033),
    [sym_redirect_string] = ACTIONS(2033),
    [sym_random_number] = ACTIONS(2033),
    [sym_get_unique_id] = ACTIONS(2033),
    [sym_get_system_number] = ACTIONS(2033),
    [sym_set_system_number] = ACTIONS(2033),
    [sym_get_serial_number] = ACTIONS(2033),
    [sym_get_master_build] = ACTIONS(2033),
    [sym_set_validation_code] = ACTIONS(2033),
    [sym_reboot] = ACTIONS(2033),
    [sym_load_duet_module] = ACTIONS(2033),
    [sym_unload_duet_module] = ACTIONS(2033),
    [sym_duet_mem_size_set] = ACTIONS(2033),
    [sym_duet_mem_size_get] = ACTIONS(2033),
    [sym_dynamic_application_device] = ACTIONS(2033),
    [sym_dynamic_polled_port] = ACTIONS(2033),
    [sym_static_port_binding] = ACTIONS(2033),
    [sym_static_ip_binding] = ACTIONS(2033),
    [sym_get_dns_list] = ACTIONS(2033),
    [sym_set_dns_list] = ACTIONS(2033),
    [sym_get_ip_address] = ACTIONS(2033),
    [sym_set_ip_address] = ACTIONS(2033),
    [sym_get_url_list] = ACTIONS(2033),
    [sym_add_url_entry] = ACTIONS(2033),
    [sym_delete_url_entry] = ACTIONS(2033),
    [sym_abs_value] = ACTIONS(2033),
    [sym_max_value] = ACTIONS(2033),
    [sym_min_value] = ACTIONS(2033),
    [sym_exp_value] = ACTIONS(2033),
    [sym_log_value] = ACTIONS(2033),
    [sym_log10_value] = ACTIONS(2033),
    [sym_power_value] = ACTIONS(2033),
    [sym_sqrt_value] = ACTIONS(2033),
    [sym_raw_be] = ACTIONS(2033),
    [sym_raw_le] = ACTIONS(2033),
    [sym_internal_queue_size_set] = ACTIONS(2033),
    [sym_internal_queue_size_get] = ACTIONS(2033),
    [sym_internal_threshold_set] = ACTIONS(2033),
    [sym_internal_threshold_get] = ACTIONS(2033),
    [sym_rebuilt_event] = ACTIONS(2033),
    [sym_do_custom_event] = ACTIONS(2033),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2033),
    [sym_clkmgr_set_clk_source] = ACTIONS(2033),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2033),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2033),
    [sym_clkmgr_get_timezone] = ACTIONS(2033),
    [sym_clkmgr_set_timezone] = ACTIONS(2033),
    [sym_clkmgr_get_resync_period] = ACTIONS(2033),
    [sym_clkmgr_set_resync_period] = ACTIONS(2033),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2033),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2033),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2033),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2033),
    [sym_clkmgr_get_timeservers] = ACTIONS(2033),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2033),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2033),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2033),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2033),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2033),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2033),
    [sym_set_log_level] = ACTIONS(2033),
    [sym_get_log_level] = ACTIONS(2033),
    [sym_amx_log] = ACTIONS(2033),
    [sym_smtp_server_config_set] = ACTIONS(2033),
    [sym_smtp_server_config_get] = ACTIONS(2033),
    [sym_smtp_send] = ACTIONS(2033),
    [sym_validate_netlinx_account] = ACTIONS(2033),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2033),
    [sym_authenticate_certificate] = ACTIONS(2033),
    [sym_audit_netlinx_session_event] = ACTIONS(2033),
    [sym_audit_netlinx_generic_event] = ACTIONS(2033),
    [sym_get_available_flash_disk_space] = ACTIONS(2033),
    [sym_get_max_flash_disk_space] = ACTIONS(2033),
    [sym_data_create_feed] = ACTIONS(2033),
    [sym_data_delete_feed] = ACTIONS(2033),
    [sym_data_publish_feed] = ACTIONS(2033),
    [sym_data_get_published_feed] = ACTIONS(2033),
    [sym_data_add_record] = ACTIONS(2033),
    [sym_data_get_event_record] = ACTIONS(2033),
    [sym__wc_data_create_feed] = ACTIONS(2033),
    [sym__wc_data_add_record] = ACTIONS(2033),
    [sym__wc_data_get_event_record] = ACTIONS(2033),
    [sym_format] = ACTIONS(2033),
    [sym_astro_clock] = ACTIONS(2033),
    [sym_set_outdoor_temperature] = ACTIONS(2033),
    [sym_set_virtual_level_count] = ACTIONS(2033),
    [sym_set_virtual_channel_count] = ACTIONS(2033),
    [sym_set_virtual_port_count] = ACTIONS(2033),
    [sym__wc_encode_int] = ACTIONS(2033),
    [sym__wc_decode_int] = ACTIONS(2033),
    [sym__wc_int] = ACTIONS(2033),
    [sym__wc_to_ch_int] = ACTIONS(2033),
    [sym__ch_to_wc_int] = ACTIONS(2033),
    [sym__wc_find_string_int] = ACTIONS(2033),
    [sym__wc_left_string_int] = ACTIONS(2033),
    [sym__wc_lower_string_int] = ACTIONS(2033),
    [sym__wc_mid_string_int] = ACTIONS(2033),
    [sym__wc_remove_string_int] = ACTIONS(2033),
    [sym__wc_right_string_int] = ACTIONS(2033),
    [sym__wc_upper_string_int] = ACTIONS(2033),
    [sym__wc_compare_string_int] = ACTIONS(2033),
    [sym__wc_get_buffer_char_int] = ACTIONS(2033),
    [sym__wc_get_buffer_string_int] = ACTIONS(2033),
    [sym__wc_concat_string_int] = ACTIONS(2033),
    [sym__wc_file_open_int] = ACTIONS(2033),
    [sym__wc_file_close_int] = ACTIONS(2033),
    [sym__wc_file_read_int] = ACTIONS(2033),
    [sym__wc_file_read_line_int] = ACTIONS(2033),
    [sym__wc_file_write_int] = ACTIONS(2033),
    [sym__wc_file_write_line_int] = ACTIONS(2033),
    [aux_sym_preproc_include_token1] = ACTIONS(2033),
    [aux_sym_preproc_define_token1] = ACTIONS(2035),
    [aux_sym_preproc_warn_token1] = ACTIONS(2035),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2035),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2035),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2035),
    [anon_sym_LBRACK] = ACTIONS(2035),
    [anon_sym_LBRACE] = ACTIONS(2035),
    [anon_sym_LPAREN] = ACTIONS(2033),
    [anon_sym_BANG] = ACTIONS(2035),
    [anon_sym_TILDE] = ACTIONS(2035),
    [anon_sym_DASH] = ACTIONS(2033),
    [anon_sym_PLUS] = ACTIONS(2033),
    [anon_sym_DASH_DASH] = ACTIONS(2035),
    [anon_sym_PLUS_PLUS] = ACTIONS(2035),
    [anon_sym_DQUOTE] = ACTIONS(2035),
    [anon_sym_SQUOTE] = ACTIONS(2035),
    [sym_number_literal] = ACTIONS(2035),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(435)] = {
    [ts_builtin_sym_end] = ACTIONS(1897),
    [sym_identifier] = ACTIONS(1895),
    [sym_program_name_keyword] = ACTIONS(1895),
    [sym_module_name_keyword] = ACTIONS(1895),
    [sym_define_device_keyword] = ACTIONS(1895),
    [sym_define_combine_keyword] = ACTIONS(1895),
    [sym_define_constant_keyword] = ACTIONS(1895),
    [sym_define_type_keyword] = ACTIONS(1895),
    [sym_define_variable_keyword] = ACTIONS(1895),
    [sym_define_system_variable_keyword] = ACTIONS(1895),
    [sym_define_connect_level_keyword] = ACTIONS(1895),
    [sym_define_latching_keyword] = ACTIONS(1895),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1895),
    [sym_define_toggling_keyword] = ACTIONS(1895),
    [sym_define_call_keyword] = ACTIONS(1895),
    [sym_define_function_keyword] = ACTIONS(1895),
    [sym_define_library_function_keyword] = ACTIONS(1895),
    [sym_define_start_keyword] = ACTIONS(1895),
    [sym_define_event_keyword] = ACTIONS(1895),
    [sym_define_program_keyword] = ACTIONS(1895),
    [sym_define_module_keyword] = ACTIONS(1895),
    [sym_char_keyword] = ACTIONS(1895),
    [sym_widechar_keyword] = ACTIONS(1895),
    [sym_integer_keyword] = ACTIONS(1895),
    [sym_sinteger_keyword] = ACTIONS(1895),
    [sym_long_keyword] = ACTIONS(1895),
    [sym_slong_keyword] = ACTIONS(1895),
    [sym_float_keyword] = ACTIONS(1895),
    [sym_double_keyword] = ACTIONS(1895),
    [sym_variant_keyword] = ACTIONS(1895),
    [sym_variantarray_keyword] = ACTIONS(1895),
    [sym_local_var_keyword] = ACTIONS(1895),
    [sym_stack_var_keyword] = ACTIONS(1895),
    [sym_constant_keyword] = ACTIONS(1895),
    [sym_volatile_keyword] = ACTIONS(1895),
    [sym_non_volatile_keyword] = ACTIONS(1895),
    [sym_persistent_keyword] = ACTIONS(1895),
    [sym_dev_keyword] = ACTIONS(1895),
    [sym_devlev_keyword] = ACTIONS(1895),
    [sym_devchan_keyword] = ACTIONS(1895),
    [sym_if_keyword] = ACTIONS(1895),
    [sym_switch_keyword] = ACTIONS(1895),
    [sym_case_keyword] = ACTIONS(1895),
    [sym_default_keyword] = ACTIONS(1895),
    [sym_while_keyword] = ACTIONS(1895),
    [sym_for_keyword] = ACTIONS(1895),
    [sym_break_keyword] = ACTIONS(1895),
    [sym_continue_keyword] = ACTIONS(1895),
    [sym_return_keyword] = ACTIONS(1895),
    [sym_select_keyword] = ACTIONS(1895),
    [sym_active_keyword] = ACTIONS(1895),
    [sym_struct_keyword] = ACTIONS(1895),
    [sym_structure_keyword] = ACTIONS(1895),
    [sym_button_event_keyword] = ACTIONS(1895),
    [sym_channel_event_keyword] = ACTIONS(1895),
    [sym_level_event_keyword] = ACTIONS(1895),
    [sym_data_event_keyword] = ACTIONS(1895),
    [sym_timeline_event_keyword] = ACTIONS(1895),
    [sym_custom_event_keyword] = ACTIONS(1895),
    [sym_push_keyword] = ACTIONS(1895),
    [sym_release_keyword] = ACTIONS(1895),
    [aux_sym_on_keyword_token1] = ACTIONS(1895),
    [aux_sym_off_keyword_token1] = ACTIONS(1895),
    [sym_send_level_keyword] = ACTIONS(1895),
    [sym_send_string_keyword] = ACTIONS(1895),
    [sym_send_command_keyword] = ACTIONS(1895),
    [sym_clear_buffer_keyword] = ACTIONS(1895),
    [sym_create_buffer_keyword] = ACTIONS(1895),
    [sym_create_multi_buffer_keyword] = ACTIONS(1895),
    [sym_call_keyword] = ACTIONS(1895),
    [sym_system_call_keyword] = ACTIONS(1895),
    [sym_devchan_to_keyword] = ACTIONS(1895),
    [sym_devchan_min_to_keyword] = ACTIONS(1895),
    [sym_devchan_total_off_keyword] = ACTIONS(1895),
    [sym_devchan_pulse_keyword] = ACTIONS(1895),
    [sym_wait_keyword] = ACTIONS(1895),
    [sym_cancel_wait_keyword] = ACTIONS(1895),
    [sym_wait_until_keyword] = ACTIONS(1895),
    [sym_cancel_wait_until_keyword] = ACTIONS(1895),
    [sym_cancel_all_wait_keyword] = ACTIONS(1895),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1895),
    [sym_not] = ACTIONS(1895),
    [sym_bnot] = ACTIONS(1895),
    [sym___file__] = ACTIONS(1895),
    [sym___date__] = ACTIONS(1895),
    [sym___ldate__] = ACTIONS(1895),
    [sym___time__] = ACTIONS(1895),
    [sym___line__] = ACTIONS(1895),
    [sym___name__] = ACTIONS(1895),
    [sym___netlinx__] = ACTIONS(1895),
    [sym_netlinx_axi_version] = ACTIONS(1895),
    [sym_url_flg_tcp] = ACTIONS(1895),
    [sym_url_flg_accinfopresent] = ACTIONS(1895),
    [sym_url_flg_temp] = ACTIONS(1895),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1895),
    [sym_url_flg_stat_mask] = ACTIONS(1895),
    [sym_url_flg_stat_lookup] = ACTIONS(1895),
    [sym_url_flg_stat_connecting] = ACTIONS(1895),
    [sym_url_flg_stat_waiting] = ACTIONS(1895),
    [sym_url_flg_stat_connected] = ACTIONS(1895),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1895),
    [sym_first_virtual_device] = ACTIONS(1895),
    [sym_first_local_port] = ACTIONS(1895),
    [sym_dynamic_virtual_device] = ACTIONS(1895),
    [sym_timeline_once] = ACTIONS(1895),
    [sym_timeline_repeat] = ACTIONS(1895),
    [sym_timeline_absolute] = ACTIONS(1895),
    [sym_timeline_relative] = ACTIONS(1895),
    [sym_do_push_timed_infinite] = ACTIONS(1895),
    [sym_source_type_no_address] = ACTIONS(1895),
    [sym_source_type_neuron_id] = ACTIONS(1895),
    [sym_source_type_ip_address] = ACTIONS(1895),
    [sym_source_type_axlink] = ACTIONS(1895),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1895),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1895),
    [sym_source_type_ip_socket_address] = ACTIONS(1895),
    [sym_source_type_rs232] = ACTIONS(1895),
    [sym_source_type_internal] = ACTIONS(1895),
    [sym_source_type_mac_address] = ACTIONS(1895),
    [sym_source_type_ipv4_port] = ACTIONS(1895),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1895),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1895),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1895),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1895),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1895),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1895),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1895),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1895),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1895),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1895),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1895),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1895),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1895),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1895),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1895),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1895),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1895),
    [sym_true] = ACTIONS(1895),
    [sym_false] = ACTIONS(1895),
    [sym_file_read_only] = ACTIONS(1895),
    [sym_file_rw_new] = ACTIONS(1895),
    [sym_file_rw_append] = ACTIONS(1895),
    [sym_ip_tcp] = ACTIONS(1895),
    [sym_ip_udp] = ACTIONS(1895),
    [sym_ip_udp_2way] = ACTIONS(1895),
    [sym_xml_encode_types] = ACTIONS(1895),
    [sym_xml_encode_char_as_list] = ACTIONS(1895),
    [sym_xml_encode_le] = ACTIONS(1895),
    [sym_xml_decode_types] = ACTIONS(1895),
    [sym_xml_decode_no_preserve] = ACTIONS(1895),
    [sym_duet_dev_type_display_device] = ACTIONS(1895),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1895),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1895),
    [sym_duet_dev_type_io_device] = ACTIONS(1895),
    [sym_duet_dev_type_relay_device] = ACTIONS(1895),
    [sym_duet_dev_type_ups] = ACTIONS(1895),
    [sym_duet_dev_type_amplifier] = ACTIONS(1895),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1895),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1895),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1895),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1895),
    [sym_duet_dev_type_camera] = ACTIONS(1895),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1895),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1895),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1895),
    [sym_duet_dev_type_dss] = ACTIONS(1895),
    [sym_duet_dev_type_dvr] = ACTIONS(1895),
    [sym_duet_dev_type_disc_device] = ACTIONS(1895),
    [sym_duet_dev_type_document_camera] = ACTIONS(1895),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1895),
    [sym_duet_dev_type_hvac] = ACTIONS(1895),
    [sym_duet_dev_type_keypad] = ACTIONS(1895),
    [sym_duet_dev_type_light] = ACTIONS(1895),
    [sym_duet_dev_type_monitor] = ACTIONS(1895),
    [sym_duet_dev_type_motor] = ACTIONS(1895),
    [sym_duet_dev_type_multi_window] = ACTIONS(1895),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1895),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1895),
    [sym_duet_dev_type_receiver] = ACTIONS(1895),
    [sym_duet_dev_type_security_system] = ACTIONS(1895),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1895),
    [sym_duet_dev_type_settop_box] = ACTIONS(1895),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1895),
    [sym_duet_dev_type_switcher] = ACTIONS(1895),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1895),
    [sym_duet_dev_type_tv] = ACTIONS(1895),
    [sym_duet_dev_type_utility] = ACTIONS(1895),
    [sym_duet_dev_type_vcr] = ACTIONS(1895),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1895),
    [sym_duet_dev_type_video_processor] = ACTIONS(1895),
    [sym_duet_dev_type_video_projector] = ACTIONS(1895),
    [sym_duet_dev_type_video_wall] = ACTIONS(1895),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1895),
    [sym_duet_dev_type_weather] = ACTIONS(1895),
    [sym_duet_dev_not_polled] = ACTIONS(1895),
    [sym_duet_dev_polled] = ACTIONS(1895),
    [sym_clkmgr_mode_network] = ACTIONS(1895),
    [sym_clkmgr_mode_standalone] = ACTIONS(1895),
    [sym_normal_standby] = ACTIONS(1895),
    [sym_normal_wake] = ACTIONS(1895),
    [sym_ip_multicast_ttl_option] = ACTIONS(1895),
    [sym_ip_ttl_subnet] = ACTIONS(1895),
    [sym_ip_ttl_site] = ACTIONS(1895),
    [sym_ip_ttl_region] = ACTIONS(1895),
    [sym_ip_ttl_continent] = ACTIONS(1895),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1895),
    [sym_ip_nodelay_on] = ACTIONS(1895),
    [sym_ip_nodelay_off] = ACTIONS(1895),
    [sym_amx_error] = ACTIONS(1895),
    [sym_amx_warning] = ACTIONS(1895),
    [sym_amx_info] = ACTIONS(1895),
    [sym_amx_debug] = ACTIONS(1895),
    [sym_netlinx_login_success] = ACTIONS(1895),
    [sym_netlinx_login_fail] = ACTIONS(1895),
    [sym_netlinx_logout] = ACTIONS(1895),
    [sym_null_str] = ACTIONS(1895),
    [sym_smtp_address] = ACTIONS(1895),
    [sym_smtp_port_number] = ACTIONS(1895),
    [sym_smtp_username] = ACTIONS(1895),
    [sym_smtp_password] = ACTIONS(1895),
    [sym_smtp_require_tls] = ACTIONS(1895),
    [sym_smtp_from] = ACTIONS(1895),
    [sym_smtp_tls_true] = ACTIONS(1895),
    [sym_smtp_tls_false] = ACTIONS(1895),
    [sym_listview_on_row_select_event] = ACTIONS(1895),
    [sym_data_structure_unknown] = ACTIONS(1895),
    [sym_data_structure_datafeed] = ACTIONS(1895),
    [sym_data_structure_datarecordset] = ACTIONS(1895),
    [sym_data_structure_datarecord] = ACTIONS(1895),
    [sym_data_structure_datafield] = ACTIONS(1895),
    [sym_data_max_content_field_count] = ACTIONS(1895),
    [sym_data_max_metadata_field_count] = ACTIONS(1895),
    [sym_data_max_id_length] = ACTIONS(1895),
    [sym_data_max_name_length] = ACTIONS(1895),
    [sym_data_max_description_length] = ACTIONS(1895),
    [sym_data_max_type_length] = ACTIONS(1895),
    [sym_data_max_format_length] = ACTIONS(1895),
    [sym_data_max_source_length] = ACTIONS(1895),
    [sym_data_max_value_length] = ACTIONS(1895),
    [sym_data_max_label_length] = ACTIONS(1895),
    [sym_data_max_url_length] = ACTIONS(1895),
    [sym_max_last_login_info_length] = ACTIONS(1895),
    [sym_data_type_unknown] = ACTIONS(1895),
    [sym_data_type_string] = ACTIONS(1895),
    [sym_data_type_datetime] = ACTIONS(1895),
    [sym_data_type_date] = ACTIONS(1895),
    [sym_data_type_time] = ACTIONS(1895),
    [sym_data_type_image] = ACTIONS(1895),
    [sym_data_format_url] = ACTIONS(1895),
    [sym_data_format_phone] = ACTIONS(1895),
    [sym_data_format_email] = ACTIONS(1895),
    [sym_data_format_iso8601] = ACTIONS(1895),
    [sym_auditlog_priv] = ACTIONS(1895),
    [sym_configuration_priv] = ACTIONS(1895),
    [sym_deviceconfig_priv] = ACTIONS(1895),
    [sym_ftp_priv] = ACTIONS(1895),
    [sym_http_priv] = ACTIONS(1895),
    [sym_networkconfig_priv] = ACTIONS(1895),
    [sym_programport_priv] = ACTIONS(1895),
    [sym_remoteui_priv] = ACTIONS(1895),
    [sym_securitycontrol_priv] = ACTIONS(1895),
    [sym_softwaremanagement_priv] = ACTIONS(1895),
    [sym_terminal_priv] = ACTIONS(1895),
    [sym_tpadmin_priv] = ACTIONS(1895),
    [sym_user1_priv] = ACTIONS(1895),
    [sym_user2_priv] = ACTIONS(1895),
    [sym_user3_priv] = ACTIONS(1895),
    [sym_user4_priv] = ACTIONS(1895),
    [sym_usermanagement_priv] = ACTIONS(1895),
    [sym_valid_account] = ACTIONS(1895),
    [sym_err_name_invalid] = ACTIONS(1895),
    [sym_err_password_invalid] = ACTIONS(1895),
    [sym_err_invalid_account] = ACTIONS(1895),
    [sym_err_no_authorization] = ACTIONS(1895),
    [sym_err_invalid_parameter] = ACTIONS(1895),
    [sym_err_user_locked_out] = ACTIONS(1895),
    [sym_err_user_account_expired] = ACTIONS(1895),
    [sym_err_general_error] = ACTIONS(1895),
    [sym_err_authentication_error] = ACTIONS(1895),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1895),
    [sym_tls_validate_certificate] = ACTIONS(1895),
    [sym_tdata] = ACTIONS(1895),
    [sym_tchannel] = ACTIONS(1895),
    [sym_tlevel] = ACTIONS(1895),
    [sym_tbutton] = ACTIONS(1895),
    [sym_ttimeline] = ACTIONS(1895),
    [sym_tcustom] = ACTIONS(1895),
    [sym_url_struct] = ACTIONS(1895),
    [sym_dns_struct] = ACTIONS(1895),
    [sym_ip_address_struct] = ACTIONS(1895),
    [sym_dev_info_struct] = ACTIONS(1895),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1895),
    [sym_data_feed] = ACTIONS(1895),
    [sym_data_field] = ACTIONS(1895),
    [sym_data_record] = ACTIONS(1895),
    [sym_wc_data_feed] = ACTIONS(1895),
    [sym_wc_data_field] = ACTIONS(1895),
    [sym_wc_data_record] = ACTIONS(1895),
    [sym_last_login_info] = ACTIONS(1895),
    [sym____reserved___] = ACTIONS(1895),
    [sym_date] = ACTIONS(1895),
    [sym_day] = ACTIONS(1895),
    [sym_time] = ACTIONS(1895),
    [sym_ldate] = ACTIONS(1895),
    [sym_push_device] = ACTIONS(1895),
    [sym_push_channel] = ACTIONS(1895),
    [sym_push_devchan] = ACTIONS(1895),
    [sym_release_device] = ACTIONS(1895),
    [sym_release_channel] = ACTIONS(1895),
    [sym_release_devchan] = ACTIONS(1895),
    [sym_master_slot] = ACTIONS(1895),
    [sym_get_pulse_time] = ACTIONS(1895),
    [sym_get_timer] = ACTIONS(1895),
    [sym_master_sn] = ACTIONS(1895),
    [sym_system_number] = ACTIONS(1895),
    [sym_button] = ACTIONS(1895),
    [sym_dv_channel] = ACTIONS(1895),
    [sym_data] = ACTIONS(1895),
    [sym_level] = ACTIONS(1895),
    [sym_timeline] = ACTIONS(1895),
    [sym_channel] = ACTIONS(1895),
    [sym_custom] = ACTIONS(1895),
    [sym_atoi] = ACTIONS(1895),
    [sym_atol] = ACTIONS(1895),
    [sym_atof] = ACTIONS(1895),
    [sym_itoa] = ACTIONS(1895),
    [sym_itohex] = ACTIONS(1895),
    [sym_hextoi] = ACTIONS(1895),
    [sym_ftoa] = ACTIONS(1895),
    [sym_find_string] = ACTIONS(1895),
    [sym_left_string] = ACTIONS(1895),
    [sym_right_string] = ACTIONS(1895),
    [sym_mid_string] = ACTIONS(1895),
    [sym_remove_string] = ACTIONS(1895),
    [sym_get_buffer_string] = ACTIONS(1895),
    [sym_get_buffer_char] = ACTIONS(1895),
    [sym_get_multi_buffer_string] = ACTIONS(1895),
    [sym_length_string] = ACTIONS(1895),
    [sym_max_length_string] = ACTIONS(1895),
    [sym_set_length_string] = ACTIONS(1895),
    [sym_compare_string] = ACTIONS(1895),
    [sym_upper_string] = ACTIONS(1895),
    [sym_lower_string] = ACTIONS(1895),
    [sym_get_last] = ACTIONS(1895),
    [sym_date_to_day] = ACTIONS(1895),
    [sym_date_to_month] = ACTIONS(1895),
    [sym_date_to_year] = ACTIONS(1895),
    [sym_time_to_hour] = ACTIONS(1895),
    [sym_time_to_minute] = ACTIONS(1895),
    [sym_time_to_second] = ACTIONS(1895),
    [sym_day_of_week] = ACTIONS(1895),
    [sym_set_pulse_time] = ACTIONS(1895),
    [sym_set_timer] = ACTIONS(1895),
    [sym_file_open] = ACTIONS(1895),
    [sym_file_close] = ACTIONS(1895),
    [sym_file_read] = ACTIONS(1895),
    [sym_file_read_line] = ACTIONS(1895),
    [sym_file_write] = ACTIONS(1895),
    [sym_file_write_line] = ACTIONS(1895),
    [sym_file_copy] = ACTIONS(1895),
    [sym_file_delete] = ACTIONS(1895),
    [sym_file_rename] = ACTIONS(1895),
    [sym_file_dir] = ACTIONS(1895),
    [sym_file_getdir] = ACTIONS(1895),
    [sym_file_setdir] = ACTIONS(1895),
    [sym_file_create_dir] = ACTIONS(1895),
    [sym_file_remove_dir] = ACTIONS(1895),
    [sym_file_seek] = ACTIONS(1895),
    [sym_length_array] = ACTIONS(1895),
    [sym_max_length_array] = ACTIONS(1895),
    [sym_set_length_array] = ACTIONS(1895),
    [sym_ip_client_open] = ACTIONS(1895),
    [sym_ip_client_close] = ACTIONS(1895),
    [sym_ip_server_open] = ACTIONS(1895),
    [sym_ip_server_close] = ACTIONS(1895),
    [sym_ip_bound_client_open] = ACTIONS(1895),
    [sym_ip_mc_server_open] = ACTIONS(1895),
    [sym_ip_set_option] = ACTIONS(1895),
    [sym_ssh_client_open] = ACTIONS(1895),
    [sym_ssh_client_close] = ACTIONS(1895),
    [sym_tls_client_open] = ACTIONS(1895),
    [sym_tls_client_close] = ACTIONS(1895),
    [sym_do_push] = ACTIONS(1895),
    [sym_do_push_timed] = ACTIONS(1895),
    [sym_do_release] = ACTIONS(1895),
    [sym_device_id] = ACTIONS(1895),
    [sym_device_info] = ACTIONS(1895),
    [sym_device_id_string] = ACTIONS(1895),
    [sym_device_standby] = ACTIONS(1895),
    [sym_device_wake] = ACTIONS(1895),
    [sym_timeline_create] = ACTIONS(1895),
    [sym_timeline_kill] = ACTIONS(1895),
    [sym_timeline_reload] = ACTIONS(1895),
    [sym_timeline_pause] = ACTIONS(1895),
    [sym_timeline_restart] = ACTIONS(1895),
    [sym_timeline_set] = ACTIONS(1895),
    [sym_timeline_get] = ACTIONS(1895),
    [sym_timeline_active] = ACTIONS(1895),
    [sym_timeline_dynamic_id] = ACTIONS(1895),
    [sym_variable_to_string] = ACTIONS(1895),
    [sym_string_to_variable] = ACTIONS(1895),
    [sym_length_variable_to_string] = ACTIONS(1895),
    [sym_type_cast] = ACTIONS(1895),
    [sym_variable_to_xml] = ACTIONS(1895),
    [sym_xml_to_variable] = ACTIONS(1895),
    [sym_length_variable_to_xml] = ACTIONS(1895),
    [sym_redirect_string] = ACTIONS(1895),
    [sym_random_number] = ACTIONS(1895),
    [sym_get_unique_id] = ACTIONS(1895),
    [sym_get_system_number] = ACTIONS(1895),
    [sym_set_system_number] = ACTIONS(1895),
    [sym_get_serial_number] = ACTIONS(1895),
    [sym_get_master_build] = ACTIONS(1895),
    [sym_set_validation_code] = ACTIONS(1895),
    [sym_reboot] = ACTIONS(1895),
    [sym_load_duet_module] = ACTIONS(1895),
    [sym_unload_duet_module] = ACTIONS(1895),
    [sym_duet_mem_size_set] = ACTIONS(1895),
    [sym_duet_mem_size_get] = ACTIONS(1895),
    [sym_dynamic_application_device] = ACTIONS(1895),
    [sym_dynamic_polled_port] = ACTIONS(1895),
    [sym_static_port_binding] = ACTIONS(1895),
    [sym_static_ip_binding] = ACTIONS(1895),
    [sym_get_dns_list] = ACTIONS(1895),
    [sym_set_dns_list] = ACTIONS(1895),
    [sym_get_ip_address] = ACTIONS(1895),
    [sym_set_ip_address] = ACTIONS(1895),
    [sym_get_url_list] = ACTIONS(1895),
    [sym_add_url_entry] = ACTIONS(1895),
    [sym_delete_url_entry] = ACTIONS(1895),
    [sym_abs_value] = ACTIONS(1895),
    [sym_max_value] = ACTIONS(1895),
    [sym_min_value] = ACTIONS(1895),
    [sym_exp_value] = ACTIONS(1895),
    [sym_log_value] = ACTIONS(1895),
    [sym_log10_value] = ACTIONS(1895),
    [sym_power_value] = ACTIONS(1895),
    [sym_sqrt_value] = ACTIONS(1895),
    [sym_raw_be] = ACTIONS(1895),
    [sym_raw_le] = ACTIONS(1895),
    [sym_internal_queue_size_set] = ACTIONS(1895),
    [sym_internal_queue_size_get] = ACTIONS(1895),
    [sym_internal_threshold_set] = ACTIONS(1895),
    [sym_internal_threshold_get] = ACTIONS(1895),
    [sym_rebuilt_event] = ACTIONS(1895),
    [sym_do_custom_event] = ACTIONS(1895),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1895),
    [sym_clkmgr_set_clk_source] = ACTIONS(1895),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1895),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1895),
    [sym_clkmgr_get_timezone] = ACTIONS(1895),
    [sym_clkmgr_set_timezone] = ACTIONS(1895),
    [sym_clkmgr_get_resync_period] = ACTIONS(1895),
    [sym_clkmgr_set_resync_period] = ACTIONS(1895),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1895),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1895),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1895),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1895),
    [sym_clkmgr_get_timeservers] = ACTIONS(1895),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1895),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1895),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1895),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1895),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1895),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1895),
    [sym_set_log_level] = ACTIONS(1895),
    [sym_get_log_level] = ACTIONS(1895),
    [sym_amx_log] = ACTIONS(1895),
    [sym_smtp_server_config_set] = ACTIONS(1895),
    [sym_smtp_server_config_get] = ACTIONS(1895),
    [sym_smtp_send] = ACTIONS(1895),
    [sym_validate_netlinx_account] = ACTIONS(1895),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1895),
    [sym_authenticate_certificate] = ACTIONS(1895),
    [sym_audit_netlinx_session_event] = ACTIONS(1895),
    [sym_audit_netlinx_generic_event] = ACTIONS(1895),
    [sym_get_available_flash_disk_space] = ACTIONS(1895),
    [sym_get_max_flash_disk_space] = ACTIONS(1895),
    [sym_data_create_feed] = ACTIONS(1895),
    [sym_data_delete_feed] = ACTIONS(1895),
    [sym_data_publish_feed] = ACTIONS(1895),
    [sym_data_get_published_feed] = ACTIONS(1895),
    [sym_data_add_record] = ACTIONS(1895),
    [sym_data_get_event_record] = ACTIONS(1895),
    [sym__wc_data_create_feed] = ACTIONS(1895),
    [sym__wc_data_add_record] = ACTIONS(1895),
    [sym__wc_data_get_event_record] = ACTIONS(1895),
    [sym_format] = ACTIONS(1895),
    [sym_astro_clock] = ACTIONS(1895),
    [sym_set_outdoor_temperature] = ACTIONS(1895),
    [sym_set_virtual_level_count] = ACTIONS(1895),
    [sym_set_virtual_channel_count] = ACTIONS(1895),
    [sym_set_virtual_port_count] = ACTIONS(1895),
    [sym__wc_encode_int] = ACTIONS(1895),
    [sym__wc_decode_int] = ACTIONS(1895),
    [sym__wc_int] = ACTIONS(1895),
    [sym__wc_to_ch_int] = ACTIONS(1895),
    [sym__ch_to_wc_int] = ACTIONS(1895),
    [sym__wc_find_string_int] = ACTIONS(1895),
    [sym__wc_left_string_int] = ACTIONS(1895),
    [sym__wc_lower_string_int] = ACTIONS(1895),
    [sym__wc_mid_string_int] = ACTIONS(1895),
    [sym__wc_remove_string_int] = ACTIONS(1895),
    [sym__wc_right_string_int] = ACTIONS(1895),
    [sym__wc_upper_string_int] = ACTIONS(1895),
    [sym__wc_compare_string_int] = ACTIONS(1895),
    [sym__wc_get_buffer_char_int] = ACTIONS(1895),
    [sym__wc_get_buffer_string_int] = ACTIONS(1895),
    [sym__wc_concat_string_int] = ACTIONS(1895),
    [sym__wc_file_open_int] = ACTIONS(1895),
    [sym__wc_file_close_int] = ACTIONS(1895),
    [sym__wc_file_read_int] = ACTIONS(1895),
    [sym__wc_file_read_line_int] = ACTIONS(1895),
    [sym__wc_file_write_int] = ACTIONS(1895),
    [sym__wc_file_write_line_int] = ACTIONS(1895),
    [aux_sym_preproc_include_token1] = ACTIONS(1895),
    [aux_sym_preproc_define_token1] = ACTIONS(1897),
    [aux_sym_preproc_warn_token1] = ACTIONS(1897),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1897),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1897),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1897),
    [anon_sym_LBRACK] = ACTIONS(1897),
    [anon_sym_LBRACE] = ACTIONS(1897),
    [anon_sym_LPAREN] = ACTIONS(1895),
    [anon_sym_BANG] = ACTIONS(1897),
    [anon_sym_TILDE] = ACTIONS(1897),
    [anon_sym_DASH] = ACTIONS(1895),
    [anon_sym_PLUS] = ACTIONS(1895),
    [anon_sym_DASH_DASH] = ACTIONS(1897),
    [anon_sym_PLUS_PLUS] = ACTIONS(1897),
    [anon_sym_DQUOTE] = ACTIONS(1897),
    [anon_sym_SQUOTE] = ACTIONS(1897),
    [sym_number_literal] = ACTIONS(1897),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(436)] = {
    [ts_builtin_sym_end] = ACTIONS(1919),
    [sym_identifier] = ACTIONS(1917),
    [sym_program_name_keyword] = ACTIONS(1917),
    [sym_module_name_keyword] = ACTIONS(1917),
    [sym_define_device_keyword] = ACTIONS(1917),
    [sym_define_combine_keyword] = ACTIONS(1917),
    [sym_define_constant_keyword] = ACTIONS(1917),
    [sym_define_type_keyword] = ACTIONS(1917),
    [sym_define_variable_keyword] = ACTIONS(1917),
    [sym_define_system_variable_keyword] = ACTIONS(1917),
    [sym_define_connect_level_keyword] = ACTIONS(1917),
    [sym_define_latching_keyword] = ACTIONS(1917),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1917),
    [sym_define_toggling_keyword] = ACTIONS(1917),
    [sym_define_call_keyword] = ACTIONS(1917),
    [sym_define_function_keyword] = ACTIONS(1917),
    [sym_define_library_function_keyword] = ACTIONS(1917),
    [sym_define_start_keyword] = ACTIONS(1917),
    [sym_define_event_keyword] = ACTIONS(1917),
    [sym_define_program_keyword] = ACTIONS(1917),
    [sym_define_module_keyword] = ACTIONS(1917),
    [sym_char_keyword] = ACTIONS(1917),
    [sym_widechar_keyword] = ACTIONS(1917),
    [sym_integer_keyword] = ACTIONS(1917),
    [sym_sinteger_keyword] = ACTIONS(1917),
    [sym_long_keyword] = ACTIONS(1917),
    [sym_slong_keyword] = ACTIONS(1917),
    [sym_float_keyword] = ACTIONS(1917),
    [sym_double_keyword] = ACTIONS(1917),
    [sym_variant_keyword] = ACTIONS(1917),
    [sym_variantarray_keyword] = ACTIONS(1917),
    [sym_local_var_keyword] = ACTIONS(1917),
    [sym_stack_var_keyword] = ACTIONS(1917),
    [sym_constant_keyword] = ACTIONS(1917),
    [sym_volatile_keyword] = ACTIONS(1917),
    [sym_non_volatile_keyword] = ACTIONS(1917),
    [sym_persistent_keyword] = ACTIONS(1917),
    [sym_dev_keyword] = ACTIONS(1917),
    [sym_devlev_keyword] = ACTIONS(1917),
    [sym_devchan_keyword] = ACTIONS(1917),
    [sym_if_keyword] = ACTIONS(1917),
    [sym_switch_keyword] = ACTIONS(1917),
    [sym_case_keyword] = ACTIONS(1917),
    [sym_default_keyword] = ACTIONS(1917),
    [sym_while_keyword] = ACTIONS(1917),
    [sym_for_keyword] = ACTIONS(1917),
    [sym_break_keyword] = ACTIONS(1917),
    [sym_continue_keyword] = ACTIONS(1917),
    [sym_return_keyword] = ACTIONS(1917),
    [sym_select_keyword] = ACTIONS(1917),
    [sym_active_keyword] = ACTIONS(1917),
    [sym_struct_keyword] = ACTIONS(1917),
    [sym_structure_keyword] = ACTIONS(1917),
    [sym_button_event_keyword] = ACTIONS(1917),
    [sym_channel_event_keyword] = ACTIONS(1917),
    [sym_level_event_keyword] = ACTIONS(1917),
    [sym_data_event_keyword] = ACTIONS(1917),
    [sym_timeline_event_keyword] = ACTIONS(1917),
    [sym_custom_event_keyword] = ACTIONS(1917),
    [sym_push_keyword] = ACTIONS(1917),
    [sym_release_keyword] = ACTIONS(1917),
    [aux_sym_on_keyword_token1] = ACTIONS(1917),
    [aux_sym_off_keyword_token1] = ACTIONS(1917),
    [sym_send_level_keyword] = ACTIONS(1917),
    [sym_send_string_keyword] = ACTIONS(1917),
    [sym_send_command_keyword] = ACTIONS(1917),
    [sym_clear_buffer_keyword] = ACTIONS(1917),
    [sym_create_buffer_keyword] = ACTIONS(1917),
    [sym_create_multi_buffer_keyword] = ACTIONS(1917),
    [sym_call_keyword] = ACTIONS(1917),
    [sym_system_call_keyword] = ACTIONS(1917),
    [sym_devchan_to_keyword] = ACTIONS(1917),
    [sym_devchan_min_to_keyword] = ACTIONS(1917),
    [sym_devchan_total_off_keyword] = ACTIONS(1917),
    [sym_devchan_pulse_keyword] = ACTIONS(1917),
    [sym_wait_keyword] = ACTIONS(1917),
    [sym_cancel_wait_keyword] = ACTIONS(1917),
    [sym_wait_until_keyword] = ACTIONS(1917),
    [sym_cancel_wait_until_keyword] = ACTIONS(1917),
    [sym_cancel_all_wait_keyword] = ACTIONS(1917),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1917),
    [sym_not] = ACTIONS(1917),
    [sym_bnot] = ACTIONS(1917),
    [sym___file__] = ACTIONS(1917),
    [sym___date__] = ACTIONS(1917),
    [sym___ldate__] = ACTIONS(1917),
    [sym___time__] = ACTIONS(1917),
    [sym___line__] = ACTIONS(1917),
    [sym___name__] = ACTIONS(1917),
    [sym___netlinx__] = ACTIONS(1917),
    [sym_netlinx_axi_version] = ACTIONS(1917),
    [sym_url_flg_tcp] = ACTIONS(1917),
    [sym_url_flg_accinfopresent] = ACTIONS(1917),
    [sym_url_flg_temp] = ACTIONS(1917),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1917),
    [sym_url_flg_stat_mask] = ACTIONS(1917),
    [sym_url_flg_stat_lookup] = ACTIONS(1917),
    [sym_url_flg_stat_connecting] = ACTIONS(1917),
    [sym_url_flg_stat_waiting] = ACTIONS(1917),
    [sym_url_flg_stat_connected] = ACTIONS(1917),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1917),
    [sym_first_virtual_device] = ACTIONS(1917),
    [sym_first_local_port] = ACTIONS(1917),
    [sym_dynamic_virtual_device] = ACTIONS(1917),
    [sym_timeline_once] = ACTIONS(1917),
    [sym_timeline_repeat] = ACTIONS(1917),
    [sym_timeline_absolute] = ACTIONS(1917),
    [sym_timeline_relative] = ACTIONS(1917),
    [sym_do_push_timed_infinite] = ACTIONS(1917),
    [sym_source_type_no_address] = ACTIONS(1917),
    [sym_source_type_neuron_id] = ACTIONS(1917),
    [sym_source_type_ip_address] = ACTIONS(1917),
    [sym_source_type_axlink] = ACTIONS(1917),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1917),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1917),
    [sym_source_type_ip_socket_address] = ACTIONS(1917),
    [sym_source_type_rs232] = ACTIONS(1917),
    [sym_source_type_internal] = ACTIONS(1917),
    [sym_source_type_mac_address] = ACTIONS(1917),
    [sym_source_type_ipv4_port] = ACTIONS(1917),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1917),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1917),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1917),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1917),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1917),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1917),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1917),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1917),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1917),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1917),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1917),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1917),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1917),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1917),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1917),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1917),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1917),
    [sym_true] = ACTIONS(1917),
    [sym_false] = ACTIONS(1917),
    [sym_file_read_only] = ACTIONS(1917),
    [sym_file_rw_new] = ACTIONS(1917),
    [sym_file_rw_append] = ACTIONS(1917),
    [sym_ip_tcp] = ACTIONS(1917),
    [sym_ip_udp] = ACTIONS(1917),
    [sym_ip_udp_2way] = ACTIONS(1917),
    [sym_xml_encode_types] = ACTIONS(1917),
    [sym_xml_encode_char_as_list] = ACTIONS(1917),
    [sym_xml_encode_le] = ACTIONS(1917),
    [sym_xml_decode_types] = ACTIONS(1917),
    [sym_xml_decode_no_preserve] = ACTIONS(1917),
    [sym_duet_dev_type_display_device] = ACTIONS(1917),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1917),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1917),
    [sym_duet_dev_type_io_device] = ACTIONS(1917),
    [sym_duet_dev_type_relay_device] = ACTIONS(1917),
    [sym_duet_dev_type_ups] = ACTIONS(1917),
    [sym_duet_dev_type_amplifier] = ACTIONS(1917),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1917),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1917),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1917),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1917),
    [sym_duet_dev_type_camera] = ACTIONS(1917),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1917),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1917),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1917),
    [sym_duet_dev_type_dss] = ACTIONS(1917),
    [sym_duet_dev_type_dvr] = ACTIONS(1917),
    [sym_duet_dev_type_disc_device] = ACTIONS(1917),
    [sym_duet_dev_type_document_camera] = ACTIONS(1917),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1917),
    [sym_duet_dev_type_hvac] = ACTIONS(1917),
    [sym_duet_dev_type_keypad] = ACTIONS(1917),
    [sym_duet_dev_type_light] = ACTIONS(1917),
    [sym_duet_dev_type_monitor] = ACTIONS(1917),
    [sym_duet_dev_type_motor] = ACTIONS(1917),
    [sym_duet_dev_type_multi_window] = ACTIONS(1917),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1917),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1917),
    [sym_duet_dev_type_receiver] = ACTIONS(1917),
    [sym_duet_dev_type_security_system] = ACTIONS(1917),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1917),
    [sym_duet_dev_type_settop_box] = ACTIONS(1917),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1917),
    [sym_duet_dev_type_switcher] = ACTIONS(1917),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1917),
    [sym_duet_dev_type_tv] = ACTIONS(1917),
    [sym_duet_dev_type_utility] = ACTIONS(1917),
    [sym_duet_dev_type_vcr] = ACTIONS(1917),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1917),
    [sym_duet_dev_type_video_processor] = ACTIONS(1917),
    [sym_duet_dev_type_video_projector] = ACTIONS(1917),
    [sym_duet_dev_type_video_wall] = ACTIONS(1917),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1917),
    [sym_duet_dev_type_weather] = ACTIONS(1917),
    [sym_duet_dev_not_polled] = ACTIONS(1917),
    [sym_duet_dev_polled] = ACTIONS(1917),
    [sym_clkmgr_mode_network] = ACTIONS(1917),
    [sym_clkmgr_mode_standalone] = ACTIONS(1917),
    [sym_normal_standby] = ACTIONS(1917),
    [sym_normal_wake] = ACTIONS(1917),
    [sym_ip_multicast_ttl_option] = ACTIONS(1917),
    [sym_ip_ttl_subnet] = ACTIONS(1917),
    [sym_ip_ttl_site] = ACTIONS(1917),
    [sym_ip_ttl_region] = ACTIONS(1917),
    [sym_ip_ttl_continent] = ACTIONS(1917),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1917),
    [sym_ip_nodelay_on] = ACTIONS(1917),
    [sym_ip_nodelay_off] = ACTIONS(1917),
    [sym_amx_error] = ACTIONS(1917),
    [sym_amx_warning] = ACTIONS(1917),
    [sym_amx_info] = ACTIONS(1917),
    [sym_amx_debug] = ACTIONS(1917),
    [sym_netlinx_login_success] = ACTIONS(1917),
    [sym_netlinx_login_fail] = ACTIONS(1917),
    [sym_netlinx_logout] = ACTIONS(1917),
    [sym_null_str] = ACTIONS(1917),
    [sym_smtp_address] = ACTIONS(1917),
    [sym_smtp_port_number] = ACTIONS(1917),
    [sym_smtp_username] = ACTIONS(1917),
    [sym_smtp_password] = ACTIONS(1917),
    [sym_smtp_require_tls] = ACTIONS(1917),
    [sym_smtp_from] = ACTIONS(1917),
    [sym_smtp_tls_true] = ACTIONS(1917),
    [sym_smtp_tls_false] = ACTIONS(1917),
    [sym_listview_on_row_select_event] = ACTIONS(1917),
    [sym_data_structure_unknown] = ACTIONS(1917),
    [sym_data_structure_datafeed] = ACTIONS(1917),
    [sym_data_structure_datarecordset] = ACTIONS(1917),
    [sym_data_structure_datarecord] = ACTIONS(1917),
    [sym_data_structure_datafield] = ACTIONS(1917),
    [sym_data_max_content_field_count] = ACTIONS(1917),
    [sym_data_max_metadata_field_count] = ACTIONS(1917),
    [sym_data_max_id_length] = ACTIONS(1917),
    [sym_data_max_name_length] = ACTIONS(1917),
    [sym_data_max_description_length] = ACTIONS(1917),
    [sym_data_max_type_length] = ACTIONS(1917),
    [sym_data_max_format_length] = ACTIONS(1917),
    [sym_data_max_source_length] = ACTIONS(1917),
    [sym_data_max_value_length] = ACTIONS(1917),
    [sym_data_max_label_length] = ACTIONS(1917),
    [sym_data_max_url_length] = ACTIONS(1917),
    [sym_max_last_login_info_length] = ACTIONS(1917),
    [sym_data_type_unknown] = ACTIONS(1917),
    [sym_data_type_string] = ACTIONS(1917),
    [sym_data_type_datetime] = ACTIONS(1917),
    [sym_data_type_date] = ACTIONS(1917),
    [sym_data_type_time] = ACTIONS(1917),
    [sym_data_type_image] = ACTIONS(1917),
    [sym_data_format_url] = ACTIONS(1917),
    [sym_data_format_phone] = ACTIONS(1917),
    [sym_data_format_email] = ACTIONS(1917),
    [sym_data_format_iso8601] = ACTIONS(1917),
    [sym_auditlog_priv] = ACTIONS(1917),
    [sym_configuration_priv] = ACTIONS(1917),
    [sym_deviceconfig_priv] = ACTIONS(1917),
    [sym_ftp_priv] = ACTIONS(1917),
    [sym_http_priv] = ACTIONS(1917),
    [sym_networkconfig_priv] = ACTIONS(1917),
    [sym_programport_priv] = ACTIONS(1917),
    [sym_remoteui_priv] = ACTIONS(1917),
    [sym_securitycontrol_priv] = ACTIONS(1917),
    [sym_softwaremanagement_priv] = ACTIONS(1917),
    [sym_terminal_priv] = ACTIONS(1917),
    [sym_tpadmin_priv] = ACTIONS(1917),
    [sym_user1_priv] = ACTIONS(1917),
    [sym_user2_priv] = ACTIONS(1917),
    [sym_user3_priv] = ACTIONS(1917),
    [sym_user4_priv] = ACTIONS(1917),
    [sym_usermanagement_priv] = ACTIONS(1917),
    [sym_valid_account] = ACTIONS(1917),
    [sym_err_name_invalid] = ACTIONS(1917),
    [sym_err_password_invalid] = ACTIONS(1917),
    [sym_err_invalid_account] = ACTIONS(1917),
    [sym_err_no_authorization] = ACTIONS(1917),
    [sym_err_invalid_parameter] = ACTIONS(1917),
    [sym_err_user_locked_out] = ACTIONS(1917),
    [sym_err_user_account_expired] = ACTIONS(1917),
    [sym_err_general_error] = ACTIONS(1917),
    [sym_err_authentication_error] = ACTIONS(1917),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1917),
    [sym_tls_validate_certificate] = ACTIONS(1917),
    [sym_tdata] = ACTIONS(1917),
    [sym_tchannel] = ACTIONS(1917),
    [sym_tlevel] = ACTIONS(1917),
    [sym_tbutton] = ACTIONS(1917),
    [sym_ttimeline] = ACTIONS(1917),
    [sym_tcustom] = ACTIONS(1917),
    [sym_url_struct] = ACTIONS(1917),
    [sym_dns_struct] = ACTIONS(1917),
    [sym_ip_address_struct] = ACTIONS(1917),
    [sym_dev_info_struct] = ACTIONS(1917),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1917),
    [sym_data_feed] = ACTIONS(1917),
    [sym_data_field] = ACTIONS(1917),
    [sym_data_record] = ACTIONS(1917),
    [sym_wc_data_feed] = ACTIONS(1917),
    [sym_wc_data_field] = ACTIONS(1917),
    [sym_wc_data_record] = ACTIONS(1917),
    [sym_last_login_info] = ACTIONS(1917),
    [sym____reserved___] = ACTIONS(1917),
    [sym_date] = ACTIONS(1917),
    [sym_day] = ACTIONS(1917),
    [sym_time] = ACTIONS(1917),
    [sym_ldate] = ACTIONS(1917),
    [sym_push_device] = ACTIONS(1917),
    [sym_push_channel] = ACTIONS(1917),
    [sym_push_devchan] = ACTIONS(1917),
    [sym_release_device] = ACTIONS(1917),
    [sym_release_channel] = ACTIONS(1917),
    [sym_release_devchan] = ACTIONS(1917),
    [sym_master_slot] = ACTIONS(1917),
    [sym_get_pulse_time] = ACTIONS(1917),
    [sym_get_timer] = ACTIONS(1917),
    [sym_master_sn] = ACTIONS(1917),
    [sym_system_number] = ACTIONS(1917),
    [sym_button] = ACTIONS(1917),
    [sym_dv_channel] = ACTIONS(1917),
    [sym_data] = ACTIONS(1917),
    [sym_level] = ACTIONS(1917),
    [sym_timeline] = ACTIONS(1917),
    [sym_channel] = ACTIONS(1917),
    [sym_custom] = ACTIONS(1917),
    [sym_atoi] = ACTIONS(1917),
    [sym_atol] = ACTIONS(1917),
    [sym_atof] = ACTIONS(1917),
    [sym_itoa] = ACTIONS(1917),
    [sym_itohex] = ACTIONS(1917),
    [sym_hextoi] = ACTIONS(1917),
    [sym_ftoa] = ACTIONS(1917),
    [sym_find_string] = ACTIONS(1917),
    [sym_left_string] = ACTIONS(1917),
    [sym_right_string] = ACTIONS(1917),
    [sym_mid_string] = ACTIONS(1917),
    [sym_remove_string] = ACTIONS(1917),
    [sym_get_buffer_string] = ACTIONS(1917),
    [sym_get_buffer_char] = ACTIONS(1917),
    [sym_get_multi_buffer_string] = ACTIONS(1917),
    [sym_length_string] = ACTIONS(1917),
    [sym_max_length_string] = ACTIONS(1917),
    [sym_set_length_string] = ACTIONS(1917),
    [sym_compare_string] = ACTIONS(1917),
    [sym_upper_string] = ACTIONS(1917),
    [sym_lower_string] = ACTIONS(1917),
    [sym_get_last] = ACTIONS(1917),
    [sym_date_to_day] = ACTIONS(1917),
    [sym_date_to_month] = ACTIONS(1917),
    [sym_date_to_year] = ACTIONS(1917),
    [sym_time_to_hour] = ACTIONS(1917),
    [sym_time_to_minute] = ACTIONS(1917),
    [sym_time_to_second] = ACTIONS(1917),
    [sym_day_of_week] = ACTIONS(1917),
    [sym_set_pulse_time] = ACTIONS(1917),
    [sym_set_timer] = ACTIONS(1917),
    [sym_file_open] = ACTIONS(1917),
    [sym_file_close] = ACTIONS(1917),
    [sym_file_read] = ACTIONS(1917),
    [sym_file_read_line] = ACTIONS(1917),
    [sym_file_write] = ACTIONS(1917),
    [sym_file_write_line] = ACTIONS(1917),
    [sym_file_copy] = ACTIONS(1917),
    [sym_file_delete] = ACTIONS(1917),
    [sym_file_rename] = ACTIONS(1917),
    [sym_file_dir] = ACTIONS(1917),
    [sym_file_getdir] = ACTIONS(1917),
    [sym_file_setdir] = ACTIONS(1917),
    [sym_file_create_dir] = ACTIONS(1917),
    [sym_file_remove_dir] = ACTIONS(1917),
    [sym_file_seek] = ACTIONS(1917),
    [sym_length_array] = ACTIONS(1917),
    [sym_max_length_array] = ACTIONS(1917),
    [sym_set_length_array] = ACTIONS(1917),
    [sym_ip_client_open] = ACTIONS(1917),
    [sym_ip_client_close] = ACTIONS(1917),
    [sym_ip_server_open] = ACTIONS(1917),
    [sym_ip_server_close] = ACTIONS(1917),
    [sym_ip_bound_client_open] = ACTIONS(1917),
    [sym_ip_mc_server_open] = ACTIONS(1917),
    [sym_ip_set_option] = ACTIONS(1917),
    [sym_ssh_client_open] = ACTIONS(1917),
    [sym_ssh_client_close] = ACTIONS(1917),
    [sym_tls_client_open] = ACTIONS(1917),
    [sym_tls_client_close] = ACTIONS(1917),
    [sym_do_push] = ACTIONS(1917),
    [sym_do_push_timed] = ACTIONS(1917),
    [sym_do_release] = ACTIONS(1917),
    [sym_device_id] = ACTIONS(1917),
    [sym_device_info] = ACTIONS(1917),
    [sym_device_id_string] = ACTIONS(1917),
    [sym_device_standby] = ACTIONS(1917),
    [sym_device_wake] = ACTIONS(1917),
    [sym_timeline_create] = ACTIONS(1917),
    [sym_timeline_kill] = ACTIONS(1917),
    [sym_timeline_reload] = ACTIONS(1917),
    [sym_timeline_pause] = ACTIONS(1917),
    [sym_timeline_restart] = ACTIONS(1917),
    [sym_timeline_set] = ACTIONS(1917),
    [sym_timeline_get] = ACTIONS(1917),
    [sym_timeline_active] = ACTIONS(1917),
    [sym_timeline_dynamic_id] = ACTIONS(1917),
    [sym_variable_to_string] = ACTIONS(1917),
    [sym_string_to_variable] = ACTIONS(1917),
    [sym_length_variable_to_string] = ACTIONS(1917),
    [sym_type_cast] = ACTIONS(1917),
    [sym_variable_to_xml] = ACTIONS(1917),
    [sym_xml_to_variable] = ACTIONS(1917),
    [sym_length_variable_to_xml] = ACTIONS(1917),
    [sym_redirect_string] = ACTIONS(1917),
    [sym_random_number] = ACTIONS(1917),
    [sym_get_unique_id] = ACTIONS(1917),
    [sym_get_system_number] = ACTIONS(1917),
    [sym_set_system_number] = ACTIONS(1917),
    [sym_get_serial_number] = ACTIONS(1917),
    [sym_get_master_build] = ACTIONS(1917),
    [sym_set_validation_code] = ACTIONS(1917),
    [sym_reboot] = ACTIONS(1917),
    [sym_load_duet_module] = ACTIONS(1917),
    [sym_unload_duet_module] = ACTIONS(1917),
    [sym_duet_mem_size_set] = ACTIONS(1917),
    [sym_duet_mem_size_get] = ACTIONS(1917),
    [sym_dynamic_application_device] = ACTIONS(1917),
    [sym_dynamic_polled_port] = ACTIONS(1917),
    [sym_static_port_binding] = ACTIONS(1917),
    [sym_static_ip_binding] = ACTIONS(1917),
    [sym_get_dns_list] = ACTIONS(1917),
    [sym_set_dns_list] = ACTIONS(1917),
    [sym_get_ip_address] = ACTIONS(1917),
    [sym_set_ip_address] = ACTIONS(1917),
    [sym_get_url_list] = ACTIONS(1917),
    [sym_add_url_entry] = ACTIONS(1917),
    [sym_delete_url_entry] = ACTIONS(1917),
    [sym_abs_value] = ACTIONS(1917),
    [sym_max_value] = ACTIONS(1917),
    [sym_min_value] = ACTIONS(1917),
    [sym_exp_value] = ACTIONS(1917),
    [sym_log_value] = ACTIONS(1917),
    [sym_log10_value] = ACTIONS(1917),
    [sym_power_value] = ACTIONS(1917),
    [sym_sqrt_value] = ACTIONS(1917),
    [sym_raw_be] = ACTIONS(1917),
    [sym_raw_le] = ACTIONS(1917),
    [sym_internal_queue_size_set] = ACTIONS(1917),
    [sym_internal_queue_size_get] = ACTIONS(1917),
    [sym_internal_threshold_set] = ACTIONS(1917),
    [sym_internal_threshold_get] = ACTIONS(1917),
    [sym_rebuilt_event] = ACTIONS(1917),
    [sym_do_custom_event] = ACTIONS(1917),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1917),
    [sym_clkmgr_set_clk_source] = ACTIONS(1917),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1917),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1917),
    [sym_clkmgr_get_timezone] = ACTIONS(1917),
    [sym_clkmgr_set_timezone] = ACTIONS(1917),
    [sym_clkmgr_get_resync_period] = ACTIONS(1917),
    [sym_clkmgr_set_resync_period] = ACTIONS(1917),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1917),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1917),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1917),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1917),
    [sym_clkmgr_get_timeservers] = ACTIONS(1917),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1917),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1917),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1917),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1917),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1917),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1917),
    [sym_set_log_level] = ACTIONS(1917),
    [sym_get_log_level] = ACTIONS(1917),
    [sym_amx_log] = ACTIONS(1917),
    [sym_smtp_server_config_set] = ACTIONS(1917),
    [sym_smtp_server_config_get] = ACTIONS(1917),
    [sym_smtp_send] = ACTIONS(1917),
    [sym_validate_netlinx_account] = ACTIONS(1917),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1917),
    [sym_authenticate_certificate] = ACTIONS(1917),
    [sym_audit_netlinx_session_event] = ACTIONS(1917),
    [sym_audit_netlinx_generic_event] = ACTIONS(1917),
    [sym_get_available_flash_disk_space] = ACTIONS(1917),
    [sym_get_max_flash_disk_space] = ACTIONS(1917),
    [sym_data_create_feed] = ACTIONS(1917),
    [sym_data_delete_feed] = ACTIONS(1917),
    [sym_data_publish_feed] = ACTIONS(1917),
    [sym_data_get_published_feed] = ACTIONS(1917),
    [sym_data_add_record] = ACTIONS(1917),
    [sym_data_get_event_record] = ACTIONS(1917),
    [sym__wc_data_create_feed] = ACTIONS(1917),
    [sym__wc_data_add_record] = ACTIONS(1917),
    [sym__wc_data_get_event_record] = ACTIONS(1917),
    [sym_format] = ACTIONS(1917),
    [sym_astro_clock] = ACTIONS(1917),
    [sym_set_outdoor_temperature] = ACTIONS(1917),
    [sym_set_virtual_level_count] = ACTIONS(1917),
    [sym_set_virtual_channel_count] = ACTIONS(1917),
    [sym_set_virtual_port_count] = ACTIONS(1917),
    [sym__wc_encode_int] = ACTIONS(1917),
    [sym__wc_decode_int] = ACTIONS(1917),
    [sym__wc_int] = ACTIONS(1917),
    [sym__wc_to_ch_int] = ACTIONS(1917),
    [sym__ch_to_wc_int] = ACTIONS(1917),
    [sym__wc_find_string_int] = ACTIONS(1917),
    [sym__wc_left_string_int] = ACTIONS(1917),
    [sym__wc_lower_string_int] = ACTIONS(1917),
    [sym__wc_mid_string_int] = ACTIONS(1917),
    [sym__wc_remove_string_int] = ACTIONS(1917),
    [sym__wc_right_string_int] = ACTIONS(1917),
    [sym__wc_upper_string_int] = ACTIONS(1917),
    [sym__wc_compare_string_int] = ACTIONS(1917),
    [sym__wc_get_buffer_char_int] = ACTIONS(1917),
    [sym__wc_get_buffer_string_int] = ACTIONS(1917),
    [sym__wc_concat_string_int] = ACTIONS(1917),
    [sym__wc_file_open_int] = ACTIONS(1917),
    [sym__wc_file_close_int] = ACTIONS(1917),
    [sym__wc_file_read_int] = ACTIONS(1917),
    [sym__wc_file_read_line_int] = ACTIONS(1917),
    [sym__wc_file_write_int] = ACTIONS(1917),
    [sym__wc_file_write_line_int] = ACTIONS(1917),
    [aux_sym_preproc_include_token1] = ACTIONS(1917),
    [aux_sym_preproc_define_token1] = ACTIONS(1919),
    [aux_sym_preproc_warn_token1] = ACTIONS(1919),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1919),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1919),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1919),
    [anon_sym_LBRACK] = ACTIONS(1919),
    [anon_sym_LBRACE] = ACTIONS(1919),
    [anon_sym_LPAREN] = ACTIONS(1917),
    [anon_sym_BANG] = ACTIONS(1919),
    [anon_sym_TILDE] = ACTIONS(1919),
    [anon_sym_DASH] = ACTIONS(1917),
    [anon_sym_PLUS] = ACTIONS(1917),
    [anon_sym_DASH_DASH] = ACTIONS(1919),
    [anon_sym_PLUS_PLUS] = ACTIONS(1919),
    [anon_sym_DQUOTE] = ACTIONS(1919),
    [anon_sym_SQUOTE] = ACTIONS(1919),
    [sym_number_literal] = ACTIONS(1919),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(437)] = {
    [ts_builtin_sym_end] = ACTIONS(1927),
    [sym_identifier] = ACTIONS(1925),
    [sym_program_name_keyword] = ACTIONS(1925),
    [sym_module_name_keyword] = ACTIONS(1925),
    [sym_define_device_keyword] = ACTIONS(1925),
    [sym_define_combine_keyword] = ACTIONS(1925),
    [sym_define_constant_keyword] = ACTIONS(1925),
    [sym_define_type_keyword] = ACTIONS(1925),
    [sym_define_variable_keyword] = ACTIONS(1925),
    [sym_define_system_variable_keyword] = ACTIONS(1925),
    [sym_define_connect_level_keyword] = ACTIONS(1925),
    [sym_define_latching_keyword] = ACTIONS(1925),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1925),
    [sym_define_toggling_keyword] = ACTIONS(1925),
    [sym_define_call_keyword] = ACTIONS(1925),
    [sym_define_function_keyword] = ACTIONS(1925),
    [sym_define_library_function_keyword] = ACTIONS(1925),
    [sym_define_start_keyword] = ACTIONS(1925),
    [sym_define_event_keyword] = ACTIONS(1925),
    [sym_define_program_keyword] = ACTIONS(1925),
    [sym_define_module_keyword] = ACTIONS(1925),
    [sym_char_keyword] = ACTIONS(1925),
    [sym_widechar_keyword] = ACTIONS(1925),
    [sym_integer_keyword] = ACTIONS(1925),
    [sym_sinteger_keyword] = ACTIONS(1925),
    [sym_long_keyword] = ACTIONS(1925),
    [sym_slong_keyword] = ACTIONS(1925),
    [sym_float_keyword] = ACTIONS(1925),
    [sym_double_keyword] = ACTIONS(1925),
    [sym_variant_keyword] = ACTIONS(1925),
    [sym_variantarray_keyword] = ACTIONS(1925),
    [sym_local_var_keyword] = ACTIONS(1925),
    [sym_stack_var_keyword] = ACTIONS(1925),
    [sym_constant_keyword] = ACTIONS(1925),
    [sym_volatile_keyword] = ACTIONS(1925),
    [sym_non_volatile_keyword] = ACTIONS(1925),
    [sym_persistent_keyword] = ACTIONS(1925),
    [sym_dev_keyword] = ACTIONS(1925),
    [sym_devlev_keyword] = ACTIONS(1925),
    [sym_devchan_keyword] = ACTIONS(1925),
    [sym_if_keyword] = ACTIONS(1925),
    [sym_switch_keyword] = ACTIONS(1925),
    [sym_case_keyword] = ACTIONS(1925),
    [sym_default_keyword] = ACTIONS(1925),
    [sym_while_keyword] = ACTIONS(1925),
    [sym_for_keyword] = ACTIONS(1925),
    [sym_break_keyword] = ACTIONS(1925),
    [sym_continue_keyword] = ACTIONS(1925),
    [sym_return_keyword] = ACTIONS(1925),
    [sym_select_keyword] = ACTIONS(1925),
    [sym_active_keyword] = ACTIONS(1925),
    [sym_struct_keyword] = ACTIONS(1925),
    [sym_structure_keyword] = ACTIONS(1925),
    [sym_button_event_keyword] = ACTIONS(1925),
    [sym_channel_event_keyword] = ACTIONS(1925),
    [sym_level_event_keyword] = ACTIONS(1925),
    [sym_data_event_keyword] = ACTIONS(1925),
    [sym_timeline_event_keyword] = ACTIONS(1925),
    [sym_custom_event_keyword] = ACTIONS(1925),
    [sym_push_keyword] = ACTIONS(1925),
    [sym_release_keyword] = ACTIONS(1925),
    [aux_sym_on_keyword_token1] = ACTIONS(1925),
    [aux_sym_off_keyword_token1] = ACTIONS(1925),
    [sym_send_level_keyword] = ACTIONS(1925),
    [sym_send_string_keyword] = ACTIONS(1925),
    [sym_send_command_keyword] = ACTIONS(1925),
    [sym_clear_buffer_keyword] = ACTIONS(1925),
    [sym_create_buffer_keyword] = ACTIONS(1925),
    [sym_create_multi_buffer_keyword] = ACTIONS(1925),
    [sym_call_keyword] = ACTIONS(1925),
    [sym_system_call_keyword] = ACTIONS(1925),
    [sym_devchan_to_keyword] = ACTIONS(1925),
    [sym_devchan_min_to_keyword] = ACTIONS(1925),
    [sym_devchan_total_off_keyword] = ACTIONS(1925),
    [sym_devchan_pulse_keyword] = ACTIONS(1925),
    [sym_wait_keyword] = ACTIONS(1925),
    [sym_cancel_wait_keyword] = ACTIONS(1925),
    [sym_wait_until_keyword] = ACTIONS(1925),
    [sym_cancel_wait_until_keyword] = ACTIONS(1925),
    [sym_cancel_all_wait_keyword] = ACTIONS(1925),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1925),
    [sym_not] = ACTIONS(1925),
    [sym_bnot] = ACTIONS(1925),
    [sym___file__] = ACTIONS(1925),
    [sym___date__] = ACTIONS(1925),
    [sym___ldate__] = ACTIONS(1925),
    [sym___time__] = ACTIONS(1925),
    [sym___line__] = ACTIONS(1925),
    [sym___name__] = ACTIONS(1925),
    [sym___netlinx__] = ACTIONS(1925),
    [sym_netlinx_axi_version] = ACTIONS(1925),
    [sym_url_flg_tcp] = ACTIONS(1925),
    [sym_url_flg_accinfopresent] = ACTIONS(1925),
    [sym_url_flg_temp] = ACTIONS(1925),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1925),
    [sym_url_flg_stat_mask] = ACTIONS(1925),
    [sym_url_flg_stat_lookup] = ACTIONS(1925),
    [sym_url_flg_stat_connecting] = ACTIONS(1925),
    [sym_url_flg_stat_waiting] = ACTIONS(1925),
    [sym_url_flg_stat_connected] = ACTIONS(1925),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1925),
    [sym_first_virtual_device] = ACTIONS(1925),
    [sym_first_local_port] = ACTIONS(1925),
    [sym_dynamic_virtual_device] = ACTIONS(1925),
    [sym_timeline_once] = ACTIONS(1925),
    [sym_timeline_repeat] = ACTIONS(1925),
    [sym_timeline_absolute] = ACTIONS(1925),
    [sym_timeline_relative] = ACTIONS(1925),
    [sym_do_push_timed_infinite] = ACTIONS(1925),
    [sym_source_type_no_address] = ACTIONS(1925),
    [sym_source_type_neuron_id] = ACTIONS(1925),
    [sym_source_type_ip_address] = ACTIONS(1925),
    [sym_source_type_axlink] = ACTIONS(1925),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1925),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1925),
    [sym_source_type_ip_socket_address] = ACTIONS(1925),
    [sym_source_type_rs232] = ACTIONS(1925),
    [sym_source_type_internal] = ACTIONS(1925),
    [sym_source_type_mac_address] = ACTIONS(1925),
    [sym_source_type_ipv4_port] = ACTIONS(1925),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1925),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1925),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1925),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1925),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1925),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1925),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1925),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1925),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1925),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1925),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1925),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1925),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1925),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1925),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1925),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1925),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1925),
    [sym_true] = ACTIONS(1925),
    [sym_false] = ACTIONS(1925),
    [sym_file_read_only] = ACTIONS(1925),
    [sym_file_rw_new] = ACTIONS(1925),
    [sym_file_rw_append] = ACTIONS(1925),
    [sym_ip_tcp] = ACTIONS(1925),
    [sym_ip_udp] = ACTIONS(1925),
    [sym_ip_udp_2way] = ACTIONS(1925),
    [sym_xml_encode_types] = ACTIONS(1925),
    [sym_xml_encode_char_as_list] = ACTIONS(1925),
    [sym_xml_encode_le] = ACTIONS(1925),
    [sym_xml_decode_types] = ACTIONS(1925),
    [sym_xml_decode_no_preserve] = ACTIONS(1925),
    [sym_duet_dev_type_display_device] = ACTIONS(1925),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1925),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1925),
    [sym_duet_dev_type_io_device] = ACTIONS(1925),
    [sym_duet_dev_type_relay_device] = ACTIONS(1925),
    [sym_duet_dev_type_ups] = ACTIONS(1925),
    [sym_duet_dev_type_amplifier] = ACTIONS(1925),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1925),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1925),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1925),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1925),
    [sym_duet_dev_type_camera] = ACTIONS(1925),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1925),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1925),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1925),
    [sym_duet_dev_type_dss] = ACTIONS(1925),
    [sym_duet_dev_type_dvr] = ACTIONS(1925),
    [sym_duet_dev_type_disc_device] = ACTIONS(1925),
    [sym_duet_dev_type_document_camera] = ACTIONS(1925),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1925),
    [sym_duet_dev_type_hvac] = ACTIONS(1925),
    [sym_duet_dev_type_keypad] = ACTIONS(1925),
    [sym_duet_dev_type_light] = ACTIONS(1925),
    [sym_duet_dev_type_monitor] = ACTIONS(1925),
    [sym_duet_dev_type_motor] = ACTIONS(1925),
    [sym_duet_dev_type_multi_window] = ACTIONS(1925),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1925),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1925),
    [sym_duet_dev_type_receiver] = ACTIONS(1925),
    [sym_duet_dev_type_security_system] = ACTIONS(1925),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1925),
    [sym_duet_dev_type_settop_box] = ACTIONS(1925),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1925),
    [sym_duet_dev_type_switcher] = ACTIONS(1925),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1925),
    [sym_duet_dev_type_tv] = ACTIONS(1925),
    [sym_duet_dev_type_utility] = ACTIONS(1925),
    [sym_duet_dev_type_vcr] = ACTIONS(1925),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1925),
    [sym_duet_dev_type_video_processor] = ACTIONS(1925),
    [sym_duet_dev_type_video_projector] = ACTIONS(1925),
    [sym_duet_dev_type_video_wall] = ACTIONS(1925),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1925),
    [sym_duet_dev_type_weather] = ACTIONS(1925),
    [sym_duet_dev_not_polled] = ACTIONS(1925),
    [sym_duet_dev_polled] = ACTIONS(1925),
    [sym_clkmgr_mode_network] = ACTIONS(1925),
    [sym_clkmgr_mode_standalone] = ACTIONS(1925),
    [sym_normal_standby] = ACTIONS(1925),
    [sym_normal_wake] = ACTIONS(1925),
    [sym_ip_multicast_ttl_option] = ACTIONS(1925),
    [sym_ip_ttl_subnet] = ACTIONS(1925),
    [sym_ip_ttl_site] = ACTIONS(1925),
    [sym_ip_ttl_region] = ACTIONS(1925),
    [sym_ip_ttl_continent] = ACTIONS(1925),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1925),
    [sym_ip_nodelay_on] = ACTIONS(1925),
    [sym_ip_nodelay_off] = ACTIONS(1925),
    [sym_amx_error] = ACTIONS(1925),
    [sym_amx_warning] = ACTIONS(1925),
    [sym_amx_info] = ACTIONS(1925),
    [sym_amx_debug] = ACTIONS(1925),
    [sym_netlinx_login_success] = ACTIONS(1925),
    [sym_netlinx_login_fail] = ACTIONS(1925),
    [sym_netlinx_logout] = ACTIONS(1925),
    [sym_null_str] = ACTIONS(1925),
    [sym_smtp_address] = ACTIONS(1925),
    [sym_smtp_port_number] = ACTIONS(1925),
    [sym_smtp_username] = ACTIONS(1925),
    [sym_smtp_password] = ACTIONS(1925),
    [sym_smtp_require_tls] = ACTIONS(1925),
    [sym_smtp_from] = ACTIONS(1925),
    [sym_smtp_tls_true] = ACTIONS(1925),
    [sym_smtp_tls_false] = ACTIONS(1925),
    [sym_listview_on_row_select_event] = ACTIONS(1925),
    [sym_data_structure_unknown] = ACTIONS(1925),
    [sym_data_structure_datafeed] = ACTIONS(1925),
    [sym_data_structure_datarecordset] = ACTIONS(1925),
    [sym_data_structure_datarecord] = ACTIONS(1925),
    [sym_data_structure_datafield] = ACTIONS(1925),
    [sym_data_max_content_field_count] = ACTIONS(1925),
    [sym_data_max_metadata_field_count] = ACTIONS(1925),
    [sym_data_max_id_length] = ACTIONS(1925),
    [sym_data_max_name_length] = ACTIONS(1925),
    [sym_data_max_description_length] = ACTIONS(1925),
    [sym_data_max_type_length] = ACTIONS(1925),
    [sym_data_max_format_length] = ACTIONS(1925),
    [sym_data_max_source_length] = ACTIONS(1925),
    [sym_data_max_value_length] = ACTIONS(1925),
    [sym_data_max_label_length] = ACTIONS(1925),
    [sym_data_max_url_length] = ACTIONS(1925),
    [sym_max_last_login_info_length] = ACTIONS(1925),
    [sym_data_type_unknown] = ACTIONS(1925),
    [sym_data_type_string] = ACTIONS(1925),
    [sym_data_type_datetime] = ACTIONS(1925),
    [sym_data_type_date] = ACTIONS(1925),
    [sym_data_type_time] = ACTIONS(1925),
    [sym_data_type_image] = ACTIONS(1925),
    [sym_data_format_url] = ACTIONS(1925),
    [sym_data_format_phone] = ACTIONS(1925),
    [sym_data_format_email] = ACTIONS(1925),
    [sym_data_format_iso8601] = ACTIONS(1925),
    [sym_auditlog_priv] = ACTIONS(1925),
    [sym_configuration_priv] = ACTIONS(1925),
    [sym_deviceconfig_priv] = ACTIONS(1925),
    [sym_ftp_priv] = ACTIONS(1925),
    [sym_http_priv] = ACTIONS(1925),
    [sym_networkconfig_priv] = ACTIONS(1925),
    [sym_programport_priv] = ACTIONS(1925),
    [sym_remoteui_priv] = ACTIONS(1925),
    [sym_securitycontrol_priv] = ACTIONS(1925),
    [sym_softwaremanagement_priv] = ACTIONS(1925),
    [sym_terminal_priv] = ACTIONS(1925),
    [sym_tpadmin_priv] = ACTIONS(1925),
    [sym_user1_priv] = ACTIONS(1925),
    [sym_user2_priv] = ACTIONS(1925),
    [sym_user3_priv] = ACTIONS(1925),
    [sym_user4_priv] = ACTIONS(1925),
    [sym_usermanagement_priv] = ACTIONS(1925),
    [sym_valid_account] = ACTIONS(1925),
    [sym_err_name_invalid] = ACTIONS(1925),
    [sym_err_password_invalid] = ACTIONS(1925),
    [sym_err_invalid_account] = ACTIONS(1925),
    [sym_err_no_authorization] = ACTIONS(1925),
    [sym_err_invalid_parameter] = ACTIONS(1925),
    [sym_err_user_locked_out] = ACTIONS(1925),
    [sym_err_user_account_expired] = ACTIONS(1925),
    [sym_err_general_error] = ACTIONS(1925),
    [sym_err_authentication_error] = ACTIONS(1925),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1925),
    [sym_tls_validate_certificate] = ACTIONS(1925),
    [sym_tdata] = ACTIONS(1925),
    [sym_tchannel] = ACTIONS(1925),
    [sym_tlevel] = ACTIONS(1925),
    [sym_tbutton] = ACTIONS(1925),
    [sym_ttimeline] = ACTIONS(1925),
    [sym_tcustom] = ACTIONS(1925),
    [sym_url_struct] = ACTIONS(1925),
    [sym_dns_struct] = ACTIONS(1925),
    [sym_ip_address_struct] = ACTIONS(1925),
    [sym_dev_info_struct] = ACTIONS(1925),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1925),
    [sym_data_feed] = ACTIONS(1925),
    [sym_data_field] = ACTIONS(1925),
    [sym_data_record] = ACTIONS(1925),
    [sym_wc_data_feed] = ACTIONS(1925),
    [sym_wc_data_field] = ACTIONS(1925),
    [sym_wc_data_record] = ACTIONS(1925),
    [sym_last_login_info] = ACTIONS(1925),
    [sym____reserved___] = ACTIONS(1925),
    [sym_date] = ACTIONS(1925),
    [sym_day] = ACTIONS(1925),
    [sym_time] = ACTIONS(1925),
    [sym_ldate] = ACTIONS(1925),
    [sym_push_device] = ACTIONS(1925),
    [sym_push_channel] = ACTIONS(1925),
    [sym_push_devchan] = ACTIONS(1925),
    [sym_release_device] = ACTIONS(1925),
    [sym_release_channel] = ACTIONS(1925),
    [sym_release_devchan] = ACTIONS(1925),
    [sym_master_slot] = ACTIONS(1925),
    [sym_get_pulse_time] = ACTIONS(1925),
    [sym_get_timer] = ACTIONS(1925),
    [sym_master_sn] = ACTIONS(1925),
    [sym_system_number] = ACTIONS(1925),
    [sym_button] = ACTIONS(1925),
    [sym_dv_channel] = ACTIONS(1925),
    [sym_data] = ACTIONS(1925),
    [sym_level] = ACTIONS(1925),
    [sym_timeline] = ACTIONS(1925),
    [sym_channel] = ACTIONS(1925),
    [sym_custom] = ACTIONS(1925),
    [sym_atoi] = ACTIONS(1925),
    [sym_atol] = ACTIONS(1925),
    [sym_atof] = ACTIONS(1925),
    [sym_itoa] = ACTIONS(1925),
    [sym_itohex] = ACTIONS(1925),
    [sym_hextoi] = ACTIONS(1925),
    [sym_ftoa] = ACTIONS(1925),
    [sym_find_string] = ACTIONS(1925),
    [sym_left_string] = ACTIONS(1925),
    [sym_right_string] = ACTIONS(1925),
    [sym_mid_string] = ACTIONS(1925),
    [sym_remove_string] = ACTIONS(1925),
    [sym_get_buffer_string] = ACTIONS(1925),
    [sym_get_buffer_char] = ACTIONS(1925),
    [sym_get_multi_buffer_string] = ACTIONS(1925),
    [sym_length_string] = ACTIONS(1925),
    [sym_max_length_string] = ACTIONS(1925),
    [sym_set_length_string] = ACTIONS(1925),
    [sym_compare_string] = ACTIONS(1925),
    [sym_upper_string] = ACTIONS(1925),
    [sym_lower_string] = ACTIONS(1925),
    [sym_get_last] = ACTIONS(1925),
    [sym_date_to_day] = ACTIONS(1925),
    [sym_date_to_month] = ACTIONS(1925),
    [sym_date_to_year] = ACTIONS(1925),
    [sym_time_to_hour] = ACTIONS(1925),
    [sym_time_to_minute] = ACTIONS(1925),
    [sym_time_to_second] = ACTIONS(1925),
    [sym_day_of_week] = ACTIONS(1925),
    [sym_set_pulse_time] = ACTIONS(1925),
    [sym_set_timer] = ACTIONS(1925),
    [sym_file_open] = ACTIONS(1925),
    [sym_file_close] = ACTIONS(1925),
    [sym_file_read] = ACTIONS(1925),
    [sym_file_read_line] = ACTIONS(1925),
    [sym_file_write] = ACTIONS(1925),
    [sym_file_write_line] = ACTIONS(1925),
    [sym_file_copy] = ACTIONS(1925),
    [sym_file_delete] = ACTIONS(1925),
    [sym_file_rename] = ACTIONS(1925),
    [sym_file_dir] = ACTIONS(1925),
    [sym_file_getdir] = ACTIONS(1925),
    [sym_file_setdir] = ACTIONS(1925),
    [sym_file_create_dir] = ACTIONS(1925),
    [sym_file_remove_dir] = ACTIONS(1925),
    [sym_file_seek] = ACTIONS(1925),
    [sym_length_array] = ACTIONS(1925),
    [sym_max_length_array] = ACTIONS(1925),
    [sym_set_length_array] = ACTIONS(1925),
    [sym_ip_client_open] = ACTIONS(1925),
    [sym_ip_client_close] = ACTIONS(1925),
    [sym_ip_server_open] = ACTIONS(1925),
    [sym_ip_server_close] = ACTIONS(1925),
    [sym_ip_bound_client_open] = ACTIONS(1925),
    [sym_ip_mc_server_open] = ACTIONS(1925),
    [sym_ip_set_option] = ACTIONS(1925),
    [sym_ssh_client_open] = ACTIONS(1925),
    [sym_ssh_client_close] = ACTIONS(1925),
    [sym_tls_client_open] = ACTIONS(1925),
    [sym_tls_client_close] = ACTIONS(1925),
    [sym_do_push] = ACTIONS(1925),
    [sym_do_push_timed] = ACTIONS(1925),
    [sym_do_release] = ACTIONS(1925),
    [sym_device_id] = ACTIONS(1925),
    [sym_device_info] = ACTIONS(1925),
    [sym_device_id_string] = ACTIONS(1925),
    [sym_device_standby] = ACTIONS(1925),
    [sym_device_wake] = ACTIONS(1925),
    [sym_timeline_create] = ACTIONS(1925),
    [sym_timeline_kill] = ACTIONS(1925),
    [sym_timeline_reload] = ACTIONS(1925),
    [sym_timeline_pause] = ACTIONS(1925),
    [sym_timeline_restart] = ACTIONS(1925),
    [sym_timeline_set] = ACTIONS(1925),
    [sym_timeline_get] = ACTIONS(1925),
    [sym_timeline_active] = ACTIONS(1925),
    [sym_timeline_dynamic_id] = ACTIONS(1925),
    [sym_variable_to_string] = ACTIONS(1925),
    [sym_string_to_variable] = ACTIONS(1925),
    [sym_length_variable_to_string] = ACTIONS(1925),
    [sym_type_cast] = ACTIONS(1925),
    [sym_variable_to_xml] = ACTIONS(1925),
    [sym_xml_to_variable] = ACTIONS(1925),
    [sym_length_variable_to_xml] = ACTIONS(1925),
    [sym_redirect_string] = ACTIONS(1925),
    [sym_random_number] = ACTIONS(1925),
    [sym_get_unique_id] = ACTIONS(1925),
    [sym_get_system_number] = ACTIONS(1925),
    [sym_set_system_number] = ACTIONS(1925),
    [sym_get_serial_number] = ACTIONS(1925),
    [sym_get_master_build] = ACTIONS(1925),
    [sym_set_validation_code] = ACTIONS(1925),
    [sym_reboot] = ACTIONS(1925),
    [sym_load_duet_module] = ACTIONS(1925),
    [sym_unload_duet_module] = ACTIONS(1925),
    [sym_duet_mem_size_set] = ACTIONS(1925),
    [sym_duet_mem_size_get] = ACTIONS(1925),
    [sym_dynamic_application_device] = ACTIONS(1925),
    [sym_dynamic_polled_port] = ACTIONS(1925),
    [sym_static_port_binding] = ACTIONS(1925),
    [sym_static_ip_binding] = ACTIONS(1925),
    [sym_get_dns_list] = ACTIONS(1925),
    [sym_set_dns_list] = ACTIONS(1925),
    [sym_get_ip_address] = ACTIONS(1925),
    [sym_set_ip_address] = ACTIONS(1925),
    [sym_get_url_list] = ACTIONS(1925),
    [sym_add_url_entry] = ACTIONS(1925),
    [sym_delete_url_entry] = ACTIONS(1925),
    [sym_abs_value] = ACTIONS(1925),
    [sym_max_value] = ACTIONS(1925),
    [sym_min_value] = ACTIONS(1925),
    [sym_exp_value] = ACTIONS(1925),
    [sym_log_value] = ACTIONS(1925),
    [sym_log10_value] = ACTIONS(1925),
    [sym_power_value] = ACTIONS(1925),
    [sym_sqrt_value] = ACTIONS(1925),
    [sym_raw_be] = ACTIONS(1925),
    [sym_raw_le] = ACTIONS(1925),
    [sym_internal_queue_size_set] = ACTIONS(1925),
    [sym_internal_queue_size_get] = ACTIONS(1925),
    [sym_internal_threshold_set] = ACTIONS(1925),
    [sym_internal_threshold_get] = ACTIONS(1925),
    [sym_rebuilt_event] = ACTIONS(1925),
    [sym_do_custom_event] = ACTIONS(1925),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1925),
    [sym_clkmgr_set_clk_source] = ACTIONS(1925),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1925),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1925),
    [sym_clkmgr_get_timezone] = ACTIONS(1925),
    [sym_clkmgr_set_timezone] = ACTIONS(1925),
    [sym_clkmgr_get_resync_period] = ACTIONS(1925),
    [sym_clkmgr_set_resync_period] = ACTIONS(1925),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1925),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1925),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1925),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1925),
    [sym_clkmgr_get_timeservers] = ACTIONS(1925),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1925),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1925),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1925),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1925),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1925),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1925),
    [sym_set_log_level] = ACTIONS(1925),
    [sym_get_log_level] = ACTIONS(1925),
    [sym_amx_log] = ACTIONS(1925),
    [sym_smtp_server_config_set] = ACTIONS(1925),
    [sym_smtp_server_config_get] = ACTIONS(1925),
    [sym_smtp_send] = ACTIONS(1925),
    [sym_validate_netlinx_account] = ACTIONS(1925),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1925),
    [sym_authenticate_certificate] = ACTIONS(1925),
    [sym_audit_netlinx_session_event] = ACTIONS(1925),
    [sym_audit_netlinx_generic_event] = ACTIONS(1925),
    [sym_get_available_flash_disk_space] = ACTIONS(1925),
    [sym_get_max_flash_disk_space] = ACTIONS(1925),
    [sym_data_create_feed] = ACTIONS(1925),
    [sym_data_delete_feed] = ACTIONS(1925),
    [sym_data_publish_feed] = ACTIONS(1925),
    [sym_data_get_published_feed] = ACTIONS(1925),
    [sym_data_add_record] = ACTIONS(1925),
    [sym_data_get_event_record] = ACTIONS(1925),
    [sym__wc_data_create_feed] = ACTIONS(1925),
    [sym__wc_data_add_record] = ACTIONS(1925),
    [sym__wc_data_get_event_record] = ACTIONS(1925),
    [sym_format] = ACTIONS(1925),
    [sym_astro_clock] = ACTIONS(1925),
    [sym_set_outdoor_temperature] = ACTIONS(1925),
    [sym_set_virtual_level_count] = ACTIONS(1925),
    [sym_set_virtual_channel_count] = ACTIONS(1925),
    [sym_set_virtual_port_count] = ACTIONS(1925),
    [sym__wc_encode_int] = ACTIONS(1925),
    [sym__wc_decode_int] = ACTIONS(1925),
    [sym__wc_int] = ACTIONS(1925),
    [sym__wc_to_ch_int] = ACTIONS(1925),
    [sym__ch_to_wc_int] = ACTIONS(1925),
    [sym__wc_find_string_int] = ACTIONS(1925),
    [sym__wc_left_string_int] = ACTIONS(1925),
    [sym__wc_lower_string_int] = ACTIONS(1925),
    [sym__wc_mid_string_int] = ACTIONS(1925),
    [sym__wc_remove_string_int] = ACTIONS(1925),
    [sym__wc_right_string_int] = ACTIONS(1925),
    [sym__wc_upper_string_int] = ACTIONS(1925),
    [sym__wc_compare_string_int] = ACTIONS(1925),
    [sym__wc_get_buffer_char_int] = ACTIONS(1925),
    [sym__wc_get_buffer_string_int] = ACTIONS(1925),
    [sym__wc_concat_string_int] = ACTIONS(1925),
    [sym__wc_file_open_int] = ACTIONS(1925),
    [sym__wc_file_close_int] = ACTIONS(1925),
    [sym__wc_file_read_int] = ACTIONS(1925),
    [sym__wc_file_read_line_int] = ACTIONS(1925),
    [sym__wc_file_write_int] = ACTIONS(1925),
    [sym__wc_file_write_line_int] = ACTIONS(1925),
    [aux_sym_preproc_include_token1] = ACTIONS(1925),
    [aux_sym_preproc_define_token1] = ACTIONS(1927),
    [aux_sym_preproc_warn_token1] = ACTIONS(1927),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1927),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1927),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1927),
    [anon_sym_LBRACK] = ACTIONS(1927),
    [anon_sym_LBRACE] = ACTIONS(1927),
    [anon_sym_LPAREN] = ACTIONS(1925),
    [anon_sym_BANG] = ACTIONS(1927),
    [anon_sym_TILDE] = ACTIONS(1927),
    [anon_sym_DASH] = ACTIONS(1925),
    [anon_sym_PLUS] = ACTIONS(1925),
    [anon_sym_DASH_DASH] = ACTIONS(1927),
    [anon_sym_PLUS_PLUS] = ACTIONS(1927),
    [anon_sym_DQUOTE] = ACTIONS(1927),
    [anon_sym_SQUOTE] = ACTIONS(1927),
    [sym_number_literal] = ACTIONS(1927),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(438)] = {
    [ts_builtin_sym_end] = ACTIONS(1931),
    [sym_identifier] = ACTIONS(1929),
    [sym_program_name_keyword] = ACTIONS(1929),
    [sym_module_name_keyword] = ACTIONS(1929),
    [sym_define_device_keyword] = ACTIONS(1929),
    [sym_define_combine_keyword] = ACTIONS(1929),
    [sym_define_constant_keyword] = ACTIONS(1929),
    [sym_define_type_keyword] = ACTIONS(1929),
    [sym_define_variable_keyword] = ACTIONS(1929),
    [sym_define_system_variable_keyword] = ACTIONS(1929),
    [sym_define_connect_level_keyword] = ACTIONS(1929),
    [sym_define_latching_keyword] = ACTIONS(1929),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1929),
    [sym_define_toggling_keyword] = ACTIONS(1929),
    [sym_define_call_keyword] = ACTIONS(1929),
    [sym_define_function_keyword] = ACTIONS(1929),
    [sym_define_library_function_keyword] = ACTIONS(1929),
    [sym_define_start_keyword] = ACTIONS(1929),
    [sym_define_event_keyword] = ACTIONS(1929),
    [sym_define_program_keyword] = ACTIONS(1929),
    [sym_define_module_keyword] = ACTIONS(1929),
    [sym_char_keyword] = ACTIONS(1929),
    [sym_widechar_keyword] = ACTIONS(1929),
    [sym_integer_keyword] = ACTIONS(1929),
    [sym_sinteger_keyword] = ACTIONS(1929),
    [sym_long_keyword] = ACTIONS(1929),
    [sym_slong_keyword] = ACTIONS(1929),
    [sym_float_keyword] = ACTIONS(1929),
    [sym_double_keyword] = ACTIONS(1929),
    [sym_variant_keyword] = ACTIONS(1929),
    [sym_variantarray_keyword] = ACTIONS(1929),
    [sym_local_var_keyword] = ACTIONS(1929),
    [sym_stack_var_keyword] = ACTIONS(1929),
    [sym_constant_keyword] = ACTIONS(1929),
    [sym_volatile_keyword] = ACTIONS(1929),
    [sym_non_volatile_keyword] = ACTIONS(1929),
    [sym_persistent_keyword] = ACTIONS(1929),
    [sym_dev_keyword] = ACTIONS(1929),
    [sym_devlev_keyword] = ACTIONS(1929),
    [sym_devchan_keyword] = ACTIONS(1929),
    [sym_if_keyword] = ACTIONS(1929),
    [sym_switch_keyword] = ACTIONS(1929),
    [sym_case_keyword] = ACTIONS(1929),
    [sym_default_keyword] = ACTIONS(1929),
    [sym_while_keyword] = ACTIONS(1929),
    [sym_for_keyword] = ACTIONS(1929),
    [sym_break_keyword] = ACTIONS(1929),
    [sym_continue_keyword] = ACTIONS(1929),
    [sym_return_keyword] = ACTIONS(1929),
    [sym_select_keyword] = ACTIONS(1929),
    [sym_active_keyword] = ACTIONS(1929),
    [sym_struct_keyword] = ACTIONS(1929),
    [sym_structure_keyword] = ACTIONS(1929),
    [sym_button_event_keyword] = ACTIONS(1929),
    [sym_channel_event_keyword] = ACTIONS(1929),
    [sym_level_event_keyword] = ACTIONS(1929),
    [sym_data_event_keyword] = ACTIONS(1929),
    [sym_timeline_event_keyword] = ACTIONS(1929),
    [sym_custom_event_keyword] = ACTIONS(1929),
    [sym_push_keyword] = ACTIONS(1929),
    [sym_release_keyword] = ACTIONS(1929),
    [aux_sym_on_keyword_token1] = ACTIONS(1929),
    [aux_sym_off_keyword_token1] = ACTIONS(1929),
    [sym_send_level_keyword] = ACTIONS(1929),
    [sym_send_string_keyword] = ACTIONS(1929),
    [sym_send_command_keyword] = ACTIONS(1929),
    [sym_clear_buffer_keyword] = ACTIONS(1929),
    [sym_create_buffer_keyword] = ACTIONS(1929),
    [sym_create_multi_buffer_keyword] = ACTIONS(1929),
    [sym_call_keyword] = ACTIONS(1929),
    [sym_system_call_keyword] = ACTIONS(1929),
    [sym_devchan_to_keyword] = ACTIONS(1929),
    [sym_devchan_min_to_keyword] = ACTIONS(1929),
    [sym_devchan_total_off_keyword] = ACTIONS(1929),
    [sym_devchan_pulse_keyword] = ACTIONS(1929),
    [sym_wait_keyword] = ACTIONS(1929),
    [sym_cancel_wait_keyword] = ACTIONS(1929),
    [sym_wait_until_keyword] = ACTIONS(1929),
    [sym_cancel_wait_until_keyword] = ACTIONS(1929),
    [sym_cancel_all_wait_keyword] = ACTIONS(1929),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1929),
    [sym_not] = ACTIONS(1929),
    [sym_bnot] = ACTIONS(1929),
    [sym___file__] = ACTIONS(1929),
    [sym___date__] = ACTIONS(1929),
    [sym___ldate__] = ACTIONS(1929),
    [sym___time__] = ACTIONS(1929),
    [sym___line__] = ACTIONS(1929),
    [sym___name__] = ACTIONS(1929),
    [sym___netlinx__] = ACTIONS(1929),
    [sym_netlinx_axi_version] = ACTIONS(1929),
    [sym_url_flg_tcp] = ACTIONS(1929),
    [sym_url_flg_accinfopresent] = ACTIONS(1929),
    [sym_url_flg_temp] = ACTIONS(1929),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1929),
    [sym_url_flg_stat_mask] = ACTIONS(1929),
    [sym_url_flg_stat_lookup] = ACTIONS(1929),
    [sym_url_flg_stat_connecting] = ACTIONS(1929),
    [sym_url_flg_stat_waiting] = ACTIONS(1929),
    [sym_url_flg_stat_connected] = ACTIONS(1929),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1929),
    [sym_first_virtual_device] = ACTIONS(1929),
    [sym_first_local_port] = ACTIONS(1929),
    [sym_dynamic_virtual_device] = ACTIONS(1929),
    [sym_timeline_once] = ACTIONS(1929),
    [sym_timeline_repeat] = ACTIONS(1929),
    [sym_timeline_absolute] = ACTIONS(1929),
    [sym_timeline_relative] = ACTIONS(1929),
    [sym_do_push_timed_infinite] = ACTIONS(1929),
    [sym_source_type_no_address] = ACTIONS(1929),
    [sym_source_type_neuron_id] = ACTIONS(1929),
    [sym_source_type_ip_address] = ACTIONS(1929),
    [sym_source_type_axlink] = ACTIONS(1929),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1929),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1929),
    [sym_source_type_ip_socket_address] = ACTIONS(1929),
    [sym_source_type_rs232] = ACTIONS(1929),
    [sym_source_type_internal] = ACTIONS(1929),
    [sym_source_type_mac_address] = ACTIONS(1929),
    [sym_source_type_ipv4_port] = ACTIONS(1929),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1929),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1929),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1929),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1929),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1929),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1929),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1929),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1929),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1929),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1929),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1929),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1929),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1929),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1929),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1929),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1929),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1929),
    [sym_true] = ACTIONS(1929),
    [sym_false] = ACTIONS(1929),
    [sym_file_read_only] = ACTIONS(1929),
    [sym_file_rw_new] = ACTIONS(1929),
    [sym_file_rw_append] = ACTIONS(1929),
    [sym_ip_tcp] = ACTIONS(1929),
    [sym_ip_udp] = ACTIONS(1929),
    [sym_ip_udp_2way] = ACTIONS(1929),
    [sym_xml_encode_types] = ACTIONS(1929),
    [sym_xml_encode_char_as_list] = ACTIONS(1929),
    [sym_xml_encode_le] = ACTIONS(1929),
    [sym_xml_decode_types] = ACTIONS(1929),
    [sym_xml_decode_no_preserve] = ACTIONS(1929),
    [sym_duet_dev_type_display_device] = ACTIONS(1929),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1929),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1929),
    [sym_duet_dev_type_io_device] = ACTIONS(1929),
    [sym_duet_dev_type_relay_device] = ACTIONS(1929),
    [sym_duet_dev_type_ups] = ACTIONS(1929),
    [sym_duet_dev_type_amplifier] = ACTIONS(1929),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1929),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1929),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1929),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1929),
    [sym_duet_dev_type_camera] = ACTIONS(1929),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1929),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1929),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1929),
    [sym_duet_dev_type_dss] = ACTIONS(1929),
    [sym_duet_dev_type_dvr] = ACTIONS(1929),
    [sym_duet_dev_type_disc_device] = ACTIONS(1929),
    [sym_duet_dev_type_document_camera] = ACTIONS(1929),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1929),
    [sym_duet_dev_type_hvac] = ACTIONS(1929),
    [sym_duet_dev_type_keypad] = ACTIONS(1929),
    [sym_duet_dev_type_light] = ACTIONS(1929),
    [sym_duet_dev_type_monitor] = ACTIONS(1929),
    [sym_duet_dev_type_motor] = ACTIONS(1929),
    [sym_duet_dev_type_multi_window] = ACTIONS(1929),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1929),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1929),
    [sym_duet_dev_type_receiver] = ACTIONS(1929),
    [sym_duet_dev_type_security_system] = ACTIONS(1929),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1929),
    [sym_duet_dev_type_settop_box] = ACTIONS(1929),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1929),
    [sym_duet_dev_type_switcher] = ACTIONS(1929),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1929),
    [sym_duet_dev_type_tv] = ACTIONS(1929),
    [sym_duet_dev_type_utility] = ACTIONS(1929),
    [sym_duet_dev_type_vcr] = ACTIONS(1929),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1929),
    [sym_duet_dev_type_video_processor] = ACTIONS(1929),
    [sym_duet_dev_type_video_projector] = ACTIONS(1929),
    [sym_duet_dev_type_video_wall] = ACTIONS(1929),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1929),
    [sym_duet_dev_type_weather] = ACTIONS(1929),
    [sym_duet_dev_not_polled] = ACTIONS(1929),
    [sym_duet_dev_polled] = ACTIONS(1929),
    [sym_clkmgr_mode_network] = ACTIONS(1929),
    [sym_clkmgr_mode_standalone] = ACTIONS(1929),
    [sym_normal_standby] = ACTIONS(1929),
    [sym_normal_wake] = ACTIONS(1929),
    [sym_ip_multicast_ttl_option] = ACTIONS(1929),
    [sym_ip_ttl_subnet] = ACTIONS(1929),
    [sym_ip_ttl_site] = ACTIONS(1929),
    [sym_ip_ttl_region] = ACTIONS(1929),
    [sym_ip_ttl_continent] = ACTIONS(1929),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1929),
    [sym_ip_nodelay_on] = ACTIONS(1929),
    [sym_ip_nodelay_off] = ACTIONS(1929),
    [sym_amx_error] = ACTIONS(1929),
    [sym_amx_warning] = ACTIONS(1929),
    [sym_amx_info] = ACTIONS(1929),
    [sym_amx_debug] = ACTIONS(1929),
    [sym_netlinx_login_success] = ACTIONS(1929),
    [sym_netlinx_login_fail] = ACTIONS(1929),
    [sym_netlinx_logout] = ACTIONS(1929),
    [sym_null_str] = ACTIONS(1929),
    [sym_smtp_address] = ACTIONS(1929),
    [sym_smtp_port_number] = ACTIONS(1929),
    [sym_smtp_username] = ACTIONS(1929),
    [sym_smtp_password] = ACTIONS(1929),
    [sym_smtp_require_tls] = ACTIONS(1929),
    [sym_smtp_from] = ACTIONS(1929),
    [sym_smtp_tls_true] = ACTIONS(1929),
    [sym_smtp_tls_false] = ACTIONS(1929),
    [sym_listview_on_row_select_event] = ACTIONS(1929),
    [sym_data_structure_unknown] = ACTIONS(1929),
    [sym_data_structure_datafeed] = ACTIONS(1929),
    [sym_data_structure_datarecordset] = ACTIONS(1929),
    [sym_data_structure_datarecord] = ACTIONS(1929),
    [sym_data_structure_datafield] = ACTIONS(1929),
    [sym_data_max_content_field_count] = ACTIONS(1929),
    [sym_data_max_metadata_field_count] = ACTIONS(1929),
    [sym_data_max_id_length] = ACTIONS(1929),
    [sym_data_max_name_length] = ACTIONS(1929),
    [sym_data_max_description_length] = ACTIONS(1929),
    [sym_data_max_type_length] = ACTIONS(1929),
    [sym_data_max_format_length] = ACTIONS(1929),
    [sym_data_max_source_length] = ACTIONS(1929),
    [sym_data_max_value_length] = ACTIONS(1929),
    [sym_data_max_label_length] = ACTIONS(1929),
    [sym_data_max_url_length] = ACTIONS(1929),
    [sym_max_last_login_info_length] = ACTIONS(1929),
    [sym_data_type_unknown] = ACTIONS(1929),
    [sym_data_type_string] = ACTIONS(1929),
    [sym_data_type_datetime] = ACTIONS(1929),
    [sym_data_type_date] = ACTIONS(1929),
    [sym_data_type_time] = ACTIONS(1929),
    [sym_data_type_image] = ACTIONS(1929),
    [sym_data_format_url] = ACTIONS(1929),
    [sym_data_format_phone] = ACTIONS(1929),
    [sym_data_format_email] = ACTIONS(1929),
    [sym_data_format_iso8601] = ACTIONS(1929),
    [sym_auditlog_priv] = ACTIONS(1929),
    [sym_configuration_priv] = ACTIONS(1929),
    [sym_deviceconfig_priv] = ACTIONS(1929),
    [sym_ftp_priv] = ACTIONS(1929),
    [sym_http_priv] = ACTIONS(1929),
    [sym_networkconfig_priv] = ACTIONS(1929),
    [sym_programport_priv] = ACTIONS(1929),
    [sym_remoteui_priv] = ACTIONS(1929),
    [sym_securitycontrol_priv] = ACTIONS(1929),
    [sym_softwaremanagement_priv] = ACTIONS(1929),
    [sym_terminal_priv] = ACTIONS(1929),
    [sym_tpadmin_priv] = ACTIONS(1929),
    [sym_user1_priv] = ACTIONS(1929),
    [sym_user2_priv] = ACTIONS(1929),
    [sym_user3_priv] = ACTIONS(1929),
    [sym_user4_priv] = ACTIONS(1929),
    [sym_usermanagement_priv] = ACTIONS(1929),
    [sym_valid_account] = ACTIONS(1929),
    [sym_err_name_invalid] = ACTIONS(1929),
    [sym_err_password_invalid] = ACTIONS(1929),
    [sym_err_invalid_account] = ACTIONS(1929),
    [sym_err_no_authorization] = ACTIONS(1929),
    [sym_err_invalid_parameter] = ACTIONS(1929),
    [sym_err_user_locked_out] = ACTIONS(1929),
    [sym_err_user_account_expired] = ACTIONS(1929),
    [sym_err_general_error] = ACTIONS(1929),
    [sym_err_authentication_error] = ACTIONS(1929),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1929),
    [sym_tls_validate_certificate] = ACTIONS(1929),
    [sym_tdata] = ACTIONS(1929),
    [sym_tchannel] = ACTIONS(1929),
    [sym_tlevel] = ACTIONS(1929),
    [sym_tbutton] = ACTIONS(1929),
    [sym_ttimeline] = ACTIONS(1929),
    [sym_tcustom] = ACTIONS(1929),
    [sym_url_struct] = ACTIONS(1929),
    [sym_dns_struct] = ACTIONS(1929),
    [sym_ip_address_struct] = ACTIONS(1929),
    [sym_dev_info_struct] = ACTIONS(1929),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1929),
    [sym_data_feed] = ACTIONS(1929),
    [sym_data_field] = ACTIONS(1929),
    [sym_data_record] = ACTIONS(1929),
    [sym_wc_data_feed] = ACTIONS(1929),
    [sym_wc_data_field] = ACTIONS(1929),
    [sym_wc_data_record] = ACTIONS(1929),
    [sym_last_login_info] = ACTIONS(1929),
    [sym____reserved___] = ACTIONS(1929),
    [sym_date] = ACTIONS(1929),
    [sym_day] = ACTIONS(1929),
    [sym_time] = ACTIONS(1929),
    [sym_ldate] = ACTIONS(1929),
    [sym_push_device] = ACTIONS(1929),
    [sym_push_channel] = ACTIONS(1929),
    [sym_push_devchan] = ACTIONS(1929),
    [sym_release_device] = ACTIONS(1929),
    [sym_release_channel] = ACTIONS(1929),
    [sym_release_devchan] = ACTIONS(1929),
    [sym_master_slot] = ACTIONS(1929),
    [sym_get_pulse_time] = ACTIONS(1929),
    [sym_get_timer] = ACTIONS(1929),
    [sym_master_sn] = ACTIONS(1929),
    [sym_system_number] = ACTIONS(1929),
    [sym_button] = ACTIONS(1929),
    [sym_dv_channel] = ACTIONS(1929),
    [sym_data] = ACTIONS(1929),
    [sym_level] = ACTIONS(1929),
    [sym_timeline] = ACTIONS(1929),
    [sym_channel] = ACTIONS(1929),
    [sym_custom] = ACTIONS(1929),
    [sym_atoi] = ACTIONS(1929),
    [sym_atol] = ACTIONS(1929),
    [sym_atof] = ACTIONS(1929),
    [sym_itoa] = ACTIONS(1929),
    [sym_itohex] = ACTIONS(1929),
    [sym_hextoi] = ACTIONS(1929),
    [sym_ftoa] = ACTIONS(1929),
    [sym_find_string] = ACTIONS(1929),
    [sym_left_string] = ACTIONS(1929),
    [sym_right_string] = ACTIONS(1929),
    [sym_mid_string] = ACTIONS(1929),
    [sym_remove_string] = ACTIONS(1929),
    [sym_get_buffer_string] = ACTIONS(1929),
    [sym_get_buffer_char] = ACTIONS(1929),
    [sym_get_multi_buffer_string] = ACTIONS(1929),
    [sym_length_string] = ACTIONS(1929),
    [sym_max_length_string] = ACTIONS(1929),
    [sym_set_length_string] = ACTIONS(1929),
    [sym_compare_string] = ACTIONS(1929),
    [sym_upper_string] = ACTIONS(1929),
    [sym_lower_string] = ACTIONS(1929),
    [sym_get_last] = ACTIONS(1929),
    [sym_date_to_day] = ACTIONS(1929),
    [sym_date_to_month] = ACTIONS(1929),
    [sym_date_to_year] = ACTIONS(1929),
    [sym_time_to_hour] = ACTIONS(1929),
    [sym_time_to_minute] = ACTIONS(1929),
    [sym_time_to_second] = ACTIONS(1929),
    [sym_day_of_week] = ACTIONS(1929),
    [sym_set_pulse_time] = ACTIONS(1929),
    [sym_set_timer] = ACTIONS(1929),
    [sym_file_open] = ACTIONS(1929),
    [sym_file_close] = ACTIONS(1929),
    [sym_file_read] = ACTIONS(1929),
    [sym_file_read_line] = ACTIONS(1929),
    [sym_file_write] = ACTIONS(1929),
    [sym_file_write_line] = ACTIONS(1929),
    [sym_file_copy] = ACTIONS(1929),
    [sym_file_delete] = ACTIONS(1929),
    [sym_file_rename] = ACTIONS(1929),
    [sym_file_dir] = ACTIONS(1929),
    [sym_file_getdir] = ACTIONS(1929),
    [sym_file_setdir] = ACTIONS(1929),
    [sym_file_create_dir] = ACTIONS(1929),
    [sym_file_remove_dir] = ACTIONS(1929),
    [sym_file_seek] = ACTIONS(1929),
    [sym_length_array] = ACTIONS(1929),
    [sym_max_length_array] = ACTIONS(1929),
    [sym_set_length_array] = ACTIONS(1929),
    [sym_ip_client_open] = ACTIONS(1929),
    [sym_ip_client_close] = ACTIONS(1929),
    [sym_ip_server_open] = ACTIONS(1929),
    [sym_ip_server_close] = ACTIONS(1929),
    [sym_ip_bound_client_open] = ACTIONS(1929),
    [sym_ip_mc_server_open] = ACTIONS(1929),
    [sym_ip_set_option] = ACTIONS(1929),
    [sym_ssh_client_open] = ACTIONS(1929),
    [sym_ssh_client_close] = ACTIONS(1929),
    [sym_tls_client_open] = ACTIONS(1929),
    [sym_tls_client_close] = ACTIONS(1929),
    [sym_do_push] = ACTIONS(1929),
    [sym_do_push_timed] = ACTIONS(1929),
    [sym_do_release] = ACTIONS(1929),
    [sym_device_id] = ACTIONS(1929),
    [sym_device_info] = ACTIONS(1929),
    [sym_device_id_string] = ACTIONS(1929),
    [sym_device_standby] = ACTIONS(1929),
    [sym_device_wake] = ACTIONS(1929),
    [sym_timeline_create] = ACTIONS(1929),
    [sym_timeline_kill] = ACTIONS(1929),
    [sym_timeline_reload] = ACTIONS(1929),
    [sym_timeline_pause] = ACTIONS(1929),
    [sym_timeline_restart] = ACTIONS(1929),
    [sym_timeline_set] = ACTIONS(1929),
    [sym_timeline_get] = ACTIONS(1929),
    [sym_timeline_active] = ACTIONS(1929),
    [sym_timeline_dynamic_id] = ACTIONS(1929),
    [sym_variable_to_string] = ACTIONS(1929),
    [sym_string_to_variable] = ACTIONS(1929),
    [sym_length_variable_to_string] = ACTIONS(1929),
    [sym_type_cast] = ACTIONS(1929),
    [sym_variable_to_xml] = ACTIONS(1929),
    [sym_xml_to_variable] = ACTIONS(1929),
    [sym_length_variable_to_xml] = ACTIONS(1929),
    [sym_redirect_string] = ACTIONS(1929),
    [sym_random_number] = ACTIONS(1929),
    [sym_get_unique_id] = ACTIONS(1929),
    [sym_get_system_number] = ACTIONS(1929),
    [sym_set_system_number] = ACTIONS(1929),
    [sym_get_serial_number] = ACTIONS(1929),
    [sym_get_master_build] = ACTIONS(1929),
    [sym_set_validation_code] = ACTIONS(1929),
    [sym_reboot] = ACTIONS(1929),
    [sym_load_duet_module] = ACTIONS(1929),
    [sym_unload_duet_module] = ACTIONS(1929),
    [sym_duet_mem_size_set] = ACTIONS(1929),
    [sym_duet_mem_size_get] = ACTIONS(1929),
    [sym_dynamic_application_device] = ACTIONS(1929),
    [sym_dynamic_polled_port] = ACTIONS(1929),
    [sym_static_port_binding] = ACTIONS(1929),
    [sym_static_ip_binding] = ACTIONS(1929),
    [sym_get_dns_list] = ACTIONS(1929),
    [sym_set_dns_list] = ACTIONS(1929),
    [sym_get_ip_address] = ACTIONS(1929),
    [sym_set_ip_address] = ACTIONS(1929),
    [sym_get_url_list] = ACTIONS(1929),
    [sym_add_url_entry] = ACTIONS(1929),
    [sym_delete_url_entry] = ACTIONS(1929),
    [sym_abs_value] = ACTIONS(1929),
    [sym_max_value] = ACTIONS(1929),
    [sym_min_value] = ACTIONS(1929),
    [sym_exp_value] = ACTIONS(1929),
    [sym_log_value] = ACTIONS(1929),
    [sym_log10_value] = ACTIONS(1929),
    [sym_power_value] = ACTIONS(1929),
    [sym_sqrt_value] = ACTIONS(1929),
    [sym_raw_be] = ACTIONS(1929),
    [sym_raw_le] = ACTIONS(1929),
    [sym_internal_queue_size_set] = ACTIONS(1929),
    [sym_internal_queue_size_get] = ACTIONS(1929),
    [sym_internal_threshold_set] = ACTIONS(1929),
    [sym_internal_threshold_get] = ACTIONS(1929),
    [sym_rebuilt_event] = ACTIONS(1929),
    [sym_do_custom_event] = ACTIONS(1929),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1929),
    [sym_clkmgr_set_clk_source] = ACTIONS(1929),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1929),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1929),
    [sym_clkmgr_get_timezone] = ACTIONS(1929),
    [sym_clkmgr_set_timezone] = ACTIONS(1929),
    [sym_clkmgr_get_resync_period] = ACTIONS(1929),
    [sym_clkmgr_set_resync_period] = ACTIONS(1929),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1929),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1929),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1929),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1929),
    [sym_clkmgr_get_timeservers] = ACTIONS(1929),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1929),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1929),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1929),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1929),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1929),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1929),
    [sym_set_log_level] = ACTIONS(1929),
    [sym_get_log_level] = ACTIONS(1929),
    [sym_amx_log] = ACTIONS(1929),
    [sym_smtp_server_config_set] = ACTIONS(1929),
    [sym_smtp_server_config_get] = ACTIONS(1929),
    [sym_smtp_send] = ACTIONS(1929),
    [sym_validate_netlinx_account] = ACTIONS(1929),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1929),
    [sym_authenticate_certificate] = ACTIONS(1929),
    [sym_audit_netlinx_session_event] = ACTIONS(1929),
    [sym_audit_netlinx_generic_event] = ACTIONS(1929),
    [sym_get_available_flash_disk_space] = ACTIONS(1929),
    [sym_get_max_flash_disk_space] = ACTIONS(1929),
    [sym_data_create_feed] = ACTIONS(1929),
    [sym_data_delete_feed] = ACTIONS(1929),
    [sym_data_publish_feed] = ACTIONS(1929),
    [sym_data_get_published_feed] = ACTIONS(1929),
    [sym_data_add_record] = ACTIONS(1929),
    [sym_data_get_event_record] = ACTIONS(1929),
    [sym__wc_data_create_feed] = ACTIONS(1929),
    [sym__wc_data_add_record] = ACTIONS(1929),
    [sym__wc_data_get_event_record] = ACTIONS(1929),
    [sym_format] = ACTIONS(1929),
    [sym_astro_clock] = ACTIONS(1929),
    [sym_set_outdoor_temperature] = ACTIONS(1929),
    [sym_set_virtual_level_count] = ACTIONS(1929),
    [sym_set_virtual_channel_count] = ACTIONS(1929),
    [sym_set_virtual_port_count] = ACTIONS(1929),
    [sym__wc_encode_int] = ACTIONS(1929),
    [sym__wc_decode_int] = ACTIONS(1929),
    [sym__wc_int] = ACTIONS(1929),
    [sym__wc_to_ch_int] = ACTIONS(1929),
    [sym__ch_to_wc_int] = ACTIONS(1929),
    [sym__wc_find_string_int] = ACTIONS(1929),
    [sym__wc_left_string_int] = ACTIONS(1929),
    [sym__wc_lower_string_int] = ACTIONS(1929),
    [sym__wc_mid_string_int] = ACTIONS(1929),
    [sym__wc_remove_string_int] = ACTIONS(1929),
    [sym__wc_right_string_int] = ACTIONS(1929),
    [sym__wc_upper_string_int] = ACTIONS(1929),
    [sym__wc_compare_string_int] = ACTIONS(1929),
    [sym__wc_get_buffer_char_int] = ACTIONS(1929),
    [sym__wc_get_buffer_string_int] = ACTIONS(1929),
    [sym__wc_concat_string_int] = ACTIONS(1929),
    [sym__wc_file_open_int] = ACTIONS(1929),
    [sym__wc_file_close_int] = ACTIONS(1929),
    [sym__wc_file_read_int] = ACTIONS(1929),
    [sym__wc_file_read_line_int] = ACTIONS(1929),
    [sym__wc_file_write_int] = ACTIONS(1929),
    [sym__wc_file_write_line_int] = ACTIONS(1929),
    [aux_sym_preproc_include_token1] = ACTIONS(1929),
    [aux_sym_preproc_define_token1] = ACTIONS(1931),
    [aux_sym_preproc_warn_token1] = ACTIONS(1931),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1931),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1931),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1931),
    [anon_sym_LBRACK] = ACTIONS(1931),
    [anon_sym_LBRACE] = ACTIONS(1931),
    [anon_sym_LPAREN] = ACTIONS(1929),
    [anon_sym_BANG] = ACTIONS(1931),
    [anon_sym_TILDE] = ACTIONS(1931),
    [anon_sym_DASH] = ACTIONS(1929),
    [anon_sym_PLUS] = ACTIONS(1929),
    [anon_sym_DASH_DASH] = ACTIONS(1931),
    [anon_sym_PLUS_PLUS] = ACTIONS(1931),
    [anon_sym_DQUOTE] = ACTIONS(1931),
    [anon_sym_SQUOTE] = ACTIONS(1931),
    [sym_number_literal] = ACTIONS(1931),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(439)] = {
    [ts_builtin_sym_end] = ACTIONS(2027),
    [sym_identifier] = ACTIONS(2025),
    [sym_program_name_keyword] = ACTIONS(2025),
    [sym_module_name_keyword] = ACTIONS(2025),
    [sym_define_device_keyword] = ACTIONS(2025),
    [sym_define_combine_keyword] = ACTIONS(2025),
    [sym_define_constant_keyword] = ACTIONS(2025),
    [sym_define_type_keyword] = ACTIONS(2025),
    [sym_define_variable_keyword] = ACTIONS(2025),
    [sym_define_system_variable_keyword] = ACTIONS(2025),
    [sym_define_connect_level_keyword] = ACTIONS(2025),
    [sym_define_latching_keyword] = ACTIONS(2025),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2025),
    [sym_define_toggling_keyword] = ACTIONS(2025),
    [sym_define_call_keyword] = ACTIONS(2025),
    [sym_define_function_keyword] = ACTIONS(2025),
    [sym_define_library_function_keyword] = ACTIONS(2025),
    [sym_define_start_keyword] = ACTIONS(2025),
    [sym_define_event_keyword] = ACTIONS(2025),
    [sym_define_program_keyword] = ACTIONS(2025),
    [sym_define_module_keyword] = ACTIONS(2025),
    [sym_char_keyword] = ACTIONS(2025),
    [sym_widechar_keyword] = ACTIONS(2025),
    [sym_integer_keyword] = ACTIONS(2025),
    [sym_sinteger_keyword] = ACTIONS(2025),
    [sym_long_keyword] = ACTIONS(2025),
    [sym_slong_keyword] = ACTIONS(2025),
    [sym_float_keyword] = ACTIONS(2025),
    [sym_double_keyword] = ACTIONS(2025),
    [sym_variant_keyword] = ACTIONS(2025),
    [sym_variantarray_keyword] = ACTIONS(2025),
    [sym_local_var_keyword] = ACTIONS(2025),
    [sym_stack_var_keyword] = ACTIONS(2025),
    [sym_constant_keyword] = ACTIONS(2025),
    [sym_volatile_keyword] = ACTIONS(2025),
    [sym_non_volatile_keyword] = ACTIONS(2025),
    [sym_persistent_keyword] = ACTIONS(2025),
    [sym_dev_keyword] = ACTIONS(2025),
    [sym_devlev_keyword] = ACTIONS(2025),
    [sym_devchan_keyword] = ACTIONS(2025),
    [sym_if_keyword] = ACTIONS(2025),
    [sym_switch_keyword] = ACTIONS(2025),
    [sym_case_keyword] = ACTIONS(2025),
    [sym_default_keyword] = ACTIONS(2025),
    [sym_while_keyword] = ACTIONS(2025),
    [sym_for_keyword] = ACTIONS(2025),
    [sym_break_keyword] = ACTIONS(2025),
    [sym_continue_keyword] = ACTIONS(2025),
    [sym_return_keyword] = ACTIONS(2025),
    [sym_select_keyword] = ACTIONS(2025),
    [sym_active_keyword] = ACTIONS(2025),
    [sym_struct_keyword] = ACTIONS(2025),
    [sym_structure_keyword] = ACTIONS(2025),
    [sym_button_event_keyword] = ACTIONS(2025),
    [sym_channel_event_keyword] = ACTIONS(2025),
    [sym_level_event_keyword] = ACTIONS(2025),
    [sym_data_event_keyword] = ACTIONS(2025),
    [sym_timeline_event_keyword] = ACTIONS(2025),
    [sym_custom_event_keyword] = ACTIONS(2025),
    [sym_push_keyword] = ACTIONS(2025),
    [sym_release_keyword] = ACTIONS(2025),
    [aux_sym_on_keyword_token1] = ACTIONS(2025),
    [aux_sym_off_keyword_token1] = ACTIONS(2025),
    [sym_send_level_keyword] = ACTIONS(2025),
    [sym_send_string_keyword] = ACTIONS(2025),
    [sym_send_command_keyword] = ACTIONS(2025),
    [sym_clear_buffer_keyword] = ACTIONS(2025),
    [sym_create_buffer_keyword] = ACTIONS(2025),
    [sym_create_multi_buffer_keyword] = ACTIONS(2025),
    [sym_call_keyword] = ACTIONS(2025),
    [sym_system_call_keyword] = ACTIONS(2025),
    [sym_devchan_to_keyword] = ACTIONS(2025),
    [sym_devchan_min_to_keyword] = ACTIONS(2025),
    [sym_devchan_total_off_keyword] = ACTIONS(2025),
    [sym_devchan_pulse_keyword] = ACTIONS(2025),
    [sym_wait_keyword] = ACTIONS(2025),
    [sym_cancel_wait_keyword] = ACTIONS(2025),
    [sym_wait_until_keyword] = ACTIONS(2025),
    [sym_cancel_wait_until_keyword] = ACTIONS(2025),
    [sym_cancel_all_wait_keyword] = ACTIONS(2025),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2025),
    [sym_not] = ACTIONS(2025),
    [sym_bnot] = ACTIONS(2025),
    [sym___file__] = ACTIONS(2025),
    [sym___date__] = ACTIONS(2025),
    [sym___ldate__] = ACTIONS(2025),
    [sym___time__] = ACTIONS(2025),
    [sym___line__] = ACTIONS(2025),
    [sym___name__] = ACTIONS(2025),
    [sym___netlinx__] = ACTIONS(2025),
    [sym_netlinx_axi_version] = ACTIONS(2025),
    [sym_url_flg_tcp] = ACTIONS(2025),
    [sym_url_flg_accinfopresent] = ACTIONS(2025),
    [sym_url_flg_temp] = ACTIONS(2025),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2025),
    [sym_url_flg_stat_mask] = ACTIONS(2025),
    [sym_url_flg_stat_lookup] = ACTIONS(2025),
    [sym_url_flg_stat_connecting] = ACTIONS(2025),
    [sym_url_flg_stat_waiting] = ACTIONS(2025),
    [sym_url_flg_stat_connected] = ACTIONS(2025),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2025),
    [sym_first_virtual_device] = ACTIONS(2025),
    [sym_first_local_port] = ACTIONS(2025),
    [sym_dynamic_virtual_device] = ACTIONS(2025),
    [sym_timeline_once] = ACTIONS(2025),
    [sym_timeline_repeat] = ACTIONS(2025),
    [sym_timeline_absolute] = ACTIONS(2025),
    [sym_timeline_relative] = ACTIONS(2025),
    [sym_do_push_timed_infinite] = ACTIONS(2025),
    [sym_source_type_no_address] = ACTIONS(2025),
    [sym_source_type_neuron_id] = ACTIONS(2025),
    [sym_source_type_ip_address] = ACTIONS(2025),
    [sym_source_type_axlink] = ACTIONS(2025),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2025),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2025),
    [sym_source_type_ip_socket_address] = ACTIONS(2025),
    [sym_source_type_rs232] = ACTIONS(2025),
    [sym_source_type_internal] = ACTIONS(2025),
    [sym_source_type_mac_address] = ACTIONS(2025),
    [sym_source_type_ipv4_port] = ACTIONS(2025),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2025),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2025),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2025),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2025),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2025),
    [sym_true] = ACTIONS(2025),
    [sym_false] = ACTIONS(2025),
    [sym_file_read_only] = ACTIONS(2025),
    [sym_file_rw_new] = ACTIONS(2025),
    [sym_file_rw_append] = ACTIONS(2025),
    [sym_ip_tcp] = ACTIONS(2025),
    [sym_ip_udp] = ACTIONS(2025),
    [sym_ip_udp_2way] = ACTIONS(2025),
    [sym_xml_encode_types] = ACTIONS(2025),
    [sym_xml_encode_char_as_list] = ACTIONS(2025),
    [sym_xml_encode_le] = ACTIONS(2025),
    [sym_xml_decode_types] = ACTIONS(2025),
    [sym_xml_decode_no_preserve] = ACTIONS(2025),
    [sym_duet_dev_type_display_device] = ACTIONS(2025),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2025),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2025),
    [sym_duet_dev_type_io_device] = ACTIONS(2025),
    [sym_duet_dev_type_relay_device] = ACTIONS(2025),
    [sym_duet_dev_type_ups] = ACTIONS(2025),
    [sym_duet_dev_type_amplifier] = ACTIONS(2025),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2025),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2025),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2025),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2025),
    [sym_duet_dev_type_camera] = ACTIONS(2025),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2025),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2025),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2025),
    [sym_duet_dev_type_dss] = ACTIONS(2025),
    [sym_duet_dev_type_dvr] = ACTIONS(2025),
    [sym_duet_dev_type_disc_device] = ACTIONS(2025),
    [sym_duet_dev_type_document_camera] = ACTIONS(2025),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2025),
    [sym_duet_dev_type_hvac] = ACTIONS(2025),
    [sym_duet_dev_type_keypad] = ACTIONS(2025),
    [sym_duet_dev_type_light] = ACTIONS(2025),
    [sym_duet_dev_type_monitor] = ACTIONS(2025),
    [sym_duet_dev_type_motor] = ACTIONS(2025),
    [sym_duet_dev_type_multi_window] = ACTIONS(2025),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2025),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2025),
    [sym_duet_dev_type_receiver] = ACTIONS(2025),
    [sym_duet_dev_type_security_system] = ACTIONS(2025),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2025),
    [sym_duet_dev_type_settop_box] = ACTIONS(2025),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2025),
    [sym_duet_dev_type_switcher] = ACTIONS(2025),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2025),
    [sym_duet_dev_type_tv] = ACTIONS(2025),
    [sym_duet_dev_type_utility] = ACTIONS(2025),
    [sym_duet_dev_type_vcr] = ACTIONS(2025),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2025),
    [sym_duet_dev_type_video_processor] = ACTIONS(2025),
    [sym_duet_dev_type_video_projector] = ACTIONS(2025),
    [sym_duet_dev_type_video_wall] = ACTIONS(2025),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2025),
    [sym_duet_dev_type_weather] = ACTIONS(2025),
    [sym_duet_dev_not_polled] = ACTIONS(2025),
    [sym_duet_dev_polled] = ACTIONS(2025),
    [sym_clkmgr_mode_network] = ACTIONS(2025),
    [sym_clkmgr_mode_standalone] = ACTIONS(2025),
    [sym_normal_standby] = ACTIONS(2025),
    [sym_normal_wake] = ACTIONS(2025),
    [sym_ip_multicast_ttl_option] = ACTIONS(2025),
    [sym_ip_ttl_subnet] = ACTIONS(2025),
    [sym_ip_ttl_site] = ACTIONS(2025),
    [sym_ip_ttl_region] = ACTIONS(2025),
    [sym_ip_ttl_continent] = ACTIONS(2025),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2025),
    [sym_ip_nodelay_on] = ACTIONS(2025),
    [sym_ip_nodelay_off] = ACTIONS(2025),
    [sym_amx_error] = ACTIONS(2025),
    [sym_amx_warning] = ACTIONS(2025),
    [sym_amx_info] = ACTIONS(2025),
    [sym_amx_debug] = ACTIONS(2025),
    [sym_netlinx_login_success] = ACTIONS(2025),
    [sym_netlinx_login_fail] = ACTIONS(2025),
    [sym_netlinx_logout] = ACTIONS(2025),
    [sym_null_str] = ACTIONS(2025),
    [sym_smtp_address] = ACTIONS(2025),
    [sym_smtp_port_number] = ACTIONS(2025),
    [sym_smtp_username] = ACTIONS(2025),
    [sym_smtp_password] = ACTIONS(2025),
    [sym_smtp_require_tls] = ACTIONS(2025),
    [sym_smtp_from] = ACTIONS(2025),
    [sym_smtp_tls_true] = ACTIONS(2025),
    [sym_smtp_tls_false] = ACTIONS(2025),
    [sym_listview_on_row_select_event] = ACTIONS(2025),
    [sym_data_structure_unknown] = ACTIONS(2025),
    [sym_data_structure_datafeed] = ACTIONS(2025),
    [sym_data_structure_datarecordset] = ACTIONS(2025),
    [sym_data_structure_datarecord] = ACTIONS(2025),
    [sym_data_structure_datafield] = ACTIONS(2025),
    [sym_data_max_content_field_count] = ACTIONS(2025),
    [sym_data_max_metadata_field_count] = ACTIONS(2025),
    [sym_data_max_id_length] = ACTIONS(2025),
    [sym_data_max_name_length] = ACTIONS(2025),
    [sym_data_max_description_length] = ACTIONS(2025),
    [sym_data_max_type_length] = ACTIONS(2025),
    [sym_data_max_format_length] = ACTIONS(2025),
    [sym_data_max_source_length] = ACTIONS(2025),
    [sym_data_max_value_length] = ACTIONS(2025),
    [sym_data_max_label_length] = ACTIONS(2025),
    [sym_data_max_url_length] = ACTIONS(2025),
    [sym_max_last_login_info_length] = ACTIONS(2025),
    [sym_data_type_unknown] = ACTIONS(2025),
    [sym_data_type_string] = ACTIONS(2025),
    [sym_data_type_datetime] = ACTIONS(2025),
    [sym_data_type_date] = ACTIONS(2025),
    [sym_data_type_time] = ACTIONS(2025),
    [sym_data_type_image] = ACTIONS(2025),
    [sym_data_format_url] = ACTIONS(2025),
    [sym_data_format_phone] = ACTIONS(2025),
    [sym_data_format_email] = ACTIONS(2025),
    [sym_data_format_iso8601] = ACTIONS(2025),
    [sym_auditlog_priv] = ACTIONS(2025),
    [sym_configuration_priv] = ACTIONS(2025),
    [sym_deviceconfig_priv] = ACTIONS(2025),
    [sym_ftp_priv] = ACTIONS(2025),
    [sym_http_priv] = ACTIONS(2025),
    [sym_networkconfig_priv] = ACTIONS(2025),
    [sym_programport_priv] = ACTIONS(2025),
    [sym_remoteui_priv] = ACTIONS(2025),
    [sym_securitycontrol_priv] = ACTIONS(2025),
    [sym_softwaremanagement_priv] = ACTIONS(2025),
    [sym_terminal_priv] = ACTIONS(2025),
    [sym_tpadmin_priv] = ACTIONS(2025),
    [sym_user1_priv] = ACTIONS(2025),
    [sym_user2_priv] = ACTIONS(2025),
    [sym_user3_priv] = ACTIONS(2025),
    [sym_user4_priv] = ACTIONS(2025),
    [sym_usermanagement_priv] = ACTIONS(2025),
    [sym_valid_account] = ACTIONS(2025),
    [sym_err_name_invalid] = ACTIONS(2025),
    [sym_err_password_invalid] = ACTIONS(2025),
    [sym_err_invalid_account] = ACTIONS(2025),
    [sym_err_no_authorization] = ACTIONS(2025),
    [sym_err_invalid_parameter] = ACTIONS(2025),
    [sym_err_user_locked_out] = ACTIONS(2025),
    [sym_err_user_account_expired] = ACTIONS(2025),
    [sym_err_general_error] = ACTIONS(2025),
    [sym_err_authentication_error] = ACTIONS(2025),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2025),
    [sym_tls_validate_certificate] = ACTIONS(2025),
    [sym_tdata] = ACTIONS(2025),
    [sym_tchannel] = ACTIONS(2025),
    [sym_tlevel] = ACTIONS(2025),
    [sym_tbutton] = ACTIONS(2025),
    [sym_ttimeline] = ACTIONS(2025),
    [sym_tcustom] = ACTIONS(2025),
    [sym_url_struct] = ACTIONS(2025),
    [sym_dns_struct] = ACTIONS(2025),
    [sym_ip_address_struct] = ACTIONS(2025),
    [sym_dev_info_struct] = ACTIONS(2025),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2025),
    [sym_data_feed] = ACTIONS(2025),
    [sym_data_field] = ACTIONS(2025),
    [sym_data_record] = ACTIONS(2025),
    [sym_wc_data_feed] = ACTIONS(2025),
    [sym_wc_data_field] = ACTIONS(2025),
    [sym_wc_data_record] = ACTIONS(2025),
    [sym_last_login_info] = ACTIONS(2025),
    [sym____reserved___] = ACTIONS(2025),
    [sym_date] = ACTIONS(2025),
    [sym_day] = ACTIONS(2025),
    [sym_time] = ACTIONS(2025),
    [sym_ldate] = ACTIONS(2025),
    [sym_push_device] = ACTIONS(2025),
    [sym_push_channel] = ACTIONS(2025),
    [sym_push_devchan] = ACTIONS(2025),
    [sym_release_device] = ACTIONS(2025),
    [sym_release_channel] = ACTIONS(2025),
    [sym_release_devchan] = ACTIONS(2025),
    [sym_master_slot] = ACTIONS(2025),
    [sym_get_pulse_time] = ACTIONS(2025),
    [sym_get_timer] = ACTIONS(2025),
    [sym_master_sn] = ACTIONS(2025),
    [sym_system_number] = ACTIONS(2025),
    [sym_button] = ACTIONS(2025),
    [sym_dv_channel] = ACTIONS(2025),
    [sym_data] = ACTIONS(2025),
    [sym_level] = ACTIONS(2025),
    [sym_timeline] = ACTIONS(2025),
    [sym_channel] = ACTIONS(2025),
    [sym_custom] = ACTIONS(2025),
    [sym_atoi] = ACTIONS(2025),
    [sym_atol] = ACTIONS(2025),
    [sym_atof] = ACTIONS(2025),
    [sym_itoa] = ACTIONS(2025),
    [sym_itohex] = ACTIONS(2025),
    [sym_hextoi] = ACTIONS(2025),
    [sym_ftoa] = ACTIONS(2025),
    [sym_find_string] = ACTIONS(2025),
    [sym_left_string] = ACTIONS(2025),
    [sym_right_string] = ACTIONS(2025),
    [sym_mid_string] = ACTIONS(2025),
    [sym_remove_string] = ACTIONS(2025),
    [sym_get_buffer_string] = ACTIONS(2025),
    [sym_get_buffer_char] = ACTIONS(2025),
    [sym_get_multi_buffer_string] = ACTIONS(2025),
    [sym_length_string] = ACTIONS(2025),
    [sym_max_length_string] = ACTIONS(2025),
    [sym_set_length_string] = ACTIONS(2025),
    [sym_compare_string] = ACTIONS(2025),
    [sym_upper_string] = ACTIONS(2025),
    [sym_lower_string] = ACTIONS(2025),
    [sym_get_last] = ACTIONS(2025),
    [sym_date_to_day] = ACTIONS(2025),
    [sym_date_to_month] = ACTIONS(2025),
    [sym_date_to_year] = ACTIONS(2025),
    [sym_time_to_hour] = ACTIONS(2025),
    [sym_time_to_minute] = ACTIONS(2025),
    [sym_time_to_second] = ACTIONS(2025),
    [sym_day_of_week] = ACTIONS(2025),
    [sym_set_pulse_time] = ACTIONS(2025),
    [sym_set_timer] = ACTIONS(2025),
    [sym_file_open] = ACTIONS(2025),
    [sym_file_close] = ACTIONS(2025),
    [sym_file_read] = ACTIONS(2025),
    [sym_file_read_line] = ACTIONS(2025),
    [sym_file_write] = ACTIONS(2025),
    [sym_file_write_line] = ACTIONS(2025),
    [sym_file_copy] = ACTIONS(2025),
    [sym_file_delete] = ACTIONS(2025),
    [sym_file_rename] = ACTIONS(2025),
    [sym_file_dir] = ACTIONS(2025),
    [sym_file_getdir] = ACTIONS(2025),
    [sym_file_setdir] = ACTIONS(2025),
    [sym_file_create_dir] = ACTIONS(2025),
    [sym_file_remove_dir] = ACTIONS(2025),
    [sym_file_seek] = ACTIONS(2025),
    [sym_length_array] = ACTIONS(2025),
    [sym_max_length_array] = ACTIONS(2025),
    [sym_set_length_array] = ACTIONS(2025),
    [sym_ip_client_open] = ACTIONS(2025),
    [sym_ip_client_close] = ACTIONS(2025),
    [sym_ip_server_open] = ACTIONS(2025),
    [sym_ip_server_close] = ACTIONS(2025),
    [sym_ip_bound_client_open] = ACTIONS(2025),
    [sym_ip_mc_server_open] = ACTIONS(2025),
    [sym_ip_set_option] = ACTIONS(2025),
    [sym_ssh_client_open] = ACTIONS(2025),
    [sym_ssh_client_close] = ACTIONS(2025),
    [sym_tls_client_open] = ACTIONS(2025),
    [sym_tls_client_close] = ACTIONS(2025),
    [sym_do_push] = ACTIONS(2025),
    [sym_do_push_timed] = ACTIONS(2025),
    [sym_do_release] = ACTIONS(2025),
    [sym_device_id] = ACTIONS(2025),
    [sym_device_info] = ACTIONS(2025),
    [sym_device_id_string] = ACTIONS(2025),
    [sym_device_standby] = ACTIONS(2025),
    [sym_device_wake] = ACTIONS(2025),
    [sym_timeline_create] = ACTIONS(2025),
    [sym_timeline_kill] = ACTIONS(2025),
    [sym_timeline_reload] = ACTIONS(2025),
    [sym_timeline_pause] = ACTIONS(2025),
    [sym_timeline_restart] = ACTIONS(2025),
    [sym_timeline_set] = ACTIONS(2025),
    [sym_timeline_get] = ACTIONS(2025),
    [sym_timeline_active] = ACTIONS(2025),
    [sym_timeline_dynamic_id] = ACTIONS(2025),
    [sym_variable_to_string] = ACTIONS(2025),
    [sym_string_to_variable] = ACTIONS(2025),
    [sym_length_variable_to_string] = ACTIONS(2025),
    [sym_type_cast] = ACTIONS(2025),
    [sym_variable_to_xml] = ACTIONS(2025),
    [sym_xml_to_variable] = ACTIONS(2025),
    [sym_length_variable_to_xml] = ACTIONS(2025),
    [sym_redirect_string] = ACTIONS(2025),
    [sym_random_number] = ACTIONS(2025),
    [sym_get_unique_id] = ACTIONS(2025),
    [sym_get_system_number] = ACTIONS(2025),
    [sym_set_system_number] = ACTIONS(2025),
    [sym_get_serial_number] = ACTIONS(2025),
    [sym_get_master_build] = ACTIONS(2025),
    [sym_set_validation_code] = ACTIONS(2025),
    [sym_reboot] = ACTIONS(2025),
    [sym_load_duet_module] = ACTIONS(2025),
    [sym_unload_duet_module] = ACTIONS(2025),
    [sym_duet_mem_size_set] = ACTIONS(2025),
    [sym_duet_mem_size_get] = ACTIONS(2025),
    [sym_dynamic_application_device] = ACTIONS(2025),
    [sym_dynamic_polled_port] = ACTIONS(2025),
    [sym_static_port_binding] = ACTIONS(2025),
    [sym_static_ip_binding] = ACTIONS(2025),
    [sym_get_dns_list] = ACTIONS(2025),
    [sym_set_dns_list] = ACTIONS(2025),
    [sym_get_ip_address] = ACTIONS(2025),
    [sym_set_ip_address] = ACTIONS(2025),
    [sym_get_url_list] = ACTIONS(2025),
    [sym_add_url_entry] = ACTIONS(2025),
    [sym_delete_url_entry] = ACTIONS(2025),
    [sym_abs_value] = ACTIONS(2025),
    [sym_max_value] = ACTIONS(2025),
    [sym_min_value] = ACTIONS(2025),
    [sym_exp_value] = ACTIONS(2025),
    [sym_log_value] = ACTIONS(2025),
    [sym_log10_value] = ACTIONS(2025),
    [sym_power_value] = ACTIONS(2025),
    [sym_sqrt_value] = ACTIONS(2025),
    [sym_raw_be] = ACTIONS(2025),
    [sym_raw_le] = ACTIONS(2025),
    [sym_internal_queue_size_set] = ACTIONS(2025),
    [sym_internal_queue_size_get] = ACTIONS(2025),
    [sym_internal_threshold_set] = ACTIONS(2025),
    [sym_internal_threshold_get] = ACTIONS(2025),
    [sym_rebuilt_event] = ACTIONS(2025),
    [sym_do_custom_event] = ACTIONS(2025),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2025),
    [sym_clkmgr_set_clk_source] = ACTIONS(2025),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2025),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2025),
    [sym_clkmgr_get_timezone] = ACTIONS(2025),
    [sym_clkmgr_set_timezone] = ACTIONS(2025),
    [sym_clkmgr_get_resync_period] = ACTIONS(2025),
    [sym_clkmgr_set_resync_period] = ACTIONS(2025),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2025),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2025),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2025),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2025),
    [sym_clkmgr_get_timeservers] = ACTIONS(2025),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2025),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2025),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2025),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2025),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2025),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2025),
    [sym_set_log_level] = ACTIONS(2025),
    [sym_get_log_level] = ACTIONS(2025),
    [sym_amx_log] = ACTIONS(2025),
    [sym_smtp_server_config_set] = ACTIONS(2025),
    [sym_smtp_server_config_get] = ACTIONS(2025),
    [sym_smtp_send] = ACTIONS(2025),
    [sym_validate_netlinx_account] = ACTIONS(2025),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2025),
    [sym_authenticate_certificate] = ACTIONS(2025),
    [sym_audit_netlinx_session_event] = ACTIONS(2025),
    [sym_audit_netlinx_generic_event] = ACTIONS(2025),
    [sym_get_available_flash_disk_space] = ACTIONS(2025),
    [sym_get_max_flash_disk_space] = ACTIONS(2025),
    [sym_data_create_feed] = ACTIONS(2025),
    [sym_data_delete_feed] = ACTIONS(2025),
    [sym_data_publish_feed] = ACTIONS(2025),
    [sym_data_get_published_feed] = ACTIONS(2025),
    [sym_data_add_record] = ACTIONS(2025),
    [sym_data_get_event_record] = ACTIONS(2025),
    [sym__wc_data_create_feed] = ACTIONS(2025),
    [sym__wc_data_add_record] = ACTIONS(2025),
    [sym__wc_data_get_event_record] = ACTIONS(2025),
    [sym_format] = ACTIONS(2025),
    [sym_astro_clock] = ACTIONS(2025),
    [sym_set_outdoor_temperature] = ACTIONS(2025),
    [sym_set_virtual_level_count] = ACTIONS(2025),
    [sym_set_virtual_channel_count] = ACTIONS(2025),
    [sym_set_virtual_port_count] = ACTIONS(2025),
    [sym__wc_encode_int] = ACTIONS(2025),
    [sym__wc_decode_int] = ACTIONS(2025),
    [sym__wc_int] = ACTIONS(2025),
    [sym__wc_to_ch_int] = ACTIONS(2025),
    [sym__ch_to_wc_int] = ACTIONS(2025),
    [sym__wc_find_string_int] = ACTIONS(2025),
    [sym__wc_left_string_int] = ACTIONS(2025),
    [sym__wc_lower_string_int] = ACTIONS(2025),
    [sym__wc_mid_string_int] = ACTIONS(2025),
    [sym__wc_remove_string_int] = ACTIONS(2025),
    [sym__wc_right_string_int] = ACTIONS(2025),
    [sym__wc_upper_string_int] = ACTIONS(2025),
    [sym__wc_compare_string_int] = ACTIONS(2025),
    [sym__wc_get_buffer_char_int] = ACTIONS(2025),
    [sym__wc_get_buffer_string_int] = ACTIONS(2025),
    [sym__wc_concat_string_int] = ACTIONS(2025),
    [sym__wc_file_open_int] = ACTIONS(2025),
    [sym__wc_file_close_int] = ACTIONS(2025),
    [sym__wc_file_read_int] = ACTIONS(2025),
    [sym__wc_file_read_line_int] = ACTIONS(2025),
    [sym__wc_file_write_int] = ACTIONS(2025),
    [sym__wc_file_write_line_int] = ACTIONS(2025),
    [aux_sym_preproc_include_token1] = ACTIONS(2025),
    [aux_sym_preproc_define_token1] = ACTIONS(2027),
    [aux_sym_preproc_warn_token1] = ACTIONS(2027),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2027),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2027),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2027),
    [anon_sym_LBRACK] = ACTIONS(2027),
    [anon_sym_LBRACE] = ACTIONS(2027),
    [anon_sym_LPAREN] = ACTIONS(2025),
    [anon_sym_BANG] = ACTIONS(2027),
    [anon_sym_TILDE] = ACTIONS(2027),
    [anon_sym_DASH] = ACTIONS(2025),
    [anon_sym_PLUS] = ACTIONS(2025),
    [anon_sym_DASH_DASH] = ACTIONS(2027),
    [anon_sym_PLUS_PLUS] = ACTIONS(2027),
    [anon_sym_DQUOTE] = ACTIONS(2027),
    [anon_sym_SQUOTE] = ACTIONS(2027),
    [sym_number_literal] = ACTIONS(2027),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(440)] = {
    [ts_builtin_sym_end] = ACTIONS(2039),
    [sym_identifier] = ACTIONS(2037),
    [sym_program_name_keyword] = ACTIONS(2037),
    [sym_module_name_keyword] = ACTIONS(2037),
    [sym_define_device_keyword] = ACTIONS(2037),
    [sym_define_combine_keyword] = ACTIONS(2037),
    [sym_define_constant_keyword] = ACTIONS(2037),
    [sym_define_type_keyword] = ACTIONS(2037),
    [sym_define_variable_keyword] = ACTIONS(2037),
    [sym_define_system_variable_keyword] = ACTIONS(2037),
    [sym_define_connect_level_keyword] = ACTIONS(2037),
    [sym_define_latching_keyword] = ACTIONS(2037),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2037),
    [sym_define_toggling_keyword] = ACTIONS(2037),
    [sym_define_call_keyword] = ACTIONS(2037),
    [sym_define_function_keyword] = ACTIONS(2037),
    [sym_define_library_function_keyword] = ACTIONS(2037),
    [sym_define_start_keyword] = ACTIONS(2037),
    [sym_define_event_keyword] = ACTIONS(2037),
    [sym_define_program_keyword] = ACTIONS(2037),
    [sym_define_module_keyword] = ACTIONS(2037),
    [sym_char_keyword] = ACTIONS(2037),
    [sym_widechar_keyword] = ACTIONS(2037),
    [sym_integer_keyword] = ACTIONS(2037),
    [sym_sinteger_keyword] = ACTIONS(2037),
    [sym_long_keyword] = ACTIONS(2037),
    [sym_slong_keyword] = ACTIONS(2037),
    [sym_float_keyword] = ACTIONS(2037),
    [sym_double_keyword] = ACTIONS(2037),
    [sym_variant_keyword] = ACTIONS(2037),
    [sym_variantarray_keyword] = ACTIONS(2037),
    [sym_local_var_keyword] = ACTIONS(2037),
    [sym_stack_var_keyword] = ACTIONS(2037),
    [sym_constant_keyword] = ACTIONS(2037),
    [sym_volatile_keyword] = ACTIONS(2037),
    [sym_non_volatile_keyword] = ACTIONS(2037),
    [sym_persistent_keyword] = ACTIONS(2037),
    [sym_dev_keyword] = ACTIONS(2037),
    [sym_devlev_keyword] = ACTIONS(2037),
    [sym_devchan_keyword] = ACTIONS(2037),
    [sym_if_keyword] = ACTIONS(2037),
    [sym_switch_keyword] = ACTIONS(2037),
    [sym_case_keyword] = ACTIONS(2037),
    [sym_default_keyword] = ACTIONS(2037),
    [sym_while_keyword] = ACTIONS(2037),
    [sym_for_keyword] = ACTIONS(2037),
    [sym_break_keyword] = ACTIONS(2037),
    [sym_continue_keyword] = ACTIONS(2037),
    [sym_return_keyword] = ACTIONS(2037),
    [sym_select_keyword] = ACTIONS(2037),
    [sym_active_keyword] = ACTIONS(2037),
    [sym_struct_keyword] = ACTIONS(2037),
    [sym_structure_keyword] = ACTIONS(2037),
    [sym_button_event_keyword] = ACTIONS(2037),
    [sym_channel_event_keyword] = ACTIONS(2037),
    [sym_level_event_keyword] = ACTIONS(2037),
    [sym_data_event_keyword] = ACTIONS(2037),
    [sym_timeline_event_keyword] = ACTIONS(2037),
    [sym_custom_event_keyword] = ACTIONS(2037),
    [sym_push_keyword] = ACTIONS(2037),
    [sym_release_keyword] = ACTIONS(2037),
    [aux_sym_on_keyword_token1] = ACTIONS(2037),
    [aux_sym_off_keyword_token1] = ACTIONS(2037),
    [sym_send_level_keyword] = ACTIONS(2037),
    [sym_send_string_keyword] = ACTIONS(2037),
    [sym_send_command_keyword] = ACTIONS(2037),
    [sym_clear_buffer_keyword] = ACTIONS(2037),
    [sym_create_buffer_keyword] = ACTIONS(2037),
    [sym_create_multi_buffer_keyword] = ACTIONS(2037),
    [sym_call_keyword] = ACTIONS(2037),
    [sym_system_call_keyword] = ACTIONS(2037),
    [sym_devchan_to_keyword] = ACTIONS(2037),
    [sym_devchan_min_to_keyword] = ACTIONS(2037),
    [sym_devchan_total_off_keyword] = ACTIONS(2037),
    [sym_devchan_pulse_keyword] = ACTIONS(2037),
    [sym_wait_keyword] = ACTIONS(2037),
    [sym_cancel_wait_keyword] = ACTIONS(2037),
    [sym_wait_until_keyword] = ACTIONS(2037),
    [sym_cancel_wait_until_keyword] = ACTIONS(2037),
    [sym_cancel_all_wait_keyword] = ACTIONS(2037),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2037),
    [sym_not] = ACTIONS(2037),
    [sym_bnot] = ACTIONS(2037),
    [sym___file__] = ACTIONS(2037),
    [sym___date__] = ACTIONS(2037),
    [sym___ldate__] = ACTIONS(2037),
    [sym___time__] = ACTIONS(2037),
    [sym___line__] = ACTIONS(2037),
    [sym___name__] = ACTIONS(2037),
    [sym___netlinx__] = ACTIONS(2037),
    [sym_netlinx_axi_version] = ACTIONS(2037),
    [sym_url_flg_tcp] = ACTIONS(2037),
    [sym_url_flg_accinfopresent] = ACTIONS(2037),
    [sym_url_flg_temp] = ACTIONS(2037),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2037),
    [sym_url_flg_stat_mask] = ACTIONS(2037),
    [sym_url_flg_stat_lookup] = ACTIONS(2037),
    [sym_url_flg_stat_connecting] = ACTIONS(2037),
    [sym_url_flg_stat_waiting] = ACTIONS(2037),
    [sym_url_flg_stat_connected] = ACTIONS(2037),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2037),
    [sym_first_virtual_device] = ACTIONS(2037),
    [sym_first_local_port] = ACTIONS(2037),
    [sym_dynamic_virtual_device] = ACTIONS(2037),
    [sym_timeline_once] = ACTIONS(2037),
    [sym_timeline_repeat] = ACTIONS(2037),
    [sym_timeline_absolute] = ACTIONS(2037),
    [sym_timeline_relative] = ACTIONS(2037),
    [sym_do_push_timed_infinite] = ACTIONS(2037),
    [sym_source_type_no_address] = ACTIONS(2037),
    [sym_source_type_neuron_id] = ACTIONS(2037),
    [sym_source_type_ip_address] = ACTIONS(2037),
    [sym_source_type_axlink] = ACTIONS(2037),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2037),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2037),
    [sym_source_type_ip_socket_address] = ACTIONS(2037),
    [sym_source_type_rs232] = ACTIONS(2037),
    [sym_source_type_internal] = ACTIONS(2037),
    [sym_source_type_mac_address] = ACTIONS(2037),
    [sym_source_type_ipv4_port] = ACTIONS(2037),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2037),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2037),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2037),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2037),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2037),
    [sym_true] = ACTIONS(2037),
    [sym_false] = ACTIONS(2037),
    [sym_file_read_only] = ACTIONS(2037),
    [sym_file_rw_new] = ACTIONS(2037),
    [sym_file_rw_append] = ACTIONS(2037),
    [sym_ip_tcp] = ACTIONS(2037),
    [sym_ip_udp] = ACTIONS(2037),
    [sym_ip_udp_2way] = ACTIONS(2037),
    [sym_xml_encode_types] = ACTIONS(2037),
    [sym_xml_encode_char_as_list] = ACTIONS(2037),
    [sym_xml_encode_le] = ACTIONS(2037),
    [sym_xml_decode_types] = ACTIONS(2037),
    [sym_xml_decode_no_preserve] = ACTIONS(2037),
    [sym_duet_dev_type_display_device] = ACTIONS(2037),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2037),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2037),
    [sym_duet_dev_type_io_device] = ACTIONS(2037),
    [sym_duet_dev_type_relay_device] = ACTIONS(2037),
    [sym_duet_dev_type_ups] = ACTIONS(2037),
    [sym_duet_dev_type_amplifier] = ACTIONS(2037),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2037),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2037),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2037),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2037),
    [sym_duet_dev_type_camera] = ACTIONS(2037),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2037),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2037),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2037),
    [sym_duet_dev_type_dss] = ACTIONS(2037),
    [sym_duet_dev_type_dvr] = ACTIONS(2037),
    [sym_duet_dev_type_disc_device] = ACTIONS(2037),
    [sym_duet_dev_type_document_camera] = ACTIONS(2037),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2037),
    [sym_duet_dev_type_hvac] = ACTIONS(2037),
    [sym_duet_dev_type_keypad] = ACTIONS(2037),
    [sym_duet_dev_type_light] = ACTIONS(2037),
    [sym_duet_dev_type_monitor] = ACTIONS(2037),
    [sym_duet_dev_type_motor] = ACTIONS(2037),
    [sym_duet_dev_type_multi_window] = ACTIONS(2037),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2037),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2037),
    [sym_duet_dev_type_receiver] = ACTIONS(2037),
    [sym_duet_dev_type_security_system] = ACTIONS(2037),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2037),
    [sym_duet_dev_type_settop_box] = ACTIONS(2037),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2037),
    [sym_duet_dev_type_switcher] = ACTIONS(2037),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2037),
    [sym_duet_dev_type_tv] = ACTIONS(2037),
    [sym_duet_dev_type_utility] = ACTIONS(2037),
    [sym_duet_dev_type_vcr] = ACTIONS(2037),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2037),
    [sym_duet_dev_type_video_processor] = ACTIONS(2037),
    [sym_duet_dev_type_video_projector] = ACTIONS(2037),
    [sym_duet_dev_type_video_wall] = ACTIONS(2037),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2037),
    [sym_duet_dev_type_weather] = ACTIONS(2037),
    [sym_duet_dev_not_polled] = ACTIONS(2037),
    [sym_duet_dev_polled] = ACTIONS(2037),
    [sym_clkmgr_mode_network] = ACTIONS(2037),
    [sym_clkmgr_mode_standalone] = ACTIONS(2037),
    [sym_normal_standby] = ACTIONS(2037),
    [sym_normal_wake] = ACTIONS(2037),
    [sym_ip_multicast_ttl_option] = ACTIONS(2037),
    [sym_ip_ttl_subnet] = ACTIONS(2037),
    [sym_ip_ttl_site] = ACTIONS(2037),
    [sym_ip_ttl_region] = ACTIONS(2037),
    [sym_ip_ttl_continent] = ACTIONS(2037),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2037),
    [sym_ip_nodelay_on] = ACTIONS(2037),
    [sym_ip_nodelay_off] = ACTIONS(2037),
    [sym_amx_error] = ACTIONS(2037),
    [sym_amx_warning] = ACTIONS(2037),
    [sym_amx_info] = ACTIONS(2037),
    [sym_amx_debug] = ACTIONS(2037),
    [sym_netlinx_login_success] = ACTIONS(2037),
    [sym_netlinx_login_fail] = ACTIONS(2037),
    [sym_netlinx_logout] = ACTIONS(2037),
    [sym_null_str] = ACTIONS(2037),
    [sym_smtp_address] = ACTIONS(2037),
    [sym_smtp_port_number] = ACTIONS(2037),
    [sym_smtp_username] = ACTIONS(2037),
    [sym_smtp_password] = ACTIONS(2037),
    [sym_smtp_require_tls] = ACTIONS(2037),
    [sym_smtp_from] = ACTIONS(2037),
    [sym_smtp_tls_true] = ACTIONS(2037),
    [sym_smtp_tls_false] = ACTIONS(2037),
    [sym_listview_on_row_select_event] = ACTIONS(2037),
    [sym_data_structure_unknown] = ACTIONS(2037),
    [sym_data_structure_datafeed] = ACTIONS(2037),
    [sym_data_structure_datarecordset] = ACTIONS(2037),
    [sym_data_structure_datarecord] = ACTIONS(2037),
    [sym_data_structure_datafield] = ACTIONS(2037),
    [sym_data_max_content_field_count] = ACTIONS(2037),
    [sym_data_max_metadata_field_count] = ACTIONS(2037),
    [sym_data_max_id_length] = ACTIONS(2037),
    [sym_data_max_name_length] = ACTIONS(2037),
    [sym_data_max_description_length] = ACTIONS(2037),
    [sym_data_max_type_length] = ACTIONS(2037),
    [sym_data_max_format_length] = ACTIONS(2037),
    [sym_data_max_source_length] = ACTIONS(2037),
    [sym_data_max_value_length] = ACTIONS(2037),
    [sym_data_max_label_length] = ACTIONS(2037),
    [sym_data_max_url_length] = ACTIONS(2037),
    [sym_max_last_login_info_length] = ACTIONS(2037),
    [sym_data_type_unknown] = ACTIONS(2037),
    [sym_data_type_string] = ACTIONS(2037),
    [sym_data_type_datetime] = ACTIONS(2037),
    [sym_data_type_date] = ACTIONS(2037),
    [sym_data_type_time] = ACTIONS(2037),
    [sym_data_type_image] = ACTIONS(2037),
    [sym_data_format_url] = ACTIONS(2037),
    [sym_data_format_phone] = ACTIONS(2037),
    [sym_data_format_email] = ACTIONS(2037),
    [sym_data_format_iso8601] = ACTIONS(2037),
    [sym_auditlog_priv] = ACTIONS(2037),
    [sym_configuration_priv] = ACTIONS(2037),
    [sym_deviceconfig_priv] = ACTIONS(2037),
    [sym_ftp_priv] = ACTIONS(2037),
    [sym_http_priv] = ACTIONS(2037),
    [sym_networkconfig_priv] = ACTIONS(2037),
    [sym_programport_priv] = ACTIONS(2037),
    [sym_remoteui_priv] = ACTIONS(2037),
    [sym_securitycontrol_priv] = ACTIONS(2037),
    [sym_softwaremanagement_priv] = ACTIONS(2037),
    [sym_terminal_priv] = ACTIONS(2037),
    [sym_tpadmin_priv] = ACTIONS(2037),
    [sym_user1_priv] = ACTIONS(2037),
    [sym_user2_priv] = ACTIONS(2037),
    [sym_user3_priv] = ACTIONS(2037),
    [sym_user4_priv] = ACTIONS(2037),
    [sym_usermanagement_priv] = ACTIONS(2037),
    [sym_valid_account] = ACTIONS(2037),
    [sym_err_name_invalid] = ACTIONS(2037),
    [sym_err_password_invalid] = ACTIONS(2037),
    [sym_err_invalid_account] = ACTIONS(2037),
    [sym_err_no_authorization] = ACTIONS(2037),
    [sym_err_invalid_parameter] = ACTIONS(2037),
    [sym_err_user_locked_out] = ACTIONS(2037),
    [sym_err_user_account_expired] = ACTIONS(2037),
    [sym_err_general_error] = ACTIONS(2037),
    [sym_err_authentication_error] = ACTIONS(2037),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2037),
    [sym_tls_validate_certificate] = ACTIONS(2037),
    [sym_tdata] = ACTIONS(2037),
    [sym_tchannel] = ACTIONS(2037),
    [sym_tlevel] = ACTIONS(2037),
    [sym_tbutton] = ACTIONS(2037),
    [sym_ttimeline] = ACTIONS(2037),
    [sym_tcustom] = ACTIONS(2037),
    [sym_url_struct] = ACTIONS(2037),
    [sym_dns_struct] = ACTIONS(2037),
    [sym_ip_address_struct] = ACTIONS(2037),
    [sym_dev_info_struct] = ACTIONS(2037),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2037),
    [sym_data_feed] = ACTIONS(2037),
    [sym_data_field] = ACTIONS(2037),
    [sym_data_record] = ACTIONS(2037),
    [sym_wc_data_feed] = ACTIONS(2037),
    [sym_wc_data_field] = ACTIONS(2037),
    [sym_wc_data_record] = ACTIONS(2037),
    [sym_last_login_info] = ACTIONS(2037),
    [sym____reserved___] = ACTIONS(2037),
    [sym_date] = ACTIONS(2037),
    [sym_day] = ACTIONS(2037),
    [sym_time] = ACTIONS(2037),
    [sym_ldate] = ACTIONS(2037),
    [sym_push_device] = ACTIONS(2037),
    [sym_push_channel] = ACTIONS(2037),
    [sym_push_devchan] = ACTIONS(2037),
    [sym_release_device] = ACTIONS(2037),
    [sym_release_channel] = ACTIONS(2037),
    [sym_release_devchan] = ACTIONS(2037),
    [sym_master_slot] = ACTIONS(2037),
    [sym_get_pulse_time] = ACTIONS(2037),
    [sym_get_timer] = ACTIONS(2037),
    [sym_master_sn] = ACTIONS(2037),
    [sym_system_number] = ACTIONS(2037),
    [sym_button] = ACTIONS(2037),
    [sym_dv_channel] = ACTIONS(2037),
    [sym_data] = ACTIONS(2037),
    [sym_level] = ACTIONS(2037),
    [sym_timeline] = ACTIONS(2037),
    [sym_channel] = ACTIONS(2037),
    [sym_custom] = ACTIONS(2037),
    [sym_atoi] = ACTIONS(2037),
    [sym_atol] = ACTIONS(2037),
    [sym_atof] = ACTIONS(2037),
    [sym_itoa] = ACTIONS(2037),
    [sym_itohex] = ACTIONS(2037),
    [sym_hextoi] = ACTIONS(2037),
    [sym_ftoa] = ACTIONS(2037),
    [sym_find_string] = ACTIONS(2037),
    [sym_left_string] = ACTIONS(2037),
    [sym_right_string] = ACTIONS(2037),
    [sym_mid_string] = ACTIONS(2037),
    [sym_remove_string] = ACTIONS(2037),
    [sym_get_buffer_string] = ACTIONS(2037),
    [sym_get_buffer_char] = ACTIONS(2037),
    [sym_get_multi_buffer_string] = ACTIONS(2037),
    [sym_length_string] = ACTIONS(2037),
    [sym_max_length_string] = ACTIONS(2037),
    [sym_set_length_string] = ACTIONS(2037),
    [sym_compare_string] = ACTIONS(2037),
    [sym_upper_string] = ACTIONS(2037),
    [sym_lower_string] = ACTIONS(2037),
    [sym_get_last] = ACTIONS(2037),
    [sym_date_to_day] = ACTIONS(2037),
    [sym_date_to_month] = ACTIONS(2037),
    [sym_date_to_year] = ACTIONS(2037),
    [sym_time_to_hour] = ACTIONS(2037),
    [sym_time_to_minute] = ACTIONS(2037),
    [sym_time_to_second] = ACTIONS(2037),
    [sym_day_of_week] = ACTIONS(2037),
    [sym_set_pulse_time] = ACTIONS(2037),
    [sym_set_timer] = ACTIONS(2037),
    [sym_file_open] = ACTIONS(2037),
    [sym_file_close] = ACTIONS(2037),
    [sym_file_read] = ACTIONS(2037),
    [sym_file_read_line] = ACTIONS(2037),
    [sym_file_write] = ACTIONS(2037),
    [sym_file_write_line] = ACTIONS(2037),
    [sym_file_copy] = ACTIONS(2037),
    [sym_file_delete] = ACTIONS(2037),
    [sym_file_rename] = ACTIONS(2037),
    [sym_file_dir] = ACTIONS(2037),
    [sym_file_getdir] = ACTIONS(2037),
    [sym_file_setdir] = ACTIONS(2037),
    [sym_file_create_dir] = ACTIONS(2037),
    [sym_file_remove_dir] = ACTIONS(2037),
    [sym_file_seek] = ACTIONS(2037),
    [sym_length_array] = ACTIONS(2037),
    [sym_max_length_array] = ACTIONS(2037),
    [sym_set_length_array] = ACTIONS(2037),
    [sym_ip_client_open] = ACTIONS(2037),
    [sym_ip_client_close] = ACTIONS(2037),
    [sym_ip_server_open] = ACTIONS(2037),
    [sym_ip_server_close] = ACTIONS(2037),
    [sym_ip_bound_client_open] = ACTIONS(2037),
    [sym_ip_mc_server_open] = ACTIONS(2037),
    [sym_ip_set_option] = ACTIONS(2037),
    [sym_ssh_client_open] = ACTIONS(2037),
    [sym_ssh_client_close] = ACTIONS(2037),
    [sym_tls_client_open] = ACTIONS(2037),
    [sym_tls_client_close] = ACTIONS(2037),
    [sym_do_push] = ACTIONS(2037),
    [sym_do_push_timed] = ACTIONS(2037),
    [sym_do_release] = ACTIONS(2037),
    [sym_device_id] = ACTIONS(2037),
    [sym_device_info] = ACTIONS(2037),
    [sym_device_id_string] = ACTIONS(2037),
    [sym_device_standby] = ACTIONS(2037),
    [sym_device_wake] = ACTIONS(2037),
    [sym_timeline_create] = ACTIONS(2037),
    [sym_timeline_kill] = ACTIONS(2037),
    [sym_timeline_reload] = ACTIONS(2037),
    [sym_timeline_pause] = ACTIONS(2037),
    [sym_timeline_restart] = ACTIONS(2037),
    [sym_timeline_set] = ACTIONS(2037),
    [sym_timeline_get] = ACTIONS(2037),
    [sym_timeline_active] = ACTIONS(2037),
    [sym_timeline_dynamic_id] = ACTIONS(2037),
    [sym_variable_to_string] = ACTIONS(2037),
    [sym_string_to_variable] = ACTIONS(2037),
    [sym_length_variable_to_string] = ACTIONS(2037),
    [sym_type_cast] = ACTIONS(2037),
    [sym_variable_to_xml] = ACTIONS(2037),
    [sym_xml_to_variable] = ACTIONS(2037),
    [sym_length_variable_to_xml] = ACTIONS(2037),
    [sym_redirect_string] = ACTIONS(2037),
    [sym_random_number] = ACTIONS(2037),
    [sym_get_unique_id] = ACTIONS(2037),
    [sym_get_system_number] = ACTIONS(2037),
    [sym_set_system_number] = ACTIONS(2037),
    [sym_get_serial_number] = ACTIONS(2037),
    [sym_get_master_build] = ACTIONS(2037),
    [sym_set_validation_code] = ACTIONS(2037),
    [sym_reboot] = ACTIONS(2037),
    [sym_load_duet_module] = ACTIONS(2037),
    [sym_unload_duet_module] = ACTIONS(2037),
    [sym_duet_mem_size_set] = ACTIONS(2037),
    [sym_duet_mem_size_get] = ACTIONS(2037),
    [sym_dynamic_application_device] = ACTIONS(2037),
    [sym_dynamic_polled_port] = ACTIONS(2037),
    [sym_static_port_binding] = ACTIONS(2037),
    [sym_static_ip_binding] = ACTIONS(2037),
    [sym_get_dns_list] = ACTIONS(2037),
    [sym_set_dns_list] = ACTIONS(2037),
    [sym_get_ip_address] = ACTIONS(2037),
    [sym_set_ip_address] = ACTIONS(2037),
    [sym_get_url_list] = ACTIONS(2037),
    [sym_add_url_entry] = ACTIONS(2037),
    [sym_delete_url_entry] = ACTIONS(2037),
    [sym_abs_value] = ACTIONS(2037),
    [sym_max_value] = ACTIONS(2037),
    [sym_min_value] = ACTIONS(2037),
    [sym_exp_value] = ACTIONS(2037),
    [sym_log_value] = ACTIONS(2037),
    [sym_log10_value] = ACTIONS(2037),
    [sym_power_value] = ACTIONS(2037),
    [sym_sqrt_value] = ACTIONS(2037),
    [sym_raw_be] = ACTIONS(2037),
    [sym_raw_le] = ACTIONS(2037),
    [sym_internal_queue_size_set] = ACTIONS(2037),
    [sym_internal_queue_size_get] = ACTIONS(2037),
    [sym_internal_threshold_set] = ACTIONS(2037),
    [sym_internal_threshold_get] = ACTIONS(2037),
    [sym_rebuilt_event] = ACTIONS(2037),
    [sym_do_custom_event] = ACTIONS(2037),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2037),
    [sym_clkmgr_set_clk_source] = ACTIONS(2037),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2037),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2037),
    [sym_clkmgr_get_timezone] = ACTIONS(2037),
    [sym_clkmgr_set_timezone] = ACTIONS(2037),
    [sym_clkmgr_get_resync_period] = ACTIONS(2037),
    [sym_clkmgr_set_resync_period] = ACTIONS(2037),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2037),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2037),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2037),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2037),
    [sym_clkmgr_get_timeservers] = ACTIONS(2037),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2037),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2037),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2037),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2037),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2037),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2037),
    [sym_set_log_level] = ACTIONS(2037),
    [sym_get_log_level] = ACTIONS(2037),
    [sym_amx_log] = ACTIONS(2037),
    [sym_smtp_server_config_set] = ACTIONS(2037),
    [sym_smtp_server_config_get] = ACTIONS(2037),
    [sym_smtp_send] = ACTIONS(2037),
    [sym_validate_netlinx_account] = ACTIONS(2037),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2037),
    [sym_authenticate_certificate] = ACTIONS(2037),
    [sym_audit_netlinx_session_event] = ACTIONS(2037),
    [sym_audit_netlinx_generic_event] = ACTIONS(2037),
    [sym_get_available_flash_disk_space] = ACTIONS(2037),
    [sym_get_max_flash_disk_space] = ACTIONS(2037),
    [sym_data_create_feed] = ACTIONS(2037),
    [sym_data_delete_feed] = ACTIONS(2037),
    [sym_data_publish_feed] = ACTIONS(2037),
    [sym_data_get_published_feed] = ACTIONS(2037),
    [sym_data_add_record] = ACTIONS(2037),
    [sym_data_get_event_record] = ACTIONS(2037),
    [sym__wc_data_create_feed] = ACTIONS(2037),
    [sym__wc_data_add_record] = ACTIONS(2037),
    [sym__wc_data_get_event_record] = ACTIONS(2037),
    [sym_format] = ACTIONS(2037),
    [sym_astro_clock] = ACTIONS(2037),
    [sym_set_outdoor_temperature] = ACTIONS(2037),
    [sym_set_virtual_level_count] = ACTIONS(2037),
    [sym_set_virtual_channel_count] = ACTIONS(2037),
    [sym_set_virtual_port_count] = ACTIONS(2037),
    [sym__wc_encode_int] = ACTIONS(2037),
    [sym__wc_decode_int] = ACTIONS(2037),
    [sym__wc_int] = ACTIONS(2037),
    [sym__wc_to_ch_int] = ACTIONS(2037),
    [sym__ch_to_wc_int] = ACTIONS(2037),
    [sym__wc_find_string_int] = ACTIONS(2037),
    [sym__wc_left_string_int] = ACTIONS(2037),
    [sym__wc_lower_string_int] = ACTIONS(2037),
    [sym__wc_mid_string_int] = ACTIONS(2037),
    [sym__wc_remove_string_int] = ACTIONS(2037),
    [sym__wc_right_string_int] = ACTIONS(2037),
    [sym__wc_upper_string_int] = ACTIONS(2037),
    [sym__wc_compare_string_int] = ACTIONS(2037),
    [sym__wc_get_buffer_char_int] = ACTIONS(2037),
    [sym__wc_get_buffer_string_int] = ACTIONS(2037),
    [sym__wc_concat_string_int] = ACTIONS(2037),
    [sym__wc_file_open_int] = ACTIONS(2037),
    [sym__wc_file_close_int] = ACTIONS(2037),
    [sym__wc_file_read_int] = ACTIONS(2037),
    [sym__wc_file_read_line_int] = ACTIONS(2037),
    [sym__wc_file_write_int] = ACTIONS(2037),
    [sym__wc_file_write_line_int] = ACTIONS(2037),
    [aux_sym_preproc_include_token1] = ACTIONS(2037),
    [aux_sym_preproc_define_token1] = ACTIONS(2039),
    [aux_sym_preproc_warn_token1] = ACTIONS(2039),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2039),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2039),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2039),
    [anon_sym_LBRACK] = ACTIONS(2039),
    [anon_sym_LBRACE] = ACTIONS(2039),
    [anon_sym_LPAREN] = ACTIONS(2037),
    [anon_sym_BANG] = ACTIONS(2039),
    [anon_sym_TILDE] = ACTIONS(2039),
    [anon_sym_DASH] = ACTIONS(2037),
    [anon_sym_PLUS] = ACTIONS(2037),
    [anon_sym_DASH_DASH] = ACTIONS(2039),
    [anon_sym_PLUS_PLUS] = ACTIONS(2039),
    [anon_sym_DQUOTE] = ACTIONS(2039),
    [anon_sym_SQUOTE] = ACTIONS(2039),
    [sym_number_literal] = ACTIONS(2039),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(441)] = {
    [ts_builtin_sym_end] = ACTIONS(2043),
    [sym_identifier] = ACTIONS(2041),
    [sym_program_name_keyword] = ACTIONS(2041),
    [sym_module_name_keyword] = ACTIONS(2041),
    [sym_define_device_keyword] = ACTIONS(2041),
    [sym_define_combine_keyword] = ACTIONS(2041),
    [sym_define_constant_keyword] = ACTIONS(2041),
    [sym_define_type_keyword] = ACTIONS(2041),
    [sym_define_variable_keyword] = ACTIONS(2041),
    [sym_define_system_variable_keyword] = ACTIONS(2041),
    [sym_define_connect_level_keyword] = ACTIONS(2041),
    [sym_define_latching_keyword] = ACTIONS(2041),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2041),
    [sym_define_toggling_keyword] = ACTIONS(2041),
    [sym_define_call_keyword] = ACTIONS(2041),
    [sym_define_function_keyword] = ACTIONS(2041),
    [sym_define_library_function_keyword] = ACTIONS(2041),
    [sym_define_start_keyword] = ACTIONS(2041),
    [sym_define_event_keyword] = ACTIONS(2041),
    [sym_define_program_keyword] = ACTIONS(2041),
    [sym_define_module_keyword] = ACTIONS(2041),
    [sym_char_keyword] = ACTIONS(2041),
    [sym_widechar_keyword] = ACTIONS(2041),
    [sym_integer_keyword] = ACTIONS(2041),
    [sym_sinteger_keyword] = ACTIONS(2041),
    [sym_long_keyword] = ACTIONS(2041),
    [sym_slong_keyword] = ACTIONS(2041),
    [sym_float_keyword] = ACTIONS(2041),
    [sym_double_keyword] = ACTIONS(2041),
    [sym_variant_keyword] = ACTIONS(2041),
    [sym_variantarray_keyword] = ACTIONS(2041),
    [sym_local_var_keyword] = ACTIONS(2041),
    [sym_stack_var_keyword] = ACTIONS(2041),
    [sym_constant_keyword] = ACTIONS(2041),
    [sym_volatile_keyword] = ACTIONS(2041),
    [sym_non_volatile_keyword] = ACTIONS(2041),
    [sym_persistent_keyword] = ACTIONS(2041),
    [sym_dev_keyword] = ACTIONS(2041),
    [sym_devlev_keyword] = ACTIONS(2041),
    [sym_devchan_keyword] = ACTIONS(2041),
    [sym_if_keyword] = ACTIONS(2041),
    [sym_switch_keyword] = ACTIONS(2041),
    [sym_case_keyword] = ACTIONS(2041),
    [sym_default_keyword] = ACTIONS(2041),
    [sym_while_keyword] = ACTIONS(2041),
    [sym_for_keyword] = ACTIONS(2041),
    [sym_break_keyword] = ACTIONS(2041),
    [sym_continue_keyword] = ACTIONS(2041),
    [sym_return_keyword] = ACTIONS(2041),
    [sym_select_keyword] = ACTIONS(2041),
    [sym_active_keyword] = ACTIONS(2041),
    [sym_struct_keyword] = ACTIONS(2041),
    [sym_structure_keyword] = ACTIONS(2041),
    [sym_button_event_keyword] = ACTIONS(2041),
    [sym_channel_event_keyword] = ACTIONS(2041),
    [sym_level_event_keyword] = ACTIONS(2041),
    [sym_data_event_keyword] = ACTIONS(2041),
    [sym_timeline_event_keyword] = ACTIONS(2041),
    [sym_custom_event_keyword] = ACTIONS(2041),
    [sym_push_keyword] = ACTIONS(2041),
    [sym_release_keyword] = ACTIONS(2041),
    [aux_sym_on_keyword_token1] = ACTIONS(2041),
    [aux_sym_off_keyword_token1] = ACTIONS(2041),
    [sym_send_level_keyword] = ACTIONS(2041),
    [sym_send_string_keyword] = ACTIONS(2041),
    [sym_send_command_keyword] = ACTIONS(2041),
    [sym_clear_buffer_keyword] = ACTIONS(2041),
    [sym_create_buffer_keyword] = ACTIONS(2041),
    [sym_create_multi_buffer_keyword] = ACTIONS(2041),
    [sym_call_keyword] = ACTIONS(2041),
    [sym_system_call_keyword] = ACTIONS(2041),
    [sym_devchan_to_keyword] = ACTIONS(2041),
    [sym_devchan_min_to_keyword] = ACTIONS(2041),
    [sym_devchan_total_off_keyword] = ACTIONS(2041),
    [sym_devchan_pulse_keyword] = ACTIONS(2041),
    [sym_wait_keyword] = ACTIONS(2041),
    [sym_cancel_wait_keyword] = ACTIONS(2041),
    [sym_wait_until_keyword] = ACTIONS(2041),
    [sym_cancel_wait_until_keyword] = ACTIONS(2041),
    [sym_cancel_all_wait_keyword] = ACTIONS(2041),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2041),
    [sym_not] = ACTIONS(2041),
    [sym_bnot] = ACTIONS(2041),
    [sym___file__] = ACTIONS(2041),
    [sym___date__] = ACTIONS(2041),
    [sym___ldate__] = ACTIONS(2041),
    [sym___time__] = ACTIONS(2041),
    [sym___line__] = ACTIONS(2041),
    [sym___name__] = ACTIONS(2041),
    [sym___netlinx__] = ACTIONS(2041),
    [sym_netlinx_axi_version] = ACTIONS(2041),
    [sym_url_flg_tcp] = ACTIONS(2041),
    [sym_url_flg_accinfopresent] = ACTIONS(2041),
    [sym_url_flg_temp] = ACTIONS(2041),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2041),
    [sym_url_flg_stat_mask] = ACTIONS(2041),
    [sym_url_flg_stat_lookup] = ACTIONS(2041),
    [sym_url_flg_stat_connecting] = ACTIONS(2041),
    [sym_url_flg_stat_waiting] = ACTIONS(2041),
    [sym_url_flg_stat_connected] = ACTIONS(2041),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2041),
    [sym_first_virtual_device] = ACTIONS(2041),
    [sym_first_local_port] = ACTIONS(2041),
    [sym_dynamic_virtual_device] = ACTIONS(2041),
    [sym_timeline_once] = ACTIONS(2041),
    [sym_timeline_repeat] = ACTIONS(2041),
    [sym_timeline_absolute] = ACTIONS(2041),
    [sym_timeline_relative] = ACTIONS(2041),
    [sym_do_push_timed_infinite] = ACTIONS(2041),
    [sym_source_type_no_address] = ACTIONS(2041),
    [sym_source_type_neuron_id] = ACTIONS(2041),
    [sym_source_type_ip_address] = ACTIONS(2041),
    [sym_source_type_axlink] = ACTIONS(2041),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2041),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2041),
    [sym_source_type_ip_socket_address] = ACTIONS(2041),
    [sym_source_type_rs232] = ACTIONS(2041),
    [sym_source_type_internal] = ACTIONS(2041),
    [sym_source_type_mac_address] = ACTIONS(2041),
    [sym_source_type_ipv4_port] = ACTIONS(2041),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2041),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2041),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2041),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2041),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2041),
    [sym_true] = ACTIONS(2041),
    [sym_false] = ACTIONS(2041),
    [sym_file_read_only] = ACTIONS(2041),
    [sym_file_rw_new] = ACTIONS(2041),
    [sym_file_rw_append] = ACTIONS(2041),
    [sym_ip_tcp] = ACTIONS(2041),
    [sym_ip_udp] = ACTIONS(2041),
    [sym_ip_udp_2way] = ACTIONS(2041),
    [sym_xml_encode_types] = ACTIONS(2041),
    [sym_xml_encode_char_as_list] = ACTIONS(2041),
    [sym_xml_encode_le] = ACTIONS(2041),
    [sym_xml_decode_types] = ACTIONS(2041),
    [sym_xml_decode_no_preserve] = ACTIONS(2041),
    [sym_duet_dev_type_display_device] = ACTIONS(2041),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2041),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2041),
    [sym_duet_dev_type_io_device] = ACTIONS(2041),
    [sym_duet_dev_type_relay_device] = ACTIONS(2041),
    [sym_duet_dev_type_ups] = ACTIONS(2041),
    [sym_duet_dev_type_amplifier] = ACTIONS(2041),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2041),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2041),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2041),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2041),
    [sym_duet_dev_type_camera] = ACTIONS(2041),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2041),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2041),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2041),
    [sym_duet_dev_type_dss] = ACTIONS(2041),
    [sym_duet_dev_type_dvr] = ACTIONS(2041),
    [sym_duet_dev_type_disc_device] = ACTIONS(2041),
    [sym_duet_dev_type_document_camera] = ACTIONS(2041),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2041),
    [sym_duet_dev_type_hvac] = ACTIONS(2041),
    [sym_duet_dev_type_keypad] = ACTIONS(2041),
    [sym_duet_dev_type_light] = ACTIONS(2041),
    [sym_duet_dev_type_monitor] = ACTIONS(2041),
    [sym_duet_dev_type_motor] = ACTIONS(2041),
    [sym_duet_dev_type_multi_window] = ACTIONS(2041),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2041),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2041),
    [sym_duet_dev_type_receiver] = ACTIONS(2041),
    [sym_duet_dev_type_security_system] = ACTIONS(2041),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2041),
    [sym_duet_dev_type_settop_box] = ACTIONS(2041),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2041),
    [sym_duet_dev_type_switcher] = ACTIONS(2041),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2041),
    [sym_duet_dev_type_tv] = ACTIONS(2041),
    [sym_duet_dev_type_utility] = ACTIONS(2041),
    [sym_duet_dev_type_vcr] = ACTIONS(2041),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2041),
    [sym_duet_dev_type_video_processor] = ACTIONS(2041),
    [sym_duet_dev_type_video_projector] = ACTIONS(2041),
    [sym_duet_dev_type_video_wall] = ACTIONS(2041),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2041),
    [sym_duet_dev_type_weather] = ACTIONS(2041),
    [sym_duet_dev_not_polled] = ACTIONS(2041),
    [sym_duet_dev_polled] = ACTIONS(2041),
    [sym_clkmgr_mode_network] = ACTIONS(2041),
    [sym_clkmgr_mode_standalone] = ACTIONS(2041),
    [sym_normal_standby] = ACTIONS(2041),
    [sym_normal_wake] = ACTIONS(2041),
    [sym_ip_multicast_ttl_option] = ACTIONS(2041),
    [sym_ip_ttl_subnet] = ACTIONS(2041),
    [sym_ip_ttl_site] = ACTIONS(2041),
    [sym_ip_ttl_region] = ACTIONS(2041),
    [sym_ip_ttl_continent] = ACTIONS(2041),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2041),
    [sym_ip_nodelay_on] = ACTIONS(2041),
    [sym_ip_nodelay_off] = ACTIONS(2041),
    [sym_amx_error] = ACTIONS(2041),
    [sym_amx_warning] = ACTIONS(2041),
    [sym_amx_info] = ACTIONS(2041),
    [sym_amx_debug] = ACTIONS(2041),
    [sym_netlinx_login_success] = ACTIONS(2041),
    [sym_netlinx_login_fail] = ACTIONS(2041),
    [sym_netlinx_logout] = ACTIONS(2041),
    [sym_null_str] = ACTIONS(2041),
    [sym_smtp_address] = ACTIONS(2041),
    [sym_smtp_port_number] = ACTIONS(2041),
    [sym_smtp_username] = ACTIONS(2041),
    [sym_smtp_password] = ACTIONS(2041),
    [sym_smtp_require_tls] = ACTIONS(2041),
    [sym_smtp_from] = ACTIONS(2041),
    [sym_smtp_tls_true] = ACTIONS(2041),
    [sym_smtp_tls_false] = ACTIONS(2041),
    [sym_listview_on_row_select_event] = ACTIONS(2041),
    [sym_data_structure_unknown] = ACTIONS(2041),
    [sym_data_structure_datafeed] = ACTIONS(2041),
    [sym_data_structure_datarecordset] = ACTIONS(2041),
    [sym_data_structure_datarecord] = ACTIONS(2041),
    [sym_data_structure_datafield] = ACTIONS(2041),
    [sym_data_max_content_field_count] = ACTIONS(2041),
    [sym_data_max_metadata_field_count] = ACTIONS(2041),
    [sym_data_max_id_length] = ACTIONS(2041),
    [sym_data_max_name_length] = ACTIONS(2041),
    [sym_data_max_description_length] = ACTIONS(2041),
    [sym_data_max_type_length] = ACTIONS(2041),
    [sym_data_max_format_length] = ACTIONS(2041),
    [sym_data_max_source_length] = ACTIONS(2041),
    [sym_data_max_value_length] = ACTIONS(2041),
    [sym_data_max_label_length] = ACTIONS(2041),
    [sym_data_max_url_length] = ACTIONS(2041),
    [sym_max_last_login_info_length] = ACTIONS(2041),
    [sym_data_type_unknown] = ACTIONS(2041),
    [sym_data_type_string] = ACTIONS(2041),
    [sym_data_type_datetime] = ACTIONS(2041),
    [sym_data_type_date] = ACTIONS(2041),
    [sym_data_type_time] = ACTIONS(2041),
    [sym_data_type_image] = ACTIONS(2041),
    [sym_data_format_url] = ACTIONS(2041),
    [sym_data_format_phone] = ACTIONS(2041),
    [sym_data_format_email] = ACTIONS(2041),
    [sym_data_format_iso8601] = ACTIONS(2041),
    [sym_auditlog_priv] = ACTIONS(2041),
    [sym_configuration_priv] = ACTIONS(2041),
    [sym_deviceconfig_priv] = ACTIONS(2041),
    [sym_ftp_priv] = ACTIONS(2041),
    [sym_http_priv] = ACTIONS(2041),
    [sym_networkconfig_priv] = ACTIONS(2041),
    [sym_programport_priv] = ACTIONS(2041),
    [sym_remoteui_priv] = ACTIONS(2041),
    [sym_securitycontrol_priv] = ACTIONS(2041),
    [sym_softwaremanagement_priv] = ACTIONS(2041),
    [sym_terminal_priv] = ACTIONS(2041),
    [sym_tpadmin_priv] = ACTIONS(2041),
    [sym_user1_priv] = ACTIONS(2041),
    [sym_user2_priv] = ACTIONS(2041),
    [sym_user3_priv] = ACTIONS(2041),
    [sym_user4_priv] = ACTIONS(2041),
    [sym_usermanagement_priv] = ACTIONS(2041),
    [sym_valid_account] = ACTIONS(2041),
    [sym_err_name_invalid] = ACTIONS(2041),
    [sym_err_password_invalid] = ACTIONS(2041),
    [sym_err_invalid_account] = ACTIONS(2041),
    [sym_err_no_authorization] = ACTIONS(2041),
    [sym_err_invalid_parameter] = ACTIONS(2041),
    [sym_err_user_locked_out] = ACTIONS(2041),
    [sym_err_user_account_expired] = ACTIONS(2041),
    [sym_err_general_error] = ACTIONS(2041),
    [sym_err_authentication_error] = ACTIONS(2041),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2041),
    [sym_tls_validate_certificate] = ACTIONS(2041),
    [sym_tdata] = ACTIONS(2041),
    [sym_tchannel] = ACTIONS(2041),
    [sym_tlevel] = ACTIONS(2041),
    [sym_tbutton] = ACTIONS(2041),
    [sym_ttimeline] = ACTIONS(2041),
    [sym_tcustom] = ACTIONS(2041),
    [sym_url_struct] = ACTIONS(2041),
    [sym_dns_struct] = ACTIONS(2041),
    [sym_ip_address_struct] = ACTIONS(2041),
    [sym_dev_info_struct] = ACTIONS(2041),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2041),
    [sym_data_feed] = ACTIONS(2041),
    [sym_data_field] = ACTIONS(2041),
    [sym_data_record] = ACTIONS(2041),
    [sym_wc_data_feed] = ACTIONS(2041),
    [sym_wc_data_field] = ACTIONS(2041),
    [sym_wc_data_record] = ACTIONS(2041),
    [sym_last_login_info] = ACTIONS(2041),
    [sym____reserved___] = ACTIONS(2041),
    [sym_date] = ACTIONS(2041),
    [sym_day] = ACTIONS(2041),
    [sym_time] = ACTIONS(2041),
    [sym_ldate] = ACTIONS(2041),
    [sym_push_device] = ACTIONS(2041),
    [sym_push_channel] = ACTIONS(2041),
    [sym_push_devchan] = ACTIONS(2041),
    [sym_release_device] = ACTIONS(2041),
    [sym_release_channel] = ACTIONS(2041),
    [sym_release_devchan] = ACTIONS(2041),
    [sym_master_slot] = ACTIONS(2041),
    [sym_get_pulse_time] = ACTIONS(2041),
    [sym_get_timer] = ACTIONS(2041),
    [sym_master_sn] = ACTIONS(2041),
    [sym_system_number] = ACTIONS(2041),
    [sym_button] = ACTIONS(2041),
    [sym_dv_channel] = ACTIONS(2041),
    [sym_data] = ACTIONS(2041),
    [sym_level] = ACTIONS(2041),
    [sym_timeline] = ACTIONS(2041),
    [sym_channel] = ACTIONS(2041),
    [sym_custom] = ACTIONS(2041),
    [sym_atoi] = ACTIONS(2041),
    [sym_atol] = ACTIONS(2041),
    [sym_atof] = ACTIONS(2041),
    [sym_itoa] = ACTIONS(2041),
    [sym_itohex] = ACTIONS(2041),
    [sym_hextoi] = ACTIONS(2041),
    [sym_ftoa] = ACTIONS(2041),
    [sym_find_string] = ACTIONS(2041),
    [sym_left_string] = ACTIONS(2041),
    [sym_right_string] = ACTIONS(2041),
    [sym_mid_string] = ACTIONS(2041),
    [sym_remove_string] = ACTIONS(2041),
    [sym_get_buffer_string] = ACTIONS(2041),
    [sym_get_buffer_char] = ACTIONS(2041),
    [sym_get_multi_buffer_string] = ACTIONS(2041),
    [sym_length_string] = ACTIONS(2041),
    [sym_max_length_string] = ACTIONS(2041),
    [sym_set_length_string] = ACTIONS(2041),
    [sym_compare_string] = ACTIONS(2041),
    [sym_upper_string] = ACTIONS(2041),
    [sym_lower_string] = ACTIONS(2041),
    [sym_get_last] = ACTIONS(2041),
    [sym_date_to_day] = ACTIONS(2041),
    [sym_date_to_month] = ACTIONS(2041),
    [sym_date_to_year] = ACTIONS(2041),
    [sym_time_to_hour] = ACTIONS(2041),
    [sym_time_to_minute] = ACTIONS(2041),
    [sym_time_to_second] = ACTIONS(2041),
    [sym_day_of_week] = ACTIONS(2041),
    [sym_set_pulse_time] = ACTIONS(2041),
    [sym_set_timer] = ACTIONS(2041),
    [sym_file_open] = ACTIONS(2041),
    [sym_file_close] = ACTIONS(2041),
    [sym_file_read] = ACTIONS(2041),
    [sym_file_read_line] = ACTIONS(2041),
    [sym_file_write] = ACTIONS(2041),
    [sym_file_write_line] = ACTIONS(2041),
    [sym_file_copy] = ACTIONS(2041),
    [sym_file_delete] = ACTIONS(2041),
    [sym_file_rename] = ACTIONS(2041),
    [sym_file_dir] = ACTIONS(2041),
    [sym_file_getdir] = ACTIONS(2041),
    [sym_file_setdir] = ACTIONS(2041),
    [sym_file_create_dir] = ACTIONS(2041),
    [sym_file_remove_dir] = ACTIONS(2041),
    [sym_file_seek] = ACTIONS(2041),
    [sym_length_array] = ACTIONS(2041),
    [sym_max_length_array] = ACTIONS(2041),
    [sym_set_length_array] = ACTIONS(2041),
    [sym_ip_client_open] = ACTIONS(2041),
    [sym_ip_client_close] = ACTIONS(2041),
    [sym_ip_server_open] = ACTIONS(2041),
    [sym_ip_server_close] = ACTIONS(2041),
    [sym_ip_bound_client_open] = ACTIONS(2041),
    [sym_ip_mc_server_open] = ACTIONS(2041),
    [sym_ip_set_option] = ACTIONS(2041),
    [sym_ssh_client_open] = ACTIONS(2041),
    [sym_ssh_client_close] = ACTIONS(2041),
    [sym_tls_client_open] = ACTIONS(2041),
    [sym_tls_client_close] = ACTIONS(2041),
    [sym_do_push] = ACTIONS(2041),
    [sym_do_push_timed] = ACTIONS(2041),
    [sym_do_release] = ACTIONS(2041),
    [sym_device_id] = ACTIONS(2041),
    [sym_device_info] = ACTIONS(2041),
    [sym_device_id_string] = ACTIONS(2041),
    [sym_device_standby] = ACTIONS(2041),
    [sym_device_wake] = ACTIONS(2041),
    [sym_timeline_create] = ACTIONS(2041),
    [sym_timeline_kill] = ACTIONS(2041),
    [sym_timeline_reload] = ACTIONS(2041),
    [sym_timeline_pause] = ACTIONS(2041),
    [sym_timeline_restart] = ACTIONS(2041),
    [sym_timeline_set] = ACTIONS(2041),
    [sym_timeline_get] = ACTIONS(2041),
    [sym_timeline_active] = ACTIONS(2041),
    [sym_timeline_dynamic_id] = ACTIONS(2041),
    [sym_variable_to_string] = ACTIONS(2041),
    [sym_string_to_variable] = ACTIONS(2041),
    [sym_length_variable_to_string] = ACTIONS(2041),
    [sym_type_cast] = ACTIONS(2041),
    [sym_variable_to_xml] = ACTIONS(2041),
    [sym_xml_to_variable] = ACTIONS(2041),
    [sym_length_variable_to_xml] = ACTIONS(2041),
    [sym_redirect_string] = ACTIONS(2041),
    [sym_random_number] = ACTIONS(2041),
    [sym_get_unique_id] = ACTIONS(2041),
    [sym_get_system_number] = ACTIONS(2041),
    [sym_set_system_number] = ACTIONS(2041),
    [sym_get_serial_number] = ACTIONS(2041),
    [sym_get_master_build] = ACTIONS(2041),
    [sym_set_validation_code] = ACTIONS(2041),
    [sym_reboot] = ACTIONS(2041),
    [sym_load_duet_module] = ACTIONS(2041),
    [sym_unload_duet_module] = ACTIONS(2041),
    [sym_duet_mem_size_set] = ACTIONS(2041),
    [sym_duet_mem_size_get] = ACTIONS(2041),
    [sym_dynamic_application_device] = ACTIONS(2041),
    [sym_dynamic_polled_port] = ACTIONS(2041),
    [sym_static_port_binding] = ACTIONS(2041),
    [sym_static_ip_binding] = ACTIONS(2041),
    [sym_get_dns_list] = ACTIONS(2041),
    [sym_set_dns_list] = ACTIONS(2041),
    [sym_get_ip_address] = ACTIONS(2041),
    [sym_set_ip_address] = ACTIONS(2041),
    [sym_get_url_list] = ACTIONS(2041),
    [sym_add_url_entry] = ACTIONS(2041),
    [sym_delete_url_entry] = ACTIONS(2041),
    [sym_abs_value] = ACTIONS(2041),
    [sym_max_value] = ACTIONS(2041),
    [sym_min_value] = ACTIONS(2041),
    [sym_exp_value] = ACTIONS(2041),
    [sym_log_value] = ACTIONS(2041),
    [sym_log10_value] = ACTIONS(2041),
    [sym_power_value] = ACTIONS(2041),
    [sym_sqrt_value] = ACTIONS(2041),
    [sym_raw_be] = ACTIONS(2041),
    [sym_raw_le] = ACTIONS(2041),
    [sym_internal_queue_size_set] = ACTIONS(2041),
    [sym_internal_queue_size_get] = ACTIONS(2041),
    [sym_internal_threshold_set] = ACTIONS(2041),
    [sym_internal_threshold_get] = ACTIONS(2041),
    [sym_rebuilt_event] = ACTIONS(2041),
    [sym_do_custom_event] = ACTIONS(2041),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2041),
    [sym_clkmgr_set_clk_source] = ACTIONS(2041),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2041),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2041),
    [sym_clkmgr_get_timezone] = ACTIONS(2041),
    [sym_clkmgr_set_timezone] = ACTIONS(2041),
    [sym_clkmgr_get_resync_period] = ACTIONS(2041),
    [sym_clkmgr_set_resync_period] = ACTIONS(2041),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2041),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2041),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2041),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2041),
    [sym_clkmgr_get_timeservers] = ACTIONS(2041),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2041),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2041),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2041),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2041),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2041),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2041),
    [sym_set_log_level] = ACTIONS(2041),
    [sym_get_log_level] = ACTIONS(2041),
    [sym_amx_log] = ACTIONS(2041),
    [sym_smtp_server_config_set] = ACTIONS(2041),
    [sym_smtp_server_config_get] = ACTIONS(2041),
    [sym_smtp_send] = ACTIONS(2041),
    [sym_validate_netlinx_account] = ACTIONS(2041),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2041),
    [sym_authenticate_certificate] = ACTIONS(2041),
    [sym_audit_netlinx_session_event] = ACTIONS(2041),
    [sym_audit_netlinx_generic_event] = ACTIONS(2041),
    [sym_get_available_flash_disk_space] = ACTIONS(2041),
    [sym_get_max_flash_disk_space] = ACTIONS(2041),
    [sym_data_create_feed] = ACTIONS(2041),
    [sym_data_delete_feed] = ACTIONS(2041),
    [sym_data_publish_feed] = ACTIONS(2041),
    [sym_data_get_published_feed] = ACTIONS(2041),
    [sym_data_add_record] = ACTIONS(2041),
    [sym_data_get_event_record] = ACTIONS(2041),
    [sym__wc_data_create_feed] = ACTIONS(2041),
    [sym__wc_data_add_record] = ACTIONS(2041),
    [sym__wc_data_get_event_record] = ACTIONS(2041),
    [sym_format] = ACTIONS(2041),
    [sym_astro_clock] = ACTIONS(2041),
    [sym_set_outdoor_temperature] = ACTIONS(2041),
    [sym_set_virtual_level_count] = ACTIONS(2041),
    [sym_set_virtual_channel_count] = ACTIONS(2041),
    [sym_set_virtual_port_count] = ACTIONS(2041),
    [sym__wc_encode_int] = ACTIONS(2041),
    [sym__wc_decode_int] = ACTIONS(2041),
    [sym__wc_int] = ACTIONS(2041),
    [sym__wc_to_ch_int] = ACTIONS(2041),
    [sym__ch_to_wc_int] = ACTIONS(2041),
    [sym__wc_find_string_int] = ACTIONS(2041),
    [sym__wc_left_string_int] = ACTIONS(2041),
    [sym__wc_lower_string_int] = ACTIONS(2041),
    [sym__wc_mid_string_int] = ACTIONS(2041),
    [sym__wc_remove_string_int] = ACTIONS(2041),
    [sym__wc_right_string_int] = ACTIONS(2041),
    [sym__wc_upper_string_int] = ACTIONS(2041),
    [sym__wc_compare_string_int] = ACTIONS(2041),
    [sym__wc_get_buffer_char_int] = ACTIONS(2041),
    [sym__wc_get_buffer_string_int] = ACTIONS(2041),
    [sym__wc_concat_string_int] = ACTIONS(2041),
    [sym__wc_file_open_int] = ACTIONS(2041),
    [sym__wc_file_close_int] = ACTIONS(2041),
    [sym__wc_file_read_int] = ACTIONS(2041),
    [sym__wc_file_read_line_int] = ACTIONS(2041),
    [sym__wc_file_write_int] = ACTIONS(2041),
    [sym__wc_file_write_line_int] = ACTIONS(2041),
    [aux_sym_preproc_include_token1] = ACTIONS(2041),
    [aux_sym_preproc_define_token1] = ACTIONS(2043),
    [aux_sym_preproc_warn_token1] = ACTIONS(2043),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2043),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2043),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2043),
    [anon_sym_LBRACK] = ACTIONS(2043),
    [anon_sym_LBRACE] = ACTIONS(2043),
    [anon_sym_LPAREN] = ACTIONS(2041),
    [anon_sym_BANG] = ACTIONS(2043),
    [anon_sym_TILDE] = ACTIONS(2043),
    [anon_sym_DASH] = ACTIONS(2041),
    [anon_sym_PLUS] = ACTIONS(2041),
    [anon_sym_DASH_DASH] = ACTIONS(2043),
    [anon_sym_PLUS_PLUS] = ACTIONS(2043),
    [anon_sym_DQUOTE] = ACTIONS(2043),
    [anon_sym_SQUOTE] = ACTIONS(2043),
    [sym_number_literal] = ACTIONS(2043),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(442)] = {
    [ts_builtin_sym_end] = ACTIONS(1893),
    [sym_identifier] = ACTIONS(1891),
    [sym_program_name_keyword] = ACTIONS(1891),
    [sym_module_name_keyword] = ACTIONS(1891),
    [sym_define_device_keyword] = ACTIONS(1891),
    [sym_define_combine_keyword] = ACTIONS(1891),
    [sym_define_constant_keyword] = ACTIONS(1891),
    [sym_define_type_keyword] = ACTIONS(1891),
    [sym_define_variable_keyword] = ACTIONS(1891),
    [sym_define_system_variable_keyword] = ACTIONS(1891),
    [sym_define_connect_level_keyword] = ACTIONS(1891),
    [sym_define_latching_keyword] = ACTIONS(1891),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1891),
    [sym_define_toggling_keyword] = ACTIONS(1891),
    [sym_define_call_keyword] = ACTIONS(1891),
    [sym_define_function_keyword] = ACTIONS(1891),
    [sym_define_library_function_keyword] = ACTIONS(1891),
    [sym_define_start_keyword] = ACTIONS(1891),
    [sym_define_event_keyword] = ACTIONS(1891),
    [sym_define_program_keyword] = ACTIONS(1891),
    [sym_define_module_keyword] = ACTIONS(1891),
    [sym_char_keyword] = ACTIONS(1891),
    [sym_widechar_keyword] = ACTIONS(1891),
    [sym_integer_keyword] = ACTIONS(1891),
    [sym_sinteger_keyword] = ACTIONS(1891),
    [sym_long_keyword] = ACTIONS(1891),
    [sym_slong_keyword] = ACTIONS(1891),
    [sym_float_keyword] = ACTIONS(1891),
    [sym_double_keyword] = ACTIONS(1891),
    [sym_variant_keyword] = ACTIONS(1891),
    [sym_variantarray_keyword] = ACTIONS(1891),
    [sym_local_var_keyword] = ACTIONS(1891),
    [sym_stack_var_keyword] = ACTIONS(1891),
    [sym_constant_keyword] = ACTIONS(1891),
    [sym_volatile_keyword] = ACTIONS(1891),
    [sym_non_volatile_keyword] = ACTIONS(1891),
    [sym_persistent_keyword] = ACTIONS(1891),
    [sym_dev_keyword] = ACTIONS(1891),
    [sym_devlev_keyword] = ACTIONS(1891),
    [sym_devchan_keyword] = ACTIONS(1891),
    [sym_if_keyword] = ACTIONS(1891),
    [sym_switch_keyword] = ACTIONS(1891),
    [sym_case_keyword] = ACTIONS(1891),
    [sym_default_keyword] = ACTIONS(1891),
    [sym_while_keyword] = ACTIONS(1891),
    [sym_for_keyword] = ACTIONS(1891),
    [sym_break_keyword] = ACTIONS(1891),
    [sym_continue_keyword] = ACTIONS(1891),
    [sym_return_keyword] = ACTIONS(1891),
    [sym_select_keyword] = ACTIONS(1891),
    [sym_active_keyword] = ACTIONS(1891),
    [sym_struct_keyword] = ACTIONS(1891),
    [sym_structure_keyword] = ACTIONS(1891),
    [sym_button_event_keyword] = ACTIONS(1891),
    [sym_channel_event_keyword] = ACTIONS(1891),
    [sym_level_event_keyword] = ACTIONS(1891),
    [sym_data_event_keyword] = ACTIONS(1891),
    [sym_timeline_event_keyword] = ACTIONS(1891),
    [sym_custom_event_keyword] = ACTIONS(1891),
    [sym_push_keyword] = ACTIONS(1891),
    [sym_release_keyword] = ACTIONS(1891),
    [aux_sym_on_keyword_token1] = ACTIONS(1891),
    [aux_sym_off_keyword_token1] = ACTIONS(1891),
    [sym_send_level_keyword] = ACTIONS(1891),
    [sym_send_string_keyword] = ACTIONS(1891),
    [sym_send_command_keyword] = ACTIONS(1891),
    [sym_clear_buffer_keyword] = ACTIONS(1891),
    [sym_create_buffer_keyword] = ACTIONS(1891),
    [sym_create_multi_buffer_keyword] = ACTIONS(1891),
    [sym_call_keyword] = ACTIONS(1891),
    [sym_system_call_keyword] = ACTIONS(1891),
    [sym_devchan_to_keyword] = ACTIONS(1891),
    [sym_devchan_min_to_keyword] = ACTIONS(1891),
    [sym_devchan_total_off_keyword] = ACTIONS(1891),
    [sym_devchan_pulse_keyword] = ACTIONS(1891),
    [sym_wait_keyword] = ACTIONS(1891),
    [sym_cancel_wait_keyword] = ACTIONS(1891),
    [sym_wait_until_keyword] = ACTIONS(1891),
    [sym_cancel_wait_until_keyword] = ACTIONS(1891),
    [sym_cancel_all_wait_keyword] = ACTIONS(1891),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1891),
    [sym_not] = ACTIONS(1891),
    [sym_bnot] = ACTIONS(1891),
    [sym___file__] = ACTIONS(1891),
    [sym___date__] = ACTIONS(1891),
    [sym___ldate__] = ACTIONS(1891),
    [sym___time__] = ACTIONS(1891),
    [sym___line__] = ACTIONS(1891),
    [sym___name__] = ACTIONS(1891),
    [sym___netlinx__] = ACTIONS(1891),
    [sym_netlinx_axi_version] = ACTIONS(1891),
    [sym_url_flg_tcp] = ACTIONS(1891),
    [sym_url_flg_accinfopresent] = ACTIONS(1891),
    [sym_url_flg_temp] = ACTIONS(1891),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1891),
    [sym_url_flg_stat_mask] = ACTIONS(1891),
    [sym_url_flg_stat_lookup] = ACTIONS(1891),
    [sym_url_flg_stat_connecting] = ACTIONS(1891),
    [sym_url_flg_stat_waiting] = ACTIONS(1891),
    [sym_url_flg_stat_connected] = ACTIONS(1891),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1891),
    [sym_first_virtual_device] = ACTIONS(1891),
    [sym_first_local_port] = ACTIONS(1891),
    [sym_dynamic_virtual_device] = ACTIONS(1891),
    [sym_timeline_once] = ACTIONS(1891),
    [sym_timeline_repeat] = ACTIONS(1891),
    [sym_timeline_absolute] = ACTIONS(1891),
    [sym_timeline_relative] = ACTIONS(1891),
    [sym_do_push_timed_infinite] = ACTIONS(1891),
    [sym_source_type_no_address] = ACTIONS(1891),
    [sym_source_type_neuron_id] = ACTIONS(1891),
    [sym_source_type_ip_address] = ACTIONS(1891),
    [sym_source_type_axlink] = ACTIONS(1891),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1891),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1891),
    [sym_source_type_ip_socket_address] = ACTIONS(1891),
    [sym_source_type_rs232] = ACTIONS(1891),
    [sym_source_type_internal] = ACTIONS(1891),
    [sym_source_type_mac_address] = ACTIONS(1891),
    [sym_source_type_ipv4_port] = ACTIONS(1891),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1891),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1891),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1891),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1891),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1891),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1891),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1891),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1891),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1891),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1891),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1891),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1891),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1891),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1891),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1891),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1891),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1891),
    [sym_true] = ACTIONS(1891),
    [sym_false] = ACTIONS(1891),
    [sym_file_read_only] = ACTIONS(1891),
    [sym_file_rw_new] = ACTIONS(1891),
    [sym_file_rw_append] = ACTIONS(1891),
    [sym_ip_tcp] = ACTIONS(1891),
    [sym_ip_udp] = ACTIONS(1891),
    [sym_ip_udp_2way] = ACTIONS(1891),
    [sym_xml_encode_types] = ACTIONS(1891),
    [sym_xml_encode_char_as_list] = ACTIONS(1891),
    [sym_xml_encode_le] = ACTIONS(1891),
    [sym_xml_decode_types] = ACTIONS(1891),
    [sym_xml_decode_no_preserve] = ACTIONS(1891),
    [sym_duet_dev_type_display_device] = ACTIONS(1891),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1891),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1891),
    [sym_duet_dev_type_io_device] = ACTIONS(1891),
    [sym_duet_dev_type_relay_device] = ACTIONS(1891),
    [sym_duet_dev_type_ups] = ACTIONS(1891),
    [sym_duet_dev_type_amplifier] = ACTIONS(1891),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1891),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1891),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1891),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1891),
    [sym_duet_dev_type_camera] = ACTIONS(1891),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1891),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1891),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1891),
    [sym_duet_dev_type_dss] = ACTIONS(1891),
    [sym_duet_dev_type_dvr] = ACTIONS(1891),
    [sym_duet_dev_type_disc_device] = ACTIONS(1891),
    [sym_duet_dev_type_document_camera] = ACTIONS(1891),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1891),
    [sym_duet_dev_type_hvac] = ACTIONS(1891),
    [sym_duet_dev_type_keypad] = ACTIONS(1891),
    [sym_duet_dev_type_light] = ACTIONS(1891),
    [sym_duet_dev_type_monitor] = ACTIONS(1891),
    [sym_duet_dev_type_motor] = ACTIONS(1891),
    [sym_duet_dev_type_multi_window] = ACTIONS(1891),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1891),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1891),
    [sym_duet_dev_type_receiver] = ACTIONS(1891),
    [sym_duet_dev_type_security_system] = ACTIONS(1891),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1891),
    [sym_duet_dev_type_settop_box] = ACTIONS(1891),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1891),
    [sym_duet_dev_type_switcher] = ACTIONS(1891),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1891),
    [sym_duet_dev_type_tv] = ACTIONS(1891),
    [sym_duet_dev_type_utility] = ACTIONS(1891),
    [sym_duet_dev_type_vcr] = ACTIONS(1891),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1891),
    [sym_duet_dev_type_video_processor] = ACTIONS(1891),
    [sym_duet_dev_type_video_projector] = ACTIONS(1891),
    [sym_duet_dev_type_video_wall] = ACTIONS(1891),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1891),
    [sym_duet_dev_type_weather] = ACTIONS(1891),
    [sym_duet_dev_not_polled] = ACTIONS(1891),
    [sym_duet_dev_polled] = ACTIONS(1891),
    [sym_clkmgr_mode_network] = ACTIONS(1891),
    [sym_clkmgr_mode_standalone] = ACTIONS(1891),
    [sym_normal_standby] = ACTIONS(1891),
    [sym_normal_wake] = ACTIONS(1891),
    [sym_ip_multicast_ttl_option] = ACTIONS(1891),
    [sym_ip_ttl_subnet] = ACTIONS(1891),
    [sym_ip_ttl_site] = ACTIONS(1891),
    [sym_ip_ttl_region] = ACTIONS(1891),
    [sym_ip_ttl_continent] = ACTIONS(1891),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1891),
    [sym_ip_nodelay_on] = ACTIONS(1891),
    [sym_ip_nodelay_off] = ACTIONS(1891),
    [sym_amx_error] = ACTIONS(1891),
    [sym_amx_warning] = ACTIONS(1891),
    [sym_amx_info] = ACTIONS(1891),
    [sym_amx_debug] = ACTIONS(1891),
    [sym_netlinx_login_success] = ACTIONS(1891),
    [sym_netlinx_login_fail] = ACTIONS(1891),
    [sym_netlinx_logout] = ACTIONS(1891),
    [sym_null_str] = ACTIONS(1891),
    [sym_smtp_address] = ACTIONS(1891),
    [sym_smtp_port_number] = ACTIONS(1891),
    [sym_smtp_username] = ACTIONS(1891),
    [sym_smtp_password] = ACTIONS(1891),
    [sym_smtp_require_tls] = ACTIONS(1891),
    [sym_smtp_from] = ACTIONS(1891),
    [sym_smtp_tls_true] = ACTIONS(1891),
    [sym_smtp_tls_false] = ACTIONS(1891),
    [sym_listview_on_row_select_event] = ACTIONS(1891),
    [sym_data_structure_unknown] = ACTIONS(1891),
    [sym_data_structure_datafeed] = ACTIONS(1891),
    [sym_data_structure_datarecordset] = ACTIONS(1891),
    [sym_data_structure_datarecord] = ACTIONS(1891),
    [sym_data_structure_datafield] = ACTIONS(1891),
    [sym_data_max_content_field_count] = ACTIONS(1891),
    [sym_data_max_metadata_field_count] = ACTIONS(1891),
    [sym_data_max_id_length] = ACTIONS(1891),
    [sym_data_max_name_length] = ACTIONS(1891),
    [sym_data_max_description_length] = ACTIONS(1891),
    [sym_data_max_type_length] = ACTIONS(1891),
    [sym_data_max_format_length] = ACTIONS(1891),
    [sym_data_max_source_length] = ACTIONS(1891),
    [sym_data_max_value_length] = ACTIONS(1891),
    [sym_data_max_label_length] = ACTIONS(1891),
    [sym_data_max_url_length] = ACTIONS(1891),
    [sym_max_last_login_info_length] = ACTIONS(1891),
    [sym_data_type_unknown] = ACTIONS(1891),
    [sym_data_type_string] = ACTIONS(1891),
    [sym_data_type_datetime] = ACTIONS(1891),
    [sym_data_type_date] = ACTIONS(1891),
    [sym_data_type_time] = ACTIONS(1891),
    [sym_data_type_image] = ACTIONS(1891),
    [sym_data_format_url] = ACTIONS(1891),
    [sym_data_format_phone] = ACTIONS(1891),
    [sym_data_format_email] = ACTIONS(1891),
    [sym_data_format_iso8601] = ACTIONS(1891),
    [sym_auditlog_priv] = ACTIONS(1891),
    [sym_configuration_priv] = ACTIONS(1891),
    [sym_deviceconfig_priv] = ACTIONS(1891),
    [sym_ftp_priv] = ACTIONS(1891),
    [sym_http_priv] = ACTIONS(1891),
    [sym_networkconfig_priv] = ACTIONS(1891),
    [sym_programport_priv] = ACTIONS(1891),
    [sym_remoteui_priv] = ACTIONS(1891),
    [sym_securitycontrol_priv] = ACTIONS(1891),
    [sym_softwaremanagement_priv] = ACTIONS(1891),
    [sym_terminal_priv] = ACTIONS(1891),
    [sym_tpadmin_priv] = ACTIONS(1891),
    [sym_user1_priv] = ACTIONS(1891),
    [sym_user2_priv] = ACTIONS(1891),
    [sym_user3_priv] = ACTIONS(1891),
    [sym_user4_priv] = ACTIONS(1891),
    [sym_usermanagement_priv] = ACTIONS(1891),
    [sym_valid_account] = ACTIONS(1891),
    [sym_err_name_invalid] = ACTIONS(1891),
    [sym_err_password_invalid] = ACTIONS(1891),
    [sym_err_invalid_account] = ACTIONS(1891),
    [sym_err_no_authorization] = ACTIONS(1891),
    [sym_err_invalid_parameter] = ACTIONS(1891),
    [sym_err_user_locked_out] = ACTIONS(1891),
    [sym_err_user_account_expired] = ACTIONS(1891),
    [sym_err_general_error] = ACTIONS(1891),
    [sym_err_authentication_error] = ACTIONS(1891),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1891),
    [sym_tls_validate_certificate] = ACTIONS(1891),
    [sym_tdata] = ACTIONS(1891),
    [sym_tchannel] = ACTIONS(1891),
    [sym_tlevel] = ACTIONS(1891),
    [sym_tbutton] = ACTIONS(1891),
    [sym_ttimeline] = ACTIONS(1891),
    [sym_tcustom] = ACTIONS(1891),
    [sym_url_struct] = ACTIONS(1891),
    [sym_dns_struct] = ACTIONS(1891),
    [sym_ip_address_struct] = ACTIONS(1891),
    [sym_dev_info_struct] = ACTIONS(1891),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1891),
    [sym_data_feed] = ACTIONS(1891),
    [sym_data_field] = ACTIONS(1891),
    [sym_data_record] = ACTIONS(1891),
    [sym_wc_data_feed] = ACTIONS(1891),
    [sym_wc_data_field] = ACTIONS(1891),
    [sym_wc_data_record] = ACTIONS(1891),
    [sym_last_login_info] = ACTIONS(1891),
    [sym____reserved___] = ACTIONS(1891),
    [sym_date] = ACTIONS(1891),
    [sym_day] = ACTIONS(1891),
    [sym_time] = ACTIONS(1891),
    [sym_ldate] = ACTIONS(1891),
    [sym_push_device] = ACTIONS(1891),
    [sym_push_channel] = ACTIONS(1891),
    [sym_push_devchan] = ACTIONS(1891),
    [sym_release_device] = ACTIONS(1891),
    [sym_release_channel] = ACTIONS(1891),
    [sym_release_devchan] = ACTIONS(1891),
    [sym_master_slot] = ACTIONS(1891),
    [sym_get_pulse_time] = ACTIONS(1891),
    [sym_get_timer] = ACTIONS(1891),
    [sym_master_sn] = ACTIONS(1891),
    [sym_system_number] = ACTIONS(1891),
    [sym_button] = ACTIONS(1891),
    [sym_dv_channel] = ACTIONS(1891),
    [sym_data] = ACTIONS(1891),
    [sym_level] = ACTIONS(1891),
    [sym_timeline] = ACTIONS(1891),
    [sym_channel] = ACTIONS(1891),
    [sym_custom] = ACTIONS(1891),
    [sym_atoi] = ACTIONS(1891),
    [sym_atol] = ACTIONS(1891),
    [sym_atof] = ACTIONS(1891),
    [sym_itoa] = ACTIONS(1891),
    [sym_itohex] = ACTIONS(1891),
    [sym_hextoi] = ACTIONS(1891),
    [sym_ftoa] = ACTIONS(1891),
    [sym_find_string] = ACTIONS(1891),
    [sym_left_string] = ACTIONS(1891),
    [sym_right_string] = ACTIONS(1891),
    [sym_mid_string] = ACTIONS(1891),
    [sym_remove_string] = ACTIONS(1891),
    [sym_get_buffer_string] = ACTIONS(1891),
    [sym_get_buffer_char] = ACTIONS(1891),
    [sym_get_multi_buffer_string] = ACTIONS(1891),
    [sym_length_string] = ACTIONS(1891),
    [sym_max_length_string] = ACTIONS(1891),
    [sym_set_length_string] = ACTIONS(1891),
    [sym_compare_string] = ACTIONS(1891),
    [sym_upper_string] = ACTIONS(1891),
    [sym_lower_string] = ACTIONS(1891),
    [sym_get_last] = ACTIONS(1891),
    [sym_date_to_day] = ACTIONS(1891),
    [sym_date_to_month] = ACTIONS(1891),
    [sym_date_to_year] = ACTIONS(1891),
    [sym_time_to_hour] = ACTIONS(1891),
    [sym_time_to_minute] = ACTIONS(1891),
    [sym_time_to_second] = ACTIONS(1891),
    [sym_day_of_week] = ACTIONS(1891),
    [sym_set_pulse_time] = ACTIONS(1891),
    [sym_set_timer] = ACTIONS(1891),
    [sym_file_open] = ACTIONS(1891),
    [sym_file_close] = ACTIONS(1891),
    [sym_file_read] = ACTIONS(1891),
    [sym_file_read_line] = ACTIONS(1891),
    [sym_file_write] = ACTIONS(1891),
    [sym_file_write_line] = ACTIONS(1891),
    [sym_file_copy] = ACTIONS(1891),
    [sym_file_delete] = ACTIONS(1891),
    [sym_file_rename] = ACTIONS(1891),
    [sym_file_dir] = ACTIONS(1891),
    [sym_file_getdir] = ACTIONS(1891),
    [sym_file_setdir] = ACTIONS(1891),
    [sym_file_create_dir] = ACTIONS(1891),
    [sym_file_remove_dir] = ACTIONS(1891),
    [sym_file_seek] = ACTIONS(1891),
    [sym_length_array] = ACTIONS(1891),
    [sym_max_length_array] = ACTIONS(1891),
    [sym_set_length_array] = ACTIONS(1891),
    [sym_ip_client_open] = ACTIONS(1891),
    [sym_ip_client_close] = ACTIONS(1891),
    [sym_ip_server_open] = ACTIONS(1891),
    [sym_ip_server_close] = ACTIONS(1891),
    [sym_ip_bound_client_open] = ACTIONS(1891),
    [sym_ip_mc_server_open] = ACTIONS(1891),
    [sym_ip_set_option] = ACTIONS(1891),
    [sym_ssh_client_open] = ACTIONS(1891),
    [sym_ssh_client_close] = ACTIONS(1891),
    [sym_tls_client_open] = ACTIONS(1891),
    [sym_tls_client_close] = ACTIONS(1891),
    [sym_do_push] = ACTIONS(1891),
    [sym_do_push_timed] = ACTIONS(1891),
    [sym_do_release] = ACTIONS(1891),
    [sym_device_id] = ACTIONS(1891),
    [sym_device_info] = ACTIONS(1891),
    [sym_device_id_string] = ACTIONS(1891),
    [sym_device_standby] = ACTIONS(1891),
    [sym_device_wake] = ACTIONS(1891),
    [sym_timeline_create] = ACTIONS(1891),
    [sym_timeline_kill] = ACTIONS(1891),
    [sym_timeline_reload] = ACTIONS(1891),
    [sym_timeline_pause] = ACTIONS(1891),
    [sym_timeline_restart] = ACTIONS(1891),
    [sym_timeline_set] = ACTIONS(1891),
    [sym_timeline_get] = ACTIONS(1891),
    [sym_timeline_active] = ACTIONS(1891),
    [sym_timeline_dynamic_id] = ACTIONS(1891),
    [sym_variable_to_string] = ACTIONS(1891),
    [sym_string_to_variable] = ACTIONS(1891),
    [sym_length_variable_to_string] = ACTIONS(1891),
    [sym_type_cast] = ACTIONS(1891),
    [sym_variable_to_xml] = ACTIONS(1891),
    [sym_xml_to_variable] = ACTIONS(1891),
    [sym_length_variable_to_xml] = ACTIONS(1891),
    [sym_redirect_string] = ACTIONS(1891),
    [sym_random_number] = ACTIONS(1891),
    [sym_get_unique_id] = ACTIONS(1891),
    [sym_get_system_number] = ACTIONS(1891),
    [sym_set_system_number] = ACTIONS(1891),
    [sym_get_serial_number] = ACTIONS(1891),
    [sym_get_master_build] = ACTIONS(1891),
    [sym_set_validation_code] = ACTIONS(1891),
    [sym_reboot] = ACTIONS(1891),
    [sym_load_duet_module] = ACTIONS(1891),
    [sym_unload_duet_module] = ACTIONS(1891),
    [sym_duet_mem_size_set] = ACTIONS(1891),
    [sym_duet_mem_size_get] = ACTIONS(1891),
    [sym_dynamic_application_device] = ACTIONS(1891),
    [sym_dynamic_polled_port] = ACTIONS(1891),
    [sym_static_port_binding] = ACTIONS(1891),
    [sym_static_ip_binding] = ACTIONS(1891),
    [sym_get_dns_list] = ACTIONS(1891),
    [sym_set_dns_list] = ACTIONS(1891),
    [sym_get_ip_address] = ACTIONS(1891),
    [sym_set_ip_address] = ACTIONS(1891),
    [sym_get_url_list] = ACTIONS(1891),
    [sym_add_url_entry] = ACTIONS(1891),
    [sym_delete_url_entry] = ACTIONS(1891),
    [sym_abs_value] = ACTIONS(1891),
    [sym_max_value] = ACTIONS(1891),
    [sym_min_value] = ACTIONS(1891),
    [sym_exp_value] = ACTIONS(1891),
    [sym_log_value] = ACTIONS(1891),
    [sym_log10_value] = ACTIONS(1891),
    [sym_power_value] = ACTIONS(1891),
    [sym_sqrt_value] = ACTIONS(1891),
    [sym_raw_be] = ACTIONS(1891),
    [sym_raw_le] = ACTIONS(1891),
    [sym_internal_queue_size_set] = ACTIONS(1891),
    [sym_internal_queue_size_get] = ACTIONS(1891),
    [sym_internal_threshold_set] = ACTIONS(1891),
    [sym_internal_threshold_get] = ACTIONS(1891),
    [sym_rebuilt_event] = ACTIONS(1891),
    [sym_do_custom_event] = ACTIONS(1891),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1891),
    [sym_clkmgr_set_clk_source] = ACTIONS(1891),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1891),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1891),
    [sym_clkmgr_get_timezone] = ACTIONS(1891),
    [sym_clkmgr_set_timezone] = ACTIONS(1891),
    [sym_clkmgr_get_resync_period] = ACTIONS(1891),
    [sym_clkmgr_set_resync_period] = ACTIONS(1891),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1891),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1891),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1891),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1891),
    [sym_clkmgr_get_timeservers] = ACTIONS(1891),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1891),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1891),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1891),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1891),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1891),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1891),
    [sym_set_log_level] = ACTIONS(1891),
    [sym_get_log_level] = ACTIONS(1891),
    [sym_amx_log] = ACTIONS(1891),
    [sym_smtp_server_config_set] = ACTIONS(1891),
    [sym_smtp_server_config_get] = ACTIONS(1891),
    [sym_smtp_send] = ACTIONS(1891),
    [sym_validate_netlinx_account] = ACTIONS(1891),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1891),
    [sym_authenticate_certificate] = ACTIONS(1891),
    [sym_audit_netlinx_session_event] = ACTIONS(1891),
    [sym_audit_netlinx_generic_event] = ACTIONS(1891),
    [sym_get_available_flash_disk_space] = ACTIONS(1891),
    [sym_get_max_flash_disk_space] = ACTIONS(1891),
    [sym_data_create_feed] = ACTIONS(1891),
    [sym_data_delete_feed] = ACTIONS(1891),
    [sym_data_publish_feed] = ACTIONS(1891),
    [sym_data_get_published_feed] = ACTIONS(1891),
    [sym_data_add_record] = ACTIONS(1891),
    [sym_data_get_event_record] = ACTIONS(1891),
    [sym__wc_data_create_feed] = ACTIONS(1891),
    [sym__wc_data_add_record] = ACTIONS(1891),
    [sym__wc_data_get_event_record] = ACTIONS(1891),
    [sym_format] = ACTIONS(1891),
    [sym_astro_clock] = ACTIONS(1891),
    [sym_set_outdoor_temperature] = ACTIONS(1891),
    [sym_set_virtual_level_count] = ACTIONS(1891),
    [sym_set_virtual_channel_count] = ACTIONS(1891),
    [sym_set_virtual_port_count] = ACTIONS(1891),
    [sym__wc_encode_int] = ACTIONS(1891),
    [sym__wc_decode_int] = ACTIONS(1891),
    [sym__wc_int] = ACTIONS(1891),
    [sym__wc_to_ch_int] = ACTIONS(1891),
    [sym__ch_to_wc_int] = ACTIONS(1891),
    [sym__wc_find_string_int] = ACTIONS(1891),
    [sym__wc_left_string_int] = ACTIONS(1891),
    [sym__wc_lower_string_int] = ACTIONS(1891),
    [sym__wc_mid_string_int] = ACTIONS(1891),
    [sym__wc_remove_string_int] = ACTIONS(1891),
    [sym__wc_right_string_int] = ACTIONS(1891),
    [sym__wc_upper_string_int] = ACTIONS(1891),
    [sym__wc_compare_string_int] = ACTIONS(1891),
    [sym__wc_get_buffer_char_int] = ACTIONS(1891),
    [sym__wc_get_buffer_string_int] = ACTIONS(1891),
    [sym__wc_concat_string_int] = ACTIONS(1891),
    [sym__wc_file_open_int] = ACTIONS(1891),
    [sym__wc_file_close_int] = ACTIONS(1891),
    [sym__wc_file_read_int] = ACTIONS(1891),
    [sym__wc_file_read_line_int] = ACTIONS(1891),
    [sym__wc_file_write_int] = ACTIONS(1891),
    [sym__wc_file_write_line_int] = ACTIONS(1891),
    [aux_sym_preproc_include_token1] = ACTIONS(1891),
    [aux_sym_preproc_define_token1] = ACTIONS(1893),
    [aux_sym_preproc_warn_token1] = ACTIONS(1893),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1893),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1893),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1893),
    [anon_sym_LBRACK] = ACTIONS(1893),
    [anon_sym_LBRACE] = ACTIONS(1893),
    [anon_sym_LPAREN] = ACTIONS(1891),
    [anon_sym_BANG] = ACTIONS(1893),
    [anon_sym_TILDE] = ACTIONS(1893),
    [anon_sym_DASH] = ACTIONS(1891),
    [anon_sym_PLUS] = ACTIONS(1891),
    [anon_sym_DASH_DASH] = ACTIONS(1893),
    [anon_sym_PLUS_PLUS] = ACTIONS(1893),
    [anon_sym_DQUOTE] = ACTIONS(1893),
    [anon_sym_SQUOTE] = ACTIONS(1893),
    [sym_number_literal] = ACTIONS(1893),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(443)] = {
    [ts_builtin_sym_end] = ACTIONS(2047),
    [sym_identifier] = ACTIONS(2045),
    [sym_program_name_keyword] = ACTIONS(2045),
    [sym_module_name_keyword] = ACTIONS(2045),
    [sym_define_device_keyword] = ACTIONS(2045),
    [sym_define_combine_keyword] = ACTIONS(2045),
    [sym_define_constant_keyword] = ACTIONS(2045),
    [sym_define_type_keyword] = ACTIONS(2045),
    [sym_define_variable_keyword] = ACTIONS(2045),
    [sym_define_system_variable_keyword] = ACTIONS(2045),
    [sym_define_connect_level_keyword] = ACTIONS(2045),
    [sym_define_latching_keyword] = ACTIONS(2045),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2045),
    [sym_define_toggling_keyword] = ACTIONS(2045),
    [sym_define_call_keyword] = ACTIONS(2045),
    [sym_define_function_keyword] = ACTIONS(2045),
    [sym_define_library_function_keyword] = ACTIONS(2045),
    [sym_define_start_keyword] = ACTIONS(2045),
    [sym_define_event_keyword] = ACTIONS(2045),
    [sym_define_program_keyword] = ACTIONS(2045),
    [sym_define_module_keyword] = ACTIONS(2045),
    [sym_char_keyword] = ACTIONS(2045),
    [sym_widechar_keyword] = ACTIONS(2045),
    [sym_integer_keyword] = ACTIONS(2045),
    [sym_sinteger_keyword] = ACTIONS(2045),
    [sym_long_keyword] = ACTIONS(2045),
    [sym_slong_keyword] = ACTIONS(2045),
    [sym_float_keyword] = ACTIONS(2045),
    [sym_double_keyword] = ACTIONS(2045),
    [sym_variant_keyword] = ACTIONS(2045),
    [sym_variantarray_keyword] = ACTIONS(2045),
    [sym_local_var_keyword] = ACTIONS(2045),
    [sym_stack_var_keyword] = ACTIONS(2045),
    [sym_constant_keyword] = ACTIONS(2045),
    [sym_volatile_keyword] = ACTIONS(2045),
    [sym_non_volatile_keyword] = ACTIONS(2045),
    [sym_persistent_keyword] = ACTIONS(2045),
    [sym_dev_keyword] = ACTIONS(2045),
    [sym_devlev_keyword] = ACTIONS(2045),
    [sym_devchan_keyword] = ACTIONS(2045),
    [sym_if_keyword] = ACTIONS(2045),
    [sym_switch_keyword] = ACTIONS(2045),
    [sym_case_keyword] = ACTIONS(2045),
    [sym_default_keyword] = ACTIONS(2045),
    [sym_while_keyword] = ACTIONS(2045),
    [sym_for_keyword] = ACTIONS(2045),
    [sym_break_keyword] = ACTIONS(2045),
    [sym_continue_keyword] = ACTIONS(2045),
    [sym_return_keyword] = ACTIONS(2045),
    [sym_select_keyword] = ACTIONS(2045),
    [sym_active_keyword] = ACTIONS(2045),
    [sym_struct_keyword] = ACTIONS(2045),
    [sym_structure_keyword] = ACTIONS(2045),
    [sym_button_event_keyword] = ACTIONS(2045),
    [sym_channel_event_keyword] = ACTIONS(2045),
    [sym_level_event_keyword] = ACTIONS(2045),
    [sym_data_event_keyword] = ACTIONS(2045),
    [sym_timeline_event_keyword] = ACTIONS(2045),
    [sym_custom_event_keyword] = ACTIONS(2045),
    [sym_push_keyword] = ACTIONS(2045),
    [sym_release_keyword] = ACTIONS(2045),
    [aux_sym_on_keyword_token1] = ACTIONS(2045),
    [aux_sym_off_keyword_token1] = ACTIONS(2045),
    [sym_send_level_keyword] = ACTIONS(2045),
    [sym_send_string_keyword] = ACTIONS(2045),
    [sym_send_command_keyword] = ACTIONS(2045),
    [sym_clear_buffer_keyword] = ACTIONS(2045),
    [sym_create_buffer_keyword] = ACTIONS(2045),
    [sym_create_multi_buffer_keyword] = ACTIONS(2045),
    [sym_call_keyword] = ACTIONS(2045),
    [sym_system_call_keyword] = ACTIONS(2045),
    [sym_devchan_to_keyword] = ACTIONS(2045),
    [sym_devchan_min_to_keyword] = ACTIONS(2045),
    [sym_devchan_total_off_keyword] = ACTIONS(2045),
    [sym_devchan_pulse_keyword] = ACTIONS(2045),
    [sym_wait_keyword] = ACTIONS(2045),
    [sym_cancel_wait_keyword] = ACTIONS(2045),
    [sym_wait_until_keyword] = ACTIONS(2045),
    [sym_cancel_wait_until_keyword] = ACTIONS(2045),
    [sym_cancel_all_wait_keyword] = ACTIONS(2045),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2045),
    [sym_not] = ACTIONS(2045),
    [sym_bnot] = ACTIONS(2045),
    [sym___file__] = ACTIONS(2045),
    [sym___date__] = ACTIONS(2045),
    [sym___ldate__] = ACTIONS(2045),
    [sym___time__] = ACTIONS(2045),
    [sym___line__] = ACTIONS(2045),
    [sym___name__] = ACTIONS(2045),
    [sym___netlinx__] = ACTIONS(2045),
    [sym_netlinx_axi_version] = ACTIONS(2045),
    [sym_url_flg_tcp] = ACTIONS(2045),
    [sym_url_flg_accinfopresent] = ACTIONS(2045),
    [sym_url_flg_temp] = ACTIONS(2045),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2045),
    [sym_url_flg_stat_mask] = ACTIONS(2045),
    [sym_url_flg_stat_lookup] = ACTIONS(2045),
    [sym_url_flg_stat_connecting] = ACTIONS(2045),
    [sym_url_flg_stat_waiting] = ACTIONS(2045),
    [sym_url_flg_stat_connected] = ACTIONS(2045),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2045),
    [sym_first_virtual_device] = ACTIONS(2045),
    [sym_first_local_port] = ACTIONS(2045),
    [sym_dynamic_virtual_device] = ACTIONS(2045),
    [sym_timeline_once] = ACTIONS(2045),
    [sym_timeline_repeat] = ACTIONS(2045),
    [sym_timeline_absolute] = ACTIONS(2045),
    [sym_timeline_relative] = ACTIONS(2045),
    [sym_do_push_timed_infinite] = ACTIONS(2045),
    [sym_source_type_no_address] = ACTIONS(2045),
    [sym_source_type_neuron_id] = ACTIONS(2045),
    [sym_source_type_ip_address] = ACTIONS(2045),
    [sym_source_type_axlink] = ACTIONS(2045),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2045),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2045),
    [sym_source_type_ip_socket_address] = ACTIONS(2045),
    [sym_source_type_rs232] = ACTIONS(2045),
    [sym_source_type_internal] = ACTIONS(2045),
    [sym_source_type_mac_address] = ACTIONS(2045),
    [sym_source_type_ipv4_port] = ACTIONS(2045),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2045),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2045),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2045),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2045),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2045),
    [sym_true] = ACTIONS(2045),
    [sym_false] = ACTIONS(2045),
    [sym_file_read_only] = ACTIONS(2045),
    [sym_file_rw_new] = ACTIONS(2045),
    [sym_file_rw_append] = ACTIONS(2045),
    [sym_ip_tcp] = ACTIONS(2045),
    [sym_ip_udp] = ACTIONS(2045),
    [sym_ip_udp_2way] = ACTIONS(2045),
    [sym_xml_encode_types] = ACTIONS(2045),
    [sym_xml_encode_char_as_list] = ACTIONS(2045),
    [sym_xml_encode_le] = ACTIONS(2045),
    [sym_xml_decode_types] = ACTIONS(2045),
    [sym_xml_decode_no_preserve] = ACTIONS(2045),
    [sym_duet_dev_type_display_device] = ACTIONS(2045),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2045),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2045),
    [sym_duet_dev_type_io_device] = ACTIONS(2045),
    [sym_duet_dev_type_relay_device] = ACTIONS(2045),
    [sym_duet_dev_type_ups] = ACTIONS(2045),
    [sym_duet_dev_type_amplifier] = ACTIONS(2045),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2045),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2045),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2045),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2045),
    [sym_duet_dev_type_camera] = ACTIONS(2045),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2045),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2045),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2045),
    [sym_duet_dev_type_dss] = ACTIONS(2045),
    [sym_duet_dev_type_dvr] = ACTIONS(2045),
    [sym_duet_dev_type_disc_device] = ACTIONS(2045),
    [sym_duet_dev_type_document_camera] = ACTIONS(2045),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2045),
    [sym_duet_dev_type_hvac] = ACTIONS(2045),
    [sym_duet_dev_type_keypad] = ACTIONS(2045),
    [sym_duet_dev_type_light] = ACTIONS(2045),
    [sym_duet_dev_type_monitor] = ACTIONS(2045),
    [sym_duet_dev_type_motor] = ACTIONS(2045),
    [sym_duet_dev_type_multi_window] = ACTIONS(2045),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2045),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2045),
    [sym_duet_dev_type_receiver] = ACTIONS(2045),
    [sym_duet_dev_type_security_system] = ACTIONS(2045),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2045),
    [sym_duet_dev_type_settop_box] = ACTIONS(2045),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2045),
    [sym_duet_dev_type_switcher] = ACTIONS(2045),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2045),
    [sym_duet_dev_type_tv] = ACTIONS(2045),
    [sym_duet_dev_type_utility] = ACTIONS(2045),
    [sym_duet_dev_type_vcr] = ACTIONS(2045),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2045),
    [sym_duet_dev_type_video_processor] = ACTIONS(2045),
    [sym_duet_dev_type_video_projector] = ACTIONS(2045),
    [sym_duet_dev_type_video_wall] = ACTIONS(2045),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2045),
    [sym_duet_dev_type_weather] = ACTIONS(2045),
    [sym_duet_dev_not_polled] = ACTIONS(2045),
    [sym_duet_dev_polled] = ACTIONS(2045),
    [sym_clkmgr_mode_network] = ACTIONS(2045),
    [sym_clkmgr_mode_standalone] = ACTIONS(2045),
    [sym_normal_standby] = ACTIONS(2045),
    [sym_normal_wake] = ACTIONS(2045),
    [sym_ip_multicast_ttl_option] = ACTIONS(2045),
    [sym_ip_ttl_subnet] = ACTIONS(2045),
    [sym_ip_ttl_site] = ACTIONS(2045),
    [sym_ip_ttl_region] = ACTIONS(2045),
    [sym_ip_ttl_continent] = ACTIONS(2045),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2045),
    [sym_ip_nodelay_on] = ACTIONS(2045),
    [sym_ip_nodelay_off] = ACTIONS(2045),
    [sym_amx_error] = ACTIONS(2045),
    [sym_amx_warning] = ACTIONS(2045),
    [sym_amx_info] = ACTIONS(2045),
    [sym_amx_debug] = ACTIONS(2045),
    [sym_netlinx_login_success] = ACTIONS(2045),
    [sym_netlinx_login_fail] = ACTIONS(2045),
    [sym_netlinx_logout] = ACTIONS(2045),
    [sym_null_str] = ACTIONS(2045),
    [sym_smtp_address] = ACTIONS(2045),
    [sym_smtp_port_number] = ACTIONS(2045),
    [sym_smtp_username] = ACTIONS(2045),
    [sym_smtp_password] = ACTIONS(2045),
    [sym_smtp_require_tls] = ACTIONS(2045),
    [sym_smtp_from] = ACTIONS(2045),
    [sym_smtp_tls_true] = ACTIONS(2045),
    [sym_smtp_tls_false] = ACTIONS(2045),
    [sym_listview_on_row_select_event] = ACTIONS(2045),
    [sym_data_structure_unknown] = ACTIONS(2045),
    [sym_data_structure_datafeed] = ACTIONS(2045),
    [sym_data_structure_datarecordset] = ACTIONS(2045),
    [sym_data_structure_datarecord] = ACTIONS(2045),
    [sym_data_structure_datafield] = ACTIONS(2045),
    [sym_data_max_content_field_count] = ACTIONS(2045),
    [sym_data_max_metadata_field_count] = ACTIONS(2045),
    [sym_data_max_id_length] = ACTIONS(2045),
    [sym_data_max_name_length] = ACTIONS(2045),
    [sym_data_max_description_length] = ACTIONS(2045),
    [sym_data_max_type_length] = ACTIONS(2045),
    [sym_data_max_format_length] = ACTIONS(2045),
    [sym_data_max_source_length] = ACTIONS(2045),
    [sym_data_max_value_length] = ACTIONS(2045),
    [sym_data_max_label_length] = ACTIONS(2045),
    [sym_data_max_url_length] = ACTIONS(2045),
    [sym_max_last_login_info_length] = ACTIONS(2045),
    [sym_data_type_unknown] = ACTIONS(2045),
    [sym_data_type_string] = ACTIONS(2045),
    [sym_data_type_datetime] = ACTIONS(2045),
    [sym_data_type_date] = ACTIONS(2045),
    [sym_data_type_time] = ACTIONS(2045),
    [sym_data_type_image] = ACTIONS(2045),
    [sym_data_format_url] = ACTIONS(2045),
    [sym_data_format_phone] = ACTIONS(2045),
    [sym_data_format_email] = ACTIONS(2045),
    [sym_data_format_iso8601] = ACTIONS(2045),
    [sym_auditlog_priv] = ACTIONS(2045),
    [sym_configuration_priv] = ACTIONS(2045),
    [sym_deviceconfig_priv] = ACTIONS(2045),
    [sym_ftp_priv] = ACTIONS(2045),
    [sym_http_priv] = ACTIONS(2045),
    [sym_networkconfig_priv] = ACTIONS(2045),
    [sym_programport_priv] = ACTIONS(2045),
    [sym_remoteui_priv] = ACTIONS(2045),
    [sym_securitycontrol_priv] = ACTIONS(2045),
    [sym_softwaremanagement_priv] = ACTIONS(2045),
    [sym_terminal_priv] = ACTIONS(2045),
    [sym_tpadmin_priv] = ACTIONS(2045),
    [sym_user1_priv] = ACTIONS(2045),
    [sym_user2_priv] = ACTIONS(2045),
    [sym_user3_priv] = ACTIONS(2045),
    [sym_user4_priv] = ACTIONS(2045),
    [sym_usermanagement_priv] = ACTIONS(2045),
    [sym_valid_account] = ACTIONS(2045),
    [sym_err_name_invalid] = ACTIONS(2045),
    [sym_err_password_invalid] = ACTIONS(2045),
    [sym_err_invalid_account] = ACTIONS(2045),
    [sym_err_no_authorization] = ACTIONS(2045),
    [sym_err_invalid_parameter] = ACTIONS(2045),
    [sym_err_user_locked_out] = ACTIONS(2045),
    [sym_err_user_account_expired] = ACTIONS(2045),
    [sym_err_general_error] = ACTIONS(2045),
    [sym_err_authentication_error] = ACTIONS(2045),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2045),
    [sym_tls_validate_certificate] = ACTIONS(2045),
    [sym_tdata] = ACTIONS(2045),
    [sym_tchannel] = ACTIONS(2045),
    [sym_tlevel] = ACTIONS(2045),
    [sym_tbutton] = ACTIONS(2045),
    [sym_ttimeline] = ACTIONS(2045),
    [sym_tcustom] = ACTIONS(2045),
    [sym_url_struct] = ACTIONS(2045),
    [sym_dns_struct] = ACTIONS(2045),
    [sym_ip_address_struct] = ACTIONS(2045),
    [sym_dev_info_struct] = ACTIONS(2045),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2045),
    [sym_data_feed] = ACTIONS(2045),
    [sym_data_field] = ACTIONS(2045),
    [sym_data_record] = ACTIONS(2045),
    [sym_wc_data_feed] = ACTIONS(2045),
    [sym_wc_data_field] = ACTIONS(2045),
    [sym_wc_data_record] = ACTIONS(2045),
    [sym_last_login_info] = ACTIONS(2045),
    [sym____reserved___] = ACTIONS(2045),
    [sym_date] = ACTIONS(2045),
    [sym_day] = ACTIONS(2045),
    [sym_time] = ACTIONS(2045),
    [sym_ldate] = ACTIONS(2045),
    [sym_push_device] = ACTIONS(2045),
    [sym_push_channel] = ACTIONS(2045),
    [sym_push_devchan] = ACTIONS(2045),
    [sym_release_device] = ACTIONS(2045),
    [sym_release_channel] = ACTIONS(2045),
    [sym_release_devchan] = ACTIONS(2045),
    [sym_master_slot] = ACTIONS(2045),
    [sym_get_pulse_time] = ACTIONS(2045),
    [sym_get_timer] = ACTIONS(2045),
    [sym_master_sn] = ACTIONS(2045),
    [sym_system_number] = ACTIONS(2045),
    [sym_button] = ACTIONS(2045),
    [sym_dv_channel] = ACTIONS(2045),
    [sym_data] = ACTIONS(2045),
    [sym_level] = ACTIONS(2045),
    [sym_timeline] = ACTIONS(2045),
    [sym_channel] = ACTIONS(2045),
    [sym_custom] = ACTIONS(2045),
    [sym_atoi] = ACTIONS(2045),
    [sym_atol] = ACTIONS(2045),
    [sym_atof] = ACTIONS(2045),
    [sym_itoa] = ACTIONS(2045),
    [sym_itohex] = ACTIONS(2045),
    [sym_hextoi] = ACTIONS(2045),
    [sym_ftoa] = ACTIONS(2045),
    [sym_find_string] = ACTIONS(2045),
    [sym_left_string] = ACTIONS(2045),
    [sym_right_string] = ACTIONS(2045),
    [sym_mid_string] = ACTIONS(2045),
    [sym_remove_string] = ACTIONS(2045),
    [sym_get_buffer_string] = ACTIONS(2045),
    [sym_get_buffer_char] = ACTIONS(2045),
    [sym_get_multi_buffer_string] = ACTIONS(2045),
    [sym_length_string] = ACTIONS(2045),
    [sym_max_length_string] = ACTIONS(2045),
    [sym_set_length_string] = ACTIONS(2045),
    [sym_compare_string] = ACTIONS(2045),
    [sym_upper_string] = ACTIONS(2045),
    [sym_lower_string] = ACTIONS(2045),
    [sym_get_last] = ACTIONS(2045),
    [sym_date_to_day] = ACTIONS(2045),
    [sym_date_to_month] = ACTIONS(2045),
    [sym_date_to_year] = ACTIONS(2045),
    [sym_time_to_hour] = ACTIONS(2045),
    [sym_time_to_minute] = ACTIONS(2045),
    [sym_time_to_second] = ACTIONS(2045),
    [sym_day_of_week] = ACTIONS(2045),
    [sym_set_pulse_time] = ACTIONS(2045),
    [sym_set_timer] = ACTIONS(2045),
    [sym_file_open] = ACTIONS(2045),
    [sym_file_close] = ACTIONS(2045),
    [sym_file_read] = ACTIONS(2045),
    [sym_file_read_line] = ACTIONS(2045),
    [sym_file_write] = ACTIONS(2045),
    [sym_file_write_line] = ACTIONS(2045),
    [sym_file_copy] = ACTIONS(2045),
    [sym_file_delete] = ACTIONS(2045),
    [sym_file_rename] = ACTIONS(2045),
    [sym_file_dir] = ACTIONS(2045),
    [sym_file_getdir] = ACTIONS(2045),
    [sym_file_setdir] = ACTIONS(2045),
    [sym_file_create_dir] = ACTIONS(2045),
    [sym_file_remove_dir] = ACTIONS(2045),
    [sym_file_seek] = ACTIONS(2045),
    [sym_length_array] = ACTIONS(2045),
    [sym_max_length_array] = ACTIONS(2045),
    [sym_set_length_array] = ACTIONS(2045),
    [sym_ip_client_open] = ACTIONS(2045),
    [sym_ip_client_close] = ACTIONS(2045),
    [sym_ip_server_open] = ACTIONS(2045),
    [sym_ip_server_close] = ACTIONS(2045),
    [sym_ip_bound_client_open] = ACTIONS(2045),
    [sym_ip_mc_server_open] = ACTIONS(2045),
    [sym_ip_set_option] = ACTIONS(2045),
    [sym_ssh_client_open] = ACTIONS(2045),
    [sym_ssh_client_close] = ACTIONS(2045),
    [sym_tls_client_open] = ACTIONS(2045),
    [sym_tls_client_close] = ACTIONS(2045),
    [sym_do_push] = ACTIONS(2045),
    [sym_do_push_timed] = ACTIONS(2045),
    [sym_do_release] = ACTIONS(2045),
    [sym_device_id] = ACTIONS(2045),
    [sym_device_info] = ACTIONS(2045),
    [sym_device_id_string] = ACTIONS(2045),
    [sym_device_standby] = ACTIONS(2045),
    [sym_device_wake] = ACTIONS(2045),
    [sym_timeline_create] = ACTIONS(2045),
    [sym_timeline_kill] = ACTIONS(2045),
    [sym_timeline_reload] = ACTIONS(2045),
    [sym_timeline_pause] = ACTIONS(2045),
    [sym_timeline_restart] = ACTIONS(2045),
    [sym_timeline_set] = ACTIONS(2045),
    [sym_timeline_get] = ACTIONS(2045),
    [sym_timeline_active] = ACTIONS(2045),
    [sym_timeline_dynamic_id] = ACTIONS(2045),
    [sym_variable_to_string] = ACTIONS(2045),
    [sym_string_to_variable] = ACTIONS(2045),
    [sym_length_variable_to_string] = ACTIONS(2045),
    [sym_type_cast] = ACTIONS(2045),
    [sym_variable_to_xml] = ACTIONS(2045),
    [sym_xml_to_variable] = ACTIONS(2045),
    [sym_length_variable_to_xml] = ACTIONS(2045),
    [sym_redirect_string] = ACTIONS(2045),
    [sym_random_number] = ACTIONS(2045),
    [sym_get_unique_id] = ACTIONS(2045),
    [sym_get_system_number] = ACTIONS(2045),
    [sym_set_system_number] = ACTIONS(2045),
    [sym_get_serial_number] = ACTIONS(2045),
    [sym_get_master_build] = ACTIONS(2045),
    [sym_set_validation_code] = ACTIONS(2045),
    [sym_reboot] = ACTIONS(2045),
    [sym_load_duet_module] = ACTIONS(2045),
    [sym_unload_duet_module] = ACTIONS(2045),
    [sym_duet_mem_size_set] = ACTIONS(2045),
    [sym_duet_mem_size_get] = ACTIONS(2045),
    [sym_dynamic_application_device] = ACTIONS(2045),
    [sym_dynamic_polled_port] = ACTIONS(2045),
    [sym_static_port_binding] = ACTIONS(2045),
    [sym_static_ip_binding] = ACTIONS(2045),
    [sym_get_dns_list] = ACTIONS(2045),
    [sym_set_dns_list] = ACTIONS(2045),
    [sym_get_ip_address] = ACTIONS(2045),
    [sym_set_ip_address] = ACTIONS(2045),
    [sym_get_url_list] = ACTIONS(2045),
    [sym_add_url_entry] = ACTIONS(2045),
    [sym_delete_url_entry] = ACTIONS(2045),
    [sym_abs_value] = ACTIONS(2045),
    [sym_max_value] = ACTIONS(2045),
    [sym_min_value] = ACTIONS(2045),
    [sym_exp_value] = ACTIONS(2045),
    [sym_log_value] = ACTIONS(2045),
    [sym_log10_value] = ACTIONS(2045),
    [sym_power_value] = ACTIONS(2045),
    [sym_sqrt_value] = ACTIONS(2045),
    [sym_raw_be] = ACTIONS(2045),
    [sym_raw_le] = ACTIONS(2045),
    [sym_internal_queue_size_set] = ACTIONS(2045),
    [sym_internal_queue_size_get] = ACTIONS(2045),
    [sym_internal_threshold_set] = ACTIONS(2045),
    [sym_internal_threshold_get] = ACTIONS(2045),
    [sym_rebuilt_event] = ACTIONS(2045),
    [sym_do_custom_event] = ACTIONS(2045),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2045),
    [sym_clkmgr_set_clk_source] = ACTIONS(2045),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2045),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2045),
    [sym_clkmgr_get_timezone] = ACTIONS(2045),
    [sym_clkmgr_set_timezone] = ACTIONS(2045),
    [sym_clkmgr_get_resync_period] = ACTIONS(2045),
    [sym_clkmgr_set_resync_period] = ACTIONS(2045),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2045),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2045),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2045),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2045),
    [sym_clkmgr_get_timeservers] = ACTIONS(2045),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2045),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2045),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2045),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2045),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2045),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2045),
    [sym_set_log_level] = ACTIONS(2045),
    [sym_get_log_level] = ACTIONS(2045),
    [sym_amx_log] = ACTIONS(2045),
    [sym_smtp_server_config_set] = ACTIONS(2045),
    [sym_smtp_server_config_get] = ACTIONS(2045),
    [sym_smtp_send] = ACTIONS(2045),
    [sym_validate_netlinx_account] = ACTIONS(2045),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2045),
    [sym_authenticate_certificate] = ACTIONS(2045),
    [sym_audit_netlinx_session_event] = ACTIONS(2045),
    [sym_audit_netlinx_generic_event] = ACTIONS(2045),
    [sym_get_available_flash_disk_space] = ACTIONS(2045),
    [sym_get_max_flash_disk_space] = ACTIONS(2045),
    [sym_data_create_feed] = ACTIONS(2045),
    [sym_data_delete_feed] = ACTIONS(2045),
    [sym_data_publish_feed] = ACTIONS(2045),
    [sym_data_get_published_feed] = ACTIONS(2045),
    [sym_data_add_record] = ACTIONS(2045),
    [sym_data_get_event_record] = ACTIONS(2045),
    [sym__wc_data_create_feed] = ACTIONS(2045),
    [sym__wc_data_add_record] = ACTIONS(2045),
    [sym__wc_data_get_event_record] = ACTIONS(2045),
    [sym_format] = ACTIONS(2045),
    [sym_astro_clock] = ACTIONS(2045),
    [sym_set_outdoor_temperature] = ACTIONS(2045),
    [sym_set_virtual_level_count] = ACTIONS(2045),
    [sym_set_virtual_channel_count] = ACTIONS(2045),
    [sym_set_virtual_port_count] = ACTIONS(2045),
    [sym__wc_encode_int] = ACTIONS(2045),
    [sym__wc_decode_int] = ACTIONS(2045),
    [sym__wc_int] = ACTIONS(2045),
    [sym__wc_to_ch_int] = ACTIONS(2045),
    [sym__ch_to_wc_int] = ACTIONS(2045),
    [sym__wc_find_string_int] = ACTIONS(2045),
    [sym__wc_left_string_int] = ACTIONS(2045),
    [sym__wc_lower_string_int] = ACTIONS(2045),
    [sym__wc_mid_string_int] = ACTIONS(2045),
    [sym__wc_remove_string_int] = ACTIONS(2045),
    [sym__wc_right_string_int] = ACTIONS(2045),
    [sym__wc_upper_string_int] = ACTIONS(2045),
    [sym__wc_compare_string_int] = ACTIONS(2045),
    [sym__wc_get_buffer_char_int] = ACTIONS(2045),
    [sym__wc_get_buffer_string_int] = ACTIONS(2045),
    [sym__wc_concat_string_int] = ACTIONS(2045),
    [sym__wc_file_open_int] = ACTIONS(2045),
    [sym__wc_file_close_int] = ACTIONS(2045),
    [sym__wc_file_read_int] = ACTIONS(2045),
    [sym__wc_file_read_line_int] = ACTIONS(2045),
    [sym__wc_file_write_int] = ACTIONS(2045),
    [sym__wc_file_write_line_int] = ACTIONS(2045),
    [aux_sym_preproc_include_token1] = ACTIONS(2045),
    [aux_sym_preproc_define_token1] = ACTIONS(2047),
    [aux_sym_preproc_warn_token1] = ACTIONS(2047),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2047),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2047),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2047),
    [anon_sym_LBRACK] = ACTIONS(2047),
    [anon_sym_LBRACE] = ACTIONS(2047),
    [anon_sym_LPAREN] = ACTIONS(2045),
    [anon_sym_BANG] = ACTIONS(2047),
    [anon_sym_TILDE] = ACTIONS(2047),
    [anon_sym_DASH] = ACTIONS(2045),
    [anon_sym_PLUS] = ACTIONS(2045),
    [anon_sym_DASH_DASH] = ACTIONS(2047),
    [anon_sym_PLUS_PLUS] = ACTIONS(2047),
    [anon_sym_DQUOTE] = ACTIONS(2047),
    [anon_sym_SQUOTE] = ACTIONS(2047),
    [sym_number_literal] = ACTIONS(2047),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(444)] = {
    [ts_builtin_sym_end] = ACTIONS(2051),
    [sym_identifier] = ACTIONS(2049),
    [sym_program_name_keyword] = ACTIONS(2049),
    [sym_module_name_keyword] = ACTIONS(2049),
    [sym_define_device_keyword] = ACTIONS(2049),
    [sym_define_combine_keyword] = ACTIONS(2049),
    [sym_define_constant_keyword] = ACTIONS(2049),
    [sym_define_type_keyword] = ACTIONS(2049),
    [sym_define_variable_keyword] = ACTIONS(2049),
    [sym_define_system_variable_keyword] = ACTIONS(2049),
    [sym_define_connect_level_keyword] = ACTIONS(2049),
    [sym_define_latching_keyword] = ACTIONS(2049),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2049),
    [sym_define_toggling_keyword] = ACTIONS(2049),
    [sym_define_call_keyword] = ACTIONS(2049),
    [sym_define_function_keyword] = ACTIONS(2049),
    [sym_define_library_function_keyword] = ACTIONS(2049),
    [sym_define_start_keyword] = ACTIONS(2049),
    [sym_define_event_keyword] = ACTIONS(2049),
    [sym_define_program_keyword] = ACTIONS(2049),
    [sym_define_module_keyword] = ACTIONS(2049),
    [sym_char_keyword] = ACTIONS(2049),
    [sym_widechar_keyword] = ACTIONS(2049),
    [sym_integer_keyword] = ACTIONS(2049),
    [sym_sinteger_keyword] = ACTIONS(2049),
    [sym_long_keyword] = ACTIONS(2049),
    [sym_slong_keyword] = ACTIONS(2049),
    [sym_float_keyword] = ACTIONS(2049),
    [sym_double_keyword] = ACTIONS(2049),
    [sym_variant_keyword] = ACTIONS(2049),
    [sym_variantarray_keyword] = ACTIONS(2049),
    [sym_local_var_keyword] = ACTIONS(2049),
    [sym_stack_var_keyword] = ACTIONS(2049),
    [sym_constant_keyword] = ACTIONS(2049),
    [sym_volatile_keyword] = ACTIONS(2049),
    [sym_non_volatile_keyword] = ACTIONS(2049),
    [sym_persistent_keyword] = ACTIONS(2049),
    [sym_dev_keyword] = ACTIONS(2049),
    [sym_devlev_keyword] = ACTIONS(2049),
    [sym_devchan_keyword] = ACTIONS(2049),
    [sym_if_keyword] = ACTIONS(2049),
    [sym_switch_keyword] = ACTIONS(2049),
    [sym_case_keyword] = ACTIONS(2049),
    [sym_default_keyword] = ACTIONS(2049),
    [sym_while_keyword] = ACTIONS(2049),
    [sym_for_keyword] = ACTIONS(2049),
    [sym_break_keyword] = ACTIONS(2049),
    [sym_continue_keyword] = ACTIONS(2049),
    [sym_return_keyword] = ACTIONS(2049),
    [sym_select_keyword] = ACTIONS(2049),
    [sym_active_keyword] = ACTIONS(2049),
    [sym_struct_keyword] = ACTIONS(2049),
    [sym_structure_keyword] = ACTIONS(2049),
    [sym_button_event_keyword] = ACTIONS(2049),
    [sym_channel_event_keyword] = ACTIONS(2049),
    [sym_level_event_keyword] = ACTIONS(2049),
    [sym_data_event_keyword] = ACTIONS(2049),
    [sym_timeline_event_keyword] = ACTIONS(2049),
    [sym_custom_event_keyword] = ACTIONS(2049),
    [sym_push_keyword] = ACTIONS(2049),
    [sym_release_keyword] = ACTIONS(2049),
    [aux_sym_on_keyword_token1] = ACTIONS(2049),
    [aux_sym_off_keyword_token1] = ACTIONS(2049),
    [sym_send_level_keyword] = ACTIONS(2049),
    [sym_send_string_keyword] = ACTIONS(2049),
    [sym_send_command_keyword] = ACTIONS(2049),
    [sym_clear_buffer_keyword] = ACTIONS(2049),
    [sym_create_buffer_keyword] = ACTIONS(2049),
    [sym_create_multi_buffer_keyword] = ACTIONS(2049),
    [sym_call_keyword] = ACTIONS(2049),
    [sym_system_call_keyword] = ACTIONS(2049),
    [sym_devchan_to_keyword] = ACTIONS(2049),
    [sym_devchan_min_to_keyword] = ACTIONS(2049),
    [sym_devchan_total_off_keyword] = ACTIONS(2049),
    [sym_devchan_pulse_keyword] = ACTIONS(2049),
    [sym_wait_keyword] = ACTIONS(2049),
    [sym_cancel_wait_keyword] = ACTIONS(2049),
    [sym_wait_until_keyword] = ACTIONS(2049),
    [sym_cancel_wait_until_keyword] = ACTIONS(2049),
    [sym_cancel_all_wait_keyword] = ACTIONS(2049),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2049),
    [sym_not] = ACTIONS(2049),
    [sym_bnot] = ACTIONS(2049),
    [sym___file__] = ACTIONS(2049),
    [sym___date__] = ACTIONS(2049),
    [sym___ldate__] = ACTIONS(2049),
    [sym___time__] = ACTIONS(2049),
    [sym___line__] = ACTIONS(2049),
    [sym___name__] = ACTIONS(2049),
    [sym___netlinx__] = ACTIONS(2049),
    [sym_netlinx_axi_version] = ACTIONS(2049),
    [sym_url_flg_tcp] = ACTIONS(2049),
    [sym_url_flg_accinfopresent] = ACTIONS(2049),
    [sym_url_flg_temp] = ACTIONS(2049),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2049),
    [sym_url_flg_stat_mask] = ACTIONS(2049),
    [sym_url_flg_stat_lookup] = ACTIONS(2049),
    [sym_url_flg_stat_connecting] = ACTIONS(2049),
    [sym_url_flg_stat_waiting] = ACTIONS(2049),
    [sym_url_flg_stat_connected] = ACTIONS(2049),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2049),
    [sym_first_virtual_device] = ACTIONS(2049),
    [sym_first_local_port] = ACTIONS(2049),
    [sym_dynamic_virtual_device] = ACTIONS(2049),
    [sym_timeline_once] = ACTIONS(2049),
    [sym_timeline_repeat] = ACTIONS(2049),
    [sym_timeline_absolute] = ACTIONS(2049),
    [sym_timeline_relative] = ACTIONS(2049),
    [sym_do_push_timed_infinite] = ACTIONS(2049),
    [sym_source_type_no_address] = ACTIONS(2049),
    [sym_source_type_neuron_id] = ACTIONS(2049),
    [sym_source_type_ip_address] = ACTIONS(2049),
    [sym_source_type_axlink] = ACTIONS(2049),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2049),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2049),
    [sym_source_type_ip_socket_address] = ACTIONS(2049),
    [sym_source_type_rs232] = ACTIONS(2049),
    [sym_source_type_internal] = ACTIONS(2049),
    [sym_source_type_mac_address] = ACTIONS(2049),
    [sym_source_type_ipv4_port] = ACTIONS(2049),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2049),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2049),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2049),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2049),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2049),
    [sym_true] = ACTIONS(2049),
    [sym_false] = ACTIONS(2049),
    [sym_file_read_only] = ACTIONS(2049),
    [sym_file_rw_new] = ACTIONS(2049),
    [sym_file_rw_append] = ACTIONS(2049),
    [sym_ip_tcp] = ACTIONS(2049),
    [sym_ip_udp] = ACTIONS(2049),
    [sym_ip_udp_2way] = ACTIONS(2049),
    [sym_xml_encode_types] = ACTIONS(2049),
    [sym_xml_encode_char_as_list] = ACTIONS(2049),
    [sym_xml_encode_le] = ACTIONS(2049),
    [sym_xml_decode_types] = ACTIONS(2049),
    [sym_xml_decode_no_preserve] = ACTIONS(2049),
    [sym_duet_dev_type_display_device] = ACTIONS(2049),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2049),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2049),
    [sym_duet_dev_type_io_device] = ACTIONS(2049),
    [sym_duet_dev_type_relay_device] = ACTIONS(2049),
    [sym_duet_dev_type_ups] = ACTIONS(2049),
    [sym_duet_dev_type_amplifier] = ACTIONS(2049),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2049),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2049),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2049),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2049),
    [sym_duet_dev_type_camera] = ACTIONS(2049),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2049),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2049),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2049),
    [sym_duet_dev_type_dss] = ACTIONS(2049),
    [sym_duet_dev_type_dvr] = ACTIONS(2049),
    [sym_duet_dev_type_disc_device] = ACTIONS(2049),
    [sym_duet_dev_type_document_camera] = ACTIONS(2049),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2049),
    [sym_duet_dev_type_hvac] = ACTIONS(2049),
    [sym_duet_dev_type_keypad] = ACTIONS(2049),
    [sym_duet_dev_type_light] = ACTIONS(2049),
    [sym_duet_dev_type_monitor] = ACTIONS(2049),
    [sym_duet_dev_type_motor] = ACTIONS(2049),
    [sym_duet_dev_type_multi_window] = ACTIONS(2049),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2049),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2049),
    [sym_duet_dev_type_receiver] = ACTIONS(2049),
    [sym_duet_dev_type_security_system] = ACTIONS(2049),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2049),
    [sym_duet_dev_type_settop_box] = ACTIONS(2049),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2049),
    [sym_duet_dev_type_switcher] = ACTIONS(2049),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2049),
    [sym_duet_dev_type_tv] = ACTIONS(2049),
    [sym_duet_dev_type_utility] = ACTIONS(2049),
    [sym_duet_dev_type_vcr] = ACTIONS(2049),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2049),
    [sym_duet_dev_type_video_processor] = ACTIONS(2049),
    [sym_duet_dev_type_video_projector] = ACTIONS(2049),
    [sym_duet_dev_type_video_wall] = ACTIONS(2049),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2049),
    [sym_duet_dev_type_weather] = ACTIONS(2049),
    [sym_duet_dev_not_polled] = ACTIONS(2049),
    [sym_duet_dev_polled] = ACTIONS(2049),
    [sym_clkmgr_mode_network] = ACTIONS(2049),
    [sym_clkmgr_mode_standalone] = ACTIONS(2049),
    [sym_normal_standby] = ACTIONS(2049),
    [sym_normal_wake] = ACTIONS(2049),
    [sym_ip_multicast_ttl_option] = ACTIONS(2049),
    [sym_ip_ttl_subnet] = ACTIONS(2049),
    [sym_ip_ttl_site] = ACTIONS(2049),
    [sym_ip_ttl_region] = ACTIONS(2049),
    [sym_ip_ttl_continent] = ACTIONS(2049),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2049),
    [sym_ip_nodelay_on] = ACTIONS(2049),
    [sym_ip_nodelay_off] = ACTIONS(2049),
    [sym_amx_error] = ACTIONS(2049),
    [sym_amx_warning] = ACTIONS(2049),
    [sym_amx_info] = ACTIONS(2049),
    [sym_amx_debug] = ACTIONS(2049),
    [sym_netlinx_login_success] = ACTIONS(2049),
    [sym_netlinx_login_fail] = ACTIONS(2049),
    [sym_netlinx_logout] = ACTIONS(2049),
    [sym_null_str] = ACTIONS(2049),
    [sym_smtp_address] = ACTIONS(2049),
    [sym_smtp_port_number] = ACTIONS(2049),
    [sym_smtp_username] = ACTIONS(2049),
    [sym_smtp_password] = ACTIONS(2049),
    [sym_smtp_require_tls] = ACTIONS(2049),
    [sym_smtp_from] = ACTIONS(2049),
    [sym_smtp_tls_true] = ACTIONS(2049),
    [sym_smtp_tls_false] = ACTIONS(2049),
    [sym_listview_on_row_select_event] = ACTIONS(2049),
    [sym_data_structure_unknown] = ACTIONS(2049),
    [sym_data_structure_datafeed] = ACTIONS(2049),
    [sym_data_structure_datarecordset] = ACTIONS(2049),
    [sym_data_structure_datarecord] = ACTIONS(2049),
    [sym_data_structure_datafield] = ACTIONS(2049),
    [sym_data_max_content_field_count] = ACTIONS(2049),
    [sym_data_max_metadata_field_count] = ACTIONS(2049),
    [sym_data_max_id_length] = ACTIONS(2049),
    [sym_data_max_name_length] = ACTIONS(2049),
    [sym_data_max_description_length] = ACTIONS(2049),
    [sym_data_max_type_length] = ACTIONS(2049),
    [sym_data_max_format_length] = ACTIONS(2049),
    [sym_data_max_source_length] = ACTIONS(2049),
    [sym_data_max_value_length] = ACTIONS(2049),
    [sym_data_max_label_length] = ACTIONS(2049),
    [sym_data_max_url_length] = ACTIONS(2049),
    [sym_max_last_login_info_length] = ACTIONS(2049),
    [sym_data_type_unknown] = ACTIONS(2049),
    [sym_data_type_string] = ACTIONS(2049),
    [sym_data_type_datetime] = ACTIONS(2049),
    [sym_data_type_date] = ACTIONS(2049),
    [sym_data_type_time] = ACTIONS(2049),
    [sym_data_type_image] = ACTIONS(2049),
    [sym_data_format_url] = ACTIONS(2049),
    [sym_data_format_phone] = ACTIONS(2049),
    [sym_data_format_email] = ACTIONS(2049),
    [sym_data_format_iso8601] = ACTIONS(2049),
    [sym_auditlog_priv] = ACTIONS(2049),
    [sym_configuration_priv] = ACTIONS(2049),
    [sym_deviceconfig_priv] = ACTIONS(2049),
    [sym_ftp_priv] = ACTIONS(2049),
    [sym_http_priv] = ACTIONS(2049),
    [sym_networkconfig_priv] = ACTIONS(2049),
    [sym_programport_priv] = ACTIONS(2049),
    [sym_remoteui_priv] = ACTIONS(2049),
    [sym_securitycontrol_priv] = ACTIONS(2049),
    [sym_softwaremanagement_priv] = ACTIONS(2049),
    [sym_terminal_priv] = ACTIONS(2049),
    [sym_tpadmin_priv] = ACTIONS(2049),
    [sym_user1_priv] = ACTIONS(2049),
    [sym_user2_priv] = ACTIONS(2049),
    [sym_user3_priv] = ACTIONS(2049),
    [sym_user4_priv] = ACTIONS(2049),
    [sym_usermanagement_priv] = ACTIONS(2049),
    [sym_valid_account] = ACTIONS(2049),
    [sym_err_name_invalid] = ACTIONS(2049),
    [sym_err_password_invalid] = ACTIONS(2049),
    [sym_err_invalid_account] = ACTIONS(2049),
    [sym_err_no_authorization] = ACTIONS(2049),
    [sym_err_invalid_parameter] = ACTIONS(2049),
    [sym_err_user_locked_out] = ACTIONS(2049),
    [sym_err_user_account_expired] = ACTIONS(2049),
    [sym_err_general_error] = ACTIONS(2049),
    [sym_err_authentication_error] = ACTIONS(2049),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2049),
    [sym_tls_validate_certificate] = ACTIONS(2049),
    [sym_tdata] = ACTIONS(2049),
    [sym_tchannel] = ACTIONS(2049),
    [sym_tlevel] = ACTIONS(2049),
    [sym_tbutton] = ACTIONS(2049),
    [sym_ttimeline] = ACTIONS(2049),
    [sym_tcustom] = ACTIONS(2049),
    [sym_url_struct] = ACTIONS(2049),
    [sym_dns_struct] = ACTIONS(2049),
    [sym_ip_address_struct] = ACTIONS(2049),
    [sym_dev_info_struct] = ACTIONS(2049),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2049),
    [sym_data_feed] = ACTIONS(2049),
    [sym_data_field] = ACTIONS(2049),
    [sym_data_record] = ACTIONS(2049),
    [sym_wc_data_feed] = ACTIONS(2049),
    [sym_wc_data_field] = ACTIONS(2049),
    [sym_wc_data_record] = ACTIONS(2049),
    [sym_last_login_info] = ACTIONS(2049),
    [sym____reserved___] = ACTIONS(2049),
    [sym_date] = ACTIONS(2049),
    [sym_day] = ACTIONS(2049),
    [sym_time] = ACTIONS(2049),
    [sym_ldate] = ACTIONS(2049),
    [sym_push_device] = ACTIONS(2049),
    [sym_push_channel] = ACTIONS(2049),
    [sym_push_devchan] = ACTIONS(2049),
    [sym_release_device] = ACTIONS(2049),
    [sym_release_channel] = ACTIONS(2049),
    [sym_release_devchan] = ACTIONS(2049),
    [sym_master_slot] = ACTIONS(2049),
    [sym_get_pulse_time] = ACTIONS(2049),
    [sym_get_timer] = ACTIONS(2049),
    [sym_master_sn] = ACTIONS(2049),
    [sym_system_number] = ACTIONS(2049),
    [sym_button] = ACTIONS(2049),
    [sym_dv_channel] = ACTIONS(2049),
    [sym_data] = ACTIONS(2049),
    [sym_level] = ACTIONS(2049),
    [sym_timeline] = ACTIONS(2049),
    [sym_channel] = ACTIONS(2049),
    [sym_custom] = ACTIONS(2049),
    [sym_atoi] = ACTIONS(2049),
    [sym_atol] = ACTIONS(2049),
    [sym_atof] = ACTIONS(2049),
    [sym_itoa] = ACTIONS(2049),
    [sym_itohex] = ACTIONS(2049),
    [sym_hextoi] = ACTIONS(2049),
    [sym_ftoa] = ACTIONS(2049),
    [sym_find_string] = ACTIONS(2049),
    [sym_left_string] = ACTIONS(2049),
    [sym_right_string] = ACTIONS(2049),
    [sym_mid_string] = ACTIONS(2049),
    [sym_remove_string] = ACTIONS(2049),
    [sym_get_buffer_string] = ACTIONS(2049),
    [sym_get_buffer_char] = ACTIONS(2049),
    [sym_get_multi_buffer_string] = ACTIONS(2049),
    [sym_length_string] = ACTIONS(2049),
    [sym_max_length_string] = ACTIONS(2049),
    [sym_set_length_string] = ACTIONS(2049),
    [sym_compare_string] = ACTIONS(2049),
    [sym_upper_string] = ACTIONS(2049),
    [sym_lower_string] = ACTIONS(2049),
    [sym_get_last] = ACTIONS(2049),
    [sym_date_to_day] = ACTIONS(2049),
    [sym_date_to_month] = ACTIONS(2049),
    [sym_date_to_year] = ACTIONS(2049),
    [sym_time_to_hour] = ACTIONS(2049),
    [sym_time_to_minute] = ACTIONS(2049),
    [sym_time_to_second] = ACTIONS(2049),
    [sym_day_of_week] = ACTIONS(2049),
    [sym_set_pulse_time] = ACTIONS(2049),
    [sym_set_timer] = ACTIONS(2049),
    [sym_file_open] = ACTIONS(2049),
    [sym_file_close] = ACTIONS(2049),
    [sym_file_read] = ACTIONS(2049),
    [sym_file_read_line] = ACTIONS(2049),
    [sym_file_write] = ACTIONS(2049),
    [sym_file_write_line] = ACTIONS(2049),
    [sym_file_copy] = ACTIONS(2049),
    [sym_file_delete] = ACTIONS(2049),
    [sym_file_rename] = ACTIONS(2049),
    [sym_file_dir] = ACTIONS(2049),
    [sym_file_getdir] = ACTIONS(2049),
    [sym_file_setdir] = ACTIONS(2049),
    [sym_file_create_dir] = ACTIONS(2049),
    [sym_file_remove_dir] = ACTIONS(2049),
    [sym_file_seek] = ACTIONS(2049),
    [sym_length_array] = ACTIONS(2049),
    [sym_max_length_array] = ACTIONS(2049),
    [sym_set_length_array] = ACTIONS(2049),
    [sym_ip_client_open] = ACTIONS(2049),
    [sym_ip_client_close] = ACTIONS(2049),
    [sym_ip_server_open] = ACTIONS(2049),
    [sym_ip_server_close] = ACTIONS(2049),
    [sym_ip_bound_client_open] = ACTIONS(2049),
    [sym_ip_mc_server_open] = ACTIONS(2049),
    [sym_ip_set_option] = ACTIONS(2049),
    [sym_ssh_client_open] = ACTIONS(2049),
    [sym_ssh_client_close] = ACTIONS(2049),
    [sym_tls_client_open] = ACTIONS(2049),
    [sym_tls_client_close] = ACTIONS(2049),
    [sym_do_push] = ACTIONS(2049),
    [sym_do_push_timed] = ACTIONS(2049),
    [sym_do_release] = ACTIONS(2049),
    [sym_device_id] = ACTIONS(2049),
    [sym_device_info] = ACTIONS(2049),
    [sym_device_id_string] = ACTIONS(2049),
    [sym_device_standby] = ACTIONS(2049),
    [sym_device_wake] = ACTIONS(2049),
    [sym_timeline_create] = ACTIONS(2049),
    [sym_timeline_kill] = ACTIONS(2049),
    [sym_timeline_reload] = ACTIONS(2049),
    [sym_timeline_pause] = ACTIONS(2049),
    [sym_timeline_restart] = ACTIONS(2049),
    [sym_timeline_set] = ACTIONS(2049),
    [sym_timeline_get] = ACTIONS(2049),
    [sym_timeline_active] = ACTIONS(2049),
    [sym_timeline_dynamic_id] = ACTIONS(2049),
    [sym_variable_to_string] = ACTIONS(2049),
    [sym_string_to_variable] = ACTIONS(2049),
    [sym_length_variable_to_string] = ACTIONS(2049),
    [sym_type_cast] = ACTIONS(2049),
    [sym_variable_to_xml] = ACTIONS(2049),
    [sym_xml_to_variable] = ACTIONS(2049),
    [sym_length_variable_to_xml] = ACTIONS(2049),
    [sym_redirect_string] = ACTIONS(2049),
    [sym_random_number] = ACTIONS(2049),
    [sym_get_unique_id] = ACTIONS(2049),
    [sym_get_system_number] = ACTIONS(2049),
    [sym_set_system_number] = ACTIONS(2049),
    [sym_get_serial_number] = ACTIONS(2049),
    [sym_get_master_build] = ACTIONS(2049),
    [sym_set_validation_code] = ACTIONS(2049),
    [sym_reboot] = ACTIONS(2049),
    [sym_load_duet_module] = ACTIONS(2049),
    [sym_unload_duet_module] = ACTIONS(2049),
    [sym_duet_mem_size_set] = ACTIONS(2049),
    [sym_duet_mem_size_get] = ACTIONS(2049),
    [sym_dynamic_application_device] = ACTIONS(2049),
    [sym_dynamic_polled_port] = ACTIONS(2049),
    [sym_static_port_binding] = ACTIONS(2049),
    [sym_static_ip_binding] = ACTIONS(2049),
    [sym_get_dns_list] = ACTIONS(2049),
    [sym_set_dns_list] = ACTIONS(2049),
    [sym_get_ip_address] = ACTIONS(2049),
    [sym_set_ip_address] = ACTIONS(2049),
    [sym_get_url_list] = ACTIONS(2049),
    [sym_add_url_entry] = ACTIONS(2049),
    [sym_delete_url_entry] = ACTIONS(2049),
    [sym_abs_value] = ACTIONS(2049),
    [sym_max_value] = ACTIONS(2049),
    [sym_min_value] = ACTIONS(2049),
    [sym_exp_value] = ACTIONS(2049),
    [sym_log_value] = ACTIONS(2049),
    [sym_log10_value] = ACTIONS(2049),
    [sym_power_value] = ACTIONS(2049),
    [sym_sqrt_value] = ACTIONS(2049),
    [sym_raw_be] = ACTIONS(2049),
    [sym_raw_le] = ACTIONS(2049),
    [sym_internal_queue_size_set] = ACTIONS(2049),
    [sym_internal_queue_size_get] = ACTIONS(2049),
    [sym_internal_threshold_set] = ACTIONS(2049),
    [sym_internal_threshold_get] = ACTIONS(2049),
    [sym_rebuilt_event] = ACTIONS(2049),
    [sym_do_custom_event] = ACTIONS(2049),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2049),
    [sym_clkmgr_set_clk_source] = ACTIONS(2049),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2049),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2049),
    [sym_clkmgr_get_timezone] = ACTIONS(2049),
    [sym_clkmgr_set_timezone] = ACTIONS(2049),
    [sym_clkmgr_get_resync_period] = ACTIONS(2049),
    [sym_clkmgr_set_resync_period] = ACTIONS(2049),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2049),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2049),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2049),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2049),
    [sym_clkmgr_get_timeservers] = ACTIONS(2049),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2049),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2049),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2049),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2049),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2049),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2049),
    [sym_set_log_level] = ACTIONS(2049),
    [sym_get_log_level] = ACTIONS(2049),
    [sym_amx_log] = ACTIONS(2049),
    [sym_smtp_server_config_set] = ACTIONS(2049),
    [sym_smtp_server_config_get] = ACTIONS(2049),
    [sym_smtp_send] = ACTIONS(2049),
    [sym_validate_netlinx_account] = ACTIONS(2049),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2049),
    [sym_authenticate_certificate] = ACTIONS(2049),
    [sym_audit_netlinx_session_event] = ACTIONS(2049),
    [sym_audit_netlinx_generic_event] = ACTIONS(2049),
    [sym_get_available_flash_disk_space] = ACTIONS(2049),
    [sym_get_max_flash_disk_space] = ACTIONS(2049),
    [sym_data_create_feed] = ACTIONS(2049),
    [sym_data_delete_feed] = ACTIONS(2049),
    [sym_data_publish_feed] = ACTIONS(2049),
    [sym_data_get_published_feed] = ACTIONS(2049),
    [sym_data_add_record] = ACTIONS(2049),
    [sym_data_get_event_record] = ACTIONS(2049),
    [sym__wc_data_create_feed] = ACTIONS(2049),
    [sym__wc_data_add_record] = ACTIONS(2049),
    [sym__wc_data_get_event_record] = ACTIONS(2049),
    [sym_format] = ACTIONS(2049),
    [sym_astro_clock] = ACTIONS(2049),
    [sym_set_outdoor_temperature] = ACTIONS(2049),
    [sym_set_virtual_level_count] = ACTIONS(2049),
    [sym_set_virtual_channel_count] = ACTIONS(2049),
    [sym_set_virtual_port_count] = ACTIONS(2049),
    [sym__wc_encode_int] = ACTIONS(2049),
    [sym__wc_decode_int] = ACTIONS(2049),
    [sym__wc_int] = ACTIONS(2049),
    [sym__wc_to_ch_int] = ACTIONS(2049),
    [sym__ch_to_wc_int] = ACTIONS(2049),
    [sym__wc_find_string_int] = ACTIONS(2049),
    [sym__wc_left_string_int] = ACTIONS(2049),
    [sym__wc_lower_string_int] = ACTIONS(2049),
    [sym__wc_mid_string_int] = ACTIONS(2049),
    [sym__wc_remove_string_int] = ACTIONS(2049),
    [sym__wc_right_string_int] = ACTIONS(2049),
    [sym__wc_upper_string_int] = ACTIONS(2049),
    [sym__wc_compare_string_int] = ACTIONS(2049),
    [sym__wc_get_buffer_char_int] = ACTIONS(2049),
    [sym__wc_get_buffer_string_int] = ACTIONS(2049),
    [sym__wc_concat_string_int] = ACTIONS(2049),
    [sym__wc_file_open_int] = ACTIONS(2049),
    [sym__wc_file_close_int] = ACTIONS(2049),
    [sym__wc_file_read_int] = ACTIONS(2049),
    [sym__wc_file_read_line_int] = ACTIONS(2049),
    [sym__wc_file_write_int] = ACTIONS(2049),
    [sym__wc_file_write_line_int] = ACTIONS(2049),
    [aux_sym_preproc_include_token1] = ACTIONS(2049),
    [aux_sym_preproc_define_token1] = ACTIONS(2051),
    [aux_sym_preproc_warn_token1] = ACTIONS(2051),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2051),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2051),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2051),
    [anon_sym_LBRACK] = ACTIONS(2051),
    [anon_sym_LBRACE] = ACTIONS(2051),
    [anon_sym_LPAREN] = ACTIONS(2049),
    [anon_sym_BANG] = ACTIONS(2051),
    [anon_sym_TILDE] = ACTIONS(2051),
    [anon_sym_DASH] = ACTIONS(2049),
    [anon_sym_PLUS] = ACTIONS(2049),
    [anon_sym_DASH_DASH] = ACTIONS(2051),
    [anon_sym_PLUS_PLUS] = ACTIONS(2051),
    [anon_sym_DQUOTE] = ACTIONS(2051),
    [anon_sym_SQUOTE] = ACTIONS(2051),
    [sym_number_literal] = ACTIONS(2051),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(445)] = {
    [ts_builtin_sym_end] = ACTIONS(1935),
    [sym_identifier] = ACTIONS(1933),
    [sym_program_name_keyword] = ACTIONS(1933),
    [sym_module_name_keyword] = ACTIONS(1933),
    [sym_define_device_keyword] = ACTIONS(1933),
    [sym_define_combine_keyword] = ACTIONS(1933),
    [sym_define_constant_keyword] = ACTIONS(1933),
    [sym_define_type_keyword] = ACTIONS(1933),
    [sym_define_variable_keyword] = ACTIONS(1933),
    [sym_define_system_variable_keyword] = ACTIONS(1933),
    [sym_define_connect_level_keyword] = ACTIONS(1933),
    [sym_define_latching_keyword] = ACTIONS(1933),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1933),
    [sym_define_toggling_keyword] = ACTIONS(1933),
    [sym_define_call_keyword] = ACTIONS(1933),
    [sym_define_function_keyword] = ACTIONS(1933),
    [sym_define_library_function_keyword] = ACTIONS(1933),
    [sym_define_start_keyword] = ACTIONS(1933),
    [sym_define_event_keyword] = ACTIONS(1933),
    [sym_define_program_keyword] = ACTIONS(1933),
    [sym_define_module_keyword] = ACTIONS(1933),
    [sym_char_keyword] = ACTIONS(1933),
    [sym_widechar_keyword] = ACTIONS(1933),
    [sym_integer_keyword] = ACTIONS(1933),
    [sym_sinteger_keyword] = ACTIONS(1933),
    [sym_long_keyword] = ACTIONS(1933),
    [sym_slong_keyword] = ACTIONS(1933),
    [sym_float_keyword] = ACTIONS(1933),
    [sym_double_keyword] = ACTIONS(1933),
    [sym_variant_keyword] = ACTIONS(1933),
    [sym_variantarray_keyword] = ACTIONS(1933),
    [sym_local_var_keyword] = ACTIONS(1933),
    [sym_stack_var_keyword] = ACTIONS(1933),
    [sym_constant_keyword] = ACTIONS(1933),
    [sym_volatile_keyword] = ACTIONS(1933),
    [sym_non_volatile_keyword] = ACTIONS(1933),
    [sym_persistent_keyword] = ACTIONS(1933),
    [sym_dev_keyword] = ACTIONS(1933),
    [sym_devlev_keyword] = ACTIONS(1933),
    [sym_devchan_keyword] = ACTIONS(1933),
    [sym_if_keyword] = ACTIONS(1933),
    [sym_switch_keyword] = ACTIONS(1933),
    [sym_case_keyword] = ACTIONS(1933),
    [sym_default_keyword] = ACTIONS(1933),
    [sym_while_keyword] = ACTIONS(1933),
    [sym_for_keyword] = ACTIONS(1933),
    [sym_break_keyword] = ACTIONS(1933),
    [sym_continue_keyword] = ACTIONS(1933),
    [sym_return_keyword] = ACTIONS(1933),
    [sym_select_keyword] = ACTIONS(1933),
    [sym_active_keyword] = ACTIONS(1933),
    [sym_struct_keyword] = ACTIONS(1933),
    [sym_structure_keyword] = ACTIONS(1933),
    [sym_button_event_keyword] = ACTIONS(1933),
    [sym_channel_event_keyword] = ACTIONS(1933),
    [sym_level_event_keyword] = ACTIONS(1933),
    [sym_data_event_keyword] = ACTIONS(1933),
    [sym_timeline_event_keyword] = ACTIONS(1933),
    [sym_custom_event_keyword] = ACTIONS(1933),
    [sym_push_keyword] = ACTIONS(1933),
    [sym_release_keyword] = ACTIONS(1933),
    [aux_sym_on_keyword_token1] = ACTIONS(1933),
    [aux_sym_off_keyword_token1] = ACTIONS(1933),
    [sym_send_level_keyword] = ACTIONS(1933),
    [sym_send_string_keyword] = ACTIONS(1933),
    [sym_send_command_keyword] = ACTIONS(1933),
    [sym_clear_buffer_keyword] = ACTIONS(1933),
    [sym_create_buffer_keyword] = ACTIONS(1933),
    [sym_create_multi_buffer_keyword] = ACTIONS(1933),
    [sym_call_keyword] = ACTIONS(1933),
    [sym_system_call_keyword] = ACTIONS(1933),
    [sym_devchan_to_keyword] = ACTIONS(1933),
    [sym_devchan_min_to_keyword] = ACTIONS(1933),
    [sym_devchan_total_off_keyword] = ACTIONS(1933),
    [sym_devchan_pulse_keyword] = ACTIONS(1933),
    [sym_wait_keyword] = ACTIONS(1933),
    [sym_cancel_wait_keyword] = ACTIONS(1933),
    [sym_wait_until_keyword] = ACTIONS(1933),
    [sym_cancel_wait_until_keyword] = ACTIONS(1933),
    [sym_cancel_all_wait_keyword] = ACTIONS(1933),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1933),
    [sym_not] = ACTIONS(1933),
    [sym_bnot] = ACTIONS(1933),
    [sym___file__] = ACTIONS(1933),
    [sym___date__] = ACTIONS(1933),
    [sym___ldate__] = ACTIONS(1933),
    [sym___time__] = ACTIONS(1933),
    [sym___line__] = ACTIONS(1933),
    [sym___name__] = ACTIONS(1933),
    [sym___netlinx__] = ACTIONS(1933),
    [sym_netlinx_axi_version] = ACTIONS(1933),
    [sym_url_flg_tcp] = ACTIONS(1933),
    [sym_url_flg_accinfopresent] = ACTIONS(1933),
    [sym_url_flg_temp] = ACTIONS(1933),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1933),
    [sym_url_flg_stat_mask] = ACTIONS(1933),
    [sym_url_flg_stat_lookup] = ACTIONS(1933),
    [sym_url_flg_stat_connecting] = ACTIONS(1933),
    [sym_url_flg_stat_waiting] = ACTIONS(1933),
    [sym_url_flg_stat_connected] = ACTIONS(1933),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1933),
    [sym_first_virtual_device] = ACTIONS(1933),
    [sym_first_local_port] = ACTIONS(1933),
    [sym_dynamic_virtual_device] = ACTIONS(1933),
    [sym_timeline_once] = ACTIONS(1933),
    [sym_timeline_repeat] = ACTIONS(1933),
    [sym_timeline_absolute] = ACTIONS(1933),
    [sym_timeline_relative] = ACTIONS(1933),
    [sym_do_push_timed_infinite] = ACTIONS(1933),
    [sym_source_type_no_address] = ACTIONS(1933),
    [sym_source_type_neuron_id] = ACTIONS(1933),
    [sym_source_type_ip_address] = ACTIONS(1933),
    [sym_source_type_axlink] = ACTIONS(1933),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1933),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1933),
    [sym_source_type_ip_socket_address] = ACTIONS(1933),
    [sym_source_type_rs232] = ACTIONS(1933),
    [sym_source_type_internal] = ACTIONS(1933),
    [sym_source_type_mac_address] = ACTIONS(1933),
    [sym_source_type_ipv4_port] = ACTIONS(1933),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1933),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1933),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1933),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1933),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1933),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1933),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1933),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1933),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1933),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1933),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1933),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1933),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1933),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1933),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1933),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1933),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1933),
    [sym_true] = ACTIONS(1933),
    [sym_false] = ACTIONS(1933),
    [sym_file_read_only] = ACTIONS(1933),
    [sym_file_rw_new] = ACTIONS(1933),
    [sym_file_rw_append] = ACTIONS(1933),
    [sym_ip_tcp] = ACTIONS(1933),
    [sym_ip_udp] = ACTIONS(1933),
    [sym_ip_udp_2way] = ACTIONS(1933),
    [sym_xml_encode_types] = ACTIONS(1933),
    [sym_xml_encode_char_as_list] = ACTIONS(1933),
    [sym_xml_encode_le] = ACTIONS(1933),
    [sym_xml_decode_types] = ACTIONS(1933),
    [sym_xml_decode_no_preserve] = ACTIONS(1933),
    [sym_duet_dev_type_display_device] = ACTIONS(1933),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1933),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1933),
    [sym_duet_dev_type_io_device] = ACTIONS(1933),
    [sym_duet_dev_type_relay_device] = ACTIONS(1933),
    [sym_duet_dev_type_ups] = ACTIONS(1933),
    [sym_duet_dev_type_amplifier] = ACTIONS(1933),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1933),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1933),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1933),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1933),
    [sym_duet_dev_type_camera] = ACTIONS(1933),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1933),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1933),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1933),
    [sym_duet_dev_type_dss] = ACTIONS(1933),
    [sym_duet_dev_type_dvr] = ACTIONS(1933),
    [sym_duet_dev_type_disc_device] = ACTIONS(1933),
    [sym_duet_dev_type_document_camera] = ACTIONS(1933),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1933),
    [sym_duet_dev_type_hvac] = ACTIONS(1933),
    [sym_duet_dev_type_keypad] = ACTIONS(1933),
    [sym_duet_dev_type_light] = ACTIONS(1933),
    [sym_duet_dev_type_monitor] = ACTIONS(1933),
    [sym_duet_dev_type_motor] = ACTIONS(1933),
    [sym_duet_dev_type_multi_window] = ACTIONS(1933),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1933),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1933),
    [sym_duet_dev_type_receiver] = ACTIONS(1933),
    [sym_duet_dev_type_security_system] = ACTIONS(1933),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1933),
    [sym_duet_dev_type_settop_box] = ACTIONS(1933),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1933),
    [sym_duet_dev_type_switcher] = ACTIONS(1933),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1933),
    [sym_duet_dev_type_tv] = ACTIONS(1933),
    [sym_duet_dev_type_utility] = ACTIONS(1933),
    [sym_duet_dev_type_vcr] = ACTIONS(1933),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1933),
    [sym_duet_dev_type_video_processor] = ACTIONS(1933),
    [sym_duet_dev_type_video_projector] = ACTIONS(1933),
    [sym_duet_dev_type_video_wall] = ACTIONS(1933),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1933),
    [sym_duet_dev_type_weather] = ACTIONS(1933),
    [sym_duet_dev_not_polled] = ACTIONS(1933),
    [sym_duet_dev_polled] = ACTIONS(1933),
    [sym_clkmgr_mode_network] = ACTIONS(1933),
    [sym_clkmgr_mode_standalone] = ACTIONS(1933),
    [sym_normal_standby] = ACTIONS(1933),
    [sym_normal_wake] = ACTIONS(1933),
    [sym_ip_multicast_ttl_option] = ACTIONS(1933),
    [sym_ip_ttl_subnet] = ACTIONS(1933),
    [sym_ip_ttl_site] = ACTIONS(1933),
    [sym_ip_ttl_region] = ACTIONS(1933),
    [sym_ip_ttl_continent] = ACTIONS(1933),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1933),
    [sym_ip_nodelay_on] = ACTIONS(1933),
    [sym_ip_nodelay_off] = ACTIONS(1933),
    [sym_amx_error] = ACTIONS(1933),
    [sym_amx_warning] = ACTIONS(1933),
    [sym_amx_info] = ACTIONS(1933),
    [sym_amx_debug] = ACTIONS(1933),
    [sym_netlinx_login_success] = ACTIONS(1933),
    [sym_netlinx_login_fail] = ACTIONS(1933),
    [sym_netlinx_logout] = ACTIONS(1933),
    [sym_null_str] = ACTIONS(1933),
    [sym_smtp_address] = ACTIONS(1933),
    [sym_smtp_port_number] = ACTIONS(1933),
    [sym_smtp_username] = ACTIONS(1933),
    [sym_smtp_password] = ACTIONS(1933),
    [sym_smtp_require_tls] = ACTIONS(1933),
    [sym_smtp_from] = ACTIONS(1933),
    [sym_smtp_tls_true] = ACTIONS(1933),
    [sym_smtp_tls_false] = ACTIONS(1933),
    [sym_listview_on_row_select_event] = ACTIONS(1933),
    [sym_data_structure_unknown] = ACTIONS(1933),
    [sym_data_structure_datafeed] = ACTIONS(1933),
    [sym_data_structure_datarecordset] = ACTIONS(1933),
    [sym_data_structure_datarecord] = ACTIONS(1933),
    [sym_data_structure_datafield] = ACTIONS(1933),
    [sym_data_max_content_field_count] = ACTIONS(1933),
    [sym_data_max_metadata_field_count] = ACTIONS(1933),
    [sym_data_max_id_length] = ACTIONS(1933),
    [sym_data_max_name_length] = ACTIONS(1933),
    [sym_data_max_description_length] = ACTIONS(1933),
    [sym_data_max_type_length] = ACTIONS(1933),
    [sym_data_max_format_length] = ACTIONS(1933),
    [sym_data_max_source_length] = ACTIONS(1933),
    [sym_data_max_value_length] = ACTIONS(1933),
    [sym_data_max_label_length] = ACTIONS(1933),
    [sym_data_max_url_length] = ACTIONS(1933),
    [sym_max_last_login_info_length] = ACTIONS(1933),
    [sym_data_type_unknown] = ACTIONS(1933),
    [sym_data_type_string] = ACTIONS(1933),
    [sym_data_type_datetime] = ACTIONS(1933),
    [sym_data_type_date] = ACTIONS(1933),
    [sym_data_type_time] = ACTIONS(1933),
    [sym_data_type_image] = ACTIONS(1933),
    [sym_data_format_url] = ACTIONS(1933),
    [sym_data_format_phone] = ACTIONS(1933),
    [sym_data_format_email] = ACTIONS(1933),
    [sym_data_format_iso8601] = ACTIONS(1933),
    [sym_auditlog_priv] = ACTIONS(1933),
    [sym_configuration_priv] = ACTIONS(1933),
    [sym_deviceconfig_priv] = ACTIONS(1933),
    [sym_ftp_priv] = ACTIONS(1933),
    [sym_http_priv] = ACTIONS(1933),
    [sym_networkconfig_priv] = ACTIONS(1933),
    [sym_programport_priv] = ACTIONS(1933),
    [sym_remoteui_priv] = ACTIONS(1933),
    [sym_securitycontrol_priv] = ACTIONS(1933),
    [sym_softwaremanagement_priv] = ACTIONS(1933),
    [sym_terminal_priv] = ACTIONS(1933),
    [sym_tpadmin_priv] = ACTIONS(1933),
    [sym_user1_priv] = ACTIONS(1933),
    [sym_user2_priv] = ACTIONS(1933),
    [sym_user3_priv] = ACTIONS(1933),
    [sym_user4_priv] = ACTIONS(1933),
    [sym_usermanagement_priv] = ACTIONS(1933),
    [sym_valid_account] = ACTIONS(1933),
    [sym_err_name_invalid] = ACTIONS(1933),
    [sym_err_password_invalid] = ACTIONS(1933),
    [sym_err_invalid_account] = ACTIONS(1933),
    [sym_err_no_authorization] = ACTIONS(1933),
    [sym_err_invalid_parameter] = ACTIONS(1933),
    [sym_err_user_locked_out] = ACTIONS(1933),
    [sym_err_user_account_expired] = ACTIONS(1933),
    [sym_err_general_error] = ACTIONS(1933),
    [sym_err_authentication_error] = ACTIONS(1933),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1933),
    [sym_tls_validate_certificate] = ACTIONS(1933),
    [sym_tdata] = ACTIONS(1933),
    [sym_tchannel] = ACTIONS(1933),
    [sym_tlevel] = ACTIONS(1933),
    [sym_tbutton] = ACTIONS(1933),
    [sym_ttimeline] = ACTIONS(1933),
    [sym_tcustom] = ACTIONS(1933),
    [sym_url_struct] = ACTIONS(1933),
    [sym_dns_struct] = ACTIONS(1933),
    [sym_ip_address_struct] = ACTIONS(1933),
    [sym_dev_info_struct] = ACTIONS(1933),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1933),
    [sym_data_feed] = ACTIONS(1933),
    [sym_data_field] = ACTIONS(1933),
    [sym_data_record] = ACTIONS(1933),
    [sym_wc_data_feed] = ACTIONS(1933),
    [sym_wc_data_field] = ACTIONS(1933),
    [sym_wc_data_record] = ACTIONS(1933),
    [sym_last_login_info] = ACTIONS(1933),
    [sym____reserved___] = ACTIONS(1933),
    [sym_date] = ACTIONS(1933),
    [sym_day] = ACTIONS(1933),
    [sym_time] = ACTIONS(1933),
    [sym_ldate] = ACTIONS(1933),
    [sym_push_device] = ACTIONS(1933),
    [sym_push_channel] = ACTIONS(1933),
    [sym_push_devchan] = ACTIONS(1933),
    [sym_release_device] = ACTIONS(1933),
    [sym_release_channel] = ACTIONS(1933),
    [sym_release_devchan] = ACTIONS(1933),
    [sym_master_slot] = ACTIONS(1933),
    [sym_get_pulse_time] = ACTIONS(1933),
    [sym_get_timer] = ACTIONS(1933),
    [sym_master_sn] = ACTIONS(1933),
    [sym_system_number] = ACTIONS(1933),
    [sym_button] = ACTIONS(1933),
    [sym_dv_channel] = ACTIONS(1933),
    [sym_data] = ACTIONS(1933),
    [sym_level] = ACTIONS(1933),
    [sym_timeline] = ACTIONS(1933),
    [sym_channel] = ACTIONS(1933),
    [sym_custom] = ACTIONS(1933),
    [sym_atoi] = ACTIONS(1933),
    [sym_atol] = ACTIONS(1933),
    [sym_atof] = ACTIONS(1933),
    [sym_itoa] = ACTIONS(1933),
    [sym_itohex] = ACTIONS(1933),
    [sym_hextoi] = ACTIONS(1933),
    [sym_ftoa] = ACTIONS(1933),
    [sym_find_string] = ACTIONS(1933),
    [sym_left_string] = ACTIONS(1933),
    [sym_right_string] = ACTIONS(1933),
    [sym_mid_string] = ACTIONS(1933),
    [sym_remove_string] = ACTIONS(1933),
    [sym_get_buffer_string] = ACTIONS(1933),
    [sym_get_buffer_char] = ACTIONS(1933),
    [sym_get_multi_buffer_string] = ACTIONS(1933),
    [sym_length_string] = ACTIONS(1933),
    [sym_max_length_string] = ACTIONS(1933),
    [sym_set_length_string] = ACTIONS(1933),
    [sym_compare_string] = ACTIONS(1933),
    [sym_upper_string] = ACTIONS(1933),
    [sym_lower_string] = ACTIONS(1933),
    [sym_get_last] = ACTIONS(1933),
    [sym_date_to_day] = ACTIONS(1933),
    [sym_date_to_month] = ACTIONS(1933),
    [sym_date_to_year] = ACTIONS(1933),
    [sym_time_to_hour] = ACTIONS(1933),
    [sym_time_to_minute] = ACTIONS(1933),
    [sym_time_to_second] = ACTIONS(1933),
    [sym_day_of_week] = ACTIONS(1933),
    [sym_set_pulse_time] = ACTIONS(1933),
    [sym_set_timer] = ACTIONS(1933),
    [sym_file_open] = ACTIONS(1933),
    [sym_file_close] = ACTIONS(1933),
    [sym_file_read] = ACTIONS(1933),
    [sym_file_read_line] = ACTIONS(1933),
    [sym_file_write] = ACTIONS(1933),
    [sym_file_write_line] = ACTIONS(1933),
    [sym_file_copy] = ACTIONS(1933),
    [sym_file_delete] = ACTIONS(1933),
    [sym_file_rename] = ACTIONS(1933),
    [sym_file_dir] = ACTIONS(1933),
    [sym_file_getdir] = ACTIONS(1933),
    [sym_file_setdir] = ACTIONS(1933),
    [sym_file_create_dir] = ACTIONS(1933),
    [sym_file_remove_dir] = ACTIONS(1933),
    [sym_file_seek] = ACTIONS(1933),
    [sym_length_array] = ACTIONS(1933),
    [sym_max_length_array] = ACTIONS(1933),
    [sym_set_length_array] = ACTIONS(1933),
    [sym_ip_client_open] = ACTIONS(1933),
    [sym_ip_client_close] = ACTIONS(1933),
    [sym_ip_server_open] = ACTIONS(1933),
    [sym_ip_server_close] = ACTIONS(1933),
    [sym_ip_bound_client_open] = ACTIONS(1933),
    [sym_ip_mc_server_open] = ACTIONS(1933),
    [sym_ip_set_option] = ACTIONS(1933),
    [sym_ssh_client_open] = ACTIONS(1933),
    [sym_ssh_client_close] = ACTIONS(1933),
    [sym_tls_client_open] = ACTIONS(1933),
    [sym_tls_client_close] = ACTIONS(1933),
    [sym_do_push] = ACTIONS(1933),
    [sym_do_push_timed] = ACTIONS(1933),
    [sym_do_release] = ACTIONS(1933),
    [sym_device_id] = ACTIONS(1933),
    [sym_device_info] = ACTIONS(1933),
    [sym_device_id_string] = ACTIONS(1933),
    [sym_device_standby] = ACTIONS(1933),
    [sym_device_wake] = ACTIONS(1933),
    [sym_timeline_create] = ACTIONS(1933),
    [sym_timeline_kill] = ACTIONS(1933),
    [sym_timeline_reload] = ACTIONS(1933),
    [sym_timeline_pause] = ACTIONS(1933),
    [sym_timeline_restart] = ACTIONS(1933),
    [sym_timeline_set] = ACTIONS(1933),
    [sym_timeline_get] = ACTIONS(1933),
    [sym_timeline_active] = ACTIONS(1933),
    [sym_timeline_dynamic_id] = ACTIONS(1933),
    [sym_variable_to_string] = ACTIONS(1933),
    [sym_string_to_variable] = ACTIONS(1933),
    [sym_length_variable_to_string] = ACTIONS(1933),
    [sym_type_cast] = ACTIONS(1933),
    [sym_variable_to_xml] = ACTIONS(1933),
    [sym_xml_to_variable] = ACTIONS(1933),
    [sym_length_variable_to_xml] = ACTIONS(1933),
    [sym_redirect_string] = ACTIONS(1933),
    [sym_random_number] = ACTIONS(1933),
    [sym_get_unique_id] = ACTIONS(1933),
    [sym_get_system_number] = ACTIONS(1933),
    [sym_set_system_number] = ACTIONS(1933),
    [sym_get_serial_number] = ACTIONS(1933),
    [sym_get_master_build] = ACTIONS(1933),
    [sym_set_validation_code] = ACTIONS(1933),
    [sym_reboot] = ACTIONS(1933),
    [sym_load_duet_module] = ACTIONS(1933),
    [sym_unload_duet_module] = ACTIONS(1933),
    [sym_duet_mem_size_set] = ACTIONS(1933),
    [sym_duet_mem_size_get] = ACTIONS(1933),
    [sym_dynamic_application_device] = ACTIONS(1933),
    [sym_dynamic_polled_port] = ACTIONS(1933),
    [sym_static_port_binding] = ACTIONS(1933),
    [sym_static_ip_binding] = ACTIONS(1933),
    [sym_get_dns_list] = ACTIONS(1933),
    [sym_set_dns_list] = ACTIONS(1933),
    [sym_get_ip_address] = ACTIONS(1933),
    [sym_set_ip_address] = ACTIONS(1933),
    [sym_get_url_list] = ACTIONS(1933),
    [sym_add_url_entry] = ACTIONS(1933),
    [sym_delete_url_entry] = ACTIONS(1933),
    [sym_abs_value] = ACTIONS(1933),
    [sym_max_value] = ACTIONS(1933),
    [sym_min_value] = ACTIONS(1933),
    [sym_exp_value] = ACTIONS(1933),
    [sym_log_value] = ACTIONS(1933),
    [sym_log10_value] = ACTIONS(1933),
    [sym_power_value] = ACTIONS(1933),
    [sym_sqrt_value] = ACTIONS(1933),
    [sym_raw_be] = ACTIONS(1933),
    [sym_raw_le] = ACTIONS(1933),
    [sym_internal_queue_size_set] = ACTIONS(1933),
    [sym_internal_queue_size_get] = ACTIONS(1933),
    [sym_internal_threshold_set] = ACTIONS(1933),
    [sym_internal_threshold_get] = ACTIONS(1933),
    [sym_rebuilt_event] = ACTIONS(1933),
    [sym_do_custom_event] = ACTIONS(1933),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1933),
    [sym_clkmgr_set_clk_source] = ACTIONS(1933),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1933),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1933),
    [sym_clkmgr_get_timezone] = ACTIONS(1933),
    [sym_clkmgr_set_timezone] = ACTIONS(1933),
    [sym_clkmgr_get_resync_period] = ACTIONS(1933),
    [sym_clkmgr_set_resync_period] = ACTIONS(1933),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1933),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1933),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1933),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1933),
    [sym_clkmgr_get_timeservers] = ACTIONS(1933),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1933),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1933),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1933),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1933),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1933),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1933),
    [sym_set_log_level] = ACTIONS(1933),
    [sym_get_log_level] = ACTIONS(1933),
    [sym_amx_log] = ACTIONS(1933),
    [sym_smtp_server_config_set] = ACTIONS(1933),
    [sym_smtp_server_config_get] = ACTIONS(1933),
    [sym_smtp_send] = ACTIONS(1933),
    [sym_validate_netlinx_account] = ACTIONS(1933),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1933),
    [sym_authenticate_certificate] = ACTIONS(1933),
    [sym_audit_netlinx_session_event] = ACTIONS(1933),
    [sym_audit_netlinx_generic_event] = ACTIONS(1933),
    [sym_get_available_flash_disk_space] = ACTIONS(1933),
    [sym_get_max_flash_disk_space] = ACTIONS(1933),
    [sym_data_create_feed] = ACTIONS(1933),
    [sym_data_delete_feed] = ACTIONS(1933),
    [sym_data_publish_feed] = ACTIONS(1933),
    [sym_data_get_published_feed] = ACTIONS(1933),
    [sym_data_add_record] = ACTIONS(1933),
    [sym_data_get_event_record] = ACTIONS(1933),
    [sym__wc_data_create_feed] = ACTIONS(1933),
    [sym__wc_data_add_record] = ACTIONS(1933),
    [sym__wc_data_get_event_record] = ACTIONS(1933),
    [sym_format] = ACTIONS(1933),
    [sym_astro_clock] = ACTIONS(1933),
    [sym_set_outdoor_temperature] = ACTIONS(1933),
    [sym_set_virtual_level_count] = ACTIONS(1933),
    [sym_set_virtual_channel_count] = ACTIONS(1933),
    [sym_set_virtual_port_count] = ACTIONS(1933),
    [sym__wc_encode_int] = ACTIONS(1933),
    [sym__wc_decode_int] = ACTIONS(1933),
    [sym__wc_int] = ACTIONS(1933),
    [sym__wc_to_ch_int] = ACTIONS(1933),
    [sym__ch_to_wc_int] = ACTIONS(1933),
    [sym__wc_find_string_int] = ACTIONS(1933),
    [sym__wc_left_string_int] = ACTIONS(1933),
    [sym__wc_lower_string_int] = ACTIONS(1933),
    [sym__wc_mid_string_int] = ACTIONS(1933),
    [sym__wc_remove_string_int] = ACTIONS(1933),
    [sym__wc_right_string_int] = ACTIONS(1933),
    [sym__wc_upper_string_int] = ACTIONS(1933),
    [sym__wc_compare_string_int] = ACTIONS(1933),
    [sym__wc_get_buffer_char_int] = ACTIONS(1933),
    [sym__wc_get_buffer_string_int] = ACTIONS(1933),
    [sym__wc_concat_string_int] = ACTIONS(1933),
    [sym__wc_file_open_int] = ACTIONS(1933),
    [sym__wc_file_close_int] = ACTIONS(1933),
    [sym__wc_file_read_int] = ACTIONS(1933),
    [sym__wc_file_read_line_int] = ACTIONS(1933),
    [sym__wc_file_write_int] = ACTIONS(1933),
    [sym__wc_file_write_line_int] = ACTIONS(1933),
    [aux_sym_preproc_include_token1] = ACTIONS(1933),
    [aux_sym_preproc_define_token1] = ACTIONS(1935),
    [aux_sym_preproc_warn_token1] = ACTIONS(1935),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1935),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1935),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1935),
    [anon_sym_LBRACK] = ACTIONS(1935),
    [anon_sym_LBRACE] = ACTIONS(1935),
    [anon_sym_LPAREN] = ACTIONS(1933),
    [anon_sym_BANG] = ACTIONS(1935),
    [anon_sym_TILDE] = ACTIONS(1935),
    [anon_sym_DASH] = ACTIONS(1933),
    [anon_sym_PLUS] = ACTIONS(1933),
    [anon_sym_DASH_DASH] = ACTIONS(1935),
    [anon_sym_PLUS_PLUS] = ACTIONS(1935),
    [anon_sym_DQUOTE] = ACTIONS(1935),
    [anon_sym_SQUOTE] = ACTIONS(1935),
    [sym_number_literal] = ACTIONS(1935),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(446)] = {
    [ts_builtin_sym_end] = ACTIONS(1923),
    [sym_identifier] = ACTIONS(1921),
    [sym_program_name_keyword] = ACTIONS(1921),
    [sym_module_name_keyword] = ACTIONS(1921),
    [sym_define_device_keyword] = ACTIONS(1921),
    [sym_define_combine_keyword] = ACTIONS(1921),
    [sym_define_constant_keyword] = ACTIONS(1921),
    [sym_define_type_keyword] = ACTIONS(1921),
    [sym_define_variable_keyword] = ACTIONS(1921),
    [sym_define_system_variable_keyword] = ACTIONS(1921),
    [sym_define_connect_level_keyword] = ACTIONS(1921),
    [sym_define_latching_keyword] = ACTIONS(1921),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1921),
    [sym_define_toggling_keyword] = ACTIONS(1921),
    [sym_define_call_keyword] = ACTIONS(1921),
    [sym_define_function_keyword] = ACTIONS(1921),
    [sym_define_library_function_keyword] = ACTIONS(1921),
    [sym_define_start_keyword] = ACTIONS(1921),
    [sym_define_event_keyword] = ACTIONS(1921),
    [sym_define_program_keyword] = ACTIONS(1921),
    [sym_define_module_keyword] = ACTIONS(1921),
    [sym_char_keyword] = ACTIONS(1921),
    [sym_widechar_keyword] = ACTIONS(1921),
    [sym_integer_keyword] = ACTIONS(1921),
    [sym_sinteger_keyword] = ACTIONS(1921),
    [sym_long_keyword] = ACTIONS(1921),
    [sym_slong_keyword] = ACTIONS(1921),
    [sym_float_keyword] = ACTIONS(1921),
    [sym_double_keyword] = ACTIONS(1921),
    [sym_variant_keyword] = ACTIONS(1921),
    [sym_variantarray_keyword] = ACTIONS(1921),
    [sym_local_var_keyword] = ACTIONS(1921),
    [sym_stack_var_keyword] = ACTIONS(1921),
    [sym_constant_keyword] = ACTIONS(1921),
    [sym_volatile_keyword] = ACTIONS(1921),
    [sym_non_volatile_keyword] = ACTIONS(1921),
    [sym_persistent_keyword] = ACTIONS(1921),
    [sym_dev_keyword] = ACTIONS(1921),
    [sym_devlev_keyword] = ACTIONS(1921),
    [sym_devchan_keyword] = ACTIONS(1921),
    [sym_if_keyword] = ACTIONS(1921),
    [sym_switch_keyword] = ACTIONS(1921),
    [sym_case_keyword] = ACTIONS(1921),
    [sym_default_keyword] = ACTIONS(1921),
    [sym_while_keyword] = ACTIONS(1921),
    [sym_for_keyword] = ACTIONS(1921),
    [sym_break_keyword] = ACTIONS(1921),
    [sym_continue_keyword] = ACTIONS(1921),
    [sym_return_keyword] = ACTIONS(1921),
    [sym_select_keyword] = ACTIONS(1921),
    [sym_active_keyword] = ACTIONS(1921),
    [sym_struct_keyword] = ACTIONS(1921),
    [sym_structure_keyword] = ACTIONS(1921),
    [sym_button_event_keyword] = ACTIONS(1921),
    [sym_channel_event_keyword] = ACTIONS(1921),
    [sym_level_event_keyword] = ACTIONS(1921),
    [sym_data_event_keyword] = ACTIONS(1921),
    [sym_timeline_event_keyword] = ACTIONS(1921),
    [sym_custom_event_keyword] = ACTIONS(1921),
    [sym_push_keyword] = ACTIONS(1921),
    [sym_release_keyword] = ACTIONS(1921),
    [aux_sym_on_keyword_token1] = ACTIONS(1921),
    [aux_sym_off_keyword_token1] = ACTIONS(1921),
    [sym_send_level_keyword] = ACTIONS(1921),
    [sym_send_string_keyword] = ACTIONS(1921),
    [sym_send_command_keyword] = ACTIONS(1921),
    [sym_clear_buffer_keyword] = ACTIONS(1921),
    [sym_create_buffer_keyword] = ACTIONS(1921),
    [sym_create_multi_buffer_keyword] = ACTIONS(1921),
    [sym_call_keyword] = ACTIONS(1921),
    [sym_system_call_keyword] = ACTIONS(1921),
    [sym_devchan_to_keyword] = ACTIONS(1921),
    [sym_devchan_min_to_keyword] = ACTIONS(1921),
    [sym_devchan_total_off_keyword] = ACTIONS(1921),
    [sym_devchan_pulse_keyword] = ACTIONS(1921),
    [sym_wait_keyword] = ACTIONS(1921),
    [sym_cancel_wait_keyword] = ACTIONS(1921),
    [sym_wait_until_keyword] = ACTIONS(1921),
    [sym_cancel_wait_until_keyword] = ACTIONS(1921),
    [sym_cancel_all_wait_keyword] = ACTIONS(1921),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1921),
    [sym_not] = ACTIONS(1921),
    [sym_bnot] = ACTIONS(1921),
    [sym___file__] = ACTIONS(1921),
    [sym___date__] = ACTIONS(1921),
    [sym___ldate__] = ACTIONS(1921),
    [sym___time__] = ACTIONS(1921),
    [sym___line__] = ACTIONS(1921),
    [sym___name__] = ACTIONS(1921),
    [sym___netlinx__] = ACTIONS(1921),
    [sym_netlinx_axi_version] = ACTIONS(1921),
    [sym_url_flg_tcp] = ACTIONS(1921),
    [sym_url_flg_accinfopresent] = ACTIONS(1921),
    [sym_url_flg_temp] = ACTIONS(1921),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1921),
    [sym_url_flg_stat_mask] = ACTIONS(1921),
    [sym_url_flg_stat_lookup] = ACTIONS(1921),
    [sym_url_flg_stat_connecting] = ACTIONS(1921),
    [sym_url_flg_stat_waiting] = ACTIONS(1921),
    [sym_url_flg_stat_connected] = ACTIONS(1921),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1921),
    [sym_first_virtual_device] = ACTIONS(1921),
    [sym_first_local_port] = ACTIONS(1921),
    [sym_dynamic_virtual_device] = ACTIONS(1921),
    [sym_timeline_once] = ACTIONS(1921),
    [sym_timeline_repeat] = ACTIONS(1921),
    [sym_timeline_absolute] = ACTIONS(1921),
    [sym_timeline_relative] = ACTIONS(1921),
    [sym_do_push_timed_infinite] = ACTIONS(1921),
    [sym_source_type_no_address] = ACTIONS(1921),
    [sym_source_type_neuron_id] = ACTIONS(1921),
    [sym_source_type_ip_address] = ACTIONS(1921),
    [sym_source_type_axlink] = ACTIONS(1921),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1921),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1921),
    [sym_source_type_ip_socket_address] = ACTIONS(1921),
    [sym_source_type_rs232] = ACTIONS(1921),
    [sym_source_type_internal] = ACTIONS(1921),
    [sym_source_type_mac_address] = ACTIONS(1921),
    [sym_source_type_ipv4_port] = ACTIONS(1921),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1921),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1921),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1921),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1921),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1921),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1921),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1921),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1921),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1921),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1921),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1921),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1921),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1921),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1921),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1921),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1921),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1921),
    [sym_true] = ACTIONS(1921),
    [sym_false] = ACTIONS(1921),
    [sym_file_read_only] = ACTIONS(1921),
    [sym_file_rw_new] = ACTIONS(1921),
    [sym_file_rw_append] = ACTIONS(1921),
    [sym_ip_tcp] = ACTIONS(1921),
    [sym_ip_udp] = ACTIONS(1921),
    [sym_ip_udp_2way] = ACTIONS(1921),
    [sym_xml_encode_types] = ACTIONS(1921),
    [sym_xml_encode_char_as_list] = ACTIONS(1921),
    [sym_xml_encode_le] = ACTIONS(1921),
    [sym_xml_decode_types] = ACTIONS(1921),
    [sym_xml_decode_no_preserve] = ACTIONS(1921),
    [sym_duet_dev_type_display_device] = ACTIONS(1921),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1921),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1921),
    [sym_duet_dev_type_io_device] = ACTIONS(1921),
    [sym_duet_dev_type_relay_device] = ACTIONS(1921),
    [sym_duet_dev_type_ups] = ACTIONS(1921),
    [sym_duet_dev_type_amplifier] = ACTIONS(1921),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1921),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1921),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1921),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1921),
    [sym_duet_dev_type_camera] = ACTIONS(1921),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1921),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1921),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1921),
    [sym_duet_dev_type_dss] = ACTIONS(1921),
    [sym_duet_dev_type_dvr] = ACTIONS(1921),
    [sym_duet_dev_type_disc_device] = ACTIONS(1921),
    [sym_duet_dev_type_document_camera] = ACTIONS(1921),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1921),
    [sym_duet_dev_type_hvac] = ACTIONS(1921),
    [sym_duet_dev_type_keypad] = ACTIONS(1921),
    [sym_duet_dev_type_light] = ACTIONS(1921),
    [sym_duet_dev_type_monitor] = ACTIONS(1921),
    [sym_duet_dev_type_motor] = ACTIONS(1921),
    [sym_duet_dev_type_multi_window] = ACTIONS(1921),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1921),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1921),
    [sym_duet_dev_type_receiver] = ACTIONS(1921),
    [sym_duet_dev_type_security_system] = ACTIONS(1921),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1921),
    [sym_duet_dev_type_settop_box] = ACTIONS(1921),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1921),
    [sym_duet_dev_type_switcher] = ACTIONS(1921),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1921),
    [sym_duet_dev_type_tv] = ACTIONS(1921),
    [sym_duet_dev_type_utility] = ACTIONS(1921),
    [sym_duet_dev_type_vcr] = ACTIONS(1921),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1921),
    [sym_duet_dev_type_video_processor] = ACTIONS(1921),
    [sym_duet_dev_type_video_projector] = ACTIONS(1921),
    [sym_duet_dev_type_video_wall] = ACTIONS(1921),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1921),
    [sym_duet_dev_type_weather] = ACTIONS(1921),
    [sym_duet_dev_not_polled] = ACTIONS(1921),
    [sym_duet_dev_polled] = ACTIONS(1921),
    [sym_clkmgr_mode_network] = ACTIONS(1921),
    [sym_clkmgr_mode_standalone] = ACTIONS(1921),
    [sym_normal_standby] = ACTIONS(1921),
    [sym_normal_wake] = ACTIONS(1921),
    [sym_ip_multicast_ttl_option] = ACTIONS(1921),
    [sym_ip_ttl_subnet] = ACTIONS(1921),
    [sym_ip_ttl_site] = ACTIONS(1921),
    [sym_ip_ttl_region] = ACTIONS(1921),
    [sym_ip_ttl_continent] = ACTIONS(1921),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1921),
    [sym_ip_nodelay_on] = ACTIONS(1921),
    [sym_ip_nodelay_off] = ACTIONS(1921),
    [sym_amx_error] = ACTIONS(1921),
    [sym_amx_warning] = ACTIONS(1921),
    [sym_amx_info] = ACTIONS(1921),
    [sym_amx_debug] = ACTIONS(1921),
    [sym_netlinx_login_success] = ACTIONS(1921),
    [sym_netlinx_login_fail] = ACTIONS(1921),
    [sym_netlinx_logout] = ACTIONS(1921),
    [sym_null_str] = ACTIONS(1921),
    [sym_smtp_address] = ACTIONS(1921),
    [sym_smtp_port_number] = ACTIONS(1921),
    [sym_smtp_username] = ACTIONS(1921),
    [sym_smtp_password] = ACTIONS(1921),
    [sym_smtp_require_tls] = ACTIONS(1921),
    [sym_smtp_from] = ACTIONS(1921),
    [sym_smtp_tls_true] = ACTIONS(1921),
    [sym_smtp_tls_false] = ACTIONS(1921),
    [sym_listview_on_row_select_event] = ACTIONS(1921),
    [sym_data_structure_unknown] = ACTIONS(1921),
    [sym_data_structure_datafeed] = ACTIONS(1921),
    [sym_data_structure_datarecordset] = ACTIONS(1921),
    [sym_data_structure_datarecord] = ACTIONS(1921),
    [sym_data_structure_datafield] = ACTIONS(1921),
    [sym_data_max_content_field_count] = ACTIONS(1921),
    [sym_data_max_metadata_field_count] = ACTIONS(1921),
    [sym_data_max_id_length] = ACTIONS(1921),
    [sym_data_max_name_length] = ACTIONS(1921),
    [sym_data_max_description_length] = ACTIONS(1921),
    [sym_data_max_type_length] = ACTIONS(1921),
    [sym_data_max_format_length] = ACTIONS(1921),
    [sym_data_max_source_length] = ACTIONS(1921),
    [sym_data_max_value_length] = ACTIONS(1921),
    [sym_data_max_label_length] = ACTIONS(1921),
    [sym_data_max_url_length] = ACTIONS(1921),
    [sym_max_last_login_info_length] = ACTIONS(1921),
    [sym_data_type_unknown] = ACTIONS(1921),
    [sym_data_type_string] = ACTIONS(1921),
    [sym_data_type_datetime] = ACTIONS(1921),
    [sym_data_type_date] = ACTIONS(1921),
    [sym_data_type_time] = ACTIONS(1921),
    [sym_data_type_image] = ACTIONS(1921),
    [sym_data_format_url] = ACTIONS(1921),
    [sym_data_format_phone] = ACTIONS(1921),
    [sym_data_format_email] = ACTIONS(1921),
    [sym_data_format_iso8601] = ACTIONS(1921),
    [sym_auditlog_priv] = ACTIONS(1921),
    [sym_configuration_priv] = ACTIONS(1921),
    [sym_deviceconfig_priv] = ACTIONS(1921),
    [sym_ftp_priv] = ACTIONS(1921),
    [sym_http_priv] = ACTIONS(1921),
    [sym_networkconfig_priv] = ACTIONS(1921),
    [sym_programport_priv] = ACTIONS(1921),
    [sym_remoteui_priv] = ACTIONS(1921),
    [sym_securitycontrol_priv] = ACTIONS(1921),
    [sym_softwaremanagement_priv] = ACTIONS(1921),
    [sym_terminal_priv] = ACTIONS(1921),
    [sym_tpadmin_priv] = ACTIONS(1921),
    [sym_user1_priv] = ACTIONS(1921),
    [sym_user2_priv] = ACTIONS(1921),
    [sym_user3_priv] = ACTIONS(1921),
    [sym_user4_priv] = ACTIONS(1921),
    [sym_usermanagement_priv] = ACTIONS(1921),
    [sym_valid_account] = ACTIONS(1921),
    [sym_err_name_invalid] = ACTIONS(1921),
    [sym_err_password_invalid] = ACTIONS(1921),
    [sym_err_invalid_account] = ACTIONS(1921),
    [sym_err_no_authorization] = ACTIONS(1921),
    [sym_err_invalid_parameter] = ACTIONS(1921),
    [sym_err_user_locked_out] = ACTIONS(1921),
    [sym_err_user_account_expired] = ACTIONS(1921),
    [sym_err_general_error] = ACTIONS(1921),
    [sym_err_authentication_error] = ACTIONS(1921),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1921),
    [sym_tls_validate_certificate] = ACTIONS(1921),
    [sym_tdata] = ACTIONS(1921),
    [sym_tchannel] = ACTIONS(1921),
    [sym_tlevel] = ACTIONS(1921),
    [sym_tbutton] = ACTIONS(1921),
    [sym_ttimeline] = ACTIONS(1921),
    [sym_tcustom] = ACTIONS(1921),
    [sym_url_struct] = ACTIONS(1921),
    [sym_dns_struct] = ACTIONS(1921),
    [sym_ip_address_struct] = ACTIONS(1921),
    [sym_dev_info_struct] = ACTIONS(1921),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1921),
    [sym_data_feed] = ACTIONS(1921),
    [sym_data_field] = ACTIONS(1921),
    [sym_data_record] = ACTIONS(1921),
    [sym_wc_data_feed] = ACTIONS(1921),
    [sym_wc_data_field] = ACTIONS(1921),
    [sym_wc_data_record] = ACTIONS(1921),
    [sym_last_login_info] = ACTIONS(1921),
    [sym____reserved___] = ACTIONS(1921),
    [sym_date] = ACTIONS(1921),
    [sym_day] = ACTIONS(1921),
    [sym_time] = ACTIONS(1921),
    [sym_ldate] = ACTIONS(1921),
    [sym_push_device] = ACTIONS(1921),
    [sym_push_channel] = ACTIONS(1921),
    [sym_push_devchan] = ACTIONS(1921),
    [sym_release_device] = ACTIONS(1921),
    [sym_release_channel] = ACTIONS(1921),
    [sym_release_devchan] = ACTIONS(1921),
    [sym_master_slot] = ACTIONS(1921),
    [sym_get_pulse_time] = ACTIONS(1921),
    [sym_get_timer] = ACTIONS(1921),
    [sym_master_sn] = ACTIONS(1921),
    [sym_system_number] = ACTIONS(1921),
    [sym_button] = ACTIONS(1921),
    [sym_dv_channel] = ACTIONS(1921),
    [sym_data] = ACTIONS(1921),
    [sym_level] = ACTIONS(1921),
    [sym_timeline] = ACTIONS(1921),
    [sym_channel] = ACTIONS(1921),
    [sym_custom] = ACTIONS(1921),
    [sym_atoi] = ACTIONS(1921),
    [sym_atol] = ACTIONS(1921),
    [sym_atof] = ACTIONS(1921),
    [sym_itoa] = ACTIONS(1921),
    [sym_itohex] = ACTIONS(1921),
    [sym_hextoi] = ACTIONS(1921),
    [sym_ftoa] = ACTIONS(1921),
    [sym_find_string] = ACTIONS(1921),
    [sym_left_string] = ACTIONS(1921),
    [sym_right_string] = ACTIONS(1921),
    [sym_mid_string] = ACTIONS(1921),
    [sym_remove_string] = ACTIONS(1921),
    [sym_get_buffer_string] = ACTIONS(1921),
    [sym_get_buffer_char] = ACTIONS(1921),
    [sym_get_multi_buffer_string] = ACTIONS(1921),
    [sym_length_string] = ACTIONS(1921),
    [sym_max_length_string] = ACTIONS(1921),
    [sym_set_length_string] = ACTIONS(1921),
    [sym_compare_string] = ACTIONS(1921),
    [sym_upper_string] = ACTIONS(1921),
    [sym_lower_string] = ACTIONS(1921),
    [sym_get_last] = ACTIONS(1921),
    [sym_date_to_day] = ACTIONS(1921),
    [sym_date_to_month] = ACTIONS(1921),
    [sym_date_to_year] = ACTIONS(1921),
    [sym_time_to_hour] = ACTIONS(1921),
    [sym_time_to_minute] = ACTIONS(1921),
    [sym_time_to_second] = ACTIONS(1921),
    [sym_day_of_week] = ACTIONS(1921),
    [sym_set_pulse_time] = ACTIONS(1921),
    [sym_set_timer] = ACTIONS(1921),
    [sym_file_open] = ACTIONS(1921),
    [sym_file_close] = ACTIONS(1921),
    [sym_file_read] = ACTIONS(1921),
    [sym_file_read_line] = ACTIONS(1921),
    [sym_file_write] = ACTIONS(1921),
    [sym_file_write_line] = ACTIONS(1921),
    [sym_file_copy] = ACTIONS(1921),
    [sym_file_delete] = ACTIONS(1921),
    [sym_file_rename] = ACTIONS(1921),
    [sym_file_dir] = ACTIONS(1921),
    [sym_file_getdir] = ACTIONS(1921),
    [sym_file_setdir] = ACTIONS(1921),
    [sym_file_create_dir] = ACTIONS(1921),
    [sym_file_remove_dir] = ACTIONS(1921),
    [sym_file_seek] = ACTIONS(1921),
    [sym_length_array] = ACTIONS(1921),
    [sym_max_length_array] = ACTIONS(1921),
    [sym_set_length_array] = ACTIONS(1921),
    [sym_ip_client_open] = ACTIONS(1921),
    [sym_ip_client_close] = ACTIONS(1921),
    [sym_ip_server_open] = ACTIONS(1921),
    [sym_ip_server_close] = ACTIONS(1921),
    [sym_ip_bound_client_open] = ACTIONS(1921),
    [sym_ip_mc_server_open] = ACTIONS(1921),
    [sym_ip_set_option] = ACTIONS(1921),
    [sym_ssh_client_open] = ACTIONS(1921),
    [sym_ssh_client_close] = ACTIONS(1921),
    [sym_tls_client_open] = ACTIONS(1921),
    [sym_tls_client_close] = ACTIONS(1921),
    [sym_do_push] = ACTIONS(1921),
    [sym_do_push_timed] = ACTIONS(1921),
    [sym_do_release] = ACTIONS(1921),
    [sym_device_id] = ACTIONS(1921),
    [sym_device_info] = ACTIONS(1921),
    [sym_device_id_string] = ACTIONS(1921),
    [sym_device_standby] = ACTIONS(1921),
    [sym_device_wake] = ACTIONS(1921),
    [sym_timeline_create] = ACTIONS(1921),
    [sym_timeline_kill] = ACTIONS(1921),
    [sym_timeline_reload] = ACTIONS(1921),
    [sym_timeline_pause] = ACTIONS(1921),
    [sym_timeline_restart] = ACTIONS(1921),
    [sym_timeline_set] = ACTIONS(1921),
    [sym_timeline_get] = ACTIONS(1921),
    [sym_timeline_active] = ACTIONS(1921),
    [sym_timeline_dynamic_id] = ACTIONS(1921),
    [sym_variable_to_string] = ACTIONS(1921),
    [sym_string_to_variable] = ACTIONS(1921),
    [sym_length_variable_to_string] = ACTIONS(1921),
    [sym_type_cast] = ACTIONS(1921),
    [sym_variable_to_xml] = ACTIONS(1921),
    [sym_xml_to_variable] = ACTIONS(1921),
    [sym_length_variable_to_xml] = ACTIONS(1921),
    [sym_redirect_string] = ACTIONS(1921),
    [sym_random_number] = ACTIONS(1921),
    [sym_get_unique_id] = ACTIONS(1921),
    [sym_get_system_number] = ACTIONS(1921),
    [sym_set_system_number] = ACTIONS(1921),
    [sym_get_serial_number] = ACTIONS(1921),
    [sym_get_master_build] = ACTIONS(1921),
    [sym_set_validation_code] = ACTIONS(1921),
    [sym_reboot] = ACTIONS(1921),
    [sym_load_duet_module] = ACTIONS(1921),
    [sym_unload_duet_module] = ACTIONS(1921),
    [sym_duet_mem_size_set] = ACTIONS(1921),
    [sym_duet_mem_size_get] = ACTIONS(1921),
    [sym_dynamic_application_device] = ACTIONS(1921),
    [sym_dynamic_polled_port] = ACTIONS(1921),
    [sym_static_port_binding] = ACTIONS(1921),
    [sym_static_ip_binding] = ACTIONS(1921),
    [sym_get_dns_list] = ACTIONS(1921),
    [sym_set_dns_list] = ACTIONS(1921),
    [sym_get_ip_address] = ACTIONS(1921),
    [sym_set_ip_address] = ACTIONS(1921),
    [sym_get_url_list] = ACTIONS(1921),
    [sym_add_url_entry] = ACTIONS(1921),
    [sym_delete_url_entry] = ACTIONS(1921),
    [sym_abs_value] = ACTIONS(1921),
    [sym_max_value] = ACTIONS(1921),
    [sym_min_value] = ACTIONS(1921),
    [sym_exp_value] = ACTIONS(1921),
    [sym_log_value] = ACTIONS(1921),
    [sym_log10_value] = ACTIONS(1921),
    [sym_power_value] = ACTIONS(1921),
    [sym_sqrt_value] = ACTIONS(1921),
    [sym_raw_be] = ACTIONS(1921),
    [sym_raw_le] = ACTIONS(1921),
    [sym_internal_queue_size_set] = ACTIONS(1921),
    [sym_internal_queue_size_get] = ACTIONS(1921),
    [sym_internal_threshold_set] = ACTIONS(1921),
    [sym_internal_threshold_get] = ACTIONS(1921),
    [sym_rebuilt_event] = ACTIONS(1921),
    [sym_do_custom_event] = ACTIONS(1921),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1921),
    [sym_clkmgr_set_clk_source] = ACTIONS(1921),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1921),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1921),
    [sym_clkmgr_get_timezone] = ACTIONS(1921),
    [sym_clkmgr_set_timezone] = ACTIONS(1921),
    [sym_clkmgr_get_resync_period] = ACTIONS(1921),
    [sym_clkmgr_set_resync_period] = ACTIONS(1921),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1921),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1921),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1921),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1921),
    [sym_clkmgr_get_timeservers] = ACTIONS(1921),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1921),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1921),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1921),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1921),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1921),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1921),
    [sym_set_log_level] = ACTIONS(1921),
    [sym_get_log_level] = ACTIONS(1921),
    [sym_amx_log] = ACTIONS(1921),
    [sym_smtp_server_config_set] = ACTIONS(1921),
    [sym_smtp_server_config_get] = ACTIONS(1921),
    [sym_smtp_send] = ACTIONS(1921),
    [sym_validate_netlinx_account] = ACTIONS(1921),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1921),
    [sym_authenticate_certificate] = ACTIONS(1921),
    [sym_audit_netlinx_session_event] = ACTIONS(1921),
    [sym_audit_netlinx_generic_event] = ACTIONS(1921),
    [sym_get_available_flash_disk_space] = ACTIONS(1921),
    [sym_get_max_flash_disk_space] = ACTIONS(1921),
    [sym_data_create_feed] = ACTIONS(1921),
    [sym_data_delete_feed] = ACTIONS(1921),
    [sym_data_publish_feed] = ACTIONS(1921),
    [sym_data_get_published_feed] = ACTIONS(1921),
    [sym_data_add_record] = ACTIONS(1921),
    [sym_data_get_event_record] = ACTIONS(1921),
    [sym__wc_data_create_feed] = ACTIONS(1921),
    [sym__wc_data_add_record] = ACTIONS(1921),
    [sym__wc_data_get_event_record] = ACTIONS(1921),
    [sym_format] = ACTIONS(1921),
    [sym_astro_clock] = ACTIONS(1921),
    [sym_set_outdoor_temperature] = ACTIONS(1921),
    [sym_set_virtual_level_count] = ACTIONS(1921),
    [sym_set_virtual_channel_count] = ACTIONS(1921),
    [sym_set_virtual_port_count] = ACTIONS(1921),
    [sym__wc_encode_int] = ACTIONS(1921),
    [sym__wc_decode_int] = ACTIONS(1921),
    [sym__wc_int] = ACTIONS(1921),
    [sym__wc_to_ch_int] = ACTIONS(1921),
    [sym__ch_to_wc_int] = ACTIONS(1921),
    [sym__wc_find_string_int] = ACTIONS(1921),
    [sym__wc_left_string_int] = ACTIONS(1921),
    [sym__wc_lower_string_int] = ACTIONS(1921),
    [sym__wc_mid_string_int] = ACTIONS(1921),
    [sym__wc_remove_string_int] = ACTIONS(1921),
    [sym__wc_right_string_int] = ACTIONS(1921),
    [sym__wc_upper_string_int] = ACTIONS(1921),
    [sym__wc_compare_string_int] = ACTIONS(1921),
    [sym__wc_get_buffer_char_int] = ACTIONS(1921),
    [sym__wc_get_buffer_string_int] = ACTIONS(1921),
    [sym__wc_concat_string_int] = ACTIONS(1921),
    [sym__wc_file_open_int] = ACTIONS(1921),
    [sym__wc_file_close_int] = ACTIONS(1921),
    [sym__wc_file_read_int] = ACTIONS(1921),
    [sym__wc_file_read_line_int] = ACTIONS(1921),
    [sym__wc_file_write_int] = ACTIONS(1921),
    [sym__wc_file_write_line_int] = ACTIONS(1921),
    [aux_sym_preproc_include_token1] = ACTIONS(1921),
    [aux_sym_preproc_define_token1] = ACTIONS(1923),
    [aux_sym_preproc_warn_token1] = ACTIONS(1923),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1923),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1923),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1923),
    [anon_sym_LBRACK] = ACTIONS(1923),
    [anon_sym_LBRACE] = ACTIONS(1923),
    [anon_sym_LPAREN] = ACTIONS(1921),
    [anon_sym_BANG] = ACTIONS(1923),
    [anon_sym_TILDE] = ACTIONS(1923),
    [anon_sym_DASH] = ACTIONS(1921),
    [anon_sym_PLUS] = ACTIONS(1921),
    [anon_sym_DASH_DASH] = ACTIONS(1923),
    [anon_sym_PLUS_PLUS] = ACTIONS(1923),
    [anon_sym_DQUOTE] = ACTIONS(1923),
    [anon_sym_SQUOTE] = ACTIONS(1923),
    [sym_number_literal] = ACTIONS(1923),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(447)] = {
    [ts_builtin_sym_end] = ACTIONS(2071),
    [sym_identifier] = ACTIONS(2069),
    [sym_program_name_keyword] = ACTIONS(2069),
    [sym_module_name_keyword] = ACTIONS(2069),
    [sym_define_device_keyword] = ACTIONS(2069),
    [sym_define_combine_keyword] = ACTIONS(2069),
    [sym_define_constant_keyword] = ACTIONS(2069),
    [sym_define_type_keyword] = ACTIONS(2069),
    [sym_define_variable_keyword] = ACTIONS(2069),
    [sym_define_system_variable_keyword] = ACTIONS(2069),
    [sym_define_connect_level_keyword] = ACTIONS(2069),
    [sym_define_latching_keyword] = ACTIONS(2069),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2069),
    [sym_define_toggling_keyword] = ACTIONS(2069),
    [sym_define_call_keyword] = ACTIONS(2069),
    [sym_define_function_keyword] = ACTIONS(2069),
    [sym_define_library_function_keyword] = ACTIONS(2069),
    [sym_define_start_keyword] = ACTIONS(2069),
    [sym_define_event_keyword] = ACTIONS(2069),
    [sym_define_program_keyword] = ACTIONS(2069),
    [sym_define_module_keyword] = ACTIONS(2069),
    [sym_char_keyword] = ACTIONS(2069),
    [sym_widechar_keyword] = ACTIONS(2069),
    [sym_integer_keyword] = ACTIONS(2069),
    [sym_sinteger_keyword] = ACTIONS(2069),
    [sym_long_keyword] = ACTIONS(2069),
    [sym_slong_keyword] = ACTIONS(2069),
    [sym_float_keyword] = ACTIONS(2069),
    [sym_double_keyword] = ACTIONS(2069),
    [sym_variant_keyword] = ACTIONS(2069),
    [sym_variantarray_keyword] = ACTIONS(2069),
    [sym_local_var_keyword] = ACTIONS(2069),
    [sym_stack_var_keyword] = ACTIONS(2069),
    [sym_constant_keyword] = ACTIONS(2069),
    [sym_volatile_keyword] = ACTIONS(2069),
    [sym_non_volatile_keyword] = ACTIONS(2069),
    [sym_persistent_keyword] = ACTIONS(2069),
    [sym_dev_keyword] = ACTIONS(2069),
    [sym_devlev_keyword] = ACTIONS(2069),
    [sym_devchan_keyword] = ACTIONS(2069),
    [sym_if_keyword] = ACTIONS(2069),
    [sym_switch_keyword] = ACTIONS(2069),
    [sym_case_keyword] = ACTIONS(2069),
    [sym_default_keyword] = ACTIONS(2069),
    [sym_while_keyword] = ACTIONS(2069),
    [sym_for_keyword] = ACTIONS(2069),
    [sym_break_keyword] = ACTIONS(2069),
    [sym_continue_keyword] = ACTIONS(2069),
    [sym_return_keyword] = ACTIONS(2069),
    [sym_select_keyword] = ACTIONS(2069),
    [sym_active_keyword] = ACTIONS(2069),
    [sym_struct_keyword] = ACTIONS(2069),
    [sym_structure_keyword] = ACTIONS(2069),
    [sym_button_event_keyword] = ACTIONS(2069),
    [sym_channel_event_keyword] = ACTIONS(2069),
    [sym_level_event_keyword] = ACTIONS(2069),
    [sym_data_event_keyword] = ACTIONS(2069),
    [sym_timeline_event_keyword] = ACTIONS(2069),
    [sym_custom_event_keyword] = ACTIONS(2069),
    [sym_push_keyword] = ACTIONS(2069),
    [sym_release_keyword] = ACTIONS(2069),
    [aux_sym_on_keyword_token1] = ACTIONS(2069),
    [aux_sym_off_keyword_token1] = ACTIONS(2069),
    [sym_send_level_keyword] = ACTIONS(2069),
    [sym_send_string_keyword] = ACTIONS(2069),
    [sym_send_command_keyword] = ACTIONS(2069),
    [sym_clear_buffer_keyword] = ACTIONS(2069),
    [sym_create_buffer_keyword] = ACTIONS(2069),
    [sym_create_multi_buffer_keyword] = ACTIONS(2069),
    [sym_call_keyword] = ACTIONS(2069),
    [sym_system_call_keyword] = ACTIONS(2069),
    [sym_devchan_to_keyword] = ACTIONS(2069),
    [sym_devchan_min_to_keyword] = ACTIONS(2069),
    [sym_devchan_total_off_keyword] = ACTIONS(2069),
    [sym_devchan_pulse_keyword] = ACTIONS(2069),
    [sym_wait_keyword] = ACTIONS(2069),
    [sym_cancel_wait_keyword] = ACTIONS(2069),
    [sym_wait_until_keyword] = ACTIONS(2069),
    [sym_cancel_wait_until_keyword] = ACTIONS(2069),
    [sym_cancel_all_wait_keyword] = ACTIONS(2069),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2069),
    [sym_not] = ACTIONS(2069),
    [sym_bnot] = ACTIONS(2069),
    [sym___file__] = ACTIONS(2069),
    [sym___date__] = ACTIONS(2069),
    [sym___ldate__] = ACTIONS(2069),
    [sym___time__] = ACTIONS(2069),
    [sym___line__] = ACTIONS(2069),
    [sym___name__] = ACTIONS(2069),
    [sym___netlinx__] = ACTIONS(2069),
    [sym_netlinx_axi_version] = ACTIONS(2069),
    [sym_url_flg_tcp] = ACTIONS(2069),
    [sym_url_flg_accinfopresent] = ACTIONS(2069),
    [sym_url_flg_temp] = ACTIONS(2069),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2069),
    [sym_url_flg_stat_mask] = ACTIONS(2069),
    [sym_url_flg_stat_lookup] = ACTIONS(2069),
    [sym_url_flg_stat_connecting] = ACTIONS(2069),
    [sym_url_flg_stat_waiting] = ACTIONS(2069),
    [sym_url_flg_stat_connected] = ACTIONS(2069),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2069),
    [sym_first_virtual_device] = ACTIONS(2069),
    [sym_first_local_port] = ACTIONS(2069),
    [sym_dynamic_virtual_device] = ACTIONS(2069),
    [sym_timeline_once] = ACTIONS(2069),
    [sym_timeline_repeat] = ACTIONS(2069),
    [sym_timeline_absolute] = ACTIONS(2069),
    [sym_timeline_relative] = ACTIONS(2069),
    [sym_do_push_timed_infinite] = ACTIONS(2069),
    [sym_source_type_no_address] = ACTIONS(2069),
    [sym_source_type_neuron_id] = ACTIONS(2069),
    [sym_source_type_ip_address] = ACTIONS(2069),
    [sym_source_type_axlink] = ACTIONS(2069),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2069),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2069),
    [sym_source_type_ip_socket_address] = ACTIONS(2069),
    [sym_source_type_rs232] = ACTIONS(2069),
    [sym_source_type_internal] = ACTIONS(2069),
    [sym_source_type_mac_address] = ACTIONS(2069),
    [sym_source_type_ipv4_port] = ACTIONS(2069),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2069),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2069),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2069),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2069),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2069),
    [sym_true] = ACTIONS(2069),
    [sym_false] = ACTIONS(2069),
    [sym_file_read_only] = ACTIONS(2069),
    [sym_file_rw_new] = ACTIONS(2069),
    [sym_file_rw_append] = ACTIONS(2069),
    [sym_ip_tcp] = ACTIONS(2069),
    [sym_ip_udp] = ACTIONS(2069),
    [sym_ip_udp_2way] = ACTIONS(2069),
    [sym_xml_encode_types] = ACTIONS(2069),
    [sym_xml_encode_char_as_list] = ACTIONS(2069),
    [sym_xml_encode_le] = ACTIONS(2069),
    [sym_xml_decode_types] = ACTIONS(2069),
    [sym_xml_decode_no_preserve] = ACTIONS(2069),
    [sym_duet_dev_type_display_device] = ACTIONS(2069),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2069),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2069),
    [sym_duet_dev_type_io_device] = ACTIONS(2069),
    [sym_duet_dev_type_relay_device] = ACTIONS(2069),
    [sym_duet_dev_type_ups] = ACTIONS(2069),
    [sym_duet_dev_type_amplifier] = ACTIONS(2069),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2069),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2069),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2069),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2069),
    [sym_duet_dev_type_camera] = ACTIONS(2069),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2069),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2069),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2069),
    [sym_duet_dev_type_dss] = ACTIONS(2069),
    [sym_duet_dev_type_dvr] = ACTIONS(2069),
    [sym_duet_dev_type_disc_device] = ACTIONS(2069),
    [sym_duet_dev_type_document_camera] = ACTIONS(2069),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2069),
    [sym_duet_dev_type_hvac] = ACTIONS(2069),
    [sym_duet_dev_type_keypad] = ACTIONS(2069),
    [sym_duet_dev_type_light] = ACTIONS(2069),
    [sym_duet_dev_type_monitor] = ACTIONS(2069),
    [sym_duet_dev_type_motor] = ACTIONS(2069),
    [sym_duet_dev_type_multi_window] = ACTIONS(2069),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2069),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2069),
    [sym_duet_dev_type_receiver] = ACTIONS(2069),
    [sym_duet_dev_type_security_system] = ACTIONS(2069),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2069),
    [sym_duet_dev_type_settop_box] = ACTIONS(2069),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2069),
    [sym_duet_dev_type_switcher] = ACTIONS(2069),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2069),
    [sym_duet_dev_type_tv] = ACTIONS(2069),
    [sym_duet_dev_type_utility] = ACTIONS(2069),
    [sym_duet_dev_type_vcr] = ACTIONS(2069),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2069),
    [sym_duet_dev_type_video_processor] = ACTIONS(2069),
    [sym_duet_dev_type_video_projector] = ACTIONS(2069),
    [sym_duet_dev_type_video_wall] = ACTIONS(2069),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2069),
    [sym_duet_dev_type_weather] = ACTIONS(2069),
    [sym_duet_dev_not_polled] = ACTIONS(2069),
    [sym_duet_dev_polled] = ACTIONS(2069),
    [sym_clkmgr_mode_network] = ACTIONS(2069),
    [sym_clkmgr_mode_standalone] = ACTIONS(2069),
    [sym_normal_standby] = ACTIONS(2069),
    [sym_normal_wake] = ACTIONS(2069),
    [sym_ip_multicast_ttl_option] = ACTIONS(2069),
    [sym_ip_ttl_subnet] = ACTIONS(2069),
    [sym_ip_ttl_site] = ACTIONS(2069),
    [sym_ip_ttl_region] = ACTIONS(2069),
    [sym_ip_ttl_continent] = ACTIONS(2069),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2069),
    [sym_ip_nodelay_on] = ACTIONS(2069),
    [sym_ip_nodelay_off] = ACTIONS(2069),
    [sym_amx_error] = ACTIONS(2069),
    [sym_amx_warning] = ACTIONS(2069),
    [sym_amx_info] = ACTIONS(2069),
    [sym_amx_debug] = ACTIONS(2069),
    [sym_netlinx_login_success] = ACTIONS(2069),
    [sym_netlinx_login_fail] = ACTIONS(2069),
    [sym_netlinx_logout] = ACTIONS(2069),
    [sym_null_str] = ACTIONS(2069),
    [sym_smtp_address] = ACTIONS(2069),
    [sym_smtp_port_number] = ACTIONS(2069),
    [sym_smtp_username] = ACTIONS(2069),
    [sym_smtp_password] = ACTIONS(2069),
    [sym_smtp_require_tls] = ACTIONS(2069),
    [sym_smtp_from] = ACTIONS(2069),
    [sym_smtp_tls_true] = ACTIONS(2069),
    [sym_smtp_tls_false] = ACTIONS(2069),
    [sym_listview_on_row_select_event] = ACTIONS(2069),
    [sym_data_structure_unknown] = ACTIONS(2069),
    [sym_data_structure_datafeed] = ACTIONS(2069),
    [sym_data_structure_datarecordset] = ACTIONS(2069),
    [sym_data_structure_datarecord] = ACTIONS(2069),
    [sym_data_structure_datafield] = ACTIONS(2069),
    [sym_data_max_content_field_count] = ACTIONS(2069),
    [sym_data_max_metadata_field_count] = ACTIONS(2069),
    [sym_data_max_id_length] = ACTIONS(2069),
    [sym_data_max_name_length] = ACTIONS(2069),
    [sym_data_max_description_length] = ACTIONS(2069),
    [sym_data_max_type_length] = ACTIONS(2069),
    [sym_data_max_format_length] = ACTIONS(2069),
    [sym_data_max_source_length] = ACTIONS(2069),
    [sym_data_max_value_length] = ACTIONS(2069),
    [sym_data_max_label_length] = ACTIONS(2069),
    [sym_data_max_url_length] = ACTIONS(2069),
    [sym_max_last_login_info_length] = ACTIONS(2069),
    [sym_data_type_unknown] = ACTIONS(2069),
    [sym_data_type_string] = ACTIONS(2069),
    [sym_data_type_datetime] = ACTIONS(2069),
    [sym_data_type_date] = ACTIONS(2069),
    [sym_data_type_time] = ACTIONS(2069),
    [sym_data_type_image] = ACTIONS(2069),
    [sym_data_format_url] = ACTIONS(2069),
    [sym_data_format_phone] = ACTIONS(2069),
    [sym_data_format_email] = ACTIONS(2069),
    [sym_data_format_iso8601] = ACTIONS(2069),
    [sym_auditlog_priv] = ACTIONS(2069),
    [sym_configuration_priv] = ACTIONS(2069),
    [sym_deviceconfig_priv] = ACTIONS(2069),
    [sym_ftp_priv] = ACTIONS(2069),
    [sym_http_priv] = ACTIONS(2069),
    [sym_networkconfig_priv] = ACTIONS(2069),
    [sym_programport_priv] = ACTIONS(2069),
    [sym_remoteui_priv] = ACTIONS(2069),
    [sym_securitycontrol_priv] = ACTIONS(2069),
    [sym_softwaremanagement_priv] = ACTIONS(2069),
    [sym_terminal_priv] = ACTIONS(2069),
    [sym_tpadmin_priv] = ACTIONS(2069),
    [sym_user1_priv] = ACTIONS(2069),
    [sym_user2_priv] = ACTIONS(2069),
    [sym_user3_priv] = ACTIONS(2069),
    [sym_user4_priv] = ACTIONS(2069),
    [sym_usermanagement_priv] = ACTIONS(2069),
    [sym_valid_account] = ACTIONS(2069),
    [sym_err_name_invalid] = ACTIONS(2069),
    [sym_err_password_invalid] = ACTIONS(2069),
    [sym_err_invalid_account] = ACTIONS(2069),
    [sym_err_no_authorization] = ACTIONS(2069),
    [sym_err_invalid_parameter] = ACTIONS(2069),
    [sym_err_user_locked_out] = ACTIONS(2069),
    [sym_err_user_account_expired] = ACTIONS(2069),
    [sym_err_general_error] = ACTIONS(2069),
    [sym_err_authentication_error] = ACTIONS(2069),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2069),
    [sym_tls_validate_certificate] = ACTIONS(2069),
    [sym_tdata] = ACTIONS(2069),
    [sym_tchannel] = ACTIONS(2069),
    [sym_tlevel] = ACTIONS(2069),
    [sym_tbutton] = ACTIONS(2069),
    [sym_ttimeline] = ACTIONS(2069),
    [sym_tcustom] = ACTIONS(2069),
    [sym_url_struct] = ACTIONS(2069),
    [sym_dns_struct] = ACTIONS(2069),
    [sym_ip_address_struct] = ACTIONS(2069),
    [sym_dev_info_struct] = ACTIONS(2069),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2069),
    [sym_data_feed] = ACTIONS(2069),
    [sym_data_field] = ACTIONS(2069),
    [sym_data_record] = ACTIONS(2069),
    [sym_wc_data_feed] = ACTIONS(2069),
    [sym_wc_data_field] = ACTIONS(2069),
    [sym_wc_data_record] = ACTIONS(2069),
    [sym_last_login_info] = ACTIONS(2069),
    [sym____reserved___] = ACTIONS(2069),
    [sym_date] = ACTIONS(2069),
    [sym_day] = ACTIONS(2069),
    [sym_time] = ACTIONS(2069),
    [sym_ldate] = ACTIONS(2069),
    [sym_push_device] = ACTIONS(2069),
    [sym_push_channel] = ACTIONS(2069),
    [sym_push_devchan] = ACTIONS(2069),
    [sym_release_device] = ACTIONS(2069),
    [sym_release_channel] = ACTIONS(2069),
    [sym_release_devchan] = ACTIONS(2069),
    [sym_master_slot] = ACTIONS(2069),
    [sym_get_pulse_time] = ACTIONS(2069),
    [sym_get_timer] = ACTIONS(2069),
    [sym_master_sn] = ACTIONS(2069),
    [sym_system_number] = ACTIONS(2069),
    [sym_button] = ACTIONS(2069),
    [sym_dv_channel] = ACTIONS(2069),
    [sym_data] = ACTIONS(2069),
    [sym_level] = ACTIONS(2069),
    [sym_timeline] = ACTIONS(2069),
    [sym_channel] = ACTIONS(2069),
    [sym_custom] = ACTIONS(2069),
    [sym_atoi] = ACTIONS(2069),
    [sym_atol] = ACTIONS(2069),
    [sym_atof] = ACTIONS(2069),
    [sym_itoa] = ACTIONS(2069),
    [sym_itohex] = ACTIONS(2069),
    [sym_hextoi] = ACTIONS(2069),
    [sym_ftoa] = ACTIONS(2069),
    [sym_find_string] = ACTIONS(2069),
    [sym_left_string] = ACTIONS(2069),
    [sym_right_string] = ACTIONS(2069),
    [sym_mid_string] = ACTIONS(2069),
    [sym_remove_string] = ACTIONS(2069),
    [sym_get_buffer_string] = ACTIONS(2069),
    [sym_get_buffer_char] = ACTIONS(2069),
    [sym_get_multi_buffer_string] = ACTIONS(2069),
    [sym_length_string] = ACTIONS(2069),
    [sym_max_length_string] = ACTIONS(2069),
    [sym_set_length_string] = ACTIONS(2069),
    [sym_compare_string] = ACTIONS(2069),
    [sym_upper_string] = ACTIONS(2069),
    [sym_lower_string] = ACTIONS(2069),
    [sym_get_last] = ACTIONS(2069),
    [sym_date_to_day] = ACTIONS(2069),
    [sym_date_to_month] = ACTIONS(2069),
    [sym_date_to_year] = ACTIONS(2069),
    [sym_time_to_hour] = ACTIONS(2069),
    [sym_time_to_minute] = ACTIONS(2069),
    [sym_time_to_second] = ACTIONS(2069),
    [sym_day_of_week] = ACTIONS(2069),
    [sym_set_pulse_time] = ACTIONS(2069),
    [sym_set_timer] = ACTIONS(2069),
    [sym_file_open] = ACTIONS(2069),
    [sym_file_close] = ACTIONS(2069),
    [sym_file_read] = ACTIONS(2069),
    [sym_file_read_line] = ACTIONS(2069),
    [sym_file_write] = ACTIONS(2069),
    [sym_file_write_line] = ACTIONS(2069),
    [sym_file_copy] = ACTIONS(2069),
    [sym_file_delete] = ACTIONS(2069),
    [sym_file_rename] = ACTIONS(2069),
    [sym_file_dir] = ACTIONS(2069),
    [sym_file_getdir] = ACTIONS(2069),
    [sym_file_setdir] = ACTIONS(2069),
    [sym_file_create_dir] = ACTIONS(2069),
    [sym_file_remove_dir] = ACTIONS(2069),
    [sym_file_seek] = ACTIONS(2069),
    [sym_length_array] = ACTIONS(2069),
    [sym_max_length_array] = ACTIONS(2069),
    [sym_set_length_array] = ACTIONS(2069),
    [sym_ip_client_open] = ACTIONS(2069),
    [sym_ip_client_close] = ACTIONS(2069),
    [sym_ip_server_open] = ACTIONS(2069),
    [sym_ip_server_close] = ACTIONS(2069),
    [sym_ip_bound_client_open] = ACTIONS(2069),
    [sym_ip_mc_server_open] = ACTIONS(2069),
    [sym_ip_set_option] = ACTIONS(2069),
    [sym_ssh_client_open] = ACTIONS(2069),
    [sym_ssh_client_close] = ACTIONS(2069),
    [sym_tls_client_open] = ACTIONS(2069),
    [sym_tls_client_close] = ACTIONS(2069),
    [sym_do_push] = ACTIONS(2069),
    [sym_do_push_timed] = ACTIONS(2069),
    [sym_do_release] = ACTIONS(2069),
    [sym_device_id] = ACTIONS(2069),
    [sym_device_info] = ACTIONS(2069),
    [sym_device_id_string] = ACTIONS(2069),
    [sym_device_standby] = ACTIONS(2069),
    [sym_device_wake] = ACTIONS(2069),
    [sym_timeline_create] = ACTIONS(2069),
    [sym_timeline_kill] = ACTIONS(2069),
    [sym_timeline_reload] = ACTIONS(2069),
    [sym_timeline_pause] = ACTIONS(2069),
    [sym_timeline_restart] = ACTIONS(2069),
    [sym_timeline_set] = ACTIONS(2069),
    [sym_timeline_get] = ACTIONS(2069),
    [sym_timeline_active] = ACTIONS(2069),
    [sym_timeline_dynamic_id] = ACTIONS(2069),
    [sym_variable_to_string] = ACTIONS(2069),
    [sym_string_to_variable] = ACTIONS(2069),
    [sym_length_variable_to_string] = ACTIONS(2069),
    [sym_type_cast] = ACTIONS(2069),
    [sym_variable_to_xml] = ACTIONS(2069),
    [sym_xml_to_variable] = ACTIONS(2069),
    [sym_length_variable_to_xml] = ACTIONS(2069),
    [sym_redirect_string] = ACTIONS(2069),
    [sym_random_number] = ACTIONS(2069),
    [sym_get_unique_id] = ACTIONS(2069),
    [sym_get_system_number] = ACTIONS(2069),
    [sym_set_system_number] = ACTIONS(2069),
    [sym_get_serial_number] = ACTIONS(2069),
    [sym_get_master_build] = ACTIONS(2069),
    [sym_set_validation_code] = ACTIONS(2069),
    [sym_reboot] = ACTIONS(2069),
    [sym_load_duet_module] = ACTIONS(2069),
    [sym_unload_duet_module] = ACTIONS(2069),
    [sym_duet_mem_size_set] = ACTIONS(2069),
    [sym_duet_mem_size_get] = ACTIONS(2069),
    [sym_dynamic_application_device] = ACTIONS(2069),
    [sym_dynamic_polled_port] = ACTIONS(2069),
    [sym_static_port_binding] = ACTIONS(2069),
    [sym_static_ip_binding] = ACTIONS(2069),
    [sym_get_dns_list] = ACTIONS(2069),
    [sym_set_dns_list] = ACTIONS(2069),
    [sym_get_ip_address] = ACTIONS(2069),
    [sym_set_ip_address] = ACTIONS(2069),
    [sym_get_url_list] = ACTIONS(2069),
    [sym_add_url_entry] = ACTIONS(2069),
    [sym_delete_url_entry] = ACTIONS(2069),
    [sym_abs_value] = ACTIONS(2069),
    [sym_max_value] = ACTIONS(2069),
    [sym_min_value] = ACTIONS(2069),
    [sym_exp_value] = ACTIONS(2069),
    [sym_log_value] = ACTIONS(2069),
    [sym_log10_value] = ACTIONS(2069),
    [sym_power_value] = ACTIONS(2069),
    [sym_sqrt_value] = ACTIONS(2069),
    [sym_raw_be] = ACTIONS(2069),
    [sym_raw_le] = ACTIONS(2069),
    [sym_internal_queue_size_set] = ACTIONS(2069),
    [sym_internal_queue_size_get] = ACTIONS(2069),
    [sym_internal_threshold_set] = ACTIONS(2069),
    [sym_internal_threshold_get] = ACTIONS(2069),
    [sym_rebuilt_event] = ACTIONS(2069),
    [sym_do_custom_event] = ACTIONS(2069),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2069),
    [sym_clkmgr_set_clk_source] = ACTIONS(2069),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2069),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2069),
    [sym_clkmgr_get_timezone] = ACTIONS(2069),
    [sym_clkmgr_set_timezone] = ACTIONS(2069),
    [sym_clkmgr_get_resync_period] = ACTIONS(2069),
    [sym_clkmgr_set_resync_period] = ACTIONS(2069),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2069),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2069),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2069),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2069),
    [sym_clkmgr_get_timeservers] = ACTIONS(2069),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2069),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2069),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2069),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2069),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2069),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2069),
    [sym_set_log_level] = ACTIONS(2069),
    [sym_get_log_level] = ACTIONS(2069),
    [sym_amx_log] = ACTIONS(2069),
    [sym_smtp_server_config_set] = ACTIONS(2069),
    [sym_smtp_server_config_get] = ACTIONS(2069),
    [sym_smtp_send] = ACTIONS(2069),
    [sym_validate_netlinx_account] = ACTIONS(2069),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2069),
    [sym_authenticate_certificate] = ACTIONS(2069),
    [sym_audit_netlinx_session_event] = ACTIONS(2069),
    [sym_audit_netlinx_generic_event] = ACTIONS(2069),
    [sym_get_available_flash_disk_space] = ACTIONS(2069),
    [sym_get_max_flash_disk_space] = ACTIONS(2069),
    [sym_data_create_feed] = ACTIONS(2069),
    [sym_data_delete_feed] = ACTIONS(2069),
    [sym_data_publish_feed] = ACTIONS(2069),
    [sym_data_get_published_feed] = ACTIONS(2069),
    [sym_data_add_record] = ACTIONS(2069),
    [sym_data_get_event_record] = ACTIONS(2069),
    [sym__wc_data_create_feed] = ACTIONS(2069),
    [sym__wc_data_add_record] = ACTIONS(2069),
    [sym__wc_data_get_event_record] = ACTIONS(2069),
    [sym_format] = ACTIONS(2069),
    [sym_astro_clock] = ACTIONS(2069),
    [sym_set_outdoor_temperature] = ACTIONS(2069),
    [sym_set_virtual_level_count] = ACTIONS(2069),
    [sym_set_virtual_channel_count] = ACTIONS(2069),
    [sym_set_virtual_port_count] = ACTIONS(2069),
    [sym__wc_encode_int] = ACTIONS(2069),
    [sym__wc_decode_int] = ACTIONS(2069),
    [sym__wc_int] = ACTIONS(2069),
    [sym__wc_to_ch_int] = ACTIONS(2069),
    [sym__ch_to_wc_int] = ACTIONS(2069),
    [sym__wc_find_string_int] = ACTIONS(2069),
    [sym__wc_left_string_int] = ACTIONS(2069),
    [sym__wc_lower_string_int] = ACTIONS(2069),
    [sym__wc_mid_string_int] = ACTIONS(2069),
    [sym__wc_remove_string_int] = ACTIONS(2069),
    [sym__wc_right_string_int] = ACTIONS(2069),
    [sym__wc_upper_string_int] = ACTIONS(2069),
    [sym__wc_compare_string_int] = ACTIONS(2069),
    [sym__wc_get_buffer_char_int] = ACTIONS(2069),
    [sym__wc_get_buffer_string_int] = ACTIONS(2069),
    [sym__wc_concat_string_int] = ACTIONS(2069),
    [sym__wc_file_open_int] = ACTIONS(2069),
    [sym__wc_file_close_int] = ACTIONS(2069),
    [sym__wc_file_read_int] = ACTIONS(2069),
    [sym__wc_file_read_line_int] = ACTIONS(2069),
    [sym__wc_file_write_int] = ACTIONS(2069),
    [sym__wc_file_write_line_int] = ACTIONS(2069),
    [aux_sym_preproc_include_token1] = ACTIONS(2069),
    [aux_sym_preproc_define_token1] = ACTIONS(2071),
    [aux_sym_preproc_warn_token1] = ACTIONS(2071),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2071),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2071),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2071),
    [anon_sym_LBRACK] = ACTIONS(2071),
    [anon_sym_LBRACE] = ACTIONS(2071),
    [anon_sym_LPAREN] = ACTIONS(2069),
    [anon_sym_BANG] = ACTIONS(2071),
    [anon_sym_TILDE] = ACTIONS(2071),
    [anon_sym_DASH] = ACTIONS(2069),
    [anon_sym_PLUS] = ACTIONS(2069),
    [anon_sym_DASH_DASH] = ACTIONS(2071),
    [anon_sym_PLUS_PLUS] = ACTIONS(2071),
    [anon_sym_DQUOTE] = ACTIONS(2071),
    [anon_sym_SQUOTE] = ACTIONS(2071),
    [sym_number_literal] = ACTIONS(2071),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(448)] = {
    [ts_builtin_sym_end] = ACTIONS(2075),
    [sym_identifier] = ACTIONS(2073),
    [sym_program_name_keyword] = ACTIONS(2073),
    [sym_module_name_keyword] = ACTIONS(2073),
    [sym_define_device_keyword] = ACTIONS(2073),
    [sym_define_combine_keyword] = ACTIONS(2073),
    [sym_define_constant_keyword] = ACTIONS(2073),
    [sym_define_type_keyword] = ACTIONS(2073),
    [sym_define_variable_keyword] = ACTIONS(2073),
    [sym_define_system_variable_keyword] = ACTIONS(2073),
    [sym_define_connect_level_keyword] = ACTIONS(2073),
    [sym_define_latching_keyword] = ACTIONS(2073),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(2073),
    [sym_define_toggling_keyword] = ACTIONS(2073),
    [sym_define_call_keyword] = ACTIONS(2073),
    [sym_define_function_keyword] = ACTIONS(2073),
    [sym_define_library_function_keyword] = ACTIONS(2073),
    [sym_define_start_keyword] = ACTIONS(2073),
    [sym_define_event_keyword] = ACTIONS(2073),
    [sym_define_program_keyword] = ACTIONS(2073),
    [sym_define_module_keyword] = ACTIONS(2073),
    [sym_char_keyword] = ACTIONS(2073),
    [sym_widechar_keyword] = ACTIONS(2073),
    [sym_integer_keyword] = ACTIONS(2073),
    [sym_sinteger_keyword] = ACTIONS(2073),
    [sym_long_keyword] = ACTIONS(2073),
    [sym_slong_keyword] = ACTIONS(2073),
    [sym_float_keyword] = ACTIONS(2073),
    [sym_double_keyword] = ACTIONS(2073),
    [sym_variant_keyword] = ACTIONS(2073),
    [sym_variantarray_keyword] = ACTIONS(2073),
    [sym_local_var_keyword] = ACTIONS(2073),
    [sym_stack_var_keyword] = ACTIONS(2073),
    [sym_constant_keyword] = ACTIONS(2073),
    [sym_volatile_keyword] = ACTIONS(2073),
    [sym_non_volatile_keyword] = ACTIONS(2073),
    [sym_persistent_keyword] = ACTIONS(2073),
    [sym_dev_keyword] = ACTIONS(2073),
    [sym_devlev_keyword] = ACTIONS(2073),
    [sym_devchan_keyword] = ACTIONS(2073),
    [sym_if_keyword] = ACTIONS(2073),
    [sym_switch_keyword] = ACTIONS(2073),
    [sym_case_keyword] = ACTIONS(2073),
    [sym_default_keyword] = ACTIONS(2073),
    [sym_while_keyword] = ACTIONS(2073),
    [sym_for_keyword] = ACTIONS(2073),
    [sym_break_keyword] = ACTIONS(2073),
    [sym_continue_keyword] = ACTIONS(2073),
    [sym_return_keyword] = ACTIONS(2073),
    [sym_select_keyword] = ACTIONS(2073),
    [sym_active_keyword] = ACTIONS(2073),
    [sym_struct_keyword] = ACTIONS(2073),
    [sym_structure_keyword] = ACTIONS(2073),
    [sym_button_event_keyword] = ACTIONS(2073),
    [sym_channel_event_keyword] = ACTIONS(2073),
    [sym_level_event_keyword] = ACTIONS(2073),
    [sym_data_event_keyword] = ACTIONS(2073),
    [sym_timeline_event_keyword] = ACTIONS(2073),
    [sym_custom_event_keyword] = ACTIONS(2073),
    [sym_push_keyword] = ACTIONS(2073),
    [sym_release_keyword] = ACTIONS(2073),
    [aux_sym_on_keyword_token1] = ACTIONS(2073),
    [aux_sym_off_keyword_token1] = ACTIONS(2073),
    [sym_send_level_keyword] = ACTIONS(2073),
    [sym_send_string_keyword] = ACTIONS(2073),
    [sym_send_command_keyword] = ACTIONS(2073),
    [sym_clear_buffer_keyword] = ACTIONS(2073),
    [sym_create_buffer_keyword] = ACTIONS(2073),
    [sym_create_multi_buffer_keyword] = ACTIONS(2073),
    [sym_call_keyword] = ACTIONS(2073),
    [sym_system_call_keyword] = ACTIONS(2073),
    [sym_devchan_to_keyword] = ACTIONS(2073),
    [sym_devchan_min_to_keyword] = ACTIONS(2073),
    [sym_devchan_total_off_keyword] = ACTIONS(2073),
    [sym_devchan_pulse_keyword] = ACTIONS(2073),
    [sym_wait_keyword] = ACTIONS(2073),
    [sym_cancel_wait_keyword] = ACTIONS(2073),
    [sym_wait_until_keyword] = ACTIONS(2073),
    [sym_cancel_wait_until_keyword] = ACTIONS(2073),
    [sym_cancel_all_wait_keyword] = ACTIONS(2073),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2073),
    [sym_not] = ACTIONS(2073),
    [sym_bnot] = ACTIONS(2073),
    [sym___file__] = ACTIONS(2073),
    [sym___date__] = ACTIONS(2073),
    [sym___ldate__] = ACTIONS(2073),
    [sym___time__] = ACTIONS(2073),
    [sym___line__] = ACTIONS(2073),
    [sym___name__] = ACTIONS(2073),
    [sym___netlinx__] = ACTIONS(2073),
    [sym_netlinx_axi_version] = ACTIONS(2073),
    [sym_url_flg_tcp] = ACTIONS(2073),
    [sym_url_flg_accinfopresent] = ACTIONS(2073),
    [sym_url_flg_temp] = ACTIONS(2073),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2073),
    [sym_url_flg_stat_mask] = ACTIONS(2073),
    [sym_url_flg_stat_lookup] = ACTIONS(2073),
    [sym_url_flg_stat_connecting] = ACTIONS(2073),
    [sym_url_flg_stat_waiting] = ACTIONS(2073),
    [sym_url_flg_stat_connected] = ACTIONS(2073),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2073),
    [sym_first_virtual_device] = ACTIONS(2073),
    [sym_first_local_port] = ACTIONS(2073),
    [sym_dynamic_virtual_device] = ACTIONS(2073),
    [sym_timeline_once] = ACTIONS(2073),
    [sym_timeline_repeat] = ACTIONS(2073),
    [sym_timeline_absolute] = ACTIONS(2073),
    [sym_timeline_relative] = ACTIONS(2073),
    [sym_do_push_timed_infinite] = ACTIONS(2073),
    [sym_source_type_no_address] = ACTIONS(2073),
    [sym_source_type_neuron_id] = ACTIONS(2073),
    [sym_source_type_ip_address] = ACTIONS(2073),
    [sym_source_type_axlink] = ACTIONS(2073),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2073),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2073),
    [sym_source_type_ip_socket_address] = ACTIONS(2073),
    [sym_source_type_rs232] = ACTIONS(2073),
    [sym_source_type_internal] = ACTIONS(2073),
    [sym_source_type_mac_address] = ACTIONS(2073),
    [sym_source_type_ipv4_port] = ACTIONS(2073),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2073),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2073),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2073),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2073),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2073),
    [sym_true] = ACTIONS(2073),
    [sym_false] = ACTIONS(2073),
    [sym_file_read_only] = ACTIONS(2073),
    [sym_file_rw_new] = ACTIONS(2073),
    [sym_file_rw_append] = ACTIONS(2073),
    [sym_ip_tcp] = ACTIONS(2073),
    [sym_ip_udp] = ACTIONS(2073),
    [sym_ip_udp_2way] = ACTIONS(2073),
    [sym_xml_encode_types] = ACTIONS(2073),
    [sym_xml_encode_char_as_list] = ACTIONS(2073),
    [sym_xml_encode_le] = ACTIONS(2073),
    [sym_xml_decode_types] = ACTIONS(2073),
    [sym_xml_decode_no_preserve] = ACTIONS(2073),
    [sym_duet_dev_type_display_device] = ACTIONS(2073),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2073),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2073),
    [sym_duet_dev_type_io_device] = ACTIONS(2073),
    [sym_duet_dev_type_relay_device] = ACTIONS(2073),
    [sym_duet_dev_type_ups] = ACTIONS(2073),
    [sym_duet_dev_type_amplifier] = ACTIONS(2073),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2073),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2073),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2073),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2073),
    [sym_duet_dev_type_camera] = ACTIONS(2073),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2073),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2073),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2073),
    [sym_duet_dev_type_dss] = ACTIONS(2073),
    [sym_duet_dev_type_dvr] = ACTIONS(2073),
    [sym_duet_dev_type_disc_device] = ACTIONS(2073),
    [sym_duet_dev_type_document_camera] = ACTIONS(2073),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2073),
    [sym_duet_dev_type_hvac] = ACTIONS(2073),
    [sym_duet_dev_type_keypad] = ACTIONS(2073),
    [sym_duet_dev_type_light] = ACTIONS(2073),
    [sym_duet_dev_type_monitor] = ACTIONS(2073),
    [sym_duet_dev_type_motor] = ACTIONS(2073),
    [sym_duet_dev_type_multi_window] = ACTIONS(2073),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2073),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2073),
    [sym_duet_dev_type_receiver] = ACTIONS(2073),
    [sym_duet_dev_type_security_system] = ACTIONS(2073),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2073),
    [sym_duet_dev_type_settop_box] = ACTIONS(2073),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2073),
    [sym_duet_dev_type_switcher] = ACTIONS(2073),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2073),
    [sym_duet_dev_type_tv] = ACTIONS(2073),
    [sym_duet_dev_type_utility] = ACTIONS(2073),
    [sym_duet_dev_type_vcr] = ACTIONS(2073),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2073),
    [sym_duet_dev_type_video_processor] = ACTIONS(2073),
    [sym_duet_dev_type_video_projector] = ACTIONS(2073),
    [sym_duet_dev_type_video_wall] = ACTIONS(2073),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2073),
    [sym_duet_dev_type_weather] = ACTIONS(2073),
    [sym_duet_dev_not_polled] = ACTIONS(2073),
    [sym_duet_dev_polled] = ACTIONS(2073),
    [sym_clkmgr_mode_network] = ACTIONS(2073),
    [sym_clkmgr_mode_standalone] = ACTIONS(2073),
    [sym_normal_standby] = ACTIONS(2073),
    [sym_normal_wake] = ACTIONS(2073),
    [sym_ip_multicast_ttl_option] = ACTIONS(2073),
    [sym_ip_ttl_subnet] = ACTIONS(2073),
    [sym_ip_ttl_site] = ACTIONS(2073),
    [sym_ip_ttl_region] = ACTIONS(2073),
    [sym_ip_ttl_continent] = ACTIONS(2073),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2073),
    [sym_ip_nodelay_on] = ACTIONS(2073),
    [sym_ip_nodelay_off] = ACTIONS(2073),
    [sym_amx_error] = ACTIONS(2073),
    [sym_amx_warning] = ACTIONS(2073),
    [sym_amx_info] = ACTIONS(2073),
    [sym_amx_debug] = ACTIONS(2073),
    [sym_netlinx_login_success] = ACTIONS(2073),
    [sym_netlinx_login_fail] = ACTIONS(2073),
    [sym_netlinx_logout] = ACTIONS(2073),
    [sym_null_str] = ACTIONS(2073),
    [sym_smtp_address] = ACTIONS(2073),
    [sym_smtp_port_number] = ACTIONS(2073),
    [sym_smtp_username] = ACTIONS(2073),
    [sym_smtp_password] = ACTIONS(2073),
    [sym_smtp_require_tls] = ACTIONS(2073),
    [sym_smtp_from] = ACTIONS(2073),
    [sym_smtp_tls_true] = ACTIONS(2073),
    [sym_smtp_tls_false] = ACTIONS(2073),
    [sym_listview_on_row_select_event] = ACTIONS(2073),
    [sym_data_structure_unknown] = ACTIONS(2073),
    [sym_data_structure_datafeed] = ACTIONS(2073),
    [sym_data_structure_datarecordset] = ACTIONS(2073),
    [sym_data_structure_datarecord] = ACTIONS(2073),
    [sym_data_structure_datafield] = ACTIONS(2073),
    [sym_data_max_content_field_count] = ACTIONS(2073),
    [sym_data_max_metadata_field_count] = ACTIONS(2073),
    [sym_data_max_id_length] = ACTIONS(2073),
    [sym_data_max_name_length] = ACTIONS(2073),
    [sym_data_max_description_length] = ACTIONS(2073),
    [sym_data_max_type_length] = ACTIONS(2073),
    [sym_data_max_format_length] = ACTIONS(2073),
    [sym_data_max_source_length] = ACTIONS(2073),
    [sym_data_max_value_length] = ACTIONS(2073),
    [sym_data_max_label_length] = ACTIONS(2073),
    [sym_data_max_url_length] = ACTIONS(2073),
    [sym_max_last_login_info_length] = ACTIONS(2073),
    [sym_data_type_unknown] = ACTIONS(2073),
    [sym_data_type_string] = ACTIONS(2073),
    [sym_data_type_datetime] = ACTIONS(2073),
    [sym_data_type_date] = ACTIONS(2073),
    [sym_data_type_time] = ACTIONS(2073),
    [sym_data_type_image] = ACTIONS(2073),
    [sym_data_format_url] = ACTIONS(2073),
    [sym_data_format_phone] = ACTIONS(2073),
    [sym_data_format_email] = ACTIONS(2073),
    [sym_data_format_iso8601] = ACTIONS(2073),
    [sym_auditlog_priv] = ACTIONS(2073),
    [sym_configuration_priv] = ACTIONS(2073),
    [sym_deviceconfig_priv] = ACTIONS(2073),
    [sym_ftp_priv] = ACTIONS(2073),
    [sym_http_priv] = ACTIONS(2073),
    [sym_networkconfig_priv] = ACTIONS(2073),
    [sym_programport_priv] = ACTIONS(2073),
    [sym_remoteui_priv] = ACTIONS(2073),
    [sym_securitycontrol_priv] = ACTIONS(2073),
    [sym_softwaremanagement_priv] = ACTIONS(2073),
    [sym_terminal_priv] = ACTIONS(2073),
    [sym_tpadmin_priv] = ACTIONS(2073),
    [sym_user1_priv] = ACTIONS(2073),
    [sym_user2_priv] = ACTIONS(2073),
    [sym_user3_priv] = ACTIONS(2073),
    [sym_user4_priv] = ACTIONS(2073),
    [sym_usermanagement_priv] = ACTIONS(2073),
    [sym_valid_account] = ACTIONS(2073),
    [sym_err_name_invalid] = ACTIONS(2073),
    [sym_err_password_invalid] = ACTIONS(2073),
    [sym_err_invalid_account] = ACTIONS(2073),
    [sym_err_no_authorization] = ACTIONS(2073),
    [sym_err_invalid_parameter] = ACTIONS(2073),
    [sym_err_user_locked_out] = ACTIONS(2073),
    [sym_err_user_account_expired] = ACTIONS(2073),
    [sym_err_general_error] = ACTIONS(2073),
    [sym_err_authentication_error] = ACTIONS(2073),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2073),
    [sym_tls_validate_certificate] = ACTIONS(2073),
    [sym_tdata] = ACTIONS(2073),
    [sym_tchannel] = ACTIONS(2073),
    [sym_tlevel] = ACTIONS(2073),
    [sym_tbutton] = ACTIONS(2073),
    [sym_ttimeline] = ACTIONS(2073),
    [sym_tcustom] = ACTIONS(2073),
    [sym_url_struct] = ACTIONS(2073),
    [sym_dns_struct] = ACTIONS(2073),
    [sym_ip_address_struct] = ACTIONS(2073),
    [sym_dev_info_struct] = ACTIONS(2073),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2073),
    [sym_data_feed] = ACTIONS(2073),
    [sym_data_field] = ACTIONS(2073),
    [sym_data_record] = ACTIONS(2073),
    [sym_wc_data_feed] = ACTIONS(2073),
    [sym_wc_data_field] = ACTIONS(2073),
    [sym_wc_data_record] = ACTIONS(2073),
    [sym_last_login_info] = ACTIONS(2073),
    [sym____reserved___] = ACTIONS(2073),
    [sym_date] = ACTIONS(2073),
    [sym_day] = ACTIONS(2073),
    [sym_time] = ACTIONS(2073),
    [sym_ldate] = ACTIONS(2073),
    [sym_push_device] = ACTIONS(2073),
    [sym_push_channel] = ACTIONS(2073),
    [sym_push_devchan] = ACTIONS(2073),
    [sym_release_device] = ACTIONS(2073),
    [sym_release_channel] = ACTIONS(2073),
    [sym_release_devchan] = ACTIONS(2073),
    [sym_master_slot] = ACTIONS(2073),
    [sym_get_pulse_time] = ACTIONS(2073),
    [sym_get_timer] = ACTIONS(2073),
    [sym_master_sn] = ACTIONS(2073),
    [sym_system_number] = ACTIONS(2073),
    [sym_button] = ACTIONS(2073),
    [sym_dv_channel] = ACTIONS(2073),
    [sym_data] = ACTIONS(2073),
    [sym_level] = ACTIONS(2073),
    [sym_timeline] = ACTIONS(2073),
    [sym_channel] = ACTIONS(2073),
    [sym_custom] = ACTIONS(2073),
    [sym_atoi] = ACTIONS(2073),
    [sym_atol] = ACTIONS(2073),
    [sym_atof] = ACTIONS(2073),
    [sym_itoa] = ACTIONS(2073),
    [sym_itohex] = ACTIONS(2073),
    [sym_hextoi] = ACTIONS(2073),
    [sym_ftoa] = ACTIONS(2073),
    [sym_find_string] = ACTIONS(2073),
    [sym_left_string] = ACTIONS(2073),
    [sym_right_string] = ACTIONS(2073),
    [sym_mid_string] = ACTIONS(2073),
    [sym_remove_string] = ACTIONS(2073),
    [sym_get_buffer_string] = ACTIONS(2073),
    [sym_get_buffer_char] = ACTIONS(2073),
    [sym_get_multi_buffer_string] = ACTIONS(2073),
    [sym_length_string] = ACTIONS(2073),
    [sym_max_length_string] = ACTIONS(2073),
    [sym_set_length_string] = ACTIONS(2073),
    [sym_compare_string] = ACTIONS(2073),
    [sym_upper_string] = ACTIONS(2073),
    [sym_lower_string] = ACTIONS(2073),
    [sym_get_last] = ACTIONS(2073),
    [sym_date_to_day] = ACTIONS(2073),
    [sym_date_to_month] = ACTIONS(2073),
    [sym_date_to_year] = ACTIONS(2073),
    [sym_time_to_hour] = ACTIONS(2073),
    [sym_time_to_minute] = ACTIONS(2073),
    [sym_time_to_second] = ACTIONS(2073),
    [sym_day_of_week] = ACTIONS(2073),
    [sym_set_pulse_time] = ACTIONS(2073),
    [sym_set_timer] = ACTIONS(2073),
    [sym_file_open] = ACTIONS(2073),
    [sym_file_close] = ACTIONS(2073),
    [sym_file_read] = ACTIONS(2073),
    [sym_file_read_line] = ACTIONS(2073),
    [sym_file_write] = ACTIONS(2073),
    [sym_file_write_line] = ACTIONS(2073),
    [sym_file_copy] = ACTIONS(2073),
    [sym_file_delete] = ACTIONS(2073),
    [sym_file_rename] = ACTIONS(2073),
    [sym_file_dir] = ACTIONS(2073),
    [sym_file_getdir] = ACTIONS(2073),
    [sym_file_setdir] = ACTIONS(2073),
    [sym_file_create_dir] = ACTIONS(2073),
    [sym_file_remove_dir] = ACTIONS(2073),
    [sym_file_seek] = ACTIONS(2073),
    [sym_length_array] = ACTIONS(2073),
    [sym_max_length_array] = ACTIONS(2073),
    [sym_set_length_array] = ACTIONS(2073),
    [sym_ip_client_open] = ACTIONS(2073),
    [sym_ip_client_close] = ACTIONS(2073),
    [sym_ip_server_open] = ACTIONS(2073),
    [sym_ip_server_close] = ACTIONS(2073),
    [sym_ip_bound_client_open] = ACTIONS(2073),
    [sym_ip_mc_server_open] = ACTIONS(2073),
    [sym_ip_set_option] = ACTIONS(2073),
    [sym_ssh_client_open] = ACTIONS(2073),
    [sym_ssh_client_close] = ACTIONS(2073),
    [sym_tls_client_open] = ACTIONS(2073),
    [sym_tls_client_close] = ACTIONS(2073),
    [sym_do_push] = ACTIONS(2073),
    [sym_do_push_timed] = ACTIONS(2073),
    [sym_do_release] = ACTIONS(2073),
    [sym_device_id] = ACTIONS(2073),
    [sym_device_info] = ACTIONS(2073),
    [sym_device_id_string] = ACTIONS(2073),
    [sym_device_standby] = ACTIONS(2073),
    [sym_device_wake] = ACTIONS(2073),
    [sym_timeline_create] = ACTIONS(2073),
    [sym_timeline_kill] = ACTIONS(2073),
    [sym_timeline_reload] = ACTIONS(2073),
    [sym_timeline_pause] = ACTIONS(2073),
    [sym_timeline_restart] = ACTIONS(2073),
    [sym_timeline_set] = ACTIONS(2073),
    [sym_timeline_get] = ACTIONS(2073),
    [sym_timeline_active] = ACTIONS(2073),
    [sym_timeline_dynamic_id] = ACTIONS(2073),
    [sym_variable_to_string] = ACTIONS(2073),
    [sym_string_to_variable] = ACTIONS(2073),
    [sym_length_variable_to_string] = ACTIONS(2073),
    [sym_type_cast] = ACTIONS(2073),
    [sym_variable_to_xml] = ACTIONS(2073),
    [sym_xml_to_variable] = ACTIONS(2073),
    [sym_length_variable_to_xml] = ACTIONS(2073),
    [sym_redirect_string] = ACTIONS(2073),
    [sym_random_number] = ACTIONS(2073),
    [sym_get_unique_id] = ACTIONS(2073),
    [sym_get_system_number] = ACTIONS(2073),
    [sym_set_system_number] = ACTIONS(2073),
    [sym_get_serial_number] = ACTIONS(2073),
    [sym_get_master_build] = ACTIONS(2073),
    [sym_set_validation_code] = ACTIONS(2073),
    [sym_reboot] = ACTIONS(2073),
    [sym_load_duet_module] = ACTIONS(2073),
    [sym_unload_duet_module] = ACTIONS(2073),
    [sym_duet_mem_size_set] = ACTIONS(2073),
    [sym_duet_mem_size_get] = ACTIONS(2073),
    [sym_dynamic_application_device] = ACTIONS(2073),
    [sym_dynamic_polled_port] = ACTIONS(2073),
    [sym_static_port_binding] = ACTIONS(2073),
    [sym_static_ip_binding] = ACTIONS(2073),
    [sym_get_dns_list] = ACTIONS(2073),
    [sym_set_dns_list] = ACTIONS(2073),
    [sym_get_ip_address] = ACTIONS(2073),
    [sym_set_ip_address] = ACTIONS(2073),
    [sym_get_url_list] = ACTIONS(2073),
    [sym_add_url_entry] = ACTIONS(2073),
    [sym_delete_url_entry] = ACTIONS(2073),
    [sym_abs_value] = ACTIONS(2073),
    [sym_max_value] = ACTIONS(2073),
    [sym_min_value] = ACTIONS(2073),
    [sym_exp_value] = ACTIONS(2073),
    [sym_log_value] = ACTIONS(2073),
    [sym_log10_value] = ACTIONS(2073),
    [sym_power_value] = ACTIONS(2073),
    [sym_sqrt_value] = ACTIONS(2073),
    [sym_raw_be] = ACTIONS(2073),
    [sym_raw_le] = ACTIONS(2073),
    [sym_internal_queue_size_set] = ACTIONS(2073),
    [sym_internal_queue_size_get] = ACTIONS(2073),
    [sym_internal_threshold_set] = ACTIONS(2073),
    [sym_internal_threshold_get] = ACTIONS(2073),
    [sym_rebuilt_event] = ACTIONS(2073),
    [sym_do_custom_event] = ACTIONS(2073),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2073),
    [sym_clkmgr_set_clk_source] = ACTIONS(2073),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2073),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2073),
    [sym_clkmgr_get_timezone] = ACTIONS(2073),
    [sym_clkmgr_set_timezone] = ACTIONS(2073),
    [sym_clkmgr_get_resync_period] = ACTIONS(2073),
    [sym_clkmgr_set_resync_period] = ACTIONS(2073),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2073),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2073),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2073),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2073),
    [sym_clkmgr_get_timeservers] = ACTIONS(2073),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2073),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2073),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2073),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2073),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2073),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2073),
    [sym_set_log_level] = ACTIONS(2073),
    [sym_get_log_level] = ACTIONS(2073),
    [sym_amx_log] = ACTIONS(2073),
    [sym_smtp_server_config_set] = ACTIONS(2073),
    [sym_smtp_server_config_get] = ACTIONS(2073),
    [sym_smtp_send] = ACTIONS(2073),
    [sym_validate_netlinx_account] = ACTIONS(2073),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2073),
    [sym_authenticate_certificate] = ACTIONS(2073),
    [sym_audit_netlinx_session_event] = ACTIONS(2073),
    [sym_audit_netlinx_generic_event] = ACTIONS(2073),
    [sym_get_available_flash_disk_space] = ACTIONS(2073),
    [sym_get_max_flash_disk_space] = ACTIONS(2073),
    [sym_data_create_feed] = ACTIONS(2073),
    [sym_data_delete_feed] = ACTIONS(2073),
    [sym_data_publish_feed] = ACTIONS(2073),
    [sym_data_get_published_feed] = ACTIONS(2073),
    [sym_data_add_record] = ACTIONS(2073),
    [sym_data_get_event_record] = ACTIONS(2073),
    [sym__wc_data_create_feed] = ACTIONS(2073),
    [sym__wc_data_add_record] = ACTIONS(2073),
    [sym__wc_data_get_event_record] = ACTIONS(2073),
    [sym_format] = ACTIONS(2073),
    [sym_astro_clock] = ACTIONS(2073),
    [sym_set_outdoor_temperature] = ACTIONS(2073),
    [sym_set_virtual_level_count] = ACTIONS(2073),
    [sym_set_virtual_channel_count] = ACTIONS(2073),
    [sym_set_virtual_port_count] = ACTIONS(2073),
    [sym__wc_encode_int] = ACTIONS(2073),
    [sym__wc_decode_int] = ACTIONS(2073),
    [sym__wc_int] = ACTIONS(2073),
    [sym__wc_to_ch_int] = ACTIONS(2073),
    [sym__ch_to_wc_int] = ACTIONS(2073),
    [sym__wc_find_string_int] = ACTIONS(2073),
    [sym__wc_left_string_int] = ACTIONS(2073),
    [sym__wc_lower_string_int] = ACTIONS(2073),
    [sym__wc_mid_string_int] = ACTIONS(2073),
    [sym__wc_remove_string_int] = ACTIONS(2073),
    [sym__wc_right_string_int] = ACTIONS(2073),
    [sym__wc_upper_string_int] = ACTIONS(2073),
    [sym__wc_compare_string_int] = ACTIONS(2073),
    [sym__wc_get_buffer_char_int] = ACTIONS(2073),
    [sym__wc_get_buffer_string_int] = ACTIONS(2073),
    [sym__wc_concat_string_int] = ACTIONS(2073),
    [sym__wc_file_open_int] = ACTIONS(2073),
    [sym__wc_file_close_int] = ACTIONS(2073),
    [sym__wc_file_read_int] = ACTIONS(2073),
    [sym__wc_file_read_line_int] = ACTIONS(2073),
    [sym__wc_file_write_int] = ACTIONS(2073),
    [sym__wc_file_write_line_int] = ACTIONS(2073),
    [aux_sym_preproc_include_token1] = ACTIONS(2073),
    [aux_sym_preproc_define_token1] = ACTIONS(2075),
    [aux_sym_preproc_warn_token1] = ACTIONS(2075),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2075),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2075),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2075),
    [anon_sym_LBRACK] = ACTIONS(2075),
    [anon_sym_LBRACE] = ACTIONS(2075),
    [anon_sym_LPAREN] = ACTIONS(2073),
    [anon_sym_BANG] = ACTIONS(2075),
    [anon_sym_TILDE] = ACTIONS(2075),
    [anon_sym_DASH] = ACTIONS(2073),
    [anon_sym_PLUS] = ACTIONS(2073),
    [anon_sym_DASH_DASH] = ACTIONS(2075),
    [anon_sym_PLUS_PLUS] = ACTIONS(2075),
    [anon_sym_DQUOTE] = ACTIONS(2075),
    [anon_sym_SQUOTE] = ACTIONS(2075),
    [sym_number_literal] = ACTIONS(2075),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(449)] = {
    [ts_builtin_sym_end] = ACTIONS(1885),
    [sym_identifier] = ACTIONS(1883),
    [sym_program_name_keyword] = ACTIONS(1883),
    [sym_module_name_keyword] = ACTIONS(1883),
    [sym_define_device_keyword] = ACTIONS(1883),
    [sym_define_combine_keyword] = ACTIONS(1883),
    [sym_define_constant_keyword] = ACTIONS(1883),
    [sym_define_type_keyword] = ACTIONS(1883),
    [sym_define_variable_keyword] = ACTIONS(1883),
    [sym_define_system_variable_keyword] = ACTIONS(1883),
    [sym_define_connect_level_keyword] = ACTIONS(1883),
    [sym_define_latching_keyword] = ACTIONS(1883),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1883),
    [sym_define_toggling_keyword] = ACTIONS(1883),
    [sym_define_call_keyword] = ACTIONS(1883),
    [sym_define_function_keyword] = ACTIONS(1883),
    [sym_define_library_function_keyword] = ACTIONS(1883),
    [sym_define_start_keyword] = ACTIONS(1883),
    [sym_define_event_keyword] = ACTIONS(1883),
    [sym_define_program_keyword] = ACTIONS(1883),
    [sym_define_module_keyword] = ACTIONS(1883),
    [sym_char_keyword] = ACTIONS(1883),
    [sym_widechar_keyword] = ACTIONS(1883),
    [sym_integer_keyword] = ACTIONS(1883),
    [sym_sinteger_keyword] = ACTIONS(1883),
    [sym_long_keyword] = ACTIONS(1883),
    [sym_slong_keyword] = ACTIONS(1883),
    [sym_float_keyword] = ACTIONS(1883),
    [sym_double_keyword] = ACTIONS(1883),
    [sym_variant_keyword] = ACTIONS(1883),
    [sym_variantarray_keyword] = ACTIONS(1883),
    [sym_local_var_keyword] = ACTIONS(1883),
    [sym_stack_var_keyword] = ACTIONS(1883),
    [sym_constant_keyword] = ACTIONS(1883),
    [sym_volatile_keyword] = ACTIONS(1883),
    [sym_non_volatile_keyword] = ACTIONS(1883),
    [sym_persistent_keyword] = ACTIONS(1883),
    [sym_dev_keyword] = ACTIONS(1883),
    [sym_devlev_keyword] = ACTIONS(1883),
    [sym_devchan_keyword] = ACTIONS(1883),
    [sym_if_keyword] = ACTIONS(1883),
    [sym_switch_keyword] = ACTIONS(1883),
    [sym_case_keyword] = ACTIONS(1883),
    [sym_default_keyword] = ACTIONS(1883),
    [sym_while_keyword] = ACTIONS(1883),
    [sym_for_keyword] = ACTIONS(1883),
    [sym_break_keyword] = ACTIONS(1883),
    [sym_continue_keyword] = ACTIONS(1883),
    [sym_return_keyword] = ACTIONS(1883),
    [sym_select_keyword] = ACTIONS(1883),
    [sym_active_keyword] = ACTIONS(1883),
    [sym_struct_keyword] = ACTIONS(1883),
    [sym_structure_keyword] = ACTIONS(1883),
    [sym_button_event_keyword] = ACTIONS(1883),
    [sym_channel_event_keyword] = ACTIONS(1883),
    [sym_level_event_keyword] = ACTIONS(1883),
    [sym_data_event_keyword] = ACTIONS(1883),
    [sym_timeline_event_keyword] = ACTIONS(1883),
    [sym_custom_event_keyword] = ACTIONS(1883),
    [sym_push_keyword] = ACTIONS(1883),
    [sym_release_keyword] = ACTIONS(1883),
    [aux_sym_on_keyword_token1] = ACTIONS(1883),
    [aux_sym_off_keyword_token1] = ACTIONS(1883),
    [sym_send_level_keyword] = ACTIONS(1883),
    [sym_send_string_keyword] = ACTIONS(1883),
    [sym_send_command_keyword] = ACTIONS(1883),
    [sym_clear_buffer_keyword] = ACTIONS(1883),
    [sym_create_buffer_keyword] = ACTIONS(1883),
    [sym_create_multi_buffer_keyword] = ACTIONS(1883),
    [sym_call_keyword] = ACTIONS(1883),
    [sym_system_call_keyword] = ACTIONS(1883),
    [sym_devchan_to_keyword] = ACTIONS(1883),
    [sym_devchan_min_to_keyword] = ACTIONS(1883),
    [sym_devchan_total_off_keyword] = ACTIONS(1883),
    [sym_devchan_pulse_keyword] = ACTIONS(1883),
    [sym_wait_keyword] = ACTIONS(1883),
    [sym_cancel_wait_keyword] = ACTIONS(1883),
    [sym_wait_until_keyword] = ACTIONS(1883),
    [sym_cancel_wait_until_keyword] = ACTIONS(1883),
    [sym_cancel_all_wait_keyword] = ACTIONS(1883),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1883),
    [sym_not] = ACTIONS(1883),
    [sym_bnot] = ACTIONS(1883),
    [sym___file__] = ACTIONS(1883),
    [sym___date__] = ACTIONS(1883),
    [sym___ldate__] = ACTIONS(1883),
    [sym___time__] = ACTIONS(1883),
    [sym___line__] = ACTIONS(1883),
    [sym___name__] = ACTIONS(1883),
    [sym___netlinx__] = ACTIONS(1883),
    [sym_netlinx_axi_version] = ACTIONS(1883),
    [sym_url_flg_tcp] = ACTIONS(1883),
    [sym_url_flg_accinfopresent] = ACTIONS(1883),
    [sym_url_flg_temp] = ACTIONS(1883),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1883),
    [sym_url_flg_stat_mask] = ACTIONS(1883),
    [sym_url_flg_stat_lookup] = ACTIONS(1883),
    [sym_url_flg_stat_connecting] = ACTIONS(1883),
    [sym_url_flg_stat_waiting] = ACTIONS(1883),
    [sym_url_flg_stat_connected] = ACTIONS(1883),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1883),
    [sym_first_virtual_device] = ACTIONS(1883),
    [sym_first_local_port] = ACTIONS(1883),
    [sym_dynamic_virtual_device] = ACTIONS(1883),
    [sym_timeline_once] = ACTIONS(1883),
    [sym_timeline_repeat] = ACTIONS(1883),
    [sym_timeline_absolute] = ACTIONS(1883),
    [sym_timeline_relative] = ACTIONS(1883),
    [sym_do_push_timed_infinite] = ACTIONS(1883),
    [sym_source_type_no_address] = ACTIONS(1883),
    [sym_source_type_neuron_id] = ACTIONS(1883),
    [sym_source_type_ip_address] = ACTIONS(1883),
    [sym_source_type_axlink] = ACTIONS(1883),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1883),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1883),
    [sym_source_type_ip_socket_address] = ACTIONS(1883),
    [sym_source_type_rs232] = ACTIONS(1883),
    [sym_source_type_internal] = ACTIONS(1883),
    [sym_source_type_mac_address] = ACTIONS(1883),
    [sym_source_type_ipv4_port] = ACTIONS(1883),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1883),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1883),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1883),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1883),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1883),
    [sym_true] = ACTIONS(1883),
    [sym_false] = ACTIONS(1883),
    [sym_file_read_only] = ACTIONS(1883),
    [sym_file_rw_new] = ACTIONS(1883),
    [sym_file_rw_append] = ACTIONS(1883),
    [sym_ip_tcp] = ACTIONS(1883),
    [sym_ip_udp] = ACTIONS(1883),
    [sym_ip_udp_2way] = ACTIONS(1883),
    [sym_xml_encode_types] = ACTIONS(1883),
    [sym_xml_encode_char_as_list] = ACTIONS(1883),
    [sym_xml_encode_le] = ACTIONS(1883),
    [sym_xml_decode_types] = ACTIONS(1883),
    [sym_xml_decode_no_preserve] = ACTIONS(1883),
    [sym_duet_dev_type_display_device] = ACTIONS(1883),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1883),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1883),
    [sym_duet_dev_type_io_device] = ACTIONS(1883),
    [sym_duet_dev_type_relay_device] = ACTIONS(1883),
    [sym_duet_dev_type_ups] = ACTIONS(1883),
    [sym_duet_dev_type_amplifier] = ACTIONS(1883),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1883),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1883),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1883),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1883),
    [sym_duet_dev_type_camera] = ACTIONS(1883),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1883),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1883),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1883),
    [sym_duet_dev_type_dss] = ACTIONS(1883),
    [sym_duet_dev_type_dvr] = ACTIONS(1883),
    [sym_duet_dev_type_disc_device] = ACTIONS(1883),
    [sym_duet_dev_type_document_camera] = ACTIONS(1883),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1883),
    [sym_duet_dev_type_hvac] = ACTIONS(1883),
    [sym_duet_dev_type_keypad] = ACTIONS(1883),
    [sym_duet_dev_type_light] = ACTIONS(1883),
    [sym_duet_dev_type_monitor] = ACTIONS(1883),
    [sym_duet_dev_type_motor] = ACTIONS(1883),
    [sym_duet_dev_type_multi_window] = ACTIONS(1883),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1883),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1883),
    [sym_duet_dev_type_receiver] = ACTIONS(1883),
    [sym_duet_dev_type_security_system] = ACTIONS(1883),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1883),
    [sym_duet_dev_type_settop_box] = ACTIONS(1883),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1883),
    [sym_duet_dev_type_switcher] = ACTIONS(1883),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1883),
    [sym_duet_dev_type_tv] = ACTIONS(1883),
    [sym_duet_dev_type_utility] = ACTIONS(1883),
    [sym_duet_dev_type_vcr] = ACTIONS(1883),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1883),
    [sym_duet_dev_type_video_processor] = ACTIONS(1883),
    [sym_duet_dev_type_video_projector] = ACTIONS(1883),
    [sym_duet_dev_type_video_wall] = ACTIONS(1883),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1883),
    [sym_duet_dev_type_weather] = ACTIONS(1883),
    [sym_duet_dev_not_polled] = ACTIONS(1883),
    [sym_duet_dev_polled] = ACTIONS(1883),
    [sym_clkmgr_mode_network] = ACTIONS(1883),
    [sym_clkmgr_mode_standalone] = ACTIONS(1883),
    [sym_normal_standby] = ACTIONS(1883),
    [sym_normal_wake] = ACTIONS(1883),
    [sym_ip_multicast_ttl_option] = ACTIONS(1883),
    [sym_ip_ttl_subnet] = ACTIONS(1883),
    [sym_ip_ttl_site] = ACTIONS(1883),
    [sym_ip_ttl_region] = ACTIONS(1883),
    [sym_ip_ttl_continent] = ACTIONS(1883),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1883),
    [sym_ip_nodelay_on] = ACTIONS(1883),
    [sym_ip_nodelay_off] = ACTIONS(1883),
    [sym_amx_error] = ACTIONS(1883),
    [sym_amx_warning] = ACTIONS(1883),
    [sym_amx_info] = ACTIONS(1883),
    [sym_amx_debug] = ACTIONS(1883),
    [sym_netlinx_login_success] = ACTIONS(1883),
    [sym_netlinx_login_fail] = ACTIONS(1883),
    [sym_netlinx_logout] = ACTIONS(1883),
    [sym_null_str] = ACTIONS(1883),
    [sym_smtp_address] = ACTIONS(1883),
    [sym_smtp_port_number] = ACTIONS(1883),
    [sym_smtp_username] = ACTIONS(1883),
    [sym_smtp_password] = ACTIONS(1883),
    [sym_smtp_require_tls] = ACTIONS(1883),
    [sym_smtp_from] = ACTIONS(1883),
    [sym_smtp_tls_true] = ACTIONS(1883),
    [sym_smtp_tls_false] = ACTIONS(1883),
    [sym_listview_on_row_select_event] = ACTIONS(1883),
    [sym_data_structure_unknown] = ACTIONS(1883),
    [sym_data_structure_datafeed] = ACTIONS(1883),
    [sym_data_structure_datarecordset] = ACTIONS(1883),
    [sym_data_structure_datarecord] = ACTIONS(1883),
    [sym_data_structure_datafield] = ACTIONS(1883),
    [sym_data_max_content_field_count] = ACTIONS(1883),
    [sym_data_max_metadata_field_count] = ACTIONS(1883),
    [sym_data_max_id_length] = ACTIONS(1883),
    [sym_data_max_name_length] = ACTIONS(1883),
    [sym_data_max_description_length] = ACTIONS(1883),
    [sym_data_max_type_length] = ACTIONS(1883),
    [sym_data_max_format_length] = ACTIONS(1883),
    [sym_data_max_source_length] = ACTIONS(1883),
    [sym_data_max_value_length] = ACTIONS(1883),
    [sym_data_max_label_length] = ACTIONS(1883),
    [sym_data_max_url_length] = ACTIONS(1883),
    [sym_max_last_login_info_length] = ACTIONS(1883),
    [sym_data_type_unknown] = ACTIONS(1883),
    [sym_data_type_string] = ACTIONS(1883),
    [sym_data_type_datetime] = ACTIONS(1883),
    [sym_data_type_date] = ACTIONS(1883),
    [sym_data_type_time] = ACTIONS(1883),
    [sym_data_type_image] = ACTIONS(1883),
    [sym_data_format_url] = ACTIONS(1883),
    [sym_data_format_phone] = ACTIONS(1883),
    [sym_data_format_email] = ACTIONS(1883),
    [sym_data_format_iso8601] = ACTIONS(1883),
    [sym_auditlog_priv] = ACTIONS(1883),
    [sym_configuration_priv] = ACTIONS(1883),
    [sym_deviceconfig_priv] = ACTIONS(1883),
    [sym_ftp_priv] = ACTIONS(1883),
    [sym_http_priv] = ACTIONS(1883),
    [sym_networkconfig_priv] = ACTIONS(1883),
    [sym_programport_priv] = ACTIONS(1883),
    [sym_remoteui_priv] = ACTIONS(1883),
    [sym_securitycontrol_priv] = ACTIONS(1883),
    [sym_softwaremanagement_priv] = ACTIONS(1883),
    [sym_terminal_priv] = ACTIONS(1883),
    [sym_tpadmin_priv] = ACTIONS(1883),
    [sym_user1_priv] = ACTIONS(1883),
    [sym_user2_priv] = ACTIONS(1883),
    [sym_user3_priv] = ACTIONS(1883),
    [sym_user4_priv] = ACTIONS(1883),
    [sym_usermanagement_priv] = ACTIONS(1883),
    [sym_valid_account] = ACTIONS(1883),
    [sym_err_name_invalid] = ACTIONS(1883),
    [sym_err_password_invalid] = ACTIONS(1883),
    [sym_err_invalid_account] = ACTIONS(1883),
    [sym_err_no_authorization] = ACTIONS(1883),
    [sym_err_invalid_parameter] = ACTIONS(1883),
    [sym_err_user_locked_out] = ACTIONS(1883),
    [sym_err_user_account_expired] = ACTIONS(1883),
    [sym_err_general_error] = ACTIONS(1883),
    [sym_err_authentication_error] = ACTIONS(1883),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1883),
    [sym_tls_validate_certificate] = ACTIONS(1883),
    [sym_tdata] = ACTIONS(1883),
    [sym_tchannel] = ACTIONS(1883),
    [sym_tlevel] = ACTIONS(1883),
    [sym_tbutton] = ACTIONS(1883),
    [sym_ttimeline] = ACTIONS(1883),
    [sym_tcustom] = ACTIONS(1883),
    [sym_url_struct] = ACTIONS(1883),
    [sym_dns_struct] = ACTIONS(1883),
    [sym_ip_address_struct] = ACTIONS(1883),
    [sym_dev_info_struct] = ACTIONS(1883),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1883),
    [sym_data_feed] = ACTIONS(1883),
    [sym_data_field] = ACTIONS(1883),
    [sym_data_record] = ACTIONS(1883),
    [sym_wc_data_feed] = ACTIONS(1883),
    [sym_wc_data_field] = ACTIONS(1883),
    [sym_wc_data_record] = ACTIONS(1883),
    [sym_last_login_info] = ACTIONS(1883),
    [sym____reserved___] = ACTIONS(1883),
    [sym_date] = ACTIONS(1883),
    [sym_day] = ACTIONS(1883),
    [sym_time] = ACTIONS(1883),
    [sym_ldate] = ACTIONS(1883),
    [sym_push_device] = ACTIONS(1883),
    [sym_push_channel] = ACTIONS(1883),
    [sym_push_devchan] = ACTIONS(1883),
    [sym_release_device] = ACTIONS(1883),
    [sym_release_channel] = ACTIONS(1883),
    [sym_release_devchan] = ACTIONS(1883),
    [sym_master_slot] = ACTIONS(1883),
    [sym_get_pulse_time] = ACTIONS(1883),
    [sym_get_timer] = ACTIONS(1883),
    [sym_master_sn] = ACTIONS(1883),
    [sym_system_number] = ACTIONS(1883),
    [sym_button] = ACTIONS(1883),
    [sym_dv_channel] = ACTIONS(1883),
    [sym_data] = ACTIONS(1883),
    [sym_level] = ACTIONS(1883),
    [sym_timeline] = ACTIONS(1883),
    [sym_channel] = ACTIONS(1883),
    [sym_custom] = ACTIONS(1883),
    [sym_atoi] = ACTIONS(1883),
    [sym_atol] = ACTIONS(1883),
    [sym_atof] = ACTIONS(1883),
    [sym_itoa] = ACTIONS(1883),
    [sym_itohex] = ACTIONS(1883),
    [sym_hextoi] = ACTIONS(1883),
    [sym_ftoa] = ACTIONS(1883),
    [sym_find_string] = ACTIONS(1883),
    [sym_left_string] = ACTIONS(1883),
    [sym_right_string] = ACTIONS(1883),
    [sym_mid_string] = ACTIONS(1883),
    [sym_remove_string] = ACTIONS(1883),
    [sym_get_buffer_string] = ACTIONS(1883),
    [sym_get_buffer_char] = ACTIONS(1883),
    [sym_get_multi_buffer_string] = ACTIONS(1883),
    [sym_length_string] = ACTIONS(1883),
    [sym_max_length_string] = ACTIONS(1883),
    [sym_set_length_string] = ACTIONS(1883),
    [sym_compare_string] = ACTIONS(1883),
    [sym_upper_string] = ACTIONS(1883),
    [sym_lower_string] = ACTIONS(1883),
    [sym_get_last] = ACTIONS(1883),
    [sym_date_to_day] = ACTIONS(1883),
    [sym_date_to_month] = ACTIONS(1883),
    [sym_date_to_year] = ACTIONS(1883),
    [sym_time_to_hour] = ACTIONS(1883),
    [sym_time_to_minute] = ACTIONS(1883),
    [sym_time_to_second] = ACTIONS(1883),
    [sym_day_of_week] = ACTIONS(1883),
    [sym_set_pulse_time] = ACTIONS(1883),
    [sym_set_timer] = ACTIONS(1883),
    [sym_file_open] = ACTIONS(1883),
    [sym_file_close] = ACTIONS(1883),
    [sym_file_read] = ACTIONS(1883),
    [sym_file_read_line] = ACTIONS(1883),
    [sym_file_write] = ACTIONS(1883),
    [sym_file_write_line] = ACTIONS(1883),
    [sym_file_copy] = ACTIONS(1883),
    [sym_file_delete] = ACTIONS(1883),
    [sym_file_rename] = ACTIONS(1883),
    [sym_file_dir] = ACTIONS(1883),
    [sym_file_getdir] = ACTIONS(1883),
    [sym_file_setdir] = ACTIONS(1883),
    [sym_file_create_dir] = ACTIONS(1883),
    [sym_file_remove_dir] = ACTIONS(1883),
    [sym_file_seek] = ACTIONS(1883),
    [sym_length_array] = ACTIONS(1883),
    [sym_max_length_array] = ACTIONS(1883),
    [sym_set_length_array] = ACTIONS(1883),
    [sym_ip_client_open] = ACTIONS(1883),
    [sym_ip_client_close] = ACTIONS(1883),
    [sym_ip_server_open] = ACTIONS(1883),
    [sym_ip_server_close] = ACTIONS(1883),
    [sym_ip_bound_client_open] = ACTIONS(1883),
    [sym_ip_mc_server_open] = ACTIONS(1883),
    [sym_ip_set_option] = ACTIONS(1883),
    [sym_ssh_client_open] = ACTIONS(1883),
    [sym_ssh_client_close] = ACTIONS(1883),
    [sym_tls_client_open] = ACTIONS(1883),
    [sym_tls_client_close] = ACTIONS(1883),
    [sym_do_push] = ACTIONS(1883),
    [sym_do_push_timed] = ACTIONS(1883),
    [sym_do_release] = ACTIONS(1883),
    [sym_device_id] = ACTIONS(1883),
    [sym_device_info] = ACTIONS(1883),
    [sym_device_id_string] = ACTIONS(1883),
    [sym_device_standby] = ACTIONS(1883),
    [sym_device_wake] = ACTIONS(1883),
    [sym_timeline_create] = ACTIONS(1883),
    [sym_timeline_kill] = ACTIONS(1883),
    [sym_timeline_reload] = ACTIONS(1883),
    [sym_timeline_pause] = ACTIONS(1883),
    [sym_timeline_restart] = ACTIONS(1883),
    [sym_timeline_set] = ACTIONS(1883),
    [sym_timeline_get] = ACTIONS(1883),
    [sym_timeline_active] = ACTIONS(1883),
    [sym_timeline_dynamic_id] = ACTIONS(1883),
    [sym_variable_to_string] = ACTIONS(1883),
    [sym_string_to_variable] = ACTIONS(1883),
    [sym_length_variable_to_string] = ACTIONS(1883),
    [sym_type_cast] = ACTIONS(1883),
    [sym_variable_to_xml] = ACTIONS(1883),
    [sym_xml_to_variable] = ACTIONS(1883),
    [sym_length_variable_to_xml] = ACTIONS(1883),
    [sym_redirect_string] = ACTIONS(1883),
    [sym_random_number] = ACTIONS(1883),
    [sym_get_unique_id] = ACTIONS(1883),
    [sym_get_system_number] = ACTIONS(1883),
    [sym_set_system_number] = ACTIONS(1883),
    [sym_get_serial_number] = ACTIONS(1883),
    [sym_get_master_build] = ACTIONS(1883),
    [sym_set_validation_code] = ACTIONS(1883),
    [sym_reboot] = ACTIONS(1883),
    [sym_load_duet_module] = ACTIONS(1883),
    [sym_unload_duet_module] = ACTIONS(1883),
    [sym_duet_mem_size_set] = ACTIONS(1883),
    [sym_duet_mem_size_get] = ACTIONS(1883),
    [sym_dynamic_application_device] = ACTIONS(1883),
    [sym_dynamic_polled_port] = ACTIONS(1883),
    [sym_static_port_binding] = ACTIONS(1883),
    [sym_static_ip_binding] = ACTIONS(1883),
    [sym_get_dns_list] = ACTIONS(1883),
    [sym_set_dns_list] = ACTIONS(1883),
    [sym_get_ip_address] = ACTIONS(1883),
    [sym_set_ip_address] = ACTIONS(1883),
    [sym_get_url_list] = ACTIONS(1883),
    [sym_add_url_entry] = ACTIONS(1883),
    [sym_delete_url_entry] = ACTIONS(1883),
    [sym_abs_value] = ACTIONS(1883),
    [sym_max_value] = ACTIONS(1883),
    [sym_min_value] = ACTIONS(1883),
    [sym_exp_value] = ACTIONS(1883),
    [sym_log_value] = ACTIONS(1883),
    [sym_log10_value] = ACTIONS(1883),
    [sym_power_value] = ACTIONS(1883),
    [sym_sqrt_value] = ACTIONS(1883),
    [sym_raw_be] = ACTIONS(1883),
    [sym_raw_le] = ACTIONS(1883),
    [sym_internal_queue_size_set] = ACTIONS(1883),
    [sym_internal_queue_size_get] = ACTIONS(1883),
    [sym_internal_threshold_set] = ACTIONS(1883),
    [sym_internal_threshold_get] = ACTIONS(1883),
    [sym_rebuilt_event] = ACTIONS(1883),
    [sym_do_custom_event] = ACTIONS(1883),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1883),
    [sym_clkmgr_set_clk_source] = ACTIONS(1883),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1883),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1883),
    [sym_clkmgr_get_timezone] = ACTIONS(1883),
    [sym_clkmgr_set_timezone] = ACTIONS(1883),
    [sym_clkmgr_get_resync_period] = ACTIONS(1883),
    [sym_clkmgr_set_resync_period] = ACTIONS(1883),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1883),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1883),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1883),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1883),
    [sym_clkmgr_get_timeservers] = ACTIONS(1883),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1883),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1883),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1883),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1883),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1883),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1883),
    [sym_set_log_level] = ACTIONS(1883),
    [sym_get_log_level] = ACTIONS(1883),
    [sym_amx_log] = ACTIONS(1883),
    [sym_smtp_server_config_set] = ACTIONS(1883),
    [sym_smtp_server_config_get] = ACTIONS(1883),
    [sym_smtp_send] = ACTIONS(1883),
    [sym_validate_netlinx_account] = ACTIONS(1883),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1883),
    [sym_authenticate_certificate] = ACTIONS(1883),
    [sym_audit_netlinx_session_event] = ACTIONS(1883),
    [sym_audit_netlinx_generic_event] = ACTIONS(1883),
    [sym_get_available_flash_disk_space] = ACTIONS(1883),
    [sym_get_max_flash_disk_space] = ACTIONS(1883),
    [sym_data_create_feed] = ACTIONS(1883),
    [sym_data_delete_feed] = ACTIONS(1883),
    [sym_data_publish_feed] = ACTIONS(1883),
    [sym_data_get_published_feed] = ACTIONS(1883),
    [sym_data_add_record] = ACTIONS(1883),
    [sym_data_get_event_record] = ACTIONS(1883),
    [sym__wc_data_create_feed] = ACTIONS(1883),
    [sym__wc_data_add_record] = ACTIONS(1883),
    [sym__wc_data_get_event_record] = ACTIONS(1883),
    [sym_format] = ACTIONS(1883),
    [sym_astro_clock] = ACTIONS(1883),
    [sym_set_outdoor_temperature] = ACTIONS(1883),
    [sym_set_virtual_level_count] = ACTIONS(1883),
    [sym_set_virtual_channel_count] = ACTIONS(1883),
    [sym_set_virtual_port_count] = ACTIONS(1883),
    [sym__wc_encode_int] = ACTIONS(1883),
    [sym__wc_decode_int] = ACTIONS(1883),
    [sym__wc_int] = ACTIONS(1883),
    [sym__wc_to_ch_int] = ACTIONS(1883),
    [sym__ch_to_wc_int] = ACTIONS(1883),
    [sym__wc_find_string_int] = ACTIONS(1883),
    [sym__wc_left_string_int] = ACTIONS(1883),
    [sym__wc_lower_string_int] = ACTIONS(1883),
    [sym__wc_mid_string_int] = ACTIONS(1883),
    [sym__wc_remove_string_int] = ACTIONS(1883),
    [sym__wc_right_string_int] = ACTIONS(1883),
    [sym__wc_upper_string_int] = ACTIONS(1883),
    [sym__wc_compare_string_int] = ACTIONS(1883),
    [sym__wc_get_buffer_char_int] = ACTIONS(1883),
    [sym__wc_get_buffer_string_int] = ACTIONS(1883),
    [sym__wc_concat_string_int] = ACTIONS(1883),
    [sym__wc_file_open_int] = ACTIONS(1883),
    [sym__wc_file_close_int] = ACTIONS(1883),
    [sym__wc_file_read_int] = ACTIONS(1883),
    [sym__wc_file_read_line_int] = ACTIONS(1883),
    [sym__wc_file_write_int] = ACTIONS(1883),
    [sym__wc_file_write_line_int] = ACTIONS(1883),
    [aux_sym_preproc_include_token1] = ACTIONS(1883),
    [aux_sym_preproc_define_token1] = ACTIONS(1885),
    [aux_sym_preproc_warn_token1] = ACTIONS(1885),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1885),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1885),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1885),
    [anon_sym_LBRACK] = ACTIONS(1885),
    [anon_sym_LBRACE] = ACTIONS(1885),
    [anon_sym_LPAREN] = ACTIONS(1883),
    [anon_sym_BANG] = ACTIONS(1885),
    [anon_sym_TILDE] = ACTIONS(1885),
    [anon_sym_DASH] = ACTIONS(1883),
    [anon_sym_PLUS] = ACTIONS(1883),
    [anon_sym_DASH_DASH] = ACTIONS(1885),
    [anon_sym_PLUS_PLUS] = ACTIONS(1885),
    [anon_sym_DQUOTE] = ACTIONS(1885),
    [anon_sym_SQUOTE] = ACTIONS(1885),
    [sym_number_literal] = ACTIONS(1885),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(450)] = {
    [ts_builtin_sym_end] = ACTIONS(1889),
    [sym_identifier] = ACTIONS(1887),
    [sym_program_name_keyword] = ACTIONS(1887),
    [sym_module_name_keyword] = ACTIONS(1887),
    [sym_define_device_keyword] = ACTIONS(1887),
    [sym_define_combine_keyword] = ACTIONS(1887),
    [sym_define_constant_keyword] = ACTIONS(1887),
    [sym_define_type_keyword] = ACTIONS(1887),
    [sym_define_variable_keyword] = ACTIONS(1887),
    [sym_define_system_variable_keyword] = ACTIONS(1887),
    [sym_define_connect_level_keyword] = ACTIONS(1887),
    [sym_define_latching_keyword] = ACTIONS(1887),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1887),
    [sym_define_toggling_keyword] = ACTIONS(1887),
    [sym_define_call_keyword] = ACTIONS(1887),
    [sym_define_function_keyword] = ACTIONS(1887),
    [sym_define_library_function_keyword] = ACTIONS(1887),
    [sym_define_start_keyword] = ACTIONS(1887),
    [sym_define_event_keyword] = ACTIONS(1887),
    [sym_define_program_keyword] = ACTIONS(1887),
    [sym_define_module_keyword] = ACTIONS(1887),
    [sym_char_keyword] = ACTIONS(1887),
    [sym_widechar_keyword] = ACTIONS(1887),
    [sym_integer_keyword] = ACTIONS(1887),
    [sym_sinteger_keyword] = ACTIONS(1887),
    [sym_long_keyword] = ACTIONS(1887),
    [sym_slong_keyword] = ACTIONS(1887),
    [sym_float_keyword] = ACTIONS(1887),
    [sym_double_keyword] = ACTIONS(1887),
    [sym_variant_keyword] = ACTIONS(1887),
    [sym_variantarray_keyword] = ACTIONS(1887),
    [sym_local_var_keyword] = ACTIONS(1887),
    [sym_stack_var_keyword] = ACTIONS(1887),
    [sym_constant_keyword] = ACTIONS(1887),
    [sym_volatile_keyword] = ACTIONS(1887),
    [sym_non_volatile_keyword] = ACTIONS(1887),
    [sym_persistent_keyword] = ACTIONS(1887),
    [sym_dev_keyword] = ACTIONS(1887),
    [sym_devlev_keyword] = ACTIONS(1887),
    [sym_devchan_keyword] = ACTIONS(1887),
    [sym_if_keyword] = ACTIONS(1887),
    [sym_switch_keyword] = ACTIONS(1887),
    [sym_case_keyword] = ACTIONS(1887),
    [sym_default_keyword] = ACTIONS(1887),
    [sym_while_keyword] = ACTIONS(1887),
    [sym_for_keyword] = ACTIONS(1887),
    [sym_break_keyword] = ACTIONS(1887),
    [sym_continue_keyword] = ACTIONS(1887),
    [sym_return_keyword] = ACTIONS(1887),
    [sym_select_keyword] = ACTIONS(1887),
    [sym_active_keyword] = ACTIONS(1887),
    [sym_struct_keyword] = ACTIONS(1887),
    [sym_structure_keyword] = ACTIONS(1887),
    [sym_button_event_keyword] = ACTIONS(1887),
    [sym_channel_event_keyword] = ACTIONS(1887),
    [sym_level_event_keyword] = ACTIONS(1887),
    [sym_data_event_keyword] = ACTIONS(1887),
    [sym_timeline_event_keyword] = ACTIONS(1887),
    [sym_custom_event_keyword] = ACTIONS(1887),
    [sym_push_keyword] = ACTIONS(1887),
    [sym_release_keyword] = ACTIONS(1887),
    [aux_sym_on_keyword_token1] = ACTIONS(1887),
    [aux_sym_off_keyword_token1] = ACTIONS(1887),
    [sym_send_level_keyword] = ACTIONS(1887),
    [sym_send_string_keyword] = ACTIONS(1887),
    [sym_send_command_keyword] = ACTIONS(1887),
    [sym_clear_buffer_keyword] = ACTIONS(1887),
    [sym_create_buffer_keyword] = ACTIONS(1887),
    [sym_create_multi_buffer_keyword] = ACTIONS(1887),
    [sym_call_keyword] = ACTIONS(1887),
    [sym_system_call_keyword] = ACTIONS(1887),
    [sym_devchan_to_keyword] = ACTIONS(1887),
    [sym_devchan_min_to_keyword] = ACTIONS(1887),
    [sym_devchan_total_off_keyword] = ACTIONS(1887),
    [sym_devchan_pulse_keyword] = ACTIONS(1887),
    [sym_wait_keyword] = ACTIONS(1887),
    [sym_cancel_wait_keyword] = ACTIONS(1887),
    [sym_wait_until_keyword] = ACTIONS(1887),
    [sym_cancel_wait_until_keyword] = ACTIONS(1887),
    [sym_cancel_all_wait_keyword] = ACTIONS(1887),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1887),
    [sym_not] = ACTIONS(1887),
    [sym_bnot] = ACTIONS(1887),
    [sym___file__] = ACTIONS(1887),
    [sym___date__] = ACTIONS(1887),
    [sym___ldate__] = ACTIONS(1887),
    [sym___time__] = ACTIONS(1887),
    [sym___line__] = ACTIONS(1887),
    [sym___name__] = ACTIONS(1887),
    [sym___netlinx__] = ACTIONS(1887),
    [sym_netlinx_axi_version] = ACTIONS(1887),
    [sym_url_flg_tcp] = ACTIONS(1887),
    [sym_url_flg_accinfopresent] = ACTIONS(1887),
    [sym_url_flg_temp] = ACTIONS(1887),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1887),
    [sym_url_flg_stat_mask] = ACTIONS(1887),
    [sym_url_flg_stat_lookup] = ACTIONS(1887),
    [sym_url_flg_stat_connecting] = ACTIONS(1887),
    [sym_url_flg_stat_waiting] = ACTIONS(1887),
    [sym_url_flg_stat_connected] = ACTIONS(1887),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1887),
    [sym_first_virtual_device] = ACTIONS(1887),
    [sym_first_local_port] = ACTIONS(1887),
    [sym_dynamic_virtual_device] = ACTIONS(1887),
    [sym_timeline_once] = ACTIONS(1887),
    [sym_timeline_repeat] = ACTIONS(1887),
    [sym_timeline_absolute] = ACTIONS(1887),
    [sym_timeline_relative] = ACTIONS(1887),
    [sym_do_push_timed_infinite] = ACTIONS(1887),
    [sym_source_type_no_address] = ACTIONS(1887),
    [sym_source_type_neuron_id] = ACTIONS(1887),
    [sym_source_type_ip_address] = ACTIONS(1887),
    [sym_source_type_axlink] = ACTIONS(1887),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1887),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1887),
    [sym_source_type_ip_socket_address] = ACTIONS(1887),
    [sym_source_type_rs232] = ACTIONS(1887),
    [sym_source_type_internal] = ACTIONS(1887),
    [sym_source_type_mac_address] = ACTIONS(1887),
    [sym_source_type_ipv4_port] = ACTIONS(1887),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1887),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1887),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1887),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1887),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1887),
    [sym_true] = ACTIONS(1887),
    [sym_false] = ACTIONS(1887),
    [sym_file_read_only] = ACTIONS(1887),
    [sym_file_rw_new] = ACTIONS(1887),
    [sym_file_rw_append] = ACTIONS(1887),
    [sym_ip_tcp] = ACTIONS(1887),
    [sym_ip_udp] = ACTIONS(1887),
    [sym_ip_udp_2way] = ACTIONS(1887),
    [sym_xml_encode_types] = ACTIONS(1887),
    [sym_xml_encode_char_as_list] = ACTIONS(1887),
    [sym_xml_encode_le] = ACTIONS(1887),
    [sym_xml_decode_types] = ACTIONS(1887),
    [sym_xml_decode_no_preserve] = ACTIONS(1887),
    [sym_duet_dev_type_display_device] = ACTIONS(1887),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1887),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1887),
    [sym_duet_dev_type_io_device] = ACTIONS(1887),
    [sym_duet_dev_type_relay_device] = ACTIONS(1887),
    [sym_duet_dev_type_ups] = ACTIONS(1887),
    [sym_duet_dev_type_amplifier] = ACTIONS(1887),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1887),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1887),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1887),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1887),
    [sym_duet_dev_type_camera] = ACTIONS(1887),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1887),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1887),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1887),
    [sym_duet_dev_type_dss] = ACTIONS(1887),
    [sym_duet_dev_type_dvr] = ACTIONS(1887),
    [sym_duet_dev_type_disc_device] = ACTIONS(1887),
    [sym_duet_dev_type_document_camera] = ACTIONS(1887),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1887),
    [sym_duet_dev_type_hvac] = ACTIONS(1887),
    [sym_duet_dev_type_keypad] = ACTIONS(1887),
    [sym_duet_dev_type_light] = ACTIONS(1887),
    [sym_duet_dev_type_monitor] = ACTIONS(1887),
    [sym_duet_dev_type_motor] = ACTIONS(1887),
    [sym_duet_dev_type_multi_window] = ACTIONS(1887),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1887),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1887),
    [sym_duet_dev_type_receiver] = ACTIONS(1887),
    [sym_duet_dev_type_security_system] = ACTIONS(1887),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1887),
    [sym_duet_dev_type_settop_box] = ACTIONS(1887),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1887),
    [sym_duet_dev_type_switcher] = ACTIONS(1887),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1887),
    [sym_duet_dev_type_tv] = ACTIONS(1887),
    [sym_duet_dev_type_utility] = ACTIONS(1887),
    [sym_duet_dev_type_vcr] = ACTIONS(1887),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1887),
    [sym_duet_dev_type_video_processor] = ACTIONS(1887),
    [sym_duet_dev_type_video_projector] = ACTIONS(1887),
    [sym_duet_dev_type_video_wall] = ACTIONS(1887),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1887),
    [sym_duet_dev_type_weather] = ACTIONS(1887),
    [sym_duet_dev_not_polled] = ACTIONS(1887),
    [sym_duet_dev_polled] = ACTIONS(1887),
    [sym_clkmgr_mode_network] = ACTIONS(1887),
    [sym_clkmgr_mode_standalone] = ACTIONS(1887),
    [sym_normal_standby] = ACTIONS(1887),
    [sym_normal_wake] = ACTIONS(1887),
    [sym_ip_multicast_ttl_option] = ACTIONS(1887),
    [sym_ip_ttl_subnet] = ACTIONS(1887),
    [sym_ip_ttl_site] = ACTIONS(1887),
    [sym_ip_ttl_region] = ACTIONS(1887),
    [sym_ip_ttl_continent] = ACTIONS(1887),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1887),
    [sym_ip_nodelay_on] = ACTIONS(1887),
    [sym_ip_nodelay_off] = ACTIONS(1887),
    [sym_amx_error] = ACTIONS(1887),
    [sym_amx_warning] = ACTIONS(1887),
    [sym_amx_info] = ACTIONS(1887),
    [sym_amx_debug] = ACTIONS(1887),
    [sym_netlinx_login_success] = ACTIONS(1887),
    [sym_netlinx_login_fail] = ACTIONS(1887),
    [sym_netlinx_logout] = ACTIONS(1887),
    [sym_null_str] = ACTIONS(1887),
    [sym_smtp_address] = ACTIONS(1887),
    [sym_smtp_port_number] = ACTIONS(1887),
    [sym_smtp_username] = ACTIONS(1887),
    [sym_smtp_password] = ACTIONS(1887),
    [sym_smtp_require_tls] = ACTIONS(1887),
    [sym_smtp_from] = ACTIONS(1887),
    [sym_smtp_tls_true] = ACTIONS(1887),
    [sym_smtp_tls_false] = ACTIONS(1887),
    [sym_listview_on_row_select_event] = ACTIONS(1887),
    [sym_data_structure_unknown] = ACTIONS(1887),
    [sym_data_structure_datafeed] = ACTIONS(1887),
    [sym_data_structure_datarecordset] = ACTIONS(1887),
    [sym_data_structure_datarecord] = ACTIONS(1887),
    [sym_data_structure_datafield] = ACTIONS(1887),
    [sym_data_max_content_field_count] = ACTIONS(1887),
    [sym_data_max_metadata_field_count] = ACTIONS(1887),
    [sym_data_max_id_length] = ACTIONS(1887),
    [sym_data_max_name_length] = ACTIONS(1887),
    [sym_data_max_description_length] = ACTIONS(1887),
    [sym_data_max_type_length] = ACTIONS(1887),
    [sym_data_max_format_length] = ACTIONS(1887),
    [sym_data_max_source_length] = ACTIONS(1887),
    [sym_data_max_value_length] = ACTIONS(1887),
    [sym_data_max_label_length] = ACTIONS(1887),
    [sym_data_max_url_length] = ACTIONS(1887),
    [sym_max_last_login_info_length] = ACTIONS(1887),
    [sym_data_type_unknown] = ACTIONS(1887),
    [sym_data_type_string] = ACTIONS(1887),
    [sym_data_type_datetime] = ACTIONS(1887),
    [sym_data_type_date] = ACTIONS(1887),
    [sym_data_type_time] = ACTIONS(1887),
    [sym_data_type_image] = ACTIONS(1887),
    [sym_data_format_url] = ACTIONS(1887),
    [sym_data_format_phone] = ACTIONS(1887),
    [sym_data_format_email] = ACTIONS(1887),
    [sym_data_format_iso8601] = ACTIONS(1887),
    [sym_auditlog_priv] = ACTIONS(1887),
    [sym_configuration_priv] = ACTIONS(1887),
    [sym_deviceconfig_priv] = ACTIONS(1887),
    [sym_ftp_priv] = ACTIONS(1887),
    [sym_http_priv] = ACTIONS(1887),
    [sym_networkconfig_priv] = ACTIONS(1887),
    [sym_programport_priv] = ACTIONS(1887),
    [sym_remoteui_priv] = ACTIONS(1887),
    [sym_securitycontrol_priv] = ACTIONS(1887),
    [sym_softwaremanagement_priv] = ACTIONS(1887),
    [sym_terminal_priv] = ACTIONS(1887),
    [sym_tpadmin_priv] = ACTIONS(1887),
    [sym_user1_priv] = ACTIONS(1887),
    [sym_user2_priv] = ACTIONS(1887),
    [sym_user3_priv] = ACTIONS(1887),
    [sym_user4_priv] = ACTIONS(1887),
    [sym_usermanagement_priv] = ACTIONS(1887),
    [sym_valid_account] = ACTIONS(1887),
    [sym_err_name_invalid] = ACTIONS(1887),
    [sym_err_password_invalid] = ACTIONS(1887),
    [sym_err_invalid_account] = ACTIONS(1887),
    [sym_err_no_authorization] = ACTIONS(1887),
    [sym_err_invalid_parameter] = ACTIONS(1887),
    [sym_err_user_locked_out] = ACTIONS(1887),
    [sym_err_user_account_expired] = ACTIONS(1887),
    [sym_err_general_error] = ACTIONS(1887),
    [sym_err_authentication_error] = ACTIONS(1887),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1887),
    [sym_tls_validate_certificate] = ACTIONS(1887),
    [sym_tdata] = ACTIONS(1887),
    [sym_tchannel] = ACTIONS(1887),
    [sym_tlevel] = ACTIONS(1887),
    [sym_tbutton] = ACTIONS(1887),
    [sym_ttimeline] = ACTIONS(1887),
    [sym_tcustom] = ACTIONS(1887),
    [sym_url_struct] = ACTIONS(1887),
    [sym_dns_struct] = ACTIONS(1887),
    [sym_ip_address_struct] = ACTIONS(1887),
    [sym_dev_info_struct] = ACTIONS(1887),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1887),
    [sym_data_feed] = ACTIONS(1887),
    [sym_data_field] = ACTIONS(1887),
    [sym_data_record] = ACTIONS(1887),
    [sym_wc_data_feed] = ACTIONS(1887),
    [sym_wc_data_field] = ACTIONS(1887),
    [sym_wc_data_record] = ACTIONS(1887),
    [sym_last_login_info] = ACTIONS(1887),
    [sym____reserved___] = ACTIONS(1887),
    [sym_date] = ACTIONS(1887),
    [sym_day] = ACTIONS(1887),
    [sym_time] = ACTIONS(1887),
    [sym_ldate] = ACTIONS(1887),
    [sym_push_device] = ACTIONS(1887),
    [sym_push_channel] = ACTIONS(1887),
    [sym_push_devchan] = ACTIONS(1887),
    [sym_release_device] = ACTIONS(1887),
    [sym_release_channel] = ACTIONS(1887),
    [sym_release_devchan] = ACTIONS(1887),
    [sym_master_slot] = ACTIONS(1887),
    [sym_get_pulse_time] = ACTIONS(1887),
    [sym_get_timer] = ACTIONS(1887),
    [sym_master_sn] = ACTIONS(1887),
    [sym_system_number] = ACTIONS(1887),
    [sym_button] = ACTIONS(1887),
    [sym_dv_channel] = ACTIONS(1887),
    [sym_data] = ACTIONS(1887),
    [sym_level] = ACTIONS(1887),
    [sym_timeline] = ACTIONS(1887),
    [sym_channel] = ACTIONS(1887),
    [sym_custom] = ACTIONS(1887),
    [sym_atoi] = ACTIONS(1887),
    [sym_atol] = ACTIONS(1887),
    [sym_atof] = ACTIONS(1887),
    [sym_itoa] = ACTIONS(1887),
    [sym_itohex] = ACTIONS(1887),
    [sym_hextoi] = ACTIONS(1887),
    [sym_ftoa] = ACTIONS(1887),
    [sym_find_string] = ACTIONS(1887),
    [sym_left_string] = ACTIONS(1887),
    [sym_right_string] = ACTIONS(1887),
    [sym_mid_string] = ACTIONS(1887),
    [sym_remove_string] = ACTIONS(1887),
    [sym_get_buffer_string] = ACTIONS(1887),
    [sym_get_buffer_char] = ACTIONS(1887),
    [sym_get_multi_buffer_string] = ACTIONS(1887),
    [sym_length_string] = ACTIONS(1887),
    [sym_max_length_string] = ACTIONS(1887),
    [sym_set_length_string] = ACTIONS(1887),
    [sym_compare_string] = ACTIONS(1887),
    [sym_upper_string] = ACTIONS(1887),
    [sym_lower_string] = ACTIONS(1887),
    [sym_get_last] = ACTIONS(1887),
    [sym_date_to_day] = ACTIONS(1887),
    [sym_date_to_month] = ACTIONS(1887),
    [sym_date_to_year] = ACTIONS(1887),
    [sym_time_to_hour] = ACTIONS(1887),
    [sym_time_to_minute] = ACTIONS(1887),
    [sym_time_to_second] = ACTIONS(1887),
    [sym_day_of_week] = ACTIONS(1887),
    [sym_set_pulse_time] = ACTIONS(1887),
    [sym_set_timer] = ACTIONS(1887),
    [sym_file_open] = ACTIONS(1887),
    [sym_file_close] = ACTIONS(1887),
    [sym_file_read] = ACTIONS(1887),
    [sym_file_read_line] = ACTIONS(1887),
    [sym_file_write] = ACTIONS(1887),
    [sym_file_write_line] = ACTIONS(1887),
    [sym_file_copy] = ACTIONS(1887),
    [sym_file_delete] = ACTIONS(1887),
    [sym_file_rename] = ACTIONS(1887),
    [sym_file_dir] = ACTIONS(1887),
    [sym_file_getdir] = ACTIONS(1887),
    [sym_file_setdir] = ACTIONS(1887),
    [sym_file_create_dir] = ACTIONS(1887),
    [sym_file_remove_dir] = ACTIONS(1887),
    [sym_file_seek] = ACTIONS(1887),
    [sym_length_array] = ACTIONS(1887),
    [sym_max_length_array] = ACTIONS(1887),
    [sym_set_length_array] = ACTIONS(1887),
    [sym_ip_client_open] = ACTIONS(1887),
    [sym_ip_client_close] = ACTIONS(1887),
    [sym_ip_server_open] = ACTIONS(1887),
    [sym_ip_server_close] = ACTIONS(1887),
    [sym_ip_bound_client_open] = ACTIONS(1887),
    [sym_ip_mc_server_open] = ACTIONS(1887),
    [sym_ip_set_option] = ACTIONS(1887),
    [sym_ssh_client_open] = ACTIONS(1887),
    [sym_ssh_client_close] = ACTIONS(1887),
    [sym_tls_client_open] = ACTIONS(1887),
    [sym_tls_client_close] = ACTIONS(1887),
    [sym_do_push] = ACTIONS(1887),
    [sym_do_push_timed] = ACTIONS(1887),
    [sym_do_release] = ACTIONS(1887),
    [sym_device_id] = ACTIONS(1887),
    [sym_device_info] = ACTIONS(1887),
    [sym_device_id_string] = ACTIONS(1887),
    [sym_device_standby] = ACTIONS(1887),
    [sym_device_wake] = ACTIONS(1887),
    [sym_timeline_create] = ACTIONS(1887),
    [sym_timeline_kill] = ACTIONS(1887),
    [sym_timeline_reload] = ACTIONS(1887),
    [sym_timeline_pause] = ACTIONS(1887),
    [sym_timeline_restart] = ACTIONS(1887),
    [sym_timeline_set] = ACTIONS(1887),
    [sym_timeline_get] = ACTIONS(1887),
    [sym_timeline_active] = ACTIONS(1887),
    [sym_timeline_dynamic_id] = ACTIONS(1887),
    [sym_variable_to_string] = ACTIONS(1887),
    [sym_string_to_variable] = ACTIONS(1887),
    [sym_length_variable_to_string] = ACTIONS(1887),
    [sym_type_cast] = ACTIONS(1887),
    [sym_variable_to_xml] = ACTIONS(1887),
    [sym_xml_to_variable] = ACTIONS(1887),
    [sym_length_variable_to_xml] = ACTIONS(1887),
    [sym_redirect_string] = ACTIONS(1887),
    [sym_random_number] = ACTIONS(1887),
    [sym_get_unique_id] = ACTIONS(1887),
    [sym_get_system_number] = ACTIONS(1887),
    [sym_set_system_number] = ACTIONS(1887),
    [sym_get_serial_number] = ACTIONS(1887),
    [sym_get_master_build] = ACTIONS(1887),
    [sym_set_validation_code] = ACTIONS(1887),
    [sym_reboot] = ACTIONS(1887),
    [sym_load_duet_module] = ACTIONS(1887),
    [sym_unload_duet_module] = ACTIONS(1887),
    [sym_duet_mem_size_set] = ACTIONS(1887),
    [sym_duet_mem_size_get] = ACTIONS(1887),
    [sym_dynamic_application_device] = ACTIONS(1887),
    [sym_dynamic_polled_port] = ACTIONS(1887),
    [sym_static_port_binding] = ACTIONS(1887),
    [sym_static_ip_binding] = ACTIONS(1887),
    [sym_get_dns_list] = ACTIONS(1887),
    [sym_set_dns_list] = ACTIONS(1887),
    [sym_get_ip_address] = ACTIONS(1887),
    [sym_set_ip_address] = ACTIONS(1887),
    [sym_get_url_list] = ACTIONS(1887),
    [sym_add_url_entry] = ACTIONS(1887),
    [sym_delete_url_entry] = ACTIONS(1887),
    [sym_abs_value] = ACTIONS(1887),
    [sym_max_value] = ACTIONS(1887),
    [sym_min_value] = ACTIONS(1887),
    [sym_exp_value] = ACTIONS(1887),
    [sym_log_value] = ACTIONS(1887),
    [sym_log10_value] = ACTIONS(1887),
    [sym_power_value] = ACTIONS(1887),
    [sym_sqrt_value] = ACTIONS(1887),
    [sym_raw_be] = ACTIONS(1887),
    [sym_raw_le] = ACTIONS(1887),
    [sym_internal_queue_size_set] = ACTIONS(1887),
    [sym_internal_queue_size_get] = ACTIONS(1887),
    [sym_internal_threshold_set] = ACTIONS(1887),
    [sym_internal_threshold_get] = ACTIONS(1887),
    [sym_rebuilt_event] = ACTIONS(1887),
    [sym_do_custom_event] = ACTIONS(1887),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1887),
    [sym_clkmgr_set_clk_source] = ACTIONS(1887),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1887),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1887),
    [sym_clkmgr_get_timezone] = ACTIONS(1887),
    [sym_clkmgr_set_timezone] = ACTIONS(1887),
    [sym_clkmgr_get_resync_period] = ACTIONS(1887),
    [sym_clkmgr_set_resync_period] = ACTIONS(1887),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1887),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1887),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1887),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1887),
    [sym_clkmgr_get_timeservers] = ACTIONS(1887),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1887),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1887),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1887),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1887),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1887),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1887),
    [sym_set_log_level] = ACTIONS(1887),
    [sym_get_log_level] = ACTIONS(1887),
    [sym_amx_log] = ACTIONS(1887),
    [sym_smtp_server_config_set] = ACTIONS(1887),
    [sym_smtp_server_config_get] = ACTIONS(1887),
    [sym_smtp_send] = ACTIONS(1887),
    [sym_validate_netlinx_account] = ACTIONS(1887),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1887),
    [sym_authenticate_certificate] = ACTIONS(1887),
    [sym_audit_netlinx_session_event] = ACTIONS(1887),
    [sym_audit_netlinx_generic_event] = ACTIONS(1887),
    [sym_get_available_flash_disk_space] = ACTIONS(1887),
    [sym_get_max_flash_disk_space] = ACTIONS(1887),
    [sym_data_create_feed] = ACTIONS(1887),
    [sym_data_delete_feed] = ACTIONS(1887),
    [sym_data_publish_feed] = ACTIONS(1887),
    [sym_data_get_published_feed] = ACTIONS(1887),
    [sym_data_add_record] = ACTIONS(1887),
    [sym_data_get_event_record] = ACTIONS(1887),
    [sym__wc_data_create_feed] = ACTIONS(1887),
    [sym__wc_data_add_record] = ACTIONS(1887),
    [sym__wc_data_get_event_record] = ACTIONS(1887),
    [sym_format] = ACTIONS(1887),
    [sym_astro_clock] = ACTIONS(1887),
    [sym_set_outdoor_temperature] = ACTIONS(1887),
    [sym_set_virtual_level_count] = ACTIONS(1887),
    [sym_set_virtual_channel_count] = ACTIONS(1887),
    [sym_set_virtual_port_count] = ACTIONS(1887),
    [sym__wc_encode_int] = ACTIONS(1887),
    [sym__wc_decode_int] = ACTIONS(1887),
    [sym__wc_int] = ACTIONS(1887),
    [sym__wc_to_ch_int] = ACTIONS(1887),
    [sym__ch_to_wc_int] = ACTIONS(1887),
    [sym__wc_find_string_int] = ACTIONS(1887),
    [sym__wc_left_string_int] = ACTIONS(1887),
    [sym__wc_lower_string_int] = ACTIONS(1887),
    [sym__wc_mid_string_int] = ACTIONS(1887),
    [sym__wc_remove_string_int] = ACTIONS(1887),
    [sym__wc_right_string_int] = ACTIONS(1887),
    [sym__wc_upper_string_int] = ACTIONS(1887),
    [sym__wc_compare_string_int] = ACTIONS(1887),
    [sym__wc_get_buffer_char_int] = ACTIONS(1887),
    [sym__wc_get_buffer_string_int] = ACTIONS(1887),
    [sym__wc_concat_string_int] = ACTIONS(1887),
    [sym__wc_file_open_int] = ACTIONS(1887),
    [sym__wc_file_close_int] = ACTIONS(1887),
    [sym__wc_file_read_int] = ACTIONS(1887),
    [sym__wc_file_read_line_int] = ACTIONS(1887),
    [sym__wc_file_write_int] = ACTIONS(1887),
    [sym__wc_file_write_line_int] = ACTIONS(1887),
    [aux_sym_preproc_include_token1] = ACTIONS(1887),
    [aux_sym_preproc_define_token1] = ACTIONS(1889),
    [aux_sym_preproc_warn_token1] = ACTIONS(1889),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1889),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1889),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1889),
    [anon_sym_LBRACK] = ACTIONS(1889),
    [anon_sym_LBRACE] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(1887),
    [anon_sym_BANG] = ACTIONS(1889),
    [anon_sym_TILDE] = ACTIONS(1889),
    [anon_sym_DASH] = ACTIONS(1887),
    [anon_sym_PLUS] = ACTIONS(1887),
    [anon_sym_DASH_DASH] = ACTIONS(1889),
    [anon_sym_PLUS_PLUS] = ACTIONS(1889),
    [anon_sym_DQUOTE] = ACTIONS(1889),
    [anon_sym_SQUOTE] = ACTIONS(1889),
    [sym_number_literal] = ACTIONS(1889),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(451)] = {
    [ts_builtin_sym_end] = ACTIONS(1903),
    [sym_identifier] = ACTIONS(1901),
    [sym_program_name_keyword] = ACTIONS(1901),
    [sym_module_name_keyword] = ACTIONS(1901),
    [sym_define_device_keyword] = ACTIONS(1901),
    [sym_define_combine_keyword] = ACTIONS(1901),
    [sym_define_constant_keyword] = ACTIONS(1901),
    [sym_define_type_keyword] = ACTIONS(1901),
    [sym_define_variable_keyword] = ACTIONS(1901),
    [sym_define_system_variable_keyword] = ACTIONS(1901),
    [sym_define_connect_level_keyword] = ACTIONS(1901),
    [sym_define_latching_keyword] = ACTIONS(1901),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1901),
    [sym_define_toggling_keyword] = ACTIONS(1901),
    [sym_define_call_keyword] = ACTIONS(1901),
    [sym_define_function_keyword] = ACTIONS(1901),
    [sym_define_library_function_keyword] = ACTIONS(1901),
    [sym_define_start_keyword] = ACTIONS(1901),
    [sym_define_event_keyword] = ACTIONS(1901),
    [sym_define_program_keyword] = ACTIONS(1901),
    [sym_define_module_keyword] = ACTIONS(1901),
    [sym_char_keyword] = ACTIONS(1901),
    [sym_widechar_keyword] = ACTIONS(1901),
    [sym_integer_keyword] = ACTIONS(1901),
    [sym_sinteger_keyword] = ACTIONS(1901),
    [sym_long_keyword] = ACTIONS(1901),
    [sym_slong_keyword] = ACTIONS(1901),
    [sym_float_keyword] = ACTIONS(1901),
    [sym_double_keyword] = ACTIONS(1901),
    [sym_variant_keyword] = ACTIONS(1901),
    [sym_variantarray_keyword] = ACTIONS(1901),
    [sym_local_var_keyword] = ACTIONS(1901),
    [sym_stack_var_keyword] = ACTIONS(1901),
    [sym_constant_keyword] = ACTIONS(1901),
    [sym_volatile_keyword] = ACTIONS(1901),
    [sym_non_volatile_keyword] = ACTIONS(1901),
    [sym_persistent_keyword] = ACTIONS(1901),
    [sym_dev_keyword] = ACTIONS(1901),
    [sym_devlev_keyword] = ACTIONS(1901),
    [sym_devchan_keyword] = ACTIONS(1901),
    [sym_if_keyword] = ACTIONS(1901),
    [sym_switch_keyword] = ACTIONS(1901),
    [sym_case_keyword] = ACTIONS(1901),
    [sym_default_keyword] = ACTIONS(1901),
    [sym_while_keyword] = ACTIONS(1901),
    [sym_for_keyword] = ACTIONS(1901),
    [sym_break_keyword] = ACTIONS(1901),
    [sym_continue_keyword] = ACTIONS(1901),
    [sym_return_keyword] = ACTIONS(1901),
    [sym_select_keyword] = ACTIONS(1901),
    [sym_active_keyword] = ACTIONS(1901),
    [sym_struct_keyword] = ACTIONS(1901),
    [sym_structure_keyword] = ACTIONS(1901),
    [sym_button_event_keyword] = ACTIONS(1901),
    [sym_channel_event_keyword] = ACTIONS(1901),
    [sym_level_event_keyword] = ACTIONS(1901),
    [sym_data_event_keyword] = ACTIONS(1901),
    [sym_timeline_event_keyword] = ACTIONS(1901),
    [sym_custom_event_keyword] = ACTIONS(1901),
    [sym_push_keyword] = ACTIONS(1901),
    [sym_release_keyword] = ACTIONS(1901),
    [aux_sym_on_keyword_token1] = ACTIONS(1901),
    [aux_sym_off_keyword_token1] = ACTIONS(1901),
    [sym_send_level_keyword] = ACTIONS(1901),
    [sym_send_string_keyword] = ACTIONS(1901),
    [sym_send_command_keyword] = ACTIONS(1901),
    [sym_clear_buffer_keyword] = ACTIONS(1901),
    [sym_create_buffer_keyword] = ACTIONS(1901),
    [sym_create_multi_buffer_keyword] = ACTIONS(1901),
    [sym_call_keyword] = ACTIONS(1901),
    [sym_system_call_keyword] = ACTIONS(1901),
    [sym_devchan_to_keyword] = ACTIONS(1901),
    [sym_devchan_min_to_keyword] = ACTIONS(1901),
    [sym_devchan_total_off_keyword] = ACTIONS(1901),
    [sym_devchan_pulse_keyword] = ACTIONS(1901),
    [sym_wait_keyword] = ACTIONS(1901),
    [sym_cancel_wait_keyword] = ACTIONS(1901),
    [sym_wait_until_keyword] = ACTIONS(1901),
    [sym_cancel_wait_until_keyword] = ACTIONS(1901),
    [sym_cancel_all_wait_keyword] = ACTIONS(1901),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1901),
    [sym_not] = ACTIONS(1901),
    [sym_bnot] = ACTIONS(1901),
    [sym___file__] = ACTIONS(1901),
    [sym___date__] = ACTIONS(1901),
    [sym___ldate__] = ACTIONS(1901),
    [sym___time__] = ACTIONS(1901),
    [sym___line__] = ACTIONS(1901),
    [sym___name__] = ACTIONS(1901),
    [sym___netlinx__] = ACTIONS(1901),
    [sym_netlinx_axi_version] = ACTIONS(1901),
    [sym_url_flg_tcp] = ACTIONS(1901),
    [sym_url_flg_accinfopresent] = ACTIONS(1901),
    [sym_url_flg_temp] = ACTIONS(1901),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1901),
    [sym_url_flg_stat_mask] = ACTIONS(1901),
    [sym_url_flg_stat_lookup] = ACTIONS(1901),
    [sym_url_flg_stat_connecting] = ACTIONS(1901),
    [sym_url_flg_stat_waiting] = ACTIONS(1901),
    [sym_url_flg_stat_connected] = ACTIONS(1901),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1901),
    [sym_first_virtual_device] = ACTIONS(1901),
    [sym_first_local_port] = ACTIONS(1901),
    [sym_dynamic_virtual_device] = ACTIONS(1901),
    [sym_timeline_once] = ACTIONS(1901),
    [sym_timeline_repeat] = ACTIONS(1901),
    [sym_timeline_absolute] = ACTIONS(1901),
    [sym_timeline_relative] = ACTIONS(1901),
    [sym_do_push_timed_infinite] = ACTIONS(1901),
    [sym_source_type_no_address] = ACTIONS(1901),
    [sym_source_type_neuron_id] = ACTIONS(1901),
    [sym_source_type_ip_address] = ACTIONS(1901),
    [sym_source_type_axlink] = ACTIONS(1901),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1901),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1901),
    [sym_source_type_ip_socket_address] = ACTIONS(1901),
    [sym_source_type_rs232] = ACTIONS(1901),
    [sym_source_type_internal] = ACTIONS(1901),
    [sym_source_type_mac_address] = ACTIONS(1901),
    [sym_source_type_ipv4_port] = ACTIONS(1901),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1901),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1901),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1901),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1901),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1901),
    [sym_true] = ACTIONS(1901),
    [sym_false] = ACTIONS(1901),
    [sym_file_read_only] = ACTIONS(1901),
    [sym_file_rw_new] = ACTIONS(1901),
    [sym_file_rw_append] = ACTIONS(1901),
    [sym_ip_tcp] = ACTIONS(1901),
    [sym_ip_udp] = ACTIONS(1901),
    [sym_ip_udp_2way] = ACTIONS(1901),
    [sym_xml_encode_types] = ACTIONS(1901),
    [sym_xml_encode_char_as_list] = ACTIONS(1901),
    [sym_xml_encode_le] = ACTIONS(1901),
    [sym_xml_decode_types] = ACTIONS(1901),
    [sym_xml_decode_no_preserve] = ACTIONS(1901),
    [sym_duet_dev_type_display_device] = ACTIONS(1901),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1901),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1901),
    [sym_duet_dev_type_io_device] = ACTIONS(1901),
    [sym_duet_dev_type_relay_device] = ACTIONS(1901),
    [sym_duet_dev_type_ups] = ACTIONS(1901),
    [sym_duet_dev_type_amplifier] = ACTIONS(1901),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1901),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1901),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1901),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1901),
    [sym_duet_dev_type_camera] = ACTIONS(1901),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1901),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1901),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1901),
    [sym_duet_dev_type_dss] = ACTIONS(1901),
    [sym_duet_dev_type_dvr] = ACTIONS(1901),
    [sym_duet_dev_type_disc_device] = ACTIONS(1901),
    [sym_duet_dev_type_document_camera] = ACTIONS(1901),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1901),
    [sym_duet_dev_type_hvac] = ACTIONS(1901),
    [sym_duet_dev_type_keypad] = ACTIONS(1901),
    [sym_duet_dev_type_light] = ACTIONS(1901),
    [sym_duet_dev_type_monitor] = ACTIONS(1901),
    [sym_duet_dev_type_motor] = ACTIONS(1901),
    [sym_duet_dev_type_multi_window] = ACTIONS(1901),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1901),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1901),
    [sym_duet_dev_type_receiver] = ACTIONS(1901),
    [sym_duet_dev_type_security_system] = ACTIONS(1901),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1901),
    [sym_duet_dev_type_settop_box] = ACTIONS(1901),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1901),
    [sym_duet_dev_type_switcher] = ACTIONS(1901),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1901),
    [sym_duet_dev_type_tv] = ACTIONS(1901),
    [sym_duet_dev_type_utility] = ACTIONS(1901),
    [sym_duet_dev_type_vcr] = ACTIONS(1901),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1901),
    [sym_duet_dev_type_video_processor] = ACTIONS(1901),
    [sym_duet_dev_type_video_projector] = ACTIONS(1901),
    [sym_duet_dev_type_video_wall] = ACTIONS(1901),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1901),
    [sym_duet_dev_type_weather] = ACTIONS(1901),
    [sym_duet_dev_not_polled] = ACTIONS(1901),
    [sym_duet_dev_polled] = ACTIONS(1901),
    [sym_clkmgr_mode_network] = ACTIONS(1901),
    [sym_clkmgr_mode_standalone] = ACTIONS(1901),
    [sym_normal_standby] = ACTIONS(1901),
    [sym_normal_wake] = ACTIONS(1901),
    [sym_ip_multicast_ttl_option] = ACTIONS(1901),
    [sym_ip_ttl_subnet] = ACTIONS(1901),
    [sym_ip_ttl_site] = ACTIONS(1901),
    [sym_ip_ttl_region] = ACTIONS(1901),
    [sym_ip_ttl_continent] = ACTIONS(1901),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1901),
    [sym_ip_nodelay_on] = ACTIONS(1901),
    [sym_ip_nodelay_off] = ACTIONS(1901),
    [sym_amx_error] = ACTIONS(1901),
    [sym_amx_warning] = ACTIONS(1901),
    [sym_amx_info] = ACTIONS(1901),
    [sym_amx_debug] = ACTIONS(1901),
    [sym_netlinx_login_success] = ACTIONS(1901),
    [sym_netlinx_login_fail] = ACTIONS(1901),
    [sym_netlinx_logout] = ACTIONS(1901),
    [sym_null_str] = ACTIONS(1901),
    [sym_smtp_address] = ACTIONS(1901),
    [sym_smtp_port_number] = ACTIONS(1901),
    [sym_smtp_username] = ACTIONS(1901),
    [sym_smtp_password] = ACTIONS(1901),
    [sym_smtp_require_tls] = ACTIONS(1901),
    [sym_smtp_from] = ACTIONS(1901),
    [sym_smtp_tls_true] = ACTIONS(1901),
    [sym_smtp_tls_false] = ACTIONS(1901),
    [sym_listview_on_row_select_event] = ACTIONS(1901),
    [sym_data_structure_unknown] = ACTIONS(1901),
    [sym_data_structure_datafeed] = ACTIONS(1901),
    [sym_data_structure_datarecordset] = ACTIONS(1901),
    [sym_data_structure_datarecord] = ACTIONS(1901),
    [sym_data_structure_datafield] = ACTIONS(1901),
    [sym_data_max_content_field_count] = ACTIONS(1901),
    [sym_data_max_metadata_field_count] = ACTIONS(1901),
    [sym_data_max_id_length] = ACTIONS(1901),
    [sym_data_max_name_length] = ACTIONS(1901),
    [sym_data_max_description_length] = ACTIONS(1901),
    [sym_data_max_type_length] = ACTIONS(1901),
    [sym_data_max_format_length] = ACTIONS(1901),
    [sym_data_max_source_length] = ACTIONS(1901),
    [sym_data_max_value_length] = ACTIONS(1901),
    [sym_data_max_label_length] = ACTIONS(1901),
    [sym_data_max_url_length] = ACTIONS(1901),
    [sym_max_last_login_info_length] = ACTIONS(1901),
    [sym_data_type_unknown] = ACTIONS(1901),
    [sym_data_type_string] = ACTIONS(1901),
    [sym_data_type_datetime] = ACTIONS(1901),
    [sym_data_type_date] = ACTIONS(1901),
    [sym_data_type_time] = ACTIONS(1901),
    [sym_data_type_image] = ACTIONS(1901),
    [sym_data_format_url] = ACTIONS(1901),
    [sym_data_format_phone] = ACTIONS(1901),
    [sym_data_format_email] = ACTIONS(1901),
    [sym_data_format_iso8601] = ACTIONS(1901),
    [sym_auditlog_priv] = ACTIONS(1901),
    [sym_configuration_priv] = ACTIONS(1901),
    [sym_deviceconfig_priv] = ACTIONS(1901),
    [sym_ftp_priv] = ACTIONS(1901),
    [sym_http_priv] = ACTIONS(1901),
    [sym_networkconfig_priv] = ACTIONS(1901),
    [sym_programport_priv] = ACTIONS(1901),
    [sym_remoteui_priv] = ACTIONS(1901),
    [sym_securitycontrol_priv] = ACTIONS(1901),
    [sym_softwaremanagement_priv] = ACTIONS(1901),
    [sym_terminal_priv] = ACTIONS(1901),
    [sym_tpadmin_priv] = ACTIONS(1901),
    [sym_user1_priv] = ACTIONS(1901),
    [sym_user2_priv] = ACTIONS(1901),
    [sym_user3_priv] = ACTIONS(1901),
    [sym_user4_priv] = ACTIONS(1901),
    [sym_usermanagement_priv] = ACTIONS(1901),
    [sym_valid_account] = ACTIONS(1901),
    [sym_err_name_invalid] = ACTIONS(1901),
    [sym_err_password_invalid] = ACTIONS(1901),
    [sym_err_invalid_account] = ACTIONS(1901),
    [sym_err_no_authorization] = ACTIONS(1901),
    [sym_err_invalid_parameter] = ACTIONS(1901),
    [sym_err_user_locked_out] = ACTIONS(1901),
    [sym_err_user_account_expired] = ACTIONS(1901),
    [sym_err_general_error] = ACTIONS(1901),
    [sym_err_authentication_error] = ACTIONS(1901),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1901),
    [sym_tls_validate_certificate] = ACTIONS(1901),
    [sym_tdata] = ACTIONS(1901),
    [sym_tchannel] = ACTIONS(1901),
    [sym_tlevel] = ACTIONS(1901),
    [sym_tbutton] = ACTIONS(1901),
    [sym_ttimeline] = ACTIONS(1901),
    [sym_tcustom] = ACTIONS(1901),
    [sym_url_struct] = ACTIONS(1901),
    [sym_dns_struct] = ACTIONS(1901),
    [sym_ip_address_struct] = ACTIONS(1901),
    [sym_dev_info_struct] = ACTIONS(1901),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1901),
    [sym_data_feed] = ACTIONS(1901),
    [sym_data_field] = ACTIONS(1901),
    [sym_data_record] = ACTIONS(1901),
    [sym_wc_data_feed] = ACTIONS(1901),
    [sym_wc_data_field] = ACTIONS(1901),
    [sym_wc_data_record] = ACTIONS(1901),
    [sym_last_login_info] = ACTIONS(1901),
    [sym____reserved___] = ACTIONS(1901),
    [sym_date] = ACTIONS(1901),
    [sym_day] = ACTIONS(1901),
    [sym_time] = ACTIONS(1901),
    [sym_ldate] = ACTIONS(1901),
    [sym_push_device] = ACTIONS(1901),
    [sym_push_channel] = ACTIONS(1901),
    [sym_push_devchan] = ACTIONS(1901),
    [sym_release_device] = ACTIONS(1901),
    [sym_release_channel] = ACTIONS(1901),
    [sym_release_devchan] = ACTIONS(1901),
    [sym_master_slot] = ACTIONS(1901),
    [sym_get_pulse_time] = ACTIONS(1901),
    [sym_get_timer] = ACTIONS(1901),
    [sym_master_sn] = ACTIONS(1901),
    [sym_system_number] = ACTIONS(1901),
    [sym_button] = ACTIONS(1901),
    [sym_dv_channel] = ACTIONS(1901),
    [sym_data] = ACTIONS(1901),
    [sym_level] = ACTIONS(1901),
    [sym_timeline] = ACTIONS(1901),
    [sym_channel] = ACTIONS(1901),
    [sym_custom] = ACTIONS(1901),
    [sym_atoi] = ACTIONS(1901),
    [sym_atol] = ACTIONS(1901),
    [sym_atof] = ACTIONS(1901),
    [sym_itoa] = ACTIONS(1901),
    [sym_itohex] = ACTIONS(1901),
    [sym_hextoi] = ACTIONS(1901),
    [sym_ftoa] = ACTIONS(1901),
    [sym_find_string] = ACTIONS(1901),
    [sym_left_string] = ACTIONS(1901),
    [sym_right_string] = ACTIONS(1901),
    [sym_mid_string] = ACTIONS(1901),
    [sym_remove_string] = ACTIONS(1901),
    [sym_get_buffer_string] = ACTIONS(1901),
    [sym_get_buffer_char] = ACTIONS(1901),
    [sym_get_multi_buffer_string] = ACTIONS(1901),
    [sym_length_string] = ACTIONS(1901),
    [sym_max_length_string] = ACTIONS(1901),
    [sym_set_length_string] = ACTIONS(1901),
    [sym_compare_string] = ACTIONS(1901),
    [sym_upper_string] = ACTIONS(1901),
    [sym_lower_string] = ACTIONS(1901),
    [sym_get_last] = ACTIONS(1901),
    [sym_date_to_day] = ACTIONS(1901),
    [sym_date_to_month] = ACTIONS(1901),
    [sym_date_to_year] = ACTIONS(1901),
    [sym_time_to_hour] = ACTIONS(1901),
    [sym_time_to_minute] = ACTIONS(1901),
    [sym_time_to_second] = ACTIONS(1901),
    [sym_day_of_week] = ACTIONS(1901),
    [sym_set_pulse_time] = ACTIONS(1901),
    [sym_set_timer] = ACTIONS(1901),
    [sym_file_open] = ACTIONS(1901),
    [sym_file_close] = ACTIONS(1901),
    [sym_file_read] = ACTIONS(1901),
    [sym_file_read_line] = ACTIONS(1901),
    [sym_file_write] = ACTIONS(1901),
    [sym_file_write_line] = ACTIONS(1901),
    [sym_file_copy] = ACTIONS(1901),
    [sym_file_delete] = ACTIONS(1901),
    [sym_file_rename] = ACTIONS(1901),
    [sym_file_dir] = ACTIONS(1901),
    [sym_file_getdir] = ACTIONS(1901),
    [sym_file_setdir] = ACTIONS(1901),
    [sym_file_create_dir] = ACTIONS(1901),
    [sym_file_remove_dir] = ACTIONS(1901),
    [sym_file_seek] = ACTIONS(1901),
    [sym_length_array] = ACTIONS(1901),
    [sym_max_length_array] = ACTIONS(1901),
    [sym_set_length_array] = ACTIONS(1901),
    [sym_ip_client_open] = ACTIONS(1901),
    [sym_ip_client_close] = ACTIONS(1901),
    [sym_ip_server_open] = ACTIONS(1901),
    [sym_ip_server_close] = ACTIONS(1901),
    [sym_ip_bound_client_open] = ACTIONS(1901),
    [sym_ip_mc_server_open] = ACTIONS(1901),
    [sym_ip_set_option] = ACTIONS(1901),
    [sym_ssh_client_open] = ACTIONS(1901),
    [sym_ssh_client_close] = ACTIONS(1901),
    [sym_tls_client_open] = ACTIONS(1901),
    [sym_tls_client_close] = ACTIONS(1901),
    [sym_do_push] = ACTIONS(1901),
    [sym_do_push_timed] = ACTIONS(1901),
    [sym_do_release] = ACTIONS(1901),
    [sym_device_id] = ACTIONS(1901),
    [sym_device_info] = ACTIONS(1901),
    [sym_device_id_string] = ACTIONS(1901),
    [sym_device_standby] = ACTIONS(1901),
    [sym_device_wake] = ACTIONS(1901),
    [sym_timeline_create] = ACTIONS(1901),
    [sym_timeline_kill] = ACTIONS(1901),
    [sym_timeline_reload] = ACTIONS(1901),
    [sym_timeline_pause] = ACTIONS(1901),
    [sym_timeline_restart] = ACTIONS(1901),
    [sym_timeline_set] = ACTIONS(1901),
    [sym_timeline_get] = ACTIONS(1901),
    [sym_timeline_active] = ACTIONS(1901),
    [sym_timeline_dynamic_id] = ACTIONS(1901),
    [sym_variable_to_string] = ACTIONS(1901),
    [sym_string_to_variable] = ACTIONS(1901),
    [sym_length_variable_to_string] = ACTIONS(1901),
    [sym_type_cast] = ACTIONS(1901),
    [sym_variable_to_xml] = ACTIONS(1901),
    [sym_xml_to_variable] = ACTIONS(1901),
    [sym_length_variable_to_xml] = ACTIONS(1901),
    [sym_redirect_string] = ACTIONS(1901),
    [sym_random_number] = ACTIONS(1901),
    [sym_get_unique_id] = ACTIONS(1901),
    [sym_get_system_number] = ACTIONS(1901),
    [sym_set_system_number] = ACTIONS(1901),
    [sym_get_serial_number] = ACTIONS(1901),
    [sym_get_master_build] = ACTIONS(1901),
    [sym_set_validation_code] = ACTIONS(1901),
    [sym_reboot] = ACTIONS(1901),
    [sym_load_duet_module] = ACTIONS(1901),
    [sym_unload_duet_module] = ACTIONS(1901),
    [sym_duet_mem_size_set] = ACTIONS(1901),
    [sym_duet_mem_size_get] = ACTIONS(1901),
    [sym_dynamic_application_device] = ACTIONS(1901),
    [sym_dynamic_polled_port] = ACTIONS(1901),
    [sym_static_port_binding] = ACTIONS(1901),
    [sym_static_ip_binding] = ACTIONS(1901),
    [sym_get_dns_list] = ACTIONS(1901),
    [sym_set_dns_list] = ACTIONS(1901),
    [sym_get_ip_address] = ACTIONS(1901),
    [sym_set_ip_address] = ACTIONS(1901),
    [sym_get_url_list] = ACTIONS(1901),
    [sym_add_url_entry] = ACTIONS(1901),
    [sym_delete_url_entry] = ACTIONS(1901),
    [sym_abs_value] = ACTIONS(1901),
    [sym_max_value] = ACTIONS(1901),
    [sym_min_value] = ACTIONS(1901),
    [sym_exp_value] = ACTIONS(1901),
    [sym_log_value] = ACTIONS(1901),
    [sym_log10_value] = ACTIONS(1901),
    [sym_power_value] = ACTIONS(1901),
    [sym_sqrt_value] = ACTIONS(1901),
    [sym_raw_be] = ACTIONS(1901),
    [sym_raw_le] = ACTIONS(1901),
    [sym_internal_queue_size_set] = ACTIONS(1901),
    [sym_internal_queue_size_get] = ACTIONS(1901),
    [sym_internal_threshold_set] = ACTIONS(1901),
    [sym_internal_threshold_get] = ACTIONS(1901),
    [sym_rebuilt_event] = ACTIONS(1901),
    [sym_do_custom_event] = ACTIONS(1901),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1901),
    [sym_clkmgr_set_clk_source] = ACTIONS(1901),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1901),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1901),
    [sym_clkmgr_get_timezone] = ACTIONS(1901),
    [sym_clkmgr_set_timezone] = ACTIONS(1901),
    [sym_clkmgr_get_resync_period] = ACTIONS(1901),
    [sym_clkmgr_set_resync_period] = ACTIONS(1901),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1901),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1901),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1901),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1901),
    [sym_clkmgr_get_timeservers] = ACTIONS(1901),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1901),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1901),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1901),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1901),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1901),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1901),
    [sym_set_log_level] = ACTIONS(1901),
    [sym_get_log_level] = ACTIONS(1901),
    [sym_amx_log] = ACTIONS(1901),
    [sym_smtp_server_config_set] = ACTIONS(1901),
    [sym_smtp_server_config_get] = ACTIONS(1901),
    [sym_smtp_send] = ACTIONS(1901),
    [sym_validate_netlinx_account] = ACTIONS(1901),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1901),
    [sym_authenticate_certificate] = ACTIONS(1901),
    [sym_audit_netlinx_session_event] = ACTIONS(1901),
    [sym_audit_netlinx_generic_event] = ACTIONS(1901),
    [sym_get_available_flash_disk_space] = ACTIONS(1901),
    [sym_get_max_flash_disk_space] = ACTIONS(1901),
    [sym_data_create_feed] = ACTIONS(1901),
    [sym_data_delete_feed] = ACTIONS(1901),
    [sym_data_publish_feed] = ACTIONS(1901),
    [sym_data_get_published_feed] = ACTIONS(1901),
    [sym_data_add_record] = ACTIONS(1901),
    [sym_data_get_event_record] = ACTIONS(1901),
    [sym__wc_data_create_feed] = ACTIONS(1901),
    [sym__wc_data_add_record] = ACTIONS(1901),
    [sym__wc_data_get_event_record] = ACTIONS(1901),
    [sym_format] = ACTIONS(1901),
    [sym_astro_clock] = ACTIONS(1901),
    [sym_set_outdoor_temperature] = ACTIONS(1901),
    [sym_set_virtual_level_count] = ACTIONS(1901),
    [sym_set_virtual_channel_count] = ACTIONS(1901),
    [sym_set_virtual_port_count] = ACTIONS(1901),
    [sym__wc_encode_int] = ACTIONS(1901),
    [sym__wc_decode_int] = ACTIONS(1901),
    [sym__wc_int] = ACTIONS(1901),
    [sym__wc_to_ch_int] = ACTIONS(1901),
    [sym__ch_to_wc_int] = ACTIONS(1901),
    [sym__wc_find_string_int] = ACTIONS(1901),
    [sym__wc_left_string_int] = ACTIONS(1901),
    [sym__wc_lower_string_int] = ACTIONS(1901),
    [sym__wc_mid_string_int] = ACTIONS(1901),
    [sym__wc_remove_string_int] = ACTIONS(1901),
    [sym__wc_right_string_int] = ACTIONS(1901),
    [sym__wc_upper_string_int] = ACTIONS(1901),
    [sym__wc_compare_string_int] = ACTIONS(1901),
    [sym__wc_get_buffer_char_int] = ACTIONS(1901),
    [sym__wc_get_buffer_string_int] = ACTIONS(1901),
    [sym__wc_concat_string_int] = ACTIONS(1901),
    [sym__wc_file_open_int] = ACTIONS(1901),
    [sym__wc_file_close_int] = ACTIONS(1901),
    [sym__wc_file_read_int] = ACTIONS(1901),
    [sym__wc_file_read_line_int] = ACTIONS(1901),
    [sym__wc_file_write_int] = ACTIONS(1901),
    [sym__wc_file_write_line_int] = ACTIONS(1901),
    [aux_sym_preproc_include_token1] = ACTIONS(1901),
    [aux_sym_preproc_define_token1] = ACTIONS(1903),
    [aux_sym_preproc_warn_token1] = ACTIONS(1903),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1903),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1903),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1903),
    [anon_sym_LBRACK] = ACTIONS(1903),
    [anon_sym_LBRACE] = ACTIONS(1903),
    [anon_sym_LPAREN] = ACTIONS(1901),
    [anon_sym_BANG] = ACTIONS(1903),
    [anon_sym_TILDE] = ACTIONS(1903),
    [anon_sym_DASH] = ACTIONS(1901),
    [anon_sym_PLUS] = ACTIONS(1901),
    [anon_sym_DASH_DASH] = ACTIONS(1903),
    [anon_sym_PLUS_PLUS] = ACTIONS(1903),
    [anon_sym_DQUOTE] = ACTIONS(1903),
    [anon_sym_SQUOTE] = ACTIONS(1903),
    [sym_number_literal] = ACTIONS(1903),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(452)] = {
    [ts_builtin_sym_end] = ACTIONS(1907),
    [sym_identifier] = ACTIONS(1905),
    [sym_program_name_keyword] = ACTIONS(1905),
    [sym_module_name_keyword] = ACTIONS(1905),
    [sym_define_device_keyword] = ACTIONS(1905),
    [sym_define_combine_keyword] = ACTIONS(1905),
    [sym_define_constant_keyword] = ACTIONS(1905),
    [sym_define_type_keyword] = ACTIONS(1905),
    [sym_define_variable_keyword] = ACTIONS(1905),
    [sym_define_system_variable_keyword] = ACTIONS(1905),
    [sym_define_connect_level_keyword] = ACTIONS(1905),
    [sym_define_latching_keyword] = ACTIONS(1905),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1905),
    [sym_define_toggling_keyword] = ACTIONS(1905),
    [sym_define_call_keyword] = ACTIONS(1905),
    [sym_define_function_keyword] = ACTIONS(1905),
    [sym_define_library_function_keyword] = ACTIONS(1905),
    [sym_define_start_keyword] = ACTIONS(1905),
    [sym_define_event_keyword] = ACTIONS(1905),
    [sym_define_program_keyword] = ACTIONS(1905),
    [sym_define_module_keyword] = ACTIONS(1905),
    [sym_char_keyword] = ACTIONS(1905),
    [sym_widechar_keyword] = ACTIONS(1905),
    [sym_integer_keyword] = ACTIONS(1905),
    [sym_sinteger_keyword] = ACTIONS(1905),
    [sym_long_keyword] = ACTIONS(1905),
    [sym_slong_keyword] = ACTIONS(1905),
    [sym_float_keyword] = ACTIONS(1905),
    [sym_double_keyword] = ACTIONS(1905),
    [sym_variant_keyword] = ACTIONS(1905),
    [sym_variantarray_keyword] = ACTIONS(1905),
    [sym_local_var_keyword] = ACTIONS(1905),
    [sym_stack_var_keyword] = ACTIONS(1905),
    [sym_constant_keyword] = ACTIONS(1905),
    [sym_volatile_keyword] = ACTIONS(1905),
    [sym_non_volatile_keyword] = ACTIONS(1905),
    [sym_persistent_keyword] = ACTIONS(1905),
    [sym_dev_keyword] = ACTIONS(1905),
    [sym_devlev_keyword] = ACTIONS(1905),
    [sym_devchan_keyword] = ACTIONS(1905),
    [sym_if_keyword] = ACTIONS(1905),
    [sym_switch_keyword] = ACTIONS(1905),
    [sym_case_keyword] = ACTIONS(1905),
    [sym_default_keyword] = ACTIONS(1905),
    [sym_while_keyword] = ACTIONS(1905),
    [sym_for_keyword] = ACTIONS(1905),
    [sym_break_keyword] = ACTIONS(1905),
    [sym_continue_keyword] = ACTIONS(1905),
    [sym_return_keyword] = ACTIONS(1905),
    [sym_select_keyword] = ACTIONS(1905),
    [sym_active_keyword] = ACTIONS(1905),
    [sym_struct_keyword] = ACTIONS(1905),
    [sym_structure_keyword] = ACTIONS(1905),
    [sym_button_event_keyword] = ACTIONS(1905),
    [sym_channel_event_keyword] = ACTIONS(1905),
    [sym_level_event_keyword] = ACTIONS(1905),
    [sym_data_event_keyword] = ACTIONS(1905),
    [sym_timeline_event_keyword] = ACTIONS(1905),
    [sym_custom_event_keyword] = ACTIONS(1905),
    [sym_push_keyword] = ACTIONS(1905),
    [sym_release_keyword] = ACTIONS(1905),
    [aux_sym_on_keyword_token1] = ACTIONS(1905),
    [aux_sym_off_keyword_token1] = ACTIONS(1905),
    [sym_send_level_keyword] = ACTIONS(1905),
    [sym_send_string_keyword] = ACTIONS(1905),
    [sym_send_command_keyword] = ACTIONS(1905),
    [sym_clear_buffer_keyword] = ACTIONS(1905),
    [sym_create_buffer_keyword] = ACTIONS(1905),
    [sym_create_multi_buffer_keyword] = ACTIONS(1905),
    [sym_call_keyword] = ACTIONS(1905),
    [sym_system_call_keyword] = ACTIONS(1905),
    [sym_devchan_to_keyword] = ACTIONS(1905),
    [sym_devchan_min_to_keyword] = ACTIONS(1905),
    [sym_devchan_total_off_keyword] = ACTIONS(1905),
    [sym_devchan_pulse_keyword] = ACTIONS(1905),
    [sym_wait_keyword] = ACTIONS(1905),
    [sym_cancel_wait_keyword] = ACTIONS(1905),
    [sym_wait_until_keyword] = ACTIONS(1905),
    [sym_cancel_wait_until_keyword] = ACTIONS(1905),
    [sym_cancel_all_wait_keyword] = ACTIONS(1905),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1905),
    [sym_not] = ACTIONS(1905),
    [sym_bnot] = ACTIONS(1905),
    [sym___file__] = ACTIONS(1905),
    [sym___date__] = ACTIONS(1905),
    [sym___ldate__] = ACTIONS(1905),
    [sym___time__] = ACTIONS(1905),
    [sym___line__] = ACTIONS(1905),
    [sym___name__] = ACTIONS(1905),
    [sym___netlinx__] = ACTIONS(1905),
    [sym_netlinx_axi_version] = ACTIONS(1905),
    [sym_url_flg_tcp] = ACTIONS(1905),
    [sym_url_flg_accinfopresent] = ACTIONS(1905),
    [sym_url_flg_temp] = ACTIONS(1905),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1905),
    [sym_url_flg_stat_mask] = ACTIONS(1905),
    [sym_url_flg_stat_lookup] = ACTIONS(1905),
    [sym_url_flg_stat_connecting] = ACTIONS(1905),
    [sym_url_flg_stat_waiting] = ACTIONS(1905),
    [sym_url_flg_stat_connected] = ACTIONS(1905),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1905),
    [sym_first_virtual_device] = ACTIONS(1905),
    [sym_first_local_port] = ACTIONS(1905),
    [sym_dynamic_virtual_device] = ACTIONS(1905),
    [sym_timeline_once] = ACTIONS(1905),
    [sym_timeline_repeat] = ACTIONS(1905),
    [sym_timeline_absolute] = ACTIONS(1905),
    [sym_timeline_relative] = ACTIONS(1905),
    [sym_do_push_timed_infinite] = ACTIONS(1905),
    [sym_source_type_no_address] = ACTIONS(1905),
    [sym_source_type_neuron_id] = ACTIONS(1905),
    [sym_source_type_ip_address] = ACTIONS(1905),
    [sym_source_type_axlink] = ACTIONS(1905),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1905),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1905),
    [sym_source_type_ip_socket_address] = ACTIONS(1905),
    [sym_source_type_rs232] = ACTIONS(1905),
    [sym_source_type_internal] = ACTIONS(1905),
    [sym_source_type_mac_address] = ACTIONS(1905),
    [sym_source_type_ipv4_port] = ACTIONS(1905),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1905),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1905),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1905),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1905),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1905),
    [sym_true] = ACTIONS(1905),
    [sym_false] = ACTIONS(1905),
    [sym_file_read_only] = ACTIONS(1905),
    [sym_file_rw_new] = ACTIONS(1905),
    [sym_file_rw_append] = ACTIONS(1905),
    [sym_ip_tcp] = ACTIONS(1905),
    [sym_ip_udp] = ACTIONS(1905),
    [sym_ip_udp_2way] = ACTIONS(1905),
    [sym_xml_encode_types] = ACTIONS(1905),
    [sym_xml_encode_char_as_list] = ACTIONS(1905),
    [sym_xml_encode_le] = ACTIONS(1905),
    [sym_xml_decode_types] = ACTIONS(1905),
    [sym_xml_decode_no_preserve] = ACTIONS(1905),
    [sym_duet_dev_type_display_device] = ACTIONS(1905),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1905),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1905),
    [sym_duet_dev_type_io_device] = ACTIONS(1905),
    [sym_duet_dev_type_relay_device] = ACTIONS(1905),
    [sym_duet_dev_type_ups] = ACTIONS(1905),
    [sym_duet_dev_type_amplifier] = ACTIONS(1905),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1905),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1905),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1905),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1905),
    [sym_duet_dev_type_camera] = ACTIONS(1905),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1905),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1905),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1905),
    [sym_duet_dev_type_dss] = ACTIONS(1905),
    [sym_duet_dev_type_dvr] = ACTIONS(1905),
    [sym_duet_dev_type_disc_device] = ACTIONS(1905),
    [sym_duet_dev_type_document_camera] = ACTIONS(1905),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1905),
    [sym_duet_dev_type_hvac] = ACTIONS(1905),
    [sym_duet_dev_type_keypad] = ACTIONS(1905),
    [sym_duet_dev_type_light] = ACTIONS(1905),
    [sym_duet_dev_type_monitor] = ACTIONS(1905),
    [sym_duet_dev_type_motor] = ACTIONS(1905),
    [sym_duet_dev_type_multi_window] = ACTIONS(1905),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1905),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1905),
    [sym_duet_dev_type_receiver] = ACTIONS(1905),
    [sym_duet_dev_type_security_system] = ACTIONS(1905),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1905),
    [sym_duet_dev_type_settop_box] = ACTIONS(1905),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1905),
    [sym_duet_dev_type_switcher] = ACTIONS(1905),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1905),
    [sym_duet_dev_type_tv] = ACTIONS(1905),
    [sym_duet_dev_type_utility] = ACTIONS(1905),
    [sym_duet_dev_type_vcr] = ACTIONS(1905),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1905),
    [sym_duet_dev_type_video_processor] = ACTIONS(1905),
    [sym_duet_dev_type_video_projector] = ACTIONS(1905),
    [sym_duet_dev_type_video_wall] = ACTIONS(1905),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1905),
    [sym_duet_dev_type_weather] = ACTIONS(1905),
    [sym_duet_dev_not_polled] = ACTIONS(1905),
    [sym_duet_dev_polled] = ACTIONS(1905),
    [sym_clkmgr_mode_network] = ACTIONS(1905),
    [sym_clkmgr_mode_standalone] = ACTIONS(1905),
    [sym_normal_standby] = ACTIONS(1905),
    [sym_normal_wake] = ACTIONS(1905),
    [sym_ip_multicast_ttl_option] = ACTIONS(1905),
    [sym_ip_ttl_subnet] = ACTIONS(1905),
    [sym_ip_ttl_site] = ACTIONS(1905),
    [sym_ip_ttl_region] = ACTIONS(1905),
    [sym_ip_ttl_continent] = ACTIONS(1905),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1905),
    [sym_ip_nodelay_on] = ACTIONS(1905),
    [sym_ip_nodelay_off] = ACTIONS(1905),
    [sym_amx_error] = ACTIONS(1905),
    [sym_amx_warning] = ACTIONS(1905),
    [sym_amx_info] = ACTIONS(1905),
    [sym_amx_debug] = ACTIONS(1905),
    [sym_netlinx_login_success] = ACTIONS(1905),
    [sym_netlinx_login_fail] = ACTIONS(1905),
    [sym_netlinx_logout] = ACTIONS(1905),
    [sym_null_str] = ACTIONS(1905),
    [sym_smtp_address] = ACTIONS(1905),
    [sym_smtp_port_number] = ACTIONS(1905),
    [sym_smtp_username] = ACTIONS(1905),
    [sym_smtp_password] = ACTIONS(1905),
    [sym_smtp_require_tls] = ACTIONS(1905),
    [sym_smtp_from] = ACTIONS(1905),
    [sym_smtp_tls_true] = ACTIONS(1905),
    [sym_smtp_tls_false] = ACTIONS(1905),
    [sym_listview_on_row_select_event] = ACTIONS(1905),
    [sym_data_structure_unknown] = ACTIONS(1905),
    [sym_data_structure_datafeed] = ACTIONS(1905),
    [sym_data_structure_datarecordset] = ACTIONS(1905),
    [sym_data_structure_datarecord] = ACTIONS(1905),
    [sym_data_structure_datafield] = ACTIONS(1905),
    [sym_data_max_content_field_count] = ACTIONS(1905),
    [sym_data_max_metadata_field_count] = ACTIONS(1905),
    [sym_data_max_id_length] = ACTIONS(1905),
    [sym_data_max_name_length] = ACTIONS(1905),
    [sym_data_max_description_length] = ACTIONS(1905),
    [sym_data_max_type_length] = ACTIONS(1905),
    [sym_data_max_format_length] = ACTIONS(1905),
    [sym_data_max_source_length] = ACTIONS(1905),
    [sym_data_max_value_length] = ACTIONS(1905),
    [sym_data_max_label_length] = ACTIONS(1905),
    [sym_data_max_url_length] = ACTIONS(1905),
    [sym_max_last_login_info_length] = ACTIONS(1905),
    [sym_data_type_unknown] = ACTIONS(1905),
    [sym_data_type_string] = ACTIONS(1905),
    [sym_data_type_datetime] = ACTIONS(1905),
    [sym_data_type_date] = ACTIONS(1905),
    [sym_data_type_time] = ACTIONS(1905),
    [sym_data_type_image] = ACTIONS(1905),
    [sym_data_format_url] = ACTIONS(1905),
    [sym_data_format_phone] = ACTIONS(1905),
    [sym_data_format_email] = ACTIONS(1905),
    [sym_data_format_iso8601] = ACTIONS(1905),
    [sym_auditlog_priv] = ACTIONS(1905),
    [sym_configuration_priv] = ACTIONS(1905),
    [sym_deviceconfig_priv] = ACTIONS(1905),
    [sym_ftp_priv] = ACTIONS(1905),
    [sym_http_priv] = ACTIONS(1905),
    [sym_networkconfig_priv] = ACTIONS(1905),
    [sym_programport_priv] = ACTIONS(1905),
    [sym_remoteui_priv] = ACTIONS(1905),
    [sym_securitycontrol_priv] = ACTIONS(1905),
    [sym_softwaremanagement_priv] = ACTIONS(1905),
    [sym_terminal_priv] = ACTIONS(1905),
    [sym_tpadmin_priv] = ACTIONS(1905),
    [sym_user1_priv] = ACTIONS(1905),
    [sym_user2_priv] = ACTIONS(1905),
    [sym_user3_priv] = ACTIONS(1905),
    [sym_user4_priv] = ACTIONS(1905),
    [sym_usermanagement_priv] = ACTIONS(1905),
    [sym_valid_account] = ACTIONS(1905),
    [sym_err_name_invalid] = ACTIONS(1905),
    [sym_err_password_invalid] = ACTIONS(1905),
    [sym_err_invalid_account] = ACTIONS(1905),
    [sym_err_no_authorization] = ACTIONS(1905),
    [sym_err_invalid_parameter] = ACTIONS(1905),
    [sym_err_user_locked_out] = ACTIONS(1905),
    [sym_err_user_account_expired] = ACTIONS(1905),
    [sym_err_general_error] = ACTIONS(1905),
    [sym_err_authentication_error] = ACTIONS(1905),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1905),
    [sym_tls_validate_certificate] = ACTIONS(1905),
    [sym_tdata] = ACTIONS(1905),
    [sym_tchannel] = ACTIONS(1905),
    [sym_tlevel] = ACTIONS(1905),
    [sym_tbutton] = ACTIONS(1905),
    [sym_ttimeline] = ACTIONS(1905),
    [sym_tcustom] = ACTIONS(1905),
    [sym_url_struct] = ACTIONS(1905),
    [sym_dns_struct] = ACTIONS(1905),
    [sym_ip_address_struct] = ACTIONS(1905),
    [sym_dev_info_struct] = ACTIONS(1905),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1905),
    [sym_data_feed] = ACTIONS(1905),
    [sym_data_field] = ACTIONS(1905),
    [sym_data_record] = ACTIONS(1905),
    [sym_wc_data_feed] = ACTIONS(1905),
    [sym_wc_data_field] = ACTIONS(1905),
    [sym_wc_data_record] = ACTIONS(1905),
    [sym_last_login_info] = ACTIONS(1905),
    [sym____reserved___] = ACTIONS(1905),
    [sym_date] = ACTIONS(1905),
    [sym_day] = ACTIONS(1905),
    [sym_time] = ACTIONS(1905),
    [sym_ldate] = ACTIONS(1905),
    [sym_push_device] = ACTIONS(1905),
    [sym_push_channel] = ACTIONS(1905),
    [sym_push_devchan] = ACTIONS(1905),
    [sym_release_device] = ACTIONS(1905),
    [sym_release_channel] = ACTIONS(1905),
    [sym_release_devchan] = ACTIONS(1905),
    [sym_master_slot] = ACTIONS(1905),
    [sym_get_pulse_time] = ACTIONS(1905),
    [sym_get_timer] = ACTIONS(1905),
    [sym_master_sn] = ACTIONS(1905),
    [sym_system_number] = ACTIONS(1905),
    [sym_button] = ACTIONS(1905),
    [sym_dv_channel] = ACTIONS(1905),
    [sym_data] = ACTIONS(1905),
    [sym_level] = ACTIONS(1905),
    [sym_timeline] = ACTIONS(1905),
    [sym_channel] = ACTIONS(1905),
    [sym_custom] = ACTIONS(1905),
    [sym_atoi] = ACTIONS(1905),
    [sym_atol] = ACTIONS(1905),
    [sym_atof] = ACTIONS(1905),
    [sym_itoa] = ACTIONS(1905),
    [sym_itohex] = ACTIONS(1905),
    [sym_hextoi] = ACTIONS(1905),
    [sym_ftoa] = ACTIONS(1905),
    [sym_find_string] = ACTIONS(1905),
    [sym_left_string] = ACTIONS(1905),
    [sym_right_string] = ACTIONS(1905),
    [sym_mid_string] = ACTIONS(1905),
    [sym_remove_string] = ACTIONS(1905),
    [sym_get_buffer_string] = ACTIONS(1905),
    [sym_get_buffer_char] = ACTIONS(1905),
    [sym_get_multi_buffer_string] = ACTIONS(1905),
    [sym_length_string] = ACTIONS(1905),
    [sym_max_length_string] = ACTIONS(1905),
    [sym_set_length_string] = ACTIONS(1905),
    [sym_compare_string] = ACTIONS(1905),
    [sym_upper_string] = ACTIONS(1905),
    [sym_lower_string] = ACTIONS(1905),
    [sym_get_last] = ACTIONS(1905),
    [sym_date_to_day] = ACTIONS(1905),
    [sym_date_to_month] = ACTIONS(1905),
    [sym_date_to_year] = ACTIONS(1905),
    [sym_time_to_hour] = ACTIONS(1905),
    [sym_time_to_minute] = ACTIONS(1905),
    [sym_time_to_second] = ACTIONS(1905),
    [sym_day_of_week] = ACTIONS(1905),
    [sym_set_pulse_time] = ACTIONS(1905),
    [sym_set_timer] = ACTIONS(1905),
    [sym_file_open] = ACTIONS(1905),
    [sym_file_close] = ACTIONS(1905),
    [sym_file_read] = ACTIONS(1905),
    [sym_file_read_line] = ACTIONS(1905),
    [sym_file_write] = ACTIONS(1905),
    [sym_file_write_line] = ACTIONS(1905),
    [sym_file_copy] = ACTIONS(1905),
    [sym_file_delete] = ACTIONS(1905),
    [sym_file_rename] = ACTIONS(1905),
    [sym_file_dir] = ACTIONS(1905),
    [sym_file_getdir] = ACTIONS(1905),
    [sym_file_setdir] = ACTIONS(1905),
    [sym_file_create_dir] = ACTIONS(1905),
    [sym_file_remove_dir] = ACTIONS(1905),
    [sym_file_seek] = ACTIONS(1905),
    [sym_length_array] = ACTIONS(1905),
    [sym_max_length_array] = ACTIONS(1905),
    [sym_set_length_array] = ACTIONS(1905),
    [sym_ip_client_open] = ACTIONS(1905),
    [sym_ip_client_close] = ACTIONS(1905),
    [sym_ip_server_open] = ACTIONS(1905),
    [sym_ip_server_close] = ACTIONS(1905),
    [sym_ip_bound_client_open] = ACTIONS(1905),
    [sym_ip_mc_server_open] = ACTIONS(1905),
    [sym_ip_set_option] = ACTIONS(1905),
    [sym_ssh_client_open] = ACTIONS(1905),
    [sym_ssh_client_close] = ACTIONS(1905),
    [sym_tls_client_open] = ACTIONS(1905),
    [sym_tls_client_close] = ACTIONS(1905),
    [sym_do_push] = ACTIONS(1905),
    [sym_do_push_timed] = ACTIONS(1905),
    [sym_do_release] = ACTIONS(1905),
    [sym_device_id] = ACTIONS(1905),
    [sym_device_info] = ACTIONS(1905),
    [sym_device_id_string] = ACTIONS(1905),
    [sym_device_standby] = ACTIONS(1905),
    [sym_device_wake] = ACTIONS(1905),
    [sym_timeline_create] = ACTIONS(1905),
    [sym_timeline_kill] = ACTIONS(1905),
    [sym_timeline_reload] = ACTIONS(1905),
    [sym_timeline_pause] = ACTIONS(1905),
    [sym_timeline_restart] = ACTIONS(1905),
    [sym_timeline_set] = ACTIONS(1905),
    [sym_timeline_get] = ACTIONS(1905),
    [sym_timeline_active] = ACTIONS(1905),
    [sym_timeline_dynamic_id] = ACTIONS(1905),
    [sym_variable_to_string] = ACTIONS(1905),
    [sym_string_to_variable] = ACTIONS(1905),
    [sym_length_variable_to_string] = ACTIONS(1905),
    [sym_type_cast] = ACTIONS(1905),
    [sym_variable_to_xml] = ACTIONS(1905),
    [sym_xml_to_variable] = ACTIONS(1905),
    [sym_length_variable_to_xml] = ACTIONS(1905),
    [sym_redirect_string] = ACTIONS(1905),
    [sym_random_number] = ACTIONS(1905),
    [sym_get_unique_id] = ACTIONS(1905),
    [sym_get_system_number] = ACTIONS(1905),
    [sym_set_system_number] = ACTIONS(1905),
    [sym_get_serial_number] = ACTIONS(1905),
    [sym_get_master_build] = ACTIONS(1905),
    [sym_set_validation_code] = ACTIONS(1905),
    [sym_reboot] = ACTIONS(1905),
    [sym_load_duet_module] = ACTIONS(1905),
    [sym_unload_duet_module] = ACTIONS(1905),
    [sym_duet_mem_size_set] = ACTIONS(1905),
    [sym_duet_mem_size_get] = ACTIONS(1905),
    [sym_dynamic_application_device] = ACTIONS(1905),
    [sym_dynamic_polled_port] = ACTIONS(1905),
    [sym_static_port_binding] = ACTIONS(1905),
    [sym_static_ip_binding] = ACTIONS(1905),
    [sym_get_dns_list] = ACTIONS(1905),
    [sym_set_dns_list] = ACTIONS(1905),
    [sym_get_ip_address] = ACTIONS(1905),
    [sym_set_ip_address] = ACTIONS(1905),
    [sym_get_url_list] = ACTIONS(1905),
    [sym_add_url_entry] = ACTIONS(1905),
    [sym_delete_url_entry] = ACTIONS(1905),
    [sym_abs_value] = ACTIONS(1905),
    [sym_max_value] = ACTIONS(1905),
    [sym_min_value] = ACTIONS(1905),
    [sym_exp_value] = ACTIONS(1905),
    [sym_log_value] = ACTIONS(1905),
    [sym_log10_value] = ACTIONS(1905),
    [sym_power_value] = ACTIONS(1905),
    [sym_sqrt_value] = ACTIONS(1905),
    [sym_raw_be] = ACTIONS(1905),
    [sym_raw_le] = ACTIONS(1905),
    [sym_internal_queue_size_set] = ACTIONS(1905),
    [sym_internal_queue_size_get] = ACTIONS(1905),
    [sym_internal_threshold_set] = ACTIONS(1905),
    [sym_internal_threshold_get] = ACTIONS(1905),
    [sym_rebuilt_event] = ACTIONS(1905),
    [sym_do_custom_event] = ACTIONS(1905),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1905),
    [sym_clkmgr_set_clk_source] = ACTIONS(1905),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1905),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1905),
    [sym_clkmgr_get_timezone] = ACTIONS(1905),
    [sym_clkmgr_set_timezone] = ACTIONS(1905),
    [sym_clkmgr_get_resync_period] = ACTIONS(1905),
    [sym_clkmgr_set_resync_period] = ACTIONS(1905),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1905),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1905),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1905),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1905),
    [sym_clkmgr_get_timeservers] = ACTIONS(1905),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1905),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1905),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1905),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1905),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1905),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1905),
    [sym_set_log_level] = ACTIONS(1905),
    [sym_get_log_level] = ACTIONS(1905),
    [sym_amx_log] = ACTIONS(1905),
    [sym_smtp_server_config_set] = ACTIONS(1905),
    [sym_smtp_server_config_get] = ACTIONS(1905),
    [sym_smtp_send] = ACTIONS(1905),
    [sym_validate_netlinx_account] = ACTIONS(1905),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1905),
    [sym_authenticate_certificate] = ACTIONS(1905),
    [sym_audit_netlinx_session_event] = ACTIONS(1905),
    [sym_audit_netlinx_generic_event] = ACTIONS(1905),
    [sym_get_available_flash_disk_space] = ACTIONS(1905),
    [sym_get_max_flash_disk_space] = ACTIONS(1905),
    [sym_data_create_feed] = ACTIONS(1905),
    [sym_data_delete_feed] = ACTIONS(1905),
    [sym_data_publish_feed] = ACTIONS(1905),
    [sym_data_get_published_feed] = ACTIONS(1905),
    [sym_data_add_record] = ACTIONS(1905),
    [sym_data_get_event_record] = ACTIONS(1905),
    [sym__wc_data_create_feed] = ACTIONS(1905),
    [sym__wc_data_add_record] = ACTIONS(1905),
    [sym__wc_data_get_event_record] = ACTIONS(1905),
    [sym_format] = ACTIONS(1905),
    [sym_astro_clock] = ACTIONS(1905),
    [sym_set_outdoor_temperature] = ACTIONS(1905),
    [sym_set_virtual_level_count] = ACTIONS(1905),
    [sym_set_virtual_channel_count] = ACTIONS(1905),
    [sym_set_virtual_port_count] = ACTIONS(1905),
    [sym__wc_encode_int] = ACTIONS(1905),
    [sym__wc_decode_int] = ACTIONS(1905),
    [sym__wc_int] = ACTIONS(1905),
    [sym__wc_to_ch_int] = ACTIONS(1905),
    [sym__ch_to_wc_int] = ACTIONS(1905),
    [sym__wc_find_string_int] = ACTIONS(1905),
    [sym__wc_left_string_int] = ACTIONS(1905),
    [sym__wc_lower_string_int] = ACTIONS(1905),
    [sym__wc_mid_string_int] = ACTIONS(1905),
    [sym__wc_remove_string_int] = ACTIONS(1905),
    [sym__wc_right_string_int] = ACTIONS(1905),
    [sym__wc_upper_string_int] = ACTIONS(1905),
    [sym__wc_compare_string_int] = ACTIONS(1905),
    [sym__wc_get_buffer_char_int] = ACTIONS(1905),
    [sym__wc_get_buffer_string_int] = ACTIONS(1905),
    [sym__wc_concat_string_int] = ACTIONS(1905),
    [sym__wc_file_open_int] = ACTIONS(1905),
    [sym__wc_file_close_int] = ACTIONS(1905),
    [sym__wc_file_read_int] = ACTIONS(1905),
    [sym__wc_file_read_line_int] = ACTIONS(1905),
    [sym__wc_file_write_int] = ACTIONS(1905),
    [sym__wc_file_write_line_int] = ACTIONS(1905),
    [aux_sym_preproc_include_token1] = ACTIONS(1905),
    [aux_sym_preproc_define_token1] = ACTIONS(1907),
    [aux_sym_preproc_warn_token1] = ACTIONS(1907),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1907),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1907),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1907),
    [anon_sym_LBRACK] = ACTIONS(1907),
    [anon_sym_LBRACE] = ACTIONS(1907),
    [anon_sym_LPAREN] = ACTIONS(1905),
    [anon_sym_BANG] = ACTIONS(1907),
    [anon_sym_TILDE] = ACTIONS(1907),
    [anon_sym_DASH] = ACTIONS(1905),
    [anon_sym_PLUS] = ACTIONS(1905),
    [anon_sym_DASH_DASH] = ACTIONS(1907),
    [anon_sym_PLUS_PLUS] = ACTIONS(1907),
    [anon_sym_DQUOTE] = ACTIONS(1907),
    [anon_sym_SQUOTE] = ACTIONS(1907),
    [sym_number_literal] = ACTIONS(1907),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(453)] = {
    [ts_builtin_sym_end] = ACTIONS(1599),
    [sym_identifier] = ACTIONS(1597),
    [sym_program_name_keyword] = ACTIONS(1597),
    [sym_module_name_keyword] = ACTIONS(1597),
    [sym_define_device_keyword] = ACTIONS(1597),
    [sym_define_combine_keyword] = ACTIONS(1597),
    [sym_define_constant_keyword] = ACTIONS(1597),
    [sym_define_type_keyword] = ACTIONS(1597),
    [sym_define_variable_keyword] = ACTIONS(1597),
    [sym_define_system_variable_keyword] = ACTIONS(1597),
    [sym_define_connect_level_keyword] = ACTIONS(1597),
    [sym_define_latching_keyword] = ACTIONS(1597),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1597),
    [sym_define_toggling_keyword] = ACTIONS(1597),
    [sym_define_call_keyword] = ACTIONS(1597),
    [sym_define_function_keyword] = ACTIONS(1597),
    [sym_define_library_function_keyword] = ACTIONS(1597),
    [sym_define_start_keyword] = ACTIONS(1597),
    [sym_define_event_keyword] = ACTIONS(1597),
    [sym_define_program_keyword] = ACTIONS(1597),
    [sym_define_module_keyword] = ACTIONS(1597),
    [sym_char_keyword] = ACTIONS(1597),
    [sym_widechar_keyword] = ACTIONS(1597),
    [sym_integer_keyword] = ACTIONS(1597),
    [sym_sinteger_keyword] = ACTIONS(1597),
    [sym_long_keyword] = ACTIONS(1597),
    [sym_slong_keyword] = ACTIONS(1597),
    [sym_float_keyword] = ACTIONS(1597),
    [sym_double_keyword] = ACTIONS(1597),
    [sym_variant_keyword] = ACTIONS(1597),
    [sym_variantarray_keyword] = ACTIONS(1597),
    [sym_local_var_keyword] = ACTIONS(1597),
    [sym_stack_var_keyword] = ACTIONS(1597),
    [sym_constant_keyword] = ACTIONS(1597),
    [sym_volatile_keyword] = ACTIONS(1597),
    [sym_non_volatile_keyword] = ACTIONS(1597),
    [sym_persistent_keyword] = ACTIONS(1597),
    [sym_dev_keyword] = ACTIONS(1597),
    [sym_devlev_keyword] = ACTIONS(1597),
    [sym_devchan_keyword] = ACTIONS(1597),
    [sym_if_keyword] = ACTIONS(1597),
    [sym_switch_keyword] = ACTIONS(1597),
    [sym_case_keyword] = ACTIONS(1597),
    [sym_default_keyword] = ACTIONS(1597),
    [sym_while_keyword] = ACTIONS(1597),
    [sym_for_keyword] = ACTIONS(1597),
    [sym_break_keyword] = ACTIONS(1597),
    [sym_continue_keyword] = ACTIONS(1597),
    [sym_return_keyword] = ACTIONS(1597),
    [sym_select_keyword] = ACTIONS(1597),
    [sym_active_keyword] = ACTIONS(1597),
    [sym_struct_keyword] = ACTIONS(1597),
    [sym_structure_keyword] = ACTIONS(1597),
    [sym_button_event_keyword] = ACTIONS(1597),
    [sym_channel_event_keyword] = ACTIONS(1597),
    [sym_level_event_keyword] = ACTIONS(1597),
    [sym_data_event_keyword] = ACTIONS(1597),
    [sym_timeline_event_keyword] = ACTIONS(1597),
    [sym_custom_event_keyword] = ACTIONS(1597),
    [sym_push_keyword] = ACTIONS(1597),
    [sym_release_keyword] = ACTIONS(1597),
    [aux_sym_on_keyword_token1] = ACTIONS(1597),
    [aux_sym_off_keyword_token1] = ACTIONS(1597),
    [sym_send_level_keyword] = ACTIONS(1597),
    [sym_send_string_keyword] = ACTIONS(1597),
    [sym_send_command_keyword] = ACTIONS(1597),
    [sym_clear_buffer_keyword] = ACTIONS(1597),
    [sym_create_buffer_keyword] = ACTIONS(1597),
    [sym_create_multi_buffer_keyword] = ACTIONS(1597),
    [sym_call_keyword] = ACTIONS(1597),
    [sym_system_call_keyword] = ACTIONS(1597),
    [sym_devchan_to_keyword] = ACTIONS(1597),
    [sym_devchan_min_to_keyword] = ACTIONS(1597),
    [sym_devchan_total_off_keyword] = ACTIONS(1597),
    [sym_devchan_pulse_keyword] = ACTIONS(1597),
    [sym_wait_keyword] = ACTIONS(1597),
    [sym_cancel_wait_keyword] = ACTIONS(1597),
    [sym_wait_until_keyword] = ACTIONS(1597),
    [sym_cancel_wait_until_keyword] = ACTIONS(1597),
    [sym_cancel_all_wait_keyword] = ACTIONS(1597),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1597),
    [sym_not] = ACTIONS(1597),
    [sym_bnot] = ACTIONS(1597),
    [sym___file__] = ACTIONS(1597),
    [sym___date__] = ACTIONS(1597),
    [sym___ldate__] = ACTIONS(1597),
    [sym___time__] = ACTIONS(1597),
    [sym___line__] = ACTIONS(1597),
    [sym___name__] = ACTIONS(1597),
    [sym___netlinx__] = ACTIONS(1597),
    [sym_netlinx_axi_version] = ACTIONS(1597),
    [sym_url_flg_tcp] = ACTIONS(1597),
    [sym_url_flg_accinfopresent] = ACTIONS(1597),
    [sym_url_flg_temp] = ACTIONS(1597),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1597),
    [sym_url_flg_stat_mask] = ACTIONS(1597),
    [sym_url_flg_stat_lookup] = ACTIONS(1597),
    [sym_url_flg_stat_connecting] = ACTIONS(1597),
    [sym_url_flg_stat_waiting] = ACTIONS(1597),
    [sym_url_flg_stat_connected] = ACTIONS(1597),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1597),
    [sym_first_virtual_device] = ACTIONS(1597),
    [sym_first_local_port] = ACTIONS(1597),
    [sym_dynamic_virtual_device] = ACTIONS(1597),
    [sym_timeline_once] = ACTIONS(1597),
    [sym_timeline_repeat] = ACTIONS(1597),
    [sym_timeline_absolute] = ACTIONS(1597),
    [sym_timeline_relative] = ACTIONS(1597),
    [sym_do_push_timed_infinite] = ACTIONS(1597),
    [sym_source_type_no_address] = ACTIONS(1597),
    [sym_source_type_neuron_id] = ACTIONS(1597),
    [sym_source_type_ip_address] = ACTIONS(1597),
    [sym_source_type_axlink] = ACTIONS(1597),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1597),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1597),
    [sym_source_type_ip_socket_address] = ACTIONS(1597),
    [sym_source_type_rs232] = ACTIONS(1597),
    [sym_source_type_internal] = ACTIONS(1597),
    [sym_source_type_mac_address] = ACTIONS(1597),
    [sym_source_type_ipv4_port] = ACTIONS(1597),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1597),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1597),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1597),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1597),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1597),
    [sym_true] = ACTIONS(1597),
    [sym_false] = ACTIONS(1597),
    [sym_file_read_only] = ACTIONS(1597),
    [sym_file_rw_new] = ACTIONS(1597),
    [sym_file_rw_append] = ACTIONS(1597),
    [sym_ip_tcp] = ACTIONS(1597),
    [sym_ip_udp] = ACTIONS(1597),
    [sym_ip_udp_2way] = ACTIONS(1597),
    [sym_xml_encode_types] = ACTIONS(1597),
    [sym_xml_encode_char_as_list] = ACTIONS(1597),
    [sym_xml_encode_le] = ACTIONS(1597),
    [sym_xml_decode_types] = ACTIONS(1597),
    [sym_xml_decode_no_preserve] = ACTIONS(1597),
    [sym_duet_dev_type_display_device] = ACTIONS(1597),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1597),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1597),
    [sym_duet_dev_type_io_device] = ACTIONS(1597),
    [sym_duet_dev_type_relay_device] = ACTIONS(1597),
    [sym_duet_dev_type_ups] = ACTIONS(1597),
    [sym_duet_dev_type_amplifier] = ACTIONS(1597),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1597),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1597),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1597),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1597),
    [sym_duet_dev_type_camera] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1597),
    [sym_duet_dev_type_dss] = ACTIONS(1597),
    [sym_duet_dev_type_dvr] = ACTIONS(1597),
    [sym_duet_dev_type_disc_device] = ACTIONS(1597),
    [sym_duet_dev_type_document_camera] = ACTIONS(1597),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1597),
    [sym_duet_dev_type_hvac] = ACTIONS(1597),
    [sym_duet_dev_type_keypad] = ACTIONS(1597),
    [sym_duet_dev_type_light] = ACTIONS(1597),
    [sym_duet_dev_type_monitor] = ACTIONS(1597),
    [sym_duet_dev_type_motor] = ACTIONS(1597),
    [sym_duet_dev_type_multi_window] = ACTIONS(1597),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1597),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1597),
    [sym_duet_dev_type_receiver] = ACTIONS(1597),
    [sym_duet_dev_type_security_system] = ACTIONS(1597),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1597),
    [sym_duet_dev_type_settop_box] = ACTIONS(1597),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1597),
    [sym_duet_dev_type_switcher] = ACTIONS(1597),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1597),
    [sym_duet_dev_type_tv] = ACTIONS(1597),
    [sym_duet_dev_type_utility] = ACTIONS(1597),
    [sym_duet_dev_type_vcr] = ACTIONS(1597),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1597),
    [sym_duet_dev_type_video_processor] = ACTIONS(1597),
    [sym_duet_dev_type_video_projector] = ACTIONS(1597),
    [sym_duet_dev_type_video_wall] = ACTIONS(1597),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1597),
    [sym_duet_dev_type_weather] = ACTIONS(1597),
    [sym_duet_dev_not_polled] = ACTIONS(1597),
    [sym_duet_dev_polled] = ACTIONS(1597),
    [sym_clkmgr_mode_network] = ACTIONS(1597),
    [sym_clkmgr_mode_standalone] = ACTIONS(1597),
    [sym_normal_standby] = ACTIONS(1597),
    [sym_normal_wake] = ACTIONS(1597),
    [sym_ip_multicast_ttl_option] = ACTIONS(1597),
    [sym_ip_ttl_subnet] = ACTIONS(1597),
    [sym_ip_ttl_site] = ACTIONS(1597),
    [sym_ip_ttl_region] = ACTIONS(1597),
    [sym_ip_ttl_continent] = ACTIONS(1597),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1597),
    [sym_ip_nodelay_on] = ACTIONS(1597),
    [sym_ip_nodelay_off] = ACTIONS(1597),
    [sym_amx_error] = ACTIONS(1597),
    [sym_amx_warning] = ACTIONS(1597),
    [sym_amx_info] = ACTIONS(1597),
    [sym_amx_debug] = ACTIONS(1597),
    [sym_netlinx_login_success] = ACTIONS(1597),
    [sym_netlinx_login_fail] = ACTIONS(1597),
    [sym_netlinx_logout] = ACTIONS(1597),
    [sym_null_str] = ACTIONS(1597),
    [sym_smtp_address] = ACTIONS(1597),
    [sym_smtp_port_number] = ACTIONS(1597),
    [sym_smtp_username] = ACTIONS(1597),
    [sym_smtp_password] = ACTIONS(1597),
    [sym_smtp_require_tls] = ACTIONS(1597),
    [sym_smtp_from] = ACTIONS(1597),
    [sym_smtp_tls_true] = ACTIONS(1597),
    [sym_smtp_tls_false] = ACTIONS(1597),
    [sym_listview_on_row_select_event] = ACTIONS(1597),
    [sym_data_structure_unknown] = ACTIONS(1597),
    [sym_data_structure_datafeed] = ACTIONS(1597),
    [sym_data_structure_datarecordset] = ACTIONS(1597),
    [sym_data_structure_datarecord] = ACTIONS(1597),
    [sym_data_structure_datafield] = ACTIONS(1597),
    [sym_data_max_content_field_count] = ACTIONS(1597),
    [sym_data_max_metadata_field_count] = ACTIONS(1597),
    [sym_data_max_id_length] = ACTIONS(1597),
    [sym_data_max_name_length] = ACTIONS(1597),
    [sym_data_max_description_length] = ACTIONS(1597),
    [sym_data_max_type_length] = ACTIONS(1597),
    [sym_data_max_format_length] = ACTIONS(1597),
    [sym_data_max_source_length] = ACTIONS(1597),
    [sym_data_max_value_length] = ACTIONS(1597),
    [sym_data_max_label_length] = ACTIONS(1597),
    [sym_data_max_url_length] = ACTIONS(1597),
    [sym_max_last_login_info_length] = ACTIONS(1597),
    [sym_data_type_unknown] = ACTIONS(1597),
    [sym_data_type_string] = ACTIONS(1597),
    [sym_data_type_datetime] = ACTIONS(1597),
    [sym_data_type_date] = ACTIONS(1597),
    [sym_data_type_time] = ACTIONS(1597),
    [sym_data_type_image] = ACTIONS(1597),
    [sym_data_format_url] = ACTIONS(1597),
    [sym_data_format_phone] = ACTIONS(1597),
    [sym_data_format_email] = ACTIONS(1597),
    [sym_data_format_iso8601] = ACTIONS(1597),
    [sym_auditlog_priv] = ACTIONS(1597),
    [sym_configuration_priv] = ACTIONS(1597),
    [sym_deviceconfig_priv] = ACTIONS(1597),
    [sym_ftp_priv] = ACTIONS(1597),
    [sym_http_priv] = ACTIONS(1597),
    [sym_networkconfig_priv] = ACTIONS(1597),
    [sym_programport_priv] = ACTIONS(1597),
    [sym_remoteui_priv] = ACTIONS(1597),
    [sym_securitycontrol_priv] = ACTIONS(1597),
    [sym_softwaremanagement_priv] = ACTIONS(1597),
    [sym_terminal_priv] = ACTIONS(1597),
    [sym_tpadmin_priv] = ACTIONS(1597),
    [sym_user1_priv] = ACTIONS(1597),
    [sym_user2_priv] = ACTIONS(1597),
    [sym_user3_priv] = ACTIONS(1597),
    [sym_user4_priv] = ACTIONS(1597),
    [sym_usermanagement_priv] = ACTIONS(1597),
    [sym_valid_account] = ACTIONS(1597),
    [sym_err_name_invalid] = ACTIONS(1597),
    [sym_err_password_invalid] = ACTIONS(1597),
    [sym_err_invalid_account] = ACTIONS(1597),
    [sym_err_no_authorization] = ACTIONS(1597),
    [sym_err_invalid_parameter] = ACTIONS(1597),
    [sym_err_user_locked_out] = ACTIONS(1597),
    [sym_err_user_account_expired] = ACTIONS(1597),
    [sym_err_general_error] = ACTIONS(1597),
    [sym_err_authentication_error] = ACTIONS(1597),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1597),
    [sym_tls_validate_certificate] = ACTIONS(1597),
    [sym_tdata] = ACTIONS(1597),
    [sym_tchannel] = ACTIONS(1597),
    [sym_tlevel] = ACTIONS(1597),
    [sym_tbutton] = ACTIONS(1597),
    [sym_ttimeline] = ACTIONS(1597),
    [sym_tcustom] = ACTIONS(1597),
    [sym_url_struct] = ACTIONS(1597),
    [sym_dns_struct] = ACTIONS(1597),
    [sym_ip_address_struct] = ACTIONS(1597),
    [sym_dev_info_struct] = ACTIONS(1597),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1597),
    [sym_data_feed] = ACTIONS(1597),
    [sym_data_field] = ACTIONS(1597),
    [sym_data_record] = ACTIONS(1597),
    [sym_wc_data_feed] = ACTIONS(1597),
    [sym_wc_data_field] = ACTIONS(1597),
    [sym_wc_data_record] = ACTIONS(1597),
    [sym_last_login_info] = ACTIONS(1597),
    [sym____reserved___] = ACTIONS(1597),
    [sym_date] = ACTIONS(1597),
    [sym_day] = ACTIONS(1597),
    [sym_time] = ACTIONS(1597),
    [sym_ldate] = ACTIONS(1597),
    [sym_push_device] = ACTIONS(1597),
    [sym_push_channel] = ACTIONS(1597),
    [sym_push_devchan] = ACTIONS(1597),
    [sym_release_device] = ACTIONS(1597),
    [sym_release_channel] = ACTIONS(1597),
    [sym_release_devchan] = ACTIONS(1597),
    [sym_master_slot] = ACTIONS(1597),
    [sym_get_pulse_time] = ACTIONS(1597),
    [sym_get_timer] = ACTIONS(1597),
    [sym_master_sn] = ACTIONS(1597),
    [sym_system_number] = ACTIONS(1597),
    [sym_button] = ACTIONS(1597),
    [sym_dv_channel] = ACTIONS(1597),
    [sym_data] = ACTIONS(1597),
    [sym_level] = ACTIONS(1597),
    [sym_timeline] = ACTIONS(1597),
    [sym_channel] = ACTIONS(1597),
    [sym_custom] = ACTIONS(1597),
    [sym_atoi] = ACTIONS(1597),
    [sym_atol] = ACTIONS(1597),
    [sym_atof] = ACTIONS(1597),
    [sym_itoa] = ACTIONS(1597),
    [sym_itohex] = ACTIONS(1597),
    [sym_hextoi] = ACTIONS(1597),
    [sym_ftoa] = ACTIONS(1597),
    [sym_find_string] = ACTIONS(1597),
    [sym_left_string] = ACTIONS(1597),
    [sym_right_string] = ACTIONS(1597),
    [sym_mid_string] = ACTIONS(1597),
    [sym_remove_string] = ACTIONS(1597),
    [sym_get_buffer_string] = ACTIONS(1597),
    [sym_get_buffer_char] = ACTIONS(1597),
    [sym_get_multi_buffer_string] = ACTIONS(1597),
    [sym_length_string] = ACTIONS(1597),
    [sym_max_length_string] = ACTIONS(1597),
    [sym_set_length_string] = ACTIONS(1597),
    [sym_compare_string] = ACTIONS(1597),
    [sym_upper_string] = ACTIONS(1597),
    [sym_lower_string] = ACTIONS(1597),
    [sym_get_last] = ACTIONS(1597),
    [sym_date_to_day] = ACTIONS(1597),
    [sym_date_to_month] = ACTIONS(1597),
    [sym_date_to_year] = ACTIONS(1597),
    [sym_time_to_hour] = ACTIONS(1597),
    [sym_time_to_minute] = ACTIONS(1597),
    [sym_time_to_second] = ACTIONS(1597),
    [sym_day_of_week] = ACTIONS(1597),
    [sym_set_pulse_time] = ACTIONS(1597),
    [sym_set_timer] = ACTIONS(1597),
    [sym_file_open] = ACTIONS(1597),
    [sym_file_close] = ACTIONS(1597),
    [sym_file_read] = ACTIONS(1597),
    [sym_file_read_line] = ACTIONS(1597),
    [sym_file_write] = ACTIONS(1597),
    [sym_file_write_line] = ACTIONS(1597),
    [sym_file_copy] = ACTIONS(1597),
    [sym_file_delete] = ACTIONS(1597),
    [sym_file_rename] = ACTIONS(1597),
    [sym_file_dir] = ACTIONS(1597),
    [sym_file_getdir] = ACTIONS(1597),
    [sym_file_setdir] = ACTIONS(1597),
    [sym_file_create_dir] = ACTIONS(1597),
    [sym_file_remove_dir] = ACTIONS(1597),
    [sym_file_seek] = ACTIONS(1597),
    [sym_length_array] = ACTIONS(1597),
    [sym_max_length_array] = ACTIONS(1597),
    [sym_set_length_array] = ACTIONS(1597),
    [sym_ip_client_open] = ACTIONS(1597),
    [sym_ip_client_close] = ACTIONS(1597),
    [sym_ip_server_open] = ACTIONS(1597),
    [sym_ip_server_close] = ACTIONS(1597),
    [sym_ip_bound_client_open] = ACTIONS(1597),
    [sym_ip_mc_server_open] = ACTIONS(1597),
    [sym_ip_set_option] = ACTIONS(1597),
    [sym_ssh_client_open] = ACTIONS(1597),
    [sym_ssh_client_close] = ACTIONS(1597),
    [sym_tls_client_open] = ACTIONS(1597),
    [sym_tls_client_close] = ACTIONS(1597),
    [sym_do_push] = ACTIONS(1597),
    [sym_do_push_timed] = ACTIONS(1597),
    [sym_do_release] = ACTIONS(1597),
    [sym_device_id] = ACTIONS(1597),
    [sym_device_info] = ACTIONS(1597),
    [sym_device_id_string] = ACTIONS(1597),
    [sym_device_standby] = ACTIONS(1597),
    [sym_device_wake] = ACTIONS(1597),
    [sym_timeline_create] = ACTIONS(1597),
    [sym_timeline_kill] = ACTIONS(1597),
    [sym_timeline_reload] = ACTIONS(1597),
    [sym_timeline_pause] = ACTIONS(1597),
    [sym_timeline_restart] = ACTIONS(1597),
    [sym_timeline_set] = ACTIONS(1597),
    [sym_timeline_get] = ACTIONS(1597),
    [sym_timeline_active] = ACTIONS(1597),
    [sym_timeline_dynamic_id] = ACTIONS(1597),
    [sym_variable_to_string] = ACTIONS(1597),
    [sym_string_to_variable] = ACTIONS(1597),
    [sym_length_variable_to_string] = ACTIONS(1597),
    [sym_type_cast] = ACTIONS(1597),
    [sym_variable_to_xml] = ACTIONS(1597),
    [sym_xml_to_variable] = ACTIONS(1597),
    [sym_length_variable_to_xml] = ACTIONS(1597),
    [sym_redirect_string] = ACTIONS(1597),
    [sym_random_number] = ACTIONS(1597),
    [sym_get_unique_id] = ACTIONS(1597),
    [sym_get_system_number] = ACTIONS(1597),
    [sym_set_system_number] = ACTIONS(1597),
    [sym_get_serial_number] = ACTIONS(1597),
    [sym_get_master_build] = ACTIONS(1597),
    [sym_set_validation_code] = ACTIONS(1597),
    [sym_reboot] = ACTIONS(1597),
    [sym_load_duet_module] = ACTIONS(1597),
    [sym_unload_duet_module] = ACTIONS(1597),
    [sym_duet_mem_size_set] = ACTIONS(1597),
    [sym_duet_mem_size_get] = ACTIONS(1597),
    [sym_dynamic_application_device] = ACTIONS(1597),
    [sym_dynamic_polled_port] = ACTIONS(1597),
    [sym_static_port_binding] = ACTIONS(1597),
    [sym_static_ip_binding] = ACTIONS(1597),
    [sym_get_dns_list] = ACTIONS(1597),
    [sym_set_dns_list] = ACTIONS(1597),
    [sym_get_ip_address] = ACTIONS(1597),
    [sym_set_ip_address] = ACTIONS(1597),
    [sym_get_url_list] = ACTIONS(1597),
    [sym_add_url_entry] = ACTIONS(1597),
    [sym_delete_url_entry] = ACTIONS(1597),
    [sym_abs_value] = ACTIONS(1597),
    [sym_max_value] = ACTIONS(1597),
    [sym_min_value] = ACTIONS(1597),
    [sym_exp_value] = ACTIONS(1597),
    [sym_log_value] = ACTIONS(1597),
    [sym_log10_value] = ACTIONS(1597),
    [sym_power_value] = ACTIONS(1597),
    [sym_sqrt_value] = ACTIONS(1597),
    [sym_raw_be] = ACTIONS(1597),
    [sym_raw_le] = ACTIONS(1597),
    [sym_internal_queue_size_set] = ACTIONS(1597),
    [sym_internal_queue_size_get] = ACTIONS(1597),
    [sym_internal_threshold_set] = ACTIONS(1597),
    [sym_internal_threshold_get] = ACTIONS(1597),
    [sym_rebuilt_event] = ACTIONS(1597),
    [sym_do_custom_event] = ACTIONS(1597),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1597),
    [sym_clkmgr_set_clk_source] = ACTIONS(1597),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1597),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1597),
    [sym_clkmgr_get_timezone] = ACTIONS(1597),
    [sym_clkmgr_set_timezone] = ACTIONS(1597),
    [sym_clkmgr_get_resync_period] = ACTIONS(1597),
    [sym_clkmgr_set_resync_period] = ACTIONS(1597),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1597),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1597),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1597),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1597),
    [sym_clkmgr_get_timeservers] = ACTIONS(1597),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1597),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1597),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1597),
    [sym_set_log_level] = ACTIONS(1597),
    [sym_get_log_level] = ACTIONS(1597),
    [sym_amx_log] = ACTIONS(1597),
    [sym_smtp_server_config_set] = ACTIONS(1597),
    [sym_smtp_server_config_get] = ACTIONS(1597),
    [sym_smtp_send] = ACTIONS(1597),
    [sym_validate_netlinx_account] = ACTIONS(1597),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1597),
    [sym_authenticate_certificate] = ACTIONS(1597),
    [sym_audit_netlinx_session_event] = ACTIONS(1597),
    [sym_audit_netlinx_generic_event] = ACTIONS(1597),
    [sym_get_available_flash_disk_space] = ACTIONS(1597),
    [sym_get_max_flash_disk_space] = ACTIONS(1597),
    [sym_data_create_feed] = ACTIONS(1597),
    [sym_data_delete_feed] = ACTIONS(1597),
    [sym_data_publish_feed] = ACTIONS(1597),
    [sym_data_get_published_feed] = ACTIONS(1597),
    [sym_data_add_record] = ACTIONS(1597),
    [sym_data_get_event_record] = ACTIONS(1597),
    [sym__wc_data_create_feed] = ACTIONS(1597),
    [sym__wc_data_add_record] = ACTIONS(1597),
    [sym__wc_data_get_event_record] = ACTIONS(1597),
    [sym_format] = ACTIONS(1597),
    [sym_astro_clock] = ACTIONS(1597),
    [sym_set_outdoor_temperature] = ACTIONS(1597),
    [sym_set_virtual_level_count] = ACTIONS(1597),
    [sym_set_virtual_channel_count] = ACTIONS(1597),
    [sym_set_virtual_port_count] = ACTIONS(1597),
    [sym__wc_encode_int] = ACTIONS(1597),
    [sym__wc_decode_int] = ACTIONS(1597),
    [sym__wc_int] = ACTIONS(1597),
    [sym__wc_to_ch_int] = ACTIONS(1597),
    [sym__ch_to_wc_int] = ACTIONS(1597),
    [sym__wc_find_string_int] = ACTIONS(1597),
    [sym__wc_left_string_int] = ACTIONS(1597),
    [sym__wc_lower_string_int] = ACTIONS(1597),
    [sym__wc_mid_string_int] = ACTIONS(1597),
    [sym__wc_remove_string_int] = ACTIONS(1597),
    [sym__wc_right_string_int] = ACTIONS(1597),
    [sym__wc_upper_string_int] = ACTIONS(1597),
    [sym__wc_compare_string_int] = ACTIONS(1597),
    [sym__wc_get_buffer_char_int] = ACTIONS(1597),
    [sym__wc_get_buffer_string_int] = ACTIONS(1597),
    [sym__wc_concat_string_int] = ACTIONS(1597),
    [sym__wc_file_open_int] = ACTIONS(1597),
    [sym__wc_file_close_int] = ACTIONS(1597),
    [sym__wc_file_read_int] = ACTIONS(1597),
    [sym__wc_file_read_line_int] = ACTIONS(1597),
    [sym__wc_file_write_int] = ACTIONS(1597),
    [sym__wc_file_write_line_int] = ACTIONS(1597),
    [aux_sym_preproc_include_token1] = ACTIONS(1597),
    [aux_sym_preproc_define_token1] = ACTIONS(1599),
    [aux_sym_preproc_warn_token1] = ACTIONS(1599),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1599),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1599),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1599),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1599),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_BANG] = ACTIONS(1599),
    [anon_sym_TILDE] = ACTIONS(1599),
    [anon_sym_DASH] = ACTIONS(1597),
    [anon_sym_PLUS] = ACTIONS(1597),
    [anon_sym_DASH_DASH] = ACTIONS(1599),
    [anon_sym_PLUS_PLUS] = ACTIONS(1599),
    [anon_sym_DQUOTE] = ACTIONS(1599),
    [anon_sym_SQUOTE] = ACTIONS(1599),
    [sym_number_literal] = ACTIONS(1599),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(454)] = {
    [ts_builtin_sym_end] = ACTIONS(1603),
    [sym_identifier] = ACTIONS(1601),
    [sym_program_name_keyword] = ACTIONS(1601),
    [sym_module_name_keyword] = ACTIONS(1601),
    [sym_define_device_keyword] = ACTIONS(1601),
    [sym_define_combine_keyword] = ACTIONS(1601),
    [sym_define_constant_keyword] = ACTIONS(1601),
    [sym_define_type_keyword] = ACTIONS(1601),
    [sym_define_variable_keyword] = ACTIONS(1601),
    [sym_define_system_variable_keyword] = ACTIONS(1601),
    [sym_define_connect_level_keyword] = ACTIONS(1601),
    [sym_define_latching_keyword] = ACTIONS(1601),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1601),
    [sym_define_toggling_keyword] = ACTIONS(1601),
    [sym_define_call_keyword] = ACTIONS(1601),
    [sym_define_function_keyword] = ACTIONS(1601),
    [sym_define_library_function_keyword] = ACTIONS(1601),
    [sym_define_start_keyword] = ACTIONS(1601),
    [sym_define_event_keyword] = ACTIONS(1601),
    [sym_define_program_keyword] = ACTIONS(1601),
    [sym_define_module_keyword] = ACTIONS(1601),
    [sym_char_keyword] = ACTIONS(1601),
    [sym_widechar_keyword] = ACTIONS(1601),
    [sym_integer_keyword] = ACTIONS(1601),
    [sym_sinteger_keyword] = ACTIONS(1601),
    [sym_long_keyword] = ACTIONS(1601),
    [sym_slong_keyword] = ACTIONS(1601),
    [sym_float_keyword] = ACTIONS(1601),
    [sym_double_keyword] = ACTIONS(1601),
    [sym_variant_keyword] = ACTIONS(1601),
    [sym_variantarray_keyword] = ACTIONS(1601),
    [sym_local_var_keyword] = ACTIONS(1601),
    [sym_stack_var_keyword] = ACTIONS(1601),
    [sym_constant_keyword] = ACTIONS(1601),
    [sym_volatile_keyword] = ACTIONS(1601),
    [sym_non_volatile_keyword] = ACTIONS(1601),
    [sym_persistent_keyword] = ACTIONS(1601),
    [sym_dev_keyword] = ACTIONS(1601),
    [sym_devlev_keyword] = ACTIONS(1601),
    [sym_devchan_keyword] = ACTIONS(1601),
    [sym_if_keyword] = ACTIONS(1601),
    [sym_switch_keyword] = ACTIONS(1601),
    [sym_case_keyword] = ACTIONS(1601),
    [sym_default_keyword] = ACTIONS(1601),
    [sym_while_keyword] = ACTIONS(1601),
    [sym_for_keyword] = ACTIONS(1601),
    [sym_break_keyword] = ACTIONS(1601),
    [sym_continue_keyword] = ACTIONS(1601),
    [sym_return_keyword] = ACTIONS(1601),
    [sym_select_keyword] = ACTIONS(1601),
    [sym_active_keyword] = ACTIONS(1601),
    [sym_struct_keyword] = ACTIONS(1601),
    [sym_structure_keyword] = ACTIONS(1601),
    [sym_button_event_keyword] = ACTIONS(1601),
    [sym_channel_event_keyword] = ACTIONS(1601),
    [sym_level_event_keyword] = ACTIONS(1601),
    [sym_data_event_keyword] = ACTIONS(1601),
    [sym_timeline_event_keyword] = ACTIONS(1601),
    [sym_custom_event_keyword] = ACTIONS(1601),
    [sym_push_keyword] = ACTIONS(1601),
    [sym_release_keyword] = ACTIONS(1601),
    [aux_sym_on_keyword_token1] = ACTIONS(1601),
    [aux_sym_off_keyword_token1] = ACTIONS(1601),
    [sym_send_level_keyword] = ACTIONS(1601),
    [sym_send_string_keyword] = ACTIONS(1601),
    [sym_send_command_keyword] = ACTIONS(1601),
    [sym_clear_buffer_keyword] = ACTIONS(1601),
    [sym_create_buffer_keyword] = ACTIONS(1601),
    [sym_create_multi_buffer_keyword] = ACTIONS(1601),
    [sym_call_keyword] = ACTIONS(1601),
    [sym_system_call_keyword] = ACTIONS(1601),
    [sym_devchan_to_keyword] = ACTIONS(1601),
    [sym_devchan_min_to_keyword] = ACTIONS(1601),
    [sym_devchan_total_off_keyword] = ACTIONS(1601),
    [sym_devchan_pulse_keyword] = ACTIONS(1601),
    [sym_wait_keyword] = ACTIONS(1601),
    [sym_cancel_wait_keyword] = ACTIONS(1601),
    [sym_wait_until_keyword] = ACTIONS(1601),
    [sym_cancel_wait_until_keyword] = ACTIONS(1601),
    [sym_cancel_all_wait_keyword] = ACTIONS(1601),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1601),
    [sym_not] = ACTIONS(1601),
    [sym_bnot] = ACTIONS(1601),
    [sym___file__] = ACTIONS(1601),
    [sym___date__] = ACTIONS(1601),
    [sym___ldate__] = ACTIONS(1601),
    [sym___time__] = ACTIONS(1601),
    [sym___line__] = ACTIONS(1601),
    [sym___name__] = ACTIONS(1601),
    [sym___netlinx__] = ACTIONS(1601),
    [sym_netlinx_axi_version] = ACTIONS(1601),
    [sym_url_flg_tcp] = ACTIONS(1601),
    [sym_url_flg_accinfopresent] = ACTIONS(1601),
    [sym_url_flg_temp] = ACTIONS(1601),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1601),
    [sym_url_flg_stat_mask] = ACTIONS(1601),
    [sym_url_flg_stat_lookup] = ACTIONS(1601),
    [sym_url_flg_stat_connecting] = ACTIONS(1601),
    [sym_url_flg_stat_waiting] = ACTIONS(1601),
    [sym_url_flg_stat_connected] = ACTIONS(1601),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1601),
    [sym_first_virtual_device] = ACTIONS(1601),
    [sym_first_local_port] = ACTIONS(1601),
    [sym_dynamic_virtual_device] = ACTIONS(1601),
    [sym_timeline_once] = ACTIONS(1601),
    [sym_timeline_repeat] = ACTIONS(1601),
    [sym_timeline_absolute] = ACTIONS(1601),
    [sym_timeline_relative] = ACTIONS(1601),
    [sym_do_push_timed_infinite] = ACTIONS(1601),
    [sym_source_type_no_address] = ACTIONS(1601),
    [sym_source_type_neuron_id] = ACTIONS(1601),
    [sym_source_type_ip_address] = ACTIONS(1601),
    [sym_source_type_axlink] = ACTIONS(1601),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1601),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1601),
    [sym_source_type_ip_socket_address] = ACTIONS(1601),
    [sym_source_type_rs232] = ACTIONS(1601),
    [sym_source_type_internal] = ACTIONS(1601),
    [sym_source_type_mac_address] = ACTIONS(1601),
    [sym_source_type_ipv4_port] = ACTIONS(1601),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1601),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1601),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1601),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1601),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1601),
    [sym_true] = ACTIONS(1601),
    [sym_false] = ACTIONS(1601),
    [sym_file_read_only] = ACTIONS(1601),
    [sym_file_rw_new] = ACTIONS(1601),
    [sym_file_rw_append] = ACTIONS(1601),
    [sym_ip_tcp] = ACTIONS(1601),
    [sym_ip_udp] = ACTIONS(1601),
    [sym_ip_udp_2way] = ACTIONS(1601),
    [sym_xml_encode_types] = ACTIONS(1601),
    [sym_xml_encode_char_as_list] = ACTIONS(1601),
    [sym_xml_encode_le] = ACTIONS(1601),
    [sym_xml_decode_types] = ACTIONS(1601),
    [sym_xml_decode_no_preserve] = ACTIONS(1601),
    [sym_duet_dev_type_display_device] = ACTIONS(1601),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1601),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1601),
    [sym_duet_dev_type_io_device] = ACTIONS(1601),
    [sym_duet_dev_type_relay_device] = ACTIONS(1601),
    [sym_duet_dev_type_ups] = ACTIONS(1601),
    [sym_duet_dev_type_amplifier] = ACTIONS(1601),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1601),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1601),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1601),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1601),
    [sym_duet_dev_type_camera] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1601),
    [sym_duet_dev_type_dss] = ACTIONS(1601),
    [sym_duet_dev_type_dvr] = ACTIONS(1601),
    [sym_duet_dev_type_disc_device] = ACTIONS(1601),
    [sym_duet_dev_type_document_camera] = ACTIONS(1601),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1601),
    [sym_duet_dev_type_hvac] = ACTIONS(1601),
    [sym_duet_dev_type_keypad] = ACTIONS(1601),
    [sym_duet_dev_type_light] = ACTIONS(1601),
    [sym_duet_dev_type_monitor] = ACTIONS(1601),
    [sym_duet_dev_type_motor] = ACTIONS(1601),
    [sym_duet_dev_type_multi_window] = ACTIONS(1601),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1601),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1601),
    [sym_duet_dev_type_receiver] = ACTIONS(1601),
    [sym_duet_dev_type_security_system] = ACTIONS(1601),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1601),
    [sym_duet_dev_type_settop_box] = ACTIONS(1601),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1601),
    [sym_duet_dev_type_switcher] = ACTIONS(1601),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1601),
    [sym_duet_dev_type_tv] = ACTIONS(1601),
    [sym_duet_dev_type_utility] = ACTIONS(1601),
    [sym_duet_dev_type_vcr] = ACTIONS(1601),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1601),
    [sym_duet_dev_type_video_processor] = ACTIONS(1601),
    [sym_duet_dev_type_video_projector] = ACTIONS(1601),
    [sym_duet_dev_type_video_wall] = ACTIONS(1601),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1601),
    [sym_duet_dev_type_weather] = ACTIONS(1601),
    [sym_duet_dev_not_polled] = ACTIONS(1601),
    [sym_duet_dev_polled] = ACTIONS(1601),
    [sym_clkmgr_mode_network] = ACTIONS(1601),
    [sym_clkmgr_mode_standalone] = ACTIONS(1601),
    [sym_normal_standby] = ACTIONS(1601),
    [sym_normal_wake] = ACTIONS(1601),
    [sym_ip_multicast_ttl_option] = ACTIONS(1601),
    [sym_ip_ttl_subnet] = ACTIONS(1601),
    [sym_ip_ttl_site] = ACTIONS(1601),
    [sym_ip_ttl_region] = ACTIONS(1601),
    [sym_ip_ttl_continent] = ACTIONS(1601),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1601),
    [sym_ip_nodelay_on] = ACTIONS(1601),
    [sym_ip_nodelay_off] = ACTIONS(1601),
    [sym_amx_error] = ACTIONS(1601),
    [sym_amx_warning] = ACTIONS(1601),
    [sym_amx_info] = ACTIONS(1601),
    [sym_amx_debug] = ACTIONS(1601),
    [sym_netlinx_login_success] = ACTIONS(1601),
    [sym_netlinx_login_fail] = ACTIONS(1601),
    [sym_netlinx_logout] = ACTIONS(1601),
    [sym_null_str] = ACTIONS(1601),
    [sym_smtp_address] = ACTIONS(1601),
    [sym_smtp_port_number] = ACTIONS(1601),
    [sym_smtp_username] = ACTIONS(1601),
    [sym_smtp_password] = ACTIONS(1601),
    [sym_smtp_require_tls] = ACTIONS(1601),
    [sym_smtp_from] = ACTIONS(1601),
    [sym_smtp_tls_true] = ACTIONS(1601),
    [sym_smtp_tls_false] = ACTIONS(1601),
    [sym_listview_on_row_select_event] = ACTIONS(1601),
    [sym_data_structure_unknown] = ACTIONS(1601),
    [sym_data_structure_datafeed] = ACTIONS(1601),
    [sym_data_structure_datarecordset] = ACTIONS(1601),
    [sym_data_structure_datarecord] = ACTIONS(1601),
    [sym_data_structure_datafield] = ACTIONS(1601),
    [sym_data_max_content_field_count] = ACTIONS(1601),
    [sym_data_max_metadata_field_count] = ACTIONS(1601),
    [sym_data_max_id_length] = ACTIONS(1601),
    [sym_data_max_name_length] = ACTIONS(1601),
    [sym_data_max_description_length] = ACTIONS(1601),
    [sym_data_max_type_length] = ACTIONS(1601),
    [sym_data_max_format_length] = ACTIONS(1601),
    [sym_data_max_source_length] = ACTIONS(1601),
    [sym_data_max_value_length] = ACTIONS(1601),
    [sym_data_max_label_length] = ACTIONS(1601),
    [sym_data_max_url_length] = ACTIONS(1601),
    [sym_max_last_login_info_length] = ACTIONS(1601),
    [sym_data_type_unknown] = ACTIONS(1601),
    [sym_data_type_string] = ACTIONS(1601),
    [sym_data_type_datetime] = ACTIONS(1601),
    [sym_data_type_date] = ACTIONS(1601),
    [sym_data_type_time] = ACTIONS(1601),
    [sym_data_type_image] = ACTIONS(1601),
    [sym_data_format_url] = ACTIONS(1601),
    [sym_data_format_phone] = ACTIONS(1601),
    [sym_data_format_email] = ACTIONS(1601),
    [sym_data_format_iso8601] = ACTIONS(1601),
    [sym_auditlog_priv] = ACTIONS(1601),
    [sym_configuration_priv] = ACTIONS(1601),
    [sym_deviceconfig_priv] = ACTIONS(1601),
    [sym_ftp_priv] = ACTIONS(1601),
    [sym_http_priv] = ACTIONS(1601),
    [sym_networkconfig_priv] = ACTIONS(1601),
    [sym_programport_priv] = ACTIONS(1601),
    [sym_remoteui_priv] = ACTIONS(1601),
    [sym_securitycontrol_priv] = ACTIONS(1601),
    [sym_softwaremanagement_priv] = ACTIONS(1601),
    [sym_terminal_priv] = ACTIONS(1601),
    [sym_tpadmin_priv] = ACTIONS(1601),
    [sym_user1_priv] = ACTIONS(1601),
    [sym_user2_priv] = ACTIONS(1601),
    [sym_user3_priv] = ACTIONS(1601),
    [sym_user4_priv] = ACTIONS(1601),
    [sym_usermanagement_priv] = ACTIONS(1601),
    [sym_valid_account] = ACTIONS(1601),
    [sym_err_name_invalid] = ACTIONS(1601),
    [sym_err_password_invalid] = ACTIONS(1601),
    [sym_err_invalid_account] = ACTIONS(1601),
    [sym_err_no_authorization] = ACTIONS(1601),
    [sym_err_invalid_parameter] = ACTIONS(1601),
    [sym_err_user_locked_out] = ACTIONS(1601),
    [sym_err_user_account_expired] = ACTIONS(1601),
    [sym_err_general_error] = ACTIONS(1601),
    [sym_err_authentication_error] = ACTIONS(1601),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1601),
    [sym_tls_validate_certificate] = ACTIONS(1601),
    [sym_tdata] = ACTIONS(1601),
    [sym_tchannel] = ACTIONS(1601),
    [sym_tlevel] = ACTIONS(1601),
    [sym_tbutton] = ACTIONS(1601),
    [sym_ttimeline] = ACTIONS(1601),
    [sym_tcustom] = ACTIONS(1601),
    [sym_url_struct] = ACTIONS(1601),
    [sym_dns_struct] = ACTIONS(1601),
    [sym_ip_address_struct] = ACTIONS(1601),
    [sym_dev_info_struct] = ACTIONS(1601),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1601),
    [sym_data_feed] = ACTIONS(1601),
    [sym_data_field] = ACTIONS(1601),
    [sym_data_record] = ACTIONS(1601),
    [sym_wc_data_feed] = ACTIONS(1601),
    [sym_wc_data_field] = ACTIONS(1601),
    [sym_wc_data_record] = ACTIONS(1601),
    [sym_last_login_info] = ACTIONS(1601),
    [sym____reserved___] = ACTIONS(1601),
    [sym_date] = ACTIONS(1601),
    [sym_day] = ACTIONS(1601),
    [sym_time] = ACTIONS(1601),
    [sym_ldate] = ACTIONS(1601),
    [sym_push_device] = ACTIONS(1601),
    [sym_push_channel] = ACTIONS(1601),
    [sym_push_devchan] = ACTIONS(1601),
    [sym_release_device] = ACTIONS(1601),
    [sym_release_channel] = ACTIONS(1601),
    [sym_release_devchan] = ACTIONS(1601),
    [sym_master_slot] = ACTIONS(1601),
    [sym_get_pulse_time] = ACTIONS(1601),
    [sym_get_timer] = ACTIONS(1601),
    [sym_master_sn] = ACTIONS(1601),
    [sym_system_number] = ACTIONS(1601),
    [sym_button] = ACTIONS(1601),
    [sym_dv_channel] = ACTIONS(1601),
    [sym_data] = ACTIONS(1601),
    [sym_level] = ACTIONS(1601),
    [sym_timeline] = ACTIONS(1601),
    [sym_channel] = ACTIONS(1601),
    [sym_custom] = ACTIONS(1601),
    [sym_atoi] = ACTIONS(1601),
    [sym_atol] = ACTIONS(1601),
    [sym_atof] = ACTIONS(1601),
    [sym_itoa] = ACTIONS(1601),
    [sym_itohex] = ACTIONS(1601),
    [sym_hextoi] = ACTIONS(1601),
    [sym_ftoa] = ACTIONS(1601),
    [sym_find_string] = ACTIONS(1601),
    [sym_left_string] = ACTIONS(1601),
    [sym_right_string] = ACTIONS(1601),
    [sym_mid_string] = ACTIONS(1601),
    [sym_remove_string] = ACTIONS(1601),
    [sym_get_buffer_string] = ACTIONS(1601),
    [sym_get_buffer_char] = ACTIONS(1601),
    [sym_get_multi_buffer_string] = ACTIONS(1601),
    [sym_length_string] = ACTIONS(1601),
    [sym_max_length_string] = ACTIONS(1601),
    [sym_set_length_string] = ACTIONS(1601),
    [sym_compare_string] = ACTIONS(1601),
    [sym_upper_string] = ACTIONS(1601),
    [sym_lower_string] = ACTIONS(1601),
    [sym_get_last] = ACTIONS(1601),
    [sym_date_to_day] = ACTIONS(1601),
    [sym_date_to_month] = ACTIONS(1601),
    [sym_date_to_year] = ACTIONS(1601),
    [sym_time_to_hour] = ACTIONS(1601),
    [sym_time_to_minute] = ACTIONS(1601),
    [sym_time_to_second] = ACTIONS(1601),
    [sym_day_of_week] = ACTIONS(1601),
    [sym_set_pulse_time] = ACTIONS(1601),
    [sym_set_timer] = ACTIONS(1601),
    [sym_file_open] = ACTIONS(1601),
    [sym_file_close] = ACTIONS(1601),
    [sym_file_read] = ACTIONS(1601),
    [sym_file_read_line] = ACTIONS(1601),
    [sym_file_write] = ACTIONS(1601),
    [sym_file_write_line] = ACTIONS(1601),
    [sym_file_copy] = ACTIONS(1601),
    [sym_file_delete] = ACTIONS(1601),
    [sym_file_rename] = ACTIONS(1601),
    [sym_file_dir] = ACTIONS(1601),
    [sym_file_getdir] = ACTIONS(1601),
    [sym_file_setdir] = ACTIONS(1601),
    [sym_file_create_dir] = ACTIONS(1601),
    [sym_file_remove_dir] = ACTIONS(1601),
    [sym_file_seek] = ACTIONS(1601),
    [sym_length_array] = ACTIONS(1601),
    [sym_max_length_array] = ACTIONS(1601),
    [sym_set_length_array] = ACTIONS(1601),
    [sym_ip_client_open] = ACTIONS(1601),
    [sym_ip_client_close] = ACTIONS(1601),
    [sym_ip_server_open] = ACTIONS(1601),
    [sym_ip_server_close] = ACTIONS(1601),
    [sym_ip_bound_client_open] = ACTIONS(1601),
    [sym_ip_mc_server_open] = ACTIONS(1601),
    [sym_ip_set_option] = ACTIONS(1601),
    [sym_ssh_client_open] = ACTIONS(1601),
    [sym_ssh_client_close] = ACTIONS(1601),
    [sym_tls_client_open] = ACTIONS(1601),
    [sym_tls_client_close] = ACTIONS(1601),
    [sym_do_push] = ACTIONS(1601),
    [sym_do_push_timed] = ACTIONS(1601),
    [sym_do_release] = ACTIONS(1601),
    [sym_device_id] = ACTIONS(1601),
    [sym_device_info] = ACTIONS(1601),
    [sym_device_id_string] = ACTIONS(1601),
    [sym_device_standby] = ACTIONS(1601),
    [sym_device_wake] = ACTIONS(1601),
    [sym_timeline_create] = ACTIONS(1601),
    [sym_timeline_kill] = ACTIONS(1601),
    [sym_timeline_reload] = ACTIONS(1601),
    [sym_timeline_pause] = ACTIONS(1601),
    [sym_timeline_restart] = ACTIONS(1601),
    [sym_timeline_set] = ACTIONS(1601),
    [sym_timeline_get] = ACTIONS(1601),
    [sym_timeline_active] = ACTIONS(1601),
    [sym_timeline_dynamic_id] = ACTIONS(1601),
    [sym_variable_to_string] = ACTIONS(1601),
    [sym_string_to_variable] = ACTIONS(1601),
    [sym_length_variable_to_string] = ACTIONS(1601),
    [sym_type_cast] = ACTIONS(1601),
    [sym_variable_to_xml] = ACTIONS(1601),
    [sym_xml_to_variable] = ACTIONS(1601),
    [sym_length_variable_to_xml] = ACTIONS(1601),
    [sym_redirect_string] = ACTIONS(1601),
    [sym_random_number] = ACTIONS(1601),
    [sym_get_unique_id] = ACTIONS(1601),
    [sym_get_system_number] = ACTIONS(1601),
    [sym_set_system_number] = ACTIONS(1601),
    [sym_get_serial_number] = ACTIONS(1601),
    [sym_get_master_build] = ACTIONS(1601),
    [sym_set_validation_code] = ACTIONS(1601),
    [sym_reboot] = ACTIONS(1601),
    [sym_load_duet_module] = ACTIONS(1601),
    [sym_unload_duet_module] = ACTIONS(1601),
    [sym_duet_mem_size_set] = ACTIONS(1601),
    [sym_duet_mem_size_get] = ACTIONS(1601),
    [sym_dynamic_application_device] = ACTIONS(1601),
    [sym_dynamic_polled_port] = ACTIONS(1601),
    [sym_static_port_binding] = ACTIONS(1601),
    [sym_static_ip_binding] = ACTIONS(1601),
    [sym_get_dns_list] = ACTIONS(1601),
    [sym_set_dns_list] = ACTIONS(1601),
    [sym_get_ip_address] = ACTIONS(1601),
    [sym_set_ip_address] = ACTIONS(1601),
    [sym_get_url_list] = ACTIONS(1601),
    [sym_add_url_entry] = ACTIONS(1601),
    [sym_delete_url_entry] = ACTIONS(1601),
    [sym_abs_value] = ACTIONS(1601),
    [sym_max_value] = ACTIONS(1601),
    [sym_min_value] = ACTIONS(1601),
    [sym_exp_value] = ACTIONS(1601),
    [sym_log_value] = ACTIONS(1601),
    [sym_log10_value] = ACTIONS(1601),
    [sym_power_value] = ACTIONS(1601),
    [sym_sqrt_value] = ACTIONS(1601),
    [sym_raw_be] = ACTIONS(1601),
    [sym_raw_le] = ACTIONS(1601),
    [sym_internal_queue_size_set] = ACTIONS(1601),
    [sym_internal_queue_size_get] = ACTIONS(1601),
    [sym_internal_threshold_set] = ACTIONS(1601),
    [sym_internal_threshold_get] = ACTIONS(1601),
    [sym_rebuilt_event] = ACTIONS(1601),
    [sym_do_custom_event] = ACTIONS(1601),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1601),
    [sym_clkmgr_set_clk_source] = ACTIONS(1601),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1601),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1601),
    [sym_clkmgr_get_timezone] = ACTIONS(1601),
    [sym_clkmgr_set_timezone] = ACTIONS(1601),
    [sym_clkmgr_get_resync_period] = ACTIONS(1601),
    [sym_clkmgr_set_resync_period] = ACTIONS(1601),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1601),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1601),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1601),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1601),
    [sym_clkmgr_get_timeservers] = ACTIONS(1601),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1601),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1601),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1601),
    [sym_set_log_level] = ACTIONS(1601),
    [sym_get_log_level] = ACTIONS(1601),
    [sym_amx_log] = ACTIONS(1601),
    [sym_smtp_server_config_set] = ACTIONS(1601),
    [sym_smtp_server_config_get] = ACTIONS(1601),
    [sym_smtp_send] = ACTIONS(1601),
    [sym_validate_netlinx_account] = ACTIONS(1601),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1601),
    [sym_authenticate_certificate] = ACTIONS(1601),
    [sym_audit_netlinx_session_event] = ACTIONS(1601),
    [sym_audit_netlinx_generic_event] = ACTIONS(1601),
    [sym_get_available_flash_disk_space] = ACTIONS(1601),
    [sym_get_max_flash_disk_space] = ACTIONS(1601),
    [sym_data_create_feed] = ACTIONS(1601),
    [sym_data_delete_feed] = ACTIONS(1601),
    [sym_data_publish_feed] = ACTIONS(1601),
    [sym_data_get_published_feed] = ACTIONS(1601),
    [sym_data_add_record] = ACTIONS(1601),
    [sym_data_get_event_record] = ACTIONS(1601),
    [sym__wc_data_create_feed] = ACTIONS(1601),
    [sym__wc_data_add_record] = ACTIONS(1601),
    [sym__wc_data_get_event_record] = ACTIONS(1601),
    [sym_format] = ACTIONS(1601),
    [sym_astro_clock] = ACTIONS(1601),
    [sym_set_outdoor_temperature] = ACTIONS(1601),
    [sym_set_virtual_level_count] = ACTIONS(1601),
    [sym_set_virtual_channel_count] = ACTIONS(1601),
    [sym_set_virtual_port_count] = ACTIONS(1601),
    [sym__wc_encode_int] = ACTIONS(1601),
    [sym__wc_decode_int] = ACTIONS(1601),
    [sym__wc_int] = ACTIONS(1601),
    [sym__wc_to_ch_int] = ACTIONS(1601),
    [sym__ch_to_wc_int] = ACTIONS(1601),
    [sym__wc_find_string_int] = ACTIONS(1601),
    [sym__wc_left_string_int] = ACTIONS(1601),
    [sym__wc_lower_string_int] = ACTIONS(1601),
    [sym__wc_mid_string_int] = ACTIONS(1601),
    [sym__wc_remove_string_int] = ACTIONS(1601),
    [sym__wc_right_string_int] = ACTIONS(1601),
    [sym__wc_upper_string_int] = ACTIONS(1601),
    [sym__wc_compare_string_int] = ACTIONS(1601),
    [sym__wc_get_buffer_char_int] = ACTIONS(1601),
    [sym__wc_get_buffer_string_int] = ACTIONS(1601),
    [sym__wc_concat_string_int] = ACTIONS(1601),
    [sym__wc_file_open_int] = ACTIONS(1601),
    [sym__wc_file_close_int] = ACTIONS(1601),
    [sym__wc_file_read_int] = ACTIONS(1601),
    [sym__wc_file_read_line_int] = ACTIONS(1601),
    [sym__wc_file_write_int] = ACTIONS(1601),
    [sym__wc_file_write_line_int] = ACTIONS(1601),
    [aux_sym_preproc_include_token1] = ACTIONS(1601),
    [aux_sym_preproc_define_token1] = ACTIONS(1603),
    [aux_sym_preproc_warn_token1] = ACTIONS(1603),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1603),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1603),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1603),
    [anon_sym_LBRACK] = ACTIONS(1603),
    [anon_sym_LBRACE] = ACTIONS(1603),
    [anon_sym_LPAREN] = ACTIONS(1601),
    [anon_sym_BANG] = ACTIONS(1603),
    [anon_sym_TILDE] = ACTIONS(1603),
    [anon_sym_DASH] = ACTIONS(1601),
    [anon_sym_PLUS] = ACTIONS(1601),
    [anon_sym_DASH_DASH] = ACTIONS(1603),
    [anon_sym_PLUS_PLUS] = ACTIONS(1603),
    [anon_sym_DQUOTE] = ACTIONS(1603),
    [anon_sym_SQUOTE] = ACTIONS(1603),
    [sym_number_literal] = ACTIONS(1603),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(455)] = {
    [ts_builtin_sym_end] = ACTIONS(1911),
    [sym_identifier] = ACTIONS(1909),
    [sym_program_name_keyword] = ACTIONS(1909),
    [sym_module_name_keyword] = ACTIONS(1909),
    [sym_define_device_keyword] = ACTIONS(1909),
    [sym_define_combine_keyword] = ACTIONS(1909),
    [sym_define_constant_keyword] = ACTIONS(1909),
    [sym_define_type_keyword] = ACTIONS(1909),
    [sym_define_variable_keyword] = ACTIONS(1909),
    [sym_define_system_variable_keyword] = ACTIONS(1909),
    [sym_define_connect_level_keyword] = ACTIONS(1909),
    [sym_define_latching_keyword] = ACTIONS(1909),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1909),
    [sym_define_toggling_keyword] = ACTIONS(1909),
    [sym_define_call_keyword] = ACTIONS(1909),
    [sym_define_function_keyword] = ACTIONS(1909),
    [sym_define_library_function_keyword] = ACTIONS(1909),
    [sym_define_start_keyword] = ACTIONS(1909),
    [sym_define_event_keyword] = ACTIONS(1909),
    [sym_define_program_keyword] = ACTIONS(1909),
    [sym_define_module_keyword] = ACTIONS(1909),
    [sym_char_keyword] = ACTIONS(1909),
    [sym_widechar_keyword] = ACTIONS(1909),
    [sym_integer_keyword] = ACTIONS(1909),
    [sym_sinteger_keyword] = ACTIONS(1909),
    [sym_long_keyword] = ACTIONS(1909),
    [sym_slong_keyword] = ACTIONS(1909),
    [sym_float_keyword] = ACTIONS(1909),
    [sym_double_keyword] = ACTIONS(1909),
    [sym_variant_keyword] = ACTIONS(1909),
    [sym_variantarray_keyword] = ACTIONS(1909),
    [sym_local_var_keyword] = ACTIONS(1909),
    [sym_stack_var_keyword] = ACTIONS(1909),
    [sym_constant_keyword] = ACTIONS(1909),
    [sym_volatile_keyword] = ACTIONS(1909),
    [sym_non_volatile_keyword] = ACTIONS(1909),
    [sym_persistent_keyword] = ACTIONS(1909),
    [sym_dev_keyword] = ACTIONS(1909),
    [sym_devlev_keyword] = ACTIONS(1909),
    [sym_devchan_keyword] = ACTIONS(1909),
    [sym_if_keyword] = ACTIONS(1909),
    [sym_switch_keyword] = ACTIONS(1909),
    [sym_case_keyword] = ACTIONS(1909),
    [sym_default_keyword] = ACTIONS(1909),
    [sym_while_keyword] = ACTIONS(1909),
    [sym_for_keyword] = ACTIONS(1909),
    [sym_break_keyword] = ACTIONS(1909),
    [sym_continue_keyword] = ACTIONS(1909),
    [sym_return_keyword] = ACTIONS(1909),
    [sym_select_keyword] = ACTIONS(1909),
    [sym_active_keyword] = ACTIONS(1909),
    [sym_struct_keyword] = ACTIONS(1909),
    [sym_structure_keyword] = ACTIONS(1909),
    [sym_button_event_keyword] = ACTIONS(1909),
    [sym_channel_event_keyword] = ACTIONS(1909),
    [sym_level_event_keyword] = ACTIONS(1909),
    [sym_data_event_keyword] = ACTIONS(1909),
    [sym_timeline_event_keyword] = ACTIONS(1909),
    [sym_custom_event_keyword] = ACTIONS(1909),
    [sym_push_keyword] = ACTIONS(1909),
    [sym_release_keyword] = ACTIONS(1909),
    [aux_sym_on_keyword_token1] = ACTIONS(1909),
    [aux_sym_off_keyword_token1] = ACTIONS(1909),
    [sym_send_level_keyword] = ACTIONS(1909),
    [sym_send_string_keyword] = ACTIONS(1909),
    [sym_send_command_keyword] = ACTIONS(1909),
    [sym_clear_buffer_keyword] = ACTIONS(1909),
    [sym_create_buffer_keyword] = ACTIONS(1909),
    [sym_create_multi_buffer_keyword] = ACTIONS(1909),
    [sym_call_keyword] = ACTIONS(1909),
    [sym_system_call_keyword] = ACTIONS(1909),
    [sym_devchan_to_keyword] = ACTIONS(1909),
    [sym_devchan_min_to_keyword] = ACTIONS(1909),
    [sym_devchan_total_off_keyword] = ACTIONS(1909),
    [sym_devchan_pulse_keyword] = ACTIONS(1909),
    [sym_wait_keyword] = ACTIONS(1909),
    [sym_cancel_wait_keyword] = ACTIONS(1909),
    [sym_wait_until_keyword] = ACTIONS(1909),
    [sym_cancel_wait_until_keyword] = ACTIONS(1909),
    [sym_cancel_all_wait_keyword] = ACTIONS(1909),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1909),
    [sym_not] = ACTIONS(1909),
    [sym_bnot] = ACTIONS(1909),
    [sym___file__] = ACTIONS(1909),
    [sym___date__] = ACTIONS(1909),
    [sym___ldate__] = ACTIONS(1909),
    [sym___time__] = ACTIONS(1909),
    [sym___line__] = ACTIONS(1909),
    [sym___name__] = ACTIONS(1909),
    [sym___netlinx__] = ACTIONS(1909),
    [sym_netlinx_axi_version] = ACTIONS(1909),
    [sym_url_flg_tcp] = ACTIONS(1909),
    [sym_url_flg_accinfopresent] = ACTIONS(1909),
    [sym_url_flg_temp] = ACTIONS(1909),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1909),
    [sym_url_flg_stat_mask] = ACTIONS(1909),
    [sym_url_flg_stat_lookup] = ACTIONS(1909),
    [sym_url_flg_stat_connecting] = ACTIONS(1909),
    [sym_url_flg_stat_waiting] = ACTIONS(1909),
    [sym_url_flg_stat_connected] = ACTIONS(1909),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1909),
    [sym_first_virtual_device] = ACTIONS(1909),
    [sym_first_local_port] = ACTIONS(1909),
    [sym_dynamic_virtual_device] = ACTIONS(1909),
    [sym_timeline_once] = ACTIONS(1909),
    [sym_timeline_repeat] = ACTIONS(1909),
    [sym_timeline_absolute] = ACTIONS(1909),
    [sym_timeline_relative] = ACTIONS(1909),
    [sym_do_push_timed_infinite] = ACTIONS(1909),
    [sym_source_type_no_address] = ACTIONS(1909),
    [sym_source_type_neuron_id] = ACTIONS(1909),
    [sym_source_type_ip_address] = ACTIONS(1909),
    [sym_source_type_axlink] = ACTIONS(1909),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1909),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1909),
    [sym_source_type_ip_socket_address] = ACTIONS(1909),
    [sym_source_type_rs232] = ACTIONS(1909),
    [sym_source_type_internal] = ACTIONS(1909),
    [sym_source_type_mac_address] = ACTIONS(1909),
    [sym_source_type_ipv4_port] = ACTIONS(1909),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1909),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1909),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1909),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1909),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1909),
    [sym_true] = ACTIONS(1909),
    [sym_false] = ACTIONS(1909),
    [sym_file_read_only] = ACTIONS(1909),
    [sym_file_rw_new] = ACTIONS(1909),
    [sym_file_rw_append] = ACTIONS(1909),
    [sym_ip_tcp] = ACTIONS(1909),
    [sym_ip_udp] = ACTIONS(1909),
    [sym_ip_udp_2way] = ACTIONS(1909),
    [sym_xml_encode_types] = ACTIONS(1909),
    [sym_xml_encode_char_as_list] = ACTIONS(1909),
    [sym_xml_encode_le] = ACTIONS(1909),
    [sym_xml_decode_types] = ACTIONS(1909),
    [sym_xml_decode_no_preserve] = ACTIONS(1909),
    [sym_duet_dev_type_display_device] = ACTIONS(1909),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1909),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1909),
    [sym_duet_dev_type_io_device] = ACTIONS(1909),
    [sym_duet_dev_type_relay_device] = ACTIONS(1909),
    [sym_duet_dev_type_ups] = ACTIONS(1909),
    [sym_duet_dev_type_amplifier] = ACTIONS(1909),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1909),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1909),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1909),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1909),
    [sym_duet_dev_type_camera] = ACTIONS(1909),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1909),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1909),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1909),
    [sym_duet_dev_type_dss] = ACTIONS(1909),
    [sym_duet_dev_type_dvr] = ACTIONS(1909),
    [sym_duet_dev_type_disc_device] = ACTIONS(1909),
    [sym_duet_dev_type_document_camera] = ACTIONS(1909),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1909),
    [sym_duet_dev_type_hvac] = ACTIONS(1909),
    [sym_duet_dev_type_keypad] = ACTIONS(1909),
    [sym_duet_dev_type_light] = ACTIONS(1909),
    [sym_duet_dev_type_monitor] = ACTIONS(1909),
    [sym_duet_dev_type_motor] = ACTIONS(1909),
    [sym_duet_dev_type_multi_window] = ACTIONS(1909),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1909),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1909),
    [sym_duet_dev_type_receiver] = ACTIONS(1909),
    [sym_duet_dev_type_security_system] = ACTIONS(1909),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1909),
    [sym_duet_dev_type_settop_box] = ACTIONS(1909),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1909),
    [sym_duet_dev_type_switcher] = ACTIONS(1909),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1909),
    [sym_duet_dev_type_tv] = ACTIONS(1909),
    [sym_duet_dev_type_utility] = ACTIONS(1909),
    [sym_duet_dev_type_vcr] = ACTIONS(1909),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1909),
    [sym_duet_dev_type_video_processor] = ACTIONS(1909),
    [sym_duet_dev_type_video_projector] = ACTIONS(1909),
    [sym_duet_dev_type_video_wall] = ACTIONS(1909),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1909),
    [sym_duet_dev_type_weather] = ACTIONS(1909),
    [sym_duet_dev_not_polled] = ACTIONS(1909),
    [sym_duet_dev_polled] = ACTIONS(1909),
    [sym_clkmgr_mode_network] = ACTIONS(1909),
    [sym_clkmgr_mode_standalone] = ACTIONS(1909),
    [sym_normal_standby] = ACTIONS(1909),
    [sym_normal_wake] = ACTIONS(1909),
    [sym_ip_multicast_ttl_option] = ACTIONS(1909),
    [sym_ip_ttl_subnet] = ACTIONS(1909),
    [sym_ip_ttl_site] = ACTIONS(1909),
    [sym_ip_ttl_region] = ACTIONS(1909),
    [sym_ip_ttl_continent] = ACTIONS(1909),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1909),
    [sym_ip_nodelay_on] = ACTIONS(1909),
    [sym_ip_nodelay_off] = ACTIONS(1909),
    [sym_amx_error] = ACTIONS(1909),
    [sym_amx_warning] = ACTIONS(1909),
    [sym_amx_info] = ACTIONS(1909),
    [sym_amx_debug] = ACTIONS(1909),
    [sym_netlinx_login_success] = ACTIONS(1909),
    [sym_netlinx_login_fail] = ACTIONS(1909),
    [sym_netlinx_logout] = ACTIONS(1909),
    [sym_null_str] = ACTIONS(1909),
    [sym_smtp_address] = ACTIONS(1909),
    [sym_smtp_port_number] = ACTIONS(1909),
    [sym_smtp_username] = ACTIONS(1909),
    [sym_smtp_password] = ACTIONS(1909),
    [sym_smtp_require_tls] = ACTIONS(1909),
    [sym_smtp_from] = ACTIONS(1909),
    [sym_smtp_tls_true] = ACTIONS(1909),
    [sym_smtp_tls_false] = ACTIONS(1909),
    [sym_listview_on_row_select_event] = ACTIONS(1909),
    [sym_data_structure_unknown] = ACTIONS(1909),
    [sym_data_structure_datafeed] = ACTIONS(1909),
    [sym_data_structure_datarecordset] = ACTIONS(1909),
    [sym_data_structure_datarecord] = ACTIONS(1909),
    [sym_data_structure_datafield] = ACTIONS(1909),
    [sym_data_max_content_field_count] = ACTIONS(1909),
    [sym_data_max_metadata_field_count] = ACTIONS(1909),
    [sym_data_max_id_length] = ACTIONS(1909),
    [sym_data_max_name_length] = ACTIONS(1909),
    [sym_data_max_description_length] = ACTIONS(1909),
    [sym_data_max_type_length] = ACTIONS(1909),
    [sym_data_max_format_length] = ACTIONS(1909),
    [sym_data_max_source_length] = ACTIONS(1909),
    [sym_data_max_value_length] = ACTIONS(1909),
    [sym_data_max_label_length] = ACTIONS(1909),
    [sym_data_max_url_length] = ACTIONS(1909),
    [sym_max_last_login_info_length] = ACTIONS(1909),
    [sym_data_type_unknown] = ACTIONS(1909),
    [sym_data_type_string] = ACTIONS(1909),
    [sym_data_type_datetime] = ACTIONS(1909),
    [sym_data_type_date] = ACTIONS(1909),
    [sym_data_type_time] = ACTIONS(1909),
    [sym_data_type_image] = ACTIONS(1909),
    [sym_data_format_url] = ACTIONS(1909),
    [sym_data_format_phone] = ACTIONS(1909),
    [sym_data_format_email] = ACTIONS(1909),
    [sym_data_format_iso8601] = ACTIONS(1909),
    [sym_auditlog_priv] = ACTIONS(1909),
    [sym_configuration_priv] = ACTIONS(1909),
    [sym_deviceconfig_priv] = ACTIONS(1909),
    [sym_ftp_priv] = ACTIONS(1909),
    [sym_http_priv] = ACTIONS(1909),
    [sym_networkconfig_priv] = ACTIONS(1909),
    [sym_programport_priv] = ACTIONS(1909),
    [sym_remoteui_priv] = ACTIONS(1909),
    [sym_securitycontrol_priv] = ACTIONS(1909),
    [sym_softwaremanagement_priv] = ACTIONS(1909),
    [sym_terminal_priv] = ACTIONS(1909),
    [sym_tpadmin_priv] = ACTIONS(1909),
    [sym_user1_priv] = ACTIONS(1909),
    [sym_user2_priv] = ACTIONS(1909),
    [sym_user3_priv] = ACTIONS(1909),
    [sym_user4_priv] = ACTIONS(1909),
    [sym_usermanagement_priv] = ACTIONS(1909),
    [sym_valid_account] = ACTIONS(1909),
    [sym_err_name_invalid] = ACTIONS(1909),
    [sym_err_password_invalid] = ACTIONS(1909),
    [sym_err_invalid_account] = ACTIONS(1909),
    [sym_err_no_authorization] = ACTIONS(1909),
    [sym_err_invalid_parameter] = ACTIONS(1909),
    [sym_err_user_locked_out] = ACTIONS(1909),
    [sym_err_user_account_expired] = ACTIONS(1909),
    [sym_err_general_error] = ACTIONS(1909),
    [sym_err_authentication_error] = ACTIONS(1909),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1909),
    [sym_tls_validate_certificate] = ACTIONS(1909),
    [sym_tdata] = ACTIONS(1909),
    [sym_tchannel] = ACTIONS(1909),
    [sym_tlevel] = ACTIONS(1909),
    [sym_tbutton] = ACTIONS(1909),
    [sym_ttimeline] = ACTIONS(1909),
    [sym_tcustom] = ACTIONS(1909),
    [sym_url_struct] = ACTIONS(1909),
    [sym_dns_struct] = ACTIONS(1909),
    [sym_ip_address_struct] = ACTIONS(1909),
    [sym_dev_info_struct] = ACTIONS(1909),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1909),
    [sym_data_feed] = ACTIONS(1909),
    [sym_data_field] = ACTIONS(1909),
    [sym_data_record] = ACTIONS(1909),
    [sym_wc_data_feed] = ACTIONS(1909),
    [sym_wc_data_field] = ACTIONS(1909),
    [sym_wc_data_record] = ACTIONS(1909),
    [sym_last_login_info] = ACTIONS(1909),
    [sym____reserved___] = ACTIONS(1909),
    [sym_date] = ACTIONS(1909),
    [sym_day] = ACTIONS(1909),
    [sym_time] = ACTIONS(1909),
    [sym_ldate] = ACTIONS(1909),
    [sym_push_device] = ACTIONS(1909),
    [sym_push_channel] = ACTIONS(1909),
    [sym_push_devchan] = ACTIONS(1909),
    [sym_release_device] = ACTIONS(1909),
    [sym_release_channel] = ACTIONS(1909),
    [sym_release_devchan] = ACTIONS(1909),
    [sym_master_slot] = ACTIONS(1909),
    [sym_get_pulse_time] = ACTIONS(1909),
    [sym_get_timer] = ACTIONS(1909),
    [sym_master_sn] = ACTIONS(1909),
    [sym_system_number] = ACTIONS(1909),
    [sym_button] = ACTIONS(1909),
    [sym_dv_channel] = ACTIONS(1909),
    [sym_data] = ACTIONS(1909),
    [sym_level] = ACTIONS(1909),
    [sym_timeline] = ACTIONS(1909),
    [sym_channel] = ACTIONS(1909),
    [sym_custom] = ACTIONS(1909),
    [sym_atoi] = ACTIONS(1909),
    [sym_atol] = ACTIONS(1909),
    [sym_atof] = ACTIONS(1909),
    [sym_itoa] = ACTIONS(1909),
    [sym_itohex] = ACTIONS(1909),
    [sym_hextoi] = ACTIONS(1909),
    [sym_ftoa] = ACTIONS(1909),
    [sym_find_string] = ACTIONS(1909),
    [sym_left_string] = ACTIONS(1909),
    [sym_right_string] = ACTIONS(1909),
    [sym_mid_string] = ACTIONS(1909),
    [sym_remove_string] = ACTIONS(1909),
    [sym_get_buffer_string] = ACTIONS(1909),
    [sym_get_buffer_char] = ACTIONS(1909),
    [sym_get_multi_buffer_string] = ACTIONS(1909),
    [sym_length_string] = ACTIONS(1909),
    [sym_max_length_string] = ACTIONS(1909),
    [sym_set_length_string] = ACTIONS(1909),
    [sym_compare_string] = ACTIONS(1909),
    [sym_upper_string] = ACTIONS(1909),
    [sym_lower_string] = ACTIONS(1909),
    [sym_get_last] = ACTIONS(1909),
    [sym_date_to_day] = ACTIONS(1909),
    [sym_date_to_month] = ACTIONS(1909),
    [sym_date_to_year] = ACTIONS(1909),
    [sym_time_to_hour] = ACTIONS(1909),
    [sym_time_to_minute] = ACTIONS(1909),
    [sym_time_to_second] = ACTIONS(1909),
    [sym_day_of_week] = ACTIONS(1909),
    [sym_set_pulse_time] = ACTIONS(1909),
    [sym_set_timer] = ACTIONS(1909),
    [sym_file_open] = ACTIONS(1909),
    [sym_file_close] = ACTIONS(1909),
    [sym_file_read] = ACTIONS(1909),
    [sym_file_read_line] = ACTIONS(1909),
    [sym_file_write] = ACTIONS(1909),
    [sym_file_write_line] = ACTIONS(1909),
    [sym_file_copy] = ACTIONS(1909),
    [sym_file_delete] = ACTIONS(1909),
    [sym_file_rename] = ACTIONS(1909),
    [sym_file_dir] = ACTIONS(1909),
    [sym_file_getdir] = ACTIONS(1909),
    [sym_file_setdir] = ACTIONS(1909),
    [sym_file_create_dir] = ACTIONS(1909),
    [sym_file_remove_dir] = ACTIONS(1909),
    [sym_file_seek] = ACTIONS(1909),
    [sym_length_array] = ACTIONS(1909),
    [sym_max_length_array] = ACTIONS(1909),
    [sym_set_length_array] = ACTIONS(1909),
    [sym_ip_client_open] = ACTIONS(1909),
    [sym_ip_client_close] = ACTIONS(1909),
    [sym_ip_server_open] = ACTIONS(1909),
    [sym_ip_server_close] = ACTIONS(1909),
    [sym_ip_bound_client_open] = ACTIONS(1909),
    [sym_ip_mc_server_open] = ACTIONS(1909),
    [sym_ip_set_option] = ACTIONS(1909),
    [sym_ssh_client_open] = ACTIONS(1909),
    [sym_ssh_client_close] = ACTIONS(1909),
    [sym_tls_client_open] = ACTIONS(1909),
    [sym_tls_client_close] = ACTIONS(1909),
    [sym_do_push] = ACTIONS(1909),
    [sym_do_push_timed] = ACTIONS(1909),
    [sym_do_release] = ACTIONS(1909),
    [sym_device_id] = ACTIONS(1909),
    [sym_device_info] = ACTIONS(1909),
    [sym_device_id_string] = ACTIONS(1909),
    [sym_device_standby] = ACTIONS(1909),
    [sym_device_wake] = ACTIONS(1909),
    [sym_timeline_create] = ACTIONS(1909),
    [sym_timeline_kill] = ACTIONS(1909),
    [sym_timeline_reload] = ACTIONS(1909),
    [sym_timeline_pause] = ACTIONS(1909),
    [sym_timeline_restart] = ACTIONS(1909),
    [sym_timeline_set] = ACTIONS(1909),
    [sym_timeline_get] = ACTIONS(1909),
    [sym_timeline_active] = ACTIONS(1909),
    [sym_timeline_dynamic_id] = ACTIONS(1909),
    [sym_variable_to_string] = ACTIONS(1909),
    [sym_string_to_variable] = ACTIONS(1909),
    [sym_length_variable_to_string] = ACTIONS(1909),
    [sym_type_cast] = ACTIONS(1909),
    [sym_variable_to_xml] = ACTIONS(1909),
    [sym_xml_to_variable] = ACTIONS(1909),
    [sym_length_variable_to_xml] = ACTIONS(1909),
    [sym_redirect_string] = ACTIONS(1909),
    [sym_random_number] = ACTIONS(1909),
    [sym_get_unique_id] = ACTIONS(1909),
    [sym_get_system_number] = ACTIONS(1909),
    [sym_set_system_number] = ACTIONS(1909),
    [sym_get_serial_number] = ACTIONS(1909),
    [sym_get_master_build] = ACTIONS(1909),
    [sym_set_validation_code] = ACTIONS(1909),
    [sym_reboot] = ACTIONS(1909),
    [sym_load_duet_module] = ACTIONS(1909),
    [sym_unload_duet_module] = ACTIONS(1909),
    [sym_duet_mem_size_set] = ACTIONS(1909),
    [sym_duet_mem_size_get] = ACTIONS(1909),
    [sym_dynamic_application_device] = ACTIONS(1909),
    [sym_dynamic_polled_port] = ACTIONS(1909),
    [sym_static_port_binding] = ACTIONS(1909),
    [sym_static_ip_binding] = ACTIONS(1909),
    [sym_get_dns_list] = ACTIONS(1909),
    [sym_set_dns_list] = ACTIONS(1909),
    [sym_get_ip_address] = ACTIONS(1909),
    [sym_set_ip_address] = ACTIONS(1909),
    [sym_get_url_list] = ACTIONS(1909),
    [sym_add_url_entry] = ACTIONS(1909),
    [sym_delete_url_entry] = ACTIONS(1909),
    [sym_abs_value] = ACTIONS(1909),
    [sym_max_value] = ACTIONS(1909),
    [sym_min_value] = ACTIONS(1909),
    [sym_exp_value] = ACTIONS(1909),
    [sym_log_value] = ACTIONS(1909),
    [sym_log10_value] = ACTIONS(1909),
    [sym_power_value] = ACTIONS(1909),
    [sym_sqrt_value] = ACTIONS(1909),
    [sym_raw_be] = ACTIONS(1909),
    [sym_raw_le] = ACTIONS(1909),
    [sym_internal_queue_size_set] = ACTIONS(1909),
    [sym_internal_queue_size_get] = ACTIONS(1909),
    [sym_internal_threshold_set] = ACTIONS(1909),
    [sym_internal_threshold_get] = ACTIONS(1909),
    [sym_rebuilt_event] = ACTIONS(1909),
    [sym_do_custom_event] = ACTIONS(1909),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1909),
    [sym_clkmgr_set_clk_source] = ACTIONS(1909),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1909),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1909),
    [sym_clkmgr_get_timezone] = ACTIONS(1909),
    [sym_clkmgr_set_timezone] = ACTIONS(1909),
    [sym_clkmgr_get_resync_period] = ACTIONS(1909),
    [sym_clkmgr_set_resync_period] = ACTIONS(1909),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1909),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1909),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1909),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1909),
    [sym_clkmgr_get_timeservers] = ACTIONS(1909),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1909),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1909),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1909),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1909),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1909),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1909),
    [sym_set_log_level] = ACTIONS(1909),
    [sym_get_log_level] = ACTIONS(1909),
    [sym_amx_log] = ACTIONS(1909),
    [sym_smtp_server_config_set] = ACTIONS(1909),
    [sym_smtp_server_config_get] = ACTIONS(1909),
    [sym_smtp_send] = ACTIONS(1909),
    [sym_validate_netlinx_account] = ACTIONS(1909),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1909),
    [sym_authenticate_certificate] = ACTIONS(1909),
    [sym_audit_netlinx_session_event] = ACTIONS(1909),
    [sym_audit_netlinx_generic_event] = ACTIONS(1909),
    [sym_get_available_flash_disk_space] = ACTIONS(1909),
    [sym_get_max_flash_disk_space] = ACTIONS(1909),
    [sym_data_create_feed] = ACTIONS(1909),
    [sym_data_delete_feed] = ACTIONS(1909),
    [sym_data_publish_feed] = ACTIONS(1909),
    [sym_data_get_published_feed] = ACTIONS(1909),
    [sym_data_add_record] = ACTIONS(1909),
    [sym_data_get_event_record] = ACTIONS(1909),
    [sym__wc_data_create_feed] = ACTIONS(1909),
    [sym__wc_data_add_record] = ACTIONS(1909),
    [sym__wc_data_get_event_record] = ACTIONS(1909),
    [sym_format] = ACTIONS(1909),
    [sym_astro_clock] = ACTIONS(1909),
    [sym_set_outdoor_temperature] = ACTIONS(1909),
    [sym_set_virtual_level_count] = ACTIONS(1909),
    [sym_set_virtual_channel_count] = ACTIONS(1909),
    [sym_set_virtual_port_count] = ACTIONS(1909),
    [sym__wc_encode_int] = ACTIONS(1909),
    [sym__wc_decode_int] = ACTIONS(1909),
    [sym__wc_int] = ACTIONS(1909),
    [sym__wc_to_ch_int] = ACTIONS(1909),
    [sym__ch_to_wc_int] = ACTIONS(1909),
    [sym__wc_find_string_int] = ACTIONS(1909),
    [sym__wc_left_string_int] = ACTIONS(1909),
    [sym__wc_lower_string_int] = ACTIONS(1909),
    [sym__wc_mid_string_int] = ACTIONS(1909),
    [sym__wc_remove_string_int] = ACTIONS(1909),
    [sym__wc_right_string_int] = ACTIONS(1909),
    [sym__wc_upper_string_int] = ACTIONS(1909),
    [sym__wc_compare_string_int] = ACTIONS(1909),
    [sym__wc_get_buffer_char_int] = ACTIONS(1909),
    [sym__wc_get_buffer_string_int] = ACTIONS(1909),
    [sym__wc_concat_string_int] = ACTIONS(1909),
    [sym__wc_file_open_int] = ACTIONS(1909),
    [sym__wc_file_close_int] = ACTIONS(1909),
    [sym__wc_file_read_int] = ACTIONS(1909),
    [sym__wc_file_read_line_int] = ACTIONS(1909),
    [sym__wc_file_write_int] = ACTIONS(1909),
    [sym__wc_file_write_line_int] = ACTIONS(1909),
    [aux_sym_preproc_include_token1] = ACTIONS(1909),
    [aux_sym_preproc_define_token1] = ACTIONS(1911),
    [aux_sym_preproc_warn_token1] = ACTIONS(1911),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1911),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1911),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1911),
    [anon_sym_LBRACK] = ACTIONS(1911),
    [anon_sym_LBRACE] = ACTIONS(1911),
    [anon_sym_LPAREN] = ACTIONS(1909),
    [anon_sym_BANG] = ACTIONS(1911),
    [anon_sym_TILDE] = ACTIONS(1911),
    [anon_sym_DASH] = ACTIONS(1909),
    [anon_sym_PLUS] = ACTIONS(1909),
    [anon_sym_DASH_DASH] = ACTIONS(1911),
    [anon_sym_PLUS_PLUS] = ACTIONS(1911),
    [anon_sym_DQUOTE] = ACTIONS(1911),
    [anon_sym_SQUOTE] = ACTIONS(1911),
    [sym_number_literal] = ACTIONS(1911),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(456)] = {
    [ts_builtin_sym_end] = ACTIONS(1915),
    [sym_identifier] = ACTIONS(1913),
    [sym_program_name_keyword] = ACTIONS(1913),
    [sym_module_name_keyword] = ACTIONS(1913),
    [sym_define_device_keyword] = ACTIONS(1913),
    [sym_define_combine_keyword] = ACTIONS(1913),
    [sym_define_constant_keyword] = ACTIONS(1913),
    [sym_define_type_keyword] = ACTIONS(1913),
    [sym_define_variable_keyword] = ACTIONS(1913),
    [sym_define_system_variable_keyword] = ACTIONS(1913),
    [sym_define_connect_level_keyword] = ACTIONS(1913),
    [sym_define_latching_keyword] = ACTIONS(1913),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1913),
    [sym_define_toggling_keyword] = ACTIONS(1913),
    [sym_define_call_keyword] = ACTIONS(1913),
    [sym_define_function_keyword] = ACTIONS(1913),
    [sym_define_library_function_keyword] = ACTIONS(1913),
    [sym_define_start_keyword] = ACTIONS(1913),
    [sym_define_event_keyword] = ACTIONS(1913),
    [sym_define_program_keyword] = ACTIONS(1913),
    [sym_define_module_keyword] = ACTIONS(1913),
    [sym_char_keyword] = ACTIONS(1913),
    [sym_widechar_keyword] = ACTIONS(1913),
    [sym_integer_keyword] = ACTIONS(1913),
    [sym_sinteger_keyword] = ACTIONS(1913),
    [sym_long_keyword] = ACTIONS(1913),
    [sym_slong_keyword] = ACTIONS(1913),
    [sym_float_keyword] = ACTIONS(1913),
    [sym_double_keyword] = ACTIONS(1913),
    [sym_variant_keyword] = ACTIONS(1913),
    [sym_variantarray_keyword] = ACTIONS(1913),
    [sym_local_var_keyword] = ACTIONS(1913),
    [sym_stack_var_keyword] = ACTIONS(1913),
    [sym_constant_keyword] = ACTIONS(1913),
    [sym_volatile_keyword] = ACTIONS(1913),
    [sym_non_volatile_keyword] = ACTIONS(1913),
    [sym_persistent_keyword] = ACTIONS(1913),
    [sym_dev_keyword] = ACTIONS(1913),
    [sym_devlev_keyword] = ACTIONS(1913),
    [sym_devchan_keyword] = ACTIONS(1913),
    [sym_if_keyword] = ACTIONS(1913),
    [sym_switch_keyword] = ACTIONS(1913),
    [sym_case_keyword] = ACTIONS(1913),
    [sym_default_keyword] = ACTIONS(1913),
    [sym_while_keyword] = ACTIONS(1913),
    [sym_for_keyword] = ACTIONS(1913),
    [sym_break_keyword] = ACTIONS(1913),
    [sym_continue_keyword] = ACTIONS(1913),
    [sym_return_keyword] = ACTIONS(1913),
    [sym_select_keyword] = ACTIONS(1913),
    [sym_active_keyword] = ACTIONS(1913),
    [sym_struct_keyword] = ACTIONS(1913),
    [sym_structure_keyword] = ACTIONS(1913),
    [sym_button_event_keyword] = ACTIONS(1913),
    [sym_channel_event_keyword] = ACTIONS(1913),
    [sym_level_event_keyword] = ACTIONS(1913),
    [sym_data_event_keyword] = ACTIONS(1913),
    [sym_timeline_event_keyword] = ACTIONS(1913),
    [sym_custom_event_keyword] = ACTIONS(1913),
    [sym_push_keyword] = ACTIONS(1913),
    [sym_release_keyword] = ACTIONS(1913),
    [aux_sym_on_keyword_token1] = ACTIONS(1913),
    [aux_sym_off_keyword_token1] = ACTIONS(1913),
    [sym_send_level_keyword] = ACTIONS(1913),
    [sym_send_string_keyword] = ACTIONS(1913),
    [sym_send_command_keyword] = ACTIONS(1913),
    [sym_clear_buffer_keyword] = ACTIONS(1913),
    [sym_create_buffer_keyword] = ACTIONS(1913),
    [sym_create_multi_buffer_keyword] = ACTIONS(1913),
    [sym_call_keyword] = ACTIONS(1913),
    [sym_system_call_keyword] = ACTIONS(1913),
    [sym_devchan_to_keyword] = ACTIONS(1913),
    [sym_devchan_min_to_keyword] = ACTIONS(1913),
    [sym_devchan_total_off_keyword] = ACTIONS(1913),
    [sym_devchan_pulse_keyword] = ACTIONS(1913),
    [sym_wait_keyword] = ACTIONS(1913),
    [sym_cancel_wait_keyword] = ACTIONS(1913),
    [sym_wait_until_keyword] = ACTIONS(1913),
    [sym_cancel_wait_until_keyword] = ACTIONS(1913),
    [sym_cancel_all_wait_keyword] = ACTIONS(1913),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1913),
    [sym_not] = ACTIONS(1913),
    [sym_bnot] = ACTIONS(1913),
    [sym___file__] = ACTIONS(1913),
    [sym___date__] = ACTIONS(1913),
    [sym___ldate__] = ACTIONS(1913),
    [sym___time__] = ACTIONS(1913),
    [sym___line__] = ACTIONS(1913),
    [sym___name__] = ACTIONS(1913),
    [sym___netlinx__] = ACTIONS(1913),
    [sym_netlinx_axi_version] = ACTIONS(1913),
    [sym_url_flg_tcp] = ACTIONS(1913),
    [sym_url_flg_accinfopresent] = ACTIONS(1913),
    [sym_url_flg_temp] = ACTIONS(1913),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1913),
    [sym_url_flg_stat_mask] = ACTIONS(1913),
    [sym_url_flg_stat_lookup] = ACTIONS(1913),
    [sym_url_flg_stat_connecting] = ACTIONS(1913),
    [sym_url_flg_stat_waiting] = ACTIONS(1913),
    [sym_url_flg_stat_connected] = ACTIONS(1913),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1913),
    [sym_first_virtual_device] = ACTIONS(1913),
    [sym_first_local_port] = ACTIONS(1913),
    [sym_dynamic_virtual_device] = ACTIONS(1913),
    [sym_timeline_once] = ACTIONS(1913),
    [sym_timeline_repeat] = ACTIONS(1913),
    [sym_timeline_absolute] = ACTIONS(1913),
    [sym_timeline_relative] = ACTIONS(1913),
    [sym_do_push_timed_infinite] = ACTIONS(1913),
    [sym_source_type_no_address] = ACTIONS(1913),
    [sym_source_type_neuron_id] = ACTIONS(1913),
    [sym_source_type_ip_address] = ACTIONS(1913),
    [sym_source_type_axlink] = ACTIONS(1913),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1913),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1913),
    [sym_source_type_ip_socket_address] = ACTIONS(1913),
    [sym_source_type_rs232] = ACTIONS(1913),
    [sym_source_type_internal] = ACTIONS(1913),
    [sym_source_type_mac_address] = ACTIONS(1913),
    [sym_source_type_ipv4_port] = ACTIONS(1913),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1913),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1913),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1913),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1913),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1913),
    [sym_true] = ACTIONS(1913),
    [sym_false] = ACTIONS(1913),
    [sym_file_read_only] = ACTIONS(1913),
    [sym_file_rw_new] = ACTIONS(1913),
    [sym_file_rw_append] = ACTIONS(1913),
    [sym_ip_tcp] = ACTIONS(1913),
    [sym_ip_udp] = ACTIONS(1913),
    [sym_ip_udp_2way] = ACTIONS(1913),
    [sym_xml_encode_types] = ACTIONS(1913),
    [sym_xml_encode_char_as_list] = ACTIONS(1913),
    [sym_xml_encode_le] = ACTIONS(1913),
    [sym_xml_decode_types] = ACTIONS(1913),
    [sym_xml_decode_no_preserve] = ACTIONS(1913),
    [sym_duet_dev_type_display_device] = ACTIONS(1913),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1913),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1913),
    [sym_duet_dev_type_io_device] = ACTIONS(1913),
    [sym_duet_dev_type_relay_device] = ACTIONS(1913),
    [sym_duet_dev_type_ups] = ACTIONS(1913),
    [sym_duet_dev_type_amplifier] = ACTIONS(1913),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1913),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1913),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1913),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1913),
    [sym_duet_dev_type_camera] = ACTIONS(1913),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1913),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1913),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1913),
    [sym_duet_dev_type_dss] = ACTIONS(1913),
    [sym_duet_dev_type_dvr] = ACTIONS(1913),
    [sym_duet_dev_type_disc_device] = ACTIONS(1913),
    [sym_duet_dev_type_document_camera] = ACTIONS(1913),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1913),
    [sym_duet_dev_type_hvac] = ACTIONS(1913),
    [sym_duet_dev_type_keypad] = ACTIONS(1913),
    [sym_duet_dev_type_light] = ACTIONS(1913),
    [sym_duet_dev_type_monitor] = ACTIONS(1913),
    [sym_duet_dev_type_motor] = ACTIONS(1913),
    [sym_duet_dev_type_multi_window] = ACTIONS(1913),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1913),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1913),
    [sym_duet_dev_type_receiver] = ACTIONS(1913),
    [sym_duet_dev_type_security_system] = ACTIONS(1913),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1913),
    [sym_duet_dev_type_settop_box] = ACTIONS(1913),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1913),
    [sym_duet_dev_type_switcher] = ACTIONS(1913),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1913),
    [sym_duet_dev_type_tv] = ACTIONS(1913),
    [sym_duet_dev_type_utility] = ACTIONS(1913),
    [sym_duet_dev_type_vcr] = ACTIONS(1913),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1913),
    [sym_duet_dev_type_video_processor] = ACTIONS(1913),
    [sym_duet_dev_type_video_projector] = ACTIONS(1913),
    [sym_duet_dev_type_video_wall] = ACTIONS(1913),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1913),
    [sym_duet_dev_type_weather] = ACTIONS(1913),
    [sym_duet_dev_not_polled] = ACTIONS(1913),
    [sym_duet_dev_polled] = ACTIONS(1913),
    [sym_clkmgr_mode_network] = ACTIONS(1913),
    [sym_clkmgr_mode_standalone] = ACTIONS(1913),
    [sym_normal_standby] = ACTIONS(1913),
    [sym_normal_wake] = ACTIONS(1913),
    [sym_ip_multicast_ttl_option] = ACTIONS(1913),
    [sym_ip_ttl_subnet] = ACTIONS(1913),
    [sym_ip_ttl_site] = ACTIONS(1913),
    [sym_ip_ttl_region] = ACTIONS(1913),
    [sym_ip_ttl_continent] = ACTIONS(1913),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1913),
    [sym_ip_nodelay_on] = ACTIONS(1913),
    [sym_ip_nodelay_off] = ACTIONS(1913),
    [sym_amx_error] = ACTIONS(1913),
    [sym_amx_warning] = ACTIONS(1913),
    [sym_amx_info] = ACTIONS(1913),
    [sym_amx_debug] = ACTIONS(1913),
    [sym_netlinx_login_success] = ACTIONS(1913),
    [sym_netlinx_login_fail] = ACTIONS(1913),
    [sym_netlinx_logout] = ACTIONS(1913),
    [sym_null_str] = ACTIONS(1913),
    [sym_smtp_address] = ACTIONS(1913),
    [sym_smtp_port_number] = ACTIONS(1913),
    [sym_smtp_username] = ACTIONS(1913),
    [sym_smtp_password] = ACTIONS(1913),
    [sym_smtp_require_tls] = ACTIONS(1913),
    [sym_smtp_from] = ACTIONS(1913),
    [sym_smtp_tls_true] = ACTIONS(1913),
    [sym_smtp_tls_false] = ACTIONS(1913),
    [sym_listview_on_row_select_event] = ACTIONS(1913),
    [sym_data_structure_unknown] = ACTIONS(1913),
    [sym_data_structure_datafeed] = ACTIONS(1913),
    [sym_data_structure_datarecordset] = ACTIONS(1913),
    [sym_data_structure_datarecord] = ACTIONS(1913),
    [sym_data_structure_datafield] = ACTIONS(1913),
    [sym_data_max_content_field_count] = ACTIONS(1913),
    [sym_data_max_metadata_field_count] = ACTIONS(1913),
    [sym_data_max_id_length] = ACTIONS(1913),
    [sym_data_max_name_length] = ACTIONS(1913),
    [sym_data_max_description_length] = ACTIONS(1913),
    [sym_data_max_type_length] = ACTIONS(1913),
    [sym_data_max_format_length] = ACTIONS(1913),
    [sym_data_max_source_length] = ACTIONS(1913),
    [sym_data_max_value_length] = ACTIONS(1913),
    [sym_data_max_label_length] = ACTIONS(1913),
    [sym_data_max_url_length] = ACTIONS(1913),
    [sym_max_last_login_info_length] = ACTIONS(1913),
    [sym_data_type_unknown] = ACTIONS(1913),
    [sym_data_type_string] = ACTIONS(1913),
    [sym_data_type_datetime] = ACTIONS(1913),
    [sym_data_type_date] = ACTIONS(1913),
    [sym_data_type_time] = ACTIONS(1913),
    [sym_data_type_image] = ACTIONS(1913),
    [sym_data_format_url] = ACTIONS(1913),
    [sym_data_format_phone] = ACTIONS(1913),
    [sym_data_format_email] = ACTIONS(1913),
    [sym_data_format_iso8601] = ACTIONS(1913),
    [sym_auditlog_priv] = ACTIONS(1913),
    [sym_configuration_priv] = ACTIONS(1913),
    [sym_deviceconfig_priv] = ACTIONS(1913),
    [sym_ftp_priv] = ACTIONS(1913),
    [sym_http_priv] = ACTIONS(1913),
    [sym_networkconfig_priv] = ACTIONS(1913),
    [sym_programport_priv] = ACTIONS(1913),
    [sym_remoteui_priv] = ACTIONS(1913),
    [sym_securitycontrol_priv] = ACTIONS(1913),
    [sym_softwaremanagement_priv] = ACTIONS(1913),
    [sym_terminal_priv] = ACTIONS(1913),
    [sym_tpadmin_priv] = ACTIONS(1913),
    [sym_user1_priv] = ACTIONS(1913),
    [sym_user2_priv] = ACTIONS(1913),
    [sym_user3_priv] = ACTIONS(1913),
    [sym_user4_priv] = ACTIONS(1913),
    [sym_usermanagement_priv] = ACTIONS(1913),
    [sym_valid_account] = ACTIONS(1913),
    [sym_err_name_invalid] = ACTIONS(1913),
    [sym_err_password_invalid] = ACTIONS(1913),
    [sym_err_invalid_account] = ACTIONS(1913),
    [sym_err_no_authorization] = ACTIONS(1913),
    [sym_err_invalid_parameter] = ACTIONS(1913),
    [sym_err_user_locked_out] = ACTIONS(1913),
    [sym_err_user_account_expired] = ACTIONS(1913),
    [sym_err_general_error] = ACTIONS(1913),
    [sym_err_authentication_error] = ACTIONS(1913),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1913),
    [sym_tls_validate_certificate] = ACTIONS(1913),
    [sym_tdata] = ACTIONS(1913),
    [sym_tchannel] = ACTIONS(1913),
    [sym_tlevel] = ACTIONS(1913),
    [sym_tbutton] = ACTIONS(1913),
    [sym_ttimeline] = ACTIONS(1913),
    [sym_tcustom] = ACTIONS(1913),
    [sym_url_struct] = ACTIONS(1913),
    [sym_dns_struct] = ACTIONS(1913),
    [sym_ip_address_struct] = ACTIONS(1913),
    [sym_dev_info_struct] = ACTIONS(1913),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1913),
    [sym_data_feed] = ACTIONS(1913),
    [sym_data_field] = ACTIONS(1913),
    [sym_data_record] = ACTIONS(1913),
    [sym_wc_data_feed] = ACTIONS(1913),
    [sym_wc_data_field] = ACTIONS(1913),
    [sym_wc_data_record] = ACTIONS(1913),
    [sym_last_login_info] = ACTIONS(1913),
    [sym____reserved___] = ACTIONS(1913),
    [sym_date] = ACTIONS(1913),
    [sym_day] = ACTIONS(1913),
    [sym_time] = ACTIONS(1913),
    [sym_ldate] = ACTIONS(1913),
    [sym_push_device] = ACTIONS(1913),
    [sym_push_channel] = ACTIONS(1913),
    [sym_push_devchan] = ACTIONS(1913),
    [sym_release_device] = ACTIONS(1913),
    [sym_release_channel] = ACTIONS(1913),
    [sym_release_devchan] = ACTIONS(1913),
    [sym_master_slot] = ACTIONS(1913),
    [sym_get_pulse_time] = ACTIONS(1913),
    [sym_get_timer] = ACTIONS(1913),
    [sym_master_sn] = ACTIONS(1913),
    [sym_system_number] = ACTIONS(1913),
    [sym_button] = ACTIONS(1913),
    [sym_dv_channel] = ACTIONS(1913),
    [sym_data] = ACTIONS(1913),
    [sym_level] = ACTIONS(1913),
    [sym_timeline] = ACTIONS(1913),
    [sym_channel] = ACTIONS(1913),
    [sym_custom] = ACTIONS(1913),
    [sym_atoi] = ACTIONS(1913),
    [sym_atol] = ACTIONS(1913),
    [sym_atof] = ACTIONS(1913),
    [sym_itoa] = ACTIONS(1913),
    [sym_itohex] = ACTIONS(1913),
    [sym_hextoi] = ACTIONS(1913),
    [sym_ftoa] = ACTIONS(1913),
    [sym_find_string] = ACTIONS(1913),
    [sym_left_string] = ACTIONS(1913),
    [sym_right_string] = ACTIONS(1913),
    [sym_mid_string] = ACTIONS(1913),
    [sym_remove_string] = ACTIONS(1913),
    [sym_get_buffer_string] = ACTIONS(1913),
    [sym_get_buffer_char] = ACTIONS(1913),
    [sym_get_multi_buffer_string] = ACTIONS(1913),
    [sym_length_string] = ACTIONS(1913),
    [sym_max_length_string] = ACTIONS(1913),
    [sym_set_length_string] = ACTIONS(1913),
    [sym_compare_string] = ACTIONS(1913),
    [sym_upper_string] = ACTIONS(1913),
    [sym_lower_string] = ACTIONS(1913),
    [sym_get_last] = ACTIONS(1913),
    [sym_date_to_day] = ACTIONS(1913),
    [sym_date_to_month] = ACTIONS(1913),
    [sym_date_to_year] = ACTIONS(1913),
    [sym_time_to_hour] = ACTIONS(1913),
    [sym_time_to_minute] = ACTIONS(1913),
    [sym_time_to_second] = ACTIONS(1913),
    [sym_day_of_week] = ACTIONS(1913),
    [sym_set_pulse_time] = ACTIONS(1913),
    [sym_set_timer] = ACTIONS(1913),
    [sym_file_open] = ACTIONS(1913),
    [sym_file_close] = ACTIONS(1913),
    [sym_file_read] = ACTIONS(1913),
    [sym_file_read_line] = ACTIONS(1913),
    [sym_file_write] = ACTIONS(1913),
    [sym_file_write_line] = ACTIONS(1913),
    [sym_file_copy] = ACTIONS(1913),
    [sym_file_delete] = ACTIONS(1913),
    [sym_file_rename] = ACTIONS(1913),
    [sym_file_dir] = ACTIONS(1913),
    [sym_file_getdir] = ACTIONS(1913),
    [sym_file_setdir] = ACTIONS(1913),
    [sym_file_create_dir] = ACTIONS(1913),
    [sym_file_remove_dir] = ACTIONS(1913),
    [sym_file_seek] = ACTIONS(1913),
    [sym_length_array] = ACTIONS(1913),
    [sym_max_length_array] = ACTIONS(1913),
    [sym_set_length_array] = ACTIONS(1913),
    [sym_ip_client_open] = ACTIONS(1913),
    [sym_ip_client_close] = ACTIONS(1913),
    [sym_ip_server_open] = ACTIONS(1913),
    [sym_ip_server_close] = ACTIONS(1913),
    [sym_ip_bound_client_open] = ACTIONS(1913),
    [sym_ip_mc_server_open] = ACTIONS(1913),
    [sym_ip_set_option] = ACTIONS(1913),
    [sym_ssh_client_open] = ACTIONS(1913),
    [sym_ssh_client_close] = ACTIONS(1913),
    [sym_tls_client_open] = ACTIONS(1913),
    [sym_tls_client_close] = ACTIONS(1913),
    [sym_do_push] = ACTIONS(1913),
    [sym_do_push_timed] = ACTIONS(1913),
    [sym_do_release] = ACTIONS(1913),
    [sym_device_id] = ACTIONS(1913),
    [sym_device_info] = ACTIONS(1913),
    [sym_device_id_string] = ACTIONS(1913),
    [sym_device_standby] = ACTIONS(1913),
    [sym_device_wake] = ACTIONS(1913),
    [sym_timeline_create] = ACTIONS(1913),
    [sym_timeline_kill] = ACTIONS(1913),
    [sym_timeline_reload] = ACTIONS(1913),
    [sym_timeline_pause] = ACTIONS(1913),
    [sym_timeline_restart] = ACTIONS(1913),
    [sym_timeline_set] = ACTIONS(1913),
    [sym_timeline_get] = ACTIONS(1913),
    [sym_timeline_active] = ACTIONS(1913),
    [sym_timeline_dynamic_id] = ACTIONS(1913),
    [sym_variable_to_string] = ACTIONS(1913),
    [sym_string_to_variable] = ACTIONS(1913),
    [sym_length_variable_to_string] = ACTIONS(1913),
    [sym_type_cast] = ACTIONS(1913),
    [sym_variable_to_xml] = ACTIONS(1913),
    [sym_xml_to_variable] = ACTIONS(1913),
    [sym_length_variable_to_xml] = ACTIONS(1913),
    [sym_redirect_string] = ACTIONS(1913),
    [sym_random_number] = ACTIONS(1913),
    [sym_get_unique_id] = ACTIONS(1913),
    [sym_get_system_number] = ACTIONS(1913),
    [sym_set_system_number] = ACTIONS(1913),
    [sym_get_serial_number] = ACTIONS(1913),
    [sym_get_master_build] = ACTIONS(1913),
    [sym_set_validation_code] = ACTIONS(1913),
    [sym_reboot] = ACTIONS(1913),
    [sym_load_duet_module] = ACTIONS(1913),
    [sym_unload_duet_module] = ACTIONS(1913),
    [sym_duet_mem_size_set] = ACTIONS(1913),
    [sym_duet_mem_size_get] = ACTIONS(1913),
    [sym_dynamic_application_device] = ACTIONS(1913),
    [sym_dynamic_polled_port] = ACTIONS(1913),
    [sym_static_port_binding] = ACTIONS(1913),
    [sym_static_ip_binding] = ACTIONS(1913),
    [sym_get_dns_list] = ACTIONS(1913),
    [sym_set_dns_list] = ACTIONS(1913),
    [sym_get_ip_address] = ACTIONS(1913),
    [sym_set_ip_address] = ACTIONS(1913),
    [sym_get_url_list] = ACTIONS(1913),
    [sym_add_url_entry] = ACTIONS(1913),
    [sym_delete_url_entry] = ACTIONS(1913),
    [sym_abs_value] = ACTIONS(1913),
    [sym_max_value] = ACTIONS(1913),
    [sym_min_value] = ACTIONS(1913),
    [sym_exp_value] = ACTIONS(1913),
    [sym_log_value] = ACTIONS(1913),
    [sym_log10_value] = ACTIONS(1913),
    [sym_power_value] = ACTIONS(1913),
    [sym_sqrt_value] = ACTIONS(1913),
    [sym_raw_be] = ACTIONS(1913),
    [sym_raw_le] = ACTIONS(1913),
    [sym_internal_queue_size_set] = ACTIONS(1913),
    [sym_internal_queue_size_get] = ACTIONS(1913),
    [sym_internal_threshold_set] = ACTIONS(1913),
    [sym_internal_threshold_get] = ACTIONS(1913),
    [sym_rebuilt_event] = ACTIONS(1913),
    [sym_do_custom_event] = ACTIONS(1913),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1913),
    [sym_clkmgr_set_clk_source] = ACTIONS(1913),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1913),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1913),
    [sym_clkmgr_get_timezone] = ACTIONS(1913),
    [sym_clkmgr_set_timezone] = ACTIONS(1913),
    [sym_clkmgr_get_resync_period] = ACTIONS(1913),
    [sym_clkmgr_set_resync_period] = ACTIONS(1913),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1913),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1913),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1913),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1913),
    [sym_clkmgr_get_timeservers] = ACTIONS(1913),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1913),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1913),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1913),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1913),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1913),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1913),
    [sym_set_log_level] = ACTIONS(1913),
    [sym_get_log_level] = ACTIONS(1913),
    [sym_amx_log] = ACTIONS(1913),
    [sym_smtp_server_config_set] = ACTIONS(1913),
    [sym_smtp_server_config_get] = ACTIONS(1913),
    [sym_smtp_send] = ACTIONS(1913),
    [sym_validate_netlinx_account] = ACTIONS(1913),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1913),
    [sym_authenticate_certificate] = ACTIONS(1913),
    [sym_audit_netlinx_session_event] = ACTIONS(1913),
    [sym_audit_netlinx_generic_event] = ACTIONS(1913),
    [sym_get_available_flash_disk_space] = ACTIONS(1913),
    [sym_get_max_flash_disk_space] = ACTIONS(1913),
    [sym_data_create_feed] = ACTIONS(1913),
    [sym_data_delete_feed] = ACTIONS(1913),
    [sym_data_publish_feed] = ACTIONS(1913),
    [sym_data_get_published_feed] = ACTIONS(1913),
    [sym_data_add_record] = ACTIONS(1913),
    [sym_data_get_event_record] = ACTIONS(1913),
    [sym__wc_data_create_feed] = ACTIONS(1913),
    [sym__wc_data_add_record] = ACTIONS(1913),
    [sym__wc_data_get_event_record] = ACTIONS(1913),
    [sym_format] = ACTIONS(1913),
    [sym_astro_clock] = ACTIONS(1913),
    [sym_set_outdoor_temperature] = ACTIONS(1913),
    [sym_set_virtual_level_count] = ACTIONS(1913),
    [sym_set_virtual_channel_count] = ACTIONS(1913),
    [sym_set_virtual_port_count] = ACTIONS(1913),
    [sym__wc_encode_int] = ACTIONS(1913),
    [sym__wc_decode_int] = ACTIONS(1913),
    [sym__wc_int] = ACTIONS(1913),
    [sym__wc_to_ch_int] = ACTIONS(1913),
    [sym__ch_to_wc_int] = ACTIONS(1913),
    [sym__wc_find_string_int] = ACTIONS(1913),
    [sym__wc_left_string_int] = ACTIONS(1913),
    [sym__wc_lower_string_int] = ACTIONS(1913),
    [sym__wc_mid_string_int] = ACTIONS(1913),
    [sym__wc_remove_string_int] = ACTIONS(1913),
    [sym__wc_right_string_int] = ACTIONS(1913),
    [sym__wc_upper_string_int] = ACTIONS(1913),
    [sym__wc_compare_string_int] = ACTIONS(1913),
    [sym__wc_get_buffer_char_int] = ACTIONS(1913),
    [sym__wc_get_buffer_string_int] = ACTIONS(1913),
    [sym__wc_concat_string_int] = ACTIONS(1913),
    [sym__wc_file_open_int] = ACTIONS(1913),
    [sym__wc_file_close_int] = ACTIONS(1913),
    [sym__wc_file_read_int] = ACTIONS(1913),
    [sym__wc_file_read_line_int] = ACTIONS(1913),
    [sym__wc_file_write_int] = ACTIONS(1913),
    [sym__wc_file_write_line_int] = ACTIONS(1913),
    [aux_sym_preproc_include_token1] = ACTIONS(1913),
    [aux_sym_preproc_define_token1] = ACTIONS(1915),
    [aux_sym_preproc_warn_token1] = ACTIONS(1915),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1915),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1915),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1915),
    [anon_sym_LBRACK] = ACTIONS(1915),
    [anon_sym_LBRACE] = ACTIONS(1915),
    [anon_sym_LPAREN] = ACTIONS(1913),
    [anon_sym_BANG] = ACTIONS(1915),
    [anon_sym_TILDE] = ACTIONS(1915),
    [anon_sym_DASH] = ACTIONS(1913),
    [anon_sym_PLUS] = ACTIONS(1913),
    [anon_sym_DASH_DASH] = ACTIONS(1915),
    [anon_sym_PLUS_PLUS] = ACTIONS(1915),
    [anon_sym_DQUOTE] = ACTIONS(1915),
    [anon_sym_SQUOTE] = ACTIONS(1915),
    [sym_number_literal] = ACTIONS(1915),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(457)] = {
    [ts_builtin_sym_end] = ACTIONS(1943),
    [sym_identifier] = ACTIONS(1941),
    [sym_program_name_keyword] = ACTIONS(1941),
    [sym_module_name_keyword] = ACTIONS(1941),
    [sym_define_device_keyword] = ACTIONS(1941),
    [sym_define_combine_keyword] = ACTIONS(1941),
    [sym_define_constant_keyword] = ACTIONS(1941),
    [sym_define_type_keyword] = ACTIONS(1941),
    [sym_define_variable_keyword] = ACTIONS(1941),
    [sym_define_system_variable_keyword] = ACTIONS(1941),
    [sym_define_connect_level_keyword] = ACTIONS(1941),
    [sym_define_latching_keyword] = ACTIONS(1941),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1941),
    [sym_define_toggling_keyword] = ACTIONS(1941),
    [sym_define_call_keyword] = ACTIONS(1941),
    [sym_define_function_keyword] = ACTIONS(1941),
    [sym_define_library_function_keyword] = ACTIONS(1941),
    [sym_define_start_keyword] = ACTIONS(1941),
    [sym_define_event_keyword] = ACTIONS(1941),
    [sym_define_program_keyword] = ACTIONS(1941),
    [sym_define_module_keyword] = ACTIONS(1941),
    [sym_char_keyword] = ACTIONS(1941),
    [sym_widechar_keyword] = ACTIONS(1941),
    [sym_integer_keyword] = ACTIONS(1941),
    [sym_sinteger_keyword] = ACTIONS(1941),
    [sym_long_keyword] = ACTIONS(1941),
    [sym_slong_keyword] = ACTIONS(1941),
    [sym_float_keyword] = ACTIONS(1941),
    [sym_double_keyword] = ACTIONS(1941),
    [sym_variant_keyword] = ACTIONS(1941),
    [sym_variantarray_keyword] = ACTIONS(1941),
    [sym_local_var_keyword] = ACTIONS(1941),
    [sym_stack_var_keyword] = ACTIONS(1941),
    [sym_constant_keyword] = ACTIONS(1941),
    [sym_volatile_keyword] = ACTIONS(1941),
    [sym_non_volatile_keyword] = ACTIONS(1941),
    [sym_persistent_keyword] = ACTIONS(1941),
    [sym_dev_keyword] = ACTIONS(1941),
    [sym_devlev_keyword] = ACTIONS(1941),
    [sym_devchan_keyword] = ACTIONS(1941),
    [sym_if_keyword] = ACTIONS(1941),
    [sym_switch_keyword] = ACTIONS(1941),
    [sym_case_keyword] = ACTIONS(1941),
    [sym_default_keyword] = ACTIONS(1941),
    [sym_while_keyword] = ACTIONS(1941),
    [sym_for_keyword] = ACTIONS(1941),
    [sym_break_keyword] = ACTIONS(1941),
    [sym_continue_keyword] = ACTIONS(1941),
    [sym_return_keyword] = ACTIONS(1941),
    [sym_select_keyword] = ACTIONS(1941),
    [sym_active_keyword] = ACTIONS(1941),
    [sym_struct_keyword] = ACTIONS(1941),
    [sym_structure_keyword] = ACTIONS(1941),
    [sym_button_event_keyword] = ACTIONS(1941),
    [sym_channel_event_keyword] = ACTIONS(1941),
    [sym_level_event_keyword] = ACTIONS(1941),
    [sym_data_event_keyword] = ACTIONS(1941),
    [sym_timeline_event_keyword] = ACTIONS(1941),
    [sym_custom_event_keyword] = ACTIONS(1941),
    [sym_push_keyword] = ACTIONS(1941),
    [sym_release_keyword] = ACTIONS(1941),
    [aux_sym_on_keyword_token1] = ACTIONS(1941),
    [aux_sym_off_keyword_token1] = ACTIONS(1941),
    [sym_send_level_keyword] = ACTIONS(1941),
    [sym_send_string_keyword] = ACTIONS(1941),
    [sym_send_command_keyword] = ACTIONS(1941),
    [sym_clear_buffer_keyword] = ACTIONS(1941),
    [sym_create_buffer_keyword] = ACTIONS(1941),
    [sym_create_multi_buffer_keyword] = ACTIONS(1941),
    [sym_call_keyword] = ACTIONS(1941),
    [sym_system_call_keyword] = ACTIONS(1941),
    [sym_devchan_to_keyword] = ACTIONS(1941),
    [sym_devchan_min_to_keyword] = ACTIONS(1941),
    [sym_devchan_total_off_keyword] = ACTIONS(1941),
    [sym_devchan_pulse_keyword] = ACTIONS(1941),
    [sym_wait_keyword] = ACTIONS(1941),
    [sym_cancel_wait_keyword] = ACTIONS(1941),
    [sym_wait_until_keyword] = ACTIONS(1941),
    [sym_cancel_wait_until_keyword] = ACTIONS(1941),
    [sym_cancel_all_wait_keyword] = ACTIONS(1941),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1941),
    [sym_not] = ACTIONS(1941),
    [sym_bnot] = ACTIONS(1941),
    [sym___file__] = ACTIONS(1941),
    [sym___date__] = ACTIONS(1941),
    [sym___ldate__] = ACTIONS(1941),
    [sym___time__] = ACTIONS(1941),
    [sym___line__] = ACTIONS(1941),
    [sym___name__] = ACTIONS(1941),
    [sym___netlinx__] = ACTIONS(1941),
    [sym_netlinx_axi_version] = ACTIONS(1941),
    [sym_url_flg_tcp] = ACTIONS(1941),
    [sym_url_flg_accinfopresent] = ACTIONS(1941),
    [sym_url_flg_temp] = ACTIONS(1941),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1941),
    [sym_url_flg_stat_mask] = ACTIONS(1941),
    [sym_url_flg_stat_lookup] = ACTIONS(1941),
    [sym_url_flg_stat_connecting] = ACTIONS(1941),
    [sym_url_flg_stat_waiting] = ACTIONS(1941),
    [sym_url_flg_stat_connected] = ACTIONS(1941),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1941),
    [sym_first_virtual_device] = ACTIONS(1941),
    [sym_first_local_port] = ACTIONS(1941),
    [sym_dynamic_virtual_device] = ACTIONS(1941),
    [sym_timeline_once] = ACTIONS(1941),
    [sym_timeline_repeat] = ACTIONS(1941),
    [sym_timeline_absolute] = ACTIONS(1941),
    [sym_timeline_relative] = ACTIONS(1941),
    [sym_do_push_timed_infinite] = ACTIONS(1941),
    [sym_source_type_no_address] = ACTIONS(1941),
    [sym_source_type_neuron_id] = ACTIONS(1941),
    [sym_source_type_ip_address] = ACTIONS(1941),
    [sym_source_type_axlink] = ACTIONS(1941),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1941),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1941),
    [sym_source_type_ip_socket_address] = ACTIONS(1941),
    [sym_source_type_rs232] = ACTIONS(1941),
    [sym_source_type_internal] = ACTIONS(1941),
    [sym_source_type_mac_address] = ACTIONS(1941),
    [sym_source_type_ipv4_port] = ACTIONS(1941),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1941),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1941),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1941),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1941),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1941),
    [sym_true] = ACTIONS(1941),
    [sym_false] = ACTIONS(1941),
    [sym_file_read_only] = ACTIONS(1941),
    [sym_file_rw_new] = ACTIONS(1941),
    [sym_file_rw_append] = ACTIONS(1941),
    [sym_ip_tcp] = ACTIONS(1941),
    [sym_ip_udp] = ACTIONS(1941),
    [sym_ip_udp_2way] = ACTIONS(1941),
    [sym_xml_encode_types] = ACTIONS(1941),
    [sym_xml_encode_char_as_list] = ACTIONS(1941),
    [sym_xml_encode_le] = ACTIONS(1941),
    [sym_xml_decode_types] = ACTIONS(1941),
    [sym_xml_decode_no_preserve] = ACTIONS(1941),
    [sym_duet_dev_type_display_device] = ACTIONS(1941),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1941),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1941),
    [sym_duet_dev_type_io_device] = ACTIONS(1941),
    [sym_duet_dev_type_relay_device] = ACTIONS(1941),
    [sym_duet_dev_type_ups] = ACTIONS(1941),
    [sym_duet_dev_type_amplifier] = ACTIONS(1941),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1941),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1941),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1941),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1941),
    [sym_duet_dev_type_camera] = ACTIONS(1941),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1941),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1941),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1941),
    [sym_duet_dev_type_dss] = ACTIONS(1941),
    [sym_duet_dev_type_dvr] = ACTIONS(1941),
    [sym_duet_dev_type_disc_device] = ACTIONS(1941),
    [sym_duet_dev_type_document_camera] = ACTIONS(1941),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1941),
    [sym_duet_dev_type_hvac] = ACTIONS(1941),
    [sym_duet_dev_type_keypad] = ACTIONS(1941),
    [sym_duet_dev_type_light] = ACTIONS(1941),
    [sym_duet_dev_type_monitor] = ACTIONS(1941),
    [sym_duet_dev_type_motor] = ACTIONS(1941),
    [sym_duet_dev_type_multi_window] = ACTIONS(1941),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1941),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1941),
    [sym_duet_dev_type_receiver] = ACTIONS(1941),
    [sym_duet_dev_type_security_system] = ACTIONS(1941),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1941),
    [sym_duet_dev_type_settop_box] = ACTIONS(1941),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1941),
    [sym_duet_dev_type_switcher] = ACTIONS(1941),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1941),
    [sym_duet_dev_type_tv] = ACTIONS(1941),
    [sym_duet_dev_type_utility] = ACTIONS(1941),
    [sym_duet_dev_type_vcr] = ACTIONS(1941),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1941),
    [sym_duet_dev_type_video_processor] = ACTIONS(1941),
    [sym_duet_dev_type_video_projector] = ACTIONS(1941),
    [sym_duet_dev_type_video_wall] = ACTIONS(1941),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1941),
    [sym_duet_dev_type_weather] = ACTIONS(1941),
    [sym_duet_dev_not_polled] = ACTIONS(1941),
    [sym_duet_dev_polled] = ACTIONS(1941),
    [sym_clkmgr_mode_network] = ACTIONS(1941),
    [sym_clkmgr_mode_standalone] = ACTIONS(1941),
    [sym_normal_standby] = ACTIONS(1941),
    [sym_normal_wake] = ACTIONS(1941),
    [sym_ip_multicast_ttl_option] = ACTIONS(1941),
    [sym_ip_ttl_subnet] = ACTIONS(1941),
    [sym_ip_ttl_site] = ACTIONS(1941),
    [sym_ip_ttl_region] = ACTIONS(1941),
    [sym_ip_ttl_continent] = ACTIONS(1941),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1941),
    [sym_ip_nodelay_on] = ACTIONS(1941),
    [sym_ip_nodelay_off] = ACTIONS(1941),
    [sym_amx_error] = ACTIONS(1941),
    [sym_amx_warning] = ACTIONS(1941),
    [sym_amx_info] = ACTIONS(1941),
    [sym_amx_debug] = ACTIONS(1941),
    [sym_netlinx_login_success] = ACTIONS(1941),
    [sym_netlinx_login_fail] = ACTIONS(1941),
    [sym_netlinx_logout] = ACTIONS(1941),
    [sym_null_str] = ACTIONS(1941),
    [sym_smtp_address] = ACTIONS(1941),
    [sym_smtp_port_number] = ACTIONS(1941),
    [sym_smtp_username] = ACTIONS(1941),
    [sym_smtp_password] = ACTIONS(1941),
    [sym_smtp_require_tls] = ACTIONS(1941),
    [sym_smtp_from] = ACTIONS(1941),
    [sym_smtp_tls_true] = ACTIONS(1941),
    [sym_smtp_tls_false] = ACTIONS(1941),
    [sym_listview_on_row_select_event] = ACTIONS(1941),
    [sym_data_structure_unknown] = ACTIONS(1941),
    [sym_data_structure_datafeed] = ACTIONS(1941),
    [sym_data_structure_datarecordset] = ACTIONS(1941),
    [sym_data_structure_datarecord] = ACTIONS(1941),
    [sym_data_structure_datafield] = ACTIONS(1941),
    [sym_data_max_content_field_count] = ACTIONS(1941),
    [sym_data_max_metadata_field_count] = ACTIONS(1941),
    [sym_data_max_id_length] = ACTIONS(1941),
    [sym_data_max_name_length] = ACTIONS(1941),
    [sym_data_max_description_length] = ACTIONS(1941),
    [sym_data_max_type_length] = ACTIONS(1941),
    [sym_data_max_format_length] = ACTIONS(1941),
    [sym_data_max_source_length] = ACTIONS(1941),
    [sym_data_max_value_length] = ACTIONS(1941),
    [sym_data_max_label_length] = ACTIONS(1941),
    [sym_data_max_url_length] = ACTIONS(1941),
    [sym_max_last_login_info_length] = ACTIONS(1941),
    [sym_data_type_unknown] = ACTIONS(1941),
    [sym_data_type_string] = ACTIONS(1941),
    [sym_data_type_datetime] = ACTIONS(1941),
    [sym_data_type_date] = ACTIONS(1941),
    [sym_data_type_time] = ACTIONS(1941),
    [sym_data_type_image] = ACTIONS(1941),
    [sym_data_format_url] = ACTIONS(1941),
    [sym_data_format_phone] = ACTIONS(1941),
    [sym_data_format_email] = ACTIONS(1941),
    [sym_data_format_iso8601] = ACTIONS(1941),
    [sym_auditlog_priv] = ACTIONS(1941),
    [sym_configuration_priv] = ACTIONS(1941),
    [sym_deviceconfig_priv] = ACTIONS(1941),
    [sym_ftp_priv] = ACTIONS(1941),
    [sym_http_priv] = ACTIONS(1941),
    [sym_networkconfig_priv] = ACTIONS(1941),
    [sym_programport_priv] = ACTIONS(1941),
    [sym_remoteui_priv] = ACTIONS(1941),
    [sym_securitycontrol_priv] = ACTIONS(1941),
    [sym_softwaremanagement_priv] = ACTIONS(1941),
    [sym_terminal_priv] = ACTIONS(1941),
    [sym_tpadmin_priv] = ACTIONS(1941),
    [sym_user1_priv] = ACTIONS(1941),
    [sym_user2_priv] = ACTIONS(1941),
    [sym_user3_priv] = ACTIONS(1941),
    [sym_user4_priv] = ACTIONS(1941),
    [sym_usermanagement_priv] = ACTIONS(1941),
    [sym_valid_account] = ACTIONS(1941),
    [sym_err_name_invalid] = ACTIONS(1941),
    [sym_err_password_invalid] = ACTIONS(1941),
    [sym_err_invalid_account] = ACTIONS(1941),
    [sym_err_no_authorization] = ACTIONS(1941),
    [sym_err_invalid_parameter] = ACTIONS(1941),
    [sym_err_user_locked_out] = ACTIONS(1941),
    [sym_err_user_account_expired] = ACTIONS(1941),
    [sym_err_general_error] = ACTIONS(1941),
    [sym_err_authentication_error] = ACTIONS(1941),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1941),
    [sym_tls_validate_certificate] = ACTIONS(1941),
    [sym_tdata] = ACTIONS(1941),
    [sym_tchannel] = ACTIONS(1941),
    [sym_tlevel] = ACTIONS(1941),
    [sym_tbutton] = ACTIONS(1941),
    [sym_ttimeline] = ACTIONS(1941),
    [sym_tcustom] = ACTIONS(1941),
    [sym_url_struct] = ACTIONS(1941),
    [sym_dns_struct] = ACTIONS(1941),
    [sym_ip_address_struct] = ACTIONS(1941),
    [sym_dev_info_struct] = ACTIONS(1941),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1941),
    [sym_data_feed] = ACTIONS(1941),
    [sym_data_field] = ACTIONS(1941),
    [sym_data_record] = ACTIONS(1941),
    [sym_wc_data_feed] = ACTIONS(1941),
    [sym_wc_data_field] = ACTIONS(1941),
    [sym_wc_data_record] = ACTIONS(1941),
    [sym_last_login_info] = ACTIONS(1941),
    [sym____reserved___] = ACTIONS(1941),
    [sym_date] = ACTIONS(1941),
    [sym_day] = ACTIONS(1941),
    [sym_time] = ACTIONS(1941),
    [sym_ldate] = ACTIONS(1941),
    [sym_push_device] = ACTIONS(1941),
    [sym_push_channel] = ACTIONS(1941),
    [sym_push_devchan] = ACTIONS(1941),
    [sym_release_device] = ACTIONS(1941),
    [sym_release_channel] = ACTIONS(1941),
    [sym_release_devchan] = ACTIONS(1941),
    [sym_master_slot] = ACTIONS(1941),
    [sym_get_pulse_time] = ACTIONS(1941),
    [sym_get_timer] = ACTIONS(1941),
    [sym_master_sn] = ACTIONS(1941),
    [sym_system_number] = ACTIONS(1941),
    [sym_button] = ACTIONS(1941),
    [sym_dv_channel] = ACTIONS(1941),
    [sym_data] = ACTIONS(1941),
    [sym_level] = ACTIONS(1941),
    [sym_timeline] = ACTIONS(1941),
    [sym_channel] = ACTIONS(1941),
    [sym_custom] = ACTIONS(1941),
    [sym_atoi] = ACTIONS(1941),
    [sym_atol] = ACTIONS(1941),
    [sym_atof] = ACTIONS(1941),
    [sym_itoa] = ACTIONS(1941),
    [sym_itohex] = ACTIONS(1941),
    [sym_hextoi] = ACTIONS(1941),
    [sym_ftoa] = ACTIONS(1941),
    [sym_find_string] = ACTIONS(1941),
    [sym_left_string] = ACTIONS(1941),
    [sym_right_string] = ACTIONS(1941),
    [sym_mid_string] = ACTIONS(1941),
    [sym_remove_string] = ACTIONS(1941),
    [sym_get_buffer_string] = ACTIONS(1941),
    [sym_get_buffer_char] = ACTIONS(1941),
    [sym_get_multi_buffer_string] = ACTIONS(1941),
    [sym_length_string] = ACTIONS(1941),
    [sym_max_length_string] = ACTIONS(1941),
    [sym_set_length_string] = ACTIONS(1941),
    [sym_compare_string] = ACTIONS(1941),
    [sym_upper_string] = ACTIONS(1941),
    [sym_lower_string] = ACTIONS(1941),
    [sym_get_last] = ACTIONS(1941),
    [sym_date_to_day] = ACTIONS(1941),
    [sym_date_to_month] = ACTIONS(1941),
    [sym_date_to_year] = ACTIONS(1941),
    [sym_time_to_hour] = ACTIONS(1941),
    [sym_time_to_minute] = ACTIONS(1941),
    [sym_time_to_second] = ACTIONS(1941),
    [sym_day_of_week] = ACTIONS(1941),
    [sym_set_pulse_time] = ACTIONS(1941),
    [sym_set_timer] = ACTIONS(1941),
    [sym_file_open] = ACTIONS(1941),
    [sym_file_close] = ACTIONS(1941),
    [sym_file_read] = ACTIONS(1941),
    [sym_file_read_line] = ACTIONS(1941),
    [sym_file_write] = ACTIONS(1941),
    [sym_file_write_line] = ACTIONS(1941),
    [sym_file_copy] = ACTIONS(1941),
    [sym_file_delete] = ACTIONS(1941),
    [sym_file_rename] = ACTIONS(1941),
    [sym_file_dir] = ACTIONS(1941),
    [sym_file_getdir] = ACTIONS(1941),
    [sym_file_setdir] = ACTIONS(1941),
    [sym_file_create_dir] = ACTIONS(1941),
    [sym_file_remove_dir] = ACTIONS(1941),
    [sym_file_seek] = ACTIONS(1941),
    [sym_length_array] = ACTIONS(1941),
    [sym_max_length_array] = ACTIONS(1941),
    [sym_set_length_array] = ACTIONS(1941),
    [sym_ip_client_open] = ACTIONS(1941),
    [sym_ip_client_close] = ACTIONS(1941),
    [sym_ip_server_open] = ACTIONS(1941),
    [sym_ip_server_close] = ACTIONS(1941),
    [sym_ip_bound_client_open] = ACTIONS(1941),
    [sym_ip_mc_server_open] = ACTIONS(1941),
    [sym_ip_set_option] = ACTIONS(1941),
    [sym_ssh_client_open] = ACTIONS(1941),
    [sym_ssh_client_close] = ACTIONS(1941),
    [sym_tls_client_open] = ACTIONS(1941),
    [sym_tls_client_close] = ACTIONS(1941),
    [sym_do_push] = ACTIONS(1941),
    [sym_do_push_timed] = ACTIONS(1941),
    [sym_do_release] = ACTIONS(1941),
    [sym_device_id] = ACTIONS(1941),
    [sym_device_info] = ACTIONS(1941),
    [sym_device_id_string] = ACTIONS(1941),
    [sym_device_standby] = ACTIONS(1941),
    [sym_device_wake] = ACTIONS(1941),
    [sym_timeline_create] = ACTIONS(1941),
    [sym_timeline_kill] = ACTIONS(1941),
    [sym_timeline_reload] = ACTIONS(1941),
    [sym_timeline_pause] = ACTIONS(1941),
    [sym_timeline_restart] = ACTIONS(1941),
    [sym_timeline_set] = ACTIONS(1941),
    [sym_timeline_get] = ACTIONS(1941),
    [sym_timeline_active] = ACTIONS(1941),
    [sym_timeline_dynamic_id] = ACTIONS(1941),
    [sym_variable_to_string] = ACTIONS(1941),
    [sym_string_to_variable] = ACTIONS(1941),
    [sym_length_variable_to_string] = ACTIONS(1941),
    [sym_type_cast] = ACTIONS(1941),
    [sym_variable_to_xml] = ACTIONS(1941),
    [sym_xml_to_variable] = ACTIONS(1941),
    [sym_length_variable_to_xml] = ACTIONS(1941),
    [sym_redirect_string] = ACTIONS(1941),
    [sym_random_number] = ACTIONS(1941),
    [sym_get_unique_id] = ACTIONS(1941),
    [sym_get_system_number] = ACTIONS(1941),
    [sym_set_system_number] = ACTIONS(1941),
    [sym_get_serial_number] = ACTIONS(1941),
    [sym_get_master_build] = ACTIONS(1941),
    [sym_set_validation_code] = ACTIONS(1941),
    [sym_reboot] = ACTIONS(1941),
    [sym_load_duet_module] = ACTIONS(1941),
    [sym_unload_duet_module] = ACTIONS(1941),
    [sym_duet_mem_size_set] = ACTIONS(1941),
    [sym_duet_mem_size_get] = ACTIONS(1941),
    [sym_dynamic_application_device] = ACTIONS(1941),
    [sym_dynamic_polled_port] = ACTIONS(1941),
    [sym_static_port_binding] = ACTIONS(1941),
    [sym_static_ip_binding] = ACTIONS(1941),
    [sym_get_dns_list] = ACTIONS(1941),
    [sym_set_dns_list] = ACTIONS(1941),
    [sym_get_ip_address] = ACTIONS(1941),
    [sym_set_ip_address] = ACTIONS(1941),
    [sym_get_url_list] = ACTIONS(1941),
    [sym_add_url_entry] = ACTIONS(1941),
    [sym_delete_url_entry] = ACTIONS(1941),
    [sym_abs_value] = ACTIONS(1941),
    [sym_max_value] = ACTIONS(1941),
    [sym_min_value] = ACTIONS(1941),
    [sym_exp_value] = ACTIONS(1941),
    [sym_log_value] = ACTIONS(1941),
    [sym_log10_value] = ACTIONS(1941),
    [sym_power_value] = ACTIONS(1941),
    [sym_sqrt_value] = ACTIONS(1941),
    [sym_raw_be] = ACTIONS(1941),
    [sym_raw_le] = ACTIONS(1941),
    [sym_internal_queue_size_set] = ACTIONS(1941),
    [sym_internal_queue_size_get] = ACTIONS(1941),
    [sym_internal_threshold_set] = ACTIONS(1941),
    [sym_internal_threshold_get] = ACTIONS(1941),
    [sym_rebuilt_event] = ACTIONS(1941),
    [sym_do_custom_event] = ACTIONS(1941),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1941),
    [sym_clkmgr_set_clk_source] = ACTIONS(1941),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1941),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1941),
    [sym_clkmgr_get_timezone] = ACTIONS(1941),
    [sym_clkmgr_set_timezone] = ACTIONS(1941),
    [sym_clkmgr_get_resync_period] = ACTIONS(1941),
    [sym_clkmgr_set_resync_period] = ACTIONS(1941),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1941),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1941),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1941),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1941),
    [sym_clkmgr_get_timeservers] = ACTIONS(1941),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1941),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1941),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1941),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1941),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1941),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1941),
    [sym_set_log_level] = ACTIONS(1941),
    [sym_get_log_level] = ACTIONS(1941),
    [sym_amx_log] = ACTIONS(1941),
    [sym_smtp_server_config_set] = ACTIONS(1941),
    [sym_smtp_server_config_get] = ACTIONS(1941),
    [sym_smtp_send] = ACTIONS(1941),
    [sym_validate_netlinx_account] = ACTIONS(1941),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1941),
    [sym_authenticate_certificate] = ACTIONS(1941),
    [sym_audit_netlinx_session_event] = ACTIONS(1941),
    [sym_audit_netlinx_generic_event] = ACTIONS(1941),
    [sym_get_available_flash_disk_space] = ACTIONS(1941),
    [sym_get_max_flash_disk_space] = ACTIONS(1941),
    [sym_data_create_feed] = ACTIONS(1941),
    [sym_data_delete_feed] = ACTIONS(1941),
    [sym_data_publish_feed] = ACTIONS(1941),
    [sym_data_get_published_feed] = ACTIONS(1941),
    [sym_data_add_record] = ACTIONS(1941),
    [sym_data_get_event_record] = ACTIONS(1941),
    [sym__wc_data_create_feed] = ACTIONS(1941),
    [sym__wc_data_add_record] = ACTIONS(1941),
    [sym__wc_data_get_event_record] = ACTIONS(1941),
    [sym_format] = ACTIONS(1941),
    [sym_astro_clock] = ACTIONS(1941),
    [sym_set_outdoor_temperature] = ACTIONS(1941),
    [sym_set_virtual_level_count] = ACTIONS(1941),
    [sym_set_virtual_channel_count] = ACTIONS(1941),
    [sym_set_virtual_port_count] = ACTIONS(1941),
    [sym__wc_encode_int] = ACTIONS(1941),
    [sym__wc_decode_int] = ACTIONS(1941),
    [sym__wc_int] = ACTIONS(1941),
    [sym__wc_to_ch_int] = ACTIONS(1941),
    [sym__ch_to_wc_int] = ACTIONS(1941),
    [sym__wc_find_string_int] = ACTIONS(1941),
    [sym__wc_left_string_int] = ACTIONS(1941),
    [sym__wc_lower_string_int] = ACTIONS(1941),
    [sym__wc_mid_string_int] = ACTIONS(1941),
    [sym__wc_remove_string_int] = ACTIONS(1941),
    [sym__wc_right_string_int] = ACTIONS(1941),
    [sym__wc_upper_string_int] = ACTIONS(1941),
    [sym__wc_compare_string_int] = ACTIONS(1941),
    [sym__wc_get_buffer_char_int] = ACTIONS(1941),
    [sym__wc_get_buffer_string_int] = ACTIONS(1941),
    [sym__wc_concat_string_int] = ACTIONS(1941),
    [sym__wc_file_open_int] = ACTIONS(1941),
    [sym__wc_file_close_int] = ACTIONS(1941),
    [sym__wc_file_read_int] = ACTIONS(1941),
    [sym__wc_file_read_line_int] = ACTIONS(1941),
    [sym__wc_file_write_int] = ACTIONS(1941),
    [sym__wc_file_write_line_int] = ACTIONS(1941),
    [aux_sym_preproc_include_token1] = ACTIONS(1941),
    [aux_sym_preproc_define_token1] = ACTIONS(1943),
    [aux_sym_preproc_warn_token1] = ACTIONS(1943),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1943),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1943),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1943),
    [anon_sym_LBRACK] = ACTIONS(1943),
    [anon_sym_LBRACE] = ACTIONS(1943),
    [anon_sym_LPAREN] = ACTIONS(1941),
    [anon_sym_BANG] = ACTIONS(1943),
    [anon_sym_TILDE] = ACTIONS(1943),
    [anon_sym_DASH] = ACTIONS(1941),
    [anon_sym_PLUS] = ACTIONS(1941),
    [anon_sym_DASH_DASH] = ACTIONS(1943),
    [anon_sym_PLUS_PLUS] = ACTIONS(1943),
    [anon_sym_DQUOTE] = ACTIONS(1943),
    [anon_sym_SQUOTE] = ACTIONS(1943),
    [sym_number_literal] = ACTIONS(1943),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(458)] = {
    [ts_builtin_sym_end] = ACTIONS(1939),
    [sym_identifier] = ACTIONS(1937),
    [sym_program_name_keyword] = ACTIONS(1937),
    [sym_module_name_keyword] = ACTIONS(1937),
    [sym_define_device_keyword] = ACTIONS(1937),
    [sym_define_combine_keyword] = ACTIONS(1937),
    [sym_define_constant_keyword] = ACTIONS(1937),
    [sym_define_type_keyword] = ACTIONS(1937),
    [sym_define_variable_keyword] = ACTIONS(1937),
    [sym_define_system_variable_keyword] = ACTIONS(1937),
    [sym_define_connect_level_keyword] = ACTIONS(1937),
    [sym_define_latching_keyword] = ACTIONS(1937),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1937),
    [sym_define_toggling_keyword] = ACTIONS(1937),
    [sym_define_call_keyword] = ACTIONS(1937),
    [sym_define_function_keyword] = ACTIONS(1937),
    [sym_define_library_function_keyword] = ACTIONS(1937),
    [sym_define_start_keyword] = ACTIONS(1937),
    [sym_define_event_keyword] = ACTIONS(1937),
    [sym_define_program_keyword] = ACTIONS(1937),
    [sym_define_module_keyword] = ACTIONS(1937),
    [sym_char_keyword] = ACTIONS(1937),
    [sym_widechar_keyword] = ACTIONS(1937),
    [sym_integer_keyword] = ACTIONS(1937),
    [sym_sinteger_keyword] = ACTIONS(1937),
    [sym_long_keyword] = ACTIONS(1937),
    [sym_slong_keyword] = ACTIONS(1937),
    [sym_float_keyword] = ACTIONS(1937),
    [sym_double_keyword] = ACTIONS(1937),
    [sym_variant_keyword] = ACTIONS(1937),
    [sym_variantarray_keyword] = ACTIONS(1937),
    [sym_local_var_keyword] = ACTIONS(1937),
    [sym_stack_var_keyword] = ACTIONS(1937),
    [sym_constant_keyword] = ACTIONS(1937),
    [sym_volatile_keyword] = ACTIONS(1937),
    [sym_non_volatile_keyword] = ACTIONS(1937),
    [sym_persistent_keyword] = ACTIONS(1937),
    [sym_dev_keyword] = ACTIONS(1937),
    [sym_devlev_keyword] = ACTIONS(1937),
    [sym_devchan_keyword] = ACTIONS(1937),
    [sym_if_keyword] = ACTIONS(1937),
    [sym_switch_keyword] = ACTIONS(1937),
    [sym_case_keyword] = ACTIONS(1937),
    [sym_default_keyword] = ACTIONS(1937),
    [sym_while_keyword] = ACTIONS(1937),
    [sym_for_keyword] = ACTIONS(1937),
    [sym_break_keyword] = ACTIONS(1937),
    [sym_continue_keyword] = ACTIONS(1937),
    [sym_return_keyword] = ACTIONS(1937),
    [sym_select_keyword] = ACTIONS(1937),
    [sym_active_keyword] = ACTIONS(1937),
    [sym_struct_keyword] = ACTIONS(1937),
    [sym_structure_keyword] = ACTIONS(1937),
    [sym_button_event_keyword] = ACTIONS(1937),
    [sym_channel_event_keyword] = ACTIONS(1937),
    [sym_level_event_keyword] = ACTIONS(1937),
    [sym_data_event_keyword] = ACTIONS(1937),
    [sym_timeline_event_keyword] = ACTIONS(1937),
    [sym_custom_event_keyword] = ACTIONS(1937),
    [sym_push_keyword] = ACTIONS(1937),
    [sym_release_keyword] = ACTIONS(1937),
    [aux_sym_on_keyword_token1] = ACTIONS(1937),
    [aux_sym_off_keyword_token1] = ACTIONS(1937),
    [sym_send_level_keyword] = ACTIONS(1937),
    [sym_send_string_keyword] = ACTIONS(1937),
    [sym_send_command_keyword] = ACTIONS(1937),
    [sym_clear_buffer_keyword] = ACTIONS(1937),
    [sym_create_buffer_keyword] = ACTIONS(1937),
    [sym_create_multi_buffer_keyword] = ACTIONS(1937),
    [sym_call_keyword] = ACTIONS(1937),
    [sym_system_call_keyword] = ACTIONS(1937),
    [sym_devchan_to_keyword] = ACTIONS(1937),
    [sym_devchan_min_to_keyword] = ACTIONS(1937),
    [sym_devchan_total_off_keyword] = ACTIONS(1937),
    [sym_devchan_pulse_keyword] = ACTIONS(1937),
    [sym_wait_keyword] = ACTIONS(1937),
    [sym_cancel_wait_keyword] = ACTIONS(1937),
    [sym_wait_until_keyword] = ACTIONS(1937),
    [sym_cancel_wait_until_keyword] = ACTIONS(1937),
    [sym_cancel_all_wait_keyword] = ACTIONS(1937),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1937),
    [sym_not] = ACTIONS(1937),
    [sym_bnot] = ACTIONS(1937),
    [sym___file__] = ACTIONS(1937),
    [sym___date__] = ACTIONS(1937),
    [sym___ldate__] = ACTIONS(1937),
    [sym___time__] = ACTIONS(1937),
    [sym___line__] = ACTIONS(1937),
    [sym___name__] = ACTIONS(1937),
    [sym___netlinx__] = ACTIONS(1937),
    [sym_netlinx_axi_version] = ACTIONS(1937),
    [sym_url_flg_tcp] = ACTIONS(1937),
    [sym_url_flg_accinfopresent] = ACTIONS(1937),
    [sym_url_flg_temp] = ACTIONS(1937),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1937),
    [sym_url_flg_stat_mask] = ACTIONS(1937),
    [sym_url_flg_stat_lookup] = ACTIONS(1937),
    [sym_url_flg_stat_connecting] = ACTIONS(1937),
    [sym_url_flg_stat_waiting] = ACTIONS(1937),
    [sym_url_flg_stat_connected] = ACTIONS(1937),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1937),
    [sym_first_virtual_device] = ACTIONS(1937),
    [sym_first_local_port] = ACTIONS(1937),
    [sym_dynamic_virtual_device] = ACTIONS(1937),
    [sym_timeline_once] = ACTIONS(1937),
    [sym_timeline_repeat] = ACTIONS(1937),
    [sym_timeline_absolute] = ACTIONS(1937),
    [sym_timeline_relative] = ACTIONS(1937),
    [sym_do_push_timed_infinite] = ACTIONS(1937),
    [sym_source_type_no_address] = ACTIONS(1937),
    [sym_source_type_neuron_id] = ACTIONS(1937),
    [sym_source_type_ip_address] = ACTIONS(1937),
    [sym_source_type_axlink] = ACTIONS(1937),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1937),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1937),
    [sym_source_type_ip_socket_address] = ACTIONS(1937),
    [sym_source_type_rs232] = ACTIONS(1937),
    [sym_source_type_internal] = ACTIONS(1937),
    [sym_source_type_mac_address] = ACTIONS(1937),
    [sym_source_type_ipv4_port] = ACTIONS(1937),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1937),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1937),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1937),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1937),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1937),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1937),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1937),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1937),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1937),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1937),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1937),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1937),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1937),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1937),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1937),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1937),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1937),
    [sym_true] = ACTIONS(1937),
    [sym_false] = ACTIONS(1937),
    [sym_file_read_only] = ACTIONS(1937),
    [sym_file_rw_new] = ACTIONS(1937),
    [sym_file_rw_append] = ACTIONS(1937),
    [sym_ip_tcp] = ACTIONS(1937),
    [sym_ip_udp] = ACTIONS(1937),
    [sym_ip_udp_2way] = ACTIONS(1937),
    [sym_xml_encode_types] = ACTIONS(1937),
    [sym_xml_encode_char_as_list] = ACTIONS(1937),
    [sym_xml_encode_le] = ACTIONS(1937),
    [sym_xml_decode_types] = ACTIONS(1937),
    [sym_xml_decode_no_preserve] = ACTIONS(1937),
    [sym_duet_dev_type_display_device] = ACTIONS(1937),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1937),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1937),
    [sym_duet_dev_type_io_device] = ACTIONS(1937),
    [sym_duet_dev_type_relay_device] = ACTIONS(1937),
    [sym_duet_dev_type_ups] = ACTIONS(1937),
    [sym_duet_dev_type_amplifier] = ACTIONS(1937),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1937),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1937),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1937),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1937),
    [sym_duet_dev_type_camera] = ACTIONS(1937),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1937),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1937),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1937),
    [sym_duet_dev_type_dss] = ACTIONS(1937),
    [sym_duet_dev_type_dvr] = ACTIONS(1937),
    [sym_duet_dev_type_disc_device] = ACTIONS(1937),
    [sym_duet_dev_type_document_camera] = ACTIONS(1937),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1937),
    [sym_duet_dev_type_hvac] = ACTIONS(1937),
    [sym_duet_dev_type_keypad] = ACTIONS(1937),
    [sym_duet_dev_type_light] = ACTIONS(1937),
    [sym_duet_dev_type_monitor] = ACTIONS(1937),
    [sym_duet_dev_type_motor] = ACTIONS(1937),
    [sym_duet_dev_type_multi_window] = ACTIONS(1937),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1937),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1937),
    [sym_duet_dev_type_receiver] = ACTIONS(1937),
    [sym_duet_dev_type_security_system] = ACTIONS(1937),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1937),
    [sym_duet_dev_type_settop_box] = ACTIONS(1937),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1937),
    [sym_duet_dev_type_switcher] = ACTIONS(1937),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1937),
    [sym_duet_dev_type_tv] = ACTIONS(1937),
    [sym_duet_dev_type_utility] = ACTIONS(1937),
    [sym_duet_dev_type_vcr] = ACTIONS(1937),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1937),
    [sym_duet_dev_type_video_processor] = ACTIONS(1937),
    [sym_duet_dev_type_video_projector] = ACTIONS(1937),
    [sym_duet_dev_type_video_wall] = ACTIONS(1937),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1937),
    [sym_duet_dev_type_weather] = ACTIONS(1937),
    [sym_duet_dev_not_polled] = ACTIONS(1937),
    [sym_duet_dev_polled] = ACTIONS(1937),
    [sym_clkmgr_mode_network] = ACTIONS(1937),
    [sym_clkmgr_mode_standalone] = ACTIONS(1937),
    [sym_normal_standby] = ACTIONS(1937),
    [sym_normal_wake] = ACTIONS(1937),
    [sym_ip_multicast_ttl_option] = ACTIONS(1937),
    [sym_ip_ttl_subnet] = ACTIONS(1937),
    [sym_ip_ttl_site] = ACTIONS(1937),
    [sym_ip_ttl_region] = ACTIONS(1937),
    [sym_ip_ttl_continent] = ACTIONS(1937),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1937),
    [sym_ip_nodelay_on] = ACTIONS(1937),
    [sym_ip_nodelay_off] = ACTIONS(1937),
    [sym_amx_error] = ACTIONS(1937),
    [sym_amx_warning] = ACTIONS(1937),
    [sym_amx_info] = ACTIONS(1937),
    [sym_amx_debug] = ACTIONS(1937),
    [sym_netlinx_login_success] = ACTIONS(1937),
    [sym_netlinx_login_fail] = ACTIONS(1937),
    [sym_netlinx_logout] = ACTIONS(1937),
    [sym_null_str] = ACTIONS(1937),
    [sym_smtp_address] = ACTIONS(1937),
    [sym_smtp_port_number] = ACTIONS(1937),
    [sym_smtp_username] = ACTIONS(1937),
    [sym_smtp_password] = ACTIONS(1937),
    [sym_smtp_require_tls] = ACTIONS(1937),
    [sym_smtp_from] = ACTIONS(1937),
    [sym_smtp_tls_true] = ACTIONS(1937),
    [sym_smtp_tls_false] = ACTIONS(1937),
    [sym_listview_on_row_select_event] = ACTIONS(1937),
    [sym_data_structure_unknown] = ACTIONS(1937),
    [sym_data_structure_datafeed] = ACTIONS(1937),
    [sym_data_structure_datarecordset] = ACTIONS(1937),
    [sym_data_structure_datarecord] = ACTIONS(1937),
    [sym_data_structure_datafield] = ACTIONS(1937),
    [sym_data_max_content_field_count] = ACTIONS(1937),
    [sym_data_max_metadata_field_count] = ACTIONS(1937),
    [sym_data_max_id_length] = ACTIONS(1937),
    [sym_data_max_name_length] = ACTIONS(1937),
    [sym_data_max_description_length] = ACTIONS(1937),
    [sym_data_max_type_length] = ACTIONS(1937),
    [sym_data_max_format_length] = ACTIONS(1937),
    [sym_data_max_source_length] = ACTIONS(1937),
    [sym_data_max_value_length] = ACTIONS(1937),
    [sym_data_max_label_length] = ACTIONS(1937),
    [sym_data_max_url_length] = ACTIONS(1937),
    [sym_max_last_login_info_length] = ACTIONS(1937),
    [sym_data_type_unknown] = ACTIONS(1937),
    [sym_data_type_string] = ACTIONS(1937),
    [sym_data_type_datetime] = ACTIONS(1937),
    [sym_data_type_date] = ACTIONS(1937),
    [sym_data_type_time] = ACTIONS(1937),
    [sym_data_type_image] = ACTIONS(1937),
    [sym_data_format_url] = ACTIONS(1937),
    [sym_data_format_phone] = ACTIONS(1937),
    [sym_data_format_email] = ACTIONS(1937),
    [sym_data_format_iso8601] = ACTIONS(1937),
    [sym_auditlog_priv] = ACTIONS(1937),
    [sym_configuration_priv] = ACTIONS(1937),
    [sym_deviceconfig_priv] = ACTIONS(1937),
    [sym_ftp_priv] = ACTIONS(1937),
    [sym_http_priv] = ACTIONS(1937),
    [sym_networkconfig_priv] = ACTIONS(1937),
    [sym_programport_priv] = ACTIONS(1937),
    [sym_remoteui_priv] = ACTIONS(1937),
    [sym_securitycontrol_priv] = ACTIONS(1937),
    [sym_softwaremanagement_priv] = ACTIONS(1937),
    [sym_terminal_priv] = ACTIONS(1937),
    [sym_tpadmin_priv] = ACTIONS(1937),
    [sym_user1_priv] = ACTIONS(1937),
    [sym_user2_priv] = ACTIONS(1937),
    [sym_user3_priv] = ACTIONS(1937),
    [sym_user4_priv] = ACTIONS(1937),
    [sym_usermanagement_priv] = ACTIONS(1937),
    [sym_valid_account] = ACTIONS(1937),
    [sym_err_name_invalid] = ACTIONS(1937),
    [sym_err_password_invalid] = ACTIONS(1937),
    [sym_err_invalid_account] = ACTIONS(1937),
    [sym_err_no_authorization] = ACTIONS(1937),
    [sym_err_invalid_parameter] = ACTIONS(1937),
    [sym_err_user_locked_out] = ACTIONS(1937),
    [sym_err_user_account_expired] = ACTIONS(1937),
    [sym_err_general_error] = ACTIONS(1937),
    [sym_err_authentication_error] = ACTIONS(1937),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1937),
    [sym_tls_validate_certificate] = ACTIONS(1937),
    [sym_tdata] = ACTIONS(1937),
    [sym_tchannel] = ACTIONS(1937),
    [sym_tlevel] = ACTIONS(1937),
    [sym_tbutton] = ACTIONS(1937),
    [sym_ttimeline] = ACTIONS(1937),
    [sym_tcustom] = ACTIONS(1937),
    [sym_url_struct] = ACTIONS(1937),
    [sym_dns_struct] = ACTIONS(1937),
    [sym_ip_address_struct] = ACTIONS(1937),
    [sym_dev_info_struct] = ACTIONS(1937),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1937),
    [sym_data_feed] = ACTIONS(1937),
    [sym_data_field] = ACTIONS(1937),
    [sym_data_record] = ACTIONS(1937),
    [sym_wc_data_feed] = ACTIONS(1937),
    [sym_wc_data_field] = ACTIONS(1937),
    [sym_wc_data_record] = ACTIONS(1937),
    [sym_last_login_info] = ACTIONS(1937),
    [sym____reserved___] = ACTIONS(1937),
    [sym_date] = ACTIONS(1937),
    [sym_day] = ACTIONS(1937),
    [sym_time] = ACTIONS(1937),
    [sym_ldate] = ACTIONS(1937),
    [sym_push_device] = ACTIONS(1937),
    [sym_push_channel] = ACTIONS(1937),
    [sym_push_devchan] = ACTIONS(1937),
    [sym_release_device] = ACTIONS(1937),
    [sym_release_channel] = ACTIONS(1937),
    [sym_release_devchan] = ACTIONS(1937),
    [sym_master_slot] = ACTIONS(1937),
    [sym_get_pulse_time] = ACTIONS(1937),
    [sym_get_timer] = ACTIONS(1937),
    [sym_master_sn] = ACTIONS(1937),
    [sym_system_number] = ACTIONS(1937),
    [sym_button] = ACTIONS(1937),
    [sym_dv_channel] = ACTIONS(1937),
    [sym_data] = ACTIONS(1937),
    [sym_level] = ACTIONS(1937),
    [sym_timeline] = ACTIONS(1937),
    [sym_channel] = ACTIONS(1937),
    [sym_custom] = ACTIONS(1937),
    [sym_atoi] = ACTIONS(1937),
    [sym_atol] = ACTIONS(1937),
    [sym_atof] = ACTIONS(1937),
    [sym_itoa] = ACTIONS(1937),
    [sym_itohex] = ACTIONS(1937),
    [sym_hextoi] = ACTIONS(1937),
    [sym_ftoa] = ACTIONS(1937),
    [sym_find_string] = ACTIONS(1937),
    [sym_left_string] = ACTIONS(1937),
    [sym_right_string] = ACTIONS(1937),
    [sym_mid_string] = ACTIONS(1937),
    [sym_remove_string] = ACTIONS(1937),
    [sym_get_buffer_string] = ACTIONS(1937),
    [sym_get_buffer_char] = ACTIONS(1937),
    [sym_get_multi_buffer_string] = ACTIONS(1937),
    [sym_length_string] = ACTIONS(1937),
    [sym_max_length_string] = ACTIONS(1937),
    [sym_set_length_string] = ACTIONS(1937),
    [sym_compare_string] = ACTIONS(1937),
    [sym_upper_string] = ACTIONS(1937),
    [sym_lower_string] = ACTIONS(1937),
    [sym_get_last] = ACTIONS(1937),
    [sym_date_to_day] = ACTIONS(1937),
    [sym_date_to_month] = ACTIONS(1937),
    [sym_date_to_year] = ACTIONS(1937),
    [sym_time_to_hour] = ACTIONS(1937),
    [sym_time_to_minute] = ACTIONS(1937),
    [sym_time_to_second] = ACTIONS(1937),
    [sym_day_of_week] = ACTIONS(1937),
    [sym_set_pulse_time] = ACTIONS(1937),
    [sym_set_timer] = ACTIONS(1937),
    [sym_file_open] = ACTIONS(1937),
    [sym_file_close] = ACTIONS(1937),
    [sym_file_read] = ACTIONS(1937),
    [sym_file_read_line] = ACTIONS(1937),
    [sym_file_write] = ACTIONS(1937),
    [sym_file_write_line] = ACTIONS(1937),
    [sym_file_copy] = ACTIONS(1937),
    [sym_file_delete] = ACTIONS(1937),
    [sym_file_rename] = ACTIONS(1937),
    [sym_file_dir] = ACTIONS(1937),
    [sym_file_getdir] = ACTIONS(1937),
    [sym_file_setdir] = ACTIONS(1937),
    [sym_file_create_dir] = ACTIONS(1937),
    [sym_file_remove_dir] = ACTIONS(1937),
    [sym_file_seek] = ACTIONS(1937),
    [sym_length_array] = ACTIONS(1937),
    [sym_max_length_array] = ACTIONS(1937),
    [sym_set_length_array] = ACTIONS(1937),
    [sym_ip_client_open] = ACTIONS(1937),
    [sym_ip_client_close] = ACTIONS(1937),
    [sym_ip_server_open] = ACTIONS(1937),
    [sym_ip_server_close] = ACTIONS(1937),
    [sym_ip_bound_client_open] = ACTIONS(1937),
    [sym_ip_mc_server_open] = ACTIONS(1937),
    [sym_ip_set_option] = ACTIONS(1937),
    [sym_ssh_client_open] = ACTIONS(1937),
    [sym_ssh_client_close] = ACTIONS(1937),
    [sym_tls_client_open] = ACTIONS(1937),
    [sym_tls_client_close] = ACTIONS(1937),
    [sym_do_push] = ACTIONS(1937),
    [sym_do_push_timed] = ACTIONS(1937),
    [sym_do_release] = ACTIONS(1937),
    [sym_device_id] = ACTIONS(1937),
    [sym_device_info] = ACTIONS(1937),
    [sym_device_id_string] = ACTIONS(1937),
    [sym_device_standby] = ACTIONS(1937),
    [sym_device_wake] = ACTIONS(1937),
    [sym_timeline_create] = ACTIONS(1937),
    [sym_timeline_kill] = ACTIONS(1937),
    [sym_timeline_reload] = ACTIONS(1937),
    [sym_timeline_pause] = ACTIONS(1937),
    [sym_timeline_restart] = ACTIONS(1937),
    [sym_timeline_set] = ACTIONS(1937),
    [sym_timeline_get] = ACTIONS(1937),
    [sym_timeline_active] = ACTIONS(1937),
    [sym_timeline_dynamic_id] = ACTIONS(1937),
    [sym_variable_to_string] = ACTIONS(1937),
    [sym_string_to_variable] = ACTIONS(1937),
    [sym_length_variable_to_string] = ACTIONS(1937),
    [sym_type_cast] = ACTIONS(1937),
    [sym_variable_to_xml] = ACTIONS(1937),
    [sym_xml_to_variable] = ACTIONS(1937),
    [sym_length_variable_to_xml] = ACTIONS(1937),
    [sym_redirect_string] = ACTIONS(1937),
    [sym_random_number] = ACTIONS(1937),
    [sym_get_unique_id] = ACTIONS(1937),
    [sym_get_system_number] = ACTIONS(1937),
    [sym_set_system_number] = ACTIONS(1937),
    [sym_get_serial_number] = ACTIONS(1937),
    [sym_get_master_build] = ACTIONS(1937),
    [sym_set_validation_code] = ACTIONS(1937),
    [sym_reboot] = ACTIONS(1937),
    [sym_load_duet_module] = ACTIONS(1937),
    [sym_unload_duet_module] = ACTIONS(1937),
    [sym_duet_mem_size_set] = ACTIONS(1937),
    [sym_duet_mem_size_get] = ACTIONS(1937),
    [sym_dynamic_application_device] = ACTIONS(1937),
    [sym_dynamic_polled_port] = ACTIONS(1937),
    [sym_static_port_binding] = ACTIONS(1937),
    [sym_static_ip_binding] = ACTIONS(1937),
    [sym_get_dns_list] = ACTIONS(1937),
    [sym_set_dns_list] = ACTIONS(1937),
    [sym_get_ip_address] = ACTIONS(1937),
    [sym_set_ip_address] = ACTIONS(1937),
    [sym_get_url_list] = ACTIONS(1937),
    [sym_add_url_entry] = ACTIONS(1937),
    [sym_delete_url_entry] = ACTIONS(1937),
    [sym_abs_value] = ACTIONS(1937),
    [sym_max_value] = ACTIONS(1937),
    [sym_min_value] = ACTIONS(1937),
    [sym_exp_value] = ACTIONS(1937),
    [sym_log_value] = ACTIONS(1937),
    [sym_log10_value] = ACTIONS(1937),
    [sym_power_value] = ACTIONS(1937),
    [sym_sqrt_value] = ACTIONS(1937),
    [sym_raw_be] = ACTIONS(1937),
    [sym_raw_le] = ACTIONS(1937),
    [sym_internal_queue_size_set] = ACTIONS(1937),
    [sym_internal_queue_size_get] = ACTIONS(1937),
    [sym_internal_threshold_set] = ACTIONS(1937),
    [sym_internal_threshold_get] = ACTIONS(1937),
    [sym_rebuilt_event] = ACTIONS(1937),
    [sym_do_custom_event] = ACTIONS(1937),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1937),
    [sym_clkmgr_set_clk_source] = ACTIONS(1937),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1937),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1937),
    [sym_clkmgr_get_timezone] = ACTIONS(1937),
    [sym_clkmgr_set_timezone] = ACTIONS(1937),
    [sym_clkmgr_get_resync_period] = ACTIONS(1937),
    [sym_clkmgr_set_resync_period] = ACTIONS(1937),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1937),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1937),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1937),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1937),
    [sym_clkmgr_get_timeservers] = ACTIONS(1937),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1937),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1937),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1937),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1937),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1937),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1937),
    [sym_set_log_level] = ACTIONS(1937),
    [sym_get_log_level] = ACTIONS(1937),
    [sym_amx_log] = ACTIONS(1937),
    [sym_smtp_server_config_set] = ACTIONS(1937),
    [sym_smtp_server_config_get] = ACTIONS(1937),
    [sym_smtp_send] = ACTIONS(1937),
    [sym_validate_netlinx_account] = ACTIONS(1937),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1937),
    [sym_authenticate_certificate] = ACTIONS(1937),
    [sym_audit_netlinx_session_event] = ACTIONS(1937),
    [sym_audit_netlinx_generic_event] = ACTIONS(1937),
    [sym_get_available_flash_disk_space] = ACTIONS(1937),
    [sym_get_max_flash_disk_space] = ACTIONS(1937),
    [sym_data_create_feed] = ACTIONS(1937),
    [sym_data_delete_feed] = ACTIONS(1937),
    [sym_data_publish_feed] = ACTIONS(1937),
    [sym_data_get_published_feed] = ACTIONS(1937),
    [sym_data_add_record] = ACTIONS(1937),
    [sym_data_get_event_record] = ACTIONS(1937),
    [sym__wc_data_create_feed] = ACTIONS(1937),
    [sym__wc_data_add_record] = ACTIONS(1937),
    [sym__wc_data_get_event_record] = ACTIONS(1937),
    [sym_format] = ACTIONS(1937),
    [sym_astro_clock] = ACTIONS(1937),
    [sym_set_outdoor_temperature] = ACTIONS(1937),
    [sym_set_virtual_level_count] = ACTIONS(1937),
    [sym_set_virtual_channel_count] = ACTIONS(1937),
    [sym_set_virtual_port_count] = ACTIONS(1937),
    [sym__wc_encode_int] = ACTIONS(1937),
    [sym__wc_decode_int] = ACTIONS(1937),
    [sym__wc_int] = ACTIONS(1937),
    [sym__wc_to_ch_int] = ACTIONS(1937),
    [sym__ch_to_wc_int] = ACTIONS(1937),
    [sym__wc_find_string_int] = ACTIONS(1937),
    [sym__wc_left_string_int] = ACTIONS(1937),
    [sym__wc_lower_string_int] = ACTIONS(1937),
    [sym__wc_mid_string_int] = ACTIONS(1937),
    [sym__wc_remove_string_int] = ACTIONS(1937),
    [sym__wc_right_string_int] = ACTIONS(1937),
    [sym__wc_upper_string_int] = ACTIONS(1937),
    [sym__wc_compare_string_int] = ACTIONS(1937),
    [sym__wc_get_buffer_char_int] = ACTIONS(1937),
    [sym__wc_get_buffer_string_int] = ACTIONS(1937),
    [sym__wc_concat_string_int] = ACTIONS(1937),
    [sym__wc_file_open_int] = ACTIONS(1937),
    [sym__wc_file_close_int] = ACTIONS(1937),
    [sym__wc_file_read_int] = ACTIONS(1937),
    [sym__wc_file_read_line_int] = ACTIONS(1937),
    [sym__wc_file_write_int] = ACTIONS(1937),
    [sym__wc_file_write_line_int] = ACTIONS(1937),
    [aux_sym_preproc_include_token1] = ACTIONS(1937),
    [aux_sym_preproc_define_token1] = ACTIONS(1939),
    [aux_sym_preproc_warn_token1] = ACTIONS(1939),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1939),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1939),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1939),
    [anon_sym_LBRACK] = ACTIONS(1939),
    [anon_sym_LBRACE] = ACTIONS(1939),
    [anon_sym_LPAREN] = ACTIONS(1937),
    [anon_sym_BANG] = ACTIONS(1939),
    [anon_sym_TILDE] = ACTIONS(1939),
    [anon_sym_DASH] = ACTIONS(1937),
    [anon_sym_PLUS] = ACTIONS(1937),
    [anon_sym_DASH_DASH] = ACTIONS(1939),
    [anon_sym_PLUS_PLUS] = ACTIONS(1939),
    [anon_sym_DQUOTE] = ACTIONS(1939),
    [anon_sym_SQUOTE] = ACTIONS(1939),
    [sym_number_literal] = ACTIONS(1939),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(459)] = {
    [ts_builtin_sym_end] = ACTIONS(1971),
    [sym_identifier] = ACTIONS(1969),
    [sym_program_name_keyword] = ACTIONS(1969),
    [sym_module_name_keyword] = ACTIONS(1969),
    [sym_define_device_keyword] = ACTIONS(1969),
    [sym_define_combine_keyword] = ACTIONS(1969),
    [sym_define_constant_keyword] = ACTIONS(1969),
    [sym_define_type_keyword] = ACTIONS(1969),
    [sym_define_variable_keyword] = ACTIONS(1969),
    [sym_define_system_variable_keyword] = ACTIONS(1969),
    [sym_define_connect_level_keyword] = ACTIONS(1969),
    [sym_define_latching_keyword] = ACTIONS(1969),
    [sym_define_mutually_exclusive_keyword] = ACTIONS(1969),
    [sym_define_toggling_keyword] = ACTIONS(1969),
    [sym_define_call_keyword] = ACTIONS(1969),
    [sym_define_function_keyword] = ACTIONS(1969),
    [sym_define_library_function_keyword] = ACTIONS(1969),
    [sym_define_start_keyword] = ACTIONS(1969),
    [sym_define_event_keyword] = ACTIONS(1969),
    [sym_define_program_keyword] = ACTIONS(1969),
    [sym_define_module_keyword] = ACTIONS(1969),
    [sym_char_keyword] = ACTIONS(1969),
    [sym_widechar_keyword] = ACTIONS(1969),
    [sym_integer_keyword] = ACTIONS(1969),
    [sym_sinteger_keyword] = ACTIONS(1969),
    [sym_long_keyword] = ACTIONS(1969),
    [sym_slong_keyword] = ACTIONS(1969),
    [sym_float_keyword] = ACTIONS(1969),
    [sym_double_keyword] = ACTIONS(1969),
    [sym_variant_keyword] = ACTIONS(1969),
    [sym_variantarray_keyword] = ACTIONS(1969),
    [sym_local_var_keyword] = ACTIONS(1969),
    [sym_stack_var_keyword] = ACTIONS(1969),
    [sym_constant_keyword] = ACTIONS(1969),
    [sym_volatile_keyword] = ACTIONS(1969),
    [sym_non_volatile_keyword] = ACTIONS(1969),
    [sym_persistent_keyword] = ACTIONS(1969),
    [sym_dev_keyword] = ACTIONS(1969),
    [sym_devlev_keyword] = ACTIONS(1969),
    [sym_devchan_keyword] = ACTIONS(1969),
    [sym_if_keyword] = ACTIONS(1969),
    [sym_switch_keyword] = ACTIONS(1969),
    [sym_case_keyword] = ACTIONS(1969),
    [sym_default_keyword] = ACTIONS(1969),
    [sym_while_keyword] = ACTIONS(1969),
    [sym_for_keyword] = ACTIONS(1969),
    [sym_break_keyword] = ACTIONS(1969),
    [sym_continue_keyword] = ACTIONS(1969),
    [sym_return_keyword] = ACTIONS(1969),
    [sym_select_keyword] = ACTIONS(1969),
    [sym_active_keyword] = ACTIONS(1969),
    [sym_struct_keyword] = ACTIONS(1969),
    [sym_structure_keyword] = ACTIONS(1969),
    [sym_button_event_keyword] = ACTIONS(1969),
    [sym_channel_event_keyword] = ACTIONS(1969),
    [sym_level_event_keyword] = ACTIONS(1969),
    [sym_data_event_keyword] = ACTIONS(1969),
    [sym_timeline_event_keyword] = ACTIONS(1969),
    [sym_custom_event_keyword] = ACTIONS(1969),
    [sym_push_keyword] = ACTIONS(1969),
    [sym_release_keyword] = ACTIONS(1969),
    [aux_sym_on_keyword_token1] = ACTIONS(1969),
    [aux_sym_off_keyword_token1] = ACTIONS(1969),
    [sym_send_level_keyword] = ACTIONS(1969),
    [sym_send_string_keyword] = ACTIONS(1969),
    [sym_send_command_keyword] = ACTIONS(1969),
    [sym_clear_buffer_keyword] = ACTIONS(1969),
    [sym_create_buffer_keyword] = ACTIONS(1969),
    [sym_create_multi_buffer_keyword] = ACTIONS(1969),
    [sym_call_keyword] = ACTIONS(1969),
    [sym_system_call_keyword] = ACTIONS(1969),
    [sym_devchan_to_keyword] = ACTIONS(1969),
    [sym_devchan_min_to_keyword] = ACTIONS(1969),
    [sym_devchan_total_off_keyword] = ACTIONS(1969),
    [sym_devchan_pulse_keyword] = ACTIONS(1969),
    [sym_wait_keyword] = ACTIONS(1969),
    [sym_cancel_wait_keyword] = ACTIONS(1969),
    [sym_wait_until_keyword] = ACTIONS(1969),
    [sym_cancel_wait_until_keyword] = ACTIONS(1969),
    [sym_cancel_all_wait_keyword] = ACTIONS(1969),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1969),
    [sym_not] = ACTIONS(1969),
    [sym_bnot] = ACTIONS(1969),
    [sym___file__] = ACTIONS(1969),
    [sym___date__] = ACTIONS(1969),
    [sym___ldate__] = ACTIONS(1969),
    [sym___time__] = ACTIONS(1969),
    [sym___line__] = ACTIONS(1969),
    [sym___name__] = ACTIONS(1969),
    [sym___netlinx__] = ACTIONS(1969),
    [sym_netlinx_axi_version] = ACTIONS(1969),
    [sym_url_flg_tcp] = ACTIONS(1969),
    [sym_url_flg_accinfopresent] = ACTIONS(1969),
    [sym_url_flg_temp] = ACTIONS(1969),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1969),
    [sym_url_flg_stat_mask] = ACTIONS(1969),
    [sym_url_flg_stat_lookup] = ACTIONS(1969),
    [sym_url_flg_stat_connecting] = ACTIONS(1969),
    [sym_url_flg_stat_waiting] = ACTIONS(1969),
    [sym_url_flg_stat_connected] = ACTIONS(1969),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1969),
    [sym_first_virtual_device] = ACTIONS(1969),
    [sym_first_local_port] = ACTIONS(1969),
    [sym_dynamic_virtual_device] = ACTIONS(1969),
    [sym_timeline_once] = ACTIONS(1969),
    [sym_timeline_repeat] = ACTIONS(1969),
    [sym_timeline_absolute] = ACTIONS(1969),
    [sym_timeline_relative] = ACTIONS(1969),
    [sym_do_push_timed_infinite] = ACTIONS(1969),
    [sym_source_type_no_address] = ACTIONS(1969),
    [sym_source_type_neuron_id] = ACTIONS(1969),
    [sym_source_type_ip_address] = ACTIONS(1969),
    [sym_source_type_axlink] = ACTIONS(1969),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1969),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1969),
    [sym_source_type_ip_socket_address] = ACTIONS(1969),
    [sym_source_type_rs232] = ACTIONS(1969),
    [sym_source_type_internal] = ACTIONS(1969),
    [sym_source_type_mac_address] = ACTIONS(1969),
    [sym_source_type_ipv4_port] = ACTIONS(1969),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1969),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1969),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1969),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1969),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1969),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1969),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1969),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1969),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1969),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1969),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1969),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1969),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1969),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1969),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1969),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1969),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1969),
    [sym_true] = ACTIONS(1969),
    [sym_false] = ACTIONS(1969),
    [sym_file_read_only] = ACTIONS(1969),
    [sym_file_rw_new] = ACTIONS(1969),
    [sym_file_rw_append] = ACTIONS(1969),
    [sym_ip_tcp] = ACTIONS(1969),
    [sym_ip_udp] = ACTIONS(1969),
    [sym_ip_udp_2way] = ACTIONS(1969),
    [sym_xml_encode_types] = ACTIONS(1969),
    [sym_xml_encode_char_as_list] = ACTIONS(1969),
    [sym_xml_encode_le] = ACTIONS(1969),
    [sym_xml_decode_types] = ACTIONS(1969),
    [sym_xml_decode_no_preserve] = ACTIONS(1969),
    [sym_duet_dev_type_display_device] = ACTIONS(1969),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1969),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1969),
    [sym_duet_dev_type_io_device] = ACTIONS(1969),
    [sym_duet_dev_type_relay_device] = ACTIONS(1969),
    [sym_duet_dev_type_ups] = ACTIONS(1969),
    [sym_duet_dev_type_amplifier] = ACTIONS(1969),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1969),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1969),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1969),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1969),
    [sym_duet_dev_type_camera] = ACTIONS(1969),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1969),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1969),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1969),
    [sym_duet_dev_type_dss] = ACTIONS(1969),
    [sym_duet_dev_type_dvr] = ACTIONS(1969),
    [sym_duet_dev_type_disc_device] = ACTIONS(1969),
    [sym_duet_dev_type_document_camera] = ACTIONS(1969),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1969),
    [sym_duet_dev_type_hvac] = ACTIONS(1969),
    [sym_duet_dev_type_keypad] = ACTIONS(1969),
    [sym_duet_dev_type_light] = ACTIONS(1969),
    [sym_duet_dev_type_monitor] = ACTIONS(1969),
    [sym_duet_dev_type_motor] = ACTIONS(1969),
    [sym_duet_dev_type_multi_window] = ACTIONS(1969),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1969),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1969),
    [sym_duet_dev_type_receiver] = ACTIONS(1969),
    [sym_duet_dev_type_security_system] = ACTIONS(1969),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1969),
    [sym_duet_dev_type_settop_box] = ACTIONS(1969),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1969),
    [sym_duet_dev_type_switcher] = ACTIONS(1969),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1969),
    [sym_duet_dev_type_tv] = ACTIONS(1969),
    [sym_duet_dev_type_utility] = ACTIONS(1969),
    [sym_duet_dev_type_vcr] = ACTIONS(1969),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1969),
    [sym_duet_dev_type_video_processor] = ACTIONS(1969),
    [sym_duet_dev_type_video_projector] = ACTIONS(1969),
    [sym_duet_dev_type_video_wall] = ACTIONS(1969),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1969),
    [sym_duet_dev_type_weather] = ACTIONS(1969),
    [sym_duet_dev_not_polled] = ACTIONS(1969),
    [sym_duet_dev_polled] = ACTIONS(1969),
    [sym_clkmgr_mode_network] = ACTIONS(1969),
    [sym_clkmgr_mode_standalone] = ACTIONS(1969),
    [sym_normal_standby] = ACTIONS(1969),
    [sym_normal_wake] = ACTIONS(1969),
    [sym_ip_multicast_ttl_option] = ACTIONS(1969),
    [sym_ip_ttl_subnet] = ACTIONS(1969),
    [sym_ip_ttl_site] = ACTIONS(1969),
    [sym_ip_ttl_region] = ACTIONS(1969),
    [sym_ip_ttl_continent] = ACTIONS(1969),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1969),
    [sym_ip_nodelay_on] = ACTIONS(1969),
    [sym_ip_nodelay_off] = ACTIONS(1969),
    [sym_amx_error] = ACTIONS(1969),
    [sym_amx_warning] = ACTIONS(1969),
    [sym_amx_info] = ACTIONS(1969),
    [sym_amx_debug] = ACTIONS(1969),
    [sym_netlinx_login_success] = ACTIONS(1969),
    [sym_netlinx_login_fail] = ACTIONS(1969),
    [sym_netlinx_logout] = ACTIONS(1969),
    [sym_null_str] = ACTIONS(1969),
    [sym_smtp_address] = ACTIONS(1969),
    [sym_smtp_port_number] = ACTIONS(1969),
    [sym_smtp_username] = ACTIONS(1969),
    [sym_smtp_password] = ACTIONS(1969),
    [sym_smtp_require_tls] = ACTIONS(1969),
    [sym_smtp_from] = ACTIONS(1969),
    [sym_smtp_tls_true] = ACTIONS(1969),
    [sym_smtp_tls_false] = ACTIONS(1969),
    [sym_listview_on_row_select_event] = ACTIONS(1969),
    [sym_data_structure_unknown] = ACTIONS(1969),
    [sym_data_structure_datafeed] = ACTIONS(1969),
    [sym_data_structure_datarecordset] = ACTIONS(1969),
    [sym_data_structure_datarecord] = ACTIONS(1969),
    [sym_data_structure_datafield] = ACTIONS(1969),
    [sym_data_max_content_field_count] = ACTIONS(1969),
    [sym_data_max_metadata_field_count] = ACTIONS(1969),
    [sym_data_max_id_length] = ACTIONS(1969),
    [sym_data_max_name_length] = ACTIONS(1969),
    [sym_data_max_description_length] = ACTIONS(1969),
    [sym_data_max_type_length] = ACTIONS(1969),
    [sym_data_max_format_length] = ACTIONS(1969),
    [sym_data_max_source_length] = ACTIONS(1969),
    [sym_data_max_value_length] = ACTIONS(1969),
    [sym_data_max_label_length] = ACTIONS(1969),
    [sym_data_max_url_length] = ACTIONS(1969),
    [sym_max_last_login_info_length] = ACTIONS(1969),
    [sym_data_type_unknown] = ACTIONS(1969),
    [sym_data_type_string] = ACTIONS(1969),
    [sym_data_type_datetime] = ACTIONS(1969),
    [sym_data_type_date] = ACTIONS(1969),
    [sym_data_type_time] = ACTIONS(1969),
    [sym_data_type_image] = ACTIONS(1969),
    [sym_data_format_url] = ACTIONS(1969),
    [sym_data_format_phone] = ACTIONS(1969),
    [sym_data_format_email] = ACTIONS(1969),
    [sym_data_format_iso8601] = ACTIONS(1969),
    [sym_auditlog_priv] = ACTIONS(1969),
    [sym_configuration_priv] = ACTIONS(1969),
    [sym_deviceconfig_priv] = ACTIONS(1969),
    [sym_ftp_priv] = ACTIONS(1969),
    [sym_http_priv] = ACTIONS(1969),
    [sym_networkconfig_priv] = ACTIONS(1969),
    [sym_programport_priv] = ACTIONS(1969),
    [sym_remoteui_priv] = ACTIONS(1969),
    [sym_securitycontrol_priv] = ACTIONS(1969),
    [sym_softwaremanagement_priv] = ACTIONS(1969),
    [sym_terminal_priv] = ACTIONS(1969),
    [sym_tpadmin_priv] = ACTIONS(1969),
    [sym_user1_priv] = ACTIONS(1969),
    [sym_user2_priv] = ACTIONS(1969),
    [sym_user3_priv] = ACTIONS(1969),
    [sym_user4_priv] = ACTIONS(1969),
    [sym_usermanagement_priv] = ACTIONS(1969),
    [sym_valid_account] = ACTIONS(1969),
    [sym_err_name_invalid] = ACTIONS(1969),
    [sym_err_password_invalid] = ACTIONS(1969),
    [sym_err_invalid_account] = ACTIONS(1969),
    [sym_err_no_authorization] = ACTIONS(1969),
    [sym_err_invalid_parameter] = ACTIONS(1969),
    [sym_err_user_locked_out] = ACTIONS(1969),
    [sym_err_user_account_expired] = ACTIONS(1969),
    [sym_err_general_error] = ACTIONS(1969),
    [sym_err_authentication_error] = ACTIONS(1969),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1969),
    [sym_tls_validate_certificate] = ACTIONS(1969),
    [sym_tdata] = ACTIONS(1969),
    [sym_tchannel] = ACTIONS(1969),
    [sym_tlevel] = ACTIONS(1969),
    [sym_tbutton] = ACTIONS(1969),
    [sym_ttimeline] = ACTIONS(1969),
    [sym_tcustom] = ACTIONS(1969),
    [sym_url_struct] = ACTIONS(1969),
    [sym_dns_struct] = ACTIONS(1969),
    [sym_ip_address_struct] = ACTIONS(1969),
    [sym_dev_info_struct] = ACTIONS(1969),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1969),
    [sym_data_feed] = ACTIONS(1969),
    [sym_data_field] = ACTIONS(1969),
    [sym_data_record] = ACTIONS(1969),
    [sym_wc_data_feed] = ACTIONS(1969),
    [sym_wc_data_field] = ACTIONS(1969),
    [sym_wc_data_record] = ACTIONS(1969),
    [sym_last_login_info] = ACTIONS(1969),
    [sym____reserved___] = ACTIONS(1969),
    [sym_date] = ACTIONS(1969),
    [sym_day] = ACTIONS(1969),
    [sym_time] = ACTIONS(1969),
    [sym_ldate] = ACTIONS(1969),
    [sym_push_device] = ACTIONS(1969),
    [sym_push_channel] = ACTIONS(1969),
    [sym_push_devchan] = ACTIONS(1969),
    [sym_release_device] = ACTIONS(1969),
    [sym_release_channel] = ACTIONS(1969),
    [sym_release_devchan] = ACTIONS(1969),
    [sym_master_slot] = ACTIONS(1969),
    [sym_get_pulse_time] = ACTIONS(1969),
    [sym_get_timer] = ACTIONS(1969),
    [sym_master_sn] = ACTIONS(1969),
    [sym_system_number] = ACTIONS(1969),
    [sym_button] = ACTIONS(1969),
    [sym_dv_channel] = ACTIONS(1969),
    [sym_data] = ACTIONS(1969),
    [sym_level] = ACTIONS(1969),
    [sym_timeline] = ACTIONS(1969),
    [sym_channel] = ACTIONS(1969),
    [sym_custom] = ACTIONS(1969),
    [sym_atoi] = ACTIONS(1969),
    [sym_atol] = ACTIONS(1969),
    [sym_atof] = ACTIONS(1969),
    [sym_itoa] = ACTIONS(1969),
    [sym_itohex] = ACTIONS(1969),
    [sym_hextoi] = ACTIONS(1969),
    [sym_ftoa] = ACTIONS(1969),
    [sym_find_string] = ACTIONS(1969),
    [sym_left_string] = ACTIONS(1969),
    [sym_right_string] = ACTIONS(1969),
    [sym_mid_string] = ACTIONS(1969),
    [sym_remove_string] = ACTIONS(1969),
    [sym_get_buffer_string] = ACTIONS(1969),
    [sym_get_buffer_char] = ACTIONS(1969),
    [sym_get_multi_buffer_string] = ACTIONS(1969),
    [sym_length_string] = ACTIONS(1969),
    [sym_max_length_string] = ACTIONS(1969),
    [sym_set_length_string] = ACTIONS(1969),
    [sym_compare_string] = ACTIONS(1969),
    [sym_upper_string] = ACTIONS(1969),
    [sym_lower_string] = ACTIONS(1969),
    [sym_get_last] = ACTIONS(1969),
    [sym_date_to_day] = ACTIONS(1969),
    [sym_date_to_month] = ACTIONS(1969),
    [sym_date_to_year] = ACTIONS(1969),
    [sym_time_to_hour] = ACTIONS(1969),
    [sym_time_to_minute] = ACTIONS(1969),
    [sym_time_to_second] = ACTIONS(1969),
    [sym_day_of_week] = ACTIONS(1969),
    [sym_set_pulse_time] = ACTIONS(1969),
    [sym_set_timer] = ACTIONS(1969),
    [sym_file_open] = ACTIONS(1969),
    [sym_file_close] = ACTIONS(1969),
    [sym_file_read] = ACTIONS(1969),
    [sym_file_read_line] = ACTIONS(1969),
    [sym_file_write] = ACTIONS(1969),
    [sym_file_write_line] = ACTIONS(1969),
    [sym_file_copy] = ACTIONS(1969),
    [sym_file_delete] = ACTIONS(1969),
    [sym_file_rename] = ACTIONS(1969),
    [sym_file_dir] = ACTIONS(1969),
    [sym_file_getdir] = ACTIONS(1969),
    [sym_file_setdir] = ACTIONS(1969),
    [sym_file_create_dir] = ACTIONS(1969),
    [sym_file_remove_dir] = ACTIONS(1969),
    [sym_file_seek] = ACTIONS(1969),
    [sym_length_array] = ACTIONS(1969),
    [sym_max_length_array] = ACTIONS(1969),
    [sym_set_length_array] = ACTIONS(1969),
    [sym_ip_client_open] = ACTIONS(1969),
    [sym_ip_client_close] = ACTIONS(1969),
    [sym_ip_server_open] = ACTIONS(1969),
    [sym_ip_server_close] = ACTIONS(1969),
    [sym_ip_bound_client_open] = ACTIONS(1969),
    [sym_ip_mc_server_open] = ACTIONS(1969),
    [sym_ip_set_option] = ACTIONS(1969),
    [sym_ssh_client_open] = ACTIONS(1969),
    [sym_ssh_client_close] = ACTIONS(1969),
    [sym_tls_client_open] = ACTIONS(1969),
    [sym_tls_client_close] = ACTIONS(1969),
    [sym_do_push] = ACTIONS(1969),
    [sym_do_push_timed] = ACTIONS(1969),
    [sym_do_release] = ACTIONS(1969),
    [sym_device_id] = ACTIONS(1969),
    [sym_device_info] = ACTIONS(1969),
    [sym_device_id_string] = ACTIONS(1969),
    [sym_device_standby] = ACTIONS(1969),
    [sym_device_wake] = ACTIONS(1969),
    [sym_timeline_create] = ACTIONS(1969),
    [sym_timeline_kill] = ACTIONS(1969),
    [sym_timeline_reload] = ACTIONS(1969),
    [sym_timeline_pause] = ACTIONS(1969),
    [sym_timeline_restart] = ACTIONS(1969),
    [sym_timeline_set] = ACTIONS(1969),
    [sym_timeline_get] = ACTIONS(1969),
    [sym_timeline_active] = ACTIONS(1969),
    [sym_timeline_dynamic_id] = ACTIONS(1969),
    [sym_variable_to_string] = ACTIONS(1969),
    [sym_string_to_variable] = ACTIONS(1969),
    [sym_length_variable_to_string] = ACTIONS(1969),
    [sym_type_cast] = ACTIONS(1969),
    [sym_variable_to_xml] = ACTIONS(1969),
    [sym_xml_to_variable] = ACTIONS(1969),
    [sym_length_variable_to_xml] = ACTIONS(1969),
    [sym_redirect_string] = ACTIONS(1969),
    [sym_random_number] = ACTIONS(1969),
    [sym_get_unique_id] = ACTIONS(1969),
    [sym_get_system_number] = ACTIONS(1969),
    [sym_set_system_number] = ACTIONS(1969),
    [sym_get_serial_number] = ACTIONS(1969),
    [sym_get_master_build] = ACTIONS(1969),
    [sym_set_validation_code] = ACTIONS(1969),
    [sym_reboot] = ACTIONS(1969),
    [sym_load_duet_module] = ACTIONS(1969),
    [sym_unload_duet_module] = ACTIONS(1969),
    [sym_duet_mem_size_set] = ACTIONS(1969),
    [sym_duet_mem_size_get] = ACTIONS(1969),
    [sym_dynamic_application_device] = ACTIONS(1969),
    [sym_dynamic_polled_port] = ACTIONS(1969),
    [sym_static_port_binding] = ACTIONS(1969),
    [sym_static_ip_binding] = ACTIONS(1969),
    [sym_get_dns_list] = ACTIONS(1969),
    [sym_set_dns_list] = ACTIONS(1969),
    [sym_get_ip_address] = ACTIONS(1969),
    [sym_set_ip_address] = ACTIONS(1969),
    [sym_get_url_list] = ACTIONS(1969),
    [sym_add_url_entry] = ACTIONS(1969),
    [sym_delete_url_entry] = ACTIONS(1969),
    [sym_abs_value] = ACTIONS(1969),
    [sym_max_value] = ACTIONS(1969),
    [sym_min_value] = ACTIONS(1969),
    [sym_exp_value] = ACTIONS(1969),
    [sym_log_value] = ACTIONS(1969),
    [sym_log10_value] = ACTIONS(1969),
    [sym_power_value] = ACTIONS(1969),
    [sym_sqrt_value] = ACTIONS(1969),
    [sym_raw_be] = ACTIONS(1969),
    [sym_raw_le] = ACTIONS(1969),
    [sym_internal_queue_size_set] = ACTIONS(1969),
    [sym_internal_queue_size_get] = ACTIONS(1969),
    [sym_internal_threshold_set] = ACTIONS(1969),
    [sym_internal_threshold_get] = ACTIONS(1969),
    [sym_rebuilt_event] = ACTIONS(1969),
    [sym_do_custom_event] = ACTIONS(1969),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1969),
    [sym_clkmgr_set_clk_source] = ACTIONS(1969),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1969),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1969),
    [sym_clkmgr_get_timezone] = ACTIONS(1969),
    [sym_clkmgr_set_timezone] = ACTIONS(1969),
    [sym_clkmgr_get_resync_period] = ACTIONS(1969),
    [sym_clkmgr_set_resync_period] = ACTIONS(1969),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1969),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1969),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1969),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1969),
    [sym_clkmgr_get_timeservers] = ACTIONS(1969),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1969),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1969),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1969),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1969),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1969),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1969),
    [sym_set_log_level] = ACTIONS(1969),
    [sym_get_log_level] = ACTIONS(1969),
    [sym_amx_log] = ACTIONS(1969),
    [sym_smtp_server_config_set] = ACTIONS(1969),
    [sym_smtp_server_config_get] = ACTIONS(1969),
    [sym_smtp_send] = ACTIONS(1969),
    [sym_validate_netlinx_account] = ACTIONS(1969),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1969),
    [sym_authenticate_certificate] = ACTIONS(1969),
    [sym_audit_netlinx_session_event] = ACTIONS(1969),
    [sym_audit_netlinx_generic_event] = ACTIONS(1969),
    [sym_get_available_flash_disk_space] = ACTIONS(1969),
    [sym_get_max_flash_disk_space] = ACTIONS(1969),
    [sym_data_create_feed] = ACTIONS(1969),
    [sym_data_delete_feed] = ACTIONS(1969),
    [sym_data_publish_feed] = ACTIONS(1969),
    [sym_data_get_published_feed] = ACTIONS(1969),
    [sym_data_add_record] = ACTIONS(1969),
    [sym_data_get_event_record] = ACTIONS(1969),
    [sym__wc_data_create_feed] = ACTIONS(1969),
    [sym__wc_data_add_record] = ACTIONS(1969),
    [sym__wc_data_get_event_record] = ACTIONS(1969),
    [sym_format] = ACTIONS(1969),
    [sym_astro_clock] = ACTIONS(1969),
    [sym_set_outdoor_temperature] = ACTIONS(1969),
    [sym_set_virtual_level_count] = ACTIONS(1969),
    [sym_set_virtual_channel_count] = ACTIONS(1969),
    [sym_set_virtual_port_count] = ACTIONS(1969),
    [sym__wc_encode_int] = ACTIONS(1969),
    [sym__wc_decode_int] = ACTIONS(1969),
    [sym__wc_int] = ACTIONS(1969),
    [sym__wc_to_ch_int] = ACTIONS(1969),
    [sym__ch_to_wc_int] = ACTIONS(1969),
    [sym__wc_find_string_int] = ACTIONS(1969),
    [sym__wc_left_string_int] = ACTIONS(1969),
    [sym__wc_lower_string_int] = ACTIONS(1969),
    [sym__wc_mid_string_int] = ACTIONS(1969),
    [sym__wc_remove_string_int] = ACTIONS(1969),
    [sym__wc_right_string_int] = ACTIONS(1969),
    [sym__wc_upper_string_int] = ACTIONS(1969),
    [sym__wc_compare_string_int] = ACTIONS(1969),
    [sym__wc_get_buffer_char_int] = ACTIONS(1969),
    [sym__wc_get_buffer_string_int] = ACTIONS(1969),
    [sym__wc_concat_string_int] = ACTIONS(1969),
    [sym__wc_file_open_int] = ACTIONS(1969),
    [sym__wc_file_close_int] = ACTIONS(1969),
    [sym__wc_file_read_int] = ACTIONS(1969),
    [sym__wc_file_read_line_int] = ACTIONS(1969),
    [sym__wc_file_write_int] = ACTIONS(1969),
    [sym__wc_file_write_line_int] = ACTIONS(1969),
    [aux_sym_preproc_include_token1] = ACTIONS(1969),
    [aux_sym_preproc_define_token1] = ACTIONS(1971),
    [aux_sym_preproc_warn_token1] = ACTIONS(1971),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1971),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1971),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1971),
    [anon_sym_LBRACK] = ACTIONS(1971),
    [anon_sym_LBRACE] = ACTIONS(1971),
    [anon_sym_LPAREN] = ACTIONS(1969),
    [anon_sym_BANG] = ACTIONS(1971),
    [anon_sym_TILDE] = ACTIONS(1971),
    [anon_sym_DASH] = ACTIONS(1969),
    [anon_sym_PLUS] = ACTIONS(1969),
    [anon_sym_DASH_DASH] = ACTIONS(1971),
    [anon_sym_PLUS_PLUS] = ACTIONS(1971),
    [anon_sym_DQUOTE] = ACTIONS(1971),
    [anon_sym_SQUOTE] = ACTIONS(1971),
    [sym_number_literal] = ACTIONS(1971),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(460)] = {
    [sym_identifier] = ACTIONS(1679),
    [sym_define_call_keyword] = ACTIONS(1679),
    [sym_define_function_keyword] = ACTIONS(1679),
    [sym_define_library_function_keyword] = ACTIONS(1679),
    [sym_define_module_keyword] = ACTIONS(1679),
    [sym_char_keyword] = ACTIONS(1679),
    [sym_widechar_keyword] = ACTIONS(1679),
    [sym_integer_keyword] = ACTIONS(1679),
    [sym_sinteger_keyword] = ACTIONS(1679),
    [sym_long_keyword] = ACTIONS(1679),
    [sym_slong_keyword] = ACTIONS(1679),
    [sym_float_keyword] = ACTIONS(1679),
    [sym_double_keyword] = ACTIONS(1679),
    [sym_variant_keyword] = ACTIONS(1679),
    [sym_variantarray_keyword] = ACTIONS(1679),
    [sym_local_var_keyword] = ACTIONS(1679),
    [sym_stack_var_keyword] = ACTIONS(1679),
    [sym_constant_keyword] = ACTIONS(1679),
    [sym_volatile_keyword] = ACTIONS(1679),
    [sym_non_volatile_keyword] = ACTIONS(1679),
    [sym_persistent_keyword] = ACTIONS(1679),
    [sym_dev_keyword] = ACTIONS(1679),
    [sym_devlev_keyword] = ACTIONS(1679),
    [sym_devchan_keyword] = ACTIONS(1679),
    [sym_if_keyword] = ACTIONS(1679),
    [sym_else_keyword] = ACTIONS(1679),
    [sym_switch_keyword] = ACTIONS(1679),
    [sym_case_keyword] = ACTIONS(1679),
    [sym_default_keyword] = ACTIONS(1679),
    [sym_while_keyword] = ACTIONS(1679),
    [sym_for_keyword] = ACTIONS(1679),
    [sym_break_keyword] = ACTIONS(1679),
    [sym_continue_keyword] = ACTIONS(1679),
    [sym_return_keyword] = ACTIONS(1679),
    [sym_select_keyword] = ACTIONS(1679),
    [sym_active_keyword] = ACTIONS(1679),
    [sym_struct_keyword] = ACTIONS(1679),
    [sym_structure_keyword] = ACTIONS(1679),
    [aux_sym_on_keyword_token1] = ACTIONS(1679),
    [aux_sym_off_keyword_token1] = ACTIONS(1679),
    [sym_send_level_keyword] = ACTIONS(1679),
    [sym_send_string_keyword] = ACTIONS(1679),
    [sym_send_command_keyword] = ACTIONS(1679),
    [sym_clear_buffer_keyword] = ACTIONS(1679),
    [sym_create_buffer_keyword] = ACTIONS(1679),
    [sym_create_multi_buffer_keyword] = ACTIONS(1679),
    [sym_call_keyword] = ACTIONS(1679),
    [sym_system_call_keyword] = ACTIONS(1679),
    [sym_devchan_to_keyword] = ACTIONS(1679),
    [sym_devchan_min_to_keyword] = ACTIONS(1679),
    [sym_devchan_total_off_keyword] = ACTIONS(1679),
    [sym_devchan_pulse_keyword] = ACTIONS(1679),
    [sym_wait_keyword] = ACTIONS(1679),
    [sym_cancel_wait_keyword] = ACTIONS(1679),
    [sym_wait_until_keyword] = ACTIONS(1679),
    [sym_cancel_wait_until_keyword] = ACTIONS(1679),
    [sym_cancel_all_wait_keyword] = ACTIONS(1679),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1679),
    [sym_not] = ACTIONS(1679),
    [sym_bnot] = ACTIONS(1679),
    [sym___file__] = ACTIONS(1679),
    [sym___date__] = ACTIONS(1679),
    [sym___ldate__] = ACTIONS(1679),
    [sym___time__] = ACTIONS(1679),
    [sym___line__] = ACTIONS(1679),
    [sym___name__] = ACTIONS(1679),
    [sym___netlinx__] = ACTIONS(1679),
    [sym_netlinx_axi_version] = ACTIONS(1679),
    [sym_url_flg_tcp] = ACTIONS(1679),
    [sym_url_flg_accinfopresent] = ACTIONS(1679),
    [sym_url_flg_temp] = ACTIONS(1679),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1679),
    [sym_url_flg_stat_mask] = ACTIONS(1679),
    [sym_url_flg_stat_lookup] = ACTIONS(1679),
    [sym_url_flg_stat_connecting] = ACTIONS(1679),
    [sym_url_flg_stat_waiting] = ACTIONS(1679),
    [sym_url_flg_stat_connected] = ACTIONS(1679),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1679),
    [sym_first_virtual_device] = ACTIONS(1679),
    [sym_first_local_port] = ACTIONS(1679),
    [sym_dynamic_virtual_device] = ACTIONS(1679),
    [sym_timeline_once] = ACTIONS(1679),
    [sym_timeline_repeat] = ACTIONS(1679),
    [sym_timeline_absolute] = ACTIONS(1679),
    [sym_timeline_relative] = ACTIONS(1679),
    [sym_do_push_timed_infinite] = ACTIONS(1679),
    [sym_source_type_no_address] = ACTIONS(1679),
    [sym_source_type_neuron_id] = ACTIONS(1679),
    [sym_source_type_ip_address] = ACTIONS(1679),
    [sym_source_type_axlink] = ACTIONS(1679),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1679),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1679),
    [sym_source_type_ip_socket_address] = ACTIONS(1679),
    [sym_source_type_rs232] = ACTIONS(1679),
    [sym_source_type_internal] = ACTIONS(1679),
    [sym_source_type_mac_address] = ACTIONS(1679),
    [sym_source_type_ipv4_port] = ACTIONS(1679),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1679),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1679),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1679),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1679),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1679),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1679),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1679),
    [sym_true] = ACTIONS(1679),
    [sym_false] = ACTIONS(1679),
    [sym_file_read_only] = ACTIONS(1679),
    [sym_file_rw_new] = ACTIONS(1679),
    [sym_file_rw_append] = ACTIONS(1679),
    [sym_ip_tcp] = ACTIONS(1679),
    [sym_ip_udp] = ACTIONS(1679),
    [sym_ip_udp_2way] = ACTIONS(1679),
    [sym_xml_encode_types] = ACTIONS(1679),
    [sym_xml_encode_char_as_list] = ACTIONS(1679),
    [sym_xml_encode_le] = ACTIONS(1679),
    [sym_xml_decode_types] = ACTIONS(1679),
    [sym_xml_decode_no_preserve] = ACTIONS(1679),
    [sym_duet_dev_type_display_device] = ACTIONS(1679),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1679),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1679),
    [sym_duet_dev_type_io_device] = ACTIONS(1679),
    [sym_duet_dev_type_relay_device] = ACTIONS(1679),
    [sym_duet_dev_type_ups] = ACTIONS(1679),
    [sym_duet_dev_type_amplifier] = ACTIONS(1679),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1679),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1679),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1679),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1679),
    [sym_duet_dev_type_camera] = ACTIONS(1679),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1679),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1679),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1679),
    [sym_duet_dev_type_dss] = ACTIONS(1679),
    [sym_duet_dev_type_dvr] = ACTIONS(1679),
    [sym_duet_dev_type_disc_device] = ACTIONS(1679),
    [sym_duet_dev_type_document_camera] = ACTIONS(1679),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1679),
    [sym_duet_dev_type_hvac] = ACTIONS(1679),
    [sym_duet_dev_type_keypad] = ACTIONS(1679),
    [sym_duet_dev_type_light] = ACTIONS(1679),
    [sym_duet_dev_type_monitor] = ACTIONS(1679),
    [sym_duet_dev_type_motor] = ACTIONS(1679),
    [sym_duet_dev_type_multi_window] = ACTIONS(1679),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1679),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1679),
    [sym_duet_dev_type_receiver] = ACTIONS(1679),
    [sym_duet_dev_type_security_system] = ACTIONS(1679),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1679),
    [sym_duet_dev_type_settop_box] = ACTIONS(1679),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1679),
    [sym_duet_dev_type_switcher] = ACTIONS(1679),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1679),
    [sym_duet_dev_type_tv] = ACTIONS(1679),
    [sym_duet_dev_type_utility] = ACTIONS(1679),
    [sym_duet_dev_type_vcr] = ACTIONS(1679),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1679),
    [sym_duet_dev_type_video_processor] = ACTIONS(1679),
    [sym_duet_dev_type_video_projector] = ACTIONS(1679),
    [sym_duet_dev_type_video_wall] = ACTIONS(1679),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1679),
    [sym_duet_dev_type_weather] = ACTIONS(1679),
    [sym_duet_dev_not_polled] = ACTIONS(1679),
    [sym_duet_dev_polled] = ACTIONS(1679),
    [sym_clkmgr_mode_network] = ACTIONS(1679),
    [sym_clkmgr_mode_standalone] = ACTIONS(1679),
    [sym_normal_standby] = ACTIONS(1679),
    [sym_normal_wake] = ACTIONS(1679),
    [sym_ip_multicast_ttl_option] = ACTIONS(1679),
    [sym_ip_ttl_subnet] = ACTIONS(1679),
    [sym_ip_ttl_site] = ACTIONS(1679),
    [sym_ip_ttl_region] = ACTIONS(1679),
    [sym_ip_ttl_continent] = ACTIONS(1679),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1679),
    [sym_ip_nodelay_on] = ACTIONS(1679),
    [sym_ip_nodelay_off] = ACTIONS(1679),
    [sym_amx_error] = ACTIONS(1679),
    [sym_amx_warning] = ACTIONS(1679),
    [sym_amx_info] = ACTIONS(1679),
    [sym_amx_debug] = ACTIONS(1679),
    [sym_netlinx_login_success] = ACTIONS(1679),
    [sym_netlinx_login_fail] = ACTIONS(1679),
    [sym_netlinx_logout] = ACTIONS(1679),
    [sym_null_str] = ACTIONS(1679),
    [sym_smtp_address] = ACTIONS(1679),
    [sym_smtp_port_number] = ACTIONS(1679),
    [sym_smtp_username] = ACTIONS(1679),
    [sym_smtp_password] = ACTIONS(1679),
    [sym_smtp_require_tls] = ACTIONS(1679),
    [sym_smtp_from] = ACTIONS(1679),
    [sym_smtp_tls_true] = ACTIONS(1679),
    [sym_smtp_tls_false] = ACTIONS(1679),
    [sym_listview_on_row_select_event] = ACTIONS(1679),
    [sym_data_structure_unknown] = ACTIONS(1679),
    [sym_data_structure_datafeed] = ACTIONS(1679),
    [sym_data_structure_datarecordset] = ACTIONS(1679),
    [sym_data_structure_datarecord] = ACTIONS(1679),
    [sym_data_structure_datafield] = ACTIONS(1679),
    [sym_data_max_content_field_count] = ACTIONS(1679),
    [sym_data_max_metadata_field_count] = ACTIONS(1679),
    [sym_data_max_id_length] = ACTIONS(1679),
    [sym_data_max_name_length] = ACTIONS(1679),
    [sym_data_max_description_length] = ACTIONS(1679),
    [sym_data_max_type_length] = ACTIONS(1679),
    [sym_data_max_format_length] = ACTIONS(1679),
    [sym_data_max_source_length] = ACTIONS(1679),
    [sym_data_max_value_length] = ACTIONS(1679),
    [sym_data_max_label_length] = ACTIONS(1679),
    [sym_data_max_url_length] = ACTIONS(1679),
    [sym_max_last_login_info_length] = ACTIONS(1679),
    [sym_data_type_unknown] = ACTIONS(1679),
    [sym_data_type_string] = ACTIONS(1679),
    [sym_data_type_datetime] = ACTIONS(1679),
    [sym_data_type_date] = ACTIONS(1679),
    [sym_data_type_time] = ACTIONS(1679),
    [sym_data_type_image] = ACTIONS(1679),
    [sym_data_format_url] = ACTIONS(1679),
    [sym_data_format_phone] = ACTIONS(1679),
    [sym_data_format_email] = ACTIONS(1679),
    [sym_data_format_iso8601] = ACTIONS(1679),
    [sym_auditlog_priv] = ACTIONS(1679),
    [sym_configuration_priv] = ACTIONS(1679),
    [sym_deviceconfig_priv] = ACTIONS(1679),
    [sym_ftp_priv] = ACTIONS(1679),
    [sym_http_priv] = ACTIONS(1679),
    [sym_networkconfig_priv] = ACTIONS(1679),
    [sym_programport_priv] = ACTIONS(1679),
    [sym_remoteui_priv] = ACTIONS(1679),
    [sym_securitycontrol_priv] = ACTIONS(1679),
    [sym_softwaremanagement_priv] = ACTIONS(1679),
    [sym_terminal_priv] = ACTIONS(1679),
    [sym_tpadmin_priv] = ACTIONS(1679),
    [sym_user1_priv] = ACTIONS(1679),
    [sym_user2_priv] = ACTIONS(1679),
    [sym_user3_priv] = ACTIONS(1679),
    [sym_user4_priv] = ACTIONS(1679),
    [sym_usermanagement_priv] = ACTIONS(1679),
    [sym_valid_account] = ACTIONS(1679),
    [sym_err_name_invalid] = ACTIONS(1679),
    [sym_err_password_invalid] = ACTIONS(1679),
    [sym_err_invalid_account] = ACTIONS(1679),
    [sym_err_no_authorization] = ACTIONS(1679),
    [sym_err_invalid_parameter] = ACTIONS(1679),
    [sym_err_user_locked_out] = ACTIONS(1679),
    [sym_err_user_account_expired] = ACTIONS(1679),
    [sym_err_general_error] = ACTIONS(1679),
    [sym_err_authentication_error] = ACTIONS(1679),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1679),
    [sym_tls_validate_certificate] = ACTIONS(1679),
    [sym_tdata] = ACTIONS(1679),
    [sym_tchannel] = ACTIONS(1679),
    [sym_tlevel] = ACTIONS(1679),
    [sym_tbutton] = ACTIONS(1679),
    [sym_ttimeline] = ACTIONS(1679),
    [sym_tcustom] = ACTIONS(1679),
    [sym_url_struct] = ACTIONS(1679),
    [sym_dns_struct] = ACTIONS(1679),
    [sym_ip_address_struct] = ACTIONS(1679),
    [sym_dev_info_struct] = ACTIONS(1679),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1679),
    [sym_data_feed] = ACTIONS(1679),
    [sym_data_field] = ACTIONS(1679),
    [sym_data_record] = ACTIONS(1679),
    [sym_wc_data_feed] = ACTIONS(1679),
    [sym_wc_data_field] = ACTIONS(1679),
    [sym_wc_data_record] = ACTIONS(1679),
    [sym_last_login_info] = ACTIONS(1679),
    [sym____reserved___] = ACTIONS(1679),
    [sym_date] = ACTIONS(1679),
    [sym_day] = ACTIONS(1679),
    [sym_time] = ACTIONS(1679),
    [sym_ldate] = ACTIONS(1679),
    [sym_push_device] = ACTIONS(1679),
    [sym_push_channel] = ACTIONS(1679),
    [sym_push_devchan] = ACTIONS(1679),
    [sym_release_device] = ACTIONS(1679),
    [sym_release_channel] = ACTIONS(1679),
    [sym_release_devchan] = ACTIONS(1679),
    [sym_master_slot] = ACTIONS(1679),
    [sym_get_pulse_time] = ACTIONS(1679),
    [sym_get_timer] = ACTIONS(1679),
    [sym_master_sn] = ACTIONS(1679),
    [sym_system_number] = ACTIONS(1679),
    [sym_button] = ACTIONS(1679),
    [sym_dv_channel] = ACTIONS(1679),
    [sym_data] = ACTIONS(1679),
    [sym_level] = ACTIONS(1679),
    [sym_timeline] = ACTIONS(1679),
    [sym_channel] = ACTIONS(1679),
    [sym_custom] = ACTIONS(1679),
    [sym_atoi] = ACTIONS(1679),
    [sym_atol] = ACTIONS(1679),
    [sym_atof] = ACTIONS(1679),
    [sym_itoa] = ACTIONS(1679),
    [sym_itohex] = ACTIONS(1679),
    [sym_hextoi] = ACTIONS(1679),
    [sym_ftoa] = ACTIONS(1679),
    [sym_find_string] = ACTIONS(1679),
    [sym_left_string] = ACTIONS(1679),
    [sym_right_string] = ACTIONS(1679),
    [sym_mid_string] = ACTIONS(1679),
    [sym_remove_string] = ACTIONS(1679),
    [sym_get_buffer_string] = ACTIONS(1679),
    [sym_get_buffer_char] = ACTIONS(1679),
    [sym_get_multi_buffer_string] = ACTIONS(1679),
    [sym_length_string] = ACTIONS(1679),
    [sym_max_length_string] = ACTIONS(1679),
    [sym_set_length_string] = ACTIONS(1679),
    [sym_compare_string] = ACTIONS(1679),
    [sym_upper_string] = ACTIONS(1679),
    [sym_lower_string] = ACTIONS(1679),
    [sym_get_last] = ACTIONS(1679),
    [sym_date_to_day] = ACTIONS(1679),
    [sym_date_to_month] = ACTIONS(1679),
    [sym_date_to_year] = ACTIONS(1679),
    [sym_time_to_hour] = ACTIONS(1679),
    [sym_time_to_minute] = ACTIONS(1679),
    [sym_time_to_second] = ACTIONS(1679),
    [sym_day_of_week] = ACTIONS(1679),
    [sym_set_pulse_time] = ACTIONS(1679),
    [sym_set_timer] = ACTIONS(1679),
    [sym_file_open] = ACTIONS(1679),
    [sym_file_close] = ACTIONS(1679),
    [sym_file_read] = ACTIONS(1679),
    [sym_file_read_line] = ACTIONS(1679),
    [sym_file_write] = ACTIONS(1679),
    [sym_file_write_line] = ACTIONS(1679),
    [sym_file_copy] = ACTIONS(1679),
    [sym_file_delete] = ACTIONS(1679),
    [sym_file_rename] = ACTIONS(1679),
    [sym_file_dir] = ACTIONS(1679),
    [sym_file_getdir] = ACTIONS(1679),
    [sym_file_setdir] = ACTIONS(1679),
    [sym_file_create_dir] = ACTIONS(1679),
    [sym_file_remove_dir] = ACTIONS(1679),
    [sym_file_seek] = ACTIONS(1679),
    [sym_length_array] = ACTIONS(1679),
    [sym_max_length_array] = ACTIONS(1679),
    [sym_set_length_array] = ACTIONS(1679),
    [sym_ip_client_open] = ACTIONS(1679),
    [sym_ip_client_close] = ACTIONS(1679),
    [sym_ip_server_open] = ACTIONS(1679),
    [sym_ip_server_close] = ACTIONS(1679),
    [sym_ip_bound_client_open] = ACTIONS(1679),
    [sym_ip_mc_server_open] = ACTIONS(1679),
    [sym_ip_set_option] = ACTIONS(1679),
    [sym_ssh_client_open] = ACTIONS(1679),
    [sym_ssh_client_close] = ACTIONS(1679),
    [sym_tls_client_open] = ACTIONS(1679),
    [sym_tls_client_close] = ACTIONS(1679),
    [sym_do_push] = ACTIONS(1679),
    [sym_do_push_timed] = ACTIONS(1679),
    [sym_do_release] = ACTIONS(1679),
    [sym_device_id] = ACTIONS(1679),
    [sym_device_info] = ACTIONS(1679),
    [sym_device_id_string] = ACTIONS(1679),
    [sym_device_standby] = ACTIONS(1679),
    [sym_device_wake] = ACTIONS(1679),
    [sym_timeline_create] = ACTIONS(1679),
    [sym_timeline_kill] = ACTIONS(1679),
    [sym_timeline_reload] = ACTIONS(1679),
    [sym_timeline_pause] = ACTIONS(1679),
    [sym_timeline_restart] = ACTIONS(1679),
    [sym_timeline_set] = ACTIONS(1679),
    [sym_timeline_get] = ACTIONS(1679),
    [sym_timeline_active] = ACTIONS(1679),
    [sym_timeline_dynamic_id] = ACTIONS(1679),
    [sym_variable_to_string] = ACTIONS(1679),
    [sym_string_to_variable] = ACTIONS(1679),
    [sym_length_variable_to_string] = ACTIONS(1679),
    [sym_type_cast] = ACTIONS(1679),
    [sym_variable_to_xml] = ACTIONS(1679),
    [sym_xml_to_variable] = ACTIONS(1679),
    [sym_length_variable_to_xml] = ACTIONS(1679),
    [sym_redirect_string] = ACTIONS(1679),
    [sym_random_number] = ACTIONS(1679),
    [sym_get_unique_id] = ACTIONS(1679),
    [sym_get_system_number] = ACTIONS(1679),
    [sym_set_system_number] = ACTIONS(1679),
    [sym_get_serial_number] = ACTIONS(1679),
    [sym_get_master_build] = ACTIONS(1679),
    [sym_set_validation_code] = ACTIONS(1679),
    [sym_reboot] = ACTIONS(1679),
    [sym_load_duet_module] = ACTIONS(1679),
    [sym_unload_duet_module] = ACTIONS(1679),
    [sym_duet_mem_size_set] = ACTIONS(1679),
    [sym_duet_mem_size_get] = ACTIONS(1679),
    [sym_dynamic_application_device] = ACTIONS(1679),
    [sym_dynamic_polled_port] = ACTIONS(1679),
    [sym_static_port_binding] = ACTIONS(1679),
    [sym_static_ip_binding] = ACTIONS(1679),
    [sym_get_dns_list] = ACTIONS(1679),
    [sym_set_dns_list] = ACTIONS(1679),
    [sym_get_ip_address] = ACTIONS(1679),
    [sym_set_ip_address] = ACTIONS(1679),
    [sym_get_url_list] = ACTIONS(1679),
    [sym_add_url_entry] = ACTIONS(1679),
    [sym_delete_url_entry] = ACTIONS(1679),
    [sym_abs_value] = ACTIONS(1679),
    [sym_max_value] = ACTIONS(1679),
    [sym_min_value] = ACTIONS(1679),
    [sym_exp_value] = ACTIONS(1679),
    [sym_log_value] = ACTIONS(1679),
    [sym_log10_value] = ACTIONS(1679),
    [sym_power_value] = ACTIONS(1679),
    [sym_sqrt_value] = ACTIONS(1679),
    [sym_raw_be] = ACTIONS(1679),
    [sym_raw_le] = ACTIONS(1679),
    [sym_internal_queue_size_set] = ACTIONS(1679),
    [sym_internal_queue_size_get] = ACTIONS(1679),
    [sym_internal_threshold_set] = ACTIONS(1679),
    [sym_internal_threshold_get] = ACTIONS(1679),
    [sym_rebuilt_event] = ACTIONS(1679),
    [sym_do_custom_event] = ACTIONS(1679),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1679),
    [sym_clkmgr_set_clk_source] = ACTIONS(1679),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1679),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1679),
    [sym_clkmgr_get_timezone] = ACTIONS(1679),
    [sym_clkmgr_set_timezone] = ACTIONS(1679),
    [sym_clkmgr_get_resync_period] = ACTIONS(1679),
    [sym_clkmgr_set_resync_period] = ACTIONS(1679),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1679),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1679),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1679),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1679),
    [sym_clkmgr_get_timeservers] = ACTIONS(1679),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1679),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1679),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1679),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1679),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1679),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1679),
    [sym_set_log_level] = ACTIONS(1679),
    [sym_get_log_level] = ACTIONS(1679),
    [sym_amx_log] = ACTIONS(1679),
    [sym_smtp_server_config_set] = ACTIONS(1679),
    [sym_smtp_server_config_get] = ACTIONS(1679),
    [sym_smtp_send] = ACTIONS(1679),
    [sym_validate_netlinx_account] = ACTIONS(1679),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1679),
    [sym_authenticate_certificate] = ACTIONS(1679),
    [sym_audit_netlinx_session_event] = ACTIONS(1679),
    [sym_audit_netlinx_generic_event] = ACTIONS(1679),
    [sym_get_available_flash_disk_space] = ACTIONS(1679),
    [sym_get_max_flash_disk_space] = ACTIONS(1679),
    [sym_data_create_feed] = ACTIONS(1679),
    [sym_data_delete_feed] = ACTIONS(1679),
    [sym_data_publish_feed] = ACTIONS(1679),
    [sym_data_get_published_feed] = ACTIONS(1679),
    [sym_data_add_record] = ACTIONS(1679),
    [sym_data_get_event_record] = ACTIONS(1679),
    [sym__wc_data_create_feed] = ACTIONS(1679),
    [sym__wc_data_add_record] = ACTIONS(1679),
    [sym__wc_data_get_event_record] = ACTIONS(1679),
    [sym_format] = ACTIONS(1679),
    [sym_astro_clock] = ACTIONS(1679),
    [sym_set_outdoor_temperature] = ACTIONS(1679),
    [sym_set_virtual_level_count] = ACTIONS(1679),
    [sym_set_virtual_channel_count] = ACTIONS(1679),
    [sym_set_virtual_port_count] = ACTIONS(1679),
    [sym__wc_encode_int] = ACTIONS(1679),
    [sym__wc_decode_int] = ACTIONS(1679),
    [sym__wc_int] = ACTIONS(1679),
    [sym__wc_to_ch_int] = ACTIONS(1679),
    [sym__ch_to_wc_int] = ACTIONS(1679),
    [sym__wc_find_string_int] = ACTIONS(1679),
    [sym__wc_left_string_int] = ACTIONS(1679),
    [sym__wc_lower_string_int] = ACTIONS(1679),
    [sym__wc_mid_string_int] = ACTIONS(1679),
    [sym__wc_remove_string_int] = ACTIONS(1679),
    [sym__wc_right_string_int] = ACTIONS(1679),
    [sym__wc_upper_string_int] = ACTIONS(1679),
    [sym__wc_compare_string_int] = ACTIONS(1679),
    [sym__wc_get_buffer_char_int] = ACTIONS(1679),
    [sym__wc_get_buffer_string_int] = ACTIONS(1679),
    [sym__wc_concat_string_int] = ACTIONS(1679),
    [sym__wc_file_open_int] = ACTIONS(1679),
    [sym__wc_file_close_int] = ACTIONS(1679),
    [sym__wc_file_read_int] = ACTIONS(1679),
    [sym__wc_file_read_line_int] = ACTIONS(1679),
    [sym__wc_file_write_int] = ACTIONS(1679),
    [sym__wc_file_write_line_int] = ACTIONS(1679),
    [aux_sym_preproc_include_token1] = ACTIONS(1679),
    [aux_sym_preproc_define_token1] = ACTIONS(1681),
    [aux_sym_preproc_warn_token1] = ACTIONS(1681),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1681),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1681),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1681),
    [anon_sym_LBRACK] = ACTIONS(1681),
    [anon_sym_LBRACE] = ACTIONS(1681),
    [anon_sym_RBRACE] = ACTIONS(1681),
    [anon_sym_LPAREN] = ACTIONS(1679),
    [anon_sym_SEMI] = ACTIONS(2137),
    [anon_sym_BANG] = ACTIONS(1681),
    [anon_sym_TILDE] = ACTIONS(1681),
    [anon_sym_DASH] = ACTIONS(1679),
    [anon_sym_PLUS] = ACTIONS(1679),
    [anon_sym_DASH_DASH] = ACTIONS(1681),
    [anon_sym_PLUS_PLUS] = ACTIONS(1681),
    [anon_sym_DQUOTE] = ACTIONS(1681),
    [anon_sym_SQUOTE] = ACTIONS(1681),
    [sym_number_literal] = ACTIONS(1681),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(2137),
  },
  [STATE(461)] = {
    [sym_identifier] = ACTIONS(1673),
    [sym_define_call_keyword] = ACTIONS(1673),
    [sym_define_function_keyword] = ACTIONS(1673),
    [sym_define_library_function_keyword] = ACTIONS(1673),
    [sym_define_module_keyword] = ACTIONS(1673),
    [sym_char_keyword] = ACTIONS(1673),
    [sym_widechar_keyword] = ACTIONS(1673),
    [sym_integer_keyword] = ACTIONS(1673),
    [sym_sinteger_keyword] = ACTIONS(1673),
    [sym_long_keyword] = ACTIONS(1673),
    [sym_slong_keyword] = ACTIONS(1673),
    [sym_float_keyword] = ACTIONS(1673),
    [sym_double_keyword] = ACTIONS(1673),
    [sym_variant_keyword] = ACTIONS(1673),
    [sym_variantarray_keyword] = ACTIONS(1673),
    [sym_local_var_keyword] = ACTIONS(1673),
    [sym_stack_var_keyword] = ACTIONS(1673),
    [sym_constant_keyword] = ACTIONS(1673),
    [sym_volatile_keyword] = ACTIONS(1673),
    [sym_non_volatile_keyword] = ACTIONS(1673),
    [sym_persistent_keyword] = ACTIONS(1673),
    [sym_dev_keyword] = ACTIONS(1673),
    [sym_devlev_keyword] = ACTIONS(1673),
    [sym_devchan_keyword] = ACTIONS(1673),
    [sym_if_keyword] = ACTIONS(1673),
    [sym_else_keyword] = ACTIONS(1673),
    [sym_switch_keyword] = ACTIONS(1673),
    [sym_case_keyword] = ACTIONS(1673),
    [sym_default_keyword] = ACTIONS(1673),
    [sym_while_keyword] = ACTIONS(1673),
    [sym_for_keyword] = ACTIONS(1673),
    [sym_break_keyword] = ACTIONS(1673),
    [sym_continue_keyword] = ACTIONS(1673),
    [sym_return_keyword] = ACTIONS(1673),
    [sym_select_keyword] = ACTIONS(1673),
    [sym_active_keyword] = ACTIONS(1673),
    [sym_struct_keyword] = ACTIONS(1673),
    [sym_structure_keyword] = ACTIONS(1673),
    [aux_sym_on_keyword_token1] = ACTIONS(1673),
    [aux_sym_off_keyword_token1] = ACTIONS(1673),
    [sym_send_level_keyword] = ACTIONS(1673),
    [sym_send_string_keyword] = ACTIONS(1673),
    [sym_send_command_keyword] = ACTIONS(1673),
    [sym_clear_buffer_keyword] = ACTIONS(1673),
    [sym_create_buffer_keyword] = ACTIONS(1673),
    [sym_create_multi_buffer_keyword] = ACTIONS(1673),
    [sym_call_keyword] = ACTIONS(1673),
    [sym_system_call_keyword] = ACTIONS(1673),
    [sym_devchan_to_keyword] = ACTIONS(1673),
    [sym_devchan_min_to_keyword] = ACTIONS(1673),
    [sym_devchan_total_off_keyword] = ACTIONS(1673),
    [sym_devchan_pulse_keyword] = ACTIONS(1673),
    [sym_wait_keyword] = ACTIONS(1673),
    [sym_cancel_wait_keyword] = ACTIONS(1673),
    [sym_wait_until_keyword] = ACTIONS(1673),
    [sym_cancel_wait_until_keyword] = ACTIONS(1673),
    [sym_cancel_all_wait_keyword] = ACTIONS(1673),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1673),
    [sym_not] = ACTIONS(1673),
    [sym_bnot] = ACTIONS(1673),
    [sym___file__] = ACTIONS(1673),
    [sym___date__] = ACTIONS(1673),
    [sym___ldate__] = ACTIONS(1673),
    [sym___time__] = ACTIONS(1673),
    [sym___line__] = ACTIONS(1673),
    [sym___name__] = ACTIONS(1673),
    [sym___netlinx__] = ACTIONS(1673),
    [sym_netlinx_axi_version] = ACTIONS(1673),
    [sym_url_flg_tcp] = ACTIONS(1673),
    [sym_url_flg_accinfopresent] = ACTIONS(1673),
    [sym_url_flg_temp] = ACTIONS(1673),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1673),
    [sym_url_flg_stat_mask] = ACTIONS(1673),
    [sym_url_flg_stat_lookup] = ACTIONS(1673),
    [sym_url_flg_stat_connecting] = ACTIONS(1673),
    [sym_url_flg_stat_waiting] = ACTIONS(1673),
    [sym_url_flg_stat_connected] = ACTIONS(1673),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1673),
    [sym_first_virtual_device] = ACTIONS(1673),
    [sym_first_local_port] = ACTIONS(1673),
    [sym_dynamic_virtual_device] = ACTIONS(1673),
    [sym_timeline_once] = ACTIONS(1673),
    [sym_timeline_repeat] = ACTIONS(1673),
    [sym_timeline_absolute] = ACTIONS(1673),
    [sym_timeline_relative] = ACTIONS(1673),
    [sym_do_push_timed_infinite] = ACTIONS(1673),
    [sym_source_type_no_address] = ACTIONS(1673),
    [sym_source_type_neuron_id] = ACTIONS(1673),
    [sym_source_type_ip_address] = ACTIONS(1673),
    [sym_source_type_axlink] = ACTIONS(1673),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1673),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1673),
    [sym_source_type_ip_socket_address] = ACTIONS(1673),
    [sym_source_type_rs232] = ACTIONS(1673),
    [sym_source_type_internal] = ACTIONS(1673),
    [sym_source_type_mac_address] = ACTIONS(1673),
    [sym_source_type_ipv4_port] = ACTIONS(1673),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1673),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1673),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1673),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1673),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1673),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1673),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1673),
    [sym_true] = ACTIONS(1673),
    [sym_false] = ACTIONS(1673),
    [sym_file_read_only] = ACTIONS(1673),
    [sym_file_rw_new] = ACTIONS(1673),
    [sym_file_rw_append] = ACTIONS(1673),
    [sym_ip_tcp] = ACTIONS(1673),
    [sym_ip_udp] = ACTIONS(1673),
    [sym_ip_udp_2way] = ACTIONS(1673),
    [sym_xml_encode_types] = ACTIONS(1673),
    [sym_xml_encode_char_as_list] = ACTIONS(1673),
    [sym_xml_encode_le] = ACTIONS(1673),
    [sym_xml_decode_types] = ACTIONS(1673),
    [sym_xml_decode_no_preserve] = ACTIONS(1673),
    [sym_duet_dev_type_display_device] = ACTIONS(1673),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1673),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1673),
    [sym_duet_dev_type_io_device] = ACTIONS(1673),
    [sym_duet_dev_type_relay_device] = ACTIONS(1673),
    [sym_duet_dev_type_ups] = ACTIONS(1673),
    [sym_duet_dev_type_amplifier] = ACTIONS(1673),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1673),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1673),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1673),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1673),
    [sym_duet_dev_type_camera] = ACTIONS(1673),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1673),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1673),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1673),
    [sym_duet_dev_type_dss] = ACTIONS(1673),
    [sym_duet_dev_type_dvr] = ACTIONS(1673),
    [sym_duet_dev_type_disc_device] = ACTIONS(1673),
    [sym_duet_dev_type_document_camera] = ACTIONS(1673),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1673),
    [sym_duet_dev_type_hvac] = ACTIONS(1673),
    [sym_duet_dev_type_keypad] = ACTIONS(1673),
    [sym_duet_dev_type_light] = ACTIONS(1673),
    [sym_duet_dev_type_monitor] = ACTIONS(1673),
    [sym_duet_dev_type_motor] = ACTIONS(1673),
    [sym_duet_dev_type_multi_window] = ACTIONS(1673),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1673),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1673),
    [sym_duet_dev_type_receiver] = ACTIONS(1673),
    [sym_duet_dev_type_security_system] = ACTIONS(1673),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1673),
    [sym_duet_dev_type_settop_box] = ACTIONS(1673),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1673),
    [sym_duet_dev_type_switcher] = ACTIONS(1673),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1673),
    [sym_duet_dev_type_tv] = ACTIONS(1673),
    [sym_duet_dev_type_utility] = ACTIONS(1673),
    [sym_duet_dev_type_vcr] = ACTIONS(1673),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1673),
    [sym_duet_dev_type_video_processor] = ACTIONS(1673),
    [sym_duet_dev_type_video_projector] = ACTIONS(1673),
    [sym_duet_dev_type_video_wall] = ACTIONS(1673),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1673),
    [sym_duet_dev_type_weather] = ACTIONS(1673),
    [sym_duet_dev_not_polled] = ACTIONS(1673),
    [sym_duet_dev_polled] = ACTIONS(1673),
    [sym_clkmgr_mode_network] = ACTIONS(1673),
    [sym_clkmgr_mode_standalone] = ACTIONS(1673),
    [sym_normal_standby] = ACTIONS(1673),
    [sym_normal_wake] = ACTIONS(1673),
    [sym_ip_multicast_ttl_option] = ACTIONS(1673),
    [sym_ip_ttl_subnet] = ACTIONS(1673),
    [sym_ip_ttl_site] = ACTIONS(1673),
    [sym_ip_ttl_region] = ACTIONS(1673),
    [sym_ip_ttl_continent] = ACTIONS(1673),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1673),
    [sym_ip_nodelay_on] = ACTIONS(1673),
    [sym_ip_nodelay_off] = ACTIONS(1673),
    [sym_amx_error] = ACTIONS(1673),
    [sym_amx_warning] = ACTIONS(1673),
    [sym_amx_info] = ACTIONS(1673),
    [sym_amx_debug] = ACTIONS(1673),
    [sym_netlinx_login_success] = ACTIONS(1673),
    [sym_netlinx_login_fail] = ACTIONS(1673),
    [sym_netlinx_logout] = ACTIONS(1673),
    [sym_null_str] = ACTIONS(1673),
    [sym_smtp_address] = ACTIONS(1673),
    [sym_smtp_port_number] = ACTIONS(1673),
    [sym_smtp_username] = ACTIONS(1673),
    [sym_smtp_password] = ACTIONS(1673),
    [sym_smtp_require_tls] = ACTIONS(1673),
    [sym_smtp_from] = ACTIONS(1673),
    [sym_smtp_tls_true] = ACTIONS(1673),
    [sym_smtp_tls_false] = ACTIONS(1673),
    [sym_listview_on_row_select_event] = ACTIONS(1673),
    [sym_data_structure_unknown] = ACTIONS(1673),
    [sym_data_structure_datafeed] = ACTIONS(1673),
    [sym_data_structure_datarecordset] = ACTIONS(1673),
    [sym_data_structure_datarecord] = ACTIONS(1673),
    [sym_data_structure_datafield] = ACTIONS(1673),
    [sym_data_max_content_field_count] = ACTIONS(1673),
    [sym_data_max_metadata_field_count] = ACTIONS(1673),
    [sym_data_max_id_length] = ACTIONS(1673),
    [sym_data_max_name_length] = ACTIONS(1673),
    [sym_data_max_description_length] = ACTIONS(1673),
    [sym_data_max_type_length] = ACTIONS(1673),
    [sym_data_max_format_length] = ACTIONS(1673),
    [sym_data_max_source_length] = ACTIONS(1673),
    [sym_data_max_value_length] = ACTIONS(1673),
    [sym_data_max_label_length] = ACTIONS(1673),
    [sym_data_max_url_length] = ACTIONS(1673),
    [sym_max_last_login_info_length] = ACTIONS(1673),
    [sym_data_type_unknown] = ACTIONS(1673),
    [sym_data_type_string] = ACTIONS(1673),
    [sym_data_type_datetime] = ACTIONS(1673),
    [sym_data_type_date] = ACTIONS(1673),
    [sym_data_type_time] = ACTIONS(1673),
    [sym_data_type_image] = ACTIONS(1673),
    [sym_data_format_url] = ACTIONS(1673),
    [sym_data_format_phone] = ACTIONS(1673),
    [sym_data_format_email] = ACTIONS(1673),
    [sym_data_format_iso8601] = ACTIONS(1673),
    [sym_auditlog_priv] = ACTIONS(1673),
    [sym_configuration_priv] = ACTIONS(1673),
    [sym_deviceconfig_priv] = ACTIONS(1673),
    [sym_ftp_priv] = ACTIONS(1673),
    [sym_http_priv] = ACTIONS(1673),
    [sym_networkconfig_priv] = ACTIONS(1673),
    [sym_programport_priv] = ACTIONS(1673),
    [sym_remoteui_priv] = ACTIONS(1673),
    [sym_securitycontrol_priv] = ACTIONS(1673),
    [sym_softwaremanagement_priv] = ACTIONS(1673),
    [sym_terminal_priv] = ACTIONS(1673),
    [sym_tpadmin_priv] = ACTIONS(1673),
    [sym_user1_priv] = ACTIONS(1673),
    [sym_user2_priv] = ACTIONS(1673),
    [sym_user3_priv] = ACTIONS(1673),
    [sym_user4_priv] = ACTIONS(1673),
    [sym_usermanagement_priv] = ACTIONS(1673),
    [sym_valid_account] = ACTIONS(1673),
    [sym_err_name_invalid] = ACTIONS(1673),
    [sym_err_password_invalid] = ACTIONS(1673),
    [sym_err_invalid_account] = ACTIONS(1673),
    [sym_err_no_authorization] = ACTIONS(1673),
    [sym_err_invalid_parameter] = ACTIONS(1673),
    [sym_err_user_locked_out] = ACTIONS(1673),
    [sym_err_user_account_expired] = ACTIONS(1673),
    [sym_err_general_error] = ACTIONS(1673),
    [sym_err_authentication_error] = ACTIONS(1673),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1673),
    [sym_tls_validate_certificate] = ACTIONS(1673),
    [sym_tdata] = ACTIONS(1673),
    [sym_tchannel] = ACTIONS(1673),
    [sym_tlevel] = ACTIONS(1673),
    [sym_tbutton] = ACTIONS(1673),
    [sym_ttimeline] = ACTIONS(1673),
    [sym_tcustom] = ACTIONS(1673),
    [sym_url_struct] = ACTIONS(1673),
    [sym_dns_struct] = ACTIONS(1673),
    [sym_ip_address_struct] = ACTIONS(1673),
    [sym_dev_info_struct] = ACTIONS(1673),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1673),
    [sym_data_feed] = ACTIONS(1673),
    [sym_data_field] = ACTIONS(1673),
    [sym_data_record] = ACTIONS(1673),
    [sym_wc_data_feed] = ACTIONS(1673),
    [sym_wc_data_field] = ACTIONS(1673),
    [sym_wc_data_record] = ACTIONS(1673),
    [sym_last_login_info] = ACTIONS(1673),
    [sym____reserved___] = ACTIONS(1673),
    [sym_date] = ACTIONS(1673),
    [sym_day] = ACTIONS(1673),
    [sym_time] = ACTIONS(1673),
    [sym_ldate] = ACTIONS(1673),
    [sym_push_device] = ACTIONS(1673),
    [sym_push_channel] = ACTIONS(1673),
    [sym_push_devchan] = ACTIONS(1673),
    [sym_release_device] = ACTIONS(1673),
    [sym_release_channel] = ACTIONS(1673),
    [sym_release_devchan] = ACTIONS(1673),
    [sym_master_slot] = ACTIONS(1673),
    [sym_get_pulse_time] = ACTIONS(1673),
    [sym_get_timer] = ACTIONS(1673),
    [sym_master_sn] = ACTIONS(1673),
    [sym_system_number] = ACTIONS(1673),
    [sym_button] = ACTIONS(1673),
    [sym_dv_channel] = ACTIONS(1673),
    [sym_data] = ACTIONS(1673),
    [sym_level] = ACTIONS(1673),
    [sym_timeline] = ACTIONS(1673),
    [sym_channel] = ACTIONS(1673),
    [sym_custom] = ACTIONS(1673),
    [sym_atoi] = ACTIONS(1673),
    [sym_atol] = ACTIONS(1673),
    [sym_atof] = ACTIONS(1673),
    [sym_itoa] = ACTIONS(1673),
    [sym_itohex] = ACTIONS(1673),
    [sym_hextoi] = ACTIONS(1673),
    [sym_ftoa] = ACTIONS(1673),
    [sym_find_string] = ACTIONS(1673),
    [sym_left_string] = ACTIONS(1673),
    [sym_right_string] = ACTIONS(1673),
    [sym_mid_string] = ACTIONS(1673),
    [sym_remove_string] = ACTIONS(1673),
    [sym_get_buffer_string] = ACTIONS(1673),
    [sym_get_buffer_char] = ACTIONS(1673),
    [sym_get_multi_buffer_string] = ACTIONS(1673),
    [sym_length_string] = ACTIONS(1673),
    [sym_max_length_string] = ACTIONS(1673),
    [sym_set_length_string] = ACTIONS(1673),
    [sym_compare_string] = ACTIONS(1673),
    [sym_upper_string] = ACTIONS(1673),
    [sym_lower_string] = ACTIONS(1673),
    [sym_get_last] = ACTIONS(1673),
    [sym_date_to_day] = ACTIONS(1673),
    [sym_date_to_month] = ACTIONS(1673),
    [sym_date_to_year] = ACTIONS(1673),
    [sym_time_to_hour] = ACTIONS(1673),
    [sym_time_to_minute] = ACTIONS(1673),
    [sym_time_to_second] = ACTIONS(1673),
    [sym_day_of_week] = ACTIONS(1673),
    [sym_set_pulse_time] = ACTIONS(1673),
    [sym_set_timer] = ACTIONS(1673),
    [sym_file_open] = ACTIONS(1673),
    [sym_file_close] = ACTIONS(1673),
    [sym_file_read] = ACTIONS(1673),
    [sym_file_read_line] = ACTIONS(1673),
    [sym_file_write] = ACTIONS(1673),
    [sym_file_write_line] = ACTIONS(1673),
    [sym_file_copy] = ACTIONS(1673),
    [sym_file_delete] = ACTIONS(1673),
    [sym_file_rename] = ACTIONS(1673),
    [sym_file_dir] = ACTIONS(1673),
    [sym_file_getdir] = ACTIONS(1673),
    [sym_file_setdir] = ACTIONS(1673),
    [sym_file_create_dir] = ACTIONS(1673),
    [sym_file_remove_dir] = ACTIONS(1673),
    [sym_file_seek] = ACTIONS(1673),
    [sym_length_array] = ACTIONS(1673),
    [sym_max_length_array] = ACTIONS(1673),
    [sym_set_length_array] = ACTIONS(1673),
    [sym_ip_client_open] = ACTIONS(1673),
    [sym_ip_client_close] = ACTIONS(1673),
    [sym_ip_server_open] = ACTIONS(1673),
    [sym_ip_server_close] = ACTIONS(1673),
    [sym_ip_bound_client_open] = ACTIONS(1673),
    [sym_ip_mc_server_open] = ACTIONS(1673),
    [sym_ip_set_option] = ACTIONS(1673),
    [sym_ssh_client_open] = ACTIONS(1673),
    [sym_ssh_client_close] = ACTIONS(1673),
    [sym_tls_client_open] = ACTIONS(1673),
    [sym_tls_client_close] = ACTIONS(1673),
    [sym_do_push] = ACTIONS(1673),
    [sym_do_push_timed] = ACTIONS(1673),
    [sym_do_release] = ACTIONS(1673),
    [sym_device_id] = ACTIONS(1673),
    [sym_device_info] = ACTIONS(1673),
    [sym_device_id_string] = ACTIONS(1673),
    [sym_device_standby] = ACTIONS(1673),
    [sym_device_wake] = ACTIONS(1673),
    [sym_timeline_create] = ACTIONS(1673),
    [sym_timeline_kill] = ACTIONS(1673),
    [sym_timeline_reload] = ACTIONS(1673),
    [sym_timeline_pause] = ACTIONS(1673),
    [sym_timeline_restart] = ACTIONS(1673),
    [sym_timeline_set] = ACTIONS(1673),
    [sym_timeline_get] = ACTIONS(1673),
    [sym_timeline_active] = ACTIONS(1673),
    [sym_timeline_dynamic_id] = ACTIONS(1673),
    [sym_variable_to_string] = ACTIONS(1673),
    [sym_string_to_variable] = ACTIONS(1673),
    [sym_length_variable_to_string] = ACTIONS(1673),
    [sym_type_cast] = ACTIONS(1673),
    [sym_variable_to_xml] = ACTIONS(1673),
    [sym_xml_to_variable] = ACTIONS(1673),
    [sym_length_variable_to_xml] = ACTIONS(1673),
    [sym_redirect_string] = ACTIONS(1673),
    [sym_random_number] = ACTIONS(1673),
    [sym_get_unique_id] = ACTIONS(1673),
    [sym_get_system_number] = ACTIONS(1673),
    [sym_set_system_number] = ACTIONS(1673),
    [sym_get_serial_number] = ACTIONS(1673),
    [sym_get_master_build] = ACTIONS(1673),
    [sym_set_validation_code] = ACTIONS(1673),
    [sym_reboot] = ACTIONS(1673),
    [sym_load_duet_module] = ACTIONS(1673),
    [sym_unload_duet_module] = ACTIONS(1673),
    [sym_duet_mem_size_set] = ACTIONS(1673),
    [sym_duet_mem_size_get] = ACTIONS(1673),
    [sym_dynamic_application_device] = ACTIONS(1673),
    [sym_dynamic_polled_port] = ACTIONS(1673),
    [sym_static_port_binding] = ACTIONS(1673),
    [sym_static_ip_binding] = ACTIONS(1673),
    [sym_get_dns_list] = ACTIONS(1673),
    [sym_set_dns_list] = ACTIONS(1673),
    [sym_get_ip_address] = ACTIONS(1673),
    [sym_set_ip_address] = ACTIONS(1673),
    [sym_get_url_list] = ACTIONS(1673),
    [sym_add_url_entry] = ACTIONS(1673),
    [sym_delete_url_entry] = ACTIONS(1673),
    [sym_abs_value] = ACTIONS(1673),
    [sym_max_value] = ACTIONS(1673),
    [sym_min_value] = ACTIONS(1673),
    [sym_exp_value] = ACTIONS(1673),
    [sym_log_value] = ACTIONS(1673),
    [sym_log10_value] = ACTIONS(1673),
    [sym_power_value] = ACTIONS(1673),
    [sym_sqrt_value] = ACTIONS(1673),
    [sym_raw_be] = ACTIONS(1673),
    [sym_raw_le] = ACTIONS(1673),
    [sym_internal_queue_size_set] = ACTIONS(1673),
    [sym_internal_queue_size_get] = ACTIONS(1673),
    [sym_internal_threshold_set] = ACTIONS(1673),
    [sym_internal_threshold_get] = ACTIONS(1673),
    [sym_rebuilt_event] = ACTIONS(1673),
    [sym_do_custom_event] = ACTIONS(1673),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1673),
    [sym_clkmgr_set_clk_source] = ACTIONS(1673),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1673),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1673),
    [sym_clkmgr_get_timezone] = ACTIONS(1673),
    [sym_clkmgr_set_timezone] = ACTIONS(1673),
    [sym_clkmgr_get_resync_period] = ACTIONS(1673),
    [sym_clkmgr_set_resync_period] = ACTIONS(1673),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1673),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1673),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1673),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1673),
    [sym_clkmgr_get_timeservers] = ACTIONS(1673),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1673),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1673),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1673),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1673),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1673),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1673),
    [sym_set_log_level] = ACTIONS(1673),
    [sym_get_log_level] = ACTIONS(1673),
    [sym_amx_log] = ACTIONS(1673),
    [sym_smtp_server_config_set] = ACTIONS(1673),
    [sym_smtp_server_config_get] = ACTIONS(1673),
    [sym_smtp_send] = ACTIONS(1673),
    [sym_validate_netlinx_account] = ACTIONS(1673),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1673),
    [sym_authenticate_certificate] = ACTIONS(1673),
    [sym_audit_netlinx_session_event] = ACTIONS(1673),
    [sym_audit_netlinx_generic_event] = ACTIONS(1673),
    [sym_get_available_flash_disk_space] = ACTIONS(1673),
    [sym_get_max_flash_disk_space] = ACTIONS(1673),
    [sym_data_create_feed] = ACTIONS(1673),
    [sym_data_delete_feed] = ACTIONS(1673),
    [sym_data_publish_feed] = ACTIONS(1673),
    [sym_data_get_published_feed] = ACTIONS(1673),
    [sym_data_add_record] = ACTIONS(1673),
    [sym_data_get_event_record] = ACTIONS(1673),
    [sym__wc_data_create_feed] = ACTIONS(1673),
    [sym__wc_data_add_record] = ACTIONS(1673),
    [sym__wc_data_get_event_record] = ACTIONS(1673),
    [sym_format] = ACTIONS(1673),
    [sym_astro_clock] = ACTIONS(1673),
    [sym_set_outdoor_temperature] = ACTIONS(1673),
    [sym_set_virtual_level_count] = ACTIONS(1673),
    [sym_set_virtual_channel_count] = ACTIONS(1673),
    [sym_set_virtual_port_count] = ACTIONS(1673),
    [sym__wc_encode_int] = ACTIONS(1673),
    [sym__wc_decode_int] = ACTIONS(1673),
    [sym__wc_int] = ACTIONS(1673),
    [sym__wc_to_ch_int] = ACTIONS(1673),
    [sym__ch_to_wc_int] = ACTIONS(1673),
    [sym__wc_find_string_int] = ACTIONS(1673),
    [sym__wc_left_string_int] = ACTIONS(1673),
    [sym__wc_lower_string_int] = ACTIONS(1673),
    [sym__wc_mid_string_int] = ACTIONS(1673),
    [sym__wc_remove_string_int] = ACTIONS(1673),
    [sym__wc_right_string_int] = ACTIONS(1673),
    [sym__wc_upper_string_int] = ACTIONS(1673),
    [sym__wc_compare_string_int] = ACTIONS(1673),
    [sym__wc_get_buffer_char_int] = ACTIONS(1673),
    [sym__wc_get_buffer_string_int] = ACTIONS(1673),
    [sym__wc_concat_string_int] = ACTIONS(1673),
    [sym__wc_file_open_int] = ACTIONS(1673),
    [sym__wc_file_close_int] = ACTIONS(1673),
    [sym__wc_file_read_int] = ACTIONS(1673),
    [sym__wc_file_read_line_int] = ACTIONS(1673),
    [sym__wc_file_write_int] = ACTIONS(1673),
    [sym__wc_file_write_line_int] = ACTIONS(1673),
    [aux_sym_preproc_include_token1] = ACTIONS(1673),
    [aux_sym_preproc_define_token1] = ACTIONS(1675),
    [aux_sym_preproc_warn_token1] = ACTIONS(1675),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1675),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1675),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1675),
    [anon_sym_LBRACK] = ACTIONS(1675),
    [anon_sym_LBRACE] = ACTIONS(1675),
    [anon_sym_RBRACE] = ACTIONS(1675),
    [anon_sym_LPAREN] = ACTIONS(1673),
    [anon_sym_SEMI] = ACTIONS(2139),
    [anon_sym_BANG] = ACTIONS(1675),
    [anon_sym_TILDE] = ACTIONS(1675),
    [anon_sym_DASH] = ACTIONS(1673),
    [anon_sym_PLUS] = ACTIONS(1673),
    [anon_sym_DASH_DASH] = ACTIONS(1675),
    [anon_sym_PLUS_PLUS] = ACTIONS(1675),
    [anon_sym_DQUOTE] = ACTIONS(1675),
    [anon_sym_SQUOTE] = ACTIONS(1675),
    [sym_number_literal] = ACTIONS(1675),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(2139),
  },
  [STATE(462)] = {
    [sym_else_clause] = STATE(489),
    [sym_identifier] = ACTIONS(1687),
    [sym_define_call_keyword] = ACTIONS(1687),
    [sym_define_function_keyword] = ACTIONS(1687),
    [sym_define_library_function_keyword] = ACTIONS(1687),
    [sym_define_module_keyword] = ACTIONS(1687),
    [sym_char_keyword] = ACTIONS(1687),
    [sym_widechar_keyword] = ACTIONS(1687),
    [sym_integer_keyword] = ACTIONS(1687),
    [sym_sinteger_keyword] = ACTIONS(1687),
    [sym_long_keyword] = ACTIONS(1687),
    [sym_slong_keyword] = ACTIONS(1687),
    [sym_float_keyword] = ACTIONS(1687),
    [sym_double_keyword] = ACTIONS(1687),
    [sym_variant_keyword] = ACTIONS(1687),
    [sym_variantarray_keyword] = ACTIONS(1687),
    [sym_local_var_keyword] = ACTIONS(1687),
    [sym_stack_var_keyword] = ACTIONS(1687),
    [sym_constant_keyword] = ACTIONS(1687),
    [sym_volatile_keyword] = ACTIONS(1687),
    [sym_non_volatile_keyword] = ACTIONS(1687),
    [sym_persistent_keyword] = ACTIONS(1687),
    [sym_dev_keyword] = ACTIONS(1687),
    [sym_devlev_keyword] = ACTIONS(1687),
    [sym_devchan_keyword] = ACTIONS(1687),
    [sym_if_keyword] = ACTIONS(1687),
    [sym_else_keyword] = ACTIONS(2141),
    [sym_switch_keyword] = ACTIONS(1687),
    [sym_case_keyword] = ACTIONS(1687),
    [sym_default_keyword] = ACTIONS(1687),
    [sym_while_keyword] = ACTIONS(1687),
    [sym_for_keyword] = ACTIONS(1687),
    [sym_break_keyword] = ACTIONS(1687),
    [sym_continue_keyword] = ACTIONS(1687),
    [sym_return_keyword] = ACTIONS(1687),
    [sym_select_keyword] = ACTIONS(1687),
    [sym_active_keyword] = ACTIONS(1687),
    [sym_struct_keyword] = ACTIONS(1687),
    [sym_structure_keyword] = ACTIONS(1687),
    [aux_sym_on_keyword_token1] = ACTIONS(1687),
    [aux_sym_off_keyword_token1] = ACTIONS(1687),
    [sym_send_level_keyword] = ACTIONS(1687),
    [sym_send_string_keyword] = ACTIONS(1687),
    [sym_send_command_keyword] = ACTIONS(1687),
    [sym_clear_buffer_keyword] = ACTIONS(1687),
    [sym_create_buffer_keyword] = ACTIONS(1687),
    [sym_create_multi_buffer_keyword] = ACTIONS(1687),
    [sym_call_keyword] = ACTIONS(1687),
    [sym_system_call_keyword] = ACTIONS(1687),
    [sym_devchan_to_keyword] = ACTIONS(1687),
    [sym_devchan_min_to_keyword] = ACTIONS(1687),
    [sym_devchan_total_off_keyword] = ACTIONS(1687),
    [sym_devchan_pulse_keyword] = ACTIONS(1687),
    [sym_wait_keyword] = ACTIONS(1687),
    [sym_cancel_wait_keyword] = ACTIONS(1687),
    [sym_wait_until_keyword] = ACTIONS(1687),
    [sym_cancel_wait_until_keyword] = ACTIONS(1687),
    [sym_cancel_all_wait_keyword] = ACTIONS(1687),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1687),
    [sym_not] = ACTIONS(1687),
    [sym_bnot] = ACTIONS(1687),
    [sym___file__] = ACTIONS(1687),
    [sym___date__] = ACTIONS(1687),
    [sym___ldate__] = ACTIONS(1687),
    [sym___time__] = ACTIONS(1687),
    [sym___line__] = ACTIONS(1687),
    [sym___name__] = ACTIONS(1687),
    [sym___netlinx__] = ACTIONS(1687),
    [sym_netlinx_axi_version] = ACTIONS(1687),
    [sym_url_flg_tcp] = ACTIONS(1687),
    [sym_url_flg_accinfopresent] = ACTIONS(1687),
    [sym_url_flg_temp] = ACTIONS(1687),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1687),
    [sym_url_flg_stat_mask] = ACTIONS(1687),
    [sym_url_flg_stat_lookup] = ACTIONS(1687),
    [sym_url_flg_stat_connecting] = ACTIONS(1687),
    [sym_url_flg_stat_waiting] = ACTIONS(1687),
    [sym_url_flg_stat_connected] = ACTIONS(1687),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1687),
    [sym_first_virtual_device] = ACTIONS(1687),
    [sym_first_local_port] = ACTIONS(1687),
    [sym_dynamic_virtual_device] = ACTIONS(1687),
    [sym_timeline_once] = ACTIONS(1687),
    [sym_timeline_repeat] = ACTIONS(1687),
    [sym_timeline_absolute] = ACTIONS(1687),
    [sym_timeline_relative] = ACTIONS(1687),
    [sym_do_push_timed_infinite] = ACTIONS(1687),
    [sym_source_type_no_address] = ACTIONS(1687),
    [sym_source_type_neuron_id] = ACTIONS(1687),
    [sym_source_type_ip_address] = ACTIONS(1687),
    [sym_source_type_axlink] = ACTIONS(1687),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1687),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1687),
    [sym_source_type_ip_socket_address] = ACTIONS(1687),
    [sym_source_type_rs232] = ACTIONS(1687),
    [sym_source_type_internal] = ACTIONS(1687),
    [sym_source_type_mac_address] = ACTIONS(1687),
    [sym_source_type_ipv4_port] = ACTIONS(1687),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1687),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1687),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1687),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1687),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1687),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1687),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1687),
    [sym_true] = ACTIONS(1687),
    [sym_false] = ACTIONS(1687),
    [sym_file_read_only] = ACTIONS(1687),
    [sym_file_rw_new] = ACTIONS(1687),
    [sym_file_rw_append] = ACTIONS(1687),
    [sym_ip_tcp] = ACTIONS(1687),
    [sym_ip_udp] = ACTIONS(1687),
    [sym_ip_udp_2way] = ACTIONS(1687),
    [sym_xml_encode_types] = ACTIONS(1687),
    [sym_xml_encode_char_as_list] = ACTIONS(1687),
    [sym_xml_encode_le] = ACTIONS(1687),
    [sym_xml_decode_types] = ACTIONS(1687),
    [sym_xml_decode_no_preserve] = ACTIONS(1687),
    [sym_duet_dev_type_display_device] = ACTIONS(1687),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1687),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1687),
    [sym_duet_dev_type_io_device] = ACTIONS(1687),
    [sym_duet_dev_type_relay_device] = ACTIONS(1687),
    [sym_duet_dev_type_ups] = ACTIONS(1687),
    [sym_duet_dev_type_amplifier] = ACTIONS(1687),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1687),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1687),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1687),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1687),
    [sym_duet_dev_type_camera] = ACTIONS(1687),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1687),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1687),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1687),
    [sym_duet_dev_type_dss] = ACTIONS(1687),
    [sym_duet_dev_type_dvr] = ACTIONS(1687),
    [sym_duet_dev_type_disc_device] = ACTIONS(1687),
    [sym_duet_dev_type_document_camera] = ACTIONS(1687),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1687),
    [sym_duet_dev_type_hvac] = ACTIONS(1687),
    [sym_duet_dev_type_keypad] = ACTIONS(1687),
    [sym_duet_dev_type_light] = ACTIONS(1687),
    [sym_duet_dev_type_monitor] = ACTIONS(1687),
    [sym_duet_dev_type_motor] = ACTIONS(1687),
    [sym_duet_dev_type_multi_window] = ACTIONS(1687),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1687),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1687),
    [sym_duet_dev_type_receiver] = ACTIONS(1687),
    [sym_duet_dev_type_security_system] = ACTIONS(1687),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1687),
    [sym_duet_dev_type_settop_box] = ACTIONS(1687),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1687),
    [sym_duet_dev_type_switcher] = ACTIONS(1687),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1687),
    [sym_duet_dev_type_tv] = ACTIONS(1687),
    [sym_duet_dev_type_utility] = ACTIONS(1687),
    [sym_duet_dev_type_vcr] = ACTIONS(1687),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1687),
    [sym_duet_dev_type_video_processor] = ACTIONS(1687),
    [sym_duet_dev_type_video_projector] = ACTIONS(1687),
    [sym_duet_dev_type_video_wall] = ACTIONS(1687),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1687),
    [sym_duet_dev_type_weather] = ACTIONS(1687),
    [sym_duet_dev_not_polled] = ACTIONS(1687),
    [sym_duet_dev_polled] = ACTIONS(1687),
    [sym_clkmgr_mode_network] = ACTIONS(1687),
    [sym_clkmgr_mode_standalone] = ACTIONS(1687),
    [sym_normal_standby] = ACTIONS(1687),
    [sym_normal_wake] = ACTIONS(1687),
    [sym_ip_multicast_ttl_option] = ACTIONS(1687),
    [sym_ip_ttl_subnet] = ACTIONS(1687),
    [sym_ip_ttl_site] = ACTIONS(1687),
    [sym_ip_ttl_region] = ACTIONS(1687),
    [sym_ip_ttl_continent] = ACTIONS(1687),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1687),
    [sym_ip_nodelay_on] = ACTIONS(1687),
    [sym_ip_nodelay_off] = ACTIONS(1687),
    [sym_amx_error] = ACTIONS(1687),
    [sym_amx_warning] = ACTIONS(1687),
    [sym_amx_info] = ACTIONS(1687),
    [sym_amx_debug] = ACTIONS(1687),
    [sym_netlinx_login_success] = ACTIONS(1687),
    [sym_netlinx_login_fail] = ACTIONS(1687),
    [sym_netlinx_logout] = ACTIONS(1687),
    [sym_null_str] = ACTIONS(1687),
    [sym_smtp_address] = ACTIONS(1687),
    [sym_smtp_port_number] = ACTIONS(1687),
    [sym_smtp_username] = ACTIONS(1687),
    [sym_smtp_password] = ACTIONS(1687),
    [sym_smtp_require_tls] = ACTIONS(1687),
    [sym_smtp_from] = ACTIONS(1687),
    [sym_smtp_tls_true] = ACTIONS(1687),
    [sym_smtp_tls_false] = ACTIONS(1687),
    [sym_listview_on_row_select_event] = ACTIONS(1687),
    [sym_data_structure_unknown] = ACTIONS(1687),
    [sym_data_structure_datafeed] = ACTIONS(1687),
    [sym_data_structure_datarecordset] = ACTIONS(1687),
    [sym_data_structure_datarecord] = ACTIONS(1687),
    [sym_data_structure_datafield] = ACTIONS(1687),
    [sym_data_max_content_field_count] = ACTIONS(1687),
    [sym_data_max_metadata_field_count] = ACTIONS(1687),
    [sym_data_max_id_length] = ACTIONS(1687),
    [sym_data_max_name_length] = ACTIONS(1687),
    [sym_data_max_description_length] = ACTIONS(1687),
    [sym_data_max_type_length] = ACTIONS(1687),
    [sym_data_max_format_length] = ACTIONS(1687),
    [sym_data_max_source_length] = ACTIONS(1687),
    [sym_data_max_value_length] = ACTIONS(1687),
    [sym_data_max_label_length] = ACTIONS(1687),
    [sym_data_max_url_length] = ACTIONS(1687),
    [sym_max_last_login_info_length] = ACTIONS(1687),
    [sym_data_type_unknown] = ACTIONS(1687),
    [sym_data_type_string] = ACTIONS(1687),
    [sym_data_type_datetime] = ACTIONS(1687),
    [sym_data_type_date] = ACTIONS(1687),
    [sym_data_type_time] = ACTIONS(1687),
    [sym_data_type_image] = ACTIONS(1687),
    [sym_data_format_url] = ACTIONS(1687),
    [sym_data_format_phone] = ACTIONS(1687),
    [sym_data_format_email] = ACTIONS(1687),
    [sym_data_format_iso8601] = ACTIONS(1687),
    [sym_auditlog_priv] = ACTIONS(1687),
    [sym_configuration_priv] = ACTIONS(1687),
    [sym_deviceconfig_priv] = ACTIONS(1687),
    [sym_ftp_priv] = ACTIONS(1687),
    [sym_http_priv] = ACTIONS(1687),
    [sym_networkconfig_priv] = ACTIONS(1687),
    [sym_programport_priv] = ACTIONS(1687),
    [sym_remoteui_priv] = ACTIONS(1687),
    [sym_securitycontrol_priv] = ACTIONS(1687),
    [sym_softwaremanagement_priv] = ACTIONS(1687),
    [sym_terminal_priv] = ACTIONS(1687),
    [sym_tpadmin_priv] = ACTIONS(1687),
    [sym_user1_priv] = ACTIONS(1687),
    [sym_user2_priv] = ACTIONS(1687),
    [sym_user3_priv] = ACTIONS(1687),
    [sym_user4_priv] = ACTIONS(1687),
    [sym_usermanagement_priv] = ACTIONS(1687),
    [sym_valid_account] = ACTIONS(1687),
    [sym_err_name_invalid] = ACTIONS(1687),
    [sym_err_password_invalid] = ACTIONS(1687),
    [sym_err_invalid_account] = ACTIONS(1687),
    [sym_err_no_authorization] = ACTIONS(1687),
    [sym_err_invalid_parameter] = ACTIONS(1687),
    [sym_err_user_locked_out] = ACTIONS(1687),
    [sym_err_user_account_expired] = ACTIONS(1687),
    [sym_err_general_error] = ACTIONS(1687),
    [sym_err_authentication_error] = ACTIONS(1687),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1687),
    [sym_tls_validate_certificate] = ACTIONS(1687),
    [sym_tdata] = ACTIONS(1687),
    [sym_tchannel] = ACTIONS(1687),
    [sym_tlevel] = ACTIONS(1687),
    [sym_tbutton] = ACTIONS(1687),
    [sym_ttimeline] = ACTIONS(1687),
    [sym_tcustom] = ACTIONS(1687),
    [sym_url_struct] = ACTIONS(1687),
    [sym_dns_struct] = ACTIONS(1687),
    [sym_ip_address_struct] = ACTIONS(1687),
    [sym_dev_info_struct] = ACTIONS(1687),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1687),
    [sym_data_feed] = ACTIONS(1687),
    [sym_data_field] = ACTIONS(1687),
    [sym_data_record] = ACTIONS(1687),
    [sym_wc_data_feed] = ACTIONS(1687),
    [sym_wc_data_field] = ACTIONS(1687),
    [sym_wc_data_record] = ACTIONS(1687),
    [sym_last_login_info] = ACTIONS(1687),
    [sym____reserved___] = ACTIONS(1687),
    [sym_date] = ACTIONS(1687),
    [sym_day] = ACTIONS(1687),
    [sym_time] = ACTIONS(1687),
    [sym_ldate] = ACTIONS(1687),
    [sym_push_device] = ACTIONS(1687),
    [sym_push_channel] = ACTIONS(1687),
    [sym_push_devchan] = ACTIONS(1687),
    [sym_release_device] = ACTIONS(1687),
    [sym_release_channel] = ACTIONS(1687),
    [sym_release_devchan] = ACTIONS(1687),
    [sym_master_slot] = ACTIONS(1687),
    [sym_get_pulse_time] = ACTIONS(1687),
    [sym_get_timer] = ACTIONS(1687),
    [sym_master_sn] = ACTIONS(1687),
    [sym_system_number] = ACTIONS(1687),
    [sym_button] = ACTIONS(1687),
    [sym_dv_channel] = ACTIONS(1687),
    [sym_data] = ACTIONS(1687),
    [sym_level] = ACTIONS(1687),
    [sym_timeline] = ACTIONS(1687),
    [sym_channel] = ACTIONS(1687),
    [sym_custom] = ACTIONS(1687),
    [sym_atoi] = ACTIONS(1687),
    [sym_atol] = ACTIONS(1687),
    [sym_atof] = ACTIONS(1687),
    [sym_itoa] = ACTIONS(1687),
    [sym_itohex] = ACTIONS(1687),
    [sym_hextoi] = ACTIONS(1687),
    [sym_ftoa] = ACTIONS(1687),
    [sym_find_string] = ACTIONS(1687),
    [sym_left_string] = ACTIONS(1687),
    [sym_right_string] = ACTIONS(1687),
    [sym_mid_string] = ACTIONS(1687),
    [sym_remove_string] = ACTIONS(1687),
    [sym_get_buffer_string] = ACTIONS(1687),
    [sym_get_buffer_char] = ACTIONS(1687),
    [sym_get_multi_buffer_string] = ACTIONS(1687),
    [sym_length_string] = ACTIONS(1687),
    [sym_max_length_string] = ACTIONS(1687),
    [sym_set_length_string] = ACTIONS(1687),
    [sym_compare_string] = ACTIONS(1687),
    [sym_upper_string] = ACTIONS(1687),
    [sym_lower_string] = ACTIONS(1687),
    [sym_get_last] = ACTIONS(1687),
    [sym_date_to_day] = ACTIONS(1687),
    [sym_date_to_month] = ACTIONS(1687),
    [sym_date_to_year] = ACTIONS(1687),
    [sym_time_to_hour] = ACTIONS(1687),
    [sym_time_to_minute] = ACTIONS(1687),
    [sym_time_to_second] = ACTIONS(1687),
    [sym_day_of_week] = ACTIONS(1687),
    [sym_set_pulse_time] = ACTIONS(1687),
    [sym_set_timer] = ACTIONS(1687),
    [sym_file_open] = ACTIONS(1687),
    [sym_file_close] = ACTIONS(1687),
    [sym_file_read] = ACTIONS(1687),
    [sym_file_read_line] = ACTIONS(1687),
    [sym_file_write] = ACTIONS(1687),
    [sym_file_write_line] = ACTIONS(1687),
    [sym_file_copy] = ACTIONS(1687),
    [sym_file_delete] = ACTIONS(1687),
    [sym_file_rename] = ACTIONS(1687),
    [sym_file_dir] = ACTIONS(1687),
    [sym_file_getdir] = ACTIONS(1687),
    [sym_file_setdir] = ACTIONS(1687),
    [sym_file_create_dir] = ACTIONS(1687),
    [sym_file_remove_dir] = ACTIONS(1687),
    [sym_file_seek] = ACTIONS(1687),
    [sym_length_array] = ACTIONS(1687),
    [sym_max_length_array] = ACTIONS(1687),
    [sym_set_length_array] = ACTIONS(1687),
    [sym_ip_client_open] = ACTIONS(1687),
    [sym_ip_client_close] = ACTIONS(1687),
    [sym_ip_server_open] = ACTIONS(1687),
    [sym_ip_server_close] = ACTIONS(1687),
    [sym_ip_bound_client_open] = ACTIONS(1687),
    [sym_ip_mc_server_open] = ACTIONS(1687),
    [sym_ip_set_option] = ACTIONS(1687),
    [sym_ssh_client_open] = ACTIONS(1687),
    [sym_ssh_client_close] = ACTIONS(1687),
    [sym_tls_client_open] = ACTIONS(1687),
    [sym_tls_client_close] = ACTIONS(1687),
    [sym_do_push] = ACTIONS(1687),
    [sym_do_push_timed] = ACTIONS(1687),
    [sym_do_release] = ACTIONS(1687),
    [sym_device_id] = ACTIONS(1687),
    [sym_device_info] = ACTIONS(1687),
    [sym_device_id_string] = ACTIONS(1687),
    [sym_device_standby] = ACTIONS(1687),
    [sym_device_wake] = ACTIONS(1687),
    [sym_timeline_create] = ACTIONS(1687),
    [sym_timeline_kill] = ACTIONS(1687),
    [sym_timeline_reload] = ACTIONS(1687),
    [sym_timeline_pause] = ACTIONS(1687),
    [sym_timeline_restart] = ACTIONS(1687),
    [sym_timeline_set] = ACTIONS(1687),
    [sym_timeline_get] = ACTIONS(1687),
    [sym_timeline_active] = ACTIONS(1687),
    [sym_timeline_dynamic_id] = ACTIONS(1687),
    [sym_variable_to_string] = ACTIONS(1687),
    [sym_string_to_variable] = ACTIONS(1687),
    [sym_length_variable_to_string] = ACTIONS(1687),
    [sym_type_cast] = ACTIONS(1687),
    [sym_variable_to_xml] = ACTIONS(1687),
    [sym_xml_to_variable] = ACTIONS(1687),
    [sym_length_variable_to_xml] = ACTIONS(1687),
    [sym_redirect_string] = ACTIONS(1687),
    [sym_random_number] = ACTIONS(1687),
    [sym_get_unique_id] = ACTIONS(1687),
    [sym_get_system_number] = ACTIONS(1687),
    [sym_set_system_number] = ACTIONS(1687),
    [sym_get_serial_number] = ACTIONS(1687),
    [sym_get_master_build] = ACTIONS(1687),
    [sym_set_validation_code] = ACTIONS(1687),
    [sym_reboot] = ACTIONS(1687),
    [sym_load_duet_module] = ACTIONS(1687),
    [sym_unload_duet_module] = ACTIONS(1687),
    [sym_duet_mem_size_set] = ACTIONS(1687),
    [sym_duet_mem_size_get] = ACTIONS(1687),
    [sym_dynamic_application_device] = ACTIONS(1687),
    [sym_dynamic_polled_port] = ACTIONS(1687),
    [sym_static_port_binding] = ACTIONS(1687),
    [sym_static_ip_binding] = ACTIONS(1687),
    [sym_get_dns_list] = ACTIONS(1687),
    [sym_set_dns_list] = ACTIONS(1687),
    [sym_get_ip_address] = ACTIONS(1687),
    [sym_set_ip_address] = ACTIONS(1687),
    [sym_get_url_list] = ACTIONS(1687),
    [sym_add_url_entry] = ACTIONS(1687),
    [sym_delete_url_entry] = ACTIONS(1687),
    [sym_abs_value] = ACTIONS(1687),
    [sym_max_value] = ACTIONS(1687),
    [sym_min_value] = ACTIONS(1687),
    [sym_exp_value] = ACTIONS(1687),
    [sym_log_value] = ACTIONS(1687),
    [sym_log10_value] = ACTIONS(1687),
    [sym_power_value] = ACTIONS(1687),
    [sym_sqrt_value] = ACTIONS(1687),
    [sym_raw_be] = ACTIONS(1687),
    [sym_raw_le] = ACTIONS(1687),
    [sym_internal_queue_size_set] = ACTIONS(1687),
    [sym_internal_queue_size_get] = ACTIONS(1687),
    [sym_internal_threshold_set] = ACTIONS(1687),
    [sym_internal_threshold_get] = ACTIONS(1687),
    [sym_rebuilt_event] = ACTIONS(1687),
    [sym_do_custom_event] = ACTIONS(1687),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1687),
    [sym_clkmgr_set_clk_source] = ACTIONS(1687),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1687),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1687),
    [sym_clkmgr_get_timezone] = ACTIONS(1687),
    [sym_clkmgr_set_timezone] = ACTIONS(1687),
    [sym_clkmgr_get_resync_period] = ACTIONS(1687),
    [sym_clkmgr_set_resync_period] = ACTIONS(1687),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1687),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1687),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1687),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1687),
    [sym_clkmgr_get_timeservers] = ACTIONS(1687),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1687),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1687),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1687),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1687),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1687),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1687),
    [sym_set_log_level] = ACTIONS(1687),
    [sym_get_log_level] = ACTIONS(1687),
    [sym_amx_log] = ACTIONS(1687),
    [sym_smtp_server_config_set] = ACTIONS(1687),
    [sym_smtp_server_config_get] = ACTIONS(1687),
    [sym_smtp_send] = ACTIONS(1687),
    [sym_validate_netlinx_account] = ACTIONS(1687),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1687),
    [sym_authenticate_certificate] = ACTIONS(1687),
    [sym_audit_netlinx_session_event] = ACTIONS(1687),
    [sym_audit_netlinx_generic_event] = ACTIONS(1687),
    [sym_get_available_flash_disk_space] = ACTIONS(1687),
    [sym_get_max_flash_disk_space] = ACTIONS(1687),
    [sym_data_create_feed] = ACTIONS(1687),
    [sym_data_delete_feed] = ACTIONS(1687),
    [sym_data_publish_feed] = ACTIONS(1687),
    [sym_data_get_published_feed] = ACTIONS(1687),
    [sym_data_add_record] = ACTIONS(1687),
    [sym_data_get_event_record] = ACTIONS(1687),
    [sym__wc_data_create_feed] = ACTIONS(1687),
    [sym__wc_data_add_record] = ACTIONS(1687),
    [sym__wc_data_get_event_record] = ACTIONS(1687),
    [sym_format] = ACTIONS(1687),
    [sym_astro_clock] = ACTIONS(1687),
    [sym_set_outdoor_temperature] = ACTIONS(1687),
    [sym_set_virtual_level_count] = ACTIONS(1687),
    [sym_set_virtual_channel_count] = ACTIONS(1687),
    [sym_set_virtual_port_count] = ACTIONS(1687),
    [sym__wc_encode_int] = ACTIONS(1687),
    [sym__wc_decode_int] = ACTIONS(1687),
    [sym__wc_int] = ACTIONS(1687),
    [sym__wc_to_ch_int] = ACTIONS(1687),
    [sym__ch_to_wc_int] = ACTIONS(1687),
    [sym__wc_find_string_int] = ACTIONS(1687),
    [sym__wc_left_string_int] = ACTIONS(1687),
    [sym__wc_lower_string_int] = ACTIONS(1687),
    [sym__wc_mid_string_int] = ACTIONS(1687),
    [sym__wc_remove_string_int] = ACTIONS(1687),
    [sym__wc_right_string_int] = ACTIONS(1687),
    [sym__wc_upper_string_int] = ACTIONS(1687),
    [sym__wc_compare_string_int] = ACTIONS(1687),
    [sym__wc_get_buffer_char_int] = ACTIONS(1687),
    [sym__wc_get_buffer_string_int] = ACTIONS(1687),
    [sym__wc_concat_string_int] = ACTIONS(1687),
    [sym__wc_file_open_int] = ACTIONS(1687),
    [sym__wc_file_close_int] = ACTIONS(1687),
    [sym__wc_file_read_int] = ACTIONS(1687),
    [sym__wc_file_read_line_int] = ACTIONS(1687),
    [sym__wc_file_write_int] = ACTIONS(1687),
    [sym__wc_file_write_line_int] = ACTIONS(1687),
    [aux_sym_preproc_include_token1] = ACTIONS(1687),
    [aux_sym_preproc_define_token1] = ACTIONS(1691),
    [aux_sym_preproc_warn_token1] = ACTIONS(1691),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1691),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1691),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1691),
    [anon_sym_LBRACK] = ACTIONS(1691),
    [anon_sym_LBRACE] = ACTIONS(1691),
    [anon_sym_RBRACE] = ACTIONS(1691),
    [anon_sym_LPAREN] = ACTIONS(1687),
    [anon_sym_BANG] = ACTIONS(1691),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1687),
    [anon_sym_PLUS] = ACTIONS(1687),
    [anon_sym_DASH_DASH] = ACTIONS(1691),
    [anon_sym_PLUS_PLUS] = ACTIONS(1691),
    [anon_sym_DQUOTE] = ACTIONS(1691),
    [anon_sym_SQUOTE] = ACTIONS(1691),
    [sym_number_literal] = ACTIONS(1691),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(463)] = {
    [sym_identifier] = ACTIONS(1819),
    [sym_define_call_keyword] = ACTIONS(1819),
    [sym_define_function_keyword] = ACTIONS(1819),
    [sym_define_library_function_keyword] = ACTIONS(1819),
    [sym_define_module_keyword] = ACTIONS(1819),
    [sym_char_keyword] = ACTIONS(1819),
    [sym_widechar_keyword] = ACTIONS(1819),
    [sym_integer_keyword] = ACTIONS(1819),
    [sym_sinteger_keyword] = ACTIONS(1819),
    [sym_long_keyword] = ACTIONS(1819),
    [sym_slong_keyword] = ACTIONS(1819),
    [sym_float_keyword] = ACTIONS(1819),
    [sym_double_keyword] = ACTIONS(1819),
    [sym_variant_keyword] = ACTIONS(1819),
    [sym_variantarray_keyword] = ACTIONS(1819),
    [sym_local_var_keyword] = ACTIONS(1819),
    [sym_stack_var_keyword] = ACTIONS(1819),
    [sym_constant_keyword] = ACTIONS(1819),
    [sym_volatile_keyword] = ACTIONS(1819),
    [sym_non_volatile_keyword] = ACTIONS(1819),
    [sym_persistent_keyword] = ACTIONS(1819),
    [sym_dev_keyword] = ACTIONS(1819),
    [sym_devlev_keyword] = ACTIONS(1819),
    [sym_devchan_keyword] = ACTIONS(1819),
    [sym_if_keyword] = ACTIONS(1819),
    [sym_else_keyword] = ACTIONS(1819),
    [sym_switch_keyword] = ACTIONS(1819),
    [sym_case_keyword] = ACTIONS(1819),
    [sym_default_keyword] = ACTIONS(1819),
    [sym_while_keyword] = ACTIONS(1819),
    [sym_for_keyword] = ACTIONS(1819),
    [sym_break_keyword] = ACTIONS(1819),
    [sym_continue_keyword] = ACTIONS(1819),
    [sym_return_keyword] = ACTIONS(1819),
    [sym_select_keyword] = ACTIONS(1819),
    [sym_active_keyword] = ACTIONS(1819),
    [sym_struct_keyword] = ACTIONS(1819),
    [sym_structure_keyword] = ACTIONS(1819),
    [aux_sym_on_keyword_token1] = ACTIONS(1819),
    [aux_sym_off_keyword_token1] = ACTIONS(1819),
    [sym_send_level_keyword] = ACTIONS(1819),
    [sym_send_string_keyword] = ACTIONS(1819),
    [sym_send_command_keyword] = ACTIONS(1819),
    [sym_clear_buffer_keyword] = ACTIONS(1819),
    [sym_create_buffer_keyword] = ACTIONS(1819),
    [sym_create_multi_buffer_keyword] = ACTIONS(1819),
    [sym_call_keyword] = ACTIONS(1819),
    [sym_system_call_keyword] = ACTIONS(1819),
    [sym_devchan_to_keyword] = ACTIONS(1819),
    [sym_devchan_min_to_keyword] = ACTIONS(1819),
    [sym_devchan_total_off_keyword] = ACTIONS(1819),
    [sym_devchan_pulse_keyword] = ACTIONS(1819),
    [sym_wait_keyword] = ACTIONS(1819),
    [sym_cancel_wait_keyword] = ACTIONS(1819),
    [sym_wait_until_keyword] = ACTIONS(1819),
    [sym_cancel_wait_until_keyword] = ACTIONS(1819),
    [sym_cancel_all_wait_keyword] = ACTIONS(1819),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1819),
    [sym_not] = ACTIONS(1819),
    [sym_bnot] = ACTIONS(1819),
    [sym___file__] = ACTIONS(1819),
    [sym___date__] = ACTIONS(1819),
    [sym___ldate__] = ACTIONS(1819),
    [sym___time__] = ACTIONS(1819),
    [sym___line__] = ACTIONS(1819),
    [sym___name__] = ACTIONS(1819),
    [sym___netlinx__] = ACTIONS(1819),
    [sym_netlinx_axi_version] = ACTIONS(1819),
    [sym_url_flg_tcp] = ACTIONS(1819),
    [sym_url_flg_accinfopresent] = ACTIONS(1819),
    [sym_url_flg_temp] = ACTIONS(1819),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1819),
    [sym_url_flg_stat_mask] = ACTIONS(1819),
    [sym_url_flg_stat_lookup] = ACTIONS(1819),
    [sym_url_flg_stat_connecting] = ACTIONS(1819),
    [sym_url_flg_stat_waiting] = ACTIONS(1819),
    [sym_url_flg_stat_connected] = ACTIONS(1819),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1819),
    [sym_first_virtual_device] = ACTIONS(1819),
    [sym_first_local_port] = ACTIONS(1819),
    [sym_dynamic_virtual_device] = ACTIONS(1819),
    [sym_timeline_once] = ACTIONS(1819),
    [sym_timeline_repeat] = ACTIONS(1819),
    [sym_timeline_absolute] = ACTIONS(1819),
    [sym_timeline_relative] = ACTIONS(1819),
    [sym_do_push_timed_infinite] = ACTIONS(1819),
    [sym_source_type_no_address] = ACTIONS(1819),
    [sym_source_type_neuron_id] = ACTIONS(1819),
    [sym_source_type_ip_address] = ACTIONS(1819),
    [sym_source_type_axlink] = ACTIONS(1819),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1819),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1819),
    [sym_source_type_ip_socket_address] = ACTIONS(1819),
    [sym_source_type_rs232] = ACTIONS(1819),
    [sym_source_type_internal] = ACTIONS(1819),
    [sym_source_type_mac_address] = ACTIONS(1819),
    [sym_source_type_ipv4_port] = ACTIONS(1819),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1819),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1819),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1819),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1819),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1819),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1819),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1819),
    [sym_true] = ACTIONS(1819),
    [sym_false] = ACTIONS(1819),
    [sym_file_read_only] = ACTIONS(1819),
    [sym_file_rw_new] = ACTIONS(1819),
    [sym_file_rw_append] = ACTIONS(1819),
    [sym_ip_tcp] = ACTIONS(1819),
    [sym_ip_udp] = ACTIONS(1819),
    [sym_ip_udp_2way] = ACTIONS(1819),
    [sym_xml_encode_types] = ACTIONS(1819),
    [sym_xml_encode_char_as_list] = ACTIONS(1819),
    [sym_xml_encode_le] = ACTIONS(1819),
    [sym_xml_decode_types] = ACTIONS(1819),
    [sym_xml_decode_no_preserve] = ACTIONS(1819),
    [sym_duet_dev_type_display_device] = ACTIONS(1819),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1819),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1819),
    [sym_duet_dev_type_io_device] = ACTIONS(1819),
    [sym_duet_dev_type_relay_device] = ACTIONS(1819),
    [sym_duet_dev_type_ups] = ACTIONS(1819),
    [sym_duet_dev_type_amplifier] = ACTIONS(1819),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1819),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1819),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1819),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1819),
    [sym_duet_dev_type_camera] = ACTIONS(1819),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1819),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1819),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1819),
    [sym_duet_dev_type_dss] = ACTIONS(1819),
    [sym_duet_dev_type_dvr] = ACTIONS(1819),
    [sym_duet_dev_type_disc_device] = ACTIONS(1819),
    [sym_duet_dev_type_document_camera] = ACTIONS(1819),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1819),
    [sym_duet_dev_type_hvac] = ACTIONS(1819),
    [sym_duet_dev_type_keypad] = ACTIONS(1819),
    [sym_duet_dev_type_light] = ACTIONS(1819),
    [sym_duet_dev_type_monitor] = ACTIONS(1819),
    [sym_duet_dev_type_motor] = ACTIONS(1819),
    [sym_duet_dev_type_multi_window] = ACTIONS(1819),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1819),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1819),
    [sym_duet_dev_type_receiver] = ACTIONS(1819),
    [sym_duet_dev_type_security_system] = ACTIONS(1819),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1819),
    [sym_duet_dev_type_settop_box] = ACTIONS(1819),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1819),
    [sym_duet_dev_type_switcher] = ACTIONS(1819),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1819),
    [sym_duet_dev_type_tv] = ACTIONS(1819),
    [sym_duet_dev_type_utility] = ACTIONS(1819),
    [sym_duet_dev_type_vcr] = ACTIONS(1819),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1819),
    [sym_duet_dev_type_video_processor] = ACTIONS(1819),
    [sym_duet_dev_type_video_projector] = ACTIONS(1819),
    [sym_duet_dev_type_video_wall] = ACTIONS(1819),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1819),
    [sym_duet_dev_type_weather] = ACTIONS(1819),
    [sym_duet_dev_not_polled] = ACTIONS(1819),
    [sym_duet_dev_polled] = ACTIONS(1819),
    [sym_clkmgr_mode_network] = ACTIONS(1819),
    [sym_clkmgr_mode_standalone] = ACTIONS(1819),
    [sym_normal_standby] = ACTIONS(1819),
    [sym_normal_wake] = ACTIONS(1819),
    [sym_ip_multicast_ttl_option] = ACTIONS(1819),
    [sym_ip_ttl_subnet] = ACTIONS(1819),
    [sym_ip_ttl_site] = ACTIONS(1819),
    [sym_ip_ttl_region] = ACTIONS(1819),
    [sym_ip_ttl_continent] = ACTIONS(1819),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1819),
    [sym_ip_nodelay_on] = ACTIONS(1819),
    [sym_ip_nodelay_off] = ACTIONS(1819),
    [sym_amx_error] = ACTIONS(1819),
    [sym_amx_warning] = ACTIONS(1819),
    [sym_amx_info] = ACTIONS(1819),
    [sym_amx_debug] = ACTIONS(1819),
    [sym_netlinx_login_success] = ACTIONS(1819),
    [sym_netlinx_login_fail] = ACTIONS(1819),
    [sym_netlinx_logout] = ACTIONS(1819),
    [sym_null_str] = ACTIONS(1819),
    [sym_smtp_address] = ACTIONS(1819),
    [sym_smtp_port_number] = ACTIONS(1819),
    [sym_smtp_username] = ACTIONS(1819),
    [sym_smtp_password] = ACTIONS(1819),
    [sym_smtp_require_tls] = ACTIONS(1819),
    [sym_smtp_from] = ACTIONS(1819),
    [sym_smtp_tls_true] = ACTIONS(1819),
    [sym_smtp_tls_false] = ACTIONS(1819),
    [sym_listview_on_row_select_event] = ACTIONS(1819),
    [sym_data_structure_unknown] = ACTIONS(1819),
    [sym_data_structure_datafeed] = ACTIONS(1819),
    [sym_data_structure_datarecordset] = ACTIONS(1819),
    [sym_data_structure_datarecord] = ACTIONS(1819),
    [sym_data_structure_datafield] = ACTIONS(1819),
    [sym_data_max_content_field_count] = ACTIONS(1819),
    [sym_data_max_metadata_field_count] = ACTIONS(1819),
    [sym_data_max_id_length] = ACTIONS(1819),
    [sym_data_max_name_length] = ACTIONS(1819),
    [sym_data_max_description_length] = ACTIONS(1819),
    [sym_data_max_type_length] = ACTIONS(1819),
    [sym_data_max_format_length] = ACTIONS(1819),
    [sym_data_max_source_length] = ACTIONS(1819),
    [sym_data_max_value_length] = ACTIONS(1819),
    [sym_data_max_label_length] = ACTIONS(1819),
    [sym_data_max_url_length] = ACTIONS(1819),
    [sym_max_last_login_info_length] = ACTIONS(1819),
    [sym_data_type_unknown] = ACTIONS(1819),
    [sym_data_type_string] = ACTIONS(1819),
    [sym_data_type_datetime] = ACTIONS(1819),
    [sym_data_type_date] = ACTIONS(1819),
    [sym_data_type_time] = ACTIONS(1819),
    [sym_data_type_image] = ACTIONS(1819),
    [sym_data_format_url] = ACTIONS(1819),
    [sym_data_format_phone] = ACTIONS(1819),
    [sym_data_format_email] = ACTIONS(1819),
    [sym_data_format_iso8601] = ACTIONS(1819),
    [sym_auditlog_priv] = ACTIONS(1819),
    [sym_configuration_priv] = ACTIONS(1819),
    [sym_deviceconfig_priv] = ACTIONS(1819),
    [sym_ftp_priv] = ACTIONS(1819),
    [sym_http_priv] = ACTIONS(1819),
    [sym_networkconfig_priv] = ACTIONS(1819),
    [sym_programport_priv] = ACTIONS(1819),
    [sym_remoteui_priv] = ACTIONS(1819),
    [sym_securitycontrol_priv] = ACTIONS(1819),
    [sym_softwaremanagement_priv] = ACTIONS(1819),
    [sym_terminal_priv] = ACTIONS(1819),
    [sym_tpadmin_priv] = ACTIONS(1819),
    [sym_user1_priv] = ACTIONS(1819),
    [sym_user2_priv] = ACTIONS(1819),
    [sym_user3_priv] = ACTIONS(1819),
    [sym_user4_priv] = ACTIONS(1819),
    [sym_usermanagement_priv] = ACTIONS(1819),
    [sym_valid_account] = ACTIONS(1819),
    [sym_err_name_invalid] = ACTIONS(1819),
    [sym_err_password_invalid] = ACTIONS(1819),
    [sym_err_invalid_account] = ACTIONS(1819),
    [sym_err_no_authorization] = ACTIONS(1819),
    [sym_err_invalid_parameter] = ACTIONS(1819),
    [sym_err_user_locked_out] = ACTIONS(1819),
    [sym_err_user_account_expired] = ACTIONS(1819),
    [sym_err_general_error] = ACTIONS(1819),
    [sym_err_authentication_error] = ACTIONS(1819),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1819),
    [sym_tls_validate_certificate] = ACTIONS(1819),
    [sym_tdata] = ACTIONS(1819),
    [sym_tchannel] = ACTIONS(1819),
    [sym_tlevel] = ACTIONS(1819),
    [sym_tbutton] = ACTIONS(1819),
    [sym_ttimeline] = ACTIONS(1819),
    [sym_tcustom] = ACTIONS(1819),
    [sym_url_struct] = ACTIONS(1819),
    [sym_dns_struct] = ACTIONS(1819),
    [sym_ip_address_struct] = ACTIONS(1819),
    [sym_dev_info_struct] = ACTIONS(1819),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1819),
    [sym_data_feed] = ACTIONS(1819),
    [sym_data_field] = ACTIONS(1819),
    [sym_data_record] = ACTIONS(1819),
    [sym_wc_data_feed] = ACTIONS(1819),
    [sym_wc_data_field] = ACTIONS(1819),
    [sym_wc_data_record] = ACTIONS(1819),
    [sym_last_login_info] = ACTIONS(1819),
    [sym____reserved___] = ACTIONS(1819),
    [sym_date] = ACTIONS(1819),
    [sym_day] = ACTIONS(1819),
    [sym_time] = ACTIONS(1819),
    [sym_ldate] = ACTIONS(1819),
    [sym_push_device] = ACTIONS(1819),
    [sym_push_channel] = ACTIONS(1819),
    [sym_push_devchan] = ACTIONS(1819),
    [sym_release_device] = ACTIONS(1819),
    [sym_release_channel] = ACTIONS(1819),
    [sym_release_devchan] = ACTIONS(1819),
    [sym_master_slot] = ACTIONS(1819),
    [sym_get_pulse_time] = ACTIONS(1819),
    [sym_get_timer] = ACTIONS(1819),
    [sym_master_sn] = ACTIONS(1819),
    [sym_system_number] = ACTIONS(1819),
    [sym_button] = ACTIONS(1819),
    [sym_dv_channel] = ACTIONS(1819),
    [sym_data] = ACTIONS(1819),
    [sym_level] = ACTIONS(1819),
    [sym_timeline] = ACTIONS(1819),
    [sym_channel] = ACTIONS(1819),
    [sym_custom] = ACTIONS(1819),
    [sym_atoi] = ACTIONS(1819),
    [sym_atol] = ACTIONS(1819),
    [sym_atof] = ACTIONS(1819),
    [sym_itoa] = ACTIONS(1819),
    [sym_itohex] = ACTIONS(1819),
    [sym_hextoi] = ACTIONS(1819),
    [sym_ftoa] = ACTIONS(1819),
    [sym_find_string] = ACTIONS(1819),
    [sym_left_string] = ACTIONS(1819),
    [sym_right_string] = ACTIONS(1819),
    [sym_mid_string] = ACTIONS(1819),
    [sym_remove_string] = ACTIONS(1819),
    [sym_get_buffer_string] = ACTIONS(1819),
    [sym_get_buffer_char] = ACTIONS(1819),
    [sym_get_multi_buffer_string] = ACTIONS(1819),
    [sym_length_string] = ACTIONS(1819),
    [sym_max_length_string] = ACTIONS(1819),
    [sym_set_length_string] = ACTIONS(1819),
    [sym_compare_string] = ACTIONS(1819),
    [sym_upper_string] = ACTIONS(1819),
    [sym_lower_string] = ACTIONS(1819),
    [sym_get_last] = ACTIONS(1819),
    [sym_date_to_day] = ACTIONS(1819),
    [sym_date_to_month] = ACTIONS(1819),
    [sym_date_to_year] = ACTIONS(1819),
    [sym_time_to_hour] = ACTIONS(1819),
    [sym_time_to_minute] = ACTIONS(1819),
    [sym_time_to_second] = ACTIONS(1819),
    [sym_day_of_week] = ACTIONS(1819),
    [sym_set_pulse_time] = ACTIONS(1819),
    [sym_set_timer] = ACTIONS(1819),
    [sym_file_open] = ACTIONS(1819),
    [sym_file_close] = ACTIONS(1819),
    [sym_file_read] = ACTIONS(1819),
    [sym_file_read_line] = ACTIONS(1819),
    [sym_file_write] = ACTIONS(1819),
    [sym_file_write_line] = ACTIONS(1819),
    [sym_file_copy] = ACTIONS(1819),
    [sym_file_delete] = ACTIONS(1819),
    [sym_file_rename] = ACTIONS(1819),
    [sym_file_dir] = ACTIONS(1819),
    [sym_file_getdir] = ACTIONS(1819),
    [sym_file_setdir] = ACTIONS(1819),
    [sym_file_create_dir] = ACTIONS(1819),
    [sym_file_remove_dir] = ACTIONS(1819),
    [sym_file_seek] = ACTIONS(1819),
    [sym_length_array] = ACTIONS(1819),
    [sym_max_length_array] = ACTIONS(1819),
    [sym_set_length_array] = ACTIONS(1819),
    [sym_ip_client_open] = ACTIONS(1819),
    [sym_ip_client_close] = ACTIONS(1819),
    [sym_ip_server_open] = ACTIONS(1819),
    [sym_ip_server_close] = ACTIONS(1819),
    [sym_ip_bound_client_open] = ACTIONS(1819),
    [sym_ip_mc_server_open] = ACTIONS(1819),
    [sym_ip_set_option] = ACTIONS(1819),
    [sym_ssh_client_open] = ACTIONS(1819),
    [sym_ssh_client_close] = ACTIONS(1819),
    [sym_tls_client_open] = ACTIONS(1819),
    [sym_tls_client_close] = ACTIONS(1819),
    [sym_do_push] = ACTIONS(1819),
    [sym_do_push_timed] = ACTIONS(1819),
    [sym_do_release] = ACTIONS(1819),
    [sym_device_id] = ACTIONS(1819),
    [sym_device_info] = ACTIONS(1819),
    [sym_device_id_string] = ACTIONS(1819),
    [sym_device_standby] = ACTIONS(1819),
    [sym_device_wake] = ACTIONS(1819),
    [sym_timeline_create] = ACTIONS(1819),
    [sym_timeline_kill] = ACTIONS(1819),
    [sym_timeline_reload] = ACTIONS(1819),
    [sym_timeline_pause] = ACTIONS(1819),
    [sym_timeline_restart] = ACTIONS(1819),
    [sym_timeline_set] = ACTIONS(1819),
    [sym_timeline_get] = ACTIONS(1819),
    [sym_timeline_active] = ACTIONS(1819),
    [sym_timeline_dynamic_id] = ACTIONS(1819),
    [sym_variable_to_string] = ACTIONS(1819),
    [sym_string_to_variable] = ACTIONS(1819),
    [sym_length_variable_to_string] = ACTIONS(1819),
    [sym_type_cast] = ACTIONS(1819),
    [sym_variable_to_xml] = ACTIONS(1819),
    [sym_xml_to_variable] = ACTIONS(1819),
    [sym_length_variable_to_xml] = ACTIONS(1819),
    [sym_redirect_string] = ACTIONS(1819),
    [sym_random_number] = ACTIONS(1819),
    [sym_get_unique_id] = ACTIONS(1819),
    [sym_get_system_number] = ACTIONS(1819),
    [sym_set_system_number] = ACTIONS(1819),
    [sym_get_serial_number] = ACTIONS(1819),
    [sym_get_master_build] = ACTIONS(1819),
    [sym_set_validation_code] = ACTIONS(1819),
    [sym_reboot] = ACTIONS(1819),
    [sym_load_duet_module] = ACTIONS(1819),
    [sym_unload_duet_module] = ACTIONS(1819),
    [sym_duet_mem_size_set] = ACTIONS(1819),
    [sym_duet_mem_size_get] = ACTIONS(1819),
    [sym_dynamic_application_device] = ACTIONS(1819),
    [sym_dynamic_polled_port] = ACTIONS(1819),
    [sym_static_port_binding] = ACTIONS(1819),
    [sym_static_ip_binding] = ACTIONS(1819),
    [sym_get_dns_list] = ACTIONS(1819),
    [sym_set_dns_list] = ACTIONS(1819),
    [sym_get_ip_address] = ACTIONS(1819),
    [sym_set_ip_address] = ACTIONS(1819),
    [sym_get_url_list] = ACTIONS(1819),
    [sym_add_url_entry] = ACTIONS(1819),
    [sym_delete_url_entry] = ACTIONS(1819),
    [sym_abs_value] = ACTIONS(1819),
    [sym_max_value] = ACTIONS(1819),
    [sym_min_value] = ACTIONS(1819),
    [sym_exp_value] = ACTIONS(1819),
    [sym_log_value] = ACTIONS(1819),
    [sym_log10_value] = ACTIONS(1819),
    [sym_power_value] = ACTIONS(1819),
    [sym_sqrt_value] = ACTIONS(1819),
    [sym_raw_be] = ACTIONS(1819),
    [sym_raw_le] = ACTIONS(1819),
    [sym_internal_queue_size_set] = ACTIONS(1819),
    [sym_internal_queue_size_get] = ACTIONS(1819),
    [sym_internal_threshold_set] = ACTIONS(1819),
    [sym_internal_threshold_get] = ACTIONS(1819),
    [sym_rebuilt_event] = ACTIONS(1819),
    [sym_do_custom_event] = ACTIONS(1819),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1819),
    [sym_clkmgr_set_clk_source] = ACTIONS(1819),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1819),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1819),
    [sym_clkmgr_get_timezone] = ACTIONS(1819),
    [sym_clkmgr_set_timezone] = ACTIONS(1819),
    [sym_clkmgr_get_resync_period] = ACTIONS(1819),
    [sym_clkmgr_set_resync_period] = ACTIONS(1819),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1819),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1819),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1819),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1819),
    [sym_clkmgr_get_timeservers] = ACTIONS(1819),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1819),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1819),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1819),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1819),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1819),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1819),
    [sym_set_log_level] = ACTIONS(1819),
    [sym_get_log_level] = ACTIONS(1819),
    [sym_amx_log] = ACTIONS(1819),
    [sym_smtp_server_config_set] = ACTIONS(1819),
    [sym_smtp_server_config_get] = ACTIONS(1819),
    [sym_smtp_send] = ACTIONS(1819),
    [sym_validate_netlinx_account] = ACTIONS(1819),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1819),
    [sym_authenticate_certificate] = ACTIONS(1819),
    [sym_audit_netlinx_session_event] = ACTIONS(1819),
    [sym_audit_netlinx_generic_event] = ACTIONS(1819),
    [sym_get_available_flash_disk_space] = ACTIONS(1819),
    [sym_get_max_flash_disk_space] = ACTIONS(1819),
    [sym_data_create_feed] = ACTIONS(1819),
    [sym_data_delete_feed] = ACTIONS(1819),
    [sym_data_publish_feed] = ACTIONS(1819),
    [sym_data_get_published_feed] = ACTIONS(1819),
    [sym_data_add_record] = ACTIONS(1819),
    [sym_data_get_event_record] = ACTIONS(1819),
    [sym__wc_data_create_feed] = ACTIONS(1819),
    [sym__wc_data_add_record] = ACTIONS(1819),
    [sym__wc_data_get_event_record] = ACTIONS(1819),
    [sym_format] = ACTIONS(1819),
    [sym_astro_clock] = ACTIONS(1819),
    [sym_set_outdoor_temperature] = ACTIONS(1819),
    [sym_set_virtual_level_count] = ACTIONS(1819),
    [sym_set_virtual_channel_count] = ACTIONS(1819),
    [sym_set_virtual_port_count] = ACTIONS(1819),
    [sym__wc_encode_int] = ACTIONS(1819),
    [sym__wc_decode_int] = ACTIONS(1819),
    [sym__wc_int] = ACTIONS(1819),
    [sym__wc_to_ch_int] = ACTIONS(1819),
    [sym__ch_to_wc_int] = ACTIONS(1819),
    [sym__wc_find_string_int] = ACTIONS(1819),
    [sym__wc_left_string_int] = ACTIONS(1819),
    [sym__wc_lower_string_int] = ACTIONS(1819),
    [sym__wc_mid_string_int] = ACTIONS(1819),
    [sym__wc_remove_string_int] = ACTIONS(1819),
    [sym__wc_right_string_int] = ACTIONS(1819),
    [sym__wc_upper_string_int] = ACTIONS(1819),
    [sym__wc_compare_string_int] = ACTIONS(1819),
    [sym__wc_get_buffer_char_int] = ACTIONS(1819),
    [sym__wc_get_buffer_string_int] = ACTIONS(1819),
    [sym__wc_concat_string_int] = ACTIONS(1819),
    [sym__wc_file_open_int] = ACTIONS(1819),
    [sym__wc_file_close_int] = ACTIONS(1819),
    [sym__wc_file_read_int] = ACTIONS(1819),
    [sym__wc_file_read_line_int] = ACTIONS(1819),
    [sym__wc_file_write_int] = ACTIONS(1819),
    [sym__wc_file_write_line_int] = ACTIONS(1819),
    [aux_sym_preproc_include_token1] = ACTIONS(1819),
    [aux_sym_preproc_define_token1] = ACTIONS(1821),
    [aux_sym_preproc_warn_token1] = ACTIONS(1821),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1821),
    [anon_sym_LBRACK] = ACTIONS(1821),
    [anon_sym_LBRACE] = ACTIONS(1821),
    [anon_sym_RBRACE] = ACTIONS(1821),
    [anon_sym_LPAREN] = ACTIONS(1819),
    [anon_sym_BANG] = ACTIONS(1821),
    [anon_sym_TILDE] = ACTIONS(1821),
    [anon_sym_DASH] = ACTIONS(1819),
    [anon_sym_PLUS] = ACTIONS(1819),
    [anon_sym_DASH_DASH] = ACTIONS(1821),
    [anon_sym_PLUS_PLUS] = ACTIONS(1821),
    [anon_sym_DQUOTE] = ACTIONS(1821),
    [anon_sym_SQUOTE] = ACTIONS(1821),
    [sym_number_literal] = ACTIONS(1821),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(464)] = {
    [sym_identifier] = ACTIONS(1789),
    [sym_define_call_keyword] = ACTIONS(1789),
    [sym_define_function_keyword] = ACTIONS(1789),
    [sym_define_library_function_keyword] = ACTIONS(1789),
    [sym_define_module_keyword] = ACTIONS(1789),
    [sym_char_keyword] = ACTIONS(1789),
    [sym_widechar_keyword] = ACTIONS(1789),
    [sym_integer_keyword] = ACTIONS(1789),
    [sym_sinteger_keyword] = ACTIONS(1789),
    [sym_long_keyword] = ACTIONS(1789),
    [sym_slong_keyword] = ACTIONS(1789),
    [sym_float_keyword] = ACTIONS(1789),
    [sym_double_keyword] = ACTIONS(1789),
    [sym_variant_keyword] = ACTIONS(1789),
    [sym_variantarray_keyword] = ACTIONS(1789),
    [sym_local_var_keyword] = ACTIONS(1789),
    [sym_stack_var_keyword] = ACTIONS(1789),
    [sym_constant_keyword] = ACTIONS(1789),
    [sym_volatile_keyword] = ACTIONS(1789),
    [sym_non_volatile_keyword] = ACTIONS(1789),
    [sym_persistent_keyword] = ACTIONS(1789),
    [sym_dev_keyword] = ACTIONS(1789),
    [sym_devlev_keyword] = ACTIONS(1789),
    [sym_devchan_keyword] = ACTIONS(1789),
    [sym_if_keyword] = ACTIONS(1789),
    [sym_else_keyword] = ACTIONS(1789),
    [sym_switch_keyword] = ACTIONS(1789),
    [sym_case_keyword] = ACTIONS(1789),
    [sym_default_keyword] = ACTIONS(1789),
    [sym_while_keyword] = ACTIONS(1789),
    [sym_for_keyword] = ACTIONS(1789),
    [sym_break_keyword] = ACTIONS(1789),
    [sym_continue_keyword] = ACTIONS(1789),
    [sym_return_keyword] = ACTIONS(1789),
    [sym_select_keyword] = ACTIONS(1789),
    [sym_active_keyword] = ACTIONS(1789),
    [sym_struct_keyword] = ACTIONS(1789),
    [sym_structure_keyword] = ACTIONS(1789),
    [aux_sym_on_keyword_token1] = ACTIONS(1789),
    [aux_sym_off_keyword_token1] = ACTIONS(1789),
    [sym_send_level_keyword] = ACTIONS(1789),
    [sym_send_string_keyword] = ACTIONS(1789),
    [sym_send_command_keyword] = ACTIONS(1789),
    [sym_clear_buffer_keyword] = ACTIONS(1789),
    [sym_create_buffer_keyword] = ACTIONS(1789),
    [sym_create_multi_buffer_keyword] = ACTIONS(1789),
    [sym_call_keyword] = ACTIONS(1789),
    [sym_system_call_keyword] = ACTIONS(1789),
    [sym_devchan_to_keyword] = ACTIONS(1789),
    [sym_devchan_min_to_keyword] = ACTIONS(1789),
    [sym_devchan_total_off_keyword] = ACTIONS(1789),
    [sym_devchan_pulse_keyword] = ACTIONS(1789),
    [sym_wait_keyword] = ACTIONS(1789),
    [sym_cancel_wait_keyword] = ACTIONS(1789),
    [sym_wait_until_keyword] = ACTIONS(1789),
    [sym_cancel_wait_until_keyword] = ACTIONS(1789),
    [sym_cancel_all_wait_keyword] = ACTIONS(1789),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1789),
    [sym_not] = ACTIONS(1789),
    [sym_bnot] = ACTIONS(1789),
    [sym___file__] = ACTIONS(1789),
    [sym___date__] = ACTIONS(1789),
    [sym___ldate__] = ACTIONS(1789),
    [sym___time__] = ACTIONS(1789),
    [sym___line__] = ACTIONS(1789),
    [sym___name__] = ACTIONS(1789),
    [sym___netlinx__] = ACTIONS(1789),
    [sym_netlinx_axi_version] = ACTIONS(1789),
    [sym_url_flg_tcp] = ACTIONS(1789),
    [sym_url_flg_accinfopresent] = ACTIONS(1789),
    [sym_url_flg_temp] = ACTIONS(1789),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1789),
    [sym_url_flg_stat_mask] = ACTIONS(1789),
    [sym_url_flg_stat_lookup] = ACTIONS(1789),
    [sym_url_flg_stat_connecting] = ACTIONS(1789),
    [sym_url_flg_stat_waiting] = ACTIONS(1789),
    [sym_url_flg_stat_connected] = ACTIONS(1789),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1789),
    [sym_first_virtual_device] = ACTIONS(1789),
    [sym_first_local_port] = ACTIONS(1789),
    [sym_dynamic_virtual_device] = ACTIONS(1789),
    [sym_timeline_once] = ACTIONS(1789),
    [sym_timeline_repeat] = ACTIONS(1789),
    [sym_timeline_absolute] = ACTIONS(1789),
    [sym_timeline_relative] = ACTIONS(1789),
    [sym_do_push_timed_infinite] = ACTIONS(1789),
    [sym_source_type_no_address] = ACTIONS(1789),
    [sym_source_type_neuron_id] = ACTIONS(1789),
    [sym_source_type_ip_address] = ACTIONS(1789),
    [sym_source_type_axlink] = ACTIONS(1789),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1789),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1789),
    [sym_source_type_ip_socket_address] = ACTIONS(1789),
    [sym_source_type_rs232] = ACTIONS(1789),
    [sym_source_type_internal] = ACTIONS(1789),
    [sym_source_type_mac_address] = ACTIONS(1789),
    [sym_source_type_ipv4_port] = ACTIONS(1789),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1789),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1789),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1789),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1789),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1789),
    [sym_true] = ACTIONS(1789),
    [sym_false] = ACTIONS(1789),
    [sym_file_read_only] = ACTIONS(1789),
    [sym_file_rw_new] = ACTIONS(1789),
    [sym_file_rw_append] = ACTIONS(1789),
    [sym_ip_tcp] = ACTIONS(1789),
    [sym_ip_udp] = ACTIONS(1789),
    [sym_ip_udp_2way] = ACTIONS(1789),
    [sym_xml_encode_types] = ACTIONS(1789),
    [sym_xml_encode_char_as_list] = ACTIONS(1789),
    [sym_xml_encode_le] = ACTIONS(1789),
    [sym_xml_decode_types] = ACTIONS(1789),
    [sym_xml_decode_no_preserve] = ACTIONS(1789),
    [sym_duet_dev_type_display_device] = ACTIONS(1789),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1789),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1789),
    [sym_duet_dev_type_io_device] = ACTIONS(1789),
    [sym_duet_dev_type_relay_device] = ACTIONS(1789),
    [sym_duet_dev_type_ups] = ACTIONS(1789),
    [sym_duet_dev_type_amplifier] = ACTIONS(1789),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1789),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1789),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1789),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1789),
    [sym_duet_dev_type_camera] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1789),
    [sym_duet_dev_type_dss] = ACTIONS(1789),
    [sym_duet_dev_type_dvr] = ACTIONS(1789),
    [sym_duet_dev_type_disc_device] = ACTIONS(1789),
    [sym_duet_dev_type_document_camera] = ACTIONS(1789),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1789),
    [sym_duet_dev_type_hvac] = ACTIONS(1789),
    [sym_duet_dev_type_keypad] = ACTIONS(1789),
    [sym_duet_dev_type_light] = ACTIONS(1789),
    [sym_duet_dev_type_monitor] = ACTIONS(1789),
    [sym_duet_dev_type_motor] = ACTIONS(1789),
    [sym_duet_dev_type_multi_window] = ACTIONS(1789),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1789),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1789),
    [sym_duet_dev_type_receiver] = ACTIONS(1789),
    [sym_duet_dev_type_security_system] = ACTIONS(1789),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1789),
    [sym_duet_dev_type_settop_box] = ACTIONS(1789),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1789),
    [sym_duet_dev_type_switcher] = ACTIONS(1789),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1789),
    [sym_duet_dev_type_tv] = ACTIONS(1789),
    [sym_duet_dev_type_utility] = ACTIONS(1789),
    [sym_duet_dev_type_vcr] = ACTIONS(1789),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1789),
    [sym_duet_dev_type_video_processor] = ACTIONS(1789),
    [sym_duet_dev_type_video_projector] = ACTIONS(1789),
    [sym_duet_dev_type_video_wall] = ACTIONS(1789),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1789),
    [sym_duet_dev_type_weather] = ACTIONS(1789),
    [sym_duet_dev_not_polled] = ACTIONS(1789),
    [sym_duet_dev_polled] = ACTIONS(1789),
    [sym_clkmgr_mode_network] = ACTIONS(1789),
    [sym_clkmgr_mode_standalone] = ACTIONS(1789),
    [sym_normal_standby] = ACTIONS(1789),
    [sym_normal_wake] = ACTIONS(1789),
    [sym_ip_multicast_ttl_option] = ACTIONS(1789),
    [sym_ip_ttl_subnet] = ACTIONS(1789),
    [sym_ip_ttl_site] = ACTIONS(1789),
    [sym_ip_ttl_region] = ACTIONS(1789),
    [sym_ip_ttl_continent] = ACTIONS(1789),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1789),
    [sym_ip_nodelay_on] = ACTIONS(1789),
    [sym_ip_nodelay_off] = ACTIONS(1789),
    [sym_amx_error] = ACTIONS(1789),
    [sym_amx_warning] = ACTIONS(1789),
    [sym_amx_info] = ACTIONS(1789),
    [sym_amx_debug] = ACTIONS(1789),
    [sym_netlinx_login_success] = ACTIONS(1789),
    [sym_netlinx_login_fail] = ACTIONS(1789),
    [sym_netlinx_logout] = ACTIONS(1789),
    [sym_null_str] = ACTIONS(1789),
    [sym_smtp_address] = ACTIONS(1789),
    [sym_smtp_port_number] = ACTIONS(1789),
    [sym_smtp_username] = ACTIONS(1789),
    [sym_smtp_password] = ACTIONS(1789),
    [sym_smtp_require_tls] = ACTIONS(1789),
    [sym_smtp_from] = ACTIONS(1789),
    [sym_smtp_tls_true] = ACTIONS(1789),
    [sym_smtp_tls_false] = ACTIONS(1789),
    [sym_listview_on_row_select_event] = ACTIONS(1789),
    [sym_data_structure_unknown] = ACTIONS(1789),
    [sym_data_structure_datafeed] = ACTIONS(1789),
    [sym_data_structure_datarecordset] = ACTIONS(1789),
    [sym_data_structure_datarecord] = ACTIONS(1789),
    [sym_data_structure_datafield] = ACTIONS(1789),
    [sym_data_max_content_field_count] = ACTIONS(1789),
    [sym_data_max_metadata_field_count] = ACTIONS(1789),
    [sym_data_max_id_length] = ACTIONS(1789),
    [sym_data_max_name_length] = ACTIONS(1789),
    [sym_data_max_description_length] = ACTIONS(1789),
    [sym_data_max_type_length] = ACTIONS(1789),
    [sym_data_max_format_length] = ACTIONS(1789),
    [sym_data_max_source_length] = ACTIONS(1789),
    [sym_data_max_value_length] = ACTIONS(1789),
    [sym_data_max_label_length] = ACTIONS(1789),
    [sym_data_max_url_length] = ACTIONS(1789),
    [sym_max_last_login_info_length] = ACTIONS(1789),
    [sym_data_type_unknown] = ACTIONS(1789),
    [sym_data_type_string] = ACTIONS(1789),
    [sym_data_type_datetime] = ACTIONS(1789),
    [sym_data_type_date] = ACTIONS(1789),
    [sym_data_type_time] = ACTIONS(1789),
    [sym_data_type_image] = ACTIONS(1789),
    [sym_data_format_url] = ACTIONS(1789),
    [sym_data_format_phone] = ACTIONS(1789),
    [sym_data_format_email] = ACTIONS(1789),
    [sym_data_format_iso8601] = ACTIONS(1789),
    [sym_auditlog_priv] = ACTIONS(1789),
    [sym_configuration_priv] = ACTIONS(1789),
    [sym_deviceconfig_priv] = ACTIONS(1789),
    [sym_ftp_priv] = ACTIONS(1789),
    [sym_http_priv] = ACTIONS(1789),
    [sym_networkconfig_priv] = ACTIONS(1789),
    [sym_programport_priv] = ACTIONS(1789),
    [sym_remoteui_priv] = ACTIONS(1789),
    [sym_securitycontrol_priv] = ACTIONS(1789),
    [sym_softwaremanagement_priv] = ACTIONS(1789),
    [sym_terminal_priv] = ACTIONS(1789),
    [sym_tpadmin_priv] = ACTIONS(1789),
    [sym_user1_priv] = ACTIONS(1789),
    [sym_user2_priv] = ACTIONS(1789),
    [sym_user3_priv] = ACTIONS(1789),
    [sym_user4_priv] = ACTIONS(1789),
    [sym_usermanagement_priv] = ACTIONS(1789),
    [sym_valid_account] = ACTIONS(1789),
    [sym_err_name_invalid] = ACTIONS(1789),
    [sym_err_password_invalid] = ACTIONS(1789),
    [sym_err_invalid_account] = ACTIONS(1789),
    [sym_err_no_authorization] = ACTIONS(1789),
    [sym_err_invalid_parameter] = ACTIONS(1789),
    [sym_err_user_locked_out] = ACTIONS(1789),
    [sym_err_user_account_expired] = ACTIONS(1789),
    [sym_err_general_error] = ACTIONS(1789),
    [sym_err_authentication_error] = ACTIONS(1789),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1789),
    [sym_tls_validate_certificate] = ACTIONS(1789),
    [sym_tdata] = ACTIONS(1789),
    [sym_tchannel] = ACTIONS(1789),
    [sym_tlevel] = ACTIONS(1789),
    [sym_tbutton] = ACTIONS(1789),
    [sym_ttimeline] = ACTIONS(1789),
    [sym_tcustom] = ACTIONS(1789),
    [sym_url_struct] = ACTIONS(1789),
    [sym_dns_struct] = ACTIONS(1789),
    [sym_ip_address_struct] = ACTIONS(1789),
    [sym_dev_info_struct] = ACTIONS(1789),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1789),
    [sym_data_feed] = ACTIONS(1789),
    [sym_data_field] = ACTIONS(1789),
    [sym_data_record] = ACTIONS(1789),
    [sym_wc_data_feed] = ACTIONS(1789),
    [sym_wc_data_field] = ACTIONS(1789),
    [sym_wc_data_record] = ACTIONS(1789),
    [sym_last_login_info] = ACTIONS(1789),
    [sym____reserved___] = ACTIONS(1789),
    [sym_date] = ACTIONS(1789),
    [sym_day] = ACTIONS(1789),
    [sym_time] = ACTIONS(1789),
    [sym_ldate] = ACTIONS(1789),
    [sym_push_device] = ACTIONS(1789),
    [sym_push_channel] = ACTIONS(1789),
    [sym_push_devchan] = ACTIONS(1789),
    [sym_release_device] = ACTIONS(1789),
    [sym_release_channel] = ACTIONS(1789),
    [sym_release_devchan] = ACTIONS(1789),
    [sym_master_slot] = ACTIONS(1789),
    [sym_get_pulse_time] = ACTIONS(1789),
    [sym_get_timer] = ACTIONS(1789),
    [sym_master_sn] = ACTIONS(1789),
    [sym_system_number] = ACTIONS(1789),
    [sym_button] = ACTIONS(1789),
    [sym_dv_channel] = ACTIONS(1789),
    [sym_data] = ACTIONS(1789),
    [sym_level] = ACTIONS(1789),
    [sym_timeline] = ACTIONS(1789),
    [sym_channel] = ACTIONS(1789),
    [sym_custom] = ACTIONS(1789),
    [sym_atoi] = ACTIONS(1789),
    [sym_atol] = ACTIONS(1789),
    [sym_atof] = ACTIONS(1789),
    [sym_itoa] = ACTIONS(1789),
    [sym_itohex] = ACTIONS(1789),
    [sym_hextoi] = ACTIONS(1789),
    [sym_ftoa] = ACTIONS(1789),
    [sym_find_string] = ACTIONS(1789),
    [sym_left_string] = ACTIONS(1789),
    [sym_right_string] = ACTIONS(1789),
    [sym_mid_string] = ACTIONS(1789),
    [sym_remove_string] = ACTIONS(1789),
    [sym_get_buffer_string] = ACTIONS(1789),
    [sym_get_buffer_char] = ACTIONS(1789),
    [sym_get_multi_buffer_string] = ACTIONS(1789),
    [sym_length_string] = ACTIONS(1789),
    [sym_max_length_string] = ACTIONS(1789),
    [sym_set_length_string] = ACTIONS(1789),
    [sym_compare_string] = ACTIONS(1789),
    [sym_upper_string] = ACTIONS(1789),
    [sym_lower_string] = ACTIONS(1789),
    [sym_get_last] = ACTIONS(1789),
    [sym_date_to_day] = ACTIONS(1789),
    [sym_date_to_month] = ACTIONS(1789),
    [sym_date_to_year] = ACTIONS(1789),
    [sym_time_to_hour] = ACTIONS(1789),
    [sym_time_to_minute] = ACTIONS(1789),
    [sym_time_to_second] = ACTIONS(1789),
    [sym_day_of_week] = ACTIONS(1789),
    [sym_set_pulse_time] = ACTIONS(1789),
    [sym_set_timer] = ACTIONS(1789),
    [sym_file_open] = ACTIONS(1789),
    [sym_file_close] = ACTIONS(1789),
    [sym_file_read] = ACTIONS(1789),
    [sym_file_read_line] = ACTIONS(1789),
    [sym_file_write] = ACTIONS(1789),
    [sym_file_write_line] = ACTIONS(1789),
    [sym_file_copy] = ACTIONS(1789),
    [sym_file_delete] = ACTIONS(1789),
    [sym_file_rename] = ACTIONS(1789),
    [sym_file_dir] = ACTIONS(1789),
    [sym_file_getdir] = ACTIONS(1789),
    [sym_file_setdir] = ACTIONS(1789),
    [sym_file_create_dir] = ACTIONS(1789),
    [sym_file_remove_dir] = ACTIONS(1789),
    [sym_file_seek] = ACTIONS(1789),
    [sym_length_array] = ACTIONS(1789),
    [sym_max_length_array] = ACTIONS(1789),
    [sym_set_length_array] = ACTIONS(1789),
    [sym_ip_client_open] = ACTIONS(1789),
    [sym_ip_client_close] = ACTIONS(1789),
    [sym_ip_server_open] = ACTIONS(1789),
    [sym_ip_server_close] = ACTIONS(1789),
    [sym_ip_bound_client_open] = ACTIONS(1789),
    [sym_ip_mc_server_open] = ACTIONS(1789),
    [sym_ip_set_option] = ACTIONS(1789),
    [sym_ssh_client_open] = ACTIONS(1789),
    [sym_ssh_client_close] = ACTIONS(1789),
    [sym_tls_client_open] = ACTIONS(1789),
    [sym_tls_client_close] = ACTIONS(1789),
    [sym_do_push] = ACTIONS(1789),
    [sym_do_push_timed] = ACTIONS(1789),
    [sym_do_release] = ACTIONS(1789),
    [sym_device_id] = ACTIONS(1789),
    [sym_device_info] = ACTIONS(1789),
    [sym_device_id_string] = ACTIONS(1789),
    [sym_device_standby] = ACTIONS(1789),
    [sym_device_wake] = ACTIONS(1789),
    [sym_timeline_create] = ACTIONS(1789),
    [sym_timeline_kill] = ACTIONS(1789),
    [sym_timeline_reload] = ACTIONS(1789),
    [sym_timeline_pause] = ACTIONS(1789),
    [sym_timeline_restart] = ACTIONS(1789),
    [sym_timeline_set] = ACTIONS(1789),
    [sym_timeline_get] = ACTIONS(1789),
    [sym_timeline_active] = ACTIONS(1789),
    [sym_timeline_dynamic_id] = ACTIONS(1789),
    [sym_variable_to_string] = ACTIONS(1789),
    [sym_string_to_variable] = ACTIONS(1789),
    [sym_length_variable_to_string] = ACTIONS(1789),
    [sym_type_cast] = ACTIONS(1789),
    [sym_variable_to_xml] = ACTIONS(1789),
    [sym_xml_to_variable] = ACTIONS(1789),
    [sym_length_variable_to_xml] = ACTIONS(1789),
    [sym_redirect_string] = ACTIONS(1789),
    [sym_random_number] = ACTIONS(1789),
    [sym_get_unique_id] = ACTIONS(1789),
    [sym_get_system_number] = ACTIONS(1789),
    [sym_set_system_number] = ACTIONS(1789),
    [sym_get_serial_number] = ACTIONS(1789),
    [sym_get_master_build] = ACTIONS(1789),
    [sym_set_validation_code] = ACTIONS(1789),
    [sym_reboot] = ACTIONS(1789),
    [sym_load_duet_module] = ACTIONS(1789),
    [sym_unload_duet_module] = ACTIONS(1789),
    [sym_duet_mem_size_set] = ACTIONS(1789),
    [sym_duet_mem_size_get] = ACTIONS(1789),
    [sym_dynamic_application_device] = ACTIONS(1789),
    [sym_dynamic_polled_port] = ACTIONS(1789),
    [sym_static_port_binding] = ACTIONS(1789),
    [sym_static_ip_binding] = ACTIONS(1789),
    [sym_get_dns_list] = ACTIONS(1789),
    [sym_set_dns_list] = ACTIONS(1789),
    [sym_get_ip_address] = ACTIONS(1789),
    [sym_set_ip_address] = ACTIONS(1789),
    [sym_get_url_list] = ACTIONS(1789),
    [sym_add_url_entry] = ACTIONS(1789),
    [sym_delete_url_entry] = ACTIONS(1789),
    [sym_abs_value] = ACTIONS(1789),
    [sym_max_value] = ACTIONS(1789),
    [sym_min_value] = ACTIONS(1789),
    [sym_exp_value] = ACTIONS(1789),
    [sym_log_value] = ACTIONS(1789),
    [sym_log10_value] = ACTIONS(1789),
    [sym_power_value] = ACTIONS(1789),
    [sym_sqrt_value] = ACTIONS(1789),
    [sym_raw_be] = ACTIONS(1789),
    [sym_raw_le] = ACTIONS(1789),
    [sym_internal_queue_size_set] = ACTIONS(1789),
    [sym_internal_queue_size_get] = ACTIONS(1789),
    [sym_internal_threshold_set] = ACTIONS(1789),
    [sym_internal_threshold_get] = ACTIONS(1789),
    [sym_rebuilt_event] = ACTIONS(1789),
    [sym_do_custom_event] = ACTIONS(1789),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1789),
    [sym_clkmgr_set_clk_source] = ACTIONS(1789),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1789),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1789),
    [sym_clkmgr_get_timezone] = ACTIONS(1789),
    [sym_clkmgr_set_timezone] = ACTIONS(1789),
    [sym_clkmgr_get_resync_period] = ACTIONS(1789),
    [sym_clkmgr_set_resync_period] = ACTIONS(1789),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1789),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1789),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1789),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1789),
    [sym_clkmgr_get_timeservers] = ACTIONS(1789),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1789),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1789),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1789),
    [sym_set_log_level] = ACTIONS(1789),
    [sym_get_log_level] = ACTIONS(1789),
    [sym_amx_log] = ACTIONS(1789),
    [sym_smtp_server_config_set] = ACTIONS(1789),
    [sym_smtp_server_config_get] = ACTIONS(1789),
    [sym_smtp_send] = ACTIONS(1789),
    [sym_validate_netlinx_account] = ACTIONS(1789),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1789),
    [sym_authenticate_certificate] = ACTIONS(1789),
    [sym_audit_netlinx_session_event] = ACTIONS(1789),
    [sym_audit_netlinx_generic_event] = ACTIONS(1789),
    [sym_get_available_flash_disk_space] = ACTIONS(1789),
    [sym_get_max_flash_disk_space] = ACTIONS(1789),
    [sym_data_create_feed] = ACTIONS(1789),
    [sym_data_delete_feed] = ACTIONS(1789),
    [sym_data_publish_feed] = ACTIONS(1789),
    [sym_data_get_published_feed] = ACTIONS(1789),
    [sym_data_add_record] = ACTIONS(1789),
    [sym_data_get_event_record] = ACTIONS(1789),
    [sym__wc_data_create_feed] = ACTIONS(1789),
    [sym__wc_data_add_record] = ACTIONS(1789),
    [sym__wc_data_get_event_record] = ACTIONS(1789),
    [sym_format] = ACTIONS(1789),
    [sym_astro_clock] = ACTIONS(1789),
    [sym_set_outdoor_temperature] = ACTIONS(1789),
    [sym_set_virtual_level_count] = ACTIONS(1789),
    [sym_set_virtual_channel_count] = ACTIONS(1789),
    [sym_set_virtual_port_count] = ACTIONS(1789),
    [sym__wc_encode_int] = ACTIONS(1789),
    [sym__wc_decode_int] = ACTIONS(1789),
    [sym__wc_int] = ACTIONS(1789),
    [sym__wc_to_ch_int] = ACTIONS(1789),
    [sym__ch_to_wc_int] = ACTIONS(1789),
    [sym__wc_find_string_int] = ACTIONS(1789),
    [sym__wc_left_string_int] = ACTIONS(1789),
    [sym__wc_lower_string_int] = ACTIONS(1789),
    [sym__wc_mid_string_int] = ACTIONS(1789),
    [sym__wc_remove_string_int] = ACTIONS(1789),
    [sym__wc_right_string_int] = ACTIONS(1789),
    [sym__wc_upper_string_int] = ACTIONS(1789),
    [sym__wc_compare_string_int] = ACTIONS(1789),
    [sym__wc_get_buffer_char_int] = ACTIONS(1789),
    [sym__wc_get_buffer_string_int] = ACTIONS(1789),
    [sym__wc_concat_string_int] = ACTIONS(1789),
    [sym__wc_file_open_int] = ACTIONS(1789),
    [sym__wc_file_close_int] = ACTIONS(1789),
    [sym__wc_file_read_int] = ACTIONS(1789),
    [sym__wc_file_read_line_int] = ACTIONS(1789),
    [sym__wc_file_write_int] = ACTIONS(1789),
    [sym__wc_file_write_line_int] = ACTIONS(1789),
    [aux_sym_preproc_include_token1] = ACTIONS(1789),
    [aux_sym_preproc_define_token1] = ACTIONS(1791),
    [aux_sym_preproc_warn_token1] = ACTIONS(1791),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1791),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1791),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1791),
    [anon_sym_LBRACK] = ACTIONS(1791),
    [anon_sym_LBRACE] = ACTIONS(1791),
    [anon_sym_RBRACE] = ACTIONS(1791),
    [anon_sym_LPAREN] = ACTIONS(1789),
    [anon_sym_BANG] = ACTIONS(1791),
    [anon_sym_TILDE] = ACTIONS(1791),
    [anon_sym_DASH] = ACTIONS(1789),
    [anon_sym_PLUS] = ACTIONS(1789),
    [anon_sym_DASH_DASH] = ACTIONS(1791),
    [anon_sym_PLUS_PLUS] = ACTIONS(1791),
    [anon_sym_DQUOTE] = ACTIONS(1791),
    [anon_sym_SQUOTE] = ACTIONS(1791),
    [sym_number_literal] = ACTIONS(1791),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(465)] = {
    [sym_identifier] = ACTIONS(1789),
    [sym_define_call_keyword] = ACTIONS(1789),
    [sym_define_function_keyword] = ACTIONS(1789),
    [sym_define_library_function_keyword] = ACTIONS(1789),
    [sym_define_module_keyword] = ACTIONS(1789),
    [sym_char_keyword] = ACTIONS(1789),
    [sym_widechar_keyword] = ACTIONS(1789),
    [sym_integer_keyword] = ACTIONS(1789),
    [sym_sinteger_keyword] = ACTIONS(1789),
    [sym_long_keyword] = ACTIONS(1789),
    [sym_slong_keyword] = ACTIONS(1789),
    [sym_float_keyword] = ACTIONS(1789),
    [sym_double_keyword] = ACTIONS(1789),
    [sym_variant_keyword] = ACTIONS(1789),
    [sym_variantarray_keyword] = ACTIONS(1789),
    [sym_local_var_keyword] = ACTIONS(1789),
    [sym_stack_var_keyword] = ACTIONS(1789),
    [sym_constant_keyword] = ACTIONS(1789),
    [sym_volatile_keyword] = ACTIONS(1789),
    [sym_non_volatile_keyword] = ACTIONS(1789),
    [sym_persistent_keyword] = ACTIONS(1789),
    [sym_dev_keyword] = ACTIONS(1789),
    [sym_devlev_keyword] = ACTIONS(1789),
    [sym_devchan_keyword] = ACTIONS(1789),
    [sym_if_keyword] = ACTIONS(1789),
    [sym_else_keyword] = ACTIONS(1789),
    [sym_switch_keyword] = ACTIONS(1789),
    [sym_case_keyword] = ACTIONS(1789),
    [sym_default_keyword] = ACTIONS(1789),
    [sym_while_keyword] = ACTIONS(1789),
    [sym_for_keyword] = ACTIONS(1789),
    [sym_break_keyword] = ACTIONS(1789),
    [sym_continue_keyword] = ACTIONS(1789),
    [sym_return_keyword] = ACTIONS(1789),
    [sym_select_keyword] = ACTIONS(1789),
    [sym_active_keyword] = ACTIONS(1789),
    [sym_struct_keyword] = ACTIONS(1789),
    [sym_structure_keyword] = ACTIONS(1789),
    [aux_sym_on_keyword_token1] = ACTIONS(1789),
    [aux_sym_off_keyword_token1] = ACTIONS(1789),
    [sym_send_level_keyword] = ACTIONS(1789),
    [sym_send_string_keyword] = ACTIONS(1789),
    [sym_send_command_keyword] = ACTIONS(1789),
    [sym_clear_buffer_keyword] = ACTIONS(1789),
    [sym_create_buffer_keyword] = ACTIONS(1789),
    [sym_create_multi_buffer_keyword] = ACTIONS(1789),
    [sym_call_keyword] = ACTIONS(1789),
    [sym_system_call_keyword] = ACTIONS(1789),
    [sym_devchan_to_keyword] = ACTIONS(1789),
    [sym_devchan_min_to_keyword] = ACTIONS(1789),
    [sym_devchan_total_off_keyword] = ACTIONS(1789),
    [sym_devchan_pulse_keyword] = ACTIONS(1789),
    [sym_wait_keyword] = ACTIONS(1789),
    [sym_cancel_wait_keyword] = ACTIONS(1789),
    [sym_wait_until_keyword] = ACTIONS(1789),
    [sym_cancel_wait_until_keyword] = ACTIONS(1789),
    [sym_cancel_all_wait_keyword] = ACTIONS(1789),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1789),
    [sym_not] = ACTIONS(1789),
    [sym_bnot] = ACTIONS(1789),
    [sym___file__] = ACTIONS(1789),
    [sym___date__] = ACTIONS(1789),
    [sym___ldate__] = ACTIONS(1789),
    [sym___time__] = ACTIONS(1789),
    [sym___line__] = ACTIONS(1789),
    [sym___name__] = ACTIONS(1789),
    [sym___netlinx__] = ACTIONS(1789),
    [sym_netlinx_axi_version] = ACTIONS(1789),
    [sym_url_flg_tcp] = ACTIONS(1789),
    [sym_url_flg_accinfopresent] = ACTIONS(1789),
    [sym_url_flg_temp] = ACTIONS(1789),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1789),
    [sym_url_flg_stat_mask] = ACTIONS(1789),
    [sym_url_flg_stat_lookup] = ACTIONS(1789),
    [sym_url_flg_stat_connecting] = ACTIONS(1789),
    [sym_url_flg_stat_waiting] = ACTIONS(1789),
    [sym_url_flg_stat_connected] = ACTIONS(1789),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1789),
    [sym_first_virtual_device] = ACTIONS(1789),
    [sym_first_local_port] = ACTIONS(1789),
    [sym_dynamic_virtual_device] = ACTIONS(1789),
    [sym_timeline_once] = ACTIONS(1789),
    [sym_timeline_repeat] = ACTIONS(1789),
    [sym_timeline_absolute] = ACTIONS(1789),
    [sym_timeline_relative] = ACTIONS(1789),
    [sym_do_push_timed_infinite] = ACTIONS(1789),
    [sym_source_type_no_address] = ACTIONS(1789),
    [sym_source_type_neuron_id] = ACTIONS(1789),
    [sym_source_type_ip_address] = ACTIONS(1789),
    [sym_source_type_axlink] = ACTIONS(1789),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1789),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1789),
    [sym_source_type_ip_socket_address] = ACTIONS(1789),
    [sym_source_type_rs232] = ACTIONS(1789),
    [sym_source_type_internal] = ACTIONS(1789),
    [sym_source_type_mac_address] = ACTIONS(1789),
    [sym_source_type_ipv4_port] = ACTIONS(1789),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1789),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1789),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1789),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1789),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1789),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1789),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1789),
    [sym_true] = ACTIONS(1789),
    [sym_false] = ACTIONS(1789),
    [sym_file_read_only] = ACTIONS(1789),
    [sym_file_rw_new] = ACTIONS(1789),
    [sym_file_rw_append] = ACTIONS(1789),
    [sym_ip_tcp] = ACTIONS(1789),
    [sym_ip_udp] = ACTIONS(1789),
    [sym_ip_udp_2way] = ACTIONS(1789),
    [sym_xml_encode_types] = ACTIONS(1789),
    [sym_xml_encode_char_as_list] = ACTIONS(1789),
    [sym_xml_encode_le] = ACTIONS(1789),
    [sym_xml_decode_types] = ACTIONS(1789),
    [sym_xml_decode_no_preserve] = ACTIONS(1789),
    [sym_duet_dev_type_display_device] = ACTIONS(1789),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1789),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1789),
    [sym_duet_dev_type_io_device] = ACTIONS(1789),
    [sym_duet_dev_type_relay_device] = ACTIONS(1789),
    [sym_duet_dev_type_ups] = ACTIONS(1789),
    [sym_duet_dev_type_amplifier] = ACTIONS(1789),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1789),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1789),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1789),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1789),
    [sym_duet_dev_type_camera] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1789),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1789),
    [sym_duet_dev_type_dss] = ACTIONS(1789),
    [sym_duet_dev_type_dvr] = ACTIONS(1789),
    [sym_duet_dev_type_disc_device] = ACTIONS(1789),
    [sym_duet_dev_type_document_camera] = ACTIONS(1789),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1789),
    [sym_duet_dev_type_hvac] = ACTIONS(1789),
    [sym_duet_dev_type_keypad] = ACTIONS(1789),
    [sym_duet_dev_type_light] = ACTIONS(1789),
    [sym_duet_dev_type_monitor] = ACTIONS(1789),
    [sym_duet_dev_type_motor] = ACTIONS(1789),
    [sym_duet_dev_type_multi_window] = ACTIONS(1789),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1789),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1789),
    [sym_duet_dev_type_receiver] = ACTIONS(1789),
    [sym_duet_dev_type_security_system] = ACTIONS(1789),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1789),
    [sym_duet_dev_type_settop_box] = ACTIONS(1789),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1789),
    [sym_duet_dev_type_switcher] = ACTIONS(1789),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1789),
    [sym_duet_dev_type_tv] = ACTIONS(1789),
    [sym_duet_dev_type_utility] = ACTIONS(1789),
    [sym_duet_dev_type_vcr] = ACTIONS(1789),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1789),
    [sym_duet_dev_type_video_processor] = ACTIONS(1789),
    [sym_duet_dev_type_video_projector] = ACTIONS(1789),
    [sym_duet_dev_type_video_wall] = ACTIONS(1789),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1789),
    [sym_duet_dev_type_weather] = ACTIONS(1789),
    [sym_duet_dev_not_polled] = ACTIONS(1789),
    [sym_duet_dev_polled] = ACTIONS(1789),
    [sym_clkmgr_mode_network] = ACTIONS(1789),
    [sym_clkmgr_mode_standalone] = ACTIONS(1789),
    [sym_normal_standby] = ACTIONS(1789),
    [sym_normal_wake] = ACTIONS(1789),
    [sym_ip_multicast_ttl_option] = ACTIONS(1789),
    [sym_ip_ttl_subnet] = ACTIONS(1789),
    [sym_ip_ttl_site] = ACTIONS(1789),
    [sym_ip_ttl_region] = ACTIONS(1789),
    [sym_ip_ttl_continent] = ACTIONS(1789),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1789),
    [sym_ip_nodelay_on] = ACTIONS(1789),
    [sym_ip_nodelay_off] = ACTIONS(1789),
    [sym_amx_error] = ACTIONS(1789),
    [sym_amx_warning] = ACTIONS(1789),
    [sym_amx_info] = ACTIONS(1789),
    [sym_amx_debug] = ACTIONS(1789),
    [sym_netlinx_login_success] = ACTIONS(1789),
    [sym_netlinx_login_fail] = ACTIONS(1789),
    [sym_netlinx_logout] = ACTIONS(1789),
    [sym_null_str] = ACTIONS(1789),
    [sym_smtp_address] = ACTIONS(1789),
    [sym_smtp_port_number] = ACTIONS(1789),
    [sym_smtp_username] = ACTIONS(1789),
    [sym_smtp_password] = ACTIONS(1789),
    [sym_smtp_require_tls] = ACTIONS(1789),
    [sym_smtp_from] = ACTIONS(1789),
    [sym_smtp_tls_true] = ACTIONS(1789),
    [sym_smtp_tls_false] = ACTIONS(1789),
    [sym_listview_on_row_select_event] = ACTIONS(1789),
    [sym_data_structure_unknown] = ACTIONS(1789),
    [sym_data_structure_datafeed] = ACTIONS(1789),
    [sym_data_structure_datarecordset] = ACTIONS(1789),
    [sym_data_structure_datarecord] = ACTIONS(1789),
    [sym_data_structure_datafield] = ACTIONS(1789),
    [sym_data_max_content_field_count] = ACTIONS(1789),
    [sym_data_max_metadata_field_count] = ACTIONS(1789),
    [sym_data_max_id_length] = ACTIONS(1789),
    [sym_data_max_name_length] = ACTIONS(1789),
    [sym_data_max_description_length] = ACTIONS(1789),
    [sym_data_max_type_length] = ACTIONS(1789),
    [sym_data_max_format_length] = ACTIONS(1789),
    [sym_data_max_source_length] = ACTIONS(1789),
    [sym_data_max_value_length] = ACTIONS(1789),
    [sym_data_max_label_length] = ACTIONS(1789),
    [sym_data_max_url_length] = ACTIONS(1789),
    [sym_max_last_login_info_length] = ACTIONS(1789),
    [sym_data_type_unknown] = ACTIONS(1789),
    [sym_data_type_string] = ACTIONS(1789),
    [sym_data_type_datetime] = ACTIONS(1789),
    [sym_data_type_date] = ACTIONS(1789),
    [sym_data_type_time] = ACTIONS(1789),
    [sym_data_type_image] = ACTIONS(1789),
    [sym_data_format_url] = ACTIONS(1789),
    [sym_data_format_phone] = ACTIONS(1789),
    [sym_data_format_email] = ACTIONS(1789),
    [sym_data_format_iso8601] = ACTIONS(1789),
    [sym_auditlog_priv] = ACTIONS(1789),
    [sym_configuration_priv] = ACTIONS(1789),
    [sym_deviceconfig_priv] = ACTIONS(1789),
    [sym_ftp_priv] = ACTIONS(1789),
    [sym_http_priv] = ACTIONS(1789),
    [sym_networkconfig_priv] = ACTIONS(1789),
    [sym_programport_priv] = ACTIONS(1789),
    [sym_remoteui_priv] = ACTIONS(1789),
    [sym_securitycontrol_priv] = ACTIONS(1789),
    [sym_softwaremanagement_priv] = ACTIONS(1789),
    [sym_terminal_priv] = ACTIONS(1789),
    [sym_tpadmin_priv] = ACTIONS(1789),
    [sym_user1_priv] = ACTIONS(1789),
    [sym_user2_priv] = ACTIONS(1789),
    [sym_user3_priv] = ACTIONS(1789),
    [sym_user4_priv] = ACTIONS(1789),
    [sym_usermanagement_priv] = ACTIONS(1789),
    [sym_valid_account] = ACTIONS(1789),
    [sym_err_name_invalid] = ACTIONS(1789),
    [sym_err_password_invalid] = ACTIONS(1789),
    [sym_err_invalid_account] = ACTIONS(1789),
    [sym_err_no_authorization] = ACTIONS(1789),
    [sym_err_invalid_parameter] = ACTIONS(1789),
    [sym_err_user_locked_out] = ACTIONS(1789),
    [sym_err_user_account_expired] = ACTIONS(1789),
    [sym_err_general_error] = ACTIONS(1789),
    [sym_err_authentication_error] = ACTIONS(1789),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1789),
    [sym_tls_validate_certificate] = ACTIONS(1789),
    [sym_tdata] = ACTIONS(1789),
    [sym_tchannel] = ACTIONS(1789),
    [sym_tlevel] = ACTIONS(1789),
    [sym_tbutton] = ACTIONS(1789),
    [sym_ttimeline] = ACTIONS(1789),
    [sym_tcustom] = ACTIONS(1789),
    [sym_url_struct] = ACTIONS(1789),
    [sym_dns_struct] = ACTIONS(1789),
    [sym_ip_address_struct] = ACTIONS(1789),
    [sym_dev_info_struct] = ACTIONS(1789),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1789),
    [sym_data_feed] = ACTIONS(1789),
    [sym_data_field] = ACTIONS(1789),
    [sym_data_record] = ACTIONS(1789),
    [sym_wc_data_feed] = ACTIONS(1789),
    [sym_wc_data_field] = ACTIONS(1789),
    [sym_wc_data_record] = ACTIONS(1789),
    [sym_last_login_info] = ACTIONS(1789),
    [sym____reserved___] = ACTIONS(1789),
    [sym_date] = ACTIONS(1789),
    [sym_day] = ACTIONS(1789),
    [sym_time] = ACTIONS(1789),
    [sym_ldate] = ACTIONS(1789),
    [sym_push_device] = ACTIONS(1789),
    [sym_push_channel] = ACTIONS(1789),
    [sym_push_devchan] = ACTIONS(1789),
    [sym_release_device] = ACTIONS(1789),
    [sym_release_channel] = ACTIONS(1789),
    [sym_release_devchan] = ACTIONS(1789),
    [sym_master_slot] = ACTIONS(1789),
    [sym_get_pulse_time] = ACTIONS(1789),
    [sym_get_timer] = ACTIONS(1789),
    [sym_master_sn] = ACTIONS(1789),
    [sym_system_number] = ACTIONS(1789),
    [sym_button] = ACTIONS(1789),
    [sym_dv_channel] = ACTIONS(1789),
    [sym_data] = ACTIONS(1789),
    [sym_level] = ACTIONS(1789),
    [sym_timeline] = ACTIONS(1789),
    [sym_channel] = ACTIONS(1789),
    [sym_custom] = ACTIONS(1789),
    [sym_atoi] = ACTIONS(1789),
    [sym_atol] = ACTIONS(1789),
    [sym_atof] = ACTIONS(1789),
    [sym_itoa] = ACTIONS(1789),
    [sym_itohex] = ACTIONS(1789),
    [sym_hextoi] = ACTIONS(1789),
    [sym_ftoa] = ACTIONS(1789),
    [sym_find_string] = ACTIONS(1789),
    [sym_left_string] = ACTIONS(1789),
    [sym_right_string] = ACTIONS(1789),
    [sym_mid_string] = ACTIONS(1789),
    [sym_remove_string] = ACTIONS(1789),
    [sym_get_buffer_string] = ACTIONS(1789),
    [sym_get_buffer_char] = ACTIONS(1789),
    [sym_get_multi_buffer_string] = ACTIONS(1789),
    [sym_length_string] = ACTIONS(1789),
    [sym_max_length_string] = ACTIONS(1789),
    [sym_set_length_string] = ACTIONS(1789),
    [sym_compare_string] = ACTIONS(1789),
    [sym_upper_string] = ACTIONS(1789),
    [sym_lower_string] = ACTIONS(1789),
    [sym_get_last] = ACTIONS(1789),
    [sym_date_to_day] = ACTIONS(1789),
    [sym_date_to_month] = ACTIONS(1789),
    [sym_date_to_year] = ACTIONS(1789),
    [sym_time_to_hour] = ACTIONS(1789),
    [sym_time_to_minute] = ACTIONS(1789),
    [sym_time_to_second] = ACTIONS(1789),
    [sym_day_of_week] = ACTIONS(1789),
    [sym_set_pulse_time] = ACTIONS(1789),
    [sym_set_timer] = ACTIONS(1789),
    [sym_file_open] = ACTIONS(1789),
    [sym_file_close] = ACTIONS(1789),
    [sym_file_read] = ACTIONS(1789),
    [sym_file_read_line] = ACTIONS(1789),
    [sym_file_write] = ACTIONS(1789),
    [sym_file_write_line] = ACTIONS(1789),
    [sym_file_copy] = ACTIONS(1789),
    [sym_file_delete] = ACTIONS(1789),
    [sym_file_rename] = ACTIONS(1789),
    [sym_file_dir] = ACTIONS(1789),
    [sym_file_getdir] = ACTIONS(1789),
    [sym_file_setdir] = ACTIONS(1789),
    [sym_file_create_dir] = ACTIONS(1789),
    [sym_file_remove_dir] = ACTIONS(1789),
    [sym_file_seek] = ACTIONS(1789),
    [sym_length_array] = ACTIONS(1789),
    [sym_max_length_array] = ACTIONS(1789),
    [sym_set_length_array] = ACTIONS(1789),
    [sym_ip_client_open] = ACTIONS(1789),
    [sym_ip_client_close] = ACTIONS(1789),
    [sym_ip_server_open] = ACTIONS(1789),
    [sym_ip_server_close] = ACTIONS(1789),
    [sym_ip_bound_client_open] = ACTIONS(1789),
    [sym_ip_mc_server_open] = ACTIONS(1789),
    [sym_ip_set_option] = ACTIONS(1789),
    [sym_ssh_client_open] = ACTIONS(1789),
    [sym_ssh_client_close] = ACTIONS(1789),
    [sym_tls_client_open] = ACTIONS(1789),
    [sym_tls_client_close] = ACTIONS(1789),
    [sym_do_push] = ACTIONS(1789),
    [sym_do_push_timed] = ACTIONS(1789),
    [sym_do_release] = ACTIONS(1789),
    [sym_device_id] = ACTIONS(1789),
    [sym_device_info] = ACTIONS(1789),
    [sym_device_id_string] = ACTIONS(1789),
    [sym_device_standby] = ACTIONS(1789),
    [sym_device_wake] = ACTIONS(1789),
    [sym_timeline_create] = ACTIONS(1789),
    [sym_timeline_kill] = ACTIONS(1789),
    [sym_timeline_reload] = ACTIONS(1789),
    [sym_timeline_pause] = ACTIONS(1789),
    [sym_timeline_restart] = ACTIONS(1789),
    [sym_timeline_set] = ACTIONS(1789),
    [sym_timeline_get] = ACTIONS(1789),
    [sym_timeline_active] = ACTIONS(1789),
    [sym_timeline_dynamic_id] = ACTIONS(1789),
    [sym_variable_to_string] = ACTIONS(1789),
    [sym_string_to_variable] = ACTIONS(1789),
    [sym_length_variable_to_string] = ACTIONS(1789),
    [sym_type_cast] = ACTIONS(1789),
    [sym_variable_to_xml] = ACTIONS(1789),
    [sym_xml_to_variable] = ACTIONS(1789),
    [sym_length_variable_to_xml] = ACTIONS(1789),
    [sym_redirect_string] = ACTIONS(1789),
    [sym_random_number] = ACTIONS(1789),
    [sym_get_unique_id] = ACTIONS(1789),
    [sym_get_system_number] = ACTIONS(1789),
    [sym_set_system_number] = ACTIONS(1789),
    [sym_get_serial_number] = ACTIONS(1789),
    [sym_get_master_build] = ACTIONS(1789),
    [sym_set_validation_code] = ACTIONS(1789),
    [sym_reboot] = ACTIONS(1789),
    [sym_load_duet_module] = ACTIONS(1789),
    [sym_unload_duet_module] = ACTIONS(1789),
    [sym_duet_mem_size_set] = ACTIONS(1789),
    [sym_duet_mem_size_get] = ACTIONS(1789),
    [sym_dynamic_application_device] = ACTIONS(1789),
    [sym_dynamic_polled_port] = ACTIONS(1789),
    [sym_static_port_binding] = ACTIONS(1789),
    [sym_static_ip_binding] = ACTIONS(1789),
    [sym_get_dns_list] = ACTIONS(1789),
    [sym_set_dns_list] = ACTIONS(1789),
    [sym_get_ip_address] = ACTIONS(1789),
    [sym_set_ip_address] = ACTIONS(1789),
    [sym_get_url_list] = ACTIONS(1789),
    [sym_add_url_entry] = ACTIONS(1789),
    [sym_delete_url_entry] = ACTIONS(1789),
    [sym_abs_value] = ACTIONS(1789),
    [sym_max_value] = ACTIONS(1789),
    [sym_min_value] = ACTIONS(1789),
    [sym_exp_value] = ACTIONS(1789),
    [sym_log_value] = ACTIONS(1789),
    [sym_log10_value] = ACTIONS(1789),
    [sym_power_value] = ACTIONS(1789),
    [sym_sqrt_value] = ACTIONS(1789),
    [sym_raw_be] = ACTIONS(1789),
    [sym_raw_le] = ACTIONS(1789),
    [sym_internal_queue_size_set] = ACTIONS(1789),
    [sym_internal_queue_size_get] = ACTIONS(1789),
    [sym_internal_threshold_set] = ACTIONS(1789),
    [sym_internal_threshold_get] = ACTIONS(1789),
    [sym_rebuilt_event] = ACTIONS(1789),
    [sym_do_custom_event] = ACTIONS(1789),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1789),
    [sym_clkmgr_set_clk_source] = ACTIONS(1789),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1789),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1789),
    [sym_clkmgr_get_timezone] = ACTIONS(1789),
    [sym_clkmgr_set_timezone] = ACTIONS(1789),
    [sym_clkmgr_get_resync_period] = ACTIONS(1789),
    [sym_clkmgr_set_resync_period] = ACTIONS(1789),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1789),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1789),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1789),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1789),
    [sym_clkmgr_get_timeservers] = ACTIONS(1789),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1789),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1789),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1789),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1789),
    [sym_set_log_level] = ACTIONS(1789),
    [sym_get_log_level] = ACTIONS(1789),
    [sym_amx_log] = ACTIONS(1789),
    [sym_smtp_server_config_set] = ACTIONS(1789),
    [sym_smtp_server_config_get] = ACTIONS(1789),
    [sym_smtp_send] = ACTIONS(1789),
    [sym_validate_netlinx_account] = ACTIONS(1789),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1789),
    [sym_authenticate_certificate] = ACTIONS(1789),
    [sym_audit_netlinx_session_event] = ACTIONS(1789),
    [sym_audit_netlinx_generic_event] = ACTIONS(1789),
    [sym_get_available_flash_disk_space] = ACTIONS(1789),
    [sym_get_max_flash_disk_space] = ACTIONS(1789),
    [sym_data_create_feed] = ACTIONS(1789),
    [sym_data_delete_feed] = ACTIONS(1789),
    [sym_data_publish_feed] = ACTIONS(1789),
    [sym_data_get_published_feed] = ACTIONS(1789),
    [sym_data_add_record] = ACTIONS(1789),
    [sym_data_get_event_record] = ACTIONS(1789),
    [sym__wc_data_create_feed] = ACTIONS(1789),
    [sym__wc_data_add_record] = ACTIONS(1789),
    [sym__wc_data_get_event_record] = ACTIONS(1789),
    [sym_format] = ACTIONS(1789),
    [sym_astro_clock] = ACTIONS(1789),
    [sym_set_outdoor_temperature] = ACTIONS(1789),
    [sym_set_virtual_level_count] = ACTIONS(1789),
    [sym_set_virtual_channel_count] = ACTIONS(1789),
    [sym_set_virtual_port_count] = ACTIONS(1789),
    [sym__wc_encode_int] = ACTIONS(1789),
    [sym__wc_decode_int] = ACTIONS(1789),
    [sym__wc_int] = ACTIONS(1789),
    [sym__wc_to_ch_int] = ACTIONS(1789),
    [sym__ch_to_wc_int] = ACTIONS(1789),
    [sym__wc_find_string_int] = ACTIONS(1789),
    [sym__wc_left_string_int] = ACTIONS(1789),
    [sym__wc_lower_string_int] = ACTIONS(1789),
    [sym__wc_mid_string_int] = ACTIONS(1789),
    [sym__wc_remove_string_int] = ACTIONS(1789),
    [sym__wc_right_string_int] = ACTIONS(1789),
    [sym__wc_upper_string_int] = ACTIONS(1789),
    [sym__wc_compare_string_int] = ACTIONS(1789),
    [sym__wc_get_buffer_char_int] = ACTIONS(1789),
    [sym__wc_get_buffer_string_int] = ACTIONS(1789),
    [sym__wc_concat_string_int] = ACTIONS(1789),
    [sym__wc_file_open_int] = ACTIONS(1789),
    [sym__wc_file_close_int] = ACTIONS(1789),
    [sym__wc_file_read_int] = ACTIONS(1789),
    [sym__wc_file_read_line_int] = ACTIONS(1789),
    [sym__wc_file_write_int] = ACTIONS(1789),
    [sym__wc_file_write_line_int] = ACTIONS(1789),
    [aux_sym_preproc_include_token1] = ACTIONS(1789),
    [aux_sym_preproc_define_token1] = ACTIONS(1791),
    [aux_sym_preproc_warn_token1] = ACTIONS(1791),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1791),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1791),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1791),
    [anon_sym_LBRACK] = ACTIONS(1791),
    [anon_sym_LBRACE] = ACTIONS(1791),
    [anon_sym_RBRACE] = ACTIONS(1791),
    [anon_sym_LPAREN] = ACTIONS(1789),
    [anon_sym_BANG] = ACTIONS(1791),
    [anon_sym_TILDE] = ACTIONS(1791),
    [anon_sym_DASH] = ACTIONS(1789),
    [anon_sym_PLUS] = ACTIONS(1789),
    [anon_sym_DASH_DASH] = ACTIONS(1791),
    [anon_sym_PLUS_PLUS] = ACTIONS(1791),
    [anon_sym_DQUOTE] = ACTIONS(1791),
    [anon_sym_SQUOTE] = ACTIONS(1791),
    [sym_number_literal] = ACTIONS(1791),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(466)] = {
    [sym_identifier] = ACTIONS(1793),
    [sym_define_call_keyword] = ACTIONS(1793),
    [sym_define_function_keyword] = ACTIONS(1793),
    [sym_define_library_function_keyword] = ACTIONS(1793),
    [sym_define_module_keyword] = ACTIONS(1793),
    [sym_char_keyword] = ACTIONS(1793),
    [sym_widechar_keyword] = ACTIONS(1793),
    [sym_integer_keyword] = ACTIONS(1793),
    [sym_sinteger_keyword] = ACTIONS(1793),
    [sym_long_keyword] = ACTIONS(1793),
    [sym_slong_keyword] = ACTIONS(1793),
    [sym_float_keyword] = ACTIONS(1793),
    [sym_double_keyword] = ACTIONS(1793),
    [sym_variant_keyword] = ACTIONS(1793),
    [sym_variantarray_keyword] = ACTIONS(1793),
    [sym_local_var_keyword] = ACTIONS(1793),
    [sym_stack_var_keyword] = ACTIONS(1793),
    [sym_constant_keyword] = ACTIONS(1793),
    [sym_volatile_keyword] = ACTIONS(1793),
    [sym_non_volatile_keyword] = ACTIONS(1793),
    [sym_persistent_keyword] = ACTIONS(1793),
    [sym_dev_keyword] = ACTIONS(1793),
    [sym_devlev_keyword] = ACTIONS(1793),
    [sym_devchan_keyword] = ACTIONS(1793),
    [sym_if_keyword] = ACTIONS(1793),
    [sym_else_keyword] = ACTIONS(1793),
    [sym_switch_keyword] = ACTIONS(1793),
    [sym_case_keyword] = ACTIONS(1793),
    [sym_default_keyword] = ACTIONS(1793),
    [sym_while_keyword] = ACTIONS(1793),
    [sym_for_keyword] = ACTIONS(1793),
    [sym_break_keyword] = ACTIONS(1793),
    [sym_continue_keyword] = ACTIONS(1793),
    [sym_return_keyword] = ACTIONS(1793),
    [sym_select_keyword] = ACTIONS(1793),
    [sym_active_keyword] = ACTIONS(1793),
    [sym_struct_keyword] = ACTIONS(1793),
    [sym_structure_keyword] = ACTIONS(1793),
    [aux_sym_on_keyword_token1] = ACTIONS(1793),
    [aux_sym_off_keyword_token1] = ACTIONS(1793),
    [sym_send_level_keyword] = ACTIONS(1793),
    [sym_send_string_keyword] = ACTIONS(1793),
    [sym_send_command_keyword] = ACTIONS(1793),
    [sym_clear_buffer_keyword] = ACTIONS(1793),
    [sym_create_buffer_keyword] = ACTIONS(1793),
    [sym_create_multi_buffer_keyword] = ACTIONS(1793),
    [sym_call_keyword] = ACTIONS(1793),
    [sym_system_call_keyword] = ACTIONS(1793),
    [sym_devchan_to_keyword] = ACTIONS(1793),
    [sym_devchan_min_to_keyword] = ACTIONS(1793),
    [sym_devchan_total_off_keyword] = ACTIONS(1793),
    [sym_devchan_pulse_keyword] = ACTIONS(1793),
    [sym_wait_keyword] = ACTIONS(1793),
    [sym_cancel_wait_keyword] = ACTIONS(1793),
    [sym_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1793),
    [sym_not] = ACTIONS(1793),
    [sym_bnot] = ACTIONS(1793),
    [sym___file__] = ACTIONS(1793),
    [sym___date__] = ACTIONS(1793),
    [sym___ldate__] = ACTIONS(1793),
    [sym___time__] = ACTIONS(1793),
    [sym___line__] = ACTIONS(1793),
    [sym___name__] = ACTIONS(1793),
    [sym___netlinx__] = ACTIONS(1793),
    [sym_netlinx_axi_version] = ACTIONS(1793),
    [sym_url_flg_tcp] = ACTIONS(1793),
    [sym_url_flg_accinfopresent] = ACTIONS(1793),
    [sym_url_flg_temp] = ACTIONS(1793),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1793),
    [sym_url_flg_stat_mask] = ACTIONS(1793),
    [sym_url_flg_stat_lookup] = ACTIONS(1793),
    [sym_url_flg_stat_connecting] = ACTIONS(1793),
    [sym_url_flg_stat_waiting] = ACTIONS(1793),
    [sym_url_flg_stat_connected] = ACTIONS(1793),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1793),
    [sym_first_virtual_device] = ACTIONS(1793),
    [sym_first_local_port] = ACTIONS(1793),
    [sym_dynamic_virtual_device] = ACTIONS(1793),
    [sym_timeline_once] = ACTIONS(1793),
    [sym_timeline_repeat] = ACTIONS(1793),
    [sym_timeline_absolute] = ACTIONS(1793),
    [sym_timeline_relative] = ACTIONS(1793),
    [sym_do_push_timed_infinite] = ACTIONS(1793),
    [sym_source_type_no_address] = ACTIONS(1793),
    [sym_source_type_neuron_id] = ACTIONS(1793),
    [sym_source_type_ip_address] = ACTIONS(1793),
    [sym_source_type_axlink] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1793),
    [sym_source_type_ip_socket_address] = ACTIONS(1793),
    [sym_source_type_rs232] = ACTIONS(1793),
    [sym_source_type_internal] = ACTIONS(1793),
    [sym_source_type_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1793),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1793),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1793),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1793),
    [sym_true] = ACTIONS(1793),
    [sym_false] = ACTIONS(1793),
    [sym_file_read_only] = ACTIONS(1793),
    [sym_file_rw_new] = ACTIONS(1793),
    [sym_file_rw_append] = ACTIONS(1793),
    [sym_ip_tcp] = ACTIONS(1793),
    [sym_ip_udp] = ACTIONS(1793),
    [sym_ip_udp_2way] = ACTIONS(1793),
    [sym_xml_encode_types] = ACTIONS(1793),
    [sym_xml_encode_char_as_list] = ACTIONS(1793),
    [sym_xml_encode_le] = ACTIONS(1793),
    [sym_xml_decode_types] = ACTIONS(1793),
    [sym_xml_decode_no_preserve] = ACTIONS(1793),
    [sym_duet_dev_type_display_device] = ACTIONS(1793),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_io_device] = ACTIONS(1793),
    [sym_duet_dev_type_relay_device] = ACTIONS(1793),
    [sym_duet_dev_type_ups] = ACTIONS(1793),
    [sym_duet_dev_type_amplifier] = ACTIONS(1793),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1793),
    [sym_duet_dev_type_camera] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1793),
    [sym_duet_dev_type_dss] = ACTIONS(1793),
    [sym_duet_dev_type_dvr] = ACTIONS(1793),
    [sym_duet_dev_type_disc_device] = ACTIONS(1793),
    [sym_duet_dev_type_document_camera] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1793),
    [sym_duet_dev_type_hvac] = ACTIONS(1793),
    [sym_duet_dev_type_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_light] = ACTIONS(1793),
    [sym_duet_dev_type_monitor] = ACTIONS(1793),
    [sym_duet_dev_type_motor] = ACTIONS(1793),
    [sym_duet_dev_type_multi_window] = ACTIONS(1793),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1793),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1793),
    [sym_duet_dev_type_receiver] = ACTIONS(1793),
    [sym_duet_dev_type_security_system] = ACTIONS(1793),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1793),
    [sym_duet_dev_type_settop_box] = ACTIONS(1793),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1793),
    [sym_duet_dev_type_switcher] = ACTIONS(1793),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_tv] = ACTIONS(1793),
    [sym_duet_dev_type_utility] = ACTIONS(1793),
    [sym_duet_dev_type_vcr] = ACTIONS(1793),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_video_processor] = ACTIONS(1793),
    [sym_duet_dev_type_video_projector] = ACTIONS(1793),
    [sym_duet_dev_type_video_wall] = ACTIONS(1793),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1793),
    [sym_duet_dev_type_weather] = ACTIONS(1793),
    [sym_duet_dev_not_polled] = ACTIONS(1793),
    [sym_duet_dev_polled] = ACTIONS(1793),
    [sym_clkmgr_mode_network] = ACTIONS(1793),
    [sym_clkmgr_mode_standalone] = ACTIONS(1793),
    [sym_normal_standby] = ACTIONS(1793),
    [sym_normal_wake] = ACTIONS(1793),
    [sym_ip_multicast_ttl_option] = ACTIONS(1793),
    [sym_ip_ttl_subnet] = ACTIONS(1793),
    [sym_ip_ttl_site] = ACTIONS(1793),
    [sym_ip_ttl_region] = ACTIONS(1793),
    [sym_ip_ttl_continent] = ACTIONS(1793),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1793),
    [sym_ip_nodelay_on] = ACTIONS(1793),
    [sym_ip_nodelay_off] = ACTIONS(1793),
    [sym_amx_error] = ACTIONS(1793),
    [sym_amx_warning] = ACTIONS(1793),
    [sym_amx_info] = ACTIONS(1793),
    [sym_amx_debug] = ACTIONS(1793),
    [sym_netlinx_login_success] = ACTIONS(1793),
    [sym_netlinx_login_fail] = ACTIONS(1793),
    [sym_netlinx_logout] = ACTIONS(1793),
    [sym_null_str] = ACTIONS(1793),
    [sym_smtp_address] = ACTIONS(1793),
    [sym_smtp_port_number] = ACTIONS(1793),
    [sym_smtp_username] = ACTIONS(1793),
    [sym_smtp_password] = ACTIONS(1793),
    [sym_smtp_require_tls] = ACTIONS(1793),
    [sym_smtp_from] = ACTIONS(1793),
    [sym_smtp_tls_true] = ACTIONS(1793),
    [sym_smtp_tls_false] = ACTIONS(1793),
    [sym_listview_on_row_select_event] = ACTIONS(1793),
    [sym_data_structure_unknown] = ACTIONS(1793),
    [sym_data_structure_datafeed] = ACTIONS(1793),
    [sym_data_structure_datarecordset] = ACTIONS(1793),
    [sym_data_structure_datarecord] = ACTIONS(1793),
    [sym_data_structure_datafield] = ACTIONS(1793),
    [sym_data_max_content_field_count] = ACTIONS(1793),
    [sym_data_max_metadata_field_count] = ACTIONS(1793),
    [sym_data_max_id_length] = ACTIONS(1793),
    [sym_data_max_name_length] = ACTIONS(1793),
    [sym_data_max_description_length] = ACTIONS(1793),
    [sym_data_max_type_length] = ACTIONS(1793),
    [sym_data_max_format_length] = ACTIONS(1793),
    [sym_data_max_source_length] = ACTIONS(1793),
    [sym_data_max_value_length] = ACTIONS(1793),
    [sym_data_max_label_length] = ACTIONS(1793),
    [sym_data_max_url_length] = ACTIONS(1793),
    [sym_max_last_login_info_length] = ACTIONS(1793),
    [sym_data_type_unknown] = ACTIONS(1793),
    [sym_data_type_string] = ACTIONS(1793),
    [sym_data_type_datetime] = ACTIONS(1793),
    [sym_data_type_date] = ACTIONS(1793),
    [sym_data_type_time] = ACTIONS(1793),
    [sym_data_type_image] = ACTIONS(1793),
    [sym_data_format_url] = ACTIONS(1793),
    [sym_data_format_phone] = ACTIONS(1793),
    [sym_data_format_email] = ACTIONS(1793),
    [sym_data_format_iso8601] = ACTIONS(1793),
    [sym_auditlog_priv] = ACTIONS(1793),
    [sym_configuration_priv] = ACTIONS(1793),
    [sym_deviceconfig_priv] = ACTIONS(1793),
    [sym_ftp_priv] = ACTIONS(1793),
    [sym_http_priv] = ACTIONS(1793),
    [sym_networkconfig_priv] = ACTIONS(1793),
    [sym_programport_priv] = ACTIONS(1793),
    [sym_remoteui_priv] = ACTIONS(1793),
    [sym_securitycontrol_priv] = ACTIONS(1793),
    [sym_softwaremanagement_priv] = ACTIONS(1793),
    [sym_terminal_priv] = ACTIONS(1793),
    [sym_tpadmin_priv] = ACTIONS(1793),
    [sym_user1_priv] = ACTIONS(1793),
    [sym_user2_priv] = ACTIONS(1793),
    [sym_user3_priv] = ACTIONS(1793),
    [sym_user4_priv] = ACTIONS(1793),
    [sym_usermanagement_priv] = ACTIONS(1793),
    [sym_valid_account] = ACTIONS(1793),
    [sym_err_name_invalid] = ACTIONS(1793),
    [sym_err_password_invalid] = ACTIONS(1793),
    [sym_err_invalid_account] = ACTIONS(1793),
    [sym_err_no_authorization] = ACTIONS(1793),
    [sym_err_invalid_parameter] = ACTIONS(1793),
    [sym_err_user_locked_out] = ACTIONS(1793),
    [sym_err_user_account_expired] = ACTIONS(1793),
    [sym_err_general_error] = ACTIONS(1793),
    [sym_err_authentication_error] = ACTIONS(1793),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1793),
    [sym_tls_validate_certificate] = ACTIONS(1793),
    [sym_tdata] = ACTIONS(1793),
    [sym_tchannel] = ACTIONS(1793),
    [sym_tlevel] = ACTIONS(1793),
    [sym_tbutton] = ACTIONS(1793),
    [sym_ttimeline] = ACTIONS(1793),
    [sym_tcustom] = ACTIONS(1793),
    [sym_url_struct] = ACTIONS(1793),
    [sym_dns_struct] = ACTIONS(1793),
    [sym_ip_address_struct] = ACTIONS(1793),
    [sym_dev_info_struct] = ACTIONS(1793),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1793),
    [sym_data_feed] = ACTIONS(1793),
    [sym_data_field] = ACTIONS(1793),
    [sym_data_record] = ACTIONS(1793),
    [sym_wc_data_feed] = ACTIONS(1793),
    [sym_wc_data_field] = ACTIONS(1793),
    [sym_wc_data_record] = ACTIONS(1793),
    [sym_last_login_info] = ACTIONS(1793),
    [sym____reserved___] = ACTIONS(1793),
    [sym_date] = ACTIONS(1793),
    [sym_day] = ACTIONS(1793),
    [sym_time] = ACTIONS(1793),
    [sym_ldate] = ACTIONS(1793),
    [sym_push_device] = ACTIONS(1793),
    [sym_push_channel] = ACTIONS(1793),
    [sym_push_devchan] = ACTIONS(1793),
    [sym_release_device] = ACTIONS(1793),
    [sym_release_channel] = ACTIONS(1793),
    [sym_release_devchan] = ACTIONS(1793),
    [sym_master_slot] = ACTIONS(1793),
    [sym_get_pulse_time] = ACTIONS(1793),
    [sym_get_timer] = ACTIONS(1793),
    [sym_master_sn] = ACTIONS(1793),
    [sym_system_number] = ACTIONS(1793),
    [sym_button] = ACTIONS(1793),
    [sym_dv_channel] = ACTIONS(1793),
    [sym_data] = ACTIONS(1793),
    [sym_level] = ACTIONS(1793),
    [sym_timeline] = ACTIONS(1793),
    [sym_channel] = ACTIONS(1793),
    [sym_custom] = ACTIONS(1793),
    [sym_atoi] = ACTIONS(1793),
    [sym_atol] = ACTIONS(1793),
    [sym_atof] = ACTIONS(1793),
    [sym_itoa] = ACTIONS(1793),
    [sym_itohex] = ACTIONS(1793),
    [sym_hextoi] = ACTIONS(1793),
    [sym_ftoa] = ACTIONS(1793),
    [sym_find_string] = ACTIONS(1793),
    [sym_left_string] = ACTIONS(1793),
    [sym_right_string] = ACTIONS(1793),
    [sym_mid_string] = ACTIONS(1793),
    [sym_remove_string] = ACTIONS(1793),
    [sym_get_buffer_string] = ACTIONS(1793),
    [sym_get_buffer_char] = ACTIONS(1793),
    [sym_get_multi_buffer_string] = ACTIONS(1793),
    [sym_length_string] = ACTIONS(1793),
    [sym_max_length_string] = ACTIONS(1793),
    [sym_set_length_string] = ACTIONS(1793),
    [sym_compare_string] = ACTIONS(1793),
    [sym_upper_string] = ACTIONS(1793),
    [sym_lower_string] = ACTIONS(1793),
    [sym_get_last] = ACTIONS(1793),
    [sym_date_to_day] = ACTIONS(1793),
    [sym_date_to_month] = ACTIONS(1793),
    [sym_date_to_year] = ACTIONS(1793),
    [sym_time_to_hour] = ACTIONS(1793),
    [sym_time_to_minute] = ACTIONS(1793),
    [sym_time_to_second] = ACTIONS(1793),
    [sym_day_of_week] = ACTIONS(1793),
    [sym_set_pulse_time] = ACTIONS(1793),
    [sym_set_timer] = ACTIONS(1793),
    [sym_file_open] = ACTIONS(1793),
    [sym_file_close] = ACTIONS(1793),
    [sym_file_read] = ACTIONS(1793),
    [sym_file_read_line] = ACTIONS(1793),
    [sym_file_write] = ACTIONS(1793),
    [sym_file_write_line] = ACTIONS(1793),
    [sym_file_copy] = ACTIONS(1793),
    [sym_file_delete] = ACTIONS(1793),
    [sym_file_rename] = ACTIONS(1793),
    [sym_file_dir] = ACTIONS(1793),
    [sym_file_getdir] = ACTIONS(1793),
    [sym_file_setdir] = ACTIONS(1793),
    [sym_file_create_dir] = ACTIONS(1793),
    [sym_file_remove_dir] = ACTIONS(1793),
    [sym_file_seek] = ACTIONS(1793),
    [sym_length_array] = ACTIONS(1793),
    [sym_max_length_array] = ACTIONS(1793),
    [sym_set_length_array] = ACTIONS(1793),
    [sym_ip_client_open] = ACTIONS(1793),
    [sym_ip_client_close] = ACTIONS(1793),
    [sym_ip_server_open] = ACTIONS(1793),
    [sym_ip_server_close] = ACTIONS(1793),
    [sym_ip_bound_client_open] = ACTIONS(1793),
    [sym_ip_mc_server_open] = ACTIONS(1793),
    [sym_ip_set_option] = ACTIONS(1793),
    [sym_ssh_client_open] = ACTIONS(1793),
    [sym_ssh_client_close] = ACTIONS(1793),
    [sym_tls_client_open] = ACTIONS(1793),
    [sym_tls_client_close] = ACTIONS(1793),
    [sym_do_push] = ACTIONS(1793),
    [sym_do_push_timed] = ACTIONS(1793),
    [sym_do_release] = ACTIONS(1793),
    [sym_device_id] = ACTIONS(1793),
    [sym_device_info] = ACTIONS(1793),
    [sym_device_id_string] = ACTIONS(1793),
    [sym_device_standby] = ACTIONS(1793),
    [sym_device_wake] = ACTIONS(1793),
    [sym_timeline_create] = ACTIONS(1793),
    [sym_timeline_kill] = ACTIONS(1793),
    [sym_timeline_reload] = ACTIONS(1793),
    [sym_timeline_pause] = ACTIONS(1793),
    [sym_timeline_restart] = ACTIONS(1793),
    [sym_timeline_set] = ACTIONS(1793),
    [sym_timeline_get] = ACTIONS(1793),
    [sym_timeline_active] = ACTIONS(1793),
    [sym_timeline_dynamic_id] = ACTIONS(1793),
    [sym_variable_to_string] = ACTIONS(1793),
    [sym_string_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_string] = ACTIONS(1793),
    [sym_type_cast] = ACTIONS(1793),
    [sym_variable_to_xml] = ACTIONS(1793),
    [sym_xml_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_xml] = ACTIONS(1793),
    [sym_redirect_string] = ACTIONS(1793),
    [sym_random_number] = ACTIONS(1793),
    [sym_get_unique_id] = ACTIONS(1793),
    [sym_get_system_number] = ACTIONS(1793),
    [sym_set_system_number] = ACTIONS(1793),
    [sym_get_serial_number] = ACTIONS(1793),
    [sym_get_master_build] = ACTIONS(1793),
    [sym_set_validation_code] = ACTIONS(1793),
    [sym_reboot] = ACTIONS(1793),
    [sym_load_duet_module] = ACTIONS(1793),
    [sym_unload_duet_module] = ACTIONS(1793),
    [sym_duet_mem_size_set] = ACTIONS(1793),
    [sym_duet_mem_size_get] = ACTIONS(1793),
    [sym_dynamic_application_device] = ACTIONS(1793),
    [sym_dynamic_polled_port] = ACTIONS(1793),
    [sym_static_port_binding] = ACTIONS(1793),
    [sym_static_ip_binding] = ACTIONS(1793),
    [sym_get_dns_list] = ACTIONS(1793),
    [sym_set_dns_list] = ACTIONS(1793),
    [sym_get_ip_address] = ACTIONS(1793),
    [sym_set_ip_address] = ACTIONS(1793),
    [sym_get_url_list] = ACTIONS(1793),
    [sym_add_url_entry] = ACTIONS(1793),
    [sym_delete_url_entry] = ACTIONS(1793),
    [sym_abs_value] = ACTIONS(1793),
    [sym_max_value] = ACTIONS(1793),
    [sym_min_value] = ACTIONS(1793),
    [sym_exp_value] = ACTIONS(1793),
    [sym_log_value] = ACTIONS(1793),
    [sym_log10_value] = ACTIONS(1793),
    [sym_power_value] = ACTIONS(1793),
    [sym_sqrt_value] = ACTIONS(1793),
    [sym_raw_be] = ACTIONS(1793),
    [sym_raw_le] = ACTIONS(1793),
    [sym_internal_queue_size_set] = ACTIONS(1793),
    [sym_internal_queue_size_get] = ACTIONS(1793),
    [sym_internal_threshold_set] = ACTIONS(1793),
    [sym_internal_threshold_get] = ACTIONS(1793),
    [sym_rebuilt_event] = ACTIONS(1793),
    [sym_do_custom_event] = ACTIONS(1793),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1793),
    [sym_clkmgr_set_clk_source] = ACTIONS(1793),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1793),
    [sym_clkmgr_get_timezone] = ACTIONS(1793),
    [sym_clkmgr_set_timezone] = ACTIONS(1793),
    [sym_clkmgr_get_resync_period] = ACTIONS(1793),
    [sym_clkmgr_set_resync_period] = ACTIONS(1793),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_timeservers] = ACTIONS(1793),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_set_log_level] = ACTIONS(1793),
    [sym_get_log_level] = ACTIONS(1793),
    [sym_amx_log] = ACTIONS(1793),
    [sym_smtp_server_config_set] = ACTIONS(1793),
    [sym_smtp_server_config_get] = ACTIONS(1793),
    [sym_smtp_send] = ACTIONS(1793),
    [sym_validate_netlinx_account] = ACTIONS(1793),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1793),
    [sym_authenticate_certificate] = ACTIONS(1793),
    [sym_audit_netlinx_session_event] = ACTIONS(1793),
    [sym_audit_netlinx_generic_event] = ACTIONS(1793),
    [sym_get_available_flash_disk_space] = ACTIONS(1793),
    [sym_get_max_flash_disk_space] = ACTIONS(1793),
    [sym_data_create_feed] = ACTIONS(1793),
    [sym_data_delete_feed] = ACTIONS(1793),
    [sym_data_publish_feed] = ACTIONS(1793),
    [sym_data_get_published_feed] = ACTIONS(1793),
    [sym_data_add_record] = ACTIONS(1793),
    [sym_data_get_event_record] = ACTIONS(1793),
    [sym__wc_data_create_feed] = ACTIONS(1793),
    [sym__wc_data_add_record] = ACTIONS(1793),
    [sym__wc_data_get_event_record] = ACTIONS(1793),
    [sym_format] = ACTIONS(1793),
    [sym_astro_clock] = ACTIONS(1793),
    [sym_set_outdoor_temperature] = ACTIONS(1793),
    [sym_set_virtual_level_count] = ACTIONS(1793),
    [sym_set_virtual_channel_count] = ACTIONS(1793),
    [sym_set_virtual_port_count] = ACTIONS(1793),
    [sym__wc_encode_int] = ACTIONS(1793),
    [sym__wc_decode_int] = ACTIONS(1793),
    [sym__wc_int] = ACTIONS(1793),
    [sym__wc_to_ch_int] = ACTIONS(1793),
    [sym__ch_to_wc_int] = ACTIONS(1793),
    [sym__wc_find_string_int] = ACTIONS(1793),
    [sym__wc_left_string_int] = ACTIONS(1793),
    [sym__wc_lower_string_int] = ACTIONS(1793),
    [sym__wc_mid_string_int] = ACTIONS(1793),
    [sym__wc_remove_string_int] = ACTIONS(1793),
    [sym__wc_right_string_int] = ACTIONS(1793),
    [sym__wc_upper_string_int] = ACTIONS(1793),
    [sym__wc_compare_string_int] = ACTIONS(1793),
    [sym__wc_get_buffer_char_int] = ACTIONS(1793),
    [sym__wc_get_buffer_string_int] = ACTIONS(1793),
    [sym__wc_concat_string_int] = ACTIONS(1793),
    [sym__wc_file_open_int] = ACTIONS(1793),
    [sym__wc_file_close_int] = ACTIONS(1793),
    [sym__wc_file_read_int] = ACTIONS(1793),
    [sym__wc_file_read_line_int] = ACTIONS(1793),
    [sym__wc_file_write_int] = ACTIONS(1793),
    [sym__wc_file_write_line_int] = ACTIONS(1793),
    [aux_sym_preproc_include_token1] = ACTIONS(1793),
    [aux_sym_preproc_define_token1] = ACTIONS(1795),
    [aux_sym_preproc_warn_token1] = ACTIONS(1795),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1795),
    [anon_sym_LBRACK] = ACTIONS(1795),
    [anon_sym_LBRACE] = ACTIONS(1795),
    [anon_sym_RBRACE] = ACTIONS(1795),
    [anon_sym_LPAREN] = ACTIONS(1793),
    [anon_sym_BANG] = ACTIONS(1795),
    [anon_sym_TILDE] = ACTIONS(1795),
    [anon_sym_DASH] = ACTIONS(1793),
    [anon_sym_PLUS] = ACTIONS(1793),
    [anon_sym_DASH_DASH] = ACTIONS(1795),
    [anon_sym_PLUS_PLUS] = ACTIONS(1795),
    [anon_sym_DQUOTE] = ACTIONS(1795),
    [anon_sym_SQUOTE] = ACTIONS(1795),
    [sym_number_literal] = ACTIONS(1795),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(467)] = {
    [sym_identifier] = ACTIONS(1793),
    [sym_define_call_keyword] = ACTIONS(1793),
    [sym_define_function_keyword] = ACTIONS(1793),
    [sym_define_library_function_keyword] = ACTIONS(1793),
    [sym_define_module_keyword] = ACTIONS(1793),
    [sym_char_keyword] = ACTIONS(1793),
    [sym_widechar_keyword] = ACTIONS(1793),
    [sym_integer_keyword] = ACTIONS(1793),
    [sym_sinteger_keyword] = ACTIONS(1793),
    [sym_long_keyword] = ACTIONS(1793),
    [sym_slong_keyword] = ACTIONS(1793),
    [sym_float_keyword] = ACTIONS(1793),
    [sym_double_keyword] = ACTIONS(1793),
    [sym_variant_keyword] = ACTIONS(1793),
    [sym_variantarray_keyword] = ACTIONS(1793),
    [sym_local_var_keyword] = ACTIONS(1793),
    [sym_stack_var_keyword] = ACTIONS(1793),
    [sym_constant_keyword] = ACTIONS(1793),
    [sym_volatile_keyword] = ACTIONS(1793),
    [sym_non_volatile_keyword] = ACTIONS(1793),
    [sym_persistent_keyword] = ACTIONS(1793),
    [sym_dev_keyword] = ACTIONS(1793),
    [sym_devlev_keyword] = ACTIONS(1793),
    [sym_devchan_keyword] = ACTIONS(1793),
    [sym_if_keyword] = ACTIONS(1793),
    [sym_else_keyword] = ACTIONS(1793),
    [sym_switch_keyword] = ACTIONS(1793),
    [sym_case_keyword] = ACTIONS(1793),
    [sym_default_keyword] = ACTIONS(1793),
    [sym_while_keyword] = ACTIONS(1793),
    [sym_for_keyword] = ACTIONS(1793),
    [sym_break_keyword] = ACTIONS(1793),
    [sym_continue_keyword] = ACTIONS(1793),
    [sym_return_keyword] = ACTIONS(1793),
    [sym_select_keyword] = ACTIONS(1793),
    [sym_active_keyword] = ACTIONS(1793),
    [sym_struct_keyword] = ACTIONS(1793),
    [sym_structure_keyword] = ACTIONS(1793),
    [aux_sym_on_keyword_token1] = ACTIONS(1793),
    [aux_sym_off_keyword_token1] = ACTIONS(1793),
    [sym_send_level_keyword] = ACTIONS(1793),
    [sym_send_string_keyword] = ACTIONS(1793),
    [sym_send_command_keyword] = ACTIONS(1793),
    [sym_clear_buffer_keyword] = ACTIONS(1793),
    [sym_create_buffer_keyword] = ACTIONS(1793),
    [sym_create_multi_buffer_keyword] = ACTIONS(1793),
    [sym_call_keyword] = ACTIONS(1793),
    [sym_system_call_keyword] = ACTIONS(1793),
    [sym_devchan_to_keyword] = ACTIONS(1793),
    [sym_devchan_min_to_keyword] = ACTIONS(1793),
    [sym_devchan_total_off_keyword] = ACTIONS(1793),
    [sym_devchan_pulse_keyword] = ACTIONS(1793),
    [sym_wait_keyword] = ACTIONS(1793),
    [sym_cancel_wait_keyword] = ACTIONS(1793),
    [sym_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1793),
    [sym_not] = ACTIONS(1793),
    [sym_bnot] = ACTIONS(1793),
    [sym___file__] = ACTIONS(1793),
    [sym___date__] = ACTIONS(1793),
    [sym___ldate__] = ACTIONS(1793),
    [sym___time__] = ACTIONS(1793),
    [sym___line__] = ACTIONS(1793),
    [sym___name__] = ACTIONS(1793),
    [sym___netlinx__] = ACTIONS(1793),
    [sym_netlinx_axi_version] = ACTIONS(1793),
    [sym_url_flg_tcp] = ACTIONS(1793),
    [sym_url_flg_accinfopresent] = ACTIONS(1793),
    [sym_url_flg_temp] = ACTIONS(1793),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1793),
    [sym_url_flg_stat_mask] = ACTIONS(1793),
    [sym_url_flg_stat_lookup] = ACTIONS(1793),
    [sym_url_flg_stat_connecting] = ACTIONS(1793),
    [sym_url_flg_stat_waiting] = ACTIONS(1793),
    [sym_url_flg_stat_connected] = ACTIONS(1793),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1793),
    [sym_first_virtual_device] = ACTIONS(1793),
    [sym_first_local_port] = ACTIONS(1793),
    [sym_dynamic_virtual_device] = ACTIONS(1793),
    [sym_timeline_once] = ACTIONS(1793),
    [sym_timeline_repeat] = ACTIONS(1793),
    [sym_timeline_absolute] = ACTIONS(1793),
    [sym_timeline_relative] = ACTIONS(1793),
    [sym_do_push_timed_infinite] = ACTIONS(1793),
    [sym_source_type_no_address] = ACTIONS(1793),
    [sym_source_type_neuron_id] = ACTIONS(1793),
    [sym_source_type_ip_address] = ACTIONS(1793),
    [sym_source_type_axlink] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1793),
    [sym_source_type_ip_socket_address] = ACTIONS(1793),
    [sym_source_type_rs232] = ACTIONS(1793),
    [sym_source_type_internal] = ACTIONS(1793),
    [sym_source_type_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1793),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1793),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1793),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1793),
    [sym_true] = ACTIONS(1793),
    [sym_false] = ACTIONS(1793),
    [sym_file_read_only] = ACTIONS(1793),
    [sym_file_rw_new] = ACTIONS(1793),
    [sym_file_rw_append] = ACTIONS(1793),
    [sym_ip_tcp] = ACTIONS(1793),
    [sym_ip_udp] = ACTIONS(1793),
    [sym_ip_udp_2way] = ACTIONS(1793),
    [sym_xml_encode_types] = ACTIONS(1793),
    [sym_xml_encode_char_as_list] = ACTIONS(1793),
    [sym_xml_encode_le] = ACTIONS(1793),
    [sym_xml_decode_types] = ACTIONS(1793),
    [sym_xml_decode_no_preserve] = ACTIONS(1793),
    [sym_duet_dev_type_display_device] = ACTIONS(1793),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_io_device] = ACTIONS(1793),
    [sym_duet_dev_type_relay_device] = ACTIONS(1793),
    [sym_duet_dev_type_ups] = ACTIONS(1793),
    [sym_duet_dev_type_amplifier] = ACTIONS(1793),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1793),
    [sym_duet_dev_type_camera] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1793),
    [sym_duet_dev_type_dss] = ACTIONS(1793),
    [sym_duet_dev_type_dvr] = ACTIONS(1793),
    [sym_duet_dev_type_disc_device] = ACTIONS(1793),
    [sym_duet_dev_type_document_camera] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1793),
    [sym_duet_dev_type_hvac] = ACTIONS(1793),
    [sym_duet_dev_type_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_light] = ACTIONS(1793),
    [sym_duet_dev_type_monitor] = ACTIONS(1793),
    [sym_duet_dev_type_motor] = ACTIONS(1793),
    [sym_duet_dev_type_multi_window] = ACTIONS(1793),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1793),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1793),
    [sym_duet_dev_type_receiver] = ACTIONS(1793),
    [sym_duet_dev_type_security_system] = ACTIONS(1793),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1793),
    [sym_duet_dev_type_settop_box] = ACTIONS(1793),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1793),
    [sym_duet_dev_type_switcher] = ACTIONS(1793),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_tv] = ACTIONS(1793),
    [sym_duet_dev_type_utility] = ACTIONS(1793),
    [sym_duet_dev_type_vcr] = ACTIONS(1793),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_video_processor] = ACTIONS(1793),
    [sym_duet_dev_type_video_projector] = ACTIONS(1793),
    [sym_duet_dev_type_video_wall] = ACTIONS(1793),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1793),
    [sym_duet_dev_type_weather] = ACTIONS(1793),
    [sym_duet_dev_not_polled] = ACTIONS(1793),
    [sym_duet_dev_polled] = ACTIONS(1793),
    [sym_clkmgr_mode_network] = ACTIONS(1793),
    [sym_clkmgr_mode_standalone] = ACTIONS(1793),
    [sym_normal_standby] = ACTIONS(1793),
    [sym_normal_wake] = ACTIONS(1793),
    [sym_ip_multicast_ttl_option] = ACTIONS(1793),
    [sym_ip_ttl_subnet] = ACTIONS(1793),
    [sym_ip_ttl_site] = ACTIONS(1793),
    [sym_ip_ttl_region] = ACTIONS(1793),
    [sym_ip_ttl_continent] = ACTIONS(1793),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1793),
    [sym_ip_nodelay_on] = ACTIONS(1793),
    [sym_ip_nodelay_off] = ACTIONS(1793),
    [sym_amx_error] = ACTIONS(1793),
    [sym_amx_warning] = ACTIONS(1793),
    [sym_amx_info] = ACTIONS(1793),
    [sym_amx_debug] = ACTIONS(1793),
    [sym_netlinx_login_success] = ACTIONS(1793),
    [sym_netlinx_login_fail] = ACTIONS(1793),
    [sym_netlinx_logout] = ACTIONS(1793),
    [sym_null_str] = ACTIONS(1793),
    [sym_smtp_address] = ACTIONS(1793),
    [sym_smtp_port_number] = ACTIONS(1793),
    [sym_smtp_username] = ACTIONS(1793),
    [sym_smtp_password] = ACTIONS(1793),
    [sym_smtp_require_tls] = ACTIONS(1793),
    [sym_smtp_from] = ACTIONS(1793),
    [sym_smtp_tls_true] = ACTIONS(1793),
    [sym_smtp_tls_false] = ACTIONS(1793),
    [sym_listview_on_row_select_event] = ACTIONS(1793),
    [sym_data_structure_unknown] = ACTIONS(1793),
    [sym_data_structure_datafeed] = ACTIONS(1793),
    [sym_data_structure_datarecordset] = ACTIONS(1793),
    [sym_data_structure_datarecord] = ACTIONS(1793),
    [sym_data_structure_datafield] = ACTIONS(1793),
    [sym_data_max_content_field_count] = ACTIONS(1793),
    [sym_data_max_metadata_field_count] = ACTIONS(1793),
    [sym_data_max_id_length] = ACTIONS(1793),
    [sym_data_max_name_length] = ACTIONS(1793),
    [sym_data_max_description_length] = ACTIONS(1793),
    [sym_data_max_type_length] = ACTIONS(1793),
    [sym_data_max_format_length] = ACTIONS(1793),
    [sym_data_max_source_length] = ACTIONS(1793),
    [sym_data_max_value_length] = ACTIONS(1793),
    [sym_data_max_label_length] = ACTIONS(1793),
    [sym_data_max_url_length] = ACTIONS(1793),
    [sym_max_last_login_info_length] = ACTIONS(1793),
    [sym_data_type_unknown] = ACTIONS(1793),
    [sym_data_type_string] = ACTIONS(1793),
    [sym_data_type_datetime] = ACTIONS(1793),
    [sym_data_type_date] = ACTIONS(1793),
    [sym_data_type_time] = ACTIONS(1793),
    [sym_data_type_image] = ACTIONS(1793),
    [sym_data_format_url] = ACTIONS(1793),
    [sym_data_format_phone] = ACTIONS(1793),
    [sym_data_format_email] = ACTIONS(1793),
    [sym_data_format_iso8601] = ACTIONS(1793),
    [sym_auditlog_priv] = ACTIONS(1793),
    [sym_configuration_priv] = ACTIONS(1793),
    [sym_deviceconfig_priv] = ACTIONS(1793),
    [sym_ftp_priv] = ACTIONS(1793),
    [sym_http_priv] = ACTIONS(1793),
    [sym_networkconfig_priv] = ACTIONS(1793),
    [sym_programport_priv] = ACTIONS(1793),
    [sym_remoteui_priv] = ACTIONS(1793),
    [sym_securitycontrol_priv] = ACTIONS(1793),
    [sym_softwaremanagement_priv] = ACTIONS(1793),
    [sym_terminal_priv] = ACTIONS(1793),
    [sym_tpadmin_priv] = ACTIONS(1793),
    [sym_user1_priv] = ACTIONS(1793),
    [sym_user2_priv] = ACTIONS(1793),
    [sym_user3_priv] = ACTIONS(1793),
    [sym_user4_priv] = ACTIONS(1793),
    [sym_usermanagement_priv] = ACTIONS(1793),
    [sym_valid_account] = ACTIONS(1793),
    [sym_err_name_invalid] = ACTIONS(1793),
    [sym_err_password_invalid] = ACTIONS(1793),
    [sym_err_invalid_account] = ACTIONS(1793),
    [sym_err_no_authorization] = ACTIONS(1793),
    [sym_err_invalid_parameter] = ACTIONS(1793),
    [sym_err_user_locked_out] = ACTIONS(1793),
    [sym_err_user_account_expired] = ACTIONS(1793),
    [sym_err_general_error] = ACTIONS(1793),
    [sym_err_authentication_error] = ACTIONS(1793),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1793),
    [sym_tls_validate_certificate] = ACTIONS(1793),
    [sym_tdata] = ACTIONS(1793),
    [sym_tchannel] = ACTIONS(1793),
    [sym_tlevel] = ACTIONS(1793),
    [sym_tbutton] = ACTIONS(1793),
    [sym_ttimeline] = ACTIONS(1793),
    [sym_tcustom] = ACTIONS(1793),
    [sym_url_struct] = ACTIONS(1793),
    [sym_dns_struct] = ACTIONS(1793),
    [sym_ip_address_struct] = ACTIONS(1793),
    [sym_dev_info_struct] = ACTIONS(1793),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1793),
    [sym_data_feed] = ACTIONS(1793),
    [sym_data_field] = ACTIONS(1793),
    [sym_data_record] = ACTIONS(1793),
    [sym_wc_data_feed] = ACTIONS(1793),
    [sym_wc_data_field] = ACTIONS(1793),
    [sym_wc_data_record] = ACTIONS(1793),
    [sym_last_login_info] = ACTIONS(1793),
    [sym____reserved___] = ACTIONS(1793),
    [sym_date] = ACTIONS(1793),
    [sym_day] = ACTIONS(1793),
    [sym_time] = ACTIONS(1793),
    [sym_ldate] = ACTIONS(1793),
    [sym_push_device] = ACTIONS(1793),
    [sym_push_channel] = ACTIONS(1793),
    [sym_push_devchan] = ACTIONS(1793),
    [sym_release_device] = ACTIONS(1793),
    [sym_release_channel] = ACTIONS(1793),
    [sym_release_devchan] = ACTIONS(1793),
    [sym_master_slot] = ACTIONS(1793),
    [sym_get_pulse_time] = ACTIONS(1793),
    [sym_get_timer] = ACTIONS(1793),
    [sym_master_sn] = ACTIONS(1793),
    [sym_system_number] = ACTIONS(1793),
    [sym_button] = ACTIONS(1793),
    [sym_dv_channel] = ACTIONS(1793),
    [sym_data] = ACTIONS(1793),
    [sym_level] = ACTIONS(1793),
    [sym_timeline] = ACTIONS(1793),
    [sym_channel] = ACTIONS(1793),
    [sym_custom] = ACTIONS(1793),
    [sym_atoi] = ACTIONS(1793),
    [sym_atol] = ACTIONS(1793),
    [sym_atof] = ACTIONS(1793),
    [sym_itoa] = ACTIONS(1793),
    [sym_itohex] = ACTIONS(1793),
    [sym_hextoi] = ACTIONS(1793),
    [sym_ftoa] = ACTIONS(1793),
    [sym_find_string] = ACTIONS(1793),
    [sym_left_string] = ACTIONS(1793),
    [sym_right_string] = ACTIONS(1793),
    [sym_mid_string] = ACTIONS(1793),
    [sym_remove_string] = ACTIONS(1793),
    [sym_get_buffer_string] = ACTIONS(1793),
    [sym_get_buffer_char] = ACTIONS(1793),
    [sym_get_multi_buffer_string] = ACTIONS(1793),
    [sym_length_string] = ACTIONS(1793),
    [sym_max_length_string] = ACTIONS(1793),
    [sym_set_length_string] = ACTIONS(1793),
    [sym_compare_string] = ACTIONS(1793),
    [sym_upper_string] = ACTIONS(1793),
    [sym_lower_string] = ACTIONS(1793),
    [sym_get_last] = ACTIONS(1793),
    [sym_date_to_day] = ACTIONS(1793),
    [sym_date_to_month] = ACTIONS(1793),
    [sym_date_to_year] = ACTIONS(1793),
    [sym_time_to_hour] = ACTIONS(1793),
    [sym_time_to_minute] = ACTIONS(1793),
    [sym_time_to_second] = ACTIONS(1793),
    [sym_day_of_week] = ACTIONS(1793),
    [sym_set_pulse_time] = ACTIONS(1793),
    [sym_set_timer] = ACTIONS(1793),
    [sym_file_open] = ACTIONS(1793),
    [sym_file_close] = ACTIONS(1793),
    [sym_file_read] = ACTIONS(1793),
    [sym_file_read_line] = ACTIONS(1793),
    [sym_file_write] = ACTIONS(1793),
    [sym_file_write_line] = ACTIONS(1793),
    [sym_file_copy] = ACTIONS(1793),
    [sym_file_delete] = ACTIONS(1793),
    [sym_file_rename] = ACTIONS(1793),
    [sym_file_dir] = ACTIONS(1793),
    [sym_file_getdir] = ACTIONS(1793),
    [sym_file_setdir] = ACTIONS(1793),
    [sym_file_create_dir] = ACTIONS(1793),
    [sym_file_remove_dir] = ACTIONS(1793),
    [sym_file_seek] = ACTIONS(1793),
    [sym_length_array] = ACTIONS(1793),
    [sym_max_length_array] = ACTIONS(1793),
    [sym_set_length_array] = ACTIONS(1793),
    [sym_ip_client_open] = ACTIONS(1793),
    [sym_ip_client_close] = ACTIONS(1793),
    [sym_ip_server_open] = ACTIONS(1793),
    [sym_ip_server_close] = ACTIONS(1793),
    [sym_ip_bound_client_open] = ACTIONS(1793),
    [sym_ip_mc_server_open] = ACTIONS(1793),
    [sym_ip_set_option] = ACTIONS(1793),
    [sym_ssh_client_open] = ACTIONS(1793),
    [sym_ssh_client_close] = ACTIONS(1793),
    [sym_tls_client_open] = ACTIONS(1793),
    [sym_tls_client_close] = ACTIONS(1793),
    [sym_do_push] = ACTIONS(1793),
    [sym_do_push_timed] = ACTIONS(1793),
    [sym_do_release] = ACTIONS(1793),
    [sym_device_id] = ACTIONS(1793),
    [sym_device_info] = ACTIONS(1793),
    [sym_device_id_string] = ACTIONS(1793),
    [sym_device_standby] = ACTIONS(1793),
    [sym_device_wake] = ACTIONS(1793),
    [sym_timeline_create] = ACTIONS(1793),
    [sym_timeline_kill] = ACTIONS(1793),
    [sym_timeline_reload] = ACTIONS(1793),
    [sym_timeline_pause] = ACTIONS(1793),
    [sym_timeline_restart] = ACTIONS(1793),
    [sym_timeline_set] = ACTIONS(1793),
    [sym_timeline_get] = ACTIONS(1793),
    [sym_timeline_active] = ACTIONS(1793),
    [sym_timeline_dynamic_id] = ACTIONS(1793),
    [sym_variable_to_string] = ACTIONS(1793),
    [sym_string_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_string] = ACTIONS(1793),
    [sym_type_cast] = ACTIONS(1793),
    [sym_variable_to_xml] = ACTIONS(1793),
    [sym_xml_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_xml] = ACTIONS(1793),
    [sym_redirect_string] = ACTIONS(1793),
    [sym_random_number] = ACTIONS(1793),
    [sym_get_unique_id] = ACTIONS(1793),
    [sym_get_system_number] = ACTIONS(1793),
    [sym_set_system_number] = ACTIONS(1793),
    [sym_get_serial_number] = ACTIONS(1793),
    [sym_get_master_build] = ACTIONS(1793),
    [sym_set_validation_code] = ACTIONS(1793),
    [sym_reboot] = ACTIONS(1793),
    [sym_load_duet_module] = ACTIONS(1793),
    [sym_unload_duet_module] = ACTIONS(1793),
    [sym_duet_mem_size_set] = ACTIONS(1793),
    [sym_duet_mem_size_get] = ACTIONS(1793),
    [sym_dynamic_application_device] = ACTIONS(1793),
    [sym_dynamic_polled_port] = ACTIONS(1793),
    [sym_static_port_binding] = ACTIONS(1793),
    [sym_static_ip_binding] = ACTIONS(1793),
    [sym_get_dns_list] = ACTIONS(1793),
    [sym_set_dns_list] = ACTIONS(1793),
    [sym_get_ip_address] = ACTIONS(1793),
    [sym_set_ip_address] = ACTIONS(1793),
    [sym_get_url_list] = ACTIONS(1793),
    [sym_add_url_entry] = ACTIONS(1793),
    [sym_delete_url_entry] = ACTIONS(1793),
    [sym_abs_value] = ACTIONS(1793),
    [sym_max_value] = ACTIONS(1793),
    [sym_min_value] = ACTIONS(1793),
    [sym_exp_value] = ACTIONS(1793),
    [sym_log_value] = ACTIONS(1793),
    [sym_log10_value] = ACTIONS(1793),
    [sym_power_value] = ACTIONS(1793),
    [sym_sqrt_value] = ACTIONS(1793),
    [sym_raw_be] = ACTIONS(1793),
    [sym_raw_le] = ACTIONS(1793),
    [sym_internal_queue_size_set] = ACTIONS(1793),
    [sym_internal_queue_size_get] = ACTIONS(1793),
    [sym_internal_threshold_set] = ACTIONS(1793),
    [sym_internal_threshold_get] = ACTIONS(1793),
    [sym_rebuilt_event] = ACTIONS(1793),
    [sym_do_custom_event] = ACTIONS(1793),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1793),
    [sym_clkmgr_set_clk_source] = ACTIONS(1793),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1793),
    [sym_clkmgr_get_timezone] = ACTIONS(1793),
    [sym_clkmgr_set_timezone] = ACTIONS(1793),
    [sym_clkmgr_get_resync_period] = ACTIONS(1793),
    [sym_clkmgr_set_resync_period] = ACTIONS(1793),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_timeservers] = ACTIONS(1793),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_set_log_level] = ACTIONS(1793),
    [sym_get_log_level] = ACTIONS(1793),
    [sym_amx_log] = ACTIONS(1793),
    [sym_smtp_server_config_set] = ACTIONS(1793),
    [sym_smtp_server_config_get] = ACTIONS(1793),
    [sym_smtp_send] = ACTIONS(1793),
    [sym_validate_netlinx_account] = ACTIONS(1793),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1793),
    [sym_authenticate_certificate] = ACTIONS(1793),
    [sym_audit_netlinx_session_event] = ACTIONS(1793),
    [sym_audit_netlinx_generic_event] = ACTIONS(1793),
    [sym_get_available_flash_disk_space] = ACTIONS(1793),
    [sym_get_max_flash_disk_space] = ACTIONS(1793),
    [sym_data_create_feed] = ACTIONS(1793),
    [sym_data_delete_feed] = ACTIONS(1793),
    [sym_data_publish_feed] = ACTIONS(1793),
    [sym_data_get_published_feed] = ACTIONS(1793),
    [sym_data_add_record] = ACTIONS(1793),
    [sym_data_get_event_record] = ACTIONS(1793),
    [sym__wc_data_create_feed] = ACTIONS(1793),
    [sym__wc_data_add_record] = ACTIONS(1793),
    [sym__wc_data_get_event_record] = ACTIONS(1793),
    [sym_format] = ACTIONS(1793),
    [sym_astro_clock] = ACTIONS(1793),
    [sym_set_outdoor_temperature] = ACTIONS(1793),
    [sym_set_virtual_level_count] = ACTIONS(1793),
    [sym_set_virtual_channel_count] = ACTIONS(1793),
    [sym_set_virtual_port_count] = ACTIONS(1793),
    [sym__wc_encode_int] = ACTIONS(1793),
    [sym__wc_decode_int] = ACTIONS(1793),
    [sym__wc_int] = ACTIONS(1793),
    [sym__wc_to_ch_int] = ACTIONS(1793),
    [sym__ch_to_wc_int] = ACTIONS(1793),
    [sym__wc_find_string_int] = ACTIONS(1793),
    [sym__wc_left_string_int] = ACTIONS(1793),
    [sym__wc_lower_string_int] = ACTIONS(1793),
    [sym__wc_mid_string_int] = ACTIONS(1793),
    [sym__wc_remove_string_int] = ACTIONS(1793),
    [sym__wc_right_string_int] = ACTIONS(1793),
    [sym__wc_upper_string_int] = ACTIONS(1793),
    [sym__wc_compare_string_int] = ACTIONS(1793),
    [sym__wc_get_buffer_char_int] = ACTIONS(1793),
    [sym__wc_get_buffer_string_int] = ACTIONS(1793),
    [sym__wc_concat_string_int] = ACTIONS(1793),
    [sym__wc_file_open_int] = ACTIONS(1793),
    [sym__wc_file_close_int] = ACTIONS(1793),
    [sym__wc_file_read_int] = ACTIONS(1793),
    [sym__wc_file_read_line_int] = ACTIONS(1793),
    [sym__wc_file_write_int] = ACTIONS(1793),
    [sym__wc_file_write_line_int] = ACTIONS(1793),
    [aux_sym_preproc_include_token1] = ACTIONS(1793),
    [aux_sym_preproc_define_token1] = ACTIONS(1795),
    [aux_sym_preproc_warn_token1] = ACTIONS(1795),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1795),
    [anon_sym_LBRACK] = ACTIONS(1795),
    [anon_sym_LBRACE] = ACTIONS(1795),
    [anon_sym_RBRACE] = ACTIONS(1795),
    [anon_sym_LPAREN] = ACTIONS(1793),
    [anon_sym_BANG] = ACTIONS(1795),
    [anon_sym_TILDE] = ACTIONS(1795),
    [anon_sym_DASH] = ACTIONS(1793),
    [anon_sym_PLUS] = ACTIONS(1793),
    [anon_sym_DASH_DASH] = ACTIONS(1795),
    [anon_sym_PLUS_PLUS] = ACTIONS(1795),
    [anon_sym_DQUOTE] = ACTIONS(1795),
    [anon_sym_SQUOTE] = ACTIONS(1795),
    [sym_number_literal] = ACTIONS(1795),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(468)] = {
    [sym_identifier] = ACTIONS(1793),
    [sym_define_call_keyword] = ACTIONS(1793),
    [sym_define_function_keyword] = ACTIONS(1793),
    [sym_define_library_function_keyword] = ACTIONS(1793),
    [sym_define_module_keyword] = ACTIONS(1793),
    [sym_char_keyword] = ACTIONS(1793),
    [sym_widechar_keyword] = ACTIONS(1793),
    [sym_integer_keyword] = ACTIONS(1793),
    [sym_sinteger_keyword] = ACTIONS(1793),
    [sym_long_keyword] = ACTIONS(1793),
    [sym_slong_keyword] = ACTIONS(1793),
    [sym_float_keyword] = ACTIONS(1793),
    [sym_double_keyword] = ACTIONS(1793),
    [sym_variant_keyword] = ACTIONS(1793),
    [sym_variantarray_keyword] = ACTIONS(1793),
    [sym_local_var_keyword] = ACTIONS(1793),
    [sym_stack_var_keyword] = ACTIONS(1793),
    [sym_constant_keyword] = ACTIONS(1793),
    [sym_volatile_keyword] = ACTIONS(1793),
    [sym_non_volatile_keyword] = ACTIONS(1793),
    [sym_persistent_keyword] = ACTIONS(1793),
    [sym_dev_keyword] = ACTIONS(1793),
    [sym_devlev_keyword] = ACTIONS(1793),
    [sym_devchan_keyword] = ACTIONS(1793),
    [sym_if_keyword] = ACTIONS(1793),
    [sym_else_keyword] = ACTIONS(1793),
    [sym_switch_keyword] = ACTIONS(1793),
    [sym_case_keyword] = ACTIONS(1793),
    [sym_default_keyword] = ACTIONS(1793),
    [sym_while_keyword] = ACTIONS(1793),
    [sym_for_keyword] = ACTIONS(1793),
    [sym_break_keyword] = ACTIONS(1793),
    [sym_continue_keyword] = ACTIONS(1793),
    [sym_return_keyword] = ACTIONS(1793),
    [sym_select_keyword] = ACTIONS(1793),
    [sym_active_keyword] = ACTIONS(1793),
    [sym_struct_keyword] = ACTIONS(1793),
    [sym_structure_keyword] = ACTIONS(1793),
    [aux_sym_on_keyword_token1] = ACTIONS(1793),
    [aux_sym_off_keyword_token1] = ACTIONS(1793),
    [sym_send_level_keyword] = ACTIONS(1793),
    [sym_send_string_keyword] = ACTIONS(1793),
    [sym_send_command_keyword] = ACTIONS(1793),
    [sym_clear_buffer_keyword] = ACTIONS(1793),
    [sym_create_buffer_keyword] = ACTIONS(1793),
    [sym_create_multi_buffer_keyword] = ACTIONS(1793),
    [sym_call_keyword] = ACTIONS(1793),
    [sym_system_call_keyword] = ACTIONS(1793),
    [sym_devchan_to_keyword] = ACTIONS(1793),
    [sym_devchan_min_to_keyword] = ACTIONS(1793),
    [sym_devchan_total_off_keyword] = ACTIONS(1793),
    [sym_devchan_pulse_keyword] = ACTIONS(1793),
    [sym_wait_keyword] = ACTIONS(1793),
    [sym_cancel_wait_keyword] = ACTIONS(1793),
    [sym_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1793),
    [sym_not] = ACTIONS(1793),
    [sym_bnot] = ACTIONS(1793),
    [sym___file__] = ACTIONS(1793),
    [sym___date__] = ACTIONS(1793),
    [sym___ldate__] = ACTIONS(1793),
    [sym___time__] = ACTIONS(1793),
    [sym___line__] = ACTIONS(1793),
    [sym___name__] = ACTIONS(1793),
    [sym___netlinx__] = ACTIONS(1793),
    [sym_netlinx_axi_version] = ACTIONS(1793),
    [sym_url_flg_tcp] = ACTIONS(1793),
    [sym_url_flg_accinfopresent] = ACTIONS(1793),
    [sym_url_flg_temp] = ACTIONS(1793),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1793),
    [sym_url_flg_stat_mask] = ACTIONS(1793),
    [sym_url_flg_stat_lookup] = ACTIONS(1793),
    [sym_url_flg_stat_connecting] = ACTIONS(1793),
    [sym_url_flg_stat_waiting] = ACTIONS(1793),
    [sym_url_flg_stat_connected] = ACTIONS(1793),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1793),
    [sym_first_virtual_device] = ACTIONS(1793),
    [sym_first_local_port] = ACTIONS(1793),
    [sym_dynamic_virtual_device] = ACTIONS(1793),
    [sym_timeline_once] = ACTIONS(1793),
    [sym_timeline_repeat] = ACTIONS(1793),
    [sym_timeline_absolute] = ACTIONS(1793),
    [sym_timeline_relative] = ACTIONS(1793),
    [sym_do_push_timed_infinite] = ACTIONS(1793),
    [sym_source_type_no_address] = ACTIONS(1793),
    [sym_source_type_neuron_id] = ACTIONS(1793),
    [sym_source_type_ip_address] = ACTIONS(1793),
    [sym_source_type_axlink] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1793),
    [sym_source_type_ip_socket_address] = ACTIONS(1793),
    [sym_source_type_rs232] = ACTIONS(1793),
    [sym_source_type_internal] = ACTIONS(1793),
    [sym_source_type_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1793),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1793),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1793),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1793),
    [sym_true] = ACTIONS(1793),
    [sym_false] = ACTIONS(1793),
    [sym_file_read_only] = ACTIONS(1793),
    [sym_file_rw_new] = ACTIONS(1793),
    [sym_file_rw_append] = ACTIONS(1793),
    [sym_ip_tcp] = ACTIONS(1793),
    [sym_ip_udp] = ACTIONS(1793),
    [sym_ip_udp_2way] = ACTIONS(1793),
    [sym_xml_encode_types] = ACTIONS(1793),
    [sym_xml_encode_char_as_list] = ACTIONS(1793),
    [sym_xml_encode_le] = ACTIONS(1793),
    [sym_xml_decode_types] = ACTIONS(1793),
    [sym_xml_decode_no_preserve] = ACTIONS(1793),
    [sym_duet_dev_type_display_device] = ACTIONS(1793),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_io_device] = ACTIONS(1793),
    [sym_duet_dev_type_relay_device] = ACTIONS(1793),
    [sym_duet_dev_type_ups] = ACTIONS(1793),
    [sym_duet_dev_type_amplifier] = ACTIONS(1793),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1793),
    [sym_duet_dev_type_camera] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1793),
    [sym_duet_dev_type_dss] = ACTIONS(1793),
    [sym_duet_dev_type_dvr] = ACTIONS(1793),
    [sym_duet_dev_type_disc_device] = ACTIONS(1793),
    [sym_duet_dev_type_document_camera] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1793),
    [sym_duet_dev_type_hvac] = ACTIONS(1793),
    [sym_duet_dev_type_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_light] = ACTIONS(1793),
    [sym_duet_dev_type_monitor] = ACTIONS(1793),
    [sym_duet_dev_type_motor] = ACTIONS(1793),
    [sym_duet_dev_type_multi_window] = ACTIONS(1793),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1793),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1793),
    [sym_duet_dev_type_receiver] = ACTIONS(1793),
    [sym_duet_dev_type_security_system] = ACTIONS(1793),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1793),
    [sym_duet_dev_type_settop_box] = ACTIONS(1793),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1793),
    [sym_duet_dev_type_switcher] = ACTIONS(1793),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_tv] = ACTIONS(1793),
    [sym_duet_dev_type_utility] = ACTIONS(1793),
    [sym_duet_dev_type_vcr] = ACTIONS(1793),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_video_processor] = ACTIONS(1793),
    [sym_duet_dev_type_video_projector] = ACTIONS(1793),
    [sym_duet_dev_type_video_wall] = ACTIONS(1793),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1793),
    [sym_duet_dev_type_weather] = ACTIONS(1793),
    [sym_duet_dev_not_polled] = ACTIONS(1793),
    [sym_duet_dev_polled] = ACTIONS(1793),
    [sym_clkmgr_mode_network] = ACTIONS(1793),
    [sym_clkmgr_mode_standalone] = ACTIONS(1793),
    [sym_normal_standby] = ACTIONS(1793),
    [sym_normal_wake] = ACTIONS(1793),
    [sym_ip_multicast_ttl_option] = ACTIONS(1793),
    [sym_ip_ttl_subnet] = ACTIONS(1793),
    [sym_ip_ttl_site] = ACTIONS(1793),
    [sym_ip_ttl_region] = ACTIONS(1793),
    [sym_ip_ttl_continent] = ACTIONS(1793),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1793),
    [sym_ip_nodelay_on] = ACTIONS(1793),
    [sym_ip_nodelay_off] = ACTIONS(1793),
    [sym_amx_error] = ACTIONS(1793),
    [sym_amx_warning] = ACTIONS(1793),
    [sym_amx_info] = ACTIONS(1793),
    [sym_amx_debug] = ACTIONS(1793),
    [sym_netlinx_login_success] = ACTIONS(1793),
    [sym_netlinx_login_fail] = ACTIONS(1793),
    [sym_netlinx_logout] = ACTIONS(1793),
    [sym_null_str] = ACTIONS(1793),
    [sym_smtp_address] = ACTIONS(1793),
    [sym_smtp_port_number] = ACTIONS(1793),
    [sym_smtp_username] = ACTIONS(1793),
    [sym_smtp_password] = ACTIONS(1793),
    [sym_smtp_require_tls] = ACTIONS(1793),
    [sym_smtp_from] = ACTIONS(1793),
    [sym_smtp_tls_true] = ACTIONS(1793),
    [sym_smtp_tls_false] = ACTIONS(1793),
    [sym_listview_on_row_select_event] = ACTIONS(1793),
    [sym_data_structure_unknown] = ACTIONS(1793),
    [sym_data_structure_datafeed] = ACTIONS(1793),
    [sym_data_structure_datarecordset] = ACTIONS(1793),
    [sym_data_structure_datarecord] = ACTIONS(1793),
    [sym_data_structure_datafield] = ACTIONS(1793),
    [sym_data_max_content_field_count] = ACTIONS(1793),
    [sym_data_max_metadata_field_count] = ACTIONS(1793),
    [sym_data_max_id_length] = ACTIONS(1793),
    [sym_data_max_name_length] = ACTIONS(1793),
    [sym_data_max_description_length] = ACTIONS(1793),
    [sym_data_max_type_length] = ACTIONS(1793),
    [sym_data_max_format_length] = ACTIONS(1793),
    [sym_data_max_source_length] = ACTIONS(1793),
    [sym_data_max_value_length] = ACTIONS(1793),
    [sym_data_max_label_length] = ACTIONS(1793),
    [sym_data_max_url_length] = ACTIONS(1793),
    [sym_max_last_login_info_length] = ACTIONS(1793),
    [sym_data_type_unknown] = ACTIONS(1793),
    [sym_data_type_string] = ACTIONS(1793),
    [sym_data_type_datetime] = ACTIONS(1793),
    [sym_data_type_date] = ACTIONS(1793),
    [sym_data_type_time] = ACTIONS(1793),
    [sym_data_type_image] = ACTIONS(1793),
    [sym_data_format_url] = ACTIONS(1793),
    [sym_data_format_phone] = ACTIONS(1793),
    [sym_data_format_email] = ACTIONS(1793),
    [sym_data_format_iso8601] = ACTIONS(1793),
    [sym_auditlog_priv] = ACTIONS(1793),
    [sym_configuration_priv] = ACTIONS(1793),
    [sym_deviceconfig_priv] = ACTIONS(1793),
    [sym_ftp_priv] = ACTIONS(1793),
    [sym_http_priv] = ACTIONS(1793),
    [sym_networkconfig_priv] = ACTIONS(1793),
    [sym_programport_priv] = ACTIONS(1793),
    [sym_remoteui_priv] = ACTIONS(1793),
    [sym_securitycontrol_priv] = ACTIONS(1793),
    [sym_softwaremanagement_priv] = ACTIONS(1793),
    [sym_terminal_priv] = ACTIONS(1793),
    [sym_tpadmin_priv] = ACTIONS(1793),
    [sym_user1_priv] = ACTIONS(1793),
    [sym_user2_priv] = ACTIONS(1793),
    [sym_user3_priv] = ACTIONS(1793),
    [sym_user4_priv] = ACTIONS(1793),
    [sym_usermanagement_priv] = ACTIONS(1793),
    [sym_valid_account] = ACTIONS(1793),
    [sym_err_name_invalid] = ACTIONS(1793),
    [sym_err_password_invalid] = ACTIONS(1793),
    [sym_err_invalid_account] = ACTIONS(1793),
    [sym_err_no_authorization] = ACTIONS(1793),
    [sym_err_invalid_parameter] = ACTIONS(1793),
    [sym_err_user_locked_out] = ACTIONS(1793),
    [sym_err_user_account_expired] = ACTIONS(1793),
    [sym_err_general_error] = ACTIONS(1793),
    [sym_err_authentication_error] = ACTIONS(1793),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1793),
    [sym_tls_validate_certificate] = ACTIONS(1793),
    [sym_tdata] = ACTIONS(1793),
    [sym_tchannel] = ACTIONS(1793),
    [sym_tlevel] = ACTIONS(1793),
    [sym_tbutton] = ACTIONS(1793),
    [sym_ttimeline] = ACTIONS(1793),
    [sym_tcustom] = ACTIONS(1793),
    [sym_url_struct] = ACTIONS(1793),
    [sym_dns_struct] = ACTIONS(1793),
    [sym_ip_address_struct] = ACTIONS(1793),
    [sym_dev_info_struct] = ACTIONS(1793),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1793),
    [sym_data_feed] = ACTIONS(1793),
    [sym_data_field] = ACTIONS(1793),
    [sym_data_record] = ACTIONS(1793),
    [sym_wc_data_feed] = ACTIONS(1793),
    [sym_wc_data_field] = ACTIONS(1793),
    [sym_wc_data_record] = ACTIONS(1793),
    [sym_last_login_info] = ACTIONS(1793),
    [sym____reserved___] = ACTIONS(1793),
    [sym_date] = ACTIONS(1793),
    [sym_day] = ACTIONS(1793),
    [sym_time] = ACTIONS(1793),
    [sym_ldate] = ACTIONS(1793),
    [sym_push_device] = ACTIONS(1793),
    [sym_push_channel] = ACTIONS(1793),
    [sym_push_devchan] = ACTIONS(1793),
    [sym_release_device] = ACTIONS(1793),
    [sym_release_channel] = ACTIONS(1793),
    [sym_release_devchan] = ACTIONS(1793),
    [sym_master_slot] = ACTIONS(1793),
    [sym_get_pulse_time] = ACTIONS(1793),
    [sym_get_timer] = ACTIONS(1793),
    [sym_master_sn] = ACTIONS(1793),
    [sym_system_number] = ACTIONS(1793),
    [sym_button] = ACTIONS(1793),
    [sym_dv_channel] = ACTIONS(1793),
    [sym_data] = ACTIONS(1793),
    [sym_level] = ACTIONS(1793),
    [sym_timeline] = ACTIONS(1793),
    [sym_channel] = ACTIONS(1793),
    [sym_custom] = ACTIONS(1793),
    [sym_atoi] = ACTIONS(1793),
    [sym_atol] = ACTIONS(1793),
    [sym_atof] = ACTIONS(1793),
    [sym_itoa] = ACTIONS(1793),
    [sym_itohex] = ACTIONS(1793),
    [sym_hextoi] = ACTIONS(1793),
    [sym_ftoa] = ACTIONS(1793),
    [sym_find_string] = ACTIONS(1793),
    [sym_left_string] = ACTIONS(1793),
    [sym_right_string] = ACTIONS(1793),
    [sym_mid_string] = ACTIONS(1793),
    [sym_remove_string] = ACTIONS(1793),
    [sym_get_buffer_string] = ACTIONS(1793),
    [sym_get_buffer_char] = ACTIONS(1793),
    [sym_get_multi_buffer_string] = ACTIONS(1793),
    [sym_length_string] = ACTIONS(1793),
    [sym_max_length_string] = ACTIONS(1793),
    [sym_set_length_string] = ACTIONS(1793),
    [sym_compare_string] = ACTIONS(1793),
    [sym_upper_string] = ACTIONS(1793),
    [sym_lower_string] = ACTIONS(1793),
    [sym_get_last] = ACTIONS(1793),
    [sym_date_to_day] = ACTIONS(1793),
    [sym_date_to_month] = ACTIONS(1793),
    [sym_date_to_year] = ACTIONS(1793),
    [sym_time_to_hour] = ACTIONS(1793),
    [sym_time_to_minute] = ACTIONS(1793),
    [sym_time_to_second] = ACTIONS(1793),
    [sym_day_of_week] = ACTIONS(1793),
    [sym_set_pulse_time] = ACTIONS(1793),
    [sym_set_timer] = ACTIONS(1793),
    [sym_file_open] = ACTIONS(1793),
    [sym_file_close] = ACTIONS(1793),
    [sym_file_read] = ACTIONS(1793),
    [sym_file_read_line] = ACTIONS(1793),
    [sym_file_write] = ACTIONS(1793),
    [sym_file_write_line] = ACTIONS(1793),
    [sym_file_copy] = ACTIONS(1793),
    [sym_file_delete] = ACTIONS(1793),
    [sym_file_rename] = ACTIONS(1793),
    [sym_file_dir] = ACTIONS(1793),
    [sym_file_getdir] = ACTIONS(1793),
    [sym_file_setdir] = ACTIONS(1793),
    [sym_file_create_dir] = ACTIONS(1793),
    [sym_file_remove_dir] = ACTIONS(1793),
    [sym_file_seek] = ACTIONS(1793),
    [sym_length_array] = ACTIONS(1793),
    [sym_max_length_array] = ACTIONS(1793),
    [sym_set_length_array] = ACTIONS(1793),
    [sym_ip_client_open] = ACTIONS(1793),
    [sym_ip_client_close] = ACTIONS(1793),
    [sym_ip_server_open] = ACTIONS(1793),
    [sym_ip_server_close] = ACTIONS(1793),
    [sym_ip_bound_client_open] = ACTIONS(1793),
    [sym_ip_mc_server_open] = ACTIONS(1793),
    [sym_ip_set_option] = ACTIONS(1793),
    [sym_ssh_client_open] = ACTIONS(1793),
    [sym_ssh_client_close] = ACTIONS(1793),
    [sym_tls_client_open] = ACTIONS(1793),
    [sym_tls_client_close] = ACTIONS(1793),
    [sym_do_push] = ACTIONS(1793),
    [sym_do_push_timed] = ACTIONS(1793),
    [sym_do_release] = ACTIONS(1793),
    [sym_device_id] = ACTIONS(1793),
    [sym_device_info] = ACTIONS(1793),
    [sym_device_id_string] = ACTIONS(1793),
    [sym_device_standby] = ACTIONS(1793),
    [sym_device_wake] = ACTIONS(1793),
    [sym_timeline_create] = ACTIONS(1793),
    [sym_timeline_kill] = ACTIONS(1793),
    [sym_timeline_reload] = ACTIONS(1793),
    [sym_timeline_pause] = ACTIONS(1793),
    [sym_timeline_restart] = ACTIONS(1793),
    [sym_timeline_set] = ACTIONS(1793),
    [sym_timeline_get] = ACTIONS(1793),
    [sym_timeline_active] = ACTIONS(1793),
    [sym_timeline_dynamic_id] = ACTIONS(1793),
    [sym_variable_to_string] = ACTIONS(1793),
    [sym_string_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_string] = ACTIONS(1793),
    [sym_type_cast] = ACTIONS(1793),
    [sym_variable_to_xml] = ACTIONS(1793),
    [sym_xml_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_xml] = ACTIONS(1793),
    [sym_redirect_string] = ACTIONS(1793),
    [sym_random_number] = ACTIONS(1793),
    [sym_get_unique_id] = ACTIONS(1793),
    [sym_get_system_number] = ACTIONS(1793),
    [sym_set_system_number] = ACTIONS(1793),
    [sym_get_serial_number] = ACTIONS(1793),
    [sym_get_master_build] = ACTIONS(1793),
    [sym_set_validation_code] = ACTIONS(1793),
    [sym_reboot] = ACTIONS(1793),
    [sym_load_duet_module] = ACTIONS(1793),
    [sym_unload_duet_module] = ACTIONS(1793),
    [sym_duet_mem_size_set] = ACTIONS(1793),
    [sym_duet_mem_size_get] = ACTIONS(1793),
    [sym_dynamic_application_device] = ACTIONS(1793),
    [sym_dynamic_polled_port] = ACTIONS(1793),
    [sym_static_port_binding] = ACTIONS(1793),
    [sym_static_ip_binding] = ACTIONS(1793),
    [sym_get_dns_list] = ACTIONS(1793),
    [sym_set_dns_list] = ACTIONS(1793),
    [sym_get_ip_address] = ACTIONS(1793),
    [sym_set_ip_address] = ACTIONS(1793),
    [sym_get_url_list] = ACTIONS(1793),
    [sym_add_url_entry] = ACTIONS(1793),
    [sym_delete_url_entry] = ACTIONS(1793),
    [sym_abs_value] = ACTIONS(1793),
    [sym_max_value] = ACTIONS(1793),
    [sym_min_value] = ACTIONS(1793),
    [sym_exp_value] = ACTIONS(1793),
    [sym_log_value] = ACTIONS(1793),
    [sym_log10_value] = ACTIONS(1793),
    [sym_power_value] = ACTIONS(1793),
    [sym_sqrt_value] = ACTIONS(1793),
    [sym_raw_be] = ACTIONS(1793),
    [sym_raw_le] = ACTIONS(1793),
    [sym_internal_queue_size_set] = ACTIONS(1793),
    [sym_internal_queue_size_get] = ACTIONS(1793),
    [sym_internal_threshold_set] = ACTIONS(1793),
    [sym_internal_threshold_get] = ACTIONS(1793),
    [sym_rebuilt_event] = ACTIONS(1793),
    [sym_do_custom_event] = ACTIONS(1793),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1793),
    [sym_clkmgr_set_clk_source] = ACTIONS(1793),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1793),
    [sym_clkmgr_get_timezone] = ACTIONS(1793),
    [sym_clkmgr_set_timezone] = ACTIONS(1793),
    [sym_clkmgr_get_resync_period] = ACTIONS(1793),
    [sym_clkmgr_set_resync_period] = ACTIONS(1793),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_timeservers] = ACTIONS(1793),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_set_log_level] = ACTIONS(1793),
    [sym_get_log_level] = ACTIONS(1793),
    [sym_amx_log] = ACTIONS(1793),
    [sym_smtp_server_config_set] = ACTIONS(1793),
    [sym_smtp_server_config_get] = ACTIONS(1793),
    [sym_smtp_send] = ACTIONS(1793),
    [sym_validate_netlinx_account] = ACTIONS(1793),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1793),
    [sym_authenticate_certificate] = ACTIONS(1793),
    [sym_audit_netlinx_session_event] = ACTIONS(1793),
    [sym_audit_netlinx_generic_event] = ACTIONS(1793),
    [sym_get_available_flash_disk_space] = ACTIONS(1793),
    [sym_get_max_flash_disk_space] = ACTIONS(1793),
    [sym_data_create_feed] = ACTIONS(1793),
    [sym_data_delete_feed] = ACTIONS(1793),
    [sym_data_publish_feed] = ACTIONS(1793),
    [sym_data_get_published_feed] = ACTIONS(1793),
    [sym_data_add_record] = ACTIONS(1793),
    [sym_data_get_event_record] = ACTIONS(1793),
    [sym__wc_data_create_feed] = ACTIONS(1793),
    [sym__wc_data_add_record] = ACTIONS(1793),
    [sym__wc_data_get_event_record] = ACTIONS(1793),
    [sym_format] = ACTIONS(1793),
    [sym_astro_clock] = ACTIONS(1793),
    [sym_set_outdoor_temperature] = ACTIONS(1793),
    [sym_set_virtual_level_count] = ACTIONS(1793),
    [sym_set_virtual_channel_count] = ACTIONS(1793),
    [sym_set_virtual_port_count] = ACTIONS(1793),
    [sym__wc_encode_int] = ACTIONS(1793),
    [sym__wc_decode_int] = ACTIONS(1793),
    [sym__wc_int] = ACTIONS(1793),
    [sym__wc_to_ch_int] = ACTIONS(1793),
    [sym__ch_to_wc_int] = ACTIONS(1793),
    [sym__wc_find_string_int] = ACTIONS(1793),
    [sym__wc_left_string_int] = ACTIONS(1793),
    [sym__wc_lower_string_int] = ACTIONS(1793),
    [sym__wc_mid_string_int] = ACTIONS(1793),
    [sym__wc_remove_string_int] = ACTIONS(1793),
    [sym__wc_right_string_int] = ACTIONS(1793),
    [sym__wc_upper_string_int] = ACTIONS(1793),
    [sym__wc_compare_string_int] = ACTIONS(1793),
    [sym__wc_get_buffer_char_int] = ACTIONS(1793),
    [sym__wc_get_buffer_string_int] = ACTIONS(1793),
    [sym__wc_concat_string_int] = ACTIONS(1793),
    [sym__wc_file_open_int] = ACTIONS(1793),
    [sym__wc_file_close_int] = ACTIONS(1793),
    [sym__wc_file_read_int] = ACTIONS(1793),
    [sym__wc_file_read_line_int] = ACTIONS(1793),
    [sym__wc_file_write_int] = ACTIONS(1793),
    [sym__wc_file_write_line_int] = ACTIONS(1793),
    [aux_sym_preproc_include_token1] = ACTIONS(1793),
    [aux_sym_preproc_define_token1] = ACTIONS(1795),
    [aux_sym_preproc_warn_token1] = ACTIONS(1795),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1795),
    [anon_sym_LBRACK] = ACTIONS(1795),
    [anon_sym_LBRACE] = ACTIONS(1795),
    [anon_sym_RBRACE] = ACTIONS(1795),
    [anon_sym_LPAREN] = ACTIONS(1793),
    [anon_sym_BANG] = ACTIONS(1795),
    [anon_sym_TILDE] = ACTIONS(1795),
    [anon_sym_DASH] = ACTIONS(1793),
    [anon_sym_PLUS] = ACTIONS(1793),
    [anon_sym_DASH_DASH] = ACTIONS(1795),
    [anon_sym_PLUS_PLUS] = ACTIONS(1795),
    [anon_sym_DQUOTE] = ACTIONS(1795),
    [anon_sym_SQUOTE] = ACTIONS(1795),
    [sym_number_literal] = ACTIONS(1795),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(469)] = {
    [sym_identifier] = ACTIONS(1793),
    [sym_define_call_keyword] = ACTIONS(1793),
    [sym_define_function_keyword] = ACTIONS(1793),
    [sym_define_library_function_keyword] = ACTIONS(1793),
    [sym_define_module_keyword] = ACTIONS(1793),
    [sym_char_keyword] = ACTIONS(1793),
    [sym_widechar_keyword] = ACTIONS(1793),
    [sym_integer_keyword] = ACTIONS(1793),
    [sym_sinteger_keyword] = ACTIONS(1793),
    [sym_long_keyword] = ACTIONS(1793),
    [sym_slong_keyword] = ACTIONS(1793),
    [sym_float_keyword] = ACTIONS(1793),
    [sym_double_keyword] = ACTIONS(1793),
    [sym_variant_keyword] = ACTIONS(1793),
    [sym_variantarray_keyword] = ACTIONS(1793),
    [sym_local_var_keyword] = ACTIONS(1793),
    [sym_stack_var_keyword] = ACTIONS(1793),
    [sym_constant_keyword] = ACTIONS(1793),
    [sym_volatile_keyword] = ACTIONS(1793),
    [sym_non_volatile_keyword] = ACTIONS(1793),
    [sym_persistent_keyword] = ACTIONS(1793),
    [sym_dev_keyword] = ACTIONS(1793),
    [sym_devlev_keyword] = ACTIONS(1793),
    [sym_devchan_keyword] = ACTIONS(1793),
    [sym_if_keyword] = ACTIONS(1793),
    [sym_else_keyword] = ACTIONS(1793),
    [sym_switch_keyword] = ACTIONS(1793),
    [sym_case_keyword] = ACTIONS(1793),
    [sym_default_keyword] = ACTIONS(1793),
    [sym_while_keyword] = ACTIONS(1793),
    [sym_for_keyword] = ACTIONS(1793),
    [sym_break_keyword] = ACTIONS(1793),
    [sym_continue_keyword] = ACTIONS(1793),
    [sym_return_keyword] = ACTIONS(1793),
    [sym_select_keyword] = ACTIONS(1793),
    [sym_active_keyword] = ACTIONS(1793),
    [sym_struct_keyword] = ACTIONS(1793),
    [sym_structure_keyword] = ACTIONS(1793),
    [aux_sym_on_keyword_token1] = ACTIONS(1793),
    [aux_sym_off_keyword_token1] = ACTIONS(1793),
    [sym_send_level_keyword] = ACTIONS(1793),
    [sym_send_string_keyword] = ACTIONS(1793),
    [sym_send_command_keyword] = ACTIONS(1793),
    [sym_clear_buffer_keyword] = ACTIONS(1793),
    [sym_create_buffer_keyword] = ACTIONS(1793),
    [sym_create_multi_buffer_keyword] = ACTIONS(1793),
    [sym_call_keyword] = ACTIONS(1793),
    [sym_system_call_keyword] = ACTIONS(1793),
    [sym_devchan_to_keyword] = ACTIONS(1793),
    [sym_devchan_min_to_keyword] = ACTIONS(1793),
    [sym_devchan_total_off_keyword] = ACTIONS(1793),
    [sym_devchan_pulse_keyword] = ACTIONS(1793),
    [sym_wait_keyword] = ACTIONS(1793),
    [sym_cancel_wait_keyword] = ACTIONS(1793),
    [sym_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_wait_until_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_keyword] = ACTIONS(1793),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1793),
    [sym_not] = ACTIONS(1793),
    [sym_bnot] = ACTIONS(1793),
    [sym___file__] = ACTIONS(1793),
    [sym___date__] = ACTIONS(1793),
    [sym___ldate__] = ACTIONS(1793),
    [sym___time__] = ACTIONS(1793),
    [sym___line__] = ACTIONS(1793),
    [sym___name__] = ACTIONS(1793),
    [sym___netlinx__] = ACTIONS(1793),
    [sym_netlinx_axi_version] = ACTIONS(1793),
    [sym_url_flg_tcp] = ACTIONS(1793),
    [sym_url_flg_accinfopresent] = ACTIONS(1793),
    [sym_url_flg_temp] = ACTIONS(1793),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1793),
    [sym_url_flg_stat_mask] = ACTIONS(1793),
    [sym_url_flg_stat_lookup] = ACTIONS(1793),
    [sym_url_flg_stat_connecting] = ACTIONS(1793),
    [sym_url_flg_stat_waiting] = ACTIONS(1793),
    [sym_url_flg_stat_connected] = ACTIONS(1793),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1793),
    [sym_first_virtual_device] = ACTIONS(1793),
    [sym_first_local_port] = ACTIONS(1793),
    [sym_dynamic_virtual_device] = ACTIONS(1793),
    [sym_timeline_once] = ACTIONS(1793),
    [sym_timeline_repeat] = ACTIONS(1793),
    [sym_timeline_absolute] = ACTIONS(1793),
    [sym_timeline_relative] = ACTIONS(1793),
    [sym_do_push_timed_infinite] = ACTIONS(1793),
    [sym_source_type_no_address] = ACTIONS(1793),
    [sym_source_type_neuron_id] = ACTIONS(1793),
    [sym_source_type_ip_address] = ACTIONS(1793),
    [sym_source_type_axlink] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1793),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1793),
    [sym_source_type_ip_socket_address] = ACTIONS(1793),
    [sym_source_type_rs232] = ACTIONS(1793),
    [sym_source_type_internal] = ACTIONS(1793),
    [sym_source_type_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1793),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1793),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1793),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1793),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1793),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1793),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1793),
    [sym_true] = ACTIONS(1793),
    [sym_false] = ACTIONS(1793),
    [sym_file_read_only] = ACTIONS(1793),
    [sym_file_rw_new] = ACTIONS(1793),
    [sym_file_rw_append] = ACTIONS(1793),
    [sym_ip_tcp] = ACTIONS(1793),
    [sym_ip_udp] = ACTIONS(1793),
    [sym_ip_udp_2way] = ACTIONS(1793),
    [sym_xml_encode_types] = ACTIONS(1793),
    [sym_xml_encode_char_as_list] = ACTIONS(1793),
    [sym_xml_encode_le] = ACTIONS(1793),
    [sym_xml_decode_types] = ACTIONS(1793),
    [sym_xml_decode_no_preserve] = ACTIONS(1793),
    [sym_duet_dev_type_display_device] = ACTIONS(1793),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1793),
    [sym_duet_dev_type_io_device] = ACTIONS(1793),
    [sym_duet_dev_type_relay_device] = ACTIONS(1793),
    [sym_duet_dev_type_ups] = ACTIONS(1793),
    [sym_duet_dev_type_amplifier] = ACTIONS(1793),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1793),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1793),
    [sym_duet_dev_type_camera] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1793),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1793),
    [sym_duet_dev_type_dss] = ACTIONS(1793),
    [sym_duet_dev_type_dvr] = ACTIONS(1793),
    [sym_duet_dev_type_disc_device] = ACTIONS(1793),
    [sym_duet_dev_type_document_camera] = ACTIONS(1793),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1793),
    [sym_duet_dev_type_hvac] = ACTIONS(1793),
    [sym_duet_dev_type_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_light] = ACTIONS(1793),
    [sym_duet_dev_type_monitor] = ACTIONS(1793),
    [sym_duet_dev_type_motor] = ACTIONS(1793),
    [sym_duet_dev_type_multi_window] = ACTIONS(1793),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1793),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1793),
    [sym_duet_dev_type_receiver] = ACTIONS(1793),
    [sym_duet_dev_type_security_system] = ACTIONS(1793),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1793),
    [sym_duet_dev_type_settop_box] = ACTIONS(1793),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1793),
    [sym_duet_dev_type_switcher] = ACTIONS(1793),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1793),
    [sym_duet_dev_type_tv] = ACTIONS(1793),
    [sym_duet_dev_type_utility] = ACTIONS(1793),
    [sym_duet_dev_type_vcr] = ACTIONS(1793),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1793),
    [sym_duet_dev_type_video_processor] = ACTIONS(1793),
    [sym_duet_dev_type_video_projector] = ACTIONS(1793),
    [sym_duet_dev_type_video_wall] = ACTIONS(1793),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1793),
    [sym_duet_dev_type_weather] = ACTIONS(1793),
    [sym_duet_dev_not_polled] = ACTIONS(1793),
    [sym_duet_dev_polled] = ACTIONS(1793),
    [sym_clkmgr_mode_network] = ACTIONS(1793),
    [sym_clkmgr_mode_standalone] = ACTIONS(1793),
    [sym_normal_standby] = ACTIONS(1793),
    [sym_normal_wake] = ACTIONS(1793),
    [sym_ip_multicast_ttl_option] = ACTIONS(1793),
    [sym_ip_ttl_subnet] = ACTIONS(1793),
    [sym_ip_ttl_site] = ACTIONS(1793),
    [sym_ip_ttl_region] = ACTIONS(1793),
    [sym_ip_ttl_continent] = ACTIONS(1793),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1793),
    [sym_ip_nodelay_on] = ACTIONS(1793),
    [sym_ip_nodelay_off] = ACTIONS(1793),
    [sym_amx_error] = ACTIONS(1793),
    [sym_amx_warning] = ACTIONS(1793),
    [sym_amx_info] = ACTIONS(1793),
    [sym_amx_debug] = ACTIONS(1793),
    [sym_netlinx_login_success] = ACTIONS(1793),
    [sym_netlinx_login_fail] = ACTIONS(1793),
    [sym_netlinx_logout] = ACTIONS(1793),
    [sym_null_str] = ACTIONS(1793),
    [sym_smtp_address] = ACTIONS(1793),
    [sym_smtp_port_number] = ACTIONS(1793),
    [sym_smtp_username] = ACTIONS(1793),
    [sym_smtp_password] = ACTIONS(1793),
    [sym_smtp_require_tls] = ACTIONS(1793),
    [sym_smtp_from] = ACTIONS(1793),
    [sym_smtp_tls_true] = ACTIONS(1793),
    [sym_smtp_tls_false] = ACTIONS(1793),
    [sym_listview_on_row_select_event] = ACTIONS(1793),
    [sym_data_structure_unknown] = ACTIONS(1793),
    [sym_data_structure_datafeed] = ACTIONS(1793),
    [sym_data_structure_datarecordset] = ACTIONS(1793),
    [sym_data_structure_datarecord] = ACTIONS(1793),
    [sym_data_structure_datafield] = ACTIONS(1793),
    [sym_data_max_content_field_count] = ACTIONS(1793),
    [sym_data_max_metadata_field_count] = ACTIONS(1793),
    [sym_data_max_id_length] = ACTIONS(1793),
    [sym_data_max_name_length] = ACTIONS(1793),
    [sym_data_max_description_length] = ACTIONS(1793),
    [sym_data_max_type_length] = ACTIONS(1793),
    [sym_data_max_format_length] = ACTIONS(1793),
    [sym_data_max_source_length] = ACTIONS(1793),
    [sym_data_max_value_length] = ACTIONS(1793),
    [sym_data_max_label_length] = ACTIONS(1793),
    [sym_data_max_url_length] = ACTIONS(1793),
    [sym_max_last_login_info_length] = ACTIONS(1793),
    [sym_data_type_unknown] = ACTIONS(1793),
    [sym_data_type_string] = ACTIONS(1793),
    [sym_data_type_datetime] = ACTIONS(1793),
    [sym_data_type_date] = ACTIONS(1793),
    [sym_data_type_time] = ACTIONS(1793),
    [sym_data_type_image] = ACTIONS(1793),
    [sym_data_format_url] = ACTIONS(1793),
    [sym_data_format_phone] = ACTIONS(1793),
    [sym_data_format_email] = ACTIONS(1793),
    [sym_data_format_iso8601] = ACTIONS(1793),
    [sym_auditlog_priv] = ACTIONS(1793),
    [sym_configuration_priv] = ACTIONS(1793),
    [sym_deviceconfig_priv] = ACTIONS(1793),
    [sym_ftp_priv] = ACTIONS(1793),
    [sym_http_priv] = ACTIONS(1793),
    [sym_networkconfig_priv] = ACTIONS(1793),
    [sym_programport_priv] = ACTIONS(1793),
    [sym_remoteui_priv] = ACTIONS(1793),
    [sym_securitycontrol_priv] = ACTIONS(1793),
    [sym_softwaremanagement_priv] = ACTIONS(1793),
    [sym_terminal_priv] = ACTIONS(1793),
    [sym_tpadmin_priv] = ACTIONS(1793),
    [sym_user1_priv] = ACTIONS(1793),
    [sym_user2_priv] = ACTIONS(1793),
    [sym_user3_priv] = ACTIONS(1793),
    [sym_user4_priv] = ACTIONS(1793),
    [sym_usermanagement_priv] = ACTIONS(1793),
    [sym_valid_account] = ACTIONS(1793),
    [sym_err_name_invalid] = ACTIONS(1793),
    [sym_err_password_invalid] = ACTIONS(1793),
    [sym_err_invalid_account] = ACTIONS(1793),
    [sym_err_no_authorization] = ACTIONS(1793),
    [sym_err_invalid_parameter] = ACTIONS(1793),
    [sym_err_user_locked_out] = ACTIONS(1793),
    [sym_err_user_account_expired] = ACTIONS(1793),
    [sym_err_general_error] = ACTIONS(1793),
    [sym_err_authentication_error] = ACTIONS(1793),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1793),
    [sym_tls_validate_certificate] = ACTIONS(1793),
    [sym_tdata] = ACTIONS(1793),
    [sym_tchannel] = ACTIONS(1793),
    [sym_tlevel] = ACTIONS(1793),
    [sym_tbutton] = ACTIONS(1793),
    [sym_ttimeline] = ACTIONS(1793),
    [sym_tcustom] = ACTIONS(1793),
    [sym_url_struct] = ACTIONS(1793),
    [sym_dns_struct] = ACTIONS(1793),
    [sym_ip_address_struct] = ACTIONS(1793),
    [sym_dev_info_struct] = ACTIONS(1793),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1793),
    [sym_data_feed] = ACTIONS(1793),
    [sym_data_field] = ACTIONS(1793),
    [sym_data_record] = ACTIONS(1793),
    [sym_wc_data_feed] = ACTIONS(1793),
    [sym_wc_data_field] = ACTIONS(1793),
    [sym_wc_data_record] = ACTIONS(1793),
    [sym_last_login_info] = ACTIONS(1793),
    [sym____reserved___] = ACTIONS(1793),
    [sym_date] = ACTIONS(1793),
    [sym_day] = ACTIONS(1793),
    [sym_time] = ACTIONS(1793),
    [sym_ldate] = ACTIONS(1793),
    [sym_push_device] = ACTIONS(1793),
    [sym_push_channel] = ACTIONS(1793),
    [sym_push_devchan] = ACTIONS(1793),
    [sym_release_device] = ACTIONS(1793),
    [sym_release_channel] = ACTIONS(1793),
    [sym_release_devchan] = ACTIONS(1793),
    [sym_master_slot] = ACTIONS(1793),
    [sym_get_pulse_time] = ACTIONS(1793),
    [sym_get_timer] = ACTIONS(1793),
    [sym_master_sn] = ACTIONS(1793),
    [sym_system_number] = ACTIONS(1793),
    [sym_button] = ACTIONS(1793),
    [sym_dv_channel] = ACTIONS(1793),
    [sym_data] = ACTIONS(1793),
    [sym_level] = ACTIONS(1793),
    [sym_timeline] = ACTIONS(1793),
    [sym_channel] = ACTIONS(1793),
    [sym_custom] = ACTIONS(1793),
    [sym_atoi] = ACTIONS(1793),
    [sym_atol] = ACTIONS(1793),
    [sym_atof] = ACTIONS(1793),
    [sym_itoa] = ACTIONS(1793),
    [sym_itohex] = ACTIONS(1793),
    [sym_hextoi] = ACTIONS(1793),
    [sym_ftoa] = ACTIONS(1793),
    [sym_find_string] = ACTIONS(1793),
    [sym_left_string] = ACTIONS(1793),
    [sym_right_string] = ACTIONS(1793),
    [sym_mid_string] = ACTIONS(1793),
    [sym_remove_string] = ACTIONS(1793),
    [sym_get_buffer_string] = ACTIONS(1793),
    [sym_get_buffer_char] = ACTIONS(1793),
    [sym_get_multi_buffer_string] = ACTIONS(1793),
    [sym_length_string] = ACTIONS(1793),
    [sym_max_length_string] = ACTIONS(1793),
    [sym_set_length_string] = ACTIONS(1793),
    [sym_compare_string] = ACTIONS(1793),
    [sym_upper_string] = ACTIONS(1793),
    [sym_lower_string] = ACTIONS(1793),
    [sym_get_last] = ACTIONS(1793),
    [sym_date_to_day] = ACTIONS(1793),
    [sym_date_to_month] = ACTIONS(1793),
    [sym_date_to_year] = ACTIONS(1793),
    [sym_time_to_hour] = ACTIONS(1793),
    [sym_time_to_minute] = ACTIONS(1793),
    [sym_time_to_second] = ACTIONS(1793),
    [sym_day_of_week] = ACTIONS(1793),
    [sym_set_pulse_time] = ACTIONS(1793),
    [sym_set_timer] = ACTIONS(1793),
    [sym_file_open] = ACTIONS(1793),
    [sym_file_close] = ACTIONS(1793),
    [sym_file_read] = ACTIONS(1793),
    [sym_file_read_line] = ACTIONS(1793),
    [sym_file_write] = ACTIONS(1793),
    [sym_file_write_line] = ACTIONS(1793),
    [sym_file_copy] = ACTIONS(1793),
    [sym_file_delete] = ACTIONS(1793),
    [sym_file_rename] = ACTIONS(1793),
    [sym_file_dir] = ACTIONS(1793),
    [sym_file_getdir] = ACTIONS(1793),
    [sym_file_setdir] = ACTIONS(1793),
    [sym_file_create_dir] = ACTIONS(1793),
    [sym_file_remove_dir] = ACTIONS(1793),
    [sym_file_seek] = ACTIONS(1793),
    [sym_length_array] = ACTIONS(1793),
    [sym_max_length_array] = ACTIONS(1793),
    [sym_set_length_array] = ACTIONS(1793),
    [sym_ip_client_open] = ACTIONS(1793),
    [sym_ip_client_close] = ACTIONS(1793),
    [sym_ip_server_open] = ACTIONS(1793),
    [sym_ip_server_close] = ACTIONS(1793),
    [sym_ip_bound_client_open] = ACTIONS(1793),
    [sym_ip_mc_server_open] = ACTIONS(1793),
    [sym_ip_set_option] = ACTIONS(1793),
    [sym_ssh_client_open] = ACTIONS(1793),
    [sym_ssh_client_close] = ACTIONS(1793),
    [sym_tls_client_open] = ACTIONS(1793),
    [sym_tls_client_close] = ACTIONS(1793),
    [sym_do_push] = ACTIONS(1793),
    [sym_do_push_timed] = ACTIONS(1793),
    [sym_do_release] = ACTIONS(1793),
    [sym_device_id] = ACTIONS(1793),
    [sym_device_info] = ACTIONS(1793),
    [sym_device_id_string] = ACTIONS(1793),
    [sym_device_standby] = ACTIONS(1793),
    [sym_device_wake] = ACTIONS(1793),
    [sym_timeline_create] = ACTIONS(1793),
    [sym_timeline_kill] = ACTIONS(1793),
    [sym_timeline_reload] = ACTIONS(1793),
    [sym_timeline_pause] = ACTIONS(1793),
    [sym_timeline_restart] = ACTIONS(1793),
    [sym_timeline_set] = ACTIONS(1793),
    [sym_timeline_get] = ACTIONS(1793),
    [sym_timeline_active] = ACTIONS(1793),
    [sym_timeline_dynamic_id] = ACTIONS(1793),
    [sym_variable_to_string] = ACTIONS(1793),
    [sym_string_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_string] = ACTIONS(1793),
    [sym_type_cast] = ACTIONS(1793),
    [sym_variable_to_xml] = ACTIONS(1793),
    [sym_xml_to_variable] = ACTIONS(1793),
    [sym_length_variable_to_xml] = ACTIONS(1793),
    [sym_redirect_string] = ACTIONS(1793),
    [sym_random_number] = ACTIONS(1793),
    [sym_get_unique_id] = ACTIONS(1793),
    [sym_get_system_number] = ACTIONS(1793),
    [sym_set_system_number] = ACTIONS(1793),
    [sym_get_serial_number] = ACTIONS(1793),
    [sym_get_master_build] = ACTIONS(1793),
    [sym_set_validation_code] = ACTIONS(1793),
    [sym_reboot] = ACTIONS(1793),
    [sym_load_duet_module] = ACTIONS(1793),
    [sym_unload_duet_module] = ACTIONS(1793),
    [sym_duet_mem_size_set] = ACTIONS(1793),
    [sym_duet_mem_size_get] = ACTIONS(1793),
    [sym_dynamic_application_device] = ACTIONS(1793),
    [sym_dynamic_polled_port] = ACTIONS(1793),
    [sym_static_port_binding] = ACTIONS(1793),
    [sym_static_ip_binding] = ACTIONS(1793),
    [sym_get_dns_list] = ACTIONS(1793),
    [sym_set_dns_list] = ACTIONS(1793),
    [sym_get_ip_address] = ACTIONS(1793),
    [sym_set_ip_address] = ACTIONS(1793),
    [sym_get_url_list] = ACTIONS(1793),
    [sym_add_url_entry] = ACTIONS(1793),
    [sym_delete_url_entry] = ACTIONS(1793),
    [sym_abs_value] = ACTIONS(1793),
    [sym_max_value] = ACTIONS(1793),
    [sym_min_value] = ACTIONS(1793),
    [sym_exp_value] = ACTIONS(1793),
    [sym_log_value] = ACTIONS(1793),
    [sym_log10_value] = ACTIONS(1793),
    [sym_power_value] = ACTIONS(1793),
    [sym_sqrt_value] = ACTIONS(1793),
    [sym_raw_be] = ACTIONS(1793),
    [sym_raw_le] = ACTIONS(1793),
    [sym_internal_queue_size_set] = ACTIONS(1793),
    [sym_internal_queue_size_get] = ACTIONS(1793),
    [sym_internal_threshold_set] = ACTIONS(1793),
    [sym_internal_threshold_get] = ACTIONS(1793),
    [sym_rebuilt_event] = ACTIONS(1793),
    [sym_do_custom_event] = ACTIONS(1793),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1793),
    [sym_clkmgr_set_clk_source] = ACTIONS(1793),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1793),
    [sym_clkmgr_get_timezone] = ACTIONS(1793),
    [sym_clkmgr_set_timezone] = ACTIONS(1793),
    [sym_clkmgr_get_resync_period] = ACTIONS(1793),
    [sym_clkmgr_set_resync_period] = ACTIONS(1793),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1793),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_timeservers] = ACTIONS(1793),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1793),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1793),
    [sym_set_log_level] = ACTIONS(1793),
    [sym_get_log_level] = ACTIONS(1793),
    [sym_amx_log] = ACTIONS(1793),
    [sym_smtp_server_config_set] = ACTIONS(1793),
    [sym_smtp_server_config_get] = ACTIONS(1793),
    [sym_smtp_send] = ACTIONS(1793),
    [sym_validate_netlinx_account] = ACTIONS(1793),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1793),
    [sym_authenticate_certificate] = ACTIONS(1793),
    [sym_audit_netlinx_session_event] = ACTIONS(1793),
    [sym_audit_netlinx_generic_event] = ACTIONS(1793),
    [sym_get_available_flash_disk_space] = ACTIONS(1793),
    [sym_get_max_flash_disk_space] = ACTIONS(1793),
    [sym_data_create_feed] = ACTIONS(1793),
    [sym_data_delete_feed] = ACTIONS(1793),
    [sym_data_publish_feed] = ACTIONS(1793),
    [sym_data_get_published_feed] = ACTIONS(1793),
    [sym_data_add_record] = ACTIONS(1793),
    [sym_data_get_event_record] = ACTIONS(1793),
    [sym__wc_data_create_feed] = ACTIONS(1793),
    [sym__wc_data_add_record] = ACTIONS(1793),
    [sym__wc_data_get_event_record] = ACTIONS(1793),
    [sym_format] = ACTIONS(1793),
    [sym_astro_clock] = ACTIONS(1793),
    [sym_set_outdoor_temperature] = ACTIONS(1793),
    [sym_set_virtual_level_count] = ACTIONS(1793),
    [sym_set_virtual_channel_count] = ACTIONS(1793),
    [sym_set_virtual_port_count] = ACTIONS(1793),
    [sym__wc_encode_int] = ACTIONS(1793),
    [sym__wc_decode_int] = ACTIONS(1793),
    [sym__wc_int] = ACTIONS(1793),
    [sym__wc_to_ch_int] = ACTIONS(1793),
    [sym__ch_to_wc_int] = ACTIONS(1793),
    [sym__wc_find_string_int] = ACTIONS(1793),
    [sym__wc_left_string_int] = ACTIONS(1793),
    [sym__wc_lower_string_int] = ACTIONS(1793),
    [sym__wc_mid_string_int] = ACTIONS(1793),
    [sym__wc_remove_string_int] = ACTIONS(1793),
    [sym__wc_right_string_int] = ACTIONS(1793),
    [sym__wc_upper_string_int] = ACTIONS(1793),
    [sym__wc_compare_string_int] = ACTIONS(1793),
    [sym__wc_get_buffer_char_int] = ACTIONS(1793),
    [sym__wc_get_buffer_string_int] = ACTIONS(1793),
    [sym__wc_concat_string_int] = ACTIONS(1793),
    [sym__wc_file_open_int] = ACTIONS(1793),
    [sym__wc_file_close_int] = ACTIONS(1793),
    [sym__wc_file_read_int] = ACTIONS(1793),
    [sym__wc_file_read_line_int] = ACTIONS(1793),
    [sym__wc_file_write_int] = ACTIONS(1793),
    [sym__wc_file_write_line_int] = ACTIONS(1793),
    [aux_sym_preproc_include_token1] = ACTIONS(1793),
    [aux_sym_preproc_define_token1] = ACTIONS(1795),
    [aux_sym_preproc_warn_token1] = ACTIONS(1795),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1795),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1795),
    [anon_sym_LBRACK] = ACTIONS(1795),
    [anon_sym_LBRACE] = ACTIONS(1795),
    [anon_sym_RBRACE] = ACTIONS(1795),
    [anon_sym_LPAREN] = ACTIONS(1793),
    [anon_sym_BANG] = ACTIONS(1795),
    [anon_sym_TILDE] = ACTIONS(1795),
    [anon_sym_DASH] = ACTIONS(1793),
    [anon_sym_PLUS] = ACTIONS(1793),
    [anon_sym_DASH_DASH] = ACTIONS(1795),
    [anon_sym_PLUS_PLUS] = ACTIONS(1795),
    [anon_sym_DQUOTE] = ACTIONS(1795),
    [anon_sym_SQUOTE] = ACTIONS(1795),
    [sym_number_literal] = ACTIONS(1795),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(470)] = {
    [sym_identifier] = ACTIONS(1759),
    [sym_define_call_keyword] = ACTIONS(1759),
    [sym_define_function_keyword] = ACTIONS(1759),
    [sym_define_library_function_keyword] = ACTIONS(1759),
    [sym_define_module_keyword] = ACTIONS(1759),
    [sym_char_keyword] = ACTIONS(1759),
    [sym_widechar_keyword] = ACTIONS(1759),
    [sym_integer_keyword] = ACTIONS(1759),
    [sym_sinteger_keyword] = ACTIONS(1759),
    [sym_long_keyword] = ACTIONS(1759),
    [sym_slong_keyword] = ACTIONS(1759),
    [sym_float_keyword] = ACTIONS(1759),
    [sym_double_keyword] = ACTIONS(1759),
    [sym_variant_keyword] = ACTIONS(1759),
    [sym_variantarray_keyword] = ACTIONS(1759),
    [sym_local_var_keyword] = ACTIONS(1759),
    [sym_stack_var_keyword] = ACTIONS(1759),
    [sym_constant_keyword] = ACTIONS(1759),
    [sym_volatile_keyword] = ACTIONS(1759),
    [sym_non_volatile_keyword] = ACTIONS(1759),
    [sym_persistent_keyword] = ACTIONS(1759),
    [sym_dev_keyword] = ACTIONS(1759),
    [sym_devlev_keyword] = ACTIONS(1759),
    [sym_devchan_keyword] = ACTIONS(1759),
    [sym_if_keyword] = ACTIONS(1759),
    [sym_else_keyword] = ACTIONS(1759),
    [sym_switch_keyword] = ACTIONS(1759),
    [sym_case_keyword] = ACTIONS(1759),
    [sym_default_keyword] = ACTIONS(1759),
    [sym_while_keyword] = ACTIONS(1759),
    [sym_for_keyword] = ACTIONS(1759),
    [sym_break_keyword] = ACTIONS(1759),
    [sym_continue_keyword] = ACTIONS(1759),
    [sym_return_keyword] = ACTIONS(1759),
    [sym_select_keyword] = ACTIONS(1759),
    [sym_active_keyword] = ACTIONS(1759),
    [sym_struct_keyword] = ACTIONS(1759),
    [sym_structure_keyword] = ACTIONS(1759),
    [aux_sym_on_keyword_token1] = ACTIONS(1759),
    [aux_sym_off_keyword_token1] = ACTIONS(1759),
    [sym_send_level_keyword] = ACTIONS(1759),
    [sym_send_string_keyword] = ACTIONS(1759),
    [sym_send_command_keyword] = ACTIONS(1759),
    [sym_clear_buffer_keyword] = ACTIONS(1759),
    [sym_create_buffer_keyword] = ACTIONS(1759),
    [sym_create_multi_buffer_keyword] = ACTIONS(1759),
    [sym_call_keyword] = ACTIONS(1759),
    [sym_system_call_keyword] = ACTIONS(1759),
    [sym_devchan_to_keyword] = ACTIONS(1759),
    [sym_devchan_min_to_keyword] = ACTIONS(1759),
    [sym_devchan_total_off_keyword] = ACTIONS(1759),
    [sym_devchan_pulse_keyword] = ACTIONS(1759),
    [sym_wait_keyword] = ACTIONS(1759),
    [sym_cancel_wait_keyword] = ACTIONS(1759),
    [sym_wait_until_keyword] = ACTIONS(1759),
    [sym_cancel_wait_until_keyword] = ACTIONS(1759),
    [sym_cancel_all_wait_keyword] = ACTIONS(1759),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1759),
    [sym_not] = ACTIONS(1759),
    [sym_bnot] = ACTIONS(1759),
    [sym___file__] = ACTIONS(1759),
    [sym___date__] = ACTIONS(1759),
    [sym___ldate__] = ACTIONS(1759),
    [sym___time__] = ACTIONS(1759),
    [sym___line__] = ACTIONS(1759),
    [sym___name__] = ACTIONS(1759),
    [sym___netlinx__] = ACTIONS(1759),
    [sym_netlinx_axi_version] = ACTIONS(1759),
    [sym_url_flg_tcp] = ACTIONS(1759),
    [sym_url_flg_accinfopresent] = ACTIONS(1759),
    [sym_url_flg_temp] = ACTIONS(1759),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1759),
    [sym_url_flg_stat_mask] = ACTIONS(1759),
    [sym_url_flg_stat_lookup] = ACTIONS(1759),
    [sym_url_flg_stat_connecting] = ACTIONS(1759),
    [sym_url_flg_stat_waiting] = ACTIONS(1759),
    [sym_url_flg_stat_connected] = ACTIONS(1759),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1759),
    [sym_first_virtual_device] = ACTIONS(1759),
    [sym_first_local_port] = ACTIONS(1759),
    [sym_dynamic_virtual_device] = ACTIONS(1759),
    [sym_timeline_once] = ACTIONS(1759),
    [sym_timeline_repeat] = ACTIONS(1759),
    [sym_timeline_absolute] = ACTIONS(1759),
    [sym_timeline_relative] = ACTIONS(1759),
    [sym_do_push_timed_infinite] = ACTIONS(1759),
    [sym_source_type_no_address] = ACTIONS(1759),
    [sym_source_type_neuron_id] = ACTIONS(1759),
    [sym_source_type_ip_address] = ACTIONS(1759),
    [sym_source_type_axlink] = ACTIONS(1759),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1759),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1759),
    [sym_source_type_ip_socket_address] = ACTIONS(1759),
    [sym_source_type_rs232] = ACTIONS(1759),
    [sym_source_type_internal] = ACTIONS(1759),
    [sym_source_type_mac_address] = ACTIONS(1759),
    [sym_source_type_ipv4_port] = ACTIONS(1759),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1759),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1759),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1759),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1759),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1759),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1759),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1759),
    [sym_true] = ACTIONS(1759),
    [sym_false] = ACTIONS(1759),
    [sym_file_read_only] = ACTIONS(1759),
    [sym_file_rw_new] = ACTIONS(1759),
    [sym_file_rw_append] = ACTIONS(1759),
    [sym_ip_tcp] = ACTIONS(1759),
    [sym_ip_udp] = ACTIONS(1759),
    [sym_ip_udp_2way] = ACTIONS(1759),
    [sym_xml_encode_types] = ACTIONS(1759),
    [sym_xml_encode_char_as_list] = ACTIONS(1759),
    [sym_xml_encode_le] = ACTIONS(1759),
    [sym_xml_decode_types] = ACTIONS(1759),
    [sym_xml_decode_no_preserve] = ACTIONS(1759),
    [sym_duet_dev_type_display_device] = ACTIONS(1759),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1759),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1759),
    [sym_duet_dev_type_io_device] = ACTIONS(1759),
    [sym_duet_dev_type_relay_device] = ACTIONS(1759),
    [sym_duet_dev_type_ups] = ACTIONS(1759),
    [sym_duet_dev_type_amplifier] = ACTIONS(1759),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1759),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1759),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1759),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1759),
    [sym_duet_dev_type_camera] = ACTIONS(1759),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1759),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1759),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1759),
    [sym_duet_dev_type_dss] = ACTIONS(1759),
    [sym_duet_dev_type_dvr] = ACTIONS(1759),
    [sym_duet_dev_type_disc_device] = ACTIONS(1759),
    [sym_duet_dev_type_document_camera] = ACTIONS(1759),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1759),
    [sym_duet_dev_type_hvac] = ACTIONS(1759),
    [sym_duet_dev_type_keypad] = ACTIONS(1759),
    [sym_duet_dev_type_light] = ACTIONS(1759),
    [sym_duet_dev_type_monitor] = ACTIONS(1759),
    [sym_duet_dev_type_motor] = ACTIONS(1759),
    [sym_duet_dev_type_multi_window] = ACTIONS(1759),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1759),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1759),
    [sym_duet_dev_type_receiver] = ACTIONS(1759),
    [sym_duet_dev_type_security_system] = ACTIONS(1759),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1759),
    [sym_duet_dev_type_settop_box] = ACTIONS(1759),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1759),
    [sym_duet_dev_type_switcher] = ACTIONS(1759),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1759),
    [sym_duet_dev_type_tv] = ACTIONS(1759),
    [sym_duet_dev_type_utility] = ACTIONS(1759),
    [sym_duet_dev_type_vcr] = ACTIONS(1759),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1759),
    [sym_duet_dev_type_video_processor] = ACTIONS(1759),
    [sym_duet_dev_type_video_projector] = ACTIONS(1759),
    [sym_duet_dev_type_video_wall] = ACTIONS(1759),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1759),
    [sym_duet_dev_type_weather] = ACTIONS(1759),
    [sym_duet_dev_not_polled] = ACTIONS(1759),
    [sym_duet_dev_polled] = ACTIONS(1759),
    [sym_clkmgr_mode_network] = ACTIONS(1759),
    [sym_clkmgr_mode_standalone] = ACTIONS(1759),
    [sym_normal_standby] = ACTIONS(1759),
    [sym_normal_wake] = ACTIONS(1759),
    [sym_ip_multicast_ttl_option] = ACTIONS(1759),
    [sym_ip_ttl_subnet] = ACTIONS(1759),
    [sym_ip_ttl_site] = ACTIONS(1759),
    [sym_ip_ttl_region] = ACTIONS(1759),
    [sym_ip_ttl_continent] = ACTIONS(1759),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1759),
    [sym_ip_nodelay_on] = ACTIONS(1759),
    [sym_ip_nodelay_off] = ACTIONS(1759),
    [sym_amx_error] = ACTIONS(1759),
    [sym_amx_warning] = ACTIONS(1759),
    [sym_amx_info] = ACTIONS(1759),
    [sym_amx_debug] = ACTIONS(1759),
    [sym_netlinx_login_success] = ACTIONS(1759),
    [sym_netlinx_login_fail] = ACTIONS(1759),
    [sym_netlinx_logout] = ACTIONS(1759),
    [sym_null_str] = ACTIONS(1759),
    [sym_smtp_address] = ACTIONS(1759),
    [sym_smtp_port_number] = ACTIONS(1759),
    [sym_smtp_username] = ACTIONS(1759),
    [sym_smtp_password] = ACTIONS(1759),
    [sym_smtp_require_tls] = ACTIONS(1759),
    [sym_smtp_from] = ACTIONS(1759),
    [sym_smtp_tls_true] = ACTIONS(1759),
    [sym_smtp_tls_false] = ACTIONS(1759),
    [sym_listview_on_row_select_event] = ACTIONS(1759),
    [sym_data_structure_unknown] = ACTIONS(1759),
    [sym_data_structure_datafeed] = ACTIONS(1759),
    [sym_data_structure_datarecordset] = ACTIONS(1759),
    [sym_data_structure_datarecord] = ACTIONS(1759),
    [sym_data_structure_datafield] = ACTIONS(1759),
    [sym_data_max_content_field_count] = ACTIONS(1759),
    [sym_data_max_metadata_field_count] = ACTIONS(1759),
    [sym_data_max_id_length] = ACTIONS(1759),
    [sym_data_max_name_length] = ACTIONS(1759),
    [sym_data_max_description_length] = ACTIONS(1759),
    [sym_data_max_type_length] = ACTIONS(1759),
    [sym_data_max_format_length] = ACTIONS(1759),
    [sym_data_max_source_length] = ACTIONS(1759),
    [sym_data_max_value_length] = ACTIONS(1759),
    [sym_data_max_label_length] = ACTIONS(1759),
    [sym_data_max_url_length] = ACTIONS(1759),
    [sym_max_last_login_info_length] = ACTIONS(1759),
    [sym_data_type_unknown] = ACTIONS(1759),
    [sym_data_type_string] = ACTIONS(1759),
    [sym_data_type_datetime] = ACTIONS(1759),
    [sym_data_type_date] = ACTIONS(1759),
    [sym_data_type_time] = ACTIONS(1759),
    [sym_data_type_image] = ACTIONS(1759),
    [sym_data_format_url] = ACTIONS(1759),
    [sym_data_format_phone] = ACTIONS(1759),
    [sym_data_format_email] = ACTIONS(1759),
    [sym_data_format_iso8601] = ACTIONS(1759),
    [sym_auditlog_priv] = ACTIONS(1759),
    [sym_configuration_priv] = ACTIONS(1759),
    [sym_deviceconfig_priv] = ACTIONS(1759),
    [sym_ftp_priv] = ACTIONS(1759),
    [sym_http_priv] = ACTIONS(1759),
    [sym_networkconfig_priv] = ACTIONS(1759),
    [sym_programport_priv] = ACTIONS(1759),
    [sym_remoteui_priv] = ACTIONS(1759),
    [sym_securitycontrol_priv] = ACTIONS(1759),
    [sym_softwaremanagement_priv] = ACTIONS(1759),
    [sym_terminal_priv] = ACTIONS(1759),
    [sym_tpadmin_priv] = ACTIONS(1759),
    [sym_user1_priv] = ACTIONS(1759),
    [sym_user2_priv] = ACTIONS(1759),
    [sym_user3_priv] = ACTIONS(1759),
    [sym_user4_priv] = ACTIONS(1759),
    [sym_usermanagement_priv] = ACTIONS(1759),
    [sym_valid_account] = ACTIONS(1759),
    [sym_err_name_invalid] = ACTIONS(1759),
    [sym_err_password_invalid] = ACTIONS(1759),
    [sym_err_invalid_account] = ACTIONS(1759),
    [sym_err_no_authorization] = ACTIONS(1759),
    [sym_err_invalid_parameter] = ACTIONS(1759),
    [sym_err_user_locked_out] = ACTIONS(1759),
    [sym_err_user_account_expired] = ACTIONS(1759),
    [sym_err_general_error] = ACTIONS(1759),
    [sym_err_authentication_error] = ACTIONS(1759),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1759),
    [sym_tls_validate_certificate] = ACTIONS(1759),
    [sym_tdata] = ACTIONS(1759),
    [sym_tchannel] = ACTIONS(1759),
    [sym_tlevel] = ACTIONS(1759),
    [sym_tbutton] = ACTIONS(1759),
    [sym_ttimeline] = ACTIONS(1759),
    [sym_tcustom] = ACTIONS(1759),
    [sym_url_struct] = ACTIONS(1759),
    [sym_dns_struct] = ACTIONS(1759),
    [sym_ip_address_struct] = ACTIONS(1759),
    [sym_dev_info_struct] = ACTIONS(1759),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1759),
    [sym_data_feed] = ACTIONS(1759),
    [sym_data_field] = ACTIONS(1759),
    [sym_data_record] = ACTIONS(1759),
    [sym_wc_data_feed] = ACTIONS(1759),
    [sym_wc_data_field] = ACTIONS(1759),
    [sym_wc_data_record] = ACTIONS(1759),
    [sym_last_login_info] = ACTIONS(1759),
    [sym____reserved___] = ACTIONS(1759),
    [sym_date] = ACTIONS(1759),
    [sym_day] = ACTIONS(1759),
    [sym_time] = ACTIONS(1759),
    [sym_ldate] = ACTIONS(1759),
    [sym_push_device] = ACTIONS(1759),
    [sym_push_channel] = ACTIONS(1759),
    [sym_push_devchan] = ACTIONS(1759),
    [sym_release_device] = ACTIONS(1759),
    [sym_release_channel] = ACTIONS(1759),
    [sym_release_devchan] = ACTIONS(1759),
    [sym_master_slot] = ACTIONS(1759),
    [sym_get_pulse_time] = ACTIONS(1759),
    [sym_get_timer] = ACTIONS(1759),
    [sym_master_sn] = ACTIONS(1759),
    [sym_system_number] = ACTIONS(1759),
    [sym_button] = ACTIONS(1759),
    [sym_dv_channel] = ACTIONS(1759),
    [sym_data] = ACTIONS(1759),
    [sym_level] = ACTIONS(1759),
    [sym_timeline] = ACTIONS(1759),
    [sym_channel] = ACTIONS(1759),
    [sym_custom] = ACTIONS(1759),
    [sym_atoi] = ACTIONS(1759),
    [sym_atol] = ACTIONS(1759),
    [sym_atof] = ACTIONS(1759),
    [sym_itoa] = ACTIONS(1759),
    [sym_itohex] = ACTIONS(1759),
    [sym_hextoi] = ACTIONS(1759),
    [sym_ftoa] = ACTIONS(1759),
    [sym_find_string] = ACTIONS(1759),
    [sym_left_string] = ACTIONS(1759),
    [sym_right_string] = ACTIONS(1759),
    [sym_mid_string] = ACTIONS(1759),
    [sym_remove_string] = ACTIONS(1759),
    [sym_get_buffer_string] = ACTIONS(1759),
    [sym_get_buffer_char] = ACTIONS(1759),
    [sym_get_multi_buffer_string] = ACTIONS(1759),
    [sym_length_string] = ACTIONS(1759),
    [sym_max_length_string] = ACTIONS(1759),
    [sym_set_length_string] = ACTIONS(1759),
    [sym_compare_string] = ACTIONS(1759),
    [sym_upper_string] = ACTIONS(1759),
    [sym_lower_string] = ACTIONS(1759),
    [sym_get_last] = ACTIONS(1759),
    [sym_date_to_day] = ACTIONS(1759),
    [sym_date_to_month] = ACTIONS(1759),
    [sym_date_to_year] = ACTIONS(1759),
    [sym_time_to_hour] = ACTIONS(1759),
    [sym_time_to_minute] = ACTIONS(1759),
    [sym_time_to_second] = ACTIONS(1759),
    [sym_day_of_week] = ACTIONS(1759),
    [sym_set_pulse_time] = ACTIONS(1759),
    [sym_set_timer] = ACTIONS(1759),
    [sym_file_open] = ACTIONS(1759),
    [sym_file_close] = ACTIONS(1759),
    [sym_file_read] = ACTIONS(1759),
    [sym_file_read_line] = ACTIONS(1759),
    [sym_file_write] = ACTIONS(1759),
    [sym_file_write_line] = ACTIONS(1759),
    [sym_file_copy] = ACTIONS(1759),
    [sym_file_delete] = ACTIONS(1759),
    [sym_file_rename] = ACTIONS(1759),
    [sym_file_dir] = ACTIONS(1759),
    [sym_file_getdir] = ACTIONS(1759),
    [sym_file_setdir] = ACTIONS(1759),
    [sym_file_create_dir] = ACTIONS(1759),
    [sym_file_remove_dir] = ACTIONS(1759),
    [sym_file_seek] = ACTIONS(1759),
    [sym_length_array] = ACTIONS(1759),
    [sym_max_length_array] = ACTIONS(1759),
    [sym_set_length_array] = ACTIONS(1759),
    [sym_ip_client_open] = ACTIONS(1759),
    [sym_ip_client_close] = ACTIONS(1759),
    [sym_ip_server_open] = ACTIONS(1759),
    [sym_ip_server_close] = ACTIONS(1759),
    [sym_ip_bound_client_open] = ACTIONS(1759),
    [sym_ip_mc_server_open] = ACTIONS(1759),
    [sym_ip_set_option] = ACTIONS(1759),
    [sym_ssh_client_open] = ACTIONS(1759),
    [sym_ssh_client_close] = ACTIONS(1759),
    [sym_tls_client_open] = ACTIONS(1759),
    [sym_tls_client_close] = ACTIONS(1759),
    [sym_do_push] = ACTIONS(1759),
    [sym_do_push_timed] = ACTIONS(1759),
    [sym_do_release] = ACTIONS(1759),
    [sym_device_id] = ACTIONS(1759),
    [sym_device_info] = ACTIONS(1759),
    [sym_device_id_string] = ACTIONS(1759),
    [sym_device_standby] = ACTIONS(1759),
    [sym_device_wake] = ACTIONS(1759),
    [sym_timeline_create] = ACTIONS(1759),
    [sym_timeline_kill] = ACTIONS(1759),
    [sym_timeline_reload] = ACTIONS(1759),
    [sym_timeline_pause] = ACTIONS(1759),
    [sym_timeline_restart] = ACTIONS(1759),
    [sym_timeline_set] = ACTIONS(1759),
    [sym_timeline_get] = ACTIONS(1759),
    [sym_timeline_active] = ACTIONS(1759),
    [sym_timeline_dynamic_id] = ACTIONS(1759),
    [sym_variable_to_string] = ACTIONS(1759),
    [sym_string_to_variable] = ACTIONS(1759),
    [sym_length_variable_to_string] = ACTIONS(1759),
    [sym_type_cast] = ACTIONS(1759),
    [sym_variable_to_xml] = ACTIONS(1759),
    [sym_xml_to_variable] = ACTIONS(1759),
    [sym_length_variable_to_xml] = ACTIONS(1759),
    [sym_redirect_string] = ACTIONS(1759),
    [sym_random_number] = ACTIONS(1759),
    [sym_get_unique_id] = ACTIONS(1759),
    [sym_get_system_number] = ACTIONS(1759),
    [sym_set_system_number] = ACTIONS(1759),
    [sym_get_serial_number] = ACTIONS(1759),
    [sym_get_master_build] = ACTIONS(1759),
    [sym_set_validation_code] = ACTIONS(1759),
    [sym_reboot] = ACTIONS(1759),
    [sym_load_duet_module] = ACTIONS(1759),
    [sym_unload_duet_module] = ACTIONS(1759),
    [sym_duet_mem_size_set] = ACTIONS(1759),
    [sym_duet_mem_size_get] = ACTIONS(1759),
    [sym_dynamic_application_device] = ACTIONS(1759),
    [sym_dynamic_polled_port] = ACTIONS(1759),
    [sym_static_port_binding] = ACTIONS(1759),
    [sym_static_ip_binding] = ACTIONS(1759),
    [sym_get_dns_list] = ACTIONS(1759),
    [sym_set_dns_list] = ACTIONS(1759),
    [sym_get_ip_address] = ACTIONS(1759),
    [sym_set_ip_address] = ACTIONS(1759),
    [sym_get_url_list] = ACTIONS(1759),
    [sym_add_url_entry] = ACTIONS(1759),
    [sym_delete_url_entry] = ACTIONS(1759),
    [sym_abs_value] = ACTIONS(1759),
    [sym_max_value] = ACTIONS(1759),
    [sym_min_value] = ACTIONS(1759),
    [sym_exp_value] = ACTIONS(1759),
    [sym_log_value] = ACTIONS(1759),
    [sym_log10_value] = ACTIONS(1759),
    [sym_power_value] = ACTIONS(1759),
    [sym_sqrt_value] = ACTIONS(1759),
    [sym_raw_be] = ACTIONS(1759),
    [sym_raw_le] = ACTIONS(1759),
    [sym_internal_queue_size_set] = ACTIONS(1759),
    [sym_internal_queue_size_get] = ACTIONS(1759),
    [sym_internal_threshold_set] = ACTIONS(1759),
    [sym_internal_threshold_get] = ACTIONS(1759),
    [sym_rebuilt_event] = ACTIONS(1759),
    [sym_do_custom_event] = ACTIONS(1759),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1759),
    [sym_clkmgr_set_clk_source] = ACTIONS(1759),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1759),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1759),
    [sym_clkmgr_get_timezone] = ACTIONS(1759),
    [sym_clkmgr_set_timezone] = ACTIONS(1759),
    [sym_clkmgr_get_resync_period] = ACTIONS(1759),
    [sym_clkmgr_set_resync_period] = ACTIONS(1759),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1759),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1759),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1759),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1759),
    [sym_clkmgr_get_timeservers] = ACTIONS(1759),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1759),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1759),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1759),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1759),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1759),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1759),
    [sym_set_log_level] = ACTIONS(1759),
    [sym_get_log_level] = ACTIONS(1759),
    [sym_amx_log] = ACTIONS(1759),
    [sym_smtp_server_config_set] = ACTIONS(1759),
    [sym_smtp_server_config_get] = ACTIONS(1759),
    [sym_smtp_send] = ACTIONS(1759),
    [sym_validate_netlinx_account] = ACTIONS(1759),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1759),
    [sym_authenticate_certificate] = ACTIONS(1759),
    [sym_audit_netlinx_session_event] = ACTIONS(1759),
    [sym_audit_netlinx_generic_event] = ACTIONS(1759),
    [sym_get_available_flash_disk_space] = ACTIONS(1759),
    [sym_get_max_flash_disk_space] = ACTIONS(1759),
    [sym_data_create_feed] = ACTIONS(1759),
    [sym_data_delete_feed] = ACTIONS(1759),
    [sym_data_publish_feed] = ACTIONS(1759),
    [sym_data_get_published_feed] = ACTIONS(1759),
    [sym_data_add_record] = ACTIONS(1759),
    [sym_data_get_event_record] = ACTIONS(1759),
    [sym__wc_data_create_feed] = ACTIONS(1759),
    [sym__wc_data_add_record] = ACTIONS(1759),
    [sym__wc_data_get_event_record] = ACTIONS(1759),
    [sym_format] = ACTIONS(1759),
    [sym_astro_clock] = ACTIONS(1759),
    [sym_set_outdoor_temperature] = ACTIONS(1759),
    [sym_set_virtual_level_count] = ACTIONS(1759),
    [sym_set_virtual_channel_count] = ACTIONS(1759),
    [sym_set_virtual_port_count] = ACTIONS(1759),
    [sym__wc_encode_int] = ACTIONS(1759),
    [sym__wc_decode_int] = ACTIONS(1759),
    [sym__wc_int] = ACTIONS(1759),
    [sym__wc_to_ch_int] = ACTIONS(1759),
    [sym__ch_to_wc_int] = ACTIONS(1759),
    [sym__wc_find_string_int] = ACTIONS(1759),
    [sym__wc_left_string_int] = ACTIONS(1759),
    [sym__wc_lower_string_int] = ACTIONS(1759),
    [sym__wc_mid_string_int] = ACTIONS(1759),
    [sym__wc_remove_string_int] = ACTIONS(1759),
    [sym__wc_right_string_int] = ACTIONS(1759),
    [sym__wc_upper_string_int] = ACTIONS(1759),
    [sym__wc_compare_string_int] = ACTIONS(1759),
    [sym__wc_get_buffer_char_int] = ACTIONS(1759),
    [sym__wc_get_buffer_string_int] = ACTIONS(1759),
    [sym__wc_concat_string_int] = ACTIONS(1759),
    [sym__wc_file_open_int] = ACTIONS(1759),
    [sym__wc_file_close_int] = ACTIONS(1759),
    [sym__wc_file_read_int] = ACTIONS(1759),
    [sym__wc_file_read_line_int] = ACTIONS(1759),
    [sym__wc_file_write_int] = ACTIONS(1759),
    [sym__wc_file_write_line_int] = ACTIONS(1759),
    [aux_sym_preproc_include_token1] = ACTIONS(1759),
    [aux_sym_preproc_define_token1] = ACTIONS(1761),
    [aux_sym_preproc_warn_token1] = ACTIONS(1761),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1761),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1761),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1761),
    [anon_sym_LBRACK] = ACTIONS(1761),
    [anon_sym_LBRACE] = ACTIONS(1761),
    [anon_sym_RBRACE] = ACTIONS(1761),
    [anon_sym_LPAREN] = ACTIONS(1759),
    [anon_sym_BANG] = ACTIONS(1761),
    [anon_sym_TILDE] = ACTIONS(1761),
    [anon_sym_DASH] = ACTIONS(1759),
    [anon_sym_PLUS] = ACTIONS(1759),
    [anon_sym_DASH_DASH] = ACTIONS(1761),
    [anon_sym_PLUS_PLUS] = ACTIONS(1761),
    [anon_sym_DQUOTE] = ACTIONS(1761),
    [anon_sym_SQUOTE] = ACTIONS(1761),
    [sym_number_literal] = ACTIONS(1761),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(471)] = {
    [sym_identifier] = ACTIONS(1851),
    [sym_define_call_keyword] = ACTIONS(1851),
    [sym_define_function_keyword] = ACTIONS(1851),
    [sym_define_library_function_keyword] = ACTIONS(1851),
    [sym_define_module_keyword] = ACTIONS(1851),
    [sym_char_keyword] = ACTIONS(1851),
    [sym_widechar_keyword] = ACTIONS(1851),
    [sym_integer_keyword] = ACTIONS(1851),
    [sym_sinteger_keyword] = ACTIONS(1851),
    [sym_long_keyword] = ACTIONS(1851),
    [sym_slong_keyword] = ACTIONS(1851),
    [sym_float_keyword] = ACTIONS(1851),
    [sym_double_keyword] = ACTIONS(1851),
    [sym_variant_keyword] = ACTIONS(1851),
    [sym_variantarray_keyword] = ACTIONS(1851),
    [sym_local_var_keyword] = ACTIONS(1851),
    [sym_stack_var_keyword] = ACTIONS(1851),
    [sym_constant_keyword] = ACTIONS(1851),
    [sym_volatile_keyword] = ACTIONS(1851),
    [sym_non_volatile_keyword] = ACTIONS(1851),
    [sym_persistent_keyword] = ACTIONS(1851),
    [sym_dev_keyword] = ACTIONS(1851),
    [sym_devlev_keyword] = ACTIONS(1851),
    [sym_devchan_keyword] = ACTIONS(1851),
    [sym_if_keyword] = ACTIONS(1851),
    [sym_else_keyword] = ACTIONS(1851),
    [sym_switch_keyword] = ACTIONS(1851),
    [sym_case_keyword] = ACTIONS(1851),
    [sym_default_keyword] = ACTIONS(1851),
    [sym_while_keyword] = ACTIONS(1851),
    [sym_for_keyword] = ACTIONS(1851),
    [sym_break_keyword] = ACTIONS(1851),
    [sym_continue_keyword] = ACTIONS(1851),
    [sym_return_keyword] = ACTIONS(1851),
    [sym_select_keyword] = ACTIONS(1851),
    [sym_active_keyword] = ACTIONS(1851),
    [sym_struct_keyword] = ACTIONS(1851),
    [sym_structure_keyword] = ACTIONS(1851),
    [aux_sym_on_keyword_token1] = ACTIONS(1851),
    [aux_sym_off_keyword_token1] = ACTIONS(1851),
    [sym_send_level_keyword] = ACTIONS(1851),
    [sym_send_string_keyword] = ACTIONS(1851),
    [sym_send_command_keyword] = ACTIONS(1851),
    [sym_clear_buffer_keyword] = ACTIONS(1851),
    [sym_create_buffer_keyword] = ACTIONS(1851),
    [sym_create_multi_buffer_keyword] = ACTIONS(1851),
    [sym_call_keyword] = ACTIONS(1851),
    [sym_system_call_keyword] = ACTIONS(1851),
    [sym_devchan_to_keyword] = ACTIONS(1851),
    [sym_devchan_min_to_keyword] = ACTIONS(1851),
    [sym_devchan_total_off_keyword] = ACTIONS(1851),
    [sym_devchan_pulse_keyword] = ACTIONS(1851),
    [sym_wait_keyword] = ACTIONS(1851),
    [sym_cancel_wait_keyword] = ACTIONS(1851),
    [sym_wait_until_keyword] = ACTIONS(1851),
    [sym_cancel_wait_until_keyword] = ACTIONS(1851),
    [sym_cancel_all_wait_keyword] = ACTIONS(1851),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1851),
    [sym_not] = ACTIONS(1851),
    [sym_bnot] = ACTIONS(1851),
    [sym___file__] = ACTIONS(1851),
    [sym___date__] = ACTIONS(1851),
    [sym___ldate__] = ACTIONS(1851),
    [sym___time__] = ACTIONS(1851),
    [sym___line__] = ACTIONS(1851),
    [sym___name__] = ACTIONS(1851),
    [sym___netlinx__] = ACTIONS(1851),
    [sym_netlinx_axi_version] = ACTIONS(1851),
    [sym_url_flg_tcp] = ACTIONS(1851),
    [sym_url_flg_accinfopresent] = ACTIONS(1851),
    [sym_url_flg_temp] = ACTIONS(1851),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1851),
    [sym_url_flg_stat_mask] = ACTIONS(1851),
    [sym_url_flg_stat_lookup] = ACTIONS(1851),
    [sym_url_flg_stat_connecting] = ACTIONS(1851),
    [sym_url_flg_stat_waiting] = ACTIONS(1851),
    [sym_url_flg_stat_connected] = ACTIONS(1851),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1851),
    [sym_first_virtual_device] = ACTIONS(1851),
    [sym_first_local_port] = ACTIONS(1851),
    [sym_dynamic_virtual_device] = ACTIONS(1851),
    [sym_timeline_once] = ACTIONS(1851),
    [sym_timeline_repeat] = ACTIONS(1851),
    [sym_timeline_absolute] = ACTIONS(1851),
    [sym_timeline_relative] = ACTIONS(1851),
    [sym_do_push_timed_infinite] = ACTIONS(1851),
    [sym_source_type_no_address] = ACTIONS(1851),
    [sym_source_type_neuron_id] = ACTIONS(1851),
    [sym_source_type_ip_address] = ACTIONS(1851),
    [sym_source_type_axlink] = ACTIONS(1851),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1851),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1851),
    [sym_source_type_ip_socket_address] = ACTIONS(1851),
    [sym_source_type_rs232] = ACTIONS(1851),
    [sym_source_type_internal] = ACTIONS(1851),
    [sym_source_type_mac_address] = ACTIONS(1851),
    [sym_source_type_ipv4_port] = ACTIONS(1851),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1851),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1851),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1851),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1851),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1851),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1851),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1851),
    [sym_true] = ACTIONS(1851),
    [sym_false] = ACTIONS(1851),
    [sym_file_read_only] = ACTIONS(1851),
    [sym_file_rw_new] = ACTIONS(1851),
    [sym_file_rw_append] = ACTIONS(1851),
    [sym_ip_tcp] = ACTIONS(1851),
    [sym_ip_udp] = ACTIONS(1851),
    [sym_ip_udp_2way] = ACTIONS(1851),
    [sym_xml_encode_types] = ACTIONS(1851),
    [sym_xml_encode_char_as_list] = ACTIONS(1851),
    [sym_xml_encode_le] = ACTIONS(1851),
    [sym_xml_decode_types] = ACTIONS(1851),
    [sym_xml_decode_no_preserve] = ACTIONS(1851),
    [sym_duet_dev_type_display_device] = ACTIONS(1851),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1851),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1851),
    [sym_duet_dev_type_io_device] = ACTIONS(1851),
    [sym_duet_dev_type_relay_device] = ACTIONS(1851),
    [sym_duet_dev_type_ups] = ACTIONS(1851),
    [sym_duet_dev_type_amplifier] = ACTIONS(1851),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1851),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1851),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1851),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1851),
    [sym_duet_dev_type_camera] = ACTIONS(1851),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1851),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1851),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1851),
    [sym_duet_dev_type_dss] = ACTIONS(1851),
    [sym_duet_dev_type_dvr] = ACTIONS(1851),
    [sym_duet_dev_type_disc_device] = ACTIONS(1851),
    [sym_duet_dev_type_document_camera] = ACTIONS(1851),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1851),
    [sym_duet_dev_type_hvac] = ACTIONS(1851),
    [sym_duet_dev_type_keypad] = ACTIONS(1851),
    [sym_duet_dev_type_light] = ACTIONS(1851),
    [sym_duet_dev_type_monitor] = ACTIONS(1851),
    [sym_duet_dev_type_motor] = ACTIONS(1851),
    [sym_duet_dev_type_multi_window] = ACTIONS(1851),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1851),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1851),
    [sym_duet_dev_type_receiver] = ACTIONS(1851),
    [sym_duet_dev_type_security_system] = ACTIONS(1851),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1851),
    [sym_duet_dev_type_settop_box] = ACTIONS(1851),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1851),
    [sym_duet_dev_type_switcher] = ACTIONS(1851),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1851),
    [sym_duet_dev_type_tv] = ACTIONS(1851),
    [sym_duet_dev_type_utility] = ACTIONS(1851),
    [sym_duet_dev_type_vcr] = ACTIONS(1851),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1851),
    [sym_duet_dev_type_video_processor] = ACTIONS(1851),
    [sym_duet_dev_type_video_projector] = ACTIONS(1851),
    [sym_duet_dev_type_video_wall] = ACTIONS(1851),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1851),
    [sym_duet_dev_type_weather] = ACTIONS(1851),
    [sym_duet_dev_not_polled] = ACTIONS(1851),
    [sym_duet_dev_polled] = ACTIONS(1851),
    [sym_clkmgr_mode_network] = ACTIONS(1851),
    [sym_clkmgr_mode_standalone] = ACTIONS(1851),
    [sym_normal_standby] = ACTIONS(1851),
    [sym_normal_wake] = ACTIONS(1851),
    [sym_ip_multicast_ttl_option] = ACTIONS(1851),
    [sym_ip_ttl_subnet] = ACTIONS(1851),
    [sym_ip_ttl_site] = ACTIONS(1851),
    [sym_ip_ttl_region] = ACTIONS(1851),
    [sym_ip_ttl_continent] = ACTIONS(1851),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1851),
    [sym_ip_nodelay_on] = ACTIONS(1851),
    [sym_ip_nodelay_off] = ACTIONS(1851),
    [sym_amx_error] = ACTIONS(1851),
    [sym_amx_warning] = ACTIONS(1851),
    [sym_amx_info] = ACTIONS(1851),
    [sym_amx_debug] = ACTIONS(1851),
    [sym_netlinx_login_success] = ACTIONS(1851),
    [sym_netlinx_login_fail] = ACTIONS(1851),
    [sym_netlinx_logout] = ACTIONS(1851),
    [sym_null_str] = ACTIONS(1851),
    [sym_smtp_address] = ACTIONS(1851),
    [sym_smtp_port_number] = ACTIONS(1851),
    [sym_smtp_username] = ACTIONS(1851),
    [sym_smtp_password] = ACTIONS(1851),
    [sym_smtp_require_tls] = ACTIONS(1851),
    [sym_smtp_from] = ACTIONS(1851),
    [sym_smtp_tls_true] = ACTIONS(1851),
    [sym_smtp_tls_false] = ACTIONS(1851),
    [sym_listview_on_row_select_event] = ACTIONS(1851),
    [sym_data_structure_unknown] = ACTIONS(1851),
    [sym_data_structure_datafeed] = ACTIONS(1851),
    [sym_data_structure_datarecordset] = ACTIONS(1851),
    [sym_data_structure_datarecord] = ACTIONS(1851),
    [sym_data_structure_datafield] = ACTIONS(1851),
    [sym_data_max_content_field_count] = ACTIONS(1851),
    [sym_data_max_metadata_field_count] = ACTIONS(1851),
    [sym_data_max_id_length] = ACTIONS(1851),
    [sym_data_max_name_length] = ACTIONS(1851),
    [sym_data_max_description_length] = ACTIONS(1851),
    [sym_data_max_type_length] = ACTIONS(1851),
    [sym_data_max_format_length] = ACTIONS(1851),
    [sym_data_max_source_length] = ACTIONS(1851),
    [sym_data_max_value_length] = ACTIONS(1851),
    [sym_data_max_label_length] = ACTIONS(1851),
    [sym_data_max_url_length] = ACTIONS(1851),
    [sym_max_last_login_info_length] = ACTIONS(1851),
    [sym_data_type_unknown] = ACTIONS(1851),
    [sym_data_type_string] = ACTIONS(1851),
    [sym_data_type_datetime] = ACTIONS(1851),
    [sym_data_type_date] = ACTIONS(1851),
    [sym_data_type_time] = ACTIONS(1851),
    [sym_data_type_image] = ACTIONS(1851),
    [sym_data_format_url] = ACTIONS(1851),
    [sym_data_format_phone] = ACTIONS(1851),
    [sym_data_format_email] = ACTIONS(1851),
    [sym_data_format_iso8601] = ACTIONS(1851),
    [sym_auditlog_priv] = ACTIONS(1851),
    [sym_configuration_priv] = ACTIONS(1851),
    [sym_deviceconfig_priv] = ACTIONS(1851),
    [sym_ftp_priv] = ACTIONS(1851),
    [sym_http_priv] = ACTIONS(1851),
    [sym_networkconfig_priv] = ACTIONS(1851),
    [sym_programport_priv] = ACTIONS(1851),
    [sym_remoteui_priv] = ACTIONS(1851),
    [sym_securitycontrol_priv] = ACTIONS(1851),
    [sym_softwaremanagement_priv] = ACTIONS(1851),
    [sym_terminal_priv] = ACTIONS(1851),
    [sym_tpadmin_priv] = ACTIONS(1851),
    [sym_user1_priv] = ACTIONS(1851),
    [sym_user2_priv] = ACTIONS(1851),
    [sym_user3_priv] = ACTIONS(1851),
    [sym_user4_priv] = ACTIONS(1851),
    [sym_usermanagement_priv] = ACTIONS(1851),
    [sym_valid_account] = ACTIONS(1851),
    [sym_err_name_invalid] = ACTIONS(1851),
    [sym_err_password_invalid] = ACTIONS(1851),
    [sym_err_invalid_account] = ACTIONS(1851),
    [sym_err_no_authorization] = ACTIONS(1851),
    [sym_err_invalid_parameter] = ACTIONS(1851),
    [sym_err_user_locked_out] = ACTIONS(1851),
    [sym_err_user_account_expired] = ACTIONS(1851),
    [sym_err_general_error] = ACTIONS(1851),
    [sym_err_authentication_error] = ACTIONS(1851),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1851),
    [sym_tls_validate_certificate] = ACTIONS(1851),
    [sym_tdata] = ACTIONS(1851),
    [sym_tchannel] = ACTIONS(1851),
    [sym_tlevel] = ACTIONS(1851),
    [sym_tbutton] = ACTIONS(1851),
    [sym_ttimeline] = ACTIONS(1851),
    [sym_tcustom] = ACTIONS(1851),
    [sym_url_struct] = ACTIONS(1851),
    [sym_dns_struct] = ACTIONS(1851),
    [sym_ip_address_struct] = ACTIONS(1851),
    [sym_dev_info_struct] = ACTIONS(1851),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1851),
    [sym_data_feed] = ACTIONS(1851),
    [sym_data_field] = ACTIONS(1851),
    [sym_data_record] = ACTIONS(1851),
    [sym_wc_data_feed] = ACTIONS(1851),
    [sym_wc_data_field] = ACTIONS(1851),
    [sym_wc_data_record] = ACTIONS(1851),
    [sym_last_login_info] = ACTIONS(1851),
    [sym____reserved___] = ACTIONS(1851),
    [sym_date] = ACTIONS(1851),
    [sym_day] = ACTIONS(1851),
    [sym_time] = ACTIONS(1851),
    [sym_ldate] = ACTIONS(1851),
    [sym_push_device] = ACTIONS(1851),
    [sym_push_channel] = ACTIONS(1851),
    [sym_push_devchan] = ACTIONS(1851),
    [sym_release_device] = ACTIONS(1851),
    [sym_release_channel] = ACTIONS(1851),
    [sym_release_devchan] = ACTIONS(1851),
    [sym_master_slot] = ACTIONS(1851),
    [sym_get_pulse_time] = ACTIONS(1851),
    [sym_get_timer] = ACTIONS(1851),
    [sym_master_sn] = ACTIONS(1851),
    [sym_system_number] = ACTIONS(1851),
    [sym_button] = ACTIONS(1851),
    [sym_dv_channel] = ACTIONS(1851),
    [sym_data] = ACTIONS(1851),
    [sym_level] = ACTIONS(1851),
    [sym_timeline] = ACTIONS(1851),
    [sym_channel] = ACTIONS(1851),
    [sym_custom] = ACTIONS(1851),
    [sym_atoi] = ACTIONS(1851),
    [sym_atol] = ACTIONS(1851),
    [sym_atof] = ACTIONS(1851),
    [sym_itoa] = ACTIONS(1851),
    [sym_itohex] = ACTIONS(1851),
    [sym_hextoi] = ACTIONS(1851),
    [sym_ftoa] = ACTIONS(1851),
    [sym_find_string] = ACTIONS(1851),
    [sym_left_string] = ACTIONS(1851),
    [sym_right_string] = ACTIONS(1851),
    [sym_mid_string] = ACTIONS(1851),
    [sym_remove_string] = ACTIONS(1851),
    [sym_get_buffer_string] = ACTIONS(1851),
    [sym_get_buffer_char] = ACTIONS(1851),
    [sym_get_multi_buffer_string] = ACTIONS(1851),
    [sym_length_string] = ACTIONS(1851),
    [sym_max_length_string] = ACTIONS(1851),
    [sym_set_length_string] = ACTIONS(1851),
    [sym_compare_string] = ACTIONS(1851),
    [sym_upper_string] = ACTIONS(1851),
    [sym_lower_string] = ACTIONS(1851),
    [sym_get_last] = ACTIONS(1851),
    [sym_date_to_day] = ACTIONS(1851),
    [sym_date_to_month] = ACTIONS(1851),
    [sym_date_to_year] = ACTIONS(1851),
    [sym_time_to_hour] = ACTIONS(1851),
    [sym_time_to_minute] = ACTIONS(1851),
    [sym_time_to_second] = ACTIONS(1851),
    [sym_day_of_week] = ACTIONS(1851),
    [sym_set_pulse_time] = ACTIONS(1851),
    [sym_set_timer] = ACTIONS(1851),
    [sym_file_open] = ACTIONS(1851),
    [sym_file_close] = ACTIONS(1851),
    [sym_file_read] = ACTIONS(1851),
    [sym_file_read_line] = ACTIONS(1851),
    [sym_file_write] = ACTIONS(1851),
    [sym_file_write_line] = ACTIONS(1851),
    [sym_file_copy] = ACTIONS(1851),
    [sym_file_delete] = ACTIONS(1851),
    [sym_file_rename] = ACTIONS(1851),
    [sym_file_dir] = ACTIONS(1851),
    [sym_file_getdir] = ACTIONS(1851),
    [sym_file_setdir] = ACTIONS(1851),
    [sym_file_create_dir] = ACTIONS(1851),
    [sym_file_remove_dir] = ACTIONS(1851),
    [sym_file_seek] = ACTIONS(1851),
    [sym_length_array] = ACTIONS(1851),
    [sym_max_length_array] = ACTIONS(1851),
    [sym_set_length_array] = ACTIONS(1851),
    [sym_ip_client_open] = ACTIONS(1851),
    [sym_ip_client_close] = ACTIONS(1851),
    [sym_ip_server_open] = ACTIONS(1851),
    [sym_ip_server_close] = ACTIONS(1851),
    [sym_ip_bound_client_open] = ACTIONS(1851),
    [sym_ip_mc_server_open] = ACTIONS(1851),
    [sym_ip_set_option] = ACTIONS(1851),
    [sym_ssh_client_open] = ACTIONS(1851),
    [sym_ssh_client_close] = ACTIONS(1851),
    [sym_tls_client_open] = ACTIONS(1851),
    [sym_tls_client_close] = ACTIONS(1851),
    [sym_do_push] = ACTIONS(1851),
    [sym_do_push_timed] = ACTIONS(1851),
    [sym_do_release] = ACTIONS(1851),
    [sym_device_id] = ACTIONS(1851),
    [sym_device_info] = ACTIONS(1851),
    [sym_device_id_string] = ACTIONS(1851),
    [sym_device_standby] = ACTIONS(1851),
    [sym_device_wake] = ACTIONS(1851),
    [sym_timeline_create] = ACTIONS(1851),
    [sym_timeline_kill] = ACTIONS(1851),
    [sym_timeline_reload] = ACTIONS(1851),
    [sym_timeline_pause] = ACTIONS(1851),
    [sym_timeline_restart] = ACTIONS(1851),
    [sym_timeline_set] = ACTIONS(1851),
    [sym_timeline_get] = ACTIONS(1851),
    [sym_timeline_active] = ACTIONS(1851),
    [sym_timeline_dynamic_id] = ACTIONS(1851),
    [sym_variable_to_string] = ACTIONS(1851),
    [sym_string_to_variable] = ACTIONS(1851),
    [sym_length_variable_to_string] = ACTIONS(1851),
    [sym_type_cast] = ACTIONS(1851),
    [sym_variable_to_xml] = ACTIONS(1851),
    [sym_xml_to_variable] = ACTIONS(1851),
    [sym_length_variable_to_xml] = ACTIONS(1851),
    [sym_redirect_string] = ACTIONS(1851),
    [sym_random_number] = ACTIONS(1851),
    [sym_get_unique_id] = ACTIONS(1851),
    [sym_get_system_number] = ACTIONS(1851),
    [sym_set_system_number] = ACTIONS(1851),
    [sym_get_serial_number] = ACTIONS(1851),
    [sym_get_master_build] = ACTIONS(1851),
    [sym_set_validation_code] = ACTIONS(1851),
    [sym_reboot] = ACTIONS(1851),
    [sym_load_duet_module] = ACTIONS(1851),
    [sym_unload_duet_module] = ACTIONS(1851),
    [sym_duet_mem_size_set] = ACTIONS(1851),
    [sym_duet_mem_size_get] = ACTIONS(1851),
    [sym_dynamic_application_device] = ACTIONS(1851),
    [sym_dynamic_polled_port] = ACTIONS(1851),
    [sym_static_port_binding] = ACTIONS(1851),
    [sym_static_ip_binding] = ACTIONS(1851),
    [sym_get_dns_list] = ACTIONS(1851),
    [sym_set_dns_list] = ACTIONS(1851),
    [sym_get_ip_address] = ACTIONS(1851),
    [sym_set_ip_address] = ACTIONS(1851),
    [sym_get_url_list] = ACTIONS(1851),
    [sym_add_url_entry] = ACTIONS(1851),
    [sym_delete_url_entry] = ACTIONS(1851),
    [sym_abs_value] = ACTIONS(1851),
    [sym_max_value] = ACTIONS(1851),
    [sym_min_value] = ACTIONS(1851),
    [sym_exp_value] = ACTIONS(1851),
    [sym_log_value] = ACTIONS(1851),
    [sym_log10_value] = ACTIONS(1851),
    [sym_power_value] = ACTIONS(1851),
    [sym_sqrt_value] = ACTIONS(1851),
    [sym_raw_be] = ACTIONS(1851),
    [sym_raw_le] = ACTIONS(1851),
    [sym_internal_queue_size_set] = ACTIONS(1851),
    [sym_internal_queue_size_get] = ACTIONS(1851),
    [sym_internal_threshold_set] = ACTIONS(1851),
    [sym_internal_threshold_get] = ACTIONS(1851),
    [sym_rebuilt_event] = ACTIONS(1851),
    [sym_do_custom_event] = ACTIONS(1851),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1851),
    [sym_clkmgr_set_clk_source] = ACTIONS(1851),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1851),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1851),
    [sym_clkmgr_get_timezone] = ACTIONS(1851),
    [sym_clkmgr_set_timezone] = ACTIONS(1851),
    [sym_clkmgr_get_resync_period] = ACTIONS(1851),
    [sym_clkmgr_set_resync_period] = ACTIONS(1851),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1851),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1851),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1851),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1851),
    [sym_clkmgr_get_timeservers] = ACTIONS(1851),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1851),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1851),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1851),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1851),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1851),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1851),
    [sym_set_log_level] = ACTIONS(1851),
    [sym_get_log_level] = ACTIONS(1851),
    [sym_amx_log] = ACTIONS(1851),
    [sym_smtp_server_config_set] = ACTIONS(1851),
    [sym_smtp_server_config_get] = ACTIONS(1851),
    [sym_smtp_send] = ACTIONS(1851),
    [sym_validate_netlinx_account] = ACTIONS(1851),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1851),
    [sym_authenticate_certificate] = ACTIONS(1851),
    [sym_audit_netlinx_session_event] = ACTIONS(1851),
    [sym_audit_netlinx_generic_event] = ACTIONS(1851),
    [sym_get_available_flash_disk_space] = ACTIONS(1851),
    [sym_get_max_flash_disk_space] = ACTIONS(1851),
    [sym_data_create_feed] = ACTIONS(1851),
    [sym_data_delete_feed] = ACTIONS(1851),
    [sym_data_publish_feed] = ACTIONS(1851),
    [sym_data_get_published_feed] = ACTIONS(1851),
    [sym_data_add_record] = ACTIONS(1851),
    [sym_data_get_event_record] = ACTIONS(1851),
    [sym__wc_data_create_feed] = ACTIONS(1851),
    [sym__wc_data_add_record] = ACTIONS(1851),
    [sym__wc_data_get_event_record] = ACTIONS(1851),
    [sym_format] = ACTIONS(1851),
    [sym_astro_clock] = ACTIONS(1851),
    [sym_set_outdoor_temperature] = ACTIONS(1851),
    [sym_set_virtual_level_count] = ACTIONS(1851),
    [sym_set_virtual_channel_count] = ACTIONS(1851),
    [sym_set_virtual_port_count] = ACTIONS(1851),
    [sym__wc_encode_int] = ACTIONS(1851),
    [sym__wc_decode_int] = ACTIONS(1851),
    [sym__wc_int] = ACTIONS(1851),
    [sym__wc_to_ch_int] = ACTIONS(1851),
    [sym__ch_to_wc_int] = ACTIONS(1851),
    [sym__wc_find_string_int] = ACTIONS(1851),
    [sym__wc_left_string_int] = ACTIONS(1851),
    [sym__wc_lower_string_int] = ACTIONS(1851),
    [sym__wc_mid_string_int] = ACTIONS(1851),
    [sym__wc_remove_string_int] = ACTIONS(1851),
    [sym__wc_right_string_int] = ACTIONS(1851),
    [sym__wc_upper_string_int] = ACTIONS(1851),
    [sym__wc_compare_string_int] = ACTIONS(1851),
    [sym__wc_get_buffer_char_int] = ACTIONS(1851),
    [sym__wc_get_buffer_string_int] = ACTIONS(1851),
    [sym__wc_concat_string_int] = ACTIONS(1851),
    [sym__wc_file_open_int] = ACTIONS(1851),
    [sym__wc_file_close_int] = ACTIONS(1851),
    [sym__wc_file_read_int] = ACTIONS(1851),
    [sym__wc_file_read_line_int] = ACTIONS(1851),
    [sym__wc_file_write_int] = ACTIONS(1851),
    [sym__wc_file_write_line_int] = ACTIONS(1851),
    [aux_sym_preproc_include_token1] = ACTIONS(1851),
    [aux_sym_preproc_define_token1] = ACTIONS(1853),
    [aux_sym_preproc_warn_token1] = ACTIONS(1853),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1853),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1853),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1853),
    [anon_sym_LBRACK] = ACTIONS(1853),
    [anon_sym_LBRACE] = ACTIONS(1853),
    [anon_sym_RBRACE] = ACTIONS(1853),
    [anon_sym_LPAREN] = ACTIONS(1851),
    [anon_sym_BANG] = ACTIONS(1853),
    [anon_sym_TILDE] = ACTIONS(1853),
    [anon_sym_DASH] = ACTIONS(1851),
    [anon_sym_PLUS] = ACTIONS(1851),
    [anon_sym_DASH_DASH] = ACTIONS(1853),
    [anon_sym_PLUS_PLUS] = ACTIONS(1853),
    [anon_sym_DQUOTE] = ACTIONS(1853),
    [anon_sym_SQUOTE] = ACTIONS(1853),
    [sym_number_literal] = ACTIONS(1853),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(472)] = {
    [sym_identifier] = ACTIONS(1855),
    [sym_define_call_keyword] = ACTIONS(1855),
    [sym_define_function_keyword] = ACTIONS(1855),
    [sym_define_library_function_keyword] = ACTIONS(1855),
    [sym_define_module_keyword] = ACTIONS(1855),
    [sym_char_keyword] = ACTIONS(1855),
    [sym_widechar_keyword] = ACTIONS(1855),
    [sym_integer_keyword] = ACTIONS(1855),
    [sym_sinteger_keyword] = ACTIONS(1855),
    [sym_long_keyword] = ACTIONS(1855),
    [sym_slong_keyword] = ACTIONS(1855),
    [sym_float_keyword] = ACTIONS(1855),
    [sym_double_keyword] = ACTIONS(1855),
    [sym_variant_keyword] = ACTIONS(1855),
    [sym_variantarray_keyword] = ACTIONS(1855),
    [sym_local_var_keyword] = ACTIONS(1855),
    [sym_stack_var_keyword] = ACTIONS(1855),
    [sym_constant_keyword] = ACTIONS(1855),
    [sym_volatile_keyword] = ACTIONS(1855),
    [sym_non_volatile_keyword] = ACTIONS(1855),
    [sym_persistent_keyword] = ACTIONS(1855),
    [sym_dev_keyword] = ACTIONS(1855),
    [sym_devlev_keyword] = ACTIONS(1855),
    [sym_devchan_keyword] = ACTIONS(1855),
    [sym_if_keyword] = ACTIONS(1855),
    [sym_else_keyword] = ACTIONS(1855),
    [sym_switch_keyword] = ACTIONS(1855),
    [sym_case_keyword] = ACTIONS(1855),
    [sym_default_keyword] = ACTIONS(1855),
    [sym_while_keyword] = ACTIONS(1855),
    [sym_for_keyword] = ACTIONS(1855),
    [sym_break_keyword] = ACTIONS(1855),
    [sym_continue_keyword] = ACTIONS(1855),
    [sym_return_keyword] = ACTIONS(1855),
    [sym_select_keyword] = ACTIONS(1855),
    [sym_active_keyword] = ACTIONS(1855),
    [sym_struct_keyword] = ACTIONS(1855),
    [sym_structure_keyword] = ACTIONS(1855),
    [aux_sym_on_keyword_token1] = ACTIONS(1855),
    [aux_sym_off_keyword_token1] = ACTIONS(1855),
    [sym_send_level_keyword] = ACTIONS(1855),
    [sym_send_string_keyword] = ACTIONS(1855),
    [sym_send_command_keyword] = ACTIONS(1855),
    [sym_clear_buffer_keyword] = ACTIONS(1855),
    [sym_create_buffer_keyword] = ACTIONS(1855),
    [sym_create_multi_buffer_keyword] = ACTIONS(1855),
    [sym_call_keyword] = ACTIONS(1855),
    [sym_system_call_keyword] = ACTIONS(1855),
    [sym_devchan_to_keyword] = ACTIONS(1855),
    [sym_devchan_min_to_keyword] = ACTIONS(1855),
    [sym_devchan_total_off_keyword] = ACTIONS(1855),
    [sym_devchan_pulse_keyword] = ACTIONS(1855),
    [sym_wait_keyword] = ACTIONS(1855),
    [sym_cancel_wait_keyword] = ACTIONS(1855),
    [sym_wait_until_keyword] = ACTIONS(1855),
    [sym_cancel_wait_until_keyword] = ACTIONS(1855),
    [sym_cancel_all_wait_keyword] = ACTIONS(1855),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1855),
    [sym_not] = ACTIONS(1855),
    [sym_bnot] = ACTIONS(1855),
    [sym___file__] = ACTIONS(1855),
    [sym___date__] = ACTIONS(1855),
    [sym___ldate__] = ACTIONS(1855),
    [sym___time__] = ACTIONS(1855),
    [sym___line__] = ACTIONS(1855),
    [sym___name__] = ACTIONS(1855),
    [sym___netlinx__] = ACTIONS(1855),
    [sym_netlinx_axi_version] = ACTIONS(1855),
    [sym_url_flg_tcp] = ACTIONS(1855),
    [sym_url_flg_accinfopresent] = ACTIONS(1855),
    [sym_url_flg_temp] = ACTIONS(1855),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1855),
    [sym_url_flg_stat_mask] = ACTIONS(1855),
    [sym_url_flg_stat_lookup] = ACTIONS(1855),
    [sym_url_flg_stat_connecting] = ACTIONS(1855),
    [sym_url_flg_stat_waiting] = ACTIONS(1855),
    [sym_url_flg_stat_connected] = ACTIONS(1855),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1855),
    [sym_first_virtual_device] = ACTIONS(1855),
    [sym_first_local_port] = ACTIONS(1855),
    [sym_dynamic_virtual_device] = ACTIONS(1855),
    [sym_timeline_once] = ACTIONS(1855),
    [sym_timeline_repeat] = ACTIONS(1855),
    [sym_timeline_absolute] = ACTIONS(1855),
    [sym_timeline_relative] = ACTIONS(1855),
    [sym_do_push_timed_infinite] = ACTIONS(1855),
    [sym_source_type_no_address] = ACTIONS(1855),
    [sym_source_type_neuron_id] = ACTIONS(1855),
    [sym_source_type_ip_address] = ACTIONS(1855),
    [sym_source_type_axlink] = ACTIONS(1855),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1855),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1855),
    [sym_source_type_ip_socket_address] = ACTIONS(1855),
    [sym_source_type_rs232] = ACTIONS(1855),
    [sym_source_type_internal] = ACTIONS(1855),
    [sym_source_type_mac_address] = ACTIONS(1855),
    [sym_source_type_ipv4_port] = ACTIONS(1855),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1855),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1855),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1855),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1855),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1855),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1855),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1855),
    [sym_true] = ACTIONS(1855),
    [sym_false] = ACTIONS(1855),
    [sym_file_read_only] = ACTIONS(1855),
    [sym_file_rw_new] = ACTIONS(1855),
    [sym_file_rw_append] = ACTIONS(1855),
    [sym_ip_tcp] = ACTIONS(1855),
    [sym_ip_udp] = ACTIONS(1855),
    [sym_ip_udp_2way] = ACTIONS(1855),
    [sym_xml_encode_types] = ACTIONS(1855),
    [sym_xml_encode_char_as_list] = ACTIONS(1855),
    [sym_xml_encode_le] = ACTIONS(1855),
    [sym_xml_decode_types] = ACTIONS(1855),
    [sym_xml_decode_no_preserve] = ACTIONS(1855),
    [sym_duet_dev_type_display_device] = ACTIONS(1855),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1855),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1855),
    [sym_duet_dev_type_io_device] = ACTIONS(1855),
    [sym_duet_dev_type_relay_device] = ACTIONS(1855),
    [sym_duet_dev_type_ups] = ACTIONS(1855),
    [sym_duet_dev_type_amplifier] = ACTIONS(1855),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1855),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1855),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1855),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1855),
    [sym_duet_dev_type_camera] = ACTIONS(1855),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1855),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1855),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1855),
    [sym_duet_dev_type_dss] = ACTIONS(1855),
    [sym_duet_dev_type_dvr] = ACTIONS(1855),
    [sym_duet_dev_type_disc_device] = ACTIONS(1855),
    [sym_duet_dev_type_document_camera] = ACTIONS(1855),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1855),
    [sym_duet_dev_type_hvac] = ACTIONS(1855),
    [sym_duet_dev_type_keypad] = ACTIONS(1855),
    [sym_duet_dev_type_light] = ACTIONS(1855),
    [sym_duet_dev_type_monitor] = ACTIONS(1855),
    [sym_duet_dev_type_motor] = ACTIONS(1855),
    [sym_duet_dev_type_multi_window] = ACTIONS(1855),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1855),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1855),
    [sym_duet_dev_type_receiver] = ACTIONS(1855),
    [sym_duet_dev_type_security_system] = ACTIONS(1855),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1855),
    [sym_duet_dev_type_settop_box] = ACTIONS(1855),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1855),
    [sym_duet_dev_type_switcher] = ACTIONS(1855),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1855),
    [sym_duet_dev_type_tv] = ACTIONS(1855),
    [sym_duet_dev_type_utility] = ACTIONS(1855),
    [sym_duet_dev_type_vcr] = ACTIONS(1855),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1855),
    [sym_duet_dev_type_video_processor] = ACTIONS(1855),
    [sym_duet_dev_type_video_projector] = ACTIONS(1855),
    [sym_duet_dev_type_video_wall] = ACTIONS(1855),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1855),
    [sym_duet_dev_type_weather] = ACTIONS(1855),
    [sym_duet_dev_not_polled] = ACTIONS(1855),
    [sym_duet_dev_polled] = ACTIONS(1855),
    [sym_clkmgr_mode_network] = ACTIONS(1855),
    [sym_clkmgr_mode_standalone] = ACTIONS(1855),
    [sym_normal_standby] = ACTIONS(1855),
    [sym_normal_wake] = ACTIONS(1855),
    [sym_ip_multicast_ttl_option] = ACTIONS(1855),
    [sym_ip_ttl_subnet] = ACTIONS(1855),
    [sym_ip_ttl_site] = ACTIONS(1855),
    [sym_ip_ttl_region] = ACTIONS(1855),
    [sym_ip_ttl_continent] = ACTIONS(1855),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1855),
    [sym_ip_nodelay_on] = ACTIONS(1855),
    [sym_ip_nodelay_off] = ACTIONS(1855),
    [sym_amx_error] = ACTIONS(1855),
    [sym_amx_warning] = ACTIONS(1855),
    [sym_amx_info] = ACTIONS(1855),
    [sym_amx_debug] = ACTIONS(1855),
    [sym_netlinx_login_success] = ACTIONS(1855),
    [sym_netlinx_login_fail] = ACTIONS(1855),
    [sym_netlinx_logout] = ACTIONS(1855),
    [sym_null_str] = ACTIONS(1855),
    [sym_smtp_address] = ACTIONS(1855),
    [sym_smtp_port_number] = ACTIONS(1855),
    [sym_smtp_username] = ACTIONS(1855),
    [sym_smtp_password] = ACTIONS(1855),
    [sym_smtp_require_tls] = ACTIONS(1855),
    [sym_smtp_from] = ACTIONS(1855),
    [sym_smtp_tls_true] = ACTIONS(1855),
    [sym_smtp_tls_false] = ACTIONS(1855),
    [sym_listview_on_row_select_event] = ACTIONS(1855),
    [sym_data_structure_unknown] = ACTIONS(1855),
    [sym_data_structure_datafeed] = ACTIONS(1855),
    [sym_data_structure_datarecordset] = ACTIONS(1855),
    [sym_data_structure_datarecord] = ACTIONS(1855),
    [sym_data_structure_datafield] = ACTIONS(1855),
    [sym_data_max_content_field_count] = ACTIONS(1855),
    [sym_data_max_metadata_field_count] = ACTIONS(1855),
    [sym_data_max_id_length] = ACTIONS(1855),
    [sym_data_max_name_length] = ACTIONS(1855),
    [sym_data_max_description_length] = ACTIONS(1855),
    [sym_data_max_type_length] = ACTIONS(1855),
    [sym_data_max_format_length] = ACTIONS(1855),
    [sym_data_max_source_length] = ACTIONS(1855),
    [sym_data_max_value_length] = ACTIONS(1855),
    [sym_data_max_label_length] = ACTIONS(1855),
    [sym_data_max_url_length] = ACTIONS(1855),
    [sym_max_last_login_info_length] = ACTIONS(1855),
    [sym_data_type_unknown] = ACTIONS(1855),
    [sym_data_type_string] = ACTIONS(1855),
    [sym_data_type_datetime] = ACTIONS(1855),
    [sym_data_type_date] = ACTIONS(1855),
    [sym_data_type_time] = ACTIONS(1855),
    [sym_data_type_image] = ACTIONS(1855),
    [sym_data_format_url] = ACTIONS(1855),
    [sym_data_format_phone] = ACTIONS(1855),
    [sym_data_format_email] = ACTIONS(1855),
    [sym_data_format_iso8601] = ACTIONS(1855),
    [sym_auditlog_priv] = ACTIONS(1855),
    [sym_configuration_priv] = ACTIONS(1855),
    [sym_deviceconfig_priv] = ACTIONS(1855),
    [sym_ftp_priv] = ACTIONS(1855),
    [sym_http_priv] = ACTIONS(1855),
    [sym_networkconfig_priv] = ACTIONS(1855),
    [sym_programport_priv] = ACTIONS(1855),
    [sym_remoteui_priv] = ACTIONS(1855),
    [sym_securitycontrol_priv] = ACTIONS(1855),
    [sym_softwaremanagement_priv] = ACTIONS(1855),
    [sym_terminal_priv] = ACTIONS(1855),
    [sym_tpadmin_priv] = ACTIONS(1855),
    [sym_user1_priv] = ACTIONS(1855),
    [sym_user2_priv] = ACTIONS(1855),
    [sym_user3_priv] = ACTIONS(1855),
    [sym_user4_priv] = ACTIONS(1855),
    [sym_usermanagement_priv] = ACTIONS(1855),
    [sym_valid_account] = ACTIONS(1855),
    [sym_err_name_invalid] = ACTIONS(1855),
    [sym_err_password_invalid] = ACTIONS(1855),
    [sym_err_invalid_account] = ACTIONS(1855),
    [sym_err_no_authorization] = ACTIONS(1855),
    [sym_err_invalid_parameter] = ACTIONS(1855),
    [sym_err_user_locked_out] = ACTIONS(1855),
    [sym_err_user_account_expired] = ACTIONS(1855),
    [sym_err_general_error] = ACTIONS(1855),
    [sym_err_authentication_error] = ACTIONS(1855),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1855),
    [sym_tls_validate_certificate] = ACTIONS(1855),
    [sym_tdata] = ACTIONS(1855),
    [sym_tchannel] = ACTIONS(1855),
    [sym_tlevel] = ACTIONS(1855),
    [sym_tbutton] = ACTIONS(1855),
    [sym_ttimeline] = ACTIONS(1855),
    [sym_tcustom] = ACTIONS(1855),
    [sym_url_struct] = ACTIONS(1855),
    [sym_dns_struct] = ACTIONS(1855),
    [sym_ip_address_struct] = ACTIONS(1855),
    [sym_dev_info_struct] = ACTIONS(1855),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1855),
    [sym_data_feed] = ACTIONS(1855),
    [sym_data_field] = ACTIONS(1855),
    [sym_data_record] = ACTIONS(1855),
    [sym_wc_data_feed] = ACTIONS(1855),
    [sym_wc_data_field] = ACTIONS(1855),
    [sym_wc_data_record] = ACTIONS(1855),
    [sym_last_login_info] = ACTIONS(1855),
    [sym____reserved___] = ACTIONS(1855),
    [sym_date] = ACTIONS(1855),
    [sym_day] = ACTIONS(1855),
    [sym_time] = ACTIONS(1855),
    [sym_ldate] = ACTIONS(1855),
    [sym_push_device] = ACTIONS(1855),
    [sym_push_channel] = ACTIONS(1855),
    [sym_push_devchan] = ACTIONS(1855),
    [sym_release_device] = ACTIONS(1855),
    [sym_release_channel] = ACTIONS(1855),
    [sym_release_devchan] = ACTIONS(1855),
    [sym_master_slot] = ACTIONS(1855),
    [sym_get_pulse_time] = ACTIONS(1855),
    [sym_get_timer] = ACTIONS(1855),
    [sym_master_sn] = ACTIONS(1855),
    [sym_system_number] = ACTIONS(1855),
    [sym_button] = ACTIONS(1855),
    [sym_dv_channel] = ACTIONS(1855),
    [sym_data] = ACTIONS(1855),
    [sym_level] = ACTIONS(1855),
    [sym_timeline] = ACTIONS(1855),
    [sym_channel] = ACTIONS(1855),
    [sym_custom] = ACTIONS(1855),
    [sym_atoi] = ACTIONS(1855),
    [sym_atol] = ACTIONS(1855),
    [sym_atof] = ACTIONS(1855),
    [sym_itoa] = ACTIONS(1855),
    [sym_itohex] = ACTIONS(1855),
    [sym_hextoi] = ACTIONS(1855),
    [sym_ftoa] = ACTIONS(1855),
    [sym_find_string] = ACTIONS(1855),
    [sym_left_string] = ACTIONS(1855),
    [sym_right_string] = ACTIONS(1855),
    [sym_mid_string] = ACTIONS(1855),
    [sym_remove_string] = ACTIONS(1855),
    [sym_get_buffer_string] = ACTIONS(1855),
    [sym_get_buffer_char] = ACTIONS(1855),
    [sym_get_multi_buffer_string] = ACTIONS(1855),
    [sym_length_string] = ACTIONS(1855),
    [sym_max_length_string] = ACTIONS(1855),
    [sym_set_length_string] = ACTIONS(1855),
    [sym_compare_string] = ACTIONS(1855),
    [sym_upper_string] = ACTIONS(1855),
    [sym_lower_string] = ACTIONS(1855),
    [sym_get_last] = ACTIONS(1855),
    [sym_date_to_day] = ACTIONS(1855),
    [sym_date_to_month] = ACTIONS(1855),
    [sym_date_to_year] = ACTIONS(1855),
    [sym_time_to_hour] = ACTIONS(1855),
    [sym_time_to_minute] = ACTIONS(1855),
    [sym_time_to_second] = ACTIONS(1855),
    [sym_day_of_week] = ACTIONS(1855),
    [sym_set_pulse_time] = ACTIONS(1855),
    [sym_set_timer] = ACTIONS(1855),
    [sym_file_open] = ACTIONS(1855),
    [sym_file_close] = ACTIONS(1855),
    [sym_file_read] = ACTIONS(1855),
    [sym_file_read_line] = ACTIONS(1855),
    [sym_file_write] = ACTIONS(1855),
    [sym_file_write_line] = ACTIONS(1855),
    [sym_file_copy] = ACTIONS(1855),
    [sym_file_delete] = ACTIONS(1855),
    [sym_file_rename] = ACTIONS(1855),
    [sym_file_dir] = ACTIONS(1855),
    [sym_file_getdir] = ACTIONS(1855),
    [sym_file_setdir] = ACTIONS(1855),
    [sym_file_create_dir] = ACTIONS(1855),
    [sym_file_remove_dir] = ACTIONS(1855),
    [sym_file_seek] = ACTIONS(1855),
    [sym_length_array] = ACTIONS(1855),
    [sym_max_length_array] = ACTIONS(1855),
    [sym_set_length_array] = ACTIONS(1855),
    [sym_ip_client_open] = ACTIONS(1855),
    [sym_ip_client_close] = ACTIONS(1855),
    [sym_ip_server_open] = ACTIONS(1855),
    [sym_ip_server_close] = ACTIONS(1855),
    [sym_ip_bound_client_open] = ACTIONS(1855),
    [sym_ip_mc_server_open] = ACTIONS(1855),
    [sym_ip_set_option] = ACTIONS(1855),
    [sym_ssh_client_open] = ACTIONS(1855),
    [sym_ssh_client_close] = ACTIONS(1855),
    [sym_tls_client_open] = ACTIONS(1855),
    [sym_tls_client_close] = ACTIONS(1855),
    [sym_do_push] = ACTIONS(1855),
    [sym_do_push_timed] = ACTIONS(1855),
    [sym_do_release] = ACTIONS(1855),
    [sym_device_id] = ACTIONS(1855),
    [sym_device_info] = ACTIONS(1855),
    [sym_device_id_string] = ACTIONS(1855),
    [sym_device_standby] = ACTIONS(1855),
    [sym_device_wake] = ACTIONS(1855),
    [sym_timeline_create] = ACTIONS(1855),
    [sym_timeline_kill] = ACTIONS(1855),
    [sym_timeline_reload] = ACTIONS(1855),
    [sym_timeline_pause] = ACTIONS(1855),
    [sym_timeline_restart] = ACTIONS(1855),
    [sym_timeline_set] = ACTIONS(1855),
    [sym_timeline_get] = ACTIONS(1855),
    [sym_timeline_active] = ACTIONS(1855),
    [sym_timeline_dynamic_id] = ACTIONS(1855),
    [sym_variable_to_string] = ACTIONS(1855),
    [sym_string_to_variable] = ACTIONS(1855),
    [sym_length_variable_to_string] = ACTIONS(1855),
    [sym_type_cast] = ACTIONS(1855),
    [sym_variable_to_xml] = ACTIONS(1855),
    [sym_xml_to_variable] = ACTIONS(1855),
    [sym_length_variable_to_xml] = ACTIONS(1855),
    [sym_redirect_string] = ACTIONS(1855),
    [sym_random_number] = ACTIONS(1855),
    [sym_get_unique_id] = ACTIONS(1855),
    [sym_get_system_number] = ACTIONS(1855),
    [sym_set_system_number] = ACTIONS(1855),
    [sym_get_serial_number] = ACTIONS(1855),
    [sym_get_master_build] = ACTIONS(1855),
    [sym_set_validation_code] = ACTIONS(1855),
    [sym_reboot] = ACTIONS(1855),
    [sym_load_duet_module] = ACTIONS(1855),
    [sym_unload_duet_module] = ACTIONS(1855),
    [sym_duet_mem_size_set] = ACTIONS(1855),
    [sym_duet_mem_size_get] = ACTIONS(1855),
    [sym_dynamic_application_device] = ACTIONS(1855),
    [sym_dynamic_polled_port] = ACTIONS(1855),
    [sym_static_port_binding] = ACTIONS(1855),
    [sym_static_ip_binding] = ACTIONS(1855),
    [sym_get_dns_list] = ACTIONS(1855),
    [sym_set_dns_list] = ACTIONS(1855),
    [sym_get_ip_address] = ACTIONS(1855),
    [sym_set_ip_address] = ACTIONS(1855),
    [sym_get_url_list] = ACTIONS(1855),
    [sym_add_url_entry] = ACTIONS(1855),
    [sym_delete_url_entry] = ACTIONS(1855),
    [sym_abs_value] = ACTIONS(1855),
    [sym_max_value] = ACTIONS(1855),
    [sym_min_value] = ACTIONS(1855),
    [sym_exp_value] = ACTIONS(1855),
    [sym_log_value] = ACTIONS(1855),
    [sym_log10_value] = ACTIONS(1855),
    [sym_power_value] = ACTIONS(1855),
    [sym_sqrt_value] = ACTIONS(1855),
    [sym_raw_be] = ACTIONS(1855),
    [sym_raw_le] = ACTIONS(1855),
    [sym_internal_queue_size_set] = ACTIONS(1855),
    [sym_internal_queue_size_get] = ACTIONS(1855),
    [sym_internal_threshold_set] = ACTIONS(1855),
    [sym_internal_threshold_get] = ACTIONS(1855),
    [sym_rebuilt_event] = ACTIONS(1855),
    [sym_do_custom_event] = ACTIONS(1855),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1855),
    [sym_clkmgr_set_clk_source] = ACTIONS(1855),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1855),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1855),
    [sym_clkmgr_get_timezone] = ACTIONS(1855),
    [sym_clkmgr_set_timezone] = ACTIONS(1855),
    [sym_clkmgr_get_resync_period] = ACTIONS(1855),
    [sym_clkmgr_set_resync_period] = ACTIONS(1855),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1855),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1855),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1855),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1855),
    [sym_clkmgr_get_timeservers] = ACTIONS(1855),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1855),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1855),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1855),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1855),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1855),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1855),
    [sym_set_log_level] = ACTIONS(1855),
    [sym_get_log_level] = ACTIONS(1855),
    [sym_amx_log] = ACTIONS(1855),
    [sym_smtp_server_config_set] = ACTIONS(1855),
    [sym_smtp_server_config_get] = ACTIONS(1855),
    [sym_smtp_send] = ACTIONS(1855),
    [sym_validate_netlinx_account] = ACTIONS(1855),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1855),
    [sym_authenticate_certificate] = ACTIONS(1855),
    [sym_audit_netlinx_session_event] = ACTIONS(1855),
    [sym_audit_netlinx_generic_event] = ACTIONS(1855),
    [sym_get_available_flash_disk_space] = ACTIONS(1855),
    [sym_get_max_flash_disk_space] = ACTIONS(1855),
    [sym_data_create_feed] = ACTIONS(1855),
    [sym_data_delete_feed] = ACTIONS(1855),
    [sym_data_publish_feed] = ACTIONS(1855),
    [sym_data_get_published_feed] = ACTIONS(1855),
    [sym_data_add_record] = ACTIONS(1855),
    [sym_data_get_event_record] = ACTIONS(1855),
    [sym__wc_data_create_feed] = ACTIONS(1855),
    [sym__wc_data_add_record] = ACTIONS(1855),
    [sym__wc_data_get_event_record] = ACTIONS(1855),
    [sym_format] = ACTIONS(1855),
    [sym_astro_clock] = ACTIONS(1855),
    [sym_set_outdoor_temperature] = ACTIONS(1855),
    [sym_set_virtual_level_count] = ACTIONS(1855),
    [sym_set_virtual_channel_count] = ACTIONS(1855),
    [sym_set_virtual_port_count] = ACTIONS(1855),
    [sym__wc_encode_int] = ACTIONS(1855),
    [sym__wc_decode_int] = ACTIONS(1855),
    [sym__wc_int] = ACTIONS(1855),
    [sym__wc_to_ch_int] = ACTIONS(1855),
    [sym__ch_to_wc_int] = ACTIONS(1855),
    [sym__wc_find_string_int] = ACTIONS(1855),
    [sym__wc_left_string_int] = ACTIONS(1855),
    [sym__wc_lower_string_int] = ACTIONS(1855),
    [sym__wc_mid_string_int] = ACTIONS(1855),
    [sym__wc_remove_string_int] = ACTIONS(1855),
    [sym__wc_right_string_int] = ACTIONS(1855),
    [sym__wc_upper_string_int] = ACTIONS(1855),
    [sym__wc_compare_string_int] = ACTIONS(1855),
    [sym__wc_get_buffer_char_int] = ACTIONS(1855),
    [sym__wc_get_buffer_string_int] = ACTIONS(1855),
    [sym__wc_concat_string_int] = ACTIONS(1855),
    [sym__wc_file_open_int] = ACTIONS(1855),
    [sym__wc_file_close_int] = ACTIONS(1855),
    [sym__wc_file_read_int] = ACTIONS(1855),
    [sym__wc_file_read_line_int] = ACTIONS(1855),
    [sym__wc_file_write_int] = ACTIONS(1855),
    [sym__wc_file_write_line_int] = ACTIONS(1855),
    [aux_sym_preproc_include_token1] = ACTIONS(1855),
    [aux_sym_preproc_define_token1] = ACTIONS(1857),
    [aux_sym_preproc_warn_token1] = ACTIONS(1857),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1857),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1857),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1857),
    [anon_sym_LBRACK] = ACTIONS(1857),
    [anon_sym_LBRACE] = ACTIONS(1857),
    [anon_sym_RBRACE] = ACTIONS(1857),
    [anon_sym_LPAREN] = ACTIONS(1855),
    [anon_sym_BANG] = ACTIONS(1857),
    [anon_sym_TILDE] = ACTIONS(1857),
    [anon_sym_DASH] = ACTIONS(1855),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_DASH_DASH] = ACTIONS(1857),
    [anon_sym_PLUS_PLUS] = ACTIONS(1857),
    [anon_sym_DQUOTE] = ACTIONS(1857),
    [anon_sym_SQUOTE] = ACTIONS(1857),
    [sym_number_literal] = ACTIONS(1857),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(473)] = {
    [sym_identifier] = ACTIONS(1859),
    [sym_define_call_keyword] = ACTIONS(1859),
    [sym_define_function_keyword] = ACTIONS(1859),
    [sym_define_library_function_keyword] = ACTIONS(1859),
    [sym_define_module_keyword] = ACTIONS(1859),
    [sym_char_keyword] = ACTIONS(1859),
    [sym_widechar_keyword] = ACTIONS(1859),
    [sym_integer_keyword] = ACTIONS(1859),
    [sym_sinteger_keyword] = ACTIONS(1859),
    [sym_long_keyword] = ACTIONS(1859),
    [sym_slong_keyword] = ACTIONS(1859),
    [sym_float_keyword] = ACTIONS(1859),
    [sym_double_keyword] = ACTIONS(1859),
    [sym_variant_keyword] = ACTIONS(1859),
    [sym_variantarray_keyword] = ACTIONS(1859),
    [sym_local_var_keyword] = ACTIONS(1859),
    [sym_stack_var_keyword] = ACTIONS(1859),
    [sym_constant_keyword] = ACTIONS(1859),
    [sym_volatile_keyword] = ACTIONS(1859),
    [sym_non_volatile_keyword] = ACTIONS(1859),
    [sym_persistent_keyword] = ACTIONS(1859),
    [sym_dev_keyword] = ACTIONS(1859),
    [sym_devlev_keyword] = ACTIONS(1859),
    [sym_devchan_keyword] = ACTIONS(1859),
    [sym_if_keyword] = ACTIONS(1859),
    [sym_else_keyword] = ACTIONS(1859),
    [sym_switch_keyword] = ACTIONS(1859),
    [sym_case_keyword] = ACTIONS(1859),
    [sym_default_keyword] = ACTIONS(1859),
    [sym_while_keyword] = ACTIONS(1859),
    [sym_for_keyword] = ACTIONS(1859),
    [sym_break_keyword] = ACTIONS(1859),
    [sym_continue_keyword] = ACTIONS(1859),
    [sym_return_keyword] = ACTIONS(1859),
    [sym_select_keyword] = ACTIONS(1859),
    [sym_active_keyword] = ACTIONS(1859),
    [sym_struct_keyword] = ACTIONS(1859),
    [sym_structure_keyword] = ACTIONS(1859),
    [aux_sym_on_keyword_token1] = ACTIONS(1859),
    [aux_sym_off_keyword_token1] = ACTIONS(1859),
    [sym_send_level_keyword] = ACTIONS(1859),
    [sym_send_string_keyword] = ACTIONS(1859),
    [sym_send_command_keyword] = ACTIONS(1859),
    [sym_clear_buffer_keyword] = ACTIONS(1859),
    [sym_create_buffer_keyword] = ACTIONS(1859),
    [sym_create_multi_buffer_keyword] = ACTIONS(1859),
    [sym_call_keyword] = ACTIONS(1859),
    [sym_system_call_keyword] = ACTIONS(1859),
    [sym_devchan_to_keyword] = ACTIONS(1859),
    [sym_devchan_min_to_keyword] = ACTIONS(1859),
    [sym_devchan_total_off_keyword] = ACTIONS(1859),
    [sym_devchan_pulse_keyword] = ACTIONS(1859),
    [sym_wait_keyword] = ACTIONS(1859),
    [sym_cancel_wait_keyword] = ACTIONS(1859),
    [sym_wait_until_keyword] = ACTIONS(1859),
    [sym_cancel_wait_until_keyword] = ACTIONS(1859),
    [sym_cancel_all_wait_keyword] = ACTIONS(1859),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1859),
    [sym_not] = ACTIONS(1859),
    [sym_bnot] = ACTIONS(1859),
    [sym___file__] = ACTIONS(1859),
    [sym___date__] = ACTIONS(1859),
    [sym___ldate__] = ACTIONS(1859),
    [sym___time__] = ACTIONS(1859),
    [sym___line__] = ACTIONS(1859),
    [sym___name__] = ACTIONS(1859),
    [sym___netlinx__] = ACTIONS(1859),
    [sym_netlinx_axi_version] = ACTIONS(1859),
    [sym_url_flg_tcp] = ACTIONS(1859),
    [sym_url_flg_accinfopresent] = ACTIONS(1859),
    [sym_url_flg_temp] = ACTIONS(1859),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1859),
    [sym_url_flg_stat_mask] = ACTIONS(1859),
    [sym_url_flg_stat_lookup] = ACTIONS(1859),
    [sym_url_flg_stat_connecting] = ACTIONS(1859),
    [sym_url_flg_stat_waiting] = ACTIONS(1859),
    [sym_url_flg_stat_connected] = ACTIONS(1859),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1859),
    [sym_first_virtual_device] = ACTIONS(1859),
    [sym_first_local_port] = ACTIONS(1859),
    [sym_dynamic_virtual_device] = ACTIONS(1859),
    [sym_timeline_once] = ACTIONS(1859),
    [sym_timeline_repeat] = ACTIONS(1859),
    [sym_timeline_absolute] = ACTIONS(1859),
    [sym_timeline_relative] = ACTIONS(1859),
    [sym_do_push_timed_infinite] = ACTIONS(1859),
    [sym_source_type_no_address] = ACTIONS(1859),
    [sym_source_type_neuron_id] = ACTIONS(1859),
    [sym_source_type_ip_address] = ACTIONS(1859),
    [sym_source_type_axlink] = ACTIONS(1859),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1859),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1859),
    [sym_source_type_ip_socket_address] = ACTIONS(1859),
    [sym_source_type_rs232] = ACTIONS(1859),
    [sym_source_type_internal] = ACTIONS(1859),
    [sym_source_type_mac_address] = ACTIONS(1859),
    [sym_source_type_ipv4_port] = ACTIONS(1859),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1859),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1859),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1859),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1859),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1859),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1859),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1859),
    [sym_true] = ACTIONS(1859),
    [sym_false] = ACTIONS(1859),
    [sym_file_read_only] = ACTIONS(1859),
    [sym_file_rw_new] = ACTIONS(1859),
    [sym_file_rw_append] = ACTIONS(1859),
    [sym_ip_tcp] = ACTIONS(1859),
    [sym_ip_udp] = ACTIONS(1859),
    [sym_ip_udp_2way] = ACTIONS(1859),
    [sym_xml_encode_types] = ACTIONS(1859),
    [sym_xml_encode_char_as_list] = ACTIONS(1859),
    [sym_xml_encode_le] = ACTIONS(1859),
    [sym_xml_decode_types] = ACTIONS(1859),
    [sym_xml_decode_no_preserve] = ACTIONS(1859),
    [sym_duet_dev_type_display_device] = ACTIONS(1859),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1859),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1859),
    [sym_duet_dev_type_io_device] = ACTIONS(1859),
    [sym_duet_dev_type_relay_device] = ACTIONS(1859),
    [sym_duet_dev_type_ups] = ACTIONS(1859),
    [sym_duet_dev_type_amplifier] = ACTIONS(1859),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1859),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1859),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1859),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1859),
    [sym_duet_dev_type_camera] = ACTIONS(1859),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1859),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1859),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1859),
    [sym_duet_dev_type_dss] = ACTIONS(1859),
    [sym_duet_dev_type_dvr] = ACTIONS(1859),
    [sym_duet_dev_type_disc_device] = ACTIONS(1859),
    [sym_duet_dev_type_document_camera] = ACTIONS(1859),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1859),
    [sym_duet_dev_type_hvac] = ACTIONS(1859),
    [sym_duet_dev_type_keypad] = ACTIONS(1859),
    [sym_duet_dev_type_light] = ACTIONS(1859),
    [sym_duet_dev_type_monitor] = ACTIONS(1859),
    [sym_duet_dev_type_motor] = ACTIONS(1859),
    [sym_duet_dev_type_multi_window] = ACTIONS(1859),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1859),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1859),
    [sym_duet_dev_type_receiver] = ACTIONS(1859),
    [sym_duet_dev_type_security_system] = ACTIONS(1859),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1859),
    [sym_duet_dev_type_settop_box] = ACTIONS(1859),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1859),
    [sym_duet_dev_type_switcher] = ACTIONS(1859),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1859),
    [sym_duet_dev_type_tv] = ACTIONS(1859),
    [sym_duet_dev_type_utility] = ACTIONS(1859),
    [sym_duet_dev_type_vcr] = ACTIONS(1859),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1859),
    [sym_duet_dev_type_video_processor] = ACTIONS(1859),
    [sym_duet_dev_type_video_projector] = ACTIONS(1859),
    [sym_duet_dev_type_video_wall] = ACTIONS(1859),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1859),
    [sym_duet_dev_type_weather] = ACTIONS(1859),
    [sym_duet_dev_not_polled] = ACTIONS(1859),
    [sym_duet_dev_polled] = ACTIONS(1859),
    [sym_clkmgr_mode_network] = ACTIONS(1859),
    [sym_clkmgr_mode_standalone] = ACTIONS(1859),
    [sym_normal_standby] = ACTIONS(1859),
    [sym_normal_wake] = ACTIONS(1859),
    [sym_ip_multicast_ttl_option] = ACTIONS(1859),
    [sym_ip_ttl_subnet] = ACTIONS(1859),
    [sym_ip_ttl_site] = ACTIONS(1859),
    [sym_ip_ttl_region] = ACTIONS(1859),
    [sym_ip_ttl_continent] = ACTIONS(1859),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1859),
    [sym_ip_nodelay_on] = ACTIONS(1859),
    [sym_ip_nodelay_off] = ACTIONS(1859),
    [sym_amx_error] = ACTIONS(1859),
    [sym_amx_warning] = ACTIONS(1859),
    [sym_amx_info] = ACTIONS(1859),
    [sym_amx_debug] = ACTIONS(1859),
    [sym_netlinx_login_success] = ACTIONS(1859),
    [sym_netlinx_login_fail] = ACTIONS(1859),
    [sym_netlinx_logout] = ACTIONS(1859),
    [sym_null_str] = ACTIONS(1859),
    [sym_smtp_address] = ACTIONS(1859),
    [sym_smtp_port_number] = ACTIONS(1859),
    [sym_smtp_username] = ACTIONS(1859),
    [sym_smtp_password] = ACTIONS(1859),
    [sym_smtp_require_tls] = ACTIONS(1859),
    [sym_smtp_from] = ACTIONS(1859),
    [sym_smtp_tls_true] = ACTIONS(1859),
    [sym_smtp_tls_false] = ACTIONS(1859),
    [sym_listview_on_row_select_event] = ACTIONS(1859),
    [sym_data_structure_unknown] = ACTIONS(1859),
    [sym_data_structure_datafeed] = ACTIONS(1859),
    [sym_data_structure_datarecordset] = ACTIONS(1859),
    [sym_data_structure_datarecord] = ACTIONS(1859),
    [sym_data_structure_datafield] = ACTIONS(1859),
    [sym_data_max_content_field_count] = ACTIONS(1859),
    [sym_data_max_metadata_field_count] = ACTIONS(1859),
    [sym_data_max_id_length] = ACTIONS(1859),
    [sym_data_max_name_length] = ACTIONS(1859),
    [sym_data_max_description_length] = ACTIONS(1859),
    [sym_data_max_type_length] = ACTIONS(1859),
    [sym_data_max_format_length] = ACTIONS(1859),
    [sym_data_max_source_length] = ACTIONS(1859),
    [sym_data_max_value_length] = ACTIONS(1859),
    [sym_data_max_label_length] = ACTIONS(1859),
    [sym_data_max_url_length] = ACTIONS(1859),
    [sym_max_last_login_info_length] = ACTIONS(1859),
    [sym_data_type_unknown] = ACTIONS(1859),
    [sym_data_type_string] = ACTIONS(1859),
    [sym_data_type_datetime] = ACTIONS(1859),
    [sym_data_type_date] = ACTIONS(1859),
    [sym_data_type_time] = ACTIONS(1859),
    [sym_data_type_image] = ACTIONS(1859),
    [sym_data_format_url] = ACTIONS(1859),
    [sym_data_format_phone] = ACTIONS(1859),
    [sym_data_format_email] = ACTIONS(1859),
    [sym_data_format_iso8601] = ACTIONS(1859),
    [sym_auditlog_priv] = ACTIONS(1859),
    [sym_configuration_priv] = ACTIONS(1859),
    [sym_deviceconfig_priv] = ACTIONS(1859),
    [sym_ftp_priv] = ACTIONS(1859),
    [sym_http_priv] = ACTIONS(1859),
    [sym_networkconfig_priv] = ACTIONS(1859),
    [sym_programport_priv] = ACTIONS(1859),
    [sym_remoteui_priv] = ACTIONS(1859),
    [sym_securitycontrol_priv] = ACTIONS(1859),
    [sym_softwaremanagement_priv] = ACTIONS(1859),
    [sym_terminal_priv] = ACTIONS(1859),
    [sym_tpadmin_priv] = ACTIONS(1859),
    [sym_user1_priv] = ACTIONS(1859),
    [sym_user2_priv] = ACTIONS(1859),
    [sym_user3_priv] = ACTIONS(1859),
    [sym_user4_priv] = ACTIONS(1859),
    [sym_usermanagement_priv] = ACTIONS(1859),
    [sym_valid_account] = ACTIONS(1859),
    [sym_err_name_invalid] = ACTIONS(1859),
    [sym_err_password_invalid] = ACTIONS(1859),
    [sym_err_invalid_account] = ACTIONS(1859),
    [sym_err_no_authorization] = ACTIONS(1859),
    [sym_err_invalid_parameter] = ACTIONS(1859),
    [sym_err_user_locked_out] = ACTIONS(1859),
    [sym_err_user_account_expired] = ACTIONS(1859),
    [sym_err_general_error] = ACTIONS(1859),
    [sym_err_authentication_error] = ACTIONS(1859),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1859),
    [sym_tls_validate_certificate] = ACTIONS(1859),
    [sym_tdata] = ACTIONS(1859),
    [sym_tchannel] = ACTIONS(1859),
    [sym_tlevel] = ACTIONS(1859),
    [sym_tbutton] = ACTIONS(1859),
    [sym_ttimeline] = ACTIONS(1859),
    [sym_tcustom] = ACTIONS(1859),
    [sym_url_struct] = ACTIONS(1859),
    [sym_dns_struct] = ACTIONS(1859),
    [sym_ip_address_struct] = ACTIONS(1859),
    [sym_dev_info_struct] = ACTIONS(1859),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1859),
    [sym_data_feed] = ACTIONS(1859),
    [sym_data_field] = ACTIONS(1859),
    [sym_data_record] = ACTIONS(1859),
    [sym_wc_data_feed] = ACTIONS(1859),
    [sym_wc_data_field] = ACTIONS(1859),
    [sym_wc_data_record] = ACTIONS(1859),
    [sym_last_login_info] = ACTIONS(1859),
    [sym____reserved___] = ACTIONS(1859),
    [sym_date] = ACTIONS(1859),
    [sym_day] = ACTIONS(1859),
    [sym_time] = ACTIONS(1859),
    [sym_ldate] = ACTIONS(1859),
    [sym_push_device] = ACTIONS(1859),
    [sym_push_channel] = ACTIONS(1859),
    [sym_push_devchan] = ACTIONS(1859),
    [sym_release_device] = ACTIONS(1859),
    [sym_release_channel] = ACTIONS(1859),
    [sym_release_devchan] = ACTIONS(1859),
    [sym_master_slot] = ACTIONS(1859),
    [sym_get_pulse_time] = ACTIONS(1859),
    [sym_get_timer] = ACTIONS(1859),
    [sym_master_sn] = ACTIONS(1859),
    [sym_system_number] = ACTIONS(1859),
    [sym_button] = ACTIONS(1859),
    [sym_dv_channel] = ACTIONS(1859),
    [sym_data] = ACTIONS(1859),
    [sym_level] = ACTIONS(1859),
    [sym_timeline] = ACTIONS(1859),
    [sym_channel] = ACTIONS(1859),
    [sym_custom] = ACTIONS(1859),
    [sym_atoi] = ACTIONS(1859),
    [sym_atol] = ACTIONS(1859),
    [sym_atof] = ACTIONS(1859),
    [sym_itoa] = ACTIONS(1859),
    [sym_itohex] = ACTIONS(1859),
    [sym_hextoi] = ACTIONS(1859),
    [sym_ftoa] = ACTIONS(1859),
    [sym_find_string] = ACTIONS(1859),
    [sym_left_string] = ACTIONS(1859),
    [sym_right_string] = ACTIONS(1859),
    [sym_mid_string] = ACTIONS(1859),
    [sym_remove_string] = ACTIONS(1859),
    [sym_get_buffer_string] = ACTIONS(1859),
    [sym_get_buffer_char] = ACTIONS(1859),
    [sym_get_multi_buffer_string] = ACTIONS(1859),
    [sym_length_string] = ACTIONS(1859),
    [sym_max_length_string] = ACTIONS(1859),
    [sym_set_length_string] = ACTIONS(1859),
    [sym_compare_string] = ACTIONS(1859),
    [sym_upper_string] = ACTIONS(1859),
    [sym_lower_string] = ACTIONS(1859),
    [sym_get_last] = ACTIONS(1859),
    [sym_date_to_day] = ACTIONS(1859),
    [sym_date_to_month] = ACTIONS(1859),
    [sym_date_to_year] = ACTIONS(1859),
    [sym_time_to_hour] = ACTIONS(1859),
    [sym_time_to_minute] = ACTIONS(1859),
    [sym_time_to_second] = ACTIONS(1859),
    [sym_day_of_week] = ACTIONS(1859),
    [sym_set_pulse_time] = ACTIONS(1859),
    [sym_set_timer] = ACTIONS(1859),
    [sym_file_open] = ACTIONS(1859),
    [sym_file_close] = ACTIONS(1859),
    [sym_file_read] = ACTIONS(1859),
    [sym_file_read_line] = ACTIONS(1859),
    [sym_file_write] = ACTIONS(1859),
    [sym_file_write_line] = ACTIONS(1859),
    [sym_file_copy] = ACTIONS(1859),
    [sym_file_delete] = ACTIONS(1859),
    [sym_file_rename] = ACTIONS(1859),
    [sym_file_dir] = ACTIONS(1859),
    [sym_file_getdir] = ACTIONS(1859),
    [sym_file_setdir] = ACTIONS(1859),
    [sym_file_create_dir] = ACTIONS(1859),
    [sym_file_remove_dir] = ACTIONS(1859),
    [sym_file_seek] = ACTIONS(1859),
    [sym_length_array] = ACTIONS(1859),
    [sym_max_length_array] = ACTIONS(1859),
    [sym_set_length_array] = ACTIONS(1859),
    [sym_ip_client_open] = ACTIONS(1859),
    [sym_ip_client_close] = ACTIONS(1859),
    [sym_ip_server_open] = ACTIONS(1859),
    [sym_ip_server_close] = ACTIONS(1859),
    [sym_ip_bound_client_open] = ACTIONS(1859),
    [sym_ip_mc_server_open] = ACTIONS(1859),
    [sym_ip_set_option] = ACTIONS(1859),
    [sym_ssh_client_open] = ACTIONS(1859),
    [sym_ssh_client_close] = ACTIONS(1859),
    [sym_tls_client_open] = ACTIONS(1859),
    [sym_tls_client_close] = ACTIONS(1859),
    [sym_do_push] = ACTIONS(1859),
    [sym_do_push_timed] = ACTIONS(1859),
    [sym_do_release] = ACTIONS(1859),
    [sym_device_id] = ACTIONS(1859),
    [sym_device_info] = ACTIONS(1859),
    [sym_device_id_string] = ACTIONS(1859),
    [sym_device_standby] = ACTIONS(1859),
    [sym_device_wake] = ACTIONS(1859),
    [sym_timeline_create] = ACTIONS(1859),
    [sym_timeline_kill] = ACTIONS(1859),
    [sym_timeline_reload] = ACTIONS(1859),
    [sym_timeline_pause] = ACTIONS(1859),
    [sym_timeline_restart] = ACTIONS(1859),
    [sym_timeline_set] = ACTIONS(1859),
    [sym_timeline_get] = ACTIONS(1859),
    [sym_timeline_active] = ACTIONS(1859),
    [sym_timeline_dynamic_id] = ACTIONS(1859),
    [sym_variable_to_string] = ACTIONS(1859),
    [sym_string_to_variable] = ACTIONS(1859),
    [sym_length_variable_to_string] = ACTIONS(1859),
    [sym_type_cast] = ACTIONS(1859),
    [sym_variable_to_xml] = ACTIONS(1859),
    [sym_xml_to_variable] = ACTIONS(1859),
    [sym_length_variable_to_xml] = ACTIONS(1859),
    [sym_redirect_string] = ACTIONS(1859),
    [sym_random_number] = ACTIONS(1859),
    [sym_get_unique_id] = ACTIONS(1859),
    [sym_get_system_number] = ACTIONS(1859),
    [sym_set_system_number] = ACTIONS(1859),
    [sym_get_serial_number] = ACTIONS(1859),
    [sym_get_master_build] = ACTIONS(1859),
    [sym_set_validation_code] = ACTIONS(1859),
    [sym_reboot] = ACTIONS(1859),
    [sym_load_duet_module] = ACTIONS(1859),
    [sym_unload_duet_module] = ACTIONS(1859),
    [sym_duet_mem_size_set] = ACTIONS(1859),
    [sym_duet_mem_size_get] = ACTIONS(1859),
    [sym_dynamic_application_device] = ACTIONS(1859),
    [sym_dynamic_polled_port] = ACTIONS(1859),
    [sym_static_port_binding] = ACTIONS(1859),
    [sym_static_ip_binding] = ACTIONS(1859),
    [sym_get_dns_list] = ACTIONS(1859),
    [sym_set_dns_list] = ACTIONS(1859),
    [sym_get_ip_address] = ACTIONS(1859),
    [sym_set_ip_address] = ACTIONS(1859),
    [sym_get_url_list] = ACTIONS(1859),
    [sym_add_url_entry] = ACTIONS(1859),
    [sym_delete_url_entry] = ACTIONS(1859),
    [sym_abs_value] = ACTIONS(1859),
    [sym_max_value] = ACTIONS(1859),
    [sym_min_value] = ACTIONS(1859),
    [sym_exp_value] = ACTIONS(1859),
    [sym_log_value] = ACTIONS(1859),
    [sym_log10_value] = ACTIONS(1859),
    [sym_power_value] = ACTIONS(1859),
    [sym_sqrt_value] = ACTIONS(1859),
    [sym_raw_be] = ACTIONS(1859),
    [sym_raw_le] = ACTIONS(1859),
    [sym_internal_queue_size_set] = ACTIONS(1859),
    [sym_internal_queue_size_get] = ACTIONS(1859),
    [sym_internal_threshold_set] = ACTIONS(1859),
    [sym_internal_threshold_get] = ACTIONS(1859),
    [sym_rebuilt_event] = ACTIONS(1859),
    [sym_do_custom_event] = ACTIONS(1859),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1859),
    [sym_clkmgr_set_clk_source] = ACTIONS(1859),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1859),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1859),
    [sym_clkmgr_get_timezone] = ACTIONS(1859),
    [sym_clkmgr_set_timezone] = ACTIONS(1859),
    [sym_clkmgr_get_resync_period] = ACTIONS(1859),
    [sym_clkmgr_set_resync_period] = ACTIONS(1859),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1859),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1859),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1859),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1859),
    [sym_clkmgr_get_timeservers] = ACTIONS(1859),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1859),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1859),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1859),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1859),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1859),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1859),
    [sym_set_log_level] = ACTIONS(1859),
    [sym_get_log_level] = ACTIONS(1859),
    [sym_amx_log] = ACTIONS(1859),
    [sym_smtp_server_config_set] = ACTIONS(1859),
    [sym_smtp_server_config_get] = ACTIONS(1859),
    [sym_smtp_send] = ACTIONS(1859),
    [sym_validate_netlinx_account] = ACTIONS(1859),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1859),
    [sym_authenticate_certificate] = ACTIONS(1859),
    [sym_audit_netlinx_session_event] = ACTIONS(1859),
    [sym_audit_netlinx_generic_event] = ACTIONS(1859),
    [sym_get_available_flash_disk_space] = ACTIONS(1859),
    [sym_get_max_flash_disk_space] = ACTIONS(1859),
    [sym_data_create_feed] = ACTIONS(1859),
    [sym_data_delete_feed] = ACTIONS(1859),
    [sym_data_publish_feed] = ACTIONS(1859),
    [sym_data_get_published_feed] = ACTIONS(1859),
    [sym_data_add_record] = ACTIONS(1859),
    [sym_data_get_event_record] = ACTIONS(1859),
    [sym__wc_data_create_feed] = ACTIONS(1859),
    [sym__wc_data_add_record] = ACTIONS(1859),
    [sym__wc_data_get_event_record] = ACTIONS(1859),
    [sym_format] = ACTIONS(1859),
    [sym_astro_clock] = ACTIONS(1859),
    [sym_set_outdoor_temperature] = ACTIONS(1859),
    [sym_set_virtual_level_count] = ACTIONS(1859),
    [sym_set_virtual_channel_count] = ACTIONS(1859),
    [sym_set_virtual_port_count] = ACTIONS(1859),
    [sym__wc_encode_int] = ACTIONS(1859),
    [sym__wc_decode_int] = ACTIONS(1859),
    [sym__wc_int] = ACTIONS(1859),
    [sym__wc_to_ch_int] = ACTIONS(1859),
    [sym__ch_to_wc_int] = ACTIONS(1859),
    [sym__wc_find_string_int] = ACTIONS(1859),
    [sym__wc_left_string_int] = ACTIONS(1859),
    [sym__wc_lower_string_int] = ACTIONS(1859),
    [sym__wc_mid_string_int] = ACTIONS(1859),
    [sym__wc_remove_string_int] = ACTIONS(1859),
    [sym__wc_right_string_int] = ACTIONS(1859),
    [sym__wc_upper_string_int] = ACTIONS(1859),
    [sym__wc_compare_string_int] = ACTIONS(1859),
    [sym__wc_get_buffer_char_int] = ACTIONS(1859),
    [sym__wc_get_buffer_string_int] = ACTIONS(1859),
    [sym__wc_concat_string_int] = ACTIONS(1859),
    [sym__wc_file_open_int] = ACTIONS(1859),
    [sym__wc_file_close_int] = ACTIONS(1859),
    [sym__wc_file_read_int] = ACTIONS(1859),
    [sym__wc_file_read_line_int] = ACTIONS(1859),
    [sym__wc_file_write_int] = ACTIONS(1859),
    [sym__wc_file_write_line_int] = ACTIONS(1859),
    [aux_sym_preproc_include_token1] = ACTIONS(1859),
    [aux_sym_preproc_define_token1] = ACTIONS(1861),
    [aux_sym_preproc_warn_token1] = ACTIONS(1861),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1861),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1861),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1861),
    [anon_sym_LBRACK] = ACTIONS(1861),
    [anon_sym_LBRACE] = ACTIONS(1861),
    [anon_sym_RBRACE] = ACTIONS(1861),
    [anon_sym_LPAREN] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1861),
    [anon_sym_TILDE] = ACTIONS(1861),
    [anon_sym_DASH] = ACTIONS(1859),
    [anon_sym_PLUS] = ACTIONS(1859),
    [anon_sym_DASH_DASH] = ACTIONS(1861),
    [anon_sym_PLUS_PLUS] = ACTIONS(1861),
    [anon_sym_DQUOTE] = ACTIONS(1861),
    [anon_sym_SQUOTE] = ACTIONS(1861),
    [sym_number_literal] = ACTIONS(1861),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(474)] = {
    [sym_identifier] = ACTIONS(1863),
    [sym_define_call_keyword] = ACTIONS(1863),
    [sym_define_function_keyword] = ACTIONS(1863),
    [sym_define_library_function_keyword] = ACTIONS(1863),
    [sym_define_module_keyword] = ACTIONS(1863),
    [sym_char_keyword] = ACTIONS(1863),
    [sym_widechar_keyword] = ACTIONS(1863),
    [sym_integer_keyword] = ACTIONS(1863),
    [sym_sinteger_keyword] = ACTIONS(1863),
    [sym_long_keyword] = ACTIONS(1863),
    [sym_slong_keyword] = ACTIONS(1863),
    [sym_float_keyword] = ACTIONS(1863),
    [sym_double_keyword] = ACTIONS(1863),
    [sym_variant_keyword] = ACTIONS(1863),
    [sym_variantarray_keyword] = ACTIONS(1863),
    [sym_local_var_keyword] = ACTIONS(1863),
    [sym_stack_var_keyword] = ACTIONS(1863),
    [sym_constant_keyword] = ACTIONS(1863),
    [sym_volatile_keyword] = ACTIONS(1863),
    [sym_non_volatile_keyword] = ACTIONS(1863),
    [sym_persistent_keyword] = ACTIONS(1863),
    [sym_dev_keyword] = ACTIONS(1863),
    [sym_devlev_keyword] = ACTIONS(1863),
    [sym_devchan_keyword] = ACTIONS(1863),
    [sym_if_keyword] = ACTIONS(1863),
    [sym_else_keyword] = ACTIONS(1863),
    [sym_switch_keyword] = ACTIONS(1863),
    [sym_case_keyword] = ACTIONS(1863),
    [sym_default_keyword] = ACTIONS(1863),
    [sym_while_keyword] = ACTIONS(1863),
    [sym_for_keyword] = ACTIONS(1863),
    [sym_break_keyword] = ACTIONS(1863),
    [sym_continue_keyword] = ACTIONS(1863),
    [sym_return_keyword] = ACTIONS(1863),
    [sym_select_keyword] = ACTIONS(1863),
    [sym_active_keyword] = ACTIONS(1863),
    [sym_struct_keyword] = ACTIONS(1863),
    [sym_structure_keyword] = ACTIONS(1863),
    [aux_sym_on_keyword_token1] = ACTIONS(1863),
    [aux_sym_off_keyword_token1] = ACTIONS(1863),
    [sym_send_level_keyword] = ACTIONS(1863),
    [sym_send_string_keyword] = ACTIONS(1863),
    [sym_send_command_keyword] = ACTIONS(1863),
    [sym_clear_buffer_keyword] = ACTIONS(1863),
    [sym_create_buffer_keyword] = ACTIONS(1863),
    [sym_create_multi_buffer_keyword] = ACTIONS(1863),
    [sym_call_keyword] = ACTIONS(1863),
    [sym_system_call_keyword] = ACTIONS(1863),
    [sym_devchan_to_keyword] = ACTIONS(1863),
    [sym_devchan_min_to_keyword] = ACTIONS(1863),
    [sym_devchan_total_off_keyword] = ACTIONS(1863),
    [sym_devchan_pulse_keyword] = ACTIONS(1863),
    [sym_wait_keyword] = ACTIONS(1863),
    [sym_cancel_wait_keyword] = ACTIONS(1863),
    [sym_wait_until_keyword] = ACTIONS(1863),
    [sym_cancel_wait_until_keyword] = ACTIONS(1863),
    [sym_cancel_all_wait_keyword] = ACTIONS(1863),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1863),
    [sym_not] = ACTIONS(1863),
    [sym_bnot] = ACTIONS(1863),
    [sym___file__] = ACTIONS(1863),
    [sym___date__] = ACTIONS(1863),
    [sym___ldate__] = ACTIONS(1863),
    [sym___time__] = ACTIONS(1863),
    [sym___line__] = ACTIONS(1863),
    [sym___name__] = ACTIONS(1863),
    [sym___netlinx__] = ACTIONS(1863),
    [sym_netlinx_axi_version] = ACTIONS(1863),
    [sym_url_flg_tcp] = ACTIONS(1863),
    [sym_url_flg_accinfopresent] = ACTIONS(1863),
    [sym_url_flg_temp] = ACTIONS(1863),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1863),
    [sym_url_flg_stat_mask] = ACTIONS(1863),
    [sym_url_flg_stat_lookup] = ACTIONS(1863),
    [sym_url_flg_stat_connecting] = ACTIONS(1863),
    [sym_url_flg_stat_waiting] = ACTIONS(1863),
    [sym_url_flg_stat_connected] = ACTIONS(1863),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1863),
    [sym_first_virtual_device] = ACTIONS(1863),
    [sym_first_local_port] = ACTIONS(1863),
    [sym_dynamic_virtual_device] = ACTIONS(1863),
    [sym_timeline_once] = ACTIONS(1863),
    [sym_timeline_repeat] = ACTIONS(1863),
    [sym_timeline_absolute] = ACTIONS(1863),
    [sym_timeline_relative] = ACTIONS(1863),
    [sym_do_push_timed_infinite] = ACTIONS(1863),
    [sym_source_type_no_address] = ACTIONS(1863),
    [sym_source_type_neuron_id] = ACTIONS(1863),
    [sym_source_type_ip_address] = ACTIONS(1863),
    [sym_source_type_axlink] = ACTIONS(1863),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1863),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1863),
    [sym_source_type_ip_socket_address] = ACTIONS(1863),
    [sym_source_type_rs232] = ACTIONS(1863),
    [sym_source_type_internal] = ACTIONS(1863),
    [sym_source_type_mac_address] = ACTIONS(1863),
    [sym_source_type_ipv4_port] = ACTIONS(1863),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1863),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1863),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1863),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1863),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1863),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1863),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1863),
    [sym_true] = ACTIONS(1863),
    [sym_false] = ACTIONS(1863),
    [sym_file_read_only] = ACTIONS(1863),
    [sym_file_rw_new] = ACTIONS(1863),
    [sym_file_rw_append] = ACTIONS(1863),
    [sym_ip_tcp] = ACTIONS(1863),
    [sym_ip_udp] = ACTIONS(1863),
    [sym_ip_udp_2way] = ACTIONS(1863),
    [sym_xml_encode_types] = ACTIONS(1863),
    [sym_xml_encode_char_as_list] = ACTIONS(1863),
    [sym_xml_encode_le] = ACTIONS(1863),
    [sym_xml_decode_types] = ACTIONS(1863),
    [sym_xml_decode_no_preserve] = ACTIONS(1863),
    [sym_duet_dev_type_display_device] = ACTIONS(1863),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1863),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1863),
    [sym_duet_dev_type_io_device] = ACTIONS(1863),
    [sym_duet_dev_type_relay_device] = ACTIONS(1863),
    [sym_duet_dev_type_ups] = ACTIONS(1863),
    [sym_duet_dev_type_amplifier] = ACTIONS(1863),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1863),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1863),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1863),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1863),
    [sym_duet_dev_type_camera] = ACTIONS(1863),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1863),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1863),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1863),
    [sym_duet_dev_type_dss] = ACTIONS(1863),
    [sym_duet_dev_type_dvr] = ACTIONS(1863),
    [sym_duet_dev_type_disc_device] = ACTIONS(1863),
    [sym_duet_dev_type_document_camera] = ACTIONS(1863),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1863),
    [sym_duet_dev_type_hvac] = ACTIONS(1863),
    [sym_duet_dev_type_keypad] = ACTIONS(1863),
    [sym_duet_dev_type_light] = ACTIONS(1863),
    [sym_duet_dev_type_monitor] = ACTIONS(1863),
    [sym_duet_dev_type_motor] = ACTIONS(1863),
    [sym_duet_dev_type_multi_window] = ACTIONS(1863),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1863),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1863),
    [sym_duet_dev_type_receiver] = ACTIONS(1863),
    [sym_duet_dev_type_security_system] = ACTIONS(1863),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1863),
    [sym_duet_dev_type_settop_box] = ACTIONS(1863),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1863),
    [sym_duet_dev_type_switcher] = ACTIONS(1863),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1863),
    [sym_duet_dev_type_tv] = ACTIONS(1863),
    [sym_duet_dev_type_utility] = ACTIONS(1863),
    [sym_duet_dev_type_vcr] = ACTIONS(1863),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1863),
    [sym_duet_dev_type_video_processor] = ACTIONS(1863),
    [sym_duet_dev_type_video_projector] = ACTIONS(1863),
    [sym_duet_dev_type_video_wall] = ACTIONS(1863),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1863),
    [sym_duet_dev_type_weather] = ACTIONS(1863),
    [sym_duet_dev_not_polled] = ACTIONS(1863),
    [sym_duet_dev_polled] = ACTIONS(1863),
    [sym_clkmgr_mode_network] = ACTIONS(1863),
    [sym_clkmgr_mode_standalone] = ACTIONS(1863),
    [sym_normal_standby] = ACTIONS(1863),
    [sym_normal_wake] = ACTIONS(1863),
    [sym_ip_multicast_ttl_option] = ACTIONS(1863),
    [sym_ip_ttl_subnet] = ACTIONS(1863),
    [sym_ip_ttl_site] = ACTIONS(1863),
    [sym_ip_ttl_region] = ACTIONS(1863),
    [sym_ip_ttl_continent] = ACTIONS(1863),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1863),
    [sym_ip_nodelay_on] = ACTIONS(1863),
    [sym_ip_nodelay_off] = ACTIONS(1863),
    [sym_amx_error] = ACTIONS(1863),
    [sym_amx_warning] = ACTIONS(1863),
    [sym_amx_info] = ACTIONS(1863),
    [sym_amx_debug] = ACTIONS(1863),
    [sym_netlinx_login_success] = ACTIONS(1863),
    [sym_netlinx_login_fail] = ACTIONS(1863),
    [sym_netlinx_logout] = ACTIONS(1863),
    [sym_null_str] = ACTIONS(1863),
    [sym_smtp_address] = ACTIONS(1863),
    [sym_smtp_port_number] = ACTIONS(1863),
    [sym_smtp_username] = ACTIONS(1863),
    [sym_smtp_password] = ACTIONS(1863),
    [sym_smtp_require_tls] = ACTIONS(1863),
    [sym_smtp_from] = ACTIONS(1863),
    [sym_smtp_tls_true] = ACTIONS(1863),
    [sym_smtp_tls_false] = ACTIONS(1863),
    [sym_listview_on_row_select_event] = ACTIONS(1863),
    [sym_data_structure_unknown] = ACTIONS(1863),
    [sym_data_structure_datafeed] = ACTIONS(1863),
    [sym_data_structure_datarecordset] = ACTIONS(1863),
    [sym_data_structure_datarecord] = ACTIONS(1863),
    [sym_data_structure_datafield] = ACTIONS(1863),
    [sym_data_max_content_field_count] = ACTIONS(1863),
    [sym_data_max_metadata_field_count] = ACTIONS(1863),
    [sym_data_max_id_length] = ACTIONS(1863),
    [sym_data_max_name_length] = ACTIONS(1863),
    [sym_data_max_description_length] = ACTIONS(1863),
    [sym_data_max_type_length] = ACTIONS(1863),
    [sym_data_max_format_length] = ACTIONS(1863),
    [sym_data_max_source_length] = ACTIONS(1863),
    [sym_data_max_value_length] = ACTIONS(1863),
    [sym_data_max_label_length] = ACTIONS(1863),
    [sym_data_max_url_length] = ACTIONS(1863),
    [sym_max_last_login_info_length] = ACTIONS(1863),
    [sym_data_type_unknown] = ACTIONS(1863),
    [sym_data_type_string] = ACTIONS(1863),
    [sym_data_type_datetime] = ACTIONS(1863),
    [sym_data_type_date] = ACTIONS(1863),
    [sym_data_type_time] = ACTIONS(1863),
    [sym_data_type_image] = ACTIONS(1863),
    [sym_data_format_url] = ACTIONS(1863),
    [sym_data_format_phone] = ACTIONS(1863),
    [sym_data_format_email] = ACTIONS(1863),
    [sym_data_format_iso8601] = ACTIONS(1863),
    [sym_auditlog_priv] = ACTIONS(1863),
    [sym_configuration_priv] = ACTIONS(1863),
    [sym_deviceconfig_priv] = ACTIONS(1863),
    [sym_ftp_priv] = ACTIONS(1863),
    [sym_http_priv] = ACTIONS(1863),
    [sym_networkconfig_priv] = ACTIONS(1863),
    [sym_programport_priv] = ACTIONS(1863),
    [sym_remoteui_priv] = ACTIONS(1863),
    [sym_securitycontrol_priv] = ACTIONS(1863),
    [sym_softwaremanagement_priv] = ACTIONS(1863),
    [sym_terminal_priv] = ACTIONS(1863),
    [sym_tpadmin_priv] = ACTIONS(1863),
    [sym_user1_priv] = ACTIONS(1863),
    [sym_user2_priv] = ACTIONS(1863),
    [sym_user3_priv] = ACTIONS(1863),
    [sym_user4_priv] = ACTIONS(1863),
    [sym_usermanagement_priv] = ACTIONS(1863),
    [sym_valid_account] = ACTIONS(1863),
    [sym_err_name_invalid] = ACTIONS(1863),
    [sym_err_password_invalid] = ACTIONS(1863),
    [sym_err_invalid_account] = ACTIONS(1863),
    [sym_err_no_authorization] = ACTIONS(1863),
    [sym_err_invalid_parameter] = ACTIONS(1863),
    [sym_err_user_locked_out] = ACTIONS(1863),
    [sym_err_user_account_expired] = ACTIONS(1863),
    [sym_err_general_error] = ACTIONS(1863),
    [sym_err_authentication_error] = ACTIONS(1863),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1863),
    [sym_tls_validate_certificate] = ACTIONS(1863),
    [sym_tdata] = ACTIONS(1863),
    [sym_tchannel] = ACTIONS(1863),
    [sym_tlevel] = ACTIONS(1863),
    [sym_tbutton] = ACTIONS(1863),
    [sym_ttimeline] = ACTIONS(1863),
    [sym_tcustom] = ACTIONS(1863),
    [sym_url_struct] = ACTIONS(1863),
    [sym_dns_struct] = ACTIONS(1863),
    [sym_ip_address_struct] = ACTIONS(1863),
    [sym_dev_info_struct] = ACTIONS(1863),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1863),
    [sym_data_feed] = ACTIONS(1863),
    [sym_data_field] = ACTIONS(1863),
    [sym_data_record] = ACTIONS(1863),
    [sym_wc_data_feed] = ACTIONS(1863),
    [sym_wc_data_field] = ACTIONS(1863),
    [sym_wc_data_record] = ACTIONS(1863),
    [sym_last_login_info] = ACTIONS(1863),
    [sym____reserved___] = ACTIONS(1863),
    [sym_date] = ACTIONS(1863),
    [sym_day] = ACTIONS(1863),
    [sym_time] = ACTIONS(1863),
    [sym_ldate] = ACTIONS(1863),
    [sym_push_device] = ACTIONS(1863),
    [sym_push_channel] = ACTIONS(1863),
    [sym_push_devchan] = ACTIONS(1863),
    [sym_release_device] = ACTIONS(1863),
    [sym_release_channel] = ACTIONS(1863),
    [sym_release_devchan] = ACTIONS(1863),
    [sym_master_slot] = ACTIONS(1863),
    [sym_get_pulse_time] = ACTIONS(1863),
    [sym_get_timer] = ACTIONS(1863),
    [sym_master_sn] = ACTIONS(1863),
    [sym_system_number] = ACTIONS(1863),
    [sym_button] = ACTIONS(1863),
    [sym_dv_channel] = ACTIONS(1863),
    [sym_data] = ACTIONS(1863),
    [sym_level] = ACTIONS(1863),
    [sym_timeline] = ACTIONS(1863),
    [sym_channel] = ACTIONS(1863),
    [sym_custom] = ACTIONS(1863),
    [sym_atoi] = ACTIONS(1863),
    [sym_atol] = ACTIONS(1863),
    [sym_atof] = ACTIONS(1863),
    [sym_itoa] = ACTIONS(1863),
    [sym_itohex] = ACTIONS(1863),
    [sym_hextoi] = ACTIONS(1863),
    [sym_ftoa] = ACTIONS(1863),
    [sym_find_string] = ACTIONS(1863),
    [sym_left_string] = ACTIONS(1863),
    [sym_right_string] = ACTIONS(1863),
    [sym_mid_string] = ACTIONS(1863),
    [sym_remove_string] = ACTIONS(1863),
    [sym_get_buffer_string] = ACTIONS(1863),
    [sym_get_buffer_char] = ACTIONS(1863),
    [sym_get_multi_buffer_string] = ACTIONS(1863),
    [sym_length_string] = ACTIONS(1863),
    [sym_max_length_string] = ACTIONS(1863),
    [sym_set_length_string] = ACTIONS(1863),
    [sym_compare_string] = ACTIONS(1863),
    [sym_upper_string] = ACTIONS(1863),
    [sym_lower_string] = ACTIONS(1863),
    [sym_get_last] = ACTIONS(1863),
    [sym_date_to_day] = ACTIONS(1863),
    [sym_date_to_month] = ACTIONS(1863),
    [sym_date_to_year] = ACTIONS(1863),
    [sym_time_to_hour] = ACTIONS(1863),
    [sym_time_to_minute] = ACTIONS(1863),
    [sym_time_to_second] = ACTIONS(1863),
    [sym_day_of_week] = ACTIONS(1863),
    [sym_set_pulse_time] = ACTIONS(1863),
    [sym_set_timer] = ACTIONS(1863),
    [sym_file_open] = ACTIONS(1863),
    [sym_file_close] = ACTIONS(1863),
    [sym_file_read] = ACTIONS(1863),
    [sym_file_read_line] = ACTIONS(1863),
    [sym_file_write] = ACTIONS(1863),
    [sym_file_write_line] = ACTIONS(1863),
    [sym_file_copy] = ACTIONS(1863),
    [sym_file_delete] = ACTIONS(1863),
    [sym_file_rename] = ACTIONS(1863),
    [sym_file_dir] = ACTIONS(1863),
    [sym_file_getdir] = ACTIONS(1863),
    [sym_file_setdir] = ACTIONS(1863),
    [sym_file_create_dir] = ACTIONS(1863),
    [sym_file_remove_dir] = ACTIONS(1863),
    [sym_file_seek] = ACTIONS(1863),
    [sym_length_array] = ACTIONS(1863),
    [sym_max_length_array] = ACTIONS(1863),
    [sym_set_length_array] = ACTIONS(1863),
    [sym_ip_client_open] = ACTIONS(1863),
    [sym_ip_client_close] = ACTIONS(1863),
    [sym_ip_server_open] = ACTIONS(1863),
    [sym_ip_server_close] = ACTIONS(1863),
    [sym_ip_bound_client_open] = ACTIONS(1863),
    [sym_ip_mc_server_open] = ACTIONS(1863),
    [sym_ip_set_option] = ACTIONS(1863),
    [sym_ssh_client_open] = ACTIONS(1863),
    [sym_ssh_client_close] = ACTIONS(1863),
    [sym_tls_client_open] = ACTIONS(1863),
    [sym_tls_client_close] = ACTIONS(1863),
    [sym_do_push] = ACTIONS(1863),
    [sym_do_push_timed] = ACTIONS(1863),
    [sym_do_release] = ACTIONS(1863),
    [sym_device_id] = ACTIONS(1863),
    [sym_device_info] = ACTIONS(1863),
    [sym_device_id_string] = ACTIONS(1863),
    [sym_device_standby] = ACTIONS(1863),
    [sym_device_wake] = ACTIONS(1863),
    [sym_timeline_create] = ACTIONS(1863),
    [sym_timeline_kill] = ACTIONS(1863),
    [sym_timeline_reload] = ACTIONS(1863),
    [sym_timeline_pause] = ACTIONS(1863),
    [sym_timeline_restart] = ACTIONS(1863),
    [sym_timeline_set] = ACTIONS(1863),
    [sym_timeline_get] = ACTIONS(1863),
    [sym_timeline_active] = ACTIONS(1863),
    [sym_timeline_dynamic_id] = ACTIONS(1863),
    [sym_variable_to_string] = ACTIONS(1863),
    [sym_string_to_variable] = ACTIONS(1863),
    [sym_length_variable_to_string] = ACTIONS(1863),
    [sym_type_cast] = ACTIONS(1863),
    [sym_variable_to_xml] = ACTIONS(1863),
    [sym_xml_to_variable] = ACTIONS(1863),
    [sym_length_variable_to_xml] = ACTIONS(1863),
    [sym_redirect_string] = ACTIONS(1863),
    [sym_random_number] = ACTIONS(1863),
    [sym_get_unique_id] = ACTIONS(1863),
    [sym_get_system_number] = ACTIONS(1863),
    [sym_set_system_number] = ACTIONS(1863),
    [sym_get_serial_number] = ACTIONS(1863),
    [sym_get_master_build] = ACTIONS(1863),
    [sym_set_validation_code] = ACTIONS(1863),
    [sym_reboot] = ACTIONS(1863),
    [sym_load_duet_module] = ACTIONS(1863),
    [sym_unload_duet_module] = ACTIONS(1863),
    [sym_duet_mem_size_set] = ACTIONS(1863),
    [sym_duet_mem_size_get] = ACTIONS(1863),
    [sym_dynamic_application_device] = ACTIONS(1863),
    [sym_dynamic_polled_port] = ACTIONS(1863),
    [sym_static_port_binding] = ACTIONS(1863),
    [sym_static_ip_binding] = ACTIONS(1863),
    [sym_get_dns_list] = ACTIONS(1863),
    [sym_set_dns_list] = ACTIONS(1863),
    [sym_get_ip_address] = ACTIONS(1863),
    [sym_set_ip_address] = ACTIONS(1863),
    [sym_get_url_list] = ACTIONS(1863),
    [sym_add_url_entry] = ACTIONS(1863),
    [sym_delete_url_entry] = ACTIONS(1863),
    [sym_abs_value] = ACTIONS(1863),
    [sym_max_value] = ACTIONS(1863),
    [sym_min_value] = ACTIONS(1863),
    [sym_exp_value] = ACTIONS(1863),
    [sym_log_value] = ACTIONS(1863),
    [sym_log10_value] = ACTIONS(1863),
    [sym_power_value] = ACTIONS(1863),
    [sym_sqrt_value] = ACTIONS(1863),
    [sym_raw_be] = ACTIONS(1863),
    [sym_raw_le] = ACTIONS(1863),
    [sym_internal_queue_size_set] = ACTIONS(1863),
    [sym_internal_queue_size_get] = ACTIONS(1863),
    [sym_internal_threshold_set] = ACTIONS(1863),
    [sym_internal_threshold_get] = ACTIONS(1863),
    [sym_rebuilt_event] = ACTIONS(1863),
    [sym_do_custom_event] = ACTIONS(1863),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1863),
    [sym_clkmgr_set_clk_source] = ACTIONS(1863),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1863),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1863),
    [sym_clkmgr_get_timezone] = ACTIONS(1863),
    [sym_clkmgr_set_timezone] = ACTIONS(1863),
    [sym_clkmgr_get_resync_period] = ACTIONS(1863),
    [sym_clkmgr_set_resync_period] = ACTIONS(1863),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1863),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1863),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1863),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1863),
    [sym_clkmgr_get_timeservers] = ACTIONS(1863),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1863),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1863),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1863),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1863),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1863),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1863),
    [sym_set_log_level] = ACTIONS(1863),
    [sym_get_log_level] = ACTIONS(1863),
    [sym_amx_log] = ACTIONS(1863),
    [sym_smtp_server_config_set] = ACTIONS(1863),
    [sym_smtp_server_config_get] = ACTIONS(1863),
    [sym_smtp_send] = ACTIONS(1863),
    [sym_validate_netlinx_account] = ACTIONS(1863),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1863),
    [sym_authenticate_certificate] = ACTIONS(1863),
    [sym_audit_netlinx_session_event] = ACTIONS(1863),
    [sym_audit_netlinx_generic_event] = ACTIONS(1863),
    [sym_get_available_flash_disk_space] = ACTIONS(1863),
    [sym_get_max_flash_disk_space] = ACTIONS(1863),
    [sym_data_create_feed] = ACTIONS(1863),
    [sym_data_delete_feed] = ACTIONS(1863),
    [sym_data_publish_feed] = ACTIONS(1863),
    [sym_data_get_published_feed] = ACTIONS(1863),
    [sym_data_add_record] = ACTIONS(1863),
    [sym_data_get_event_record] = ACTIONS(1863),
    [sym__wc_data_create_feed] = ACTIONS(1863),
    [sym__wc_data_add_record] = ACTIONS(1863),
    [sym__wc_data_get_event_record] = ACTIONS(1863),
    [sym_format] = ACTIONS(1863),
    [sym_astro_clock] = ACTIONS(1863),
    [sym_set_outdoor_temperature] = ACTIONS(1863),
    [sym_set_virtual_level_count] = ACTIONS(1863),
    [sym_set_virtual_channel_count] = ACTIONS(1863),
    [sym_set_virtual_port_count] = ACTIONS(1863),
    [sym__wc_encode_int] = ACTIONS(1863),
    [sym__wc_decode_int] = ACTIONS(1863),
    [sym__wc_int] = ACTIONS(1863),
    [sym__wc_to_ch_int] = ACTIONS(1863),
    [sym__ch_to_wc_int] = ACTIONS(1863),
    [sym__wc_find_string_int] = ACTIONS(1863),
    [sym__wc_left_string_int] = ACTIONS(1863),
    [sym__wc_lower_string_int] = ACTIONS(1863),
    [sym__wc_mid_string_int] = ACTIONS(1863),
    [sym__wc_remove_string_int] = ACTIONS(1863),
    [sym__wc_right_string_int] = ACTIONS(1863),
    [sym__wc_upper_string_int] = ACTIONS(1863),
    [sym__wc_compare_string_int] = ACTIONS(1863),
    [sym__wc_get_buffer_char_int] = ACTIONS(1863),
    [sym__wc_get_buffer_string_int] = ACTIONS(1863),
    [sym__wc_concat_string_int] = ACTIONS(1863),
    [sym__wc_file_open_int] = ACTIONS(1863),
    [sym__wc_file_close_int] = ACTIONS(1863),
    [sym__wc_file_read_int] = ACTIONS(1863),
    [sym__wc_file_read_line_int] = ACTIONS(1863),
    [sym__wc_file_write_int] = ACTIONS(1863),
    [sym__wc_file_write_line_int] = ACTIONS(1863),
    [aux_sym_preproc_include_token1] = ACTIONS(1863),
    [aux_sym_preproc_define_token1] = ACTIONS(1865),
    [aux_sym_preproc_warn_token1] = ACTIONS(1865),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1865),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1865),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1865),
    [anon_sym_LBRACK] = ACTIONS(1865),
    [anon_sym_LBRACE] = ACTIONS(1865),
    [anon_sym_RBRACE] = ACTIONS(1865),
    [anon_sym_LPAREN] = ACTIONS(1863),
    [anon_sym_BANG] = ACTIONS(1865),
    [anon_sym_TILDE] = ACTIONS(1865),
    [anon_sym_DASH] = ACTIONS(1863),
    [anon_sym_PLUS] = ACTIONS(1863),
    [anon_sym_DASH_DASH] = ACTIONS(1865),
    [anon_sym_PLUS_PLUS] = ACTIONS(1865),
    [anon_sym_DQUOTE] = ACTIONS(1865),
    [anon_sym_SQUOTE] = ACTIONS(1865),
    [sym_number_literal] = ACTIONS(1865),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(475)] = {
    [sym_identifier] = ACTIONS(1867),
    [sym_define_call_keyword] = ACTIONS(1867),
    [sym_define_function_keyword] = ACTIONS(1867),
    [sym_define_library_function_keyword] = ACTIONS(1867),
    [sym_define_module_keyword] = ACTIONS(1867),
    [sym_char_keyword] = ACTIONS(1867),
    [sym_widechar_keyword] = ACTIONS(1867),
    [sym_integer_keyword] = ACTIONS(1867),
    [sym_sinteger_keyword] = ACTIONS(1867),
    [sym_long_keyword] = ACTIONS(1867),
    [sym_slong_keyword] = ACTIONS(1867),
    [sym_float_keyword] = ACTIONS(1867),
    [sym_double_keyword] = ACTIONS(1867),
    [sym_variant_keyword] = ACTIONS(1867),
    [sym_variantarray_keyword] = ACTIONS(1867),
    [sym_local_var_keyword] = ACTIONS(1867),
    [sym_stack_var_keyword] = ACTIONS(1867),
    [sym_constant_keyword] = ACTIONS(1867),
    [sym_volatile_keyword] = ACTIONS(1867),
    [sym_non_volatile_keyword] = ACTIONS(1867),
    [sym_persistent_keyword] = ACTIONS(1867),
    [sym_dev_keyword] = ACTIONS(1867),
    [sym_devlev_keyword] = ACTIONS(1867),
    [sym_devchan_keyword] = ACTIONS(1867),
    [sym_if_keyword] = ACTIONS(1867),
    [sym_else_keyword] = ACTIONS(1867),
    [sym_switch_keyword] = ACTIONS(1867),
    [sym_case_keyword] = ACTIONS(1867),
    [sym_default_keyword] = ACTIONS(1867),
    [sym_while_keyword] = ACTIONS(1867),
    [sym_for_keyword] = ACTIONS(1867),
    [sym_break_keyword] = ACTIONS(1867),
    [sym_continue_keyword] = ACTIONS(1867),
    [sym_return_keyword] = ACTIONS(1867),
    [sym_select_keyword] = ACTIONS(1867),
    [sym_active_keyword] = ACTIONS(1867),
    [sym_struct_keyword] = ACTIONS(1867),
    [sym_structure_keyword] = ACTIONS(1867),
    [aux_sym_on_keyword_token1] = ACTIONS(1867),
    [aux_sym_off_keyword_token1] = ACTIONS(1867),
    [sym_send_level_keyword] = ACTIONS(1867),
    [sym_send_string_keyword] = ACTIONS(1867),
    [sym_send_command_keyword] = ACTIONS(1867),
    [sym_clear_buffer_keyword] = ACTIONS(1867),
    [sym_create_buffer_keyword] = ACTIONS(1867),
    [sym_create_multi_buffer_keyword] = ACTIONS(1867),
    [sym_call_keyword] = ACTIONS(1867),
    [sym_system_call_keyword] = ACTIONS(1867),
    [sym_devchan_to_keyword] = ACTIONS(1867),
    [sym_devchan_min_to_keyword] = ACTIONS(1867),
    [sym_devchan_total_off_keyword] = ACTIONS(1867),
    [sym_devchan_pulse_keyword] = ACTIONS(1867),
    [sym_wait_keyword] = ACTIONS(1867),
    [sym_cancel_wait_keyword] = ACTIONS(1867),
    [sym_wait_until_keyword] = ACTIONS(1867),
    [sym_cancel_wait_until_keyword] = ACTIONS(1867),
    [sym_cancel_all_wait_keyword] = ACTIONS(1867),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1867),
    [sym_not] = ACTIONS(1867),
    [sym_bnot] = ACTIONS(1867),
    [sym___file__] = ACTIONS(1867),
    [sym___date__] = ACTIONS(1867),
    [sym___ldate__] = ACTIONS(1867),
    [sym___time__] = ACTIONS(1867),
    [sym___line__] = ACTIONS(1867),
    [sym___name__] = ACTIONS(1867),
    [sym___netlinx__] = ACTIONS(1867),
    [sym_netlinx_axi_version] = ACTIONS(1867),
    [sym_url_flg_tcp] = ACTIONS(1867),
    [sym_url_flg_accinfopresent] = ACTIONS(1867),
    [sym_url_flg_temp] = ACTIONS(1867),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1867),
    [sym_url_flg_stat_mask] = ACTIONS(1867),
    [sym_url_flg_stat_lookup] = ACTIONS(1867),
    [sym_url_flg_stat_connecting] = ACTIONS(1867),
    [sym_url_flg_stat_waiting] = ACTIONS(1867),
    [sym_url_flg_stat_connected] = ACTIONS(1867),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1867),
    [sym_first_virtual_device] = ACTIONS(1867),
    [sym_first_local_port] = ACTIONS(1867),
    [sym_dynamic_virtual_device] = ACTIONS(1867),
    [sym_timeline_once] = ACTIONS(1867),
    [sym_timeline_repeat] = ACTIONS(1867),
    [sym_timeline_absolute] = ACTIONS(1867),
    [sym_timeline_relative] = ACTIONS(1867),
    [sym_do_push_timed_infinite] = ACTIONS(1867),
    [sym_source_type_no_address] = ACTIONS(1867),
    [sym_source_type_neuron_id] = ACTIONS(1867),
    [sym_source_type_ip_address] = ACTIONS(1867),
    [sym_source_type_axlink] = ACTIONS(1867),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1867),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1867),
    [sym_source_type_ip_socket_address] = ACTIONS(1867),
    [sym_source_type_rs232] = ACTIONS(1867),
    [sym_source_type_internal] = ACTIONS(1867),
    [sym_source_type_mac_address] = ACTIONS(1867),
    [sym_source_type_ipv4_port] = ACTIONS(1867),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1867),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1867),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1867),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1867),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1867),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1867),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1867),
    [sym_true] = ACTIONS(1867),
    [sym_false] = ACTIONS(1867),
    [sym_file_read_only] = ACTIONS(1867),
    [sym_file_rw_new] = ACTIONS(1867),
    [sym_file_rw_append] = ACTIONS(1867),
    [sym_ip_tcp] = ACTIONS(1867),
    [sym_ip_udp] = ACTIONS(1867),
    [sym_ip_udp_2way] = ACTIONS(1867),
    [sym_xml_encode_types] = ACTIONS(1867),
    [sym_xml_encode_char_as_list] = ACTIONS(1867),
    [sym_xml_encode_le] = ACTIONS(1867),
    [sym_xml_decode_types] = ACTIONS(1867),
    [sym_xml_decode_no_preserve] = ACTIONS(1867),
    [sym_duet_dev_type_display_device] = ACTIONS(1867),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1867),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1867),
    [sym_duet_dev_type_io_device] = ACTIONS(1867),
    [sym_duet_dev_type_relay_device] = ACTIONS(1867),
    [sym_duet_dev_type_ups] = ACTIONS(1867),
    [sym_duet_dev_type_amplifier] = ACTIONS(1867),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1867),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1867),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1867),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1867),
    [sym_duet_dev_type_camera] = ACTIONS(1867),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1867),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1867),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1867),
    [sym_duet_dev_type_dss] = ACTIONS(1867),
    [sym_duet_dev_type_dvr] = ACTIONS(1867),
    [sym_duet_dev_type_disc_device] = ACTIONS(1867),
    [sym_duet_dev_type_document_camera] = ACTIONS(1867),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1867),
    [sym_duet_dev_type_hvac] = ACTIONS(1867),
    [sym_duet_dev_type_keypad] = ACTIONS(1867),
    [sym_duet_dev_type_light] = ACTIONS(1867),
    [sym_duet_dev_type_monitor] = ACTIONS(1867),
    [sym_duet_dev_type_motor] = ACTIONS(1867),
    [sym_duet_dev_type_multi_window] = ACTIONS(1867),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1867),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1867),
    [sym_duet_dev_type_receiver] = ACTIONS(1867),
    [sym_duet_dev_type_security_system] = ACTIONS(1867),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1867),
    [sym_duet_dev_type_settop_box] = ACTIONS(1867),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1867),
    [sym_duet_dev_type_switcher] = ACTIONS(1867),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1867),
    [sym_duet_dev_type_tv] = ACTIONS(1867),
    [sym_duet_dev_type_utility] = ACTIONS(1867),
    [sym_duet_dev_type_vcr] = ACTIONS(1867),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1867),
    [sym_duet_dev_type_video_processor] = ACTIONS(1867),
    [sym_duet_dev_type_video_projector] = ACTIONS(1867),
    [sym_duet_dev_type_video_wall] = ACTIONS(1867),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1867),
    [sym_duet_dev_type_weather] = ACTIONS(1867),
    [sym_duet_dev_not_polled] = ACTIONS(1867),
    [sym_duet_dev_polled] = ACTIONS(1867),
    [sym_clkmgr_mode_network] = ACTIONS(1867),
    [sym_clkmgr_mode_standalone] = ACTIONS(1867),
    [sym_normal_standby] = ACTIONS(1867),
    [sym_normal_wake] = ACTIONS(1867),
    [sym_ip_multicast_ttl_option] = ACTIONS(1867),
    [sym_ip_ttl_subnet] = ACTIONS(1867),
    [sym_ip_ttl_site] = ACTIONS(1867),
    [sym_ip_ttl_region] = ACTIONS(1867),
    [sym_ip_ttl_continent] = ACTIONS(1867),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1867),
    [sym_ip_nodelay_on] = ACTIONS(1867),
    [sym_ip_nodelay_off] = ACTIONS(1867),
    [sym_amx_error] = ACTIONS(1867),
    [sym_amx_warning] = ACTIONS(1867),
    [sym_amx_info] = ACTIONS(1867),
    [sym_amx_debug] = ACTIONS(1867),
    [sym_netlinx_login_success] = ACTIONS(1867),
    [sym_netlinx_login_fail] = ACTIONS(1867),
    [sym_netlinx_logout] = ACTIONS(1867),
    [sym_null_str] = ACTIONS(1867),
    [sym_smtp_address] = ACTIONS(1867),
    [sym_smtp_port_number] = ACTIONS(1867),
    [sym_smtp_username] = ACTIONS(1867),
    [sym_smtp_password] = ACTIONS(1867),
    [sym_smtp_require_tls] = ACTIONS(1867),
    [sym_smtp_from] = ACTIONS(1867),
    [sym_smtp_tls_true] = ACTIONS(1867),
    [sym_smtp_tls_false] = ACTIONS(1867),
    [sym_listview_on_row_select_event] = ACTIONS(1867),
    [sym_data_structure_unknown] = ACTIONS(1867),
    [sym_data_structure_datafeed] = ACTIONS(1867),
    [sym_data_structure_datarecordset] = ACTIONS(1867),
    [sym_data_structure_datarecord] = ACTIONS(1867),
    [sym_data_structure_datafield] = ACTIONS(1867),
    [sym_data_max_content_field_count] = ACTIONS(1867),
    [sym_data_max_metadata_field_count] = ACTIONS(1867),
    [sym_data_max_id_length] = ACTIONS(1867),
    [sym_data_max_name_length] = ACTIONS(1867),
    [sym_data_max_description_length] = ACTIONS(1867),
    [sym_data_max_type_length] = ACTIONS(1867),
    [sym_data_max_format_length] = ACTIONS(1867),
    [sym_data_max_source_length] = ACTIONS(1867),
    [sym_data_max_value_length] = ACTIONS(1867),
    [sym_data_max_label_length] = ACTIONS(1867),
    [sym_data_max_url_length] = ACTIONS(1867),
    [sym_max_last_login_info_length] = ACTIONS(1867),
    [sym_data_type_unknown] = ACTIONS(1867),
    [sym_data_type_string] = ACTIONS(1867),
    [sym_data_type_datetime] = ACTIONS(1867),
    [sym_data_type_date] = ACTIONS(1867),
    [sym_data_type_time] = ACTIONS(1867),
    [sym_data_type_image] = ACTIONS(1867),
    [sym_data_format_url] = ACTIONS(1867),
    [sym_data_format_phone] = ACTIONS(1867),
    [sym_data_format_email] = ACTIONS(1867),
    [sym_data_format_iso8601] = ACTIONS(1867),
    [sym_auditlog_priv] = ACTIONS(1867),
    [sym_configuration_priv] = ACTIONS(1867),
    [sym_deviceconfig_priv] = ACTIONS(1867),
    [sym_ftp_priv] = ACTIONS(1867),
    [sym_http_priv] = ACTIONS(1867),
    [sym_networkconfig_priv] = ACTIONS(1867),
    [sym_programport_priv] = ACTIONS(1867),
    [sym_remoteui_priv] = ACTIONS(1867),
    [sym_securitycontrol_priv] = ACTIONS(1867),
    [sym_softwaremanagement_priv] = ACTIONS(1867),
    [sym_terminal_priv] = ACTIONS(1867),
    [sym_tpadmin_priv] = ACTIONS(1867),
    [sym_user1_priv] = ACTIONS(1867),
    [sym_user2_priv] = ACTIONS(1867),
    [sym_user3_priv] = ACTIONS(1867),
    [sym_user4_priv] = ACTIONS(1867),
    [sym_usermanagement_priv] = ACTIONS(1867),
    [sym_valid_account] = ACTIONS(1867),
    [sym_err_name_invalid] = ACTIONS(1867),
    [sym_err_password_invalid] = ACTIONS(1867),
    [sym_err_invalid_account] = ACTIONS(1867),
    [sym_err_no_authorization] = ACTIONS(1867),
    [sym_err_invalid_parameter] = ACTIONS(1867),
    [sym_err_user_locked_out] = ACTIONS(1867),
    [sym_err_user_account_expired] = ACTIONS(1867),
    [sym_err_general_error] = ACTIONS(1867),
    [sym_err_authentication_error] = ACTIONS(1867),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1867),
    [sym_tls_validate_certificate] = ACTIONS(1867),
    [sym_tdata] = ACTIONS(1867),
    [sym_tchannel] = ACTIONS(1867),
    [sym_tlevel] = ACTIONS(1867),
    [sym_tbutton] = ACTIONS(1867),
    [sym_ttimeline] = ACTIONS(1867),
    [sym_tcustom] = ACTIONS(1867),
    [sym_url_struct] = ACTIONS(1867),
    [sym_dns_struct] = ACTIONS(1867),
    [sym_ip_address_struct] = ACTIONS(1867),
    [sym_dev_info_struct] = ACTIONS(1867),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1867),
    [sym_data_feed] = ACTIONS(1867),
    [sym_data_field] = ACTIONS(1867),
    [sym_data_record] = ACTIONS(1867),
    [sym_wc_data_feed] = ACTIONS(1867),
    [sym_wc_data_field] = ACTIONS(1867),
    [sym_wc_data_record] = ACTIONS(1867),
    [sym_last_login_info] = ACTIONS(1867),
    [sym____reserved___] = ACTIONS(1867),
    [sym_date] = ACTIONS(1867),
    [sym_day] = ACTIONS(1867),
    [sym_time] = ACTIONS(1867),
    [sym_ldate] = ACTIONS(1867),
    [sym_push_device] = ACTIONS(1867),
    [sym_push_channel] = ACTIONS(1867),
    [sym_push_devchan] = ACTIONS(1867),
    [sym_release_device] = ACTIONS(1867),
    [sym_release_channel] = ACTIONS(1867),
    [sym_release_devchan] = ACTIONS(1867),
    [sym_master_slot] = ACTIONS(1867),
    [sym_get_pulse_time] = ACTIONS(1867),
    [sym_get_timer] = ACTIONS(1867),
    [sym_master_sn] = ACTIONS(1867),
    [sym_system_number] = ACTIONS(1867),
    [sym_button] = ACTIONS(1867),
    [sym_dv_channel] = ACTIONS(1867),
    [sym_data] = ACTIONS(1867),
    [sym_level] = ACTIONS(1867),
    [sym_timeline] = ACTIONS(1867),
    [sym_channel] = ACTIONS(1867),
    [sym_custom] = ACTIONS(1867),
    [sym_atoi] = ACTIONS(1867),
    [sym_atol] = ACTIONS(1867),
    [sym_atof] = ACTIONS(1867),
    [sym_itoa] = ACTIONS(1867),
    [sym_itohex] = ACTIONS(1867),
    [sym_hextoi] = ACTIONS(1867),
    [sym_ftoa] = ACTIONS(1867),
    [sym_find_string] = ACTIONS(1867),
    [sym_left_string] = ACTIONS(1867),
    [sym_right_string] = ACTIONS(1867),
    [sym_mid_string] = ACTIONS(1867),
    [sym_remove_string] = ACTIONS(1867),
    [sym_get_buffer_string] = ACTIONS(1867),
    [sym_get_buffer_char] = ACTIONS(1867),
    [sym_get_multi_buffer_string] = ACTIONS(1867),
    [sym_length_string] = ACTIONS(1867),
    [sym_max_length_string] = ACTIONS(1867),
    [sym_set_length_string] = ACTIONS(1867),
    [sym_compare_string] = ACTIONS(1867),
    [sym_upper_string] = ACTIONS(1867),
    [sym_lower_string] = ACTIONS(1867),
    [sym_get_last] = ACTIONS(1867),
    [sym_date_to_day] = ACTIONS(1867),
    [sym_date_to_month] = ACTIONS(1867),
    [sym_date_to_year] = ACTIONS(1867),
    [sym_time_to_hour] = ACTIONS(1867),
    [sym_time_to_minute] = ACTIONS(1867),
    [sym_time_to_second] = ACTIONS(1867),
    [sym_day_of_week] = ACTIONS(1867),
    [sym_set_pulse_time] = ACTIONS(1867),
    [sym_set_timer] = ACTIONS(1867),
    [sym_file_open] = ACTIONS(1867),
    [sym_file_close] = ACTIONS(1867),
    [sym_file_read] = ACTIONS(1867),
    [sym_file_read_line] = ACTIONS(1867),
    [sym_file_write] = ACTIONS(1867),
    [sym_file_write_line] = ACTIONS(1867),
    [sym_file_copy] = ACTIONS(1867),
    [sym_file_delete] = ACTIONS(1867),
    [sym_file_rename] = ACTIONS(1867),
    [sym_file_dir] = ACTIONS(1867),
    [sym_file_getdir] = ACTIONS(1867),
    [sym_file_setdir] = ACTIONS(1867),
    [sym_file_create_dir] = ACTIONS(1867),
    [sym_file_remove_dir] = ACTIONS(1867),
    [sym_file_seek] = ACTIONS(1867),
    [sym_length_array] = ACTIONS(1867),
    [sym_max_length_array] = ACTIONS(1867),
    [sym_set_length_array] = ACTIONS(1867),
    [sym_ip_client_open] = ACTIONS(1867),
    [sym_ip_client_close] = ACTIONS(1867),
    [sym_ip_server_open] = ACTIONS(1867),
    [sym_ip_server_close] = ACTIONS(1867),
    [sym_ip_bound_client_open] = ACTIONS(1867),
    [sym_ip_mc_server_open] = ACTIONS(1867),
    [sym_ip_set_option] = ACTIONS(1867),
    [sym_ssh_client_open] = ACTIONS(1867),
    [sym_ssh_client_close] = ACTIONS(1867),
    [sym_tls_client_open] = ACTIONS(1867),
    [sym_tls_client_close] = ACTIONS(1867),
    [sym_do_push] = ACTIONS(1867),
    [sym_do_push_timed] = ACTIONS(1867),
    [sym_do_release] = ACTIONS(1867),
    [sym_device_id] = ACTIONS(1867),
    [sym_device_info] = ACTIONS(1867),
    [sym_device_id_string] = ACTIONS(1867),
    [sym_device_standby] = ACTIONS(1867),
    [sym_device_wake] = ACTIONS(1867),
    [sym_timeline_create] = ACTIONS(1867),
    [sym_timeline_kill] = ACTIONS(1867),
    [sym_timeline_reload] = ACTIONS(1867),
    [sym_timeline_pause] = ACTIONS(1867),
    [sym_timeline_restart] = ACTIONS(1867),
    [sym_timeline_set] = ACTIONS(1867),
    [sym_timeline_get] = ACTIONS(1867),
    [sym_timeline_active] = ACTIONS(1867),
    [sym_timeline_dynamic_id] = ACTIONS(1867),
    [sym_variable_to_string] = ACTIONS(1867),
    [sym_string_to_variable] = ACTIONS(1867),
    [sym_length_variable_to_string] = ACTIONS(1867),
    [sym_type_cast] = ACTIONS(1867),
    [sym_variable_to_xml] = ACTIONS(1867),
    [sym_xml_to_variable] = ACTIONS(1867),
    [sym_length_variable_to_xml] = ACTIONS(1867),
    [sym_redirect_string] = ACTIONS(1867),
    [sym_random_number] = ACTIONS(1867),
    [sym_get_unique_id] = ACTIONS(1867),
    [sym_get_system_number] = ACTIONS(1867),
    [sym_set_system_number] = ACTIONS(1867),
    [sym_get_serial_number] = ACTIONS(1867),
    [sym_get_master_build] = ACTIONS(1867),
    [sym_set_validation_code] = ACTIONS(1867),
    [sym_reboot] = ACTIONS(1867),
    [sym_load_duet_module] = ACTIONS(1867),
    [sym_unload_duet_module] = ACTIONS(1867),
    [sym_duet_mem_size_set] = ACTIONS(1867),
    [sym_duet_mem_size_get] = ACTIONS(1867),
    [sym_dynamic_application_device] = ACTIONS(1867),
    [sym_dynamic_polled_port] = ACTIONS(1867),
    [sym_static_port_binding] = ACTIONS(1867),
    [sym_static_ip_binding] = ACTIONS(1867),
    [sym_get_dns_list] = ACTIONS(1867),
    [sym_set_dns_list] = ACTIONS(1867),
    [sym_get_ip_address] = ACTIONS(1867),
    [sym_set_ip_address] = ACTIONS(1867),
    [sym_get_url_list] = ACTIONS(1867),
    [sym_add_url_entry] = ACTIONS(1867),
    [sym_delete_url_entry] = ACTIONS(1867),
    [sym_abs_value] = ACTIONS(1867),
    [sym_max_value] = ACTIONS(1867),
    [sym_min_value] = ACTIONS(1867),
    [sym_exp_value] = ACTIONS(1867),
    [sym_log_value] = ACTIONS(1867),
    [sym_log10_value] = ACTIONS(1867),
    [sym_power_value] = ACTIONS(1867),
    [sym_sqrt_value] = ACTIONS(1867),
    [sym_raw_be] = ACTIONS(1867),
    [sym_raw_le] = ACTIONS(1867),
    [sym_internal_queue_size_set] = ACTIONS(1867),
    [sym_internal_queue_size_get] = ACTIONS(1867),
    [sym_internal_threshold_set] = ACTIONS(1867),
    [sym_internal_threshold_get] = ACTIONS(1867),
    [sym_rebuilt_event] = ACTIONS(1867),
    [sym_do_custom_event] = ACTIONS(1867),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1867),
    [sym_clkmgr_set_clk_source] = ACTIONS(1867),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1867),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1867),
    [sym_clkmgr_get_timezone] = ACTIONS(1867),
    [sym_clkmgr_set_timezone] = ACTIONS(1867),
    [sym_clkmgr_get_resync_period] = ACTIONS(1867),
    [sym_clkmgr_set_resync_period] = ACTIONS(1867),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1867),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1867),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1867),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1867),
    [sym_clkmgr_get_timeservers] = ACTIONS(1867),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1867),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1867),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1867),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1867),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1867),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1867),
    [sym_set_log_level] = ACTIONS(1867),
    [sym_get_log_level] = ACTIONS(1867),
    [sym_amx_log] = ACTIONS(1867),
    [sym_smtp_server_config_set] = ACTIONS(1867),
    [sym_smtp_server_config_get] = ACTIONS(1867),
    [sym_smtp_send] = ACTIONS(1867),
    [sym_validate_netlinx_account] = ACTIONS(1867),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1867),
    [sym_authenticate_certificate] = ACTIONS(1867),
    [sym_audit_netlinx_session_event] = ACTIONS(1867),
    [sym_audit_netlinx_generic_event] = ACTIONS(1867),
    [sym_get_available_flash_disk_space] = ACTIONS(1867),
    [sym_get_max_flash_disk_space] = ACTIONS(1867),
    [sym_data_create_feed] = ACTIONS(1867),
    [sym_data_delete_feed] = ACTIONS(1867),
    [sym_data_publish_feed] = ACTIONS(1867),
    [sym_data_get_published_feed] = ACTIONS(1867),
    [sym_data_add_record] = ACTIONS(1867),
    [sym_data_get_event_record] = ACTIONS(1867),
    [sym__wc_data_create_feed] = ACTIONS(1867),
    [sym__wc_data_add_record] = ACTIONS(1867),
    [sym__wc_data_get_event_record] = ACTIONS(1867),
    [sym_format] = ACTIONS(1867),
    [sym_astro_clock] = ACTIONS(1867),
    [sym_set_outdoor_temperature] = ACTIONS(1867),
    [sym_set_virtual_level_count] = ACTIONS(1867),
    [sym_set_virtual_channel_count] = ACTIONS(1867),
    [sym_set_virtual_port_count] = ACTIONS(1867),
    [sym__wc_encode_int] = ACTIONS(1867),
    [sym__wc_decode_int] = ACTIONS(1867),
    [sym__wc_int] = ACTIONS(1867),
    [sym__wc_to_ch_int] = ACTIONS(1867),
    [sym__ch_to_wc_int] = ACTIONS(1867),
    [sym__wc_find_string_int] = ACTIONS(1867),
    [sym__wc_left_string_int] = ACTIONS(1867),
    [sym__wc_lower_string_int] = ACTIONS(1867),
    [sym__wc_mid_string_int] = ACTIONS(1867),
    [sym__wc_remove_string_int] = ACTIONS(1867),
    [sym__wc_right_string_int] = ACTIONS(1867),
    [sym__wc_upper_string_int] = ACTIONS(1867),
    [sym__wc_compare_string_int] = ACTIONS(1867),
    [sym__wc_get_buffer_char_int] = ACTIONS(1867),
    [sym__wc_get_buffer_string_int] = ACTIONS(1867),
    [sym__wc_concat_string_int] = ACTIONS(1867),
    [sym__wc_file_open_int] = ACTIONS(1867),
    [sym__wc_file_close_int] = ACTIONS(1867),
    [sym__wc_file_read_int] = ACTIONS(1867),
    [sym__wc_file_read_line_int] = ACTIONS(1867),
    [sym__wc_file_write_int] = ACTIONS(1867),
    [sym__wc_file_write_line_int] = ACTIONS(1867),
    [aux_sym_preproc_include_token1] = ACTIONS(1867),
    [aux_sym_preproc_define_token1] = ACTIONS(1869),
    [aux_sym_preproc_warn_token1] = ACTIONS(1869),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1869),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1869),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1869),
    [anon_sym_LBRACK] = ACTIONS(1869),
    [anon_sym_LBRACE] = ACTIONS(1869),
    [anon_sym_RBRACE] = ACTIONS(1869),
    [anon_sym_LPAREN] = ACTIONS(1867),
    [anon_sym_BANG] = ACTIONS(1869),
    [anon_sym_TILDE] = ACTIONS(1869),
    [anon_sym_DASH] = ACTIONS(1867),
    [anon_sym_PLUS] = ACTIONS(1867),
    [anon_sym_DASH_DASH] = ACTIONS(1869),
    [anon_sym_PLUS_PLUS] = ACTIONS(1869),
    [anon_sym_DQUOTE] = ACTIONS(1869),
    [anon_sym_SQUOTE] = ACTIONS(1869),
    [sym_number_literal] = ACTIONS(1869),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(476)] = {
    [sym_identifier] = ACTIONS(1835),
    [sym_define_call_keyword] = ACTIONS(1835),
    [sym_define_function_keyword] = ACTIONS(1835),
    [sym_define_library_function_keyword] = ACTIONS(1835),
    [sym_define_module_keyword] = ACTIONS(1835),
    [sym_char_keyword] = ACTIONS(1835),
    [sym_widechar_keyword] = ACTIONS(1835),
    [sym_integer_keyword] = ACTIONS(1835),
    [sym_sinteger_keyword] = ACTIONS(1835),
    [sym_long_keyword] = ACTIONS(1835),
    [sym_slong_keyword] = ACTIONS(1835),
    [sym_float_keyword] = ACTIONS(1835),
    [sym_double_keyword] = ACTIONS(1835),
    [sym_variant_keyword] = ACTIONS(1835),
    [sym_variantarray_keyword] = ACTIONS(1835),
    [sym_local_var_keyword] = ACTIONS(1835),
    [sym_stack_var_keyword] = ACTIONS(1835),
    [sym_constant_keyword] = ACTIONS(1835),
    [sym_volatile_keyword] = ACTIONS(1835),
    [sym_non_volatile_keyword] = ACTIONS(1835),
    [sym_persistent_keyword] = ACTIONS(1835),
    [sym_dev_keyword] = ACTIONS(1835),
    [sym_devlev_keyword] = ACTIONS(1835),
    [sym_devchan_keyword] = ACTIONS(1835),
    [sym_if_keyword] = ACTIONS(1835),
    [sym_else_keyword] = ACTIONS(1835),
    [sym_switch_keyword] = ACTIONS(1835),
    [sym_case_keyword] = ACTIONS(1835),
    [sym_default_keyword] = ACTIONS(1835),
    [sym_while_keyword] = ACTIONS(1835),
    [sym_for_keyword] = ACTIONS(1835),
    [sym_break_keyword] = ACTIONS(1835),
    [sym_continue_keyword] = ACTIONS(1835),
    [sym_return_keyword] = ACTIONS(1835),
    [sym_select_keyword] = ACTIONS(1835),
    [sym_active_keyword] = ACTIONS(1835),
    [sym_struct_keyword] = ACTIONS(1835),
    [sym_structure_keyword] = ACTIONS(1835),
    [aux_sym_on_keyword_token1] = ACTIONS(1835),
    [aux_sym_off_keyword_token1] = ACTIONS(1835),
    [sym_send_level_keyword] = ACTIONS(1835),
    [sym_send_string_keyword] = ACTIONS(1835),
    [sym_send_command_keyword] = ACTIONS(1835),
    [sym_clear_buffer_keyword] = ACTIONS(1835),
    [sym_create_buffer_keyword] = ACTIONS(1835),
    [sym_create_multi_buffer_keyword] = ACTIONS(1835),
    [sym_call_keyword] = ACTIONS(1835),
    [sym_system_call_keyword] = ACTIONS(1835),
    [sym_devchan_to_keyword] = ACTIONS(1835),
    [sym_devchan_min_to_keyword] = ACTIONS(1835),
    [sym_devchan_total_off_keyword] = ACTIONS(1835),
    [sym_devchan_pulse_keyword] = ACTIONS(1835),
    [sym_wait_keyword] = ACTIONS(1835),
    [sym_cancel_wait_keyword] = ACTIONS(1835),
    [sym_wait_until_keyword] = ACTIONS(1835),
    [sym_cancel_wait_until_keyword] = ACTIONS(1835),
    [sym_cancel_all_wait_keyword] = ACTIONS(1835),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1835),
    [sym_not] = ACTIONS(1835),
    [sym_bnot] = ACTIONS(1835),
    [sym___file__] = ACTIONS(1835),
    [sym___date__] = ACTIONS(1835),
    [sym___ldate__] = ACTIONS(1835),
    [sym___time__] = ACTIONS(1835),
    [sym___line__] = ACTIONS(1835),
    [sym___name__] = ACTIONS(1835),
    [sym___netlinx__] = ACTIONS(1835),
    [sym_netlinx_axi_version] = ACTIONS(1835),
    [sym_url_flg_tcp] = ACTIONS(1835),
    [sym_url_flg_accinfopresent] = ACTIONS(1835),
    [sym_url_flg_temp] = ACTIONS(1835),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1835),
    [sym_url_flg_stat_mask] = ACTIONS(1835),
    [sym_url_flg_stat_lookup] = ACTIONS(1835),
    [sym_url_flg_stat_connecting] = ACTIONS(1835),
    [sym_url_flg_stat_waiting] = ACTIONS(1835),
    [sym_url_flg_stat_connected] = ACTIONS(1835),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1835),
    [sym_first_virtual_device] = ACTIONS(1835),
    [sym_first_local_port] = ACTIONS(1835),
    [sym_dynamic_virtual_device] = ACTIONS(1835),
    [sym_timeline_once] = ACTIONS(1835),
    [sym_timeline_repeat] = ACTIONS(1835),
    [sym_timeline_absolute] = ACTIONS(1835),
    [sym_timeline_relative] = ACTIONS(1835),
    [sym_do_push_timed_infinite] = ACTIONS(1835),
    [sym_source_type_no_address] = ACTIONS(1835),
    [sym_source_type_neuron_id] = ACTIONS(1835),
    [sym_source_type_ip_address] = ACTIONS(1835),
    [sym_source_type_axlink] = ACTIONS(1835),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1835),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1835),
    [sym_source_type_ip_socket_address] = ACTIONS(1835),
    [sym_source_type_rs232] = ACTIONS(1835),
    [sym_source_type_internal] = ACTIONS(1835),
    [sym_source_type_mac_address] = ACTIONS(1835),
    [sym_source_type_ipv4_port] = ACTIONS(1835),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1835),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1835),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1835),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1835),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1835),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1835),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1835),
    [sym_true] = ACTIONS(1835),
    [sym_false] = ACTIONS(1835),
    [sym_file_read_only] = ACTIONS(1835),
    [sym_file_rw_new] = ACTIONS(1835),
    [sym_file_rw_append] = ACTIONS(1835),
    [sym_ip_tcp] = ACTIONS(1835),
    [sym_ip_udp] = ACTIONS(1835),
    [sym_ip_udp_2way] = ACTIONS(1835),
    [sym_xml_encode_types] = ACTIONS(1835),
    [sym_xml_encode_char_as_list] = ACTIONS(1835),
    [sym_xml_encode_le] = ACTIONS(1835),
    [sym_xml_decode_types] = ACTIONS(1835),
    [sym_xml_decode_no_preserve] = ACTIONS(1835),
    [sym_duet_dev_type_display_device] = ACTIONS(1835),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1835),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1835),
    [sym_duet_dev_type_io_device] = ACTIONS(1835),
    [sym_duet_dev_type_relay_device] = ACTIONS(1835),
    [sym_duet_dev_type_ups] = ACTIONS(1835),
    [sym_duet_dev_type_amplifier] = ACTIONS(1835),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1835),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1835),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1835),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1835),
    [sym_duet_dev_type_camera] = ACTIONS(1835),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1835),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1835),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1835),
    [sym_duet_dev_type_dss] = ACTIONS(1835),
    [sym_duet_dev_type_dvr] = ACTIONS(1835),
    [sym_duet_dev_type_disc_device] = ACTIONS(1835),
    [sym_duet_dev_type_document_camera] = ACTIONS(1835),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1835),
    [sym_duet_dev_type_hvac] = ACTIONS(1835),
    [sym_duet_dev_type_keypad] = ACTIONS(1835),
    [sym_duet_dev_type_light] = ACTIONS(1835),
    [sym_duet_dev_type_monitor] = ACTIONS(1835),
    [sym_duet_dev_type_motor] = ACTIONS(1835),
    [sym_duet_dev_type_multi_window] = ACTIONS(1835),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1835),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1835),
    [sym_duet_dev_type_receiver] = ACTIONS(1835),
    [sym_duet_dev_type_security_system] = ACTIONS(1835),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1835),
    [sym_duet_dev_type_settop_box] = ACTIONS(1835),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1835),
    [sym_duet_dev_type_switcher] = ACTIONS(1835),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1835),
    [sym_duet_dev_type_tv] = ACTIONS(1835),
    [sym_duet_dev_type_utility] = ACTIONS(1835),
    [sym_duet_dev_type_vcr] = ACTIONS(1835),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1835),
    [sym_duet_dev_type_video_processor] = ACTIONS(1835),
    [sym_duet_dev_type_video_projector] = ACTIONS(1835),
    [sym_duet_dev_type_video_wall] = ACTIONS(1835),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1835),
    [sym_duet_dev_type_weather] = ACTIONS(1835),
    [sym_duet_dev_not_polled] = ACTIONS(1835),
    [sym_duet_dev_polled] = ACTIONS(1835),
    [sym_clkmgr_mode_network] = ACTIONS(1835),
    [sym_clkmgr_mode_standalone] = ACTIONS(1835),
    [sym_normal_standby] = ACTIONS(1835),
    [sym_normal_wake] = ACTIONS(1835),
    [sym_ip_multicast_ttl_option] = ACTIONS(1835),
    [sym_ip_ttl_subnet] = ACTIONS(1835),
    [sym_ip_ttl_site] = ACTIONS(1835),
    [sym_ip_ttl_region] = ACTIONS(1835),
    [sym_ip_ttl_continent] = ACTIONS(1835),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1835),
    [sym_ip_nodelay_on] = ACTIONS(1835),
    [sym_ip_nodelay_off] = ACTIONS(1835),
    [sym_amx_error] = ACTIONS(1835),
    [sym_amx_warning] = ACTIONS(1835),
    [sym_amx_info] = ACTIONS(1835),
    [sym_amx_debug] = ACTIONS(1835),
    [sym_netlinx_login_success] = ACTIONS(1835),
    [sym_netlinx_login_fail] = ACTIONS(1835),
    [sym_netlinx_logout] = ACTIONS(1835),
    [sym_null_str] = ACTIONS(1835),
    [sym_smtp_address] = ACTIONS(1835),
    [sym_smtp_port_number] = ACTIONS(1835),
    [sym_smtp_username] = ACTIONS(1835),
    [sym_smtp_password] = ACTIONS(1835),
    [sym_smtp_require_tls] = ACTIONS(1835),
    [sym_smtp_from] = ACTIONS(1835),
    [sym_smtp_tls_true] = ACTIONS(1835),
    [sym_smtp_tls_false] = ACTIONS(1835),
    [sym_listview_on_row_select_event] = ACTIONS(1835),
    [sym_data_structure_unknown] = ACTIONS(1835),
    [sym_data_structure_datafeed] = ACTIONS(1835),
    [sym_data_structure_datarecordset] = ACTIONS(1835),
    [sym_data_structure_datarecord] = ACTIONS(1835),
    [sym_data_structure_datafield] = ACTIONS(1835),
    [sym_data_max_content_field_count] = ACTIONS(1835),
    [sym_data_max_metadata_field_count] = ACTIONS(1835),
    [sym_data_max_id_length] = ACTIONS(1835),
    [sym_data_max_name_length] = ACTIONS(1835),
    [sym_data_max_description_length] = ACTIONS(1835),
    [sym_data_max_type_length] = ACTIONS(1835),
    [sym_data_max_format_length] = ACTIONS(1835),
    [sym_data_max_source_length] = ACTIONS(1835),
    [sym_data_max_value_length] = ACTIONS(1835),
    [sym_data_max_label_length] = ACTIONS(1835),
    [sym_data_max_url_length] = ACTIONS(1835),
    [sym_max_last_login_info_length] = ACTIONS(1835),
    [sym_data_type_unknown] = ACTIONS(1835),
    [sym_data_type_string] = ACTIONS(1835),
    [sym_data_type_datetime] = ACTIONS(1835),
    [sym_data_type_date] = ACTIONS(1835),
    [sym_data_type_time] = ACTIONS(1835),
    [sym_data_type_image] = ACTIONS(1835),
    [sym_data_format_url] = ACTIONS(1835),
    [sym_data_format_phone] = ACTIONS(1835),
    [sym_data_format_email] = ACTIONS(1835),
    [sym_data_format_iso8601] = ACTIONS(1835),
    [sym_auditlog_priv] = ACTIONS(1835),
    [sym_configuration_priv] = ACTIONS(1835),
    [sym_deviceconfig_priv] = ACTIONS(1835),
    [sym_ftp_priv] = ACTIONS(1835),
    [sym_http_priv] = ACTIONS(1835),
    [sym_networkconfig_priv] = ACTIONS(1835),
    [sym_programport_priv] = ACTIONS(1835),
    [sym_remoteui_priv] = ACTIONS(1835),
    [sym_securitycontrol_priv] = ACTIONS(1835),
    [sym_softwaremanagement_priv] = ACTIONS(1835),
    [sym_terminal_priv] = ACTIONS(1835),
    [sym_tpadmin_priv] = ACTIONS(1835),
    [sym_user1_priv] = ACTIONS(1835),
    [sym_user2_priv] = ACTIONS(1835),
    [sym_user3_priv] = ACTIONS(1835),
    [sym_user4_priv] = ACTIONS(1835),
    [sym_usermanagement_priv] = ACTIONS(1835),
    [sym_valid_account] = ACTIONS(1835),
    [sym_err_name_invalid] = ACTIONS(1835),
    [sym_err_password_invalid] = ACTIONS(1835),
    [sym_err_invalid_account] = ACTIONS(1835),
    [sym_err_no_authorization] = ACTIONS(1835),
    [sym_err_invalid_parameter] = ACTIONS(1835),
    [sym_err_user_locked_out] = ACTIONS(1835),
    [sym_err_user_account_expired] = ACTIONS(1835),
    [sym_err_general_error] = ACTIONS(1835),
    [sym_err_authentication_error] = ACTIONS(1835),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1835),
    [sym_tls_validate_certificate] = ACTIONS(1835),
    [sym_tdata] = ACTIONS(1835),
    [sym_tchannel] = ACTIONS(1835),
    [sym_tlevel] = ACTIONS(1835),
    [sym_tbutton] = ACTIONS(1835),
    [sym_ttimeline] = ACTIONS(1835),
    [sym_tcustom] = ACTIONS(1835),
    [sym_url_struct] = ACTIONS(1835),
    [sym_dns_struct] = ACTIONS(1835),
    [sym_ip_address_struct] = ACTIONS(1835),
    [sym_dev_info_struct] = ACTIONS(1835),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1835),
    [sym_data_feed] = ACTIONS(1835),
    [sym_data_field] = ACTIONS(1835),
    [sym_data_record] = ACTIONS(1835),
    [sym_wc_data_feed] = ACTIONS(1835),
    [sym_wc_data_field] = ACTIONS(1835),
    [sym_wc_data_record] = ACTIONS(1835),
    [sym_last_login_info] = ACTIONS(1835),
    [sym____reserved___] = ACTIONS(1835),
    [sym_date] = ACTIONS(1835),
    [sym_day] = ACTIONS(1835),
    [sym_time] = ACTIONS(1835),
    [sym_ldate] = ACTIONS(1835),
    [sym_push_device] = ACTIONS(1835),
    [sym_push_channel] = ACTIONS(1835),
    [sym_push_devchan] = ACTIONS(1835),
    [sym_release_device] = ACTIONS(1835),
    [sym_release_channel] = ACTIONS(1835),
    [sym_release_devchan] = ACTIONS(1835),
    [sym_master_slot] = ACTIONS(1835),
    [sym_get_pulse_time] = ACTIONS(1835),
    [sym_get_timer] = ACTIONS(1835),
    [sym_master_sn] = ACTIONS(1835),
    [sym_system_number] = ACTIONS(1835),
    [sym_button] = ACTIONS(1835),
    [sym_dv_channel] = ACTIONS(1835),
    [sym_data] = ACTIONS(1835),
    [sym_level] = ACTIONS(1835),
    [sym_timeline] = ACTIONS(1835),
    [sym_channel] = ACTIONS(1835),
    [sym_custom] = ACTIONS(1835),
    [sym_atoi] = ACTIONS(1835),
    [sym_atol] = ACTIONS(1835),
    [sym_atof] = ACTIONS(1835),
    [sym_itoa] = ACTIONS(1835),
    [sym_itohex] = ACTIONS(1835),
    [sym_hextoi] = ACTIONS(1835),
    [sym_ftoa] = ACTIONS(1835),
    [sym_find_string] = ACTIONS(1835),
    [sym_left_string] = ACTIONS(1835),
    [sym_right_string] = ACTIONS(1835),
    [sym_mid_string] = ACTIONS(1835),
    [sym_remove_string] = ACTIONS(1835),
    [sym_get_buffer_string] = ACTIONS(1835),
    [sym_get_buffer_char] = ACTIONS(1835),
    [sym_get_multi_buffer_string] = ACTIONS(1835),
    [sym_length_string] = ACTIONS(1835),
    [sym_max_length_string] = ACTIONS(1835),
    [sym_set_length_string] = ACTIONS(1835),
    [sym_compare_string] = ACTIONS(1835),
    [sym_upper_string] = ACTIONS(1835),
    [sym_lower_string] = ACTIONS(1835),
    [sym_get_last] = ACTIONS(1835),
    [sym_date_to_day] = ACTIONS(1835),
    [sym_date_to_month] = ACTIONS(1835),
    [sym_date_to_year] = ACTIONS(1835),
    [sym_time_to_hour] = ACTIONS(1835),
    [sym_time_to_minute] = ACTIONS(1835),
    [sym_time_to_second] = ACTIONS(1835),
    [sym_day_of_week] = ACTIONS(1835),
    [sym_set_pulse_time] = ACTIONS(1835),
    [sym_set_timer] = ACTIONS(1835),
    [sym_file_open] = ACTIONS(1835),
    [sym_file_close] = ACTIONS(1835),
    [sym_file_read] = ACTIONS(1835),
    [sym_file_read_line] = ACTIONS(1835),
    [sym_file_write] = ACTIONS(1835),
    [sym_file_write_line] = ACTIONS(1835),
    [sym_file_copy] = ACTIONS(1835),
    [sym_file_delete] = ACTIONS(1835),
    [sym_file_rename] = ACTIONS(1835),
    [sym_file_dir] = ACTIONS(1835),
    [sym_file_getdir] = ACTIONS(1835),
    [sym_file_setdir] = ACTIONS(1835),
    [sym_file_create_dir] = ACTIONS(1835),
    [sym_file_remove_dir] = ACTIONS(1835),
    [sym_file_seek] = ACTIONS(1835),
    [sym_length_array] = ACTIONS(1835),
    [sym_max_length_array] = ACTIONS(1835),
    [sym_set_length_array] = ACTIONS(1835),
    [sym_ip_client_open] = ACTIONS(1835),
    [sym_ip_client_close] = ACTIONS(1835),
    [sym_ip_server_open] = ACTIONS(1835),
    [sym_ip_server_close] = ACTIONS(1835),
    [sym_ip_bound_client_open] = ACTIONS(1835),
    [sym_ip_mc_server_open] = ACTIONS(1835),
    [sym_ip_set_option] = ACTIONS(1835),
    [sym_ssh_client_open] = ACTIONS(1835),
    [sym_ssh_client_close] = ACTIONS(1835),
    [sym_tls_client_open] = ACTIONS(1835),
    [sym_tls_client_close] = ACTIONS(1835),
    [sym_do_push] = ACTIONS(1835),
    [sym_do_push_timed] = ACTIONS(1835),
    [sym_do_release] = ACTIONS(1835),
    [sym_device_id] = ACTIONS(1835),
    [sym_device_info] = ACTIONS(1835),
    [sym_device_id_string] = ACTIONS(1835),
    [sym_device_standby] = ACTIONS(1835),
    [sym_device_wake] = ACTIONS(1835),
    [sym_timeline_create] = ACTIONS(1835),
    [sym_timeline_kill] = ACTIONS(1835),
    [sym_timeline_reload] = ACTIONS(1835),
    [sym_timeline_pause] = ACTIONS(1835),
    [sym_timeline_restart] = ACTIONS(1835),
    [sym_timeline_set] = ACTIONS(1835),
    [sym_timeline_get] = ACTIONS(1835),
    [sym_timeline_active] = ACTIONS(1835),
    [sym_timeline_dynamic_id] = ACTIONS(1835),
    [sym_variable_to_string] = ACTIONS(1835),
    [sym_string_to_variable] = ACTIONS(1835),
    [sym_length_variable_to_string] = ACTIONS(1835),
    [sym_type_cast] = ACTIONS(1835),
    [sym_variable_to_xml] = ACTIONS(1835),
    [sym_xml_to_variable] = ACTIONS(1835),
    [sym_length_variable_to_xml] = ACTIONS(1835),
    [sym_redirect_string] = ACTIONS(1835),
    [sym_random_number] = ACTIONS(1835),
    [sym_get_unique_id] = ACTIONS(1835),
    [sym_get_system_number] = ACTIONS(1835),
    [sym_set_system_number] = ACTIONS(1835),
    [sym_get_serial_number] = ACTIONS(1835),
    [sym_get_master_build] = ACTIONS(1835),
    [sym_set_validation_code] = ACTIONS(1835),
    [sym_reboot] = ACTIONS(1835),
    [sym_load_duet_module] = ACTIONS(1835),
    [sym_unload_duet_module] = ACTIONS(1835),
    [sym_duet_mem_size_set] = ACTIONS(1835),
    [sym_duet_mem_size_get] = ACTIONS(1835),
    [sym_dynamic_application_device] = ACTIONS(1835),
    [sym_dynamic_polled_port] = ACTIONS(1835),
    [sym_static_port_binding] = ACTIONS(1835),
    [sym_static_ip_binding] = ACTIONS(1835),
    [sym_get_dns_list] = ACTIONS(1835),
    [sym_set_dns_list] = ACTIONS(1835),
    [sym_get_ip_address] = ACTIONS(1835),
    [sym_set_ip_address] = ACTIONS(1835),
    [sym_get_url_list] = ACTIONS(1835),
    [sym_add_url_entry] = ACTIONS(1835),
    [sym_delete_url_entry] = ACTIONS(1835),
    [sym_abs_value] = ACTIONS(1835),
    [sym_max_value] = ACTIONS(1835),
    [sym_min_value] = ACTIONS(1835),
    [sym_exp_value] = ACTIONS(1835),
    [sym_log_value] = ACTIONS(1835),
    [sym_log10_value] = ACTIONS(1835),
    [sym_power_value] = ACTIONS(1835),
    [sym_sqrt_value] = ACTIONS(1835),
    [sym_raw_be] = ACTIONS(1835),
    [sym_raw_le] = ACTIONS(1835),
    [sym_internal_queue_size_set] = ACTIONS(1835),
    [sym_internal_queue_size_get] = ACTIONS(1835),
    [sym_internal_threshold_set] = ACTIONS(1835),
    [sym_internal_threshold_get] = ACTIONS(1835),
    [sym_rebuilt_event] = ACTIONS(1835),
    [sym_do_custom_event] = ACTIONS(1835),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1835),
    [sym_clkmgr_set_clk_source] = ACTIONS(1835),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1835),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1835),
    [sym_clkmgr_get_timezone] = ACTIONS(1835),
    [sym_clkmgr_set_timezone] = ACTIONS(1835),
    [sym_clkmgr_get_resync_period] = ACTIONS(1835),
    [sym_clkmgr_set_resync_period] = ACTIONS(1835),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1835),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1835),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1835),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1835),
    [sym_clkmgr_get_timeservers] = ACTIONS(1835),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1835),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1835),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1835),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1835),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1835),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1835),
    [sym_set_log_level] = ACTIONS(1835),
    [sym_get_log_level] = ACTIONS(1835),
    [sym_amx_log] = ACTIONS(1835),
    [sym_smtp_server_config_set] = ACTIONS(1835),
    [sym_smtp_server_config_get] = ACTIONS(1835),
    [sym_smtp_send] = ACTIONS(1835),
    [sym_validate_netlinx_account] = ACTIONS(1835),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1835),
    [sym_authenticate_certificate] = ACTIONS(1835),
    [sym_audit_netlinx_session_event] = ACTIONS(1835),
    [sym_audit_netlinx_generic_event] = ACTIONS(1835),
    [sym_get_available_flash_disk_space] = ACTIONS(1835),
    [sym_get_max_flash_disk_space] = ACTIONS(1835),
    [sym_data_create_feed] = ACTIONS(1835),
    [sym_data_delete_feed] = ACTIONS(1835),
    [sym_data_publish_feed] = ACTIONS(1835),
    [sym_data_get_published_feed] = ACTIONS(1835),
    [sym_data_add_record] = ACTIONS(1835),
    [sym_data_get_event_record] = ACTIONS(1835),
    [sym__wc_data_create_feed] = ACTIONS(1835),
    [sym__wc_data_add_record] = ACTIONS(1835),
    [sym__wc_data_get_event_record] = ACTIONS(1835),
    [sym_format] = ACTIONS(1835),
    [sym_astro_clock] = ACTIONS(1835),
    [sym_set_outdoor_temperature] = ACTIONS(1835),
    [sym_set_virtual_level_count] = ACTIONS(1835),
    [sym_set_virtual_channel_count] = ACTIONS(1835),
    [sym_set_virtual_port_count] = ACTIONS(1835),
    [sym__wc_encode_int] = ACTIONS(1835),
    [sym__wc_decode_int] = ACTIONS(1835),
    [sym__wc_int] = ACTIONS(1835),
    [sym__wc_to_ch_int] = ACTIONS(1835),
    [sym__ch_to_wc_int] = ACTIONS(1835),
    [sym__wc_find_string_int] = ACTIONS(1835),
    [sym__wc_left_string_int] = ACTIONS(1835),
    [sym__wc_lower_string_int] = ACTIONS(1835),
    [sym__wc_mid_string_int] = ACTIONS(1835),
    [sym__wc_remove_string_int] = ACTIONS(1835),
    [sym__wc_right_string_int] = ACTIONS(1835),
    [sym__wc_upper_string_int] = ACTIONS(1835),
    [sym__wc_compare_string_int] = ACTIONS(1835),
    [sym__wc_get_buffer_char_int] = ACTIONS(1835),
    [sym__wc_get_buffer_string_int] = ACTIONS(1835),
    [sym__wc_concat_string_int] = ACTIONS(1835),
    [sym__wc_file_open_int] = ACTIONS(1835),
    [sym__wc_file_close_int] = ACTIONS(1835),
    [sym__wc_file_read_int] = ACTIONS(1835),
    [sym__wc_file_read_line_int] = ACTIONS(1835),
    [sym__wc_file_write_int] = ACTIONS(1835),
    [sym__wc_file_write_line_int] = ACTIONS(1835),
    [aux_sym_preproc_include_token1] = ACTIONS(1835),
    [aux_sym_preproc_define_token1] = ACTIONS(1837),
    [aux_sym_preproc_warn_token1] = ACTIONS(1837),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1837),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1837),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1837),
    [anon_sym_LBRACK] = ACTIONS(1837),
    [anon_sym_LBRACE] = ACTIONS(1837),
    [anon_sym_RBRACE] = ACTIONS(1837),
    [anon_sym_LPAREN] = ACTIONS(1835),
    [anon_sym_BANG] = ACTIONS(1837),
    [anon_sym_TILDE] = ACTIONS(1837),
    [anon_sym_DASH] = ACTIONS(1835),
    [anon_sym_PLUS] = ACTIONS(1835),
    [anon_sym_DASH_DASH] = ACTIONS(1837),
    [anon_sym_PLUS_PLUS] = ACTIONS(1837),
    [anon_sym_DQUOTE] = ACTIONS(1837),
    [anon_sym_SQUOTE] = ACTIONS(1837),
    [sym_number_literal] = ACTIONS(1837),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(477)] = {
    [sym_identifier] = ACTIONS(1871),
    [sym_define_call_keyword] = ACTIONS(1871),
    [sym_define_function_keyword] = ACTIONS(1871),
    [sym_define_library_function_keyword] = ACTIONS(1871),
    [sym_define_module_keyword] = ACTIONS(1871),
    [sym_char_keyword] = ACTIONS(1871),
    [sym_widechar_keyword] = ACTIONS(1871),
    [sym_integer_keyword] = ACTIONS(1871),
    [sym_sinteger_keyword] = ACTIONS(1871),
    [sym_long_keyword] = ACTIONS(1871),
    [sym_slong_keyword] = ACTIONS(1871),
    [sym_float_keyword] = ACTIONS(1871),
    [sym_double_keyword] = ACTIONS(1871),
    [sym_variant_keyword] = ACTIONS(1871),
    [sym_variantarray_keyword] = ACTIONS(1871),
    [sym_local_var_keyword] = ACTIONS(1871),
    [sym_stack_var_keyword] = ACTIONS(1871),
    [sym_constant_keyword] = ACTIONS(1871),
    [sym_volatile_keyword] = ACTIONS(1871),
    [sym_non_volatile_keyword] = ACTIONS(1871),
    [sym_persistent_keyword] = ACTIONS(1871),
    [sym_dev_keyword] = ACTIONS(1871),
    [sym_devlev_keyword] = ACTIONS(1871),
    [sym_devchan_keyword] = ACTIONS(1871),
    [sym_if_keyword] = ACTIONS(1871),
    [sym_else_keyword] = ACTIONS(1871),
    [sym_switch_keyword] = ACTIONS(1871),
    [sym_case_keyword] = ACTIONS(1871),
    [sym_default_keyword] = ACTIONS(1871),
    [sym_while_keyword] = ACTIONS(1871),
    [sym_for_keyword] = ACTIONS(1871),
    [sym_break_keyword] = ACTIONS(1871),
    [sym_continue_keyword] = ACTIONS(1871),
    [sym_return_keyword] = ACTIONS(1871),
    [sym_select_keyword] = ACTIONS(1871),
    [sym_active_keyword] = ACTIONS(1871),
    [sym_struct_keyword] = ACTIONS(1871),
    [sym_structure_keyword] = ACTIONS(1871),
    [aux_sym_on_keyword_token1] = ACTIONS(1871),
    [aux_sym_off_keyword_token1] = ACTIONS(1871),
    [sym_send_level_keyword] = ACTIONS(1871),
    [sym_send_string_keyword] = ACTIONS(1871),
    [sym_send_command_keyword] = ACTIONS(1871),
    [sym_clear_buffer_keyword] = ACTIONS(1871),
    [sym_create_buffer_keyword] = ACTIONS(1871),
    [sym_create_multi_buffer_keyword] = ACTIONS(1871),
    [sym_call_keyword] = ACTIONS(1871),
    [sym_system_call_keyword] = ACTIONS(1871),
    [sym_devchan_to_keyword] = ACTIONS(1871),
    [sym_devchan_min_to_keyword] = ACTIONS(1871),
    [sym_devchan_total_off_keyword] = ACTIONS(1871),
    [sym_devchan_pulse_keyword] = ACTIONS(1871),
    [sym_wait_keyword] = ACTIONS(1871),
    [sym_cancel_wait_keyword] = ACTIONS(1871),
    [sym_wait_until_keyword] = ACTIONS(1871),
    [sym_cancel_wait_until_keyword] = ACTIONS(1871),
    [sym_cancel_all_wait_keyword] = ACTIONS(1871),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1871),
    [sym_not] = ACTIONS(1871),
    [sym_bnot] = ACTIONS(1871),
    [sym___file__] = ACTIONS(1871),
    [sym___date__] = ACTIONS(1871),
    [sym___ldate__] = ACTIONS(1871),
    [sym___time__] = ACTIONS(1871),
    [sym___line__] = ACTIONS(1871),
    [sym___name__] = ACTIONS(1871),
    [sym___netlinx__] = ACTIONS(1871),
    [sym_netlinx_axi_version] = ACTIONS(1871),
    [sym_url_flg_tcp] = ACTIONS(1871),
    [sym_url_flg_accinfopresent] = ACTIONS(1871),
    [sym_url_flg_temp] = ACTIONS(1871),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1871),
    [sym_url_flg_stat_mask] = ACTIONS(1871),
    [sym_url_flg_stat_lookup] = ACTIONS(1871),
    [sym_url_flg_stat_connecting] = ACTIONS(1871),
    [sym_url_flg_stat_waiting] = ACTIONS(1871),
    [sym_url_flg_stat_connected] = ACTIONS(1871),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1871),
    [sym_first_virtual_device] = ACTIONS(1871),
    [sym_first_local_port] = ACTIONS(1871),
    [sym_dynamic_virtual_device] = ACTIONS(1871),
    [sym_timeline_once] = ACTIONS(1871),
    [sym_timeline_repeat] = ACTIONS(1871),
    [sym_timeline_absolute] = ACTIONS(1871),
    [sym_timeline_relative] = ACTIONS(1871),
    [sym_do_push_timed_infinite] = ACTIONS(1871),
    [sym_source_type_no_address] = ACTIONS(1871),
    [sym_source_type_neuron_id] = ACTIONS(1871),
    [sym_source_type_ip_address] = ACTIONS(1871),
    [sym_source_type_axlink] = ACTIONS(1871),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1871),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1871),
    [sym_source_type_ip_socket_address] = ACTIONS(1871),
    [sym_source_type_rs232] = ACTIONS(1871),
    [sym_source_type_internal] = ACTIONS(1871),
    [sym_source_type_mac_address] = ACTIONS(1871),
    [sym_source_type_ipv4_port] = ACTIONS(1871),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1871),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1871),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1871),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1871),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1871),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1871),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1871),
    [sym_true] = ACTIONS(1871),
    [sym_false] = ACTIONS(1871),
    [sym_file_read_only] = ACTIONS(1871),
    [sym_file_rw_new] = ACTIONS(1871),
    [sym_file_rw_append] = ACTIONS(1871),
    [sym_ip_tcp] = ACTIONS(1871),
    [sym_ip_udp] = ACTIONS(1871),
    [sym_ip_udp_2way] = ACTIONS(1871),
    [sym_xml_encode_types] = ACTIONS(1871),
    [sym_xml_encode_char_as_list] = ACTIONS(1871),
    [sym_xml_encode_le] = ACTIONS(1871),
    [sym_xml_decode_types] = ACTIONS(1871),
    [sym_xml_decode_no_preserve] = ACTIONS(1871),
    [sym_duet_dev_type_display_device] = ACTIONS(1871),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1871),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1871),
    [sym_duet_dev_type_io_device] = ACTIONS(1871),
    [sym_duet_dev_type_relay_device] = ACTIONS(1871),
    [sym_duet_dev_type_ups] = ACTIONS(1871),
    [sym_duet_dev_type_amplifier] = ACTIONS(1871),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1871),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1871),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1871),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1871),
    [sym_duet_dev_type_camera] = ACTIONS(1871),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1871),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1871),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1871),
    [sym_duet_dev_type_dss] = ACTIONS(1871),
    [sym_duet_dev_type_dvr] = ACTIONS(1871),
    [sym_duet_dev_type_disc_device] = ACTIONS(1871),
    [sym_duet_dev_type_document_camera] = ACTIONS(1871),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1871),
    [sym_duet_dev_type_hvac] = ACTIONS(1871),
    [sym_duet_dev_type_keypad] = ACTIONS(1871),
    [sym_duet_dev_type_light] = ACTIONS(1871),
    [sym_duet_dev_type_monitor] = ACTIONS(1871),
    [sym_duet_dev_type_motor] = ACTIONS(1871),
    [sym_duet_dev_type_multi_window] = ACTIONS(1871),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1871),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1871),
    [sym_duet_dev_type_receiver] = ACTIONS(1871),
    [sym_duet_dev_type_security_system] = ACTIONS(1871),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1871),
    [sym_duet_dev_type_settop_box] = ACTIONS(1871),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1871),
    [sym_duet_dev_type_switcher] = ACTIONS(1871),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1871),
    [sym_duet_dev_type_tv] = ACTIONS(1871),
    [sym_duet_dev_type_utility] = ACTIONS(1871),
    [sym_duet_dev_type_vcr] = ACTIONS(1871),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1871),
    [sym_duet_dev_type_video_processor] = ACTIONS(1871),
    [sym_duet_dev_type_video_projector] = ACTIONS(1871),
    [sym_duet_dev_type_video_wall] = ACTIONS(1871),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1871),
    [sym_duet_dev_type_weather] = ACTIONS(1871),
    [sym_duet_dev_not_polled] = ACTIONS(1871),
    [sym_duet_dev_polled] = ACTIONS(1871),
    [sym_clkmgr_mode_network] = ACTIONS(1871),
    [sym_clkmgr_mode_standalone] = ACTIONS(1871),
    [sym_normal_standby] = ACTIONS(1871),
    [sym_normal_wake] = ACTIONS(1871),
    [sym_ip_multicast_ttl_option] = ACTIONS(1871),
    [sym_ip_ttl_subnet] = ACTIONS(1871),
    [sym_ip_ttl_site] = ACTIONS(1871),
    [sym_ip_ttl_region] = ACTIONS(1871),
    [sym_ip_ttl_continent] = ACTIONS(1871),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1871),
    [sym_ip_nodelay_on] = ACTIONS(1871),
    [sym_ip_nodelay_off] = ACTIONS(1871),
    [sym_amx_error] = ACTIONS(1871),
    [sym_amx_warning] = ACTIONS(1871),
    [sym_amx_info] = ACTIONS(1871),
    [sym_amx_debug] = ACTIONS(1871),
    [sym_netlinx_login_success] = ACTIONS(1871),
    [sym_netlinx_login_fail] = ACTIONS(1871),
    [sym_netlinx_logout] = ACTIONS(1871),
    [sym_null_str] = ACTIONS(1871),
    [sym_smtp_address] = ACTIONS(1871),
    [sym_smtp_port_number] = ACTIONS(1871),
    [sym_smtp_username] = ACTIONS(1871),
    [sym_smtp_password] = ACTIONS(1871),
    [sym_smtp_require_tls] = ACTIONS(1871),
    [sym_smtp_from] = ACTIONS(1871),
    [sym_smtp_tls_true] = ACTIONS(1871),
    [sym_smtp_tls_false] = ACTIONS(1871),
    [sym_listview_on_row_select_event] = ACTIONS(1871),
    [sym_data_structure_unknown] = ACTIONS(1871),
    [sym_data_structure_datafeed] = ACTIONS(1871),
    [sym_data_structure_datarecordset] = ACTIONS(1871),
    [sym_data_structure_datarecord] = ACTIONS(1871),
    [sym_data_structure_datafield] = ACTIONS(1871),
    [sym_data_max_content_field_count] = ACTIONS(1871),
    [sym_data_max_metadata_field_count] = ACTIONS(1871),
    [sym_data_max_id_length] = ACTIONS(1871),
    [sym_data_max_name_length] = ACTIONS(1871),
    [sym_data_max_description_length] = ACTIONS(1871),
    [sym_data_max_type_length] = ACTIONS(1871),
    [sym_data_max_format_length] = ACTIONS(1871),
    [sym_data_max_source_length] = ACTIONS(1871),
    [sym_data_max_value_length] = ACTIONS(1871),
    [sym_data_max_label_length] = ACTIONS(1871),
    [sym_data_max_url_length] = ACTIONS(1871),
    [sym_max_last_login_info_length] = ACTIONS(1871),
    [sym_data_type_unknown] = ACTIONS(1871),
    [sym_data_type_string] = ACTIONS(1871),
    [sym_data_type_datetime] = ACTIONS(1871),
    [sym_data_type_date] = ACTIONS(1871),
    [sym_data_type_time] = ACTIONS(1871),
    [sym_data_type_image] = ACTIONS(1871),
    [sym_data_format_url] = ACTIONS(1871),
    [sym_data_format_phone] = ACTIONS(1871),
    [sym_data_format_email] = ACTIONS(1871),
    [sym_data_format_iso8601] = ACTIONS(1871),
    [sym_auditlog_priv] = ACTIONS(1871),
    [sym_configuration_priv] = ACTIONS(1871),
    [sym_deviceconfig_priv] = ACTIONS(1871),
    [sym_ftp_priv] = ACTIONS(1871),
    [sym_http_priv] = ACTIONS(1871),
    [sym_networkconfig_priv] = ACTIONS(1871),
    [sym_programport_priv] = ACTIONS(1871),
    [sym_remoteui_priv] = ACTIONS(1871),
    [sym_securitycontrol_priv] = ACTIONS(1871),
    [sym_softwaremanagement_priv] = ACTIONS(1871),
    [sym_terminal_priv] = ACTIONS(1871),
    [sym_tpadmin_priv] = ACTIONS(1871),
    [sym_user1_priv] = ACTIONS(1871),
    [sym_user2_priv] = ACTIONS(1871),
    [sym_user3_priv] = ACTIONS(1871),
    [sym_user4_priv] = ACTIONS(1871),
    [sym_usermanagement_priv] = ACTIONS(1871),
    [sym_valid_account] = ACTIONS(1871),
    [sym_err_name_invalid] = ACTIONS(1871),
    [sym_err_password_invalid] = ACTIONS(1871),
    [sym_err_invalid_account] = ACTIONS(1871),
    [sym_err_no_authorization] = ACTIONS(1871),
    [sym_err_invalid_parameter] = ACTIONS(1871),
    [sym_err_user_locked_out] = ACTIONS(1871),
    [sym_err_user_account_expired] = ACTIONS(1871),
    [sym_err_general_error] = ACTIONS(1871),
    [sym_err_authentication_error] = ACTIONS(1871),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1871),
    [sym_tls_validate_certificate] = ACTIONS(1871),
    [sym_tdata] = ACTIONS(1871),
    [sym_tchannel] = ACTIONS(1871),
    [sym_tlevel] = ACTIONS(1871),
    [sym_tbutton] = ACTIONS(1871),
    [sym_ttimeline] = ACTIONS(1871),
    [sym_tcustom] = ACTIONS(1871),
    [sym_url_struct] = ACTIONS(1871),
    [sym_dns_struct] = ACTIONS(1871),
    [sym_ip_address_struct] = ACTIONS(1871),
    [sym_dev_info_struct] = ACTIONS(1871),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1871),
    [sym_data_feed] = ACTIONS(1871),
    [sym_data_field] = ACTIONS(1871),
    [sym_data_record] = ACTIONS(1871),
    [sym_wc_data_feed] = ACTIONS(1871),
    [sym_wc_data_field] = ACTIONS(1871),
    [sym_wc_data_record] = ACTIONS(1871),
    [sym_last_login_info] = ACTIONS(1871),
    [sym____reserved___] = ACTIONS(1871),
    [sym_date] = ACTIONS(1871),
    [sym_day] = ACTIONS(1871),
    [sym_time] = ACTIONS(1871),
    [sym_ldate] = ACTIONS(1871),
    [sym_push_device] = ACTIONS(1871),
    [sym_push_channel] = ACTIONS(1871),
    [sym_push_devchan] = ACTIONS(1871),
    [sym_release_device] = ACTIONS(1871),
    [sym_release_channel] = ACTIONS(1871),
    [sym_release_devchan] = ACTIONS(1871),
    [sym_master_slot] = ACTIONS(1871),
    [sym_get_pulse_time] = ACTIONS(1871),
    [sym_get_timer] = ACTIONS(1871),
    [sym_master_sn] = ACTIONS(1871),
    [sym_system_number] = ACTIONS(1871),
    [sym_button] = ACTIONS(1871),
    [sym_dv_channel] = ACTIONS(1871),
    [sym_data] = ACTIONS(1871),
    [sym_level] = ACTIONS(1871),
    [sym_timeline] = ACTIONS(1871),
    [sym_channel] = ACTIONS(1871),
    [sym_custom] = ACTIONS(1871),
    [sym_atoi] = ACTIONS(1871),
    [sym_atol] = ACTIONS(1871),
    [sym_atof] = ACTIONS(1871),
    [sym_itoa] = ACTIONS(1871),
    [sym_itohex] = ACTIONS(1871),
    [sym_hextoi] = ACTIONS(1871),
    [sym_ftoa] = ACTIONS(1871),
    [sym_find_string] = ACTIONS(1871),
    [sym_left_string] = ACTIONS(1871),
    [sym_right_string] = ACTIONS(1871),
    [sym_mid_string] = ACTIONS(1871),
    [sym_remove_string] = ACTIONS(1871),
    [sym_get_buffer_string] = ACTIONS(1871),
    [sym_get_buffer_char] = ACTIONS(1871),
    [sym_get_multi_buffer_string] = ACTIONS(1871),
    [sym_length_string] = ACTIONS(1871),
    [sym_max_length_string] = ACTIONS(1871),
    [sym_set_length_string] = ACTIONS(1871),
    [sym_compare_string] = ACTIONS(1871),
    [sym_upper_string] = ACTIONS(1871),
    [sym_lower_string] = ACTIONS(1871),
    [sym_get_last] = ACTIONS(1871),
    [sym_date_to_day] = ACTIONS(1871),
    [sym_date_to_month] = ACTIONS(1871),
    [sym_date_to_year] = ACTIONS(1871),
    [sym_time_to_hour] = ACTIONS(1871),
    [sym_time_to_minute] = ACTIONS(1871),
    [sym_time_to_second] = ACTIONS(1871),
    [sym_day_of_week] = ACTIONS(1871),
    [sym_set_pulse_time] = ACTIONS(1871),
    [sym_set_timer] = ACTIONS(1871),
    [sym_file_open] = ACTIONS(1871),
    [sym_file_close] = ACTIONS(1871),
    [sym_file_read] = ACTIONS(1871),
    [sym_file_read_line] = ACTIONS(1871),
    [sym_file_write] = ACTIONS(1871),
    [sym_file_write_line] = ACTIONS(1871),
    [sym_file_copy] = ACTIONS(1871),
    [sym_file_delete] = ACTIONS(1871),
    [sym_file_rename] = ACTIONS(1871),
    [sym_file_dir] = ACTIONS(1871),
    [sym_file_getdir] = ACTIONS(1871),
    [sym_file_setdir] = ACTIONS(1871),
    [sym_file_create_dir] = ACTIONS(1871),
    [sym_file_remove_dir] = ACTIONS(1871),
    [sym_file_seek] = ACTIONS(1871),
    [sym_length_array] = ACTIONS(1871),
    [sym_max_length_array] = ACTIONS(1871),
    [sym_set_length_array] = ACTIONS(1871),
    [sym_ip_client_open] = ACTIONS(1871),
    [sym_ip_client_close] = ACTIONS(1871),
    [sym_ip_server_open] = ACTIONS(1871),
    [sym_ip_server_close] = ACTIONS(1871),
    [sym_ip_bound_client_open] = ACTIONS(1871),
    [sym_ip_mc_server_open] = ACTIONS(1871),
    [sym_ip_set_option] = ACTIONS(1871),
    [sym_ssh_client_open] = ACTIONS(1871),
    [sym_ssh_client_close] = ACTIONS(1871),
    [sym_tls_client_open] = ACTIONS(1871),
    [sym_tls_client_close] = ACTIONS(1871),
    [sym_do_push] = ACTIONS(1871),
    [sym_do_push_timed] = ACTIONS(1871),
    [sym_do_release] = ACTIONS(1871),
    [sym_device_id] = ACTIONS(1871),
    [sym_device_info] = ACTIONS(1871),
    [sym_device_id_string] = ACTIONS(1871),
    [sym_device_standby] = ACTIONS(1871),
    [sym_device_wake] = ACTIONS(1871),
    [sym_timeline_create] = ACTIONS(1871),
    [sym_timeline_kill] = ACTIONS(1871),
    [sym_timeline_reload] = ACTIONS(1871),
    [sym_timeline_pause] = ACTIONS(1871),
    [sym_timeline_restart] = ACTIONS(1871),
    [sym_timeline_set] = ACTIONS(1871),
    [sym_timeline_get] = ACTIONS(1871),
    [sym_timeline_active] = ACTIONS(1871),
    [sym_timeline_dynamic_id] = ACTIONS(1871),
    [sym_variable_to_string] = ACTIONS(1871),
    [sym_string_to_variable] = ACTIONS(1871),
    [sym_length_variable_to_string] = ACTIONS(1871),
    [sym_type_cast] = ACTIONS(1871),
    [sym_variable_to_xml] = ACTIONS(1871),
    [sym_xml_to_variable] = ACTIONS(1871),
    [sym_length_variable_to_xml] = ACTIONS(1871),
    [sym_redirect_string] = ACTIONS(1871),
    [sym_random_number] = ACTIONS(1871),
    [sym_get_unique_id] = ACTIONS(1871),
    [sym_get_system_number] = ACTIONS(1871),
    [sym_set_system_number] = ACTIONS(1871),
    [sym_get_serial_number] = ACTIONS(1871),
    [sym_get_master_build] = ACTIONS(1871),
    [sym_set_validation_code] = ACTIONS(1871),
    [sym_reboot] = ACTIONS(1871),
    [sym_load_duet_module] = ACTIONS(1871),
    [sym_unload_duet_module] = ACTIONS(1871),
    [sym_duet_mem_size_set] = ACTIONS(1871),
    [sym_duet_mem_size_get] = ACTIONS(1871),
    [sym_dynamic_application_device] = ACTIONS(1871),
    [sym_dynamic_polled_port] = ACTIONS(1871),
    [sym_static_port_binding] = ACTIONS(1871),
    [sym_static_ip_binding] = ACTIONS(1871),
    [sym_get_dns_list] = ACTIONS(1871),
    [sym_set_dns_list] = ACTIONS(1871),
    [sym_get_ip_address] = ACTIONS(1871),
    [sym_set_ip_address] = ACTIONS(1871),
    [sym_get_url_list] = ACTIONS(1871),
    [sym_add_url_entry] = ACTIONS(1871),
    [sym_delete_url_entry] = ACTIONS(1871),
    [sym_abs_value] = ACTIONS(1871),
    [sym_max_value] = ACTIONS(1871),
    [sym_min_value] = ACTIONS(1871),
    [sym_exp_value] = ACTIONS(1871),
    [sym_log_value] = ACTIONS(1871),
    [sym_log10_value] = ACTIONS(1871),
    [sym_power_value] = ACTIONS(1871),
    [sym_sqrt_value] = ACTIONS(1871),
    [sym_raw_be] = ACTIONS(1871),
    [sym_raw_le] = ACTIONS(1871),
    [sym_internal_queue_size_set] = ACTIONS(1871),
    [sym_internal_queue_size_get] = ACTIONS(1871),
    [sym_internal_threshold_set] = ACTIONS(1871),
    [sym_internal_threshold_get] = ACTIONS(1871),
    [sym_rebuilt_event] = ACTIONS(1871),
    [sym_do_custom_event] = ACTIONS(1871),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1871),
    [sym_clkmgr_set_clk_source] = ACTIONS(1871),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1871),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1871),
    [sym_clkmgr_get_timezone] = ACTIONS(1871),
    [sym_clkmgr_set_timezone] = ACTIONS(1871),
    [sym_clkmgr_get_resync_period] = ACTIONS(1871),
    [sym_clkmgr_set_resync_period] = ACTIONS(1871),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1871),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1871),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1871),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1871),
    [sym_clkmgr_get_timeservers] = ACTIONS(1871),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1871),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1871),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1871),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1871),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1871),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1871),
    [sym_set_log_level] = ACTIONS(1871),
    [sym_get_log_level] = ACTIONS(1871),
    [sym_amx_log] = ACTIONS(1871),
    [sym_smtp_server_config_set] = ACTIONS(1871),
    [sym_smtp_server_config_get] = ACTIONS(1871),
    [sym_smtp_send] = ACTIONS(1871),
    [sym_validate_netlinx_account] = ACTIONS(1871),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1871),
    [sym_authenticate_certificate] = ACTIONS(1871),
    [sym_audit_netlinx_session_event] = ACTIONS(1871),
    [sym_audit_netlinx_generic_event] = ACTIONS(1871),
    [sym_get_available_flash_disk_space] = ACTIONS(1871),
    [sym_get_max_flash_disk_space] = ACTIONS(1871),
    [sym_data_create_feed] = ACTIONS(1871),
    [sym_data_delete_feed] = ACTIONS(1871),
    [sym_data_publish_feed] = ACTIONS(1871),
    [sym_data_get_published_feed] = ACTIONS(1871),
    [sym_data_add_record] = ACTIONS(1871),
    [sym_data_get_event_record] = ACTIONS(1871),
    [sym__wc_data_create_feed] = ACTIONS(1871),
    [sym__wc_data_add_record] = ACTIONS(1871),
    [sym__wc_data_get_event_record] = ACTIONS(1871),
    [sym_format] = ACTIONS(1871),
    [sym_astro_clock] = ACTIONS(1871),
    [sym_set_outdoor_temperature] = ACTIONS(1871),
    [sym_set_virtual_level_count] = ACTIONS(1871),
    [sym_set_virtual_channel_count] = ACTIONS(1871),
    [sym_set_virtual_port_count] = ACTIONS(1871),
    [sym__wc_encode_int] = ACTIONS(1871),
    [sym__wc_decode_int] = ACTIONS(1871),
    [sym__wc_int] = ACTIONS(1871),
    [sym__wc_to_ch_int] = ACTIONS(1871),
    [sym__ch_to_wc_int] = ACTIONS(1871),
    [sym__wc_find_string_int] = ACTIONS(1871),
    [sym__wc_left_string_int] = ACTIONS(1871),
    [sym__wc_lower_string_int] = ACTIONS(1871),
    [sym__wc_mid_string_int] = ACTIONS(1871),
    [sym__wc_remove_string_int] = ACTIONS(1871),
    [sym__wc_right_string_int] = ACTIONS(1871),
    [sym__wc_upper_string_int] = ACTIONS(1871),
    [sym__wc_compare_string_int] = ACTIONS(1871),
    [sym__wc_get_buffer_char_int] = ACTIONS(1871),
    [sym__wc_get_buffer_string_int] = ACTIONS(1871),
    [sym__wc_concat_string_int] = ACTIONS(1871),
    [sym__wc_file_open_int] = ACTIONS(1871),
    [sym__wc_file_close_int] = ACTIONS(1871),
    [sym__wc_file_read_int] = ACTIONS(1871),
    [sym__wc_file_read_line_int] = ACTIONS(1871),
    [sym__wc_file_write_int] = ACTIONS(1871),
    [sym__wc_file_write_line_int] = ACTIONS(1871),
    [aux_sym_preproc_include_token1] = ACTIONS(1871),
    [aux_sym_preproc_define_token1] = ACTIONS(1873),
    [aux_sym_preproc_warn_token1] = ACTIONS(1873),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1873),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1873),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1873),
    [anon_sym_LBRACK] = ACTIONS(1873),
    [anon_sym_LBRACE] = ACTIONS(1873),
    [anon_sym_RBRACE] = ACTIONS(1873),
    [anon_sym_LPAREN] = ACTIONS(1871),
    [anon_sym_BANG] = ACTIONS(1873),
    [anon_sym_TILDE] = ACTIONS(1873),
    [anon_sym_DASH] = ACTIONS(1871),
    [anon_sym_PLUS] = ACTIONS(1871),
    [anon_sym_DASH_DASH] = ACTIONS(1873),
    [anon_sym_PLUS_PLUS] = ACTIONS(1873),
    [anon_sym_DQUOTE] = ACTIONS(1873),
    [anon_sym_SQUOTE] = ACTIONS(1873),
    [sym_number_literal] = ACTIONS(1873),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(478)] = {
    [sym_identifier] = ACTIONS(1875),
    [sym_define_call_keyword] = ACTIONS(1875),
    [sym_define_function_keyword] = ACTIONS(1875),
    [sym_define_library_function_keyword] = ACTIONS(1875),
    [sym_define_module_keyword] = ACTIONS(1875),
    [sym_char_keyword] = ACTIONS(1875),
    [sym_widechar_keyword] = ACTIONS(1875),
    [sym_integer_keyword] = ACTIONS(1875),
    [sym_sinteger_keyword] = ACTIONS(1875),
    [sym_long_keyword] = ACTIONS(1875),
    [sym_slong_keyword] = ACTIONS(1875),
    [sym_float_keyword] = ACTIONS(1875),
    [sym_double_keyword] = ACTIONS(1875),
    [sym_variant_keyword] = ACTIONS(1875),
    [sym_variantarray_keyword] = ACTIONS(1875),
    [sym_local_var_keyword] = ACTIONS(1875),
    [sym_stack_var_keyword] = ACTIONS(1875),
    [sym_constant_keyword] = ACTIONS(1875),
    [sym_volatile_keyword] = ACTIONS(1875),
    [sym_non_volatile_keyword] = ACTIONS(1875),
    [sym_persistent_keyword] = ACTIONS(1875),
    [sym_dev_keyword] = ACTIONS(1875),
    [sym_devlev_keyword] = ACTIONS(1875),
    [sym_devchan_keyword] = ACTIONS(1875),
    [sym_if_keyword] = ACTIONS(1875),
    [sym_else_keyword] = ACTIONS(1875),
    [sym_switch_keyword] = ACTIONS(1875),
    [sym_case_keyword] = ACTIONS(1875),
    [sym_default_keyword] = ACTIONS(1875),
    [sym_while_keyword] = ACTIONS(1875),
    [sym_for_keyword] = ACTIONS(1875),
    [sym_break_keyword] = ACTIONS(1875),
    [sym_continue_keyword] = ACTIONS(1875),
    [sym_return_keyword] = ACTIONS(1875),
    [sym_select_keyword] = ACTIONS(1875),
    [sym_active_keyword] = ACTIONS(1875),
    [sym_struct_keyword] = ACTIONS(1875),
    [sym_structure_keyword] = ACTIONS(1875),
    [aux_sym_on_keyword_token1] = ACTIONS(1875),
    [aux_sym_off_keyword_token1] = ACTIONS(1875),
    [sym_send_level_keyword] = ACTIONS(1875),
    [sym_send_string_keyword] = ACTIONS(1875),
    [sym_send_command_keyword] = ACTIONS(1875),
    [sym_clear_buffer_keyword] = ACTIONS(1875),
    [sym_create_buffer_keyword] = ACTIONS(1875),
    [sym_create_multi_buffer_keyword] = ACTIONS(1875),
    [sym_call_keyword] = ACTIONS(1875),
    [sym_system_call_keyword] = ACTIONS(1875),
    [sym_devchan_to_keyword] = ACTIONS(1875),
    [sym_devchan_min_to_keyword] = ACTIONS(1875),
    [sym_devchan_total_off_keyword] = ACTIONS(1875),
    [sym_devchan_pulse_keyword] = ACTIONS(1875),
    [sym_wait_keyword] = ACTIONS(1875),
    [sym_cancel_wait_keyword] = ACTIONS(1875),
    [sym_wait_until_keyword] = ACTIONS(1875),
    [sym_cancel_wait_until_keyword] = ACTIONS(1875),
    [sym_cancel_all_wait_keyword] = ACTIONS(1875),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1875),
    [sym_not] = ACTIONS(1875),
    [sym_bnot] = ACTIONS(1875),
    [sym___file__] = ACTIONS(1875),
    [sym___date__] = ACTIONS(1875),
    [sym___ldate__] = ACTIONS(1875),
    [sym___time__] = ACTIONS(1875),
    [sym___line__] = ACTIONS(1875),
    [sym___name__] = ACTIONS(1875),
    [sym___netlinx__] = ACTIONS(1875),
    [sym_netlinx_axi_version] = ACTIONS(1875),
    [sym_url_flg_tcp] = ACTIONS(1875),
    [sym_url_flg_accinfopresent] = ACTIONS(1875),
    [sym_url_flg_temp] = ACTIONS(1875),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1875),
    [sym_url_flg_stat_mask] = ACTIONS(1875),
    [sym_url_flg_stat_lookup] = ACTIONS(1875),
    [sym_url_flg_stat_connecting] = ACTIONS(1875),
    [sym_url_flg_stat_waiting] = ACTIONS(1875),
    [sym_url_flg_stat_connected] = ACTIONS(1875),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1875),
    [sym_first_virtual_device] = ACTIONS(1875),
    [sym_first_local_port] = ACTIONS(1875),
    [sym_dynamic_virtual_device] = ACTIONS(1875),
    [sym_timeline_once] = ACTIONS(1875),
    [sym_timeline_repeat] = ACTIONS(1875),
    [sym_timeline_absolute] = ACTIONS(1875),
    [sym_timeline_relative] = ACTIONS(1875),
    [sym_do_push_timed_infinite] = ACTIONS(1875),
    [sym_source_type_no_address] = ACTIONS(1875),
    [sym_source_type_neuron_id] = ACTIONS(1875),
    [sym_source_type_ip_address] = ACTIONS(1875),
    [sym_source_type_axlink] = ACTIONS(1875),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1875),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1875),
    [sym_source_type_ip_socket_address] = ACTIONS(1875),
    [sym_source_type_rs232] = ACTIONS(1875),
    [sym_source_type_internal] = ACTIONS(1875),
    [sym_source_type_mac_address] = ACTIONS(1875),
    [sym_source_type_ipv4_port] = ACTIONS(1875),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1875),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1875),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1875),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1875),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1875),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1875),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1875),
    [sym_true] = ACTIONS(1875),
    [sym_false] = ACTIONS(1875),
    [sym_file_read_only] = ACTIONS(1875),
    [sym_file_rw_new] = ACTIONS(1875),
    [sym_file_rw_append] = ACTIONS(1875),
    [sym_ip_tcp] = ACTIONS(1875),
    [sym_ip_udp] = ACTIONS(1875),
    [sym_ip_udp_2way] = ACTIONS(1875),
    [sym_xml_encode_types] = ACTIONS(1875),
    [sym_xml_encode_char_as_list] = ACTIONS(1875),
    [sym_xml_encode_le] = ACTIONS(1875),
    [sym_xml_decode_types] = ACTIONS(1875),
    [sym_xml_decode_no_preserve] = ACTIONS(1875),
    [sym_duet_dev_type_display_device] = ACTIONS(1875),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1875),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1875),
    [sym_duet_dev_type_io_device] = ACTIONS(1875),
    [sym_duet_dev_type_relay_device] = ACTIONS(1875),
    [sym_duet_dev_type_ups] = ACTIONS(1875),
    [sym_duet_dev_type_amplifier] = ACTIONS(1875),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1875),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1875),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1875),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1875),
    [sym_duet_dev_type_camera] = ACTIONS(1875),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1875),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1875),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1875),
    [sym_duet_dev_type_dss] = ACTIONS(1875),
    [sym_duet_dev_type_dvr] = ACTIONS(1875),
    [sym_duet_dev_type_disc_device] = ACTIONS(1875),
    [sym_duet_dev_type_document_camera] = ACTIONS(1875),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1875),
    [sym_duet_dev_type_hvac] = ACTIONS(1875),
    [sym_duet_dev_type_keypad] = ACTIONS(1875),
    [sym_duet_dev_type_light] = ACTIONS(1875),
    [sym_duet_dev_type_monitor] = ACTIONS(1875),
    [sym_duet_dev_type_motor] = ACTIONS(1875),
    [sym_duet_dev_type_multi_window] = ACTIONS(1875),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1875),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1875),
    [sym_duet_dev_type_receiver] = ACTIONS(1875),
    [sym_duet_dev_type_security_system] = ACTIONS(1875),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1875),
    [sym_duet_dev_type_settop_box] = ACTIONS(1875),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1875),
    [sym_duet_dev_type_switcher] = ACTIONS(1875),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1875),
    [sym_duet_dev_type_tv] = ACTIONS(1875),
    [sym_duet_dev_type_utility] = ACTIONS(1875),
    [sym_duet_dev_type_vcr] = ACTIONS(1875),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1875),
    [sym_duet_dev_type_video_processor] = ACTIONS(1875),
    [sym_duet_dev_type_video_projector] = ACTIONS(1875),
    [sym_duet_dev_type_video_wall] = ACTIONS(1875),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1875),
    [sym_duet_dev_type_weather] = ACTIONS(1875),
    [sym_duet_dev_not_polled] = ACTIONS(1875),
    [sym_duet_dev_polled] = ACTIONS(1875),
    [sym_clkmgr_mode_network] = ACTIONS(1875),
    [sym_clkmgr_mode_standalone] = ACTIONS(1875),
    [sym_normal_standby] = ACTIONS(1875),
    [sym_normal_wake] = ACTIONS(1875),
    [sym_ip_multicast_ttl_option] = ACTIONS(1875),
    [sym_ip_ttl_subnet] = ACTIONS(1875),
    [sym_ip_ttl_site] = ACTIONS(1875),
    [sym_ip_ttl_region] = ACTIONS(1875),
    [sym_ip_ttl_continent] = ACTIONS(1875),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1875),
    [sym_ip_nodelay_on] = ACTIONS(1875),
    [sym_ip_nodelay_off] = ACTIONS(1875),
    [sym_amx_error] = ACTIONS(1875),
    [sym_amx_warning] = ACTIONS(1875),
    [sym_amx_info] = ACTIONS(1875),
    [sym_amx_debug] = ACTIONS(1875),
    [sym_netlinx_login_success] = ACTIONS(1875),
    [sym_netlinx_login_fail] = ACTIONS(1875),
    [sym_netlinx_logout] = ACTIONS(1875),
    [sym_null_str] = ACTIONS(1875),
    [sym_smtp_address] = ACTIONS(1875),
    [sym_smtp_port_number] = ACTIONS(1875),
    [sym_smtp_username] = ACTIONS(1875),
    [sym_smtp_password] = ACTIONS(1875),
    [sym_smtp_require_tls] = ACTIONS(1875),
    [sym_smtp_from] = ACTIONS(1875),
    [sym_smtp_tls_true] = ACTIONS(1875),
    [sym_smtp_tls_false] = ACTIONS(1875),
    [sym_listview_on_row_select_event] = ACTIONS(1875),
    [sym_data_structure_unknown] = ACTIONS(1875),
    [sym_data_structure_datafeed] = ACTIONS(1875),
    [sym_data_structure_datarecordset] = ACTIONS(1875),
    [sym_data_structure_datarecord] = ACTIONS(1875),
    [sym_data_structure_datafield] = ACTIONS(1875),
    [sym_data_max_content_field_count] = ACTIONS(1875),
    [sym_data_max_metadata_field_count] = ACTIONS(1875),
    [sym_data_max_id_length] = ACTIONS(1875),
    [sym_data_max_name_length] = ACTIONS(1875),
    [sym_data_max_description_length] = ACTIONS(1875),
    [sym_data_max_type_length] = ACTIONS(1875),
    [sym_data_max_format_length] = ACTIONS(1875),
    [sym_data_max_source_length] = ACTIONS(1875),
    [sym_data_max_value_length] = ACTIONS(1875),
    [sym_data_max_label_length] = ACTIONS(1875),
    [sym_data_max_url_length] = ACTIONS(1875),
    [sym_max_last_login_info_length] = ACTIONS(1875),
    [sym_data_type_unknown] = ACTIONS(1875),
    [sym_data_type_string] = ACTIONS(1875),
    [sym_data_type_datetime] = ACTIONS(1875),
    [sym_data_type_date] = ACTIONS(1875),
    [sym_data_type_time] = ACTIONS(1875),
    [sym_data_type_image] = ACTIONS(1875),
    [sym_data_format_url] = ACTIONS(1875),
    [sym_data_format_phone] = ACTIONS(1875),
    [sym_data_format_email] = ACTIONS(1875),
    [sym_data_format_iso8601] = ACTIONS(1875),
    [sym_auditlog_priv] = ACTIONS(1875),
    [sym_configuration_priv] = ACTIONS(1875),
    [sym_deviceconfig_priv] = ACTIONS(1875),
    [sym_ftp_priv] = ACTIONS(1875),
    [sym_http_priv] = ACTIONS(1875),
    [sym_networkconfig_priv] = ACTIONS(1875),
    [sym_programport_priv] = ACTIONS(1875),
    [sym_remoteui_priv] = ACTIONS(1875),
    [sym_securitycontrol_priv] = ACTIONS(1875),
    [sym_softwaremanagement_priv] = ACTIONS(1875),
    [sym_terminal_priv] = ACTIONS(1875),
    [sym_tpadmin_priv] = ACTIONS(1875),
    [sym_user1_priv] = ACTIONS(1875),
    [sym_user2_priv] = ACTIONS(1875),
    [sym_user3_priv] = ACTIONS(1875),
    [sym_user4_priv] = ACTIONS(1875),
    [sym_usermanagement_priv] = ACTIONS(1875),
    [sym_valid_account] = ACTIONS(1875),
    [sym_err_name_invalid] = ACTIONS(1875),
    [sym_err_password_invalid] = ACTIONS(1875),
    [sym_err_invalid_account] = ACTIONS(1875),
    [sym_err_no_authorization] = ACTIONS(1875),
    [sym_err_invalid_parameter] = ACTIONS(1875),
    [sym_err_user_locked_out] = ACTIONS(1875),
    [sym_err_user_account_expired] = ACTIONS(1875),
    [sym_err_general_error] = ACTIONS(1875),
    [sym_err_authentication_error] = ACTIONS(1875),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1875),
    [sym_tls_validate_certificate] = ACTIONS(1875),
    [sym_tdata] = ACTIONS(1875),
    [sym_tchannel] = ACTIONS(1875),
    [sym_tlevel] = ACTIONS(1875),
    [sym_tbutton] = ACTIONS(1875),
    [sym_ttimeline] = ACTIONS(1875),
    [sym_tcustom] = ACTIONS(1875),
    [sym_url_struct] = ACTIONS(1875),
    [sym_dns_struct] = ACTIONS(1875),
    [sym_ip_address_struct] = ACTIONS(1875),
    [sym_dev_info_struct] = ACTIONS(1875),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1875),
    [sym_data_feed] = ACTIONS(1875),
    [sym_data_field] = ACTIONS(1875),
    [sym_data_record] = ACTIONS(1875),
    [sym_wc_data_feed] = ACTIONS(1875),
    [sym_wc_data_field] = ACTIONS(1875),
    [sym_wc_data_record] = ACTIONS(1875),
    [sym_last_login_info] = ACTIONS(1875),
    [sym____reserved___] = ACTIONS(1875),
    [sym_date] = ACTIONS(1875),
    [sym_day] = ACTIONS(1875),
    [sym_time] = ACTIONS(1875),
    [sym_ldate] = ACTIONS(1875),
    [sym_push_device] = ACTIONS(1875),
    [sym_push_channel] = ACTIONS(1875),
    [sym_push_devchan] = ACTIONS(1875),
    [sym_release_device] = ACTIONS(1875),
    [sym_release_channel] = ACTIONS(1875),
    [sym_release_devchan] = ACTIONS(1875),
    [sym_master_slot] = ACTIONS(1875),
    [sym_get_pulse_time] = ACTIONS(1875),
    [sym_get_timer] = ACTIONS(1875),
    [sym_master_sn] = ACTIONS(1875),
    [sym_system_number] = ACTIONS(1875),
    [sym_button] = ACTIONS(1875),
    [sym_dv_channel] = ACTIONS(1875),
    [sym_data] = ACTIONS(1875),
    [sym_level] = ACTIONS(1875),
    [sym_timeline] = ACTIONS(1875),
    [sym_channel] = ACTIONS(1875),
    [sym_custom] = ACTIONS(1875),
    [sym_atoi] = ACTIONS(1875),
    [sym_atol] = ACTIONS(1875),
    [sym_atof] = ACTIONS(1875),
    [sym_itoa] = ACTIONS(1875),
    [sym_itohex] = ACTIONS(1875),
    [sym_hextoi] = ACTIONS(1875),
    [sym_ftoa] = ACTIONS(1875),
    [sym_find_string] = ACTIONS(1875),
    [sym_left_string] = ACTIONS(1875),
    [sym_right_string] = ACTIONS(1875),
    [sym_mid_string] = ACTIONS(1875),
    [sym_remove_string] = ACTIONS(1875),
    [sym_get_buffer_string] = ACTIONS(1875),
    [sym_get_buffer_char] = ACTIONS(1875),
    [sym_get_multi_buffer_string] = ACTIONS(1875),
    [sym_length_string] = ACTIONS(1875),
    [sym_max_length_string] = ACTIONS(1875),
    [sym_set_length_string] = ACTIONS(1875),
    [sym_compare_string] = ACTIONS(1875),
    [sym_upper_string] = ACTIONS(1875),
    [sym_lower_string] = ACTIONS(1875),
    [sym_get_last] = ACTIONS(1875),
    [sym_date_to_day] = ACTIONS(1875),
    [sym_date_to_month] = ACTIONS(1875),
    [sym_date_to_year] = ACTIONS(1875),
    [sym_time_to_hour] = ACTIONS(1875),
    [sym_time_to_minute] = ACTIONS(1875),
    [sym_time_to_second] = ACTIONS(1875),
    [sym_day_of_week] = ACTIONS(1875),
    [sym_set_pulse_time] = ACTIONS(1875),
    [sym_set_timer] = ACTIONS(1875),
    [sym_file_open] = ACTIONS(1875),
    [sym_file_close] = ACTIONS(1875),
    [sym_file_read] = ACTIONS(1875),
    [sym_file_read_line] = ACTIONS(1875),
    [sym_file_write] = ACTIONS(1875),
    [sym_file_write_line] = ACTIONS(1875),
    [sym_file_copy] = ACTIONS(1875),
    [sym_file_delete] = ACTIONS(1875),
    [sym_file_rename] = ACTIONS(1875),
    [sym_file_dir] = ACTIONS(1875),
    [sym_file_getdir] = ACTIONS(1875),
    [sym_file_setdir] = ACTIONS(1875),
    [sym_file_create_dir] = ACTIONS(1875),
    [sym_file_remove_dir] = ACTIONS(1875),
    [sym_file_seek] = ACTIONS(1875),
    [sym_length_array] = ACTIONS(1875),
    [sym_max_length_array] = ACTIONS(1875),
    [sym_set_length_array] = ACTIONS(1875),
    [sym_ip_client_open] = ACTIONS(1875),
    [sym_ip_client_close] = ACTIONS(1875),
    [sym_ip_server_open] = ACTIONS(1875),
    [sym_ip_server_close] = ACTIONS(1875),
    [sym_ip_bound_client_open] = ACTIONS(1875),
    [sym_ip_mc_server_open] = ACTIONS(1875),
    [sym_ip_set_option] = ACTIONS(1875),
    [sym_ssh_client_open] = ACTIONS(1875),
    [sym_ssh_client_close] = ACTIONS(1875),
    [sym_tls_client_open] = ACTIONS(1875),
    [sym_tls_client_close] = ACTIONS(1875),
    [sym_do_push] = ACTIONS(1875),
    [sym_do_push_timed] = ACTIONS(1875),
    [sym_do_release] = ACTIONS(1875),
    [sym_device_id] = ACTIONS(1875),
    [sym_device_info] = ACTIONS(1875),
    [sym_device_id_string] = ACTIONS(1875),
    [sym_device_standby] = ACTIONS(1875),
    [sym_device_wake] = ACTIONS(1875),
    [sym_timeline_create] = ACTIONS(1875),
    [sym_timeline_kill] = ACTIONS(1875),
    [sym_timeline_reload] = ACTIONS(1875),
    [sym_timeline_pause] = ACTIONS(1875),
    [sym_timeline_restart] = ACTIONS(1875),
    [sym_timeline_set] = ACTIONS(1875),
    [sym_timeline_get] = ACTIONS(1875),
    [sym_timeline_active] = ACTIONS(1875),
    [sym_timeline_dynamic_id] = ACTIONS(1875),
    [sym_variable_to_string] = ACTIONS(1875),
    [sym_string_to_variable] = ACTIONS(1875),
    [sym_length_variable_to_string] = ACTIONS(1875),
    [sym_type_cast] = ACTIONS(1875),
    [sym_variable_to_xml] = ACTIONS(1875),
    [sym_xml_to_variable] = ACTIONS(1875),
    [sym_length_variable_to_xml] = ACTIONS(1875),
    [sym_redirect_string] = ACTIONS(1875),
    [sym_random_number] = ACTIONS(1875),
    [sym_get_unique_id] = ACTIONS(1875),
    [sym_get_system_number] = ACTIONS(1875),
    [sym_set_system_number] = ACTIONS(1875),
    [sym_get_serial_number] = ACTIONS(1875),
    [sym_get_master_build] = ACTIONS(1875),
    [sym_set_validation_code] = ACTIONS(1875),
    [sym_reboot] = ACTIONS(1875),
    [sym_load_duet_module] = ACTIONS(1875),
    [sym_unload_duet_module] = ACTIONS(1875),
    [sym_duet_mem_size_set] = ACTIONS(1875),
    [sym_duet_mem_size_get] = ACTIONS(1875),
    [sym_dynamic_application_device] = ACTIONS(1875),
    [sym_dynamic_polled_port] = ACTIONS(1875),
    [sym_static_port_binding] = ACTIONS(1875),
    [sym_static_ip_binding] = ACTIONS(1875),
    [sym_get_dns_list] = ACTIONS(1875),
    [sym_set_dns_list] = ACTIONS(1875),
    [sym_get_ip_address] = ACTIONS(1875),
    [sym_set_ip_address] = ACTIONS(1875),
    [sym_get_url_list] = ACTIONS(1875),
    [sym_add_url_entry] = ACTIONS(1875),
    [sym_delete_url_entry] = ACTIONS(1875),
    [sym_abs_value] = ACTIONS(1875),
    [sym_max_value] = ACTIONS(1875),
    [sym_min_value] = ACTIONS(1875),
    [sym_exp_value] = ACTIONS(1875),
    [sym_log_value] = ACTIONS(1875),
    [sym_log10_value] = ACTIONS(1875),
    [sym_power_value] = ACTIONS(1875),
    [sym_sqrt_value] = ACTIONS(1875),
    [sym_raw_be] = ACTIONS(1875),
    [sym_raw_le] = ACTIONS(1875),
    [sym_internal_queue_size_set] = ACTIONS(1875),
    [sym_internal_queue_size_get] = ACTIONS(1875),
    [sym_internal_threshold_set] = ACTIONS(1875),
    [sym_internal_threshold_get] = ACTIONS(1875),
    [sym_rebuilt_event] = ACTIONS(1875),
    [sym_do_custom_event] = ACTIONS(1875),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1875),
    [sym_clkmgr_set_clk_source] = ACTIONS(1875),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1875),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1875),
    [sym_clkmgr_get_timezone] = ACTIONS(1875),
    [sym_clkmgr_set_timezone] = ACTIONS(1875),
    [sym_clkmgr_get_resync_period] = ACTIONS(1875),
    [sym_clkmgr_set_resync_period] = ACTIONS(1875),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1875),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1875),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1875),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1875),
    [sym_clkmgr_get_timeservers] = ACTIONS(1875),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1875),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1875),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1875),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1875),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1875),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1875),
    [sym_set_log_level] = ACTIONS(1875),
    [sym_get_log_level] = ACTIONS(1875),
    [sym_amx_log] = ACTIONS(1875),
    [sym_smtp_server_config_set] = ACTIONS(1875),
    [sym_smtp_server_config_get] = ACTIONS(1875),
    [sym_smtp_send] = ACTIONS(1875),
    [sym_validate_netlinx_account] = ACTIONS(1875),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1875),
    [sym_authenticate_certificate] = ACTIONS(1875),
    [sym_audit_netlinx_session_event] = ACTIONS(1875),
    [sym_audit_netlinx_generic_event] = ACTIONS(1875),
    [sym_get_available_flash_disk_space] = ACTIONS(1875),
    [sym_get_max_flash_disk_space] = ACTIONS(1875),
    [sym_data_create_feed] = ACTIONS(1875),
    [sym_data_delete_feed] = ACTIONS(1875),
    [sym_data_publish_feed] = ACTIONS(1875),
    [sym_data_get_published_feed] = ACTIONS(1875),
    [sym_data_add_record] = ACTIONS(1875),
    [sym_data_get_event_record] = ACTIONS(1875),
    [sym__wc_data_create_feed] = ACTIONS(1875),
    [sym__wc_data_add_record] = ACTIONS(1875),
    [sym__wc_data_get_event_record] = ACTIONS(1875),
    [sym_format] = ACTIONS(1875),
    [sym_astro_clock] = ACTIONS(1875),
    [sym_set_outdoor_temperature] = ACTIONS(1875),
    [sym_set_virtual_level_count] = ACTIONS(1875),
    [sym_set_virtual_channel_count] = ACTIONS(1875),
    [sym_set_virtual_port_count] = ACTIONS(1875),
    [sym__wc_encode_int] = ACTIONS(1875),
    [sym__wc_decode_int] = ACTIONS(1875),
    [sym__wc_int] = ACTIONS(1875),
    [sym__wc_to_ch_int] = ACTIONS(1875),
    [sym__ch_to_wc_int] = ACTIONS(1875),
    [sym__wc_find_string_int] = ACTIONS(1875),
    [sym__wc_left_string_int] = ACTIONS(1875),
    [sym__wc_lower_string_int] = ACTIONS(1875),
    [sym__wc_mid_string_int] = ACTIONS(1875),
    [sym__wc_remove_string_int] = ACTIONS(1875),
    [sym__wc_right_string_int] = ACTIONS(1875),
    [sym__wc_upper_string_int] = ACTIONS(1875),
    [sym__wc_compare_string_int] = ACTIONS(1875),
    [sym__wc_get_buffer_char_int] = ACTIONS(1875),
    [sym__wc_get_buffer_string_int] = ACTIONS(1875),
    [sym__wc_concat_string_int] = ACTIONS(1875),
    [sym__wc_file_open_int] = ACTIONS(1875),
    [sym__wc_file_close_int] = ACTIONS(1875),
    [sym__wc_file_read_int] = ACTIONS(1875),
    [sym__wc_file_read_line_int] = ACTIONS(1875),
    [sym__wc_file_write_int] = ACTIONS(1875),
    [sym__wc_file_write_line_int] = ACTIONS(1875),
    [aux_sym_preproc_include_token1] = ACTIONS(1875),
    [aux_sym_preproc_define_token1] = ACTIONS(1877),
    [aux_sym_preproc_warn_token1] = ACTIONS(1877),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1877),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1877),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1877),
    [anon_sym_LBRACK] = ACTIONS(1877),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_RBRACE] = ACTIONS(1877),
    [anon_sym_LPAREN] = ACTIONS(1875),
    [anon_sym_BANG] = ACTIONS(1877),
    [anon_sym_TILDE] = ACTIONS(1877),
    [anon_sym_DASH] = ACTIONS(1875),
    [anon_sym_PLUS] = ACTIONS(1875),
    [anon_sym_DASH_DASH] = ACTIONS(1877),
    [anon_sym_PLUS_PLUS] = ACTIONS(1877),
    [anon_sym_DQUOTE] = ACTIONS(1877),
    [anon_sym_SQUOTE] = ACTIONS(1877),
    [sym_number_literal] = ACTIONS(1877),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(479)] = {
    [sym_identifier] = ACTIONS(1839),
    [sym_define_call_keyword] = ACTIONS(1839),
    [sym_define_function_keyword] = ACTIONS(1839),
    [sym_define_library_function_keyword] = ACTIONS(1839),
    [sym_define_module_keyword] = ACTIONS(1839),
    [sym_char_keyword] = ACTIONS(1839),
    [sym_widechar_keyword] = ACTIONS(1839),
    [sym_integer_keyword] = ACTIONS(1839),
    [sym_sinteger_keyword] = ACTIONS(1839),
    [sym_long_keyword] = ACTIONS(1839),
    [sym_slong_keyword] = ACTIONS(1839),
    [sym_float_keyword] = ACTIONS(1839),
    [sym_double_keyword] = ACTIONS(1839),
    [sym_variant_keyword] = ACTIONS(1839),
    [sym_variantarray_keyword] = ACTIONS(1839),
    [sym_local_var_keyword] = ACTIONS(1839),
    [sym_stack_var_keyword] = ACTIONS(1839),
    [sym_constant_keyword] = ACTIONS(1839),
    [sym_volatile_keyword] = ACTIONS(1839),
    [sym_non_volatile_keyword] = ACTIONS(1839),
    [sym_persistent_keyword] = ACTIONS(1839),
    [sym_dev_keyword] = ACTIONS(1839),
    [sym_devlev_keyword] = ACTIONS(1839),
    [sym_devchan_keyword] = ACTIONS(1839),
    [sym_if_keyword] = ACTIONS(1839),
    [sym_else_keyword] = ACTIONS(1839),
    [sym_switch_keyword] = ACTIONS(1839),
    [sym_case_keyword] = ACTIONS(1839),
    [sym_default_keyword] = ACTIONS(1839),
    [sym_while_keyword] = ACTIONS(1839),
    [sym_for_keyword] = ACTIONS(1839),
    [sym_break_keyword] = ACTIONS(1839),
    [sym_continue_keyword] = ACTIONS(1839),
    [sym_return_keyword] = ACTIONS(1839),
    [sym_select_keyword] = ACTIONS(1839),
    [sym_active_keyword] = ACTIONS(1839),
    [sym_struct_keyword] = ACTIONS(1839),
    [sym_structure_keyword] = ACTIONS(1839),
    [aux_sym_on_keyword_token1] = ACTIONS(1839),
    [aux_sym_off_keyword_token1] = ACTIONS(1839),
    [sym_send_level_keyword] = ACTIONS(1839),
    [sym_send_string_keyword] = ACTIONS(1839),
    [sym_send_command_keyword] = ACTIONS(1839),
    [sym_clear_buffer_keyword] = ACTIONS(1839),
    [sym_create_buffer_keyword] = ACTIONS(1839),
    [sym_create_multi_buffer_keyword] = ACTIONS(1839),
    [sym_call_keyword] = ACTIONS(1839),
    [sym_system_call_keyword] = ACTIONS(1839),
    [sym_devchan_to_keyword] = ACTIONS(1839),
    [sym_devchan_min_to_keyword] = ACTIONS(1839),
    [sym_devchan_total_off_keyword] = ACTIONS(1839),
    [sym_devchan_pulse_keyword] = ACTIONS(1839),
    [sym_wait_keyword] = ACTIONS(1839),
    [sym_cancel_wait_keyword] = ACTIONS(1839),
    [sym_wait_until_keyword] = ACTIONS(1839),
    [sym_cancel_wait_until_keyword] = ACTIONS(1839),
    [sym_cancel_all_wait_keyword] = ACTIONS(1839),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1839),
    [sym_not] = ACTIONS(1839),
    [sym_bnot] = ACTIONS(1839),
    [sym___file__] = ACTIONS(1839),
    [sym___date__] = ACTIONS(1839),
    [sym___ldate__] = ACTIONS(1839),
    [sym___time__] = ACTIONS(1839),
    [sym___line__] = ACTIONS(1839),
    [sym___name__] = ACTIONS(1839),
    [sym___netlinx__] = ACTIONS(1839),
    [sym_netlinx_axi_version] = ACTIONS(1839),
    [sym_url_flg_tcp] = ACTIONS(1839),
    [sym_url_flg_accinfopresent] = ACTIONS(1839),
    [sym_url_flg_temp] = ACTIONS(1839),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1839),
    [sym_url_flg_stat_mask] = ACTIONS(1839),
    [sym_url_flg_stat_lookup] = ACTIONS(1839),
    [sym_url_flg_stat_connecting] = ACTIONS(1839),
    [sym_url_flg_stat_waiting] = ACTIONS(1839),
    [sym_url_flg_stat_connected] = ACTIONS(1839),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1839),
    [sym_first_virtual_device] = ACTIONS(1839),
    [sym_first_local_port] = ACTIONS(1839),
    [sym_dynamic_virtual_device] = ACTIONS(1839),
    [sym_timeline_once] = ACTIONS(1839),
    [sym_timeline_repeat] = ACTIONS(1839),
    [sym_timeline_absolute] = ACTIONS(1839),
    [sym_timeline_relative] = ACTIONS(1839),
    [sym_do_push_timed_infinite] = ACTIONS(1839),
    [sym_source_type_no_address] = ACTIONS(1839),
    [sym_source_type_neuron_id] = ACTIONS(1839),
    [sym_source_type_ip_address] = ACTIONS(1839),
    [sym_source_type_axlink] = ACTIONS(1839),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1839),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1839),
    [sym_source_type_ip_socket_address] = ACTIONS(1839),
    [sym_source_type_rs232] = ACTIONS(1839),
    [sym_source_type_internal] = ACTIONS(1839),
    [sym_source_type_mac_address] = ACTIONS(1839),
    [sym_source_type_ipv4_port] = ACTIONS(1839),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1839),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1839),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1839),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1839),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1839),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1839),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1839),
    [sym_true] = ACTIONS(1839),
    [sym_false] = ACTIONS(1839),
    [sym_file_read_only] = ACTIONS(1839),
    [sym_file_rw_new] = ACTIONS(1839),
    [sym_file_rw_append] = ACTIONS(1839),
    [sym_ip_tcp] = ACTIONS(1839),
    [sym_ip_udp] = ACTIONS(1839),
    [sym_ip_udp_2way] = ACTIONS(1839),
    [sym_xml_encode_types] = ACTIONS(1839),
    [sym_xml_encode_char_as_list] = ACTIONS(1839),
    [sym_xml_encode_le] = ACTIONS(1839),
    [sym_xml_decode_types] = ACTIONS(1839),
    [sym_xml_decode_no_preserve] = ACTIONS(1839),
    [sym_duet_dev_type_display_device] = ACTIONS(1839),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1839),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1839),
    [sym_duet_dev_type_io_device] = ACTIONS(1839),
    [sym_duet_dev_type_relay_device] = ACTIONS(1839),
    [sym_duet_dev_type_ups] = ACTIONS(1839),
    [sym_duet_dev_type_amplifier] = ACTIONS(1839),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1839),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1839),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1839),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1839),
    [sym_duet_dev_type_camera] = ACTIONS(1839),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1839),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1839),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1839),
    [sym_duet_dev_type_dss] = ACTIONS(1839),
    [sym_duet_dev_type_dvr] = ACTIONS(1839),
    [sym_duet_dev_type_disc_device] = ACTIONS(1839),
    [sym_duet_dev_type_document_camera] = ACTIONS(1839),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1839),
    [sym_duet_dev_type_hvac] = ACTIONS(1839),
    [sym_duet_dev_type_keypad] = ACTIONS(1839),
    [sym_duet_dev_type_light] = ACTIONS(1839),
    [sym_duet_dev_type_monitor] = ACTIONS(1839),
    [sym_duet_dev_type_motor] = ACTIONS(1839),
    [sym_duet_dev_type_multi_window] = ACTIONS(1839),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1839),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1839),
    [sym_duet_dev_type_receiver] = ACTIONS(1839),
    [sym_duet_dev_type_security_system] = ACTIONS(1839),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1839),
    [sym_duet_dev_type_settop_box] = ACTIONS(1839),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1839),
    [sym_duet_dev_type_switcher] = ACTIONS(1839),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1839),
    [sym_duet_dev_type_tv] = ACTIONS(1839),
    [sym_duet_dev_type_utility] = ACTIONS(1839),
    [sym_duet_dev_type_vcr] = ACTIONS(1839),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1839),
    [sym_duet_dev_type_video_processor] = ACTIONS(1839),
    [sym_duet_dev_type_video_projector] = ACTIONS(1839),
    [sym_duet_dev_type_video_wall] = ACTIONS(1839),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1839),
    [sym_duet_dev_type_weather] = ACTIONS(1839),
    [sym_duet_dev_not_polled] = ACTIONS(1839),
    [sym_duet_dev_polled] = ACTIONS(1839),
    [sym_clkmgr_mode_network] = ACTIONS(1839),
    [sym_clkmgr_mode_standalone] = ACTIONS(1839),
    [sym_normal_standby] = ACTIONS(1839),
    [sym_normal_wake] = ACTIONS(1839),
    [sym_ip_multicast_ttl_option] = ACTIONS(1839),
    [sym_ip_ttl_subnet] = ACTIONS(1839),
    [sym_ip_ttl_site] = ACTIONS(1839),
    [sym_ip_ttl_region] = ACTIONS(1839),
    [sym_ip_ttl_continent] = ACTIONS(1839),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1839),
    [sym_ip_nodelay_on] = ACTIONS(1839),
    [sym_ip_nodelay_off] = ACTIONS(1839),
    [sym_amx_error] = ACTIONS(1839),
    [sym_amx_warning] = ACTIONS(1839),
    [sym_amx_info] = ACTIONS(1839),
    [sym_amx_debug] = ACTIONS(1839),
    [sym_netlinx_login_success] = ACTIONS(1839),
    [sym_netlinx_login_fail] = ACTIONS(1839),
    [sym_netlinx_logout] = ACTIONS(1839),
    [sym_null_str] = ACTIONS(1839),
    [sym_smtp_address] = ACTIONS(1839),
    [sym_smtp_port_number] = ACTIONS(1839),
    [sym_smtp_username] = ACTIONS(1839),
    [sym_smtp_password] = ACTIONS(1839),
    [sym_smtp_require_tls] = ACTIONS(1839),
    [sym_smtp_from] = ACTIONS(1839),
    [sym_smtp_tls_true] = ACTIONS(1839),
    [sym_smtp_tls_false] = ACTIONS(1839),
    [sym_listview_on_row_select_event] = ACTIONS(1839),
    [sym_data_structure_unknown] = ACTIONS(1839),
    [sym_data_structure_datafeed] = ACTIONS(1839),
    [sym_data_structure_datarecordset] = ACTIONS(1839),
    [sym_data_structure_datarecord] = ACTIONS(1839),
    [sym_data_structure_datafield] = ACTIONS(1839),
    [sym_data_max_content_field_count] = ACTIONS(1839),
    [sym_data_max_metadata_field_count] = ACTIONS(1839),
    [sym_data_max_id_length] = ACTIONS(1839),
    [sym_data_max_name_length] = ACTIONS(1839),
    [sym_data_max_description_length] = ACTIONS(1839),
    [sym_data_max_type_length] = ACTIONS(1839),
    [sym_data_max_format_length] = ACTIONS(1839),
    [sym_data_max_source_length] = ACTIONS(1839),
    [sym_data_max_value_length] = ACTIONS(1839),
    [sym_data_max_label_length] = ACTIONS(1839),
    [sym_data_max_url_length] = ACTIONS(1839),
    [sym_max_last_login_info_length] = ACTIONS(1839),
    [sym_data_type_unknown] = ACTIONS(1839),
    [sym_data_type_string] = ACTIONS(1839),
    [sym_data_type_datetime] = ACTIONS(1839),
    [sym_data_type_date] = ACTIONS(1839),
    [sym_data_type_time] = ACTIONS(1839),
    [sym_data_type_image] = ACTIONS(1839),
    [sym_data_format_url] = ACTIONS(1839),
    [sym_data_format_phone] = ACTIONS(1839),
    [sym_data_format_email] = ACTIONS(1839),
    [sym_data_format_iso8601] = ACTIONS(1839),
    [sym_auditlog_priv] = ACTIONS(1839),
    [sym_configuration_priv] = ACTIONS(1839),
    [sym_deviceconfig_priv] = ACTIONS(1839),
    [sym_ftp_priv] = ACTIONS(1839),
    [sym_http_priv] = ACTIONS(1839),
    [sym_networkconfig_priv] = ACTIONS(1839),
    [sym_programport_priv] = ACTIONS(1839),
    [sym_remoteui_priv] = ACTIONS(1839),
    [sym_securitycontrol_priv] = ACTIONS(1839),
    [sym_softwaremanagement_priv] = ACTIONS(1839),
    [sym_terminal_priv] = ACTIONS(1839),
    [sym_tpadmin_priv] = ACTIONS(1839),
    [sym_user1_priv] = ACTIONS(1839),
    [sym_user2_priv] = ACTIONS(1839),
    [sym_user3_priv] = ACTIONS(1839),
    [sym_user4_priv] = ACTIONS(1839),
    [sym_usermanagement_priv] = ACTIONS(1839),
    [sym_valid_account] = ACTIONS(1839),
    [sym_err_name_invalid] = ACTIONS(1839),
    [sym_err_password_invalid] = ACTIONS(1839),
    [sym_err_invalid_account] = ACTIONS(1839),
    [sym_err_no_authorization] = ACTIONS(1839),
    [sym_err_invalid_parameter] = ACTIONS(1839),
    [sym_err_user_locked_out] = ACTIONS(1839),
    [sym_err_user_account_expired] = ACTIONS(1839),
    [sym_err_general_error] = ACTIONS(1839),
    [sym_err_authentication_error] = ACTIONS(1839),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1839),
    [sym_tls_validate_certificate] = ACTIONS(1839),
    [sym_tdata] = ACTIONS(1839),
    [sym_tchannel] = ACTIONS(1839),
    [sym_tlevel] = ACTIONS(1839),
    [sym_tbutton] = ACTIONS(1839),
    [sym_ttimeline] = ACTIONS(1839),
    [sym_tcustom] = ACTIONS(1839),
    [sym_url_struct] = ACTIONS(1839),
    [sym_dns_struct] = ACTIONS(1839),
    [sym_ip_address_struct] = ACTIONS(1839),
    [sym_dev_info_struct] = ACTIONS(1839),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1839),
    [sym_data_feed] = ACTIONS(1839),
    [sym_data_field] = ACTIONS(1839),
    [sym_data_record] = ACTIONS(1839),
    [sym_wc_data_feed] = ACTIONS(1839),
    [sym_wc_data_field] = ACTIONS(1839),
    [sym_wc_data_record] = ACTIONS(1839),
    [sym_last_login_info] = ACTIONS(1839),
    [sym____reserved___] = ACTIONS(1839),
    [sym_date] = ACTIONS(1839),
    [sym_day] = ACTIONS(1839),
    [sym_time] = ACTIONS(1839),
    [sym_ldate] = ACTIONS(1839),
    [sym_push_device] = ACTIONS(1839),
    [sym_push_channel] = ACTIONS(1839),
    [sym_push_devchan] = ACTIONS(1839),
    [sym_release_device] = ACTIONS(1839),
    [sym_release_channel] = ACTIONS(1839),
    [sym_release_devchan] = ACTIONS(1839),
    [sym_master_slot] = ACTIONS(1839),
    [sym_get_pulse_time] = ACTIONS(1839),
    [sym_get_timer] = ACTIONS(1839),
    [sym_master_sn] = ACTIONS(1839),
    [sym_system_number] = ACTIONS(1839),
    [sym_button] = ACTIONS(1839),
    [sym_dv_channel] = ACTIONS(1839),
    [sym_data] = ACTIONS(1839),
    [sym_level] = ACTIONS(1839),
    [sym_timeline] = ACTIONS(1839),
    [sym_channel] = ACTIONS(1839),
    [sym_custom] = ACTIONS(1839),
    [sym_atoi] = ACTIONS(1839),
    [sym_atol] = ACTIONS(1839),
    [sym_atof] = ACTIONS(1839),
    [sym_itoa] = ACTIONS(1839),
    [sym_itohex] = ACTIONS(1839),
    [sym_hextoi] = ACTIONS(1839),
    [sym_ftoa] = ACTIONS(1839),
    [sym_find_string] = ACTIONS(1839),
    [sym_left_string] = ACTIONS(1839),
    [sym_right_string] = ACTIONS(1839),
    [sym_mid_string] = ACTIONS(1839),
    [sym_remove_string] = ACTIONS(1839),
    [sym_get_buffer_string] = ACTIONS(1839),
    [sym_get_buffer_char] = ACTIONS(1839),
    [sym_get_multi_buffer_string] = ACTIONS(1839),
    [sym_length_string] = ACTIONS(1839),
    [sym_max_length_string] = ACTIONS(1839),
    [sym_set_length_string] = ACTIONS(1839),
    [sym_compare_string] = ACTIONS(1839),
    [sym_upper_string] = ACTIONS(1839),
    [sym_lower_string] = ACTIONS(1839),
    [sym_get_last] = ACTIONS(1839),
    [sym_date_to_day] = ACTIONS(1839),
    [sym_date_to_month] = ACTIONS(1839),
    [sym_date_to_year] = ACTIONS(1839),
    [sym_time_to_hour] = ACTIONS(1839),
    [sym_time_to_minute] = ACTIONS(1839),
    [sym_time_to_second] = ACTIONS(1839),
    [sym_day_of_week] = ACTIONS(1839),
    [sym_set_pulse_time] = ACTIONS(1839),
    [sym_set_timer] = ACTIONS(1839),
    [sym_file_open] = ACTIONS(1839),
    [sym_file_close] = ACTIONS(1839),
    [sym_file_read] = ACTIONS(1839),
    [sym_file_read_line] = ACTIONS(1839),
    [sym_file_write] = ACTIONS(1839),
    [sym_file_write_line] = ACTIONS(1839),
    [sym_file_copy] = ACTIONS(1839),
    [sym_file_delete] = ACTIONS(1839),
    [sym_file_rename] = ACTIONS(1839),
    [sym_file_dir] = ACTIONS(1839),
    [sym_file_getdir] = ACTIONS(1839),
    [sym_file_setdir] = ACTIONS(1839),
    [sym_file_create_dir] = ACTIONS(1839),
    [sym_file_remove_dir] = ACTIONS(1839),
    [sym_file_seek] = ACTIONS(1839),
    [sym_length_array] = ACTIONS(1839),
    [sym_max_length_array] = ACTIONS(1839),
    [sym_set_length_array] = ACTIONS(1839),
    [sym_ip_client_open] = ACTIONS(1839),
    [sym_ip_client_close] = ACTIONS(1839),
    [sym_ip_server_open] = ACTIONS(1839),
    [sym_ip_server_close] = ACTIONS(1839),
    [sym_ip_bound_client_open] = ACTIONS(1839),
    [sym_ip_mc_server_open] = ACTIONS(1839),
    [sym_ip_set_option] = ACTIONS(1839),
    [sym_ssh_client_open] = ACTIONS(1839),
    [sym_ssh_client_close] = ACTIONS(1839),
    [sym_tls_client_open] = ACTIONS(1839),
    [sym_tls_client_close] = ACTIONS(1839),
    [sym_do_push] = ACTIONS(1839),
    [sym_do_push_timed] = ACTIONS(1839),
    [sym_do_release] = ACTIONS(1839),
    [sym_device_id] = ACTIONS(1839),
    [sym_device_info] = ACTIONS(1839),
    [sym_device_id_string] = ACTIONS(1839),
    [sym_device_standby] = ACTIONS(1839),
    [sym_device_wake] = ACTIONS(1839),
    [sym_timeline_create] = ACTIONS(1839),
    [sym_timeline_kill] = ACTIONS(1839),
    [sym_timeline_reload] = ACTIONS(1839),
    [sym_timeline_pause] = ACTIONS(1839),
    [sym_timeline_restart] = ACTIONS(1839),
    [sym_timeline_set] = ACTIONS(1839),
    [sym_timeline_get] = ACTIONS(1839),
    [sym_timeline_active] = ACTIONS(1839),
    [sym_timeline_dynamic_id] = ACTIONS(1839),
    [sym_variable_to_string] = ACTIONS(1839),
    [sym_string_to_variable] = ACTIONS(1839),
    [sym_length_variable_to_string] = ACTIONS(1839),
    [sym_type_cast] = ACTIONS(1839),
    [sym_variable_to_xml] = ACTIONS(1839),
    [sym_xml_to_variable] = ACTIONS(1839),
    [sym_length_variable_to_xml] = ACTIONS(1839),
    [sym_redirect_string] = ACTIONS(1839),
    [sym_random_number] = ACTIONS(1839),
    [sym_get_unique_id] = ACTIONS(1839),
    [sym_get_system_number] = ACTIONS(1839),
    [sym_set_system_number] = ACTIONS(1839),
    [sym_get_serial_number] = ACTIONS(1839),
    [sym_get_master_build] = ACTIONS(1839),
    [sym_set_validation_code] = ACTIONS(1839),
    [sym_reboot] = ACTIONS(1839),
    [sym_load_duet_module] = ACTIONS(1839),
    [sym_unload_duet_module] = ACTIONS(1839),
    [sym_duet_mem_size_set] = ACTIONS(1839),
    [sym_duet_mem_size_get] = ACTIONS(1839),
    [sym_dynamic_application_device] = ACTIONS(1839),
    [sym_dynamic_polled_port] = ACTIONS(1839),
    [sym_static_port_binding] = ACTIONS(1839),
    [sym_static_ip_binding] = ACTIONS(1839),
    [sym_get_dns_list] = ACTIONS(1839),
    [sym_set_dns_list] = ACTIONS(1839),
    [sym_get_ip_address] = ACTIONS(1839),
    [sym_set_ip_address] = ACTIONS(1839),
    [sym_get_url_list] = ACTIONS(1839),
    [sym_add_url_entry] = ACTIONS(1839),
    [sym_delete_url_entry] = ACTIONS(1839),
    [sym_abs_value] = ACTIONS(1839),
    [sym_max_value] = ACTIONS(1839),
    [sym_min_value] = ACTIONS(1839),
    [sym_exp_value] = ACTIONS(1839),
    [sym_log_value] = ACTIONS(1839),
    [sym_log10_value] = ACTIONS(1839),
    [sym_power_value] = ACTIONS(1839),
    [sym_sqrt_value] = ACTIONS(1839),
    [sym_raw_be] = ACTIONS(1839),
    [sym_raw_le] = ACTIONS(1839),
    [sym_internal_queue_size_set] = ACTIONS(1839),
    [sym_internal_queue_size_get] = ACTIONS(1839),
    [sym_internal_threshold_set] = ACTIONS(1839),
    [sym_internal_threshold_get] = ACTIONS(1839),
    [sym_rebuilt_event] = ACTIONS(1839),
    [sym_do_custom_event] = ACTIONS(1839),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1839),
    [sym_clkmgr_set_clk_source] = ACTIONS(1839),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1839),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1839),
    [sym_clkmgr_get_timezone] = ACTIONS(1839),
    [sym_clkmgr_set_timezone] = ACTIONS(1839),
    [sym_clkmgr_get_resync_period] = ACTIONS(1839),
    [sym_clkmgr_set_resync_period] = ACTIONS(1839),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1839),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1839),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1839),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1839),
    [sym_clkmgr_get_timeservers] = ACTIONS(1839),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1839),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1839),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1839),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1839),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1839),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1839),
    [sym_set_log_level] = ACTIONS(1839),
    [sym_get_log_level] = ACTIONS(1839),
    [sym_amx_log] = ACTIONS(1839),
    [sym_smtp_server_config_set] = ACTIONS(1839),
    [sym_smtp_server_config_get] = ACTIONS(1839),
    [sym_smtp_send] = ACTIONS(1839),
    [sym_validate_netlinx_account] = ACTIONS(1839),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1839),
    [sym_authenticate_certificate] = ACTIONS(1839),
    [sym_audit_netlinx_session_event] = ACTIONS(1839),
    [sym_audit_netlinx_generic_event] = ACTIONS(1839),
    [sym_get_available_flash_disk_space] = ACTIONS(1839),
    [sym_get_max_flash_disk_space] = ACTIONS(1839),
    [sym_data_create_feed] = ACTIONS(1839),
    [sym_data_delete_feed] = ACTIONS(1839),
    [sym_data_publish_feed] = ACTIONS(1839),
    [sym_data_get_published_feed] = ACTIONS(1839),
    [sym_data_add_record] = ACTIONS(1839),
    [sym_data_get_event_record] = ACTIONS(1839),
    [sym__wc_data_create_feed] = ACTIONS(1839),
    [sym__wc_data_add_record] = ACTIONS(1839),
    [sym__wc_data_get_event_record] = ACTIONS(1839),
    [sym_format] = ACTIONS(1839),
    [sym_astro_clock] = ACTIONS(1839),
    [sym_set_outdoor_temperature] = ACTIONS(1839),
    [sym_set_virtual_level_count] = ACTIONS(1839),
    [sym_set_virtual_channel_count] = ACTIONS(1839),
    [sym_set_virtual_port_count] = ACTIONS(1839),
    [sym__wc_encode_int] = ACTIONS(1839),
    [sym__wc_decode_int] = ACTIONS(1839),
    [sym__wc_int] = ACTIONS(1839),
    [sym__wc_to_ch_int] = ACTIONS(1839),
    [sym__ch_to_wc_int] = ACTIONS(1839),
    [sym__wc_find_string_int] = ACTIONS(1839),
    [sym__wc_left_string_int] = ACTIONS(1839),
    [sym__wc_lower_string_int] = ACTIONS(1839),
    [sym__wc_mid_string_int] = ACTIONS(1839),
    [sym__wc_remove_string_int] = ACTIONS(1839),
    [sym__wc_right_string_int] = ACTIONS(1839),
    [sym__wc_upper_string_int] = ACTIONS(1839),
    [sym__wc_compare_string_int] = ACTIONS(1839),
    [sym__wc_get_buffer_char_int] = ACTIONS(1839),
    [sym__wc_get_buffer_string_int] = ACTIONS(1839),
    [sym__wc_concat_string_int] = ACTIONS(1839),
    [sym__wc_file_open_int] = ACTIONS(1839),
    [sym__wc_file_close_int] = ACTIONS(1839),
    [sym__wc_file_read_int] = ACTIONS(1839),
    [sym__wc_file_read_line_int] = ACTIONS(1839),
    [sym__wc_file_write_int] = ACTIONS(1839),
    [sym__wc_file_write_line_int] = ACTIONS(1839),
    [aux_sym_preproc_include_token1] = ACTIONS(1839),
    [aux_sym_preproc_define_token1] = ACTIONS(1841),
    [aux_sym_preproc_warn_token1] = ACTIONS(1841),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1841),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1841),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1841),
    [anon_sym_LBRACK] = ACTIONS(1841),
    [anon_sym_LBRACE] = ACTIONS(1841),
    [anon_sym_RBRACE] = ACTIONS(1841),
    [anon_sym_LPAREN] = ACTIONS(1839),
    [anon_sym_BANG] = ACTIONS(1841),
    [anon_sym_TILDE] = ACTIONS(1841),
    [anon_sym_DASH] = ACTIONS(1839),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_DASH_DASH] = ACTIONS(1841),
    [anon_sym_PLUS_PLUS] = ACTIONS(1841),
    [anon_sym_DQUOTE] = ACTIONS(1841),
    [anon_sym_SQUOTE] = ACTIONS(1841),
    [sym_number_literal] = ACTIONS(1841),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(480)] = {
    [sym_identifier] = ACTIONS(1879),
    [sym_define_call_keyword] = ACTIONS(1879),
    [sym_define_function_keyword] = ACTIONS(1879),
    [sym_define_library_function_keyword] = ACTIONS(1879),
    [sym_define_module_keyword] = ACTIONS(1879),
    [sym_char_keyword] = ACTIONS(1879),
    [sym_widechar_keyword] = ACTIONS(1879),
    [sym_integer_keyword] = ACTIONS(1879),
    [sym_sinteger_keyword] = ACTIONS(1879),
    [sym_long_keyword] = ACTIONS(1879),
    [sym_slong_keyword] = ACTIONS(1879),
    [sym_float_keyword] = ACTIONS(1879),
    [sym_double_keyword] = ACTIONS(1879),
    [sym_variant_keyword] = ACTIONS(1879),
    [sym_variantarray_keyword] = ACTIONS(1879),
    [sym_local_var_keyword] = ACTIONS(1879),
    [sym_stack_var_keyword] = ACTIONS(1879),
    [sym_constant_keyword] = ACTIONS(1879),
    [sym_volatile_keyword] = ACTIONS(1879),
    [sym_non_volatile_keyword] = ACTIONS(1879),
    [sym_persistent_keyword] = ACTIONS(1879),
    [sym_dev_keyword] = ACTIONS(1879),
    [sym_devlev_keyword] = ACTIONS(1879),
    [sym_devchan_keyword] = ACTIONS(1879),
    [sym_if_keyword] = ACTIONS(1879),
    [sym_else_keyword] = ACTIONS(1879),
    [sym_switch_keyword] = ACTIONS(1879),
    [sym_case_keyword] = ACTIONS(1879),
    [sym_default_keyword] = ACTIONS(1879),
    [sym_while_keyword] = ACTIONS(1879),
    [sym_for_keyword] = ACTIONS(1879),
    [sym_break_keyword] = ACTIONS(1879),
    [sym_continue_keyword] = ACTIONS(1879),
    [sym_return_keyword] = ACTIONS(1879),
    [sym_select_keyword] = ACTIONS(1879),
    [sym_active_keyword] = ACTIONS(1879),
    [sym_struct_keyword] = ACTIONS(1879),
    [sym_structure_keyword] = ACTIONS(1879),
    [aux_sym_on_keyword_token1] = ACTIONS(1879),
    [aux_sym_off_keyword_token1] = ACTIONS(1879),
    [sym_send_level_keyword] = ACTIONS(1879),
    [sym_send_string_keyword] = ACTIONS(1879),
    [sym_send_command_keyword] = ACTIONS(1879),
    [sym_clear_buffer_keyword] = ACTIONS(1879),
    [sym_create_buffer_keyword] = ACTIONS(1879),
    [sym_create_multi_buffer_keyword] = ACTIONS(1879),
    [sym_call_keyword] = ACTIONS(1879),
    [sym_system_call_keyword] = ACTIONS(1879),
    [sym_devchan_to_keyword] = ACTIONS(1879),
    [sym_devchan_min_to_keyword] = ACTIONS(1879),
    [sym_devchan_total_off_keyword] = ACTIONS(1879),
    [sym_devchan_pulse_keyword] = ACTIONS(1879),
    [sym_wait_keyword] = ACTIONS(1879),
    [sym_cancel_wait_keyword] = ACTIONS(1879),
    [sym_wait_until_keyword] = ACTIONS(1879),
    [sym_cancel_wait_until_keyword] = ACTIONS(1879),
    [sym_cancel_all_wait_keyword] = ACTIONS(1879),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1879),
    [sym_not] = ACTIONS(1879),
    [sym_bnot] = ACTIONS(1879),
    [sym___file__] = ACTIONS(1879),
    [sym___date__] = ACTIONS(1879),
    [sym___ldate__] = ACTIONS(1879),
    [sym___time__] = ACTIONS(1879),
    [sym___line__] = ACTIONS(1879),
    [sym___name__] = ACTIONS(1879),
    [sym___netlinx__] = ACTIONS(1879),
    [sym_netlinx_axi_version] = ACTIONS(1879),
    [sym_url_flg_tcp] = ACTIONS(1879),
    [sym_url_flg_accinfopresent] = ACTIONS(1879),
    [sym_url_flg_temp] = ACTIONS(1879),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1879),
    [sym_url_flg_stat_mask] = ACTIONS(1879),
    [sym_url_flg_stat_lookup] = ACTIONS(1879),
    [sym_url_flg_stat_connecting] = ACTIONS(1879),
    [sym_url_flg_stat_waiting] = ACTIONS(1879),
    [sym_url_flg_stat_connected] = ACTIONS(1879),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1879),
    [sym_first_virtual_device] = ACTIONS(1879),
    [sym_first_local_port] = ACTIONS(1879),
    [sym_dynamic_virtual_device] = ACTIONS(1879),
    [sym_timeline_once] = ACTIONS(1879),
    [sym_timeline_repeat] = ACTIONS(1879),
    [sym_timeline_absolute] = ACTIONS(1879),
    [sym_timeline_relative] = ACTIONS(1879),
    [sym_do_push_timed_infinite] = ACTIONS(1879),
    [sym_source_type_no_address] = ACTIONS(1879),
    [sym_source_type_neuron_id] = ACTIONS(1879),
    [sym_source_type_ip_address] = ACTIONS(1879),
    [sym_source_type_axlink] = ACTIONS(1879),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1879),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1879),
    [sym_source_type_ip_socket_address] = ACTIONS(1879),
    [sym_source_type_rs232] = ACTIONS(1879),
    [sym_source_type_internal] = ACTIONS(1879),
    [sym_source_type_mac_address] = ACTIONS(1879),
    [sym_source_type_ipv4_port] = ACTIONS(1879),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1879),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1879),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1879),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1879),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1879),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1879),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1879),
    [sym_true] = ACTIONS(1879),
    [sym_false] = ACTIONS(1879),
    [sym_file_read_only] = ACTIONS(1879),
    [sym_file_rw_new] = ACTIONS(1879),
    [sym_file_rw_append] = ACTIONS(1879),
    [sym_ip_tcp] = ACTIONS(1879),
    [sym_ip_udp] = ACTIONS(1879),
    [sym_ip_udp_2way] = ACTIONS(1879),
    [sym_xml_encode_types] = ACTIONS(1879),
    [sym_xml_encode_char_as_list] = ACTIONS(1879),
    [sym_xml_encode_le] = ACTIONS(1879),
    [sym_xml_decode_types] = ACTIONS(1879),
    [sym_xml_decode_no_preserve] = ACTIONS(1879),
    [sym_duet_dev_type_display_device] = ACTIONS(1879),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1879),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1879),
    [sym_duet_dev_type_io_device] = ACTIONS(1879),
    [sym_duet_dev_type_relay_device] = ACTIONS(1879),
    [sym_duet_dev_type_ups] = ACTIONS(1879),
    [sym_duet_dev_type_amplifier] = ACTIONS(1879),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1879),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1879),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1879),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1879),
    [sym_duet_dev_type_camera] = ACTIONS(1879),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1879),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1879),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1879),
    [sym_duet_dev_type_dss] = ACTIONS(1879),
    [sym_duet_dev_type_dvr] = ACTIONS(1879),
    [sym_duet_dev_type_disc_device] = ACTIONS(1879),
    [sym_duet_dev_type_document_camera] = ACTIONS(1879),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1879),
    [sym_duet_dev_type_hvac] = ACTIONS(1879),
    [sym_duet_dev_type_keypad] = ACTIONS(1879),
    [sym_duet_dev_type_light] = ACTIONS(1879),
    [sym_duet_dev_type_monitor] = ACTIONS(1879),
    [sym_duet_dev_type_motor] = ACTIONS(1879),
    [sym_duet_dev_type_multi_window] = ACTIONS(1879),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1879),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1879),
    [sym_duet_dev_type_receiver] = ACTIONS(1879),
    [sym_duet_dev_type_security_system] = ACTIONS(1879),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1879),
    [sym_duet_dev_type_settop_box] = ACTIONS(1879),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1879),
    [sym_duet_dev_type_switcher] = ACTIONS(1879),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1879),
    [sym_duet_dev_type_tv] = ACTIONS(1879),
    [sym_duet_dev_type_utility] = ACTIONS(1879),
    [sym_duet_dev_type_vcr] = ACTIONS(1879),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1879),
    [sym_duet_dev_type_video_processor] = ACTIONS(1879),
    [sym_duet_dev_type_video_projector] = ACTIONS(1879),
    [sym_duet_dev_type_video_wall] = ACTIONS(1879),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1879),
    [sym_duet_dev_type_weather] = ACTIONS(1879),
    [sym_duet_dev_not_polled] = ACTIONS(1879),
    [sym_duet_dev_polled] = ACTIONS(1879),
    [sym_clkmgr_mode_network] = ACTIONS(1879),
    [sym_clkmgr_mode_standalone] = ACTIONS(1879),
    [sym_normal_standby] = ACTIONS(1879),
    [sym_normal_wake] = ACTIONS(1879),
    [sym_ip_multicast_ttl_option] = ACTIONS(1879),
    [sym_ip_ttl_subnet] = ACTIONS(1879),
    [sym_ip_ttl_site] = ACTIONS(1879),
    [sym_ip_ttl_region] = ACTIONS(1879),
    [sym_ip_ttl_continent] = ACTIONS(1879),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1879),
    [sym_ip_nodelay_on] = ACTIONS(1879),
    [sym_ip_nodelay_off] = ACTIONS(1879),
    [sym_amx_error] = ACTIONS(1879),
    [sym_amx_warning] = ACTIONS(1879),
    [sym_amx_info] = ACTIONS(1879),
    [sym_amx_debug] = ACTIONS(1879),
    [sym_netlinx_login_success] = ACTIONS(1879),
    [sym_netlinx_login_fail] = ACTIONS(1879),
    [sym_netlinx_logout] = ACTIONS(1879),
    [sym_null_str] = ACTIONS(1879),
    [sym_smtp_address] = ACTIONS(1879),
    [sym_smtp_port_number] = ACTIONS(1879),
    [sym_smtp_username] = ACTIONS(1879),
    [sym_smtp_password] = ACTIONS(1879),
    [sym_smtp_require_tls] = ACTIONS(1879),
    [sym_smtp_from] = ACTIONS(1879),
    [sym_smtp_tls_true] = ACTIONS(1879),
    [sym_smtp_tls_false] = ACTIONS(1879),
    [sym_listview_on_row_select_event] = ACTIONS(1879),
    [sym_data_structure_unknown] = ACTIONS(1879),
    [sym_data_structure_datafeed] = ACTIONS(1879),
    [sym_data_structure_datarecordset] = ACTIONS(1879),
    [sym_data_structure_datarecord] = ACTIONS(1879),
    [sym_data_structure_datafield] = ACTIONS(1879),
    [sym_data_max_content_field_count] = ACTIONS(1879),
    [sym_data_max_metadata_field_count] = ACTIONS(1879),
    [sym_data_max_id_length] = ACTIONS(1879),
    [sym_data_max_name_length] = ACTIONS(1879),
    [sym_data_max_description_length] = ACTIONS(1879),
    [sym_data_max_type_length] = ACTIONS(1879),
    [sym_data_max_format_length] = ACTIONS(1879),
    [sym_data_max_source_length] = ACTIONS(1879),
    [sym_data_max_value_length] = ACTIONS(1879),
    [sym_data_max_label_length] = ACTIONS(1879),
    [sym_data_max_url_length] = ACTIONS(1879),
    [sym_max_last_login_info_length] = ACTIONS(1879),
    [sym_data_type_unknown] = ACTIONS(1879),
    [sym_data_type_string] = ACTIONS(1879),
    [sym_data_type_datetime] = ACTIONS(1879),
    [sym_data_type_date] = ACTIONS(1879),
    [sym_data_type_time] = ACTIONS(1879),
    [sym_data_type_image] = ACTIONS(1879),
    [sym_data_format_url] = ACTIONS(1879),
    [sym_data_format_phone] = ACTIONS(1879),
    [sym_data_format_email] = ACTIONS(1879),
    [sym_data_format_iso8601] = ACTIONS(1879),
    [sym_auditlog_priv] = ACTIONS(1879),
    [sym_configuration_priv] = ACTIONS(1879),
    [sym_deviceconfig_priv] = ACTIONS(1879),
    [sym_ftp_priv] = ACTIONS(1879),
    [sym_http_priv] = ACTIONS(1879),
    [sym_networkconfig_priv] = ACTIONS(1879),
    [sym_programport_priv] = ACTIONS(1879),
    [sym_remoteui_priv] = ACTIONS(1879),
    [sym_securitycontrol_priv] = ACTIONS(1879),
    [sym_softwaremanagement_priv] = ACTIONS(1879),
    [sym_terminal_priv] = ACTIONS(1879),
    [sym_tpadmin_priv] = ACTIONS(1879),
    [sym_user1_priv] = ACTIONS(1879),
    [sym_user2_priv] = ACTIONS(1879),
    [sym_user3_priv] = ACTIONS(1879),
    [sym_user4_priv] = ACTIONS(1879),
    [sym_usermanagement_priv] = ACTIONS(1879),
    [sym_valid_account] = ACTIONS(1879),
    [sym_err_name_invalid] = ACTIONS(1879),
    [sym_err_password_invalid] = ACTIONS(1879),
    [sym_err_invalid_account] = ACTIONS(1879),
    [sym_err_no_authorization] = ACTIONS(1879),
    [sym_err_invalid_parameter] = ACTIONS(1879),
    [sym_err_user_locked_out] = ACTIONS(1879),
    [sym_err_user_account_expired] = ACTIONS(1879),
    [sym_err_general_error] = ACTIONS(1879),
    [sym_err_authentication_error] = ACTIONS(1879),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1879),
    [sym_tls_validate_certificate] = ACTIONS(1879),
    [sym_tdata] = ACTIONS(1879),
    [sym_tchannel] = ACTIONS(1879),
    [sym_tlevel] = ACTIONS(1879),
    [sym_tbutton] = ACTIONS(1879),
    [sym_ttimeline] = ACTIONS(1879),
    [sym_tcustom] = ACTIONS(1879),
    [sym_url_struct] = ACTIONS(1879),
    [sym_dns_struct] = ACTIONS(1879),
    [sym_ip_address_struct] = ACTIONS(1879),
    [sym_dev_info_struct] = ACTIONS(1879),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1879),
    [sym_data_feed] = ACTIONS(1879),
    [sym_data_field] = ACTIONS(1879),
    [sym_data_record] = ACTIONS(1879),
    [sym_wc_data_feed] = ACTIONS(1879),
    [sym_wc_data_field] = ACTIONS(1879),
    [sym_wc_data_record] = ACTIONS(1879),
    [sym_last_login_info] = ACTIONS(1879),
    [sym____reserved___] = ACTIONS(1879),
    [sym_date] = ACTIONS(1879),
    [sym_day] = ACTIONS(1879),
    [sym_time] = ACTIONS(1879),
    [sym_ldate] = ACTIONS(1879),
    [sym_push_device] = ACTIONS(1879),
    [sym_push_channel] = ACTIONS(1879),
    [sym_push_devchan] = ACTIONS(1879),
    [sym_release_device] = ACTIONS(1879),
    [sym_release_channel] = ACTIONS(1879),
    [sym_release_devchan] = ACTIONS(1879),
    [sym_master_slot] = ACTIONS(1879),
    [sym_get_pulse_time] = ACTIONS(1879),
    [sym_get_timer] = ACTIONS(1879),
    [sym_master_sn] = ACTIONS(1879),
    [sym_system_number] = ACTIONS(1879),
    [sym_button] = ACTIONS(1879),
    [sym_dv_channel] = ACTIONS(1879),
    [sym_data] = ACTIONS(1879),
    [sym_level] = ACTIONS(1879),
    [sym_timeline] = ACTIONS(1879),
    [sym_channel] = ACTIONS(1879),
    [sym_custom] = ACTIONS(1879),
    [sym_atoi] = ACTIONS(1879),
    [sym_atol] = ACTIONS(1879),
    [sym_atof] = ACTIONS(1879),
    [sym_itoa] = ACTIONS(1879),
    [sym_itohex] = ACTIONS(1879),
    [sym_hextoi] = ACTIONS(1879),
    [sym_ftoa] = ACTIONS(1879),
    [sym_find_string] = ACTIONS(1879),
    [sym_left_string] = ACTIONS(1879),
    [sym_right_string] = ACTIONS(1879),
    [sym_mid_string] = ACTIONS(1879),
    [sym_remove_string] = ACTIONS(1879),
    [sym_get_buffer_string] = ACTIONS(1879),
    [sym_get_buffer_char] = ACTIONS(1879),
    [sym_get_multi_buffer_string] = ACTIONS(1879),
    [sym_length_string] = ACTIONS(1879),
    [sym_max_length_string] = ACTIONS(1879),
    [sym_set_length_string] = ACTIONS(1879),
    [sym_compare_string] = ACTIONS(1879),
    [sym_upper_string] = ACTIONS(1879),
    [sym_lower_string] = ACTIONS(1879),
    [sym_get_last] = ACTIONS(1879),
    [sym_date_to_day] = ACTIONS(1879),
    [sym_date_to_month] = ACTIONS(1879),
    [sym_date_to_year] = ACTIONS(1879),
    [sym_time_to_hour] = ACTIONS(1879),
    [sym_time_to_minute] = ACTIONS(1879),
    [sym_time_to_second] = ACTIONS(1879),
    [sym_day_of_week] = ACTIONS(1879),
    [sym_set_pulse_time] = ACTIONS(1879),
    [sym_set_timer] = ACTIONS(1879),
    [sym_file_open] = ACTIONS(1879),
    [sym_file_close] = ACTIONS(1879),
    [sym_file_read] = ACTIONS(1879),
    [sym_file_read_line] = ACTIONS(1879),
    [sym_file_write] = ACTIONS(1879),
    [sym_file_write_line] = ACTIONS(1879),
    [sym_file_copy] = ACTIONS(1879),
    [sym_file_delete] = ACTIONS(1879),
    [sym_file_rename] = ACTIONS(1879),
    [sym_file_dir] = ACTIONS(1879),
    [sym_file_getdir] = ACTIONS(1879),
    [sym_file_setdir] = ACTIONS(1879),
    [sym_file_create_dir] = ACTIONS(1879),
    [sym_file_remove_dir] = ACTIONS(1879),
    [sym_file_seek] = ACTIONS(1879),
    [sym_length_array] = ACTIONS(1879),
    [sym_max_length_array] = ACTIONS(1879),
    [sym_set_length_array] = ACTIONS(1879),
    [sym_ip_client_open] = ACTIONS(1879),
    [sym_ip_client_close] = ACTIONS(1879),
    [sym_ip_server_open] = ACTIONS(1879),
    [sym_ip_server_close] = ACTIONS(1879),
    [sym_ip_bound_client_open] = ACTIONS(1879),
    [sym_ip_mc_server_open] = ACTIONS(1879),
    [sym_ip_set_option] = ACTIONS(1879),
    [sym_ssh_client_open] = ACTIONS(1879),
    [sym_ssh_client_close] = ACTIONS(1879),
    [sym_tls_client_open] = ACTIONS(1879),
    [sym_tls_client_close] = ACTIONS(1879),
    [sym_do_push] = ACTIONS(1879),
    [sym_do_push_timed] = ACTIONS(1879),
    [sym_do_release] = ACTIONS(1879),
    [sym_device_id] = ACTIONS(1879),
    [sym_device_info] = ACTIONS(1879),
    [sym_device_id_string] = ACTIONS(1879),
    [sym_device_standby] = ACTIONS(1879),
    [sym_device_wake] = ACTIONS(1879),
    [sym_timeline_create] = ACTIONS(1879),
    [sym_timeline_kill] = ACTIONS(1879),
    [sym_timeline_reload] = ACTIONS(1879),
    [sym_timeline_pause] = ACTIONS(1879),
    [sym_timeline_restart] = ACTIONS(1879),
    [sym_timeline_set] = ACTIONS(1879),
    [sym_timeline_get] = ACTIONS(1879),
    [sym_timeline_active] = ACTIONS(1879),
    [sym_timeline_dynamic_id] = ACTIONS(1879),
    [sym_variable_to_string] = ACTIONS(1879),
    [sym_string_to_variable] = ACTIONS(1879),
    [sym_length_variable_to_string] = ACTIONS(1879),
    [sym_type_cast] = ACTIONS(1879),
    [sym_variable_to_xml] = ACTIONS(1879),
    [sym_xml_to_variable] = ACTIONS(1879),
    [sym_length_variable_to_xml] = ACTIONS(1879),
    [sym_redirect_string] = ACTIONS(1879),
    [sym_random_number] = ACTIONS(1879),
    [sym_get_unique_id] = ACTIONS(1879),
    [sym_get_system_number] = ACTIONS(1879),
    [sym_set_system_number] = ACTIONS(1879),
    [sym_get_serial_number] = ACTIONS(1879),
    [sym_get_master_build] = ACTIONS(1879),
    [sym_set_validation_code] = ACTIONS(1879),
    [sym_reboot] = ACTIONS(1879),
    [sym_load_duet_module] = ACTIONS(1879),
    [sym_unload_duet_module] = ACTIONS(1879),
    [sym_duet_mem_size_set] = ACTIONS(1879),
    [sym_duet_mem_size_get] = ACTIONS(1879),
    [sym_dynamic_application_device] = ACTIONS(1879),
    [sym_dynamic_polled_port] = ACTIONS(1879),
    [sym_static_port_binding] = ACTIONS(1879),
    [sym_static_ip_binding] = ACTIONS(1879),
    [sym_get_dns_list] = ACTIONS(1879),
    [sym_set_dns_list] = ACTIONS(1879),
    [sym_get_ip_address] = ACTIONS(1879),
    [sym_set_ip_address] = ACTIONS(1879),
    [sym_get_url_list] = ACTIONS(1879),
    [sym_add_url_entry] = ACTIONS(1879),
    [sym_delete_url_entry] = ACTIONS(1879),
    [sym_abs_value] = ACTIONS(1879),
    [sym_max_value] = ACTIONS(1879),
    [sym_min_value] = ACTIONS(1879),
    [sym_exp_value] = ACTIONS(1879),
    [sym_log_value] = ACTIONS(1879),
    [sym_log10_value] = ACTIONS(1879),
    [sym_power_value] = ACTIONS(1879),
    [sym_sqrt_value] = ACTIONS(1879),
    [sym_raw_be] = ACTIONS(1879),
    [sym_raw_le] = ACTIONS(1879),
    [sym_internal_queue_size_set] = ACTIONS(1879),
    [sym_internal_queue_size_get] = ACTIONS(1879),
    [sym_internal_threshold_set] = ACTIONS(1879),
    [sym_internal_threshold_get] = ACTIONS(1879),
    [sym_rebuilt_event] = ACTIONS(1879),
    [sym_do_custom_event] = ACTIONS(1879),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1879),
    [sym_clkmgr_set_clk_source] = ACTIONS(1879),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1879),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1879),
    [sym_clkmgr_get_timezone] = ACTIONS(1879),
    [sym_clkmgr_set_timezone] = ACTIONS(1879),
    [sym_clkmgr_get_resync_period] = ACTIONS(1879),
    [sym_clkmgr_set_resync_period] = ACTIONS(1879),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1879),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1879),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1879),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1879),
    [sym_clkmgr_get_timeservers] = ACTIONS(1879),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1879),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1879),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1879),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1879),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1879),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1879),
    [sym_set_log_level] = ACTIONS(1879),
    [sym_get_log_level] = ACTIONS(1879),
    [sym_amx_log] = ACTIONS(1879),
    [sym_smtp_server_config_set] = ACTIONS(1879),
    [sym_smtp_server_config_get] = ACTIONS(1879),
    [sym_smtp_send] = ACTIONS(1879),
    [sym_validate_netlinx_account] = ACTIONS(1879),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1879),
    [sym_authenticate_certificate] = ACTIONS(1879),
    [sym_audit_netlinx_session_event] = ACTIONS(1879),
    [sym_audit_netlinx_generic_event] = ACTIONS(1879),
    [sym_get_available_flash_disk_space] = ACTIONS(1879),
    [sym_get_max_flash_disk_space] = ACTIONS(1879),
    [sym_data_create_feed] = ACTIONS(1879),
    [sym_data_delete_feed] = ACTIONS(1879),
    [sym_data_publish_feed] = ACTIONS(1879),
    [sym_data_get_published_feed] = ACTIONS(1879),
    [sym_data_add_record] = ACTIONS(1879),
    [sym_data_get_event_record] = ACTIONS(1879),
    [sym__wc_data_create_feed] = ACTIONS(1879),
    [sym__wc_data_add_record] = ACTIONS(1879),
    [sym__wc_data_get_event_record] = ACTIONS(1879),
    [sym_format] = ACTIONS(1879),
    [sym_astro_clock] = ACTIONS(1879),
    [sym_set_outdoor_temperature] = ACTIONS(1879),
    [sym_set_virtual_level_count] = ACTIONS(1879),
    [sym_set_virtual_channel_count] = ACTIONS(1879),
    [sym_set_virtual_port_count] = ACTIONS(1879),
    [sym__wc_encode_int] = ACTIONS(1879),
    [sym__wc_decode_int] = ACTIONS(1879),
    [sym__wc_int] = ACTIONS(1879),
    [sym__wc_to_ch_int] = ACTIONS(1879),
    [sym__ch_to_wc_int] = ACTIONS(1879),
    [sym__wc_find_string_int] = ACTIONS(1879),
    [sym__wc_left_string_int] = ACTIONS(1879),
    [sym__wc_lower_string_int] = ACTIONS(1879),
    [sym__wc_mid_string_int] = ACTIONS(1879),
    [sym__wc_remove_string_int] = ACTIONS(1879),
    [sym__wc_right_string_int] = ACTIONS(1879),
    [sym__wc_upper_string_int] = ACTIONS(1879),
    [sym__wc_compare_string_int] = ACTIONS(1879),
    [sym__wc_get_buffer_char_int] = ACTIONS(1879),
    [sym__wc_get_buffer_string_int] = ACTIONS(1879),
    [sym__wc_concat_string_int] = ACTIONS(1879),
    [sym__wc_file_open_int] = ACTIONS(1879),
    [sym__wc_file_close_int] = ACTIONS(1879),
    [sym__wc_file_read_int] = ACTIONS(1879),
    [sym__wc_file_read_line_int] = ACTIONS(1879),
    [sym__wc_file_write_int] = ACTIONS(1879),
    [sym__wc_file_write_line_int] = ACTIONS(1879),
    [aux_sym_preproc_include_token1] = ACTIONS(1879),
    [aux_sym_preproc_define_token1] = ACTIONS(1881),
    [aux_sym_preproc_warn_token1] = ACTIONS(1881),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1881),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1881),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1881),
    [anon_sym_LBRACK] = ACTIONS(1881),
    [anon_sym_LBRACE] = ACTIONS(1881),
    [anon_sym_RBRACE] = ACTIONS(1881),
    [anon_sym_LPAREN] = ACTIONS(1879),
    [anon_sym_BANG] = ACTIONS(1881),
    [anon_sym_TILDE] = ACTIONS(1881),
    [anon_sym_DASH] = ACTIONS(1879),
    [anon_sym_PLUS] = ACTIONS(1879),
    [anon_sym_DASH_DASH] = ACTIONS(1881),
    [anon_sym_PLUS_PLUS] = ACTIONS(1881),
    [anon_sym_DQUOTE] = ACTIONS(1881),
    [anon_sym_SQUOTE] = ACTIONS(1881),
    [sym_number_literal] = ACTIONS(1881),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(481)] = {
    [sym_identifier] = ACTIONS(1667),
    [sym_define_call_keyword] = ACTIONS(1667),
    [sym_define_function_keyword] = ACTIONS(1667),
    [sym_define_library_function_keyword] = ACTIONS(1667),
    [sym_define_module_keyword] = ACTIONS(1667),
    [sym_char_keyword] = ACTIONS(1667),
    [sym_widechar_keyword] = ACTIONS(1667),
    [sym_integer_keyword] = ACTIONS(1667),
    [sym_sinteger_keyword] = ACTIONS(1667),
    [sym_long_keyword] = ACTIONS(1667),
    [sym_slong_keyword] = ACTIONS(1667),
    [sym_float_keyword] = ACTIONS(1667),
    [sym_double_keyword] = ACTIONS(1667),
    [sym_variant_keyword] = ACTIONS(1667),
    [sym_variantarray_keyword] = ACTIONS(1667),
    [sym_local_var_keyword] = ACTIONS(1667),
    [sym_stack_var_keyword] = ACTIONS(1667),
    [sym_constant_keyword] = ACTIONS(1667),
    [sym_volatile_keyword] = ACTIONS(1667),
    [sym_non_volatile_keyword] = ACTIONS(1667),
    [sym_persistent_keyword] = ACTIONS(1667),
    [sym_dev_keyword] = ACTIONS(1667),
    [sym_devlev_keyword] = ACTIONS(1667),
    [sym_devchan_keyword] = ACTIONS(1667),
    [sym_if_keyword] = ACTIONS(1667),
    [sym_else_keyword] = ACTIONS(1667),
    [sym_switch_keyword] = ACTIONS(1667),
    [sym_case_keyword] = ACTIONS(1667),
    [sym_default_keyword] = ACTIONS(1667),
    [sym_while_keyword] = ACTIONS(1667),
    [sym_for_keyword] = ACTIONS(1667),
    [sym_break_keyword] = ACTIONS(1667),
    [sym_continue_keyword] = ACTIONS(1667),
    [sym_return_keyword] = ACTIONS(1667),
    [sym_select_keyword] = ACTIONS(1667),
    [sym_active_keyword] = ACTIONS(1667),
    [sym_struct_keyword] = ACTIONS(1667),
    [sym_structure_keyword] = ACTIONS(1667),
    [aux_sym_on_keyword_token1] = ACTIONS(1667),
    [aux_sym_off_keyword_token1] = ACTIONS(1667),
    [sym_send_level_keyword] = ACTIONS(1667),
    [sym_send_string_keyword] = ACTIONS(1667),
    [sym_send_command_keyword] = ACTIONS(1667),
    [sym_clear_buffer_keyword] = ACTIONS(1667),
    [sym_create_buffer_keyword] = ACTIONS(1667),
    [sym_create_multi_buffer_keyword] = ACTIONS(1667),
    [sym_call_keyword] = ACTIONS(1667),
    [sym_system_call_keyword] = ACTIONS(1667),
    [sym_devchan_to_keyword] = ACTIONS(1667),
    [sym_devchan_min_to_keyword] = ACTIONS(1667),
    [sym_devchan_total_off_keyword] = ACTIONS(1667),
    [sym_devchan_pulse_keyword] = ACTIONS(1667),
    [sym_wait_keyword] = ACTIONS(1667),
    [sym_cancel_wait_keyword] = ACTIONS(1667),
    [sym_wait_until_keyword] = ACTIONS(1667),
    [sym_cancel_wait_until_keyword] = ACTIONS(1667),
    [sym_cancel_all_wait_keyword] = ACTIONS(1667),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1667),
    [sym_not] = ACTIONS(1667),
    [sym_bnot] = ACTIONS(1667),
    [sym___file__] = ACTIONS(1667),
    [sym___date__] = ACTIONS(1667),
    [sym___ldate__] = ACTIONS(1667),
    [sym___time__] = ACTIONS(1667),
    [sym___line__] = ACTIONS(1667),
    [sym___name__] = ACTIONS(1667),
    [sym___netlinx__] = ACTIONS(1667),
    [sym_netlinx_axi_version] = ACTIONS(1667),
    [sym_url_flg_tcp] = ACTIONS(1667),
    [sym_url_flg_accinfopresent] = ACTIONS(1667),
    [sym_url_flg_temp] = ACTIONS(1667),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1667),
    [sym_url_flg_stat_mask] = ACTIONS(1667),
    [sym_url_flg_stat_lookup] = ACTIONS(1667),
    [sym_url_flg_stat_connecting] = ACTIONS(1667),
    [sym_url_flg_stat_waiting] = ACTIONS(1667),
    [sym_url_flg_stat_connected] = ACTIONS(1667),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1667),
    [sym_first_virtual_device] = ACTIONS(1667),
    [sym_first_local_port] = ACTIONS(1667),
    [sym_dynamic_virtual_device] = ACTIONS(1667),
    [sym_timeline_once] = ACTIONS(1667),
    [sym_timeline_repeat] = ACTIONS(1667),
    [sym_timeline_absolute] = ACTIONS(1667),
    [sym_timeline_relative] = ACTIONS(1667),
    [sym_do_push_timed_infinite] = ACTIONS(1667),
    [sym_source_type_no_address] = ACTIONS(1667),
    [sym_source_type_neuron_id] = ACTIONS(1667),
    [sym_source_type_ip_address] = ACTIONS(1667),
    [sym_source_type_axlink] = ACTIONS(1667),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1667),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1667),
    [sym_source_type_ip_socket_address] = ACTIONS(1667),
    [sym_source_type_rs232] = ACTIONS(1667),
    [sym_source_type_internal] = ACTIONS(1667),
    [sym_source_type_mac_address] = ACTIONS(1667),
    [sym_source_type_ipv4_port] = ACTIONS(1667),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1667),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1667),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1667),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1667),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1667),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1667),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1667),
    [sym_true] = ACTIONS(1667),
    [sym_false] = ACTIONS(1667),
    [sym_file_read_only] = ACTIONS(1667),
    [sym_file_rw_new] = ACTIONS(1667),
    [sym_file_rw_append] = ACTIONS(1667),
    [sym_ip_tcp] = ACTIONS(1667),
    [sym_ip_udp] = ACTIONS(1667),
    [sym_ip_udp_2way] = ACTIONS(1667),
    [sym_xml_encode_types] = ACTIONS(1667),
    [sym_xml_encode_char_as_list] = ACTIONS(1667),
    [sym_xml_encode_le] = ACTIONS(1667),
    [sym_xml_decode_types] = ACTIONS(1667),
    [sym_xml_decode_no_preserve] = ACTIONS(1667),
    [sym_duet_dev_type_display_device] = ACTIONS(1667),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1667),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1667),
    [sym_duet_dev_type_io_device] = ACTIONS(1667),
    [sym_duet_dev_type_relay_device] = ACTIONS(1667),
    [sym_duet_dev_type_ups] = ACTIONS(1667),
    [sym_duet_dev_type_amplifier] = ACTIONS(1667),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1667),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1667),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1667),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1667),
    [sym_duet_dev_type_camera] = ACTIONS(1667),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1667),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1667),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1667),
    [sym_duet_dev_type_dss] = ACTIONS(1667),
    [sym_duet_dev_type_dvr] = ACTIONS(1667),
    [sym_duet_dev_type_disc_device] = ACTIONS(1667),
    [sym_duet_dev_type_document_camera] = ACTIONS(1667),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1667),
    [sym_duet_dev_type_hvac] = ACTIONS(1667),
    [sym_duet_dev_type_keypad] = ACTIONS(1667),
    [sym_duet_dev_type_light] = ACTIONS(1667),
    [sym_duet_dev_type_monitor] = ACTIONS(1667),
    [sym_duet_dev_type_motor] = ACTIONS(1667),
    [sym_duet_dev_type_multi_window] = ACTIONS(1667),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1667),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1667),
    [sym_duet_dev_type_receiver] = ACTIONS(1667),
    [sym_duet_dev_type_security_system] = ACTIONS(1667),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1667),
    [sym_duet_dev_type_settop_box] = ACTIONS(1667),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1667),
    [sym_duet_dev_type_switcher] = ACTIONS(1667),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1667),
    [sym_duet_dev_type_tv] = ACTIONS(1667),
    [sym_duet_dev_type_utility] = ACTIONS(1667),
    [sym_duet_dev_type_vcr] = ACTIONS(1667),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1667),
    [sym_duet_dev_type_video_processor] = ACTIONS(1667),
    [sym_duet_dev_type_video_projector] = ACTIONS(1667),
    [sym_duet_dev_type_video_wall] = ACTIONS(1667),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1667),
    [sym_duet_dev_type_weather] = ACTIONS(1667),
    [sym_duet_dev_not_polled] = ACTIONS(1667),
    [sym_duet_dev_polled] = ACTIONS(1667),
    [sym_clkmgr_mode_network] = ACTIONS(1667),
    [sym_clkmgr_mode_standalone] = ACTIONS(1667),
    [sym_normal_standby] = ACTIONS(1667),
    [sym_normal_wake] = ACTIONS(1667),
    [sym_ip_multicast_ttl_option] = ACTIONS(1667),
    [sym_ip_ttl_subnet] = ACTIONS(1667),
    [sym_ip_ttl_site] = ACTIONS(1667),
    [sym_ip_ttl_region] = ACTIONS(1667),
    [sym_ip_ttl_continent] = ACTIONS(1667),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1667),
    [sym_ip_nodelay_on] = ACTIONS(1667),
    [sym_ip_nodelay_off] = ACTIONS(1667),
    [sym_amx_error] = ACTIONS(1667),
    [sym_amx_warning] = ACTIONS(1667),
    [sym_amx_info] = ACTIONS(1667),
    [sym_amx_debug] = ACTIONS(1667),
    [sym_netlinx_login_success] = ACTIONS(1667),
    [sym_netlinx_login_fail] = ACTIONS(1667),
    [sym_netlinx_logout] = ACTIONS(1667),
    [sym_null_str] = ACTIONS(1667),
    [sym_smtp_address] = ACTIONS(1667),
    [sym_smtp_port_number] = ACTIONS(1667),
    [sym_smtp_username] = ACTIONS(1667),
    [sym_smtp_password] = ACTIONS(1667),
    [sym_smtp_require_tls] = ACTIONS(1667),
    [sym_smtp_from] = ACTIONS(1667),
    [sym_smtp_tls_true] = ACTIONS(1667),
    [sym_smtp_tls_false] = ACTIONS(1667),
    [sym_listview_on_row_select_event] = ACTIONS(1667),
    [sym_data_structure_unknown] = ACTIONS(1667),
    [sym_data_structure_datafeed] = ACTIONS(1667),
    [sym_data_structure_datarecordset] = ACTIONS(1667),
    [sym_data_structure_datarecord] = ACTIONS(1667),
    [sym_data_structure_datafield] = ACTIONS(1667),
    [sym_data_max_content_field_count] = ACTIONS(1667),
    [sym_data_max_metadata_field_count] = ACTIONS(1667),
    [sym_data_max_id_length] = ACTIONS(1667),
    [sym_data_max_name_length] = ACTIONS(1667),
    [sym_data_max_description_length] = ACTIONS(1667),
    [sym_data_max_type_length] = ACTIONS(1667),
    [sym_data_max_format_length] = ACTIONS(1667),
    [sym_data_max_source_length] = ACTIONS(1667),
    [sym_data_max_value_length] = ACTIONS(1667),
    [sym_data_max_label_length] = ACTIONS(1667),
    [sym_data_max_url_length] = ACTIONS(1667),
    [sym_max_last_login_info_length] = ACTIONS(1667),
    [sym_data_type_unknown] = ACTIONS(1667),
    [sym_data_type_string] = ACTIONS(1667),
    [sym_data_type_datetime] = ACTIONS(1667),
    [sym_data_type_date] = ACTIONS(1667),
    [sym_data_type_time] = ACTIONS(1667),
    [sym_data_type_image] = ACTIONS(1667),
    [sym_data_format_url] = ACTIONS(1667),
    [sym_data_format_phone] = ACTIONS(1667),
    [sym_data_format_email] = ACTIONS(1667),
    [sym_data_format_iso8601] = ACTIONS(1667),
    [sym_auditlog_priv] = ACTIONS(1667),
    [sym_configuration_priv] = ACTIONS(1667),
    [sym_deviceconfig_priv] = ACTIONS(1667),
    [sym_ftp_priv] = ACTIONS(1667),
    [sym_http_priv] = ACTIONS(1667),
    [sym_networkconfig_priv] = ACTIONS(1667),
    [sym_programport_priv] = ACTIONS(1667),
    [sym_remoteui_priv] = ACTIONS(1667),
    [sym_securitycontrol_priv] = ACTIONS(1667),
    [sym_softwaremanagement_priv] = ACTIONS(1667),
    [sym_terminal_priv] = ACTIONS(1667),
    [sym_tpadmin_priv] = ACTIONS(1667),
    [sym_user1_priv] = ACTIONS(1667),
    [sym_user2_priv] = ACTIONS(1667),
    [sym_user3_priv] = ACTIONS(1667),
    [sym_user4_priv] = ACTIONS(1667),
    [sym_usermanagement_priv] = ACTIONS(1667),
    [sym_valid_account] = ACTIONS(1667),
    [sym_err_name_invalid] = ACTIONS(1667),
    [sym_err_password_invalid] = ACTIONS(1667),
    [sym_err_invalid_account] = ACTIONS(1667),
    [sym_err_no_authorization] = ACTIONS(1667),
    [sym_err_invalid_parameter] = ACTIONS(1667),
    [sym_err_user_locked_out] = ACTIONS(1667),
    [sym_err_user_account_expired] = ACTIONS(1667),
    [sym_err_general_error] = ACTIONS(1667),
    [sym_err_authentication_error] = ACTIONS(1667),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1667),
    [sym_tls_validate_certificate] = ACTIONS(1667),
    [sym_tdata] = ACTIONS(1667),
    [sym_tchannel] = ACTIONS(1667),
    [sym_tlevel] = ACTIONS(1667),
    [sym_tbutton] = ACTIONS(1667),
    [sym_ttimeline] = ACTIONS(1667),
    [sym_tcustom] = ACTIONS(1667),
    [sym_url_struct] = ACTIONS(1667),
    [sym_dns_struct] = ACTIONS(1667),
    [sym_ip_address_struct] = ACTIONS(1667),
    [sym_dev_info_struct] = ACTIONS(1667),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1667),
    [sym_data_feed] = ACTIONS(1667),
    [sym_data_field] = ACTIONS(1667),
    [sym_data_record] = ACTIONS(1667),
    [sym_wc_data_feed] = ACTIONS(1667),
    [sym_wc_data_field] = ACTIONS(1667),
    [sym_wc_data_record] = ACTIONS(1667),
    [sym_last_login_info] = ACTIONS(1667),
    [sym____reserved___] = ACTIONS(1667),
    [sym_date] = ACTIONS(1667),
    [sym_day] = ACTIONS(1667),
    [sym_time] = ACTIONS(1667),
    [sym_ldate] = ACTIONS(1667),
    [sym_push_device] = ACTIONS(1667),
    [sym_push_channel] = ACTIONS(1667),
    [sym_push_devchan] = ACTIONS(1667),
    [sym_release_device] = ACTIONS(1667),
    [sym_release_channel] = ACTIONS(1667),
    [sym_release_devchan] = ACTIONS(1667),
    [sym_master_slot] = ACTIONS(1667),
    [sym_get_pulse_time] = ACTIONS(1667),
    [sym_get_timer] = ACTIONS(1667),
    [sym_master_sn] = ACTIONS(1667),
    [sym_system_number] = ACTIONS(1667),
    [sym_button] = ACTIONS(1667),
    [sym_dv_channel] = ACTIONS(1667),
    [sym_data] = ACTIONS(1667),
    [sym_level] = ACTIONS(1667),
    [sym_timeline] = ACTIONS(1667),
    [sym_channel] = ACTIONS(1667),
    [sym_custom] = ACTIONS(1667),
    [sym_atoi] = ACTIONS(1667),
    [sym_atol] = ACTIONS(1667),
    [sym_atof] = ACTIONS(1667),
    [sym_itoa] = ACTIONS(1667),
    [sym_itohex] = ACTIONS(1667),
    [sym_hextoi] = ACTIONS(1667),
    [sym_ftoa] = ACTIONS(1667),
    [sym_find_string] = ACTIONS(1667),
    [sym_left_string] = ACTIONS(1667),
    [sym_right_string] = ACTIONS(1667),
    [sym_mid_string] = ACTIONS(1667),
    [sym_remove_string] = ACTIONS(1667),
    [sym_get_buffer_string] = ACTIONS(1667),
    [sym_get_buffer_char] = ACTIONS(1667),
    [sym_get_multi_buffer_string] = ACTIONS(1667),
    [sym_length_string] = ACTIONS(1667),
    [sym_max_length_string] = ACTIONS(1667),
    [sym_set_length_string] = ACTIONS(1667),
    [sym_compare_string] = ACTIONS(1667),
    [sym_upper_string] = ACTIONS(1667),
    [sym_lower_string] = ACTIONS(1667),
    [sym_get_last] = ACTIONS(1667),
    [sym_date_to_day] = ACTIONS(1667),
    [sym_date_to_month] = ACTIONS(1667),
    [sym_date_to_year] = ACTIONS(1667),
    [sym_time_to_hour] = ACTIONS(1667),
    [sym_time_to_minute] = ACTIONS(1667),
    [sym_time_to_second] = ACTIONS(1667),
    [sym_day_of_week] = ACTIONS(1667),
    [sym_set_pulse_time] = ACTIONS(1667),
    [sym_set_timer] = ACTIONS(1667),
    [sym_file_open] = ACTIONS(1667),
    [sym_file_close] = ACTIONS(1667),
    [sym_file_read] = ACTIONS(1667),
    [sym_file_read_line] = ACTIONS(1667),
    [sym_file_write] = ACTIONS(1667),
    [sym_file_write_line] = ACTIONS(1667),
    [sym_file_copy] = ACTIONS(1667),
    [sym_file_delete] = ACTIONS(1667),
    [sym_file_rename] = ACTIONS(1667),
    [sym_file_dir] = ACTIONS(1667),
    [sym_file_getdir] = ACTIONS(1667),
    [sym_file_setdir] = ACTIONS(1667),
    [sym_file_create_dir] = ACTIONS(1667),
    [sym_file_remove_dir] = ACTIONS(1667),
    [sym_file_seek] = ACTIONS(1667),
    [sym_length_array] = ACTIONS(1667),
    [sym_max_length_array] = ACTIONS(1667),
    [sym_set_length_array] = ACTIONS(1667),
    [sym_ip_client_open] = ACTIONS(1667),
    [sym_ip_client_close] = ACTIONS(1667),
    [sym_ip_server_open] = ACTIONS(1667),
    [sym_ip_server_close] = ACTIONS(1667),
    [sym_ip_bound_client_open] = ACTIONS(1667),
    [sym_ip_mc_server_open] = ACTIONS(1667),
    [sym_ip_set_option] = ACTIONS(1667),
    [sym_ssh_client_open] = ACTIONS(1667),
    [sym_ssh_client_close] = ACTIONS(1667),
    [sym_tls_client_open] = ACTIONS(1667),
    [sym_tls_client_close] = ACTIONS(1667),
    [sym_do_push] = ACTIONS(1667),
    [sym_do_push_timed] = ACTIONS(1667),
    [sym_do_release] = ACTIONS(1667),
    [sym_device_id] = ACTIONS(1667),
    [sym_device_info] = ACTIONS(1667),
    [sym_device_id_string] = ACTIONS(1667),
    [sym_device_standby] = ACTIONS(1667),
    [sym_device_wake] = ACTIONS(1667),
    [sym_timeline_create] = ACTIONS(1667),
    [sym_timeline_kill] = ACTIONS(1667),
    [sym_timeline_reload] = ACTIONS(1667),
    [sym_timeline_pause] = ACTIONS(1667),
    [sym_timeline_restart] = ACTIONS(1667),
    [sym_timeline_set] = ACTIONS(1667),
    [sym_timeline_get] = ACTIONS(1667),
    [sym_timeline_active] = ACTIONS(1667),
    [sym_timeline_dynamic_id] = ACTIONS(1667),
    [sym_variable_to_string] = ACTIONS(1667),
    [sym_string_to_variable] = ACTIONS(1667),
    [sym_length_variable_to_string] = ACTIONS(1667),
    [sym_type_cast] = ACTIONS(1667),
    [sym_variable_to_xml] = ACTIONS(1667),
    [sym_xml_to_variable] = ACTIONS(1667),
    [sym_length_variable_to_xml] = ACTIONS(1667),
    [sym_redirect_string] = ACTIONS(1667),
    [sym_random_number] = ACTIONS(1667),
    [sym_get_unique_id] = ACTIONS(1667),
    [sym_get_system_number] = ACTIONS(1667),
    [sym_set_system_number] = ACTIONS(1667),
    [sym_get_serial_number] = ACTIONS(1667),
    [sym_get_master_build] = ACTIONS(1667),
    [sym_set_validation_code] = ACTIONS(1667),
    [sym_reboot] = ACTIONS(1667),
    [sym_load_duet_module] = ACTIONS(1667),
    [sym_unload_duet_module] = ACTIONS(1667),
    [sym_duet_mem_size_set] = ACTIONS(1667),
    [sym_duet_mem_size_get] = ACTIONS(1667),
    [sym_dynamic_application_device] = ACTIONS(1667),
    [sym_dynamic_polled_port] = ACTIONS(1667),
    [sym_static_port_binding] = ACTIONS(1667),
    [sym_static_ip_binding] = ACTIONS(1667),
    [sym_get_dns_list] = ACTIONS(1667),
    [sym_set_dns_list] = ACTIONS(1667),
    [sym_get_ip_address] = ACTIONS(1667),
    [sym_set_ip_address] = ACTIONS(1667),
    [sym_get_url_list] = ACTIONS(1667),
    [sym_add_url_entry] = ACTIONS(1667),
    [sym_delete_url_entry] = ACTIONS(1667),
    [sym_abs_value] = ACTIONS(1667),
    [sym_max_value] = ACTIONS(1667),
    [sym_min_value] = ACTIONS(1667),
    [sym_exp_value] = ACTIONS(1667),
    [sym_log_value] = ACTIONS(1667),
    [sym_log10_value] = ACTIONS(1667),
    [sym_power_value] = ACTIONS(1667),
    [sym_sqrt_value] = ACTIONS(1667),
    [sym_raw_be] = ACTIONS(1667),
    [sym_raw_le] = ACTIONS(1667),
    [sym_internal_queue_size_set] = ACTIONS(1667),
    [sym_internal_queue_size_get] = ACTIONS(1667),
    [sym_internal_threshold_set] = ACTIONS(1667),
    [sym_internal_threshold_get] = ACTIONS(1667),
    [sym_rebuilt_event] = ACTIONS(1667),
    [sym_do_custom_event] = ACTIONS(1667),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1667),
    [sym_clkmgr_set_clk_source] = ACTIONS(1667),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1667),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1667),
    [sym_clkmgr_get_timezone] = ACTIONS(1667),
    [sym_clkmgr_set_timezone] = ACTIONS(1667),
    [sym_clkmgr_get_resync_period] = ACTIONS(1667),
    [sym_clkmgr_set_resync_period] = ACTIONS(1667),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1667),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1667),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1667),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1667),
    [sym_clkmgr_get_timeservers] = ACTIONS(1667),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1667),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1667),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1667),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1667),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1667),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1667),
    [sym_set_log_level] = ACTIONS(1667),
    [sym_get_log_level] = ACTIONS(1667),
    [sym_amx_log] = ACTIONS(1667),
    [sym_smtp_server_config_set] = ACTIONS(1667),
    [sym_smtp_server_config_get] = ACTIONS(1667),
    [sym_smtp_send] = ACTIONS(1667),
    [sym_validate_netlinx_account] = ACTIONS(1667),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1667),
    [sym_authenticate_certificate] = ACTIONS(1667),
    [sym_audit_netlinx_session_event] = ACTIONS(1667),
    [sym_audit_netlinx_generic_event] = ACTIONS(1667),
    [sym_get_available_flash_disk_space] = ACTIONS(1667),
    [sym_get_max_flash_disk_space] = ACTIONS(1667),
    [sym_data_create_feed] = ACTIONS(1667),
    [sym_data_delete_feed] = ACTIONS(1667),
    [sym_data_publish_feed] = ACTIONS(1667),
    [sym_data_get_published_feed] = ACTIONS(1667),
    [sym_data_add_record] = ACTIONS(1667),
    [sym_data_get_event_record] = ACTIONS(1667),
    [sym__wc_data_create_feed] = ACTIONS(1667),
    [sym__wc_data_add_record] = ACTIONS(1667),
    [sym__wc_data_get_event_record] = ACTIONS(1667),
    [sym_format] = ACTIONS(1667),
    [sym_astro_clock] = ACTIONS(1667),
    [sym_set_outdoor_temperature] = ACTIONS(1667),
    [sym_set_virtual_level_count] = ACTIONS(1667),
    [sym_set_virtual_channel_count] = ACTIONS(1667),
    [sym_set_virtual_port_count] = ACTIONS(1667),
    [sym__wc_encode_int] = ACTIONS(1667),
    [sym__wc_decode_int] = ACTIONS(1667),
    [sym__wc_int] = ACTIONS(1667),
    [sym__wc_to_ch_int] = ACTIONS(1667),
    [sym__ch_to_wc_int] = ACTIONS(1667),
    [sym__wc_find_string_int] = ACTIONS(1667),
    [sym__wc_left_string_int] = ACTIONS(1667),
    [sym__wc_lower_string_int] = ACTIONS(1667),
    [sym__wc_mid_string_int] = ACTIONS(1667),
    [sym__wc_remove_string_int] = ACTIONS(1667),
    [sym__wc_right_string_int] = ACTIONS(1667),
    [sym__wc_upper_string_int] = ACTIONS(1667),
    [sym__wc_compare_string_int] = ACTIONS(1667),
    [sym__wc_get_buffer_char_int] = ACTIONS(1667),
    [sym__wc_get_buffer_string_int] = ACTIONS(1667),
    [sym__wc_concat_string_int] = ACTIONS(1667),
    [sym__wc_file_open_int] = ACTIONS(1667),
    [sym__wc_file_close_int] = ACTIONS(1667),
    [sym__wc_file_read_int] = ACTIONS(1667),
    [sym__wc_file_read_line_int] = ACTIONS(1667),
    [sym__wc_file_write_int] = ACTIONS(1667),
    [sym__wc_file_write_line_int] = ACTIONS(1667),
    [aux_sym_preproc_include_token1] = ACTIONS(1667),
    [aux_sym_preproc_define_token1] = ACTIONS(1665),
    [aux_sym_preproc_warn_token1] = ACTIONS(1665),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1665),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1665),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1665),
    [anon_sym_LBRACK] = ACTIONS(1665),
    [anon_sym_LBRACE] = ACTIONS(1665),
    [anon_sym_RBRACE] = ACTIONS(1665),
    [anon_sym_LPAREN] = ACTIONS(1667),
    [anon_sym_BANG] = ACTIONS(1665),
    [anon_sym_TILDE] = ACTIONS(1665),
    [anon_sym_DASH] = ACTIONS(1667),
    [anon_sym_PLUS] = ACTIONS(1667),
    [anon_sym_DASH_DASH] = ACTIONS(1665),
    [anon_sym_PLUS_PLUS] = ACTIONS(1665),
    [anon_sym_DQUOTE] = ACTIONS(1665),
    [anon_sym_SQUOTE] = ACTIONS(1665),
    [sym_number_literal] = ACTIONS(1665),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(482)] = {
    [sym_identifier] = ACTIONS(1765),
    [sym_define_call_keyword] = ACTIONS(1765),
    [sym_define_function_keyword] = ACTIONS(1765),
    [sym_define_library_function_keyword] = ACTIONS(1765),
    [sym_define_module_keyword] = ACTIONS(1765),
    [sym_char_keyword] = ACTIONS(1765),
    [sym_widechar_keyword] = ACTIONS(1765),
    [sym_integer_keyword] = ACTIONS(1765),
    [sym_sinteger_keyword] = ACTIONS(1765),
    [sym_long_keyword] = ACTIONS(1765),
    [sym_slong_keyword] = ACTIONS(1765),
    [sym_float_keyword] = ACTIONS(1765),
    [sym_double_keyword] = ACTIONS(1765),
    [sym_variant_keyword] = ACTIONS(1765),
    [sym_variantarray_keyword] = ACTIONS(1765),
    [sym_local_var_keyword] = ACTIONS(1765),
    [sym_stack_var_keyword] = ACTIONS(1765),
    [sym_constant_keyword] = ACTIONS(1765),
    [sym_volatile_keyword] = ACTIONS(1765),
    [sym_non_volatile_keyword] = ACTIONS(1765),
    [sym_persistent_keyword] = ACTIONS(1765),
    [sym_dev_keyword] = ACTIONS(1765),
    [sym_devlev_keyword] = ACTIONS(1765),
    [sym_devchan_keyword] = ACTIONS(1765),
    [sym_if_keyword] = ACTIONS(1765),
    [sym_else_keyword] = ACTIONS(1765),
    [sym_switch_keyword] = ACTIONS(1765),
    [sym_case_keyword] = ACTIONS(1765),
    [sym_default_keyword] = ACTIONS(1765),
    [sym_while_keyword] = ACTIONS(1765),
    [sym_for_keyword] = ACTIONS(1765),
    [sym_break_keyword] = ACTIONS(1765),
    [sym_continue_keyword] = ACTIONS(1765),
    [sym_return_keyword] = ACTIONS(1765),
    [sym_select_keyword] = ACTIONS(1765),
    [sym_active_keyword] = ACTIONS(1765),
    [sym_struct_keyword] = ACTIONS(1765),
    [sym_structure_keyword] = ACTIONS(1765),
    [aux_sym_on_keyword_token1] = ACTIONS(1765),
    [aux_sym_off_keyword_token1] = ACTIONS(1765),
    [sym_send_level_keyword] = ACTIONS(1765),
    [sym_send_string_keyword] = ACTIONS(1765),
    [sym_send_command_keyword] = ACTIONS(1765),
    [sym_clear_buffer_keyword] = ACTIONS(1765),
    [sym_create_buffer_keyword] = ACTIONS(1765),
    [sym_create_multi_buffer_keyword] = ACTIONS(1765),
    [sym_call_keyword] = ACTIONS(1765),
    [sym_system_call_keyword] = ACTIONS(1765),
    [sym_devchan_to_keyword] = ACTIONS(1765),
    [sym_devchan_min_to_keyword] = ACTIONS(1765),
    [sym_devchan_total_off_keyword] = ACTIONS(1765),
    [sym_devchan_pulse_keyword] = ACTIONS(1765),
    [sym_wait_keyword] = ACTIONS(1765),
    [sym_cancel_wait_keyword] = ACTIONS(1765),
    [sym_wait_until_keyword] = ACTIONS(1765),
    [sym_cancel_wait_until_keyword] = ACTIONS(1765),
    [sym_cancel_all_wait_keyword] = ACTIONS(1765),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1765),
    [sym_not] = ACTIONS(1765),
    [sym_bnot] = ACTIONS(1765),
    [sym___file__] = ACTIONS(1765),
    [sym___date__] = ACTIONS(1765),
    [sym___ldate__] = ACTIONS(1765),
    [sym___time__] = ACTIONS(1765),
    [sym___line__] = ACTIONS(1765),
    [sym___name__] = ACTIONS(1765),
    [sym___netlinx__] = ACTIONS(1765),
    [sym_netlinx_axi_version] = ACTIONS(1765),
    [sym_url_flg_tcp] = ACTIONS(1765),
    [sym_url_flg_accinfopresent] = ACTIONS(1765),
    [sym_url_flg_temp] = ACTIONS(1765),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1765),
    [sym_url_flg_stat_mask] = ACTIONS(1765),
    [sym_url_flg_stat_lookup] = ACTIONS(1765),
    [sym_url_flg_stat_connecting] = ACTIONS(1765),
    [sym_url_flg_stat_waiting] = ACTIONS(1765),
    [sym_url_flg_stat_connected] = ACTIONS(1765),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1765),
    [sym_first_virtual_device] = ACTIONS(1765),
    [sym_first_local_port] = ACTIONS(1765),
    [sym_dynamic_virtual_device] = ACTIONS(1765),
    [sym_timeline_once] = ACTIONS(1765),
    [sym_timeline_repeat] = ACTIONS(1765),
    [sym_timeline_absolute] = ACTIONS(1765),
    [sym_timeline_relative] = ACTIONS(1765),
    [sym_do_push_timed_infinite] = ACTIONS(1765),
    [sym_source_type_no_address] = ACTIONS(1765),
    [sym_source_type_neuron_id] = ACTIONS(1765),
    [sym_source_type_ip_address] = ACTIONS(1765),
    [sym_source_type_axlink] = ACTIONS(1765),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1765),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1765),
    [sym_source_type_ip_socket_address] = ACTIONS(1765),
    [sym_source_type_rs232] = ACTIONS(1765),
    [sym_source_type_internal] = ACTIONS(1765),
    [sym_source_type_mac_address] = ACTIONS(1765),
    [sym_source_type_ipv4_port] = ACTIONS(1765),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1765),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1765),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1765),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1765),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1765),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1765),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1765),
    [sym_true] = ACTIONS(1765),
    [sym_false] = ACTIONS(1765),
    [sym_file_read_only] = ACTIONS(1765),
    [sym_file_rw_new] = ACTIONS(1765),
    [sym_file_rw_append] = ACTIONS(1765),
    [sym_ip_tcp] = ACTIONS(1765),
    [sym_ip_udp] = ACTIONS(1765),
    [sym_ip_udp_2way] = ACTIONS(1765),
    [sym_xml_encode_types] = ACTIONS(1765),
    [sym_xml_encode_char_as_list] = ACTIONS(1765),
    [sym_xml_encode_le] = ACTIONS(1765),
    [sym_xml_decode_types] = ACTIONS(1765),
    [sym_xml_decode_no_preserve] = ACTIONS(1765),
    [sym_duet_dev_type_display_device] = ACTIONS(1765),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1765),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1765),
    [sym_duet_dev_type_io_device] = ACTIONS(1765),
    [sym_duet_dev_type_relay_device] = ACTIONS(1765),
    [sym_duet_dev_type_ups] = ACTIONS(1765),
    [sym_duet_dev_type_amplifier] = ACTIONS(1765),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1765),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1765),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1765),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1765),
    [sym_duet_dev_type_camera] = ACTIONS(1765),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1765),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1765),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1765),
    [sym_duet_dev_type_dss] = ACTIONS(1765),
    [sym_duet_dev_type_dvr] = ACTIONS(1765),
    [sym_duet_dev_type_disc_device] = ACTIONS(1765),
    [sym_duet_dev_type_document_camera] = ACTIONS(1765),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1765),
    [sym_duet_dev_type_hvac] = ACTIONS(1765),
    [sym_duet_dev_type_keypad] = ACTIONS(1765),
    [sym_duet_dev_type_light] = ACTIONS(1765),
    [sym_duet_dev_type_monitor] = ACTIONS(1765),
    [sym_duet_dev_type_motor] = ACTIONS(1765),
    [sym_duet_dev_type_multi_window] = ACTIONS(1765),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1765),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1765),
    [sym_duet_dev_type_receiver] = ACTIONS(1765),
    [sym_duet_dev_type_security_system] = ACTIONS(1765),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1765),
    [sym_duet_dev_type_settop_box] = ACTIONS(1765),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1765),
    [sym_duet_dev_type_switcher] = ACTIONS(1765),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1765),
    [sym_duet_dev_type_tv] = ACTIONS(1765),
    [sym_duet_dev_type_utility] = ACTIONS(1765),
    [sym_duet_dev_type_vcr] = ACTIONS(1765),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1765),
    [sym_duet_dev_type_video_processor] = ACTIONS(1765),
    [sym_duet_dev_type_video_projector] = ACTIONS(1765),
    [sym_duet_dev_type_video_wall] = ACTIONS(1765),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1765),
    [sym_duet_dev_type_weather] = ACTIONS(1765),
    [sym_duet_dev_not_polled] = ACTIONS(1765),
    [sym_duet_dev_polled] = ACTIONS(1765),
    [sym_clkmgr_mode_network] = ACTIONS(1765),
    [sym_clkmgr_mode_standalone] = ACTIONS(1765),
    [sym_normal_standby] = ACTIONS(1765),
    [sym_normal_wake] = ACTIONS(1765),
    [sym_ip_multicast_ttl_option] = ACTIONS(1765),
    [sym_ip_ttl_subnet] = ACTIONS(1765),
    [sym_ip_ttl_site] = ACTIONS(1765),
    [sym_ip_ttl_region] = ACTIONS(1765),
    [sym_ip_ttl_continent] = ACTIONS(1765),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1765),
    [sym_ip_nodelay_on] = ACTIONS(1765),
    [sym_ip_nodelay_off] = ACTIONS(1765),
    [sym_amx_error] = ACTIONS(1765),
    [sym_amx_warning] = ACTIONS(1765),
    [sym_amx_info] = ACTIONS(1765),
    [sym_amx_debug] = ACTIONS(1765),
    [sym_netlinx_login_success] = ACTIONS(1765),
    [sym_netlinx_login_fail] = ACTIONS(1765),
    [sym_netlinx_logout] = ACTIONS(1765),
    [sym_null_str] = ACTIONS(1765),
    [sym_smtp_address] = ACTIONS(1765),
    [sym_smtp_port_number] = ACTIONS(1765),
    [sym_smtp_username] = ACTIONS(1765),
    [sym_smtp_password] = ACTIONS(1765),
    [sym_smtp_require_tls] = ACTIONS(1765),
    [sym_smtp_from] = ACTIONS(1765),
    [sym_smtp_tls_true] = ACTIONS(1765),
    [sym_smtp_tls_false] = ACTIONS(1765),
    [sym_listview_on_row_select_event] = ACTIONS(1765),
    [sym_data_structure_unknown] = ACTIONS(1765),
    [sym_data_structure_datafeed] = ACTIONS(1765),
    [sym_data_structure_datarecordset] = ACTIONS(1765),
    [sym_data_structure_datarecord] = ACTIONS(1765),
    [sym_data_structure_datafield] = ACTIONS(1765),
    [sym_data_max_content_field_count] = ACTIONS(1765),
    [sym_data_max_metadata_field_count] = ACTIONS(1765),
    [sym_data_max_id_length] = ACTIONS(1765),
    [sym_data_max_name_length] = ACTIONS(1765),
    [sym_data_max_description_length] = ACTIONS(1765),
    [sym_data_max_type_length] = ACTIONS(1765),
    [sym_data_max_format_length] = ACTIONS(1765),
    [sym_data_max_source_length] = ACTIONS(1765),
    [sym_data_max_value_length] = ACTIONS(1765),
    [sym_data_max_label_length] = ACTIONS(1765),
    [sym_data_max_url_length] = ACTIONS(1765),
    [sym_max_last_login_info_length] = ACTIONS(1765),
    [sym_data_type_unknown] = ACTIONS(1765),
    [sym_data_type_string] = ACTIONS(1765),
    [sym_data_type_datetime] = ACTIONS(1765),
    [sym_data_type_date] = ACTIONS(1765),
    [sym_data_type_time] = ACTIONS(1765),
    [sym_data_type_image] = ACTIONS(1765),
    [sym_data_format_url] = ACTIONS(1765),
    [sym_data_format_phone] = ACTIONS(1765),
    [sym_data_format_email] = ACTIONS(1765),
    [sym_data_format_iso8601] = ACTIONS(1765),
    [sym_auditlog_priv] = ACTIONS(1765),
    [sym_configuration_priv] = ACTIONS(1765),
    [sym_deviceconfig_priv] = ACTIONS(1765),
    [sym_ftp_priv] = ACTIONS(1765),
    [sym_http_priv] = ACTIONS(1765),
    [sym_networkconfig_priv] = ACTIONS(1765),
    [sym_programport_priv] = ACTIONS(1765),
    [sym_remoteui_priv] = ACTIONS(1765),
    [sym_securitycontrol_priv] = ACTIONS(1765),
    [sym_softwaremanagement_priv] = ACTIONS(1765),
    [sym_terminal_priv] = ACTIONS(1765),
    [sym_tpadmin_priv] = ACTIONS(1765),
    [sym_user1_priv] = ACTIONS(1765),
    [sym_user2_priv] = ACTIONS(1765),
    [sym_user3_priv] = ACTIONS(1765),
    [sym_user4_priv] = ACTIONS(1765),
    [sym_usermanagement_priv] = ACTIONS(1765),
    [sym_valid_account] = ACTIONS(1765),
    [sym_err_name_invalid] = ACTIONS(1765),
    [sym_err_password_invalid] = ACTIONS(1765),
    [sym_err_invalid_account] = ACTIONS(1765),
    [sym_err_no_authorization] = ACTIONS(1765),
    [sym_err_invalid_parameter] = ACTIONS(1765),
    [sym_err_user_locked_out] = ACTIONS(1765),
    [sym_err_user_account_expired] = ACTIONS(1765),
    [sym_err_general_error] = ACTIONS(1765),
    [sym_err_authentication_error] = ACTIONS(1765),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1765),
    [sym_tls_validate_certificate] = ACTIONS(1765),
    [sym_tdata] = ACTIONS(1765),
    [sym_tchannel] = ACTIONS(1765),
    [sym_tlevel] = ACTIONS(1765),
    [sym_tbutton] = ACTIONS(1765),
    [sym_ttimeline] = ACTIONS(1765),
    [sym_tcustom] = ACTIONS(1765),
    [sym_url_struct] = ACTIONS(1765),
    [sym_dns_struct] = ACTIONS(1765),
    [sym_ip_address_struct] = ACTIONS(1765),
    [sym_dev_info_struct] = ACTIONS(1765),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1765),
    [sym_data_feed] = ACTIONS(1765),
    [sym_data_field] = ACTIONS(1765),
    [sym_data_record] = ACTIONS(1765),
    [sym_wc_data_feed] = ACTIONS(1765),
    [sym_wc_data_field] = ACTIONS(1765),
    [sym_wc_data_record] = ACTIONS(1765),
    [sym_last_login_info] = ACTIONS(1765),
    [sym____reserved___] = ACTIONS(1765),
    [sym_date] = ACTIONS(1765),
    [sym_day] = ACTIONS(1765),
    [sym_time] = ACTIONS(1765),
    [sym_ldate] = ACTIONS(1765),
    [sym_push_device] = ACTIONS(1765),
    [sym_push_channel] = ACTIONS(1765),
    [sym_push_devchan] = ACTIONS(1765),
    [sym_release_device] = ACTIONS(1765),
    [sym_release_channel] = ACTIONS(1765),
    [sym_release_devchan] = ACTIONS(1765),
    [sym_master_slot] = ACTIONS(1765),
    [sym_get_pulse_time] = ACTIONS(1765),
    [sym_get_timer] = ACTIONS(1765),
    [sym_master_sn] = ACTIONS(1765),
    [sym_system_number] = ACTIONS(1765),
    [sym_button] = ACTIONS(1765),
    [sym_dv_channel] = ACTIONS(1765),
    [sym_data] = ACTIONS(1765),
    [sym_level] = ACTIONS(1765),
    [sym_timeline] = ACTIONS(1765),
    [sym_channel] = ACTIONS(1765),
    [sym_custom] = ACTIONS(1765),
    [sym_atoi] = ACTIONS(1765),
    [sym_atol] = ACTIONS(1765),
    [sym_atof] = ACTIONS(1765),
    [sym_itoa] = ACTIONS(1765),
    [sym_itohex] = ACTIONS(1765),
    [sym_hextoi] = ACTIONS(1765),
    [sym_ftoa] = ACTIONS(1765),
    [sym_find_string] = ACTIONS(1765),
    [sym_left_string] = ACTIONS(1765),
    [sym_right_string] = ACTIONS(1765),
    [sym_mid_string] = ACTIONS(1765),
    [sym_remove_string] = ACTIONS(1765),
    [sym_get_buffer_string] = ACTIONS(1765),
    [sym_get_buffer_char] = ACTIONS(1765),
    [sym_get_multi_buffer_string] = ACTIONS(1765),
    [sym_length_string] = ACTIONS(1765),
    [sym_max_length_string] = ACTIONS(1765),
    [sym_set_length_string] = ACTIONS(1765),
    [sym_compare_string] = ACTIONS(1765),
    [sym_upper_string] = ACTIONS(1765),
    [sym_lower_string] = ACTIONS(1765),
    [sym_get_last] = ACTIONS(1765),
    [sym_date_to_day] = ACTIONS(1765),
    [sym_date_to_month] = ACTIONS(1765),
    [sym_date_to_year] = ACTIONS(1765),
    [sym_time_to_hour] = ACTIONS(1765),
    [sym_time_to_minute] = ACTIONS(1765),
    [sym_time_to_second] = ACTIONS(1765),
    [sym_day_of_week] = ACTIONS(1765),
    [sym_set_pulse_time] = ACTIONS(1765),
    [sym_set_timer] = ACTIONS(1765),
    [sym_file_open] = ACTIONS(1765),
    [sym_file_close] = ACTIONS(1765),
    [sym_file_read] = ACTIONS(1765),
    [sym_file_read_line] = ACTIONS(1765),
    [sym_file_write] = ACTIONS(1765),
    [sym_file_write_line] = ACTIONS(1765),
    [sym_file_copy] = ACTIONS(1765),
    [sym_file_delete] = ACTIONS(1765),
    [sym_file_rename] = ACTIONS(1765),
    [sym_file_dir] = ACTIONS(1765),
    [sym_file_getdir] = ACTIONS(1765),
    [sym_file_setdir] = ACTIONS(1765),
    [sym_file_create_dir] = ACTIONS(1765),
    [sym_file_remove_dir] = ACTIONS(1765),
    [sym_file_seek] = ACTIONS(1765),
    [sym_length_array] = ACTIONS(1765),
    [sym_max_length_array] = ACTIONS(1765),
    [sym_set_length_array] = ACTIONS(1765),
    [sym_ip_client_open] = ACTIONS(1765),
    [sym_ip_client_close] = ACTIONS(1765),
    [sym_ip_server_open] = ACTIONS(1765),
    [sym_ip_server_close] = ACTIONS(1765),
    [sym_ip_bound_client_open] = ACTIONS(1765),
    [sym_ip_mc_server_open] = ACTIONS(1765),
    [sym_ip_set_option] = ACTIONS(1765),
    [sym_ssh_client_open] = ACTIONS(1765),
    [sym_ssh_client_close] = ACTIONS(1765),
    [sym_tls_client_open] = ACTIONS(1765),
    [sym_tls_client_close] = ACTIONS(1765),
    [sym_do_push] = ACTIONS(1765),
    [sym_do_push_timed] = ACTIONS(1765),
    [sym_do_release] = ACTIONS(1765),
    [sym_device_id] = ACTIONS(1765),
    [sym_device_info] = ACTIONS(1765),
    [sym_device_id_string] = ACTIONS(1765),
    [sym_device_standby] = ACTIONS(1765),
    [sym_device_wake] = ACTIONS(1765),
    [sym_timeline_create] = ACTIONS(1765),
    [sym_timeline_kill] = ACTIONS(1765),
    [sym_timeline_reload] = ACTIONS(1765),
    [sym_timeline_pause] = ACTIONS(1765),
    [sym_timeline_restart] = ACTIONS(1765),
    [sym_timeline_set] = ACTIONS(1765),
    [sym_timeline_get] = ACTIONS(1765),
    [sym_timeline_active] = ACTIONS(1765),
    [sym_timeline_dynamic_id] = ACTIONS(1765),
    [sym_variable_to_string] = ACTIONS(1765),
    [sym_string_to_variable] = ACTIONS(1765),
    [sym_length_variable_to_string] = ACTIONS(1765),
    [sym_type_cast] = ACTIONS(1765),
    [sym_variable_to_xml] = ACTIONS(1765),
    [sym_xml_to_variable] = ACTIONS(1765),
    [sym_length_variable_to_xml] = ACTIONS(1765),
    [sym_redirect_string] = ACTIONS(1765),
    [sym_random_number] = ACTIONS(1765),
    [sym_get_unique_id] = ACTIONS(1765),
    [sym_get_system_number] = ACTIONS(1765),
    [sym_set_system_number] = ACTIONS(1765),
    [sym_get_serial_number] = ACTIONS(1765),
    [sym_get_master_build] = ACTIONS(1765),
    [sym_set_validation_code] = ACTIONS(1765),
    [sym_reboot] = ACTIONS(1765),
    [sym_load_duet_module] = ACTIONS(1765),
    [sym_unload_duet_module] = ACTIONS(1765),
    [sym_duet_mem_size_set] = ACTIONS(1765),
    [sym_duet_mem_size_get] = ACTIONS(1765),
    [sym_dynamic_application_device] = ACTIONS(1765),
    [sym_dynamic_polled_port] = ACTIONS(1765),
    [sym_static_port_binding] = ACTIONS(1765),
    [sym_static_ip_binding] = ACTIONS(1765),
    [sym_get_dns_list] = ACTIONS(1765),
    [sym_set_dns_list] = ACTIONS(1765),
    [sym_get_ip_address] = ACTIONS(1765),
    [sym_set_ip_address] = ACTIONS(1765),
    [sym_get_url_list] = ACTIONS(1765),
    [sym_add_url_entry] = ACTIONS(1765),
    [sym_delete_url_entry] = ACTIONS(1765),
    [sym_abs_value] = ACTIONS(1765),
    [sym_max_value] = ACTIONS(1765),
    [sym_min_value] = ACTIONS(1765),
    [sym_exp_value] = ACTIONS(1765),
    [sym_log_value] = ACTIONS(1765),
    [sym_log10_value] = ACTIONS(1765),
    [sym_power_value] = ACTIONS(1765),
    [sym_sqrt_value] = ACTIONS(1765),
    [sym_raw_be] = ACTIONS(1765),
    [sym_raw_le] = ACTIONS(1765),
    [sym_internal_queue_size_set] = ACTIONS(1765),
    [sym_internal_queue_size_get] = ACTIONS(1765),
    [sym_internal_threshold_set] = ACTIONS(1765),
    [sym_internal_threshold_get] = ACTIONS(1765),
    [sym_rebuilt_event] = ACTIONS(1765),
    [sym_do_custom_event] = ACTIONS(1765),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1765),
    [sym_clkmgr_set_clk_source] = ACTIONS(1765),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1765),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1765),
    [sym_clkmgr_get_timezone] = ACTIONS(1765),
    [sym_clkmgr_set_timezone] = ACTIONS(1765),
    [sym_clkmgr_get_resync_period] = ACTIONS(1765),
    [sym_clkmgr_set_resync_period] = ACTIONS(1765),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1765),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1765),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1765),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1765),
    [sym_clkmgr_get_timeservers] = ACTIONS(1765),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1765),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1765),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1765),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1765),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1765),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1765),
    [sym_set_log_level] = ACTIONS(1765),
    [sym_get_log_level] = ACTIONS(1765),
    [sym_amx_log] = ACTIONS(1765),
    [sym_smtp_server_config_set] = ACTIONS(1765),
    [sym_smtp_server_config_get] = ACTIONS(1765),
    [sym_smtp_send] = ACTIONS(1765),
    [sym_validate_netlinx_account] = ACTIONS(1765),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1765),
    [sym_authenticate_certificate] = ACTIONS(1765),
    [sym_audit_netlinx_session_event] = ACTIONS(1765),
    [sym_audit_netlinx_generic_event] = ACTIONS(1765),
    [sym_get_available_flash_disk_space] = ACTIONS(1765),
    [sym_get_max_flash_disk_space] = ACTIONS(1765),
    [sym_data_create_feed] = ACTIONS(1765),
    [sym_data_delete_feed] = ACTIONS(1765),
    [sym_data_publish_feed] = ACTIONS(1765),
    [sym_data_get_published_feed] = ACTIONS(1765),
    [sym_data_add_record] = ACTIONS(1765),
    [sym_data_get_event_record] = ACTIONS(1765),
    [sym__wc_data_create_feed] = ACTIONS(1765),
    [sym__wc_data_add_record] = ACTIONS(1765),
    [sym__wc_data_get_event_record] = ACTIONS(1765),
    [sym_format] = ACTIONS(1765),
    [sym_astro_clock] = ACTIONS(1765),
    [sym_set_outdoor_temperature] = ACTIONS(1765),
    [sym_set_virtual_level_count] = ACTIONS(1765),
    [sym_set_virtual_channel_count] = ACTIONS(1765),
    [sym_set_virtual_port_count] = ACTIONS(1765),
    [sym__wc_encode_int] = ACTIONS(1765),
    [sym__wc_decode_int] = ACTIONS(1765),
    [sym__wc_int] = ACTIONS(1765),
    [sym__wc_to_ch_int] = ACTIONS(1765),
    [sym__ch_to_wc_int] = ACTIONS(1765),
    [sym__wc_find_string_int] = ACTIONS(1765),
    [sym__wc_left_string_int] = ACTIONS(1765),
    [sym__wc_lower_string_int] = ACTIONS(1765),
    [sym__wc_mid_string_int] = ACTIONS(1765),
    [sym__wc_remove_string_int] = ACTIONS(1765),
    [sym__wc_right_string_int] = ACTIONS(1765),
    [sym__wc_upper_string_int] = ACTIONS(1765),
    [sym__wc_compare_string_int] = ACTIONS(1765),
    [sym__wc_get_buffer_char_int] = ACTIONS(1765),
    [sym__wc_get_buffer_string_int] = ACTIONS(1765),
    [sym__wc_concat_string_int] = ACTIONS(1765),
    [sym__wc_file_open_int] = ACTIONS(1765),
    [sym__wc_file_close_int] = ACTIONS(1765),
    [sym__wc_file_read_int] = ACTIONS(1765),
    [sym__wc_file_read_line_int] = ACTIONS(1765),
    [sym__wc_file_write_int] = ACTIONS(1765),
    [sym__wc_file_write_line_int] = ACTIONS(1765),
    [aux_sym_preproc_include_token1] = ACTIONS(1765),
    [aux_sym_preproc_define_token1] = ACTIONS(1767),
    [aux_sym_preproc_warn_token1] = ACTIONS(1767),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1767),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1767),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1767),
    [anon_sym_LBRACK] = ACTIONS(1767),
    [anon_sym_LBRACE] = ACTIONS(1767),
    [anon_sym_RBRACE] = ACTIONS(1767),
    [anon_sym_LPAREN] = ACTIONS(1765),
    [anon_sym_BANG] = ACTIONS(1767),
    [anon_sym_TILDE] = ACTIONS(1767),
    [anon_sym_DASH] = ACTIONS(1765),
    [anon_sym_PLUS] = ACTIONS(1765),
    [anon_sym_DASH_DASH] = ACTIONS(1767),
    [anon_sym_PLUS_PLUS] = ACTIONS(1767),
    [anon_sym_DQUOTE] = ACTIONS(1767),
    [anon_sym_SQUOTE] = ACTIONS(1767),
    [sym_number_literal] = ACTIONS(1767),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(483)] = {
    [sym_identifier] = ACTIONS(1769),
    [sym_define_call_keyword] = ACTIONS(1769),
    [sym_define_function_keyword] = ACTIONS(1769),
    [sym_define_library_function_keyword] = ACTIONS(1769),
    [sym_define_module_keyword] = ACTIONS(1769),
    [sym_char_keyword] = ACTIONS(1769),
    [sym_widechar_keyword] = ACTIONS(1769),
    [sym_integer_keyword] = ACTIONS(1769),
    [sym_sinteger_keyword] = ACTIONS(1769),
    [sym_long_keyword] = ACTIONS(1769),
    [sym_slong_keyword] = ACTIONS(1769),
    [sym_float_keyword] = ACTIONS(1769),
    [sym_double_keyword] = ACTIONS(1769),
    [sym_variant_keyword] = ACTIONS(1769),
    [sym_variantarray_keyword] = ACTIONS(1769),
    [sym_local_var_keyword] = ACTIONS(1769),
    [sym_stack_var_keyword] = ACTIONS(1769),
    [sym_constant_keyword] = ACTIONS(1769),
    [sym_volatile_keyword] = ACTIONS(1769),
    [sym_non_volatile_keyword] = ACTIONS(1769),
    [sym_persistent_keyword] = ACTIONS(1769),
    [sym_dev_keyword] = ACTIONS(1769),
    [sym_devlev_keyword] = ACTIONS(1769),
    [sym_devchan_keyword] = ACTIONS(1769),
    [sym_if_keyword] = ACTIONS(1769),
    [sym_else_keyword] = ACTIONS(1769),
    [sym_switch_keyword] = ACTIONS(1769),
    [sym_case_keyword] = ACTIONS(1769),
    [sym_default_keyword] = ACTIONS(1769),
    [sym_while_keyword] = ACTIONS(1769),
    [sym_for_keyword] = ACTIONS(1769),
    [sym_break_keyword] = ACTIONS(1769),
    [sym_continue_keyword] = ACTIONS(1769),
    [sym_return_keyword] = ACTIONS(1769),
    [sym_select_keyword] = ACTIONS(1769),
    [sym_active_keyword] = ACTIONS(1769),
    [sym_struct_keyword] = ACTIONS(1769),
    [sym_structure_keyword] = ACTIONS(1769),
    [aux_sym_on_keyword_token1] = ACTIONS(1769),
    [aux_sym_off_keyword_token1] = ACTIONS(1769),
    [sym_send_level_keyword] = ACTIONS(1769),
    [sym_send_string_keyword] = ACTIONS(1769),
    [sym_send_command_keyword] = ACTIONS(1769),
    [sym_clear_buffer_keyword] = ACTIONS(1769),
    [sym_create_buffer_keyword] = ACTIONS(1769),
    [sym_create_multi_buffer_keyword] = ACTIONS(1769),
    [sym_call_keyword] = ACTIONS(1769),
    [sym_system_call_keyword] = ACTIONS(1769),
    [sym_devchan_to_keyword] = ACTIONS(1769),
    [sym_devchan_min_to_keyword] = ACTIONS(1769),
    [sym_devchan_total_off_keyword] = ACTIONS(1769),
    [sym_devchan_pulse_keyword] = ACTIONS(1769),
    [sym_wait_keyword] = ACTIONS(1769),
    [sym_cancel_wait_keyword] = ACTIONS(1769),
    [sym_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1769),
    [sym_not] = ACTIONS(1769),
    [sym_bnot] = ACTIONS(1769),
    [sym___file__] = ACTIONS(1769),
    [sym___date__] = ACTIONS(1769),
    [sym___ldate__] = ACTIONS(1769),
    [sym___time__] = ACTIONS(1769),
    [sym___line__] = ACTIONS(1769),
    [sym___name__] = ACTIONS(1769),
    [sym___netlinx__] = ACTIONS(1769),
    [sym_netlinx_axi_version] = ACTIONS(1769),
    [sym_url_flg_tcp] = ACTIONS(1769),
    [sym_url_flg_accinfopresent] = ACTIONS(1769),
    [sym_url_flg_temp] = ACTIONS(1769),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1769),
    [sym_url_flg_stat_mask] = ACTIONS(1769),
    [sym_url_flg_stat_lookup] = ACTIONS(1769),
    [sym_url_flg_stat_connecting] = ACTIONS(1769),
    [sym_url_flg_stat_waiting] = ACTIONS(1769),
    [sym_url_flg_stat_connected] = ACTIONS(1769),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1769),
    [sym_first_virtual_device] = ACTIONS(1769),
    [sym_first_local_port] = ACTIONS(1769),
    [sym_dynamic_virtual_device] = ACTIONS(1769),
    [sym_timeline_once] = ACTIONS(1769),
    [sym_timeline_repeat] = ACTIONS(1769),
    [sym_timeline_absolute] = ACTIONS(1769),
    [sym_timeline_relative] = ACTIONS(1769),
    [sym_do_push_timed_infinite] = ACTIONS(1769),
    [sym_source_type_no_address] = ACTIONS(1769),
    [sym_source_type_neuron_id] = ACTIONS(1769),
    [sym_source_type_ip_address] = ACTIONS(1769),
    [sym_source_type_axlink] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1769),
    [sym_source_type_ip_socket_address] = ACTIONS(1769),
    [sym_source_type_rs232] = ACTIONS(1769),
    [sym_source_type_internal] = ACTIONS(1769),
    [sym_source_type_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1769),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1769),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1769),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1769),
    [sym_true] = ACTIONS(1769),
    [sym_false] = ACTIONS(1769),
    [sym_file_read_only] = ACTIONS(1769),
    [sym_file_rw_new] = ACTIONS(1769),
    [sym_file_rw_append] = ACTIONS(1769),
    [sym_ip_tcp] = ACTIONS(1769),
    [sym_ip_udp] = ACTIONS(1769),
    [sym_ip_udp_2way] = ACTIONS(1769),
    [sym_xml_encode_types] = ACTIONS(1769),
    [sym_xml_encode_char_as_list] = ACTIONS(1769),
    [sym_xml_encode_le] = ACTIONS(1769),
    [sym_xml_decode_types] = ACTIONS(1769),
    [sym_xml_decode_no_preserve] = ACTIONS(1769),
    [sym_duet_dev_type_display_device] = ACTIONS(1769),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_io_device] = ACTIONS(1769),
    [sym_duet_dev_type_relay_device] = ACTIONS(1769),
    [sym_duet_dev_type_ups] = ACTIONS(1769),
    [sym_duet_dev_type_amplifier] = ACTIONS(1769),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1769),
    [sym_duet_dev_type_camera] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1769),
    [sym_duet_dev_type_dss] = ACTIONS(1769),
    [sym_duet_dev_type_dvr] = ACTIONS(1769),
    [sym_duet_dev_type_disc_device] = ACTIONS(1769),
    [sym_duet_dev_type_document_camera] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1769),
    [sym_duet_dev_type_hvac] = ACTIONS(1769),
    [sym_duet_dev_type_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_light] = ACTIONS(1769),
    [sym_duet_dev_type_monitor] = ACTIONS(1769),
    [sym_duet_dev_type_motor] = ACTIONS(1769),
    [sym_duet_dev_type_multi_window] = ACTIONS(1769),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1769),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1769),
    [sym_duet_dev_type_receiver] = ACTIONS(1769),
    [sym_duet_dev_type_security_system] = ACTIONS(1769),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1769),
    [sym_duet_dev_type_settop_box] = ACTIONS(1769),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1769),
    [sym_duet_dev_type_switcher] = ACTIONS(1769),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_tv] = ACTIONS(1769),
    [sym_duet_dev_type_utility] = ACTIONS(1769),
    [sym_duet_dev_type_vcr] = ACTIONS(1769),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_video_processor] = ACTIONS(1769),
    [sym_duet_dev_type_video_projector] = ACTIONS(1769),
    [sym_duet_dev_type_video_wall] = ACTIONS(1769),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1769),
    [sym_duet_dev_type_weather] = ACTIONS(1769),
    [sym_duet_dev_not_polled] = ACTIONS(1769),
    [sym_duet_dev_polled] = ACTIONS(1769),
    [sym_clkmgr_mode_network] = ACTIONS(1769),
    [sym_clkmgr_mode_standalone] = ACTIONS(1769),
    [sym_normal_standby] = ACTIONS(1769),
    [sym_normal_wake] = ACTIONS(1769),
    [sym_ip_multicast_ttl_option] = ACTIONS(1769),
    [sym_ip_ttl_subnet] = ACTIONS(1769),
    [sym_ip_ttl_site] = ACTIONS(1769),
    [sym_ip_ttl_region] = ACTIONS(1769),
    [sym_ip_ttl_continent] = ACTIONS(1769),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1769),
    [sym_ip_nodelay_on] = ACTIONS(1769),
    [sym_ip_nodelay_off] = ACTIONS(1769),
    [sym_amx_error] = ACTIONS(1769),
    [sym_amx_warning] = ACTIONS(1769),
    [sym_amx_info] = ACTIONS(1769),
    [sym_amx_debug] = ACTIONS(1769),
    [sym_netlinx_login_success] = ACTIONS(1769),
    [sym_netlinx_login_fail] = ACTIONS(1769),
    [sym_netlinx_logout] = ACTIONS(1769),
    [sym_null_str] = ACTIONS(1769),
    [sym_smtp_address] = ACTIONS(1769),
    [sym_smtp_port_number] = ACTIONS(1769),
    [sym_smtp_username] = ACTIONS(1769),
    [sym_smtp_password] = ACTIONS(1769),
    [sym_smtp_require_tls] = ACTIONS(1769),
    [sym_smtp_from] = ACTIONS(1769),
    [sym_smtp_tls_true] = ACTIONS(1769),
    [sym_smtp_tls_false] = ACTIONS(1769),
    [sym_listview_on_row_select_event] = ACTIONS(1769),
    [sym_data_structure_unknown] = ACTIONS(1769),
    [sym_data_structure_datafeed] = ACTIONS(1769),
    [sym_data_structure_datarecordset] = ACTIONS(1769),
    [sym_data_structure_datarecord] = ACTIONS(1769),
    [sym_data_structure_datafield] = ACTIONS(1769),
    [sym_data_max_content_field_count] = ACTIONS(1769),
    [sym_data_max_metadata_field_count] = ACTIONS(1769),
    [sym_data_max_id_length] = ACTIONS(1769),
    [sym_data_max_name_length] = ACTIONS(1769),
    [sym_data_max_description_length] = ACTIONS(1769),
    [sym_data_max_type_length] = ACTIONS(1769),
    [sym_data_max_format_length] = ACTIONS(1769),
    [sym_data_max_source_length] = ACTIONS(1769),
    [sym_data_max_value_length] = ACTIONS(1769),
    [sym_data_max_label_length] = ACTIONS(1769),
    [sym_data_max_url_length] = ACTIONS(1769),
    [sym_max_last_login_info_length] = ACTIONS(1769),
    [sym_data_type_unknown] = ACTIONS(1769),
    [sym_data_type_string] = ACTIONS(1769),
    [sym_data_type_datetime] = ACTIONS(1769),
    [sym_data_type_date] = ACTIONS(1769),
    [sym_data_type_time] = ACTIONS(1769),
    [sym_data_type_image] = ACTIONS(1769),
    [sym_data_format_url] = ACTIONS(1769),
    [sym_data_format_phone] = ACTIONS(1769),
    [sym_data_format_email] = ACTIONS(1769),
    [sym_data_format_iso8601] = ACTIONS(1769),
    [sym_auditlog_priv] = ACTIONS(1769),
    [sym_configuration_priv] = ACTIONS(1769),
    [sym_deviceconfig_priv] = ACTIONS(1769),
    [sym_ftp_priv] = ACTIONS(1769),
    [sym_http_priv] = ACTIONS(1769),
    [sym_networkconfig_priv] = ACTIONS(1769),
    [sym_programport_priv] = ACTIONS(1769),
    [sym_remoteui_priv] = ACTIONS(1769),
    [sym_securitycontrol_priv] = ACTIONS(1769),
    [sym_softwaremanagement_priv] = ACTIONS(1769),
    [sym_terminal_priv] = ACTIONS(1769),
    [sym_tpadmin_priv] = ACTIONS(1769),
    [sym_user1_priv] = ACTIONS(1769),
    [sym_user2_priv] = ACTIONS(1769),
    [sym_user3_priv] = ACTIONS(1769),
    [sym_user4_priv] = ACTIONS(1769),
    [sym_usermanagement_priv] = ACTIONS(1769),
    [sym_valid_account] = ACTIONS(1769),
    [sym_err_name_invalid] = ACTIONS(1769),
    [sym_err_password_invalid] = ACTIONS(1769),
    [sym_err_invalid_account] = ACTIONS(1769),
    [sym_err_no_authorization] = ACTIONS(1769),
    [sym_err_invalid_parameter] = ACTIONS(1769),
    [sym_err_user_locked_out] = ACTIONS(1769),
    [sym_err_user_account_expired] = ACTIONS(1769),
    [sym_err_general_error] = ACTIONS(1769),
    [sym_err_authentication_error] = ACTIONS(1769),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1769),
    [sym_tls_validate_certificate] = ACTIONS(1769),
    [sym_tdata] = ACTIONS(1769),
    [sym_tchannel] = ACTIONS(1769),
    [sym_tlevel] = ACTIONS(1769),
    [sym_tbutton] = ACTIONS(1769),
    [sym_ttimeline] = ACTIONS(1769),
    [sym_tcustom] = ACTIONS(1769),
    [sym_url_struct] = ACTIONS(1769),
    [sym_dns_struct] = ACTIONS(1769),
    [sym_ip_address_struct] = ACTIONS(1769),
    [sym_dev_info_struct] = ACTIONS(1769),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1769),
    [sym_data_feed] = ACTIONS(1769),
    [sym_data_field] = ACTIONS(1769),
    [sym_data_record] = ACTIONS(1769),
    [sym_wc_data_feed] = ACTIONS(1769),
    [sym_wc_data_field] = ACTIONS(1769),
    [sym_wc_data_record] = ACTIONS(1769),
    [sym_last_login_info] = ACTIONS(1769),
    [sym____reserved___] = ACTIONS(1769),
    [sym_date] = ACTIONS(1769),
    [sym_day] = ACTIONS(1769),
    [sym_time] = ACTIONS(1769),
    [sym_ldate] = ACTIONS(1769),
    [sym_push_device] = ACTIONS(1769),
    [sym_push_channel] = ACTIONS(1769),
    [sym_push_devchan] = ACTIONS(1769),
    [sym_release_device] = ACTIONS(1769),
    [sym_release_channel] = ACTIONS(1769),
    [sym_release_devchan] = ACTIONS(1769),
    [sym_master_slot] = ACTIONS(1769),
    [sym_get_pulse_time] = ACTIONS(1769),
    [sym_get_timer] = ACTIONS(1769),
    [sym_master_sn] = ACTIONS(1769),
    [sym_system_number] = ACTIONS(1769),
    [sym_button] = ACTIONS(1769),
    [sym_dv_channel] = ACTIONS(1769),
    [sym_data] = ACTIONS(1769),
    [sym_level] = ACTIONS(1769),
    [sym_timeline] = ACTIONS(1769),
    [sym_channel] = ACTIONS(1769),
    [sym_custom] = ACTIONS(1769),
    [sym_atoi] = ACTIONS(1769),
    [sym_atol] = ACTIONS(1769),
    [sym_atof] = ACTIONS(1769),
    [sym_itoa] = ACTIONS(1769),
    [sym_itohex] = ACTIONS(1769),
    [sym_hextoi] = ACTIONS(1769),
    [sym_ftoa] = ACTIONS(1769),
    [sym_find_string] = ACTIONS(1769),
    [sym_left_string] = ACTIONS(1769),
    [sym_right_string] = ACTIONS(1769),
    [sym_mid_string] = ACTIONS(1769),
    [sym_remove_string] = ACTIONS(1769),
    [sym_get_buffer_string] = ACTIONS(1769),
    [sym_get_buffer_char] = ACTIONS(1769),
    [sym_get_multi_buffer_string] = ACTIONS(1769),
    [sym_length_string] = ACTIONS(1769),
    [sym_max_length_string] = ACTIONS(1769),
    [sym_set_length_string] = ACTIONS(1769),
    [sym_compare_string] = ACTIONS(1769),
    [sym_upper_string] = ACTIONS(1769),
    [sym_lower_string] = ACTIONS(1769),
    [sym_get_last] = ACTIONS(1769),
    [sym_date_to_day] = ACTIONS(1769),
    [sym_date_to_month] = ACTIONS(1769),
    [sym_date_to_year] = ACTIONS(1769),
    [sym_time_to_hour] = ACTIONS(1769),
    [sym_time_to_minute] = ACTIONS(1769),
    [sym_time_to_second] = ACTIONS(1769),
    [sym_day_of_week] = ACTIONS(1769),
    [sym_set_pulse_time] = ACTIONS(1769),
    [sym_set_timer] = ACTIONS(1769),
    [sym_file_open] = ACTIONS(1769),
    [sym_file_close] = ACTIONS(1769),
    [sym_file_read] = ACTIONS(1769),
    [sym_file_read_line] = ACTIONS(1769),
    [sym_file_write] = ACTIONS(1769),
    [sym_file_write_line] = ACTIONS(1769),
    [sym_file_copy] = ACTIONS(1769),
    [sym_file_delete] = ACTIONS(1769),
    [sym_file_rename] = ACTIONS(1769),
    [sym_file_dir] = ACTIONS(1769),
    [sym_file_getdir] = ACTIONS(1769),
    [sym_file_setdir] = ACTIONS(1769),
    [sym_file_create_dir] = ACTIONS(1769),
    [sym_file_remove_dir] = ACTIONS(1769),
    [sym_file_seek] = ACTIONS(1769),
    [sym_length_array] = ACTIONS(1769),
    [sym_max_length_array] = ACTIONS(1769),
    [sym_set_length_array] = ACTIONS(1769),
    [sym_ip_client_open] = ACTIONS(1769),
    [sym_ip_client_close] = ACTIONS(1769),
    [sym_ip_server_open] = ACTIONS(1769),
    [sym_ip_server_close] = ACTIONS(1769),
    [sym_ip_bound_client_open] = ACTIONS(1769),
    [sym_ip_mc_server_open] = ACTIONS(1769),
    [sym_ip_set_option] = ACTIONS(1769),
    [sym_ssh_client_open] = ACTIONS(1769),
    [sym_ssh_client_close] = ACTIONS(1769),
    [sym_tls_client_open] = ACTIONS(1769),
    [sym_tls_client_close] = ACTIONS(1769),
    [sym_do_push] = ACTIONS(1769),
    [sym_do_push_timed] = ACTIONS(1769),
    [sym_do_release] = ACTIONS(1769),
    [sym_device_id] = ACTIONS(1769),
    [sym_device_info] = ACTIONS(1769),
    [sym_device_id_string] = ACTIONS(1769),
    [sym_device_standby] = ACTIONS(1769),
    [sym_device_wake] = ACTIONS(1769),
    [sym_timeline_create] = ACTIONS(1769),
    [sym_timeline_kill] = ACTIONS(1769),
    [sym_timeline_reload] = ACTIONS(1769),
    [sym_timeline_pause] = ACTIONS(1769),
    [sym_timeline_restart] = ACTIONS(1769),
    [sym_timeline_set] = ACTIONS(1769),
    [sym_timeline_get] = ACTIONS(1769),
    [sym_timeline_active] = ACTIONS(1769),
    [sym_timeline_dynamic_id] = ACTIONS(1769),
    [sym_variable_to_string] = ACTIONS(1769),
    [sym_string_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_string] = ACTIONS(1769),
    [sym_type_cast] = ACTIONS(1769),
    [sym_variable_to_xml] = ACTIONS(1769),
    [sym_xml_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_xml] = ACTIONS(1769),
    [sym_redirect_string] = ACTIONS(1769),
    [sym_random_number] = ACTIONS(1769),
    [sym_get_unique_id] = ACTIONS(1769),
    [sym_get_system_number] = ACTIONS(1769),
    [sym_set_system_number] = ACTIONS(1769),
    [sym_get_serial_number] = ACTIONS(1769),
    [sym_get_master_build] = ACTIONS(1769),
    [sym_set_validation_code] = ACTIONS(1769),
    [sym_reboot] = ACTIONS(1769),
    [sym_load_duet_module] = ACTIONS(1769),
    [sym_unload_duet_module] = ACTIONS(1769),
    [sym_duet_mem_size_set] = ACTIONS(1769),
    [sym_duet_mem_size_get] = ACTIONS(1769),
    [sym_dynamic_application_device] = ACTIONS(1769),
    [sym_dynamic_polled_port] = ACTIONS(1769),
    [sym_static_port_binding] = ACTIONS(1769),
    [sym_static_ip_binding] = ACTIONS(1769),
    [sym_get_dns_list] = ACTIONS(1769),
    [sym_set_dns_list] = ACTIONS(1769),
    [sym_get_ip_address] = ACTIONS(1769),
    [sym_set_ip_address] = ACTIONS(1769),
    [sym_get_url_list] = ACTIONS(1769),
    [sym_add_url_entry] = ACTIONS(1769),
    [sym_delete_url_entry] = ACTIONS(1769),
    [sym_abs_value] = ACTIONS(1769),
    [sym_max_value] = ACTIONS(1769),
    [sym_min_value] = ACTIONS(1769),
    [sym_exp_value] = ACTIONS(1769),
    [sym_log_value] = ACTIONS(1769),
    [sym_log10_value] = ACTIONS(1769),
    [sym_power_value] = ACTIONS(1769),
    [sym_sqrt_value] = ACTIONS(1769),
    [sym_raw_be] = ACTIONS(1769),
    [sym_raw_le] = ACTIONS(1769),
    [sym_internal_queue_size_set] = ACTIONS(1769),
    [sym_internal_queue_size_get] = ACTIONS(1769),
    [sym_internal_threshold_set] = ACTIONS(1769),
    [sym_internal_threshold_get] = ACTIONS(1769),
    [sym_rebuilt_event] = ACTIONS(1769),
    [sym_do_custom_event] = ACTIONS(1769),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1769),
    [sym_clkmgr_set_clk_source] = ACTIONS(1769),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1769),
    [sym_clkmgr_get_timezone] = ACTIONS(1769),
    [sym_clkmgr_set_timezone] = ACTIONS(1769),
    [sym_clkmgr_get_resync_period] = ACTIONS(1769),
    [sym_clkmgr_set_resync_period] = ACTIONS(1769),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_timeservers] = ACTIONS(1769),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_set_log_level] = ACTIONS(1769),
    [sym_get_log_level] = ACTIONS(1769),
    [sym_amx_log] = ACTIONS(1769),
    [sym_smtp_server_config_set] = ACTIONS(1769),
    [sym_smtp_server_config_get] = ACTIONS(1769),
    [sym_smtp_send] = ACTIONS(1769),
    [sym_validate_netlinx_account] = ACTIONS(1769),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1769),
    [sym_authenticate_certificate] = ACTIONS(1769),
    [sym_audit_netlinx_session_event] = ACTIONS(1769),
    [sym_audit_netlinx_generic_event] = ACTIONS(1769),
    [sym_get_available_flash_disk_space] = ACTIONS(1769),
    [sym_get_max_flash_disk_space] = ACTIONS(1769),
    [sym_data_create_feed] = ACTIONS(1769),
    [sym_data_delete_feed] = ACTIONS(1769),
    [sym_data_publish_feed] = ACTIONS(1769),
    [sym_data_get_published_feed] = ACTIONS(1769),
    [sym_data_add_record] = ACTIONS(1769),
    [sym_data_get_event_record] = ACTIONS(1769),
    [sym__wc_data_create_feed] = ACTIONS(1769),
    [sym__wc_data_add_record] = ACTIONS(1769),
    [sym__wc_data_get_event_record] = ACTIONS(1769),
    [sym_format] = ACTIONS(1769),
    [sym_astro_clock] = ACTIONS(1769),
    [sym_set_outdoor_temperature] = ACTIONS(1769),
    [sym_set_virtual_level_count] = ACTIONS(1769),
    [sym_set_virtual_channel_count] = ACTIONS(1769),
    [sym_set_virtual_port_count] = ACTIONS(1769),
    [sym__wc_encode_int] = ACTIONS(1769),
    [sym__wc_decode_int] = ACTIONS(1769),
    [sym__wc_int] = ACTIONS(1769),
    [sym__wc_to_ch_int] = ACTIONS(1769),
    [sym__ch_to_wc_int] = ACTIONS(1769),
    [sym__wc_find_string_int] = ACTIONS(1769),
    [sym__wc_left_string_int] = ACTIONS(1769),
    [sym__wc_lower_string_int] = ACTIONS(1769),
    [sym__wc_mid_string_int] = ACTIONS(1769),
    [sym__wc_remove_string_int] = ACTIONS(1769),
    [sym__wc_right_string_int] = ACTIONS(1769),
    [sym__wc_upper_string_int] = ACTIONS(1769),
    [sym__wc_compare_string_int] = ACTIONS(1769),
    [sym__wc_get_buffer_char_int] = ACTIONS(1769),
    [sym__wc_get_buffer_string_int] = ACTIONS(1769),
    [sym__wc_concat_string_int] = ACTIONS(1769),
    [sym__wc_file_open_int] = ACTIONS(1769),
    [sym__wc_file_close_int] = ACTIONS(1769),
    [sym__wc_file_read_int] = ACTIONS(1769),
    [sym__wc_file_read_line_int] = ACTIONS(1769),
    [sym__wc_file_write_int] = ACTIONS(1769),
    [sym__wc_file_write_line_int] = ACTIONS(1769),
    [aux_sym_preproc_include_token1] = ACTIONS(1769),
    [aux_sym_preproc_define_token1] = ACTIONS(1771),
    [aux_sym_preproc_warn_token1] = ACTIONS(1771),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1771),
    [anon_sym_LBRACK] = ACTIONS(1771),
    [anon_sym_LBRACE] = ACTIONS(1771),
    [anon_sym_RBRACE] = ACTIONS(1771),
    [anon_sym_LPAREN] = ACTIONS(1769),
    [anon_sym_BANG] = ACTIONS(1771),
    [anon_sym_TILDE] = ACTIONS(1771),
    [anon_sym_DASH] = ACTIONS(1769),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_DASH_DASH] = ACTIONS(1771),
    [anon_sym_PLUS_PLUS] = ACTIONS(1771),
    [anon_sym_DQUOTE] = ACTIONS(1771),
    [anon_sym_SQUOTE] = ACTIONS(1771),
    [sym_number_literal] = ACTIONS(1771),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(484)] = {
    [sym_identifier] = ACTIONS(1769),
    [sym_define_call_keyword] = ACTIONS(1769),
    [sym_define_function_keyword] = ACTIONS(1769),
    [sym_define_library_function_keyword] = ACTIONS(1769),
    [sym_define_module_keyword] = ACTIONS(1769),
    [sym_char_keyword] = ACTIONS(1769),
    [sym_widechar_keyword] = ACTIONS(1769),
    [sym_integer_keyword] = ACTIONS(1769),
    [sym_sinteger_keyword] = ACTIONS(1769),
    [sym_long_keyword] = ACTIONS(1769),
    [sym_slong_keyword] = ACTIONS(1769),
    [sym_float_keyword] = ACTIONS(1769),
    [sym_double_keyword] = ACTIONS(1769),
    [sym_variant_keyword] = ACTIONS(1769),
    [sym_variantarray_keyword] = ACTIONS(1769),
    [sym_local_var_keyword] = ACTIONS(1769),
    [sym_stack_var_keyword] = ACTIONS(1769),
    [sym_constant_keyword] = ACTIONS(1769),
    [sym_volatile_keyword] = ACTIONS(1769),
    [sym_non_volatile_keyword] = ACTIONS(1769),
    [sym_persistent_keyword] = ACTIONS(1769),
    [sym_dev_keyword] = ACTIONS(1769),
    [sym_devlev_keyword] = ACTIONS(1769),
    [sym_devchan_keyword] = ACTIONS(1769),
    [sym_if_keyword] = ACTIONS(1769),
    [sym_else_keyword] = ACTIONS(1769),
    [sym_switch_keyword] = ACTIONS(1769),
    [sym_case_keyword] = ACTIONS(1769),
    [sym_default_keyword] = ACTIONS(1769),
    [sym_while_keyword] = ACTIONS(1769),
    [sym_for_keyword] = ACTIONS(1769),
    [sym_break_keyword] = ACTIONS(1769),
    [sym_continue_keyword] = ACTIONS(1769),
    [sym_return_keyword] = ACTIONS(1769),
    [sym_select_keyword] = ACTIONS(1769),
    [sym_active_keyword] = ACTIONS(1769),
    [sym_struct_keyword] = ACTIONS(1769),
    [sym_structure_keyword] = ACTIONS(1769),
    [aux_sym_on_keyword_token1] = ACTIONS(1769),
    [aux_sym_off_keyword_token1] = ACTIONS(1769),
    [sym_send_level_keyword] = ACTIONS(1769),
    [sym_send_string_keyword] = ACTIONS(1769),
    [sym_send_command_keyword] = ACTIONS(1769),
    [sym_clear_buffer_keyword] = ACTIONS(1769),
    [sym_create_buffer_keyword] = ACTIONS(1769),
    [sym_create_multi_buffer_keyword] = ACTIONS(1769),
    [sym_call_keyword] = ACTIONS(1769),
    [sym_system_call_keyword] = ACTIONS(1769),
    [sym_devchan_to_keyword] = ACTIONS(1769),
    [sym_devchan_min_to_keyword] = ACTIONS(1769),
    [sym_devchan_total_off_keyword] = ACTIONS(1769),
    [sym_devchan_pulse_keyword] = ACTIONS(1769),
    [sym_wait_keyword] = ACTIONS(1769),
    [sym_cancel_wait_keyword] = ACTIONS(1769),
    [sym_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1769),
    [sym_not] = ACTIONS(1769),
    [sym_bnot] = ACTIONS(1769),
    [sym___file__] = ACTIONS(1769),
    [sym___date__] = ACTIONS(1769),
    [sym___ldate__] = ACTIONS(1769),
    [sym___time__] = ACTIONS(1769),
    [sym___line__] = ACTIONS(1769),
    [sym___name__] = ACTIONS(1769),
    [sym___netlinx__] = ACTIONS(1769),
    [sym_netlinx_axi_version] = ACTIONS(1769),
    [sym_url_flg_tcp] = ACTIONS(1769),
    [sym_url_flg_accinfopresent] = ACTIONS(1769),
    [sym_url_flg_temp] = ACTIONS(1769),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1769),
    [sym_url_flg_stat_mask] = ACTIONS(1769),
    [sym_url_flg_stat_lookup] = ACTIONS(1769),
    [sym_url_flg_stat_connecting] = ACTIONS(1769),
    [sym_url_flg_stat_waiting] = ACTIONS(1769),
    [sym_url_flg_stat_connected] = ACTIONS(1769),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1769),
    [sym_first_virtual_device] = ACTIONS(1769),
    [sym_first_local_port] = ACTIONS(1769),
    [sym_dynamic_virtual_device] = ACTIONS(1769),
    [sym_timeline_once] = ACTIONS(1769),
    [sym_timeline_repeat] = ACTIONS(1769),
    [sym_timeline_absolute] = ACTIONS(1769),
    [sym_timeline_relative] = ACTIONS(1769),
    [sym_do_push_timed_infinite] = ACTIONS(1769),
    [sym_source_type_no_address] = ACTIONS(1769),
    [sym_source_type_neuron_id] = ACTIONS(1769),
    [sym_source_type_ip_address] = ACTIONS(1769),
    [sym_source_type_axlink] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1769),
    [sym_source_type_ip_socket_address] = ACTIONS(1769),
    [sym_source_type_rs232] = ACTIONS(1769),
    [sym_source_type_internal] = ACTIONS(1769),
    [sym_source_type_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1769),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1769),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1769),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1769),
    [sym_true] = ACTIONS(1769),
    [sym_false] = ACTIONS(1769),
    [sym_file_read_only] = ACTIONS(1769),
    [sym_file_rw_new] = ACTIONS(1769),
    [sym_file_rw_append] = ACTIONS(1769),
    [sym_ip_tcp] = ACTIONS(1769),
    [sym_ip_udp] = ACTIONS(1769),
    [sym_ip_udp_2way] = ACTIONS(1769),
    [sym_xml_encode_types] = ACTIONS(1769),
    [sym_xml_encode_char_as_list] = ACTIONS(1769),
    [sym_xml_encode_le] = ACTIONS(1769),
    [sym_xml_decode_types] = ACTIONS(1769),
    [sym_xml_decode_no_preserve] = ACTIONS(1769),
    [sym_duet_dev_type_display_device] = ACTIONS(1769),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_io_device] = ACTIONS(1769),
    [sym_duet_dev_type_relay_device] = ACTIONS(1769),
    [sym_duet_dev_type_ups] = ACTIONS(1769),
    [sym_duet_dev_type_amplifier] = ACTIONS(1769),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1769),
    [sym_duet_dev_type_camera] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1769),
    [sym_duet_dev_type_dss] = ACTIONS(1769),
    [sym_duet_dev_type_dvr] = ACTIONS(1769),
    [sym_duet_dev_type_disc_device] = ACTIONS(1769),
    [sym_duet_dev_type_document_camera] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1769),
    [sym_duet_dev_type_hvac] = ACTIONS(1769),
    [sym_duet_dev_type_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_light] = ACTIONS(1769),
    [sym_duet_dev_type_monitor] = ACTIONS(1769),
    [sym_duet_dev_type_motor] = ACTIONS(1769),
    [sym_duet_dev_type_multi_window] = ACTIONS(1769),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1769),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1769),
    [sym_duet_dev_type_receiver] = ACTIONS(1769),
    [sym_duet_dev_type_security_system] = ACTIONS(1769),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1769),
    [sym_duet_dev_type_settop_box] = ACTIONS(1769),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1769),
    [sym_duet_dev_type_switcher] = ACTIONS(1769),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_tv] = ACTIONS(1769),
    [sym_duet_dev_type_utility] = ACTIONS(1769),
    [sym_duet_dev_type_vcr] = ACTIONS(1769),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_video_processor] = ACTIONS(1769),
    [sym_duet_dev_type_video_projector] = ACTIONS(1769),
    [sym_duet_dev_type_video_wall] = ACTIONS(1769),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1769),
    [sym_duet_dev_type_weather] = ACTIONS(1769),
    [sym_duet_dev_not_polled] = ACTIONS(1769),
    [sym_duet_dev_polled] = ACTIONS(1769),
    [sym_clkmgr_mode_network] = ACTIONS(1769),
    [sym_clkmgr_mode_standalone] = ACTIONS(1769),
    [sym_normal_standby] = ACTIONS(1769),
    [sym_normal_wake] = ACTIONS(1769),
    [sym_ip_multicast_ttl_option] = ACTIONS(1769),
    [sym_ip_ttl_subnet] = ACTIONS(1769),
    [sym_ip_ttl_site] = ACTIONS(1769),
    [sym_ip_ttl_region] = ACTIONS(1769),
    [sym_ip_ttl_continent] = ACTIONS(1769),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1769),
    [sym_ip_nodelay_on] = ACTIONS(1769),
    [sym_ip_nodelay_off] = ACTIONS(1769),
    [sym_amx_error] = ACTIONS(1769),
    [sym_amx_warning] = ACTIONS(1769),
    [sym_amx_info] = ACTIONS(1769),
    [sym_amx_debug] = ACTIONS(1769),
    [sym_netlinx_login_success] = ACTIONS(1769),
    [sym_netlinx_login_fail] = ACTIONS(1769),
    [sym_netlinx_logout] = ACTIONS(1769),
    [sym_null_str] = ACTIONS(1769),
    [sym_smtp_address] = ACTIONS(1769),
    [sym_smtp_port_number] = ACTIONS(1769),
    [sym_smtp_username] = ACTIONS(1769),
    [sym_smtp_password] = ACTIONS(1769),
    [sym_smtp_require_tls] = ACTIONS(1769),
    [sym_smtp_from] = ACTIONS(1769),
    [sym_smtp_tls_true] = ACTIONS(1769),
    [sym_smtp_tls_false] = ACTIONS(1769),
    [sym_listview_on_row_select_event] = ACTIONS(1769),
    [sym_data_structure_unknown] = ACTIONS(1769),
    [sym_data_structure_datafeed] = ACTIONS(1769),
    [sym_data_structure_datarecordset] = ACTIONS(1769),
    [sym_data_structure_datarecord] = ACTIONS(1769),
    [sym_data_structure_datafield] = ACTIONS(1769),
    [sym_data_max_content_field_count] = ACTIONS(1769),
    [sym_data_max_metadata_field_count] = ACTIONS(1769),
    [sym_data_max_id_length] = ACTIONS(1769),
    [sym_data_max_name_length] = ACTIONS(1769),
    [sym_data_max_description_length] = ACTIONS(1769),
    [sym_data_max_type_length] = ACTIONS(1769),
    [sym_data_max_format_length] = ACTIONS(1769),
    [sym_data_max_source_length] = ACTIONS(1769),
    [sym_data_max_value_length] = ACTIONS(1769),
    [sym_data_max_label_length] = ACTIONS(1769),
    [sym_data_max_url_length] = ACTIONS(1769),
    [sym_max_last_login_info_length] = ACTIONS(1769),
    [sym_data_type_unknown] = ACTIONS(1769),
    [sym_data_type_string] = ACTIONS(1769),
    [sym_data_type_datetime] = ACTIONS(1769),
    [sym_data_type_date] = ACTIONS(1769),
    [sym_data_type_time] = ACTIONS(1769),
    [sym_data_type_image] = ACTIONS(1769),
    [sym_data_format_url] = ACTIONS(1769),
    [sym_data_format_phone] = ACTIONS(1769),
    [sym_data_format_email] = ACTIONS(1769),
    [sym_data_format_iso8601] = ACTIONS(1769),
    [sym_auditlog_priv] = ACTIONS(1769),
    [sym_configuration_priv] = ACTIONS(1769),
    [sym_deviceconfig_priv] = ACTIONS(1769),
    [sym_ftp_priv] = ACTIONS(1769),
    [sym_http_priv] = ACTIONS(1769),
    [sym_networkconfig_priv] = ACTIONS(1769),
    [sym_programport_priv] = ACTIONS(1769),
    [sym_remoteui_priv] = ACTIONS(1769),
    [sym_securitycontrol_priv] = ACTIONS(1769),
    [sym_softwaremanagement_priv] = ACTIONS(1769),
    [sym_terminal_priv] = ACTIONS(1769),
    [sym_tpadmin_priv] = ACTIONS(1769),
    [sym_user1_priv] = ACTIONS(1769),
    [sym_user2_priv] = ACTIONS(1769),
    [sym_user3_priv] = ACTIONS(1769),
    [sym_user4_priv] = ACTIONS(1769),
    [sym_usermanagement_priv] = ACTIONS(1769),
    [sym_valid_account] = ACTIONS(1769),
    [sym_err_name_invalid] = ACTIONS(1769),
    [sym_err_password_invalid] = ACTIONS(1769),
    [sym_err_invalid_account] = ACTIONS(1769),
    [sym_err_no_authorization] = ACTIONS(1769),
    [sym_err_invalid_parameter] = ACTIONS(1769),
    [sym_err_user_locked_out] = ACTIONS(1769),
    [sym_err_user_account_expired] = ACTIONS(1769),
    [sym_err_general_error] = ACTIONS(1769),
    [sym_err_authentication_error] = ACTIONS(1769),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1769),
    [sym_tls_validate_certificate] = ACTIONS(1769),
    [sym_tdata] = ACTIONS(1769),
    [sym_tchannel] = ACTIONS(1769),
    [sym_tlevel] = ACTIONS(1769),
    [sym_tbutton] = ACTIONS(1769),
    [sym_ttimeline] = ACTIONS(1769),
    [sym_tcustom] = ACTIONS(1769),
    [sym_url_struct] = ACTIONS(1769),
    [sym_dns_struct] = ACTIONS(1769),
    [sym_ip_address_struct] = ACTIONS(1769),
    [sym_dev_info_struct] = ACTIONS(1769),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1769),
    [sym_data_feed] = ACTIONS(1769),
    [sym_data_field] = ACTIONS(1769),
    [sym_data_record] = ACTIONS(1769),
    [sym_wc_data_feed] = ACTIONS(1769),
    [sym_wc_data_field] = ACTIONS(1769),
    [sym_wc_data_record] = ACTIONS(1769),
    [sym_last_login_info] = ACTIONS(1769),
    [sym____reserved___] = ACTIONS(1769),
    [sym_date] = ACTIONS(1769),
    [sym_day] = ACTIONS(1769),
    [sym_time] = ACTIONS(1769),
    [sym_ldate] = ACTIONS(1769),
    [sym_push_device] = ACTIONS(1769),
    [sym_push_channel] = ACTIONS(1769),
    [sym_push_devchan] = ACTIONS(1769),
    [sym_release_device] = ACTIONS(1769),
    [sym_release_channel] = ACTIONS(1769),
    [sym_release_devchan] = ACTIONS(1769),
    [sym_master_slot] = ACTIONS(1769),
    [sym_get_pulse_time] = ACTIONS(1769),
    [sym_get_timer] = ACTIONS(1769),
    [sym_master_sn] = ACTIONS(1769),
    [sym_system_number] = ACTIONS(1769),
    [sym_button] = ACTIONS(1769),
    [sym_dv_channel] = ACTIONS(1769),
    [sym_data] = ACTIONS(1769),
    [sym_level] = ACTIONS(1769),
    [sym_timeline] = ACTIONS(1769),
    [sym_channel] = ACTIONS(1769),
    [sym_custom] = ACTIONS(1769),
    [sym_atoi] = ACTIONS(1769),
    [sym_atol] = ACTIONS(1769),
    [sym_atof] = ACTIONS(1769),
    [sym_itoa] = ACTIONS(1769),
    [sym_itohex] = ACTIONS(1769),
    [sym_hextoi] = ACTIONS(1769),
    [sym_ftoa] = ACTIONS(1769),
    [sym_find_string] = ACTIONS(1769),
    [sym_left_string] = ACTIONS(1769),
    [sym_right_string] = ACTIONS(1769),
    [sym_mid_string] = ACTIONS(1769),
    [sym_remove_string] = ACTIONS(1769),
    [sym_get_buffer_string] = ACTIONS(1769),
    [sym_get_buffer_char] = ACTIONS(1769),
    [sym_get_multi_buffer_string] = ACTIONS(1769),
    [sym_length_string] = ACTIONS(1769),
    [sym_max_length_string] = ACTIONS(1769),
    [sym_set_length_string] = ACTIONS(1769),
    [sym_compare_string] = ACTIONS(1769),
    [sym_upper_string] = ACTIONS(1769),
    [sym_lower_string] = ACTIONS(1769),
    [sym_get_last] = ACTIONS(1769),
    [sym_date_to_day] = ACTIONS(1769),
    [sym_date_to_month] = ACTIONS(1769),
    [sym_date_to_year] = ACTIONS(1769),
    [sym_time_to_hour] = ACTIONS(1769),
    [sym_time_to_minute] = ACTIONS(1769),
    [sym_time_to_second] = ACTIONS(1769),
    [sym_day_of_week] = ACTIONS(1769),
    [sym_set_pulse_time] = ACTIONS(1769),
    [sym_set_timer] = ACTIONS(1769),
    [sym_file_open] = ACTIONS(1769),
    [sym_file_close] = ACTIONS(1769),
    [sym_file_read] = ACTIONS(1769),
    [sym_file_read_line] = ACTIONS(1769),
    [sym_file_write] = ACTIONS(1769),
    [sym_file_write_line] = ACTIONS(1769),
    [sym_file_copy] = ACTIONS(1769),
    [sym_file_delete] = ACTIONS(1769),
    [sym_file_rename] = ACTIONS(1769),
    [sym_file_dir] = ACTIONS(1769),
    [sym_file_getdir] = ACTIONS(1769),
    [sym_file_setdir] = ACTIONS(1769),
    [sym_file_create_dir] = ACTIONS(1769),
    [sym_file_remove_dir] = ACTIONS(1769),
    [sym_file_seek] = ACTIONS(1769),
    [sym_length_array] = ACTIONS(1769),
    [sym_max_length_array] = ACTIONS(1769),
    [sym_set_length_array] = ACTIONS(1769),
    [sym_ip_client_open] = ACTIONS(1769),
    [sym_ip_client_close] = ACTIONS(1769),
    [sym_ip_server_open] = ACTIONS(1769),
    [sym_ip_server_close] = ACTIONS(1769),
    [sym_ip_bound_client_open] = ACTIONS(1769),
    [sym_ip_mc_server_open] = ACTIONS(1769),
    [sym_ip_set_option] = ACTIONS(1769),
    [sym_ssh_client_open] = ACTIONS(1769),
    [sym_ssh_client_close] = ACTIONS(1769),
    [sym_tls_client_open] = ACTIONS(1769),
    [sym_tls_client_close] = ACTIONS(1769),
    [sym_do_push] = ACTIONS(1769),
    [sym_do_push_timed] = ACTIONS(1769),
    [sym_do_release] = ACTIONS(1769),
    [sym_device_id] = ACTIONS(1769),
    [sym_device_info] = ACTIONS(1769),
    [sym_device_id_string] = ACTIONS(1769),
    [sym_device_standby] = ACTIONS(1769),
    [sym_device_wake] = ACTIONS(1769),
    [sym_timeline_create] = ACTIONS(1769),
    [sym_timeline_kill] = ACTIONS(1769),
    [sym_timeline_reload] = ACTIONS(1769),
    [sym_timeline_pause] = ACTIONS(1769),
    [sym_timeline_restart] = ACTIONS(1769),
    [sym_timeline_set] = ACTIONS(1769),
    [sym_timeline_get] = ACTIONS(1769),
    [sym_timeline_active] = ACTIONS(1769),
    [sym_timeline_dynamic_id] = ACTIONS(1769),
    [sym_variable_to_string] = ACTIONS(1769),
    [sym_string_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_string] = ACTIONS(1769),
    [sym_type_cast] = ACTIONS(1769),
    [sym_variable_to_xml] = ACTIONS(1769),
    [sym_xml_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_xml] = ACTIONS(1769),
    [sym_redirect_string] = ACTIONS(1769),
    [sym_random_number] = ACTIONS(1769),
    [sym_get_unique_id] = ACTIONS(1769),
    [sym_get_system_number] = ACTIONS(1769),
    [sym_set_system_number] = ACTIONS(1769),
    [sym_get_serial_number] = ACTIONS(1769),
    [sym_get_master_build] = ACTIONS(1769),
    [sym_set_validation_code] = ACTIONS(1769),
    [sym_reboot] = ACTIONS(1769),
    [sym_load_duet_module] = ACTIONS(1769),
    [sym_unload_duet_module] = ACTIONS(1769),
    [sym_duet_mem_size_set] = ACTIONS(1769),
    [sym_duet_mem_size_get] = ACTIONS(1769),
    [sym_dynamic_application_device] = ACTIONS(1769),
    [sym_dynamic_polled_port] = ACTIONS(1769),
    [sym_static_port_binding] = ACTIONS(1769),
    [sym_static_ip_binding] = ACTIONS(1769),
    [sym_get_dns_list] = ACTIONS(1769),
    [sym_set_dns_list] = ACTIONS(1769),
    [sym_get_ip_address] = ACTIONS(1769),
    [sym_set_ip_address] = ACTIONS(1769),
    [sym_get_url_list] = ACTIONS(1769),
    [sym_add_url_entry] = ACTIONS(1769),
    [sym_delete_url_entry] = ACTIONS(1769),
    [sym_abs_value] = ACTIONS(1769),
    [sym_max_value] = ACTIONS(1769),
    [sym_min_value] = ACTIONS(1769),
    [sym_exp_value] = ACTIONS(1769),
    [sym_log_value] = ACTIONS(1769),
    [sym_log10_value] = ACTIONS(1769),
    [sym_power_value] = ACTIONS(1769),
    [sym_sqrt_value] = ACTIONS(1769),
    [sym_raw_be] = ACTIONS(1769),
    [sym_raw_le] = ACTIONS(1769),
    [sym_internal_queue_size_set] = ACTIONS(1769),
    [sym_internal_queue_size_get] = ACTIONS(1769),
    [sym_internal_threshold_set] = ACTIONS(1769),
    [sym_internal_threshold_get] = ACTIONS(1769),
    [sym_rebuilt_event] = ACTIONS(1769),
    [sym_do_custom_event] = ACTIONS(1769),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1769),
    [sym_clkmgr_set_clk_source] = ACTIONS(1769),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1769),
    [sym_clkmgr_get_timezone] = ACTIONS(1769),
    [sym_clkmgr_set_timezone] = ACTIONS(1769),
    [sym_clkmgr_get_resync_period] = ACTIONS(1769),
    [sym_clkmgr_set_resync_period] = ACTIONS(1769),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_timeservers] = ACTIONS(1769),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_set_log_level] = ACTIONS(1769),
    [sym_get_log_level] = ACTIONS(1769),
    [sym_amx_log] = ACTIONS(1769),
    [sym_smtp_server_config_set] = ACTIONS(1769),
    [sym_smtp_server_config_get] = ACTIONS(1769),
    [sym_smtp_send] = ACTIONS(1769),
    [sym_validate_netlinx_account] = ACTIONS(1769),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1769),
    [sym_authenticate_certificate] = ACTIONS(1769),
    [sym_audit_netlinx_session_event] = ACTIONS(1769),
    [sym_audit_netlinx_generic_event] = ACTIONS(1769),
    [sym_get_available_flash_disk_space] = ACTIONS(1769),
    [sym_get_max_flash_disk_space] = ACTIONS(1769),
    [sym_data_create_feed] = ACTIONS(1769),
    [sym_data_delete_feed] = ACTIONS(1769),
    [sym_data_publish_feed] = ACTIONS(1769),
    [sym_data_get_published_feed] = ACTIONS(1769),
    [sym_data_add_record] = ACTIONS(1769),
    [sym_data_get_event_record] = ACTIONS(1769),
    [sym__wc_data_create_feed] = ACTIONS(1769),
    [sym__wc_data_add_record] = ACTIONS(1769),
    [sym__wc_data_get_event_record] = ACTIONS(1769),
    [sym_format] = ACTIONS(1769),
    [sym_astro_clock] = ACTIONS(1769),
    [sym_set_outdoor_temperature] = ACTIONS(1769),
    [sym_set_virtual_level_count] = ACTIONS(1769),
    [sym_set_virtual_channel_count] = ACTIONS(1769),
    [sym_set_virtual_port_count] = ACTIONS(1769),
    [sym__wc_encode_int] = ACTIONS(1769),
    [sym__wc_decode_int] = ACTIONS(1769),
    [sym__wc_int] = ACTIONS(1769),
    [sym__wc_to_ch_int] = ACTIONS(1769),
    [sym__ch_to_wc_int] = ACTIONS(1769),
    [sym__wc_find_string_int] = ACTIONS(1769),
    [sym__wc_left_string_int] = ACTIONS(1769),
    [sym__wc_lower_string_int] = ACTIONS(1769),
    [sym__wc_mid_string_int] = ACTIONS(1769),
    [sym__wc_remove_string_int] = ACTIONS(1769),
    [sym__wc_right_string_int] = ACTIONS(1769),
    [sym__wc_upper_string_int] = ACTIONS(1769),
    [sym__wc_compare_string_int] = ACTIONS(1769),
    [sym__wc_get_buffer_char_int] = ACTIONS(1769),
    [sym__wc_get_buffer_string_int] = ACTIONS(1769),
    [sym__wc_concat_string_int] = ACTIONS(1769),
    [sym__wc_file_open_int] = ACTIONS(1769),
    [sym__wc_file_close_int] = ACTIONS(1769),
    [sym__wc_file_read_int] = ACTIONS(1769),
    [sym__wc_file_read_line_int] = ACTIONS(1769),
    [sym__wc_file_write_int] = ACTIONS(1769),
    [sym__wc_file_write_line_int] = ACTIONS(1769),
    [aux_sym_preproc_include_token1] = ACTIONS(1769),
    [aux_sym_preproc_define_token1] = ACTIONS(1771),
    [aux_sym_preproc_warn_token1] = ACTIONS(1771),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1771),
    [anon_sym_LBRACK] = ACTIONS(1771),
    [anon_sym_LBRACE] = ACTIONS(1771),
    [anon_sym_RBRACE] = ACTIONS(1771),
    [anon_sym_LPAREN] = ACTIONS(1769),
    [anon_sym_BANG] = ACTIONS(1771),
    [anon_sym_TILDE] = ACTIONS(1771),
    [anon_sym_DASH] = ACTIONS(1769),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_DASH_DASH] = ACTIONS(1771),
    [anon_sym_PLUS_PLUS] = ACTIONS(1771),
    [anon_sym_DQUOTE] = ACTIONS(1771),
    [anon_sym_SQUOTE] = ACTIONS(1771),
    [sym_number_literal] = ACTIONS(1771),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(485)] = {
    [sym_identifier] = ACTIONS(1769),
    [sym_define_call_keyword] = ACTIONS(1769),
    [sym_define_function_keyword] = ACTIONS(1769),
    [sym_define_library_function_keyword] = ACTIONS(1769),
    [sym_define_module_keyword] = ACTIONS(1769),
    [sym_char_keyword] = ACTIONS(1769),
    [sym_widechar_keyword] = ACTIONS(1769),
    [sym_integer_keyword] = ACTIONS(1769),
    [sym_sinteger_keyword] = ACTIONS(1769),
    [sym_long_keyword] = ACTIONS(1769),
    [sym_slong_keyword] = ACTIONS(1769),
    [sym_float_keyword] = ACTIONS(1769),
    [sym_double_keyword] = ACTIONS(1769),
    [sym_variant_keyword] = ACTIONS(1769),
    [sym_variantarray_keyword] = ACTIONS(1769),
    [sym_local_var_keyword] = ACTIONS(1769),
    [sym_stack_var_keyword] = ACTIONS(1769),
    [sym_constant_keyword] = ACTIONS(1769),
    [sym_volatile_keyword] = ACTIONS(1769),
    [sym_non_volatile_keyword] = ACTIONS(1769),
    [sym_persistent_keyword] = ACTIONS(1769),
    [sym_dev_keyword] = ACTIONS(1769),
    [sym_devlev_keyword] = ACTIONS(1769),
    [sym_devchan_keyword] = ACTIONS(1769),
    [sym_if_keyword] = ACTIONS(1769),
    [sym_else_keyword] = ACTIONS(1769),
    [sym_switch_keyword] = ACTIONS(1769),
    [sym_case_keyword] = ACTIONS(1769),
    [sym_default_keyword] = ACTIONS(1769),
    [sym_while_keyword] = ACTIONS(1769),
    [sym_for_keyword] = ACTIONS(1769),
    [sym_break_keyword] = ACTIONS(1769),
    [sym_continue_keyword] = ACTIONS(1769),
    [sym_return_keyword] = ACTIONS(1769),
    [sym_select_keyword] = ACTIONS(1769),
    [sym_active_keyword] = ACTIONS(1769),
    [sym_struct_keyword] = ACTIONS(1769),
    [sym_structure_keyword] = ACTIONS(1769),
    [aux_sym_on_keyword_token1] = ACTIONS(1769),
    [aux_sym_off_keyword_token1] = ACTIONS(1769),
    [sym_send_level_keyword] = ACTIONS(1769),
    [sym_send_string_keyword] = ACTIONS(1769),
    [sym_send_command_keyword] = ACTIONS(1769),
    [sym_clear_buffer_keyword] = ACTIONS(1769),
    [sym_create_buffer_keyword] = ACTIONS(1769),
    [sym_create_multi_buffer_keyword] = ACTIONS(1769),
    [sym_call_keyword] = ACTIONS(1769),
    [sym_system_call_keyword] = ACTIONS(1769),
    [sym_devchan_to_keyword] = ACTIONS(1769),
    [sym_devchan_min_to_keyword] = ACTIONS(1769),
    [sym_devchan_total_off_keyword] = ACTIONS(1769),
    [sym_devchan_pulse_keyword] = ACTIONS(1769),
    [sym_wait_keyword] = ACTIONS(1769),
    [sym_cancel_wait_keyword] = ACTIONS(1769),
    [sym_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1769),
    [sym_not] = ACTIONS(1769),
    [sym_bnot] = ACTIONS(1769),
    [sym___file__] = ACTIONS(1769),
    [sym___date__] = ACTIONS(1769),
    [sym___ldate__] = ACTIONS(1769),
    [sym___time__] = ACTIONS(1769),
    [sym___line__] = ACTIONS(1769),
    [sym___name__] = ACTIONS(1769),
    [sym___netlinx__] = ACTIONS(1769),
    [sym_netlinx_axi_version] = ACTIONS(1769),
    [sym_url_flg_tcp] = ACTIONS(1769),
    [sym_url_flg_accinfopresent] = ACTIONS(1769),
    [sym_url_flg_temp] = ACTIONS(1769),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1769),
    [sym_url_flg_stat_mask] = ACTIONS(1769),
    [sym_url_flg_stat_lookup] = ACTIONS(1769),
    [sym_url_flg_stat_connecting] = ACTIONS(1769),
    [sym_url_flg_stat_waiting] = ACTIONS(1769),
    [sym_url_flg_stat_connected] = ACTIONS(1769),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1769),
    [sym_first_virtual_device] = ACTIONS(1769),
    [sym_first_local_port] = ACTIONS(1769),
    [sym_dynamic_virtual_device] = ACTIONS(1769),
    [sym_timeline_once] = ACTIONS(1769),
    [sym_timeline_repeat] = ACTIONS(1769),
    [sym_timeline_absolute] = ACTIONS(1769),
    [sym_timeline_relative] = ACTIONS(1769),
    [sym_do_push_timed_infinite] = ACTIONS(1769),
    [sym_source_type_no_address] = ACTIONS(1769),
    [sym_source_type_neuron_id] = ACTIONS(1769),
    [sym_source_type_ip_address] = ACTIONS(1769),
    [sym_source_type_axlink] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1769),
    [sym_source_type_ip_socket_address] = ACTIONS(1769),
    [sym_source_type_rs232] = ACTIONS(1769),
    [sym_source_type_internal] = ACTIONS(1769),
    [sym_source_type_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1769),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1769),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1769),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1769),
    [sym_true] = ACTIONS(1769),
    [sym_false] = ACTIONS(1769),
    [sym_file_read_only] = ACTIONS(1769),
    [sym_file_rw_new] = ACTIONS(1769),
    [sym_file_rw_append] = ACTIONS(1769),
    [sym_ip_tcp] = ACTIONS(1769),
    [sym_ip_udp] = ACTIONS(1769),
    [sym_ip_udp_2way] = ACTIONS(1769),
    [sym_xml_encode_types] = ACTIONS(1769),
    [sym_xml_encode_char_as_list] = ACTIONS(1769),
    [sym_xml_encode_le] = ACTIONS(1769),
    [sym_xml_decode_types] = ACTIONS(1769),
    [sym_xml_decode_no_preserve] = ACTIONS(1769),
    [sym_duet_dev_type_display_device] = ACTIONS(1769),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_io_device] = ACTIONS(1769),
    [sym_duet_dev_type_relay_device] = ACTIONS(1769),
    [sym_duet_dev_type_ups] = ACTIONS(1769),
    [sym_duet_dev_type_amplifier] = ACTIONS(1769),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1769),
    [sym_duet_dev_type_camera] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1769),
    [sym_duet_dev_type_dss] = ACTIONS(1769),
    [sym_duet_dev_type_dvr] = ACTIONS(1769),
    [sym_duet_dev_type_disc_device] = ACTIONS(1769),
    [sym_duet_dev_type_document_camera] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1769),
    [sym_duet_dev_type_hvac] = ACTIONS(1769),
    [sym_duet_dev_type_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_light] = ACTIONS(1769),
    [sym_duet_dev_type_monitor] = ACTIONS(1769),
    [sym_duet_dev_type_motor] = ACTIONS(1769),
    [sym_duet_dev_type_multi_window] = ACTIONS(1769),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1769),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1769),
    [sym_duet_dev_type_receiver] = ACTIONS(1769),
    [sym_duet_dev_type_security_system] = ACTIONS(1769),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1769),
    [sym_duet_dev_type_settop_box] = ACTIONS(1769),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1769),
    [sym_duet_dev_type_switcher] = ACTIONS(1769),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_tv] = ACTIONS(1769),
    [sym_duet_dev_type_utility] = ACTIONS(1769),
    [sym_duet_dev_type_vcr] = ACTIONS(1769),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_video_processor] = ACTIONS(1769),
    [sym_duet_dev_type_video_projector] = ACTIONS(1769),
    [sym_duet_dev_type_video_wall] = ACTIONS(1769),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1769),
    [sym_duet_dev_type_weather] = ACTIONS(1769),
    [sym_duet_dev_not_polled] = ACTIONS(1769),
    [sym_duet_dev_polled] = ACTIONS(1769),
    [sym_clkmgr_mode_network] = ACTIONS(1769),
    [sym_clkmgr_mode_standalone] = ACTIONS(1769),
    [sym_normal_standby] = ACTIONS(1769),
    [sym_normal_wake] = ACTIONS(1769),
    [sym_ip_multicast_ttl_option] = ACTIONS(1769),
    [sym_ip_ttl_subnet] = ACTIONS(1769),
    [sym_ip_ttl_site] = ACTIONS(1769),
    [sym_ip_ttl_region] = ACTIONS(1769),
    [sym_ip_ttl_continent] = ACTIONS(1769),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1769),
    [sym_ip_nodelay_on] = ACTIONS(1769),
    [sym_ip_nodelay_off] = ACTIONS(1769),
    [sym_amx_error] = ACTIONS(1769),
    [sym_amx_warning] = ACTIONS(1769),
    [sym_amx_info] = ACTIONS(1769),
    [sym_amx_debug] = ACTIONS(1769),
    [sym_netlinx_login_success] = ACTIONS(1769),
    [sym_netlinx_login_fail] = ACTIONS(1769),
    [sym_netlinx_logout] = ACTIONS(1769),
    [sym_null_str] = ACTIONS(1769),
    [sym_smtp_address] = ACTIONS(1769),
    [sym_smtp_port_number] = ACTIONS(1769),
    [sym_smtp_username] = ACTIONS(1769),
    [sym_smtp_password] = ACTIONS(1769),
    [sym_smtp_require_tls] = ACTIONS(1769),
    [sym_smtp_from] = ACTIONS(1769),
    [sym_smtp_tls_true] = ACTIONS(1769),
    [sym_smtp_tls_false] = ACTIONS(1769),
    [sym_listview_on_row_select_event] = ACTIONS(1769),
    [sym_data_structure_unknown] = ACTIONS(1769),
    [sym_data_structure_datafeed] = ACTIONS(1769),
    [sym_data_structure_datarecordset] = ACTIONS(1769),
    [sym_data_structure_datarecord] = ACTIONS(1769),
    [sym_data_structure_datafield] = ACTIONS(1769),
    [sym_data_max_content_field_count] = ACTIONS(1769),
    [sym_data_max_metadata_field_count] = ACTIONS(1769),
    [sym_data_max_id_length] = ACTIONS(1769),
    [sym_data_max_name_length] = ACTIONS(1769),
    [sym_data_max_description_length] = ACTIONS(1769),
    [sym_data_max_type_length] = ACTIONS(1769),
    [sym_data_max_format_length] = ACTIONS(1769),
    [sym_data_max_source_length] = ACTIONS(1769),
    [sym_data_max_value_length] = ACTIONS(1769),
    [sym_data_max_label_length] = ACTIONS(1769),
    [sym_data_max_url_length] = ACTIONS(1769),
    [sym_max_last_login_info_length] = ACTIONS(1769),
    [sym_data_type_unknown] = ACTIONS(1769),
    [sym_data_type_string] = ACTIONS(1769),
    [sym_data_type_datetime] = ACTIONS(1769),
    [sym_data_type_date] = ACTIONS(1769),
    [sym_data_type_time] = ACTIONS(1769),
    [sym_data_type_image] = ACTIONS(1769),
    [sym_data_format_url] = ACTIONS(1769),
    [sym_data_format_phone] = ACTIONS(1769),
    [sym_data_format_email] = ACTIONS(1769),
    [sym_data_format_iso8601] = ACTIONS(1769),
    [sym_auditlog_priv] = ACTIONS(1769),
    [sym_configuration_priv] = ACTIONS(1769),
    [sym_deviceconfig_priv] = ACTIONS(1769),
    [sym_ftp_priv] = ACTIONS(1769),
    [sym_http_priv] = ACTIONS(1769),
    [sym_networkconfig_priv] = ACTIONS(1769),
    [sym_programport_priv] = ACTIONS(1769),
    [sym_remoteui_priv] = ACTIONS(1769),
    [sym_securitycontrol_priv] = ACTIONS(1769),
    [sym_softwaremanagement_priv] = ACTIONS(1769),
    [sym_terminal_priv] = ACTIONS(1769),
    [sym_tpadmin_priv] = ACTIONS(1769),
    [sym_user1_priv] = ACTIONS(1769),
    [sym_user2_priv] = ACTIONS(1769),
    [sym_user3_priv] = ACTIONS(1769),
    [sym_user4_priv] = ACTIONS(1769),
    [sym_usermanagement_priv] = ACTIONS(1769),
    [sym_valid_account] = ACTIONS(1769),
    [sym_err_name_invalid] = ACTIONS(1769),
    [sym_err_password_invalid] = ACTIONS(1769),
    [sym_err_invalid_account] = ACTIONS(1769),
    [sym_err_no_authorization] = ACTIONS(1769),
    [sym_err_invalid_parameter] = ACTIONS(1769),
    [sym_err_user_locked_out] = ACTIONS(1769),
    [sym_err_user_account_expired] = ACTIONS(1769),
    [sym_err_general_error] = ACTIONS(1769),
    [sym_err_authentication_error] = ACTIONS(1769),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1769),
    [sym_tls_validate_certificate] = ACTIONS(1769),
    [sym_tdata] = ACTIONS(1769),
    [sym_tchannel] = ACTIONS(1769),
    [sym_tlevel] = ACTIONS(1769),
    [sym_tbutton] = ACTIONS(1769),
    [sym_ttimeline] = ACTIONS(1769),
    [sym_tcustom] = ACTIONS(1769),
    [sym_url_struct] = ACTIONS(1769),
    [sym_dns_struct] = ACTIONS(1769),
    [sym_ip_address_struct] = ACTIONS(1769),
    [sym_dev_info_struct] = ACTIONS(1769),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1769),
    [sym_data_feed] = ACTIONS(1769),
    [sym_data_field] = ACTIONS(1769),
    [sym_data_record] = ACTIONS(1769),
    [sym_wc_data_feed] = ACTIONS(1769),
    [sym_wc_data_field] = ACTIONS(1769),
    [sym_wc_data_record] = ACTIONS(1769),
    [sym_last_login_info] = ACTIONS(1769),
    [sym____reserved___] = ACTIONS(1769),
    [sym_date] = ACTIONS(1769),
    [sym_day] = ACTIONS(1769),
    [sym_time] = ACTIONS(1769),
    [sym_ldate] = ACTIONS(1769),
    [sym_push_device] = ACTIONS(1769),
    [sym_push_channel] = ACTIONS(1769),
    [sym_push_devchan] = ACTIONS(1769),
    [sym_release_device] = ACTIONS(1769),
    [sym_release_channel] = ACTIONS(1769),
    [sym_release_devchan] = ACTIONS(1769),
    [sym_master_slot] = ACTIONS(1769),
    [sym_get_pulse_time] = ACTIONS(1769),
    [sym_get_timer] = ACTIONS(1769),
    [sym_master_sn] = ACTIONS(1769),
    [sym_system_number] = ACTIONS(1769),
    [sym_button] = ACTIONS(1769),
    [sym_dv_channel] = ACTIONS(1769),
    [sym_data] = ACTIONS(1769),
    [sym_level] = ACTIONS(1769),
    [sym_timeline] = ACTIONS(1769),
    [sym_channel] = ACTIONS(1769),
    [sym_custom] = ACTIONS(1769),
    [sym_atoi] = ACTIONS(1769),
    [sym_atol] = ACTIONS(1769),
    [sym_atof] = ACTIONS(1769),
    [sym_itoa] = ACTIONS(1769),
    [sym_itohex] = ACTIONS(1769),
    [sym_hextoi] = ACTIONS(1769),
    [sym_ftoa] = ACTIONS(1769),
    [sym_find_string] = ACTIONS(1769),
    [sym_left_string] = ACTIONS(1769),
    [sym_right_string] = ACTIONS(1769),
    [sym_mid_string] = ACTIONS(1769),
    [sym_remove_string] = ACTIONS(1769),
    [sym_get_buffer_string] = ACTIONS(1769),
    [sym_get_buffer_char] = ACTIONS(1769),
    [sym_get_multi_buffer_string] = ACTIONS(1769),
    [sym_length_string] = ACTIONS(1769),
    [sym_max_length_string] = ACTIONS(1769),
    [sym_set_length_string] = ACTIONS(1769),
    [sym_compare_string] = ACTIONS(1769),
    [sym_upper_string] = ACTIONS(1769),
    [sym_lower_string] = ACTIONS(1769),
    [sym_get_last] = ACTIONS(1769),
    [sym_date_to_day] = ACTIONS(1769),
    [sym_date_to_month] = ACTIONS(1769),
    [sym_date_to_year] = ACTIONS(1769),
    [sym_time_to_hour] = ACTIONS(1769),
    [sym_time_to_minute] = ACTIONS(1769),
    [sym_time_to_second] = ACTIONS(1769),
    [sym_day_of_week] = ACTIONS(1769),
    [sym_set_pulse_time] = ACTIONS(1769),
    [sym_set_timer] = ACTIONS(1769),
    [sym_file_open] = ACTIONS(1769),
    [sym_file_close] = ACTIONS(1769),
    [sym_file_read] = ACTIONS(1769),
    [sym_file_read_line] = ACTIONS(1769),
    [sym_file_write] = ACTIONS(1769),
    [sym_file_write_line] = ACTIONS(1769),
    [sym_file_copy] = ACTIONS(1769),
    [sym_file_delete] = ACTIONS(1769),
    [sym_file_rename] = ACTIONS(1769),
    [sym_file_dir] = ACTIONS(1769),
    [sym_file_getdir] = ACTIONS(1769),
    [sym_file_setdir] = ACTIONS(1769),
    [sym_file_create_dir] = ACTIONS(1769),
    [sym_file_remove_dir] = ACTIONS(1769),
    [sym_file_seek] = ACTIONS(1769),
    [sym_length_array] = ACTIONS(1769),
    [sym_max_length_array] = ACTIONS(1769),
    [sym_set_length_array] = ACTIONS(1769),
    [sym_ip_client_open] = ACTIONS(1769),
    [sym_ip_client_close] = ACTIONS(1769),
    [sym_ip_server_open] = ACTIONS(1769),
    [sym_ip_server_close] = ACTIONS(1769),
    [sym_ip_bound_client_open] = ACTIONS(1769),
    [sym_ip_mc_server_open] = ACTIONS(1769),
    [sym_ip_set_option] = ACTIONS(1769),
    [sym_ssh_client_open] = ACTIONS(1769),
    [sym_ssh_client_close] = ACTIONS(1769),
    [sym_tls_client_open] = ACTIONS(1769),
    [sym_tls_client_close] = ACTIONS(1769),
    [sym_do_push] = ACTIONS(1769),
    [sym_do_push_timed] = ACTIONS(1769),
    [sym_do_release] = ACTIONS(1769),
    [sym_device_id] = ACTIONS(1769),
    [sym_device_info] = ACTIONS(1769),
    [sym_device_id_string] = ACTIONS(1769),
    [sym_device_standby] = ACTIONS(1769),
    [sym_device_wake] = ACTIONS(1769),
    [sym_timeline_create] = ACTIONS(1769),
    [sym_timeline_kill] = ACTIONS(1769),
    [sym_timeline_reload] = ACTIONS(1769),
    [sym_timeline_pause] = ACTIONS(1769),
    [sym_timeline_restart] = ACTIONS(1769),
    [sym_timeline_set] = ACTIONS(1769),
    [sym_timeline_get] = ACTIONS(1769),
    [sym_timeline_active] = ACTIONS(1769),
    [sym_timeline_dynamic_id] = ACTIONS(1769),
    [sym_variable_to_string] = ACTIONS(1769),
    [sym_string_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_string] = ACTIONS(1769),
    [sym_type_cast] = ACTIONS(1769),
    [sym_variable_to_xml] = ACTIONS(1769),
    [sym_xml_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_xml] = ACTIONS(1769),
    [sym_redirect_string] = ACTIONS(1769),
    [sym_random_number] = ACTIONS(1769),
    [sym_get_unique_id] = ACTIONS(1769),
    [sym_get_system_number] = ACTIONS(1769),
    [sym_set_system_number] = ACTIONS(1769),
    [sym_get_serial_number] = ACTIONS(1769),
    [sym_get_master_build] = ACTIONS(1769),
    [sym_set_validation_code] = ACTIONS(1769),
    [sym_reboot] = ACTIONS(1769),
    [sym_load_duet_module] = ACTIONS(1769),
    [sym_unload_duet_module] = ACTIONS(1769),
    [sym_duet_mem_size_set] = ACTIONS(1769),
    [sym_duet_mem_size_get] = ACTIONS(1769),
    [sym_dynamic_application_device] = ACTIONS(1769),
    [sym_dynamic_polled_port] = ACTIONS(1769),
    [sym_static_port_binding] = ACTIONS(1769),
    [sym_static_ip_binding] = ACTIONS(1769),
    [sym_get_dns_list] = ACTIONS(1769),
    [sym_set_dns_list] = ACTIONS(1769),
    [sym_get_ip_address] = ACTIONS(1769),
    [sym_set_ip_address] = ACTIONS(1769),
    [sym_get_url_list] = ACTIONS(1769),
    [sym_add_url_entry] = ACTIONS(1769),
    [sym_delete_url_entry] = ACTIONS(1769),
    [sym_abs_value] = ACTIONS(1769),
    [sym_max_value] = ACTIONS(1769),
    [sym_min_value] = ACTIONS(1769),
    [sym_exp_value] = ACTIONS(1769),
    [sym_log_value] = ACTIONS(1769),
    [sym_log10_value] = ACTIONS(1769),
    [sym_power_value] = ACTIONS(1769),
    [sym_sqrt_value] = ACTIONS(1769),
    [sym_raw_be] = ACTIONS(1769),
    [sym_raw_le] = ACTIONS(1769),
    [sym_internal_queue_size_set] = ACTIONS(1769),
    [sym_internal_queue_size_get] = ACTIONS(1769),
    [sym_internal_threshold_set] = ACTIONS(1769),
    [sym_internal_threshold_get] = ACTIONS(1769),
    [sym_rebuilt_event] = ACTIONS(1769),
    [sym_do_custom_event] = ACTIONS(1769),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1769),
    [sym_clkmgr_set_clk_source] = ACTIONS(1769),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1769),
    [sym_clkmgr_get_timezone] = ACTIONS(1769),
    [sym_clkmgr_set_timezone] = ACTIONS(1769),
    [sym_clkmgr_get_resync_period] = ACTIONS(1769),
    [sym_clkmgr_set_resync_period] = ACTIONS(1769),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_timeservers] = ACTIONS(1769),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_set_log_level] = ACTIONS(1769),
    [sym_get_log_level] = ACTIONS(1769),
    [sym_amx_log] = ACTIONS(1769),
    [sym_smtp_server_config_set] = ACTIONS(1769),
    [sym_smtp_server_config_get] = ACTIONS(1769),
    [sym_smtp_send] = ACTIONS(1769),
    [sym_validate_netlinx_account] = ACTIONS(1769),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1769),
    [sym_authenticate_certificate] = ACTIONS(1769),
    [sym_audit_netlinx_session_event] = ACTIONS(1769),
    [sym_audit_netlinx_generic_event] = ACTIONS(1769),
    [sym_get_available_flash_disk_space] = ACTIONS(1769),
    [sym_get_max_flash_disk_space] = ACTIONS(1769),
    [sym_data_create_feed] = ACTIONS(1769),
    [sym_data_delete_feed] = ACTIONS(1769),
    [sym_data_publish_feed] = ACTIONS(1769),
    [sym_data_get_published_feed] = ACTIONS(1769),
    [sym_data_add_record] = ACTIONS(1769),
    [sym_data_get_event_record] = ACTIONS(1769),
    [sym__wc_data_create_feed] = ACTIONS(1769),
    [sym__wc_data_add_record] = ACTIONS(1769),
    [sym__wc_data_get_event_record] = ACTIONS(1769),
    [sym_format] = ACTIONS(1769),
    [sym_astro_clock] = ACTIONS(1769),
    [sym_set_outdoor_temperature] = ACTIONS(1769),
    [sym_set_virtual_level_count] = ACTIONS(1769),
    [sym_set_virtual_channel_count] = ACTIONS(1769),
    [sym_set_virtual_port_count] = ACTIONS(1769),
    [sym__wc_encode_int] = ACTIONS(1769),
    [sym__wc_decode_int] = ACTIONS(1769),
    [sym__wc_int] = ACTIONS(1769),
    [sym__wc_to_ch_int] = ACTIONS(1769),
    [sym__ch_to_wc_int] = ACTIONS(1769),
    [sym__wc_find_string_int] = ACTIONS(1769),
    [sym__wc_left_string_int] = ACTIONS(1769),
    [sym__wc_lower_string_int] = ACTIONS(1769),
    [sym__wc_mid_string_int] = ACTIONS(1769),
    [sym__wc_remove_string_int] = ACTIONS(1769),
    [sym__wc_right_string_int] = ACTIONS(1769),
    [sym__wc_upper_string_int] = ACTIONS(1769),
    [sym__wc_compare_string_int] = ACTIONS(1769),
    [sym__wc_get_buffer_char_int] = ACTIONS(1769),
    [sym__wc_get_buffer_string_int] = ACTIONS(1769),
    [sym__wc_concat_string_int] = ACTIONS(1769),
    [sym__wc_file_open_int] = ACTIONS(1769),
    [sym__wc_file_close_int] = ACTIONS(1769),
    [sym__wc_file_read_int] = ACTIONS(1769),
    [sym__wc_file_read_line_int] = ACTIONS(1769),
    [sym__wc_file_write_int] = ACTIONS(1769),
    [sym__wc_file_write_line_int] = ACTIONS(1769),
    [aux_sym_preproc_include_token1] = ACTIONS(1769),
    [aux_sym_preproc_define_token1] = ACTIONS(1771),
    [aux_sym_preproc_warn_token1] = ACTIONS(1771),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1771),
    [anon_sym_LBRACK] = ACTIONS(1771),
    [anon_sym_LBRACE] = ACTIONS(1771),
    [anon_sym_RBRACE] = ACTIONS(1771),
    [anon_sym_LPAREN] = ACTIONS(1769),
    [anon_sym_BANG] = ACTIONS(1771),
    [anon_sym_TILDE] = ACTIONS(1771),
    [anon_sym_DASH] = ACTIONS(1769),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_DASH_DASH] = ACTIONS(1771),
    [anon_sym_PLUS_PLUS] = ACTIONS(1771),
    [anon_sym_DQUOTE] = ACTIONS(1771),
    [anon_sym_SQUOTE] = ACTIONS(1771),
    [sym_number_literal] = ACTIONS(1771),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(486)] = {
    [sym_identifier] = ACTIONS(1769),
    [sym_define_call_keyword] = ACTIONS(1769),
    [sym_define_function_keyword] = ACTIONS(1769),
    [sym_define_library_function_keyword] = ACTIONS(1769),
    [sym_define_module_keyword] = ACTIONS(1769),
    [sym_char_keyword] = ACTIONS(1769),
    [sym_widechar_keyword] = ACTIONS(1769),
    [sym_integer_keyword] = ACTIONS(1769),
    [sym_sinteger_keyword] = ACTIONS(1769),
    [sym_long_keyword] = ACTIONS(1769),
    [sym_slong_keyword] = ACTIONS(1769),
    [sym_float_keyword] = ACTIONS(1769),
    [sym_double_keyword] = ACTIONS(1769),
    [sym_variant_keyword] = ACTIONS(1769),
    [sym_variantarray_keyword] = ACTIONS(1769),
    [sym_local_var_keyword] = ACTIONS(1769),
    [sym_stack_var_keyword] = ACTIONS(1769),
    [sym_constant_keyword] = ACTIONS(1769),
    [sym_volatile_keyword] = ACTIONS(1769),
    [sym_non_volatile_keyword] = ACTIONS(1769),
    [sym_persistent_keyword] = ACTIONS(1769),
    [sym_dev_keyword] = ACTIONS(1769),
    [sym_devlev_keyword] = ACTIONS(1769),
    [sym_devchan_keyword] = ACTIONS(1769),
    [sym_if_keyword] = ACTIONS(1769),
    [sym_else_keyword] = ACTIONS(1769),
    [sym_switch_keyword] = ACTIONS(1769),
    [sym_case_keyword] = ACTIONS(1769),
    [sym_default_keyword] = ACTIONS(1769),
    [sym_while_keyword] = ACTIONS(1769),
    [sym_for_keyword] = ACTIONS(1769),
    [sym_break_keyword] = ACTIONS(1769),
    [sym_continue_keyword] = ACTIONS(1769),
    [sym_return_keyword] = ACTIONS(1769),
    [sym_select_keyword] = ACTIONS(1769),
    [sym_active_keyword] = ACTIONS(1769),
    [sym_struct_keyword] = ACTIONS(1769),
    [sym_structure_keyword] = ACTIONS(1769),
    [aux_sym_on_keyword_token1] = ACTIONS(1769),
    [aux_sym_off_keyword_token1] = ACTIONS(1769),
    [sym_send_level_keyword] = ACTIONS(1769),
    [sym_send_string_keyword] = ACTIONS(1769),
    [sym_send_command_keyword] = ACTIONS(1769),
    [sym_clear_buffer_keyword] = ACTIONS(1769),
    [sym_create_buffer_keyword] = ACTIONS(1769),
    [sym_create_multi_buffer_keyword] = ACTIONS(1769),
    [sym_call_keyword] = ACTIONS(1769),
    [sym_system_call_keyword] = ACTIONS(1769),
    [sym_devchan_to_keyword] = ACTIONS(1769),
    [sym_devchan_min_to_keyword] = ACTIONS(1769),
    [sym_devchan_total_off_keyword] = ACTIONS(1769),
    [sym_devchan_pulse_keyword] = ACTIONS(1769),
    [sym_wait_keyword] = ACTIONS(1769),
    [sym_cancel_wait_keyword] = ACTIONS(1769),
    [sym_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_wait_until_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_keyword] = ACTIONS(1769),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1769),
    [sym_not] = ACTIONS(1769),
    [sym_bnot] = ACTIONS(1769),
    [sym___file__] = ACTIONS(1769),
    [sym___date__] = ACTIONS(1769),
    [sym___ldate__] = ACTIONS(1769),
    [sym___time__] = ACTIONS(1769),
    [sym___line__] = ACTIONS(1769),
    [sym___name__] = ACTIONS(1769),
    [sym___netlinx__] = ACTIONS(1769),
    [sym_netlinx_axi_version] = ACTIONS(1769),
    [sym_url_flg_tcp] = ACTIONS(1769),
    [sym_url_flg_accinfopresent] = ACTIONS(1769),
    [sym_url_flg_temp] = ACTIONS(1769),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1769),
    [sym_url_flg_stat_mask] = ACTIONS(1769),
    [sym_url_flg_stat_lookup] = ACTIONS(1769),
    [sym_url_flg_stat_connecting] = ACTIONS(1769),
    [sym_url_flg_stat_waiting] = ACTIONS(1769),
    [sym_url_flg_stat_connected] = ACTIONS(1769),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1769),
    [sym_first_virtual_device] = ACTIONS(1769),
    [sym_first_local_port] = ACTIONS(1769),
    [sym_dynamic_virtual_device] = ACTIONS(1769),
    [sym_timeline_once] = ACTIONS(1769),
    [sym_timeline_repeat] = ACTIONS(1769),
    [sym_timeline_absolute] = ACTIONS(1769),
    [sym_timeline_relative] = ACTIONS(1769),
    [sym_do_push_timed_infinite] = ACTIONS(1769),
    [sym_source_type_no_address] = ACTIONS(1769),
    [sym_source_type_neuron_id] = ACTIONS(1769),
    [sym_source_type_ip_address] = ACTIONS(1769),
    [sym_source_type_axlink] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1769),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1769),
    [sym_source_type_ip_socket_address] = ACTIONS(1769),
    [sym_source_type_rs232] = ACTIONS(1769),
    [sym_source_type_internal] = ACTIONS(1769),
    [sym_source_type_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1769),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1769),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1769),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1769),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1769),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1769),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1769),
    [sym_true] = ACTIONS(1769),
    [sym_false] = ACTIONS(1769),
    [sym_file_read_only] = ACTIONS(1769),
    [sym_file_rw_new] = ACTIONS(1769),
    [sym_file_rw_append] = ACTIONS(1769),
    [sym_ip_tcp] = ACTIONS(1769),
    [sym_ip_udp] = ACTIONS(1769),
    [sym_ip_udp_2way] = ACTIONS(1769),
    [sym_xml_encode_types] = ACTIONS(1769),
    [sym_xml_encode_char_as_list] = ACTIONS(1769),
    [sym_xml_encode_le] = ACTIONS(1769),
    [sym_xml_decode_types] = ACTIONS(1769),
    [sym_xml_decode_no_preserve] = ACTIONS(1769),
    [sym_duet_dev_type_display_device] = ACTIONS(1769),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1769),
    [sym_duet_dev_type_io_device] = ACTIONS(1769),
    [sym_duet_dev_type_relay_device] = ACTIONS(1769),
    [sym_duet_dev_type_ups] = ACTIONS(1769),
    [sym_duet_dev_type_amplifier] = ACTIONS(1769),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1769),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1769),
    [sym_duet_dev_type_camera] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1769),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1769),
    [sym_duet_dev_type_dss] = ACTIONS(1769),
    [sym_duet_dev_type_dvr] = ACTIONS(1769),
    [sym_duet_dev_type_disc_device] = ACTIONS(1769),
    [sym_duet_dev_type_document_camera] = ACTIONS(1769),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1769),
    [sym_duet_dev_type_hvac] = ACTIONS(1769),
    [sym_duet_dev_type_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_light] = ACTIONS(1769),
    [sym_duet_dev_type_monitor] = ACTIONS(1769),
    [sym_duet_dev_type_motor] = ACTIONS(1769),
    [sym_duet_dev_type_multi_window] = ACTIONS(1769),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1769),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1769),
    [sym_duet_dev_type_receiver] = ACTIONS(1769),
    [sym_duet_dev_type_security_system] = ACTIONS(1769),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1769),
    [sym_duet_dev_type_settop_box] = ACTIONS(1769),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1769),
    [sym_duet_dev_type_switcher] = ACTIONS(1769),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1769),
    [sym_duet_dev_type_tv] = ACTIONS(1769),
    [sym_duet_dev_type_utility] = ACTIONS(1769),
    [sym_duet_dev_type_vcr] = ACTIONS(1769),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1769),
    [sym_duet_dev_type_video_processor] = ACTIONS(1769),
    [sym_duet_dev_type_video_projector] = ACTIONS(1769),
    [sym_duet_dev_type_video_wall] = ACTIONS(1769),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1769),
    [sym_duet_dev_type_weather] = ACTIONS(1769),
    [sym_duet_dev_not_polled] = ACTIONS(1769),
    [sym_duet_dev_polled] = ACTIONS(1769),
    [sym_clkmgr_mode_network] = ACTIONS(1769),
    [sym_clkmgr_mode_standalone] = ACTIONS(1769),
    [sym_normal_standby] = ACTIONS(1769),
    [sym_normal_wake] = ACTIONS(1769),
    [sym_ip_multicast_ttl_option] = ACTIONS(1769),
    [sym_ip_ttl_subnet] = ACTIONS(1769),
    [sym_ip_ttl_site] = ACTIONS(1769),
    [sym_ip_ttl_region] = ACTIONS(1769),
    [sym_ip_ttl_continent] = ACTIONS(1769),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1769),
    [sym_ip_nodelay_on] = ACTIONS(1769),
    [sym_ip_nodelay_off] = ACTIONS(1769),
    [sym_amx_error] = ACTIONS(1769),
    [sym_amx_warning] = ACTIONS(1769),
    [sym_amx_info] = ACTIONS(1769),
    [sym_amx_debug] = ACTIONS(1769),
    [sym_netlinx_login_success] = ACTIONS(1769),
    [sym_netlinx_login_fail] = ACTIONS(1769),
    [sym_netlinx_logout] = ACTIONS(1769),
    [sym_null_str] = ACTIONS(1769),
    [sym_smtp_address] = ACTIONS(1769),
    [sym_smtp_port_number] = ACTIONS(1769),
    [sym_smtp_username] = ACTIONS(1769),
    [sym_smtp_password] = ACTIONS(1769),
    [sym_smtp_require_tls] = ACTIONS(1769),
    [sym_smtp_from] = ACTIONS(1769),
    [sym_smtp_tls_true] = ACTIONS(1769),
    [sym_smtp_tls_false] = ACTIONS(1769),
    [sym_listview_on_row_select_event] = ACTIONS(1769),
    [sym_data_structure_unknown] = ACTIONS(1769),
    [sym_data_structure_datafeed] = ACTIONS(1769),
    [sym_data_structure_datarecordset] = ACTIONS(1769),
    [sym_data_structure_datarecord] = ACTIONS(1769),
    [sym_data_structure_datafield] = ACTIONS(1769),
    [sym_data_max_content_field_count] = ACTIONS(1769),
    [sym_data_max_metadata_field_count] = ACTIONS(1769),
    [sym_data_max_id_length] = ACTIONS(1769),
    [sym_data_max_name_length] = ACTIONS(1769),
    [sym_data_max_description_length] = ACTIONS(1769),
    [sym_data_max_type_length] = ACTIONS(1769),
    [sym_data_max_format_length] = ACTIONS(1769),
    [sym_data_max_source_length] = ACTIONS(1769),
    [sym_data_max_value_length] = ACTIONS(1769),
    [sym_data_max_label_length] = ACTIONS(1769),
    [sym_data_max_url_length] = ACTIONS(1769),
    [sym_max_last_login_info_length] = ACTIONS(1769),
    [sym_data_type_unknown] = ACTIONS(1769),
    [sym_data_type_string] = ACTIONS(1769),
    [sym_data_type_datetime] = ACTIONS(1769),
    [sym_data_type_date] = ACTIONS(1769),
    [sym_data_type_time] = ACTIONS(1769),
    [sym_data_type_image] = ACTIONS(1769),
    [sym_data_format_url] = ACTIONS(1769),
    [sym_data_format_phone] = ACTIONS(1769),
    [sym_data_format_email] = ACTIONS(1769),
    [sym_data_format_iso8601] = ACTIONS(1769),
    [sym_auditlog_priv] = ACTIONS(1769),
    [sym_configuration_priv] = ACTIONS(1769),
    [sym_deviceconfig_priv] = ACTIONS(1769),
    [sym_ftp_priv] = ACTIONS(1769),
    [sym_http_priv] = ACTIONS(1769),
    [sym_networkconfig_priv] = ACTIONS(1769),
    [sym_programport_priv] = ACTIONS(1769),
    [sym_remoteui_priv] = ACTIONS(1769),
    [sym_securitycontrol_priv] = ACTIONS(1769),
    [sym_softwaremanagement_priv] = ACTIONS(1769),
    [sym_terminal_priv] = ACTIONS(1769),
    [sym_tpadmin_priv] = ACTIONS(1769),
    [sym_user1_priv] = ACTIONS(1769),
    [sym_user2_priv] = ACTIONS(1769),
    [sym_user3_priv] = ACTIONS(1769),
    [sym_user4_priv] = ACTIONS(1769),
    [sym_usermanagement_priv] = ACTIONS(1769),
    [sym_valid_account] = ACTIONS(1769),
    [sym_err_name_invalid] = ACTIONS(1769),
    [sym_err_password_invalid] = ACTIONS(1769),
    [sym_err_invalid_account] = ACTIONS(1769),
    [sym_err_no_authorization] = ACTIONS(1769),
    [sym_err_invalid_parameter] = ACTIONS(1769),
    [sym_err_user_locked_out] = ACTIONS(1769),
    [sym_err_user_account_expired] = ACTIONS(1769),
    [sym_err_general_error] = ACTIONS(1769),
    [sym_err_authentication_error] = ACTIONS(1769),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1769),
    [sym_tls_validate_certificate] = ACTIONS(1769),
    [sym_tdata] = ACTIONS(1769),
    [sym_tchannel] = ACTIONS(1769),
    [sym_tlevel] = ACTIONS(1769),
    [sym_tbutton] = ACTIONS(1769),
    [sym_ttimeline] = ACTIONS(1769),
    [sym_tcustom] = ACTIONS(1769),
    [sym_url_struct] = ACTIONS(1769),
    [sym_dns_struct] = ACTIONS(1769),
    [sym_ip_address_struct] = ACTIONS(1769),
    [sym_dev_info_struct] = ACTIONS(1769),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1769),
    [sym_data_feed] = ACTIONS(1769),
    [sym_data_field] = ACTIONS(1769),
    [sym_data_record] = ACTIONS(1769),
    [sym_wc_data_feed] = ACTIONS(1769),
    [sym_wc_data_field] = ACTIONS(1769),
    [sym_wc_data_record] = ACTIONS(1769),
    [sym_last_login_info] = ACTIONS(1769),
    [sym____reserved___] = ACTIONS(1769),
    [sym_date] = ACTIONS(1769),
    [sym_day] = ACTIONS(1769),
    [sym_time] = ACTIONS(1769),
    [sym_ldate] = ACTIONS(1769),
    [sym_push_device] = ACTIONS(1769),
    [sym_push_channel] = ACTIONS(1769),
    [sym_push_devchan] = ACTIONS(1769),
    [sym_release_device] = ACTIONS(1769),
    [sym_release_channel] = ACTIONS(1769),
    [sym_release_devchan] = ACTIONS(1769),
    [sym_master_slot] = ACTIONS(1769),
    [sym_get_pulse_time] = ACTIONS(1769),
    [sym_get_timer] = ACTIONS(1769),
    [sym_master_sn] = ACTIONS(1769),
    [sym_system_number] = ACTIONS(1769),
    [sym_button] = ACTIONS(1769),
    [sym_dv_channel] = ACTIONS(1769),
    [sym_data] = ACTIONS(1769),
    [sym_level] = ACTIONS(1769),
    [sym_timeline] = ACTIONS(1769),
    [sym_channel] = ACTIONS(1769),
    [sym_custom] = ACTIONS(1769),
    [sym_atoi] = ACTIONS(1769),
    [sym_atol] = ACTIONS(1769),
    [sym_atof] = ACTIONS(1769),
    [sym_itoa] = ACTIONS(1769),
    [sym_itohex] = ACTIONS(1769),
    [sym_hextoi] = ACTIONS(1769),
    [sym_ftoa] = ACTIONS(1769),
    [sym_find_string] = ACTIONS(1769),
    [sym_left_string] = ACTIONS(1769),
    [sym_right_string] = ACTIONS(1769),
    [sym_mid_string] = ACTIONS(1769),
    [sym_remove_string] = ACTIONS(1769),
    [sym_get_buffer_string] = ACTIONS(1769),
    [sym_get_buffer_char] = ACTIONS(1769),
    [sym_get_multi_buffer_string] = ACTIONS(1769),
    [sym_length_string] = ACTIONS(1769),
    [sym_max_length_string] = ACTIONS(1769),
    [sym_set_length_string] = ACTIONS(1769),
    [sym_compare_string] = ACTIONS(1769),
    [sym_upper_string] = ACTIONS(1769),
    [sym_lower_string] = ACTIONS(1769),
    [sym_get_last] = ACTIONS(1769),
    [sym_date_to_day] = ACTIONS(1769),
    [sym_date_to_month] = ACTIONS(1769),
    [sym_date_to_year] = ACTIONS(1769),
    [sym_time_to_hour] = ACTIONS(1769),
    [sym_time_to_minute] = ACTIONS(1769),
    [sym_time_to_second] = ACTIONS(1769),
    [sym_day_of_week] = ACTIONS(1769),
    [sym_set_pulse_time] = ACTIONS(1769),
    [sym_set_timer] = ACTIONS(1769),
    [sym_file_open] = ACTIONS(1769),
    [sym_file_close] = ACTIONS(1769),
    [sym_file_read] = ACTIONS(1769),
    [sym_file_read_line] = ACTIONS(1769),
    [sym_file_write] = ACTIONS(1769),
    [sym_file_write_line] = ACTIONS(1769),
    [sym_file_copy] = ACTIONS(1769),
    [sym_file_delete] = ACTIONS(1769),
    [sym_file_rename] = ACTIONS(1769),
    [sym_file_dir] = ACTIONS(1769),
    [sym_file_getdir] = ACTIONS(1769),
    [sym_file_setdir] = ACTIONS(1769),
    [sym_file_create_dir] = ACTIONS(1769),
    [sym_file_remove_dir] = ACTIONS(1769),
    [sym_file_seek] = ACTIONS(1769),
    [sym_length_array] = ACTIONS(1769),
    [sym_max_length_array] = ACTIONS(1769),
    [sym_set_length_array] = ACTIONS(1769),
    [sym_ip_client_open] = ACTIONS(1769),
    [sym_ip_client_close] = ACTIONS(1769),
    [sym_ip_server_open] = ACTIONS(1769),
    [sym_ip_server_close] = ACTIONS(1769),
    [sym_ip_bound_client_open] = ACTIONS(1769),
    [sym_ip_mc_server_open] = ACTIONS(1769),
    [sym_ip_set_option] = ACTIONS(1769),
    [sym_ssh_client_open] = ACTIONS(1769),
    [sym_ssh_client_close] = ACTIONS(1769),
    [sym_tls_client_open] = ACTIONS(1769),
    [sym_tls_client_close] = ACTIONS(1769),
    [sym_do_push] = ACTIONS(1769),
    [sym_do_push_timed] = ACTIONS(1769),
    [sym_do_release] = ACTIONS(1769),
    [sym_device_id] = ACTIONS(1769),
    [sym_device_info] = ACTIONS(1769),
    [sym_device_id_string] = ACTIONS(1769),
    [sym_device_standby] = ACTIONS(1769),
    [sym_device_wake] = ACTIONS(1769),
    [sym_timeline_create] = ACTIONS(1769),
    [sym_timeline_kill] = ACTIONS(1769),
    [sym_timeline_reload] = ACTIONS(1769),
    [sym_timeline_pause] = ACTIONS(1769),
    [sym_timeline_restart] = ACTIONS(1769),
    [sym_timeline_set] = ACTIONS(1769),
    [sym_timeline_get] = ACTIONS(1769),
    [sym_timeline_active] = ACTIONS(1769),
    [sym_timeline_dynamic_id] = ACTIONS(1769),
    [sym_variable_to_string] = ACTIONS(1769),
    [sym_string_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_string] = ACTIONS(1769),
    [sym_type_cast] = ACTIONS(1769),
    [sym_variable_to_xml] = ACTIONS(1769),
    [sym_xml_to_variable] = ACTIONS(1769),
    [sym_length_variable_to_xml] = ACTIONS(1769),
    [sym_redirect_string] = ACTIONS(1769),
    [sym_random_number] = ACTIONS(1769),
    [sym_get_unique_id] = ACTIONS(1769),
    [sym_get_system_number] = ACTIONS(1769),
    [sym_set_system_number] = ACTIONS(1769),
    [sym_get_serial_number] = ACTIONS(1769),
    [sym_get_master_build] = ACTIONS(1769),
    [sym_set_validation_code] = ACTIONS(1769),
    [sym_reboot] = ACTIONS(1769),
    [sym_load_duet_module] = ACTIONS(1769),
    [sym_unload_duet_module] = ACTIONS(1769),
    [sym_duet_mem_size_set] = ACTIONS(1769),
    [sym_duet_mem_size_get] = ACTIONS(1769),
    [sym_dynamic_application_device] = ACTIONS(1769),
    [sym_dynamic_polled_port] = ACTIONS(1769),
    [sym_static_port_binding] = ACTIONS(1769),
    [sym_static_ip_binding] = ACTIONS(1769),
    [sym_get_dns_list] = ACTIONS(1769),
    [sym_set_dns_list] = ACTIONS(1769),
    [sym_get_ip_address] = ACTIONS(1769),
    [sym_set_ip_address] = ACTIONS(1769),
    [sym_get_url_list] = ACTIONS(1769),
    [sym_add_url_entry] = ACTIONS(1769),
    [sym_delete_url_entry] = ACTIONS(1769),
    [sym_abs_value] = ACTIONS(1769),
    [sym_max_value] = ACTIONS(1769),
    [sym_min_value] = ACTIONS(1769),
    [sym_exp_value] = ACTIONS(1769),
    [sym_log_value] = ACTIONS(1769),
    [sym_log10_value] = ACTIONS(1769),
    [sym_power_value] = ACTIONS(1769),
    [sym_sqrt_value] = ACTIONS(1769),
    [sym_raw_be] = ACTIONS(1769),
    [sym_raw_le] = ACTIONS(1769),
    [sym_internal_queue_size_set] = ACTIONS(1769),
    [sym_internal_queue_size_get] = ACTIONS(1769),
    [sym_internal_threshold_set] = ACTIONS(1769),
    [sym_internal_threshold_get] = ACTIONS(1769),
    [sym_rebuilt_event] = ACTIONS(1769),
    [sym_do_custom_event] = ACTIONS(1769),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1769),
    [sym_clkmgr_set_clk_source] = ACTIONS(1769),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1769),
    [sym_clkmgr_get_timezone] = ACTIONS(1769),
    [sym_clkmgr_set_timezone] = ACTIONS(1769),
    [sym_clkmgr_get_resync_period] = ACTIONS(1769),
    [sym_clkmgr_set_resync_period] = ACTIONS(1769),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1769),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_timeservers] = ACTIONS(1769),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1769),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1769),
    [sym_set_log_level] = ACTIONS(1769),
    [sym_get_log_level] = ACTIONS(1769),
    [sym_amx_log] = ACTIONS(1769),
    [sym_smtp_server_config_set] = ACTIONS(1769),
    [sym_smtp_server_config_get] = ACTIONS(1769),
    [sym_smtp_send] = ACTIONS(1769),
    [sym_validate_netlinx_account] = ACTIONS(1769),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1769),
    [sym_authenticate_certificate] = ACTIONS(1769),
    [sym_audit_netlinx_session_event] = ACTIONS(1769),
    [sym_audit_netlinx_generic_event] = ACTIONS(1769),
    [sym_get_available_flash_disk_space] = ACTIONS(1769),
    [sym_get_max_flash_disk_space] = ACTIONS(1769),
    [sym_data_create_feed] = ACTIONS(1769),
    [sym_data_delete_feed] = ACTIONS(1769),
    [sym_data_publish_feed] = ACTIONS(1769),
    [sym_data_get_published_feed] = ACTIONS(1769),
    [sym_data_add_record] = ACTIONS(1769),
    [sym_data_get_event_record] = ACTIONS(1769),
    [sym__wc_data_create_feed] = ACTIONS(1769),
    [sym__wc_data_add_record] = ACTIONS(1769),
    [sym__wc_data_get_event_record] = ACTIONS(1769),
    [sym_format] = ACTIONS(1769),
    [sym_astro_clock] = ACTIONS(1769),
    [sym_set_outdoor_temperature] = ACTIONS(1769),
    [sym_set_virtual_level_count] = ACTIONS(1769),
    [sym_set_virtual_channel_count] = ACTIONS(1769),
    [sym_set_virtual_port_count] = ACTIONS(1769),
    [sym__wc_encode_int] = ACTIONS(1769),
    [sym__wc_decode_int] = ACTIONS(1769),
    [sym__wc_int] = ACTIONS(1769),
    [sym__wc_to_ch_int] = ACTIONS(1769),
    [sym__ch_to_wc_int] = ACTIONS(1769),
    [sym__wc_find_string_int] = ACTIONS(1769),
    [sym__wc_left_string_int] = ACTIONS(1769),
    [sym__wc_lower_string_int] = ACTIONS(1769),
    [sym__wc_mid_string_int] = ACTIONS(1769),
    [sym__wc_remove_string_int] = ACTIONS(1769),
    [sym__wc_right_string_int] = ACTIONS(1769),
    [sym__wc_upper_string_int] = ACTIONS(1769),
    [sym__wc_compare_string_int] = ACTIONS(1769),
    [sym__wc_get_buffer_char_int] = ACTIONS(1769),
    [sym__wc_get_buffer_string_int] = ACTIONS(1769),
    [sym__wc_concat_string_int] = ACTIONS(1769),
    [sym__wc_file_open_int] = ACTIONS(1769),
    [sym__wc_file_close_int] = ACTIONS(1769),
    [sym__wc_file_read_int] = ACTIONS(1769),
    [sym__wc_file_read_line_int] = ACTIONS(1769),
    [sym__wc_file_write_int] = ACTIONS(1769),
    [sym__wc_file_write_line_int] = ACTIONS(1769),
    [aux_sym_preproc_include_token1] = ACTIONS(1769),
    [aux_sym_preproc_define_token1] = ACTIONS(1771),
    [aux_sym_preproc_warn_token1] = ACTIONS(1771),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1771),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1771),
    [anon_sym_LBRACK] = ACTIONS(1771),
    [anon_sym_LBRACE] = ACTIONS(1771),
    [anon_sym_RBRACE] = ACTIONS(1771),
    [anon_sym_LPAREN] = ACTIONS(1769),
    [anon_sym_BANG] = ACTIONS(1771),
    [anon_sym_TILDE] = ACTIONS(1771),
    [anon_sym_DASH] = ACTIONS(1769),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_DASH_DASH] = ACTIONS(1771),
    [anon_sym_PLUS_PLUS] = ACTIONS(1771),
    [anon_sym_DQUOTE] = ACTIONS(1771),
    [anon_sym_SQUOTE] = ACTIONS(1771),
    [sym_number_literal] = ACTIONS(1771),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(487)] = {
    [sym_identifier] = ACTIONS(1785),
    [sym_define_call_keyword] = ACTIONS(1785),
    [sym_define_function_keyword] = ACTIONS(1785),
    [sym_define_library_function_keyword] = ACTIONS(1785),
    [sym_define_module_keyword] = ACTIONS(1785),
    [sym_char_keyword] = ACTIONS(1785),
    [sym_widechar_keyword] = ACTIONS(1785),
    [sym_integer_keyword] = ACTIONS(1785),
    [sym_sinteger_keyword] = ACTIONS(1785),
    [sym_long_keyword] = ACTIONS(1785),
    [sym_slong_keyword] = ACTIONS(1785),
    [sym_float_keyword] = ACTIONS(1785),
    [sym_double_keyword] = ACTIONS(1785),
    [sym_variant_keyword] = ACTIONS(1785),
    [sym_variantarray_keyword] = ACTIONS(1785),
    [sym_local_var_keyword] = ACTIONS(1785),
    [sym_stack_var_keyword] = ACTIONS(1785),
    [sym_constant_keyword] = ACTIONS(1785),
    [sym_volatile_keyword] = ACTIONS(1785),
    [sym_non_volatile_keyword] = ACTIONS(1785),
    [sym_persistent_keyword] = ACTIONS(1785),
    [sym_dev_keyword] = ACTIONS(1785),
    [sym_devlev_keyword] = ACTIONS(1785),
    [sym_devchan_keyword] = ACTIONS(1785),
    [sym_if_keyword] = ACTIONS(1785),
    [sym_else_keyword] = ACTIONS(1785),
    [sym_switch_keyword] = ACTIONS(1785),
    [sym_case_keyword] = ACTIONS(1785),
    [sym_default_keyword] = ACTIONS(1785),
    [sym_while_keyword] = ACTIONS(1785),
    [sym_for_keyword] = ACTIONS(1785),
    [sym_break_keyword] = ACTIONS(1785),
    [sym_continue_keyword] = ACTIONS(1785),
    [sym_return_keyword] = ACTIONS(1785),
    [sym_select_keyword] = ACTIONS(1785),
    [sym_active_keyword] = ACTIONS(1785),
    [sym_struct_keyword] = ACTIONS(1785),
    [sym_structure_keyword] = ACTIONS(1785),
    [aux_sym_on_keyword_token1] = ACTIONS(1785),
    [aux_sym_off_keyword_token1] = ACTIONS(1785),
    [sym_send_level_keyword] = ACTIONS(1785),
    [sym_send_string_keyword] = ACTIONS(1785),
    [sym_send_command_keyword] = ACTIONS(1785),
    [sym_clear_buffer_keyword] = ACTIONS(1785),
    [sym_create_buffer_keyword] = ACTIONS(1785),
    [sym_create_multi_buffer_keyword] = ACTIONS(1785),
    [sym_call_keyword] = ACTIONS(1785),
    [sym_system_call_keyword] = ACTIONS(1785),
    [sym_devchan_to_keyword] = ACTIONS(1785),
    [sym_devchan_min_to_keyword] = ACTIONS(1785),
    [sym_devchan_total_off_keyword] = ACTIONS(1785),
    [sym_devchan_pulse_keyword] = ACTIONS(1785),
    [sym_wait_keyword] = ACTIONS(1785),
    [sym_cancel_wait_keyword] = ACTIONS(1785),
    [sym_wait_until_keyword] = ACTIONS(1785),
    [sym_cancel_wait_until_keyword] = ACTIONS(1785),
    [sym_cancel_all_wait_keyword] = ACTIONS(1785),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1785),
    [sym_not] = ACTIONS(1785),
    [sym_bnot] = ACTIONS(1785),
    [sym___file__] = ACTIONS(1785),
    [sym___date__] = ACTIONS(1785),
    [sym___ldate__] = ACTIONS(1785),
    [sym___time__] = ACTIONS(1785),
    [sym___line__] = ACTIONS(1785),
    [sym___name__] = ACTIONS(1785),
    [sym___netlinx__] = ACTIONS(1785),
    [sym_netlinx_axi_version] = ACTIONS(1785),
    [sym_url_flg_tcp] = ACTIONS(1785),
    [sym_url_flg_accinfopresent] = ACTIONS(1785),
    [sym_url_flg_temp] = ACTIONS(1785),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1785),
    [sym_url_flg_stat_mask] = ACTIONS(1785),
    [sym_url_flg_stat_lookup] = ACTIONS(1785),
    [sym_url_flg_stat_connecting] = ACTIONS(1785),
    [sym_url_flg_stat_waiting] = ACTIONS(1785),
    [sym_url_flg_stat_connected] = ACTIONS(1785),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1785),
    [sym_first_virtual_device] = ACTIONS(1785),
    [sym_first_local_port] = ACTIONS(1785),
    [sym_dynamic_virtual_device] = ACTIONS(1785),
    [sym_timeline_once] = ACTIONS(1785),
    [sym_timeline_repeat] = ACTIONS(1785),
    [sym_timeline_absolute] = ACTIONS(1785),
    [sym_timeline_relative] = ACTIONS(1785),
    [sym_do_push_timed_infinite] = ACTIONS(1785),
    [sym_source_type_no_address] = ACTIONS(1785),
    [sym_source_type_neuron_id] = ACTIONS(1785),
    [sym_source_type_ip_address] = ACTIONS(1785),
    [sym_source_type_axlink] = ACTIONS(1785),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1785),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1785),
    [sym_source_type_ip_socket_address] = ACTIONS(1785),
    [sym_source_type_rs232] = ACTIONS(1785),
    [sym_source_type_internal] = ACTIONS(1785),
    [sym_source_type_mac_address] = ACTIONS(1785),
    [sym_source_type_ipv4_port] = ACTIONS(1785),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1785),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1785),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1785),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1785),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1785),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1785),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1785),
    [sym_true] = ACTIONS(1785),
    [sym_false] = ACTIONS(1785),
    [sym_file_read_only] = ACTIONS(1785),
    [sym_file_rw_new] = ACTIONS(1785),
    [sym_file_rw_append] = ACTIONS(1785),
    [sym_ip_tcp] = ACTIONS(1785),
    [sym_ip_udp] = ACTIONS(1785),
    [sym_ip_udp_2way] = ACTIONS(1785),
    [sym_xml_encode_types] = ACTIONS(1785),
    [sym_xml_encode_char_as_list] = ACTIONS(1785),
    [sym_xml_encode_le] = ACTIONS(1785),
    [sym_xml_decode_types] = ACTIONS(1785),
    [sym_xml_decode_no_preserve] = ACTIONS(1785),
    [sym_duet_dev_type_display_device] = ACTIONS(1785),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1785),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1785),
    [sym_duet_dev_type_io_device] = ACTIONS(1785),
    [sym_duet_dev_type_relay_device] = ACTIONS(1785),
    [sym_duet_dev_type_ups] = ACTIONS(1785),
    [sym_duet_dev_type_amplifier] = ACTIONS(1785),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1785),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1785),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1785),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1785),
    [sym_duet_dev_type_camera] = ACTIONS(1785),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1785),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1785),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1785),
    [sym_duet_dev_type_dss] = ACTIONS(1785),
    [sym_duet_dev_type_dvr] = ACTIONS(1785),
    [sym_duet_dev_type_disc_device] = ACTIONS(1785),
    [sym_duet_dev_type_document_camera] = ACTIONS(1785),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1785),
    [sym_duet_dev_type_hvac] = ACTIONS(1785),
    [sym_duet_dev_type_keypad] = ACTIONS(1785),
    [sym_duet_dev_type_light] = ACTIONS(1785),
    [sym_duet_dev_type_monitor] = ACTIONS(1785),
    [sym_duet_dev_type_motor] = ACTIONS(1785),
    [sym_duet_dev_type_multi_window] = ACTIONS(1785),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1785),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1785),
    [sym_duet_dev_type_receiver] = ACTIONS(1785),
    [sym_duet_dev_type_security_system] = ACTIONS(1785),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1785),
    [sym_duet_dev_type_settop_box] = ACTIONS(1785),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1785),
    [sym_duet_dev_type_switcher] = ACTIONS(1785),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1785),
    [sym_duet_dev_type_tv] = ACTIONS(1785),
    [sym_duet_dev_type_utility] = ACTIONS(1785),
    [sym_duet_dev_type_vcr] = ACTIONS(1785),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1785),
    [sym_duet_dev_type_video_processor] = ACTIONS(1785),
    [sym_duet_dev_type_video_projector] = ACTIONS(1785),
    [sym_duet_dev_type_video_wall] = ACTIONS(1785),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1785),
    [sym_duet_dev_type_weather] = ACTIONS(1785),
    [sym_duet_dev_not_polled] = ACTIONS(1785),
    [sym_duet_dev_polled] = ACTIONS(1785),
    [sym_clkmgr_mode_network] = ACTIONS(1785),
    [sym_clkmgr_mode_standalone] = ACTIONS(1785),
    [sym_normal_standby] = ACTIONS(1785),
    [sym_normal_wake] = ACTIONS(1785),
    [sym_ip_multicast_ttl_option] = ACTIONS(1785),
    [sym_ip_ttl_subnet] = ACTIONS(1785),
    [sym_ip_ttl_site] = ACTIONS(1785),
    [sym_ip_ttl_region] = ACTIONS(1785),
    [sym_ip_ttl_continent] = ACTIONS(1785),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1785),
    [sym_ip_nodelay_on] = ACTIONS(1785),
    [sym_ip_nodelay_off] = ACTIONS(1785),
    [sym_amx_error] = ACTIONS(1785),
    [sym_amx_warning] = ACTIONS(1785),
    [sym_amx_info] = ACTIONS(1785),
    [sym_amx_debug] = ACTIONS(1785),
    [sym_netlinx_login_success] = ACTIONS(1785),
    [sym_netlinx_login_fail] = ACTIONS(1785),
    [sym_netlinx_logout] = ACTIONS(1785),
    [sym_null_str] = ACTIONS(1785),
    [sym_smtp_address] = ACTIONS(1785),
    [sym_smtp_port_number] = ACTIONS(1785),
    [sym_smtp_username] = ACTIONS(1785),
    [sym_smtp_password] = ACTIONS(1785),
    [sym_smtp_require_tls] = ACTIONS(1785),
    [sym_smtp_from] = ACTIONS(1785),
    [sym_smtp_tls_true] = ACTIONS(1785),
    [sym_smtp_tls_false] = ACTIONS(1785),
    [sym_listview_on_row_select_event] = ACTIONS(1785),
    [sym_data_structure_unknown] = ACTIONS(1785),
    [sym_data_structure_datafeed] = ACTIONS(1785),
    [sym_data_structure_datarecordset] = ACTIONS(1785),
    [sym_data_structure_datarecord] = ACTIONS(1785),
    [sym_data_structure_datafield] = ACTIONS(1785),
    [sym_data_max_content_field_count] = ACTIONS(1785),
    [sym_data_max_metadata_field_count] = ACTIONS(1785),
    [sym_data_max_id_length] = ACTIONS(1785),
    [sym_data_max_name_length] = ACTIONS(1785),
    [sym_data_max_description_length] = ACTIONS(1785),
    [sym_data_max_type_length] = ACTIONS(1785),
    [sym_data_max_format_length] = ACTIONS(1785),
    [sym_data_max_source_length] = ACTIONS(1785),
    [sym_data_max_value_length] = ACTIONS(1785),
    [sym_data_max_label_length] = ACTIONS(1785),
    [sym_data_max_url_length] = ACTIONS(1785),
    [sym_max_last_login_info_length] = ACTIONS(1785),
    [sym_data_type_unknown] = ACTIONS(1785),
    [sym_data_type_string] = ACTIONS(1785),
    [sym_data_type_datetime] = ACTIONS(1785),
    [sym_data_type_date] = ACTIONS(1785),
    [sym_data_type_time] = ACTIONS(1785),
    [sym_data_type_image] = ACTIONS(1785),
    [sym_data_format_url] = ACTIONS(1785),
    [sym_data_format_phone] = ACTIONS(1785),
    [sym_data_format_email] = ACTIONS(1785),
    [sym_data_format_iso8601] = ACTIONS(1785),
    [sym_auditlog_priv] = ACTIONS(1785),
    [sym_configuration_priv] = ACTIONS(1785),
    [sym_deviceconfig_priv] = ACTIONS(1785),
    [sym_ftp_priv] = ACTIONS(1785),
    [sym_http_priv] = ACTIONS(1785),
    [sym_networkconfig_priv] = ACTIONS(1785),
    [sym_programport_priv] = ACTIONS(1785),
    [sym_remoteui_priv] = ACTIONS(1785),
    [sym_securitycontrol_priv] = ACTIONS(1785),
    [sym_softwaremanagement_priv] = ACTIONS(1785),
    [sym_terminal_priv] = ACTIONS(1785),
    [sym_tpadmin_priv] = ACTIONS(1785),
    [sym_user1_priv] = ACTIONS(1785),
    [sym_user2_priv] = ACTIONS(1785),
    [sym_user3_priv] = ACTIONS(1785),
    [sym_user4_priv] = ACTIONS(1785),
    [sym_usermanagement_priv] = ACTIONS(1785),
    [sym_valid_account] = ACTIONS(1785),
    [sym_err_name_invalid] = ACTIONS(1785),
    [sym_err_password_invalid] = ACTIONS(1785),
    [sym_err_invalid_account] = ACTIONS(1785),
    [sym_err_no_authorization] = ACTIONS(1785),
    [sym_err_invalid_parameter] = ACTIONS(1785),
    [sym_err_user_locked_out] = ACTIONS(1785),
    [sym_err_user_account_expired] = ACTIONS(1785),
    [sym_err_general_error] = ACTIONS(1785),
    [sym_err_authentication_error] = ACTIONS(1785),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1785),
    [sym_tls_validate_certificate] = ACTIONS(1785),
    [sym_tdata] = ACTIONS(1785),
    [sym_tchannel] = ACTIONS(1785),
    [sym_tlevel] = ACTIONS(1785),
    [sym_tbutton] = ACTIONS(1785),
    [sym_ttimeline] = ACTIONS(1785),
    [sym_tcustom] = ACTIONS(1785),
    [sym_url_struct] = ACTIONS(1785),
    [sym_dns_struct] = ACTIONS(1785),
    [sym_ip_address_struct] = ACTIONS(1785),
    [sym_dev_info_struct] = ACTIONS(1785),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1785),
    [sym_data_feed] = ACTIONS(1785),
    [sym_data_field] = ACTIONS(1785),
    [sym_data_record] = ACTIONS(1785),
    [sym_wc_data_feed] = ACTIONS(1785),
    [sym_wc_data_field] = ACTIONS(1785),
    [sym_wc_data_record] = ACTIONS(1785),
    [sym_last_login_info] = ACTIONS(1785),
    [sym____reserved___] = ACTIONS(1785),
    [sym_date] = ACTIONS(1785),
    [sym_day] = ACTIONS(1785),
    [sym_time] = ACTIONS(1785),
    [sym_ldate] = ACTIONS(1785),
    [sym_push_device] = ACTIONS(1785),
    [sym_push_channel] = ACTIONS(1785),
    [sym_push_devchan] = ACTIONS(1785),
    [sym_release_device] = ACTIONS(1785),
    [sym_release_channel] = ACTIONS(1785),
    [sym_release_devchan] = ACTIONS(1785),
    [sym_master_slot] = ACTIONS(1785),
    [sym_get_pulse_time] = ACTIONS(1785),
    [sym_get_timer] = ACTIONS(1785),
    [sym_master_sn] = ACTIONS(1785),
    [sym_system_number] = ACTIONS(1785),
    [sym_button] = ACTIONS(1785),
    [sym_dv_channel] = ACTIONS(1785),
    [sym_data] = ACTIONS(1785),
    [sym_level] = ACTIONS(1785),
    [sym_timeline] = ACTIONS(1785),
    [sym_channel] = ACTIONS(1785),
    [sym_custom] = ACTIONS(1785),
    [sym_atoi] = ACTIONS(1785),
    [sym_atol] = ACTIONS(1785),
    [sym_atof] = ACTIONS(1785),
    [sym_itoa] = ACTIONS(1785),
    [sym_itohex] = ACTIONS(1785),
    [sym_hextoi] = ACTIONS(1785),
    [sym_ftoa] = ACTIONS(1785),
    [sym_find_string] = ACTIONS(1785),
    [sym_left_string] = ACTIONS(1785),
    [sym_right_string] = ACTIONS(1785),
    [sym_mid_string] = ACTIONS(1785),
    [sym_remove_string] = ACTIONS(1785),
    [sym_get_buffer_string] = ACTIONS(1785),
    [sym_get_buffer_char] = ACTIONS(1785),
    [sym_get_multi_buffer_string] = ACTIONS(1785),
    [sym_length_string] = ACTIONS(1785),
    [sym_max_length_string] = ACTIONS(1785),
    [sym_set_length_string] = ACTIONS(1785),
    [sym_compare_string] = ACTIONS(1785),
    [sym_upper_string] = ACTIONS(1785),
    [sym_lower_string] = ACTIONS(1785),
    [sym_get_last] = ACTIONS(1785),
    [sym_date_to_day] = ACTIONS(1785),
    [sym_date_to_month] = ACTIONS(1785),
    [sym_date_to_year] = ACTIONS(1785),
    [sym_time_to_hour] = ACTIONS(1785),
    [sym_time_to_minute] = ACTIONS(1785),
    [sym_time_to_second] = ACTIONS(1785),
    [sym_day_of_week] = ACTIONS(1785),
    [sym_set_pulse_time] = ACTIONS(1785),
    [sym_set_timer] = ACTIONS(1785),
    [sym_file_open] = ACTIONS(1785),
    [sym_file_close] = ACTIONS(1785),
    [sym_file_read] = ACTIONS(1785),
    [sym_file_read_line] = ACTIONS(1785),
    [sym_file_write] = ACTIONS(1785),
    [sym_file_write_line] = ACTIONS(1785),
    [sym_file_copy] = ACTIONS(1785),
    [sym_file_delete] = ACTIONS(1785),
    [sym_file_rename] = ACTIONS(1785),
    [sym_file_dir] = ACTIONS(1785),
    [sym_file_getdir] = ACTIONS(1785),
    [sym_file_setdir] = ACTIONS(1785),
    [sym_file_create_dir] = ACTIONS(1785),
    [sym_file_remove_dir] = ACTIONS(1785),
    [sym_file_seek] = ACTIONS(1785),
    [sym_length_array] = ACTIONS(1785),
    [sym_max_length_array] = ACTIONS(1785),
    [sym_set_length_array] = ACTIONS(1785),
    [sym_ip_client_open] = ACTIONS(1785),
    [sym_ip_client_close] = ACTIONS(1785),
    [sym_ip_server_open] = ACTIONS(1785),
    [sym_ip_server_close] = ACTIONS(1785),
    [sym_ip_bound_client_open] = ACTIONS(1785),
    [sym_ip_mc_server_open] = ACTIONS(1785),
    [sym_ip_set_option] = ACTIONS(1785),
    [sym_ssh_client_open] = ACTIONS(1785),
    [sym_ssh_client_close] = ACTIONS(1785),
    [sym_tls_client_open] = ACTIONS(1785),
    [sym_tls_client_close] = ACTIONS(1785),
    [sym_do_push] = ACTIONS(1785),
    [sym_do_push_timed] = ACTIONS(1785),
    [sym_do_release] = ACTIONS(1785),
    [sym_device_id] = ACTIONS(1785),
    [sym_device_info] = ACTIONS(1785),
    [sym_device_id_string] = ACTIONS(1785),
    [sym_device_standby] = ACTIONS(1785),
    [sym_device_wake] = ACTIONS(1785),
    [sym_timeline_create] = ACTIONS(1785),
    [sym_timeline_kill] = ACTIONS(1785),
    [sym_timeline_reload] = ACTIONS(1785),
    [sym_timeline_pause] = ACTIONS(1785),
    [sym_timeline_restart] = ACTIONS(1785),
    [sym_timeline_set] = ACTIONS(1785),
    [sym_timeline_get] = ACTIONS(1785),
    [sym_timeline_active] = ACTIONS(1785),
    [sym_timeline_dynamic_id] = ACTIONS(1785),
    [sym_variable_to_string] = ACTIONS(1785),
    [sym_string_to_variable] = ACTIONS(1785),
    [sym_length_variable_to_string] = ACTIONS(1785),
    [sym_type_cast] = ACTIONS(1785),
    [sym_variable_to_xml] = ACTIONS(1785),
    [sym_xml_to_variable] = ACTIONS(1785),
    [sym_length_variable_to_xml] = ACTIONS(1785),
    [sym_redirect_string] = ACTIONS(1785),
    [sym_random_number] = ACTIONS(1785),
    [sym_get_unique_id] = ACTIONS(1785),
    [sym_get_system_number] = ACTIONS(1785),
    [sym_set_system_number] = ACTIONS(1785),
    [sym_get_serial_number] = ACTIONS(1785),
    [sym_get_master_build] = ACTIONS(1785),
    [sym_set_validation_code] = ACTIONS(1785),
    [sym_reboot] = ACTIONS(1785),
    [sym_load_duet_module] = ACTIONS(1785),
    [sym_unload_duet_module] = ACTIONS(1785),
    [sym_duet_mem_size_set] = ACTIONS(1785),
    [sym_duet_mem_size_get] = ACTIONS(1785),
    [sym_dynamic_application_device] = ACTIONS(1785),
    [sym_dynamic_polled_port] = ACTIONS(1785),
    [sym_static_port_binding] = ACTIONS(1785),
    [sym_static_ip_binding] = ACTIONS(1785),
    [sym_get_dns_list] = ACTIONS(1785),
    [sym_set_dns_list] = ACTIONS(1785),
    [sym_get_ip_address] = ACTIONS(1785),
    [sym_set_ip_address] = ACTIONS(1785),
    [sym_get_url_list] = ACTIONS(1785),
    [sym_add_url_entry] = ACTIONS(1785),
    [sym_delete_url_entry] = ACTIONS(1785),
    [sym_abs_value] = ACTIONS(1785),
    [sym_max_value] = ACTIONS(1785),
    [sym_min_value] = ACTIONS(1785),
    [sym_exp_value] = ACTIONS(1785),
    [sym_log_value] = ACTIONS(1785),
    [sym_log10_value] = ACTIONS(1785),
    [sym_power_value] = ACTIONS(1785),
    [sym_sqrt_value] = ACTIONS(1785),
    [sym_raw_be] = ACTIONS(1785),
    [sym_raw_le] = ACTIONS(1785),
    [sym_internal_queue_size_set] = ACTIONS(1785),
    [sym_internal_queue_size_get] = ACTIONS(1785),
    [sym_internal_threshold_set] = ACTIONS(1785),
    [sym_internal_threshold_get] = ACTIONS(1785),
    [sym_rebuilt_event] = ACTIONS(1785),
    [sym_do_custom_event] = ACTIONS(1785),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1785),
    [sym_clkmgr_set_clk_source] = ACTIONS(1785),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1785),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1785),
    [sym_clkmgr_get_timezone] = ACTIONS(1785),
    [sym_clkmgr_set_timezone] = ACTIONS(1785),
    [sym_clkmgr_get_resync_period] = ACTIONS(1785),
    [sym_clkmgr_set_resync_period] = ACTIONS(1785),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1785),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1785),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1785),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1785),
    [sym_clkmgr_get_timeservers] = ACTIONS(1785),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1785),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1785),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1785),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1785),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1785),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1785),
    [sym_set_log_level] = ACTIONS(1785),
    [sym_get_log_level] = ACTIONS(1785),
    [sym_amx_log] = ACTIONS(1785),
    [sym_smtp_server_config_set] = ACTIONS(1785),
    [sym_smtp_server_config_get] = ACTIONS(1785),
    [sym_smtp_send] = ACTIONS(1785),
    [sym_validate_netlinx_account] = ACTIONS(1785),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1785),
    [sym_authenticate_certificate] = ACTIONS(1785),
    [sym_audit_netlinx_session_event] = ACTIONS(1785),
    [sym_audit_netlinx_generic_event] = ACTIONS(1785),
    [sym_get_available_flash_disk_space] = ACTIONS(1785),
    [sym_get_max_flash_disk_space] = ACTIONS(1785),
    [sym_data_create_feed] = ACTIONS(1785),
    [sym_data_delete_feed] = ACTIONS(1785),
    [sym_data_publish_feed] = ACTIONS(1785),
    [sym_data_get_published_feed] = ACTIONS(1785),
    [sym_data_add_record] = ACTIONS(1785),
    [sym_data_get_event_record] = ACTIONS(1785),
    [sym__wc_data_create_feed] = ACTIONS(1785),
    [sym__wc_data_add_record] = ACTIONS(1785),
    [sym__wc_data_get_event_record] = ACTIONS(1785),
    [sym_format] = ACTIONS(1785),
    [sym_astro_clock] = ACTIONS(1785),
    [sym_set_outdoor_temperature] = ACTIONS(1785),
    [sym_set_virtual_level_count] = ACTIONS(1785),
    [sym_set_virtual_channel_count] = ACTIONS(1785),
    [sym_set_virtual_port_count] = ACTIONS(1785),
    [sym__wc_encode_int] = ACTIONS(1785),
    [sym__wc_decode_int] = ACTIONS(1785),
    [sym__wc_int] = ACTIONS(1785),
    [sym__wc_to_ch_int] = ACTIONS(1785),
    [sym__ch_to_wc_int] = ACTIONS(1785),
    [sym__wc_find_string_int] = ACTIONS(1785),
    [sym__wc_left_string_int] = ACTIONS(1785),
    [sym__wc_lower_string_int] = ACTIONS(1785),
    [sym__wc_mid_string_int] = ACTIONS(1785),
    [sym__wc_remove_string_int] = ACTIONS(1785),
    [sym__wc_right_string_int] = ACTIONS(1785),
    [sym__wc_upper_string_int] = ACTIONS(1785),
    [sym__wc_compare_string_int] = ACTIONS(1785),
    [sym__wc_get_buffer_char_int] = ACTIONS(1785),
    [sym__wc_get_buffer_string_int] = ACTIONS(1785),
    [sym__wc_concat_string_int] = ACTIONS(1785),
    [sym__wc_file_open_int] = ACTIONS(1785),
    [sym__wc_file_close_int] = ACTIONS(1785),
    [sym__wc_file_read_int] = ACTIONS(1785),
    [sym__wc_file_read_line_int] = ACTIONS(1785),
    [sym__wc_file_write_int] = ACTIONS(1785),
    [sym__wc_file_write_line_int] = ACTIONS(1785),
    [aux_sym_preproc_include_token1] = ACTIONS(1785),
    [aux_sym_preproc_define_token1] = ACTIONS(1787),
    [aux_sym_preproc_warn_token1] = ACTIONS(1787),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1787),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1787),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1787),
    [anon_sym_LBRACK] = ACTIONS(1787),
    [anon_sym_LBRACE] = ACTIONS(1787),
    [anon_sym_RBRACE] = ACTIONS(1787),
    [anon_sym_LPAREN] = ACTIONS(1785),
    [anon_sym_BANG] = ACTIONS(1787),
    [anon_sym_TILDE] = ACTIONS(1787),
    [anon_sym_DASH] = ACTIONS(1785),
    [anon_sym_PLUS] = ACTIONS(1785),
    [anon_sym_DASH_DASH] = ACTIONS(1787),
    [anon_sym_PLUS_PLUS] = ACTIONS(1787),
    [anon_sym_DQUOTE] = ACTIONS(1787),
    [anon_sym_SQUOTE] = ACTIONS(1787),
    [sym_number_literal] = ACTIONS(1787),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(488)] = {
    [sym_identifier] = ACTIONS(1843),
    [sym_define_call_keyword] = ACTIONS(1843),
    [sym_define_function_keyword] = ACTIONS(1843),
    [sym_define_library_function_keyword] = ACTIONS(1843),
    [sym_define_module_keyword] = ACTIONS(1843),
    [sym_char_keyword] = ACTIONS(1843),
    [sym_widechar_keyword] = ACTIONS(1843),
    [sym_integer_keyword] = ACTIONS(1843),
    [sym_sinteger_keyword] = ACTIONS(1843),
    [sym_long_keyword] = ACTIONS(1843),
    [sym_slong_keyword] = ACTIONS(1843),
    [sym_float_keyword] = ACTIONS(1843),
    [sym_double_keyword] = ACTIONS(1843),
    [sym_variant_keyword] = ACTIONS(1843),
    [sym_variantarray_keyword] = ACTIONS(1843),
    [sym_local_var_keyword] = ACTIONS(1843),
    [sym_stack_var_keyword] = ACTIONS(1843),
    [sym_constant_keyword] = ACTIONS(1843),
    [sym_volatile_keyword] = ACTIONS(1843),
    [sym_non_volatile_keyword] = ACTIONS(1843),
    [sym_persistent_keyword] = ACTIONS(1843),
    [sym_dev_keyword] = ACTIONS(1843),
    [sym_devlev_keyword] = ACTIONS(1843),
    [sym_devchan_keyword] = ACTIONS(1843),
    [sym_if_keyword] = ACTIONS(1843),
    [sym_else_keyword] = ACTIONS(1843),
    [sym_switch_keyword] = ACTIONS(1843),
    [sym_case_keyword] = ACTIONS(1843),
    [sym_default_keyword] = ACTIONS(1843),
    [sym_while_keyword] = ACTIONS(1843),
    [sym_for_keyword] = ACTIONS(1843),
    [sym_break_keyword] = ACTIONS(1843),
    [sym_continue_keyword] = ACTIONS(1843),
    [sym_return_keyword] = ACTIONS(1843),
    [sym_select_keyword] = ACTIONS(1843),
    [sym_active_keyword] = ACTIONS(1843),
    [sym_struct_keyword] = ACTIONS(1843),
    [sym_structure_keyword] = ACTIONS(1843),
    [aux_sym_on_keyword_token1] = ACTIONS(1843),
    [aux_sym_off_keyword_token1] = ACTIONS(1843),
    [sym_send_level_keyword] = ACTIONS(1843),
    [sym_send_string_keyword] = ACTIONS(1843),
    [sym_send_command_keyword] = ACTIONS(1843),
    [sym_clear_buffer_keyword] = ACTIONS(1843),
    [sym_create_buffer_keyword] = ACTIONS(1843),
    [sym_create_multi_buffer_keyword] = ACTIONS(1843),
    [sym_call_keyword] = ACTIONS(1843),
    [sym_system_call_keyword] = ACTIONS(1843),
    [sym_devchan_to_keyword] = ACTIONS(1843),
    [sym_devchan_min_to_keyword] = ACTIONS(1843),
    [sym_devchan_total_off_keyword] = ACTIONS(1843),
    [sym_devchan_pulse_keyword] = ACTIONS(1843),
    [sym_wait_keyword] = ACTIONS(1843),
    [sym_cancel_wait_keyword] = ACTIONS(1843),
    [sym_wait_until_keyword] = ACTIONS(1843),
    [sym_cancel_wait_until_keyword] = ACTIONS(1843),
    [sym_cancel_all_wait_keyword] = ACTIONS(1843),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1843),
    [sym_not] = ACTIONS(1843),
    [sym_bnot] = ACTIONS(1843),
    [sym___file__] = ACTIONS(1843),
    [sym___date__] = ACTIONS(1843),
    [sym___ldate__] = ACTIONS(1843),
    [sym___time__] = ACTIONS(1843),
    [sym___line__] = ACTIONS(1843),
    [sym___name__] = ACTIONS(1843),
    [sym___netlinx__] = ACTIONS(1843),
    [sym_netlinx_axi_version] = ACTIONS(1843),
    [sym_url_flg_tcp] = ACTIONS(1843),
    [sym_url_flg_accinfopresent] = ACTIONS(1843),
    [sym_url_flg_temp] = ACTIONS(1843),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1843),
    [sym_url_flg_stat_mask] = ACTIONS(1843),
    [sym_url_flg_stat_lookup] = ACTIONS(1843),
    [sym_url_flg_stat_connecting] = ACTIONS(1843),
    [sym_url_flg_stat_waiting] = ACTIONS(1843),
    [sym_url_flg_stat_connected] = ACTIONS(1843),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1843),
    [sym_first_virtual_device] = ACTIONS(1843),
    [sym_first_local_port] = ACTIONS(1843),
    [sym_dynamic_virtual_device] = ACTIONS(1843),
    [sym_timeline_once] = ACTIONS(1843),
    [sym_timeline_repeat] = ACTIONS(1843),
    [sym_timeline_absolute] = ACTIONS(1843),
    [sym_timeline_relative] = ACTIONS(1843),
    [sym_do_push_timed_infinite] = ACTIONS(1843),
    [sym_source_type_no_address] = ACTIONS(1843),
    [sym_source_type_neuron_id] = ACTIONS(1843),
    [sym_source_type_ip_address] = ACTIONS(1843),
    [sym_source_type_axlink] = ACTIONS(1843),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1843),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1843),
    [sym_source_type_ip_socket_address] = ACTIONS(1843),
    [sym_source_type_rs232] = ACTIONS(1843),
    [sym_source_type_internal] = ACTIONS(1843),
    [sym_source_type_mac_address] = ACTIONS(1843),
    [sym_source_type_ipv4_port] = ACTIONS(1843),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1843),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1843),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1843),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1843),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1843),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1843),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1843),
    [sym_true] = ACTIONS(1843),
    [sym_false] = ACTIONS(1843),
    [sym_file_read_only] = ACTIONS(1843),
    [sym_file_rw_new] = ACTIONS(1843),
    [sym_file_rw_append] = ACTIONS(1843),
    [sym_ip_tcp] = ACTIONS(1843),
    [sym_ip_udp] = ACTIONS(1843),
    [sym_ip_udp_2way] = ACTIONS(1843),
    [sym_xml_encode_types] = ACTIONS(1843),
    [sym_xml_encode_char_as_list] = ACTIONS(1843),
    [sym_xml_encode_le] = ACTIONS(1843),
    [sym_xml_decode_types] = ACTIONS(1843),
    [sym_xml_decode_no_preserve] = ACTIONS(1843),
    [sym_duet_dev_type_display_device] = ACTIONS(1843),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1843),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1843),
    [sym_duet_dev_type_io_device] = ACTIONS(1843),
    [sym_duet_dev_type_relay_device] = ACTIONS(1843),
    [sym_duet_dev_type_ups] = ACTIONS(1843),
    [sym_duet_dev_type_amplifier] = ACTIONS(1843),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1843),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1843),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1843),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1843),
    [sym_duet_dev_type_camera] = ACTIONS(1843),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1843),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1843),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1843),
    [sym_duet_dev_type_dss] = ACTIONS(1843),
    [sym_duet_dev_type_dvr] = ACTIONS(1843),
    [sym_duet_dev_type_disc_device] = ACTIONS(1843),
    [sym_duet_dev_type_document_camera] = ACTIONS(1843),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1843),
    [sym_duet_dev_type_hvac] = ACTIONS(1843),
    [sym_duet_dev_type_keypad] = ACTIONS(1843),
    [sym_duet_dev_type_light] = ACTIONS(1843),
    [sym_duet_dev_type_monitor] = ACTIONS(1843),
    [sym_duet_dev_type_motor] = ACTIONS(1843),
    [sym_duet_dev_type_multi_window] = ACTIONS(1843),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1843),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1843),
    [sym_duet_dev_type_receiver] = ACTIONS(1843),
    [sym_duet_dev_type_security_system] = ACTIONS(1843),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1843),
    [sym_duet_dev_type_settop_box] = ACTIONS(1843),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1843),
    [sym_duet_dev_type_switcher] = ACTIONS(1843),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1843),
    [sym_duet_dev_type_tv] = ACTIONS(1843),
    [sym_duet_dev_type_utility] = ACTIONS(1843),
    [sym_duet_dev_type_vcr] = ACTIONS(1843),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1843),
    [sym_duet_dev_type_video_processor] = ACTIONS(1843),
    [sym_duet_dev_type_video_projector] = ACTIONS(1843),
    [sym_duet_dev_type_video_wall] = ACTIONS(1843),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1843),
    [sym_duet_dev_type_weather] = ACTIONS(1843),
    [sym_duet_dev_not_polled] = ACTIONS(1843),
    [sym_duet_dev_polled] = ACTIONS(1843),
    [sym_clkmgr_mode_network] = ACTIONS(1843),
    [sym_clkmgr_mode_standalone] = ACTIONS(1843),
    [sym_normal_standby] = ACTIONS(1843),
    [sym_normal_wake] = ACTIONS(1843),
    [sym_ip_multicast_ttl_option] = ACTIONS(1843),
    [sym_ip_ttl_subnet] = ACTIONS(1843),
    [sym_ip_ttl_site] = ACTIONS(1843),
    [sym_ip_ttl_region] = ACTIONS(1843),
    [sym_ip_ttl_continent] = ACTIONS(1843),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1843),
    [sym_ip_nodelay_on] = ACTIONS(1843),
    [sym_ip_nodelay_off] = ACTIONS(1843),
    [sym_amx_error] = ACTIONS(1843),
    [sym_amx_warning] = ACTIONS(1843),
    [sym_amx_info] = ACTIONS(1843),
    [sym_amx_debug] = ACTIONS(1843),
    [sym_netlinx_login_success] = ACTIONS(1843),
    [sym_netlinx_login_fail] = ACTIONS(1843),
    [sym_netlinx_logout] = ACTIONS(1843),
    [sym_null_str] = ACTIONS(1843),
    [sym_smtp_address] = ACTIONS(1843),
    [sym_smtp_port_number] = ACTIONS(1843),
    [sym_smtp_username] = ACTIONS(1843),
    [sym_smtp_password] = ACTIONS(1843),
    [sym_smtp_require_tls] = ACTIONS(1843),
    [sym_smtp_from] = ACTIONS(1843),
    [sym_smtp_tls_true] = ACTIONS(1843),
    [sym_smtp_tls_false] = ACTIONS(1843),
    [sym_listview_on_row_select_event] = ACTIONS(1843),
    [sym_data_structure_unknown] = ACTIONS(1843),
    [sym_data_structure_datafeed] = ACTIONS(1843),
    [sym_data_structure_datarecordset] = ACTIONS(1843),
    [sym_data_structure_datarecord] = ACTIONS(1843),
    [sym_data_structure_datafield] = ACTIONS(1843),
    [sym_data_max_content_field_count] = ACTIONS(1843),
    [sym_data_max_metadata_field_count] = ACTIONS(1843),
    [sym_data_max_id_length] = ACTIONS(1843),
    [sym_data_max_name_length] = ACTIONS(1843),
    [sym_data_max_description_length] = ACTIONS(1843),
    [sym_data_max_type_length] = ACTIONS(1843),
    [sym_data_max_format_length] = ACTIONS(1843),
    [sym_data_max_source_length] = ACTIONS(1843),
    [sym_data_max_value_length] = ACTIONS(1843),
    [sym_data_max_label_length] = ACTIONS(1843),
    [sym_data_max_url_length] = ACTIONS(1843),
    [sym_max_last_login_info_length] = ACTIONS(1843),
    [sym_data_type_unknown] = ACTIONS(1843),
    [sym_data_type_string] = ACTIONS(1843),
    [sym_data_type_datetime] = ACTIONS(1843),
    [sym_data_type_date] = ACTIONS(1843),
    [sym_data_type_time] = ACTIONS(1843),
    [sym_data_type_image] = ACTIONS(1843),
    [sym_data_format_url] = ACTIONS(1843),
    [sym_data_format_phone] = ACTIONS(1843),
    [sym_data_format_email] = ACTIONS(1843),
    [sym_data_format_iso8601] = ACTIONS(1843),
    [sym_auditlog_priv] = ACTIONS(1843),
    [sym_configuration_priv] = ACTIONS(1843),
    [sym_deviceconfig_priv] = ACTIONS(1843),
    [sym_ftp_priv] = ACTIONS(1843),
    [sym_http_priv] = ACTIONS(1843),
    [sym_networkconfig_priv] = ACTIONS(1843),
    [sym_programport_priv] = ACTIONS(1843),
    [sym_remoteui_priv] = ACTIONS(1843),
    [sym_securitycontrol_priv] = ACTIONS(1843),
    [sym_softwaremanagement_priv] = ACTIONS(1843),
    [sym_terminal_priv] = ACTIONS(1843),
    [sym_tpadmin_priv] = ACTIONS(1843),
    [sym_user1_priv] = ACTIONS(1843),
    [sym_user2_priv] = ACTIONS(1843),
    [sym_user3_priv] = ACTIONS(1843),
    [sym_user4_priv] = ACTIONS(1843),
    [sym_usermanagement_priv] = ACTIONS(1843),
    [sym_valid_account] = ACTIONS(1843),
    [sym_err_name_invalid] = ACTIONS(1843),
    [sym_err_password_invalid] = ACTIONS(1843),
    [sym_err_invalid_account] = ACTIONS(1843),
    [sym_err_no_authorization] = ACTIONS(1843),
    [sym_err_invalid_parameter] = ACTIONS(1843),
    [sym_err_user_locked_out] = ACTIONS(1843),
    [sym_err_user_account_expired] = ACTIONS(1843),
    [sym_err_general_error] = ACTIONS(1843),
    [sym_err_authentication_error] = ACTIONS(1843),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1843),
    [sym_tls_validate_certificate] = ACTIONS(1843),
    [sym_tdata] = ACTIONS(1843),
    [sym_tchannel] = ACTIONS(1843),
    [sym_tlevel] = ACTIONS(1843),
    [sym_tbutton] = ACTIONS(1843),
    [sym_ttimeline] = ACTIONS(1843),
    [sym_tcustom] = ACTIONS(1843),
    [sym_url_struct] = ACTIONS(1843),
    [sym_dns_struct] = ACTIONS(1843),
    [sym_ip_address_struct] = ACTIONS(1843),
    [sym_dev_info_struct] = ACTIONS(1843),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1843),
    [sym_data_feed] = ACTIONS(1843),
    [sym_data_field] = ACTIONS(1843),
    [sym_data_record] = ACTIONS(1843),
    [sym_wc_data_feed] = ACTIONS(1843),
    [sym_wc_data_field] = ACTIONS(1843),
    [sym_wc_data_record] = ACTIONS(1843),
    [sym_last_login_info] = ACTIONS(1843),
    [sym____reserved___] = ACTIONS(1843),
    [sym_date] = ACTIONS(1843),
    [sym_day] = ACTIONS(1843),
    [sym_time] = ACTIONS(1843),
    [sym_ldate] = ACTIONS(1843),
    [sym_push_device] = ACTIONS(1843),
    [sym_push_channel] = ACTIONS(1843),
    [sym_push_devchan] = ACTIONS(1843),
    [sym_release_device] = ACTIONS(1843),
    [sym_release_channel] = ACTIONS(1843),
    [sym_release_devchan] = ACTIONS(1843),
    [sym_master_slot] = ACTIONS(1843),
    [sym_get_pulse_time] = ACTIONS(1843),
    [sym_get_timer] = ACTIONS(1843),
    [sym_master_sn] = ACTIONS(1843),
    [sym_system_number] = ACTIONS(1843),
    [sym_button] = ACTIONS(1843),
    [sym_dv_channel] = ACTIONS(1843),
    [sym_data] = ACTIONS(1843),
    [sym_level] = ACTIONS(1843),
    [sym_timeline] = ACTIONS(1843),
    [sym_channel] = ACTIONS(1843),
    [sym_custom] = ACTIONS(1843),
    [sym_atoi] = ACTIONS(1843),
    [sym_atol] = ACTIONS(1843),
    [sym_atof] = ACTIONS(1843),
    [sym_itoa] = ACTIONS(1843),
    [sym_itohex] = ACTIONS(1843),
    [sym_hextoi] = ACTIONS(1843),
    [sym_ftoa] = ACTIONS(1843),
    [sym_find_string] = ACTIONS(1843),
    [sym_left_string] = ACTIONS(1843),
    [sym_right_string] = ACTIONS(1843),
    [sym_mid_string] = ACTIONS(1843),
    [sym_remove_string] = ACTIONS(1843),
    [sym_get_buffer_string] = ACTIONS(1843),
    [sym_get_buffer_char] = ACTIONS(1843),
    [sym_get_multi_buffer_string] = ACTIONS(1843),
    [sym_length_string] = ACTIONS(1843),
    [sym_max_length_string] = ACTIONS(1843),
    [sym_set_length_string] = ACTIONS(1843),
    [sym_compare_string] = ACTIONS(1843),
    [sym_upper_string] = ACTIONS(1843),
    [sym_lower_string] = ACTIONS(1843),
    [sym_get_last] = ACTIONS(1843),
    [sym_date_to_day] = ACTIONS(1843),
    [sym_date_to_month] = ACTIONS(1843),
    [sym_date_to_year] = ACTIONS(1843),
    [sym_time_to_hour] = ACTIONS(1843),
    [sym_time_to_minute] = ACTIONS(1843),
    [sym_time_to_second] = ACTIONS(1843),
    [sym_day_of_week] = ACTIONS(1843),
    [sym_set_pulse_time] = ACTIONS(1843),
    [sym_set_timer] = ACTIONS(1843),
    [sym_file_open] = ACTIONS(1843),
    [sym_file_close] = ACTIONS(1843),
    [sym_file_read] = ACTIONS(1843),
    [sym_file_read_line] = ACTIONS(1843),
    [sym_file_write] = ACTIONS(1843),
    [sym_file_write_line] = ACTIONS(1843),
    [sym_file_copy] = ACTIONS(1843),
    [sym_file_delete] = ACTIONS(1843),
    [sym_file_rename] = ACTIONS(1843),
    [sym_file_dir] = ACTIONS(1843),
    [sym_file_getdir] = ACTIONS(1843),
    [sym_file_setdir] = ACTIONS(1843),
    [sym_file_create_dir] = ACTIONS(1843),
    [sym_file_remove_dir] = ACTIONS(1843),
    [sym_file_seek] = ACTIONS(1843),
    [sym_length_array] = ACTIONS(1843),
    [sym_max_length_array] = ACTIONS(1843),
    [sym_set_length_array] = ACTIONS(1843),
    [sym_ip_client_open] = ACTIONS(1843),
    [sym_ip_client_close] = ACTIONS(1843),
    [sym_ip_server_open] = ACTIONS(1843),
    [sym_ip_server_close] = ACTIONS(1843),
    [sym_ip_bound_client_open] = ACTIONS(1843),
    [sym_ip_mc_server_open] = ACTIONS(1843),
    [sym_ip_set_option] = ACTIONS(1843),
    [sym_ssh_client_open] = ACTIONS(1843),
    [sym_ssh_client_close] = ACTIONS(1843),
    [sym_tls_client_open] = ACTIONS(1843),
    [sym_tls_client_close] = ACTIONS(1843),
    [sym_do_push] = ACTIONS(1843),
    [sym_do_push_timed] = ACTIONS(1843),
    [sym_do_release] = ACTIONS(1843),
    [sym_device_id] = ACTIONS(1843),
    [sym_device_info] = ACTIONS(1843),
    [sym_device_id_string] = ACTIONS(1843),
    [sym_device_standby] = ACTIONS(1843),
    [sym_device_wake] = ACTIONS(1843),
    [sym_timeline_create] = ACTIONS(1843),
    [sym_timeline_kill] = ACTIONS(1843),
    [sym_timeline_reload] = ACTIONS(1843),
    [sym_timeline_pause] = ACTIONS(1843),
    [sym_timeline_restart] = ACTIONS(1843),
    [sym_timeline_set] = ACTIONS(1843),
    [sym_timeline_get] = ACTIONS(1843),
    [sym_timeline_active] = ACTIONS(1843),
    [sym_timeline_dynamic_id] = ACTIONS(1843),
    [sym_variable_to_string] = ACTIONS(1843),
    [sym_string_to_variable] = ACTIONS(1843),
    [sym_length_variable_to_string] = ACTIONS(1843),
    [sym_type_cast] = ACTIONS(1843),
    [sym_variable_to_xml] = ACTIONS(1843),
    [sym_xml_to_variable] = ACTIONS(1843),
    [sym_length_variable_to_xml] = ACTIONS(1843),
    [sym_redirect_string] = ACTIONS(1843),
    [sym_random_number] = ACTIONS(1843),
    [sym_get_unique_id] = ACTIONS(1843),
    [sym_get_system_number] = ACTIONS(1843),
    [sym_set_system_number] = ACTIONS(1843),
    [sym_get_serial_number] = ACTIONS(1843),
    [sym_get_master_build] = ACTIONS(1843),
    [sym_set_validation_code] = ACTIONS(1843),
    [sym_reboot] = ACTIONS(1843),
    [sym_load_duet_module] = ACTIONS(1843),
    [sym_unload_duet_module] = ACTIONS(1843),
    [sym_duet_mem_size_set] = ACTIONS(1843),
    [sym_duet_mem_size_get] = ACTIONS(1843),
    [sym_dynamic_application_device] = ACTIONS(1843),
    [sym_dynamic_polled_port] = ACTIONS(1843),
    [sym_static_port_binding] = ACTIONS(1843),
    [sym_static_ip_binding] = ACTIONS(1843),
    [sym_get_dns_list] = ACTIONS(1843),
    [sym_set_dns_list] = ACTIONS(1843),
    [sym_get_ip_address] = ACTIONS(1843),
    [sym_set_ip_address] = ACTIONS(1843),
    [sym_get_url_list] = ACTIONS(1843),
    [sym_add_url_entry] = ACTIONS(1843),
    [sym_delete_url_entry] = ACTIONS(1843),
    [sym_abs_value] = ACTIONS(1843),
    [sym_max_value] = ACTIONS(1843),
    [sym_min_value] = ACTIONS(1843),
    [sym_exp_value] = ACTIONS(1843),
    [sym_log_value] = ACTIONS(1843),
    [sym_log10_value] = ACTIONS(1843),
    [sym_power_value] = ACTIONS(1843),
    [sym_sqrt_value] = ACTIONS(1843),
    [sym_raw_be] = ACTIONS(1843),
    [sym_raw_le] = ACTIONS(1843),
    [sym_internal_queue_size_set] = ACTIONS(1843),
    [sym_internal_queue_size_get] = ACTIONS(1843),
    [sym_internal_threshold_set] = ACTIONS(1843),
    [sym_internal_threshold_get] = ACTIONS(1843),
    [sym_rebuilt_event] = ACTIONS(1843),
    [sym_do_custom_event] = ACTIONS(1843),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1843),
    [sym_clkmgr_set_clk_source] = ACTIONS(1843),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1843),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1843),
    [sym_clkmgr_get_timezone] = ACTIONS(1843),
    [sym_clkmgr_set_timezone] = ACTIONS(1843),
    [sym_clkmgr_get_resync_period] = ACTIONS(1843),
    [sym_clkmgr_set_resync_period] = ACTIONS(1843),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1843),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1843),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1843),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1843),
    [sym_clkmgr_get_timeservers] = ACTIONS(1843),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1843),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1843),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1843),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1843),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1843),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1843),
    [sym_set_log_level] = ACTIONS(1843),
    [sym_get_log_level] = ACTIONS(1843),
    [sym_amx_log] = ACTIONS(1843),
    [sym_smtp_server_config_set] = ACTIONS(1843),
    [sym_smtp_server_config_get] = ACTIONS(1843),
    [sym_smtp_send] = ACTIONS(1843),
    [sym_validate_netlinx_account] = ACTIONS(1843),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1843),
    [sym_authenticate_certificate] = ACTIONS(1843),
    [sym_audit_netlinx_session_event] = ACTIONS(1843),
    [sym_audit_netlinx_generic_event] = ACTIONS(1843),
    [sym_get_available_flash_disk_space] = ACTIONS(1843),
    [sym_get_max_flash_disk_space] = ACTIONS(1843),
    [sym_data_create_feed] = ACTIONS(1843),
    [sym_data_delete_feed] = ACTIONS(1843),
    [sym_data_publish_feed] = ACTIONS(1843),
    [sym_data_get_published_feed] = ACTIONS(1843),
    [sym_data_add_record] = ACTIONS(1843),
    [sym_data_get_event_record] = ACTIONS(1843),
    [sym__wc_data_create_feed] = ACTIONS(1843),
    [sym__wc_data_add_record] = ACTIONS(1843),
    [sym__wc_data_get_event_record] = ACTIONS(1843),
    [sym_format] = ACTIONS(1843),
    [sym_astro_clock] = ACTIONS(1843),
    [sym_set_outdoor_temperature] = ACTIONS(1843),
    [sym_set_virtual_level_count] = ACTIONS(1843),
    [sym_set_virtual_channel_count] = ACTIONS(1843),
    [sym_set_virtual_port_count] = ACTIONS(1843),
    [sym__wc_encode_int] = ACTIONS(1843),
    [sym__wc_decode_int] = ACTIONS(1843),
    [sym__wc_int] = ACTIONS(1843),
    [sym__wc_to_ch_int] = ACTIONS(1843),
    [sym__ch_to_wc_int] = ACTIONS(1843),
    [sym__wc_find_string_int] = ACTIONS(1843),
    [sym__wc_left_string_int] = ACTIONS(1843),
    [sym__wc_lower_string_int] = ACTIONS(1843),
    [sym__wc_mid_string_int] = ACTIONS(1843),
    [sym__wc_remove_string_int] = ACTIONS(1843),
    [sym__wc_right_string_int] = ACTIONS(1843),
    [sym__wc_upper_string_int] = ACTIONS(1843),
    [sym__wc_compare_string_int] = ACTIONS(1843),
    [sym__wc_get_buffer_char_int] = ACTIONS(1843),
    [sym__wc_get_buffer_string_int] = ACTIONS(1843),
    [sym__wc_concat_string_int] = ACTIONS(1843),
    [sym__wc_file_open_int] = ACTIONS(1843),
    [sym__wc_file_close_int] = ACTIONS(1843),
    [sym__wc_file_read_int] = ACTIONS(1843),
    [sym__wc_file_read_line_int] = ACTIONS(1843),
    [sym__wc_file_write_int] = ACTIONS(1843),
    [sym__wc_file_write_line_int] = ACTIONS(1843),
    [aux_sym_preproc_include_token1] = ACTIONS(1843),
    [aux_sym_preproc_define_token1] = ACTIONS(1845),
    [aux_sym_preproc_warn_token1] = ACTIONS(1845),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1845),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1845),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1845),
    [anon_sym_LBRACK] = ACTIONS(1845),
    [anon_sym_LBRACE] = ACTIONS(1845),
    [anon_sym_RBRACE] = ACTIONS(1845),
    [anon_sym_LPAREN] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [anon_sym_DASH] = ACTIONS(1843),
    [anon_sym_PLUS] = ACTIONS(1843),
    [anon_sym_DASH_DASH] = ACTIONS(1845),
    [anon_sym_PLUS_PLUS] = ACTIONS(1845),
    [anon_sym_DQUOTE] = ACTIONS(1845),
    [anon_sym_SQUOTE] = ACTIONS(1845),
    [sym_number_literal] = ACTIONS(1845),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(489)] = {
    [sym_identifier] = ACTIONS(1719),
    [sym_define_call_keyword] = ACTIONS(1719),
    [sym_define_function_keyword] = ACTIONS(1719),
    [sym_define_library_function_keyword] = ACTIONS(1719),
    [sym_define_module_keyword] = ACTIONS(1719),
    [sym_char_keyword] = ACTIONS(1719),
    [sym_widechar_keyword] = ACTIONS(1719),
    [sym_integer_keyword] = ACTIONS(1719),
    [sym_sinteger_keyword] = ACTIONS(1719),
    [sym_long_keyword] = ACTIONS(1719),
    [sym_slong_keyword] = ACTIONS(1719),
    [sym_float_keyword] = ACTIONS(1719),
    [sym_double_keyword] = ACTIONS(1719),
    [sym_variant_keyword] = ACTIONS(1719),
    [sym_variantarray_keyword] = ACTIONS(1719),
    [sym_local_var_keyword] = ACTIONS(1719),
    [sym_stack_var_keyword] = ACTIONS(1719),
    [sym_constant_keyword] = ACTIONS(1719),
    [sym_volatile_keyword] = ACTIONS(1719),
    [sym_non_volatile_keyword] = ACTIONS(1719),
    [sym_persistent_keyword] = ACTIONS(1719),
    [sym_dev_keyword] = ACTIONS(1719),
    [sym_devlev_keyword] = ACTIONS(1719),
    [sym_devchan_keyword] = ACTIONS(1719),
    [sym_if_keyword] = ACTIONS(1719),
    [sym_else_keyword] = ACTIONS(1719),
    [sym_switch_keyword] = ACTIONS(1719),
    [sym_case_keyword] = ACTIONS(1719),
    [sym_default_keyword] = ACTIONS(1719),
    [sym_while_keyword] = ACTIONS(1719),
    [sym_for_keyword] = ACTIONS(1719),
    [sym_break_keyword] = ACTIONS(1719),
    [sym_continue_keyword] = ACTIONS(1719),
    [sym_return_keyword] = ACTIONS(1719),
    [sym_select_keyword] = ACTIONS(1719),
    [sym_active_keyword] = ACTIONS(1719),
    [sym_struct_keyword] = ACTIONS(1719),
    [sym_structure_keyword] = ACTIONS(1719),
    [aux_sym_on_keyword_token1] = ACTIONS(1719),
    [aux_sym_off_keyword_token1] = ACTIONS(1719),
    [sym_send_level_keyword] = ACTIONS(1719),
    [sym_send_string_keyword] = ACTIONS(1719),
    [sym_send_command_keyword] = ACTIONS(1719),
    [sym_clear_buffer_keyword] = ACTIONS(1719),
    [sym_create_buffer_keyword] = ACTIONS(1719),
    [sym_create_multi_buffer_keyword] = ACTIONS(1719),
    [sym_call_keyword] = ACTIONS(1719),
    [sym_system_call_keyword] = ACTIONS(1719),
    [sym_devchan_to_keyword] = ACTIONS(1719),
    [sym_devchan_min_to_keyword] = ACTIONS(1719),
    [sym_devchan_total_off_keyword] = ACTIONS(1719),
    [sym_devchan_pulse_keyword] = ACTIONS(1719),
    [sym_wait_keyword] = ACTIONS(1719),
    [sym_cancel_wait_keyword] = ACTIONS(1719),
    [sym_wait_until_keyword] = ACTIONS(1719),
    [sym_cancel_wait_until_keyword] = ACTIONS(1719),
    [sym_cancel_all_wait_keyword] = ACTIONS(1719),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1719),
    [sym_not] = ACTIONS(1719),
    [sym_bnot] = ACTIONS(1719),
    [sym___file__] = ACTIONS(1719),
    [sym___date__] = ACTIONS(1719),
    [sym___ldate__] = ACTIONS(1719),
    [sym___time__] = ACTIONS(1719),
    [sym___line__] = ACTIONS(1719),
    [sym___name__] = ACTIONS(1719),
    [sym___netlinx__] = ACTIONS(1719),
    [sym_netlinx_axi_version] = ACTIONS(1719),
    [sym_url_flg_tcp] = ACTIONS(1719),
    [sym_url_flg_accinfopresent] = ACTIONS(1719),
    [sym_url_flg_temp] = ACTIONS(1719),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1719),
    [sym_url_flg_stat_mask] = ACTIONS(1719),
    [sym_url_flg_stat_lookup] = ACTIONS(1719),
    [sym_url_flg_stat_connecting] = ACTIONS(1719),
    [sym_url_flg_stat_waiting] = ACTIONS(1719),
    [sym_url_flg_stat_connected] = ACTIONS(1719),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1719),
    [sym_first_virtual_device] = ACTIONS(1719),
    [sym_first_local_port] = ACTIONS(1719),
    [sym_dynamic_virtual_device] = ACTIONS(1719),
    [sym_timeline_once] = ACTIONS(1719),
    [sym_timeline_repeat] = ACTIONS(1719),
    [sym_timeline_absolute] = ACTIONS(1719),
    [sym_timeline_relative] = ACTIONS(1719),
    [sym_do_push_timed_infinite] = ACTIONS(1719),
    [sym_source_type_no_address] = ACTIONS(1719),
    [sym_source_type_neuron_id] = ACTIONS(1719),
    [sym_source_type_ip_address] = ACTIONS(1719),
    [sym_source_type_axlink] = ACTIONS(1719),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1719),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1719),
    [sym_source_type_ip_socket_address] = ACTIONS(1719),
    [sym_source_type_rs232] = ACTIONS(1719),
    [sym_source_type_internal] = ACTIONS(1719),
    [sym_source_type_mac_address] = ACTIONS(1719),
    [sym_source_type_ipv4_port] = ACTIONS(1719),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1719),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1719),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1719),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1719),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1719),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1719),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1719),
    [sym_true] = ACTIONS(1719),
    [sym_false] = ACTIONS(1719),
    [sym_file_read_only] = ACTIONS(1719),
    [sym_file_rw_new] = ACTIONS(1719),
    [sym_file_rw_append] = ACTIONS(1719),
    [sym_ip_tcp] = ACTIONS(1719),
    [sym_ip_udp] = ACTIONS(1719),
    [sym_ip_udp_2way] = ACTIONS(1719),
    [sym_xml_encode_types] = ACTIONS(1719),
    [sym_xml_encode_char_as_list] = ACTIONS(1719),
    [sym_xml_encode_le] = ACTIONS(1719),
    [sym_xml_decode_types] = ACTIONS(1719),
    [sym_xml_decode_no_preserve] = ACTIONS(1719),
    [sym_duet_dev_type_display_device] = ACTIONS(1719),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1719),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1719),
    [sym_duet_dev_type_io_device] = ACTIONS(1719),
    [sym_duet_dev_type_relay_device] = ACTIONS(1719),
    [sym_duet_dev_type_ups] = ACTIONS(1719),
    [sym_duet_dev_type_amplifier] = ACTIONS(1719),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1719),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1719),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1719),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1719),
    [sym_duet_dev_type_camera] = ACTIONS(1719),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1719),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1719),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1719),
    [sym_duet_dev_type_dss] = ACTIONS(1719),
    [sym_duet_dev_type_dvr] = ACTIONS(1719),
    [sym_duet_dev_type_disc_device] = ACTIONS(1719),
    [sym_duet_dev_type_document_camera] = ACTIONS(1719),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1719),
    [sym_duet_dev_type_hvac] = ACTIONS(1719),
    [sym_duet_dev_type_keypad] = ACTIONS(1719),
    [sym_duet_dev_type_light] = ACTIONS(1719),
    [sym_duet_dev_type_monitor] = ACTIONS(1719),
    [sym_duet_dev_type_motor] = ACTIONS(1719),
    [sym_duet_dev_type_multi_window] = ACTIONS(1719),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1719),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1719),
    [sym_duet_dev_type_receiver] = ACTIONS(1719),
    [sym_duet_dev_type_security_system] = ACTIONS(1719),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1719),
    [sym_duet_dev_type_settop_box] = ACTIONS(1719),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1719),
    [sym_duet_dev_type_switcher] = ACTIONS(1719),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1719),
    [sym_duet_dev_type_tv] = ACTIONS(1719),
    [sym_duet_dev_type_utility] = ACTIONS(1719),
    [sym_duet_dev_type_vcr] = ACTIONS(1719),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1719),
    [sym_duet_dev_type_video_processor] = ACTIONS(1719),
    [sym_duet_dev_type_video_projector] = ACTIONS(1719),
    [sym_duet_dev_type_video_wall] = ACTIONS(1719),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1719),
    [sym_duet_dev_type_weather] = ACTIONS(1719),
    [sym_duet_dev_not_polled] = ACTIONS(1719),
    [sym_duet_dev_polled] = ACTIONS(1719),
    [sym_clkmgr_mode_network] = ACTIONS(1719),
    [sym_clkmgr_mode_standalone] = ACTIONS(1719),
    [sym_normal_standby] = ACTIONS(1719),
    [sym_normal_wake] = ACTIONS(1719),
    [sym_ip_multicast_ttl_option] = ACTIONS(1719),
    [sym_ip_ttl_subnet] = ACTIONS(1719),
    [sym_ip_ttl_site] = ACTIONS(1719),
    [sym_ip_ttl_region] = ACTIONS(1719),
    [sym_ip_ttl_continent] = ACTIONS(1719),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1719),
    [sym_ip_nodelay_on] = ACTIONS(1719),
    [sym_ip_nodelay_off] = ACTIONS(1719),
    [sym_amx_error] = ACTIONS(1719),
    [sym_amx_warning] = ACTIONS(1719),
    [sym_amx_info] = ACTIONS(1719),
    [sym_amx_debug] = ACTIONS(1719),
    [sym_netlinx_login_success] = ACTIONS(1719),
    [sym_netlinx_login_fail] = ACTIONS(1719),
    [sym_netlinx_logout] = ACTIONS(1719),
    [sym_null_str] = ACTIONS(1719),
    [sym_smtp_address] = ACTIONS(1719),
    [sym_smtp_port_number] = ACTIONS(1719),
    [sym_smtp_username] = ACTIONS(1719),
    [sym_smtp_password] = ACTIONS(1719),
    [sym_smtp_require_tls] = ACTIONS(1719),
    [sym_smtp_from] = ACTIONS(1719),
    [sym_smtp_tls_true] = ACTIONS(1719),
    [sym_smtp_tls_false] = ACTIONS(1719),
    [sym_listview_on_row_select_event] = ACTIONS(1719),
    [sym_data_structure_unknown] = ACTIONS(1719),
    [sym_data_structure_datafeed] = ACTIONS(1719),
    [sym_data_structure_datarecordset] = ACTIONS(1719),
    [sym_data_structure_datarecord] = ACTIONS(1719),
    [sym_data_structure_datafield] = ACTIONS(1719),
    [sym_data_max_content_field_count] = ACTIONS(1719),
    [sym_data_max_metadata_field_count] = ACTIONS(1719),
    [sym_data_max_id_length] = ACTIONS(1719),
    [sym_data_max_name_length] = ACTIONS(1719),
    [sym_data_max_description_length] = ACTIONS(1719),
    [sym_data_max_type_length] = ACTIONS(1719),
    [sym_data_max_format_length] = ACTIONS(1719),
    [sym_data_max_source_length] = ACTIONS(1719),
    [sym_data_max_value_length] = ACTIONS(1719),
    [sym_data_max_label_length] = ACTIONS(1719),
    [sym_data_max_url_length] = ACTIONS(1719),
    [sym_max_last_login_info_length] = ACTIONS(1719),
    [sym_data_type_unknown] = ACTIONS(1719),
    [sym_data_type_string] = ACTIONS(1719),
    [sym_data_type_datetime] = ACTIONS(1719),
    [sym_data_type_date] = ACTIONS(1719),
    [sym_data_type_time] = ACTIONS(1719),
    [sym_data_type_image] = ACTIONS(1719),
    [sym_data_format_url] = ACTIONS(1719),
    [sym_data_format_phone] = ACTIONS(1719),
    [sym_data_format_email] = ACTIONS(1719),
    [sym_data_format_iso8601] = ACTIONS(1719),
    [sym_auditlog_priv] = ACTIONS(1719),
    [sym_configuration_priv] = ACTIONS(1719),
    [sym_deviceconfig_priv] = ACTIONS(1719),
    [sym_ftp_priv] = ACTIONS(1719),
    [sym_http_priv] = ACTIONS(1719),
    [sym_networkconfig_priv] = ACTIONS(1719),
    [sym_programport_priv] = ACTIONS(1719),
    [sym_remoteui_priv] = ACTIONS(1719),
    [sym_securitycontrol_priv] = ACTIONS(1719),
    [sym_softwaremanagement_priv] = ACTIONS(1719),
    [sym_terminal_priv] = ACTIONS(1719),
    [sym_tpadmin_priv] = ACTIONS(1719),
    [sym_user1_priv] = ACTIONS(1719),
    [sym_user2_priv] = ACTIONS(1719),
    [sym_user3_priv] = ACTIONS(1719),
    [sym_user4_priv] = ACTIONS(1719),
    [sym_usermanagement_priv] = ACTIONS(1719),
    [sym_valid_account] = ACTIONS(1719),
    [sym_err_name_invalid] = ACTIONS(1719),
    [sym_err_password_invalid] = ACTIONS(1719),
    [sym_err_invalid_account] = ACTIONS(1719),
    [sym_err_no_authorization] = ACTIONS(1719),
    [sym_err_invalid_parameter] = ACTIONS(1719),
    [sym_err_user_locked_out] = ACTIONS(1719),
    [sym_err_user_account_expired] = ACTIONS(1719),
    [sym_err_general_error] = ACTIONS(1719),
    [sym_err_authentication_error] = ACTIONS(1719),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1719),
    [sym_tls_validate_certificate] = ACTIONS(1719),
    [sym_tdata] = ACTIONS(1719),
    [sym_tchannel] = ACTIONS(1719),
    [sym_tlevel] = ACTIONS(1719),
    [sym_tbutton] = ACTIONS(1719),
    [sym_ttimeline] = ACTIONS(1719),
    [sym_tcustom] = ACTIONS(1719),
    [sym_url_struct] = ACTIONS(1719),
    [sym_dns_struct] = ACTIONS(1719),
    [sym_ip_address_struct] = ACTIONS(1719),
    [sym_dev_info_struct] = ACTIONS(1719),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1719),
    [sym_data_feed] = ACTIONS(1719),
    [sym_data_field] = ACTIONS(1719),
    [sym_data_record] = ACTIONS(1719),
    [sym_wc_data_feed] = ACTIONS(1719),
    [sym_wc_data_field] = ACTIONS(1719),
    [sym_wc_data_record] = ACTIONS(1719),
    [sym_last_login_info] = ACTIONS(1719),
    [sym____reserved___] = ACTIONS(1719),
    [sym_date] = ACTIONS(1719),
    [sym_day] = ACTIONS(1719),
    [sym_time] = ACTIONS(1719),
    [sym_ldate] = ACTIONS(1719),
    [sym_push_device] = ACTIONS(1719),
    [sym_push_channel] = ACTIONS(1719),
    [sym_push_devchan] = ACTIONS(1719),
    [sym_release_device] = ACTIONS(1719),
    [sym_release_channel] = ACTIONS(1719),
    [sym_release_devchan] = ACTIONS(1719),
    [sym_master_slot] = ACTIONS(1719),
    [sym_get_pulse_time] = ACTIONS(1719),
    [sym_get_timer] = ACTIONS(1719),
    [sym_master_sn] = ACTIONS(1719),
    [sym_system_number] = ACTIONS(1719),
    [sym_button] = ACTIONS(1719),
    [sym_dv_channel] = ACTIONS(1719),
    [sym_data] = ACTIONS(1719),
    [sym_level] = ACTIONS(1719),
    [sym_timeline] = ACTIONS(1719),
    [sym_channel] = ACTIONS(1719),
    [sym_custom] = ACTIONS(1719),
    [sym_atoi] = ACTIONS(1719),
    [sym_atol] = ACTIONS(1719),
    [sym_atof] = ACTIONS(1719),
    [sym_itoa] = ACTIONS(1719),
    [sym_itohex] = ACTIONS(1719),
    [sym_hextoi] = ACTIONS(1719),
    [sym_ftoa] = ACTIONS(1719),
    [sym_find_string] = ACTIONS(1719),
    [sym_left_string] = ACTIONS(1719),
    [sym_right_string] = ACTIONS(1719),
    [sym_mid_string] = ACTIONS(1719),
    [sym_remove_string] = ACTIONS(1719),
    [sym_get_buffer_string] = ACTIONS(1719),
    [sym_get_buffer_char] = ACTIONS(1719),
    [sym_get_multi_buffer_string] = ACTIONS(1719),
    [sym_length_string] = ACTIONS(1719),
    [sym_max_length_string] = ACTIONS(1719),
    [sym_set_length_string] = ACTIONS(1719),
    [sym_compare_string] = ACTIONS(1719),
    [sym_upper_string] = ACTIONS(1719),
    [sym_lower_string] = ACTIONS(1719),
    [sym_get_last] = ACTIONS(1719),
    [sym_date_to_day] = ACTIONS(1719),
    [sym_date_to_month] = ACTIONS(1719),
    [sym_date_to_year] = ACTIONS(1719),
    [sym_time_to_hour] = ACTIONS(1719),
    [sym_time_to_minute] = ACTIONS(1719),
    [sym_time_to_second] = ACTIONS(1719),
    [sym_day_of_week] = ACTIONS(1719),
    [sym_set_pulse_time] = ACTIONS(1719),
    [sym_set_timer] = ACTIONS(1719),
    [sym_file_open] = ACTIONS(1719),
    [sym_file_close] = ACTIONS(1719),
    [sym_file_read] = ACTIONS(1719),
    [sym_file_read_line] = ACTIONS(1719),
    [sym_file_write] = ACTIONS(1719),
    [sym_file_write_line] = ACTIONS(1719),
    [sym_file_copy] = ACTIONS(1719),
    [sym_file_delete] = ACTIONS(1719),
    [sym_file_rename] = ACTIONS(1719),
    [sym_file_dir] = ACTIONS(1719),
    [sym_file_getdir] = ACTIONS(1719),
    [sym_file_setdir] = ACTIONS(1719),
    [sym_file_create_dir] = ACTIONS(1719),
    [sym_file_remove_dir] = ACTIONS(1719),
    [sym_file_seek] = ACTIONS(1719),
    [sym_length_array] = ACTIONS(1719),
    [sym_max_length_array] = ACTIONS(1719),
    [sym_set_length_array] = ACTIONS(1719),
    [sym_ip_client_open] = ACTIONS(1719),
    [sym_ip_client_close] = ACTIONS(1719),
    [sym_ip_server_open] = ACTIONS(1719),
    [sym_ip_server_close] = ACTIONS(1719),
    [sym_ip_bound_client_open] = ACTIONS(1719),
    [sym_ip_mc_server_open] = ACTIONS(1719),
    [sym_ip_set_option] = ACTIONS(1719),
    [sym_ssh_client_open] = ACTIONS(1719),
    [sym_ssh_client_close] = ACTIONS(1719),
    [sym_tls_client_open] = ACTIONS(1719),
    [sym_tls_client_close] = ACTIONS(1719),
    [sym_do_push] = ACTIONS(1719),
    [sym_do_push_timed] = ACTIONS(1719),
    [sym_do_release] = ACTIONS(1719),
    [sym_device_id] = ACTIONS(1719),
    [sym_device_info] = ACTIONS(1719),
    [sym_device_id_string] = ACTIONS(1719),
    [sym_device_standby] = ACTIONS(1719),
    [sym_device_wake] = ACTIONS(1719),
    [sym_timeline_create] = ACTIONS(1719),
    [sym_timeline_kill] = ACTIONS(1719),
    [sym_timeline_reload] = ACTIONS(1719),
    [sym_timeline_pause] = ACTIONS(1719),
    [sym_timeline_restart] = ACTIONS(1719),
    [sym_timeline_set] = ACTIONS(1719),
    [sym_timeline_get] = ACTIONS(1719),
    [sym_timeline_active] = ACTIONS(1719),
    [sym_timeline_dynamic_id] = ACTIONS(1719),
    [sym_variable_to_string] = ACTIONS(1719),
    [sym_string_to_variable] = ACTIONS(1719),
    [sym_length_variable_to_string] = ACTIONS(1719),
    [sym_type_cast] = ACTIONS(1719),
    [sym_variable_to_xml] = ACTIONS(1719),
    [sym_xml_to_variable] = ACTIONS(1719),
    [sym_length_variable_to_xml] = ACTIONS(1719),
    [sym_redirect_string] = ACTIONS(1719),
    [sym_random_number] = ACTIONS(1719),
    [sym_get_unique_id] = ACTIONS(1719),
    [sym_get_system_number] = ACTIONS(1719),
    [sym_set_system_number] = ACTIONS(1719),
    [sym_get_serial_number] = ACTIONS(1719),
    [sym_get_master_build] = ACTIONS(1719),
    [sym_set_validation_code] = ACTIONS(1719),
    [sym_reboot] = ACTIONS(1719),
    [sym_load_duet_module] = ACTIONS(1719),
    [sym_unload_duet_module] = ACTIONS(1719),
    [sym_duet_mem_size_set] = ACTIONS(1719),
    [sym_duet_mem_size_get] = ACTIONS(1719),
    [sym_dynamic_application_device] = ACTIONS(1719),
    [sym_dynamic_polled_port] = ACTIONS(1719),
    [sym_static_port_binding] = ACTIONS(1719),
    [sym_static_ip_binding] = ACTIONS(1719),
    [sym_get_dns_list] = ACTIONS(1719),
    [sym_set_dns_list] = ACTIONS(1719),
    [sym_get_ip_address] = ACTIONS(1719),
    [sym_set_ip_address] = ACTIONS(1719),
    [sym_get_url_list] = ACTIONS(1719),
    [sym_add_url_entry] = ACTIONS(1719),
    [sym_delete_url_entry] = ACTIONS(1719),
    [sym_abs_value] = ACTIONS(1719),
    [sym_max_value] = ACTIONS(1719),
    [sym_min_value] = ACTIONS(1719),
    [sym_exp_value] = ACTIONS(1719),
    [sym_log_value] = ACTIONS(1719),
    [sym_log10_value] = ACTIONS(1719),
    [sym_power_value] = ACTIONS(1719),
    [sym_sqrt_value] = ACTIONS(1719),
    [sym_raw_be] = ACTIONS(1719),
    [sym_raw_le] = ACTIONS(1719),
    [sym_internal_queue_size_set] = ACTIONS(1719),
    [sym_internal_queue_size_get] = ACTIONS(1719),
    [sym_internal_threshold_set] = ACTIONS(1719),
    [sym_internal_threshold_get] = ACTIONS(1719),
    [sym_rebuilt_event] = ACTIONS(1719),
    [sym_do_custom_event] = ACTIONS(1719),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1719),
    [sym_clkmgr_set_clk_source] = ACTIONS(1719),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1719),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1719),
    [sym_clkmgr_get_timezone] = ACTIONS(1719),
    [sym_clkmgr_set_timezone] = ACTIONS(1719),
    [sym_clkmgr_get_resync_period] = ACTIONS(1719),
    [sym_clkmgr_set_resync_period] = ACTIONS(1719),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1719),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1719),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1719),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1719),
    [sym_clkmgr_get_timeservers] = ACTIONS(1719),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1719),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1719),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1719),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1719),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1719),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1719),
    [sym_set_log_level] = ACTIONS(1719),
    [sym_get_log_level] = ACTIONS(1719),
    [sym_amx_log] = ACTIONS(1719),
    [sym_smtp_server_config_set] = ACTIONS(1719),
    [sym_smtp_server_config_get] = ACTIONS(1719),
    [sym_smtp_send] = ACTIONS(1719),
    [sym_validate_netlinx_account] = ACTIONS(1719),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1719),
    [sym_authenticate_certificate] = ACTIONS(1719),
    [sym_audit_netlinx_session_event] = ACTIONS(1719),
    [sym_audit_netlinx_generic_event] = ACTIONS(1719),
    [sym_get_available_flash_disk_space] = ACTIONS(1719),
    [sym_get_max_flash_disk_space] = ACTIONS(1719),
    [sym_data_create_feed] = ACTIONS(1719),
    [sym_data_delete_feed] = ACTIONS(1719),
    [sym_data_publish_feed] = ACTIONS(1719),
    [sym_data_get_published_feed] = ACTIONS(1719),
    [sym_data_add_record] = ACTIONS(1719),
    [sym_data_get_event_record] = ACTIONS(1719),
    [sym__wc_data_create_feed] = ACTIONS(1719),
    [sym__wc_data_add_record] = ACTIONS(1719),
    [sym__wc_data_get_event_record] = ACTIONS(1719),
    [sym_format] = ACTIONS(1719),
    [sym_astro_clock] = ACTIONS(1719),
    [sym_set_outdoor_temperature] = ACTIONS(1719),
    [sym_set_virtual_level_count] = ACTIONS(1719),
    [sym_set_virtual_channel_count] = ACTIONS(1719),
    [sym_set_virtual_port_count] = ACTIONS(1719),
    [sym__wc_encode_int] = ACTIONS(1719),
    [sym__wc_decode_int] = ACTIONS(1719),
    [sym__wc_int] = ACTIONS(1719),
    [sym__wc_to_ch_int] = ACTIONS(1719),
    [sym__ch_to_wc_int] = ACTIONS(1719),
    [sym__wc_find_string_int] = ACTIONS(1719),
    [sym__wc_left_string_int] = ACTIONS(1719),
    [sym__wc_lower_string_int] = ACTIONS(1719),
    [sym__wc_mid_string_int] = ACTIONS(1719),
    [sym__wc_remove_string_int] = ACTIONS(1719),
    [sym__wc_right_string_int] = ACTIONS(1719),
    [sym__wc_upper_string_int] = ACTIONS(1719),
    [sym__wc_compare_string_int] = ACTIONS(1719),
    [sym__wc_get_buffer_char_int] = ACTIONS(1719),
    [sym__wc_get_buffer_string_int] = ACTIONS(1719),
    [sym__wc_concat_string_int] = ACTIONS(1719),
    [sym__wc_file_open_int] = ACTIONS(1719),
    [sym__wc_file_close_int] = ACTIONS(1719),
    [sym__wc_file_read_int] = ACTIONS(1719),
    [sym__wc_file_read_line_int] = ACTIONS(1719),
    [sym__wc_file_write_int] = ACTIONS(1719),
    [sym__wc_file_write_line_int] = ACTIONS(1719),
    [aux_sym_preproc_include_token1] = ACTIONS(1719),
    [aux_sym_preproc_define_token1] = ACTIONS(1721),
    [aux_sym_preproc_warn_token1] = ACTIONS(1721),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1721),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1721),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1721),
    [anon_sym_LBRACK] = ACTIONS(1721),
    [anon_sym_LBRACE] = ACTIONS(1721),
    [anon_sym_RBRACE] = ACTIONS(1721),
    [anon_sym_LPAREN] = ACTIONS(1719),
    [anon_sym_BANG] = ACTIONS(1721),
    [anon_sym_TILDE] = ACTIONS(1721),
    [anon_sym_DASH] = ACTIONS(1719),
    [anon_sym_PLUS] = ACTIONS(1719),
    [anon_sym_DASH_DASH] = ACTIONS(1721),
    [anon_sym_PLUS_PLUS] = ACTIONS(1721),
    [anon_sym_DQUOTE] = ACTIONS(1721),
    [anon_sym_SQUOTE] = ACTIONS(1721),
    [sym_number_literal] = ACTIONS(1721),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(490)] = {
    [sym_identifier] = ACTIONS(1723),
    [sym_define_call_keyword] = ACTIONS(1723),
    [sym_define_function_keyword] = ACTIONS(1723),
    [sym_define_library_function_keyword] = ACTIONS(1723),
    [sym_define_module_keyword] = ACTIONS(1723),
    [sym_char_keyword] = ACTIONS(1723),
    [sym_widechar_keyword] = ACTIONS(1723),
    [sym_integer_keyword] = ACTIONS(1723),
    [sym_sinteger_keyword] = ACTIONS(1723),
    [sym_long_keyword] = ACTIONS(1723),
    [sym_slong_keyword] = ACTIONS(1723),
    [sym_float_keyword] = ACTIONS(1723),
    [sym_double_keyword] = ACTIONS(1723),
    [sym_variant_keyword] = ACTIONS(1723),
    [sym_variantarray_keyword] = ACTIONS(1723),
    [sym_local_var_keyword] = ACTIONS(1723),
    [sym_stack_var_keyword] = ACTIONS(1723),
    [sym_constant_keyword] = ACTIONS(1723),
    [sym_volatile_keyword] = ACTIONS(1723),
    [sym_non_volatile_keyword] = ACTIONS(1723),
    [sym_persistent_keyword] = ACTIONS(1723),
    [sym_dev_keyword] = ACTIONS(1723),
    [sym_devlev_keyword] = ACTIONS(1723),
    [sym_devchan_keyword] = ACTIONS(1723),
    [sym_if_keyword] = ACTIONS(1723),
    [sym_else_keyword] = ACTIONS(1723),
    [sym_switch_keyword] = ACTIONS(1723),
    [sym_case_keyword] = ACTIONS(1723),
    [sym_default_keyword] = ACTIONS(1723),
    [sym_while_keyword] = ACTIONS(1723),
    [sym_for_keyword] = ACTIONS(1723),
    [sym_break_keyword] = ACTIONS(1723),
    [sym_continue_keyword] = ACTIONS(1723),
    [sym_return_keyword] = ACTIONS(1723),
    [sym_select_keyword] = ACTIONS(1723),
    [sym_active_keyword] = ACTIONS(1723),
    [sym_struct_keyword] = ACTIONS(1723),
    [sym_structure_keyword] = ACTIONS(1723),
    [aux_sym_on_keyword_token1] = ACTIONS(1723),
    [aux_sym_off_keyword_token1] = ACTIONS(1723),
    [sym_send_level_keyword] = ACTIONS(1723),
    [sym_send_string_keyword] = ACTIONS(1723),
    [sym_send_command_keyword] = ACTIONS(1723),
    [sym_clear_buffer_keyword] = ACTIONS(1723),
    [sym_create_buffer_keyword] = ACTIONS(1723),
    [sym_create_multi_buffer_keyword] = ACTIONS(1723),
    [sym_call_keyword] = ACTIONS(1723),
    [sym_system_call_keyword] = ACTIONS(1723),
    [sym_devchan_to_keyword] = ACTIONS(1723),
    [sym_devchan_min_to_keyword] = ACTIONS(1723),
    [sym_devchan_total_off_keyword] = ACTIONS(1723),
    [sym_devchan_pulse_keyword] = ACTIONS(1723),
    [sym_wait_keyword] = ACTIONS(1723),
    [sym_cancel_wait_keyword] = ACTIONS(1723),
    [sym_wait_until_keyword] = ACTIONS(1723),
    [sym_cancel_wait_until_keyword] = ACTIONS(1723),
    [sym_cancel_all_wait_keyword] = ACTIONS(1723),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1723),
    [sym_not] = ACTIONS(1723),
    [sym_bnot] = ACTIONS(1723),
    [sym___file__] = ACTIONS(1723),
    [sym___date__] = ACTIONS(1723),
    [sym___ldate__] = ACTIONS(1723),
    [sym___time__] = ACTIONS(1723),
    [sym___line__] = ACTIONS(1723),
    [sym___name__] = ACTIONS(1723),
    [sym___netlinx__] = ACTIONS(1723),
    [sym_netlinx_axi_version] = ACTIONS(1723),
    [sym_url_flg_tcp] = ACTIONS(1723),
    [sym_url_flg_accinfopresent] = ACTIONS(1723),
    [sym_url_flg_temp] = ACTIONS(1723),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1723),
    [sym_url_flg_stat_mask] = ACTIONS(1723),
    [sym_url_flg_stat_lookup] = ACTIONS(1723),
    [sym_url_flg_stat_connecting] = ACTIONS(1723),
    [sym_url_flg_stat_waiting] = ACTIONS(1723),
    [sym_url_flg_stat_connected] = ACTIONS(1723),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1723),
    [sym_first_virtual_device] = ACTIONS(1723),
    [sym_first_local_port] = ACTIONS(1723),
    [sym_dynamic_virtual_device] = ACTIONS(1723),
    [sym_timeline_once] = ACTIONS(1723),
    [sym_timeline_repeat] = ACTIONS(1723),
    [sym_timeline_absolute] = ACTIONS(1723),
    [sym_timeline_relative] = ACTIONS(1723),
    [sym_do_push_timed_infinite] = ACTIONS(1723),
    [sym_source_type_no_address] = ACTIONS(1723),
    [sym_source_type_neuron_id] = ACTIONS(1723),
    [sym_source_type_ip_address] = ACTIONS(1723),
    [sym_source_type_axlink] = ACTIONS(1723),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1723),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1723),
    [sym_source_type_ip_socket_address] = ACTIONS(1723),
    [sym_source_type_rs232] = ACTIONS(1723),
    [sym_source_type_internal] = ACTIONS(1723),
    [sym_source_type_mac_address] = ACTIONS(1723),
    [sym_source_type_ipv4_port] = ACTIONS(1723),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1723),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1723),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1723),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1723),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1723),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1723),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1723),
    [sym_true] = ACTIONS(1723),
    [sym_false] = ACTIONS(1723),
    [sym_file_read_only] = ACTIONS(1723),
    [sym_file_rw_new] = ACTIONS(1723),
    [sym_file_rw_append] = ACTIONS(1723),
    [sym_ip_tcp] = ACTIONS(1723),
    [sym_ip_udp] = ACTIONS(1723),
    [sym_ip_udp_2way] = ACTIONS(1723),
    [sym_xml_encode_types] = ACTIONS(1723),
    [sym_xml_encode_char_as_list] = ACTIONS(1723),
    [sym_xml_encode_le] = ACTIONS(1723),
    [sym_xml_decode_types] = ACTIONS(1723),
    [sym_xml_decode_no_preserve] = ACTIONS(1723),
    [sym_duet_dev_type_display_device] = ACTIONS(1723),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1723),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1723),
    [sym_duet_dev_type_io_device] = ACTIONS(1723),
    [sym_duet_dev_type_relay_device] = ACTIONS(1723),
    [sym_duet_dev_type_ups] = ACTIONS(1723),
    [sym_duet_dev_type_amplifier] = ACTIONS(1723),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1723),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1723),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1723),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1723),
    [sym_duet_dev_type_camera] = ACTIONS(1723),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1723),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1723),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1723),
    [sym_duet_dev_type_dss] = ACTIONS(1723),
    [sym_duet_dev_type_dvr] = ACTIONS(1723),
    [sym_duet_dev_type_disc_device] = ACTIONS(1723),
    [sym_duet_dev_type_document_camera] = ACTIONS(1723),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1723),
    [sym_duet_dev_type_hvac] = ACTIONS(1723),
    [sym_duet_dev_type_keypad] = ACTIONS(1723),
    [sym_duet_dev_type_light] = ACTIONS(1723),
    [sym_duet_dev_type_monitor] = ACTIONS(1723),
    [sym_duet_dev_type_motor] = ACTIONS(1723),
    [sym_duet_dev_type_multi_window] = ACTIONS(1723),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1723),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1723),
    [sym_duet_dev_type_receiver] = ACTIONS(1723),
    [sym_duet_dev_type_security_system] = ACTIONS(1723),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1723),
    [sym_duet_dev_type_settop_box] = ACTIONS(1723),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1723),
    [sym_duet_dev_type_switcher] = ACTIONS(1723),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1723),
    [sym_duet_dev_type_tv] = ACTIONS(1723),
    [sym_duet_dev_type_utility] = ACTIONS(1723),
    [sym_duet_dev_type_vcr] = ACTIONS(1723),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1723),
    [sym_duet_dev_type_video_processor] = ACTIONS(1723),
    [sym_duet_dev_type_video_projector] = ACTIONS(1723),
    [sym_duet_dev_type_video_wall] = ACTIONS(1723),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1723),
    [sym_duet_dev_type_weather] = ACTIONS(1723),
    [sym_duet_dev_not_polled] = ACTIONS(1723),
    [sym_duet_dev_polled] = ACTIONS(1723),
    [sym_clkmgr_mode_network] = ACTIONS(1723),
    [sym_clkmgr_mode_standalone] = ACTIONS(1723),
    [sym_normal_standby] = ACTIONS(1723),
    [sym_normal_wake] = ACTIONS(1723),
    [sym_ip_multicast_ttl_option] = ACTIONS(1723),
    [sym_ip_ttl_subnet] = ACTIONS(1723),
    [sym_ip_ttl_site] = ACTIONS(1723),
    [sym_ip_ttl_region] = ACTIONS(1723),
    [sym_ip_ttl_continent] = ACTIONS(1723),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1723),
    [sym_ip_nodelay_on] = ACTIONS(1723),
    [sym_ip_nodelay_off] = ACTIONS(1723),
    [sym_amx_error] = ACTIONS(1723),
    [sym_amx_warning] = ACTIONS(1723),
    [sym_amx_info] = ACTIONS(1723),
    [sym_amx_debug] = ACTIONS(1723),
    [sym_netlinx_login_success] = ACTIONS(1723),
    [sym_netlinx_login_fail] = ACTIONS(1723),
    [sym_netlinx_logout] = ACTIONS(1723),
    [sym_null_str] = ACTIONS(1723),
    [sym_smtp_address] = ACTIONS(1723),
    [sym_smtp_port_number] = ACTIONS(1723),
    [sym_smtp_username] = ACTIONS(1723),
    [sym_smtp_password] = ACTIONS(1723),
    [sym_smtp_require_tls] = ACTIONS(1723),
    [sym_smtp_from] = ACTIONS(1723),
    [sym_smtp_tls_true] = ACTIONS(1723),
    [sym_smtp_tls_false] = ACTIONS(1723),
    [sym_listview_on_row_select_event] = ACTIONS(1723),
    [sym_data_structure_unknown] = ACTIONS(1723),
    [sym_data_structure_datafeed] = ACTIONS(1723),
    [sym_data_structure_datarecordset] = ACTIONS(1723),
    [sym_data_structure_datarecord] = ACTIONS(1723),
    [sym_data_structure_datafield] = ACTIONS(1723),
    [sym_data_max_content_field_count] = ACTIONS(1723),
    [sym_data_max_metadata_field_count] = ACTIONS(1723),
    [sym_data_max_id_length] = ACTIONS(1723),
    [sym_data_max_name_length] = ACTIONS(1723),
    [sym_data_max_description_length] = ACTIONS(1723),
    [sym_data_max_type_length] = ACTIONS(1723),
    [sym_data_max_format_length] = ACTIONS(1723),
    [sym_data_max_source_length] = ACTIONS(1723),
    [sym_data_max_value_length] = ACTIONS(1723),
    [sym_data_max_label_length] = ACTIONS(1723),
    [sym_data_max_url_length] = ACTIONS(1723),
    [sym_max_last_login_info_length] = ACTIONS(1723),
    [sym_data_type_unknown] = ACTIONS(1723),
    [sym_data_type_string] = ACTIONS(1723),
    [sym_data_type_datetime] = ACTIONS(1723),
    [sym_data_type_date] = ACTIONS(1723),
    [sym_data_type_time] = ACTIONS(1723),
    [sym_data_type_image] = ACTIONS(1723),
    [sym_data_format_url] = ACTIONS(1723),
    [sym_data_format_phone] = ACTIONS(1723),
    [sym_data_format_email] = ACTIONS(1723),
    [sym_data_format_iso8601] = ACTIONS(1723),
    [sym_auditlog_priv] = ACTIONS(1723),
    [sym_configuration_priv] = ACTIONS(1723),
    [sym_deviceconfig_priv] = ACTIONS(1723),
    [sym_ftp_priv] = ACTIONS(1723),
    [sym_http_priv] = ACTIONS(1723),
    [sym_networkconfig_priv] = ACTIONS(1723),
    [sym_programport_priv] = ACTIONS(1723),
    [sym_remoteui_priv] = ACTIONS(1723),
    [sym_securitycontrol_priv] = ACTIONS(1723),
    [sym_softwaremanagement_priv] = ACTIONS(1723),
    [sym_terminal_priv] = ACTIONS(1723),
    [sym_tpadmin_priv] = ACTIONS(1723),
    [sym_user1_priv] = ACTIONS(1723),
    [sym_user2_priv] = ACTIONS(1723),
    [sym_user3_priv] = ACTIONS(1723),
    [sym_user4_priv] = ACTIONS(1723),
    [sym_usermanagement_priv] = ACTIONS(1723),
    [sym_valid_account] = ACTIONS(1723),
    [sym_err_name_invalid] = ACTIONS(1723),
    [sym_err_password_invalid] = ACTIONS(1723),
    [sym_err_invalid_account] = ACTIONS(1723),
    [sym_err_no_authorization] = ACTIONS(1723),
    [sym_err_invalid_parameter] = ACTIONS(1723),
    [sym_err_user_locked_out] = ACTIONS(1723),
    [sym_err_user_account_expired] = ACTIONS(1723),
    [sym_err_general_error] = ACTIONS(1723),
    [sym_err_authentication_error] = ACTIONS(1723),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1723),
    [sym_tls_validate_certificate] = ACTIONS(1723),
    [sym_tdata] = ACTIONS(1723),
    [sym_tchannel] = ACTIONS(1723),
    [sym_tlevel] = ACTIONS(1723),
    [sym_tbutton] = ACTIONS(1723),
    [sym_ttimeline] = ACTIONS(1723),
    [sym_tcustom] = ACTIONS(1723),
    [sym_url_struct] = ACTIONS(1723),
    [sym_dns_struct] = ACTIONS(1723),
    [sym_ip_address_struct] = ACTIONS(1723),
    [sym_dev_info_struct] = ACTIONS(1723),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1723),
    [sym_data_feed] = ACTIONS(1723),
    [sym_data_field] = ACTIONS(1723),
    [sym_data_record] = ACTIONS(1723),
    [sym_wc_data_feed] = ACTIONS(1723),
    [sym_wc_data_field] = ACTIONS(1723),
    [sym_wc_data_record] = ACTIONS(1723),
    [sym_last_login_info] = ACTIONS(1723),
    [sym____reserved___] = ACTIONS(1723),
    [sym_date] = ACTIONS(1723),
    [sym_day] = ACTIONS(1723),
    [sym_time] = ACTIONS(1723),
    [sym_ldate] = ACTIONS(1723),
    [sym_push_device] = ACTIONS(1723),
    [sym_push_channel] = ACTIONS(1723),
    [sym_push_devchan] = ACTIONS(1723),
    [sym_release_device] = ACTIONS(1723),
    [sym_release_channel] = ACTIONS(1723),
    [sym_release_devchan] = ACTIONS(1723),
    [sym_master_slot] = ACTIONS(1723),
    [sym_get_pulse_time] = ACTIONS(1723),
    [sym_get_timer] = ACTIONS(1723),
    [sym_master_sn] = ACTIONS(1723),
    [sym_system_number] = ACTIONS(1723),
    [sym_button] = ACTIONS(1723),
    [sym_dv_channel] = ACTIONS(1723),
    [sym_data] = ACTIONS(1723),
    [sym_level] = ACTIONS(1723),
    [sym_timeline] = ACTIONS(1723),
    [sym_channel] = ACTIONS(1723),
    [sym_custom] = ACTIONS(1723),
    [sym_atoi] = ACTIONS(1723),
    [sym_atol] = ACTIONS(1723),
    [sym_atof] = ACTIONS(1723),
    [sym_itoa] = ACTIONS(1723),
    [sym_itohex] = ACTIONS(1723),
    [sym_hextoi] = ACTIONS(1723),
    [sym_ftoa] = ACTIONS(1723),
    [sym_find_string] = ACTIONS(1723),
    [sym_left_string] = ACTIONS(1723),
    [sym_right_string] = ACTIONS(1723),
    [sym_mid_string] = ACTIONS(1723),
    [sym_remove_string] = ACTIONS(1723),
    [sym_get_buffer_string] = ACTIONS(1723),
    [sym_get_buffer_char] = ACTIONS(1723),
    [sym_get_multi_buffer_string] = ACTIONS(1723),
    [sym_length_string] = ACTIONS(1723),
    [sym_max_length_string] = ACTIONS(1723),
    [sym_set_length_string] = ACTIONS(1723),
    [sym_compare_string] = ACTIONS(1723),
    [sym_upper_string] = ACTIONS(1723),
    [sym_lower_string] = ACTIONS(1723),
    [sym_get_last] = ACTIONS(1723),
    [sym_date_to_day] = ACTIONS(1723),
    [sym_date_to_month] = ACTIONS(1723),
    [sym_date_to_year] = ACTIONS(1723),
    [sym_time_to_hour] = ACTIONS(1723),
    [sym_time_to_minute] = ACTIONS(1723),
    [sym_time_to_second] = ACTIONS(1723),
    [sym_day_of_week] = ACTIONS(1723),
    [sym_set_pulse_time] = ACTIONS(1723),
    [sym_set_timer] = ACTIONS(1723),
    [sym_file_open] = ACTIONS(1723),
    [sym_file_close] = ACTIONS(1723),
    [sym_file_read] = ACTIONS(1723),
    [sym_file_read_line] = ACTIONS(1723),
    [sym_file_write] = ACTIONS(1723),
    [sym_file_write_line] = ACTIONS(1723),
    [sym_file_copy] = ACTIONS(1723),
    [sym_file_delete] = ACTIONS(1723),
    [sym_file_rename] = ACTIONS(1723),
    [sym_file_dir] = ACTIONS(1723),
    [sym_file_getdir] = ACTIONS(1723),
    [sym_file_setdir] = ACTIONS(1723),
    [sym_file_create_dir] = ACTIONS(1723),
    [sym_file_remove_dir] = ACTIONS(1723),
    [sym_file_seek] = ACTIONS(1723),
    [sym_length_array] = ACTIONS(1723),
    [sym_max_length_array] = ACTIONS(1723),
    [sym_set_length_array] = ACTIONS(1723),
    [sym_ip_client_open] = ACTIONS(1723),
    [sym_ip_client_close] = ACTIONS(1723),
    [sym_ip_server_open] = ACTIONS(1723),
    [sym_ip_server_close] = ACTIONS(1723),
    [sym_ip_bound_client_open] = ACTIONS(1723),
    [sym_ip_mc_server_open] = ACTIONS(1723),
    [sym_ip_set_option] = ACTIONS(1723),
    [sym_ssh_client_open] = ACTIONS(1723),
    [sym_ssh_client_close] = ACTIONS(1723),
    [sym_tls_client_open] = ACTIONS(1723),
    [sym_tls_client_close] = ACTIONS(1723),
    [sym_do_push] = ACTIONS(1723),
    [sym_do_push_timed] = ACTIONS(1723),
    [sym_do_release] = ACTIONS(1723),
    [sym_device_id] = ACTIONS(1723),
    [sym_device_info] = ACTIONS(1723),
    [sym_device_id_string] = ACTIONS(1723),
    [sym_device_standby] = ACTIONS(1723),
    [sym_device_wake] = ACTIONS(1723),
    [sym_timeline_create] = ACTIONS(1723),
    [sym_timeline_kill] = ACTIONS(1723),
    [sym_timeline_reload] = ACTIONS(1723),
    [sym_timeline_pause] = ACTIONS(1723),
    [sym_timeline_restart] = ACTIONS(1723),
    [sym_timeline_set] = ACTIONS(1723),
    [sym_timeline_get] = ACTIONS(1723),
    [sym_timeline_active] = ACTIONS(1723),
    [sym_timeline_dynamic_id] = ACTIONS(1723),
    [sym_variable_to_string] = ACTIONS(1723),
    [sym_string_to_variable] = ACTIONS(1723),
    [sym_length_variable_to_string] = ACTIONS(1723),
    [sym_type_cast] = ACTIONS(1723),
    [sym_variable_to_xml] = ACTIONS(1723),
    [sym_xml_to_variable] = ACTIONS(1723),
    [sym_length_variable_to_xml] = ACTIONS(1723),
    [sym_redirect_string] = ACTIONS(1723),
    [sym_random_number] = ACTIONS(1723),
    [sym_get_unique_id] = ACTIONS(1723),
    [sym_get_system_number] = ACTIONS(1723),
    [sym_set_system_number] = ACTIONS(1723),
    [sym_get_serial_number] = ACTIONS(1723),
    [sym_get_master_build] = ACTIONS(1723),
    [sym_set_validation_code] = ACTIONS(1723),
    [sym_reboot] = ACTIONS(1723),
    [sym_load_duet_module] = ACTIONS(1723),
    [sym_unload_duet_module] = ACTIONS(1723),
    [sym_duet_mem_size_set] = ACTIONS(1723),
    [sym_duet_mem_size_get] = ACTIONS(1723),
    [sym_dynamic_application_device] = ACTIONS(1723),
    [sym_dynamic_polled_port] = ACTIONS(1723),
    [sym_static_port_binding] = ACTIONS(1723),
    [sym_static_ip_binding] = ACTIONS(1723),
    [sym_get_dns_list] = ACTIONS(1723),
    [sym_set_dns_list] = ACTIONS(1723),
    [sym_get_ip_address] = ACTIONS(1723),
    [sym_set_ip_address] = ACTIONS(1723),
    [sym_get_url_list] = ACTIONS(1723),
    [sym_add_url_entry] = ACTIONS(1723),
    [sym_delete_url_entry] = ACTIONS(1723),
    [sym_abs_value] = ACTIONS(1723),
    [sym_max_value] = ACTIONS(1723),
    [sym_min_value] = ACTIONS(1723),
    [sym_exp_value] = ACTIONS(1723),
    [sym_log_value] = ACTIONS(1723),
    [sym_log10_value] = ACTIONS(1723),
    [sym_power_value] = ACTIONS(1723),
    [sym_sqrt_value] = ACTIONS(1723),
    [sym_raw_be] = ACTIONS(1723),
    [sym_raw_le] = ACTIONS(1723),
    [sym_internal_queue_size_set] = ACTIONS(1723),
    [sym_internal_queue_size_get] = ACTIONS(1723),
    [sym_internal_threshold_set] = ACTIONS(1723),
    [sym_internal_threshold_get] = ACTIONS(1723),
    [sym_rebuilt_event] = ACTIONS(1723),
    [sym_do_custom_event] = ACTIONS(1723),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1723),
    [sym_clkmgr_set_clk_source] = ACTIONS(1723),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1723),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1723),
    [sym_clkmgr_get_timezone] = ACTIONS(1723),
    [sym_clkmgr_set_timezone] = ACTIONS(1723),
    [sym_clkmgr_get_resync_period] = ACTIONS(1723),
    [sym_clkmgr_set_resync_period] = ACTIONS(1723),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1723),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1723),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1723),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1723),
    [sym_clkmgr_get_timeservers] = ACTIONS(1723),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1723),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1723),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1723),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1723),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1723),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1723),
    [sym_set_log_level] = ACTIONS(1723),
    [sym_get_log_level] = ACTIONS(1723),
    [sym_amx_log] = ACTIONS(1723),
    [sym_smtp_server_config_set] = ACTIONS(1723),
    [sym_smtp_server_config_get] = ACTIONS(1723),
    [sym_smtp_send] = ACTIONS(1723),
    [sym_validate_netlinx_account] = ACTIONS(1723),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1723),
    [sym_authenticate_certificate] = ACTIONS(1723),
    [sym_audit_netlinx_session_event] = ACTIONS(1723),
    [sym_audit_netlinx_generic_event] = ACTIONS(1723),
    [sym_get_available_flash_disk_space] = ACTIONS(1723),
    [sym_get_max_flash_disk_space] = ACTIONS(1723),
    [sym_data_create_feed] = ACTIONS(1723),
    [sym_data_delete_feed] = ACTIONS(1723),
    [sym_data_publish_feed] = ACTIONS(1723),
    [sym_data_get_published_feed] = ACTIONS(1723),
    [sym_data_add_record] = ACTIONS(1723),
    [sym_data_get_event_record] = ACTIONS(1723),
    [sym__wc_data_create_feed] = ACTIONS(1723),
    [sym__wc_data_add_record] = ACTIONS(1723),
    [sym__wc_data_get_event_record] = ACTIONS(1723),
    [sym_format] = ACTIONS(1723),
    [sym_astro_clock] = ACTIONS(1723),
    [sym_set_outdoor_temperature] = ACTIONS(1723),
    [sym_set_virtual_level_count] = ACTIONS(1723),
    [sym_set_virtual_channel_count] = ACTIONS(1723),
    [sym_set_virtual_port_count] = ACTIONS(1723),
    [sym__wc_encode_int] = ACTIONS(1723),
    [sym__wc_decode_int] = ACTIONS(1723),
    [sym__wc_int] = ACTIONS(1723),
    [sym__wc_to_ch_int] = ACTIONS(1723),
    [sym__ch_to_wc_int] = ACTIONS(1723),
    [sym__wc_find_string_int] = ACTIONS(1723),
    [sym__wc_left_string_int] = ACTIONS(1723),
    [sym__wc_lower_string_int] = ACTIONS(1723),
    [sym__wc_mid_string_int] = ACTIONS(1723),
    [sym__wc_remove_string_int] = ACTIONS(1723),
    [sym__wc_right_string_int] = ACTIONS(1723),
    [sym__wc_upper_string_int] = ACTIONS(1723),
    [sym__wc_compare_string_int] = ACTIONS(1723),
    [sym__wc_get_buffer_char_int] = ACTIONS(1723),
    [sym__wc_get_buffer_string_int] = ACTIONS(1723),
    [sym__wc_concat_string_int] = ACTIONS(1723),
    [sym__wc_file_open_int] = ACTIONS(1723),
    [sym__wc_file_close_int] = ACTIONS(1723),
    [sym__wc_file_read_int] = ACTIONS(1723),
    [sym__wc_file_read_line_int] = ACTIONS(1723),
    [sym__wc_file_write_int] = ACTIONS(1723),
    [sym__wc_file_write_line_int] = ACTIONS(1723),
    [aux_sym_preproc_include_token1] = ACTIONS(1723),
    [aux_sym_preproc_define_token1] = ACTIONS(1725),
    [aux_sym_preproc_warn_token1] = ACTIONS(1725),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1725),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1725),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1725),
    [anon_sym_LBRACK] = ACTIONS(1725),
    [anon_sym_LBRACE] = ACTIONS(1725),
    [anon_sym_RBRACE] = ACTIONS(1725),
    [anon_sym_LPAREN] = ACTIONS(1723),
    [anon_sym_BANG] = ACTIONS(1725),
    [anon_sym_TILDE] = ACTIONS(1725),
    [anon_sym_DASH] = ACTIONS(1723),
    [anon_sym_PLUS] = ACTIONS(1723),
    [anon_sym_DASH_DASH] = ACTIONS(1725),
    [anon_sym_PLUS_PLUS] = ACTIONS(1725),
    [anon_sym_DQUOTE] = ACTIONS(1725),
    [anon_sym_SQUOTE] = ACTIONS(1725),
    [sym_number_literal] = ACTIONS(1725),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(491)] = {
    [sym_identifier] = ACTIONS(1755),
    [sym_define_call_keyword] = ACTIONS(1755),
    [sym_define_function_keyword] = ACTIONS(1755),
    [sym_define_library_function_keyword] = ACTIONS(1755),
    [sym_define_module_keyword] = ACTIONS(1755),
    [sym_char_keyword] = ACTIONS(1755),
    [sym_widechar_keyword] = ACTIONS(1755),
    [sym_integer_keyword] = ACTIONS(1755),
    [sym_sinteger_keyword] = ACTIONS(1755),
    [sym_long_keyword] = ACTIONS(1755),
    [sym_slong_keyword] = ACTIONS(1755),
    [sym_float_keyword] = ACTIONS(1755),
    [sym_double_keyword] = ACTIONS(1755),
    [sym_variant_keyword] = ACTIONS(1755),
    [sym_variantarray_keyword] = ACTIONS(1755),
    [sym_local_var_keyword] = ACTIONS(1755),
    [sym_stack_var_keyword] = ACTIONS(1755),
    [sym_constant_keyword] = ACTIONS(1755),
    [sym_volatile_keyword] = ACTIONS(1755),
    [sym_non_volatile_keyword] = ACTIONS(1755),
    [sym_persistent_keyword] = ACTIONS(1755),
    [sym_dev_keyword] = ACTIONS(1755),
    [sym_devlev_keyword] = ACTIONS(1755),
    [sym_devchan_keyword] = ACTIONS(1755),
    [sym_if_keyword] = ACTIONS(1755),
    [sym_else_keyword] = ACTIONS(1755),
    [sym_switch_keyword] = ACTIONS(1755),
    [sym_case_keyword] = ACTIONS(1755),
    [sym_default_keyword] = ACTIONS(1755),
    [sym_while_keyword] = ACTIONS(1755),
    [sym_for_keyword] = ACTIONS(1755),
    [sym_break_keyword] = ACTIONS(1755),
    [sym_continue_keyword] = ACTIONS(1755),
    [sym_return_keyword] = ACTIONS(1755),
    [sym_select_keyword] = ACTIONS(1755),
    [sym_active_keyword] = ACTIONS(1755),
    [sym_struct_keyword] = ACTIONS(1755),
    [sym_structure_keyword] = ACTIONS(1755),
    [aux_sym_on_keyword_token1] = ACTIONS(1755),
    [aux_sym_off_keyword_token1] = ACTIONS(1755),
    [sym_send_level_keyword] = ACTIONS(1755),
    [sym_send_string_keyword] = ACTIONS(1755),
    [sym_send_command_keyword] = ACTIONS(1755),
    [sym_clear_buffer_keyword] = ACTIONS(1755),
    [sym_create_buffer_keyword] = ACTIONS(1755),
    [sym_create_multi_buffer_keyword] = ACTIONS(1755),
    [sym_call_keyword] = ACTIONS(1755),
    [sym_system_call_keyword] = ACTIONS(1755),
    [sym_devchan_to_keyword] = ACTIONS(1755),
    [sym_devchan_min_to_keyword] = ACTIONS(1755),
    [sym_devchan_total_off_keyword] = ACTIONS(1755),
    [sym_devchan_pulse_keyword] = ACTIONS(1755),
    [sym_wait_keyword] = ACTIONS(1755),
    [sym_cancel_wait_keyword] = ACTIONS(1755),
    [sym_wait_until_keyword] = ACTIONS(1755),
    [sym_cancel_wait_until_keyword] = ACTIONS(1755),
    [sym_cancel_all_wait_keyword] = ACTIONS(1755),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1755),
    [sym_not] = ACTIONS(1755),
    [sym_bnot] = ACTIONS(1755),
    [sym___file__] = ACTIONS(1755),
    [sym___date__] = ACTIONS(1755),
    [sym___ldate__] = ACTIONS(1755),
    [sym___time__] = ACTIONS(1755),
    [sym___line__] = ACTIONS(1755),
    [sym___name__] = ACTIONS(1755),
    [sym___netlinx__] = ACTIONS(1755),
    [sym_netlinx_axi_version] = ACTIONS(1755),
    [sym_url_flg_tcp] = ACTIONS(1755),
    [sym_url_flg_accinfopresent] = ACTIONS(1755),
    [sym_url_flg_temp] = ACTIONS(1755),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1755),
    [sym_url_flg_stat_mask] = ACTIONS(1755),
    [sym_url_flg_stat_lookup] = ACTIONS(1755),
    [sym_url_flg_stat_connecting] = ACTIONS(1755),
    [sym_url_flg_stat_waiting] = ACTIONS(1755),
    [sym_url_flg_stat_connected] = ACTIONS(1755),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1755),
    [sym_first_virtual_device] = ACTIONS(1755),
    [sym_first_local_port] = ACTIONS(1755),
    [sym_dynamic_virtual_device] = ACTIONS(1755),
    [sym_timeline_once] = ACTIONS(1755),
    [sym_timeline_repeat] = ACTIONS(1755),
    [sym_timeline_absolute] = ACTIONS(1755),
    [sym_timeline_relative] = ACTIONS(1755),
    [sym_do_push_timed_infinite] = ACTIONS(1755),
    [sym_source_type_no_address] = ACTIONS(1755),
    [sym_source_type_neuron_id] = ACTIONS(1755),
    [sym_source_type_ip_address] = ACTIONS(1755),
    [sym_source_type_axlink] = ACTIONS(1755),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1755),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1755),
    [sym_source_type_ip_socket_address] = ACTIONS(1755),
    [sym_source_type_rs232] = ACTIONS(1755),
    [sym_source_type_internal] = ACTIONS(1755),
    [sym_source_type_mac_address] = ACTIONS(1755),
    [sym_source_type_ipv4_port] = ACTIONS(1755),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1755),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1755),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1755),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1755),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1755),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1755),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1755),
    [sym_true] = ACTIONS(1755),
    [sym_false] = ACTIONS(1755),
    [sym_file_read_only] = ACTIONS(1755),
    [sym_file_rw_new] = ACTIONS(1755),
    [sym_file_rw_append] = ACTIONS(1755),
    [sym_ip_tcp] = ACTIONS(1755),
    [sym_ip_udp] = ACTIONS(1755),
    [sym_ip_udp_2way] = ACTIONS(1755),
    [sym_xml_encode_types] = ACTIONS(1755),
    [sym_xml_encode_char_as_list] = ACTIONS(1755),
    [sym_xml_encode_le] = ACTIONS(1755),
    [sym_xml_decode_types] = ACTIONS(1755),
    [sym_xml_decode_no_preserve] = ACTIONS(1755),
    [sym_duet_dev_type_display_device] = ACTIONS(1755),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1755),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1755),
    [sym_duet_dev_type_io_device] = ACTIONS(1755),
    [sym_duet_dev_type_relay_device] = ACTIONS(1755),
    [sym_duet_dev_type_ups] = ACTIONS(1755),
    [sym_duet_dev_type_amplifier] = ACTIONS(1755),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1755),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1755),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1755),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1755),
    [sym_duet_dev_type_camera] = ACTIONS(1755),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1755),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1755),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1755),
    [sym_duet_dev_type_dss] = ACTIONS(1755),
    [sym_duet_dev_type_dvr] = ACTIONS(1755),
    [sym_duet_dev_type_disc_device] = ACTIONS(1755),
    [sym_duet_dev_type_document_camera] = ACTIONS(1755),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1755),
    [sym_duet_dev_type_hvac] = ACTIONS(1755),
    [sym_duet_dev_type_keypad] = ACTIONS(1755),
    [sym_duet_dev_type_light] = ACTIONS(1755),
    [sym_duet_dev_type_monitor] = ACTIONS(1755),
    [sym_duet_dev_type_motor] = ACTIONS(1755),
    [sym_duet_dev_type_multi_window] = ACTIONS(1755),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1755),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1755),
    [sym_duet_dev_type_receiver] = ACTIONS(1755),
    [sym_duet_dev_type_security_system] = ACTIONS(1755),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1755),
    [sym_duet_dev_type_settop_box] = ACTIONS(1755),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1755),
    [sym_duet_dev_type_switcher] = ACTIONS(1755),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1755),
    [sym_duet_dev_type_tv] = ACTIONS(1755),
    [sym_duet_dev_type_utility] = ACTIONS(1755),
    [sym_duet_dev_type_vcr] = ACTIONS(1755),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1755),
    [sym_duet_dev_type_video_processor] = ACTIONS(1755),
    [sym_duet_dev_type_video_projector] = ACTIONS(1755),
    [sym_duet_dev_type_video_wall] = ACTIONS(1755),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1755),
    [sym_duet_dev_type_weather] = ACTIONS(1755),
    [sym_duet_dev_not_polled] = ACTIONS(1755),
    [sym_duet_dev_polled] = ACTIONS(1755),
    [sym_clkmgr_mode_network] = ACTIONS(1755),
    [sym_clkmgr_mode_standalone] = ACTIONS(1755),
    [sym_normal_standby] = ACTIONS(1755),
    [sym_normal_wake] = ACTIONS(1755),
    [sym_ip_multicast_ttl_option] = ACTIONS(1755),
    [sym_ip_ttl_subnet] = ACTIONS(1755),
    [sym_ip_ttl_site] = ACTIONS(1755),
    [sym_ip_ttl_region] = ACTIONS(1755),
    [sym_ip_ttl_continent] = ACTIONS(1755),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1755),
    [sym_ip_nodelay_on] = ACTIONS(1755),
    [sym_ip_nodelay_off] = ACTIONS(1755),
    [sym_amx_error] = ACTIONS(1755),
    [sym_amx_warning] = ACTIONS(1755),
    [sym_amx_info] = ACTIONS(1755),
    [sym_amx_debug] = ACTIONS(1755),
    [sym_netlinx_login_success] = ACTIONS(1755),
    [sym_netlinx_login_fail] = ACTIONS(1755),
    [sym_netlinx_logout] = ACTIONS(1755),
    [sym_null_str] = ACTIONS(1755),
    [sym_smtp_address] = ACTIONS(1755),
    [sym_smtp_port_number] = ACTIONS(1755),
    [sym_smtp_username] = ACTIONS(1755),
    [sym_smtp_password] = ACTIONS(1755),
    [sym_smtp_require_tls] = ACTIONS(1755),
    [sym_smtp_from] = ACTIONS(1755),
    [sym_smtp_tls_true] = ACTIONS(1755),
    [sym_smtp_tls_false] = ACTIONS(1755),
    [sym_listview_on_row_select_event] = ACTIONS(1755),
    [sym_data_structure_unknown] = ACTIONS(1755),
    [sym_data_structure_datafeed] = ACTIONS(1755),
    [sym_data_structure_datarecordset] = ACTIONS(1755),
    [sym_data_structure_datarecord] = ACTIONS(1755),
    [sym_data_structure_datafield] = ACTIONS(1755),
    [sym_data_max_content_field_count] = ACTIONS(1755),
    [sym_data_max_metadata_field_count] = ACTIONS(1755),
    [sym_data_max_id_length] = ACTIONS(1755),
    [sym_data_max_name_length] = ACTIONS(1755),
    [sym_data_max_description_length] = ACTIONS(1755),
    [sym_data_max_type_length] = ACTIONS(1755),
    [sym_data_max_format_length] = ACTIONS(1755),
    [sym_data_max_source_length] = ACTIONS(1755),
    [sym_data_max_value_length] = ACTIONS(1755),
    [sym_data_max_label_length] = ACTIONS(1755),
    [sym_data_max_url_length] = ACTIONS(1755),
    [sym_max_last_login_info_length] = ACTIONS(1755),
    [sym_data_type_unknown] = ACTIONS(1755),
    [sym_data_type_string] = ACTIONS(1755),
    [sym_data_type_datetime] = ACTIONS(1755),
    [sym_data_type_date] = ACTIONS(1755),
    [sym_data_type_time] = ACTIONS(1755),
    [sym_data_type_image] = ACTIONS(1755),
    [sym_data_format_url] = ACTIONS(1755),
    [sym_data_format_phone] = ACTIONS(1755),
    [sym_data_format_email] = ACTIONS(1755),
    [sym_data_format_iso8601] = ACTIONS(1755),
    [sym_auditlog_priv] = ACTIONS(1755),
    [sym_configuration_priv] = ACTIONS(1755),
    [sym_deviceconfig_priv] = ACTIONS(1755),
    [sym_ftp_priv] = ACTIONS(1755),
    [sym_http_priv] = ACTIONS(1755),
    [sym_networkconfig_priv] = ACTIONS(1755),
    [sym_programport_priv] = ACTIONS(1755),
    [sym_remoteui_priv] = ACTIONS(1755),
    [sym_securitycontrol_priv] = ACTIONS(1755),
    [sym_softwaremanagement_priv] = ACTIONS(1755),
    [sym_terminal_priv] = ACTIONS(1755),
    [sym_tpadmin_priv] = ACTIONS(1755),
    [sym_user1_priv] = ACTIONS(1755),
    [sym_user2_priv] = ACTIONS(1755),
    [sym_user3_priv] = ACTIONS(1755),
    [sym_user4_priv] = ACTIONS(1755),
    [sym_usermanagement_priv] = ACTIONS(1755),
    [sym_valid_account] = ACTIONS(1755),
    [sym_err_name_invalid] = ACTIONS(1755),
    [sym_err_password_invalid] = ACTIONS(1755),
    [sym_err_invalid_account] = ACTIONS(1755),
    [sym_err_no_authorization] = ACTIONS(1755),
    [sym_err_invalid_parameter] = ACTIONS(1755),
    [sym_err_user_locked_out] = ACTIONS(1755),
    [sym_err_user_account_expired] = ACTIONS(1755),
    [sym_err_general_error] = ACTIONS(1755),
    [sym_err_authentication_error] = ACTIONS(1755),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1755),
    [sym_tls_validate_certificate] = ACTIONS(1755),
    [sym_tdata] = ACTIONS(1755),
    [sym_tchannel] = ACTIONS(1755),
    [sym_tlevel] = ACTIONS(1755),
    [sym_tbutton] = ACTIONS(1755),
    [sym_ttimeline] = ACTIONS(1755),
    [sym_tcustom] = ACTIONS(1755),
    [sym_url_struct] = ACTIONS(1755),
    [sym_dns_struct] = ACTIONS(1755),
    [sym_ip_address_struct] = ACTIONS(1755),
    [sym_dev_info_struct] = ACTIONS(1755),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1755),
    [sym_data_feed] = ACTIONS(1755),
    [sym_data_field] = ACTIONS(1755),
    [sym_data_record] = ACTIONS(1755),
    [sym_wc_data_feed] = ACTIONS(1755),
    [sym_wc_data_field] = ACTIONS(1755),
    [sym_wc_data_record] = ACTIONS(1755),
    [sym_last_login_info] = ACTIONS(1755),
    [sym____reserved___] = ACTIONS(1755),
    [sym_date] = ACTIONS(1755),
    [sym_day] = ACTIONS(1755),
    [sym_time] = ACTIONS(1755),
    [sym_ldate] = ACTIONS(1755),
    [sym_push_device] = ACTIONS(1755),
    [sym_push_channel] = ACTIONS(1755),
    [sym_push_devchan] = ACTIONS(1755),
    [sym_release_device] = ACTIONS(1755),
    [sym_release_channel] = ACTIONS(1755),
    [sym_release_devchan] = ACTIONS(1755),
    [sym_master_slot] = ACTIONS(1755),
    [sym_get_pulse_time] = ACTIONS(1755),
    [sym_get_timer] = ACTIONS(1755),
    [sym_master_sn] = ACTIONS(1755),
    [sym_system_number] = ACTIONS(1755),
    [sym_button] = ACTIONS(1755),
    [sym_dv_channel] = ACTIONS(1755),
    [sym_data] = ACTIONS(1755),
    [sym_level] = ACTIONS(1755),
    [sym_timeline] = ACTIONS(1755),
    [sym_channel] = ACTIONS(1755),
    [sym_custom] = ACTIONS(1755),
    [sym_atoi] = ACTIONS(1755),
    [sym_atol] = ACTIONS(1755),
    [sym_atof] = ACTIONS(1755),
    [sym_itoa] = ACTIONS(1755),
    [sym_itohex] = ACTIONS(1755),
    [sym_hextoi] = ACTIONS(1755),
    [sym_ftoa] = ACTIONS(1755),
    [sym_find_string] = ACTIONS(1755),
    [sym_left_string] = ACTIONS(1755),
    [sym_right_string] = ACTIONS(1755),
    [sym_mid_string] = ACTIONS(1755),
    [sym_remove_string] = ACTIONS(1755),
    [sym_get_buffer_string] = ACTIONS(1755),
    [sym_get_buffer_char] = ACTIONS(1755),
    [sym_get_multi_buffer_string] = ACTIONS(1755),
    [sym_length_string] = ACTIONS(1755),
    [sym_max_length_string] = ACTIONS(1755),
    [sym_set_length_string] = ACTIONS(1755),
    [sym_compare_string] = ACTIONS(1755),
    [sym_upper_string] = ACTIONS(1755),
    [sym_lower_string] = ACTIONS(1755),
    [sym_get_last] = ACTIONS(1755),
    [sym_date_to_day] = ACTIONS(1755),
    [sym_date_to_month] = ACTIONS(1755),
    [sym_date_to_year] = ACTIONS(1755),
    [sym_time_to_hour] = ACTIONS(1755),
    [sym_time_to_minute] = ACTIONS(1755),
    [sym_time_to_second] = ACTIONS(1755),
    [sym_day_of_week] = ACTIONS(1755),
    [sym_set_pulse_time] = ACTIONS(1755),
    [sym_set_timer] = ACTIONS(1755),
    [sym_file_open] = ACTIONS(1755),
    [sym_file_close] = ACTIONS(1755),
    [sym_file_read] = ACTIONS(1755),
    [sym_file_read_line] = ACTIONS(1755),
    [sym_file_write] = ACTIONS(1755),
    [sym_file_write_line] = ACTIONS(1755),
    [sym_file_copy] = ACTIONS(1755),
    [sym_file_delete] = ACTIONS(1755),
    [sym_file_rename] = ACTIONS(1755),
    [sym_file_dir] = ACTIONS(1755),
    [sym_file_getdir] = ACTIONS(1755),
    [sym_file_setdir] = ACTIONS(1755),
    [sym_file_create_dir] = ACTIONS(1755),
    [sym_file_remove_dir] = ACTIONS(1755),
    [sym_file_seek] = ACTIONS(1755),
    [sym_length_array] = ACTIONS(1755),
    [sym_max_length_array] = ACTIONS(1755),
    [sym_set_length_array] = ACTIONS(1755),
    [sym_ip_client_open] = ACTIONS(1755),
    [sym_ip_client_close] = ACTIONS(1755),
    [sym_ip_server_open] = ACTIONS(1755),
    [sym_ip_server_close] = ACTIONS(1755),
    [sym_ip_bound_client_open] = ACTIONS(1755),
    [sym_ip_mc_server_open] = ACTIONS(1755),
    [sym_ip_set_option] = ACTIONS(1755),
    [sym_ssh_client_open] = ACTIONS(1755),
    [sym_ssh_client_close] = ACTIONS(1755),
    [sym_tls_client_open] = ACTIONS(1755),
    [sym_tls_client_close] = ACTIONS(1755),
    [sym_do_push] = ACTIONS(1755),
    [sym_do_push_timed] = ACTIONS(1755),
    [sym_do_release] = ACTIONS(1755),
    [sym_device_id] = ACTIONS(1755),
    [sym_device_info] = ACTIONS(1755),
    [sym_device_id_string] = ACTIONS(1755),
    [sym_device_standby] = ACTIONS(1755),
    [sym_device_wake] = ACTIONS(1755),
    [sym_timeline_create] = ACTIONS(1755),
    [sym_timeline_kill] = ACTIONS(1755),
    [sym_timeline_reload] = ACTIONS(1755),
    [sym_timeline_pause] = ACTIONS(1755),
    [sym_timeline_restart] = ACTIONS(1755),
    [sym_timeline_set] = ACTIONS(1755),
    [sym_timeline_get] = ACTIONS(1755),
    [sym_timeline_active] = ACTIONS(1755),
    [sym_timeline_dynamic_id] = ACTIONS(1755),
    [sym_variable_to_string] = ACTIONS(1755),
    [sym_string_to_variable] = ACTIONS(1755),
    [sym_length_variable_to_string] = ACTIONS(1755),
    [sym_type_cast] = ACTIONS(1755),
    [sym_variable_to_xml] = ACTIONS(1755),
    [sym_xml_to_variable] = ACTIONS(1755),
    [sym_length_variable_to_xml] = ACTIONS(1755),
    [sym_redirect_string] = ACTIONS(1755),
    [sym_random_number] = ACTIONS(1755),
    [sym_get_unique_id] = ACTIONS(1755),
    [sym_get_system_number] = ACTIONS(1755),
    [sym_set_system_number] = ACTIONS(1755),
    [sym_get_serial_number] = ACTIONS(1755),
    [sym_get_master_build] = ACTIONS(1755),
    [sym_set_validation_code] = ACTIONS(1755),
    [sym_reboot] = ACTIONS(1755),
    [sym_load_duet_module] = ACTIONS(1755),
    [sym_unload_duet_module] = ACTIONS(1755),
    [sym_duet_mem_size_set] = ACTIONS(1755),
    [sym_duet_mem_size_get] = ACTIONS(1755),
    [sym_dynamic_application_device] = ACTIONS(1755),
    [sym_dynamic_polled_port] = ACTIONS(1755),
    [sym_static_port_binding] = ACTIONS(1755),
    [sym_static_ip_binding] = ACTIONS(1755),
    [sym_get_dns_list] = ACTIONS(1755),
    [sym_set_dns_list] = ACTIONS(1755),
    [sym_get_ip_address] = ACTIONS(1755),
    [sym_set_ip_address] = ACTIONS(1755),
    [sym_get_url_list] = ACTIONS(1755),
    [sym_add_url_entry] = ACTIONS(1755),
    [sym_delete_url_entry] = ACTIONS(1755),
    [sym_abs_value] = ACTIONS(1755),
    [sym_max_value] = ACTIONS(1755),
    [sym_min_value] = ACTIONS(1755),
    [sym_exp_value] = ACTIONS(1755),
    [sym_log_value] = ACTIONS(1755),
    [sym_log10_value] = ACTIONS(1755),
    [sym_power_value] = ACTIONS(1755),
    [sym_sqrt_value] = ACTIONS(1755),
    [sym_raw_be] = ACTIONS(1755),
    [sym_raw_le] = ACTIONS(1755),
    [sym_internal_queue_size_set] = ACTIONS(1755),
    [sym_internal_queue_size_get] = ACTIONS(1755),
    [sym_internal_threshold_set] = ACTIONS(1755),
    [sym_internal_threshold_get] = ACTIONS(1755),
    [sym_rebuilt_event] = ACTIONS(1755),
    [sym_do_custom_event] = ACTIONS(1755),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1755),
    [sym_clkmgr_set_clk_source] = ACTIONS(1755),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1755),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1755),
    [sym_clkmgr_get_timezone] = ACTIONS(1755),
    [sym_clkmgr_set_timezone] = ACTIONS(1755),
    [sym_clkmgr_get_resync_period] = ACTIONS(1755),
    [sym_clkmgr_set_resync_period] = ACTIONS(1755),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1755),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1755),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1755),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1755),
    [sym_clkmgr_get_timeservers] = ACTIONS(1755),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1755),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1755),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1755),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1755),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1755),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1755),
    [sym_set_log_level] = ACTIONS(1755),
    [sym_get_log_level] = ACTIONS(1755),
    [sym_amx_log] = ACTIONS(1755),
    [sym_smtp_server_config_set] = ACTIONS(1755),
    [sym_smtp_server_config_get] = ACTIONS(1755),
    [sym_smtp_send] = ACTIONS(1755),
    [sym_validate_netlinx_account] = ACTIONS(1755),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1755),
    [sym_authenticate_certificate] = ACTIONS(1755),
    [sym_audit_netlinx_session_event] = ACTIONS(1755),
    [sym_audit_netlinx_generic_event] = ACTIONS(1755),
    [sym_get_available_flash_disk_space] = ACTIONS(1755),
    [sym_get_max_flash_disk_space] = ACTIONS(1755),
    [sym_data_create_feed] = ACTIONS(1755),
    [sym_data_delete_feed] = ACTIONS(1755),
    [sym_data_publish_feed] = ACTIONS(1755),
    [sym_data_get_published_feed] = ACTIONS(1755),
    [sym_data_add_record] = ACTIONS(1755),
    [sym_data_get_event_record] = ACTIONS(1755),
    [sym__wc_data_create_feed] = ACTIONS(1755),
    [sym__wc_data_add_record] = ACTIONS(1755),
    [sym__wc_data_get_event_record] = ACTIONS(1755),
    [sym_format] = ACTIONS(1755),
    [sym_astro_clock] = ACTIONS(1755),
    [sym_set_outdoor_temperature] = ACTIONS(1755),
    [sym_set_virtual_level_count] = ACTIONS(1755),
    [sym_set_virtual_channel_count] = ACTIONS(1755),
    [sym_set_virtual_port_count] = ACTIONS(1755),
    [sym__wc_encode_int] = ACTIONS(1755),
    [sym__wc_decode_int] = ACTIONS(1755),
    [sym__wc_int] = ACTIONS(1755),
    [sym__wc_to_ch_int] = ACTIONS(1755),
    [sym__ch_to_wc_int] = ACTIONS(1755),
    [sym__wc_find_string_int] = ACTIONS(1755),
    [sym__wc_left_string_int] = ACTIONS(1755),
    [sym__wc_lower_string_int] = ACTIONS(1755),
    [sym__wc_mid_string_int] = ACTIONS(1755),
    [sym__wc_remove_string_int] = ACTIONS(1755),
    [sym__wc_right_string_int] = ACTIONS(1755),
    [sym__wc_upper_string_int] = ACTIONS(1755),
    [sym__wc_compare_string_int] = ACTIONS(1755),
    [sym__wc_get_buffer_char_int] = ACTIONS(1755),
    [sym__wc_get_buffer_string_int] = ACTIONS(1755),
    [sym__wc_concat_string_int] = ACTIONS(1755),
    [sym__wc_file_open_int] = ACTIONS(1755),
    [sym__wc_file_close_int] = ACTIONS(1755),
    [sym__wc_file_read_int] = ACTIONS(1755),
    [sym__wc_file_read_line_int] = ACTIONS(1755),
    [sym__wc_file_write_int] = ACTIONS(1755),
    [sym__wc_file_write_line_int] = ACTIONS(1755),
    [aux_sym_preproc_include_token1] = ACTIONS(1755),
    [aux_sym_preproc_define_token1] = ACTIONS(1757),
    [aux_sym_preproc_warn_token1] = ACTIONS(1757),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1757),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1757),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1757),
    [anon_sym_LBRACK] = ACTIONS(1757),
    [anon_sym_LBRACE] = ACTIONS(1757),
    [anon_sym_RBRACE] = ACTIONS(1757),
    [anon_sym_LPAREN] = ACTIONS(1755),
    [anon_sym_BANG] = ACTIONS(1757),
    [anon_sym_TILDE] = ACTIONS(1757),
    [anon_sym_DASH] = ACTIONS(1755),
    [anon_sym_PLUS] = ACTIONS(1755),
    [anon_sym_DASH_DASH] = ACTIONS(1757),
    [anon_sym_PLUS_PLUS] = ACTIONS(1757),
    [anon_sym_DQUOTE] = ACTIONS(1757),
    [anon_sym_SQUOTE] = ACTIONS(1757),
    [sym_number_literal] = ACTIONS(1757),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(492)] = {
    [sym_identifier] = ACTIONS(1731),
    [sym_define_call_keyword] = ACTIONS(1731),
    [sym_define_function_keyword] = ACTIONS(1731),
    [sym_define_library_function_keyword] = ACTIONS(1731),
    [sym_define_module_keyword] = ACTIONS(1731),
    [sym_char_keyword] = ACTIONS(1731),
    [sym_widechar_keyword] = ACTIONS(1731),
    [sym_integer_keyword] = ACTIONS(1731),
    [sym_sinteger_keyword] = ACTIONS(1731),
    [sym_long_keyword] = ACTIONS(1731),
    [sym_slong_keyword] = ACTIONS(1731),
    [sym_float_keyword] = ACTIONS(1731),
    [sym_double_keyword] = ACTIONS(1731),
    [sym_variant_keyword] = ACTIONS(1731),
    [sym_variantarray_keyword] = ACTIONS(1731),
    [sym_local_var_keyword] = ACTIONS(1731),
    [sym_stack_var_keyword] = ACTIONS(1731),
    [sym_constant_keyword] = ACTIONS(1731),
    [sym_volatile_keyword] = ACTIONS(1731),
    [sym_non_volatile_keyword] = ACTIONS(1731),
    [sym_persistent_keyword] = ACTIONS(1731),
    [sym_dev_keyword] = ACTIONS(1731),
    [sym_devlev_keyword] = ACTIONS(1731),
    [sym_devchan_keyword] = ACTIONS(1731),
    [sym_if_keyword] = ACTIONS(1731),
    [sym_else_keyword] = ACTIONS(1731),
    [sym_switch_keyword] = ACTIONS(1731),
    [sym_case_keyword] = ACTIONS(1731),
    [sym_default_keyword] = ACTIONS(1731),
    [sym_while_keyword] = ACTIONS(1731),
    [sym_for_keyword] = ACTIONS(1731),
    [sym_break_keyword] = ACTIONS(1731),
    [sym_continue_keyword] = ACTIONS(1731),
    [sym_return_keyword] = ACTIONS(1731),
    [sym_select_keyword] = ACTIONS(1731),
    [sym_active_keyword] = ACTIONS(1731),
    [sym_struct_keyword] = ACTIONS(1731),
    [sym_structure_keyword] = ACTIONS(1731),
    [aux_sym_on_keyword_token1] = ACTIONS(1731),
    [aux_sym_off_keyword_token1] = ACTIONS(1731),
    [sym_send_level_keyword] = ACTIONS(1731),
    [sym_send_string_keyword] = ACTIONS(1731),
    [sym_send_command_keyword] = ACTIONS(1731),
    [sym_clear_buffer_keyword] = ACTIONS(1731),
    [sym_create_buffer_keyword] = ACTIONS(1731),
    [sym_create_multi_buffer_keyword] = ACTIONS(1731),
    [sym_call_keyword] = ACTIONS(1731),
    [sym_system_call_keyword] = ACTIONS(1731),
    [sym_devchan_to_keyword] = ACTIONS(1731),
    [sym_devchan_min_to_keyword] = ACTIONS(1731),
    [sym_devchan_total_off_keyword] = ACTIONS(1731),
    [sym_devchan_pulse_keyword] = ACTIONS(1731),
    [sym_wait_keyword] = ACTIONS(1731),
    [sym_cancel_wait_keyword] = ACTIONS(1731),
    [sym_wait_until_keyword] = ACTIONS(1731),
    [sym_cancel_wait_until_keyword] = ACTIONS(1731),
    [sym_cancel_all_wait_keyword] = ACTIONS(1731),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1731),
    [sym_not] = ACTIONS(1731),
    [sym_bnot] = ACTIONS(1731),
    [sym___file__] = ACTIONS(1731),
    [sym___date__] = ACTIONS(1731),
    [sym___ldate__] = ACTIONS(1731),
    [sym___time__] = ACTIONS(1731),
    [sym___line__] = ACTIONS(1731),
    [sym___name__] = ACTIONS(1731),
    [sym___netlinx__] = ACTIONS(1731),
    [sym_netlinx_axi_version] = ACTIONS(1731),
    [sym_url_flg_tcp] = ACTIONS(1731),
    [sym_url_flg_accinfopresent] = ACTIONS(1731),
    [sym_url_flg_temp] = ACTIONS(1731),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1731),
    [sym_url_flg_stat_mask] = ACTIONS(1731),
    [sym_url_flg_stat_lookup] = ACTIONS(1731),
    [sym_url_flg_stat_connecting] = ACTIONS(1731),
    [sym_url_flg_stat_waiting] = ACTIONS(1731),
    [sym_url_flg_stat_connected] = ACTIONS(1731),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1731),
    [sym_first_virtual_device] = ACTIONS(1731),
    [sym_first_local_port] = ACTIONS(1731),
    [sym_dynamic_virtual_device] = ACTIONS(1731),
    [sym_timeline_once] = ACTIONS(1731),
    [sym_timeline_repeat] = ACTIONS(1731),
    [sym_timeline_absolute] = ACTIONS(1731),
    [sym_timeline_relative] = ACTIONS(1731),
    [sym_do_push_timed_infinite] = ACTIONS(1731),
    [sym_source_type_no_address] = ACTIONS(1731),
    [sym_source_type_neuron_id] = ACTIONS(1731),
    [sym_source_type_ip_address] = ACTIONS(1731),
    [sym_source_type_axlink] = ACTIONS(1731),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1731),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1731),
    [sym_source_type_ip_socket_address] = ACTIONS(1731),
    [sym_source_type_rs232] = ACTIONS(1731),
    [sym_source_type_internal] = ACTIONS(1731),
    [sym_source_type_mac_address] = ACTIONS(1731),
    [sym_source_type_ipv4_port] = ACTIONS(1731),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1731),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1731),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1731),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1731),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1731),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1731),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1731),
    [sym_true] = ACTIONS(1731),
    [sym_false] = ACTIONS(1731),
    [sym_file_read_only] = ACTIONS(1731),
    [sym_file_rw_new] = ACTIONS(1731),
    [sym_file_rw_append] = ACTIONS(1731),
    [sym_ip_tcp] = ACTIONS(1731),
    [sym_ip_udp] = ACTIONS(1731),
    [sym_ip_udp_2way] = ACTIONS(1731),
    [sym_xml_encode_types] = ACTIONS(1731),
    [sym_xml_encode_char_as_list] = ACTIONS(1731),
    [sym_xml_encode_le] = ACTIONS(1731),
    [sym_xml_decode_types] = ACTIONS(1731),
    [sym_xml_decode_no_preserve] = ACTIONS(1731),
    [sym_duet_dev_type_display_device] = ACTIONS(1731),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1731),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1731),
    [sym_duet_dev_type_io_device] = ACTIONS(1731),
    [sym_duet_dev_type_relay_device] = ACTIONS(1731),
    [sym_duet_dev_type_ups] = ACTIONS(1731),
    [sym_duet_dev_type_amplifier] = ACTIONS(1731),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1731),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1731),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1731),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1731),
    [sym_duet_dev_type_camera] = ACTIONS(1731),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1731),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1731),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1731),
    [sym_duet_dev_type_dss] = ACTIONS(1731),
    [sym_duet_dev_type_dvr] = ACTIONS(1731),
    [sym_duet_dev_type_disc_device] = ACTIONS(1731),
    [sym_duet_dev_type_document_camera] = ACTIONS(1731),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1731),
    [sym_duet_dev_type_hvac] = ACTIONS(1731),
    [sym_duet_dev_type_keypad] = ACTIONS(1731),
    [sym_duet_dev_type_light] = ACTIONS(1731),
    [sym_duet_dev_type_monitor] = ACTIONS(1731),
    [sym_duet_dev_type_motor] = ACTIONS(1731),
    [sym_duet_dev_type_multi_window] = ACTIONS(1731),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1731),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1731),
    [sym_duet_dev_type_receiver] = ACTIONS(1731),
    [sym_duet_dev_type_security_system] = ACTIONS(1731),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1731),
    [sym_duet_dev_type_settop_box] = ACTIONS(1731),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1731),
    [sym_duet_dev_type_switcher] = ACTIONS(1731),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1731),
    [sym_duet_dev_type_tv] = ACTIONS(1731),
    [sym_duet_dev_type_utility] = ACTIONS(1731),
    [sym_duet_dev_type_vcr] = ACTIONS(1731),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1731),
    [sym_duet_dev_type_video_processor] = ACTIONS(1731),
    [sym_duet_dev_type_video_projector] = ACTIONS(1731),
    [sym_duet_dev_type_video_wall] = ACTIONS(1731),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1731),
    [sym_duet_dev_type_weather] = ACTIONS(1731),
    [sym_duet_dev_not_polled] = ACTIONS(1731),
    [sym_duet_dev_polled] = ACTIONS(1731),
    [sym_clkmgr_mode_network] = ACTIONS(1731),
    [sym_clkmgr_mode_standalone] = ACTIONS(1731),
    [sym_normal_standby] = ACTIONS(1731),
    [sym_normal_wake] = ACTIONS(1731),
    [sym_ip_multicast_ttl_option] = ACTIONS(1731),
    [sym_ip_ttl_subnet] = ACTIONS(1731),
    [sym_ip_ttl_site] = ACTIONS(1731),
    [sym_ip_ttl_region] = ACTIONS(1731),
    [sym_ip_ttl_continent] = ACTIONS(1731),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1731),
    [sym_ip_nodelay_on] = ACTIONS(1731),
    [sym_ip_nodelay_off] = ACTIONS(1731),
    [sym_amx_error] = ACTIONS(1731),
    [sym_amx_warning] = ACTIONS(1731),
    [sym_amx_info] = ACTIONS(1731),
    [sym_amx_debug] = ACTIONS(1731),
    [sym_netlinx_login_success] = ACTIONS(1731),
    [sym_netlinx_login_fail] = ACTIONS(1731),
    [sym_netlinx_logout] = ACTIONS(1731),
    [sym_null_str] = ACTIONS(1731),
    [sym_smtp_address] = ACTIONS(1731),
    [sym_smtp_port_number] = ACTIONS(1731),
    [sym_smtp_username] = ACTIONS(1731),
    [sym_smtp_password] = ACTIONS(1731),
    [sym_smtp_require_tls] = ACTIONS(1731),
    [sym_smtp_from] = ACTIONS(1731),
    [sym_smtp_tls_true] = ACTIONS(1731),
    [sym_smtp_tls_false] = ACTIONS(1731),
    [sym_listview_on_row_select_event] = ACTIONS(1731),
    [sym_data_structure_unknown] = ACTIONS(1731),
    [sym_data_structure_datafeed] = ACTIONS(1731),
    [sym_data_structure_datarecordset] = ACTIONS(1731),
    [sym_data_structure_datarecord] = ACTIONS(1731),
    [sym_data_structure_datafield] = ACTIONS(1731),
    [sym_data_max_content_field_count] = ACTIONS(1731),
    [sym_data_max_metadata_field_count] = ACTIONS(1731),
    [sym_data_max_id_length] = ACTIONS(1731),
    [sym_data_max_name_length] = ACTIONS(1731),
    [sym_data_max_description_length] = ACTIONS(1731),
    [sym_data_max_type_length] = ACTIONS(1731),
    [sym_data_max_format_length] = ACTIONS(1731),
    [sym_data_max_source_length] = ACTIONS(1731),
    [sym_data_max_value_length] = ACTIONS(1731),
    [sym_data_max_label_length] = ACTIONS(1731),
    [sym_data_max_url_length] = ACTIONS(1731),
    [sym_max_last_login_info_length] = ACTIONS(1731),
    [sym_data_type_unknown] = ACTIONS(1731),
    [sym_data_type_string] = ACTIONS(1731),
    [sym_data_type_datetime] = ACTIONS(1731),
    [sym_data_type_date] = ACTIONS(1731),
    [sym_data_type_time] = ACTIONS(1731),
    [sym_data_type_image] = ACTIONS(1731),
    [sym_data_format_url] = ACTIONS(1731),
    [sym_data_format_phone] = ACTIONS(1731),
    [sym_data_format_email] = ACTIONS(1731),
    [sym_data_format_iso8601] = ACTIONS(1731),
    [sym_auditlog_priv] = ACTIONS(1731),
    [sym_configuration_priv] = ACTIONS(1731),
    [sym_deviceconfig_priv] = ACTIONS(1731),
    [sym_ftp_priv] = ACTIONS(1731),
    [sym_http_priv] = ACTIONS(1731),
    [sym_networkconfig_priv] = ACTIONS(1731),
    [sym_programport_priv] = ACTIONS(1731),
    [sym_remoteui_priv] = ACTIONS(1731),
    [sym_securitycontrol_priv] = ACTIONS(1731),
    [sym_softwaremanagement_priv] = ACTIONS(1731),
    [sym_terminal_priv] = ACTIONS(1731),
    [sym_tpadmin_priv] = ACTIONS(1731),
    [sym_user1_priv] = ACTIONS(1731),
    [sym_user2_priv] = ACTIONS(1731),
    [sym_user3_priv] = ACTIONS(1731),
    [sym_user4_priv] = ACTIONS(1731),
    [sym_usermanagement_priv] = ACTIONS(1731),
    [sym_valid_account] = ACTIONS(1731),
    [sym_err_name_invalid] = ACTIONS(1731),
    [sym_err_password_invalid] = ACTIONS(1731),
    [sym_err_invalid_account] = ACTIONS(1731),
    [sym_err_no_authorization] = ACTIONS(1731),
    [sym_err_invalid_parameter] = ACTIONS(1731),
    [sym_err_user_locked_out] = ACTIONS(1731),
    [sym_err_user_account_expired] = ACTIONS(1731),
    [sym_err_general_error] = ACTIONS(1731),
    [sym_err_authentication_error] = ACTIONS(1731),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1731),
    [sym_tls_validate_certificate] = ACTIONS(1731),
    [sym_tdata] = ACTIONS(1731),
    [sym_tchannel] = ACTIONS(1731),
    [sym_tlevel] = ACTIONS(1731),
    [sym_tbutton] = ACTIONS(1731),
    [sym_ttimeline] = ACTIONS(1731),
    [sym_tcustom] = ACTIONS(1731),
    [sym_url_struct] = ACTIONS(1731),
    [sym_dns_struct] = ACTIONS(1731),
    [sym_ip_address_struct] = ACTIONS(1731),
    [sym_dev_info_struct] = ACTIONS(1731),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1731),
    [sym_data_feed] = ACTIONS(1731),
    [sym_data_field] = ACTIONS(1731),
    [sym_data_record] = ACTIONS(1731),
    [sym_wc_data_feed] = ACTIONS(1731),
    [sym_wc_data_field] = ACTIONS(1731),
    [sym_wc_data_record] = ACTIONS(1731),
    [sym_last_login_info] = ACTIONS(1731),
    [sym____reserved___] = ACTIONS(1731),
    [sym_date] = ACTIONS(1731),
    [sym_day] = ACTIONS(1731),
    [sym_time] = ACTIONS(1731),
    [sym_ldate] = ACTIONS(1731),
    [sym_push_device] = ACTIONS(1731),
    [sym_push_channel] = ACTIONS(1731),
    [sym_push_devchan] = ACTIONS(1731),
    [sym_release_device] = ACTIONS(1731),
    [sym_release_channel] = ACTIONS(1731),
    [sym_release_devchan] = ACTIONS(1731),
    [sym_master_slot] = ACTIONS(1731),
    [sym_get_pulse_time] = ACTIONS(1731),
    [sym_get_timer] = ACTIONS(1731),
    [sym_master_sn] = ACTIONS(1731),
    [sym_system_number] = ACTIONS(1731),
    [sym_button] = ACTIONS(1731),
    [sym_dv_channel] = ACTIONS(1731),
    [sym_data] = ACTIONS(1731),
    [sym_level] = ACTIONS(1731),
    [sym_timeline] = ACTIONS(1731),
    [sym_channel] = ACTIONS(1731),
    [sym_custom] = ACTIONS(1731),
    [sym_atoi] = ACTIONS(1731),
    [sym_atol] = ACTIONS(1731),
    [sym_atof] = ACTIONS(1731),
    [sym_itoa] = ACTIONS(1731),
    [sym_itohex] = ACTIONS(1731),
    [sym_hextoi] = ACTIONS(1731),
    [sym_ftoa] = ACTIONS(1731),
    [sym_find_string] = ACTIONS(1731),
    [sym_left_string] = ACTIONS(1731),
    [sym_right_string] = ACTIONS(1731),
    [sym_mid_string] = ACTIONS(1731),
    [sym_remove_string] = ACTIONS(1731),
    [sym_get_buffer_string] = ACTIONS(1731),
    [sym_get_buffer_char] = ACTIONS(1731),
    [sym_get_multi_buffer_string] = ACTIONS(1731),
    [sym_length_string] = ACTIONS(1731),
    [sym_max_length_string] = ACTIONS(1731),
    [sym_set_length_string] = ACTIONS(1731),
    [sym_compare_string] = ACTIONS(1731),
    [sym_upper_string] = ACTIONS(1731),
    [sym_lower_string] = ACTIONS(1731),
    [sym_get_last] = ACTIONS(1731),
    [sym_date_to_day] = ACTIONS(1731),
    [sym_date_to_month] = ACTIONS(1731),
    [sym_date_to_year] = ACTIONS(1731),
    [sym_time_to_hour] = ACTIONS(1731),
    [sym_time_to_minute] = ACTIONS(1731),
    [sym_time_to_second] = ACTIONS(1731),
    [sym_day_of_week] = ACTIONS(1731),
    [sym_set_pulse_time] = ACTIONS(1731),
    [sym_set_timer] = ACTIONS(1731),
    [sym_file_open] = ACTIONS(1731),
    [sym_file_close] = ACTIONS(1731),
    [sym_file_read] = ACTIONS(1731),
    [sym_file_read_line] = ACTIONS(1731),
    [sym_file_write] = ACTIONS(1731),
    [sym_file_write_line] = ACTIONS(1731),
    [sym_file_copy] = ACTIONS(1731),
    [sym_file_delete] = ACTIONS(1731),
    [sym_file_rename] = ACTIONS(1731),
    [sym_file_dir] = ACTIONS(1731),
    [sym_file_getdir] = ACTIONS(1731),
    [sym_file_setdir] = ACTIONS(1731),
    [sym_file_create_dir] = ACTIONS(1731),
    [sym_file_remove_dir] = ACTIONS(1731),
    [sym_file_seek] = ACTIONS(1731),
    [sym_length_array] = ACTIONS(1731),
    [sym_max_length_array] = ACTIONS(1731),
    [sym_set_length_array] = ACTIONS(1731),
    [sym_ip_client_open] = ACTIONS(1731),
    [sym_ip_client_close] = ACTIONS(1731),
    [sym_ip_server_open] = ACTIONS(1731),
    [sym_ip_server_close] = ACTIONS(1731),
    [sym_ip_bound_client_open] = ACTIONS(1731),
    [sym_ip_mc_server_open] = ACTIONS(1731),
    [sym_ip_set_option] = ACTIONS(1731),
    [sym_ssh_client_open] = ACTIONS(1731),
    [sym_ssh_client_close] = ACTIONS(1731),
    [sym_tls_client_open] = ACTIONS(1731),
    [sym_tls_client_close] = ACTIONS(1731),
    [sym_do_push] = ACTIONS(1731),
    [sym_do_push_timed] = ACTIONS(1731),
    [sym_do_release] = ACTIONS(1731),
    [sym_device_id] = ACTIONS(1731),
    [sym_device_info] = ACTIONS(1731),
    [sym_device_id_string] = ACTIONS(1731),
    [sym_device_standby] = ACTIONS(1731),
    [sym_device_wake] = ACTIONS(1731),
    [sym_timeline_create] = ACTIONS(1731),
    [sym_timeline_kill] = ACTIONS(1731),
    [sym_timeline_reload] = ACTIONS(1731),
    [sym_timeline_pause] = ACTIONS(1731),
    [sym_timeline_restart] = ACTIONS(1731),
    [sym_timeline_set] = ACTIONS(1731),
    [sym_timeline_get] = ACTIONS(1731),
    [sym_timeline_active] = ACTIONS(1731),
    [sym_timeline_dynamic_id] = ACTIONS(1731),
    [sym_variable_to_string] = ACTIONS(1731),
    [sym_string_to_variable] = ACTIONS(1731),
    [sym_length_variable_to_string] = ACTIONS(1731),
    [sym_type_cast] = ACTIONS(1731),
    [sym_variable_to_xml] = ACTIONS(1731),
    [sym_xml_to_variable] = ACTIONS(1731),
    [sym_length_variable_to_xml] = ACTIONS(1731),
    [sym_redirect_string] = ACTIONS(1731),
    [sym_random_number] = ACTIONS(1731),
    [sym_get_unique_id] = ACTIONS(1731),
    [sym_get_system_number] = ACTIONS(1731),
    [sym_set_system_number] = ACTIONS(1731),
    [sym_get_serial_number] = ACTIONS(1731),
    [sym_get_master_build] = ACTIONS(1731),
    [sym_set_validation_code] = ACTIONS(1731),
    [sym_reboot] = ACTIONS(1731),
    [sym_load_duet_module] = ACTIONS(1731),
    [sym_unload_duet_module] = ACTIONS(1731),
    [sym_duet_mem_size_set] = ACTIONS(1731),
    [sym_duet_mem_size_get] = ACTIONS(1731),
    [sym_dynamic_application_device] = ACTIONS(1731),
    [sym_dynamic_polled_port] = ACTIONS(1731),
    [sym_static_port_binding] = ACTIONS(1731),
    [sym_static_ip_binding] = ACTIONS(1731),
    [sym_get_dns_list] = ACTIONS(1731),
    [sym_set_dns_list] = ACTIONS(1731),
    [sym_get_ip_address] = ACTIONS(1731),
    [sym_set_ip_address] = ACTIONS(1731),
    [sym_get_url_list] = ACTIONS(1731),
    [sym_add_url_entry] = ACTIONS(1731),
    [sym_delete_url_entry] = ACTIONS(1731),
    [sym_abs_value] = ACTIONS(1731),
    [sym_max_value] = ACTIONS(1731),
    [sym_min_value] = ACTIONS(1731),
    [sym_exp_value] = ACTIONS(1731),
    [sym_log_value] = ACTIONS(1731),
    [sym_log10_value] = ACTIONS(1731),
    [sym_power_value] = ACTIONS(1731),
    [sym_sqrt_value] = ACTIONS(1731),
    [sym_raw_be] = ACTIONS(1731),
    [sym_raw_le] = ACTIONS(1731),
    [sym_internal_queue_size_set] = ACTIONS(1731),
    [sym_internal_queue_size_get] = ACTIONS(1731),
    [sym_internal_threshold_set] = ACTIONS(1731),
    [sym_internal_threshold_get] = ACTIONS(1731),
    [sym_rebuilt_event] = ACTIONS(1731),
    [sym_do_custom_event] = ACTIONS(1731),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1731),
    [sym_clkmgr_set_clk_source] = ACTIONS(1731),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1731),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1731),
    [sym_clkmgr_get_timezone] = ACTIONS(1731),
    [sym_clkmgr_set_timezone] = ACTIONS(1731),
    [sym_clkmgr_get_resync_period] = ACTIONS(1731),
    [sym_clkmgr_set_resync_period] = ACTIONS(1731),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1731),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1731),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1731),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1731),
    [sym_clkmgr_get_timeservers] = ACTIONS(1731),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1731),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1731),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1731),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1731),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1731),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1731),
    [sym_set_log_level] = ACTIONS(1731),
    [sym_get_log_level] = ACTIONS(1731),
    [sym_amx_log] = ACTIONS(1731),
    [sym_smtp_server_config_set] = ACTIONS(1731),
    [sym_smtp_server_config_get] = ACTIONS(1731),
    [sym_smtp_send] = ACTIONS(1731),
    [sym_validate_netlinx_account] = ACTIONS(1731),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1731),
    [sym_authenticate_certificate] = ACTIONS(1731),
    [sym_audit_netlinx_session_event] = ACTIONS(1731),
    [sym_audit_netlinx_generic_event] = ACTIONS(1731),
    [sym_get_available_flash_disk_space] = ACTIONS(1731),
    [sym_get_max_flash_disk_space] = ACTIONS(1731),
    [sym_data_create_feed] = ACTIONS(1731),
    [sym_data_delete_feed] = ACTIONS(1731),
    [sym_data_publish_feed] = ACTIONS(1731),
    [sym_data_get_published_feed] = ACTIONS(1731),
    [sym_data_add_record] = ACTIONS(1731),
    [sym_data_get_event_record] = ACTIONS(1731),
    [sym__wc_data_create_feed] = ACTIONS(1731),
    [sym__wc_data_add_record] = ACTIONS(1731),
    [sym__wc_data_get_event_record] = ACTIONS(1731),
    [sym_format] = ACTIONS(1731),
    [sym_astro_clock] = ACTIONS(1731),
    [sym_set_outdoor_temperature] = ACTIONS(1731),
    [sym_set_virtual_level_count] = ACTIONS(1731),
    [sym_set_virtual_channel_count] = ACTIONS(1731),
    [sym_set_virtual_port_count] = ACTIONS(1731),
    [sym__wc_encode_int] = ACTIONS(1731),
    [sym__wc_decode_int] = ACTIONS(1731),
    [sym__wc_int] = ACTIONS(1731),
    [sym__wc_to_ch_int] = ACTIONS(1731),
    [sym__ch_to_wc_int] = ACTIONS(1731),
    [sym__wc_find_string_int] = ACTIONS(1731),
    [sym__wc_left_string_int] = ACTIONS(1731),
    [sym__wc_lower_string_int] = ACTIONS(1731),
    [sym__wc_mid_string_int] = ACTIONS(1731),
    [sym__wc_remove_string_int] = ACTIONS(1731),
    [sym__wc_right_string_int] = ACTIONS(1731),
    [sym__wc_upper_string_int] = ACTIONS(1731),
    [sym__wc_compare_string_int] = ACTIONS(1731),
    [sym__wc_get_buffer_char_int] = ACTIONS(1731),
    [sym__wc_get_buffer_string_int] = ACTIONS(1731),
    [sym__wc_concat_string_int] = ACTIONS(1731),
    [sym__wc_file_open_int] = ACTIONS(1731),
    [sym__wc_file_close_int] = ACTIONS(1731),
    [sym__wc_file_read_int] = ACTIONS(1731),
    [sym__wc_file_read_line_int] = ACTIONS(1731),
    [sym__wc_file_write_int] = ACTIONS(1731),
    [sym__wc_file_write_line_int] = ACTIONS(1731),
    [aux_sym_preproc_include_token1] = ACTIONS(1731),
    [aux_sym_preproc_define_token1] = ACTIONS(1733),
    [aux_sym_preproc_warn_token1] = ACTIONS(1733),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1733),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1733),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1733),
    [anon_sym_LBRACK] = ACTIONS(1733),
    [anon_sym_LBRACE] = ACTIONS(1733),
    [anon_sym_RBRACE] = ACTIONS(1733),
    [anon_sym_LPAREN] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [anon_sym_DASH] = ACTIONS(1731),
    [anon_sym_PLUS] = ACTIONS(1731),
    [anon_sym_DASH_DASH] = ACTIONS(1733),
    [anon_sym_PLUS_PLUS] = ACTIONS(1733),
    [anon_sym_DQUOTE] = ACTIONS(1733),
    [anon_sym_SQUOTE] = ACTIONS(1733),
    [sym_number_literal] = ACTIONS(1733),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(493)] = {
    [sym_identifier] = ACTIONS(1735),
    [sym_define_call_keyword] = ACTIONS(1735),
    [sym_define_function_keyword] = ACTIONS(1735),
    [sym_define_library_function_keyword] = ACTIONS(1735),
    [sym_define_module_keyword] = ACTIONS(1735),
    [sym_char_keyword] = ACTIONS(1735),
    [sym_widechar_keyword] = ACTIONS(1735),
    [sym_integer_keyword] = ACTIONS(1735),
    [sym_sinteger_keyword] = ACTIONS(1735),
    [sym_long_keyword] = ACTIONS(1735),
    [sym_slong_keyword] = ACTIONS(1735),
    [sym_float_keyword] = ACTIONS(1735),
    [sym_double_keyword] = ACTIONS(1735),
    [sym_variant_keyword] = ACTIONS(1735),
    [sym_variantarray_keyword] = ACTIONS(1735),
    [sym_local_var_keyword] = ACTIONS(1735),
    [sym_stack_var_keyword] = ACTIONS(1735),
    [sym_constant_keyword] = ACTIONS(1735),
    [sym_volatile_keyword] = ACTIONS(1735),
    [sym_non_volatile_keyword] = ACTIONS(1735),
    [sym_persistent_keyword] = ACTIONS(1735),
    [sym_dev_keyword] = ACTIONS(1735),
    [sym_devlev_keyword] = ACTIONS(1735),
    [sym_devchan_keyword] = ACTIONS(1735),
    [sym_if_keyword] = ACTIONS(1735),
    [sym_else_keyword] = ACTIONS(1735),
    [sym_switch_keyword] = ACTIONS(1735),
    [sym_case_keyword] = ACTIONS(1735),
    [sym_default_keyword] = ACTIONS(1735),
    [sym_while_keyword] = ACTIONS(1735),
    [sym_for_keyword] = ACTIONS(1735),
    [sym_break_keyword] = ACTIONS(1735),
    [sym_continue_keyword] = ACTIONS(1735),
    [sym_return_keyword] = ACTIONS(1735),
    [sym_select_keyword] = ACTIONS(1735),
    [sym_active_keyword] = ACTIONS(1735),
    [sym_struct_keyword] = ACTIONS(1735),
    [sym_structure_keyword] = ACTIONS(1735),
    [aux_sym_on_keyword_token1] = ACTIONS(1735),
    [aux_sym_off_keyword_token1] = ACTIONS(1735),
    [sym_send_level_keyword] = ACTIONS(1735),
    [sym_send_string_keyword] = ACTIONS(1735),
    [sym_send_command_keyword] = ACTIONS(1735),
    [sym_clear_buffer_keyword] = ACTIONS(1735),
    [sym_create_buffer_keyword] = ACTIONS(1735),
    [sym_create_multi_buffer_keyword] = ACTIONS(1735),
    [sym_call_keyword] = ACTIONS(1735),
    [sym_system_call_keyword] = ACTIONS(1735),
    [sym_devchan_to_keyword] = ACTIONS(1735),
    [sym_devchan_min_to_keyword] = ACTIONS(1735),
    [sym_devchan_total_off_keyword] = ACTIONS(1735),
    [sym_devchan_pulse_keyword] = ACTIONS(1735),
    [sym_wait_keyword] = ACTIONS(1735),
    [sym_cancel_wait_keyword] = ACTIONS(1735),
    [sym_wait_until_keyword] = ACTIONS(1735),
    [sym_cancel_wait_until_keyword] = ACTIONS(1735),
    [sym_cancel_all_wait_keyword] = ACTIONS(1735),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1735),
    [sym_not] = ACTIONS(1735),
    [sym_bnot] = ACTIONS(1735),
    [sym___file__] = ACTIONS(1735),
    [sym___date__] = ACTIONS(1735),
    [sym___ldate__] = ACTIONS(1735),
    [sym___time__] = ACTIONS(1735),
    [sym___line__] = ACTIONS(1735),
    [sym___name__] = ACTIONS(1735),
    [sym___netlinx__] = ACTIONS(1735),
    [sym_netlinx_axi_version] = ACTIONS(1735),
    [sym_url_flg_tcp] = ACTIONS(1735),
    [sym_url_flg_accinfopresent] = ACTIONS(1735),
    [sym_url_flg_temp] = ACTIONS(1735),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1735),
    [sym_url_flg_stat_mask] = ACTIONS(1735),
    [sym_url_flg_stat_lookup] = ACTIONS(1735),
    [sym_url_flg_stat_connecting] = ACTIONS(1735),
    [sym_url_flg_stat_waiting] = ACTIONS(1735),
    [sym_url_flg_stat_connected] = ACTIONS(1735),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1735),
    [sym_first_virtual_device] = ACTIONS(1735),
    [sym_first_local_port] = ACTIONS(1735),
    [sym_dynamic_virtual_device] = ACTIONS(1735),
    [sym_timeline_once] = ACTIONS(1735),
    [sym_timeline_repeat] = ACTIONS(1735),
    [sym_timeline_absolute] = ACTIONS(1735),
    [sym_timeline_relative] = ACTIONS(1735),
    [sym_do_push_timed_infinite] = ACTIONS(1735),
    [sym_source_type_no_address] = ACTIONS(1735),
    [sym_source_type_neuron_id] = ACTIONS(1735),
    [sym_source_type_ip_address] = ACTIONS(1735),
    [sym_source_type_axlink] = ACTIONS(1735),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1735),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1735),
    [sym_source_type_ip_socket_address] = ACTIONS(1735),
    [sym_source_type_rs232] = ACTIONS(1735),
    [sym_source_type_internal] = ACTIONS(1735),
    [sym_source_type_mac_address] = ACTIONS(1735),
    [sym_source_type_ipv4_port] = ACTIONS(1735),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1735),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1735),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1735),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1735),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1735),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1735),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1735),
    [sym_true] = ACTIONS(1735),
    [sym_false] = ACTIONS(1735),
    [sym_file_read_only] = ACTIONS(1735),
    [sym_file_rw_new] = ACTIONS(1735),
    [sym_file_rw_append] = ACTIONS(1735),
    [sym_ip_tcp] = ACTIONS(1735),
    [sym_ip_udp] = ACTIONS(1735),
    [sym_ip_udp_2way] = ACTIONS(1735),
    [sym_xml_encode_types] = ACTIONS(1735),
    [sym_xml_encode_char_as_list] = ACTIONS(1735),
    [sym_xml_encode_le] = ACTIONS(1735),
    [sym_xml_decode_types] = ACTIONS(1735),
    [sym_xml_decode_no_preserve] = ACTIONS(1735),
    [sym_duet_dev_type_display_device] = ACTIONS(1735),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1735),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1735),
    [sym_duet_dev_type_io_device] = ACTIONS(1735),
    [sym_duet_dev_type_relay_device] = ACTIONS(1735),
    [sym_duet_dev_type_ups] = ACTIONS(1735),
    [sym_duet_dev_type_amplifier] = ACTIONS(1735),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1735),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1735),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1735),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1735),
    [sym_duet_dev_type_camera] = ACTIONS(1735),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1735),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1735),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1735),
    [sym_duet_dev_type_dss] = ACTIONS(1735),
    [sym_duet_dev_type_dvr] = ACTIONS(1735),
    [sym_duet_dev_type_disc_device] = ACTIONS(1735),
    [sym_duet_dev_type_document_camera] = ACTIONS(1735),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1735),
    [sym_duet_dev_type_hvac] = ACTIONS(1735),
    [sym_duet_dev_type_keypad] = ACTIONS(1735),
    [sym_duet_dev_type_light] = ACTIONS(1735),
    [sym_duet_dev_type_monitor] = ACTIONS(1735),
    [sym_duet_dev_type_motor] = ACTIONS(1735),
    [sym_duet_dev_type_multi_window] = ACTIONS(1735),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1735),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1735),
    [sym_duet_dev_type_receiver] = ACTIONS(1735),
    [sym_duet_dev_type_security_system] = ACTIONS(1735),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1735),
    [sym_duet_dev_type_settop_box] = ACTIONS(1735),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1735),
    [sym_duet_dev_type_switcher] = ACTIONS(1735),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1735),
    [sym_duet_dev_type_tv] = ACTIONS(1735),
    [sym_duet_dev_type_utility] = ACTIONS(1735),
    [sym_duet_dev_type_vcr] = ACTIONS(1735),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1735),
    [sym_duet_dev_type_video_processor] = ACTIONS(1735),
    [sym_duet_dev_type_video_projector] = ACTIONS(1735),
    [sym_duet_dev_type_video_wall] = ACTIONS(1735),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1735),
    [sym_duet_dev_type_weather] = ACTIONS(1735),
    [sym_duet_dev_not_polled] = ACTIONS(1735),
    [sym_duet_dev_polled] = ACTIONS(1735),
    [sym_clkmgr_mode_network] = ACTIONS(1735),
    [sym_clkmgr_mode_standalone] = ACTIONS(1735),
    [sym_normal_standby] = ACTIONS(1735),
    [sym_normal_wake] = ACTIONS(1735),
    [sym_ip_multicast_ttl_option] = ACTIONS(1735),
    [sym_ip_ttl_subnet] = ACTIONS(1735),
    [sym_ip_ttl_site] = ACTIONS(1735),
    [sym_ip_ttl_region] = ACTIONS(1735),
    [sym_ip_ttl_continent] = ACTIONS(1735),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1735),
    [sym_ip_nodelay_on] = ACTIONS(1735),
    [sym_ip_nodelay_off] = ACTIONS(1735),
    [sym_amx_error] = ACTIONS(1735),
    [sym_amx_warning] = ACTIONS(1735),
    [sym_amx_info] = ACTIONS(1735),
    [sym_amx_debug] = ACTIONS(1735),
    [sym_netlinx_login_success] = ACTIONS(1735),
    [sym_netlinx_login_fail] = ACTIONS(1735),
    [sym_netlinx_logout] = ACTIONS(1735),
    [sym_null_str] = ACTIONS(1735),
    [sym_smtp_address] = ACTIONS(1735),
    [sym_smtp_port_number] = ACTIONS(1735),
    [sym_smtp_username] = ACTIONS(1735),
    [sym_smtp_password] = ACTIONS(1735),
    [sym_smtp_require_tls] = ACTIONS(1735),
    [sym_smtp_from] = ACTIONS(1735),
    [sym_smtp_tls_true] = ACTIONS(1735),
    [sym_smtp_tls_false] = ACTIONS(1735),
    [sym_listview_on_row_select_event] = ACTIONS(1735),
    [sym_data_structure_unknown] = ACTIONS(1735),
    [sym_data_structure_datafeed] = ACTIONS(1735),
    [sym_data_structure_datarecordset] = ACTIONS(1735),
    [sym_data_structure_datarecord] = ACTIONS(1735),
    [sym_data_structure_datafield] = ACTIONS(1735),
    [sym_data_max_content_field_count] = ACTIONS(1735),
    [sym_data_max_metadata_field_count] = ACTIONS(1735),
    [sym_data_max_id_length] = ACTIONS(1735),
    [sym_data_max_name_length] = ACTIONS(1735),
    [sym_data_max_description_length] = ACTIONS(1735),
    [sym_data_max_type_length] = ACTIONS(1735),
    [sym_data_max_format_length] = ACTIONS(1735),
    [sym_data_max_source_length] = ACTIONS(1735),
    [sym_data_max_value_length] = ACTIONS(1735),
    [sym_data_max_label_length] = ACTIONS(1735),
    [sym_data_max_url_length] = ACTIONS(1735),
    [sym_max_last_login_info_length] = ACTIONS(1735),
    [sym_data_type_unknown] = ACTIONS(1735),
    [sym_data_type_string] = ACTIONS(1735),
    [sym_data_type_datetime] = ACTIONS(1735),
    [sym_data_type_date] = ACTIONS(1735),
    [sym_data_type_time] = ACTIONS(1735),
    [sym_data_type_image] = ACTIONS(1735),
    [sym_data_format_url] = ACTIONS(1735),
    [sym_data_format_phone] = ACTIONS(1735),
    [sym_data_format_email] = ACTIONS(1735),
    [sym_data_format_iso8601] = ACTIONS(1735),
    [sym_auditlog_priv] = ACTIONS(1735),
    [sym_configuration_priv] = ACTIONS(1735),
    [sym_deviceconfig_priv] = ACTIONS(1735),
    [sym_ftp_priv] = ACTIONS(1735),
    [sym_http_priv] = ACTIONS(1735),
    [sym_networkconfig_priv] = ACTIONS(1735),
    [sym_programport_priv] = ACTIONS(1735),
    [sym_remoteui_priv] = ACTIONS(1735),
    [sym_securitycontrol_priv] = ACTIONS(1735),
    [sym_softwaremanagement_priv] = ACTIONS(1735),
    [sym_terminal_priv] = ACTIONS(1735),
    [sym_tpadmin_priv] = ACTIONS(1735),
    [sym_user1_priv] = ACTIONS(1735),
    [sym_user2_priv] = ACTIONS(1735),
    [sym_user3_priv] = ACTIONS(1735),
    [sym_user4_priv] = ACTIONS(1735),
    [sym_usermanagement_priv] = ACTIONS(1735),
    [sym_valid_account] = ACTIONS(1735),
    [sym_err_name_invalid] = ACTIONS(1735),
    [sym_err_password_invalid] = ACTIONS(1735),
    [sym_err_invalid_account] = ACTIONS(1735),
    [sym_err_no_authorization] = ACTIONS(1735),
    [sym_err_invalid_parameter] = ACTIONS(1735),
    [sym_err_user_locked_out] = ACTIONS(1735),
    [sym_err_user_account_expired] = ACTIONS(1735),
    [sym_err_general_error] = ACTIONS(1735),
    [sym_err_authentication_error] = ACTIONS(1735),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1735),
    [sym_tls_validate_certificate] = ACTIONS(1735),
    [sym_tdata] = ACTIONS(1735),
    [sym_tchannel] = ACTIONS(1735),
    [sym_tlevel] = ACTIONS(1735),
    [sym_tbutton] = ACTIONS(1735),
    [sym_ttimeline] = ACTIONS(1735),
    [sym_tcustom] = ACTIONS(1735),
    [sym_url_struct] = ACTIONS(1735),
    [sym_dns_struct] = ACTIONS(1735),
    [sym_ip_address_struct] = ACTIONS(1735),
    [sym_dev_info_struct] = ACTIONS(1735),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1735),
    [sym_data_feed] = ACTIONS(1735),
    [sym_data_field] = ACTIONS(1735),
    [sym_data_record] = ACTIONS(1735),
    [sym_wc_data_feed] = ACTIONS(1735),
    [sym_wc_data_field] = ACTIONS(1735),
    [sym_wc_data_record] = ACTIONS(1735),
    [sym_last_login_info] = ACTIONS(1735),
    [sym____reserved___] = ACTIONS(1735),
    [sym_date] = ACTIONS(1735),
    [sym_day] = ACTIONS(1735),
    [sym_time] = ACTIONS(1735),
    [sym_ldate] = ACTIONS(1735),
    [sym_push_device] = ACTIONS(1735),
    [sym_push_channel] = ACTIONS(1735),
    [sym_push_devchan] = ACTIONS(1735),
    [sym_release_device] = ACTIONS(1735),
    [sym_release_channel] = ACTIONS(1735),
    [sym_release_devchan] = ACTIONS(1735),
    [sym_master_slot] = ACTIONS(1735),
    [sym_get_pulse_time] = ACTIONS(1735),
    [sym_get_timer] = ACTIONS(1735),
    [sym_master_sn] = ACTIONS(1735),
    [sym_system_number] = ACTIONS(1735),
    [sym_button] = ACTIONS(1735),
    [sym_dv_channel] = ACTIONS(1735),
    [sym_data] = ACTIONS(1735),
    [sym_level] = ACTIONS(1735),
    [sym_timeline] = ACTIONS(1735),
    [sym_channel] = ACTIONS(1735),
    [sym_custom] = ACTIONS(1735),
    [sym_atoi] = ACTIONS(1735),
    [sym_atol] = ACTIONS(1735),
    [sym_atof] = ACTIONS(1735),
    [sym_itoa] = ACTIONS(1735),
    [sym_itohex] = ACTIONS(1735),
    [sym_hextoi] = ACTIONS(1735),
    [sym_ftoa] = ACTIONS(1735),
    [sym_find_string] = ACTIONS(1735),
    [sym_left_string] = ACTIONS(1735),
    [sym_right_string] = ACTIONS(1735),
    [sym_mid_string] = ACTIONS(1735),
    [sym_remove_string] = ACTIONS(1735),
    [sym_get_buffer_string] = ACTIONS(1735),
    [sym_get_buffer_char] = ACTIONS(1735),
    [sym_get_multi_buffer_string] = ACTIONS(1735),
    [sym_length_string] = ACTIONS(1735),
    [sym_max_length_string] = ACTIONS(1735),
    [sym_set_length_string] = ACTIONS(1735),
    [sym_compare_string] = ACTIONS(1735),
    [sym_upper_string] = ACTIONS(1735),
    [sym_lower_string] = ACTIONS(1735),
    [sym_get_last] = ACTIONS(1735),
    [sym_date_to_day] = ACTIONS(1735),
    [sym_date_to_month] = ACTIONS(1735),
    [sym_date_to_year] = ACTIONS(1735),
    [sym_time_to_hour] = ACTIONS(1735),
    [sym_time_to_minute] = ACTIONS(1735),
    [sym_time_to_second] = ACTIONS(1735),
    [sym_day_of_week] = ACTIONS(1735),
    [sym_set_pulse_time] = ACTIONS(1735),
    [sym_set_timer] = ACTIONS(1735),
    [sym_file_open] = ACTIONS(1735),
    [sym_file_close] = ACTIONS(1735),
    [sym_file_read] = ACTIONS(1735),
    [sym_file_read_line] = ACTIONS(1735),
    [sym_file_write] = ACTIONS(1735),
    [sym_file_write_line] = ACTIONS(1735),
    [sym_file_copy] = ACTIONS(1735),
    [sym_file_delete] = ACTIONS(1735),
    [sym_file_rename] = ACTIONS(1735),
    [sym_file_dir] = ACTIONS(1735),
    [sym_file_getdir] = ACTIONS(1735),
    [sym_file_setdir] = ACTIONS(1735),
    [sym_file_create_dir] = ACTIONS(1735),
    [sym_file_remove_dir] = ACTIONS(1735),
    [sym_file_seek] = ACTIONS(1735),
    [sym_length_array] = ACTIONS(1735),
    [sym_max_length_array] = ACTIONS(1735),
    [sym_set_length_array] = ACTIONS(1735),
    [sym_ip_client_open] = ACTIONS(1735),
    [sym_ip_client_close] = ACTIONS(1735),
    [sym_ip_server_open] = ACTIONS(1735),
    [sym_ip_server_close] = ACTIONS(1735),
    [sym_ip_bound_client_open] = ACTIONS(1735),
    [sym_ip_mc_server_open] = ACTIONS(1735),
    [sym_ip_set_option] = ACTIONS(1735),
    [sym_ssh_client_open] = ACTIONS(1735),
    [sym_ssh_client_close] = ACTIONS(1735),
    [sym_tls_client_open] = ACTIONS(1735),
    [sym_tls_client_close] = ACTIONS(1735),
    [sym_do_push] = ACTIONS(1735),
    [sym_do_push_timed] = ACTIONS(1735),
    [sym_do_release] = ACTIONS(1735),
    [sym_device_id] = ACTIONS(1735),
    [sym_device_info] = ACTIONS(1735),
    [sym_device_id_string] = ACTIONS(1735),
    [sym_device_standby] = ACTIONS(1735),
    [sym_device_wake] = ACTIONS(1735),
    [sym_timeline_create] = ACTIONS(1735),
    [sym_timeline_kill] = ACTIONS(1735),
    [sym_timeline_reload] = ACTIONS(1735),
    [sym_timeline_pause] = ACTIONS(1735),
    [sym_timeline_restart] = ACTIONS(1735),
    [sym_timeline_set] = ACTIONS(1735),
    [sym_timeline_get] = ACTIONS(1735),
    [sym_timeline_active] = ACTIONS(1735),
    [sym_timeline_dynamic_id] = ACTIONS(1735),
    [sym_variable_to_string] = ACTIONS(1735),
    [sym_string_to_variable] = ACTIONS(1735),
    [sym_length_variable_to_string] = ACTIONS(1735),
    [sym_type_cast] = ACTIONS(1735),
    [sym_variable_to_xml] = ACTIONS(1735),
    [sym_xml_to_variable] = ACTIONS(1735),
    [sym_length_variable_to_xml] = ACTIONS(1735),
    [sym_redirect_string] = ACTIONS(1735),
    [sym_random_number] = ACTIONS(1735),
    [sym_get_unique_id] = ACTIONS(1735),
    [sym_get_system_number] = ACTIONS(1735),
    [sym_set_system_number] = ACTIONS(1735),
    [sym_get_serial_number] = ACTIONS(1735),
    [sym_get_master_build] = ACTIONS(1735),
    [sym_set_validation_code] = ACTIONS(1735),
    [sym_reboot] = ACTIONS(1735),
    [sym_load_duet_module] = ACTIONS(1735),
    [sym_unload_duet_module] = ACTIONS(1735),
    [sym_duet_mem_size_set] = ACTIONS(1735),
    [sym_duet_mem_size_get] = ACTIONS(1735),
    [sym_dynamic_application_device] = ACTIONS(1735),
    [sym_dynamic_polled_port] = ACTIONS(1735),
    [sym_static_port_binding] = ACTIONS(1735),
    [sym_static_ip_binding] = ACTIONS(1735),
    [sym_get_dns_list] = ACTIONS(1735),
    [sym_set_dns_list] = ACTIONS(1735),
    [sym_get_ip_address] = ACTIONS(1735),
    [sym_set_ip_address] = ACTIONS(1735),
    [sym_get_url_list] = ACTIONS(1735),
    [sym_add_url_entry] = ACTIONS(1735),
    [sym_delete_url_entry] = ACTIONS(1735),
    [sym_abs_value] = ACTIONS(1735),
    [sym_max_value] = ACTIONS(1735),
    [sym_min_value] = ACTIONS(1735),
    [sym_exp_value] = ACTIONS(1735),
    [sym_log_value] = ACTIONS(1735),
    [sym_log10_value] = ACTIONS(1735),
    [sym_power_value] = ACTIONS(1735),
    [sym_sqrt_value] = ACTIONS(1735),
    [sym_raw_be] = ACTIONS(1735),
    [sym_raw_le] = ACTIONS(1735),
    [sym_internal_queue_size_set] = ACTIONS(1735),
    [sym_internal_queue_size_get] = ACTIONS(1735),
    [sym_internal_threshold_set] = ACTIONS(1735),
    [sym_internal_threshold_get] = ACTIONS(1735),
    [sym_rebuilt_event] = ACTIONS(1735),
    [sym_do_custom_event] = ACTIONS(1735),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1735),
    [sym_clkmgr_set_clk_source] = ACTIONS(1735),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1735),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1735),
    [sym_clkmgr_get_timezone] = ACTIONS(1735),
    [sym_clkmgr_set_timezone] = ACTIONS(1735),
    [sym_clkmgr_get_resync_period] = ACTIONS(1735),
    [sym_clkmgr_set_resync_period] = ACTIONS(1735),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1735),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1735),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1735),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1735),
    [sym_clkmgr_get_timeservers] = ACTIONS(1735),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1735),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1735),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1735),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1735),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1735),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1735),
    [sym_set_log_level] = ACTIONS(1735),
    [sym_get_log_level] = ACTIONS(1735),
    [sym_amx_log] = ACTIONS(1735),
    [sym_smtp_server_config_set] = ACTIONS(1735),
    [sym_smtp_server_config_get] = ACTIONS(1735),
    [sym_smtp_send] = ACTIONS(1735),
    [sym_validate_netlinx_account] = ACTIONS(1735),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1735),
    [sym_authenticate_certificate] = ACTIONS(1735),
    [sym_audit_netlinx_session_event] = ACTIONS(1735),
    [sym_audit_netlinx_generic_event] = ACTIONS(1735),
    [sym_get_available_flash_disk_space] = ACTIONS(1735),
    [sym_get_max_flash_disk_space] = ACTIONS(1735),
    [sym_data_create_feed] = ACTIONS(1735),
    [sym_data_delete_feed] = ACTIONS(1735),
    [sym_data_publish_feed] = ACTIONS(1735),
    [sym_data_get_published_feed] = ACTIONS(1735),
    [sym_data_add_record] = ACTIONS(1735),
    [sym_data_get_event_record] = ACTIONS(1735),
    [sym__wc_data_create_feed] = ACTIONS(1735),
    [sym__wc_data_add_record] = ACTIONS(1735),
    [sym__wc_data_get_event_record] = ACTIONS(1735),
    [sym_format] = ACTIONS(1735),
    [sym_astro_clock] = ACTIONS(1735),
    [sym_set_outdoor_temperature] = ACTIONS(1735),
    [sym_set_virtual_level_count] = ACTIONS(1735),
    [sym_set_virtual_channel_count] = ACTIONS(1735),
    [sym_set_virtual_port_count] = ACTIONS(1735),
    [sym__wc_encode_int] = ACTIONS(1735),
    [sym__wc_decode_int] = ACTIONS(1735),
    [sym__wc_int] = ACTIONS(1735),
    [sym__wc_to_ch_int] = ACTIONS(1735),
    [sym__ch_to_wc_int] = ACTIONS(1735),
    [sym__wc_find_string_int] = ACTIONS(1735),
    [sym__wc_left_string_int] = ACTIONS(1735),
    [sym__wc_lower_string_int] = ACTIONS(1735),
    [sym__wc_mid_string_int] = ACTIONS(1735),
    [sym__wc_remove_string_int] = ACTIONS(1735),
    [sym__wc_right_string_int] = ACTIONS(1735),
    [sym__wc_upper_string_int] = ACTIONS(1735),
    [sym__wc_compare_string_int] = ACTIONS(1735),
    [sym__wc_get_buffer_char_int] = ACTIONS(1735),
    [sym__wc_get_buffer_string_int] = ACTIONS(1735),
    [sym__wc_concat_string_int] = ACTIONS(1735),
    [sym__wc_file_open_int] = ACTIONS(1735),
    [sym__wc_file_close_int] = ACTIONS(1735),
    [sym__wc_file_read_int] = ACTIONS(1735),
    [sym__wc_file_read_line_int] = ACTIONS(1735),
    [sym__wc_file_write_int] = ACTIONS(1735),
    [sym__wc_file_write_line_int] = ACTIONS(1735),
    [aux_sym_preproc_include_token1] = ACTIONS(1735),
    [aux_sym_preproc_define_token1] = ACTIONS(1737),
    [aux_sym_preproc_warn_token1] = ACTIONS(1737),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1737),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1737),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1737),
    [anon_sym_LBRACK] = ACTIONS(1737),
    [anon_sym_LBRACE] = ACTIONS(1737),
    [anon_sym_RBRACE] = ACTIONS(1737),
    [anon_sym_LPAREN] = ACTIONS(1735),
    [anon_sym_BANG] = ACTIONS(1737),
    [anon_sym_TILDE] = ACTIONS(1737),
    [anon_sym_DASH] = ACTIONS(1735),
    [anon_sym_PLUS] = ACTIONS(1735),
    [anon_sym_DASH_DASH] = ACTIONS(1737),
    [anon_sym_PLUS_PLUS] = ACTIONS(1737),
    [anon_sym_DQUOTE] = ACTIONS(1737),
    [anon_sym_SQUOTE] = ACTIONS(1737),
    [sym_number_literal] = ACTIONS(1737),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(494)] = {
    [sym_identifier] = ACTIONS(1827),
    [sym_define_call_keyword] = ACTIONS(1827),
    [sym_define_function_keyword] = ACTIONS(1827),
    [sym_define_library_function_keyword] = ACTIONS(1827),
    [sym_define_module_keyword] = ACTIONS(1827),
    [sym_char_keyword] = ACTIONS(1827),
    [sym_widechar_keyword] = ACTIONS(1827),
    [sym_integer_keyword] = ACTIONS(1827),
    [sym_sinteger_keyword] = ACTIONS(1827),
    [sym_long_keyword] = ACTIONS(1827),
    [sym_slong_keyword] = ACTIONS(1827),
    [sym_float_keyword] = ACTIONS(1827),
    [sym_double_keyword] = ACTIONS(1827),
    [sym_variant_keyword] = ACTIONS(1827),
    [sym_variantarray_keyword] = ACTIONS(1827),
    [sym_local_var_keyword] = ACTIONS(1827),
    [sym_stack_var_keyword] = ACTIONS(1827),
    [sym_constant_keyword] = ACTIONS(1827),
    [sym_volatile_keyword] = ACTIONS(1827),
    [sym_non_volatile_keyword] = ACTIONS(1827),
    [sym_persistent_keyword] = ACTIONS(1827),
    [sym_dev_keyword] = ACTIONS(1827),
    [sym_devlev_keyword] = ACTIONS(1827),
    [sym_devchan_keyword] = ACTIONS(1827),
    [sym_if_keyword] = ACTIONS(1827),
    [sym_else_keyword] = ACTIONS(1827),
    [sym_switch_keyword] = ACTIONS(1827),
    [sym_case_keyword] = ACTIONS(1827),
    [sym_default_keyword] = ACTIONS(1827),
    [sym_while_keyword] = ACTIONS(1827),
    [sym_for_keyword] = ACTIONS(1827),
    [sym_break_keyword] = ACTIONS(1827),
    [sym_continue_keyword] = ACTIONS(1827),
    [sym_return_keyword] = ACTIONS(1827),
    [sym_select_keyword] = ACTIONS(1827),
    [sym_active_keyword] = ACTIONS(1827),
    [sym_struct_keyword] = ACTIONS(1827),
    [sym_structure_keyword] = ACTIONS(1827),
    [aux_sym_on_keyword_token1] = ACTIONS(1827),
    [aux_sym_off_keyword_token1] = ACTIONS(1827),
    [sym_send_level_keyword] = ACTIONS(1827),
    [sym_send_string_keyword] = ACTIONS(1827),
    [sym_send_command_keyword] = ACTIONS(1827),
    [sym_clear_buffer_keyword] = ACTIONS(1827),
    [sym_create_buffer_keyword] = ACTIONS(1827),
    [sym_create_multi_buffer_keyword] = ACTIONS(1827),
    [sym_call_keyword] = ACTIONS(1827),
    [sym_system_call_keyword] = ACTIONS(1827),
    [sym_devchan_to_keyword] = ACTIONS(1827),
    [sym_devchan_min_to_keyword] = ACTIONS(1827),
    [sym_devchan_total_off_keyword] = ACTIONS(1827),
    [sym_devchan_pulse_keyword] = ACTIONS(1827),
    [sym_wait_keyword] = ACTIONS(1827),
    [sym_cancel_wait_keyword] = ACTIONS(1827),
    [sym_wait_until_keyword] = ACTIONS(1827),
    [sym_cancel_wait_until_keyword] = ACTIONS(1827),
    [sym_cancel_all_wait_keyword] = ACTIONS(1827),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1827),
    [sym_not] = ACTIONS(1827),
    [sym_bnot] = ACTIONS(1827),
    [sym___file__] = ACTIONS(1827),
    [sym___date__] = ACTIONS(1827),
    [sym___ldate__] = ACTIONS(1827),
    [sym___time__] = ACTIONS(1827),
    [sym___line__] = ACTIONS(1827),
    [sym___name__] = ACTIONS(1827),
    [sym___netlinx__] = ACTIONS(1827),
    [sym_netlinx_axi_version] = ACTIONS(1827),
    [sym_url_flg_tcp] = ACTIONS(1827),
    [sym_url_flg_accinfopresent] = ACTIONS(1827),
    [sym_url_flg_temp] = ACTIONS(1827),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1827),
    [sym_url_flg_stat_mask] = ACTIONS(1827),
    [sym_url_flg_stat_lookup] = ACTIONS(1827),
    [sym_url_flg_stat_connecting] = ACTIONS(1827),
    [sym_url_flg_stat_waiting] = ACTIONS(1827),
    [sym_url_flg_stat_connected] = ACTIONS(1827),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1827),
    [sym_first_virtual_device] = ACTIONS(1827),
    [sym_first_local_port] = ACTIONS(1827),
    [sym_dynamic_virtual_device] = ACTIONS(1827),
    [sym_timeline_once] = ACTIONS(1827),
    [sym_timeline_repeat] = ACTIONS(1827),
    [sym_timeline_absolute] = ACTIONS(1827),
    [sym_timeline_relative] = ACTIONS(1827),
    [sym_do_push_timed_infinite] = ACTIONS(1827),
    [sym_source_type_no_address] = ACTIONS(1827),
    [sym_source_type_neuron_id] = ACTIONS(1827),
    [sym_source_type_ip_address] = ACTIONS(1827),
    [sym_source_type_axlink] = ACTIONS(1827),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1827),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1827),
    [sym_source_type_ip_socket_address] = ACTIONS(1827),
    [sym_source_type_rs232] = ACTIONS(1827),
    [sym_source_type_internal] = ACTIONS(1827),
    [sym_source_type_mac_address] = ACTIONS(1827),
    [sym_source_type_ipv4_port] = ACTIONS(1827),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1827),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1827),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1827),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1827),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1827),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1827),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1827),
    [sym_true] = ACTIONS(1827),
    [sym_false] = ACTIONS(1827),
    [sym_file_read_only] = ACTIONS(1827),
    [sym_file_rw_new] = ACTIONS(1827),
    [sym_file_rw_append] = ACTIONS(1827),
    [sym_ip_tcp] = ACTIONS(1827),
    [sym_ip_udp] = ACTIONS(1827),
    [sym_ip_udp_2way] = ACTIONS(1827),
    [sym_xml_encode_types] = ACTIONS(1827),
    [sym_xml_encode_char_as_list] = ACTIONS(1827),
    [sym_xml_encode_le] = ACTIONS(1827),
    [sym_xml_decode_types] = ACTIONS(1827),
    [sym_xml_decode_no_preserve] = ACTIONS(1827),
    [sym_duet_dev_type_display_device] = ACTIONS(1827),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1827),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1827),
    [sym_duet_dev_type_io_device] = ACTIONS(1827),
    [sym_duet_dev_type_relay_device] = ACTIONS(1827),
    [sym_duet_dev_type_ups] = ACTIONS(1827),
    [sym_duet_dev_type_amplifier] = ACTIONS(1827),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1827),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1827),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1827),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1827),
    [sym_duet_dev_type_camera] = ACTIONS(1827),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1827),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1827),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1827),
    [sym_duet_dev_type_dss] = ACTIONS(1827),
    [sym_duet_dev_type_dvr] = ACTIONS(1827),
    [sym_duet_dev_type_disc_device] = ACTIONS(1827),
    [sym_duet_dev_type_document_camera] = ACTIONS(1827),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1827),
    [sym_duet_dev_type_hvac] = ACTIONS(1827),
    [sym_duet_dev_type_keypad] = ACTIONS(1827),
    [sym_duet_dev_type_light] = ACTIONS(1827),
    [sym_duet_dev_type_monitor] = ACTIONS(1827),
    [sym_duet_dev_type_motor] = ACTIONS(1827),
    [sym_duet_dev_type_multi_window] = ACTIONS(1827),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1827),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1827),
    [sym_duet_dev_type_receiver] = ACTIONS(1827),
    [sym_duet_dev_type_security_system] = ACTIONS(1827),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1827),
    [sym_duet_dev_type_settop_box] = ACTIONS(1827),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1827),
    [sym_duet_dev_type_switcher] = ACTIONS(1827),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1827),
    [sym_duet_dev_type_tv] = ACTIONS(1827),
    [sym_duet_dev_type_utility] = ACTIONS(1827),
    [sym_duet_dev_type_vcr] = ACTIONS(1827),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1827),
    [sym_duet_dev_type_video_processor] = ACTIONS(1827),
    [sym_duet_dev_type_video_projector] = ACTIONS(1827),
    [sym_duet_dev_type_video_wall] = ACTIONS(1827),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1827),
    [sym_duet_dev_type_weather] = ACTIONS(1827),
    [sym_duet_dev_not_polled] = ACTIONS(1827),
    [sym_duet_dev_polled] = ACTIONS(1827),
    [sym_clkmgr_mode_network] = ACTIONS(1827),
    [sym_clkmgr_mode_standalone] = ACTIONS(1827),
    [sym_normal_standby] = ACTIONS(1827),
    [sym_normal_wake] = ACTIONS(1827),
    [sym_ip_multicast_ttl_option] = ACTIONS(1827),
    [sym_ip_ttl_subnet] = ACTIONS(1827),
    [sym_ip_ttl_site] = ACTIONS(1827),
    [sym_ip_ttl_region] = ACTIONS(1827),
    [sym_ip_ttl_continent] = ACTIONS(1827),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1827),
    [sym_ip_nodelay_on] = ACTIONS(1827),
    [sym_ip_nodelay_off] = ACTIONS(1827),
    [sym_amx_error] = ACTIONS(1827),
    [sym_amx_warning] = ACTIONS(1827),
    [sym_amx_info] = ACTIONS(1827),
    [sym_amx_debug] = ACTIONS(1827),
    [sym_netlinx_login_success] = ACTIONS(1827),
    [sym_netlinx_login_fail] = ACTIONS(1827),
    [sym_netlinx_logout] = ACTIONS(1827),
    [sym_null_str] = ACTIONS(1827),
    [sym_smtp_address] = ACTIONS(1827),
    [sym_smtp_port_number] = ACTIONS(1827),
    [sym_smtp_username] = ACTIONS(1827),
    [sym_smtp_password] = ACTIONS(1827),
    [sym_smtp_require_tls] = ACTIONS(1827),
    [sym_smtp_from] = ACTIONS(1827),
    [sym_smtp_tls_true] = ACTIONS(1827),
    [sym_smtp_tls_false] = ACTIONS(1827),
    [sym_listview_on_row_select_event] = ACTIONS(1827),
    [sym_data_structure_unknown] = ACTIONS(1827),
    [sym_data_structure_datafeed] = ACTIONS(1827),
    [sym_data_structure_datarecordset] = ACTIONS(1827),
    [sym_data_structure_datarecord] = ACTIONS(1827),
    [sym_data_structure_datafield] = ACTIONS(1827),
    [sym_data_max_content_field_count] = ACTIONS(1827),
    [sym_data_max_metadata_field_count] = ACTIONS(1827),
    [sym_data_max_id_length] = ACTIONS(1827),
    [sym_data_max_name_length] = ACTIONS(1827),
    [sym_data_max_description_length] = ACTIONS(1827),
    [sym_data_max_type_length] = ACTIONS(1827),
    [sym_data_max_format_length] = ACTIONS(1827),
    [sym_data_max_source_length] = ACTIONS(1827),
    [sym_data_max_value_length] = ACTIONS(1827),
    [sym_data_max_label_length] = ACTIONS(1827),
    [sym_data_max_url_length] = ACTIONS(1827),
    [sym_max_last_login_info_length] = ACTIONS(1827),
    [sym_data_type_unknown] = ACTIONS(1827),
    [sym_data_type_string] = ACTIONS(1827),
    [sym_data_type_datetime] = ACTIONS(1827),
    [sym_data_type_date] = ACTIONS(1827),
    [sym_data_type_time] = ACTIONS(1827),
    [sym_data_type_image] = ACTIONS(1827),
    [sym_data_format_url] = ACTIONS(1827),
    [sym_data_format_phone] = ACTIONS(1827),
    [sym_data_format_email] = ACTIONS(1827),
    [sym_data_format_iso8601] = ACTIONS(1827),
    [sym_auditlog_priv] = ACTIONS(1827),
    [sym_configuration_priv] = ACTIONS(1827),
    [sym_deviceconfig_priv] = ACTIONS(1827),
    [sym_ftp_priv] = ACTIONS(1827),
    [sym_http_priv] = ACTIONS(1827),
    [sym_networkconfig_priv] = ACTIONS(1827),
    [sym_programport_priv] = ACTIONS(1827),
    [sym_remoteui_priv] = ACTIONS(1827),
    [sym_securitycontrol_priv] = ACTIONS(1827),
    [sym_softwaremanagement_priv] = ACTIONS(1827),
    [sym_terminal_priv] = ACTIONS(1827),
    [sym_tpadmin_priv] = ACTIONS(1827),
    [sym_user1_priv] = ACTIONS(1827),
    [sym_user2_priv] = ACTIONS(1827),
    [sym_user3_priv] = ACTIONS(1827),
    [sym_user4_priv] = ACTIONS(1827),
    [sym_usermanagement_priv] = ACTIONS(1827),
    [sym_valid_account] = ACTIONS(1827),
    [sym_err_name_invalid] = ACTIONS(1827),
    [sym_err_password_invalid] = ACTIONS(1827),
    [sym_err_invalid_account] = ACTIONS(1827),
    [sym_err_no_authorization] = ACTIONS(1827),
    [sym_err_invalid_parameter] = ACTIONS(1827),
    [sym_err_user_locked_out] = ACTIONS(1827),
    [sym_err_user_account_expired] = ACTIONS(1827),
    [sym_err_general_error] = ACTIONS(1827),
    [sym_err_authentication_error] = ACTIONS(1827),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1827),
    [sym_tls_validate_certificate] = ACTIONS(1827),
    [sym_tdata] = ACTIONS(1827),
    [sym_tchannel] = ACTIONS(1827),
    [sym_tlevel] = ACTIONS(1827),
    [sym_tbutton] = ACTIONS(1827),
    [sym_ttimeline] = ACTIONS(1827),
    [sym_tcustom] = ACTIONS(1827),
    [sym_url_struct] = ACTIONS(1827),
    [sym_dns_struct] = ACTIONS(1827),
    [sym_ip_address_struct] = ACTIONS(1827),
    [sym_dev_info_struct] = ACTIONS(1827),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1827),
    [sym_data_feed] = ACTIONS(1827),
    [sym_data_field] = ACTIONS(1827),
    [sym_data_record] = ACTIONS(1827),
    [sym_wc_data_feed] = ACTIONS(1827),
    [sym_wc_data_field] = ACTIONS(1827),
    [sym_wc_data_record] = ACTIONS(1827),
    [sym_last_login_info] = ACTIONS(1827),
    [sym____reserved___] = ACTIONS(1827),
    [sym_date] = ACTIONS(1827),
    [sym_day] = ACTIONS(1827),
    [sym_time] = ACTIONS(1827),
    [sym_ldate] = ACTIONS(1827),
    [sym_push_device] = ACTIONS(1827),
    [sym_push_channel] = ACTIONS(1827),
    [sym_push_devchan] = ACTIONS(1827),
    [sym_release_device] = ACTIONS(1827),
    [sym_release_channel] = ACTIONS(1827),
    [sym_release_devchan] = ACTIONS(1827),
    [sym_master_slot] = ACTIONS(1827),
    [sym_get_pulse_time] = ACTIONS(1827),
    [sym_get_timer] = ACTIONS(1827),
    [sym_master_sn] = ACTIONS(1827),
    [sym_system_number] = ACTIONS(1827),
    [sym_button] = ACTIONS(1827),
    [sym_dv_channel] = ACTIONS(1827),
    [sym_data] = ACTIONS(1827),
    [sym_level] = ACTIONS(1827),
    [sym_timeline] = ACTIONS(1827),
    [sym_channel] = ACTIONS(1827),
    [sym_custom] = ACTIONS(1827),
    [sym_atoi] = ACTIONS(1827),
    [sym_atol] = ACTIONS(1827),
    [sym_atof] = ACTIONS(1827),
    [sym_itoa] = ACTIONS(1827),
    [sym_itohex] = ACTIONS(1827),
    [sym_hextoi] = ACTIONS(1827),
    [sym_ftoa] = ACTIONS(1827),
    [sym_find_string] = ACTIONS(1827),
    [sym_left_string] = ACTIONS(1827),
    [sym_right_string] = ACTIONS(1827),
    [sym_mid_string] = ACTIONS(1827),
    [sym_remove_string] = ACTIONS(1827),
    [sym_get_buffer_string] = ACTIONS(1827),
    [sym_get_buffer_char] = ACTIONS(1827),
    [sym_get_multi_buffer_string] = ACTIONS(1827),
    [sym_length_string] = ACTIONS(1827),
    [sym_max_length_string] = ACTIONS(1827),
    [sym_set_length_string] = ACTIONS(1827),
    [sym_compare_string] = ACTIONS(1827),
    [sym_upper_string] = ACTIONS(1827),
    [sym_lower_string] = ACTIONS(1827),
    [sym_get_last] = ACTIONS(1827),
    [sym_date_to_day] = ACTIONS(1827),
    [sym_date_to_month] = ACTIONS(1827),
    [sym_date_to_year] = ACTIONS(1827),
    [sym_time_to_hour] = ACTIONS(1827),
    [sym_time_to_minute] = ACTIONS(1827),
    [sym_time_to_second] = ACTIONS(1827),
    [sym_day_of_week] = ACTIONS(1827),
    [sym_set_pulse_time] = ACTIONS(1827),
    [sym_set_timer] = ACTIONS(1827),
    [sym_file_open] = ACTIONS(1827),
    [sym_file_close] = ACTIONS(1827),
    [sym_file_read] = ACTIONS(1827),
    [sym_file_read_line] = ACTIONS(1827),
    [sym_file_write] = ACTIONS(1827),
    [sym_file_write_line] = ACTIONS(1827),
    [sym_file_copy] = ACTIONS(1827),
    [sym_file_delete] = ACTIONS(1827),
    [sym_file_rename] = ACTIONS(1827),
    [sym_file_dir] = ACTIONS(1827),
    [sym_file_getdir] = ACTIONS(1827),
    [sym_file_setdir] = ACTIONS(1827),
    [sym_file_create_dir] = ACTIONS(1827),
    [sym_file_remove_dir] = ACTIONS(1827),
    [sym_file_seek] = ACTIONS(1827),
    [sym_length_array] = ACTIONS(1827),
    [sym_max_length_array] = ACTIONS(1827),
    [sym_set_length_array] = ACTIONS(1827),
    [sym_ip_client_open] = ACTIONS(1827),
    [sym_ip_client_close] = ACTIONS(1827),
    [sym_ip_server_open] = ACTIONS(1827),
    [sym_ip_server_close] = ACTIONS(1827),
    [sym_ip_bound_client_open] = ACTIONS(1827),
    [sym_ip_mc_server_open] = ACTIONS(1827),
    [sym_ip_set_option] = ACTIONS(1827),
    [sym_ssh_client_open] = ACTIONS(1827),
    [sym_ssh_client_close] = ACTIONS(1827),
    [sym_tls_client_open] = ACTIONS(1827),
    [sym_tls_client_close] = ACTIONS(1827),
    [sym_do_push] = ACTIONS(1827),
    [sym_do_push_timed] = ACTIONS(1827),
    [sym_do_release] = ACTIONS(1827),
    [sym_device_id] = ACTIONS(1827),
    [sym_device_info] = ACTIONS(1827),
    [sym_device_id_string] = ACTIONS(1827),
    [sym_device_standby] = ACTIONS(1827),
    [sym_device_wake] = ACTIONS(1827),
    [sym_timeline_create] = ACTIONS(1827),
    [sym_timeline_kill] = ACTIONS(1827),
    [sym_timeline_reload] = ACTIONS(1827),
    [sym_timeline_pause] = ACTIONS(1827),
    [sym_timeline_restart] = ACTIONS(1827),
    [sym_timeline_set] = ACTIONS(1827),
    [sym_timeline_get] = ACTIONS(1827),
    [sym_timeline_active] = ACTIONS(1827),
    [sym_timeline_dynamic_id] = ACTIONS(1827),
    [sym_variable_to_string] = ACTIONS(1827),
    [sym_string_to_variable] = ACTIONS(1827),
    [sym_length_variable_to_string] = ACTIONS(1827),
    [sym_type_cast] = ACTIONS(1827),
    [sym_variable_to_xml] = ACTIONS(1827),
    [sym_xml_to_variable] = ACTIONS(1827),
    [sym_length_variable_to_xml] = ACTIONS(1827),
    [sym_redirect_string] = ACTIONS(1827),
    [sym_random_number] = ACTIONS(1827),
    [sym_get_unique_id] = ACTIONS(1827),
    [sym_get_system_number] = ACTIONS(1827),
    [sym_set_system_number] = ACTIONS(1827),
    [sym_get_serial_number] = ACTIONS(1827),
    [sym_get_master_build] = ACTIONS(1827),
    [sym_set_validation_code] = ACTIONS(1827),
    [sym_reboot] = ACTIONS(1827),
    [sym_load_duet_module] = ACTIONS(1827),
    [sym_unload_duet_module] = ACTIONS(1827),
    [sym_duet_mem_size_set] = ACTIONS(1827),
    [sym_duet_mem_size_get] = ACTIONS(1827),
    [sym_dynamic_application_device] = ACTIONS(1827),
    [sym_dynamic_polled_port] = ACTIONS(1827),
    [sym_static_port_binding] = ACTIONS(1827),
    [sym_static_ip_binding] = ACTIONS(1827),
    [sym_get_dns_list] = ACTIONS(1827),
    [sym_set_dns_list] = ACTIONS(1827),
    [sym_get_ip_address] = ACTIONS(1827),
    [sym_set_ip_address] = ACTIONS(1827),
    [sym_get_url_list] = ACTIONS(1827),
    [sym_add_url_entry] = ACTIONS(1827),
    [sym_delete_url_entry] = ACTIONS(1827),
    [sym_abs_value] = ACTIONS(1827),
    [sym_max_value] = ACTIONS(1827),
    [sym_min_value] = ACTIONS(1827),
    [sym_exp_value] = ACTIONS(1827),
    [sym_log_value] = ACTIONS(1827),
    [sym_log10_value] = ACTIONS(1827),
    [sym_power_value] = ACTIONS(1827),
    [sym_sqrt_value] = ACTIONS(1827),
    [sym_raw_be] = ACTIONS(1827),
    [sym_raw_le] = ACTIONS(1827),
    [sym_internal_queue_size_set] = ACTIONS(1827),
    [sym_internal_queue_size_get] = ACTIONS(1827),
    [sym_internal_threshold_set] = ACTIONS(1827),
    [sym_internal_threshold_get] = ACTIONS(1827),
    [sym_rebuilt_event] = ACTIONS(1827),
    [sym_do_custom_event] = ACTIONS(1827),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1827),
    [sym_clkmgr_set_clk_source] = ACTIONS(1827),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1827),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1827),
    [sym_clkmgr_get_timezone] = ACTIONS(1827),
    [sym_clkmgr_set_timezone] = ACTIONS(1827),
    [sym_clkmgr_get_resync_period] = ACTIONS(1827),
    [sym_clkmgr_set_resync_period] = ACTIONS(1827),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1827),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1827),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1827),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1827),
    [sym_clkmgr_get_timeservers] = ACTIONS(1827),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1827),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1827),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1827),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1827),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1827),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1827),
    [sym_set_log_level] = ACTIONS(1827),
    [sym_get_log_level] = ACTIONS(1827),
    [sym_amx_log] = ACTIONS(1827),
    [sym_smtp_server_config_set] = ACTIONS(1827),
    [sym_smtp_server_config_get] = ACTIONS(1827),
    [sym_smtp_send] = ACTIONS(1827),
    [sym_validate_netlinx_account] = ACTIONS(1827),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1827),
    [sym_authenticate_certificate] = ACTIONS(1827),
    [sym_audit_netlinx_session_event] = ACTIONS(1827),
    [sym_audit_netlinx_generic_event] = ACTIONS(1827),
    [sym_get_available_flash_disk_space] = ACTIONS(1827),
    [sym_get_max_flash_disk_space] = ACTIONS(1827),
    [sym_data_create_feed] = ACTIONS(1827),
    [sym_data_delete_feed] = ACTIONS(1827),
    [sym_data_publish_feed] = ACTIONS(1827),
    [sym_data_get_published_feed] = ACTIONS(1827),
    [sym_data_add_record] = ACTIONS(1827),
    [sym_data_get_event_record] = ACTIONS(1827),
    [sym__wc_data_create_feed] = ACTIONS(1827),
    [sym__wc_data_add_record] = ACTIONS(1827),
    [sym__wc_data_get_event_record] = ACTIONS(1827),
    [sym_format] = ACTIONS(1827),
    [sym_astro_clock] = ACTIONS(1827),
    [sym_set_outdoor_temperature] = ACTIONS(1827),
    [sym_set_virtual_level_count] = ACTIONS(1827),
    [sym_set_virtual_channel_count] = ACTIONS(1827),
    [sym_set_virtual_port_count] = ACTIONS(1827),
    [sym__wc_encode_int] = ACTIONS(1827),
    [sym__wc_decode_int] = ACTIONS(1827),
    [sym__wc_int] = ACTIONS(1827),
    [sym__wc_to_ch_int] = ACTIONS(1827),
    [sym__ch_to_wc_int] = ACTIONS(1827),
    [sym__wc_find_string_int] = ACTIONS(1827),
    [sym__wc_left_string_int] = ACTIONS(1827),
    [sym__wc_lower_string_int] = ACTIONS(1827),
    [sym__wc_mid_string_int] = ACTIONS(1827),
    [sym__wc_remove_string_int] = ACTIONS(1827),
    [sym__wc_right_string_int] = ACTIONS(1827),
    [sym__wc_upper_string_int] = ACTIONS(1827),
    [sym__wc_compare_string_int] = ACTIONS(1827),
    [sym__wc_get_buffer_char_int] = ACTIONS(1827),
    [sym__wc_get_buffer_string_int] = ACTIONS(1827),
    [sym__wc_concat_string_int] = ACTIONS(1827),
    [sym__wc_file_open_int] = ACTIONS(1827),
    [sym__wc_file_close_int] = ACTIONS(1827),
    [sym__wc_file_read_int] = ACTIONS(1827),
    [sym__wc_file_read_line_int] = ACTIONS(1827),
    [sym__wc_file_write_int] = ACTIONS(1827),
    [sym__wc_file_write_line_int] = ACTIONS(1827),
    [aux_sym_preproc_include_token1] = ACTIONS(1827),
    [aux_sym_preproc_define_token1] = ACTIONS(1829),
    [aux_sym_preproc_warn_token1] = ACTIONS(1829),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1829),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1829),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1829),
    [anon_sym_LBRACK] = ACTIONS(1829),
    [anon_sym_LBRACE] = ACTIONS(1829),
    [anon_sym_RBRACE] = ACTIONS(1829),
    [anon_sym_LPAREN] = ACTIONS(1827),
    [anon_sym_BANG] = ACTIONS(1829),
    [anon_sym_TILDE] = ACTIONS(1829),
    [anon_sym_DASH] = ACTIONS(1827),
    [anon_sym_PLUS] = ACTIONS(1827),
    [anon_sym_DASH_DASH] = ACTIONS(1829),
    [anon_sym_PLUS_PLUS] = ACTIONS(1829),
    [anon_sym_DQUOTE] = ACTIONS(1829),
    [anon_sym_SQUOTE] = ACTIONS(1829),
    [sym_number_literal] = ACTIONS(1829),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(495)] = {
    [sym_identifier] = ACTIONS(1739),
    [sym_define_call_keyword] = ACTIONS(1739),
    [sym_define_function_keyword] = ACTIONS(1739),
    [sym_define_library_function_keyword] = ACTIONS(1739),
    [sym_define_module_keyword] = ACTIONS(1739),
    [sym_char_keyword] = ACTIONS(1739),
    [sym_widechar_keyword] = ACTIONS(1739),
    [sym_integer_keyword] = ACTIONS(1739),
    [sym_sinteger_keyword] = ACTIONS(1739),
    [sym_long_keyword] = ACTIONS(1739),
    [sym_slong_keyword] = ACTIONS(1739),
    [sym_float_keyword] = ACTIONS(1739),
    [sym_double_keyword] = ACTIONS(1739),
    [sym_variant_keyword] = ACTIONS(1739),
    [sym_variantarray_keyword] = ACTIONS(1739),
    [sym_local_var_keyword] = ACTIONS(1739),
    [sym_stack_var_keyword] = ACTIONS(1739),
    [sym_constant_keyword] = ACTIONS(1739),
    [sym_volatile_keyword] = ACTIONS(1739),
    [sym_non_volatile_keyword] = ACTIONS(1739),
    [sym_persistent_keyword] = ACTIONS(1739),
    [sym_dev_keyword] = ACTIONS(1739),
    [sym_devlev_keyword] = ACTIONS(1739),
    [sym_devchan_keyword] = ACTIONS(1739),
    [sym_if_keyword] = ACTIONS(1739),
    [sym_else_keyword] = ACTIONS(1739),
    [sym_switch_keyword] = ACTIONS(1739),
    [sym_case_keyword] = ACTIONS(1739),
    [sym_default_keyword] = ACTIONS(1739),
    [sym_while_keyword] = ACTIONS(1739),
    [sym_for_keyword] = ACTIONS(1739),
    [sym_break_keyword] = ACTIONS(1739),
    [sym_continue_keyword] = ACTIONS(1739),
    [sym_return_keyword] = ACTIONS(1739),
    [sym_select_keyword] = ACTIONS(1739),
    [sym_active_keyword] = ACTIONS(1739),
    [sym_struct_keyword] = ACTIONS(1739),
    [sym_structure_keyword] = ACTIONS(1739),
    [aux_sym_on_keyword_token1] = ACTIONS(1739),
    [aux_sym_off_keyword_token1] = ACTIONS(1739),
    [sym_send_level_keyword] = ACTIONS(1739),
    [sym_send_string_keyword] = ACTIONS(1739),
    [sym_send_command_keyword] = ACTIONS(1739),
    [sym_clear_buffer_keyword] = ACTIONS(1739),
    [sym_create_buffer_keyword] = ACTIONS(1739),
    [sym_create_multi_buffer_keyword] = ACTIONS(1739),
    [sym_call_keyword] = ACTIONS(1739),
    [sym_system_call_keyword] = ACTIONS(1739),
    [sym_devchan_to_keyword] = ACTIONS(1739),
    [sym_devchan_min_to_keyword] = ACTIONS(1739),
    [sym_devchan_total_off_keyword] = ACTIONS(1739),
    [sym_devchan_pulse_keyword] = ACTIONS(1739),
    [sym_wait_keyword] = ACTIONS(1739),
    [sym_cancel_wait_keyword] = ACTIONS(1739),
    [sym_wait_until_keyword] = ACTIONS(1739),
    [sym_cancel_wait_until_keyword] = ACTIONS(1739),
    [sym_cancel_all_wait_keyword] = ACTIONS(1739),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1739),
    [sym_not] = ACTIONS(1739),
    [sym_bnot] = ACTIONS(1739),
    [sym___file__] = ACTIONS(1739),
    [sym___date__] = ACTIONS(1739),
    [sym___ldate__] = ACTIONS(1739),
    [sym___time__] = ACTIONS(1739),
    [sym___line__] = ACTIONS(1739),
    [sym___name__] = ACTIONS(1739),
    [sym___netlinx__] = ACTIONS(1739),
    [sym_netlinx_axi_version] = ACTIONS(1739),
    [sym_url_flg_tcp] = ACTIONS(1739),
    [sym_url_flg_accinfopresent] = ACTIONS(1739),
    [sym_url_flg_temp] = ACTIONS(1739),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1739),
    [sym_url_flg_stat_mask] = ACTIONS(1739),
    [sym_url_flg_stat_lookup] = ACTIONS(1739),
    [sym_url_flg_stat_connecting] = ACTIONS(1739),
    [sym_url_flg_stat_waiting] = ACTIONS(1739),
    [sym_url_flg_stat_connected] = ACTIONS(1739),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1739),
    [sym_first_virtual_device] = ACTIONS(1739),
    [sym_first_local_port] = ACTIONS(1739),
    [sym_dynamic_virtual_device] = ACTIONS(1739),
    [sym_timeline_once] = ACTIONS(1739),
    [sym_timeline_repeat] = ACTIONS(1739),
    [sym_timeline_absolute] = ACTIONS(1739),
    [sym_timeline_relative] = ACTIONS(1739),
    [sym_do_push_timed_infinite] = ACTIONS(1739),
    [sym_source_type_no_address] = ACTIONS(1739),
    [sym_source_type_neuron_id] = ACTIONS(1739),
    [sym_source_type_ip_address] = ACTIONS(1739),
    [sym_source_type_axlink] = ACTIONS(1739),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1739),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1739),
    [sym_source_type_ip_socket_address] = ACTIONS(1739),
    [sym_source_type_rs232] = ACTIONS(1739),
    [sym_source_type_internal] = ACTIONS(1739),
    [sym_source_type_mac_address] = ACTIONS(1739),
    [sym_source_type_ipv4_port] = ACTIONS(1739),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1739),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1739),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1739),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1739),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1739),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1739),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1739),
    [sym_true] = ACTIONS(1739),
    [sym_false] = ACTIONS(1739),
    [sym_file_read_only] = ACTIONS(1739),
    [sym_file_rw_new] = ACTIONS(1739),
    [sym_file_rw_append] = ACTIONS(1739),
    [sym_ip_tcp] = ACTIONS(1739),
    [sym_ip_udp] = ACTIONS(1739),
    [sym_ip_udp_2way] = ACTIONS(1739),
    [sym_xml_encode_types] = ACTIONS(1739),
    [sym_xml_encode_char_as_list] = ACTIONS(1739),
    [sym_xml_encode_le] = ACTIONS(1739),
    [sym_xml_decode_types] = ACTIONS(1739),
    [sym_xml_decode_no_preserve] = ACTIONS(1739),
    [sym_duet_dev_type_display_device] = ACTIONS(1739),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1739),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1739),
    [sym_duet_dev_type_io_device] = ACTIONS(1739),
    [sym_duet_dev_type_relay_device] = ACTIONS(1739),
    [sym_duet_dev_type_ups] = ACTIONS(1739),
    [sym_duet_dev_type_amplifier] = ACTIONS(1739),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1739),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1739),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1739),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1739),
    [sym_duet_dev_type_camera] = ACTIONS(1739),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1739),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1739),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1739),
    [sym_duet_dev_type_dss] = ACTIONS(1739),
    [sym_duet_dev_type_dvr] = ACTIONS(1739),
    [sym_duet_dev_type_disc_device] = ACTIONS(1739),
    [sym_duet_dev_type_document_camera] = ACTIONS(1739),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1739),
    [sym_duet_dev_type_hvac] = ACTIONS(1739),
    [sym_duet_dev_type_keypad] = ACTIONS(1739),
    [sym_duet_dev_type_light] = ACTIONS(1739),
    [sym_duet_dev_type_monitor] = ACTIONS(1739),
    [sym_duet_dev_type_motor] = ACTIONS(1739),
    [sym_duet_dev_type_multi_window] = ACTIONS(1739),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1739),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1739),
    [sym_duet_dev_type_receiver] = ACTIONS(1739),
    [sym_duet_dev_type_security_system] = ACTIONS(1739),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1739),
    [sym_duet_dev_type_settop_box] = ACTIONS(1739),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1739),
    [sym_duet_dev_type_switcher] = ACTIONS(1739),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1739),
    [sym_duet_dev_type_tv] = ACTIONS(1739),
    [sym_duet_dev_type_utility] = ACTIONS(1739),
    [sym_duet_dev_type_vcr] = ACTIONS(1739),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1739),
    [sym_duet_dev_type_video_processor] = ACTIONS(1739),
    [sym_duet_dev_type_video_projector] = ACTIONS(1739),
    [sym_duet_dev_type_video_wall] = ACTIONS(1739),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1739),
    [sym_duet_dev_type_weather] = ACTIONS(1739),
    [sym_duet_dev_not_polled] = ACTIONS(1739),
    [sym_duet_dev_polled] = ACTIONS(1739),
    [sym_clkmgr_mode_network] = ACTIONS(1739),
    [sym_clkmgr_mode_standalone] = ACTIONS(1739),
    [sym_normal_standby] = ACTIONS(1739),
    [sym_normal_wake] = ACTIONS(1739),
    [sym_ip_multicast_ttl_option] = ACTIONS(1739),
    [sym_ip_ttl_subnet] = ACTIONS(1739),
    [sym_ip_ttl_site] = ACTIONS(1739),
    [sym_ip_ttl_region] = ACTIONS(1739),
    [sym_ip_ttl_continent] = ACTIONS(1739),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1739),
    [sym_ip_nodelay_on] = ACTIONS(1739),
    [sym_ip_nodelay_off] = ACTIONS(1739),
    [sym_amx_error] = ACTIONS(1739),
    [sym_amx_warning] = ACTIONS(1739),
    [sym_amx_info] = ACTIONS(1739),
    [sym_amx_debug] = ACTIONS(1739),
    [sym_netlinx_login_success] = ACTIONS(1739),
    [sym_netlinx_login_fail] = ACTIONS(1739),
    [sym_netlinx_logout] = ACTIONS(1739),
    [sym_null_str] = ACTIONS(1739),
    [sym_smtp_address] = ACTIONS(1739),
    [sym_smtp_port_number] = ACTIONS(1739),
    [sym_smtp_username] = ACTIONS(1739),
    [sym_smtp_password] = ACTIONS(1739),
    [sym_smtp_require_tls] = ACTIONS(1739),
    [sym_smtp_from] = ACTIONS(1739),
    [sym_smtp_tls_true] = ACTIONS(1739),
    [sym_smtp_tls_false] = ACTIONS(1739),
    [sym_listview_on_row_select_event] = ACTIONS(1739),
    [sym_data_structure_unknown] = ACTIONS(1739),
    [sym_data_structure_datafeed] = ACTIONS(1739),
    [sym_data_structure_datarecordset] = ACTIONS(1739),
    [sym_data_structure_datarecord] = ACTIONS(1739),
    [sym_data_structure_datafield] = ACTIONS(1739),
    [sym_data_max_content_field_count] = ACTIONS(1739),
    [sym_data_max_metadata_field_count] = ACTIONS(1739),
    [sym_data_max_id_length] = ACTIONS(1739),
    [sym_data_max_name_length] = ACTIONS(1739),
    [sym_data_max_description_length] = ACTIONS(1739),
    [sym_data_max_type_length] = ACTIONS(1739),
    [sym_data_max_format_length] = ACTIONS(1739),
    [sym_data_max_source_length] = ACTIONS(1739),
    [sym_data_max_value_length] = ACTIONS(1739),
    [sym_data_max_label_length] = ACTIONS(1739),
    [sym_data_max_url_length] = ACTIONS(1739),
    [sym_max_last_login_info_length] = ACTIONS(1739),
    [sym_data_type_unknown] = ACTIONS(1739),
    [sym_data_type_string] = ACTIONS(1739),
    [sym_data_type_datetime] = ACTIONS(1739),
    [sym_data_type_date] = ACTIONS(1739),
    [sym_data_type_time] = ACTIONS(1739),
    [sym_data_type_image] = ACTIONS(1739),
    [sym_data_format_url] = ACTIONS(1739),
    [sym_data_format_phone] = ACTIONS(1739),
    [sym_data_format_email] = ACTIONS(1739),
    [sym_data_format_iso8601] = ACTIONS(1739),
    [sym_auditlog_priv] = ACTIONS(1739),
    [sym_configuration_priv] = ACTIONS(1739),
    [sym_deviceconfig_priv] = ACTIONS(1739),
    [sym_ftp_priv] = ACTIONS(1739),
    [sym_http_priv] = ACTIONS(1739),
    [sym_networkconfig_priv] = ACTIONS(1739),
    [sym_programport_priv] = ACTIONS(1739),
    [sym_remoteui_priv] = ACTIONS(1739),
    [sym_securitycontrol_priv] = ACTIONS(1739),
    [sym_softwaremanagement_priv] = ACTIONS(1739),
    [sym_terminal_priv] = ACTIONS(1739),
    [sym_tpadmin_priv] = ACTIONS(1739),
    [sym_user1_priv] = ACTIONS(1739),
    [sym_user2_priv] = ACTIONS(1739),
    [sym_user3_priv] = ACTIONS(1739),
    [sym_user4_priv] = ACTIONS(1739),
    [sym_usermanagement_priv] = ACTIONS(1739),
    [sym_valid_account] = ACTIONS(1739),
    [sym_err_name_invalid] = ACTIONS(1739),
    [sym_err_password_invalid] = ACTIONS(1739),
    [sym_err_invalid_account] = ACTIONS(1739),
    [sym_err_no_authorization] = ACTIONS(1739),
    [sym_err_invalid_parameter] = ACTIONS(1739),
    [sym_err_user_locked_out] = ACTIONS(1739),
    [sym_err_user_account_expired] = ACTIONS(1739),
    [sym_err_general_error] = ACTIONS(1739),
    [sym_err_authentication_error] = ACTIONS(1739),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1739),
    [sym_tls_validate_certificate] = ACTIONS(1739),
    [sym_tdata] = ACTIONS(1739),
    [sym_tchannel] = ACTIONS(1739),
    [sym_tlevel] = ACTIONS(1739),
    [sym_tbutton] = ACTIONS(1739),
    [sym_ttimeline] = ACTIONS(1739),
    [sym_tcustom] = ACTIONS(1739),
    [sym_url_struct] = ACTIONS(1739),
    [sym_dns_struct] = ACTIONS(1739),
    [sym_ip_address_struct] = ACTIONS(1739),
    [sym_dev_info_struct] = ACTIONS(1739),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1739),
    [sym_data_feed] = ACTIONS(1739),
    [sym_data_field] = ACTIONS(1739),
    [sym_data_record] = ACTIONS(1739),
    [sym_wc_data_feed] = ACTIONS(1739),
    [sym_wc_data_field] = ACTIONS(1739),
    [sym_wc_data_record] = ACTIONS(1739),
    [sym_last_login_info] = ACTIONS(1739),
    [sym____reserved___] = ACTIONS(1739),
    [sym_date] = ACTIONS(1739),
    [sym_day] = ACTIONS(1739),
    [sym_time] = ACTIONS(1739),
    [sym_ldate] = ACTIONS(1739),
    [sym_push_device] = ACTIONS(1739),
    [sym_push_channel] = ACTIONS(1739),
    [sym_push_devchan] = ACTIONS(1739),
    [sym_release_device] = ACTIONS(1739),
    [sym_release_channel] = ACTIONS(1739),
    [sym_release_devchan] = ACTIONS(1739),
    [sym_master_slot] = ACTIONS(1739),
    [sym_get_pulse_time] = ACTIONS(1739),
    [sym_get_timer] = ACTIONS(1739),
    [sym_master_sn] = ACTIONS(1739),
    [sym_system_number] = ACTIONS(1739),
    [sym_button] = ACTIONS(1739),
    [sym_dv_channel] = ACTIONS(1739),
    [sym_data] = ACTIONS(1739),
    [sym_level] = ACTIONS(1739),
    [sym_timeline] = ACTIONS(1739),
    [sym_channel] = ACTIONS(1739),
    [sym_custom] = ACTIONS(1739),
    [sym_atoi] = ACTIONS(1739),
    [sym_atol] = ACTIONS(1739),
    [sym_atof] = ACTIONS(1739),
    [sym_itoa] = ACTIONS(1739),
    [sym_itohex] = ACTIONS(1739),
    [sym_hextoi] = ACTIONS(1739),
    [sym_ftoa] = ACTIONS(1739),
    [sym_find_string] = ACTIONS(1739),
    [sym_left_string] = ACTIONS(1739),
    [sym_right_string] = ACTIONS(1739),
    [sym_mid_string] = ACTIONS(1739),
    [sym_remove_string] = ACTIONS(1739),
    [sym_get_buffer_string] = ACTIONS(1739),
    [sym_get_buffer_char] = ACTIONS(1739),
    [sym_get_multi_buffer_string] = ACTIONS(1739),
    [sym_length_string] = ACTIONS(1739),
    [sym_max_length_string] = ACTIONS(1739),
    [sym_set_length_string] = ACTIONS(1739),
    [sym_compare_string] = ACTIONS(1739),
    [sym_upper_string] = ACTIONS(1739),
    [sym_lower_string] = ACTIONS(1739),
    [sym_get_last] = ACTIONS(1739),
    [sym_date_to_day] = ACTIONS(1739),
    [sym_date_to_month] = ACTIONS(1739),
    [sym_date_to_year] = ACTIONS(1739),
    [sym_time_to_hour] = ACTIONS(1739),
    [sym_time_to_minute] = ACTIONS(1739),
    [sym_time_to_second] = ACTIONS(1739),
    [sym_day_of_week] = ACTIONS(1739),
    [sym_set_pulse_time] = ACTIONS(1739),
    [sym_set_timer] = ACTIONS(1739),
    [sym_file_open] = ACTIONS(1739),
    [sym_file_close] = ACTIONS(1739),
    [sym_file_read] = ACTIONS(1739),
    [sym_file_read_line] = ACTIONS(1739),
    [sym_file_write] = ACTIONS(1739),
    [sym_file_write_line] = ACTIONS(1739),
    [sym_file_copy] = ACTIONS(1739),
    [sym_file_delete] = ACTIONS(1739),
    [sym_file_rename] = ACTIONS(1739),
    [sym_file_dir] = ACTIONS(1739),
    [sym_file_getdir] = ACTIONS(1739),
    [sym_file_setdir] = ACTIONS(1739),
    [sym_file_create_dir] = ACTIONS(1739),
    [sym_file_remove_dir] = ACTIONS(1739),
    [sym_file_seek] = ACTIONS(1739),
    [sym_length_array] = ACTIONS(1739),
    [sym_max_length_array] = ACTIONS(1739),
    [sym_set_length_array] = ACTIONS(1739),
    [sym_ip_client_open] = ACTIONS(1739),
    [sym_ip_client_close] = ACTIONS(1739),
    [sym_ip_server_open] = ACTIONS(1739),
    [sym_ip_server_close] = ACTIONS(1739),
    [sym_ip_bound_client_open] = ACTIONS(1739),
    [sym_ip_mc_server_open] = ACTIONS(1739),
    [sym_ip_set_option] = ACTIONS(1739),
    [sym_ssh_client_open] = ACTIONS(1739),
    [sym_ssh_client_close] = ACTIONS(1739),
    [sym_tls_client_open] = ACTIONS(1739),
    [sym_tls_client_close] = ACTIONS(1739),
    [sym_do_push] = ACTIONS(1739),
    [sym_do_push_timed] = ACTIONS(1739),
    [sym_do_release] = ACTIONS(1739),
    [sym_device_id] = ACTIONS(1739),
    [sym_device_info] = ACTIONS(1739),
    [sym_device_id_string] = ACTIONS(1739),
    [sym_device_standby] = ACTIONS(1739),
    [sym_device_wake] = ACTIONS(1739),
    [sym_timeline_create] = ACTIONS(1739),
    [sym_timeline_kill] = ACTIONS(1739),
    [sym_timeline_reload] = ACTIONS(1739),
    [sym_timeline_pause] = ACTIONS(1739),
    [sym_timeline_restart] = ACTIONS(1739),
    [sym_timeline_set] = ACTIONS(1739),
    [sym_timeline_get] = ACTIONS(1739),
    [sym_timeline_active] = ACTIONS(1739),
    [sym_timeline_dynamic_id] = ACTIONS(1739),
    [sym_variable_to_string] = ACTIONS(1739),
    [sym_string_to_variable] = ACTIONS(1739),
    [sym_length_variable_to_string] = ACTIONS(1739),
    [sym_type_cast] = ACTIONS(1739),
    [sym_variable_to_xml] = ACTIONS(1739),
    [sym_xml_to_variable] = ACTIONS(1739),
    [sym_length_variable_to_xml] = ACTIONS(1739),
    [sym_redirect_string] = ACTIONS(1739),
    [sym_random_number] = ACTIONS(1739),
    [sym_get_unique_id] = ACTIONS(1739),
    [sym_get_system_number] = ACTIONS(1739),
    [sym_set_system_number] = ACTIONS(1739),
    [sym_get_serial_number] = ACTIONS(1739),
    [sym_get_master_build] = ACTIONS(1739),
    [sym_set_validation_code] = ACTIONS(1739),
    [sym_reboot] = ACTIONS(1739),
    [sym_load_duet_module] = ACTIONS(1739),
    [sym_unload_duet_module] = ACTIONS(1739),
    [sym_duet_mem_size_set] = ACTIONS(1739),
    [sym_duet_mem_size_get] = ACTIONS(1739),
    [sym_dynamic_application_device] = ACTIONS(1739),
    [sym_dynamic_polled_port] = ACTIONS(1739),
    [sym_static_port_binding] = ACTIONS(1739),
    [sym_static_ip_binding] = ACTIONS(1739),
    [sym_get_dns_list] = ACTIONS(1739),
    [sym_set_dns_list] = ACTIONS(1739),
    [sym_get_ip_address] = ACTIONS(1739),
    [sym_set_ip_address] = ACTIONS(1739),
    [sym_get_url_list] = ACTIONS(1739),
    [sym_add_url_entry] = ACTIONS(1739),
    [sym_delete_url_entry] = ACTIONS(1739),
    [sym_abs_value] = ACTIONS(1739),
    [sym_max_value] = ACTIONS(1739),
    [sym_min_value] = ACTIONS(1739),
    [sym_exp_value] = ACTIONS(1739),
    [sym_log_value] = ACTIONS(1739),
    [sym_log10_value] = ACTIONS(1739),
    [sym_power_value] = ACTIONS(1739),
    [sym_sqrt_value] = ACTIONS(1739),
    [sym_raw_be] = ACTIONS(1739),
    [sym_raw_le] = ACTIONS(1739),
    [sym_internal_queue_size_set] = ACTIONS(1739),
    [sym_internal_queue_size_get] = ACTIONS(1739),
    [sym_internal_threshold_set] = ACTIONS(1739),
    [sym_internal_threshold_get] = ACTIONS(1739),
    [sym_rebuilt_event] = ACTIONS(1739),
    [sym_do_custom_event] = ACTIONS(1739),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1739),
    [sym_clkmgr_set_clk_source] = ACTIONS(1739),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1739),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1739),
    [sym_clkmgr_get_timezone] = ACTIONS(1739),
    [sym_clkmgr_set_timezone] = ACTIONS(1739),
    [sym_clkmgr_get_resync_period] = ACTIONS(1739),
    [sym_clkmgr_set_resync_period] = ACTIONS(1739),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1739),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1739),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1739),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1739),
    [sym_clkmgr_get_timeservers] = ACTIONS(1739),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1739),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1739),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1739),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1739),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1739),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1739),
    [sym_set_log_level] = ACTIONS(1739),
    [sym_get_log_level] = ACTIONS(1739),
    [sym_amx_log] = ACTIONS(1739),
    [sym_smtp_server_config_set] = ACTIONS(1739),
    [sym_smtp_server_config_get] = ACTIONS(1739),
    [sym_smtp_send] = ACTIONS(1739),
    [sym_validate_netlinx_account] = ACTIONS(1739),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1739),
    [sym_authenticate_certificate] = ACTIONS(1739),
    [sym_audit_netlinx_session_event] = ACTIONS(1739),
    [sym_audit_netlinx_generic_event] = ACTIONS(1739),
    [sym_get_available_flash_disk_space] = ACTIONS(1739),
    [sym_get_max_flash_disk_space] = ACTIONS(1739),
    [sym_data_create_feed] = ACTIONS(1739),
    [sym_data_delete_feed] = ACTIONS(1739),
    [sym_data_publish_feed] = ACTIONS(1739),
    [sym_data_get_published_feed] = ACTIONS(1739),
    [sym_data_add_record] = ACTIONS(1739),
    [sym_data_get_event_record] = ACTIONS(1739),
    [sym__wc_data_create_feed] = ACTIONS(1739),
    [sym__wc_data_add_record] = ACTIONS(1739),
    [sym__wc_data_get_event_record] = ACTIONS(1739),
    [sym_format] = ACTIONS(1739),
    [sym_astro_clock] = ACTIONS(1739),
    [sym_set_outdoor_temperature] = ACTIONS(1739),
    [sym_set_virtual_level_count] = ACTIONS(1739),
    [sym_set_virtual_channel_count] = ACTIONS(1739),
    [sym_set_virtual_port_count] = ACTIONS(1739),
    [sym__wc_encode_int] = ACTIONS(1739),
    [sym__wc_decode_int] = ACTIONS(1739),
    [sym__wc_int] = ACTIONS(1739),
    [sym__wc_to_ch_int] = ACTIONS(1739),
    [sym__ch_to_wc_int] = ACTIONS(1739),
    [sym__wc_find_string_int] = ACTIONS(1739),
    [sym__wc_left_string_int] = ACTIONS(1739),
    [sym__wc_lower_string_int] = ACTIONS(1739),
    [sym__wc_mid_string_int] = ACTIONS(1739),
    [sym__wc_remove_string_int] = ACTIONS(1739),
    [sym__wc_right_string_int] = ACTIONS(1739),
    [sym__wc_upper_string_int] = ACTIONS(1739),
    [sym__wc_compare_string_int] = ACTIONS(1739),
    [sym__wc_get_buffer_char_int] = ACTIONS(1739),
    [sym__wc_get_buffer_string_int] = ACTIONS(1739),
    [sym__wc_concat_string_int] = ACTIONS(1739),
    [sym__wc_file_open_int] = ACTIONS(1739),
    [sym__wc_file_close_int] = ACTIONS(1739),
    [sym__wc_file_read_int] = ACTIONS(1739),
    [sym__wc_file_read_line_int] = ACTIONS(1739),
    [sym__wc_file_write_int] = ACTIONS(1739),
    [sym__wc_file_write_line_int] = ACTIONS(1739),
    [aux_sym_preproc_include_token1] = ACTIONS(1739),
    [aux_sym_preproc_define_token1] = ACTIONS(1741),
    [aux_sym_preproc_warn_token1] = ACTIONS(1741),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1741),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1741),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1741),
    [anon_sym_LBRACK] = ACTIONS(1741),
    [anon_sym_LBRACE] = ACTIONS(1741),
    [anon_sym_RBRACE] = ACTIONS(1741),
    [anon_sym_LPAREN] = ACTIONS(1739),
    [anon_sym_BANG] = ACTIONS(1741),
    [anon_sym_TILDE] = ACTIONS(1741),
    [anon_sym_DASH] = ACTIONS(1739),
    [anon_sym_PLUS] = ACTIONS(1739),
    [anon_sym_DASH_DASH] = ACTIONS(1741),
    [anon_sym_PLUS_PLUS] = ACTIONS(1741),
    [anon_sym_DQUOTE] = ACTIONS(1741),
    [anon_sym_SQUOTE] = ACTIONS(1741),
    [sym_number_literal] = ACTIONS(1741),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(496)] = {
    [sym_identifier] = ACTIONS(1847),
    [sym_define_call_keyword] = ACTIONS(1847),
    [sym_define_function_keyword] = ACTIONS(1847),
    [sym_define_library_function_keyword] = ACTIONS(1847),
    [sym_define_module_keyword] = ACTIONS(1847),
    [sym_char_keyword] = ACTIONS(1847),
    [sym_widechar_keyword] = ACTIONS(1847),
    [sym_integer_keyword] = ACTIONS(1847),
    [sym_sinteger_keyword] = ACTIONS(1847),
    [sym_long_keyword] = ACTIONS(1847),
    [sym_slong_keyword] = ACTIONS(1847),
    [sym_float_keyword] = ACTIONS(1847),
    [sym_double_keyword] = ACTIONS(1847),
    [sym_variant_keyword] = ACTIONS(1847),
    [sym_variantarray_keyword] = ACTIONS(1847),
    [sym_local_var_keyword] = ACTIONS(1847),
    [sym_stack_var_keyword] = ACTIONS(1847),
    [sym_constant_keyword] = ACTIONS(1847),
    [sym_volatile_keyword] = ACTIONS(1847),
    [sym_non_volatile_keyword] = ACTIONS(1847),
    [sym_persistent_keyword] = ACTIONS(1847),
    [sym_dev_keyword] = ACTIONS(1847),
    [sym_devlev_keyword] = ACTIONS(1847),
    [sym_devchan_keyword] = ACTIONS(1847),
    [sym_if_keyword] = ACTIONS(1847),
    [sym_else_keyword] = ACTIONS(1847),
    [sym_switch_keyword] = ACTIONS(1847),
    [sym_case_keyword] = ACTIONS(1847),
    [sym_default_keyword] = ACTIONS(1847),
    [sym_while_keyword] = ACTIONS(1847),
    [sym_for_keyword] = ACTIONS(1847),
    [sym_break_keyword] = ACTIONS(1847),
    [sym_continue_keyword] = ACTIONS(1847),
    [sym_return_keyword] = ACTIONS(1847),
    [sym_select_keyword] = ACTIONS(1847),
    [sym_active_keyword] = ACTIONS(1847),
    [sym_struct_keyword] = ACTIONS(1847),
    [sym_structure_keyword] = ACTIONS(1847),
    [aux_sym_on_keyword_token1] = ACTIONS(1847),
    [aux_sym_off_keyword_token1] = ACTIONS(1847),
    [sym_send_level_keyword] = ACTIONS(1847),
    [sym_send_string_keyword] = ACTIONS(1847),
    [sym_send_command_keyword] = ACTIONS(1847),
    [sym_clear_buffer_keyword] = ACTIONS(1847),
    [sym_create_buffer_keyword] = ACTIONS(1847),
    [sym_create_multi_buffer_keyword] = ACTIONS(1847),
    [sym_call_keyword] = ACTIONS(1847),
    [sym_system_call_keyword] = ACTIONS(1847),
    [sym_devchan_to_keyword] = ACTIONS(1847),
    [sym_devchan_min_to_keyword] = ACTIONS(1847),
    [sym_devchan_total_off_keyword] = ACTIONS(1847),
    [sym_devchan_pulse_keyword] = ACTIONS(1847),
    [sym_wait_keyword] = ACTIONS(1847),
    [sym_cancel_wait_keyword] = ACTIONS(1847),
    [sym_wait_until_keyword] = ACTIONS(1847),
    [sym_cancel_wait_until_keyword] = ACTIONS(1847),
    [sym_cancel_all_wait_keyword] = ACTIONS(1847),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1847),
    [sym_not] = ACTIONS(1847),
    [sym_bnot] = ACTIONS(1847),
    [sym___file__] = ACTIONS(1847),
    [sym___date__] = ACTIONS(1847),
    [sym___ldate__] = ACTIONS(1847),
    [sym___time__] = ACTIONS(1847),
    [sym___line__] = ACTIONS(1847),
    [sym___name__] = ACTIONS(1847),
    [sym___netlinx__] = ACTIONS(1847),
    [sym_netlinx_axi_version] = ACTIONS(1847),
    [sym_url_flg_tcp] = ACTIONS(1847),
    [sym_url_flg_accinfopresent] = ACTIONS(1847),
    [sym_url_flg_temp] = ACTIONS(1847),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1847),
    [sym_url_flg_stat_mask] = ACTIONS(1847),
    [sym_url_flg_stat_lookup] = ACTIONS(1847),
    [sym_url_flg_stat_connecting] = ACTIONS(1847),
    [sym_url_flg_stat_waiting] = ACTIONS(1847),
    [sym_url_flg_stat_connected] = ACTIONS(1847),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1847),
    [sym_first_virtual_device] = ACTIONS(1847),
    [sym_first_local_port] = ACTIONS(1847),
    [sym_dynamic_virtual_device] = ACTIONS(1847),
    [sym_timeline_once] = ACTIONS(1847),
    [sym_timeline_repeat] = ACTIONS(1847),
    [sym_timeline_absolute] = ACTIONS(1847),
    [sym_timeline_relative] = ACTIONS(1847),
    [sym_do_push_timed_infinite] = ACTIONS(1847),
    [sym_source_type_no_address] = ACTIONS(1847),
    [sym_source_type_neuron_id] = ACTIONS(1847),
    [sym_source_type_ip_address] = ACTIONS(1847),
    [sym_source_type_axlink] = ACTIONS(1847),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1847),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1847),
    [sym_source_type_ip_socket_address] = ACTIONS(1847),
    [sym_source_type_rs232] = ACTIONS(1847),
    [sym_source_type_internal] = ACTIONS(1847),
    [sym_source_type_mac_address] = ACTIONS(1847),
    [sym_source_type_ipv4_port] = ACTIONS(1847),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1847),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1847),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1847),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1847),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1847),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1847),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1847),
    [sym_true] = ACTIONS(1847),
    [sym_false] = ACTIONS(1847),
    [sym_file_read_only] = ACTIONS(1847),
    [sym_file_rw_new] = ACTIONS(1847),
    [sym_file_rw_append] = ACTIONS(1847),
    [sym_ip_tcp] = ACTIONS(1847),
    [sym_ip_udp] = ACTIONS(1847),
    [sym_ip_udp_2way] = ACTIONS(1847),
    [sym_xml_encode_types] = ACTIONS(1847),
    [sym_xml_encode_char_as_list] = ACTIONS(1847),
    [sym_xml_encode_le] = ACTIONS(1847),
    [sym_xml_decode_types] = ACTIONS(1847),
    [sym_xml_decode_no_preserve] = ACTIONS(1847),
    [sym_duet_dev_type_display_device] = ACTIONS(1847),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1847),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1847),
    [sym_duet_dev_type_io_device] = ACTIONS(1847),
    [sym_duet_dev_type_relay_device] = ACTIONS(1847),
    [sym_duet_dev_type_ups] = ACTIONS(1847),
    [sym_duet_dev_type_amplifier] = ACTIONS(1847),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1847),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1847),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1847),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1847),
    [sym_duet_dev_type_camera] = ACTIONS(1847),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1847),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1847),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1847),
    [sym_duet_dev_type_dss] = ACTIONS(1847),
    [sym_duet_dev_type_dvr] = ACTIONS(1847),
    [sym_duet_dev_type_disc_device] = ACTIONS(1847),
    [sym_duet_dev_type_document_camera] = ACTIONS(1847),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1847),
    [sym_duet_dev_type_hvac] = ACTIONS(1847),
    [sym_duet_dev_type_keypad] = ACTIONS(1847),
    [sym_duet_dev_type_light] = ACTIONS(1847),
    [sym_duet_dev_type_monitor] = ACTIONS(1847),
    [sym_duet_dev_type_motor] = ACTIONS(1847),
    [sym_duet_dev_type_multi_window] = ACTIONS(1847),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1847),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1847),
    [sym_duet_dev_type_receiver] = ACTIONS(1847),
    [sym_duet_dev_type_security_system] = ACTIONS(1847),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1847),
    [sym_duet_dev_type_settop_box] = ACTIONS(1847),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1847),
    [sym_duet_dev_type_switcher] = ACTIONS(1847),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1847),
    [sym_duet_dev_type_tv] = ACTIONS(1847),
    [sym_duet_dev_type_utility] = ACTIONS(1847),
    [sym_duet_dev_type_vcr] = ACTIONS(1847),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1847),
    [sym_duet_dev_type_video_processor] = ACTIONS(1847),
    [sym_duet_dev_type_video_projector] = ACTIONS(1847),
    [sym_duet_dev_type_video_wall] = ACTIONS(1847),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1847),
    [sym_duet_dev_type_weather] = ACTIONS(1847),
    [sym_duet_dev_not_polled] = ACTIONS(1847),
    [sym_duet_dev_polled] = ACTIONS(1847),
    [sym_clkmgr_mode_network] = ACTIONS(1847),
    [sym_clkmgr_mode_standalone] = ACTIONS(1847),
    [sym_normal_standby] = ACTIONS(1847),
    [sym_normal_wake] = ACTIONS(1847),
    [sym_ip_multicast_ttl_option] = ACTIONS(1847),
    [sym_ip_ttl_subnet] = ACTIONS(1847),
    [sym_ip_ttl_site] = ACTIONS(1847),
    [sym_ip_ttl_region] = ACTIONS(1847),
    [sym_ip_ttl_continent] = ACTIONS(1847),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1847),
    [sym_ip_nodelay_on] = ACTIONS(1847),
    [sym_ip_nodelay_off] = ACTIONS(1847),
    [sym_amx_error] = ACTIONS(1847),
    [sym_amx_warning] = ACTIONS(1847),
    [sym_amx_info] = ACTIONS(1847),
    [sym_amx_debug] = ACTIONS(1847),
    [sym_netlinx_login_success] = ACTIONS(1847),
    [sym_netlinx_login_fail] = ACTIONS(1847),
    [sym_netlinx_logout] = ACTIONS(1847),
    [sym_null_str] = ACTIONS(1847),
    [sym_smtp_address] = ACTIONS(1847),
    [sym_smtp_port_number] = ACTIONS(1847),
    [sym_smtp_username] = ACTIONS(1847),
    [sym_smtp_password] = ACTIONS(1847),
    [sym_smtp_require_tls] = ACTIONS(1847),
    [sym_smtp_from] = ACTIONS(1847),
    [sym_smtp_tls_true] = ACTIONS(1847),
    [sym_smtp_tls_false] = ACTIONS(1847),
    [sym_listview_on_row_select_event] = ACTIONS(1847),
    [sym_data_structure_unknown] = ACTIONS(1847),
    [sym_data_structure_datafeed] = ACTIONS(1847),
    [sym_data_structure_datarecordset] = ACTIONS(1847),
    [sym_data_structure_datarecord] = ACTIONS(1847),
    [sym_data_structure_datafield] = ACTIONS(1847),
    [sym_data_max_content_field_count] = ACTIONS(1847),
    [sym_data_max_metadata_field_count] = ACTIONS(1847),
    [sym_data_max_id_length] = ACTIONS(1847),
    [sym_data_max_name_length] = ACTIONS(1847),
    [sym_data_max_description_length] = ACTIONS(1847),
    [sym_data_max_type_length] = ACTIONS(1847),
    [sym_data_max_format_length] = ACTIONS(1847),
    [sym_data_max_source_length] = ACTIONS(1847),
    [sym_data_max_value_length] = ACTIONS(1847),
    [sym_data_max_label_length] = ACTIONS(1847),
    [sym_data_max_url_length] = ACTIONS(1847),
    [sym_max_last_login_info_length] = ACTIONS(1847),
    [sym_data_type_unknown] = ACTIONS(1847),
    [sym_data_type_string] = ACTIONS(1847),
    [sym_data_type_datetime] = ACTIONS(1847),
    [sym_data_type_date] = ACTIONS(1847),
    [sym_data_type_time] = ACTIONS(1847),
    [sym_data_type_image] = ACTIONS(1847),
    [sym_data_format_url] = ACTIONS(1847),
    [sym_data_format_phone] = ACTIONS(1847),
    [sym_data_format_email] = ACTIONS(1847),
    [sym_data_format_iso8601] = ACTIONS(1847),
    [sym_auditlog_priv] = ACTIONS(1847),
    [sym_configuration_priv] = ACTIONS(1847),
    [sym_deviceconfig_priv] = ACTIONS(1847),
    [sym_ftp_priv] = ACTIONS(1847),
    [sym_http_priv] = ACTIONS(1847),
    [sym_networkconfig_priv] = ACTIONS(1847),
    [sym_programport_priv] = ACTIONS(1847),
    [sym_remoteui_priv] = ACTIONS(1847),
    [sym_securitycontrol_priv] = ACTIONS(1847),
    [sym_softwaremanagement_priv] = ACTIONS(1847),
    [sym_terminal_priv] = ACTIONS(1847),
    [sym_tpadmin_priv] = ACTIONS(1847),
    [sym_user1_priv] = ACTIONS(1847),
    [sym_user2_priv] = ACTIONS(1847),
    [sym_user3_priv] = ACTIONS(1847),
    [sym_user4_priv] = ACTIONS(1847),
    [sym_usermanagement_priv] = ACTIONS(1847),
    [sym_valid_account] = ACTIONS(1847),
    [sym_err_name_invalid] = ACTIONS(1847),
    [sym_err_password_invalid] = ACTIONS(1847),
    [sym_err_invalid_account] = ACTIONS(1847),
    [sym_err_no_authorization] = ACTIONS(1847),
    [sym_err_invalid_parameter] = ACTIONS(1847),
    [sym_err_user_locked_out] = ACTIONS(1847),
    [sym_err_user_account_expired] = ACTIONS(1847),
    [sym_err_general_error] = ACTIONS(1847),
    [sym_err_authentication_error] = ACTIONS(1847),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1847),
    [sym_tls_validate_certificate] = ACTIONS(1847),
    [sym_tdata] = ACTIONS(1847),
    [sym_tchannel] = ACTIONS(1847),
    [sym_tlevel] = ACTIONS(1847),
    [sym_tbutton] = ACTIONS(1847),
    [sym_ttimeline] = ACTIONS(1847),
    [sym_tcustom] = ACTIONS(1847),
    [sym_url_struct] = ACTIONS(1847),
    [sym_dns_struct] = ACTIONS(1847),
    [sym_ip_address_struct] = ACTIONS(1847),
    [sym_dev_info_struct] = ACTIONS(1847),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1847),
    [sym_data_feed] = ACTIONS(1847),
    [sym_data_field] = ACTIONS(1847),
    [sym_data_record] = ACTIONS(1847),
    [sym_wc_data_feed] = ACTIONS(1847),
    [sym_wc_data_field] = ACTIONS(1847),
    [sym_wc_data_record] = ACTIONS(1847),
    [sym_last_login_info] = ACTIONS(1847),
    [sym____reserved___] = ACTIONS(1847),
    [sym_date] = ACTIONS(1847),
    [sym_day] = ACTIONS(1847),
    [sym_time] = ACTIONS(1847),
    [sym_ldate] = ACTIONS(1847),
    [sym_push_device] = ACTIONS(1847),
    [sym_push_channel] = ACTIONS(1847),
    [sym_push_devchan] = ACTIONS(1847),
    [sym_release_device] = ACTIONS(1847),
    [sym_release_channel] = ACTIONS(1847),
    [sym_release_devchan] = ACTIONS(1847),
    [sym_master_slot] = ACTIONS(1847),
    [sym_get_pulse_time] = ACTIONS(1847),
    [sym_get_timer] = ACTIONS(1847),
    [sym_master_sn] = ACTIONS(1847),
    [sym_system_number] = ACTIONS(1847),
    [sym_button] = ACTIONS(1847),
    [sym_dv_channel] = ACTIONS(1847),
    [sym_data] = ACTIONS(1847),
    [sym_level] = ACTIONS(1847),
    [sym_timeline] = ACTIONS(1847),
    [sym_channel] = ACTIONS(1847),
    [sym_custom] = ACTIONS(1847),
    [sym_atoi] = ACTIONS(1847),
    [sym_atol] = ACTIONS(1847),
    [sym_atof] = ACTIONS(1847),
    [sym_itoa] = ACTIONS(1847),
    [sym_itohex] = ACTIONS(1847),
    [sym_hextoi] = ACTIONS(1847),
    [sym_ftoa] = ACTIONS(1847),
    [sym_find_string] = ACTIONS(1847),
    [sym_left_string] = ACTIONS(1847),
    [sym_right_string] = ACTIONS(1847),
    [sym_mid_string] = ACTIONS(1847),
    [sym_remove_string] = ACTIONS(1847),
    [sym_get_buffer_string] = ACTIONS(1847),
    [sym_get_buffer_char] = ACTIONS(1847),
    [sym_get_multi_buffer_string] = ACTIONS(1847),
    [sym_length_string] = ACTIONS(1847),
    [sym_max_length_string] = ACTIONS(1847),
    [sym_set_length_string] = ACTIONS(1847),
    [sym_compare_string] = ACTIONS(1847),
    [sym_upper_string] = ACTIONS(1847),
    [sym_lower_string] = ACTIONS(1847),
    [sym_get_last] = ACTIONS(1847),
    [sym_date_to_day] = ACTIONS(1847),
    [sym_date_to_month] = ACTIONS(1847),
    [sym_date_to_year] = ACTIONS(1847),
    [sym_time_to_hour] = ACTIONS(1847),
    [sym_time_to_minute] = ACTIONS(1847),
    [sym_time_to_second] = ACTIONS(1847),
    [sym_day_of_week] = ACTIONS(1847),
    [sym_set_pulse_time] = ACTIONS(1847),
    [sym_set_timer] = ACTIONS(1847),
    [sym_file_open] = ACTIONS(1847),
    [sym_file_close] = ACTIONS(1847),
    [sym_file_read] = ACTIONS(1847),
    [sym_file_read_line] = ACTIONS(1847),
    [sym_file_write] = ACTIONS(1847),
    [sym_file_write_line] = ACTIONS(1847),
    [sym_file_copy] = ACTIONS(1847),
    [sym_file_delete] = ACTIONS(1847),
    [sym_file_rename] = ACTIONS(1847),
    [sym_file_dir] = ACTIONS(1847),
    [sym_file_getdir] = ACTIONS(1847),
    [sym_file_setdir] = ACTIONS(1847),
    [sym_file_create_dir] = ACTIONS(1847),
    [sym_file_remove_dir] = ACTIONS(1847),
    [sym_file_seek] = ACTIONS(1847),
    [sym_length_array] = ACTIONS(1847),
    [sym_max_length_array] = ACTIONS(1847),
    [sym_set_length_array] = ACTIONS(1847),
    [sym_ip_client_open] = ACTIONS(1847),
    [sym_ip_client_close] = ACTIONS(1847),
    [sym_ip_server_open] = ACTIONS(1847),
    [sym_ip_server_close] = ACTIONS(1847),
    [sym_ip_bound_client_open] = ACTIONS(1847),
    [sym_ip_mc_server_open] = ACTIONS(1847),
    [sym_ip_set_option] = ACTIONS(1847),
    [sym_ssh_client_open] = ACTIONS(1847),
    [sym_ssh_client_close] = ACTIONS(1847),
    [sym_tls_client_open] = ACTIONS(1847),
    [sym_tls_client_close] = ACTIONS(1847),
    [sym_do_push] = ACTIONS(1847),
    [sym_do_push_timed] = ACTIONS(1847),
    [sym_do_release] = ACTIONS(1847),
    [sym_device_id] = ACTIONS(1847),
    [sym_device_info] = ACTIONS(1847),
    [sym_device_id_string] = ACTIONS(1847),
    [sym_device_standby] = ACTIONS(1847),
    [sym_device_wake] = ACTIONS(1847),
    [sym_timeline_create] = ACTIONS(1847),
    [sym_timeline_kill] = ACTIONS(1847),
    [sym_timeline_reload] = ACTIONS(1847),
    [sym_timeline_pause] = ACTIONS(1847),
    [sym_timeline_restart] = ACTIONS(1847),
    [sym_timeline_set] = ACTIONS(1847),
    [sym_timeline_get] = ACTIONS(1847),
    [sym_timeline_active] = ACTIONS(1847),
    [sym_timeline_dynamic_id] = ACTIONS(1847),
    [sym_variable_to_string] = ACTIONS(1847),
    [sym_string_to_variable] = ACTIONS(1847),
    [sym_length_variable_to_string] = ACTIONS(1847),
    [sym_type_cast] = ACTIONS(1847),
    [sym_variable_to_xml] = ACTIONS(1847),
    [sym_xml_to_variable] = ACTIONS(1847),
    [sym_length_variable_to_xml] = ACTIONS(1847),
    [sym_redirect_string] = ACTIONS(1847),
    [sym_random_number] = ACTIONS(1847),
    [sym_get_unique_id] = ACTIONS(1847),
    [sym_get_system_number] = ACTIONS(1847),
    [sym_set_system_number] = ACTIONS(1847),
    [sym_get_serial_number] = ACTIONS(1847),
    [sym_get_master_build] = ACTIONS(1847),
    [sym_set_validation_code] = ACTIONS(1847),
    [sym_reboot] = ACTIONS(1847),
    [sym_load_duet_module] = ACTIONS(1847),
    [sym_unload_duet_module] = ACTIONS(1847),
    [sym_duet_mem_size_set] = ACTIONS(1847),
    [sym_duet_mem_size_get] = ACTIONS(1847),
    [sym_dynamic_application_device] = ACTIONS(1847),
    [sym_dynamic_polled_port] = ACTIONS(1847),
    [sym_static_port_binding] = ACTIONS(1847),
    [sym_static_ip_binding] = ACTIONS(1847),
    [sym_get_dns_list] = ACTIONS(1847),
    [sym_set_dns_list] = ACTIONS(1847),
    [sym_get_ip_address] = ACTIONS(1847),
    [sym_set_ip_address] = ACTIONS(1847),
    [sym_get_url_list] = ACTIONS(1847),
    [sym_add_url_entry] = ACTIONS(1847),
    [sym_delete_url_entry] = ACTIONS(1847),
    [sym_abs_value] = ACTIONS(1847),
    [sym_max_value] = ACTIONS(1847),
    [sym_min_value] = ACTIONS(1847),
    [sym_exp_value] = ACTIONS(1847),
    [sym_log_value] = ACTIONS(1847),
    [sym_log10_value] = ACTIONS(1847),
    [sym_power_value] = ACTIONS(1847),
    [sym_sqrt_value] = ACTIONS(1847),
    [sym_raw_be] = ACTIONS(1847),
    [sym_raw_le] = ACTIONS(1847),
    [sym_internal_queue_size_set] = ACTIONS(1847),
    [sym_internal_queue_size_get] = ACTIONS(1847),
    [sym_internal_threshold_set] = ACTIONS(1847),
    [sym_internal_threshold_get] = ACTIONS(1847),
    [sym_rebuilt_event] = ACTIONS(1847),
    [sym_do_custom_event] = ACTIONS(1847),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1847),
    [sym_clkmgr_set_clk_source] = ACTIONS(1847),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1847),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1847),
    [sym_clkmgr_get_timezone] = ACTIONS(1847),
    [sym_clkmgr_set_timezone] = ACTIONS(1847),
    [sym_clkmgr_get_resync_period] = ACTIONS(1847),
    [sym_clkmgr_set_resync_period] = ACTIONS(1847),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1847),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1847),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1847),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1847),
    [sym_clkmgr_get_timeservers] = ACTIONS(1847),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1847),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1847),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1847),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1847),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1847),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1847),
    [sym_set_log_level] = ACTIONS(1847),
    [sym_get_log_level] = ACTIONS(1847),
    [sym_amx_log] = ACTIONS(1847),
    [sym_smtp_server_config_set] = ACTIONS(1847),
    [sym_smtp_server_config_get] = ACTIONS(1847),
    [sym_smtp_send] = ACTIONS(1847),
    [sym_validate_netlinx_account] = ACTIONS(1847),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1847),
    [sym_authenticate_certificate] = ACTIONS(1847),
    [sym_audit_netlinx_session_event] = ACTIONS(1847),
    [sym_audit_netlinx_generic_event] = ACTIONS(1847),
    [sym_get_available_flash_disk_space] = ACTIONS(1847),
    [sym_get_max_flash_disk_space] = ACTIONS(1847),
    [sym_data_create_feed] = ACTIONS(1847),
    [sym_data_delete_feed] = ACTIONS(1847),
    [sym_data_publish_feed] = ACTIONS(1847),
    [sym_data_get_published_feed] = ACTIONS(1847),
    [sym_data_add_record] = ACTIONS(1847),
    [sym_data_get_event_record] = ACTIONS(1847),
    [sym__wc_data_create_feed] = ACTIONS(1847),
    [sym__wc_data_add_record] = ACTIONS(1847),
    [sym__wc_data_get_event_record] = ACTIONS(1847),
    [sym_format] = ACTIONS(1847),
    [sym_astro_clock] = ACTIONS(1847),
    [sym_set_outdoor_temperature] = ACTIONS(1847),
    [sym_set_virtual_level_count] = ACTIONS(1847),
    [sym_set_virtual_channel_count] = ACTIONS(1847),
    [sym_set_virtual_port_count] = ACTIONS(1847),
    [sym__wc_encode_int] = ACTIONS(1847),
    [sym__wc_decode_int] = ACTIONS(1847),
    [sym__wc_int] = ACTIONS(1847),
    [sym__wc_to_ch_int] = ACTIONS(1847),
    [sym__ch_to_wc_int] = ACTIONS(1847),
    [sym__wc_find_string_int] = ACTIONS(1847),
    [sym__wc_left_string_int] = ACTIONS(1847),
    [sym__wc_lower_string_int] = ACTIONS(1847),
    [sym__wc_mid_string_int] = ACTIONS(1847),
    [sym__wc_remove_string_int] = ACTIONS(1847),
    [sym__wc_right_string_int] = ACTIONS(1847),
    [sym__wc_upper_string_int] = ACTIONS(1847),
    [sym__wc_compare_string_int] = ACTIONS(1847),
    [sym__wc_get_buffer_char_int] = ACTIONS(1847),
    [sym__wc_get_buffer_string_int] = ACTIONS(1847),
    [sym__wc_concat_string_int] = ACTIONS(1847),
    [sym__wc_file_open_int] = ACTIONS(1847),
    [sym__wc_file_close_int] = ACTIONS(1847),
    [sym__wc_file_read_int] = ACTIONS(1847),
    [sym__wc_file_read_line_int] = ACTIONS(1847),
    [sym__wc_file_write_int] = ACTIONS(1847),
    [sym__wc_file_write_line_int] = ACTIONS(1847),
    [aux_sym_preproc_include_token1] = ACTIONS(1847),
    [aux_sym_preproc_define_token1] = ACTIONS(1849),
    [aux_sym_preproc_warn_token1] = ACTIONS(1849),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1849),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1849),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1849),
    [anon_sym_LBRACK] = ACTIONS(1849),
    [anon_sym_LBRACE] = ACTIONS(1849),
    [anon_sym_RBRACE] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(1847),
    [anon_sym_BANG] = ACTIONS(1849),
    [anon_sym_TILDE] = ACTIONS(1849),
    [anon_sym_DASH] = ACTIONS(1847),
    [anon_sym_PLUS] = ACTIONS(1847),
    [anon_sym_DASH_DASH] = ACTIONS(1849),
    [anon_sym_PLUS_PLUS] = ACTIONS(1849),
    [anon_sym_DQUOTE] = ACTIONS(1849),
    [anon_sym_SQUOTE] = ACTIONS(1849),
    [sym_number_literal] = ACTIONS(1849),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(497)] = {
    [sym_identifier] = ACTIONS(1715),
    [sym_define_call_keyword] = ACTIONS(1715),
    [sym_define_function_keyword] = ACTIONS(1715),
    [sym_define_library_function_keyword] = ACTIONS(1715),
    [sym_define_module_keyword] = ACTIONS(1715),
    [sym_char_keyword] = ACTIONS(1715),
    [sym_widechar_keyword] = ACTIONS(1715),
    [sym_integer_keyword] = ACTIONS(1715),
    [sym_sinteger_keyword] = ACTIONS(1715),
    [sym_long_keyword] = ACTIONS(1715),
    [sym_slong_keyword] = ACTIONS(1715),
    [sym_float_keyword] = ACTIONS(1715),
    [sym_double_keyword] = ACTIONS(1715),
    [sym_variant_keyword] = ACTIONS(1715),
    [sym_variantarray_keyword] = ACTIONS(1715),
    [sym_local_var_keyword] = ACTIONS(1715),
    [sym_stack_var_keyword] = ACTIONS(1715),
    [sym_constant_keyword] = ACTIONS(1715),
    [sym_volatile_keyword] = ACTIONS(1715),
    [sym_non_volatile_keyword] = ACTIONS(1715),
    [sym_persistent_keyword] = ACTIONS(1715),
    [sym_dev_keyword] = ACTIONS(1715),
    [sym_devlev_keyword] = ACTIONS(1715),
    [sym_devchan_keyword] = ACTIONS(1715),
    [sym_if_keyword] = ACTIONS(1715),
    [sym_else_keyword] = ACTIONS(1715),
    [sym_switch_keyword] = ACTIONS(1715),
    [sym_case_keyword] = ACTIONS(1715),
    [sym_default_keyword] = ACTIONS(1715),
    [sym_while_keyword] = ACTIONS(1715),
    [sym_for_keyword] = ACTIONS(1715),
    [sym_break_keyword] = ACTIONS(1715),
    [sym_continue_keyword] = ACTIONS(1715),
    [sym_return_keyword] = ACTIONS(1715),
    [sym_select_keyword] = ACTIONS(1715),
    [sym_active_keyword] = ACTIONS(1715),
    [sym_struct_keyword] = ACTIONS(1715),
    [sym_structure_keyword] = ACTIONS(1715),
    [aux_sym_on_keyword_token1] = ACTIONS(1715),
    [aux_sym_off_keyword_token1] = ACTIONS(1715),
    [sym_send_level_keyword] = ACTIONS(1715),
    [sym_send_string_keyword] = ACTIONS(1715),
    [sym_send_command_keyword] = ACTIONS(1715),
    [sym_clear_buffer_keyword] = ACTIONS(1715),
    [sym_create_buffer_keyword] = ACTIONS(1715),
    [sym_create_multi_buffer_keyword] = ACTIONS(1715),
    [sym_call_keyword] = ACTIONS(1715),
    [sym_system_call_keyword] = ACTIONS(1715),
    [sym_devchan_to_keyword] = ACTIONS(1715),
    [sym_devchan_min_to_keyword] = ACTIONS(1715),
    [sym_devchan_total_off_keyword] = ACTIONS(1715),
    [sym_devchan_pulse_keyword] = ACTIONS(1715),
    [sym_wait_keyword] = ACTIONS(1715),
    [sym_cancel_wait_keyword] = ACTIONS(1715),
    [sym_wait_until_keyword] = ACTIONS(1715),
    [sym_cancel_wait_until_keyword] = ACTIONS(1715),
    [sym_cancel_all_wait_keyword] = ACTIONS(1715),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1715),
    [sym_not] = ACTIONS(1715),
    [sym_bnot] = ACTIONS(1715),
    [sym___file__] = ACTIONS(1715),
    [sym___date__] = ACTIONS(1715),
    [sym___ldate__] = ACTIONS(1715),
    [sym___time__] = ACTIONS(1715),
    [sym___line__] = ACTIONS(1715),
    [sym___name__] = ACTIONS(1715),
    [sym___netlinx__] = ACTIONS(1715),
    [sym_netlinx_axi_version] = ACTIONS(1715),
    [sym_url_flg_tcp] = ACTIONS(1715),
    [sym_url_flg_accinfopresent] = ACTIONS(1715),
    [sym_url_flg_temp] = ACTIONS(1715),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1715),
    [sym_url_flg_stat_mask] = ACTIONS(1715),
    [sym_url_flg_stat_lookup] = ACTIONS(1715),
    [sym_url_flg_stat_connecting] = ACTIONS(1715),
    [sym_url_flg_stat_waiting] = ACTIONS(1715),
    [sym_url_flg_stat_connected] = ACTIONS(1715),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1715),
    [sym_first_virtual_device] = ACTIONS(1715),
    [sym_first_local_port] = ACTIONS(1715),
    [sym_dynamic_virtual_device] = ACTIONS(1715),
    [sym_timeline_once] = ACTIONS(1715),
    [sym_timeline_repeat] = ACTIONS(1715),
    [sym_timeline_absolute] = ACTIONS(1715),
    [sym_timeline_relative] = ACTIONS(1715),
    [sym_do_push_timed_infinite] = ACTIONS(1715),
    [sym_source_type_no_address] = ACTIONS(1715),
    [sym_source_type_neuron_id] = ACTIONS(1715),
    [sym_source_type_ip_address] = ACTIONS(1715),
    [sym_source_type_axlink] = ACTIONS(1715),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1715),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1715),
    [sym_source_type_ip_socket_address] = ACTIONS(1715),
    [sym_source_type_rs232] = ACTIONS(1715),
    [sym_source_type_internal] = ACTIONS(1715),
    [sym_source_type_mac_address] = ACTIONS(1715),
    [sym_source_type_ipv4_port] = ACTIONS(1715),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1715),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1715),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1715),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1715),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1715),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1715),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1715),
    [sym_true] = ACTIONS(1715),
    [sym_false] = ACTIONS(1715),
    [sym_file_read_only] = ACTIONS(1715),
    [sym_file_rw_new] = ACTIONS(1715),
    [sym_file_rw_append] = ACTIONS(1715),
    [sym_ip_tcp] = ACTIONS(1715),
    [sym_ip_udp] = ACTIONS(1715),
    [sym_ip_udp_2way] = ACTIONS(1715),
    [sym_xml_encode_types] = ACTIONS(1715),
    [sym_xml_encode_char_as_list] = ACTIONS(1715),
    [sym_xml_encode_le] = ACTIONS(1715),
    [sym_xml_decode_types] = ACTIONS(1715),
    [sym_xml_decode_no_preserve] = ACTIONS(1715),
    [sym_duet_dev_type_display_device] = ACTIONS(1715),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1715),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1715),
    [sym_duet_dev_type_io_device] = ACTIONS(1715),
    [sym_duet_dev_type_relay_device] = ACTIONS(1715),
    [sym_duet_dev_type_ups] = ACTIONS(1715),
    [sym_duet_dev_type_amplifier] = ACTIONS(1715),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1715),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1715),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1715),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1715),
    [sym_duet_dev_type_camera] = ACTIONS(1715),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1715),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1715),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1715),
    [sym_duet_dev_type_dss] = ACTIONS(1715),
    [sym_duet_dev_type_dvr] = ACTIONS(1715),
    [sym_duet_dev_type_disc_device] = ACTIONS(1715),
    [sym_duet_dev_type_document_camera] = ACTIONS(1715),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1715),
    [sym_duet_dev_type_hvac] = ACTIONS(1715),
    [sym_duet_dev_type_keypad] = ACTIONS(1715),
    [sym_duet_dev_type_light] = ACTIONS(1715),
    [sym_duet_dev_type_monitor] = ACTIONS(1715),
    [sym_duet_dev_type_motor] = ACTIONS(1715),
    [sym_duet_dev_type_multi_window] = ACTIONS(1715),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1715),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1715),
    [sym_duet_dev_type_receiver] = ACTIONS(1715),
    [sym_duet_dev_type_security_system] = ACTIONS(1715),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1715),
    [sym_duet_dev_type_settop_box] = ACTIONS(1715),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1715),
    [sym_duet_dev_type_switcher] = ACTIONS(1715),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1715),
    [sym_duet_dev_type_tv] = ACTIONS(1715),
    [sym_duet_dev_type_utility] = ACTIONS(1715),
    [sym_duet_dev_type_vcr] = ACTIONS(1715),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1715),
    [sym_duet_dev_type_video_processor] = ACTIONS(1715),
    [sym_duet_dev_type_video_projector] = ACTIONS(1715),
    [sym_duet_dev_type_video_wall] = ACTIONS(1715),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1715),
    [sym_duet_dev_type_weather] = ACTIONS(1715),
    [sym_duet_dev_not_polled] = ACTIONS(1715),
    [sym_duet_dev_polled] = ACTIONS(1715),
    [sym_clkmgr_mode_network] = ACTIONS(1715),
    [sym_clkmgr_mode_standalone] = ACTIONS(1715),
    [sym_normal_standby] = ACTIONS(1715),
    [sym_normal_wake] = ACTIONS(1715),
    [sym_ip_multicast_ttl_option] = ACTIONS(1715),
    [sym_ip_ttl_subnet] = ACTIONS(1715),
    [sym_ip_ttl_site] = ACTIONS(1715),
    [sym_ip_ttl_region] = ACTIONS(1715),
    [sym_ip_ttl_continent] = ACTIONS(1715),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1715),
    [sym_ip_nodelay_on] = ACTIONS(1715),
    [sym_ip_nodelay_off] = ACTIONS(1715),
    [sym_amx_error] = ACTIONS(1715),
    [sym_amx_warning] = ACTIONS(1715),
    [sym_amx_info] = ACTIONS(1715),
    [sym_amx_debug] = ACTIONS(1715),
    [sym_netlinx_login_success] = ACTIONS(1715),
    [sym_netlinx_login_fail] = ACTIONS(1715),
    [sym_netlinx_logout] = ACTIONS(1715),
    [sym_null_str] = ACTIONS(1715),
    [sym_smtp_address] = ACTIONS(1715),
    [sym_smtp_port_number] = ACTIONS(1715),
    [sym_smtp_username] = ACTIONS(1715),
    [sym_smtp_password] = ACTIONS(1715),
    [sym_smtp_require_tls] = ACTIONS(1715),
    [sym_smtp_from] = ACTIONS(1715),
    [sym_smtp_tls_true] = ACTIONS(1715),
    [sym_smtp_tls_false] = ACTIONS(1715),
    [sym_listview_on_row_select_event] = ACTIONS(1715),
    [sym_data_structure_unknown] = ACTIONS(1715),
    [sym_data_structure_datafeed] = ACTIONS(1715),
    [sym_data_structure_datarecordset] = ACTIONS(1715),
    [sym_data_structure_datarecord] = ACTIONS(1715),
    [sym_data_structure_datafield] = ACTIONS(1715),
    [sym_data_max_content_field_count] = ACTIONS(1715),
    [sym_data_max_metadata_field_count] = ACTIONS(1715),
    [sym_data_max_id_length] = ACTIONS(1715),
    [sym_data_max_name_length] = ACTIONS(1715),
    [sym_data_max_description_length] = ACTIONS(1715),
    [sym_data_max_type_length] = ACTIONS(1715),
    [sym_data_max_format_length] = ACTIONS(1715),
    [sym_data_max_source_length] = ACTIONS(1715),
    [sym_data_max_value_length] = ACTIONS(1715),
    [sym_data_max_label_length] = ACTIONS(1715),
    [sym_data_max_url_length] = ACTIONS(1715),
    [sym_max_last_login_info_length] = ACTIONS(1715),
    [sym_data_type_unknown] = ACTIONS(1715),
    [sym_data_type_string] = ACTIONS(1715),
    [sym_data_type_datetime] = ACTIONS(1715),
    [sym_data_type_date] = ACTIONS(1715),
    [sym_data_type_time] = ACTIONS(1715),
    [sym_data_type_image] = ACTIONS(1715),
    [sym_data_format_url] = ACTIONS(1715),
    [sym_data_format_phone] = ACTIONS(1715),
    [sym_data_format_email] = ACTIONS(1715),
    [sym_data_format_iso8601] = ACTIONS(1715),
    [sym_auditlog_priv] = ACTIONS(1715),
    [sym_configuration_priv] = ACTIONS(1715),
    [sym_deviceconfig_priv] = ACTIONS(1715),
    [sym_ftp_priv] = ACTIONS(1715),
    [sym_http_priv] = ACTIONS(1715),
    [sym_networkconfig_priv] = ACTIONS(1715),
    [sym_programport_priv] = ACTIONS(1715),
    [sym_remoteui_priv] = ACTIONS(1715),
    [sym_securitycontrol_priv] = ACTIONS(1715),
    [sym_softwaremanagement_priv] = ACTIONS(1715),
    [sym_terminal_priv] = ACTIONS(1715),
    [sym_tpadmin_priv] = ACTIONS(1715),
    [sym_user1_priv] = ACTIONS(1715),
    [sym_user2_priv] = ACTIONS(1715),
    [sym_user3_priv] = ACTIONS(1715),
    [sym_user4_priv] = ACTIONS(1715),
    [sym_usermanagement_priv] = ACTIONS(1715),
    [sym_valid_account] = ACTIONS(1715),
    [sym_err_name_invalid] = ACTIONS(1715),
    [sym_err_password_invalid] = ACTIONS(1715),
    [sym_err_invalid_account] = ACTIONS(1715),
    [sym_err_no_authorization] = ACTIONS(1715),
    [sym_err_invalid_parameter] = ACTIONS(1715),
    [sym_err_user_locked_out] = ACTIONS(1715),
    [sym_err_user_account_expired] = ACTIONS(1715),
    [sym_err_general_error] = ACTIONS(1715),
    [sym_err_authentication_error] = ACTIONS(1715),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1715),
    [sym_tls_validate_certificate] = ACTIONS(1715),
    [sym_tdata] = ACTIONS(1715),
    [sym_tchannel] = ACTIONS(1715),
    [sym_tlevel] = ACTIONS(1715),
    [sym_tbutton] = ACTIONS(1715),
    [sym_ttimeline] = ACTIONS(1715),
    [sym_tcustom] = ACTIONS(1715),
    [sym_url_struct] = ACTIONS(1715),
    [sym_dns_struct] = ACTIONS(1715),
    [sym_ip_address_struct] = ACTIONS(1715),
    [sym_dev_info_struct] = ACTIONS(1715),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1715),
    [sym_data_feed] = ACTIONS(1715),
    [sym_data_field] = ACTIONS(1715),
    [sym_data_record] = ACTIONS(1715),
    [sym_wc_data_feed] = ACTIONS(1715),
    [sym_wc_data_field] = ACTIONS(1715),
    [sym_wc_data_record] = ACTIONS(1715),
    [sym_last_login_info] = ACTIONS(1715),
    [sym____reserved___] = ACTIONS(1715),
    [sym_date] = ACTIONS(1715),
    [sym_day] = ACTIONS(1715),
    [sym_time] = ACTIONS(1715),
    [sym_ldate] = ACTIONS(1715),
    [sym_push_device] = ACTIONS(1715),
    [sym_push_channel] = ACTIONS(1715),
    [sym_push_devchan] = ACTIONS(1715),
    [sym_release_device] = ACTIONS(1715),
    [sym_release_channel] = ACTIONS(1715),
    [sym_release_devchan] = ACTIONS(1715),
    [sym_master_slot] = ACTIONS(1715),
    [sym_get_pulse_time] = ACTIONS(1715),
    [sym_get_timer] = ACTIONS(1715),
    [sym_master_sn] = ACTIONS(1715),
    [sym_system_number] = ACTIONS(1715),
    [sym_button] = ACTIONS(1715),
    [sym_dv_channel] = ACTIONS(1715),
    [sym_data] = ACTIONS(1715),
    [sym_level] = ACTIONS(1715),
    [sym_timeline] = ACTIONS(1715),
    [sym_channel] = ACTIONS(1715),
    [sym_custom] = ACTIONS(1715),
    [sym_atoi] = ACTIONS(1715),
    [sym_atol] = ACTIONS(1715),
    [sym_atof] = ACTIONS(1715),
    [sym_itoa] = ACTIONS(1715),
    [sym_itohex] = ACTIONS(1715),
    [sym_hextoi] = ACTIONS(1715),
    [sym_ftoa] = ACTIONS(1715),
    [sym_find_string] = ACTIONS(1715),
    [sym_left_string] = ACTIONS(1715),
    [sym_right_string] = ACTIONS(1715),
    [sym_mid_string] = ACTIONS(1715),
    [sym_remove_string] = ACTIONS(1715),
    [sym_get_buffer_string] = ACTIONS(1715),
    [sym_get_buffer_char] = ACTIONS(1715),
    [sym_get_multi_buffer_string] = ACTIONS(1715),
    [sym_length_string] = ACTIONS(1715),
    [sym_max_length_string] = ACTIONS(1715),
    [sym_set_length_string] = ACTIONS(1715),
    [sym_compare_string] = ACTIONS(1715),
    [sym_upper_string] = ACTIONS(1715),
    [sym_lower_string] = ACTIONS(1715),
    [sym_get_last] = ACTIONS(1715),
    [sym_date_to_day] = ACTIONS(1715),
    [sym_date_to_month] = ACTIONS(1715),
    [sym_date_to_year] = ACTIONS(1715),
    [sym_time_to_hour] = ACTIONS(1715),
    [sym_time_to_minute] = ACTIONS(1715),
    [sym_time_to_second] = ACTIONS(1715),
    [sym_day_of_week] = ACTIONS(1715),
    [sym_set_pulse_time] = ACTIONS(1715),
    [sym_set_timer] = ACTIONS(1715),
    [sym_file_open] = ACTIONS(1715),
    [sym_file_close] = ACTIONS(1715),
    [sym_file_read] = ACTIONS(1715),
    [sym_file_read_line] = ACTIONS(1715),
    [sym_file_write] = ACTIONS(1715),
    [sym_file_write_line] = ACTIONS(1715),
    [sym_file_copy] = ACTIONS(1715),
    [sym_file_delete] = ACTIONS(1715),
    [sym_file_rename] = ACTIONS(1715),
    [sym_file_dir] = ACTIONS(1715),
    [sym_file_getdir] = ACTIONS(1715),
    [sym_file_setdir] = ACTIONS(1715),
    [sym_file_create_dir] = ACTIONS(1715),
    [sym_file_remove_dir] = ACTIONS(1715),
    [sym_file_seek] = ACTIONS(1715),
    [sym_length_array] = ACTIONS(1715),
    [sym_max_length_array] = ACTIONS(1715),
    [sym_set_length_array] = ACTIONS(1715),
    [sym_ip_client_open] = ACTIONS(1715),
    [sym_ip_client_close] = ACTIONS(1715),
    [sym_ip_server_open] = ACTIONS(1715),
    [sym_ip_server_close] = ACTIONS(1715),
    [sym_ip_bound_client_open] = ACTIONS(1715),
    [sym_ip_mc_server_open] = ACTIONS(1715),
    [sym_ip_set_option] = ACTIONS(1715),
    [sym_ssh_client_open] = ACTIONS(1715),
    [sym_ssh_client_close] = ACTIONS(1715),
    [sym_tls_client_open] = ACTIONS(1715),
    [sym_tls_client_close] = ACTIONS(1715),
    [sym_do_push] = ACTIONS(1715),
    [sym_do_push_timed] = ACTIONS(1715),
    [sym_do_release] = ACTIONS(1715),
    [sym_device_id] = ACTIONS(1715),
    [sym_device_info] = ACTIONS(1715),
    [sym_device_id_string] = ACTIONS(1715),
    [sym_device_standby] = ACTIONS(1715),
    [sym_device_wake] = ACTIONS(1715),
    [sym_timeline_create] = ACTIONS(1715),
    [sym_timeline_kill] = ACTIONS(1715),
    [sym_timeline_reload] = ACTIONS(1715),
    [sym_timeline_pause] = ACTIONS(1715),
    [sym_timeline_restart] = ACTIONS(1715),
    [sym_timeline_set] = ACTIONS(1715),
    [sym_timeline_get] = ACTIONS(1715),
    [sym_timeline_active] = ACTIONS(1715),
    [sym_timeline_dynamic_id] = ACTIONS(1715),
    [sym_variable_to_string] = ACTIONS(1715),
    [sym_string_to_variable] = ACTIONS(1715),
    [sym_length_variable_to_string] = ACTIONS(1715),
    [sym_type_cast] = ACTIONS(1715),
    [sym_variable_to_xml] = ACTIONS(1715),
    [sym_xml_to_variable] = ACTIONS(1715),
    [sym_length_variable_to_xml] = ACTIONS(1715),
    [sym_redirect_string] = ACTIONS(1715),
    [sym_random_number] = ACTIONS(1715),
    [sym_get_unique_id] = ACTIONS(1715),
    [sym_get_system_number] = ACTIONS(1715),
    [sym_set_system_number] = ACTIONS(1715),
    [sym_get_serial_number] = ACTIONS(1715),
    [sym_get_master_build] = ACTIONS(1715),
    [sym_set_validation_code] = ACTIONS(1715),
    [sym_reboot] = ACTIONS(1715),
    [sym_load_duet_module] = ACTIONS(1715),
    [sym_unload_duet_module] = ACTIONS(1715),
    [sym_duet_mem_size_set] = ACTIONS(1715),
    [sym_duet_mem_size_get] = ACTIONS(1715),
    [sym_dynamic_application_device] = ACTIONS(1715),
    [sym_dynamic_polled_port] = ACTIONS(1715),
    [sym_static_port_binding] = ACTIONS(1715),
    [sym_static_ip_binding] = ACTIONS(1715),
    [sym_get_dns_list] = ACTIONS(1715),
    [sym_set_dns_list] = ACTIONS(1715),
    [sym_get_ip_address] = ACTIONS(1715),
    [sym_set_ip_address] = ACTIONS(1715),
    [sym_get_url_list] = ACTIONS(1715),
    [sym_add_url_entry] = ACTIONS(1715),
    [sym_delete_url_entry] = ACTIONS(1715),
    [sym_abs_value] = ACTIONS(1715),
    [sym_max_value] = ACTIONS(1715),
    [sym_min_value] = ACTIONS(1715),
    [sym_exp_value] = ACTIONS(1715),
    [sym_log_value] = ACTIONS(1715),
    [sym_log10_value] = ACTIONS(1715),
    [sym_power_value] = ACTIONS(1715),
    [sym_sqrt_value] = ACTIONS(1715),
    [sym_raw_be] = ACTIONS(1715),
    [sym_raw_le] = ACTIONS(1715),
    [sym_internal_queue_size_set] = ACTIONS(1715),
    [sym_internal_queue_size_get] = ACTIONS(1715),
    [sym_internal_threshold_set] = ACTIONS(1715),
    [sym_internal_threshold_get] = ACTIONS(1715),
    [sym_rebuilt_event] = ACTIONS(1715),
    [sym_do_custom_event] = ACTIONS(1715),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1715),
    [sym_clkmgr_set_clk_source] = ACTIONS(1715),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1715),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1715),
    [sym_clkmgr_get_timezone] = ACTIONS(1715),
    [sym_clkmgr_set_timezone] = ACTIONS(1715),
    [sym_clkmgr_get_resync_period] = ACTIONS(1715),
    [sym_clkmgr_set_resync_period] = ACTIONS(1715),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1715),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1715),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1715),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1715),
    [sym_clkmgr_get_timeservers] = ACTIONS(1715),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1715),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1715),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1715),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1715),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1715),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1715),
    [sym_set_log_level] = ACTIONS(1715),
    [sym_get_log_level] = ACTIONS(1715),
    [sym_amx_log] = ACTIONS(1715),
    [sym_smtp_server_config_set] = ACTIONS(1715),
    [sym_smtp_server_config_get] = ACTIONS(1715),
    [sym_smtp_send] = ACTIONS(1715),
    [sym_validate_netlinx_account] = ACTIONS(1715),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1715),
    [sym_authenticate_certificate] = ACTIONS(1715),
    [sym_audit_netlinx_session_event] = ACTIONS(1715),
    [sym_audit_netlinx_generic_event] = ACTIONS(1715),
    [sym_get_available_flash_disk_space] = ACTIONS(1715),
    [sym_get_max_flash_disk_space] = ACTIONS(1715),
    [sym_data_create_feed] = ACTIONS(1715),
    [sym_data_delete_feed] = ACTIONS(1715),
    [sym_data_publish_feed] = ACTIONS(1715),
    [sym_data_get_published_feed] = ACTIONS(1715),
    [sym_data_add_record] = ACTIONS(1715),
    [sym_data_get_event_record] = ACTIONS(1715),
    [sym__wc_data_create_feed] = ACTIONS(1715),
    [sym__wc_data_add_record] = ACTIONS(1715),
    [sym__wc_data_get_event_record] = ACTIONS(1715),
    [sym_format] = ACTIONS(1715),
    [sym_astro_clock] = ACTIONS(1715),
    [sym_set_outdoor_temperature] = ACTIONS(1715),
    [sym_set_virtual_level_count] = ACTIONS(1715),
    [sym_set_virtual_channel_count] = ACTIONS(1715),
    [sym_set_virtual_port_count] = ACTIONS(1715),
    [sym__wc_encode_int] = ACTIONS(1715),
    [sym__wc_decode_int] = ACTIONS(1715),
    [sym__wc_int] = ACTIONS(1715),
    [sym__wc_to_ch_int] = ACTIONS(1715),
    [sym__ch_to_wc_int] = ACTIONS(1715),
    [sym__wc_find_string_int] = ACTIONS(1715),
    [sym__wc_left_string_int] = ACTIONS(1715),
    [sym__wc_lower_string_int] = ACTIONS(1715),
    [sym__wc_mid_string_int] = ACTIONS(1715),
    [sym__wc_remove_string_int] = ACTIONS(1715),
    [sym__wc_right_string_int] = ACTIONS(1715),
    [sym__wc_upper_string_int] = ACTIONS(1715),
    [sym__wc_compare_string_int] = ACTIONS(1715),
    [sym__wc_get_buffer_char_int] = ACTIONS(1715),
    [sym__wc_get_buffer_string_int] = ACTIONS(1715),
    [sym__wc_concat_string_int] = ACTIONS(1715),
    [sym__wc_file_open_int] = ACTIONS(1715),
    [sym__wc_file_close_int] = ACTIONS(1715),
    [sym__wc_file_read_int] = ACTIONS(1715),
    [sym__wc_file_read_line_int] = ACTIONS(1715),
    [sym__wc_file_write_int] = ACTIONS(1715),
    [sym__wc_file_write_line_int] = ACTIONS(1715),
    [aux_sym_preproc_include_token1] = ACTIONS(1715),
    [aux_sym_preproc_define_token1] = ACTIONS(1717),
    [aux_sym_preproc_warn_token1] = ACTIONS(1717),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1717),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1717),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1717),
    [anon_sym_LBRACK] = ACTIONS(1717),
    [anon_sym_LBRACE] = ACTIONS(1717),
    [anon_sym_RBRACE] = ACTIONS(1717),
    [anon_sym_LPAREN] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [anon_sym_DASH] = ACTIONS(1715),
    [anon_sym_PLUS] = ACTIONS(1715),
    [anon_sym_DASH_DASH] = ACTIONS(1717),
    [anon_sym_PLUS_PLUS] = ACTIONS(1717),
    [anon_sym_DQUOTE] = ACTIONS(1717),
    [anon_sym_SQUOTE] = ACTIONS(1717),
    [sym_number_literal] = ACTIONS(1717),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(498)] = {
    [sym_identifier] = ACTIONS(1773),
    [sym_define_call_keyword] = ACTIONS(1773),
    [sym_define_function_keyword] = ACTIONS(1773),
    [sym_define_library_function_keyword] = ACTIONS(1773),
    [sym_define_module_keyword] = ACTIONS(1773),
    [sym_char_keyword] = ACTIONS(1773),
    [sym_widechar_keyword] = ACTIONS(1773),
    [sym_integer_keyword] = ACTIONS(1773),
    [sym_sinteger_keyword] = ACTIONS(1773),
    [sym_long_keyword] = ACTIONS(1773),
    [sym_slong_keyword] = ACTIONS(1773),
    [sym_float_keyword] = ACTIONS(1773),
    [sym_double_keyword] = ACTIONS(1773),
    [sym_variant_keyword] = ACTIONS(1773),
    [sym_variantarray_keyword] = ACTIONS(1773),
    [sym_local_var_keyword] = ACTIONS(1773),
    [sym_stack_var_keyword] = ACTIONS(1773),
    [sym_constant_keyword] = ACTIONS(1773),
    [sym_volatile_keyword] = ACTIONS(1773),
    [sym_non_volatile_keyword] = ACTIONS(1773),
    [sym_persistent_keyword] = ACTIONS(1773),
    [sym_dev_keyword] = ACTIONS(1773),
    [sym_devlev_keyword] = ACTIONS(1773),
    [sym_devchan_keyword] = ACTIONS(1773),
    [sym_if_keyword] = ACTIONS(1773),
    [sym_else_keyword] = ACTIONS(1773),
    [sym_switch_keyword] = ACTIONS(1773),
    [sym_case_keyword] = ACTIONS(1773),
    [sym_default_keyword] = ACTIONS(1773),
    [sym_while_keyword] = ACTIONS(1773),
    [sym_for_keyword] = ACTIONS(1773),
    [sym_break_keyword] = ACTIONS(1773),
    [sym_continue_keyword] = ACTIONS(1773),
    [sym_return_keyword] = ACTIONS(1773),
    [sym_select_keyword] = ACTIONS(1773),
    [sym_active_keyword] = ACTIONS(1773),
    [sym_struct_keyword] = ACTIONS(1773),
    [sym_structure_keyword] = ACTIONS(1773),
    [aux_sym_on_keyword_token1] = ACTIONS(1773),
    [aux_sym_off_keyword_token1] = ACTIONS(1773),
    [sym_send_level_keyword] = ACTIONS(1773),
    [sym_send_string_keyword] = ACTIONS(1773),
    [sym_send_command_keyword] = ACTIONS(1773),
    [sym_clear_buffer_keyword] = ACTIONS(1773),
    [sym_create_buffer_keyword] = ACTIONS(1773),
    [sym_create_multi_buffer_keyword] = ACTIONS(1773),
    [sym_call_keyword] = ACTIONS(1773),
    [sym_system_call_keyword] = ACTIONS(1773),
    [sym_devchan_to_keyword] = ACTIONS(1773),
    [sym_devchan_min_to_keyword] = ACTIONS(1773),
    [sym_devchan_total_off_keyword] = ACTIONS(1773),
    [sym_devchan_pulse_keyword] = ACTIONS(1773),
    [sym_wait_keyword] = ACTIONS(1773),
    [sym_cancel_wait_keyword] = ACTIONS(1773),
    [sym_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1773),
    [sym_not] = ACTIONS(1773),
    [sym_bnot] = ACTIONS(1773),
    [sym___file__] = ACTIONS(1773),
    [sym___date__] = ACTIONS(1773),
    [sym___ldate__] = ACTIONS(1773),
    [sym___time__] = ACTIONS(1773),
    [sym___line__] = ACTIONS(1773),
    [sym___name__] = ACTIONS(1773),
    [sym___netlinx__] = ACTIONS(1773),
    [sym_netlinx_axi_version] = ACTIONS(1773),
    [sym_url_flg_tcp] = ACTIONS(1773),
    [sym_url_flg_accinfopresent] = ACTIONS(1773),
    [sym_url_flg_temp] = ACTIONS(1773),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1773),
    [sym_url_flg_stat_mask] = ACTIONS(1773),
    [sym_url_flg_stat_lookup] = ACTIONS(1773),
    [sym_url_flg_stat_connecting] = ACTIONS(1773),
    [sym_url_flg_stat_waiting] = ACTIONS(1773),
    [sym_url_flg_stat_connected] = ACTIONS(1773),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1773),
    [sym_first_virtual_device] = ACTIONS(1773),
    [sym_first_local_port] = ACTIONS(1773),
    [sym_dynamic_virtual_device] = ACTIONS(1773),
    [sym_timeline_once] = ACTIONS(1773),
    [sym_timeline_repeat] = ACTIONS(1773),
    [sym_timeline_absolute] = ACTIONS(1773),
    [sym_timeline_relative] = ACTIONS(1773),
    [sym_do_push_timed_infinite] = ACTIONS(1773),
    [sym_source_type_no_address] = ACTIONS(1773),
    [sym_source_type_neuron_id] = ACTIONS(1773),
    [sym_source_type_ip_address] = ACTIONS(1773),
    [sym_source_type_axlink] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1773),
    [sym_source_type_ip_socket_address] = ACTIONS(1773),
    [sym_source_type_rs232] = ACTIONS(1773),
    [sym_source_type_internal] = ACTIONS(1773),
    [sym_source_type_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1773),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1773),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1773),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1773),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_file_read_only] = ACTIONS(1773),
    [sym_file_rw_new] = ACTIONS(1773),
    [sym_file_rw_append] = ACTIONS(1773),
    [sym_ip_tcp] = ACTIONS(1773),
    [sym_ip_udp] = ACTIONS(1773),
    [sym_ip_udp_2way] = ACTIONS(1773),
    [sym_xml_encode_types] = ACTIONS(1773),
    [sym_xml_encode_char_as_list] = ACTIONS(1773),
    [sym_xml_encode_le] = ACTIONS(1773),
    [sym_xml_decode_types] = ACTIONS(1773),
    [sym_xml_decode_no_preserve] = ACTIONS(1773),
    [sym_duet_dev_type_display_device] = ACTIONS(1773),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_io_device] = ACTIONS(1773),
    [sym_duet_dev_type_relay_device] = ACTIONS(1773),
    [sym_duet_dev_type_ups] = ACTIONS(1773),
    [sym_duet_dev_type_amplifier] = ACTIONS(1773),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1773),
    [sym_duet_dev_type_camera] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1773),
    [sym_duet_dev_type_dss] = ACTIONS(1773),
    [sym_duet_dev_type_dvr] = ACTIONS(1773),
    [sym_duet_dev_type_disc_device] = ACTIONS(1773),
    [sym_duet_dev_type_document_camera] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1773),
    [sym_duet_dev_type_hvac] = ACTIONS(1773),
    [sym_duet_dev_type_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_light] = ACTIONS(1773),
    [sym_duet_dev_type_monitor] = ACTIONS(1773),
    [sym_duet_dev_type_motor] = ACTIONS(1773),
    [sym_duet_dev_type_multi_window] = ACTIONS(1773),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1773),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1773),
    [sym_duet_dev_type_receiver] = ACTIONS(1773),
    [sym_duet_dev_type_security_system] = ACTIONS(1773),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1773),
    [sym_duet_dev_type_settop_box] = ACTIONS(1773),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1773),
    [sym_duet_dev_type_switcher] = ACTIONS(1773),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_tv] = ACTIONS(1773),
    [sym_duet_dev_type_utility] = ACTIONS(1773),
    [sym_duet_dev_type_vcr] = ACTIONS(1773),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_video_processor] = ACTIONS(1773),
    [sym_duet_dev_type_video_projector] = ACTIONS(1773),
    [sym_duet_dev_type_video_wall] = ACTIONS(1773),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1773),
    [sym_duet_dev_type_weather] = ACTIONS(1773),
    [sym_duet_dev_not_polled] = ACTIONS(1773),
    [sym_duet_dev_polled] = ACTIONS(1773),
    [sym_clkmgr_mode_network] = ACTIONS(1773),
    [sym_clkmgr_mode_standalone] = ACTIONS(1773),
    [sym_normal_standby] = ACTIONS(1773),
    [sym_normal_wake] = ACTIONS(1773),
    [sym_ip_multicast_ttl_option] = ACTIONS(1773),
    [sym_ip_ttl_subnet] = ACTIONS(1773),
    [sym_ip_ttl_site] = ACTIONS(1773),
    [sym_ip_ttl_region] = ACTIONS(1773),
    [sym_ip_ttl_continent] = ACTIONS(1773),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1773),
    [sym_ip_nodelay_on] = ACTIONS(1773),
    [sym_ip_nodelay_off] = ACTIONS(1773),
    [sym_amx_error] = ACTIONS(1773),
    [sym_amx_warning] = ACTIONS(1773),
    [sym_amx_info] = ACTIONS(1773),
    [sym_amx_debug] = ACTIONS(1773),
    [sym_netlinx_login_success] = ACTIONS(1773),
    [sym_netlinx_login_fail] = ACTIONS(1773),
    [sym_netlinx_logout] = ACTIONS(1773),
    [sym_null_str] = ACTIONS(1773),
    [sym_smtp_address] = ACTIONS(1773),
    [sym_smtp_port_number] = ACTIONS(1773),
    [sym_smtp_username] = ACTIONS(1773),
    [sym_smtp_password] = ACTIONS(1773),
    [sym_smtp_require_tls] = ACTIONS(1773),
    [sym_smtp_from] = ACTIONS(1773),
    [sym_smtp_tls_true] = ACTIONS(1773),
    [sym_smtp_tls_false] = ACTIONS(1773),
    [sym_listview_on_row_select_event] = ACTIONS(1773),
    [sym_data_structure_unknown] = ACTIONS(1773),
    [sym_data_structure_datafeed] = ACTIONS(1773),
    [sym_data_structure_datarecordset] = ACTIONS(1773),
    [sym_data_structure_datarecord] = ACTIONS(1773),
    [sym_data_structure_datafield] = ACTIONS(1773),
    [sym_data_max_content_field_count] = ACTIONS(1773),
    [sym_data_max_metadata_field_count] = ACTIONS(1773),
    [sym_data_max_id_length] = ACTIONS(1773),
    [sym_data_max_name_length] = ACTIONS(1773),
    [sym_data_max_description_length] = ACTIONS(1773),
    [sym_data_max_type_length] = ACTIONS(1773),
    [sym_data_max_format_length] = ACTIONS(1773),
    [sym_data_max_source_length] = ACTIONS(1773),
    [sym_data_max_value_length] = ACTIONS(1773),
    [sym_data_max_label_length] = ACTIONS(1773),
    [sym_data_max_url_length] = ACTIONS(1773),
    [sym_max_last_login_info_length] = ACTIONS(1773),
    [sym_data_type_unknown] = ACTIONS(1773),
    [sym_data_type_string] = ACTIONS(1773),
    [sym_data_type_datetime] = ACTIONS(1773),
    [sym_data_type_date] = ACTIONS(1773),
    [sym_data_type_time] = ACTIONS(1773),
    [sym_data_type_image] = ACTIONS(1773),
    [sym_data_format_url] = ACTIONS(1773),
    [sym_data_format_phone] = ACTIONS(1773),
    [sym_data_format_email] = ACTIONS(1773),
    [sym_data_format_iso8601] = ACTIONS(1773),
    [sym_auditlog_priv] = ACTIONS(1773),
    [sym_configuration_priv] = ACTIONS(1773),
    [sym_deviceconfig_priv] = ACTIONS(1773),
    [sym_ftp_priv] = ACTIONS(1773),
    [sym_http_priv] = ACTIONS(1773),
    [sym_networkconfig_priv] = ACTIONS(1773),
    [sym_programport_priv] = ACTIONS(1773),
    [sym_remoteui_priv] = ACTIONS(1773),
    [sym_securitycontrol_priv] = ACTIONS(1773),
    [sym_softwaremanagement_priv] = ACTIONS(1773),
    [sym_terminal_priv] = ACTIONS(1773),
    [sym_tpadmin_priv] = ACTIONS(1773),
    [sym_user1_priv] = ACTIONS(1773),
    [sym_user2_priv] = ACTIONS(1773),
    [sym_user3_priv] = ACTIONS(1773),
    [sym_user4_priv] = ACTIONS(1773),
    [sym_usermanagement_priv] = ACTIONS(1773),
    [sym_valid_account] = ACTIONS(1773),
    [sym_err_name_invalid] = ACTIONS(1773),
    [sym_err_password_invalid] = ACTIONS(1773),
    [sym_err_invalid_account] = ACTIONS(1773),
    [sym_err_no_authorization] = ACTIONS(1773),
    [sym_err_invalid_parameter] = ACTIONS(1773),
    [sym_err_user_locked_out] = ACTIONS(1773),
    [sym_err_user_account_expired] = ACTIONS(1773),
    [sym_err_general_error] = ACTIONS(1773),
    [sym_err_authentication_error] = ACTIONS(1773),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1773),
    [sym_tls_validate_certificate] = ACTIONS(1773),
    [sym_tdata] = ACTIONS(1773),
    [sym_tchannel] = ACTIONS(1773),
    [sym_tlevel] = ACTIONS(1773),
    [sym_tbutton] = ACTIONS(1773),
    [sym_ttimeline] = ACTIONS(1773),
    [sym_tcustom] = ACTIONS(1773),
    [sym_url_struct] = ACTIONS(1773),
    [sym_dns_struct] = ACTIONS(1773),
    [sym_ip_address_struct] = ACTIONS(1773),
    [sym_dev_info_struct] = ACTIONS(1773),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1773),
    [sym_data_feed] = ACTIONS(1773),
    [sym_data_field] = ACTIONS(1773),
    [sym_data_record] = ACTIONS(1773),
    [sym_wc_data_feed] = ACTIONS(1773),
    [sym_wc_data_field] = ACTIONS(1773),
    [sym_wc_data_record] = ACTIONS(1773),
    [sym_last_login_info] = ACTIONS(1773),
    [sym____reserved___] = ACTIONS(1773),
    [sym_date] = ACTIONS(1773),
    [sym_day] = ACTIONS(1773),
    [sym_time] = ACTIONS(1773),
    [sym_ldate] = ACTIONS(1773),
    [sym_push_device] = ACTIONS(1773),
    [sym_push_channel] = ACTIONS(1773),
    [sym_push_devchan] = ACTIONS(1773),
    [sym_release_device] = ACTIONS(1773),
    [sym_release_channel] = ACTIONS(1773),
    [sym_release_devchan] = ACTIONS(1773),
    [sym_master_slot] = ACTIONS(1773),
    [sym_get_pulse_time] = ACTIONS(1773),
    [sym_get_timer] = ACTIONS(1773),
    [sym_master_sn] = ACTIONS(1773),
    [sym_system_number] = ACTIONS(1773),
    [sym_button] = ACTIONS(1773),
    [sym_dv_channel] = ACTIONS(1773),
    [sym_data] = ACTIONS(1773),
    [sym_level] = ACTIONS(1773),
    [sym_timeline] = ACTIONS(1773),
    [sym_channel] = ACTIONS(1773),
    [sym_custom] = ACTIONS(1773),
    [sym_atoi] = ACTIONS(1773),
    [sym_atol] = ACTIONS(1773),
    [sym_atof] = ACTIONS(1773),
    [sym_itoa] = ACTIONS(1773),
    [sym_itohex] = ACTIONS(1773),
    [sym_hextoi] = ACTIONS(1773),
    [sym_ftoa] = ACTIONS(1773),
    [sym_find_string] = ACTIONS(1773),
    [sym_left_string] = ACTIONS(1773),
    [sym_right_string] = ACTIONS(1773),
    [sym_mid_string] = ACTIONS(1773),
    [sym_remove_string] = ACTIONS(1773),
    [sym_get_buffer_string] = ACTIONS(1773),
    [sym_get_buffer_char] = ACTIONS(1773),
    [sym_get_multi_buffer_string] = ACTIONS(1773),
    [sym_length_string] = ACTIONS(1773),
    [sym_max_length_string] = ACTIONS(1773),
    [sym_set_length_string] = ACTIONS(1773),
    [sym_compare_string] = ACTIONS(1773),
    [sym_upper_string] = ACTIONS(1773),
    [sym_lower_string] = ACTIONS(1773),
    [sym_get_last] = ACTIONS(1773),
    [sym_date_to_day] = ACTIONS(1773),
    [sym_date_to_month] = ACTIONS(1773),
    [sym_date_to_year] = ACTIONS(1773),
    [sym_time_to_hour] = ACTIONS(1773),
    [sym_time_to_minute] = ACTIONS(1773),
    [sym_time_to_second] = ACTIONS(1773),
    [sym_day_of_week] = ACTIONS(1773),
    [sym_set_pulse_time] = ACTIONS(1773),
    [sym_set_timer] = ACTIONS(1773),
    [sym_file_open] = ACTIONS(1773),
    [sym_file_close] = ACTIONS(1773),
    [sym_file_read] = ACTIONS(1773),
    [sym_file_read_line] = ACTIONS(1773),
    [sym_file_write] = ACTIONS(1773),
    [sym_file_write_line] = ACTIONS(1773),
    [sym_file_copy] = ACTIONS(1773),
    [sym_file_delete] = ACTIONS(1773),
    [sym_file_rename] = ACTIONS(1773),
    [sym_file_dir] = ACTIONS(1773),
    [sym_file_getdir] = ACTIONS(1773),
    [sym_file_setdir] = ACTIONS(1773),
    [sym_file_create_dir] = ACTIONS(1773),
    [sym_file_remove_dir] = ACTIONS(1773),
    [sym_file_seek] = ACTIONS(1773),
    [sym_length_array] = ACTIONS(1773),
    [sym_max_length_array] = ACTIONS(1773),
    [sym_set_length_array] = ACTIONS(1773),
    [sym_ip_client_open] = ACTIONS(1773),
    [sym_ip_client_close] = ACTIONS(1773),
    [sym_ip_server_open] = ACTIONS(1773),
    [sym_ip_server_close] = ACTIONS(1773),
    [sym_ip_bound_client_open] = ACTIONS(1773),
    [sym_ip_mc_server_open] = ACTIONS(1773),
    [sym_ip_set_option] = ACTIONS(1773),
    [sym_ssh_client_open] = ACTIONS(1773),
    [sym_ssh_client_close] = ACTIONS(1773),
    [sym_tls_client_open] = ACTIONS(1773),
    [sym_tls_client_close] = ACTIONS(1773),
    [sym_do_push] = ACTIONS(1773),
    [sym_do_push_timed] = ACTIONS(1773),
    [sym_do_release] = ACTIONS(1773),
    [sym_device_id] = ACTIONS(1773),
    [sym_device_info] = ACTIONS(1773),
    [sym_device_id_string] = ACTIONS(1773),
    [sym_device_standby] = ACTIONS(1773),
    [sym_device_wake] = ACTIONS(1773),
    [sym_timeline_create] = ACTIONS(1773),
    [sym_timeline_kill] = ACTIONS(1773),
    [sym_timeline_reload] = ACTIONS(1773),
    [sym_timeline_pause] = ACTIONS(1773),
    [sym_timeline_restart] = ACTIONS(1773),
    [sym_timeline_set] = ACTIONS(1773),
    [sym_timeline_get] = ACTIONS(1773),
    [sym_timeline_active] = ACTIONS(1773),
    [sym_timeline_dynamic_id] = ACTIONS(1773),
    [sym_variable_to_string] = ACTIONS(1773),
    [sym_string_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_string] = ACTIONS(1773),
    [sym_type_cast] = ACTIONS(1773),
    [sym_variable_to_xml] = ACTIONS(1773),
    [sym_xml_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_xml] = ACTIONS(1773),
    [sym_redirect_string] = ACTIONS(1773),
    [sym_random_number] = ACTIONS(1773),
    [sym_get_unique_id] = ACTIONS(1773),
    [sym_get_system_number] = ACTIONS(1773),
    [sym_set_system_number] = ACTIONS(1773),
    [sym_get_serial_number] = ACTIONS(1773),
    [sym_get_master_build] = ACTIONS(1773),
    [sym_set_validation_code] = ACTIONS(1773),
    [sym_reboot] = ACTIONS(1773),
    [sym_load_duet_module] = ACTIONS(1773),
    [sym_unload_duet_module] = ACTIONS(1773),
    [sym_duet_mem_size_set] = ACTIONS(1773),
    [sym_duet_mem_size_get] = ACTIONS(1773),
    [sym_dynamic_application_device] = ACTIONS(1773),
    [sym_dynamic_polled_port] = ACTIONS(1773),
    [sym_static_port_binding] = ACTIONS(1773),
    [sym_static_ip_binding] = ACTIONS(1773),
    [sym_get_dns_list] = ACTIONS(1773),
    [sym_set_dns_list] = ACTIONS(1773),
    [sym_get_ip_address] = ACTIONS(1773),
    [sym_set_ip_address] = ACTIONS(1773),
    [sym_get_url_list] = ACTIONS(1773),
    [sym_add_url_entry] = ACTIONS(1773),
    [sym_delete_url_entry] = ACTIONS(1773),
    [sym_abs_value] = ACTIONS(1773),
    [sym_max_value] = ACTIONS(1773),
    [sym_min_value] = ACTIONS(1773),
    [sym_exp_value] = ACTIONS(1773),
    [sym_log_value] = ACTIONS(1773),
    [sym_log10_value] = ACTIONS(1773),
    [sym_power_value] = ACTIONS(1773),
    [sym_sqrt_value] = ACTIONS(1773),
    [sym_raw_be] = ACTIONS(1773),
    [sym_raw_le] = ACTIONS(1773),
    [sym_internal_queue_size_set] = ACTIONS(1773),
    [sym_internal_queue_size_get] = ACTIONS(1773),
    [sym_internal_threshold_set] = ACTIONS(1773),
    [sym_internal_threshold_get] = ACTIONS(1773),
    [sym_rebuilt_event] = ACTIONS(1773),
    [sym_do_custom_event] = ACTIONS(1773),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1773),
    [sym_clkmgr_set_clk_source] = ACTIONS(1773),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1773),
    [sym_clkmgr_get_timezone] = ACTIONS(1773),
    [sym_clkmgr_set_timezone] = ACTIONS(1773),
    [sym_clkmgr_get_resync_period] = ACTIONS(1773),
    [sym_clkmgr_set_resync_period] = ACTIONS(1773),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_timeservers] = ACTIONS(1773),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_set_log_level] = ACTIONS(1773),
    [sym_get_log_level] = ACTIONS(1773),
    [sym_amx_log] = ACTIONS(1773),
    [sym_smtp_server_config_set] = ACTIONS(1773),
    [sym_smtp_server_config_get] = ACTIONS(1773),
    [sym_smtp_send] = ACTIONS(1773),
    [sym_validate_netlinx_account] = ACTIONS(1773),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1773),
    [sym_authenticate_certificate] = ACTIONS(1773),
    [sym_audit_netlinx_session_event] = ACTIONS(1773),
    [sym_audit_netlinx_generic_event] = ACTIONS(1773),
    [sym_get_available_flash_disk_space] = ACTIONS(1773),
    [sym_get_max_flash_disk_space] = ACTIONS(1773),
    [sym_data_create_feed] = ACTIONS(1773),
    [sym_data_delete_feed] = ACTIONS(1773),
    [sym_data_publish_feed] = ACTIONS(1773),
    [sym_data_get_published_feed] = ACTIONS(1773),
    [sym_data_add_record] = ACTIONS(1773),
    [sym_data_get_event_record] = ACTIONS(1773),
    [sym__wc_data_create_feed] = ACTIONS(1773),
    [sym__wc_data_add_record] = ACTIONS(1773),
    [sym__wc_data_get_event_record] = ACTIONS(1773),
    [sym_format] = ACTIONS(1773),
    [sym_astro_clock] = ACTIONS(1773),
    [sym_set_outdoor_temperature] = ACTIONS(1773),
    [sym_set_virtual_level_count] = ACTIONS(1773),
    [sym_set_virtual_channel_count] = ACTIONS(1773),
    [sym_set_virtual_port_count] = ACTIONS(1773),
    [sym__wc_encode_int] = ACTIONS(1773),
    [sym__wc_decode_int] = ACTIONS(1773),
    [sym__wc_int] = ACTIONS(1773),
    [sym__wc_to_ch_int] = ACTIONS(1773),
    [sym__ch_to_wc_int] = ACTIONS(1773),
    [sym__wc_find_string_int] = ACTIONS(1773),
    [sym__wc_left_string_int] = ACTIONS(1773),
    [sym__wc_lower_string_int] = ACTIONS(1773),
    [sym__wc_mid_string_int] = ACTIONS(1773),
    [sym__wc_remove_string_int] = ACTIONS(1773),
    [sym__wc_right_string_int] = ACTIONS(1773),
    [sym__wc_upper_string_int] = ACTIONS(1773),
    [sym__wc_compare_string_int] = ACTIONS(1773),
    [sym__wc_get_buffer_char_int] = ACTIONS(1773),
    [sym__wc_get_buffer_string_int] = ACTIONS(1773),
    [sym__wc_concat_string_int] = ACTIONS(1773),
    [sym__wc_file_open_int] = ACTIONS(1773),
    [sym__wc_file_close_int] = ACTIONS(1773),
    [sym__wc_file_read_int] = ACTIONS(1773),
    [sym__wc_file_read_line_int] = ACTIONS(1773),
    [sym__wc_file_write_int] = ACTIONS(1773),
    [sym__wc_file_write_line_int] = ACTIONS(1773),
    [aux_sym_preproc_include_token1] = ACTIONS(1773),
    [aux_sym_preproc_define_token1] = ACTIONS(1775),
    [aux_sym_preproc_warn_token1] = ACTIONS(1775),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1775),
    [anon_sym_LBRACK] = ACTIONS(1775),
    [anon_sym_LBRACE] = ACTIONS(1775),
    [anon_sym_RBRACE] = ACTIONS(1775),
    [anon_sym_LPAREN] = ACTIONS(1773),
    [anon_sym_BANG] = ACTIONS(1775),
    [anon_sym_TILDE] = ACTIONS(1775),
    [anon_sym_DASH] = ACTIONS(1773),
    [anon_sym_PLUS] = ACTIONS(1773),
    [anon_sym_DASH_DASH] = ACTIONS(1775),
    [anon_sym_PLUS_PLUS] = ACTIONS(1775),
    [anon_sym_DQUOTE] = ACTIONS(1775),
    [anon_sym_SQUOTE] = ACTIONS(1775),
    [sym_number_literal] = ACTIONS(1775),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(499)] = {
    [sym_identifier] = ACTIONS(1773),
    [sym_define_call_keyword] = ACTIONS(1773),
    [sym_define_function_keyword] = ACTIONS(1773),
    [sym_define_library_function_keyword] = ACTIONS(1773),
    [sym_define_module_keyword] = ACTIONS(1773),
    [sym_char_keyword] = ACTIONS(1773),
    [sym_widechar_keyword] = ACTIONS(1773),
    [sym_integer_keyword] = ACTIONS(1773),
    [sym_sinteger_keyword] = ACTIONS(1773),
    [sym_long_keyword] = ACTIONS(1773),
    [sym_slong_keyword] = ACTIONS(1773),
    [sym_float_keyword] = ACTIONS(1773),
    [sym_double_keyword] = ACTIONS(1773),
    [sym_variant_keyword] = ACTIONS(1773),
    [sym_variantarray_keyword] = ACTIONS(1773),
    [sym_local_var_keyword] = ACTIONS(1773),
    [sym_stack_var_keyword] = ACTIONS(1773),
    [sym_constant_keyword] = ACTIONS(1773),
    [sym_volatile_keyword] = ACTIONS(1773),
    [sym_non_volatile_keyword] = ACTIONS(1773),
    [sym_persistent_keyword] = ACTIONS(1773),
    [sym_dev_keyword] = ACTIONS(1773),
    [sym_devlev_keyword] = ACTIONS(1773),
    [sym_devchan_keyword] = ACTIONS(1773),
    [sym_if_keyword] = ACTIONS(1773),
    [sym_else_keyword] = ACTIONS(1773),
    [sym_switch_keyword] = ACTIONS(1773),
    [sym_case_keyword] = ACTIONS(1773),
    [sym_default_keyword] = ACTIONS(1773),
    [sym_while_keyword] = ACTIONS(1773),
    [sym_for_keyword] = ACTIONS(1773),
    [sym_break_keyword] = ACTIONS(1773),
    [sym_continue_keyword] = ACTIONS(1773),
    [sym_return_keyword] = ACTIONS(1773),
    [sym_select_keyword] = ACTIONS(1773),
    [sym_active_keyword] = ACTIONS(1773),
    [sym_struct_keyword] = ACTIONS(1773),
    [sym_structure_keyword] = ACTIONS(1773),
    [aux_sym_on_keyword_token1] = ACTIONS(1773),
    [aux_sym_off_keyword_token1] = ACTIONS(1773),
    [sym_send_level_keyword] = ACTIONS(1773),
    [sym_send_string_keyword] = ACTIONS(1773),
    [sym_send_command_keyword] = ACTIONS(1773),
    [sym_clear_buffer_keyword] = ACTIONS(1773),
    [sym_create_buffer_keyword] = ACTIONS(1773),
    [sym_create_multi_buffer_keyword] = ACTIONS(1773),
    [sym_call_keyword] = ACTIONS(1773),
    [sym_system_call_keyword] = ACTIONS(1773),
    [sym_devchan_to_keyword] = ACTIONS(1773),
    [sym_devchan_min_to_keyword] = ACTIONS(1773),
    [sym_devchan_total_off_keyword] = ACTIONS(1773),
    [sym_devchan_pulse_keyword] = ACTIONS(1773),
    [sym_wait_keyword] = ACTIONS(1773),
    [sym_cancel_wait_keyword] = ACTIONS(1773),
    [sym_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1773),
    [sym_not] = ACTIONS(1773),
    [sym_bnot] = ACTIONS(1773),
    [sym___file__] = ACTIONS(1773),
    [sym___date__] = ACTIONS(1773),
    [sym___ldate__] = ACTIONS(1773),
    [sym___time__] = ACTIONS(1773),
    [sym___line__] = ACTIONS(1773),
    [sym___name__] = ACTIONS(1773),
    [sym___netlinx__] = ACTIONS(1773),
    [sym_netlinx_axi_version] = ACTIONS(1773),
    [sym_url_flg_tcp] = ACTIONS(1773),
    [sym_url_flg_accinfopresent] = ACTIONS(1773),
    [sym_url_flg_temp] = ACTIONS(1773),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1773),
    [sym_url_flg_stat_mask] = ACTIONS(1773),
    [sym_url_flg_stat_lookup] = ACTIONS(1773),
    [sym_url_flg_stat_connecting] = ACTIONS(1773),
    [sym_url_flg_stat_waiting] = ACTIONS(1773),
    [sym_url_flg_stat_connected] = ACTIONS(1773),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1773),
    [sym_first_virtual_device] = ACTIONS(1773),
    [sym_first_local_port] = ACTIONS(1773),
    [sym_dynamic_virtual_device] = ACTIONS(1773),
    [sym_timeline_once] = ACTIONS(1773),
    [sym_timeline_repeat] = ACTIONS(1773),
    [sym_timeline_absolute] = ACTIONS(1773),
    [sym_timeline_relative] = ACTIONS(1773),
    [sym_do_push_timed_infinite] = ACTIONS(1773),
    [sym_source_type_no_address] = ACTIONS(1773),
    [sym_source_type_neuron_id] = ACTIONS(1773),
    [sym_source_type_ip_address] = ACTIONS(1773),
    [sym_source_type_axlink] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1773),
    [sym_source_type_ip_socket_address] = ACTIONS(1773),
    [sym_source_type_rs232] = ACTIONS(1773),
    [sym_source_type_internal] = ACTIONS(1773),
    [sym_source_type_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1773),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1773),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1773),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1773),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_file_read_only] = ACTIONS(1773),
    [sym_file_rw_new] = ACTIONS(1773),
    [sym_file_rw_append] = ACTIONS(1773),
    [sym_ip_tcp] = ACTIONS(1773),
    [sym_ip_udp] = ACTIONS(1773),
    [sym_ip_udp_2way] = ACTIONS(1773),
    [sym_xml_encode_types] = ACTIONS(1773),
    [sym_xml_encode_char_as_list] = ACTIONS(1773),
    [sym_xml_encode_le] = ACTIONS(1773),
    [sym_xml_decode_types] = ACTIONS(1773),
    [sym_xml_decode_no_preserve] = ACTIONS(1773),
    [sym_duet_dev_type_display_device] = ACTIONS(1773),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_io_device] = ACTIONS(1773),
    [sym_duet_dev_type_relay_device] = ACTIONS(1773),
    [sym_duet_dev_type_ups] = ACTIONS(1773),
    [sym_duet_dev_type_amplifier] = ACTIONS(1773),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1773),
    [sym_duet_dev_type_camera] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1773),
    [sym_duet_dev_type_dss] = ACTIONS(1773),
    [sym_duet_dev_type_dvr] = ACTIONS(1773),
    [sym_duet_dev_type_disc_device] = ACTIONS(1773),
    [sym_duet_dev_type_document_camera] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1773),
    [sym_duet_dev_type_hvac] = ACTIONS(1773),
    [sym_duet_dev_type_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_light] = ACTIONS(1773),
    [sym_duet_dev_type_monitor] = ACTIONS(1773),
    [sym_duet_dev_type_motor] = ACTIONS(1773),
    [sym_duet_dev_type_multi_window] = ACTIONS(1773),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1773),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1773),
    [sym_duet_dev_type_receiver] = ACTIONS(1773),
    [sym_duet_dev_type_security_system] = ACTIONS(1773),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1773),
    [sym_duet_dev_type_settop_box] = ACTIONS(1773),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1773),
    [sym_duet_dev_type_switcher] = ACTIONS(1773),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_tv] = ACTIONS(1773),
    [sym_duet_dev_type_utility] = ACTIONS(1773),
    [sym_duet_dev_type_vcr] = ACTIONS(1773),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_video_processor] = ACTIONS(1773),
    [sym_duet_dev_type_video_projector] = ACTIONS(1773),
    [sym_duet_dev_type_video_wall] = ACTIONS(1773),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1773),
    [sym_duet_dev_type_weather] = ACTIONS(1773),
    [sym_duet_dev_not_polled] = ACTIONS(1773),
    [sym_duet_dev_polled] = ACTIONS(1773),
    [sym_clkmgr_mode_network] = ACTIONS(1773),
    [sym_clkmgr_mode_standalone] = ACTIONS(1773),
    [sym_normal_standby] = ACTIONS(1773),
    [sym_normal_wake] = ACTIONS(1773),
    [sym_ip_multicast_ttl_option] = ACTIONS(1773),
    [sym_ip_ttl_subnet] = ACTIONS(1773),
    [sym_ip_ttl_site] = ACTIONS(1773),
    [sym_ip_ttl_region] = ACTIONS(1773),
    [sym_ip_ttl_continent] = ACTIONS(1773),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1773),
    [sym_ip_nodelay_on] = ACTIONS(1773),
    [sym_ip_nodelay_off] = ACTIONS(1773),
    [sym_amx_error] = ACTIONS(1773),
    [sym_amx_warning] = ACTIONS(1773),
    [sym_amx_info] = ACTIONS(1773),
    [sym_amx_debug] = ACTIONS(1773),
    [sym_netlinx_login_success] = ACTIONS(1773),
    [sym_netlinx_login_fail] = ACTIONS(1773),
    [sym_netlinx_logout] = ACTIONS(1773),
    [sym_null_str] = ACTIONS(1773),
    [sym_smtp_address] = ACTIONS(1773),
    [sym_smtp_port_number] = ACTIONS(1773),
    [sym_smtp_username] = ACTIONS(1773),
    [sym_smtp_password] = ACTIONS(1773),
    [sym_smtp_require_tls] = ACTIONS(1773),
    [sym_smtp_from] = ACTIONS(1773),
    [sym_smtp_tls_true] = ACTIONS(1773),
    [sym_smtp_tls_false] = ACTIONS(1773),
    [sym_listview_on_row_select_event] = ACTIONS(1773),
    [sym_data_structure_unknown] = ACTIONS(1773),
    [sym_data_structure_datafeed] = ACTIONS(1773),
    [sym_data_structure_datarecordset] = ACTIONS(1773),
    [sym_data_structure_datarecord] = ACTIONS(1773),
    [sym_data_structure_datafield] = ACTIONS(1773),
    [sym_data_max_content_field_count] = ACTIONS(1773),
    [sym_data_max_metadata_field_count] = ACTIONS(1773),
    [sym_data_max_id_length] = ACTIONS(1773),
    [sym_data_max_name_length] = ACTIONS(1773),
    [sym_data_max_description_length] = ACTIONS(1773),
    [sym_data_max_type_length] = ACTIONS(1773),
    [sym_data_max_format_length] = ACTIONS(1773),
    [sym_data_max_source_length] = ACTIONS(1773),
    [sym_data_max_value_length] = ACTIONS(1773),
    [sym_data_max_label_length] = ACTIONS(1773),
    [sym_data_max_url_length] = ACTIONS(1773),
    [sym_max_last_login_info_length] = ACTIONS(1773),
    [sym_data_type_unknown] = ACTIONS(1773),
    [sym_data_type_string] = ACTIONS(1773),
    [sym_data_type_datetime] = ACTIONS(1773),
    [sym_data_type_date] = ACTIONS(1773),
    [sym_data_type_time] = ACTIONS(1773),
    [sym_data_type_image] = ACTIONS(1773),
    [sym_data_format_url] = ACTIONS(1773),
    [sym_data_format_phone] = ACTIONS(1773),
    [sym_data_format_email] = ACTIONS(1773),
    [sym_data_format_iso8601] = ACTIONS(1773),
    [sym_auditlog_priv] = ACTIONS(1773),
    [sym_configuration_priv] = ACTIONS(1773),
    [sym_deviceconfig_priv] = ACTIONS(1773),
    [sym_ftp_priv] = ACTIONS(1773),
    [sym_http_priv] = ACTIONS(1773),
    [sym_networkconfig_priv] = ACTIONS(1773),
    [sym_programport_priv] = ACTIONS(1773),
    [sym_remoteui_priv] = ACTIONS(1773),
    [sym_securitycontrol_priv] = ACTIONS(1773),
    [sym_softwaremanagement_priv] = ACTIONS(1773),
    [sym_terminal_priv] = ACTIONS(1773),
    [sym_tpadmin_priv] = ACTIONS(1773),
    [sym_user1_priv] = ACTIONS(1773),
    [sym_user2_priv] = ACTIONS(1773),
    [sym_user3_priv] = ACTIONS(1773),
    [sym_user4_priv] = ACTIONS(1773),
    [sym_usermanagement_priv] = ACTIONS(1773),
    [sym_valid_account] = ACTIONS(1773),
    [sym_err_name_invalid] = ACTIONS(1773),
    [sym_err_password_invalid] = ACTIONS(1773),
    [sym_err_invalid_account] = ACTIONS(1773),
    [sym_err_no_authorization] = ACTIONS(1773),
    [sym_err_invalid_parameter] = ACTIONS(1773),
    [sym_err_user_locked_out] = ACTIONS(1773),
    [sym_err_user_account_expired] = ACTIONS(1773),
    [sym_err_general_error] = ACTIONS(1773),
    [sym_err_authentication_error] = ACTIONS(1773),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1773),
    [sym_tls_validate_certificate] = ACTIONS(1773),
    [sym_tdata] = ACTIONS(1773),
    [sym_tchannel] = ACTIONS(1773),
    [sym_tlevel] = ACTIONS(1773),
    [sym_tbutton] = ACTIONS(1773),
    [sym_ttimeline] = ACTIONS(1773),
    [sym_tcustom] = ACTIONS(1773),
    [sym_url_struct] = ACTIONS(1773),
    [sym_dns_struct] = ACTIONS(1773),
    [sym_ip_address_struct] = ACTIONS(1773),
    [sym_dev_info_struct] = ACTIONS(1773),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1773),
    [sym_data_feed] = ACTIONS(1773),
    [sym_data_field] = ACTIONS(1773),
    [sym_data_record] = ACTIONS(1773),
    [sym_wc_data_feed] = ACTIONS(1773),
    [sym_wc_data_field] = ACTIONS(1773),
    [sym_wc_data_record] = ACTIONS(1773),
    [sym_last_login_info] = ACTIONS(1773),
    [sym____reserved___] = ACTIONS(1773),
    [sym_date] = ACTIONS(1773),
    [sym_day] = ACTIONS(1773),
    [sym_time] = ACTIONS(1773),
    [sym_ldate] = ACTIONS(1773),
    [sym_push_device] = ACTIONS(1773),
    [sym_push_channel] = ACTIONS(1773),
    [sym_push_devchan] = ACTIONS(1773),
    [sym_release_device] = ACTIONS(1773),
    [sym_release_channel] = ACTIONS(1773),
    [sym_release_devchan] = ACTIONS(1773),
    [sym_master_slot] = ACTIONS(1773),
    [sym_get_pulse_time] = ACTIONS(1773),
    [sym_get_timer] = ACTIONS(1773),
    [sym_master_sn] = ACTIONS(1773),
    [sym_system_number] = ACTIONS(1773),
    [sym_button] = ACTIONS(1773),
    [sym_dv_channel] = ACTIONS(1773),
    [sym_data] = ACTIONS(1773),
    [sym_level] = ACTIONS(1773),
    [sym_timeline] = ACTIONS(1773),
    [sym_channel] = ACTIONS(1773),
    [sym_custom] = ACTIONS(1773),
    [sym_atoi] = ACTIONS(1773),
    [sym_atol] = ACTIONS(1773),
    [sym_atof] = ACTIONS(1773),
    [sym_itoa] = ACTIONS(1773),
    [sym_itohex] = ACTIONS(1773),
    [sym_hextoi] = ACTIONS(1773),
    [sym_ftoa] = ACTIONS(1773),
    [sym_find_string] = ACTIONS(1773),
    [sym_left_string] = ACTIONS(1773),
    [sym_right_string] = ACTIONS(1773),
    [sym_mid_string] = ACTIONS(1773),
    [sym_remove_string] = ACTIONS(1773),
    [sym_get_buffer_string] = ACTIONS(1773),
    [sym_get_buffer_char] = ACTIONS(1773),
    [sym_get_multi_buffer_string] = ACTIONS(1773),
    [sym_length_string] = ACTIONS(1773),
    [sym_max_length_string] = ACTIONS(1773),
    [sym_set_length_string] = ACTIONS(1773),
    [sym_compare_string] = ACTIONS(1773),
    [sym_upper_string] = ACTIONS(1773),
    [sym_lower_string] = ACTIONS(1773),
    [sym_get_last] = ACTIONS(1773),
    [sym_date_to_day] = ACTIONS(1773),
    [sym_date_to_month] = ACTIONS(1773),
    [sym_date_to_year] = ACTIONS(1773),
    [sym_time_to_hour] = ACTIONS(1773),
    [sym_time_to_minute] = ACTIONS(1773),
    [sym_time_to_second] = ACTIONS(1773),
    [sym_day_of_week] = ACTIONS(1773),
    [sym_set_pulse_time] = ACTIONS(1773),
    [sym_set_timer] = ACTIONS(1773),
    [sym_file_open] = ACTIONS(1773),
    [sym_file_close] = ACTIONS(1773),
    [sym_file_read] = ACTIONS(1773),
    [sym_file_read_line] = ACTIONS(1773),
    [sym_file_write] = ACTIONS(1773),
    [sym_file_write_line] = ACTIONS(1773),
    [sym_file_copy] = ACTIONS(1773),
    [sym_file_delete] = ACTIONS(1773),
    [sym_file_rename] = ACTIONS(1773),
    [sym_file_dir] = ACTIONS(1773),
    [sym_file_getdir] = ACTIONS(1773),
    [sym_file_setdir] = ACTIONS(1773),
    [sym_file_create_dir] = ACTIONS(1773),
    [sym_file_remove_dir] = ACTIONS(1773),
    [sym_file_seek] = ACTIONS(1773),
    [sym_length_array] = ACTIONS(1773),
    [sym_max_length_array] = ACTIONS(1773),
    [sym_set_length_array] = ACTIONS(1773),
    [sym_ip_client_open] = ACTIONS(1773),
    [sym_ip_client_close] = ACTIONS(1773),
    [sym_ip_server_open] = ACTIONS(1773),
    [sym_ip_server_close] = ACTIONS(1773),
    [sym_ip_bound_client_open] = ACTIONS(1773),
    [sym_ip_mc_server_open] = ACTIONS(1773),
    [sym_ip_set_option] = ACTIONS(1773),
    [sym_ssh_client_open] = ACTIONS(1773),
    [sym_ssh_client_close] = ACTIONS(1773),
    [sym_tls_client_open] = ACTIONS(1773),
    [sym_tls_client_close] = ACTIONS(1773),
    [sym_do_push] = ACTIONS(1773),
    [sym_do_push_timed] = ACTIONS(1773),
    [sym_do_release] = ACTIONS(1773),
    [sym_device_id] = ACTIONS(1773),
    [sym_device_info] = ACTIONS(1773),
    [sym_device_id_string] = ACTIONS(1773),
    [sym_device_standby] = ACTIONS(1773),
    [sym_device_wake] = ACTIONS(1773),
    [sym_timeline_create] = ACTIONS(1773),
    [sym_timeline_kill] = ACTIONS(1773),
    [sym_timeline_reload] = ACTIONS(1773),
    [sym_timeline_pause] = ACTIONS(1773),
    [sym_timeline_restart] = ACTIONS(1773),
    [sym_timeline_set] = ACTIONS(1773),
    [sym_timeline_get] = ACTIONS(1773),
    [sym_timeline_active] = ACTIONS(1773),
    [sym_timeline_dynamic_id] = ACTIONS(1773),
    [sym_variable_to_string] = ACTIONS(1773),
    [sym_string_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_string] = ACTIONS(1773),
    [sym_type_cast] = ACTIONS(1773),
    [sym_variable_to_xml] = ACTIONS(1773),
    [sym_xml_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_xml] = ACTIONS(1773),
    [sym_redirect_string] = ACTIONS(1773),
    [sym_random_number] = ACTIONS(1773),
    [sym_get_unique_id] = ACTIONS(1773),
    [sym_get_system_number] = ACTIONS(1773),
    [sym_set_system_number] = ACTIONS(1773),
    [sym_get_serial_number] = ACTIONS(1773),
    [sym_get_master_build] = ACTIONS(1773),
    [sym_set_validation_code] = ACTIONS(1773),
    [sym_reboot] = ACTIONS(1773),
    [sym_load_duet_module] = ACTIONS(1773),
    [sym_unload_duet_module] = ACTIONS(1773),
    [sym_duet_mem_size_set] = ACTIONS(1773),
    [sym_duet_mem_size_get] = ACTIONS(1773),
    [sym_dynamic_application_device] = ACTIONS(1773),
    [sym_dynamic_polled_port] = ACTIONS(1773),
    [sym_static_port_binding] = ACTIONS(1773),
    [sym_static_ip_binding] = ACTIONS(1773),
    [sym_get_dns_list] = ACTIONS(1773),
    [sym_set_dns_list] = ACTIONS(1773),
    [sym_get_ip_address] = ACTIONS(1773),
    [sym_set_ip_address] = ACTIONS(1773),
    [sym_get_url_list] = ACTIONS(1773),
    [sym_add_url_entry] = ACTIONS(1773),
    [sym_delete_url_entry] = ACTIONS(1773),
    [sym_abs_value] = ACTIONS(1773),
    [sym_max_value] = ACTIONS(1773),
    [sym_min_value] = ACTIONS(1773),
    [sym_exp_value] = ACTIONS(1773),
    [sym_log_value] = ACTIONS(1773),
    [sym_log10_value] = ACTIONS(1773),
    [sym_power_value] = ACTIONS(1773),
    [sym_sqrt_value] = ACTIONS(1773),
    [sym_raw_be] = ACTIONS(1773),
    [sym_raw_le] = ACTIONS(1773),
    [sym_internal_queue_size_set] = ACTIONS(1773),
    [sym_internal_queue_size_get] = ACTIONS(1773),
    [sym_internal_threshold_set] = ACTIONS(1773),
    [sym_internal_threshold_get] = ACTIONS(1773),
    [sym_rebuilt_event] = ACTIONS(1773),
    [sym_do_custom_event] = ACTIONS(1773),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1773),
    [sym_clkmgr_set_clk_source] = ACTIONS(1773),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1773),
    [sym_clkmgr_get_timezone] = ACTIONS(1773),
    [sym_clkmgr_set_timezone] = ACTIONS(1773),
    [sym_clkmgr_get_resync_period] = ACTIONS(1773),
    [sym_clkmgr_set_resync_period] = ACTIONS(1773),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_timeservers] = ACTIONS(1773),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_set_log_level] = ACTIONS(1773),
    [sym_get_log_level] = ACTIONS(1773),
    [sym_amx_log] = ACTIONS(1773),
    [sym_smtp_server_config_set] = ACTIONS(1773),
    [sym_smtp_server_config_get] = ACTIONS(1773),
    [sym_smtp_send] = ACTIONS(1773),
    [sym_validate_netlinx_account] = ACTIONS(1773),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1773),
    [sym_authenticate_certificate] = ACTIONS(1773),
    [sym_audit_netlinx_session_event] = ACTIONS(1773),
    [sym_audit_netlinx_generic_event] = ACTIONS(1773),
    [sym_get_available_flash_disk_space] = ACTIONS(1773),
    [sym_get_max_flash_disk_space] = ACTIONS(1773),
    [sym_data_create_feed] = ACTIONS(1773),
    [sym_data_delete_feed] = ACTIONS(1773),
    [sym_data_publish_feed] = ACTIONS(1773),
    [sym_data_get_published_feed] = ACTIONS(1773),
    [sym_data_add_record] = ACTIONS(1773),
    [sym_data_get_event_record] = ACTIONS(1773),
    [sym__wc_data_create_feed] = ACTIONS(1773),
    [sym__wc_data_add_record] = ACTIONS(1773),
    [sym__wc_data_get_event_record] = ACTIONS(1773),
    [sym_format] = ACTIONS(1773),
    [sym_astro_clock] = ACTIONS(1773),
    [sym_set_outdoor_temperature] = ACTIONS(1773),
    [sym_set_virtual_level_count] = ACTIONS(1773),
    [sym_set_virtual_channel_count] = ACTIONS(1773),
    [sym_set_virtual_port_count] = ACTIONS(1773),
    [sym__wc_encode_int] = ACTIONS(1773),
    [sym__wc_decode_int] = ACTIONS(1773),
    [sym__wc_int] = ACTIONS(1773),
    [sym__wc_to_ch_int] = ACTIONS(1773),
    [sym__ch_to_wc_int] = ACTIONS(1773),
    [sym__wc_find_string_int] = ACTIONS(1773),
    [sym__wc_left_string_int] = ACTIONS(1773),
    [sym__wc_lower_string_int] = ACTIONS(1773),
    [sym__wc_mid_string_int] = ACTIONS(1773),
    [sym__wc_remove_string_int] = ACTIONS(1773),
    [sym__wc_right_string_int] = ACTIONS(1773),
    [sym__wc_upper_string_int] = ACTIONS(1773),
    [sym__wc_compare_string_int] = ACTIONS(1773),
    [sym__wc_get_buffer_char_int] = ACTIONS(1773),
    [sym__wc_get_buffer_string_int] = ACTIONS(1773),
    [sym__wc_concat_string_int] = ACTIONS(1773),
    [sym__wc_file_open_int] = ACTIONS(1773),
    [sym__wc_file_close_int] = ACTIONS(1773),
    [sym__wc_file_read_int] = ACTIONS(1773),
    [sym__wc_file_read_line_int] = ACTIONS(1773),
    [sym__wc_file_write_int] = ACTIONS(1773),
    [sym__wc_file_write_line_int] = ACTIONS(1773),
    [aux_sym_preproc_include_token1] = ACTIONS(1773),
    [aux_sym_preproc_define_token1] = ACTIONS(1775),
    [aux_sym_preproc_warn_token1] = ACTIONS(1775),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1775),
    [anon_sym_LBRACK] = ACTIONS(1775),
    [anon_sym_LBRACE] = ACTIONS(1775),
    [anon_sym_RBRACE] = ACTIONS(1775),
    [anon_sym_LPAREN] = ACTIONS(1773),
    [anon_sym_BANG] = ACTIONS(1775),
    [anon_sym_TILDE] = ACTIONS(1775),
    [anon_sym_DASH] = ACTIONS(1773),
    [anon_sym_PLUS] = ACTIONS(1773),
    [anon_sym_DASH_DASH] = ACTIONS(1775),
    [anon_sym_PLUS_PLUS] = ACTIONS(1775),
    [anon_sym_DQUOTE] = ACTIONS(1775),
    [anon_sym_SQUOTE] = ACTIONS(1775),
    [sym_number_literal] = ACTIONS(1775),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(500)] = {
    [sym_identifier] = ACTIONS(1773),
    [sym_define_call_keyword] = ACTIONS(1773),
    [sym_define_function_keyword] = ACTIONS(1773),
    [sym_define_library_function_keyword] = ACTIONS(1773),
    [sym_define_module_keyword] = ACTIONS(1773),
    [sym_char_keyword] = ACTIONS(1773),
    [sym_widechar_keyword] = ACTIONS(1773),
    [sym_integer_keyword] = ACTIONS(1773),
    [sym_sinteger_keyword] = ACTIONS(1773),
    [sym_long_keyword] = ACTIONS(1773),
    [sym_slong_keyword] = ACTIONS(1773),
    [sym_float_keyword] = ACTIONS(1773),
    [sym_double_keyword] = ACTIONS(1773),
    [sym_variant_keyword] = ACTIONS(1773),
    [sym_variantarray_keyword] = ACTIONS(1773),
    [sym_local_var_keyword] = ACTIONS(1773),
    [sym_stack_var_keyword] = ACTIONS(1773),
    [sym_constant_keyword] = ACTIONS(1773),
    [sym_volatile_keyword] = ACTIONS(1773),
    [sym_non_volatile_keyword] = ACTIONS(1773),
    [sym_persistent_keyword] = ACTIONS(1773),
    [sym_dev_keyword] = ACTIONS(1773),
    [sym_devlev_keyword] = ACTIONS(1773),
    [sym_devchan_keyword] = ACTIONS(1773),
    [sym_if_keyword] = ACTIONS(1773),
    [sym_else_keyword] = ACTIONS(1773),
    [sym_switch_keyword] = ACTIONS(1773),
    [sym_case_keyword] = ACTIONS(1773),
    [sym_default_keyword] = ACTIONS(1773),
    [sym_while_keyword] = ACTIONS(1773),
    [sym_for_keyword] = ACTIONS(1773),
    [sym_break_keyword] = ACTIONS(1773),
    [sym_continue_keyword] = ACTIONS(1773),
    [sym_return_keyword] = ACTIONS(1773),
    [sym_select_keyword] = ACTIONS(1773),
    [sym_active_keyword] = ACTIONS(1773),
    [sym_struct_keyword] = ACTIONS(1773),
    [sym_structure_keyword] = ACTIONS(1773),
    [aux_sym_on_keyword_token1] = ACTIONS(1773),
    [aux_sym_off_keyword_token1] = ACTIONS(1773),
    [sym_send_level_keyword] = ACTIONS(1773),
    [sym_send_string_keyword] = ACTIONS(1773),
    [sym_send_command_keyword] = ACTIONS(1773),
    [sym_clear_buffer_keyword] = ACTIONS(1773),
    [sym_create_buffer_keyword] = ACTIONS(1773),
    [sym_create_multi_buffer_keyword] = ACTIONS(1773),
    [sym_call_keyword] = ACTIONS(1773),
    [sym_system_call_keyword] = ACTIONS(1773),
    [sym_devchan_to_keyword] = ACTIONS(1773),
    [sym_devchan_min_to_keyword] = ACTIONS(1773),
    [sym_devchan_total_off_keyword] = ACTIONS(1773),
    [sym_devchan_pulse_keyword] = ACTIONS(1773),
    [sym_wait_keyword] = ACTIONS(1773),
    [sym_cancel_wait_keyword] = ACTIONS(1773),
    [sym_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1773),
    [sym_not] = ACTIONS(1773),
    [sym_bnot] = ACTIONS(1773),
    [sym___file__] = ACTIONS(1773),
    [sym___date__] = ACTIONS(1773),
    [sym___ldate__] = ACTIONS(1773),
    [sym___time__] = ACTIONS(1773),
    [sym___line__] = ACTIONS(1773),
    [sym___name__] = ACTIONS(1773),
    [sym___netlinx__] = ACTIONS(1773),
    [sym_netlinx_axi_version] = ACTIONS(1773),
    [sym_url_flg_tcp] = ACTIONS(1773),
    [sym_url_flg_accinfopresent] = ACTIONS(1773),
    [sym_url_flg_temp] = ACTIONS(1773),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1773),
    [sym_url_flg_stat_mask] = ACTIONS(1773),
    [sym_url_flg_stat_lookup] = ACTIONS(1773),
    [sym_url_flg_stat_connecting] = ACTIONS(1773),
    [sym_url_flg_stat_waiting] = ACTIONS(1773),
    [sym_url_flg_stat_connected] = ACTIONS(1773),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1773),
    [sym_first_virtual_device] = ACTIONS(1773),
    [sym_first_local_port] = ACTIONS(1773),
    [sym_dynamic_virtual_device] = ACTIONS(1773),
    [sym_timeline_once] = ACTIONS(1773),
    [sym_timeline_repeat] = ACTIONS(1773),
    [sym_timeline_absolute] = ACTIONS(1773),
    [sym_timeline_relative] = ACTIONS(1773),
    [sym_do_push_timed_infinite] = ACTIONS(1773),
    [sym_source_type_no_address] = ACTIONS(1773),
    [sym_source_type_neuron_id] = ACTIONS(1773),
    [sym_source_type_ip_address] = ACTIONS(1773),
    [sym_source_type_axlink] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1773),
    [sym_source_type_ip_socket_address] = ACTIONS(1773),
    [sym_source_type_rs232] = ACTIONS(1773),
    [sym_source_type_internal] = ACTIONS(1773),
    [sym_source_type_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1773),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1773),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1773),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1773),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_file_read_only] = ACTIONS(1773),
    [sym_file_rw_new] = ACTIONS(1773),
    [sym_file_rw_append] = ACTIONS(1773),
    [sym_ip_tcp] = ACTIONS(1773),
    [sym_ip_udp] = ACTIONS(1773),
    [sym_ip_udp_2way] = ACTIONS(1773),
    [sym_xml_encode_types] = ACTIONS(1773),
    [sym_xml_encode_char_as_list] = ACTIONS(1773),
    [sym_xml_encode_le] = ACTIONS(1773),
    [sym_xml_decode_types] = ACTIONS(1773),
    [sym_xml_decode_no_preserve] = ACTIONS(1773),
    [sym_duet_dev_type_display_device] = ACTIONS(1773),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_io_device] = ACTIONS(1773),
    [sym_duet_dev_type_relay_device] = ACTIONS(1773),
    [sym_duet_dev_type_ups] = ACTIONS(1773),
    [sym_duet_dev_type_amplifier] = ACTIONS(1773),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1773),
    [sym_duet_dev_type_camera] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1773),
    [sym_duet_dev_type_dss] = ACTIONS(1773),
    [sym_duet_dev_type_dvr] = ACTIONS(1773),
    [sym_duet_dev_type_disc_device] = ACTIONS(1773),
    [sym_duet_dev_type_document_camera] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1773),
    [sym_duet_dev_type_hvac] = ACTIONS(1773),
    [sym_duet_dev_type_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_light] = ACTIONS(1773),
    [sym_duet_dev_type_monitor] = ACTIONS(1773),
    [sym_duet_dev_type_motor] = ACTIONS(1773),
    [sym_duet_dev_type_multi_window] = ACTIONS(1773),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1773),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1773),
    [sym_duet_dev_type_receiver] = ACTIONS(1773),
    [sym_duet_dev_type_security_system] = ACTIONS(1773),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1773),
    [sym_duet_dev_type_settop_box] = ACTIONS(1773),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1773),
    [sym_duet_dev_type_switcher] = ACTIONS(1773),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_tv] = ACTIONS(1773),
    [sym_duet_dev_type_utility] = ACTIONS(1773),
    [sym_duet_dev_type_vcr] = ACTIONS(1773),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_video_processor] = ACTIONS(1773),
    [sym_duet_dev_type_video_projector] = ACTIONS(1773),
    [sym_duet_dev_type_video_wall] = ACTIONS(1773),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1773),
    [sym_duet_dev_type_weather] = ACTIONS(1773),
    [sym_duet_dev_not_polled] = ACTIONS(1773),
    [sym_duet_dev_polled] = ACTIONS(1773),
    [sym_clkmgr_mode_network] = ACTIONS(1773),
    [sym_clkmgr_mode_standalone] = ACTIONS(1773),
    [sym_normal_standby] = ACTIONS(1773),
    [sym_normal_wake] = ACTIONS(1773),
    [sym_ip_multicast_ttl_option] = ACTIONS(1773),
    [sym_ip_ttl_subnet] = ACTIONS(1773),
    [sym_ip_ttl_site] = ACTIONS(1773),
    [sym_ip_ttl_region] = ACTIONS(1773),
    [sym_ip_ttl_continent] = ACTIONS(1773),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1773),
    [sym_ip_nodelay_on] = ACTIONS(1773),
    [sym_ip_nodelay_off] = ACTIONS(1773),
    [sym_amx_error] = ACTIONS(1773),
    [sym_amx_warning] = ACTIONS(1773),
    [sym_amx_info] = ACTIONS(1773),
    [sym_amx_debug] = ACTIONS(1773),
    [sym_netlinx_login_success] = ACTIONS(1773),
    [sym_netlinx_login_fail] = ACTIONS(1773),
    [sym_netlinx_logout] = ACTIONS(1773),
    [sym_null_str] = ACTIONS(1773),
    [sym_smtp_address] = ACTIONS(1773),
    [sym_smtp_port_number] = ACTIONS(1773),
    [sym_smtp_username] = ACTIONS(1773),
    [sym_smtp_password] = ACTIONS(1773),
    [sym_smtp_require_tls] = ACTIONS(1773),
    [sym_smtp_from] = ACTIONS(1773),
    [sym_smtp_tls_true] = ACTIONS(1773),
    [sym_smtp_tls_false] = ACTIONS(1773),
    [sym_listview_on_row_select_event] = ACTIONS(1773),
    [sym_data_structure_unknown] = ACTIONS(1773),
    [sym_data_structure_datafeed] = ACTIONS(1773),
    [sym_data_structure_datarecordset] = ACTIONS(1773),
    [sym_data_structure_datarecord] = ACTIONS(1773),
    [sym_data_structure_datafield] = ACTIONS(1773),
    [sym_data_max_content_field_count] = ACTIONS(1773),
    [sym_data_max_metadata_field_count] = ACTIONS(1773),
    [sym_data_max_id_length] = ACTIONS(1773),
    [sym_data_max_name_length] = ACTIONS(1773),
    [sym_data_max_description_length] = ACTIONS(1773),
    [sym_data_max_type_length] = ACTIONS(1773),
    [sym_data_max_format_length] = ACTIONS(1773),
    [sym_data_max_source_length] = ACTIONS(1773),
    [sym_data_max_value_length] = ACTIONS(1773),
    [sym_data_max_label_length] = ACTIONS(1773),
    [sym_data_max_url_length] = ACTIONS(1773),
    [sym_max_last_login_info_length] = ACTIONS(1773),
    [sym_data_type_unknown] = ACTIONS(1773),
    [sym_data_type_string] = ACTIONS(1773),
    [sym_data_type_datetime] = ACTIONS(1773),
    [sym_data_type_date] = ACTIONS(1773),
    [sym_data_type_time] = ACTIONS(1773),
    [sym_data_type_image] = ACTIONS(1773),
    [sym_data_format_url] = ACTIONS(1773),
    [sym_data_format_phone] = ACTIONS(1773),
    [sym_data_format_email] = ACTIONS(1773),
    [sym_data_format_iso8601] = ACTIONS(1773),
    [sym_auditlog_priv] = ACTIONS(1773),
    [sym_configuration_priv] = ACTIONS(1773),
    [sym_deviceconfig_priv] = ACTIONS(1773),
    [sym_ftp_priv] = ACTIONS(1773),
    [sym_http_priv] = ACTIONS(1773),
    [sym_networkconfig_priv] = ACTIONS(1773),
    [sym_programport_priv] = ACTIONS(1773),
    [sym_remoteui_priv] = ACTIONS(1773),
    [sym_securitycontrol_priv] = ACTIONS(1773),
    [sym_softwaremanagement_priv] = ACTIONS(1773),
    [sym_terminal_priv] = ACTIONS(1773),
    [sym_tpadmin_priv] = ACTIONS(1773),
    [sym_user1_priv] = ACTIONS(1773),
    [sym_user2_priv] = ACTIONS(1773),
    [sym_user3_priv] = ACTIONS(1773),
    [sym_user4_priv] = ACTIONS(1773),
    [sym_usermanagement_priv] = ACTIONS(1773),
    [sym_valid_account] = ACTIONS(1773),
    [sym_err_name_invalid] = ACTIONS(1773),
    [sym_err_password_invalid] = ACTIONS(1773),
    [sym_err_invalid_account] = ACTIONS(1773),
    [sym_err_no_authorization] = ACTIONS(1773),
    [sym_err_invalid_parameter] = ACTIONS(1773),
    [sym_err_user_locked_out] = ACTIONS(1773),
    [sym_err_user_account_expired] = ACTIONS(1773),
    [sym_err_general_error] = ACTIONS(1773),
    [sym_err_authentication_error] = ACTIONS(1773),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1773),
    [sym_tls_validate_certificate] = ACTIONS(1773),
    [sym_tdata] = ACTIONS(1773),
    [sym_tchannel] = ACTIONS(1773),
    [sym_tlevel] = ACTIONS(1773),
    [sym_tbutton] = ACTIONS(1773),
    [sym_ttimeline] = ACTIONS(1773),
    [sym_tcustom] = ACTIONS(1773),
    [sym_url_struct] = ACTIONS(1773),
    [sym_dns_struct] = ACTIONS(1773),
    [sym_ip_address_struct] = ACTIONS(1773),
    [sym_dev_info_struct] = ACTIONS(1773),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1773),
    [sym_data_feed] = ACTIONS(1773),
    [sym_data_field] = ACTIONS(1773),
    [sym_data_record] = ACTIONS(1773),
    [sym_wc_data_feed] = ACTIONS(1773),
    [sym_wc_data_field] = ACTIONS(1773),
    [sym_wc_data_record] = ACTIONS(1773),
    [sym_last_login_info] = ACTIONS(1773),
    [sym____reserved___] = ACTIONS(1773),
    [sym_date] = ACTIONS(1773),
    [sym_day] = ACTIONS(1773),
    [sym_time] = ACTIONS(1773),
    [sym_ldate] = ACTIONS(1773),
    [sym_push_device] = ACTIONS(1773),
    [sym_push_channel] = ACTIONS(1773),
    [sym_push_devchan] = ACTIONS(1773),
    [sym_release_device] = ACTIONS(1773),
    [sym_release_channel] = ACTIONS(1773),
    [sym_release_devchan] = ACTIONS(1773),
    [sym_master_slot] = ACTIONS(1773),
    [sym_get_pulse_time] = ACTIONS(1773),
    [sym_get_timer] = ACTIONS(1773),
    [sym_master_sn] = ACTIONS(1773),
    [sym_system_number] = ACTIONS(1773),
    [sym_button] = ACTIONS(1773),
    [sym_dv_channel] = ACTIONS(1773),
    [sym_data] = ACTIONS(1773),
    [sym_level] = ACTIONS(1773),
    [sym_timeline] = ACTIONS(1773),
    [sym_channel] = ACTIONS(1773),
    [sym_custom] = ACTIONS(1773),
    [sym_atoi] = ACTIONS(1773),
    [sym_atol] = ACTIONS(1773),
    [sym_atof] = ACTIONS(1773),
    [sym_itoa] = ACTIONS(1773),
    [sym_itohex] = ACTIONS(1773),
    [sym_hextoi] = ACTIONS(1773),
    [sym_ftoa] = ACTIONS(1773),
    [sym_find_string] = ACTIONS(1773),
    [sym_left_string] = ACTIONS(1773),
    [sym_right_string] = ACTIONS(1773),
    [sym_mid_string] = ACTIONS(1773),
    [sym_remove_string] = ACTIONS(1773),
    [sym_get_buffer_string] = ACTIONS(1773),
    [sym_get_buffer_char] = ACTIONS(1773),
    [sym_get_multi_buffer_string] = ACTIONS(1773),
    [sym_length_string] = ACTIONS(1773),
    [sym_max_length_string] = ACTIONS(1773),
    [sym_set_length_string] = ACTIONS(1773),
    [sym_compare_string] = ACTIONS(1773),
    [sym_upper_string] = ACTIONS(1773),
    [sym_lower_string] = ACTIONS(1773),
    [sym_get_last] = ACTIONS(1773),
    [sym_date_to_day] = ACTIONS(1773),
    [sym_date_to_month] = ACTIONS(1773),
    [sym_date_to_year] = ACTIONS(1773),
    [sym_time_to_hour] = ACTIONS(1773),
    [sym_time_to_minute] = ACTIONS(1773),
    [sym_time_to_second] = ACTIONS(1773),
    [sym_day_of_week] = ACTIONS(1773),
    [sym_set_pulse_time] = ACTIONS(1773),
    [sym_set_timer] = ACTIONS(1773),
    [sym_file_open] = ACTIONS(1773),
    [sym_file_close] = ACTIONS(1773),
    [sym_file_read] = ACTIONS(1773),
    [sym_file_read_line] = ACTIONS(1773),
    [sym_file_write] = ACTIONS(1773),
    [sym_file_write_line] = ACTIONS(1773),
    [sym_file_copy] = ACTIONS(1773),
    [sym_file_delete] = ACTIONS(1773),
    [sym_file_rename] = ACTIONS(1773),
    [sym_file_dir] = ACTIONS(1773),
    [sym_file_getdir] = ACTIONS(1773),
    [sym_file_setdir] = ACTIONS(1773),
    [sym_file_create_dir] = ACTIONS(1773),
    [sym_file_remove_dir] = ACTIONS(1773),
    [sym_file_seek] = ACTIONS(1773),
    [sym_length_array] = ACTIONS(1773),
    [sym_max_length_array] = ACTIONS(1773),
    [sym_set_length_array] = ACTIONS(1773),
    [sym_ip_client_open] = ACTIONS(1773),
    [sym_ip_client_close] = ACTIONS(1773),
    [sym_ip_server_open] = ACTIONS(1773),
    [sym_ip_server_close] = ACTIONS(1773),
    [sym_ip_bound_client_open] = ACTIONS(1773),
    [sym_ip_mc_server_open] = ACTIONS(1773),
    [sym_ip_set_option] = ACTIONS(1773),
    [sym_ssh_client_open] = ACTIONS(1773),
    [sym_ssh_client_close] = ACTIONS(1773),
    [sym_tls_client_open] = ACTIONS(1773),
    [sym_tls_client_close] = ACTIONS(1773),
    [sym_do_push] = ACTIONS(1773),
    [sym_do_push_timed] = ACTIONS(1773),
    [sym_do_release] = ACTIONS(1773),
    [sym_device_id] = ACTIONS(1773),
    [sym_device_info] = ACTIONS(1773),
    [sym_device_id_string] = ACTIONS(1773),
    [sym_device_standby] = ACTIONS(1773),
    [sym_device_wake] = ACTIONS(1773),
    [sym_timeline_create] = ACTIONS(1773),
    [sym_timeline_kill] = ACTIONS(1773),
    [sym_timeline_reload] = ACTIONS(1773),
    [sym_timeline_pause] = ACTIONS(1773),
    [sym_timeline_restart] = ACTIONS(1773),
    [sym_timeline_set] = ACTIONS(1773),
    [sym_timeline_get] = ACTIONS(1773),
    [sym_timeline_active] = ACTIONS(1773),
    [sym_timeline_dynamic_id] = ACTIONS(1773),
    [sym_variable_to_string] = ACTIONS(1773),
    [sym_string_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_string] = ACTIONS(1773),
    [sym_type_cast] = ACTIONS(1773),
    [sym_variable_to_xml] = ACTIONS(1773),
    [sym_xml_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_xml] = ACTIONS(1773),
    [sym_redirect_string] = ACTIONS(1773),
    [sym_random_number] = ACTIONS(1773),
    [sym_get_unique_id] = ACTIONS(1773),
    [sym_get_system_number] = ACTIONS(1773),
    [sym_set_system_number] = ACTIONS(1773),
    [sym_get_serial_number] = ACTIONS(1773),
    [sym_get_master_build] = ACTIONS(1773),
    [sym_set_validation_code] = ACTIONS(1773),
    [sym_reboot] = ACTIONS(1773),
    [sym_load_duet_module] = ACTIONS(1773),
    [sym_unload_duet_module] = ACTIONS(1773),
    [sym_duet_mem_size_set] = ACTIONS(1773),
    [sym_duet_mem_size_get] = ACTIONS(1773),
    [sym_dynamic_application_device] = ACTIONS(1773),
    [sym_dynamic_polled_port] = ACTIONS(1773),
    [sym_static_port_binding] = ACTIONS(1773),
    [sym_static_ip_binding] = ACTIONS(1773),
    [sym_get_dns_list] = ACTIONS(1773),
    [sym_set_dns_list] = ACTIONS(1773),
    [sym_get_ip_address] = ACTIONS(1773),
    [sym_set_ip_address] = ACTIONS(1773),
    [sym_get_url_list] = ACTIONS(1773),
    [sym_add_url_entry] = ACTIONS(1773),
    [sym_delete_url_entry] = ACTIONS(1773),
    [sym_abs_value] = ACTIONS(1773),
    [sym_max_value] = ACTIONS(1773),
    [sym_min_value] = ACTIONS(1773),
    [sym_exp_value] = ACTIONS(1773),
    [sym_log_value] = ACTIONS(1773),
    [sym_log10_value] = ACTIONS(1773),
    [sym_power_value] = ACTIONS(1773),
    [sym_sqrt_value] = ACTIONS(1773),
    [sym_raw_be] = ACTIONS(1773),
    [sym_raw_le] = ACTIONS(1773),
    [sym_internal_queue_size_set] = ACTIONS(1773),
    [sym_internal_queue_size_get] = ACTIONS(1773),
    [sym_internal_threshold_set] = ACTIONS(1773),
    [sym_internal_threshold_get] = ACTIONS(1773),
    [sym_rebuilt_event] = ACTIONS(1773),
    [sym_do_custom_event] = ACTIONS(1773),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1773),
    [sym_clkmgr_set_clk_source] = ACTIONS(1773),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1773),
    [sym_clkmgr_get_timezone] = ACTIONS(1773),
    [sym_clkmgr_set_timezone] = ACTIONS(1773),
    [sym_clkmgr_get_resync_period] = ACTIONS(1773),
    [sym_clkmgr_set_resync_period] = ACTIONS(1773),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_timeservers] = ACTIONS(1773),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_set_log_level] = ACTIONS(1773),
    [sym_get_log_level] = ACTIONS(1773),
    [sym_amx_log] = ACTIONS(1773),
    [sym_smtp_server_config_set] = ACTIONS(1773),
    [sym_smtp_server_config_get] = ACTIONS(1773),
    [sym_smtp_send] = ACTIONS(1773),
    [sym_validate_netlinx_account] = ACTIONS(1773),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1773),
    [sym_authenticate_certificate] = ACTIONS(1773),
    [sym_audit_netlinx_session_event] = ACTIONS(1773),
    [sym_audit_netlinx_generic_event] = ACTIONS(1773),
    [sym_get_available_flash_disk_space] = ACTIONS(1773),
    [sym_get_max_flash_disk_space] = ACTIONS(1773),
    [sym_data_create_feed] = ACTIONS(1773),
    [sym_data_delete_feed] = ACTIONS(1773),
    [sym_data_publish_feed] = ACTIONS(1773),
    [sym_data_get_published_feed] = ACTIONS(1773),
    [sym_data_add_record] = ACTIONS(1773),
    [sym_data_get_event_record] = ACTIONS(1773),
    [sym__wc_data_create_feed] = ACTIONS(1773),
    [sym__wc_data_add_record] = ACTIONS(1773),
    [sym__wc_data_get_event_record] = ACTIONS(1773),
    [sym_format] = ACTIONS(1773),
    [sym_astro_clock] = ACTIONS(1773),
    [sym_set_outdoor_temperature] = ACTIONS(1773),
    [sym_set_virtual_level_count] = ACTIONS(1773),
    [sym_set_virtual_channel_count] = ACTIONS(1773),
    [sym_set_virtual_port_count] = ACTIONS(1773),
    [sym__wc_encode_int] = ACTIONS(1773),
    [sym__wc_decode_int] = ACTIONS(1773),
    [sym__wc_int] = ACTIONS(1773),
    [sym__wc_to_ch_int] = ACTIONS(1773),
    [sym__ch_to_wc_int] = ACTIONS(1773),
    [sym__wc_find_string_int] = ACTIONS(1773),
    [sym__wc_left_string_int] = ACTIONS(1773),
    [sym__wc_lower_string_int] = ACTIONS(1773),
    [sym__wc_mid_string_int] = ACTIONS(1773),
    [sym__wc_remove_string_int] = ACTIONS(1773),
    [sym__wc_right_string_int] = ACTIONS(1773),
    [sym__wc_upper_string_int] = ACTIONS(1773),
    [sym__wc_compare_string_int] = ACTIONS(1773),
    [sym__wc_get_buffer_char_int] = ACTIONS(1773),
    [sym__wc_get_buffer_string_int] = ACTIONS(1773),
    [sym__wc_concat_string_int] = ACTIONS(1773),
    [sym__wc_file_open_int] = ACTIONS(1773),
    [sym__wc_file_close_int] = ACTIONS(1773),
    [sym__wc_file_read_int] = ACTIONS(1773),
    [sym__wc_file_read_line_int] = ACTIONS(1773),
    [sym__wc_file_write_int] = ACTIONS(1773),
    [sym__wc_file_write_line_int] = ACTIONS(1773),
    [aux_sym_preproc_include_token1] = ACTIONS(1773),
    [aux_sym_preproc_define_token1] = ACTIONS(1775),
    [aux_sym_preproc_warn_token1] = ACTIONS(1775),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1775),
    [anon_sym_LBRACK] = ACTIONS(1775),
    [anon_sym_LBRACE] = ACTIONS(1775),
    [anon_sym_RBRACE] = ACTIONS(1775),
    [anon_sym_LPAREN] = ACTIONS(1773),
    [anon_sym_BANG] = ACTIONS(1775),
    [anon_sym_TILDE] = ACTIONS(1775),
    [anon_sym_DASH] = ACTIONS(1773),
    [anon_sym_PLUS] = ACTIONS(1773),
    [anon_sym_DASH_DASH] = ACTIONS(1775),
    [anon_sym_PLUS_PLUS] = ACTIONS(1775),
    [anon_sym_DQUOTE] = ACTIONS(1775),
    [anon_sym_SQUOTE] = ACTIONS(1775),
    [sym_number_literal] = ACTIONS(1775),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(501)] = {
    [sym_identifier] = ACTIONS(1773),
    [sym_define_call_keyword] = ACTIONS(1773),
    [sym_define_function_keyword] = ACTIONS(1773),
    [sym_define_library_function_keyword] = ACTIONS(1773),
    [sym_define_module_keyword] = ACTIONS(1773),
    [sym_char_keyword] = ACTIONS(1773),
    [sym_widechar_keyword] = ACTIONS(1773),
    [sym_integer_keyword] = ACTIONS(1773),
    [sym_sinteger_keyword] = ACTIONS(1773),
    [sym_long_keyword] = ACTIONS(1773),
    [sym_slong_keyword] = ACTIONS(1773),
    [sym_float_keyword] = ACTIONS(1773),
    [sym_double_keyword] = ACTIONS(1773),
    [sym_variant_keyword] = ACTIONS(1773),
    [sym_variantarray_keyword] = ACTIONS(1773),
    [sym_local_var_keyword] = ACTIONS(1773),
    [sym_stack_var_keyword] = ACTIONS(1773),
    [sym_constant_keyword] = ACTIONS(1773),
    [sym_volatile_keyword] = ACTIONS(1773),
    [sym_non_volatile_keyword] = ACTIONS(1773),
    [sym_persistent_keyword] = ACTIONS(1773),
    [sym_dev_keyword] = ACTIONS(1773),
    [sym_devlev_keyword] = ACTIONS(1773),
    [sym_devchan_keyword] = ACTIONS(1773),
    [sym_if_keyword] = ACTIONS(1773),
    [sym_else_keyword] = ACTIONS(1773),
    [sym_switch_keyword] = ACTIONS(1773),
    [sym_case_keyword] = ACTIONS(1773),
    [sym_default_keyword] = ACTIONS(1773),
    [sym_while_keyword] = ACTIONS(1773),
    [sym_for_keyword] = ACTIONS(1773),
    [sym_break_keyword] = ACTIONS(1773),
    [sym_continue_keyword] = ACTIONS(1773),
    [sym_return_keyword] = ACTIONS(1773),
    [sym_select_keyword] = ACTIONS(1773),
    [sym_active_keyword] = ACTIONS(1773),
    [sym_struct_keyword] = ACTIONS(1773),
    [sym_structure_keyword] = ACTIONS(1773),
    [aux_sym_on_keyword_token1] = ACTIONS(1773),
    [aux_sym_off_keyword_token1] = ACTIONS(1773),
    [sym_send_level_keyword] = ACTIONS(1773),
    [sym_send_string_keyword] = ACTIONS(1773),
    [sym_send_command_keyword] = ACTIONS(1773),
    [sym_clear_buffer_keyword] = ACTIONS(1773),
    [sym_create_buffer_keyword] = ACTIONS(1773),
    [sym_create_multi_buffer_keyword] = ACTIONS(1773),
    [sym_call_keyword] = ACTIONS(1773),
    [sym_system_call_keyword] = ACTIONS(1773),
    [sym_devchan_to_keyword] = ACTIONS(1773),
    [sym_devchan_min_to_keyword] = ACTIONS(1773),
    [sym_devchan_total_off_keyword] = ACTIONS(1773),
    [sym_devchan_pulse_keyword] = ACTIONS(1773),
    [sym_wait_keyword] = ACTIONS(1773),
    [sym_cancel_wait_keyword] = ACTIONS(1773),
    [sym_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_wait_until_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_keyword] = ACTIONS(1773),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1773),
    [sym_not] = ACTIONS(1773),
    [sym_bnot] = ACTIONS(1773),
    [sym___file__] = ACTIONS(1773),
    [sym___date__] = ACTIONS(1773),
    [sym___ldate__] = ACTIONS(1773),
    [sym___time__] = ACTIONS(1773),
    [sym___line__] = ACTIONS(1773),
    [sym___name__] = ACTIONS(1773),
    [sym___netlinx__] = ACTIONS(1773),
    [sym_netlinx_axi_version] = ACTIONS(1773),
    [sym_url_flg_tcp] = ACTIONS(1773),
    [sym_url_flg_accinfopresent] = ACTIONS(1773),
    [sym_url_flg_temp] = ACTIONS(1773),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1773),
    [sym_url_flg_stat_mask] = ACTIONS(1773),
    [sym_url_flg_stat_lookup] = ACTIONS(1773),
    [sym_url_flg_stat_connecting] = ACTIONS(1773),
    [sym_url_flg_stat_waiting] = ACTIONS(1773),
    [sym_url_flg_stat_connected] = ACTIONS(1773),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1773),
    [sym_first_virtual_device] = ACTIONS(1773),
    [sym_first_local_port] = ACTIONS(1773),
    [sym_dynamic_virtual_device] = ACTIONS(1773),
    [sym_timeline_once] = ACTIONS(1773),
    [sym_timeline_repeat] = ACTIONS(1773),
    [sym_timeline_absolute] = ACTIONS(1773),
    [sym_timeline_relative] = ACTIONS(1773),
    [sym_do_push_timed_infinite] = ACTIONS(1773),
    [sym_source_type_no_address] = ACTIONS(1773),
    [sym_source_type_neuron_id] = ACTIONS(1773),
    [sym_source_type_ip_address] = ACTIONS(1773),
    [sym_source_type_axlink] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1773),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1773),
    [sym_source_type_ip_socket_address] = ACTIONS(1773),
    [sym_source_type_rs232] = ACTIONS(1773),
    [sym_source_type_internal] = ACTIONS(1773),
    [sym_source_type_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1773),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1773),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1773),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1773),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1773),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1773),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1773),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_file_read_only] = ACTIONS(1773),
    [sym_file_rw_new] = ACTIONS(1773),
    [sym_file_rw_append] = ACTIONS(1773),
    [sym_ip_tcp] = ACTIONS(1773),
    [sym_ip_udp] = ACTIONS(1773),
    [sym_ip_udp_2way] = ACTIONS(1773),
    [sym_xml_encode_types] = ACTIONS(1773),
    [sym_xml_encode_char_as_list] = ACTIONS(1773),
    [sym_xml_encode_le] = ACTIONS(1773),
    [sym_xml_decode_types] = ACTIONS(1773),
    [sym_xml_decode_no_preserve] = ACTIONS(1773),
    [sym_duet_dev_type_display_device] = ACTIONS(1773),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1773),
    [sym_duet_dev_type_io_device] = ACTIONS(1773),
    [sym_duet_dev_type_relay_device] = ACTIONS(1773),
    [sym_duet_dev_type_ups] = ACTIONS(1773),
    [sym_duet_dev_type_amplifier] = ACTIONS(1773),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1773),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1773),
    [sym_duet_dev_type_camera] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1773),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1773),
    [sym_duet_dev_type_dss] = ACTIONS(1773),
    [sym_duet_dev_type_dvr] = ACTIONS(1773),
    [sym_duet_dev_type_disc_device] = ACTIONS(1773),
    [sym_duet_dev_type_document_camera] = ACTIONS(1773),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1773),
    [sym_duet_dev_type_hvac] = ACTIONS(1773),
    [sym_duet_dev_type_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_light] = ACTIONS(1773),
    [sym_duet_dev_type_monitor] = ACTIONS(1773),
    [sym_duet_dev_type_motor] = ACTIONS(1773),
    [sym_duet_dev_type_multi_window] = ACTIONS(1773),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1773),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1773),
    [sym_duet_dev_type_receiver] = ACTIONS(1773),
    [sym_duet_dev_type_security_system] = ACTIONS(1773),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1773),
    [sym_duet_dev_type_settop_box] = ACTIONS(1773),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1773),
    [sym_duet_dev_type_switcher] = ACTIONS(1773),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1773),
    [sym_duet_dev_type_tv] = ACTIONS(1773),
    [sym_duet_dev_type_utility] = ACTIONS(1773),
    [sym_duet_dev_type_vcr] = ACTIONS(1773),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1773),
    [sym_duet_dev_type_video_processor] = ACTIONS(1773),
    [sym_duet_dev_type_video_projector] = ACTIONS(1773),
    [sym_duet_dev_type_video_wall] = ACTIONS(1773),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1773),
    [sym_duet_dev_type_weather] = ACTIONS(1773),
    [sym_duet_dev_not_polled] = ACTIONS(1773),
    [sym_duet_dev_polled] = ACTIONS(1773),
    [sym_clkmgr_mode_network] = ACTIONS(1773),
    [sym_clkmgr_mode_standalone] = ACTIONS(1773),
    [sym_normal_standby] = ACTIONS(1773),
    [sym_normal_wake] = ACTIONS(1773),
    [sym_ip_multicast_ttl_option] = ACTIONS(1773),
    [sym_ip_ttl_subnet] = ACTIONS(1773),
    [sym_ip_ttl_site] = ACTIONS(1773),
    [sym_ip_ttl_region] = ACTIONS(1773),
    [sym_ip_ttl_continent] = ACTIONS(1773),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1773),
    [sym_ip_nodelay_on] = ACTIONS(1773),
    [sym_ip_nodelay_off] = ACTIONS(1773),
    [sym_amx_error] = ACTIONS(1773),
    [sym_amx_warning] = ACTIONS(1773),
    [sym_amx_info] = ACTIONS(1773),
    [sym_amx_debug] = ACTIONS(1773),
    [sym_netlinx_login_success] = ACTIONS(1773),
    [sym_netlinx_login_fail] = ACTIONS(1773),
    [sym_netlinx_logout] = ACTIONS(1773),
    [sym_null_str] = ACTIONS(1773),
    [sym_smtp_address] = ACTIONS(1773),
    [sym_smtp_port_number] = ACTIONS(1773),
    [sym_smtp_username] = ACTIONS(1773),
    [sym_smtp_password] = ACTIONS(1773),
    [sym_smtp_require_tls] = ACTIONS(1773),
    [sym_smtp_from] = ACTIONS(1773),
    [sym_smtp_tls_true] = ACTIONS(1773),
    [sym_smtp_tls_false] = ACTIONS(1773),
    [sym_listview_on_row_select_event] = ACTIONS(1773),
    [sym_data_structure_unknown] = ACTIONS(1773),
    [sym_data_structure_datafeed] = ACTIONS(1773),
    [sym_data_structure_datarecordset] = ACTIONS(1773),
    [sym_data_structure_datarecord] = ACTIONS(1773),
    [sym_data_structure_datafield] = ACTIONS(1773),
    [sym_data_max_content_field_count] = ACTIONS(1773),
    [sym_data_max_metadata_field_count] = ACTIONS(1773),
    [sym_data_max_id_length] = ACTIONS(1773),
    [sym_data_max_name_length] = ACTIONS(1773),
    [sym_data_max_description_length] = ACTIONS(1773),
    [sym_data_max_type_length] = ACTIONS(1773),
    [sym_data_max_format_length] = ACTIONS(1773),
    [sym_data_max_source_length] = ACTIONS(1773),
    [sym_data_max_value_length] = ACTIONS(1773),
    [sym_data_max_label_length] = ACTIONS(1773),
    [sym_data_max_url_length] = ACTIONS(1773),
    [sym_max_last_login_info_length] = ACTIONS(1773),
    [sym_data_type_unknown] = ACTIONS(1773),
    [sym_data_type_string] = ACTIONS(1773),
    [sym_data_type_datetime] = ACTIONS(1773),
    [sym_data_type_date] = ACTIONS(1773),
    [sym_data_type_time] = ACTIONS(1773),
    [sym_data_type_image] = ACTIONS(1773),
    [sym_data_format_url] = ACTIONS(1773),
    [sym_data_format_phone] = ACTIONS(1773),
    [sym_data_format_email] = ACTIONS(1773),
    [sym_data_format_iso8601] = ACTIONS(1773),
    [sym_auditlog_priv] = ACTIONS(1773),
    [sym_configuration_priv] = ACTIONS(1773),
    [sym_deviceconfig_priv] = ACTIONS(1773),
    [sym_ftp_priv] = ACTIONS(1773),
    [sym_http_priv] = ACTIONS(1773),
    [sym_networkconfig_priv] = ACTIONS(1773),
    [sym_programport_priv] = ACTIONS(1773),
    [sym_remoteui_priv] = ACTIONS(1773),
    [sym_securitycontrol_priv] = ACTIONS(1773),
    [sym_softwaremanagement_priv] = ACTIONS(1773),
    [sym_terminal_priv] = ACTIONS(1773),
    [sym_tpadmin_priv] = ACTIONS(1773),
    [sym_user1_priv] = ACTIONS(1773),
    [sym_user2_priv] = ACTIONS(1773),
    [sym_user3_priv] = ACTIONS(1773),
    [sym_user4_priv] = ACTIONS(1773),
    [sym_usermanagement_priv] = ACTIONS(1773),
    [sym_valid_account] = ACTIONS(1773),
    [sym_err_name_invalid] = ACTIONS(1773),
    [sym_err_password_invalid] = ACTIONS(1773),
    [sym_err_invalid_account] = ACTIONS(1773),
    [sym_err_no_authorization] = ACTIONS(1773),
    [sym_err_invalid_parameter] = ACTIONS(1773),
    [sym_err_user_locked_out] = ACTIONS(1773),
    [sym_err_user_account_expired] = ACTIONS(1773),
    [sym_err_general_error] = ACTIONS(1773),
    [sym_err_authentication_error] = ACTIONS(1773),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1773),
    [sym_tls_validate_certificate] = ACTIONS(1773),
    [sym_tdata] = ACTIONS(1773),
    [sym_tchannel] = ACTIONS(1773),
    [sym_tlevel] = ACTIONS(1773),
    [sym_tbutton] = ACTIONS(1773),
    [sym_ttimeline] = ACTIONS(1773),
    [sym_tcustom] = ACTIONS(1773),
    [sym_url_struct] = ACTIONS(1773),
    [sym_dns_struct] = ACTIONS(1773),
    [sym_ip_address_struct] = ACTIONS(1773),
    [sym_dev_info_struct] = ACTIONS(1773),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1773),
    [sym_data_feed] = ACTIONS(1773),
    [sym_data_field] = ACTIONS(1773),
    [sym_data_record] = ACTIONS(1773),
    [sym_wc_data_feed] = ACTIONS(1773),
    [sym_wc_data_field] = ACTIONS(1773),
    [sym_wc_data_record] = ACTIONS(1773),
    [sym_last_login_info] = ACTIONS(1773),
    [sym____reserved___] = ACTIONS(1773),
    [sym_date] = ACTIONS(1773),
    [sym_day] = ACTIONS(1773),
    [sym_time] = ACTIONS(1773),
    [sym_ldate] = ACTIONS(1773),
    [sym_push_device] = ACTIONS(1773),
    [sym_push_channel] = ACTIONS(1773),
    [sym_push_devchan] = ACTIONS(1773),
    [sym_release_device] = ACTIONS(1773),
    [sym_release_channel] = ACTIONS(1773),
    [sym_release_devchan] = ACTIONS(1773),
    [sym_master_slot] = ACTIONS(1773),
    [sym_get_pulse_time] = ACTIONS(1773),
    [sym_get_timer] = ACTIONS(1773),
    [sym_master_sn] = ACTIONS(1773),
    [sym_system_number] = ACTIONS(1773),
    [sym_button] = ACTIONS(1773),
    [sym_dv_channel] = ACTIONS(1773),
    [sym_data] = ACTIONS(1773),
    [sym_level] = ACTIONS(1773),
    [sym_timeline] = ACTIONS(1773),
    [sym_channel] = ACTIONS(1773),
    [sym_custom] = ACTIONS(1773),
    [sym_atoi] = ACTIONS(1773),
    [sym_atol] = ACTIONS(1773),
    [sym_atof] = ACTIONS(1773),
    [sym_itoa] = ACTIONS(1773),
    [sym_itohex] = ACTIONS(1773),
    [sym_hextoi] = ACTIONS(1773),
    [sym_ftoa] = ACTIONS(1773),
    [sym_find_string] = ACTIONS(1773),
    [sym_left_string] = ACTIONS(1773),
    [sym_right_string] = ACTIONS(1773),
    [sym_mid_string] = ACTIONS(1773),
    [sym_remove_string] = ACTIONS(1773),
    [sym_get_buffer_string] = ACTIONS(1773),
    [sym_get_buffer_char] = ACTIONS(1773),
    [sym_get_multi_buffer_string] = ACTIONS(1773),
    [sym_length_string] = ACTIONS(1773),
    [sym_max_length_string] = ACTIONS(1773),
    [sym_set_length_string] = ACTIONS(1773),
    [sym_compare_string] = ACTIONS(1773),
    [sym_upper_string] = ACTIONS(1773),
    [sym_lower_string] = ACTIONS(1773),
    [sym_get_last] = ACTIONS(1773),
    [sym_date_to_day] = ACTIONS(1773),
    [sym_date_to_month] = ACTIONS(1773),
    [sym_date_to_year] = ACTIONS(1773),
    [sym_time_to_hour] = ACTIONS(1773),
    [sym_time_to_minute] = ACTIONS(1773),
    [sym_time_to_second] = ACTIONS(1773),
    [sym_day_of_week] = ACTIONS(1773),
    [sym_set_pulse_time] = ACTIONS(1773),
    [sym_set_timer] = ACTIONS(1773),
    [sym_file_open] = ACTIONS(1773),
    [sym_file_close] = ACTIONS(1773),
    [sym_file_read] = ACTIONS(1773),
    [sym_file_read_line] = ACTIONS(1773),
    [sym_file_write] = ACTIONS(1773),
    [sym_file_write_line] = ACTIONS(1773),
    [sym_file_copy] = ACTIONS(1773),
    [sym_file_delete] = ACTIONS(1773),
    [sym_file_rename] = ACTIONS(1773),
    [sym_file_dir] = ACTIONS(1773),
    [sym_file_getdir] = ACTIONS(1773),
    [sym_file_setdir] = ACTIONS(1773),
    [sym_file_create_dir] = ACTIONS(1773),
    [sym_file_remove_dir] = ACTIONS(1773),
    [sym_file_seek] = ACTIONS(1773),
    [sym_length_array] = ACTIONS(1773),
    [sym_max_length_array] = ACTIONS(1773),
    [sym_set_length_array] = ACTIONS(1773),
    [sym_ip_client_open] = ACTIONS(1773),
    [sym_ip_client_close] = ACTIONS(1773),
    [sym_ip_server_open] = ACTIONS(1773),
    [sym_ip_server_close] = ACTIONS(1773),
    [sym_ip_bound_client_open] = ACTIONS(1773),
    [sym_ip_mc_server_open] = ACTIONS(1773),
    [sym_ip_set_option] = ACTIONS(1773),
    [sym_ssh_client_open] = ACTIONS(1773),
    [sym_ssh_client_close] = ACTIONS(1773),
    [sym_tls_client_open] = ACTIONS(1773),
    [sym_tls_client_close] = ACTIONS(1773),
    [sym_do_push] = ACTIONS(1773),
    [sym_do_push_timed] = ACTIONS(1773),
    [sym_do_release] = ACTIONS(1773),
    [sym_device_id] = ACTIONS(1773),
    [sym_device_info] = ACTIONS(1773),
    [sym_device_id_string] = ACTIONS(1773),
    [sym_device_standby] = ACTIONS(1773),
    [sym_device_wake] = ACTIONS(1773),
    [sym_timeline_create] = ACTIONS(1773),
    [sym_timeline_kill] = ACTIONS(1773),
    [sym_timeline_reload] = ACTIONS(1773),
    [sym_timeline_pause] = ACTIONS(1773),
    [sym_timeline_restart] = ACTIONS(1773),
    [sym_timeline_set] = ACTIONS(1773),
    [sym_timeline_get] = ACTIONS(1773),
    [sym_timeline_active] = ACTIONS(1773),
    [sym_timeline_dynamic_id] = ACTIONS(1773),
    [sym_variable_to_string] = ACTIONS(1773),
    [sym_string_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_string] = ACTIONS(1773),
    [sym_type_cast] = ACTIONS(1773),
    [sym_variable_to_xml] = ACTIONS(1773),
    [sym_xml_to_variable] = ACTIONS(1773),
    [sym_length_variable_to_xml] = ACTIONS(1773),
    [sym_redirect_string] = ACTIONS(1773),
    [sym_random_number] = ACTIONS(1773),
    [sym_get_unique_id] = ACTIONS(1773),
    [sym_get_system_number] = ACTIONS(1773),
    [sym_set_system_number] = ACTIONS(1773),
    [sym_get_serial_number] = ACTIONS(1773),
    [sym_get_master_build] = ACTIONS(1773),
    [sym_set_validation_code] = ACTIONS(1773),
    [sym_reboot] = ACTIONS(1773),
    [sym_load_duet_module] = ACTIONS(1773),
    [sym_unload_duet_module] = ACTIONS(1773),
    [sym_duet_mem_size_set] = ACTIONS(1773),
    [sym_duet_mem_size_get] = ACTIONS(1773),
    [sym_dynamic_application_device] = ACTIONS(1773),
    [sym_dynamic_polled_port] = ACTIONS(1773),
    [sym_static_port_binding] = ACTIONS(1773),
    [sym_static_ip_binding] = ACTIONS(1773),
    [sym_get_dns_list] = ACTIONS(1773),
    [sym_set_dns_list] = ACTIONS(1773),
    [sym_get_ip_address] = ACTIONS(1773),
    [sym_set_ip_address] = ACTIONS(1773),
    [sym_get_url_list] = ACTIONS(1773),
    [sym_add_url_entry] = ACTIONS(1773),
    [sym_delete_url_entry] = ACTIONS(1773),
    [sym_abs_value] = ACTIONS(1773),
    [sym_max_value] = ACTIONS(1773),
    [sym_min_value] = ACTIONS(1773),
    [sym_exp_value] = ACTIONS(1773),
    [sym_log_value] = ACTIONS(1773),
    [sym_log10_value] = ACTIONS(1773),
    [sym_power_value] = ACTIONS(1773),
    [sym_sqrt_value] = ACTIONS(1773),
    [sym_raw_be] = ACTIONS(1773),
    [sym_raw_le] = ACTIONS(1773),
    [sym_internal_queue_size_set] = ACTIONS(1773),
    [sym_internal_queue_size_get] = ACTIONS(1773),
    [sym_internal_threshold_set] = ACTIONS(1773),
    [sym_internal_threshold_get] = ACTIONS(1773),
    [sym_rebuilt_event] = ACTIONS(1773),
    [sym_do_custom_event] = ACTIONS(1773),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1773),
    [sym_clkmgr_set_clk_source] = ACTIONS(1773),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1773),
    [sym_clkmgr_get_timezone] = ACTIONS(1773),
    [sym_clkmgr_set_timezone] = ACTIONS(1773),
    [sym_clkmgr_get_resync_period] = ACTIONS(1773),
    [sym_clkmgr_set_resync_period] = ACTIONS(1773),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1773),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_timeservers] = ACTIONS(1773),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1773),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1773),
    [sym_set_log_level] = ACTIONS(1773),
    [sym_get_log_level] = ACTIONS(1773),
    [sym_amx_log] = ACTIONS(1773),
    [sym_smtp_server_config_set] = ACTIONS(1773),
    [sym_smtp_server_config_get] = ACTIONS(1773),
    [sym_smtp_send] = ACTIONS(1773),
    [sym_validate_netlinx_account] = ACTIONS(1773),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1773),
    [sym_authenticate_certificate] = ACTIONS(1773),
    [sym_audit_netlinx_session_event] = ACTIONS(1773),
    [sym_audit_netlinx_generic_event] = ACTIONS(1773),
    [sym_get_available_flash_disk_space] = ACTIONS(1773),
    [sym_get_max_flash_disk_space] = ACTIONS(1773),
    [sym_data_create_feed] = ACTIONS(1773),
    [sym_data_delete_feed] = ACTIONS(1773),
    [sym_data_publish_feed] = ACTIONS(1773),
    [sym_data_get_published_feed] = ACTIONS(1773),
    [sym_data_add_record] = ACTIONS(1773),
    [sym_data_get_event_record] = ACTIONS(1773),
    [sym__wc_data_create_feed] = ACTIONS(1773),
    [sym__wc_data_add_record] = ACTIONS(1773),
    [sym__wc_data_get_event_record] = ACTIONS(1773),
    [sym_format] = ACTIONS(1773),
    [sym_astro_clock] = ACTIONS(1773),
    [sym_set_outdoor_temperature] = ACTIONS(1773),
    [sym_set_virtual_level_count] = ACTIONS(1773),
    [sym_set_virtual_channel_count] = ACTIONS(1773),
    [sym_set_virtual_port_count] = ACTIONS(1773),
    [sym__wc_encode_int] = ACTIONS(1773),
    [sym__wc_decode_int] = ACTIONS(1773),
    [sym__wc_int] = ACTIONS(1773),
    [sym__wc_to_ch_int] = ACTIONS(1773),
    [sym__ch_to_wc_int] = ACTIONS(1773),
    [sym__wc_find_string_int] = ACTIONS(1773),
    [sym__wc_left_string_int] = ACTIONS(1773),
    [sym__wc_lower_string_int] = ACTIONS(1773),
    [sym__wc_mid_string_int] = ACTIONS(1773),
    [sym__wc_remove_string_int] = ACTIONS(1773),
    [sym__wc_right_string_int] = ACTIONS(1773),
    [sym__wc_upper_string_int] = ACTIONS(1773),
    [sym__wc_compare_string_int] = ACTIONS(1773),
    [sym__wc_get_buffer_char_int] = ACTIONS(1773),
    [sym__wc_get_buffer_string_int] = ACTIONS(1773),
    [sym__wc_concat_string_int] = ACTIONS(1773),
    [sym__wc_file_open_int] = ACTIONS(1773),
    [sym__wc_file_close_int] = ACTIONS(1773),
    [sym__wc_file_read_int] = ACTIONS(1773),
    [sym__wc_file_read_line_int] = ACTIONS(1773),
    [sym__wc_file_write_int] = ACTIONS(1773),
    [sym__wc_file_write_line_int] = ACTIONS(1773),
    [aux_sym_preproc_include_token1] = ACTIONS(1773),
    [aux_sym_preproc_define_token1] = ACTIONS(1775),
    [aux_sym_preproc_warn_token1] = ACTIONS(1775),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1775),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1775),
    [anon_sym_LBRACK] = ACTIONS(1775),
    [anon_sym_LBRACE] = ACTIONS(1775),
    [anon_sym_RBRACE] = ACTIONS(1775),
    [anon_sym_LPAREN] = ACTIONS(1773),
    [anon_sym_BANG] = ACTIONS(1775),
    [anon_sym_TILDE] = ACTIONS(1775),
    [anon_sym_DASH] = ACTIONS(1773),
    [anon_sym_PLUS] = ACTIONS(1773),
    [anon_sym_DASH_DASH] = ACTIONS(1775),
    [anon_sym_PLUS_PLUS] = ACTIONS(1775),
    [anon_sym_DQUOTE] = ACTIONS(1775),
    [anon_sym_SQUOTE] = ACTIONS(1775),
    [sym_number_literal] = ACTIONS(1775),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(502)] = {
    [sym_identifier] = ACTIONS(1777),
    [sym_define_call_keyword] = ACTIONS(1777),
    [sym_define_function_keyword] = ACTIONS(1777),
    [sym_define_library_function_keyword] = ACTIONS(1777),
    [sym_define_module_keyword] = ACTIONS(1777),
    [sym_char_keyword] = ACTIONS(1777),
    [sym_widechar_keyword] = ACTIONS(1777),
    [sym_integer_keyword] = ACTIONS(1777),
    [sym_sinteger_keyword] = ACTIONS(1777),
    [sym_long_keyword] = ACTIONS(1777),
    [sym_slong_keyword] = ACTIONS(1777),
    [sym_float_keyword] = ACTIONS(1777),
    [sym_double_keyword] = ACTIONS(1777),
    [sym_variant_keyword] = ACTIONS(1777),
    [sym_variantarray_keyword] = ACTIONS(1777),
    [sym_local_var_keyword] = ACTIONS(1777),
    [sym_stack_var_keyword] = ACTIONS(1777),
    [sym_constant_keyword] = ACTIONS(1777),
    [sym_volatile_keyword] = ACTIONS(1777),
    [sym_non_volatile_keyword] = ACTIONS(1777),
    [sym_persistent_keyword] = ACTIONS(1777),
    [sym_dev_keyword] = ACTIONS(1777),
    [sym_devlev_keyword] = ACTIONS(1777),
    [sym_devchan_keyword] = ACTIONS(1777),
    [sym_if_keyword] = ACTIONS(1777),
    [sym_else_keyword] = ACTIONS(1777),
    [sym_switch_keyword] = ACTIONS(1777),
    [sym_case_keyword] = ACTIONS(1777),
    [sym_default_keyword] = ACTIONS(1777),
    [sym_while_keyword] = ACTIONS(1777),
    [sym_for_keyword] = ACTIONS(1777),
    [sym_break_keyword] = ACTIONS(1777),
    [sym_continue_keyword] = ACTIONS(1777),
    [sym_return_keyword] = ACTIONS(1777),
    [sym_select_keyword] = ACTIONS(1777),
    [sym_active_keyword] = ACTIONS(1777),
    [sym_struct_keyword] = ACTIONS(1777),
    [sym_structure_keyword] = ACTIONS(1777),
    [aux_sym_on_keyword_token1] = ACTIONS(1777),
    [aux_sym_off_keyword_token1] = ACTIONS(1777),
    [sym_send_level_keyword] = ACTIONS(1777),
    [sym_send_string_keyword] = ACTIONS(1777),
    [sym_send_command_keyword] = ACTIONS(1777),
    [sym_clear_buffer_keyword] = ACTIONS(1777),
    [sym_create_buffer_keyword] = ACTIONS(1777),
    [sym_create_multi_buffer_keyword] = ACTIONS(1777),
    [sym_call_keyword] = ACTIONS(1777),
    [sym_system_call_keyword] = ACTIONS(1777),
    [sym_devchan_to_keyword] = ACTIONS(1777),
    [sym_devchan_min_to_keyword] = ACTIONS(1777),
    [sym_devchan_total_off_keyword] = ACTIONS(1777),
    [sym_devchan_pulse_keyword] = ACTIONS(1777),
    [sym_wait_keyword] = ACTIONS(1777),
    [sym_cancel_wait_keyword] = ACTIONS(1777),
    [sym_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1777),
    [sym_not] = ACTIONS(1777),
    [sym_bnot] = ACTIONS(1777),
    [sym___file__] = ACTIONS(1777),
    [sym___date__] = ACTIONS(1777),
    [sym___ldate__] = ACTIONS(1777),
    [sym___time__] = ACTIONS(1777),
    [sym___line__] = ACTIONS(1777),
    [sym___name__] = ACTIONS(1777),
    [sym___netlinx__] = ACTIONS(1777),
    [sym_netlinx_axi_version] = ACTIONS(1777),
    [sym_url_flg_tcp] = ACTIONS(1777),
    [sym_url_flg_accinfopresent] = ACTIONS(1777),
    [sym_url_flg_temp] = ACTIONS(1777),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1777),
    [sym_url_flg_stat_mask] = ACTIONS(1777),
    [sym_url_flg_stat_lookup] = ACTIONS(1777),
    [sym_url_flg_stat_connecting] = ACTIONS(1777),
    [sym_url_flg_stat_waiting] = ACTIONS(1777),
    [sym_url_flg_stat_connected] = ACTIONS(1777),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1777),
    [sym_first_virtual_device] = ACTIONS(1777),
    [sym_first_local_port] = ACTIONS(1777),
    [sym_dynamic_virtual_device] = ACTIONS(1777),
    [sym_timeline_once] = ACTIONS(1777),
    [sym_timeline_repeat] = ACTIONS(1777),
    [sym_timeline_absolute] = ACTIONS(1777),
    [sym_timeline_relative] = ACTIONS(1777),
    [sym_do_push_timed_infinite] = ACTIONS(1777),
    [sym_source_type_no_address] = ACTIONS(1777),
    [sym_source_type_neuron_id] = ACTIONS(1777),
    [sym_source_type_ip_address] = ACTIONS(1777),
    [sym_source_type_axlink] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1777),
    [sym_source_type_ip_socket_address] = ACTIONS(1777),
    [sym_source_type_rs232] = ACTIONS(1777),
    [sym_source_type_internal] = ACTIONS(1777),
    [sym_source_type_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1777),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1777),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1777),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1777),
    [sym_true] = ACTIONS(1777),
    [sym_false] = ACTIONS(1777),
    [sym_file_read_only] = ACTIONS(1777),
    [sym_file_rw_new] = ACTIONS(1777),
    [sym_file_rw_append] = ACTIONS(1777),
    [sym_ip_tcp] = ACTIONS(1777),
    [sym_ip_udp] = ACTIONS(1777),
    [sym_ip_udp_2way] = ACTIONS(1777),
    [sym_xml_encode_types] = ACTIONS(1777),
    [sym_xml_encode_char_as_list] = ACTIONS(1777),
    [sym_xml_encode_le] = ACTIONS(1777),
    [sym_xml_decode_types] = ACTIONS(1777),
    [sym_xml_decode_no_preserve] = ACTIONS(1777),
    [sym_duet_dev_type_display_device] = ACTIONS(1777),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_io_device] = ACTIONS(1777),
    [sym_duet_dev_type_relay_device] = ACTIONS(1777),
    [sym_duet_dev_type_ups] = ACTIONS(1777),
    [sym_duet_dev_type_amplifier] = ACTIONS(1777),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1777),
    [sym_duet_dev_type_camera] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1777),
    [sym_duet_dev_type_dss] = ACTIONS(1777),
    [sym_duet_dev_type_dvr] = ACTIONS(1777),
    [sym_duet_dev_type_disc_device] = ACTIONS(1777),
    [sym_duet_dev_type_document_camera] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1777),
    [sym_duet_dev_type_hvac] = ACTIONS(1777),
    [sym_duet_dev_type_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_light] = ACTIONS(1777),
    [sym_duet_dev_type_monitor] = ACTIONS(1777),
    [sym_duet_dev_type_motor] = ACTIONS(1777),
    [sym_duet_dev_type_multi_window] = ACTIONS(1777),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1777),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1777),
    [sym_duet_dev_type_receiver] = ACTIONS(1777),
    [sym_duet_dev_type_security_system] = ACTIONS(1777),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1777),
    [sym_duet_dev_type_settop_box] = ACTIONS(1777),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1777),
    [sym_duet_dev_type_switcher] = ACTIONS(1777),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_tv] = ACTIONS(1777),
    [sym_duet_dev_type_utility] = ACTIONS(1777),
    [sym_duet_dev_type_vcr] = ACTIONS(1777),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_video_processor] = ACTIONS(1777),
    [sym_duet_dev_type_video_projector] = ACTIONS(1777),
    [sym_duet_dev_type_video_wall] = ACTIONS(1777),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1777),
    [sym_duet_dev_type_weather] = ACTIONS(1777),
    [sym_duet_dev_not_polled] = ACTIONS(1777),
    [sym_duet_dev_polled] = ACTIONS(1777),
    [sym_clkmgr_mode_network] = ACTIONS(1777),
    [sym_clkmgr_mode_standalone] = ACTIONS(1777),
    [sym_normal_standby] = ACTIONS(1777),
    [sym_normal_wake] = ACTIONS(1777),
    [sym_ip_multicast_ttl_option] = ACTIONS(1777),
    [sym_ip_ttl_subnet] = ACTIONS(1777),
    [sym_ip_ttl_site] = ACTIONS(1777),
    [sym_ip_ttl_region] = ACTIONS(1777),
    [sym_ip_ttl_continent] = ACTIONS(1777),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1777),
    [sym_ip_nodelay_on] = ACTIONS(1777),
    [sym_ip_nodelay_off] = ACTIONS(1777),
    [sym_amx_error] = ACTIONS(1777),
    [sym_amx_warning] = ACTIONS(1777),
    [sym_amx_info] = ACTIONS(1777),
    [sym_amx_debug] = ACTIONS(1777),
    [sym_netlinx_login_success] = ACTIONS(1777),
    [sym_netlinx_login_fail] = ACTIONS(1777),
    [sym_netlinx_logout] = ACTIONS(1777),
    [sym_null_str] = ACTIONS(1777),
    [sym_smtp_address] = ACTIONS(1777),
    [sym_smtp_port_number] = ACTIONS(1777),
    [sym_smtp_username] = ACTIONS(1777),
    [sym_smtp_password] = ACTIONS(1777),
    [sym_smtp_require_tls] = ACTIONS(1777),
    [sym_smtp_from] = ACTIONS(1777),
    [sym_smtp_tls_true] = ACTIONS(1777),
    [sym_smtp_tls_false] = ACTIONS(1777),
    [sym_listview_on_row_select_event] = ACTIONS(1777),
    [sym_data_structure_unknown] = ACTIONS(1777),
    [sym_data_structure_datafeed] = ACTIONS(1777),
    [sym_data_structure_datarecordset] = ACTIONS(1777),
    [sym_data_structure_datarecord] = ACTIONS(1777),
    [sym_data_structure_datafield] = ACTIONS(1777),
    [sym_data_max_content_field_count] = ACTIONS(1777),
    [sym_data_max_metadata_field_count] = ACTIONS(1777),
    [sym_data_max_id_length] = ACTIONS(1777),
    [sym_data_max_name_length] = ACTIONS(1777),
    [sym_data_max_description_length] = ACTIONS(1777),
    [sym_data_max_type_length] = ACTIONS(1777),
    [sym_data_max_format_length] = ACTIONS(1777),
    [sym_data_max_source_length] = ACTIONS(1777),
    [sym_data_max_value_length] = ACTIONS(1777),
    [sym_data_max_label_length] = ACTIONS(1777),
    [sym_data_max_url_length] = ACTIONS(1777),
    [sym_max_last_login_info_length] = ACTIONS(1777),
    [sym_data_type_unknown] = ACTIONS(1777),
    [sym_data_type_string] = ACTIONS(1777),
    [sym_data_type_datetime] = ACTIONS(1777),
    [sym_data_type_date] = ACTIONS(1777),
    [sym_data_type_time] = ACTIONS(1777),
    [sym_data_type_image] = ACTIONS(1777),
    [sym_data_format_url] = ACTIONS(1777),
    [sym_data_format_phone] = ACTIONS(1777),
    [sym_data_format_email] = ACTIONS(1777),
    [sym_data_format_iso8601] = ACTIONS(1777),
    [sym_auditlog_priv] = ACTIONS(1777),
    [sym_configuration_priv] = ACTIONS(1777),
    [sym_deviceconfig_priv] = ACTIONS(1777),
    [sym_ftp_priv] = ACTIONS(1777),
    [sym_http_priv] = ACTIONS(1777),
    [sym_networkconfig_priv] = ACTIONS(1777),
    [sym_programport_priv] = ACTIONS(1777),
    [sym_remoteui_priv] = ACTIONS(1777),
    [sym_securitycontrol_priv] = ACTIONS(1777),
    [sym_softwaremanagement_priv] = ACTIONS(1777),
    [sym_terminal_priv] = ACTIONS(1777),
    [sym_tpadmin_priv] = ACTIONS(1777),
    [sym_user1_priv] = ACTIONS(1777),
    [sym_user2_priv] = ACTIONS(1777),
    [sym_user3_priv] = ACTIONS(1777),
    [sym_user4_priv] = ACTIONS(1777),
    [sym_usermanagement_priv] = ACTIONS(1777),
    [sym_valid_account] = ACTIONS(1777),
    [sym_err_name_invalid] = ACTIONS(1777),
    [sym_err_password_invalid] = ACTIONS(1777),
    [sym_err_invalid_account] = ACTIONS(1777),
    [sym_err_no_authorization] = ACTIONS(1777),
    [sym_err_invalid_parameter] = ACTIONS(1777),
    [sym_err_user_locked_out] = ACTIONS(1777),
    [sym_err_user_account_expired] = ACTIONS(1777),
    [sym_err_general_error] = ACTIONS(1777),
    [sym_err_authentication_error] = ACTIONS(1777),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1777),
    [sym_tls_validate_certificate] = ACTIONS(1777),
    [sym_tdata] = ACTIONS(1777),
    [sym_tchannel] = ACTIONS(1777),
    [sym_tlevel] = ACTIONS(1777),
    [sym_tbutton] = ACTIONS(1777),
    [sym_ttimeline] = ACTIONS(1777),
    [sym_tcustom] = ACTIONS(1777),
    [sym_url_struct] = ACTIONS(1777),
    [sym_dns_struct] = ACTIONS(1777),
    [sym_ip_address_struct] = ACTIONS(1777),
    [sym_dev_info_struct] = ACTIONS(1777),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1777),
    [sym_data_feed] = ACTIONS(1777),
    [sym_data_field] = ACTIONS(1777),
    [sym_data_record] = ACTIONS(1777),
    [sym_wc_data_feed] = ACTIONS(1777),
    [sym_wc_data_field] = ACTIONS(1777),
    [sym_wc_data_record] = ACTIONS(1777),
    [sym_last_login_info] = ACTIONS(1777),
    [sym____reserved___] = ACTIONS(1777),
    [sym_date] = ACTIONS(1777),
    [sym_day] = ACTIONS(1777),
    [sym_time] = ACTIONS(1777),
    [sym_ldate] = ACTIONS(1777),
    [sym_push_device] = ACTIONS(1777),
    [sym_push_channel] = ACTIONS(1777),
    [sym_push_devchan] = ACTIONS(1777),
    [sym_release_device] = ACTIONS(1777),
    [sym_release_channel] = ACTIONS(1777),
    [sym_release_devchan] = ACTIONS(1777),
    [sym_master_slot] = ACTIONS(1777),
    [sym_get_pulse_time] = ACTIONS(1777),
    [sym_get_timer] = ACTIONS(1777),
    [sym_master_sn] = ACTIONS(1777),
    [sym_system_number] = ACTIONS(1777),
    [sym_button] = ACTIONS(1777),
    [sym_dv_channel] = ACTIONS(1777),
    [sym_data] = ACTIONS(1777),
    [sym_level] = ACTIONS(1777),
    [sym_timeline] = ACTIONS(1777),
    [sym_channel] = ACTIONS(1777),
    [sym_custom] = ACTIONS(1777),
    [sym_atoi] = ACTIONS(1777),
    [sym_atol] = ACTIONS(1777),
    [sym_atof] = ACTIONS(1777),
    [sym_itoa] = ACTIONS(1777),
    [sym_itohex] = ACTIONS(1777),
    [sym_hextoi] = ACTIONS(1777),
    [sym_ftoa] = ACTIONS(1777),
    [sym_find_string] = ACTIONS(1777),
    [sym_left_string] = ACTIONS(1777),
    [sym_right_string] = ACTIONS(1777),
    [sym_mid_string] = ACTIONS(1777),
    [sym_remove_string] = ACTIONS(1777),
    [sym_get_buffer_string] = ACTIONS(1777),
    [sym_get_buffer_char] = ACTIONS(1777),
    [sym_get_multi_buffer_string] = ACTIONS(1777),
    [sym_length_string] = ACTIONS(1777),
    [sym_max_length_string] = ACTIONS(1777),
    [sym_set_length_string] = ACTIONS(1777),
    [sym_compare_string] = ACTIONS(1777),
    [sym_upper_string] = ACTIONS(1777),
    [sym_lower_string] = ACTIONS(1777),
    [sym_get_last] = ACTIONS(1777),
    [sym_date_to_day] = ACTIONS(1777),
    [sym_date_to_month] = ACTIONS(1777),
    [sym_date_to_year] = ACTIONS(1777),
    [sym_time_to_hour] = ACTIONS(1777),
    [sym_time_to_minute] = ACTIONS(1777),
    [sym_time_to_second] = ACTIONS(1777),
    [sym_day_of_week] = ACTIONS(1777),
    [sym_set_pulse_time] = ACTIONS(1777),
    [sym_set_timer] = ACTIONS(1777),
    [sym_file_open] = ACTIONS(1777),
    [sym_file_close] = ACTIONS(1777),
    [sym_file_read] = ACTIONS(1777),
    [sym_file_read_line] = ACTIONS(1777),
    [sym_file_write] = ACTIONS(1777),
    [sym_file_write_line] = ACTIONS(1777),
    [sym_file_copy] = ACTIONS(1777),
    [sym_file_delete] = ACTIONS(1777),
    [sym_file_rename] = ACTIONS(1777),
    [sym_file_dir] = ACTIONS(1777),
    [sym_file_getdir] = ACTIONS(1777),
    [sym_file_setdir] = ACTIONS(1777),
    [sym_file_create_dir] = ACTIONS(1777),
    [sym_file_remove_dir] = ACTIONS(1777),
    [sym_file_seek] = ACTIONS(1777),
    [sym_length_array] = ACTIONS(1777),
    [sym_max_length_array] = ACTIONS(1777),
    [sym_set_length_array] = ACTIONS(1777),
    [sym_ip_client_open] = ACTIONS(1777),
    [sym_ip_client_close] = ACTIONS(1777),
    [sym_ip_server_open] = ACTIONS(1777),
    [sym_ip_server_close] = ACTIONS(1777),
    [sym_ip_bound_client_open] = ACTIONS(1777),
    [sym_ip_mc_server_open] = ACTIONS(1777),
    [sym_ip_set_option] = ACTIONS(1777),
    [sym_ssh_client_open] = ACTIONS(1777),
    [sym_ssh_client_close] = ACTIONS(1777),
    [sym_tls_client_open] = ACTIONS(1777),
    [sym_tls_client_close] = ACTIONS(1777),
    [sym_do_push] = ACTIONS(1777),
    [sym_do_push_timed] = ACTIONS(1777),
    [sym_do_release] = ACTIONS(1777),
    [sym_device_id] = ACTIONS(1777),
    [sym_device_info] = ACTIONS(1777),
    [sym_device_id_string] = ACTIONS(1777),
    [sym_device_standby] = ACTIONS(1777),
    [sym_device_wake] = ACTIONS(1777),
    [sym_timeline_create] = ACTIONS(1777),
    [sym_timeline_kill] = ACTIONS(1777),
    [sym_timeline_reload] = ACTIONS(1777),
    [sym_timeline_pause] = ACTIONS(1777),
    [sym_timeline_restart] = ACTIONS(1777),
    [sym_timeline_set] = ACTIONS(1777),
    [sym_timeline_get] = ACTIONS(1777),
    [sym_timeline_active] = ACTIONS(1777),
    [sym_timeline_dynamic_id] = ACTIONS(1777),
    [sym_variable_to_string] = ACTIONS(1777),
    [sym_string_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_string] = ACTIONS(1777),
    [sym_type_cast] = ACTIONS(1777),
    [sym_variable_to_xml] = ACTIONS(1777),
    [sym_xml_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_xml] = ACTIONS(1777),
    [sym_redirect_string] = ACTIONS(1777),
    [sym_random_number] = ACTIONS(1777),
    [sym_get_unique_id] = ACTIONS(1777),
    [sym_get_system_number] = ACTIONS(1777),
    [sym_set_system_number] = ACTIONS(1777),
    [sym_get_serial_number] = ACTIONS(1777),
    [sym_get_master_build] = ACTIONS(1777),
    [sym_set_validation_code] = ACTIONS(1777),
    [sym_reboot] = ACTIONS(1777),
    [sym_load_duet_module] = ACTIONS(1777),
    [sym_unload_duet_module] = ACTIONS(1777),
    [sym_duet_mem_size_set] = ACTIONS(1777),
    [sym_duet_mem_size_get] = ACTIONS(1777),
    [sym_dynamic_application_device] = ACTIONS(1777),
    [sym_dynamic_polled_port] = ACTIONS(1777),
    [sym_static_port_binding] = ACTIONS(1777),
    [sym_static_ip_binding] = ACTIONS(1777),
    [sym_get_dns_list] = ACTIONS(1777),
    [sym_set_dns_list] = ACTIONS(1777),
    [sym_get_ip_address] = ACTIONS(1777),
    [sym_set_ip_address] = ACTIONS(1777),
    [sym_get_url_list] = ACTIONS(1777),
    [sym_add_url_entry] = ACTIONS(1777),
    [sym_delete_url_entry] = ACTIONS(1777),
    [sym_abs_value] = ACTIONS(1777),
    [sym_max_value] = ACTIONS(1777),
    [sym_min_value] = ACTIONS(1777),
    [sym_exp_value] = ACTIONS(1777),
    [sym_log_value] = ACTIONS(1777),
    [sym_log10_value] = ACTIONS(1777),
    [sym_power_value] = ACTIONS(1777),
    [sym_sqrt_value] = ACTIONS(1777),
    [sym_raw_be] = ACTIONS(1777),
    [sym_raw_le] = ACTIONS(1777),
    [sym_internal_queue_size_set] = ACTIONS(1777),
    [sym_internal_queue_size_get] = ACTIONS(1777),
    [sym_internal_threshold_set] = ACTIONS(1777),
    [sym_internal_threshold_get] = ACTIONS(1777),
    [sym_rebuilt_event] = ACTIONS(1777),
    [sym_do_custom_event] = ACTIONS(1777),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1777),
    [sym_clkmgr_set_clk_source] = ACTIONS(1777),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1777),
    [sym_clkmgr_get_timezone] = ACTIONS(1777),
    [sym_clkmgr_set_timezone] = ACTIONS(1777),
    [sym_clkmgr_get_resync_period] = ACTIONS(1777),
    [sym_clkmgr_set_resync_period] = ACTIONS(1777),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_timeservers] = ACTIONS(1777),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_set_log_level] = ACTIONS(1777),
    [sym_get_log_level] = ACTIONS(1777),
    [sym_amx_log] = ACTIONS(1777),
    [sym_smtp_server_config_set] = ACTIONS(1777),
    [sym_smtp_server_config_get] = ACTIONS(1777),
    [sym_smtp_send] = ACTIONS(1777),
    [sym_validate_netlinx_account] = ACTIONS(1777),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1777),
    [sym_authenticate_certificate] = ACTIONS(1777),
    [sym_audit_netlinx_session_event] = ACTIONS(1777),
    [sym_audit_netlinx_generic_event] = ACTIONS(1777),
    [sym_get_available_flash_disk_space] = ACTIONS(1777),
    [sym_get_max_flash_disk_space] = ACTIONS(1777),
    [sym_data_create_feed] = ACTIONS(1777),
    [sym_data_delete_feed] = ACTIONS(1777),
    [sym_data_publish_feed] = ACTIONS(1777),
    [sym_data_get_published_feed] = ACTIONS(1777),
    [sym_data_add_record] = ACTIONS(1777),
    [sym_data_get_event_record] = ACTIONS(1777),
    [sym__wc_data_create_feed] = ACTIONS(1777),
    [sym__wc_data_add_record] = ACTIONS(1777),
    [sym__wc_data_get_event_record] = ACTIONS(1777),
    [sym_format] = ACTIONS(1777),
    [sym_astro_clock] = ACTIONS(1777),
    [sym_set_outdoor_temperature] = ACTIONS(1777),
    [sym_set_virtual_level_count] = ACTIONS(1777),
    [sym_set_virtual_channel_count] = ACTIONS(1777),
    [sym_set_virtual_port_count] = ACTIONS(1777),
    [sym__wc_encode_int] = ACTIONS(1777),
    [sym__wc_decode_int] = ACTIONS(1777),
    [sym__wc_int] = ACTIONS(1777),
    [sym__wc_to_ch_int] = ACTIONS(1777),
    [sym__ch_to_wc_int] = ACTIONS(1777),
    [sym__wc_find_string_int] = ACTIONS(1777),
    [sym__wc_left_string_int] = ACTIONS(1777),
    [sym__wc_lower_string_int] = ACTIONS(1777),
    [sym__wc_mid_string_int] = ACTIONS(1777),
    [sym__wc_remove_string_int] = ACTIONS(1777),
    [sym__wc_right_string_int] = ACTIONS(1777),
    [sym__wc_upper_string_int] = ACTIONS(1777),
    [sym__wc_compare_string_int] = ACTIONS(1777),
    [sym__wc_get_buffer_char_int] = ACTIONS(1777),
    [sym__wc_get_buffer_string_int] = ACTIONS(1777),
    [sym__wc_concat_string_int] = ACTIONS(1777),
    [sym__wc_file_open_int] = ACTIONS(1777),
    [sym__wc_file_close_int] = ACTIONS(1777),
    [sym__wc_file_read_int] = ACTIONS(1777),
    [sym__wc_file_read_line_int] = ACTIONS(1777),
    [sym__wc_file_write_int] = ACTIONS(1777),
    [sym__wc_file_write_line_int] = ACTIONS(1777),
    [aux_sym_preproc_include_token1] = ACTIONS(1777),
    [aux_sym_preproc_define_token1] = ACTIONS(1779),
    [aux_sym_preproc_warn_token1] = ACTIONS(1779),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1779),
    [anon_sym_LBRACK] = ACTIONS(1779),
    [anon_sym_LBRACE] = ACTIONS(1779),
    [anon_sym_RBRACE] = ACTIONS(1779),
    [anon_sym_LPAREN] = ACTIONS(1777),
    [anon_sym_BANG] = ACTIONS(1779),
    [anon_sym_TILDE] = ACTIONS(1779),
    [anon_sym_DASH] = ACTIONS(1777),
    [anon_sym_PLUS] = ACTIONS(1777),
    [anon_sym_DASH_DASH] = ACTIONS(1779),
    [anon_sym_PLUS_PLUS] = ACTIONS(1779),
    [anon_sym_DQUOTE] = ACTIONS(1779),
    [anon_sym_SQUOTE] = ACTIONS(1779),
    [sym_number_literal] = ACTIONS(1779),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(503)] = {
    [sym_identifier] = ACTIONS(1777),
    [sym_define_call_keyword] = ACTIONS(1777),
    [sym_define_function_keyword] = ACTIONS(1777),
    [sym_define_library_function_keyword] = ACTIONS(1777),
    [sym_define_module_keyword] = ACTIONS(1777),
    [sym_char_keyword] = ACTIONS(1777),
    [sym_widechar_keyword] = ACTIONS(1777),
    [sym_integer_keyword] = ACTIONS(1777),
    [sym_sinteger_keyword] = ACTIONS(1777),
    [sym_long_keyword] = ACTIONS(1777),
    [sym_slong_keyword] = ACTIONS(1777),
    [sym_float_keyword] = ACTIONS(1777),
    [sym_double_keyword] = ACTIONS(1777),
    [sym_variant_keyword] = ACTIONS(1777),
    [sym_variantarray_keyword] = ACTIONS(1777),
    [sym_local_var_keyword] = ACTIONS(1777),
    [sym_stack_var_keyword] = ACTIONS(1777),
    [sym_constant_keyword] = ACTIONS(1777),
    [sym_volatile_keyword] = ACTIONS(1777),
    [sym_non_volatile_keyword] = ACTIONS(1777),
    [sym_persistent_keyword] = ACTIONS(1777),
    [sym_dev_keyword] = ACTIONS(1777),
    [sym_devlev_keyword] = ACTIONS(1777),
    [sym_devchan_keyword] = ACTIONS(1777),
    [sym_if_keyword] = ACTIONS(1777),
    [sym_else_keyword] = ACTIONS(1777),
    [sym_switch_keyword] = ACTIONS(1777),
    [sym_case_keyword] = ACTIONS(1777),
    [sym_default_keyword] = ACTIONS(1777),
    [sym_while_keyword] = ACTIONS(1777),
    [sym_for_keyword] = ACTIONS(1777),
    [sym_break_keyword] = ACTIONS(1777),
    [sym_continue_keyword] = ACTIONS(1777),
    [sym_return_keyword] = ACTIONS(1777),
    [sym_select_keyword] = ACTIONS(1777),
    [sym_active_keyword] = ACTIONS(1777),
    [sym_struct_keyword] = ACTIONS(1777),
    [sym_structure_keyword] = ACTIONS(1777),
    [aux_sym_on_keyword_token1] = ACTIONS(1777),
    [aux_sym_off_keyword_token1] = ACTIONS(1777),
    [sym_send_level_keyword] = ACTIONS(1777),
    [sym_send_string_keyword] = ACTIONS(1777),
    [sym_send_command_keyword] = ACTIONS(1777),
    [sym_clear_buffer_keyword] = ACTIONS(1777),
    [sym_create_buffer_keyword] = ACTIONS(1777),
    [sym_create_multi_buffer_keyword] = ACTIONS(1777),
    [sym_call_keyword] = ACTIONS(1777),
    [sym_system_call_keyword] = ACTIONS(1777),
    [sym_devchan_to_keyword] = ACTIONS(1777),
    [sym_devchan_min_to_keyword] = ACTIONS(1777),
    [sym_devchan_total_off_keyword] = ACTIONS(1777),
    [sym_devchan_pulse_keyword] = ACTIONS(1777),
    [sym_wait_keyword] = ACTIONS(1777),
    [sym_cancel_wait_keyword] = ACTIONS(1777),
    [sym_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1777),
    [sym_not] = ACTIONS(1777),
    [sym_bnot] = ACTIONS(1777),
    [sym___file__] = ACTIONS(1777),
    [sym___date__] = ACTIONS(1777),
    [sym___ldate__] = ACTIONS(1777),
    [sym___time__] = ACTIONS(1777),
    [sym___line__] = ACTIONS(1777),
    [sym___name__] = ACTIONS(1777),
    [sym___netlinx__] = ACTIONS(1777),
    [sym_netlinx_axi_version] = ACTIONS(1777),
    [sym_url_flg_tcp] = ACTIONS(1777),
    [sym_url_flg_accinfopresent] = ACTIONS(1777),
    [sym_url_flg_temp] = ACTIONS(1777),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1777),
    [sym_url_flg_stat_mask] = ACTIONS(1777),
    [sym_url_flg_stat_lookup] = ACTIONS(1777),
    [sym_url_flg_stat_connecting] = ACTIONS(1777),
    [sym_url_flg_stat_waiting] = ACTIONS(1777),
    [sym_url_flg_stat_connected] = ACTIONS(1777),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1777),
    [sym_first_virtual_device] = ACTIONS(1777),
    [sym_first_local_port] = ACTIONS(1777),
    [sym_dynamic_virtual_device] = ACTIONS(1777),
    [sym_timeline_once] = ACTIONS(1777),
    [sym_timeline_repeat] = ACTIONS(1777),
    [sym_timeline_absolute] = ACTIONS(1777),
    [sym_timeline_relative] = ACTIONS(1777),
    [sym_do_push_timed_infinite] = ACTIONS(1777),
    [sym_source_type_no_address] = ACTIONS(1777),
    [sym_source_type_neuron_id] = ACTIONS(1777),
    [sym_source_type_ip_address] = ACTIONS(1777),
    [sym_source_type_axlink] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1777),
    [sym_source_type_ip_socket_address] = ACTIONS(1777),
    [sym_source_type_rs232] = ACTIONS(1777),
    [sym_source_type_internal] = ACTIONS(1777),
    [sym_source_type_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1777),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1777),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1777),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1777),
    [sym_true] = ACTIONS(1777),
    [sym_false] = ACTIONS(1777),
    [sym_file_read_only] = ACTIONS(1777),
    [sym_file_rw_new] = ACTIONS(1777),
    [sym_file_rw_append] = ACTIONS(1777),
    [sym_ip_tcp] = ACTIONS(1777),
    [sym_ip_udp] = ACTIONS(1777),
    [sym_ip_udp_2way] = ACTIONS(1777),
    [sym_xml_encode_types] = ACTIONS(1777),
    [sym_xml_encode_char_as_list] = ACTIONS(1777),
    [sym_xml_encode_le] = ACTIONS(1777),
    [sym_xml_decode_types] = ACTIONS(1777),
    [sym_xml_decode_no_preserve] = ACTIONS(1777),
    [sym_duet_dev_type_display_device] = ACTIONS(1777),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_io_device] = ACTIONS(1777),
    [sym_duet_dev_type_relay_device] = ACTIONS(1777),
    [sym_duet_dev_type_ups] = ACTIONS(1777),
    [sym_duet_dev_type_amplifier] = ACTIONS(1777),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1777),
    [sym_duet_dev_type_camera] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1777),
    [sym_duet_dev_type_dss] = ACTIONS(1777),
    [sym_duet_dev_type_dvr] = ACTIONS(1777),
    [sym_duet_dev_type_disc_device] = ACTIONS(1777),
    [sym_duet_dev_type_document_camera] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1777),
    [sym_duet_dev_type_hvac] = ACTIONS(1777),
    [sym_duet_dev_type_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_light] = ACTIONS(1777),
    [sym_duet_dev_type_monitor] = ACTIONS(1777),
    [sym_duet_dev_type_motor] = ACTIONS(1777),
    [sym_duet_dev_type_multi_window] = ACTIONS(1777),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1777),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1777),
    [sym_duet_dev_type_receiver] = ACTIONS(1777),
    [sym_duet_dev_type_security_system] = ACTIONS(1777),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1777),
    [sym_duet_dev_type_settop_box] = ACTIONS(1777),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1777),
    [sym_duet_dev_type_switcher] = ACTIONS(1777),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_tv] = ACTIONS(1777),
    [sym_duet_dev_type_utility] = ACTIONS(1777),
    [sym_duet_dev_type_vcr] = ACTIONS(1777),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_video_processor] = ACTIONS(1777),
    [sym_duet_dev_type_video_projector] = ACTIONS(1777),
    [sym_duet_dev_type_video_wall] = ACTIONS(1777),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1777),
    [sym_duet_dev_type_weather] = ACTIONS(1777),
    [sym_duet_dev_not_polled] = ACTIONS(1777),
    [sym_duet_dev_polled] = ACTIONS(1777),
    [sym_clkmgr_mode_network] = ACTIONS(1777),
    [sym_clkmgr_mode_standalone] = ACTIONS(1777),
    [sym_normal_standby] = ACTIONS(1777),
    [sym_normal_wake] = ACTIONS(1777),
    [sym_ip_multicast_ttl_option] = ACTIONS(1777),
    [sym_ip_ttl_subnet] = ACTIONS(1777),
    [sym_ip_ttl_site] = ACTIONS(1777),
    [sym_ip_ttl_region] = ACTIONS(1777),
    [sym_ip_ttl_continent] = ACTIONS(1777),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1777),
    [sym_ip_nodelay_on] = ACTIONS(1777),
    [sym_ip_nodelay_off] = ACTIONS(1777),
    [sym_amx_error] = ACTIONS(1777),
    [sym_amx_warning] = ACTIONS(1777),
    [sym_amx_info] = ACTIONS(1777),
    [sym_amx_debug] = ACTIONS(1777),
    [sym_netlinx_login_success] = ACTIONS(1777),
    [sym_netlinx_login_fail] = ACTIONS(1777),
    [sym_netlinx_logout] = ACTIONS(1777),
    [sym_null_str] = ACTIONS(1777),
    [sym_smtp_address] = ACTIONS(1777),
    [sym_smtp_port_number] = ACTIONS(1777),
    [sym_smtp_username] = ACTIONS(1777),
    [sym_smtp_password] = ACTIONS(1777),
    [sym_smtp_require_tls] = ACTIONS(1777),
    [sym_smtp_from] = ACTIONS(1777),
    [sym_smtp_tls_true] = ACTIONS(1777),
    [sym_smtp_tls_false] = ACTIONS(1777),
    [sym_listview_on_row_select_event] = ACTIONS(1777),
    [sym_data_structure_unknown] = ACTIONS(1777),
    [sym_data_structure_datafeed] = ACTIONS(1777),
    [sym_data_structure_datarecordset] = ACTIONS(1777),
    [sym_data_structure_datarecord] = ACTIONS(1777),
    [sym_data_structure_datafield] = ACTIONS(1777),
    [sym_data_max_content_field_count] = ACTIONS(1777),
    [sym_data_max_metadata_field_count] = ACTIONS(1777),
    [sym_data_max_id_length] = ACTIONS(1777),
    [sym_data_max_name_length] = ACTIONS(1777),
    [sym_data_max_description_length] = ACTIONS(1777),
    [sym_data_max_type_length] = ACTIONS(1777),
    [sym_data_max_format_length] = ACTIONS(1777),
    [sym_data_max_source_length] = ACTIONS(1777),
    [sym_data_max_value_length] = ACTIONS(1777),
    [sym_data_max_label_length] = ACTIONS(1777),
    [sym_data_max_url_length] = ACTIONS(1777),
    [sym_max_last_login_info_length] = ACTIONS(1777),
    [sym_data_type_unknown] = ACTIONS(1777),
    [sym_data_type_string] = ACTIONS(1777),
    [sym_data_type_datetime] = ACTIONS(1777),
    [sym_data_type_date] = ACTIONS(1777),
    [sym_data_type_time] = ACTIONS(1777),
    [sym_data_type_image] = ACTIONS(1777),
    [sym_data_format_url] = ACTIONS(1777),
    [sym_data_format_phone] = ACTIONS(1777),
    [sym_data_format_email] = ACTIONS(1777),
    [sym_data_format_iso8601] = ACTIONS(1777),
    [sym_auditlog_priv] = ACTIONS(1777),
    [sym_configuration_priv] = ACTIONS(1777),
    [sym_deviceconfig_priv] = ACTIONS(1777),
    [sym_ftp_priv] = ACTIONS(1777),
    [sym_http_priv] = ACTIONS(1777),
    [sym_networkconfig_priv] = ACTIONS(1777),
    [sym_programport_priv] = ACTIONS(1777),
    [sym_remoteui_priv] = ACTIONS(1777),
    [sym_securitycontrol_priv] = ACTIONS(1777),
    [sym_softwaremanagement_priv] = ACTIONS(1777),
    [sym_terminal_priv] = ACTIONS(1777),
    [sym_tpadmin_priv] = ACTIONS(1777),
    [sym_user1_priv] = ACTIONS(1777),
    [sym_user2_priv] = ACTIONS(1777),
    [sym_user3_priv] = ACTIONS(1777),
    [sym_user4_priv] = ACTIONS(1777),
    [sym_usermanagement_priv] = ACTIONS(1777),
    [sym_valid_account] = ACTIONS(1777),
    [sym_err_name_invalid] = ACTIONS(1777),
    [sym_err_password_invalid] = ACTIONS(1777),
    [sym_err_invalid_account] = ACTIONS(1777),
    [sym_err_no_authorization] = ACTIONS(1777),
    [sym_err_invalid_parameter] = ACTIONS(1777),
    [sym_err_user_locked_out] = ACTIONS(1777),
    [sym_err_user_account_expired] = ACTIONS(1777),
    [sym_err_general_error] = ACTIONS(1777),
    [sym_err_authentication_error] = ACTIONS(1777),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1777),
    [sym_tls_validate_certificate] = ACTIONS(1777),
    [sym_tdata] = ACTIONS(1777),
    [sym_tchannel] = ACTIONS(1777),
    [sym_tlevel] = ACTIONS(1777),
    [sym_tbutton] = ACTIONS(1777),
    [sym_ttimeline] = ACTIONS(1777),
    [sym_tcustom] = ACTIONS(1777),
    [sym_url_struct] = ACTIONS(1777),
    [sym_dns_struct] = ACTIONS(1777),
    [sym_ip_address_struct] = ACTIONS(1777),
    [sym_dev_info_struct] = ACTIONS(1777),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1777),
    [sym_data_feed] = ACTIONS(1777),
    [sym_data_field] = ACTIONS(1777),
    [sym_data_record] = ACTIONS(1777),
    [sym_wc_data_feed] = ACTIONS(1777),
    [sym_wc_data_field] = ACTIONS(1777),
    [sym_wc_data_record] = ACTIONS(1777),
    [sym_last_login_info] = ACTIONS(1777),
    [sym____reserved___] = ACTIONS(1777),
    [sym_date] = ACTIONS(1777),
    [sym_day] = ACTIONS(1777),
    [sym_time] = ACTIONS(1777),
    [sym_ldate] = ACTIONS(1777),
    [sym_push_device] = ACTIONS(1777),
    [sym_push_channel] = ACTIONS(1777),
    [sym_push_devchan] = ACTIONS(1777),
    [sym_release_device] = ACTIONS(1777),
    [sym_release_channel] = ACTIONS(1777),
    [sym_release_devchan] = ACTIONS(1777),
    [sym_master_slot] = ACTIONS(1777),
    [sym_get_pulse_time] = ACTIONS(1777),
    [sym_get_timer] = ACTIONS(1777),
    [sym_master_sn] = ACTIONS(1777),
    [sym_system_number] = ACTIONS(1777),
    [sym_button] = ACTIONS(1777),
    [sym_dv_channel] = ACTIONS(1777),
    [sym_data] = ACTIONS(1777),
    [sym_level] = ACTIONS(1777),
    [sym_timeline] = ACTIONS(1777),
    [sym_channel] = ACTIONS(1777),
    [sym_custom] = ACTIONS(1777),
    [sym_atoi] = ACTIONS(1777),
    [sym_atol] = ACTIONS(1777),
    [sym_atof] = ACTIONS(1777),
    [sym_itoa] = ACTIONS(1777),
    [sym_itohex] = ACTIONS(1777),
    [sym_hextoi] = ACTIONS(1777),
    [sym_ftoa] = ACTIONS(1777),
    [sym_find_string] = ACTIONS(1777),
    [sym_left_string] = ACTIONS(1777),
    [sym_right_string] = ACTIONS(1777),
    [sym_mid_string] = ACTIONS(1777),
    [sym_remove_string] = ACTIONS(1777),
    [sym_get_buffer_string] = ACTIONS(1777),
    [sym_get_buffer_char] = ACTIONS(1777),
    [sym_get_multi_buffer_string] = ACTIONS(1777),
    [sym_length_string] = ACTIONS(1777),
    [sym_max_length_string] = ACTIONS(1777),
    [sym_set_length_string] = ACTIONS(1777),
    [sym_compare_string] = ACTIONS(1777),
    [sym_upper_string] = ACTIONS(1777),
    [sym_lower_string] = ACTIONS(1777),
    [sym_get_last] = ACTIONS(1777),
    [sym_date_to_day] = ACTIONS(1777),
    [sym_date_to_month] = ACTIONS(1777),
    [sym_date_to_year] = ACTIONS(1777),
    [sym_time_to_hour] = ACTIONS(1777),
    [sym_time_to_minute] = ACTIONS(1777),
    [sym_time_to_second] = ACTIONS(1777),
    [sym_day_of_week] = ACTIONS(1777),
    [sym_set_pulse_time] = ACTIONS(1777),
    [sym_set_timer] = ACTIONS(1777),
    [sym_file_open] = ACTIONS(1777),
    [sym_file_close] = ACTIONS(1777),
    [sym_file_read] = ACTIONS(1777),
    [sym_file_read_line] = ACTIONS(1777),
    [sym_file_write] = ACTIONS(1777),
    [sym_file_write_line] = ACTIONS(1777),
    [sym_file_copy] = ACTIONS(1777),
    [sym_file_delete] = ACTIONS(1777),
    [sym_file_rename] = ACTIONS(1777),
    [sym_file_dir] = ACTIONS(1777),
    [sym_file_getdir] = ACTIONS(1777),
    [sym_file_setdir] = ACTIONS(1777),
    [sym_file_create_dir] = ACTIONS(1777),
    [sym_file_remove_dir] = ACTIONS(1777),
    [sym_file_seek] = ACTIONS(1777),
    [sym_length_array] = ACTIONS(1777),
    [sym_max_length_array] = ACTIONS(1777),
    [sym_set_length_array] = ACTIONS(1777),
    [sym_ip_client_open] = ACTIONS(1777),
    [sym_ip_client_close] = ACTIONS(1777),
    [sym_ip_server_open] = ACTIONS(1777),
    [sym_ip_server_close] = ACTIONS(1777),
    [sym_ip_bound_client_open] = ACTIONS(1777),
    [sym_ip_mc_server_open] = ACTIONS(1777),
    [sym_ip_set_option] = ACTIONS(1777),
    [sym_ssh_client_open] = ACTIONS(1777),
    [sym_ssh_client_close] = ACTIONS(1777),
    [sym_tls_client_open] = ACTIONS(1777),
    [sym_tls_client_close] = ACTIONS(1777),
    [sym_do_push] = ACTIONS(1777),
    [sym_do_push_timed] = ACTIONS(1777),
    [sym_do_release] = ACTIONS(1777),
    [sym_device_id] = ACTIONS(1777),
    [sym_device_info] = ACTIONS(1777),
    [sym_device_id_string] = ACTIONS(1777),
    [sym_device_standby] = ACTIONS(1777),
    [sym_device_wake] = ACTIONS(1777),
    [sym_timeline_create] = ACTIONS(1777),
    [sym_timeline_kill] = ACTIONS(1777),
    [sym_timeline_reload] = ACTIONS(1777),
    [sym_timeline_pause] = ACTIONS(1777),
    [sym_timeline_restart] = ACTIONS(1777),
    [sym_timeline_set] = ACTIONS(1777),
    [sym_timeline_get] = ACTIONS(1777),
    [sym_timeline_active] = ACTIONS(1777),
    [sym_timeline_dynamic_id] = ACTIONS(1777),
    [sym_variable_to_string] = ACTIONS(1777),
    [sym_string_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_string] = ACTIONS(1777),
    [sym_type_cast] = ACTIONS(1777),
    [sym_variable_to_xml] = ACTIONS(1777),
    [sym_xml_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_xml] = ACTIONS(1777),
    [sym_redirect_string] = ACTIONS(1777),
    [sym_random_number] = ACTIONS(1777),
    [sym_get_unique_id] = ACTIONS(1777),
    [sym_get_system_number] = ACTIONS(1777),
    [sym_set_system_number] = ACTIONS(1777),
    [sym_get_serial_number] = ACTIONS(1777),
    [sym_get_master_build] = ACTIONS(1777),
    [sym_set_validation_code] = ACTIONS(1777),
    [sym_reboot] = ACTIONS(1777),
    [sym_load_duet_module] = ACTIONS(1777),
    [sym_unload_duet_module] = ACTIONS(1777),
    [sym_duet_mem_size_set] = ACTIONS(1777),
    [sym_duet_mem_size_get] = ACTIONS(1777),
    [sym_dynamic_application_device] = ACTIONS(1777),
    [sym_dynamic_polled_port] = ACTIONS(1777),
    [sym_static_port_binding] = ACTIONS(1777),
    [sym_static_ip_binding] = ACTIONS(1777),
    [sym_get_dns_list] = ACTIONS(1777),
    [sym_set_dns_list] = ACTIONS(1777),
    [sym_get_ip_address] = ACTIONS(1777),
    [sym_set_ip_address] = ACTIONS(1777),
    [sym_get_url_list] = ACTIONS(1777),
    [sym_add_url_entry] = ACTIONS(1777),
    [sym_delete_url_entry] = ACTIONS(1777),
    [sym_abs_value] = ACTIONS(1777),
    [sym_max_value] = ACTIONS(1777),
    [sym_min_value] = ACTIONS(1777),
    [sym_exp_value] = ACTIONS(1777),
    [sym_log_value] = ACTIONS(1777),
    [sym_log10_value] = ACTIONS(1777),
    [sym_power_value] = ACTIONS(1777),
    [sym_sqrt_value] = ACTIONS(1777),
    [sym_raw_be] = ACTIONS(1777),
    [sym_raw_le] = ACTIONS(1777),
    [sym_internal_queue_size_set] = ACTIONS(1777),
    [sym_internal_queue_size_get] = ACTIONS(1777),
    [sym_internal_threshold_set] = ACTIONS(1777),
    [sym_internal_threshold_get] = ACTIONS(1777),
    [sym_rebuilt_event] = ACTIONS(1777),
    [sym_do_custom_event] = ACTIONS(1777),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1777),
    [sym_clkmgr_set_clk_source] = ACTIONS(1777),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1777),
    [sym_clkmgr_get_timezone] = ACTIONS(1777),
    [sym_clkmgr_set_timezone] = ACTIONS(1777),
    [sym_clkmgr_get_resync_period] = ACTIONS(1777),
    [sym_clkmgr_set_resync_period] = ACTIONS(1777),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_timeservers] = ACTIONS(1777),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_set_log_level] = ACTIONS(1777),
    [sym_get_log_level] = ACTIONS(1777),
    [sym_amx_log] = ACTIONS(1777),
    [sym_smtp_server_config_set] = ACTIONS(1777),
    [sym_smtp_server_config_get] = ACTIONS(1777),
    [sym_smtp_send] = ACTIONS(1777),
    [sym_validate_netlinx_account] = ACTIONS(1777),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1777),
    [sym_authenticate_certificate] = ACTIONS(1777),
    [sym_audit_netlinx_session_event] = ACTIONS(1777),
    [sym_audit_netlinx_generic_event] = ACTIONS(1777),
    [sym_get_available_flash_disk_space] = ACTIONS(1777),
    [sym_get_max_flash_disk_space] = ACTIONS(1777),
    [sym_data_create_feed] = ACTIONS(1777),
    [sym_data_delete_feed] = ACTIONS(1777),
    [sym_data_publish_feed] = ACTIONS(1777),
    [sym_data_get_published_feed] = ACTIONS(1777),
    [sym_data_add_record] = ACTIONS(1777),
    [sym_data_get_event_record] = ACTIONS(1777),
    [sym__wc_data_create_feed] = ACTIONS(1777),
    [sym__wc_data_add_record] = ACTIONS(1777),
    [sym__wc_data_get_event_record] = ACTIONS(1777),
    [sym_format] = ACTIONS(1777),
    [sym_astro_clock] = ACTIONS(1777),
    [sym_set_outdoor_temperature] = ACTIONS(1777),
    [sym_set_virtual_level_count] = ACTIONS(1777),
    [sym_set_virtual_channel_count] = ACTIONS(1777),
    [sym_set_virtual_port_count] = ACTIONS(1777),
    [sym__wc_encode_int] = ACTIONS(1777),
    [sym__wc_decode_int] = ACTIONS(1777),
    [sym__wc_int] = ACTIONS(1777),
    [sym__wc_to_ch_int] = ACTIONS(1777),
    [sym__ch_to_wc_int] = ACTIONS(1777),
    [sym__wc_find_string_int] = ACTIONS(1777),
    [sym__wc_left_string_int] = ACTIONS(1777),
    [sym__wc_lower_string_int] = ACTIONS(1777),
    [sym__wc_mid_string_int] = ACTIONS(1777),
    [sym__wc_remove_string_int] = ACTIONS(1777),
    [sym__wc_right_string_int] = ACTIONS(1777),
    [sym__wc_upper_string_int] = ACTIONS(1777),
    [sym__wc_compare_string_int] = ACTIONS(1777),
    [sym__wc_get_buffer_char_int] = ACTIONS(1777),
    [sym__wc_get_buffer_string_int] = ACTIONS(1777),
    [sym__wc_concat_string_int] = ACTIONS(1777),
    [sym__wc_file_open_int] = ACTIONS(1777),
    [sym__wc_file_close_int] = ACTIONS(1777),
    [sym__wc_file_read_int] = ACTIONS(1777),
    [sym__wc_file_read_line_int] = ACTIONS(1777),
    [sym__wc_file_write_int] = ACTIONS(1777),
    [sym__wc_file_write_line_int] = ACTIONS(1777),
    [aux_sym_preproc_include_token1] = ACTIONS(1777),
    [aux_sym_preproc_define_token1] = ACTIONS(1779),
    [aux_sym_preproc_warn_token1] = ACTIONS(1779),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1779),
    [anon_sym_LBRACK] = ACTIONS(1779),
    [anon_sym_LBRACE] = ACTIONS(1779),
    [anon_sym_RBRACE] = ACTIONS(1779),
    [anon_sym_LPAREN] = ACTIONS(1777),
    [anon_sym_BANG] = ACTIONS(1779),
    [anon_sym_TILDE] = ACTIONS(1779),
    [anon_sym_DASH] = ACTIONS(1777),
    [anon_sym_PLUS] = ACTIONS(1777),
    [anon_sym_DASH_DASH] = ACTIONS(1779),
    [anon_sym_PLUS_PLUS] = ACTIONS(1779),
    [anon_sym_DQUOTE] = ACTIONS(1779),
    [anon_sym_SQUOTE] = ACTIONS(1779),
    [sym_number_literal] = ACTIONS(1779),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(504)] = {
    [sym_identifier] = ACTIONS(1777),
    [sym_define_call_keyword] = ACTIONS(1777),
    [sym_define_function_keyword] = ACTIONS(1777),
    [sym_define_library_function_keyword] = ACTIONS(1777),
    [sym_define_module_keyword] = ACTIONS(1777),
    [sym_char_keyword] = ACTIONS(1777),
    [sym_widechar_keyword] = ACTIONS(1777),
    [sym_integer_keyword] = ACTIONS(1777),
    [sym_sinteger_keyword] = ACTIONS(1777),
    [sym_long_keyword] = ACTIONS(1777),
    [sym_slong_keyword] = ACTIONS(1777),
    [sym_float_keyword] = ACTIONS(1777),
    [sym_double_keyword] = ACTIONS(1777),
    [sym_variant_keyword] = ACTIONS(1777),
    [sym_variantarray_keyword] = ACTIONS(1777),
    [sym_local_var_keyword] = ACTIONS(1777),
    [sym_stack_var_keyword] = ACTIONS(1777),
    [sym_constant_keyword] = ACTIONS(1777),
    [sym_volatile_keyword] = ACTIONS(1777),
    [sym_non_volatile_keyword] = ACTIONS(1777),
    [sym_persistent_keyword] = ACTIONS(1777),
    [sym_dev_keyword] = ACTIONS(1777),
    [sym_devlev_keyword] = ACTIONS(1777),
    [sym_devchan_keyword] = ACTIONS(1777),
    [sym_if_keyword] = ACTIONS(1777),
    [sym_else_keyword] = ACTIONS(1777),
    [sym_switch_keyword] = ACTIONS(1777),
    [sym_case_keyword] = ACTIONS(1777),
    [sym_default_keyword] = ACTIONS(1777),
    [sym_while_keyword] = ACTIONS(1777),
    [sym_for_keyword] = ACTIONS(1777),
    [sym_break_keyword] = ACTIONS(1777),
    [sym_continue_keyword] = ACTIONS(1777),
    [sym_return_keyword] = ACTIONS(1777),
    [sym_select_keyword] = ACTIONS(1777),
    [sym_active_keyword] = ACTIONS(1777),
    [sym_struct_keyword] = ACTIONS(1777),
    [sym_structure_keyword] = ACTIONS(1777),
    [aux_sym_on_keyword_token1] = ACTIONS(1777),
    [aux_sym_off_keyword_token1] = ACTIONS(1777),
    [sym_send_level_keyword] = ACTIONS(1777),
    [sym_send_string_keyword] = ACTIONS(1777),
    [sym_send_command_keyword] = ACTIONS(1777),
    [sym_clear_buffer_keyword] = ACTIONS(1777),
    [sym_create_buffer_keyword] = ACTIONS(1777),
    [sym_create_multi_buffer_keyword] = ACTIONS(1777),
    [sym_call_keyword] = ACTIONS(1777),
    [sym_system_call_keyword] = ACTIONS(1777),
    [sym_devchan_to_keyword] = ACTIONS(1777),
    [sym_devchan_min_to_keyword] = ACTIONS(1777),
    [sym_devchan_total_off_keyword] = ACTIONS(1777),
    [sym_devchan_pulse_keyword] = ACTIONS(1777),
    [sym_wait_keyword] = ACTIONS(1777),
    [sym_cancel_wait_keyword] = ACTIONS(1777),
    [sym_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1777),
    [sym_not] = ACTIONS(1777),
    [sym_bnot] = ACTIONS(1777),
    [sym___file__] = ACTIONS(1777),
    [sym___date__] = ACTIONS(1777),
    [sym___ldate__] = ACTIONS(1777),
    [sym___time__] = ACTIONS(1777),
    [sym___line__] = ACTIONS(1777),
    [sym___name__] = ACTIONS(1777),
    [sym___netlinx__] = ACTIONS(1777),
    [sym_netlinx_axi_version] = ACTIONS(1777),
    [sym_url_flg_tcp] = ACTIONS(1777),
    [sym_url_flg_accinfopresent] = ACTIONS(1777),
    [sym_url_flg_temp] = ACTIONS(1777),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1777),
    [sym_url_flg_stat_mask] = ACTIONS(1777),
    [sym_url_flg_stat_lookup] = ACTIONS(1777),
    [sym_url_flg_stat_connecting] = ACTIONS(1777),
    [sym_url_flg_stat_waiting] = ACTIONS(1777),
    [sym_url_flg_stat_connected] = ACTIONS(1777),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1777),
    [sym_first_virtual_device] = ACTIONS(1777),
    [sym_first_local_port] = ACTIONS(1777),
    [sym_dynamic_virtual_device] = ACTIONS(1777),
    [sym_timeline_once] = ACTIONS(1777),
    [sym_timeline_repeat] = ACTIONS(1777),
    [sym_timeline_absolute] = ACTIONS(1777),
    [sym_timeline_relative] = ACTIONS(1777),
    [sym_do_push_timed_infinite] = ACTIONS(1777),
    [sym_source_type_no_address] = ACTIONS(1777),
    [sym_source_type_neuron_id] = ACTIONS(1777),
    [sym_source_type_ip_address] = ACTIONS(1777),
    [sym_source_type_axlink] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1777),
    [sym_source_type_ip_socket_address] = ACTIONS(1777),
    [sym_source_type_rs232] = ACTIONS(1777),
    [sym_source_type_internal] = ACTIONS(1777),
    [sym_source_type_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1777),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1777),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1777),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1777),
    [sym_true] = ACTIONS(1777),
    [sym_false] = ACTIONS(1777),
    [sym_file_read_only] = ACTIONS(1777),
    [sym_file_rw_new] = ACTIONS(1777),
    [sym_file_rw_append] = ACTIONS(1777),
    [sym_ip_tcp] = ACTIONS(1777),
    [sym_ip_udp] = ACTIONS(1777),
    [sym_ip_udp_2way] = ACTIONS(1777),
    [sym_xml_encode_types] = ACTIONS(1777),
    [sym_xml_encode_char_as_list] = ACTIONS(1777),
    [sym_xml_encode_le] = ACTIONS(1777),
    [sym_xml_decode_types] = ACTIONS(1777),
    [sym_xml_decode_no_preserve] = ACTIONS(1777),
    [sym_duet_dev_type_display_device] = ACTIONS(1777),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_io_device] = ACTIONS(1777),
    [sym_duet_dev_type_relay_device] = ACTIONS(1777),
    [sym_duet_dev_type_ups] = ACTIONS(1777),
    [sym_duet_dev_type_amplifier] = ACTIONS(1777),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1777),
    [sym_duet_dev_type_camera] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1777),
    [sym_duet_dev_type_dss] = ACTIONS(1777),
    [sym_duet_dev_type_dvr] = ACTIONS(1777),
    [sym_duet_dev_type_disc_device] = ACTIONS(1777),
    [sym_duet_dev_type_document_camera] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1777),
    [sym_duet_dev_type_hvac] = ACTIONS(1777),
    [sym_duet_dev_type_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_light] = ACTIONS(1777),
    [sym_duet_dev_type_monitor] = ACTIONS(1777),
    [sym_duet_dev_type_motor] = ACTIONS(1777),
    [sym_duet_dev_type_multi_window] = ACTIONS(1777),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1777),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1777),
    [sym_duet_dev_type_receiver] = ACTIONS(1777),
    [sym_duet_dev_type_security_system] = ACTIONS(1777),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1777),
    [sym_duet_dev_type_settop_box] = ACTIONS(1777),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1777),
    [sym_duet_dev_type_switcher] = ACTIONS(1777),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_tv] = ACTIONS(1777),
    [sym_duet_dev_type_utility] = ACTIONS(1777),
    [sym_duet_dev_type_vcr] = ACTIONS(1777),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_video_processor] = ACTIONS(1777),
    [sym_duet_dev_type_video_projector] = ACTIONS(1777),
    [sym_duet_dev_type_video_wall] = ACTIONS(1777),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1777),
    [sym_duet_dev_type_weather] = ACTIONS(1777),
    [sym_duet_dev_not_polled] = ACTIONS(1777),
    [sym_duet_dev_polled] = ACTIONS(1777),
    [sym_clkmgr_mode_network] = ACTIONS(1777),
    [sym_clkmgr_mode_standalone] = ACTIONS(1777),
    [sym_normal_standby] = ACTIONS(1777),
    [sym_normal_wake] = ACTIONS(1777),
    [sym_ip_multicast_ttl_option] = ACTIONS(1777),
    [sym_ip_ttl_subnet] = ACTIONS(1777),
    [sym_ip_ttl_site] = ACTIONS(1777),
    [sym_ip_ttl_region] = ACTIONS(1777),
    [sym_ip_ttl_continent] = ACTIONS(1777),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1777),
    [sym_ip_nodelay_on] = ACTIONS(1777),
    [sym_ip_nodelay_off] = ACTIONS(1777),
    [sym_amx_error] = ACTIONS(1777),
    [sym_amx_warning] = ACTIONS(1777),
    [sym_amx_info] = ACTIONS(1777),
    [sym_amx_debug] = ACTIONS(1777),
    [sym_netlinx_login_success] = ACTIONS(1777),
    [sym_netlinx_login_fail] = ACTIONS(1777),
    [sym_netlinx_logout] = ACTIONS(1777),
    [sym_null_str] = ACTIONS(1777),
    [sym_smtp_address] = ACTIONS(1777),
    [sym_smtp_port_number] = ACTIONS(1777),
    [sym_smtp_username] = ACTIONS(1777),
    [sym_smtp_password] = ACTIONS(1777),
    [sym_smtp_require_tls] = ACTIONS(1777),
    [sym_smtp_from] = ACTIONS(1777),
    [sym_smtp_tls_true] = ACTIONS(1777),
    [sym_smtp_tls_false] = ACTIONS(1777),
    [sym_listview_on_row_select_event] = ACTIONS(1777),
    [sym_data_structure_unknown] = ACTIONS(1777),
    [sym_data_structure_datafeed] = ACTIONS(1777),
    [sym_data_structure_datarecordset] = ACTIONS(1777),
    [sym_data_structure_datarecord] = ACTIONS(1777),
    [sym_data_structure_datafield] = ACTIONS(1777),
    [sym_data_max_content_field_count] = ACTIONS(1777),
    [sym_data_max_metadata_field_count] = ACTIONS(1777),
    [sym_data_max_id_length] = ACTIONS(1777),
    [sym_data_max_name_length] = ACTIONS(1777),
    [sym_data_max_description_length] = ACTIONS(1777),
    [sym_data_max_type_length] = ACTIONS(1777),
    [sym_data_max_format_length] = ACTIONS(1777),
    [sym_data_max_source_length] = ACTIONS(1777),
    [sym_data_max_value_length] = ACTIONS(1777),
    [sym_data_max_label_length] = ACTIONS(1777),
    [sym_data_max_url_length] = ACTIONS(1777),
    [sym_max_last_login_info_length] = ACTIONS(1777),
    [sym_data_type_unknown] = ACTIONS(1777),
    [sym_data_type_string] = ACTIONS(1777),
    [sym_data_type_datetime] = ACTIONS(1777),
    [sym_data_type_date] = ACTIONS(1777),
    [sym_data_type_time] = ACTIONS(1777),
    [sym_data_type_image] = ACTIONS(1777),
    [sym_data_format_url] = ACTIONS(1777),
    [sym_data_format_phone] = ACTIONS(1777),
    [sym_data_format_email] = ACTIONS(1777),
    [sym_data_format_iso8601] = ACTIONS(1777),
    [sym_auditlog_priv] = ACTIONS(1777),
    [sym_configuration_priv] = ACTIONS(1777),
    [sym_deviceconfig_priv] = ACTIONS(1777),
    [sym_ftp_priv] = ACTIONS(1777),
    [sym_http_priv] = ACTIONS(1777),
    [sym_networkconfig_priv] = ACTIONS(1777),
    [sym_programport_priv] = ACTIONS(1777),
    [sym_remoteui_priv] = ACTIONS(1777),
    [sym_securitycontrol_priv] = ACTIONS(1777),
    [sym_softwaremanagement_priv] = ACTIONS(1777),
    [sym_terminal_priv] = ACTIONS(1777),
    [sym_tpadmin_priv] = ACTIONS(1777),
    [sym_user1_priv] = ACTIONS(1777),
    [sym_user2_priv] = ACTIONS(1777),
    [sym_user3_priv] = ACTIONS(1777),
    [sym_user4_priv] = ACTIONS(1777),
    [sym_usermanagement_priv] = ACTIONS(1777),
    [sym_valid_account] = ACTIONS(1777),
    [sym_err_name_invalid] = ACTIONS(1777),
    [sym_err_password_invalid] = ACTIONS(1777),
    [sym_err_invalid_account] = ACTIONS(1777),
    [sym_err_no_authorization] = ACTIONS(1777),
    [sym_err_invalid_parameter] = ACTIONS(1777),
    [sym_err_user_locked_out] = ACTIONS(1777),
    [sym_err_user_account_expired] = ACTIONS(1777),
    [sym_err_general_error] = ACTIONS(1777),
    [sym_err_authentication_error] = ACTIONS(1777),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1777),
    [sym_tls_validate_certificate] = ACTIONS(1777),
    [sym_tdata] = ACTIONS(1777),
    [sym_tchannel] = ACTIONS(1777),
    [sym_tlevel] = ACTIONS(1777),
    [sym_tbutton] = ACTIONS(1777),
    [sym_ttimeline] = ACTIONS(1777),
    [sym_tcustom] = ACTIONS(1777),
    [sym_url_struct] = ACTIONS(1777),
    [sym_dns_struct] = ACTIONS(1777),
    [sym_ip_address_struct] = ACTIONS(1777),
    [sym_dev_info_struct] = ACTIONS(1777),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1777),
    [sym_data_feed] = ACTIONS(1777),
    [sym_data_field] = ACTIONS(1777),
    [sym_data_record] = ACTIONS(1777),
    [sym_wc_data_feed] = ACTIONS(1777),
    [sym_wc_data_field] = ACTIONS(1777),
    [sym_wc_data_record] = ACTIONS(1777),
    [sym_last_login_info] = ACTIONS(1777),
    [sym____reserved___] = ACTIONS(1777),
    [sym_date] = ACTIONS(1777),
    [sym_day] = ACTIONS(1777),
    [sym_time] = ACTIONS(1777),
    [sym_ldate] = ACTIONS(1777),
    [sym_push_device] = ACTIONS(1777),
    [sym_push_channel] = ACTIONS(1777),
    [sym_push_devchan] = ACTIONS(1777),
    [sym_release_device] = ACTIONS(1777),
    [sym_release_channel] = ACTIONS(1777),
    [sym_release_devchan] = ACTIONS(1777),
    [sym_master_slot] = ACTIONS(1777),
    [sym_get_pulse_time] = ACTIONS(1777),
    [sym_get_timer] = ACTIONS(1777),
    [sym_master_sn] = ACTIONS(1777),
    [sym_system_number] = ACTIONS(1777),
    [sym_button] = ACTIONS(1777),
    [sym_dv_channel] = ACTIONS(1777),
    [sym_data] = ACTIONS(1777),
    [sym_level] = ACTIONS(1777),
    [sym_timeline] = ACTIONS(1777),
    [sym_channel] = ACTIONS(1777),
    [sym_custom] = ACTIONS(1777),
    [sym_atoi] = ACTIONS(1777),
    [sym_atol] = ACTIONS(1777),
    [sym_atof] = ACTIONS(1777),
    [sym_itoa] = ACTIONS(1777),
    [sym_itohex] = ACTIONS(1777),
    [sym_hextoi] = ACTIONS(1777),
    [sym_ftoa] = ACTIONS(1777),
    [sym_find_string] = ACTIONS(1777),
    [sym_left_string] = ACTIONS(1777),
    [sym_right_string] = ACTIONS(1777),
    [sym_mid_string] = ACTIONS(1777),
    [sym_remove_string] = ACTIONS(1777),
    [sym_get_buffer_string] = ACTIONS(1777),
    [sym_get_buffer_char] = ACTIONS(1777),
    [sym_get_multi_buffer_string] = ACTIONS(1777),
    [sym_length_string] = ACTIONS(1777),
    [sym_max_length_string] = ACTIONS(1777),
    [sym_set_length_string] = ACTIONS(1777),
    [sym_compare_string] = ACTIONS(1777),
    [sym_upper_string] = ACTIONS(1777),
    [sym_lower_string] = ACTIONS(1777),
    [sym_get_last] = ACTIONS(1777),
    [sym_date_to_day] = ACTIONS(1777),
    [sym_date_to_month] = ACTIONS(1777),
    [sym_date_to_year] = ACTIONS(1777),
    [sym_time_to_hour] = ACTIONS(1777),
    [sym_time_to_minute] = ACTIONS(1777),
    [sym_time_to_second] = ACTIONS(1777),
    [sym_day_of_week] = ACTIONS(1777),
    [sym_set_pulse_time] = ACTIONS(1777),
    [sym_set_timer] = ACTIONS(1777),
    [sym_file_open] = ACTIONS(1777),
    [sym_file_close] = ACTIONS(1777),
    [sym_file_read] = ACTIONS(1777),
    [sym_file_read_line] = ACTIONS(1777),
    [sym_file_write] = ACTIONS(1777),
    [sym_file_write_line] = ACTIONS(1777),
    [sym_file_copy] = ACTIONS(1777),
    [sym_file_delete] = ACTIONS(1777),
    [sym_file_rename] = ACTIONS(1777),
    [sym_file_dir] = ACTIONS(1777),
    [sym_file_getdir] = ACTIONS(1777),
    [sym_file_setdir] = ACTIONS(1777),
    [sym_file_create_dir] = ACTIONS(1777),
    [sym_file_remove_dir] = ACTIONS(1777),
    [sym_file_seek] = ACTIONS(1777),
    [sym_length_array] = ACTIONS(1777),
    [sym_max_length_array] = ACTIONS(1777),
    [sym_set_length_array] = ACTIONS(1777),
    [sym_ip_client_open] = ACTIONS(1777),
    [sym_ip_client_close] = ACTIONS(1777),
    [sym_ip_server_open] = ACTIONS(1777),
    [sym_ip_server_close] = ACTIONS(1777),
    [sym_ip_bound_client_open] = ACTIONS(1777),
    [sym_ip_mc_server_open] = ACTIONS(1777),
    [sym_ip_set_option] = ACTIONS(1777),
    [sym_ssh_client_open] = ACTIONS(1777),
    [sym_ssh_client_close] = ACTIONS(1777),
    [sym_tls_client_open] = ACTIONS(1777),
    [sym_tls_client_close] = ACTIONS(1777),
    [sym_do_push] = ACTIONS(1777),
    [sym_do_push_timed] = ACTIONS(1777),
    [sym_do_release] = ACTIONS(1777),
    [sym_device_id] = ACTIONS(1777),
    [sym_device_info] = ACTIONS(1777),
    [sym_device_id_string] = ACTIONS(1777),
    [sym_device_standby] = ACTIONS(1777),
    [sym_device_wake] = ACTIONS(1777),
    [sym_timeline_create] = ACTIONS(1777),
    [sym_timeline_kill] = ACTIONS(1777),
    [sym_timeline_reload] = ACTIONS(1777),
    [sym_timeline_pause] = ACTIONS(1777),
    [sym_timeline_restart] = ACTIONS(1777),
    [sym_timeline_set] = ACTIONS(1777),
    [sym_timeline_get] = ACTIONS(1777),
    [sym_timeline_active] = ACTIONS(1777),
    [sym_timeline_dynamic_id] = ACTIONS(1777),
    [sym_variable_to_string] = ACTIONS(1777),
    [sym_string_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_string] = ACTIONS(1777),
    [sym_type_cast] = ACTIONS(1777),
    [sym_variable_to_xml] = ACTIONS(1777),
    [sym_xml_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_xml] = ACTIONS(1777),
    [sym_redirect_string] = ACTIONS(1777),
    [sym_random_number] = ACTIONS(1777),
    [sym_get_unique_id] = ACTIONS(1777),
    [sym_get_system_number] = ACTIONS(1777),
    [sym_set_system_number] = ACTIONS(1777),
    [sym_get_serial_number] = ACTIONS(1777),
    [sym_get_master_build] = ACTIONS(1777),
    [sym_set_validation_code] = ACTIONS(1777),
    [sym_reboot] = ACTIONS(1777),
    [sym_load_duet_module] = ACTIONS(1777),
    [sym_unload_duet_module] = ACTIONS(1777),
    [sym_duet_mem_size_set] = ACTIONS(1777),
    [sym_duet_mem_size_get] = ACTIONS(1777),
    [sym_dynamic_application_device] = ACTIONS(1777),
    [sym_dynamic_polled_port] = ACTIONS(1777),
    [sym_static_port_binding] = ACTIONS(1777),
    [sym_static_ip_binding] = ACTIONS(1777),
    [sym_get_dns_list] = ACTIONS(1777),
    [sym_set_dns_list] = ACTIONS(1777),
    [sym_get_ip_address] = ACTIONS(1777),
    [sym_set_ip_address] = ACTIONS(1777),
    [sym_get_url_list] = ACTIONS(1777),
    [sym_add_url_entry] = ACTIONS(1777),
    [sym_delete_url_entry] = ACTIONS(1777),
    [sym_abs_value] = ACTIONS(1777),
    [sym_max_value] = ACTIONS(1777),
    [sym_min_value] = ACTIONS(1777),
    [sym_exp_value] = ACTIONS(1777),
    [sym_log_value] = ACTIONS(1777),
    [sym_log10_value] = ACTIONS(1777),
    [sym_power_value] = ACTIONS(1777),
    [sym_sqrt_value] = ACTIONS(1777),
    [sym_raw_be] = ACTIONS(1777),
    [sym_raw_le] = ACTIONS(1777),
    [sym_internal_queue_size_set] = ACTIONS(1777),
    [sym_internal_queue_size_get] = ACTIONS(1777),
    [sym_internal_threshold_set] = ACTIONS(1777),
    [sym_internal_threshold_get] = ACTIONS(1777),
    [sym_rebuilt_event] = ACTIONS(1777),
    [sym_do_custom_event] = ACTIONS(1777),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1777),
    [sym_clkmgr_set_clk_source] = ACTIONS(1777),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1777),
    [sym_clkmgr_get_timezone] = ACTIONS(1777),
    [sym_clkmgr_set_timezone] = ACTIONS(1777),
    [sym_clkmgr_get_resync_period] = ACTIONS(1777),
    [sym_clkmgr_set_resync_period] = ACTIONS(1777),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_timeservers] = ACTIONS(1777),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_set_log_level] = ACTIONS(1777),
    [sym_get_log_level] = ACTIONS(1777),
    [sym_amx_log] = ACTIONS(1777),
    [sym_smtp_server_config_set] = ACTIONS(1777),
    [sym_smtp_server_config_get] = ACTIONS(1777),
    [sym_smtp_send] = ACTIONS(1777),
    [sym_validate_netlinx_account] = ACTIONS(1777),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1777),
    [sym_authenticate_certificate] = ACTIONS(1777),
    [sym_audit_netlinx_session_event] = ACTIONS(1777),
    [sym_audit_netlinx_generic_event] = ACTIONS(1777),
    [sym_get_available_flash_disk_space] = ACTIONS(1777),
    [sym_get_max_flash_disk_space] = ACTIONS(1777),
    [sym_data_create_feed] = ACTIONS(1777),
    [sym_data_delete_feed] = ACTIONS(1777),
    [sym_data_publish_feed] = ACTIONS(1777),
    [sym_data_get_published_feed] = ACTIONS(1777),
    [sym_data_add_record] = ACTIONS(1777),
    [sym_data_get_event_record] = ACTIONS(1777),
    [sym__wc_data_create_feed] = ACTIONS(1777),
    [sym__wc_data_add_record] = ACTIONS(1777),
    [sym__wc_data_get_event_record] = ACTIONS(1777),
    [sym_format] = ACTIONS(1777),
    [sym_astro_clock] = ACTIONS(1777),
    [sym_set_outdoor_temperature] = ACTIONS(1777),
    [sym_set_virtual_level_count] = ACTIONS(1777),
    [sym_set_virtual_channel_count] = ACTIONS(1777),
    [sym_set_virtual_port_count] = ACTIONS(1777),
    [sym__wc_encode_int] = ACTIONS(1777),
    [sym__wc_decode_int] = ACTIONS(1777),
    [sym__wc_int] = ACTIONS(1777),
    [sym__wc_to_ch_int] = ACTIONS(1777),
    [sym__ch_to_wc_int] = ACTIONS(1777),
    [sym__wc_find_string_int] = ACTIONS(1777),
    [sym__wc_left_string_int] = ACTIONS(1777),
    [sym__wc_lower_string_int] = ACTIONS(1777),
    [sym__wc_mid_string_int] = ACTIONS(1777),
    [sym__wc_remove_string_int] = ACTIONS(1777),
    [sym__wc_right_string_int] = ACTIONS(1777),
    [sym__wc_upper_string_int] = ACTIONS(1777),
    [sym__wc_compare_string_int] = ACTIONS(1777),
    [sym__wc_get_buffer_char_int] = ACTIONS(1777),
    [sym__wc_get_buffer_string_int] = ACTIONS(1777),
    [sym__wc_concat_string_int] = ACTIONS(1777),
    [sym__wc_file_open_int] = ACTIONS(1777),
    [sym__wc_file_close_int] = ACTIONS(1777),
    [sym__wc_file_read_int] = ACTIONS(1777),
    [sym__wc_file_read_line_int] = ACTIONS(1777),
    [sym__wc_file_write_int] = ACTIONS(1777),
    [sym__wc_file_write_line_int] = ACTIONS(1777),
    [aux_sym_preproc_include_token1] = ACTIONS(1777),
    [aux_sym_preproc_define_token1] = ACTIONS(1779),
    [aux_sym_preproc_warn_token1] = ACTIONS(1779),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1779),
    [anon_sym_LBRACK] = ACTIONS(1779),
    [anon_sym_LBRACE] = ACTIONS(1779),
    [anon_sym_RBRACE] = ACTIONS(1779),
    [anon_sym_LPAREN] = ACTIONS(1777),
    [anon_sym_BANG] = ACTIONS(1779),
    [anon_sym_TILDE] = ACTIONS(1779),
    [anon_sym_DASH] = ACTIONS(1777),
    [anon_sym_PLUS] = ACTIONS(1777),
    [anon_sym_DASH_DASH] = ACTIONS(1779),
    [anon_sym_PLUS_PLUS] = ACTIONS(1779),
    [anon_sym_DQUOTE] = ACTIONS(1779),
    [anon_sym_SQUOTE] = ACTIONS(1779),
    [sym_number_literal] = ACTIONS(1779),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(505)] = {
    [sym_identifier] = ACTIONS(1777),
    [sym_define_call_keyword] = ACTIONS(1777),
    [sym_define_function_keyword] = ACTIONS(1777),
    [sym_define_library_function_keyword] = ACTIONS(1777),
    [sym_define_module_keyword] = ACTIONS(1777),
    [sym_char_keyword] = ACTIONS(1777),
    [sym_widechar_keyword] = ACTIONS(1777),
    [sym_integer_keyword] = ACTIONS(1777),
    [sym_sinteger_keyword] = ACTIONS(1777),
    [sym_long_keyword] = ACTIONS(1777),
    [sym_slong_keyword] = ACTIONS(1777),
    [sym_float_keyword] = ACTIONS(1777),
    [sym_double_keyword] = ACTIONS(1777),
    [sym_variant_keyword] = ACTIONS(1777),
    [sym_variantarray_keyword] = ACTIONS(1777),
    [sym_local_var_keyword] = ACTIONS(1777),
    [sym_stack_var_keyword] = ACTIONS(1777),
    [sym_constant_keyword] = ACTIONS(1777),
    [sym_volatile_keyword] = ACTIONS(1777),
    [sym_non_volatile_keyword] = ACTIONS(1777),
    [sym_persistent_keyword] = ACTIONS(1777),
    [sym_dev_keyword] = ACTIONS(1777),
    [sym_devlev_keyword] = ACTIONS(1777),
    [sym_devchan_keyword] = ACTIONS(1777),
    [sym_if_keyword] = ACTIONS(1777),
    [sym_else_keyword] = ACTIONS(1777),
    [sym_switch_keyword] = ACTIONS(1777),
    [sym_case_keyword] = ACTIONS(1777),
    [sym_default_keyword] = ACTIONS(1777),
    [sym_while_keyword] = ACTIONS(1777),
    [sym_for_keyword] = ACTIONS(1777),
    [sym_break_keyword] = ACTIONS(1777),
    [sym_continue_keyword] = ACTIONS(1777),
    [sym_return_keyword] = ACTIONS(1777),
    [sym_select_keyword] = ACTIONS(1777),
    [sym_active_keyword] = ACTIONS(1777),
    [sym_struct_keyword] = ACTIONS(1777),
    [sym_structure_keyword] = ACTIONS(1777),
    [aux_sym_on_keyword_token1] = ACTIONS(1777),
    [aux_sym_off_keyword_token1] = ACTIONS(1777),
    [sym_send_level_keyword] = ACTIONS(1777),
    [sym_send_string_keyword] = ACTIONS(1777),
    [sym_send_command_keyword] = ACTIONS(1777),
    [sym_clear_buffer_keyword] = ACTIONS(1777),
    [sym_create_buffer_keyword] = ACTIONS(1777),
    [sym_create_multi_buffer_keyword] = ACTIONS(1777),
    [sym_call_keyword] = ACTIONS(1777),
    [sym_system_call_keyword] = ACTIONS(1777),
    [sym_devchan_to_keyword] = ACTIONS(1777),
    [sym_devchan_min_to_keyword] = ACTIONS(1777),
    [sym_devchan_total_off_keyword] = ACTIONS(1777),
    [sym_devchan_pulse_keyword] = ACTIONS(1777),
    [sym_wait_keyword] = ACTIONS(1777),
    [sym_cancel_wait_keyword] = ACTIONS(1777),
    [sym_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_wait_until_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_keyword] = ACTIONS(1777),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1777),
    [sym_not] = ACTIONS(1777),
    [sym_bnot] = ACTIONS(1777),
    [sym___file__] = ACTIONS(1777),
    [sym___date__] = ACTIONS(1777),
    [sym___ldate__] = ACTIONS(1777),
    [sym___time__] = ACTIONS(1777),
    [sym___line__] = ACTIONS(1777),
    [sym___name__] = ACTIONS(1777),
    [sym___netlinx__] = ACTIONS(1777),
    [sym_netlinx_axi_version] = ACTIONS(1777),
    [sym_url_flg_tcp] = ACTIONS(1777),
    [sym_url_flg_accinfopresent] = ACTIONS(1777),
    [sym_url_flg_temp] = ACTIONS(1777),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1777),
    [sym_url_flg_stat_mask] = ACTIONS(1777),
    [sym_url_flg_stat_lookup] = ACTIONS(1777),
    [sym_url_flg_stat_connecting] = ACTIONS(1777),
    [sym_url_flg_stat_waiting] = ACTIONS(1777),
    [sym_url_flg_stat_connected] = ACTIONS(1777),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1777),
    [sym_first_virtual_device] = ACTIONS(1777),
    [sym_first_local_port] = ACTIONS(1777),
    [sym_dynamic_virtual_device] = ACTIONS(1777),
    [sym_timeline_once] = ACTIONS(1777),
    [sym_timeline_repeat] = ACTIONS(1777),
    [sym_timeline_absolute] = ACTIONS(1777),
    [sym_timeline_relative] = ACTIONS(1777),
    [sym_do_push_timed_infinite] = ACTIONS(1777),
    [sym_source_type_no_address] = ACTIONS(1777),
    [sym_source_type_neuron_id] = ACTIONS(1777),
    [sym_source_type_ip_address] = ACTIONS(1777),
    [sym_source_type_axlink] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1777),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1777),
    [sym_source_type_ip_socket_address] = ACTIONS(1777),
    [sym_source_type_rs232] = ACTIONS(1777),
    [sym_source_type_internal] = ACTIONS(1777),
    [sym_source_type_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1777),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1777),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1777),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1777),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1777),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1777),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1777),
    [sym_true] = ACTIONS(1777),
    [sym_false] = ACTIONS(1777),
    [sym_file_read_only] = ACTIONS(1777),
    [sym_file_rw_new] = ACTIONS(1777),
    [sym_file_rw_append] = ACTIONS(1777),
    [sym_ip_tcp] = ACTIONS(1777),
    [sym_ip_udp] = ACTIONS(1777),
    [sym_ip_udp_2way] = ACTIONS(1777),
    [sym_xml_encode_types] = ACTIONS(1777),
    [sym_xml_encode_char_as_list] = ACTIONS(1777),
    [sym_xml_encode_le] = ACTIONS(1777),
    [sym_xml_decode_types] = ACTIONS(1777),
    [sym_xml_decode_no_preserve] = ACTIONS(1777),
    [sym_duet_dev_type_display_device] = ACTIONS(1777),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1777),
    [sym_duet_dev_type_io_device] = ACTIONS(1777),
    [sym_duet_dev_type_relay_device] = ACTIONS(1777),
    [sym_duet_dev_type_ups] = ACTIONS(1777),
    [sym_duet_dev_type_amplifier] = ACTIONS(1777),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1777),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1777),
    [sym_duet_dev_type_camera] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1777),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1777),
    [sym_duet_dev_type_dss] = ACTIONS(1777),
    [sym_duet_dev_type_dvr] = ACTIONS(1777),
    [sym_duet_dev_type_disc_device] = ACTIONS(1777),
    [sym_duet_dev_type_document_camera] = ACTIONS(1777),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1777),
    [sym_duet_dev_type_hvac] = ACTIONS(1777),
    [sym_duet_dev_type_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_light] = ACTIONS(1777),
    [sym_duet_dev_type_monitor] = ACTIONS(1777),
    [sym_duet_dev_type_motor] = ACTIONS(1777),
    [sym_duet_dev_type_multi_window] = ACTIONS(1777),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1777),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1777),
    [sym_duet_dev_type_receiver] = ACTIONS(1777),
    [sym_duet_dev_type_security_system] = ACTIONS(1777),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1777),
    [sym_duet_dev_type_settop_box] = ACTIONS(1777),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1777),
    [sym_duet_dev_type_switcher] = ACTIONS(1777),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1777),
    [sym_duet_dev_type_tv] = ACTIONS(1777),
    [sym_duet_dev_type_utility] = ACTIONS(1777),
    [sym_duet_dev_type_vcr] = ACTIONS(1777),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1777),
    [sym_duet_dev_type_video_processor] = ACTIONS(1777),
    [sym_duet_dev_type_video_projector] = ACTIONS(1777),
    [sym_duet_dev_type_video_wall] = ACTIONS(1777),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1777),
    [sym_duet_dev_type_weather] = ACTIONS(1777),
    [sym_duet_dev_not_polled] = ACTIONS(1777),
    [sym_duet_dev_polled] = ACTIONS(1777),
    [sym_clkmgr_mode_network] = ACTIONS(1777),
    [sym_clkmgr_mode_standalone] = ACTIONS(1777),
    [sym_normal_standby] = ACTIONS(1777),
    [sym_normal_wake] = ACTIONS(1777),
    [sym_ip_multicast_ttl_option] = ACTIONS(1777),
    [sym_ip_ttl_subnet] = ACTIONS(1777),
    [sym_ip_ttl_site] = ACTIONS(1777),
    [sym_ip_ttl_region] = ACTIONS(1777),
    [sym_ip_ttl_continent] = ACTIONS(1777),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1777),
    [sym_ip_nodelay_on] = ACTIONS(1777),
    [sym_ip_nodelay_off] = ACTIONS(1777),
    [sym_amx_error] = ACTIONS(1777),
    [sym_amx_warning] = ACTIONS(1777),
    [sym_amx_info] = ACTIONS(1777),
    [sym_amx_debug] = ACTIONS(1777),
    [sym_netlinx_login_success] = ACTIONS(1777),
    [sym_netlinx_login_fail] = ACTIONS(1777),
    [sym_netlinx_logout] = ACTIONS(1777),
    [sym_null_str] = ACTIONS(1777),
    [sym_smtp_address] = ACTIONS(1777),
    [sym_smtp_port_number] = ACTIONS(1777),
    [sym_smtp_username] = ACTIONS(1777),
    [sym_smtp_password] = ACTIONS(1777),
    [sym_smtp_require_tls] = ACTIONS(1777),
    [sym_smtp_from] = ACTIONS(1777),
    [sym_smtp_tls_true] = ACTIONS(1777),
    [sym_smtp_tls_false] = ACTIONS(1777),
    [sym_listview_on_row_select_event] = ACTIONS(1777),
    [sym_data_structure_unknown] = ACTIONS(1777),
    [sym_data_structure_datafeed] = ACTIONS(1777),
    [sym_data_structure_datarecordset] = ACTIONS(1777),
    [sym_data_structure_datarecord] = ACTIONS(1777),
    [sym_data_structure_datafield] = ACTIONS(1777),
    [sym_data_max_content_field_count] = ACTIONS(1777),
    [sym_data_max_metadata_field_count] = ACTIONS(1777),
    [sym_data_max_id_length] = ACTIONS(1777),
    [sym_data_max_name_length] = ACTIONS(1777),
    [sym_data_max_description_length] = ACTIONS(1777),
    [sym_data_max_type_length] = ACTIONS(1777),
    [sym_data_max_format_length] = ACTIONS(1777),
    [sym_data_max_source_length] = ACTIONS(1777),
    [sym_data_max_value_length] = ACTIONS(1777),
    [sym_data_max_label_length] = ACTIONS(1777),
    [sym_data_max_url_length] = ACTIONS(1777),
    [sym_max_last_login_info_length] = ACTIONS(1777),
    [sym_data_type_unknown] = ACTIONS(1777),
    [sym_data_type_string] = ACTIONS(1777),
    [sym_data_type_datetime] = ACTIONS(1777),
    [sym_data_type_date] = ACTIONS(1777),
    [sym_data_type_time] = ACTIONS(1777),
    [sym_data_type_image] = ACTIONS(1777),
    [sym_data_format_url] = ACTIONS(1777),
    [sym_data_format_phone] = ACTIONS(1777),
    [sym_data_format_email] = ACTIONS(1777),
    [sym_data_format_iso8601] = ACTIONS(1777),
    [sym_auditlog_priv] = ACTIONS(1777),
    [sym_configuration_priv] = ACTIONS(1777),
    [sym_deviceconfig_priv] = ACTIONS(1777),
    [sym_ftp_priv] = ACTIONS(1777),
    [sym_http_priv] = ACTIONS(1777),
    [sym_networkconfig_priv] = ACTIONS(1777),
    [sym_programport_priv] = ACTIONS(1777),
    [sym_remoteui_priv] = ACTIONS(1777),
    [sym_securitycontrol_priv] = ACTIONS(1777),
    [sym_softwaremanagement_priv] = ACTIONS(1777),
    [sym_terminal_priv] = ACTIONS(1777),
    [sym_tpadmin_priv] = ACTIONS(1777),
    [sym_user1_priv] = ACTIONS(1777),
    [sym_user2_priv] = ACTIONS(1777),
    [sym_user3_priv] = ACTIONS(1777),
    [sym_user4_priv] = ACTIONS(1777),
    [sym_usermanagement_priv] = ACTIONS(1777),
    [sym_valid_account] = ACTIONS(1777),
    [sym_err_name_invalid] = ACTIONS(1777),
    [sym_err_password_invalid] = ACTIONS(1777),
    [sym_err_invalid_account] = ACTIONS(1777),
    [sym_err_no_authorization] = ACTIONS(1777),
    [sym_err_invalid_parameter] = ACTIONS(1777),
    [sym_err_user_locked_out] = ACTIONS(1777),
    [sym_err_user_account_expired] = ACTIONS(1777),
    [sym_err_general_error] = ACTIONS(1777),
    [sym_err_authentication_error] = ACTIONS(1777),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1777),
    [sym_tls_validate_certificate] = ACTIONS(1777),
    [sym_tdata] = ACTIONS(1777),
    [sym_tchannel] = ACTIONS(1777),
    [sym_tlevel] = ACTIONS(1777),
    [sym_tbutton] = ACTIONS(1777),
    [sym_ttimeline] = ACTIONS(1777),
    [sym_tcustom] = ACTIONS(1777),
    [sym_url_struct] = ACTIONS(1777),
    [sym_dns_struct] = ACTIONS(1777),
    [sym_ip_address_struct] = ACTIONS(1777),
    [sym_dev_info_struct] = ACTIONS(1777),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1777),
    [sym_data_feed] = ACTIONS(1777),
    [sym_data_field] = ACTIONS(1777),
    [sym_data_record] = ACTIONS(1777),
    [sym_wc_data_feed] = ACTIONS(1777),
    [sym_wc_data_field] = ACTIONS(1777),
    [sym_wc_data_record] = ACTIONS(1777),
    [sym_last_login_info] = ACTIONS(1777),
    [sym____reserved___] = ACTIONS(1777),
    [sym_date] = ACTIONS(1777),
    [sym_day] = ACTIONS(1777),
    [sym_time] = ACTIONS(1777),
    [sym_ldate] = ACTIONS(1777),
    [sym_push_device] = ACTIONS(1777),
    [sym_push_channel] = ACTIONS(1777),
    [sym_push_devchan] = ACTIONS(1777),
    [sym_release_device] = ACTIONS(1777),
    [sym_release_channel] = ACTIONS(1777),
    [sym_release_devchan] = ACTIONS(1777),
    [sym_master_slot] = ACTIONS(1777),
    [sym_get_pulse_time] = ACTIONS(1777),
    [sym_get_timer] = ACTIONS(1777),
    [sym_master_sn] = ACTIONS(1777),
    [sym_system_number] = ACTIONS(1777),
    [sym_button] = ACTIONS(1777),
    [sym_dv_channel] = ACTIONS(1777),
    [sym_data] = ACTIONS(1777),
    [sym_level] = ACTIONS(1777),
    [sym_timeline] = ACTIONS(1777),
    [sym_channel] = ACTIONS(1777),
    [sym_custom] = ACTIONS(1777),
    [sym_atoi] = ACTIONS(1777),
    [sym_atol] = ACTIONS(1777),
    [sym_atof] = ACTIONS(1777),
    [sym_itoa] = ACTIONS(1777),
    [sym_itohex] = ACTIONS(1777),
    [sym_hextoi] = ACTIONS(1777),
    [sym_ftoa] = ACTIONS(1777),
    [sym_find_string] = ACTIONS(1777),
    [sym_left_string] = ACTIONS(1777),
    [sym_right_string] = ACTIONS(1777),
    [sym_mid_string] = ACTIONS(1777),
    [sym_remove_string] = ACTIONS(1777),
    [sym_get_buffer_string] = ACTIONS(1777),
    [sym_get_buffer_char] = ACTIONS(1777),
    [sym_get_multi_buffer_string] = ACTIONS(1777),
    [sym_length_string] = ACTIONS(1777),
    [sym_max_length_string] = ACTIONS(1777),
    [sym_set_length_string] = ACTIONS(1777),
    [sym_compare_string] = ACTIONS(1777),
    [sym_upper_string] = ACTIONS(1777),
    [sym_lower_string] = ACTIONS(1777),
    [sym_get_last] = ACTIONS(1777),
    [sym_date_to_day] = ACTIONS(1777),
    [sym_date_to_month] = ACTIONS(1777),
    [sym_date_to_year] = ACTIONS(1777),
    [sym_time_to_hour] = ACTIONS(1777),
    [sym_time_to_minute] = ACTIONS(1777),
    [sym_time_to_second] = ACTIONS(1777),
    [sym_day_of_week] = ACTIONS(1777),
    [sym_set_pulse_time] = ACTIONS(1777),
    [sym_set_timer] = ACTIONS(1777),
    [sym_file_open] = ACTIONS(1777),
    [sym_file_close] = ACTIONS(1777),
    [sym_file_read] = ACTIONS(1777),
    [sym_file_read_line] = ACTIONS(1777),
    [sym_file_write] = ACTIONS(1777),
    [sym_file_write_line] = ACTIONS(1777),
    [sym_file_copy] = ACTIONS(1777),
    [sym_file_delete] = ACTIONS(1777),
    [sym_file_rename] = ACTIONS(1777),
    [sym_file_dir] = ACTIONS(1777),
    [sym_file_getdir] = ACTIONS(1777),
    [sym_file_setdir] = ACTIONS(1777),
    [sym_file_create_dir] = ACTIONS(1777),
    [sym_file_remove_dir] = ACTIONS(1777),
    [sym_file_seek] = ACTIONS(1777),
    [sym_length_array] = ACTIONS(1777),
    [sym_max_length_array] = ACTIONS(1777),
    [sym_set_length_array] = ACTIONS(1777),
    [sym_ip_client_open] = ACTIONS(1777),
    [sym_ip_client_close] = ACTIONS(1777),
    [sym_ip_server_open] = ACTIONS(1777),
    [sym_ip_server_close] = ACTIONS(1777),
    [sym_ip_bound_client_open] = ACTIONS(1777),
    [sym_ip_mc_server_open] = ACTIONS(1777),
    [sym_ip_set_option] = ACTIONS(1777),
    [sym_ssh_client_open] = ACTIONS(1777),
    [sym_ssh_client_close] = ACTIONS(1777),
    [sym_tls_client_open] = ACTIONS(1777),
    [sym_tls_client_close] = ACTIONS(1777),
    [sym_do_push] = ACTIONS(1777),
    [sym_do_push_timed] = ACTIONS(1777),
    [sym_do_release] = ACTIONS(1777),
    [sym_device_id] = ACTIONS(1777),
    [sym_device_info] = ACTIONS(1777),
    [sym_device_id_string] = ACTIONS(1777),
    [sym_device_standby] = ACTIONS(1777),
    [sym_device_wake] = ACTIONS(1777),
    [sym_timeline_create] = ACTIONS(1777),
    [sym_timeline_kill] = ACTIONS(1777),
    [sym_timeline_reload] = ACTIONS(1777),
    [sym_timeline_pause] = ACTIONS(1777),
    [sym_timeline_restart] = ACTIONS(1777),
    [sym_timeline_set] = ACTIONS(1777),
    [sym_timeline_get] = ACTIONS(1777),
    [sym_timeline_active] = ACTIONS(1777),
    [sym_timeline_dynamic_id] = ACTIONS(1777),
    [sym_variable_to_string] = ACTIONS(1777),
    [sym_string_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_string] = ACTIONS(1777),
    [sym_type_cast] = ACTIONS(1777),
    [sym_variable_to_xml] = ACTIONS(1777),
    [sym_xml_to_variable] = ACTIONS(1777),
    [sym_length_variable_to_xml] = ACTIONS(1777),
    [sym_redirect_string] = ACTIONS(1777),
    [sym_random_number] = ACTIONS(1777),
    [sym_get_unique_id] = ACTIONS(1777),
    [sym_get_system_number] = ACTIONS(1777),
    [sym_set_system_number] = ACTIONS(1777),
    [sym_get_serial_number] = ACTIONS(1777),
    [sym_get_master_build] = ACTIONS(1777),
    [sym_set_validation_code] = ACTIONS(1777),
    [sym_reboot] = ACTIONS(1777),
    [sym_load_duet_module] = ACTIONS(1777),
    [sym_unload_duet_module] = ACTIONS(1777),
    [sym_duet_mem_size_set] = ACTIONS(1777),
    [sym_duet_mem_size_get] = ACTIONS(1777),
    [sym_dynamic_application_device] = ACTIONS(1777),
    [sym_dynamic_polled_port] = ACTIONS(1777),
    [sym_static_port_binding] = ACTIONS(1777),
    [sym_static_ip_binding] = ACTIONS(1777),
    [sym_get_dns_list] = ACTIONS(1777),
    [sym_set_dns_list] = ACTIONS(1777),
    [sym_get_ip_address] = ACTIONS(1777),
    [sym_set_ip_address] = ACTIONS(1777),
    [sym_get_url_list] = ACTIONS(1777),
    [sym_add_url_entry] = ACTIONS(1777),
    [sym_delete_url_entry] = ACTIONS(1777),
    [sym_abs_value] = ACTIONS(1777),
    [sym_max_value] = ACTIONS(1777),
    [sym_min_value] = ACTIONS(1777),
    [sym_exp_value] = ACTIONS(1777),
    [sym_log_value] = ACTIONS(1777),
    [sym_log10_value] = ACTIONS(1777),
    [sym_power_value] = ACTIONS(1777),
    [sym_sqrt_value] = ACTIONS(1777),
    [sym_raw_be] = ACTIONS(1777),
    [sym_raw_le] = ACTIONS(1777),
    [sym_internal_queue_size_set] = ACTIONS(1777),
    [sym_internal_queue_size_get] = ACTIONS(1777),
    [sym_internal_threshold_set] = ACTIONS(1777),
    [sym_internal_threshold_get] = ACTIONS(1777),
    [sym_rebuilt_event] = ACTIONS(1777),
    [sym_do_custom_event] = ACTIONS(1777),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1777),
    [sym_clkmgr_set_clk_source] = ACTIONS(1777),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1777),
    [sym_clkmgr_get_timezone] = ACTIONS(1777),
    [sym_clkmgr_set_timezone] = ACTIONS(1777),
    [sym_clkmgr_get_resync_period] = ACTIONS(1777),
    [sym_clkmgr_set_resync_period] = ACTIONS(1777),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1777),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_timeservers] = ACTIONS(1777),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1777),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1777),
    [sym_set_log_level] = ACTIONS(1777),
    [sym_get_log_level] = ACTIONS(1777),
    [sym_amx_log] = ACTIONS(1777),
    [sym_smtp_server_config_set] = ACTIONS(1777),
    [sym_smtp_server_config_get] = ACTIONS(1777),
    [sym_smtp_send] = ACTIONS(1777),
    [sym_validate_netlinx_account] = ACTIONS(1777),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1777),
    [sym_authenticate_certificate] = ACTIONS(1777),
    [sym_audit_netlinx_session_event] = ACTIONS(1777),
    [sym_audit_netlinx_generic_event] = ACTIONS(1777),
    [sym_get_available_flash_disk_space] = ACTIONS(1777),
    [sym_get_max_flash_disk_space] = ACTIONS(1777),
    [sym_data_create_feed] = ACTIONS(1777),
    [sym_data_delete_feed] = ACTIONS(1777),
    [sym_data_publish_feed] = ACTIONS(1777),
    [sym_data_get_published_feed] = ACTIONS(1777),
    [sym_data_add_record] = ACTIONS(1777),
    [sym_data_get_event_record] = ACTIONS(1777),
    [sym__wc_data_create_feed] = ACTIONS(1777),
    [sym__wc_data_add_record] = ACTIONS(1777),
    [sym__wc_data_get_event_record] = ACTIONS(1777),
    [sym_format] = ACTIONS(1777),
    [sym_astro_clock] = ACTIONS(1777),
    [sym_set_outdoor_temperature] = ACTIONS(1777),
    [sym_set_virtual_level_count] = ACTIONS(1777),
    [sym_set_virtual_channel_count] = ACTIONS(1777),
    [sym_set_virtual_port_count] = ACTIONS(1777),
    [sym__wc_encode_int] = ACTIONS(1777),
    [sym__wc_decode_int] = ACTIONS(1777),
    [sym__wc_int] = ACTIONS(1777),
    [sym__wc_to_ch_int] = ACTIONS(1777),
    [sym__ch_to_wc_int] = ACTIONS(1777),
    [sym__wc_find_string_int] = ACTIONS(1777),
    [sym__wc_left_string_int] = ACTIONS(1777),
    [sym__wc_lower_string_int] = ACTIONS(1777),
    [sym__wc_mid_string_int] = ACTIONS(1777),
    [sym__wc_remove_string_int] = ACTIONS(1777),
    [sym__wc_right_string_int] = ACTIONS(1777),
    [sym__wc_upper_string_int] = ACTIONS(1777),
    [sym__wc_compare_string_int] = ACTIONS(1777),
    [sym__wc_get_buffer_char_int] = ACTIONS(1777),
    [sym__wc_get_buffer_string_int] = ACTIONS(1777),
    [sym__wc_concat_string_int] = ACTIONS(1777),
    [sym__wc_file_open_int] = ACTIONS(1777),
    [sym__wc_file_close_int] = ACTIONS(1777),
    [sym__wc_file_read_int] = ACTIONS(1777),
    [sym__wc_file_read_line_int] = ACTIONS(1777),
    [sym__wc_file_write_int] = ACTIONS(1777),
    [sym__wc_file_write_line_int] = ACTIONS(1777),
    [aux_sym_preproc_include_token1] = ACTIONS(1777),
    [aux_sym_preproc_define_token1] = ACTIONS(1779),
    [aux_sym_preproc_warn_token1] = ACTIONS(1779),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1779),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1779),
    [anon_sym_LBRACK] = ACTIONS(1779),
    [anon_sym_LBRACE] = ACTIONS(1779),
    [anon_sym_RBRACE] = ACTIONS(1779),
    [anon_sym_LPAREN] = ACTIONS(1777),
    [anon_sym_BANG] = ACTIONS(1779),
    [anon_sym_TILDE] = ACTIONS(1779),
    [anon_sym_DASH] = ACTIONS(1777),
    [anon_sym_PLUS] = ACTIONS(1777),
    [anon_sym_DASH_DASH] = ACTIONS(1779),
    [anon_sym_PLUS_PLUS] = ACTIONS(1779),
    [anon_sym_DQUOTE] = ACTIONS(1779),
    [anon_sym_SQUOTE] = ACTIONS(1779),
    [sym_number_literal] = ACTIONS(1779),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(506)] = {
    [sym_identifier] = ACTIONS(1781),
    [sym_define_call_keyword] = ACTIONS(1781),
    [sym_define_function_keyword] = ACTIONS(1781),
    [sym_define_library_function_keyword] = ACTIONS(1781),
    [sym_define_module_keyword] = ACTIONS(1781),
    [sym_char_keyword] = ACTIONS(1781),
    [sym_widechar_keyword] = ACTIONS(1781),
    [sym_integer_keyword] = ACTIONS(1781),
    [sym_sinteger_keyword] = ACTIONS(1781),
    [sym_long_keyword] = ACTIONS(1781),
    [sym_slong_keyword] = ACTIONS(1781),
    [sym_float_keyword] = ACTIONS(1781),
    [sym_double_keyword] = ACTIONS(1781),
    [sym_variant_keyword] = ACTIONS(1781),
    [sym_variantarray_keyword] = ACTIONS(1781),
    [sym_local_var_keyword] = ACTIONS(1781),
    [sym_stack_var_keyword] = ACTIONS(1781),
    [sym_constant_keyword] = ACTIONS(1781),
    [sym_volatile_keyword] = ACTIONS(1781),
    [sym_non_volatile_keyword] = ACTIONS(1781),
    [sym_persistent_keyword] = ACTIONS(1781),
    [sym_dev_keyword] = ACTIONS(1781),
    [sym_devlev_keyword] = ACTIONS(1781),
    [sym_devchan_keyword] = ACTIONS(1781),
    [sym_if_keyword] = ACTIONS(1781),
    [sym_else_keyword] = ACTIONS(1781),
    [sym_switch_keyword] = ACTIONS(1781),
    [sym_case_keyword] = ACTIONS(1781),
    [sym_default_keyword] = ACTIONS(1781),
    [sym_while_keyword] = ACTIONS(1781),
    [sym_for_keyword] = ACTIONS(1781),
    [sym_break_keyword] = ACTIONS(1781),
    [sym_continue_keyword] = ACTIONS(1781),
    [sym_return_keyword] = ACTIONS(1781),
    [sym_select_keyword] = ACTIONS(1781),
    [sym_active_keyword] = ACTIONS(1781),
    [sym_struct_keyword] = ACTIONS(1781),
    [sym_structure_keyword] = ACTIONS(1781),
    [aux_sym_on_keyword_token1] = ACTIONS(1781),
    [aux_sym_off_keyword_token1] = ACTIONS(1781),
    [sym_send_level_keyword] = ACTIONS(1781),
    [sym_send_string_keyword] = ACTIONS(1781),
    [sym_send_command_keyword] = ACTIONS(1781),
    [sym_clear_buffer_keyword] = ACTIONS(1781),
    [sym_create_buffer_keyword] = ACTIONS(1781),
    [sym_create_multi_buffer_keyword] = ACTIONS(1781),
    [sym_call_keyword] = ACTIONS(1781),
    [sym_system_call_keyword] = ACTIONS(1781),
    [sym_devchan_to_keyword] = ACTIONS(1781),
    [sym_devchan_min_to_keyword] = ACTIONS(1781),
    [sym_devchan_total_off_keyword] = ACTIONS(1781),
    [sym_devchan_pulse_keyword] = ACTIONS(1781),
    [sym_wait_keyword] = ACTIONS(1781),
    [sym_cancel_wait_keyword] = ACTIONS(1781),
    [sym_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1781),
    [sym_not] = ACTIONS(1781),
    [sym_bnot] = ACTIONS(1781),
    [sym___file__] = ACTIONS(1781),
    [sym___date__] = ACTIONS(1781),
    [sym___ldate__] = ACTIONS(1781),
    [sym___time__] = ACTIONS(1781),
    [sym___line__] = ACTIONS(1781),
    [sym___name__] = ACTIONS(1781),
    [sym___netlinx__] = ACTIONS(1781),
    [sym_netlinx_axi_version] = ACTIONS(1781),
    [sym_url_flg_tcp] = ACTIONS(1781),
    [sym_url_flg_accinfopresent] = ACTIONS(1781),
    [sym_url_flg_temp] = ACTIONS(1781),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1781),
    [sym_url_flg_stat_mask] = ACTIONS(1781),
    [sym_url_flg_stat_lookup] = ACTIONS(1781),
    [sym_url_flg_stat_connecting] = ACTIONS(1781),
    [sym_url_flg_stat_waiting] = ACTIONS(1781),
    [sym_url_flg_stat_connected] = ACTIONS(1781),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1781),
    [sym_first_virtual_device] = ACTIONS(1781),
    [sym_first_local_port] = ACTIONS(1781),
    [sym_dynamic_virtual_device] = ACTIONS(1781),
    [sym_timeline_once] = ACTIONS(1781),
    [sym_timeline_repeat] = ACTIONS(1781),
    [sym_timeline_absolute] = ACTIONS(1781),
    [sym_timeline_relative] = ACTIONS(1781),
    [sym_do_push_timed_infinite] = ACTIONS(1781),
    [sym_source_type_no_address] = ACTIONS(1781),
    [sym_source_type_neuron_id] = ACTIONS(1781),
    [sym_source_type_ip_address] = ACTIONS(1781),
    [sym_source_type_axlink] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1781),
    [sym_source_type_ip_socket_address] = ACTIONS(1781),
    [sym_source_type_rs232] = ACTIONS(1781),
    [sym_source_type_internal] = ACTIONS(1781),
    [sym_source_type_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1781),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1781),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1781),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1781),
    [sym_true] = ACTIONS(1781),
    [sym_false] = ACTIONS(1781),
    [sym_file_read_only] = ACTIONS(1781),
    [sym_file_rw_new] = ACTIONS(1781),
    [sym_file_rw_append] = ACTIONS(1781),
    [sym_ip_tcp] = ACTIONS(1781),
    [sym_ip_udp] = ACTIONS(1781),
    [sym_ip_udp_2way] = ACTIONS(1781),
    [sym_xml_encode_types] = ACTIONS(1781),
    [sym_xml_encode_char_as_list] = ACTIONS(1781),
    [sym_xml_encode_le] = ACTIONS(1781),
    [sym_xml_decode_types] = ACTIONS(1781),
    [sym_xml_decode_no_preserve] = ACTIONS(1781),
    [sym_duet_dev_type_display_device] = ACTIONS(1781),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_io_device] = ACTIONS(1781),
    [sym_duet_dev_type_relay_device] = ACTIONS(1781),
    [sym_duet_dev_type_ups] = ACTIONS(1781),
    [sym_duet_dev_type_amplifier] = ACTIONS(1781),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1781),
    [sym_duet_dev_type_camera] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1781),
    [sym_duet_dev_type_dss] = ACTIONS(1781),
    [sym_duet_dev_type_dvr] = ACTIONS(1781),
    [sym_duet_dev_type_disc_device] = ACTIONS(1781),
    [sym_duet_dev_type_document_camera] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1781),
    [sym_duet_dev_type_hvac] = ACTIONS(1781),
    [sym_duet_dev_type_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_light] = ACTIONS(1781),
    [sym_duet_dev_type_monitor] = ACTIONS(1781),
    [sym_duet_dev_type_motor] = ACTIONS(1781),
    [sym_duet_dev_type_multi_window] = ACTIONS(1781),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1781),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1781),
    [sym_duet_dev_type_receiver] = ACTIONS(1781),
    [sym_duet_dev_type_security_system] = ACTIONS(1781),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1781),
    [sym_duet_dev_type_settop_box] = ACTIONS(1781),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1781),
    [sym_duet_dev_type_switcher] = ACTIONS(1781),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_tv] = ACTIONS(1781),
    [sym_duet_dev_type_utility] = ACTIONS(1781),
    [sym_duet_dev_type_vcr] = ACTIONS(1781),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_video_processor] = ACTIONS(1781),
    [sym_duet_dev_type_video_projector] = ACTIONS(1781),
    [sym_duet_dev_type_video_wall] = ACTIONS(1781),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1781),
    [sym_duet_dev_type_weather] = ACTIONS(1781),
    [sym_duet_dev_not_polled] = ACTIONS(1781),
    [sym_duet_dev_polled] = ACTIONS(1781),
    [sym_clkmgr_mode_network] = ACTIONS(1781),
    [sym_clkmgr_mode_standalone] = ACTIONS(1781),
    [sym_normal_standby] = ACTIONS(1781),
    [sym_normal_wake] = ACTIONS(1781),
    [sym_ip_multicast_ttl_option] = ACTIONS(1781),
    [sym_ip_ttl_subnet] = ACTIONS(1781),
    [sym_ip_ttl_site] = ACTIONS(1781),
    [sym_ip_ttl_region] = ACTIONS(1781),
    [sym_ip_ttl_continent] = ACTIONS(1781),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1781),
    [sym_ip_nodelay_on] = ACTIONS(1781),
    [sym_ip_nodelay_off] = ACTIONS(1781),
    [sym_amx_error] = ACTIONS(1781),
    [sym_amx_warning] = ACTIONS(1781),
    [sym_amx_info] = ACTIONS(1781),
    [sym_amx_debug] = ACTIONS(1781),
    [sym_netlinx_login_success] = ACTIONS(1781),
    [sym_netlinx_login_fail] = ACTIONS(1781),
    [sym_netlinx_logout] = ACTIONS(1781),
    [sym_null_str] = ACTIONS(1781),
    [sym_smtp_address] = ACTIONS(1781),
    [sym_smtp_port_number] = ACTIONS(1781),
    [sym_smtp_username] = ACTIONS(1781),
    [sym_smtp_password] = ACTIONS(1781),
    [sym_smtp_require_tls] = ACTIONS(1781),
    [sym_smtp_from] = ACTIONS(1781),
    [sym_smtp_tls_true] = ACTIONS(1781),
    [sym_smtp_tls_false] = ACTIONS(1781),
    [sym_listview_on_row_select_event] = ACTIONS(1781),
    [sym_data_structure_unknown] = ACTIONS(1781),
    [sym_data_structure_datafeed] = ACTIONS(1781),
    [sym_data_structure_datarecordset] = ACTIONS(1781),
    [sym_data_structure_datarecord] = ACTIONS(1781),
    [sym_data_structure_datafield] = ACTIONS(1781),
    [sym_data_max_content_field_count] = ACTIONS(1781),
    [sym_data_max_metadata_field_count] = ACTIONS(1781),
    [sym_data_max_id_length] = ACTIONS(1781),
    [sym_data_max_name_length] = ACTIONS(1781),
    [sym_data_max_description_length] = ACTIONS(1781),
    [sym_data_max_type_length] = ACTIONS(1781),
    [sym_data_max_format_length] = ACTIONS(1781),
    [sym_data_max_source_length] = ACTIONS(1781),
    [sym_data_max_value_length] = ACTIONS(1781),
    [sym_data_max_label_length] = ACTIONS(1781),
    [sym_data_max_url_length] = ACTIONS(1781),
    [sym_max_last_login_info_length] = ACTIONS(1781),
    [sym_data_type_unknown] = ACTIONS(1781),
    [sym_data_type_string] = ACTIONS(1781),
    [sym_data_type_datetime] = ACTIONS(1781),
    [sym_data_type_date] = ACTIONS(1781),
    [sym_data_type_time] = ACTIONS(1781),
    [sym_data_type_image] = ACTIONS(1781),
    [sym_data_format_url] = ACTIONS(1781),
    [sym_data_format_phone] = ACTIONS(1781),
    [sym_data_format_email] = ACTIONS(1781),
    [sym_data_format_iso8601] = ACTIONS(1781),
    [sym_auditlog_priv] = ACTIONS(1781),
    [sym_configuration_priv] = ACTIONS(1781),
    [sym_deviceconfig_priv] = ACTIONS(1781),
    [sym_ftp_priv] = ACTIONS(1781),
    [sym_http_priv] = ACTIONS(1781),
    [sym_networkconfig_priv] = ACTIONS(1781),
    [sym_programport_priv] = ACTIONS(1781),
    [sym_remoteui_priv] = ACTIONS(1781),
    [sym_securitycontrol_priv] = ACTIONS(1781),
    [sym_softwaremanagement_priv] = ACTIONS(1781),
    [sym_terminal_priv] = ACTIONS(1781),
    [sym_tpadmin_priv] = ACTIONS(1781),
    [sym_user1_priv] = ACTIONS(1781),
    [sym_user2_priv] = ACTIONS(1781),
    [sym_user3_priv] = ACTIONS(1781),
    [sym_user4_priv] = ACTIONS(1781),
    [sym_usermanagement_priv] = ACTIONS(1781),
    [sym_valid_account] = ACTIONS(1781),
    [sym_err_name_invalid] = ACTIONS(1781),
    [sym_err_password_invalid] = ACTIONS(1781),
    [sym_err_invalid_account] = ACTIONS(1781),
    [sym_err_no_authorization] = ACTIONS(1781),
    [sym_err_invalid_parameter] = ACTIONS(1781),
    [sym_err_user_locked_out] = ACTIONS(1781),
    [sym_err_user_account_expired] = ACTIONS(1781),
    [sym_err_general_error] = ACTIONS(1781),
    [sym_err_authentication_error] = ACTIONS(1781),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1781),
    [sym_tls_validate_certificate] = ACTIONS(1781),
    [sym_tdata] = ACTIONS(1781),
    [sym_tchannel] = ACTIONS(1781),
    [sym_tlevel] = ACTIONS(1781),
    [sym_tbutton] = ACTIONS(1781),
    [sym_ttimeline] = ACTIONS(1781),
    [sym_tcustom] = ACTIONS(1781),
    [sym_url_struct] = ACTIONS(1781),
    [sym_dns_struct] = ACTIONS(1781),
    [sym_ip_address_struct] = ACTIONS(1781),
    [sym_dev_info_struct] = ACTIONS(1781),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1781),
    [sym_data_feed] = ACTIONS(1781),
    [sym_data_field] = ACTIONS(1781),
    [sym_data_record] = ACTIONS(1781),
    [sym_wc_data_feed] = ACTIONS(1781),
    [sym_wc_data_field] = ACTIONS(1781),
    [sym_wc_data_record] = ACTIONS(1781),
    [sym_last_login_info] = ACTIONS(1781),
    [sym____reserved___] = ACTIONS(1781),
    [sym_date] = ACTIONS(1781),
    [sym_day] = ACTIONS(1781),
    [sym_time] = ACTIONS(1781),
    [sym_ldate] = ACTIONS(1781),
    [sym_push_device] = ACTIONS(1781),
    [sym_push_channel] = ACTIONS(1781),
    [sym_push_devchan] = ACTIONS(1781),
    [sym_release_device] = ACTIONS(1781),
    [sym_release_channel] = ACTIONS(1781),
    [sym_release_devchan] = ACTIONS(1781),
    [sym_master_slot] = ACTIONS(1781),
    [sym_get_pulse_time] = ACTIONS(1781),
    [sym_get_timer] = ACTIONS(1781),
    [sym_master_sn] = ACTIONS(1781),
    [sym_system_number] = ACTIONS(1781),
    [sym_button] = ACTIONS(1781),
    [sym_dv_channel] = ACTIONS(1781),
    [sym_data] = ACTIONS(1781),
    [sym_level] = ACTIONS(1781),
    [sym_timeline] = ACTIONS(1781),
    [sym_channel] = ACTIONS(1781),
    [sym_custom] = ACTIONS(1781),
    [sym_atoi] = ACTIONS(1781),
    [sym_atol] = ACTIONS(1781),
    [sym_atof] = ACTIONS(1781),
    [sym_itoa] = ACTIONS(1781),
    [sym_itohex] = ACTIONS(1781),
    [sym_hextoi] = ACTIONS(1781),
    [sym_ftoa] = ACTIONS(1781),
    [sym_find_string] = ACTIONS(1781),
    [sym_left_string] = ACTIONS(1781),
    [sym_right_string] = ACTIONS(1781),
    [sym_mid_string] = ACTIONS(1781),
    [sym_remove_string] = ACTIONS(1781),
    [sym_get_buffer_string] = ACTIONS(1781),
    [sym_get_buffer_char] = ACTIONS(1781),
    [sym_get_multi_buffer_string] = ACTIONS(1781),
    [sym_length_string] = ACTIONS(1781),
    [sym_max_length_string] = ACTIONS(1781),
    [sym_set_length_string] = ACTIONS(1781),
    [sym_compare_string] = ACTIONS(1781),
    [sym_upper_string] = ACTIONS(1781),
    [sym_lower_string] = ACTIONS(1781),
    [sym_get_last] = ACTIONS(1781),
    [sym_date_to_day] = ACTIONS(1781),
    [sym_date_to_month] = ACTIONS(1781),
    [sym_date_to_year] = ACTIONS(1781),
    [sym_time_to_hour] = ACTIONS(1781),
    [sym_time_to_minute] = ACTIONS(1781),
    [sym_time_to_second] = ACTIONS(1781),
    [sym_day_of_week] = ACTIONS(1781),
    [sym_set_pulse_time] = ACTIONS(1781),
    [sym_set_timer] = ACTIONS(1781),
    [sym_file_open] = ACTIONS(1781),
    [sym_file_close] = ACTIONS(1781),
    [sym_file_read] = ACTIONS(1781),
    [sym_file_read_line] = ACTIONS(1781),
    [sym_file_write] = ACTIONS(1781),
    [sym_file_write_line] = ACTIONS(1781),
    [sym_file_copy] = ACTIONS(1781),
    [sym_file_delete] = ACTIONS(1781),
    [sym_file_rename] = ACTIONS(1781),
    [sym_file_dir] = ACTIONS(1781),
    [sym_file_getdir] = ACTIONS(1781),
    [sym_file_setdir] = ACTIONS(1781),
    [sym_file_create_dir] = ACTIONS(1781),
    [sym_file_remove_dir] = ACTIONS(1781),
    [sym_file_seek] = ACTIONS(1781),
    [sym_length_array] = ACTIONS(1781),
    [sym_max_length_array] = ACTIONS(1781),
    [sym_set_length_array] = ACTIONS(1781),
    [sym_ip_client_open] = ACTIONS(1781),
    [sym_ip_client_close] = ACTIONS(1781),
    [sym_ip_server_open] = ACTIONS(1781),
    [sym_ip_server_close] = ACTIONS(1781),
    [sym_ip_bound_client_open] = ACTIONS(1781),
    [sym_ip_mc_server_open] = ACTIONS(1781),
    [sym_ip_set_option] = ACTIONS(1781),
    [sym_ssh_client_open] = ACTIONS(1781),
    [sym_ssh_client_close] = ACTIONS(1781),
    [sym_tls_client_open] = ACTIONS(1781),
    [sym_tls_client_close] = ACTIONS(1781),
    [sym_do_push] = ACTIONS(1781),
    [sym_do_push_timed] = ACTIONS(1781),
    [sym_do_release] = ACTIONS(1781),
    [sym_device_id] = ACTIONS(1781),
    [sym_device_info] = ACTIONS(1781),
    [sym_device_id_string] = ACTIONS(1781),
    [sym_device_standby] = ACTIONS(1781),
    [sym_device_wake] = ACTIONS(1781),
    [sym_timeline_create] = ACTIONS(1781),
    [sym_timeline_kill] = ACTIONS(1781),
    [sym_timeline_reload] = ACTIONS(1781),
    [sym_timeline_pause] = ACTIONS(1781),
    [sym_timeline_restart] = ACTIONS(1781),
    [sym_timeline_set] = ACTIONS(1781),
    [sym_timeline_get] = ACTIONS(1781),
    [sym_timeline_active] = ACTIONS(1781),
    [sym_timeline_dynamic_id] = ACTIONS(1781),
    [sym_variable_to_string] = ACTIONS(1781),
    [sym_string_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_string] = ACTIONS(1781),
    [sym_type_cast] = ACTIONS(1781),
    [sym_variable_to_xml] = ACTIONS(1781),
    [sym_xml_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_xml] = ACTIONS(1781),
    [sym_redirect_string] = ACTIONS(1781),
    [sym_random_number] = ACTIONS(1781),
    [sym_get_unique_id] = ACTIONS(1781),
    [sym_get_system_number] = ACTIONS(1781),
    [sym_set_system_number] = ACTIONS(1781),
    [sym_get_serial_number] = ACTIONS(1781),
    [sym_get_master_build] = ACTIONS(1781),
    [sym_set_validation_code] = ACTIONS(1781),
    [sym_reboot] = ACTIONS(1781),
    [sym_load_duet_module] = ACTIONS(1781),
    [sym_unload_duet_module] = ACTIONS(1781),
    [sym_duet_mem_size_set] = ACTIONS(1781),
    [sym_duet_mem_size_get] = ACTIONS(1781),
    [sym_dynamic_application_device] = ACTIONS(1781),
    [sym_dynamic_polled_port] = ACTIONS(1781),
    [sym_static_port_binding] = ACTIONS(1781),
    [sym_static_ip_binding] = ACTIONS(1781),
    [sym_get_dns_list] = ACTIONS(1781),
    [sym_set_dns_list] = ACTIONS(1781),
    [sym_get_ip_address] = ACTIONS(1781),
    [sym_set_ip_address] = ACTIONS(1781),
    [sym_get_url_list] = ACTIONS(1781),
    [sym_add_url_entry] = ACTIONS(1781),
    [sym_delete_url_entry] = ACTIONS(1781),
    [sym_abs_value] = ACTIONS(1781),
    [sym_max_value] = ACTIONS(1781),
    [sym_min_value] = ACTIONS(1781),
    [sym_exp_value] = ACTIONS(1781),
    [sym_log_value] = ACTIONS(1781),
    [sym_log10_value] = ACTIONS(1781),
    [sym_power_value] = ACTIONS(1781),
    [sym_sqrt_value] = ACTIONS(1781),
    [sym_raw_be] = ACTIONS(1781),
    [sym_raw_le] = ACTIONS(1781),
    [sym_internal_queue_size_set] = ACTIONS(1781),
    [sym_internal_queue_size_get] = ACTIONS(1781),
    [sym_internal_threshold_set] = ACTIONS(1781),
    [sym_internal_threshold_get] = ACTIONS(1781),
    [sym_rebuilt_event] = ACTIONS(1781),
    [sym_do_custom_event] = ACTIONS(1781),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1781),
    [sym_clkmgr_set_clk_source] = ACTIONS(1781),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1781),
    [sym_clkmgr_get_timezone] = ACTIONS(1781),
    [sym_clkmgr_set_timezone] = ACTIONS(1781),
    [sym_clkmgr_get_resync_period] = ACTIONS(1781),
    [sym_clkmgr_set_resync_period] = ACTIONS(1781),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_timeservers] = ACTIONS(1781),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_set_log_level] = ACTIONS(1781),
    [sym_get_log_level] = ACTIONS(1781),
    [sym_amx_log] = ACTIONS(1781),
    [sym_smtp_server_config_set] = ACTIONS(1781),
    [sym_smtp_server_config_get] = ACTIONS(1781),
    [sym_smtp_send] = ACTIONS(1781),
    [sym_validate_netlinx_account] = ACTIONS(1781),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1781),
    [sym_authenticate_certificate] = ACTIONS(1781),
    [sym_audit_netlinx_session_event] = ACTIONS(1781),
    [sym_audit_netlinx_generic_event] = ACTIONS(1781),
    [sym_get_available_flash_disk_space] = ACTIONS(1781),
    [sym_get_max_flash_disk_space] = ACTIONS(1781),
    [sym_data_create_feed] = ACTIONS(1781),
    [sym_data_delete_feed] = ACTIONS(1781),
    [sym_data_publish_feed] = ACTIONS(1781),
    [sym_data_get_published_feed] = ACTIONS(1781),
    [sym_data_add_record] = ACTIONS(1781),
    [sym_data_get_event_record] = ACTIONS(1781),
    [sym__wc_data_create_feed] = ACTIONS(1781),
    [sym__wc_data_add_record] = ACTIONS(1781),
    [sym__wc_data_get_event_record] = ACTIONS(1781),
    [sym_format] = ACTIONS(1781),
    [sym_astro_clock] = ACTIONS(1781),
    [sym_set_outdoor_temperature] = ACTIONS(1781),
    [sym_set_virtual_level_count] = ACTIONS(1781),
    [sym_set_virtual_channel_count] = ACTIONS(1781),
    [sym_set_virtual_port_count] = ACTIONS(1781),
    [sym__wc_encode_int] = ACTIONS(1781),
    [sym__wc_decode_int] = ACTIONS(1781),
    [sym__wc_int] = ACTIONS(1781),
    [sym__wc_to_ch_int] = ACTIONS(1781),
    [sym__ch_to_wc_int] = ACTIONS(1781),
    [sym__wc_find_string_int] = ACTIONS(1781),
    [sym__wc_left_string_int] = ACTIONS(1781),
    [sym__wc_lower_string_int] = ACTIONS(1781),
    [sym__wc_mid_string_int] = ACTIONS(1781),
    [sym__wc_remove_string_int] = ACTIONS(1781),
    [sym__wc_right_string_int] = ACTIONS(1781),
    [sym__wc_upper_string_int] = ACTIONS(1781),
    [sym__wc_compare_string_int] = ACTIONS(1781),
    [sym__wc_get_buffer_char_int] = ACTIONS(1781),
    [sym__wc_get_buffer_string_int] = ACTIONS(1781),
    [sym__wc_concat_string_int] = ACTIONS(1781),
    [sym__wc_file_open_int] = ACTIONS(1781),
    [sym__wc_file_close_int] = ACTIONS(1781),
    [sym__wc_file_read_int] = ACTIONS(1781),
    [sym__wc_file_read_line_int] = ACTIONS(1781),
    [sym__wc_file_write_int] = ACTIONS(1781),
    [sym__wc_file_write_line_int] = ACTIONS(1781),
    [aux_sym_preproc_include_token1] = ACTIONS(1781),
    [aux_sym_preproc_define_token1] = ACTIONS(1783),
    [aux_sym_preproc_warn_token1] = ACTIONS(1783),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1783),
    [anon_sym_LBRACK] = ACTIONS(1783),
    [anon_sym_LBRACE] = ACTIONS(1783),
    [anon_sym_RBRACE] = ACTIONS(1783),
    [anon_sym_LPAREN] = ACTIONS(1781),
    [anon_sym_BANG] = ACTIONS(1783),
    [anon_sym_TILDE] = ACTIONS(1783),
    [anon_sym_DASH] = ACTIONS(1781),
    [anon_sym_PLUS] = ACTIONS(1781),
    [anon_sym_DASH_DASH] = ACTIONS(1783),
    [anon_sym_PLUS_PLUS] = ACTIONS(1783),
    [anon_sym_DQUOTE] = ACTIONS(1783),
    [anon_sym_SQUOTE] = ACTIONS(1783),
    [sym_number_literal] = ACTIONS(1783),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(507)] = {
    [sym_identifier] = ACTIONS(1781),
    [sym_define_call_keyword] = ACTIONS(1781),
    [sym_define_function_keyword] = ACTIONS(1781),
    [sym_define_library_function_keyword] = ACTIONS(1781),
    [sym_define_module_keyword] = ACTIONS(1781),
    [sym_char_keyword] = ACTIONS(1781),
    [sym_widechar_keyword] = ACTIONS(1781),
    [sym_integer_keyword] = ACTIONS(1781),
    [sym_sinteger_keyword] = ACTIONS(1781),
    [sym_long_keyword] = ACTIONS(1781),
    [sym_slong_keyword] = ACTIONS(1781),
    [sym_float_keyword] = ACTIONS(1781),
    [sym_double_keyword] = ACTIONS(1781),
    [sym_variant_keyword] = ACTIONS(1781),
    [sym_variantarray_keyword] = ACTIONS(1781),
    [sym_local_var_keyword] = ACTIONS(1781),
    [sym_stack_var_keyword] = ACTIONS(1781),
    [sym_constant_keyword] = ACTIONS(1781),
    [sym_volatile_keyword] = ACTIONS(1781),
    [sym_non_volatile_keyword] = ACTIONS(1781),
    [sym_persistent_keyword] = ACTIONS(1781),
    [sym_dev_keyword] = ACTIONS(1781),
    [sym_devlev_keyword] = ACTIONS(1781),
    [sym_devchan_keyword] = ACTIONS(1781),
    [sym_if_keyword] = ACTIONS(1781),
    [sym_else_keyword] = ACTIONS(1781),
    [sym_switch_keyword] = ACTIONS(1781),
    [sym_case_keyword] = ACTIONS(1781),
    [sym_default_keyword] = ACTIONS(1781),
    [sym_while_keyword] = ACTIONS(1781),
    [sym_for_keyword] = ACTIONS(1781),
    [sym_break_keyword] = ACTIONS(1781),
    [sym_continue_keyword] = ACTIONS(1781),
    [sym_return_keyword] = ACTIONS(1781),
    [sym_select_keyword] = ACTIONS(1781),
    [sym_active_keyword] = ACTIONS(1781),
    [sym_struct_keyword] = ACTIONS(1781),
    [sym_structure_keyword] = ACTIONS(1781),
    [aux_sym_on_keyword_token1] = ACTIONS(1781),
    [aux_sym_off_keyword_token1] = ACTIONS(1781),
    [sym_send_level_keyword] = ACTIONS(1781),
    [sym_send_string_keyword] = ACTIONS(1781),
    [sym_send_command_keyword] = ACTIONS(1781),
    [sym_clear_buffer_keyword] = ACTIONS(1781),
    [sym_create_buffer_keyword] = ACTIONS(1781),
    [sym_create_multi_buffer_keyword] = ACTIONS(1781),
    [sym_call_keyword] = ACTIONS(1781),
    [sym_system_call_keyword] = ACTIONS(1781),
    [sym_devchan_to_keyword] = ACTIONS(1781),
    [sym_devchan_min_to_keyword] = ACTIONS(1781),
    [sym_devchan_total_off_keyword] = ACTIONS(1781),
    [sym_devchan_pulse_keyword] = ACTIONS(1781),
    [sym_wait_keyword] = ACTIONS(1781),
    [sym_cancel_wait_keyword] = ACTIONS(1781),
    [sym_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1781),
    [sym_not] = ACTIONS(1781),
    [sym_bnot] = ACTIONS(1781),
    [sym___file__] = ACTIONS(1781),
    [sym___date__] = ACTIONS(1781),
    [sym___ldate__] = ACTIONS(1781),
    [sym___time__] = ACTIONS(1781),
    [sym___line__] = ACTIONS(1781),
    [sym___name__] = ACTIONS(1781),
    [sym___netlinx__] = ACTIONS(1781),
    [sym_netlinx_axi_version] = ACTIONS(1781),
    [sym_url_flg_tcp] = ACTIONS(1781),
    [sym_url_flg_accinfopresent] = ACTIONS(1781),
    [sym_url_flg_temp] = ACTIONS(1781),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1781),
    [sym_url_flg_stat_mask] = ACTIONS(1781),
    [sym_url_flg_stat_lookup] = ACTIONS(1781),
    [sym_url_flg_stat_connecting] = ACTIONS(1781),
    [sym_url_flg_stat_waiting] = ACTIONS(1781),
    [sym_url_flg_stat_connected] = ACTIONS(1781),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1781),
    [sym_first_virtual_device] = ACTIONS(1781),
    [sym_first_local_port] = ACTIONS(1781),
    [sym_dynamic_virtual_device] = ACTIONS(1781),
    [sym_timeline_once] = ACTIONS(1781),
    [sym_timeline_repeat] = ACTIONS(1781),
    [sym_timeline_absolute] = ACTIONS(1781),
    [sym_timeline_relative] = ACTIONS(1781),
    [sym_do_push_timed_infinite] = ACTIONS(1781),
    [sym_source_type_no_address] = ACTIONS(1781),
    [sym_source_type_neuron_id] = ACTIONS(1781),
    [sym_source_type_ip_address] = ACTIONS(1781),
    [sym_source_type_axlink] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1781),
    [sym_source_type_ip_socket_address] = ACTIONS(1781),
    [sym_source_type_rs232] = ACTIONS(1781),
    [sym_source_type_internal] = ACTIONS(1781),
    [sym_source_type_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1781),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1781),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1781),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1781),
    [sym_true] = ACTIONS(1781),
    [sym_false] = ACTIONS(1781),
    [sym_file_read_only] = ACTIONS(1781),
    [sym_file_rw_new] = ACTIONS(1781),
    [sym_file_rw_append] = ACTIONS(1781),
    [sym_ip_tcp] = ACTIONS(1781),
    [sym_ip_udp] = ACTIONS(1781),
    [sym_ip_udp_2way] = ACTIONS(1781),
    [sym_xml_encode_types] = ACTIONS(1781),
    [sym_xml_encode_char_as_list] = ACTIONS(1781),
    [sym_xml_encode_le] = ACTIONS(1781),
    [sym_xml_decode_types] = ACTIONS(1781),
    [sym_xml_decode_no_preserve] = ACTIONS(1781),
    [sym_duet_dev_type_display_device] = ACTIONS(1781),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_io_device] = ACTIONS(1781),
    [sym_duet_dev_type_relay_device] = ACTIONS(1781),
    [sym_duet_dev_type_ups] = ACTIONS(1781),
    [sym_duet_dev_type_amplifier] = ACTIONS(1781),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1781),
    [sym_duet_dev_type_camera] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1781),
    [sym_duet_dev_type_dss] = ACTIONS(1781),
    [sym_duet_dev_type_dvr] = ACTIONS(1781),
    [sym_duet_dev_type_disc_device] = ACTIONS(1781),
    [sym_duet_dev_type_document_camera] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1781),
    [sym_duet_dev_type_hvac] = ACTIONS(1781),
    [sym_duet_dev_type_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_light] = ACTIONS(1781),
    [sym_duet_dev_type_monitor] = ACTIONS(1781),
    [sym_duet_dev_type_motor] = ACTIONS(1781),
    [sym_duet_dev_type_multi_window] = ACTIONS(1781),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1781),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1781),
    [sym_duet_dev_type_receiver] = ACTIONS(1781),
    [sym_duet_dev_type_security_system] = ACTIONS(1781),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1781),
    [sym_duet_dev_type_settop_box] = ACTIONS(1781),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1781),
    [sym_duet_dev_type_switcher] = ACTIONS(1781),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_tv] = ACTIONS(1781),
    [sym_duet_dev_type_utility] = ACTIONS(1781),
    [sym_duet_dev_type_vcr] = ACTIONS(1781),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_video_processor] = ACTIONS(1781),
    [sym_duet_dev_type_video_projector] = ACTIONS(1781),
    [sym_duet_dev_type_video_wall] = ACTIONS(1781),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1781),
    [sym_duet_dev_type_weather] = ACTIONS(1781),
    [sym_duet_dev_not_polled] = ACTIONS(1781),
    [sym_duet_dev_polled] = ACTIONS(1781),
    [sym_clkmgr_mode_network] = ACTIONS(1781),
    [sym_clkmgr_mode_standalone] = ACTIONS(1781),
    [sym_normal_standby] = ACTIONS(1781),
    [sym_normal_wake] = ACTIONS(1781),
    [sym_ip_multicast_ttl_option] = ACTIONS(1781),
    [sym_ip_ttl_subnet] = ACTIONS(1781),
    [sym_ip_ttl_site] = ACTIONS(1781),
    [sym_ip_ttl_region] = ACTIONS(1781),
    [sym_ip_ttl_continent] = ACTIONS(1781),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1781),
    [sym_ip_nodelay_on] = ACTIONS(1781),
    [sym_ip_nodelay_off] = ACTIONS(1781),
    [sym_amx_error] = ACTIONS(1781),
    [sym_amx_warning] = ACTIONS(1781),
    [sym_amx_info] = ACTIONS(1781),
    [sym_amx_debug] = ACTIONS(1781),
    [sym_netlinx_login_success] = ACTIONS(1781),
    [sym_netlinx_login_fail] = ACTIONS(1781),
    [sym_netlinx_logout] = ACTIONS(1781),
    [sym_null_str] = ACTIONS(1781),
    [sym_smtp_address] = ACTIONS(1781),
    [sym_smtp_port_number] = ACTIONS(1781),
    [sym_smtp_username] = ACTIONS(1781),
    [sym_smtp_password] = ACTIONS(1781),
    [sym_smtp_require_tls] = ACTIONS(1781),
    [sym_smtp_from] = ACTIONS(1781),
    [sym_smtp_tls_true] = ACTIONS(1781),
    [sym_smtp_tls_false] = ACTIONS(1781),
    [sym_listview_on_row_select_event] = ACTIONS(1781),
    [sym_data_structure_unknown] = ACTIONS(1781),
    [sym_data_structure_datafeed] = ACTIONS(1781),
    [sym_data_structure_datarecordset] = ACTIONS(1781),
    [sym_data_structure_datarecord] = ACTIONS(1781),
    [sym_data_structure_datafield] = ACTIONS(1781),
    [sym_data_max_content_field_count] = ACTIONS(1781),
    [sym_data_max_metadata_field_count] = ACTIONS(1781),
    [sym_data_max_id_length] = ACTIONS(1781),
    [sym_data_max_name_length] = ACTIONS(1781),
    [sym_data_max_description_length] = ACTIONS(1781),
    [sym_data_max_type_length] = ACTIONS(1781),
    [sym_data_max_format_length] = ACTIONS(1781),
    [sym_data_max_source_length] = ACTIONS(1781),
    [sym_data_max_value_length] = ACTIONS(1781),
    [sym_data_max_label_length] = ACTIONS(1781),
    [sym_data_max_url_length] = ACTIONS(1781),
    [sym_max_last_login_info_length] = ACTIONS(1781),
    [sym_data_type_unknown] = ACTIONS(1781),
    [sym_data_type_string] = ACTIONS(1781),
    [sym_data_type_datetime] = ACTIONS(1781),
    [sym_data_type_date] = ACTIONS(1781),
    [sym_data_type_time] = ACTIONS(1781),
    [sym_data_type_image] = ACTIONS(1781),
    [sym_data_format_url] = ACTIONS(1781),
    [sym_data_format_phone] = ACTIONS(1781),
    [sym_data_format_email] = ACTIONS(1781),
    [sym_data_format_iso8601] = ACTIONS(1781),
    [sym_auditlog_priv] = ACTIONS(1781),
    [sym_configuration_priv] = ACTIONS(1781),
    [sym_deviceconfig_priv] = ACTIONS(1781),
    [sym_ftp_priv] = ACTIONS(1781),
    [sym_http_priv] = ACTIONS(1781),
    [sym_networkconfig_priv] = ACTIONS(1781),
    [sym_programport_priv] = ACTIONS(1781),
    [sym_remoteui_priv] = ACTIONS(1781),
    [sym_securitycontrol_priv] = ACTIONS(1781),
    [sym_softwaremanagement_priv] = ACTIONS(1781),
    [sym_terminal_priv] = ACTIONS(1781),
    [sym_tpadmin_priv] = ACTIONS(1781),
    [sym_user1_priv] = ACTIONS(1781),
    [sym_user2_priv] = ACTIONS(1781),
    [sym_user3_priv] = ACTIONS(1781),
    [sym_user4_priv] = ACTIONS(1781),
    [sym_usermanagement_priv] = ACTIONS(1781),
    [sym_valid_account] = ACTIONS(1781),
    [sym_err_name_invalid] = ACTIONS(1781),
    [sym_err_password_invalid] = ACTIONS(1781),
    [sym_err_invalid_account] = ACTIONS(1781),
    [sym_err_no_authorization] = ACTIONS(1781),
    [sym_err_invalid_parameter] = ACTIONS(1781),
    [sym_err_user_locked_out] = ACTIONS(1781),
    [sym_err_user_account_expired] = ACTIONS(1781),
    [sym_err_general_error] = ACTIONS(1781),
    [sym_err_authentication_error] = ACTIONS(1781),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1781),
    [sym_tls_validate_certificate] = ACTIONS(1781),
    [sym_tdata] = ACTIONS(1781),
    [sym_tchannel] = ACTIONS(1781),
    [sym_tlevel] = ACTIONS(1781),
    [sym_tbutton] = ACTIONS(1781),
    [sym_ttimeline] = ACTIONS(1781),
    [sym_tcustom] = ACTIONS(1781),
    [sym_url_struct] = ACTIONS(1781),
    [sym_dns_struct] = ACTIONS(1781),
    [sym_ip_address_struct] = ACTIONS(1781),
    [sym_dev_info_struct] = ACTIONS(1781),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1781),
    [sym_data_feed] = ACTIONS(1781),
    [sym_data_field] = ACTIONS(1781),
    [sym_data_record] = ACTIONS(1781),
    [sym_wc_data_feed] = ACTIONS(1781),
    [sym_wc_data_field] = ACTIONS(1781),
    [sym_wc_data_record] = ACTIONS(1781),
    [sym_last_login_info] = ACTIONS(1781),
    [sym____reserved___] = ACTIONS(1781),
    [sym_date] = ACTIONS(1781),
    [sym_day] = ACTIONS(1781),
    [sym_time] = ACTIONS(1781),
    [sym_ldate] = ACTIONS(1781),
    [sym_push_device] = ACTIONS(1781),
    [sym_push_channel] = ACTIONS(1781),
    [sym_push_devchan] = ACTIONS(1781),
    [sym_release_device] = ACTIONS(1781),
    [sym_release_channel] = ACTIONS(1781),
    [sym_release_devchan] = ACTIONS(1781),
    [sym_master_slot] = ACTIONS(1781),
    [sym_get_pulse_time] = ACTIONS(1781),
    [sym_get_timer] = ACTIONS(1781),
    [sym_master_sn] = ACTIONS(1781),
    [sym_system_number] = ACTIONS(1781),
    [sym_button] = ACTIONS(1781),
    [sym_dv_channel] = ACTIONS(1781),
    [sym_data] = ACTIONS(1781),
    [sym_level] = ACTIONS(1781),
    [sym_timeline] = ACTIONS(1781),
    [sym_channel] = ACTIONS(1781),
    [sym_custom] = ACTIONS(1781),
    [sym_atoi] = ACTIONS(1781),
    [sym_atol] = ACTIONS(1781),
    [sym_atof] = ACTIONS(1781),
    [sym_itoa] = ACTIONS(1781),
    [sym_itohex] = ACTIONS(1781),
    [sym_hextoi] = ACTIONS(1781),
    [sym_ftoa] = ACTIONS(1781),
    [sym_find_string] = ACTIONS(1781),
    [sym_left_string] = ACTIONS(1781),
    [sym_right_string] = ACTIONS(1781),
    [sym_mid_string] = ACTIONS(1781),
    [sym_remove_string] = ACTIONS(1781),
    [sym_get_buffer_string] = ACTIONS(1781),
    [sym_get_buffer_char] = ACTIONS(1781),
    [sym_get_multi_buffer_string] = ACTIONS(1781),
    [sym_length_string] = ACTIONS(1781),
    [sym_max_length_string] = ACTIONS(1781),
    [sym_set_length_string] = ACTIONS(1781),
    [sym_compare_string] = ACTIONS(1781),
    [sym_upper_string] = ACTIONS(1781),
    [sym_lower_string] = ACTIONS(1781),
    [sym_get_last] = ACTIONS(1781),
    [sym_date_to_day] = ACTIONS(1781),
    [sym_date_to_month] = ACTIONS(1781),
    [sym_date_to_year] = ACTIONS(1781),
    [sym_time_to_hour] = ACTIONS(1781),
    [sym_time_to_minute] = ACTIONS(1781),
    [sym_time_to_second] = ACTIONS(1781),
    [sym_day_of_week] = ACTIONS(1781),
    [sym_set_pulse_time] = ACTIONS(1781),
    [sym_set_timer] = ACTIONS(1781),
    [sym_file_open] = ACTIONS(1781),
    [sym_file_close] = ACTIONS(1781),
    [sym_file_read] = ACTIONS(1781),
    [sym_file_read_line] = ACTIONS(1781),
    [sym_file_write] = ACTIONS(1781),
    [sym_file_write_line] = ACTIONS(1781),
    [sym_file_copy] = ACTIONS(1781),
    [sym_file_delete] = ACTIONS(1781),
    [sym_file_rename] = ACTIONS(1781),
    [sym_file_dir] = ACTIONS(1781),
    [sym_file_getdir] = ACTIONS(1781),
    [sym_file_setdir] = ACTIONS(1781),
    [sym_file_create_dir] = ACTIONS(1781),
    [sym_file_remove_dir] = ACTIONS(1781),
    [sym_file_seek] = ACTIONS(1781),
    [sym_length_array] = ACTIONS(1781),
    [sym_max_length_array] = ACTIONS(1781),
    [sym_set_length_array] = ACTIONS(1781),
    [sym_ip_client_open] = ACTIONS(1781),
    [sym_ip_client_close] = ACTIONS(1781),
    [sym_ip_server_open] = ACTIONS(1781),
    [sym_ip_server_close] = ACTIONS(1781),
    [sym_ip_bound_client_open] = ACTIONS(1781),
    [sym_ip_mc_server_open] = ACTIONS(1781),
    [sym_ip_set_option] = ACTIONS(1781),
    [sym_ssh_client_open] = ACTIONS(1781),
    [sym_ssh_client_close] = ACTIONS(1781),
    [sym_tls_client_open] = ACTIONS(1781),
    [sym_tls_client_close] = ACTIONS(1781),
    [sym_do_push] = ACTIONS(1781),
    [sym_do_push_timed] = ACTIONS(1781),
    [sym_do_release] = ACTIONS(1781),
    [sym_device_id] = ACTIONS(1781),
    [sym_device_info] = ACTIONS(1781),
    [sym_device_id_string] = ACTIONS(1781),
    [sym_device_standby] = ACTIONS(1781),
    [sym_device_wake] = ACTIONS(1781),
    [sym_timeline_create] = ACTIONS(1781),
    [sym_timeline_kill] = ACTIONS(1781),
    [sym_timeline_reload] = ACTIONS(1781),
    [sym_timeline_pause] = ACTIONS(1781),
    [sym_timeline_restart] = ACTIONS(1781),
    [sym_timeline_set] = ACTIONS(1781),
    [sym_timeline_get] = ACTIONS(1781),
    [sym_timeline_active] = ACTIONS(1781),
    [sym_timeline_dynamic_id] = ACTIONS(1781),
    [sym_variable_to_string] = ACTIONS(1781),
    [sym_string_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_string] = ACTIONS(1781),
    [sym_type_cast] = ACTIONS(1781),
    [sym_variable_to_xml] = ACTIONS(1781),
    [sym_xml_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_xml] = ACTIONS(1781),
    [sym_redirect_string] = ACTIONS(1781),
    [sym_random_number] = ACTIONS(1781),
    [sym_get_unique_id] = ACTIONS(1781),
    [sym_get_system_number] = ACTIONS(1781),
    [sym_set_system_number] = ACTIONS(1781),
    [sym_get_serial_number] = ACTIONS(1781),
    [sym_get_master_build] = ACTIONS(1781),
    [sym_set_validation_code] = ACTIONS(1781),
    [sym_reboot] = ACTIONS(1781),
    [sym_load_duet_module] = ACTIONS(1781),
    [sym_unload_duet_module] = ACTIONS(1781),
    [sym_duet_mem_size_set] = ACTIONS(1781),
    [sym_duet_mem_size_get] = ACTIONS(1781),
    [sym_dynamic_application_device] = ACTIONS(1781),
    [sym_dynamic_polled_port] = ACTIONS(1781),
    [sym_static_port_binding] = ACTIONS(1781),
    [sym_static_ip_binding] = ACTIONS(1781),
    [sym_get_dns_list] = ACTIONS(1781),
    [sym_set_dns_list] = ACTIONS(1781),
    [sym_get_ip_address] = ACTIONS(1781),
    [sym_set_ip_address] = ACTIONS(1781),
    [sym_get_url_list] = ACTIONS(1781),
    [sym_add_url_entry] = ACTIONS(1781),
    [sym_delete_url_entry] = ACTIONS(1781),
    [sym_abs_value] = ACTIONS(1781),
    [sym_max_value] = ACTIONS(1781),
    [sym_min_value] = ACTIONS(1781),
    [sym_exp_value] = ACTIONS(1781),
    [sym_log_value] = ACTIONS(1781),
    [sym_log10_value] = ACTIONS(1781),
    [sym_power_value] = ACTIONS(1781),
    [sym_sqrt_value] = ACTIONS(1781),
    [sym_raw_be] = ACTIONS(1781),
    [sym_raw_le] = ACTIONS(1781),
    [sym_internal_queue_size_set] = ACTIONS(1781),
    [sym_internal_queue_size_get] = ACTIONS(1781),
    [sym_internal_threshold_set] = ACTIONS(1781),
    [sym_internal_threshold_get] = ACTIONS(1781),
    [sym_rebuilt_event] = ACTIONS(1781),
    [sym_do_custom_event] = ACTIONS(1781),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1781),
    [sym_clkmgr_set_clk_source] = ACTIONS(1781),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1781),
    [sym_clkmgr_get_timezone] = ACTIONS(1781),
    [sym_clkmgr_set_timezone] = ACTIONS(1781),
    [sym_clkmgr_get_resync_period] = ACTIONS(1781),
    [sym_clkmgr_set_resync_period] = ACTIONS(1781),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_timeservers] = ACTIONS(1781),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_set_log_level] = ACTIONS(1781),
    [sym_get_log_level] = ACTIONS(1781),
    [sym_amx_log] = ACTIONS(1781),
    [sym_smtp_server_config_set] = ACTIONS(1781),
    [sym_smtp_server_config_get] = ACTIONS(1781),
    [sym_smtp_send] = ACTIONS(1781),
    [sym_validate_netlinx_account] = ACTIONS(1781),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1781),
    [sym_authenticate_certificate] = ACTIONS(1781),
    [sym_audit_netlinx_session_event] = ACTIONS(1781),
    [sym_audit_netlinx_generic_event] = ACTIONS(1781),
    [sym_get_available_flash_disk_space] = ACTIONS(1781),
    [sym_get_max_flash_disk_space] = ACTIONS(1781),
    [sym_data_create_feed] = ACTIONS(1781),
    [sym_data_delete_feed] = ACTIONS(1781),
    [sym_data_publish_feed] = ACTIONS(1781),
    [sym_data_get_published_feed] = ACTIONS(1781),
    [sym_data_add_record] = ACTIONS(1781),
    [sym_data_get_event_record] = ACTIONS(1781),
    [sym__wc_data_create_feed] = ACTIONS(1781),
    [sym__wc_data_add_record] = ACTIONS(1781),
    [sym__wc_data_get_event_record] = ACTIONS(1781),
    [sym_format] = ACTIONS(1781),
    [sym_astro_clock] = ACTIONS(1781),
    [sym_set_outdoor_temperature] = ACTIONS(1781),
    [sym_set_virtual_level_count] = ACTIONS(1781),
    [sym_set_virtual_channel_count] = ACTIONS(1781),
    [sym_set_virtual_port_count] = ACTIONS(1781),
    [sym__wc_encode_int] = ACTIONS(1781),
    [sym__wc_decode_int] = ACTIONS(1781),
    [sym__wc_int] = ACTIONS(1781),
    [sym__wc_to_ch_int] = ACTIONS(1781),
    [sym__ch_to_wc_int] = ACTIONS(1781),
    [sym__wc_find_string_int] = ACTIONS(1781),
    [sym__wc_left_string_int] = ACTIONS(1781),
    [sym__wc_lower_string_int] = ACTIONS(1781),
    [sym__wc_mid_string_int] = ACTIONS(1781),
    [sym__wc_remove_string_int] = ACTIONS(1781),
    [sym__wc_right_string_int] = ACTIONS(1781),
    [sym__wc_upper_string_int] = ACTIONS(1781),
    [sym__wc_compare_string_int] = ACTIONS(1781),
    [sym__wc_get_buffer_char_int] = ACTIONS(1781),
    [sym__wc_get_buffer_string_int] = ACTIONS(1781),
    [sym__wc_concat_string_int] = ACTIONS(1781),
    [sym__wc_file_open_int] = ACTIONS(1781),
    [sym__wc_file_close_int] = ACTIONS(1781),
    [sym__wc_file_read_int] = ACTIONS(1781),
    [sym__wc_file_read_line_int] = ACTIONS(1781),
    [sym__wc_file_write_int] = ACTIONS(1781),
    [sym__wc_file_write_line_int] = ACTIONS(1781),
    [aux_sym_preproc_include_token1] = ACTIONS(1781),
    [aux_sym_preproc_define_token1] = ACTIONS(1783),
    [aux_sym_preproc_warn_token1] = ACTIONS(1783),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1783),
    [anon_sym_LBRACK] = ACTIONS(1783),
    [anon_sym_LBRACE] = ACTIONS(1783),
    [anon_sym_RBRACE] = ACTIONS(1783),
    [anon_sym_LPAREN] = ACTIONS(1781),
    [anon_sym_BANG] = ACTIONS(1783),
    [anon_sym_TILDE] = ACTIONS(1783),
    [anon_sym_DASH] = ACTIONS(1781),
    [anon_sym_PLUS] = ACTIONS(1781),
    [anon_sym_DASH_DASH] = ACTIONS(1783),
    [anon_sym_PLUS_PLUS] = ACTIONS(1783),
    [anon_sym_DQUOTE] = ACTIONS(1783),
    [anon_sym_SQUOTE] = ACTIONS(1783),
    [sym_number_literal] = ACTIONS(1783),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(508)] = {
    [sym_identifier] = ACTIONS(1781),
    [sym_define_call_keyword] = ACTIONS(1781),
    [sym_define_function_keyword] = ACTIONS(1781),
    [sym_define_library_function_keyword] = ACTIONS(1781),
    [sym_define_module_keyword] = ACTIONS(1781),
    [sym_char_keyword] = ACTIONS(1781),
    [sym_widechar_keyword] = ACTIONS(1781),
    [sym_integer_keyword] = ACTIONS(1781),
    [sym_sinteger_keyword] = ACTIONS(1781),
    [sym_long_keyword] = ACTIONS(1781),
    [sym_slong_keyword] = ACTIONS(1781),
    [sym_float_keyword] = ACTIONS(1781),
    [sym_double_keyword] = ACTIONS(1781),
    [sym_variant_keyword] = ACTIONS(1781),
    [sym_variantarray_keyword] = ACTIONS(1781),
    [sym_local_var_keyword] = ACTIONS(1781),
    [sym_stack_var_keyword] = ACTIONS(1781),
    [sym_constant_keyword] = ACTIONS(1781),
    [sym_volatile_keyword] = ACTIONS(1781),
    [sym_non_volatile_keyword] = ACTIONS(1781),
    [sym_persistent_keyword] = ACTIONS(1781),
    [sym_dev_keyword] = ACTIONS(1781),
    [sym_devlev_keyword] = ACTIONS(1781),
    [sym_devchan_keyword] = ACTIONS(1781),
    [sym_if_keyword] = ACTIONS(1781),
    [sym_else_keyword] = ACTIONS(1781),
    [sym_switch_keyword] = ACTIONS(1781),
    [sym_case_keyword] = ACTIONS(1781),
    [sym_default_keyword] = ACTIONS(1781),
    [sym_while_keyword] = ACTIONS(1781),
    [sym_for_keyword] = ACTIONS(1781),
    [sym_break_keyword] = ACTIONS(1781),
    [sym_continue_keyword] = ACTIONS(1781),
    [sym_return_keyword] = ACTIONS(1781),
    [sym_select_keyword] = ACTIONS(1781),
    [sym_active_keyword] = ACTIONS(1781),
    [sym_struct_keyword] = ACTIONS(1781),
    [sym_structure_keyword] = ACTIONS(1781),
    [aux_sym_on_keyword_token1] = ACTIONS(1781),
    [aux_sym_off_keyword_token1] = ACTIONS(1781),
    [sym_send_level_keyword] = ACTIONS(1781),
    [sym_send_string_keyword] = ACTIONS(1781),
    [sym_send_command_keyword] = ACTIONS(1781),
    [sym_clear_buffer_keyword] = ACTIONS(1781),
    [sym_create_buffer_keyword] = ACTIONS(1781),
    [sym_create_multi_buffer_keyword] = ACTIONS(1781),
    [sym_call_keyword] = ACTIONS(1781),
    [sym_system_call_keyword] = ACTIONS(1781),
    [sym_devchan_to_keyword] = ACTIONS(1781),
    [sym_devchan_min_to_keyword] = ACTIONS(1781),
    [sym_devchan_total_off_keyword] = ACTIONS(1781),
    [sym_devchan_pulse_keyword] = ACTIONS(1781),
    [sym_wait_keyword] = ACTIONS(1781),
    [sym_cancel_wait_keyword] = ACTIONS(1781),
    [sym_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1781),
    [sym_not] = ACTIONS(1781),
    [sym_bnot] = ACTIONS(1781),
    [sym___file__] = ACTIONS(1781),
    [sym___date__] = ACTIONS(1781),
    [sym___ldate__] = ACTIONS(1781),
    [sym___time__] = ACTIONS(1781),
    [sym___line__] = ACTIONS(1781),
    [sym___name__] = ACTIONS(1781),
    [sym___netlinx__] = ACTIONS(1781),
    [sym_netlinx_axi_version] = ACTIONS(1781),
    [sym_url_flg_tcp] = ACTIONS(1781),
    [sym_url_flg_accinfopresent] = ACTIONS(1781),
    [sym_url_flg_temp] = ACTIONS(1781),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1781),
    [sym_url_flg_stat_mask] = ACTIONS(1781),
    [sym_url_flg_stat_lookup] = ACTIONS(1781),
    [sym_url_flg_stat_connecting] = ACTIONS(1781),
    [sym_url_flg_stat_waiting] = ACTIONS(1781),
    [sym_url_flg_stat_connected] = ACTIONS(1781),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1781),
    [sym_first_virtual_device] = ACTIONS(1781),
    [sym_first_local_port] = ACTIONS(1781),
    [sym_dynamic_virtual_device] = ACTIONS(1781),
    [sym_timeline_once] = ACTIONS(1781),
    [sym_timeline_repeat] = ACTIONS(1781),
    [sym_timeline_absolute] = ACTIONS(1781),
    [sym_timeline_relative] = ACTIONS(1781),
    [sym_do_push_timed_infinite] = ACTIONS(1781),
    [sym_source_type_no_address] = ACTIONS(1781),
    [sym_source_type_neuron_id] = ACTIONS(1781),
    [sym_source_type_ip_address] = ACTIONS(1781),
    [sym_source_type_axlink] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1781),
    [sym_source_type_ip_socket_address] = ACTIONS(1781),
    [sym_source_type_rs232] = ACTIONS(1781),
    [sym_source_type_internal] = ACTIONS(1781),
    [sym_source_type_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1781),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1781),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1781),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1781),
    [sym_true] = ACTIONS(1781),
    [sym_false] = ACTIONS(1781),
    [sym_file_read_only] = ACTIONS(1781),
    [sym_file_rw_new] = ACTIONS(1781),
    [sym_file_rw_append] = ACTIONS(1781),
    [sym_ip_tcp] = ACTIONS(1781),
    [sym_ip_udp] = ACTIONS(1781),
    [sym_ip_udp_2way] = ACTIONS(1781),
    [sym_xml_encode_types] = ACTIONS(1781),
    [sym_xml_encode_char_as_list] = ACTIONS(1781),
    [sym_xml_encode_le] = ACTIONS(1781),
    [sym_xml_decode_types] = ACTIONS(1781),
    [sym_xml_decode_no_preserve] = ACTIONS(1781),
    [sym_duet_dev_type_display_device] = ACTIONS(1781),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_io_device] = ACTIONS(1781),
    [sym_duet_dev_type_relay_device] = ACTIONS(1781),
    [sym_duet_dev_type_ups] = ACTIONS(1781),
    [sym_duet_dev_type_amplifier] = ACTIONS(1781),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1781),
    [sym_duet_dev_type_camera] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1781),
    [sym_duet_dev_type_dss] = ACTIONS(1781),
    [sym_duet_dev_type_dvr] = ACTIONS(1781),
    [sym_duet_dev_type_disc_device] = ACTIONS(1781),
    [sym_duet_dev_type_document_camera] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1781),
    [sym_duet_dev_type_hvac] = ACTIONS(1781),
    [sym_duet_dev_type_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_light] = ACTIONS(1781),
    [sym_duet_dev_type_monitor] = ACTIONS(1781),
    [sym_duet_dev_type_motor] = ACTIONS(1781),
    [sym_duet_dev_type_multi_window] = ACTIONS(1781),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1781),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1781),
    [sym_duet_dev_type_receiver] = ACTIONS(1781),
    [sym_duet_dev_type_security_system] = ACTIONS(1781),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1781),
    [sym_duet_dev_type_settop_box] = ACTIONS(1781),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1781),
    [sym_duet_dev_type_switcher] = ACTIONS(1781),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_tv] = ACTIONS(1781),
    [sym_duet_dev_type_utility] = ACTIONS(1781),
    [sym_duet_dev_type_vcr] = ACTIONS(1781),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_video_processor] = ACTIONS(1781),
    [sym_duet_dev_type_video_projector] = ACTIONS(1781),
    [sym_duet_dev_type_video_wall] = ACTIONS(1781),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1781),
    [sym_duet_dev_type_weather] = ACTIONS(1781),
    [sym_duet_dev_not_polled] = ACTIONS(1781),
    [sym_duet_dev_polled] = ACTIONS(1781),
    [sym_clkmgr_mode_network] = ACTIONS(1781),
    [sym_clkmgr_mode_standalone] = ACTIONS(1781),
    [sym_normal_standby] = ACTIONS(1781),
    [sym_normal_wake] = ACTIONS(1781),
    [sym_ip_multicast_ttl_option] = ACTIONS(1781),
    [sym_ip_ttl_subnet] = ACTIONS(1781),
    [sym_ip_ttl_site] = ACTIONS(1781),
    [sym_ip_ttl_region] = ACTIONS(1781),
    [sym_ip_ttl_continent] = ACTIONS(1781),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1781),
    [sym_ip_nodelay_on] = ACTIONS(1781),
    [sym_ip_nodelay_off] = ACTIONS(1781),
    [sym_amx_error] = ACTIONS(1781),
    [sym_amx_warning] = ACTIONS(1781),
    [sym_amx_info] = ACTIONS(1781),
    [sym_amx_debug] = ACTIONS(1781),
    [sym_netlinx_login_success] = ACTIONS(1781),
    [sym_netlinx_login_fail] = ACTIONS(1781),
    [sym_netlinx_logout] = ACTIONS(1781),
    [sym_null_str] = ACTIONS(1781),
    [sym_smtp_address] = ACTIONS(1781),
    [sym_smtp_port_number] = ACTIONS(1781),
    [sym_smtp_username] = ACTIONS(1781),
    [sym_smtp_password] = ACTIONS(1781),
    [sym_smtp_require_tls] = ACTIONS(1781),
    [sym_smtp_from] = ACTIONS(1781),
    [sym_smtp_tls_true] = ACTIONS(1781),
    [sym_smtp_tls_false] = ACTIONS(1781),
    [sym_listview_on_row_select_event] = ACTIONS(1781),
    [sym_data_structure_unknown] = ACTIONS(1781),
    [sym_data_structure_datafeed] = ACTIONS(1781),
    [sym_data_structure_datarecordset] = ACTIONS(1781),
    [sym_data_structure_datarecord] = ACTIONS(1781),
    [sym_data_structure_datafield] = ACTIONS(1781),
    [sym_data_max_content_field_count] = ACTIONS(1781),
    [sym_data_max_metadata_field_count] = ACTIONS(1781),
    [sym_data_max_id_length] = ACTIONS(1781),
    [sym_data_max_name_length] = ACTIONS(1781),
    [sym_data_max_description_length] = ACTIONS(1781),
    [sym_data_max_type_length] = ACTIONS(1781),
    [sym_data_max_format_length] = ACTIONS(1781),
    [sym_data_max_source_length] = ACTIONS(1781),
    [sym_data_max_value_length] = ACTIONS(1781),
    [sym_data_max_label_length] = ACTIONS(1781),
    [sym_data_max_url_length] = ACTIONS(1781),
    [sym_max_last_login_info_length] = ACTIONS(1781),
    [sym_data_type_unknown] = ACTIONS(1781),
    [sym_data_type_string] = ACTIONS(1781),
    [sym_data_type_datetime] = ACTIONS(1781),
    [sym_data_type_date] = ACTIONS(1781),
    [sym_data_type_time] = ACTIONS(1781),
    [sym_data_type_image] = ACTIONS(1781),
    [sym_data_format_url] = ACTIONS(1781),
    [sym_data_format_phone] = ACTIONS(1781),
    [sym_data_format_email] = ACTIONS(1781),
    [sym_data_format_iso8601] = ACTIONS(1781),
    [sym_auditlog_priv] = ACTIONS(1781),
    [sym_configuration_priv] = ACTIONS(1781),
    [sym_deviceconfig_priv] = ACTIONS(1781),
    [sym_ftp_priv] = ACTIONS(1781),
    [sym_http_priv] = ACTIONS(1781),
    [sym_networkconfig_priv] = ACTIONS(1781),
    [sym_programport_priv] = ACTIONS(1781),
    [sym_remoteui_priv] = ACTIONS(1781),
    [sym_securitycontrol_priv] = ACTIONS(1781),
    [sym_softwaremanagement_priv] = ACTIONS(1781),
    [sym_terminal_priv] = ACTIONS(1781),
    [sym_tpadmin_priv] = ACTIONS(1781),
    [sym_user1_priv] = ACTIONS(1781),
    [sym_user2_priv] = ACTIONS(1781),
    [sym_user3_priv] = ACTIONS(1781),
    [sym_user4_priv] = ACTIONS(1781),
    [sym_usermanagement_priv] = ACTIONS(1781),
    [sym_valid_account] = ACTIONS(1781),
    [sym_err_name_invalid] = ACTIONS(1781),
    [sym_err_password_invalid] = ACTIONS(1781),
    [sym_err_invalid_account] = ACTIONS(1781),
    [sym_err_no_authorization] = ACTIONS(1781),
    [sym_err_invalid_parameter] = ACTIONS(1781),
    [sym_err_user_locked_out] = ACTIONS(1781),
    [sym_err_user_account_expired] = ACTIONS(1781),
    [sym_err_general_error] = ACTIONS(1781),
    [sym_err_authentication_error] = ACTIONS(1781),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1781),
    [sym_tls_validate_certificate] = ACTIONS(1781),
    [sym_tdata] = ACTIONS(1781),
    [sym_tchannel] = ACTIONS(1781),
    [sym_tlevel] = ACTIONS(1781),
    [sym_tbutton] = ACTIONS(1781),
    [sym_ttimeline] = ACTIONS(1781),
    [sym_tcustom] = ACTIONS(1781),
    [sym_url_struct] = ACTIONS(1781),
    [sym_dns_struct] = ACTIONS(1781),
    [sym_ip_address_struct] = ACTIONS(1781),
    [sym_dev_info_struct] = ACTIONS(1781),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1781),
    [sym_data_feed] = ACTIONS(1781),
    [sym_data_field] = ACTIONS(1781),
    [sym_data_record] = ACTIONS(1781),
    [sym_wc_data_feed] = ACTIONS(1781),
    [sym_wc_data_field] = ACTIONS(1781),
    [sym_wc_data_record] = ACTIONS(1781),
    [sym_last_login_info] = ACTIONS(1781),
    [sym____reserved___] = ACTIONS(1781),
    [sym_date] = ACTIONS(1781),
    [sym_day] = ACTIONS(1781),
    [sym_time] = ACTIONS(1781),
    [sym_ldate] = ACTIONS(1781),
    [sym_push_device] = ACTIONS(1781),
    [sym_push_channel] = ACTIONS(1781),
    [sym_push_devchan] = ACTIONS(1781),
    [sym_release_device] = ACTIONS(1781),
    [sym_release_channel] = ACTIONS(1781),
    [sym_release_devchan] = ACTIONS(1781),
    [sym_master_slot] = ACTIONS(1781),
    [sym_get_pulse_time] = ACTIONS(1781),
    [sym_get_timer] = ACTIONS(1781),
    [sym_master_sn] = ACTIONS(1781),
    [sym_system_number] = ACTIONS(1781),
    [sym_button] = ACTIONS(1781),
    [sym_dv_channel] = ACTIONS(1781),
    [sym_data] = ACTIONS(1781),
    [sym_level] = ACTIONS(1781),
    [sym_timeline] = ACTIONS(1781),
    [sym_channel] = ACTIONS(1781),
    [sym_custom] = ACTIONS(1781),
    [sym_atoi] = ACTIONS(1781),
    [sym_atol] = ACTIONS(1781),
    [sym_atof] = ACTIONS(1781),
    [sym_itoa] = ACTIONS(1781),
    [sym_itohex] = ACTIONS(1781),
    [sym_hextoi] = ACTIONS(1781),
    [sym_ftoa] = ACTIONS(1781),
    [sym_find_string] = ACTIONS(1781),
    [sym_left_string] = ACTIONS(1781),
    [sym_right_string] = ACTIONS(1781),
    [sym_mid_string] = ACTIONS(1781),
    [sym_remove_string] = ACTIONS(1781),
    [sym_get_buffer_string] = ACTIONS(1781),
    [sym_get_buffer_char] = ACTIONS(1781),
    [sym_get_multi_buffer_string] = ACTIONS(1781),
    [sym_length_string] = ACTIONS(1781),
    [sym_max_length_string] = ACTIONS(1781),
    [sym_set_length_string] = ACTIONS(1781),
    [sym_compare_string] = ACTIONS(1781),
    [sym_upper_string] = ACTIONS(1781),
    [sym_lower_string] = ACTIONS(1781),
    [sym_get_last] = ACTIONS(1781),
    [sym_date_to_day] = ACTIONS(1781),
    [sym_date_to_month] = ACTIONS(1781),
    [sym_date_to_year] = ACTIONS(1781),
    [sym_time_to_hour] = ACTIONS(1781),
    [sym_time_to_minute] = ACTIONS(1781),
    [sym_time_to_second] = ACTIONS(1781),
    [sym_day_of_week] = ACTIONS(1781),
    [sym_set_pulse_time] = ACTIONS(1781),
    [sym_set_timer] = ACTIONS(1781),
    [sym_file_open] = ACTIONS(1781),
    [sym_file_close] = ACTIONS(1781),
    [sym_file_read] = ACTIONS(1781),
    [sym_file_read_line] = ACTIONS(1781),
    [sym_file_write] = ACTIONS(1781),
    [sym_file_write_line] = ACTIONS(1781),
    [sym_file_copy] = ACTIONS(1781),
    [sym_file_delete] = ACTIONS(1781),
    [sym_file_rename] = ACTIONS(1781),
    [sym_file_dir] = ACTIONS(1781),
    [sym_file_getdir] = ACTIONS(1781),
    [sym_file_setdir] = ACTIONS(1781),
    [sym_file_create_dir] = ACTIONS(1781),
    [sym_file_remove_dir] = ACTIONS(1781),
    [sym_file_seek] = ACTIONS(1781),
    [sym_length_array] = ACTIONS(1781),
    [sym_max_length_array] = ACTIONS(1781),
    [sym_set_length_array] = ACTIONS(1781),
    [sym_ip_client_open] = ACTIONS(1781),
    [sym_ip_client_close] = ACTIONS(1781),
    [sym_ip_server_open] = ACTIONS(1781),
    [sym_ip_server_close] = ACTIONS(1781),
    [sym_ip_bound_client_open] = ACTIONS(1781),
    [sym_ip_mc_server_open] = ACTIONS(1781),
    [sym_ip_set_option] = ACTIONS(1781),
    [sym_ssh_client_open] = ACTIONS(1781),
    [sym_ssh_client_close] = ACTIONS(1781),
    [sym_tls_client_open] = ACTIONS(1781),
    [sym_tls_client_close] = ACTIONS(1781),
    [sym_do_push] = ACTIONS(1781),
    [sym_do_push_timed] = ACTIONS(1781),
    [sym_do_release] = ACTIONS(1781),
    [sym_device_id] = ACTIONS(1781),
    [sym_device_info] = ACTIONS(1781),
    [sym_device_id_string] = ACTIONS(1781),
    [sym_device_standby] = ACTIONS(1781),
    [sym_device_wake] = ACTIONS(1781),
    [sym_timeline_create] = ACTIONS(1781),
    [sym_timeline_kill] = ACTIONS(1781),
    [sym_timeline_reload] = ACTIONS(1781),
    [sym_timeline_pause] = ACTIONS(1781),
    [sym_timeline_restart] = ACTIONS(1781),
    [sym_timeline_set] = ACTIONS(1781),
    [sym_timeline_get] = ACTIONS(1781),
    [sym_timeline_active] = ACTIONS(1781),
    [sym_timeline_dynamic_id] = ACTIONS(1781),
    [sym_variable_to_string] = ACTIONS(1781),
    [sym_string_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_string] = ACTIONS(1781),
    [sym_type_cast] = ACTIONS(1781),
    [sym_variable_to_xml] = ACTIONS(1781),
    [sym_xml_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_xml] = ACTIONS(1781),
    [sym_redirect_string] = ACTIONS(1781),
    [sym_random_number] = ACTIONS(1781),
    [sym_get_unique_id] = ACTIONS(1781),
    [sym_get_system_number] = ACTIONS(1781),
    [sym_set_system_number] = ACTIONS(1781),
    [sym_get_serial_number] = ACTIONS(1781),
    [sym_get_master_build] = ACTIONS(1781),
    [sym_set_validation_code] = ACTIONS(1781),
    [sym_reboot] = ACTIONS(1781),
    [sym_load_duet_module] = ACTIONS(1781),
    [sym_unload_duet_module] = ACTIONS(1781),
    [sym_duet_mem_size_set] = ACTIONS(1781),
    [sym_duet_mem_size_get] = ACTIONS(1781),
    [sym_dynamic_application_device] = ACTIONS(1781),
    [sym_dynamic_polled_port] = ACTIONS(1781),
    [sym_static_port_binding] = ACTIONS(1781),
    [sym_static_ip_binding] = ACTIONS(1781),
    [sym_get_dns_list] = ACTIONS(1781),
    [sym_set_dns_list] = ACTIONS(1781),
    [sym_get_ip_address] = ACTIONS(1781),
    [sym_set_ip_address] = ACTIONS(1781),
    [sym_get_url_list] = ACTIONS(1781),
    [sym_add_url_entry] = ACTIONS(1781),
    [sym_delete_url_entry] = ACTIONS(1781),
    [sym_abs_value] = ACTIONS(1781),
    [sym_max_value] = ACTIONS(1781),
    [sym_min_value] = ACTIONS(1781),
    [sym_exp_value] = ACTIONS(1781),
    [sym_log_value] = ACTIONS(1781),
    [sym_log10_value] = ACTIONS(1781),
    [sym_power_value] = ACTIONS(1781),
    [sym_sqrt_value] = ACTIONS(1781),
    [sym_raw_be] = ACTIONS(1781),
    [sym_raw_le] = ACTIONS(1781),
    [sym_internal_queue_size_set] = ACTIONS(1781),
    [sym_internal_queue_size_get] = ACTIONS(1781),
    [sym_internal_threshold_set] = ACTIONS(1781),
    [sym_internal_threshold_get] = ACTIONS(1781),
    [sym_rebuilt_event] = ACTIONS(1781),
    [sym_do_custom_event] = ACTIONS(1781),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1781),
    [sym_clkmgr_set_clk_source] = ACTIONS(1781),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1781),
    [sym_clkmgr_get_timezone] = ACTIONS(1781),
    [sym_clkmgr_set_timezone] = ACTIONS(1781),
    [sym_clkmgr_get_resync_period] = ACTIONS(1781),
    [sym_clkmgr_set_resync_period] = ACTIONS(1781),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_timeservers] = ACTIONS(1781),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_set_log_level] = ACTIONS(1781),
    [sym_get_log_level] = ACTIONS(1781),
    [sym_amx_log] = ACTIONS(1781),
    [sym_smtp_server_config_set] = ACTIONS(1781),
    [sym_smtp_server_config_get] = ACTIONS(1781),
    [sym_smtp_send] = ACTIONS(1781),
    [sym_validate_netlinx_account] = ACTIONS(1781),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1781),
    [sym_authenticate_certificate] = ACTIONS(1781),
    [sym_audit_netlinx_session_event] = ACTIONS(1781),
    [sym_audit_netlinx_generic_event] = ACTIONS(1781),
    [sym_get_available_flash_disk_space] = ACTIONS(1781),
    [sym_get_max_flash_disk_space] = ACTIONS(1781),
    [sym_data_create_feed] = ACTIONS(1781),
    [sym_data_delete_feed] = ACTIONS(1781),
    [sym_data_publish_feed] = ACTIONS(1781),
    [sym_data_get_published_feed] = ACTIONS(1781),
    [sym_data_add_record] = ACTIONS(1781),
    [sym_data_get_event_record] = ACTIONS(1781),
    [sym__wc_data_create_feed] = ACTIONS(1781),
    [sym__wc_data_add_record] = ACTIONS(1781),
    [sym__wc_data_get_event_record] = ACTIONS(1781),
    [sym_format] = ACTIONS(1781),
    [sym_astro_clock] = ACTIONS(1781),
    [sym_set_outdoor_temperature] = ACTIONS(1781),
    [sym_set_virtual_level_count] = ACTIONS(1781),
    [sym_set_virtual_channel_count] = ACTIONS(1781),
    [sym_set_virtual_port_count] = ACTIONS(1781),
    [sym__wc_encode_int] = ACTIONS(1781),
    [sym__wc_decode_int] = ACTIONS(1781),
    [sym__wc_int] = ACTIONS(1781),
    [sym__wc_to_ch_int] = ACTIONS(1781),
    [sym__ch_to_wc_int] = ACTIONS(1781),
    [sym__wc_find_string_int] = ACTIONS(1781),
    [sym__wc_left_string_int] = ACTIONS(1781),
    [sym__wc_lower_string_int] = ACTIONS(1781),
    [sym__wc_mid_string_int] = ACTIONS(1781),
    [sym__wc_remove_string_int] = ACTIONS(1781),
    [sym__wc_right_string_int] = ACTIONS(1781),
    [sym__wc_upper_string_int] = ACTIONS(1781),
    [sym__wc_compare_string_int] = ACTIONS(1781),
    [sym__wc_get_buffer_char_int] = ACTIONS(1781),
    [sym__wc_get_buffer_string_int] = ACTIONS(1781),
    [sym__wc_concat_string_int] = ACTIONS(1781),
    [sym__wc_file_open_int] = ACTIONS(1781),
    [sym__wc_file_close_int] = ACTIONS(1781),
    [sym__wc_file_read_int] = ACTIONS(1781),
    [sym__wc_file_read_line_int] = ACTIONS(1781),
    [sym__wc_file_write_int] = ACTIONS(1781),
    [sym__wc_file_write_line_int] = ACTIONS(1781),
    [aux_sym_preproc_include_token1] = ACTIONS(1781),
    [aux_sym_preproc_define_token1] = ACTIONS(1783),
    [aux_sym_preproc_warn_token1] = ACTIONS(1783),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1783),
    [anon_sym_LBRACK] = ACTIONS(1783),
    [anon_sym_LBRACE] = ACTIONS(1783),
    [anon_sym_RBRACE] = ACTIONS(1783),
    [anon_sym_LPAREN] = ACTIONS(1781),
    [anon_sym_BANG] = ACTIONS(1783),
    [anon_sym_TILDE] = ACTIONS(1783),
    [anon_sym_DASH] = ACTIONS(1781),
    [anon_sym_PLUS] = ACTIONS(1781),
    [anon_sym_DASH_DASH] = ACTIONS(1783),
    [anon_sym_PLUS_PLUS] = ACTIONS(1783),
    [anon_sym_DQUOTE] = ACTIONS(1783),
    [anon_sym_SQUOTE] = ACTIONS(1783),
    [sym_number_literal] = ACTIONS(1783),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(509)] = {
    [sym_identifier] = ACTIONS(1781),
    [sym_define_call_keyword] = ACTIONS(1781),
    [sym_define_function_keyword] = ACTIONS(1781),
    [sym_define_library_function_keyword] = ACTIONS(1781),
    [sym_define_module_keyword] = ACTIONS(1781),
    [sym_char_keyword] = ACTIONS(1781),
    [sym_widechar_keyword] = ACTIONS(1781),
    [sym_integer_keyword] = ACTIONS(1781),
    [sym_sinteger_keyword] = ACTIONS(1781),
    [sym_long_keyword] = ACTIONS(1781),
    [sym_slong_keyword] = ACTIONS(1781),
    [sym_float_keyword] = ACTIONS(1781),
    [sym_double_keyword] = ACTIONS(1781),
    [sym_variant_keyword] = ACTIONS(1781),
    [sym_variantarray_keyword] = ACTIONS(1781),
    [sym_local_var_keyword] = ACTIONS(1781),
    [sym_stack_var_keyword] = ACTIONS(1781),
    [sym_constant_keyword] = ACTIONS(1781),
    [sym_volatile_keyword] = ACTIONS(1781),
    [sym_non_volatile_keyword] = ACTIONS(1781),
    [sym_persistent_keyword] = ACTIONS(1781),
    [sym_dev_keyword] = ACTIONS(1781),
    [sym_devlev_keyword] = ACTIONS(1781),
    [sym_devchan_keyword] = ACTIONS(1781),
    [sym_if_keyword] = ACTIONS(1781),
    [sym_else_keyword] = ACTIONS(1781),
    [sym_switch_keyword] = ACTIONS(1781),
    [sym_case_keyword] = ACTIONS(1781),
    [sym_default_keyword] = ACTIONS(1781),
    [sym_while_keyword] = ACTIONS(1781),
    [sym_for_keyword] = ACTIONS(1781),
    [sym_break_keyword] = ACTIONS(1781),
    [sym_continue_keyword] = ACTIONS(1781),
    [sym_return_keyword] = ACTIONS(1781),
    [sym_select_keyword] = ACTIONS(1781),
    [sym_active_keyword] = ACTIONS(1781),
    [sym_struct_keyword] = ACTIONS(1781),
    [sym_structure_keyword] = ACTIONS(1781),
    [aux_sym_on_keyword_token1] = ACTIONS(1781),
    [aux_sym_off_keyword_token1] = ACTIONS(1781),
    [sym_send_level_keyword] = ACTIONS(1781),
    [sym_send_string_keyword] = ACTIONS(1781),
    [sym_send_command_keyword] = ACTIONS(1781),
    [sym_clear_buffer_keyword] = ACTIONS(1781),
    [sym_create_buffer_keyword] = ACTIONS(1781),
    [sym_create_multi_buffer_keyword] = ACTIONS(1781),
    [sym_call_keyword] = ACTIONS(1781),
    [sym_system_call_keyword] = ACTIONS(1781),
    [sym_devchan_to_keyword] = ACTIONS(1781),
    [sym_devchan_min_to_keyword] = ACTIONS(1781),
    [sym_devchan_total_off_keyword] = ACTIONS(1781),
    [sym_devchan_pulse_keyword] = ACTIONS(1781),
    [sym_wait_keyword] = ACTIONS(1781),
    [sym_cancel_wait_keyword] = ACTIONS(1781),
    [sym_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_wait_until_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_keyword] = ACTIONS(1781),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1781),
    [sym_not] = ACTIONS(1781),
    [sym_bnot] = ACTIONS(1781),
    [sym___file__] = ACTIONS(1781),
    [sym___date__] = ACTIONS(1781),
    [sym___ldate__] = ACTIONS(1781),
    [sym___time__] = ACTIONS(1781),
    [sym___line__] = ACTIONS(1781),
    [sym___name__] = ACTIONS(1781),
    [sym___netlinx__] = ACTIONS(1781),
    [sym_netlinx_axi_version] = ACTIONS(1781),
    [sym_url_flg_tcp] = ACTIONS(1781),
    [sym_url_flg_accinfopresent] = ACTIONS(1781),
    [sym_url_flg_temp] = ACTIONS(1781),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1781),
    [sym_url_flg_stat_mask] = ACTIONS(1781),
    [sym_url_flg_stat_lookup] = ACTIONS(1781),
    [sym_url_flg_stat_connecting] = ACTIONS(1781),
    [sym_url_flg_stat_waiting] = ACTIONS(1781),
    [sym_url_flg_stat_connected] = ACTIONS(1781),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1781),
    [sym_first_virtual_device] = ACTIONS(1781),
    [sym_first_local_port] = ACTIONS(1781),
    [sym_dynamic_virtual_device] = ACTIONS(1781),
    [sym_timeline_once] = ACTIONS(1781),
    [sym_timeline_repeat] = ACTIONS(1781),
    [sym_timeline_absolute] = ACTIONS(1781),
    [sym_timeline_relative] = ACTIONS(1781),
    [sym_do_push_timed_infinite] = ACTIONS(1781),
    [sym_source_type_no_address] = ACTIONS(1781),
    [sym_source_type_neuron_id] = ACTIONS(1781),
    [sym_source_type_ip_address] = ACTIONS(1781),
    [sym_source_type_axlink] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1781),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1781),
    [sym_source_type_ip_socket_address] = ACTIONS(1781),
    [sym_source_type_rs232] = ACTIONS(1781),
    [sym_source_type_internal] = ACTIONS(1781),
    [sym_source_type_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1781),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1781),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1781),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1781),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1781),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1781),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1781),
    [sym_true] = ACTIONS(1781),
    [sym_false] = ACTIONS(1781),
    [sym_file_read_only] = ACTIONS(1781),
    [sym_file_rw_new] = ACTIONS(1781),
    [sym_file_rw_append] = ACTIONS(1781),
    [sym_ip_tcp] = ACTIONS(1781),
    [sym_ip_udp] = ACTIONS(1781),
    [sym_ip_udp_2way] = ACTIONS(1781),
    [sym_xml_encode_types] = ACTIONS(1781),
    [sym_xml_encode_char_as_list] = ACTIONS(1781),
    [sym_xml_encode_le] = ACTIONS(1781),
    [sym_xml_decode_types] = ACTIONS(1781),
    [sym_xml_decode_no_preserve] = ACTIONS(1781),
    [sym_duet_dev_type_display_device] = ACTIONS(1781),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1781),
    [sym_duet_dev_type_io_device] = ACTIONS(1781),
    [sym_duet_dev_type_relay_device] = ACTIONS(1781),
    [sym_duet_dev_type_ups] = ACTIONS(1781),
    [sym_duet_dev_type_amplifier] = ACTIONS(1781),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1781),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1781),
    [sym_duet_dev_type_camera] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1781),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1781),
    [sym_duet_dev_type_dss] = ACTIONS(1781),
    [sym_duet_dev_type_dvr] = ACTIONS(1781),
    [sym_duet_dev_type_disc_device] = ACTIONS(1781),
    [sym_duet_dev_type_document_camera] = ACTIONS(1781),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1781),
    [sym_duet_dev_type_hvac] = ACTIONS(1781),
    [sym_duet_dev_type_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_light] = ACTIONS(1781),
    [sym_duet_dev_type_monitor] = ACTIONS(1781),
    [sym_duet_dev_type_motor] = ACTIONS(1781),
    [sym_duet_dev_type_multi_window] = ACTIONS(1781),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1781),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1781),
    [sym_duet_dev_type_receiver] = ACTIONS(1781),
    [sym_duet_dev_type_security_system] = ACTIONS(1781),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1781),
    [sym_duet_dev_type_settop_box] = ACTIONS(1781),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1781),
    [sym_duet_dev_type_switcher] = ACTIONS(1781),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1781),
    [sym_duet_dev_type_tv] = ACTIONS(1781),
    [sym_duet_dev_type_utility] = ACTIONS(1781),
    [sym_duet_dev_type_vcr] = ACTIONS(1781),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1781),
    [sym_duet_dev_type_video_processor] = ACTIONS(1781),
    [sym_duet_dev_type_video_projector] = ACTIONS(1781),
    [sym_duet_dev_type_video_wall] = ACTIONS(1781),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1781),
    [sym_duet_dev_type_weather] = ACTIONS(1781),
    [sym_duet_dev_not_polled] = ACTIONS(1781),
    [sym_duet_dev_polled] = ACTIONS(1781),
    [sym_clkmgr_mode_network] = ACTIONS(1781),
    [sym_clkmgr_mode_standalone] = ACTIONS(1781),
    [sym_normal_standby] = ACTIONS(1781),
    [sym_normal_wake] = ACTIONS(1781),
    [sym_ip_multicast_ttl_option] = ACTIONS(1781),
    [sym_ip_ttl_subnet] = ACTIONS(1781),
    [sym_ip_ttl_site] = ACTIONS(1781),
    [sym_ip_ttl_region] = ACTIONS(1781),
    [sym_ip_ttl_continent] = ACTIONS(1781),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1781),
    [sym_ip_nodelay_on] = ACTIONS(1781),
    [sym_ip_nodelay_off] = ACTIONS(1781),
    [sym_amx_error] = ACTIONS(1781),
    [sym_amx_warning] = ACTIONS(1781),
    [sym_amx_info] = ACTIONS(1781),
    [sym_amx_debug] = ACTIONS(1781),
    [sym_netlinx_login_success] = ACTIONS(1781),
    [sym_netlinx_login_fail] = ACTIONS(1781),
    [sym_netlinx_logout] = ACTIONS(1781),
    [sym_null_str] = ACTIONS(1781),
    [sym_smtp_address] = ACTIONS(1781),
    [sym_smtp_port_number] = ACTIONS(1781),
    [sym_smtp_username] = ACTIONS(1781),
    [sym_smtp_password] = ACTIONS(1781),
    [sym_smtp_require_tls] = ACTIONS(1781),
    [sym_smtp_from] = ACTIONS(1781),
    [sym_smtp_tls_true] = ACTIONS(1781),
    [sym_smtp_tls_false] = ACTIONS(1781),
    [sym_listview_on_row_select_event] = ACTIONS(1781),
    [sym_data_structure_unknown] = ACTIONS(1781),
    [sym_data_structure_datafeed] = ACTIONS(1781),
    [sym_data_structure_datarecordset] = ACTIONS(1781),
    [sym_data_structure_datarecord] = ACTIONS(1781),
    [sym_data_structure_datafield] = ACTIONS(1781),
    [sym_data_max_content_field_count] = ACTIONS(1781),
    [sym_data_max_metadata_field_count] = ACTIONS(1781),
    [sym_data_max_id_length] = ACTIONS(1781),
    [sym_data_max_name_length] = ACTIONS(1781),
    [sym_data_max_description_length] = ACTIONS(1781),
    [sym_data_max_type_length] = ACTIONS(1781),
    [sym_data_max_format_length] = ACTIONS(1781),
    [sym_data_max_source_length] = ACTIONS(1781),
    [sym_data_max_value_length] = ACTIONS(1781),
    [sym_data_max_label_length] = ACTIONS(1781),
    [sym_data_max_url_length] = ACTIONS(1781),
    [sym_max_last_login_info_length] = ACTIONS(1781),
    [sym_data_type_unknown] = ACTIONS(1781),
    [sym_data_type_string] = ACTIONS(1781),
    [sym_data_type_datetime] = ACTIONS(1781),
    [sym_data_type_date] = ACTIONS(1781),
    [sym_data_type_time] = ACTIONS(1781),
    [sym_data_type_image] = ACTIONS(1781),
    [sym_data_format_url] = ACTIONS(1781),
    [sym_data_format_phone] = ACTIONS(1781),
    [sym_data_format_email] = ACTIONS(1781),
    [sym_data_format_iso8601] = ACTIONS(1781),
    [sym_auditlog_priv] = ACTIONS(1781),
    [sym_configuration_priv] = ACTIONS(1781),
    [sym_deviceconfig_priv] = ACTIONS(1781),
    [sym_ftp_priv] = ACTIONS(1781),
    [sym_http_priv] = ACTIONS(1781),
    [sym_networkconfig_priv] = ACTIONS(1781),
    [sym_programport_priv] = ACTIONS(1781),
    [sym_remoteui_priv] = ACTIONS(1781),
    [sym_securitycontrol_priv] = ACTIONS(1781),
    [sym_softwaremanagement_priv] = ACTIONS(1781),
    [sym_terminal_priv] = ACTIONS(1781),
    [sym_tpadmin_priv] = ACTIONS(1781),
    [sym_user1_priv] = ACTIONS(1781),
    [sym_user2_priv] = ACTIONS(1781),
    [sym_user3_priv] = ACTIONS(1781),
    [sym_user4_priv] = ACTIONS(1781),
    [sym_usermanagement_priv] = ACTIONS(1781),
    [sym_valid_account] = ACTIONS(1781),
    [sym_err_name_invalid] = ACTIONS(1781),
    [sym_err_password_invalid] = ACTIONS(1781),
    [sym_err_invalid_account] = ACTIONS(1781),
    [sym_err_no_authorization] = ACTIONS(1781),
    [sym_err_invalid_parameter] = ACTIONS(1781),
    [sym_err_user_locked_out] = ACTIONS(1781),
    [sym_err_user_account_expired] = ACTIONS(1781),
    [sym_err_general_error] = ACTIONS(1781),
    [sym_err_authentication_error] = ACTIONS(1781),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1781),
    [sym_tls_validate_certificate] = ACTIONS(1781),
    [sym_tdata] = ACTIONS(1781),
    [sym_tchannel] = ACTIONS(1781),
    [sym_tlevel] = ACTIONS(1781),
    [sym_tbutton] = ACTIONS(1781),
    [sym_ttimeline] = ACTIONS(1781),
    [sym_tcustom] = ACTIONS(1781),
    [sym_url_struct] = ACTIONS(1781),
    [sym_dns_struct] = ACTIONS(1781),
    [sym_ip_address_struct] = ACTIONS(1781),
    [sym_dev_info_struct] = ACTIONS(1781),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1781),
    [sym_data_feed] = ACTIONS(1781),
    [sym_data_field] = ACTIONS(1781),
    [sym_data_record] = ACTIONS(1781),
    [sym_wc_data_feed] = ACTIONS(1781),
    [sym_wc_data_field] = ACTIONS(1781),
    [sym_wc_data_record] = ACTIONS(1781),
    [sym_last_login_info] = ACTIONS(1781),
    [sym____reserved___] = ACTIONS(1781),
    [sym_date] = ACTIONS(1781),
    [sym_day] = ACTIONS(1781),
    [sym_time] = ACTIONS(1781),
    [sym_ldate] = ACTIONS(1781),
    [sym_push_device] = ACTIONS(1781),
    [sym_push_channel] = ACTIONS(1781),
    [sym_push_devchan] = ACTIONS(1781),
    [sym_release_device] = ACTIONS(1781),
    [sym_release_channel] = ACTIONS(1781),
    [sym_release_devchan] = ACTIONS(1781),
    [sym_master_slot] = ACTIONS(1781),
    [sym_get_pulse_time] = ACTIONS(1781),
    [sym_get_timer] = ACTIONS(1781),
    [sym_master_sn] = ACTIONS(1781),
    [sym_system_number] = ACTIONS(1781),
    [sym_button] = ACTIONS(1781),
    [sym_dv_channel] = ACTIONS(1781),
    [sym_data] = ACTIONS(1781),
    [sym_level] = ACTIONS(1781),
    [sym_timeline] = ACTIONS(1781),
    [sym_channel] = ACTIONS(1781),
    [sym_custom] = ACTIONS(1781),
    [sym_atoi] = ACTIONS(1781),
    [sym_atol] = ACTIONS(1781),
    [sym_atof] = ACTIONS(1781),
    [sym_itoa] = ACTIONS(1781),
    [sym_itohex] = ACTIONS(1781),
    [sym_hextoi] = ACTIONS(1781),
    [sym_ftoa] = ACTIONS(1781),
    [sym_find_string] = ACTIONS(1781),
    [sym_left_string] = ACTIONS(1781),
    [sym_right_string] = ACTIONS(1781),
    [sym_mid_string] = ACTIONS(1781),
    [sym_remove_string] = ACTIONS(1781),
    [sym_get_buffer_string] = ACTIONS(1781),
    [sym_get_buffer_char] = ACTIONS(1781),
    [sym_get_multi_buffer_string] = ACTIONS(1781),
    [sym_length_string] = ACTIONS(1781),
    [sym_max_length_string] = ACTIONS(1781),
    [sym_set_length_string] = ACTIONS(1781),
    [sym_compare_string] = ACTIONS(1781),
    [sym_upper_string] = ACTIONS(1781),
    [sym_lower_string] = ACTIONS(1781),
    [sym_get_last] = ACTIONS(1781),
    [sym_date_to_day] = ACTIONS(1781),
    [sym_date_to_month] = ACTIONS(1781),
    [sym_date_to_year] = ACTIONS(1781),
    [sym_time_to_hour] = ACTIONS(1781),
    [sym_time_to_minute] = ACTIONS(1781),
    [sym_time_to_second] = ACTIONS(1781),
    [sym_day_of_week] = ACTIONS(1781),
    [sym_set_pulse_time] = ACTIONS(1781),
    [sym_set_timer] = ACTIONS(1781),
    [sym_file_open] = ACTIONS(1781),
    [sym_file_close] = ACTIONS(1781),
    [sym_file_read] = ACTIONS(1781),
    [sym_file_read_line] = ACTIONS(1781),
    [sym_file_write] = ACTIONS(1781),
    [sym_file_write_line] = ACTIONS(1781),
    [sym_file_copy] = ACTIONS(1781),
    [sym_file_delete] = ACTIONS(1781),
    [sym_file_rename] = ACTIONS(1781),
    [sym_file_dir] = ACTIONS(1781),
    [sym_file_getdir] = ACTIONS(1781),
    [sym_file_setdir] = ACTIONS(1781),
    [sym_file_create_dir] = ACTIONS(1781),
    [sym_file_remove_dir] = ACTIONS(1781),
    [sym_file_seek] = ACTIONS(1781),
    [sym_length_array] = ACTIONS(1781),
    [sym_max_length_array] = ACTIONS(1781),
    [sym_set_length_array] = ACTIONS(1781),
    [sym_ip_client_open] = ACTIONS(1781),
    [sym_ip_client_close] = ACTIONS(1781),
    [sym_ip_server_open] = ACTIONS(1781),
    [sym_ip_server_close] = ACTIONS(1781),
    [sym_ip_bound_client_open] = ACTIONS(1781),
    [sym_ip_mc_server_open] = ACTIONS(1781),
    [sym_ip_set_option] = ACTIONS(1781),
    [sym_ssh_client_open] = ACTIONS(1781),
    [sym_ssh_client_close] = ACTIONS(1781),
    [sym_tls_client_open] = ACTIONS(1781),
    [sym_tls_client_close] = ACTIONS(1781),
    [sym_do_push] = ACTIONS(1781),
    [sym_do_push_timed] = ACTIONS(1781),
    [sym_do_release] = ACTIONS(1781),
    [sym_device_id] = ACTIONS(1781),
    [sym_device_info] = ACTIONS(1781),
    [sym_device_id_string] = ACTIONS(1781),
    [sym_device_standby] = ACTIONS(1781),
    [sym_device_wake] = ACTIONS(1781),
    [sym_timeline_create] = ACTIONS(1781),
    [sym_timeline_kill] = ACTIONS(1781),
    [sym_timeline_reload] = ACTIONS(1781),
    [sym_timeline_pause] = ACTIONS(1781),
    [sym_timeline_restart] = ACTIONS(1781),
    [sym_timeline_set] = ACTIONS(1781),
    [sym_timeline_get] = ACTIONS(1781),
    [sym_timeline_active] = ACTIONS(1781),
    [sym_timeline_dynamic_id] = ACTIONS(1781),
    [sym_variable_to_string] = ACTIONS(1781),
    [sym_string_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_string] = ACTIONS(1781),
    [sym_type_cast] = ACTIONS(1781),
    [sym_variable_to_xml] = ACTIONS(1781),
    [sym_xml_to_variable] = ACTIONS(1781),
    [sym_length_variable_to_xml] = ACTIONS(1781),
    [sym_redirect_string] = ACTIONS(1781),
    [sym_random_number] = ACTIONS(1781),
    [sym_get_unique_id] = ACTIONS(1781),
    [sym_get_system_number] = ACTIONS(1781),
    [sym_set_system_number] = ACTIONS(1781),
    [sym_get_serial_number] = ACTIONS(1781),
    [sym_get_master_build] = ACTIONS(1781),
    [sym_set_validation_code] = ACTIONS(1781),
    [sym_reboot] = ACTIONS(1781),
    [sym_load_duet_module] = ACTIONS(1781),
    [sym_unload_duet_module] = ACTIONS(1781),
    [sym_duet_mem_size_set] = ACTIONS(1781),
    [sym_duet_mem_size_get] = ACTIONS(1781),
    [sym_dynamic_application_device] = ACTIONS(1781),
    [sym_dynamic_polled_port] = ACTIONS(1781),
    [sym_static_port_binding] = ACTIONS(1781),
    [sym_static_ip_binding] = ACTIONS(1781),
    [sym_get_dns_list] = ACTIONS(1781),
    [sym_set_dns_list] = ACTIONS(1781),
    [sym_get_ip_address] = ACTIONS(1781),
    [sym_set_ip_address] = ACTIONS(1781),
    [sym_get_url_list] = ACTIONS(1781),
    [sym_add_url_entry] = ACTIONS(1781),
    [sym_delete_url_entry] = ACTIONS(1781),
    [sym_abs_value] = ACTIONS(1781),
    [sym_max_value] = ACTIONS(1781),
    [sym_min_value] = ACTIONS(1781),
    [sym_exp_value] = ACTIONS(1781),
    [sym_log_value] = ACTIONS(1781),
    [sym_log10_value] = ACTIONS(1781),
    [sym_power_value] = ACTIONS(1781),
    [sym_sqrt_value] = ACTIONS(1781),
    [sym_raw_be] = ACTIONS(1781),
    [sym_raw_le] = ACTIONS(1781),
    [sym_internal_queue_size_set] = ACTIONS(1781),
    [sym_internal_queue_size_get] = ACTIONS(1781),
    [sym_internal_threshold_set] = ACTIONS(1781),
    [sym_internal_threshold_get] = ACTIONS(1781),
    [sym_rebuilt_event] = ACTIONS(1781),
    [sym_do_custom_event] = ACTIONS(1781),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1781),
    [sym_clkmgr_set_clk_source] = ACTIONS(1781),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1781),
    [sym_clkmgr_get_timezone] = ACTIONS(1781),
    [sym_clkmgr_set_timezone] = ACTIONS(1781),
    [sym_clkmgr_get_resync_period] = ACTIONS(1781),
    [sym_clkmgr_set_resync_period] = ACTIONS(1781),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1781),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_timeservers] = ACTIONS(1781),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1781),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1781),
    [sym_set_log_level] = ACTIONS(1781),
    [sym_get_log_level] = ACTIONS(1781),
    [sym_amx_log] = ACTIONS(1781),
    [sym_smtp_server_config_set] = ACTIONS(1781),
    [sym_smtp_server_config_get] = ACTIONS(1781),
    [sym_smtp_send] = ACTIONS(1781),
    [sym_validate_netlinx_account] = ACTIONS(1781),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1781),
    [sym_authenticate_certificate] = ACTIONS(1781),
    [sym_audit_netlinx_session_event] = ACTIONS(1781),
    [sym_audit_netlinx_generic_event] = ACTIONS(1781),
    [sym_get_available_flash_disk_space] = ACTIONS(1781),
    [sym_get_max_flash_disk_space] = ACTIONS(1781),
    [sym_data_create_feed] = ACTIONS(1781),
    [sym_data_delete_feed] = ACTIONS(1781),
    [sym_data_publish_feed] = ACTIONS(1781),
    [sym_data_get_published_feed] = ACTIONS(1781),
    [sym_data_add_record] = ACTIONS(1781),
    [sym_data_get_event_record] = ACTIONS(1781),
    [sym__wc_data_create_feed] = ACTIONS(1781),
    [sym__wc_data_add_record] = ACTIONS(1781),
    [sym__wc_data_get_event_record] = ACTIONS(1781),
    [sym_format] = ACTIONS(1781),
    [sym_astro_clock] = ACTIONS(1781),
    [sym_set_outdoor_temperature] = ACTIONS(1781),
    [sym_set_virtual_level_count] = ACTIONS(1781),
    [sym_set_virtual_channel_count] = ACTIONS(1781),
    [sym_set_virtual_port_count] = ACTIONS(1781),
    [sym__wc_encode_int] = ACTIONS(1781),
    [sym__wc_decode_int] = ACTIONS(1781),
    [sym__wc_int] = ACTIONS(1781),
    [sym__wc_to_ch_int] = ACTIONS(1781),
    [sym__ch_to_wc_int] = ACTIONS(1781),
    [sym__wc_find_string_int] = ACTIONS(1781),
    [sym__wc_left_string_int] = ACTIONS(1781),
    [sym__wc_lower_string_int] = ACTIONS(1781),
    [sym__wc_mid_string_int] = ACTIONS(1781),
    [sym__wc_remove_string_int] = ACTIONS(1781),
    [sym__wc_right_string_int] = ACTIONS(1781),
    [sym__wc_upper_string_int] = ACTIONS(1781),
    [sym__wc_compare_string_int] = ACTIONS(1781),
    [sym__wc_get_buffer_char_int] = ACTIONS(1781),
    [sym__wc_get_buffer_string_int] = ACTIONS(1781),
    [sym__wc_concat_string_int] = ACTIONS(1781),
    [sym__wc_file_open_int] = ACTIONS(1781),
    [sym__wc_file_close_int] = ACTIONS(1781),
    [sym__wc_file_read_int] = ACTIONS(1781),
    [sym__wc_file_read_line_int] = ACTIONS(1781),
    [sym__wc_file_write_int] = ACTIONS(1781),
    [sym__wc_file_write_line_int] = ACTIONS(1781),
    [aux_sym_preproc_include_token1] = ACTIONS(1781),
    [aux_sym_preproc_define_token1] = ACTIONS(1783),
    [aux_sym_preproc_warn_token1] = ACTIONS(1783),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1783),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1783),
    [anon_sym_LBRACK] = ACTIONS(1783),
    [anon_sym_LBRACE] = ACTIONS(1783),
    [anon_sym_RBRACE] = ACTIONS(1783),
    [anon_sym_LPAREN] = ACTIONS(1781),
    [anon_sym_BANG] = ACTIONS(1783),
    [anon_sym_TILDE] = ACTIONS(1783),
    [anon_sym_DASH] = ACTIONS(1781),
    [anon_sym_PLUS] = ACTIONS(1781),
    [anon_sym_DASH_DASH] = ACTIONS(1783),
    [anon_sym_PLUS_PLUS] = ACTIONS(1783),
    [anon_sym_DQUOTE] = ACTIONS(1783),
    [anon_sym_SQUOTE] = ACTIONS(1783),
    [sym_number_literal] = ACTIONS(1783),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(510)] = {
    [sym_identifier] = ACTIONS(1743),
    [sym_define_call_keyword] = ACTIONS(1743),
    [sym_define_function_keyword] = ACTIONS(1743),
    [sym_define_library_function_keyword] = ACTIONS(1743),
    [sym_define_module_keyword] = ACTIONS(1743),
    [sym_char_keyword] = ACTIONS(1743),
    [sym_widechar_keyword] = ACTIONS(1743),
    [sym_integer_keyword] = ACTIONS(1743),
    [sym_sinteger_keyword] = ACTIONS(1743),
    [sym_long_keyword] = ACTIONS(1743),
    [sym_slong_keyword] = ACTIONS(1743),
    [sym_float_keyword] = ACTIONS(1743),
    [sym_double_keyword] = ACTIONS(1743),
    [sym_variant_keyword] = ACTIONS(1743),
    [sym_variantarray_keyword] = ACTIONS(1743),
    [sym_local_var_keyword] = ACTIONS(1743),
    [sym_stack_var_keyword] = ACTIONS(1743),
    [sym_constant_keyword] = ACTIONS(1743),
    [sym_volatile_keyword] = ACTIONS(1743),
    [sym_non_volatile_keyword] = ACTIONS(1743),
    [sym_persistent_keyword] = ACTIONS(1743),
    [sym_dev_keyword] = ACTIONS(1743),
    [sym_devlev_keyword] = ACTIONS(1743),
    [sym_devchan_keyword] = ACTIONS(1743),
    [sym_if_keyword] = ACTIONS(1743),
    [sym_else_keyword] = ACTIONS(1743),
    [sym_switch_keyword] = ACTIONS(1743),
    [sym_case_keyword] = ACTIONS(1743),
    [sym_default_keyword] = ACTIONS(1743),
    [sym_while_keyword] = ACTIONS(1743),
    [sym_for_keyword] = ACTIONS(1743),
    [sym_break_keyword] = ACTIONS(1743),
    [sym_continue_keyword] = ACTIONS(1743),
    [sym_return_keyword] = ACTIONS(1743),
    [sym_select_keyword] = ACTIONS(1743),
    [sym_active_keyword] = ACTIONS(1743),
    [sym_struct_keyword] = ACTIONS(1743),
    [sym_structure_keyword] = ACTIONS(1743),
    [aux_sym_on_keyword_token1] = ACTIONS(1743),
    [aux_sym_off_keyword_token1] = ACTIONS(1743),
    [sym_send_level_keyword] = ACTIONS(1743),
    [sym_send_string_keyword] = ACTIONS(1743),
    [sym_send_command_keyword] = ACTIONS(1743),
    [sym_clear_buffer_keyword] = ACTIONS(1743),
    [sym_create_buffer_keyword] = ACTIONS(1743),
    [sym_create_multi_buffer_keyword] = ACTIONS(1743),
    [sym_call_keyword] = ACTIONS(1743),
    [sym_system_call_keyword] = ACTIONS(1743),
    [sym_devchan_to_keyword] = ACTIONS(1743),
    [sym_devchan_min_to_keyword] = ACTIONS(1743),
    [sym_devchan_total_off_keyword] = ACTIONS(1743),
    [sym_devchan_pulse_keyword] = ACTIONS(1743),
    [sym_wait_keyword] = ACTIONS(1743),
    [sym_cancel_wait_keyword] = ACTIONS(1743),
    [sym_wait_until_keyword] = ACTIONS(1743),
    [sym_cancel_wait_until_keyword] = ACTIONS(1743),
    [sym_cancel_all_wait_keyword] = ACTIONS(1743),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1743),
    [sym_not] = ACTIONS(1743),
    [sym_bnot] = ACTIONS(1743),
    [sym___file__] = ACTIONS(1743),
    [sym___date__] = ACTIONS(1743),
    [sym___ldate__] = ACTIONS(1743),
    [sym___time__] = ACTIONS(1743),
    [sym___line__] = ACTIONS(1743),
    [sym___name__] = ACTIONS(1743),
    [sym___netlinx__] = ACTIONS(1743),
    [sym_netlinx_axi_version] = ACTIONS(1743),
    [sym_url_flg_tcp] = ACTIONS(1743),
    [sym_url_flg_accinfopresent] = ACTIONS(1743),
    [sym_url_flg_temp] = ACTIONS(1743),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1743),
    [sym_url_flg_stat_mask] = ACTIONS(1743),
    [sym_url_flg_stat_lookup] = ACTIONS(1743),
    [sym_url_flg_stat_connecting] = ACTIONS(1743),
    [sym_url_flg_stat_waiting] = ACTIONS(1743),
    [sym_url_flg_stat_connected] = ACTIONS(1743),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1743),
    [sym_first_virtual_device] = ACTIONS(1743),
    [sym_first_local_port] = ACTIONS(1743),
    [sym_dynamic_virtual_device] = ACTIONS(1743),
    [sym_timeline_once] = ACTIONS(1743),
    [sym_timeline_repeat] = ACTIONS(1743),
    [sym_timeline_absolute] = ACTIONS(1743),
    [sym_timeline_relative] = ACTIONS(1743),
    [sym_do_push_timed_infinite] = ACTIONS(1743),
    [sym_source_type_no_address] = ACTIONS(1743),
    [sym_source_type_neuron_id] = ACTIONS(1743),
    [sym_source_type_ip_address] = ACTIONS(1743),
    [sym_source_type_axlink] = ACTIONS(1743),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1743),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1743),
    [sym_source_type_ip_socket_address] = ACTIONS(1743),
    [sym_source_type_rs232] = ACTIONS(1743),
    [sym_source_type_internal] = ACTIONS(1743),
    [sym_source_type_mac_address] = ACTIONS(1743),
    [sym_source_type_ipv4_port] = ACTIONS(1743),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1743),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1743),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1743),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1743),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1743),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1743),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1743),
    [sym_true] = ACTIONS(1743),
    [sym_false] = ACTIONS(1743),
    [sym_file_read_only] = ACTIONS(1743),
    [sym_file_rw_new] = ACTIONS(1743),
    [sym_file_rw_append] = ACTIONS(1743),
    [sym_ip_tcp] = ACTIONS(1743),
    [sym_ip_udp] = ACTIONS(1743),
    [sym_ip_udp_2way] = ACTIONS(1743),
    [sym_xml_encode_types] = ACTIONS(1743),
    [sym_xml_encode_char_as_list] = ACTIONS(1743),
    [sym_xml_encode_le] = ACTIONS(1743),
    [sym_xml_decode_types] = ACTIONS(1743),
    [sym_xml_decode_no_preserve] = ACTIONS(1743),
    [sym_duet_dev_type_display_device] = ACTIONS(1743),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1743),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1743),
    [sym_duet_dev_type_io_device] = ACTIONS(1743),
    [sym_duet_dev_type_relay_device] = ACTIONS(1743),
    [sym_duet_dev_type_ups] = ACTIONS(1743),
    [sym_duet_dev_type_amplifier] = ACTIONS(1743),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1743),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1743),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1743),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1743),
    [sym_duet_dev_type_camera] = ACTIONS(1743),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1743),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1743),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1743),
    [sym_duet_dev_type_dss] = ACTIONS(1743),
    [sym_duet_dev_type_dvr] = ACTIONS(1743),
    [sym_duet_dev_type_disc_device] = ACTIONS(1743),
    [sym_duet_dev_type_document_camera] = ACTIONS(1743),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1743),
    [sym_duet_dev_type_hvac] = ACTIONS(1743),
    [sym_duet_dev_type_keypad] = ACTIONS(1743),
    [sym_duet_dev_type_light] = ACTIONS(1743),
    [sym_duet_dev_type_monitor] = ACTIONS(1743),
    [sym_duet_dev_type_motor] = ACTIONS(1743),
    [sym_duet_dev_type_multi_window] = ACTIONS(1743),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1743),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1743),
    [sym_duet_dev_type_receiver] = ACTIONS(1743),
    [sym_duet_dev_type_security_system] = ACTIONS(1743),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1743),
    [sym_duet_dev_type_settop_box] = ACTIONS(1743),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1743),
    [sym_duet_dev_type_switcher] = ACTIONS(1743),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1743),
    [sym_duet_dev_type_tv] = ACTIONS(1743),
    [sym_duet_dev_type_utility] = ACTIONS(1743),
    [sym_duet_dev_type_vcr] = ACTIONS(1743),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1743),
    [sym_duet_dev_type_video_processor] = ACTIONS(1743),
    [sym_duet_dev_type_video_projector] = ACTIONS(1743),
    [sym_duet_dev_type_video_wall] = ACTIONS(1743),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1743),
    [sym_duet_dev_type_weather] = ACTIONS(1743),
    [sym_duet_dev_not_polled] = ACTIONS(1743),
    [sym_duet_dev_polled] = ACTIONS(1743),
    [sym_clkmgr_mode_network] = ACTIONS(1743),
    [sym_clkmgr_mode_standalone] = ACTIONS(1743),
    [sym_normal_standby] = ACTIONS(1743),
    [sym_normal_wake] = ACTIONS(1743),
    [sym_ip_multicast_ttl_option] = ACTIONS(1743),
    [sym_ip_ttl_subnet] = ACTIONS(1743),
    [sym_ip_ttl_site] = ACTIONS(1743),
    [sym_ip_ttl_region] = ACTIONS(1743),
    [sym_ip_ttl_continent] = ACTIONS(1743),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1743),
    [sym_ip_nodelay_on] = ACTIONS(1743),
    [sym_ip_nodelay_off] = ACTIONS(1743),
    [sym_amx_error] = ACTIONS(1743),
    [sym_amx_warning] = ACTIONS(1743),
    [sym_amx_info] = ACTIONS(1743),
    [sym_amx_debug] = ACTIONS(1743),
    [sym_netlinx_login_success] = ACTIONS(1743),
    [sym_netlinx_login_fail] = ACTIONS(1743),
    [sym_netlinx_logout] = ACTIONS(1743),
    [sym_null_str] = ACTIONS(1743),
    [sym_smtp_address] = ACTIONS(1743),
    [sym_smtp_port_number] = ACTIONS(1743),
    [sym_smtp_username] = ACTIONS(1743),
    [sym_smtp_password] = ACTIONS(1743),
    [sym_smtp_require_tls] = ACTIONS(1743),
    [sym_smtp_from] = ACTIONS(1743),
    [sym_smtp_tls_true] = ACTIONS(1743),
    [sym_smtp_tls_false] = ACTIONS(1743),
    [sym_listview_on_row_select_event] = ACTIONS(1743),
    [sym_data_structure_unknown] = ACTIONS(1743),
    [sym_data_structure_datafeed] = ACTIONS(1743),
    [sym_data_structure_datarecordset] = ACTIONS(1743),
    [sym_data_structure_datarecord] = ACTIONS(1743),
    [sym_data_structure_datafield] = ACTIONS(1743),
    [sym_data_max_content_field_count] = ACTIONS(1743),
    [sym_data_max_metadata_field_count] = ACTIONS(1743),
    [sym_data_max_id_length] = ACTIONS(1743),
    [sym_data_max_name_length] = ACTIONS(1743),
    [sym_data_max_description_length] = ACTIONS(1743),
    [sym_data_max_type_length] = ACTIONS(1743),
    [sym_data_max_format_length] = ACTIONS(1743),
    [sym_data_max_source_length] = ACTIONS(1743),
    [sym_data_max_value_length] = ACTIONS(1743),
    [sym_data_max_label_length] = ACTIONS(1743),
    [sym_data_max_url_length] = ACTIONS(1743),
    [sym_max_last_login_info_length] = ACTIONS(1743),
    [sym_data_type_unknown] = ACTIONS(1743),
    [sym_data_type_string] = ACTIONS(1743),
    [sym_data_type_datetime] = ACTIONS(1743),
    [sym_data_type_date] = ACTIONS(1743),
    [sym_data_type_time] = ACTIONS(1743),
    [sym_data_type_image] = ACTIONS(1743),
    [sym_data_format_url] = ACTIONS(1743),
    [sym_data_format_phone] = ACTIONS(1743),
    [sym_data_format_email] = ACTIONS(1743),
    [sym_data_format_iso8601] = ACTIONS(1743),
    [sym_auditlog_priv] = ACTIONS(1743),
    [sym_configuration_priv] = ACTIONS(1743),
    [sym_deviceconfig_priv] = ACTIONS(1743),
    [sym_ftp_priv] = ACTIONS(1743),
    [sym_http_priv] = ACTIONS(1743),
    [sym_networkconfig_priv] = ACTIONS(1743),
    [sym_programport_priv] = ACTIONS(1743),
    [sym_remoteui_priv] = ACTIONS(1743),
    [sym_securitycontrol_priv] = ACTIONS(1743),
    [sym_softwaremanagement_priv] = ACTIONS(1743),
    [sym_terminal_priv] = ACTIONS(1743),
    [sym_tpadmin_priv] = ACTIONS(1743),
    [sym_user1_priv] = ACTIONS(1743),
    [sym_user2_priv] = ACTIONS(1743),
    [sym_user3_priv] = ACTIONS(1743),
    [sym_user4_priv] = ACTIONS(1743),
    [sym_usermanagement_priv] = ACTIONS(1743),
    [sym_valid_account] = ACTIONS(1743),
    [sym_err_name_invalid] = ACTIONS(1743),
    [sym_err_password_invalid] = ACTIONS(1743),
    [sym_err_invalid_account] = ACTIONS(1743),
    [sym_err_no_authorization] = ACTIONS(1743),
    [sym_err_invalid_parameter] = ACTIONS(1743),
    [sym_err_user_locked_out] = ACTIONS(1743),
    [sym_err_user_account_expired] = ACTIONS(1743),
    [sym_err_general_error] = ACTIONS(1743),
    [sym_err_authentication_error] = ACTIONS(1743),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1743),
    [sym_tls_validate_certificate] = ACTIONS(1743),
    [sym_tdata] = ACTIONS(1743),
    [sym_tchannel] = ACTIONS(1743),
    [sym_tlevel] = ACTIONS(1743),
    [sym_tbutton] = ACTIONS(1743),
    [sym_ttimeline] = ACTIONS(1743),
    [sym_tcustom] = ACTIONS(1743),
    [sym_url_struct] = ACTIONS(1743),
    [sym_dns_struct] = ACTIONS(1743),
    [sym_ip_address_struct] = ACTIONS(1743),
    [sym_dev_info_struct] = ACTIONS(1743),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1743),
    [sym_data_feed] = ACTIONS(1743),
    [sym_data_field] = ACTIONS(1743),
    [sym_data_record] = ACTIONS(1743),
    [sym_wc_data_feed] = ACTIONS(1743),
    [sym_wc_data_field] = ACTIONS(1743),
    [sym_wc_data_record] = ACTIONS(1743),
    [sym_last_login_info] = ACTIONS(1743),
    [sym____reserved___] = ACTIONS(1743),
    [sym_date] = ACTIONS(1743),
    [sym_day] = ACTIONS(1743),
    [sym_time] = ACTIONS(1743),
    [sym_ldate] = ACTIONS(1743),
    [sym_push_device] = ACTIONS(1743),
    [sym_push_channel] = ACTIONS(1743),
    [sym_push_devchan] = ACTIONS(1743),
    [sym_release_device] = ACTIONS(1743),
    [sym_release_channel] = ACTIONS(1743),
    [sym_release_devchan] = ACTIONS(1743),
    [sym_master_slot] = ACTIONS(1743),
    [sym_get_pulse_time] = ACTIONS(1743),
    [sym_get_timer] = ACTIONS(1743),
    [sym_master_sn] = ACTIONS(1743),
    [sym_system_number] = ACTIONS(1743),
    [sym_button] = ACTIONS(1743),
    [sym_dv_channel] = ACTIONS(1743),
    [sym_data] = ACTIONS(1743),
    [sym_level] = ACTIONS(1743),
    [sym_timeline] = ACTIONS(1743),
    [sym_channel] = ACTIONS(1743),
    [sym_custom] = ACTIONS(1743),
    [sym_atoi] = ACTIONS(1743),
    [sym_atol] = ACTIONS(1743),
    [sym_atof] = ACTIONS(1743),
    [sym_itoa] = ACTIONS(1743),
    [sym_itohex] = ACTIONS(1743),
    [sym_hextoi] = ACTIONS(1743),
    [sym_ftoa] = ACTIONS(1743),
    [sym_find_string] = ACTIONS(1743),
    [sym_left_string] = ACTIONS(1743),
    [sym_right_string] = ACTIONS(1743),
    [sym_mid_string] = ACTIONS(1743),
    [sym_remove_string] = ACTIONS(1743),
    [sym_get_buffer_string] = ACTIONS(1743),
    [sym_get_buffer_char] = ACTIONS(1743),
    [sym_get_multi_buffer_string] = ACTIONS(1743),
    [sym_length_string] = ACTIONS(1743),
    [sym_max_length_string] = ACTIONS(1743),
    [sym_set_length_string] = ACTIONS(1743),
    [sym_compare_string] = ACTIONS(1743),
    [sym_upper_string] = ACTIONS(1743),
    [sym_lower_string] = ACTIONS(1743),
    [sym_get_last] = ACTIONS(1743),
    [sym_date_to_day] = ACTIONS(1743),
    [sym_date_to_month] = ACTIONS(1743),
    [sym_date_to_year] = ACTIONS(1743),
    [sym_time_to_hour] = ACTIONS(1743),
    [sym_time_to_minute] = ACTIONS(1743),
    [sym_time_to_second] = ACTIONS(1743),
    [sym_day_of_week] = ACTIONS(1743),
    [sym_set_pulse_time] = ACTIONS(1743),
    [sym_set_timer] = ACTIONS(1743),
    [sym_file_open] = ACTIONS(1743),
    [sym_file_close] = ACTIONS(1743),
    [sym_file_read] = ACTIONS(1743),
    [sym_file_read_line] = ACTIONS(1743),
    [sym_file_write] = ACTIONS(1743),
    [sym_file_write_line] = ACTIONS(1743),
    [sym_file_copy] = ACTIONS(1743),
    [sym_file_delete] = ACTIONS(1743),
    [sym_file_rename] = ACTIONS(1743),
    [sym_file_dir] = ACTIONS(1743),
    [sym_file_getdir] = ACTIONS(1743),
    [sym_file_setdir] = ACTIONS(1743),
    [sym_file_create_dir] = ACTIONS(1743),
    [sym_file_remove_dir] = ACTIONS(1743),
    [sym_file_seek] = ACTIONS(1743),
    [sym_length_array] = ACTIONS(1743),
    [sym_max_length_array] = ACTIONS(1743),
    [sym_set_length_array] = ACTIONS(1743),
    [sym_ip_client_open] = ACTIONS(1743),
    [sym_ip_client_close] = ACTIONS(1743),
    [sym_ip_server_open] = ACTIONS(1743),
    [sym_ip_server_close] = ACTIONS(1743),
    [sym_ip_bound_client_open] = ACTIONS(1743),
    [sym_ip_mc_server_open] = ACTIONS(1743),
    [sym_ip_set_option] = ACTIONS(1743),
    [sym_ssh_client_open] = ACTIONS(1743),
    [sym_ssh_client_close] = ACTIONS(1743),
    [sym_tls_client_open] = ACTIONS(1743),
    [sym_tls_client_close] = ACTIONS(1743),
    [sym_do_push] = ACTIONS(1743),
    [sym_do_push_timed] = ACTIONS(1743),
    [sym_do_release] = ACTIONS(1743),
    [sym_device_id] = ACTIONS(1743),
    [sym_device_info] = ACTIONS(1743),
    [sym_device_id_string] = ACTIONS(1743),
    [sym_device_standby] = ACTIONS(1743),
    [sym_device_wake] = ACTIONS(1743),
    [sym_timeline_create] = ACTIONS(1743),
    [sym_timeline_kill] = ACTIONS(1743),
    [sym_timeline_reload] = ACTIONS(1743),
    [sym_timeline_pause] = ACTIONS(1743),
    [sym_timeline_restart] = ACTIONS(1743),
    [sym_timeline_set] = ACTIONS(1743),
    [sym_timeline_get] = ACTIONS(1743),
    [sym_timeline_active] = ACTIONS(1743),
    [sym_timeline_dynamic_id] = ACTIONS(1743),
    [sym_variable_to_string] = ACTIONS(1743),
    [sym_string_to_variable] = ACTIONS(1743),
    [sym_length_variable_to_string] = ACTIONS(1743),
    [sym_type_cast] = ACTIONS(1743),
    [sym_variable_to_xml] = ACTIONS(1743),
    [sym_xml_to_variable] = ACTIONS(1743),
    [sym_length_variable_to_xml] = ACTIONS(1743),
    [sym_redirect_string] = ACTIONS(1743),
    [sym_random_number] = ACTIONS(1743),
    [sym_get_unique_id] = ACTIONS(1743),
    [sym_get_system_number] = ACTIONS(1743),
    [sym_set_system_number] = ACTIONS(1743),
    [sym_get_serial_number] = ACTIONS(1743),
    [sym_get_master_build] = ACTIONS(1743),
    [sym_set_validation_code] = ACTIONS(1743),
    [sym_reboot] = ACTIONS(1743),
    [sym_load_duet_module] = ACTIONS(1743),
    [sym_unload_duet_module] = ACTIONS(1743),
    [sym_duet_mem_size_set] = ACTIONS(1743),
    [sym_duet_mem_size_get] = ACTIONS(1743),
    [sym_dynamic_application_device] = ACTIONS(1743),
    [sym_dynamic_polled_port] = ACTIONS(1743),
    [sym_static_port_binding] = ACTIONS(1743),
    [sym_static_ip_binding] = ACTIONS(1743),
    [sym_get_dns_list] = ACTIONS(1743),
    [sym_set_dns_list] = ACTIONS(1743),
    [sym_get_ip_address] = ACTIONS(1743),
    [sym_set_ip_address] = ACTIONS(1743),
    [sym_get_url_list] = ACTIONS(1743),
    [sym_add_url_entry] = ACTIONS(1743),
    [sym_delete_url_entry] = ACTIONS(1743),
    [sym_abs_value] = ACTIONS(1743),
    [sym_max_value] = ACTIONS(1743),
    [sym_min_value] = ACTIONS(1743),
    [sym_exp_value] = ACTIONS(1743),
    [sym_log_value] = ACTIONS(1743),
    [sym_log10_value] = ACTIONS(1743),
    [sym_power_value] = ACTIONS(1743),
    [sym_sqrt_value] = ACTIONS(1743),
    [sym_raw_be] = ACTIONS(1743),
    [sym_raw_le] = ACTIONS(1743),
    [sym_internal_queue_size_set] = ACTIONS(1743),
    [sym_internal_queue_size_get] = ACTIONS(1743),
    [sym_internal_threshold_set] = ACTIONS(1743),
    [sym_internal_threshold_get] = ACTIONS(1743),
    [sym_rebuilt_event] = ACTIONS(1743),
    [sym_do_custom_event] = ACTIONS(1743),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1743),
    [sym_clkmgr_set_clk_source] = ACTIONS(1743),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1743),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1743),
    [sym_clkmgr_get_timezone] = ACTIONS(1743),
    [sym_clkmgr_set_timezone] = ACTIONS(1743),
    [sym_clkmgr_get_resync_period] = ACTIONS(1743),
    [sym_clkmgr_set_resync_period] = ACTIONS(1743),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1743),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1743),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1743),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1743),
    [sym_clkmgr_get_timeservers] = ACTIONS(1743),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1743),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1743),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1743),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1743),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1743),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1743),
    [sym_set_log_level] = ACTIONS(1743),
    [sym_get_log_level] = ACTIONS(1743),
    [sym_amx_log] = ACTIONS(1743),
    [sym_smtp_server_config_set] = ACTIONS(1743),
    [sym_smtp_server_config_get] = ACTIONS(1743),
    [sym_smtp_send] = ACTIONS(1743),
    [sym_validate_netlinx_account] = ACTIONS(1743),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1743),
    [sym_authenticate_certificate] = ACTIONS(1743),
    [sym_audit_netlinx_session_event] = ACTIONS(1743),
    [sym_audit_netlinx_generic_event] = ACTIONS(1743),
    [sym_get_available_flash_disk_space] = ACTIONS(1743),
    [sym_get_max_flash_disk_space] = ACTIONS(1743),
    [sym_data_create_feed] = ACTIONS(1743),
    [sym_data_delete_feed] = ACTIONS(1743),
    [sym_data_publish_feed] = ACTIONS(1743),
    [sym_data_get_published_feed] = ACTIONS(1743),
    [sym_data_add_record] = ACTIONS(1743),
    [sym_data_get_event_record] = ACTIONS(1743),
    [sym__wc_data_create_feed] = ACTIONS(1743),
    [sym__wc_data_add_record] = ACTIONS(1743),
    [sym__wc_data_get_event_record] = ACTIONS(1743),
    [sym_format] = ACTIONS(1743),
    [sym_astro_clock] = ACTIONS(1743),
    [sym_set_outdoor_temperature] = ACTIONS(1743),
    [sym_set_virtual_level_count] = ACTIONS(1743),
    [sym_set_virtual_channel_count] = ACTIONS(1743),
    [sym_set_virtual_port_count] = ACTIONS(1743),
    [sym__wc_encode_int] = ACTIONS(1743),
    [sym__wc_decode_int] = ACTIONS(1743),
    [sym__wc_int] = ACTIONS(1743),
    [sym__wc_to_ch_int] = ACTIONS(1743),
    [sym__ch_to_wc_int] = ACTIONS(1743),
    [sym__wc_find_string_int] = ACTIONS(1743),
    [sym__wc_left_string_int] = ACTIONS(1743),
    [sym__wc_lower_string_int] = ACTIONS(1743),
    [sym__wc_mid_string_int] = ACTIONS(1743),
    [sym__wc_remove_string_int] = ACTIONS(1743),
    [sym__wc_right_string_int] = ACTIONS(1743),
    [sym__wc_upper_string_int] = ACTIONS(1743),
    [sym__wc_compare_string_int] = ACTIONS(1743),
    [sym__wc_get_buffer_char_int] = ACTIONS(1743),
    [sym__wc_get_buffer_string_int] = ACTIONS(1743),
    [sym__wc_concat_string_int] = ACTIONS(1743),
    [sym__wc_file_open_int] = ACTIONS(1743),
    [sym__wc_file_close_int] = ACTIONS(1743),
    [sym__wc_file_read_int] = ACTIONS(1743),
    [sym__wc_file_read_line_int] = ACTIONS(1743),
    [sym__wc_file_write_int] = ACTIONS(1743),
    [sym__wc_file_write_line_int] = ACTIONS(1743),
    [aux_sym_preproc_include_token1] = ACTIONS(1743),
    [aux_sym_preproc_define_token1] = ACTIONS(1745),
    [aux_sym_preproc_warn_token1] = ACTIONS(1745),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1745),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1745),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1745),
    [anon_sym_LBRACK] = ACTIONS(1745),
    [anon_sym_LBRACE] = ACTIONS(1745),
    [anon_sym_RBRACE] = ACTIONS(1745),
    [anon_sym_LPAREN] = ACTIONS(1743),
    [anon_sym_BANG] = ACTIONS(1745),
    [anon_sym_TILDE] = ACTIONS(1745),
    [anon_sym_DASH] = ACTIONS(1743),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_DASH_DASH] = ACTIONS(1745),
    [anon_sym_PLUS_PLUS] = ACTIONS(1745),
    [anon_sym_DQUOTE] = ACTIONS(1745),
    [anon_sym_SQUOTE] = ACTIONS(1745),
    [sym_number_literal] = ACTIONS(1745),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(511)] = {
    [sym_identifier] = ACTIONS(1747),
    [sym_define_call_keyword] = ACTIONS(1747),
    [sym_define_function_keyword] = ACTIONS(1747),
    [sym_define_library_function_keyword] = ACTIONS(1747),
    [sym_define_module_keyword] = ACTIONS(1747),
    [sym_char_keyword] = ACTIONS(1747),
    [sym_widechar_keyword] = ACTIONS(1747),
    [sym_integer_keyword] = ACTIONS(1747),
    [sym_sinteger_keyword] = ACTIONS(1747),
    [sym_long_keyword] = ACTIONS(1747),
    [sym_slong_keyword] = ACTIONS(1747),
    [sym_float_keyword] = ACTIONS(1747),
    [sym_double_keyword] = ACTIONS(1747),
    [sym_variant_keyword] = ACTIONS(1747),
    [sym_variantarray_keyword] = ACTIONS(1747),
    [sym_local_var_keyword] = ACTIONS(1747),
    [sym_stack_var_keyword] = ACTIONS(1747),
    [sym_constant_keyword] = ACTIONS(1747),
    [sym_volatile_keyword] = ACTIONS(1747),
    [sym_non_volatile_keyword] = ACTIONS(1747),
    [sym_persistent_keyword] = ACTIONS(1747),
    [sym_dev_keyword] = ACTIONS(1747),
    [sym_devlev_keyword] = ACTIONS(1747),
    [sym_devchan_keyword] = ACTIONS(1747),
    [sym_if_keyword] = ACTIONS(1747),
    [sym_else_keyword] = ACTIONS(1747),
    [sym_switch_keyword] = ACTIONS(1747),
    [sym_case_keyword] = ACTIONS(1747),
    [sym_default_keyword] = ACTIONS(1747),
    [sym_while_keyword] = ACTIONS(1747),
    [sym_for_keyword] = ACTIONS(1747),
    [sym_break_keyword] = ACTIONS(1747),
    [sym_continue_keyword] = ACTIONS(1747),
    [sym_return_keyword] = ACTIONS(1747),
    [sym_select_keyword] = ACTIONS(1747),
    [sym_active_keyword] = ACTIONS(1747),
    [sym_struct_keyword] = ACTIONS(1747),
    [sym_structure_keyword] = ACTIONS(1747),
    [aux_sym_on_keyword_token1] = ACTIONS(1747),
    [aux_sym_off_keyword_token1] = ACTIONS(1747),
    [sym_send_level_keyword] = ACTIONS(1747),
    [sym_send_string_keyword] = ACTIONS(1747),
    [sym_send_command_keyword] = ACTIONS(1747),
    [sym_clear_buffer_keyword] = ACTIONS(1747),
    [sym_create_buffer_keyword] = ACTIONS(1747),
    [sym_create_multi_buffer_keyword] = ACTIONS(1747),
    [sym_call_keyword] = ACTIONS(1747),
    [sym_system_call_keyword] = ACTIONS(1747),
    [sym_devchan_to_keyword] = ACTIONS(1747),
    [sym_devchan_min_to_keyword] = ACTIONS(1747),
    [sym_devchan_total_off_keyword] = ACTIONS(1747),
    [sym_devchan_pulse_keyword] = ACTIONS(1747),
    [sym_wait_keyword] = ACTIONS(1747),
    [sym_cancel_wait_keyword] = ACTIONS(1747),
    [sym_wait_until_keyword] = ACTIONS(1747),
    [sym_cancel_wait_until_keyword] = ACTIONS(1747),
    [sym_cancel_all_wait_keyword] = ACTIONS(1747),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1747),
    [sym_not] = ACTIONS(1747),
    [sym_bnot] = ACTIONS(1747),
    [sym___file__] = ACTIONS(1747),
    [sym___date__] = ACTIONS(1747),
    [sym___ldate__] = ACTIONS(1747),
    [sym___time__] = ACTIONS(1747),
    [sym___line__] = ACTIONS(1747),
    [sym___name__] = ACTIONS(1747),
    [sym___netlinx__] = ACTIONS(1747),
    [sym_netlinx_axi_version] = ACTIONS(1747),
    [sym_url_flg_tcp] = ACTIONS(1747),
    [sym_url_flg_accinfopresent] = ACTIONS(1747),
    [sym_url_flg_temp] = ACTIONS(1747),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1747),
    [sym_url_flg_stat_mask] = ACTIONS(1747),
    [sym_url_flg_stat_lookup] = ACTIONS(1747),
    [sym_url_flg_stat_connecting] = ACTIONS(1747),
    [sym_url_flg_stat_waiting] = ACTIONS(1747),
    [sym_url_flg_stat_connected] = ACTIONS(1747),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1747),
    [sym_first_virtual_device] = ACTIONS(1747),
    [sym_first_local_port] = ACTIONS(1747),
    [sym_dynamic_virtual_device] = ACTIONS(1747),
    [sym_timeline_once] = ACTIONS(1747),
    [sym_timeline_repeat] = ACTIONS(1747),
    [sym_timeline_absolute] = ACTIONS(1747),
    [sym_timeline_relative] = ACTIONS(1747),
    [sym_do_push_timed_infinite] = ACTIONS(1747),
    [sym_source_type_no_address] = ACTIONS(1747),
    [sym_source_type_neuron_id] = ACTIONS(1747),
    [sym_source_type_ip_address] = ACTIONS(1747),
    [sym_source_type_axlink] = ACTIONS(1747),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1747),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1747),
    [sym_source_type_ip_socket_address] = ACTIONS(1747),
    [sym_source_type_rs232] = ACTIONS(1747),
    [sym_source_type_internal] = ACTIONS(1747),
    [sym_source_type_mac_address] = ACTIONS(1747),
    [sym_source_type_ipv4_port] = ACTIONS(1747),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1747),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1747),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1747),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1747),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1747),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1747),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1747),
    [sym_true] = ACTIONS(1747),
    [sym_false] = ACTIONS(1747),
    [sym_file_read_only] = ACTIONS(1747),
    [sym_file_rw_new] = ACTIONS(1747),
    [sym_file_rw_append] = ACTIONS(1747),
    [sym_ip_tcp] = ACTIONS(1747),
    [sym_ip_udp] = ACTIONS(1747),
    [sym_ip_udp_2way] = ACTIONS(1747),
    [sym_xml_encode_types] = ACTIONS(1747),
    [sym_xml_encode_char_as_list] = ACTIONS(1747),
    [sym_xml_encode_le] = ACTIONS(1747),
    [sym_xml_decode_types] = ACTIONS(1747),
    [sym_xml_decode_no_preserve] = ACTIONS(1747),
    [sym_duet_dev_type_display_device] = ACTIONS(1747),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1747),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1747),
    [sym_duet_dev_type_io_device] = ACTIONS(1747),
    [sym_duet_dev_type_relay_device] = ACTIONS(1747),
    [sym_duet_dev_type_ups] = ACTIONS(1747),
    [sym_duet_dev_type_amplifier] = ACTIONS(1747),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1747),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1747),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1747),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1747),
    [sym_duet_dev_type_camera] = ACTIONS(1747),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1747),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1747),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1747),
    [sym_duet_dev_type_dss] = ACTIONS(1747),
    [sym_duet_dev_type_dvr] = ACTIONS(1747),
    [sym_duet_dev_type_disc_device] = ACTIONS(1747),
    [sym_duet_dev_type_document_camera] = ACTIONS(1747),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1747),
    [sym_duet_dev_type_hvac] = ACTIONS(1747),
    [sym_duet_dev_type_keypad] = ACTIONS(1747),
    [sym_duet_dev_type_light] = ACTIONS(1747),
    [sym_duet_dev_type_monitor] = ACTIONS(1747),
    [sym_duet_dev_type_motor] = ACTIONS(1747),
    [sym_duet_dev_type_multi_window] = ACTIONS(1747),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1747),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1747),
    [sym_duet_dev_type_receiver] = ACTIONS(1747),
    [sym_duet_dev_type_security_system] = ACTIONS(1747),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1747),
    [sym_duet_dev_type_settop_box] = ACTIONS(1747),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1747),
    [sym_duet_dev_type_switcher] = ACTIONS(1747),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1747),
    [sym_duet_dev_type_tv] = ACTIONS(1747),
    [sym_duet_dev_type_utility] = ACTIONS(1747),
    [sym_duet_dev_type_vcr] = ACTIONS(1747),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1747),
    [sym_duet_dev_type_video_processor] = ACTIONS(1747),
    [sym_duet_dev_type_video_projector] = ACTIONS(1747),
    [sym_duet_dev_type_video_wall] = ACTIONS(1747),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1747),
    [sym_duet_dev_type_weather] = ACTIONS(1747),
    [sym_duet_dev_not_polled] = ACTIONS(1747),
    [sym_duet_dev_polled] = ACTIONS(1747),
    [sym_clkmgr_mode_network] = ACTIONS(1747),
    [sym_clkmgr_mode_standalone] = ACTIONS(1747),
    [sym_normal_standby] = ACTIONS(1747),
    [sym_normal_wake] = ACTIONS(1747),
    [sym_ip_multicast_ttl_option] = ACTIONS(1747),
    [sym_ip_ttl_subnet] = ACTIONS(1747),
    [sym_ip_ttl_site] = ACTIONS(1747),
    [sym_ip_ttl_region] = ACTIONS(1747),
    [sym_ip_ttl_continent] = ACTIONS(1747),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1747),
    [sym_ip_nodelay_on] = ACTIONS(1747),
    [sym_ip_nodelay_off] = ACTIONS(1747),
    [sym_amx_error] = ACTIONS(1747),
    [sym_amx_warning] = ACTIONS(1747),
    [sym_amx_info] = ACTIONS(1747),
    [sym_amx_debug] = ACTIONS(1747),
    [sym_netlinx_login_success] = ACTIONS(1747),
    [sym_netlinx_login_fail] = ACTIONS(1747),
    [sym_netlinx_logout] = ACTIONS(1747),
    [sym_null_str] = ACTIONS(1747),
    [sym_smtp_address] = ACTIONS(1747),
    [sym_smtp_port_number] = ACTIONS(1747),
    [sym_smtp_username] = ACTIONS(1747),
    [sym_smtp_password] = ACTIONS(1747),
    [sym_smtp_require_tls] = ACTIONS(1747),
    [sym_smtp_from] = ACTIONS(1747),
    [sym_smtp_tls_true] = ACTIONS(1747),
    [sym_smtp_tls_false] = ACTIONS(1747),
    [sym_listview_on_row_select_event] = ACTIONS(1747),
    [sym_data_structure_unknown] = ACTIONS(1747),
    [sym_data_structure_datafeed] = ACTIONS(1747),
    [sym_data_structure_datarecordset] = ACTIONS(1747),
    [sym_data_structure_datarecord] = ACTIONS(1747),
    [sym_data_structure_datafield] = ACTIONS(1747),
    [sym_data_max_content_field_count] = ACTIONS(1747),
    [sym_data_max_metadata_field_count] = ACTIONS(1747),
    [sym_data_max_id_length] = ACTIONS(1747),
    [sym_data_max_name_length] = ACTIONS(1747),
    [sym_data_max_description_length] = ACTIONS(1747),
    [sym_data_max_type_length] = ACTIONS(1747),
    [sym_data_max_format_length] = ACTIONS(1747),
    [sym_data_max_source_length] = ACTIONS(1747),
    [sym_data_max_value_length] = ACTIONS(1747),
    [sym_data_max_label_length] = ACTIONS(1747),
    [sym_data_max_url_length] = ACTIONS(1747),
    [sym_max_last_login_info_length] = ACTIONS(1747),
    [sym_data_type_unknown] = ACTIONS(1747),
    [sym_data_type_string] = ACTIONS(1747),
    [sym_data_type_datetime] = ACTIONS(1747),
    [sym_data_type_date] = ACTIONS(1747),
    [sym_data_type_time] = ACTIONS(1747),
    [sym_data_type_image] = ACTIONS(1747),
    [sym_data_format_url] = ACTIONS(1747),
    [sym_data_format_phone] = ACTIONS(1747),
    [sym_data_format_email] = ACTIONS(1747),
    [sym_data_format_iso8601] = ACTIONS(1747),
    [sym_auditlog_priv] = ACTIONS(1747),
    [sym_configuration_priv] = ACTIONS(1747),
    [sym_deviceconfig_priv] = ACTIONS(1747),
    [sym_ftp_priv] = ACTIONS(1747),
    [sym_http_priv] = ACTIONS(1747),
    [sym_networkconfig_priv] = ACTIONS(1747),
    [sym_programport_priv] = ACTIONS(1747),
    [sym_remoteui_priv] = ACTIONS(1747),
    [sym_securitycontrol_priv] = ACTIONS(1747),
    [sym_softwaremanagement_priv] = ACTIONS(1747),
    [sym_terminal_priv] = ACTIONS(1747),
    [sym_tpadmin_priv] = ACTIONS(1747),
    [sym_user1_priv] = ACTIONS(1747),
    [sym_user2_priv] = ACTIONS(1747),
    [sym_user3_priv] = ACTIONS(1747),
    [sym_user4_priv] = ACTIONS(1747),
    [sym_usermanagement_priv] = ACTIONS(1747),
    [sym_valid_account] = ACTIONS(1747),
    [sym_err_name_invalid] = ACTIONS(1747),
    [sym_err_password_invalid] = ACTIONS(1747),
    [sym_err_invalid_account] = ACTIONS(1747),
    [sym_err_no_authorization] = ACTIONS(1747),
    [sym_err_invalid_parameter] = ACTIONS(1747),
    [sym_err_user_locked_out] = ACTIONS(1747),
    [sym_err_user_account_expired] = ACTIONS(1747),
    [sym_err_general_error] = ACTIONS(1747),
    [sym_err_authentication_error] = ACTIONS(1747),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1747),
    [sym_tls_validate_certificate] = ACTIONS(1747),
    [sym_tdata] = ACTIONS(1747),
    [sym_tchannel] = ACTIONS(1747),
    [sym_tlevel] = ACTIONS(1747),
    [sym_tbutton] = ACTIONS(1747),
    [sym_ttimeline] = ACTIONS(1747),
    [sym_tcustom] = ACTIONS(1747),
    [sym_url_struct] = ACTIONS(1747),
    [sym_dns_struct] = ACTIONS(1747),
    [sym_ip_address_struct] = ACTIONS(1747),
    [sym_dev_info_struct] = ACTIONS(1747),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1747),
    [sym_data_feed] = ACTIONS(1747),
    [sym_data_field] = ACTIONS(1747),
    [sym_data_record] = ACTIONS(1747),
    [sym_wc_data_feed] = ACTIONS(1747),
    [sym_wc_data_field] = ACTIONS(1747),
    [sym_wc_data_record] = ACTIONS(1747),
    [sym_last_login_info] = ACTIONS(1747),
    [sym____reserved___] = ACTIONS(1747),
    [sym_date] = ACTIONS(1747),
    [sym_day] = ACTIONS(1747),
    [sym_time] = ACTIONS(1747),
    [sym_ldate] = ACTIONS(1747),
    [sym_push_device] = ACTIONS(1747),
    [sym_push_channel] = ACTIONS(1747),
    [sym_push_devchan] = ACTIONS(1747),
    [sym_release_device] = ACTIONS(1747),
    [sym_release_channel] = ACTIONS(1747),
    [sym_release_devchan] = ACTIONS(1747),
    [sym_master_slot] = ACTIONS(1747),
    [sym_get_pulse_time] = ACTIONS(1747),
    [sym_get_timer] = ACTIONS(1747),
    [sym_master_sn] = ACTIONS(1747),
    [sym_system_number] = ACTIONS(1747),
    [sym_button] = ACTIONS(1747),
    [sym_dv_channel] = ACTIONS(1747),
    [sym_data] = ACTIONS(1747),
    [sym_level] = ACTIONS(1747),
    [sym_timeline] = ACTIONS(1747),
    [sym_channel] = ACTIONS(1747),
    [sym_custom] = ACTIONS(1747),
    [sym_atoi] = ACTIONS(1747),
    [sym_atol] = ACTIONS(1747),
    [sym_atof] = ACTIONS(1747),
    [sym_itoa] = ACTIONS(1747),
    [sym_itohex] = ACTIONS(1747),
    [sym_hextoi] = ACTIONS(1747),
    [sym_ftoa] = ACTIONS(1747),
    [sym_find_string] = ACTIONS(1747),
    [sym_left_string] = ACTIONS(1747),
    [sym_right_string] = ACTIONS(1747),
    [sym_mid_string] = ACTIONS(1747),
    [sym_remove_string] = ACTIONS(1747),
    [sym_get_buffer_string] = ACTIONS(1747),
    [sym_get_buffer_char] = ACTIONS(1747),
    [sym_get_multi_buffer_string] = ACTIONS(1747),
    [sym_length_string] = ACTIONS(1747),
    [sym_max_length_string] = ACTIONS(1747),
    [sym_set_length_string] = ACTIONS(1747),
    [sym_compare_string] = ACTIONS(1747),
    [sym_upper_string] = ACTIONS(1747),
    [sym_lower_string] = ACTIONS(1747),
    [sym_get_last] = ACTIONS(1747),
    [sym_date_to_day] = ACTIONS(1747),
    [sym_date_to_month] = ACTIONS(1747),
    [sym_date_to_year] = ACTIONS(1747),
    [sym_time_to_hour] = ACTIONS(1747),
    [sym_time_to_minute] = ACTIONS(1747),
    [sym_time_to_second] = ACTIONS(1747),
    [sym_day_of_week] = ACTIONS(1747),
    [sym_set_pulse_time] = ACTIONS(1747),
    [sym_set_timer] = ACTIONS(1747),
    [sym_file_open] = ACTIONS(1747),
    [sym_file_close] = ACTIONS(1747),
    [sym_file_read] = ACTIONS(1747),
    [sym_file_read_line] = ACTIONS(1747),
    [sym_file_write] = ACTIONS(1747),
    [sym_file_write_line] = ACTIONS(1747),
    [sym_file_copy] = ACTIONS(1747),
    [sym_file_delete] = ACTIONS(1747),
    [sym_file_rename] = ACTIONS(1747),
    [sym_file_dir] = ACTIONS(1747),
    [sym_file_getdir] = ACTIONS(1747),
    [sym_file_setdir] = ACTIONS(1747),
    [sym_file_create_dir] = ACTIONS(1747),
    [sym_file_remove_dir] = ACTIONS(1747),
    [sym_file_seek] = ACTIONS(1747),
    [sym_length_array] = ACTIONS(1747),
    [sym_max_length_array] = ACTIONS(1747),
    [sym_set_length_array] = ACTIONS(1747),
    [sym_ip_client_open] = ACTIONS(1747),
    [sym_ip_client_close] = ACTIONS(1747),
    [sym_ip_server_open] = ACTIONS(1747),
    [sym_ip_server_close] = ACTIONS(1747),
    [sym_ip_bound_client_open] = ACTIONS(1747),
    [sym_ip_mc_server_open] = ACTIONS(1747),
    [sym_ip_set_option] = ACTIONS(1747),
    [sym_ssh_client_open] = ACTIONS(1747),
    [sym_ssh_client_close] = ACTIONS(1747),
    [sym_tls_client_open] = ACTIONS(1747),
    [sym_tls_client_close] = ACTIONS(1747),
    [sym_do_push] = ACTIONS(1747),
    [sym_do_push_timed] = ACTIONS(1747),
    [sym_do_release] = ACTIONS(1747),
    [sym_device_id] = ACTIONS(1747),
    [sym_device_info] = ACTIONS(1747),
    [sym_device_id_string] = ACTIONS(1747),
    [sym_device_standby] = ACTIONS(1747),
    [sym_device_wake] = ACTIONS(1747),
    [sym_timeline_create] = ACTIONS(1747),
    [sym_timeline_kill] = ACTIONS(1747),
    [sym_timeline_reload] = ACTIONS(1747),
    [sym_timeline_pause] = ACTIONS(1747),
    [sym_timeline_restart] = ACTIONS(1747),
    [sym_timeline_set] = ACTIONS(1747),
    [sym_timeline_get] = ACTIONS(1747),
    [sym_timeline_active] = ACTIONS(1747),
    [sym_timeline_dynamic_id] = ACTIONS(1747),
    [sym_variable_to_string] = ACTIONS(1747),
    [sym_string_to_variable] = ACTIONS(1747),
    [sym_length_variable_to_string] = ACTIONS(1747),
    [sym_type_cast] = ACTIONS(1747),
    [sym_variable_to_xml] = ACTIONS(1747),
    [sym_xml_to_variable] = ACTIONS(1747),
    [sym_length_variable_to_xml] = ACTIONS(1747),
    [sym_redirect_string] = ACTIONS(1747),
    [sym_random_number] = ACTIONS(1747),
    [sym_get_unique_id] = ACTIONS(1747),
    [sym_get_system_number] = ACTIONS(1747),
    [sym_set_system_number] = ACTIONS(1747),
    [sym_get_serial_number] = ACTIONS(1747),
    [sym_get_master_build] = ACTIONS(1747),
    [sym_set_validation_code] = ACTIONS(1747),
    [sym_reboot] = ACTIONS(1747),
    [sym_load_duet_module] = ACTIONS(1747),
    [sym_unload_duet_module] = ACTIONS(1747),
    [sym_duet_mem_size_set] = ACTIONS(1747),
    [sym_duet_mem_size_get] = ACTIONS(1747),
    [sym_dynamic_application_device] = ACTIONS(1747),
    [sym_dynamic_polled_port] = ACTIONS(1747),
    [sym_static_port_binding] = ACTIONS(1747),
    [sym_static_ip_binding] = ACTIONS(1747),
    [sym_get_dns_list] = ACTIONS(1747),
    [sym_set_dns_list] = ACTIONS(1747),
    [sym_get_ip_address] = ACTIONS(1747),
    [sym_set_ip_address] = ACTIONS(1747),
    [sym_get_url_list] = ACTIONS(1747),
    [sym_add_url_entry] = ACTIONS(1747),
    [sym_delete_url_entry] = ACTIONS(1747),
    [sym_abs_value] = ACTIONS(1747),
    [sym_max_value] = ACTIONS(1747),
    [sym_min_value] = ACTIONS(1747),
    [sym_exp_value] = ACTIONS(1747),
    [sym_log_value] = ACTIONS(1747),
    [sym_log10_value] = ACTIONS(1747),
    [sym_power_value] = ACTIONS(1747),
    [sym_sqrt_value] = ACTIONS(1747),
    [sym_raw_be] = ACTIONS(1747),
    [sym_raw_le] = ACTIONS(1747),
    [sym_internal_queue_size_set] = ACTIONS(1747),
    [sym_internal_queue_size_get] = ACTIONS(1747),
    [sym_internal_threshold_set] = ACTIONS(1747),
    [sym_internal_threshold_get] = ACTIONS(1747),
    [sym_rebuilt_event] = ACTIONS(1747),
    [sym_do_custom_event] = ACTIONS(1747),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1747),
    [sym_clkmgr_set_clk_source] = ACTIONS(1747),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1747),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1747),
    [sym_clkmgr_get_timezone] = ACTIONS(1747),
    [sym_clkmgr_set_timezone] = ACTIONS(1747),
    [sym_clkmgr_get_resync_period] = ACTIONS(1747),
    [sym_clkmgr_set_resync_period] = ACTIONS(1747),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1747),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1747),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1747),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1747),
    [sym_clkmgr_get_timeservers] = ACTIONS(1747),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1747),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1747),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1747),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1747),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1747),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1747),
    [sym_set_log_level] = ACTIONS(1747),
    [sym_get_log_level] = ACTIONS(1747),
    [sym_amx_log] = ACTIONS(1747),
    [sym_smtp_server_config_set] = ACTIONS(1747),
    [sym_smtp_server_config_get] = ACTIONS(1747),
    [sym_smtp_send] = ACTIONS(1747),
    [sym_validate_netlinx_account] = ACTIONS(1747),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1747),
    [sym_authenticate_certificate] = ACTIONS(1747),
    [sym_audit_netlinx_session_event] = ACTIONS(1747),
    [sym_audit_netlinx_generic_event] = ACTIONS(1747),
    [sym_get_available_flash_disk_space] = ACTIONS(1747),
    [sym_get_max_flash_disk_space] = ACTIONS(1747),
    [sym_data_create_feed] = ACTIONS(1747),
    [sym_data_delete_feed] = ACTIONS(1747),
    [sym_data_publish_feed] = ACTIONS(1747),
    [sym_data_get_published_feed] = ACTIONS(1747),
    [sym_data_add_record] = ACTIONS(1747),
    [sym_data_get_event_record] = ACTIONS(1747),
    [sym__wc_data_create_feed] = ACTIONS(1747),
    [sym__wc_data_add_record] = ACTIONS(1747),
    [sym__wc_data_get_event_record] = ACTIONS(1747),
    [sym_format] = ACTIONS(1747),
    [sym_astro_clock] = ACTIONS(1747),
    [sym_set_outdoor_temperature] = ACTIONS(1747),
    [sym_set_virtual_level_count] = ACTIONS(1747),
    [sym_set_virtual_channel_count] = ACTIONS(1747),
    [sym_set_virtual_port_count] = ACTIONS(1747),
    [sym__wc_encode_int] = ACTIONS(1747),
    [sym__wc_decode_int] = ACTIONS(1747),
    [sym__wc_int] = ACTIONS(1747),
    [sym__wc_to_ch_int] = ACTIONS(1747),
    [sym__ch_to_wc_int] = ACTIONS(1747),
    [sym__wc_find_string_int] = ACTIONS(1747),
    [sym__wc_left_string_int] = ACTIONS(1747),
    [sym__wc_lower_string_int] = ACTIONS(1747),
    [sym__wc_mid_string_int] = ACTIONS(1747),
    [sym__wc_remove_string_int] = ACTIONS(1747),
    [sym__wc_right_string_int] = ACTIONS(1747),
    [sym__wc_upper_string_int] = ACTIONS(1747),
    [sym__wc_compare_string_int] = ACTIONS(1747),
    [sym__wc_get_buffer_char_int] = ACTIONS(1747),
    [sym__wc_get_buffer_string_int] = ACTIONS(1747),
    [sym__wc_concat_string_int] = ACTIONS(1747),
    [sym__wc_file_open_int] = ACTIONS(1747),
    [sym__wc_file_close_int] = ACTIONS(1747),
    [sym__wc_file_read_int] = ACTIONS(1747),
    [sym__wc_file_read_line_int] = ACTIONS(1747),
    [sym__wc_file_write_int] = ACTIONS(1747),
    [sym__wc_file_write_line_int] = ACTIONS(1747),
    [aux_sym_preproc_include_token1] = ACTIONS(1747),
    [aux_sym_preproc_define_token1] = ACTIONS(1749),
    [aux_sym_preproc_warn_token1] = ACTIONS(1749),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1749),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1749),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1749),
    [anon_sym_LBRACK] = ACTIONS(1749),
    [anon_sym_LBRACE] = ACTIONS(1749),
    [anon_sym_RBRACE] = ACTIONS(1749),
    [anon_sym_LPAREN] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [anon_sym_DASH] = ACTIONS(1747),
    [anon_sym_PLUS] = ACTIONS(1747),
    [anon_sym_DASH_DASH] = ACTIONS(1749),
    [anon_sym_PLUS_PLUS] = ACTIONS(1749),
    [anon_sym_DQUOTE] = ACTIONS(1749),
    [anon_sym_SQUOTE] = ACTIONS(1749),
    [sym_number_literal] = ACTIONS(1749),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(512)] = {
    [sym_identifier] = ACTIONS(1751),
    [sym_define_call_keyword] = ACTIONS(1751),
    [sym_define_function_keyword] = ACTIONS(1751),
    [sym_define_library_function_keyword] = ACTIONS(1751),
    [sym_define_module_keyword] = ACTIONS(1751),
    [sym_char_keyword] = ACTIONS(1751),
    [sym_widechar_keyword] = ACTIONS(1751),
    [sym_integer_keyword] = ACTIONS(1751),
    [sym_sinteger_keyword] = ACTIONS(1751),
    [sym_long_keyword] = ACTIONS(1751),
    [sym_slong_keyword] = ACTIONS(1751),
    [sym_float_keyword] = ACTIONS(1751),
    [sym_double_keyword] = ACTIONS(1751),
    [sym_variant_keyword] = ACTIONS(1751),
    [sym_variantarray_keyword] = ACTIONS(1751),
    [sym_local_var_keyword] = ACTIONS(1751),
    [sym_stack_var_keyword] = ACTIONS(1751),
    [sym_constant_keyword] = ACTIONS(1751),
    [sym_volatile_keyword] = ACTIONS(1751),
    [sym_non_volatile_keyword] = ACTIONS(1751),
    [sym_persistent_keyword] = ACTIONS(1751),
    [sym_dev_keyword] = ACTIONS(1751),
    [sym_devlev_keyword] = ACTIONS(1751),
    [sym_devchan_keyword] = ACTIONS(1751),
    [sym_if_keyword] = ACTIONS(1751),
    [sym_else_keyword] = ACTIONS(1751),
    [sym_switch_keyword] = ACTIONS(1751),
    [sym_case_keyword] = ACTIONS(1751),
    [sym_default_keyword] = ACTIONS(1751),
    [sym_while_keyword] = ACTIONS(1751),
    [sym_for_keyword] = ACTIONS(1751),
    [sym_break_keyword] = ACTIONS(1751),
    [sym_continue_keyword] = ACTIONS(1751),
    [sym_return_keyword] = ACTIONS(1751),
    [sym_select_keyword] = ACTIONS(1751),
    [sym_active_keyword] = ACTIONS(1751),
    [sym_struct_keyword] = ACTIONS(1751),
    [sym_structure_keyword] = ACTIONS(1751),
    [aux_sym_on_keyword_token1] = ACTIONS(1751),
    [aux_sym_off_keyword_token1] = ACTIONS(1751),
    [sym_send_level_keyword] = ACTIONS(1751),
    [sym_send_string_keyword] = ACTIONS(1751),
    [sym_send_command_keyword] = ACTIONS(1751),
    [sym_clear_buffer_keyword] = ACTIONS(1751),
    [sym_create_buffer_keyword] = ACTIONS(1751),
    [sym_create_multi_buffer_keyword] = ACTIONS(1751),
    [sym_call_keyword] = ACTIONS(1751),
    [sym_system_call_keyword] = ACTIONS(1751),
    [sym_devchan_to_keyword] = ACTIONS(1751),
    [sym_devchan_min_to_keyword] = ACTIONS(1751),
    [sym_devchan_total_off_keyword] = ACTIONS(1751),
    [sym_devchan_pulse_keyword] = ACTIONS(1751),
    [sym_wait_keyword] = ACTIONS(1751),
    [sym_cancel_wait_keyword] = ACTIONS(1751),
    [sym_wait_until_keyword] = ACTIONS(1751),
    [sym_cancel_wait_until_keyword] = ACTIONS(1751),
    [sym_cancel_all_wait_keyword] = ACTIONS(1751),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1751),
    [sym_not] = ACTIONS(1751),
    [sym_bnot] = ACTIONS(1751),
    [sym___file__] = ACTIONS(1751),
    [sym___date__] = ACTIONS(1751),
    [sym___ldate__] = ACTIONS(1751),
    [sym___time__] = ACTIONS(1751),
    [sym___line__] = ACTIONS(1751),
    [sym___name__] = ACTIONS(1751),
    [sym___netlinx__] = ACTIONS(1751),
    [sym_netlinx_axi_version] = ACTIONS(1751),
    [sym_url_flg_tcp] = ACTIONS(1751),
    [sym_url_flg_accinfopresent] = ACTIONS(1751),
    [sym_url_flg_temp] = ACTIONS(1751),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1751),
    [sym_url_flg_stat_mask] = ACTIONS(1751),
    [sym_url_flg_stat_lookup] = ACTIONS(1751),
    [sym_url_flg_stat_connecting] = ACTIONS(1751),
    [sym_url_flg_stat_waiting] = ACTIONS(1751),
    [sym_url_flg_stat_connected] = ACTIONS(1751),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1751),
    [sym_first_virtual_device] = ACTIONS(1751),
    [sym_first_local_port] = ACTIONS(1751),
    [sym_dynamic_virtual_device] = ACTIONS(1751),
    [sym_timeline_once] = ACTIONS(1751),
    [sym_timeline_repeat] = ACTIONS(1751),
    [sym_timeline_absolute] = ACTIONS(1751),
    [sym_timeline_relative] = ACTIONS(1751),
    [sym_do_push_timed_infinite] = ACTIONS(1751),
    [sym_source_type_no_address] = ACTIONS(1751),
    [sym_source_type_neuron_id] = ACTIONS(1751),
    [sym_source_type_ip_address] = ACTIONS(1751),
    [sym_source_type_axlink] = ACTIONS(1751),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1751),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1751),
    [sym_source_type_ip_socket_address] = ACTIONS(1751),
    [sym_source_type_rs232] = ACTIONS(1751),
    [sym_source_type_internal] = ACTIONS(1751),
    [sym_source_type_mac_address] = ACTIONS(1751),
    [sym_source_type_ipv4_port] = ACTIONS(1751),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1751),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1751),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1751),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1751),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1751),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1751),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1751),
    [sym_true] = ACTIONS(1751),
    [sym_false] = ACTIONS(1751),
    [sym_file_read_only] = ACTIONS(1751),
    [sym_file_rw_new] = ACTIONS(1751),
    [sym_file_rw_append] = ACTIONS(1751),
    [sym_ip_tcp] = ACTIONS(1751),
    [sym_ip_udp] = ACTIONS(1751),
    [sym_ip_udp_2way] = ACTIONS(1751),
    [sym_xml_encode_types] = ACTIONS(1751),
    [sym_xml_encode_char_as_list] = ACTIONS(1751),
    [sym_xml_encode_le] = ACTIONS(1751),
    [sym_xml_decode_types] = ACTIONS(1751),
    [sym_xml_decode_no_preserve] = ACTIONS(1751),
    [sym_duet_dev_type_display_device] = ACTIONS(1751),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1751),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1751),
    [sym_duet_dev_type_io_device] = ACTIONS(1751),
    [sym_duet_dev_type_relay_device] = ACTIONS(1751),
    [sym_duet_dev_type_ups] = ACTIONS(1751),
    [sym_duet_dev_type_amplifier] = ACTIONS(1751),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1751),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1751),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1751),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1751),
    [sym_duet_dev_type_camera] = ACTIONS(1751),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1751),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1751),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1751),
    [sym_duet_dev_type_dss] = ACTIONS(1751),
    [sym_duet_dev_type_dvr] = ACTIONS(1751),
    [sym_duet_dev_type_disc_device] = ACTIONS(1751),
    [sym_duet_dev_type_document_camera] = ACTIONS(1751),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1751),
    [sym_duet_dev_type_hvac] = ACTIONS(1751),
    [sym_duet_dev_type_keypad] = ACTIONS(1751),
    [sym_duet_dev_type_light] = ACTIONS(1751),
    [sym_duet_dev_type_monitor] = ACTIONS(1751),
    [sym_duet_dev_type_motor] = ACTIONS(1751),
    [sym_duet_dev_type_multi_window] = ACTIONS(1751),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1751),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1751),
    [sym_duet_dev_type_receiver] = ACTIONS(1751),
    [sym_duet_dev_type_security_system] = ACTIONS(1751),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1751),
    [sym_duet_dev_type_settop_box] = ACTIONS(1751),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1751),
    [sym_duet_dev_type_switcher] = ACTIONS(1751),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1751),
    [sym_duet_dev_type_tv] = ACTIONS(1751),
    [sym_duet_dev_type_utility] = ACTIONS(1751),
    [sym_duet_dev_type_vcr] = ACTIONS(1751),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1751),
    [sym_duet_dev_type_video_processor] = ACTIONS(1751),
    [sym_duet_dev_type_video_projector] = ACTIONS(1751),
    [sym_duet_dev_type_video_wall] = ACTIONS(1751),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1751),
    [sym_duet_dev_type_weather] = ACTIONS(1751),
    [sym_duet_dev_not_polled] = ACTIONS(1751),
    [sym_duet_dev_polled] = ACTIONS(1751),
    [sym_clkmgr_mode_network] = ACTIONS(1751),
    [sym_clkmgr_mode_standalone] = ACTIONS(1751),
    [sym_normal_standby] = ACTIONS(1751),
    [sym_normal_wake] = ACTIONS(1751),
    [sym_ip_multicast_ttl_option] = ACTIONS(1751),
    [sym_ip_ttl_subnet] = ACTIONS(1751),
    [sym_ip_ttl_site] = ACTIONS(1751),
    [sym_ip_ttl_region] = ACTIONS(1751),
    [sym_ip_ttl_continent] = ACTIONS(1751),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1751),
    [sym_ip_nodelay_on] = ACTIONS(1751),
    [sym_ip_nodelay_off] = ACTIONS(1751),
    [sym_amx_error] = ACTIONS(1751),
    [sym_amx_warning] = ACTIONS(1751),
    [sym_amx_info] = ACTIONS(1751),
    [sym_amx_debug] = ACTIONS(1751),
    [sym_netlinx_login_success] = ACTIONS(1751),
    [sym_netlinx_login_fail] = ACTIONS(1751),
    [sym_netlinx_logout] = ACTIONS(1751),
    [sym_null_str] = ACTIONS(1751),
    [sym_smtp_address] = ACTIONS(1751),
    [sym_smtp_port_number] = ACTIONS(1751),
    [sym_smtp_username] = ACTIONS(1751),
    [sym_smtp_password] = ACTIONS(1751),
    [sym_smtp_require_tls] = ACTIONS(1751),
    [sym_smtp_from] = ACTIONS(1751),
    [sym_smtp_tls_true] = ACTIONS(1751),
    [sym_smtp_tls_false] = ACTIONS(1751),
    [sym_listview_on_row_select_event] = ACTIONS(1751),
    [sym_data_structure_unknown] = ACTIONS(1751),
    [sym_data_structure_datafeed] = ACTIONS(1751),
    [sym_data_structure_datarecordset] = ACTIONS(1751),
    [sym_data_structure_datarecord] = ACTIONS(1751),
    [sym_data_structure_datafield] = ACTIONS(1751),
    [sym_data_max_content_field_count] = ACTIONS(1751),
    [sym_data_max_metadata_field_count] = ACTIONS(1751),
    [sym_data_max_id_length] = ACTIONS(1751),
    [sym_data_max_name_length] = ACTIONS(1751),
    [sym_data_max_description_length] = ACTIONS(1751),
    [sym_data_max_type_length] = ACTIONS(1751),
    [sym_data_max_format_length] = ACTIONS(1751),
    [sym_data_max_source_length] = ACTIONS(1751),
    [sym_data_max_value_length] = ACTIONS(1751),
    [sym_data_max_label_length] = ACTIONS(1751),
    [sym_data_max_url_length] = ACTIONS(1751),
    [sym_max_last_login_info_length] = ACTIONS(1751),
    [sym_data_type_unknown] = ACTIONS(1751),
    [sym_data_type_string] = ACTIONS(1751),
    [sym_data_type_datetime] = ACTIONS(1751),
    [sym_data_type_date] = ACTIONS(1751),
    [sym_data_type_time] = ACTIONS(1751),
    [sym_data_type_image] = ACTIONS(1751),
    [sym_data_format_url] = ACTIONS(1751),
    [sym_data_format_phone] = ACTIONS(1751),
    [sym_data_format_email] = ACTIONS(1751),
    [sym_data_format_iso8601] = ACTIONS(1751),
    [sym_auditlog_priv] = ACTIONS(1751),
    [sym_configuration_priv] = ACTIONS(1751),
    [sym_deviceconfig_priv] = ACTIONS(1751),
    [sym_ftp_priv] = ACTIONS(1751),
    [sym_http_priv] = ACTIONS(1751),
    [sym_networkconfig_priv] = ACTIONS(1751),
    [sym_programport_priv] = ACTIONS(1751),
    [sym_remoteui_priv] = ACTIONS(1751),
    [sym_securitycontrol_priv] = ACTIONS(1751),
    [sym_softwaremanagement_priv] = ACTIONS(1751),
    [sym_terminal_priv] = ACTIONS(1751),
    [sym_tpadmin_priv] = ACTIONS(1751),
    [sym_user1_priv] = ACTIONS(1751),
    [sym_user2_priv] = ACTIONS(1751),
    [sym_user3_priv] = ACTIONS(1751),
    [sym_user4_priv] = ACTIONS(1751),
    [sym_usermanagement_priv] = ACTIONS(1751),
    [sym_valid_account] = ACTIONS(1751),
    [sym_err_name_invalid] = ACTIONS(1751),
    [sym_err_password_invalid] = ACTIONS(1751),
    [sym_err_invalid_account] = ACTIONS(1751),
    [sym_err_no_authorization] = ACTIONS(1751),
    [sym_err_invalid_parameter] = ACTIONS(1751),
    [sym_err_user_locked_out] = ACTIONS(1751),
    [sym_err_user_account_expired] = ACTIONS(1751),
    [sym_err_general_error] = ACTIONS(1751),
    [sym_err_authentication_error] = ACTIONS(1751),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1751),
    [sym_tls_validate_certificate] = ACTIONS(1751),
    [sym_tdata] = ACTIONS(1751),
    [sym_tchannel] = ACTIONS(1751),
    [sym_tlevel] = ACTIONS(1751),
    [sym_tbutton] = ACTIONS(1751),
    [sym_ttimeline] = ACTIONS(1751),
    [sym_tcustom] = ACTIONS(1751),
    [sym_url_struct] = ACTIONS(1751),
    [sym_dns_struct] = ACTIONS(1751),
    [sym_ip_address_struct] = ACTIONS(1751),
    [sym_dev_info_struct] = ACTIONS(1751),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1751),
    [sym_data_feed] = ACTIONS(1751),
    [sym_data_field] = ACTIONS(1751),
    [sym_data_record] = ACTIONS(1751),
    [sym_wc_data_feed] = ACTIONS(1751),
    [sym_wc_data_field] = ACTIONS(1751),
    [sym_wc_data_record] = ACTIONS(1751),
    [sym_last_login_info] = ACTIONS(1751),
    [sym____reserved___] = ACTIONS(1751),
    [sym_date] = ACTIONS(1751),
    [sym_day] = ACTIONS(1751),
    [sym_time] = ACTIONS(1751),
    [sym_ldate] = ACTIONS(1751),
    [sym_push_device] = ACTIONS(1751),
    [sym_push_channel] = ACTIONS(1751),
    [sym_push_devchan] = ACTIONS(1751),
    [sym_release_device] = ACTIONS(1751),
    [sym_release_channel] = ACTIONS(1751),
    [sym_release_devchan] = ACTIONS(1751),
    [sym_master_slot] = ACTIONS(1751),
    [sym_get_pulse_time] = ACTIONS(1751),
    [sym_get_timer] = ACTIONS(1751),
    [sym_master_sn] = ACTIONS(1751),
    [sym_system_number] = ACTIONS(1751),
    [sym_button] = ACTIONS(1751),
    [sym_dv_channel] = ACTIONS(1751),
    [sym_data] = ACTIONS(1751),
    [sym_level] = ACTIONS(1751),
    [sym_timeline] = ACTIONS(1751),
    [sym_channel] = ACTIONS(1751),
    [sym_custom] = ACTIONS(1751),
    [sym_atoi] = ACTIONS(1751),
    [sym_atol] = ACTIONS(1751),
    [sym_atof] = ACTIONS(1751),
    [sym_itoa] = ACTIONS(1751),
    [sym_itohex] = ACTIONS(1751),
    [sym_hextoi] = ACTIONS(1751),
    [sym_ftoa] = ACTIONS(1751),
    [sym_find_string] = ACTIONS(1751),
    [sym_left_string] = ACTIONS(1751),
    [sym_right_string] = ACTIONS(1751),
    [sym_mid_string] = ACTIONS(1751),
    [sym_remove_string] = ACTIONS(1751),
    [sym_get_buffer_string] = ACTIONS(1751),
    [sym_get_buffer_char] = ACTIONS(1751),
    [sym_get_multi_buffer_string] = ACTIONS(1751),
    [sym_length_string] = ACTIONS(1751),
    [sym_max_length_string] = ACTIONS(1751),
    [sym_set_length_string] = ACTIONS(1751),
    [sym_compare_string] = ACTIONS(1751),
    [sym_upper_string] = ACTIONS(1751),
    [sym_lower_string] = ACTIONS(1751),
    [sym_get_last] = ACTIONS(1751),
    [sym_date_to_day] = ACTIONS(1751),
    [sym_date_to_month] = ACTIONS(1751),
    [sym_date_to_year] = ACTIONS(1751),
    [sym_time_to_hour] = ACTIONS(1751),
    [sym_time_to_minute] = ACTIONS(1751),
    [sym_time_to_second] = ACTIONS(1751),
    [sym_day_of_week] = ACTIONS(1751),
    [sym_set_pulse_time] = ACTIONS(1751),
    [sym_set_timer] = ACTIONS(1751),
    [sym_file_open] = ACTIONS(1751),
    [sym_file_close] = ACTIONS(1751),
    [sym_file_read] = ACTIONS(1751),
    [sym_file_read_line] = ACTIONS(1751),
    [sym_file_write] = ACTIONS(1751),
    [sym_file_write_line] = ACTIONS(1751),
    [sym_file_copy] = ACTIONS(1751),
    [sym_file_delete] = ACTIONS(1751),
    [sym_file_rename] = ACTIONS(1751),
    [sym_file_dir] = ACTIONS(1751),
    [sym_file_getdir] = ACTIONS(1751),
    [sym_file_setdir] = ACTIONS(1751),
    [sym_file_create_dir] = ACTIONS(1751),
    [sym_file_remove_dir] = ACTIONS(1751),
    [sym_file_seek] = ACTIONS(1751),
    [sym_length_array] = ACTIONS(1751),
    [sym_max_length_array] = ACTIONS(1751),
    [sym_set_length_array] = ACTIONS(1751),
    [sym_ip_client_open] = ACTIONS(1751),
    [sym_ip_client_close] = ACTIONS(1751),
    [sym_ip_server_open] = ACTIONS(1751),
    [sym_ip_server_close] = ACTIONS(1751),
    [sym_ip_bound_client_open] = ACTIONS(1751),
    [sym_ip_mc_server_open] = ACTIONS(1751),
    [sym_ip_set_option] = ACTIONS(1751),
    [sym_ssh_client_open] = ACTIONS(1751),
    [sym_ssh_client_close] = ACTIONS(1751),
    [sym_tls_client_open] = ACTIONS(1751),
    [sym_tls_client_close] = ACTIONS(1751),
    [sym_do_push] = ACTIONS(1751),
    [sym_do_push_timed] = ACTIONS(1751),
    [sym_do_release] = ACTIONS(1751),
    [sym_device_id] = ACTIONS(1751),
    [sym_device_info] = ACTIONS(1751),
    [sym_device_id_string] = ACTIONS(1751),
    [sym_device_standby] = ACTIONS(1751),
    [sym_device_wake] = ACTIONS(1751),
    [sym_timeline_create] = ACTIONS(1751),
    [sym_timeline_kill] = ACTIONS(1751),
    [sym_timeline_reload] = ACTIONS(1751),
    [sym_timeline_pause] = ACTIONS(1751),
    [sym_timeline_restart] = ACTIONS(1751),
    [sym_timeline_set] = ACTIONS(1751),
    [sym_timeline_get] = ACTIONS(1751),
    [sym_timeline_active] = ACTIONS(1751),
    [sym_timeline_dynamic_id] = ACTIONS(1751),
    [sym_variable_to_string] = ACTIONS(1751),
    [sym_string_to_variable] = ACTIONS(1751),
    [sym_length_variable_to_string] = ACTIONS(1751),
    [sym_type_cast] = ACTIONS(1751),
    [sym_variable_to_xml] = ACTIONS(1751),
    [sym_xml_to_variable] = ACTIONS(1751),
    [sym_length_variable_to_xml] = ACTIONS(1751),
    [sym_redirect_string] = ACTIONS(1751),
    [sym_random_number] = ACTIONS(1751),
    [sym_get_unique_id] = ACTIONS(1751),
    [sym_get_system_number] = ACTIONS(1751),
    [sym_set_system_number] = ACTIONS(1751),
    [sym_get_serial_number] = ACTIONS(1751),
    [sym_get_master_build] = ACTIONS(1751),
    [sym_set_validation_code] = ACTIONS(1751),
    [sym_reboot] = ACTIONS(1751),
    [sym_load_duet_module] = ACTIONS(1751),
    [sym_unload_duet_module] = ACTIONS(1751),
    [sym_duet_mem_size_set] = ACTIONS(1751),
    [sym_duet_mem_size_get] = ACTIONS(1751),
    [sym_dynamic_application_device] = ACTIONS(1751),
    [sym_dynamic_polled_port] = ACTIONS(1751),
    [sym_static_port_binding] = ACTIONS(1751),
    [sym_static_ip_binding] = ACTIONS(1751),
    [sym_get_dns_list] = ACTIONS(1751),
    [sym_set_dns_list] = ACTIONS(1751),
    [sym_get_ip_address] = ACTIONS(1751),
    [sym_set_ip_address] = ACTIONS(1751),
    [sym_get_url_list] = ACTIONS(1751),
    [sym_add_url_entry] = ACTIONS(1751),
    [sym_delete_url_entry] = ACTIONS(1751),
    [sym_abs_value] = ACTIONS(1751),
    [sym_max_value] = ACTIONS(1751),
    [sym_min_value] = ACTIONS(1751),
    [sym_exp_value] = ACTIONS(1751),
    [sym_log_value] = ACTIONS(1751),
    [sym_log10_value] = ACTIONS(1751),
    [sym_power_value] = ACTIONS(1751),
    [sym_sqrt_value] = ACTIONS(1751),
    [sym_raw_be] = ACTIONS(1751),
    [sym_raw_le] = ACTIONS(1751),
    [sym_internal_queue_size_set] = ACTIONS(1751),
    [sym_internal_queue_size_get] = ACTIONS(1751),
    [sym_internal_threshold_set] = ACTIONS(1751),
    [sym_internal_threshold_get] = ACTIONS(1751),
    [sym_rebuilt_event] = ACTIONS(1751),
    [sym_do_custom_event] = ACTIONS(1751),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1751),
    [sym_clkmgr_set_clk_source] = ACTIONS(1751),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1751),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1751),
    [sym_clkmgr_get_timezone] = ACTIONS(1751),
    [sym_clkmgr_set_timezone] = ACTIONS(1751),
    [sym_clkmgr_get_resync_period] = ACTIONS(1751),
    [sym_clkmgr_set_resync_period] = ACTIONS(1751),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1751),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1751),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1751),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1751),
    [sym_clkmgr_get_timeservers] = ACTIONS(1751),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1751),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1751),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1751),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1751),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1751),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1751),
    [sym_set_log_level] = ACTIONS(1751),
    [sym_get_log_level] = ACTIONS(1751),
    [sym_amx_log] = ACTIONS(1751),
    [sym_smtp_server_config_set] = ACTIONS(1751),
    [sym_smtp_server_config_get] = ACTIONS(1751),
    [sym_smtp_send] = ACTIONS(1751),
    [sym_validate_netlinx_account] = ACTIONS(1751),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1751),
    [sym_authenticate_certificate] = ACTIONS(1751),
    [sym_audit_netlinx_session_event] = ACTIONS(1751),
    [sym_audit_netlinx_generic_event] = ACTIONS(1751),
    [sym_get_available_flash_disk_space] = ACTIONS(1751),
    [sym_get_max_flash_disk_space] = ACTIONS(1751),
    [sym_data_create_feed] = ACTIONS(1751),
    [sym_data_delete_feed] = ACTIONS(1751),
    [sym_data_publish_feed] = ACTIONS(1751),
    [sym_data_get_published_feed] = ACTIONS(1751),
    [sym_data_add_record] = ACTIONS(1751),
    [sym_data_get_event_record] = ACTIONS(1751),
    [sym__wc_data_create_feed] = ACTIONS(1751),
    [sym__wc_data_add_record] = ACTIONS(1751),
    [sym__wc_data_get_event_record] = ACTIONS(1751),
    [sym_format] = ACTIONS(1751),
    [sym_astro_clock] = ACTIONS(1751),
    [sym_set_outdoor_temperature] = ACTIONS(1751),
    [sym_set_virtual_level_count] = ACTIONS(1751),
    [sym_set_virtual_channel_count] = ACTIONS(1751),
    [sym_set_virtual_port_count] = ACTIONS(1751),
    [sym__wc_encode_int] = ACTIONS(1751),
    [sym__wc_decode_int] = ACTIONS(1751),
    [sym__wc_int] = ACTIONS(1751),
    [sym__wc_to_ch_int] = ACTIONS(1751),
    [sym__ch_to_wc_int] = ACTIONS(1751),
    [sym__wc_find_string_int] = ACTIONS(1751),
    [sym__wc_left_string_int] = ACTIONS(1751),
    [sym__wc_lower_string_int] = ACTIONS(1751),
    [sym__wc_mid_string_int] = ACTIONS(1751),
    [sym__wc_remove_string_int] = ACTIONS(1751),
    [sym__wc_right_string_int] = ACTIONS(1751),
    [sym__wc_upper_string_int] = ACTIONS(1751),
    [sym__wc_compare_string_int] = ACTIONS(1751),
    [sym__wc_get_buffer_char_int] = ACTIONS(1751),
    [sym__wc_get_buffer_string_int] = ACTIONS(1751),
    [sym__wc_concat_string_int] = ACTIONS(1751),
    [sym__wc_file_open_int] = ACTIONS(1751),
    [sym__wc_file_close_int] = ACTIONS(1751),
    [sym__wc_file_read_int] = ACTIONS(1751),
    [sym__wc_file_read_line_int] = ACTIONS(1751),
    [sym__wc_file_write_int] = ACTIONS(1751),
    [sym__wc_file_write_line_int] = ACTIONS(1751),
    [aux_sym_preproc_include_token1] = ACTIONS(1751),
    [aux_sym_preproc_define_token1] = ACTIONS(1753),
    [aux_sym_preproc_warn_token1] = ACTIONS(1753),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1753),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1753),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1753),
    [anon_sym_LBRACK] = ACTIONS(1753),
    [anon_sym_LBRACE] = ACTIONS(1753),
    [anon_sym_RBRACE] = ACTIONS(1753),
    [anon_sym_LPAREN] = ACTIONS(1751),
    [anon_sym_BANG] = ACTIONS(1753),
    [anon_sym_TILDE] = ACTIONS(1753),
    [anon_sym_DASH] = ACTIONS(1751),
    [anon_sym_PLUS] = ACTIONS(1751),
    [anon_sym_DASH_DASH] = ACTIONS(1753),
    [anon_sym_PLUS_PLUS] = ACTIONS(1753),
    [anon_sym_DQUOTE] = ACTIONS(1753),
    [anon_sym_SQUOTE] = ACTIONS(1753),
    [sym_number_literal] = ACTIONS(1753),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(513)] = {
    [sym_identifier] = ACTIONS(1803),
    [sym_define_call_keyword] = ACTIONS(1803),
    [sym_define_function_keyword] = ACTIONS(1803),
    [sym_define_library_function_keyword] = ACTIONS(1803),
    [sym_define_module_keyword] = ACTIONS(1803),
    [sym_char_keyword] = ACTIONS(1803),
    [sym_widechar_keyword] = ACTIONS(1803),
    [sym_integer_keyword] = ACTIONS(1803),
    [sym_sinteger_keyword] = ACTIONS(1803),
    [sym_long_keyword] = ACTIONS(1803),
    [sym_slong_keyword] = ACTIONS(1803),
    [sym_float_keyword] = ACTIONS(1803),
    [sym_double_keyword] = ACTIONS(1803),
    [sym_variant_keyword] = ACTIONS(1803),
    [sym_variantarray_keyword] = ACTIONS(1803),
    [sym_local_var_keyword] = ACTIONS(1803),
    [sym_stack_var_keyword] = ACTIONS(1803),
    [sym_constant_keyword] = ACTIONS(1803),
    [sym_volatile_keyword] = ACTIONS(1803),
    [sym_non_volatile_keyword] = ACTIONS(1803),
    [sym_persistent_keyword] = ACTIONS(1803),
    [sym_dev_keyword] = ACTIONS(1803),
    [sym_devlev_keyword] = ACTIONS(1803),
    [sym_devchan_keyword] = ACTIONS(1803),
    [sym_if_keyword] = ACTIONS(1803),
    [sym_else_keyword] = ACTIONS(1803),
    [sym_switch_keyword] = ACTIONS(1803),
    [sym_case_keyword] = ACTIONS(1803),
    [sym_default_keyword] = ACTIONS(1803),
    [sym_while_keyword] = ACTIONS(1803),
    [sym_for_keyword] = ACTIONS(1803),
    [sym_break_keyword] = ACTIONS(1803),
    [sym_continue_keyword] = ACTIONS(1803),
    [sym_return_keyword] = ACTIONS(1803),
    [sym_select_keyword] = ACTIONS(1803),
    [sym_active_keyword] = ACTIONS(1803),
    [sym_struct_keyword] = ACTIONS(1803),
    [sym_structure_keyword] = ACTIONS(1803),
    [aux_sym_on_keyword_token1] = ACTIONS(1803),
    [aux_sym_off_keyword_token1] = ACTIONS(1803),
    [sym_send_level_keyword] = ACTIONS(1803),
    [sym_send_string_keyword] = ACTIONS(1803),
    [sym_send_command_keyword] = ACTIONS(1803),
    [sym_clear_buffer_keyword] = ACTIONS(1803),
    [sym_create_buffer_keyword] = ACTIONS(1803),
    [sym_create_multi_buffer_keyword] = ACTIONS(1803),
    [sym_call_keyword] = ACTIONS(1803),
    [sym_system_call_keyword] = ACTIONS(1803),
    [sym_devchan_to_keyword] = ACTIONS(1803),
    [sym_devchan_min_to_keyword] = ACTIONS(1803),
    [sym_devchan_total_off_keyword] = ACTIONS(1803),
    [sym_devchan_pulse_keyword] = ACTIONS(1803),
    [sym_wait_keyword] = ACTIONS(1803),
    [sym_cancel_wait_keyword] = ACTIONS(1803),
    [sym_wait_until_keyword] = ACTIONS(1803),
    [sym_cancel_wait_until_keyword] = ACTIONS(1803),
    [sym_cancel_all_wait_keyword] = ACTIONS(1803),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1803),
    [sym_not] = ACTIONS(1803),
    [sym_bnot] = ACTIONS(1803),
    [sym___file__] = ACTIONS(1803),
    [sym___date__] = ACTIONS(1803),
    [sym___ldate__] = ACTIONS(1803),
    [sym___time__] = ACTIONS(1803),
    [sym___line__] = ACTIONS(1803),
    [sym___name__] = ACTIONS(1803),
    [sym___netlinx__] = ACTIONS(1803),
    [sym_netlinx_axi_version] = ACTIONS(1803),
    [sym_url_flg_tcp] = ACTIONS(1803),
    [sym_url_flg_accinfopresent] = ACTIONS(1803),
    [sym_url_flg_temp] = ACTIONS(1803),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1803),
    [sym_url_flg_stat_mask] = ACTIONS(1803),
    [sym_url_flg_stat_lookup] = ACTIONS(1803),
    [sym_url_flg_stat_connecting] = ACTIONS(1803),
    [sym_url_flg_stat_waiting] = ACTIONS(1803),
    [sym_url_flg_stat_connected] = ACTIONS(1803),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1803),
    [sym_first_virtual_device] = ACTIONS(1803),
    [sym_first_local_port] = ACTIONS(1803),
    [sym_dynamic_virtual_device] = ACTIONS(1803),
    [sym_timeline_once] = ACTIONS(1803),
    [sym_timeline_repeat] = ACTIONS(1803),
    [sym_timeline_absolute] = ACTIONS(1803),
    [sym_timeline_relative] = ACTIONS(1803),
    [sym_do_push_timed_infinite] = ACTIONS(1803),
    [sym_source_type_no_address] = ACTIONS(1803),
    [sym_source_type_neuron_id] = ACTIONS(1803),
    [sym_source_type_ip_address] = ACTIONS(1803),
    [sym_source_type_axlink] = ACTIONS(1803),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1803),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1803),
    [sym_source_type_ip_socket_address] = ACTIONS(1803),
    [sym_source_type_rs232] = ACTIONS(1803),
    [sym_source_type_internal] = ACTIONS(1803),
    [sym_source_type_mac_address] = ACTIONS(1803),
    [sym_source_type_ipv4_port] = ACTIONS(1803),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1803),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1803),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1803),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1803),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1803),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1803),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1803),
    [sym_true] = ACTIONS(1803),
    [sym_false] = ACTIONS(1803),
    [sym_file_read_only] = ACTIONS(1803),
    [sym_file_rw_new] = ACTIONS(1803),
    [sym_file_rw_append] = ACTIONS(1803),
    [sym_ip_tcp] = ACTIONS(1803),
    [sym_ip_udp] = ACTIONS(1803),
    [sym_ip_udp_2way] = ACTIONS(1803),
    [sym_xml_encode_types] = ACTIONS(1803),
    [sym_xml_encode_char_as_list] = ACTIONS(1803),
    [sym_xml_encode_le] = ACTIONS(1803),
    [sym_xml_decode_types] = ACTIONS(1803),
    [sym_xml_decode_no_preserve] = ACTIONS(1803),
    [sym_duet_dev_type_display_device] = ACTIONS(1803),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1803),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1803),
    [sym_duet_dev_type_io_device] = ACTIONS(1803),
    [sym_duet_dev_type_relay_device] = ACTIONS(1803),
    [sym_duet_dev_type_ups] = ACTIONS(1803),
    [sym_duet_dev_type_amplifier] = ACTIONS(1803),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1803),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1803),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1803),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1803),
    [sym_duet_dev_type_camera] = ACTIONS(1803),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1803),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1803),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1803),
    [sym_duet_dev_type_dss] = ACTIONS(1803),
    [sym_duet_dev_type_dvr] = ACTIONS(1803),
    [sym_duet_dev_type_disc_device] = ACTIONS(1803),
    [sym_duet_dev_type_document_camera] = ACTIONS(1803),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1803),
    [sym_duet_dev_type_hvac] = ACTIONS(1803),
    [sym_duet_dev_type_keypad] = ACTIONS(1803),
    [sym_duet_dev_type_light] = ACTIONS(1803),
    [sym_duet_dev_type_monitor] = ACTIONS(1803),
    [sym_duet_dev_type_motor] = ACTIONS(1803),
    [sym_duet_dev_type_multi_window] = ACTIONS(1803),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1803),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1803),
    [sym_duet_dev_type_receiver] = ACTIONS(1803),
    [sym_duet_dev_type_security_system] = ACTIONS(1803),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1803),
    [sym_duet_dev_type_settop_box] = ACTIONS(1803),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1803),
    [sym_duet_dev_type_switcher] = ACTIONS(1803),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1803),
    [sym_duet_dev_type_tv] = ACTIONS(1803),
    [sym_duet_dev_type_utility] = ACTIONS(1803),
    [sym_duet_dev_type_vcr] = ACTIONS(1803),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1803),
    [sym_duet_dev_type_video_processor] = ACTIONS(1803),
    [sym_duet_dev_type_video_projector] = ACTIONS(1803),
    [sym_duet_dev_type_video_wall] = ACTIONS(1803),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1803),
    [sym_duet_dev_type_weather] = ACTIONS(1803),
    [sym_duet_dev_not_polled] = ACTIONS(1803),
    [sym_duet_dev_polled] = ACTIONS(1803),
    [sym_clkmgr_mode_network] = ACTIONS(1803),
    [sym_clkmgr_mode_standalone] = ACTIONS(1803),
    [sym_normal_standby] = ACTIONS(1803),
    [sym_normal_wake] = ACTIONS(1803),
    [sym_ip_multicast_ttl_option] = ACTIONS(1803),
    [sym_ip_ttl_subnet] = ACTIONS(1803),
    [sym_ip_ttl_site] = ACTIONS(1803),
    [sym_ip_ttl_region] = ACTIONS(1803),
    [sym_ip_ttl_continent] = ACTIONS(1803),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1803),
    [sym_ip_nodelay_on] = ACTIONS(1803),
    [sym_ip_nodelay_off] = ACTIONS(1803),
    [sym_amx_error] = ACTIONS(1803),
    [sym_amx_warning] = ACTIONS(1803),
    [sym_amx_info] = ACTIONS(1803),
    [sym_amx_debug] = ACTIONS(1803),
    [sym_netlinx_login_success] = ACTIONS(1803),
    [sym_netlinx_login_fail] = ACTIONS(1803),
    [sym_netlinx_logout] = ACTIONS(1803),
    [sym_null_str] = ACTIONS(1803),
    [sym_smtp_address] = ACTIONS(1803),
    [sym_smtp_port_number] = ACTIONS(1803),
    [sym_smtp_username] = ACTIONS(1803),
    [sym_smtp_password] = ACTIONS(1803),
    [sym_smtp_require_tls] = ACTIONS(1803),
    [sym_smtp_from] = ACTIONS(1803),
    [sym_smtp_tls_true] = ACTIONS(1803),
    [sym_smtp_tls_false] = ACTIONS(1803),
    [sym_listview_on_row_select_event] = ACTIONS(1803),
    [sym_data_structure_unknown] = ACTIONS(1803),
    [sym_data_structure_datafeed] = ACTIONS(1803),
    [sym_data_structure_datarecordset] = ACTIONS(1803),
    [sym_data_structure_datarecord] = ACTIONS(1803),
    [sym_data_structure_datafield] = ACTIONS(1803),
    [sym_data_max_content_field_count] = ACTIONS(1803),
    [sym_data_max_metadata_field_count] = ACTIONS(1803),
    [sym_data_max_id_length] = ACTIONS(1803),
    [sym_data_max_name_length] = ACTIONS(1803),
    [sym_data_max_description_length] = ACTIONS(1803),
    [sym_data_max_type_length] = ACTIONS(1803),
    [sym_data_max_format_length] = ACTIONS(1803),
    [sym_data_max_source_length] = ACTIONS(1803),
    [sym_data_max_value_length] = ACTIONS(1803),
    [sym_data_max_label_length] = ACTIONS(1803),
    [sym_data_max_url_length] = ACTIONS(1803),
    [sym_max_last_login_info_length] = ACTIONS(1803),
    [sym_data_type_unknown] = ACTIONS(1803),
    [sym_data_type_string] = ACTIONS(1803),
    [sym_data_type_datetime] = ACTIONS(1803),
    [sym_data_type_date] = ACTIONS(1803),
    [sym_data_type_time] = ACTIONS(1803),
    [sym_data_type_image] = ACTIONS(1803),
    [sym_data_format_url] = ACTIONS(1803),
    [sym_data_format_phone] = ACTIONS(1803),
    [sym_data_format_email] = ACTIONS(1803),
    [sym_data_format_iso8601] = ACTIONS(1803),
    [sym_auditlog_priv] = ACTIONS(1803),
    [sym_configuration_priv] = ACTIONS(1803),
    [sym_deviceconfig_priv] = ACTIONS(1803),
    [sym_ftp_priv] = ACTIONS(1803),
    [sym_http_priv] = ACTIONS(1803),
    [sym_networkconfig_priv] = ACTIONS(1803),
    [sym_programport_priv] = ACTIONS(1803),
    [sym_remoteui_priv] = ACTIONS(1803),
    [sym_securitycontrol_priv] = ACTIONS(1803),
    [sym_softwaremanagement_priv] = ACTIONS(1803),
    [sym_terminal_priv] = ACTIONS(1803),
    [sym_tpadmin_priv] = ACTIONS(1803),
    [sym_user1_priv] = ACTIONS(1803),
    [sym_user2_priv] = ACTIONS(1803),
    [sym_user3_priv] = ACTIONS(1803),
    [sym_user4_priv] = ACTIONS(1803),
    [sym_usermanagement_priv] = ACTIONS(1803),
    [sym_valid_account] = ACTIONS(1803),
    [sym_err_name_invalid] = ACTIONS(1803),
    [sym_err_password_invalid] = ACTIONS(1803),
    [sym_err_invalid_account] = ACTIONS(1803),
    [sym_err_no_authorization] = ACTIONS(1803),
    [sym_err_invalid_parameter] = ACTIONS(1803),
    [sym_err_user_locked_out] = ACTIONS(1803),
    [sym_err_user_account_expired] = ACTIONS(1803),
    [sym_err_general_error] = ACTIONS(1803),
    [sym_err_authentication_error] = ACTIONS(1803),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1803),
    [sym_tls_validate_certificate] = ACTIONS(1803),
    [sym_tdata] = ACTIONS(1803),
    [sym_tchannel] = ACTIONS(1803),
    [sym_tlevel] = ACTIONS(1803),
    [sym_tbutton] = ACTIONS(1803),
    [sym_ttimeline] = ACTIONS(1803),
    [sym_tcustom] = ACTIONS(1803),
    [sym_url_struct] = ACTIONS(1803),
    [sym_dns_struct] = ACTIONS(1803),
    [sym_ip_address_struct] = ACTIONS(1803),
    [sym_dev_info_struct] = ACTIONS(1803),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1803),
    [sym_data_feed] = ACTIONS(1803),
    [sym_data_field] = ACTIONS(1803),
    [sym_data_record] = ACTIONS(1803),
    [sym_wc_data_feed] = ACTIONS(1803),
    [sym_wc_data_field] = ACTIONS(1803),
    [sym_wc_data_record] = ACTIONS(1803),
    [sym_last_login_info] = ACTIONS(1803),
    [sym____reserved___] = ACTIONS(1803),
    [sym_date] = ACTIONS(1803),
    [sym_day] = ACTIONS(1803),
    [sym_time] = ACTIONS(1803),
    [sym_ldate] = ACTIONS(1803),
    [sym_push_device] = ACTIONS(1803),
    [sym_push_channel] = ACTIONS(1803),
    [sym_push_devchan] = ACTIONS(1803),
    [sym_release_device] = ACTIONS(1803),
    [sym_release_channel] = ACTIONS(1803),
    [sym_release_devchan] = ACTIONS(1803),
    [sym_master_slot] = ACTIONS(1803),
    [sym_get_pulse_time] = ACTIONS(1803),
    [sym_get_timer] = ACTIONS(1803),
    [sym_master_sn] = ACTIONS(1803),
    [sym_system_number] = ACTIONS(1803),
    [sym_button] = ACTIONS(1803),
    [sym_dv_channel] = ACTIONS(1803),
    [sym_data] = ACTIONS(1803),
    [sym_level] = ACTIONS(1803),
    [sym_timeline] = ACTIONS(1803),
    [sym_channel] = ACTIONS(1803),
    [sym_custom] = ACTIONS(1803),
    [sym_atoi] = ACTIONS(1803),
    [sym_atol] = ACTIONS(1803),
    [sym_atof] = ACTIONS(1803),
    [sym_itoa] = ACTIONS(1803),
    [sym_itohex] = ACTIONS(1803),
    [sym_hextoi] = ACTIONS(1803),
    [sym_ftoa] = ACTIONS(1803),
    [sym_find_string] = ACTIONS(1803),
    [sym_left_string] = ACTIONS(1803),
    [sym_right_string] = ACTIONS(1803),
    [sym_mid_string] = ACTIONS(1803),
    [sym_remove_string] = ACTIONS(1803),
    [sym_get_buffer_string] = ACTIONS(1803),
    [sym_get_buffer_char] = ACTIONS(1803),
    [sym_get_multi_buffer_string] = ACTIONS(1803),
    [sym_length_string] = ACTIONS(1803),
    [sym_max_length_string] = ACTIONS(1803),
    [sym_set_length_string] = ACTIONS(1803),
    [sym_compare_string] = ACTIONS(1803),
    [sym_upper_string] = ACTIONS(1803),
    [sym_lower_string] = ACTIONS(1803),
    [sym_get_last] = ACTIONS(1803),
    [sym_date_to_day] = ACTIONS(1803),
    [sym_date_to_month] = ACTIONS(1803),
    [sym_date_to_year] = ACTIONS(1803),
    [sym_time_to_hour] = ACTIONS(1803),
    [sym_time_to_minute] = ACTIONS(1803),
    [sym_time_to_second] = ACTIONS(1803),
    [sym_day_of_week] = ACTIONS(1803),
    [sym_set_pulse_time] = ACTIONS(1803),
    [sym_set_timer] = ACTIONS(1803),
    [sym_file_open] = ACTIONS(1803),
    [sym_file_close] = ACTIONS(1803),
    [sym_file_read] = ACTIONS(1803),
    [sym_file_read_line] = ACTIONS(1803),
    [sym_file_write] = ACTIONS(1803),
    [sym_file_write_line] = ACTIONS(1803),
    [sym_file_copy] = ACTIONS(1803),
    [sym_file_delete] = ACTIONS(1803),
    [sym_file_rename] = ACTIONS(1803),
    [sym_file_dir] = ACTIONS(1803),
    [sym_file_getdir] = ACTIONS(1803),
    [sym_file_setdir] = ACTIONS(1803),
    [sym_file_create_dir] = ACTIONS(1803),
    [sym_file_remove_dir] = ACTIONS(1803),
    [sym_file_seek] = ACTIONS(1803),
    [sym_length_array] = ACTIONS(1803),
    [sym_max_length_array] = ACTIONS(1803),
    [sym_set_length_array] = ACTIONS(1803),
    [sym_ip_client_open] = ACTIONS(1803),
    [sym_ip_client_close] = ACTIONS(1803),
    [sym_ip_server_open] = ACTIONS(1803),
    [sym_ip_server_close] = ACTIONS(1803),
    [sym_ip_bound_client_open] = ACTIONS(1803),
    [sym_ip_mc_server_open] = ACTIONS(1803),
    [sym_ip_set_option] = ACTIONS(1803),
    [sym_ssh_client_open] = ACTIONS(1803),
    [sym_ssh_client_close] = ACTIONS(1803),
    [sym_tls_client_open] = ACTIONS(1803),
    [sym_tls_client_close] = ACTIONS(1803),
    [sym_do_push] = ACTIONS(1803),
    [sym_do_push_timed] = ACTIONS(1803),
    [sym_do_release] = ACTIONS(1803),
    [sym_device_id] = ACTIONS(1803),
    [sym_device_info] = ACTIONS(1803),
    [sym_device_id_string] = ACTIONS(1803),
    [sym_device_standby] = ACTIONS(1803),
    [sym_device_wake] = ACTIONS(1803),
    [sym_timeline_create] = ACTIONS(1803),
    [sym_timeline_kill] = ACTIONS(1803),
    [sym_timeline_reload] = ACTIONS(1803),
    [sym_timeline_pause] = ACTIONS(1803),
    [sym_timeline_restart] = ACTIONS(1803),
    [sym_timeline_set] = ACTIONS(1803),
    [sym_timeline_get] = ACTIONS(1803),
    [sym_timeline_active] = ACTIONS(1803),
    [sym_timeline_dynamic_id] = ACTIONS(1803),
    [sym_variable_to_string] = ACTIONS(1803),
    [sym_string_to_variable] = ACTIONS(1803),
    [sym_length_variable_to_string] = ACTIONS(1803),
    [sym_type_cast] = ACTIONS(1803),
    [sym_variable_to_xml] = ACTIONS(1803),
    [sym_xml_to_variable] = ACTIONS(1803),
    [sym_length_variable_to_xml] = ACTIONS(1803),
    [sym_redirect_string] = ACTIONS(1803),
    [sym_random_number] = ACTIONS(1803),
    [sym_get_unique_id] = ACTIONS(1803),
    [sym_get_system_number] = ACTIONS(1803),
    [sym_set_system_number] = ACTIONS(1803),
    [sym_get_serial_number] = ACTIONS(1803),
    [sym_get_master_build] = ACTIONS(1803),
    [sym_set_validation_code] = ACTIONS(1803),
    [sym_reboot] = ACTIONS(1803),
    [sym_load_duet_module] = ACTIONS(1803),
    [sym_unload_duet_module] = ACTIONS(1803),
    [sym_duet_mem_size_set] = ACTIONS(1803),
    [sym_duet_mem_size_get] = ACTIONS(1803),
    [sym_dynamic_application_device] = ACTIONS(1803),
    [sym_dynamic_polled_port] = ACTIONS(1803),
    [sym_static_port_binding] = ACTIONS(1803),
    [sym_static_ip_binding] = ACTIONS(1803),
    [sym_get_dns_list] = ACTIONS(1803),
    [sym_set_dns_list] = ACTIONS(1803),
    [sym_get_ip_address] = ACTIONS(1803),
    [sym_set_ip_address] = ACTIONS(1803),
    [sym_get_url_list] = ACTIONS(1803),
    [sym_add_url_entry] = ACTIONS(1803),
    [sym_delete_url_entry] = ACTIONS(1803),
    [sym_abs_value] = ACTIONS(1803),
    [sym_max_value] = ACTIONS(1803),
    [sym_min_value] = ACTIONS(1803),
    [sym_exp_value] = ACTIONS(1803),
    [sym_log_value] = ACTIONS(1803),
    [sym_log10_value] = ACTIONS(1803),
    [sym_power_value] = ACTIONS(1803),
    [sym_sqrt_value] = ACTIONS(1803),
    [sym_raw_be] = ACTIONS(1803),
    [sym_raw_le] = ACTIONS(1803),
    [sym_internal_queue_size_set] = ACTIONS(1803),
    [sym_internal_queue_size_get] = ACTIONS(1803),
    [sym_internal_threshold_set] = ACTIONS(1803),
    [sym_internal_threshold_get] = ACTIONS(1803),
    [sym_rebuilt_event] = ACTIONS(1803),
    [sym_do_custom_event] = ACTIONS(1803),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1803),
    [sym_clkmgr_set_clk_source] = ACTIONS(1803),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1803),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1803),
    [sym_clkmgr_get_timezone] = ACTIONS(1803),
    [sym_clkmgr_set_timezone] = ACTIONS(1803),
    [sym_clkmgr_get_resync_period] = ACTIONS(1803),
    [sym_clkmgr_set_resync_period] = ACTIONS(1803),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1803),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1803),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1803),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1803),
    [sym_clkmgr_get_timeservers] = ACTIONS(1803),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1803),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1803),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1803),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1803),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1803),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1803),
    [sym_set_log_level] = ACTIONS(1803),
    [sym_get_log_level] = ACTIONS(1803),
    [sym_amx_log] = ACTIONS(1803),
    [sym_smtp_server_config_set] = ACTIONS(1803),
    [sym_smtp_server_config_get] = ACTIONS(1803),
    [sym_smtp_send] = ACTIONS(1803),
    [sym_validate_netlinx_account] = ACTIONS(1803),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1803),
    [sym_authenticate_certificate] = ACTIONS(1803),
    [sym_audit_netlinx_session_event] = ACTIONS(1803),
    [sym_audit_netlinx_generic_event] = ACTIONS(1803),
    [sym_get_available_flash_disk_space] = ACTIONS(1803),
    [sym_get_max_flash_disk_space] = ACTIONS(1803),
    [sym_data_create_feed] = ACTIONS(1803),
    [sym_data_delete_feed] = ACTIONS(1803),
    [sym_data_publish_feed] = ACTIONS(1803),
    [sym_data_get_published_feed] = ACTIONS(1803),
    [sym_data_add_record] = ACTIONS(1803),
    [sym_data_get_event_record] = ACTIONS(1803),
    [sym__wc_data_create_feed] = ACTIONS(1803),
    [sym__wc_data_add_record] = ACTIONS(1803),
    [sym__wc_data_get_event_record] = ACTIONS(1803),
    [sym_format] = ACTIONS(1803),
    [sym_astro_clock] = ACTIONS(1803),
    [sym_set_outdoor_temperature] = ACTIONS(1803),
    [sym_set_virtual_level_count] = ACTIONS(1803),
    [sym_set_virtual_channel_count] = ACTIONS(1803),
    [sym_set_virtual_port_count] = ACTIONS(1803),
    [sym__wc_encode_int] = ACTIONS(1803),
    [sym__wc_decode_int] = ACTIONS(1803),
    [sym__wc_int] = ACTIONS(1803),
    [sym__wc_to_ch_int] = ACTIONS(1803),
    [sym__ch_to_wc_int] = ACTIONS(1803),
    [sym__wc_find_string_int] = ACTIONS(1803),
    [sym__wc_left_string_int] = ACTIONS(1803),
    [sym__wc_lower_string_int] = ACTIONS(1803),
    [sym__wc_mid_string_int] = ACTIONS(1803),
    [sym__wc_remove_string_int] = ACTIONS(1803),
    [sym__wc_right_string_int] = ACTIONS(1803),
    [sym__wc_upper_string_int] = ACTIONS(1803),
    [sym__wc_compare_string_int] = ACTIONS(1803),
    [sym__wc_get_buffer_char_int] = ACTIONS(1803),
    [sym__wc_get_buffer_string_int] = ACTIONS(1803),
    [sym__wc_concat_string_int] = ACTIONS(1803),
    [sym__wc_file_open_int] = ACTIONS(1803),
    [sym__wc_file_close_int] = ACTIONS(1803),
    [sym__wc_file_read_int] = ACTIONS(1803),
    [sym__wc_file_read_line_int] = ACTIONS(1803),
    [sym__wc_file_write_int] = ACTIONS(1803),
    [sym__wc_file_write_line_int] = ACTIONS(1803),
    [aux_sym_preproc_include_token1] = ACTIONS(1803),
    [aux_sym_preproc_define_token1] = ACTIONS(1805),
    [aux_sym_preproc_warn_token1] = ACTIONS(1805),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1805),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1805),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1805),
    [anon_sym_LBRACK] = ACTIONS(1805),
    [anon_sym_LBRACE] = ACTIONS(1805),
    [anon_sym_RBRACE] = ACTIONS(1805),
    [anon_sym_LPAREN] = ACTIONS(1803),
    [anon_sym_BANG] = ACTIONS(1805),
    [anon_sym_TILDE] = ACTIONS(1805),
    [anon_sym_DASH] = ACTIONS(1803),
    [anon_sym_PLUS] = ACTIONS(1803),
    [anon_sym_DASH_DASH] = ACTIONS(1805),
    [anon_sym_PLUS_PLUS] = ACTIONS(1805),
    [anon_sym_DQUOTE] = ACTIONS(1805),
    [anon_sym_SQUOTE] = ACTIONS(1805),
    [sym_number_literal] = ACTIONS(1805),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(514)] = {
    [sym_identifier] = ACTIONS(1807),
    [sym_define_call_keyword] = ACTIONS(1807),
    [sym_define_function_keyword] = ACTIONS(1807),
    [sym_define_library_function_keyword] = ACTIONS(1807),
    [sym_define_module_keyword] = ACTIONS(1807),
    [sym_char_keyword] = ACTIONS(1807),
    [sym_widechar_keyword] = ACTIONS(1807),
    [sym_integer_keyword] = ACTIONS(1807),
    [sym_sinteger_keyword] = ACTIONS(1807),
    [sym_long_keyword] = ACTIONS(1807),
    [sym_slong_keyword] = ACTIONS(1807),
    [sym_float_keyword] = ACTIONS(1807),
    [sym_double_keyword] = ACTIONS(1807),
    [sym_variant_keyword] = ACTIONS(1807),
    [sym_variantarray_keyword] = ACTIONS(1807),
    [sym_local_var_keyword] = ACTIONS(1807),
    [sym_stack_var_keyword] = ACTIONS(1807),
    [sym_constant_keyword] = ACTIONS(1807),
    [sym_volatile_keyword] = ACTIONS(1807),
    [sym_non_volatile_keyword] = ACTIONS(1807),
    [sym_persistent_keyword] = ACTIONS(1807),
    [sym_dev_keyword] = ACTIONS(1807),
    [sym_devlev_keyword] = ACTIONS(1807),
    [sym_devchan_keyword] = ACTIONS(1807),
    [sym_if_keyword] = ACTIONS(1807),
    [sym_else_keyword] = ACTIONS(1807),
    [sym_switch_keyword] = ACTIONS(1807),
    [sym_case_keyword] = ACTIONS(1807),
    [sym_default_keyword] = ACTIONS(1807),
    [sym_while_keyword] = ACTIONS(1807),
    [sym_for_keyword] = ACTIONS(1807),
    [sym_break_keyword] = ACTIONS(1807),
    [sym_continue_keyword] = ACTIONS(1807),
    [sym_return_keyword] = ACTIONS(1807),
    [sym_select_keyword] = ACTIONS(1807),
    [sym_active_keyword] = ACTIONS(1807),
    [sym_struct_keyword] = ACTIONS(1807),
    [sym_structure_keyword] = ACTIONS(1807),
    [aux_sym_on_keyword_token1] = ACTIONS(1807),
    [aux_sym_off_keyword_token1] = ACTIONS(1807),
    [sym_send_level_keyword] = ACTIONS(1807),
    [sym_send_string_keyword] = ACTIONS(1807),
    [sym_send_command_keyword] = ACTIONS(1807),
    [sym_clear_buffer_keyword] = ACTIONS(1807),
    [sym_create_buffer_keyword] = ACTIONS(1807),
    [sym_create_multi_buffer_keyword] = ACTIONS(1807),
    [sym_call_keyword] = ACTIONS(1807),
    [sym_system_call_keyword] = ACTIONS(1807),
    [sym_devchan_to_keyword] = ACTIONS(1807),
    [sym_devchan_min_to_keyword] = ACTIONS(1807),
    [sym_devchan_total_off_keyword] = ACTIONS(1807),
    [sym_devchan_pulse_keyword] = ACTIONS(1807),
    [sym_wait_keyword] = ACTIONS(1807),
    [sym_cancel_wait_keyword] = ACTIONS(1807),
    [sym_wait_until_keyword] = ACTIONS(1807),
    [sym_cancel_wait_until_keyword] = ACTIONS(1807),
    [sym_cancel_all_wait_keyword] = ACTIONS(1807),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1807),
    [sym_not] = ACTIONS(1807),
    [sym_bnot] = ACTIONS(1807),
    [sym___file__] = ACTIONS(1807),
    [sym___date__] = ACTIONS(1807),
    [sym___ldate__] = ACTIONS(1807),
    [sym___time__] = ACTIONS(1807),
    [sym___line__] = ACTIONS(1807),
    [sym___name__] = ACTIONS(1807),
    [sym___netlinx__] = ACTIONS(1807),
    [sym_netlinx_axi_version] = ACTIONS(1807),
    [sym_url_flg_tcp] = ACTIONS(1807),
    [sym_url_flg_accinfopresent] = ACTIONS(1807),
    [sym_url_flg_temp] = ACTIONS(1807),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1807),
    [sym_url_flg_stat_mask] = ACTIONS(1807),
    [sym_url_flg_stat_lookup] = ACTIONS(1807),
    [sym_url_flg_stat_connecting] = ACTIONS(1807),
    [sym_url_flg_stat_waiting] = ACTIONS(1807),
    [sym_url_flg_stat_connected] = ACTIONS(1807),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1807),
    [sym_first_virtual_device] = ACTIONS(1807),
    [sym_first_local_port] = ACTIONS(1807),
    [sym_dynamic_virtual_device] = ACTIONS(1807),
    [sym_timeline_once] = ACTIONS(1807),
    [sym_timeline_repeat] = ACTIONS(1807),
    [sym_timeline_absolute] = ACTIONS(1807),
    [sym_timeline_relative] = ACTIONS(1807),
    [sym_do_push_timed_infinite] = ACTIONS(1807),
    [sym_source_type_no_address] = ACTIONS(1807),
    [sym_source_type_neuron_id] = ACTIONS(1807),
    [sym_source_type_ip_address] = ACTIONS(1807),
    [sym_source_type_axlink] = ACTIONS(1807),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1807),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1807),
    [sym_source_type_ip_socket_address] = ACTIONS(1807),
    [sym_source_type_rs232] = ACTIONS(1807),
    [sym_source_type_internal] = ACTIONS(1807),
    [sym_source_type_mac_address] = ACTIONS(1807),
    [sym_source_type_ipv4_port] = ACTIONS(1807),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1807),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1807),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1807),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1807),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1807),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1807),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1807),
    [sym_true] = ACTIONS(1807),
    [sym_false] = ACTIONS(1807),
    [sym_file_read_only] = ACTIONS(1807),
    [sym_file_rw_new] = ACTIONS(1807),
    [sym_file_rw_append] = ACTIONS(1807),
    [sym_ip_tcp] = ACTIONS(1807),
    [sym_ip_udp] = ACTIONS(1807),
    [sym_ip_udp_2way] = ACTIONS(1807),
    [sym_xml_encode_types] = ACTIONS(1807),
    [sym_xml_encode_char_as_list] = ACTIONS(1807),
    [sym_xml_encode_le] = ACTIONS(1807),
    [sym_xml_decode_types] = ACTIONS(1807),
    [sym_xml_decode_no_preserve] = ACTIONS(1807),
    [sym_duet_dev_type_display_device] = ACTIONS(1807),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1807),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1807),
    [sym_duet_dev_type_io_device] = ACTIONS(1807),
    [sym_duet_dev_type_relay_device] = ACTIONS(1807),
    [sym_duet_dev_type_ups] = ACTIONS(1807),
    [sym_duet_dev_type_amplifier] = ACTIONS(1807),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1807),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1807),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1807),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1807),
    [sym_duet_dev_type_camera] = ACTIONS(1807),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1807),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1807),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1807),
    [sym_duet_dev_type_dss] = ACTIONS(1807),
    [sym_duet_dev_type_dvr] = ACTIONS(1807),
    [sym_duet_dev_type_disc_device] = ACTIONS(1807),
    [sym_duet_dev_type_document_camera] = ACTIONS(1807),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1807),
    [sym_duet_dev_type_hvac] = ACTIONS(1807),
    [sym_duet_dev_type_keypad] = ACTIONS(1807),
    [sym_duet_dev_type_light] = ACTIONS(1807),
    [sym_duet_dev_type_monitor] = ACTIONS(1807),
    [sym_duet_dev_type_motor] = ACTIONS(1807),
    [sym_duet_dev_type_multi_window] = ACTIONS(1807),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1807),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1807),
    [sym_duet_dev_type_receiver] = ACTIONS(1807),
    [sym_duet_dev_type_security_system] = ACTIONS(1807),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1807),
    [sym_duet_dev_type_settop_box] = ACTIONS(1807),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1807),
    [sym_duet_dev_type_switcher] = ACTIONS(1807),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1807),
    [sym_duet_dev_type_tv] = ACTIONS(1807),
    [sym_duet_dev_type_utility] = ACTIONS(1807),
    [sym_duet_dev_type_vcr] = ACTIONS(1807),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1807),
    [sym_duet_dev_type_video_processor] = ACTIONS(1807),
    [sym_duet_dev_type_video_projector] = ACTIONS(1807),
    [sym_duet_dev_type_video_wall] = ACTIONS(1807),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1807),
    [sym_duet_dev_type_weather] = ACTIONS(1807),
    [sym_duet_dev_not_polled] = ACTIONS(1807),
    [sym_duet_dev_polled] = ACTIONS(1807),
    [sym_clkmgr_mode_network] = ACTIONS(1807),
    [sym_clkmgr_mode_standalone] = ACTIONS(1807),
    [sym_normal_standby] = ACTIONS(1807),
    [sym_normal_wake] = ACTIONS(1807),
    [sym_ip_multicast_ttl_option] = ACTIONS(1807),
    [sym_ip_ttl_subnet] = ACTIONS(1807),
    [sym_ip_ttl_site] = ACTIONS(1807),
    [sym_ip_ttl_region] = ACTIONS(1807),
    [sym_ip_ttl_continent] = ACTIONS(1807),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1807),
    [sym_ip_nodelay_on] = ACTIONS(1807),
    [sym_ip_nodelay_off] = ACTIONS(1807),
    [sym_amx_error] = ACTIONS(1807),
    [sym_amx_warning] = ACTIONS(1807),
    [sym_amx_info] = ACTIONS(1807),
    [sym_amx_debug] = ACTIONS(1807),
    [sym_netlinx_login_success] = ACTIONS(1807),
    [sym_netlinx_login_fail] = ACTIONS(1807),
    [sym_netlinx_logout] = ACTIONS(1807),
    [sym_null_str] = ACTIONS(1807),
    [sym_smtp_address] = ACTIONS(1807),
    [sym_smtp_port_number] = ACTIONS(1807),
    [sym_smtp_username] = ACTIONS(1807),
    [sym_smtp_password] = ACTIONS(1807),
    [sym_smtp_require_tls] = ACTIONS(1807),
    [sym_smtp_from] = ACTIONS(1807),
    [sym_smtp_tls_true] = ACTIONS(1807),
    [sym_smtp_tls_false] = ACTIONS(1807),
    [sym_listview_on_row_select_event] = ACTIONS(1807),
    [sym_data_structure_unknown] = ACTIONS(1807),
    [sym_data_structure_datafeed] = ACTIONS(1807),
    [sym_data_structure_datarecordset] = ACTIONS(1807),
    [sym_data_structure_datarecord] = ACTIONS(1807),
    [sym_data_structure_datafield] = ACTIONS(1807),
    [sym_data_max_content_field_count] = ACTIONS(1807),
    [sym_data_max_metadata_field_count] = ACTIONS(1807),
    [sym_data_max_id_length] = ACTIONS(1807),
    [sym_data_max_name_length] = ACTIONS(1807),
    [sym_data_max_description_length] = ACTIONS(1807),
    [sym_data_max_type_length] = ACTIONS(1807),
    [sym_data_max_format_length] = ACTIONS(1807),
    [sym_data_max_source_length] = ACTIONS(1807),
    [sym_data_max_value_length] = ACTIONS(1807),
    [sym_data_max_label_length] = ACTIONS(1807),
    [sym_data_max_url_length] = ACTIONS(1807),
    [sym_max_last_login_info_length] = ACTIONS(1807),
    [sym_data_type_unknown] = ACTIONS(1807),
    [sym_data_type_string] = ACTIONS(1807),
    [sym_data_type_datetime] = ACTIONS(1807),
    [sym_data_type_date] = ACTIONS(1807),
    [sym_data_type_time] = ACTIONS(1807),
    [sym_data_type_image] = ACTIONS(1807),
    [sym_data_format_url] = ACTIONS(1807),
    [sym_data_format_phone] = ACTIONS(1807),
    [sym_data_format_email] = ACTIONS(1807),
    [sym_data_format_iso8601] = ACTIONS(1807),
    [sym_auditlog_priv] = ACTIONS(1807),
    [sym_configuration_priv] = ACTIONS(1807),
    [sym_deviceconfig_priv] = ACTIONS(1807),
    [sym_ftp_priv] = ACTIONS(1807),
    [sym_http_priv] = ACTIONS(1807),
    [sym_networkconfig_priv] = ACTIONS(1807),
    [sym_programport_priv] = ACTIONS(1807),
    [sym_remoteui_priv] = ACTIONS(1807),
    [sym_securitycontrol_priv] = ACTIONS(1807),
    [sym_softwaremanagement_priv] = ACTIONS(1807),
    [sym_terminal_priv] = ACTIONS(1807),
    [sym_tpadmin_priv] = ACTIONS(1807),
    [sym_user1_priv] = ACTIONS(1807),
    [sym_user2_priv] = ACTIONS(1807),
    [sym_user3_priv] = ACTIONS(1807),
    [sym_user4_priv] = ACTIONS(1807),
    [sym_usermanagement_priv] = ACTIONS(1807),
    [sym_valid_account] = ACTIONS(1807),
    [sym_err_name_invalid] = ACTIONS(1807),
    [sym_err_password_invalid] = ACTIONS(1807),
    [sym_err_invalid_account] = ACTIONS(1807),
    [sym_err_no_authorization] = ACTIONS(1807),
    [sym_err_invalid_parameter] = ACTIONS(1807),
    [sym_err_user_locked_out] = ACTIONS(1807),
    [sym_err_user_account_expired] = ACTIONS(1807),
    [sym_err_general_error] = ACTIONS(1807),
    [sym_err_authentication_error] = ACTIONS(1807),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1807),
    [sym_tls_validate_certificate] = ACTIONS(1807),
    [sym_tdata] = ACTIONS(1807),
    [sym_tchannel] = ACTIONS(1807),
    [sym_tlevel] = ACTIONS(1807),
    [sym_tbutton] = ACTIONS(1807),
    [sym_ttimeline] = ACTIONS(1807),
    [sym_tcustom] = ACTIONS(1807),
    [sym_url_struct] = ACTIONS(1807),
    [sym_dns_struct] = ACTIONS(1807),
    [sym_ip_address_struct] = ACTIONS(1807),
    [sym_dev_info_struct] = ACTIONS(1807),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1807),
    [sym_data_feed] = ACTIONS(1807),
    [sym_data_field] = ACTIONS(1807),
    [sym_data_record] = ACTIONS(1807),
    [sym_wc_data_feed] = ACTIONS(1807),
    [sym_wc_data_field] = ACTIONS(1807),
    [sym_wc_data_record] = ACTIONS(1807),
    [sym_last_login_info] = ACTIONS(1807),
    [sym____reserved___] = ACTIONS(1807),
    [sym_date] = ACTIONS(1807),
    [sym_day] = ACTIONS(1807),
    [sym_time] = ACTIONS(1807),
    [sym_ldate] = ACTIONS(1807),
    [sym_push_device] = ACTIONS(1807),
    [sym_push_channel] = ACTIONS(1807),
    [sym_push_devchan] = ACTIONS(1807),
    [sym_release_device] = ACTIONS(1807),
    [sym_release_channel] = ACTIONS(1807),
    [sym_release_devchan] = ACTIONS(1807),
    [sym_master_slot] = ACTIONS(1807),
    [sym_get_pulse_time] = ACTIONS(1807),
    [sym_get_timer] = ACTIONS(1807),
    [sym_master_sn] = ACTIONS(1807),
    [sym_system_number] = ACTIONS(1807),
    [sym_button] = ACTIONS(1807),
    [sym_dv_channel] = ACTIONS(1807),
    [sym_data] = ACTIONS(1807),
    [sym_level] = ACTIONS(1807),
    [sym_timeline] = ACTIONS(1807),
    [sym_channel] = ACTIONS(1807),
    [sym_custom] = ACTIONS(1807),
    [sym_atoi] = ACTIONS(1807),
    [sym_atol] = ACTIONS(1807),
    [sym_atof] = ACTIONS(1807),
    [sym_itoa] = ACTIONS(1807),
    [sym_itohex] = ACTIONS(1807),
    [sym_hextoi] = ACTIONS(1807),
    [sym_ftoa] = ACTIONS(1807),
    [sym_find_string] = ACTIONS(1807),
    [sym_left_string] = ACTIONS(1807),
    [sym_right_string] = ACTIONS(1807),
    [sym_mid_string] = ACTIONS(1807),
    [sym_remove_string] = ACTIONS(1807),
    [sym_get_buffer_string] = ACTIONS(1807),
    [sym_get_buffer_char] = ACTIONS(1807),
    [sym_get_multi_buffer_string] = ACTIONS(1807),
    [sym_length_string] = ACTIONS(1807),
    [sym_max_length_string] = ACTIONS(1807),
    [sym_set_length_string] = ACTIONS(1807),
    [sym_compare_string] = ACTIONS(1807),
    [sym_upper_string] = ACTIONS(1807),
    [sym_lower_string] = ACTIONS(1807),
    [sym_get_last] = ACTIONS(1807),
    [sym_date_to_day] = ACTIONS(1807),
    [sym_date_to_month] = ACTIONS(1807),
    [sym_date_to_year] = ACTIONS(1807),
    [sym_time_to_hour] = ACTIONS(1807),
    [sym_time_to_minute] = ACTIONS(1807),
    [sym_time_to_second] = ACTIONS(1807),
    [sym_day_of_week] = ACTIONS(1807),
    [sym_set_pulse_time] = ACTIONS(1807),
    [sym_set_timer] = ACTIONS(1807),
    [sym_file_open] = ACTIONS(1807),
    [sym_file_close] = ACTIONS(1807),
    [sym_file_read] = ACTIONS(1807),
    [sym_file_read_line] = ACTIONS(1807),
    [sym_file_write] = ACTIONS(1807),
    [sym_file_write_line] = ACTIONS(1807),
    [sym_file_copy] = ACTIONS(1807),
    [sym_file_delete] = ACTIONS(1807),
    [sym_file_rename] = ACTIONS(1807),
    [sym_file_dir] = ACTIONS(1807),
    [sym_file_getdir] = ACTIONS(1807),
    [sym_file_setdir] = ACTIONS(1807),
    [sym_file_create_dir] = ACTIONS(1807),
    [sym_file_remove_dir] = ACTIONS(1807),
    [sym_file_seek] = ACTIONS(1807),
    [sym_length_array] = ACTIONS(1807),
    [sym_max_length_array] = ACTIONS(1807),
    [sym_set_length_array] = ACTIONS(1807),
    [sym_ip_client_open] = ACTIONS(1807),
    [sym_ip_client_close] = ACTIONS(1807),
    [sym_ip_server_open] = ACTIONS(1807),
    [sym_ip_server_close] = ACTIONS(1807),
    [sym_ip_bound_client_open] = ACTIONS(1807),
    [sym_ip_mc_server_open] = ACTIONS(1807),
    [sym_ip_set_option] = ACTIONS(1807),
    [sym_ssh_client_open] = ACTIONS(1807),
    [sym_ssh_client_close] = ACTIONS(1807),
    [sym_tls_client_open] = ACTIONS(1807),
    [sym_tls_client_close] = ACTIONS(1807),
    [sym_do_push] = ACTIONS(1807),
    [sym_do_push_timed] = ACTIONS(1807),
    [sym_do_release] = ACTIONS(1807),
    [sym_device_id] = ACTIONS(1807),
    [sym_device_info] = ACTIONS(1807),
    [sym_device_id_string] = ACTIONS(1807),
    [sym_device_standby] = ACTIONS(1807),
    [sym_device_wake] = ACTIONS(1807),
    [sym_timeline_create] = ACTIONS(1807),
    [sym_timeline_kill] = ACTIONS(1807),
    [sym_timeline_reload] = ACTIONS(1807),
    [sym_timeline_pause] = ACTIONS(1807),
    [sym_timeline_restart] = ACTIONS(1807),
    [sym_timeline_set] = ACTIONS(1807),
    [sym_timeline_get] = ACTIONS(1807),
    [sym_timeline_active] = ACTIONS(1807),
    [sym_timeline_dynamic_id] = ACTIONS(1807),
    [sym_variable_to_string] = ACTIONS(1807),
    [sym_string_to_variable] = ACTIONS(1807),
    [sym_length_variable_to_string] = ACTIONS(1807),
    [sym_type_cast] = ACTIONS(1807),
    [sym_variable_to_xml] = ACTIONS(1807),
    [sym_xml_to_variable] = ACTIONS(1807),
    [sym_length_variable_to_xml] = ACTIONS(1807),
    [sym_redirect_string] = ACTIONS(1807),
    [sym_random_number] = ACTIONS(1807),
    [sym_get_unique_id] = ACTIONS(1807),
    [sym_get_system_number] = ACTIONS(1807),
    [sym_set_system_number] = ACTIONS(1807),
    [sym_get_serial_number] = ACTIONS(1807),
    [sym_get_master_build] = ACTIONS(1807),
    [sym_set_validation_code] = ACTIONS(1807),
    [sym_reboot] = ACTIONS(1807),
    [sym_load_duet_module] = ACTIONS(1807),
    [sym_unload_duet_module] = ACTIONS(1807),
    [sym_duet_mem_size_set] = ACTIONS(1807),
    [sym_duet_mem_size_get] = ACTIONS(1807),
    [sym_dynamic_application_device] = ACTIONS(1807),
    [sym_dynamic_polled_port] = ACTIONS(1807),
    [sym_static_port_binding] = ACTIONS(1807),
    [sym_static_ip_binding] = ACTIONS(1807),
    [sym_get_dns_list] = ACTIONS(1807),
    [sym_set_dns_list] = ACTIONS(1807),
    [sym_get_ip_address] = ACTIONS(1807),
    [sym_set_ip_address] = ACTIONS(1807),
    [sym_get_url_list] = ACTIONS(1807),
    [sym_add_url_entry] = ACTIONS(1807),
    [sym_delete_url_entry] = ACTIONS(1807),
    [sym_abs_value] = ACTIONS(1807),
    [sym_max_value] = ACTIONS(1807),
    [sym_min_value] = ACTIONS(1807),
    [sym_exp_value] = ACTIONS(1807),
    [sym_log_value] = ACTIONS(1807),
    [sym_log10_value] = ACTIONS(1807),
    [sym_power_value] = ACTIONS(1807),
    [sym_sqrt_value] = ACTIONS(1807),
    [sym_raw_be] = ACTIONS(1807),
    [sym_raw_le] = ACTIONS(1807),
    [sym_internal_queue_size_set] = ACTIONS(1807),
    [sym_internal_queue_size_get] = ACTIONS(1807),
    [sym_internal_threshold_set] = ACTIONS(1807),
    [sym_internal_threshold_get] = ACTIONS(1807),
    [sym_rebuilt_event] = ACTIONS(1807),
    [sym_do_custom_event] = ACTIONS(1807),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1807),
    [sym_clkmgr_set_clk_source] = ACTIONS(1807),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1807),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1807),
    [sym_clkmgr_get_timezone] = ACTIONS(1807),
    [sym_clkmgr_set_timezone] = ACTIONS(1807),
    [sym_clkmgr_get_resync_period] = ACTIONS(1807),
    [sym_clkmgr_set_resync_period] = ACTIONS(1807),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1807),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1807),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1807),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1807),
    [sym_clkmgr_get_timeservers] = ACTIONS(1807),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1807),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1807),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1807),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1807),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1807),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1807),
    [sym_set_log_level] = ACTIONS(1807),
    [sym_get_log_level] = ACTIONS(1807),
    [sym_amx_log] = ACTIONS(1807),
    [sym_smtp_server_config_set] = ACTIONS(1807),
    [sym_smtp_server_config_get] = ACTIONS(1807),
    [sym_smtp_send] = ACTIONS(1807),
    [sym_validate_netlinx_account] = ACTIONS(1807),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1807),
    [sym_authenticate_certificate] = ACTIONS(1807),
    [sym_audit_netlinx_session_event] = ACTIONS(1807),
    [sym_audit_netlinx_generic_event] = ACTIONS(1807),
    [sym_get_available_flash_disk_space] = ACTIONS(1807),
    [sym_get_max_flash_disk_space] = ACTIONS(1807),
    [sym_data_create_feed] = ACTIONS(1807),
    [sym_data_delete_feed] = ACTIONS(1807),
    [sym_data_publish_feed] = ACTIONS(1807),
    [sym_data_get_published_feed] = ACTIONS(1807),
    [sym_data_add_record] = ACTIONS(1807),
    [sym_data_get_event_record] = ACTIONS(1807),
    [sym__wc_data_create_feed] = ACTIONS(1807),
    [sym__wc_data_add_record] = ACTIONS(1807),
    [sym__wc_data_get_event_record] = ACTIONS(1807),
    [sym_format] = ACTIONS(1807),
    [sym_astro_clock] = ACTIONS(1807),
    [sym_set_outdoor_temperature] = ACTIONS(1807),
    [sym_set_virtual_level_count] = ACTIONS(1807),
    [sym_set_virtual_channel_count] = ACTIONS(1807),
    [sym_set_virtual_port_count] = ACTIONS(1807),
    [sym__wc_encode_int] = ACTIONS(1807),
    [sym__wc_decode_int] = ACTIONS(1807),
    [sym__wc_int] = ACTIONS(1807),
    [sym__wc_to_ch_int] = ACTIONS(1807),
    [sym__ch_to_wc_int] = ACTIONS(1807),
    [sym__wc_find_string_int] = ACTIONS(1807),
    [sym__wc_left_string_int] = ACTIONS(1807),
    [sym__wc_lower_string_int] = ACTIONS(1807),
    [sym__wc_mid_string_int] = ACTIONS(1807),
    [sym__wc_remove_string_int] = ACTIONS(1807),
    [sym__wc_right_string_int] = ACTIONS(1807),
    [sym__wc_upper_string_int] = ACTIONS(1807),
    [sym__wc_compare_string_int] = ACTIONS(1807),
    [sym__wc_get_buffer_char_int] = ACTIONS(1807),
    [sym__wc_get_buffer_string_int] = ACTIONS(1807),
    [sym__wc_concat_string_int] = ACTIONS(1807),
    [sym__wc_file_open_int] = ACTIONS(1807),
    [sym__wc_file_close_int] = ACTIONS(1807),
    [sym__wc_file_read_int] = ACTIONS(1807),
    [sym__wc_file_read_line_int] = ACTIONS(1807),
    [sym__wc_file_write_int] = ACTIONS(1807),
    [sym__wc_file_write_line_int] = ACTIONS(1807),
    [aux_sym_preproc_include_token1] = ACTIONS(1807),
    [aux_sym_preproc_define_token1] = ACTIONS(1809),
    [aux_sym_preproc_warn_token1] = ACTIONS(1809),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1809),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1809),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1809),
    [anon_sym_LBRACK] = ACTIONS(1809),
    [anon_sym_LBRACE] = ACTIONS(1809),
    [anon_sym_RBRACE] = ACTIONS(1809),
    [anon_sym_LPAREN] = ACTIONS(1807),
    [anon_sym_BANG] = ACTIONS(1809),
    [anon_sym_TILDE] = ACTIONS(1809),
    [anon_sym_DASH] = ACTIONS(1807),
    [anon_sym_PLUS] = ACTIONS(1807),
    [anon_sym_DASH_DASH] = ACTIONS(1809),
    [anon_sym_PLUS_PLUS] = ACTIONS(1809),
    [anon_sym_DQUOTE] = ACTIONS(1809),
    [anon_sym_SQUOTE] = ACTIONS(1809),
    [sym_number_literal] = ACTIONS(1809),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(515)] = {
    [sym_identifier] = ACTIONS(1811),
    [sym_define_call_keyword] = ACTIONS(1811),
    [sym_define_function_keyword] = ACTIONS(1811),
    [sym_define_library_function_keyword] = ACTIONS(1811),
    [sym_define_module_keyword] = ACTIONS(1811),
    [sym_char_keyword] = ACTIONS(1811),
    [sym_widechar_keyword] = ACTIONS(1811),
    [sym_integer_keyword] = ACTIONS(1811),
    [sym_sinteger_keyword] = ACTIONS(1811),
    [sym_long_keyword] = ACTIONS(1811),
    [sym_slong_keyword] = ACTIONS(1811),
    [sym_float_keyword] = ACTIONS(1811),
    [sym_double_keyword] = ACTIONS(1811),
    [sym_variant_keyword] = ACTIONS(1811),
    [sym_variantarray_keyword] = ACTIONS(1811),
    [sym_local_var_keyword] = ACTIONS(1811),
    [sym_stack_var_keyword] = ACTIONS(1811),
    [sym_constant_keyword] = ACTIONS(1811),
    [sym_volatile_keyword] = ACTIONS(1811),
    [sym_non_volatile_keyword] = ACTIONS(1811),
    [sym_persistent_keyword] = ACTIONS(1811),
    [sym_dev_keyword] = ACTIONS(1811),
    [sym_devlev_keyword] = ACTIONS(1811),
    [sym_devchan_keyword] = ACTIONS(1811),
    [sym_if_keyword] = ACTIONS(1811),
    [sym_else_keyword] = ACTIONS(1811),
    [sym_switch_keyword] = ACTIONS(1811),
    [sym_case_keyword] = ACTIONS(1811),
    [sym_default_keyword] = ACTIONS(1811),
    [sym_while_keyword] = ACTIONS(1811),
    [sym_for_keyword] = ACTIONS(1811),
    [sym_break_keyword] = ACTIONS(1811),
    [sym_continue_keyword] = ACTIONS(1811),
    [sym_return_keyword] = ACTIONS(1811),
    [sym_select_keyword] = ACTIONS(1811),
    [sym_active_keyword] = ACTIONS(1811),
    [sym_struct_keyword] = ACTIONS(1811),
    [sym_structure_keyword] = ACTIONS(1811),
    [aux_sym_on_keyword_token1] = ACTIONS(1811),
    [aux_sym_off_keyword_token1] = ACTIONS(1811),
    [sym_send_level_keyword] = ACTIONS(1811),
    [sym_send_string_keyword] = ACTIONS(1811),
    [sym_send_command_keyword] = ACTIONS(1811),
    [sym_clear_buffer_keyword] = ACTIONS(1811),
    [sym_create_buffer_keyword] = ACTIONS(1811),
    [sym_create_multi_buffer_keyword] = ACTIONS(1811),
    [sym_call_keyword] = ACTIONS(1811),
    [sym_system_call_keyword] = ACTIONS(1811),
    [sym_devchan_to_keyword] = ACTIONS(1811),
    [sym_devchan_min_to_keyword] = ACTIONS(1811),
    [sym_devchan_total_off_keyword] = ACTIONS(1811),
    [sym_devchan_pulse_keyword] = ACTIONS(1811),
    [sym_wait_keyword] = ACTIONS(1811),
    [sym_cancel_wait_keyword] = ACTIONS(1811),
    [sym_wait_until_keyword] = ACTIONS(1811),
    [sym_cancel_wait_until_keyword] = ACTIONS(1811),
    [sym_cancel_all_wait_keyword] = ACTIONS(1811),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1811),
    [sym_not] = ACTIONS(1811),
    [sym_bnot] = ACTIONS(1811),
    [sym___file__] = ACTIONS(1811),
    [sym___date__] = ACTIONS(1811),
    [sym___ldate__] = ACTIONS(1811),
    [sym___time__] = ACTIONS(1811),
    [sym___line__] = ACTIONS(1811),
    [sym___name__] = ACTIONS(1811),
    [sym___netlinx__] = ACTIONS(1811),
    [sym_netlinx_axi_version] = ACTIONS(1811),
    [sym_url_flg_tcp] = ACTIONS(1811),
    [sym_url_flg_accinfopresent] = ACTIONS(1811),
    [sym_url_flg_temp] = ACTIONS(1811),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1811),
    [sym_url_flg_stat_mask] = ACTIONS(1811),
    [sym_url_flg_stat_lookup] = ACTIONS(1811),
    [sym_url_flg_stat_connecting] = ACTIONS(1811),
    [sym_url_flg_stat_waiting] = ACTIONS(1811),
    [sym_url_flg_stat_connected] = ACTIONS(1811),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1811),
    [sym_first_virtual_device] = ACTIONS(1811),
    [sym_first_local_port] = ACTIONS(1811),
    [sym_dynamic_virtual_device] = ACTIONS(1811),
    [sym_timeline_once] = ACTIONS(1811),
    [sym_timeline_repeat] = ACTIONS(1811),
    [sym_timeline_absolute] = ACTIONS(1811),
    [sym_timeline_relative] = ACTIONS(1811),
    [sym_do_push_timed_infinite] = ACTIONS(1811),
    [sym_source_type_no_address] = ACTIONS(1811),
    [sym_source_type_neuron_id] = ACTIONS(1811),
    [sym_source_type_ip_address] = ACTIONS(1811),
    [sym_source_type_axlink] = ACTIONS(1811),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1811),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1811),
    [sym_source_type_ip_socket_address] = ACTIONS(1811),
    [sym_source_type_rs232] = ACTIONS(1811),
    [sym_source_type_internal] = ACTIONS(1811),
    [sym_source_type_mac_address] = ACTIONS(1811),
    [sym_source_type_ipv4_port] = ACTIONS(1811),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1811),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1811),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1811),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1811),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1811),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1811),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1811),
    [sym_true] = ACTIONS(1811),
    [sym_false] = ACTIONS(1811),
    [sym_file_read_only] = ACTIONS(1811),
    [sym_file_rw_new] = ACTIONS(1811),
    [sym_file_rw_append] = ACTIONS(1811),
    [sym_ip_tcp] = ACTIONS(1811),
    [sym_ip_udp] = ACTIONS(1811),
    [sym_ip_udp_2way] = ACTIONS(1811),
    [sym_xml_encode_types] = ACTIONS(1811),
    [sym_xml_encode_char_as_list] = ACTIONS(1811),
    [sym_xml_encode_le] = ACTIONS(1811),
    [sym_xml_decode_types] = ACTIONS(1811),
    [sym_xml_decode_no_preserve] = ACTIONS(1811),
    [sym_duet_dev_type_display_device] = ACTIONS(1811),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1811),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1811),
    [sym_duet_dev_type_io_device] = ACTIONS(1811),
    [sym_duet_dev_type_relay_device] = ACTIONS(1811),
    [sym_duet_dev_type_ups] = ACTIONS(1811),
    [sym_duet_dev_type_amplifier] = ACTIONS(1811),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1811),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1811),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1811),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1811),
    [sym_duet_dev_type_camera] = ACTIONS(1811),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1811),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1811),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1811),
    [sym_duet_dev_type_dss] = ACTIONS(1811),
    [sym_duet_dev_type_dvr] = ACTIONS(1811),
    [sym_duet_dev_type_disc_device] = ACTIONS(1811),
    [sym_duet_dev_type_document_camera] = ACTIONS(1811),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1811),
    [sym_duet_dev_type_hvac] = ACTIONS(1811),
    [sym_duet_dev_type_keypad] = ACTIONS(1811),
    [sym_duet_dev_type_light] = ACTIONS(1811),
    [sym_duet_dev_type_monitor] = ACTIONS(1811),
    [sym_duet_dev_type_motor] = ACTIONS(1811),
    [sym_duet_dev_type_multi_window] = ACTIONS(1811),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1811),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1811),
    [sym_duet_dev_type_receiver] = ACTIONS(1811),
    [sym_duet_dev_type_security_system] = ACTIONS(1811),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1811),
    [sym_duet_dev_type_settop_box] = ACTIONS(1811),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1811),
    [sym_duet_dev_type_switcher] = ACTIONS(1811),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1811),
    [sym_duet_dev_type_tv] = ACTIONS(1811),
    [sym_duet_dev_type_utility] = ACTIONS(1811),
    [sym_duet_dev_type_vcr] = ACTIONS(1811),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1811),
    [sym_duet_dev_type_video_processor] = ACTIONS(1811),
    [sym_duet_dev_type_video_projector] = ACTIONS(1811),
    [sym_duet_dev_type_video_wall] = ACTIONS(1811),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1811),
    [sym_duet_dev_type_weather] = ACTIONS(1811),
    [sym_duet_dev_not_polled] = ACTIONS(1811),
    [sym_duet_dev_polled] = ACTIONS(1811),
    [sym_clkmgr_mode_network] = ACTIONS(1811),
    [sym_clkmgr_mode_standalone] = ACTIONS(1811),
    [sym_normal_standby] = ACTIONS(1811),
    [sym_normal_wake] = ACTIONS(1811),
    [sym_ip_multicast_ttl_option] = ACTIONS(1811),
    [sym_ip_ttl_subnet] = ACTIONS(1811),
    [sym_ip_ttl_site] = ACTIONS(1811),
    [sym_ip_ttl_region] = ACTIONS(1811),
    [sym_ip_ttl_continent] = ACTIONS(1811),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1811),
    [sym_ip_nodelay_on] = ACTIONS(1811),
    [sym_ip_nodelay_off] = ACTIONS(1811),
    [sym_amx_error] = ACTIONS(1811),
    [sym_amx_warning] = ACTIONS(1811),
    [sym_amx_info] = ACTIONS(1811),
    [sym_amx_debug] = ACTIONS(1811),
    [sym_netlinx_login_success] = ACTIONS(1811),
    [sym_netlinx_login_fail] = ACTIONS(1811),
    [sym_netlinx_logout] = ACTIONS(1811),
    [sym_null_str] = ACTIONS(1811),
    [sym_smtp_address] = ACTIONS(1811),
    [sym_smtp_port_number] = ACTIONS(1811),
    [sym_smtp_username] = ACTIONS(1811),
    [sym_smtp_password] = ACTIONS(1811),
    [sym_smtp_require_tls] = ACTIONS(1811),
    [sym_smtp_from] = ACTIONS(1811),
    [sym_smtp_tls_true] = ACTIONS(1811),
    [sym_smtp_tls_false] = ACTIONS(1811),
    [sym_listview_on_row_select_event] = ACTIONS(1811),
    [sym_data_structure_unknown] = ACTIONS(1811),
    [sym_data_structure_datafeed] = ACTIONS(1811),
    [sym_data_structure_datarecordset] = ACTIONS(1811),
    [sym_data_structure_datarecord] = ACTIONS(1811),
    [sym_data_structure_datafield] = ACTIONS(1811),
    [sym_data_max_content_field_count] = ACTIONS(1811),
    [sym_data_max_metadata_field_count] = ACTIONS(1811),
    [sym_data_max_id_length] = ACTIONS(1811),
    [sym_data_max_name_length] = ACTIONS(1811),
    [sym_data_max_description_length] = ACTIONS(1811),
    [sym_data_max_type_length] = ACTIONS(1811),
    [sym_data_max_format_length] = ACTIONS(1811),
    [sym_data_max_source_length] = ACTIONS(1811),
    [sym_data_max_value_length] = ACTIONS(1811),
    [sym_data_max_label_length] = ACTIONS(1811),
    [sym_data_max_url_length] = ACTIONS(1811),
    [sym_max_last_login_info_length] = ACTIONS(1811),
    [sym_data_type_unknown] = ACTIONS(1811),
    [sym_data_type_string] = ACTIONS(1811),
    [sym_data_type_datetime] = ACTIONS(1811),
    [sym_data_type_date] = ACTIONS(1811),
    [sym_data_type_time] = ACTIONS(1811),
    [sym_data_type_image] = ACTIONS(1811),
    [sym_data_format_url] = ACTIONS(1811),
    [sym_data_format_phone] = ACTIONS(1811),
    [sym_data_format_email] = ACTIONS(1811),
    [sym_data_format_iso8601] = ACTIONS(1811),
    [sym_auditlog_priv] = ACTIONS(1811),
    [sym_configuration_priv] = ACTIONS(1811),
    [sym_deviceconfig_priv] = ACTIONS(1811),
    [sym_ftp_priv] = ACTIONS(1811),
    [sym_http_priv] = ACTIONS(1811),
    [sym_networkconfig_priv] = ACTIONS(1811),
    [sym_programport_priv] = ACTIONS(1811),
    [sym_remoteui_priv] = ACTIONS(1811),
    [sym_securitycontrol_priv] = ACTIONS(1811),
    [sym_softwaremanagement_priv] = ACTIONS(1811),
    [sym_terminal_priv] = ACTIONS(1811),
    [sym_tpadmin_priv] = ACTIONS(1811),
    [sym_user1_priv] = ACTIONS(1811),
    [sym_user2_priv] = ACTIONS(1811),
    [sym_user3_priv] = ACTIONS(1811),
    [sym_user4_priv] = ACTIONS(1811),
    [sym_usermanagement_priv] = ACTIONS(1811),
    [sym_valid_account] = ACTIONS(1811),
    [sym_err_name_invalid] = ACTIONS(1811),
    [sym_err_password_invalid] = ACTIONS(1811),
    [sym_err_invalid_account] = ACTIONS(1811),
    [sym_err_no_authorization] = ACTIONS(1811),
    [sym_err_invalid_parameter] = ACTIONS(1811),
    [sym_err_user_locked_out] = ACTIONS(1811),
    [sym_err_user_account_expired] = ACTIONS(1811),
    [sym_err_general_error] = ACTIONS(1811),
    [sym_err_authentication_error] = ACTIONS(1811),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1811),
    [sym_tls_validate_certificate] = ACTIONS(1811),
    [sym_tdata] = ACTIONS(1811),
    [sym_tchannel] = ACTIONS(1811),
    [sym_tlevel] = ACTIONS(1811),
    [sym_tbutton] = ACTIONS(1811),
    [sym_ttimeline] = ACTIONS(1811),
    [sym_tcustom] = ACTIONS(1811),
    [sym_url_struct] = ACTIONS(1811),
    [sym_dns_struct] = ACTIONS(1811),
    [sym_ip_address_struct] = ACTIONS(1811),
    [sym_dev_info_struct] = ACTIONS(1811),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1811),
    [sym_data_feed] = ACTIONS(1811),
    [sym_data_field] = ACTIONS(1811),
    [sym_data_record] = ACTIONS(1811),
    [sym_wc_data_feed] = ACTIONS(1811),
    [sym_wc_data_field] = ACTIONS(1811),
    [sym_wc_data_record] = ACTIONS(1811),
    [sym_last_login_info] = ACTIONS(1811),
    [sym____reserved___] = ACTIONS(1811),
    [sym_date] = ACTIONS(1811),
    [sym_day] = ACTIONS(1811),
    [sym_time] = ACTIONS(1811),
    [sym_ldate] = ACTIONS(1811),
    [sym_push_device] = ACTIONS(1811),
    [sym_push_channel] = ACTIONS(1811),
    [sym_push_devchan] = ACTIONS(1811),
    [sym_release_device] = ACTIONS(1811),
    [sym_release_channel] = ACTIONS(1811),
    [sym_release_devchan] = ACTIONS(1811),
    [sym_master_slot] = ACTIONS(1811),
    [sym_get_pulse_time] = ACTIONS(1811),
    [sym_get_timer] = ACTIONS(1811),
    [sym_master_sn] = ACTIONS(1811),
    [sym_system_number] = ACTIONS(1811),
    [sym_button] = ACTIONS(1811),
    [sym_dv_channel] = ACTIONS(1811),
    [sym_data] = ACTIONS(1811),
    [sym_level] = ACTIONS(1811),
    [sym_timeline] = ACTIONS(1811),
    [sym_channel] = ACTIONS(1811),
    [sym_custom] = ACTIONS(1811),
    [sym_atoi] = ACTIONS(1811),
    [sym_atol] = ACTIONS(1811),
    [sym_atof] = ACTIONS(1811),
    [sym_itoa] = ACTIONS(1811),
    [sym_itohex] = ACTIONS(1811),
    [sym_hextoi] = ACTIONS(1811),
    [sym_ftoa] = ACTIONS(1811),
    [sym_find_string] = ACTIONS(1811),
    [sym_left_string] = ACTIONS(1811),
    [sym_right_string] = ACTIONS(1811),
    [sym_mid_string] = ACTIONS(1811),
    [sym_remove_string] = ACTIONS(1811),
    [sym_get_buffer_string] = ACTIONS(1811),
    [sym_get_buffer_char] = ACTIONS(1811),
    [sym_get_multi_buffer_string] = ACTIONS(1811),
    [sym_length_string] = ACTIONS(1811),
    [sym_max_length_string] = ACTIONS(1811),
    [sym_set_length_string] = ACTIONS(1811),
    [sym_compare_string] = ACTIONS(1811),
    [sym_upper_string] = ACTIONS(1811),
    [sym_lower_string] = ACTIONS(1811),
    [sym_get_last] = ACTIONS(1811),
    [sym_date_to_day] = ACTIONS(1811),
    [sym_date_to_month] = ACTIONS(1811),
    [sym_date_to_year] = ACTIONS(1811),
    [sym_time_to_hour] = ACTIONS(1811),
    [sym_time_to_minute] = ACTIONS(1811),
    [sym_time_to_second] = ACTIONS(1811),
    [sym_day_of_week] = ACTIONS(1811),
    [sym_set_pulse_time] = ACTIONS(1811),
    [sym_set_timer] = ACTIONS(1811),
    [sym_file_open] = ACTIONS(1811),
    [sym_file_close] = ACTIONS(1811),
    [sym_file_read] = ACTIONS(1811),
    [sym_file_read_line] = ACTIONS(1811),
    [sym_file_write] = ACTIONS(1811),
    [sym_file_write_line] = ACTIONS(1811),
    [sym_file_copy] = ACTIONS(1811),
    [sym_file_delete] = ACTIONS(1811),
    [sym_file_rename] = ACTIONS(1811),
    [sym_file_dir] = ACTIONS(1811),
    [sym_file_getdir] = ACTIONS(1811),
    [sym_file_setdir] = ACTIONS(1811),
    [sym_file_create_dir] = ACTIONS(1811),
    [sym_file_remove_dir] = ACTIONS(1811),
    [sym_file_seek] = ACTIONS(1811),
    [sym_length_array] = ACTIONS(1811),
    [sym_max_length_array] = ACTIONS(1811),
    [sym_set_length_array] = ACTIONS(1811),
    [sym_ip_client_open] = ACTIONS(1811),
    [sym_ip_client_close] = ACTIONS(1811),
    [sym_ip_server_open] = ACTIONS(1811),
    [sym_ip_server_close] = ACTIONS(1811),
    [sym_ip_bound_client_open] = ACTIONS(1811),
    [sym_ip_mc_server_open] = ACTIONS(1811),
    [sym_ip_set_option] = ACTIONS(1811),
    [sym_ssh_client_open] = ACTIONS(1811),
    [sym_ssh_client_close] = ACTIONS(1811),
    [sym_tls_client_open] = ACTIONS(1811),
    [sym_tls_client_close] = ACTIONS(1811),
    [sym_do_push] = ACTIONS(1811),
    [sym_do_push_timed] = ACTIONS(1811),
    [sym_do_release] = ACTIONS(1811),
    [sym_device_id] = ACTIONS(1811),
    [sym_device_info] = ACTIONS(1811),
    [sym_device_id_string] = ACTIONS(1811),
    [sym_device_standby] = ACTIONS(1811),
    [sym_device_wake] = ACTIONS(1811),
    [sym_timeline_create] = ACTIONS(1811),
    [sym_timeline_kill] = ACTIONS(1811),
    [sym_timeline_reload] = ACTIONS(1811),
    [sym_timeline_pause] = ACTIONS(1811),
    [sym_timeline_restart] = ACTIONS(1811),
    [sym_timeline_set] = ACTIONS(1811),
    [sym_timeline_get] = ACTIONS(1811),
    [sym_timeline_active] = ACTIONS(1811),
    [sym_timeline_dynamic_id] = ACTIONS(1811),
    [sym_variable_to_string] = ACTIONS(1811),
    [sym_string_to_variable] = ACTIONS(1811),
    [sym_length_variable_to_string] = ACTIONS(1811),
    [sym_type_cast] = ACTIONS(1811),
    [sym_variable_to_xml] = ACTIONS(1811),
    [sym_xml_to_variable] = ACTIONS(1811),
    [sym_length_variable_to_xml] = ACTIONS(1811),
    [sym_redirect_string] = ACTIONS(1811),
    [sym_random_number] = ACTIONS(1811),
    [sym_get_unique_id] = ACTIONS(1811),
    [sym_get_system_number] = ACTIONS(1811),
    [sym_set_system_number] = ACTIONS(1811),
    [sym_get_serial_number] = ACTIONS(1811),
    [sym_get_master_build] = ACTIONS(1811),
    [sym_set_validation_code] = ACTIONS(1811),
    [sym_reboot] = ACTIONS(1811),
    [sym_load_duet_module] = ACTIONS(1811),
    [sym_unload_duet_module] = ACTIONS(1811),
    [sym_duet_mem_size_set] = ACTIONS(1811),
    [sym_duet_mem_size_get] = ACTIONS(1811),
    [sym_dynamic_application_device] = ACTIONS(1811),
    [sym_dynamic_polled_port] = ACTIONS(1811),
    [sym_static_port_binding] = ACTIONS(1811),
    [sym_static_ip_binding] = ACTIONS(1811),
    [sym_get_dns_list] = ACTIONS(1811),
    [sym_set_dns_list] = ACTIONS(1811),
    [sym_get_ip_address] = ACTIONS(1811),
    [sym_set_ip_address] = ACTIONS(1811),
    [sym_get_url_list] = ACTIONS(1811),
    [sym_add_url_entry] = ACTIONS(1811),
    [sym_delete_url_entry] = ACTIONS(1811),
    [sym_abs_value] = ACTIONS(1811),
    [sym_max_value] = ACTIONS(1811),
    [sym_min_value] = ACTIONS(1811),
    [sym_exp_value] = ACTIONS(1811),
    [sym_log_value] = ACTIONS(1811),
    [sym_log10_value] = ACTIONS(1811),
    [sym_power_value] = ACTIONS(1811),
    [sym_sqrt_value] = ACTIONS(1811),
    [sym_raw_be] = ACTIONS(1811),
    [sym_raw_le] = ACTIONS(1811),
    [sym_internal_queue_size_set] = ACTIONS(1811),
    [sym_internal_queue_size_get] = ACTIONS(1811),
    [sym_internal_threshold_set] = ACTIONS(1811),
    [sym_internal_threshold_get] = ACTIONS(1811),
    [sym_rebuilt_event] = ACTIONS(1811),
    [sym_do_custom_event] = ACTIONS(1811),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1811),
    [sym_clkmgr_set_clk_source] = ACTIONS(1811),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1811),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1811),
    [sym_clkmgr_get_timezone] = ACTIONS(1811),
    [sym_clkmgr_set_timezone] = ACTIONS(1811),
    [sym_clkmgr_get_resync_period] = ACTIONS(1811),
    [sym_clkmgr_set_resync_period] = ACTIONS(1811),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1811),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1811),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1811),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1811),
    [sym_clkmgr_get_timeservers] = ACTIONS(1811),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1811),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1811),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1811),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1811),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1811),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1811),
    [sym_set_log_level] = ACTIONS(1811),
    [sym_get_log_level] = ACTIONS(1811),
    [sym_amx_log] = ACTIONS(1811),
    [sym_smtp_server_config_set] = ACTIONS(1811),
    [sym_smtp_server_config_get] = ACTIONS(1811),
    [sym_smtp_send] = ACTIONS(1811),
    [sym_validate_netlinx_account] = ACTIONS(1811),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1811),
    [sym_authenticate_certificate] = ACTIONS(1811),
    [sym_audit_netlinx_session_event] = ACTIONS(1811),
    [sym_audit_netlinx_generic_event] = ACTIONS(1811),
    [sym_get_available_flash_disk_space] = ACTIONS(1811),
    [sym_get_max_flash_disk_space] = ACTIONS(1811),
    [sym_data_create_feed] = ACTIONS(1811),
    [sym_data_delete_feed] = ACTIONS(1811),
    [sym_data_publish_feed] = ACTIONS(1811),
    [sym_data_get_published_feed] = ACTIONS(1811),
    [sym_data_add_record] = ACTIONS(1811),
    [sym_data_get_event_record] = ACTIONS(1811),
    [sym__wc_data_create_feed] = ACTIONS(1811),
    [sym__wc_data_add_record] = ACTIONS(1811),
    [sym__wc_data_get_event_record] = ACTIONS(1811),
    [sym_format] = ACTIONS(1811),
    [sym_astro_clock] = ACTIONS(1811),
    [sym_set_outdoor_temperature] = ACTIONS(1811),
    [sym_set_virtual_level_count] = ACTIONS(1811),
    [sym_set_virtual_channel_count] = ACTIONS(1811),
    [sym_set_virtual_port_count] = ACTIONS(1811),
    [sym__wc_encode_int] = ACTIONS(1811),
    [sym__wc_decode_int] = ACTIONS(1811),
    [sym__wc_int] = ACTIONS(1811),
    [sym__wc_to_ch_int] = ACTIONS(1811),
    [sym__ch_to_wc_int] = ACTIONS(1811),
    [sym__wc_find_string_int] = ACTIONS(1811),
    [sym__wc_left_string_int] = ACTIONS(1811),
    [sym__wc_lower_string_int] = ACTIONS(1811),
    [sym__wc_mid_string_int] = ACTIONS(1811),
    [sym__wc_remove_string_int] = ACTIONS(1811),
    [sym__wc_right_string_int] = ACTIONS(1811),
    [sym__wc_upper_string_int] = ACTIONS(1811),
    [sym__wc_compare_string_int] = ACTIONS(1811),
    [sym__wc_get_buffer_char_int] = ACTIONS(1811),
    [sym__wc_get_buffer_string_int] = ACTIONS(1811),
    [sym__wc_concat_string_int] = ACTIONS(1811),
    [sym__wc_file_open_int] = ACTIONS(1811),
    [sym__wc_file_close_int] = ACTIONS(1811),
    [sym__wc_file_read_int] = ACTIONS(1811),
    [sym__wc_file_read_line_int] = ACTIONS(1811),
    [sym__wc_file_write_int] = ACTIONS(1811),
    [sym__wc_file_write_line_int] = ACTIONS(1811),
    [aux_sym_preproc_include_token1] = ACTIONS(1811),
    [aux_sym_preproc_define_token1] = ACTIONS(1813),
    [aux_sym_preproc_warn_token1] = ACTIONS(1813),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1813),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1813),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1813),
    [anon_sym_LBRACK] = ACTIONS(1813),
    [anon_sym_LBRACE] = ACTIONS(1813),
    [anon_sym_RBRACE] = ACTIONS(1813),
    [anon_sym_LPAREN] = ACTIONS(1811),
    [anon_sym_BANG] = ACTIONS(1813),
    [anon_sym_TILDE] = ACTIONS(1813),
    [anon_sym_DASH] = ACTIONS(1811),
    [anon_sym_PLUS] = ACTIONS(1811),
    [anon_sym_DASH_DASH] = ACTIONS(1813),
    [anon_sym_PLUS_PLUS] = ACTIONS(1813),
    [anon_sym_DQUOTE] = ACTIONS(1813),
    [anon_sym_SQUOTE] = ACTIONS(1813),
    [sym_number_literal] = ACTIONS(1813),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(516)] = {
    [sym_identifier] = ACTIONS(1815),
    [sym_define_call_keyword] = ACTIONS(1815),
    [sym_define_function_keyword] = ACTIONS(1815),
    [sym_define_library_function_keyword] = ACTIONS(1815),
    [sym_define_module_keyword] = ACTIONS(1815),
    [sym_char_keyword] = ACTIONS(1815),
    [sym_widechar_keyword] = ACTIONS(1815),
    [sym_integer_keyword] = ACTIONS(1815),
    [sym_sinteger_keyword] = ACTIONS(1815),
    [sym_long_keyword] = ACTIONS(1815),
    [sym_slong_keyword] = ACTIONS(1815),
    [sym_float_keyword] = ACTIONS(1815),
    [sym_double_keyword] = ACTIONS(1815),
    [sym_variant_keyword] = ACTIONS(1815),
    [sym_variantarray_keyword] = ACTIONS(1815),
    [sym_local_var_keyword] = ACTIONS(1815),
    [sym_stack_var_keyword] = ACTIONS(1815),
    [sym_constant_keyword] = ACTIONS(1815),
    [sym_volatile_keyword] = ACTIONS(1815),
    [sym_non_volatile_keyword] = ACTIONS(1815),
    [sym_persistent_keyword] = ACTIONS(1815),
    [sym_dev_keyword] = ACTIONS(1815),
    [sym_devlev_keyword] = ACTIONS(1815),
    [sym_devchan_keyword] = ACTIONS(1815),
    [sym_if_keyword] = ACTIONS(1815),
    [sym_else_keyword] = ACTIONS(1815),
    [sym_switch_keyword] = ACTIONS(1815),
    [sym_case_keyword] = ACTIONS(1815),
    [sym_default_keyword] = ACTIONS(1815),
    [sym_while_keyword] = ACTIONS(1815),
    [sym_for_keyword] = ACTIONS(1815),
    [sym_break_keyword] = ACTIONS(1815),
    [sym_continue_keyword] = ACTIONS(1815),
    [sym_return_keyword] = ACTIONS(1815),
    [sym_select_keyword] = ACTIONS(1815),
    [sym_active_keyword] = ACTIONS(1815),
    [sym_struct_keyword] = ACTIONS(1815),
    [sym_structure_keyword] = ACTIONS(1815),
    [aux_sym_on_keyword_token1] = ACTIONS(1815),
    [aux_sym_off_keyword_token1] = ACTIONS(1815),
    [sym_send_level_keyword] = ACTIONS(1815),
    [sym_send_string_keyword] = ACTIONS(1815),
    [sym_send_command_keyword] = ACTIONS(1815),
    [sym_clear_buffer_keyword] = ACTIONS(1815),
    [sym_create_buffer_keyword] = ACTIONS(1815),
    [sym_create_multi_buffer_keyword] = ACTIONS(1815),
    [sym_call_keyword] = ACTIONS(1815),
    [sym_system_call_keyword] = ACTIONS(1815),
    [sym_devchan_to_keyword] = ACTIONS(1815),
    [sym_devchan_min_to_keyword] = ACTIONS(1815),
    [sym_devchan_total_off_keyword] = ACTIONS(1815),
    [sym_devchan_pulse_keyword] = ACTIONS(1815),
    [sym_wait_keyword] = ACTIONS(1815),
    [sym_cancel_wait_keyword] = ACTIONS(1815),
    [sym_wait_until_keyword] = ACTIONS(1815),
    [sym_cancel_wait_until_keyword] = ACTIONS(1815),
    [sym_cancel_all_wait_keyword] = ACTIONS(1815),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1815),
    [sym_not] = ACTIONS(1815),
    [sym_bnot] = ACTIONS(1815),
    [sym___file__] = ACTIONS(1815),
    [sym___date__] = ACTIONS(1815),
    [sym___ldate__] = ACTIONS(1815),
    [sym___time__] = ACTIONS(1815),
    [sym___line__] = ACTIONS(1815),
    [sym___name__] = ACTIONS(1815),
    [sym___netlinx__] = ACTIONS(1815),
    [sym_netlinx_axi_version] = ACTIONS(1815),
    [sym_url_flg_tcp] = ACTIONS(1815),
    [sym_url_flg_accinfopresent] = ACTIONS(1815),
    [sym_url_flg_temp] = ACTIONS(1815),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1815),
    [sym_url_flg_stat_mask] = ACTIONS(1815),
    [sym_url_flg_stat_lookup] = ACTIONS(1815),
    [sym_url_flg_stat_connecting] = ACTIONS(1815),
    [sym_url_flg_stat_waiting] = ACTIONS(1815),
    [sym_url_flg_stat_connected] = ACTIONS(1815),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1815),
    [sym_first_virtual_device] = ACTIONS(1815),
    [sym_first_local_port] = ACTIONS(1815),
    [sym_dynamic_virtual_device] = ACTIONS(1815),
    [sym_timeline_once] = ACTIONS(1815),
    [sym_timeline_repeat] = ACTIONS(1815),
    [sym_timeline_absolute] = ACTIONS(1815),
    [sym_timeline_relative] = ACTIONS(1815),
    [sym_do_push_timed_infinite] = ACTIONS(1815),
    [sym_source_type_no_address] = ACTIONS(1815),
    [sym_source_type_neuron_id] = ACTIONS(1815),
    [sym_source_type_ip_address] = ACTIONS(1815),
    [sym_source_type_axlink] = ACTIONS(1815),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1815),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1815),
    [sym_source_type_ip_socket_address] = ACTIONS(1815),
    [sym_source_type_rs232] = ACTIONS(1815),
    [sym_source_type_internal] = ACTIONS(1815),
    [sym_source_type_mac_address] = ACTIONS(1815),
    [sym_source_type_ipv4_port] = ACTIONS(1815),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1815),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1815),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1815),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1815),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1815),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1815),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1815),
    [sym_true] = ACTIONS(1815),
    [sym_false] = ACTIONS(1815),
    [sym_file_read_only] = ACTIONS(1815),
    [sym_file_rw_new] = ACTIONS(1815),
    [sym_file_rw_append] = ACTIONS(1815),
    [sym_ip_tcp] = ACTIONS(1815),
    [sym_ip_udp] = ACTIONS(1815),
    [sym_ip_udp_2way] = ACTIONS(1815),
    [sym_xml_encode_types] = ACTIONS(1815),
    [sym_xml_encode_char_as_list] = ACTIONS(1815),
    [sym_xml_encode_le] = ACTIONS(1815),
    [sym_xml_decode_types] = ACTIONS(1815),
    [sym_xml_decode_no_preserve] = ACTIONS(1815),
    [sym_duet_dev_type_display_device] = ACTIONS(1815),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1815),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1815),
    [sym_duet_dev_type_io_device] = ACTIONS(1815),
    [sym_duet_dev_type_relay_device] = ACTIONS(1815),
    [sym_duet_dev_type_ups] = ACTIONS(1815),
    [sym_duet_dev_type_amplifier] = ACTIONS(1815),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1815),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1815),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1815),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1815),
    [sym_duet_dev_type_camera] = ACTIONS(1815),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1815),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1815),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1815),
    [sym_duet_dev_type_dss] = ACTIONS(1815),
    [sym_duet_dev_type_dvr] = ACTIONS(1815),
    [sym_duet_dev_type_disc_device] = ACTIONS(1815),
    [sym_duet_dev_type_document_camera] = ACTIONS(1815),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1815),
    [sym_duet_dev_type_hvac] = ACTIONS(1815),
    [sym_duet_dev_type_keypad] = ACTIONS(1815),
    [sym_duet_dev_type_light] = ACTIONS(1815),
    [sym_duet_dev_type_monitor] = ACTIONS(1815),
    [sym_duet_dev_type_motor] = ACTIONS(1815),
    [sym_duet_dev_type_multi_window] = ACTIONS(1815),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1815),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1815),
    [sym_duet_dev_type_receiver] = ACTIONS(1815),
    [sym_duet_dev_type_security_system] = ACTIONS(1815),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1815),
    [sym_duet_dev_type_settop_box] = ACTIONS(1815),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1815),
    [sym_duet_dev_type_switcher] = ACTIONS(1815),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1815),
    [sym_duet_dev_type_tv] = ACTIONS(1815),
    [sym_duet_dev_type_utility] = ACTIONS(1815),
    [sym_duet_dev_type_vcr] = ACTIONS(1815),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1815),
    [sym_duet_dev_type_video_processor] = ACTIONS(1815),
    [sym_duet_dev_type_video_projector] = ACTIONS(1815),
    [sym_duet_dev_type_video_wall] = ACTIONS(1815),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1815),
    [sym_duet_dev_type_weather] = ACTIONS(1815),
    [sym_duet_dev_not_polled] = ACTIONS(1815),
    [sym_duet_dev_polled] = ACTIONS(1815),
    [sym_clkmgr_mode_network] = ACTIONS(1815),
    [sym_clkmgr_mode_standalone] = ACTIONS(1815),
    [sym_normal_standby] = ACTIONS(1815),
    [sym_normal_wake] = ACTIONS(1815),
    [sym_ip_multicast_ttl_option] = ACTIONS(1815),
    [sym_ip_ttl_subnet] = ACTIONS(1815),
    [sym_ip_ttl_site] = ACTIONS(1815),
    [sym_ip_ttl_region] = ACTIONS(1815),
    [sym_ip_ttl_continent] = ACTIONS(1815),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1815),
    [sym_ip_nodelay_on] = ACTIONS(1815),
    [sym_ip_nodelay_off] = ACTIONS(1815),
    [sym_amx_error] = ACTIONS(1815),
    [sym_amx_warning] = ACTIONS(1815),
    [sym_amx_info] = ACTIONS(1815),
    [sym_amx_debug] = ACTIONS(1815),
    [sym_netlinx_login_success] = ACTIONS(1815),
    [sym_netlinx_login_fail] = ACTIONS(1815),
    [sym_netlinx_logout] = ACTIONS(1815),
    [sym_null_str] = ACTIONS(1815),
    [sym_smtp_address] = ACTIONS(1815),
    [sym_smtp_port_number] = ACTIONS(1815),
    [sym_smtp_username] = ACTIONS(1815),
    [sym_smtp_password] = ACTIONS(1815),
    [sym_smtp_require_tls] = ACTIONS(1815),
    [sym_smtp_from] = ACTIONS(1815),
    [sym_smtp_tls_true] = ACTIONS(1815),
    [sym_smtp_tls_false] = ACTIONS(1815),
    [sym_listview_on_row_select_event] = ACTIONS(1815),
    [sym_data_structure_unknown] = ACTIONS(1815),
    [sym_data_structure_datafeed] = ACTIONS(1815),
    [sym_data_structure_datarecordset] = ACTIONS(1815),
    [sym_data_structure_datarecord] = ACTIONS(1815),
    [sym_data_structure_datafield] = ACTIONS(1815),
    [sym_data_max_content_field_count] = ACTIONS(1815),
    [sym_data_max_metadata_field_count] = ACTIONS(1815),
    [sym_data_max_id_length] = ACTIONS(1815),
    [sym_data_max_name_length] = ACTIONS(1815),
    [sym_data_max_description_length] = ACTIONS(1815),
    [sym_data_max_type_length] = ACTIONS(1815),
    [sym_data_max_format_length] = ACTIONS(1815),
    [sym_data_max_source_length] = ACTIONS(1815),
    [sym_data_max_value_length] = ACTIONS(1815),
    [sym_data_max_label_length] = ACTIONS(1815),
    [sym_data_max_url_length] = ACTIONS(1815),
    [sym_max_last_login_info_length] = ACTIONS(1815),
    [sym_data_type_unknown] = ACTIONS(1815),
    [sym_data_type_string] = ACTIONS(1815),
    [sym_data_type_datetime] = ACTIONS(1815),
    [sym_data_type_date] = ACTIONS(1815),
    [sym_data_type_time] = ACTIONS(1815),
    [sym_data_type_image] = ACTIONS(1815),
    [sym_data_format_url] = ACTIONS(1815),
    [sym_data_format_phone] = ACTIONS(1815),
    [sym_data_format_email] = ACTIONS(1815),
    [sym_data_format_iso8601] = ACTIONS(1815),
    [sym_auditlog_priv] = ACTIONS(1815),
    [sym_configuration_priv] = ACTIONS(1815),
    [sym_deviceconfig_priv] = ACTIONS(1815),
    [sym_ftp_priv] = ACTIONS(1815),
    [sym_http_priv] = ACTIONS(1815),
    [sym_networkconfig_priv] = ACTIONS(1815),
    [sym_programport_priv] = ACTIONS(1815),
    [sym_remoteui_priv] = ACTIONS(1815),
    [sym_securitycontrol_priv] = ACTIONS(1815),
    [sym_softwaremanagement_priv] = ACTIONS(1815),
    [sym_terminal_priv] = ACTIONS(1815),
    [sym_tpadmin_priv] = ACTIONS(1815),
    [sym_user1_priv] = ACTIONS(1815),
    [sym_user2_priv] = ACTIONS(1815),
    [sym_user3_priv] = ACTIONS(1815),
    [sym_user4_priv] = ACTIONS(1815),
    [sym_usermanagement_priv] = ACTIONS(1815),
    [sym_valid_account] = ACTIONS(1815),
    [sym_err_name_invalid] = ACTIONS(1815),
    [sym_err_password_invalid] = ACTIONS(1815),
    [sym_err_invalid_account] = ACTIONS(1815),
    [sym_err_no_authorization] = ACTIONS(1815),
    [sym_err_invalid_parameter] = ACTIONS(1815),
    [sym_err_user_locked_out] = ACTIONS(1815),
    [sym_err_user_account_expired] = ACTIONS(1815),
    [sym_err_general_error] = ACTIONS(1815),
    [sym_err_authentication_error] = ACTIONS(1815),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1815),
    [sym_tls_validate_certificate] = ACTIONS(1815),
    [sym_tdata] = ACTIONS(1815),
    [sym_tchannel] = ACTIONS(1815),
    [sym_tlevel] = ACTIONS(1815),
    [sym_tbutton] = ACTIONS(1815),
    [sym_ttimeline] = ACTIONS(1815),
    [sym_tcustom] = ACTIONS(1815),
    [sym_url_struct] = ACTIONS(1815),
    [sym_dns_struct] = ACTIONS(1815),
    [sym_ip_address_struct] = ACTIONS(1815),
    [sym_dev_info_struct] = ACTIONS(1815),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1815),
    [sym_data_feed] = ACTIONS(1815),
    [sym_data_field] = ACTIONS(1815),
    [sym_data_record] = ACTIONS(1815),
    [sym_wc_data_feed] = ACTIONS(1815),
    [sym_wc_data_field] = ACTIONS(1815),
    [sym_wc_data_record] = ACTIONS(1815),
    [sym_last_login_info] = ACTIONS(1815),
    [sym____reserved___] = ACTIONS(1815),
    [sym_date] = ACTIONS(1815),
    [sym_day] = ACTIONS(1815),
    [sym_time] = ACTIONS(1815),
    [sym_ldate] = ACTIONS(1815),
    [sym_push_device] = ACTIONS(1815),
    [sym_push_channel] = ACTIONS(1815),
    [sym_push_devchan] = ACTIONS(1815),
    [sym_release_device] = ACTIONS(1815),
    [sym_release_channel] = ACTIONS(1815),
    [sym_release_devchan] = ACTIONS(1815),
    [sym_master_slot] = ACTIONS(1815),
    [sym_get_pulse_time] = ACTIONS(1815),
    [sym_get_timer] = ACTIONS(1815),
    [sym_master_sn] = ACTIONS(1815),
    [sym_system_number] = ACTIONS(1815),
    [sym_button] = ACTIONS(1815),
    [sym_dv_channel] = ACTIONS(1815),
    [sym_data] = ACTIONS(1815),
    [sym_level] = ACTIONS(1815),
    [sym_timeline] = ACTIONS(1815),
    [sym_channel] = ACTIONS(1815),
    [sym_custom] = ACTIONS(1815),
    [sym_atoi] = ACTIONS(1815),
    [sym_atol] = ACTIONS(1815),
    [sym_atof] = ACTIONS(1815),
    [sym_itoa] = ACTIONS(1815),
    [sym_itohex] = ACTIONS(1815),
    [sym_hextoi] = ACTIONS(1815),
    [sym_ftoa] = ACTIONS(1815),
    [sym_find_string] = ACTIONS(1815),
    [sym_left_string] = ACTIONS(1815),
    [sym_right_string] = ACTIONS(1815),
    [sym_mid_string] = ACTIONS(1815),
    [sym_remove_string] = ACTIONS(1815),
    [sym_get_buffer_string] = ACTIONS(1815),
    [sym_get_buffer_char] = ACTIONS(1815),
    [sym_get_multi_buffer_string] = ACTIONS(1815),
    [sym_length_string] = ACTIONS(1815),
    [sym_max_length_string] = ACTIONS(1815),
    [sym_set_length_string] = ACTIONS(1815),
    [sym_compare_string] = ACTIONS(1815),
    [sym_upper_string] = ACTIONS(1815),
    [sym_lower_string] = ACTIONS(1815),
    [sym_get_last] = ACTIONS(1815),
    [sym_date_to_day] = ACTIONS(1815),
    [sym_date_to_month] = ACTIONS(1815),
    [sym_date_to_year] = ACTIONS(1815),
    [sym_time_to_hour] = ACTIONS(1815),
    [sym_time_to_minute] = ACTIONS(1815),
    [sym_time_to_second] = ACTIONS(1815),
    [sym_day_of_week] = ACTIONS(1815),
    [sym_set_pulse_time] = ACTIONS(1815),
    [sym_set_timer] = ACTIONS(1815),
    [sym_file_open] = ACTIONS(1815),
    [sym_file_close] = ACTIONS(1815),
    [sym_file_read] = ACTIONS(1815),
    [sym_file_read_line] = ACTIONS(1815),
    [sym_file_write] = ACTIONS(1815),
    [sym_file_write_line] = ACTIONS(1815),
    [sym_file_copy] = ACTIONS(1815),
    [sym_file_delete] = ACTIONS(1815),
    [sym_file_rename] = ACTIONS(1815),
    [sym_file_dir] = ACTIONS(1815),
    [sym_file_getdir] = ACTIONS(1815),
    [sym_file_setdir] = ACTIONS(1815),
    [sym_file_create_dir] = ACTIONS(1815),
    [sym_file_remove_dir] = ACTIONS(1815),
    [sym_file_seek] = ACTIONS(1815),
    [sym_length_array] = ACTIONS(1815),
    [sym_max_length_array] = ACTIONS(1815),
    [sym_set_length_array] = ACTIONS(1815),
    [sym_ip_client_open] = ACTIONS(1815),
    [sym_ip_client_close] = ACTIONS(1815),
    [sym_ip_server_open] = ACTIONS(1815),
    [sym_ip_server_close] = ACTIONS(1815),
    [sym_ip_bound_client_open] = ACTIONS(1815),
    [sym_ip_mc_server_open] = ACTIONS(1815),
    [sym_ip_set_option] = ACTIONS(1815),
    [sym_ssh_client_open] = ACTIONS(1815),
    [sym_ssh_client_close] = ACTIONS(1815),
    [sym_tls_client_open] = ACTIONS(1815),
    [sym_tls_client_close] = ACTIONS(1815),
    [sym_do_push] = ACTIONS(1815),
    [sym_do_push_timed] = ACTIONS(1815),
    [sym_do_release] = ACTIONS(1815),
    [sym_device_id] = ACTIONS(1815),
    [sym_device_info] = ACTIONS(1815),
    [sym_device_id_string] = ACTIONS(1815),
    [sym_device_standby] = ACTIONS(1815),
    [sym_device_wake] = ACTIONS(1815),
    [sym_timeline_create] = ACTIONS(1815),
    [sym_timeline_kill] = ACTIONS(1815),
    [sym_timeline_reload] = ACTIONS(1815),
    [sym_timeline_pause] = ACTIONS(1815),
    [sym_timeline_restart] = ACTIONS(1815),
    [sym_timeline_set] = ACTIONS(1815),
    [sym_timeline_get] = ACTIONS(1815),
    [sym_timeline_active] = ACTIONS(1815),
    [sym_timeline_dynamic_id] = ACTIONS(1815),
    [sym_variable_to_string] = ACTIONS(1815),
    [sym_string_to_variable] = ACTIONS(1815),
    [sym_length_variable_to_string] = ACTIONS(1815),
    [sym_type_cast] = ACTIONS(1815),
    [sym_variable_to_xml] = ACTIONS(1815),
    [sym_xml_to_variable] = ACTIONS(1815),
    [sym_length_variable_to_xml] = ACTIONS(1815),
    [sym_redirect_string] = ACTIONS(1815),
    [sym_random_number] = ACTIONS(1815),
    [sym_get_unique_id] = ACTIONS(1815),
    [sym_get_system_number] = ACTIONS(1815),
    [sym_set_system_number] = ACTIONS(1815),
    [sym_get_serial_number] = ACTIONS(1815),
    [sym_get_master_build] = ACTIONS(1815),
    [sym_set_validation_code] = ACTIONS(1815),
    [sym_reboot] = ACTIONS(1815),
    [sym_load_duet_module] = ACTIONS(1815),
    [sym_unload_duet_module] = ACTIONS(1815),
    [sym_duet_mem_size_set] = ACTIONS(1815),
    [sym_duet_mem_size_get] = ACTIONS(1815),
    [sym_dynamic_application_device] = ACTIONS(1815),
    [sym_dynamic_polled_port] = ACTIONS(1815),
    [sym_static_port_binding] = ACTIONS(1815),
    [sym_static_ip_binding] = ACTIONS(1815),
    [sym_get_dns_list] = ACTIONS(1815),
    [sym_set_dns_list] = ACTIONS(1815),
    [sym_get_ip_address] = ACTIONS(1815),
    [sym_set_ip_address] = ACTIONS(1815),
    [sym_get_url_list] = ACTIONS(1815),
    [sym_add_url_entry] = ACTIONS(1815),
    [sym_delete_url_entry] = ACTIONS(1815),
    [sym_abs_value] = ACTIONS(1815),
    [sym_max_value] = ACTIONS(1815),
    [sym_min_value] = ACTIONS(1815),
    [sym_exp_value] = ACTIONS(1815),
    [sym_log_value] = ACTIONS(1815),
    [sym_log10_value] = ACTIONS(1815),
    [sym_power_value] = ACTIONS(1815),
    [sym_sqrt_value] = ACTIONS(1815),
    [sym_raw_be] = ACTIONS(1815),
    [sym_raw_le] = ACTIONS(1815),
    [sym_internal_queue_size_set] = ACTIONS(1815),
    [sym_internal_queue_size_get] = ACTIONS(1815),
    [sym_internal_threshold_set] = ACTIONS(1815),
    [sym_internal_threshold_get] = ACTIONS(1815),
    [sym_rebuilt_event] = ACTIONS(1815),
    [sym_do_custom_event] = ACTIONS(1815),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1815),
    [sym_clkmgr_set_clk_source] = ACTIONS(1815),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1815),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1815),
    [sym_clkmgr_get_timezone] = ACTIONS(1815),
    [sym_clkmgr_set_timezone] = ACTIONS(1815),
    [sym_clkmgr_get_resync_period] = ACTIONS(1815),
    [sym_clkmgr_set_resync_period] = ACTIONS(1815),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1815),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1815),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1815),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1815),
    [sym_clkmgr_get_timeservers] = ACTIONS(1815),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1815),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1815),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1815),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1815),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1815),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1815),
    [sym_set_log_level] = ACTIONS(1815),
    [sym_get_log_level] = ACTIONS(1815),
    [sym_amx_log] = ACTIONS(1815),
    [sym_smtp_server_config_set] = ACTIONS(1815),
    [sym_smtp_server_config_get] = ACTIONS(1815),
    [sym_smtp_send] = ACTIONS(1815),
    [sym_validate_netlinx_account] = ACTIONS(1815),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1815),
    [sym_authenticate_certificate] = ACTIONS(1815),
    [sym_audit_netlinx_session_event] = ACTIONS(1815),
    [sym_audit_netlinx_generic_event] = ACTIONS(1815),
    [sym_get_available_flash_disk_space] = ACTIONS(1815),
    [sym_get_max_flash_disk_space] = ACTIONS(1815),
    [sym_data_create_feed] = ACTIONS(1815),
    [sym_data_delete_feed] = ACTIONS(1815),
    [sym_data_publish_feed] = ACTIONS(1815),
    [sym_data_get_published_feed] = ACTIONS(1815),
    [sym_data_add_record] = ACTIONS(1815),
    [sym_data_get_event_record] = ACTIONS(1815),
    [sym__wc_data_create_feed] = ACTIONS(1815),
    [sym__wc_data_add_record] = ACTIONS(1815),
    [sym__wc_data_get_event_record] = ACTIONS(1815),
    [sym_format] = ACTIONS(1815),
    [sym_astro_clock] = ACTIONS(1815),
    [sym_set_outdoor_temperature] = ACTIONS(1815),
    [sym_set_virtual_level_count] = ACTIONS(1815),
    [sym_set_virtual_channel_count] = ACTIONS(1815),
    [sym_set_virtual_port_count] = ACTIONS(1815),
    [sym__wc_encode_int] = ACTIONS(1815),
    [sym__wc_decode_int] = ACTIONS(1815),
    [sym__wc_int] = ACTIONS(1815),
    [sym__wc_to_ch_int] = ACTIONS(1815),
    [sym__ch_to_wc_int] = ACTIONS(1815),
    [sym__wc_find_string_int] = ACTIONS(1815),
    [sym__wc_left_string_int] = ACTIONS(1815),
    [sym__wc_lower_string_int] = ACTIONS(1815),
    [sym__wc_mid_string_int] = ACTIONS(1815),
    [sym__wc_remove_string_int] = ACTIONS(1815),
    [sym__wc_right_string_int] = ACTIONS(1815),
    [sym__wc_upper_string_int] = ACTIONS(1815),
    [sym__wc_compare_string_int] = ACTIONS(1815),
    [sym__wc_get_buffer_char_int] = ACTIONS(1815),
    [sym__wc_get_buffer_string_int] = ACTIONS(1815),
    [sym__wc_concat_string_int] = ACTIONS(1815),
    [sym__wc_file_open_int] = ACTIONS(1815),
    [sym__wc_file_close_int] = ACTIONS(1815),
    [sym__wc_file_read_int] = ACTIONS(1815),
    [sym__wc_file_read_line_int] = ACTIONS(1815),
    [sym__wc_file_write_int] = ACTIONS(1815),
    [sym__wc_file_write_line_int] = ACTIONS(1815),
    [aux_sym_preproc_include_token1] = ACTIONS(1815),
    [aux_sym_preproc_define_token1] = ACTIONS(1817),
    [aux_sym_preproc_warn_token1] = ACTIONS(1817),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1817),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1817),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1817),
    [anon_sym_LBRACK] = ACTIONS(1817),
    [anon_sym_LBRACE] = ACTIONS(1817),
    [anon_sym_RBRACE] = ACTIONS(1817),
    [anon_sym_LPAREN] = ACTIONS(1815),
    [anon_sym_BANG] = ACTIONS(1817),
    [anon_sym_TILDE] = ACTIONS(1817),
    [anon_sym_DASH] = ACTIONS(1815),
    [anon_sym_PLUS] = ACTIONS(1815),
    [anon_sym_DASH_DASH] = ACTIONS(1817),
    [anon_sym_PLUS_PLUS] = ACTIONS(1817),
    [anon_sym_DQUOTE] = ACTIONS(1817),
    [anon_sym_SQUOTE] = ACTIONS(1817),
    [sym_number_literal] = ACTIONS(1817),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(517)] = {
    [sym_identifier] = ACTIONS(1823),
    [sym_define_call_keyword] = ACTIONS(1823),
    [sym_define_function_keyword] = ACTIONS(1823),
    [sym_define_library_function_keyword] = ACTIONS(1823),
    [sym_define_module_keyword] = ACTIONS(1823),
    [sym_char_keyword] = ACTIONS(1823),
    [sym_widechar_keyword] = ACTIONS(1823),
    [sym_integer_keyword] = ACTIONS(1823),
    [sym_sinteger_keyword] = ACTIONS(1823),
    [sym_long_keyword] = ACTIONS(1823),
    [sym_slong_keyword] = ACTIONS(1823),
    [sym_float_keyword] = ACTIONS(1823),
    [sym_double_keyword] = ACTIONS(1823),
    [sym_variant_keyword] = ACTIONS(1823),
    [sym_variantarray_keyword] = ACTIONS(1823),
    [sym_local_var_keyword] = ACTIONS(1823),
    [sym_stack_var_keyword] = ACTIONS(1823),
    [sym_constant_keyword] = ACTIONS(1823),
    [sym_volatile_keyword] = ACTIONS(1823),
    [sym_non_volatile_keyword] = ACTIONS(1823),
    [sym_persistent_keyword] = ACTIONS(1823),
    [sym_dev_keyword] = ACTIONS(1823),
    [sym_devlev_keyword] = ACTIONS(1823),
    [sym_devchan_keyword] = ACTIONS(1823),
    [sym_if_keyword] = ACTIONS(1823),
    [sym_else_keyword] = ACTIONS(1823),
    [sym_switch_keyword] = ACTIONS(1823),
    [sym_case_keyword] = ACTIONS(1823),
    [sym_default_keyword] = ACTIONS(1823),
    [sym_while_keyword] = ACTIONS(1823),
    [sym_for_keyword] = ACTIONS(1823),
    [sym_break_keyword] = ACTIONS(1823),
    [sym_continue_keyword] = ACTIONS(1823),
    [sym_return_keyword] = ACTIONS(1823),
    [sym_select_keyword] = ACTIONS(1823),
    [sym_active_keyword] = ACTIONS(1823),
    [sym_struct_keyword] = ACTIONS(1823),
    [sym_structure_keyword] = ACTIONS(1823),
    [aux_sym_on_keyword_token1] = ACTIONS(1823),
    [aux_sym_off_keyword_token1] = ACTIONS(1823),
    [sym_send_level_keyword] = ACTIONS(1823),
    [sym_send_string_keyword] = ACTIONS(1823),
    [sym_send_command_keyword] = ACTIONS(1823),
    [sym_clear_buffer_keyword] = ACTIONS(1823),
    [sym_create_buffer_keyword] = ACTIONS(1823),
    [sym_create_multi_buffer_keyword] = ACTIONS(1823),
    [sym_call_keyword] = ACTIONS(1823),
    [sym_system_call_keyword] = ACTIONS(1823),
    [sym_devchan_to_keyword] = ACTIONS(1823),
    [sym_devchan_min_to_keyword] = ACTIONS(1823),
    [sym_devchan_total_off_keyword] = ACTIONS(1823),
    [sym_devchan_pulse_keyword] = ACTIONS(1823),
    [sym_wait_keyword] = ACTIONS(1823),
    [sym_cancel_wait_keyword] = ACTIONS(1823),
    [sym_wait_until_keyword] = ACTIONS(1823),
    [sym_cancel_wait_until_keyword] = ACTIONS(1823),
    [sym_cancel_all_wait_keyword] = ACTIONS(1823),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1823),
    [sym_not] = ACTIONS(1823),
    [sym_bnot] = ACTIONS(1823),
    [sym___file__] = ACTIONS(1823),
    [sym___date__] = ACTIONS(1823),
    [sym___ldate__] = ACTIONS(1823),
    [sym___time__] = ACTIONS(1823),
    [sym___line__] = ACTIONS(1823),
    [sym___name__] = ACTIONS(1823),
    [sym___netlinx__] = ACTIONS(1823),
    [sym_netlinx_axi_version] = ACTIONS(1823),
    [sym_url_flg_tcp] = ACTIONS(1823),
    [sym_url_flg_accinfopresent] = ACTIONS(1823),
    [sym_url_flg_temp] = ACTIONS(1823),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1823),
    [sym_url_flg_stat_mask] = ACTIONS(1823),
    [sym_url_flg_stat_lookup] = ACTIONS(1823),
    [sym_url_flg_stat_connecting] = ACTIONS(1823),
    [sym_url_flg_stat_waiting] = ACTIONS(1823),
    [sym_url_flg_stat_connected] = ACTIONS(1823),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1823),
    [sym_first_virtual_device] = ACTIONS(1823),
    [sym_first_local_port] = ACTIONS(1823),
    [sym_dynamic_virtual_device] = ACTIONS(1823),
    [sym_timeline_once] = ACTIONS(1823),
    [sym_timeline_repeat] = ACTIONS(1823),
    [sym_timeline_absolute] = ACTIONS(1823),
    [sym_timeline_relative] = ACTIONS(1823),
    [sym_do_push_timed_infinite] = ACTIONS(1823),
    [sym_source_type_no_address] = ACTIONS(1823),
    [sym_source_type_neuron_id] = ACTIONS(1823),
    [sym_source_type_ip_address] = ACTIONS(1823),
    [sym_source_type_axlink] = ACTIONS(1823),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1823),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1823),
    [sym_source_type_ip_socket_address] = ACTIONS(1823),
    [sym_source_type_rs232] = ACTIONS(1823),
    [sym_source_type_internal] = ACTIONS(1823),
    [sym_source_type_mac_address] = ACTIONS(1823),
    [sym_source_type_ipv4_port] = ACTIONS(1823),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1823),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1823),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1823),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1823),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1823),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1823),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1823),
    [sym_true] = ACTIONS(1823),
    [sym_false] = ACTIONS(1823),
    [sym_file_read_only] = ACTIONS(1823),
    [sym_file_rw_new] = ACTIONS(1823),
    [sym_file_rw_append] = ACTIONS(1823),
    [sym_ip_tcp] = ACTIONS(1823),
    [sym_ip_udp] = ACTIONS(1823),
    [sym_ip_udp_2way] = ACTIONS(1823),
    [sym_xml_encode_types] = ACTIONS(1823),
    [sym_xml_encode_char_as_list] = ACTIONS(1823),
    [sym_xml_encode_le] = ACTIONS(1823),
    [sym_xml_decode_types] = ACTIONS(1823),
    [sym_xml_decode_no_preserve] = ACTIONS(1823),
    [sym_duet_dev_type_display_device] = ACTIONS(1823),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1823),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1823),
    [sym_duet_dev_type_io_device] = ACTIONS(1823),
    [sym_duet_dev_type_relay_device] = ACTIONS(1823),
    [sym_duet_dev_type_ups] = ACTIONS(1823),
    [sym_duet_dev_type_amplifier] = ACTIONS(1823),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1823),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1823),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1823),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1823),
    [sym_duet_dev_type_camera] = ACTIONS(1823),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1823),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1823),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1823),
    [sym_duet_dev_type_dss] = ACTIONS(1823),
    [sym_duet_dev_type_dvr] = ACTIONS(1823),
    [sym_duet_dev_type_disc_device] = ACTIONS(1823),
    [sym_duet_dev_type_document_camera] = ACTIONS(1823),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1823),
    [sym_duet_dev_type_hvac] = ACTIONS(1823),
    [sym_duet_dev_type_keypad] = ACTIONS(1823),
    [sym_duet_dev_type_light] = ACTIONS(1823),
    [sym_duet_dev_type_monitor] = ACTIONS(1823),
    [sym_duet_dev_type_motor] = ACTIONS(1823),
    [sym_duet_dev_type_multi_window] = ACTIONS(1823),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1823),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1823),
    [sym_duet_dev_type_receiver] = ACTIONS(1823),
    [sym_duet_dev_type_security_system] = ACTIONS(1823),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1823),
    [sym_duet_dev_type_settop_box] = ACTIONS(1823),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1823),
    [sym_duet_dev_type_switcher] = ACTIONS(1823),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1823),
    [sym_duet_dev_type_tv] = ACTIONS(1823),
    [sym_duet_dev_type_utility] = ACTIONS(1823),
    [sym_duet_dev_type_vcr] = ACTIONS(1823),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1823),
    [sym_duet_dev_type_video_processor] = ACTIONS(1823),
    [sym_duet_dev_type_video_projector] = ACTIONS(1823),
    [sym_duet_dev_type_video_wall] = ACTIONS(1823),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1823),
    [sym_duet_dev_type_weather] = ACTIONS(1823),
    [sym_duet_dev_not_polled] = ACTIONS(1823),
    [sym_duet_dev_polled] = ACTIONS(1823),
    [sym_clkmgr_mode_network] = ACTIONS(1823),
    [sym_clkmgr_mode_standalone] = ACTIONS(1823),
    [sym_normal_standby] = ACTIONS(1823),
    [sym_normal_wake] = ACTIONS(1823),
    [sym_ip_multicast_ttl_option] = ACTIONS(1823),
    [sym_ip_ttl_subnet] = ACTIONS(1823),
    [sym_ip_ttl_site] = ACTIONS(1823),
    [sym_ip_ttl_region] = ACTIONS(1823),
    [sym_ip_ttl_continent] = ACTIONS(1823),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1823),
    [sym_ip_nodelay_on] = ACTIONS(1823),
    [sym_ip_nodelay_off] = ACTIONS(1823),
    [sym_amx_error] = ACTIONS(1823),
    [sym_amx_warning] = ACTIONS(1823),
    [sym_amx_info] = ACTIONS(1823),
    [sym_amx_debug] = ACTIONS(1823),
    [sym_netlinx_login_success] = ACTIONS(1823),
    [sym_netlinx_login_fail] = ACTIONS(1823),
    [sym_netlinx_logout] = ACTIONS(1823),
    [sym_null_str] = ACTIONS(1823),
    [sym_smtp_address] = ACTIONS(1823),
    [sym_smtp_port_number] = ACTIONS(1823),
    [sym_smtp_username] = ACTIONS(1823),
    [sym_smtp_password] = ACTIONS(1823),
    [sym_smtp_require_tls] = ACTIONS(1823),
    [sym_smtp_from] = ACTIONS(1823),
    [sym_smtp_tls_true] = ACTIONS(1823),
    [sym_smtp_tls_false] = ACTIONS(1823),
    [sym_listview_on_row_select_event] = ACTIONS(1823),
    [sym_data_structure_unknown] = ACTIONS(1823),
    [sym_data_structure_datafeed] = ACTIONS(1823),
    [sym_data_structure_datarecordset] = ACTIONS(1823),
    [sym_data_structure_datarecord] = ACTIONS(1823),
    [sym_data_structure_datafield] = ACTIONS(1823),
    [sym_data_max_content_field_count] = ACTIONS(1823),
    [sym_data_max_metadata_field_count] = ACTIONS(1823),
    [sym_data_max_id_length] = ACTIONS(1823),
    [sym_data_max_name_length] = ACTIONS(1823),
    [sym_data_max_description_length] = ACTIONS(1823),
    [sym_data_max_type_length] = ACTIONS(1823),
    [sym_data_max_format_length] = ACTIONS(1823),
    [sym_data_max_source_length] = ACTIONS(1823),
    [sym_data_max_value_length] = ACTIONS(1823),
    [sym_data_max_label_length] = ACTIONS(1823),
    [sym_data_max_url_length] = ACTIONS(1823),
    [sym_max_last_login_info_length] = ACTIONS(1823),
    [sym_data_type_unknown] = ACTIONS(1823),
    [sym_data_type_string] = ACTIONS(1823),
    [sym_data_type_datetime] = ACTIONS(1823),
    [sym_data_type_date] = ACTIONS(1823),
    [sym_data_type_time] = ACTIONS(1823),
    [sym_data_type_image] = ACTIONS(1823),
    [sym_data_format_url] = ACTIONS(1823),
    [sym_data_format_phone] = ACTIONS(1823),
    [sym_data_format_email] = ACTIONS(1823),
    [sym_data_format_iso8601] = ACTIONS(1823),
    [sym_auditlog_priv] = ACTIONS(1823),
    [sym_configuration_priv] = ACTIONS(1823),
    [sym_deviceconfig_priv] = ACTIONS(1823),
    [sym_ftp_priv] = ACTIONS(1823),
    [sym_http_priv] = ACTIONS(1823),
    [sym_networkconfig_priv] = ACTIONS(1823),
    [sym_programport_priv] = ACTIONS(1823),
    [sym_remoteui_priv] = ACTIONS(1823),
    [sym_securitycontrol_priv] = ACTIONS(1823),
    [sym_softwaremanagement_priv] = ACTIONS(1823),
    [sym_terminal_priv] = ACTIONS(1823),
    [sym_tpadmin_priv] = ACTIONS(1823),
    [sym_user1_priv] = ACTIONS(1823),
    [sym_user2_priv] = ACTIONS(1823),
    [sym_user3_priv] = ACTIONS(1823),
    [sym_user4_priv] = ACTIONS(1823),
    [sym_usermanagement_priv] = ACTIONS(1823),
    [sym_valid_account] = ACTIONS(1823),
    [sym_err_name_invalid] = ACTIONS(1823),
    [sym_err_password_invalid] = ACTIONS(1823),
    [sym_err_invalid_account] = ACTIONS(1823),
    [sym_err_no_authorization] = ACTIONS(1823),
    [sym_err_invalid_parameter] = ACTIONS(1823),
    [sym_err_user_locked_out] = ACTIONS(1823),
    [sym_err_user_account_expired] = ACTIONS(1823),
    [sym_err_general_error] = ACTIONS(1823),
    [sym_err_authentication_error] = ACTIONS(1823),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1823),
    [sym_tls_validate_certificate] = ACTIONS(1823),
    [sym_tdata] = ACTIONS(1823),
    [sym_tchannel] = ACTIONS(1823),
    [sym_tlevel] = ACTIONS(1823),
    [sym_tbutton] = ACTIONS(1823),
    [sym_ttimeline] = ACTIONS(1823),
    [sym_tcustom] = ACTIONS(1823),
    [sym_url_struct] = ACTIONS(1823),
    [sym_dns_struct] = ACTIONS(1823),
    [sym_ip_address_struct] = ACTIONS(1823),
    [sym_dev_info_struct] = ACTIONS(1823),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1823),
    [sym_data_feed] = ACTIONS(1823),
    [sym_data_field] = ACTIONS(1823),
    [sym_data_record] = ACTIONS(1823),
    [sym_wc_data_feed] = ACTIONS(1823),
    [sym_wc_data_field] = ACTIONS(1823),
    [sym_wc_data_record] = ACTIONS(1823),
    [sym_last_login_info] = ACTIONS(1823),
    [sym____reserved___] = ACTIONS(1823),
    [sym_date] = ACTIONS(1823),
    [sym_day] = ACTIONS(1823),
    [sym_time] = ACTIONS(1823),
    [sym_ldate] = ACTIONS(1823),
    [sym_push_device] = ACTIONS(1823),
    [sym_push_channel] = ACTIONS(1823),
    [sym_push_devchan] = ACTIONS(1823),
    [sym_release_device] = ACTIONS(1823),
    [sym_release_channel] = ACTIONS(1823),
    [sym_release_devchan] = ACTIONS(1823),
    [sym_master_slot] = ACTIONS(1823),
    [sym_get_pulse_time] = ACTIONS(1823),
    [sym_get_timer] = ACTIONS(1823),
    [sym_master_sn] = ACTIONS(1823),
    [sym_system_number] = ACTIONS(1823),
    [sym_button] = ACTIONS(1823),
    [sym_dv_channel] = ACTIONS(1823),
    [sym_data] = ACTIONS(1823),
    [sym_level] = ACTIONS(1823),
    [sym_timeline] = ACTIONS(1823),
    [sym_channel] = ACTIONS(1823),
    [sym_custom] = ACTIONS(1823),
    [sym_atoi] = ACTIONS(1823),
    [sym_atol] = ACTIONS(1823),
    [sym_atof] = ACTIONS(1823),
    [sym_itoa] = ACTIONS(1823),
    [sym_itohex] = ACTIONS(1823),
    [sym_hextoi] = ACTIONS(1823),
    [sym_ftoa] = ACTIONS(1823),
    [sym_find_string] = ACTIONS(1823),
    [sym_left_string] = ACTIONS(1823),
    [sym_right_string] = ACTIONS(1823),
    [sym_mid_string] = ACTIONS(1823),
    [sym_remove_string] = ACTIONS(1823),
    [sym_get_buffer_string] = ACTIONS(1823),
    [sym_get_buffer_char] = ACTIONS(1823),
    [sym_get_multi_buffer_string] = ACTIONS(1823),
    [sym_length_string] = ACTIONS(1823),
    [sym_max_length_string] = ACTIONS(1823),
    [sym_set_length_string] = ACTIONS(1823),
    [sym_compare_string] = ACTIONS(1823),
    [sym_upper_string] = ACTIONS(1823),
    [sym_lower_string] = ACTIONS(1823),
    [sym_get_last] = ACTIONS(1823),
    [sym_date_to_day] = ACTIONS(1823),
    [sym_date_to_month] = ACTIONS(1823),
    [sym_date_to_year] = ACTIONS(1823),
    [sym_time_to_hour] = ACTIONS(1823),
    [sym_time_to_minute] = ACTIONS(1823),
    [sym_time_to_second] = ACTIONS(1823),
    [sym_day_of_week] = ACTIONS(1823),
    [sym_set_pulse_time] = ACTIONS(1823),
    [sym_set_timer] = ACTIONS(1823),
    [sym_file_open] = ACTIONS(1823),
    [sym_file_close] = ACTIONS(1823),
    [sym_file_read] = ACTIONS(1823),
    [sym_file_read_line] = ACTIONS(1823),
    [sym_file_write] = ACTIONS(1823),
    [sym_file_write_line] = ACTIONS(1823),
    [sym_file_copy] = ACTIONS(1823),
    [sym_file_delete] = ACTIONS(1823),
    [sym_file_rename] = ACTIONS(1823),
    [sym_file_dir] = ACTIONS(1823),
    [sym_file_getdir] = ACTIONS(1823),
    [sym_file_setdir] = ACTIONS(1823),
    [sym_file_create_dir] = ACTIONS(1823),
    [sym_file_remove_dir] = ACTIONS(1823),
    [sym_file_seek] = ACTIONS(1823),
    [sym_length_array] = ACTIONS(1823),
    [sym_max_length_array] = ACTIONS(1823),
    [sym_set_length_array] = ACTIONS(1823),
    [sym_ip_client_open] = ACTIONS(1823),
    [sym_ip_client_close] = ACTIONS(1823),
    [sym_ip_server_open] = ACTIONS(1823),
    [sym_ip_server_close] = ACTIONS(1823),
    [sym_ip_bound_client_open] = ACTIONS(1823),
    [sym_ip_mc_server_open] = ACTIONS(1823),
    [sym_ip_set_option] = ACTIONS(1823),
    [sym_ssh_client_open] = ACTIONS(1823),
    [sym_ssh_client_close] = ACTIONS(1823),
    [sym_tls_client_open] = ACTIONS(1823),
    [sym_tls_client_close] = ACTIONS(1823),
    [sym_do_push] = ACTIONS(1823),
    [sym_do_push_timed] = ACTIONS(1823),
    [sym_do_release] = ACTIONS(1823),
    [sym_device_id] = ACTIONS(1823),
    [sym_device_info] = ACTIONS(1823),
    [sym_device_id_string] = ACTIONS(1823),
    [sym_device_standby] = ACTIONS(1823),
    [sym_device_wake] = ACTIONS(1823),
    [sym_timeline_create] = ACTIONS(1823),
    [sym_timeline_kill] = ACTIONS(1823),
    [sym_timeline_reload] = ACTIONS(1823),
    [sym_timeline_pause] = ACTIONS(1823),
    [sym_timeline_restart] = ACTIONS(1823),
    [sym_timeline_set] = ACTIONS(1823),
    [sym_timeline_get] = ACTIONS(1823),
    [sym_timeline_active] = ACTIONS(1823),
    [sym_timeline_dynamic_id] = ACTIONS(1823),
    [sym_variable_to_string] = ACTIONS(1823),
    [sym_string_to_variable] = ACTIONS(1823),
    [sym_length_variable_to_string] = ACTIONS(1823),
    [sym_type_cast] = ACTIONS(1823),
    [sym_variable_to_xml] = ACTIONS(1823),
    [sym_xml_to_variable] = ACTIONS(1823),
    [sym_length_variable_to_xml] = ACTIONS(1823),
    [sym_redirect_string] = ACTIONS(1823),
    [sym_random_number] = ACTIONS(1823),
    [sym_get_unique_id] = ACTIONS(1823),
    [sym_get_system_number] = ACTIONS(1823),
    [sym_set_system_number] = ACTIONS(1823),
    [sym_get_serial_number] = ACTIONS(1823),
    [sym_get_master_build] = ACTIONS(1823),
    [sym_set_validation_code] = ACTIONS(1823),
    [sym_reboot] = ACTIONS(1823),
    [sym_load_duet_module] = ACTIONS(1823),
    [sym_unload_duet_module] = ACTIONS(1823),
    [sym_duet_mem_size_set] = ACTIONS(1823),
    [sym_duet_mem_size_get] = ACTIONS(1823),
    [sym_dynamic_application_device] = ACTIONS(1823),
    [sym_dynamic_polled_port] = ACTIONS(1823),
    [sym_static_port_binding] = ACTIONS(1823),
    [sym_static_ip_binding] = ACTIONS(1823),
    [sym_get_dns_list] = ACTIONS(1823),
    [sym_set_dns_list] = ACTIONS(1823),
    [sym_get_ip_address] = ACTIONS(1823),
    [sym_set_ip_address] = ACTIONS(1823),
    [sym_get_url_list] = ACTIONS(1823),
    [sym_add_url_entry] = ACTIONS(1823),
    [sym_delete_url_entry] = ACTIONS(1823),
    [sym_abs_value] = ACTIONS(1823),
    [sym_max_value] = ACTIONS(1823),
    [sym_min_value] = ACTIONS(1823),
    [sym_exp_value] = ACTIONS(1823),
    [sym_log_value] = ACTIONS(1823),
    [sym_log10_value] = ACTIONS(1823),
    [sym_power_value] = ACTIONS(1823),
    [sym_sqrt_value] = ACTIONS(1823),
    [sym_raw_be] = ACTIONS(1823),
    [sym_raw_le] = ACTIONS(1823),
    [sym_internal_queue_size_set] = ACTIONS(1823),
    [sym_internal_queue_size_get] = ACTIONS(1823),
    [sym_internal_threshold_set] = ACTIONS(1823),
    [sym_internal_threshold_get] = ACTIONS(1823),
    [sym_rebuilt_event] = ACTIONS(1823),
    [sym_do_custom_event] = ACTIONS(1823),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1823),
    [sym_clkmgr_set_clk_source] = ACTIONS(1823),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1823),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1823),
    [sym_clkmgr_get_timezone] = ACTIONS(1823),
    [sym_clkmgr_set_timezone] = ACTIONS(1823),
    [sym_clkmgr_get_resync_period] = ACTIONS(1823),
    [sym_clkmgr_set_resync_period] = ACTIONS(1823),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1823),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1823),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1823),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1823),
    [sym_clkmgr_get_timeservers] = ACTIONS(1823),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1823),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1823),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1823),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1823),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1823),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1823),
    [sym_set_log_level] = ACTIONS(1823),
    [sym_get_log_level] = ACTIONS(1823),
    [sym_amx_log] = ACTIONS(1823),
    [sym_smtp_server_config_set] = ACTIONS(1823),
    [sym_smtp_server_config_get] = ACTIONS(1823),
    [sym_smtp_send] = ACTIONS(1823),
    [sym_validate_netlinx_account] = ACTIONS(1823),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1823),
    [sym_authenticate_certificate] = ACTIONS(1823),
    [sym_audit_netlinx_session_event] = ACTIONS(1823),
    [sym_audit_netlinx_generic_event] = ACTIONS(1823),
    [sym_get_available_flash_disk_space] = ACTIONS(1823),
    [sym_get_max_flash_disk_space] = ACTIONS(1823),
    [sym_data_create_feed] = ACTIONS(1823),
    [sym_data_delete_feed] = ACTIONS(1823),
    [sym_data_publish_feed] = ACTIONS(1823),
    [sym_data_get_published_feed] = ACTIONS(1823),
    [sym_data_add_record] = ACTIONS(1823),
    [sym_data_get_event_record] = ACTIONS(1823),
    [sym__wc_data_create_feed] = ACTIONS(1823),
    [sym__wc_data_add_record] = ACTIONS(1823),
    [sym__wc_data_get_event_record] = ACTIONS(1823),
    [sym_format] = ACTIONS(1823),
    [sym_astro_clock] = ACTIONS(1823),
    [sym_set_outdoor_temperature] = ACTIONS(1823),
    [sym_set_virtual_level_count] = ACTIONS(1823),
    [sym_set_virtual_channel_count] = ACTIONS(1823),
    [sym_set_virtual_port_count] = ACTIONS(1823),
    [sym__wc_encode_int] = ACTIONS(1823),
    [sym__wc_decode_int] = ACTIONS(1823),
    [sym__wc_int] = ACTIONS(1823),
    [sym__wc_to_ch_int] = ACTIONS(1823),
    [sym__ch_to_wc_int] = ACTIONS(1823),
    [sym__wc_find_string_int] = ACTIONS(1823),
    [sym__wc_left_string_int] = ACTIONS(1823),
    [sym__wc_lower_string_int] = ACTIONS(1823),
    [sym__wc_mid_string_int] = ACTIONS(1823),
    [sym__wc_remove_string_int] = ACTIONS(1823),
    [sym__wc_right_string_int] = ACTIONS(1823),
    [sym__wc_upper_string_int] = ACTIONS(1823),
    [sym__wc_compare_string_int] = ACTIONS(1823),
    [sym__wc_get_buffer_char_int] = ACTIONS(1823),
    [sym__wc_get_buffer_string_int] = ACTIONS(1823),
    [sym__wc_concat_string_int] = ACTIONS(1823),
    [sym__wc_file_open_int] = ACTIONS(1823),
    [sym__wc_file_close_int] = ACTIONS(1823),
    [sym__wc_file_read_int] = ACTIONS(1823),
    [sym__wc_file_read_line_int] = ACTIONS(1823),
    [sym__wc_file_write_int] = ACTIONS(1823),
    [sym__wc_file_write_line_int] = ACTIONS(1823),
    [aux_sym_preproc_include_token1] = ACTIONS(1823),
    [aux_sym_preproc_define_token1] = ACTIONS(1825),
    [aux_sym_preproc_warn_token1] = ACTIONS(1825),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1825),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1825),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1825),
    [anon_sym_LBRACK] = ACTIONS(1825),
    [anon_sym_LBRACE] = ACTIONS(1825),
    [anon_sym_RBRACE] = ACTIONS(1825),
    [anon_sym_LPAREN] = ACTIONS(1823),
    [anon_sym_BANG] = ACTIONS(1825),
    [anon_sym_TILDE] = ACTIONS(1825),
    [anon_sym_DASH] = ACTIONS(1823),
    [anon_sym_PLUS] = ACTIONS(1823),
    [anon_sym_DASH_DASH] = ACTIONS(1825),
    [anon_sym_PLUS_PLUS] = ACTIONS(1825),
    [anon_sym_DQUOTE] = ACTIONS(1825),
    [anon_sym_SQUOTE] = ACTIONS(1825),
    [sym_number_literal] = ACTIONS(1825),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(518)] = {
    [sym_identifier] = ACTIONS(1671),
    [sym_define_call_keyword] = ACTIONS(1671),
    [sym_define_function_keyword] = ACTIONS(1671),
    [sym_define_library_function_keyword] = ACTIONS(1671),
    [sym_define_module_keyword] = ACTIONS(1671),
    [sym_char_keyword] = ACTIONS(1671),
    [sym_widechar_keyword] = ACTIONS(1671),
    [sym_integer_keyword] = ACTIONS(1671),
    [sym_sinteger_keyword] = ACTIONS(1671),
    [sym_long_keyword] = ACTIONS(1671),
    [sym_slong_keyword] = ACTIONS(1671),
    [sym_float_keyword] = ACTIONS(1671),
    [sym_double_keyword] = ACTIONS(1671),
    [sym_variant_keyword] = ACTIONS(1671),
    [sym_variantarray_keyword] = ACTIONS(1671),
    [sym_local_var_keyword] = ACTIONS(1671),
    [sym_stack_var_keyword] = ACTIONS(1671),
    [sym_constant_keyword] = ACTIONS(1671),
    [sym_volatile_keyword] = ACTIONS(1671),
    [sym_non_volatile_keyword] = ACTIONS(1671),
    [sym_persistent_keyword] = ACTIONS(1671),
    [sym_dev_keyword] = ACTIONS(1671),
    [sym_devlev_keyword] = ACTIONS(1671),
    [sym_devchan_keyword] = ACTIONS(1671),
    [sym_if_keyword] = ACTIONS(1671),
    [sym_else_keyword] = ACTIONS(1671),
    [sym_switch_keyword] = ACTIONS(1671),
    [sym_case_keyword] = ACTIONS(1671),
    [sym_default_keyword] = ACTIONS(1671),
    [sym_while_keyword] = ACTIONS(1671),
    [sym_for_keyword] = ACTIONS(1671),
    [sym_break_keyword] = ACTIONS(1671),
    [sym_continue_keyword] = ACTIONS(1671),
    [sym_return_keyword] = ACTIONS(1671),
    [sym_select_keyword] = ACTIONS(1671),
    [sym_active_keyword] = ACTIONS(1671),
    [sym_struct_keyword] = ACTIONS(1671),
    [sym_structure_keyword] = ACTIONS(1671),
    [aux_sym_on_keyword_token1] = ACTIONS(1671),
    [aux_sym_off_keyword_token1] = ACTIONS(1671),
    [sym_send_level_keyword] = ACTIONS(1671),
    [sym_send_string_keyword] = ACTIONS(1671),
    [sym_send_command_keyword] = ACTIONS(1671),
    [sym_clear_buffer_keyword] = ACTIONS(1671),
    [sym_create_buffer_keyword] = ACTIONS(1671),
    [sym_create_multi_buffer_keyword] = ACTIONS(1671),
    [sym_call_keyword] = ACTIONS(1671),
    [sym_system_call_keyword] = ACTIONS(1671),
    [sym_devchan_to_keyword] = ACTIONS(1671),
    [sym_devchan_min_to_keyword] = ACTIONS(1671),
    [sym_devchan_total_off_keyword] = ACTIONS(1671),
    [sym_devchan_pulse_keyword] = ACTIONS(1671),
    [sym_wait_keyword] = ACTIONS(1671),
    [sym_cancel_wait_keyword] = ACTIONS(1671),
    [sym_wait_until_keyword] = ACTIONS(1671),
    [sym_cancel_wait_until_keyword] = ACTIONS(1671),
    [sym_cancel_all_wait_keyword] = ACTIONS(1671),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1671),
    [sym_not] = ACTIONS(1671),
    [sym_bnot] = ACTIONS(1671),
    [sym___file__] = ACTIONS(1671),
    [sym___date__] = ACTIONS(1671),
    [sym___ldate__] = ACTIONS(1671),
    [sym___time__] = ACTIONS(1671),
    [sym___line__] = ACTIONS(1671),
    [sym___name__] = ACTIONS(1671),
    [sym___netlinx__] = ACTIONS(1671),
    [sym_netlinx_axi_version] = ACTIONS(1671),
    [sym_url_flg_tcp] = ACTIONS(1671),
    [sym_url_flg_accinfopresent] = ACTIONS(1671),
    [sym_url_flg_temp] = ACTIONS(1671),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1671),
    [sym_url_flg_stat_mask] = ACTIONS(1671),
    [sym_url_flg_stat_lookup] = ACTIONS(1671),
    [sym_url_flg_stat_connecting] = ACTIONS(1671),
    [sym_url_flg_stat_waiting] = ACTIONS(1671),
    [sym_url_flg_stat_connected] = ACTIONS(1671),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1671),
    [sym_first_virtual_device] = ACTIONS(1671),
    [sym_first_local_port] = ACTIONS(1671),
    [sym_dynamic_virtual_device] = ACTIONS(1671),
    [sym_timeline_once] = ACTIONS(1671),
    [sym_timeline_repeat] = ACTIONS(1671),
    [sym_timeline_absolute] = ACTIONS(1671),
    [sym_timeline_relative] = ACTIONS(1671),
    [sym_do_push_timed_infinite] = ACTIONS(1671),
    [sym_source_type_no_address] = ACTIONS(1671),
    [sym_source_type_neuron_id] = ACTIONS(1671),
    [sym_source_type_ip_address] = ACTIONS(1671),
    [sym_source_type_axlink] = ACTIONS(1671),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1671),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1671),
    [sym_source_type_ip_socket_address] = ACTIONS(1671),
    [sym_source_type_rs232] = ACTIONS(1671),
    [sym_source_type_internal] = ACTIONS(1671),
    [sym_source_type_mac_address] = ACTIONS(1671),
    [sym_source_type_ipv4_port] = ACTIONS(1671),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1671),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1671),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1671),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1671),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1671),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1671),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1671),
    [sym_true] = ACTIONS(1671),
    [sym_false] = ACTIONS(1671),
    [sym_file_read_only] = ACTIONS(1671),
    [sym_file_rw_new] = ACTIONS(1671),
    [sym_file_rw_append] = ACTIONS(1671),
    [sym_ip_tcp] = ACTIONS(1671),
    [sym_ip_udp] = ACTIONS(1671),
    [sym_ip_udp_2way] = ACTIONS(1671),
    [sym_xml_encode_types] = ACTIONS(1671),
    [sym_xml_encode_char_as_list] = ACTIONS(1671),
    [sym_xml_encode_le] = ACTIONS(1671),
    [sym_xml_decode_types] = ACTIONS(1671),
    [sym_xml_decode_no_preserve] = ACTIONS(1671),
    [sym_duet_dev_type_display_device] = ACTIONS(1671),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1671),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1671),
    [sym_duet_dev_type_io_device] = ACTIONS(1671),
    [sym_duet_dev_type_relay_device] = ACTIONS(1671),
    [sym_duet_dev_type_ups] = ACTIONS(1671),
    [sym_duet_dev_type_amplifier] = ACTIONS(1671),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1671),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1671),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1671),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1671),
    [sym_duet_dev_type_camera] = ACTIONS(1671),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1671),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1671),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1671),
    [sym_duet_dev_type_dss] = ACTIONS(1671),
    [sym_duet_dev_type_dvr] = ACTIONS(1671),
    [sym_duet_dev_type_disc_device] = ACTIONS(1671),
    [sym_duet_dev_type_document_camera] = ACTIONS(1671),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1671),
    [sym_duet_dev_type_hvac] = ACTIONS(1671),
    [sym_duet_dev_type_keypad] = ACTIONS(1671),
    [sym_duet_dev_type_light] = ACTIONS(1671),
    [sym_duet_dev_type_monitor] = ACTIONS(1671),
    [sym_duet_dev_type_motor] = ACTIONS(1671),
    [sym_duet_dev_type_multi_window] = ACTIONS(1671),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1671),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1671),
    [sym_duet_dev_type_receiver] = ACTIONS(1671),
    [sym_duet_dev_type_security_system] = ACTIONS(1671),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1671),
    [sym_duet_dev_type_settop_box] = ACTIONS(1671),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1671),
    [sym_duet_dev_type_switcher] = ACTIONS(1671),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1671),
    [sym_duet_dev_type_tv] = ACTIONS(1671),
    [sym_duet_dev_type_utility] = ACTIONS(1671),
    [sym_duet_dev_type_vcr] = ACTIONS(1671),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1671),
    [sym_duet_dev_type_video_processor] = ACTIONS(1671),
    [sym_duet_dev_type_video_projector] = ACTIONS(1671),
    [sym_duet_dev_type_video_wall] = ACTIONS(1671),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1671),
    [sym_duet_dev_type_weather] = ACTIONS(1671),
    [sym_duet_dev_not_polled] = ACTIONS(1671),
    [sym_duet_dev_polled] = ACTIONS(1671),
    [sym_clkmgr_mode_network] = ACTIONS(1671),
    [sym_clkmgr_mode_standalone] = ACTIONS(1671),
    [sym_normal_standby] = ACTIONS(1671),
    [sym_normal_wake] = ACTIONS(1671),
    [sym_ip_multicast_ttl_option] = ACTIONS(1671),
    [sym_ip_ttl_subnet] = ACTIONS(1671),
    [sym_ip_ttl_site] = ACTIONS(1671),
    [sym_ip_ttl_region] = ACTIONS(1671),
    [sym_ip_ttl_continent] = ACTIONS(1671),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1671),
    [sym_ip_nodelay_on] = ACTIONS(1671),
    [sym_ip_nodelay_off] = ACTIONS(1671),
    [sym_amx_error] = ACTIONS(1671),
    [sym_amx_warning] = ACTIONS(1671),
    [sym_amx_info] = ACTIONS(1671),
    [sym_amx_debug] = ACTIONS(1671),
    [sym_netlinx_login_success] = ACTIONS(1671),
    [sym_netlinx_login_fail] = ACTIONS(1671),
    [sym_netlinx_logout] = ACTIONS(1671),
    [sym_null_str] = ACTIONS(1671),
    [sym_smtp_address] = ACTIONS(1671),
    [sym_smtp_port_number] = ACTIONS(1671),
    [sym_smtp_username] = ACTIONS(1671),
    [sym_smtp_password] = ACTIONS(1671),
    [sym_smtp_require_tls] = ACTIONS(1671),
    [sym_smtp_from] = ACTIONS(1671),
    [sym_smtp_tls_true] = ACTIONS(1671),
    [sym_smtp_tls_false] = ACTIONS(1671),
    [sym_listview_on_row_select_event] = ACTIONS(1671),
    [sym_data_structure_unknown] = ACTIONS(1671),
    [sym_data_structure_datafeed] = ACTIONS(1671),
    [sym_data_structure_datarecordset] = ACTIONS(1671),
    [sym_data_structure_datarecord] = ACTIONS(1671),
    [sym_data_structure_datafield] = ACTIONS(1671),
    [sym_data_max_content_field_count] = ACTIONS(1671),
    [sym_data_max_metadata_field_count] = ACTIONS(1671),
    [sym_data_max_id_length] = ACTIONS(1671),
    [sym_data_max_name_length] = ACTIONS(1671),
    [sym_data_max_description_length] = ACTIONS(1671),
    [sym_data_max_type_length] = ACTIONS(1671),
    [sym_data_max_format_length] = ACTIONS(1671),
    [sym_data_max_source_length] = ACTIONS(1671),
    [sym_data_max_value_length] = ACTIONS(1671),
    [sym_data_max_label_length] = ACTIONS(1671),
    [sym_data_max_url_length] = ACTIONS(1671),
    [sym_max_last_login_info_length] = ACTIONS(1671),
    [sym_data_type_unknown] = ACTIONS(1671),
    [sym_data_type_string] = ACTIONS(1671),
    [sym_data_type_datetime] = ACTIONS(1671),
    [sym_data_type_date] = ACTIONS(1671),
    [sym_data_type_time] = ACTIONS(1671),
    [sym_data_type_image] = ACTIONS(1671),
    [sym_data_format_url] = ACTIONS(1671),
    [sym_data_format_phone] = ACTIONS(1671),
    [sym_data_format_email] = ACTIONS(1671),
    [sym_data_format_iso8601] = ACTIONS(1671),
    [sym_auditlog_priv] = ACTIONS(1671),
    [sym_configuration_priv] = ACTIONS(1671),
    [sym_deviceconfig_priv] = ACTIONS(1671),
    [sym_ftp_priv] = ACTIONS(1671),
    [sym_http_priv] = ACTIONS(1671),
    [sym_networkconfig_priv] = ACTIONS(1671),
    [sym_programport_priv] = ACTIONS(1671),
    [sym_remoteui_priv] = ACTIONS(1671),
    [sym_securitycontrol_priv] = ACTIONS(1671),
    [sym_softwaremanagement_priv] = ACTIONS(1671),
    [sym_terminal_priv] = ACTIONS(1671),
    [sym_tpadmin_priv] = ACTIONS(1671),
    [sym_user1_priv] = ACTIONS(1671),
    [sym_user2_priv] = ACTIONS(1671),
    [sym_user3_priv] = ACTIONS(1671),
    [sym_user4_priv] = ACTIONS(1671),
    [sym_usermanagement_priv] = ACTIONS(1671),
    [sym_valid_account] = ACTIONS(1671),
    [sym_err_name_invalid] = ACTIONS(1671),
    [sym_err_password_invalid] = ACTIONS(1671),
    [sym_err_invalid_account] = ACTIONS(1671),
    [sym_err_no_authorization] = ACTIONS(1671),
    [sym_err_invalid_parameter] = ACTIONS(1671),
    [sym_err_user_locked_out] = ACTIONS(1671),
    [sym_err_user_account_expired] = ACTIONS(1671),
    [sym_err_general_error] = ACTIONS(1671),
    [sym_err_authentication_error] = ACTIONS(1671),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1671),
    [sym_tls_validate_certificate] = ACTIONS(1671),
    [sym_tdata] = ACTIONS(1671),
    [sym_tchannel] = ACTIONS(1671),
    [sym_tlevel] = ACTIONS(1671),
    [sym_tbutton] = ACTIONS(1671),
    [sym_ttimeline] = ACTIONS(1671),
    [sym_tcustom] = ACTIONS(1671),
    [sym_url_struct] = ACTIONS(1671),
    [sym_dns_struct] = ACTIONS(1671),
    [sym_ip_address_struct] = ACTIONS(1671),
    [sym_dev_info_struct] = ACTIONS(1671),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1671),
    [sym_data_feed] = ACTIONS(1671),
    [sym_data_field] = ACTIONS(1671),
    [sym_data_record] = ACTIONS(1671),
    [sym_wc_data_feed] = ACTIONS(1671),
    [sym_wc_data_field] = ACTIONS(1671),
    [sym_wc_data_record] = ACTIONS(1671),
    [sym_last_login_info] = ACTIONS(1671),
    [sym____reserved___] = ACTIONS(1671),
    [sym_date] = ACTIONS(1671),
    [sym_day] = ACTIONS(1671),
    [sym_time] = ACTIONS(1671),
    [sym_ldate] = ACTIONS(1671),
    [sym_push_device] = ACTIONS(1671),
    [sym_push_channel] = ACTIONS(1671),
    [sym_push_devchan] = ACTIONS(1671),
    [sym_release_device] = ACTIONS(1671),
    [sym_release_channel] = ACTIONS(1671),
    [sym_release_devchan] = ACTIONS(1671),
    [sym_master_slot] = ACTIONS(1671),
    [sym_get_pulse_time] = ACTIONS(1671),
    [sym_get_timer] = ACTIONS(1671),
    [sym_master_sn] = ACTIONS(1671),
    [sym_system_number] = ACTIONS(1671),
    [sym_button] = ACTIONS(1671),
    [sym_dv_channel] = ACTIONS(1671),
    [sym_data] = ACTIONS(1671),
    [sym_level] = ACTIONS(1671),
    [sym_timeline] = ACTIONS(1671),
    [sym_channel] = ACTIONS(1671),
    [sym_custom] = ACTIONS(1671),
    [sym_atoi] = ACTIONS(1671),
    [sym_atol] = ACTIONS(1671),
    [sym_atof] = ACTIONS(1671),
    [sym_itoa] = ACTIONS(1671),
    [sym_itohex] = ACTIONS(1671),
    [sym_hextoi] = ACTIONS(1671),
    [sym_ftoa] = ACTIONS(1671),
    [sym_find_string] = ACTIONS(1671),
    [sym_left_string] = ACTIONS(1671),
    [sym_right_string] = ACTIONS(1671),
    [sym_mid_string] = ACTIONS(1671),
    [sym_remove_string] = ACTIONS(1671),
    [sym_get_buffer_string] = ACTIONS(1671),
    [sym_get_buffer_char] = ACTIONS(1671),
    [sym_get_multi_buffer_string] = ACTIONS(1671),
    [sym_length_string] = ACTIONS(1671),
    [sym_max_length_string] = ACTIONS(1671),
    [sym_set_length_string] = ACTIONS(1671),
    [sym_compare_string] = ACTIONS(1671),
    [sym_upper_string] = ACTIONS(1671),
    [sym_lower_string] = ACTIONS(1671),
    [sym_get_last] = ACTIONS(1671),
    [sym_date_to_day] = ACTIONS(1671),
    [sym_date_to_month] = ACTIONS(1671),
    [sym_date_to_year] = ACTIONS(1671),
    [sym_time_to_hour] = ACTIONS(1671),
    [sym_time_to_minute] = ACTIONS(1671),
    [sym_time_to_second] = ACTIONS(1671),
    [sym_day_of_week] = ACTIONS(1671),
    [sym_set_pulse_time] = ACTIONS(1671),
    [sym_set_timer] = ACTIONS(1671),
    [sym_file_open] = ACTIONS(1671),
    [sym_file_close] = ACTIONS(1671),
    [sym_file_read] = ACTIONS(1671),
    [sym_file_read_line] = ACTIONS(1671),
    [sym_file_write] = ACTIONS(1671),
    [sym_file_write_line] = ACTIONS(1671),
    [sym_file_copy] = ACTIONS(1671),
    [sym_file_delete] = ACTIONS(1671),
    [sym_file_rename] = ACTIONS(1671),
    [sym_file_dir] = ACTIONS(1671),
    [sym_file_getdir] = ACTIONS(1671),
    [sym_file_setdir] = ACTIONS(1671),
    [sym_file_create_dir] = ACTIONS(1671),
    [sym_file_remove_dir] = ACTIONS(1671),
    [sym_file_seek] = ACTIONS(1671),
    [sym_length_array] = ACTIONS(1671),
    [sym_max_length_array] = ACTIONS(1671),
    [sym_set_length_array] = ACTIONS(1671),
    [sym_ip_client_open] = ACTIONS(1671),
    [sym_ip_client_close] = ACTIONS(1671),
    [sym_ip_server_open] = ACTIONS(1671),
    [sym_ip_server_close] = ACTIONS(1671),
    [sym_ip_bound_client_open] = ACTIONS(1671),
    [sym_ip_mc_server_open] = ACTIONS(1671),
    [sym_ip_set_option] = ACTIONS(1671),
    [sym_ssh_client_open] = ACTIONS(1671),
    [sym_ssh_client_close] = ACTIONS(1671),
    [sym_tls_client_open] = ACTIONS(1671),
    [sym_tls_client_close] = ACTIONS(1671),
    [sym_do_push] = ACTIONS(1671),
    [sym_do_push_timed] = ACTIONS(1671),
    [sym_do_release] = ACTIONS(1671),
    [sym_device_id] = ACTIONS(1671),
    [sym_device_info] = ACTIONS(1671),
    [sym_device_id_string] = ACTIONS(1671),
    [sym_device_standby] = ACTIONS(1671),
    [sym_device_wake] = ACTIONS(1671),
    [sym_timeline_create] = ACTIONS(1671),
    [sym_timeline_kill] = ACTIONS(1671),
    [sym_timeline_reload] = ACTIONS(1671),
    [sym_timeline_pause] = ACTIONS(1671),
    [sym_timeline_restart] = ACTIONS(1671),
    [sym_timeline_set] = ACTIONS(1671),
    [sym_timeline_get] = ACTIONS(1671),
    [sym_timeline_active] = ACTIONS(1671),
    [sym_timeline_dynamic_id] = ACTIONS(1671),
    [sym_variable_to_string] = ACTIONS(1671),
    [sym_string_to_variable] = ACTIONS(1671),
    [sym_length_variable_to_string] = ACTIONS(1671),
    [sym_type_cast] = ACTIONS(1671),
    [sym_variable_to_xml] = ACTIONS(1671),
    [sym_xml_to_variable] = ACTIONS(1671),
    [sym_length_variable_to_xml] = ACTIONS(1671),
    [sym_redirect_string] = ACTIONS(1671),
    [sym_random_number] = ACTIONS(1671),
    [sym_get_unique_id] = ACTIONS(1671),
    [sym_get_system_number] = ACTIONS(1671),
    [sym_set_system_number] = ACTIONS(1671),
    [sym_get_serial_number] = ACTIONS(1671),
    [sym_get_master_build] = ACTIONS(1671),
    [sym_set_validation_code] = ACTIONS(1671),
    [sym_reboot] = ACTIONS(1671),
    [sym_load_duet_module] = ACTIONS(1671),
    [sym_unload_duet_module] = ACTIONS(1671),
    [sym_duet_mem_size_set] = ACTIONS(1671),
    [sym_duet_mem_size_get] = ACTIONS(1671),
    [sym_dynamic_application_device] = ACTIONS(1671),
    [sym_dynamic_polled_port] = ACTIONS(1671),
    [sym_static_port_binding] = ACTIONS(1671),
    [sym_static_ip_binding] = ACTIONS(1671),
    [sym_get_dns_list] = ACTIONS(1671),
    [sym_set_dns_list] = ACTIONS(1671),
    [sym_get_ip_address] = ACTIONS(1671),
    [sym_set_ip_address] = ACTIONS(1671),
    [sym_get_url_list] = ACTIONS(1671),
    [sym_add_url_entry] = ACTIONS(1671),
    [sym_delete_url_entry] = ACTIONS(1671),
    [sym_abs_value] = ACTIONS(1671),
    [sym_max_value] = ACTIONS(1671),
    [sym_min_value] = ACTIONS(1671),
    [sym_exp_value] = ACTIONS(1671),
    [sym_log_value] = ACTIONS(1671),
    [sym_log10_value] = ACTIONS(1671),
    [sym_power_value] = ACTIONS(1671),
    [sym_sqrt_value] = ACTIONS(1671),
    [sym_raw_be] = ACTIONS(1671),
    [sym_raw_le] = ACTIONS(1671),
    [sym_internal_queue_size_set] = ACTIONS(1671),
    [sym_internal_queue_size_get] = ACTIONS(1671),
    [sym_internal_threshold_set] = ACTIONS(1671),
    [sym_internal_threshold_get] = ACTIONS(1671),
    [sym_rebuilt_event] = ACTIONS(1671),
    [sym_do_custom_event] = ACTIONS(1671),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1671),
    [sym_clkmgr_set_clk_source] = ACTIONS(1671),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1671),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1671),
    [sym_clkmgr_get_timezone] = ACTIONS(1671),
    [sym_clkmgr_set_timezone] = ACTIONS(1671),
    [sym_clkmgr_get_resync_period] = ACTIONS(1671),
    [sym_clkmgr_set_resync_period] = ACTIONS(1671),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1671),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1671),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1671),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1671),
    [sym_clkmgr_get_timeservers] = ACTIONS(1671),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1671),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1671),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1671),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1671),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1671),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1671),
    [sym_set_log_level] = ACTIONS(1671),
    [sym_get_log_level] = ACTIONS(1671),
    [sym_amx_log] = ACTIONS(1671),
    [sym_smtp_server_config_set] = ACTIONS(1671),
    [sym_smtp_server_config_get] = ACTIONS(1671),
    [sym_smtp_send] = ACTIONS(1671),
    [sym_validate_netlinx_account] = ACTIONS(1671),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1671),
    [sym_authenticate_certificate] = ACTIONS(1671),
    [sym_audit_netlinx_session_event] = ACTIONS(1671),
    [sym_audit_netlinx_generic_event] = ACTIONS(1671),
    [sym_get_available_flash_disk_space] = ACTIONS(1671),
    [sym_get_max_flash_disk_space] = ACTIONS(1671),
    [sym_data_create_feed] = ACTIONS(1671),
    [sym_data_delete_feed] = ACTIONS(1671),
    [sym_data_publish_feed] = ACTIONS(1671),
    [sym_data_get_published_feed] = ACTIONS(1671),
    [sym_data_add_record] = ACTIONS(1671),
    [sym_data_get_event_record] = ACTIONS(1671),
    [sym__wc_data_create_feed] = ACTIONS(1671),
    [sym__wc_data_add_record] = ACTIONS(1671),
    [sym__wc_data_get_event_record] = ACTIONS(1671),
    [sym_format] = ACTIONS(1671),
    [sym_astro_clock] = ACTIONS(1671),
    [sym_set_outdoor_temperature] = ACTIONS(1671),
    [sym_set_virtual_level_count] = ACTIONS(1671),
    [sym_set_virtual_channel_count] = ACTIONS(1671),
    [sym_set_virtual_port_count] = ACTIONS(1671),
    [sym__wc_encode_int] = ACTIONS(1671),
    [sym__wc_decode_int] = ACTIONS(1671),
    [sym__wc_int] = ACTIONS(1671),
    [sym__wc_to_ch_int] = ACTIONS(1671),
    [sym__ch_to_wc_int] = ACTIONS(1671),
    [sym__wc_find_string_int] = ACTIONS(1671),
    [sym__wc_left_string_int] = ACTIONS(1671),
    [sym__wc_lower_string_int] = ACTIONS(1671),
    [sym__wc_mid_string_int] = ACTIONS(1671),
    [sym__wc_remove_string_int] = ACTIONS(1671),
    [sym__wc_right_string_int] = ACTIONS(1671),
    [sym__wc_upper_string_int] = ACTIONS(1671),
    [sym__wc_compare_string_int] = ACTIONS(1671),
    [sym__wc_get_buffer_char_int] = ACTIONS(1671),
    [sym__wc_get_buffer_string_int] = ACTIONS(1671),
    [sym__wc_concat_string_int] = ACTIONS(1671),
    [sym__wc_file_open_int] = ACTIONS(1671),
    [sym__wc_file_close_int] = ACTIONS(1671),
    [sym__wc_file_read_int] = ACTIONS(1671),
    [sym__wc_file_read_line_int] = ACTIONS(1671),
    [sym__wc_file_write_int] = ACTIONS(1671),
    [sym__wc_file_write_line_int] = ACTIONS(1671),
    [aux_sym_preproc_include_token1] = ACTIONS(1671),
    [aux_sym_preproc_define_token1] = ACTIONS(1669),
    [aux_sym_preproc_warn_token1] = ACTIONS(1669),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1669),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1669),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1669),
    [anon_sym_LBRACK] = ACTIONS(1669),
    [anon_sym_LBRACE] = ACTIONS(1669),
    [anon_sym_RBRACE] = ACTIONS(1669),
    [anon_sym_LPAREN] = ACTIONS(1671),
    [anon_sym_BANG] = ACTIONS(1669),
    [anon_sym_TILDE] = ACTIONS(1669),
    [anon_sym_DASH] = ACTIONS(1671),
    [anon_sym_PLUS] = ACTIONS(1671),
    [anon_sym_DASH_DASH] = ACTIONS(1669),
    [anon_sym_PLUS_PLUS] = ACTIONS(1669),
    [anon_sym_DQUOTE] = ACTIONS(1669),
    [anon_sym_SQUOTE] = ACTIONS(1669),
    [sym_number_literal] = ACTIONS(1669),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(519)] = {
    [sym_identifier] = ACTIONS(1831),
    [sym_define_call_keyword] = ACTIONS(1831),
    [sym_define_function_keyword] = ACTIONS(1831),
    [sym_define_library_function_keyword] = ACTIONS(1831),
    [sym_define_module_keyword] = ACTIONS(1831),
    [sym_char_keyword] = ACTIONS(1831),
    [sym_widechar_keyword] = ACTIONS(1831),
    [sym_integer_keyword] = ACTIONS(1831),
    [sym_sinteger_keyword] = ACTIONS(1831),
    [sym_long_keyword] = ACTIONS(1831),
    [sym_slong_keyword] = ACTIONS(1831),
    [sym_float_keyword] = ACTIONS(1831),
    [sym_double_keyword] = ACTIONS(1831),
    [sym_variant_keyword] = ACTIONS(1831),
    [sym_variantarray_keyword] = ACTIONS(1831),
    [sym_local_var_keyword] = ACTIONS(1831),
    [sym_stack_var_keyword] = ACTIONS(1831),
    [sym_constant_keyword] = ACTIONS(1831),
    [sym_volatile_keyword] = ACTIONS(1831),
    [sym_non_volatile_keyword] = ACTIONS(1831),
    [sym_persistent_keyword] = ACTIONS(1831),
    [sym_dev_keyword] = ACTIONS(1831),
    [sym_devlev_keyword] = ACTIONS(1831),
    [sym_devchan_keyword] = ACTIONS(1831),
    [sym_if_keyword] = ACTIONS(1831),
    [sym_else_keyword] = ACTIONS(1831),
    [sym_switch_keyword] = ACTIONS(1831),
    [sym_case_keyword] = ACTIONS(1831),
    [sym_default_keyword] = ACTIONS(1831),
    [sym_while_keyword] = ACTIONS(1831),
    [sym_for_keyword] = ACTIONS(1831),
    [sym_break_keyword] = ACTIONS(1831),
    [sym_continue_keyword] = ACTIONS(1831),
    [sym_return_keyword] = ACTIONS(1831),
    [sym_select_keyword] = ACTIONS(1831),
    [sym_active_keyword] = ACTIONS(1831),
    [sym_struct_keyword] = ACTIONS(1831),
    [sym_structure_keyword] = ACTIONS(1831),
    [aux_sym_on_keyword_token1] = ACTIONS(1831),
    [aux_sym_off_keyword_token1] = ACTIONS(1831),
    [sym_send_level_keyword] = ACTIONS(1831),
    [sym_send_string_keyword] = ACTIONS(1831),
    [sym_send_command_keyword] = ACTIONS(1831),
    [sym_clear_buffer_keyword] = ACTIONS(1831),
    [sym_create_buffer_keyword] = ACTIONS(1831),
    [sym_create_multi_buffer_keyword] = ACTIONS(1831),
    [sym_call_keyword] = ACTIONS(1831),
    [sym_system_call_keyword] = ACTIONS(1831),
    [sym_devchan_to_keyword] = ACTIONS(1831),
    [sym_devchan_min_to_keyword] = ACTIONS(1831),
    [sym_devchan_total_off_keyword] = ACTIONS(1831),
    [sym_devchan_pulse_keyword] = ACTIONS(1831),
    [sym_wait_keyword] = ACTIONS(1831),
    [sym_cancel_wait_keyword] = ACTIONS(1831),
    [sym_wait_until_keyword] = ACTIONS(1831),
    [sym_cancel_wait_until_keyword] = ACTIONS(1831),
    [sym_cancel_all_wait_keyword] = ACTIONS(1831),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1831),
    [sym_not] = ACTIONS(1831),
    [sym_bnot] = ACTIONS(1831),
    [sym___file__] = ACTIONS(1831),
    [sym___date__] = ACTIONS(1831),
    [sym___ldate__] = ACTIONS(1831),
    [sym___time__] = ACTIONS(1831),
    [sym___line__] = ACTIONS(1831),
    [sym___name__] = ACTIONS(1831),
    [sym___netlinx__] = ACTIONS(1831),
    [sym_netlinx_axi_version] = ACTIONS(1831),
    [sym_url_flg_tcp] = ACTIONS(1831),
    [sym_url_flg_accinfopresent] = ACTIONS(1831),
    [sym_url_flg_temp] = ACTIONS(1831),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1831),
    [sym_url_flg_stat_mask] = ACTIONS(1831),
    [sym_url_flg_stat_lookup] = ACTIONS(1831),
    [sym_url_flg_stat_connecting] = ACTIONS(1831),
    [sym_url_flg_stat_waiting] = ACTIONS(1831),
    [sym_url_flg_stat_connected] = ACTIONS(1831),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1831),
    [sym_first_virtual_device] = ACTIONS(1831),
    [sym_first_local_port] = ACTIONS(1831),
    [sym_dynamic_virtual_device] = ACTIONS(1831),
    [sym_timeline_once] = ACTIONS(1831),
    [sym_timeline_repeat] = ACTIONS(1831),
    [sym_timeline_absolute] = ACTIONS(1831),
    [sym_timeline_relative] = ACTIONS(1831),
    [sym_do_push_timed_infinite] = ACTIONS(1831),
    [sym_source_type_no_address] = ACTIONS(1831),
    [sym_source_type_neuron_id] = ACTIONS(1831),
    [sym_source_type_ip_address] = ACTIONS(1831),
    [sym_source_type_axlink] = ACTIONS(1831),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1831),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1831),
    [sym_source_type_ip_socket_address] = ACTIONS(1831),
    [sym_source_type_rs232] = ACTIONS(1831),
    [sym_source_type_internal] = ACTIONS(1831),
    [sym_source_type_mac_address] = ACTIONS(1831),
    [sym_source_type_ipv4_port] = ACTIONS(1831),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1831),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1831),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1831),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1831),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1831),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1831),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1831),
    [sym_true] = ACTIONS(1831),
    [sym_false] = ACTIONS(1831),
    [sym_file_read_only] = ACTIONS(1831),
    [sym_file_rw_new] = ACTIONS(1831),
    [sym_file_rw_append] = ACTIONS(1831),
    [sym_ip_tcp] = ACTIONS(1831),
    [sym_ip_udp] = ACTIONS(1831),
    [sym_ip_udp_2way] = ACTIONS(1831),
    [sym_xml_encode_types] = ACTIONS(1831),
    [sym_xml_encode_char_as_list] = ACTIONS(1831),
    [sym_xml_encode_le] = ACTIONS(1831),
    [sym_xml_decode_types] = ACTIONS(1831),
    [sym_xml_decode_no_preserve] = ACTIONS(1831),
    [sym_duet_dev_type_display_device] = ACTIONS(1831),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1831),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1831),
    [sym_duet_dev_type_io_device] = ACTIONS(1831),
    [sym_duet_dev_type_relay_device] = ACTIONS(1831),
    [sym_duet_dev_type_ups] = ACTIONS(1831),
    [sym_duet_dev_type_amplifier] = ACTIONS(1831),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1831),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1831),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1831),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1831),
    [sym_duet_dev_type_camera] = ACTIONS(1831),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1831),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1831),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1831),
    [sym_duet_dev_type_dss] = ACTIONS(1831),
    [sym_duet_dev_type_dvr] = ACTIONS(1831),
    [sym_duet_dev_type_disc_device] = ACTIONS(1831),
    [sym_duet_dev_type_document_camera] = ACTIONS(1831),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1831),
    [sym_duet_dev_type_hvac] = ACTIONS(1831),
    [sym_duet_dev_type_keypad] = ACTIONS(1831),
    [sym_duet_dev_type_light] = ACTIONS(1831),
    [sym_duet_dev_type_monitor] = ACTIONS(1831),
    [sym_duet_dev_type_motor] = ACTIONS(1831),
    [sym_duet_dev_type_multi_window] = ACTIONS(1831),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1831),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1831),
    [sym_duet_dev_type_receiver] = ACTIONS(1831),
    [sym_duet_dev_type_security_system] = ACTIONS(1831),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1831),
    [sym_duet_dev_type_settop_box] = ACTIONS(1831),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1831),
    [sym_duet_dev_type_switcher] = ACTIONS(1831),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1831),
    [sym_duet_dev_type_tv] = ACTIONS(1831),
    [sym_duet_dev_type_utility] = ACTIONS(1831),
    [sym_duet_dev_type_vcr] = ACTIONS(1831),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1831),
    [sym_duet_dev_type_video_processor] = ACTIONS(1831),
    [sym_duet_dev_type_video_projector] = ACTIONS(1831),
    [sym_duet_dev_type_video_wall] = ACTIONS(1831),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1831),
    [sym_duet_dev_type_weather] = ACTIONS(1831),
    [sym_duet_dev_not_polled] = ACTIONS(1831),
    [sym_duet_dev_polled] = ACTIONS(1831),
    [sym_clkmgr_mode_network] = ACTIONS(1831),
    [sym_clkmgr_mode_standalone] = ACTIONS(1831),
    [sym_normal_standby] = ACTIONS(1831),
    [sym_normal_wake] = ACTIONS(1831),
    [sym_ip_multicast_ttl_option] = ACTIONS(1831),
    [sym_ip_ttl_subnet] = ACTIONS(1831),
    [sym_ip_ttl_site] = ACTIONS(1831),
    [sym_ip_ttl_region] = ACTIONS(1831),
    [sym_ip_ttl_continent] = ACTIONS(1831),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1831),
    [sym_ip_nodelay_on] = ACTIONS(1831),
    [sym_ip_nodelay_off] = ACTIONS(1831),
    [sym_amx_error] = ACTIONS(1831),
    [sym_amx_warning] = ACTIONS(1831),
    [sym_amx_info] = ACTIONS(1831),
    [sym_amx_debug] = ACTIONS(1831),
    [sym_netlinx_login_success] = ACTIONS(1831),
    [sym_netlinx_login_fail] = ACTIONS(1831),
    [sym_netlinx_logout] = ACTIONS(1831),
    [sym_null_str] = ACTIONS(1831),
    [sym_smtp_address] = ACTIONS(1831),
    [sym_smtp_port_number] = ACTIONS(1831),
    [sym_smtp_username] = ACTIONS(1831),
    [sym_smtp_password] = ACTIONS(1831),
    [sym_smtp_require_tls] = ACTIONS(1831),
    [sym_smtp_from] = ACTIONS(1831),
    [sym_smtp_tls_true] = ACTIONS(1831),
    [sym_smtp_tls_false] = ACTIONS(1831),
    [sym_listview_on_row_select_event] = ACTIONS(1831),
    [sym_data_structure_unknown] = ACTIONS(1831),
    [sym_data_structure_datafeed] = ACTIONS(1831),
    [sym_data_structure_datarecordset] = ACTIONS(1831),
    [sym_data_structure_datarecord] = ACTIONS(1831),
    [sym_data_structure_datafield] = ACTIONS(1831),
    [sym_data_max_content_field_count] = ACTIONS(1831),
    [sym_data_max_metadata_field_count] = ACTIONS(1831),
    [sym_data_max_id_length] = ACTIONS(1831),
    [sym_data_max_name_length] = ACTIONS(1831),
    [sym_data_max_description_length] = ACTIONS(1831),
    [sym_data_max_type_length] = ACTIONS(1831),
    [sym_data_max_format_length] = ACTIONS(1831),
    [sym_data_max_source_length] = ACTIONS(1831),
    [sym_data_max_value_length] = ACTIONS(1831),
    [sym_data_max_label_length] = ACTIONS(1831),
    [sym_data_max_url_length] = ACTIONS(1831),
    [sym_max_last_login_info_length] = ACTIONS(1831),
    [sym_data_type_unknown] = ACTIONS(1831),
    [sym_data_type_string] = ACTIONS(1831),
    [sym_data_type_datetime] = ACTIONS(1831),
    [sym_data_type_date] = ACTIONS(1831),
    [sym_data_type_time] = ACTIONS(1831),
    [sym_data_type_image] = ACTIONS(1831),
    [sym_data_format_url] = ACTIONS(1831),
    [sym_data_format_phone] = ACTIONS(1831),
    [sym_data_format_email] = ACTIONS(1831),
    [sym_data_format_iso8601] = ACTIONS(1831),
    [sym_auditlog_priv] = ACTIONS(1831),
    [sym_configuration_priv] = ACTIONS(1831),
    [sym_deviceconfig_priv] = ACTIONS(1831),
    [sym_ftp_priv] = ACTIONS(1831),
    [sym_http_priv] = ACTIONS(1831),
    [sym_networkconfig_priv] = ACTIONS(1831),
    [sym_programport_priv] = ACTIONS(1831),
    [sym_remoteui_priv] = ACTIONS(1831),
    [sym_securitycontrol_priv] = ACTIONS(1831),
    [sym_softwaremanagement_priv] = ACTIONS(1831),
    [sym_terminal_priv] = ACTIONS(1831),
    [sym_tpadmin_priv] = ACTIONS(1831),
    [sym_user1_priv] = ACTIONS(1831),
    [sym_user2_priv] = ACTIONS(1831),
    [sym_user3_priv] = ACTIONS(1831),
    [sym_user4_priv] = ACTIONS(1831),
    [sym_usermanagement_priv] = ACTIONS(1831),
    [sym_valid_account] = ACTIONS(1831),
    [sym_err_name_invalid] = ACTIONS(1831),
    [sym_err_password_invalid] = ACTIONS(1831),
    [sym_err_invalid_account] = ACTIONS(1831),
    [sym_err_no_authorization] = ACTIONS(1831),
    [sym_err_invalid_parameter] = ACTIONS(1831),
    [sym_err_user_locked_out] = ACTIONS(1831),
    [sym_err_user_account_expired] = ACTIONS(1831),
    [sym_err_general_error] = ACTIONS(1831),
    [sym_err_authentication_error] = ACTIONS(1831),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1831),
    [sym_tls_validate_certificate] = ACTIONS(1831),
    [sym_tdata] = ACTIONS(1831),
    [sym_tchannel] = ACTIONS(1831),
    [sym_tlevel] = ACTIONS(1831),
    [sym_tbutton] = ACTIONS(1831),
    [sym_ttimeline] = ACTIONS(1831),
    [sym_tcustom] = ACTIONS(1831),
    [sym_url_struct] = ACTIONS(1831),
    [sym_dns_struct] = ACTIONS(1831),
    [sym_ip_address_struct] = ACTIONS(1831),
    [sym_dev_info_struct] = ACTIONS(1831),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1831),
    [sym_data_feed] = ACTIONS(1831),
    [sym_data_field] = ACTIONS(1831),
    [sym_data_record] = ACTIONS(1831),
    [sym_wc_data_feed] = ACTIONS(1831),
    [sym_wc_data_field] = ACTIONS(1831),
    [sym_wc_data_record] = ACTIONS(1831),
    [sym_last_login_info] = ACTIONS(1831),
    [sym____reserved___] = ACTIONS(1831),
    [sym_date] = ACTIONS(1831),
    [sym_day] = ACTIONS(1831),
    [sym_time] = ACTIONS(1831),
    [sym_ldate] = ACTIONS(1831),
    [sym_push_device] = ACTIONS(1831),
    [sym_push_channel] = ACTIONS(1831),
    [sym_push_devchan] = ACTIONS(1831),
    [sym_release_device] = ACTIONS(1831),
    [sym_release_channel] = ACTIONS(1831),
    [sym_release_devchan] = ACTIONS(1831),
    [sym_master_slot] = ACTIONS(1831),
    [sym_get_pulse_time] = ACTIONS(1831),
    [sym_get_timer] = ACTIONS(1831),
    [sym_master_sn] = ACTIONS(1831),
    [sym_system_number] = ACTIONS(1831),
    [sym_button] = ACTIONS(1831),
    [sym_dv_channel] = ACTIONS(1831),
    [sym_data] = ACTIONS(1831),
    [sym_level] = ACTIONS(1831),
    [sym_timeline] = ACTIONS(1831),
    [sym_channel] = ACTIONS(1831),
    [sym_custom] = ACTIONS(1831),
    [sym_atoi] = ACTIONS(1831),
    [sym_atol] = ACTIONS(1831),
    [sym_atof] = ACTIONS(1831),
    [sym_itoa] = ACTIONS(1831),
    [sym_itohex] = ACTIONS(1831),
    [sym_hextoi] = ACTIONS(1831),
    [sym_ftoa] = ACTIONS(1831),
    [sym_find_string] = ACTIONS(1831),
    [sym_left_string] = ACTIONS(1831),
    [sym_right_string] = ACTIONS(1831),
    [sym_mid_string] = ACTIONS(1831),
    [sym_remove_string] = ACTIONS(1831),
    [sym_get_buffer_string] = ACTIONS(1831),
    [sym_get_buffer_char] = ACTIONS(1831),
    [sym_get_multi_buffer_string] = ACTIONS(1831),
    [sym_length_string] = ACTIONS(1831),
    [sym_max_length_string] = ACTIONS(1831),
    [sym_set_length_string] = ACTIONS(1831),
    [sym_compare_string] = ACTIONS(1831),
    [sym_upper_string] = ACTIONS(1831),
    [sym_lower_string] = ACTIONS(1831),
    [sym_get_last] = ACTIONS(1831),
    [sym_date_to_day] = ACTIONS(1831),
    [sym_date_to_month] = ACTIONS(1831),
    [sym_date_to_year] = ACTIONS(1831),
    [sym_time_to_hour] = ACTIONS(1831),
    [sym_time_to_minute] = ACTIONS(1831),
    [sym_time_to_second] = ACTIONS(1831),
    [sym_day_of_week] = ACTIONS(1831),
    [sym_set_pulse_time] = ACTIONS(1831),
    [sym_set_timer] = ACTIONS(1831),
    [sym_file_open] = ACTIONS(1831),
    [sym_file_close] = ACTIONS(1831),
    [sym_file_read] = ACTIONS(1831),
    [sym_file_read_line] = ACTIONS(1831),
    [sym_file_write] = ACTIONS(1831),
    [sym_file_write_line] = ACTIONS(1831),
    [sym_file_copy] = ACTIONS(1831),
    [sym_file_delete] = ACTIONS(1831),
    [sym_file_rename] = ACTIONS(1831),
    [sym_file_dir] = ACTIONS(1831),
    [sym_file_getdir] = ACTIONS(1831),
    [sym_file_setdir] = ACTIONS(1831),
    [sym_file_create_dir] = ACTIONS(1831),
    [sym_file_remove_dir] = ACTIONS(1831),
    [sym_file_seek] = ACTIONS(1831),
    [sym_length_array] = ACTIONS(1831),
    [sym_max_length_array] = ACTIONS(1831),
    [sym_set_length_array] = ACTIONS(1831),
    [sym_ip_client_open] = ACTIONS(1831),
    [sym_ip_client_close] = ACTIONS(1831),
    [sym_ip_server_open] = ACTIONS(1831),
    [sym_ip_server_close] = ACTIONS(1831),
    [sym_ip_bound_client_open] = ACTIONS(1831),
    [sym_ip_mc_server_open] = ACTIONS(1831),
    [sym_ip_set_option] = ACTIONS(1831),
    [sym_ssh_client_open] = ACTIONS(1831),
    [sym_ssh_client_close] = ACTIONS(1831),
    [sym_tls_client_open] = ACTIONS(1831),
    [sym_tls_client_close] = ACTIONS(1831),
    [sym_do_push] = ACTIONS(1831),
    [sym_do_push_timed] = ACTIONS(1831),
    [sym_do_release] = ACTIONS(1831),
    [sym_device_id] = ACTIONS(1831),
    [sym_device_info] = ACTIONS(1831),
    [sym_device_id_string] = ACTIONS(1831),
    [sym_device_standby] = ACTIONS(1831),
    [sym_device_wake] = ACTIONS(1831),
    [sym_timeline_create] = ACTIONS(1831),
    [sym_timeline_kill] = ACTIONS(1831),
    [sym_timeline_reload] = ACTIONS(1831),
    [sym_timeline_pause] = ACTIONS(1831),
    [sym_timeline_restart] = ACTIONS(1831),
    [sym_timeline_set] = ACTIONS(1831),
    [sym_timeline_get] = ACTIONS(1831),
    [sym_timeline_active] = ACTIONS(1831),
    [sym_timeline_dynamic_id] = ACTIONS(1831),
    [sym_variable_to_string] = ACTIONS(1831),
    [sym_string_to_variable] = ACTIONS(1831),
    [sym_length_variable_to_string] = ACTIONS(1831),
    [sym_type_cast] = ACTIONS(1831),
    [sym_variable_to_xml] = ACTIONS(1831),
    [sym_xml_to_variable] = ACTIONS(1831),
    [sym_length_variable_to_xml] = ACTIONS(1831),
    [sym_redirect_string] = ACTIONS(1831),
    [sym_random_number] = ACTIONS(1831),
    [sym_get_unique_id] = ACTIONS(1831),
    [sym_get_system_number] = ACTIONS(1831),
    [sym_set_system_number] = ACTIONS(1831),
    [sym_get_serial_number] = ACTIONS(1831),
    [sym_get_master_build] = ACTIONS(1831),
    [sym_set_validation_code] = ACTIONS(1831),
    [sym_reboot] = ACTIONS(1831),
    [sym_load_duet_module] = ACTIONS(1831),
    [sym_unload_duet_module] = ACTIONS(1831),
    [sym_duet_mem_size_set] = ACTIONS(1831),
    [sym_duet_mem_size_get] = ACTIONS(1831),
    [sym_dynamic_application_device] = ACTIONS(1831),
    [sym_dynamic_polled_port] = ACTIONS(1831),
    [sym_static_port_binding] = ACTIONS(1831),
    [sym_static_ip_binding] = ACTIONS(1831),
    [sym_get_dns_list] = ACTIONS(1831),
    [sym_set_dns_list] = ACTIONS(1831),
    [sym_get_ip_address] = ACTIONS(1831),
    [sym_set_ip_address] = ACTIONS(1831),
    [sym_get_url_list] = ACTIONS(1831),
    [sym_add_url_entry] = ACTIONS(1831),
    [sym_delete_url_entry] = ACTIONS(1831),
    [sym_abs_value] = ACTIONS(1831),
    [sym_max_value] = ACTIONS(1831),
    [sym_min_value] = ACTIONS(1831),
    [sym_exp_value] = ACTIONS(1831),
    [sym_log_value] = ACTIONS(1831),
    [sym_log10_value] = ACTIONS(1831),
    [sym_power_value] = ACTIONS(1831),
    [sym_sqrt_value] = ACTIONS(1831),
    [sym_raw_be] = ACTIONS(1831),
    [sym_raw_le] = ACTIONS(1831),
    [sym_internal_queue_size_set] = ACTIONS(1831),
    [sym_internal_queue_size_get] = ACTIONS(1831),
    [sym_internal_threshold_set] = ACTIONS(1831),
    [sym_internal_threshold_get] = ACTIONS(1831),
    [sym_rebuilt_event] = ACTIONS(1831),
    [sym_do_custom_event] = ACTIONS(1831),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1831),
    [sym_clkmgr_set_clk_source] = ACTIONS(1831),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1831),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1831),
    [sym_clkmgr_get_timezone] = ACTIONS(1831),
    [sym_clkmgr_set_timezone] = ACTIONS(1831),
    [sym_clkmgr_get_resync_period] = ACTIONS(1831),
    [sym_clkmgr_set_resync_period] = ACTIONS(1831),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1831),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1831),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1831),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1831),
    [sym_clkmgr_get_timeservers] = ACTIONS(1831),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1831),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1831),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1831),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1831),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1831),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1831),
    [sym_set_log_level] = ACTIONS(1831),
    [sym_get_log_level] = ACTIONS(1831),
    [sym_amx_log] = ACTIONS(1831),
    [sym_smtp_server_config_set] = ACTIONS(1831),
    [sym_smtp_server_config_get] = ACTIONS(1831),
    [sym_smtp_send] = ACTIONS(1831),
    [sym_validate_netlinx_account] = ACTIONS(1831),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1831),
    [sym_authenticate_certificate] = ACTIONS(1831),
    [sym_audit_netlinx_session_event] = ACTIONS(1831),
    [sym_audit_netlinx_generic_event] = ACTIONS(1831),
    [sym_get_available_flash_disk_space] = ACTIONS(1831),
    [sym_get_max_flash_disk_space] = ACTIONS(1831),
    [sym_data_create_feed] = ACTIONS(1831),
    [sym_data_delete_feed] = ACTIONS(1831),
    [sym_data_publish_feed] = ACTIONS(1831),
    [sym_data_get_published_feed] = ACTIONS(1831),
    [sym_data_add_record] = ACTIONS(1831),
    [sym_data_get_event_record] = ACTIONS(1831),
    [sym__wc_data_create_feed] = ACTIONS(1831),
    [sym__wc_data_add_record] = ACTIONS(1831),
    [sym__wc_data_get_event_record] = ACTIONS(1831),
    [sym_format] = ACTIONS(1831),
    [sym_astro_clock] = ACTIONS(1831),
    [sym_set_outdoor_temperature] = ACTIONS(1831),
    [sym_set_virtual_level_count] = ACTIONS(1831),
    [sym_set_virtual_channel_count] = ACTIONS(1831),
    [sym_set_virtual_port_count] = ACTIONS(1831),
    [sym__wc_encode_int] = ACTIONS(1831),
    [sym__wc_decode_int] = ACTIONS(1831),
    [sym__wc_int] = ACTIONS(1831),
    [sym__wc_to_ch_int] = ACTIONS(1831),
    [sym__ch_to_wc_int] = ACTIONS(1831),
    [sym__wc_find_string_int] = ACTIONS(1831),
    [sym__wc_left_string_int] = ACTIONS(1831),
    [sym__wc_lower_string_int] = ACTIONS(1831),
    [sym__wc_mid_string_int] = ACTIONS(1831),
    [sym__wc_remove_string_int] = ACTIONS(1831),
    [sym__wc_right_string_int] = ACTIONS(1831),
    [sym__wc_upper_string_int] = ACTIONS(1831),
    [sym__wc_compare_string_int] = ACTIONS(1831),
    [sym__wc_get_buffer_char_int] = ACTIONS(1831),
    [sym__wc_get_buffer_string_int] = ACTIONS(1831),
    [sym__wc_concat_string_int] = ACTIONS(1831),
    [sym__wc_file_open_int] = ACTIONS(1831),
    [sym__wc_file_close_int] = ACTIONS(1831),
    [sym__wc_file_read_int] = ACTIONS(1831),
    [sym__wc_file_read_line_int] = ACTIONS(1831),
    [sym__wc_file_write_int] = ACTIONS(1831),
    [sym__wc_file_write_line_int] = ACTIONS(1831),
    [aux_sym_preproc_include_token1] = ACTIONS(1831),
    [aux_sym_preproc_define_token1] = ACTIONS(1833),
    [aux_sym_preproc_warn_token1] = ACTIONS(1833),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1833),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1833),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1833),
    [anon_sym_LBRACK] = ACTIONS(1833),
    [anon_sym_LBRACE] = ACTIONS(1833),
    [anon_sym_RBRACE] = ACTIONS(1833),
    [anon_sym_LPAREN] = ACTIONS(1831),
    [anon_sym_BANG] = ACTIONS(1833),
    [anon_sym_TILDE] = ACTIONS(1833),
    [anon_sym_DASH] = ACTIONS(1831),
    [anon_sym_PLUS] = ACTIONS(1831),
    [anon_sym_DASH_DASH] = ACTIONS(1833),
    [anon_sym_PLUS_PLUS] = ACTIONS(1833),
    [anon_sym_DQUOTE] = ACTIONS(1833),
    [anon_sym_SQUOTE] = ACTIONS(1833),
    [sym_number_literal] = ACTIONS(1833),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(520)] = {
    [sym_identifier] = ACTIONS(1727),
    [sym_define_call_keyword] = ACTIONS(1727),
    [sym_define_function_keyword] = ACTIONS(1727),
    [sym_define_library_function_keyword] = ACTIONS(1727),
    [sym_define_module_keyword] = ACTIONS(1727),
    [sym_char_keyword] = ACTIONS(1727),
    [sym_widechar_keyword] = ACTIONS(1727),
    [sym_integer_keyword] = ACTIONS(1727),
    [sym_sinteger_keyword] = ACTIONS(1727),
    [sym_long_keyword] = ACTIONS(1727),
    [sym_slong_keyword] = ACTIONS(1727),
    [sym_float_keyword] = ACTIONS(1727),
    [sym_double_keyword] = ACTIONS(1727),
    [sym_variant_keyword] = ACTIONS(1727),
    [sym_variantarray_keyword] = ACTIONS(1727),
    [sym_local_var_keyword] = ACTIONS(1727),
    [sym_stack_var_keyword] = ACTIONS(1727),
    [sym_constant_keyword] = ACTIONS(1727),
    [sym_volatile_keyword] = ACTIONS(1727),
    [sym_non_volatile_keyword] = ACTIONS(1727),
    [sym_persistent_keyword] = ACTIONS(1727),
    [sym_dev_keyword] = ACTIONS(1727),
    [sym_devlev_keyword] = ACTIONS(1727),
    [sym_devchan_keyword] = ACTIONS(1727),
    [sym_if_keyword] = ACTIONS(1727),
    [sym_else_keyword] = ACTIONS(1727),
    [sym_switch_keyword] = ACTIONS(1727),
    [sym_case_keyword] = ACTIONS(1727),
    [sym_default_keyword] = ACTIONS(1727),
    [sym_while_keyword] = ACTIONS(1727),
    [sym_for_keyword] = ACTIONS(1727),
    [sym_break_keyword] = ACTIONS(1727),
    [sym_continue_keyword] = ACTIONS(1727),
    [sym_return_keyword] = ACTIONS(1727),
    [sym_select_keyword] = ACTIONS(1727),
    [sym_active_keyword] = ACTIONS(1727),
    [sym_struct_keyword] = ACTIONS(1727),
    [sym_structure_keyword] = ACTIONS(1727),
    [aux_sym_on_keyword_token1] = ACTIONS(1727),
    [aux_sym_off_keyword_token1] = ACTIONS(1727),
    [sym_send_level_keyword] = ACTIONS(1727),
    [sym_send_string_keyword] = ACTIONS(1727),
    [sym_send_command_keyword] = ACTIONS(1727),
    [sym_clear_buffer_keyword] = ACTIONS(1727),
    [sym_create_buffer_keyword] = ACTIONS(1727),
    [sym_create_multi_buffer_keyword] = ACTIONS(1727),
    [sym_call_keyword] = ACTIONS(1727),
    [sym_system_call_keyword] = ACTIONS(1727),
    [sym_devchan_to_keyword] = ACTIONS(1727),
    [sym_devchan_min_to_keyword] = ACTIONS(1727),
    [sym_devchan_total_off_keyword] = ACTIONS(1727),
    [sym_devchan_pulse_keyword] = ACTIONS(1727),
    [sym_wait_keyword] = ACTIONS(1727),
    [sym_cancel_wait_keyword] = ACTIONS(1727),
    [sym_wait_until_keyword] = ACTIONS(1727),
    [sym_cancel_wait_until_keyword] = ACTIONS(1727),
    [sym_cancel_all_wait_keyword] = ACTIONS(1727),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1727),
    [sym_not] = ACTIONS(1727),
    [sym_bnot] = ACTIONS(1727),
    [sym___file__] = ACTIONS(1727),
    [sym___date__] = ACTIONS(1727),
    [sym___ldate__] = ACTIONS(1727),
    [sym___time__] = ACTIONS(1727),
    [sym___line__] = ACTIONS(1727),
    [sym___name__] = ACTIONS(1727),
    [sym___netlinx__] = ACTIONS(1727),
    [sym_netlinx_axi_version] = ACTIONS(1727),
    [sym_url_flg_tcp] = ACTIONS(1727),
    [sym_url_flg_accinfopresent] = ACTIONS(1727),
    [sym_url_flg_temp] = ACTIONS(1727),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1727),
    [sym_url_flg_stat_mask] = ACTIONS(1727),
    [sym_url_flg_stat_lookup] = ACTIONS(1727),
    [sym_url_flg_stat_connecting] = ACTIONS(1727),
    [sym_url_flg_stat_waiting] = ACTIONS(1727),
    [sym_url_flg_stat_connected] = ACTIONS(1727),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1727),
    [sym_first_virtual_device] = ACTIONS(1727),
    [sym_first_local_port] = ACTIONS(1727),
    [sym_dynamic_virtual_device] = ACTIONS(1727),
    [sym_timeline_once] = ACTIONS(1727),
    [sym_timeline_repeat] = ACTIONS(1727),
    [sym_timeline_absolute] = ACTIONS(1727),
    [sym_timeline_relative] = ACTIONS(1727),
    [sym_do_push_timed_infinite] = ACTIONS(1727),
    [sym_source_type_no_address] = ACTIONS(1727),
    [sym_source_type_neuron_id] = ACTIONS(1727),
    [sym_source_type_ip_address] = ACTIONS(1727),
    [sym_source_type_axlink] = ACTIONS(1727),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1727),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1727),
    [sym_source_type_ip_socket_address] = ACTIONS(1727),
    [sym_source_type_rs232] = ACTIONS(1727),
    [sym_source_type_internal] = ACTIONS(1727),
    [sym_source_type_mac_address] = ACTIONS(1727),
    [sym_source_type_ipv4_port] = ACTIONS(1727),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1727),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1727),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1727),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1727),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1727),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1727),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1727),
    [sym_true] = ACTIONS(1727),
    [sym_false] = ACTIONS(1727),
    [sym_file_read_only] = ACTIONS(1727),
    [sym_file_rw_new] = ACTIONS(1727),
    [sym_file_rw_append] = ACTIONS(1727),
    [sym_ip_tcp] = ACTIONS(1727),
    [sym_ip_udp] = ACTIONS(1727),
    [sym_ip_udp_2way] = ACTIONS(1727),
    [sym_xml_encode_types] = ACTIONS(1727),
    [sym_xml_encode_char_as_list] = ACTIONS(1727),
    [sym_xml_encode_le] = ACTIONS(1727),
    [sym_xml_decode_types] = ACTIONS(1727),
    [sym_xml_decode_no_preserve] = ACTIONS(1727),
    [sym_duet_dev_type_display_device] = ACTIONS(1727),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1727),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1727),
    [sym_duet_dev_type_io_device] = ACTIONS(1727),
    [sym_duet_dev_type_relay_device] = ACTIONS(1727),
    [sym_duet_dev_type_ups] = ACTIONS(1727),
    [sym_duet_dev_type_amplifier] = ACTIONS(1727),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1727),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1727),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1727),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1727),
    [sym_duet_dev_type_camera] = ACTIONS(1727),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1727),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1727),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1727),
    [sym_duet_dev_type_dss] = ACTIONS(1727),
    [sym_duet_dev_type_dvr] = ACTIONS(1727),
    [sym_duet_dev_type_disc_device] = ACTIONS(1727),
    [sym_duet_dev_type_document_camera] = ACTIONS(1727),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1727),
    [sym_duet_dev_type_hvac] = ACTIONS(1727),
    [sym_duet_dev_type_keypad] = ACTIONS(1727),
    [sym_duet_dev_type_light] = ACTIONS(1727),
    [sym_duet_dev_type_monitor] = ACTIONS(1727),
    [sym_duet_dev_type_motor] = ACTIONS(1727),
    [sym_duet_dev_type_multi_window] = ACTIONS(1727),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1727),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1727),
    [sym_duet_dev_type_receiver] = ACTIONS(1727),
    [sym_duet_dev_type_security_system] = ACTIONS(1727),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1727),
    [sym_duet_dev_type_settop_box] = ACTIONS(1727),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1727),
    [sym_duet_dev_type_switcher] = ACTIONS(1727),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1727),
    [sym_duet_dev_type_tv] = ACTIONS(1727),
    [sym_duet_dev_type_utility] = ACTIONS(1727),
    [sym_duet_dev_type_vcr] = ACTIONS(1727),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1727),
    [sym_duet_dev_type_video_processor] = ACTIONS(1727),
    [sym_duet_dev_type_video_projector] = ACTIONS(1727),
    [sym_duet_dev_type_video_wall] = ACTIONS(1727),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1727),
    [sym_duet_dev_type_weather] = ACTIONS(1727),
    [sym_duet_dev_not_polled] = ACTIONS(1727),
    [sym_duet_dev_polled] = ACTIONS(1727),
    [sym_clkmgr_mode_network] = ACTIONS(1727),
    [sym_clkmgr_mode_standalone] = ACTIONS(1727),
    [sym_normal_standby] = ACTIONS(1727),
    [sym_normal_wake] = ACTIONS(1727),
    [sym_ip_multicast_ttl_option] = ACTIONS(1727),
    [sym_ip_ttl_subnet] = ACTIONS(1727),
    [sym_ip_ttl_site] = ACTIONS(1727),
    [sym_ip_ttl_region] = ACTIONS(1727),
    [sym_ip_ttl_continent] = ACTIONS(1727),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1727),
    [sym_ip_nodelay_on] = ACTIONS(1727),
    [sym_ip_nodelay_off] = ACTIONS(1727),
    [sym_amx_error] = ACTIONS(1727),
    [sym_amx_warning] = ACTIONS(1727),
    [sym_amx_info] = ACTIONS(1727),
    [sym_amx_debug] = ACTIONS(1727),
    [sym_netlinx_login_success] = ACTIONS(1727),
    [sym_netlinx_login_fail] = ACTIONS(1727),
    [sym_netlinx_logout] = ACTIONS(1727),
    [sym_null_str] = ACTIONS(1727),
    [sym_smtp_address] = ACTIONS(1727),
    [sym_smtp_port_number] = ACTIONS(1727),
    [sym_smtp_username] = ACTIONS(1727),
    [sym_smtp_password] = ACTIONS(1727),
    [sym_smtp_require_tls] = ACTIONS(1727),
    [sym_smtp_from] = ACTIONS(1727),
    [sym_smtp_tls_true] = ACTIONS(1727),
    [sym_smtp_tls_false] = ACTIONS(1727),
    [sym_listview_on_row_select_event] = ACTIONS(1727),
    [sym_data_structure_unknown] = ACTIONS(1727),
    [sym_data_structure_datafeed] = ACTIONS(1727),
    [sym_data_structure_datarecordset] = ACTIONS(1727),
    [sym_data_structure_datarecord] = ACTIONS(1727),
    [sym_data_structure_datafield] = ACTIONS(1727),
    [sym_data_max_content_field_count] = ACTIONS(1727),
    [sym_data_max_metadata_field_count] = ACTIONS(1727),
    [sym_data_max_id_length] = ACTIONS(1727),
    [sym_data_max_name_length] = ACTIONS(1727),
    [sym_data_max_description_length] = ACTIONS(1727),
    [sym_data_max_type_length] = ACTIONS(1727),
    [sym_data_max_format_length] = ACTIONS(1727),
    [sym_data_max_source_length] = ACTIONS(1727),
    [sym_data_max_value_length] = ACTIONS(1727),
    [sym_data_max_label_length] = ACTIONS(1727),
    [sym_data_max_url_length] = ACTIONS(1727),
    [sym_max_last_login_info_length] = ACTIONS(1727),
    [sym_data_type_unknown] = ACTIONS(1727),
    [sym_data_type_string] = ACTIONS(1727),
    [sym_data_type_datetime] = ACTIONS(1727),
    [sym_data_type_date] = ACTIONS(1727),
    [sym_data_type_time] = ACTIONS(1727),
    [sym_data_type_image] = ACTIONS(1727),
    [sym_data_format_url] = ACTIONS(1727),
    [sym_data_format_phone] = ACTIONS(1727),
    [sym_data_format_email] = ACTIONS(1727),
    [sym_data_format_iso8601] = ACTIONS(1727),
    [sym_auditlog_priv] = ACTIONS(1727),
    [sym_configuration_priv] = ACTIONS(1727),
    [sym_deviceconfig_priv] = ACTIONS(1727),
    [sym_ftp_priv] = ACTIONS(1727),
    [sym_http_priv] = ACTIONS(1727),
    [sym_networkconfig_priv] = ACTIONS(1727),
    [sym_programport_priv] = ACTIONS(1727),
    [sym_remoteui_priv] = ACTIONS(1727),
    [sym_securitycontrol_priv] = ACTIONS(1727),
    [sym_softwaremanagement_priv] = ACTIONS(1727),
    [sym_terminal_priv] = ACTIONS(1727),
    [sym_tpadmin_priv] = ACTIONS(1727),
    [sym_user1_priv] = ACTIONS(1727),
    [sym_user2_priv] = ACTIONS(1727),
    [sym_user3_priv] = ACTIONS(1727),
    [sym_user4_priv] = ACTIONS(1727),
    [sym_usermanagement_priv] = ACTIONS(1727),
    [sym_valid_account] = ACTIONS(1727),
    [sym_err_name_invalid] = ACTIONS(1727),
    [sym_err_password_invalid] = ACTIONS(1727),
    [sym_err_invalid_account] = ACTIONS(1727),
    [sym_err_no_authorization] = ACTIONS(1727),
    [sym_err_invalid_parameter] = ACTIONS(1727),
    [sym_err_user_locked_out] = ACTIONS(1727),
    [sym_err_user_account_expired] = ACTIONS(1727),
    [sym_err_general_error] = ACTIONS(1727),
    [sym_err_authentication_error] = ACTIONS(1727),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1727),
    [sym_tls_validate_certificate] = ACTIONS(1727),
    [sym_tdata] = ACTIONS(1727),
    [sym_tchannel] = ACTIONS(1727),
    [sym_tlevel] = ACTIONS(1727),
    [sym_tbutton] = ACTIONS(1727),
    [sym_ttimeline] = ACTIONS(1727),
    [sym_tcustom] = ACTIONS(1727),
    [sym_url_struct] = ACTIONS(1727),
    [sym_dns_struct] = ACTIONS(1727),
    [sym_ip_address_struct] = ACTIONS(1727),
    [sym_dev_info_struct] = ACTIONS(1727),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1727),
    [sym_data_feed] = ACTIONS(1727),
    [sym_data_field] = ACTIONS(1727),
    [sym_data_record] = ACTIONS(1727),
    [sym_wc_data_feed] = ACTIONS(1727),
    [sym_wc_data_field] = ACTIONS(1727),
    [sym_wc_data_record] = ACTIONS(1727),
    [sym_last_login_info] = ACTIONS(1727),
    [sym____reserved___] = ACTIONS(1727),
    [sym_date] = ACTIONS(1727),
    [sym_day] = ACTIONS(1727),
    [sym_time] = ACTIONS(1727),
    [sym_ldate] = ACTIONS(1727),
    [sym_push_device] = ACTIONS(1727),
    [sym_push_channel] = ACTIONS(1727),
    [sym_push_devchan] = ACTIONS(1727),
    [sym_release_device] = ACTIONS(1727),
    [sym_release_channel] = ACTIONS(1727),
    [sym_release_devchan] = ACTIONS(1727),
    [sym_master_slot] = ACTIONS(1727),
    [sym_get_pulse_time] = ACTIONS(1727),
    [sym_get_timer] = ACTIONS(1727),
    [sym_master_sn] = ACTIONS(1727),
    [sym_system_number] = ACTIONS(1727),
    [sym_button] = ACTIONS(1727),
    [sym_dv_channel] = ACTIONS(1727),
    [sym_data] = ACTIONS(1727),
    [sym_level] = ACTIONS(1727),
    [sym_timeline] = ACTIONS(1727),
    [sym_channel] = ACTIONS(1727),
    [sym_custom] = ACTIONS(1727),
    [sym_atoi] = ACTIONS(1727),
    [sym_atol] = ACTIONS(1727),
    [sym_atof] = ACTIONS(1727),
    [sym_itoa] = ACTIONS(1727),
    [sym_itohex] = ACTIONS(1727),
    [sym_hextoi] = ACTIONS(1727),
    [sym_ftoa] = ACTIONS(1727),
    [sym_find_string] = ACTIONS(1727),
    [sym_left_string] = ACTIONS(1727),
    [sym_right_string] = ACTIONS(1727),
    [sym_mid_string] = ACTIONS(1727),
    [sym_remove_string] = ACTIONS(1727),
    [sym_get_buffer_string] = ACTIONS(1727),
    [sym_get_buffer_char] = ACTIONS(1727),
    [sym_get_multi_buffer_string] = ACTIONS(1727),
    [sym_length_string] = ACTIONS(1727),
    [sym_max_length_string] = ACTIONS(1727),
    [sym_set_length_string] = ACTIONS(1727),
    [sym_compare_string] = ACTIONS(1727),
    [sym_upper_string] = ACTIONS(1727),
    [sym_lower_string] = ACTIONS(1727),
    [sym_get_last] = ACTIONS(1727),
    [sym_date_to_day] = ACTIONS(1727),
    [sym_date_to_month] = ACTIONS(1727),
    [sym_date_to_year] = ACTIONS(1727),
    [sym_time_to_hour] = ACTIONS(1727),
    [sym_time_to_minute] = ACTIONS(1727),
    [sym_time_to_second] = ACTIONS(1727),
    [sym_day_of_week] = ACTIONS(1727),
    [sym_set_pulse_time] = ACTIONS(1727),
    [sym_set_timer] = ACTIONS(1727),
    [sym_file_open] = ACTIONS(1727),
    [sym_file_close] = ACTIONS(1727),
    [sym_file_read] = ACTIONS(1727),
    [sym_file_read_line] = ACTIONS(1727),
    [sym_file_write] = ACTIONS(1727),
    [sym_file_write_line] = ACTIONS(1727),
    [sym_file_copy] = ACTIONS(1727),
    [sym_file_delete] = ACTIONS(1727),
    [sym_file_rename] = ACTIONS(1727),
    [sym_file_dir] = ACTIONS(1727),
    [sym_file_getdir] = ACTIONS(1727),
    [sym_file_setdir] = ACTIONS(1727),
    [sym_file_create_dir] = ACTIONS(1727),
    [sym_file_remove_dir] = ACTIONS(1727),
    [sym_file_seek] = ACTIONS(1727),
    [sym_length_array] = ACTIONS(1727),
    [sym_max_length_array] = ACTIONS(1727),
    [sym_set_length_array] = ACTIONS(1727),
    [sym_ip_client_open] = ACTIONS(1727),
    [sym_ip_client_close] = ACTIONS(1727),
    [sym_ip_server_open] = ACTIONS(1727),
    [sym_ip_server_close] = ACTIONS(1727),
    [sym_ip_bound_client_open] = ACTIONS(1727),
    [sym_ip_mc_server_open] = ACTIONS(1727),
    [sym_ip_set_option] = ACTIONS(1727),
    [sym_ssh_client_open] = ACTIONS(1727),
    [sym_ssh_client_close] = ACTIONS(1727),
    [sym_tls_client_open] = ACTIONS(1727),
    [sym_tls_client_close] = ACTIONS(1727),
    [sym_do_push] = ACTIONS(1727),
    [sym_do_push_timed] = ACTIONS(1727),
    [sym_do_release] = ACTIONS(1727),
    [sym_device_id] = ACTIONS(1727),
    [sym_device_info] = ACTIONS(1727),
    [sym_device_id_string] = ACTIONS(1727),
    [sym_device_standby] = ACTIONS(1727),
    [sym_device_wake] = ACTIONS(1727),
    [sym_timeline_create] = ACTIONS(1727),
    [sym_timeline_kill] = ACTIONS(1727),
    [sym_timeline_reload] = ACTIONS(1727),
    [sym_timeline_pause] = ACTIONS(1727),
    [sym_timeline_restart] = ACTIONS(1727),
    [sym_timeline_set] = ACTIONS(1727),
    [sym_timeline_get] = ACTIONS(1727),
    [sym_timeline_active] = ACTIONS(1727),
    [sym_timeline_dynamic_id] = ACTIONS(1727),
    [sym_variable_to_string] = ACTIONS(1727),
    [sym_string_to_variable] = ACTIONS(1727),
    [sym_length_variable_to_string] = ACTIONS(1727),
    [sym_type_cast] = ACTIONS(1727),
    [sym_variable_to_xml] = ACTIONS(1727),
    [sym_xml_to_variable] = ACTIONS(1727),
    [sym_length_variable_to_xml] = ACTIONS(1727),
    [sym_redirect_string] = ACTIONS(1727),
    [sym_random_number] = ACTIONS(1727),
    [sym_get_unique_id] = ACTIONS(1727),
    [sym_get_system_number] = ACTIONS(1727),
    [sym_set_system_number] = ACTIONS(1727),
    [sym_get_serial_number] = ACTIONS(1727),
    [sym_get_master_build] = ACTIONS(1727),
    [sym_set_validation_code] = ACTIONS(1727),
    [sym_reboot] = ACTIONS(1727),
    [sym_load_duet_module] = ACTIONS(1727),
    [sym_unload_duet_module] = ACTIONS(1727),
    [sym_duet_mem_size_set] = ACTIONS(1727),
    [sym_duet_mem_size_get] = ACTIONS(1727),
    [sym_dynamic_application_device] = ACTIONS(1727),
    [sym_dynamic_polled_port] = ACTIONS(1727),
    [sym_static_port_binding] = ACTIONS(1727),
    [sym_static_ip_binding] = ACTIONS(1727),
    [sym_get_dns_list] = ACTIONS(1727),
    [sym_set_dns_list] = ACTIONS(1727),
    [sym_get_ip_address] = ACTIONS(1727),
    [sym_set_ip_address] = ACTIONS(1727),
    [sym_get_url_list] = ACTIONS(1727),
    [sym_add_url_entry] = ACTIONS(1727),
    [sym_delete_url_entry] = ACTIONS(1727),
    [sym_abs_value] = ACTIONS(1727),
    [sym_max_value] = ACTIONS(1727),
    [sym_min_value] = ACTIONS(1727),
    [sym_exp_value] = ACTIONS(1727),
    [sym_log_value] = ACTIONS(1727),
    [sym_log10_value] = ACTIONS(1727),
    [sym_power_value] = ACTIONS(1727),
    [sym_sqrt_value] = ACTIONS(1727),
    [sym_raw_be] = ACTIONS(1727),
    [sym_raw_le] = ACTIONS(1727),
    [sym_internal_queue_size_set] = ACTIONS(1727),
    [sym_internal_queue_size_get] = ACTIONS(1727),
    [sym_internal_threshold_set] = ACTIONS(1727),
    [sym_internal_threshold_get] = ACTIONS(1727),
    [sym_rebuilt_event] = ACTIONS(1727),
    [sym_do_custom_event] = ACTIONS(1727),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1727),
    [sym_clkmgr_set_clk_source] = ACTIONS(1727),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1727),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1727),
    [sym_clkmgr_get_timezone] = ACTIONS(1727),
    [sym_clkmgr_set_timezone] = ACTIONS(1727),
    [sym_clkmgr_get_resync_period] = ACTIONS(1727),
    [sym_clkmgr_set_resync_period] = ACTIONS(1727),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1727),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1727),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1727),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1727),
    [sym_clkmgr_get_timeservers] = ACTIONS(1727),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1727),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1727),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1727),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1727),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1727),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1727),
    [sym_set_log_level] = ACTIONS(1727),
    [sym_get_log_level] = ACTIONS(1727),
    [sym_amx_log] = ACTIONS(1727),
    [sym_smtp_server_config_set] = ACTIONS(1727),
    [sym_smtp_server_config_get] = ACTIONS(1727),
    [sym_smtp_send] = ACTIONS(1727),
    [sym_validate_netlinx_account] = ACTIONS(1727),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1727),
    [sym_authenticate_certificate] = ACTIONS(1727),
    [sym_audit_netlinx_session_event] = ACTIONS(1727),
    [sym_audit_netlinx_generic_event] = ACTIONS(1727),
    [sym_get_available_flash_disk_space] = ACTIONS(1727),
    [sym_get_max_flash_disk_space] = ACTIONS(1727),
    [sym_data_create_feed] = ACTIONS(1727),
    [sym_data_delete_feed] = ACTIONS(1727),
    [sym_data_publish_feed] = ACTIONS(1727),
    [sym_data_get_published_feed] = ACTIONS(1727),
    [sym_data_add_record] = ACTIONS(1727),
    [sym_data_get_event_record] = ACTIONS(1727),
    [sym__wc_data_create_feed] = ACTIONS(1727),
    [sym__wc_data_add_record] = ACTIONS(1727),
    [sym__wc_data_get_event_record] = ACTIONS(1727),
    [sym_format] = ACTIONS(1727),
    [sym_astro_clock] = ACTIONS(1727),
    [sym_set_outdoor_temperature] = ACTIONS(1727),
    [sym_set_virtual_level_count] = ACTIONS(1727),
    [sym_set_virtual_channel_count] = ACTIONS(1727),
    [sym_set_virtual_port_count] = ACTIONS(1727),
    [sym__wc_encode_int] = ACTIONS(1727),
    [sym__wc_decode_int] = ACTIONS(1727),
    [sym__wc_int] = ACTIONS(1727),
    [sym__wc_to_ch_int] = ACTIONS(1727),
    [sym__ch_to_wc_int] = ACTIONS(1727),
    [sym__wc_find_string_int] = ACTIONS(1727),
    [sym__wc_left_string_int] = ACTIONS(1727),
    [sym__wc_lower_string_int] = ACTIONS(1727),
    [sym__wc_mid_string_int] = ACTIONS(1727),
    [sym__wc_remove_string_int] = ACTIONS(1727),
    [sym__wc_right_string_int] = ACTIONS(1727),
    [sym__wc_upper_string_int] = ACTIONS(1727),
    [sym__wc_compare_string_int] = ACTIONS(1727),
    [sym__wc_get_buffer_char_int] = ACTIONS(1727),
    [sym__wc_get_buffer_string_int] = ACTIONS(1727),
    [sym__wc_concat_string_int] = ACTIONS(1727),
    [sym__wc_file_open_int] = ACTIONS(1727),
    [sym__wc_file_close_int] = ACTIONS(1727),
    [sym__wc_file_read_int] = ACTIONS(1727),
    [sym__wc_file_read_line_int] = ACTIONS(1727),
    [sym__wc_file_write_int] = ACTIONS(1727),
    [sym__wc_file_write_line_int] = ACTIONS(1727),
    [aux_sym_preproc_include_token1] = ACTIONS(1727),
    [aux_sym_preproc_define_token1] = ACTIONS(1729),
    [aux_sym_preproc_warn_token1] = ACTIONS(1729),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1729),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1729),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1729),
    [anon_sym_LBRACK] = ACTIONS(1729),
    [anon_sym_LBRACE] = ACTIONS(1729),
    [anon_sym_RBRACE] = ACTIONS(1729),
    [anon_sym_LPAREN] = ACTIONS(1727),
    [anon_sym_BANG] = ACTIONS(1729),
    [anon_sym_TILDE] = ACTIONS(1729),
    [anon_sym_DASH] = ACTIONS(1727),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_DASH_DASH] = ACTIONS(1729),
    [anon_sym_PLUS_PLUS] = ACTIONS(1729),
    [anon_sym_DQUOTE] = ACTIONS(1729),
    [anon_sym_SQUOTE] = ACTIONS(1729),
    [sym_number_literal] = ACTIONS(1729),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(521)] = {
    [sym_identifier] = ACTIONS(1913),
    [sym_define_call_keyword] = ACTIONS(1913),
    [sym_define_function_keyword] = ACTIONS(1913),
    [sym_define_library_function_keyword] = ACTIONS(1913),
    [sym_define_module_keyword] = ACTIONS(1913),
    [sym_char_keyword] = ACTIONS(1913),
    [sym_widechar_keyword] = ACTIONS(1913),
    [sym_integer_keyword] = ACTIONS(1913),
    [sym_sinteger_keyword] = ACTIONS(1913),
    [sym_long_keyword] = ACTIONS(1913),
    [sym_slong_keyword] = ACTIONS(1913),
    [sym_float_keyword] = ACTIONS(1913),
    [sym_double_keyword] = ACTIONS(1913),
    [sym_variant_keyword] = ACTIONS(1913),
    [sym_variantarray_keyword] = ACTIONS(1913),
    [sym_local_var_keyword] = ACTIONS(1913),
    [sym_stack_var_keyword] = ACTIONS(1913),
    [sym_constant_keyword] = ACTIONS(1913),
    [sym_volatile_keyword] = ACTIONS(1913),
    [sym_non_volatile_keyword] = ACTIONS(1913),
    [sym_persistent_keyword] = ACTIONS(1913),
    [sym_dev_keyword] = ACTIONS(1913),
    [sym_devlev_keyword] = ACTIONS(1913),
    [sym_devchan_keyword] = ACTIONS(1913),
    [sym_if_keyword] = ACTIONS(1913),
    [sym_switch_keyword] = ACTIONS(1913),
    [sym_case_keyword] = ACTIONS(1913),
    [sym_default_keyword] = ACTIONS(1913),
    [sym_while_keyword] = ACTIONS(1913),
    [sym_for_keyword] = ACTIONS(1913),
    [sym_break_keyword] = ACTIONS(1913),
    [sym_continue_keyword] = ACTIONS(1913),
    [sym_return_keyword] = ACTIONS(1913),
    [sym_select_keyword] = ACTIONS(1913),
    [sym_active_keyword] = ACTIONS(1913),
    [sym_struct_keyword] = ACTIONS(1913),
    [sym_structure_keyword] = ACTIONS(1913),
    [aux_sym_on_keyword_token1] = ACTIONS(1913),
    [aux_sym_off_keyword_token1] = ACTIONS(1913),
    [sym_send_level_keyword] = ACTIONS(1913),
    [sym_send_string_keyword] = ACTIONS(1913),
    [sym_send_command_keyword] = ACTIONS(1913),
    [sym_clear_buffer_keyword] = ACTIONS(1913),
    [sym_create_buffer_keyword] = ACTIONS(1913),
    [sym_create_multi_buffer_keyword] = ACTIONS(1913),
    [sym_call_keyword] = ACTIONS(1913),
    [sym_system_call_keyword] = ACTIONS(1913),
    [sym_devchan_to_keyword] = ACTIONS(1913),
    [sym_devchan_min_to_keyword] = ACTIONS(1913),
    [sym_devchan_total_off_keyword] = ACTIONS(1913),
    [sym_devchan_pulse_keyword] = ACTIONS(1913),
    [sym_wait_keyword] = ACTIONS(1913),
    [sym_cancel_wait_keyword] = ACTIONS(1913),
    [sym_wait_until_keyword] = ACTIONS(1913),
    [sym_cancel_wait_until_keyword] = ACTIONS(1913),
    [sym_cancel_all_wait_keyword] = ACTIONS(1913),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1913),
    [sym_not] = ACTIONS(1913),
    [sym_bnot] = ACTIONS(1913),
    [sym___file__] = ACTIONS(1913),
    [sym___date__] = ACTIONS(1913),
    [sym___ldate__] = ACTIONS(1913),
    [sym___time__] = ACTIONS(1913),
    [sym___line__] = ACTIONS(1913),
    [sym___name__] = ACTIONS(1913),
    [sym___netlinx__] = ACTIONS(1913),
    [sym_netlinx_axi_version] = ACTIONS(1913),
    [sym_url_flg_tcp] = ACTIONS(1913),
    [sym_url_flg_accinfopresent] = ACTIONS(1913),
    [sym_url_flg_temp] = ACTIONS(1913),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1913),
    [sym_url_flg_stat_mask] = ACTIONS(1913),
    [sym_url_flg_stat_lookup] = ACTIONS(1913),
    [sym_url_flg_stat_connecting] = ACTIONS(1913),
    [sym_url_flg_stat_waiting] = ACTIONS(1913),
    [sym_url_flg_stat_connected] = ACTIONS(1913),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1913),
    [sym_first_virtual_device] = ACTIONS(1913),
    [sym_first_local_port] = ACTIONS(1913),
    [sym_dynamic_virtual_device] = ACTIONS(1913),
    [sym_timeline_once] = ACTIONS(1913),
    [sym_timeline_repeat] = ACTIONS(1913),
    [sym_timeline_absolute] = ACTIONS(1913),
    [sym_timeline_relative] = ACTIONS(1913),
    [sym_do_push_timed_infinite] = ACTIONS(1913),
    [sym_source_type_no_address] = ACTIONS(1913),
    [sym_source_type_neuron_id] = ACTIONS(1913),
    [sym_source_type_ip_address] = ACTIONS(1913),
    [sym_source_type_axlink] = ACTIONS(1913),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1913),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1913),
    [sym_source_type_ip_socket_address] = ACTIONS(1913),
    [sym_source_type_rs232] = ACTIONS(1913),
    [sym_source_type_internal] = ACTIONS(1913),
    [sym_source_type_mac_address] = ACTIONS(1913),
    [sym_source_type_ipv4_port] = ACTIONS(1913),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1913),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1913),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1913),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1913),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1913),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1913),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1913),
    [sym_true] = ACTIONS(1913),
    [sym_false] = ACTIONS(1913),
    [sym_file_read_only] = ACTIONS(1913),
    [sym_file_rw_new] = ACTIONS(1913),
    [sym_file_rw_append] = ACTIONS(1913),
    [sym_ip_tcp] = ACTIONS(1913),
    [sym_ip_udp] = ACTIONS(1913),
    [sym_ip_udp_2way] = ACTIONS(1913),
    [sym_xml_encode_types] = ACTIONS(1913),
    [sym_xml_encode_char_as_list] = ACTIONS(1913),
    [sym_xml_encode_le] = ACTIONS(1913),
    [sym_xml_decode_types] = ACTIONS(1913),
    [sym_xml_decode_no_preserve] = ACTIONS(1913),
    [sym_duet_dev_type_display_device] = ACTIONS(1913),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1913),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1913),
    [sym_duet_dev_type_io_device] = ACTIONS(1913),
    [sym_duet_dev_type_relay_device] = ACTIONS(1913),
    [sym_duet_dev_type_ups] = ACTIONS(1913),
    [sym_duet_dev_type_amplifier] = ACTIONS(1913),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1913),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1913),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1913),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1913),
    [sym_duet_dev_type_camera] = ACTIONS(1913),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1913),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1913),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1913),
    [sym_duet_dev_type_dss] = ACTIONS(1913),
    [sym_duet_dev_type_dvr] = ACTIONS(1913),
    [sym_duet_dev_type_disc_device] = ACTIONS(1913),
    [sym_duet_dev_type_document_camera] = ACTIONS(1913),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1913),
    [sym_duet_dev_type_hvac] = ACTIONS(1913),
    [sym_duet_dev_type_keypad] = ACTIONS(1913),
    [sym_duet_dev_type_light] = ACTIONS(1913),
    [sym_duet_dev_type_monitor] = ACTIONS(1913),
    [sym_duet_dev_type_motor] = ACTIONS(1913),
    [sym_duet_dev_type_multi_window] = ACTIONS(1913),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1913),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1913),
    [sym_duet_dev_type_receiver] = ACTIONS(1913),
    [sym_duet_dev_type_security_system] = ACTIONS(1913),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1913),
    [sym_duet_dev_type_settop_box] = ACTIONS(1913),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1913),
    [sym_duet_dev_type_switcher] = ACTIONS(1913),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1913),
    [sym_duet_dev_type_tv] = ACTIONS(1913),
    [sym_duet_dev_type_utility] = ACTIONS(1913),
    [sym_duet_dev_type_vcr] = ACTIONS(1913),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1913),
    [sym_duet_dev_type_video_processor] = ACTIONS(1913),
    [sym_duet_dev_type_video_projector] = ACTIONS(1913),
    [sym_duet_dev_type_video_wall] = ACTIONS(1913),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1913),
    [sym_duet_dev_type_weather] = ACTIONS(1913),
    [sym_duet_dev_not_polled] = ACTIONS(1913),
    [sym_duet_dev_polled] = ACTIONS(1913),
    [sym_clkmgr_mode_network] = ACTIONS(1913),
    [sym_clkmgr_mode_standalone] = ACTIONS(1913),
    [sym_normal_standby] = ACTIONS(1913),
    [sym_normal_wake] = ACTIONS(1913),
    [sym_ip_multicast_ttl_option] = ACTIONS(1913),
    [sym_ip_ttl_subnet] = ACTIONS(1913),
    [sym_ip_ttl_site] = ACTIONS(1913),
    [sym_ip_ttl_region] = ACTIONS(1913),
    [sym_ip_ttl_continent] = ACTIONS(1913),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1913),
    [sym_ip_nodelay_on] = ACTIONS(1913),
    [sym_ip_nodelay_off] = ACTIONS(1913),
    [sym_amx_error] = ACTIONS(1913),
    [sym_amx_warning] = ACTIONS(1913),
    [sym_amx_info] = ACTIONS(1913),
    [sym_amx_debug] = ACTIONS(1913),
    [sym_netlinx_login_success] = ACTIONS(1913),
    [sym_netlinx_login_fail] = ACTIONS(1913),
    [sym_netlinx_logout] = ACTIONS(1913),
    [sym_null_str] = ACTIONS(1913),
    [sym_smtp_address] = ACTIONS(1913),
    [sym_smtp_port_number] = ACTIONS(1913),
    [sym_smtp_username] = ACTIONS(1913),
    [sym_smtp_password] = ACTIONS(1913),
    [sym_smtp_require_tls] = ACTIONS(1913),
    [sym_smtp_from] = ACTIONS(1913),
    [sym_smtp_tls_true] = ACTIONS(1913),
    [sym_smtp_tls_false] = ACTIONS(1913),
    [sym_listview_on_row_select_event] = ACTIONS(1913),
    [sym_data_structure_unknown] = ACTIONS(1913),
    [sym_data_structure_datafeed] = ACTIONS(1913),
    [sym_data_structure_datarecordset] = ACTIONS(1913),
    [sym_data_structure_datarecord] = ACTIONS(1913),
    [sym_data_structure_datafield] = ACTIONS(1913),
    [sym_data_max_content_field_count] = ACTIONS(1913),
    [sym_data_max_metadata_field_count] = ACTIONS(1913),
    [sym_data_max_id_length] = ACTIONS(1913),
    [sym_data_max_name_length] = ACTIONS(1913),
    [sym_data_max_description_length] = ACTIONS(1913),
    [sym_data_max_type_length] = ACTIONS(1913),
    [sym_data_max_format_length] = ACTIONS(1913),
    [sym_data_max_source_length] = ACTIONS(1913),
    [sym_data_max_value_length] = ACTIONS(1913),
    [sym_data_max_label_length] = ACTIONS(1913),
    [sym_data_max_url_length] = ACTIONS(1913),
    [sym_max_last_login_info_length] = ACTIONS(1913),
    [sym_data_type_unknown] = ACTIONS(1913),
    [sym_data_type_string] = ACTIONS(1913),
    [sym_data_type_datetime] = ACTIONS(1913),
    [sym_data_type_date] = ACTIONS(1913),
    [sym_data_type_time] = ACTIONS(1913),
    [sym_data_type_image] = ACTIONS(1913),
    [sym_data_format_url] = ACTIONS(1913),
    [sym_data_format_phone] = ACTIONS(1913),
    [sym_data_format_email] = ACTIONS(1913),
    [sym_data_format_iso8601] = ACTIONS(1913),
    [sym_auditlog_priv] = ACTIONS(1913),
    [sym_configuration_priv] = ACTIONS(1913),
    [sym_deviceconfig_priv] = ACTIONS(1913),
    [sym_ftp_priv] = ACTIONS(1913),
    [sym_http_priv] = ACTIONS(1913),
    [sym_networkconfig_priv] = ACTIONS(1913),
    [sym_programport_priv] = ACTIONS(1913),
    [sym_remoteui_priv] = ACTIONS(1913),
    [sym_securitycontrol_priv] = ACTIONS(1913),
    [sym_softwaremanagement_priv] = ACTIONS(1913),
    [sym_terminal_priv] = ACTIONS(1913),
    [sym_tpadmin_priv] = ACTIONS(1913),
    [sym_user1_priv] = ACTIONS(1913),
    [sym_user2_priv] = ACTIONS(1913),
    [sym_user3_priv] = ACTIONS(1913),
    [sym_user4_priv] = ACTIONS(1913),
    [sym_usermanagement_priv] = ACTIONS(1913),
    [sym_valid_account] = ACTIONS(1913),
    [sym_err_name_invalid] = ACTIONS(1913),
    [sym_err_password_invalid] = ACTIONS(1913),
    [sym_err_invalid_account] = ACTIONS(1913),
    [sym_err_no_authorization] = ACTIONS(1913),
    [sym_err_invalid_parameter] = ACTIONS(1913),
    [sym_err_user_locked_out] = ACTIONS(1913),
    [sym_err_user_account_expired] = ACTIONS(1913),
    [sym_err_general_error] = ACTIONS(1913),
    [sym_err_authentication_error] = ACTIONS(1913),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1913),
    [sym_tls_validate_certificate] = ACTIONS(1913),
    [sym_tdata] = ACTIONS(1913),
    [sym_tchannel] = ACTIONS(1913),
    [sym_tlevel] = ACTIONS(1913),
    [sym_tbutton] = ACTIONS(1913),
    [sym_ttimeline] = ACTIONS(1913),
    [sym_tcustom] = ACTIONS(1913),
    [sym_url_struct] = ACTIONS(1913),
    [sym_dns_struct] = ACTIONS(1913),
    [sym_ip_address_struct] = ACTIONS(1913),
    [sym_dev_info_struct] = ACTIONS(1913),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1913),
    [sym_data_feed] = ACTIONS(1913),
    [sym_data_field] = ACTIONS(1913),
    [sym_data_record] = ACTIONS(1913),
    [sym_wc_data_feed] = ACTIONS(1913),
    [sym_wc_data_field] = ACTIONS(1913),
    [sym_wc_data_record] = ACTIONS(1913),
    [sym_last_login_info] = ACTIONS(1913),
    [sym____reserved___] = ACTIONS(1913),
    [sym_date] = ACTIONS(1913),
    [sym_day] = ACTIONS(1913),
    [sym_time] = ACTIONS(1913),
    [sym_ldate] = ACTIONS(1913),
    [sym_push_device] = ACTIONS(1913),
    [sym_push_channel] = ACTIONS(1913),
    [sym_push_devchan] = ACTIONS(1913),
    [sym_release_device] = ACTIONS(1913),
    [sym_release_channel] = ACTIONS(1913),
    [sym_release_devchan] = ACTIONS(1913),
    [sym_master_slot] = ACTIONS(1913),
    [sym_get_pulse_time] = ACTIONS(1913),
    [sym_get_timer] = ACTIONS(1913),
    [sym_master_sn] = ACTIONS(1913),
    [sym_system_number] = ACTIONS(1913),
    [sym_button] = ACTIONS(1913),
    [sym_dv_channel] = ACTIONS(1913),
    [sym_data] = ACTIONS(1913),
    [sym_level] = ACTIONS(1913),
    [sym_timeline] = ACTIONS(1913),
    [sym_channel] = ACTIONS(1913),
    [sym_custom] = ACTIONS(1913),
    [sym_atoi] = ACTIONS(1913),
    [sym_atol] = ACTIONS(1913),
    [sym_atof] = ACTIONS(1913),
    [sym_itoa] = ACTIONS(1913),
    [sym_itohex] = ACTIONS(1913),
    [sym_hextoi] = ACTIONS(1913),
    [sym_ftoa] = ACTIONS(1913),
    [sym_find_string] = ACTIONS(1913),
    [sym_left_string] = ACTIONS(1913),
    [sym_right_string] = ACTIONS(1913),
    [sym_mid_string] = ACTIONS(1913),
    [sym_remove_string] = ACTIONS(1913),
    [sym_get_buffer_string] = ACTIONS(1913),
    [sym_get_buffer_char] = ACTIONS(1913),
    [sym_get_multi_buffer_string] = ACTIONS(1913),
    [sym_length_string] = ACTIONS(1913),
    [sym_max_length_string] = ACTIONS(1913),
    [sym_set_length_string] = ACTIONS(1913),
    [sym_compare_string] = ACTIONS(1913),
    [sym_upper_string] = ACTIONS(1913),
    [sym_lower_string] = ACTIONS(1913),
    [sym_get_last] = ACTIONS(1913),
    [sym_date_to_day] = ACTIONS(1913),
    [sym_date_to_month] = ACTIONS(1913),
    [sym_date_to_year] = ACTIONS(1913),
    [sym_time_to_hour] = ACTIONS(1913),
    [sym_time_to_minute] = ACTIONS(1913),
    [sym_time_to_second] = ACTIONS(1913),
    [sym_day_of_week] = ACTIONS(1913),
    [sym_set_pulse_time] = ACTIONS(1913),
    [sym_set_timer] = ACTIONS(1913),
    [sym_file_open] = ACTIONS(1913),
    [sym_file_close] = ACTIONS(1913),
    [sym_file_read] = ACTIONS(1913),
    [sym_file_read_line] = ACTIONS(1913),
    [sym_file_write] = ACTIONS(1913),
    [sym_file_write_line] = ACTIONS(1913),
    [sym_file_copy] = ACTIONS(1913),
    [sym_file_delete] = ACTIONS(1913),
    [sym_file_rename] = ACTIONS(1913),
    [sym_file_dir] = ACTIONS(1913),
    [sym_file_getdir] = ACTIONS(1913),
    [sym_file_setdir] = ACTIONS(1913),
    [sym_file_create_dir] = ACTIONS(1913),
    [sym_file_remove_dir] = ACTIONS(1913),
    [sym_file_seek] = ACTIONS(1913),
    [sym_length_array] = ACTIONS(1913),
    [sym_max_length_array] = ACTIONS(1913),
    [sym_set_length_array] = ACTIONS(1913),
    [sym_ip_client_open] = ACTIONS(1913),
    [sym_ip_client_close] = ACTIONS(1913),
    [sym_ip_server_open] = ACTIONS(1913),
    [sym_ip_server_close] = ACTIONS(1913),
    [sym_ip_bound_client_open] = ACTIONS(1913),
    [sym_ip_mc_server_open] = ACTIONS(1913),
    [sym_ip_set_option] = ACTIONS(1913),
    [sym_ssh_client_open] = ACTIONS(1913),
    [sym_ssh_client_close] = ACTIONS(1913),
    [sym_tls_client_open] = ACTIONS(1913),
    [sym_tls_client_close] = ACTIONS(1913),
    [sym_do_push] = ACTIONS(1913),
    [sym_do_push_timed] = ACTIONS(1913),
    [sym_do_release] = ACTIONS(1913),
    [sym_device_id] = ACTIONS(1913),
    [sym_device_info] = ACTIONS(1913),
    [sym_device_id_string] = ACTIONS(1913),
    [sym_device_standby] = ACTIONS(1913),
    [sym_device_wake] = ACTIONS(1913),
    [sym_timeline_create] = ACTIONS(1913),
    [sym_timeline_kill] = ACTIONS(1913),
    [sym_timeline_reload] = ACTIONS(1913),
    [sym_timeline_pause] = ACTIONS(1913),
    [sym_timeline_restart] = ACTIONS(1913),
    [sym_timeline_set] = ACTIONS(1913),
    [sym_timeline_get] = ACTIONS(1913),
    [sym_timeline_active] = ACTIONS(1913),
    [sym_timeline_dynamic_id] = ACTIONS(1913),
    [sym_variable_to_string] = ACTIONS(1913),
    [sym_string_to_variable] = ACTIONS(1913),
    [sym_length_variable_to_string] = ACTIONS(1913),
    [sym_type_cast] = ACTIONS(1913),
    [sym_variable_to_xml] = ACTIONS(1913),
    [sym_xml_to_variable] = ACTIONS(1913),
    [sym_length_variable_to_xml] = ACTIONS(1913),
    [sym_redirect_string] = ACTIONS(1913),
    [sym_random_number] = ACTIONS(1913),
    [sym_get_unique_id] = ACTIONS(1913),
    [sym_get_system_number] = ACTIONS(1913),
    [sym_set_system_number] = ACTIONS(1913),
    [sym_get_serial_number] = ACTIONS(1913),
    [sym_get_master_build] = ACTIONS(1913),
    [sym_set_validation_code] = ACTIONS(1913),
    [sym_reboot] = ACTIONS(1913),
    [sym_load_duet_module] = ACTIONS(1913),
    [sym_unload_duet_module] = ACTIONS(1913),
    [sym_duet_mem_size_set] = ACTIONS(1913),
    [sym_duet_mem_size_get] = ACTIONS(1913),
    [sym_dynamic_application_device] = ACTIONS(1913),
    [sym_dynamic_polled_port] = ACTIONS(1913),
    [sym_static_port_binding] = ACTIONS(1913),
    [sym_static_ip_binding] = ACTIONS(1913),
    [sym_get_dns_list] = ACTIONS(1913),
    [sym_set_dns_list] = ACTIONS(1913),
    [sym_get_ip_address] = ACTIONS(1913),
    [sym_set_ip_address] = ACTIONS(1913),
    [sym_get_url_list] = ACTIONS(1913),
    [sym_add_url_entry] = ACTIONS(1913),
    [sym_delete_url_entry] = ACTIONS(1913),
    [sym_abs_value] = ACTIONS(1913),
    [sym_max_value] = ACTIONS(1913),
    [sym_min_value] = ACTIONS(1913),
    [sym_exp_value] = ACTIONS(1913),
    [sym_log_value] = ACTIONS(1913),
    [sym_log10_value] = ACTIONS(1913),
    [sym_power_value] = ACTIONS(1913),
    [sym_sqrt_value] = ACTIONS(1913),
    [sym_raw_be] = ACTIONS(1913),
    [sym_raw_le] = ACTIONS(1913),
    [sym_internal_queue_size_set] = ACTIONS(1913),
    [sym_internal_queue_size_get] = ACTIONS(1913),
    [sym_internal_threshold_set] = ACTIONS(1913),
    [sym_internal_threshold_get] = ACTIONS(1913),
    [sym_rebuilt_event] = ACTIONS(1913),
    [sym_do_custom_event] = ACTIONS(1913),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1913),
    [sym_clkmgr_set_clk_source] = ACTIONS(1913),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1913),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1913),
    [sym_clkmgr_get_timezone] = ACTIONS(1913),
    [sym_clkmgr_set_timezone] = ACTIONS(1913),
    [sym_clkmgr_get_resync_period] = ACTIONS(1913),
    [sym_clkmgr_set_resync_period] = ACTIONS(1913),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1913),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1913),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1913),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1913),
    [sym_clkmgr_get_timeservers] = ACTIONS(1913),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1913),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1913),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1913),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1913),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1913),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1913),
    [sym_set_log_level] = ACTIONS(1913),
    [sym_get_log_level] = ACTIONS(1913),
    [sym_amx_log] = ACTIONS(1913),
    [sym_smtp_server_config_set] = ACTIONS(1913),
    [sym_smtp_server_config_get] = ACTIONS(1913),
    [sym_smtp_send] = ACTIONS(1913),
    [sym_validate_netlinx_account] = ACTIONS(1913),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1913),
    [sym_authenticate_certificate] = ACTIONS(1913),
    [sym_audit_netlinx_session_event] = ACTIONS(1913),
    [sym_audit_netlinx_generic_event] = ACTIONS(1913),
    [sym_get_available_flash_disk_space] = ACTIONS(1913),
    [sym_get_max_flash_disk_space] = ACTIONS(1913),
    [sym_data_create_feed] = ACTIONS(1913),
    [sym_data_delete_feed] = ACTIONS(1913),
    [sym_data_publish_feed] = ACTIONS(1913),
    [sym_data_get_published_feed] = ACTIONS(1913),
    [sym_data_add_record] = ACTIONS(1913),
    [sym_data_get_event_record] = ACTIONS(1913),
    [sym__wc_data_create_feed] = ACTIONS(1913),
    [sym__wc_data_add_record] = ACTIONS(1913),
    [sym__wc_data_get_event_record] = ACTIONS(1913),
    [sym_format] = ACTIONS(1913),
    [sym_astro_clock] = ACTIONS(1913),
    [sym_set_outdoor_temperature] = ACTIONS(1913),
    [sym_set_virtual_level_count] = ACTIONS(1913),
    [sym_set_virtual_channel_count] = ACTIONS(1913),
    [sym_set_virtual_port_count] = ACTIONS(1913),
    [sym__wc_encode_int] = ACTIONS(1913),
    [sym__wc_decode_int] = ACTIONS(1913),
    [sym__wc_int] = ACTIONS(1913),
    [sym__wc_to_ch_int] = ACTIONS(1913),
    [sym__ch_to_wc_int] = ACTIONS(1913),
    [sym__wc_find_string_int] = ACTIONS(1913),
    [sym__wc_left_string_int] = ACTIONS(1913),
    [sym__wc_lower_string_int] = ACTIONS(1913),
    [sym__wc_mid_string_int] = ACTIONS(1913),
    [sym__wc_remove_string_int] = ACTIONS(1913),
    [sym__wc_right_string_int] = ACTIONS(1913),
    [sym__wc_upper_string_int] = ACTIONS(1913),
    [sym__wc_compare_string_int] = ACTIONS(1913),
    [sym__wc_get_buffer_char_int] = ACTIONS(1913),
    [sym__wc_get_buffer_string_int] = ACTIONS(1913),
    [sym__wc_concat_string_int] = ACTIONS(1913),
    [sym__wc_file_open_int] = ACTIONS(1913),
    [sym__wc_file_close_int] = ACTIONS(1913),
    [sym__wc_file_read_int] = ACTIONS(1913),
    [sym__wc_file_read_line_int] = ACTIONS(1913),
    [sym__wc_file_write_int] = ACTIONS(1913),
    [sym__wc_file_write_line_int] = ACTIONS(1913),
    [aux_sym_preproc_include_token1] = ACTIONS(1913),
    [aux_sym_preproc_define_token1] = ACTIONS(1915),
    [aux_sym_preproc_warn_token1] = ACTIONS(1915),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1915),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1915),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1915),
    [anon_sym_LBRACK] = ACTIONS(1915),
    [anon_sym_LBRACE] = ACTIONS(1915),
    [anon_sym_RBRACE] = ACTIONS(1915),
    [anon_sym_LPAREN] = ACTIONS(1913),
    [anon_sym_BANG] = ACTIONS(1915),
    [anon_sym_TILDE] = ACTIONS(1915),
    [anon_sym_DASH] = ACTIONS(1913),
    [anon_sym_PLUS] = ACTIONS(1913),
    [anon_sym_DASH_DASH] = ACTIONS(1915),
    [anon_sym_PLUS_PLUS] = ACTIONS(1915),
    [anon_sym_DQUOTE] = ACTIONS(1915),
    [anon_sym_SQUOTE] = ACTIONS(1915),
    [sym_number_literal] = ACTIONS(1915),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(522)] = {
    [sym_identifier] = ACTIONS(1985),
    [sym_define_call_keyword] = ACTIONS(1985),
    [sym_define_function_keyword] = ACTIONS(1985),
    [sym_define_library_function_keyword] = ACTIONS(1985),
    [sym_define_module_keyword] = ACTIONS(1985),
    [sym_char_keyword] = ACTIONS(1985),
    [sym_widechar_keyword] = ACTIONS(1985),
    [sym_integer_keyword] = ACTIONS(1985),
    [sym_sinteger_keyword] = ACTIONS(1985),
    [sym_long_keyword] = ACTIONS(1985),
    [sym_slong_keyword] = ACTIONS(1985),
    [sym_float_keyword] = ACTIONS(1985),
    [sym_double_keyword] = ACTIONS(1985),
    [sym_variant_keyword] = ACTIONS(1985),
    [sym_variantarray_keyword] = ACTIONS(1985),
    [sym_local_var_keyword] = ACTIONS(1985),
    [sym_stack_var_keyword] = ACTIONS(1985),
    [sym_constant_keyword] = ACTIONS(1985),
    [sym_volatile_keyword] = ACTIONS(1985),
    [sym_non_volatile_keyword] = ACTIONS(1985),
    [sym_persistent_keyword] = ACTIONS(1985),
    [sym_dev_keyword] = ACTIONS(1985),
    [sym_devlev_keyword] = ACTIONS(1985),
    [sym_devchan_keyword] = ACTIONS(1985),
    [sym_if_keyword] = ACTIONS(1985),
    [sym_switch_keyword] = ACTIONS(1985),
    [sym_case_keyword] = ACTIONS(1985),
    [sym_default_keyword] = ACTIONS(1985),
    [sym_while_keyword] = ACTIONS(1985),
    [sym_for_keyword] = ACTIONS(1985),
    [sym_break_keyword] = ACTIONS(1985),
    [sym_continue_keyword] = ACTIONS(1985),
    [sym_return_keyword] = ACTIONS(1985),
    [sym_select_keyword] = ACTIONS(1985),
    [sym_active_keyword] = ACTIONS(1985),
    [sym_struct_keyword] = ACTIONS(1985),
    [sym_structure_keyword] = ACTIONS(1985),
    [aux_sym_on_keyword_token1] = ACTIONS(1985),
    [aux_sym_off_keyword_token1] = ACTIONS(1985),
    [sym_send_level_keyword] = ACTIONS(1985),
    [sym_send_string_keyword] = ACTIONS(1985),
    [sym_send_command_keyword] = ACTIONS(1985),
    [sym_clear_buffer_keyword] = ACTIONS(1985),
    [sym_create_buffer_keyword] = ACTIONS(1985),
    [sym_create_multi_buffer_keyword] = ACTIONS(1985),
    [sym_call_keyword] = ACTIONS(1985),
    [sym_system_call_keyword] = ACTIONS(1985),
    [sym_devchan_to_keyword] = ACTIONS(1985),
    [sym_devchan_min_to_keyword] = ACTIONS(1985),
    [sym_devchan_total_off_keyword] = ACTIONS(1985),
    [sym_devchan_pulse_keyword] = ACTIONS(1985),
    [sym_wait_keyword] = ACTIONS(1985),
    [sym_cancel_wait_keyword] = ACTIONS(1985),
    [sym_wait_until_keyword] = ACTIONS(1985),
    [sym_cancel_wait_until_keyword] = ACTIONS(1985),
    [sym_cancel_all_wait_keyword] = ACTIONS(1985),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1985),
    [sym_not] = ACTIONS(1985),
    [sym_bnot] = ACTIONS(1985),
    [sym___file__] = ACTIONS(1985),
    [sym___date__] = ACTIONS(1985),
    [sym___ldate__] = ACTIONS(1985),
    [sym___time__] = ACTIONS(1985),
    [sym___line__] = ACTIONS(1985),
    [sym___name__] = ACTIONS(1985),
    [sym___netlinx__] = ACTIONS(1985),
    [sym_netlinx_axi_version] = ACTIONS(1985),
    [sym_url_flg_tcp] = ACTIONS(1985),
    [sym_url_flg_accinfopresent] = ACTIONS(1985),
    [sym_url_flg_temp] = ACTIONS(1985),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1985),
    [sym_url_flg_stat_mask] = ACTIONS(1985),
    [sym_url_flg_stat_lookup] = ACTIONS(1985),
    [sym_url_flg_stat_connecting] = ACTIONS(1985),
    [sym_url_flg_stat_waiting] = ACTIONS(1985),
    [sym_url_flg_stat_connected] = ACTIONS(1985),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1985),
    [sym_first_virtual_device] = ACTIONS(1985),
    [sym_first_local_port] = ACTIONS(1985),
    [sym_dynamic_virtual_device] = ACTIONS(1985),
    [sym_timeline_once] = ACTIONS(1985),
    [sym_timeline_repeat] = ACTIONS(1985),
    [sym_timeline_absolute] = ACTIONS(1985),
    [sym_timeline_relative] = ACTIONS(1985),
    [sym_do_push_timed_infinite] = ACTIONS(1985),
    [sym_source_type_no_address] = ACTIONS(1985),
    [sym_source_type_neuron_id] = ACTIONS(1985),
    [sym_source_type_ip_address] = ACTIONS(1985),
    [sym_source_type_axlink] = ACTIONS(1985),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1985),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1985),
    [sym_source_type_ip_socket_address] = ACTIONS(1985),
    [sym_source_type_rs232] = ACTIONS(1985),
    [sym_source_type_internal] = ACTIONS(1985),
    [sym_source_type_mac_address] = ACTIONS(1985),
    [sym_source_type_ipv4_port] = ACTIONS(1985),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1985),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1985),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1985),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1985),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1985),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1985),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1985),
    [sym_true] = ACTIONS(1985),
    [sym_false] = ACTIONS(1985),
    [sym_file_read_only] = ACTIONS(1985),
    [sym_file_rw_new] = ACTIONS(1985),
    [sym_file_rw_append] = ACTIONS(1985),
    [sym_ip_tcp] = ACTIONS(1985),
    [sym_ip_udp] = ACTIONS(1985),
    [sym_ip_udp_2way] = ACTIONS(1985),
    [sym_xml_encode_types] = ACTIONS(1985),
    [sym_xml_encode_char_as_list] = ACTIONS(1985),
    [sym_xml_encode_le] = ACTIONS(1985),
    [sym_xml_decode_types] = ACTIONS(1985),
    [sym_xml_decode_no_preserve] = ACTIONS(1985),
    [sym_duet_dev_type_display_device] = ACTIONS(1985),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1985),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1985),
    [sym_duet_dev_type_io_device] = ACTIONS(1985),
    [sym_duet_dev_type_relay_device] = ACTIONS(1985),
    [sym_duet_dev_type_ups] = ACTIONS(1985),
    [sym_duet_dev_type_amplifier] = ACTIONS(1985),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1985),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1985),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1985),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1985),
    [sym_duet_dev_type_camera] = ACTIONS(1985),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1985),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1985),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1985),
    [sym_duet_dev_type_dss] = ACTIONS(1985),
    [sym_duet_dev_type_dvr] = ACTIONS(1985),
    [sym_duet_dev_type_disc_device] = ACTIONS(1985),
    [sym_duet_dev_type_document_camera] = ACTIONS(1985),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1985),
    [sym_duet_dev_type_hvac] = ACTIONS(1985),
    [sym_duet_dev_type_keypad] = ACTIONS(1985),
    [sym_duet_dev_type_light] = ACTIONS(1985),
    [sym_duet_dev_type_monitor] = ACTIONS(1985),
    [sym_duet_dev_type_motor] = ACTIONS(1985),
    [sym_duet_dev_type_multi_window] = ACTIONS(1985),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1985),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1985),
    [sym_duet_dev_type_receiver] = ACTIONS(1985),
    [sym_duet_dev_type_security_system] = ACTIONS(1985),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1985),
    [sym_duet_dev_type_settop_box] = ACTIONS(1985),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1985),
    [sym_duet_dev_type_switcher] = ACTIONS(1985),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1985),
    [sym_duet_dev_type_tv] = ACTIONS(1985),
    [sym_duet_dev_type_utility] = ACTIONS(1985),
    [sym_duet_dev_type_vcr] = ACTIONS(1985),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1985),
    [sym_duet_dev_type_video_processor] = ACTIONS(1985),
    [sym_duet_dev_type_video_projector] = ACTIONS(1985),
    [sym_duet_dev_type_video_wall] = ACTIONS(1985),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1985),
    [sym_duet_dev_type_weather] = ACTIONS(1985),
    [sym_duet_dev_not_polled] = ACTIONS(1985),
    [sym_duet_dev_polled] = ACTIONS(1985),
    [sym_clkmgr_mode_network] = ACTIONS(1985),
    [sym_clkmgr_mode_standalone] = ACTIONS(1985),
    [sym_normal_standby] = ACTIONS(1985),
    [sym_normal_wake] = ACTIONS(1985),
    [sym_ip_multicast_ttl_option] = ACTIONS(1985),
    [sym_ip_ttl_subnet] = ACTIONS(1985),
    [sym_ip_ttl_site] = ACTIONS(1985),
    [sym_ip_ttl_region] = ACTIONS(1985),
    [sym_ip_ttl_continent] = ACTIONS(1985),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1985),
    [sym_ip_nodelay_on] = ACTIONS(1985),
    [sym_ip_nodelay_off] = ACTIONS(1985),
    [sym_amx_error] = ACTIONS(1985),
    [sym_amx_warning] = ACTIONS(1985),
    [sym_amx_info] = ACTIONS(1985),
    [sym_amx_debug] = ACTIONS(1985),
    [sym_netlinx_login_success] = ACTIONS(1985),
    [sym_netlinx_login_fail] = ACTIONS(1985),
    [sym_netlinx_logout] = ACTIONS(1985),
    [sym_null_str] = ACTIONS(1985),
    [sym_smtp_address] = ACTIONS(1985),
    [sym_smtp_port_number] = ACTIONS(1985),
    [sym_smtp_username] = ACTIONS(1985),
    [sym_smtp_password] = ACTIONS(1985),
    [sym_smtp_require_tls] = ACTIONS(1985),
    [sym_smtp_from] = ACTIONS(1985),
    [sym_smtp_tls_true] = ACTIONS(1985),
    [sym_smtp_tls_false] = ACTIONS(1985),
    [sym_listview_on_row_select_event] = ACTIONS(1985),
    [sym_data_structure_unknown] = ACTIONS(1985),
    [sym_data_structure_datafeed] = ACTIONS(1985),
    [sym_data_structure_datarecordset] = ACTIONS(1985),
    [sym_data_structure_datarecord] = ACTIONS(1985),
    [sym_data_structure_datafield] = ACTIONS(1985),
    [sym_data_max_content_field_count] = ACTIONS(1985),
    [sym_data_max_metadata_field_count] = ACTIONS(1985),
    [sym_data_max_id_length] = ACTIONS(1985),
    [sym_data_max_name_length] = ACTIONS(1985),
    [sym_data_max_description_length] = ACTIONS(1985),
    [sym_data_max_type_length] = ACTIONS(1985),
    [sym_data_max_format_length] = ACTIONS(1985),
    [sym_data_max_source_length] = ACTIONS(1985),
    [sym_data_max_value_length] = ACTIONS(1985),
    [sym_data_max_label_length] = ACTIONS(1985),
    [sym_data_max_url_length] = ACTIONS(1985),
    [sym_max_last_login_info_length] = ACTIONS(1985),
    [sym_data_type_unknown] = ACTIONS(1985),
    [sym_data_type_string] = ACTIONS(1985),
    [sym_data_type_datetime] = ACTIONS(1985),
    [sym_data_type_date] = ACTIONS(1985),
    [sym_data_type_time] = ACTIONS(1985),
    [sym_data_type_image] = ACTIONS(1985),
    [sym_data_format_url] = ACTIONS(1985),
    [sym_data_format_phone] = ACTIONS(1985),
    [sym_data_format_email] = ACTIONS(1985),
    [sym_data_format_iso8601] = ACTIONS(1985),
    [sym_auditlog_priv] = ACTIONS(1985),
    [sym_configuration_priv] = ACTIONS(1985),
    [sym_deviceconfig_priv] = ACTIONS(1985),
    [sym_ftp_priv] = ACTIONS(1985),
    [sym_http_priv] = ACTIONS(1985),
    [sym_networkconfig_priv] = ACTIONS(1985),
    [sym_programport_priv] = ACTIONS(1985),
    [sym_remoteui_priv] = ACTIONS(1985),
    [sym_securitycontrol_priv] = ACTIONS(1985),
    [sym_softwaremanagement_priv] = ACTIONS(1985),
    [sym_terminal_priv] = ACTIONS(1985),
    [sym_tpadmin_priv] = ACTIONS(1985),
    [sym_user1_priv] = ACTIONS(1985),
    [sym_user2_priv] = ACTIONS(1985),
    [sym_user3_priv] = ACTIONS(1985),
    [sym_user4_priv] = ACTIONS(1985),
    [sym_usermanagement_priv] = ACTIONS(1985),
    [sym_valid_account] = ACTIONS(1985),
    [sym_err_name_invalid] = ACTIONS(1985),
    [sym_err_password_invalid] = ACTIONS(1985),
    [sym_err_invalid_account] = ACTIONS(1985),
    [sym_err_no_authorization] = ACTIONS(1985),
    [sym_err_invalid_parameter] = ACTIONS(1985),
    [sym_err_user_locked_out] = ACTIONS(1985),
    [sym_err_user_account_expired] = ACTIONS(1985),
    [sym_err_general_error] = ACTIONS(1985),
    [sym_err_authentication_error] = ACTIONS(1985),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1985),
    [sym_tls_validate_certificate] = ACTIONS(1985),
    [sym_tdata] = ACTIONS(1985),
    [sym_tchannel] = ACTIONS(1985),
    [sym_tlevel] = ACTIONS(1985),
    [sym_tbutton] = ACTIONS(1985),
    [sym_ttimeline] = ACTIONS(1985),
    [sym_tcustom] = ACTIONS(1985),
    [sym_url_struct] = ACTIONS(1985),
    [sym_dns_struct] = ACTIONS(1985),
    [sym_ip_address_struct] = ACTIONS(1985),
    [sym_dev_info_struct] = ACTIONS(1985),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1985),
    [sym_data_feed] = ACTIONS(1985),
    [sym_data_field] = ACTIONS(1985),
    [sym_data_record] = ACTIONS(1985),
    [sym_wc_data_feed] = ACTIONS(1985),
    [sym_wc_data_field] = ACTIONS(1985),
    [sym_wc_data_record] = ACTIONS(1985),
    [sym_last_login_info] = ACTIONS(1985),
    [sym____reserved___] = ACTIONS(1985),
    [sym_date] = ACTIONS(1985),
    [sym_day] = ACTIONS(1985),
    [sym_time] = ACTIONS(1985),
    [sym_ldate] = ACTIONS(1985),
    [sym_push_device] = ACTIONS(1985),
    [sym_push_channel] = ACTIONS(1985),
    [sym_push_devchan] = ACTIONS(1985),
    [sym_release_device] = ACTIONS(1985),
    [sym_release_channel] = ACTIONS(1985),
    [sym_release_devchan] = ACTIONS(1985),
    [sym_master_slot] = ACTIONS(1985),
    [sym_get_pulse_time] = ACTIONS(1985),
    [sym_get_timer] = ACTIONS(1985),
    [sym_master_sn] = ACTIONS(1985),
    [sym_system_number] = ACTIONS(1985),
    [sym_button] = ACTIONS(1985),
    [sym_dv_channel] = ACTIONS(1985),
    [sym_data] = ACTIONS(1985),
    [sym_level] = ACTIONS(1985),
    [sym_timeline] = ACTIONS(1985),
    [sym_channel] = ACTIONS(1985),
    [sym_custom] = ACTIONS(1985),
    [sym_atoi] = ACTIONS(1985),
    [sym_atol] = ACTIONS(1985),
    [sym_atof] = ACTIONS(1985),
    [sym_itoa] = ACTIONS(1985),
    [sym_itohex] = ACTIONS(1985),
    [sym_hextoi] = ACTIONS(1985),
    [sym_ftoa] = ACTIONS(1985),
    [sym_find_string] = ACTIONS(1985),
    [sym_left_string] = ACTIONS(1985),
    [sym_right_string] = ACTIONS(1985),
    [sym_mid_string] = ACTIONS(1985),
    [sym_remove_string] = ACTIONS(1985),
    [sym_get_buffer_string] = ACTIONS(1985),
    [sym_get_buffer_char] = ACTIONS(1985),
    [sym_get_multi_buffer_string] = ACTIONS(1985),
    [sym_length_string] = ACTIONS(1985),
    [sym_max_length_string] = ACTIONS(1985),
    [sym_set_length_string] = ACTIONS(1985),
    [sym_compare_string] = ACTIONS(1985),
    [sym_upper_string] = ACTIONS(1985),
    [sym_lower_string] = ACTIONS(1985),
    [sym_get_last] = ACTIONS(1985),
    [sym_date_to_day] = ACTIONS(1985),
    [sym_date_to_month] = ACTIONS(1985),
    [sym_date_to_year] = ACTIONS(1985),
    [sym_time_to_hour] = ACTIONS(1985),
    [sym_time_to_minute] = ACTIONS(1985),
    [sym_time_to_second] = ACTIONS(1985),
    [sym_day_of_week] = ACTIONS(1985),
    [sym_set_pulse_time] = ACTIONS(1985),
    [sym_set_timer] = ACTIONS(1985),
    [sym_file_open] = ACTIONS(1985),
    [sym_file_close] = ACTIONS(1985),
    [sym_file_read] = ACTIONS(1985),
    [sym_file_read_line] = ACTIONS(1985),
    [sym_file_write] = ACTIONS(1985),
    [sym_file_write_line] = ACTIONS(1985),
    [sym_file_copy] = ACTIONS(1985),
    [sym_file_delete] = ACTIONS(1985),
    [sym_file_rename] = ACTIONS(1985),
    [sym_file_dir] = ACTIONS(1985),
    [sym_file_getdir] = ACTIONS(1985),
    [sym_file_setdir] = ACTIONS(1985),
    [sym_file_create_dir] = ACTIONS(1985),
    [sym_file_remove_dir] = ACTIONS(1985),
    [sym_file_seek] = ACTIONS(1985),
    [sym_length_array] = ACTIONS(1985),
    [sym_max_length_array] = ACTIONS(1985),
    [sym_set_length_array] = ACTIONS(1985),
    [sym_ip_client_open] = ACTIONS(1985),
    [sym_ip_client_close] = ACTIONS(1985),
    [sym_ip_server_open] = ACTIONS(1985),
    [sym_ip_server_close] = ACTIONS(1985),
    [sym_ip_bound_client_open] = ACTIONS(1985),
    [sym_ip_mc_server_open] = ACTIONS(1985),
    [sym_ip_set_option] = ACTIONS(1985),
    [sym_ssh_client_open] = ACTIONS(1985),
    [sym_ssh_client_close] = ACTIONS(1985),
    [sym_tls_client_open] = ACTIONS(1985),
    [sym_tls_client_close] = ACTIONS(1985),
    [sym_do_push] = ACTIONS(1985),
    [sym_do_push_timed] = ACTIONS(1985),
    [sym_do_release] = ACTIONS(1985),
    [sym_device_id] = ACTIONS(1985),
    [sym_device_info] = ACTIONS(1985),
    [sym_device_id_string] = ACTIONS(1985),
    [sym_device_standby] = ACTIONS(1985),
    [sym_device_wake] = ACTIONS(1985),
    [sym_timeline_create] = ACTIONS(1985),
    [sym_timeline_kill] = ACTIONS(1985),
    [sym_timeline_reload] = ACTIONS(1985),
    [sym_timeline_pause] = ACTIONS(1985),
    [sym_timeline_restart] = ACTIONS(1985),
    [sym_timeline_set] = ACTIONS(1985),
    [sym_timeline_get] = ACTIONS(1985),
    [sym_timeline_active] = ACTIONS(1985),
    [sym_timeline_dynamic_id] = ACTIONS(1985),
    [sym_variable_to_string] = ACTIONS(1985),
    [sym_string_to_variable] = ACTIONS(1985),
    [sym_length_variable_to_string] = ACTIONS(1985),
    [sym_type_cast] = ACTIONS(1985),
    [sym_variable_to_xml] = ACTIONS(1985),
    [sym_xml_to_variable] = ACTIONS(1985),
    [sym_length_variable_to_xml] = ACTIONS(1985),
    [sym_redirect_string] = ACTIONS(1985),
    [sym_random_number] = ACTIONS(1985),
    [sym_get_unique_id] = ACTIONS(1985),
    [sym_get_system_number] = ACTIONS(1985),
    [sym_set_system_number] = ACTIONS(1985),
    [sym_get_serial_number] = ACTIONS(1985),
    [sym_get_master_build] = ACTIONS(1985),
    [sym_set_validation_code] = ACTIONS(1985),
    [sym_reboot] = ACTIONS(1985),
    [sym_load_duet_module] = ACTIONS(1985),
    [sym_unload_duet_module] = ACTIONS(1985),
    [sym_duet_mem_size_set] = ACTIONS(1985),
    [sym_duet_mem_size_get] = ACTIONS(1985),
    [sym_dynamic_application_device] = ACTIONS(1985),
    [sym_dynamic_polled_port] = ACTIONS(1985),
    [sym_static_port_binding] = ACTIONS(1985),
    [sym_static_ip_binding] = ACTIONS(1985),
    [sym_get_dns_list] = ACTIONS(1985),
    [sym_set_dns_list] = ACTIONS(1985),
    [sym_get_ip_address] = ACTIONS(1985),
    [sym_set_ip_address] = ACTIONS(1985),
    [sym_get_url_list] = ACTIONS(1985),
    [sym_add_url_entry] = ACTIONS(1985),
    [sym_delete_url_entry] = ACTIONS(1985),
    [sym_abs_value] = ACTIONS(1985),
    [sym_max_value] = ACTIONS(1985),
    [sym_min_value] = ACTIONS(1985),
    [sym_exp_value] = ACTIONS(1985),
    [sym_log_value] = ACTIONS(1985),
    [sym_log10_value] = ACTIONS(1985),
    [sym_power_value] = ACTIONS(1985),
    [sym_sqrt_value] = ACTIONS(1985),
    [sym_raw_be] = ACTIONS(1985),
    [sym_raw_le] = ACTIONS(1985),
    [sym_internal_queue_size_set] = ACTIONS(1985),
    [sym_internal_queue_size_get] = ACTIONS(1985),
    [sym_internal_threshold_set] = ACTIONS(1985),
    [sym_internal_threshold_get] = ACTIONS(1985),
    [sym_rebuilt_event] = ACTIONS(1985),
    [sym_do_custom_event] = ACTIONS(1985),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1985),
    [sym_clkmgr_set_clk_source] = ACTIONS(1985),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1985),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1985),
    [sym_clkmgr_get_timezone] = ACTIONS(1985),
    [sym_clkmgr_set_timezone] = ACTIONS(1985),
    [sym_clkmgr_get_resync_period] = ACTIONS(1985),
    [sym_clkmgr_set_resync_period] = ACTIONS(1985),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1985),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1985),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1985),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1985),
    [sym_clkmgr_get_timeservers] = ACTIONS(1985),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1985),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1985),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1985),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1985),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1985),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1985),
    [sym_set_log_level] = ACTIONS(1985),
    [sym_get_log_level] = ACTIONS(1985),
    [sym_amx_log] = ACTIONS(1985),
    [sym_smtp_server_config_set] = ACTIONS(1985),
    [sym_smtp_server_config_get] = ACTIONS(1985),
    [sym_smtp_send] = ACTIONS(1985),
    [sym_validate_netlinx_account] = ACTIONS(1985),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1985),
    [sym_authenticate_certificate] = ACTIONS(1985),
    [sym_audit_netlinx_session_event] = ACTIONS(1985),
    [sym_audit_netlinx_generic_event] = ACTIONS(1985),
    [sym_get_available_flash_disk_space] = ACTIONS(1985),
    [sym_get_max_flash_disk_space] = ACTIONS(1985),
    [sym_data_create_feed] = ACTIONS(1985),
    [sym_data_delete_feed] = ACTIONS(1985),
    [sym_data_publish_feed] = ACTIONS(1985),
    [sym_data_get_published_feed] = ACTIONS(1985),
    [sym_data_add_record] = ACTIONS(1985),
    [sym_data_get_event_record] = ACTIONS(1985),
    [sym__wc_data_create_feed] = ACTIONS(1985),
    [sym__wc_data_add_record] = ACTIONS(1985),
    [sym__wc_data_get_event_record] = ACTIONS(1985),
    [sym_format] = ACTIONS(1985),
    [sym_astro_clock] = ACTIONS(1985),
    [sym_set_outdoor_temperature] = ACTIONS(1985),
    [sym_set_virtual_level_count] = ACTIONS(1985),
    [sym_set_virtual_channel_count] = ACTIONS(1985),
    [sym_set_virtual_port_count] = ACTIONS(1985),
    [sym__wc_encode_int] = ACTIONS(1985),
    [sym__wc_decode_int] = ACTIONS(1985),
    [sym__wc_int] = ACTIONS(1985),
    [sym__wc_to_ch_int] = ACTIONS(1985),
    [sym__ch_to_wc_int] = ACTIONS(1985),
    [sym__wc_find_string_int] = ACTIONS(1985),
    [sym__wc_left_string_int] = ACTIONS(1985),
    [sym__wc_lower_string_int] = ACTIONS(1985),
    [sym__wc_mid_string_int] = ACTIONS(1985),
    [sym__wc_remove_string_int] = ACTIONS(1985),
    [sym__wc_right_string_int] = ACTIONS(1985),
    [sym__wc_upper_string_int] = ACTIONS(1985),
    [sym__wc_compare_string_int] = ACTIONS(1985),
    [sym__wc_get_buffer_char_int] = ACTIONS(1985),
    [sym__wc_get_buffer_string_int] = ACTIONS(1985),
    [sym__wc_concat_string_int] = ACTIONS(1985),
    [sym__wc_file_open_int] = ACTIONS(1985),
    [sym__wc_file_close_int] = ACTIONS(1985),
    [sym__wc_file_read_int] = ACTIONS(1985),
    [sym__wc_file_read_line_int] = ACTIONS(1985),
    [sym__wc_file_write_int] = ACTIONS(1985),
    [sym__wc_file_write_line_int] = ACTIONS(1985),
    [aux_sym_preproc_include_token1] = ACTIONS(1985),
    [aux_sym_preproc_define_token1] = ACTIONS(1987),
    [aux_sym_preproc_warn_token1] = ACTIONS(1987),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1987),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1987),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1987),
    [anon_sym_LBRACK] = ACTIONS(1987),
    [anon_sym_LBRACE] = ACTIONS(1987),
    [anon_sym_RBRACE] = ACTIONS(1987),
    [anon_sym_LPAREN] = ACTIONS(1985),
    [anon_sym_BANG] = ACTIONS(1987),
    [anon_sym_TILDE] = ACTIONS(1987),
    [anon_sym_DASH] = ACTIONS(1985),
    [anon_sym_PLUS] = ACTIONS(1985),
    [anon_sym_DASH_DASH] = ACTIONS(1987),
    [anon_sym_PLUS_PLUS] = ACTIONS(1987),
    [anon_sym_DQUOTE] = ACTIONS(1987),
    [anon_sym_SQUOTE] = ACTIONS(1987),
    [sym_number_literal] = ACTIONS(1987),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(523)] = {
    [sym_identifier] = ACTIONS(1989),
    [sym_define_call_keyword] = ACTIONS(1989),
    [sym_define_function_keyword] = ACTIONS(1989),
    [sym_define_library_function_keyword] = ACTIONS(1989),
    [sym_define_module_keyword] = ACTIONS(1989),
    [sym_char_keyword] = ACTIONS(1989),
    [sym_widechar_keyword] = ACTIONS(1989),
    [sym_integer_keyword] = ACTIONS(1989),
    [sym_sinteger_keyword] = ACTIONS(1989),
    [sym_long_keyword] = ACTIONS(1989),
    [sym_slong_keyword] = ACTIONS(1989),
    [sym_float_keyword] = ACTIONS(1989),
    [sym_double_keyword] = ACTIONS(1989),
    [sym_variant_keyword] = ACTIONS(1989),
    [sym_variantarray_keyword] = ACTIONS(1989),
    [sym_local_var_keyword] = ACTIONS(1989),
    [sym_stack_var_keyword] = ACTIONS(1989),
    [sym_constant_keyword] = ACTIONS(1989),
    [sym_volatile_keyword] = ACTIONS(1989),
    [sym_non_volatile_keyword] = ACTIONS(1989),
    [sym_persistent_keyword] = ACTIONS(1989),
    [sym_dev_keyword] = ACTIONS(1989),
    [sym_devlev_keyword] = ACTIONS(1989),
    [sym_devchan_keyword] = ACTIONS(1989),
    [sym_if_keyword] = ACTIONS(1989),
    [sym_switch_keyword] = ACTIONS(1989),
    [sym_case_keyword] = ACTIONS(1989),
    [sym_default_keyword] = ACTIONS(1989),
    [sym_while_keyword] = ACTIONS(1989),
    [sym_for_keyword] = ACTIONS(1989),
    [sym_break_keyword] = ACTIONS(1989),
    [sym_continue_keyword] = ACTIONS(1989),
    [sym_return_keyword] = ACTIONS(1989),
    [sym_select_keyword] = ACTIONS(1989),
    [sym_active_keyword] = ACTIONS(1989),
    [sym_struct_keyword] = ACTIONS(1989),
    [sym_structure_keyword] = ACTIONS(1989),
    [aux_sym_on_keyword_token1] = ACTIONS(1989),
    [aux_sym_off_keyword_token1] = ACTIONS(1989),
    [sym_send_level_keyword] = ACTIONS(1989),
    [sym_send_string_keyword] = ACTIONS(1989),
    [sym_send_command_keyword] = ACTIONS(1989),
    [sym_clear_buffer_keyword] = ACTIONS(1989),
    [sym_create_buffer_keyword] = ACTIONS(1989),
    [sym_create_multi_buffer_keyword] = ACTIONS(1989),
    [sym_call_keyword] = ACTIONS(1989),
    [sym_system_call_keyword] = ACTIONS(1989),
    [sym_devchan_to_keyword] = ACTIONS(1989),
    [sym_devchan_min_to_keyword] = ACTIONS(1989),
    [sym_devchan_total_off_keyword] = ACTIONS(1989),
    [sym_devchan_pulse_keyword] = ACTIONS(1989),
    [sym_wait_keyword] = ACTIONS(1989),
    [sym_cancel_wait_keyword] = ACTIONS(1989),
    [sym_wait_until_keyword] = ACTIONS(1989),
    [sym_cancel_wait_until_keyword] = ACTIONS(1989),
    [sym_cancel_all_wait_keyword] = ACTIONS(1989),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1989),
    [sym_not] = ACTIONS(1989),
    [sym_bnot] = ACTIONS(1989),
    [sym___file__] = ACTIONS(1989),
    [sym___date__] = ACTIONS(1989),
    [sym___ldate__] = ACTIONS(1989),
    [sym___time__] = ACTIONS(1989),
    [sym___line__] = ACTIONS(1989),
    [sym___name__] = ACTIONS(1989),
    [sym___netlinx__] = ACTIONS(1989),
    [sym_netlinx_axi_version] = ACTIONS(1989),
    [sym_url_flg_tcp] = ACTIONS(1989),
    [sym_url_flg_accinfopresent] = ACTIONS(1989),
    [sym_url_flg_temp] = ACTIONS(1989),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1989),
    [sym_url_flg_stat_mask] = ACTIONS(1989),
    [sym_url_flg_stat_lookup] = ACTIONS(1989),
    [sym_url_flg_stat_connecting] = ACTIONS(1989),
    [sym_url_flg_stat_waiting] = ACTIONS(1989),
    [sym_url_flg_stat_connected] = ACTIONS(1989),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1989),
    [sym_first_virtual_device] = ACTIONS(1989),
    [sym_first_local_port] = ACTIONS(1989),
    [sym_dynamic_virtual_device] = ACTIONS(1989),
    [sym_timeline_once] = ACTIONS(1989),
    [sym_timeline_repeat] = ACTIONS(1989),
    [sym_timeline_absolute] = ACTIONS(1989),
    [sym_timeline_relative] = ACTIONS(1989),
    [sym_do_push_timed_infinite] = ACTIONS(1989),
    [sym_source_type_no_address] = ACTIONS(1989),
    [sym_source_type_neuron_id] = ACTIONS(1989),
    [sym_source_type_ip_address] = ACTIONS(1989),
    [sym_source_type_axlink] = ACTIONS(1989),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1989),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1989),
    [sym_source_type_ip_socket_address] = ACTIONS(1989),
    [sym_source_type_rs232] = ACTIONS(1989),
    [sym_source_type_internal] = ACTIONS(1989),
    [sym_source_type_mac_address] = ACTIONS(1989),
    [sym_source_type_ipv4_port] = ACTIONS(1989),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1989),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1989),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1989),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1989),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1989),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1989),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1989),
    [sym_true] = ACTIONS(1989),
    [sym_false] = ACTIONS(1989),
    [sym_file_read_only] = ACTIONS(1989),
    [sym_file_rw_new] = ACTIONS(1989),
    [sym_file_rw_append] = ACTIONS(1989),
    [sym_ip_tcp] = ACTIONS(1989),
    [sym_ip_udp] = ACTIONS(1989),
    [sym_ip_udp_2way] = ACTIONS(1989),
    [sym_xml_encode_types] = ACTIONS(1989),
    [sym_xml_encode_char_as_list] = ACTIONS(1989),
    [sym_xml_encode_le] = ACTIONS(1989),
    [sym_xml_decode_types] = ACTIONS(1989),
    [sym_xml_decode_no_preserve] = ACTIONS(1989),
    [sym_duet_dev_type_display_device] = ACTIONS(1989),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1989),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1989),
    [sym_duet_dev_type_io_device] = ACTIONS(1989),
    [sym_duet_dev_type_relay_device] = ACTIONS(1989),
    [sym_duet_dev_type_ups] = ACTIONS(1989),
    [sym_duet_dev_type_amplifier] = ACTIONS(1989),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1989),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1989),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1989),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1989),
    [sym_duet_dev_type_camera] = ACTIONS(1989),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1989),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1989),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1989),
    [sym_duet_dev_type_dss] = ACTIONS(1989),
    [sym_duet_dev_type_dvr] = ACTIONS(1989),
    [sym_duet_dev_type_disc_device] = ACTIONS(1989),
    [sym_duet_dev_type_document_camera] = ACTIONS(1989),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1989),
    [sym_duet_dev_type_hvac] = ACTIONS(1989),
    [sym_duet_dev_type_keypad] = ACTIONS(1989),
    [sym_duet_dev_type_light] = ACTIONS(1989),
    [sym_duet_dev_type_monitor] = ACTIONS(1989),
    [sym_duet_dev_type_motor] = ACTIONS(1989),
    [sym_duet_dev_type_multi_window] = ACTIONS(1989),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1989),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1989),
    [sym_duet_dev_type_receiver] = ACTIONS(1989),
    [sym_duet_dev_type_security_system] = ACTIONS(1989),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1989),
    [sym_duet_dev_type_settop_box] = ACTIONS(1989),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1989),
    [sym_duet_dev_type_switcher] = ACTIONS(1989),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1989),
    [sym_duet_dev_type_tv] = ACTIONS(1989),
    [sym_duet_dev_type_utility] = ACTIONS(1989),
    [sym_duet_dev_type_vcr] = ACTIONS(1989),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1989),
    [sym_duet_dev_type_video_processor] = ACTIONS(1989),
    [sym_duet_dev_type_video_projector] = ACTIONS(1989),
    [sym_duet_dev_type_video_wall] = ACTIONS(1989),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1989),
    [sym_duet_dev_type_weather] = ACTIONS(1989),
    [sym_duet_dev_not_polled] = ACTIONS(1989),
    [sym_duet_dev_polled] = ACTIONS(1989),
    [sym_clkmgr_mode_network] = ACTIONS(1989),
    [sym_clkmgr_mode_standalone] = ACTIONS(1989),
    [sym_normal_standby] = ACTIONS(1989),
    [sym_normal_wake] = ACTIONS(1989),
    [sym_ip_multicast_ttl_option] = ACTIONS(1989),
    [sym_ip_ttl_subnet] = ACTIONS(1989),
    [sym_ip_ttl_site] = ACTIONS(1989),
    [sym_ip_ttl_region] = ACTIONS(1989),
    [sym_ip_ttl_continent] = ACTIONS(1989),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1989),
    [sym_ip_nodelay_on] = ACTIONS(1989),
    [sym_ip_nodelay_off] = ACTIONS(1989),
    [sym_amx_error] = ACTIONS(1989),
    [sym_amx_warning] = ACTIONS(1989),
    [sym_amx_info] = ACTIONS(1989),
    [sym_amx_debug] = ACTIONS(1989),
    [sym_netlinx_login_success] = ACTIONS(1989),
    [sym_netlinx_login_fail] = ACTIONS(1989),
    [sym_netlinx_logout] = ACTIONS(1989),
    [sym_null_str] = ACTIONS(1989),
    [sym_smtp_address] = ACTIONS(1989),
    [sym_smtp_port_number] = ACTIONS(1989),
    [sym_smtp_username] = ACTIONS(1989),
    [sym_smtp_password] = ACTIONS(1989),
    [sym_smtp_require_tls] = ACTIONS(1989),
    [sym_smtp_from] = ACTIONS(1989),
    [sym_smtp_tls_true] = ACTIONS(1989),
    [sym_smtp_tls_false] = ACTIONS(1989),
    [sym_listview_on_row_select_event] = ACTIONS(1989),
    [sym_data_structure_unknown] = ACTIONS(1989),
    [sym_data_structure_datafeed] = ACTIONS(1989),
    [sym_data_structure_datarecordset] = ACTIONS(1989),
    [sym_data_structure_datarecord] = ACTIONS(1989),
    [sym_data_structure_datafield] = ACTIONS(1989),
    [sym_data_max_content_field_count] = ACTIONS(1989),
    [sym_data_max_metadata_field_count] = ACTIONS(1989),
    [sym_data_max_id_length] = ACTIONS(1989),
    [sym_data_max_name_length] = ACTIONS(1989),
    [sym_data_max_description_length] = ACTIONS(1989),
    [sym_data_max_type_length] = ACTIONS(1989),
    [sym_data_max_format_length] = ACTIONS(1989),
    [sym_data_max_source_length] = ACTIONS(1989),
    [sym_data_max_value_length] = ACTIONS(1989),
    [sym_data_max_label_length] = ACTIONS(1989),
    [sym_data_max_url_length] = ACTIONS(1989),
    [sym_max_last_login_info_length] = ACTIONS(1989),
    [sym_data_type_unknown] = ACTIONS(1989),
    [sym_data_type_string] = ACTIONS(1989),
    [sym_data_type_datetime] = ACTIONS(1989),
    [sym_data_type_date] = ACTIONS(1989),
    [sym_data_type_time] = ACTIONS(1989),
    [sym_data_type_image] = ACTIONS(1989),
    [sym_data_format_url] = ACTIONS(1989),
    [sym_data_format_phone] = ACTIONS(1989),
    [sym_data_format_email] = ACTIONS(1989),
    [sym_data_format_iso8601] = ACTIONS(1989),
    [sym_auditlog_priv] = ACTIONS(1989),
    [sym_configuration_priv] = ACTIONS(1989),
    [sym_deviceconfig_priv] = ACTIONS(1989),
    [sym_ftp_priv] = ACTIONS(1989),
    [sym_http_priv] = ACTIONS(1989),
    [sym_networkconfig_priv] = ACTIONS(1989),
    [sym_programport_priv] = ACTIONS(1989),
    [sym_remoteui_priv] = ACTIONS(1989),
    [sym_securitycontrol_priv] = ACTIONS(1989),
    [sym_softwaremanagement_priv] = ACTIONS(1989),
    [sym_terminal_priv] = ACTIONS(1989),
    [sym_tpadmin_priv] = ACTIONS(1989),
    [sym_user1_priv] = ACTIONS(1989),
    [sym_user2_priv] = ACTIONS(1989),
    [sym_user3_priv] = ACTIONS(1989),
    [sym_user4_priv] = ACTIONS(1989),
    [sym_usermanagement_priv] = ACTIONS(1989),
    [sym_valid_account] = ACTIONS(1989),
    [sym_err_name_invalid] = ACTIONS(1989),
    [sym_err_password_invalid] = ACTIONS(1989),
    [sym_err_invalid_account] = ACTIONS(1989),
    [sym_err_no_authorization] = ACTIONS(1989),
    [sym_err_invalid_parameter] = ACTIONS(1989),
    [sym_err_user_locked_out] = ACTIONS(1989),
    [sym_err_user_account_expired] = ACTIONS(1989),
    [sym_err_general_error] = ACTIONS(1989),
    [sym_err_authentication_error] = ACTIONS(1989),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1989),
    [sym_tls_validate_certificate] = ACTIONS(1989),
    [sym_tdata] = ACTIONS(1989),
    [sym_tchannel] = ACTIONS(1989),
    [sym_tlevel] = ACTIONS(1989),
    [sym_tbutton] = ACTIONS(1989),
    [sym_ttimeline] = ACTIONS(1989),
    [sym_tcustom] = ACTIONS(1989),
    [sym_url_struct] = ACTIONS(1989),
    [sym_dns_struct] = ACTIONS(1989),
    [sym_ip_address_struct] = ACTIONS(1989),
    [sym_dev_info_struct] = ACTIONS(1989),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1989),
    [sym_data_feed] = ACTIONS(1989),
    [sym_data_field] = ACTIONS(1989),
    [sym_data_record] = ACTIONS(1989),
    [sym_wc_data_feed] = ACTIONS(1989),
    [sym_wc_data_field] = ACTIONS(1989),
    [sym_wc_data_record] = ACTIONS(1989),
    [sym_last_login_info] = ACTIONS(1989),
    [sym____reserved___] = ACTIONS(1989),
    [sym_date] = ACTIONS(1989),
    [sym_day] = ACTIONS(1989),
    [sym_time] = ACTIONS(1989),
    [sym_ldate] = ACTIONS(1989),
    [sym_push_device] = ACTIONS(1989),
    [sym_push_channel] = ACTIONS(1989),
    [sym_push_devchan] = ACTIONS(1989),
    [sym_release_device] = ACTIONS(1989),
    [sym_release_channel] = ACTIONS(1989),
    [sym_release_devchan] = ACTIONS(1989),
    [sym_master_slot] = ACTIONS(1989),
    [sym_get_pulse_time] = ACTIONS(1989),
    [sym_get_timer] = ACTIONS(1989),
    [sym_master_sn] = ACTIONS(1989),
    [sym_system_number] = ACTIONS(1989),
    [sym_button] = ACTIONS(1989),
    [sym_dv_channel] = ACTIONS(1989),
    [sym_data] = ACTIONS(1989),
    [sym_level] = ACTIONS(1989),
    [sym_timeline] = ACTIONS(1989),
    [sym_channel] = ACTIONS(1989),
    [sym_custom] = ACTIONS(1989),
    [sym_atoi] = ACTIONS(1989),
    [sym_atol] = ACTIONS(1989),
    [sym_atof] = ACTIONS(1989),
    [sym_itoa] = ACTIONS(1989),
    [sym_itohex] = ACTIONS(1989),
    [sym_hextoi] = ACTIONS(1989),
    [sym_ftoa] = ACTIONS(1989),
    [sym_find_string] = ACTIONS(1989),
    [sym_left_string] = ACTIONS(1989),
    [sym_right_string] = ACTIONS(1989),
    [sym_mid_string] = ACTIONS(1989),
    [sym_remove_string] = ACTIONS(1989),
    [sym_get_buffer_string] = ACTIONS(1989),
    [sym_get_buffer_char] = ACTIONS(1989),
    [sym_get_multi_buffer_string] = ACTIONS(1989),
    [sym_length_string] = ACTIONS(1989),
    [sym_max_length_string] = ACTIONS(1989),
    [sym_set_length_string] = ACTIONS(1989),
    [sym_compare_string] = ACTIONS(1989),
    [sym_upper_string] = ACTIONS(1989),
    [sym_lower_string] = ACTIONS(1989),
    [sym_get_last] = ACTIONS(1989),
    [sym_date_to_day] = ACTIONS(1989),
    [sym_date_to_month] = ACTIONS(1989),
    [sym_date_to_year] = ACTIONS(1989),
    [sym_time_to_hour] = ACTIONS(1989),
    [sym_time_to_minute] = ACTIONS(1989),
    [sym_time_to_second] = ACTIONS(1989),
    [sym_day_of_week] = ACTIONS(1989),
    [sym_set_pulse_time] = ACTIONS(1989),
    [sym_set_timer] = ACTIONS(1989),
    [sym_file_open] = ACTIONS(1989),
    [sym_file_close] = ACTIONS(1989),
    [sym_file_read] = ACTIONS(1989),
    [sym_file_read_line] = ACTIONS(1989),
    [sym_file_write] = ACTIONS(1989),
    [sym_file_write_line] = ACTIONS(1989),
    [sym_file_copy] = ACTIONS(1989),
    [sym_file_delete] = ACTIONS(1989),
    [sym_file_rename] = ACTIONS(1989),
    [sym_file_dir] = ACTIONS(1989),
    [sym_file_getdir] = ACTIONS(1989),
    [sym_file_setdir] = ACTIONS(1989),
    [sym_file_create_dir] = ACTIONS(1989),
    [sym_file_remove_dir] = ACTIONS(1989),
    [sym_file_seek] = ACTIONS(1989),
    [sym_length_array] = ACTIONS(1989),
    [sym_max_length_array] = ACTIONS(1989),
    [sym_set_length_array] = ACTIONS(1989),
    [sym_ip_client_open] = ACTIONS(1989),
    [sym_ip_client_close] = ACTIONS(1989),
    [sym_ip_server_open] = ACTIONS(1989),
    [sym_ip_server_close] = ACTIONS(1989),
    [sym_ip_bound_client_open] = ACTIONS(1989),
    [sym_ip_mc_server_open] = ACTIONS(1989),
    [sym_ip_set_option] = ACTIONS(1989),
    [sym_ssh_client_open] = ACTIONS(1989),
    [sym_ssh_client_close] = ACTIONS(1989),
    [sym_tls_client_open] = ACTIONS(1989),
    [sym_tls_client_close] = ACTIONS(1989),
    [sym_do_push] = ACTIONS(1989),
    [sym_do_push_timed] = ACTIONS(1989),
    [sym_do_release] = ACTIONS(1989),
    [sym_device_id] = ACTIONS(1989),
    [sym_device_info] = ACTIONS(1989),
    [sym_device_id_string] = ACTIONS(1989),
    [sym_device_standby] = ACTIONS(1989),
    [sym_device_wake] = ACTIONS(1989),
    [sym_timeline_create] = ACTIONS(1989),
    [sym_timeline_kill] = ACTIONS(1989),
    [sym_timeline_reload] = ACTIONS(1989),
    [sym_timeline_pause] = ACTIONS(1989),
    [sym_timeline_restart] = ACTIONS(1989),
    [sym_timeline_set] = ACTIONS(1989),
    [sym_timeline_get] = ACTIONS(1989),
    [sym_timeline_active] = ACTIONS(1989),
    [sym_timeline_dynamic_id] = ACTIONS(1989),
    [sym_variable_to_string] = ACTIONS(1989),
    [sym_string_to_variable] = ACTIONS(1989),
    [sym_length_variable_to_string] = ACTIONS(1989),
    [sym_type_cast] = ACTIONS(1989),
    [sym_variable_to_xml] = ACTIONS(1989),
    [sym_xml_to_variable] = ACTIONS(1989),
    [sym_length_variable_to_xml] = ACTIONS(1989),
    [sym_redirect_string] = ACTIONS(1989),
    [sym_random_number] = ACTIONS(1989),
    [sym_get_unique_id] = ACTIONS(1989),
    [sym_get_system_number] = ACTIONS(1989),
    [sym_set_system_number] = ACTIONS(1989),
    [sym_get_serial_number] = ACTIONS(1989),
    [sym_get_master_build] = ACTIONS(1989),
    [sym_set_validation_code] = ACTIONS(1989),
    [sym_reboot] = ACTIONS(1989),
    [sym_load_duet_module] = ACTIONS(1989),
    [sym_unload_duet_module] = ACTIONS(1989),
    [sym_duet_mem_size_set] = ACTIONS(1989),
    [sym_duet_mem_size_get] = ACTIONS(1989),
    [sym_dynamic_application_device] = ACTIONS(1989),
    [sym_dynamic_polled_port] = ACTIONS(1989),
    [sym_static_port_binding] = ACTIONS(1989),
    [sym_static_ip_binding] = ACTIONS(1989),
    [sym_get_dns_list] = ACTIONS(1989),
    [sym_set_dns_list] = ACTIONS(1989),
    [sym_get_ip_address] = ACTIONS(1989),
    [sym_set_ip_address] = ACTIONS(1989),
    [sym_get_url_list] = ACTIONS(1989),
    [sym_add_url_entry] = ACTIONS(1989),
    [sym_delete_url_entry] = ACTIONS(1989),
    [sym_abs_value] = ACTIONS(1989),
    [sym_max_value] = ACTIONS(1989),
    [sym_min_value] = ACTIONS(1989),
    [sym_exp_value] = ACTIONS(1989),
    [sym_log_value] = ACTIONS(1989),
    [sym_log10_value] = ACTIONS(1989),
    [sym_power_value] = ACTIONS(1989),
    [sym_sqrt_value] = ACTIONS(1989),
    [sym_raw_be] = ACTIONS(1989),
    [sym_raw_le] = ACTIONS(1989),
    [sym_internal_queue_size_set] = ACTIONS(1989),
    [sym_internal_queue_size_get] = ACTIONS(1989),
    [sym_internal_threshold_set] = ACTIONS(1989),
    [sym_internal_threshold_get] = ACTIONS(1989),
    [sym_rebuilt_event] = ACTIONS(1989),
    [sym_do_custom_event] = ACTIONS(1989),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1989),
    [sym_clkmgr_set_clk_source] = ACTIONS(1989),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1989),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1989),
    [sym_clkmgr_get_timezone] = ACTIONS(1989),
    [sym_clkmgr_set_timezone] = ACTIONS(1989),
    [sym_clkmgr_get_resync_period] = ACTIONS(1989),
    [sym_clkmgr_set_resync_period] = ACTIONS(1989),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1989),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1989),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1989),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1989),
    [sym_clkmgr_get_timeservers] = ACTIONS(1989),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1989),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1989),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1989),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1989),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1989),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1989),
    [sym_set_log_level] = ACTIONS(1989),
    [sym_get_log_level] = ACTIONS(1989),
    [sym_amx_log] = ACTIONS(1989),
    [sym_smtp_server_config_set] = ACTIONS(1989),
    [sym_smtp_server_config_get] = ACTIONS(1989),
    [sym_smtp_send] = ACTIONS(1989),
    [sym_validate_netlinx_account] = ACTIONS(1989),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1989),
    [sym_authenticate_certificate] = ACTIONS(1989),
    [sym_audit_netlinx_session_event] = ACTIONS(1989),
    [sym_audit_netlinx_generic_event] = ACTIONS(1989),
    [sym_get_available_flash_disk_space] = ACTIONS(1989),
    [sym_get_max_flash_disk_space] = ACTIONS(1989),
    [sym_data_create_feed] = ACTIONS(1989),
    [sym_data_delete_feed] = ACTIONS(1989),
    [sym_data_publish_feed] = ACTIONS(1989),
    [sym_data_get_published_feed] = ACTIONS(1989),
    [sym_data_add_record] = ACTIONS(1989),
    [sym_data_get_event_record] = ACTIONS(1989),
    [sym__wc_data_create_feed] = ACTIONS(1989),
    [sym__wc_data_add_record] = ACTIONS(1989),
    [sym__wc_data_get_event_record] = ACTIONS(1989),
    [sym_format] = ACTIONS(1989),
    [sym_astro_clock] = ACTIONS(1989),
    [sym_set_outdoor_temperature] = ACTIONS(1989),
    [sym_set_virtual_level_count] = ACTIONS(1989),
    [sym_set_virtual_channel_count] = ACTIONS(1989),
    [sym_set_virtual_port_count] = ACTIONS(1989),
    [sym__wc_encode_int] = ACTIONS(1989),
    [sym__wc_decode_int] = ACTIONS(1989),
    [sym__wc_int] = ACTIONS(1989),
    [sym__wc_to_ch_int] = ACTIONS(1989),
    [sym__ch_to_wc_int] = ACTIONS(1989),
    [sym__wc_find_string_int] = ACTIONS(1989),
    [sym__wc_left_string_int] = ACTIONS(1989),
    [sym__wc_lower_string_int] = ACTIONS(1989),
    [sym__wc_mid_string_int] = ACTIONS(1989),
    [sym__wc_remove_string_int] = ACTIONS(1989),
    [sym__wc_right_string_int] = ACTIONS(1989),
    [sym__wc_upper_string_int] = ACTIONS(1989),
    [sym__wc_compare_string_int] = ACTIONS(1989),
    [sym__wc_get_buffer_char_int] = ACTIONS(1989),
    [sym__wc_get_buffer_string_int] = ACTIONS(1989),
    [sym__wc_concat_string_int] = ACTIONS(1989),
    [sym__wc_file_open_int] = ACTIONS(1989),
    [sym__wc_file_close_int] = ACTIONS(1989),
    [sym__wc_file_read_int] = ACTIONS(1989),
    [sym__wc_file_read_line_int] = ACTIONS(1989),
    [sym__wc_file_write_int] = ACTIONS(1989),
    [sym__wc_file_write_line_int] = ACTIONS(1989),
    [aux_sym_preproc_include_token1] = ACTIONS(1989),
    [aux_sym_preproc_define_token1] = ACTIONS(1991),
    [aux_sym_preproc_warn_token1] = ACTIONS(1991),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1991),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1991),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1991),
    [anon_sym_LBRACK] = ACTIONS(1991),
    [anon_sym_LBRACE] = ACTIONS(1991),
    [anon_sym_RBRACE] = ACTIONS(1991),
    [anon_sym_LPAREN] = ACTIONS(1989),
    [anon_sym_BANG] = ACTIONS(1991),
    [anon_sym_TILDE] = ACTIONS(1991),
    [anon_sym_DASH] = ACTIONS(1989),
    [anon_sym_PLUS] = ACTIONS(1989),
    [anon_sym_DASH_DASH] = ACTIONS(1991),
    [anon_sym_PLUS_PLUS] = ACTIONS(1991),
    [anon_sym_DQUOTE] = ACTIONS(1991),
    [anon_sym_SQUOTE] = ACTIONS(1991),
    [sym_number_literal] = ACTIONS(1991),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(524)] = {
    [sym_identifier] = ACTIONS(2001),
    [sym_define_call_keyword] = ACTIONS(2001),
    [sym_define_function_keyword] = ACTIONS(2001),
    [sym_define_library_function_keyword] = ACTIONS(2001),
    [sym_define_module_keyword] = ACTIONS(2001),
    [sym_char_keyword] = ACTIONS(2001),
    [sym_widechar_keyword] = ACTIONS(2001),
    [sym_integer_keyword] = ACTIONS(2001),
    [sym_sinteger_keyword] = ACTIONS(2001),
    [sym_long_keyword] = ACTIONS(2001),
    [sym_slong_keyword] = ACTIONS(2001),
    [sym_float_keyword] = ACTIONS(2001),
    [sym_double_keyword] = ACTIONS(2001),
    [sym_variant_keyword] = ACTIONS(2001),
    [sym_variantarray_keyword] = ACTIONS(2001),
    [sym_local_var_keyword] = ACTIONS(2001),
    [sym_stack_var_keyword] = ACTIONS(2001),
    [sym_constant_keyword] = ACTIONS(2001),
    [sym_volatile_keyword] = ACTIONS(2001),
    [sym_non_volatile_keyword] = ACTIONS(2001),
    [sym_persistent_keyword] = ACTIONS(2001),
    [sym_dev_keyword] = ACTIONS(2001),
    [sym_devlev_keyword] = ACTIONS(2001),
    [sym_devchan_keyword] = ACTIONS(2001),
    [sym_if_keyword] = ACTIONS(2001),
    [sym_switch_keyword] = ACTIONS(2001),
    [sym_case_keyword] = ACTIONS(2001),
    [sym_default_keyword] = ACTIONS(2001),
    [sym_while_keyword] = ACTIONS(2001),
    [sym_for_keyword] = ACTIONS(2001),
    [sym_break_keyword] = ACTIONS(2001),
    [sym_continue_keyword] = ACTIONS(2001),
    [sym_return_keyword] = ACTIONS(2001),
    [sym_select_keyword] = ACTIONS(2001),
    [sym_active_keyword] = ACTIONS(2001),
    [sym_struct_keyword] = ACTIONS(2001),
    [sym_structure_keyword] = ACTIONS(2001),
    [aux_sym_on_keyword_token1] = ACTIONS(2001),
    [aux_sym_off_keyword_token1] = ACTIONS(2001),
    [sym_send_level_keyword] = ACTIONS(2001),
    [sym_send_string_keyword] = ACTIONS(2001),
    [sym_send_command_keyword] = ACTIONS(2001),
    [sym_clear_buffer_keyword] = ACTIONS(2001),
    [sym_create_buffer_keyword] = ACTIONS(2001),
    [sym_create_multi_buffer_keyword] = ACTIONS(2001),
    [sym_call_keyword] = ACTIONS(2001),
    [sym_system_call_keyword] = ACTIONS(2001),
    [sym_devchan_to_keyword] = ACTIONS(2001),
    [sym_devchan_min_to_keyword] = ACTIONS(2001),
    [sym_devchan_total_off_keyword] = ACTIONS(2001),
    [sym_devchan_pulse_keyword] = ACTIONS(2001),
    [sym_wait_keyword] = ACTIONS(2001),
    [sym_cancel_wait_keyword] = ACTIONS(2001),
    [sym_wait_until_keyword] = ACTIONS(2001),
    [sym_cancel_wait_until_keyword] = ACTIONS(2001),
    [sym_cancel_all_wait_keyword] = ACTIONS(2001),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2001),
    [sym_not] = ACTIONS(2001),
    [sym_bnot] = ACTIONS(2001),
    [sym___file__] = ACTIONS(2001),
    [sym___date__] = ACTIONS(2001),
    [sym___ldate__] = ACTIONS(2001),
    [sym___time__] = ACTIONS(2001),
    [sym___line__] = ACTIONS(2001),
    [sym___name__] = ACTIONS(2001),
    [sym___netlinx__] = ACTIONS(2001),
    [sym_netlinx_axi_version] = ACTIONS(2001),
    [sym_url_flg_tcp] = ACTIONS(2001),
    [sym_url_flg_accinfopresent] = ACTIONS(2001),
    [sym_url_flg_temp] = ACTIONS(2001),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2001),
    [sym_url_flg_stat_mask] = ACTIONS(2001),
    [sym_url_flg_stat_lookup] = ACTIONS(2001),
    [sym_url_flg_stat_connecting] = ACTIONS(2001),
    [sym_url_flg_stat_waiting] = ACTIONS(2001),
    [sym_url_flg_stat_connected] = ACTIONS(2001),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2001),
    [sym_first_virtual_device] = ACTIONS(2001),
    [sym_first_local_port] = ACTIONS(2001),
    [sym_dynamic_virtual_device] = ACTIONS(2001),
    [sym_timeline_once] = ACTIONS(2001),
    [sym_timeline_repeat] = ACTIONS(2001),
    [sym_timeline_absolute] = ACTIONS(2001),
    [sym_timeline_relative] = ACTIONS(2001),
    [sym_do_push_timed_infinite] = ACTIONS(2001),
    [sym_source_type_no_address] = ACTIONS(2001),
    [sym_source_type_neuron_id] = ACTIONS(2001),
    [sym_source_type_ip_address] = ACTIONS(2001),
    [sym_source_type_axlink] = ACTIONS(2001),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2001),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2001),
    [sym_source_type_ip_socket_address] = ACTIONS(2001),
    [sym_source_type_rs232] = ACTIONS(2001),
    [sym_source_type_internal] = ACTIONS(2001),
    [sym_source_type_mac_address] = ACTIONS(2001),
    [sym_source_type_ipv4_port] = ACTIONS(2001),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2001),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2001),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2001),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2001),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2001),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2001),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2001),
    [sym_true] = ACTIONS(2001),
    [sym_false] = ACTIONS(2001),
    [sym_file_read_only] = ACTIONS(2001),
    [sym_file_rw_new] = ACTIONS(2001),
    [sym_file_rw_append] = ACTIONS(2001),
    [sym_ip_tcp] = ACTIONS(2001),
    [sym_ip_udp] = ACTIONS(2001),
    [sym_ip_udp_2way] = ACTIONS(2001),
    [sym_xml_encode_types] = ACTIONS(2001),
    [sym_xml_encode_char_as_list] = ACTIONS(2001),
    [sym_xml_encode_le] = ACTIONS(2001),
    [sym_xml_decode_types] = ACTIONS(2001),
    [sym_xml_decode_no_preserve] = ACTIONS(2001),
    [sym_duet_dev_type_display_device] = ACTIONS(2001),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2001),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2001),
    [sym_duet_dev_type_io_device] = ACTIONS(2001),
    [sym_duet_dev_type_relay_device] = ACTIONS(2001),
    [sym_duet_dev_type_ups] = ACTIONS(2001),
    [sym_duet_dev_type_amplifier] = ACTIONS(2001),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2001),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2001),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2001),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2001),
    [sym_duet_dev_type_camera] = ACTIONS(2001),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2001),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2001),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2001),
    [sym_duet_dev_type_dss] = ACTIONS(2001),
    [sym_duet_dev_type_dvr] = ACTIONS(2001),
    [sym_duet_dev_type_disc_device] = ACTIONS(2001),
    [sym_duet_dev_type_document_camera] = ACTIONS(2001),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2001),
    [sym_duet_dev_type_hvac] = ACTIONS(2001),
    [sym_duet_dev_type_keypad] = ACTIONS(2001),
    [sym_duet_dev_type_light] = ACTIONS(2001),
    [sym_duet_dev_type_monitor] = ACTIONS(2001),
    [sym_duet_dev_type_motor] = ACTIONS(2001),
    [sym_duet_dev_type_multi_window] = ACTIONS(2001),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2001),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2001),
    [sym_duet_dev_type_receiver] = ACTIONS(2001),
    [sym_duet_dev_type_security_system] = ACTIONS(2001),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2001),
    [sym_duet_dev_type_settop_box] = ACTIONS(2001),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2001),
    [sym_duet_dev_type_switcher] = ACTIONS(2001),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2001),
    [sym_duet_dev_type_tv] = ACTIONS(2001),
    [sym_duet_dev_type_utility] = ACTIONS(2001),
    [sym_duet_dev_type_vcr] = ACTIONS(2001),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2001),
    [sym_duet_dev_type_video_processor] = ACTIONS(2001),
    [sym_duet_dev_type_video_projector] = ACTIONS(2001),
    [sym_duet_dev_type_video_wall] = ACTIONS(2001),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2001),
    [sym_duet_dev_type_weather] = ACTIONS(2001),
    [sym_duet_dev_not_polled] = ACTIONS(2001),
    [sym_duet_dev_polled] = ACTIONS(2001),
    [sym_clkmgr_mode_network] = ACTIONS(2001),
    [sym_clkmgr_mode_standalone] = ACTIONS(2001),
    [sym_normal_standby] = ACTIONS(2001),
    [sym_normal_wake] = ACTIONS(2001),
    [sym_ip_multicast_ttl_option] = ACTIONS(2001),
    [sym_ip_ttl_subnet] = ACTIONS(2001),
    [sym_ip_ttl_site] = ACTIONS(2001),
    [sym_ip_ttl_region] = ACTIONS(2001),
    [sym_ip_ttl_continent] = ACTIONS(2001),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2001),
    [sym_ip_nodelay_on] = ACTIONS(2001),
    [sym_ip_nodelay_off] = ACTIONS(2001),
    [sym_amx_error] = ACTIONS(2001),
    [sym_amx_warning] = ACTIONS(2001),
    [sym_amx_info] = ACTIONS(2001),
    [sym_amx_debug] = ACTIONS(2001),
    [sym_netlinx_login_success] = ACTIONS(2001),
    [sym_netlinx_login_fail] = ACTIONS(2001),
    [sym_netlinx_logout] = ACTIONS(2001),
    [sym_null_str] = ACTIONS(2001),
    [sym_smtp_address] = ACTIONS(2001),
    [sym_smtp_port_number] = ACTIONS(2001),
    [sym_smtp_username] = ACTIONS(2001),
    [sym_smtp_password] = ACTIONS(2001),
    [sym_smtp_require_tls] = ACTIONS(2001),
    [sym_smtp_from] = ACTIONS(2001),
    [sym_smtp_tls_true] = ACTIONS(2001),
    [sym_smtp_tls_false] = ACTIONS(2001),
    [sym_listview_on_row_select_event] = ACTIONS(2001),
    [sym_data_structure_unknown] = ACTIONS(2001),
    [sym_data_structure_datafeed] = ACTIONS(2001),
    [sym_data_structure_datarecordset] = ACTIONS(2001),
    [sym_data_structure_datarecord] = ACTIONS(2001),
    [sym_data_structure_datafield] = ACTIONS(2001),
    [sym_data_max_content_field_count] = ACTIONS(2001),
    [sym_data_max_metadata_field_count] = ACTIONS(2001),
    [sym_data_max_id_length] = ACTIONS(2001),
    [sym_data_max_name_length] = ACTIONS(2001),
    [sym_data_max_description_length] = ACTIONS(2001),
    [sym_data_max_type_length] = ACTIONS(2001),
    [sym_data_max_format_length] = ACTIONS(2001),
    [sym_data_max_source_length] = ACTIONS(2001),
    [sym_data_max_value_length] = ACTIONS(2001),
    [sym_data_max_label_length] = ACTIONS(2001),
    [sym_data_max_url_length] = ACTIONS(2001),
    [sym_max_last_login_info_length] = ACTIONS(2001),
    [sym_data_type_unknown] = ACTIONS(2001),
    [sym_data_type_string] = ACTIONS(2001),
    [sym_data_type_datetime] = ACTIONS(2001),
    [sym_data_type_date] = ACTIONS(2001),
    [sym_data_type_time] = ACTIONS(2001),
    [sym_data_type_image] = ACTIONS(2001),
    [sym_data_format_url] = ACTIONS(2001),
    [sym_data_format_phone] = ACTIONS(2001),
    [sym_data_format_email] = ACTIONS(2001),
    [sym_data_format_iso8601] = ACTIONS(2001),
    [sym_auditlog_priv] = ACTIONS(2001),
    [sym_configuration_priv] = ACTIONS(2001),
    [sym_deviceconfig_priv] = ACTIONS(2001),
    [sym_ftp_priv] = ACTIONS(2001),
    [sym_http_priv] = ACTIONS(2001),
    [sym_networkconfig_priv] = ACTIONS(2001),
    [sym_programport_priv] = ACTIONS(2001),
    [sym_remoteui_priv] = ACTIONS(2001),
    [sym_securitycontrol_priv] = ACTIONS(2001),
    [sym_softwaremanagement_priv] = ACTIONS(2001),
    [sym_terminal_priv] = ACTIONS(2001),
    [sym_tpadmin_priv] = ACTIONS(2001),
    [sym_user1_priv] = ACTIONS(2001),
    [sym_user2_priv] = ACTIONS(2001),
    [sym_user3_priv] = ACTIONS(2001),
    [sym_user4_priv] = ACTIONS(2001),
    [sym_usermanagement_priv] = ACTIONS(2001),
    [sym_valid_account] = ACTIONS(2001),
    [sym_err_name_invalid] = ACTIONS(2001),
    [sym_err_password_invalid] = ACTIONS(2001),
    [sym_err_invalid_account] = ACTIONS(2001),
    [sym_err_no_authorization] = ACTIONS(2001),
    [sym_err_invalid_parameter] = ACTIONS(2001),
    [sym_err_user_locked_out] = ACTIONS(2001),
    [sym_err_user_account_expired] = ACTIONS(2001),
    [sym_err_general_error] = ACTIONS(2001),
    [sym_err_authentication_error] = ACTIONS(2001),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2001),
    [sym_tls_validate_certificate] = ACTIONS(2001),
    [sym_tdata] = ACTIONS(2001),
    [sym_tchannel] = ACTIONS(2001),
    [sym_tlevel] = ACTIONS(2001),
    [sym_tbutton] = ACTIONS(2001),
    [sym_ttimeline] = ACTIONS(2001),
    [sym_tcustom] = ACTIONS(2001),
    [sym_url_struct] = ACTIONS(2001),
    [sym_dns_struct] = ACTIONS(2001),
    [sym_ip_address_struct] = ACTIONS(2001),
    [sym_dev_info_struct] = ACTIONS(2001),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2001),
    [sym_data_feed] = ACTIONS(2001),
    [sym_data_field] = ACTIONS(2001),
    [sym_data_record] = ACTIONS(2001),
    [sym_wc_data_feed] = ACTIONS(2001),
    [sym_wc_data_field] = ACTIONS(2001),
    [sym_wc_data_record] = ACTIONS(2001),
    [sym_last_login_info] = ACTIONS(2001),
    [sym____reserved___] = ACTIONS(2001),
    [sym_date] = ACTIONS(2001),
    [sym_day] = ACTIONS(2001),
    [sym_time] = ACTIONS(2001),
    [sym_ldate] = ACTIONS(2001),
    [sym_push_device] = ACTIONS(2001),
    [sym_push_channel] = ACTIONS(2001),
    [sym_push_devchan] = ACTIONS(2001),
    [sym_release_device] = ACTIONS(2001),
    [sym_release_channel] = ACTIONS(2001),
    [sym_release_devchan] = ACTIONS(2001),
    [sym_master_slot] = ACTIONS(2001),
    [sym_get_pulse_time] = ACTIONS(2001),
    [sym_get_timer] = ACTIONS(2001),
    [sym_master_sn] = ACTIONS(2001),
    [sym_system_number] = ACTIONS(2001),
    [sym_button] = ACTIONS(2001),
    [sym_dv_channel] = ACTIONS(2001),
    [sym_data] = ACTIONS(2001),
    [sym_level] = ACTIONS(2001),
    [sym_timeline] = ACTIONS(2001),
    [sym_channel] = ACTIONS(2001),
    [sym_custom] = ACTIONS(2001),
    [sym_atoi] = ACTIONS(2001),
    [sym_atol] = ACTIONS(2001),
    [sym_atof] = ACTIONS(2001),
    [sym_itoa] = ACTIONS(2001),
    [sym_itohex] = ACTIONS(2001),
    [sym_hextoi] = ACTIONS(2001),
    [sym_ftoa] = ACTIONS(2001),
    [sym_find_string] = ACTIONS(2001),
    [sym_left_string] = ACTIONS(2001),
    [sym_right_string] = ACTIONS(2001),
    [sym_mid_string] = ACTIONS(2001),
    [sym_remove_string] = ACTIONS(2001),
    [sym_get_buffer_string] = ACTIONS(2001),
    [sym_get_buffer_char] = ACTIONS(2001),
    [sym_get_multi_buffer_string] = ACTIONS(2001),
    [sym_length_string] = ACTIONS(2001),
    [sym_max_length_string] = ACTIONS(2001),
    [sym_set_length_string] = ACTIONS(2001),
    [sym_compare_string] = ACTIONS(2001),
    [sym_upper_string] = ACTIONS(2001),
    [sym_lower_string] = ACTIONS(2001),
    [sym_get_last] = ACTIONS(2001),
    [sym_date_to_day] = ACTIONS(2001),
    [sym_date_to_month] = ACTIONS(2001),
    [sym_date_to_year] = ACTIONS(2001),
    [sym_time_to_hour] = ACTIONS(2001),
    [sym_time_to_minute] = ACTIONS(2001),
    [sym_time_to_second] = ACTIONS(2001),
    [sym_day_of_week] = ACTIONS(2001),
    [sym_set_pulse_time] = ACTIONS(2001),
    [sym_set_timer] = ACTIONS(2001),
    [sym_file_open] = ACTIONS(2001),
    [sym_file_close] = ACTIONS(2001),
    [sym_file_read] = ACTIONS(2001),
    [sym_file_read_line] = ACTIONS(2001),
    [sym_file_write] = ACTIONS(2001),
    [sym_file_write_line] = ACTIONS(2001),
    [sym_file_copy] = ACTIONS(2001),
    [sym_file_delete] = ACTIONS(2001),
    [sym_file_rename] = ACTIONS(2001),
    [sym_file_dir] = ACTIONS(2001),
    [sym_file_getdir] = ACTIONS(2001),
    [sym_file_setdir] = ACTIONS(2001),
    [sym_file_create_dir] = ACTIONS(2001),
    [sym_file_remove_dir] = ACTIONS(2001),
    [sym_file_seek] = ACTIONS(2001),
    [sym_length_array] = ACTIONS(2001),
    [sym_max_length_array] = ACTIONS(2001),
    [sym_set_length_array] = ACTIONS(2001),
    [sym_ip_client_open] = ACTIONS(2001),
    [sym_ip_client_close] = ACTIONS(2001),
    [sym_ip_server_open] = ACTIONS(2001),
    [sym_ip_server_close] = ACTIONS(2001),
    [sym_ip_bound_client_open] = ACTIONS(2001),
    [sym_ip_mc_server_open] = ACTIONS(2001),
    [sym_ip_set_option] = ACTIONS(2001),
    [sym_ssh_client_open] = ACTIONS(2001),
    [sym_ssh_client_close] = ACTIONS(2001),
    [sym_tls_client_open] = ACTIONS(2001),
    [sym_tls_client_close] = ACTIONS(2001),
    [sym_do_push] = ACTIONS(2001),
    [sym_do_push_timed] = ACTIONS(2001),
    [sym_do_release] = ACTIONS(2001),
    [sym_device_id] = ACTIONS(2001),
    [sym_device_info] = ACTIONS(2001),
    [sym_device_id_string] = ACTIONS(2001),
    [sym_device_standby] = ACTIONS(2001),
    [sym_device_wake] = ACTIONS(2001),
    [sym_timeline_create] = ACTIONS(2001),
    [sym_timeline_kill] = ACTIONS(2001),
    [sym_timeline_reload] = ACTIONS(2001),
    [sym_timeline_pause] = ACTIONS(2001),
    [sym_timeline_restart] = ACTIONS(2001),
    [sym_timeline_set] = ACTIONS(2001),
    [sym_timeline_get] = ACTIONS(2001),
    [sym_timeline_active] = ACTIONS(2001),
    [sym_timeline_dynamic_id] = ACTIONS(2001),
    [sym_variable_to_string] = ACTIONS(2001),
    [sym_string_to_variable] = ACTIONS(2001),
    [sym_length_variable_to_string] = ACTIONS(2001),
    [sym_type_cast] = ACTIONS(2001),
    [sym_variable_to_xml] = ACTIONS(2001),
    [sym_xml_to_variable] = ACTIONS(2001),
    [sym_length_variable_to_xml] = ACTIONS(2001),
    [sym_redirect_string] = ACTIONS(2001),
    [sym_random_number] = ACTIONS(2001),
    [sym_get_unique_id] = ACTIONS(2001),
    [sym_get_system_number] = ACTIONS(2001),
    [sym_set_system_number] = ACTIONS(2001),
    [sym_get_serial_number] = ACTIONS(2001),
    [sym_get_master_build] = ACTIONS(2001),
    [sym_set_validation_code] = ACTIONS(2001),
    [sym_reboot] = ACTIONS(2001),
    [sym_load_duet_module] = ACTIONS(2001),
    [sym_unload_duet_module] = ACTIONS(2001),
    [sym_duet_mem_size_set] = ACTIONS(2001),
    [sym_duet_mem_size_get] = ACTIONS(2001),
    [sym_dynamic_application_device] = ACTIONS(2001),
    [sym_dynamic_polled_port] = ACTIONS(2001),
    [sym_static_port_binding] = ACTIONS(2001),
    [sym_static_ip_binding] = ACTIONS(2001),
    [sym_get_dns_list] = ACTIONS(2001),
    [sym_set_dns_list] = ACTIONS(2001),
    [sym_get_ip_address] = ACTIONS(2001),
    [sym_set_ip_address] = ACTIONS(2001),
    [sym_get_url_list] = ACTIONS(2001),
    [sym_add_url_entry] = ACTIONS(2001),
    [sym_delete_url_entry] = ACTIONS(2001),
    [sym_abs_value] = ACTIONS(2001),
    [sym_max_value] = ACTIONS(2001),
    [sym_min_value] = ACTIONS(2001),
    [sym_exp_value] = ACTIONS(2001),
    [sym_log_value] = ACTIONS(2001),
    [sym_log10_value] = ACTIONS(2001),
    [sym_power_value] = ACTIONS(2001),
    [sym_sqrt_value] = ACTIONS(2001),
    [sym_raw_be] = ACTIONS(2001),
    [sym_raw_le] = ACTIONS(2001),
    [sym_internal_queue_size_set] = ACTIONS(2001),
    [sym_internal_queue_size_get] = ACTIONS(2001),
    [sym_internal_threshold_set] = ACTIONS(2001),
    [sym_internal_threshold_get] = ACTIONS(2001),
    [sym_rebuilt_event] = ACTIONS(2001),
    [sym_do_custom_event] = ACTIONS(2001),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2001),
    [sym_clkmgr_set_clk_source] = ACTIONS(2001),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2001),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2001),
    [sym_clkmgr_get_timezone] = ACTIONS(2001),
    [sym_clkmgr_set_timezone] = ACTIONS(2001),
    [sym_clkmgr_get_resync_period] = ACTIONS(2001),
    [sym_clkmgr_set_resync_period] = ACTIONS(2001),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2001),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2001),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2001),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2001),
    [sym_clkmgr_get_timeservers] = ACTIONS(2001),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2001),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2001),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2001),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2001),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2001),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2001),
    [sym_set_log_level] = ACTIONS(2001),
    [sym_get_log_level] = ACTIONS(2001),
    [sym_amx_log] = ACTIONS(2001),
    [sym_smtp_server_config_set] = ACTIONS(2001),
    [sym_smtp_server_config_get] = ACTIONS(2001),
    [sym_smtp_send] = ACTIONS(2001),
    [sym_validate_netlinx_account] = ACTIONS(2001),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2001),
    [sym_authenticate_certificate] = ACTIONS(2001),
    [sym_audit_netlinx_session_event] = ACTIONS(2001),
    [sym_audit_netlinx_generic_event] = ACTIONS(2001),
    [sym_get_available_flash_disk_space] = ACTIONS(2001),
    [sym_get_max_flash_disk_space] = ACTIONS(2001),
    [sym_data_create_feed] = ACTIONS(2001),
    [sym_data_delete_feed] = ACTIONS(2001),
    [sym_data_publish_feed] = ACTIONS(2001),
    [sym_data_get_published_feed] = ACTIONS(2001),
    [sym_data_add_record] = ACTIONS(2001),
    [sym_data_get_event_record] = ACTIONS(2001),
    [sym__wc_data_create_feed] = ACTIONS(2001),
    [sym__wc_data_add_record] = ACTIONS(2001),
    [sym__wc_data_get_event_record] = ACTIONS(2001),
    [sym_format] = ACTIONS(2001),
    [sym_astro_clock] = ACTIONS(2001),
    [sym_set_outdoor_temperature] = ACTIONS(2001),
    [sym_set_virtual_level_count] = ACTIONS(2001),
    [sym_set_virtual_channel_count] = ACTIONS(2001),
    [sym_set_virtual_port_count] = ACTIONS(2001),
    [sym__wc_encode_int] = ACTIONS(2001),
    [sym__wc_decode_int] = ACTIONS(2001),
    [sym__wc_int] = ACTIONS(2001),
    [sym__wc_to_ch_int] = ACTIONS(2001),
    [sym__ch_to_wc_int] = ACTIONS(2001),
    [sym__wc_find_string_int] = ACTIONS(2001),
    [sym__wc_left_string_int] = ACTIONS(2001),
    [sym__wc_lower_string_int] = ACTIONS(2001),
    [sym__wc_mid_string_int] = ACTIONS(2001),
    [sym__wc_remove_string_int] = ACTIONS(2001),
    [sym__wc_right_string_int] = ACTIONS(2001),
    [sym__wc_upper_string_int] = ACTIONS(2001),
    [sym__wc_compare_string_int] = ACTIONS(2001),
    [sym__wc_get_buffer_char_int] = ACTIONS(2001),
    [sym__wc_get_buffer_string_int] = ACTIONS(2001),
    [sym__wc_concat_string_int] = ACTIONS(2001),
    [sym__wc_file_open_int] = ACTIONS(2001),
    [sym__wc_file_close_int] = ACTIONS(2001),
    [sym__wc_file_read_int] = ACTIONS(2001),
    [sym__wc_file_read_line_int] = ACTIONS(2001),
    [sym__wc_file_write_int] = ACTIONS(2001),
    [sym__wc_file_write_line_int] = ACTIONS(2001),
    [aux_sym_preproc_include_token1] = ACTIONS(2001),
    [aux_sym_preproc_define_token1] = ACTIONS(2003),
    [aux_sym_preproc_warn_token1] = ACTIONS(2003),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2003),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2003),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2003),
    [anon_sym_LBRACK] = ACTIONS(2003),
    [anon_sym_LBRACE] = ACTIONS(2003),
    [anon_sym_RBRACE] = ACTIONS(2003),
    [anon_sym_LPAREN] = ACTIONS(2001),
    [anon_sym_BANG] = ACTIONS(2003),
    [anon_sym_TILDE] = ACTIONS(2003),
    [anon_sym_DASH] = ACTIONS(2001),
    [anon_sym_PLUS] = ACTIONS(2001),
    [anon_sym_DASH_DASH] = ACTIONS(2003),
    [anon_sym_PLUS_PLUS] = ACTIONS(2003),
    [anon_sym_DQUOTE] = ACTIONS(2003),
    [anon_sym_SQUOTE] = ACTIONS(2003),
    [sym_number_literal] = ACTIONS(2003),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(525)] = {
    [sym_identifier] = ACTIONS(2065),
    [sym_define_call_keyword] = ACTIONS(2065),
    [sym_define_function_keyword] = ACTIONS(2065),
    [sym_define_library_function_keyword] = ACTIONS(2065),
    [sym_define_module_keyword] = ACTIONS(2065),
    [sym_char_keyword] = ACTIONS(2065),
    [sym_widechar_keyword] = ACTIONS(2065),
    [sym_integer_keyword] = ACTIONS(2065),
    [sym_sinteger_keyword] = ACTIONS(2065),
    [sym_long_keyword] = ACTIONS(2065),
    [sym_slong_keyword] = ACTIONS(2065),
    [sym_float_keyword] = ACTIONS(2065),
    [sym_double_keyword] = ACTIONS(2065),
    [sym_variant_keyword] = ACTIONS(2065),
    [sym_variantarray_keyword] = ACTIONS(2065),
    [sym_local_var_keyword] = ACTIONS(2065),
    [sym_stack_var_keyword] = ACTIONS(2065),
    [sym_constant_keyword] = ACTIONS(2065),
    [sym_volatile_keyword] = ACTIONS(2065),
    [sym_non_volatile_keyword] = ACTIONS(2065),
    [sym_persistent_keyword] = ACTIONS(2065),
    [sym_dev_keyword] = ACTIONS(2065),
    [sym_devlev_keyword] = ACTIONS(2065),
    [sym_devchan_keyword] = ACTIONS(2065),
    [sym_if_keyword] = ACTIONS(2065),
    [sym_switch_keyword] = ACTIONS(2065),
    [sym_case_keyword] = ACTIONS(2065),
    [sym_default_keyword] = ACTIONS(2065),
    [sym_while_keyword] = ACTIONS(2065),
    [sym_for_keyword] = ACTIONS(2065),
    [sym_break_keyword] = ACTIONS(2065),
    [sym_continue_keyword] = ACTIONS(2065),
    [sym_return_keyword] = ACTIONS(2065),
    [sym_select_keyword] = ACTIONS(2065),
    [sym_active_keyword] = ACTIONS(2065),
    [sym_struct_keyword] = ACTIONS(2065),
    [sym_structure_keyword] = ACTIONS(2065),
    [aux_sym_on_keyword_token1] = ACTIONS(2065),
    [aux_sym_off_keyword_token1] = ACTIONS(2065),
    [sym_send_level_keyword] = ACTIONS(2065),
    [sym_send_string_keyword] = ACTIONS(2065),
    [sym_send_command_keyword] = ACTIONS(2065),
    [sym_clear_buffer_keyword] = ACTIONS(2065),
    [sym_create_buffer_keyword] = ACTIONS(2065),
    [sym_create_multi_buffer_keyword] = ACTIONS(2065),
    [sym_call_keyword] = ACTIONS(2065),
    [sym_system_call_keyword] = ACTIONS(2065),
    [sym_devchan_to_keyword] = ACTIONS(2065),
    [sym_devchan_min_to_keyword] = ACTIONS(2065),
    [sym_devchan_total_off_keyword] = ACTIONS(2065),
    [sym_devchan_pulse_keyword] = ACTIONS(2065),
    [sym_wait_keyword] = ACTIONS(2065),
    [sym_cancel_wait_keyword] = ACTIONS(2065),
    [sym_wait_until_keyword] = ACTIONS(2065),
    [sym_cancel_wait_until_keyword] = ACTIONS(2065),
    [sym_cancel_all_wait_keyword] = ACTIONS(2065),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2065),
    [sym_not] = ACTIONS(2065),
    [sym_bnot] = ACTIONS(2065),
    [sym___file__] = ACTIONS(2065),
    [sym___date__] = ACTIONS(2065),
    [sym___ldate__] = ACTIONS(2065),
    [sym___time__] = ACTIONS(2065),
    [sym___line__] = ACTIONS(2065),
    [sym___name__] = ACTIONS(2065),
    [sym___netlinx__] = ACTIONS(2065),
    [sym_netlinx_axi_version] = ACTIONS(2065),
    [sym_url_flg_tcp] = ACTIONS(2065),
    [sym_url_flg_accinfopresent] = ACTIONS(2065),
    [sym_url_flg_temp] = ACTIONS(2065),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2065),
    [sym_url_flg_stat_mask] = ACTIONS(2065),
    [sym_url_flg_stat_lookup] = ACTIONS(2065),
    [sym_url_flg_stat_connecting] = ACTIONS(2065),
    [sym_url_flg_stat_waiting] = ACTIONS(2065),
    [sym_url_flg_stat_connected] = ACTIONS(2065),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2065),
    [sym_first_virtual_device] = ACTIONS(2065),
    [sym_first_local_port] = ACTIONS(2065),
    [sym_dynamic_virtual_device] = ACTIONS(2065),
    [sym_timeline_once] = ACTIONS(2065),
    [sym_timeline_repeat] = ACTIONS(2065),
    [sym_timeline_absolute] = ACTIONS(2065),
    [sym_timeline_relative] = ACTIONS(2065),
    [sym_do_push_timed_infinite] = ACTIONS(2065),
    [sym_source_type_no_address] = ACTIONS(2065),
    [sym_source_type_neuron_id] = ACTIONS(2065),
    [sym_source_type_ip_address] = ACTIONS(2065),
    [sym_source_type_axlink] = ACTIONS(2065),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2065),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2065),
    [sym_source_type_ip_socket_address] = ACTIONS(2065),
    [sym_source_type_rs232] = ACTIONS(2065),
    [sym_source_type_internal] = ACTIONS(2065),
    [sym_source_type_mac_address] = ACTIONS(2065),
    [sym_source_type_ipv4_port] = ACTIONS(2065),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2065),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2065),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2065),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2065),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2065),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2065),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2065),
    [sym_true] = ACTIONS(2065),
    [sym_false] = ACTIONS(2065),
    [sym_file_read_only] = ACTIONS(2065),
    [sym_file_rw_new] = ACTIONS(2065),
    [sym_file_rw_append] = ACTIONS(2065),
    [sym_ip_tcp] = ACTIONS(2065),
    [sym_ip_udp] = ACTIONS(2065),
    [sym_ip_udp_2way] = ACTIONS(2065),
    [sym_xml_encode_types] = ACTIONS(2065),
    [sym_xml_encode_char_as_list] = ACTIONS(2065),
    [sym_xml_encode_le] = ACTIONS(2065),
    [sym_xml_decode_types] = ACTIONS(2065),
    [sym_xml_decode_no_preserve] = ACTIONS(2065),
    [sym_duet_dev_type_display_device] = ACTIONS(2065),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2065),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2065),
    [sym_duet_dev_type_io_device] = ACTIONS(2065),
    [sym_duet_dev_type_relay_device] = ACTIONS(2065),
    [sym_duet_dev_type_ups] = ACTIONS(2065),
    [sym_duet_dev_type_amplifier] = ACTIONS(2065),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2065),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2065),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2065),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2065),
    [sym_duet_dev_type_camera] = ACTIONS(2065),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2065),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2065),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2065),
    [sym_duet_dev_type_dss] = ACTIONS(2065),
    [sym_duet_dev_type_dvr] = ACTIONS(2065),
    [sym_duet_dev_type_disc_device] = ACTIONS(2065),
    [sym_duet_dev_type_document_camera] = ACTIONS(2065),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2065),
    [sym_duet_dev_type_hvac] = ACTIONS(2065),
    [sym_duet_dev_type_keypad] = ACTIONS(2065),
    [sym_duet_dev_type_light] = ACTIONS(2065),
    [sym_duet_dev_type_monitor] = ACTIONS(2065),
    [sym_duet_dev_type_motor] = ACTIONS(2065),
    [sym_duet_dev_type_multi_window] = ACTIONS(2065),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2065),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2065),
    [sym_duet_dev_type_receiver] = ACTIONS(2065),
    [sym_duet_dev_type_security_system] = ACTIONS(2065),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2065),
    [sym_duet_dev_type_settop_box] = ACTIONS(2065),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2065),
    [sym_duet_dev_type_switcher] = ACTIONS(2065),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2065),
    [sym_duet_dev_type_tv] = ACTIONS(2065),
    [sym_duet_dev_type_utility] = ACTIONS(2065),
    [sym_duet_dev_type_vcr] = ACTIONS(2065),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2065),
    [sym_duet_dev_type_video_processor] = ACTIONS(2065),
    [sym_duet_dev_type_video_projector] = ACTIONS(2065),
    [sym_duet_dev_type_video_wall] = ACTIONS(2065),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2065),
    [sym_duet_dev_type_weather] = ACTIONS(2065),
    [sym_duet_dev_not_polled] = ACTIONS(2065),
    [sym_duet_dev_polled] = ACTIONS(2065),
    [sym_clkmgr_mode_network] = ACTIONS(2065),
    [sym_clkmgr_mode_standalone] = ACTIONS(2065),
    [sym_normal_standby] = ACTIONS(2065),
    [sym_normal_wake] = ACTIONS(2065),
    [sym_ip_multicast_ttl_option] = ACTIONS(2065),
    [sym_ip_ttl_subnet] = ACTIONS(2065),
    [sym_ip_ttl_site] = ACTIONS(2065),
    [sym_ip_ttl_region] = ACTIONS(2065),
    [sym_ip_ttl_continent] = ACTIONS(2065),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2065),
    [sym_ip_nodelay_on] = ACTIONS(2065),
    [sym_ip_nodelay_off] = ACTIONS(2065),
    [sym_amx_error] = ACTIONS(2065),
    [sym_amx_warning] = ACTIONS(2065),
    [sym_amx_info] = ACTIONS(2065),
    [sym_amx_debug] = ACTIONS(2065),
    [sym_netlinx_login_success] = ACTIONS(2065),
    [sym_netlinx_login_fail] = ACTIONS(2065),
    [sym_netlinx_logout] = ACTIONS(2065),
    [sym_null_str] = ACTIONS(2065),
    [sym_smtp_address] = ACTIONS(2065),
    [sym_smtp_port_number] = ACTIONS(2065),
    [sym_smtp_username] = ACTIONS(2065),
    [sym_smtp_password] = ACTIONS(2065),
    [sym_smtp_require_tls] = ACTIONS(2065),
    [sym_smtp_from] = ACTIONS(2065),
    [sym_smtp_tls_true] = ACTIONS(2065),
    [sym_smtp_tls_false] = ACTIONS(2065),
    [sym_listview_on_row_select_event] = ACTIONS(2065),
    [sym_data_structure_unknown] = ACTIONS(2065),
    [sym_data_structure_datafeed] = ACTIONS(2065),
    [sym_data_structure_datarecordset] = ACTIONS(2065),
    [sym_data_structure_datarecord] = ACTIONS(2065),
    [sym_data_structure_datafield] = ACTIONS(2065),
    [sym_data_max_content_field_count] = ACTIONS(2065),
    [sym_data_max_metadata_field_count] = ACTIONS(2065),
    [sym_data_max_id_length] = ACTIONS(2065),
    [sym_data_max_name_length] = ACTIONS(2065),
    [sym_data_max_description_length] = ACTIONS(2065),
    [sym_data_max_type_length] = ACTIONS(2065),
    [sym_data_max_format_length] = ACTIONS(2065),
    [sym_data_max_source_length] = ACTIONS(2065),
    [sym_data_max_value_length] = ACTIONS(2065),
    [sym_data_max_label_length] = ACTIONS(2065),
    [sym_data_max_url_length] = ACTIONS(2065),
    [sym_max_last_login_info_length] = ACTIONS(2065),
    [sym_data_type_unknown] = ACTIONS(2065),
    [sym_data_type_string] = ACTIONS(2065),
    [sym_data_type_datetime] = ACTIONS(2065),
    [sym_data_type_date] = ACTIONS(2065),
    [sym_data_type_time] = ACTIONS(2065),
    [sym_data_type_image] = ACTIONS(2065),
    [sym_data_format_url] = ACTIONS(2065),
    [sym_data_format_phone] = ACTIONS(2065),
    [sym_data_format_email] = ACTIONS(2065),
    [sym_data_format_iso8601] = ACTIONS(2065),
    [sym_auditlog_priv] = ACTIONS(2065),
    [sym_configuration_priv] = ACTIONS(2065),
    [sym_deviceconfig_priv] = ACTIONS(2065),
    [sym_ftp_priv] = ACTIONS(2065),
    [sym_http_priv] = ACTIONS(2065),
    [sym_networkconfig_priv] = ACTIONS(2065),
    [sym_programport_priv] = ACTIONS(2065),
    [sym_remoteui_priv] = ACTIONS(2065),
    [sym_securitycontrol_priv] = ACTIONS(2065),
    [sym_softwaremanagement_priv] = ACTIONS(2065),
    [sym_terminal_priv] = ACTIONS(2065),
    [sym_tpadmin_priv] = ACTIONS(2065),
    [sym_user1_priv] = ACTIONS(2065),
    [sym_user2_priv] = ACTIONS(2065),
    [sym_user3_priv] = ACTIONS(2065),
    [sym_user4_priv] = ACTIONS(2065),
    [sym_usermanagement_priv] = ACTIONS(2065),
    [sym_valid_account] = ACTIONS(2065),
    [sym_err_name_invalid] = ACTIONS(2065),
    [sym_err_password_invalid] = ACTIONS(2065),
    [sym_err_invalid_account] = ACTIONS(2065),
    [sym_err_no_authorization] = ACTIONS(2065),
    [sym_err_invalid_parameter] = ACTIONS(2065),
    [sym_err_user_locked_out] = ACTIONS(2065),
    [sym_err_user_account_expired] = ACTIONS(2065),
    [sym_err_general_error] = ACTIONS(2065),
    [sym_err_authentication_error] = ACTIONS(2065),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2065),
    [sym_tls_validate_certificate] = ACTIONS(2065),
    [sym_tdata] = ACTIONS(2065),
    [sym_tchannel] = ACTIONS(2065),
    [sym_tlevel] = ACTIONS(2065),
    [sym_tbutton] = ACTIONS(2065),
    [sym_ttimeline] = ACTIONS(2065),
    [sym_tcustom] = ACTIONS(2065),
    [sym_url_struct] = ACTIONS(2065),
    [sym_dns_struct] = ACTIONS(2065),
    [sym_ip_address_struct] = ACTIONS(2065),
    [sym_dev_info_struct] = ACTIONS(2065),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2065),
    [sym_data_feed] = ACTIONS(2065),
    [sym_data_field] = ACTIONS(2065),
    [sym_data_record] = ACTIONS(2065),
    [sym_wc_data_feed] = ACTIONS(2065),
    [sym_wc_data_field] = ACTIONS(2065),
    [sym_wc_data_record] = ACTIONS(2065),
    [sym_last_login_info] = ACTIONS(2065),
    [sym____reserved___] = ACTIONS(2065),
    [sym_date] = ACTIONS(2065),
    [sym_day] = ACTIONS(2065),
    [sym_time] = ACTIONS(2065),
    [sym_ldate] = ACTIONS(2065),
    [sym_push_device] = ACTIONS(2065),
    [sym_push_channel] = ACTIONS(2065),
    [sym_push_devchan] = ACTIONS(2065),
    [sym_release_device] = ACTIONS(2065),
    [sym_release_channel] = ACTIONS(2065),
    [sym_release_devchan] = ACTIONS(2065),
    [sym_master_slot] = ACTIONS(2065),
    [sym_get_pulse_time] = ACTIONS(2065),
    [sym_get_timer] = ACTIONS(2065),
    [sym_master_sn] = ACTIONS(2065),
    [sym_system_number] = ACTIONS(2065),
    [sym_button] = ACTIONS(2065),
    [sym_dv_channel] = ACTIONS(2065),
    [sym_data] = ACTIONS(2065),
    [sym_level] = ACTIONS(2065),
    [sym_timeline] = ACTIONS(2065),
    [sym_channel] = ACTIONS(2065),
    [sym_custom] = ACTIONS(2065),
    [sym_atoi] = ACTIONS(2065),
    [sym_atol] = ACTIONS(2065),
    [sym_atof] = ACTIONS(2065),
    [sym_itoa] = ACTIONS(2065),
    [sym_itohex] = ACTIONS(2065),
    [sym_hextoi] = ACTIONS(2065),
    [sym_ftoa] = ACTIONS(2065),
    [sym_find_string] = ACTIONS(2065),
    [sym_left_string] = ACTIONS(2065),
    [sym_right_string] = ACTIONS(2065),
    [sym_mid_string] = ACTIONS(2065),
    [sym_remove_string] = ACTIONS(2065),
    [sym_get_buffer_string] = ACTIONS(2065),
    [sym_get_buffer_char] = ACTIONS(2065),
    [sym_get_multi_buffer_string] = ACTIONS(2065),
    [sym_length_string] = ACTIONS(2065),
    [sym_max_length_string] = ACTIONS(2065),
    [sym_set_length_string] = ACTIONS(2065),
    [sym_compare_string] = ACTIONS(2065),
    [sym_upper_string] = ACTIONS(2065),
    [sym_lower_string] = ACTIONS(2065),
    [sym_get_last] = ACTIONS(2065),
    [sym_date_to_day] = ACTIONS(2065),
    [sym_date_to_month] = ACTIONS(2065),
    [sym_date_to_year] = ACTIONS(2065),
    [sym_time_to_hour] = ACTIONS(2065),
    [sym_time_to_minute] = ACTIONS(2065),
    [sym_time_to_second] = ACTIONS(2065),
    [sym_day_of_week] = ACTIONS(2065),
    [sym_set_pulse_time] = ACTIONS(2065),
    [sym_set_timer] = ACTIONS(2065),
    [sym_file_open] = ACTIONS(2065),
    [sym_file_close] = ACTIONS(2065),
    [sym_file_read] = ACTIONS(2065),
    [sym_file_read_line] = ACTIONS(2065),
    [sym_file_write] = ACTIONS(2065),
    [sym_file_write_line] = ACTIONS(2065),
    [sym_file_copy] = ACTIONS(2065),
    [sym_file_delete] = ACTIONS(2065),
    [sym_file_rename] = ACTIONS(2065),
    [sym_file_dir] = ACTIONS(2065),
    [sym_file_getdir] = ACTIONS(2065),
    [sym_file_setdir] = ACTIONS(2065),
    [sym_file_create_dir] = ACTIONS(2065),
    [sym_file_remove_dir] = ACTIONS(2065),
    [sym_file_seek] = ACTIONS(2065),
    [sym_length_array] = ACTIONS(2065),
    [sym_max_length_array] = ACTIONS(2065),
    [sym_set_length_array] = ACTIONS(2065),
    [sym_ip_client_open] = ACTIONS(2065),
    [sym_ip_client_close] = ACTIONS(2065),
    [sym_ip_server_open] = ACTIONS(2065),
    [sym_ip_server_close] = ACTIONS(2065),
    [sym_ip_bound_client_open] = ACTIONS(2065),
    [sym_ip_mc_server_open] = ACTIONS(2065),
    [sym_ip_set_option] = ACTIONS(2065),
    [sym_ssh_client_open] = ACTIONS(2065),
    [sym_ssh_client_close] = ACTIONS(2065),
    [sym_tls_client_open] = ACTIONS(2065),
    [sym_tls_client_close] = ACTIONS(2065),
    [sym_do_push] = ACTIONS(2065),
    [sym_do_push_timed] = ACTIONS(2065),
    [sym_do_release] = ACTIONS(2065),
    [sym_device_id] = ACTIONS(2065),
    [sym_device_info] = ACTIONS(2065),
    [sym_device_id_string] = ACTIONS(2065),
    [sym_device_standby] = ACTIONS(2065),
    [sym_device_wake] = ACTIONS(2065),
    [sym_timeline_create] = ACTIONS(2065),
    [sym_timeline_kill] = ACTIONS(2065),
    [sym_timeline_reload] = ACTIONS(2065),
    [sym_timeline_pause] = ACTIONS(2065),
    [sym_timeline_restart] = ACTIONS(2065),
    [sym_timeline_set] = ACTIONS(2065),
    [sym_timeline_get] = ACTIONS(2065),
    [sym_timeline_active] = ACTIONS(2065),
    [sym_timeline_dynamic_id] = ACTIONS(2065),
    [sym_variable_to_string] = ACTIONS(2065),
    [sym_string_to_variable] = ACTIONS(2065),
    [sym_length_variable_to_string] = ACTIONS(2065),
    [sym_type_cast] = ACTIONS(2065),
    [sym_variable_to_xml] = ACTIONS(2065),
    [sym_xml_to_variable] = ACTIONS(2065),
    [sym_length_variable_to_xml] = ACTIONS(2065),
    [sym_redirect_string] = ACTIONS(2065),
    [sym_random_number] = ACTIONS(2065),
    [sym_get_unique_id] = ACTIONS(2065),
    [sym_get_system_number] = ACTIONS(2065),
    [sym_set_system_number] = ACTIONS(2065),
    [sym_get_serial_number] = ACTIONS(2065),
    [sym_get_master_build] = ACTIONS(2065),
    [sym_set_validation_code] = ACTIONS(2065),
    [sym_reboot] = ACTIONS(2065),
    [sym_load_duet_module] = ACTIONS(2065),
    [sym_unload_duet_module] = ACTIONS(2065),
    [sym_duet_mem_size_set] = ACTIONS(2065),
    [sym_duet_mem_size_get] = ACTIONS(2065),
    [sym_dynamic_application_device] = ACTIONS(2065),
    [sym_dynamic_polled_port] = ACTIONS(2065),
    [sym_static_port_binding] = ACTIONS(2065),
    [sym_static_ip_binding] = ACTIONS(2065),
    [sym_get_dns_list] = ACTIONS(2065),
    [sym_set_dns_list] = ACTIONS(2065),
    [sym_get_ip_address] = ACTIONS(2065),
    [sym_set_ip_address] = ACTIONS(2065),
    [sym_get_url_list] = ACTIONS(2065),
    [sym_add_url_entry] = ACTIONS(2065),
    [sym_delete_url_entry] = ACTIONS(2065),
    [sym_abs_value] = ACTIONS(2065),
    [sym_max_value] = ACTIONS(2065),
    [sym_min_value] = ACTIONS(2065),
    [sym_exp_value] = ACTIONS(2065),
    [sym_log_value] = ACTIONS(2065),
    [sym_log10_value] = ACTIONS(2065),
    [sym_power_value] = ACTIONS(2065),
    [sym_sqrt_value] = ACTIONS(2065),
    [sym_raw_be] = ACTIONS(2065),
    [sym_raw_le] = ACTIONS(2065),
    [sym_internal_queue_size_set] = ACTIONS(2065),
    [sym_internal_queue_size_get] = ACTIONS(2065),
    [sym_internal_threshold_set] = ACTIONS(2065),
    [sym_internal_threshold_get] = ACTIONS(2065),
    [sym_rebuilt_event] = ACTIONS(2065),
    [sym_do_custom_event] = ACTIONS(2065),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2065),
    [sym_clkmgr_set_clk_source] = ACTIONS(2065),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2065),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2065),
    [sym_clkmgr_get_timezone] = ACTIONS(2065),
    [sym_clkmgr_set_timezone] = ACTIONS(2065),
    [sym_clkmgr_get_resync_period] = ACTIONS(2065),
    [sym_clkmgr_set_resync_period] = ACTIONS(2065),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2065),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2065),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2065),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2065),
    [sym_clkmgr_get_timeservers] = ACTIONS(2065),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2065),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2065),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2065),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2065),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2065),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2065),
    [sym_set_log_level] = ACTIONS(2065),
    [sym_get_log_level] = ACTIONS(2065),
    [sym_amx_log] = ACTIONS(2065),
    [sym_smtp_server_config_set] = ACTIONS(2065),
    [sym_smtp_server_config_get] = ACTIONS(2065),
    [sym_smtp_send] = ACTIONS(2065),
    [sym_validate_netlinx_account] = ACTIONS(2065),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2065),
    [sym_authenticate_certificate] = ACTIONS(2065),
    [sym_audit_netlinx_session_event] = ACTIONS(2065),
    [sym_audit_netlinx_generic_event] = ACTIONS(2065),
    [sym_get_available_flash_disk_space] = ACTIONS(2065),
    [sym_get_max_flash_disk_space] = ACTIONS(2065),
    [sym_data_create_feed] = ACTIONS(2065),
    [sym_data_delete_feed] = ACTIONS(2065),
    [sym_data_publish_feed] = ACTIONS(2065),
    [sym_data_get_published_feed] = ACTIONS(2065),
    [sym_data_add_record] = ACTIONS(2065),
    [sym_data_get_event_record] = ACTIONS(2065),
    [sym__wc_data_create_feed] = ACTIONS(2065),
    [sym__wc_data_add_record] = ACTIONS(2065),
    [sym__wc_data_get_event_record] = ACTIONS(2065),
    [sym_format] = ACTIONS(2065),
    [sym_astro_clock] = ACTIONS(2065),
    [sym_set_outdoor_temperature] = ACTIONS(2065),
    [sym_set_virtual_level_count] = ACTIONS(2065),
    [sym_set_virtual_channel_count] = ACTIONS(2065),
    [sym_set_virtual_port_count] = ACTIONS(2065),
    [sym__wc_encode_int] = ACTIONS(2065),
    [sym__wc_decode_int] = ACTIONS(2065),
    [sym__wc_int] = ACTIONS(2065),
    [sym__wc_to_ch_int] = ACTIONS(2065),
    [sym__ch_to_wc_int] = ACTIONS(2065),
    [sym__wc_find_string_int] = ACTIONS(2065),
    [sym__wc_left_string_int] = ACTIONS(2065),
    [sym__wc_lower_string_int] = ACTIONS(2065),
    [sym__wc_mid_string_int] = ACTIONS(2065),
    [sym__wc_remove_string_int] = ACTIONS(2065),
    [sym__wc_right_string_int] = ACTIONS(2065),
    [sym__wc_upper_string_int] = ACTIONS(2065),
    [sym__wc_compare_string_int] = ACTIONS(2065),
    [sym__wc_get_buffer_char_int] = ACTIONS(2065),
    [sym__wc_get_buffer_string_int] = ACTIONS(2065),
    [sym__wc_concat_string_int] = ACTIONS(2065),
    [sym__wc_file_open_int] = ACTIONS(2065),
    [sym__wc_file_close_int] = ACTIONS(2065),
    [sym__wc_file_read_int] = ACTIONS(2065),
    [sym__wc_file_read_line_int] = ACTIONS(2065),
    [sym__wc_file_write_int] = ACTIONS(2065),
    [sym__wc_file_write_line_int] = ACTIONS(2065),
    [aux_sym_preproc_include_token1] = ACTIONS(2065),
    [aux_sym_preproc_define_token1] = ACTIONS(2067),
    [aux_sym_preproc_warn_token1] = ACTIONS(2067),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2067),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2067),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2067),
    [anon_sym_LBRACK] = ACTIONS(2067),
    [anon_sym_LBRACE] = ACTIONS(2067),
    [anon_sym_RBRACE] = ACTIONS(2067),
    [anon_sym_LPAREN] = ACTIONS(2065),
    [anon_sym_BANG] = ACTIONS(2067),
    [anon_sym_TILDE] = ACTIONS(2067),
    [anon_sym_DASH] = ACTIONS(2065),
    [anon_sym_PLUS] = ACTIONS(2065),
    [anon_sym_DASH_DASH] = ACTIONS(2067),
    [anon_sym_PLUS_PLUS] = ACTIONS(2067),
    [anon_sym_DQUOTE] = ACTIONS(2067),
    [anon_sym_SQUOTE] = ACTIONS(2067),
    [sym_number_literal] = ACTIONS(2067),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(526)] = {
    [sym_identifier] = ACTIONS(1901),
    [sym_define_call_keyword] = ACTIONS(1901),
    [sym_define_function_keyword] = ACTIONS(1901),
    [sym_define_library_function_keyword] = ACTIONS(1901),
    [sym_define_module_keyword] = ACTIONS(1901),
    [sym_char_keyword] = ACTIONS(1901),
    [sym_widechar_keyword] = ACTIONS(1901),
    [sym_integer_keyword] = ACTIONS(1901),
    [sym_sinteger_keyword] = ACTIONS(1901),
    [sym_long_keyword] = ACTIONS(1901),
    [sym_slong_keyword] = ACTIONS(1901),
    [sym_float_keyword] = ACTIONS(1901),
    [sym_double_keyword] = ACTIONS(1901),
    [sym_variant_keyword] = ACTIONS(1901),
    [sym_variantarray_keyword] = ACTIONS(1901),
    [sym_local_var_keyword] = ACTIONS(1901),
    [sym_stack_var_keyword] = ACTIONS(1901),
    [sym_constant_keyword] = ACTIONS(1901),
    [sym_volatile_keyword] = ACTIONS(1901),
    [sym_non_volatile_keyword] = ACTIONS(1901),
    [sym_persistent_keyword] = ACTIONS(1901),
    [sym_dev_keyword] = ACTIONS(1901),
    [sym_devlev_keyword] = ACTIONS(1901),
    [sym_devchan_keyword] = ACTIONS(1901),
    [sym_if_keyword] = ACTIONS(1901),
    [sym_switch_keyword] = ACTIONS(1901),
    [sym_case_keyword] = ACTIONS(1901),
    [sym_default_keyword] = ACTIONS(1901),
    [sym_while_keyword] = ACTIONS(1901),
    [sym_for_keyword] = ACTIONS(1901),
    [sym_break_keyword] = ACTIONS(1901),
    [sym_continue_keyword] = ACTIONS(1901),
    [sym_return_keyword] = ACTIONS(1901),
    [sym_select_keyword] = ACTIONS(1901),
    [sym_active_keyword] = ACTIONS(1901),
    [sym_struct_keyword] = ACTIONS(1901),
    [sym_structure_keyword] = ACTIONS(1901),
    [aux_sym_on_keyword_token1] = ACTIONS(1901),
    [aux_sym_off_keyword_token1] = ACTIONS(1901),
    [sym_send_level_keyword] = ACTIONS(1901),
    [sym_send_string_keyword] = ACTIONS(1901),
    [sym_send_command_keyword] = ACTIONS(1901),
    [sym_clear_buffer_keyword] = ACTIONS(1901),
    [sym_create_buffer_keyword] = ACTIONS(1901),
    [sym_create_multi_buffer_keyword] = ACTIONS(1901),
    [sym_call_keyword] = ACTIONS(1901),
    [sym_system_call_keyword] = ACTIONS(1901),
    [sym_devchan_to_keyword] = ACTIONS(1901),
    [sym_devchan_min_to_keyword] = ACTIONS(1901),
    [sym_devchan_total_off_keyword] = ACTIONS(1901),
    [sym_devchan_pulse_keyword] = ACTIONS(1901),
    [sym_wait_keyword] = ACTIONS(1901),
    [sym_cancel_wait_keyword] = ACTIONS(1901),
    [sym_wait_until_keyword] = ACTIONS(1901),
    [sym_cancel_wait_until_keyword] = ACTIONS(1901),
    [sym_cancel_all_wait_keyword] = ACTIONS(1901),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1901),
    [sym_not] = ACTIONS(1901),
    [sym_bnot] = ACTIONS(1901),
    [sym___file__] = ACTIONS(1901),
    [sym___date__] = ACTIONS(1901),
    [sym___ldate__] = ACTIONS(1901),
    [sym___time__] = ACTIONS(1901),
    [sym___line__] = ACTIONS(1901),
    [sym___name__] = ACTIONS(1901),
    [sym___netlinx__] = ACTIONS(1901),
    [sym_netlinx_axi_version] = ACTIONS(1901),
    [sym_url_flg_tcp] = ACTIONS(1901),
    [sym_url_flg_accinfopresent] = ACTIONS(1901),
    [sym_url_flg_temp] = ACTIONS(1901),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1901),
    [sym_url_flg_stat_mask] = ACTIONS(1901),
    [sym_url_flg_stat_lookup] = ACTIONS(1901),
    [sym_url_flg_stat_connecting] = ACTIONS(1901),
    [sym_url_flg_stat_waiting] = ACTIONS(1901),
    [sym_url_flg_stat_connected] = ACTIONS(1901),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1901),
    [sym_first_virtual_device] = ACTIONS(1901),
    [sym_first_local_port] = ACTIONS(1901),
    [sym_dynamic_virtual_device] = ACTIONS(1901),
    [sym_timeline_once] = ACTIONS(1901),
    [sym_timeline_repeat] = ACTIONS(1901),
    [sym_timeline_absolute] = ACTIONS(1901),
    [sym_timeline_relative] = ACTIONS(1901),
    [sym_do_push_timed_infinite] = ACTIONS(1901),
    [sym_source_type_no_address] = ACTIONS(1901),
    [sym_source_type_neuron_id] = ACTIONS(1901),
    [sym_source_type_ip_address] = ACTIONS(1901),
    [sym_source_type_axlink] = ACTIONS(1901),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1901),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1901),
    [sym_source_type_ip_socket_address] = ACTIONS(1901),
    [sym_source_type_rs232] = ACTIONS(1901),
    [sym_source_type_internal] = ACTIONS(1901),
    [sym_source_type_mac_address] = ACTIONS(1901),
    [sym_source_type_ipv4_port] = ACTIONS(1901),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1901),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1901),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1901),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1901),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1901),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1901),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1901),
    [sym_true] = ACTIONS(1901),
    [sym_false] = ACTIONS(1901),
    [sym_file_read_only] = ACTIONS(1901),
    [sym_file_rw_new] = ACTIONS(1901),
    [sym_file_rw_append] = ACTIONS(1901),
    [sym_ip_tcp] = ACTIONS(1901),
    [sym_ip_udp] = ACTIONS(1901),
    [sym_ip_udp_2way] = ACTIONS(1901),
    [sym_xml_encode_types] = ACTIONS(1901),
    [sym_xml_encode_char_as_list] = ACTIONS(1901),
    [sym_xml_encode_le] = ACTIONS(1901),
    [sym_xml_decode_types] = ACTIONS(1901),
    [sym_xml_decode_no_preserve] = ACTIONS(1901),
    [sym_duet_dev_type_display_device] = ACTIONS(1901),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1901),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1901),
    [sym_duet_dev_type_io_device] = ACTIONS(1901),
    [sym_duet_dev_type_relay_device] = ACTIONS(1901),
    [sym_duet_dev_type_ups] = ACTIONS(1901),
    [sym_duet_dev_type_amplifier] = ACTIONS(1901),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1901),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1901),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1901),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1901),
    [sym_duet_dev_type_camera] = ACTIONS(1901),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1901),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1901),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1901),
    [sym_duet_dev_type_dss] = ACTIONS(1901),
    [sym_duet_dev_type_dvr] = ACTIONS(1901),
    [sym_duet_dev_type_disc_device] = ACTIONS(1901),
    [sym_duet_dev_type_document_camera] = ACTIONS(1901),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1901),
    [sym_duet_dev_type_hvac] = ACTIONS(1901),
    [sym_duet_dev_type_keypad] = ACTIONS(1901),
    [sym_duet_dev_type_light] = ACTIONS(1901),
    [sym_duet_dev_type_monitor] = ACTIONS(1901),
    [sym_duet_dev_type_motor] = ACTIONS(1901),
    [sym_duet_dev_type_multi_window] = ACTIONS(1901),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1901),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1901),
    [sym_duet_dev_type_receiver] = ACTIONS(1901),
    [sym_duet_dev_type_security_system] = ACTIONS(1901),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1901),
    [sym_duet_dev_type_settop_box] = ACTIONS(1901),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1901),
    [sym_duet_dev_type_switcher] = ACTIONS(1901),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1901),
    [sym_duet_dev_type_tv] = ACTIONS(1901),
    [sym_duet_dev_type_utility] = ACTIONS(1901),
    [sym_duet_dev_type_vcr] = ACTIONS(1901),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1901),
    [sym_duet_dev_type_video_processor] = ACTIONS(1901),
    [sym_duet_dev_type_video_projector] = ACTIONS(1901),
    [sym_duet_dev_type_video_wall] = ACTIONS(1901),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1901),
    [sym_duet_dev_type_weather] = ACTIONS(1901),
    [sym_duet_dev_not_polled] = ACTIONS(1901),
    [sym_duet_dev_polled] = ACTIONS(1901),
    [sym_clkmgr_mode_network] = ACTIONS(1901),
    [sym_clkmgr_mode_standalone] = ACTIONS(1901),
    [sym_normal_standby] = ACTIONS(1901),
    [sym_normal_wake] = ACTIONS(1901),
    [sym_ip_multicast_ttl_option] = ACTIONS(1901),
    [sym_ip_ttl_subnet] = ACTIONS(1901),
    [sym_ip_ttl_site] = ACTIONS(1901),
    [sym_ip_ttl_region] = ACTIONS(1901),
    [sym_ip_ttl_continent] = ACTIONS(1901),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1901),
    [sym_ip_nodelay_on] = ACTIONS(1901),
    [sym_ip_nodelay_off] = ACTIONS(1901),
    [sym_amx_error] = ACTIONS(1901),
    [sym_amx_warning] = ACTIONS(1901),
    [sym_amx_info] = ACTIONS(1901),
    [sym_amx_debug] = ACTIONS(1901),
    [sym_netlinx_login_success] = ACTIONS(1901),
    [sym_netlinx_login_fail] = ACTIONS(1901),
    [sym_netlinx_logout] = ACTIONS(1901),
    [sym_null_str] = ACTIONS(1901),
    [sym_smtp_address] = ACTIONS(1901),
    [sym_smtp_port_number] = ACTIONS(1901),
    [sym_smtp_username] = ACTIONS(1901),
    [sym_smtp_password] = ACTIONS(1901),
    [sym_smtp_require_tls] = ACTIONS(1901),
    [sym_smtp_from] = ACTIONS(1901),
    [sym_smtp_tls_true] = ACTIONS(1901),
    [sym_smtp_tls_false] = ACTIONS(1901),
    [sym_listview_on_row_select_event] = ACTIONS(1901),
    [sym_data_structure_unknown] = ACTIONS(1901),
    [sym_data_structure_datafeed] = ACTIONS(1901),
    [sym_data_structure_datarecordset] = ACTIONS(1901),
    [sym_data_structure_datarecord] = ACTIONS(1901),
    [sym_data_structure_datafield] = ACTIONS(1901),
    [sym_data_max_content_field_count] = ACTIONS(1901),
    [sym_data_max_metadata_field_count] = ACTIONS(1901),
    [sym_data_max_id_length] = ACTIONS(1901),
    [sym_data_max_name_length] = ACTIONS(1901),
    [sym_data_max_description_length] = ACTIONS(1901),
    [sym_data_max_type_length] = ACTIONS(1901),
    [sym_data_max_format_length] = ACTIONS(1901),
    [sym_data_max_source_length] = ACTIONS(1901),
    [sym_data_max_value_length] = ACTIONS(1901),
    [sym_data_max_label_length] = ACTIONS(1901),
    [sym_data_max_url_length] = ACTIONS(1901),
    [sym_max_last_login_info_length] = ACTIONS(1901),
    [sym_data_type_unknown] = ACTIONS(1901),
    [sym_data_type_string] = ACTIONS(1901),
    [sym_data_type_datetime] = ACTIONS(1901),
    [sym_data_type_date] = ACTIONS(1901),
    [sym_data_type_time] = ACTIONS(1901),
    [sym_data_type_image] = ACTIONS(1901),
    [sym_data_format_url] = ACTIONS(1901),
    [sym_data_format_phone] = ACTIONS(1901),
    [sym_data_format_email] = ACTIONS(1901),
    [sym_data_format_iso8601] = ACTIONS(1901),
    [sym_auditlog_priv] = ACTIONS(1901),
    [sym_configuration_priv] = ACTIONS(1901),
    [sym_deviceconfig_priv] = ACTIONS(1901),
    [sym_ftp_priv] = ACTIONS(1901),
    [sym_http_priv] = ACTIONS(1901),
    [sym_networkconfig_priv] = ACTIONS(1901),
    [sym_programport_priv] = ACTIONS(1901),
    [sym_remoteui_priv] = ACTIONS(1901),
    [sym_securitycontrol_priv] = ACTIONS(1901),
    [sym_softwaremanagement_priv] = ACTIONS(1901),
    [sym_terminal_priv] = ACTIONS(1901),
    [sym_tpadmin_priv] = ACTIONS(1901),
    [sym_user1_priv] = ACTIONS(1901),
    [sym_user2_priv] = ACTIONS(1901),
    [sym_user3_priv] = ACTIONS(1901),
    [sym_user4_priv] = ACTIONS(1901),
    [sym_usermanagement_priv] = ACTIONS(1901),
    [sym_valid_account] = ACTIONS(1901),
    [sym_err_name_invalid] = ACTIONS(1901),
    [sym_err_password_invalid] = ACTIONS(1901),
    [sym_err_invalid_account] = ACTIONS(1901),
    [sym_err_no_authorization] = ACTIONS(1901),
    [sym_err_invalid_parameter] = ACTIONS(1901),
    [sym_err_user_locked_out] = ACTIONS(1901),
    [sym_err_user_account_expired] = ACTIONS(1901),
    [sym_err_general_error] = ACTIONS(1901),
    [sym_err_authentication_error] = ACTIONS(1901),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1901),
    [sym_tls_validate_certificate] = ACTIONS(1901),
    [sym_tdata] = ACTIONS(1901),
    [sym_tchannel] = ACTIONS(1901),
    [sym_tlevel] = ACTIONS(1901),
    [sym_tbutton] = ACTIONS(1901),
    [sym_ttimeline] = ACTIONS(1901),
    [sym_tcustom] = ACTIONS(1901),
    [sym_url_struct] = ACTIONS(1901),
    [sym_dns_struct] = ACTIONS(1901),
    [sym_ip_address_struct] = ACTIONS(1901),
    [sym_dev_info_struct] = ACTIONS(1901),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1901),
    [sym_data_feed] = ACTIONS(1901),
    [sym_data_field] = ACTIONS(1901),
    [sym_data_record] = ACTIONS(1901),
    [sym_wc_data_feed] = ACTIONS(1901),
    [sym_wc_data_field] = ACTIONS(1901),
    [sym_wc_data_record] = ACTIONS(1901),
    [sym_last_login_info] = ACTIONS(1901),
    [sym____reserved___] = ACTIONS(1901),
    [sym_date] = ACTIONS(1901),
    [sym_day] = ACTIONS(1901),
    [sym_time] = ACTIONS(1901),
    [sym_ldate] = ACTIONS(1901),
    [sym_push_device] = ACTIONS(1901),
    [sym_push_channel] = ACTIONS(1901),
    [sym_push_devchan] = ACTIONS(1901),
    [sym_release_device] = ACTIONS(1901),
    [sym_release_channel] = ACTIONS(1901),
    [sym_release_devchan] = ACTIONS(1901),
    [sym_master_slot] = ACTIONS(1901),
    [sym_get_pulse_time] = ACTIONS(1901),
    [sym_get_timer] = ACTIONS(1901),
    [sym_master_sn] = ACTIONS(1901),
    [sym_system_number] = ACTIONS(1901),
    [sym_button] = ACTIONS(1901),
    [sym_dv_channel] = ACTIONS(1901),
    [sym_data] = ACTIONS(1901),
    [sym_level] = ACTIONS(1901),
    [sym_timeline] = ACTIONS(1901),
    [sym_channel] = ACTIONS(1901),
    [sym_custom] = ACTIONS(1901),
    [sym_atoi] = ACTIONS(1901),
    [sym_atol] = ACTIONS(1901),
    [sym_atof] = ACTIONS(1901),
    [sym_itoa] = ACTIONS(1901),
    [sym_itohex] = ACTIONS(1901),
    [sym_hextoi] = ACTIONS(1901),
    [sym_ftoa] = ACTIONS(1901),
    [sym_find_string] = ACTIONS(1901),
    [sym_left_string] = ACTIONS(1901),
    [sym_right_string] = ACTIONS(1901),
    [sym_mid_string] = ACTIONS(1901),
    [sym_remove_string] = ACTIONS(1901),
    [sym_get_buffer_string] = ACTIONS(1901),
    [sym_get_buffer_char] = ACTIONS(1901),
    [sym_get_multi_buffer_string] = ACTIONS(1901),
    [sym_length_string] = ACTIONS(1901),
    [sym_max_length_string] = ACTIONS(1901),
    [sym_set_length_string] = ACTIONS(1901),
    [sym_compare_string] = ACTIONS(1901),
    [sym_upper_string] = ACTIONS(1901),
    [sym_lower_string] = ACTIONS(1901),
    [sym_get_last] = ACTIONS(1901),
    [sym_date_to_day] = ACTIONS(1901),
    [sym_date_to_month] = ACTIONS(1901),
    [sym_date_to_year] = ACTIONS(1901),
    [sym_time_to_hour] = ACTIONS(1901),
    [sym_time_to_minute] = ACTIONS(1901),
    [sym_time_to_second] = ACTIONS(1901),
    [sym_day_of_week] = ACTIONS(1901),
    [sym_set_pulse_time] = ACTIONS(1901),
    [sym_set_timer] = ACTIONS(1901),
    [sym_file_open] = ACTIONS(1901),
    [sym_file_close] = ACTIONS(1901),
    [sym_file_read] = ACTIONS(1901),
    [sym_file_read_line] = ACTIONS(1901),
    [sym_file_write] = ACTIONS(1901),
    [sym_file_write_line] = ACTIONS(1901),
    [sym_file_copy] = ACTIONS(1901),
    [sym_file_delete] = ACTIONS(1901),
    [sym_file_rename] = ACTIONS(1901),
    [sym_file_dir] = ACTIONS(1901),
    [sym_file_getdir] = ACTIONS(1901),
    [sym_file_setdir] = ACTIONS(1901),
    [sym_file_create_dir] = ACTIONS(1901),
    [sym_file_remove_dir] = ACTIONS(1901),
    [sym_file_seek] = ACTIONS(1901),
    [sym_length_array] = ACTIONS(1901),
    [sym_max_length_array] = ACTIONS(1901),
    [sym_set_length_array] = ACTIONS(1901),
    [sym_ip_client_open] = ACTIONS(1901),
    [sym_ip_client_close] = ACTIONS(1901),
    [sym_ip_server_open] = ACTIONS(1901),
    [sym_ip_server_close] = ACTIONS(1901),
    [sym_ip_bound_client_open] = ACTIONS(1901),
    [sym_ip_mc_server_open] = ACTIONS(1901),
    [sym_ip_set_option] = ACTIONS(1901),
    [sym_ssh_client_open] = ACTIONS(1901),
    [sym_ssh_client_close] = ACTIONS(1901),
    [sym_tls_client_open] = ACTIONS(1901),
    [sym_tls_client_close] = ACTIONS(1901),
    [sym_do_push] = ACTIONS(1901),
    [sym_do_push_timed] = ACTIONS(1901),
    [sym_do_release] = ACTIONS(1901),
    [sym_device_id] = ACTIONS(1901),
    [sym_device_info] = ACTIONS(1901),
    [sym_device_id_string] = ACTIONS(1901),
    [sym_device_standby] = ACTIONS(1901),
    [sym_device_wake] = ACTIONS(1901),
    [sym_timeline_create] = ACTIONS(1901),
    [sym_timeline_kill] = ACTIONS(1901),
    [sym_timeline_reload] = ACTIONS(1901),
    [sym_timeline_pause] = ACTIONS(1901),
    [sym_timeline_restart] = ACTIONS(1901),
    [sym_timeline_set] = ACTIONS(1901),
    [sym_timeline_get] = ACTIONS(1901),
    [sym_timeline_active] = ACTIONS(1901),
    [sym_timeline_dynamic_id] = ACTIONS(1901),
    [sym_variable_to_string] = ACTIONS(1901),
    [sym_string_to_variable] = ACTIONS(1901),
    [sym_length_variable_to_string] = ACTIONS(1901),
    [sym_type_cast] = ACTIONS(1901),
    [sym_variable_to_xml] = ACTIONS(1901),
    [sym_xml_to_variable] = ACTIONS(1901),
    [sym_length_variable_to_xml] = ACTIONS(1901),
    [sym_redirect_string] = ACTIONS(1901),
    [sym_random_number] = ACTIONS(1901),
    [sym_get_unique_id] = ACTIONS(1901),
    [sym_get_system_number] = ACTIONS(1901),
    [sym_set_system_number] = ACTIONS(1901),
    [sym_get_serial_number] = ACTIONS(1901),
    [sym_get_master_build] = ACTIONS(1901),
    [sym_set_validation_code] = ACTIONS(1901),
    [sym_reboot] = ACTIONS(1901),
    [sym_load_duet_module] = ACTIONS(1901),
    [sym_unload_duet_module] = ACTIONS(1901),
    [sym_duet_mem_size_set] = ACTIONS(1901),
    [sym_duet_mem_size_get] = ACTIONS(1901),
    [sym_dynamic_application_device] = ACTIONS(1901),
    [sym_dynamic_polled_port] = ACTIONS(1901),
    [sym_static_port_binding] = ACTIONS(1901),
    [sym_static_ip_binding] = ACTIONS(1901),
    [sym_get_dns_list] = ACTIONS(1901),
    [sym_set_dns_list] = ACTIONS(1901),
    [sym_get_ip_address] = ACTIONS(1901),
    [sym_set_ip_address] = ACTIONS(1901),
    [sym_get_url_list] = ACTIONS(1901),
    [sym_add_url_entry] = ACTIONS(1901),
    [sym_delete_url_entry] = ACTIONS(1901),
    [sym_abs_value] = ACTIONS(1901),
    [sym_max_value] = ACTIONS(1901),
    [sym_min_value] = ACTIONS(1901),
    [sym_exp_value] = ACTIONS(1901),
    [sym_log_value] = ACTIONS(1901),
    [sym_log10_value] = ACTIONS(1901),
    [sym_power_value] = ACTIONS(1901),
    [sym_sqrt_value] = ACTIONS(1901),
    [sym_raw_be] = ACTIONS(1901),
    [sym_raw_le] = ACTIONS(1901),
    [sym_internal_queue_size_set] = ACTIONS(1901),
    [sym_internal_queue_size_get] = ACTIONS(1901),
    [sym_internal_threshold_set] = ACTIONS(1901),
    [sym_internal_threshold_get] = ACTIONS(1901),
    [sym_rebuilt_event] = ACTIONS(1901),
    [sym_do_custom_event] = ACTIONS(1901),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1901),
    [sym_clkmgr_set_clk_source] = ACTIONS(1901),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1901),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1901),
    [sym_clkmgr_get_timezone] = ACTIONS(1901),
    [sym_clkmgr_set_timezone] = ACTIONS(1901),
    [sym_clkmgr_get_resync_period] = ACTIONS(1901),
    [sym_clkmgr_set_resync_period] = ACTIONS(1901),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1901),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1901),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1901),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1901),
    [sym_clkmgr_get_timeservers] = ACTIONS(1901),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1901),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1901),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1901),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1901),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1901),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1901),
    [sym_set_log_level] = ACTIONS(1901),
    [sym_get_log_level] = ACTIONS(1901),
    [sym_amx_log] = ACTIONS(1901),
    [sym_smtp_server_config_set] = ACTIONS(1901),
    [sym_smtp_server_config_get] = ACTIONS(1901),
    [sym_smtp_send] = ACTIONS(1901),
    [sym_validate_netlinx_account] = ACTIONS(1901),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1901),
    [sym_authenticate_certificate] = ACTIONS(1901),
    [sym_audit_netlinx_session_event] = ACTIONS(1901),
    [sym_audit_netlinx_generic_event] = ACTIONS(1901),
    [sym_get_available_flash_disk_space] = ACTIONS(1901),
    [sym_get_max_flash_disk_space] = ACTIONS(1901),
    [sym_data_create_feed] = ACTIONS(1901),
    [sym_data_delete_feed] = ACTIONS(1901),
    [sym_data_publish_feed] = ACTIONS(1901),
    [sym_data_get_published_feed] = ACTIONS(1901),
    [sym_data_add_record] = ACTIONS(1901),
    [sym_data_get_event_record] = ACTIONS(1901),
    [sym__wc_data_create_feed] = ACTIONS(1901),
    [sym__wc_data_add_record] = ACTIONS(1901),
    [sym__wc_data_get_event_record] = ACTIONS(1901),
    [sym_format] = ACTIONS(1901),
    [sym_astro_clock] = ACTIONS(1901),
    [sym_set_outdoor_temperature] = ACTIONS(1901),
    [sym_set_virtual_level_count] = ACTIONS(1901),
    [sym_set_virtual_channel_count] = ACTIONS(1901),
    [sym_set_virtual_port_count] = ACTIONS(1901),
    [sym__wc_encode_int] = ACTIONS(1901),
    [sym__wc_decode_int] = ACTIONS(1901),
    [sym__wc_int] = ACTIONS(1901),
    [sym__wc_to_ch_int] = ACTIONS(1901),
    [sym__ch_to_wc_int] = ACTIONS(1901),
    [sym__wc_find_string_int] = ACTIONS(1901),
    [sym__wc_left_string_int] = ACTIONS(1901),
    [sym__wc_lower_string_int] = ACTIONS(1901),
    [sym__wc_mid_string_int] = ACTIONS(1901),
    [sym__wc_remove_string_int] = ACTIONS(1901),
    [sym__wc_right_string_int] = ACTIONS(1901),
    [sym__wc_upper_string_int] = ACTIONS(1901),
    [sym__wc_compare_string_int] = ACTIONS(1901),
    [sym__wc_get_buffer_char_int] = ACTIONS(1901),
    [sym__wc_get_buffer_string_int] = ACTIONS(1901),
    [sym__wc_concat_string_int] = ACTIONS(1901),
    [sym__wc_file_open_int] = ACTIONS(1901),
    [sym__wc_file_close_int] = ACTIONS(1901),
    [sym__wc_file_read_int] = ACTIONS(1901),
    [sym__wc_file_read_line_int] = ACTIONS(1901),
    [sym__wc_file_write_int] = ACTIONS(1901),
    [sym__wc_file_write_line_int] = ACTIONS(1901),
    [aux_sym_preproc_include_token1] = ACTIONS(1901),
    [aux_sym_preproc_define_token1] = ACTIONS(1903),
    [aux_sym_preproc_warn_token1] = ACTIONS(1903),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1903),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1903),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1903),
    [anon_sym_LBRACK] = ACTIONS(1903),
    [anon_sym_LBRACE] = ACTIONS(1903),
    [anon_sym_RBRACE] = ACTIONS(1903),
    [anon_sym_LPAREN] = ACTIONS(1901),
    [anon_sym_BANG] = ACTIONS(1903),
    [anon_sym_TILDE] = ACTIONS(1903),
    [anon_sym_DASH] = ACTIONS(1901),
    [anon_sym_PLUS] = ACTIONS(1901),
    [anon_sym_DASH_DASH] = ACTIONS(1903),
    [anon_sym_PLUS_PLUS] = ACTIONS(1903),
    [anon_sym_DQUOTE] = ACTIONS(1903),
    [anon_sym_SQUOTE] = ACTIONS(1903),
    [sym_number_literal] = ACTIONS(1903),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(527)] = {
    [sym_identifier] = ACTIONS(1905),
    [sym_define_call_keyword] = ACTIONS(1905),
    [sym_define_function_keyword] = ACTIONS(1905),
    [sym_define_library_function_keyword] = ACTIONS(1905),
    [sym_define_module_keyword] = ACTIONS(1905),
    [sym_char_keyword] = ACTIONS(1905),
    [sym_widechar_keyword] = ACTIONS(1905),
    [sym_integer_keyword] = ACTIONS(1905),
    [sym_sinteger_keyword] = ACTIONS(1905),
    [sym_long_keyword] = ACTIONS(1905),
    [sym_slong_keyword] = ACTIONS(1905),
    [sym_float_keyword] = ACTIONS(1905),
    [sym_double_keyword] = ACTIONS(1905),
    [sym_variant_keyword] = ACTIONS(1905),
    [sym_variantarray_keyword] = ACTIONS(1905),
    [sym_local_var_keyword] = ACTIONS(1905),
    [sym_stack_var_keyword] = ACTIONS(1905),
    [sym_constant_keyword] = ACTIONS(1905),
    [sym_volatile_keyword] = ACTIONS(1905),
    [sym_non_volatile_keyword] = ACTIONS(1905),
    [sym_persistent_keyword] = ACTIONS(1905),
    [sym_dev_keyword] = ACTIONS(1905),
    [sym_devlev_keyword] = ACTIONS(1905),
    [sym_devchan_keyword] = ACTIONS(1905),
    [sym_if_keyword] = ACTIONS(1905),
    [sym_switch_keyword] = ACTIONS(1905),
    [sym_case_keyword] = ACTIONS(1905),
    [sym_default_keyword] = ACTIONS(1905),
    [sym_while_keyword] = ACTIONS(1905),
    [sym_for_keyword] = ACTIONS(1905),
    [sym_break_keyword] = ACTIONS(1905),
    [sym_continue_keyword] = ACTIONS(1905),
    [sym_return_keyword] = ACTIONS(1905),
    [sym_select_keyword] = ACTIONS(1905),
    [sym_active_keyword] = ACTIONS(1905),
    [sym_struct_keyword] = ACTIONS(1905),
    [sym_structure_keyword] = ACTIONS(1905),
    [aux_sym_on_keyword_token1] = ACTIONS(1905),
    [aux_sym_off_keyword_token1] = ACTIONS(1905),
    [sym_send_level_keyword] = ACTIONS(1905),
    [sym_send_string_keyword] = ACTIONS(1905),
    [sym_send_command_keyword] = ACTIONS(1905),
    [sym_clear_buffer_keyword] = ACTIONS(1905),
    [sym_create_buffer_keyword] = ACTIONS(1905),
    [sym_create_multi_buffer_keyword] = ACTIONS(1905),
    [sym_call_keyword] = ACTIONS(1905),
    [sym_system_call_keyword] = ACTIONS(1905),
    [sym_devchan_to_keyword] = ACTIONS(1905),
    [sym_devchan_min_to_keyword] = ACTIONS(1905),
    [sym_devchan_total_off_keyword] = ACTIONS(1905),
    [sym_devchan_pulse_keyword] = ACTIONS(1905),
    [sym_wait_keyword] = ACTIONS(1905),
    [sym_cancel_wait_keyword] = ACTIONS(1905),
    [sym_wait_until_keyword] = ACTIONS(1905),
    [sym_cancel_wait_until_keyword] = ACTIONS(1905),
    [sym_cancel_all_wait_keyword] = ACTIONS(1905),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1905),
    [sym_not] = ACTIONS(1905),
    [sym_bnot] = ACTIONS(1905),
    [sym___file__] = ACTIONS(1905),
    [sym___date__] = ACTIONS(1905),
    [sym___ldate__] = ACTIONS(1905),
    [sym___time__] = ACTIONS(1905),
    [sym___line__] = ACTIONS(1905),
    [sym___name__] = ACTIONS(1905),
    [sym___netlinx__] = ACTIONS(1905),
    [sym_netlinx_axi_version] = ACTIONS(1905),
    [sym_url_flg_tcp] = ACTIONS(1905),
    [sym_url_flg_accinfopresent] = ACTIONS(1905),
    [sym_url_flg_temp] = ACTIONS(1905),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1905),
    [sym_url_flg_stat_mask] = ACTIONS(1905),
    [sym_url_flg_stat_lookup] = ACTIONS(1905),
    [sym_url_flg_stat_connecting] = ACTIONS(1905),
    [sym_url_flg_stat_waiting] = ACTIONS(1905),
    [sym_url_flg_stat_connected] = ACTIONS(1905),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1905),
    [sym_first_virtual_device] = ACTIONS(1905),
    [sym_first_local_port] = ACTIONS(1905),
    [sym_dynamic_virtual_device] = ACTIONS(1905),
    [sym_timeline_once] = ACTIONS(1905),
    [sym_timeline_repeat] = ACTIONS(1905),
    [sym_timeline_absolute] = ACTIONS(1905),
    [sym_timeline_relative] = ACTIONS(1905),
    [sym_do_push_timed_infinite] = ACTIONS(1905),
    [sym_source_type_no_address] = ACTIONS(1905),
    [sym_source_type_neuron_id] = ACTIONS(1905),
    [sym_source_type_ip_address] = ACTIONS(1905),
    [sym_source_type_axlink] = ACTIONS(1905),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1905),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1905),
    [sym_source_type_ip_socket_address] = ACTIONS(1905),
    [sym_source_type_rs232] = ACTIONS(1905),
    [sym_source_type_internal] = ACTIONS(1905),
    [sym_source_type_mac_address] = ACTIONS(1905),
    [sym_source_type_ipv4_port] = ACTIONS(1905),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1905),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1905),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1905),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1905),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1905),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1905),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1905),
    [sym_true] = ACTIONS(1905),
    [sym_false] = ACTIONS(1905),
    [sym_file_read_only] = ACTIONS(1905),
    [sym_file_rw_new] = ACTIONS(1905),
    [sym_file_rw_append] = ACTIONS(1905),
    [sym_ip_tcp] = ACTIONS(1905),
    [sym_ip_udp] = ACTIONS(1905),
    [sym_ip_udp_2way] = ACTIONS(1905),
    [sym_xml_encode_types] = ACTIONS(1905),
    [sym_xml_encode_char_as_list] = ACTIONS(1905),
    [sym_xml_encode_le] = ACTIONS(1905),
    [sym_xml_decode_types] = ACTIONS(1905),
    [sym_xml_decode_no_preserve] = ACTIONS(1905),
    [sym_duet_dev_type_display_device] = ACTIONS(1905),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1905),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1905),
    [sym_duet_dev_type_io_device] = ACTIONS(1905),
    [sym_duet_dev_type_relay_device] = ACTIONS(1905),
    [sym_duet_dev_type_ups] = ACTIONS(1905),
    [sym_duet_dev_type_amplifier] = ACTIONS(1905),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1905),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1905),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1905),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1905),
    [sym_duet_dev_type_camera] = ACTIONS(1905),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1905),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1905),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1905),
    [sym_duet_dev_type_dss] = ACTIONS(1905),
    [sym_duet_dev_type_dvr] = ACTIONS(1905),
    [sym_duet_dev_type_disc_device] = ACTIONS(1905),
    [sym_duet_dev_type_document_camera] = ACTIONS(1905),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1905),
    [sym_duet_dev_type_hvac] = ACTIONS(1905),
    [sym_duet_dev_type_keypad] = ACTIONS(1905),
    [sym_duet_dev_type_light] = ACTIONS(1905),
    [sym_duet_dev_type_monitor] = ACTIONS(1905),
    [sym_duet_dev_type_motor] = ACTIONS(1905),
    [sym_duet_dev_type_multi_window] = ACTIONS(1905),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1905),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1905),
    [sym_duet_dev_type_receiver] = ACTIONS(1905),
    [sym_duet_dev_type_security_system] = ACTIONS(1905),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1905),
    [sym_duet_dev_type_settop_box] = ACTIONS(1905),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1905),
    [sym_duet_dev_type_switcher] = ACTIONS(1905),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1905),
    [sym_duet_dev_type_tv] = ACTIONS(1905),
    [sym_duet_dev_type_utility] = ACTIONS(1905),
    [sym_duet_dev_type_vcr] = ACTIONS(1905),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1905),
    [sym_duet_dev_type_video_processor] = ACTIONS(1905),
    [sym_duet_dev_type_video_projector] = ACTIONS(1905),
    [sym_duet_dev_type_video_wall] = ACTIONS(1905),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1905),
    [sym_duet_dev_type_weather] = ACTIONS(1905),
    [sym_duet_dev_not_polled] = ACTIONS(1905),
    [sym_duet_dev_polled] = ACTIONS(1905),
    [sym_clkmgr_mode_network] = ACTIONS(1905),
    [sym_clkmgr_mode_standalone] = ACTIONS(1905),
    [sym_normal_standby] = ACTIONS(1905),
    [sym_normal_wake] = ACTIONS(1905),
    [sym_ip_multicast_ttl_option] = ACTIONS(1905),
    [sym_ip_ttl_subnet] = ACTIONS(1905),
    [sym_ip_ttl_site] = ACTIONS(1905),
    [sym_ip_ttl_region] = ACTIONS(1905),
    [sym_ip_ttl_continent] = ACTIONS(1905),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1905),
    [sym_ip_nodelay_on] = ACTIONS(1905),
    [sym_ip_nodelay_off] = ACTIONS(1905),
    [sym_amx_error] = ACTIONS(1905),
    [sym_amx_warning] = ACTIONS(1905),
    [sym_amx_info] = ACTIONS(1905),
    [sym_amx_debug] = ACTIONS(1905),
    [sym_netlinx_login_success] = ACTIONS(1905),
    [sym_netlinx_login_fail] = ACTIONS(1905),
    [sym_netlinx_logout] = ACTIONS(1905),
    [sym_null_str] = ACTIONS(1905),
    [sym_smtp_address] = ACTIONS(1905),
    [sym_smtp_port_number] = ACTIONS(1905),
    [sym_smtp_username] = ACTIONS(1905),
    [sym_smtp_password] = ACTIONS(1905),
    [sym_smtp_require_tls] = ACTIONS(1905),
    [sym_smtp_from] = ACTIONS(1905),
    [sym_smtp_tls_true] = ACTIONS(1905),
    [sym_smtp_tls_false] = ACTIONS(1905),
    [sym_listview_on_row_select_event] = ACTIONS(1905),
    [sym_data_structure_unknown] = ACTIONS(1905),
    [sym_data_structure_datafeed] = ACTIONS(1905),
    [sym_data_structure_datarecordset] = ACTIONS(1905),
    [sym_data_structure_datarecord] = ACTIONS(1905),
    [sym_data_structure_datafield] = ACTIONS(1905),
    [sym_data_max_content_field_count] = ACTIONS(1905),
    [sym_data_max_metadata_field_count] = ACTIONS(1905),
    [sym_data_max_id_length] = ACTIONS(1905),
    [sym_data_max_name_length] = ACTIONS(1905),
    [sym_data_max_description_length] = ACTIONS(1905),
    [sym_data_max_type_length] = ACTIONS(1905),
    [sym_data_max_format_length] = ACTIONS(1905),
    [sym_data_max_source_length] = ACTIONS(1905),
    [sym_data_max_value_length] = ACTIONS(1905),
    [sym_data_max_label_length] = ACTIONS(1905),
    [sym_data_max_url_length] = ACTIONS(1905),
    [sym_max_last_login_info_length] = ACTIONS(1905),
    [sym_data_type_unknown] = ACTIONS(1905),
    [sym_data_type_string] = ACTIONS(1905),
    [sym_data_type_datetime] = ACTIONS(1905),
    [sym_data_type_date] = ACTIONS(1905),
    [sym_data_type_time] = ACTIONS(1905),
    [sym_data_type_image] = ACTIONS(1905),
    [sym_data_format_url] = ACTIONS(1905),
    [sym_data_format_phone] = ACTIONS(1905),
    [sym_data_format_email] = ACTIONS(1905),
    [sym_data_format_iso8601] = ACTIONS(1905),
    [sym_auditlog_priv] = ACTIONS(1905),
    [sym_configuration_priv] = ACTIONS(1905),
    [sym_deviceconfig_priv] = ACTIONS(1905),
    [sym_ftp_priv] = ACTIONS(1905),
    [sym_http_priv] = ACTIONS(1905),
    [sym_networkconfig_priv] = ACTIONS(1905),
    [sym_programport_priv] = ACTIONS(1905),
    [sym_remoteui_priv] = ACTIONS(1905),
    [sym_securitycontrol_priv] = ACTIONS(1905),
    [sym_softwaremanagement_priv] = ACTIONS(1905),
    [sym_terminal_priv] = ACTIONS(1905),
    [sym_tpadmin_priv] = ACTIONS(1905),
    [sym_user1_priv] = ACTIONS(1905),
    [sym_user2_priv] = ACTIONS(1905),
    [sym_user3_priv] = ACTIONS(1905),
    [sym_user4_priv] = ACTIONS(1905),
    [sym_usermanagement_priv] = ACTIONS(1905),
    [sym_valid_account] = ACTIONS(1905),
    [sym_err_name_invalid] = ACTIONS(1905),
    [sym_err_password_invalid] = ACTIONS(1905),
    [sym_err_invalid_account] = ACTIONS(1905),
    [sym_err_no_authorization] = ACTIONS(1905),
    [sym_err_invalid_parameter] = ACTIONS(1905),
    [sym_err_user_locked_out] = ACTIONS(1905),
    [sym_err_user_account_expired] = ACTIONS(1905),
    [sym_err_general_error] = ACTIONS(1905),
    [sym_err_authentication_error] = ACTIONS(1905),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1905),
    [sym_tls_validate_certificate] = ACTIONS(1905),
    [sym_tdata] = ACTIONS(1905),
    [sym_tchannel] = ACTIONS(1905),
    [sym_tlevel] = ACTIONS(1905),
    [sym_tbutton] = ACTIONS(1905),
    [sym_ttimeline] = ACTIONS(1905),
    [sym_tcustom] = ACTIONS(1905),
    [sym_url_struct] = ACTIONS(1905),
    [sym_dns_struct] = ACTIONS(1905),
    [sym_ip_address_struct] = ACTIONS(1905),
    [sym_dev_info_struct] = ACTIONS(1905),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1905),
    [sym_data_feed] = ACTIONS(1905),
    [sym_data_field] = ACTIONS(1905),
    [sym_data_record] = ACTIONS(1905),
    [sym_wc_data_feed] = ACTIONS(1905),
    [sym_wc_data_field] = ACTIONS(1905),
    [sym_wc_data_record] = ACTIONS(1905),
    [sym_last_login_info] = ACTIONS(1905),
    [sym____reserved___] = ACTIONS(1905),
    [sym_date] = ACTIONS(1905),
    [sym_day] = ACTIONS(1905),
    [sym_time] = ACTIONS(1905),
    [sym_ldate] = ACTIONS(1905),
    [sym_push_device] = ACTIONS(1905),
    [sym_push_channel] = ACTIONS(1905),
    [sym_push_devchan] = ACTIONS(1905),
    [sym_release_device] = ACTIONS(1905),
    [sym_release_channel] = ACTIONS(1905),
    [sym_release_devchan] = ACTIONS(1905),
    [sym_master_slot] = ACTIONS(1905),
    [sym_get_pulse_time] = ACTIONS(1905),
    [sym_get_timer] = ACTIONS(1905),
    [sym_master_sn] = ACTIONS(1905),
    [sym_system_number] = ACTIONS(1905),
    [sym_button] = ACTIONS(1905),
    [sym_dv_channel] = ACTIONS(1905),
    [sym_data] = ACTIONS(1905),
    [sym_level] = ACTIONS(1905),
    [sym_timeline] = ACTIONS(1905),
    [sym_channel] = ACTIONS(1905),
    [sym_custom] = ACTIONS(1905),
    [sym_atoi] = ACTIONS(1905),
    [sym_atol] = ACTIONS(1905),
    [sym_atof] = ACTIONS(1905),
    [sym_itoa] = ACTIONS(1905),
    [sym_itohex] = ACTIONS(1905),
    [sym_hextoi] = ACTIONS(1905),
    [sym_ftoa] = ACTIONS(1905),
    [sym_find_string] = ACTIONS(1905),
    [sym_left_string] = ACTIONS(1905),
    [sym_right_string] = ACTIONS(1905),
    [sym_mid_string] = ACTIONS(1905),
    [sym_remove_string] = ACTIONS(1905),
    [sym_get_buffer_string] = ACTIONS(1905),
    [sym_get_buffer_char] = ACTIONS(1905),
    [sym_get_multi_buffer_string] = ACTIONS(1905),
    [sym_length_string] = ACTIONS(1905),
    [sym_max_length_string] = ACTIONS(1905),
    [sym_set_length_string] = ACTIONS(1905),
    [sym_compare_string] = ACTIONS(1905),
    [sym_upper_string] = ACTIONS(1905),
    [sym_lower_string] = ACTIONS(1905),
    [sym_get_last] = ACTIONS(1905),
    [sym_date_to_day] = ACTIONS(1905),
    [sym_date_to_month] = ACTIONS(1905),
    [sym_date_to_year] = ACTIONS(1905),
    [sym_time_to_hour] = ACTIONS(1905),
    [sym_time_to_minute] = ACTIONS(1905),
    [sym_time_to_second] = ACTIONS(1905),
    [sym_day_of_week] = ACTIONS(1905),
    [sym_set_pulse_time] = ACTIONS(1905),
    [sym_set_timer] = ACTIONS(1905),
    [sym_file_open] = ACTIONS(1905),
    [sym_file_close] = ACTIONS(1905),
    [sym_file_read] = ACTIONS(1905),
    [sym_file_read_line] = ACTIONS(1905),
    [sym_file_write] = ACTIONS(1905),
    [sym_file_write_line] = ACTIONS(1905),
    [sym_file_copy] = ACTIONS(1905),
    [sym_file_delete] = ACTIONS(1905),
    [sym_file_rename] = ACTIONS(1905),
    [sym_file_dir] = ACTIONS(1905),
    [sym_file_getdir] = ACTIONS(1905),
    [sym_file_setdir] = ACTIONS(1905),
    [sym_file_create_dir] = ACTIONS(1905),
    [sym_file_remove_dir] = ACTIONS(1905),
    [sym_file_seek] = ACTIONS(1905),
    [sym_length_array] = ACTIONS(1905),
    [sym_max_length_array] = ACTIONS(1905),
    [sym_set_length_array] = ACTIONS(1905),
    [sym_ip_client_open] = ACTIONS(1905),
    [sym_ip_client_close] = ACTIONS(1905),
    [sym_ip_server_open] = ACTIONS(1905),
    [sym_ip_server_close] = ACTIONS(1905),
    [sym_ip_bound_client_open] = ACTIONS(1905),
    [sym_ip_mc_server_open] = ACTIONS(1905),
    [sym_ip_set_option] = ACTIONS(1905),
    [sym_ssh_client_open] = ACTIONS(1905),
    [sym_ssh_client_close] = ACTIONS(1905),
    [sym_tls_client_open] = ACTIONS(1905),
    [sym_tls_client_close] = ACTIONS(1905),
    [sym_do_push] = ACTIONS(1905),
    [sym_do_push_timed] = ACTIONS(1905),
    [sym_do_release] = ACTIONS(1905),
    [sym_device_id] = ACTIONS(1905),
    [sym_device_info] = ACTIONS(1905),
    [sym_device_id_string] = ACTIONS(1905),
    [sym_device_standby] = ACTIONS(1905),
    [sym_device_wake] = ACTIONS(1905),
    [sym_timeline_create] = ACTIONS(1905),
    [sym_timeline_kill] = ACTIONS(1905),
    [sym_timeline_reload] = ACTIONS(1905),
    [sym_timeline_pause] = ACTIONS(1905),
    [sym_timeline_restart] = ACTIONS(1905),
    [sym_timeline_set] = ACTIONS(1905),
    [sym_timeline_get] = ACTIONS(1905),
    [sym_timeline_active] = ACTIONS(1905),
    [sym_timeline_dynamic_id] = ACTIONS(1905),
    [sym_variable_to_string] = ACTIONS(1905),
    [sym_string_to_variable] = ACTIONS(1905),
    [sym_length_variable_to_string] = ACTIONS(1905),
    [sym_type_cast] = ACTIONS(1905),
    [sym_variable_to_xml] = ACTIONS(1905),
    [sym_xml_to_variable] = ACTIONS(1905),
    [sym_length_variable_to_xml] = ACTIONS(1905),
    [sym_redirect_string] = ACTIONS(1905),
    [sym_random_number] = ACTIONS(1905),
    [sym_get_unique_id] = ACTIONS(1905),
    [sym_get_system_number] = ACTIONS(1905),
    [sym_set_system_number] = ACTIONS(1905),
    [sym_get_serial_number] = ACTIONS(1905),
    [sym_get_master_build] = ACTIONS(1905),
    [sym_set_validation_code] = ACTIONS(1905),
    [sym_reboot] = ACTIONS(1905),
    [sym_load_duet_module] = ACTIONS(1905),
    [sym_unload_duet_module] = ACTIONS(1905),
    [sym_duet_mem_size_set] = ACTIONS(1905),
    [sym_duet_mem_size_get] = ACTIONS(1905),
    [sym_dynamic_application_device] = ACTIONS(1905),
    [sym_dynamic_polled_port] = ACTIONS(1905),
    [sym_static_port_binding] = ACTIONS(1905),
    [sym_static_ip_binding] = ACTIONS(1905),
    [sym_get_dns_list] = ACTIONS(1905),
    [sym_set_dns_list] = ACTIONS(1905),
    [sym_get_ip_address] = ACTIONS(1905),
    [sym_set_ip_address] = ACTIONS(1905),
    [sym_get_url_list] = ACTIONS(1905),
    [sym_add_url_entry] = ACTIONS(1905),
    [sym_delete_url_entry] = ACTIONS(1905),
    [sym_abs_value] = ACTIONS(1905),
    [sym_max_value] = ACTIONS(1905),
    [sym_min_value] = ACTIONS(1905),
    [sym_exp_value] = ACTIONS(1905),
    [sym_log_value] = ACTIONS(1905),
    [sym_log10_value] = ACTIONS(1905),
    [sym_power_value] = ACTIONS(1905),
    [sym_sqrt_value] = ACTIONS(1905),
    [sym_raw_be] = ACTIONS(1905),
    [sym_raw_le] = ACTIONS(1905),
    [sym_internal_queue_size_set] = ACTIONS(1905),
    [sym_internal_queue_size_get] = ACTIONS(1905),
    [sym_internal_threshold_set] = ACTIONS(1905),
    [sym_internal_threshold_get] = ACTIONS(1905),
    [sym_rebuilt_event] = ACTIONS(1905),
    [sym_do_custom_event] = ACTIONS(1905),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1905),
    [sym_clkmgr_set_clk_source] = ACTIONS(1905),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1905),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1905),
    [sym_clkmgr_get_timezone] = ACTIONS(1905),
    [sym_clkmgr_set_timezone] = ACTIONS(1905),
    [sym_clkmgr_get_resync_period] = ACTIONS(1905),
    [sym_clkmgr_set_resync_period] = ACTIONS(1905),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1905),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1905),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1905),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1905),
    [sym_clkmgr_get_timeservers] = ACTIONS(1905),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1905),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1905),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1905),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1905),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1905),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1905),
    [sym_set_log_level] = ACTIONS(1905),
    [sym_get_log_level] = ACTIONS(1905),
    [sym_amx_log] = ACTIONS(1905),
    [sym_smtp_server_config_set] = ACTIONS(1905),
    [sym_smtp_server_config_get] = ACTIONS(1905),
    [sym_smtp_send] = ACTIONS(1905),
    [sym_validate_netlinx_account] = ACTIONS(1905),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1905),
    [sym_authenticate_certificate] = ACTIONS(1905),
    [sym_audit_netlinx_session_event] = ACTIONS(1905),
    [sym_audit_netlinx_generic_event] = ACTIONS(1905),
    [sym_get_available_flash_disk_space] = ACTIONS(1905),
    [sym_get_max_flash_disk_space] = ACTIONS(1905),
    [sym_data_create_feed] = ACTIONS(1905),
    [sym_data_delete_feed] = ACTIONS(1905),
    [sym_data_publish_feed] = ACTIONS(1905),
    [sym_data_get_published_feed] = ACTIONS(1905),
    [sym_data_add_record] = ACTIONS(1905),
    [sym_data_get_event_record] = ACTIONS(1905),
    [sym__wc_data_create_feed] = ACTIONS(1905),
    [sym__wc_data_add_record] = ACTIONS(1905),
    [sym__wc_data_get_event_record] = ACTIONS(1905),
    [sym_format] = ACTIONS(1905),
    [sym_astro_clock] = ACTIONS(1905),
    [sym_set_outdoor_temperature] = ACTIONS(1905),
    [sym_set_virtual_level_count] = ACTIONS(1905),
    [sym_set_virtual_channel_count] = ACTIONS(1905),
    [sym_set_virtual_port_count] = ACTIONS(1905),
    [sym__wc_encode_int] = ACTIONS(1905),
    [sym__wc_decode_int] = ACTIONS(1905),
    [sym__wc_int] = ACTIONS(1905),
    [sym__wc_to_ch_int] = ACTIONS(1905),
    [sym__ch_to_wc_int] = ACTIONS(1905),
    [sym__wc_find_string_int] = ACTIONS(1905),
    [sym__wc_left_string_int] = ACTIONS(1905),
    [sym__wc_lower_string_int] = ACTIONS(1905),
    [sym__wc_mid_string_int] = ACTIONS(1905),
    [sym__wc_remove_string_int] = ACTIONS(1905),
    [sym__wc_right_string_int] = ACTIONS(1905),
    [sym__wc_upper_string_int] = ACTIONS(1905),
    [sym__wc_compare_string_int] = ACTIONS(1905),
    [sym__wc_get_buffer_char_int] = ACTIONS(1905),
    [sym__wc_get_buffer_string_int] = ACTIONS(1905),
    [sym__wc_concat_string_int] = ACTIONS(1905),
    [sym__wc_file_open_int] = ACTIONS(1905),
    [sym__wc_file_close_int] = ACTIONS(1905),
    [sym__wc_file_read_int] = ACTIONS(1905),
    [sym__wc_file_read_line_int] = ACTIONS(1905),
    [sym__wc_file_write_int] = ACTIONS(1905),
    [sym__wc_file_write_line_int] = ACTIONS(1905),
    [aux_sym_preproc_include_token1] = ACTIONS(1905),
    [aux_sym_preproc_define_token1] = ACTIONS(1907),
    [aux_sym_preproc_warn_token1] = ACTIONS(1907),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1907),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1907),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1907),
    [anon_sym_LBRACK] = ACTIONS(1907),
    [anon_sym_LBRACE] = ACTIONS(1907),
    [anon_sym_RBRACE] = ACTIONS(1907),
    [anon_sym_LPAREN] = ACTIONS(1905),
    [anon_sym_BANG] = ACTIONS(1907),
    [anon_sym_TILDE] = ACTIONS(1907),
    [anon_sym_DASH] = ACTIONS(1905),
    [anon_sym_PLUS] = ACTIONS(1905),
    [anon_sym_DASH_DASH] = ACTIONS(1907),
    [anon_sym_PLUS_PLUS] = ACTIONS(1907),
    [anon_sym_DQUOTE] = ACTIONS(1907),
    [anon_sym_SQUOTE] = ACTIONS(1907),
    [sym_number_literal] = ACTIONS(1907),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(528)] = {
    [sym_identifier] = ACTIONS(1909),
    [sym_define_call_keyword] = ACTIONS(1909),
    [sym_define_function_keyword] = ACTIONS(1909),
    [sym_define_library_function_keyword] = ACTIONS(1909),
    [sym_define_module_keyword] = ACTIONS(1909),
    [sym_char_keyword] = ACTIONS(1909),
    [sym_widechar_keyword] = ACTIONS(1909),
    [sym_integer_keyword] = ACTIONS(1909),
    [sym_sinteger_keyword] = ACTIONS(1909),
    [sym_long_keyword] = ACTIONS(1909),
    [sym_slong_keyword] = ACTIONS(1909),
    [sym_float_keyword] = ACTIONS(1909),
    [sym_double_keyword] = ACTIONS(1909),
    [sym_variant_keyword] = ACTIONS(1909),
    [sym_variantarray_keyword] = ACTIONS(1909),
    [sym_local_var_keyword] = ACTIONS(1909),
    [sym_stack_var_keyword] = ACTIONS(1909),
    [sym_constant_keyword] = ACTIONS(1909),
    [sym_volatile_keyword] = ACTIONS(1909),
    [sym_non_volatile_keyword] = ACTIONS(1909),
    [sym_persistent_keyword] = ACTIONS(1909),
    [sym_dev_keyword] = ACTIONS(1909),
    [sym_devlev_keyword] = ACTIONS(1909),
    [sym_devchan_keyword] = ACTIONS(1909),
    [sym_if_keyword] = ACTIONS(1909),
    [sym_switch_keyword] = ACTIONS(1909),
    [sym_case_keyword] = ACTIONS(1909),
    [sym_default_keyword] = ACTIONS(1909),
    [sym_while_keyword] = ACTIONS(1909),
    [sym_for_keyword] = ACTIONS(1909),
    [sym_break_keyword] = ACTIONS(1909),
    [sym_continue_keyword] = ACTIONS(1909),
    [sym_return_keyword] = ACTIONS(1909),
    [sym_select_keyword] = ACTIONS(1909),
    [sym_active_keyword] = ACTIONS(1909),
    [sym_struct_keyword] = ACTIONS(1909),
    [sym_structure_keyword] = ACTIONS(1909),
    [aux_sym_on_keyword_token1] = ACTIONS(1909),
    [aux_sym_off_keyword_token1] = ACTIONS(1909),
    [sym_send_level_keyword] = ACTIONS(1909),
    [sym_send_string_keyword] = ACTIONS(1909),
    [sym_send_command_keyword] = ACTIONS(1909),
    [sym_clear_buffer_keyword] = ACTIONS(1909),
    [sym_create_buffer_keyword] = ACTIONS(1909),
    [sym_create_multi_buffer_keyword] = ACTIONS(1909),
    [sym_call_keyword] = ACTIONS(1909),
    [sym_system_call_keyword] = ACTIONS(1909),
    [sym_devchan_to_keyword] = ACTIONS(1909),
    [sym_devchan_min_to_keyword] = ACTIONS(1909),
    [sym_devchan_total_off_keyword] = ACTIONS(1909),
    [sym_devchan_pulse_keyword] = ACTIONS(1909),
    [sym_wait_keyword] = ACTIONS(1909),
    [sym_cancel_wait_keyword] = ACTIONS(1909),
    [sym_wait_until_keyword] = ACTIONS(1909),
    [sym_cancel_wait_until_keyword] = ACTIONS(1909),
    [sym_cancel_all_wait_keyword] = ACTIONS(1909),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1909),
    [sym_not] = ACTIONS(1909),
    [sym_bnot] = ACTIONS(1909),
    [sym___file__] = ACTIONS(1909),
    [sym___date__] = ACTIONS(1909),
    [sym___ldate__] = ACTIONS(1909),
    [sym___time__] = ACTIONS(1909),
    [sym___line__] = ACTIONS(1909),
    [sym___name__] = ACTIONS(1909),
    [sym___netlinx__] = ACTIONS(1909),
    [sym_netlinx_axi_version] = ACTIONS(1909),
    [sym_url_flg_tcp] = ACTIONS(1909),
    [sym_url_flg_accinfopresent] = ACTIONS(1909),
    [sym_url_flg_temp] = ACTIONS(1909),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1909),
    [sym_url_flg_stat_mask] = ACTIONS(1909),
    [sym_url_flg_stat_lookup] = ACTIONS(1909),
    [sym_url_flg_stat_connecting] = ACTIONS(1909),
    [sym_url_flg_stat_waiting] = ACTIONS(1909),
    [sym_url_flg_stat_connected] = ACTIONS(1909),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1909),
    [sym_first_virtual_device] = ACTIONS(1909),
    [sym_first_local_port] = ACTIONS(1909),
    [sym_dynamic_virtual_device] = ACTIONS(1909),
    [sym_timeline_once] = ACTIONS(1909),
    [sym_timeline_repeat] = ACTIONS(1909),
    [sym_timeline_absolute] = ACTIONS(1909),
    [sym_timeline_relative] = ACTIONS(1909),
    [sym_do_push_timed_infinite] = ACTIONS(1909),
    [sym_source_type_no_address] = ACTIONS(1909),
    [sym_source_type_neuron_id] = ACTIONS(1909),
    [sym_source_type_ip_address] = ACTIONS(1909),
    [sym_source_type_axlink] = ACTIONS(1909),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1909),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1909),
    [sym_source_type_ip_socket_address] = ACTIONS(1909),
    [sym_source_type_rs232] = ACTIONS(1909),
    [sym_source_type_internal] = ACTIONS(1909),
    [sym_source_type_mac_address] = ACTIONS(1909),
    [sym_source_type_ipv4_port] = ACTIONS(1909),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1909),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1909),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1909),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1909),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1909),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1909),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1909),
    [sym_true] = ACTIONS(1909),
    [sym_false] = ACTIONS(1909),
    [sym_file_read_only] = ACTIONS(1909),
    [sym_file_rw_new] = ACTIONS(1909),
    [sym_file_rw_append] = ACTIONS(1909),
    [sym_ip_tcp] = ACTIONS(1909),
    [sym_ip_udp] = ACTIONS(1909),
    [sym_ip_udp_2way] = ACTIONS(1909),
    [sym_xml_encode_types] = ACTIONS(1909),
    [sym_xml_encode_char_as_list] = ACTIONS(1909),
    [sym_xml_encode_le] = ACTIONS(1909),
    [sym_xml_decode_types] = ACTIONS(1909),
    [sym_xml_decode_no_preserve] = ACTIONS(1909),
    [sym_duet_dev_type_display_device] = ACTIONS(1909),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1909),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1909),
    [sym_duet_dev_type_io_device] = ACTIONS(1909),
    [sym_duet_dev_type_relay_device] = ACTIONS(1909),
    [sym_duet_dev_type_ups] = ACTIONS(1909),
    [sym_duet_dev_type_amplifier] = ACTIONS(1909),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1909),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1909),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1909),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1909),
    [sym_duet_dev_type_camera] = ACTIONS(1909),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1909),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1909),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1909),
    [sym_duet_dev_type_dss] = ACTIONS(1909),
    [sym_duet_dev_type_dvr] = ACTIONS(1909),
    [sym_duet_dev_type_disc_device] = ACTIONS(1909),
    [sym_duet_dev_type_document_camera] = ACTIONS(1909),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1909),
    [sym_duet_dev_type_hvac] = ACTIONS(1909),
    [sym_duet_dev_type_keypad] = ACTIONS(1909),
    [sym_duet_dev_type_light] = ACTIONS(1909),
    [sym_duet_dev_type_monitor] = ACTIONS(1909),
    [sym_duet_dev_type_motor] = ACTIONS(1909),
    [sym_duet_dev_type_multi_window] = ACTIONS(1909),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1909),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1909),
    [sym_duet_dev_type_receiver] = ACTIONS(1909),
    [sym_duet_dev_type_security_system] = ACTIONS(1909),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1909),
    [sym_duet_dev_type_settop_box] = ACTIONS(1909),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1909),
    [sym_duet_dev_type_switcher] = ACTIONS(1909),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1909),
    [sym_duet_dev_type_tv] = ACTIONS(1909),
    [sym_duet_dev_type_utility] = ACTIONS(1909),
    [sym_duet_dev_type_vcr] = ACTIONS(1909),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1909),
    [sym_duet_dev_type_video_processor] = ACTIONS(1909),
    [sym_duet_dev_type_video_projector] = ACTIONS(1909),
    [sym_duet_dev_type_video_wall] = ACTIONS(1909),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1909),
    [sym_duet_dev_type_weather] = ACTIONS(1909),
    [sym_duet_dev_not_polled] = ACTIONS(1909),
    [sym_duet_dev_polled] = ACTIONS(1909),
    [sym_clkmgr_mode_network] = ACTIONS(1909),
    [sym_clkmgr_mode_standalone] = ACTIONS(1909),
    [sym_normal_standby] = ACTIONS(1909),
    [sym_normal_wake] = ACTIONS(1909),
    [sym_ip_multicast_ttl_option] = ACTIONS(1909),
    [sym_ip_ttl_subnet] = ACTIONS(1909),
    [sym_ip_ttl_site] = ACTIONS(1909),
    [sym_ip_ttl_region] = ACTIONS(1909),
    [sym_ip_ttl_continent] = ACTIONS(1909),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1909),
    [sym_ip_nodelay_on] = ACTIONS(1909),
    [sym_ip_nodelay_off] = ACTIONS(1909),
    [sym_amx_error] = ACTIONS(1909),
    [sym_amx_warning] = ACTIONS(1909),
    [sym_amx_info] = ACTIONS(1909),
    [sym_amx_debug] = ACTIONS(1909),
    [sym_netlinx_login_success] = ACTIONS(1909),
    [sym_netlinx_login_fail] = ACTIONS(1909),
    [sym_netlinx_logout] = ACTIONS(1909),
    [sym_null_str] = ACTIONS(1909),
    [sym_smtp_address] = ACTIONS(1909),
    [sym_smtp_port_number] = ACTIONS(1909),
    [sym_smtp_username] = ACTIONS(1909),
    [sym_smtp_password] = ACTIONS(1909),
    [sym_smtp_require_tls] = ACTIONS(1909),
    [sym_smtp_from] = ACTIONS(1909),
    [sym_smtp_tls_true] = ACTIONS(1909),
    [sym_smtp_tls_false] = ACTIONS(1909),
    [sym_listview_on_row_select_event] = ACTIONS(1909),
    [sym_data_structure_unknown] = ACTIONS(1909),
    [sym_data_structure_datafeed] = ACTIONS(1909),
    [sym_data_structure_datarecordset] = ACTIONS(1909),
    [sym_data_structure_datarecord] = ACTIONS(1909),
    [sym_data_structure_datafield] = ACTIONS(1909),
    [sym_data_max_content_field_count] = ACTIONS(1909),
    [sym_data_max_metadata_field_count] = ACTIONS(1909),
    [sym_data_max_id_length] = ACTIONS(1909),
    [sym_data_max_name_length] = ACTIONS(1909),
    [sym_data_max_description_length] = ACTIONS(1909),
    [sym_data_max_type_length] = ACTIONS(1909),
    [sym_data_max_format_length] = ACTIONS(1909),
    [sym_data_max_source_length] = ACTIONS(1909),
    [sym_data_max_value_length] = ACTIONS(1909),
    [sym_data_max_label_length] = ACTIONS(1909),
    [sym_data_max_url_length] = ACTIONS(1909),
    [sym_max_last_login_info_length] = ACTIONS(1909),
    [sym_data_type_unknown] = ACTIONS(1909),
    [sym_data_type_string] = ACTIONS(1909),
    [sym_data_type_datetime] = ACTIONS(1909),
    [sym_data_type_date] = ACTIONS(1909),
    [sym_data_type_time] = ACTIONS(1909),
    [sym_data_type_image] = ACTIONS(1909),
    [sym_data_format_url] = ACTIONS(1909),
    [sym_data_format_phone] = ACTIONS(1909),
    [sym_data_format_email] = ACTIONS(1909),
    [sym_data_format_iso8601] = ACTIONS(1909),
    [sym_auditlog_priv] = ACTIONS(1909),
    [sym_configuration_priv] = ACTIONS(1909),
    [sym_deviceconfig_priv] = ACTIONS(1909),
    [sym_ftp_priv] = ACTIONS(1909),
    [sym_http_priv] = ACTIONS(1909),
    [sym_networkconfig_priv] = ACTIONS(1909),
    [sym_programport_priv] = ACTIONS(1909),
    [sym_remoteui_priv] = ACTIONS(1909),
    [sym_securitycontrol_priv] = ACTIONS(1909),
    [sym_softwaremanagement_priv] = ACTIONS(1909),
    [sym_terminal_priv] = ACTIONS(1909),
    [sym_tpadmin_priv] = ACTIONS(1909),
    [sym_user1_priv] = ACTIONS(1909),
    [sym_user2_priv] = ACTIONS(1909),
    [sym_user3_priv] = ACTIONS(1909),
    [sym_user4_priv] = ACTIONS(1909),
    [sym_usermanagement_priv] = ACTIONS(1909),
    [sym_valid_account] = ACTIONS(1909),
    [sym_err_name_invalid] = ACTIONS(1909),
    [sym_err_password_invalid] = ACTIONS(1909),
    [sym_err_invalid_account] = ACTIONS(1909),
    [sym_err_no_authorization] = ACTIONS(1909),
    [sym_err_invalid_parameter] = ACTIONS(1909),
    [sym_err_user_locked_out] = ACTIONS(1909),
    [sym_err_user_account_expired] = ACTIONS(1909),
    [sym_err_general_error] = ACTIONS(1909),
    [sym_err_authentication_error] = ACTIONS(1909),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1909),
    [sym_tls_validate_certificate] = ACTIONS(1909),
    [sym_tdata] = ACTIONS(1909),
    [sym_tchannel] = ACTIONS(1909),
    [sym_tlevel] = ACTIONS(1909),
    [sym_tbutton] = ACTIONS(1909),
    [sym_ttimeline] = ACTIONS(1909),
    [sym_tcustom] = ACTIONS(1909),
    [sym_url_struct] = ACTIONS(1909),
    [sym_dns_struct] = ACTIONS(1909),
    [sym_ip_address_struct] = ACTIONS(1909),
    [sym_dev_info_struct] = ACTIONS(1909),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1909),
    [sym_data_feed] = ACTIONS(1909),
    [sym_data_field] = ACTIONS(1909),
    [sym_data_record] = ACTIONS(1909),
    [sym_wc_data_feed] = ACTIONS(1909),
    [sym_wc_data_field] = ACTIONS(1909),
    [sym_wc_data_record] = ACTIONS(1909),
    [sym_last_login_info] = ACTIONS(1909),
    [sym____reserved___] = ACTIONS(1909),
    [sym_date] = ACTIONS(1909),
    [sym_day] = ACTIONS(1909),
    [sym_time] = ACTIONS(1909),
    [sym_ldate] = ACTIONS(1909),
    [sym_push_device] = ACTIONS(1909),
    [sym_push_channel] = ACTIONS(1909),
    [sym_push_devchan] = ACTIONS(1909),
    [sym_release_device] = ACTIONS(1909),
    [sym_release_channel] = ACTIONS(1909),
    [sym_release_devchan] = ACTIONS(1909),
    [sym_master_slot] = ACTIONS(1909),
    [sym_get_pulse_time] = ACTIONS(1909),
    [sym_get_timer] = ACTIONS(1909),
    [sym_master_sn] = ACTIONS(1909),
    [sym_system_number] = ACTIONS(1909),
    [sym_button] = ACTIONS(1909),
    [sym_dv_channel] = ACTIONS(1909),
    [sym_data] = ACTIONS(1909),
    [sym_level] = ACTIONS(1909),
    [sym_timeline] = ACTIONS(1909),
    [sym_channel] = ACTIONS(1909),
    [sym_custom] = ACTIONS(1909),
    [sym_atoi] = ACTIONS(1909),
    [sym_atol] = ACTIONS(1909),
    [sym_atof] = ACTIONS(1909),
    [sym_itoa] = ACTIONS(1909),
    [sym_itohex] = ACTIONS(1909),
    [sym_hextoi] = ACTIONS(1909),
    [sym_ftoa] = ACTIONS(1909),
    [sym_find_string] = ACTIONS(1909),
    [sym_left_string] = ACTIONS(1909),
    [sym_right_string] = ACTIONS(1909),
    [sym_mid_string] = ACTIONS(1909),
    [sym_remove_string] = ACTIONS(1909),
    [sym_get_buffer_string] = ACTIONS(1909),
    [sym_get_buffer_char] = ACTIONS(1909),
    [sym_get_multi_buffer_string] = ACTIONS(1909),
    [sym_length_string] = ACTIONS(1909),
    [sym_max_length_string] = ACTIONS(1909),
    [sym_set_length_string] = ACTIONS(1909),
    [sym_compare_string] = ACTIONS(1909),
    [sym_upper_string] = ACTIONS(1909),
    [sym_lower_string] = ACTIONS(1909),
    [sym_get_last] = ACTIONS(1909),
    [sym_date_to_day] = ACTIONS(1909),
    [sym_date_to_month] = ACTIONS(1909),
    [sym_date_to_year] = ACTIONS(1909),
    [sym_time_to_hour] = ACTIONS(1909),
    [sym_time_to_minute] = ACTIONS(1909),
    [sym_time_to_second] = ACTIONS(1909),
    [sym_day_of_week] = ACTIONS(1909),
    [sym_set_pulse_time] = ACTIONS(1909),
    [sym_set_timer] = ACTIONS(1909),
    [sym_file_open] = ACTIONS(1909),
    [sym_file_close] = ACTIONS(1909),
    [sym_file_read] = ACTIONS(1909),
    [sym_file_read_line] = ACTIONS(1909),
    [sym_file_write] = ACTIONS(1909),
    [sym_file_write_line] = ACTIONS(1909),
    [sym_file_copy] = ACTIONS(1909),
    [sym_file_delete] = ACTIONS(1909),
    [sym_file_rename] = ACTIONS(1909),
    [sym_file_dir] = ACTIONS(1909),
    [sym_file_getdir] = ACTIONS(1909),
    [sym_file_setdir] = ACTIONS(1909),
    [sym_file_create_dir] = ACTIONS(1909),
    [sym_file_remove_dir] = ACTIONS(1909),
    [sym_file_seek] = ACTIONS(1909),
    [sym_length_array] = ACTIONS(1909),
    [sym_max_length_array] = ACTIONS(1909),
    [sym_set_length_array] = ACTIONS(1909),
    [sym_ip_client_open] = ACTIONS(1909),
    [sym_ip_client_close] = ACTIONS(1909),
    [sym_ip_server_open] = ACTIONS(1909),
    [sym_ip_server_close] = ACTIONS(1909),
    [sym_ip_bound_client_open] = ACTIONS(1909),
    [sym_ip_mc_server_open] = ACTIONS(1909),
    [sym_ip_set_option] = ACTIONS(1909),
    [sym_ssh_client_open] = ACTIONS(1909),
    [sym_ssh_client_close] = ACTIONS(1909),
    [sym_tls_client_open] = ACTIONS(1909),
    [sym_tls_client_close] = ACTIONS(1909),
    [sym_do_push] = ACTIONS(1909),
    [sym_do_push_timed] = ACTIONS(1909),
    [sym_do_release] = ACTIONS(1909),
    [sym_device_id] = ACTIONS(1909),
    [sym_device_info] = ACTIONS(1909),
    [sym_device_id_string] = ACTIONS(1909),
    [sym_device_standby] = ACTIONS(1909),
    [sym_device_wake] = ACTIONS(1909),
    [sym_timeline_create] = ACTIONS(1909),
    [sym_timeline_kill] = ACTIONS(1909),
    [sym_timeline_reload] = ACTIONS(1909),
    [sym_timeline_pause] = ACTIONS(1909),
    [sym_timeline_restart] = ACTIONS(1909),
    [sym_timeline_set] = ACTIONS(1909),
    [sym_timeline_get] = ACTIONS(1909),
    [sym_timeline_active] = ACTIONS(1909),
    [sym_timeline_dynamic_id] = ACTIONS(1909),
    [sym_variable_to_string] = ACTIONS(1909),
    [sym_string_to_variable] = ACTIONS(1909),
    [sym_length_variable_to_string] = ACTIONS(1909),
    [sym_type_cast] = ACTIONS(1909),
    [sym_variable_to_xml] = ACTIONS(1909),
    [sym_xml_to_variable] = ACTIONS(1909),
    [sym_length_variable_to_xml] = ACTIONS(1909),
    [sym_redirect_string] = ACTIONS(1909),
    [sym_random_number] = ACTIONS(1909),
    [sym_get_unique_id] = ACTIONS(1909),
    [sym_get_system_number] = ACTIONS(1909),
    [sym_set_system_number] = ACTIONS(1909),
    [sym_get_serial_number] = ACTIONS(1909),
    [sym_get_master_build] = ACTIONS(1909),
    [sym_set_validation_code] = ACTIONS(1909),
    [sym_reboot] = ACTIONS(1909),
    [sym_load_duet_module] = ACTIONS(1909),
    [sym_unload_duet_module] = ACTIONS(1909),
    [sym_duet_mem_size_set] = ACTIONS(1909),
    [sym_duet_mem_size_get] = ACTIONS(1909),
    [sym_dynamic_application_device] = ACTIONS(1909),
    [sym_dynamic_polled_port] = ACTIONS(1909),
    [sym_static_port_binding] = ACTIONS(1909),
    [sym_static_ip_binding] = ACTIONS(1909),
    [sym_get_dns_list] = ACTIONS(1909),
    [sym_set_dns_list] = ACTIONS(1909),
    [sym_get_ip_address] = ACTIONS(1909),
    [sym_set_ip_address] = ACTIONS(1909),
    [sym_get_url_list] = ACTIONS(1909),
    [sym_add_url_entry] = ACTIONS(1909),
    [sym_delete_url_entry] = ACTIONS(1909),
    [sym_abs_value] = ACTIONS(1909),
    [sym_max_value] = ACTIONS(1909),
    [sym_min_value] = ACTIONS(1909),
    [sym_exp_value] = ACTIONS(1909),
    [sym_log_value] = ACTIONS(1909),
    [sym_log10_value] = ACTIONS(1909),
    [sym_power_value] = ACTIONS(1909),
    [sym_sqrt_value] = ACTIONS(1909),
    [sym_raw_be] = ACTIONS(1909),
    [sym_raw_le] = ACTIONS(1909),
    [sym_internal_queue_size_set] = ACTIONS(1909),
    [sym_internal_queue_size_get] = ACTIONS(1909),
    [sym_internal_threshold_set] = ACTIONS(1909),
    [sym_internal_threshold_get] = ACTIONS(1909),
    [sym_rebuilt_event] = ACTIONS(1909),
    [sym_do_custom_event] = ACTIONS(1909),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1909),
    [sym_clkmgr_set_clk_source] = ACTIONS(1909),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1909),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1909),
    [sym_clkmgr_get_timezone] = ACTIONS(1909),
    [sym_clkmgr_set_timezone] = ACTIONS(1909),
    [sym_clkmgr_get_resync_period] = ACTIONS(1909),
    [sym_clkmgr_set_resync_period] = ACTIONS(1909),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1909),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1909),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1909),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1909),
    [sym_clkmgr_get_timeservers] = ACTIONS(1909),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1909),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1909),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1909),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1909),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1909),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1909),
    [sym_set_log_level] = ACTIONS(1909),
    [sym_get_log_level] = ACTIONS(1909),
    [sym_amx_log] = ACTIONS(1909),
    [sym_smtp_server_config_set] = ACTIONS(1909),
    [sym_smtp_server_config_get] = ACTIONS(1909),
    [sym_smtp_send] = ACTIONS(1909),
    [sym_validate_netlinx_account] = ACTIONS(1909),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1909),
    [sym_authenticate_certificate] = ACTIONS(1909),
    [sym_audit_netlinx_session_event] = ACTIONS(1909),
    [sym_audit_netlinx_generic_event] = ACTIONS(1909),
    [sym_get_available_flash_disk_space] = ACTIONS(1909),
    [sym_get_max_flash_disk_space] = ACTIONS(1909),
    [sym_data_create_feed] = ACTIONS(1909),
    [sym_data_delete_feed] = ACTIONS(1909),
    [sym_data_publish_feed] = ACTIONS(1909),
    [sym_data_get_published_feed] = ACTIONS(1909),
    [sym_data_add_record] = ACTIONS(1909),
    [sym_data_get_event_record] = ACTIONS(1909),
    [sym__wc_data_create_feed] = ACTIONS(1909),
    [sym__wc_data_add_record] = ACTIONS(1909),
    [sym__wc_data_get_event_record] = ACTIONS(1909),
    [sym_format] = ACTIONS(1909),
    [sym_astro_clock] = ACTIONS(1909),
    [sym_set_outdoor_temperature] = ACTIONS(1909),
    [sym_set_virtual_level_count] = ACTIONS(1909),
    [sym_set_virtual_channel_count] = ACTIONS(1909),
    [sym_set_virtual_port_count] = ACTIONS(1909),
    [sym__wc_encode_int] = ACTIONS(1909),
    [sym__wc_decode_int] = ACTIONS(1909),
    [sym__wc_int] = ACTIONS(1909),
    [sym__wc_to_ch_int] = ACTIONS(1909),
    [sym__ch_to_wc_int] = ACTIONS(1909),
    [sym__wc_find_string_int] = ACTIONS(1909),
    [sym__wc_left_string_int] = ACTIONS(1909),
    [sym__wc_lower_string_int] = ACTIONS(1909),
    [sym__wc_mid_string_int] = ACTIONS(1909),
    [sym__wc_remove_string_int] = ACTIONS(1909),
    [sym__wc_right_string_int] = ACTIONS(1909),
    [sym__wc_upper_string_int] = ACTIONS(1909),
    [sym__wc_compare_string_int] = ACTIONS(1909),
    [sym__wc_get_buffer_char_int] = ACTIONS(1909),
    [sym__wc_get_buffer_string_int] = ACTIONS(1909),
    [sym__wc_concat_string_int] = ACTIONS(1909),
    [sym__wc_file_open_int] = ACTIONS(1909),
    [sym__wc_file_close_int] = ACTIONS(1909),
    [sym__wc_file_read_int] = ACTIONS(1909),
    [sym__wc_file_read_line_int] = ACTIONS(1909),
    [sym__wc_file_write_int] = ACTIONS(1909),
    [sym__wc_file_write_line_int] = ACTIONS(1909),
    [aux_sym_preproc_include_token1] = ACTIONS(1909),
    [aux_sym_preproc_define_token1] = ACTIONS(1911),
    [aux_sym_preproc_warn_token1] = ACTIONS(1911),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1911),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1911),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1911),
    [anon_sym_LBRACK] = ACTIONS(1911),
    [anon_sym_LBRACE] = ACTIONS(1911),
    [anon_sym_RBRACE] = ACTIONS(1911),
    [anon_sym_LPAREN] = ACTIONS(1909),
    [anon_sym_BANG] = ACTIONS(1911),
    [anon_sym_TILDE] = ACTIONS(1911),
    [anon_sym_DASH] = ACTIONS(1909),
    [anon_sym_PLUS] = ACTIONS(1909),
    [anon_sym_DASH_DASH] = ACTIONS(1911),
    [anon_sym_PLUS_PLUS] = ACTIONS(1911),
    [anon_sym_DQUOTE] = ACTIONS(1911),
    [anon_sym_SQUOTE] = ACTIONS(1911),
    [sym_number_literal] = ACTIONS(1911),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(529)] = {
    [sym_identifier] = ACTIONS(2021),
    [sym_define_call_keyword] = ACTIONS(2021),
    [sym_define_function_keyword] = ACTIONS(2021),
    [sym_define_library_function_keyword] = ACTIONS(2021),
    [sym_define_module_keyword] = ACTIONS(2021),
    [sym_char_keyword] = ACTIONS(2021),
    [sym_widechar_keyword] = ACTIONS(2021),
    [sym_integer_keyword] = ACTIONS(2021),
    [sym_sinteger_keyword] = ACTIONS(2021),
    [sym_long_keyword] = ACTIONS(2021),
    [sym_slong_keyword] = ACTIONS(2021),
    [sym_float_keyword] = ACTIONS(2021),
    [sym_double_keyword] = ACTIONS(2021),
    [sym_variant_keyword] = ACTIONS(2021),
    [sym_variantarray_keyword] = ACTIONS(2021),
    [sym_local_var_keyword] = ACTIONS(2021),
    [sym_stack_var_keyword] = ACTIONS(2021),
    [sym_constant_keyword] = ACTIONS(2021),
    [sym_volatile_keyword] = ACTIONS(2021),
    [sym_non_volatile_keyword] = ACTIONS(2021),
    [sym_persistent_keyword] = ACTIONS(2021),
    [sym_dev_keyword] = ACTIONS(2021),
    [sym_devlev_keyword] = ACTIONS(2021),
    [sym_devchan_keyword] = ACTIONS(2021),
    [sym_if_keyword] = ACTIONS(2021),
    [sym_switch_keyword] = ACTIONS(2021),
    [sym_case_keyword] = ACTIONS(2021),
    [sym_default_keyword] = ACTIONS(2021),
    [sym_while_keyword] = ACTIONS(2021),
    [sym_for_keyword] = ACTIONS(2021),
    [sym_break_keyword] = ACTIONS(2021),
    [sym_continue_keyword] = ACTIONS(2021),
    [sym_return_keyword] = ACTIONS(2021),
    [sym_select_keyword] = ACTIONS(2021),
    [sym_active_keyword] = ACTIONS(2021),
    [sym_struct_keyword] = ACTIONS(2021),
    [sym_structure_keyword] = ACTIONS(2021),
    [aux_sym_on_keyword_token1] = ACTIONS(2021),
    [aux_sym_off_keyword_token1] = ACTIONS(2021),
    [sym_send_level_keyword] = ACTIONS(2021),
    [sym_send_string_keyword] = ACTIONS(2021),
    [sym_send_command_keyword] = ACTIONS(2021),
    [sym_clear_buffer_keyword] = ACTIONS(2021),
    [sym_create_buffer_keyword] = ACTIONS(2021),
    [sym_create_multi_buffer_keyword] = ACTIONS(2021),
    [sym_call_keyword] = ACTIONS(2021),
    [sym_system_call_keyword] = ACTIONS(2021),
    [sym_devchan_to_keyword] = ACTIONS(2021),
    [sym_devchan_min_to_keyword] = ACTIONS(2021),
    [sym_devchan_total_off_keyword] = ACTIONS(2021),
    [sym_devchan_pulse_keyword] = ACTIONS(2021),
    [sym_wait_keyword] = ACTIONS(2021),
    [sym_cancel_wait_keyword] = ACTIONS(2021),
    [sym_wait_until_keyword] = ACTIONS(2021),
    [sym_cancel_wait_until_keyword] = ACTIONS(2021),
    [sym_cancel_all_wait_keyword] = ACTIONS(2021),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2021),
    [sym_not] = ACTIONS(2021),
    [sym_bnot] = ACTIONS(2021),
    [sym___file__] = ACTIONS(2021),
    [sym___date__] = ACTIONS(2021),
    [sym___ldate__] = ACTIONS(2021),
    [sym___time__] = ACTIONS(2021),
    [sym___line__] = ACTIONS(2021),
    [sym___name__] = ACTIONS(2021),
    [sym___netlinx__] = ACTIONS(2021),
    [sym_netlinx_axi_version] = ACTIONS(2021),
    [sym_url_flg_tcp] = ACTIONS(2021),
    [sym_url_flg_accinfopresent] = ACTIONS(2021),
    [sym_url_flg_temp] = ACTIONS(2021),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2021),
    [sym_url_flg_stat_mask] = ACTIONS(2021),
    [sym_url_flg_stat_lookup] = ACTIONS(2021),
    [sym_url_flg_stat_connecting] = ACTIONS(2021),
    [sym_url_flg_stat_waiting] = ACTIONS(2021),
    [sym_url_flg_stat_connected] = ACTIONS(2021),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2021),
    [sym_first_virtual_device] = ACTIONS(2021),
    [sym_first_local_port] = ACTIONS(2021),
    [sym_dynamic_virtual_device] = ACTIONS(2021),
    [sym_timeline_once] = ACTIONS(2021),
    [sym_timeline_repeat] = ACTIONS(2021),
    [sym_timeline_absolute] = ACTIONS(2021),
    [sym_timeline_relative] = ACTIONS(2021),
    [sym_do_push_timed_infinite] = ACTIONS(2021),
    [sym_source_type_no_address] = ACTIONS(2021),
    [sym_source_type_neuron_id] = ACTIONS(2021),
    [sym_source_type_ip_address] = ACTIONS(2021),
    [sym_source_type_axlink] = ACTIONS(2021),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2021),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2021),
    [sym_source_type_ip_socket_address] = ACTIONS(2021),
    [sym_source_type_rs232] = ACTIONS(2021),
    [sym_source_type_internal] = ACTIONS(2021),
    [sym_source_type_mac_address] = ACTIONS(2021),
    [sym_source_type_ipv4_port] = ACTIONS(2021),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2021),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2021),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2021),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2021),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2021),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2021),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2021),
    [sym_true] = ACTIONS(2021),
    [sym_false] = ACTIONS(2021),
    [sym_file_read_only] = ACTIONS(2021),
    [sym_file_rw_new] = ACTIONS(2021),
    [sym_file_rw_append] = ACTIONS(2021),
    [sym_ip_tcp] = ACTIONS(2021),
    [sym_ip_udp] = ACTIONS(2021),
    [sym_ip_udp_2way] = ACTIONS(2021),
    [sym_xml_encode_types] = ACTIONS(2021),
    [sym_xml_encode_char_as_list] = ACTIONS(2021),
    [sym_xml_encode_le] = ACTIONS(2021),
    [sym_xml_decode_types] = ACTIONS(2021),
    [sym_xml_decode_no_preserve] = ACTIONS(2021),
    [sym_duet_dev_type_display_device] = ACTIONS(2021),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2021),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2021),
    [sym_duet_dev_type_io_device] = ACTIONS(2021),
    [sym_duet_dev_type_relay_device] = ACTIONS(2021),
    [sym_duet_dev_type_ups] = ACTIONS(2021),
    [sym_duet_dev_type_amplifier] = ACTIONS(2021),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2021),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2021),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2021),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2021),
    [sym_duet_dev_type_camera] = ACTIONS(2021),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2021),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2021),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2021),
    [sym_duet_dev_type_dss] = ACTIONS(2021),
    [sym_duet_dev_type_dvr] = ACTIONS(2021),
    [sym_duet_dev_type_disc_device] = ACTIONS(2021),
    [sym_duet_dev_type_document_camera] = ACTIONS(2021),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2021),
    [sym_duet_dev_type_hvac] = ACTIONS(2021),
    [sym_duet_dev_type_keypad] = ACTIONS(2021),
    [sym_duet_dev_type_light] = ACTIONS(2021),
    [sym_duet_dev_type_monitor] = ACTIONS(2021),
    [sym_duet_dev_type_motor] = ACTIONS(2021),
    [sym_duet_dev_type_multi_window] = ACTIONS(2021),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2021),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2021),
    [sym_duet_dev_type_receiver] = ACTIONS(2021),
    [sym_duet_dev_type_security_system] = ACTIONS(2021),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2021),
    [sym_duet_dev_type_settop_box] = ACTIONS(2021),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2021),
    [sym_duet_dev_type_switcher] = ACTIONS(2021),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2021),
    [sym_duet_dev_type_tv] = ACTIONS(2021),
    [sym_duet_dev_type_utility] = ACTIONS(2021),
    [sym_duet_dev_type_vcr] = ACTIONS(2021),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2021),
    [sym_duet_dev_type_video_processor] = ACTIONS(2021),
    [sym_duet_dev_type_video_projector] = ACTIONS(2021),
    [sym_duet_dev_type_video_wall] = ACTIONS(2021),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2021),
    [sym_duet_dev_type_weather] = ACTIONS(2021),
    [sym_duet_dev_not_polled] = ACTIONS(2021),
    [sym_duet_dev_polled] = ACTIONS(2021),
    [sym_clkmgr_mode_network] = ACTIONS(2021),
    [sym_clkmgr_mode_standalone] = ACTIONS(2021),
    [sym_normal_standby] = ACTIONS(2021),
    [sym_normal_wake] = ACTIONS(2021),
    [sym_ip_multicast_ttl_option] = ACTIONS(2021),
    [sym_ip_ttl_subnet] = ACTIONS(2021),
    [sym_ip_ttl_site] = ACTIONS(2021),
    [sym_ip_ttl_region] = ACTIONS(2021),
    [sym_ip_ttl_continent] = ACTIONS(2021),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2021),
    [sym_ip_nodelay_on] = ACTIONS(2021),
    [sym_ip_nodelay_off] = ACTIONS(2021),
    [sym_amx_error] = ACTIONS(2021),
    [sym_amx_warning] = ACTIONS(2021),
    [sym_amx_info] = ACTIONS(2021),
    [sym_amx_debug] = ACTIONS(2021),
    [sym_netlinx_login_success] = ACTIONS(2021),
    [sym_netlinx_login_fail] = ACTIONS(2021),
    [sym_netlinx_logout] = ACTIONS(2021),
    [sym_null_str] = ACTIONS(2021),
    [sym_smtp_address] = ACTIONS(2021),
    [sym_smtp_port_number] = ACTIONS(2021),
    [sym_smtp_username] = ACTIONS(2021),
    [sym_smtp_password] = ACTIONS(2021),
    [sym_smtp_require_tls] = ACTIONS(2021),
    [sym_smtp_from] = ACTIONS(2021),
    [sym_smtp_tls_true] = ACTIONS(2021),
    [sym_smtp_tls_false] = ACTIONS(2021),
    [sym_listview_on_row_select_event] = ACTIONS(2021),
    [sym_data_structure_unknown] = ACTIONS(2021),
    [sym_data_structure_datafeed] = ACTIONS(2021),
    [sym_data_structure_datarecordset] = ACTIONS(2021),
    [sym_data_structure_datarecord] = ACTIONS(2021),
    [sym_data_structure_datafield] = ACTIONS(2021),
    [sym_data_max_content_field_count] = ACTIONS(2021),
    [sym_data_max_metadata_field_count] = ACTIONS(2021),
    [sym_data_max_id_length] = ACTIONS(2021),
    [sym_data_max_name_length] = ACTIONS(2021),
    [sym_data_max_description_length] = ACTIONS(2021),
    [sym_data_max_type_length] = ACTIONS(2021),
    [sym_data_max_format_length] = ACTIONS(2021),
    [sym_data_max_source_length] = ACTIONS(2021),
    [sym_data_max_value_length] = ACTIONS(2021),
    [sym_data_max_label_length] = ACTIONS(2021),
    [sym_data_max_url_length] = ACTIONS(2021),
    [sym_max_last_login_info_length] = ACTIONS(2021),
    [sym_data_type_unknown] = ACTIONS(2021),
    [sym_data_type_string] = ACTIONS(2021),
    [sym_data_type_datetime] = ACTIONS(2021),
    [sym_data_type_date] = ACTIONS(2021),
    [sym_data_type_time] = ACTIONS(2021),
    [sym_data_type_image] = ACTIONS(2021),
    [sym_data_format_url] = ACTIONS(2021),
    [sym_data_format_phone] = ACTIONS(2021),
    [sym_data_format_email] = ACTIONS(2021),
    [sym_data_format_iso8601] = ACTIONS(2021),
    [sym_auditlog_priv] = ACTIONS(2021),
    [sym_configuration_priv] = ACTIONS(2021),
    [sym_deviceconfig_priv] = ACTIONS(2021),
    [sym_ftp_priv] = ACTIONS(2021),
    [sym_http_priv] = ACTIONS(2021),
    [sym_networkconfig_priv] = ACTIONS(2021),
    [sym_programport_priv] = ACTIONS(2021),
    [sym_remoteui_priv] = ACTIONS(2021),
    [sym_securitycontrol_priv] = ACTIONS(2021),
    [sym_softwaremanagement_priv] = ACTIONS(2021),
    [sym_terminal_priv] = ACTIONS(2021),
    [sym_tpadmin_priv] = ACTIONS(2021),
    [sym_user1_priv] = ACTIONS(2021),
    [sym_user2_priv] = ACTIONS(2021),
    [sym_user3_priv] = ACTIONS(2021),
    [sym_user4_priv] = ACTIONS(2021),
    [sym_usermanagement_priv] = ACTIONS(2021),
    [sym_valid_account] = ACTIONS(2021),
    [sym_err_name_invalid] = ACTIONS(2021),
    [sym_err_password_invalid] = ACTIONS(2021),
    [sym_err_invalid_account] = ACTIONS(2021),
    [sym_err_no_authorization] = ACTIONS(2021),
    [sym_err_invalid_parameter] = ACTIONS(2021),
    [sym_err_user_locked_out] = ACTIONS(2021),
    [sym_err_user_account_expired] = ACTIONS(2021),
    [sym_err_general_error] = ACTIONS(2021),
    [sym_err_authentication_error] = ACTIONS(2021),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2021),
    [sym_tls_validate_certificate] = ACTIONS(2021),
    [sym_tdata] = ACTIONS(2021),
    [sym_tchannel] = ACTIONS(2021),
    [sym_tlevel] = ACTIONS(2021),
    [sym_tbutton] = ACTIONS(2021),
    [sym_ttimeline] = ACTIONS(2021),
    [sym_tcustom] = ACTIONS(2021),
    [sym_url_struct] = ACTIONS(2021),
    [sym_dns_struct] = ACTIONS(2021),
    [sym_ip_address_struct] = ACTIONS(2021),
    [sym_dev_info_struct] = ACTIONS(2021),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2021),
    [sym_data_feed] = ACTIONS(2021),
    [sym_data_field] = ACTIONS(2021),
    [sym_data_record] = ACTIONS(2021),
    [sym_wc_data_feed] = ACTIONS(2021),
    [sym_wc_data_field] = ACTIONS(2021),
    [sym_wc_data_record] = ACTIONS(2021),
    [sym_last_login_info] = ACTIONS(2021),
    [sym____reserved___] = ACTIONS(2021),
    [sym_date] = ACTIONS(2021),
    [sym_day] = ACTIONS(2021),
    [sym_time] = ACTIONS(2021),
    [sym_ldate] = ACTIONS(2021),
    [sym_push_device] = ACTIONS(2021),
    [sym_push_channel] = ACTIONS(2021),
    [sym_push_devchan] = ACTIONS(2021),
    [sym_release_device] = ACTIONS(2021),
    [sym_release_channel] = ACTIONS(2021),
    [sym_release_devchan] = ACTIONS(2021),
    [sym_master_slot] = ACTIONS(2021),
    [sym_get_pulse_time] = ACTIONS(2021),
    [sym_get_timer] = ACTIONS(2021),
    [sym_master_sn] = ACTIONS(2021),
    [sym_system_number] = ACTIONS(2021),
    [sym_button] = ACTIONS(2021),
    [sym_dv_channel] = ACTIONS(2021),
    [sym_data] = ACTIONS(2021),
    [sym_level] = ACTIONS(2021),
    [sym_timeline] = ACTIONS(2021),
    [sym_channel] = ACTIONS(2021),
    [sym_custom] = ACTIONS(2021),
    [sym_atoi] = ACTIONS(2021),
    [sym_atol] = ACTIONS(2021),
    [sym_atof] = ACTIONS(2021),
    [sym_itoa] = ACTIONS(2021),
    [sym_itohex] = ACTIONS(2021),
    [sym_hextoi] = ACTIONS(2021),
    [sym_ftoa] = ACTIONS(2021),
    [sym_find_string] = ACTIONS(2021),
    [sym_left_string] = ACTIONS(2021),
    [sym_right_string] = ACTIONS(2021),
    [sym_mid_string] = ACTIONS(2021),
    [sym_remove_string] = ACTIONS(2021),
    [sym_get_buffer_string] = ACTIONS(2021),
    [sym_get_buffer_char] = ACTIONS(2021),
    [sym_get_multi_buffer_string] = ACTIONS(2021),
    [sym_length_string] = ACTIONS(2021),
    [sym_max_length_string] = ACTIONS(2021),
    [sym_set_length_string] = ACTIONS(2021),
    [sym_compare_string] = ACTIONS(2021),
    [sym_upper_string] = ACTIONS(2021),
    [sym_lower_string] = ACTIONS(2021),
    [sym_get_last] = ACTIONS(2021),
    [sym_date_to_day] = ACTIONS(2021),
    [sym_date_to_month] = ACTIONS(2021),
    [sym_date_to_year] = ACTIONS(2021),
    [sym_time_to_hour] = ACTIONS(2021),
    [sym_time_to_minute] = ACTIONS(2021),
    [sym_time_to_second] = ACTIONS(2021),
    [sym_day_of_week] = ACTIONS(2021),
    [sym_set_pulse_time] = ACTIONS(2021),
    [sym_set_timer] = ACTIONS(2021),
    [sym_file_open] = ACTIONS(2021),
    [sym_file_close] = ACTIONS(2021),
    [sym_file_read] = ACTIONS(2021),
    [sym_file_read_line] = ACTIONS(2021),
    [sym_file_write] = ACTIONS(2021),
    [sym_file_write_line] = ACTIONS(2021),
    [sym_file_copy] = ACTIONS(2021),
    [sym_file_delete] = ACTIONS(2021),
    [sym_file_rename] = ACTIONS(2021),
    [sym_file_dir] = ACTIONS(2021),
    [sym_file_getdir] = ACTIONS(2021),
    [sym_file_setdir] = ACTIONS(2021),
    [sym_file_create_dir] = ACTIONS(2021),
    [sym_file_remove_dir] = ACTIONS(2021),
    [sym_file_seek] = ACTIONS(2021),
    [sym_length_array] = ACTIONS(2021),
    [sym_max_length_array] = ACTIONS(2021),
    [sym_set_length_array] = ACTIONS(2021),
    [sym_ip_client_open] = ACTIONS(2021),
    [sym_ip_client_close] = ACTIONS(2021),
    [sym_ip_server_open] = ACTIONS(2021),
    [sym_ip_server_close] = ACTIONS(2021),
    [sym_ip_bound_client_open] = ACTIONS(2021),
    [sym_ip_mc_server_open] = ACTIONS(2021),
    [sym_ip_set_option] = ACTIONS(2021),
    [sym_ssh_client_open] = ACTIONS(2021),
    [sym_ssh_client_close] = ACTIONS(2021),
    [sym_tls_client_open] = ACTIONS(2021),
    [sym_tls_client_close] = ACTIONS(2021),
    [sym_do_push] = ACTIONS(2021),
    [sym_do_push_timed] = ACTIONS(2021),
    [sym_do_release] = ACTIONS(2021),
    [sym_device_id] = ACTIONS(2021),
    [sym_device_info] = ACTIONS(2021),
    [sym_device_id_string] = ACTIONS(2021),
    [sym_device_standby] = ACTIONS(2021),
    [sym_device_wake] = ACTIONS(2021),
    [sym_timeline_create] = ACTIONS(2021),
    [sym_timeline_kill] = ACTIONS(2021),
    [sym_timeline_reload] = ACTIONS(2021),
    [sym_timeline_pause] = ACTIONS(2021),
    [sym_timeline_restart] = ACTIONS(2021),
    [sym_timeline_set] = ACTIONS(2021),
    [sym_timeline_get] = ACTIONS(2021),
    [sym_timeline_active] = ACTIONS(2021),
    [sym_timeline_dynamic_id] = ACTIONS(2021),
    [sym_variable_to_string] = ACTIONS(2021),
    [sym_string_to_variable] = ACTIONS(2021),
    [sym_length_variable_to_string] = ACTIONS(2021),
    [sym_type_cast] = ACTIONS(2021),
    [sym_variable_to_xml] = ACTIONS(2021),
    [sym_xml_to_variable] = ACTIONS(2021),
    [sym_length_variable_to_xml] = ACTIONS(2021),
    [sym_redirect_string] = ACTIONS(2021),
    [sym_random_number] = ACTIONS(2021),
    [sym_get_unique_id] = ACTIONS(2021),
    [sym_get_system_number] = ACTIONS(2021),
    [sym_set_system_number] = ACTIONS(2021),
    [sym_get_serial_number] = ACTIONS(2021),
    [sym_get_master_build] = ACTIONS(2021),
    [sym_set_validation_code] = ACTIONS(2021),
    [sym_reboot] = ACTIONS(2021),
    [sym_load_duet_module] = ACTIONS(2021),
    [sym_unload_duet_module] = ACTIONS(2021),
    [sym_duet_mem_size_set] = ACTIONS(2021),
    [sym_duet_mem_size_get] = ACTIONS(2021),
    [sym_dynamic_application_device] = ACTIONS(2021),
    [sym_dynamic_polled_port] = ACTIONS(2021),
    [sym_static_port_binding] = ACTIONS(2021),
    [sym_static_ip_binding] = ACTIONS(2021),
    [sym_get_dns_list] = ACTIONS(2021),
    [sym_set_dns_list] = ACTIONS(2021),
    [sym_get_ip_address] = ACTIONS(2021),
    [sym_set_ip_address] = ACTIONS(2021),
    [sym_get_url_list] = ACTIONS(2021),
    [sym_add_url_entry] = ACTIONS(2021),
    [sym_delete_url_entry] = ACTIONS(2021),
    [sym_abs_value] = ACTIONS(2021),
    [sym_max_value] = ACTIONS(2021),
    [sym_min_value] = ACTIONS(2021),
    [sym_exp_value] = ACTIONS(2021),
    [sym_log_value] = ACTIONS(2021),
    [sym_log10_value] = ACTIONS(2021),
    [sym_power_value] = ACTIONS(2021),
    [sym_sqrt_value] = ACTIONS(2021),
    [sym_raw_be] = ACTIONS(2021),
    [sym_raw_le] = ACTIONS(2021),
    [sym_internal_queue_size_set] = ACTIONS(2021),
    [sym_internal_queue_size_get] = ACTIONS(2021),
    [sym_internal_threshold_set] = ACTIONS(2021),
    [sym_internal_threshold_get] = ACTIONS(2021),
    [sym_rebuilt_event] = ACTIONS(2021),
    [sym_do_custom_event] = ACTIONS(2021),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2021),
    [sym_clkmgr_set_clk_source] = ACTIONS(2021),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2021),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2021),
    [sym_clkmgr_get_timezone] = ACTIONS(2021),
    [sym_clkmgr_set_timezone] = ACTIONS(2021),
    [sym_clkmgr_get_resync_period] = ACTIONS(2021),
    [sym_clkmgr_set_resync_period] = ACTIONS(2021),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2021),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2021),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2021),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2021),
    [sym_clkmgr_get_timeservers] = ACTIONS(2021),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2021),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2021),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2021),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2021),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2021),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2021),
    [sym_set_log_level] = ACTIONS(2021),
    [sym_get_log_level] = ACTIONS(2021),
    [sym_amx_log] = ACTIONS(2021),
    [sym_smtp_server_config_set] = ACTIONS(2021),
    [sym_smtp_server_config_get] = ACTIONS(2021),
    [sym_smtp_send] = ACTIONS(2021),
    [sym_validate_netlinx_account] = ACTIONS(2021),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2021),
    [sym_authenticate_certificate] = ACTIONS(2021),
    [sym_audit_netlinx_session_event] = ACTIONS(2021),
    [sym_audit_netlinx_generic_event] = ACTIONS(2021),
    [sym_get_available_flash_disk_space] = ACTIONS(2021),
    [sym_get_max_flash_disk_space] = ACTIONS(2021),
    [sym_data_create_feed] = ACTIONS(2021),
    [sym_data_delete_feed] = ACTIONS(2021),
    [sym_data_publish_feed] = ACTIONS(2021),
    [sym_data_get_published_feed] = ACTIONS(2021),
    [sym_data_add_record] = ACTIONS(2021),
    [sym_data_get_event_record] = ACTIONS(2021),
    [sym__wc_data_create_feed] = ACTIONS(2021),
    [sym__wc_data_add_record] = ACTIONS(2021),
    [sym__wc_data_get_event_record] = ACTIONS(2021),
    [sym_format] = ACTIONS(2021),
    [sym_astro_clock] = ACTIONS(2021),
    [sym_set_outdoor_temperature] = ACTIONS(2021),
    [sym_set_virtual_level_count] = ACTIONS(2021),
    [sym_set_virtual_channel_count] = ACTIONS(2021),
    [sym_set_virtual_port_count] = ACTIONS(2021),
    [sym__wc_encode_int] = ACTIONS(2021),
    [sym__wc_decode_int] = ACTIONS(2021),
    [sym__wc_int] = ACTIONS(2021),
    [sym__wc_to_ch_int] = ACTIONS(2021),
    [sym__ch_to_wc_int] = ACTIONS(2021),
    [sym__wc_find_string_int] = ACTIONS(2021),
    [sym__wc_left_string_int] = ACTIONS(2021),
    [sym__wc_lower_string_int] = ACTIONS(2021),
    [sym__wc_mid_string_int] = ACTIONS(2021),
    [sym__wc_remove_string_int] = ACTIONS(2021),
    [sym__wc_right_string_int] = ACTIONS(2021),
    [sym__wc_upper_string_int] = ACTIONS(2021),
    [sym__wc_compare_string_int] = ACTIONS(2021),
    [sym__wc_get_buffer_char_int] = ACTIONS(2021),
    [sym__wc_get_buffer_string_int] = ACTIONS(2021),
    [sym__wc_concat_string_int] = ACTIONS(2021),
    [sym__wc_file_open_int] = ACTIONS(2021),
    [sym__wc_file_close_int] = ACTIONS(2021),
    [sym__wc_file_read_int] = ACTIONS(2021),
    [sym__wc_file_read_line_int] = ACTIONS(2021),
    [sym__wc_file_write_int] = ACTIONS(2021),
    [sym__wc_file_write_line_int] = ACTIONS(2021),
    [aux_sym_preproc_include_token1] = ACTIONS(2021),
    [aux_sym_preproc_define_token1] = ACTIONS(2023),
    [aux_sym_preproc_warn_token1] = ACTIONS(2023),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2023),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2023),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2023),
    [anon_sym_LBRACK] = ACTIONS(2023),
    [anon_sym_LBRACE] = ACTIONS(2023),
    [anon_sym_RBRACE] = ACTIONS(2023),
    [anon_sym_LPAREN] = ACTIONS(2021),
    [anon_sym_BANG] = ACTIONS(2023),
    [anon_sym_TILDE] = ACTIONS(2023),
    [anon_sym_DASH] = ACTIONS(2021),
    [anon_sym_PLUS] = ACTIONS(2021),
    [anon_sym_DASH_DASH] = ACTIONS(2023),
    [anon_sym_PLUS_PLUS] = ACTIONS(2023),
    [anon_sym_DQUOTE] = ACTIONS(2023),
    [anon_sym_SQUOTE] = ACTIONS(2023),
    [sym_number_literal] = ACTIONS(2023),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(530)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(311),
    [sym_statement] = STATE(220),
    [sym_expression_statement] = STATE(311),
    [sym_if_statement] = STATE(311),
    [sym_switch_statement] = STATE(311),
    [sym_case_statement] = STATE(311),
    [sym_while_statement] = STATE(311),
    [sym_for_statement] = STATE(311),
    [sym_select_statement] = STATE(311),
    [sym_active_statement] = STATE(311),
    [sym_return_statement] = STATE(311),
    [sym_break_statement] = STATE(311),
    [sym_continue_statement] = STATE(311),
    [sym_devchan_operation_statement] = STATE(311),
    [sym_devchan_operation] = STATE(1878),
    [sym_send_string_statement] = STATE(311),
    [sym_send_command_statement] = STATE(311),
    [sym_send_level_statement] = STATE(311),
    [sym_create_buffer_statement] = STATE(311),
    [sym_create_multi_buffer_statement] = STATE(311),
    [sym_clear_buffer_statement] = STATE(311),
    [sym_wait_statement] = STATE(311),
    [sym_wait_until_statement] = STATE(311),
    [sym_cancel_all_wait_statement] = STATE(311),
    [sym_cancel_all_wait_until_statement] = STATE(311),
    [sym_cancel_wait_statement] = STATE(311),
    [sym_cancel_wait_until_statement] = STATE(311),
    [sym_call_statement] = STATE(311),
    [sym_system_call_statement] = STATE(311),
    [sym_expression] = STATE(1137),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1990),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(55),
    [sym_switch_keyword] = ACTIONS(57),
    [sym_case_keyword] = ACTIONS(59),
    [sym_default_keyword] = ACTIONS(61),
    [sym_while_keyword] = ACTIONS(63),
    [sym_for_keyword] = ACTIONS(65),
    [sym_break_keyword] = ACTIONS(67),
    [sym_continue_keyword] = ACTIONS(69),
    [sym_return_keyword] = ACTIONS(71),
    [sym_select_keyword] = ACTIONS(73),
    [sym_active_keyword] = ACTIONS(75),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(99),
    [sym_send_string_keyword] = ACTIONS(101),
    [sym_send_command_keyword] = ACTIONS(103),
    [sym_clear_buffer_keyword] = ACTIONS(105),
    [sym_create_buffer_keyword] = ACTIONS(107),
    [sym_create_multi_buffer_keyword] = ACTIONS(109),
    [sym_call_keyword] = ACTIONS(111),
    [sym_system_call_keyword] = ACTIONS(113),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(117),
    [sym_cancel_wait_keyword] = ACTIONS(119),
    [sym_wait_until_keyword] = ACTIONS(121),
    [sym_cancel_wait_until_keyword] = ACTIONS(123),
    [sym_cancel_all_wait_keyword] = ACTIONS(125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(127),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(531)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(311),
    [sym_statement] = STATE(340),
    [sym_expression_statement] = STATE(311),
    [sym_if_statement] = STATE(311),
    [sym_switch_statement] = STATE(311),
    [sym_case_statement] = STATE(311),
    [sym_while_statement] = STATE(311),
    [sym_for_statement] = STATE(311),
    [sym_select_statement] = STATE(311),
    [sym_active_statement] = STATE(311),
    [sym_return_statement] = STATE(311),
    [sym_break_statement] = STATE(311),
    [sym_continue_statement] = STATE(311),
    [sym_devchan_operation_statement] = STATE(311),
    [sym_devchan_operation] = STATE(1878),
    [sym_send_string_statement] = STATE(311),
    [sym_send_command_statement] = STATE(311),
    [sym_send_level_statement] = STATE(311),
    [sym_create_buffer_statement] = STATE(311),
    [sym_create_multi_buffer_statement] = STATE(311),
    [sym_clear_buffer_statement] = STATE(311),
    [sym_wait_statement] = STATE(311),
    [sym_wait_until_statement] = STATE(311),
    [sym_cancel_all_wait_statement] = STATE(311),
    [sym_cancel_all_wait_until_statement] = STATE(311),
    [sym_cancel_wait_statement] = STATE(311),
    [sym_cancel_wait_until_statement] = STATE(311),
    [sym_call_statement] = STATE(311),
    [sym_system_call_statement] = STATE(311),
    [sym_expression] = STATE(1137),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1990),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(55),
    [sym_switch_keyword] = ACTIONS(57),
    [sym_case_keyword] = ACTIONS(59),
    [sym_default_keyword] = ACTIONS(61),
    [sym_while_keyword] = ACTIONS(63),
    [sym_for_keyword] = ACTIONS(65),
    [sym_break_keyword] = ACTIONS(67),
    [sym_continue_keyword] = ACTIONS(69),
    [sym_return_keyword] = ACTIONS(71),
    [sym_select_keyword] = ACTIONS(73),
    [sym_active_keyword] = ACTIONS(75),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(99),
    [sym_send_string_keyword] = ACTIONS(101),
    [sym_send_command_keyword] = ACTIONS(103),
    [sym_clear_buffer_keyword] = ACTIONS(105),
    [sym_create_buffer_keyword] = ACTIONS(107),
    [sym_create_multi_buffer_keyword] = ACTIONS(109),
    [sym_call_keyword] = ACTIONS(111),
    [sym_system_call_keyword] = ACTIONS(113),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(117),
    [sym_cancel_wait_keyword] = ACTIONS(119),
    [sym_wait_until_keyword] = ACTIONS(121),
    [sym_cancel_wait_until_keyword] = ACTIONS(123),
    [sym_cancel_all_wait_keyword] = ACTIONS(125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(127),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(532)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(311),
    [sym_statement] = STATE(344),
    [sym_expression_statement] = STATE(311),
    [sym_if_statement] = STATE(311),
    [sym_switch_statement] = STATE(311),
    [sym_case_statement] = STATE(311),
    [sym_while_statement] = STATE(311),
    [sym_for_statement] = STATE(311),
    [sym_select_statement] = STATE(311),
    [sym_active_statement] = STATE(311),
    [sym_return_statement] = STATE(311),
    [sym_break_statement] = STATE(311),
    [sym_continue_statement] = STATE(311),
    [sym_devchan_operation_statement] = STATE(311),
    [sym_devchan_operation] = STATE(1878),
    [sym_send_string_statement] = STATE(311),
    [sym_send_command_statement] = STATE(311),
    [sym_send_level_statement] = STATE(311),
    [sym_create_buffer_statement] = STATE(311),
    [sym_create_multi_buffer_statement] = STATE(311),
    [sym_clear_buffer_statement] = STATE(311),
    [sym_wait_statement] = STATE(311),
    [sym_wait_until_statement] = STATE(311),
    [sym_cancel_all_wait_statement] = STATE(311),
    [sym_cancel_all_wait_until_statement] = STATE(311),
    [sym_cancel_wait_statement] = STATE(311),
    [sym_cancel_wait_until_statement] = STATE(311),
    [sym_call_statement] = STATE(311),
    [sym_system_call_statement] = STATE(311),
    [sym_expression] = STATE(1137),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1990),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(55),
    [sym_switch_keyword] = ACTIONS(57),
    [sym_case_keyword] = ACTIONS(59),
    [sym_default_keyword] = ACTIONS(61),
    [sym_while_keyword] = ACTIONS(63),
    [sym_for_keyword] = ACTIONS(65),
    [sym_break_keyword] = ACTIONS(67),
    [sym_continue_keyword] = ACTIONS(69),
    [sym_return_keyword] = ACTIONS(71),
    [sym_select_keyword] = ACTIONS(73),
    [sym_active_keyword] = ACTIONS(75),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(99),
    [sym_send_string_keyword] = ACTIONS(101),
    [sym_send_command_keyword] = ACTIONS(103),
    [sym_clear_buffer_keyword] = ACTIONS(105),
    [sym_create_buffer_keyword] = ACTIONS(107),
    [sym_create_multi_buffer_keyword] = ACTIONS(109),
    [sym_call_keyword] = ACTIONS(111),
    [sym_system_call_keyword] = ACTIONS(113),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(117),
    [sym_cancel_wait_keyword] = ACTIONS(119),
    [sym_wait_until_keyword] = ACTIONS(121),
    [sym_cancel_wait_until_keyword] = ACTIONS(123),
    [sym_cancel_all_wait_keyword] = ACTIONS(125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(127),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(533)] = {
    [sym_identifier] = ACTIONS(1941),
    [sym_define_call_keyword] = ACTIONS(1941),
    [sym_define_function_keyword] = ACTIONS(1941),
    [sym_define_library_function_keyword] = ACTIONS(1941),
    [sym_define_module_keyword] = ACTIONS(1941),
    [sym_char_keyword] = ACTIONS(1941),
    [sym_widechar_keyword] = ACTIONS(1941),
    [sym_integer_keyword] = ACTIONS(1941),
    [sym_sinteger_keyword] = ACTIONS(1941),
    [sym_long_keyword] = ACTIONS(1941),
    [sym_slong_keyword] = ACTIONS(1941),
    [sym_float_keyword] = ACTIONS(1941),
    [sym_double_keyword] = ACTIONS(1941),
    [sym_variant_keyword] = ACTIONS(1941),
    [sym_variantarray_keyword] = ACTIONS(1941),
    [sym_local_var_keyword] = ACTIONS(1941),
    [sym_stack_var_keyword] = ACTIONS(1941),
    [sym_constant_keyword] = ACTIONS(1941),
    [sym_volatile_keyword] = ACTIONS(1941),
    [sym_non_volatile_keyword] = ACTIONS(1941),
    [sym_persistent_keyword] = ACTIONS(1941),
    [sym_dev_keyword] = ACTIONS(1941),
    [sym_devlev_keyword] = ACTIONS(1941),
    [sym_devchan_keyword] = ACTIONS(1941),
    [sym_if_keyword] = ACTIONS(1941),
    [sym_switch_keyword] = ACTIONS(1941),
    [sym_case_keyword] = ACTIONS(1941),
    [sym_default_keyword] = ACTIONS(1941),
    [sym_while_keyword] = ACTIONS(1941),
    [sym_for_keyword] = ACTIONS(1941),
    [sym_break_keyword] = ACTIONS(1941),
    [sym_continue_keyword] = ACTIONS(1941),
    [sym_return_keyword] = ACTIONS(1941),
    [sym_select_keyword] = ACTIONS(1941),
    [sym_active_keyword] = ACTIONS(1941),
    [sym_struct_keyword] = ACTIONS(1941),
    [sym_structure_keyword] = ACTIONS(1941),
    [aux_sym_on_keyword_token1] = ACTIONS(1941),
    [aux_sym_off_keyword_token1] = ACTIONS(1941),
    [sym_send_level_keyword] = ACTIONS(1941),
    [sym_send_string_keyword] = ACTIONS(1941),
    [sym_send_command_keyword] = ACTIONS(1941),
    [sym_clear_buffer_keyword] = ACTIONS(1941),
    [sym_create_buffer_keyword] = ACTIONS(1941),
    [sym_create_multi_buffer_keyword] = ACTIONS(1941),
    [sym_call_keyword] = ACTIONS(1941),
    [sym_system_call_keyword] = ACTIONS(1941),
    [sym_devchan_to_keyword] = ACTIONS(1941),
    [sym_devchan_min_to_keyword] = ACTIONS(1941),
    [sym_devchan_total_off_keyword] = ACTIONS(1941),
    [sym_devchan_pulse_keyword] = ACTIONS(1941),
    [sym_wait_keyword] = ACTIONS(1941),
    [sym_cancel_wait_keyword] = ACTIONS(1941),
    [sym_wait_until_keyword] = ACTIONS(1941),
    [sym_cancel_wait_until_keyword] = ACTIONS(1941),
    [sym_cancel_all_wait_keyword] = ACTIONS(1941),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1941),
    [sym_not] = ACTIONS(1941),
    [sym_bnot] = ACTIONS(1941),
    [sym___file__] = ACTIONS(1941),
    [sym___date__] = ACTIONS(1941),
    [sym___ldate__] = ACTIONS(1941),
    [sym___time__] = ACTIONS(1941),
    [sym___line__] = ACTIONS(1941),
    [sym___name__] = ACTIONS(1941),
    [sym___netlinx__] = ACTIONS(1941),
    [sym_netlinx_axi_version] = ACTIONS(1941),
    [sym_url_flg_tcp] = ACTIONS(1941),
    [sym_url_flg_accinfopresent] = ACTIONS(1941),
    [sym_url_flg_temp] = ACTIONS(1941),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1941),
    [sym_url_flg_stat_mask] = ACTIONS(1941),
    [sym_url_flg_stat_lookup] = ACTIONS(1941),
    [sym_url_flg_stat_connecting] = ACTIONS(1941),
    [sym_url_flg_stat_waiting] = ACTIONS(1941),
    [sym_url_flg_stat_connected] = ACTIONS(1941),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1941),
    [sym_first_virtual_device] = ACTIONS(1941),
    [sym_first_local_port] = ACTIONS(1941),
    [sym_dynamic_virtual_device] = ACTIONS(1941),
    [sym_timeline_once] = ACTIONS(1941),
    [sym_timeline_repeat] = ACTIONS(1941),
    [sym_timeline_absolute] = ACTIONS(1941),
    [sym_timeline_relative] = ACTIONS(1941),
    [sym_do_push_timed_infinite] = ACTIONS(1941),
    [sym_source_type_no_address] = ACTIONS(1941),
    [sym_source_type_neuron_id] = ACTIONS(1941),
    [sym_source_type_ip_address] = ACTIONS(1941),
    [sym_source_type_axlink] = ACTIONS(1941),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1941),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1941),
    [sym_source_type_ip_socket_address] = ACTIONS(1941),
    [sym_source_type_rs232] = ACTIONS(1941),
    [sym_source_type_internal] = ACTIONS(1941),
    [sym_source_type_mac_address] = ACTIONS(1941),
    [sym_source_type_ipv4_port] = ACTIONS(1941),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1941),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1941),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1941),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1941),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1941),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1941),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1941),
    [sym_true] = ACTIONS(1941),
    [sym_false] = ACTIONS(1941),
    [sym_file_read_only] = ACTIONS(1941),
    [sym_file_rw_new] = ACTIONS(1941),
    [sym_file_rw_append] = ACTIONS(1941),
    [sym_ip_tcp] = ACTIONS(1941),
    [sym_ip_udp] = ACTIONS(1941),
    [sym_ip_udp_2way] = ACTIONS(1941),
    [sym_xml_encode_types] = ACTIONS(1941),
    [sym_xml_encode_char_as_list] = ACTIONS(1941),
    [sym_xml_encode_le] = ACTIONS(1941),
    [sym_xml_decode_types] = ACTIONS(1941),
    [sym_xml_decode_no_preserve] = ACTIONS(1941),
    [sym_duet_dev_type_display_device] = ACTIONS(1941),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1941),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1941),
    [sym_duet_dev_type_io_device] = ACTIONS(1941),
    [sym_duet_dev_type_relay_device] = ACTIONS(1941),
    [sym_duet_dev_type_ups] = ACTIONS(1941),
    [sym_duet_dev_type_amplifier] = ACTIONS(1941),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1941),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1941),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1941),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1941),
    [sym_duet_dev_type_camera] = ACTIONS(1941),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1941),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1941),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1941),
    [sym_duet_dev_type_dss] = ACTIONS(1941),
    [sym_duet_dev_type_dvr] = ACTIONS(1941),
    [sym_duet_dev_type_disc_device] = ACTIONS(1941),
    [sym_duet_dev_type_document_camera] = ACTIONS(1941),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1941),
    [sym_duet_dev_type_hvac] = ACTIONS(1941),
    [sym_duet_dev_type_keypad] = ACTIONS(1941),
    [sym_duet_dev_type_light] = ACTIONS(1941),
    [sym_duet_dev_type_monitor] = ACTIONS(1941),
    [sym_duet_dev_type_motor] = ACTIONS(1941),
    [sym_duet_dev_type_multi_window] = ACTIONS(1941),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1941),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1941),
    [sym_duet_dev_type_receiver] = ACTIONS(1941),
    [sym_duet_dev_type_security_system] = ACTIONS(1941),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1941),
    [sym_duet_dev_type_settop_box] = ACTIONS(1941),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1941),
    [sym_duet_dev_type_switcher] = ACTIONS(1941),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1941),
    [sym_duet_dev_type_tv] = ACTIONS(1941),
    [sym_duet_dev_type_utility] = ACTIONS(1941),
    [sym_duet_dev_type_vcr] = ACTIONS(1941),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1941),
    [sym_duet_dev_type_video_processor] = ACTIONS(1941),
    [sym_duet_dev_type_video_projector] = ACTIONS(1941),
    [sym_duet_dev_type_video_wall] = ACTIONS(1941),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1941),
    [sym_duet_dev_type_weather] = ACTIONS(1941),
    [sym_duet_dev_not_polled] = ACTIONS(1941),
    [sym_duet_dev_polled] = ACTIONS(1941),
    [sym_clkmgr_mode_network] = ACTIONS(1941),
    [sym_clkmgr_mode_standalone] = ACTIONS(1941),
    [sym_normal_standby] = ACTIONS(1941),
    [sym_normal_wake] = ACTIONS(1941),
    [sym_ip_multicast_ttl_option] = ACTIONS(1941),
    [sym_ip_ttl_subnet] = ACTIONS(1941),
    [sym_ip_ttl_site] = ACTIONS(1941),
    [sym_ip_ttl_region] = ACTIONS(1941),
    [sym_ip_ttl_continent] = ACTIONS(1941),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1941),
    [sym_ip_nodelay_on] = ACTIONS(1941),
    [sym_ip_nodelay_off] = ACTIONS(1941),
    [sym_amx_error] = ACTIONS(1941),
    [sym_amx_warning] = ACTIONS(1941),
    [sym_amx_info] = ACTIONS(1941),
    [sym_amx_debug] = ACTIONS(1941),
    [sym_netlinx_login_success] = ACTIONS(1941),
    [sym_netlinx_login_fail] = ACTIONS(1941),
    [sym_netlinx_logout] = ACTIONS(1941),
    [sym_null_str] = ACTIONS(1941),
    [sym_smtp_address] = ACTIONS(1941),
    [sym_smtp_port_number] = ACTIONS(1941),
    [sym_smtp_username] = ACTIONS(1941),
    [sym_smtp_password] = ACTIONS(1941),
    [sym_smtp_require_tls] = ACTIONS(1941),
    [sym_smtp_from] = ACTIONS(1941),
    [sym_smtp_tls_true] = ACTIONS(1941),
    [sym_smtp_tls_false] = ACTIONS(1941),
    [sym_listview_on_row_select_event] = ACTIONS(1941),
    [sym_data_structure_unknown] = ACTIONS(1941),
    [sym_data_structure_datafeed] = ACTIONS(1941),
    [sym_data_structure_datarecordset] = ACTIONS(1941),
    [sym_data_structure_datarecord] = ACTIONS(1941),
    [sym_data_structure_datafield] = ACTIONS(1941),
    [sym_data_max_content_field_count] = ACTIONS(1941),
    [sym_data_max_metadata_field_count] = ACTIONS(1941),
    [sym_data_max_id_length] = ACTIONS(1941),
    [sym_data_max_name_length] = ACTIONS(1941),
    [sym_data_max_description_length] = ACTIONS(1941),
    [sym_data_max_type_length] = ACTIONS(1941),
    [sym_data_max_format_length] = ACTIONS(1941),
    [sym_data_max_source_length] = ACTIONS(1941),
    [sym_data_max_value_length] = ACTIONS(1941),
    [sym_data_max_label_length] = ACTIONS(1941),
    [sym_data_max_url_length] = ACTIONS(1941),
    [sym_max_last_login_info_length] = ACTIONS(1941),
    [sym_data_type_unknown] = ACTIONS(1941),
    [sym_data_type_string] = ACTIONS(1941),
    [sym_data_type_datetime] = ACTIONS(1941),
    [sym_data_type_date] = ACTIONS(1941),
    [sym_data_type_time] = ACTIONS(1941),
    [sym_data_type_image] = ACTIONS(1941),
    [sym_data_format_url] = ACTIONS(1941),
    [sym_data_format_phone] = ACTIONS(1941),
    [sym_data_format_email] = ACTIONS(1941),
    [sym_data_format_iso8601] = ACTIONS(1941),
    [sym_auditlog_priv] = ACTIONS(1941),
    [sym_configuration_priv] = ACTIONS(1941),
    [sym_deviceconfig_priv] = ACTIONS(1941),
    [sym_ftp_priv] = ACTIONS(1941),
    [sym_http_priv] = ACTIONS(1941),
    [sym_networkconfig_priv] = ACTIONS(1941),
    [sym_programport_priv] = ACTIONS(1941),
    [sym_remoteui_priv] = ACTIONS(1941),
    [sym_securitycontrol_priv] = ACTIONS(1941),
    [sym_softwaremanagement_priv] = ACTIONS(1941),
    [sym_terminal_priv] = ACTIONS(1941),
    [sym_tpadmin_priv] = ACTIONS(1941),
    [sym_user1_priv] = ACTIONS(1941),
    [sym_user2_priv] = ACTIONS(1941),
    [sym_user3_priv] = ACTIONS(1941),
    [sym_user4_priv] = ACTIONS(1941),
    [sym_usermanagement_priv] = ACTIONS(1941),
    [sym_valid_account] = ACTIONS(1941),
    [sym_err_name_invalid] = ACTIONS(1941),
    [sym_err_password_invalid] = ACTIONS(1941),
    [sym_err_invalid_account] = ACTIONS(1941),
    [sym_err_no_authorization] = ACTIONS(1941),
    [sym_err_invalid_parameter] = ACTIONS(1941),
    [sym_err_user_locked_out] = ACTIONS(1941),
    [sym_err_user_account_expired] = ACTIONS(1941),
    [sym_err_general_error] = ACTIONS(1941),
    [sym_err_authentication_error] = ACTIONS(1941),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1941),
    [sym_tls_validate_certificate] = ACTIONS(1941),
    [sym_tdata] = ACTIONS(1941),
    [sym_tchannel] = ACTIONS(1941),
    [sym_tlevel] = ACTIONS(1941),
    [sym_tbutton] = ACTIONS(1941),
    [sym_ttimeline] = ACTIONS(1941),
    [sym_tcustom] = ACTIONS(1941),
    [sym_url_struct] = ACTIONS(1941),
    [sym_dns_struct] = ACTIONS(1941),
    [sym_ip_address_struct] = ACTIONS(1941),
    [sym_dev_info_struct] = ACTIONS(1941),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1941),
    [sym_data_feed] = ACTIONS(1941),
    [sym_data_field] = ACTIONS(1941),
    [sym_data_record] = ACTIONS(1941),
    [sym_wc_data_feed] = ACTIONS(1941),
    [sym_wc_data_field] = ACTIONS(1941),
    [sym_wc_data_record] = ACTIONS(1941),
    [sym_last_login_info] = ACTIONS(1941),
    [sym____reserved___] = ACTIONS(1941),
    [sym_date] = ACTIONS(1941),
    [sym_day] = ACTIONS(1941),
    [sym_time] = ACTIONS(1941),
    [sym_ldate] = ACTIONS(1941),
    [sym_push_device] = ACTIONS(1941),
    [sym_push_channel] = ACTIONS(1941),
    [sym_push_devchan] = ACTIONS(1941),
    [sym_release_device] = ACTIONS(1941),
    [sym_release_channel] = ACTIONS(1941),
    [sym_release_devchan] = ACTIONS(1941),
    [sym_master_slot] = ACTIONS(1941),
    [sym_get_pulse_time] = ACTIONS(1941),
    [sym_get_timer] = ACTIONS(1941),
    [sym_master_sn] = ACTIONS(1941),
    [sym_system_number] = ACTIONS(1941),
    [sym_button] = ACTIONS(1941),
    [sym_dv_channel] = ACTIONS(1941),
    [sym_data] = ACTIONS(1941),
    [sym_level] = ACTIONS(1941),
    [sym_timeline] = ACTIONS(1941),
    [sym_channel] = ACTIONS(1941),
    [sym_custom] = ACTIONS(1941),
    [sym_atoi] = ACTIONS(1941),
    [sym_atol] = ACTIONS(1941),
    [sym_atof] = ACTIONS(1941),
    [sym_itoa] = ACTIONS(1941),
    [sym_itohex] = ACTIONS(1941),
    [sym_hextoi] = ACTIONS(1941),
    [sym_ftoa] = ACTIONS(1941),
    [sym_find_string] = ACTIONS(1941),
    [sym_left_string] = ACTIONS(1941),
    [sym_right_string] = ACTIONS(1941),
    [sym_mid_string] = ACTIONS(1941),
    [sym_remove_string] = ACTIONS(1941),
    [sym_get_buffer_string] = ACTIONS(1941),
    [sym_get_buffer_char] = ACTIONS(1941),
    [sym_get_multi_buffer_string] = ACTIONS(1941),
    [sym_length_string] = ACTIONS(1941),
    [sym_max_length_string] = ACTIONS(1941),
    [sym_set_length_string] = ACTIONS(1941),
    [sym_compare_string] = ACTIONS(1941),
    [sym_upper_string] = ACTIONS(1941),
    [sym_lower_string] = ACTIONS(1941),
    [sym_get_last] = ACTIONS(1941),
    [sym_date_to_day] = ACTIONS(1941),
    [sym_date_to_month] = ACTIONS(1941),
    [sym_date_to_year] = ACTIONS(1941),
    [sym_time_to_hour] = ACTIONS(1941),
    [sym_time_to_minute] = ACTIONS(1941),
    [sym_time_to_second] = ACTIONS(1941),
    [sym_day_of_week] = ACTIONS(1941),
    [sym_set_pulse_time] = ACTIONS(1941),
    [sym_set_timer] = ACTIONS(1941),
    [sym_file_open] = ACTIONS(1941),
    [sym_file_close] = ACTIONS(1941),
    [sym_file_read] = ACTIONS(1941),
    [sym_file_read_line] = ACTIONS(1941),
    [sym_file_write] = ACTIONS(1941),
    [sym_file_write_line] = ACTIONS(1941),
    [sym_file_copy] = ACTIONS(1941),
    [sym_file_delete] = ACTIONS(1941),
    [sym_file_rename] = ACTIONS(1941),
    [sym_file_dir] = ACTIONS(1941),
    [sym_file_getdir] = ACTIONS(1941),
    [sym_file_setdir] = ACTIONS(1941),
    [sym_file_create_dir] = ACTIONS(1941),
    [sym_file_remove_dir] = ACTIONS(1941),
    [sym_file_seek] = ACTIONS(1941),
    [sym_length_array] = ACTIONS(1941),
    [sym_max_length_array] = ACTIONS(1941),
    [sym_set_length_array] = ACTIONS(1941),
    [sym_ip_client_open] = ACTIONS(1941),
    [sym_ip_client_close] = ACTIONS(1941),
    [sym_ip_server_open] = ACTIONS(1941),
    [sym_ip_server_close] = ACTIONS(1941),
    [sym_ip_bound_client_open] = ACTIONS(1941),
    [sym_ip_mc_server_open] = ACTIONS(1941),
    [sym_ip_set_option] = ACTIONS(1941),
    [sym_ssh_client_open] = ACTIONS(1941),
    [sym_ssh_client_close] = ACTIONS(1941),
    [sym_tls_client_open] = ACTIONS(1941),
    [sym_tls_client_close] = ACTIONS(1941),
    [sym_do_push] = ACTIONS(1941),
    [sym_do_push_timed] = ACTIONS(1941),
    [sym_do_release] = ACTIONS(1941),
    [sym_device_id] = ACTIONS(1941),
    [sym_device_info] = ACTIONS(1941),
    [sym_device_id_string] = ACTIONS(1941),
    [sym_device_standby] = ACTIONS(1941),
    [sym_device_wake] = ACTIONS(1941),
    [sym_timeline_create] = ACTIONS(1941),
    [sym_timeline_kill] = ACTIONS(1941),
    [sym_timeline_reload] = ACTIONS(1941),
    [sym_timeline_pause] = ACTIONS(1941),
    [sym_timeline_restart] = ACTIONS(1941),
    [sym_timeline_set] = ACTIONS(1941),
    [sym_timeline_get] = ACTIONS(1941),
    [sym_timeline_active] = ACTIONS(1941),
    [sym_timeline_dynamic_id] = ACTIONS(1941),
    [sym_variable_to_string] = ACTIONS(1941),
    [sym_string_to_variable] = ACTIONS(1941),
    [sym_length_variable_to_string] = ACTIONS(1941),
    [sym_type_cast] = ACTIONS(1941),
    [sym_variable_to_xml] = ACTIONS(1941),
    [sym_xml_to_variable] = ACTIONS(1941),
    [sym_length_variable_to_xml] = ACTIONS(1941),
    [sym_redirect_string] = ACTIONS(1941),
    [sym_random_number] = ACTIONS(1941),
    [sym_get_unique_id] = ACTIONS(1941),
    [sym_get_system_number] = ACTIONS(1941),
    [sym_set_system_number] = ACTIONS(1941),
    [sym_get_serial_number] = ACTIONS(1941),
    [sym_get_master_build] = ACTIONS(1941),
    [sym_set_validation_code] = ACTIONS(1941),
    [sym_reboot] = ACTIONS(1941),
    [sym_load_duet_module] = ACTIONS(1941),
    [sym_unload_duet_module] = ACTIONS(1941),
    [sym_duet_mem_size_set] = ACTIONS(1941),
    [sym_duet_mem_size_get] = ACTIONS(1941),
    [sym_dynamic_application_device] = ACTIONS(1941),
    [sym_dynamic_polled_port] = ACTIONS(1941),
    [sym_static_port_binding] = ACTIONS(1941),
    [sym_static_ip_binding] = ACTIONS(1941),
    [sym_get_dns_list] = ACTIONS(1941),
    [sym_set_dns_list] = ACTIONS(1941),
    [sym_get_ip_address] = ACTIONS(1941),
    [sym_set_ip_address] = ACTIONS(1941),
    [sym_get_url_list] = ACTIONS(1941),
    [sym_add_url_entry] = ACTIONS(1941),
    [sym_delete_url_entry] = ACTIONS(1941),
    [sym_abs_value] = ACTIONS(1941),
    [sym_max_value] = ACTIONS(1941),
    [sym_min_value] = ACTIONS(1941),
    [sym_exp_value] = ACTIONS(1941),
    [sym_log_value] = ACTIONS(1941),
    [sym_log10_value] = ACTIONS(1941),
    [sym_power_value] = ACTIONS(1941),
    [sym_sqrt_value] = ACTIONS(1941),
    [sym_raw_be] = ACTIONS(1941),
    [sym_raw_le] = ACTIONS(1941),
    [sym_internal_queue_size_set] = ACTIONS(1941),
    [sym_internal_queue_size_get] = ACTIONS(1941),
    [sym_internal_threshold_set] = ACTIONS(1941),
    [sym_internal_threshold_get] = ACTIONS(1941),
    [sym_rebuilt_event] = ACTIONS(1941),
    [sym_do_custom_event] = ACTIONS(1941),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1941),
    [sym_clkmgr_set_clk_source] = ACTIONS(1941),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1941),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1941),
    [sym_clkmgr_get_timezone] = ACTIONS(1941),
    [sym_clkmgr_set_timezone] = ACTIONS(1941),
    [sym_clkmgr_get_resync_period] = ACTIONS(1941),
    [sym_clkmgr_set_resync_period] = ACTIONS(1941),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1941),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1941),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1941),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1941),
    [sym_clkmgr_get_timeservers] = ACTIONS(1941),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1941),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1941),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1941),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1941),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1941),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1941),
    [sym_set_log_level] = ACTIONS(1941),
    [sym_get_log_level] = ACTIONS(1941),
    [sym_amx_log] = ACTIONS(1941),
    [sym_smtp_server_config_set] = ACTIONS(1941),
    [sym_smtp_server_config_get] = ACTIONS(1941),
    [sym_smtp_send] = ACTIONS(1941),
    [sym_validate_netlinx_account] = ACTIONS(1941),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1941),
    [sym_authenticate_certificate] = ACTIONS(1941),
    [sym_audit_netlinx_session_event] = ACTIONS(1941),
    [sym_audit_netlinx_generic_event] = ACTIONS(1941),
    [sym_get_available_flash_disk_space] = ACTIONS(1941),
    [sym_get_max_flash_disk_space] = ACTIONS(1941),
    [sym_data_create_feed] = ACTIONS(1941),
    [sym_data_delete_feed] = ACTIONS(1941),
    [sym_data_publish_feed] = ACTIONS(1941),
    [sym_data_get_published_feed] = ACTIONS(1941),
    [sym_data_add_record] = ACTIONS(1941),
    [sym_data_get_event_record] = ACTIONS(1941),
    [sym__wc_data_create_feed] = ACTIONS(1941),
    [sym__wc_data_add_record] = ACTIONS(1941),
    [sym__wc_data_get_event_record] = ACTIONS(1941),
    [sym_format] = ACTIONS(1941),
    [sym_astro_clock] = ACTIONS(1941),
    [sym_set_outdoor_temperature] = ACTIONS(1941),
    [sym_set_virtual_level_count] = ACTIONS(1941),
    [sym_set_virtual_channel_count] = ACTIONS(1941),
    [sym_set_virtual_port_count] = ACTIONS(1941),
    [sym__wc_encode_int] = ACTIONS(1941),
    [sym__wc_decode_int] = ACTIONS(1941),
    [sym__wc_int] = ACTIONS(1941),
    [sym__wc_to_ch_int] = ACTIONS(1941),
    [sym__ch_to_wc_int] = ACTIONS(1941),
    [sym__wc_find_string_int] = ACTIONS(1941),
    [sym__wc_left_string_int] = ACTIONS(1941),
    [sym__wc_lower_string_int] = ACTIONS(1941),
    [sym__wc_mid_string_int] = ACTIONS(1941),
    [sym__wc_remove_string_int] = ACTIONS(1941),
    [sym__wc_right_string_int] = ACTIONS(1941),
    [sym__wc_upper_string_int] = ACTIONS(1941),
    [sym__wc_compare_string_int] = ACTIONS(1941),
    [sym__wc_get_buffer_char_int] = ACTIONS(1941),
    [sym__wc_get_buffer_string_int] = ACTIONS(1941),
    [sym__wc_concat_string_int] = ACTIONS(1941),
    [sym__wc_file_open_int] = ACTIONS(1941),
    [sym__wc_file_close_int] = ACTIONS(1941),
    [sym__wc_file_read_int] = ACTIONS(1941),
    [sym__wc_file_read_line_int] = ACTIONS(1941),
    [sym__wc_file_write_int] = ACTIONS(1941),
    [sym__wc_file_write_line_int] = ACTIONS(1941),
    [aux_sym_preproc_include_token1] = ACTIONS(1941),
    [aux_sym_preproc_define_token1] = ACTIONS(1943),
    [aux_sym_preproc_warn_token1] = ACTIONS(1943),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1943),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1943),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1943),
    [anon_sym_LBRACK] = ACTIONS(1943),
    [anon_sym_LBRACE] = ACTIONS(1943),
    [anon_sym_RBRACE] = ACTIONS(1943),
    [anon_sym_LPAREN] = ACTIONS(1941),
    [anon_sym_BANG] = ACTIONS(1943),
    [anon_sym_TILDE] = ACTIONS(1943),
    [anon_sym_DASH] = ACTIONS(1941),
    [anon_sym_PLUS] = ACTIONS(1941),
    [anon_sym_DASH_DASH] = ACTIONS(1943),
    [anon_sym_PLUS_PLUS] = ACTIONS(1943),
    [anon_sym_DQUOTE] = ACTIONS(1943),
    [anon_sym_SQUOTE] = ACTIONS(1943),
    [sym_number_literal] = ACTIONS(1943),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(534)] = {
    [sym_identifier] = ACTIONS(2073),
    [sym_define_call_keyword] = ACTIONS(2073),
    [sym_define_function_keyword] = ACTIONS(2073),
    [sym_define_library_function_keyword] = ACTIONS(2073),
    [sym_define_module_keyword] = ACTIONS(2073),
    [sym_char_keyword] = ACTIONS(2073),
    [sym_widechar_keyword] = ACTIONS(2073),
    [sym_integer_keyword] = ACTIONS(2073),
    [sym_sinteger_keyword] = ACTIONS(2073),
    [sym_long_keyword] = ACTIONS(2073),
    [sym_slong_keyword] = ACTIONS(2073),
    [sym_float_keyword] = ACTIONS(2073),
    [sym_double_keyword] = ACTIONS(2073),
    [sym_variant_keyword] = ACTIONS(2073),
    [sym_variantarray_keyword] = ACTIONS(2073),
    [sym_local_var_keyword] = ACTIONS(2073),
    [sym_stack_var_keyword] = ACTIONS(2073),
    [sym_constant_keyword] = ACTIONS(2073),
    [sym_volatile_keyword] = ACTIONS(2073),
    [sym_non_volatile_keyword] = ACTIONS(2073),
    [sym_persistent_keyword] = ACTIONS(2073),
    [sym_dev_keyword] = ACTIONS(2073),
    [sym_devlev_keyword] = ACTIONS(2073),
    [sym_devchan_keyword] = ACTIONS(2073),
    [sym_if_keyword] = ACTIONS(2073),
    [sym_switch_keyword] = ACTIONS(2073),
    [sym_case_keyword] = ACTIONS(2073),
    [sym_default_keyword] = ACTIONS(2073),
    [sym_while_keyword] = ACTIONS(2073),
    [sym_for_keyword] = ACTIONS(2073),
    [sym_break_keyword] = ACTIONS(2073),
    [sym_continue_keyword] = ACTIONS(2073),
    [sym_return_keyword] = ACTIONS(2073),
    [sym_select_keyword] = ACTIONS(2073),
    [sym_active_keyword] = ACTIONS(2073),
    [sym_struct_keyword] = ACTIONS(2073),
    [sym_structure_keyword] = ACTIONS(2073),
    [aux_sym_on_keyword_token1] = ACTIONS(2073),
    [aux_sym_off_keyword_token1] = ACTIONS(2073),
    [sym_send_level_keyword] = ACTIONS(2073),
    [sym_send_string_keyword] = ACTIONS(2073),
    [sym_send_command_keyword] = ACTIONS(2073),
    [sym_clear_buffer_keyword] = ACTIONS(2073),
    [sym_create_buffer_keyword] = ACTIONS(2073),
    [sym_create_multi_buffer_keyword] = ACTIONS(2073),
    [sym_call_keyword] = ACTIONS(2073),
    [sym_system_call_keyword] = ACTIONS(2073),
    [sym_devchan_to_keyword] = ACTIONS(2073),
    [sym_devchan_min_to_keyword] = ACTIONS(2073),
    [sym_devchan_total_off_keyword] = ACTIONS(2073),
    [sym_devchan_pulse_keyword] = ACTIONS(2073),
    [sym_wait_keyword] = ACTIONS(2073),
    [sym_cancel_wait_keyword] = ACTIONS(2073),
    [sym_wait_until_keyword] = ACTIONS(2073),
    [sym_cancel_wait_until_keyword] = ACTIONS(2073),
    [sym_cancel_all_wait_keyword] = ACTIONS(2073),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2073),
    [sym_not] = ACTIONS(2073),
    [sym_bnot] = ACTIONS(2073),
    [sym___file__] = ACTIONS(2073),
    [sym___date__] = ACTIONS(2073),
    [sym___ldate__] = ACTIONS(2073),
    [sym___time__] = ACTIONS(2073),
    [sym___line__] = ACTIONS(2073),
    [sym___name__] = ACTIONS(2073),
    [sym___netlinx__] = ACTIONS(2073),
    [sym_netlinx_axi_version] = ACTIONS(2073),
    [sym_url_flg_tcp] = ACTIONS(2073),
    [sym_url_flg_accinfopresent] = ACTIONS(2073),
    [sym_url_flg_temp] = ACTIONS(2073),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2073),
    [sym_url_flg_stat_mask] = ACTIONS(2073),
    [sym_url_flg_stat_lookup] = ACTIONS(2073),
    [sym_url_flg_stat_connecting] = ACTIONS(2073),
    [sym_url_flg_stat_waiting] = ACTIONS(2073),
    [sym_url_flg_stat_connected] = ACTIONS(2073),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2073),
    [sym_first_virtual_device] = ACTIONS(2073),
    [sym_first_local_port] = ACTIONS(2073),
    [sym_dynamic_virtual_device] = ACTIONS(2073),
    [sym_timeline_once] = ACTIONS(2073),
    [sym_timeline_repeat] = ACTIONS(2073),
    [sym_timeline_absolute] = ACTIONS(2073),
    [sym_timeline_relative] = ACTIONS(2073),
    [sym_do_push_timed_infinite] = ACTIONS(2073),
    [sym_source_type_no_address] = ACTIONS(2073),
    [sym_source_type_neuron_id] = ACTIONS(2073),
    [sym_source_type_ip_address] = ACTIONS(2073),
    [sym_source_type_axlink] = ACTIONS(2073),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2073),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2073),
    [sym_source_type_ip_socket_address] = ACTIONS(2073),
    [sym_source_type_rs232] = ACTIONS(2073),
    [sym_source_type_internal] = ACTIONS(2073),
    [sym_source_type_mac_address] = ACTIONS(2073),
    [sym_source_type_ipv4_port] = ACTIONS(2073),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2073),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2073),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2073),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2073),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2073),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2073),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2073),
    [sym_true] = ACTIONS(2073),
    [sym_false] = ACTIONS(2073),
    [sym_file_read_only] = ACTIONS(2073),
    [sym_file_rw_new] = ACTIONS(2073),
    [sym_file_rw_append] = ACTIONS(2073),
    [sym_ip_tcp] = ACTIONS(2073),
    [sym_ip_udp] = ACTIONS(2073),
    [sym_ip_udp_2way] = ACTIONS(2073),
    [sym_xml_encode_types] = ACTIONS(2073),
    [sym_xml_encode_char_as_list] = ACTIONS(2073),
    [sym_xml_encode_le] = ACTIONS(2073),
    [sym_xml_decode_types] = ACTIONS(2073),
    [sym_xml_decode_no_preserve] = ACTIONS(2073),
    [sym_duet_dev_type_display_device] = ACTIONS(2073),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2073),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2073),
    [sym_duet_dev_type_io_device] = ACTIONS(2073),
    [sym_duet_dev_type_relay_device] = ACTIONS(2073),
    [sym_duet_dev_type_ups] = ACTIONS(2073),
    [sym_duet_dev_type_amplifier] = ACTIONS(2073),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2073),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2073),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2073),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2073),
    [sym_duet_dev_type_camera] = ACTIONS(2073),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2073),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2073),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2073),
    [sym_duet_dev_type_dss] = ACTIONS(2073),
    [sym_duet_dev_type_dvr] = ACTIONS(2073),
    [sym_duet_dev_type_disc_device] = ACTIONS(2073),
    [sym_duet_dev_type_document_camera] = ACTIONS(2073),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2073),
    [sym_duet_dev_type_hvac] = ACTIONS(2073),
    [sym_duet_dev_type_keypad] = ACTIONS(2073),
    [sym_duet_dev_type_light] = ACTIONS(2073),
    [sym_duet_dev_type_monitor] = ACTIONS(2073),
    [sym_duet_dev_type_motor] = ACTIONS(2073),
    [sym_duet_dev_type_multi_window] = ACTIONS(2073),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2073),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2073),
    [sym_duet_dev_type_receiver] = ACTIONS(2073),
    [sym_duet_dev_type_security_system] = ACTIONS(2073),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2073),
    [sym_duet_dev_type_settop_box] = ACTIONS(2073),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2073),
    [sym_duet_dev_type_switcher] = ACTIONS(2073),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2073),
    [sym_duet_dev_type_tv] = ACTIONS(2073),
    [sym_duet_dev_type_utility] = ACTIONS(2073),
    [sym_duet_dev_type_vcr] = ACTIONS(2073),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2073),
    [sym_duet_dev_type_video_processor] = ACTIONS(2073),
    [sym_duet_dev_type_video_projector] = ACTIONS(2073),
    [sym_duet_dev_type_video_wall] = ACTIONS(2073),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2073),
    [sym_duet_dev_type_weather] = ACTIONS(2073),
    [sym_duet_dev_not_polled] = ACTIONS(2073),
    [sym_duet_dev_polled] = ACTIONS(2073),
    [sym_clkmgr_mode_network] = ACTIONS(2073),
    [sym_clkmgr_mode_standalone] = ACTIONS(2073),
    [sym_normal_standby] = ACTIONS(2073),
    [sym_normal_wake] = ACTIONS(2073),
    [sym_ip_multicast_ttl_option] = ACTIONS(2073),
    [sym_ip_ttl_subnet] = ACTIONS(2073),
    [sym_ip_ttl_site] = ACTIONS(2073),
    [sym_ip_ttl_region] = ACTIONS(2073),
    [sym_ip_ttl_continent] = ACTIONS(2073),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2073),
    [sym_ip_nodelay_on] = ACTIONS(2073),
    [sym_ip_nodelay_off] = ACTIONS(2073),
    [sym_amx_error] = ACTIONS(2073),
    [sym_amx_warning] = ACTIONS(2073),
    [sym_amx_info] = ACTIONS(2073),
    [sym_amx_debug] = ACTIONS(2073),
    [sym_netlinx_login_success] = ACTIONS(2073),
    [sym_netlinx_login_fail] = ACTIONS(2073),
    [sym_netlinx_logout] = ACTIONS(2073),
    [sym_null_str] = ACTIONS(2073),
    [sym_smtp_address] = ACTIONS(2073),
    [sym_smtp_port_number] = ACTIONS(2073),
    [sym_smtp_username] = ACTIONS(2073),
    [sym_smtp_password] = ACTIONS(2073),
    [sym_smtp_require_tls] = ACTIONS(2073),
    [sym_smtp_from] = ACTIONS(2073),
    [sym_smtp_tls_true] = ACTIONS(2073),
    [sym_smtp_tls_false] = ACTIONS(2073),
    [sym_listview_on_row_select_event] = ACTIONS(2073),
    [sym_data_structure_unknown] = ACTIONS(2073),
    [sym_data_structure_datafeed] = ACTIONS(2073),
    [sym_data_structure_datarecordset] = ACTIONS(2073),
    [sym_data_structure_datarecord] = ACTIONS(2073),
    [sym_data_structure_datafield] = ACTIONS(2073),
    [sym_data_max_content_field_count] = ACTIONS(2073),
    [sym_data_max_metadata_field_count] = ACTIONS(2073),
    [sym_data_max_id_length] = ACTIONS(2073),
    [sym_data_max_name_length] = ACTIONS(2073),
    [sym_data_max_description_length] = ACTIONS(2073),
    [sym_data_max_type_length] = ACTIONS(2073),
    [sym_data_max_format_length] = ACTIONS(2073),
    [sym_data_max_source_length] = ACTIONS(2073),
    [sym_data_max_value_length] = ACTIONS(2073),
    [sym_data_max_label_length] = ACTIONS(2073),
    [sym_data_max_url_length] = ACTIONS(2073),
    [sym_max_last_login_info_length] = ACTIONS(2073),
    [sym_data_type_unknown] = ACTIONS(2073),
    [sym_data_type_string] = ACTIONS(2073),
    [sym_data_type_datetime] = ACTIONS(2073),
    [sym_data_type_date] = ACTIONS(2073),
    [sym_data_type_time] = ACTIONS(2073),
    [sym_data_type_image] = ACTIONS(2073),
    [sym_data_format_url] = ACTIONS(2073),
    [sym_data_format_phone] = ACTIONS(2073),
    [sym_data_format_email] = ACTIONS(2073),
    [sym_data_format_iso8601] = ACTIONS(2073),
    [sym_auditlog_priv] = ACTIONS(2073),
    [sym_configuration_priv] = ACTIONS(2073),
    [sym_deviceconfig_priv] = ACTIONS(2073),
    [sym_ftp_priv] = ACTIONS(2073),
    [sym_http_priv] = ACTIONS(2073),
    [sym_networkconfig_priv] = ACTIONS(2073),
    [sym_programport_priv] = ACTIONS(2073),
    [sym_remoteui_priv] = ACTIONS(2073),
    [sym_securitycontrol_priv] = ACTIONS(2073),
    [sym_softwaremanagement_priv] = ACTIONS(2073),
    [sym_terminal_priv] = ACTIONS(2073),
    [sym_tpadmin_priv] = ACTIONS(2073),
    [sym_user1_priv] = ACTIONS(2073),
    [sym_user2_priv] = ACTIONS(2073),
    [sym_user3_priv] = ACTIONS(2073),
    [sym_user4_priv] = ACTIONS(2073),
    [sym_usermanagement_priv] = ACTIONS(2073),
    [sym_valid_account] = ACTIONS(2073),
    [sym_err_name_invalid] = ACTIONS(2073),
    [sym_err_password_invalid] = ACTIONS(2073),
    [sym_err_invalid_account] = ACTIONS(2073),
    [sym_err_no_authorization] = ACTIONS(2073),
    [sym_err_invalid_parameter] = ACTIONS(2073),
    [sym_err_user_locked_out] = ACTIONS(2073),
    [sym_err_user_account_expired] = ACTIONS(2073),
    [sym_err_general_error] = ACTIONS(2073),
    [sym_err_authentication_error] = ACTIONS(2073),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2073),
    [sym_tls_validate_certificate] = ACTIONS(2073),
    [sym_tdata] = ACTIONS(2073),
    [sym_tchannel] = ACTIONS(2073),
    [sym_tlevel] = ACTIONS(2073),
    [sym_tbutton] = ACTIONS(2073),
    [sym_ttimeline] = ACTIONS(2073),
    [sym_tcustom] = ACTIONS(2073),
    [sym_url_struct] = ACTIONS(2073),
    [sym_dns_struct] = ACTIONS(2073),
    [sym_ip_address_struct] = ACTIONS(2073),
    [sym_dev_info_struct] = ACTIONS(2073),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2073),
    [sym_data_feed] = ACTIONS(2073),
    [sym_data_field] = ACTIONS(2073),
    [sym_data_record] = ACTIONS(2073),
    [sym_wc_data_feed] = ACTIONS(2073),
    [sym_wc_data_field] = ACTIONS(2073),
    [sym_wc_data_record] = ACTIONS(2073),
    [sym_last_login_info] = ACTIONS(2073),
    [sym____reserved___] = ACTIONS(2073),
    [sym_date] = ACTIONS(2073),
    [sym_day] = ACTIONS(2073),
    [sym_time] = ACTIONS(2073),
    [sym_ldate] = ACTIONS(2073),
    [sym_push_device] = ACTIONS(2073),
    [sym_push_channel] = ACTIONS(2073),
    [sym_push_devchan] = ACTIONS(2073),
    [sym_release_device] = ACTIONS(2073),
    [sym_release_channel] = ACTIONS(2073),
    [sym_release_devchan] = ACTIONS(2073),
    [sym_master_slot] = ACTIONS(2073),
    [sym_get_pulse_time] = ACTIONS(2073),
    [sym_get_timer] = ACTIONS(2073),
    [sym_master_sn] = ACTIONS(2073),
    [sym_system_number] = ACTIONS(2073),
    [sym_button] = ACTIONS(2073),
    [sym_dv_channel] = ACTIONS(2073),
    [sym_data] = ACTIONS(2073),
    [sym_level] = ACTIONS(2073),
    [sym_timeline] = ACTIONS(2073),
    [sym_channel] = ACTIONS(2073),
    [sym_custom] = ACTIONS(2073),
    [sym_atoi] = ACTIONS(2073),
    [sym_atol] = ACTIONS(2073),
    [sym_atof] = ACTIONS(2073),
    [sym_itoa] = ACTIONS(2073),
    [sym_itohex] = ACTIONS(2073),
    [sym_hextoi] = ACTIONS(2073),
    [sym_ftoa] = ACTIONS(2073),
    [sym_find_string] = ACTIONS(2073),
    [sym_left_string] = ACTIONS(2073),
    [sym_right_string] = ACTIONS(2073),
    [sym_mid_string] = ACTIONS(2073),
    [sym_remove_string] = ACTIONS(2073),
    [sym_get_buffer_string] = ACTIONS(2073),
    [sym_get_buffer_char] = ACTIONS(2073),
    [sym_get_multi_buffer_string] = ACTIONS(2073),
    [sym_length_string] = ACTIONS(2073),
    [sym_max_length_string] = ACTIONS(2073),
    [sym_set_length_string] = ACTIONS(2073),
    [sym_compare_string] = ACTIONS(2073),
    [sym_upper_string] = ACTIONS(2073),
    [sym_lower_string] = ACTIONS(2073),
    [sym_get_last] = ACTIONS(2073),
    [sym_date_to_day] = ACTIONS(2073),
    [sym_date_to_month] = ACTIONS(2073),
    [sym_date_to_year] = ACTIONS(2073),
    [sym_time_to_hour] = ACTIONS(2073),
    [sym_time_to_minute] = ACTIONS(2073),
    [sym_time_to_second] = ACTIONS(2073),
    [sym_day_of_week] = ACTIONS(2073),
    [sym_set_pulse_time] = ACTIONS(2073),
    [sym_set_timer] = ACTIONS(2073),
    [sym_file_open] = ACTIONS(2073),
    [sym_file_close] = ACTIONS(2073),
    [sym_file_read] = ACTIONS(2073),
    [sym_file_read_line] = ACTIONS(2073),
    [sym_file_write] = ACTIONS(2073),
    [sym_file_write_line] = ACTIONS(2073),
    [sym_file_copy] = ACTIONS(2073),
    [sym_file_delete] = ACTIONS(2073),
    [sym_file_rename] = ACTIONS(2073),
    [sym_file_dir] = ACTIONS(2073),
    [sym_file_getdir] = ACTIONS(2073),
    [sym_file_setdir] = ACTIONS(2073),
    [sym_file_create_dir] = ACTIONS(2073),
    [sym_file_remove_dir] = ACTIONS(2073),
    [sym_file_seek] = ACTIONS(2073),
    [sym_length_array] = ACTIONS(2073),
    [sym_max_length_array] = ACTIONS(2073),
    [sym_set_length_array] = ACTIONS(2073),
    [sym_ip_client_open] = ACTIONS(2073),
    [sym_ip_client_close] = ACTIONS(2073),
    [sym_ip_server_open] = ACTIONS(2073),
    [sym_ip_server_close] = ACTIONS(2073),
    [sym_ip_bound_client_open] = ACTIONS(2073),
    [sym_ip_mc_server_open] = ACTIONS(2073),
    [sym_ip_set_option] = ACTIONS(2073),
    [sym_ssh_client_open] = ACTIONS(2073),
    [sym_ssh_client_close] = ACTIONS(2073),
    [sym_tls_client_open] = ACTIONS(2073),
    [sym_tls_client_close] = ACTIONS(2073),
    [sym_do_push] = ACTIONS(2073),
    [sym_do_push_timed] = ACTIONS(2073),
    [sym_do_release] = ACTIONS(2073),
    [sym_device_id] = ACTIONS(2073),
    [sym_device_info] = ACTIONS(2073),
    [sym_device_id_string] = ACTIONS(2073),
    [sym_device_standby] = ACTIONS(2073),
    [sym_device_wake] = ACTIONS(2073),
    [sym_timeline_create] = ACTIONS(2073),
    [sym_timeline_kill] = ACTIONS(2073),
    [sym_timeline_reload] = ACTIONS(2073),
    [sym_timeline_pause] = ACTIONS(2073),
    [sym_timeline_restart] = ACTIONS(2073),
    [sym_timeline_set] = ACTIONS(2073),
    [sym_timeline_get] = ACTIONS(2073),
    [sym_timeline_active] = ACTIONS(2073),
    [sym_timeline_dynamic_id] = ACTIONS(2073),
    [sym_variable_to_string] = ACTIONS(2073),
    [sym_string_to_variable] = ACTIONS(2073),
    [sym_length_variable_to_string] = ACTIONS(2073),
    [sym_type_cast] = ACTIONS(2073),
    [sym_variable_to_xml] = ACTIONS(2073),
    [sym_xml_to_variable] = ACTIONS(2073),
    [sym_length_variable_to_xml] = ACTIONS(2073),
    [sym_redirect_string] = ACTIONS(2073),
    [sym_random_number] = ACTIONS(2073),
    [sym_get_unique_id] = ACTIONS(2073),
    [sym_get_system_number] = ACTIONS(2073),
    [sym_set_system_number] = ACTIONS(2073),
    [sym_get_serial_number] = ACTIONS(2073),
    [sym_get_master_build] = ACTIONS(2073),
    [sym_set_validation_code] = ACTIONS(2073),
    [sym_reboot] = ACTIONS(2073),
    [sym_load_duet_module] = ACTIONS(2073),
    [sym_unload_duet_module] = ACTIONS(2073),
    [sym_duet_mem_size_set] = ACTIONS(2073),
    [sym_duet_mem_size_get] = ACTIONS(2073),
    [sym_dynamic_application_device] = ACTIONS(2073),
    [sym_dynamic_polled_port] = ACTIONS(2073),
    [sym_static_port_binding] = ACTIONS(2073),
    [sym_static_ip_binding] = ACTIONS(2073),
    [sym_get_dns_list] = ACTIONS(2073),
    [sym_set_dns_list] = ACTIONS(2073),
    [sym_get_ip_address] = ACTIONS(2073),
    [sym_set_ip_address] = ACTIONS(2073),
    [sym_get_url_list] = ACTIONS(2073),
    [sym_add_url_entry] = ACTIONS(2073),
    [sym_delete_url_entry] = ACTIONS(2073),
    [sym_abs_value] = ACTIONS(2073),
    [sym_max_value] = ACTIONS(2073),
    [sym_min_value] = ACTIONS(2073),
    [sym_exp_value] = ACTIONS(2073),
    [sym_log_value] = ACTIONS(2073),
    [sym_log10_value] = ACTIONS(2073),
    [sym_power_value] = ACTIONS(2073),
    [sym_sqrt_value] = ACTIONS(2073),
    [sym_raw_be] = ACTIONS(2073),
    [sym_raw_le] = ACTIONS(2073),
    [sym_internal_queue_size_set] = ACTIONS(2073),
    [sym_internal_queue_size_get] = ACTIONS(2073),
    [sym_internal_threshold_set] = ACTIONS(2073),
    [sym_internal_threshold_get] = ACTIONS(2073),
    [sym_rebuilt_event] = ACTIONS(2073),
    [sym_do_custom_event] = ACTIONS(2073),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2073),
    [sym_clkmgr_set_clk_source] = ACTIONS(2073),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2073),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2073),
    [sym_clkmgr_get_timezone] = ACTIONS(2073),
    [sym_clkmgr_set_timezone] = ACTIONS(2073),
    [sym_clkmgr_get_resync_period] = ACTIONS(2073),
    [sym_clkmgr_set_resync_period] = ACTIONS(2073),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2073),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2073),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2073),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2073),
    [sym_clkmgr_get_timeservers] = ACTIONS(2073),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2073),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2073),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2073),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2073),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2073),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2073),
    [sym_set_log_level] = ACTIONS(2073),
    [sym_get_log_level] = ACTIONS(2073),
    [sym_amx_log] = ACTIONS(2073),
    [sym_smtp_server_config_set] = ACTIONS(2073),
    [sym_smtp_server_config_get] = ACTIONS(2073),
    [sym_smtp_send] = ACTIONS(2073),
    [sym_validate_netlinx_account] = ACTIONS(2073),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2073),
    [sym_authenticate_certificate] = ACTIONS(2073),
    [sym_audit_netlinx_session_event] = ACTIONS(2073),
    [sym_audit_netlinx_generic_event] = ACTIONS(2073),
    [sym_get_available_flash_disk_space] = ACTIONS(2073),
    [sym_get_max_flash_disk_space] = ACTIONS(2073),
    [sym_data_create_feed] = ACTIONS(2073),
    [sym_data_delete_feed] = ACTIONS(2073),
    [sym_data_publish_feed] = ACTIONS(2073),
    [sym_data_get_published_feed] = ACTIONS(2073),
    [sym_data_add_record] = ACTIONS(2073),
    [sym_data_get_event_record] = ACTIONS(2073),
    [sym__wc_data_create_feed] = ACTIONS(2073),
    [sym__wc_data_add_record] = ACTIONS(2073),
    [sym__wc_data_get_event_record] = ACTIONS(2073),
    [sym_format] = ACTIONS(2073),
    [sym_astro_clock] = ACTIONS(2073),
    [sym_set_outdoor_temperature] = ACTIONS(2073),
    [sym_set_virtual_level_count] = ACTIONS(2073),
    [sym_set_virtual_channel_count] = ACTIONS(2073),
    [sym_set_virtual_port_count] = ACTIONS(2073),
    [sym__wc_encode_int] = ACTIONS(2073),
    [sym__wc_decode_int] = ACTIONS(2073),
    [sym__wc_int] = ACTIONS(2073),
    [sym__wc_to_ch_int] = ACTIONS(2073),
    [sym__ch_to_wc_int] = ACTIONS(2073),
    [sym__wc_find_string_int] = ACTIONS(2073),
    [sym__wc_left_string_int] = ACTIONS(2073),
    [sym__wc_lower_string_int] = ACTIONS(2073),
    [sym__wc_mid_string_int] = ACTIONS(2073),
    [sym__wc_remove_string_int] = ACTIONS(2073),
    [sym__wc_right_string_int] = ACTIONS(2073),
    [sym__wc_upper_string_int] = ACTIONS(2073),
    [sym__wc_compare_string_int] = ACTIONS(2073),
    [sym__wc_get_buffer_char_int] = ACTIONS(2073),
    [sym__wc_get_buffer_string_int] = ACTIONS(2073),
    [sym__wc_concat_string_int] = ACTIONS(2073),
    [sym__wc_file_open_int] = ACTIONS(2073),
    [sym__wc_file_close_int] = ACTIONS(2073),
    [sym__wc_file_read_int] = ACTIONS(2073),
    [sym__wc_file_read_line_int] = ACTIONS(2073),
    [sym__wc_file_write_int] = ACTIONS(2073),
    [sym__wc_file_write_line_int] = ACTIONS(2073),
    [aux_sym_preproc_include_token1] = ACTIONS(2073),
    [aux_sym_preproc_define_token1] = ACTIONS(2075),
    [aux_sym_preproc_warn_token1] = ACTIONS(2075),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2075),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2075),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2075),
    [anon_sym_LBRACK] = ACTIONS(2075),
    [anon_sym_LBRACE] = ACTIONS(2075),
    [anon_sym_RBRACE] = ACTIONS(2075),
    [anon_sym_LPAREN] = ACTIONS(2073),
    [anon_sym_BANG] = ACTIONS(2075),
    [anon_sym_TILDE] = ACTIONS(2075),
    [anon_sym_DASH] = ACTIONS(2073),
    [anon_sym_PLUS] = ACTIONS(2073),
    [anon_sym_DASH_DASH] = ACTIONS(2075),
    [anon_sym_PLUS_PLUS] = ACTIONS(2075),
    [anon_sym_DQUOTE] = ACTIONS(2075),
    [anon_sym_SQUOTE] = ACTIONS(2075),
    [sym_number_literal] = ACTIONS(2075),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(535)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(311),
    [sym_statement] = STATE(287),
    [sym_expression_statement] = STATE(311),
    [sym_if_statement] = STATE(311),
    [sym_switch_statement] = STATE(311),
    [sym_case_statement] = STATE(311),
    [sym_while_statement] = STATE(311),
    [sym_for_statement] = STATE(311),
    [sym_select_statement] = STATE(311),
    [sym_active_statement] = STATE(311),
    [sym_return_statement] = STATE(311),
    [sym_break_statement] = STATE(311),
    [sym_continue_statement] = STATE(311),
    [sym_devchan_operation_statement] = STATE(311),
    [sym_devchan_operation] = STATE(1878),
    [sym_send_string_statement] = STATE(311),
    [sym_send_command_statement] = STATE(311),
    [sym_send_level_statement] = STATE(311),
    [sym_create_buffer_statement] = STATE(311),
    [sym_create_multi_buffer_statement] = STATE(311),
    [sym_clear_buffer_statement] = STATE(311),
    [sym_wait_statement] = STATE(311),
    [sym_wait_until_statement] = STATE(311),
    [sym_cancel_all_wait_statement] = STATE(311),
    [sym_cancel_all_wait_until_statement] = STATE(311),
    [sym_cancel_wait_statement] = STATE(311),
    [sym_cancel_wait_until_statement] = STATE(311),
    [sym_call_statement] = STATE(311),
    [sym_system_call_statement] = STATE(311),
    [sym_expression] = STATE(1137),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1990),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(55),
    [sym_switch_keyword] = ACTIONS(57),
    [sym_case_keyword] = ACTIONS(59),
    [sym_default_keyword] = ACTIONS(61),
    [sym_while_keyword] = ACTIONS(63),
    [sym_for_keyword] = ACTIONS(65),
    [sym_break_keyword] = ACTIONS(67),
    [sym_continue_keyword] = ACTIONS(69),
    [sym_return_keyword] = ACTIONS(71),
    [sym_select_keyword] = ACTIONS(73),
    [sym_active_keyword] = ACTIONS(75),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(99),
    [sym_send_string_keyword] = ACTIONS(101),
    [sym_send_command_keyword] = ACTIONS(103),
    [sym_clear_buffer_keyword] = ACTIONS(105),
    [sym_create_buffer_keyword] = ACTIONS(107),
    [sym_create_multi_buffer_keyword] = ACTIONS(109),
    [sym_call_keyword] = ACTIONS(111),
    [sym_system_call_keyword] = ACTIONS(113),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(117),
    [sym_cancel_wait_keyword] = ACTIONS(119),
    [sym_wait_until_keyword] = ACTIONS(121),
    [sym_cancel_wait_until_keyword] = ACTIONS(123),
    [sym_cancel_all_wait_keyword] = ACTIONS(125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(127),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(536)] = {
    [sym_identifier] = ACTIONS(2121),
    [sym_define_call_keyword] = ACTIONS(2121),
    [sym_define_function_keyword] = ACTIONS(2121),
    [sym_define_library_function_keyword] = ACTIONS(2121),
    [sym_define_module_keyword] = ACTIONS(2121),
    [sym_char_keyword] = ACTIONS(2121),
    [sym_widechar_keyword] = ACTIONS(2121),
    [sym_integer_keyword] = ACTIONS(2121),
    [sym_sinteger_keyword] = ACTIONS(2121),
    [sym_long_keyword] = ACTIONS(2121),
    [sym_slong_keyword] = ACTIONS(2121),
    [sym_float_keyword] = ACTIONS(2121),
    [sym_double_keyword] = ACTIONS(2121),
    [sym_variant_keyword] = ACTIONS(2121),
    [sym_variantarray_keyword] = ACTIONS(2121),
    [sym_local_var_keyword] = ACTIONS(2121),
    [sym_stack_var_keyword] = ACTIONS(2121),
    [sym_constant_keyword] = ACTIONS(2121),
    [sym_volatile_keyword] = ACTIONS(2121),
    [sym_non_volatile_keyword] = ACTIONS(2121),
    [sym_persistent_keyword] = ACTIONS(2121),
    [sym_dev_keyword] = ACTIONS(2121),
    [sym_devlev_keyword] = ACTIONS(2121),
    [sym_devchan_keyword] = ACTIONS(2121),
    [sym_if_keyword] = ACTIONS(2121),
    [sym_switch_keyword] = ACTIONS(2121),
    [sym_case_keyword] = ACTIONS(2121),
    [sym_default_keyword] = ACTIONS(2121),
    [sym_while_keyword] = ACTIONS(2121),
    [sym_for_keyword] = ACTIONS(2121),
    [sym_break_keyword] = ACTIONS(2121),
    [sym_continue_keyword] = ACTIONS(2121),
    [sym_return_keyword] = ACTIONS(2121),
    [sym_select_keyword] = ACTIONS(2121),
    [sym_active_keyword] = ACTIONS(2121),
    [sym_struct_keyword] = ACTIONS(2121),
    [sym_structure_keyword] = ACTIONS(2121),
    [aux_sym_on_keyword_token1] = ACTIONS(2121),
    [aux_sym_off_keyword_token1] = ACTIONS(2121),
    [sym_send_level_keyword] = ACTIONS(2121),
    [sym_send_string_keyword] = ACTIONS(2121),
    [sym_send_command_keyword] = ACTIONS(2121),
    [sym_clear_buffer_keyword] = ACTIONS(2121),
    [sym_create_buffer_keyword] = ACTIONS(2121),
    [sym_create_multi_buffer_keyword] = ACTIONS(2121),
    [sym_call_keyword] = ACTIONS(2121),
    [sym_system_call_keyword] = ACTIONS(2121),
    [sym_devchan_to_keyword] = ACTIONS(2121),
    [sym_devchan_min_to_keyword] = ACTIONS(2121),
    [sym_devchan_total_off_keyword] = ACTIONS(2121),
    [sym_devchan_pulse_keyword] = ACTIONS(2121),
    [sym_wait_keyword] = ACTIONS(2121),
    [sym_cancel_wait_keyword] = ACTIONS(2121),
    [sym_wait_until_keyword] = ACTIONS(2121),
    [sym_cancel_wait_until_keyword] = ACTIONS(2121),
    [sym_cancel_all_wait_keyword] = ACTIONS(2121),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2121),
    [sym_not] = ACTIONS(2121),
    [sym_bnot] = ACTIONS(2121),
    [sym___file__] = ACTIONS(2121),
    [sym___date__] = ACTIONS(2121),
    [sym___ldate__] = ACTIONS(2121),
    [sym___time__] = ACTIONS(2121),
    [sym___line__] = ACTIONS(2121),
    [sym___name__] = ACTIONS(2121),
    [sym___netlinx__] = ACTIONS(2121),
    [sym_netlinx_axi_version] = ACTIONS(2121),
    [sym_url_flg_tcp] = ACTIONS(2121),
    [sym_url_flg_accinfopresent] = ACTIONS(2121),
    [sym_url_flg_temp] = ACTIONS(2121),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2121),
    [sym_url_flg_stat_mask] = ACTIONS(2121),
    [sym_url_flg_stat_lookup] = ACTIONS(2121),
    [sym_url_flg_stat_connecting] = ACTIONS(2121),
    [sym_url_flg_stat_waiting] = ACTIONS(2121),
    [sym_url_flg_stat_connected] = ACTIONS(2121),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2121),
    [sym_first_virtual_device] = ACTIONS(2121),
    [sym_first_local_port] = ACTIONS(2121),
    [sym_dynamic_virtual_device] = ACTIONS(2121),
    [sym_timeline_once] = ACTIONS(2121),
    [sym_timeline_repeat] = ACTIONS(2121),
    [sym_timeline_absolute] = ACTIONS(2121),
    [sym_timeline_relative] = ACTIONS(2121),
    [sym_do_push_timed_infinite] = ACTIONS(2121),
    [sym_source_type_no_address] = ACTIONS(2121),
    [sym_source_type_neuron_id] = ACTIONS(2121),
    [sym_source_type_ip_address] = ACTIONS(2121),
    [sym_source_type_axlink] = ACTIONS(2121),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2121),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2121),
    [sym_source_type_ip_socket_address] = ACTIONS(2121),
    [sym_source_type_rs232] = ACTIONS(2121),
    [sym_source_type_internal] = ACTIONS(2121),
    [sym_source_type_mac_address] = ACTIONS(2121),
    [sym_source_type_ipv4_port] = ACTIONS(2121),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2121),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2121),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2121),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2121),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2121),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2121),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2121),
    [sym_true] = ACTIONS(2121),
    [sym_false] = ACTIONS(2121),
    [sym_file_read_only] = ACTIONS(2121),
    [sym_file_rw_new] = ACTIONS(2121),
    [sym_file_rw_append] = ACTIONS(2121),
    [sym_ip_tcp] = ACTIONS(2121),
    [sym_ip_udp] = ACTIONS(2121),
    [sym_ip_udp_2way] = ACTIONS(2121),
    [sym_xml_encode_types] = ACTIONS(2121),
    [sym_xml_encode_char_as_list] = ACTIONS(2121),
    [sym_xml_encode_le] = ACTIONS(2121),
    [sym_xml_decode_types] = ACTIONS(2121),
    [sym_xml_decode_no_preserve] = ACTIONS(2121),
    [sym_duet_dev_type_display_device] = ACTIONS(2121),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2121),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2121),
    [sym_duet_dev_type_io_device] = ACTIONS(2121),
    [sym_duet_dev_type_relay_device] = ACTIONS(2121),
    [sym_duet_dev_type_ups] = ACTIONS(2121),
    [sym_duet_dev_type_amplifier] = ACTIONS(2121),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2121),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2121),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2121),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2121),
    [sym_duet_dev_type_camera] = ACTIONS(2121),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2121),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2121),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2121),
    [sym_duet_dev_type_dss] = ACTIONS(2121),
    [sym_duet_dev_type_dvr] = ACTIONS(2121),
    [sym_duet_dev_type_disc_device] = ACTIONS(2121),
    [sym_duet_dev_type_document_camera] = ACTIONS(2121),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2121),
    [sym_duet_dev_type_hvac] = ACTIONS(2121),
    [sym_duet_dev_type_keypad] = ACTIONS(2121),
    [sym_duet_dev_type_light] = ACTIONS(2121),
    [sym_duet_dev_type_monitor] = ACTIONS(2121),
    [sym_duet_dev_type_motor] = ACTIONS(2121),
    [sym_duet_dev_type_multi_window] = ACTIONS(2121),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2121),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2121),
    [sym_duet_dev_type_receiver] = ACTIONS(2121),
    [sym_duet_dev_type_security_system] = ACTIONS(2121),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2121),
    [sym_duet_dev_type_settop_box] = ACTIONS(2121),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2121),
    [sym_duet_dev_type_switcher] = ACTIONS(2121),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2121),
    [sym_duet_dev_type_tv] = ACTIONS(2121),
    [sym_duet_dev_type_utility] = ACTIONS(2121),
    [sym_duet_dev_type_vcr] = ACTIONS(2121),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2121),
    [sym_duet_dev_type_video_processor] = ACTIONS(2121),
    [sym_duet_dev_type_video_projector] = ACTIONS(2121),
    [sym_duet_dev_type_video_wall] = ACTIONS(2121),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2121),
    [sym_duet_dev_type_weather] = ACTIONS(2121),
    [sym_duet_dev_not_polled] = ACTIONS(2121),
    [sym_duet_dev_polled] = ACTIONS(2121),
    [sym_clkmgr_mode_network] = ACTIONS(2121),
    [sym_clkmgr_mode_standalone] = ACTIONS(2121),
    [sym_normal_standby] = ACTIONS(2121),
    [sym_normal_wake] = ACTIONS(2121),
    [sym_ip_multicast_ttl_option] = ACTIONS(2121),
    [sym_ip_ttl_subnet] = ACTIONS(2121),
    [sym_ip_ttl_site] = ACTIONS(2121),
    [sym_ip_ttl_region] = ACTIONS(2121),
    [sym_ip_ttl_continent] = ACTIONS(2121),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2121),
    [sym_ip_nodelay_on] = ACTIONS(2121),
    [sym_ip_nodelay_off] = ACTIONS(2121),
    [sym_amx_error] = ACTIONS(2121),
    [sym_amx_warning] = ACTIONS(2121),
    [sym_amx_info] = ACTIONS(2121),
    [sym_amx_debug] = ACTIONS(2121),
    [sym_netlinx_login_success] = ACTIONS(2121),
    [sym_netlinx_login_fail] = ACTIONS(2121),
    [sym_netlinx_logout] = ACTIONS(2121),
    [sym_null_str] = ACTIONS(2121),
    [sym_smtp_address] = ACTIONS(2121),
    [sym_smtp_port_number] = ACTIONS(2121),
    [sym_smtp_username] = ACTIONS(2121),
    [sym_smtp_password] = ACTIONS(2121),
    [sym_smtp_require_tls] = ACTIONS(2121),
    [sym_smtp_from] = ACTIONS(2121),
    [sym_smtp_tls_true] = ACTIONS(2121),
    [sym_smtp_tls_false] = ACTIONS(2121),
    [sym_listview_on_row_select_event] = ACTIONS(2121),
    [sym_data_structure_unknown] = ACTIONS(2121),
    [sym_data_structure_datafeed] = ACTIONS(2121),
    [sym_data_structure_datarecordset] = ACTIONS(2121),
    [sym_data_structure_datarecord] = ACTIONS(2121),
    [sym_data_structure_datafield] = ACTIONS(2121),
    [sym_data_max_content_field_count] = ACTIONS(2121),
    [sym_data_max_metadata_field_count] = ACTIONS(2121),
    [sym_data_max_id_length] = ACTIONS(2121),
    [sym_data_max_name_length] = ACTIONS(2121),
    [sym_data_max_description_length] = ACTIONS(2121),
    [sym_data_max_type_length] = ACTIONS(2121),
    [sym_data_max_format_length] = ACTIONS(2121),
    [sym_data_max_source_length] = ACTIONS(2121),
    [sym_data_max_value_length] = ACTIONS(2121),
    [sym_data_max_label_length] = ACTIONS(2121),
    [sym_data_max_url_length] = ACTIONS(2121),
    [sym_max_last_login_info_length] = ACTIONS(2121),
    [sym_data_type_unknown] = ACTIONS(2121),
    [sym_data_type_string] = ACTIONS(2121),
    [sym_data_type_datetime] = ACTIONS(2121),
    [sym_data_type_date] = ACTIONS(2121),
    [sym_data_type_time] = ACTIONS(2121),
    [sym_data_type_image] = ACTIONS(2121),
    [sym_data_format_url] = ACTIONS(2121),
    [sym_data_format_phone] = ACTIONS(2121),
    [sym_data_format_email] = ACTIONS(2121),
    [sym_data_format_iso8601] = ACTIONS(2121),
    [sym_auditlog_priv] = ACTIONS(2121),
    [sym_configuration_priv] = ACTIONS(2121),
    [sym_deviceconfig_priv] = ACTIONS(2121),
    [sym_ftp_priv] = ACTIONS(2121),
    [sym_http_priv] = ACTIONS(2121),
    [sym_networkconfig_priv] = ACTIONS(2121),
    [sym_programport_priv] = ACTIONS(2121),
    [sym_remoteui_priv] = ACTIONS(2121),
    [sym_securitycontrol_priv] = ACTIONS(2121),
    [sym_softwaremanagement_priv] = ACTIONS(2121),
    [sym_terminal_priv] = ACTIONS(2121),
    [sym_tpadmin_priv] = ACTIONS(2121),
    [sym_user1_priv] = ACTIONS(2121),
    [sym_user2_priv] = ACTIONS(2121),
    [sym_user3_priv] = ACTIONS(2121),
    [sym_user4_priv] = ACTIONS(2121),
    [sym_usermanagement_priv] = ACTIONS(2121),
    [sym_valid_account] = ACTIONS(2121),
    [sym_err_name_invalid] = ACTIONS(2121),
    [sym_err_password_invalid] = ACTIONS(2121),
    [sym_err_invalid_account] = ACTIONS(2121),
    [sym_err_no_authorization] = ACTIONS(2121),
    [sym_err_invalid_parameter] = ACTIONS(2121),
    [sym_err_user_locked_out] = ACTIONS(2121),
    [sym_err_user_account_expired] = ACTIONS(2121),
    [sym_err_general_error] = ACTIONS(2121),
    [sym_err_authentication_error] = ACTIONS(2121),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2121),
    [sym_tls_validate_certificate] = ACTIONS(2121),
    [sym_tdata] = ACTIONS(2121),
    [sym_tchannel] = ACTIONS(2121),
    [sym_tlevel] = ACTIONS(2121),
    [sym_tbutton] = ACTIONS(2121),
    [sym_ttimeline] = ACTIONS(2121),
    [sym_tcustom] = ACTIONS(2121),
    [sym_url_struct] = ACTIONS(2121),
    [sym_dns_struct] = ACTIONS(2121),
    [sym_ip_address_struct] = ACTIONS(2121),
    [sym_dev_info_struct] = ACTIONS(2121),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2121),
    [sym_data_feed] = ACTIONS(2121),
    [sym_data_field] = ACTIONS(2121),
    [sym_data_record] = ACTIONS(2121),
    [sym_wc_data_feed] = ACTIONS(2121),
    [sym_wc_data_field] = ACTIONS(2121),
    [sym_wc_data_record] = ACTIONS(2121),
    [sym_last_login_info] = ACTIONS(2121),
    [sym____reserved___] = ACTIONS(2121),
    [sym_date] = ACTIONS(2121),
    [sym_day] = ACTIONS(2121),
    [sym_time] = ACTIONS(2121),
    [sym_ldate] = ACTIONS(2121),
    [sym_push_device] = ACTIONS(2121),
    [sym_push_channel] = ACTIONS(2121),
    [sym_push_devchan] = ACTIONS(2121),
    [sym_release_device] = ACTIONS(2121),
    [sym_release_channel] = ACTIONS(2121),
    [sym_release_devchan] = ACTIONS(2121),
    [sym_master_slot] = ACTIONS(2121),
    [sym_get_pulse_time] = ACTIONS(2121),
    [sym_get_timer] = ACTIONS(2121),
    [sym_master_sn] = ACTIONS(2121),
    [sym_system_number] = ACTIONS(2121),
    [sym_button] = ACTIONS(2121),
    [sym_dv_channel] = ACTIONS(2121),
    [sym_data] = ACTIONS(2121),
    [sym_level] = ACTIONS(2121),
    [sym_timeline] = ACTIONS(2121),
    [sym_channel] = ACTIONS(2121),
    [sym_custom] = ACTIONS(2121),
    [sym_atoi] = ACTIONS(2121),
    [sym_atol] = ACTIONS(2121),
    [sym_atof] = ACTIONS(2121),
    [sym_itoa] = ACTIONS(2121),
    [sym_itohex] = ACTIONS(2121),
    [sym_hextoi] = ACTIONS(2121),
    [sym_ftoa] = ACTIONS(2121),
    [sym_find_string] = ACTIONS(2121),
    [sym_left_string] = ACTIONS(2121),
    [sym_right_string] = ACTIONS(2121),
    [sym_mid_string] = ACTIONS(2121),
    [sym_remove_string] = ACTIONS(2121),
    [sym_get_buffer_string] = ACTIONS(2121),
    [sym_get_buffer_char] = ACTIONS(2121),
    [sym_get_multi_buffer_string] = ACTIONS(2121),
    [sym_length_string] = ACTIONS(2121),
    [sym_max_length_string] = ACTIONS(2121),
    [sym_set_length_string] = ACTIONS(2121),
    [sym_compare_string] = ACTIONS(2121),
    [sym_upper_string] = ACTIONS(2121),
    [sym_lower_string] = ACTIONS(2121),
    [sym_get_last] = ACTIONS(2121),
    [sym_date_to_day] = ACTIONS(2121),
    [sym_date_to_month] = ACTIONS(2121),
    [sym_date_to_year] = ACTIONS(2121),
    [sym_time_to_hour] = ACTIONS(2121),
    [sym_time_to_minute] = ACTIONS(2121),
    [sym_time_to_second] = ACTIONS(2121),
    [sym_day_of_week] = ACTIONS(2121),
    [sym_set_pulse_time] = ACTIONS(2121),
    [sym_set_timer] = ACTIONS(2121),
    [sym_file_open] = ACTIONS(2121),
    [sym_file_close] = ACTIONS(2121),
    [sym_file_read] = ACTIONS(2121),
    [sym_file_read_line] = ACTIONS(2121),
    [sym_file_write] = ACTIONS(2121),
    [sym_file_write_line] = ACTIONS(2121),
    [sym_file_copy] = ACTIONS(2121),
    [sym_file_delete] = ACTIONS(2121),
    [sym_file_rename] = ACTIONS(2121),
    [sym_file_dir] = ACTIONS(2121),
    [sym_file_getdir] = ACTIONS(2121),
    [sym_file_setdir] = ACTIONS(2121),
    [sym_file_create_dir] = ACTIONS(2121),
    [sym_file_remove_dir] = ACTIONS(2121),
    [sym_file_seek] = ACTIONS(2121),
    [sym_length_array] = ACTIONS(2121),
    [sym_max_length_array] = ACTIONS(2121),
    [sym_set_length_array] = ACTIONS(2121),
    [sym_ip_client_open] = ACTIONS(2121),
    [sym_ip_client_close] = ACTIONS(2121),
    [sym_ip_server_open] = ACTIONS(2121),
    [sym_ip_server_close] = ACTIONS(2121),
    [sym_ip_bound_client_open] = ACTIONS(2121),
    [sym_ip_mc_server_open] = ACTIONS(2121),
    [sym_ip_set_option] = ACTIONS(2121),
    [sym_ssh_client_open] = ACTIONS(2121),
    [sym_ssh_client_close] = ACTIONS(2121),
    [sym_tls_client_open] = ACTIONS(2121),
    [sym_tls_client_close] = ACTIONS(2121),
    [sym_do_push] = ACTIONS(2121),
    [sym_do_push_timed] = ACTIONS(2121),
    [sym_do_release] = ACTIONS(2121),
    [sym_device_id] = ACTIONS(2121),
    [sym_device_info] = ACTIONS(2121),
    [sym_device_id_string] = ACTIONS(2121),
    [sym_device_standby] = ACTIONS(2121),
    [sym_device_wake] = ACTIONS(2121),
    [sym_timeline_create] = ACTIONS(2121),
    [sym_timeline_kill] = ACTIONS(2121),
    [sym_timeline_reload] = ACTIONS(2121),
    [sym_timeline_pause] = ACTIONS(2121),
    [sym_timeline_restart] = ACTIONS(2121),
    [sym_timeline_set] = ACTIONS(2121),
    [sym_timeline_get] = ACTIONS(2121),
    [sym_timeline_active] = ACTIONS(2121),
    [sym_timeline_dynamic_id] = ACTIONS(2121),
    [sym_variable_to_string] = ACTIONS(2121),
    [sym_string_to_variable] = ACTIONS(2121),
    [sym_length_variable_to_string] = ACTIONS(2121),
    [sym_type_cast] = ACTIONS(2121),
    [sym_variable_to_xml] = ACTIONS(2121),
    [sym_xml_to_variable] = ACTIONS(2121),
    [sym_length_variable_to_xml] = ACTIONS(2121),
    [sym_redirect_string] = ACTIONS(2121),
    [sym_random_number] = ACTIONS(2121),
    [sym_get_unique_id] = ACTIONS(2121),
    [sym_get_system_number] = ACTIONS(2121),
    [sym_set_system_number] = ACTIONS(2121),
    [sym_get_serial_number] = ACTIONS(2121),
    [sym_get_master_build] = ACTIONS(2121),
    [sym_set_validation_code] = ACTIONS(2121),
    [sym_reboot] = ACTIONS(2121),
    [sym_load_duet_module] = ACTIONS(2121),
    [sym_unload_duet_module] = ACTIONS(2121),
    [sym_duet_mem_size_set] = ACTIONS(2121),
    [sym_duet_mem_size_get] = ACTIONS(2121),
    [sym_dynamic_application_device] = ACTIONS(2121),
    [sym_dynamic_polled_port] = ACTIONS(2121),
    [sym_static_port_binding] = ACTIONS(2121),
    [sym_static_ip_binding] = ACTIONS(2121),
    [sym_get_dns_list] = ACTIONS(2121),
    [sym_set_dns_list] = ACTIONS(2121),
    [sym_get_ip_address] = ACTIONS(2121),
    [sym_set_ip_address] = ACTIONS(2121),
    [sym_get_url_list] = ACTIONS(2121),
    [sym_add_url_entry] = ACTIONS(2121),
    [sym_delete_url_entry] = ACTIONS(2121),
    [sym_abs_value] = ACTIONS(2121),
    [sym_max_value] = ACTIONS(2121),
    [sym_min_value] = ACTIONS(2121),
    [sym_exp_value] = ACTIONS(2121),
    [sym_log_value] = ACTIONS(2121),
    [sym_log10_value] = ACTIONS(2121),
    [sym_power_value] = ACTIONS(2121),
    [sym_sqrt_value] = ACTIONS(2121),
    [sym_raw_be] = ACTIONS(2121),
    [sym_raw_le] = ACTIONS(2121),
    [sym_internal_queue_size_set] = ACTIONS(2121),
    [sym_internal_queue_size_get] = ACTIONS(2121),
    [sym_internal_threshold_set] = ACTIONS(2121),
    [sym_internal_threshold_get] = ACTIONS(2121),
    [sym_rebuilt_event] = ACTIONS(2121),
    [sym_do_custom_event] = ACTIONS(2121),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2121),
    [sym_clkmgr_set_clk_source] = ACTIONS(2121),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2121),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2121),
    [sym_clkmgr_get_timezone] = ACTIONS(2121),
    [sym_clkmgr_set_timezone] = ACTIONS(2121),
    [sym_clkmgr_get_resync_period] = ACTIONS(2121),
    [sym_clkmgr_set_resync_period] = ACTIONS(2121),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2121),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2121),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2121),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2121),
    [sym_clkmgr_get_timeservers] = ACTIONS(2121),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2121),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2121),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2121),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2121),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2121),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2121),
    [sym_set_log_level] = ACTIONS(2121),
    [sym_get_log_level] = ACTIONS(2121),
    [sym_amx_log] = ACTIONS(2121),
    [sym_smtp_server_config_set] = ACTIONS(2121),
    [sym_smtp_server_config_get] = ACTIONS(2121),
    [sym_smtp_send] = ACTIONS(2121),
    [sym_validate_netlinx_account] = ACTIONS(2121),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2121),
    [sym_authenticate_certificate] = ACTIONS(2121),
    [sym_audit_netlinx_session_event] = ACTIONS(2121),
    [sym_audit_netlinx_generic_event] = ACTIONS(2121),
    [sym_get_available_flash_disk_space] = ACTIONS(2121),
    [sym_get_max_flash_disk_space] = ACTIONS(2121),
    [sym_data_create_feed] = ACTIONS(2121),
    [sym_data_delete_feed] = ACTIONS(2121),
    [sym_data_publish_feed] = ACTIONS(2121),
    [sym_data_get_published_feed] = ACTIONS(2121),
    [sym_data_add_record] = ACTIONS(2121),
    [sym_data_get_event_record] = ACTIONS(2121),
    [sym__wc_data_create_feed] = ACTIONS(2121),
    [sym__wc_data_add_record] = ACTIONS(2121),
    [sym__wc_data_get_event_record] = ACTIONS(2121),
    [sym_format] = ACTIONS(2121),
    [sym_astro_clock] = ACTIONS(2121),
    [sym_set_outdoor_temperature] = ACTIONS(2121),
    [sym_set_virtual_level_count] = ACTIONS(2121),
    [sym_set_virtual_channel_count] = ACTIONS(2121),
    [sym_set_virtual_port_count] = ACTIONS(2121),
    [sym__wc_encode_int] = ACTIONS(2121),
    [sym__wc_decode_int] = ACTIONS(2121),
    [sym__wc_int] = ACTIONS(2121),
    [sym__wc_to_ch_int] = ACTIONS(2121),
    [sym__ch_to_wc_int] = ACTIONS(2121),
    [sym__wc_find_string_int] = ACTIONS(2121),
    [sym__wc_left_string_int] = ACTIONS(2121),
    [sym__wc_lower_string_int] = ACTIONS(2121),
    [sym__wc_mid_string_int] = ACTIONS(2121),
    [sym__wc_remove_string_int] = ACTIONS(2121),
    [sym__wc_right_string_int] = ACTIONS(2121),
    [sym__wc_upper_string_int] = ACTIONS(2121),
    [sym__wc_compare_string_int] = ACTIONS(2121),
    [sym__wc_get_buffer_char_int] = ACTIONS(2121),
    [sym__wc_get_buffer_string_int] = ACTIONS(2121),
    [sym__wc_concat_string_int] = ACTIONS(2121),
    [sym__wc_file_open_int] = ACTIONS(2121),
    [sym__wc_file_close_int] = ACTIONS(2121),
    [sym__wc_file_read_int] = ACTIONS(2121),
    [sym__wc_file_read_line_int] = ACTIONS(2121),
    [sym__wc_file_write_int] = ACTIONS(2121),
    [sym__wc_file_write_line_int] = ACTIONS(2121),
    [aux_sym_preproc_include_token1] = ACTIONS(2121),
    [aux_sym_preproc_define_token1] = ACTIONS(2123),
    [aux_sym_preproc_warn_token1] = ACTIONS(2123),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2123),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2123),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2123),
    [anon_sym_LBRACK] = ACTIONS(2123),
    [anon_sym_LBRACE] = ACTIONS(2123),
    [anon_sym_RBRACE] = ACTIONS(2123),
    [anon_sym_LPAREN] = ACTIONS(2121),
    [anon_sym_BANG] = ACTIONS(2123),
    [anon_sym_TILDE] = ACTIONS(2123),
    [anon_sym_DASH] = ACTIONS(2121),
    [anon_sym_PLUS] = ACTIONS(2121),
    [anon_sym_DASH_DASH] = ACTIONS(2123),
    [anon_sym_PLUS_PLUS] = ACTIONS(2123),
    [anon_sym_DQUOTE] = ACTIONS(2123),
    [anon_sym_SQUOTE] = ACTIONS(2123),
    [sym_number_literal] = ACTIONS(2123),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(537)] = {
    [sym_identifier] = ACTIONS(2125),
    [sym_define_call_keyword] = ACTIONS(2125),
    [sym_define_function_keyword] = ACTIONS(2125),
    [sym_define_library_function_keyword] = ACTIONS(2125),
    [sym_define_module_keyword] = ACTIONS(2125),
    [sym_char_keyword] = ACTIONS(2125),
    [sym_widechar_keyword] = ACTIONS(2125),
    [sym_integer_keyword] = ACTIONS(2125),
    [sym_sinteger_keyword] = ACTIONS(2125),
    [sym_long_keyword] = ACTIONS(2125),
    [sym_slong_keyword] = ACTIONS(2125),
    [sym_float_keyword] = ACTIONS(2125),
    [sym_double_keyword] = ACTIONS(2125),
    [sym_variant_keyword] = ACTIONS(2125),
    [sym_variantarray_keyword] = ACTIONS(2125),
    [sym_local_var_keyword] = ACTIONS(2125),
    [sym_stack_var_keyword] = ACTIONS(2125),
    [sym_constant_keyword] = ACTIONS(2125),
    [sym_volatile_keyword] = ACTIONS(2125),
    [sym_non_volatile_keyword] = ACTIONS(2125),
    [sym_persistent_keyword] = ACTIONS(2125),
    [sym_dev_keyword] = ACTIONS(2125),
    [sym_devlev_keyword] = ACTIONS(2125),
    [sym_devchan_keyword] = ACTIONS(2125),
    [sym_if_keyword] = ACTIONS(2125),
    [sym_switch_keyword] = ACTIONS(2125),
    [sym_case_keyword] = ACTIONS(2125),
    [sym_default_keyword] = ACTIONS(2125),
    [sym_while_keyword] = ACTIONS(2125),
    [sym_for_keyword] = ACTIONS(2125),
    [sym_break_keyword] = ACTIONS(2125),
    [sym_continue_keyword] = ACTIONS(2125),
    [sym_return_keyword] = ACTIONS(2125),
    [sym_select_keyword] = ACTIONS(2125),
    [sym_active_keyword] = ACTIONS(2125),
    [sym_struct_keyword] = ACTIONS(2125),
    [sym_structure_keyword] = ACTIONS(2125),
    [aux_sym_on_keyword_token1] = ACTIONS(2125),
    [aux_sym_off_keyword_token1] = ACTIONS(2125),
    [sym_send_level_keyword] = ACTIONS(2125),
    [sym_send_string_keyword] = ACTIONS(2125),
    [sym_send_command_keyword] = ACTIONS(2125),
    [sym_clear_buffer_keyword] = ACTIONS(2125),
    [sym_create_buffer_keyword] = ACTIONS(2125),
    [sym_create_multi_buffer_keyword] = ACTIONS(2125),
    [sym_call_keyword] = ACTIONS(2125),
    [sym_system_call_keyword] = ACTIONS(2125),
    [sym_devchan_to_keyword] = ACTIONS(2125),
    [sym_devchan_min_to_keyword] = ACTIONS(2125),
    [sym_devchan_total_off_keyword] = ACTIONS(2125),
    [sym_devchan_pulse_keyword] = ACTIONS(2125),
    [sym_wait_keyword] = ACTIONS(2125),
    [sym_cancel_wait_keyword] = ACTIONS(2125),
    [sym_wait_until_keyword] = ACTIONS(2125),
    [sym_cancel_wait_until_keyword] = ACTIONS(2125),
    [sym_cancel_all_wait_keyword] = ACTIONS(2125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2125),
    [sym_not] = ACTIONS(2125),
    [sym_bnot] = ACTIONS(2125),
    [sym___file__] = ACTIONS(2125),
    [sym___date__] = ACTIONS(2125),
    [sym___ldate__] = ACTIONS(2125),
    [sym___time__] = ACTIONS(2125),
    [sym___line__] = ACTIONS(2125),
    [sym___name__] = ACTIONS(2125),
    [sym___netlinx__] = ACTIONS(2125),
    [sym_netlinx_axi_version] = ACTIONS(2125),
    [sym_url_flg_tcp] = ACTIONS(2125),
    [sym_url_flg_accinfopresent] = ACTIONS(2125),
    [sym_url_flg_temp] = ACTIONS(2125),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2125),
    [sym_url_flg_stat_mask] = ACTIONS(2125),
    [sym_url_flg_stat_lookup] = ACTIONS(2125),
    [sym_url_flg_stat_connecting] = ACTIONS(2125),
    [sym_url_flg_stat_waiting] = ACTIONS(2125),
    [sym_url_flg_stat_connected] = ACTIONS(2125),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2125),
    [sym_first_virtual_device] = ACTIONS(2125),
    [sym_first_local_port] = ACTIONS(2125),
    [sym_dynamic_virtual_device] = ACTIONS(2125),
    [sym_timeline_once] = ACTIONS(2125),
    [sym_timeline_repeat] = ACTIONS(2125),
    [sym_timeline_absolute] = ACTIONS(2125),
    [sym_timeline_relative] = ACTIONS(2125),
    [sym_do_push_timed_infinite] = ACTIONS(2125),
    [sym_source_type_no_address] = ACTIONS(2125),
    [sym_source_type_neuron_id] = ACTIONS(2125),
    [sym_source_type_ip_address] = ACTIONS(2125),
    [sym_source_type_axlink] = ACTIONS(2125),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2125),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2125),
    [sym_source_type_ip_socket_address] = ACTIONS(2125),
    [sym_source_type_rs232] = ACTIONS(2125),
    [sym_source_type_internal] = ACTIONS(2125),
    [sym_source_type_mac_address] = ACTIONS(2125),
    [sym_source_type_ipv4_port] = ACTIONS(2125),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2125),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2125),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2125),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2125),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2125),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2125),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2125),
    [sym_true] = ACTIONS(2125),
    [sym_false] = ACTIONS(2125),
    [sym_file_read_only] = ACTIONS(2125),
    [sym_file_rw_new] = ACTIONS(2125),
    [sym_file_rw_append] = ACTIONS(2125),
    [sym_ip_tcp] = ACTIONS(2125),
    [sym_ip_udp] = ACTIONS(2125),
    [sym_ip_udp_2way] = ACTIONS(2125),
    [sym_xml_encode_types] = ACTIONS(2125),
    [sym_xml_encode_char_as_list] = ACTIONS(2125),
    [sym_xml_encode_le] = ACTIONS(2125),
    [sym_xml_decode_types] = ACTIONS(2125),
    [sym_xml_decode_no_preserve] = ACTIONS(2125),
    [sym_duet_dev_type_display_device] = ACTIONS(2125),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2125),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2125),
    [sym_duet_dev_type_io_device] = ACTIONS(2125),
    [sym_duet_dev_type_relay_device] = ACTIONS(2125),
    [sym_duet_dev_type_ups] = ACTIONS(2125),
    [sym_duet_dev_type_amplifier] = ACTIONS(2125),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2125),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2125),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2125),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2125),
    [sym_duet_dev_type_camera] = ACTIONS(2125),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2125),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2125),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2125),
    [sym_duet_dev_type_dss] = ACTIONS(2125),
    [sym_duet_dev_type_dvr] = ACTIONS(2125),
    [sym_duet_dev_type_disc_device] = ACTIONS(2125),
    [sym_duet_dev_type_document_camera] = ACTIONS(2125),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2125),
    [sym_duet_dev_type_hvac] = ACTIONS(2125),
    [sym_duet_dev_type_keypad] = ACTIONS(2125),
    [sym_duet_dev_type_light] = ACTIONS(2125),
    [sym_duet_dev_type_monitor] = ACTIONS(2125),
    [sym_duet_dev_type_motor] = ACTIONS(2125),
    [sym_duet_dev_type_multi_window] = ACTIONS(2125),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2125),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2125),
    [sym_duet_dev_type_receiver] = ACTIONS(2125),
    [sym_duet_dev_type_security_system] = ACTIONS(2125),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2125),
    [sym_duet_dev_type_settop_box] = ACTIONS(2125),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2125),
    [sym_duet_dev_type_switcher] = ACTIONS(2125),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2125),
    [sym_duet_dev_type_tv] = ACTIONS(2125),
    [sym_duet_dev_type_utility] = ACTIONS(2125),
    [sym_duet_dev_type_vcr] = ACTIONS(2125),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2125),
    [sym_duet_dev_type_video_processor] = ACTIONS(2125),
    [sym_duet_dev_type_video_projector] = ACTIONS(2125),
    [sym_duet_dev_type_video_wall] = ACTIONS(2125),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2125),
    [sym_duet_dev_type_weather] = ACTIONS(2125),
    [sym_duet_dev_not_polled] = ACTIONS(2125),
    [sym_duet_dev_polled] = ACTIONS(2125),
    [sym_clkmgr_mode_network] = ACTIONS(2125),
    [sym_clkmgr_mode_standalone] = ACTIONS(2125),
    [sym_normal_standby] = ACTIONS(2125),
    [sym_normal_wake] = ACTIONS(2125),
    [sym_ip_multicast_ttl_option] = ACTIONS(2125),
    [sym_ip_ttl_subnet] = ACTIONS(2125),
    [sym_ip_ttl_site] = ACTIONS(2125),
    [sym_ip_ttl_region] = ACTIONS(2125),
    [sym_ip_ttl_continent] = ACTIONS(2125),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2125),
    [sym_ip_nodelay_on] = ACTIONS(2125),
    [sym_ip_nodelay_off] = ACTIONS(2125),
    [sym_amx_error] = ACTIONS(2125),
    [sym_amx_warning] = ACTIONS(2125),
    [sym_amx_info] = ACTIONS(2125),
    [sym_amx_debug] = ACTIONS(2125),
    [sym_netlinx_login_success] = ACTIONS(2125),
    [sym_netlinx_login_fail] = ACTIONS(2125),
    [sym_netlinx_logout] = ACTIONS(2125),
    [sym_null_str] = ACTIONS(2125),
    [sym_smtp_address] = ACTIONS(2125),
    [sym_smtp_port_number] = ACTIONS(2125),
    [sym_smtp_username] = ACTIONS(2125),
    [sym_smtp_password] = ACTIONS(2125),
    [sym_smtp_require_tls] = ACTIONS(2125),
    [sym_smtp_from] = ACTIONS(2125),
    [sym_smtp_tls_true] = ACTIONS(2125),
    [sym_smtp_tls_false] = ACTIONS(2125),
    [sym_listview_on_row_select_event] = ACTIONS(2125),
    [sym_data_structure_unknown] = ACTIONS(2125),
    [sym_data_structure_datafeed] = ACTIONS(2125),
    [sym_data_structure_datarecordset] = ACTIONS(2125),
    [sym_data_structure_datarecord] = ACTIONS(2125),
    [sym_data_structure_datafield] = ACTIONS(2125),
    [sym_data_max_content_field_count] = ACTIONS(2125),
    [sym_data_max_metadata_field_count] = ACTIONS(2125),
    [sym_data_max_id_length] = ACTIONS(2125),
    [sym_data_max_name_length] = ACTIONS(2125),
    [sym_data_max_description_length] = ACTIONS(2125),
    [sym_data_max_type_length] = ACTIONS(2125),
    [sym_data_max_format_length] = ACTIONS(2125),
    [sym_data_max_source_length] = ACTIONS(2125),
    [sym_data_max_value_length] = ACTIONS(2125),
    [sym_data_max_label_length] = ACTIONS(2125),
    [sym_data_max_url_length] = ACTIONS(2125),
    [sym_max_last_login_info_length] = ACTIONS(2125),
    [sym_data_type_unknown] = ACTIONS(2125),
    [sym_data_type_string] = ACTIONS(2125),
    [sym_data_type_datetime] = ACTIONS(2125),
    [sym_data_type_date] = ACTIONS(2125),
    [sym_data_type_time] = ACTIONS(2125),
    [sym_data_type_image] = ACTIONS(2125),
    [sym_data_format_url] = ACTIONS(2125),
    [sym_data_format_phone] = ACTIONS(2125),
    [sym_data_format_email] = ACTIONS(2125),
    [sym_data_format_iso8601] = ACTIONS(2125),
    [sym_auditlog_priv] = ACTIONS(2125),
    [sym_configuration_priv] = ACTIONS(2125),
    [sym_deviceconfig_priv] = ACTIONS(2125),
    [sym_ftp_priv] = ACTIONS(2125),
    [sym_http_priv] = ACTIONS(2125),
    [sym_networkconfig_priv] = ACTIONS(2125),
    [sym_programport_priv] = ACTIONS(2125),
    [sym_remoteui_priv] = ACTIONS(2125),
    [sym_securitycontrol_priv] = ACTIONS(2125),
    [sym_softwaremanagement_priv] = ACTIONS(2125),
    [sym_terminal_priv] = ACTIONS(2125),
    [sym_tpadmin_priv] = ACTIONS(2125),
    [sym_user1_priv] = ACTIONS(2125),
    [sym_user2_priv] = ACTIONS(2125),
    [sym_user3_priv] = ACTIONS(2125),
    [sym_user4_priv] = ACTIONS(2125),
    [sym_usermanagement_priv] = ACTIONS(2125),
    [sym_valid_account] = ACTIONS(2125),
    [sym_err_name_invalid] = ACTIONS(2125),
    [sym_err_password_invalid] = ACTIONS(2125),
    [sym_err_invalid_account] = ACTIONS(2125),
    [sym_err_no_authorization] = ACTIONS(2125),
    [sym_err_invalid_parameter] = ACTIONS(2125),
    [sym_err_user_locked_out] = ACTIONS(2125),
    [sym_err_user_account_expired] = ACTIONS(2125),
    [sym_err_general_error] = ACTIONS(2125),
    [sym_err_authentication_error] = ACTIONS(2125),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2125),
    [sym_tls_validate_certificate] = ACTIONS(2125),
    [sym_tdata] = ACTIONS(2125),
    [sym_tchannel] = ACTIONS(2125),
    [sym_tlevel] = ACTIONS(2125),
    [sym_tbutton] = ACTIONS(2125),
    [sym_ttimeline] = ACTIONS(2125),
    [sym_tcustom] = ACTIONS(2125),
    [sym_url_struct] = ACTIONS(2125),
    [sym_dns_struct] = ACTIONS(2125),
    [sym_ip_address_struct] = ACTIONS(2125),
    [sym_dev_info_struct] = ACTIONS(2125),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2125),
    [sym_data_feed] = ACTIONS(2125),
    [sym_data_field] = ACTIONS(2125),
    [sym_data_record] = ACTIONS(2125),
    [sym_wc_data_feed] = ACTIONS(2125),
    [sym_wc_data_field] = ACTIONS(2125),
    [sym_wc_data_record] = ACTIONS(2125),
    [sym_last_login_info] = ACTIONS(2125),
    [sym____reserved___] = ACTIONS(2125),
    [sym_date] = ACTIONS(2125),
    [sym_day] = ACTIONS(2125),
    [sym_time] = ACTIONS(2125),
    [sym_ldate] = ACTIONS(2125),
    [sym_push_device] = ACTIONS(2125),
    [sym_push_channel] = ACTIONS(2125),
    [sym_push_devchan] = ACTIONS(2125),
    [sym_release_device] = ACTIONS(2125),
    [sym_release_channel] = ACTIONS(2125),
    [sym_release_devchan] = ACTIONS(2125),
    [sym_master_slot] = ACTIONS(2125),
    [sym_get_pulse_time] = ACTIONS(2125),
    [sym_get_timer] = ACTIONS(2125),
    [sym_master_sn] = ACTIONS(2125),
    [sym_system_number] = ACTIONS(2125),
    [sym_button] = ACTIONS(2125),
    [sym_dv_channel] = ACTIONS(2125),
    [sym_data] = ACTIONS(2125),
    [sym_level] = ACTIONS(2125),
    [sym_timeline] = ACTIONS(2125),
    [sym_channel] = ACTIONS(2125),
    [sym_custom] = ACTIONS(2125),
    [sym_atoi] = ACTIONS(2125),
    [sym_atol] = ACTIONS(2125),
    [sym_atof] = ACTIONS(2125),
    [sym_itoa] = ACTIONS(2125),
    [sym_itohex] = ACTIONS(2125),
    [sym_hextoi] = ACTIONS(2125),
    [sym_ftoa] = ACTIONS(2125),
    [sym_find_string] = ACTIONS(2125),
    [sym_left_string] = ACTIONS(2125),
    [sym_right_string] = ACTIONS(2125),
    [sym_mid_string] = ACTIONS(2125),
    [sym_remove_string] = ACTIONS(2125),
    [sym_get_buffer_string] = ACTIONS(2125),
    [sym_get_buffer_char] = ACTIONS(2125),
    [sym_get_multi_buffer_string] = ACTIONS(2125),
    [sym_length_string] = ACTIONS(2125),
    [sym_max_length_string] = ACTIONS(2125),
    [sym_set_length_string] = ACTIONS(2125),
    [sym_compare_string] = ACTIONS(2125),
    [sym_upper_string] = ACTIONS(2125),
    [sym_lower_string] = ACTIONS(2125),
    [sym_get_last] = ACTIONS(2125),
    [sym_date_to_day] = ACTIONS(2125),
    [sym_date_to_month] = ACTIONS(2125),
    [sym_date_to_year] = ACTIONS(2125),
    [sym_time_to_hour] = ACTIONS(2125),
    [sym_time_to_minute] = ACTIONS(2125),
    [sym_time_to_second] = ACTIONS(2125),
    [sym_day_of_week] = ACTIONS(2125),
    [sym_set_pulse_time] = ACTIONS(2125),
    [sym_set_timer] = ACTIONS(2125),
    [sym_file_open] = ACTIONS(2125),
    [sym_file_close] = ACTIONS(2125),
    [sym_file_read] = ACTIONS(2125),
    [sym_file_read_line] = ACTIONS(2125),
    [sym_file_write] = ACTIONS(2125),
    [sym_file_write_line] = ACTIONS(2125),
    [sym_file_copy] = ACTIONS(2125),
    [sym_file_delete] = ACTIONS(2125),
    [sym_file_rename] = ACTIONS(2125),
    [sym_file_dir] = ACTIONS(2125),
    [sym_file_getdir] = ACTIONS(2125),
    [sym_file_setdir] = ACTIONS(2125),
    [sym_file_create_dir] = ACTIONS(2125),
    [sym_file_remove_dir] = ACTIONS(2125),
    [sym_file_seek] = ACTIONS(2125),
    [sym_length_array] = ACTIONS(2125),
    [sym_max_length_array] = ACTIONS(2125),
    [sym_set_length_array] = ACTIONS(2125),
    [sym_ip_client_open] = ACTIONS(2125),
    [sym_ip_client_close] = ACTIONS(2125),
    [sym_ip_server_open] = ACTIONS(2125),
    [sym_ip_server_close] = ACTIONS(2125),
    [sym_ip_bound_client_open] = ACTIONS(2125),
    [sym_ip_mc_server_open] = ACTIONS(2125),
    [sym_ip_set_option] = ACTIONS(2125),
    [sym_ssh_client_open] = ACTIONS(2125),
    [sym_ssh_client_close] = ACTIONS(2125),
    [sym_tls_client_open] = ACTIONS(2125),
    [sym_tls_client_close] = ACTIONS(2125),
    [sym_do_push] = ACTIONS(2125),
    [sym_do_push_timed] = ACTIONS(2125),
    [sym_do_release] = ACTIONS(2125),
    [sym_device_id] = ACTIONS(2125),
    [sym_device_info] = ACTIONS(2125),
    [sym_device_id_string] = ACTIONS(2125),
    [sym_device_standby] = ACTIONS(2125),
    [sym_device_wake] = ACTIONS(2125),
    [sym_timeline_create] = ACTIONS(2125),
    [sym_timeline_kill] = ACTIONS(2125),
    [sym_timeline_reload] = ACTIONS(2125),
    [sym_timeline_pause] = ACTIONS(2125),
    [sym_timeline_restart] = ACTIONS(2125),
    [sym_timeline_set] = ACTIONS(2125),
    [sym_timeline_get] = ACTIONS(2125),
    [sym_timeline_active] = ACTIONS(2125),
    [sym_timeline_dynamic_id] = ACTIONS(2125),
    [sym_variable_to_string] = ACTIONS(2125),
    [sym_string_to_variable] = ACTIONS(2125),
    [sym_length_variable_to_string] = ACTIONS(2125),
    [sym_type_cast] = ACTIONS(2125),
    [sym_variable_to_xml] = ACTIONS(2125),
    [sym_xml_to_variable] = ACTIONS(2125),
    [sym_length_variable_to_xml] = ACTIONS(2125),
    [sym_redirect_string] = ACTIONS(2125),
    [sym_random_number] = ACTIONS(2125),
    [sym_get_unique_id] = ACTIONS(2125),
    [sym_get_system_number] = ACTIONS(2125),
    [sym_set_system_number] = ACTIONS(2125),
    [sym_get_serial_number] = ACTIONS(2125),
    [sym_get_master_build] = ACTIONS(2125),
    [sym_set_validation_code] = ACTIONS(2125),
    [sym_reboot] = ACTIONS(2125),
    [sym_load_duet_module] = ACTIONS(2125),
    [sym_unload_duet_module] = ACTIONS(2125),
    [sym_duet_mem_size_set] = ACTIONS(2125),
    [sym_duet_mem_size_get] = ACTIONS(2125),
    [sym_dynamic_application_device] = ACTIONS(2125),
    [sym_dynamic_polled_port] = ACTIONS(2125),
    [sym_static_port_binding] = ACTIONS(2125),
    [sym_static_ip_binding] = ACTIONS(2125),
    [sym_get_dns_list] = ACTIONS(2125),
    [sym_set_dns_list] = ACTIONS(2125),
    [sym_get_ip_address] = ACTIONS(2125),
    [sym_set_ip_address] = ACTIONS(2125),
    [sym_get_url_list] = ACTIONS(2125),
    [sym_add_url_entry] = ACTIONS(2125),
    [sym_delete_url_entry] = ACTIONS(2125),
    [sym_abs_value] = ACTIONS(2125),
    [sym_max_value] = ACTIONS(2125),
    [sym_min_value] = ACTIONS(2125),
    [sym_exp_value] = ACTIONS(2125),
    [sym_log_value] = ACTIONS(2125),
    [sym_log10_value] = ACTIONS(2125),
    [sym_power_value] = ACTIONS(2125),
    [sym_sqrt_value] = ACTIONS(2125),
    [sym_raw_be] = ACTIONS(2125),
    [sym_raw_le] = ACTIONS(2125),
    [sym_internal_queue_size_set] = ACTIONS(2125),
    [sym_internal_queue_size_get] = ACTIONS(2125),
    [sym_internal_threshold_set] = ACTIONS(2125),
    [sym_internal_threshold_get] = ACTIONS(2125),
    [sym_rebuilt_event] = ACTIONS(2125),
    [sym_do_custom_event] = ACTIONS(2125),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2125),
    [sym_clkmgr_set_clk_source] = ACTIONS(2125),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2125),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2125),
    [sym_clkmgr_get_timezone] = ACTIONS(2125),
    [sym_clkmgr_set_timezone] = ACTIONS(2125),
    [sym_clkmgr_get_resync_period] = ACTIONS(2125),
    [sym_clkmgr_set_resync_period] = ACTIONS(2125),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2125),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2125),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2125),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2125),
    [sym_clkmgr_get_timeservers] = ACTIONS(2125),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2125),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2125),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2125),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2125),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2125),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2125),
    [sym_set_log_level] = ACTIONS(2125),
    [sym_get_log_level] = ACTIONS(2125),
    [sym_amx_log] = ACTIONS(2125),
    [sym_smtp_server_config_set] = ACTIONS(2125),
    [sym_smtp_server_config_get] = ACTIONS(2125),
    [sym_smtp_send] = ACTIONS(2125),
    [sym_validate_netlinx_account] = ACTIONS(2125),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2125),
    [sym_authenticate_certificate] = ACTIONS(2125),
    [sym_audit_netlinx_session_event] = ACTIONS(2125),
    [sym_audit_netlinx_generic_event] = ACTIONS(2125),
    [sym_get_available_flash_disk_space] = ACTIONS(2125),
    [sym_get_max_flash_disk_space] = ACTIONS(2125),
    [sym_data_create_feed] = ACTIONS(2125),
    [sym_data_delete_feed] = ACTIONS(2125),
    [sym_data_publish_feed] = ACTIONS(2125),
    [sym_data_get_published_feed] = ACTIONS(2125),
    [sym_data_add_record] = ACTIONS(2125),
    [sym_data_get_event_record] = ACTIONS(2125),
    [sym__wc_data_create_feed] = ACTIONS(2125),
    [sym__wc_data_add_record] = ACTIONS(2125),
    [sym__wc_data_get_event_record] = ACTIONS(2125),
    [sym_format] = ACTIONS(2125),
    [sym_astro_clock] = ACTIONS(2125),
    [sym_set_outdoor_temperature] = ACTIONS(2125),
    [sym_set_virtual_level_count] = ACTIONS(2125),
    [sym_set_virtual_channel_count] = ACTIONS(2125),
    [sym_set_virtual_port_count] = ACTIONS(2125),
    [sym__wc_encode_int] = ACTIONS(2125),
    [sym__wc_decode_int] = ACTIONS(2125),
    [sym__wc_int] = ACTIONS(2125),
    [sym__wc_to_ch_int] = ACTIONS(2125),
    [sym__ch_to_wc_int] = ACTIONS(2125),
    [sym__wc_find_string_int] = ACTIONS(2125),
    [sym__wc_left_string_int] = ACTIONS(2125),
    [sym__wc_lower_string_int] = ACTIONS(2125),
    [sym__wc_mid_string_int] = ACTIONS(2125),
    [sym__wc_remove_string_int] = ACTIONS(2125),
    [sym__wc_right_string_int] = ACTIONS(2125),
    [sym__wc_upper_string_int] = ACTIONS(2125),
    [sym__wc_compare_string_int] = ACTIONS(2125),
    [sym__wc_get_buffer_char_int] = ACTIONS(2125),
    [sym__wc_get_buffer_string_int] = ACTIONS(2125),
    [sym__wc_concat_string_int] = ACTIONS(2125),
    [sym__wc_file_open_int] = ACTIONS(2125),
    [sym__wc_file_close_int] = ACTIONS(2125),
    [sym__wc_file_read_int] = ACTIONS(2125),
    [sym__wc_file_read_line_int] = ACTIONS(2125),
    [sym__wc_file_write_int] = ACTIONS(2125),
    [sym__wc_file_write_line_int] = ACTIONS(2125),
    [aux_sym_preproc_include_token1] = ACTIONS(2125),
    [aux_sym_preproc_define_token1] = ACTIONS(2127),
    [aux_sym_preproc_warn_token1] = ACTIONS(2127),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2127),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2127),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2127),
    [anon_sym_LBRACK] = ACTIONS(2127),
    [anon_sym_LBRACE] = ACTIONS(2127),
    [anon_sym_RBRACE] = ACTIONS(2127),
    [anon_sym_LPAREN] = ACTIONS(2125),
    [anon_sym_BANG] = ACTIONS(2127),
    [anon_sym_TILDE] = ACTIONS(2127),
    [anon_sym_DASH] = ACTIONS(2125),
    [anon_sym_PLUS] = ACTIONS(2125),
    [anon_sym_DASH_DASH] = ACTIONS(2127),
    [anon_sym_PLUS_PLUS] = ACTIONS(2127),
    [anon_sym_DQUOTE] = ACTIONS(2127),
    [anon_sym_SQUOTE] = ACTIONS(2127),
    [sym_number_literal] = ACTIONS(2127),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(538)] = {
    [sym_identifier] = ACTIONS(2129),
    [sym_define_call_keyword] = ACTIONS(2129),
    [sym_define_function_keyword] = ACTIONS(2129),
    [sym_define_library_function_keyword] = ACTIONS(2129),
    [sym_define_module_keyword] = ACTIONS(2129),
    [sym_char_keyword] = ACTIONS(2129),
    [sym_widechar_keyword] = ACTIONS(2129),
    [sym_integer_keyword] = ACTIONS(2129),
    [sym_sinteger_keyword] = ACTIONS(2129),
    [sym_long_keyword] = ACTIONS(2129),
    [sym_slong_keyword] = ACTIONS(2129),
    [sym_float_keyword] = ACTIONS(2129),
    [sym_double_keyword] = ACTIONS(2129),
    [sym_variant_keyword] = ACTIONS(2129),
    [sym_variantarray_keyword] = ACTIONS(2129),
    [sym_local_var_keyword] = ACTIONS(2129),
    [sym_stack_var_keyword] = ACTIONS(2129),
    [sym_constant_keyword] = ACTIONS(2129),
    [sym_volatile_keyword] = ACTIONS(2129),
    [sym_non_volatile_keyword] = ACTIONS(2129),
    [sym_persistent_keyword] = ACTIONS(2129),
    [sym_dev_keyword] = ACTIONS(2129),
    [sym_devlev_keyword] = ACTIONS(2129),
    [sym_devchan_keyword] = ACTIONS(2129),
    [sym_if_keyword] = ACTIONS(2129),
    [sym_switch_keyword] = ACTIONS(2129),
    [sym_case_keyword] = ACTIONS(2129),
    [sym_default_keyword] = ACTIONS(2129),
    [sym_while_keyword] = ACTIONS(2129),
    [sym_for_keyword] = ACTIONS(2129),
    [sym_break_keyword] = ACTIONS(2129),
    [sym_continue_keyword] = ACTIONS(2129),
    [sym_return_keyword] = ACTIONS(2129),
    [sym_select_keyword] = ACTIONS(2129),
    [sym_active_keyword] = ACTIONS(2129),
    [sym_struct_keyword] = ACTIONS(2129),
    [sym_structure_keyword] = ACTIONS(2129),
    [aux_sym_on_keyword_token1] = ACTIONS(2129),
    [aux_sym_off_keyword_token1] = ACTIONS(2129),
    [sym_send_level_keyword] = ACTIONS(2129),
    [sym_send_string_keyword] = ACTIONS(2129),
    [sym_send_command_keyword] = ACTIONS(2129),
    [sym_clear_buffer_keyword] = ACTIONS(2129),
    [sym_create_buffer_keyword] = ACTIONS(2129),
    [sym_create_multi_buffer_keyword] = ACTIONS(2129),
    [sym_call_keyword] = ACTIONS(2129),
    [sym_system_call_keyword] = ACTIONS(2129),
    [sym_devchan_to_keyword] = ACTIONS(2129),
    [sym_devchan_min_to_keyword] = ACTIONS(2129),
    [sym_devchan_total_off_keyword] = ACTIONS(2129),
    [sym_devchan_pulse_keyword] = ACTIONS(2129),
    [sym_wait_keyword] = ACTIONS(2129),
    [sym_cancel_wait_keyword] = ACTIONS(2129),
    [sym_wait_until_keyword] = ACTIONS(2129),
    [sym_cancel_wait_until_keyword] = ACTIONS(2129),
    [sym_cancel_all_wait_keyword] = ACTIONS(2129),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2129),
    [sym_not] = ACTIONS(2129),
    [sym_bnot] = ACTIONS(2129),
    [sym___file__] = ACTIONS(2129),
    [sym___date__] = ACTIONS(2129),
    [sym___ldate__] = ACTIONS(2129),
    [sym___time__] = ACTIONS(2129),
    [sym___line__] = ACTIONS(2129),
    [sym___name__] = ACTIONS(2129),
    [sym___netlinx__] = ACTIONS(2129),
    [sym_netlinx_axi_version] = ACTIONS(2129),
    [sym_url_flg_tcp] = ACTIONS(2129),
    [sym_url_flg_accinfopresent] = ACTIONS(2129),
    [sym_url_flg_temp] = ACTIONS(2129),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2129),
    [sym_url_flg_stat_mask] = ACTIONS(2129),
    [sym_url_flg_stat_lookup] = ACTIONS(2129),
    [sym_url_flg_stat_connecting] = ACTIONS(2129),
    [sym_url_flg_stat_waiting] = ACTIONS(2129),
    [sym_url_flg_stat_connected] = ACTIONS(2129),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2129),
    [sym_first_virtual_device] = ACTIONS(2129),
    [sym_first_local_port] = ACTIONS(2129),
    [sym_dynamic_virtual_device] = ACTIONS(2129),
    [sym_timeline_once] = ACTIONS(2129),
    [sym_timeline_repeat] = ACTIONS(2129),
    [sym_timeline_absolute] = ACTIONS(2129),
    [sym_timeline_relative] = ACTIONS(2129),
    [sym_do_push_timed_infinite] = ACTIONS(2129),
    [sym_source_type_no_address] = ACTIONS(2129),
    [sym_source_type_neuron_id] = ACTIONS(2129),
    [sym_source_type_ip_address] = ACTIONS(2129),
    [sym_source_type_axlink] = ACTIONS(2129),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2129),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2129),
    [sym_source_type_ip_socket_address] = ACTIONS(2129),
    [sym_source_type_rs232] = ACTIONS(2129),
    [sym_source_type_internal] = ACTIONS(2129),
    [sym_source_type_mac_address] = ACTIONS(2129),
    [sym_source_type_ipv4_port] = ACTIONS(2129),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2129),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2129),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2129),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2129),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2129),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2129),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2129),
    [sym_true] = ACTIONS(2129),
    [sym_false] = ACTIONS(2129),
    [sym_file_read_only] = ACTIONS(2129),
    [sym_file_rw_new] = ACTIONS(2129),
    [sym_file_rw_append] = ACTIONS(2129),
    [sym_ip_tcp] = ACTIONS(2129),
    [sym_ip_udp] = ACTIONS(2129),
    [sym_ip_udp_2way] = ACTIONS(2129),
    [sym_xml_encode_types] = ACTIONS(2129),
    [sym_xml_encode_char_as_list] = ACTIONS(2129),
    [sym_xml_encode_le] = ACTIONS(2129),
    [sym_xml_decode_types] = ACTIONS(2129),
    [sym_xml_decode_no_preserve] = ACTIONS(2129),
    [sym_duet_dev_type_display_device] = ACTIONS(2129),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2129),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2129),
    [sym_duet_dev_type_io_device] = ACTIONS(2129),
    [sym_duet_dev_type_relay_device] = ACTIONS(2129),
    [sym_duet_dev_type_ups] = ACTIONS(2129),
    [sym_duet_dev_type_amplifier] = ACTIONS(2129),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2129),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2129),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2129),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2129),
    [sym_duet_dev_type_camera] = ACTIONS(2129),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2129),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2129),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2129),
    [sym_duet_dev_type_dss] = ACTIONS(2129),
    [sym_duet_dev_type_dvr] = ACTIONS(2129),
    [sym_duet_dev_type_disc_device] = ACTIONS(2129),
    [sym_duet_dev_type_document_camera] = ACTIONS(2129),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2129),
    [sym_duet_dev_type_hvac] = ACTIONS(2129),
    [sym_duet_dev_type_keypad] = ACTIONS(2129),
    [sym_duet_dev_type_light] = ACTIONS(2129),
    [sym_duet_dev_type_monitor] = ACTIONS(2129),
    [sym_duet_dev_type_motor] = ACTIONS(2129),
    [sym_duet_dev_type_multi_window] = ACTIONS(2129),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2129),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2129),
    [sym_duet_dev_type_receiver] = ACTIONS(2129),
    [sym_duet_dev_type_security_system] = ACTIONS(2129),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2129),
    [sym_duet_dev_type_settop_box] = ACTIONS(2129),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2129),
    [sym_duet_dev_type_switcher] = ACTIONS(2129),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2129),
    [sym_duet_dev_type_tv] = ACTIONS(2129),
    [sym_duet_dev_type_utility] = ACTIONS(2129),
    [sym_duet_dev_type_vcr] = ACTIONS(2129),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2129),
    [sym_duet_dev_type_video_processor] = ACTIONS(2129),
    [sym_duet_dev_type_video_projector] = ACTIONS(2129),
    [sym_duet_dev_type_video_wall] = ACTIONS(2129),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2129),
    [sym_duet_dev_type_weather] = ACTIONS(2129),
    [sym_duet_dev_not_polled] = ACTIONS(2129),
    [sym_duet_dev_polled] = ACTIONS(2129),
    [sym_clkmgr_mode_network] = ACTIONS(2129),
    [sym_clkmgr_mode_standalone] = ACTIONS(2129),
    [sym_normal_standby] = ACTIONS(2129),
    [sym_normal_wake] = ACTIONS(2129),
    [sym_ip_multicast_ttl_option] = ACTIONS(2129),
    [sym_ip_ttl_subnet] = ACTIONS(2129),
    [sym_ip_ttl_site] = ACTIONS(2129),
    [sym_ip_ttl_region] = ACTIONS(2129),
    [sym_ip_ttl_continent] = ACTIONS(2129),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2129),
    [sym_ip_nodelay_on] = ACTIONS(2129),
    [sym_ip_nodelay_off] = ACTIONS(2129),
    [sym_amx_error] = ACTIONS(2129),
    [sym_amx_warning] = ACTIONS(2129),
    [sym_amx_info] = ACTIONS(2129),
    [sym_amx_debug] = ACTIONS(2129),
    [sym_netlinx_login_success] = ACTIONS(2129),
    [sym_netlinx_login_fail] = ACTIONS(2129),
    [sym_netlinx_logout] = ACTIONS(2129),
    [sym_null_str] = ACTIONS(2129),
    [sym_smtp_address] = ACTIONS(2129),
    [sym_smtp_port_number] = ACTIONS(2129),
    [sym_smtp_username] = ACTIONS(2129),
    [sym_smtp_password] = ACTIONS(2129),
    [sym_smtp_require_tls] = ACTIONS(2129),
    [sym_smtp_from] = ACTIONS(2129),
    [sym_smtp_tls_true] = ACTIONS(2129),
    [sym_smtp_tls_false] = ACTIONS(2129),
    [sym_listview_on_row_select_event] = ACTIONS(2129),
    [sym_data_structure_unknown] = ACTIONS(2129),
    [sym_data_structure_datafeed] = ACTIONS(2129),
    [sym_data_structure_datarecordset] = ACTIONS(2129),
    [sym_data_structure_datarecord] = ACTIONS(2129),
    [sym_data_structure_datafield] = ACTIONS(2129),
    [sym_data_max_content_field_count] = ACTIONS(2129),
    [sym_data_max_metadata_field_count] = ACTIONS(2129),
    [sym_data_max_id_length] = ACTIONS(2129),
    [sym_data_max_name_length] = ACTIONS(2129),
    [sym_data_max_description_length] = ACTIONS(2129),
    [sym_data_max_type_length] = ACTIONS(2129),
    [sym_data_max_format_length] = ACTIONS(2129),
    [sym_data_max_source_length] = ACTIONS(2129),
    [sym_data_max_value_length] = ACTIONS(2129),
    [sym_data_max_label_length] = ACTIONS(2129),
    [sym_data_max_url_length] = ACTIONS(2129),
    [sym_max_last_login_info_length] = ACTIONS(2129),
    [sym_data_type_unknown] = ACTIONS(2129),
    [sym_data_type_string] = ACTIONS(2129),
    [sym_data_type_datetime] = ACTIONS(2129),
    [sym_data_type_date] = ACTIONS(2129),
    [sym_data_type_time] = ACTIONS(2129),
    [sym_data_type_image] = ACTIONS(2129),
    [sym_data_format_url] = ACTIONS(2129),
    [sym_data_format_phone] = ACTIONS(2129),
    [sym_data_format_email] = ACTIONS(2129),
    [sym_data_format_iso8601] = ACTIONS(2129),
    [sym_auditlog_priv] = ACTIONS(2129),
    [sym_configuration_priv] = ACTIONS(2129),
    [sym_deviceconfig_priv] = ACTIONS(2129),
    [sym_ftp_priv] = ACTIONS(2129),
    [sym_http_priv] = ACTIONS(2129),
    [sym_networkconfig_priv] = ACTIONS(2129),
    [sym_programport_priv] = ACTIONS(2129),
    [sym_remoteui_priv] = ACTIONS(2129),
    [sym_securitycontrol_priv] = ACTIONS(2129),
    [sym_softwaremanagement_priv] = ACTIONS(2129),
    [sym_terminal_priv] = ACTIONS(2129),
    [sym_tpadmin_priv] = ACTIONS(2129),
    [sym_user1_priv] = ACTIONS(2129),
    [sym_user2_priv] = ACTIONS(2129),
    [sym_user3_priv] = ACTIONS(2129),
    [sym_user4_priv] = ACTIONS(2129),
    [sym_usermanagement_priv] = ACTIONS(2129),
    [sym_valid_account] = ACTIONS(2129),
    [sym_err_name_invalid] = ACTIONS(2129),
    [sym_err_password_invalid] = ACTIONS(2129),
    [sym_err_invalid_account] = ACTIONS(2129),
    [sym_err_no_authorization] = ACTIONS(2129),
    [sym_err_invalid_parameter] = ACTIONS(2129),
    [sym_err_user_locked_out] = ACTIONS(2129),
    [sym_err_user_account_expired] = ACTIONS(2129),
    [sym_err_general_error] = ACTIONS(2129),
    [sym_err_authentication_error] = ACTIONS(2129),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2129),
    [sym_tls_validate_certificate] = ACTIONS(2129),
    [sym_tdata] = ACTIONS(2129),
    [sym_tchannel] = ACTIONS(2129),
    [sym_tlevel] = ACTIONS(2129),
    [sym_tbutton] = ACTIONS(2129),
    [sym_ttimeline] = ACTIONS(2129),
    [sym_tcustom] = ACTIONS(2129),
    [sym_url_struct] = ACTIONS(2129),
    [sym_dns_struct] = ACTIONS(2129),
    [sym_ip_address_struct] = ACTIONS(2129),
    [sym_dev_info_struct] = ACTIONS(2129),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2129),
    [sym_data_feed] = ACTIONS(2129),
    [sym_data_field] = ACTIONS(2129),
    [sym_data_record] = ACTIONS(2129),
    [sym_wc_data_feed] = ACTIONS(2129),
    [sym_wc_data_field] = ACTIONS(2129),
    [sym_wc_data_record] = ACTIONS(2129),
    [sym_last_login_info] = ACTIONS(2129),
    [sym____reserved___] = ACTIONS(2129),
    [sym_date] = ACTIONS(2129),
    [sym_day] = ACTIONS(2129),
    [sym_time] = ACTIONS(2129),
    [sym_ldate] = ACTIONS(2129),
    [sym_push_device] = ACTIONS(2129),
    [sym_push_channel] = ACTIONS(2129),
    [sym_push_devchan] = ACTIONS(2129),
    [sym_release_device] = ACTIONS(2129),
    [sym_release_channel] = ACTIONS(2129),
    [sym_release_devchan] = ACTIONS(2129),
    [sym_master_slot] = ACTIONS(2129),
    [sym_get_pulse_time] = ACTIONS(2129),
    [sym_get_timer] = ACTIONS(2129),
    [sym_master_sn] = ACTIONS(2129),
    [sym_system_number] = ACTIONS(2129),
    [sym_button] = ACTIONS(2129),
    [sym_dv_channel] = ACTIONS(2129),
    [sym_data] = ACTIONS(2129),
    [sym_level] = ACTIONS(2129),
    [sym_timeline] = ACTIONS(2129),
    [sym_channel] = ACTIONS(2129),
    [sym_custom] = ACTIONS(2129),
    [sym_atoi] = ACTIONS(2129),
    [sym_atol] = ACTIONS(2129),
    [sym_atof] = ACTIONS(2129),
    [sym_itoa] = ACTIONS(2129),
    [sym_itohex] = ACTIONS(2129),
    [sym_hextoi] = ACTIONS(2129),
    [sym_ftoa] = ACTIONS(2129),
    [sym_find_string] = ACTIONS(2129),
    [sym_left_string] = ACTIONS(2129),
    [sym_right_string] = ACTIONS(2129),
    [sym_mid_string] = ACTIONS(2129),
    [sym_remove_string] = ACTIONS(2129),
    [sym_get_buffer_string] = ACTIONS(2129),
    [sym_get_buffer_char] = ACTIONS(2129),
    [sym_get_multi_buffer_string] = ACTIONS(2129),
    [sym_length_string] = ACTIONS(2129),
    [sym_max_length_string] = ACTIONS(2129),
    [sym_set_length_string] = ACTIONS(2129),
    [sym_compare_string] = ACTIONS(2129),
    [sym_upper_string] = ACTIONS(2129),
    [sym_lower_string] = ACTIONS(2129),
    [sym_get_last] = ACTIONS(2129),
    [sym_date_to_day] = ACTIONS(2129),
    [sym_date_to_month] = ACTIONS(2129),
    [sym_date_to_year] = ACTIONS(2129),
    [sym_time_to_hour] = ACTIONS(2129),
    [sym_time_to_minute] = ACTIONS(2129),
    [sym_time_to_second] = ACTIONS(2129),
    [sym_day_of_week] = ACTIONS(2129),
    [sym_set_pulse_time] = ACTIONS(2129),
    [sym_set_timer] = ACTIONS(2129),
    [sym_file_open] = ACTIONS(2129),
    [sym_file_close] = ACTIONS(2129),
    [sym_file_read] = ACTIONS(2129),
    [sym_file_read_line] = ACTIONS(2129),
    [sym_file_write] = ACTIONS(2129),
    [sym_file_write_line] = ACTIONS(2129),
    [sym_file_copy] = ACTIONS(2129),
    [sym_file_delete] = ACTIONS(2129),
    [sym_file_rename] = ACTIONS(2129),
    [sym_file_dir] = ACTIONS(2129),
    [sym_file_getdir] = ACTIONS(2129),
    [sym_file_setdir] = ACTIONS(2129),
    [sym_file_create_dir] = ACTIONS(2129),
    [sym_file_remove_dir] = ACTIONS(2129),
    [sym_file_seek] = ACTIONS(2129),
    [sym_length_array] = ACTIONS(2129),
    [sym_max_length_array] = ACTIONS(2129),
    [sym_set_length_array] = ACTIONS(2129),
    [sym_ip_client_open] = ACTIONS(2129),
    [sym_ip_client_close] = ACTIONS(2129),
    [sym_ip_server_open] = ACTIONS(2129),
    [sym_ip_server_close] = ACTIONS(2129),
    [sym_ip_bound_client_open] = ACTIONS(2129),
    [sym_ip_mc_server_open] = ACTIONS(2129),
    [sym_ip_set_option] = ACTIONS(2129),
    [sym_ssh_client_open] = ACTIONS(2129),
    [sym_ssh_client_close] = ACTIONS(2129),
    [sym_tls_client_open] = ACTIONS(2129),
    [sym_tls_client_close] = ACTIONS(2129),
    [sym_do_push] = ACTIONS(2129),
    [sym_do_push_timed] = ACTIONS(2129),
    [sym_do_release] = ACTIONS(2129),
    [sym_device_id] = ACTIONS(2129),
    [sym_device_info] = ACTIONS(2129),
    [sym_device_id_string] = ACTIONS(2129),
    [sym_device_standby] = ACTIONS(2129),
    [sym_device_wake] = ACTIONS(2129),
    [sym_timeline_create] = ACTIONS(2129),
    [sym_timeline_kill] = ACTIONS(2129),
    [sym_timeline_reload] = ACTIONS(2129),
    [sym_timeline_pause] = ACTIONS(2129),
    [sym_timeline_restart] = ACTIONS(2129),
    [sym_timeline_set] = ACTIONS(2129),
    [sym_timeline_get] = ACTIONS(2129),
    [sym_timeline_active] = ACTIONS(2129),
    [sym_timeline_dynamic_id] = ACTIONS(2129),
    [sym_variable_to_string] = ACTIONS(2129),
    [sym_string_to_variable] = ACTIONS(2129),
    [sym_length_variable_to_string] = ACTIONS(2129),
    [sym_type_cast] = ACTIONS(2129),
    [sym_variable_to_xml] = ACTIONS(2129),
    [sym_xml_to_variable] = ACTIONS(2129),
    [sym_length_variable_to_xml] = ACTIONS(2129),
    [sym_redirect_string] = ACTIONS(2129),
    [sym_random_number] = ACTIONS(2129),
    [sym_get_unique_id] = ACTIONS(2129),
    [sym_get_system_number] = ACTIONS(2129),
    [sym_set_system_number] = ACTIONS(2129),
    [sym_get_serial_number] = ACTIONS(2129),
    [sym_get_master_build] = ACTIONS(2129),
    [sym_set_validation_code] = ACTIONS(2129),
    [sym_reboot] = ACTIONS(2129),
    [sym_load_duet_module] = ACTIONS(2129),
    [sym_unload_duet_module] = ACTIONS(2129),
    [sym_duet_mem_size_set] = ACTIONS(2129),
    [sym_duet_mem_size_get] = ACTIONS(2129),
    [sym_dynamic_application_device] = ACTIONS(2129),
    [sym_dynamic_polled_port] = ACTIONS(2129),
    [sym_static_port_binding] = ACTIONS(2129),
    [sym_static_ip_binding] = ACTIONS(2129),
    [sym_get_dns_list] = ACTIONS(2129),
    [sym_set_dns_list] = ACTIONS(2129),
    [sym_get_ip_address] = ACTIONS(2129),
    [sym_set_ip_address] = ACTIONS(2129),
    [sym_get_url_list] = ACTIONS(2129),
    [sym_add_url_entry] = ACTIONS(2129),
    [sym_delete_url_entry] = ACTIONS(2129),
    [sym_abs_value] = ACTIONS(2129),
    [sym_max_value] = ACTIONS(2129),
    [sym_min_value] = ACTIONS(2129),
    [sym_exp_value] = ACTIONS(2129),
    [sym_log_value] = ACTIONS(2129),
    [sym_log10_value] = ACTIONS(2129),
    [sym_power_value] = ACTIONS(2129),
    [sym_sqrt_value] = ACTIONS(2129),
    [sym_raw_be] = ACTIONS(2129),
    [sym_raw_le] = ACTIONS(2129),
    [sym_internal_queue_size_set] = ACTIONS(2129),
    [sym_internal_queue_size_get] = ACTIONS(2129),
    [sym_internal_threshold_set] = ACTIONS(2129),
    [sym_internal_threshold_get] = ACTIONS(2129),
    [sym_rebuilt_event] = ACTIONS(2129),
    [sym_do_custom_event] = ACTIONS(2129),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2129),
    [sym_clkmgr_set_clk_source] = ACTIONS(2129),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2129),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2129),
    [sym_clkmgr_get_timezone] = ACTIONS(2129),
    [sym_clkmgr_set_timezone] = ACTIONS(2129),
    [sym_clkmgr_get_resync_period] = ACTIONS(2129),
    [sym_clkmgr_set_resync_period] = ACTIONS(2129),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2129),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2129),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2129),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2129),
    [sym_clkmgr_get_timeservers] = ACTIONS(2129),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2129),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2129),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2129),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2129),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2129),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2129),
    [sym_set_log_level] = ACTIONS(2129),
    [sym_get_log_level] = ACTIONS(2129),
    [sym_amx_log] = ACTIONS(2129),
    [sym_smtp_server_config_set] = ACTIONS(2129),
    [sym_smtp_server_config_get] = ACTIONS(2129),
    [sym_smtp_send] = ACTIONS(2129),
    [sym_validate_netlinx_account] = ACTIONS(2129),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2129),
    [sym_authenticate_certificate] = ACTIONS(2129),
    [sym_audit_netlinx_session_event] = ACTIONS(2129),
    [sym_audit_netlinx_generic_event] = ACTIONS(2129),
    [sym_get_available_flash_disk_space] = ACTIONS(2129),
    [sym_get_max_flash_disk_space] = ACTIONS(2129),
    [sym_data_create_feed] = ACTIONS(2129),
    [sym_data_delete_feed] = ACTIONS(2129),
    [sym_data_publish_feed] = ACTIONS(2129),
    [sym_data_get_published_feed] = ACTIONS(2129),
    [sym_data_add_record] = ACTIONS(2129),
    [sym_data_get_event_record] = ACTIONS(2129),
    [sym__wc_data_create_feed] = ACTIONS(2129),
    [sym__wc_data_add_record] = ACTIONS(2129),
    [sym__wc_data_get_event_record] = ACTIONS(2129),
    [sym_format] = ACTIONS(2129),
    [sym_astro_clock] = ACTIONS(2129),
    [sym_set_outdoor_temperature] = ACTIONS(2129),
    [sym_set_virtual_level_count] = ACTIONS(2129),
    [sym_set_virtual_channel_count] = ACTIONS(2129),
    [sym_set_virtual_port_count] = ACTIONS(2129),
    [sym__wc_encode_int] = ACTIONS(2129),
    [sym__wc_decode_int] = ACTIONS(2129),
    [sym__wc_int] = ACTIONS(2129),
    [sym__wc_to_ch_int] = ACTIONS(2129),
    [sym__ch_to_wc_int] = ACTIONS(2129),
    [sym__wc_find_string_int] = ACTIONS(2129),
    [sym__wc_left_string_int] = ACTIONS(2129),
    [sym__wc_lower_string_int] = ACTIONS(2129),
    [sym__wc_mid_string_int] = ACTIONS(2129),
    [sym__wc_remove_string_int] = ACTIONS(2129),
    [sym__wc_right_string_int] = ACTIONS(2129),
    [sym__wc_upper_string_int] = ACTIONS(2129),
    [sym__wc_compare_string_int] = ACTIONS(2129),
    [sym__wc_get_buffer_char_int] = ACTIONS(2129),
    [sym__wc_get_buffer_string_int] = ACTIONS(2129),
    [sym__wc_concat_string_int] = ACTIONS(2129),
    [sym__wc_file_open_int] = ACTIONS(2129),
    [sym__wc_file_close_int] = ACTIONS(2129),
    [sym__wc_file_read_int] = ACTIONS(2129),
    [sym__wc_file_read_line_int] = ACTIONS(2129),
    [sym__wc_file_write_int] = ACTIONS(2129),
    [sym__wc_file_write_line_int] = ACTIONS(2129),
    [aux_sym_preproc_include_token1] = ACTIONS(2129),
    [aux_sym_preproc_define_token1] = ACTIONS(2131),
    [aux_sym_preproc_warn_token1] = ACTIONS(2131),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2131),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2131),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2131),
    [anon_sym_LBRACK] = ACTIONS(2131),
    [anon_sym_LBRACE] = ACTIONS(2131),
    [anon_sym_RBRACE] = ACTIONS(2131),
    [anon_sym_LPAREN] = ACTIONS(2129),
    [anon_sym_BANG] = ACTIONS(2131),
    [anon_sym_TILDE] = ACTIONS(2131),
    [anon_sym_DASH] = ACTIONS(2129),
    [anon_sym_PLUS] = ACTIONS(2129),
    [anon_sym_DASH_DASH] = ACTIONS(2131),
    [anon_sym_PLUS_PLUS] = ACTIONS(2131),
    [anon_sym_DQUOTE] = ACTIONS(2131),
    [anon_sym_SQUOTE] = ACTIONS(2131),
    [sym_number_literal] = ACTIONS(2131),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(539)] = {
    [sym_identifier] = ACTIONS(2069),
    [sym_define_call_keyword] = ACTIONS(2069),
    [sym_define_function_keyword] = ACTIONS(2069),
    [sym_define_library_function_keyword] = ACTIONS(2069),
    [sym_define_module_keyword] = ACTIONS(2069),
    [sym_char_keyword] = ACTIONS(2069),
    [sym_widechar_keyword] = ACTIONS(2069),
    [sym_integer_keyword] = ACTIONS(2069),
    [sym_sinteger_keyword] = ACTIONS(2069),
    [sym_long_keyword] = ACTIONS(2069),
    [sym_slong_keyword] = ACTIONS(2069),
    [sym_float_keyword] = ACTIONS(2069),
    [sym_double_keyword] = ACTIONS(2069),
    [sym_variant_keyword] = ACTIONS(2069),
    [sym_variantarray_keyword] = ACTIONS(2069),
    [sym_local_var_keyword] = ACTIONS(2069),
    [sym_stack_var_keyword] = ACTIONS(2069),
    [sym_constant_keyword] = ACTIONS(2069),
    [sym_volatile_keyword] = ACTIONS(2069),
    [sym_non_volatile_keyword] = ACTIONS(2069),
    [sym_persistent_keyword] = ACTIONS(2069),
    [sym_dev_keyword] = ACTIONS(2069),
    [sym_devlev_keyword] = ACTIONS(2069),
    [sym_devchan_keyword] = ACTIONS(2069),
    [sym_if_keyword] = ACTIONS(2069),
    [sym_switch_keyword] = ACTIONS(2069),
    [sym_case_keyword] = ACTIONS(2069),
    [sym_default_keyword] = ACTIONS(2069),
    [sym_while_keyword] = ACTIONS(2069),
    [sym_for_keyword] = ACTIONS(2069),
    [sym_break_keyword] = ACTIONS(2069),
    [sym_continue_keyword] = ACTIONS(2069),
    [sym_return_keyword] = ACTIONS(2069),
    [sym_select_keyword] = ACTIONS(2069),
    [sym_active_keyword] = ACTIONS(2069),
    [sym_struct_keyword] = ACTIONS(2069),
    [sym_structure_keyword] = ACTIONS(2069),
    [aux_sym_on_keyword_token1] = ACTIONS(2069),
    [aux_sym_off_keyword_token1] = ACTIONS(2069),
    [sym_send_level_keyword] = ACTIONS(2069),
    [sym_send_string_keyword] = ACTIONS(2069),
    [sym_send_command_keyword] = ACTIONS(2069),
    [sym_clear_buffer_keyword] = ACTIONS(2069),
    [sym_create_buffer_keyword] = ACTIONS(2069),
    [sym_create_multi_buffer_keyword] = ACTIONS(2069),
    [sym_call_keyword] = ACTIONS(2069),
    [sym_system_call_keyword] = ACTIONS(2069),
    [sym_devchan_to_keyword] = ACTIONS(2069),
    [sym_devchan_min_to_keyword] = ACTIONS(2069),
    [sym_devchan_total_off_keyword] = ACTIONS(2069),
    [sym_devchan_pulse_keyword] = ACTIONS(2069),
    [sym_wait_keyword] = ACTIONS(2069),
    [sym_cancel_wait_keyword] = ACTIONS(2069),
    [sym_wait_until_keyword] = ACTIONS(2069),
    [sym_cancel_wait_until_keyword] = ACTIONS(2069),
    [sym_cancel_all_wait_keyword] = ACTIONS(2069),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2069),
    [sym_not] = ACTIONS(2069),
    [sym_bnot] = ACTIONS(2069),
    [sym___file__] = ACTIONS(2069),
    [sym___date__] = ACTIONS(2069),
    [sym___ldate__] = ACTIONS(2069),
    [sym___time__] = ACTIONS(2069),
    [sym___line__] = ACTIONS(2069),
    [sym___name__] = ACTIONS(2069),
    [sym___netlinx__] = ACTIONS(2069),
    [sym_netlinx_axi_version] = ACTIONS(2069),
    [sym_url_flg_tcp] = ACTIONS(2069),
    [sym_url_flg_accinfopresent] = ACTIONS(2069),
    [sym_url_flg_temp] = ACTIONS(2069),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2069),
    [sym_url_flg_stat_mask] = ACTIONS(2069),
    [sym_url_flg_stat_lookup] = ACTIONS(2069),
    [sym_url_flg_stat_connecting] = ACTIONS(2069),
    [sym_url_flg_stat_waiting] = ACTIONS(2069),
    [sym_url_flg_stat_connected] = ACTIONS(2069),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2069),
    [sym_first_virtual_device] = ACTIONS(2069),
    [sym_first_local_port] = ACTIONS(2069),
    [sym_dynamic_virtual_device] = ACTIONS(2069),
    [sym_timeline_once] = ACTIONS(2069),
    [sym_timeline_repeat] = ACTIONS(2069),
    [sym_timeline_absolute] = ACTIONS(2069),
    [sym_timeline_relative] = ACTIONS(2069),
    [sym_do_push_timed_infinite] = ACTIONS(2069),
    [sym_source_type_no_address] = ACTIONS(2069),
    [sym_source_type_neuron_id] = ACTIONS(2069),
    [sym_source_type_ip_address] = ACTIONS(2069),
    [sym_source_type_axlink] = ACTIONS(2069),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2069),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2069),
    [sym_source_type_ip_socket_address] = ACTIONS(2069),
    [sym_source_type_rs232] = ACTIONS(2069),
    [sym_source_type_internal] = ACTIONS(2069),
    [sym_source_type_mac_address] = ACTIONS(2069),
    [sym_source_type_ipv4_port] = ACTIONS(2069),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2069),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2069),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2069),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2069),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2069),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2069),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2069),
    [sym_true] = ACTIONS(2069),
    [sym_false] = ACTIONS(2069),
    [sym_file_read_only] = ACTIONS(2069),
    [sym_file_rw_new] = ACTIONS(2069),
    [sym_file_rw_append] = ACTIONS(2069),
    [sym_ip_tcp] = ACTIONS(2069),
    [sym_ip_udp] = ACTIONS(2069),
    [sym_ip_udp_2way] = ACTIONS(2069),
    [sym_xml_encode_types] = ACTIONS(2069),
    [sym_xml_encode_char_as_list] = ACTIONS(2069),
    [sym_xml_encode_le] = ACTIONS(2069),
    [sym_xml_decode_types] = ACTIONS(2069),
    [sym_xml_decode_no_preserve] = ACTIONS(2069),
    [sym_duet_dev_type_display_device] = ACTIONS(2069),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2069),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2069),
    [sym_duet_dev_type_io_device] = ACTIONS(2069),
    [sym_duet_dev_type_relay_device] = ACTIONS(2069),
    [sym_duet_dev_type_ups] = ACTIONS(2069),
    [sym_duet_dev_type_amplifier] = ACTIONS(2069),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2069),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2069),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2069),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2069),
    [sym_duet_dev_type_camera] = ACTIONS(2069),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2069),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2069),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2069),
    [sym_duet_dev_type_dss] = ACTIONS(2069),
    [sym_duet_dev_type_dvr] = ACTIONS(2069),
    [sym_duet_dev_type_disc_device] = ACTIONS(2069),
    [sym_duet_dev_type_document_camera] = ACTIONS(2069),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2069),
    [sym_duet_dev_type_hvac] = ACTIONS(2069),
    [sym_duet_dev_type_keypad] = ACTIONS(2069),
    [sym_duet_dev_type_light] = ACTIONS(2069),
    [sym_duet_dev_type_monitor] = ACTIONS(2069),
    [sym_duet_dev_type_motor] = ACTIONS(2069),
    [sym_duet_dev_type_multi_window] = ACTIONS(2069),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2069),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2069),
    [sym_duet_dev_type_receiver] = ACTIONS(2069),
    [sym_duet_dev_type_security_system] = ACTIONS(2069),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2069),
    [sym_duet_dev_type_settop_box] = ACTIONS(2069),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2069),
    [sym_duet_dev_type_switcher] = ACTIONS(2069),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2069),
    [sym_duet_dev_type_tv] = ACTIONS(2069),
    [sym_duet_dev_type_utility] = ACTIONS(2069),
    [sym_duet_dev_type_vcr] = ACTIONS(2069),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2069),
    [sym_duet_dev_type_video_processor] = ACTIONS(2069),
    [sym_duet_dev_type_video_projector] = ACTIONS(2069),
    [sym_duet_dev_type_video_wall] = ACTIONS(2069),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2069),
    [sym_duet_dev_type_weather] = ACTIONS(2069),
    [sym_duet_dev_not_polled] = ACTIONS(2069),
    [sym_duet_dev_polled] = ACTIONS(2069),
    [sym_clkmgr_mode_network] = ACTIONS(2069),
    [sym_clkmgr_mode_standalone] = ACTIONS(2069),
    [sym_normal_standby] = ACTIONS(2069),
    [sym_normal_wake] = ACTIONS(2069),
    [sym_ip_multicast_ttl_option] = ACTIONS(2069),
    [sym_ip_ttl_subnet] = ACTIONS(2069),
    [sym_ip_ttl_site] = ACTIONS(2069),
    [sym_ip_ttl_region] = ACTIONS(2069),
    [sym_ip_ttl_continent] = ACTIONS(2069),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2069),
    [sym_ip_nodelay_on] = ACTIONS(2069),
    [sym_ip_nodelay_off] = ACTIONS(2069),
    [sym_amx_error] = ACTIONS(2069),
    [sym_amx_warning] = ACTIONS(2069),
    [sym_amx_info] = ACTIONS(2069),
    [sym_amx_debug] = ACTIONS(2069),
    [sym_netlinx_login_success] = ACTIONS(2069),
    [sym_netlinx_login_fail] = ACTIONS(2069),
    [sym_netlinx_logout] = ACTIONS(2069),
    [sym_null_str] = ACTIONS(2069),
    [sym_smtp_address] = ACTIONS(2069),
    [sym_smtp_port_number] = ACTIONS(2069),
    [sym_smtp_username] = ACTIONS(2069),
    [sym_smtp_password] = ACTIONS(2069),
    [sym_smtp_require_tls] = ACTIONS(2069),
    [sym_smtp_from] = ACTIONS(2069),
    [sym_smtp_tls_true] = ACTIONS(2069),
    [sym_smtp_tls_false] = ACTIONS(2069),
    [sym_listview_on_row_select_event] = ACTIONS(2069),
    [sym_data_structure_unknown] = ACTIONS(2069),
    [sym_data_structure_datafeed] = ACTIONS(2069),
    [sym_data_structure_datarecordset] = ACTIONS(2069),
    [sym_data_structure_datarecord] = ACTIONS(2069),
    [sym_data_structure_datafield] = ACTIONS(2069),
    [sym_data_max_content_field_count] = ACTIONS(2069),
    [sym_data_max_metadata_field_count] = ACTIONS(2069),
    [sym_data_max_id_length] = ACTIONS(2069),
    [sym_data_max_name_length] = ACTIONS(2069),
    [sym_data_max_description_length] = ACTIONS(2069),
    [sym_data_max_type_length] = ACTIONS(2069),
    [sym_data_max_format_length] = ACTIONS(2069),
    [sym_data_max_source_length] = ACTIONS(2069),
    [sym_data_max_value_length] = ACTIONS(2069),
    [sym_data_max_label_length] = ACTIONS(2069),
    [sym_data_max_url_length] = ACTIONS(2069),
    [sym_max_last_login_info_length] = ACTIONS(2069),
    [sym_data_type_unknown] = ACTIONS(2069),
    [sym_data_type_string] = ACTIONS(2069),
    [sym_data_type_datetime] = ACTIONS(2069),
    [sym_data_type_date] = ACTIONS(2069),
    [sym_data_type_time] = ACTIONS(2069),
    [sym_data_type_image] = ACTIONS(2069),
    [sym_data_format_url] = ACTIONS(2069),
    [sym_data_format_phone] = ACTIONS(2069),
    [sym_data_format_email] = ACTIONS(2069),
    [sym_data_format_iso8601] = ACTIONS(2069),
    [sym_auditlog_priv] = ACTIONS(2069),
    [sym_configuration_priv] = ACTIONS(2069),
    [sym_deviceconfig_priv] = ACTIONS(2069),
    [sym_ftp_priv] = ACTIONS(2069),
    [sym_http_priv] = ACTIONS(2069),
    [sym_networkconfig_priv] = ACTIONS(2069),
    [sym_programport_priv] = ACTIONS(2069),
    [sym_remoteui_priv] = ACTIONS(2069),
    [sym_securitycontrol_priv] = ACTIONS(2069),
    [sym_softwaremanagement_priv] = ACTIONS(2069),
    [sym_terminal_priv] = ACTIONS(2069),
    [sym_tpadmin_priv] = ACTIONS(2069),
    [sym_user1_priv] = ACTIONS(2069),
    [sym_user2_priv] = ACTIONS(2069),
    [sym_user3_priv] = ACTIONS(2069),
    [sym_user4_priv] = ACTIONS(2069),
    [sym_usermanagement_priv] = ACTIONS(2069),
    [sym_valid_account] = ACTIONS(2069),
    [sym_err_name_invalid] = ACTIONS(2069),
    [sym_err_password_invalid] = ACTIONS(2069),
    [sym_err_invalid_account] = ACTIONS(2069),
    [sym_err_no_authorization] = ACTIONS(2069),
    [sym_err_invalid_parameter] = ACTIONS(2069),
    [sym_err_user_locked_out] = ACTIONS(2069),
    [sym_err_user_account_expired] = ACTIONS(2069),
    [sym_err_general_error] = ACTIONS(2069),
    [sym_err_authentication_error] = ACTIONS(2069),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2069),
    [sym_tls_validate_certificate] = ACTIONS(2069),
    [sym_tdata] = ACTIONS(2069),
    [sym_tchannel] = ACTIONS(2069),
    [sym_tlevel] = ACTIONS(2069),
    [sym_tbutton] = ACTIONS(2069),
    [sym_ttimeline] = ACTIONS(2069),
    [sym_tcustom] = ACTIONS(2069),
    [sym_url_struct] = ACTIONS(2069),
    [sym_dns_struct] = ACTIONS(2069),
    [sym_ip_address_struct] = ACTIONS(2069),
    [sym_dev_info_struct] = ACTIONS(2069),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2069),
    [sym_data_feed] = ACTIONS(2069),
    [sym_data_field] = ACTIONS(2069),
    [sym_data_record] = ACTIONS(2069),
    [sym_wc_data_feed] = ACTIONS(2069),
    [sym_wc_data_field] = ACTIONS(2069),
    [sym_wc_data_record] = ACTIONS(2069),
    [sym_last_login_info] = ACTIONS(2069),
    [sym____reserved___] = ACTIONS(2069),
    [sym_date] = ACTIONS(2069),
    [sym_day] = ACTIONS(2069),
    [sym_time] = ACTIONS(2069),
    [sym_ldate] = ACTIONS(2069),
    [sym_push_device] = ACTIONS(2069),
    [sym_push_channel] = ACTIONS(2069),
    [sym_push_devchan] = ACTIONS(2069),
    [sym_release_device] = ACTIONS(2069),
    [sym_release_channel] = ACTIONS(2069),
    [sym_release_devchan] = ACTIONS(2069),
    [sym_master_slot] = ACTIONS(2069),
    [sym_get_pulse_time] = ACTIONS(2069),
    [sym_get_timer] = ACTIONS(2069),
    [sym_master_sn] = ACTIONS(2069),
    [sym_system_number] = ACTIONS(2069),
    [sym_button] = ACTIONS(2069),
    [sym_dv_channel] = ACTIONS(2069),
    [sym_data] = ACTIONS(2069),
    [sym_level] = ACTIONS(2069),
    [sym_timeline] = ACTIONS(2069),
    [sym_channel] = ACTIONS(2069),
    [sym_custom] = ACTIONS(2069),
    [sym_atoi] = ACTIONS(2069),
    [sym_atol] = ACTIONS(2069),
    [sym_atof] = ACTIONS(2069),
    [sym_itoa] = ACTIONS(2069),
    [sym_itohex] = ACTIONS(2069),
    [sym_hextoi] = ACTIONS(2069),
    [sym_ftoa] = ACTIONS(2069),
    [sym_find_string] = ACTIONS(2069),
    [sym_left_string] = ACTIONS(2069),
    [sym_right_string] = ACTIONS(2069),
    [sym_mid_string] = ACTIONS(2069),
    [sym_remove_string] = ACTIONS(2069),
    [sym_get_buffer_string] = ACTIONS(2069),
    [sym_get_buffer_char] = ACTIONS(2069),
    [sym_get_multi_buffer_string] = ACTIONS(2069),
    [sym_length_string] = ACTIONS(2069),
    [sym_max_length_string] = ACTIONS(2069),
    [sym_set_length_string] = ACTIONS(2069),
    [sym_compare_string] = ACTIONS(2069),
    [sym_upper_string] = ACTIONS(2069),
    [sym_lower_string] = ACTIONS(2069),
    [sym_get_last] = ACTIONS(2069),
    [sym_date_to_day] = ACTIONS(2069),
    [sym_date_to_month] = ACTIONS(2069),
    [sym_date_to_year] = ACTIONS(2069),
    [sym_time_to_hour] = ACTIONS(2069),
    [sym_time_to_minute] = ACTIONS(2069),
    [sym_time_to_second] = ACTIONS(2069),
    [sym_day_of_week] = ACTIONS(2069),
    [sym_set_pulse_time] = ACTIONS(2069),
    [sym_set_timer] = ACTIONS(2069),
    [sym_file_open] = ACTIONS(2069),
    [sym_file_close] = ACTIONS(2069),
    [sym_file_read] = ACTIONS(2069),
    [sym_file_read_line] = ACTIONS(2069),
    [sym_file_write] = ACTIONS(2069),
    [sym_file_write_line] = ACTIONS(2069),
    [sym_file_copy] = ACTIONS(2069),
    [sym_file_delete] = ACTIONS(2069),
    [sym_file_rename] = ACTIONS(2069),
    [sym_file_dir] = ACTIONS(2069),
    [sym_file_getdir] = ACTIONS(2069),
    [sym_file_setdir] = ACTIONS(2069),
    [sym_file_create_dir] = ACTIONS(2069),
    [sym_file_remove_dir] = ACTIONS(2069),
    [sym_file_seek] = ACTIONS(2069),
    [sym_length_array] = ACTIONS(2069),
    [sym_max_length_array] = ACTIONS(2069),
    [sym_set_length_array] = ACTIONS(2069),
    [sym_ip_client_open] = ACTIONS(2069),
    [sym_ip_client_close] = ACTIONS(2069),
    [sym_ip_server_open] = ACTIONS(2069),
    [sym_ip_server_close] = ACTIONS(2069),
    [sym_ip_bound_client_open] = ACTIONS(2069),
    [sym_ip_mc_server_open] = ACTIONS(2069),
    [sym_ip_set_option] = ACTIONS(2069),
    [sym_ssh_client_open] = ACTIONS(2069),
    [sym_ssh_client_close] = ACTIONS(2069),
    [sym_tls_client_open] = ACTIONS(2069),
    [sym_tls_client_close] = ACTIONS(2069),
    [sym_do_push] = ACTIONS(2069),
    [sym_do_push_timed] = ACTIONS(2069),
    [sym_do_release] = ACTIONS(2069),
    [sym_device_id] = ACTIONS(2069),
    [sym_device_info] = ACTIONS(2069),
    [sym_device_id_string] = ACTIONS(2069),
    [sym_device_standby] = ACTIONS(2069),
    [sym_device_wake] = ACTIONS(2069),
    [sym_timeline_create] = ACTIONS(2069),
    [sym_timeline_kill] = ACTIONS(2069),
    [sym_timeline_reload] = ACTIONS(2069),
    [sym_timeline_pause] = ACTIONS(2069),
    [sym_timeline_restart] = ACTIONS(2069),
    [sym_timeline_set] = ACTIONS(2069),
    [sym_timeline_get] = ACTIONS(2069),
    [sym_timeline_active] = ACTIONS(2069),
    [sym_timeline_dynamic_id] = ACTIONS(2069),
    [sym_variable_to_string] = ACTIONS(2069),
    [sym_string_to_variable] = ACTIONS(2069),
    [sym_length_variable_to_string] = ACTIONS(2069),
    [sym_type_cast] = ACTIONS(2069),
    [sym_variable_to_xml] = ACTIONS(2069),
    [sym_xml_to_variable] = ACTIONS(2069),
    [sym_length_variable_to_xml] = ACTIONS(2069),
    [sym_redirect_string] = ACTIONS(2069),
    [sym_random_number] = ACTIONS(2069),
    [sym_get_unique_id] = ACTIONS(2069),
    [sym_get_system_number] = ACTIONS(2069),
    [sym_set_system_number] = ACTIONS(2069),
    [sym_get_serial_number] = ACTIONS(2069),
    [sym_get_master_build] = ACTIONS(2069),
    [sym_set_validation_code] = ACTIONS(2069),
    [sym_reboot] = ACTIONS(2069),
    [sym_load_duet_module] = ACTIONS(2069),
    [sym_unload_duet_module] = ACTIONS(2069),
    [sym_duet_mem_size_set] = ACTIONS(2069),
    [sym_duet_mem_size_get] = ACTIONS(2069),
    [sym_dynamic_application_device] = ACTIONS(2069),
    [sym_dynamic_polled_port] = ACTIONS(2069),
    [sym_static_port_binding] = ACTIONS(2069),
    [sym_static_ip_binding] = ACTIONS(2069),
    [sym_get_dns_list] = ACTIONS(2069),
    [sym_set_dns_list] = ACTIONS(2069),
    [sym_get_ip_address] = ACTIONS(2069),
    [sym_set_ip_address] = ACTIONS(2069),
    [sym_get_url_list] = ACTIONS(2069),
    [sym_add_url_entry] = ACTIONS(2069),
    [sym_delete_url_entry] = ACTIONS(2069),
    [sym_abs_value] = ACTIONS(2069),
    [sym_max_value] = ACTIONS(2069),
    [sym_min_value] = ACTIONS(2069),
    [sym_exp_value] = ACTIONS(2069),
    [sym_log_value] = ACTIONS(2069),
    [sym_log10_value] = ACTIONS(2069),
    [sym_power_value] = ACTIONS(2069),
    [sym_sqrt_value] = ACTIONS(2069),
    [sym_raw_be] = ACTIONS(2069),
    [sym_raw_le] = ACTIONS(2069),
    [sym_internal_queue_size_set] = ACTIONS(2069),
    [sym_internal_queue_size_get] = ACTIONS(2069),
    [sym_internal_threshold_set] = ACTIONS(2069),
    [sym_internal_threshold_get] = ACTIONS(2069),
    [sym_rebuilt_event] = ACTIONS(2069),
    [sym_do_custom_event] = ACTIONS(2069),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2069),
    [sym_clkmgr_set_clk_source] = ACTIONS(2069),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2069),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2069),
    [sym_clkmgr_get_timezone] = ACTIONS(2069),
    [sym_clkmgr_set_timezone] = ACTIONS(2069),
    [sym_clkmgr_get_resync_period] = ACTIONS(2069),
    [sym_clkmgr_set_resync_period] = ACTIONS(2069),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2069),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2069),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2069),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2069),
    [sym_clkmgr_get_timeservers] = ACTIONS(2069),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2069),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2069),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2069),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2069),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2069),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2069),
    [sym_set_log_level] = ACTIONS(2069),
    [sym_get_log_level] = ACTIONS(2069),
    [sym_amx_log] = ACTIONS(2069),
    [sym_smtp_server_config_set] = ACTIONS(2069),
    [sym_smtp_server_config_get] = ACTIONS(2069),
    [sym_smtp_send] = ACTIONS(2069),
    [sym_validate_netlinx_account] = ACTIONS(2069),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2069),
    [sym_authenticate_certificate] = ACTIONS(2069),
    [sym_audit_netlinx_session_event] = ACTIONS(2069),
    [sym_audit_netlinx_generic_event] = ACTIONS(2069),
    [sym_get_available_flash_disk_space] = ACTIONS(2069),
    [sym_get_max_flash_disk_space] = ACTIONS(2069),
    [sym_data_create_feed] = ACTIONS(2069),
    [sym_data_delete_feed] = ACTIONS(2069),
    [sym_data_publish_feed] = ACTIONS(2069),
    [sym_data_get_published_feed] = ACTIONS(2069),
    [sym_data_add_record] = ACTIONS(2069),
    [sym_data_get_event_record] = ACTIONS(2069),
    [sym__wc_data_create_feed] = ACTIONS(2069),
    [sym__wc_data_add_record] = ACTIONS(2069),
    [sym__wc_data_get_event_record] = ACTIONS(2069),
    [sym_format] = ACTIONS(2069),
    [sym_astro_clock] = ACTIONS(2069),
    [sym_set_outdoor_temperature] = ACTIONS(2069),
    [sym_set_virtual_level_count] = ACTIONS(2069),
    [sym_set_virtual_channel_count] = ACTIONS(2069),
    [sym_set_virtual_port_count] = ACTIONS(2069),
    [sym__wc_encode_int] = ACTIONS(2069),
    [sym__wc_decode_int] = ACTIONS(2069),
    [sym__wc_int] = ACTIONS(2069),
    [sym__wc_to_ch_int] = ACTIONS(2069),
    [sym__ch_to_wc_int] = ACTIONS(2069),
    [sym__wc_find_string_int] = ACTIONS(2069),
    [sym__wc_left_string_int] = ACTIONS(2069),
    [sym__wc_lower_string_int] = ACTIONS(2069),
    [sym__wc_mid_string_int] = ACTIONS(2069),
    [sym__wc_remove_string_int] = ACTIONS(2069),
    [sym__wc_right_string_int] = ACTIONS(2069),
    [sym__wc_upper_string_int] = ACTIONS(2069),
    [sym__wc_compare_string_int] = ACTIONS(2069),
    [sym__wc_get_buffer_char_int] = ACTIONS(2069),
    [sym__wc_get_buffer_string_int] = ACTIONS(2069),
    [sym__wc_concat_string_int] = ACTIONS(2069),
    [sym__wc_file_open_int] = ACTIONS(2069),
    [sym__wc_file_close_int] = ACTIONS(2069),
    [sym__wc_file_read_int] = ACTIONS(2069),
    [sym__wc_file_read_line_int] = ACTIONS(2069),
    [sym__wc_file_write_int] = ACTIONS(2069),
    [sym__wc_file_write_line_int] = ACTIONS(2069),
    [aux_sym_preproc_include_token1] = ACTIONS(2069),
    [aux_sym_preproc_define_token1] = ACTIONS(2071),
    [aux_sym_preproc_warn_token1] = ACTIONS(2071),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2071),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2071),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2071),
    [anon_sym_LBRACK] = ACTIONS(2071),
    [anon_sym_LBRACE] = ACTIONS(2071),
    [anon_sym_RBRACE] = ACTIONS(2071),
    [anon_sym_LPAREN] = ACTIONS(2069),
    [anon_sym_BANG] = ACTIONS(2071),
    [anon_sym_TILDE] = ACTIONS(2071),
    [anon_sym_DASH] = ACTIONS(2069),
    [anon_sym_PLUS] = ACTIONS(2069),
    [anon_sym_DASH_DASH] = ACTIONS(2071),
    [anon_sym_PLUS_PLUS] = ACTIONS(2071),
    [anon_sym_DQUOTE] = ACTIONS(2071),
    [anon_sym_SQUOTE] = ACTIONS(2071),
    [sym_number_literal] = ACTIONS(2071),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(540)] = {
    [sym_identifier] = ACTIONS(2037),
    [sym_define_call_keyword] = ACTIONS(2037),
    [sym_define_function_keyword] = ACTIONS(2037),
    [sym_define_library_function_keyword] = ACTIONS(2037),
    [sym_define_module_keyword] = ACTIONS(2037),
    [sym_char_keyword] = ACTIONS(2037),
    [sym_widechar_keyword] = ACTIONS(2037),
    [sym_integer_keyword] = ACTIONS(2037),
    [sym_sinteger_keyword] = ACTIONS(2037),
    [sym_long_keyword] = ACTIONS(2037),
    [sym_slong_keyword] = ACTIONS(2037),
    [sym_float_keyword] = ACTIONS(2037),
    [sym_double_keyword] = ACTIONS(2037),
    [sym_variant_keyword] = ACTIONS(2037),
    [sym_variantarray_keyword] = ACTIONS(2037),
    [sym_local_var_keyword] = ACTIONS(2037),
    [sym_stack_var_keyword] = ACTIONS(2037),
    [sym_constant_keyword] = ACTIONS(2037),
    [sym_volatile_keyword] = ACTIONS(2037),
    [sym_non_volatile_keyword] = ACTIONS(2037),
    [sym_persistent_keyword] = ACTIONS(2037),
    [sym_dev_keyword] = ACTIONS(2037),
    [sym_devlev_keyword] = ACTIONS(2037),
    [sym_devchan_keyword] = ACTIONS(2037),
    [sym_if_keyword] = ACTIONS(2037),
    [sym_switch_keyword] = ACTIONS(2037),
    [sym_case_keyword] = ACTIONS(2037),
    [sym_default_keyword] = ACTIONS(2037),
    [sym_while_keyword] = ACTIONS(2037),
    [sym_for_keyword] = ACTIONS(2037),
    [sym_break_keyword] = ACTIONS(2037),
    [sym_continue_keyword] = ACTIONS(2037),
    [sym_return_keyword] = ACTIONS(2037),
    [sym_select_keyword] = ACTIONS(2037),
    [sym_active_keyword] = ACTIONS(2037),
    [sym_struct_keyword] = ACTIONS(2037),
    [sym_structure_keyword] = ACTIONS(2037),
    [aux_sym_on_keyword_token1] = ACTIONS(2037),
    [aux_sym_off_keyword_token1] = ACTIONS(2037),
    [sym_send_level_keyword] = ACTIONS(2037),
    [sym_send_string_keyword] = ACTIONS(2037),
    [sym_send_command_keyword] = ACTIONS(2037),
    [sym_clear_buffer_keyword] = ACTIONS(2037),
    [sym_create_buffer_keyword] = ACTIONS(2037),
    [sym_create_multi_buffer_keyword] = ACTIONS(2037),
    [sym_call_keyword] = ACTIONS(2037),
    [sym_system_call_keyword] = ACTIONS(2037),
    [sym_devchan_to_keyword] = ACTIONS(2037),
    [sym_devchan_min_to_keyword] = ACTIONS(2037),
    [sym_devchan_total_off_keyword] = ACTIONS(2037),
    [sym_devchan_pulse_keyword] = ACTIONS(2037),
    [sym_wait_keyword] = ACTIONS(2037),
    [sym_cancel_wait_keyword] = ACTIONS(2037),
    [sym_wait_until_keyword] = ACTIONS(2037),
    [sym_cancel_wait_until_keyword] = ACTIONS(2037),
    [sym_cancel_all_wait_keyword] = ACTIONS(2037),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2037),
    [sym_not] = ACTIONS(2037),
    [sym_bnot] = ACTIONS(2037),
    [sym___file__] = ACTIONS(2037),
    [sym___date__] = ACTIONS(2037),
    [sym___ldate__] = ACTIONS(2037),
    [sym___time__] = ACTIONS(2037),
    [sym___line__] = ACTIONS(2037),
    [sym___name__] = ACTIONS(2037),
    [sym___netlinx__] = ACTIONS(2037),
    [sym_netlinx_axi_version] = ACTIONS(2037),
    [sym_url_flg_tcp] = ACTIONS(2037),
    [sym_url_flg_accinfopresent] = ACTIONS(2037),
    [sym_url_flg_temp] = ACTIONS(2037),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2037),
    [sym_url_flg_stat_mask] = ACTIONS(2037),
    [sym_url_flg_stat_lookup] = ACTIONS(2037),
    [sym_url_flg_stat_connecting] = ACTIONS(2037),
    [sym_url_flg_stat_waiting] = ACTIONS(2037),
    [sym_url_flg_stat_connected] = ACTIONS(2037),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2037),
    [sym_first_virtual_device] = ACTIONS(2037),
    [sym_first_local_port] = ACTIONS(2037),
    [sym_dynamic_virtual_device] = ACTIONS(2037),
    [sym_timeline_once] = ACTIONS(2037),
    [sym_timeline_repeat] = ACTIONS(2037),
    [sym_timeline_absolute] = ACTIONS(2037),
    [sym_timeline_relative] = ACTIONS(2037),
    [sym_do_push_timed_infinite] = ACTIONS(2037),
    [sym_source_type_no_address] = ACTIONS(2037),
    [sym_source_type_neuron_id] = ACTIONS(2037),
    [sym_source_type_ip_address] = ACTIONS(2037),
    [sym_source_type_axlink] = ACTIONS(2037),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2037),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2037),
    [sym_source_type_ip_socket_address] = ACTIONS(2037),
    [sym_source_type_rs232] = ACTIONS(2037),
    [sym_source_type_internal] = ACTIONS(2037),
    [sym_source_type_mac_address] = ACTIONS(2037),
    [sym_source_type_ipv4_port] = ACTIONS(2037),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2037),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2037),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2037),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2037),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2037),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2037),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2037),
    [sym_true] = ACTIONS(2037),
    [sym_false] = ACTIONS(2037),
    [sym_file_read_only] = ACTIONS(2037),
    [sym_file_rw_new] = ACTIONS(2037),
    [sym_file_rw_append] = ACTIONS(2037),
    [sym_ip_tcp] = ACTIONS(2037),
    [sym_ip_udp] = ACTIONS(2037),
    [sym_ip_udp_2way] = ACTIONS(2037),
    [sym_xml_encode_types] = ACTIONS(2037),
    [sym_xml_encode_char_as_list] = ACTIONS(2037),
    [sym_xml_encode_le] = ACTIONS(2037),
    [sym_xml_decode_types] = ACTIONS(2037),
    [sym_xml_decode_no_preserve] = ACTIONS(2037),
    [sym_duet_dev_type_display_device] = ACTIONS(2037),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2037),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2037),
    [sym_duet_dev_type_io_device] = ACTIONS(2037),
    [sym_duet_dev_type_relay_device] = ACTIONS(2037),
    [sym_duet_dev_type_ups] = ACTIONS(2037),
    [sym_duet_dev_type_amplifier] = ACTIONS(2037),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2037),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2037),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2037),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2037),
    [sym_duet_dev_type_camera] = ACTIONS(2037),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2037),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2037),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2037),
    [sym_duet_dev_type_dss] = ACTIONS(2037),
    [sym_duet_dev_type_dvr] = ACTIONS(2037),
    [sym_duet_dev_type_disc_device] = ACTIONS(2037),
    [sym_duet_dev_type_document_camera] = ACTIONS(2037),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2037),
    [sym_duet_dev_type_hvac] = ACTIONS(2037),
    [sym_duet_dev_type_keypad] = ACTIONS(2037),
    [sym_duet_dev_type_light] = ACTIONS(2037),
    [sym_duet_dev_type_monitor] = ACTIONS(2037),
    [sym_duet_dev_type_motor] = ACTIONS(2037),
    [sym_duet_dev_type_multi_window] = ACTIONS(2037),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2037),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2037),
    [sym_duet_dev_type_receiver] = ACTIONS(2037),
    [sym_duet_dev_type_security_system] = ACTIONS(2037),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2037),
    [sym_duet_dev_type_settop_box] = ACTIONS(2037),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2037),
    [sym_duet_dev_type_switcher] = ACTIONS(2037),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2037),
    [sym_duet_dev_type_tv] = ACTIONS(2037),
    [sym_duet_dev_type_utility] = ACTIONS(2037),
    [sym_duet_dev_type_vcr] = ACTIONS(2037),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2037),
    [sym_duet_dev_type_video_processor] = ACTIONS(2037),
    [sym_duet_dev_type_video_projector] = ACTIONS(2037),
    [sym_duet_dev_type_video_wall] = ACTIONS(2037),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2037),
    [sym_duet_dev_type_weather] = ACTIONS(2037),
    [sym_duet_dev_not_polled] = ACTIONS(2037),
    [sym_duet_dev_polled] = ACTIONS(2037),
    [sym_clkmgr_mode_network] = ACTIONS(2037),
    [sym_clkmgr_mode_standalone] = ACTIONS(2037),
    [sym_normal_standby] = ACTIONS(2037),
    [sym_normal_wake] = ACTIONS(2037),
    [sym_ip_multicast_ttl_option] = ACTIONS(2037),
    [sym_ip_ttl_subnet] = ACTIONS(2037),
    [sym_ip_ttl_site] = ACTIONS(2037),
    [sym_ip_ttl_region] = ACTIONS(2037),
    [sym_ip_ttl_continent] = ACTIONS(2037),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2037),
    [sym_ip_nodelay_on] = ACTIONS(2037),
    [sym_ip_nodelay_off] = ACTIONS(2037),
    [sym_amx_error] = ACTIONS(2037),
    [sym_amx_warning] = ACTIONS(2037),
    [sym_amx_info] = ACTIONS(2037),
    [sym_amx_debug] = ACTIONS(2037),
    [sym_netlinx_login_success] = ACTIONS(2037),
    [sym_netlinx_login_fail] = ACTIONS(2037),
    [sym_netlinx_logout] = ACTIONS(2037),
    [sym_null_str] = ACTIONS(2037),
    [sym_smtp_address] = ACTIONS(2037),
    [sym_smtp_port_number] = ACTIONS(2037),
    [sym_smtp_username] = ACTIONS(2037),
    [sym_smtp_password] = ACTIONS(2037),
    [sym_smtp_require_tls] = ACTIONS(2037),
    [sym_smtp_from] = ACTIONS(2037),
    [sym_smtp_tls_true] = ACTIONS(2037),
    [sym_smtp_tls_false] = ACTIONS(2037),
    [sym_listview_on_row_select_event] = ACTIONS(2037),
    [sym_data_structure_unknown] = ACTIONS(2037),
    [sym_data_structure_datafeed] = ACTIONS(2037),
    [sym_data_structure_datarecordset] = ACTIONS(2037),
    [sym_data_structure_datarecord] = ACTIONS(2037),
    [sym_data_structure_datafield] = ACTIONS(2037),
    [sym_data_max_content_field_count] = ACTIONS(2037),
    [sym_data_max_metadata_field_count] = ACTIONS(2037),
    [sym_data_max_id_length] = ACTIONS(2037),
    [sym_data_max_name_length] = ACTIONS(2037),
    [sym_data_max_description_length] = ACTIONS(2037),
    [sym_data_max_type_length] = ACTIONS(2037),
    [sym_data_max_format_length] = ACTIONS(2037),
    [sym_data_max_source_length] = ACTIONS(2037),
    [sym_data_max_value_length] = ACTIONS(2037),
    [sym_data_max_label_length] = ACTIONS(2037),
    [sym_data_max_url_length] = ACTIONS(2037),
    [sym_max_last_login_info_length] = ACTIONS(2037),
    [sym_data_type_unknown] = ACTIONS(2037),
    [sym_data_type_string] = ACTIONS(2037),
    [sym_data_type_datetime] = ACTIONS(2037),
    [sym_data_type_date] = ACTIONS(2037),
    [sym_data_type_time] = ACTIONS(2037),
    [sym_data_type_image] = ACTIONS(2037),
    [sym_data_format_url] = ACTIONS(2037),
    [sym_data_format_phone] = ACTIONS(2037),
    [sym_data_format_email] = ACTIONS(2037),
    [sym_data_format_iso8601] = ACTIONS(2037),
    [sym_auditlog_priv] = ACTIONS(2037),
    [sym_configuration_priv] = ACTIONS(2037),
    [sym_deviceconfig_priv] = ACTIONS(2037),
    [sym_ftp_priv] = ACTIONS(2037),
    [sym_http_priv] = ACTIONS(2037),
    [sym_networkconfig_priv] = ACTIONS(2037),
    [sym_programport_priv] = ACTIONS(2037),
    [sym_remoteui_priv] = ACTIONS(2037),
    [sym_securitycontrol_priv] = ACTIONS(2037),
    [sym_softwaremanagement_priv] = ACTIONS(2037),
    [sym_terminal_priv] = ACTIONS(2037),
    [sym_tpadmin_priv] = ACTIONS(2037),
    [sym_user1_priv] = ACTIONS(2037),
    [sym_user2_priv] = ACTIONS(2037),
    [sym_user3_priv] = ACTIONS(2037),
    [sym_user4_priv] = ACTIONS(2037),
    [sym_usermanagement_priv] = ACTIONS(2037),
    [sym_valid_account] = ACTIONS(2037),
    [sym_err_name_invalid] = ACTIONS(2037),
    [sym_err_password_invalid] = ACTIONS(2037),
    [sym_err_invalid_account] = ACTIONS(2037),
    [sym_err_no_authorization] = ACTIONS(2037),
    [sym_err_invalid_parameter] = ACTIONS(2037),
    [sym_err_user_locked_out] = ACTIONS(2037),
    [sym_err_user_account_expired] = ACTIONS(2037),
    [sym_err_general_error] = ACTIONS(2037),
    [sym_err_authentication_error] = ACTIONS(2037),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2037),
    [sym_tls_validate_certificate] = ACTIONS(2037),
    [sym_tdata] = ACTIONS(2037),
    [sym_tchannel] = ACTIONS(2037),
    [sym_tlevel] = ACTIONS(2037),
    [sym_tbutton] = ACTIONS(2037),
    [sym_ttimeline] = ACTIONS(2037),
    [sym_tcustom] = ACTIONS(2037),
    [sym_url_struct] = ACTIONS(2037),
    [sym_dns_struct] = ACTIONS(2037),
    [sym_ip_address_struct] = ACTIONS(2037),
    [sym_dev_info_struct] = ACTIONS(2037),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2037),
    [sym_data_feed] = ACTIONS(2037),
    [sym_data_field] = ACTIONS(2037),
    [sym_data_record] = ACTIONS(2037),
    [sym_wc_data_feed] = ACTIONS(2037),
    [sym_wc_data_field] = ACTIONS(2037),
    [sym_wc_data_record] = ACTIONS(2037),
    [sym_last_login_info] = ACTIONS(2037),
    [sym____reserved___] = ACTIONS(2037),
    [sym_date] = ACTIONS(2037),
    [sym_day] = ACTIONS(2037),
    [sym_time] = ACTIONS(2037),
    [sym_ldate] = ACTIONS(2037),
    [sym_push_device] = ACTIONS(2037),
    [sym_push_channel] = ACTIONS(2037),
    [sym_push_devchan] = ACTIONS(2037),
    [sym_release_device] = ACTIONS(2037),
    [sym_release_channel] = ACTIONS(2037),
    [sym_release_devchan] = ACTIONS(2037),
    [sym_master_slot] = ACTIONS(2037),
    [sym_get_pulse_time] = ACTIONS(2037),
    [sym_get_timer] = ACTIONS(2037),
    [sym_master_sn] = ACTIONS(2037),
    [sym_system_number] = ACTIONS(2037),
    [sym_button] = ACTIONS(2037),
    [sym_dv_channel] = ACTIONS(2037),
    [sym_data] = ACTIONS(2037),
    [sym_level] = ACTIONS(2037),
    [sym_timeline] = ACTIONS(2037),
    [sym_channel] = ACTIONS(2037),
    [sym_custom] = ACTIONS(2037),
    [sym_atoi] = ACTIONS(2037),
    [sym_atol] = ACTIONS(2037),
    [sym_atof] = ACTIONS(2037),
    [sym_itoa] = ACTIONS(2037),
    [sym_itohex] = ACTIONS(2037),
    [sym_hextoi] = ACTIONS(2037),
    [sym_ftoa] = ACTIONS(2037),
    [sym_find_string] = ACTIONS(2037),
    [sym_left_string] = ACTIONS(2037),
    [sym_right_string] = ACTIONS(2037),
    [sym_mid_string] = ACTIONS(2037),
    [sym_remove_string] = ACTIONS(2037),
    [sym_get_buffer_string] = ACTIONS(2037),
    [sym_get_buffer_char] = ACTIONS(2037),
    [sym_get_multi_buffer_string] = ACTIONS(2037),
    [sym_length_string] = ACTIONS(2037),
    [sym_max_length_string] = ACTIONS(2037),
    [sym_set_length_string] = ACTIONS(2037),
    [sym_compare_string] = ACTIONS(2037),
    [sym_upper_string] = ACTIONS(2037),
    [sym_lower_string] = ACTIONS(2037),
    [sym_get_last] = ACTIONS(2037),
    [sym_date_to_day] = ACTIONS(2037),
    [sym_date_to_month] = ACTIONS(2037),
    [sym_date_to_year] = ACTIONS(2037),
    [sym_time_to_hour] = ACTIONS(2037),
    [sym_time_to_minute] = ACTIONS(2037),
    [sym_time_to_second] = ACTIONS(2037),
    [sym_day_of_week] = ACTIONS(2037),
    [sym_set_pulse_time] = ACTIONS(2037),
    [sym_set_timer] = ACTIONS(2037),
    [sym_file_open] = ACTIONS(2037),
    [sym_file_close] = ACTIONS(2037),
    [sym_file_read] = ACTIONS(2037),
    [sym_file_read_line] = ACTIONS(2037),
    [sym_file_write] = ACTIONS(2037),
    [sym_file_write_line] = ACTIONS(2037),
    [sym_file_copy] = ACTIONS(2037),
    [sym_file_delete] = ACTIONS(2037),
    [sym_file_rename] = ACTIONS(2037),
    [sym_file_dir] = ACTIONS(2037),
    [sym_file_getdir] = ACTIONS(2037),
    [sym_file_setdir] = ACTIONS(2037),
    [sym_file_create_dir] = ACTIONS(2037),
    [sym_file_remove_dir] = ACTIONS(2037),
    [sym_file_seek] = ACTIONS(2037),
    [sym_length_array] = ACTIONS(2037),
    [sym_max_length_array] = ACTIONS(2037),
    [sym_set_length_array] = ACTIONS(2037),
    [sym_ip_client_open] = ACTIONS(2037),
    [sym_ip_client_close] = ACTIONS(2037),
    [sym_ip_server_open] = ACTIONS(2037),
    [sym_ip_server_close] = ACTIONS(2037),
    [sym_ip_bound_client_open] = ACTIONS(2037),
    [sym_ip_mc_server_open] = ACTIONS(2037),
    [sym_ip_set_option] = ACTIONS(2037),
    [sym_ssh_client_open] = ACTIONS(2037),
    [sym_ssh_client_close] = ACTIONS(2037),
    [sym_tls_client_open] = ACTIONS(2037),
    [sym_tls_client_close] = ACTIONS(2037),
    [sym_do_push] = ACTIONS(2037),
    [sym_do_push_timed] = ACTIONS(2037),
    [sym_do_release] = ACTIONS(2037),
    [sym_device_id] = ACTIONS(2037),
    [sym_device_info] = ACTIONS(2037),
    [sym_device_id_string] = ACTIONS(2037),
    [sym_device_standby] = ACTIONS(2037),
    [sym_device_wake] = ACTIONS(2037),
    [sym_timeline_create] = ACTIONS(2037),
    [sym_timeline_kill] = ACTIONS(2037),
    [sym_timeline_reload] = ACTIONS(2037),
    [sym_timeline_pause] = ACTIONS(2037),
    [sym_timeline_restart] = ACTIONS(2037),
    [sym_timeline_set] = ACTIONS(2037),
    [sym_timeline_get] = ACTIONS(2037),
    [sym_timeline_active] = ACTIONS(2037),
    [sym_timeline_dynamic_id] = ACTIONS(2037),
    [sym_variable_to_string] = ACTIONS(2037),
    [sym_string_to_variable] = ACTIONS(2037),
    [sym_length_variable_to_string] = ACTIONS(2037),
    [sym_type_cast] = ACTIONS(2037),
    [sym_variable_to_xml] = ACTIONS(2037),
    [sym_xml_to_variable] = ACTIONS(2037),
    [sym_length_variable_to_xml] = ACTIONS(2037),
    [sym_redirect_string] = ACTIONS(2037),
    [sym_random_number] = ACTIONS(2037),
    [sym_get_unique_id] = ACTIONS(2037),
    [sym_get_system_number] = ACTIONS(2037),
    [sym_set_system_number] = ACTIONS(2037),
    [sym_get_serial_number] = ACTIONS(2037),
    [sym_get_master_build] = ACTIONS(2037),
    [sym_set_validation_code] = ACTIONS(2037),
    [sym_reboot] = ACTIONS(2037),
    [sym_load_duet_module] = ACTIONS(2037),
    [sym_unload_duet_module] = ACTIONS(2037),
    [sym_duet_mem_size_set] = ACTIONS(2037),
    [sym_duet_mem_size_get] = ACTIONS(2037),
    [sym_dynamic_application_device] = ACTIONS(2037),
    [sym_dynamic_polled_port] = ACTIONS(2037),
    [sym_static_port_binding] = ACTIONS(2037),
    [sym_static_ip_binding] = ACTIONS(2037),
    [sym_get_dns_list] = ACTIONS(2037),
    [sym_set_dns_list] = ACTIONS(2037),
    [sym_get_ip_address] = ACTIONS(2037),
    [sym_set_ip_address] = ACTIONS(2037),
    [sym_get_url_list] = ACTIONS(2037),
    [sym_add_url_entry] = ACTIONS(2037),
    [sym_delete_url_entry] = ACTIONS(2037),
    [sym_abs_value] = ACTIONS(2037),
    [sym_max_value] = ACTIONS(2037),
    [sym_min_value] = ACTIONS(2037),
    [sym_exp_value] = ACTIONS(2037),
    [sym_log_value] = ACTIONS(2037),
    [sym_log10_value] = ACTIONS(2037),
    [sym_power_value] = ACTIONS(2037),
    [sym_sqrt_value] = ACTIONS(2037),
    [sym_raw_be] = ACTIONS(2037),
    [sym_raw_le] = ACTIONS(2037),
    [sym_internal_queue_size_set] = ACTIONS(2037),
    [sym_internal_queue_size_get] = ACTIONS(2037),
    [sym_internal_threshold_set] = ACTIONS(2037),
    [sym_internal_threshold_get] = ACTIONS(2037),
    [sym_rebuilt_event] = ACTIONS(2037),
    [sym_do_custom_event] = ACTIONS(2037),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2037),
    [sym_clkmgr_set_clk_source] = ACTIONS(2037),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2037),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2037),
    [sym_clkmgr_get_timezone] = ACTIONS(2037),
    [sym_clkmgr_set_timezone] = ACTIONS(2037),
    [sym_clkmgr_get_resync_period] = ACTIONS(2037),
    [sym_clkmgr_set_resync_period] = ACTIONS(2037),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2037),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2037),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2037),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2037),
    [sym_clkmgr_get_timeservers] = ACTIONS(2037),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2037),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2037),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2037),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2037),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2037),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2037),
    [sym_set_log_level] = ACTIONS(2037),
    [sym_get_log_level] = ACTIONS(2037),
    [sym_amx_log] = ACTIONS(2037),
    [sym_smtp_server_config_set] = ACTIONS(2037),
    [sym_smtp_server_config_get] = ACTIONS(2037),
    [sym_smtp_send] = ACTIONS(2037),
    [sym_validate_netlinx_account] = ACTIONS(2037),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2037),
    [sym_authenticate_certificate] = ACTIONS(2037),
    [sym_audit_netlinx_session_event] = ACTIONS(2037),
    [sym_audit_netlinx_generic_event] = ACTIONS(2037),
    [sym_get_available_flash_disk_space] = ACTIONS(2037),
    [sym_get_max_flash_disk_space] = ACTIONS(2037),
    [sym_data_create_feed] = ACTIONS(2037),
    [sym_data_delete_feed] = ACTIONS(2037),
    [sym_data_publish_feed] = ACTIONS(2037),
    [sym_data_get_published_feed] = ACTIONS(2037),
    [sym_data_add_record] = ACTIONS(2037),
    [sym_data_get_event_record] = ACTIONS(2037),
    [sym__wc_data_create_feed] = ACTIONS(2037),
    [sym__wc_data_add_record] = ACTIONS(2037),
    [sym__wc_data_get_event_record] = ACTIONS(2037),
    [sym_format] = ACTIONS(2037),
    [sym_astro_clock] = ACTIONS(2037),
    [sym_set_outdoor_temperature] = ACTIONS(2037),
    [sym_set_virtual_level_count] = ACTIONS(2037),
    [sym_set_virtual_channel_count] = ACTIONS(2037),
    [sym_set_virtual_port_count] = ACTIONS(2037),
    [sym__wc_encode_int] = ACTIONS(2037),
    [sym__wc_decode_int] = ACTIONS(2037),
    [sym__wc_int] = ACTIONS(2037),
    [sym__wc_to_ch_int] = ACTIONS(2037),
    [sym__ch_to_wc_int] = ACTIONS(2037),
    [sym__wc_find_string_int] = ACTIONS(2037),
    [sym__wc_left_string_int] = ACTIONS(2037),
    [sym__wc_lower_string_int] = ACTIONS(2037),
    [sym__wc_mid_string_int] = ACTIONS(2037),
    [sym__wc_remove_string_int] = ACTIONS(2037),
    [sym__wc_right_string_int] = ACTIONS(2037),
    [sym__wc_upper_string_int] = ACTIONS(2037),
    [sym__wc_compare_string_int] = ACTIONS(2037),
    [sym__wc_get_buffer_char_int] = ACTIONS(2037),
    [sym__wc_get_buffer_string_int] = ACTIONS(2037),
    [sym__wc_concat_string_int] = ACTIONS(2037),
    [sym__wc_file_open_int] = ACTIONS(2037),
    [sym__wc_file_close_int] = ACTIONS(2037),
    [sym__wc_file_read_int] = ACTIONS(2037),
    [sym__wc_file_read_line_int] = ACTIONS(2037),
    [sym__wc_file_write_int] = ACTIONS(2037),
    [sym__wc_file_write_line_int] = ACTIONS(2037),
    [aux_sym_preproc_include_token1] = ACTIONS(2037),
    [aux_sym_preproc_define_token1] = ACTIONS(2039),
    [aux_sym_preproc_warn_token1] = ACTIONS(2039),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2039),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2039),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2039),
    [anon_sym_LBRACK] = ACTIONS(2039),
    [anon_sym_LBRACE] = ACTIONS(2039),
    [anon_sym_RBRACE] = ACTIONS(2039),
    [anon_sym_LPAREN] = ACTIONS(2037),
    [anon_sym_BANG] = ACTIONS(2039),
    [anon_sym_TILDE] = ACTIONS(2039),
    [anon_sym_DASH] = ACTIONS(2037),
    [anon_sym_PLUS] = ACTIONS(2037),
    [anon_sym_DASH_DASH] = ACTIONS(2039),
    [anon_sym_PLUS_PLUS] = ACTIONS(2039),
    [anon_sym_DQUOTE] = ACTIONS(2039),
    [anon_sym_SQUOTE] = ACTIONS(2039),
    [sym_number_literal] = ACTIONS(2039),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(541)] = {
    [sym_identifier] = ACTIONS(2041),
    [sym_define_call_keyword] = ACTIONS(2041),
    [sym_define_function_keyword] = ACTIONS(2041),
    [sym_define_library_function_keyword] = ACTIONS(2041),
    [sym_define_module_keyword] = ACTIONS(2041),
    [sym_char_keyword] = ACTIONS(2041),
    [sym_widechar_keyword] = ACTIONS(2041),
    [sym_integer_keyword] = ACTIONS(2041),
    [sym_sinteger_keyword] = ACTIONS(2041),
    [sym_long_keyword] = ACTIONS(2041),
    [sym_slong_keyword] = ACTIONS(2041),
    [sym_float_keyword] = ACTIONS(2041),
    [sym_double_keyword] = ACTIONS(2041),
    [sym_variant_keyword] = ACTIONS(2041),
    [sym_variantarray_keyword] = ACTIONS(2041),
    [sym_local_var_keyword] = ACTIONS(2041),
    [sym_stack_var_keyword] = ACTIONS(2041),
    [sym_constant_keyword] = ACTIONS(2041),
    [sym_volatile_keyword] = ACTIONS(2041),
    [sym_non_volatile_keyword] = ACTIONS(2041),
    [sym_persistent_keyword] = ACTIONS(2041),
    [sym_dev_keyword] = ACTIONS(2041),
    [sym_devlev_keyword] = ACTIONS(2041),
    [sym_devchan_keyword] = ACTIONS(2041),
    [sym_if_keyword] = ACTIONS(2041),
    [sym_switch_keyword] = ACTIONS(2041),
    [sym_case_keyword] = ACTIONS(2041),
    [sym_default_keyword] = ACTIONS(2041),
    [sym_while_keyword] = ACTIONS(2041),
    [sym_for_keyword] = ACTIONS(2041),
    [sym_break_keyword] = ACTIONS(2041),
    [sym_continue_keyword] = ACTIONS(2041),
    [sym_return_keyword] = ACTIONS(2041),
    [sym_select_keyword] = ACTIONS(2041),
    [sym_active_keyword] = ACTIONS(2041),
    [sym_struct_keyword] = ACTIONS(2041),
    [sym_structure_keyword] = ACTIONS(2041),
    [aux_sym_on_keyword_token1] = ACTIONS(2041),
    [aux_sym_off_keyword_token1] = ACTIONS(2041),
    [sym_send_level_keyword] = ACTIONS(2041),
    [sym_send_string_keyword] = ACTIONS(2041),
    [sym_send_command_keyword] = ACTIONS(2041),
    [sym_clear_buffer_keyword] = ACTIONS(2041),
    [sym_create_buffer_keyword] = ACTIONS(2041),
    [sym_create_multi_buffer_keyword] = ACTIONS(2041),
    [sym_call_keyword] = ACTIONS(2041),
    [sym_system_call_keyword] = ACTIONS(2041),
    [sym_devchan_to_keyword] = ACTIONS(2041),
    [sym_devchan_min_to_keyword] = ACTIONS(2041),
    [sym_devchan_total_off_keyword] = ACTIONS(2041),
    [sym_devchan_pulse_keyword] = ACTIONS(2041),
    [sym_wait_keyword] = ACTIONS(2041),
    [sym_cancel_wait_keyword] = ACTIONS(2041),
    [sym_wait_until_keyword] = ACTIONS(2041),
    [sym_cancel_wait_until_keyword] = ACTIONS(2041),
    [sym_cancel_all_wait_keyword] = ACTIONS(2041),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2041),
    [sym_not] = ACTIONS(2041),
    [sym_bnot] = ACTIONS(2041),
    [sym___file__] = ACTIONS(2041),
    [sym___date__] = ACTIONS(2041),
    [sym___ldate__] = ACTIONS(2041),
    [sym___time__] = ACTIONS(2041),
    [sym___line__] = ACTIONS(2041),
    [sym___name__] = ACTIONS(2041),
    [sym___netlinx__] = ACTIONS(2041),
    [sym_netlinx_axi_version] = ACTIONS(2041),
    [sym_url_flg_tcp] = ACTIONS(2041),
    [sym_url_flg_accinfopresent] = ACTIONS(2041),
    [sym_url_flg_temp] = ACTIONS(2041),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2041),
    [sym_url_flg_stat_mask] = ACTIONS(2041),
    [sym_url_flg_stat_lookup] = ACTIONS(2041),
    [sym_url_flg_stat_connecting] = ACTIONS(2041),
    [sym_url_flg_stat_waiting] = ACTIONS(2041),
    [sym_url_flg_stat_connected] = ACTIONS(2041),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2041),
    [sym_first_virtual_device] = ACTIONS(2041),
    [sym_first_local_port] = ACTIONS(2041),
    [sym_dynamic_virtual_device] = ACTIONS(2041),
    [sym_timeline_once] = ACTIONS(2041),
    [sym_timeline_repeat] = ACTIONS(2041),
    [sym_timeline_absolute] = ACTIONS(2041),
    [sym_timeline_relative] = ACTIONS(2041),
    [sym_do_push_timed_infinite] = ACTIONS(2041),
    [sym_source_type_no_address] = ACTIONS(2041),
    [sym_source_type_neuron_id] = ACTIONS(2041),
    [sym_source_type_ip_address] = ACTIONS(2041),
    [sym_source_type_axlink] = ACTIONS(2041),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2041),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2041),
    [sym_source_type_ip_socket_address] = ACTIONS(2041),
    [sym_source_type_rs232] = ACTIONS(2041),
    [sym_source_type_internal] = ACTIONS(2041),
    [sym_source_type_mac_address] = ACTIONS(2041),
    [sym_source_type_ipv4_port] = ACTIONS(2041),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2041),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2041),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2041),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2041),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2041),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2041),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2041),
    [sym_true] = ACTIONS(2041),
    [sym_false] = ACTIONS(2041),
    [sym_file_read_only] = ACTIONS(2041),
    [sym_file_rw_new] = ACTIONS(2041),
    [sym_file_rw_append] = ACTIONS(2041),
    [sym_ip_tcp] = ACTIONS(2041),
    [sym_ip_udp] = ACTIONS(2041),
    [sym_ip_udp_2way] = ACTIONS(2041),
    [sym_xml_encode_types] = ACTIONS(2041),
    [sym_xml_encode_char_as_list] = ACTIONS(2041),
    [sym_xml_encode_le] = ACTIONS(2041),
    [sym_xml_decode_types] = ACTIONS(2041),
    [sym_xml_decode_no_preserve] = ACTIONS(2041),
    [sym_duet_dev_type_display_device] = ACTIONS(2041),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2041),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2041),
    [sym_duet_dev_type_io_device] = ACTIONS(2041),
    [sym_duet_dev_type_relay_device] = ACTIONS(2041),
    [sym_duet_dev_type_ups] = ACTIONS(2041),
    [sym_duet_dev_type_amplifier] = ACTIONS(2041),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2041),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2041),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2041),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2041),
    [sym_duet_dev_type_camera] = ACTIONS(2041),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2041),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2041),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2041),
    [sym_duet_dev_type_dss] = ACTIONS(2041),
    [sym_duet_dev_type_dvr] = ACTIONS(2041),
    [sym_duet_dev_type_disc_device] = ACTIONS(2041),
    [sym_duet_dev_type_document_camera] = ACTIONS(2041),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2041),
    [sym_duet_dev_type_hvac] = ACTIONS(2041),
    [sym_duet_dev_type_keypad] = ACTIONS(2041),
    [sym_duet_dev_type_light] = ACTIONS(2041),
    [sym_duet_dev_type_monitor] = ACTIONS(2041),
    [sym_duet_dev_type_motor] = ACTIONS(2041),
    [sym_duet_dev_type_multi_window] = ACTIONS(2041),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2041),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2041),
    [sym_duet_dev_type_receiver] = ACTIONS(2041),
    [sym_duet_dev_type_security_system] = ACTIONS(2041),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2041),
    [sym_duet_dev_type_settop_box] = ACTIONS(2041),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2041),
    [sym_duet_dev_type_switcher] = ACTIONS(2041),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2041),
    [sym_duet_dev_type_tv] = ACTIONS(2041),
    [sym_duet_dev_type_utility] = ACTIONS(2041),
    [sym_duet_dev_type_vcr] = ACTIONS(2041),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2041),
    [sym_duet_dev_type_video_processor] = ACTIONS(2041),
    [sym_duet_dev_type_video_projector] = ACTIONS(2041),
    [sym_duet_dev_type_video_wall] = ACTIONS(2041),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2041),
    [sym_duet_dev_type_weather] = ACTIONS(2041),
    [sym_duet_dev_not_polled] = ACTIONS(2041),
    [sym_duet_dev_polled] = ACTIONS(2041),
    [sym_clkmgr_mode_network] = ACTIONS(2041),
    [sym_clkmgr_mode_standalone] = ACTIONS(2041),
    [sym_normal_standby] = ACTIONS(2041),
    [sym_normal_wake] = ACTIONS(2041),
    [sym_ip_multicast_ttl_option] = ACTIONS(2041),
    [sym_ip_ttl_subnet] = ACTIONS(2041),
    [sym_ip_ttl_site] = ACTIONS(2041),
    [sym_ip_ttl_region] = ACTIONS(2041),
    [sym_ip_ttl_continent] = ACTIONS(2041),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2041),
    [sym_ip_nodelay_on] = ACTIONS(2041),
    [sym_ip_nodelay_off] = ACTIONS(2041),
    [sym_amx_error] = ACTIONS(2041),
    [sym_amx_warning] = ACTIONS(2041),
    [sym_amx_info] = ACTIONS(2041),
    [sym_amx_debug] = ACTIONS(2041),
    [sym_netlinx_login_success] = ACTIONS(2041),
    [sym_netlinx_login_fail] = ACTIONS(2041),
    [sym_netlinx_logout] = ACTIONS(2041),
    [sym_null_str] = ACTIONS(2041),
    [sym_smtp_address] = ACTIONS(2041),
    [sym_smtp_port_number] = ACTIONS(2041),
    [sym_smtp_username] = ACTIONS(2041),
    [sym_smtp_password] = ACTIONS(2041),
    [sym_smtp_require_tls] = ACTIONS(2041),
    [sym_smtp_from] = ACTIONS(2041),
    [sym_smtp_tls_true] = ACTIONS(2041),
    [sym_smtp_tls_false] = ACTIONS(2041),
    [sym_listview_on_row_select_event] = ACTIONS(2041),
    [sym_data_structure_unknown] = ACTIONS(2041),
    [sym_data_structure_datafeed] = ACTIONS(2041),
    [sym_data_structure_datarecordset] = ACTIONS(2041),
    [sym_data_structure_datarecord] = ACTIONS(2041),
    [sym_data_structure_datafield] = ACTIONS(2041),
    [sym_data_max_content_field_count] = ACTIONS(2041),
    [sym_data_max_metadata_field_count] = ACTIONS(2041),
    [sym_data_max_id_length] = ACTIONS(2041),
    [sym_data_max_name_length] = ACTIONS(2041),
    [sym_data_max_description_length] = ACTIONS(2041),
    [sym_data_max_type_length] = ACTIONS(2041),
    [sym_data_max_format_length] = ACTIONS(2041),
    [sym_data_max_source_length] = ACTIONS(2041),
    [sym_data_max_value_length] = ACTIONS(2041),
    [sym_data_max_label_length] = ACTIONS(2041),
    [sym_data_max_url_length] = ACTIONS(2041),
    [sym_max_last_login_info_length] = ACTIONS(2041),
    [sym_data_type_unknown] = ACTIONS(2041),
    [sym_data_type_string] = ACTIONS(2041),
    [sym_data_type_datetime] = ACTIONS(2041),
    [sym_data_type_date] = ACTIONS(2041),
    [sym_data_type_time] = ACTIONS(2041),
    [sym_data_type_image] = ACTIONS(2041),
    [sym_data_format_url] = ACTIONS(2041),
    [sym_data_format_phone] = ACTIONS(2041),
    [sym_data_format_email] = ACTIONS(2041),
    [sym_data_format_iso8601] = ACTIONS(2041),
    [sym_auditlog_priv] = ACTIONS(2041),
    [sym_configuration_priv] = ACTIONS(2041),
    [sym_deviceconfig_priv] = ACTIONS(2041),
    [sym_ftp_priv] = ACTIONS(2041),
    [sym_http_priv] = ACTIONS(2041),
    [sym_networkconfig_priv] = ACTIONS(2041),
    [sym_programport_priv] = ACTIONS(2041),
    [sym_remoteui_priv] = ACTIONS(2041),
    [sym_securitycontrol_priv] = ACTIONS(2041),
    [sym_softwaremanagement_priv] = ACTIONS(2041),
    [sym_terminal_priv] = ACTIONS(2041),
    [sym_tpadmin_priv] = ACTIONS(2041),
    [sym_user1_priv] = ACTIONS(2041),
    [sym_user2_priv] = ACTIONS(2041),
    [sym_user3_priv] = ACTIONS(2041),
    [sym_user4_priv] = ACTIONS(2041),
    [sym_usermanagement_priv] = ACTIONS(2041),
    [sym_valid_account] = ACTIONS(2041),
    [sym_err_name_invalid] = ACTIONS(2041),
    [sym_err_password_invalid] = ACTIONS(2041),
    [sym_err_invalid_account] = ACTIONS(2041),
    [sym_err_no_authorization] = ACTIONS(2041),
    [sym_err_invalid_parameter] = ACTIONS(2041),
    [sym_err_user_locked_out] = ACTIONS(2041),
    [sym_err_user_account_expired] = ACTIONS(2041),
    [sym_err_general_error] = ACTIONS(2041),
    [sym_err_authentication_error] = ACTIONS(2041),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2041),
    [sym_tls_validate_certificate] = ACTIONS(2041),
    [sym_tdata] = ACTIONS(2041),
    [sym_tchannel] = ACTIONS(2041),
    [sym_tlevel] = ACTIONS(2041),
    [sym_tbutton] = ACTIONS(2041),
    [sym_ttimeline] = ACTIONS(2041),
    [sym_tcustom] = ACTIONS(2041),
    [sym_url_struct] = ACTIONS(2041),
    [sym_dns_struct] = ACTIONS(2041),
    [sym_ip_address_struct] = ACTIONS(2041),
    [sym_dev_info_struct] = ACTIONS(2041),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2041),
    [sym_data_feed] = ACTIONS(2041),
    [sym_data_field] = ACTIONS(2041),
    [sym_data_record] = ACTIONS(2041),
    [sym_wc_data_feed] = ACTIONS(2041),
    [sym_wc_data_field] = ACTIONS(2041),
    [sym_wc_data_record] = ACTIONS(2041),
    [sym_last_login_info] = ACTIONS(2041),
    [sym____reserved___] = ACTIONS(2041),
    [sym_date] = ACTIONS(2041),
    [sym_day] = ACTIONS(2041),
    [sym_time] = ACTIONS(2041),
    [sym_ldate] = ACTIONS(2041),
    [sym_push_device] = ACTIONS(2041),
    [sym_push_channel] = ACTIONS(2041),
    [sym_push_devchan] = ACTIONS(2041),
    [sym_release_device] = ACTIONS(2041),
    [sym_release_channel] = ACTIONS(2041),
    [sym_release_devchan] = ACTIONS(2041),
    [sym_master_slot] = ACTIONS(2041),
    [sym_get_pulse_time] = ACTIONS(2041),
    [sym_get_timer] = ACTIONS(2041),
    [sym_master_sn] = ACTIONS(2041),
    [sym_system_number] = ACTIONS(2041),
    [sym_button] = ACTIONS(2041),
    [sym_dv_channel] = ACTIONS(2041),
    [sym_data] = ACTIONS(2041),
    [sym_level] = ACTIONS(2041),
    [sym_timeline] = ACTIONS(2041),
    [sym_channel] = ACTIONS(2041),
    [sym_custom] = ACTIONS(2041),
    [sym_atoi] = ACTIONS(2041),
    [sym_atol] = ACTIONS(2041),
    [sym_atof] = ACTIONS(2041),
    [sym_itoa] = ACTIONS(2041),
    [sym_itohex] = ACTIONS(2041),
    [sym_hextoi] = ACTIONS(2041),
    [sym_ftoa] = ACTIONS(2041),
    [sym_find_string] = ACTIONS(2041),
    [sym_left_string] = ACTIONS(2041),
    [sym_right_string] = ACTIONS(2041),
    [sym_mid_string] = ACTIONS(2041),
    [sym_remove_string] = ACTIONS(2041),
    [sym_get_buffer_string] = ACTIONS(2041),
    [sym_get_buffer_char] = ACTIONS(2041),
    [sym_get_multi_buffer_string] = ACTIONS(2041),
    [sym_length_string] = ACTIONS(2041),
    [sym_max_length_string] = ACTIONS(2041),
    [sym_set_length_string] = ACTIONS(2041),
    [sym_compare_string] = ACTIONS(2041),
    [sym_upper_string] = ACTIONS(2041),
    [sym_lower_string] = ACTIONS(2041),
    [sym_get_last] = ACTIONS(2041),
    [sym_date_to_day] = ACTIONS(2041),
    [sym_date_to_month] = ACTIONS(2041),
    [sym_date_to_year] = ACTIONS(2041),
    [sym_time_to_hour] = ACTIONS(2041),
    [sym_time_to_minute] = ACTIONS(2041),
    [sym_time_to_second] = ACTIONS(2041),
    [sym_day_of_week] = ACTIONS(2041),
    [sym_set_pulse_time] = ACTIONS(2041),
    [sym_set_timer] = ACTIONS(2041),
    [sym_file_open] = ACTIONS(2041),
    [sym_file_close] = ACTIONS(2041),
    [sym_file_read] = ACTIONS(2041),
    [sym_file_read_line] = ACTIONS(2041),
    [sym_file_write] = ACTIONS(2041),
    [sym_file_write_line] = ACTIONS(2041),
    [sym_file_copy] = ACTIONS(2041),
    [sym_file_delete] = ACTIONS(2041),
    [sym_file_rename] = ACTIONS(2041),
    [sym_file_dir] = ACTIONS(2041),
    [sym_file_getdir] = ACTIONS(2041),
    [sym_file_setdir] = ACTIONS(2041),
    [sym_file_create_dir] = ACTIONS(2041),
    [sym_file_remove_dir] = ACTIONS(2041),
    [sym_file_seek] = ACTIONS(2041),
    [sym_length_array] = ACTIONS(2041),
    [sym_max_length_array] = ACTIONS(2041),
    [sym_set_length_array] = ACTIONS(2041),
    [sym_ip_client_open] = ACTIONS(2041),
    [sym_ip_client_close] = ACTIONS(2041),
    [sym_ip_server_open] = ACTIONS(2041),
    [sym_ip_server_close] = ACTIONS(2041),
    [sym_ip_bound_client_open] = ACTIONS(2041),
    [sym_ip_mc_server_open] = ACTIONS(2041),
    [sym_ip_set_option] = ACTIONS(2041),
    [sym_ssh_client_open] = ACTIONS(2041),
    [sym_ssh_client_close] = ACTIONS(2041),
    [sym_tls_client_open] = ACTIONS(2041),
    [sym_tls_client_close] = ACTIONS(2041),
    [sym_do_push] = ACTIONS(2041),
    [sym_do_push_timed] = ACTIONS(2041),
    [sym_do_release] = ACTIONS(2041),
    [sym_device_id] = ACTIONS(2041),
    [sym_device_info] = ACTIONS(2041),
    [sym_device_id_string] = ACTIONS(2041),
    [sym_device_standby] = ACTIONS(2041),
    [sym_device_wake] = ACTIONS(2041),
    [sym_timeline_create] = ACTIONS(2041),
    [sym_timeline_kill] = ACTIONS(2041),
    [sym_timeline_reload] = ACTIONS(2041),
    [sym_timeline_pause] = ACTIONS(2041),
    [sym_timeline_restart] = ACTIONS(2041),
    [sym_timeline_set] = ACTIONS(2041),
    [sym_timeline_get] = ACTIONS(2041),
    [sym_timeline_active] = ACTIONS(2041),
    [sym_timeline_dynamic_id] = ACTIONS(2041),
    [sym_variable_to_string] = ACTIONS(2041),
    [sym_string_to_variable] = ACTIONS(2041),
    [sym_length_variable_to_string] = ACTIONS(2041),
    [sym_type_cast] = ACTIONS(2041),
    [sym_variable_to_xml] = ACTIONS(2041),
    [sym_xml_to_variable] = ACTIONS(2041),
    [sym_length_variable_to_xml] = ACTIONS(2041),
    [sym_redirect_string] = ACTIONS(2041),
    [sym_random_number] = ACTIONS(2041),
    [sym_get_unique_id] = ACTIONS(2041),
    [sym_get_system_number] = ACTIONS(2041),
    [sym_set_system_number] = ACTIONS(2041),
    [sym_get_serial_number] = ACTIONS(2041),
    [sym_get_master_build] = ACTIONS(2041),
    [sym_set_validation_code] = ACTIONS(2041),
    [sym_reboot] = ACTIONS(2041),
    [sym_load_duet_module] = ACTIONS(2041),
    [sym_unload_duet_module] = ACTIONS(2041),
    [sym_duet_mem_size_set] = ACTIONS(2041),
    [sym_duet_mem_size_get] = ACTIONS(2041),
    [sym_dynamic_application_device] = ACTIONS(2041),
    [sym_dynamic_polled_port] = ACTIONS(2041),
    [sym_static_port_binding] = ACTIONS(2041),
    [sym_static_ip_binding] = ACTIONS(2041),
    [sym_get_dns_list] = ACTIONS(2041),
    [sym_set_dns_list] = ACTIONS(2041),
    [sym_get_ip_address] = ACTIONS(2041),
    [sym_set_ip_address] = ACTIONS(2041),
    [sym_get_url_list] = ACTIONS(2041),
    [sym_add_url_entry] = ACTIONS(2041),
    [sym_delete_url_entry] = ACTIONS(2041),
    [sym_abs_value] = ACTIONS(2041),
    [sym_max_value] = ACTIONS(2041),
    [sym_min_value] = ACTIONS(2041),
    [sym_exp_value] = ACTIONS(2041),
    [sym_log_value] = ACTIONS(2041),
    [sym_log10_value] = ACTIONS(2041),
    [sym_power_value] = ACTIONS(2041),
    [sym_sqrt_value] = ACTIONS(2041),
    [sym_raw_be] = ACTIONS(2041),
    [sym_raw_le] = ACTIONS(2041),
    [sym_internal_queue_size_set] = ACTIONS(2041),
    [sym_internal_queue_size_get] = ACTIONS(2041),
    [sym_internal_threshold_set] = ACTIONS(2041),
    [sym_internal_threshold_get] = ACTIONS(2041),
    [sym_rebuilt_event] = ACTIONS(2041),
    [sym_do_custom_event] = ACTIONS(2041),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2041),
    [sym_clkmgr_set_clk_source] = ACTIONS(2041),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2041),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2041),
    [sym_clkmgr_get_timezone] = ACTIONS(2041),
    [sym_clkmgr_set_timezone] = ACTIONS(2041),
    [sym_clkmgr_get_resync_period] = ACTIONS(2041),
    [sym_clkmgr_set_resync_period] = ACTIONS(2041),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2041),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2041),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2041),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2041),
    [sym_clkmgr_get_timeservers] = ACTIONS(2041),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2041),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2041),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2041),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2041),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2041),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2041),
    [sym_set_log_level] = ACTIONS(2041),
    [sym_get_log_level] = ACTIONS(2041),
    [sym_amx_log] = ACTIONS(2041),
    [sym_smtp_server_config_set] = ACTIONS(2041),
    [sym_smtp_server_config_get] = ACTIONS(2041),
    [sym_smtp_send] = ACTIONS(2041),
    [sym_validate_netlinx_account] = ACTIONS(2041),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2041),
    [sym_authenticate_certificate] = ACTIONS(2041),
    [sym_audit_netlinx_session_event] = ACTIONS(2041),
    [sym_audit_netlinx_generic_event] = ACTIONS(2041),
    [sym_get_available_flash_disk_space] = ACTIONS(2041),
    [sym_get_max_flash_disk_space] = ACTIONS(2041),
    [sym_data_create_feed] = ACTIONS(2041),
    [sym_data_delete_feed] = ACTIONS(2041),
    [sym_data_publish_feed] = ACTIONS(2041),
    [sym_data_get_published_feed] = ACTIONS(2041),
    [sym_data_add_record] = ACTIONS(2041),
    [sym_data_get_event_record] = ACTIONS(2041),
    [sym__wc_data_create_feed] = ACTIONS(2041),
    [sym__wc_data_add_record] = ACTIONS(2041),
    [sym__wc_data_get_event_record] = ACTIONS(2041),
    [sym_format] = ACTIONS(2041),
    [sym_astro_clock] = ACTIONS(2041),
    [sym_set_outdoor_temperature] = ACTIONS(2041),
    [sym_set_virtual_level_count] = ACTIONS(2041),
    [sym_set_virtual_channel_count] = ACTIONS(2041),
    [sym_set_virtual_port_count] = ACTIONS(2041),
    [sym__wc_encode_int] = ACTIONS(2041),
    [sym__wc_decode_int] = ACTIONS(2041),
    [sym__wc_int] = ACTIONS(2041),
    [sym__wc_to_ch_int] = ACTIONS(2041),
    [sym__ch_to_wc_int] = ACTIONS(2041),
    [sym__wc_find_string_int] = ACTIONS(2041),
    [sym__wc_left_string_int] = ACTIONS(2041),
    [sym__wc_lower_string_int] = ACTIONS(2041),
    [sym__wc_mid_string_int] = ACTIONS(2041),
    [sym__wc_remove_string_int] = ACTIONS(2041),
    [sym__wc_right_string_int] = ACTIONS(2041),
    [sym__wc_upper_string_int] = ACTIONS(2041),
    [sym__wc_compare_string_int] = ACTIONS(2041),
    [sym__wc_get_buffer_char_int] = ACTIONS(2041),
    [sym__wc_get_buffer_string_int] = ACTIONS(2041),
    [sym__wc_concat_string_int] = ACTIONS(2041),
    [sym__wc_file_open_int] = ACTIONS(2041),
    [sym__wc_file_close_int] = ACTIONS(2041),
    [sym__wc_file_read_int] = ACTIONS(2041),
    [sym__wc_file_read_line_int] = ACTIONS(2041),
    [sym__wc_file_write_int] = ACTIONS(2041),
    [sym__wc_file_write_line_int] = ACTIONS(2041),
    [aux_sym_preproc_include_token1] = ACTIONS(2041),
    [aux_sym_preproc_define_token1] = ACTIONS(2043),
    [aux_sym_preproc_warn_token1] = ACTIONS(2043),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2043),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2043),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2043),
    [anon_sym_LBRACK] = ACTIONS(2043),
    [anon_sym_LBRACE] = ACTIONS(2043),
    [anon_sym_RBRACE] = ACTIONS(2043),
    [anon_sym_LPAREN] = ACTIONS(2041),
    [anon_sym_BANG] = ACTIONS(2043),
    [anon_sym_TILDE] = ACTIONS(2043),
    [anon_sym_DASH] = ACTIONS(2041),
    [anon_sym_PLUS] = ACTIONS(2041),
    [anon_sym_DASH_DASH] = ACTIONS(2043),
    [anon_sym_PLUS_PLUS] = ACTIONS(2043),
    [anon_sym_DQUOTE] = ACTIONS(2043),
    [anon_sym_SQUOTE] = ACTIONS(2043),
    [sym_number_literal] = ACTIONS(2043),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(542)] = {
    [sym_identifier] = ACTIONS(2045),
    [sym_define_call_keyword] = ACTIONS(2045),
    [sym_define_function_keyword] = ACTIONS(2045),
    [sym_define_library_function_keyword] = ACTIONS(2045),
    [sym_define_module_keyword] = ACTIONS(2045),
    [sym_char_keyword] = ACTIONS(2045),
    [sym_widechar_keyword] = ACTIONS(2045),
    [sym_integer_keyword] = ACTIONS(2045),
    [sym_sinteger_keyword] = ACTIONS(2045),
    [sym_long_keyword] = ACTIONS(2045),
    [sym_slong_keyword] = ACTIONS(2045),
    [sym_float_keyword] = ACTIONS(2045),
    [sym_double_keyword] = ACTIONS(2045),
    [sym_variant_keyword] = ACTIONS(2045),
    [sym_variantarray_keyword] = ACTIONS(2045),
    [sym_local_var_keyword] = ACTIONS(2045),
    [sym_stack_var_keyword] = ACTIONS(2045),
    [sym_constant_keyword] = ACTIONS(2045),
    [sym_volatile_keyword] = ACTIONS(2045),
    [sym_non_volatile_keyword] = ACTIONS(2045),
    [sym_persistent_keyword] = ACTIONS(2045),
    [sym_dev_keyword] = ACTIONS(2045),
    [sym_devlev_keyword] = ACTIONS(2045),
    [sym_devchan_keyword] = ACTIONS(2045),
    [sym_if_keyword] = ACTIONS(2045),
    [sym_switch_keyword] = ACTIONS(2045),
    [sym_case_keyword] = ACTIONS(2045),
    [sym_default_keyword] = ACTIONS(2045),
    [sym_while_keyword] = ACTIONS(2045),
    [sym_for_keyword] = ACTIONS(2045),
    [sym_break_keyword] = ACTIONS(2045),
    [sym_continue_keyword] = ACTIONS(2045),
    [sym_return_keyword] = ACTIONS(2045),
    [sym_select_keyword] = ACTIONS(2045),
    [sym_active_keyword] = ACTIONS(2045),
    [sym_struct_keyword] = ACTIONS(2045),
    [sym_structure_keyword] = ACTIONS(2045),
    [aux_sym_on_keyword_token1] = ACTIONS(2045),
    [aux_sym_off_keyword_token1] = ACTIONS(2045),
    [sym_send_level_keyword] = ACTIONS(2045),
    [sym_send_string_keyword] = ACTIONS(2045),
    [sym_send_command_keyword] = ACTIONS(2045),
    [sym_clear_buffer_keyword] = ACTIONS(2045),
    [sym_create_buffer_keyword] = ACTIONS(2045),
    [sym_create_multi_buffer_keyword] = ACTIONS(2045),
    [sym_call_keyword] = ACTIONS(2045),
    [sym_system_call_keyword] = ACTIONS(2045),
    [sym_devchan_to_keyword] = ACTIONS(2045),
    [sym_devchan_min_to_keyword] = ACTIONS(2045),
    [sym_devchan_total_off_keyword] = ACTIONS(2045),
    [sym_devchan_pulse_keyword] = ACTIONS(2045),
    [sym_wait_keyword] = ACTIONS(2045),
    [sym_cancel_wait_keyword] = ACTIONS(2045),
    [sym_wait_until_keyword] = ACTIONS(2045),
    [sym_cancel_wait_until_keyword] = ACTIONS(2045),
    [sym_cancel_all_wait_keyword] = ACTIONS(2045),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2045),
    [sym_not] = ACTIONS(2045),
    [sym_bnot] = ACTIONS(2045),
    [sym___file__] = ACTIONS(2045),
    [sym___date__] = ACTIONS(2045),
    [sym___ldate__] = ACTIONS(2045),
    [sym___time__] = ACTIONS(2045),
    [sym___line__] = ACTIONS(2045),
    [sym___name__] = ACTIONS(2045),
    [sym___netlinx__] = ACTIONS(2045),
    [sym_netlinx_axi_version] = ACTIONS(2045),
    [sym_url_flg_tcp] = ACTIONS(2045),
    [sym_url_flg_accinfopresent] = ACTIONS(2045),
    [sym_url_flg_temp] = ACTIONS(2045),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2045),
    [sym_url_flg_stat_mask] = ACTIONS(2045),
    [sym_url_flg_stat_lookup] = ACTIONS(2045),
    [sym_url_flg_stat_connecting] = ACTIONS(2045),
    [sym_url_flg_stat_waiting] = ACTIONS(2045),
    [sym_url_flg_stat_connected] = ACTIONS(2045),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2045),
    [sym_first_virtual_device] = ACTIONS(2045),
    [sym_first_local_port] = ACTIONS(2045),
    [sym_dynamic_virtual_device] = ACTIONS(2045),
    [sym_timeline_once] = ACTIONS(2045),
    [sym_timeline_repeat] = ACTIONS(2045),
    [sym_timeline_absolute] = ACTIONS(2045),
    [sym_timeline_relative] = ACTIONS(2045),
    [sym_do_push_timed_infinite] = ACTIONS(2045),
    [sym_source_type_no_address] = ACTIONS(2045),
    [sym_source_type_neuron_id] = ACTIONS(2045),
    [sym_source_type_ip_address] = ACTIONS(2045),
    [sym_source_type_axlink] = ACTIONS(2045),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2045),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2045),
    [sym_source_type_ip_socket_address] = ACTIONS(2045),
    [sym_source_type_rs232] = ACTIONS(2045),
    [sym_source_type_internal] = ACTIONS(2045),
    [sym_source_type_mac_address] = ACTIONS(2045),
    [sym_source_type_ipv4_port] = ACTIONS(2045),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2045),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2045),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2045),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2045),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2045),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2045),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2045),
    [sym_true] = ACTIONS(2045),
    [sym_false] = ACTIONS(2045),
    [sym_file_read_only] = ACTIONS(2045),
    [sym_file_rw_new] = ACTIONS(2045),
    [sym_file_rw_append] = ACTIONS(2045),
    [sym_ip_tcp] = ACTIONS(2045),
    [sym_ip_udp] = ACTIONS(2045),
    [sym_ip_udp_2way] = ACTIONS(2045),
    [sym_xml_encode_types] = ACTIONS(2045),
    [sym_xml_encode_char_as_list] = ACTIONS(2045),
    [sym_xml_encode_le] = ACTIONS(2045),
    [sym_xml_decode_types] = ACTIONS(2045),
    [sym_xml_decode_no_preserve] = ACTIONS(2045),
    [sym_duet_dev_type_display_device] = ACTIONS(2045),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2045),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2045),
    [sym_duet_dev_type_io_device] = ACTIONS(2045),
    [sym_duet_dev_type_relay_device] = ACTIONS(2045),
    [sym_duet_dev_type_ups] = ACTIONS(2045),
    [sym_duet_dev_type_amplifier] = ACTIONS(2045),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2045),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2045),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2045),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2045),
    [sym_duet_dev_type_camera] = ACTIONS(2045),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2045),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2045),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2045),
    [sym_duet_dev_type_dss] = ACTIONS(2045),
    [sym_duet_dev_type_dvr] = ACTIONS(2045),
    [sym_duet_dev_type_disc_device] = ACTIONS(2045),
    [sym_duet_dev_type_document_camera] = ACTIONS(2045),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2045),
    [sym_duet_dev_type_hvac] = ACTIONS(2045),
    [sym_duet_dev_type_keypad] = ACTIONS(2045),
    [sym_duet_dev_type_light] = ACTIONS(2045),
    [sym_duet_dev_type_monitor] = ACTIONS(2045),
    [sym_duet_dev_type_motor] = ACTIONS(2045),
    [sym_duet_dev_type_multi_window] = ACTIONS(2045),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2045),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2045),
    [sym_duet_dev_type_receiver] = ACTIONS(2045),
    [sym_duet_dev_type_security_system] = ACTIONS(2045),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2045),
    [sym_duet_dev_type_settop_box] = ACTIONS(2045),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2045),
    [sym_duet_dev_type_switcher] = ACTIONS(2045),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2045),
    [sym_duet_dev_type_tv] = ACTIONS(2045),
    [sym_duet_dev_type_utility] = ACTIONS(2045),
    [sym_duet_dev_type_vcr] = ACTIONS(2045),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2045),
    [sym_duet_dev_type_video_processor] = ACTIONS(2045),
    [sym_duet_dev_type_video_projector] = ACTIONS(2045),
    [sym_duet_dev_type_video_wall] = ACTIONS(2045),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2045),
    [sym_duet_dev_type_weather] = ACTIONS(2045),
    [sym_duet_dev_not_polled] = ACTIONS(2045),
    [sym_duet_dev_polled] = ACTIONS(2045),
    [sym_clkmgr_mode_network] = ACTIONS(2045),
    [sym_clkmgr_mode_standalone] = ACTIONS(2045),
    [sym_normal_standby] = ACTIONS(2045),
    [sym_normal_wake] = ACTIONS(2045),
    [sym_ip_multicast_ttl_option] = ACTIONS(2045),
    [sym_ip_ttl_subnet] = ACTIONS(2045),
    [sym_ip_ttl_site] = ACTIONS(2045),
    [sym_ip_ttl_region] = ACTIONS(2045),
    [sym_ip_ttl_continent] = ACTIONS(2045),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2045),
    [sym_ip_nodelay_on] = ACTIONS(2045),
    [sym_ip_nodelay_off] = ACTIONS(2045),
    [sym_amx_error] = ACTIONS(2045),
    [sym_amx_warning] = ACTIONS(2045),
    [sym_amx_info] = ACTIONS(2045),
    [sym_amx_debug] = ACTIONS(2045),
    [sym_netlinx_login_success] = ACTIONS(2045),
    [sym_netlinx_login_fail] = ACTIONS(2045),
    [sym_netlinx_logout] = ACTIONS(2045),
    [sym_null_str] = ACTIONS(2045),
    [sym_smtp_address] = ACTIONS(2045),
    [sym_smtp_port_number] = ACTIONS(2045),
    [sym_smtp_username] = ACTIONS(2045),
    [sym_smtp_password] = ACTIONS(2045),
    [sym_smtp_require_tls] = ACTIONS(2045),
    [sym_smtp_from] = ACTIONS(2045),
    [sym_smtp_tls_true] = ACTIONS(2045),
    [sym_smtp_tls_false] = ACTIONS(2045),
    [sym_listview_on_row_select_event] = ACTIONS(2045),
    [sym_data_structure_unknown] = ACTIONS(2045),
    [sym_data_structure_datafeed] = ACTIONS(2045),
    [sym_data_structure_datarecordset] = ACTIONS(2045),
    [sym_data_structure_datarecord] = ACTIONS(2045),
    [sym_data_structure_datafield] = ACTIONS(2045),
    [sym_data_max_content_field_count] = ACTIONS(2045),
    [sym_data_max_metadata_field_count] = ACTIONS(2045),
    [sym_data_max_id_length] = ACTIONS(2045),
    [sym_data_max_name_length] = ACTIONS(2045),
    [sym_data_max_description_length] = ACTIONS(2045),
    [sym_data_max_type_length] = ACTIONS(2045),
    [sym_data_max_format_length] = ACTIONS(2045),
    [sym_data_max_source_length] = ACTIONS(2045),
    [sym_data_max_value_length] = ACTIONS(2045),
    [sym_data_max_label_length] = ACTIONS(2045),
    [sym_data_max_url_length] = ACTIONS(2045),
    [sym_max_last_login_info_length] = ACTIONS(2045),
    [sym_data_type_unknown] = ACTIONS(2045),
    [sym_data_type_string] = ACTIONS(2045),
    [sym_data_type_datetime] = ACTIONS(2045),
    [sym_data_type_date] = ACTIONS(2045),
    [sym_data_type_time] = ACTIONS(2045),
    [sym_data_type_image] = ACTIONS(2045),
    [sym_data_format_url] = ACTIONS(2045),
    [sym_data_format_phone] = ACTIONS(2045),
    [sym_data_format_email] = ACTIONS(2045),
    [sym_data_format_iso8601] = ACTIONS(2045),
    [sym_auditlog_priv] = ACTIONS(2045),
    [sym_configuration_priv] = ACTIONS(2045),
    [sym_deviceconfig_priv] = ACTIONS(2045),
    [sym_ftp_priv] = ACTIONS(2045),
    [sym_http_priv] = ACTIONS(2045),
    [sym_networkconfig_priv] = ACTIONS(2045),
    [sym_programport_priv] = ACTIONS(2045),
    [sym_remoteui_priv] = ACTIONS(2045),
    [sym_securitycontrol_priv] = ACTIONS(2045),
    [sym_softwaremanagement_priv] = ACTIONS(2045),
    [sym_terminal_priv] = ACTIONS(2045),
    [sym_tpadmin_priv] = ACTIONS(2045),
    [sym_user1_priv] = ACTIONS(2045),
    [sym_user2_priv] = ACTIONS(2045),
    [sym_user3_priv] = ACTIONS(2045),
    [sym_user4_priv] = ACTIONS(2045),
    [sym_usermanagement_priv] = ACTIONS(2045),
    [sym_valid_account] = ACTIONS(2045),
    [sym_err_name_invalid] = ACTIONS(2045),
    [sym_err_password_invalid] = ACTIONS(2045),
    [sym_err_invalid_account] = ACTIONS(2045),
    [sym_err_no_authorization] = ACTIONS(2045),
    [sym_err_invalid_parameter] = ACTIONS(2045),
    [sym_err_user_locked_out] = ACTIONS(2045),
    [sym_err_user_account_expired] = ACTIONS(2045),
    [sym_err_general_error] = ACTIONS(2045),
    [sym_err_authentication_error] = ACTIONS(2045),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2045),
    [sym_tls_validate_certificate] = ACTIONS(2045),
    [sym_tdata] = ACTIONS(2045),
    [sym_tchannel] = ACTIONS(2045),
    [sym_tlevel] = ACTIONS(2045),
    [sym_tbutton] = ACTIONS(2045),
    [sym_ttimeline] = ACTIONS(2045),
    [sym_tcustom] = ACTIONS(2045),
    [sym_url_struct] = ACTIONS(2045),
    [sym_dns_struct] = ACTIONS(2045),
    [sym_ip_address_struct] = ACTIONS(2045),
    [sym_dev_info_struct] = ACTIONS(2045),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2045),
    [sym_data_feed] = ACTIONS(2045),
    [sym_data_field] = ACTIONS(2045),
    [sym_data_record] = ACTIONS(2045),
    [sym_wc_data_feed] = ACTIONS(2045),
    [sym_wc_data_field] = ACTIONS(2045),
    [sym_wc_data_record] = ACTIONS(2045),
    [sym_last_login_info] = ACTIONS(2045),
    [sym____reserved___] = ACTIONS(2045),
    [sym_date] = ACTIONS(2045),
    [sym_day] = ACTIONS(2045),
    [sym_time] = ACTIONS(2045),
    [sym_ldate] = ACTIONS(2045),
    [sym_push_device] = ACTIONS(2045),
    [sym_push_channel] = ACTIONS(2045),
    [sym_push_devchan] = ACTIONS(2045),
    [sym_release_device] = ACTIONS(2045),
    [sym_release_channel] = ACTIONS(2045),
    [sym_release_devchan] = ACTIONS(2045),
    [sym_master_slot] = ACTIONS(2045),
    [sym_get_pulse_time] = ACTIONS(2045),
    [sym_get_timer] = ACTIONS(2045),
    [sym_master_sn] = ACTIONS(2045),
    [sym_system_number] = ACTIONS(2045),
    [sym_button] = ACTIONS(2045),
    [sym_dv_channel] = ACTIONS(2045),
    [sym_data] = ACTIONS(2045),
    [sym_level] = ACTIONS(2045),
    [sym_timeline] = ACTIONS(2045),
    [sym_channel] = ACTIONS(2045),
    [sym_custom] = ACTIONS(2045),
    [sym_atoi] = ACTIONS(2045),
    [sym_atol] = ACTIONS(2045),
    [sym_atof] = ACTIONS(2045),
    [sym_itoa] = ACTIONS(2045),
    [sym_itohex] = ACTIONS(2045),
    [sym_hextoi] = ACTIONS(2045),
    [sym_ftoa] = ACTIONS(2045),
    [sym_find_string] = ACTIONS(2045),
    [sym_left_string] = ACTIONS(2045),
    [sym_right_string] = ACTIONS(2045),
    [sym_mid_string] = ACTIONS(2045),
    [sym_remove_string] = ACTIONS(2045),
    [sym_get_buffer_string] = ACTIONS(2045),
    [sym_get_buffer_char] = ACTIONS(2045),
    [sym_get_multi_buffer_string] = ACTIONS(2045),
    [sym_length_string] = ACTIONS(2045),
    [sym_max_length_string] = ACTIONS(2045),
    [sym_set_length_string] = ACTIONS(2045),
    [sym_compare_string] = ACTIONS(2045),
    [sym_upper_string] = ACTIONS(2045),
    [sym_lower_string] = ACTIONS(2045),
    [sym_get_last] = ACTIONS(2045),
    [sym_date_to_day] = ACTIONS(2045),
    [sym_date_to_month] = ACTIONS(2045),
    [sym_date_to_year] = ACTIONS(2045),
    [sym_time_to_hour] = ACTIONS(2045),
    [sym_time_to_minute] = ACTIONS(2045),
    [sym_time_to_second] = ACTIONS(2045),
    [sym_day_of_week] = ACTIONS(2045),
    [sym_set_pulse_time] = ACTIONS(2045),
    [sym_set_timer] = ACTIONS(2045),
    [sym_file_open] = ACTIONS(2045),
    [sym_file_close] = ACTIONS(2045),
    [sym_file_read] = ACTIONS(2045),
    [sym_file_read_line] = ACTIONS(2045),
    [sym_file_write] = ACTIONS(2045),
    [sym_file_write_line] = ACTIONS(2045),
    [sym_file_copy] = ACTIONS(2045),
    [sym_file_delete] = ACTIONS(2045),
    [sym_file_rename] = ACTIONS(2045),
    [sym_file_dir] = ACTIONS(2045),
    [sym_file_getdir] = ACTIONS(2045),
    [sym_file_setdir] = ACTIONS(2045),
    [sym_file_create_dir] = ACTIONS(2045),
    [sym_file_remove_dir] = ACTIONS(2045),
    [sym_file_seek] = ACTIONS(2045),
    [sym_length_array] = ACTIONS(2045),
    [sym_max_length_array] = ACTIONS(2045),
    [sym_set_length_array] = ACTIONS(2045),
    [sym_ip_client_open] = ACTIONS(2045),
    [sym_ip_client_close] = ACTIONS(2045),
    [sym_ip_server_open] = ACTIONS(2045),
    [sym_ip_server_close] = ACTIONS(2045),
    [sym_ip_bound_client_open] = ACTIONS(2045),
    [sym_ip_mc_server_open] = ACTIONS(2045),
    [sym_ip_set_option] = ACTIONS(2045),
    [sym_ssh_client_open] = ACTIONS(2045),
    [sym_ssh_client_close] = ACTIONS(2045),
    [sym_tls_client_open] = ACTIONS(2045),
    [sym_tls_client_close] = ACTIONS(2045),
    [sym_do_push] = ACTIONS(2045),
    [sym_do_push_timed] = ACTIONS(2045),
    [sym_do_release] = ACTIONS(2045),
    [sym_device_id] = ACTIONS(2045),
    [sym_device_info] = ACTIONS(2045),
    [sym_device_id_string] = ACTIONS(2045),
    [sym_device_standby] = ACTIONS(2045),
    [sym_device_wake] = ACTIONS(2045),
    [sym_timeline_create] = ACTIONS(2045),
    [sym_timeline_kill] = ACTIONS(2045),
    [sym_timeline_reload] = ACTIONS(2045),
    [sym_timeline_pause] = ACTIONS(2045),
    [sym_timeline_restart] = ACTIONS(2045),
    [sym_timeline_set] = ACTIONS(2045),
    [sym_timeline_get] = ACTIONS(2045),
    [sym_timeline_active] = ACTIONS(2045),
    [sym_timeline_dynamic_id] = ACTIONS(2045),
    [sym_variable_to_string] = ACTIONS(2045),
    [sym_string_to_variable] = ACTIONS(2045),
    [sym_length_variable_to_string] = ACTIONS(2045),
    [sym_type_cast] = ACTIONS(2045),
    [sym_variable_to_xml] = ACTIONS(2045),
    [sym_xml_to_variable] = ACTIONS(2045),
    [sym_length_variable_to_xml] = ACTIONS(2045),
    [sym_redirect_string] = ACTIONS(2045),
    [sym_random_number] = ACTIONS(2045),
    [sym_get_unique_id] = ACTIONS(2045),
    [sym_get_system_number] = ACTIONS(2045),
    [sym_set_system_number] = ACTIONS(2045),
    [sym_get_serial_number] = ACTIONS(2045),
    [sym_get_master_build] = ACTIONS(2045),
    [sym_set_validation_code] = ACTIONS(2045),
    [sym_reboot] = ACTIONS(2045),
    [sym_load_duet_module] = ACTIONS(2045),
    [sym_unload_duet_module] = ACTIONS(2045),
    [sym_duet_mem_size_set] = ACTIONS(2045),
    [sym_duet_mem_size_get] = ACTIONS(2045),
    [sym_dynamic_application_device] = ACTIONS(2045),
    [sym_dynamic_polled_port] = ACTIONS(2045),
    [sym_static_port_binding] = ACTIONS(2045),
    [sym_static_ip_binding] = ACTIONS(2045),
    [sym_get_dns_list] = ACTIONS(2045),
    [sym_set_dns_list] = ACTIONS(2045),
    [sym_get_ip_address] = ACTIONS(2045),
    [sym_set_ip_address] = ACTIONS(2045),
    [sym_get_url_list] = ACTIONS(2045),
    [sym_add_url_entry] = ACTIONS(2045),
    [sym_delete_url_entry] = ACTIONS(2045),
    [sym_abs_value] = ACTIONS(2045),
    [sym_max_value] = ACTIONS(2045),
    [sym_min_value] = ACTIONS(2045),
    [sym_exp_value] = ACTIONS(2045),
    [sym_log_value] = ACTIONS(2045),
    [sym_log10_value] = ACTIONS(2045),
    [sym_power_value] = ACTIONS(2045),
    [sym_sqrt_value] = ACTIONS(2045),
    [sym_raw_be] = ACTIONS(2045),
    [sym_raw_le] = ACTIONS(2045),
    [sym_internal_queue_size_set] = ACTIONS(2045),
    [sym_internal_queue_size_get] = ACTIONS(2045),
    [sym_internal_threshold_set] = ACTIONS(2045),
    [sym_internal_threshold_get] = ACTIONS(2045),
    [sym_rebuilt_event] = ACTIONS(2045),
    [sym_do_custom_event] = ACTIONS(2045),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2045),
    [sym_clkmgr_set_clk_source] = ACTIONS(2045),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2045),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2045),
    [sym_clkmgr_get_timezone] = ACTIONS(2045),
    [sym_clkmgr_set_timezone] = ACTIONS(2045),
    [sym_clkmgr_get_resync_period] = ACTIONS(2045),
    [sym_clkmgr_set_resync_period] = ACTIONS(2045),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2045),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2045),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2045),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2045),
    [sym_clkmgr_get_timeservers] = ACTIONS(2045),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2045),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2045),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2045),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2045),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2045),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2045),
    [sym_set_log_level] = ACTIONS(2045),
    [sym_get_log_level] = ACTIONS(2045),
    [sym_amx_log] = ACTIONS(2045),
    [sym_smtp_server_config_set] = ACTIONS(2045),
    [sym_smtp_server_config_get] = ACTIONS(2045),
    [sym_smtp_send] = ACTIONS(2045),
    [sym_validate_netlinx_account] = ACTIONS(2045),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2045),
    [sym_authenticate_certificate] = ACTIONS(2045),
    [sym_audit_netlinx_session_event] = ACTIONS(2045),
    [sym_audit_netlinx_generic_event] = ACTIONS(2045),
    [sym_get_available_flash_disk_space] = ACTIONS(2045),
    [sym_get_max_flash_disk_space] = ACTIONS(2045),
    [sym_data_create_feed] = ACTIONS(2045),
    [sym_data_delete_feed] = ACTIONS(2045),
    [sym_data_publish_feed] = ACTIONS(2045),
    [sym_data_get_published_feed] = ACTIONS(2045),
    [sym_data_add_record] = ACTIONS(2045),
    [sym_data_get_event_record] = ACTIONS(2045),
    [sym__wc_data_create_feed] = ACTIONS(2045),
    [sym__wc_data_add_record] = ACTIONS(2045),
    [sym__wc_data_get_event_record] = ACTIONS(2045),
    [sym_format] = ACTIONS(2045),
    [sym_astro_clock] = ACTIONS(2045),
    [sym_set_outdoor_temperature] = ACTIONS(2045),
    [sym_set_virtual_level_count] = ACTIONS(2045),
    [sym_set_virtual_channel_count] = ACTIONS(2045),
    [sym_set_virtual_port_count] = ACTIONS(2045),
    [sym__wc_encode_int] = ACTIONS(2045),
    [sym__wc_decode_int] = ACTIONS(2045),
    [sym__wc_int] = ACTIONS(2045),
    [sym__wc_to_ch_int] = ACTIONS(2045),
    [sym__ch_to_wc_int] = ACTIONS(2045),
    [sym__wc_find_string_int] = ACTIONS(2045),
    [sym__wc_left_string_int] = ACTIONS(2045),
    [sym__wc_lower_string_int] = ACTIONS(2045),
    [sym__wc_mid_string_int] = ACTIONS(2045),
    [sym__wc_remove_string_int] = ACTIONS(2045),
    [sym__wc_right_string_int] = ACTIONS(2045),
    [sym__wc_upper_string_int] = ACTIONS(2045),
    [sym__wc_compare_string_int] = ACTIONS(2045),
    [sym__wc_get_buffer_char_int] = ACTIONS(2045),
    [sym__wc_get_buffer_string_int] = ACTIONS(2045),
    [sym__wc_concat_string_int] = ACTIONS(2045),
    [sym__wc_file_open_int] = ACTIONS(2045),
    [sym__wc_file_close_int] = ACTIONS(2045),
    [sym__wc_file_read_int] = ACTIONS(2045),
    [sym__wc_file_read_line_int] = ACTIONS(2045),
    [sym__wc_file_write_int] = ACTIONS(2045),
    [sym__wc_file_write_line_int] = ACTIONS(2045),
    [aux_sym_preproc_include_token1] = ACTIONS(2045),
    [aux_sym_preproc_define_token1] = ACTIONS(2047),
    [aux_sym_preproc_warn_token1] = ACTIONS(2047),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2047),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2047),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2047),
    [anon_sym_LBRACK] = ACTIONS(2047),
    [anon_sym_LBRACE] = ACTIONS(2047),
    [anon_sym_RBRACE] = ACTIONS(2047),
    [anon_sym_LPAREN] = ACTIONS(2045),
    [anon_sym_BANG] = ACTIONS(2047),
    [anon_sym_TILDE] = ACTIONS(2047),
    [anon_sym_DASH] = ACTIONS(2045),
    [anon_sym_PLUS] = ACTIONS(2045),
    [anon_sym_DASH_DASH] = ACTIONS(2047),
    [anon_sym_PLUS_PLUS] = ACTIONS(2047),
    [anon_sym_DQUOTE] = ACTIONS(2047),
    [anon_sym_SQUOTE] = ACTIONS(2047),
    [sym_number_literal] = ACTIONS(2047),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(543)] = {
    [sym_identifier] = ACTIONS(2049),
    [sym_define_call_keyword] = ACTIONS(2049),
    [sym_define_function_keyword] = ACTIONS(2049),
    [sym_define_library_function_keyword] = ACTIONS(2049),
    [sym_define_module_keyword] = ACTIONS(2049),
    [sym_char_keyword] = ACTIONS(2049),
    [sym_widechar_keyword] = ACTIONS(2049),
    [sym_integer_keyword] = ACTIONS(2049),
    [sym_sinteger_keyword] = ACTIONS(2049),
    [sym_long_keyword] = ACTIONS(2049),
    [sym_slong_keyword] = ACTIONS(2049),
    [sym_float_keyword] = ACTIONS(2049),
    [sym_double_keyword] = ACTIONS(2049),
    [sym_variant_keyword] = ACTIONS(2049),
    [sym_variantarray_keyword] = ACTIONS(2049),
    [sym_local_var_keyword] = ACTIONS(2049),
    [sym_stack_var_keyword] = ACTIONS(2049),
    [sym_constant_keyword] = ACTIONS(2049),
    [sym_volatile_keyword] = ACTIONS(2049),
    [sym_non_volatile_keyword] = ACTIONS(2049),
    [sym_persistent_keyword] = ACTIONS(2049),
    [sym_dev_keyword] = ACTIONS(2049),
    [sym_devlev_keyword] = ACTIONS(2049),
    [sym_devchan_keyword] = ACTIONS(2049),
    [sym_if_keyword] = ACTIONS(2049),
    [sym_switch_keyword] = ACTIONS(2049),
    [sym_case_keyword] = ACTIONS(2049),
    [sym_default_keyword] = ACTIONS(2049),
    [sym_while_keyword] = ACTIONS(2049),
    [sym_for_keyword] = ACTIONS(2049),
    [sym_break_keyword] = ACTIONS(2049),
    [sym_continue_keyword] = ACTIONS(2049),
    [sym_return_keyword] = ACTIONS(2049),
    [sym_select_keyword] = ACTIONS(2049),
    [sym_active_keyword] = ACTIONS(2049),
    [sym_struct_keyword] = ACTIONS(2049),
    [sym_structure_keyword] = ACTIONS(2049),
    [aux_sym_on_keyword_token1] = ACTIONS(2049),
    [aux_sym_off_keyword_token1] = ACTIONS(2049),
    [sym_send_level_keyword] = ACTIONS(2049),
    [sym_send_string_keyword] = ACTIONS(2049),
    [sym_send_command_keyword] = ACTIONS(2049),
    [sym_clear_buffer_keyword] = ACTIONS(2049),
    [sym_create_buffer_keyword] = ACTIONS(2049),
    [sym_create_multi_buffer_keyword] = ACTIONS(2049),
    [sym_call_keyword] = ACTIONS(2049),
    [sym_system_call_keyword] = ACTIONS(2049),
    [sym_devchan_to_keyword] = ACTIONS(2049),
    [sym_devchan_min_to_keyword] = ACTIONS(2049),
    [sym_devchan_total_off_keyword] = ACTIONS(2049),
    [sym_devchan_pulse_keyword] = ACTIONS(2049),
    [sym_wait_keyword] = ACTIONS(2049),
    [sym_cancel_wait_keyword] = ACTIONS(2049),
    [sym_wait_until_keyword] = ACTIONS(2049),
    [sym_cancel_wait_until_keyword] = ACTIONS(2049),
    [sym_cancel_all_wait_keyword] = ACTIONS(2049),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2049),
    [sym_not] = ACTIONS(2049),
    [sym_bnot] = ACTIONS(2049),
    [sym___file__] = ACTIONS(2049),
    [sym___date__] = ACTIONS(2049),
    [sym___ldate__] = ACTIONS(2049),
    [sym___time__] = ACTIONS(2049),
    [sym___line__] = ACTIONS(2049),
    [sym___name__] = ACTIONS(2049),
    [sym___netlinx__] = ACTIONS(2049),
    [sym_netlinx_axi_version] = ACTIONS(2049),
    [sym_url_flg_tcp] = ACTIONS(2049),
    [sym_url_flg_accinfopresent] = ACTIONS(2049),
    [sym_url_flg_temp] = ACTIONS(2049),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2049),
    [sym_url_flg_stat_mask] = ACTIONS(2049),
    [sym_url_flg_stat_lookup] = ACTIONS(2049),
    [sym_url_flg_stat_connecting] = ACTIONS(2049),
    [sym_url_flg_stat_waiting] = ACTIONS(2049),
    [sym_url_flg_stat_connected] = ACTIONS(2049),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2049),
    [sym_first_virtual_device] = ACTIONS(2049),
    [sym_first_local_port] = ACTIONS(2049),
    [sym_dynamic_virtual_device] = ACTIONS(2049),
    [sym_timeline_once] = ACTIONS(2049),
    [sym_timeline_repeat] = ACTIONS(2049),
    [sym_timeline_absolute] = ACTIONS(2049),
    [sym_timeline_relative] = ACTIONS(2049),
    [sym_do_push_timed_infinite] = ACTIONS(2049),
    [sym_source_type_no_address] = ACTIONS(2049),
    [sym_source_type_neuron_id] = ACTIONS(2049),
    [sym_source_type_ip_address] = ACTIONS(2049),
    [sym_source_type_axlink] = ACTIONS(2049),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2049),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2049),
    [sym_source_type_ip_socket_address] = ACTIONS(2049),
    [sym_source_type_rs232] = ACTIONS(2049),
    [sym_source_type_internal] = ACTIONS(2049),
    [sym_source_type_mac_address] = ACTIONS(2049),
    [sym_source_type_ipv4_port] = ACTIONS(2049),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2049),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2049),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2049),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2049),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2049),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2049),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2049),
    [sym_true] = ACTIONS(2049),
    [sym_false] = ACTIONS(2049),
    [sym_file_read_only] = ACTIONS(2049),
    [sym_file_rw_new] = ACTIONS(2049),
    [sym_file_rw_append] = ACTIONS(2049),
    [sym_ip_tcp] = ACTIONS(2049),
    [sym_ip_udp] = ACTIONS(2049),
    [sym_ip_udp_2way] = ACTIONS(2049),
    [sym_xml_encode_types] = ACTIONS(2049),
    [sym_xml_encode_char_as_list] = ACTIONS(2049),
    [sym_xml_encode_le] = ACTIONS(2049),
    [sym_xml_decode_types] = ACTIONS(2049),
    [sym_xml_decode_no_preserve] = ACTIONS(2049),
    [sym_duet_dev_type_display_device] = ACTIONS(2049),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2049),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2049),
    [sym_duet_dev_type_io_device] = ACTIONS(2049),
    [sym_duet_dev_type_relay_device] = ACTIONS(2049),
    [sym_duet_dev_type_ups] = ACTIONS(2049),
    [sym_duet_dev_type_amplifier] = ACTIONS(2049),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2049),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2049),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2049),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2049),
    [sym_duet_dev_type_camera] = ACTIONS(2049),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2049),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2049),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2049),
    [sym_duet_dev_type_dss] = ACTIONS(2049),
    [sym_duet_dev_type_dvr] = ACTIONS(2049),
    [sym_duet_dev_type_disc_device] = ACTIONS(2049),
    [sym_duet_dev_type_document_camera] = ACTIONS(2049),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2049),
    [sym_duet_dev_type_hvac] = ACTIONS(2049),
    [sym_duet_dev_type_keypad] = ACTIONS(2049),
    [sym_duet_dev_type_light] = ACTIONS(2049),
    [sym_duet_dev_type_monitor] = ACTIONS(2049),
    [sym_duet_dev_type_motor] = ACTIONS(2049),
    [sym_duet_dev_type_multi_window] = ACTIONS(2049),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2049),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2049),
    [sym_duet_dev_type_receiver] = ACTIONS(2049),
    [sym_duet_dev_type_security_system] = ACTIONS(2049),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2049),
    [sym_duet_dev_type_settop_box] = ACTIONS(2049),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2049),
    [sym_duet_dev_type_switcher] = ACTIONS(2049),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2049),
    [sym_duet_dev_type_tv] = ACTIONS(2049),
    [sym_duet_dev_type_utility] = ACTIONS(2049),
    [sym_duet_dev_type_vcr] = ACTIONS(2049),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2049),
    [sym_duet_dev_type_video_processor] = ACTIONS(2049),
    [sym_duet_dev_type_video_projector] = ACTIONS(2049),
    [sym_duet_dev_type_video_wall] = ACTIONS(2049),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2049),
    [sym_duet_dev_type_weather] = ACTIONS(2049),
    [sym_duet_dev_not_polled] = ACTIONS(2049),
    [sym_duet_dev_polled] = ACTIONS(2049),
    [sym_clkmgr_mode_network] = ACTIONS(2049),
    [sym_clkmgr_mode_standalone] = ACTIONS(2049),
    [sym_normal_standby] = ACTIONS(2049),
    [sym_normal_wake] = ACTIONS(2049),
    [sym_ip_multicast_ttl_option] = ACTIONS(2049),
    [sym_ip_ttl_subnet] = ACTIONS(2049),
    [sym_ip_ttl_site] = ACTIONS(2049),
    [sym_ip_ttl_region] = ACTIONS(2049),
    [sym_ip_ttl_continent] = ACTIONS(2049),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2049),
    [sym_ip_nodelay_on] = ACTIONS(2049),
    [sym_ip_nodelay_off] = ACTIONS(2049),
    [sym_amx_error] = ACTIONS(2049),
    [sym_amx_warning] = ACTIONS(2049),
    [sym_amx_info] = ACTIONS(2049),
    [sym_amx_debug] = ACTIONS(2049),
    [sym_netlinx_login_success] = ACTIONS(2049),
    [sym_netlinx_login_fail] = ACTIONS(2049),
    [sym_netlinx_logout] = ACTIONS(2049),
    [sym_null_str] = ACTIONS(2049),
    [sym_smtp_address] = ACTIONS(2049),
    [sym_smtp_port_number] = ACTIONS(2049),
    [sym_smtp_username] = ACTIONS(2049),
    [sym_smtp_password] = ACTIONS(2049),
    [sym_smtp_require_tls] = ACTIONS(2049),
    [sym_smtp_from] = ACTIONS(2049),
    [sym_smtp_tls_true] = ACTIONS(2049),
    [sym_smtp_tls_false] = ACTIONS(2049),
    [sym_listview_on_row_select_event] = ACTIONS(2049),
    [sym_data_structure_unknown] = ACTIONS(2049),
    [sym_data_structure_datafeed] = ACTIONS(2049),
    [sym_data_structure_datarecordset] = ACTIONS(2049),
    [sym_data_structure_datarecord] = ACTIONS(2049),
    [sym_data_structure_datafield] = ACTIONS(2049),
    [sym_data_max_content_field_count] = ACTIONS(2049),
    [sym_data_max_metadata_field_count] = ACTIONS(2049),
    [sym_data_max_id_length] = ACTIONS(2049),
    [sym_data_max_name_length] = ACTIONS(2049),
    [sym_data_max_description_length] = ACTIONS(2049),
    [sym_data_max_type_length] = ACTIONS(2049),
    [sym_data_max_format_length] = ACTIONS(2049),
    [sym_data_max_source_length] = ACTIONS(2049),
    [sym_data_max_value_length] = ACTIONS(2049),
    [sym_data_max_label_length] = ACTIONS(2049),
    [sym_data_max_url_length] = ACTIONS(2049),
    [sym_max_last_login_info_length] = ACTIONS(2049),
    [sym_data_type_unknown] = ACTIONS(2049),
    [sym_data_type_string] = ACTIONS(2049),
    [sym_data_type_datetime] = ACTIONS(2049),
    [sym_data_type_date] = ACTIONS(2049),
    [sym_data_type_time] = ACTIONS(2049),
    [sym_data_type_image] = ACTIONS(2049),
    [sym_data_format_url] = ACTIONS(2049),
    [sym_data_format_phone] = ACTIONS(2049),
    [sym_data_format_email] = ACTIONS(2049),
    [sym_data_format_iso8601] = ACTIONS(2049),
    [sym_auditlog_priv] = ACTIONS(2049),
    [sym_configuration_priv] = ACTIONS(2049),
    [sym_deviceconfig_priv] = ACTIONS(2049),
    [sym_ftp_priv] = ACTIONS(2049),
    [sym_http_priv] = ACTIONS(2049),
    [sym_networkconfig_priv] = ACTIONS(2049),
    [sym_programport_priv] = ACTIONS(2049),
    [sym_remoteui_priv] = ACTIONS(2049),
    [sym_securitycontrol_priv] = ACTIONS(2049),
    [sym_softwaremanagement_priv] = ACTIONS(2049),
    [sym_terminal_priv] = ACTIONS(2049),
    [sym_tpadmin_priv] = ACTIONS(2049),
    [sym_user1_priv] = ACTIONS(2049),
    [sym_user2_priv] = ACTIONS(2049),
    [sym_user3_priv] = ACTIONS(2049),
    [sym_user4_priv] = ACTIONS(2049),
    [sym_usermanagement_priv] = ACTIONS(2049),
    [sym_valid_account] = ACTIONS(2049),
    [sym_err_name_invalid] = ACTIONS(2049),
    [sym_err_password_invalid] = ACTIONS(2049),
    [sym_err_invalid_account] = ACTIONS(2049),
    [sym_err_no_authorization] = ACTIONS(2049),
    [sym_err_invalid_parameter] = ACTIONS(2049),
    [sym_err_user_locked_out] = ACTIONS(2049),
    [sym_err_user_account_expired] = ACTIONS(2049),
    [sym_err_general_error] = ACTIONS(2049),
    [sym_err_authentication_error] = ACTIONS(2049),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2049),
    [sym_tls_validate_certificate] = ACTIONS(2049),
    [sym_tdata] = ACTIONS(2049),
    [sym_tchannel] = ACTIONS(2049),
    [sym_tlevel] = ACTIONS(2049),
    [sym_tbutton] = ACTIONS(2049),
    [sym_ttimeline] = ACTIONS(2049),
    [sym_tcustom] = ACTIONS(2049),
    [sym_url_struct] = ACTIONS(2049),
    [sym_dns_struct] = ACTIONS(2049),
    [sym_ip_address_struct] = ACTIONS(2049),
    [sym_dev_info_struct] = ACTIONS(2049),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2049),
    [sym_data_feed] = ACTIONS(2049),
    [sym_data_field] = ACTIONS(2049),
    [sym_data_record] = ACTIONS(2049),
    [sym_wc_data_feed] = ACTIONS(2049),
    [sym_wc_data_field] = ACTIONS(2049),
    [sym_wc_data_record] = ACTIONS(2049),
    [sym_last_login_info] = ACTIONS(2049),
    [sym____reserved___] = ACTIONS(2049),
    [sym_date] = ACTIONS(2049),
    [sym_day] = ACTIONS(2049),
    [sym_time] = ACTIONS(2049),
    [sym_ldate] = ACTIONS(2049),
    [sym_push_device] = ACTIONS(2049),
    [sym_push_channel] = ACTIONS(2049),
    [sym_push_devchan] = ACTIONS(2049),
    [sym_release_device] = ACTIONS(2049),
    [sym_release_channel] = ACTIONS(2049),
    [sym_release_devchan] = ACTIONS(2049),
    [sym_master_slot] = ACTIONS(2049),
    [sym_get_pulse_time] = ACTIONS(2049),
    [sym_get_timer] = ACTIONS(2049),
    [sym_master_sn] = ACTIONS(2049),
    [sym_system_number] = ACTIONS(2049),
    [sym_button] = ACTIONS(2049),
    [sym_dv_channel] = ACTIONS(2049),
    [sym_data] = ACTIONS(2049),
    [sym_level] = ACTIONS(2049),
    [sym_timeline] = ACTIONS(2049),
    [sym_channel] = ACTIONS(2049),
    [sym_custom] = ACTIONS(2049),
    [sym_atoi] = ACTIONS(2049),
    [sym_atol] = ACTIONS(2049),
    [sym_atof] = ACTIONS(2049),
    [sym_itoa] = ACTIONS(2049),
    [sym_itohex] = ACTIONS(2049),
    [sym_hextoi] = ACTIONS(2049),
    [sym_ftoa] = ACTIONS(2049),
    [sym_find_string] = ACTIONS(2049),
    [sym_left_string] = ACTIONS(2049),
    [sym_right_string] = ACTIONS(2049),
    [sym_mid_string] = ACTIONS(2049),
    [sym_remove_string] = ACTIONS(2049),
    [sym_get_buffer_string] = ACTIONS(2049),
    [sym_get_buffer_char] = ACTIONS(2049),
    [sym_get_multi_buffer_string] = ACTIONS(2049),
    [sym_length_string] = ACTIONS(2049),
    [sym_max_length_string] = ACTIONS(2049),
    [sym_set_length_string] = ACTIONS(2049),
    [sym_compare_string] = ACTIONS(2049),
    [sym_upper_string] = ACTIONS(2049),
    [sym_lower_string] = ACTIONS(2049),
    [sym_get_last] = ACTIONS(2049),
    [sym_date_to_day] = ACTIONS(2049),
    [sym_date_to_month] = ACTIONS(2049),
    [sym_date_to_year] = ACTIONS(2049),
    [sym_time_to_hour] = ACTIONS(2049),
    [sym_time_to_minute] = ACTIONS(2049),
    [sym_time_to_second] = ACTIONS(2049),
    [sym_day_of_week] = ACTIONS(2049),
    [sym_set_pulse_time] = ACTIONS(2049),
    [sym_set_timer] = ACTIONS(2049),
    [sym_file_open] = ACTIONS(2049),
    [sym_file_close] = ACTIONS(2049),
    [sym_file_read] = ACTIONS(2049),
    [sym_file_read_line] = ACTIONS(2049),
    [sym_file_write] = ACTIONS(2049),
    [sym_file_write_line] = ACTIONS(2049),
    [sym_file_copy] = ACTIONS(2049),
    [sym_file_delete] = ACTIONS(2049),
    [sym_file_rename] = ACTIONS(2049),
    [sym_file_dir] = ACTIONS(2049),
    [sym_file_getdir] = ACTIONS(2049),
    [sym_file_setdir] = ACTIONS(2049),
    [sym_file_create_dir] = ACTIONS(2049),
    [sym_file_remove_dir] = ACTIONS(2049),
    [sym_file_seek] = ACTIONS(2049),
    [sym_length_array] = ACTIONS(2049),
    [sym_max_length_array] = ACTIONS(2049),
    [sym_set_length_array] = ACTIONS(2049),
    [sym_ip_client_open] = ACTIONS(2049),
    [sym_ip_client_close] = ACTIONS(2049),
    [sym_ip_server_open] = ACTIONS(2049),
    [sym_ip_server_close] = ACTIONS(2049),
    [sym_ip_bound_client_open] = ACTIONS(2049),
    [sym_ip_mc_server_open] = ACTIONS(2049),
    [sym_ip_set_option] = ACTIONS(2049),
    [sym_ssh_client_open] = ACTIONS(2049),
    [sym_ssh_client_close] = ACTIONS(2049),
    [sym_tls_client_open] = ACTIONS(2049),
    [sym_tls_client_close] = ACTIONS(2049),
    [sym_do_push] = ACTIONS(2049),
    [sym_do_push_timed] = ACTIONS(2049),
    [sym_do_release] = ACTIONS(2049),
    [sym_device_id] = ACTIONS(2049),
    [sym_device_info] = ACTIONS(2049),
    [sym_device_id_string] = ACTIONS(2049),
    [sym_device_standby] = ACTIONS(2049),
    [sym_device_wake] = ACTIONS(2049),
    [sym_timeline_create] = ACTIONS(2049),
    [sym_timeline_kill] = ACTIONS(2049),
    [sym_timeline_reload] = ACTIONS(2049),
    [sym_timeline_pause] = ACTIONS(2049),
    [sym_timeline_restart] = ACTIONS(2049),
    [sym_timeline_set] = ACTIONS(2049),
    [sym_timeline_get] = ACTIONS(2049),
    [sym_timeline_active] = ACTIONS(2049),
    [sym_timeline_dynamic_id] = ACTIONS(2049),
    [sym_variable_to_string] = ACTIONS(2049),
    [sym_string_to_variable] = ACTIONS(2049),
    [sym_length_variable_to_string] = ACTIONS(2049),
    [sym_type_cast] = ACTIONS(2049),
    [sym_variable_to_xml] = ACTIONS(2049),
    [sym_xml_to_variable] = ACTIONS(2049),
    [sym_length_variable_to_xml] = ACTIONS(2049),
    [sym_redirect_string] = ACTIONS(2049),
    [sym_random_number] = ACTIONS(2049),
    [sym_get_unique_id] = ACTIONS(2049),
    [sym_get_system_number] = ACTIONS(2049),
    [sym_set_system_number] = ACTIONS(2049),
    [sym_get_serial_number] = ACTIONS(2049),
    [sym_get_master_build] = ACTIONS(2049),
    [sym_set_validation_code] = ACTIONS(2049),
    [sym_reboot] = ACTIONS(2049),
    [sym_load_duet_module] = ACTIONS(2049),
    [sym_unload_duet_module] = ACTIONS(2049),
    [sym_duet_mem_size_set] = ACTIONS(2049),
    [sym_duet_mem_size_get] = ACTIONS(2049),
    [sym_dynamic_application_device] = ACTIONS(2049),
    [sym_dynamic_polled_port] = ACTIONS(2049),
    [sym_static_port_binding] = ACTIONS(2049),
    [sym_static_ip_binding] = ACTIONS(2049),
    [sym_get_dns_list] = ACTIONS(2049),
    [sym_set_dns_list] = ACTIONS(2049),
    [sym_get_ip_address] = ACTIONS(2049),
    [sym_set_ip_address] = ACTIONS(2049),
    [sym_get_url_list] = ACTIONS(2049),
    [sym_add_url_entry] = ACTIONS(2049),
    [sym_delete_url_entry] = ACTIONS(2049),
    [sym_abs_value] = ACTIONS(2049),
    [sym_max_value] = ACTIONS(2049),
    [sym_min_value] = ACTIONS(2049),
    [sym_exp_value] = ACTIONS(2049),
    [sym_log_value] = ACTIONS(2049),
    [sym_log10_value] = ACTIONS(2049),
    [sym_power_value] = ACTIONS(2049),
    [sym_sqrt_value] = ACTIONS(2049),
    [sym_raw_be] = ACTIONS(2049),
    [sym_raw_le] = ACTIONS(2049),
    [sym_internal_queue_size_set] = ACTIONS(2049),
    [sym_internal_queue_size_get] = ACTIONS(2049),
    [sym_internal_threshold_set] = ACTIONS(2049),
    [sym_internal_threshold_get] = ACTIONS(2049),
    [sym_rebuilt_event] = ACTIONS(2049),
    [sym_do_custom_event] = ACTIONS(2049),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2049),
    [sym_clkmgr_set_clk_source] = ACTIONS(2049),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2049),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2049),
    [sym_clkmgr_get_timezone] = ACTIONS(2049),
    [sym_clkmgr_set_timezone] = ACTIONS(2049),
    [sym_clkmgr_get_resync_period] = ACTIONS(2049),
    [sym_clkmgr_set_resync_period] = ACTIONS(2049),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2049),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2049),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2049),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2049),
    [sym_clkmgr_get_timeservers] = ACTIONS(2049),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2049),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2049),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2049),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2049),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2049),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2049),
    [sym_set_log_level] = ACTIONS(2049),
    [sym_get_log_level] = ACTIONS(2049),
    [sym_amx_log] = ACTIONS(2049),
    [sym_smtp_server_config_set] = ACTIONS(2049),
    [sym_smtp_server_config_get] = ACTIONS(2049),
    [sym_smtp_send] = ACTIONS(2049),
    [sym_validate_netlinx_account] = ACTIONS(2049),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2049),
    [sym_authenticate_certificate] = ACTIONS(2049),
    [sym_audit_netlinx_session_event] = ACTIONS(2049),
    [sym_audit_netlinx_generic_event] = ACTIONS(2049),
    [sym_get_available_flash_disk_space] = ACTIONS(2049),
    [sym_get_max_flash_disk_space] = ACTIONS(2049),
    [sym_data_create_feed] = ACTIONS(2049),
    [sym_data_delete_feed] = ACTIONS(2049),
    [sym_data_publish_feed] = ACTIONS(2049),
    [sym_data_get_published_feed] = ACTIONS(2049),
    [sym_data_add_record] = ACTIONS(2049),
    [sym_data_get_event_record] = ACTIONS(2049),
    [sym__wc_data_create_feed] = ACTIONS(2049),
    [sym__wc_data_add_record] = ACTIONS(2049),
    [sym__wc_data_get_event_record] = ACTIONS(2049),
    [sym_format] = ACTIONS(2049),
    [sym_astro_clock] = ACTIONS(2049),
    [sym_set_outdoor_temperature] = ACTIONS(2049),
    [sym_set_virtual_level_count] = ACTIONS(2049),
    [sym_set_virtual_channel_count] = ACTIONS(2049),
    [sym_set_virtual_port_count] = ACTIONS(2049),
    [sym__wc_encode_int] = ACTIONS(2049),
    [sym__wc_decode_int] = ACTIONS(2049),
    [sym__wc_int] = ACTIONS(2049),
    [sym__wc_to_ch_int] = ACTIONS(2049),
    [sym__ch_to_wc_int] = ACTIONS(2049),
    [sym__wc_find_string_int] = ACTIONS(2049),
    [sym__wc_left_string_int] = ACTIONS(2049),
    [sym__wc_lower_string_int] = ACTIONS(2049),
    [sym__wc_mid_string_int] = ACTIONS(2049),
    [sym__wc_remove_string_int] = ACTIONS(2049),
    [sym__wc_right_string_int] = ACTIONS(2049),
    [sym__wc_upper_string_int] = ACTIONS(2049),
    [sym__wc_compare_string_int] = ACTIONS(2049),
    [sym__wc_get_buffer_char_int] = ACTIONS(2049),
    [sym__wc_get_buffer_string_int] = ACTIONS(2049),
    [sym__wc_concat_string_int] = ACTIONS(2049),
    [sym__wc_file_open_int] = ACTIONS(2049),
    [sym__wc_file_close_int] = ACTIONS(2049),
    [sym__wc_file_read_int] = ACTIONS(2049),
    [sym__wc_file_read_line_int] = ACTIONS(2049),
    [sym__wc_file_write_int] = ACTIONS(2049),
    [sym__wc_file_write_line_int] = ACTIONS(2049),
    [aux_sym_preproc_include_token1] = ACTIONS(2049),
    [aux_sym_preproc_define_token1] = ACTIONS(2051),
    [aux_sym_preproc_warn_token1] = ACTIONS(2051),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2051),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2051),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2051),
    [anon_sym_LBRACK] = ACTIONS(2051),
    [anon_sym_LBRACE] = ACTIONS(2051),
    [anon_sym_RBRACE] = ACTIONS(2051),
    [anon_sym_LPAREN] = ACTIONS(2049),
    [anon_sym_BANG] = ACTIONS(2051),
    [anon_sym_TILDE] = ACTIONS(2051),
    [anon_sym_DASH] = ACTIONS(2049),
    [anon_sym_PLUS] = ACTIONS(2049),
    [anon_sym_DASH_DASH] = ACTIONS(2051),
    [anon_sym_PLUS_PLUS] = ACTIONS(2051),
    [anon_sym_DQUOTE] = ACTIONS(2051),
    [anon_sym_SQUOTE] = ACTIONS(2051),
    [sym_number_literal] = ACTIONS(2051),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(544)] = {
    [sym_identifier] = ACTIONS(2029),
    [sym_define_call_keyword] = ACTIONS(2029),
    [sym_define_function_keyword] = ACTIONS(2029),
    [sym_define_library_function_keyword] = ACTIONS(2029),
    [sym_define_module_keyword] = ACTIONS(2029),
    [sym_char_keyword] = ACTIONS(2029),
    [sym_widechar_keyword] = ACTIONS(2029),
    [sym_integer_keyword] = ACTIONS(2029),
    [sym_sinteger_keyword] = ACTIONS(2029),
    [sym_long_keyword] = ACTIONS(2029),
    [sym_slong_keyword] = ACTIONS(2029),
    [sym_float_keyword] = ACTIONS(2029),
    [sym_double_keyword] = ACTIONS(2029),
    [sym_variant_keyword] = ACTIONS(2029),
    [sym_variantarray_keyword] = ACTIONS(2029),
    [sym_local_var_keyword] = ACTIONS(2029),
    [sym_stack_var_keyword] = ACTIONS(2029),
    [sym_constant_keyword] = ACTIONS(2029),
    [sym_volatile_keyword] = ACTIONS(2029),
    [sym_non_volatile_keyword] = ACTIONS(2029),
    [sym_persistent_keyword] = ACTIONS(2029),
    [sym_dev_keyword] = ACTIONS(2029),
    [sym_devlev_keyword] = ACTIONS(2029),
    [sym_devchan_keyword] = ACTIONS(2029),
    [sym_if_keyword] = ACTIONS(2029),
    [sym_switch_keyword] = ACTIONS(2029),
    [sym_case_keyword] = ACTIONS(2029),
    [sym_default_keyword] = ACTIONS(2029),
    [sym_while_keyword] = ACTIONS(2029),
    [sym_for_keyword] = ACTIONS(2029),
    [sym_break_keyword] = ACTIONS(2029),
    [sym_continue_keyword] = ACTIONS(2029),
    [sym_return_keyword] = ACTIONS(2029),
    [sym_select_keyword] = ACTIONS(2029),
    [sym_active_keyword] = ACTIONS(2029),
    [sym_struct_keyword] = ACTIONS(2029),
    [sym_structure_keyword] = ACTIONS(2029),
    [aux_sym_on_keyword_token1] = ACTIONS(2029),
    [aux_sym_off_keyword_token1] = ACTIONS(2029),
    [sym_send_level_keyword] = ACTIONS(2029),
    [sym_send_string_keyword] = ACTIONS(2029),
    [sym_send_command_keyword] = ACTIONS(2029),
    [sym_clear_buffer_keyword] = ACTIONS(2029),
    [sym_create_buffer_keyword] = ACTIONS(2029),
    [sym_create_multi_buffer_keyword] = ACTIONS(2029),
    [sym_call_keyword] = ACTIONS(2029),
    [sym_system_call_keyword] = ACTIONS(2029),
    [sym_devchan_to_keyword] = ACTIONS(2029),
    [sym_devchan_min_to_keyword] = ACTIONS(2029),
    [sym_devchan_total_off_keyword] = ACTIONS(2029),
    [sym_devchan_pulse_keyword] = ACTIONS(2029),
    [sym_wait_keyword] = ACTIONS(2029),
    [sym_cancel_wait_keyword] = ACTIONS(2029),
    [sym_wait_until_keyword] = ACTIONS(2029),
    [sym_cancel_wait_until_keyword] = ACTIONS(2029),
    [sym_cancel_all_wait_keyword] = ACTIONS(2029),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2029),
    [sym_not] = ACTIONS(2029),
    [sym_bnot] = ACTIONS(2029),
    [sym___file__] = ACTIONS(2029),
    [sym___date__] = ACTIONS(2029),
    [sym___ldate__] = ACTIONS(2029),
    [sym___time__] = ACTIONS(2029),
    [sym___line__] = ACTIONS(2029),
    [sym___name__] = ACTIONS(2029),
    [sym___netlinx__] = ACTIONS(2029),
    [sym_netlinx_axi_version] = ACTIONS(2029),
    [sym_url_flg_tcp] = ACTIONS(2029),
    [sym_url_flg_accinfopresent] = ACTIONS(2029),
    [sym_url_flg_temp] = ACTIONS(2029),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2029),
    [sym_url_flg_stat_mask] = ACTIONS(2029),
    [sym_url_flg_stat_lookup] = ACTIONS(2029),
    [sym_url_flg_stat_connecting] = ACTIONS(2029),
    [sym_url_flg_stat_waiting] = ACTIONS(2029),
    [sym_url_flg_stat_connected] = ACTIONS(2029),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2029),
    [sym_first_virtual_device] = ACTIONS(2029),
    [sym_first_local_port] = ACTIONS(2029),
    [sym_dynamic_virtual_device] = ACTIONS(2029),
    [sym_timeline_once] = ACTIONS(2029),
    [sym_timeline_repeat] = ACTIONS(2029),
    [sym_timeline_absolute] = ACTIONS(2029),
    [sym_timeline_relative] = ACTIONS(2029),
    [sym_do_push_timed_infinite] = ACTIONS(2029),
    [sym_source_type_no_address] = ACTIONS(2029),
    [sym_source_type_neuron_id] = ACTIONS(2029),
    [sym_source_type_ip_address] = ACTIONS(2029),
    [sym_source_type_axlink] = ACTIONS(2029),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2029),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2029),
    [sym_source_type_ip_socket_address] = ACTIONS(2029),
    [sym_source_type_rs232] = ACTIONS(2029),
    [sym_source_type_internal] = ACTIONS(2029),
    [sym_source_type_mac_address] = ACTIONS(2029),
    [sym_source_type_ipv4_port] = ACTIONS(2029),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2029),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2029),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2029),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2029),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2029),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2029),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2029),
    [sym_true] = ACTIONS(2029),
    [sym_false] = ACTIONS(2029),
    [sym_file_read_only] = ACTIONS(2029),
    [sym_file_rw_new] = ACTIONS(2029),
    [sym_file_rw_append] = ACTIONS(2029),
    [sym_ip_tcp] = ACTIONS(2029),
    [sym_ip_udp] = ACTIONS(2029),
    [sym_ip_udp_2way] = ACTIONS(2029),
    [sym_xml_encode_types] = ACTIONS(2029),
    [sym_xml_encode_char_as_list] = ACTIONS(2029),
    [sym_xml_encode_le] = ACTIONS(2029),
    [sym_xml_decode_types] = ACTIONS(2029),
    [sym_xml_decode_no_preserve] = ACTIONS(2029),
    [sym_duet_dev_type_display_device] = ACTIONS(2029),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2029),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2029),
    [sym_duet_dev_type_io_device] = ACTIONS(2029),
    [sym_duet_dev_type_relay_device] = ACTIONS(2029),
    [sym_duet_dev_type_ups] = ACTIONS(2029),
    [sym_duet_dev_type_amplifier] = ACTIONS(2029),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2029),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2029),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2029),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2029),
    [sym_duet_dev_type_camera] = ACTIONS(2029),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2029),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2029),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2029),
    [sym_duet_dev_type_dss] = ACTIONS(2029),
    [sym_duet_dev_type_dvr] = ACTIONS(2029),
    [sym_duet_dev_type_disc_device] = ACTIONS(2029),
    [sym_duet_dev_type_document_camera] = ACTIONS(2029),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2029),
    [sym_duet_dev_type_hvac] = ACTIONS(2029),
    [sym_duet_dev_type_keypad] = ACTIONS(2029),
    [sym_duet_dev_type_light] = ACTIONS(2029),
    [sym_duet_dev_type_monitor] = ACTIONS(2029),
    [sym_duet_dev_type_motor] = ACTIONS(2029),
    [sym_duet_dev_type_multi_window] = ACTIONS(2029),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2029),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2029),
    [sym_duet_dev_type_receiver] = ACTIONS(2029),
    [sym_duet_dev_type_security_system] = ACTIONS(2029),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2029),
    [sym_duet_dev_type_settop_box] = ACTIONS(2029),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2029),
    [sym_duet_dev_type_switcher] = ACTIONS(2029),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2029),
    [sym_duet_dev_type_tv] = ACTIONS(2029),
    [sym_duet_dev_type_utility] = ACTIONS(2029),
    [sym_duet_dev_type_vcr] = ACTIONS(2029),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2029),
    [sym_duet_dev_type_video_processor] = ACTIONS(2029),
    [sym_duet_dev_type_video_projector] = ACTIONS(2029),
    [sym_duet_dev_type_video_wall] = ACTIONS(2029),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2029),
    [sym_duet_dev_type_weather] = ACTIONS(2029),
    [sym_duet_dev_not_polled] = ACTIONS(2029),
    [sym_duet_dev_polled] = ACTIONS(2029),
    [sym_clkmgr_mode_network] = ACTIONS(2029),
    [sym_clkmgr_mode_standalone] = ACTIONS(2029),
    [sym_normal_standby] = ACTIONS(2029),
    [sym_normal_wake] = ACTIONS(2029),
    [sym_ip_multicast_ttl_option] = ACTIONS(2029),
    [sym_ip_ttl_subnet] = ACTIONS(2029),
    [sym_ip_ttl_site] = ACTIONS(2029),
    [sym_ip_ttl_region] = ACTIONS(2029),
    [sym_ip_ttl_continent] = ACTIONS(2029),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2029),
    [sym_ip_nodelay_on] = ACTIONS(2029),
    [sym_ip_nodelay_off] = ACTIONS(2029),
    [sym_amx_error] = ACTIONS(2029),
    [sym_amx_warning] = ACTIONS(2029),
    [sym_amx_info] = ACTIONS(2029),
    [sym_amx_debug] = ACTIONS(2029),
    [sym_netlinx_login_success] = ACTIONS(2029),
    [sym_netlinx_login_fail] = ACTIONS(2029),
    [sym_netlinx_logout] = ACTIONS(2029),
    [sym_null_str] = ACTIONS(2029),
    [sym_smtp_address] = ACTIONS(2029),
    [sym_smtp_port_number] = ACTIONS(2029),
    [sym_smtp_username] = ACTIONS(2029),
    [sym_smtp_password] = ACTIONS(2029),
    [sym_smtp_require_tls] = ACTIONS(2029),
    [sym_smtp_from] = ACTIONS(2029),
    [sym_smtp_tls_true] = ACTIONS(2029),
    [sym_smtp_tls_false] = ACTIONS(2029),
    [sym_listview_on_row_select_event] = ACTIONS(2029),
    [sym_data_structure_unknown] = ACTIONS(2029),
    [sym_data_structure_datafeed] = ACTIONS(2029),
    [sym_data_structure_datarecordset] = ACTIONS(2029),
    [sym_data_structure_datarecord] = ACTIONS(2029),
    [sym_data_structure_datafield] = ACTIONS(2029),
    [sym_data_max_content_field_count] = ACTIONS(2029),
    [sym_data_max_metadata_field_count] = ACTIONS(2029),
    [sym_data_max_id_length] = ACTIONS(2029),
    [sym_data_max_name_length] = ACTIONS(2029),
    [sym_data_max_description_length] = ACTIONS(2029),
    [sym_data_max_type_length] = ACTIONS(2029),
    [sym_data_max_format_length] = ACTIONS(2029),
    [sym_data_max_source_length] = ACTIONS(2029),
    [sym_data_max_value_length] = ACTIONS(2029),
    [sym_data_max_label_length] = ACTIONS(2029),
    [sym_data_max_url_length] = ACTIONS(2029),
    [sym_max_last_login_info_length] = ACTIONS(2029),
    [sym_data_type_unknown] = ACTIONS(2029),
    [sym_data_type_string] = ACTIONS(2029),
    [sym_data_type_datetime] = ACTIONS(2029),
    [sym_data_type_date] = ACTIONS(2029),
    [sym_data_type_time] = ACTIONS(2029),
    [sym_data_type_image] = ACTIONS(2029),
    [sym_data_format_url] = ACTIONS(2029),
    [sym_data_format_phone] = ACTIONS(2029),
    [sym_data_format_email] = ACTIONS(2029),
    [sym_data_format_iso8601] = ACTIONS(2029),
    [sym_auditlog_priv] = ACTIONS(2029),
    [sym_configuration_priv] = ACTIONS(2029),
    [sym_deviceconfig_priv] = ACTIONS(2029),
    [sym_ftp_priv] = ACTIONS(2029),
    [sym_http_priv] = ACTIONS(2029),
    [sym_networkconfig_priv] = ACTIONS(2029),
    [sym_programport_priv] = ACTIONS(2029),
    [sym_remoteui_priv] = ACTIONS(2029),
    [sym_securitycontrol_priv] = ACTIONS(2029),
    [sym_softwaremanagement_priv] = ACTIONS(2029),
    [sym_terminal_priv] = ACTIONS(2029),
    [sym_tpadmin_priv] = ACTIONS(2029),
    [sym_user1_priv] = ACTIONS(2029),
    [sym_user2_priv] = ACTIONS(2029),
    [sym_user3_priv] = ACTIONS(2029),
    [sym_user4_priv] = ACTIONS(2029),
    [sym_usermanagement_priv] = ACTIONS(2029),
    [sym_valid_account] = ACTIONS(2029),
    [sym_err_name_invalid] = ACTIONS(2029),
    [sym_err_password_invalid] = ACTIONS(2029),
    [sym_err_invalid_account] = ACTIONS(2029),
    [sym_err_no_authorization] = ACTIONS(2029),
    [sym_err_invalid_parameter] = ACTIONS(2029),
    [sym_err_user_locked_out] = ACTIONS(2029),
    [sym_err_user_account_expired] = ACTIONS(2029),
    [sym_err_general_error] = ACTIONS(2029),
    [sym_err_authentication_error] = ACTIONS(2029),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2029),
    [sym_tls_validate_certificate] = ACTIONS(2029),
    [sym_tdata] = ACTIONS(2029),
    [sym_tchannel] = ACTIONS(2029),
    [sym_tlevel] = ACTIONS(2029),
    [sym_tbutton] = ACTIONS(2029),
    [sym_ttimeline] = ACTIONS(2029),
    [sym_tcustom] = ACTIONS(2029),
    [sym_url_struct] = ACTIONS(2029),
    [sym_dns_struct] = ACTIONS(2029),
    [sym_ip_address_struct] = ACTIONS(2029),
    [sym_dev_info_struct] = ACTIONS(2029),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2029),
    [sym_data_feed] = ACTIONS(2029),
    [sym_data_field] = ACTIONS(2029),
    [sym_data_record] = ACTIONS(2029),
    [sym_wc_data_feed] = ACTIONS(2029),
    [sym_wc_data_field] = ACTIONS(2029),
    [sym_wc_data_record] = ACTIONS(2029),
    [sym_last_login_info] = ACTIONS(2029),
    [sym____reserved___] = ACTIONS(2029),
    [sym_date] = ACTIONS(2029),
    [sym_day] = ACTIONS(2029),
    [sym_time] = ACTIONS(2029),
    [sym_ldate] = ACTIONS(2029),
    [sym_push_device] = ACTIONS(2029),
    [sym_push_channel] = ACTIONS(2029),
    [sym_push_devchan] = ACTIONS(2029),
    [sym_release_device] = ACTIONS(2029),
    [sym_release_channel] = ACTIONS(2029),
    [sym_release_devchan] = ACTIONS(2029),
    [sym_master_slot] = ACTIONS(2029),
    [sym_get_pulse_time] = ACTIONS(2029),
    [sym_get_timer] = ACTIONS(2029),
    [sym_master_sn] = ACTIONS(2029),
    [sym_system_number] = ACTIONS(2029),
    [sym_button] = ACTIONS(2029),
    [sym_dv_channel] = ACTIONS(2029),
    [sym_data] = ACTIONS(2029),
    [sym_level] = ACTIONS(2029),
    [sym_timeline] = ACTIONS(2029),
    [sym_channel] = ACTIONS(2029),
    [sym_custom] = ACTIONS(2029),
    [sym_atoi] = ACTIONS(2029),
    [sym_atol] = ACTIONS(2029),
    [sym_atof] = ACTIONS(2029),
    [sym_itoa] = ACTIONS(2029),
    [sym_itohex] = ACTIONS(2029),
    [sym_hextoi] = ACTIONS(2029),
    [sym_ftoa] = ACTIONS(2029),
    [sym_find_string] = ACTIONS(2029),
    [sym_left_string] = ACTIONS(2029),
    [sym_right_string] = ACTIONS(2029),
    [sym_mid_string] = ACTIONS(2029),
    [sym_remove_string] = ACTIONS(2029),
    [sym_get_buffer_string] = ACTIONS(2029),
    [sym_get_buffer_char] = ACTIONS(2029),
    [sym_get_multi_buffer_string] = ACTIONS(2029),
    [sym_length_string] = ACTIONS(2029),
    [sym_max_length_string] = ACTIONS(2029),
    [sym_set_length_string] = ACTIONS(2029),
    [sym_compare_string] = ACTIONS(2029),
    [sym_upper_string] = ACTIONS(2029),
    [sym_lower_string] = ACTIONS(2029),
    [sym_get_last] = ACTIONS(2029),
    [sym_date_to_day] = ACTIONS(2029),
    [sym_date_to_month] = ACTIONS(2029),
    [sym_date_to_year] = ACTIONS(2029),
    [sym_time_to_hour] = ACTIONS(2029),
    [sym_time_to_minute] = ACTIONS(2029),
    [sym_time_to_second] = ACTIONS(2029),
    [sym_day_of_week] = ACTIONS(2029),
    [sym_set_pulse_time] = ACTIONS(2029),
    [sym_set_timer] = ACTIONS(2029),
    [sym_file_open] = ACTIONS(2029),
    [sym_file_close] = ACTIONS(2029),
    [sym_file_read] = ACTIONS(2029),
    [sym_file_read_line] = ACTIONS(2029),
    [sym_file_write] = ACTIONS(2029),
    [sym_file_write_line] = ACTIONS(2029),
    [sym_file_copy] = ACTIONS(2029),
    [sym_file_delete] = ACTIONS(2029),
    [sym_file_rename] = ACTIONS(2029),
    [sym_file_dir] = ACTIONS(2029),
    [sym_file_getdir] = ACTIONS(2029),
    [sym_file_setdir] = ACTIONS(2029),
    [sym_file_create_dir] = ACTIONS(2029),
    [sym_file_remove_dir] = ACTIONS(2029),
    [sym_file_seek] = ACTIONS(2029),
    [sym_length_array] = ACTIONS(2029),
    [sym_max_length_array] = ACTIONS(2029),
    [sym_set_length_array] = ACTIONS(2029),
    [sym_ip_client_open] = ACTIONS(2029),
    [sym_ip_client_close] = ACTIONS(2029),
    [sym_ip_server_open] = ACTIONS(2029),
    [sym_ip_server_close] = ACTIONS(2029),
    [sym_ip_bound_client_open] = ACTIONS(2029),
    [sym_ip_mc_server_open] = ACTIONS(2029),
    [sym_ip_set_option] = ACTIONS(2029),
    [sym_ssh_client_open] = ACTIONS(2029),
    [sym_ssh_client_close] = ACTIONS(2029),
    [sym_tls_client_open] = ACTIONS(2029),
    [sym_tls_client_close] = ACTIONS(2029),
    [sym_do_push] = ACTIONS(2029),
    [sym_do_push_timed] = ACTIONS(2029),
    [sym_do_release] = ACTIONS(2029),
    [sym_device_id] = ACTIONS(2029),
    [sym_device_info] = ACTIONS(2029),
    [sym_device_id_string] = ACTIONS(2029),
    [sym_device_standby] = ACTIONS(2029),
    [sym_device_wake] = ACTIONS(2029),
    [sym_timeline_create] = ACTIONS(2029),
    [sym_timeline_kill] = ACTIONS(2029),
    [sym_timeline_reload] = ACTIONS(2029),
    [sym_timeline_pause] = ACTIONS(2029),
    [sym_timeline_restart] = ACTIONS(2029),
    [sym_timeline_set] = ACTIONS(2029),
    [sym_timeline_get] = ACTIONS(2029),
    [sym_timeline_active] = ACTIONS(2029),
    [sym_timeline_dynamic_id] = ACTIONS(2029),
    [sym_variable_to_string] = ACTIONS(2029),
    [sym_string_to_variable] = ACTIONS(2029),
    [sym_length_variable_to_string] = ACTIONS(2029),
    [sym_type_cast] = ACTIONS(2029),
    [sym_variable_to_xml] = ACTIONS(2029),
    [sym_xml_to_variable] = ACTIONS(2029),
    [sym_length_variable_to_xml] = ACTIONS(2029),
    [sym_redirect_string] = ACTIONS(2029),
    [sym_random_number] = ACTIONS(2029),
    [sym_get_unique_id] = ACTIONS(2029),
    [sym_get_system_number] = ACTIONS(2029),
    [sym_set_system_number] = ACTIONS(2029),
    [sym_get_serial_number] = ACTIONS(2029),
    [sym_get_master_build] = ACTIONS(2029),
    [sym_set_validation_code] = ACTIONS(2029),
    [sym_reboot] = ACTIONS(2029),
    [sym_load_duet_module] = ACTIONS(2029),
    [sym_unload_duet_module] = ACTIONS(2029),
    [sym_duet_mem_size_set] = ACTIONS(2029),
    [sym_duet_mem_size_get] = ACTIONS(2029),
    [sym_dynamic_application_device] = ACTIONS(2029),
    [sym_dynamic_polled_port] = ACTIONS(2029),
    [sym_static_port_binding] = ACTIONS(2029),
    [sym_static_ip_binding] = ACTIONS(2029),
    [sym_get_dns_list] = ACTIONS(2029),
    [sym_set_dns_list] = ACTIONS(2029),
    [sym_get_ip_address] = ACTIONS(2029),
    [sym_set_ip_address] = ACTIONS(2029),
    [sym_get_url_list] = ACTIONS(2029),
    [sym_add_url_entry] = ACTIONS(2029),
    [sym_delete_url_entry] = ACTIONS(2029),
    [sym_abs_value] = ACTIONS(2029),
    [sym_max_value] = ACTIONS(2029),
    [sym_min_value] = ACTIONS(2029),
    [sym_exp_value] = ACTIONS(2029),
    [sym_log_value] = ACTIONS(2029),
    [sym_log10_value] = ACTIONS(2029),
    [sym_power_value] = ACTIONS(2029),
    [sym_sqrt_value] = ACTIONS(2029),
    [sym_raw_be] = ACTIONS(2029),
    [sym_raw_le] = ACTIONS(2029),
    [sym_internal_queue_size_set] = ACTIONS(2029),
    [sym_internal_queue_size_get] = ACTIONS(2029),
    [sym_internal_threshold_set] = ACTIONS(2029),
    [sym_internal_threshold_get] = ACTIONS(2029),
    [sym_rebuilt_event] = ACTIONS(2029),
    [sym_do_custom_event] = ACTIONS(2029),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2029),
    [sym_clkmgr_set_clk_source] = ACTIONS(2029),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2029),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2029),
    [sym_clkmgr_get_timezone] = ACTIONS(2029),
    [sym_clkmgr_set_timezone] = ACTIONS(2029),
    [sym_clkmgr_get_resync_period] = ACTIONS(2029),
    [sym_clkmgr_set_resync_period] = ACTIONS(2029),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2029),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2029),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2029),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2029),
    [sym_clkmgr_get_timeservers] = ACTIONS(2029),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2029),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2029),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2029),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2029),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2029),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2029),
    [sym_set_log_level] = ACTIONS(2029),
    [sym_get_log_level] = ACTIONS(2029),
    [sym_amx_log] = ACTIONS(2029),
    [sym_smtp_server_config_set] = ACTIONS(2029),
    [sym_smtp_server_config_get] = ACTIONS(2029),
    [sym_smtp_send] = ACTIONS(2029),
    [sym_validate_netlinx_account] = ACTIONS(2029),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2029),
    [sym_authenticate_certificate] = ACTIONS(2029),
    [sym_audit_netlinx_session_event] = ACTIONS(2029),
    [sym_audit_netlinx_generic_event] = ACTIONS(2029),
    [sym_get_available_flash_disk_space] = ACTIONS(2029),
    [sym_get_max_flash_disk_space] = ACTIONS(2029),
    [sym_data_create_feed] = ACTIONS(2029),
    [sym_data_delete_feed] = ACTIONS(2029),
    [sym_data_publish_feed] = ACTIONS(2029),
    [sym_data_get_published_feed] = ACTIONS(2029),
    [sym_data_add_record] = ACTIONS(2029),
    [sym_data_get_event_record] = ACTIONS(2029),
    [sym__wc_data_create_feed] = ACTIONS(2029),
    [sym__wc_data_add_record] = ACTIONS(2029),
    [sym__wc_data_get_event_record] = ACTIONS(2029),
    [sym_format] = ACTIONS(2029),
    [sym_astro_clock] = ACTIONS(2029),
    [sym_set_outdoor_temperature] = ACTIONS(2029),
    [sym_set_virtual_level_count] = ACTIONS(2029),
    [sym_set_virtual_channel_count] = ACTIONS(2029),
    [sym_set_virtual_port_count] = ACTIONS(2029),
    [sym__wc_encode_int] = ACTIONS(2029),
    [sym__wc_decode_int] = ACTIONS(2029),
    [sym__wc_int] = ACTIONS(2029),
    [sym__wc_to_ch_int] = ACTIONS(2029),
    [sym__ch_to_wc_int] = ACTIONS(2029),
    [sym__wc_find_string_int] = ACTIONS(2029),
    [sym__wc_left_string_int] = ACTIONS(2029),
    [sym__wc_lower_string_int] = ACTIONS(2029),
    [sym__wc_mid_string_int] = ACTIONS(2029),
    [sym__wc_remove_string_int] = ACTIONS(2029),
    [sym__wc_right_string_int] = ACTIONS(2029),
    [sym__wc_upper_string_int] = ACTIONS(2029),
    [sym__wc_compare_string_int] = ACTIONS(2029),
    [sym__wc_get_buffer_char_int] = ACTIONS(2029),
    [sym__wc_get_buffer_string_int] = ACTIONS(2029),
    [sym__wc_concat_string_int] = ACTIONS(2029),
    [sym__wc_file_open_int] = ACTIONS(2029),
    [sym__wc_file_close_int] = ACTIONS(2029),
    [sym__wc_file_read_int] = ACTIONS(2029),
    [sym__wc_file_read_line_int] = ACTIONS(2029),
    [sym__wc_file_write_int] = ACTIONS(2029),
    [sym__wc_file_write_line_int] = ACTIONS(2029),
    [aux_sym_preproc_include_token1] = ACTIONS(2029),
    [aux_sym_preproc_define_token1] = ACTIONS(2031),
    [aux_sym_preproc_warn_token1] = ACTIONS(2031),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2031),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2031),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2031),
    [anon_sym_LBRACK] = ACTIONS(2031),
    [anon_sym_LBRACE] = ACTIONS(2031),
    [anon_sym_RBRACE] = ACTIONS(2031),
    [anon_sym_LPAREN] = ACTIONS(2029),
    [anon_sym_BANG] = ACTIONS(2031),
    [anon_sym_TILDE] = ACTIONS(2031),
    [anon_sym_DASH] = ACTIONS(2029),
    [anon_sym_PLUS] = ACTIONS(2029),
    [anon_sym_DASH_DASH] = ACTIONS(2031),
    [anon_sym_PLUS_PLUS] = ACTIONS(2031),
    [anon_sym_DQUOTE] = ACTIONS(2031),
    [anon_sym_SQUOTE] = ACTIONS(2031),
    [sym_number_literal] = ACTIONS(2031),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(545)] = {
    [sym_identifier] = ACTIONS(1883),
    [sym_define_call_keyword] = ACTIONS(1883),
    [sym_define_function_keyword] = ACTIONS(1883),
    [sym_define_library_function_keyword] = ACTIONS(1883),
    [sym_define_module_keyword] = ACTIONS(1883),
    [sym_char_keyword] = ACTIONS(1883),
    [sym_widechar_keyword] = ACTIONS(1883),
    [sym_integer_keyword] = ACTIONS(1883),
    [sym_sinteger_keyword] = ACTIONS(1883),
    [sym_long_keyword] = ACTIONS(1883),
    [sym_slong_keyword] = ACTIONS(1883),
    [sym_float_keyword] = ACTIONS(1883),
    [sym_double_keyword] = ACTIONS(1883),
    [sym_variant_keyword] = ACTIONS(1883),
    [sym_variantarray_keyword] = ACTIONS(1883),
    [sym_local_var_keyword] = ACTIONS(1883),
    [sym_stack_var_keyword] = ACTIONS(1883),
    [sym_constant_keyword] = ACTIONS(1883),
    [sym_volatile_keyword] = ACTIONS(1883),
    [sym_non_volatile_keyword] = ACTIONS(1883),
    [sym_persistent_keyword] = ACTIONS(1883),
    [sym_dev_keyword] = ACTIONS(1883),
    [sym_devlev_keyword] = ACTIONS(1883),
    [sym_devchan_keyword] = ACTIONS(1883),
    [sym_if_keyword] = ACTIONS(1883),
    [sym_switch_keyword] = ACTIONS(1883),
    [sym_case_keyword] = ACTIONS(1883),
    [sym_default_keyword] = ACTIONS(1883),
    [sym_while_keyword] = ACTIONS(1883),
    [sym_for_keyword] = ACTIONS(1883),
    [sym_break_keyword] = ACTIONS(1883),
    [sym_continue_keyword] = ACTIONS(1883),
    [sym_return_keyword] = ACTIONS(1883),
    [sym_select_keyword] = ACTIONS(1883),
    [sym_active_keyword] = ACTIONS(1883),
    [sym_struct_keyword] = ACTIONS(1883),
    [sym_structure_keyword] = ACTIONS(1883),
    [aux_sym_on_keyword_token1] = ACTIONS(1883),
    [aux_sym_off_keyword_token1] = ACTIONS(1883),
    [sym_send_level_keyword] = ACTIONS(1883),
    [sym_send_string_keyword] = ACTIONS(1883),
    [sym_send_command_keyword] = ACTIONS(1883),
    [sym_clear_buffer_keyword] = ACTIONS(1883),
    [sym_create_buffer_keyword] = ACTIONS(1883),
    [sym_create_multi_buffer_keyword] = ACTIONS(1883),
    [sym_call_keyword] = ACTIONS(1883),
    [sym_system_call_keyword] = ACTIONS(1883),
    [sym_devchan_to_keyword] = ACTIONS(1883),
    [sym_devchan_min_to_keyword] = ACTIONS(1883),
    [sym_devchan_total_off_keyword] = ACTIONS(1883),
    [sym_devchan_pulse_keyword] = ACTIONS(1883),
    [sym_wait_keyword] = ACTIONS(1883),
    [sym_cancel_wait_keyword] = ACTIONS(1883),
    [sym_wait_until_keyword] = ACTIONS(1883),
    [sym_cancel_wait_until_keyword] = ACTIONS(1883),
    [sym_cancel_all_wait_keyword] = ACTIONS(1883),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1883),
    [sym_not] = ACTIONS(1883),
    [sym_bnot] = ACTIONS(1883),
    [sym___file__] = ACTIONS(1883),
    [sym___date__] = ACTIONS(1883),
    [sym___ldate__] = ACTIONS(1883),
    [sym___time__] = ACTIONS(1883),
    [sym___line__] = ACTIONS(1883),
    [sym___name__] = ACTIONS(1883),
    [sym___netlinx__] = ACTIONS(1883),
    [sym_netlinx_axi_version] = ACTIONS(1883),
    [sym_url_flg_tcp] = ACTIONS(1883),
    [sym_url_flg_accinfopresent] = ACTIONS(1883),
    [sym_url_flg_temp] = ACTIONS(1883),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1883),
    [sym_url_flg_stat_mask] = ACTIONS(1883),
    [sym_url_flg_stat_lookup] = ACTIONS(1883),
    [sym_url_flg_stat_connecting] = ACTIONS(1883),
    [sym_url_flg_stat_waiting] = ACTIONS(1883),
    [sym_url_flg_stat_connected] = ACTIONS(1883),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1883),
    [sym_first_virtual_device] = ACTIONS(1883),
    [sym_first_local_port] = ACTIONS(1883),
    [sym_dynamic_virtual_device] = ACTIONS(1883),
    [sym_timeline_once] = ACTIONS(1883),
    [sym_timeline_repeat] = ACTIONS(1883),
    [sym_timeline_absolute] = ACTIONS(1883),
    [sym_timeline_relative] = ACTIONS(1883),
    [sym_do_push_timed_infinite] = ACTIONS(1883),
    [sym_source_type_no_address] = ACTIONS(1883),
    [sym_source_type_neuron_id] = ACTIONS(1883),
    [sym_source_type_ip_address] = ACTIONS(1883),
    [sym_source_type_axlink] = ACTIONS(1883),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1883),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1883),
    [sym_source_type_ip_socket_address] = ACTIONS(1883),
    [sym_source_type_rs232] = ACTIONS(1883),
    [sym_source_type_internal] = ACTIONS(1883),
    [sym_source_type_mac_address] = ACTIONS(1883),
    [sym_source_type_ipv4_port] = ACTIONS(1883),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1883),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1883),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1883),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1883),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1883),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1883),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1883),
    [sym_true] = ACTIONS(1883),
    [sym_false] = ACTIONS(1883),
    [sym_file_read_only] = ACTIONS(1883),
    [sym_file_rw_new] = ACTIONS(1883),
    [sym_file_rw_append] = ACTIONS(1883),
    [sym_ip_tcp] = ACTIONS(1883),
    [sym_ip_udp] = ACTIONS(1883),
    [sym_ip_udp_2way] = ACTIONS(1883),
    [sym_xml_encode_types] = ACTIONS(1883),
    [sym_xml_encode_char_as_list] = ACTIONS(1883),
    [sym_xml_encode_le] = ACTIONS(1883),
    [sym_xml_decode_types] = ACTIONS(1883),
    [sym_xml_decode_no_preserve] = ACTIONS(1883),
    [sym_duet_dev_type_display_device] = ACTIONS(1883),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1883),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1883),
    [sym_duet_dev_type_io_device] = ACTIONS(1883),
    [sym_duet_dev_type_relay_device] = ACTIONS(1883),
    [sym_duet_dev_type_ups] = ACTIONS(1883),
    [sym_duet_dev_type_amplifier] = ACTIONS(1883),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1883),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1883),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1883),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1883),
    [sym_duet_dev_type_camera] = ACTIONS(1883),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1883),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1883),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1883),
    [sym_duet_dev_type_dss] = ACTIONS(1883),
    [sym_duet_dev_type_dvr] = ACTIONS(1883),
    [sym_duet_dev_type_disc_device] = ACTIONS(1883),
    [sym_duet_dev_type_document_camera] = ACTIONS(1883),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1883),
    [sym_duet_dev_type_hvac] = ACTIONS(1883),
    [sym_duet_dev_type_keypad] = ACTIONS(1883),
    [sym_duet_dev_type_light] = ACTIONS(1883),
    [sym_duet_dev_type_monitor] = ACTIONS(1883),
    [sym_duet_dev_type_motor] = ACTIONS(1883),
    [sym_duet_dev_type_multi_window] = ACTIONS(1883),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1883),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1883),
    [sym_duet_dev_type_receiver] = ACTIONS(1883),
    [sym_duet_dev_type_security_system] = ACTIONS(1883),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1883),
    [sym_duet_dev_type_settop_box] = ACTIONS(1883),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1883),
    [sym_duet_dev_type_switcher] = ACTIONS(1883),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1883),
    [sym_duet_dev_type_tv] = ACTIONS(1883),
    [sym_duet_dev_type_utility] = ACTIONS(1883),
    [sym_duet_dev_type_vcr] = ACTIONS(1883),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1883),
    [sym_duet_dev_type_video_processor] = ACTIONS(1883),
    [sym_duet_dev_type_video_projector] = ACTIONS(1883),
    [sym_duet_dev_type_video_wall] = ACTIONS(1883),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1883),
    [sym_duet_dev_type_weather] = ACTIONS(1883),
    [sym_duet_dev_not_polled] = ACTIONS(1883),
    [sym_duet_dev_polled] = ACTIONS(1883),
    [sym_clkmgr_mode_network] = ACTIONS(1883),
    [sym_clkmgr_mode_standalone] = ACTIONS(1883),
    [sym_normal_standby] = ACTIONS(1883),
    [sym_normal_wake] = ACTIONS(1883),
    [sym_ip_multicast_ttl_option] = ACTIONS(1883),
    [sym_ip_ttl_subnet] = ACTIONS(1883),
    [sym_ip_ttl_site] = ACTIONS(1883),
    [sym_ip_ttl_region] = ACTIONS(1883),
    [sym_ip_ttl_continent] = ACTIONS(1883),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1883),
    [sym_ip_nodelay_on] = ACTIONS(1883),
    [sym_ip_nodelay_off] = ACTIONS(1883),
    [sym_amx_error] = ACTIONS(1883),
    [sym_amx_warning] = ACTIONS(1883),
    [sym_amx_info] = ACTIONS(1883),
    [sym_amx_debug] = ACTIONS(1883),
    [sym_netlinx_login_success] = ACTIONS(1883),
    [sym_netlinx_login_fail] = ACTIONS(1883),
    [sym_netlinx_logout] = ACTIONS(1883),
    [sym_null_str] = ACTIONS(1883),
    [sym_smtp_address] = ACTIONS(1883),
    [sym_smtp_port_number] = ACTIONS(1883),
    [sym_smtp_username] = ACTIONS(1883),
    [sym_smtp_password] = ACTIONS(1883),
    [sym_smtp_require_tls] = ACTIONS(1883),
    [sym_smtp_from] = ACTIONS(1883),
    [sym_smtp_tls_true] = ACTIONS(1883),
    [sym_smtp_tls_false] = ACTIONS(1883),
    [sym_listview_on_row_select_event] = ACTIONS(1883),
    [sym_data_structure_unknown] = ACTIONS(1883),
    [sym_data_structure_datafeed] = ACTIONS(1883),
    [sym_data_structure_datarecordset] = ACTIONS(1883),
    [sym_data_structure_datarecord] = ACTIONS(1883),
    [sym_data_structure_datafield] = ACTIONS(1883),
    [sym_data_max_content_field_count] = ACTIONS(1883),
    [sym_data_max_metadata_field_count] = ACTIONS(1883),
    [sym_data_max_id_length] = ACTIONS(1883),
    [sym_data_max_name_length] = ACTIONS(1883),
    [sym_data_max_description_length] = ACTIONS(1883),
    [sym_data_max_type_length] = ACTIONS(1883),
    [sym_data_max_format_length] = ACTIONS(1883),
    [sym_data_max_source_length] = ACTIONS(1883),
    [sym_data_max_value_length] = ACTIONS(1883),
    [sym_data_max_label_length] = ACTIONS(1883),
    [sym_data_max_url_length] = ACTIONS(1883),
    [sym_max_last_login_info_length] = ACTIONS(1883),
    [sym_data_type_unknown] = ACTIONS(1883),
    [sym_data_type_string] = ACTIONS(1883),
    [sym_data_type_datetime] = ACTIONS(1883),
    [sym_data_type_date] = ACTIONS(1883),
    [sym_data_type_time] = ACTIONS(1883),
    [sym_data_type_image] = ACTIONS(1883),
    [sym_data_format_url] = ACTIONS(1883),
    [sym_data_format_phone] = ACTIONS(1883),
    [sym_data_format_email] = ACTIONS(1883),
    [sym_data_format_iso8601] = ACTIONS(1883),
    [sym_auditlog_priv] = ACTIONS(1883),
    [sym_configuration_priv] = ACTIONS(1883),
    [sym_deviceconfig_priv] = ACTIONS(1883),
    [sym_ftp_priv] = ACTIONS(1883),
    [sym_http_priv] = ACTIONS(1883),
    [sym_networkconfig_priv] = ACTIONS(1883),
    [sym_programport_priv] = ACTIONS(1883),
    [sym_remoteui_priv] = ACTIONS(1883),
    [sym_securitycontrol_priv] = ACTIONS(1883),
    [sym_softwaremanagement_priv] = ACTIONS(1883),
    [sym_terminal_priv] = ACTIONS(1883),
    [sym_tpadmin_priv] = ACTIONS(1883),
    [sym_user1_priv] = ACTIONS(1883),
    [sym_user2_priv] = ACTIONS(1883),
    [sym_user3_priv] = ACTIONS(1883),
    [sym_user4_priv] = ACTIONS(1883),
    [sym_usermanagement_priv] = ACTIONS(1883),
    [sym_valid_account] = ACTIONS(1883),
    [sym_err_name_invalid] = ACTIONS(1883),
    [sym_err_password_invalid] = ACTIONS(1883),
    [sym_err_invalid_account] = ACTIONS(1883),
    [sym_err_no_authorization] = ACTIONS(1883),
    [sym_err_invalid_parameter] = ACTIONS(1883),
    [sym_err_user_locked_out] = ACTIONS(1883),
    [sym_err_user_account_expired] = ACTIONS(1883),
    [sym_err_general_error] = ACTIONS(1883),
    [sym_err_authentication_error] = ACTIONS(1883),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1883),
    [sym_tls_validate_certificate] = ACTIONS(1883),
    [sym_tdata] = ACTIONS(1883),
    [sym_tchannel] = ACTIONS(1883),
    [sym_tlevel] = ACTIONS(1883),
    [sym_tbutton] = ACTIONS(1883),
    [sym_ttimeline] = ACTIONS(1883),
    [sym_tcustom] = ACTIONS(1883),
    [sym_url_struct] = ACTIONS(1883),
    [sym_dns_struct] = ACTIONS(1883),
    [sym_ip_address_struct] = ACTIONS(1883),
    [sym_dev_info_struct] = ACTIONS(1883),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1883),
    [sym_data_feed] = ACTIONS(1883),
    [sym_data_field] = ACTIONS(1883),
    [sym_data_record] = ACTIONS(1883),
    [sym_wc_data_feed] = ACTIONS(1883),
    [sym_wc_data_field] = ACTIONS(1883),
    [sym_wc_data_record] = ACTIONS(1883),
    [sym_last_login_info] = ACTIONS(1883),
    [sym____reserved___] = ACTIONS(1883),
    [sym_date] = ACTIONS(1883),
    [sym_day] = ACTIONS(1883),
    [sym_time] = ACTIONS(1883),
    [sym_ldate] = ACTIONS(1883),
    [sym_push_device] = ACTIONS(1883),
    [sym_push_channel] = ACTIONS(1883),
    [sym_push_devchan] = ACTIONS(1883),
    [sym_release_device] = ACTIONS(1883),
    [sym_release_channel] = ACTIONS(1883),
    [sym_release_devchan] = ACTIONS(1883),
    [sym_master_slot] = ACTIONS(1883),
    [sym_get_pulse_time] = ACTIONS(1883),
    [sym_get_timer] = ACTIONS(1883),
    [sym_master_sn] = ACTIONS(1883),
    [sym_system_number] = ACTIONS(1883),
    [sym_button] = ACTIONS(1883),
    [sym_dv_channel] = ACTIONS(1883),
    [sym_data] = ACTIONS(1883),
    [sym_level] = ACTIONS(1883),
    [sym_timeline] = ACTIONS(1883),
    [sym_channel] = ACTIONS(1883),
    [sym_custom] = ACTIONS(1883),
    [sym_atoi] = ACTIONS(1883),
    [sym_atol] = ACTIONS(1883),
    [sym_atof] = ACTIONS(1883),
    [sym_itoa] = ACTIONS(1883),
    [sym_itohex] = ACTIONS(1883),
    [sym_hextoi] = ACTIONS(1883),
    [sym_ftoa] = ACTIONS(1883),
    [sym_find_string] = ACTIONS(1883),
    [sym_left_string] = ACTIONS(1883),
    [sym_right_string] = ACTIONS(1883),
    [sym_mid_string] = ACTIONS(1883),
    [sym_remove_string] = ACTIONS(1883),
    [sym_get_buffer_string] = ACTIONS(1883),
    [sym_get_buffer_char] = ACTIONS(1883),
    [sym_get_multi_buffer_string] = ACTIONS(1883),
    [sym_length_string] = ACTIONS(1883),
    [sym_max_length_string] = ACTIONS(1883),
    [sym_set_length_string] = ACTIONS(1883),
    [sym_compare_string] = ACTIONS(1883),
    [sym_upper_string] = ACTIONS(1883),
    [sym_lower_string] = ACTIONS(1883),
    [sym_get_last] = ACTIONS(1883),
    [sym_date_to_day] = ACTIONS(1883),
    [sym_date_to_month] = ACTIONS(1883),
    [sym_date_to_year] = ACTIONS(1883),
    [sym_time_to_hour] = ACTIONS(1883),
    [sym_time_to_minute] = ACTIONS(1883),
    [sym_time_to_second] = ACTIONS(1883),
    [sym_day_of_week] = ACTIONS(1883),
    [sym_set_pulse_time] = ACTIONS(1883),
    [sym_set_timer] = ACTIONS(1883),
    [sym_file_open] = ACTIONS(1883),
    [sym_file_close] = ACTIONS(1883),
    [sym_file_read] = ACTIONS(1883),
    [sym_file_read_line] = ACTIONS(1883),
    [sym_file_write] = ACTIONS(1883),
    [sym_file_write_line] = ACTIONS(1883),
    [sym_file_copy] = ACTIONS(1883),
    [sym_file_delete] = ACTIONS(1883),
    [sym_file_rename] = ACTIONS(1883),
    [sym_file_dir] = ACTIONS(1883),
    [sym_file_getdir] = ACTIONS(1883),
    [sym_file_setdir] = ACTIONS(1883),
    [sym_file_create_dir] = ACTIONS(1883),
    [sym_file_remove_dir] = ACTIONS(1883),
    [sym_file_seek] = ACTIONS(1883),
    [sym_length_array] = ACTIONS(1883),
    [sym_max_length_array] = ACTIONS(1883),
    [sym_set_length_array] = ACTIONS(1883),
    [sym_ip_client_open] = ACTIONS(1883),
    [sym_ip_client_close] = ACTIONS(1883),
    [sym_ip_server_open] = ACTIONS(1883),
    [sym_ip_server_close] = ACTIONS(1883),
    [sym_ip_bound_client_open] = ACTIONS(1883),
    [sym_ip_mc_server_open] = ACTIONS(1883),
    [sym_ip_set_option] = ACTIONS(1883),
    [sym_ssh_client_open] = ACTIONS(1883),
    [sym_ssh_client_close] = ACTIONS(1883),
    [sym_tls_client_open] = ACTIONS(1883),
    [sym_tls_client_close] = ACTIONS(1883),
    [sym_do_push] = ACTIONS(1883),
    [sym_do_push_timed] = ACTIONS(1883),
    [sym_do_release] = ACTIONS(1883),
    [sym_device_id] = ACTIONS(1883),
    [sym_device_info] = ACTIONS(1883),
    [sym_device_id_string] = ACTIONS(1883),
    [sym_device_standby] = ACTIONS(1883),
    [sym_device_wake] = ACTIONS(1883),
    [sym_timeline_create] = ACTIONS(1883),
    [sym_timeline_kill] = ACTIONS(1883),
    [sym_timeline_reload] = ACTIONS(1883),
    [sym_timeline_pause] = ACTIONS(1883),
    [sym_timeline_restart] = ACTIONS(1883),
    [sym_timeline_set] = ACTIONS(1883),
    [sym_timeline_get] = ACTIONS(1883),
    [sym_timeline_active] = ACTIONS(1883),
    [sym_timeline_dynamic_id] = ACTIONS(1883),
    [sym_variable_to_string] = ACTIONS(1883),
    [sym_string_to_variable] = ACTIONS(1883),
    [sym_length_variable_to_string] = ACTIONS(1883),
    [sym_type_cast] = ACTIONS(1883),
    [sym_variable_to_xml] = ACTIONS(1883),
    [sym_xml_to_variable] = ACTIONS(1883),
    [sym_length_variable_to_xml] = ACTIONS(1883),
    [sym_redirect_string] = ACTIONS(1883),
    [sym_random_number] = ACTIONS(1883),
    [sym_get_unique_id] = ACTIONS(1883),
    [sym_get_system_number] = ACTIONS(1883),
    [sym_set_system_number] = ACTIONS(1883),
    [sym_get_serial_number] = ACTIONS(1883),
    [sym_get_master_build] = ACTIONS(1883),
    [sym_set_validation_code] = ACTIONS(1883),
    [sym_reboot] = ACTIONS(1883),
    [sym_load_duet_module] = ACTIONS(1883),
    [sym_unload_duet_module] = ACTIONS(1883),
    [sym_duet_mem_size_set] = ACTIONS(1883),
    [sym_duet_mem_size_get] = ACTIONS(1883),
    [sym_dynamic_application_device] = ACTIONS(1883),
    [sym_dynamic_polled_port] = ACTIONS(1883),
    [sym_static_port_binding] = ACTIONS(1883),
    [sym_static_ip_binding] = ACTIONS(1883),
    [sym_get_dns_list] = ACTIONS(1883),
    [sym_set_dns_list] = ACTIONS(1883),
    [sym_get_ip_address] = ACTIONS(1883),
    [sym_set_ip_address] = ACTIONS(1883),
    [sym_get_url_list] = ACTIONS(1883),
    [sym_add_url_entry] = ACTIONS(1883),
    [sym_delete_url_entry] = ACTIONS(1883),
    [sym_abs_value] = ACTIONS(1883),
    [sym_max_value] = ACTIONS(1883),
    [sym_min_value] = ACTIONS(1883),
    [sym_exp_value] = ACTIONS(1883),
    [sym_log_value] = ACTIONS(1883),
    [sym_log10_value] = ACTIONS(1883),
    [sym_power_value] = ACTIONS(1883),
    [sym_sqrt_value] = ACTIONS(1883),
    [sym_raw_be] = ACTIONS(1883),
    [sym_raw_le] = ACTIONS(1883),
    [sym_internal_queue_size_set] = ACTIONS(1883),
    [sym_internal_queue_size_get] = ACTIONS(1883),
    [sym_internal_threshold_set] = ACTIONS(1883),
    [sym_internal_threshold_get] = ACTIONS(1883),
    [sym_rebuilt_event] = ACTIONS(1883),
    [sym_do_custom_event] = ACTIONS(1883),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1883),
    [sym_clkmgr_set_clk_source] = ACTIONS(1883),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1883),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1883),
    [sym_clkmgr_get_timezone] = ACTIONS(1883),
    [sym_clkmgr_set_timezone] = ACTIONS(1883),
    [sym_clkmgr_get_resync_period] = ACTIONS(1883),
    [sym_clkmgr_set_resync_period] = ACTIONS(1883),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1883),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1883),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1883),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1883),
    [sym_clkmgr_get_timeservers] = ACTIONS(1883),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1883),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1883),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1883),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1883),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1883),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1883),
    [sym_set_log_level] = ACTIONS(1883),
    [sym_get_log_level] = ACTIONS(1883),
    [sym_amx_log] = ACTIONS(1883),
    [sym_smtp_server_config_set] = ACTIONS(1883),
    [sym_smtp_server_config_get] = ACTIONS(1883),
    [sym_smtp_send] = ACTIONS(1883),
    [sym_validate_netlinx_account] = ACTIONS(1883),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1883),
    [sym_authenticate_certificate] = ACTIONS(1883),
    [sym_audit_netlinx_session_event] = ACTIONS(1883),
    [sym_audit_netlinx_generic_event] = ACTIONS(1883),
    [sym_get_available_flash_disk_space] = ACTIONS(1883),
    [sym_get_max_flash_disk_space] = ACTIONS(1883),
    [sym_data_create_feed] = ACTIONS(1883),
    [sym_data_delete_feed] = ACTIONS(1883),
    [sym_data_publish_feed] = ACTIONS(1883),
    [sym_data_get_published_feed] = ACTIONS(1883),
    [sym_data_add_record] = ACTIONS(1883),
    [sym_data_get_event_record] = ACTIONS(1883),
    [sym__wc_data_create_feed] = ACTIONS(1883),
    [sym__wc_data_add_record] = ACTIONS(1883),
    [sym__wc_data_get_event_record] = ACTIONS(1883),
    [sym_format] = ACTIONS(1883),
    [sym_astro_clock] = ACTIONS(1883),
    [sym_set_outdoor_temperature] = ACTIONS(1883),
    [sym_set_virtual_level_count] = ACTIONS(1883),
    [sym_set_virtual_channel_count] = ACTIONS(1883),
    [sym_set_virtual_port_count] = ACTIONS(1883),
    [sym__wc_encode_int] = ACTIONS(1883),
    [sym__wc_decode_int] = ACTIONS(1883),
    [sym__wc_int] = ACTIONS(1883),
    [sym__wc_to_ch_int] = ACTIONS(1883),
    [sym__ch_to_wc_int] = ACTIONS(1883),
    [sym__wc_find_string_int] = ACTIONS(1883),
    [sym__wc_left_string_int] = ACTIONS(1883),
    [sym__wc_lower_string_int] = ACTIONS(1883),
    [sym__wc_mid_string_int] = ACTIONS(1883),
    [sym__wc_remove_string_int] = ACTIONS(1883),
    [sym__wc_right_string_int] = ACTIONS(1883),
    [sym__wc_upper_string_int] = ACTIONS(1883),
    [sym__wc_compare_string_int] = ACTIONS(1883),
    [sym__wc_get_buffer_char_int] = ACTIONS(1883),
    [sym__wc_get_buffer_string_int] = ACTIONS(1883),
    [sym__wc_concat_string_int] = ACTIONS(1883),
    [sym__wc_file_open_int] = ACTIONS(1883),
    [sym__wc_file_close_int] = ACTIONS(1883),
    [sym__wc_file_read_int] = ACTIONS(1883),
    [sym__wc_file_read_line_int] = ACTIONS(1883),
    [sym__wc_file_write_int] = ACTIONS(1883),
    [sym__wc_file_write_line_int] = ACTIONS(1883),
    [aux_sym_preproc_include_token1] = ACTIONS(1883),
    [aux_sym_preproc_define_token1] = ACTIONS(1885),
    [aux_sym_preproc_warn_token1] = ACTIONS(1885),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1885),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1885),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1885),
    [anon_sym_LBRACK] = ACTIONS(1885),
    [anon_sym_LBRACE] = ACTIONS(1885),
    [anon_sym_RBRACE] = ACTIONS(1885),
    [anon_sym_LPAREN] = ACTIONS(1883),
    [anon_sym_BANG] = ACTIONS(1885),
    [anon_sym_TILDE] = ACTIONS(1885),
    [anon_sym_DASH] = ACTIONS(1883),
    [anon_sym_PLUS] = ACTIONS(1883),
    [anon_sym_DASH_DASH] = ACTIONS(1885),
    [anon_sym_PLUS_PLUS] = ACTIONS(1885),
    [anon_sym_DQUOTE] = ACTIONS(1885),
    [anon_sym_SQUOTE] = ACTIONS(1885),
    [sym_number_literal] = ACTIONS(1885),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(546)] = {
    [sym_identifier] = ACTIONS(1887),
    [sym_define_call_keyword] = ACTIONS(1887),
    [sym_define_function_keyword] = ACTIONS(1887),
    [sym_define_library_function_keyword] = ACTIONS(1887),
    [sym_define_module_keyword] = ACTIONS(1887),
    [sym_char_keyword] = ACTIONS(1887),
    [sym_widechar_keyword] = ACTIONS(1887),
    [sym_integer_keyword] = ACTIONS(1887),
    [sym_sinteger_keyword] = ACTIONS(1887),
    [sym_long_keyword] = ACTIONS(1887),
    [sym_slong_keyword] = ACTIONS(1887),
    [sym_float_keyword] = ACTIONS(1887),
    [sym_double_keyword] = ACTIONS(1887),
    [sym_variant_keyword] = ACTIONS(1887),
    [sym_variantarray_keyword] = ACTIONS(1887),
    [sym_local_var_keyword] = ACTIONS(1887),
    [sym_stack_var_keyword] = ACTIONS(1887),
    [sym_constant_keyword] = ACTIONS(1887),
    [sym_volatile_keyword] = ACTIONS(1887),
    [sym_non_volatile_keyword] = ACTIONS(1887),
    [sym_persistent_keyword] = ACTIONS(1887),
    [sym_dev_keyword] = ACTIONS(1887),
    [sym_devlev_keyword] = ACTIONS(1887),
    [sym_devchan_keyword] = ACTIONS(1887),
    [sym_if_keyword] = ACTIONS(1887),
    [sym_switch_keyword] = ACTIONS(1887),
    [sym_case_keyword] = ACTIONS(1887),
    [sym_default_keyword] = ACTIONS(1887),
    [sym_while_keyword] = ACTIONS(1887),
    [sym_for_keyword] = ACTIONS(1887),
    [sym_break_keyword] = ACTIONS(1887),
    [sym_continue_keyword] = ACTIONS(1887),
    [sym_return_keyword] = ACTIONS(1887),
    [sym_select_keyword] = ACTIONS(1887),
    [sym_active_keyword] = ACTIONS(1887),
    [sym_struct_keyword] = ACTIONS(1887),
    [sym_structure_keyword] = ACTIONS(1887),
    [aux_sym_on_keyword_token1] = ACTIONS(1887),
    [aux_sym_off_keyword_token1] = ACTIONS(1887),
    [sym_send_level_keyword] = ACTIONS(1887),
    [sym_send_string_keyword] = ACTIONS(1887),
    [sym_send_command_keyword] = ACTIONS(1887),
    [sym_clear_buffer_keyword] = ACTIONS(1887),
    [sym_create_buffer_keyword] = ACTIONS(1887),
    [sym_create_multi_buffer_keyword] = ACTIONS(1887),
    [sym_call_keyword] = ACTIONS(1887),
    [sym_system_call_keyword] = ACTIONS(1887),
    [sym_devchan_to_keyword] = ACTIONS(1887),
    [sym_devchan_min_to_keyword] = ACTIONS(1887),
    [sym_devchan_total_off_keyword] = ACTIONS(1887),
    [sym_devchan_pulse_keyword] = ACTIONS(1887),
    [sym_wait_keyword] = ACTIONS(1887),
    [sym_cancel_wait_keyword] = ACTIONS(1887),
    [sym_wait_until_keyword] = ACTIONS(1887),
    [sym_cancel_wait_until_keyword] = ACTIONS(1887),
    [sym_cancel_all_wait_keyword] = ACTIONS(1887),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1887),
    [sym_not] = ACTIONS(1887),
    [sym_bnot] = ACTIONS(1887),
    [sym___file__] = ACTIONS(1887),
    [sym___date__] = ACTIONS(1887),
    [sym___ldate__] = ACTIONS(1887),
    [sym___time__] = ACTIONS(1887),
    [sym___line__] = ACTIONS(1887),
    [sym___name__] = ACTIONS(1887),
    [sym___netlinx__] = ACTIONS(1887),
    [sym_netlinx_axi_version] = ACTIONS(1887),
    [sym_url_flg_tcp] = ACTIONS(1887),
    [sym_url_flg_accinfopresent] = ACTIONS(1887),
    [sym_url_flg_temp] = ACTIONS(1887),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1887),
    [sym_url_flg_stat_mask] = ACTIONS(1887),
    [sym_url_flg_stat_lookup] = ACTIONS(1887),
    [sym_url_flg_stat_connecting] = ACTIONS(1887),
    [sym_url_flg_stat_waiting] = ACTIONS(1887),
    [sym_url_flg_stat_connected] = ACTIONS(1887),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1887),
    [sym_first_virtual_device] = ACTIONS(1887),
    [sym_first_local_port] = ACTIONS(1887),
    [sym_dynamic_virtual_device] = ACTIONS(1887),
    [sym_timeline_once] = ACTIONS(1887),
    [sym_timeline_repeat] = ACTIONS(1887),
    [sym_timeline_absolute] = ACTIONS(1887),
    [sym_timeline_relative] = ACTIONS(1887),
    [sym_do_push_timed_infinite] = ACTIONS(1887),
    [sym_source_type_no_address] = ACTIONS(1887),
    [sym_source_type_neuron_id] = ACTIONS(1887),
    [sym_source_type_ip_address] = ACTIONS(1887),
    [sym_source_type_axlink] = ACTIONS(1887),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1887),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1887),
    [sym_source_type_ip_socket_address] = ACTIONS(1887),
    [sym_source_type_rs232] = ACTIONS(1887),
    [sym_source_type_internal] = ACTIONS(1887),
    [sym_source_type_mac_address] = ACTIONS(1887),
    [sym_source_type_ipv4_port] = ACTIONS(1887),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1887),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1887),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1887),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1887),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1887),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1887),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1887),
    [sym_true] = ACTIONS(1887),
    [sym_false] = ACTIONS(1887),
    [sym_file_read_only] = ACTIONS(1887),
    [sym_file_rw_new] = ACTIONS(1887),
    [sym_file_rw_append] = ACTIONS(1887),
    [sym_ip_tcp] = ACTIONS(1887),
    [sym_ip_udp] = ACTIONS(1887),
    [sym_ip_udp_2way] = ACTIONS(1887),
    [sym_xml_encode_types] = ACTIONS(1887),
    [sym_xml_encode_char_as_list] = ACTIONS(1887),
    [sym_xml_encode_le] = ACTIONS(1887),
    [sym_xml_decode_types] = ACTIONS(1887),
    [sym_xml_decode_no_preserve] = ACTIONS(1887),
    [sym_duet_dev_type_display_device] = ACTIONS(1887),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1887),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1887),
    [sym_duet_dev_type_io_device] = ACTIONS(1887),
    [sym_duet_dev_type_relay_device] = ACTIONS(1887),
    [sym_duet_dev_type_ups] = ACTIONS(1887),
    [sym_duet_dev_type_amplifier] = ACTIONS(1887),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1887),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1887),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1887),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1887),
    [sym_duet_dev_type_camera] = ACTIONS(1887),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1887),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1887),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1887),
    [sym_duet_dev_type_dss] = ACTIONS(1887),
    [sym_duet_dev_type_dvr] = ACTIONS(1887),
    [sym_duet_dev_type_disc_device] = ACTIONS(1887),
    [sym_duet_dev_type_document_camera] = ACTIONS(1887),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1887),
    [sym_duet_dev_type_hvac] = ACTIONS(1887),
    [sym_duet_dev_type_keypad] = ACTIONS(1887),
    [sym_duet_dev_type_light] = ACTIONS(1887),
    [sym_duet_dev_type_monitor] = ACTIONS(1887),
    [sym_duet_dev_type_motor] = ACTIONS(1887),
    [sym_duet_dev_type_multi_window] = ACTIONS(1887),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1887),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1887),
    [sym_duet_dev_type_receiver] = ACTIONS(1887),
    [sym_duet_dev_type_security_system] = ACTIONS(1887),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1887),
    [sym_duet_dev_type_settop_box] = ACTIONS(1887),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1887),
    [sym_duet_dev_type_switcher] = ACTIONS(1887),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1887),
    [sym_duet_dev_type_tv] = ACTIONS(1887),
    [sym_duet_dev_type_utility] = ACTIONS(1887),
    [sym_duet_dev_type_vcr] = ACTIONS(1887),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1887),
    [sym_duet_dev_type_video_processor] = ACTIONS(1887),
    [sym_duet_dev_type_video_projector] = ACTIONS(1887),
    [sym_duet_dev_type_video_wall] = ACTIONS(1887),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1887),
    [sym_duet_dev_type_weather] = ACTIONS(1887),
    [sym_duet_dev_not_polled] = ACTIONS(1887),
    [sym_duet_dev_polled] = ACTIONS(1887),
    [sym_clkmgr_mode_network] = ACTIONS(1887),
    [sym_clkmgr_mode_standalone] = ACTIONS(1887),
    [sym_normal_standby] = ACTIONS(1887),
    [sym_normal_wake] = ACTIONS(1887),
    [sym_ip_multicast_ttl_option] = ACTIONS(1887),
    [sym_ip_ttl_subnet] = ACTIONS(1887),
    [sym_ip_ttl_site] = ACTIONS(1887),
    [sym_ip_ttl_region] = ACTIONS(1887),
    [sym_ip_ttl_continent] = ACTIONS(1887),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1887),
    [sym_ip_nodelay_on] = ACTIONS(1887),
    [sym_ip_nodelay_off] = ACTIONS(1887),
    [sym_amx_error] = ACTIONS(1887),
    [sym_amx_warning] = ACTIONS(1887),
    [sym_amx_info] = ACTIONS(1887),
    [sym_amx_debug] = ACTIONS(1887),
    [sym_netlinx_login_success] = ACTIONS(1887),
    [sym_netlinx_login_fail] = ACTIONS(1887),
    [sym_netlinx_logout] = ACTIONS(1887),
    [sym_null_str] = ACTIONS(1887),
    [sym_smtp_address] = ACTIONS(1887),
    [sym_smtp_port_number] = ACTIONS(1887),
    [sym_smtp_username] = ACTIONS(1887),
    [sym_smtp_password] = ACTIONS(1887),
    [sym_smtp_require_tls] = ACTIONS(1887),
    [sym_smtp_from] = ACTIONS(1887),
    [sym_smtp_tls_true] = ACTIONS(1887),
    [sym_smtp_tls_false] = ACTIONS(1887),
    [sym_listview_on_row_select_event] = ACTIONS(1887),
    [sym_data_structure_unknown] = ACTIONS(1887),
    [sym_data_structure_datafeed] = ACTIONS(1887),
    [sym_data_structure_datarecordset] = ACTIONS(1887),
    [sym_data_structure_datarecord] = ACTIONS(1887),
    [sym_data_structure_datafield] = ACTIONS(1887),
    [sym_data_max_content_field_count] = ACTIONS(1887),
    [sym_data_max_metadata_field_count] = ACTIONS(1887),
    [sym_data_max_id_length] = ACTIONS(1887),
    [sym_data_max_name_length] = ACTIONS(1887),
    [sym_data_max_description_length] = ACTIONS(1887),
    [sym_data_max_type_length] = ACTIONS(1887),
    [sym_data_max_format_length] = ACTIONS(1887),
    [sym_data_max_source_length] = ACTIONS(1887),
    [sym_data_max_value_length] = ACTIONS(1887),
    [sym_data_max_label_length] = ACTIONS(1887),
    [sym_data_max_url_length] = ACTIONS(1887),
    [sym_max_last_login_info_length] = ACTIONS(1887),
    [sym_data_type_unknown] = ACTIONS(1887),
    [sym_data_type_string] = ACTIONS(1887),
    [sym_data_type_datetime] = ACTIONS(1887),
    [sym_data_type_date] = ACTIONS(1887),
    [sym_data_type_time] = ACTIONS(1887),
    [sym_data_type_image] = ACTIONS(1887),
    [sym_data_format_url] = ACTIONS(1887),
    [sym_data_format_phone] = ACTIONS(1887),
    [sym_data_format_email] = ACTIONS(1887),
    [sym_data_format_iso8601] = ACTIONS(1887),
    [sym_auditlog_priv] = ACTIONS(1887),
    [sym_configuration_priv] = ACTIONS(1887),
    [sym_deviceconfig_priv] = ACTIONS(1887),
    [sym_ftp_priv] = ACTIONS(1887),
    [sym_http_priv] = ACTIONS(1887),
    [sym_networkconfig_priv] = ACTIONS(1887),
    [sym_programport_priv] = ACTIONS(1887),
    [sym_remoteui_priv] = ACTIONS(1887),
    [sym_securitycontrol_priv] = ACTIONS(1887),
    [sym_softwaremanagement_priv] = ACTIONS(1887),
    [sym_terminal_priv] = ACTIONS(1887),
    [sym_tpadmin_priv] = ACTIONS(1887),
    [sym_user1_priv] = ACTIONS(1887),
    [sym_user2_priv] = ACTIONS(1887),
    [sym_user3_priv] = ACTIONS(1887),
    [sym_user4_priv] = ACTIONS(1887),
    [sym_usermanagement_priv] = ACTIONS(1887),
    [sym_valid_account] = ACTIONS(1887),
    [sym_err_name_invalid] = ACTIONS(1887),
    [sym_err_password_invalid] = ACTIONS(1887),
    [sym_err_invalid_account] = ACTIONS(1887),
    [sym_err_no_authorization] = ACTIONS(1887),
    [sym_err_invalid_parameter] = ACTIONS(1887),
    [sym_err_user_locked_out] = ACTIONS(1887),
    [sym_err_user_account_expired] = ACTIONS(1887),
    [sym_err_general_error] = ACTIONS(1887),
    [sym_err_authentication_error] = ACTIONS(1887),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1887),
    [sym_tls_validate_certificate] = ACTIONS(1887),
    [sym_tdata] = ACTIONS(1887),
    [sym_tchannel] = ACTIONS(1887),
    [sym_tlevel] = ACTIONS(1887),
    [sym_tbutton] = ACTIONS(1887),
    [sym_ttimeline] = ACTIONS(1887),
    [sym_tcustom] = ACTIONS(1887),
    [sym_url_struct] = ACTIONS(1887),
    [sym_dns_struct] = ACTIONS(1887),
    [sym_ip_address_struct] = ACTIONS(1887),
    [sym_dev_info_struct] = ACTIONS(1887),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1887),
    [sym_data_feed] = ACTIONS(1887),
    [sym_data_field] = ACTIONS(1887),
    [sym_data_record] = ACTIONS(1887),
    [sym_wc_data_feed] = ACTIONS(1887),
    [sym_wc_data_field] = ACTIONS(1887),
    [sym_wc_data_record] = ACTIONS(1887),
    [sym_last_login_info] = ACTIONS(1887),
    [sym____reserved___] = ACTIONS(1887),
    [sym_date] = ACTIONS(1887),
    [sym_day] = ACTIONS(1887),
    [sym_time] = ACTIONS(1887),
    [sym_ldate] = ACTIONS(1887),
    [sym_push_device] = ACTIONS(1887),
    [sym_push_channel] = ACTIONS(1887),
    [sym_push_devchan] = ACTIONS(1887),
    [sym_release_device] = ACTIONS(1887),
    [sym_release_channel] = ACTIONS(1887),
    [sym_release_devchan] = ACTIONS(1887),
    [sym_master_slot] = ACTIONS(1887),
    [sym_get_pulse_time] = ACTIONS(1887),
    [sym_get_timer] = ACTIONS(1887),
    [sym_master_sn] = ACTIONS(1887),
    [sym_system_number] = ACTIONS(1887),
    [sym_button] = ACTIONS(1887),
    [sym_dv_channel] = ACTIONS(1887),
    [sym_data] = ACTIONS(1887),
    [sym_level] = ACTIONS(1887),
    [sym_timeline] = ACTIONS(1887),
    [sym_channel] = ACTIONS(1887),
    [sym_custom] = ACTIONS(1887),
    [sym_atoi] = ACTIONS(1887),
    [sym_atol] = ACTIONS(1887),
    [sym_atof] = ACTIONS(1887),
    [sym_itoa] = ACTIONS(1887),
    [sym_itohex] = ACTIONS(1887),
    [sym_hextoi] = ACTIONS(1887),
    [sym_ftoa] = ACTIONS(1887),
    [sym_find_string] = ACTIONS(1887),
    [sym_left_string] = ACTIONS(1887),
    [sym_right_string] = ACTIONS(1887),
    [sym_mid_string] = ACTIONS(1887),
    [sym_remove_string] = ACTIONS(1887),
    [sym_get_buffer_string] = ACTIONS(1887),
    [sym_get_buffer_char] = ACTIONS(1887),
    [sym_get_multi_buffer_string] = ACTIONS(1887),
    [sym_length_string] = ACTIONS(1887),
    [sym_max_length_string] = ACTIONS(1887),
    [sym_set_length_string] = ACTIONS(1887),
    [sym_compare_string] = ACTIONS(1887),
    [sym_upper_string] = ACTIONS(1887),
    [sym_lower_string] = ACTIONS(1887),
    [sym_get_last] = ACTIONS(1887),
    [sym_date_to_day] = ACTIONS(1887),
    [sym_date_to_month] = ACTIONS(1887),
    [sym_date_to_year] = ACTIONS(1887),
    [sym_time_to_hour] = ACTIONS(1887),
    [sym_time_to_minute] = ACTIONS(1887),
    [sym_time_to_second] = ACTIONS(1887),
    [sym_day_of_week] = ACTIONS(1887),
    [sym_set_pulse_time] = ACTIONS(1887),
    [sym_set_timer] = ACTIONS(1887),
    [sym_file_open] = ACTIONS(1887),
    [sym_file_close] = ACTIONS(1887),
    [sym_file_read] = ACTIONS(1887),
    [sym_file_read_line] = ACTIONS(1887),
    [sym_file_write] = ACTIONS(1887),
    [sym_file_write_line] = ACTIONS(1887),
    [sym_file_copy] = ACTIONS(1887),
    [sym_file_delete] = ACTIONS(1887),
    [sym_file_rename] = ACTIONS(1887),
    [sym_file_dir] = ACTIONS(1887),
    [sym_file_getdir] = ACTIONS(1887),
    [sym_file_setdir] = ACTIONS(1887),
    [sym_file_create_dir] = ACTIONS(1887),
    [sym_file_remove_dir] = ACTIONS(1887),
    [sym_file_seek] = ACTIONS(1887),
    [sym_length_array] = ACTIONS(1887),
    [sym_max_length_array] = ACTIONS(1887),
    [sym_set_length_array] = ACTIONS(1887),
    [sym_ip_client_open] = ACTIONS(1887),
    [sym_ip_client_close] = ACTIONS(1887),
    [sym_ip_server_open] = ACTIONS(1887),
    [sym_ip_server_close] = ACTIONS(1887),
    [sym_ip_bound_client_open] = ACTIONS(1887),
    [sym_ip_mc_server_open] = ACTIONS(1887),
    [sym_ip_set_option] = ACTIONS(1887),
    [sym_ssh_client_open] = ACTIONS(1887),
    [sym_ssh_client_close] = ACTIONS(1887),
    [sym_tls_client_open] = ACTIONS(1887),
    [sym_tls_client_close] = ACTIONS(1887),
    [sym_do_push] = ACTIONS(1887),
    [sym_do_push_timed] = ACTIONS(1887),
    [sym_do_release] = ACTIONS(1887),
    [sym_device_id] = ACTIONS(1887),
    [sym_device_info] = ACTIONS(1887),
    [sym_device_id_string] = ACTIONS(1887),
    [sym_device_standby] = ACTIONS(1887),
    [sym_device_wake] = ACTIONS(1887),
    [sym_timeline_create] = ACTIONS(1887),
    [sym_timeline_kill] = ACTIONS(1887),
    [sym_timeline_reload] = ACTIONS(1887),
    [sym_timeline_pause] = ACTIONS(1887),
    [sym_timeline_restart] = ACTIONS(1887),
    [sym_timeline_set] = ACTIONS(1887),
    [sym_timeline_get] = ACTIONS(1887),
    [sym_timeline_active] = ACTIONS(1887),
    [sym_timeline_dynamic_id] = ACTIONS(1887),
    [sym_variable_to_string] = ACTIONS(1887),
    [sym_string_to_variable] = ACTIONS(1887),
    [sym_length_variable_to_string] = ACTIONS(1887),
    [sym_type_cast] = ACTIONS(1887),
    [sym_variable_to_xml] = ACTIONS(1887),
    [sym_xml_to_variable] = ACTIONS(1887),
    [sym_length_variable_to_xml] = ACTIONS(1887),
    [sym_redirect_string] = ACTIONS(1887),
    [sym_random_number] = ACTIONS(1887),
    [sym_get_unique_id] = ACTIONS(1887),
    [sym_get_system_number] = ACTIONS(1887),
    [sym_set_system_number] = ACTIONS(1887),
    [sym_get_serial_number] = ACTIONS(1887),
    [sym_get_master_build] = ACTIONS(1887),
    [sym_set_validation_code] = ACTIONS(1887),
    [sym_reboot] = ACTIONS(1887),
    [sym_load_duet_module] = ACTIONS(1887),
    [sym_unload_duet_module] = ACTIONS(1887),
    [sym_duet_mem_size_set] = ACTIONS(1887),
    [sym_duet_mem_size_get] = ACTIONS(1887),
    [sym_dynamic_application_device] = ACTIONS(1887),
    [sym_dynamic_polled_port] = ACTIONS(1887),
    [sym_static_port_binding] = ACTIONS(1887),
    [sym_static_ip_binding] = ACTIONS(1887),
    [sym_get_dns_list] = ACTIONS(1887),
    [sym_set_dns_list] = ACTIONS(1887),
    [sym_get_ip_address] = ACTIONS(1887),
    [sym_set_ip_address] = ACTIONS(1887),
    [sym_get_url_list] = ACTIONS(1887),
    [sym_add_url_entry] = ACTIONS(1887),
    [sym_delete_url_entry] = ACTIONS(1887),
    [sym_abs_value] = ACTIONS(1887),
    [sym_max_value] = ACTIONS(1887),
    [sym_min_value] = ACTIONS(1887),
    [sym_exp_value] = ACTIONS(1887),
    [sym_log_value] = ACTIONS(1887),
    [sym_log10_value] = ACTIONS(1887),
    [sym_power_value] = ACTIONS(1887),
    [sym_sqrt_value] = ACTIONS(1887),
    [sym_raw_be] = ACTIONS(1887),
    [sym_raw_le] = ACTIONS(1887),
    [sym_internal_queue_size_set] = ACTIONS(1887),
    [sym_internal_queue_size_get] = ACTIONS(1887),
    [sym_internal_threshold_set] = ACTIONS(1887),
    [sym_internal_threshold_get] = ACTIONS(1887),
    [sym_rebuilt_event] = ACTIONS(1887),
    [sym_do_custom_event] = ACTIONS(1887),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1887),
    [sym_clkmgr_set_clk_source] = ACTIONS(1887),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1887),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1887),
    [sym_clkmgr_get_timezone] = ACTIONS(1887),
    [sym_clkmgr_set_timezone] = ACTIONS(1887),
    [sym_clkmgr_get_resync_period] = ACTIONS(1887),
    [sym_clkmgr_set_resync_period] = ACTIONS(1887),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1887),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1887),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1887),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1887),
    [sym_clkmgr_get_timeservers] = ACTIONS(1887),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1887),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1887),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1887),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1887),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1887),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1887),
    [sym_set_log_level] = ACTIONS(1887),
    [sym_get_log_level] = ACTIONS(1887),
    [sym_amx_log] = ACTIONS(1887),
    [sym_smtp_server_config_set] = ACTIONS(1887),
    [sym_smtp_server_config_get] = ACTIONS(1887),
    [sym_smtp_send] = ACTIONS(1887),
    [sym_validate_netlinx_account] = ACTIONS(1887),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1887),
    [sym_authenticate_certificate] = ACTIONS(1887),
    [sym_audit_netlinx_session_event] = ACTIONS(1887),
    [sym_audit_netlinx_generic_event] = ACTIONS(1887),
    [sym_get_available_flash_disk_space] = ACTIONS(1887),
    [sym_get_max_flash_disk_space] = ACTIONS(1887),
    [sym_data_create_feed] = ACTIONS(1887),
    [sym_data_delete_feed] = ACTIONS(1887),
    [sym_data_publish_feed] = ACTIONS(1887),
    [sym_data_get_published_feed] = ACTIONS(1887),
    [sym_data_add_record] = ACTIONS(1887),
    [sym_data_get_event_record] = ACTIONS(1887),
    [sym__wc_data_create_feed] = ACTIONS(1887),
    [sym__wc_data_add_record] = ACTIONS(1887),
    [sym__wc_data_get_event_record] = ACTIONS(1887),
    [sym_format] = ACTIONS(1887),
    [sym_astro_clock] = ACTIONS(1887),
    [sym_set_outdoor_temperature] = ACTIONS(1887),
    [sym_set_virtual_level_count] = ACTIONS(1887),
    [sym_set_virtual_channel_count] = ACTIONS(1887),
    [sym_set_virtual_port_count] = ACTIONS(1887),
    [sym__wc_encode_int] = ACTIONS(1887),
    [sym__wc_decode_int] = ACTIONS(1887),
    [sym__wc_int] = ACTIONS(1887),
    [sym__wc_to_ch_int] = ACTIONS(1887),
    [sym__ch_to_wc_int] = ACTIONS(1887),
    [sym__wc_find_string_int] = ACTIONS(1887),
    [sym__wc_left_string_int] = ACTIONS(1887),
    [sym__wc_lower_string_int] = ACTIONS(1887),
    [sym__wc_mid_string_int] = ACTIONS(1887),
    [sym__wc_remove_string_int] = ACTIONS(1887),
    [sym__wc_right_string_int] = ACTIONS(1887),
    [sym__wc_upper_string_int] = ACTIONS(1887),
    [sym__wc_compare_string_int] = ACTIONS(1887),
    [sym__wc_get_buffer_char_int] = ACTIONS(1887),
    [sym__wc_get_buffer_string_int] = ACTIONS(1887),
    [sym__wc_concat_string_int] = ACTIONS(1887),
    [sym__wc_file_open_int] = ACTIONS(1887),
    [sym__wc_file_close_int] = ACTIONS(1887),
    [sym__wc_file_read_int] = ACTIONS(1887),
    [sym__wc_file_read_line_int] = ACTIONS(1887),
    [sym__wc_file_write_int] = ACTIONS(1887),
    [sym__wc_file_write_line_int] = ACTIONS(1887),
    [aux_sym_preproc_include_token1] = ACTIONS(1887),
    [aux_sym_preproc_define_token1] = ACTIONS(1889),
    [aux_sym_preproc_warn_token1] = ACTIONS(1889),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1889),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1889),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1889),
    [anon_sym_LBRACK] = ACTIONS(1889),
    [anon_sym_LBRACE] = ACTIONS(1889),
    [anon_sym_RBRACE] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(1887),
    [anon_sym_BANG] = ACTIONS(1889),
    [anon_sym_TILDE] = ACTIONS(1889),
    [anon_sym_DASH] = ACTIONS(1887),
    [anon_sym_PLUS] = ACTIONS(1887),
    [anon_sym_DASH_DASH] = ACTIONS(1889),
    [anon_sym_PLUS_PLUS] = ACTIONS(1889),
    [anon_sym_DQUOTE] = ACTIONS(1889),
    [anon_sym_SQUOTE] = ACTIONS(1889),
    [sym_number_literal] = ACTIONS(1889),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(547)] = {
    [sym_identifier] = ACTIONS(1973),
    [sym_define_call_keyword] = ACTIONS(1973),
    [sym_define_function_keyword] = ACTIONS(1973),
    [sym_define_library_function_keyword] = ACTIONS(1973),
    [sym_define_module_keyword] = ACTIONS(1973),
    [sym_char_keyword] = ACTIONS(1973),
    [sym_widechar_keyword] = ACTIONS(1973),
    [sym_integer_keyword] = ACTIONS(1973),
    [sym_sinteger_keyword] = ACTIONS(1973),
    [sym_long_keyword] = ACTIONS(1973),
    [sym_slong_keyword] = ACTIONS(1973),
    [sym_float_keyword] = ACTIONS(1973),
    [sym_double_keyword] = ACTIONS(1973),
    [sym_variant_keyword] = ACTIONS(1973),
    [sym_variantarray_keyword] = ACTIONS(1973),
    [sym_local_var_keyword] = ACTIONS(1973),
    [sym_stack_var_keyword] = ACTIONS(1973),
    [sym_constant_keyword] = ACTIONS(1973),
    [sym_volatile_keyword] = ACTIONS(1973),
    [sym_non_volatile_keyword] = ACTIONS(1973),
    [sym_persistent_keyword] = ACTIONS(1973),
    [sym_dev_keyword] = ACTIONS(1973),
    [sym_devlev_keyword] = ACTIONS(1973),
    [sym_devchan_keyword] = ACTIONS(1973),
    [sym_if_keyword] = ACTIONS(1973),
    [sym_switch_keyword] = ACTIONS(1973),
    [sym_case_keyword] = ACTIONS(1973),
    [sym_default_keyword] = ACTIONS(1973),
    [sym_while_keyword] = ACTIONS(1973),
    [sym_for_keyword] = ACTIONS(1973),
    [sym_break_keyword] = ACTIONS(1973),
    [sym_continue_keyword] = ACTIONS(1973),
    [sym_return_keyword] = ACTIONS(1973),
    [sym_select_keyword] = ACTIONS(1973),
    [sym_active_keyword] = ACTIONS(1973),
    [sym_struct_keyword] = ACTIONS(1973),
    [sym_structure_keyword] = ACTIONS(1973),
    [aux_sym_on_keyword_token1] = ACTIONS(1973),
    [aux_sym_off_keyword_token1] = ACTIONS(1973),
    [sym_send_level_keyword] = ACTIONS(1973),
    [sym_send_string_keyword] = ACTIONS(1973),
    [sym_send_command_keyword] = ACTIONS(1973),
    [sym_clear_buffer_keyword] = ACTIONS(1973),
    [sym_create_buffer_keyword] = ACTIONS(1973),
    [sym_create_multi_buffer_keyword] = ACTIONS(1973),
    [sym_call_keyword] = ACTIONS(1973),
    [sym_system_call_keyword] = ACTIONS(1973),
    [sym_devchan_to_keyword] = ACTIONS(1973),
    [sym_devchan_min_to_keyword] = ACTIONS(1973),
    [sym_devchan_total_off_keyword] = ACTIONS(1973),
    [sym_devchan_pulse_keyword] = ACTIONS(1973),
    [sym_wait_keyword] = ACTIONS(1973),
    [sym_cancel_wait_keyword] = ACTIONS(1973),
    [sym_wait_until_keyword] = ACTIONS(1973),
    [sym_cancel_wait_until_keyword] = ACTIONS(1973),
    [sym_cancel_all_wait_keyword] = ACTIONS(1973),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1973),
    [sym_not] = ACTIONS(1973),
    [sym_bnot] = ACTIONS(1973),
    [sym___file__] = ACTIONS(1973),
    [sym___date__] = ACTIONS(1973),
    [sym___ldate__] = ACTIONS(1973),
    [sym___time__] = ACTIONS(1973),
    [sym___line__] = ACTIONS(1973),
    [sym___name__] = ACTIONS(1973),
    [sym___netlinx__] = ACTIONS(1973),
    [sym_netlinx_axi_version] = ACTIONS(1973),
    [sym_url_flg_tcp] = ACTIONS(1973),
    [sym_url_flg_accinfopresent] = ACTIONS(1973),
    [sym_url_flg_temp] = ACTIONS(1973),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1973),
    [sym_url_flg_stat_mask] = ACTIONS(1973),
    [sym_url_flg_stat_lookup] = ACTIONS(1973),
    [sym_url_flg_stat_connecting] = ACTIONS(1973),
    [sym_url_flg_stat_waiting] = ACTIONS(1973),
    [sym_url_flg_stat_connected] = ACTIONS(1973),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1973),
    [sym_first_virtual_device] = ACTIONS(1973),
    [sym_first_local_port] = ACTIONS(1973),
    [sym_dynamic_virtual_device] = ACTIONS(1973),
    [sym_timeline_once] = ACTIONS(1973),
    [sym_timeline_repeat] = ACTIONS(1973),
    [sym_timeline_absolute] = ACTIONS(1973),
    [sym_timeline_relative] = ACTIONS(1973),
    [sym_do_push_timed_infinite] = ACTIONS(1973),
    [sym_source_type_no_address] = ACTIONS(1973),
    [sym_source_type_neuron_id] = ACTIONS(1973),
    [sym_source_type_ip_address] = ACTIONS(1973),
    [sym_source_type_axlink] = ACTIONS(1973),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1973),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1973),
    [sym_source_type_ip_socket_address] = ACTIONS(1973),
    [sym_source_type_rs232] = ACTIONS(1973),
    [sym_source_type_internal] = ACTIONS(1973),
    [sym_source_type_mac_address] = ACTIONS(1973),
    [sym_source_type_ipv4_port] = ACTIONS(1973),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1973),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1973),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1973),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1973),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1973),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1973),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1973),
    [sym_true] = ACTIONS(1973),
    [sym_false] = ACTIONS(1973),
    [sym_file_read_only] = ACTIONS(1973),
    [sym_file_rw_new] = ACTIONS(1973),
    [sym_file_rw_append] = ACTIONS(1973),
    [sym_ip_tcp] = ACTIONS(1973),
    [sym_ip_udp] = ACTIONS(1973),
    [sym_ip_udp_2way] = ACTIONS(1973),
    [sym_xml_encode_types] = ACTIONS(1973),
    [sym_xml_encode_char_as_list] = ACTIONS(1973),
    [sym_xml_encode_le] = ACTIONS(1973),
    [sym_xml_decode_types] = ACTIONS(1973),
    [sym_xml_decode_no_preserve] = ACTIONS(1973),
    [sym_duet_dev_type_display_device] = ACTIONS(1973),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1973),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1973),
    [sym_duet_dev_type_io_device] = ACTIONS(1973),
    [sym_duet_dev_type_relay_device] = ACTIONS(1973),
    [sym_duet_dev_type_ups] = ACTIONS(1973),
    [sym_duet_dev_type_amplifier] = ACTIONS(1973),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1973),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1973),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1973),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1973),
    [sym_duet_dev_type_camera] = ACTIONS(1973),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1973),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1973),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1973),
    [sym_duet_dev_type_dss] = ACTIONS(1973),
    [sym_duet_dev_type_dvr] = ACTIONS(1973),
    [sym_duet_dev_type_disc_device] = ACTIONS(1973),
    [sym_duet_dev_type_document_camera] = ACTIONS(1973),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1973),
    [sym_duet_dev_type_hvac] = ACTIONS(1973),
    [sym_duet_dev_type_keypad] = ACTIONS(1973),
    [sym_duet_dev_type_light] = ACTIONS(1973),
    [sym_duet_dev_type_monitor] = ACTIONS(1973),
    [sym_duet_dev_type_motor] = ACTIONS(1973),
    [sym_duet_dev_type_multi_window] = ACTIONS(1973),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1973),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1973),
    [sym_duet_dev_type_receiver] = ACTIONS(1973),
    [sym_duet_dev_type_security_system] = ACTIONS(1973),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1973),
    [sym_duet_dev_type_settop_box] = ACTIONS(1973),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1973),
    [sym_duet_dev_type_switcher] = ACTIONS(1973),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1973),
    [sym_duet_dev_type_tv] = ACTIONS(1973),
    [sym_duet_dev_type_utility] = ACTIONS(1973),
    [sym_duet_dev_type_vcr] = ACTIONS(1973),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1973),
    [sym_duet_dev_type_video_processor] = ACTIONS(1973),
    [sym_duet_dev_type_video_projector] = ACTIONS(1973),
    [sym_duet_dev_type_video_wall] = ACTIONS(1973),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1973),
    [sym_duet_dev_type_weather] = ACTIONS(1973),
    [sym_duet_dev_not_polled] = ACTIONS(1973),
    [sym_duet_dev_polled] = ACTIONS(1973),
    [sym_clkmgr_mode_network] = ACTIONS(1973),
    [sym_clkmgr_mode_standalone] = ACTIONS(1973),
    [sym_normal_standby] = ACTIONS(1973),
    [sym_normal_wake] = ACTIONS(1973),
    [sym_ip_multicast_ttl_option] = ACTIONS(1973),
    [sym_ip_ttl_subnet] = ACTIONS(1973),
    [sym_ip_ttl_site] = ACTIONS(1973),
    [sym_ip_ttl_region] = ACTIONS(1973),
    [sym_ip_ttl_continent] = ACTIONS(1973),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1973),
    [sym_ip_nodelay_on] = ACTIONS(1973),
    [sym_ip_nodelay_off] = ACTIONS(1973),
    [sym_amx_error] = ACTIONS(1973),
    [sym_amx_warning] = ACTIONS(1973),
    [sym_amx_info] = ACTIONS(1973),
    [sym_amx_debug] = ACTIONS(1973),
    [sym_netlinx_login_success] = ACTIONS(1973),
    [sym_netlinx_login_fail] = ACTIONS(1973),
    [sym_netlinx_logout] = ACTIONS(1973),
    [sym_null_str] = ACTIONS(1973),
    [sym_smtp_address] = ACTIONS(1973),
    [sym_smtp_port_number] = ACTIONS(1973),
    [sym_smtp_username] = ACTIONS(1973),
    [sym_smtp_password] = ACTIONS(1973),
    [sym_smtp_require_tls] = ACTIONS(1973),
    [sym_smtp_from] = ACTIONS(1973),
    [sym_smtp_tls_true] = ACTIONS(1973),
    [sym_smtp_tls_false] = ACTIONS(1973),
    [sym_listview_on_row_select_event] = ACTIONS(1973),
    [sym_data_structure_unknown] = ACTIONS(1973),
    [sym_data_structure_datafeed] = ACTIONS(1973),
    [sym_data_structure_datarecordset] = ACTIONS(1973),
    [sym_data_structure_datarecord] = ACTIONS(1973),
    [sym_data_structure_datafield] = ACTIONS(1973),
    [sym_data_max_content_field_count] = ACTIONS(1973),
    [sym_data_max_metadata_field_count] = ACTIONS(1973),
    [sym_data_max_id_length] = ACTIONS(1973),
    [sym_data_max_name_length] = ACTIONS(1973),
    [sym_data_max_description_length] = ACTIONS(1973),
    [sym_data_max_type_length] = ACTIONS(1973),
    [sym_data_max_format_length] = ACTIONS(1973),
    [sym_data_max_source_length] = ACTIONS(1973),
    [sym_data_max_value_length] = ACTIONS(1973),
    [sym_data_max_label_length] = ACTIONS(1973),
    [sym_data_max_url_length] = ACTIONS(1973),
    [sym_max_last_login_info_length] = ACTIONS(1973),
    [sym_data_type_unknown] = ACTIONS(1973),
    [sym_data_type_string] = ACTIONS(1973),
    [sym_data_type_datetime] = ACTIONS(1973),
    [sym_data_type_date] = ACTIONS(1973),
    [sym_data_type_time] = ACTIONS(1973),
    [sym_data_type_image] = ACTIONS(1973),
    [sym_data_format_url] = ACTIONS(1973),
    [sym_data_format_phone] = ACTIONS(1973),
    [sym_data_format_email] = ACTIONS(1973),
    [sym_data_format_iso8601] = ACTIONS(1973),
    [sym_auditlog_priv] = ACTIONS(1973),
    [sym_configuration_priv] = ACTIONS(1973),
    [sym_deviceconfig_priv] = ACTIONS(1973),
    [sym_ftp_priv] = ACTIONS(1973),
    [sym_http_priv] = ACTIONS(1973),
    [sym_networkconfig_priv] = ACTIONS(1973),
    [sym_programport_priv] = ACTIONS(1973),
    [sym_remoteui_priv] = ACTIONS(1973),
    [sym_securitycontrol_priv] = ACTIONS(1973),
    [sym_softwaremanagement_priv] = ACTIONS(1973),
    [sym_terminal_priv] = ACTIONS(1973),
    [sym_tpadmin_priv] = ACTIONS(1973),
    [sym_user1_priv] = ACTIONS(1973),
    [sym_user2_priv] = ACTIONS(1973),
    [sym_user3_priv] = ACTIONS(1973),
    [sym_user4_priv] = ACTIONS(1973),
    [sym_usermanagement_priv] = ACTIONS(1973),
    [sym_valid_account] = ACTIONS(1973),
    [sym_err_name_invalid] = ACTIONS(1973),
    [sym_err_password_invalid] = ACTIONS(1973),
    [sym_err_invalid_account] = ACTIONS(1973),
    [sym_err_no_authorization] = ACTIONS(1973),
    [sym_err_invalid_parameter] = ACTIONS(1973),
    [sym_err_user_locked_out] = ACTIONS(1973),
    [sym_err_user_account_expired] = ACTIONS(1973),
    [sym_err_general_error] = ACTIONS(1973),
    [sym_err_authentication_error] = ACTIONS(1973),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1973),
    [sym_tls_validate_certificate] = ACTIONS(1973),
    [sym_tdata] = ACTIONS(1973),
    [sym_tchannel] = ACTIONS(1973),
    [sym_tlevel] = ACTIONS(1973),
    [sym_tbutton] = ACTIONS(1973),
    [sym_ttimeline] = ACTIONS(1973),
    [sym_tcustom] = ACTIONS(1973),
    [sym_url_struct] = ACTIONS(1973),
    [sym_dns_struct] = ACTIONS(1973),
    [sym_ip_address_struct] = ACTIONS(1973),
    [sym_dev_info_struct] = ACTIONS(1973),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1973),
    [sym_data_feed] = ACTIONS(1973),
    [sym_data_field] = ACTIONS(1973),
    [sym_data_record] = ACTIONS(1973),
    [sym_wc_data_feed] = ACTIONS(1973),
    [sym_wc_data_field] = ACTIONS(1973),
    [sym_wc_data_record] = ACTIONS(1973),
    [sym_last_login_info] = ACTIONS(1973),
    [sym____reserved___] = ACTIONS(1973),
    [sym_date] = ACTIONS(1973),
    [sym_day] = ACTIONS(1973),
    [sym_time] = ACTIONS(1973),
    [sym_ldate] = ACTIONS(1973),
    [sym_push_device] = ACTIONS(1973),
    [sym_push_channel] = ACTIONS(1973),
    [sym_push_devchan] = ACTIONS(1973),
    [sym_release_device] = ACTIONS(1973),
    [sym_release_channel] = ACTIONS(1973),
    [sym_release_devchan] = ACTIONS(1973),
    [sym_master_slot] = ACTIONS(1973),
    [sym_get_pulse_time] = ACTIONS(1973),
    [sym_get_timer] = ACTIONS(1973),
    [sym_master_sn] = ACTIONS(1973),
    [sym_system_number] = ACTIONS(1973),
    [sym_button] = ACTIONS(1973),
    [sym_dv_channel] = ACTIONS(1973),
    [sym_data] = ACTIONS(1973),
    [sym_level] = ACTIONS(1973),
    [sym_timeline] = ACTIONS(1973),
    [sym_channel] = ACTIONS(1973),
    [sym_custom] = ACTIONS(1973),
    [sym_atoi] = ACTIONS(1973),
    [sym_atol] = ACTIONS(1973),
    [sym_atof] = ACTIONS(1973),
    [sym_itoa] = ACTIONS(1973),
    [sym_itohex] = ACTIONS(1973),
    [sym_hextoi] = ACTIONS(1973),
    [sym_ftoa] = ACTIONS(1973),
    [sym_find_string] = ACTIONS(1973),
    [sym_left_string] = ACTIONS(1973),
    [sym_right_string] = ACTIONS(1973),
    [sym_mid_string] = ACTIONS(1973),
    [sym_remove_string] = ACTIONS(1973),
    [sym_get_buffer_string] = ACTIONS(1973),
    [sym_get_buffer_char] = ACTIONS(1973),
    [sym_get_multi_buffer_string] = ACTIONS(1973),
    [sym_length_string] = ACTIONS(1973),
    [sym_max_length_string] = ACTIONS(1973),
    [sym_set_length_string] = ACTIONS(1973),
    [sym_compare_string] = ACTIONS(1973),
    [sym_upper_string] = ACTIONS(1973),
    [sym_lower_string] = ACTIONS(1973),
    [sym_get_last] = ACTIONS(1973),
    [sym_date_to_day] = ACTIONS(1973),
    [sym_date_to_month] = ACTIONS(1973),
    [sym_date_to_year] = ACTIONS(1973),
    [sym_time_to_hour] = ACTIONS(1973),
    [sym_time_to_minute] = ACTIONS(1973),
    [sym_time_to_second] = ACTIONS(1973),
    [sym_day_of_week] = ACTIONS(1973),
    [sym_set_pulse_time] = ACTIONS(1973),
    [sym_set_timer] = ACTIONS(1973),
    [sym_file_open] = ACTIONS(1973),
    [sym_file_close] = ACTIONS(1973),
    [sym_file_read] = ACTIONS(1973),
    [sym_file_read_line] = ACTIONS(1973),
    [sym_file_write] = ACTIONS(1973),
    [sym_file_write_line] = ACTIONS(1973),
    [sym_file_copy] = ACTIONS(1973),
    [sym_file_delete] = ACTIONS(1973),
    [sym_file_rename] = ACTIONS(1973),
    [sym_file_dir] = ACTIONS(1973),
    [sym_file_getdir] = ACTIONS(1973),
    [sym_file_setdir] = ACTIONS(1973),
    [sym_file_create_dir] = ACTIONS(1973),
    [sym_file_remove_dir] = ACTIONS(1973),
    [sym_file_seek] = ACTIONS(1973),
    [sym_length_array] = ACTIONS(1973),
    [sym_max_length_array] = ACTIONS(1973),
    [sym_set_length_array] = ACTIONS(1973),
    [sym_ip_client_open] = ACTIONS(1973),
    [sym_ip_client_close] = ACTIONS(1973),
    [sym_ip_server_open] = ACTIONS(1973),
    [sym_ip_server_close] = ACTIONS(1973),
    [sym_ip_bound_client_open] = ACTIONS(1973),
    [sym_ip_mc_server_open] = ACTIONS(1973),
    [sym_ip_set_option] = ACTIONS(1973),
    [sym_ssh_client_open] = ACTIONS(1973),
    [sym_ssh_client_close] = ACTIONS(1973),
    [sym_tls_client_open] = ACTIONS(1973),
    [sym_tls_client_close] = ACTIONS(1973),
    [sym_do_push] = ACTIONS(1973),
    [sym_do_push_timed] = ACTIONS(1973),
    [sym_do_release] = ACTIONS(1973),
    [sym_device_id] = ACTIONS(1973),
    [sym_device_info] = ACTIONS(1973),
    [sym_device_id_string] = ACTIONS(1973),
    [sym_device_standby] = ACTIONS(1973),
    [sym_device_wake] = ACTIONS(1973),
    [sym_timeline_create] = ACTIONS(1973),
    [sym_timeline_kill] = ACTIONS(1973),
    [sym_timeline_reload] = ACTIONS(1973),
    [sym_timeline_pause] = ACTIONS(1973),
    [sym_timeline_restart] = ACTIONS(1973),
    [sym_timeline_set] = ACTIONS(1973),
    [sym_timeline_get] = ACTIONS(1973),
    [sym_timeline_active] = ACTIONS(1973),
    [sym_timeline_dynamic_id] = ACTIONS(1973),
    [sym_variable_to_string] = ACTIONS(1973),
    [sym_string_to_variable] = ACTIONS(1973),
    [sym_length_variable_to_string] = ACTIONS(1973),
    [sym_type_cast] = ACTIONS(1973),
    [sym_variable_to_xml] = ACTIONS(1973),
    [sym_xml_to_variable] = ACTIONS(1973),
    [sym_length_variable_to_xml] = ACTIONS(1973),
    [sym_redirect_string] = ACTIONS(1973),
    [sym_random_number] = ACTIONS(1973),
    [sym_get_unique_id] = ACTIONS(1973),
    [sym_get_system_number] = ACTIONS(1973),
    [sym_set_system_number] = ACTIONS(1973),
    [sym_get_serial_number] = ACTIONS(1973),
    [sym_get_master_build] = ACTIONS(1973),
    [sym_set_validation_code] = ACTIONS(1973),
    [sym_reboot] = ACTIONS(1973),
    [sym_load_duet_module] = ACTIONS(1973),
    [sym_unload_duet_module] = ACTIONS(1973),
    [sym_duet_mem_size_set] = ACTIONS(1973),
    [sym_duet_mem_size_get] = ACTIONS(1973),
    [sym_dynamic_application_device] = ACTIONS(1973),
    [sym_dynamic_polled_port] = ACTIONS(1973),
    [sym_static_port_binding] = ACTIONS(1973),
    [sym_static_ip_binding] = ACTIONS(1973),
    [sym_get_dns_list] = ACTIONS(1973),
    [sym_set_dns_list] = ACTIONS(1973),
    [sym_get_ip_address] = ACTIONS(1973),
    [sym_set_ip_address] = ACTIONS(1973),
    [sym_get_url_list] = ACTIONS(1973),
    [sym_add_url_entry] = ACTIONS(1973),
    [sym_delete_url_entry] = ACTIONS(1973),
    [sym_abs_value] = ACTIONS(1973),
    [sym_max_value] = ACTIONS(1973),
    [sym_min_value] = ACTIONS(1973),
    [sym_exp_value] = ACTIONS(1973),
    [sym_log_value] = ACTIONS(1973),
    [sym_log10_value] = ACTIONS(1973),
    [sym_power_value] = ACTIONS(1973),
    [sym_sqrt_value] = ACTIONS(1973),
    [sym_raw_be] = ACTIONS(1973),
    [sym_raw_le] = ACTIONS(1973),
    [sym_internal_queue_size_set] = ACTIONS(1973),
    [sym_internal_queue_size_get] = ACTIONS(1973),
    [sym_internal_threshold_set] = ACTIONS(1973),
    [sym_internal_threshold_get] = ACTIONS(1973),
    [sym_rebuilt_event] = ACTIONS(1973),
    [sym_do_custom_event] = ACTIONS(1973),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1973),
    [sym_clkmgr_set_clk_source] = ACTIONS(1973),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1973),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1973),
    [sym_clkmgr_get_timezone] = ACTIONS(1973),
    [sym_clkmgr_set_timezone] = ACTIONS(1973),
    [sym_clkmgr_get_resync_period] = ACTIONS(1973),
    [sym_clkmgr_set_resync_period] = ACTIONS(1973),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1973),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1973),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1973),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1973),
    [sym_clkmgr_get_timeservers] = ACTIONS(1973),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1973),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1973),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1973),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1973),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1973),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1973),
    [sym_set_log_level] = ACTIONS(1973),
    [sym_get_log_level] = ACTIONS(1973),
    [sym_amx_log] = ACTIONS(1973),
    [sym_smtp_server_config_set] = ACTIONS(1973),
    [sym_smtp_server_config_get] = ACTIONS(1973),
    [sym_smtp_send] = ACTIONS(1973),
    [sym_validate_netlinx_account] = ACTIONS(1973),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1973),
    [sym_authenticate_certificate] = ACTIONS(1973),
    [sym_audit_netlinx_session_event] = ACTIONS(1973),
    [sym_audit_netlinx_generic_event] = ACTIONS(1973),
    [sym_get_available_flash_disk_space] = ACTIONS(1973),
    [sym_get_max_flash_disk_space] = ACTIONS(1973),
    [sym_data_create_feed] = ACTIONS(1973),
    [sym_data_delete_feed] = ACTIONS(1973),
    [sym_data_publish_feed] = ACTIONS(1973),
    [sym_data_get_published_feed] = ACTIONS(1973),
    [sym_data_add_record] = ACTIONS(1973),
    [sym_data_get_event_record] = ACTIONS(1973),
    [sym__wc_data_create_feed] = ACTIONS(1973),
    [sym__wc_data_add_record] = ACTIONS(1973),
    [sym__wc_data_get_event_record] = ACTIONS(1973),
    [sym_format] = ACTIONS(1973),
    [sym_astro_clock] = ACTIONS(1973),
    [sym_set_outdoor_temperature] = ACTIONS(1973),
    [sym_set_virtual_level_count] = ACTIONS(1973),
    [sym_set_virtual_channel_count] = ACTIONS(1973),
    [sym_set_virtual_port_count] = ACTIONS(1973),
    [sym__wc_encode_int] = ACTIONS(1973),
    [sym__wc_decode_int] = ACTIONS(1973),
    [sym__wc_int] = ACTIONS(1973),
    [sym__wc_to_ch_int] = ACTIONS(1973),
    [sym__ch_to_wc_int] = ACTIONS(1973),
    [sym__wc_find_string_int] = ACTIONS(1973),
    [sym__wc_left_string_int] = ACTIONS(1973),
    [sym__wc_lower_string_int] = ACTIONS(1973),
    [sym__wc_mid_string_int] = ACTIONS(1973),
    [sym__wc_remove_string_int] = ACTIONS(1973),
    [sym__wc_right_string_int] = ACTIONS(1973),
    [sym__wc_upper_string_int] = ACTIONS(1973),
    [sym__wc_compare_string_int] = ACTIONS(1973),
    [sym__wc_get_buffer_char_int] = ACTIONS(1973),
    [sym__wc_get_buffer_string_int] = ACTIONS(1973),
    [sym__wc_concat_string_int] = ACTIONS(1973),
    [sym__wc_file_open_int] = ACTIONS(1973),
    [sym__wc_file_close_int] = ACTIONS(1973),
    [sym__wc_file_read_int] = ACTIONS(1973),
    [sym__wc_file_read_line_int] = ACTIONS(1973),
    [sym__wc_file_write_int] = ACTIONS(1973),
    [sym__wc_file_write_line_int] = ACTIONS(1973),
    [aux_sym_preproc_include_token1] = ACTIONS(1973),
    [aux_sym_preproc_define_token1] = ACTIONS(1975),
    [aux_sym_preproc_warn_token1] = ACTIONS(1975),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1975),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1975),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1975),
    [anon_sym_LBRACK] = ACTIONS(1975),
    [anon_sym_LBRACE] = ACTIONS(1975),
    [anon_sym_RBRACE] = ACTIONS(1975),
    [anon_sym_LPAREN] = ACTIONS(1973),
    [anon_sym_BANG] = ACTIONS(1975),
    [anon_sym_TILDE] = ACTIONS(1975),
    [anon_sym_DASH] = ACTIONS(1973),
    [anon_sym_PLUS] = ACTIONS(1973),
    [anon_sym_DASH_DASH] = ACTIONS(1975),
    [anon_sym_PLUS_PLUS] = ACTIONS(1975),
    [anon_sym_DQUOTE] = ACTIONS(1975),
    [anon_sym_SQUOTE] = ACTIONS(1975),
    [sym_number_literal] = ACTIONS(1975),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(548)] = {
    [sym_identifier] = ACTIONS(1977),
    [sym_define_call_keyword] = ACTIONS(1977),
    [sym_define_function_keyword] = ACTIONS(1977),
    [sym_define_library_function_keyword] = ACTIONS(1977),
    [sym_define_module_keyword] = ACTIONS(1977),
    [sym_char_keyword] = ACTIONS(1977),
    [sym_widechar_keyword] = ACTIONS(1977),
    [sym_integer_keyword] = ACTIONS(1977),
    [sym_sinteger_keyword] = ACTIONS(1977),
    [sym_long_keyword] = ACTIONS(1977),
    [sym_slong_keyword] = ACTIONS(1977),
    [sym_float_keyword] = ACTIONS(1977),
    [sym_double_keyword] = ACTIONS(1977),
    [sym_variant_keyword] = ACTIONS(1977),
    [sym_variantarray_keyword] = ACTIONS(1977),
    [sym_local_var_keyword] = ACTIONS(1977),
    [sym_stack_var_keyword] = ACTIONS(1977),
    [sym_constant_keyword] = ACTIONS(1977),
    [sym_volatile_keyword] = ACTIONS(1977),
    [sym_non_volatile_keyword] = ACTIONS(1977),
    [sym_persistent_keyword] = ACTIONS(1977),
    [sym_dev_keyword] = ACTIONS(1977),
    [sym_devlev_keyword] = ACTIONS(1977),
    [sym_devchan_keyword] = ACTIONS(1977),
    [sym_if_keyword] = ACTIONS(1977),
    [sym_switch_keyword] = ACTIONS(1977),
    [sym_case_keyword] = ACTIONS(1977),
    [sym_default_keyword] = ACTIONS(1977),
    [sym_while_keyword] = ACTIONS(1977),
    [sym_for_keyword] = ACTIONS(1977),
    [sym_break_keyword] = ACTIONS(1977),
    [sym_continue_keyword] = ACTIONS(1977),
    [sym_return_keyword] = ACTIONS(1977),
    [sym_select_keyword] = ACTIONS(1977),
    [sym_active_keyword] = ACTIONS(1977),
    [sym_struct_keyword] = ACTIONS(1977),
    [sym_structure_keyword] = ACTIONS(1977),
    [aux_sym_on_keyword_token1] = ACTIONS(1977),
    [aux_sym_off_keyword_token1] = ACTIONS(1977),
    [sym_send_level_keyword] = ACTIONS(1977),
    [sym_send_string_keyword] = ACTIONS(1977),
    [sym_send_command_keyword] = ACTIONS(1977),
    [sym_clear_buffer_keyword] = ACTIONS(1977),
    [sym_create_buffer_keyword] = ACTIONS(1977),
    [sym_create_multi_buffer_keyword] = ACTIONS(1977),
    [sym_call_keyword] = ACTIONS(1977),
    [sym_system_call_keyword] = ACTIONS(1977),
    [sym_devchan_to_keyword] = ACTIONS(1977),
    [sym_devchan_min_to_keyword] = ACTIONS(1977),
    [sym_devchan_total_off_keyword] = ACTIONS(1977),
    [sym_devchan_pulse_keyword] = ACTIONS(1977),
    [sym_wait_keyword] = ACTIONS(1977),
    [sym_cancel_wait_keyword] = ACTIONS(1977),
    [sym_wait_until_keyword] = ACTIONS(1977),
    [sym_cancel_wait_until_keyword] = ACTIONS(1977),
    [sym_cancel_all_wait_keyword] = ACTIONS(1977),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1977),
    [sym_not] = ACTIONS(1977),
    [sym_bnot] = ACTIONS(1977),
    [sym___file__] = ACTIONS(1977),
    [sym___date__] = ACTIONS(1977),
    [sym___ldate__] = ACTIONS(1977),
    [sym___time__] = ACTIONS(1977),
    [sym___line__] = ACTIONS(1977),
    [sym___name__] = ACTIONS(1977),
    [sym___netlinx__] = ACTIONS(1977),
    [sym_netlinx_axi_version] = ACTIONS(1977),
    [sym_url_flg_tcp] = ACTIONS(1977),
    [sym_url_flg_accinfopresent] = ACTIONS(1977),
    [sym_url_flg_temp] = ACTIONS(1977),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1977),
    [sym_url_flg_stat_mask] = ACTIONS(1977),
    [sym_url_flg_stat_lookup] = ACTIONS(1977),
    [sym_url_flg_stat_connecting] = ACTIONS(1977),
    [sym_url_flg_stat_waiting] = ACTIONS(1977),
    [sym_url_flg_stat_connected] = ACTIONS(1977),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1977),
    [sym_first_virtual_device] = ACTIONS(1977),
    [sym_first_local_port] = ACTIONS(1977),
    [sym_dynamic_virtual_device] = ACTIONS(1977),
    [sym_timeline_once] = ACTIONS(1977),
    [sym_timeline_repeat] = ACTIONS(1977),
    [sym_timeline_absolute] = ACTIONS(1977),
    [sym_timeline_relative] = ACTIONS(1977),
    [sym_do_push_timed_infinite] = ACTIONS(1977),
    [sym_source_type_no_address] = ACTIONS(1977),
    [sym_source_type_neuron_id] = ACTIONS(1977),
    [sym_source_type_ip_address] = ACTIONS(1977),
    [sym_source_type_axlink] = ACTIONS(1977),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1977),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1977),
    [sym_source_type_ip_socket_address] = ACTIONS(1977),
    [sym_source_type_rs232] = ACTIONS(1977),
    [sym_source_type_internal] = ACTIONS(1977),
    [sym_source_type_mac_address] = ACTIONS(1977),
    [sym_source_type_ipv4_port] = ACTIONS(1977),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1977),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1977),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1977),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1977),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1977),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1977),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1977),
    [sym_true] = ACTIONS(1977),
    [sym_false] = ACTIONS(1977),
    [sym_file_read_only] = ACTIONS(1977),
    [sym_file_rw_new] = ACTIONS(1977),
    [sym_file_rw_append] = ACTIONS(1977),
    [sym_ip_tcp] = ACTIONS(1977),
    [sym_ip_udp] = ACTIONS(1977),
    [sym_ip_udp_2way] = ACTIONS(1977),
    [sym_xml_encode_types] = ACTIONS(1977),
    [sym_xml_encode_char_as_list] = ACTIONS(1977),
    [sym_xml_encode_le] = ACTIONS(1977),
    [sym_xml_decode_types] = ACTIONS(1977),
    [sym_xml_decode_no_preserve] = ACTIONS(1977),
    [sym_duet_dev_type_display_device] = ACTIONS(1977),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1977),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1977),
    [sym_duet_dev_type_io_device] = ACTIONS(1977),
    [sym_duet_dev_type_relay_device] = ACTIONS(1977),
    [sym_duet_dev_type_ups] = ACTIONS(1977),
    [sym_duet_dev_type_amplifier] = ACTIONS(1977),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1977),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1977),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1977),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1977),
    [sym_duet_dev_type_camera] = ACTIONS(1977),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1977),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1977),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1977),
    [sym_duet_dev_type_dss] = ACTIONS(1977),
    [sym_duet_dev_type_dvr] = ACTIONS(1977),
    [sym_duet_dev_type_disc_device] = ACTIONS(1977),
    [sym_duet_dev_type_document_camera] = ACTIONS(1977),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1977),
    [sym_duet_dev_type_hvac] = ACTIONS(1977),
    [sym_duet_dev_type_keypad] = ACTIONS(1977),
    [sym_duet_dev_type_light] = ACTIONS(1977),
    [sym_duet_dev_type_monitor] = ACTIONS(1977),
    [sym_duet_dev_type_motor] = ACTIONS(1977),
    [sym_duet_dev_type_multi_window] = ACTIONS(1977),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1977),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1977),
    [sym_duet_dev_type_receiver] = ACTIONS(1977),
    [sym_duet_dev_type_security_system] = ACTIONS(1977),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1977),
    [sym_duet_dev_type_settop_box] = ACTIONS(1977),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1977),
    [sym_duet_dev_type_switcher] = ACTIONS(1977),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1977),
    [sym_duet_dev_type_tv] = ACTIONS(1977),
    [sym_duet_dev_type_utility] = ACTIONS(1977),
    [sym_duet_dev_type_vcr] = ACTIONS(1977),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1977),
    [sym_duet_dev_type_video_processor] = ACTIONS(1977),
    [sym_duet_dev_type_video_projector] = ACTIONS(1977),
    [sym_duet_dev_type_video_wall] = ACTIONS(1977),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1977),
    [sym_duet_dev_type_weather] = ACTIONS(1977),
    [sym_duet_dev_not_polled] = ACTIONS(1977),
    [sym_duet_dev_polled] = ACTIONS(1977),
    [sym_clkmgr_mode_network] = ACTIONS(1977),
    [sym_clkmgr_mode_standalone] = ACTIONS(1977),
    [sym_normal_standby] = ACTIONS(1977),
    [sym_normal_wake] = ACTIONS(1977),
    [sym_ip_multicast_ttl_option] = ACTIONS(1977),
    [sym_ip_ttl_subnet] = ACTIONS(1977),
    [sym_ip_ttl_site] = ACTIONS(1977),
    [sym_ip_ttl_region] = ACTIONS(1977),
    [sym_ip_ttl_continent] = ACTIONS(1977),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1977),
    [sym_ip_nodelay_on] = ACTIONS(1977),
    [sym_ip_nodelay_off] = ACTIONS(1977),
    [sym_amx_error] = ACTIONS(1977),
    [sym_amx_warning] = ACTIONS(1977),
    [sym_amx_info] = ACTIONS(1977),
    [sym_amx_debug] = ACTIONS(1977),
    [sym_netlinx_login_success] = ACTIONS(1977),
    [sym_netlinx_login_fail] = ACTIONS(1977),
    [sym_netlinx_logout] = ACTIONS(1977),
    [sym_null_str] = ACTIONS(1977),
    [sym_smtp_address] = ACTIONS(1977),
    [sym_smtp_port_number] = ACTIONS(1977),
    [sym_smtp_username] = ACTIONS(1977),
    [sym_smtp_password] = ACTIONS(1977),
    [sym_smtp_require_tls] = ACTIONS(1977),
    [sym_smtp_from] = ACTIONS(1977),
    [sym_smtp_tls_true] = ACTIONS(1977),
    [sym_smtp_tls_false] = ACTIONS(1977),
    [sym_listview_on_row_select_event] = ACTIONS(1977),
    [sym_data_structure_unknown] = ACTIONS(1977),
    [sym_data_structure_datafeed] = ACTIONS(1977),
    [sym_data_structure_datarecordset] = ACTIONS(1977),
    [sym_data_structure_datarecord] = ACTIONS(1977),
    [sym_data_structure_datafield] = ACTIONS(1977),
    [sym_data_max_content_field_count] = ACTIONS(1977),
    [sym_data_max_metadata_field_count] = ACTIONS(1977),
    [sym_data_max_id_length] = ACTIONS(1977),
    [sym_data_max_name_length] = ACTIONS(1977),
    [sym_data_max_description_length] = ACTIONS(1977),
    [sym_data_max_type_length] = ACTIONS(1977),
    [sym_data_max_format_length] = ACTIONS(1977),
    [sym_data_max_source_length] = ACTIONS(1977),
    [sym_data_max_value_length] = ACTIONS(1977),
    [sym_data_max_label_length] = ACTIONS(1977),
    [sym_data_max_url_length] = ACTIONS(1977),
    [sym_max_last_login_info_length] = ACTIONS(1977),
    [sym_data_type_unknown] = ACTIONS(1977),
    [sym_data_type_string] = ACTIONS(1977),
    [sym_data_type_datetime] = ACTIONS(1977),
    [sym_data_type_date] = ACTIONS(1977),
    [sym_data_type_time] = ACTIONS(1977),
    [sym_data_type_image] = ACTIONS(1977),
    [sym_data_format_url] = ACTIONS(1977),
    [sym_data_format_phone] = ACTIONS(1977),
    [sym_data_format_email] = ACTIONS(1977),
    [sym_data_format_iso8601] = ACTIONS(1977),
    [sym_auditlog_priv] = ACTIONS(1977),
    [sym_configuration_priv] = ACTIONS(1977),
    [sym_deviceconfig_priv] = ACTIONS(1977),
    [sym_ftp_priv] = ACTIONS(1977),
    [sym_http_priv] = ACTIONS(1977),
    [sym_networkconfig_priv] = ACTIONS(1977),
    [sym_programport_priv] = ACTIONS(1977),
    [sym_remoteui_priv] = ACTIONS(1977),
    [sym_securitycontrol_priv] = ACTIONS(1977),
    [sym_softwaremanagement_priv] = ACTIONS(1977),
    [sym_terminal_priv] = ACTIONS(1977),
    [sym_tpadmin_priv] = ACTIONS(1977),
    [sym_user1_priv] = ACTIONS(1977),
    [sym_user2_priv] = ACTIONS(1977),
    [sym_user3_priv] = ACTIONS(1977),
    [sym_user4_priv] = ACTIONS(1977),
    [sym_usermanagement_priv] = ACTIONS(1977),
    [sym_valid_account] = ACTIONS(1977),
    [sym_err_name_invalid] = ACTIONS(1977),
    [sym_err_password_invalid] = ACTIONS(1977),
    [sym_err_invalid_account] = ACTIONS(1977),
    [sym_err_no_authorization] = ACTIONS(1977),
    [sym_err_invalid_parameter] = ACTIONS(1977),
    [sym_err_user_locked_out] = ACTIONS(1977),
    [sym_err_user_account_expired] = ACTIONS(1977),
    [sym_err_general_error] = ACTIONS(1977),
    [sym_err_authentication_error] = ACTIONS(1977),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1977),
    [sym_tls_validate_certificate] = ACTIONS(1977),
    [sym_tdata] = ACTIONS(1977),
    [sym_tchannel] = ACTIONS(1977),
    [sym_tlevel] = ACTIONS(1977),
    [sym_tbutton] = ACTIONS(1977),
    [sym_ttimeline] = ACTIONS(1977),
    [sym_tcustom] = ACTIONS(1977),
    [sym_url_struct] = ACTIONS(1977),
    [sym_dns_struct] = ACTIONS(1977),
    [sym_ip_address_struct] = ACTIONS(1977),
    [sym_dev_info_struct] = ACTIONS(1977),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1977),
    [sym_data_feed] = ACTIONS(1977),
    [sym_data_field] = ACTIONS(1977),
    [sym_data_record] = ACTIONS(1977),
    [sym_wc_data_feed] = ACTIONS(1977),
    [sym_wc_data_field] = ACTIONS(1977),
    [sym_wc_data_record] = ACTIONS(1977),
    [sym_last_login_info] = ACTIONS(1977),
    [sym____reserved___] = ACTIONS(1977),
    [sym_date] = ACTIONS(1977),
    [sym_day] = ACTIONS(1977),
    [sym_time] = ACTIONS(1977),
    [sym_ldate] = ACTIONS(1977),
    [sym_push_device] = ACTIONS(1977),
    [sym_push_channel] = ACTIONS(1977),
    [sym_push_devchan] = ACTIONS(1977),
    [sym_release_device] = ACTIONS(1977),
    [sym_release_channel] = ACTIONS(1977),
    [sym_release_devchan] = ACTIONS(1977),
    [sym_master_slot] = ACTIONS(1977),
    [sym_get_pulse_time] = ACTIONS(1977),
    [sym_get_timer] = ACTIONS(1977),
    [sym_master_sn] = ACTIONS(1977),
    [sym_system_number] = ACTIONS(1977),
    [sym_button] = ACTIONS(1977),
    [sym_dv_channel] = ACTIONS(1977),
    [sym_data] = ACTIONS(1977),
    [sym_level] = ACTIONS(1977),
    [sym_timeline] = ACTIONS(1977),
    [sym_channel] = ACTIONS(1977),
    [sym_custom] = ACTIONS(1977),
    [sym_atoi] = ACTIONS(1977),
    [sym_atol] = ACTIONS(1977),
    [sym_atof] = ACTIONS(1977),
    [sym_itoa] = ACTIONS(1977),
    [sym_itohex] = ACTIONS(1977),
    [sym_hextoi] = ACTIONS(1977),
    [sym_ftoa] = ACTIONS(1977),
    [sym_find_string] = ACTIONS(1977),
    [sym_left_string] = ACTIONS(1977),
    [sym_right_string] = ACTIONS(1977),
    [sym_mid_string] = ACTIONS(1977),
    [sym_remove_string] = ACTIONS(1977),
    [sym_get_buffer_string] = ACTIONS(1977),
    [sym_get_buffer_char] = ACTIONS(1977),
    [sym_get_multi_buffer_string] = ACTIONS(1977),
    [sym_length_string] = ACTIONS(1977),
    [sym_max_length_string] = ACTIONS(1977),
    [sym_set_length_string] = ACTIONS(1977),
    [sym_compare_string] = ACTIONS(1977),
    [sym_upper_string] = ACTIONS(1977),
    [sym_lower_string] = ACTIONS(1977),
    [sym_get_last] = ACTIONS(1977),
    [sym_date_to_day] = ACTIONS(1977),
    [sym_date_to_month] = ACTIONS(1977),
    [sym_date_to_year] = ACTIONS(1977),
    [sym_time_to_hour] = ACTIONS(1977),
    [sym_time_to_minute] = ACTIONS(1977),
    [sym_time_to_second] = ACTIONS(1977),
    [sym_day_of_week] = ACTIONS(1977),
    [sym_set_pulse_time] = ACTIONS(1977),
    [sym_set_timer] = ACTIONS(1977),
    [sym_file_open] = ACTIONS(1977),
    [sym_file_close] = ACTIONS(1977),
    [sym_file_read] = ACTIONS(1977),
    [sym_file_read_line] = ACTIONS(1977),
    [sym_file_write] = ACTIONS(1977),
    [sym_file_write_line] = ACTIONS(1977),
    [sym_file_copy] = ACTIONS(1977),
    [sym_file_delete] = ACTIONS(1977),
    [sym_file_rename] = ACTIONS(1977),
    [sym_file_dir] = ACTIONS(1977),
    [sym_file_getdir] = ACTIONS(1977),
    [sym_file_setdir] = ACTIONS(1977),
    [sym_file_create_dir] = ACTIONS(1977),
    [sym_file_remove_dir] = ACTIONS(1977),
    [sym_file_seek] = ACTIONS(1977),
    [sym_length_array] = ACTIONS(1977),
    [sym_max_length_array] = ACTIONS(1977),
    [sym_set_length_array] = ACTIONS(1977),
    [sym_ip_client_open] = ACTIONS(1977),
    [sym_ip_client_close] = ACTIONS(1977),
    [sym_ip_server_open] = ACTIONS(1977),
    [sym_ip_server_close] = ACTIONS(1977),
    [sym_ip_bound_client_open] = ACTIONS(1977),
    [sym_ip_mc_server_open] = ACTIONS(1977),
    [sym_ip_set_option] = ACTIONS(1977),
    [sym_ssh_client_open] = ACTIONS(1977),
    [sym_ssh_client_close] = ACTIONS(1977),
    [sym_tls_client_open] = ACTIONS(1977),
    [sym_tls_client_close] = ACTIONS(1977),
    [sym_do_push] = ACTIONS(1977),
    [sym_do_push_timed] = ACTIONS(1977),
    [sym_do_release] = ACTIONS(1977),
    [sym_device_id] = ACTIONS(1977),
    [sym_device_info] = ACTIONS(1977),
    [sym_device_id_string] = ACTIONS(1977),
    [sym_device_standby] = ACTIONS(1977),
    [sym_device_wake] = ACTIONS(1977),
    [sym_timeline_create] = ACTIONS(1977),
    [sym_timeline_kill] = ACTIONS(1977),
    [sym_timeline_reload] = ACTIONS(1977),
    [sym_timeline_pause] = ACTIONS(1977),
    [sym_timeline_restart] = ACTIONS(1977),
    [sym_timeline_set] = ACTIONS(1977),
    [sym_timeline_get] = ACTIONS(1977),
    [sym_timeline_active] = ACTIONS(1977),
    [sym_timeline_dynamic_id] = ACTIONS(1977),
    [sym_variable_to_string] = ACTIONS(1977),
    [sym_string_to_variable] = ACTIONS(1977),
    [sym_length_variable_to_string] = ACTIONS(1977),
    [sym_type_cast] = ACTIONS(1977),
    [sym_variable_to_xml] = ACTIONS(1977),
    [sym_xml_to_variable] = ACTIONS(1977),
    [sym_length_variable_to_xml] = ACTIONS(1977),
    [sym_redirect_string] = ACTIONS(1977),
    [sym_random_number] = ACTIONS(1977),
    [sym_get_unique_id] = ACTIONS(1977),
    [sym_get_system_number] = ACTIONS(1977),
    [sym_set_system_number] = ACTIONS(1977),
    [sym_get_serial_number] = ACTIONS(1977),
    [sym_get_master_build] = ACTIONS(1977),
    [sym_set_validation_code] = ACTIONS(1977),
    [sym_reboot] = ACTIONS(1977),
    [sym_load_duet_module] = ACTIONS(1977),
    [sym_unload_duet_module] = ACTIONS(1977),
    [sym_duet_mem_size_set] = ACTIONS(1977),
    [sym_duet_mem_size_get] = ACTIONS(1977),
    [sym_dynamic_application_device] = ACTIONS(1977),
    [sym_dynamic_polled_port] = ACTIONS(1977),
    [sym_static_port_binding] = ACTIONS(1977),
    [sym_static_ip_binding] = ACTIONS(1977),
    [sym_get_dns_list] = ACTIONS(1977),
    [sym_set_dns_list] = ACTIONS(1977),
    [sym_get_ip_address] = ACTIONS(1977),
    [sym_set_ip_address] = ACTIONS(1977),
    [sym_get_url_list] = ACTIONS(1977),
    [sym_add_url_entry] = ACTIONS(1977),
    [sym_delete_url_entry] = ACTIONS(1977),
    [sym_abs_value] = ACTIONS(1977),
    [sym_max_value] = ACTIONS(1977),
    [sym_min_value] = ACTIONS(1977),
    [sym_exp_value] = ACTIONS(1977),
    [sym_log_value] = ACTIONS(1977),
    [sym_log10_value] = ACTIONS(1977),
    [sym_power_value] = ACTIONS(1977),
    [sym_sqrt_value] = ACTIONS(1977),
    [sym_raw_be] = ACTIONS(1977),
    [sym_raw_le] = ACTIONS(1977),
    [sym_internal_queue_size_set] = ACTIONS(1977),
    [sym_internal_queue_size_get] = ACTIONS(1977),
    [sym_internal_threshold_set] = ACTIONS(1977),
    [sym_internal_threshold_get] = ACTIONS(1977),
    [sym_rebuilt_event] = ACTIONS(1977),
    [sym_do_custom_event] = ACTIONS(1977),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1977),
    [sym_clkmgr_set_clk_source] = ACTIONS(1977),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1977),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1977),
    [sym_clkmgr_get_timezone] = ACTIONS(1977),
    [sym_clkmgr_set_timezone] = ACTIONS(1977),
    [sym_clkmgr_get_resync_period] = ACTIONS(1977),
    [sym_clkmgr_set_resync_period] = ACTIONS(1977),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1977),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1977),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1977),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1977),
    [sym_clkmgr_get_timeservers] = ACTIONS(1977),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1977),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1977),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1977),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1977),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1977),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1977),
    [sym_set_log_level] = ACTIONS(1977),
    [sym_get_log_level] = ACTIONS(1977),
    [sym_amx_log] = ACTIONS(1977),
    [sym_smtp_server_config_set] = ACTIONS(1977),
    [sym_smtp_server_config_get] = ACTIONS(1977),
    [sym_smtp_send] = ACTIONS(1977),
    [sym_validate_netlinx_account] = ACTIONS(1977),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1977),
    [sym_authenticate_certificate] = ACTIONS(1977),
    [sym_audit_netlinx_session_event] = ACTIONS(1977),
    [sym_audit_netlinx_generic_event] = ACTIONS(1977),
    [sym_get_available_flash_disk_space] = ACTIONS(1977),
    [sym_get_max_flash_disk_space] = ACTIONS(1977),
    [sym_data_create_feed] = ACTIONS(1977),
    [sym_data_delete_feed] = ACTIONS(1977),
    [sym_data_publish_feed] = ACTIONS(1977),
    [sym_data_get_published_feed] = ACTIONS(1977),
    [sym_data_add_record] = ACTIONS(1977),
    [sym_data_get_event_record] = ACTIONS(1977),
    [sym__wc_data_create_feed] = ACTIONS(1977),
    [sym__wc_data_add_record] = ACTIONS(1977),
    [sym__wc_data_get_event_record] = ACTIONS(1977),
    [sym_format] = ACTIONS(1977),
    [sym_astro_clock] = ACTIONS(1977),
    [sym_set_outdoor_temperature] = ACTIONS(1977),
    [sym_set_virtual_level_count] = ACTIONS(1977),
    [sym_set_virtual_channel_count] = ACTIONS(1977),
    [sym_set_virtual_port_count] = ACTIONS(1977),
    [sym__wc_encode_int] = ACTIONS(1977),
    [sym__wc_decode_int] = ACTIONS(1977),
    [sym__wc_int] = ACTIONS(1977),
    [sym__wc_to_ch_int] = ACTIONS(1977),
    [sym__ch_to_wc_int] = ACTIONS(1977),
    [sym__wc_find_string_int] = ACTIONS(1977),
    [sym__wc_left_string_int] = ACTIONS(1977),
    [sym__wc_lower_string_int] = ACTIONS(1977),
    [sym__wc_mid_string_int] = ACTIONS(1977),
    [sym__wc_remove_string_int] = ACTIONS(1977),
    [sym__wc_right_string_int] = ACTIONS(1977),
    [sym__wc_upper_string_int] = ACTIONS(1977),
    [sym__wc_compare_string_int] = ACTIONS(1977),
    [sym__wc_get_buffer_char_int] = ACTIONS(1977),
    [sym__wc_get_buffer_string_int] = ACTIONS(1977),
    [sym__wc_concat_string_int] = ACTIONS(1977),
    [sym__wc_file_open_int] = ACTIONS(1977),
    [sym__wc_file_close_int] = ACTIONS(1977),
    [sym__wc_file_read_int] = ACTIONS(1977),
    [sym__wc_file_read_line_int] = ACTIONS(1977),
    [sym__wc_file_write_int] = ACTIONS(1977),
    [sym__wc_file_write_line_int] = ACTIONS(1977),
    [aux_sym_preproc_include_token1] = ACTIONS(1977),
    [aux_sym_preproc_define_token1] = ACTIONS(1979),
    [aux_sym_preproc_warn_token1] = ACTIONS(1979),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1979),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1979),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1979),
    [anon_sym_LBRACK] = ACTIONS(1979),
    [anon_sym_LBRACE] = ACTIONS(1979),
    [anon_sym_RBRACE] = ACTIONS(1979),
    [anon_sym_LPAREN] = ACTIONS(1977),
    [anon_sym_BANG] = ACTIONS(1979),
    [anon_sym_TILDE] = ACTIONS(1979),
    [anon_sym_DASH] = ACTIONS(1977),
    [anon_sym_PLUS] = ACTIONS(1977),
    [anon_sym_DASH_DASH] = ACTIONS(1979),
    [anon_sym_PLUS_PLUS] = ACTIONS(1979),
    [anon_sym_DQUOTE] = ACTIONS(1979),
    [anon_sym_SQUOTE] = ACTIONS(1979),
    [sym_number_literal] = ACTIONS(1979),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(549)] = {
    [sym_identifier] = ACTIONS(1981),
    [sym_define_call_keyword] = ACTIONS(1981),
    [sym_define_function_keyword] = ACTIONS(1981),
    [sym_define_library_function_keyword] = ACTIONS(1981),
    [sym_define_module_keyword] = ACTIONS(1981),
    [sym_char_keyword] = ACTIONS(1981),
    [sym_widechar_keyword] = ACTIONS(1981),
    [sym_integer_keyword] = ACTIONS(1981),
    [sym_sinteger_keyword] = ACTIONS(1981),
    [sym_long_keyword] = ACTIONS(1981),
    [sym_slong_keyword] = ACTIONS(1981),
    [sym_float_keyword] = ACTIONS(1981),
    [sym_double_keyword] = ACTIONS(1981),
    [sym_variant_keyword] = ACTIONS(1981),
    [sym_variantarray_keyword] = ACTIONS(1981),
    [sym_local_var_keyword] = ACTIONS(1981),
    [sym_stack_var_keyword] = ACTIONS(1981),
    [sym_constant_keyword] = ACTIONS(1981),
    [sym_volatile_keyword] = ACTIONS(1981),
    [sym_non_volatile_keyword] = ACTIONS(1981),
    [sym_persistent_keyword] = ACTIONS(1981),
    [sym_dev_keyword] = ACTIONS(1981),
    [sym_devlev_keyword] = ACTIONS(1981),
    [sym_devchan_keyword] = ACTIONS(1981),
    [sym_if_keyword] = ACTIONS(1981),
    [sym_switch_keyword] = ACTIONS(1981),
    [sym_case_keyword] = ACTIONS(1981),
    [sym_default_keyword] = ACTIONS(1981),
    [sym_while_keyword] = ACTIONS(1981),
    [sym_for_keyword] = ACTIONS(1981),
    [sym_break_keyword] = ACTIONS(1981),
    [sym_continue_keyword] = ACTIONS(1981),
    [sym_return_keyword] = ACTIONS(1981),
    [sym_select_keyword] = ACTIONS(1981),
    [sym_active_keyword] = ACTIONS(1981),
    [sym_struct_keyword] = ACTIONS(1981),
    [sym_structure_keyword] = ACTIONS(1981),
    [aux_sym_on_keyword_token1] = ACTIONS(1981),
    [aux_sym_off_keyword_token1] = ACTIONS(1981),
    [sym_send_level_keyword] = ACTIONS(1981),
    [sym_send_string_keyword] = ACTIONS(1981),
    [sym_send_command_keyword] = ACTIONS(1981),
    [sym_clear_buffer_keyword] = ACTIONS(1981),
    [sym_create_buffer_keyword] = ACTIONS(1981),
    [sym_create_multi_buffer_keyword] = ACTIONS(1981),
    [sym_call_keyword] = ACTIONS(1981),
    [sym_system_call_keyword] = ACTIONS(1981),
    [sym_devchan_to_keyword] = ACTIONS(1981),
    [sym_devchan_min_to_keyword] = ACTIONS(1981),
    [sym_devchan_total_off_keyword] = ACTIONS(1981),
    [sym_devchan_pulse_keyword] = ACTIONS(1981),
    [sym_wait_keyword] = ACTIONS(1981),
    [sym_cancel_wait_keyword] = ACTIONS(1981),
    [sym_wait_until_keyword] = ACTIONS(1981),
    [sym_cancel_wait_until_keyword] = ACTIONS(1981),
    [sym_cancel_all_wait_keyword] = ACTIONS(1981),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1981),
    [sym_not] = ACTIONS(1981),
    [sym_bnot] = ACTIONS(1981),
    [sym___file__] = ACTIONS(1981),
    [sym___date__] = ACTIONS(1981),
    [sym___ldate__] = ACTIONS(1981),
    [sym___time__] = ACTIONS(1981),
    [sym___line__] = ACTIONS(1981),
    [sym___name__] = ACTIONS(1981),
    [sym___netlinx__] = ACTIONS(1981),
    [sym_netlinx_axi_version] = ACTIONS(1981),
    [sym_url_flg_tcp] = ACTIONS(1981),
    [sym_url_flg_accinfopresent] = ACTIONS(1981),
    [sym_url_flg_temp] = ACTIONS(1981),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1981),
    [sym_url_flg_stat_mask] = ACTIONS(1981),
    [sym_url_flg_stat_lookup] = ACTIONS(1981),
    [sym_url_flg_stat_connecting] = ACTIONS(1981),
    [sym_url_flg_stat_waiting] = ACTIONS(1981),
    [sym_url_flg_stat_connected] = ACTIONS(1981),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1981),
    [sym_first_virtual_device] = ACTIONS(1981),
    [sym_first_local_port] = ACTIONS(1981),
    [sym_dynamic_virtual_device] = ACTIONS(1981),
    [sym_timeline_once] = ACTIONS(1981),
    [sym_timeline_repeat] = ACTIONS(1981),
    [sym_timeline_absolute] = ACTIONS(1981),
    [sym_timeline_relative] = ACTIONS(1981),
    [sym_do_push_timed_infinite] = ACTIONS(1981),
    [sym_source_type_no_address] = ACTIONS(1981),
    [sym_source_type_neuron_id] = ACTIONS(1981),
    [sym_source_type_ip_address] = ACTIONS(1981),
    [sym_source_type_axlink] = ACTIONS(1981),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1981),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1981),
    [sym_source_type_ip_socket_address] = ACTIONS(1981),
    [sym_source_type_rs232] = ACTIONS(1981),
    [sym_source_type_internal] = ACTIONS(1981),
    [sym_source_type_mac_address] = ACTIONS(1981),
    [sym_source_type_ipv4_port] = ACTIONS(1981),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1981),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1981),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1981),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1981),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1981),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1981),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1981),
    [sym_true] = ACTIONS(1981),
    [sym_false] = ACTIONS(1981),
    [sym_file_read_only] = ACTIONS(1981),
    [sym_file_rw_new] = ACTIONS(1981),
    [sym_file_rw_append] = ACTIONS(1981),
    [sym_ip_tcp] = ACTIONS(1981),
    [sym_ip_udp] = ACTIONS(1981),
    [sym_ip_udp_2way] = ACTIONS(1981),
    [sym_xml_encode_types] = ACTIONS(1981),
    [sym_xml_encode_char_as_list] = ACTIONS(1981),
    [sym_xml_encode_le] = ACTIONS(1981),
    [sym_xml_decode_types] = ACTIONS(1981),
    [sym_xml_decode_no_preserve] = ACTIONS(1981),
    [sym_duet_dev_type_display_device] = ACTIONS(1981),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1981),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1981),
    [sym_duet_dev_type_io_device] = ACTIONS(1981),
    [sym_duet_dev_type_relay_device] = ACTIONS(1981),
    [sym_duet_dev_type_ups] = ACTIONS(1981),
    [sym_duet_dev_type_amplifier] = ACTIONS(1981),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1981),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1981),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1981),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1981),
    [sym_duet_dev_type_camera] = ACTIONS(1981),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1981),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1981),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1981),
    [sym_duet_dev_type_dss] = ACTIONS(1981),
    [sym_duet_dev_type_dvr] = ACTIONS(1981),
    [sym_duet_dev_type_disc_device] = ACTIONS(1981),
    [sym_duet_dev_type_document_camera] = ACTIONS(1981),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1981),
    [sym_duet_dev_type_hvac] = ACTIONS(1981),
    [sym_duet_dev_type_keypad] = ACTIONS(1981),
    [sym_duet_dev_type_light] = ACTIONS(1981),
    [sym_duet_dev_type_monitor] = ACTIONS(1981),
    [sym_duet_dev_type_motor] = ACTIONS(1981),
    [sym_duet_dev_type_multi_window] = ACTIONS(1981),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1981),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1981),
    [sym_duet_dev_type_receiver] = ACTIONS(1981),
    [sym_duet_dev_type_security_system] = ACTIONS(1981),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1981),
    [sym_duet_dev_type_settop_box] = ACTIONS(1981),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1981),
    [sym_duet_dev_type_switcher] = ACTIONS(1981),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1981),
    [sym_duet_dev_type_tv] = ACTIONS(1981),
    [sym_duet_dev_type_utility] = ACTIONS(1981),
    [sym_duet_dev_type_vcr] = ACTIONS(1981),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1981),
    [sym_duet_dev_type_video_processor] = ACTIONS(1981),
    [sym_duet_dev_type_video_projector] = ACTIONS(1981),
    [sym_duet_dev_type_video_wall] = ACTIONS(1981),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1981),
    [sym_duet_dev_type_weather] = ACTIONS(1981),
    [sym_duet_dev_not_polled] = ACTIONS(1981),
    [sym_duet_dev_polled] = ACTIONS(1981),
    [sym_clkmgr_mode_network] = ACTIONS(1981),
    [sym_clkmgr_mode_standalone] = ACTIONS(1981),
    [sym_normal_standby] = ACTIONS(1981),
    [sym_normal_wake] = ACTIONS(1981),
    [sym_ip_multicast_ttl_option] = ACTIONS(1981),
    [sym_ip_ttl_subnet] = ACTIONS(1981),
    [sym_ip_ttl_site] = ACTIONS(1981),
    [sym_ip_ttl_region] = ACTIONS(1981),
    [sym_ip_ttl_continent] = ACTIONS(1981),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1981),
    [sym_ip_nodelay_on] = ACTIONS(1981),
    [sym_ip_nodelay_off] = ACTIONS(1981),
    [sym_amx_error] = ACTIONS(1981),
    [sym_amx_warning] = ACTIONS(1981),
    [sym_amx_info] = ACTIONS(1981),
    [sym_amx_debug] = ACTIONS(1981),
    [sym_netlinx_login_success] = ACTIONS(1981),
    [sym_netlinx_login_fail] = ACTIONS(1981),
    [sym_netlinx_logout] = ACTIONS(1981),
    [sym_null_str] = ACTIONS(1981),
    [sym_smtp_address] = ACTIONS(1981),
    [sym_smtp_port_number] = ACTIONS(1981),
    [sym_smtp_username] = ACTIONS(1981),
    [sym_smtp_password] = ACTIONS(1981),
    [sym_smtp_require_tls] = ACTIONS(1981),
    [sym_smtp_from] = ACTIONS(1981),
    [sym_smtp_tls_true] = ACTIONS(1981),
    [sym_smtp_tls_false] = ACTIONS(1981),
    [sym_listview_on_row_select_event] = ACTIONS(1981),
    [sym_data_structure_unknown] = ACTIONS(1981),
    [sym_data_structure_datafeed] = ACTIONS(1981),
    [sym_data_structure_datarecordset] = ACTIONS(1981),
    [sym_data_structure_datarecord] = ACTIONS(1981),
    [sym_data_structure_datafield] = ACTIONS(1981),
    [sym_data_max_content_field_count] = ACTIONS(1981),
    [sym_data_max_metadata_field_count] = ACTIONS(1981),
    [sym_data_max_id_length] = ACTIONS(1981),
    [sym_data_max_name_length] = ACTIONS(1981),
    [sym_data_max_description_length] = ACTIONS(1981),
    [sym_data_max_type_length] = ACTIONS(1981),
    [sym_data_max_format_length] = ACTIONS(1981),
    [sym_data_max_source_length] = ACTIONS(1981),
    [sym_data_max_value_length] = ACTIONS(1981),
    [sym_data_max_label_length] = ACTIONS(1981),
    [sym_data_max_url_length] = ACTIONS(1981),
    [sym_max_last_login_info_length] = ACTIONS(1981),
    [sym_data_type_unknown] = ACTIONS(1981),
    [sym_data_type_string] = ACTIONS(1981),
    [sym_data_type_datetime] = ACTIONS(1981),
    [sym_data_type_date] = ACTIONS(1981),
    [sym_data_type_time] = ACTIONS(1981),
    [sym_data_type_image] = ACTIONS(1981),
    [sym_data_format_url] = ACTIONS(1981),
    [sym_data_format_phone] = ACTIONS(1981),
    [sym_data_format_email] = ACTIONS(1981),
    [sym_data_format_iso8601] = ACTIONS(1981),
    [sym_auditlog_priv] = ACTIONS(1981),
    [sym_configuration_priv] = ACTIONS(1981),
    [sym_deviceconfig_priv] = ACTIONS(1981),
    [sym_ftp_priv] = ACTIONS(1981),
    [sym_http_priv] = ACTIONS(1981),
    [sym_networkconfig_priv] = ACTIONS(1981),
    [sym_programport_priv] = ACTIONS(1981),
    [sym_remoteui_priv] = ACTIONS(1981),
    [sym_securitycontrol_priv] = ACTIONS(1981),
    [sym_softwaremanagement_priv] = ACTIONS(1981),
    [sym_terminal_priv] = ACTIONS(1981),
    [sym_tpadmin_priv] = ACTIONS(1981),
    [sym_user1_priv] = ACTIONS(1981),
    [sym_user2_priv] = ACTIONS(1981),
    [sym_user3_priv] = ACTIONS(1981),
    [sym_user4_priv] = ACTIONS(1981),
    [sym_usermanagement_priv] = ACTIONS(1981),
    [sym_valid_account] = ACTIONS(1981),
    [sym_err_name_invalid] = ACTIONS(1981),
    [sym_err_password_invalid] = ACTIONS(1981),
    [sym_err_invalid_account] = ACTIONS(1981),
    [sym_err_no_authorization] = ACTIONS(1981),
    [sym_err_invalid_parameter] = ACTIONS(1981),
    [sym_err_user_locked_out] = ACTIONS(1981),
    [sym_err_user_account_expired] = ACTIONS(1981),
    [sym_err_general_error] = ACTIONS(1981),
    [sym_err_authentication_error] = ACTIONS(1981),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1981),
    [sym_tls_validate_certificate] = ACTIONS(1981),
    [sym_tdata] = ACTIONS(1981),
    [sym_tchannel] = ACTIONS(1981),
    [sym_tlevel] = ACTIONS(1981),
    [sym_tbutton] = ACTIONS(1981),
    [sym_ttimeline] = ACTIONS(1981),
    [sym_tcustom] = ACTIONS(1981),
    [sym_url_struct] = ACTIONS(1981),
    [sym_dns_struct] = ACTIONS(1981),
    [sym_ip_address_struct] = ACTIONS(1981),
    [sym_dev_info_struct] = ACTIONS(1981),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1981),
    [sym_data_feed] = ACTIONS(1981),
    [sym_data_field] = ACTIONS(1981),
    [sym_data_record] = ACTIONS(1981),
    [sym_wc_data_feed] = ACTIONS(1981),
    [sym_wc_data_field] = ACTIONS(1981),
    [sym_wc_data_record] = ACTIONS(1981),
    [sym_last_login_info] = ACTIONS(1981),
    [sym____reserved___] = ACTIONS(1981),
    [sym_date] = ACTIONS(1981),
    [sym_day] = ACTIONS(1981),
    [sym_time] = ACTIONS(1981),
    [sym_ldate] = ACTIONS(1981),
    [sym_push_device] = ACTIONS(1981),
    [sym_push_channel] = ACTIONS(1981),
    [sym_push_devchan] = ACTIONS(1981),
    [sym_release_device] = ACTIONS(1981),
    [sym_release_channel] = ACTIONS(1981),
    [sym_release_devchan] = ACTIONS(1981),
    [sym_master_slot] = ACTIONS(1981),
    [sym_get_pulse_time] = ACTIONS(1981),
    [sym_get_timer] = ACTIONS(1981),
    [sym_master_sn] = ACTIONS(1981),
    [sym_system_number] = ACTIONS(1981),
    [sym_button] = ACTIONS(1981),
    [sym_dv_channel] = ACTIONS(1981),
    [sym_data] = ACTIONS(1981),
    [sym_level] = ACTIONS(1981),
    [sym_timeline] = ACTIONS(1981),
    [sym_channel] = ACTIONS(1981),
    [sym_custom] = ACTIONS(1981),
    [sym_atoi] = ACTIONS(1981),
    [sym_atol] = ACTIONS(1981),
    [sym_atof] = ACTIONS(1981),
    [sym_itoa] = ACTIONS(1981),
    [sym_itohex] = ACTIONS(1981),
    [sym_hextoi] = ACTIONS(1981),
    [sym_ftoa] = ACTIONS(1981),
    [sym_find_string] = ACTIONS(1981),
    [sym_left_string] = ACTIONS(1981),
    [sym_right_string] = ACTIONS(1981),
    [sym_mid_string] = ACTIONS(1981),
    [sym_remove_string] = ACTIONS(1981),
    [sym_get_buffer_string] = ACTIONS(1981),
    [sym_get_buffer_char] = ACTIONS(1981),
    [sym_get_multi_buffer_string] = ACTIONS(1981),
    [sym_length_string] = ACTIONS(1981),
    [sym_max_length_string] = ACTIONS(1981),
    [sym_set_length_string] = ACTIONS(1981),
    [sym_compare_string] = ACTIONS(1981),
    [sym_upper_string] = ACTIONS(1981),
    [sym_lower_string] = ACTIONS(1981),
    [sym_get_last] = ACTIONS(1981),
    [sym_date_to_day] = ACTIONS(1981),
    [sym_date_to_month] = ACTIONS(1981),
    [sym_date_to_year] = ACTIONS(1981),
    [sym_time_to_hour] = ACTIONS(1981),
    [sym_time_to_minute] = ACTIONS(1981),
    [sym_time_to_second] = ACTIONS(1981),
    [sym_day_of_week] = ACTIONS(1981),
    [sym_set_pulse_time] = ACTIONS(1981),
    [sym_set_timer] = ACTIONS(1981),
    [sym_file_open] = ACTIONS(1981),
    [sym_file_close] = ACTIONS(1981),
    [sym_file_read] = ACTIONS(1981),
    [sym_file_read_line] = ACTIONS(1981),
    [sym_file_write] = ACTIONS(1981),
    [sym_file_write_line] = ACTIONS(1981),
    [sym_file_copy] = ACTIONS(1981),
    [sym_file_delete] = ACTIONS(1981),
    [sym_file_rename] = ACTIONS(1981),
    [sym_file_dir] = ACTIONS(1981),
    [sym_file_getdir] = ACTIONS(1981),
    [sym_file_setdir] = ACTIONS(1981),
    [sym_file_create_dir] = ACTIONS(1981),
    [sym_file_remove_dir] = ACTIONS(1981),
    [sym_file_seek] = ACTIONS(1981),
    [sym_length_array] = ACTIONS(1981),
    [sym_max_length_array] = ACTIONS(1981),
    [sym_set_length_array] = ACTIONS(1981),
    [sym_ip_client_open] = ACTIONS(1981),
    [sym_ip_client_close] = ACTIONS(1981),
    [sym_ip_server_open] = ACTIONS(1981),
    [sym_ip_server_close] = ACTIONS(1981),
    [sym_ip_bound_client_open] = ACTIONS(1981),
    [sym_ip_mc_server_open] = ACTIONS(1981),
    [sym_ip_set_option] = ACTIONS(1981),
    [sym_ssh_client_open] = ACTIONS(1981),
    [sym_ssh_client_close] = ACTIONS(1981),
    [sym_tls_client_open] = ACTIONS(1981),
    [sym_tls_client_close] = ACTIONS(1981),
    [sym_do_push] = ACTIONS(1981),
    [sym_do_push_timed] = ACTIONS(1981),
    [sym_do_release] = ACTIONS(1981),
    [sym_device_id] = ACTIONS(1981),
    [sym_device_info] = ACTIONS(1981),
    [sym_device_id_string] = ACTIONS(1981),
    [sym_device_standby] = ACTIONS(1981),
    [sym_device_wake] = ACTIONS(1981),
    [sym_timeline_create] = ACTIONS(1981),
    [sym_timeline_kill] = ACTIONS(1981),
    [sym_timeline_reload] = ACTIONS(1981),
    [sym_timeline_pause] = ACTIONS(1981),
    [sym_timeline_restart] = ACTIONS(1981),
    [sym_timeline_set] = ACTIONS(1981),
    [sym_timeline_get] = ACTIONS(1981),
    [sym_timeline_active] = ACTIONS(1981),
    [sym_timeline_dynamic_id] = ACTIONS(1981),
    [sym_variable_to_string] = ACTIONS(1981),
    [sym_string_to_variable] = ACTIONS(1981),
    [sym_length_variable_to_string] = ACTIONS(1981),
    [sym_type_cast] = ACTIONS(1981),
    [sym_variable_to_xml] = ACTIONS(1981),
    [sym_xml_to_variable] = ACTIONS(1981),
    [sym_length_variable_to_xml] = ACTIONS(1981),
    [sym_redirect_string] = ACTIONS(1981),
    [sym_random_number] = ACTIONS(1981),
    [sym_get_unique_id] = ACTIONS(1981),
    [sym_get_system_number] = ACTIONS(1981),
    [sym_set_system_number] = ACTIONS(1981),
    [sym_get_serial_number] = ACTIONS(1981),
    [sym_get_master_build] = ACTIONS(1981),
    [sym_set_validation_code] = ACTIONS(1981),
    [sym_reboot] = ACTIONS(1981),
    [sym_load_duet_module] = ACTIONS(1981),
    [sym_unload_duet_module] = ACTIONS(1981),
    [sym_duet_mem_size_set] = ACTIONS(1981),
    [sym_duet_mem_size_get] = ACTIONS(1981),
    [sym_dynamic_application_device] = ACTIONS(1981),
    [sym_dynamic_polled_port] = ACTIONS(1981),
    [sym_static_port_binding] = ACTIONS(1981),
    [sym_static_ip_binding] = ACTIONS(1981),
    [sym_get_dns_list] = ACTIONS(1981),
    [sym_set_dns_list] = ACTIONS(1981),
    [sym_get_ip_address] = ACTIONS(1981),
    [sym_set_ip_address] = ACTIONS(1981),
    [sym_get_url_list] = ACTIONS(1981),
    [sym_add_url_entry] = ACTIONS(1981),
    [sym_delete_url_entry] = ACTIONS(1981),
    [sym_abs_value] = ACTIONS(1981),
    [sym_max_value] = ACTIONS(1981),
    [sym_min_value] = ACTIONS(1981),
    [sym_exp_value] = ACTIONS(1981),
    [sym_log_value] = ACTIONS(1981),
    [sym_log10_value] = ACTIONS(1981),
    [sym_power_value] = ACTIONS(1981),
    [sym_sqrt_value] = ACTIONS(1981),
    [sym_raw_be] = ACTIONS(1981),
    [sym_raw_le] = ACTIONS(1981),
    [sym_internal_queue_size_set] = ACTIONS(1981),
    [sym_internal_queue_size_get] = ACTIONS(1981),
    [sym_internal_threshold_set] = ACTIONS(1981),
    [sym_internal_threshold_get] = ACTIONS(1981),
    [sym_rebuilt_event] = ACTIONS(1981),
    [sym_do_custom_event] = ACTIONS(1981),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1981),
    [sym_clkmgr_set_clk_source] = ACTIONS(1981),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1981),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1981),
    [sym_clkmgr_get_timezone] = ACTIONS(1981),
    [sym_clkmgr_set_timezone] = ACTIONS(1981),
    [sym_clkmgr_get_resync_period] = ACTIONS(1981),
    [sym_clkmgr_set_resync_period] = ACTIONS(1981),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1981),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1981),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1981),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1981),
    [sym_clkmgr_get_timeservers] = ACTIONS(1981),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1981),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1981),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1981),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1981),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1981),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1981),
    [sym_set_log_level] = ACTIONS(1981),
    [sym_get_log_level] = ACTIONS(1981),
    [sym_amx_log] = ACTIONS(1981),
    [sym_smtp_server_config_set] = ACTIONS(1981),
    [sym_smtp_server_config_get] = ACTIONS(1981),
    [sym_smtp_send] = ACTIONS(1981),
    [sym_validate_netlinx_account] = ACTIONS(1981),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1981),
    [sym_authenticate_certificate] = ACTIONS(1981),
    [sym_audit_netlinx_session_event] = ACTIONS(1981),
    [sym_audit_netlinx_generic_event] = ACTIONS(1981),
    [sym_get_available_flash_disk_space] = ACTIONS(1981),
    [sym_get_max_flash_disk_space] = ACTIONS(1981),
    [sym_data_create_feed] = ACTIONS(1981),
    [sym_data_delete_feed] = ACTIONS(1981),
    [sym_data_publish_feed] = ACTIONS(1981),
    [sym_data_get_published_feed] = ACTIONS(1981),
    [sym_data_add_record] = ACTIONS(1981),
    [sym_data_get_event_record] = ACTIONS(1981),
    [sym__wc_data_create_feed] = ACTIONS(1981),
    [sym__wc_data_add_record] = ACTIONS(1981),
    [sym__wc_data_get_event_record] = ACTIONS(1981),
    [sym_format] = ACTIONS(1981),
    [sym_astro_clock] = ACTIONS(1981),
    [sym_set_outdoor_temperature] = ACTIONS(1981),
    [sym_set_virtual_level_count] = ACTIONS(1981),
    [sym_set_virtual_channel_count] = ACTIONS(1981),
    [sym_set_virtual_port_count] = ACTIONS(1981),
    [sym__wc_encode_int] = ACTIONS(1981),
    [sym__wc_decode_int] = ACTIONS(1981),
    [sym__wc_int] = ACTIONS(1981),
    [sym__wc_to_ch_int] = ACTIONS(1981),
    [sym__ch_to_wc_int] = ACTIONS(1981),
    [sym__wc_find_string_int] = ACTIONS(1981),
    [sym__wc_left_string_int] = ACTIONS(1981),
    [sym__wc_lower_string_int] = ACTIONS(1981),
    [sym__wc_mid_string_int] = ACTIONS(1981),
    [sym__wc_remove_string_int] = ACTIONS(1981),
    [sym__wc_right_string_int] = ACTIONS(1981),
    [sym__wc_upper_string_int] = ACTIONS(1981),
    [sym__wc_compare_string_int] = ACTIONS(1981),
    [sym__wc_get_buffer_char_int] = ACTIONS(1981),
    [sym__wc_get_buffer_string_int] = ACTIONS(1981),
    [sym__wc_concat_string_int] = ACTIONS(1981),
    [sym__wc_file_open_int] = ACTIONS(1981),
    [sym__wc_file_close_int] = ACTIONS(1981),
    [sym__wc_file_read_int] = ACTIONS(1981),
    [sym__wc_file_read_line_int] = ACTIONS(1981),
    [sym__wc_file_write_int] = ACTIONS(1981),
    [sym__wc_file_write_line_int] = ACTIONS(1981),
    [aux_sym_preproc_include_token1] = ACTIONS(1981),
    [aux_sym_preproc_define_token1] = ACTIONS(1983),
    [aux_sym_preproc_warn_token1] = ACTIONS(1983),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1983),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1983),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1983),
    [anon_sym_LBRACK] = ACTIONS(1983),
    [anon_sym_LBRACE] = ACTIONS(1983),
    [anon_sym_RBRACE] = ACTIONS(1983),
    [anon_sym_LPAREN] = ACTIONS(1981),
    [anon_sym_BANG] = ACTIONS(1983),
    [anon_sym_TILDE] = ACTIONS(1983),
    [anon_sym_DASH] = ACTIONS(1981),
    [anon_sym_PLUS] = ACTIONS(1981),
    [anon_sym_DASH_DASH] = ACTIONS(1983),
    [anon_sym_PLUS_PLUS] = ACTIONS(1983),
    [anon_sym_DQUOTE] = ACTIONS(1983),
    [anon_sym_SQUOTE] = ACTIONS(1983),
    [sym_number_literal] = ACTIONS(1983),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(550)] = {
    [sym_identifier] = ACTIONS(1701),
    [sym_define_call_keyword] = ACTIONS(1701),
    [sym_define_function_keyword] = ACTIONS(1701),
    [sym_define_library_function_keyword] = ACTIONS(1701),
    [sym_define_module_keyword] = ACTIONS(1701),
    [sym_char_keyword] = ACTIONS(1701),
    [sym_widechar_keyword] = ACTIONS(1701),
    [sym_integer_keyword] = ACTIONS(1701),
    [sym_sinteger_keyword] = ACTIONS(1701),
    [sym_long_keyword] = ACTIONS(1701),
    [sym_slong_keyword] = ACTIONS(1701),
    [sym_float_keyword] = ACTIONS(1701),
    [sym_double_keyword] = ACTIONS(1701),
    [sym_variant_keyword] = ACTIONS(1701),
    [sym_variantarray_keyword] = ACTIONS(1701),
    [sym_local_var_keyword] = ACTIONS(1701),
    [sym_stack_var_keyword] = ACTIONS(1701),
    [sym_constant_keyword] = ACTIONS(1701),
    [sym_volatile_keyword] = ACTIONS(1701),
    [sym_non_volatile_keyword] = ACTIONS(1701),
    [sym_persistent_keyword] = ACTIONS(1701),
    [sym_dev_keyword] = ACTIONS(1701),
    [sym_devlev_keyword] = ACTIONS(1701),
    [sym_devchan_keyword] = ACTIONS(1701),
    [sym_if_keyword] = ACTIONS(1701),
    [sym_switch_keyword] = ACTIONS(1701),
    [sym_case_keyword] = ACTIONS(1701),
    [sym_default_keyword] = ACTIONS(1701),
    [sym_while_keyword] = ACTIONS(1701),
    [sym_for_keyword] = ACTIONS(1701),
    [sym_break_keyword] = ACTIONS(1701),
    [sym_continue_keyword] = ACTIONS(1701),
    [sym_return_keyword] = ACTIONS(1701),
    [sym_select_keyword] = ACTIONS(1701),
    [sym_active_keyword] = ACTIONS(1701),
    [sym_struct_keyword] = ACTIONS(1701),
    [sym_structure_keyword] = ACTIONS(1701),
    [aux_sym_on_keyword_token1] = ACTIONS(1701),
    [aux_sym_off_keyword_token1] = ACTIONS(1701),
    [sym_send_level_keyword] = ACTIONS(1701),
    [sym_send_string_keyword] = ACTIONS(1701),
    [sym_send_command_keyword] = ACTIONS(1701),
    [sym_clear_buffer_keyword] = ACTIONS(1701),
    [sym_create_buffer_keyword] = ACTIONS(1701),
    [sym_create_multi_buffer_keyword] = ACTIONS(1701),
    [sym_call_keyword] = ACTIONS(1701),
    [sym_system_call_keyword] = ACTIONS(1701),
    [sym_devchan_to_keyword] = ACTIONS(1701),
    [sym_devchan_min_to_keyword] = ACTIONS(1701),
    [sym_devchan_total_off_keyword] = ACTIONS(1701),
    [sym_devchan_pulse_keyword] = ACTIONS(1701),
    [sym_wait_keyword] = ACTIONS(1701),
    [sym_cancel_wait_keyword] = ACTIONS(1701),
    [sym_wait_until_keyword] = ACTIONS(1701),
    [sym_cancel_wait_until_keyword] = ACTIONS(1701),
    [sym_cancel_all_wait_keyword] = ACTIONS(1701),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1701),
    [sym_not] = ACTIONS(1701),
    [sym_bnot] = ACTIONS(1701),
    [sym___file__] = ACTIONS(1701),
    [sym___date__] = ACTIONS(1701),
    [sym___ldate__] = ACTIONS(1701),
    [sym___time__] = ACTIONS(1701),
    [sym___line__] = ACTIONS(1701),
    [sym___name__] = ACTIONS(1701),
    [sym___netlinx__] = ACTIONS(1701),
    [sym_netlinx_axi_version] = ACTIONS(1701),
    [sym_url_flg_tcp] = ACTIONS(1701),
    [sym_url_flg_accinfopresent] = ACTIONS(1701),
    [sym_url_flg_temp] = ACTIONS(1701),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1701),
    [sym_url_flg_stat_mask] = ACTIONS(1701),
    [sym_url_flg_stat_lookup] = ACTIONS(1701),
    [sym_url_flg_stat_connecting] = ACTIONS(1701),
    [sym_url_flg_stat_waiting] = ACTIONS(1701),
    [sym_url_flg_stat_connected] = ACTIONS(1701),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1701),
    [sym_first_virtual_device] = ACTIONS(1701),
    [sym_first_local_port] = ACTIONS(1701),
    [sym_dynamic_virtual_device] = ACTIONS(1701),
    [sym_timeline_once] = ACTIONS(1701),
    [sym_timeline_repeat] = ACTIONS(1701),
    [sym_timeline_absolute] = ACTIONS(1701),
    [sym_timeline_relative] = ACTIONS(1701),
    [sym_do_push_timed_infinite] = ACTIONS(1701),
    [sym_source_type_no_address] = ACTIONS(1701),
    [sym_source_type_neuron_id] = ACTIONS(1701),
    [sym_source_type_ip_address] = ACTIONS(1701),
    [sym_source_type_axlink] = ACTIONS(1701),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1701),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1701),
    [sym_source_type_ip_socket_address] = ACTIONS(1701),
    [sym_source_type_rs232] = ACTIONS(1701),
    [sym_source_type_internal] = ACTIONS(1701),
    [sym_source_type_mac_address] = ACTIONS(1701),
    [sym_source_type_ipv4_port] = ACTIONS(1701),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1701),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1701),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1701),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1701),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1701),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1701),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1701),
    [sym_true] = ACTIONS(1701),
    [sym_false] = ACTIONS(1701),
    [sym_file_read_only] = ACTIONS(1701),
    [sym_file_rw_new] = ACTIONS(1701),
    [sym_file_rw_append] = ACTIONS(1701),
    [sym_ip_tcp] = ACTIONS(1701),
    [sym_ip_udp] = ACTIONS(1701),
    [sym_ip_udp_2way] = ACTIONS(1701),
    [sym_xml_encode_types] = ACTIONS(1701),
    [sym_xml_encode_char_as_list] = ACTIONS(1701),
    [sym_xml_encode_le] = ACTIONS(1701),
    [sym_xml_decode_types] = ACTIONS(1701),
    [sym_xml_decode_no_preserve] = ACTIONS(1701),
    [sym_duet_dev_type_display_device] = ACTIONS(1701),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1701),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1701),
    [sym_duet_dev_type_io_device] = ACTIONS(1701),
    [sym_duet_dev_type_relay_device] = ACTIONS(1701),
    [sym_duet_dev_type_ups] = ACTIONS(1701),
    [sym_duet_dev_type_amplifier] = ACTIONS(1701),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1701),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1701),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1701),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1701),
    [sym_duet_dev_type_camera] = ACTIONS(1701),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1701),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1701),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1701),
    [sym_duet_dev_type_dss] = ACTIONS(1701),
    [sym_duet_dev_type_dvr] = ACTIONS(1701),
    [sym_duet_dev_type_disc_device] = ACTIONS(1701),
    [sym_duet_dev_type_document_camera] = ACTIONS(1701),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1701),
    [sym_duet_dev_type_hvac] = ACTIONS(1701),
    [sym_duet_dev_type_keypad] = ACTIONS(1701),
    [sym_duet_dev_type_light] = ACTIONS(1701),
    [sym_duet_dev_type_monitor] = ACTIONS(1701),
    [sym_duet_dev_type_motor] = ACTIONS(1701),
    [sym_duet_dev_type_multi_window] = ACTIONS(1701),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1701),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1701),
    [sym_duet_dev_type_receiver] = ACTIONS(1701),
    [sym_duet_dev_type_security_system] = ACTIONS(1701),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1701),
    [sym_duet_dev_type_settop_box] = ACTIONS(1701),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1701),
    [sym_duet_dev_type_switcher] = ACTIONS(1701),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1701),
    [sym_duet_dev_type_tv] = ACTIONS(1701),
    [sym_duet_dev_type_utility] = ACTIONS(1701),
    [sym_duet_dev_type_vcr] = ACTIONS(1701),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1701),
    [sym_duet_dev_type_video_processor] = ACTIONS(1701),
    [sym_duet_dev_type_video_projector] = ACTIONS(1701),
    [sym_duet_dev_type_video_wall] = ACTIONS(1701),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1701),
    [sym_duet_dev_type_weather] = ACTIONS(1701),
    [sym_duet_dev_not_polled] = ACTIONS(1701),
    [sym_duet_dev_polled] = ACTIONS(1701),
    [sym_clkmgr_mode_network] = ACTIONS(1701),
    [sym_clkmgr_mode_standalone] = ACTIONS(1701),
    [sym_normal_standby] = ACTIONS(1701),
    [sym_normal_wake] = ACTIONS(1701),
    [sym_ip_multicast_ttl_option] = ACTIONS(1701),
    [sym_ip_ttl_subnet] = ACTIONS(1701),
    [sym_ip_ttl_site] = ACTIONS(1701),
    [sym_ip_ttl_region] = ACTIONS(1701),
    [sym_ip_ttl_continent] = ACTIONS(1701),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1701),
    [sym_ip_nodelay_on] = ACTIONS(1701),
    [sym_ip_nodelay_off] = ACTIONS(1701),
    [sym_amx_error] = ACTIONS(1701),
    [sym_amx_warning] = ACTIONS(1701),
    [sym_amx_info] = ACTIONS(1701),
    [sym_amx_debug] = ACTIONS(1701),
    [sym_netlinx_login_success] = ACTIONS(1701),
    [sym_netlinx_login_fail] = ACTIONS(1701),
    [sym_netlinx_logout] = ACTIONS(1701),
    [sym_null_str] = ACTIONS(1701),
    [sym_smtp_address] = ACTIONS(1701),
    [sym_smtp_port_number] = ACTIONS(1701),
    [sym_smtp_username] = ACTIONS(1701),
    [sym_smtp_password] = ACTIONS(1701),
    [sym_smtp_require_tls] = ACTIONS(1701),
    [sym_smtp_from] = ACTIONS(1701),
    [sym_smtp_tls_true] = ACTIONS(1701),
    [sym_smtp_tls_false] = ACTIONS(1701),
    [sym_listview_on_row_select_event] = ACTIONS(1701),
    [sym_data_structure_unknown] = ACTIONS(1701),
    [sym_data_structure_datafeed] = ACTIONS(1701),
    [sym_data_structure_datarecordset] = ACTIONS(1701),
    [sym_data_structure_datarecord] = ACTIONS(1701),
    [sym_data_structure_datafield] = ACTIONS(1701),
    [sym_data_max_content_field_count] = ACTIONS(1701),
    [sym_data_max_metadata_field_count] = ACTIONS(1701),
    [sym_data_max_id_length] = ACTIONS(1701),
    [sym_data_max_name_length] = ACTIONS(1701),
    [sym_data_max_description_length] = ACTIONS(1701),
    [sym_data_max_type_length] = ACTIONS(1701),
    [sym_data_max_format_length] = ACTIONS(1701),
    [sym_data_max_source_length] = ACTIONS(1701),
    [sym_data_max_value_length] = ACTIONS(1701),
    [sym_data_max_label_length] = ACTIONS(1701),
    [sym_data_max_url_length] = ACTIONS(1701),
    [sym_max_last_login_info_length] = ACTIONS(1701),
    [sym_data_type_unknown] = ACTIONS(1701),
    [sym_data_type_string] = ACTIONS(1701),
    [sym_data_type_datetime] = ACTIONS(1701),
    [sym_data_type_date] = ACTIONS(1701),
    [sym_data_type_time] = ACTIONS(1701),
    [sym_data_type_image] = ACTIONS(1701),
    [sym_data_format_url] = ACTIONS(1701),
    [sym_data_format_phone] = ACTIONS(1701),
    [sym_data_format_email] = ACTIONS(1701),
    [sym_data_format_iso8601] = ACTIONS(1701),
    [sym_auditlog_priv] = ACTIONS(1701),
    [sym_configuration_priv] = ACTIONS(1701),
    [sym_deviceconfig_priv] = ACTIONS(1701),
    [sym_ftp_priv] = ACTIONS(1701),
    [sym_http_priv] = ACTIONS(1701),
    [sym_networkconfig_priv] = ACTIONS(1701),
    [sym_programport_priv] = ACTIONS(1701),
    [sym_remoteui_priv] = ACTIONS(1701),
    [sym_securitycontrol_priv] = ACTIONS(1701),
    [sym_softwaremanagement_priv] = ACTIONS(1701),
    [sym_terminal_priv] = ACTIONS(1701),
    [sym_tpadmin_priv] = ACTIONS(1701),
    [sym_user1_priv] = ACTIONS(1701),
    [sym_user2_priv] = ACTIONS(1701),
    [sym_user3_priv] = ACTIONS(1701),
    [sym_user4_priv] = ACTIONS(1701),
    [sym_usermanagement_priv] = ACTIONS(1701),
    [sym_valid_account] = ACTIONS(1701),
    [sym_err_name_invalid] = ACTIONS(1701),
    [sym_err_password_invalid] = ACTIONS(1701),
    [sym_err_invalid_account] = ACTIONS(1701),
    [sym_err_no_authorization] = ACTIONS(1701),
    [sym_err_invalid_parameter] = ACTIONS(1701),
    [sym_err_user_locked_out] = ACTIONS(1701),
    [sym_err_user_account_expired] = ACTIONS(1701),
    [sym_err_general_error] = ACTIONS(1701),
    [sym_err_authentication_error] = ACTIONS(1701),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1701),
    [sym_tls_validate_certificate] = ACTIONS(1701),
    [sym_tdata] = ACTIONS(1701),
    [sym_tchannel] = ACTIONS(1701),
    [sym_tlevel] = ACTIONS(1701),
    [sym_tbutton] = ACTIONS(1701),
    [sym_ttimeline] = ACTIONS(1701),
    [sym_tcustom] = ACTIONS(1701),
    [sym_url_struct] = ACTIONS(1701),
    [sym_dns_struct] = ACTIONS(1701),
    [sym_ip_address_struct] = ACTIONS(1701),
    [sym_dev_info_struct] = ACTIONS(1701),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1701),
    [sym_data_feed] = ACTIONS(1701),
    [sym_data_field] = ACTIONS(1701),
    [sym_data_record] = ACTIONS(1701),
    [sym_wc_data_feed] = ACTIONS(1701),
    [sym_wc_data_field] = ACTIONS(1701),
    [sym_wc_data_record] = ACTIONS(1701),
    [sym_last_login_info] = ACTIONS(1701),
    [sym____reserved___] = ACTIONS(1701),
    [sym_date] = ACTIONS(1701),
    [sym_day] = ACTIONS(1701),
    [sym_time] = ACTIONS(1701),
    [sym_ldate] = ACTIONS(1701),
    [sym_push_device] = ACTIONS(1701),
    [sym_push_channel] = ACTIONS(1701),
    [sym_push_devchan] = ACTIONS(1701),
    [sym_release_device] = ACTIONS(1701),
    [sym_release_channel] = ACTIONS(1701),
    [sym_release_devchan] = ACTIONS(1701),
    [sym_master_slot] = ACTIONS(1701),
    [sym_get_pulse_time] = ACTIONS(1701),
    [sym_get_timer] = ACTIONS(1701),
    [sym_master_sn] = ACTIONS(1701),
    [sym_system_number] = ACTIONS(1701),
    [sym_button] = ACTIONS(1701),
    [sym_dv_channel] = ACTIONS(1701),
    [sym_data] = ACTIONS(1701),
    [sym_level] = ACTIONS(1701),
    [sym_timeline] = ACTIONS(1701),
    [sym_channel] = ACTIONS(1701),
    [sym_custom] = ACTIONS(1701),
    [sym_atoi] = ACTIONS(1701),
    [sym_atol] = ACTIONS(1701),
    [sym_atof] = ACTIONS(1701),
    [sym_itoa] = ACTIONS(1701),
    [sym_itohex] = ACTIONS(1701),
    [sym_hextoi] = ACTIONS(1701),
    [sym_ftoa] = ACTIONS(1701),
    [sym_find_string] = ACTIONS(1701),
    [sym_left_string] = ACTIONS(1701),
    [sym_right_string] = ACTIONS(1701),
    [sym_mid_string] = ACTIONS(1701),
    [sym_remove_string] = ACTIONS(1701),
    [sym_get_buffer_string] = ACTIONS(1701),
    [sym_get_buffer_char] = ACTIONS(1701),
    [sym_get_multi_buffer_string] = ACTIONS(1701),
    [sym_length_string] = ACTIONS(1701),
    [sym_max_length_string] = ACTIONS(1701),
    [sym_set_length_string] = ACTIONS(1701),
    [sym_compare_string] = ACTIONS(1701),
    [sym_upper_string] = ACTIONS(1701),
    [sym_lower_string] = ACTIONS(1701),
    [sym_get_last] = ACTIONS(1701),
    [sym_date_to_day] = ACTIONS(1701),
    [sym_date_to_month] = ACTIONS(1701),
    [sym_date_to_year] = ACTIONS(1701),
    [sym_time_to_hour] = ACTIONS(1701),
    [sym_time_to_minute] = ACTIONS(1701),
    [sym_time_to_second] = ACTIONS(1701),
    [sym_day_of_week] = ACTIONS(1701),
    [sym_set_pulse_time] = ACTIONS(1701),
    [sym_set_timer] = ACTIONS(1701),
    [sym_file_open] = ACTIONS(1701),
    [sym_file_close] = ACTIONS(1701),
    [sym_file_read] = ACTIONS(1701),
    [sym_file_read_line] = ACTIONS(1701),
    [sym_file_write] = ACTIONS(1701),
    [sym_file_write_line] = ACTIONS(1701),
    [sym_file_copy] = ACTIONS(1701),
    [sym_file_delete] = ACTIONS(1701),
    [sym_file_rename] = ACTIONS(1701),
    [sym_file_dir] = ACTIONS(1701),
    [sym_file_getdir] = ACTIONS(1701),
    [sym_file_setdir] = ACTIONS(1701),
    [sym_file_create_dir] = ACTIONS(1701),
    [sym_file_remove_dir] = ACTIONS(1701),
    [sym_file_seek] = ACTIONS(1701),
    [sym_length_array] = ACTIONS(1701),
    [sym_max_length_array] = ACTIONS(1701),
    [sym_set_length_array] = ACTIONS(1701),
    [sym_ip_client_open] = ACTIONS(1701),
    [sym_ip_client_close] = ACTIONS(1701),
    [sym_ip_server_open] = ACTIONS(1701),
    [sym_ip_server_close] = ACTIONS(1701),
    [sym_ip_bound_client_open] = ACTIONS(1701),
    [sym_ip_mc_server_open] = ACTIONS(1701),
    [sym_ip_set_option] = ACTIONS(1701),
    [sym_ssh_client_open] = ACTIONS(1701),
    [sym_ssh_client_close] = ACTIONS(1701),
    [sym_tls_client_open] = ACTIONS(1701),
    [sym_tls_client_close] = ACTIONS(1701),
    [sym_do_push] = ACTIONS(1701),
    [sym_do_push_timed] = ACTIONS(1701),
    [sym_do_release] = ACTIONS(1701),
    [sym_device_id] = ACTIONS(1701),
    [sym_device_info] = ACTIONS(1701),
    [sym_device_id_string] = ACTIONS(1701),
    [sym_device_standby] = ACTIONS(1701),
    [sym_device_wake] = ACTIONS(1701),
    [sym_timeline_create] = ACTIONS(1701),
    [sym_timeline_kill] = ACTIONS(1701),
    [sym_timeline_reload] = ACTIONS(1701),
    [sym_timeline_pause] = ACTIONS(1701),
    [sym_timeline_restart] = ACTIONS(1701),
    [sym_timeline_set] = ACTIONS(1701),
    [sym_timeline_get] = ACTIONS(1701),
    [sym_timeline_active] = ACTIONS(1701),
    [sym_timeline_dynamic_id] = ACTIONS(1701),
    [sym_variable_to_string] = ACTIONS(1701),
    [sym_string_to_variable] = ACTIONS(1701),
    [sym_length_variable_to_string] = ACTIONS(1701),
    [sym_type_cast] = ACTIONS(1701),
    [sym_variable_to_xml] = ACTIONS(1701),
    [sym_xml_to_variable] = ACTIONS(1701),
    [sym_length_variable_to_xml] = ACTIONS(1701),
    [sym_redirect_string] = ACTIONS(1701),
    [sym_random_number] = ACTIONS(1701),
    [sym_get_unique_id] = ACTIONS(1701),
    [sym_get_system_number] = ACTIONS(1701),
    [sym_set_system_number] = ACTIONS(1701),
    [sym_get_serial_number] = ACTIONS(1701),
    [sym_get_master_build] = ACTIONS(1701),
    [sym_set_validation_code] = ACTIONS(1701),
    [sym_reboot] = ACTIONS(1701),
    [sym_load_duet_module] = ACTIONS(1701),
    [sym_unload_duet_module] = ACTIONS(1701),
    [sym_duet_mem_size_set] = ACTIONS(1701),
    [sym_duet_mem_size_get] = ACTIONS(1701),
    [sym_dynamic_application_device] = ACTIONS(1701),
    [sym_dynamic_polled_port] = ACTIONS(1701),
    [sym_static_port_binding] = ACTIONS(1701),
    [sym_static_ip_binding] = ACTIONS(1701),
    [sym_get_dns_list] = ACTIONS(1701),
    [sym_set_dns_list] = ACTIONS(1701),
    [sym_get_ip_address] = ACTIONS(1701),
    [sym_set_ip_address] = ACTIONS(1701),
    [sym_get_url_list] = ACTIONS(1701),
    [sym_add_url_entry] = ACTIONS(1701),
    [sym_delete_url_entry] = ACTIONS(1701),
    [sym_abs_value] = ACTIONS(1701),
    [sym_max_value] = ACTIONS(1701),
    [sym_min_value] = ACTIONS(1701),
    [sym_exp_value] = ACTIONS(1701),
    [sym_log_value] = ACTIONS(1701),
    [sym_log10_value] = ACTIONS(1701),
    [sym_power_value] = ACTIONS(1701),
    [sym_sqrt_value] = ACTIONS(1701),
    [sym_raw_be] = ACTIONS(1701),
    [sym_raw_le] = ACTIONS(1701),
    [sym_internal_queue_size_set] = ACTIONS(1701),
    [sym_internal_queue_size_get] = ACTIONS(1701),
    [sym_internal_threshold_set] = ACTIONS(1701),
    [sym_internal_threshold_get] = ACTIONS(1701),
    [sym_rebuilt_event] = ACTIONS(1701),
    [sym_do_custom_event] = ACTIONS(1701),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1701),
    [sym_clkmgr_set_clk_source] = ACTIONS(1701),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1701),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1701),
    [sym_clkmgr_get_timezone] = ACTIONS(1701),
    [sym_clkmgr_set_timezone] = ACTIONS(1701),
    [sym_clkmgr_get_resync_period] = ACTIONS(1701),
    [sym_clkmgr_set_resync_period] = ACTIONS(1701),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1701),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1701),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1701),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1701),
    [sym_clkmgr_get_timeservers] = ACTIONS(1701),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1701),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1701),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1701),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1701),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1701),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1701),
    [sym_set_log_level] = ACTIONS(1701),
    [sym_get_log_level] = ACTIONS(1701),
    [sym_amx_log] = ACTIONS(1701),
    [sym_smtp_server_config_set] = ACTIONS(1701),
    [sym_smtp_server_config_get] = ACTIONS(1701),
    [sym_smtp_send] = ACTIONS(1701),
    [sym_validate_netlinx_account] = ACTIONS(1701),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1701),
    [sym_authenticate_certificate] = ACTIONS(1701),
    [sym_audit_netlinx_session_event] = ACTIONS(1701),
    [sym_audit_netlinx_generic_event] = ACTIONS(1701),
    [sym_get_available_flash_disk_space] = ACTIONS(1701),
    [sym_get_max_flash_disk_space] = ACTIONS(1701),
    [sym_data_create_feed] = ACTIONS(1701),
    [sym_data_delete_feed] = ACTIONS(1701),
    [sym_data_publish_feed] = ACTIONS(1701),
    [sym_data_get_published_feed] = ACTIONS(1701),
    [sym_data_add_record] = ACTIONS(1701),
    [sym_data_get_event_record] = ACTIONS(1701),
    [sym__wc_data_create_feed] = ACTIONS(1701),
    [sym__wc_data_add_record] = ACTIONS(1701),
    [sym__wc_data_get_event_record] = ACTIONS(1701),
    [sym_format] = ACTIONS(1701),
    [sym_astro_clock] = ACTIONS(1701),
    [sym_set_outdoor_temperature] = ACTIONS(1701),
    [sym_set_virtual_level_count] = ACTIONS(1701),
    [sym_set_virtual_channel_count] = ACTIONS(1701),
    [sym_set_virtual_port_count] = ACTIONS(1701),
    [sym__wc_encode_int] = ACTIONS(1701),
    [sym__wc_decode_int] = ACTIONS(1701),
    [sym__wc_int] = ACTIONS(1701),
    [sym__wc_to_ch_int] = ACTIONS(1701),
    [sym__ch_to_wc_int] = ACTIONS(1701),
    [sym__wc_find_string_int] = ACTIONS(1701),
    [sym__wc_left_string_int] = ACTIONS(1701),
    [sym__wc_lower_string_int] = ACTIONS(1701),
    [sym__wc_mid_string_int] = ACTIONS(1701),
    [sym__wc_remove_string_int] = ACTIONS(1701),
    [sym__wc_right_string_int] = ACTIONS(1701),
    [sym__wc_upper_string_int] = ACTIONS(1701),
    [sym__wc_compare_string_int] = ACTIONS(1701),
    [sym__wc_get_buffer_char_int] = ACTIONS(1701),
    [sym__wc_get_buffer_string_int] = ACTIONS(1701),
    [sym__wc_concat_string_int] = ACTIONS(1701),
    [sym__wc_file_open_int] = ACTIONS(1701),
    [sym__wc_file_close_int] = ACTIONS(1701),
    [sym__wc_file_read_int] = ACTIONS(1701),
    [sym__wc_file_read_line_int] = ACTIONS(1701),
    [sym__wc_file_write_int] = ACTIONS(1701),
    [sym__wc_file_write_line_int] = ACTIONS(1701),
    [aux_sym_preproc_include_token1] = ACTIONS(1701),
    [aux_sym_preproc_define_token1] = ACTIONS(1699),
    [aux_sym_preproc_warn_token1] = ACTIONS(1699),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1699),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1699),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1699),
    [anon_sym_LBRACK] = ACTIONS(1699),
    [anon_sym_LBRACE] = ACTIONS(1699),
    [anon_sym_RBRACE] = ACTIONS(1699),
    [anon_sym_LPAREN] = ACTIONS(1701),
    [anon_sym_BANG] = ACTIONS(1699),
    [anon_sym_TILDE] = ACTIONS(1699),
    [anon_sym_DASH] = ACTIONS(1701),
    [anon_sym_PLUS] = ACTIONS(1701),
    [anon_sym_DASH_DASH] = ACTIONS(1699),
    [anon_sym_PLUS_PLUS] = ACTIONS(1699),
    [anon_sym_DQUOTE] = ACTIONS(1699),
    [anon_sym_SQUOTE] = ACTIONS(1699),
    [sym_number_literal] = ACTIONS(1699),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(551)] = {
    [sym_identifier] = ACTIONS(1709),
    [sym_define_call_keyword] = ACTIONS(1709),
    [sym_define_function_keyword] = ACTIONS(1709),
    [sym_define_library_function_keyword] = ACTIONS(1709),
    [sym_define_module_keyword] = ACTIONS(1709),
    [sym_char_keyword] = ACTIONS(1709),
    [sym_widechar_keyword] = ACTIONS(1709),
    [sym_integer_keyword] = ACTIONS(1709),
    [sym_sinteger_keyword] = ACTIONS(1709),
    [sym_long_keyword] = ACTIONS(1709),
    [sym_slong_keyword] = ACTIONS(1709),
    [sym_float_keyword] = ACTIONS(1709),
    [sym_double_keyword] = ACTIONS(1709),
    [sym_variant_keyword] = ACTIONS(1709),
    [sym_variantarray_keyword] = ACTIONS(1709),
    [sym_local_var_keyword] = ACTIONS(1709),
    [sym_stack_var_keyword] = ACTIONS(1709),
    [sym_constant_keyword] = ACTIONS(1709),
    [sym_volatile_keyword] = ACTIONS(1709),
    [sym_non_volatile_keyword] = ACTIONS(1709),
    [sym_persistent_keyword] = ACTIONS(1709),
    [sym_dev_keyword] = ACTIONS(1709),
    [sym_devlev_keyword] = ACTIONS(1709),
    [sym_devchan_keyword] = ACTIONS(1709),
    [sym_if_keyword] = ACTIONS(1709),
    [sym_switch_keyword] = ACTIONS(1709),
    [sym_case_keyword] = ACTIONS(1709),
    [sym_default_keyword] = ACTIONS(1709),
    [sym_while_keyword] = ACTIONS(1709),
    [sym_for_keyword] = ACTIONS(1709),
    [sym_break_keyword] = ACTIONS(1709),
    [sym_continue_keyword] = ACTIONS(1709),
    [sym_return_keyword] = ACTIONS(1709),
    [sym_select_keyword] = ACTIONS(1709),
    [sym_active_keyword] = ACTIONS(1709),
    [sym_struct_keyword] = ACTIONS(1709),
    [sym_structure_keyword] = ACTIONS(1709),
    [aux_sym_on_keyword_token1] = ACTIONS(1709),
    [aux_sym_off_keyword_token1] = ACTIONS(1709),
    [sym_send_level_keyword] = ACTIONS(1709),
    [sym_send_string_keyword] = ACTIONS(1709),
    [sym_send_command_keyword] = ACTIONS(1709),
    [sym_clear_buffer_keyword] = ACTIONS(1709),
    [sym_create_buffer_keyword] = ACTIONS(1709),
    [sym_create_multi_buffer_keyword] = ACTIONS(1709),
    [sym_call_keyword] = ACTIONS(1709),
    [sym_system_call_keyword] = ACTIONS(1709),
    [sym_devchan_to_keyword] = ACTIONS(1709),
    [sym_devchan_min_to_keyword] = ACTIONS(1709),
    [sym_devchan_total_off_keyword] = ACTIONS(1709),
    [sym_devchan_pulse_keyword] = ACTIONS(1709),
    [sym_wait_keyword] = ACTIONS(1709),
    [sym_cancel_wait_keyword] = ACTIONS(1709),
    [sym_wait_until_keyword] = ACTIONS(1709),
    [sym_cancel_wait_until_keyword] = ACTIONS(1709),
    [sym_cancel_all_wait_keyword] = ACTIONS(1709),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1709),
    [sym_not] = ACTIONS(1709),
    [sym_bnot] = ACTIONS(1709),
    [sym___file__] = ACTIONS(1709),
    [sym___date__] = ACTIONS(1709),
    [sym___ldate__] = ACTIONS(1709),
    [sym___time__] = ACTIONS(1709),
    [sym___line__] = ACTIONS(1709),
    [sym___name__] = ACTIONS(1709),
    [sym___netlinx__] = ACTIONS(1709),
    [sym_netlinx_axi_version] = ACTIONS(1709),
    [sym_url_flg_tcp] = ACTIONS(1709),
    [sym_url_flg_accinfopresent] = ACTIONS(1709),
    [sym_url_flg_temp] = ACTIONS(1709),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1709),
    [sym_url_flg_stat_mask] = ACTIONS(1709),
    [sym_url_flg_stat_lookup] = ACTIONS(1709),
    [sym_url_flg_stat_connecting] = ACTIONS(1709),
    [sym_url_flg_stat_waiting] = ACTIONS(1709),
    [sym_url_flg_stat_connected] = ACTIONS(1709),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1709),
    [sym_first_virtual_device] = ACTIONS(1709),
    [sym_first_local_port] = ACTIONS(1709),
    [sym_dynamic_virtual_device] = ACTIONS(1709),
    [sym_timeline_once] = ACTIONS(1709),
    [sym_timeline_repeat] = ACTIONS(1709),
    [sym_timeline_absolute] = ACTIONS(1709),
    [sym_timeline_relative] = ACTIONS(1709),
    [sym_do_push_timed_infinite] = ACTIONS(1709),
    [sym_source_type_no_address] = ACTIONS(1709),
    [sym_source_type_neuron_id] = ACTIONS(1709),
    [sym_source_type_ip_address] = ACTIONS(1709),
    [sym_source_type_axlink] = ACTIONS(1709),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1709),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1709),
    [sym_source_type_ip_socket_address] = ACTIONS(1709),
    [sym_source_type_rs232] = ACTIONS(1709),
    [sym_source_type_internal] = ACTIONS(1709),
    [sym_source_type_mac_address] = ACTIONS(1709),
    [sym_source_type_ipv4_port] = ACTIONS(1709),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1709),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1709),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1709),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1709),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1709),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1709),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1709),
    [sym_true] = ACTIONS(1709),
    [sym_false] = ACTIONS(1709),
    [sym_file_read_only] = ACTIONS(1709),
    [sym_file_rw_new] = ACTIONS(1709),
    [sym_file_rw_append] = ACTIONS(1709),
    [sym_ip_tcp] = ACTIONS(1709),
    [sym_ip_udp] = ACTIONS(1709),
    [sym_ip_udp_2way] = ACTIONS(1709),
    [sym_xml_encode_types] = ACTIONS(1709),
    [sym_xml_encode_char_as_list] = ACTIONS(1709),
    [sym_xml_encode_le] = ACTIONS(1709),
    [sym_xml_decode_types] = ACTIONS(1709),
    [sym_xml_decode_no_preserve] = ACTIONS(1709),
    [sym_duet_dev_type_display_device] = ACTIONS(1709),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1709),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1709),
    [sym_duet_dev_type_io_device] = ACTIONS(1709),
    [sym_duet_dev_type_relay_device] = ACTIONS(1709),
    [sym_duet_dev_type_ups] = ACTIONS(1709),
    [sym_duet_dev_type_amplifier] = ACTIONS(1709),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1709),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1709),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1709),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1709),
    [sym_duet_dev_type_camera] = ACTIONS(1709),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1709),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1709),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1709),
    [sym_duet_dev_type_dss] = ACTIONS(1709),
    [sym_duet_dev_type_dvr] = ACTIONS(1709),
    [sym_duet_dev_type_disc_device] = ACTIONS(1709),
    [sym_duet_dev_type_document_camera] = ACTIONS(1709),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1709),
    [sym_duet_dev_type_hvac] = ACTIONS(1709),
    [sym_duet_dev_type_keypad] = ACTIONS(1709),
    [sym_duet_dev_type_light] = ACTIONS(1709),
    [sym_duet_dev_type_monitor] = ACTIONS(1709),
    [sym_duet_dev_type_motor] = ACTIONS(1709),
    [sym_duet_dev_type_multi_window] = ACTIONS(1709),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1709),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1709),
    [sym_duet_dev_type_receiver] = ACTIONS(1709),
    [sym_duet_dev_type_security_system] = ACTIONS(1709),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1709),
    [sym_duet_dev_type_settop_box] = ACTIONS(1709),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1709),
    [sym_duet_dev_type_switcher] = ACTIONS(1709),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1709),
    [sym_duet_dev_type_tv] = ACTIONS(1709),
    [sym_duet_dev_type_utility] = ACTIONS(1709),
    [sym_duet_dev_type_vcr] = ACTIONS(1709),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1709),
    [sym_duet_dev_type_video_processor] = ACTIONS(1709),
    [sym_duet_dev_type_video_projector] = ACTIONS(1709),
    [sym_duet_dev_type_video_wall] = ACTIONS(1709),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1709),
    [sym_duet_dev_type_weather] = ACTIONS(1709),
    [sym_duet_dev_not_polled] = ACTIONS(1709),
    [sym_duet_dev_polled] = ACTIONS(1709),
    [sym_clkmgr_mode_network] = ACTIONS(1709),
    [sym_clkmgr_mode_standalone] = ACTIONS(1709),
    [sym_normal_standby] = ACTIONS(1709),
    [sym_normal_wake] = ACTIONS(1709),
    [sym_ip_multicast_ttl_option] = ACTIONS(1709),
    [sym_ip_ttl_subnet] = ACTIONS(1709),
    [sym_ip_ttl_site] = ACTIONS(1709),
    [sym_ip_ttl_region] = ACTIONS(1709),
    [sym_ip_ttl_continent] = ACTIONS(1709),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1709),
    [sym_ip_nodelay_on] = ACTIONS(1709),
    [sym_ip_nodelay_off] = ACTIONS(1709),
    [sym_amx_error] = ACTIONS(1709),
    [sym_amx_warning] = ACTIONS(1709),
    [sym_amx_info] = ACTIONS(1709),
    [sym_amx_debug] = ACTIONS(1709),
    [sym_netlinx_login_success] = ACTIONS(1709),
    [sym_netlinx_login_fail] = ACTIONS(1709),
    [sym_netlinx_logout] = ACTIONS(1709),
    [sym_null_str] = ACTIONS(1709),
    [sym_smtp_address] = ACTIONS(1709),
    [sym_smtp_port_number] = ACTIONS(1709),
    [sym_smtp_username] = ACTIONS(1709),
    [sym_smtp_password] = ACTIONS(1709),
    [sym_smtp_require_tls] = ACTIONS(1709),
    [sym_smtp_from] = ACTIONS(1709),
    [sym_smtp_tls_true] = ACTIONS(1709),
    [sym_smtp_tls_false] = ACTIONS(1709),
    [sym_listview_on_row_select_event] = ACTIONS(1709),
    [sym_data_structure_unknown] = ACTIONS(1709),
    [sym_data_structure_datafeed] = ACTIONS(1709),
    [sym_data_structure_datarecordset] = ACTIONS(1709),
    [sym_data_structure_datarecord] = ACTIONS(1709),
    [sym_data_structure_datafield] = ACTIONS(1709),
    [sym_data_max_content_field_count] = ACTIONS(1709),
    [sym_data_max_metadata_field_count] = ACTIONS(1709),
    [sym_data_max_id_length] = ACTIONS(1709),
    [sym_data_max_name_length] = ACTIONS(1709),
    [sym_data_max_description_length] = ACTIONS(1709),
    [sym_data_max_type_length] = ACTIONS(1709),
    [sym_data_max_format_length] = ACTIONS(1709),
    [sym_data_max_source_length] = ACTIONS(1709),
    [sym_data_max_value_length] = ACTIONS(1709),
    [sym_data_max_label_length] = ACTIONS(1709),
    [sym_data_max_url_length] = ACTIONS(1709),
    [sym_max_last_login_info_length] = ACTIONS(1709),
    [sym_data_type_unknown] = ACTIONS(1709),
    [sym_data_type_string] = ACTIONS(1709),
    [sym_data_type_datetime] = ACTIONS(1709),
    [sym_data_type_date] = ACTIONS(1709),
    [sym_data_type_time] = ACTIONS(1709),
    [sym_data_type_image] = ACTIONS(1709),
    [sym_data_format_url] = ACTIONS(1709),
    [sym_data_format_phone] = ACTIONS(1709),
    [sym_data_format_email] = ACTIONS(1709),
    [sym_data_format_iso8601] = ACTIONS(1709),
    [sym_auditlog_priv] = ACTIONS(1709),
    [sym_configuration_priv] = ACTIONS(1709),
    [sym_deviceconfig_priv] = ACTIONS(1709),
    [sym_ftp_priv] = ACTIONS(1709),
    [sym_http_priv] = ACTIONS(1709),
    [sym_networkconfig_priv] = ACTIONS(1709),
    [sym_programport_priv] = ACTIONS(1709),
    [sym_remoteui_priv] = ACTIONS(1709),
    [sym_securitycontrol_priv] = ACTIONS(1709),
    [sym_softwaremanagement_priv] = ACTIONS(1709),
    [sym_terminal_priv] = ACTIONS(1709),
    [sym_tpadmin_priv] = ACTIONS(1709),
    [sym_user1_priv] = ACTIONS(1709),
    [sym_user2_priv] = ACTIONS(1709),
    [sym_user3_priv] = ACTIONS(1709),
    [sym_user4_priv] = ACTIONS(1709),
    [sym_usermanagement_priv] = ACTIONS(1709),
    [sym_valid_account] = ACTIONS(1709),
    [sym_err_name_invalid] = ACTIONS(1709),
    [sym_err_password_invalid] = ACTIONS(1709),
    [sym_err_invalid_account] = ACTIONS(1709),
    [sym_err_no_authorization] = ACTIONS(1709),
    [sym_err_invalid_parameter] = ACTIONS(1709),
    [sym_err_user_locked_out] = ACTIONS(1709),
    [sym_err_user_account_expired] = ACTIONS(1709),
    [sym_err_general_error] = ACTIONS(1709),
    [sym_err_authentication_error] = ACTIONS(1709),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1709),
    [sym_tls_validate_certificate] = ACTIONS(1709),
    [sym_tdata] = ACTIONS(1709),
    [sym_tchannel] = ACTIONS(1709),
    [sym_tlevel] = ACTIONS(1709),
    [sym_tbutton] = ACTIONS(1709),
    [sym_ttimeline] = ACTIONS(1709),
    [sym_tcustom] = ACTIONS(1709),
    [sym_url_struct] = ACTIONS(1709),
    [sym_dns_struct] = ACTIONS(1709),
    [sym_ip_address_struct] = ACTIONS(1709),
    [sym_dev_info_struct] = ACTIONS(1709),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1709),
    [sym_data_feed] = ACTIONS(1709),
    [sym_data_field] = ACTIONS(1709),
    [sym_data_record] = ACTIONS(1709),
    [sym_wc_data_feed] = ACTIONS(1709),
    [sym_wc_data_field] = ACTIONS(1709),
    [sym_wc_data_record] = ACTIONS(1709),
    [sym_last_login_info] = ACTIONS(1709),
    [sym____reserved___] = ACTIONS(1709),
    [sym_date] = ACTIONS(1709),
    [sym_day] = ACTIONS(1709),
    [sym_time] = ACTIONS(1709),
    [sym_ldate] = ACTIONS(1709),
    [sym_push_device] = ACTIONS(1709),
    [sym_push_channel] = ACTIONS(1709),
    [sym_push_devchan] = ACTIONS(1709),
    [sym_release_device] = ACTIONS(1709),
    [sym_release_channel] = ACTIONS(1709),
    [sym_release_devchan] = ACTIONS(1709),
    [sym_master_slot] = ACTIONS(1709),
    [sym_get_pulse_time] = ACTIONS(1709),
    [sym_get_timer] = ACTIONS(1709),
    [sym_master_sn] = ACTIONS(1709),
    [sym_system_number] = ACTIONS(1709),
    [sym_button] = ACTIONS(1709),
    [sym_dv_channel] = ACTIONS(1709),
    [sym_data] = ACTIONS(1709),
    [sym_level] = ACTIONS(1709),
    [sym_timeline] = ACTIONS(1709),
    [sym_channel] = ACTIONS(1709),
    [sym_custom] = ACTIONS(1709),
    [sym_atoi] = ACTIONS(1709),
    [sym_atol] = ACTIONS(1709),
    [sym_atof] = ACTIONS(1709),
    [sym_itoa] = ACTIONS(1709),
    [sym_itohex] = ACTIONS(1709),
    [sym_hextoi] = ACTIONS(1709),
    [sym_ftoa] = ACTIONS(1709),
    [sym_find_string] = ACTIONS(1709),
    [sym_left_string] = ACTIONS(1709),
    [sym_right_string] = ACTIONS(1709),
    [sym_mid_string] = ACTIONS(1709),
    [sym_remove_string] = ACTIONS(1709),
    [sym_get_buffer_string] = ACTIONS(1709),
    [sym_get_buffer_char] = ACTIONS(1709),
    [sym_get_multi_buffer_string] = ACTIONS(1709),
    [sym_length_string] = ACTIONS(1709),
    [sym_max_length_string] = ACTIONS(1709),
    [sym_set_length_string] = ACTIONS(1709),
    [sym_compare_string] = ACTIONS(1709),
    [sym_upper_string] = ACTIONS(1709),
    [sym_lower_string] = ACTIONS(1709),
    [sym_get_last] = ACTIONS(1709),
    [sym_date_to_day] = ACTIONS(1709),
    [sym_date_to_month] = ACTIONS(1709),
    [sym_date_to_year] = ACTIONS(1709),
    [sym_time_to_hour] = ACTIONS(1709),
    [sym_time_to_minute] = ACTIONS(1709),
    [sym_time_to_second] = ACTIONS(1709),
    [sym_day_of_week] = ACTIONS(1709),
    [sym_set_pulse_time] = ACTIONS(1709),
    [sym_set_timer] = ACTIONS(1709),
    [sym_file_open] = ACTIONS(1709),
    [sym_file_close] = ACTIONS(1709),
    [sym_file_read] = ACTIONS(1709),
    [sym_file_read_line] = ACTIONS(1709),
    [sym_file_write] = ACTIONS(1709),
    [sym_file_write_line] = ACTIONS(1709),
    [sym_file_copy] = ACTIONS(1709),
    [sym_file_delete] = ACTIONS(1709),
    [sym_file_rename] = ACTIONS(1709),
    [sym_file_dir] = ACTIONS(1709),
    [sym_file_getdir] = ACTIONS(1709),
    [sym_file_setdir] = ACTIONS(1709),
    [sym_file_create_dir] = ACTIONS(1709),
    [sym_file_remove_dir] = ACTIONS(1709),
    [sym_file_seek] = ACTIONS(1709),
    [sym_length_array] = ACTIONS(1709),
    [sym_max_length_array] = ACTIONS(1709),
    [sym_set_length_array] = ACTIONS(1709),
    [sym_ip_client_open] = ACTIONS(1709),
    [sym_ip_client_close] = ACTIONS(1709),
    [sym_ip_server_open] = ACTIONS(1709),
    [sym_ip_server_close] = ACTIONS(1709),
    [sym_ip_bound_client_open] = ACTIONS(1709),
    [sym_ip_mc_server_open] = ACTIONS(1709),
    [sym_ip_set_option] = ACTIONS(1709),
    [sym_ssh_client_open] = ACTIONS(1709),
    [sym_ssh_client_close] = ACTIONS(1709),
    [sym_tls_client_open] = ACTIONS(1709),
    [sym_tls_client_close] = ACTIONS(1709),
    [sym_do_push] = ACTIONS(1709),
    [sym_do_push_timed] = ACTIONS(1709),
    [sym_do_release] = ACTIONS(1709),
    [sym_device_id] = ACTIONS(1709),
    [sym_device_info] = ACTIONS(1709),
    [sym_device_id_string] = ACTIONS(1709),
    [sym_device_standby] = ACTIONS(1709),
    [sym_device_wake] = ACTIONS(1709),
    [sym_timeline_create] = ACTIONS(1709),
    [sym_timeline_kill] = ACTIONS(1709),
    [sym_timeline_reload] = ACTIONS(1709),
    [sym_timeline_pause] = ACTIONS(1709),
    [sym_timeline_restart] = ACTIONS(1709),
    [sym_timeline_set] = ACTIONS(1709),
    [sym_timeline_get] = ACTIONS(1709),
    [sym_timeline_active] = ACTIONS(1709),
    [sym_timeline_dynamic_id] = ACTIONS(1709),
    [sym_variable_to_string] = ACTIONS(1709),
    [sym_string_to_variable] = ACTIONS(1709),
    [sym_length_variable_to_string] = ACTIONS(1709),
    [sym_type_cast] = ACTIONS(1709),
    [sym_variable_to_xml] = ACTIONS(1709),
    [sym_xml_to_variable] = ACTIONS(1709),
    [sym_length_variable_to_xml] = ACTIONS(1709),
    [sym_redirect_string] = ACTIONS(1709),
    [sym_random_number] = ACTIONS(1709),
    [sym_get_unique_id] = ACTIONS(1709),
    [sym_get_system_number] = ACTIONS(1709),
    [sym_set_system_number] = ACTIONS(1709),
    [sym_get_serial_number] = ACTIONS(1709),
    [sym_get_master_build] = ACTIONS(1709),
    [sym_set_validation_code] = ACTIONS(1709),
    [sym_reboot] = ACTIONS(1709),
    [sym_load_duet_module] = ACTIONS(1709),
    [sym_unload_duet_module] = ACTIONS(1709),
    [sym_duet_mem_size_set] = ACTIONS(1709),
    [sym_duet_mem_size_get] = ACTIONS(1709),
    [sym_dynamic_application_device] = ACTIONS(1709),
    [sym_dynamic_polled_port] = ACTIONS(1709),
    [sym_static_port_binding] = ACTIONS(1709),
    [sym_static_ip_binding] = ACTIONS(1709),
    [sym_get_dns_list] = ACTIONS(1709),
    [sym_set_dns_list] = ACTIONS(1709),
    [sym_get_ip_address] = ACTIONS(1709),
    [sym_set_ip_address] = ACTIONS(1709),
    [sym_get_url_list] = ACTIONS(1709),
    [sym_add_url_entry] = ACTIONS(1709),
    [sym_delete_url_entry] = ACTIONS(1709),
    [sym_abs_value] = ACTIONS(1709),
    [sym_max_value] = ACTIONS(1709),
    [sym_min_value] = ACTIONS(1709),
    [sym_exp_value] = ACTIONS(1709),
    [sym_log_value] = ACTIONS(1709),
    [sym_log10_value] = ACTIONS(1709),
    [sym_power_value] = ACTIONS(1709),
    [sym_sqrt_value] = ACTIONS(1709),
    [sym_raw_be] = ACTIONS(1709),
    [sym_raw_le] = ACTIONS(1709),
    [sym_internal_queue_size_set] = ACTIONS(1709),
    [sym_internal_queue_size_get] = ACTIONS(1709),
    [sym_internal_threshold_set] = ACTIONS(1709),
    [sym_internal_threshold_get] = ACTIONS(1709),
    [sym_rebuilt_event] = ACTIONS(1709),
    [sym_do_custom_event] = ACTIONS(1709),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1709),
    [sym_clkmgr_set_clk_source] = ACTIONS(1709),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1709),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1709),
    [sym_clkmgr_get_timezone] = ACTIONS(1709),
    [sym_clkmgr_set_timezone] = ACTIONS(1709),
    [sym_clkmgr_get_resync_period] = ACTIONS(1709),
    [sym_clkmgr_set_resync_period] = ACTIONS(1709),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1709),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1709),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1709),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1709),
    [sym_clkmgr_get_timeservers] = ACTIONS(1709),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1709),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1709),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1709),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1709),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1709),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1709),
    [sym_set_log_level] = ACTIONS(1709),
    [sym_get_log_level] = ACTIONS(1709),
    [sym_amx_log] = ACTIONS(1709),
    [sym_smtp_server_config_set] = ACTIONS(1709),
    [sym_smtp_server_config_get] = ACTIONS(1709),
    [sym_smtp_send] = ACTIONS(1709),
    [sym_validate_netlinx_account] = ACTIONS(1709),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1709),
    [sym_authenticate_certificate] = ACTIONS(1709),
    [sym_audit_netlinx_session_event] = ACTIONS(1709),
    [sym_audit_netlinx_generic_event] = ACTIONS(1709),
    [sym_get_available_flash_disk_space] = ACTIONS(1709),
    [sym_get_max_flash_disk_space] = ACTIONS(1709),
    [sym_data_create_feed] = ACTIONS(1709),
    [sym_data_delete_feed] = ACTIONS(1709),
    [sym_data_publish_feed] = ACTIONS(1709),
    [sym_data_get_published_feed] = ACTIONS(1709),
    [sym_data_add_record] = ACTIONS(1709),
    [sym_data_get_event_record] = ACTIONS(1709),
    [sym__wc_data_create_feed] = ACTIONS(1709),
    [sym__wc_data_add_record] = ACTIONS(1709),
    [sym__wc_data_get_event_record] = ACTIONS(1709),
    [sym_format] = ACTIONS(1709),
    [sym_astro_clock] = ACTIONS(1709),
    [sym_set_outdoor_temperature] = ACTIONS(1709),
    [sym_set_virtual_level_count] = ACTIONS(1709),
    [sym_set_virtual_channel_count] = ACTIONS(1709),
    [sym_set_virtual_port_count] = ACTIONS(1709),
    [sym__wc_encode_int] = ACTIONS(1709),
    [sym__wc_decode_int] = ACTIONS(1709),
    [sym__wc_int] = ACTIONS(1709),
    [sym__wc_to_ch_int] = ACTIONS(1709),
    [sym__ch_to_wc_int] = ACTIONS(1709),
    [sym__wc_find_string_int] = ACTIONS(1709),
    [sym__wc_left_string_int] = ACTIONS(1709),
    [sym__wc_lower_string_int] = ACTIONS(1709),
    [sym__wc_mid_string_int] = ACTIONS(1709),
    [sym__wc_remove_string_int] = ACTIONS(1709),
    [sym__wc_right_string_int] = ACTIONS(1709),
    [sym__wc_upper_string_int] = ACTIONS(1709),
    [sym__wc_compare_string_int] = ACTIONS(1709),
    [sym__wc_get_buffer_char_int] = ACTIONS(1709),
    [sym__wc_get_buffer_string_int] = ACTIONS(1709),
    [sym__wc_concat_string_int] = ACTIONS(1709),
    [sym__wc_file_open_int] = ACTIONS(1709),
    [sym__wc_file_close_int] = ACTIONS(1709),
    [sym__wc_file_read_int] = ACTIONS(1709),
    [sym__wc_file_read_line_int] = ACTIONS(1709),
    [sym__wc_file_write_int] = ACTIONS(1709),
    [sym__wc_file_write_line_int] = ACTIONS(1709),
    [aux_sym_preproc_include_token1] = ACTIONS(1709),
    [aux_sym_preproc_define_token1] = ACTIONS(1707),
    [aux_sym_preproc_warn_token1] = ACTIONS(1707),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1707),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1707),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1707),
    [anon_sym_LBRACK] = ACTIONS(1707),
    [anon_sym_LBRACE] = ACTIONS(1707),
    [anon_sym_RBRACE] = ACTIONS(1707),
    [anon_sym_LPAREN] = ACTIONS(1709),
    [anon_sym_BANG] = ACTIONS(1707),
    [anon_sym_TILDE] = ACTIONS(1707),
    [anon_sym_DASH] = ACTIONS(1709),
    [anon_sym_PLUS] = ACTIONS(1709),
    [anon_sym_DASH_DASH] = ACTIONS(1707),
    [anon_sym_PLUS_PLUS] = ACTIONS(1707),
    [anon_sym_DQUOTE] = ACTIONS(1707),
    [anon_sym_SQUOTE] = ACTIONS(1707),
    [sym_number_literal] = ACTIONS(1707),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(552)] = {
    [sym_identifier] = ACTIONS(1705),
    [sym_define_call_keyword] = ACTIONS(1705),
    [sym_define_function_keyword] = ACTIONS(1705),
    [sym_define_library_function_keyword] = ACTIONS(1705),
    [sym_define_module_keyword] = ACTIONS(1705),
    [sym_char_keyword] = ACTIONS(1705),
    [sym_widechar_keyword] = ACTIONS(1705),
    [sym_integer_keyword] = ACTIONS(1705),
    [sym_sinteger_keyword] = ACTIONS(1705),
    [sym_long_keyword] = ACTIONS(1705),
    [sym_slong_keyword] = ACTIONS(1705),
    [sym_float_keyword] = ACTIONS(1705),
    [sym_double_keyword] = ACTIONS(1705),
    [sym_variant_keyword] = ACTIONS(1705),
    [sym_variantarray_keyword] = ACTIONS(1705),
    [sym_local_var_keyword] = ACTIONS(1705),
    [sym_stack_var_keyword] = ACTIONS(1705),
    [sym_constant_keyword] = ACTIONS(1705),
    [sym_volatile_keyword] = ACTIONS(1705),
    [sym_non_volatile_keyword] = ACTIONS(1705),
    [sym_persistent_keyword] = ACTIONS(1705),
    [sym_dev_keyword] = ACTIONS(1705),
    [sym_devlev_keyword] = ACTIONS(1705),
    [sym_devchan_keyword] = ACTIONS(1705),
    [sym_if_keyword] = ACTIONS(1705),
    [sym_switch_keyword] = ACTIONS(1705),
    [sym_case_keyword] = ACTIONS(1705),
    [sym_default_keyword] = ACTIONS(1705),
    [sym_while_keyword] = ACTIONS(1705),
    [sym_for_keyword] = ACTIONS(1705),
    [sym_break_keyword] = ACTIONS(1705),
    [sym_continue_keyword] = ACTIONS(1705),
    [sym_return_keyword] = ACTIONS(1705),
    [sym_select_keyword] = ACTIONS(1705),
    [sym_active_keyword] = ACTIONS(1705),
    [sym_struct_keyword] = ACTIONS(1705),
    [sym_structure_keyword] = ACTIONS(1705),
    [aux_sym_on_keyword_token1] = ACTIONS(1705),
    [aux_sym_off_keyword_token1] = ACTIONS(1705),
    [sym_send_level_keyword] = ACTIONS(1705),
    [sym_send_string_keyword] = ACTIONS(1705),
    [sym_send_command_keyword] = ACTIONS(1705),
    [sym_clear_buffer_keyword] = ACTIONS(1705),
    [sym_create_buffer_keyword] = ACTIONS(1705),
    [sym_create_multi_buffer_keyword] = ACTIONS(1705),
    [sym_call_keyword] = ACTIONS(1705),
    [sym_system_call_keyword] = ACTIONS(1705),
    [sym_devchan_to_keyword] = ACTIONS(1705),
    [sym_devchan_min_to_keyword] = ACTIONS(1705),
    [sym_devchan_total_off_keyword] = ACTIONS(1705),
    [sym_devchan_pulse_keyword] = ACTIONS(1705),
    [sym_wait_keyword] = ACTIONS(1705),
    [sym_cancel_wait_keyword] = ACTIONS(1705),
    [sym_wait_until_keyword] = ACTIONS(1705),
    [sym_cancel_wait_until_keyword] = ACTIONS(1705),
    [sym_cancel_all_wait_keyword] = ACTIONS(1705),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1705),
    [sym_not] = ACTIONS(1705),
    [sym_bnot] = ACTIONS(1705),
    [sym___file__] = ACTIONS(1705),
    [sym___date__] = ACTIONS(1705),
    [sym___ldate__] = ACTIONS(1705),
    [sym___time__] = ACTIONS(1705),
    [sym___line__] = ACTIONS(1705),
    [sym___name__] = ACTIONS(1705),
    [sym___netlinx__] = ACTIONS(1705),
    [sym_netlinx_axi_version] = ACTIONS(1705),
    [sym_url_flg_tcp] = ACTIONS(1705),
    [sym_url_flg_accinfopresent] = ACTIONS(1705),
    [sym_url_flg_temp] = ACTIONS(1705),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1705),
    [sym_url_flg_stat_mask] = ACTIONS(1705),
    [sym_url_flg_stat_lookup] = ACTIONS(1705),
    [sym_url_flg_stat_connecting] = ACTIONS(1705),
    [sym_url_flg_stat_waiting] = ACTIONS(1705),
    [sym_url_flg_stat_connected] = ACTIONS(1705),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1705),
    [sym_first_virtual_device] = ACTIONS(1705),
    [sym_first_local_port] = ACTIONS(1705),
    [sym_dynamic_virtual_device] = ACTIONS(1705),
    [sym_timeline_once] = ACTIONS(1705),
    [sym_timeline_repeat] = ACTIONS(1705),
    [sym_timeline_absolute] = ACTIONS(1705),
    [sym_timeline_relative] = ACTIONS(1705),
    [sym_do_push_timed_infinite] = ACTIONS(1705),
    [sym_source_type_no_address] = ACTIONS(1705),
    [sym_source_type_neuron_id] = ACTIONS(1705),
    [sym_source_type_ip_address] = ACTIONS(1705),
    [sym_source_type_axlink] = ACTIONS(1705),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1705),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1705),
    [sym_source_type_ip_socket_address] = ACTIONS(1705),
    [sym_source_type_rs232] = ACTIONS(1705),
    [sym_source_type_internal] = ACTIONS(1705),
    [sym_source_type_mac_address] = ACTIONS(1705),
    [sym_source_type_ipv4_port] = ACTIONS(1705),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1705),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1705),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1705),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1705),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1705),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1705),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1705),
    [sym_true] = ACTIONS(1705),
    [sym_false] = ACTIONS(1705),
    [sym_file_read_only] = ACTIONS(1705),
    [sym_file_rw_new] = ACTIONS(1705),
    [sym_file_rw_append] = ACTIONS(1705),
    [sym_ip_tcp] = ACTIONS(1705),
    [sym_ip_udp] = ACTIONS(1705),
    [sym_ip_udp_2way] = ACTIONS(1705),
    [sym_xml_encode_types] = ACTIONS(1705),
    [sym_xml_encode_char_as_list] = ACTIONS(1705),
    [sym_xml_encode_le] = ACTIONS(1705),
    [sym_xml_decode_types] = ACTIONS(1705),
    [sym_xml_decode_no_preserve] = ACTIONS(1705),
    [sym_duet_dev_type_display_device] = ACTIONS(1705),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1705),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1705),
    [sym_duet_dev_type_io_device] = ACTIONS(1705),
    [sym_duet_dev_type_relay_device] = ACTIONS(1705),
    [sym_duet_dev_type_ups] = ACTIONS(1705),
    [sym_duet_dev_type_amplifier] = ACTIONS(1705),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1705),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1705),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1705),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1705),
    [sym_duet_dev_type_camera] = ACTIONS(1705),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1705),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1705),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1705),
    [sym_duet_dev_type_dss] = ACTIONS(1705),
    [sym_duet_dev_type_dvr] = ACTIONS(1705),
    [sym_duet_dev_type_disc_device] = ACTIONS(1705),
    [sym_duet_dev_type_document_camera] = ACTIONS(1705),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1705),
    [sym_duet_dev_type_hvac] = ACTIONS(1705),
    [sym_duet_dev_type_keypad] = ACTIONS(1705),
    [sym_duet_dev_type_light] = ACTIONS(1705),
    [sym_duet_dev_type_monitor] = ACTIONS(1705),
    [sym_duet_dev_type_motor] = ACTIONS(1705),
    [sym_duet_dev_type_multi_window] = ACTIONS(1705),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1705),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1705),
    [sym_duet_dev_type_receiver] = ACTIONS(1705),
    [sym_duet_dev_type_security_system] = ACTIONS(1705),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1705),
    [sym_duet_dev_type_settop_box] = ACTIONS(1705),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1705),
    [sym_duet_dev_type_switcher] = ACTIONS(1705),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1705),
    [sym_duet_dev_type_tv] = ACTIONS(1705),
    [sym_duet_dev_type_utility] = ACTIONS(1705),
    [sym_duet_dev_type_vcr] = ACTIONS(1705),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1705),
    [sym_duet_dev_type_video_processor] = ACTIONS(1705),
    [sym_duet_dev_type_video_projector] = ACTIONS(1705),
    [sym_duet_dev_type_video_wall] = ACTIONS(1705),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1705),
    [sym_duet_dev_type_weather] = ACTIONS(1705),
    [sym_duet_dev_not_polled] = ACTIONS(1705),
    [sym_duet_dev_polled] = ACTIONS(1705),
    [sym_clkmgr_mode_network] = ACTIONS(1705),
    [sym_clkmgr_mode_standalone] = ACTIONS(1705),
    [sym_normal_standby] = ACTIONS(1705),
    [sym_normal_wake] = ACTIONS(1705),
    [sym_ip_multicast_ttl_option] = ACTIONS(1705),
    [sym_ip_ttl_subnet] = ACTIONS(1705),
    [sym_ip_ttl_site] = ACTIONS(1705),
    [sym_ip_ttl_region] = ACTIONS(1705),
    [sym_ip_ttl_continent] = ACTIONS(1705),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1705),
    [sym_ip_nodelay_on] = ACTIONS(1705),
    [sym_ip_nodelay_off] = ACTIONS(1705),
    [sym_amx_error] = ACTIONS(1705),
    [sym_amx_warning] = ACTIONS(1705),
    [sym_amx_info] = ACTIONS(1705),
    [sym_amx_debug] = ACTIONS(1705),
    [sym_netlinx_login_success] = ACTIONS(1705),
    [sym_netlinx_login_fail] = ACTIONS(1705),
    [sym_netlinx_logout] = ACTIONS(1705),
    [sym_null_str] = ACTIONS(1705),
    [sym_smtp_address] = ACTIONS(1705),
    [sym_smtp_port_number] = ACTIONS(1705),
    [sym_smtp_username] = ACTIONS(1705),
    [sym_smtp_password] = ACTIONS(1705),
    [sym_smtp_require_tls] = ACTIONS(1705),
    [sym_smtp_from] = ACTIONS(1705),
    [sym_smtp_tls_true] = ACTIONS(1705),
    [sym_smtp_tls_false] = ACTIONS(1705),
    [sym_listview_on_row_select_event] = ACTIONS(1705),
    [sym_data_structure_unknown] = ACTIONS(1705),
    [sym_data_structure_datafeed] = ACTIONS(1705),
    [sym_data_structure_datarecordset] = ACTIONS(1705),
    [sym_data_structure_datarecord] = ACTIONS(1705),
    [sym_data_structure_datafield] = ACTIONS(1705),
    [sym_data_max_content_field_count] = ACTIONS(1705),
    [sym_data_max_metadata_field_count] = ACTIONS(1705),
    [sym_data_max_id_length] = ACTIONS(1705),
    [sym_data_max_name_length] = ACTIONS(1705),
    [sym_data_max_description_length] = ACTIONS(1705),
    [sym_data_max_type_length] = ACTIONS(1705),
    [sym_data_max_format_length] = ACTIONS(1705),
    [sym_data_max_source_length] = ACTIONS(1705),
    [sym_data_max_value_length] = ACTIONS(1705),
    [sym_data_max_label_length] = ACTIONS(1705),
    [sym_data_max_url_length] = ACTIONS(1705),
    [sym_max_last_login_info_length] = ACTIONS(1705),
    [sym_data_type_unknown] = ACTIONS(1705),
    [sym_data_type_string] = ACTIONS(1705),
    [sym_data_type_datetime] = ACTIONS(1705),
    [sym_data_type_date] = ACTIONS(1705),
    [sym_data_type_time] = ACTIONS(1705),
    [sym_data_type_image] = ACTIONS(1705),
    [sym_data_format_url] = ACTIONS(1705),
    [sym_data_format_phone] = ACTIONS(1705),
    [sym_data_format_email] = ACTIONS(1705),
    [sym_data_format_iso8601] = ACTIONS(1705),
    [sym_auditlog_priv] = ACTIONS(1705),
    [sym_configuration_priv] = ACTIONS(1705),
    [sym_deviceconfig_priv] = ACTIONS(1705),
    [sym_ftp_priv] = ACTIONS(1705),
    [sym_http_priv] = ACTIONS(1705),
    [sym_networkconfig_priv] = ACTIONS(1705),
    [sym_programport_priv] = ACTIONS(1705),
    [sym_remoteui_priv] = ACTIONS(1705),
    [sym_securitycontrol_priv] = ACTIONS(1705),
    [sym_softwaremanagement_priv] = ACTIONS(1705),
    [sym_terminal_priv] = ACTIONS(1705),
    [sym_tpadmin_priv] = ACTIONS(1705),
    [sym_user1_priv] = ACTIONS(1705),
    [sym_user2_priv] = ACTIONS(1705),
    [sym_user3_priv] = ACTIONS(1705),
    [sym_user4_priv] = ACTIONS(1705),
    [sym_usermanagement_priv] = ACTIONS(1705),
    [sym_valid_account] = ACTIONS(1705),
    [sym_err_name_invalid] = ACTIONS(1705),
    [sym_err_password_invalid] = ACTIONS(1705),
    [sym_err_invalid_account] = ACTIONS(1705),
    [sym_err_no_authorization] = ACTIONS(1705),
    [sym_err_invalid_parameter] = ACTIONS(1705),
    [sym_err_user_locked_out] = ACTIONS(1705),
    [sym_err_user_account_expired] = ACTIONS(1705),
    [sym_err_general_error] = ACTIONS(1705),
    [sym_err_authentication_error] = ACTIONS(1705),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1705),
    [sym_tls_validate_certificate] = ACTIONS(1705),
    [sym_tdata] = ACTIONS(1705),
    [sym_tchannel] = ACTIONS(1705),
    [sym_tlevel] = ACTIONS(1705),
    [sym_tbutton] = ACTIONS(1705),
    [sym_ttimeline] = ACTIONS(1705),
    [sym_tcustom] = ACTIONS(1705),
    [sym_url_struct] = ACTIONS(1705),
    [sym_dns_struct] = ACTIONS(1705),
    [sym_ip_address_struct] = ACTIONS(1705),
    [sym_dev_info_struct] = ACTIONS(1705),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1705),
    [sym_data_feed] = ACTIONS(1705),
    [sym_data_field] = ACTIONS(1705),
    [sym_data_record] = ACTIONS(1705),
    [sym_wc_data_feed] = ACTIONS(1705),
    [sym_wc_data_field] = ACTIONS(1705),
    [sym_wc_data_record] = ACTIONS(1705),
    [sym_last_login_info] = ACTIONS(1705),
    [sym____reserved___] = ACTIONS(1705),
    [sym_date] = ACTIONS(1705),
    [sym_day] = ACTIONS(1705),
    [sym_time] = ACTIONS(1705),
    [sym_ldate] = ACTIONS(1705),
    [sym_push_device] = ACTIONS(1705),
    [sym_push_channel] = ACTIONS(1705),
    [sym_push_devchan] = ACTIONS(1705),
    [sym_release_device] = ACTIONS(1705),
    [sym_release_channel] = ACTIONS(1705),
    [sym_release_devchan] = ACTIONS(1705),
    [sym_master_slot] = ACTIONS(1705),
    [sym_get_pulse_time] = ACTIONS(1705),
    [sym_get_timer] = ACTIONS(1705),
    [sym_master_sn] = ACTIONS(1705),
    [sym_system_number] = ACTIONS(1705),
    [sym_button] = ACTIONS(1705),
    [sym_dv_channel] = ACTIONS(1705),
    [sym_data] = ACTIONS(1705),
    [sym_level] = ACTIONS(1705),
    [sym_timeline] = ACTIONS(1705),
    [sym_channel] = ACTIONS(1705),
    [sym_custom] = ACTIONS(1705),
    [sym_atoi] = ACTIONS(1705),
    [sym_atol] = ACTIONS(1705),
    [sym_atof] = ACTIONS(1705),
    [sym_itoa] = ACTIONS(1705),
    [sym_itohex] = ACTIONS(1705),
    [sym_hextoi] = ACTIONS(1705),
    [sym_ftoa] = ACTIONS(1705),
    [sym_find_string] = ACTIONS(1705),
    [sym_left_string] = ACTIONS(1705),
    [sym_right_string] = ACTIONS(1705),
    [sym_mid_string] = ACTIONS(1705),
    [sym_remove_string] = ACTIONS(1705),
    [sym_get_buffer_string] = ACTIONS(1705),
    [sym_get_buffer_char] = ACTIONS(1705),
    [sym_get_multi_buffer_string] = ACTIONS(1705),
    [sym_length_string] = ACTIONS(1705),
    [sym_max_length_string] = ACTIONS(1705),
    [sym_set_length_string] = ACTIONS(1705),
    [sym_compare_string] = ACTIONS(1705),
    [sym_upper_string] = ACTIONS(1705),
    [sym_lower_string] = ACTIONS(1705),
    [sym_get_last] = ACTIONS(1705),
    [sym_date_to_day] = ACTIONS(1705),
    [sym_date_to_month] = ACTIONS(1705),
    [sym_date_to_year] = ACTIONS(1705),
    [sym_time_to_hour] = ACTIONS(1705),
    [sym_time_to_minute] = ACTIONS(1705),
    [sym_time_to_second] = ACTIONS(1705),
    [sym_day_of_week] = ACTIONS(1705),
    [sym_set_pulse_time] = ACTIONS(1705),
    [sym_set_timer] = ACTIONS(1705),
    [sym_file_open] = ACTIONS(1705),
    [sym_file_close] = ACTIONS(1705),
    [sym_file_read] = ACTIONS(1705),
    [sym_file_read_line] = ACTIONS(1705),
    [sym_file_write] = ACTIONS(1705),
    [sym_file_write_line] = ACTIONS(1705),
    [sym_file_copy] = ACTIONS(1705),
    [sym_file_delete] = ACTIONS(1705),
    [sym_file_rename] = ACTIONS(1705),
    [sym_file_dir] = ACTIONS(1705),
    [sym_file_getdir] = ACTIONS(1705),
    [sym_file_setdir] = ACTIONS(1705),
    [sym_file_create_dir] = ACTIONS(1705),
    [sym_file_remove_dir] = ACTIONS(1705),
    [sym_file_seek] = ACTIONS(1705),
    [sym_length_array] = ACTIONS(1705),
    [sym_max_length_array] = ACTIONS(1705),
    [sym_set_length_array] = ACTIONS(1705),
    [sym_ip_client_open] = ACTIONS(1705),
    [sym_ip_client_close] = ACTIONS(1705),
    [sym_ip_server_open] = ACTIONS(1705),
    [sym_ip_server_close] = ACTIONS(1705),
    [sym_ip_bound_client_open] = ACTIONS(1705),
    [sym_ip_mc_server_open] = ACTIONS(1705),
    [sym_ip_set_option] = ACTIONS(1705),
    [sym_ssh_client_open] = ACTIONS(1705),
    [sym_ssh_client_close] = ACTIONS(1705),
    [sym_tls_client_open] = ACTIONS(1705),
    [sym_tls_client_close] = ACTIONS(1705),
    [sym_do_push] = ACTIONS(1705),
    [sym_do_push_timed] = ACTIONS(1705),
    [sym_do_release] = ACTIONS(1705),
    [sym_device_id] = ACTIONS(1705),
    [sym_device_info] = ACTIONS(1705),
    [sym_device_id_string] = ACTIONS(1705),
    [sym_device_standby] = ACTIONS(1705),
    [sym_device_wake] = ACTIONS(1705),
    [sym_timeline_create] = ACTIONS(1705),
    [sym_timeline_kill] = ACTIONS(1705),
    [sym_timeline_reload] = ACTIONS(1705),
    [sym_timeline_pause] = ACTIONS(1705),
    [sym_timeline_restart] = ACTIONS(1705),
    [sym_timeline_set] = ACTIONS(1705),
    [sym_timeline_get] = ACTIONS(1705),
    [sym_timeline_active] = ACTIONS(1705),
    [sym_timeline_dynamic_id] = ACTIONS(1705),
    [sym_variable_to_string] = ACTIONS(1705),
    [sym_string_to_variable] = ACTIONS(1705),
    [sym_length_variable_to_string] = ACTIONS(1705),
    [sym_type_cast] = ACTIONS(1705),
    [sym_variable_to_xml] = ACTIONS(1705),
    [sym_xml_to_variable] = ACTIONS(1705),
    [sym_length_variable_to_xml] = ACTIONS(1705),
    [sym_redirect_string] = ACTIONS(1705),
    [sym_random_number] = ACTIONS(1705),
    [sym_get_unique_id] = ACTIONS(1705),
    [sym_get_system_number] = ACTIONS(1705),
    [sym_set_system_number] = ACTIONS(1705),
    [sym_get_serial_number] = ACTIONS(1705),
    [sym_get_master_build] = ACTIONS(1705),
    [sym_set_validation_code] = ACTIONS(1705),
    [sym_reboot] = ACTIONS(1705),
    [sym_load_duet_module] = ACTIONS(1705),
    [sym_unload_duet_module] = ACTIONS(1705),
    [sym_duet_mem_size_set] = ACTIONS(1705),
    [sym_duet_mem_size_get] = ACTIONS(1705),
    [sym_dynamic_application_device] = ACTIONS(1705),
    [sym_dynamic_polled_port] = ACTIONS(1705),
    [sym_static_port_binding] = ACTIONS(1705),
    [sym_static_ip_binding] = ACTIONS(1705),
    [sym_get_dns_list] = ACTIONS(1705),
    [sym_set_dns_list] = ACTIONS(1705),
    [sym_get_ip_address] = ACTIONS(1705),
    [sym_set_ip_address] = ACTIONS(1705),
    [sym_get_url_list] = ACTIONS(1705),
    [sym_add_url_entry] = ACTIONS(1705),
    [sym_delete_url_entry] = ACTIONS(1705),
    [sym_abs_value] = ACTIONS(1705),
    [sym_max_value] = ACTIONS(1705),
    [sym_min_value] = ACTIONS(1705),
    [sym_exp_value] = ACTIONS(1705),
    [sym_log_value] = ACTIONS(1705),
    [sym_log10_value] = ACTIONS(1705),
    [sym_power_value] = ACTIONS(1705),
    [sym_sqrt_value] = ACTIONS(1705),
    [sym_raw_be] = ACTIONS(1705),
    [sym_raw_le] = ACTIONS(1705),
    [sym_internal_queue_size_set] = ACTIONS(1705),
    [sym_internal_queue_size_get] = ACTIONS(1705),
    [sym_internal_threshold_set] = ACTIONS(1705),
    [sym_internal_threshold_get] = ACTIONS(1705),
    [sym_rebuilt_event] = ACTIONS(1705),
    [sym_do_custom_event] = ACTIONS(1705),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1705),
    [sym_clkmgr_set_clk_source] = ACTIONS(1705),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1705),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1705),
    [sym_clkmgr_get_timezone] = ACTIONS(1705),
    [sym_clkmgr_set_timezone] = ACTIONS(1705),
    [sym_clkmgr_get_resync_period] = ACTIONS(1705),
    [sym_clkmgr_set_resync_period] = ACTIONS(1705),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1705),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1705),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1705),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1705),
    [sym_clkmgr_get_timeservers] = ACTIONS(1705),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1705),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1705),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1705),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1705),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1705),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1705),
    [sym_set_log_level] = ACTIONS(1705),
    [sym_get_log_level] = ACTIONS(1705),
    [sym_amx_log] = ACTIONS(1705),
    [sym_smtp_server_config_set] = ACTIONS(1705),
    [sym_smtp_server_config_get] = ACTIONS(1705),
    [sym_smtp_send] = ACTIONS(1705),
    [sym_validate_netlinx_account] = ACTIONS(1705),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1705),
    [sym_authenticate_certificate] = ACTIONS(1705),
    [sym_audit_netlinx_session_event] = ACTIONS(1705),
    [sym_audit_netlinx_generic_event] = ACTIONS(1705),
    [sym_get_available_flash_disk_space] = ACTIONS(1705),
    [sym_get_max_flash_disk_space] = ACTIONS(1705),
    [sym_data_create_feed] = ACTIONS(1705),
    [sym_data_delete_feed] = ACTIONS(1705),
    [sym_data_publish_feed] = ACTIONS(1705),
    [sym_data_get_published_feed] = ACTIONS(1705),
    [sym_data_add_record] = ACTIONS(1705),
    [sym_data_get_event_record] = ACTIONS(1705),
    [sym__wc_data_create_feed] = ACTIONS(1705),
    [sym__wc_data_add_record] = ACTIONS(1705),
    [sym__wc_data_get_event_record] = ACTIONS(1705),
    [sym_format] = ACTIONS(1705),
    [sym_astro_clock] = ACTIONS(1705),
    [sym_set_outdoor_temperature] = ACTIONS(1705),
    [sym_set_virtual_level_count] = ACTIONS(1705),
    [sym_set_virtual_channel_count] = ACTIONS(1705),
    [sym_set_virtual_port_count] = ACTIONS(1705),
    [sym__wc_encode_int] = ACTIONS(1705),
    [sym__wc_decode_int] = ACTIONS(1705),
    [sym__wc_int] = ACTIONS(1705),
    [sym__wc_to_ch_int] = ACTIONS(1705),
    [sym__ch_to_wc_int] = ACTIONS(1705),
    [sym__wc_find_string_int] = ACTIONS(1705),
    [sym__wc_left_string_int] = ACTIONS(1705),
    [sym__wc_lower_string_int] = ACTIONS(1705),
    [sym__wc_mid_string_int] = ACTIONS(1705),
    [sym__wc_remove_string_int] = ACTIONS(1705),
    [sym__wc_right_string_int] = ACTIONS(1705),
    [sym__wc_upper_string_int] = ACTIONS(1705),
    [sym__wc_compare_string_int] = ACTIONS(1705),
    [sym__wc_get_buffer_char_int] = ACTIONS(1705),
    [sym__wc_get_buffer_string_int] = ACTIONS(1705),
    [sym__wc_concat_string_int] = ACTIONS(1705),
    [sym__wc_file_open_int] = ACTIONS(1705),
    [sym__wc_file_close_int] = ACTIONS(1705),
    [sym__wc_file_read_int] = ACTIONS(1705),
    [sym__wc_file_read_line_int] = ACTIONS(1705),
    [sym__wc_file_write_int] = ACTIONS(1705),
    [sym__wc_file_write_line_int] = ACTIONS(1705),
    [aux_sym_preproc_include_token1] = ACTIONS(1705),
    [aux_sym_preproc_define_token1] = ACTIONS(1703),
    [aux_sym_preproc_warn_token1] = ACTIONS(1703),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1703),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1703),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1703),
    [anon_sym_LBRACK] = ACTIONS(1703),
    [anon_sym_LBRACE] = ACTIONS(1703),
    [anon_sym_RBRACE] = ACTIONS(1703),
    [anon_sym_LPAREN] = ACTIONS(1705),
    [anon_sym_BANG] = ACTIONS(1703),
    [anon_sym_TILDE] = ACTIONS(1703),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_DASH_DASH] = ACTIONS(1703),
    [anon_sym_PLUS_PLUS] = ACTIONS(1703),
    [anon_sym_DQUOTE] = ACTIONS(1703),
    [anon_sym_SQUOTE] = ACTIONS(1703),
    [sym_number_literal] = ACTIONS(1703),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(553)] = {
    [sym_identifier] = ACTIONS(1713),
    [sym_define_call_keyword] = ACTIONS(1713),
    [sym_define_function_keyword] = ACTIONS(1713),
    [sym_define_library_function_keyword] = ACTIONS(1713),
    [sym_define_module_keyword] = ACTIONS(1713),
    [sym_char_keyword] = ACTIONS(1713),
    [sym_widechar_keyword] = ACTIONS(1713),
    [sym_integer_keyword] = ACTIONS(1713),
    [sym_sinteger_keyword] = ACTIONS(1713),
    [sym_long_keyword] = ACTIONS(1713),
    [sym_slong_keyword] = ACTIONS(1713),
    [sym_float_keyword] = ACTIONS(1713),
    [sym_double_keyword] = ACTIONS(1713),
    [sym_variant_keyword] = ACTIONS(1713),
    [sym_variantarray_keyword] = ACTIONS(1713),
    [sym_local_var_keyword] = ACTIONS(1713),
    [sym_stack_var_keyword] = ACTIONS(1713),
    [sym_constant_keyword] = ACTIONS(1713),
    [sym_volatile_keyword] = ACTIONS(1713),
    [sym_non_volatile_keyword] = ACTIONS(1713),
    [sym_persistent_keyword] = ACTIONS(1713),
    [sym_dev_keyword] = ACTIONS(1713),
    [sym_devlev_keyword] = ACTIONS(1713),
    [sym_devchan_keyword] = ACTIONS(1713),
    [sym_if_keyword] = ACTIONS(1713),
    [sym_switch_keyword] = ACTIONS(1713),
    [sym_case_keyword] = ACTIONS(1713),
    [sym_default_keyword] = ACTIONS(1713),
    [sym_while_keyword] = ACTIONS(1713),
    [sym_for_keyword] = ACTIONS(1713),
    [sym_break_keyword] = ACTIONS(1713),
    [sym_continue_keyword] = ACTIONS(1713),
    [sym_return_keyword] = ACTIONS(1713),
    [sym_select_keyword] = ACTIONS(1713),
    [sym_active_keyword] = ACTIONS(1713),
    [sym_struct_keyword] = ACTIONS(1713),
    [sym_structure_keyword] = ACTIONS(1713),
    [aux_sym_on_keyword_token1] = ACTIONS(1713),
    [aux_sym_off_keyword_token1] = ACTIONS(1713),
    [sym_send_level_keyword] = ACTIONS(1713),
    [sym_send_string_keyword] = ACTIONS(1713),
    [sym_send_command_keyword] = ACTIONS(1713),
    [sym_clear_buffer_keyword] = ACTIONS(1713),
    [sym_create_buffer_keyword] = ACTIONS(1713),
    [sym_create_multi_buffer_keyword] = ACTIONS(1713),
    [sym_call_keyword] = ACTIONS(1713),
    [sym_system_call_keyword] = ACTIONS(1713),
    [sym_devchan_to_keyword] = ACTIONS(1713),
    [sym_devchan_min_to_keyword] = ACTIONS(1713),
    [sym_devchan_total_off_keyword] = ACTIONS(1713),
    [sym_devchan_pulse_keyword] = ACTIONS(1713),
    [sym_wait_keyword] = ACTIONS(1713),
    [sym_cancel_wait_keyword] = ACTIONS(1713),
    [sym_wait_until_keyword] = ACTIONS(1713),
    [sym_cancel_wait_until_keyword] = ACTIONS(1713),
    [sym_cancel_all_wait_keyword] = ACTIONS(1713),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1713),
    [sym_not] = ACTIONS(1713),
    [sym_bnot] = ACTIONS(1713),
    [sym___file__] = ACTIONS(1713),
    [sym___date__] = ACTIONS(1713),
    [sym___ldate__] = ACTIONS(1713),
    [sym___time__] = ACTIONS(1713),
    [sym___line__] = ACTIONS(1713),
    [sym___name__] = ACTIONS(1713),
    [sym___netlinx__] = ACTIONS(1713),
    [sym_netlinx_axi_version] = ACTIONS(1713),
    [sym_url_flg_tcp] = ACTIONS(1713),
    [sym_url_flg_accinfopresent] = ACTIONS(1713),
    [sym_url_flg_temp] = ACTIONS(1713),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1713),
    [sym_url_flg_stat_mask] = ACTIONS(1713),
    [sym_url_flg_stat_lookup] = ACTIONS(1713),
    [sym_url_flg_stat_connecting] = ACTIONS(1713),
    [sym_url_flg_stat_waiting] = ACTIONS(1713),
    [sym_url_flg_stat_connected] = ACTIONS(1713),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1713),
    [sym_first_virtual_device] = ACTIONS(1713),
    [sym_first_local_port] = ACTIONS(1713),
    [sym_dynamic_virtual_device] = ACTIONS(1713),
    [sym_timeline_once] = ACTIONS(1713),
    [sym_timeline_repeat] = ACTIONS(1713),
    [sym_timeline_absolute] = ACTIONS(1713),
    [sym_timeline_relative] = ACTIONS(1713),
    [sym_do_push_timed_infinite] = ACTIONS(1713),
    [sym_source_type_no_address] = ACTIONS(1713),
    [sym_source_type_neuron_id] = ACTIONS(1713),
    [sym_source_type_ip_address] = ACTIONS(1713),
    [sym_source_type_axlink] = ACTIONS(1713),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1713),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1713),
    [sym_source_type_ip_socket_address] = ACTIONS(1713),
    [sym_source_type_rs232] = ACTIONS(1713),
    [sym_source_type_internal] = ACTIONS(1713),
    [sym_source_type_mac_address] = ACTIONS(1713),
    [sym_source_type_ipv4_port] = ACTIONS(1713),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1713),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1713),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1713),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1713),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1713),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1713),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1713),
    [sym_true] = ACTIONS(1713),
    [sym_false] = ACTIONS(1713),
    [sym_file_read_only] = ACTIONS(1713),
    [sym_file_rw_new] = ACTIONS(1713),
    [sym_file_rw_append] = ACTIONS(1713),
    [sym_ip_tcp] = ACTIONS(1713),
    [sym_ip_udp] = ACTIONS(1713),
    [sym_ip_udp_2way] = ACTIONS(1713),
    [sym_xml_encode_types] = ACTIONS(1713),
    [sym_xml_encode_char_as_list] = ACTIONS(1713),
    [sym_xml_encode_le] = ACTIONS(1713),
    [sym_xml_decode_types] = ACTIONS(1713),
    [sym_xml_decode_no_preserve] = ACTIONS(1713),
    [sym_duet_dev_type_display_device] = ACTIONS(1713),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1713),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1713),
    [sym_duet_dev_type_io_device] = ACTIONS(1713),
    [sym_duet_dev_type_relay_device] = ACTIONS(1713),
    [sym_duet_dev_type_ups] = ACTIONS(1713),
    [sym_duet_dev_type_amplifier] = ACTIONS(1713),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1713),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1713),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1713),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1713),
    [sym_duet_dev_type_camera] = ACTIONS(1713),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1713),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1713),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1713),
    [sym_duet_dev_type_dss] = ACTIONS(1713),
    [sym_duet_dev_type_dvr] = ACTIONS(1713),
    [sym_duet_dev_type_disc_device] = ACTIONS(1713),
    [sym_duet_dev_type_document_camera] = ACTIONS(1713),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1713),
    [sym_duet_dev_type_hvac] = ACTIONS(1713),
    [sym_duet_dev_type_keypad] = ACTIONS(1713),
    [sym_duet_dev_type_light] = ACTIONS(1713),
    [sym_duet_dev_type_monitor] = ACTIONS(1713),
    [sym_duet_dev_type_motor] = ACTIONS(1713),
    [sym_duet_dev_type_multi_window] = ACTIONS(1713),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1713),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1713),
    [sym_duet_dev_type_receiver] = ACTIONS(1713),
    [sym_duet_dev_type_security_system] = ACTIONS(1713),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1713),
    [sym_duet_dev_type_settop_box] = ACTIONS(1713),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1713),
    [sym_duet_dev_type_switcher] = ACTIONS(1713),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1713),
    [sym_duet_dev_type_tv] = ACTIONS(1713),
    [sym_duet_dev_type_utility] = ACTIONS(1713),
    [sym_duet_dev_type_vcr] = ACTIONS(1713),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1713),
    [sym_duet_dev_type_video_processor] = ACTIONS(1713),
    [sym_duet_dev_type_video_projector] = ACTIONS(1713),
    [sym_duet_dev_type_video_wall] = ACTIONS(1713),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1713),
    [sym_duet_dev_type_weather] = ACTIONS(1713),
    [sym_duet_dev_not_polled] = ACTIONS(1713),
    [sym_duet_dev_polled] = ACTIONS(1713),
    [sym_clkmgr_mode_network] = ACTIONS(1713),
    [sym_clkmgr_mode_standalone] = ACTIONS(1713),
    [sym_normal_standby] = ACTIONS(1713),
    [sym_normal_wake] = ACTIONS(1713),
    [sym_ip_multicast_ttl_option] = ACTIONS(1713),
    [sym_ip_ttl_subnet] = ACTIONS(1713),
    [sym_ip_ttl_site] = ACTIONS(1713),
    [sym_ip_ttl_region] = ACTIONS(1713),
    [sym_ip_ttl_continent] = ACTIONS(1713),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1713),
    [sym_ip_nodelay_on] = ACTIONS(1713),
    [sym_ip_nodelay_off] = ACTIONS(1713),
    [sym_amx_error] = ACTIONS(1713),
    [sym_amx_warning] = ACTIONS(1713),
    [sym_amx_info] = ACTIONS(1713),
    [sym_amx_debug] = ACTIONS(1713),
    [sym_netlinx_login_success] = ACTIONS(1713),
    [sym_netlinx_login_fail] = ACTIONS(1713),
    [sym_netlinx_logout] = ACTIONS(1713),
    [sym_null_str] = ACTIONS(1713),
    [sym_smtp_address] = ACTIONS(1713),
    [sym_smtp_port_number] = ACTIONS(1713),
    [sym_smtp_username] = ACTIONS(1713),
    [sym_smtp_password] = ACTIONS(1713),
    [sym_smtp_require_tls] = ACTIONS(1713),
    [sym_smtp_from] = ACTIONS(1713),
    [sym_smtp_tls_true] = ACTIONS(1713),
    [sym_smtp_tls_false] = ACTIONS(1713),
    [sym_listview_on_row_select_event] = ACTIONS(1713),
    [sym_data_structure_unknown] = ACTIONS(1713),
    [sym_data_structure_datafeed] = ACTIONS(1713),
    [sym_data_structure_datarecordset] = ACTIONS(1713),
    [sym_data_structure_datarecord] = ACTIONS(1713),
    [sym_data_structure_datafield] = ACTIONS(1713),
    [sym_data_max_content_field_count] = ACTIONS(1713),
    [sym_data_max_metadata_field_count] = ACTIONS(1713),
    [sym_data_max_id_length] = ACTIONS(1713),
    [sym_data_max_name_length] = ACTIONS(1713),
    [sym_data_max_description_length] = ACTIONS(1713),
    [sym_data_max_type_length] = ACTIONS(1713),
    [sym_data_max_format_length] = ACTIONS(1713),
    [sym_data_max_source_length] = ACTIONS(1713),
    [sym_data_max_value_length] = ACTIONS(1713),
    [sym_data_max_label_length] = ACTIONS(1713),
    [sym_data_max_url_length] = ACTIONS(1713),
    [sym_max_last_login_info_length] = ACTIONS(1713),
    [sym_data_type_unknown] = ACTIONS(1713),
    [sym_data_type_string] = ACTIONS(1713),
    [sym_data_type_datetime] = ACTIONS(1713),
    [sym_data_type_date] = ACTIONS(1713),
    [sym_data_type_time] = ACTIONS(1713),
    [sym_data_type_image] = ACTIONS(1713),
    [sym_data_format_url] = ACTIONS(1713),
    [sym_data_format_phone] = ACTIONS(1713),
    [sym_data_format_email] = ACTIONS(1713),
    [sym_data_format_iso8601] = ACTIONS(1713),
    [sym_auditlog_priv] = ACTIONS(1713),
    [sym_configuration_priv] = ACTIONS(1713),
    [sym_deviceconfig_priv] = ACTIONS(1713),
    [sym_ftp_priv] = ACTIONS(1713),
    [sym_http_priv] = ACTIONS(1713),
    [sym_networkconfig_priv] = ACTIONS(1713),
    [sym_programport_priv] = ACTIONS(1713),
    [sym_remoteui_priv] = ACTIONS(1713),
    [sym_securitycontrol_priv] = ACTIONS(1713),
    [sym_softwaremanagement_priv] = ACTIONS(1713),
    [sym_terminal_priv] = ACTIONS(1713),
    [sym_tpadmin_priv] = ACTIONS(1713),
    [sym_user1_priv] = ACTIONS(1713),
    [sym_user2_priv] = ACTIONS(1713),
    [sym_user3_priv] = ACTIONS(1713),
    [sym_user4_priv] = ACTIONS(1713),
    [sym_usermanagement_priv] = ACTIONS(1713),
    [sym_valid_account] = ACTIONS(1713),
    [sym_err_name_invalid] = ACTIONS(1713),
    [sym_err_password_invalid] = ACTIONS(1713),
    [sym_err_invalid_account] = ACTIONS(1713),
    [sym_err_no_authorization] = ACTIONS(1713),
    [sym_err_invalid_parameter] = ACTIONS(1713),
    [sym_err_user_locked_out] = ACTIONS(1713),
    [sym_err_user_account_expired] = ACTIONS(1713),
    [sym_err_general_error] = ACTIONS(1713),
    [sym_err_authentication_error] = ACTIONS(1713),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1713),
    [sym_tls_validate_certificate] = ACTIONS(1713),
    [sym_tdata] = ACTIONS(1713),
    [sym_tchannel] = ACTIONS(1713),
    [sym_tlevel] = ACTIONS(1713),
    [sym_tbutton] = ACTIONS(1713),
    [sym_ttimeline] = ACTIONS(1713),
    [sym_tcustom] = ACTIONS(1713),
    [sym_url_struct] = ACTIONS(1713),
    [sym_dns_struct] = ACTIONS(1713),
    [sym_ip_address_struct] = ACTIONS(1713),
    [sym_dev_info_struct] = ACTIONS(1713),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1713),
    [sym_data_feed] = ACTIONS(1713),
    [sym_data_field] = ACTIONS(1713),
    [sym_data_record] = ACTIONS(1713),
    [sym_wc_data_feed] = ACTIONS(1713),
    [sym_wc_data_field] = ACTIONS(1713),
    [sym_wc_data_record] = ACTIONS(1713),
    [sym_last_login_info] = ACTIONS(1713),
    [sym____reserved___] = ACTIONS(1713),
    [sym_date] = ACTIONS(1713),
    [sym_day] = ACTIONS(1713),
    [sym_time] = ACTIONS(1713),
    [sym_ldate] = ACTIONS(1713),
    [sym_push_device] = ACTIONS(1713),
    [sym_push_channel] = ACTIONS(1713),
    [sym_push_devchan] = ACTIONS(1713),
    [sym_release_device] = ACTIONS(1713),
    [sym_release_channel] = ACTIONS(1713),
    [sym_release_devchan] = ACTIONS(1713),
    [sym_master_slot] = ACTIONS(1713),
    [sym_get_pulse_time] = ACTIONS(1713),
    [sym_get_timer] = ACTIONS(1713),
    [sym_master_sn] = ACTIONS(1713),
    [sym_system_number] = ACTIONS(1713),
    [sym_button] = ACTIONS(1713),
    [sym_dv_channel] = ACTIONS(1713),
    [sym_data] = ACTIONS(1713),
    [sym_level] = ACTIONS(1713),
    [sym_timeline] = ACTIONS(1713),
    [sym_channel] = ACTIONS(1713),
    [sym_custom] = ACTIONS(1713),
    [sym_atoi] = ACTIONS(1713),
    [sym_atol] = ACTIONS(1713),
    [sym_atof] = ACTIONS(1713),
    [sym_itoa] = ACTIONS(1713),
    [sym_itohex] = ACTIONS(1713),
    [sym_hextoi] = ACTIONS(1713),
    [sym_ftoa] = ACTIONS(1713),
    [sym_find_string] = ACTIONS(1713),
    [sym_left_string] = ACTIONS(1713),
    [sym_right_string] = ACTIONS(1713),
    [sym_mid_string] = ACTIONS(1713),
    [sym_remove_string] = ACTIONS(1713),
    [sym_get_buffer_string] = ACTIONS(1713),
    [sym_get_buffer_char] = ACTIONS(1713),
    [sym_get_multi_buffer_string] = ACTIONS(1713),
    [sym_length_string] = ACTIONS(1713),
    [sym_max_length_string] = ACTIONS(1713),
    [sym_set_length_string] = ACTIONS(1713),
    [sym_compare_string] = ACTIONS(1713),
    [sym_upper_string] = ACTIONS(1713),
    [sym_lower_string] = ACTIONS(1713),
    [sym_get_last] = ACTIONS(1713),
    [sym_date_to_day] = ACTIONS(1713),
    [sym_date_to_month] = ACTIONS(1713),
    [sym_date_to_year] = ACTIONS(1713),
    [sym_time_to_hour] = ACTIONS(1713),
    [sym_time_to_minute] = ACTIONS(1713),
    [sym_time_to_second] = ACTIONS(1713),
    [sym_day_of_week] = ACTIONS(1713),
    [sym_set_pulse_time] = ACTIONS(1713),
    [sym_set_timer] = ACTIONS(1713),
    [sym_file_open] = ACTIONS(1713),
    [sym_file_close] = ACTIONS(1713),
    [sym_file_read] = ACTIONS(1713),
    [sym_file_read_line] = ACTIONS(1713),
    [sym_file_write] = ACTIONS(1713),
    [sym_file_write_line] = ACTIONS(1713),
    [sym_file_copy] = ACTIONS(1713),
    [sym_file_delete] = ACTIONS(1713),
    [sym_file_rename] = ACTIONS(1713),
    [sym_file_dir] = ACTIONS(1713),
    [sym_file_getdir] = ACTIONS(1713),
    [sym_file_setdir] = ACTIONS(1713),
    [sym_file_create_dir] = ACTIONS(1713),
    [sym_file_remove_dir] = ACTIONS(1713),
    [sym_file_seek] = ACTIONS(1713),
    [sym_length_array] = ACTIONS(1713),
    [sym_max_length_array] = ACTIONS(1713),
    [sym_set_length_array] = ACTIONS(1713),
    [sym_ip_client_open] = ACTIONS(1713),
    [sym_ip_client_close] = ACTIONS(1713),
    [sym_ip_server_open] = ACTIONS(1713),
    [sym_ip_server_close] = ACTIONS(1713),
    [sym_ip_bound_client_open] = ACTIONS(1713),
    [sym_ip_mc_server_open] = ACTIONS(1713),
    [sym_ip_set_option] = ACTIONS(1713),
    [sym_ssh_client_open] = ACTIONS(1713),
    [sym_ssh_client_close] = ACTIONS(1713),
    [sym_tls_client_open] = ACTIONS(1713),
    [sym_tls_client_close] = ACTIONS(1713),
    [sym_do_push] = ACTIONS(1713),
    [sym_do_push_timed] = ACTIONS(1713),
    [sym_do_release] = ACTIONS(1713),
    [sym_device_id] = ACTIONS(1713),
    [sym_device_info] = ACTIONS(1713),
    [sym_device_id_string] = ACTIONS(1713),
    [sym_device_standby] = ACTIONS(1713),
    [sym_device_wake] = ACTIONS(1713),
    [sym_timeline_create] = ACTIONS(1713),
    [sym_timeline_kill] = ACTIONS(1713),
    [sym_timeline_reload] = ACTIONS(1713),
    [sym_timeline_pause] = ACTIONS(1713),
    [sym_timeline_restart] = ACTIONS(1713),
    [sym_timeline_set] = ACTIONS(1713),
    [sym_timeline_get] = ACTIONS(1713),
    [sym_timeline_active] = ACTIONS(1713),
    [sym_timeline_dynamic_id] = ACTIONS(1713),
    [sym_variable_to_string] = ACTIONS(1713),
    [sym_string_to_variable] = ACTIONS(1713),
    [sym_length_variable_to_string] = ACTIONS(1713),
    [sym_type_cast] = ACTIONS(1713),
    [sym_variable_to_xml] = ACTIONS(1713),
    [sym_xml_to_variable] = ACTIONS(1713),
    [sym_length_variable_to_xml] = ACTIONS(1713),
    [sym_redirect_string] = ACTIONS(1713),
    [sym_random_number] = ACTIONS(1713),
    [sym_get_unique_id] = ACTIONS(1713),
    [sym_get_system_number] = ACTIONS(1713),
    [sym_set_system_number] = ACTIONS(1713),
    [sym_get_serial_number] = ACTIONS(1713),
    [sym_get_master_build] = ACTIONS(1713),
    [sym_set_validation_code] = ACTIONS(1713),
    [sym_reboot] = ACTIONS(1713),
    [sym_load_duet_module] = ACTIONS(1713),
    [sym_unload_duet_module] = ACTIONS(1713),
    [sym_duet_mem_size_set] = ACTIONS(1713),
    [sym_duet_mem_size_get] = ACTIONS(1713),
    [sym_dynamic_application_device] = ACTIONS(1713),
    [sym_dynamic_polled_port] = ACTIONS(1713),
    [sym_static_port_binding] = ACTIONS(1713),
    [sym_static_ip_binding] = ACTIONS(1713),
    [sym_get_dns_list] = ACTIONS(1713),
    [sym_set_dns_list] = ACTIONS(1713),
    [sym_get_ip_address] = ACTIONS(1713),
    [sym_set_ip_address] = ACTIONS(1713),
    [sym_get_url_list] = ACTIONS(1713),
    [sym_add_url_entry] = ACTIONS(1713),
    [sym_delete_url_entry] = ACTIONS(1713),
    [sym_abs_value] = ACTIONS(1713),
    [sym_max_value] = ACTIONS(1713),
    [sym_min_value] = ACTIONS(1713),
    [sym_exp_value] = ACTIONS(1713),
    [sym_log_value] = ACTIONS(1713),
    [sym_log10_value] = ACTIONS(1713),
    [sym_power_value] = ACTIONS(1713),
    [sym_sqrt_value] = ACTIONS(1713),
    [sym_raw_be] = ACTIONS(1713),
    [sym_raw_le] = ACTIONS(1713),
    [sym_internal_queue_size_set] = ACTIONS(1713),
    [sym_internal_queue_size_get] = ACTIONS(1713),
    [sym_internal_threshold_set] = ACTIONS(1713),
    [sym_internal_threshold_get] = ACTIONS(1713),
    [sym_rebuilt_event] = ACTIONS(1713),
    [sym_do_custom_event] = ACTIONS(1713),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1713),
    [sym_clkmgr_set_clk_source] = ACTIONS(1713),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1713),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1713),
    [sym_clkmgr_get_timezone] = ACTIONS(1713),
    [sym_clkmgr_set_timezone] = ACTIONS(1713),
    [sym_clkmgr_get_resync_period] = ACTIONS(1713),
    [sym_clkmgr_set_resync_period] = ACTIONS(1713),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1713),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1713),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1713),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1713),
    [sym_clkmgr_get_timeservers] = ACTIONS(1713),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1713),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1713),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1713),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1713),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1713),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1713),
    [sym_set_log_level] = ACTIONS(1713),
    [sym_get_log_level] = ACTIONS(1713),
    [sym_amx_log] = ACTIONS(1713),
    [sym_smtp_server_config_set] = ACTIONS(1713),
    [sym_smtp_server_config_get] = ACTIONS(1713),
    [sym_smtp_send] = ACTIONS(1713),
    [sym_validate_netlinx_account] = ACTIONS(1713),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1713),
    [sym_authenticate_certificate] = ACTIONS(1713),
    [sym_audit_netlinx_session_event] = ACTIONS(1713),
    [sym_audit_netlinx_generic_event] = ACTIONS(1713),
    [sym_get_available_flash_disk_space] = ACTIONS(1713),
    [sym_get_max_flash_disk_space] = ACTIONS(1713),
    [sym_data_create_feed] = ACTIONS(1713),
    [sym_data_delete_feed] = ACTIONS(1713),
    [sym_data_publish_feed] = ACTIONS(1713),
    [sym_data_get_published_feed] = ACTIONS(1713),
    [sym_data_add_record] = ACTIONS(1713),
    [sym_data_get_event_record] = ACTIONS(1713),
    [sym__wc_data_create_feed] = ACTIONS(1713),
    [sym__wc_data_add_record] = ACTIONS(1713),
    [sym__wc_data_get_event_record] = ACTIONS(1713),
    [sym_format] = ACTIONS(1713),
    [sym_astro_clock] = ACTIONS(1713),
    [sym_set_outdoor_temperature] = ACTIONS(1713),
    [sym_set_virtual_level_count] = ACTIONS(1713),
    [sym_set_virtual_channel_count] = ACTIONS(1713),
    [sym_set_virtual_port_count] = ACTIONS(1713),
    [sym__wc_encode_int] = ACTIONS(1713),
    [sym__wc_decode_int] = ACTIONS(1713),
    [sym__wc_int] = ACTIONS(1713),
    [sym__wc_to_ch_int] = ACTIONS(1713),
    [sym__ch_to_wc_int] = ACTIONS(1713),
    [sym__wc_find_string_int] = ACTIONS(1713),
    [sym__wc_left_string_int] = ACTIONS(1713),
    [sym__wc_lower_string_int] = ACTIONS(1713),
    [sym__wc_mid_string_int] = ACTIONS(1713),
    [sym__wc_remove_string_int] = ACTIONS(1713),
    [sym__wc_right_string_int] = ACTIONS(1713),
    [sym__wc_upper_string_int] = ACTIONS(1713),
    [sym__wc_compare_string_int] = ACTIONS(1713),
    [sym__wc_get_buffer_char_int] = ACTIONS(1713),
    [sym__wc_get_buffer_string_int] = ACTIONS(1713),
    [sym__wc_concat_string_int] = ACTIONS(1713),
    [sym__wc_file_open_int] = ACTIONS(1713),
    [sym__wc_file_close_int] = ACTIONS(1713),
    [sym__wc_file_read_int] = ACTIONS(1713),
    [sym__wc_file_read_line_int] = ACTIONS(1713),
    [sym__wc_file_write_int] = ACTIONS(1713),
    [sym__wc_file_write_line_int] = ACTIONS(1713),
    [aux_sym_preproc_include_token1] = ACTIONS(1713),
    [aux_sym_preproc_define_token1] = ACTIONS(1711),
    [aux_sym_preproc_warn_token1] = ACTIONS(1711),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1711),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1711),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1711),
    [anon_sym_LBRACK] = ACTIONS(1711),
    [anon_sym_LBRACE] = ACTIONS(1711),
    [anon_sym_RBRACE] = ACTIONS(1711),
    [anon_sym_LPAREN] = ACTIONS(1713),
    [anon_sym_BANG] = ACTIONS(1711),
    [anon_sym_TILDE] = ACTIONS(1711),
    [anon_sym_DASH] = ACTIONS(1713),
    [anon_sym_PLUS] = ACTIONS(1713),
    [anon_sym_DASH_DASH] = ACTIONS(1711),
    [anon_sym_PLUS_PLUS] = ACTIONS(1711),
    [anon_sym_DQUOTE] = ACTIONS(1711),
    [anon_sym_SQUOTE] = ACTIONS(1711),
    [sym_number_literal] = ACTIONS(1711),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(554)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(311),
    [sym_statement] = STATE(392),
    [sym_expression_statement] = STATE(311),
    [sym_if_statement] = STATE(311),
    [sym_switch_statement] = STATE(311),
    [sym_case_statement] = STATE(311),
    [sym_while_statement] = STATE(311),
    [sym_for_statement] = STATE(311),
    [sym_select_statement] = STATE(311),
    [sym_active_statement] = STATE(311),
    [sym_return_statement] = STATE(311),
    [sym_break_statement] = STATE(311),
    [sym_continue_statement] = STATE(311),
    [sym_devchan_operation_statement] = STATE(311),
    [sym_devchan_operation] = STATE(1878),
    [sym_send_string_statement] = STATE(311),
    [sym_send_command_statement] = STATE(311),
    [sym_send_level_statement] = STATE(311),
    [sym_create_buffer_statement] = STATE(311),
    [sym_create_multi_buffer_statement] = STATE(311),
    [sym_clear_buffer_statement] = STATE(311),
    [sym_wait_statement] = STATE(311),
    [sym_wait_until_statement] = STATE(311),
    [sym_cancel_all_wait_statement] = STATE(311),
    [sym_cancel_all_wait_until_statement] = STATE(311),
    [sym_cancel_wait_statement] = STATE(311),
    [sym_cancel_wait_until_statement] = STATE(311),
    [sym_call_statement] = STATE(311),
    [sym_system_call_statement] = STATE(311),
    [sym_expression] = STATE(1137),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1990),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(55),
    [sym_switch_keyword] = ACTIONS(57),
    [sym_case_keyword] = ACTIONS(59),
    [sym_default_keyword] = ACTIONS(61),
    [sym_while_keyword] = ACTIONS(63),
    [sym_for_keyword] = ACTIONS(65),
    [sym_break_keyword] = ACTIONS(67),
    [sym_continue_keyword] = ACTIONS(69),
    [sym_return_keyword] = ACTIONS(71),
    [sym_select_keyword] = ACTIONS(73),
    [sym_active_keyword] = ACTIONS(75),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(99),
    [sym_send_string_keyword] = ACTIONS(101),
    [sym_send_command_keyword] = ACTIONS(103),
    [sym_clear_buffer_keyword] = ACTIONS(105),
    [sym_create_buffer_keyword] = ACTIONS(107),
    [sym_create_multi_buffer_keyword] = ACTIONS(109),
    [sym_call_keyword] = ACTIONS(111),
    [sym_system_call_keyword] = ACTIONS(113),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(117),
    [sym_cancel_wait_keyword] = ACTIONS(119),
    [sym_wait_until_keyword] = ACTIONS(121),
    [sym_cancel_wait_until_keyword] = ACTIONS(123),
    [sym_cancel_all_wait_keyword] = ACTIONS(125),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(127),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(555)] = {
    [sym_identifier] = ACTIONS(2061),
    [sym_define_call_keyword] = ACTIONS(2061),
    [sym_define_function_keyword] = ACTIONS(2061),
    [sym_define_library_function_keyword] = ACTIONS(2061),
    [sym_define_module_keyword] = ACTIONS(2061),
    [sym_char_keyword] = ACTIONS(2061),
    [sym_widechar_keyword] = ACTIONS(2061),
    [sym_integer_keyword] = ACTIONS(2061),
    [sym_sinteger_keyword] = ACTIONS(2061),
    [sym_long_keyword] = ACTIONS(2061),
    [sym_slong_keyword] = ACTIONS(2061),
    [sym_float_keyword] = ACTIONS(2061),
    [sym_double_keyword] = ACTIONS(2061),
    [sym_variant_keyword] = ACTIONS(2061),
    [sym_variantarray_keyword] = ACTIONS(2061),
    [sym_local_var_keyword] = ACTIONS(2061),
    [sym_stack_var_keyword] = ACTIONS(2061),
    [sym_constant_keyword] = ACTIONS(2061),
    [sym_volatile_keyword] = ACTIONS(2061),
    [sym_non_volatile_keyword] = ACTIONS(2061),
    [sym_persistent_keyword] = ACTIONS(2061),
    [sym_dev_keyword] = ACTIONS(2061),
    [sym_devlev_keyword] = ACTIONS(2061),
    [sym_devchan_keyword] = ACTIONS(2061),
    [sym_if_keyword] = ACTIONS(2061),
    [sym_switch_keyword] = ACTIONS(2061),
    [sym_case_keyword] = ACTIONS(2061),
    [sym_default_keyword] = ACTIONS(2061),
    [sym_while_keyword] = ACTIONS(2061),
    [sym_for_keyword] = ACTIONS(2061),
    [sym_break_keyword] = ACTIONS(2061),
    [sym_continue_keyword] = ACTIONS(2061),
    [sym_return_keyword] = ACTIONS(2061),
    [sym_select_keyword] = ACTIONS(2061),
    [sym_active_keyword] = ACTIONS(2061),
    [sym_struct_keyword] = ACTIONS(2061),
    [sym_structure_keyword] = ACTIONS(2061),
    [aux_sym_on_keyword_token1] = ACTIONS(2061),
    [aux_sym_off_keyword_token1] = ACTIONS(2061),
    [sym_send_level_keyword] = ACTIONS(2061),
    [sym_send_string_keyword] = ACTIONS(2061),
    [sym_send_command_keyword] = ACTIONS(2061),
    [sym_clear_buffer_keyword] = ACTIONS(2061),
    [sym_create_buffer_keyword] = ACTIONS(2061),
    [sym_create_multi_buffer_keyword] = ACTIONS(2061),
    [sym_call_keyword] = ACTIONS(2061),
    [sym_system_call_keyword] = ACTIONS(2061),
    [sym_devchan_to_keyword] = ACTIONS(2061),
    [sym_devchan_min_to_keyword] = ACTIONS(2061),
    [sym_devchan_total_off_keyword] = ACTIONS(2061),
    [sym_devchan_pulse_keyword] = ACTIONS(2061),
    [sym_wait_keyword] = ACTIONS(2061),
    [sym_cancel_wait_keyword] = ACTIONS(2061),
    [sym_wait_until_keyword] = ACTIONS(2061),
    [sym_cancel_wait_until_keyword] = ACTIONS(2061),
    [sym_cancel_all_wait_keyword] = ACTIONS(2061),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2061),
    [sym_not] = ACTIONS(2061),
    [sym_bnot] = ACTIONS(2061),
    [sym___file__] = ACTIONS(2061),
    [sym___date__] = ACTIONS(2061),
    [sym___ldate__] = ACTIONS(2061),
    [sym___time__] = ACTIONS(2061),
    [sym___line__] = ACTIONS(2061),
    [sym___name__] = ACTIONS(2061),
    [sym___netlinx__] = ACTIONS(2061),
    [sym_netlinx_axi_version] = ACTIONS(2061),
    [sym_url_flg_tcp] = ACTIONS(2061),
    [sym_url_flg_accinfopresent] = ACTIONS(2061),
    [sym_url_flg_temp] = ACTIONS(2061),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2061),
    [sym_url_flg_stat_mask] = ACTIONS(2061),
    [sym_url_flg_stat_lookup] = ACTIONS(2061),
    [sym_url_flg_stat_connecting] = ACTIONS(2061),
    [sym_url_flg_stat_waiting] = ACTIONS(2061),
    [sym_url_flg_stat_connected] = ACTIONS(2061),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2061),
    [sym_first_virtual_device] = ACTIONS(2061),
    [sym_first_local_port] = ACTIONS(2061),
    [sym_dynamic_virtual_device] = ACTIONS(2061),
    [sym_timeline_once] = ACTIONS(2061),
    [sym_timeline_repeat] = ACTIONS(2061),
    [sym_timeline_absolute] = ACTIONS(2061),
    [sym_timeline_relative] = ACTIONS(2061),
    [sym_do_push_timed_infinite] = ACTIONS(2061),
    [sym_source_type_no_address] = ACTIONS(2061),
    [sym_source_type_neuron_id] = ACTIONS(2061),
    [sym_source_type_ip_address] = ACTIONS(2061),
    [sym_source_type_axlink] = ACTIONS(2061),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2061),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2061),
    [sym_source_type_ip_socket_address] = ACTIONS(2061),
    [sym_source_type_rs232] = ACTIONS(2061),
    [sym_source_type_internal] = ACTIONS(2061),
    [sym_source_type_mac_address] = ACTIONS(2061),
    [sym_source_type_ipv4_port] = ACTIONS(2061),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2061),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2061),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2061),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2061),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2061),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2061),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2061),
    [sym_true] = ACTIONS(2061),
    [sym_false] = ACTIONS(2061),
    [sym_file_read_only] = ACTIONS(2061),
    [sym_file_rw_new] = ACTIONS(2061),
    [sym_file_rw_append] = ACTIONS(2061),
    [sym_ip_tcp] = ACTIONS(2061),
    [sym_ip_udp] = ACTIONS(2061),
    [sym_ip_udp_2way] = ACTIONS(2061),
    [sym_xml_encode_types] = ACTIONS(2061),
    [sym_xml_encode_char_as_list] = ACTIONS(2061),
    [sym_xml_encode_le] = ACTIONS(2061),
    [sym_xml_decode_types] = ACTIONS(2061),
    [sym_xml_decode_no_preserve] = ACTIONS(2061),
    [sym_duet_dev_type_display_device] = ACTIONS(2061),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2061),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2061),
    [sym_duet_dev_type_io_device] = ACTIONS(2061),
    [sym_duet_dev_type_relay_device] = ACTIONS(2061),
    [sym_duet_dev_type_ups] = ACTIONS(2061),
    [sym_duet_dev_type_amplifier] = ACTIONS(2061),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2061),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2061),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2061),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2061),
    [sym_duet_dev_type_camera] = ACTIONS(2061),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2061),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2061),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2061),
    [sym_duet_dev_type_dss] = ACTIONS(2061),
    [sym_duet_dev_type_dvr] = ACTIONS(2061),
    [sym_duet_dev_type_disc_device] = ACTIONS(2061),
    [sym_duet_dev_type_document_camera] = ACTIONS(2061),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2061),
    [sym_duet_dev_type_hvac] = ACTIONS(2061),
    [sym_duet_dev_type_keypad] = ACTIONS(2061),
    [sym_duet_dev_type_light] = ACTIONS(2061),
    [sym_duet_dev_type_monitor] = ACTIONS(2061),
    [sym_duet_dev_type_motor] = ACTIONS(2061),
    [sym_duet_dev_type_multi_window] = ACTIONS(2061),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2061),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2061),
    [sym_duet_dev_type_receiver] = ACTIONS(2061),
    [sym_duet_dev_type_security_system] = ACTIONS(2061),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2061),
    [sym_duet_dev_type_settop_box] = ACTIONS(2061),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2061),
    [sym_duet_dev_type_switcher] = ACTIONS(2061),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2061),
    [sym_duet_dev_type_tv] = ACTIONS(2061),
    [sym_duet_dev_type_utility] = ACTIONS(2061),
    [sym_duet_dev_type_vcr] = ACTIONS(2061),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2061),
    [sym_duet_dev_type_video_processor] = ACTIONS(2061),
    [sym_duet_dev_type_video_projector] = ACTIONS(2061),
    [sym_duet_dev_type_video_wall] = ACTIONS(2061),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2061),
    [sym_duet_dev_type_weather] = ACTIONS(2061),
    [sym_duet_dev_not_polled] = ACTIONS(2061),
    [sym_duet_dev_polled] = ACTIONS(2061),
    [sym_clkmgr_mode_network] = ACTIONS(2061),
    [sym_clkmgr_mode_standalone] = ACTIONS(2061),
    [sym_normal_standby] = ACTIONS(2061),
    [sym_normal_wake] = ACTIONS(2061),
    [sym_ip_multicast_ttl_option] = ACTIONS(2061),
    [sym_ip_ttl_subnet] = ACTIONS(2061),
    [sym_ip_ttl_site] = ACTIONS(2061),
    [sym_ip_ttl_region] = ACTIONS(2061),
    [sym_ip_ttl_continent] = ACTIONS(2061),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2061),
    [sym_ip_nodelay_on] = ACTIONS(2061),
    [sym_ip_nodelay_off] = ACTIONS(2061),
    [sym_amx_error] = ACTIONS(2061),
    [sym_amx_warning] = ACTIONS(2061),
    [sym_amx_info] = ACTIONS(2061),
    [sym_amx_debug] = ACTIONS(2061),
    [sym_netlinx_login_success] = ACTIONS(2061),
    [sym_netlinx_login_fail] = ACTIONS(2061),
    [sym_netlinx_logout] = ACTIONS(2061),
    [sym_null_str] = ACTIONS(2061),
    [sym_smtp_address] = ACTIONS(2061),
    [sym_smtp_port_number] = ACTIONS(2061),
    [sym_smtp_username] = ACTIONS(2061),
    [sym_smtp_password] = ACTIONS(2061),
    [sym_smtp_require_tls] = ACTIONS(2061),
    [sym_smtp_from] = ACTIONS(2061),
    [sym_smtp_tls_true] = ACTIONS(2061),
    [sym_smtp_tls_false] = ACTIONS(2061),
    [sym_listview_on_row_select_event] = ACTIONS(2061),
    [sym_data_structure_unknown] = ACTIONS(2061),
    [sym_data_structure_datafeed] = ACTIONS(2061),
    [sym_data_structure_datarecordset] = ACTIONS(2061),
    [sym_data_structure_datarecord] = ACTIONS(2061),
    [sym_data_structure_datafield] = ACTIONS(2061),
    [sym_data_max_content_field_count] = ACTIONS(2061),
    [sym_data_max_metadata_field_count] = ACTIONS(2061),
    [sym_data_max_id_length] = ACTIONS(2061),
    [sym_data_max_name_length] = ACTIONS(2061),
    [sym_data_max_description_length] = ACTIONS(2061),
    [sym_data_max_type_length] = ACTIONS(2061),
    [sym_data_max_format_length] = ACTIONS(2061),
    [sym_data_max_source_length] = ACTIONS(2061),
    [sym_data_max_value_length] = ACTIONS(2061),
    [sym_data_max_label_length] = ACTIONS(2061),
    [sym_data_max_url_length] = ACTIONS(2061),
    [sym_max_last_login_info_length] = ACTIONS(2061),
    [sym_data_type_unknown] = ACTIONS(2061),
    [sym_data_type_string] = ACTIONS(2061),
    [sym_data_type_datetime] = ACTIONS(2061),
    [sym_data_type_date] = ACTIONS(2061),
    [sym_data_type_time] = ACTIONS(2061),
    [sym_data_type_image] = ACTIONS(2061),
    [sym_data_format_url] = ACTIONS(2061),
    [sym_data_format_phone] = ACTIONS(2061),
    [sym_data_format_email] = ACTIONS(2061),
    [sym_data_format_iso8601] = ACTIONS(2061),
    [sym_auditlog_priv] = ACTIONS(2061),
    [sym_configuration_priv] = ACTIONS(2061),
    [sym_deviceconfig_priv] = ACTIONS(2061),
    [sym_ftp_priv] = ACTIONS(2061),
    [sym_http_priv] = ACTIONS(2061),
    [sym_networkconfig_priv] = ACTIONS(2061),
    [sym_programport_priv] = ACTIONS(2061),
    [sym_remoteui_priv] = ACTIONS(2061),
    [sym_securitycontrol_priv] = ACTIONS(2061),
    [sym_softwaremanagement_priv] = ACTIONS(2061),
    [sym_terminal_priv] = ACTIONS(2061),
    [sym_tpadmin_priv] = ACTIONS(2061),
    [sym_user1_priv] = ACTIONS(2061),
    [sym_user2_priv] = ACTIONS(2061),
    [sym_user3_priv] = ACTIONS(2061),
    [sym_user4_priv] = ACTIONS(2061),
    [sym_usermanagement_priv] = ACTIONS(2061),
    [sym_valid_account] = ACTIONS(2061),
    [sym_err_name_invalid] = ACTIONS(2061),
    [sym_err_password_invalid] = ACTIONS(2061),
    [sym_err_invalid_account] = ACTIONS(2061),
    [sym_err_no_authorization] = ACTIONS(2061),
    [sym_err_invalid_parameter] = ACTIONS(2061),
    [sym_err_user_locked_out] = ACTIONS(2061),
    [sym_err_user_account_expired] = ACTIONS(2061),
    [sym_err_general_error] = ACTIONS(2061),
    [sym_err_authentication_error] = ACTIONS(2061),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2061),
    [sym_tls_validate_certificate] = ACTIONS(2061),
    [sym_tdata] = ACTIONS(2061),
    [sym_tchannel] = ACTIONS(2061),
    [sym_tlevel] = ACTIONS(2061),
    [sym_tbutton] = ACTIONS(2061),
    [sym_ttimeline] = ACTIONS(2061),
    [sym_tcustom] = ACTIONS(2061),
    [sym_url_struct] = ACTIONS(2061),
    [sym_dns_struct] = ACTIONS(2061),
    [sym_ip_address_struct] = ACTIONS(2061),
    [sym_dev_info_struct] = ACTIONS(2061),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2061),
    [sym_data_feed] = ACTIONS(2061),
    [sym_data_field] = ACTIONS(2061),
    [sym_data_record] = ACTIONS(2061),
    [sym_wc_data_feed] = ACTIONS(2061),
    [sym_wc_data_field] = ACTIONS(2061),
    [sym_wc_data_record] = ACTIONS(2061),
    [sym_last_login_info] = ACTIONS(2061),
    [sym____reserved___] = ACTIONS(2061),
    [sym_date] = ACTIONS(2061),
    [sym_day] = ACTIONS(2061),
    [sym_time] = ACTIONS(2061),
    [sym_ldate] = ACTIONS(2061),
    [sym_push_device] = ACTIONS(2061),
    [sym_push_channel] = ACTIONS(2061),
    [sym_push_devchan] = ACTIONS(2061),
    [sym_release_device] = ACTIONS(2061),
    [sym_release_channel] = ACTIONS(2061),
    [sym_release_devchan] = ACTIONS(2061),
    [sym_master_slot] = ACTIONS(2061),
    [sym_get_pulse_time] = ACTIONS(2061),
    [sym_get_timer] = ACTIONS(2061),
    [sym_master_sn] = ACTIONS(2061),
    [sym_system_number] = ACTIONS(2061),
    [sym_button] = ACTIONS(2061),
    [sym_dv_channel] = ACTIONS(2061),
    [sym_data] = ACTIONS(2061),
    [sym_level] = ACTIONS(2061),
    [sym_timeline] = ACTIONS(2061),
    [sym_channel] = ACTIONS(2061),
    [sym_custom] = ACTIONS(2061),
    [sym_atoi] = ACTIONS(2061),
    [sym_atol] = ACTIONS(2061),
    [sym_atof] = ACTIONS(2061),
    [sym_itoa] = ACTIONS(2061),
    [sym_itohex] = ACTIONS(2061),
    [sym_hextoi] = ACTIONS(2061),
    [sym_ftoa] = ACTIONS(2061),
    [sym_find_string] = ACTIONS(2061),
    [sym_left_string] = ACTIONS(2061),
    [sym_right_string] = ACTIONS(2061),
    [sym_mid_string] = ACTIONS(2061),
    [sym_remove_string] = ACTIONS(2061),
    [sym_get_buffer_string] = ACTIONS(2061),
    [sym_get_buffer_char] = ACTIONS(2061),
    [sym_get_multi_buffer_string] = ACTIONS(2061),
    [sym_length_string] = ACTIONS(2061),
    [sym_max_length_string] = ACTIONS(2061),
    [sym_set_length_string] = ACTIONS(2061),
    [sym_compare_string] = ACTIONS(2061),
    [sym_upper_string] = ACTIONS(2061),
    [sym_lower_string] = ACTIONS(2061),
    [sym_get_last] = ACTIONS(2061),
    [sym_date_to_day] = ACTIONS(2061),
    [sym_date_to_month] = ACTIONS(2061),
    [sym_date_to_year] = ACTIONS(2061),
    [sym_time_to_hour] = ACTIONS(2061),
    [sym_time_to_minute] = ACTIONS(2061),
    [sym_time_to_second] = ACTIONS(2061),
    [sym_day_of_week] = ACTIONS(2061),
    [sym_set_pulse_time] = ACTIONS(2061),
    [sym_set_timer] = ACTIONS(2061),
    [sym_file_open] = ACTIONS(2061),
    [sym_file_close] = ACTIONS(2061),
    [sym_file_read] = ACTIONS(2061),
    [sym_file_read_line] = ACTIONS(2061),
    [sym_file_write] = ACTIONS(2061),
    [sym_file_write_line] = ACTIONS(2061),
    [sym_file_copy] = ACTIONS(2061),
    [sym_file_delete] = ACTIONS(2061),
    [sym_file_rename] = ACTIONS(2061),
    [sym_file_dir] = ACTIONS(2061),
    [sym_file_getdir] = ACTIONS(2061),
    [sym_file_setdir] = ACTIONS(2061),
    [sym_file_create_dir] = ACTIONS(2061),
    [sym_file_remove_dir] = ACTIONS(2061),
    [sym_file_seek] = ACTIONS(2061),
    [sym_length_array] = ACTIONS(2061),
    [sym_max_length_array] = ACTIONS(2061),
    [sym_set_length_array] = ACTIONS(2061),
    [sym_ip_client_open] = ACTIONS(2061),
    [sym_ip_client_close] = ACTIONS(2061),
    [sym_ip_server_open] = ACTIONS(2061),
    [sym_ip_server_close] = ACTIONS(2061),
    [sym_ip_bound_client_open] = ACTIONS(2061),
    [sym_ip_mc_server_open] = ACTIONS(2061),
    [sym_ip_set_option] = ACTIONS(2061),
    [sym_ssh_client_open] = ACTIONS(2061),
    [sym_ssh_client_close] = ACTIONS(2061),
    [sym_tls_client_open] = ACTIONS(2061),
    [sym_tls_client_close] = ACTIONS(2061),
    [sym_do_push] = ACTIONS(2061),
    [sym_do_push_timed] = ACTIONS(2061),
    [sym_do_release] = ACTIONS(2061),
    [sym_device_id] = ACTIONS(2061),
    [sym_device_info] = ACTIONS(2061),
    [sym_device_id_string] = ACTIONS(2061),
    [sym_device_standby] = ACTIONS(2061),
    [sym_device_wake] = ACTIONS(2061),
    [sym_timeline_create] = ACTIONS(2061),
    [sym_timeline_kill] = ACTIONS(2061),
    [sym_timeline_reload] = ACTIONS(2061),
    [sym_timeline_pause] = ACTIONS(2061),
    [sym_timeline_restart] = ACTIONS(2061),
    [sym_timeline_set] = ACTIONS(2061),
    [sym_timeline_get] = ACTIONS(2061),
    [sym_timeline_active] = ACTIONS(2061),
    [sym_timeline_dynamic_id] = ACTIONS(2061),
    [sym_variable_to_string] = ACTIONS(2061),
    [sym_string_to_variable] = ACTIONS(2061),
    [sym_length_variable_to_string] = ACTIONS(2061),
    [sym_type_cast] = ACTIONS(2061),
    [sym_variable_to_xml] = ACTIONS(2061),
    [sym_xml_to_variable] = ACTIONS(2061),
    [sym_length_variable_to_xml] = ACTIONS(2061),
    [sym_redirect_string] = ACTIONS(2061),
    [sym_random_number] = ACTIONS(2061),
    [sym_get_unique_id] = ACTIONS(2061),
    [sym_get_system_number] = ACTIONS(2061),
    [sym_set_system_number] = ACTIONS(2061),
    [sym_get_serial_number] = ACTIONS(2061),
    [sym_get_master_build] = ACTIONS(2061),
    [sym_set_validation_code] = ACTIONS(2061),
    [sym_reboot] = ACTIONS(2061),
    [sym_load_duet_module] = ACTIONS(2061),
    [sym_unload_duet_module] = ACTIONS(2061),
    [sym_duet_mem_size_set] = ACTIONS(2061),
    [sym_duet_mem_size_get] = ACTIONS(2061),
    [sym_dynamic_application_device] = ACTIONS(2061),
    [sym_dynamic_polled_port] = ACTIONS(2061),
    [sym_static_port_binding] = ACTIONS(2061),
    [sym_static_ip_binding] = ACTIONS(2061),
    [sym_get_dns_list] = ACTIONS(2061),
    [sym_set_dns_list] = ACTIONS(2061),
    [sym_get_ip_address] = ACTIONS(2061),
    [sym_set_ip_address] = ACTIONS(2061),
    [sym_get_url_list] = ACTIONS(2061),
    [sym_add_url_entry] = ACTIONS(2061),
    [sym_delete_url_entry] = ACTIONS(2061),
    [sym_abs_value] = ACTIONS(2061),
    [sym_max_value] = ACTIONS(2061),
    [sym_min_value] = ACTIONS(2061),
    [sym_exp_value] = ACTIONS(2061),
    [sym_log_value] = ACTIONS(2061),
    [sym_log10_value] = ACTIONS(2061),
    [sym_power_value] = ACTIONS(2061),
    [sym_sqrt_value] = ACTIONS(2061),
    [sym_raw_be] = ACTIONS(2061),
    [sym_raw_le] = ACTIONS(2061),
    [sym_internal_queue_size_set] = ACTIONS(2061),
    [sym_internal_queue_size_get] = ACTIONS(2061),
    [sym_internal_threshold_set] = ACTIONS(2061),
    [sym_internal_threshold_get] = ACTIONS(2061),
    [sym_rebuilt_event] = ACTIONS(2061),
    [sym_do_custom_event] = ACTIONS(2061),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2061),
    [sym_clkmgr_set_clk_source] = ACTIONS(2061),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2061),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2061),
    [sym_clkmgr_get_timezone] = ACTIONS(2061),
    [sym_clkmgr_set_timezone] = ACTIONS(2061),
    [sym_clkmgr_get_resync_period] = ACTIONS(2061),
    [sym_clkmgr_set_resync_period] = ACTIONS(2061),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2061),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2061),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2061),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2061),
    [sym_clkmgr_get_timeservers] = ACTIONS(2061),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2061),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2061),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2061),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2061),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2061),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2061),
    [sym_set_log_level] = ACTIONS(2061),
    [sym_get_log_level] = ACTIONS(2061),
    [sym_amx_log] = ACTIONS(2061),
    [sym_smtp_server_config_set] = ACTIONS(2061),
    [sym_smtp_server_config_get] = ACTIONS(2061),
    [sym_smtp_send] = ACTIONS(2061),
    [sym_validate_netlinx_account] = ACTIONS(2061),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2061),
    [sym_authenticate_certificate] = ACTIONS(2061),
    [sym_audit_netlinx_session_event] = ACTIONS(2061),
    [sym_audit_netlinx_generic_event] = ACTIONS(2061),
    [sym_get_available_flash_disk_space] = ACTIONS(2061),
    [sym_get_max_flash_disk_space] = ACTIONS(2061),
    [sym_data_create_feed] = ACTIONS(2061),
    [sym_data_delete_feed] = ACTIONS(2061),
    [sym_data_publish_feed] = ACTIONS(2061),
    [sym_data_get_published_feed] = ACTIONS(2061),
    [sym_data_add_record] = ACTIONS(2061),
    [sym_data_get_event_record] = ACTIONS(2061),
    [sym__wc_data_create_feed] = ACTIONS(2061),
    [sym__wc_data_add_record] = ACTIONS(2061),
    [sym__wc_data_get_event_record] = ACTIONS(2061),
    [sym_format] = ACTIONS(2061),
    [sym_astro_clock] = ACTIONS(2061),
    [sym_set_outdoor_temperature] = ACTIONS(2061),
    [sym_set_virtual_level_count] = ACTIONS(2061),
    [sym_set_virtual_channel_count] = ACTIONS(2061),
    [sym_set_virtual_port_count] = ACTIONS(2061),
    [sym__wc_encode_int] = ACTIONS(2061),
    [sym__wc_decode_int] = ACTIONS(2061),
    [sym__wc_int] = ACTIONS(2061),
    [sym__wc_to_ch_int] = ACTIONS(2061),
    [sym__ch_to_wc_int] = ACTIONS(2061),
    [sym__wc_find_string_int] = ACTIONS(2061),
    [sym__wc_left_string_int] = ACTIONS(2061),
    [sym__wc_lower_string_int] = ACTIONS(2061),
    [sym__wc_mid_string_int] = ACTIONS(2061),
    [sym__wc_remove_string_int] = ACTIONS(2061),
    [sym__wc_right_string_int] = ACTIONS(2061),
    [sym__wc_upper_string_int] = ACTIONS(2061),
    [sym__wc_compare_string_int] = ACTIONS(2061),
    [sym__wc_get_buffer_char_int] = ACTIONS(2061),
    [sym__wc_get_buffer_string_int] = ACTIONS(2061),
    [sym__wc_concat_string_int] = ACTIONS(2061),
    [sym__wc_file_open_int] = ACTIONS(2061),
    [sym__wc_file_close_int] = ACTIONS(2061),
    [sym__wc_file_read_int] = ACTIONS(2061),
    [sym__wc_file_read_line_int] = ACTIONS(2061),
    [sym__wc_file_write_int] = ACTIONS(2061),
    [sym__wc_file_write_line_int] = ACTIONS(2061),
    [aux_sym_preproc_include_token1] = ACTIONS(2061),
    [aux_sym_preproc_define_token1] = ACTIONS(2063),
    [aux_sym_preproc_warn_token1] = ACTIONS(2063),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2063),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2063),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2063),
    [anon_sym_LBRACK] = ACTIONS(2063),
    [anon_sym_LBRACE] = ACTIONS(2063),
    [anon_sym_RBRACE] = ACTIONS(2063),
    [anon_sym_LPAREN] = ACTIONS(2061),
    [anon_sym_BANG] = ACTIONS(2063),
    [anon_sym_TILDE] = ACTIONS(2063),
    [anon_sym_DASH] = ACTIONS(2061),
    [anon_sym_PLUS] = ACTIONS(2061),
    [anon_sym_DASH_DASH] = ACTIONS(2063),
    [anon_sym_PLUS_PLUS] = ACTIONS(2063),
    [anon_sym_DQUOTE] = ACTIONS(2063),
    [anon_sym_SQUOTE] = ACTIONS(2063),
    [sym_number_literal] = ACTIONS(2063),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(556)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(494),
    [sym_statement] = STATE(462),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(986),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(990),
    [sym_default_keyword] = ACTIONS(992),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(557)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(494),
    [sym_statement] = STATE(479),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(986),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(990),
    [sym_default_keyword] = ACTIONS(992),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(558)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(494),
    [sym_statement] = STATE(490),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(986),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(990),
    [sym_default_keyword] = ACTIONS(992),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(559)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(494),
    [sym_statement] = STATE(510),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(986),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(990),
    [sym_default_keyword] = ACTIONS(992),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(560)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(494),
    [sym_statement] = STATE(511),
    [sym_expression_statement] = STATE(494),
    [sym_if_statement] = STATE(494),
    [sym_switch_statement] = STATE(494),
    [sym_case_statement] = STATE(494),
    [sym_while_statement] = STATE(494),
    [sym_for_statement] = STATE(494),
    [sym_select_statement] = STATE(494),
    [sym_active_statement] = STATE(494),
    [sym_return_statement] = STATE(494),
    [sym_break_statement] = STATE(494),
    [sym_continue_statement] = STATE(494),
    [sym_devchan_operation_statement] = STATE(494),
    [sym_devchan_operation] = STATE(1822),
    [sym_send_string_statement] = STATE(494),
    [sym_send_command_statement] = STATE(494),
    [sym_send_level_statement] = STATE(494),
    [sym_create_buffer_statement] = STATE(494),
    [sym_create_multi_buffer_statement] = STATE(494),
    [sym_clear_buffer_statement] = STATE(494),
    [sym_wait_statement] = STATE(494),
    [sym_wait_until_statement] = STATE(494),
    [sym_cancel_all_wait_statement] = STATE(494),
    [sym_cancel_all_wait_until_statement] = STATE(494),
    [sym_cancel_wait_statement] = STATE(494),
    [sym_cancel_wait_until_statement] = STATE(494),
    [sym_call_statement] = STATE(494),
    [sym_system_call_statement] = STATE(494),
    [sym_expression] = STATE(1139),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1914),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(986),
    [sym_switch_keyword] = ACTIONS(988),
    [sym_case_keyword] = ACTIONS(990),
    [sym_default_keyword] = ACTIONS(992),
    [sym_while_keyword] = ACTIONS(994),
    [sym_for_keyword] = ACTIONS(996),
    [sym_break_keyword] = ACTIONS(998),
    [sym_continue_keyword] = ACTIONS(1000),
    [sym_return_keyword] = ACTIONS(1002),
    [sym_select_keyword] = ACTIONS(1004),
    [sym_active_keyword] = ACTIONS(1006),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(1008),
    [sym_send_string_keyword] = ACTIONS(1010),
    [sym_send_command_keyword] = ACTIONS(1012),
    [sym_clear_buffer_keyword] = ACTIONS(1014),
    [sym_create_buffer_keyword] = ACTIONS(1016),
    [sym_create_multi_buffer_keyword] = ACTIONS(1018),
    [sym_call_keyword] = ACTIONS(1020),
    [sym_system_call_keyword] = ACTIONS(1022),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(1024),
    [sym_cancel_wait_keyword] = ACTIONS(1026),
    [sym_wait_until_keyword] = ACTIONS(1028),
    [sym_cancel_wait_until_keyword] = ACTIONS(1030),
    [sym_cancel_all_wait_keyword] = ACTIONS(1032),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1034),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(561)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(252),
    [sym_statement] = STATE(201),
    [sym_expression_statement] = STATE(252),
    [sym_if_statement] = STATE(252),
    [sym_switch_statement] = STATE(252),
    [sym_case_statement] = STATE(252),
    [sym_while_statement] = STATE(252),
    [sym_for_statement] = STATE(252),
    [sym_select_statement] = STATE(252),
    [sym_active_statement] = STATE(252),
    [sym_return_statement] = STATE(252),
    [sym_break_statement] = STATE(252),
    [sym_continue_statement] = STATE(252),
    [sym_devchan_operation_statement] = STATE(252),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(252),
    [sym_send_command_statement] = STATE(252),
    [sym_send_level_statement] = STATE(252),
    [sym_create_buffer_statement] = STATE(252),
    [sym_create_multi_buffer_statement] = STATE(252),
    [sym_clear_buffer_statement] = STATE(252),
    [sym_wait_statement] = STATE(252),
    [sym_wait_until_statement] = STATE(252),
    [sym_cancel_all_wait_statement] = STATE(252),
    [sym_cancel_all_wait_until_statement] = STATE(252),
    [sym_cancel_wait_statement] = STATE(252),
    [sym_cancel_wait_until_statement] = STATE(252),
    [sym_call_statement] = STATE(252),
    [sym_system_call_statement] = STATE(252),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(562)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(252),
    [sym_statement] = STATE(255),
    [sym_expression_statement] = STATE(252),
    [sym_if_statement] = STATE(252),
    [sym_switch_statement] = STATE(252),
    [sym_case_statement] = STATE(252),
    [sym_while_statement] = STATE(252),
    [sym_for_statement] = STATE(252),
    [sym_select_statement] = STATE(252),
    [sym_active_statement] = STATE(252),
    [sym_return_statement] = STATE(252),
    [sym_break_statement] = STATE(252),
    [sym_continue_statement] = STATE(252),
    [sym_devchan_operation_statement] = STATE(252),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(252),
    [sym_send_command_statement] = STATE(252),
    [sym_send_level_statement] = STATE(252),
    [sym_create_buffer_statement] = STATE(252),
    [sym_create_multi_buffer_statement] = STATE(252),
    [sym_clear_buffer_statement] = STATE(252),
    [sym_wait_statement] = STATE(252),
    [sym_wait_until_statement] = STATE(252),
    [sym_cancel_all_wait_statement] = STATE(252),
    [sym_cancel_all_wait_until_statement] = STATE(252),
    [sym_cancel_wait_statement] = STATE(252),
    [sym_cancel_wait_until_statement] = STATE(252),
    [sym_call_statement] = STATE(252),
    [sym_system_call_statement] = STATE(252),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(563)] = {
    [sym_identifier] = ACTIONS(2013),
    [sym_define_call_keyword] = ACTIONS(2013),
    [sym_define_function_keyword] = ACTIONS(2013),
    [sym_define_library_function_keyword] = ACTIONS(2013),
    [sym_define_module_keyword] = ACTIONS(2013),
    [sym_char_keyword] = ACTIONS(2013),
    [sym_widechar_keyword] = ACTIONS(2013),
    [sym_integer_keyword] = ACTIONS(2013),
    [sym_sinteger_keyword] = ACTIONS(2013),
    [sym_long_keyword] = ACTIONS(2013),
    [sym_slong_keyword] = ACTIONS(2013),
    [sym_float_keyword] = ACTIONS(2013),
    [sym_double_keyword] = ACTIONS(2013),
    [sym_variant_keyword] = ACTIONS(2013),
    [sym_variantarray_keyword] = ACTIONS(2013),
    [sym_local_var_keyword] = ACTIONS(2013),
    [sym_stack_var_keyword] = ACTIONS(2013),
    [sym_constant_keyword] = ACTIONS(2013),
    [sym_volatile_keyword] = ACTIONS(2013),
    [sym_non_volatile_keyword] = ACTIONS(2013),
    [sym_persistent_keyword] = ACTIONS(2013),
    [sym_dev_keyword] = ACTIONS(2013),
    [sym_devlev_keyword] = ACTIONS(2013),
    [sym_devchan_keyword] = ACTIONS(2013),
    [sym_if_keyword] = ACTIONS(2013),
    [sym_switch_keyword] = ACTIONS(2013),
    [sym_case_keyword] = ACTIONS(2013),
    [sym_default_keyword] = ACTIONS(2013),
    [sym_while_keyword] = ACTIONS(2013),
    [sym_for_keyword] = ACTIONS(2013),
    [sym_break_keyword] = ACTIONS(2013),
    [sym_continue_keyword] = ACTIONS(2013),
    [sym_return_keyword] = ACTIONS(2013),
    [sym_select_keyword] = ACTIONS(2013),
    [sym_active_keyword] = ACTIONS(2013),
    [sym_struct_keyword] = ACTIONS(2013),
    [sym_structure_keyword] = ACTIONS(2013),
    [aux_sym_on_keyword_token1] = ACTIONS(2013),
    [aux_sym_off_keyword_token1] = ACTIONS(2013),
    [sym_send_level_keyword] = ACTIONS(2013),
    [sym_send_string_keyword] = ACTIONS(2013),
    [sym_send_command_keyword] = ACTIONS(2013),
    [sym_clear_buffer_keyword] = ACTIONS(2013),
    [sym_create_buffer_keyword] = ACTIONS(2013),
    [sym_create_multi_buffer_keyword] = ACTIONS(2013),
    [sym_call_keyword] = ACTIONS(2013),
    [sym_system_call_keyword] = ACTIONS(2013),
    [sym_devchan_to_keyword] = ACTIONS(2013),
    [sym_devchan_min_to_keyword] = ACTIONS(2013),
    [sym_devchan_total_off_keyword] = ACTIONS(2013),
    [sym_devchan_pulse_keyword] = ACTIONS(2013),
    [sym_wait_keyword] = ACTIONS(2013),
    [sym_cancel_wait_keyword] = ACTIONS(2013),
    [sym_wait_until_keyword] = ACTIONS(2013),
    [sym_cancel_wait_until_keyword] = ACTIONS(2013),
    [sym_cancel_all_wait_keyword] = ACTIONS(2013),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2013),
    [sym_not] = ACTIONS(2013),
    [sym_bnot] = ACTIONS(2013),
    [sym___file__] = ACTIONS(2013),
    [sym___date__] = ACTIONS(2013),
    [sym___ldate__] = ACTIONS(2013),
    [sym___time__] = ACTIONS(2013),
    [sym___line__] = ACTIONS(2013),
    [sym___name__] = ACTIONS(2013),
    [sym___netlinx__] = ACTIONS(2013),
    [sym_netlinx_axi_version] = ACTIONS(2013),
    [sym_url_flg_tcp] = ACTIONS(2013),
    [sym_url_flg_accinfopresent] = ACTIONS(2013),
    [sym_url_flg_temp] = ACTIONS(2013),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2013),
    [sym_url_flg_stat_mask] = ACTIONS(2013),
    [sym_url_flg_stat_lookup] = ACTIONS(2013),
    [sym_url_flg_stat_connecting] = ACTIONS(2013),
    [sym_url_flg_stat_waiting] = ACTIONS(2013),
    [sym_url_flg_stat_connected] = ACTIONS(2013),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2013),
    [sym_first_virtual_device] = ACTIONS(2013),
    [sym_first_local_port] = ACTIONS(2013),
    [sym_dynamic_virtual_device] = ACTIONS(2013),
    [sym_timeline_once] = ACTIONS(2013),
    [sym_timeline_repeat] = ACTIONS(2013),
    [sym_timeline_absolute] = ACTIONS(2013),
    [sym_timeline_relative] = ACTIONS(2013),
    [sym_do_push_timed_infinite] = ACTIONS(2013),
    [sym_source_type_no_address] = ACTIONS(2013),
    [sym_source_type_neuron_id] = ACTIONS(2013),
    [sym_source_type_ip_address] = ACTIONS(2013),
    [sym_source_type_axlink] = ACTIONS(2013),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2013),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2013),
    [sym_source_type_ip_socket_address] = ACTIONS(2013),
    [sym_source_type_rs232] = ACTIONS(2013),
    [sym_source_type_internal] = ACTIONS(2013),
    [sym_source_type_mac_address] = ACTIONS(2013),
    [sym_source_type_ipv4_port] = ACTIONS(2013),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2013),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2013),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2013),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2013),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2013),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2013),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2013),
    [sym_true] = ACTIONS(2013),
    [sym_false] = ACTIONS(2013),
    [sym_file_read_only] = ACTIONS(2013),
    [sym_file_rw_new] = ACTIONS(2013),
    [sym_file_rw_append] = ACTIONS(2013),
    [sym_ip_tcp] = ACTIONS(2013),
    [sym_ip_udp] = ACTIONS(2013),
    [sym_ip_udp_2way] = ACTIONS(2013),
    [sym_xml_encode_types] = ACTIONS(2013),
    [sym_xml_encode_char_as_list] = ACTIONS(2013),
    [sym_xml_encode_le] = ACTIONS(2013),
    [sym_xml_decode_types] = ACTIONS(2013),
    [sym_xml_decode_no_preserve] = ACTIONS(2013),
    [sym_duet_dev_type_display_device] = ACTIONS(2013),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2013),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2013),
    [sym_duet_dev_type_io_device] = ACTIONS(2013),
    [sym_duet_dev_type_relay_device] = ACTIONS(2013),
    [sym_duet_dev_type_ups] = ACTIONS(2013),
    [sym_duet_dev_type_amplifier] = ACTIONS(2013),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2013),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2013),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2013),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2013),
    [sym_duet_dev_type_camera] = ACTIONS(2013),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2013),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2013),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2013),
    [sym_duet_dev_type_dss] = ACTIONS(2013),
    [sym_duet_dev_type_dvr] = ACTIONS(2013),
    [sym_duet_dev_type_disc_device] = ACTIONS(2013),
    [sym_duet_dev_type_document_camera] = ACTIONS(2013),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2013),
    [sym_duet_dev_type_hvac] = ACTIONS(2013),
    [sym_duet_dev_type_keypad] = ACTIONS(2013),
    [sym_duet_dev_type_light] = ACTIONS(2013),
    [sym_duet_dev_type_monitor] = ACTIONS(2013),
    [sym_duet_dev_type_motor] = ACTIONS(2013),
    [sym_duet_dev_type_multi_window] = ACTIONS(2013),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2013),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2013),
    [sym_duet_dev_type_receiver] = ACTIONS(2013),
    [sym_duet_dev_type_security_system] = ACTIONS(2013),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2013),
    [sym_duet_dev_type_settop_box] = ACTIONS(2013),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2013),
    [sym_duet_dev_type_switcher] = ACTIONS(2013),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2013),
    [sym_duet_dev_type_tv] = ACTIONS(2013),
    [sym_duet_dev_type_utility] = ACTIONS(2013),
    [sym_duet_dev_type_vcr] = ACTIONS(2013),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2013),
    [sym_duet_dev_type_video_processor] = ACTIONS(2013),
    [sym_duet_dev_type_video_projector] = ACTIONS(2013),
    [sym_duet_dev_type_video_wall] = ACTIONS(2013),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2013),
    [sym_duet_dev_type_weather] = ACTIONS(2013),
    [sym_duet_dev_not_polled] = ACTIONS(2013),
    [sym_duet_dev_polled] = ACTIONS(2013),
    [sym_clkmgr_mode_network] = ACTIONS(2013),
    [sym_clkmgr_mode_standalone] = ACTIONS(2013),
    [sym_normal_standby] = ACTIONS(2013),
    [sym_normal_wake] = ACTIONS(2013),
    [sym_ip_multicast_ttl_option] = ACTIONS(2013),
    [sym_ip_ttl_subnet] = ACTIONS(2013),
    [sym_ip_ttl_site] = ACTIONS(2013),
    [sym_ip_ttl_region] = ACTIONS(2013),
    [sym_ip_ttl_continent] = ACTIONS(2013),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2013),
    [sym_ip_nodelay_on] = ACTIONS(2013),
    [sym_ip_nodelay_off] = ACTIONS(2013),
    [sym_amx_error] = ACTIONS(2013),
    [sym_amx_warning] = ACTIONS(2013),
    [sym_amx_info] = ACTIONS(2013),
    [sym_amx_debug] = ACTIONS(2013),
    [sym_netlinx_login_success] = ACTIONS(2013),
    [sym_netlinx_login_fail] = ACTIONS(2013),
    [sym_netlinx_logout] = ACTIONS(2013),
    [sym_null_str] = ACTIONS(2013),
    [sym_smtp_address] = ACTIONS(2013),
    [sym_smtp_port_number] = ACTIONS(2013),
    [sym_smtp_username] = ACTIONS(2013),
    [sym_smtp_password] = ACTIONS(2013),
    [sym_smtp_require_tls] = ACTIONS(2013),
    [sym_smtp_from] = ACTIONS(2013),
    [sym_smtp_tls_true] = ACTIONS(2013),
    [sym_smtp_tls_false] = ACTIONS(2013),
    [sym_listview_on_row_select_event] = ACTIONS(2013),
    [sym_data_structure_unknown] = ACTIONS(2013),
    [sym_data_structure_datafeed] = ACTIONS(2013),
    [sym_data_structure_datarecordset] = ACTIONS(2013),
    [sym_data_structure_datarecord] = ACTIONS(2013),
    [sym_data_structure_datafield] = ACTIONS(2013),
    [sym_data_max_content_field_count] = ACTIONS(2013),
    [sym_data_max_metadata_field_count] = ACTIONS(2013),
    [sym_data_max_id_length] = ACTIONS(2013),
    [sym_data_max_name_length] = ACTIONS(2013),
    [sym_data_max_description_length] = ACTIONS(2013),
    [sym_data_max_type_length] = ACTIONS(2013),
    [sym_data_max_format_length] = ACTIONS(2013),
    [sym_data_max_source_length] = ACTIONS(2013),
    [sym_data_max_value_length] = ACTIONS(2013),
    [sym_data_max_label_length] = ACTIONS(2013),
    [sym_data_max_url_length] = ACTIONS(2013),
    [sym_max_last_login_info_length] = ACTIONS(2013),
    [sym_data_type_unknown] = ACTIONS(2013),
    [sym_data_type_string] = ACTIONS(2013),
    [sym_data_type_datetime] = ACTIONS(2013),
    [sym_data_type_date] = ACTIONS(2013),
    [sym_data_type_time] = ACTIONS(2013),
    [sym_data_type_image] = ACTIONS(2013),
    [sym_data_format_url] = ACTIONS(2013),
    [sym_data_format_phone] = ACTIONS(2013),
    [sym_data_format_email] = ACTIONS(2013),
    [sym_data_format_iso8601] = ACTIONS(2013),
    [sym_auditlog_priv] = ACTIONS(2013),
    [sym_configuration_priv] = ACTIONS(2013),
    [sym_deviceconfig_priv] = ACTIONS(2013),
    [sym_ftp_priv] = ACTIONS(2013),
    [sym_http_priv] = ACTIONS(2013),
    [sym_networkconfig_priv] = ACTIONS(2013),
    [sym_programport_priv] = ACTIONS(2013),
    [sym_remoteui_priv] = ACTIONS(2013),
    [sym_securitycontrol_priv] = ACTIONS(2013),
    [sym_softwaremanagement_priv] = ACTIONS(2013),
    [sym_terminal_priv] = ACTIONS(2013),
    [sym_tpadmin_priv] = ACTIONS(2013),
    [sym_user1_priv] = ACTIONS(2013),
    [sym_user2_priv] = ACTIONS(2013),
    [sym_user3_priv] = ACTIONS(2013),
    [sym_user4_priv] = ACTIONS(2013),
    [sym_usermanagement_priv] = ACTIONS(2013),
    [sym_valid_account] = ACTIONS(2013),
    [sym_err_name_invalid] = ACTIONS(2013),
    [sym_err_password_invalid] = ACTIONS(2013),
    [sym_err_invalid_account] = ACTIONS(2013),
    [sym_err_no_authorization] = ACTIONS(2013),
    [sym_err_invalid_parameter] = ACTIONS(2013),
    [sym_err_user_locked_out] = ACTIONS(2013),
    [sym_err_user_account_expired] = ACTIONS(2013),
    [sym_err_general_error] = ACTIONS(2013),
    [sym_err_authentication_error] = ACTIONS(2013),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2013),
    [sym_tls_validate_certificate] = ACTIONS(2013),
    [sym_tdata] = ACTIONS(2013),
    [sym_tchannel] = ACTIONS(2013),
    [sym_tlevel] = ACTIONS(2013),
    [sym_tbutton] = ACTIONS(2013),
    [sym_ttimeline] = ACTIONS(2013),
    [sym_tcustom] = ACTIONS(2013),
    [sym_url_struct] = ACTIONS(2013),
    [sym_dns_struct] = ACTIONS(2013),
    [sym_ip_address_struct] = ACTIONS(2013),
    [sym_dev_info_struct] = ACTIONS(2013),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2013),
    [sym_data_feed] = ACTIONS(2013),
    [sym_data_field] = ACTIONS(2013),
    [sym_data_record] = ACTIONS(2013),
    [sym_wc_data_feed] = ACTIONS(2013),
    [sym_wc_data_field] = ACTIONS(2013),
    [sym_wc_data_record] = ACTIONS(2013),
    [sym_last_login_info] = ACTIONS(2013),
    [sym____reserved___] = ACTIONS(2013),
    [sym_date] = ACTIONS(2013),
    [sym_day] = ACTIONS(2013),
    [sym_time] = ACTIONS(2013),
    [sym_ldate] = ACTIONS(2013),
    [sym_push_device] = ACTIONS(2013),
    [sym_push_channel] = ACTIONS(2013),
    [sym_push_devchan] = ACTIONS(2013),
    [sym_release_device] = ACTIONS(2013),
    [sym_release_channel] = ACTIONS(2013),
    [sym_release_devchan] = ACTIONS(2013),
    [sym_master_slot] = ACTIONS(2013),
    [sym_get_pulse_time] = ACTIONS(2013),
    [sym_get_timer] = ACTIONS(2013),
    [sym_master_sn] = ACTIONS(2013),
    [sym_system_number] = ACTIONS(2013),
    [sym_button] = ACTIONS(2013),
    [sym_dv_channel] = ACTIONS(2013),
    [sym_data] = ACTIONS(2013),
    [sym_level] = ACTIONS(2013),
    [sym_timeline] = ACTIONS(2013),
    [sym_channel] = ACTIONS(2013),
    [sym_custom] = ACTIONS(2013),
    [sym_atoi] = ACTIONS(2013),
    [sym_atol] = ACTIONS(2013),
    [sym_atof] = ACTIONS(2013),
    [sym_itoa] = ACTIONS(2013),
    [sym_itohex] = ACTIONS(2013),
    [sym_hextoi] = ACTIONS(2013),
    [sym_ftoa] = ACTIONS(2013),
    [sym_find_string] = ACTIONS(2013),
    [sym_left_string] = ACTIONS(2013),
    [sym_right_string] = ACTIONS(2013),
    [sym_mid_string] = ACTIONS(2013),
    [sym_remove_string] = ACTIONS(2013),
    [sym_get_buffer_string] = ACTIONS(2013),
    [sym_get_buffer_char] = ACTIONS(2013),
    [sym_get_multi_buffer_string] = ACTIONS(2013),
    [sym_length_string] = ACTIONS(2013),
    [sym_max_length_string] = ACTIONS(2013),
    [sym_set_length_string] = ACTIONS(2013),
    [sym_compare_string] = ACTIONS(2013),
    [sym_upper_string] = ACTIONS(2013),
    [sym_lower_string] = ACTIONS(2013),
    [sym_get_last] = ACTIONS(2013),
    [sym_date_to_day] = ACTIONS(2013),
    [sym_date_to_month] = ACTIONS(2013),
    [sym_date_to_year] = ACTIONS(2013),
    [sym_time_to_hour] = ACTIONS(2013),
    [sym_time_to_minute] = ACTIONS(2013),
    [sym_time_to_second] = ACTIONS(2013),
    [sym_day_of_week] = ACTIONS(2013),
    [sym_set_pulse_time] = ACTIONS(2013),
    [sym_set_timer] = ACTIONS(2013),
    [sym_file_open] = ACTIONS(2013),
    [sym_file_close] = ACTIONS(2013),
    [sym_file_read] = ACTIONS(2013),
    [sym_file_read_line] = ACTIONS(2013),
    [sym_file_write] = ACTIONS(2013),
    [sym_file_write_line] = ACTIONS(2013),
    [sym_file_copy] = ACTIONS(2013),
    [sym_file_delete] = ACTIONS(2013),
    [sym_file_rename] = ACTIONS(2013),
    [sym_file_dir] = ACTIONS(2013),
    [sym_file_getdir] = ACTIONS(2013),
    [sym_file_setdir] = ACTIONS(2013),
    [sym_file_create_dir] = ACTIONS(2013),
    [sym_file_remove_dir] = ACTIONS(2013),
    [sym_file_seek] = ACTIONS(2013),
    [sym_length_array] = ACTIONS(2013),
    [sym_max_length_array] = ACTIONS(2013),
    [sym_set_length_array] = ACTIONS(2013),
    [sym_ip_client_open] = ACTIONS(2013),
    [sym_ip_client_close] = ACTIONS(2013),
    [sym_ip_server_open] = ACTIONS(2013),
    [sym_ip_server_close] = ACTIONS(2013),
    [sym_ip_bound_client_open] = ACTIONS(2013),
    [sym_ip_mc_server_open] = ACTIONS(2013),
    [sym_ip_set_option] = ACTIONS(2013),
    [sym_ssh_client_open] = ACTIONS(2013),
    [sym_ssh_client_close] = ACTIONS(2013),
    [sym_tls_client_open] = ACTIONS(2013),
    [sym_tls_client_close] = ACTIONS(2013),
    [sym_do_push] = ACTIONS(2013),
    [sym_do_push_timed] = ACTIONS(2013),
    [sym_do_release] = ACTIONS(2013),
    [sym_device_id] = ACTIONS(2013),
    [sym_device_info] = ACTIONS(2013),
    [sym_device_id_string] = ACTIONS(2013),
    [sym_device_standby] = ACTIONS(2013),
    [sym_device_wake] = ACTIONS(2013),
    [sym_timeline_create] = ACTIONS(2013),
    [sym_timeline_kill] = ACTIONS(2013),
    [sym_timeline_reload] = ACTIONS(2013),
    [sym_timeline_pause] = ACTIONS(2013),
    [sym_timeline_restart] = ACTIONS(2013),
    [sym_timeline_set] = ACTIONS(2013),
    [sym_timeline_get] = ACTIONS(2013),
    [sym_timeline_active] = ACTIONS(2013),
    [sym_timeline_dynamic_id] = ACTIONS(2013),
    [sym_variable_to_string] = ACTIONS(2013),
    [sym_string_to_variable] = ACTIONS(2013),
    [sym_length_variable_to_string] = ACTIONS(2013),
    [sym_type_cast] = ACTIONS(2013),
    [sym_variable_to_xml] = ACTIONS(2013),
    [sym_xml_to_variable] = ACTIONS(2013),
    [sym_length_variable_to_xml] = ACTIONS(2013),
    [sym_redirect_string] = ACTIONS(2013),
    [sym_random_number] = ACTIONS(2013),
    [sym_get_unique_id] = ACTIONS(2013),
    [sym_get_system_number] = ACTIONS(2013),
    [sym_set_system_number] = ACTIONS(2013),
    [sym_get_serial_number] = ACTIONS(2013),
    [sym_get_master_build] = ACTIONS(2013),
    [sym_set_validation_code] = ACTIONS(2013),
    [sym_reboot] = ACTIONS(2013),
    [sym_load_duet_module] = ACTIONS(2013),
    [sym_unload_duet_module] = ACTIONS(2013),
    [sym_duet_mem_size_set] = ACTIONS(2013),
    [sym_duet_mem_size_get] = ACTIONS(2013),
    [sym_dynamic_application_device] = ACTIONS(2013),
    [sym_dynamic_polled_port] = ACTIONS(2013),
    [sym_static_port_binding] = ACTIONS(2013),
    [sym_static_ip_binding] = ACTIONS(2013),
    [sym_get_dns_list] = ACTIONS(2013),
    [sym_set_dns_list] = ACTIONS(2013),
    [sym_get_ip_address] = ACTIONS(2013),
    [sym_set_ip_address] = ACTIONS(2013),
    [sym_get_url_list] = ACTIONS(2013),
    [sym_add_url_entry] = ACTIONS(2013),
    [sym_delete_url_entry] = ACTIONS(2013),
    [sym_abs_value] = ACTIONS(2013),
    [sym_max_value] = ACTIONS(2013),
    [sym_min_value] = ACTIONS(2013),
    [sym_exp_value] = ACTIONS(2013),
    [sym_log_value] = ACTIONS(2013),
    [sym_log10_value] = ACTIONS(2013),
    [sym_power_value] = ACTIONS(2013),
    [sym_sqrt_value] = ACTIONS(2013),
    [sym_raw_be] = ACTIONS(2013),
    [sym_raw_le] = ACTIONS(2013),
    [sym_internal_queue_size_set] = ACTIONS(2013),
    [sym_internal_queue_size_get] = ACTIONS(2013),
    [sym_internal_threshold_set] = ACTIONS(2013),
    [sym_internal_threshold_get] = ACTIONS(2013),
    [sym_rebuilt_event] = ACTIONS(2013),
    [sym_do_custom_event] = ACTIONS(2013),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2013),
    [sym_clkmgr_set_clk_source] = ACTIONS(2013),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2013),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2013),
    [sym_clkmgr_get_timezone] = ACTIONS(2013),
    [sym_clkmgr_set_timezone] = ACTIONS(2013),
    [sym_clkmgr_get_resync_period] = ACTIONS(2013),
    [sym_clkmgr_set_resync_period] = ACTIONS(2013),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2013),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2013),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2013),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2013),
    [sym_clkmgr_get_timeservers] = ACTIONS(2013),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2013),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2013),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2013),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2013),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2013),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2013),
    [sym_set_log_level] = ACTIONS(2013),
    [sym_get_log_level] = ACTIONS(2013),
    [sym_amx_log] = ACTIONS(2013),
    [sym_smtp_server_config_set] = ACTIONS(2013),
    [sym_smtp_server_config_get] = ACTIONS(2013),
    [sym_smtp_send] = ACTIONS(2013),
    [sym_validate_netlinx_account] = ACTIONS(2013),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2013),
    [sym_authenticate_certificate] = ACTIONS(2013),
    [sym_audit_netlinx_session_event] = ACTIONS(2013),
    [sym_audit_netlinx_generic_event] = ACTIONS(2013),
    [sym_get_available_flash_disk_space] = ACTIONS(2013),
    [sym_get_max_flash_disk_space] = ACTIONS(2013),
    [sym_data_create_feed] = ACTIONS(2013),
    [sym_data_delete_feed] = ACTIONS(2013),
    [sym_data_publish_feed] = ACTIONS(2013),
    [sym_data_get_published_feed] = ACTIONS(2013),
    [sym_data_add_record] = ACTIONS(2013),
    [sym_data_get_event_record] = ACTIONS(2013),
    [sym__wc_data_create_feed] = ACTIONS(2013),
    [sym__wc_data_add_record] = ACTIONS(2013),
    [sym__wc_data_get_event_record] = ACTIONS(2013),
    [sym_format] = ACTIONS(2013),
    [sym_astro_clock] = ACTIONS(2013),
    [sym_set_outdoor_temperature] = ACTIONS(2013),
    [sym_set_virtual_level_count] = ACTIONS(2013),
    [sym_set_virtual_channel_count] = ACTIONS(2013),
    [sym_set_virtual_port_count] = ACTIONS(2013),
    [sym__wc_encode_int] = ACTIONS(2013),
    [sym__wc_decode_int] = ACTIONS(2013),
    [sym__wc_int] = ACTIONS(2013),
    [sym__wc_to_ch_int] = ACTIONS(2013),
    [sym__ch_to_wc_int] = ACTIONS(2013),
    [sym__wc_find_string_int] = ACTIONS(2013),
    [sym__wc_left_string_int] = ACTIONS(2013),
    [sym__wc_lower_string_int] = ACTIONS(2013),
    [sym__wc_mid_string_int] = ACTIONS(2013),
    [sym__wc_remove_string_int] = ACTIONS(2013),
    [sym__wc_right_string_int] = ACTIONS(2013),
    [sym__wc_upper_string_int] = ACTIONS(2013),
    [sym__wc_compare_string_int] = ACTIONS(2013),
    [sym__wc_get_buffer_char_int] = ACTIONS(2013),
    [sym__wc_get_buffer_string_int] = ACTIONS(2013),
    [sym__wc_concat_string_int] = ACTIONS(2013),
    [sym__wc_file_open_int] = ACTIONS(2013),
    [sym__wc_file_close_int] = ACTIONS(2013),
    [sym__wc_file_read_int] = ACTIONS(2013),
    [sym__wc_file_read_line_int] = ACTIONS(2013),
    [sym__wc_file_write_int] = ACTIONS(2013),
    [sym__wc_file_write_line_int] = ACTIONS(2013),
    [aux_sym_preproc_include_token1] = ACTIONS(2013),
    [aux_sym_preproc_define_token1] = ACTIONS(2015),
    [aux_sym_preproc_warn_token1] = ACTIONS(2015),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2015),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2015),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2015),
    [anon_sym_LBRACK] = ACTIONS(2015),
    [anon_sym_LBRACE] = ACTIONS(2015),
    [anon_sym_RBRACE] = ACTIONS(2015),
    [anon_sym_LPAREN] = ACTIONS(2013),
    [anon_sym_BANG] = ACTIONS(2015),
    [anon_sym_TILDE] = ACTIONS(2015),
    [anon_sym_DASH] = ACTIONS(2013),
    [anon_sym_PLUS] = ACTIONS(2013),
    [anon_sym_DASH_DASH] = ACTIONS(2015),
    [anon_sym_PLUS_PLUS] = ACTIONS(2015),
    [anon_sym_DQUOTE] = ACTIONS(2015),
    [anon_sym_SQUOTE] = ACTIONS(2015),
    [sym_number_literal] = ACTIONS(2015),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(564)] = {
    [sym_identifier] = ACTIONS(1697),
    [sym_define_call_keyword] = ACTIONS(1697),
    [sym_define_function_keyword] = ACTIONS(1697),
    [sym_define_library_function_keyword] = ACTIONS(1697),
    [sym_define_module_keyword] = ACTIONS(1697),
    [sym_char_keyword] = ACTIONS(1697),
    [sym_widechar_keyword] = ACTIONS(1697),
    [sym_integer_keyword] = ACTIONS(1697),
    [sym_sinteger_keyword] = ACTIONS(1697),
    [sym_long_keyword] = ACTIONS(1697),
    [sym_slong_keyword] = ACTIONS(1697),
    [sym_float_keyword] = ACTIONS(1697),
    [sym_double_keyword] = ACTIONS(1697),
    [sym_variant_keyword] = ACTIONS(1697),
    [sym_variantarray_keyword] = ACTIONS(1697),
    [sym_local_var_keyword] = ACTIONS(1697),
    [sym_stack_var_keyword] = ACTIONS(1697),
    [sym_constant_keyword] = ACTIONS(1697),
    [sym_volatile_keyword] = ACTIONS(1697),
    [sym_non_volatile_keyword] = ACTIONS(1697),
    [sym_persistent_keyword] = ACTIONS(1697),
    [sym_dev_keyword] = ACTIONS(1697),
    [sym_devlev_keyword] = ACTIONS(1697),
    [sym_devchan_keyword] = ACTIONS(1697),
    [sym_if_keyword] = ACTIONS(1697),
    [sym_switch_keyword] = ACTIONS(1697),
    [sym_case_keyword] = ACTIONS(1697),
    [sym_default_keyword] = ACTIONS(1697),
    [sym_while_keyword] = ACTIONS(1697),
    [sym_for_keyword] = ACTIONS(1697),
    [sym_break_keyword] = ACTIONS(1697),
    [sym_continue_keyword] = ACTIONS(1697),
    [sym_return_keyword] = ACTIONS(1697),
    [sym_select_keyword] = ACTIONS(1697),
    [sym_active_keyword] = ACTIONS(1697),
    [sym_struct_keyword] = ACTIONS(1697),
    [sym_structure_keyword] = ACTIONS(1697),
    [aux_sym_on_keyword_token1] = ACTIONS(1697),
    [aux_sym_off_keyword_token1] = ACTIONS(1697),
    [sym_send_level_keyword] = ACTIONS(1697),
    [sym_send_string_keyword] = ACTIONS(1697),
    [sym_send_command_keyword] = ACTIONS(1697),
    [sym_clear_buffer_keyword] = ACTIONS(1697),
    [sym_create_buffer_keyword] = ACTIONS(1697),
    [sym_create_multi_buffer_keyword] = ACTIONS(1697),
    [sym_call_keyword] = ACTIONS(1697),
    [sym_system_call_keyword] = ACTIONS(1697),
    [sym_devchan_to_keyword] = ACTIONS(1697),
    [sym_devchan_min_to_keyword] = ACTIONS(1697),
    [sym_devchan_total_off_keyword] = ACTIONS(1697),
    [sym_devchan_pulse_keyword] = ACTIONS(1697),
    [sym_wait_keyword] = ACTIONS(1697),
    [sym_cancel_wait_keyword] = ACTIONS(1697),
    [sym_wait_until_keyword] = ACTIONS(1697),
    [sym_cancel_wait_until_keyword] = ACTIONS(1697),
    [sym_cancel_all_wait_keyword] = ACTIONS(1697),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1697),
    [sym_not] = ACTIONS(1697),
    [sym_bnot] = ACTIONS(1697),
    [sym___file__] = ACTIONS(1697),
    [sym___date__] = ACTIONS(1697),
    [sym___ldate__] = ACTIONS(1697),
    [sym___time__] = ACTIONS(1697),
    [sym___line__] = ACTIONS(1697),
    [sym___name__] = ACTIONS(1697),
    [sym___netlinx__] = ACTIONS(1697),
    [sym_netlinx_axi_version] = ACTIONS(1697),
    [sym_url_flg_tcp] = ACTIONS(1697),
    [sym_url_flg_accinfopresent] = ACTIONS(1697),
    [sym_url_flg_temp] = ACTIONS(1697),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1697),
    [sym_url_flg_stat_mask] = ACTIONS(1697),
    [sym_url_flg_stat_lookup] = ACTIONS(1697),
    [sym_url_flg_stat_connecting] = ACTIONS(1697),
    [sym_url_flg_stat_waiting] = ACTIONS(1697),
    [sym_url_flg_stat_connected] = ACTIONS(1697),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1697),
    [sym_first_virtual_device] = ACTIONS(1697),
    [sym_first_local_port] = ACTIONS(1697),
    [sym_dynamic_virtual_device] = ACTIONS(1697),
    [sym_timeline_once] = ACTIONS(1697),
    [sym_timeline_repeat] = ACTIONS(1697),
    [sym_timeline_absolute] = ACTIONS(1697),
    [sym_timeline_relative] = ACTIONS(1697),
    [sym_do_push_timed_infinite] = ACTIONS(1697),
    [sym_source_type_no_address] = ACTIONS(1697),
    [sym_source_type_neuron_id] = ACTIONS(1697),
    [sym_source_type_ip_address] = ACTIONS(1697),
    [sym_source_type_axlink] = ACTIONS(1697),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1697),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1697),
    [sym_source_type_ip_socket_address] = ACTIONS(1697),
    [sym_source_type_rs232] = ACTIONS(1697),
    [sym_source_type_internal] = ACTIONS(1697),
    [sym_source_type_mac_address] = ACTIONS(1697),
    [sym_source_type_ipv4_port] = ACTIONS(1697),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1697),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1697),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1697),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1697),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1697),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1697),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1697),
    [sym_true] = ACTIONS(1697),
    [sym_false] = ACTIONS(1697),
    [sym_file_read_only] = ACTIONS(1697),
    [sym_file_rw_new] = ACTIONS(1697),
    [sym_file_rw_append] = ACTIONS(1697),
    [sym_ip_tcp] = ACTIONS(1697),
    [sym_ip_udp] = ACTIONS(1697),
    [sym_ip_udp_2way] = ACTIONS(1697),
    [sym_xml_encode_types] = ACTIONS(1697),
    [sym_xml_encode_char_as_list] = ACTIONS(1697),
    [sym_xml_encode_le] = ACTIONS(1697),
    [sym_xml_decode_types] = ACTIONS(1697),
    [sym_xml_decode_no_preserve] = ACTIONS(1697),
    [sym_duet_dev_type_display_device] = ACTIONS(1697),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1697),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1697),
    [sym_duet_dev_type_io_device] = ACTIONS(1697),
    [sym_duet_dev_type_relay_device] = ACTIONS(1697),
    [sym_duet_dev_type_ups] = ACTIONS(1697),
    [sym_duet_dev_type_amplifier] = ACTIONS(1697),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1697),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1697),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1697),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1697),
    [sym_duet_dev_type_camera] = ACTIONS(1697),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1697),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1697),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1697),
    [sym_duet_dev_type_dss] = ACTIONS(1697),
    [sym_duet_dev_type_dvr] = ACTIONS(1697),
    [sym_duet_dev_type_disc_device] = ACTIONS(1697),
    [sym_duet_dev_type_document_camera] = ACTIONS(1697),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1697),
    [sym_duet_dev_type_hvac] = ACTIONS(1697),
    [sym_duet_dev_type_keypad] = ACTIONS(1697),
    [sym_duet_dev_type_light] = ACTIONS(1697),
    [sym_duet_dev_type_monitor] = ACTIONS(1697),
    [sym_duet_dev_type_motor] = ACTIONS(1697),
    [sym_duet_dev_type_multi_window] = ACTIONS(1697),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1697),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1697),
    [sym_duet_dev_type_receiver] = ACTIONS(1697),
    [sym_duet_dev_type_security_system] = ACTIONS(1697),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1697),
    [sym_duet_dev_type_settop_box] = ACTIONS(1697),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1697),
    [sym_duet_dev_type_switcher] = ACTIONS(1697),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1697),
    [sym_duet_dev_type_tv] = ACTIONS(1697),
    [sym_duet_dev_type_utility] = ACTIONS(1697),
    [sym_duet_dev_type_vcr] = ACTIONS(1697),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1697),
    [sym_duet_dev_type_video_processor] = ACTIONS(1697),
    [sym_duet_dev_type_video_projector] = ACTIONS(1697),
    [sym_duet_dev_type_video_wall] = ACTIONS(1697),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1697),
    [sym_duet_dev_type_weather] = ACTIONS(1697),
    [sym_duet_dev_not_polled] = ACTIONS(1697),
    [sym_duet_dev_polled] = ACTIONS(1697),
    [sym_clkmgr_mode_network] = ACTIONS(1697),
    [sym_clkmgr_mode_standalone] = ACTIONS(1697),
    [sym_normal_standby] = ACTIONS(1697),
    [sym_normal_wake] = ACTIONS(1697),
    [sym_ip_multicast_ttl_option] = ACTIONS(1697),
    [sym_ip_ttl_subnet] = ACTIONS(1697),
    [sym_ip_ttl_site] = ACTIONS(1697),
    [sym_ip_ttl_region] = ACTIONS(1697),
    [sym_ip_ttl_continent] = ACTIONS(1697),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1697),
    [sym_ip_nodelay_on] = ACTIONS(1697),
    [sym_ip_nodelay_off] = ACTIONS(1697),
    [sym_amx_error] = ACTIONS(1697),
    [sym_amx_warning] = ACTIONS(1697),
    [sym_amx_info] = ACTIONS(1697),
    [sym_amx_debug] = ACTIONS(1697),
    [sym_netlinx_login_success] = ACTIONS(1697),
    [sym_netlinx_login_fail] = ACTIONS(1697),
    [sym_netlinx_logout] = ACTIONS(1697),
    [sym_null_str] = ACTIONS(1697),
    [sym_smtp_address] = ACTIONS(1697),
    [sym_smtp_port_number] = ACTIONS(1697),
    [sym_smtp_username] = ACTIONS(1697),
    [sym_smtp_password] = ACTIONS(1697),
    [sym_smtp_require_tls] = ACTIONS(1697),
    [sym_smtp_from] = ACTIONS(1697),
    [sym_smtp_tls_true] = ACTIONS(1697),
    [sym_smtp_tls_false] = ACTIONS(1697),
    [sym_listview_on_row_select_event] = ACTIONS(1697),
    [sym_data_structure_unknown] = ACTIONS(1697),
    [sym_data_structure_datafeed] = ACTIONS(1697),
    [sym_data_structure_datarecordset] = ACTIONS(1697),
    [sym_data_structure_datarecord] = ACTIONS(1697),
    [sym_data_structure_datafield] = ACTIONS(1697),
    [sym_data_max_content_field_count] = ACTIONS(1697),
    [sym_data_max_metadata_field_count] = ACTIONS(1697),
    [sym_data_max_id_length] = ACTIONS(1697),
    [sym_data_max_name_length] = ACTIONS(1697),
    [sym_data_max_description_length] = ACTIONS(1697),
    [sym_data_max_type_length] = ACTIONS(1697),
    [sym_data_max_format_length] = ACTIONS(1697),
    [sym_data_max_source_length] = ACTIONS(1697),
    [sym_data_max_value_length] = ACTIONS(1697),
    [sym_data_max_label_length] = ACTIONS(1697),
    [sym_data_max_url_length] = ACTIONS(1697),
    [sym_max_last_login_info_length] = ACTIONS(1697),
    [sym_data_type_unknown] = ACTIONS(1697),
    [sym_data_type_string] = ACTIONS(1697),
    [sym_data_type_datetime] = ACTIONS(1697),
    [sym_data_type_date] = ACTIONS(1697),
    [sym_data_type_time] = ACTIONS(1697),
    [sym_data_type_image] = ACTIONS(1697),
    [sym_data_format_url] = ACTIONS(1697),
    [sym_data_format_phone] = ACTIONS(1697),
    [sym_data_format_email] = ACTIONS(1697),
    [sym_data_format_iso8601] = ACTIONS(1697),
    [sym_auditlog_priv] = ACTIONS(1697),
    [sym_configuration_priv] = ACTIONS(1697),
    [sym_deviceconfig_priv] = ACTIONS(1697),
    [sym_ftp_priv] = ACTIONS(1697),
    [sym_http_priv] = ACTIONS(1697),
    [sym_networkconfig_priv] = ACTIONS(1697),
    [sym_programport_priv] = ACTIONS(1697),
    [sym_remoteui_priv] = ACTIONS(1697),
    [sym_securitycontrol_priv] = ACTIONS(1697),
    [sym_softwaremanagement_priv] = ACTIONS(1697),
    [sym_terminal_priv] = ACTIONS(1697),
    [sym_tpadmin_priv] = ACTIONS(1697),
    [sym_user1_priv] = ACTIONS(1697),
    [sym_user2_priv] = ACTIONS(1697),
    [sym_user3_priv] = ACTIONS(1697),
    [sym_user4_priv] = ACTIONS(1697),
    [sym_usermanagement_priv] = ACTIONS(1697),
    [sym_valid_account] = ACTIONS(1697),
    [sym_err_name_invalid] = ACTIONS(1697),
    [sym_err_password_invalid] = ACTIONS(1697),
    [sym_err_invalid_account] = ACTIONS(1697),
    [sym_err_no_authorization] = ACTIONS(1697),
    [sym_err_invalid_parameter] = ACTIONS(1697),
    [sym_err_user_locked_out] = ACTIONS(1697),
    [sym_err_user_account_expired] = ACTIONS(1697),
    [sym_err_general_error] = ACTIONS(1697),
    [sym_err_authentication_error] = ACTIONS(1697),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1697),
    [sym_tls_validate_certificate] = ACTIONS(1697),
    [sym_tdata] = ACTIONS(1697),
    [sym_tchannel] = ACTIONS(1697),
    [sym_tlevel] = ACTIONS(1697),
    [sym_tbutton] = ACTIONS(1697),
    [sym_ttimeline] = ACTIONS(1697),
    [sym_tcustom] = ACTIONS(1697),
    [sym_url_struct] = ACTIONS(1697),
    [sym_dns_struct] = ACTIONS(1697),
    [sym_ip_address_struct] = ACTIONS(1697),
    [sym_dev_info_struct] = ACTIONS(1697),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1697),
    [sym_data_feed] = ACTIONS(1697),
    [sym_data_field] = ACTIONS(1697),
    [sym_data_record] = ACTIONS(1697),
    [sym_wc_data_feed] = ACTIONS(1697),
    [sym_wc_data_field] = ACTIONS(1697),
    [sym_wc_data_record] = ACTIONS(1697),
    [sym_last_login_info] = ACTIONS(1697),
    [sym____reserved___] = ACTIONS(1697),
    [sym_date] = ACTIONS(1697),
    [sym_day] = ACTIONS(1697),
    [sym_time] = ACTIONS(1697),
    [sym_ldate] = ACTIONS(1697),
    [sym_push_device] = ACTIONS(1697),
    [sym_push_channel] = ACTIONS(1697),
    [sym_push_devchan] = ACTIONS(1697),
    [sym_release_device] = ACTIONS(1697),
    [sym_release_channel] = ACTIONS(1697),
    [sym_release_devchan] = ACTIONS(1697),
    [sym_master_slot] = ACTIONS(1697),
    [sym_get_pulse_time] = ACTIONS(1697),
    [sym_get_timer] = ACTIONS(1697),
    [sym_master_sn] = ACTIONS(1697),
    [sym_system_number] = ACTIONS(1697),
    [sym_button] = ACTIONS(1697),
    [sym_dv_channel] = ACTIONS(1697),
    [sym_data] = ACTIONS(1697),
    [sym_level] = ACTIONS(1697),
    [sym_timeline] = ACTIONS(1697),
    [sym_channel] = ACTIONS(1697),
    [sym_custom] = ACTIONS(1697),
    [sym_atoi] = ACTIONS(1697),
    [sym_atol] = ACTIONS(1697),
    [sym_atof] = ACTIONS(1697),
    [sym_itoa] = ACTIONS(1697),
    [sym_itohex] = ACTIONS(1697),
    [sym_hextoi] = ACTIONS(1697),
    [sym_ftoa] = ACTIONS(1697),
    [sym_find_string] = ACTIONS(1697),
    [sym_left_string] = ACTIONS(1697),
    [sym_right_string] = ACTIONS(1697),
    [sym_mid_string] = ACTIONS(1697),
    [sym_remove_string] = ACTIONS(1697),
    [sym_get_buffer_string] = ACTIONS(1697),
    [sym_get_buffer_char] = ACTIONS(1697),
    [sym_get_multi_buffer_string] = ACTIONS(1697),
    [sym_length_string] = ACTIONS(1697),
    [sym_max_length_string] = ACTIONS(1697),
    [sym_set_length_string] = ACTIONS(1697),
    [sym_compare_string] = ACTIONS(1697),
    [sym_upper_string] = ACTIONS(1697),
    [sym_lower_string] = ACTIONS(1697),
    [sym_get_last] = ACTIONS(1697),
    [sym_date_to_day] = ACTIONS(1697),
    [sym_date_to_month] = ACTIONS(1697),
    [sym_date_to_year] = ACTIONS(1697),
    [sym_time_to_hour] = ACTIONS(1697),
    [sym_time_to_minute] = ACTIONS(1697),
    [sym_time_to_second] = ACTIONS(1697),
    [sym_day_of_week] = ACTIONS(1697),
    [sym_set_pulse_time] = ACTIONS(1697),
    [sym_set_timer] = ACTIONS(1697),
    [sym_file_open] = ACTIONS(1697),
    [sym_file_close] = ACTIONS(1697),
    [sym_file_read] = ACTIONS(1697),
    [sym_file_read_line] = ACTIONS(1697),
    [sym_file_write] = ACTIONS(1697),
    [sym_file_write_line] = ACTIONS(1697),
    [sym_file_copy] = ACTIONS(1697),
    [sym_file_delete] = ACTIONS(1697),
    [sym_file_rename] = ACTIONS(1697),
    [sym_file_dir] = ACTIONS(1697),
    [sym_file_getdir] = ACTIONS(1697),
    [sym_file_setdir] = ACTIONS(1697),
    [sym_file_create_dir] = ACTIONS(1697),
    [sym_file_remove_dir] = ACTIONS(1697),
    [sym_file_seek] = ACTIONS(1697),
    [sym_length_array] = ACTIONS(1697),
    [sym_max_length_array] = ACTIONS(1697),
    [sym_set_length_array] = ACTIONS(1697),
    [sym_ip_client_open] = ACTIONS(1697),
    [sym_ip_client_close] = ACTIONS(1697),
    [sym_ip_server_open] = ACTIONS(1697),
    [sym_ip_server_close] = ACTIONS(1697),
    [sym_ip_bound_client_open] = ACTIONS(1697),
    [sym_ip_mc_server_open] = ACTIONS(1697),
    [sym_ip_set_option] = ACTIONS(1697),
    [sym_ssh_client_open] = ACTIONS(1697),
    [sym_ssh_client_close] = ACTIONS(1697),
    [sym_tls_client_open] = ACTIONS(1697),
    [sym_tls_client_close] = ACTIONS(1697),
    [sym_do_push] = ACTIONS(1697),
    [sym_do_push_timed] = ACTIONS(1697),
    [sym_do_release] = ACTIONS(1697),
    [sym_device_id] = ACTIONS(1697),
    [sym_device_info] = ACTIONS(1697),
    [sym_device_id_string] = ACTIONS(1697),
    [sym_device_standby] = ACTIONS(1697),
    [sym_device_wake] = ACTIONS(1697),
    [sym_timeline_create] = ACTIONS(1697),
    [sym_timeline_kill] = ACTIONS(1697),
    [sym_timeline_reload] = ACTIONS(1697),
    [sym_timeline_pause] = ACTIONS(1697),
    [sym_timeline_restart] = ACTIONS(1697),
    [sym_timeline_set] = ACTIONS(1697),
    [sym_timeline_get] = ACTIONS(1697),
    [sym_timeline_active] = ACTIONS(1697),
    [sym_timeline_dynamic_id] = ACTIONS(1697),
    [sym_variable_to_string] = ACTIONS(1697),
    [sym_string_to_variable] = ACTIONS(1697),
    [sym_length_variable_to_string] = ACTIONS(1697),
    [sym_type_cast] = ACTIONS(1697),
    [sym_variable_to_xml] = ACTIONS(1697),
    [sym_xml_to_variable] = ACTIONS(1697),
    [sym_length_variable_to_xml] = ACTIONS(1697),
    [sym_redirect_string] = ACTIONS(1697),
    [sym_random_number] = ACTIONS(1697),
    [sym_get_unique_id] = ACTIONS(1697),
    [sym_get_system_number] = ACTIONS(1697),
    [sym_set_system_number] = ACTIONS(1697),
    [sym_get_serial_number] = ACTIONS(1697),
    [sym_get_master_build] = ACTIONS(1697),
    [sym_set_validation_code] = ACTIONS(1697),
    [sym_reboot] = ACTIONS(1697),
    [sym_load_duet_module] = ACTIONS(1697),
    [sym_unload_duet_module] = ACTIONS(1697),
    [sym_duet_mem_size_set] = ACTIONS(1697),
    [sym_duet_mem_size_get] = ACTIONS(1697),
    [sym_dynamic_application_device] = ACTIONS(1697),
    [sym_dynamic_polled_port] = ACTIONS(1697),
    [sym_static_port_binding] = ACTIONS(1697),
    [sym_static_ip_binding] = ACTIONS(1697),
    [sym_get_dns_list] = ACTIONS(1697),
    [sym_set_dns_list] = ACTIONS(1697),
    [sym_get_ip_address] = ACTIONS(1697),
    [sym_set_ip_address] = ACTIONS(1697),
    [sym_get_url_list] = ACTIONS(1697),
    [sym_add_url_entry] = ACTIONS(1697),
    [sym_delete_url_entry] = ACTIONS(1697),
    [sym_abs_value] = ACTIONS(1697),
    [sym_max_value] = ACTIONS(1697),
    [sym_min_value] = ACTIONS(1697),
    [sym_exp_value] = ACTIONS(1697),
    [sym_log_value] = ACTIONS(1697),
    [sym_log10_value] = ACTIONS(1697),
    [sym_power_value] = ACTIONS(1697),
    [sym_sqrt_value] = ACTIONS(1697),
    [sym_raw_be] = ACTIONS(1697),
    [sym_raw_le] = ACTIONS(1697),
    [sym_internal_queue_size_set] = ACTIONS(1697),
    [sym_internal_queue_size_get] = ACTIONS(1697),
    [sym_internal_threshold_set] = ACTIONS(1697),
    [sym_internal_threshold_get] = ACTIONS(1697),
    [sym_rebuilt_event] = ACTIONS(1697),
    [sym_do_custom_event] = ACTIONS(1697),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1697),
    [sym_clkmgr_set_clk_source] = ACTIONS(1697),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1697),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1697),
    [sym_clkmgr_get_timezone] = ACTIONS(1697),
    [sym_clkmgr_set_timezone] = ACTIONS(1697),
    [sym_clkmgr_get_resync_period] = ACTIONS(1697),
    [sym_clkmgr_set_resync_period] = ACTIONS(1697),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1697),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1697),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1697),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1697),
    [sym_clkmgr_get_timeservers] = ACTIONS(1697),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1697),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1697),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1697),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1697),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1697),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1697),
    [sym_set_log_level] = ACTIONS(1697),
    [sym_get_log_level] = ACTIONS(1697),
    [sym_amx_log] = ACTIONS(1697),
    [sym_smtp_server_config_set] = ACTIONS(1697),
    [sym_smtp_server_config_get] = ACTIONS(1697),
    [sym_smtp_send] = ACTIONS(1697),
    [sym_validate_netlinx_account] = ACTIONS(1697),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1697),
    [sym_authenticate_certificate] = ACTIONS(1697),
    [sym_audit_netlinx_session_event] = ACTIONS(1697),
    [sym_audit_netlinx_generic_event] = ACTIONS(1697),
    [sym_get_available_flash_disk_space] = ACTIONS(1697),
    [sym_get_max_flash_disk_space] = ACTIONS(1697),
    [sym_data_create_feed] = ACTIONS(1697),
    [sym_data_delete_feed] = ACTIONS(1697),
    [sym_data_publish_feed] = ACTIONS(1697),
    [sym_data_get_published_feed] = ACTIONS(1697),
    [sym_data_add_record] = ACTIONS(1697),
    [sym_data_get_event_record] = ACTIONS(1697),
    [sym__wc_data_create_feed] = ACTIONS(1697),
    [sym__wc_data_add_record] = ACTIONS(1697),
    [sym__wc_data_get_event_record] = ACTIONS(1697),
    [sym_format] = ACTIONS(1697),
    [sym_astro_clock] = ACTIONS(1697),
    [sym_set_outdoor_temperature] = ACTIONS(1697),
    [sym_set_virtual_level_count] = ACTIONS(1697),
    [sym_set_virtual_channel_count] = ACTIONS(1697),
    [sym_set_virtual_port_count] = ACTIONS(1697),
    [sym__wc_encode_int] = ACTIONS(1697),
    [sym__wc_decode_int] = ACTIONS(1697),
    [sym__wc_int] = ACTIONS(1697),
    [sym__wc_to_ch_int] = ACTIONS(1697),
    [sym__ch_to_wc_int] = ACTIONS(1697),
    [sym__wc_find_string_int] = ACTIONS(1697),
    [sym__wc_left_string_int] = ACTIONS(1697),
    [sym__wc_lower_string_int] = ACTIONS(1697),
    [sym__wc_mid_string_int] = ACTIONS(1697),
    [sym__wc_remove_string_int] = ACTIONS(1697),
    [sym__wc_right_string_int] = ACTIONS(1697),
    [sym__wc_upper_string_int] = ACTIONS(1697),
    [sym__wc_compare_string_int] = ACTIONS(1697),
    [sym__wc_get_buffer_char_int] = ACTIONS(1697),
    [sym__wc_get_buffer_string_int] = ACTIONS(1697),
    [sym__wc_concat_string_int] = ACTIONS(1697),
    [sym__wc_file_open_int] = ACTIONS(1697),
    [sym__wc_file_close_int] = ACTIONS(1697),
    [sym__wc_file_read_int] = ACTIONS(1697),
    [sym__wc_file_read_line_int] = ACTIONS(1697),
    [sym__wc_file_write_int] = ACTIONS(1697),
    [sym__wc_file_write_line_int] = ACTIONS(1697),
    [aux_sym_preproc_include_token1] = ACTIONS(1697),
    [aux_sym_preproc_define_token1] = ACTIONS(1695),
    [aux_sym_preproc_warn_token1] = ACTIONS(1695),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(1695),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1695),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1695),
    [anon_sym_LBRACK] = ACTIONS(1695),
    [anon_sym_LBRACE] = ACTIONS(1695),
    [anon_sym_RBRACE] = ACTIONS(1695),
    [anon_sym_LPAREN] = ACTIONS(1697),
    [anon_sym_BANG] = ACTIONS(1695),
    [anon_sym_TILDE] = ACTIONS(1695),
    [anon_sym_DASH] = ACTIONS(1697),
    [anon_sym_PLUS] = ACTIONS(1697),
    [anon_sym_DASH_DASH] = ACTIONS(1695),
    [anon_sym_PLUS_PLUS] = ACTIONS(1695),
    [anon_sym_DQUOTE] = ACTIONS(1695),
    [anon_sym_SQUOTE] = ACTIONS(1695),
    [sym_number_literal] = ACTIONS(1695),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(565)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(252),
    [sym_statement] = STATE(210),
    [sym_expression_statement] = STATE(252),
    [sym_if_statement] = STATE(252),
    [sym_switch_statement] = STATE(252),
    [sym_case_statement] = STATE(252),
    [sym_while_statement] = STATE(252),
    [sym_for_statement] = STATE(252),
    [sym_select_statement] = STATE(252),
    [sym_active_statement] = STATE(252),
    [sym_return_statement] = STATE(252),
    [sym_break_statement] = STATE(252),
    [sym_continue_statement] = STATE(252),
    [sym_devchan_operation_statement] = STATE(252),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(252),
    [sym_send_command_statement] = STATE(252),
    [sym_send_level_statement] = STATE(252),
    [sym_create_buffer_statement] = STATE(252),
    [sym_create_multi_buffer_statement] = STATE(252),
    [sym_clear_buffer_statement] = STATE(252),
    [sym_wait_statement] = STATE(252),
    [sym_wait_until_statement] = STATE(252),
    [sym_cancel_all_wait_statement] = STATE(252),
    [sym_cancel_all_wait_until_statement] = STATE(252),
    [sym_cancel_wait_statement] = STATE(252),
    [sym_cancel_wait_until_statement] = STATE(252),
    [sym_call_statement] = STATE(252),
    [sym_system_call_statement] = STATE(252),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(566)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(252),
    [sym_statement] = STATE(215),
    [sym_expression_statement] = STATE(252),
    [sym_if_statement] = STATE(252),
    [sym_switch_statement] = STATE(252),
    [sym_case_statement] = STATE(252),
    [sym_while_statement] = STATE(252),
    [sym_for_statement] = STATE(252),
    [sym_select_statement] = STATE(252),
    [sym_active_statement] = STATE(252),
    [sym_return_statement] = STATE(252),
    [sym_break_statement] = STATE(252),
    [sym_continue_statement] = STATE(252),
    [sym_devchan_operation_statement] = STATE(252),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(252),
    [sym_send_command_statement] = STATE(252),
    [sym_send_level_statement] = STATE(252),
    [sym_create_buffer_statement] = STATE(252),
    [sym_create_multi_buffer_statement] = STATE(252),
    [sym_clear_buffer_statement] = STATE(252),
    [sym_wait_statement] = STATE(252),
    [sym_wait_until_statement] = STATE(252),
    [sym_cancel_all_wait_statement] = STATE(252),
    [sym_cancel_all_wait_until_statement] = STATE(252),
    [sym_cancel_wait_statement] = STATE(252),
    [sym_cancel_wait_until_statement] = STATE(252),
    [sym_call_statement] = STATE(252),
    [sym_system_call_statement] = STATE(252),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(567)] = {
    [sym_devchan_on_keyword] = STATE(2175),
    [sym_devchan_off_keyword] = STATE(2175),
    [sym_compound_statement] = STATE(252),
    [sym_statement] = STATE(216),
    [sym_expression_statement] = STATE(252),
    [sym_if_statement] = STATE(252),
    [sym_switch_statement] = STATE(252),
    [sym_case_statement] = STATE(252),
    [sym_while_statement] = STATE(252),
    [sym_for_statement] = STATE(252),
    [sym_select_statement] = STATE(252),
    [sym_active_statement] = STATE(252),
    [sym_return_statement] = STATE(252),
    [sym_break_statement] = STATE(252),
    [sym_continue_statement] = STATE(252),
    [sym_devchan_operation_statement] = STATE(252),
    [sym_devchan_operation] = STATE(1851),
    [sym_send_string_statement] = STATE(252),
    [sym_send_command_statement] = STATE(252),
    [sym_send_level_statement] = STATE(252),
    [sym_create_buffer_statement] = STATE(252),
    [sym_create_multi_buffer_statement] = STATE(252),
    [sym_clear_buffer_statement] = STATE(252),
    [sym_wait_statement] = STATE(252),
    [sym_wait_until_statement] = STATE(252),
    [sym_cancel_all_wait_statement] = STATE(252),
    [sym_cancel_all_wait_until_statement] = STATE(252),
    [sym_cancel_wait_statement] = STATE(252),
    [sym_cancel_wait_until_statement] = STATE(252),
    [sym_call_statement] = STATE(252),
    [sym_system_call_statement] = STATE(252),
    [sym_expression] = STATE(1143),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2027),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_if_keyword] = ACTIONS(207),
    [sym_switch_keyword] = ACTIONS(209),
    [sym_case_keyword] = ACTIONS(211),
    [sym_default_keyword] = ACTIONS(213),
    [sym_while_keyword] = ACTIONS(215),
    [sym_for_keyword] = ACTIONS(217),
    [sym_break_keyword] = ACTIONS(219),
    [sym_continue_keyword] = ACTIONS(221),
    [sym_return_keyword] = ACTIONS(223),
    [sym_select_keyword] = ACTIONS(225),
    [sym_active_keyword] = ACTIONS(227),
    [aux_sym_on_keyword_token1] = ACTIONS(95),
    [aux_sym_off_keyword_token1] = ACTIONS(97),
    [sym_send_level_keyword] = ACTIONS(229),
    [sym_send_string_keyword] = ACTIONS(231),
    [sym_send_command_keyword] = ACTIONS(233),
    [sym_clear_buffer_keyword] = ACTIONS(235),
    [sym_create_buffer_keyword] = ACTIONS(237),
    [sym_create_multi_buffer_keyword] = ACTIONS(239),
    [sym_call_keyword] = ACTIONS(241),
    [sym_system_call_keyword] = ACTIONS(243),
    [sym_devchan_to_keyword] = ACTIONS(115),
    [sym_devchan_min_to_keyword] = ACTIONS(115),
    [sym_devchan_total_off_keyword] = ACTIONS(115),
    [sym_devchan_pulse_keyword] = ACTIONS(115),
    [sym_wait_keyword] = ACTIONS(245),
    [sym_cancel_wait_keyword] = ACTIONS(247),
    [sym_wait_until_keyword] = ACTIONS(249),
    [sym_cancel_wait_until_keyword] = ACTIONS(251),
    [sym_cancel_all_wait_keyword] = ACTIONS(253),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(255),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(323),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(568)] = {
    [sym_identifier] = ACTIONS(2057),
    [sym_define_call_keyword] = ACTIONS(2057),
    [sym_define_function_keyword] = ACTIONS(2057),
    [sym_define_library_function_keyword] = ACTIONS(2057),
    [sym_define_module_keyword] = ACTIONS(2057),
    [sym_char_keyword] = ACTIONS(2057),
    [sym_widechar_keyword] = ACTIONS(2057),
    [sym_integer_keyword] = ACTIONS(2057),
    [sym_sinteger_keyword] = ACTIONS(2057),
    [sym_long_keyword] = ACTIONS(2057),
    [sym_slong_keyword] = ACTIONS(2057),
    [sym_float_keyword] = ACTIONS(2057),
    [sym_double_keyword] = ACTIONS(2057),
    [sym_variant_keyword] = ACTIONS(2057),
    [sym_variantarray_keyword] = ACTIONS(2057),
    [sym_local_var_keyword] = ACTIONS(2057),
    [sym_stack_var_keyword] = ACTIONS(2057),
    [sym_constant_keyword] = ACTIONS(2057),
    [sym_volatile_keyword] = ACTIONS(2057),
    [sym_non_volatile_keyword] = ACTIONS(2057),
    [sym_persistent_keyword] = ACTIONS(2057),
    [sym_dev_keyword] = ACTIONS(2057),
    [sym_devlev_keyword] = ACTIONS(2057),
    [sym_devchan_keyword] = ACTIONS(2057),
    [sym_if_keyword] = ACTIONS(2057),
    [sym_switch_keyword] = ACTIONS(2057),
    [sym_case_keyword] = ACTIONS(2057),
    [sym_default_keyword] = ACTIONS(2057),
    [sym_while_keyword] = ACTIONS(2057),
    [sym_for_keyword] = ACTIONS(2057),
    [sym_break_keyword] = ACTIONS(2057),
    [sym_continue_keyword] = ACTIONS(2057),
    [sym_return_keyword] = ACTIONS(2057),
    [sym_select_keyword] = ACTIONS(2057),
    [sym_active_keyword] = ACTIONS(2057),
    [sym_struct_keyword] = ACTIONS(2057),
    [sym_structure_keyword] = ACTIONS(2057),
    [aux_sym_on_keyword_token1] = ACTIONS(2057),
    [aux_sym_off_keyword_token1] = ACTIONS(2057),
    [sym_send_level_keyword] = ACTIONS(2057),
    [sym_send_string_keyword] = ACTIONS(2057),
    [sym_send_command_keyword] = ACTIONS(2057),
    [sym_clear_buffer_keyword] = ACTIONS(2057),
    [sym_create_buffer_keyword] = ACTIONS(2057),
    [sym_create_multi_buffer_keyword] = ACTIONS(2057),
    [sym_call_keyword] = ACTIONS(2057),
    [sym_system_call_keyword] = ACTIONS(2057),
    [sym_devchan_to_keyword] = ACTIONS(2057),
    [sym_devchan_min_to_keyword] = ACTIONS(2057),
    [sym_devchan_total_off_keyword] = ACTIONS(2057),
    [sym_devchan_pulse_keyword] = ACTIONS(2057),
    [sym_wait_keyword] = ACTIONS(2057),
    [sym_cancel_wait_keyword] = ACTIONS(2057),
    [sym_wait_until_keyword] = ACTIONS(2057),
    [sym_cancel_wait_until_keyword] = ACTIONS(2057),
    [sym_cancel_all_wait_keyword] = ACTIONS(2057),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2057),
    [sym_not] = ACTIONS(2057),
    [sym_bnot] = ACTIONS(2057),
    [sym___file__] = ACTIONS(2057),
    [sym___date__] = ACTIONS(2057),
    [sym___ldate__] = ACTIONS(2057),
    [sym___time__] = ACTIONS(2057),
    [sym___line__] = ACTIONS(2057),
    [sym___name__] = ACTIONS(2057),
    [sym___netlinx__] = ACTIONS(2057),
    [sym_netlinx_axi_version] = ACTIONS(2057),
    [sym_url_flg_tcp] = ACTIONS(2057),
    [sym_url_flg_accinfopresent] = ACTIONS(2057),
    [sym_url_flg_temp] = ACTIONS(2057),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2057),
    [sym_url_flg_stat_mask] = ACTIONS(2057),
    [sym_url_flg_stat_lookup] = ACTIONS(2057),
    [sym_url_flg_stat_connecting] = ACTIONS(2057),
    [sym_url_flg_stat_waiting] = ACTIONS(2057),
    [sym_url_flg_stat_connected] = ACTIONS(2057),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2057),
    [sym_first_virtual_device] = ACTIONS(2057),
    [sym_first_local_port] = ACTIONS(2057),
    [sym_dynamic_virtual_device] = ACTIONS(2057),
    [sym_timeline_once] = ACTIONS(2057),
    [sym_timeline_repeat] = ACTIONS(2057),
    [sym_timeline_absolute] = ACTIONS(2057),
    [sym_timeline_relative] = ACTIONS(2057),
    [sym_do_push_timed_infinite] = ACTIONS(2057),
    [sym_source_type_no_address] = ACTIONS(2057),
    [sym_source_type_neuron_id] = ACTIONS(2057),
    [sym_source_type_ip_address] = ACTIONS(2057),
    [sym_source_type_axlink] = ACTIONS(2057),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2057),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2057),
    [sym_source_type_ip_socket_address] = ACTIONS(2057),
    [sym_source_type_rs232] = ACTIONS(2057),
    [sym_source_type_internal] = ACTIONS(2057),
    [sym_source_type_mac_address] = ACTIONS(2057),
    [sym_source_type_ipv4_port] = ACTIONS(2057),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2057),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2057),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2057),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2057),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2057),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2057),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2057),
    [sym_true] = ACTIONS(2057),
    [sym_false] = ACTIONS(2057),
    [sym_file_read_only] = ACTIONS(2057),
    [sym_file_rw_new] = ACTIONS(2057),
    [sym_file_rw_append] = ACTIONS(2057),
    [sym_ip_tcp] = ACTIONS(2057),
    [sym_ip_udp] = ACTIONS(2057),
    [sym_ip_udp_2way] = ACTIONS(2057),
    [sym_xml_encode_types] = ACTIONS(2057),
    [sym_xml_encode_char_as_list] = ACTIONS(2057),
    [sym_xml_encode_le] = ACTIONS(2057),
    [sym_xml_decode_types] = ACTIONS(2057),
    [sym_xml_decode_no_preserve] = ACTIONS(2057),
    [sym_duet_dev_type_display_device] = ACTIONS(2057),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2057),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2057),
    [sym_duet_dev_type_io_device] = ACTIONS(2057),
    [sym_duet_dev_type_relay_device] = ACTIONS(2057),
    [sym_duet_dev_type_ups] = ACTIONS(2057),
    [sym_duet_dev_type_amplifier] = ACTIONS(2057),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2057),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2057),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2057),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2057),
    [sym_duet_dev_type_camera] = ACTIONS(2057),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2057),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2057),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2057),
    [sym_duet_dev_type_dss] = ACTIONS(2057),
    [sym_duet_dev_type_dvr] = ACTIONS(2057),
    [sym_duet_dev_type_disc_device] = ACTIONS(2057),
    [sym_duet_dev_type_document_camera] = ACTIONS(2057),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2057),
    [sym_duet_dev_type_hvac] = ACTIONS(2057),
    [sym_duet_dev_type_keypad] = ACTIONS(2057),
    [sym_duet_dev_type_light] = ACTIONS(2057),
    [sym_duet_dev_type_monitor] = ACTIONS(2057),
    [sym_duet_dev_type_motor] = ACTIONS(2057),
    [sym_duet_dev_type_multi_window] = ACTIONS(2057),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2057),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2057),
    [sym_duet_dev_type_receiver] = ACTIONS(2057),
    [sym_duet_dev_type_security_system] = ACTIONS(2057),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2057),
    [sym_duet_dev_type_settop_box] = ACTIONS(2057),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2057),
    [sym_duet_dev_type_switcher] = ACTIONS(2057),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2057),
    [sym_duet_dev_type_tv] = ACTIONS(2057),
    [sym_duet_dev_type_utility] = ACTIONS(2057),
    [sym_duet_dev_type_vcr] = ACTIONS(2057),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2057),
    [sym_duet_dev_type_video_processor] = ACTIONS(2057),
    [sym_duet_dev_type_video_projector] = ACTIONS(2057),
    [sym_duet_dev_type_video_wall] = ACTIONS(2057),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2057),
    [sym_duet_dev_type_weather] = ACTIONS(2057),
    [sym_duet_dev_not_polled] = ACTIONS(2057),
    [sym_duet_dev_polled] = ACTIONS(2057),
    [sym_clkmgr_mode_network] = ACTIONS(2057),
    [sym_clkmgr_mode_standalone] = ACTIONS(2057),
    [sym_normal_standby] = ACTIONS(2057),
    [sym_normal_wake] = ACTIONS(2057),
    [sym_ip_multicast_ttl_option] = ACTIONS(2057),
    [sym_ip_ttl_subnet] = ACTIONS(2057),
    [sym_ip_ttl_site] = ACTIONS(2057),
    [sym_ip_ttl_region] = ACTIONS(2057),
    [sym_ip_ttl_continent] = ACTIONS(2057),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2057),
    [sym_ip_nodelay_on] = ACTIONS(2057),
    [sym_ip_nodelay_off] = ACTIONS(2057),
    [sym_amx_error] = ACTIONS(2057),
    [sym_amx_warning] = ACTIONS(2057),
    [sym_amx_info] = ACTIONS(2057),
    [sym_amx_debug] = ACTIONS(2057),
    [sym_netlinx_login_success] = ACTIONS(2057),
    [sym_netlinx_login_fail] = ACTIONS(2057),
    [sym_netlinx_logout] = ACTIONS(2057),
    [sym_null_str] = ACTIONS(2057),
    [sym_smtp_address] = ACTIONS(2057),
    [sym_smtp_port_number] = ACTIONS(2057),
    [sym_smtp_username] = ACTIONS(2057),
    [sym_smtp_password] = ACTIONS(2057),
    [sym_smtp_require_tls] = ACTIONS(2057),
    [sym_smtp_from] = ACTIONS(2057),
    [sym_smtp_tls_true] = ACTIONS(2057),
    [sym_smtp_tls_false] = ACTIONS(2057),
    [sym_listview_on_row_select_event] = ACTIONS(2057),
    [sym_data_structure_unknown] = ACTIONS(2057),
    [sym_data_structure_datafeed] = ACTIONS(2057),
    [sym_data_structure_datarecordset] = ACTIONS(2057),
    [sym_data_structure_datarecord] = ACTIONS(2057),
    [sym_data_structure_datafield] = ACTIONS(2057),
    [sym_data_max_content_field_count] = ACTIONS(2057),
    [sym_data_max_metadata_field_count] = ACTIONS(2057),
    [sym_data_max_id_length] = ACTIONS(2057),
    [sym_data_max_name_length] = ACTIONS(2057),
    [sym_data_max_description_length] = ACTIONS(2057),
    [sym_data_max_type_length] = ACTIONS(2057),
    [sym_data_max_format_length] = ACTIONS(2057),
    [sym_data_max_source_length] = ACTIONS(2057),
    [sym_data_max_value_length] = ACTIONS(2057),
    [sym_data_max_label_length] = ACTIONS(2057),
    [sym_data_max_url_length] = ACTIONS(2057),
    [sym_max_last_login_info_length] = ACTIONS(2057),
    [sym_data_type_unknown] = ACTIONS(2057),
    [sym_data_type_string] = ACTIONS(2057),
    [sym_data_type_datetime] = ACTIONS(2057),
    [sym_data_type_date] = ACTIONS(2057),
    [sym_data_type_time] = ACTIONS(2057),
    [sym_data_type_image] = ACTIONS(2057),
    [sym_data_format_url] = ACTIONS(2057),
    [sym_data_format_phone] = ACTIONS(2057),
    [sym_data_format_email] = ACTIONS(2057),
    [sym_data_format_iso8601] = ACTIONS(2057),
    [sym_auditlog_priv] = ACTIONS(2057),
    [sym_configuration_priv] = ACTIONS(2057),
    [sym_deviceconfig_priv] = ACTIONS(2057),
    [sym_ftp_priv] = ACTIONS(2057),
    [sym_http_priv] = ACTIONS(2057),
    [sym_networkconfig_priv] = ACTIONS(2057),
    [sym_programport_priv] = ACTIONS(2057),
    [sym_remoteui_priv] = ACTIONS(2057),
    [sym_securitycontrol_priv] = ACTIONS(2057),
    [sym_softwaremanagement_priv] = ACTIONS(2057),
    [sym_terminal_priv] = ACTIONS(2057),
    [sym_tpadmin_priv] = ACTIONS(2057),
    [sym_user1_priv] = ACTIONS(2057),
    [sym_user2_priv] = ACTIONS(2057),
    [sym_user3_priv] = ACTIONS(2057),
    [sym_user4_priv] = ACTIONS(2057),
    [sym_usermanagement_priv] = ACTIONS(2057),
    [sym_valid_account] = ACTIONS(2057),
    [sym_err_name_invalid] = ACTIONS(2057),
    [sym_err_password_invalid] = ACTIONS(2057),
    [sym_err_invalid_account] = ACTIONS(2057),
    [sym_err_no_authorization] = ACTIONS(2057),
    [sym_err_invalid_parameter] = ACTIONS(2057),
    [sym_err_user_locked_out] = ACTIONS(2057),
    [sym_err_user_account_expired] = ACTIONS(2057),
    [sym_err_general_error] = ACTIONS(2057),
    [sym_err_authentication_error] = ACTIONS(2057),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2057),
    [sym_tls_validate_certificate] = ACTIONS(2057),
    [sym_tdata] = ACTIONS(2057),
    [sym_tchannel] = ACTIONS(2057),
    [sym_tlevel] = ACTIONS(2057),
    [sym_tbutton] = ACTIONS(2057),
    [sym_ttimeline] = ACTIONS(2057),
    [sym_tcustom] = ACTIONS(2057),
    [sym_url_struct] = ACTIONS(2057),
    [sym_dns_struct] = ACTIONS(2057),
    [sym_ip_address_struct] = ACTIONS(2057),
    [sym_dev_info_struct] = ACTIONS(2057),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2057),
    [sym_data_feed] = ACTIONS(2057),
    [sym_data_field] = ACTIONS(2057),
    [sym_data_record] = ACTIONS(2057),
    [sym_wc_data_feed] = ACTIONS(2057),
    [sym_wc_data_field] = ACTIONS(2057),
    [sym_wc_data_record] = ACTIONS(2057),
    [sym_last_login_info] = ACTIONS(2057),
    [sym____reserved___] = ACTIONS(2057),
    [sym_date] = ACTIONS(2057),
    [sym_day] = ACTIONS(2057),
    [sym_time] = ACTIONS(2057),
    [sym_ldate] = ACTIONS(2057),
    [sym_push_device] = ACTIONS(2057),
    [sym_push_channel] = ACTIONS(2057),
    [sym_push_devchan] = ACTIONS(2057),
    [sym_release_device] = ACTIONS(2057),
    [sym_release_channel] = ACTIONS(2057),
    [sym_release_devchan] = ACTIONS(2057),
    [sym_master_slot] = ACTIONS(2057),
    [sym_get_pulse_time] = ACTIONS(2057),
    [sym_get_timer] = ACTIONS(2057),
    [sym_master_sn] = ACTIONS(2057),
    [sym_system_number] = ACTIONS(2057),
    [sym_button] = ACTIONS(2057),
    [sym_dv_channel] = ACTIONS(2057),
    [sym_data] = ACTIONS(2057),
    [sym_level] = ACTIONS(2057),
    [sym_timeline] = ACTIONS(2057),
    [sym_channel] = ACTIONS(2057),
    [sym_custom] = ACTIONS(2057),
    [sym_atoi] = ACTIONS(2057),
    [sym_atol] = ACTIONS(2057),
    [sym_atof] = ACTIONS(2057),
    [sym_itoa] = ACTIONS(2057),
    [sym_itohex] = ACTIONS(2057),
    [sym_hextoi] = ACTIONS(2057),
    [sym_ftoa] = ACTIONS(2057),
    [sym_find_string] = ACTIONS(2057),
    [sym_left_string] = ACTIONS(2057),
    [sym_right_string] = ACTIONS(2057),
    [sym_mid_string] = ACTIONS(2057),
    [sym_remove_string] = ACTIONS(2057),
    [sym_get_buffer_string] = ACTIONS(2057),
    [sym_get_buffer_char] = ACTIONS(2057),
    [sym_get_multi_buffer_string] = ACTIONS(2057),
    [sym_length_string] = ACTIONS(2057),
    [sym_max_length_string] = ACTIONS(2057),
    [sym_set_length_string] = ACTIONS(2057),
    [sym_compare_string] = ACTIONS(2057),
    [sym_upper_string] = ACTIONS(2057),
    [sym_lower_string] = ACTIONS(2057),
    [sym_get_last] = ACTIONS(2057),
    [sym_date_to_day] = ACTIONS(2057),
    [sym_date_to_month] = ACTIONS(2057),
    [sym_date_to_year] = ACTIONS(2057),
    [sym_time_to_hour] = ACTIONS(2057),
    [sym_time_to_minute] = ACTIONS(2057),
    [sym_time_to_second] = ACTIONS(2057),
    [sym_day_of_week] = ACTIONS(2057),
    [sym_set_pulse_time] = ACTIONS(2057),
    [sym_set_timer] = ACTIONS(2057),
    [sym_file_open] = ACTIONS(2057),
    [sym_file_close] = ACTIONS(2057),
    [sym_file_read] = ACTIONS(2057),
    [sym_file_read_line] = ACTIONS(2057),
    [sym_file_write] = ACTIONS(2057),
    [sym_file_write_line] = ACTIONS(2057),
    [sym_file_copy] = ACTIONS(2057),
    [sym_file_delete] = ACTIONS(2057),
    [sym_file_rename] = ACTIONS(2057),
    [sym_file_dir] = ACTIONS(2057),
    [sym_file_getdir] = ACTIONS(2057),
    [sym_file_setdir] = ACTIONS(2057),
    [sym_file_create_dir] = ACTIONS(2057),
    [sym_file_remove_dir] = ACTIONS(2057),
    [sym_file_seek] = ACTIONS(2057),
    [sym_length_array] = ACTIONS(2057),
    [sym_max_length_array] = ACTIONS(2057),
    [sym_set_length_array] = ACTIONS(2057),
    [sym_ip_client_open] = ACTIONS(2057),
    [sym_ip_client_close] = ACTIONS(2057),
    [sym_ip_server_open] = ACTIONS(2057),
    [sym_ip_server_close] = ACTIONS(2057),
    [sym_ip_bound_client_open] = ACTIONS(2057),
    [sym_ip_mc_server_open] = ACTIONS(2057),
    [sym_ip_set_option] = ACTIONS(2057),
    [sym_ssh_client_open] = ACTIONS(2057),
    [sym_ssh_client_close] = ACTIONS(2057),
    [sym_tls_client_open] = ACTIONS(2057),
    [sym_tls_client_close] = ACTIONS(2057),
    [sym_do_push] = ACTIONS(2057),
    [sym_do_push_timed] = ACTIONS(2057),
    [sym_do_release] = ACTIONS(2057),
    [sym_device_id] = ACTIONS(2057),
    [sym_device_info] = ACTIONS(2057),
    [sym_device_id_string] = ACTIONS(2057),
    [sym_device_standby] = ACTIONS(2057),
    [sym_device_wake] = ACTIONS(2057),
    [sym_timeline_create] = ACTIONS(2057),
    [sym_timeline_kill] = ACTIONS(2057),
    [sym_timeline_reload] = ACTIONS(2057),
    [sym_timeline_pause] = ACTIONS(2057),
    [sym_timeline_restart] = ACTIONS(2057),
    [sym_timeline_set] = ACTIONS(2057),
    [sym_timeline_get] = ACTIONS(2057),
    [sym_timeline_active] = ACTIONS(2057),
    [sym_timeline_dynamic_id] = ACTIONS(2057),
    [sym_variable_to_string] = ACTIONS(2057),
    [sym_string_to_variable] = ACTIONS(2057),
    [sym_length_variable_to_string] = ACTIONS(2057),
    [sym_type_cast] = ACTIONS(2057),
    [sym_variable_to_xml] = ACTIONS(2057),
    [sym_xml_to_variable] = ACTIONS(2057),
    [sym_length_variable_to_xml] = ACTIONS(2057),
    [sym_redirect_string] = ACTIONS(2057),
    [sym_random_number] = ACTIONS(2057),
    [sym_get_unique_id] = ACTIONS(2057),
    [sym_get_system_number] = ACTIONS(2057),
    [sym_set_system_number] = ACTIONS(2057),
    [sym_get_serial_number] = ACTIONS(2057),
    [sym_get_master_build] = ACTIONS(2057),
    [sym_set_validation_code] = ACTIONS(2057),
    [sym_reboot] = ACTIONS(2057),
    [sym_load_duet_module] = ACTIONS(2057),
    [sym_unload_duet_module] = ACTIONS(2057),
    [sym_duet_mem_size_set] = ACTIONS(2057),
    [sym_duet_mem_size_get] = ACTIONS(2057),
    [sym_dynamic_application_device] = ACTIONS(2057),
    [sym_dynamic_polled_port] = ACTIONS(2057),
    [sym_static_port_binding] = ACTIONS(2057),
    [sym_static_ip_binding] = ACTIONS(2057),
    [sym_get_dns_list] = ACTIONS(2057),
    [sym_set_dns_list] = ACTIONS(2057),
    [sym_get_ip_address] = ACTIONS(2057),
    [sym_set_ip_address] = ACTIONS(2057),
    [sym_get_url_list] = ACTIONS(2057),
    [sym_add_url_entry] = ACTIONS(2057),
    [sym_delete_url_entry] = ACTIONS(2057),
    [sym_abs_value] = ACTIONS(2057),
    [sym_max_value] = ACTIONS(2057),
    [sym_min_value] = ACTIONS(2057),
    [sym_exp_value] = ACTIONS(2057),
    [sym_log_value] = ACTIONS(2057),
    [sym_log10_value] = ACTIONS(2057),
    [sym_power_value] = ACTIONS(2057),
    [sym_sqrt_value] = ACTIONS(2057),
    [sym_raw_be] = ACTIONS(2057),
    [sym_raw_le] = ACTIONS(2057),
    [sym_internal_queue_size_set] = ACTIONS(2057),
    [sym_internal_queue_size_get] = ACTIONS(2057),
    [sym_internal_threshold_set] = ACTIONS(2057),
    [sym_internal_threshold_get] = ACTIONS(2057),
    [sym_rebuilt_event] = ACTIONS(2057),
    [sym_do_custom_event] = ACTIONS(2057),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2057),
    [sym_clkmgr_set_clk_source] = ACTIONS(2057),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2057),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2057),
    [sym_clkmgr_get_timezone] = ACTIONS(2057),
    [sym_clkmgr_set_timezone] = ACTIONS(2057),
    [sym_clkmgr_get_resync_period] = ACTIONS(2057),
    [sym_clkmgr_set_resync_period] = ACTIONS(2057),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2057),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2057),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2057),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2057),
    [sym_clkmgr_get_timeservers] = ACTIONS(2057),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2057),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2057),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2057),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2057),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2057),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2057),
    [sym_set_log_level] = ACTIONS(2057),
    [sym_get_log_level] = ACTIONS(2057),
    [sym_amx_log] = ACTIONS(2057),
    [sym_smtp_server_config_set] = ACTIONS(2057),
    [sym_smtp_server_config_get] = ACTIONS(2057),
    [sym_smtp_send] = ACTIONS(2057),
    [sym_validate_netlinx_account] = ACTIONS(2057),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2057),
    [sym_authenticate_certificate] = ACTIONS(2057),
    [sym_audit_netlinx_session_event] = ACTIONS(2057),
    [sym_audit_netlinx_generic_event] = ACTIONS(2057),
    [sym_get_available_flash_disk_space] = ACTIONS(2057),
    [sym_get_max_flash_disk_space] = ACTIONS(2057),
    [sym_data_create_feed] = ACTIONS(2057),
    [sym_data_delete_feed] = ACTIONS(2057),
    [sym_data_publish_feed] = ACTIONS(2057),
    [sym_data_get_published_feed] = ACTIONS(2057),
    [sym_data_add_record] = ACTIONS(2057),
    [sym_data_get_event_record] = ACTIONS(2057),
    [sym__wc_data_create_feed] = ACTIONS(2057),
    [sym__wc_data_add_record] = ACTIONS(2057),
    [sym__wc_data_get_event_record] = ACTIONS(2057),
    [sym_format] = ACTIONS(2057),
    [sym_astro_clock] = ACTIONS(2057),
    [sym_set_outdoor_temperature] = ACTIONS(2057),
    [sym_set_virtual_level_count] = ACTIONS(2057),
    [sym_set_virtual_channel_count] = ACTIONS(2057),
    [sym_set_virtual_port_count] = ACTIONS(2057),
    [sym__wc_encode_int] = ACTIONS(2057),
    [sym__wc_decode_int] = ACTIONS(2057),
    [sym__wc_int] = ACTIONS(2057),
    [sym__wc_to_ch_int] = ACTIONS(2057),
    [sym__ch_to_wc_int] = ACTIONS(2057),
    [sym__wc_find_string_int] = ACTIONS(2057),
    [sym__wc_left_string_int] = ACTIONS(2057),
    [sym__wc_lower_string_int] = ACTIONS(2057),
    [sym__wc_mid_string_int] = ACTIONS(2057),
    [sym__wc_remove_string_int] = ACTIONS(2057),
    [sym__wc_right_string_int] = ACTIONS(2057),
    [sym__wc_upper_string_int] = ACTIONS(2057),
    [sym__wc_compare_string_int] = ACTIONS(2057),
    [sym__wc_get_buffer_char_int] = ACTIONS(2057),
    [sym__wc_get_buffer_string_int] = ACTIONS(2057),
    [sym__wc_concat_string_int] = ACTIONS(2057),
    [sym__wc_file_open_int] = ACTIONS(2057),
    [sym__wc_file_close_int] = ACTIONS(2057),
    [sym__wc_file_read_int] = ACTIONS(2057),
    [sym__wc_file_read_line_int] = ACTIONS(2057),
    [sym__wc_file_write_int] = ACTIONS(2057),
    [sym__wc_file_write_line_int] = ACTIONS(2057),
    [aux_sym_preproc_include_token1] = ACTIONS(2057),
    [aux_sym_preproc_define_token1] = ACTIONS(2059),
    [aux_sym_preproc_warn_token1] = ACTIONS(2059),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2059),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2059),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2059),
    [anon_sym_LBRACK] = ACTIONS(2059),
    [anon_sym_LBRACE] = ACTIONS(2059),
    [anon_sym_RBRACE] = ACTIONS(2059),
    [anon_sym_LPAREN] = ACTIONS(2057),
    [anon_sym_BANG] = ACTIONS(2059),
    [anon_sym_TILDE] = ACTIONS(2059),
    [anon_sym_DASH] = ACTIONS(2057),
    [anon_sym_PLUS] = ACTIONS(2057),
    [anon_sym_DASH_DASH] = ACTIONS(2059),
    [anon_sym_PLUS_PLUS] = ACTIONS(2059),
    [anon_sym_DQUOTE] = ACTIONS(2059),
    [anon_sym_SQUOTE] = ACTIONS(2059),
    [sym_number_literal] = ACTIONS(2059),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(569)] = {
    [sym_identifier] = ACTIONS(2025),
    [sym_define_call_keyword] = ACTIONS(2025),
    [sym_define_function_keyword] = ACTIONS(2025),
    [sym_define_library_function_keyword] = ACTIONS(2025),
    [sym_define_module_keyword] = ACTIONS(2025),
    [sym_char_keyword] = ACTIONS(2025),
    [sym_widechar_keyword] = ACTIONS(2025),
    [sym_integer_keyword] = ACTIONS(2025),
    [sym_sinteger_keyword] = ACTIONS(2025),
    [sym_long_keyword] = ACTIONS(2025),
    [sym_slong_keyword] = ACTIONS(2025),
    [sym_float_keyword] = ACTIONS(2025),
    [sym_double_keyword] = ACTIONS(2025),
    [sym_variant_keyword] = ACTIONS(2025),
    [sym_variantarray_keyword] = ACTIONS(2025),
    [sym_local_var_keyword] = ACTIONS(2025),
    [sym_stack_var_keyword] = ACTIONS(2025),
    [sym_constant_keyword] = ACTIONS(2025),
    [sym_volatile_keyword] = ACTIONS(2025),
    [sym_non_volatile_keyword] = ACTIONS(2025),
    [sym_persistent_keyword] = ACTIONS(2025),
    [sym_dev_keyword] = ACTIONS(2025),
    [sym_devlev_keyword] = ACTIONS(2025),
    [sym_devchan_keyword] = ACTIONS(2025),
    [sym_if_keyword] = ACTIONS(2025),
    [sym_switch_keyword] = ACTIONS(2025),
    [sym_case_keyword] = ACTIONS(2025),
    [sym_default_keyword] = ACTIONS(2025),
    [sym_while_keyword] = ACTIONS(2025),
    [sym_for_keyword] = ACTIONS(2025),
    [sym_break_keyword] = ACTIONS(2025),
    [sym_continue_keyword] = ACTIONS(2025),
    [sym_return_keyword] = ACTIONS(2025),
    [sym_select_keyword] = ACTIONS(2025),
    [sym_active_keyword] = ACTIONS(2025),
    [sym_struct_keyword] = ACTIONS(2025),
    [sym_structure_keyword] = ACTIONS(2025),
    [aux_sym_on_keyword_token1] = ACTIONS(2025),
    [aux_sym_off_keyword_token1] = ACTIONS(2025),
    [sym_send_level_keyword] = ACTIONS(2025),
    [sym_send_string_keyword] = ACTIONS(2025),
    [sym_send_command_keyword] = ACTIONS(2025),
    [sym_clear_buffer_keyword] = ACTIONS(2025),
    [sym_create_buffer_keyword] = ACTIONS(2025),
    [sym_create_multi_buffer_keyword] = ACTIONS(2025),
    [sym_call_keyword] = ACTIONS(2025),
    [sym_system_call_keyword] = ACTIONS(2025),
    [sym_devchan_to_keyword] = ACTIONS(2025),
    [sym_devchan_min_to_keyword] = ACTIONS(2025),
    [sym_devchan_total_off_keyword] = ACTIONS(2025),
    [sym_devchan_pulse_keyword] = ACTIONS(2025),
    [sym_wait_keyword] = ACTIONS(2025),
    [sym_cancel_wait_keyword] = ACTIONS(2025),
    [sym_wait_until_keyword] = ACTIONS(2025),
    [sym_cancel_wait_until_keyword] = ACTIONS(2025),
    [sym_cancel_all_wait_keyword] = ACTIONS(2025),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(2025),
    [sym_not] = ACTIONS(2025),
    [sym_bnot] = ACTIONS(2025),
    [sym___file__] = ACTIONS(2025),
    [sym___date__] = ACTIONS(2025),
    [sym___ldate__] = ACTIONS(2025),
    [sym___time__] = ACTIONS(2025),
    [sym___line__] = ACTIONS(2025),
    [sym___name__] = ACTIONS(2025),
    [sym___netlinx__] = ACTIONS(2025),
    [sym_netlinx_axi_version] = ACTIONS(2025),
    [sym_url_flg_tcp] = ACTIONS(2025),
    [sym_url_flg_accinfopresent] = ACTIONS(2025),
    [sym_url_flg_temp] = ACTIONS(2025),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2025),
    [sym_url_flg_stat_mask] = ACTIONS(2025),
    [sym_url_flg_stat_lookup] = ACTIONS(2025),
    [sym_url_flg_stat_connecting] = ACTIONS(2025),
    [sym_url_flg_stat_waiting] = ACTIONS(2025),
    [sym_url_flg_stat_connected] = ACTIONS(2025),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2025),
    [sym_first_virtual_device] = ACTIONS(2025),
    [sym_first_local_port] = ACTIONS(2025),
    [sym_dynamic_virtual_device] = ACTIONS(2025),
    [sym_timeline_once] = ACTIONS(2025),
    [sym_timeline_repeat] = ACTIONS(2025),
    [sym_timeline_absolute] = ACTIONS(2025),
    [sym_timeline_relative] = ACTIONS(2025),
    [sym_do_push_timed_infinite] = ACTIONS(2025),
    [sym_source_type_no_address] = ACTIONS(2025),
    [sym_source_type_neuron_id] = ACTIONS(2025),
    [sym_source_type_ip_address] = ACTIONS(2025),
    [sym_source_type_axlink] = ACTIONS(2025),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2025),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2025),
    [sym_source_type_ip_socket_address] = ACTIONS(2025),
    [sym_source_type_rs232] = ACTIONS(2025),
    [sym_source_type_internal] = ACTIONS(2025),
    [sym_source_type_mac_address] = ACTIONS(2025),
    [sym_source_type_ipv4_port] = ACTIONS(2025),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2025),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2025),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2025),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2025),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2025),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2025),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2025),
    [sym_true] = ACTIONS(2025),
    [sym_false] = ACTIONS(2025),
    [sym_file_read_only] = ACTIONS(2025),
    [sym_file_rw_new] = ACTIONS(2025),
    [sym_file_rw_append] = ACTIONS(2025),
    [sym_ip_tcp] = ACTIONS(2025),
    [sym_ip_udp] = ACTIONS(2025),
    [sym_ip_udp_2way] = ACTIONS(2025),
    [sym_xml_encode_types] = ACTIONS(2025),
    [sym_xml_encode_char_as_list] = ACTIONS(2025),
    [sym_xml_encode_le] = ACTIONS(2025),
    [sym_xml_decode_types] = ACTIONS(2025),
    [sym_xml_decode_no_preserve] = ACTIONS(2025),
    [sym_duet_dev_type_display_device] = ACTIONS(2025),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2025),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2025),
    [sym_duet_dev_type_io_device] = ACTIONS(2025),
    [sym_duet_dev_type_relay_device] = ACTIONS(2025),
    [sym_duet_dev_type_ups] = ACTIONS(2025),
    [sym_duet_dev_type_amplifier] = ACTIONS(2025),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2025),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2025),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2025),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2025),
    [sym_duet_dev_type_camera] = ACTIONS(2025),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2025),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2025),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2025),
    [sym_duet_dev_type_dss] = ACTIONS(2025),
    [sym_duet_dev_type_dvr] = ACTIONS(2025),
    [sym_duet_dev_type_disc_device] = ACTIONS(2025),
    [sym_duet_dev_type_document_camera] = ACTIONS(2025),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2025),
    [sym_duet_dev_type_hvac] = ACTIONS(2025),
    [sym_duet_dev_type_keypad] = ACTIONS(2025),
    [sym_duet_dev_type_light] = ACTIONS(2025),
    [sym_duet_dev_type_monitor] = ACTIONS(2025),
    [sym_duet_dev_type_motor] = ACTIONS(2025),
    [sym_duet_dev_type_multi_window] = ACTIONS(2025),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2025),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2025),
    [sym_duet_dev_type_receiver] = ACTIONS(2025),
    [sym_duet_dev_type_security_system] = ACTIONS(2025),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2025),
    [sym_duet_dev_type_settop_box] = ACTIONS(2025),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2025),
    [sym_duet_dev_type_switcher] = ACTIONS(2025),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2025),
    [sym_duet_dev_type_tv] = ACTIONS(2025),
    [sym_duet_dev_type_utility] = ACTIONS(2025),
    [sym_duet_dev_type_vcr] = ACTIONS(2025),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2025),
    [sym_duet_dev_type_video_processor] = ACTIONS(2025),
    [sym_duet_dev_type_video_projector] = ACTIONS(2025),
    [sym_duet_dev_type_video_wall] = ACTIONS(2025),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2025),
    [sym_duet_dev_type_weather] = ACTIONS(2025),
    [sym_duet_dev_not_polled] = ACTIONS(2025),
    [sym_duet_dev_polled] = ACTIONS(2025),
    [sym_clkmgr_mode_network] = ACTIONS(2025),
    [sym_clkmgr_mode_standalone] = ACTIONS(2025),
    [sym_normal_standby] = ACTIONS(2025),
    [sym_normal_wake] = ACTIONS(2025),
    [sym_ip_multicast_ttl_option] = ACTIONS(2025),
    [sym_ip_ttl_subnet] = ACTIONS(2025),
    [sym_ip_ttl_site] = ACTIONS(2025),
    [sym_ip_ttl_region] = ACTIONS(2025),
    [sym_ip_ttl_continent] = ACTIONS(2025),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2025),
    [sym_ip_nodelay_on] = ACTIONS(2025),
    [sym_ip_nodelay_off] = ACTIONS(2025),
    [sym_amx_error] = ACTIONS(2025),
    [sym_amx_warning] = ACTIONS(2025),
    [sym_amx_info] = ACTIONS(2025),
    [sym_amx_debug] = ACTIONS(2025),
    [sym_netlinx_login_success] = ACTIONS(2025),
    [sym_netlinx_login_fail] = ACTIONS(2025),
    [sym_netlinx_logout] = ACTIONS(2025),
    [sym_null_str] = ACTIONS(2025),
    [sym_smtp_address] = ACTIONS(2025),
    [sym_smtp_port_number] = ACTIONS(2025),
    [sym_smtp_username] = ACTIONS(2025),
    [sym_smtp_password] = ACTIONS(2025),
    [sym_smtp_require_tls] = ACTIONS(2025),
    [sym_smtp_from] = ACTIONS(2025),
    [sym_smtp_tls_true] = ACTIONS(2025),
    [sym_smtp_tls_false] = ACTIONS(2025),
    [sym_listview_on_row_select_event] = ACTIONS(2025),
    [sym_data_structure_unknown] = ACTIONS(2025),
    [sym_data_structure_datafeed] = ACTIONS(2025),
    [sym_data_structure_datarecordset] = ACTIONS(2025),
    [sym_data_structure_datarecord] = ACTIONS(2025),
    [sym_data_structure_datafield] = ACTIONS(2025),
    [sym_data_max_content_field_count] = ACTIONS(2025),
    [sym_data_max_metadata_field_count] = ACTIONS(2025),
    [sym_data_max_id_length] = ACTIONS(2025),
    [sym_data_max_name_length] = ACTIONS(2025),
    [sym_data_max_description_length] = ACTIONS(2025),
    [sym_data_max_type_length] = ACTIONS(2025),
    [sym_data_max_format_length] = ACTIONS(2025),
    [sym_data_max_source_length] = ACTIONS(2025),
    [sym_data_max_value_length] = ACTIONS(2025),
    [sym_data_max_label_length] = ACTIONS(2025),
    [sym_data_max_url_length] = ACTIONS(2025),
    [sym_max_last_login_info_length] = ACTIONS(2025),
    [sym_data_type_unknown] = ACTIONS(2025),
    [sym_data_type_string] = ACTIONS(2025),
    [sym_data_type_datetime] = ACTIONS(2025),
    [sym_data_type_date] = ACTIONS(2025),
    [sym_data_type_time] = ACTIONS(2025),
    [sym_data_type_image] = ACTIONS(2025),
    [sym_data_format_url] = ACTIONS(2025),
    [sym_data_format_phone] = ACTIONS(2025),
    [sym_data_format_email] = ACTIONS(2025),
    [sym_data_format_iso8601] = ACTIONS(2025),
    [sym_auditlog_priv] = ACTIONS(2025),
    [sym_configuration_priv] = ACTIONS(2025),
    [sym_deviceconfig_priv] = ACTIONS(2025),
    [sym_ftp_priv] = ACTIONS(2025),
    [sym_http_priv] = ACTIONS(2025),
    [sym_networkconfig_priv] = ACTIONS(2025),
    [sym_programport_priv] = ACTIONS(2025),
    [sym_remoteui_priv] = ACTIONS(2025),
    [sym_securitycontrol_priv] = ACTIONS(2025),
    [sym_softwaremanagement_priv] = ACTIONS(2025),
    [sym_terminal_priv] = ACTIONS(2025),
    [sym_tpadmin_priv] = ACTIONS(2025),
    [sym_user1_priv] = ACTIONS(2025),
    [sym_user2_priv] = ACTIONS(2025),
    [sym_user3_priv] = ACTIONS(2025),
    [sym_user4_priv] = ACTIONS(2025),
    [sym_usermanagement_priv] = ACTIONS(2025),
    [sym_valid_account] = ACTIONS(2025),
    [sym_err_name_invalid] = ACTIONS(2025),
    [sym_err_password_invalid] = ACTIONS(2025),
    [sym_err_invalid_account] = ACTIONS(2025),
    [sym_err_no_authorization] = ACTIONS(2025),
    [sym_err_invalid_parameter] = ACTIONS(2025),
    [sym_err_user_locked_out] = ACTIONS(2025),
    [sym_err_user_account_expired] = ACTIONS(2025),
    [sym_err_general_error] = ACTIONS(2025),
    [sym_err_authentication_error] = ACTIONS(2025),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2025),
    [sym_tls_validate_certificate] = ACTIONS(2025),
    [sym_tdata] = ACTIONS(2025),
    [sym_tchannel] = ACTIONS(2025),
    [sym_tlevel] = ACTIONS(2025),
    [sym_tbutton] = ACTIONS(2025),
    [sym_ttimeline] = ACTIONS(2025),
    [sym_tcustom] = ACTIONS(2025),
    [sym_url_struct] = ACTIONS(2025),
    [sym_dns_struct] = ACTIONS(2025),
    [sym_ip_address_struct] = ACTIONS(2025),
    [sym_dev_info_struct] = ACTIONS(2025),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2025),
    [sym_data_feed] = ACTIONS(2025),
    [sym_data_field] = ACTIONS(2025),
    [sym_data_record] = ACTIONS(2025),
    [sym_wc_data_feed] = ACTIONS(2025),
    [sym_wc_data_field] = ACTIONS(2025),
    [sym_wc_data_record] = ACTIONS(2025),
    [sym_last_login_info] = ACTIONS(2025),
    [sym____reserved___] = ACTIONS(2025),
    [sym_date] = ACTIONS(2025),
    [sym_day] = ACTIONS(2025),
    [sym_time] = ACTIONS(2025),
    [sym_ldate] = ACTIONS(2025),
    [sym_push_device] = ACTIONS(2025),
    [sym_push_channel] = ACTIONS(2025),
    [sym_push_devchan] = ACTIONS(2025),
    [sym_release_device] = ACTIONS(2025),
    [sym_release_channel] = ACTIONS(2025),
    [sym_release_devchan] = ACTIONS(2025),
    [sym_master_slot] = ACTIONS(2025),
    [sym_get_pulse_time] = ACTIONS(2025),
    [sym_get_timer] = ACTIONS(2025),
    [sym_master_sn] = ACTIONS(2025),
    [sym_system_number] = ACTIONS(2025),
    [sym_button] = ACTIONS(2025),
    [sym_dv_channel] = ACTIONS(2025),
    [sym_data] = ACTIONS(2025),
    [sym_level] = ACTIONS(2025),
    [sym_timeline] = ACTIONS(2025),
    [sym_channel] = ACTIONS(2025),
    [sym_custom] = ACTIONS(2025),
    [sym_atoi] = ACTIONS(2025),
    [sym_atol] = ACTIONS(2025),
    [sym_atof] = ACTIONS(2025),
    [sym_itoa] = ACTIONS(2025),
    [sym_itohex] = ACTIONS(2025),
    [sym_hextoi] = ACTIONS(2025),
    [sym_ftoa] = ACTIONS(2025),
    [sym_find_string] = ACTIONS(2025),
    [sym_left_string] = ACTIONS(2025),
    [sym_right_string] = ACTIONS(2025),
    [sym_mid_string] = ACTIONS(2025),
    [sym_remove_string] = ACTIONS(2025),
    [sym_get_buffer_string] = ACTIONS(2025),
    [sym_get_buffer_char] = ACTIONS(2025),
    [sym_get_multi_buffer_string] = ACTIONS(2025),
    [sym_length_string] = ACTIONS(2025),
    [sym_max_length_string] = ACTIONS(2025),
    [sym_set_length_string] = ACTIONS(2025),
    [sym_compare_string] = ACTIONS(2025),
    [sym_upper_string] = ACTIONS(2025),
    [sym_lower_string] = ACTIONS(2025),
    [sym_get_last] = ACTIONS(2025),
    [sym_date_to_day] = ACTIONS(2025),
    [sym_date_to_month] = ACTIONS(2025),
    [sym_date_to_year] = ACTIONS(2025),
    [sym_time_to_hour] = ACTIONS(2025),
    [sym_time_to_minute] = ACTIONS(2025),
    [sym_time_to_second] = ACTIONS(2025),
    [sym_day_of_week] = ACTIONS(2025),
    [sym_set_pulse_time] = ACTIONS(2025),
    [sym_set_timer] = ACTIONS(2025),
    [sym_file_open] = ACTIONS(2025),
    [sym_file_close] = ACTIONS(2025),
    [sym_file_read] = ACTIONS(2025),
    [sym_file_read_line] = ACTIONS(2025),
    [sym_file_write] = ACTIONS(2025),
    [sym_file_write_line] = ACTIONS(2025),
    [sym_file_copy] = ACTIONS(2025),
    [sym_file_delete] = ACTIONS(2025),
    [sym_file_rename] = ACTIONS(2025),
    [sym_file_dir] = ACTIONS(2025),
    [sym_file_getdir] = ACTIONS(2025),
    [sym_file_setdir] = ACTIONS(2025),
    [sym_file_create_dir] = ACTIONS(2025),
    [sym_file_remove_dir] = ACTIONS(2025),
    [sym_file_seek] = ACTIONS(2025),
    [sym_length_array] = ACTIONS(2025),
    [sym_max_length_array] = ACTIONS(2025),
    [sym_set_length_array] = ACTIONS(2025),
    [sym_ip_client_open] = ACTIONS(2025),
    [sym_ip_client_close] = ACTIONS(2025),
    [sym_ip_server_open] = ACTIONS(2025),
    [sym_ip_server_close] = ACTIONS(2025),
    [sym_ip_bound_client_open] = ACTIONS(2025),
    [sym_ip_mc_server_open] = ACTIONS(2025),
    [sym_ip_set_option] = ACTIONS(2025),
    [sym_ssh_client_open] = ACTIONS(2025),
    [sym_ssh_client_close] = ACTIONS(2025),
    [sym_tls_client_open] = ACTIONS(2025),
    [sym_tls_client_close] = ACTIONS(2025),
    [sym_do_push] = ACTIONS(2025),
    [sym_do_push_timed] = ACTIONS(2025),
    [sym_do_release] = ACTIONS(2025),
    [sym_device_id] = ACTIONS(2025),
    [sym_device_info] = ACTIONS(2025),
    [sym_device_id_string] = ACTIONS(2025),
    [sym_device_standby] = ACTIONS(2025),
    [sym_device_wake] = ACTIONS(2025),
    [sym_timeline_create] = ACTIONS(2025),
    [sym_timeline_kill] = ACTIONS(2025),
    [sym_timeline_reload] = ACTIONS(2025),
    [sym_timeline_pause] = ACTIONS(2025),
    [sym_timeline_restart] = ACTIONS(2025),
    [sym_timeline_set] = ACTIONS(2025),
    [sym_timeline_get] = ACTIONS(2025),
    [sym_timeline_active] = ACTIONS(2025),
    [sym_timeline_dynamic_id] = ACTIONS(2025),
    [sym_variable_to_string] = ACTIONS(2025),
    [sym_string_to_variable] = ACTIONS(2025),
    [sym_length_variable_to_string] = ACTIONS(2025),
    [sym_type_cast] = ACTIONS(2025),
    [sym_variable_to_xml] = ACTIONS(2025),
    [sym_xml_to_variable] = ACTIONS(2025),
    [sym_length_variable_to_xml] = ACTIONS(2025),
    [sym_redirect_string] = ACTIONS(2025),
    [sym_random_number] = ACTIONS(2025),
    [sym_get_unique_id] = ACTIONS(2025),
    [sym_get_system_number] = ACTIONS(2025),
    [sym_set_system_number] = ACTIONS(2025),
    [sym_get_serial_number] = ACTIONS(2025),
    [sym_get_master_build] = ACTIONS(2025),
    [sym_set_validation_code] = ACTIONS(2025),
    [sym_reboot] = ACTIONS(2025),
    [sym_load_duet_module] = ACTIONS(2025),
    [sym_unload_duet_module] = ACTIONS(2025),
    [sym_duet_mem_size_set] = ACTIONS(2025),
    [sym_duet_mem_size_get] = ACTIONS(2025),
    [sym_dynamic_application_device] = ACTIONS(2025),
    [sym_dynamic_polled_port] = ACTIONS(2025),
    [sym_static_port_binding] = ACTIONS(2025),
    [sym_static_ip_binding] = ACTIONS(2025),
    [sym_get_dns_list] = ACTIONS(2025),
    [sym_set_dns_list] = ACTIONS(2025),
    [sym_get_ip_address] = ACTIONS(2025),
    [sym_set_ip_address] = ACTIONS(2025),
    [sym_get_url_list] = ACTIONS(2025),
    [sym_add_url_entry] = ACTIONS(2025),
    [sym_delete_url_entry] = ACTIONS(2025),
    [sym_abs_value] = ACTIONS(2025),
    [sym_max_value] = ACTIONS(2025),
    [sym_min_value] = ACTIONS(2025),
    [sym_exp_value] = ACTIONS(2025),
    [sym_log_value] = ACTIONS(2025),
    [sym_log10_value] = ACTIONS(2025),
    [sym_power_value] = ACTIONS(2025),
    [sym_sqrt_value] = ACTIONS(2025),
    [sym_raw_be] = ACTIONS(2025),
    [sym_raw_le] = ACTIONS(2025),
    [sym_internal_queue_size_set] = ACTIONS(2025),
    [sym_internal_queue_size_get] = ACTIONS(2025),
    [sym_internal_threshold_set] = ACTIONS(2025),
    [sym_internal_threshold_get] = ACTIONS(2025),
    [sym_rebuilt_event] = ACTIONS(2025),
    [sym_do_custom_event] = ACTIONS(2025),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2025),
    [sym_clkmgr_set_clk_source] = ACTIONS(2025),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2025),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2025),
    [sym_clkmgr_get_timezone] = ACTIONS(2025),
    [sym_clkmgr_set_timezone] = ACTIONS(2025),
    [sym_clkmgr_get_resync_period] = ACTIONS(2025),
    [sym_clkmgr_set_resync_period] = ACTIONS(2025),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2025),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2025),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2025),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2025),
    [sym_clkmgr_get_timeservers] = ACTIONS(2025),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2025),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2025),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2025),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2025),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2025),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2025),
    [sym_set_log_level] = ACTIONS(2025),
    [sym_get_log_level] = ACTIONS(2025),
    [sym_amx_log] = ACTIONS(2025),
    [sym_smtp_server_config_set] = ACTIONS(2025),
    [sym_smtp_server_config_get] = ACTIONS(2025),
    [sym_smtp_send] = ACTIONS(2025),
    [sym_validate_netlinx_account] = ACTIONS(2025),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2025),
    [sym_authenticate_certificate] = ACTIONS(2025),
    [sym_audit_netlinx_session_event] = ACTIONS(2025),
    [sym_audit_netlinx_generic_event] = ACTIONS(2025),
    [sym_get_available_flash_disk_space] = ACTIONS(2025),
    [sym_get_max_flash_disk_space] = ACTIONS(2025),
    [sym_data_create_feed] = ACTIONS(2025),
    [sym_data_delete_feed] = ACTIONS(2025),
    [sym_data_publish_feed] = ACTIONS(2025),
    [sym_data_get_published_feed] = ACTIONS(2025),
    [sym_data_add_record] = ACTIONS(2025),
    [sym_data_get_event_record] = ACTIONS(2025),
    [sym__wc_data_create_feed] = ACTIONS(2025),
    [sym__wc_data_add_record] = ACTIONS(2025),
    [sym__wc_data_get_event_record] = ACTIONS(2025),
    [sym_format] = ACTIONS(2025),
    [sym_astro_clock] = ACTIONS(2025),
    [sym_set_outdoor_temperature] = ACTIONS(2025),
    [sym_set_virtual_level_count] = ACTIONS(2025),
    [sym_set_virtual_channel_count] = ACTIONS(2025),
    [sym_set_virtual_port_count] = ACTIONS(2025),
    [sym__wc_encode_int] = ACTIONS(2025),
    [sym__wc_decode_int] = ACTIONS(2025),
    [sym__wc_int] = ACTIONS(2025),
    [sym__wc_to_ch_int] = ACTIONS(2025),
    [sym__ch_to_wc_int] = ACTIONS(2025),
    [sym__wc_find_string_int] = ACTIONS(2025),
    [sym__wc_left_string_int] = ACTIONS(2025),
    [sym__wc_lower_string_int] = ACTIONS(2025),
    [sym__wc_mid_string_int] = ACTIONS(2025),
    [sym__wc_remove_string_int] = ACTIONS(2025),
    [sym__wc_right_string_int] = ACTIONS(2025),
    [sym__wc_upper_string_int] = ACTIONS(2025),
    [sym__wc_compare_string_int] = ACTIONS(2025),
    [sym__wc_get_buffer_char_int] = ACTIONS(2025),
    [sym__wc_get_buffer_string_int] = ACTIONS(2025),
    [sym__wc_concat_string_int] = ACTIONS(2025),
    [sym__wc_file_open_int] = ACTIONS(2025),
    [sym__wc_file_close_int] = ACTIONS(2025),
    [sym__wc_file_read_int] = ACTIONS(2025),
    [sym__wc_file_read_line_int] = ACTIONS(2025),
    [sym__wc_file_write_int] = ACTIONS(2025),
    [sym__wc_file_write_line_int] = ACTIONS(2025),
    [aux_sym_preproc_include_token1] = ACTIONS(2025),
    [aux_sym_preproc_define_token1] = ACTIONS(2027),
    [aux_sym_preproc_warn_token1] = ACTIONS(2027),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2027),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2027),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2027),
    [anon_sym_LBRACK] = ACTIONS(2027),
    [anon_sym_LBRACE] = ACTIONS(2027),
    [anon_sym_RBRACE] = ACTIONS(2027),
    [anon_sym_LPAREN] = ACTIONS(2025),
    [anon_sym_BANG] = ACTIONS(2027),
    [anon_sym_TILDE] = ACTIONS(2027),
    [anon_sym_DASH] = ACTIONS(2025),
    [anon_sym_PLUS] = ACTIONS(2025),
    [anon_sym_DASH_DASH] = ACTIONS(2027),
    [anon_sym_PLUS_PLUS] = ACTIONS(2027),
    [anon_sym_DQUOTE] = ACTIONS(2027),
    [anon_sym_SQUOTE] = ACTIONS(2027),
    [sym_number_literal] = ACTIONS(2027),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(570)] = {
    [sym_expression] = STATE(1141),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(1242),
    [sym_and] = ACTIONS(1245),
    [sym_or] = ACTIONS(1245),
    [sym_not] = ACTIONS(1247),
    [sym_xor] = ACTIONS(1245),
    [sym_band] = ACTIONS(1245),
    [sym_bor] = ACTIONS(1245),
    [sym_bxor] = ACTIONS(1245),
    [sym_bnot] = ACTIONS(1247),
    [sym_lshift] = ACTIONS(1245),
    [sym_rshift] = ACTIONS(1245),
    [sym___file__] = ACTIONS(1250),
    [sym___date__] = ACTIONS(1250),
    [sym___ldate__] = ACTIONS(1250),
    [sym___time__] = ACTIONS(1250),
    [sym___line__] = ACTIONS(1250),
    [sym___name__] = ACTIONS(1250),
    [sym___netlinx__] = ACTIONS(1253),
    [sym_netlinx_axi_version] = ACTIONS(1253),
    [sym_url_flg_tcp] = ACTIONS(1253),
    [sym_url_flg_accinfopresent] = ACTIONS(1253),
    [sym_url_flg_temp] = ACTIONS(1253),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1253),
    [sym_url_flg_stat_mask] = ACTIONS(1253),
    [sym_url_flg_stat_lookup] = ACTIONS(1253),
    [sym_url_flg_stat_connecting] = ACTIONS(1253),
    [sym_url_flg_stat_waiting] = ACTIONS(1253),
    [sym_url_flg_stat_connected] = ACTIONS(1253),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1253),
    [sym_first_virtual_device] = ACTIONS(1253),
    [sym_first_local_port] = ACTIONS(1253),
    [sym_dynamic_virtual_device] = ACTIONS(1253),
    [sym_timeline_once] = ACTIONS(1253),
    [sym_timeline_repeat] = ACTIONS(1253),
    [sym_timeline_absolute] = ACTIONS(1253),
    [sym_timeline_relative] = ACTIONS(1253),
    [sym_do_push_timed_infinite] = ACTIONS(1253),
    [sym_source_type_no_address] = ACTIONS(1253),
    [sym_source_type_neuron_id] = ACTIONS(1253),
    [sym_source_type_ip_address] = ACTIONS(1253),
    [sym_source_type_axlink] = ACTIONS(1253),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1253),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1253),
    [sym_source_type_ip_socket_address] = ACTIONS(1253),
    [sym_source_type_rs232] = ACTIONS(1253),
    [sym_source_type_internal] = ACTIONS(1253),
    [sym_source_type_mac_address] = ACTIONS(1253),
    [sym_source_type_ipv4_port] = ACTIONS(1253),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1253),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1253),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1253),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1253),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1253),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1253),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1253),
    [sym_true] = ACTIONS(1253),
    [sym_false] = ACTIONS(1253),
    [sym_file_read_only] = ACTIONS(1253),
    [sym_file_rw_new] = ACTIONS(1253),
    [sym_file_rw_append] = ACTIONS(1253),
    [sym_ip_tcp] = ACTIONS(1253),
    [sym_ip_udp] = ACTIONS(1253),
    [sym_ip_udp_2way] = ACTIONS(1253),
    [sym_xml_encode_types] = ACTIONS(1253),
    [sym_xml_encode_char_as_list] = ACTIONS(1253),
    [sym_xml_encode_le] = ACTIONS(1253),
    [sym_xml_decode_types] = ACTIONS(1253),
    [sym_xml_decode_no_preserve] = ACTIONS(1253),
    [sym_duet_dev_type_display_device] = ACTIONS(1253),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1253),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1253),
    [sym_duet_dev_type_io_device] = ACTIONS(1253),
    [sym_duet_dev_type_relay_device] = ACTIONS(1253),
    [sym_duet_dev_type_ups] = ACTIONS(1253),
    [sym_duet_dev_type_amplifier] = ACTIONS(1253),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1253),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1253),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1253),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1253),
    [sym_duet_dev_type_camera] = ACTIONS(1253),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1253),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1253),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1253),
    [sym_duet_dev_type_dss] = ACTIONS(1253),
    [sym_duet_dev_type_dvr] = ACTIONS(1253),
    [sym_duet_dev_type_disc_device] = ACTIONS(1253),
    [sym_duet_dev_type_document_camera] = ACTIONS(1253),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1253),
    [sym_duet_dev_type_hvac] = ACTIONS(1253),
    [sym_duet_dev_type_keypad] = ACTIONS(1253),
    [sym_duet_dev_type_light] = ACTIONS(1253),
    [sym_duet_dev_type_monitor] = ACTIONS(1253),
    [sym_duet_dev_type_motor] = ACTIONS(1253),
    [sym_duet_dev_type_multi_window] = ACTIONS(1253),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1253),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1253),
    [sym_duet_dev_type_receiver] = ACTIONS(1253),
    [sym_duet_dev_type_security_system] = ACTIONS(1253),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1253),
    [sym_duet_dev_type_settop_box] = ACTIONS(1253),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1253),
    [sym_duet_dev_type_switcher] = ACTIONS(1253),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1253),
    [sym_duet_dev_type_tv] = ACTIONS(1253),
    [sym_duet_dev_type_utility] = ACTIONS(1253),
    [sym_duet_dev_type_vcr] = ACTIONS(1253),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1253),
    [sym_duet_dev_type_video_processor] = ACTIONS(1253),
    [sym_duet_dev_type_video_projector] = ACTIONS(1253),
    [sym_duet_dev_type_video_wall] = ACTIONS(1253),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1253),
    [sym_duet_dev_type_weather] = ACTIONS(1253),
    [sym_duet_dev_not_polled] = ACTIONS(1253),
    [sym_duet_dev_polled] = ACTIONS(1253),
    [sym_clkmgr_mode_network] = ACTIONS(1253),
    [sym_clkmgr_mode_standalone] = ACTIONS(1253),
    [sym_normal_standby] = ACTIONS(1253),
    [sym_normal_wake] = ACTIONS(1253),
    [sym_ip_multicast_ttl_option] = ACTIONS(1253),
    [sym_ip_ttl_subnet] = ACTIONS(1253),
    [sym_ip_ttl_site] = ACTIONS(1253),
    [sym_ip_ttl_region] = ACTIONS(1253),
    [sym_ip_ttl_continent] = ACTIONS(1253),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1253),
    [sym_ip_nodelay_on] = ACTIONS(1253),
    [sym_ip_nodelay_off] = ACTIONS(1253),
    [sym_amx_error] = ACTIONS(1253),
    [sym_amx_warning] = ACTIONS(1253),
    [sym_amx_info] = ACTIONS(1253),
    [sym_amx_debug] = ACTIONS(1253),
    [sym_netlinx_login_success] = ACTIONS(1253),
    [sym_netlinx_login_fail] = ACTIONS(1253),
    [sym_netlinx_logout] = ACTIONS(1253),
    [sym_null_str] = ACTIONS(1253),
    [sym_smtp_address] = ACTIONS(1253),
    [sym_smtp_port_number] = ACTIONS(1253),
    [sym_smtp_username] = ACTIONS(1253),
    [sym_smtp_password] = ACTIONS(1253),
    [sym_smtp_require_tls] = ACTIONS(1253),
    [sym_smtp_from] = ACTIONS(1253),
    [sym_smtp_tls_true] = ACTIONS(1253),
    [sym_smtp_tls_false] = ACTIONS(1253),
    [sym_listview_on_row_select_event] = ACTIONS(1253),
    [sym_data_structure_unknown] = ACTIONS(1253),
    [sym_data_structure_datafeed] = ACTIONS(1253),
    [sym_data_structure_datarecordset] = ACTIONS(1253),
    [sym_data_structure_datarecord] = ACTIONS(1253),
    [sym_data_structure_datafield] = ACTIONS(1253),
    [sym_data_max_content_field_count] = ACTIONS(1253),
    [sym_data_max_metadata_field_count] = ACTIONS(1253),
    [sym_data_max_id_length] = ACTIONS(1253),
    [sym_data_max_name_length] = ACTIONS(1253),
    [sym_data_max_description_length] = ACTIONS(1253),
    [sym_data_max_type_length] = ACTIONS(1253),
    [sym_data_max_format_length] = ACTIONS(1253),
    [sym_data_max_source_length] = ACTIONS(1253),
    [sym_data_max_value_length] = ACTIONS(1253),
    [sym_data_max_label_length] = ACTIONS(1253),
    [sym_data_max_url_length] = ACTIONS(1253),
    [sym_max_last_login_info_length] = ACTIONS(1253),
    [sym_data_type_unknown] = ACTIONS(1253),
    [sym_data_type_string] = ACTIONS(1253),
    [sym_data_type_datetime] = ACTIONS(1253),
    [sym_data_type_date] = ACTIONS(1253),
    [sym_data_type_time] = ACTIONS(1253),
    [sym_data_type_image] = ACTIONS(1253),
    [sym_data_format_url] = ACTIONS(1253),
    [sym_data_format_phone] = ACTIONS(1253),
    [sym_data_format_email] = ACTIONS(1253),
    [sym_data_format_iso8601] = ACTIONS(1253),
    [sym_auditlog_priv] = ACTIONS(1253),
    [sym_configuration_priv] = ACTIONS(1253),
    [sym_deviceconfig_priv] = ACTIONS(1253),
    [sym_ftp_priv] = ACTIONS(1253),
    [sym_http_priv] = ACTIONS(1253),
    [sym_networkconfig_priv] = ACTIONS(1253),
    [sym_programport_priv] = ACTIONS(1253),
    [sym_remoteui_priv] = ACTIONS(1253),
    [sym_securitycontrol_priv] = ACTIONS(1253),
    [sym_softwaremanagement_priv] = ACTIONS(1253),
    [sym_terminal_priv] = ACTIONS(1253),
    [sym_tpadmin_priv] = ACTIONS(1253),
    [sym_user1_priv] = ACTIONS(1253),
    [sym_user2_priv] = ACTIONS(1253),
    [sym_user3_priv] = ACTIONS(1253),
    [sym_user4_priv] = ACTIONS(1253),
    [sym_usermanagement_priv] = ACTIONS(1253),
    [sym_valid_account] = ACTIONS(1253),
    [sym_err_name_invalid] = ACTIONS(1253),
    [sym_err_password_invalid] = ACTIONS(1253),
    [sym_err_invalid_account] = ACTIONS(1253),
    [sym_err_no_authorization] = ACTIONS(1253),
    [sym_err_invalid_parameter] = ACTIONS(1253),
    [sym_err_user_locked_out] = ACTIONS(1253),
    [sym_err_user_account_expired] = ACTIONS(1253),
    [sym_err_general_error] = ACTIONS(1253),
    [sym_err_authentication_error] = ACTIONS(1253),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1253),
    [sym_tls_validate_certificate] = ACTIONS(1253),
    [sym____reserved___] = ACTIONS(1256),
    [sym_date] = ACTIONS(1256),
    [sym_day] = ACTIONS(1256),
    [sym_time] = ACTIONS(1256),
    [sym_ldate] = ACTIONS(1256),
    [sym_push_device] = ACTIONS(1256),
    [sym_push_channel] = ACTIONS(1256),
    [sym_push_devchan] = ACTIONS(1256),
    [sym_release_device] = ACTIONS(1256),
    [sym_release_channel] = ACTIONS(1256),
    [sym_release_devchan] = ACTIONS(1256),
    [sym_master_slot] = ACTIONS(1256),
    [sym_get_pulse_time] = ACTIONS(1256),
    [sym_get_timer] = ACTIONS(1256),
    [sym_master_sn] = ACTIONS(1256),
    [sym_system_number] = ACTIONS(1256),
    [sym_button] = ACTIONS(1256),
    [sym_dv_channel] = ACTIONS(1256),
    [sym_data] = ACTIONS(1256),
    [sym_level] = ACTIONS(1256),
    [sym_timeline] = ACTIONS(1256),
    [sym_channel] = ACTIONS(1256),
    [sym_custom] = ACTIONS(1256),
    [sym_atoi] = ACTIONS(1259),
    [sym_atol] = ACTIONS(1259),
    [sym_atof] = ACTIONS(1259),
    [sym_itoa] = ACTIONS(1259),
    [sym_itohex] = ACTIONS(1259),
    [sym_hextoi] = ACTIONS(1259),
    [sym_ftoa] = ACTIONS(1259),
    [sym_find_string] = ACTIONS(1259),
    [sym_left_string] = ACTIONS(1259),
    [sym_right_string] = ACTIONS(1259),
    [sym_mid_string] = ACTIONS(1259),
    [sym_remove_string] = ACTIONS(1259),
    [sym_get_buffer_string] = ACTIONS(1259),
    [sym_get_buffer_char] = ACTIONS(1259),
    [sym_get_multi_buffer_string] = ACTIONS(1259),
    [sym_length_string] = ACTIONS(1259),
    [sym_max_length_string] = ACTIONS(1259),
    [sym_set_length_string] = ACTIONS(1259),
    [sym_compare_string] = ACTIONS(1259),
    [sym_upper_string] = ACTIONS(1259),
    [sym_lower_string] = ACTIONS(1259),
    [sym_get_last] = ACTIONS(1259),
    [sym_date_to_day] = ACTIONS(1259),
    [sym_date_to_month] = ACTIONS(1259),
    [sym_date_to_year] = ACTIONS(1259),
    [sym_time_to_hour] = ACTIONS(1259),
    [sym_time_to_minute] = ACTIONS(1259),
    [sym_time_to_second] = ACTIONS(1259),
    [sym_day_of_week] = ACTIONS(1259),
    [sym_set_pulse_time] = ACTIONS(1259),
    [sym_set_timer] = ACTIONS(1259),
    [sym_file_open] = ACTIONS(1259),
    [sym_file_close] = ACTIONS(1259),
    [sym_file_read] = ACTIONS(1259),
    [sym_file_read_line] = ACTIONS(1259),
    [sym_file_write] = ACTIONS(1259),
    [sym_file_write_line] = ACTIONS(1259),
    [sym_file_copy] = ACTIONS(1259),
    [sym_file_delete] = ACTIONS(1259),
    [sym_file_rename] = ACTIONS(1259),
    [sym_file_dir] = ACTIONS(1259),
    [sym_file_getdir] = ACTIONS(1259),
    [sym_file_setdir] = ACTIONS(1259),
    [sym_file_create_dir] = ACTIONS(1259),
    [sym_file_remove_dir] = ACTIONS(1259),
    [sym_file_seek] = ACTIONS(1259),
    [sym_length_array] = ACTIONS(1259),
    [sym_max_length_array] = ACTIONS(1259),
    [sym_set_length_array] = ACTIONS(1259),
    [sym_ip_client_open] = ACTIONS(1259),
    [sym_ip_client_close] = ACTIONS(1259),
    [sym_ip_server_open] = ACTIONS(1259),
    [sym_ip_server_close] = ACTIONS(1259),
    [sym_ip_bound_client_open] = ACTIONS(1259),
    [sym_ip_mc_server_open] = ACTIONS(1259),
    [sym_ip_set_option] = ACTIONS(1259),
    [sym_ssh_client_open] = ACTIONS(1259),
    [sym_ssh_client_close] = ACTIONS(1259),
    [sym_tls_client_open] = ACTIONS(1259),
    [sym_tls_client_close] = ACTIONS(1259),
    [sym_do_push] = ACTIONS(1259),
    [sym_do_push_timed] = ACTIONS(1259),
    [sym_do_release] = ACTIONS(1259),
    [sym_device_id] = ACTIONS(1259),
    [sym_device_info] = ACTIONS(1259),
    [sym_device_id_string] = ACTIONS(1259),
    [sym_device_standby] = ACTIONS(1259),
    [sym_device_wake] = ACTIONS(1259),
    [sym_timeline_create] = ACTIONS(1259),
    [sym_timeline_kill] = ACTIONS(1259),
    [sym_timeline_reload] = ACTIONS(1259),
    [sym_timeline_pause] = ACTIONS(1259),
    [sym_timeline_restart] = ACTIONS(1259),
    [sym_timeline_set] = ACTIONS(1259),
    [sym_timeline_get] = ACTIONS(1259),
    [sym_timeline_active] = ACTIONS(1259),
    [sym_timeline_dynamic_id] = ACTIONS(1259),
    [sym_variable_to_string] = ACTIONS(1259),
    [sym_string_to_variable] = ACTIONS(1259),
    [sym_length_variable_to_string] = ACTIONS(1259),
    [sym_type_cast] = ACTIONS(1259),
    [sym_variable_to_xml] = ACTIONS(1259),
    [sym_xml_to_variable] = ACTIONS(1259),
    [sym_length_variable_to_xml] = ACTIONS(1259),
    [sym_redirect_string] = ACTIONS(1259),
    [sym_random_number] = ACTIONS(1259),
    [sym_get_unique_id] = ACTIONS(1259),
    [sym_get_system_number] = ACTIONS(1259),
    [sym_set_system_number] = ACTIONS(1259),
    [sym_get_serial_number] = ACTIONS(1259),
    [sym_get_master_build] = ACTIONS(1259),
    [sym_set_validation_code] = ACTIONS(1259),
    [sym_reboot] = ACTIONS(1259),
    [sym_load_duet_module] = ACTIONS(1259),
    [sym_unload_duet_module] = ACTIONS(1259),
    [sym_duet_mem_size_set] = ACTIONS(1259),
    [sym_duet_mem_size_get] = ACTIONS(1259),
    [sym_dynamic_application_device] = ACTIONS(1259),
    [sym_dynamic_polled_port] = ACTIONS(1259),
    [sym_static_port_binding] = ACTIONS(1259),
    [sym_static_ip_binding] = ACTIONS(1259),
    [sym_get_dns_list] = ACTIONS(1259),
    [sym_set_dns_list] = ACTIONS(1259),
    [sym_get_ip_address] = ACTIONS(1259),
    [sym_set_ip_address] = ACTIONS(1259),
    [sym_get_url_list] = ACTIONS(1259),
    [sym_add_url_entry] = ACTIONS(1259),
    [sym_delete_url_entry] = ACTIONS(1259),
    [sym_abs_value] = ACTIONS(1259),
    [sym_max_value] = ACTIONS(1259),
    [sym_min_value] = ACTIONS(1259),
    [sym_exp_value] = ACTIONS(1259),
    [sym_log_value] = ACTIONS(1259),
    [sym_log10_value] = ACTIONS(1259),
    [sym_power_value] = ACTIONS(1259),
    [sym_sqrt_value] = ACTIONS(1259),
    [sym_raw_be] = ACTIONS(1259),
    [sym_raw_le] = ACTIONS(1259),
    [sym_internal_queue_size_set] = ACTIONS(1259),
    [sym_internal_queue_size_get] = ACTIONS(1259),
    [sym_internal_threshold_set] = ACTIONS(1259),
    [sym_internal_threshold_get] = ACTIONS(1259),
    [sym_rebuilt_event] = ACTIONS(1259),
    [sym_do_custom_event] = ACTIONS(1259),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1259),
    [sym_clkmgr_set_clk_source] = ACTIONS(1259),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1259),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1259),
    [sym_clkmgr_get_timezone] = ACTIONS(1259),
    [sym_clkmgr_set_timezone] = ACTIONS(1259),
    [sym_clkmgr_get_resync_period] = ACTIONS(1259),
    [sym_clkmgr_set_resync_period] = ACTIONS(1259),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1259),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1259),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1259),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1259),
    [sym_clkmgr_get_timeservers] = ACTIONS(1259),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1259),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1259),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1259),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1259),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1259),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1259),
    [sym_set_log_level] = ACTIONS(1259),
    [sym_get_log_level] = ACTIONS(1259),
    [sym_amx_log] = ACTIONS(1259),
    [sym_smtp_server_config_set] = ACTIONS(1259),
    [sym_smtp_server_config_get] = ACTIONS(1259),
    [sym_smtp_send] = ACTIONS(1259),
    [sym_validate_netlinx_account] = ACTIONS(1259),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1259),
    [sym_authenticate_certificate] = ACTIONS(1259),
    [sym_audit_netlinx_session_event] = ACTIONS(1259),
    [sym_audit_netlinx_generic_event] = ACTIONS(1259),
    [sym_get_available_flash_disk_space] = ACTIONS(1259),
    [sym_get_max_flash_disk_space] = ACTIONS(1259),
    [sym_data_create_feed] = ACTIONS(1259),
    [sym_data_delete_feed] = ACTIONS(1259),
    [sym_data_publish_feed] = ACTIONS(1259),
    [sym_data_get_published_feed] = ACTIONS(1259),
    [sym_data_add_record] = ACTIONS(1259),
    [sym_data_get_event_record] = ACTIONS(1259),
    [sym__wc_data_create_feed] = ACTIONS(1259),
    [sym__wc_data_add_record] = ACTIONS(1259),
    [sym__wc_data_get_event_record] = ACTIONS(1259),
    [sym_format] = ACTIONS(1259),
    [sym_astro_clock] = ACTIONS(1259),
    [sym_set_outdoor_temperature] = ACTIONS(1259),
    [sym_set_virtual_level_count] = ACTIONS(1259),
    [sym_set_virtual_channel_count] = ACTIONS(1259),
    [sym_set_virtual_port_count] = ACTIONS(1259),
    [sym__wc_encode_int] = ACTIONS(1259),
    [sym__wc_decode_int] = ACTIONS(1259),
    [sym__wc_int] = ACTIONS(1259),
    [sym__wc_to_ch_int] = ACTIONS(1259),
    [sym__ch_to_wc_int] = ACTIONS(1259),
    [sym__wc_find_string_int] = ACTIONS(1259),
    [sym__wc_left_string_int] = ACTIONS(1259),
    [sym__wc_lower_string_int] = ACTIONS(1259),
    [sym__wc_mid_string_int] = ACTIONS(1259),
    [sym__wc_remove_string_int] = ACTIONS(1259),
    [sym__wc_right_string_int] = ACTIONS(1259),
    [sym__wc_upper_string_int] = ACTIONS(1259),
    [sym__wc_compare_string_int] = ACTIONS(1259),
    [sym__wc_get_buffer_char_int] = ACTIONS(1259),
    [sym__wc_get_buffer_string_int] = ACTIONS(1259),
    [sym__wc_concat_string_int] = ACTIONS(1259),
    [sym__wc_file_open_int] = ACTIONS(1259),
    [sym__wc_file_close_int] = ACTIONS(1259),
    [sym__wc_file_read_int] = ACTIONS(1259),
    [sym__wc_file_read_line_int] = ACTIONS(1259),
    [sym__wc_file_write_int] = ACTIONS(1259),
    [sym__wc_file_write_line_int] = ACTIONS(1259),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1262),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1262),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1262),
    [aux_sym_preproc_else_token1] = ACTIONS(1262),
    [anon_sym_COMMA] = ACTIONS(1262),
    [anon_sym_LBRACK] = ACTIONS(1264),
    [anon_sym_LBRACE] = ACTIONS(1262),
    [anon_sym_RBRACE] = ACTIONS(1262),
    [anon_sym_COLON] = ACTIONS(1262),
    [anon_sym_LPAREN] = ACTIONS(1267),
    [anon_sym_BANG] = ACTIONS(1247),
    [anon_sym_TILDE] = ACTIONS(1270),
    [anon_sym_DASH] = ACTIONS(1247),
    [anon_sym_PLUS] = ACTIONS(1247),
    [anon_sym_STAR] = ACTIONS(1262),
    [anon_sym_SLASH] = ACTIONS(1245),
    [anon_sym_PERCENT] = ACTIONS(1262),
    [anon_sym_PIPE_PIPE] = ACTIONS(1262),
    [anon_sym_AMP_AMP] = ACTIONS(1262),
    [anon_sym_PIPE] = ACTIONS(1245),
    [anon_sym_CARET] = ACTIONS(1262),
    [anon_sym_AMP] = ACTIONS(1245),
    [anon_sym_EQ_EQ] = ACTIONS(1262),
    [anon_sym_BANG_EQ] = ACTIONS(1262),
    [anon_sym_LT_GT] = ACTIONS(1262),
    [anon_sym_GT] = ACTIONS(1245),
    [anon_sym_GT_EQ] = ACTIONS(1262),
    [anon_sym_LT_EQ] = ACTIONS(1262),
    [anon_sym_LT] = ACTIONS(1245),
    [anon_sym_LT_LT] = ACTIONS(1262),
    [anon_sym_GT_GT] = ACTIONS(1262),
    [anon_sym_DASH_DASH] = ACTIONS(1273),
    [anon_sym_PLUS_PLUS] = ACTIONS(1273),
    [anon_sym_DOT] = ACTIONS(1245),
    [anon_sym_DQUOTE] = ACTIONS(1276),
    [anon_sym_SQUOTE] = ACTIONS(1279),
    [sym_number_literal] = ACTIONS(1282),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(571)] = {
    [sym_expression] = STATE(1141),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_and] = ACTIONS(1245),
    [sym_or] = ACTIONS(1245),
    [sym_not] = ACTIONS(2147),
    [sym_xor] = ACTIONS(1245),
    [sym_band] = ACTIONS(1245),
    [sym_bor] = ACTIONS(1245),
    [sym_bxor] = ACTIONS(1245),
    [sym_bnot] = ACTIONS(2147),
    [sym_lshift] = ACTIONS(1245),
    [sym_rshift] = ACTIONS(1245),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_COMMA] = ACTIONS(1262),
    [anon_sym_LBRACK] = ACTIONS(1264),
    [anon_sym_RBRACK] = ACTIONS(1262),
    [anon_sym_COLON] = ACTIONS(1262),
    [anon_sym_LPAREN] = ACTIONS(1267),
    [anon_sym_RPAREN] = ACTIONS(1262),
    [anon_sym_SEMI] = ACTIONS(1262),
    [anon_sym_BANG] = ACTIONS(2147),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(1247),
    [anon_sym_PLUS] = ACTIONS(1247),
    [anon_sym_STAR] = ACTIONS(1262),
    [anon_sym_SLASH] = ACTIONS(1245),
    [anon_sym_PERCENT] = ACTIONS(1262),
    [anon_sym_PIPE_PIPE] = ACTIONS(1262),
    [anon_sym_AMP_AMP] = ACTIONS(1262),
    [anon_sym_PIPE] = ACTIONS(1245),
    [anon_sym_CARET] = ACTIONS(1262),
    [anon_sym_AMP] = ACTIONS(1245),
    [anon_sym_EQ_EQ] = ACTIONS(1262),
    [anon_sym_BANG_EQ] = ACTIONS(1262),
    [anon_sym_LT_GT] = ACTIONS(1262),
    [anon_sym_GT] = ACTIONS(1245),
    [anon_sym_GT_EQ] = ACTIONS(1262),
    [anon_sym_LT_EQ] = ACTIONS(1262),
    [anon_sym_LT] = ACTIONS(1245),
    [anon_sym_LT_LT] = ACTIONS(1262),
    [anon_sym_GT_GT] = ACTIONS(1262),
    [anon_sym_DASH_DASH] = ACTIONS(1273),
    [anon_sym_PLUS_PLUS] = ACTIONS(1273),
    [anon_sym_DOT] = ACTIONS(1245),
    [anon_sym_DQUOTE] = ACTIONS(2157),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(572)] = {
    [sym_expression] = STATE(1141),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_and] = ACTIONS(1245),
    [sym_or] = ACTIONS(1245),
    [sym_not] = ACTIONS(2147),
    [sym_xor] = ACTIONS(1245),
    [sym_band] = ACTIONS(1245),
    [sym_bor] = ACTIONS(1245),
    [sym_bxor] = ACTIONS(1245),
    [sym_bnot] = ACTIONS(2147),
    [sym_lshift] = ACTIONS(1245),
    [sym_rshift] = ACTIONS(1245),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_COMMA] = ACTIONS(1262),
    [anon_sym_LBRACK] = ACTIONS(1264),
    [anon_sym_COLON] = ACTIONS(1262),
    [anon_sym_LPAREN] = ACTIONS(1267),
    [anon_sym_SEMI] = ACTIONS(1262),
    [anon_sym_BANG] = ACTIONS(2147),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(1247),
    [anon_sym_PLUS] = ACTIONS(1247),
    [anon_sym_STAR] = ACTIONS(1262),
    [anon_sym_SLASH] = ACTIONS(1245),
    [anon_sym_PERCENT] = ACTIONS(1262),
    [anon_sym_PIPE_PIPE] = ACTIONS(1262),
    [anon_sym_AMP_AMP] = ACTIONS(1262),
    [anon_sym_PIPE] = ACTIONS(1245),
    [anon_sym_CARET] = ACTIONS(1262),
    [anon_sym_AMP] = ACTIONS(1245),
    [anon_sym_EQ_EQ] = ACTIONS(1262),
    [anon_sym_BANG_EQ] = ACTIONS(1262),
    [anon_sym_LT_GT] = ACTIONS(1262),
    [anon_sym_GT] = ACTIONS(1245),
    [anon_sym_GT_EQ] = ACTIONS(1262),
    [anon_sym_LT_EQ] = ACTIONS(1262),
    [anon_sym_LT] = ACTIONS(1245),
    [anon_sym_LT_LT] = ACTIONS(1262),
    [anon_sym_GT_GT] = ACTIONS(1262),
    [anon_sym_DASH_DASH] = ACTIONS(1273),
    [anon_sym_PLUS_PLUS] = ACTIONS(1273),
    [anon_sym_DOT] = ACTIONS(1245),
    [anon_sym_DQUOTE] = ACTIONS(2157),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(1262),
  },
  [STATE(573)] = {
    [sym_expression] = STATE(1141),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_and] = ACTIONS(1245),
    [sym_or] = ACTIONS(1245),
    [sym_not] = ACTIONS(2147),
    [sym_xor] = ACTIONS(1245),
    [sym_band] = ACTIONS(1245),
    [sym_bor] = ACTIONS(1245),
    [sym_bxor] = ACTIONS(1245),
    [sym_bnot] = ACTIONS(2147),
    [sym_lshift] = ACTIONS(1245),
    [sym_rshift] = ACTIONS(1245),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_COMMA] = ACTIONS(1262),
    [anon_sym_LBRACK] = ACTIONS(1264),
    [anon_sym_COLON] = ACTIONS(1262),
    [anon_sym_LPAREN] = ACTIONS(1267),
    [anon_sym_BANG] = ACTIONS(2147),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(1247),
    [anon_sym_PLUS] = ACTIONS(1247),
    [anon_sym_STAR] = ACTIONS(1262),
    [anon_sym_SLASH] = ACTIONS(1245),
    [anon_sym_PERCENT] = ACTIONS(1262),
    [anon_sym_PIPE_PIPE] = ACTIONS(1262),
    [anon_sym_AMP_AMP] = ACTIONS(1262),
    [anon_sym_PIPE] = ACTIONS(1245),
    [anon_sym_CARET] = ACTIONS(1262),
    [anon_sym_AMP] = ACTIONS(1245),
    [anon_sym_EQ_EQ] = ACTIONS(1262),
    [anon_sym_BANG_EQ] = ACTIONS(1262),
    [anon_sym_LT_GT] = ACTIONS(1262),
    [anon_sym_GT] = ACTIONS(1245),
    [anon_sym_GT_EQ] = ACTIONS(1262),
    [anon_sym_LT_EQ] = ACTIONS(1262),
    [anon_sym_LT] = ACTIONS(1245),
    [anon_sym_LT_LT] = ACTIONS(1262),
    [anon_sym_GT_GT] = ACTIONS(1262),
    [anon_sym_DASH_DASH] = ACTIONS(1273),
    [anon_sym_PLUS_PLUS] = ACTIONS(1273),
    [anon_sym_DOT] = ACTIONS(1245),
    [anon_sym_DQUOTE] = ACTIONS(1276),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(574)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_COMMA] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(575)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1503),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(1503),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(1503),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(1503),
    [sym___date__] = ACTIONS(1503),
    [sym___ldate__] = ACTIONS(1503),
    [sym___time__] = ACTIONS(1503),
    [sym___line__] = ACTIONS(1503),
    [sym___name__] = ACTIONS(1503),
    [sym___netlinx__] = ACTIONS(1503),
    [sym_netlinx_axi_version] = ACTIONS(1503),
    [sym_url_flg_tcp] = ACTIONS(1503),
    [sym_url_flg_accinfopresent] = ACTIONS(1503),
    [sym_url_flg_temp] = ACTIONS(1503),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1503),
    [sym_url_flg_stat_mask] = ACTIONS(1503),
    [sym_url_flg_stat_lookup] = ACTIONS(1503),
    [sym_url_flg_stat_connecting] = ACTIONS(1503),
    [sym_url_flg_stat_waiting] = ACTIONS(1503),
    [sym_url_flg_stat_connected] = ACTIONS(1503),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1503),
    [sym_first_virtual_device] = ACTIONS(1503),
    [sym_first_local_port] = ACTIONS(1503),
    [sym_dynamic_virtual_device] = ACTIONS(1503),
    [sym_timeline_once] = ACTIONS(1503),
    [sym_timeline_repeat] = ACTIONS(1503),
    [sym_timeline_absolute] = ACTIONS(1503),
    [sym_timeline_relative] = ACTIONS(1503),
    [sym_do_push_timed_infinite] = ACTIONS(1503),
    [sym_source_type_no_address] = ACTIONS(1503),
    [sym_source_type_neuron_id] = ACTIONS(1503),
    [sym_source_type_ip_address] = ACTIONS(1503),
    [sym_source_type_axlink] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1503),
    [sym_source_type_ip_socket_address] = ACTIONS(1503),
    [sym_source_type_rs232] = ACTIONS(1503),
    [sym_source_type_internal] = ACTIONS(1503),
    [sym_source_type_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1503),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1503),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1503),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1503),
    [sym_true] = ACTIONS(1503),
    [sym_false] = ACTIONS(1503),
    [sym_file_read_only] = ACTIONS(1503),
    [sym_file_rw_new] = ACTIONS(1503),
    [sym_file_rw_append] = ACTIONS(1503),
    [sym_ip_tcp] = ACTIONS(1503),
    [sym_ip_udp] = ACTIONS(1503),
    [sym_ip_udp_2way] = ACTIONS(1503),
    [sym_xml_encode_types] = ACTIONS(1503),
    [sym_xml_encode_char_as_list] = ACTIONS(1503),
    [sym_xml_encode_le] = ACTIONS(1503),
    [sym_xml_decode_types] = ACTIONS(1503),
    [sym_xml_decode_no_preserve] = ACTIONS(1503),
    [sym_duet_dev_type_display_device] = ACTIONS(1503),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_io_device] = ACTIONS(1503),
    [sym_duet_dev_type_relay_device] = ACTIONS(1503),
    [sym_duet_dev_type_ups] = ACTIONS(1503),
    [sym_duet_dev_type_amplifier] = ACTIONS(1503),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1503),
    [sym_duet_dev_type_camera] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1503),
    [sym_duet_dev_type_dss] = ACTIONS(1503),
    [sym_duet_dev_type_dvr] = ACTIONS(1503),
    [sym_duet_dev_type_disc_device] = ACTIONS(1503),
    [sym_duet_dev_type_document_camera] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1503),
    [sym_duet_dev_type_hvac] = ACTIONS(1503),
    [sym_duet_dev_type_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_light] = ACTIONS(1503),
    [sym_duet_dev_type_monitor] = ACTIONS(1503),
    [sym_duet_dev_type_motor] = ACTIONS(1503),
    [sym_duet_dev_type_multi_window] = ACTIONS(1503),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1503),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1503),
    [sym_duet_dev_type_receiver] = ACTIONS(1503),
    [sym_duet_dev_type_security_system] = ACTIONS(1503),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1503),
    [sym_duet_dev_type_settop_box] = ACTIONS(1503),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1503),
    [sym_duet_dev_type_switcher] = ACTIONS(1503),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_tv] = ACTIONS(1503),
    [sym_duet_dev_type_utility] = ACTIONS(1503),
    [sym_duet_dev_type_vcr] = ACTIONS(1503),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_video_processor] = ACTIONS(1503),
    [sym_duet_dev_type_video_projector] = ACTIONS(1503),
    [sym_duet_dev_type_video_wall] = ACTIONS(1503),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1503),
    [sym_duet_dev_type_weather] = ACTIONS(1503),
    [sym_duet_dev_not_polled] = ACTIONS(1503),
    [sym_duet_dev_polled] = ACTIONS(1503),
    [sym_clkmgr_mode_network] = ACTIONS(1503),
    [sym_clkmgr_mode_standalone] = ACTIONS(1503),
    [sym_normal_standby] = ACTIONS(1503),
    [sym_normal_wake] = ACTIONS(1503),
    [sym_ip_multicast_ttl_option] = ACTIONS(1503),
    [sym_ip_ttl_subnet] = ACTIONS(1503),
    [sym_ip_ttl_site] = ACTIONS(1503),
    [sym_ip_ttl_region] = ACTIONS(1503),
    [sym_ip_ttl_continent] = ACTIONS(1503),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1503),
    [sym_ip_nodelay_on] = ACTIONS(1503),
    [sym_ip_nodelay_off] = ACTIONS(1503),
    [sym_amx_error] = ACTIONS(1503),
    [sym_amx_warning] = ACTIONS(1503),
    [sym_amx_info] = ACTIONS(1503),
    [sym_amx_debug] = ACTIONS(1503),
    [sym_netlinx_login_success] = ACTIONS(1503),
    [sym_netlinx_login_fail] = ACTIONS(1503),
    [sym_netlinx_logout] = ACTIONS(1503),
    [sym_null_str] = ACTIONS(1503),
    [sym_smtp_address] = ACTIONS(1503),
    [sym_smtp_port_number] = ACTIONS(1503),
    [sym_smtp_username] = ACTIONS(1503),
    [sym_smtp_password] = ACTIONS(1503),
    [sym_smtp_require_tls] = ACTIONS(1503),
    [sym_smtp_from] = ACTIONS(1503),
    [sym_smtp_tls_true] = ACTIONS(1503),
    [sym_smtp_tls_false] = ACTIONS(1503),
    [sym_listview_on_row_select_event] = ACTIONS(1503),
    [sym_data_structure_unknown] = ACTIONS(1503),
    [sym_data_structure_datafeed] = ACTIONS(1503),
    [sym_data_structure_datarecordset] = ACTIONS(1503),
    [sym_data_structure_datarecord] = ACTIONS(1503),
    [sym_data_structure_datafield] = ACTIONS(1503),
    [sym_data_max_content_field_count] = ACTIONS(1503),
    [sym_data_max_metadata_field_count] = ACTIONS(1503),
    [sym_data_max_id_length] = ACTIONS(1503),
    [sym_data_max_name_length] = ACTIONS(1503),
    [sym_data_max_description_length] = ACTIONS(1503),
    [sym_data_max_type_length] = ACTIONS(1503),
    [sym_data_max_format_length] = ACTIONS(1503),
    [sym_data_max_source_length] = ACTIONS(1503),
    [sym_data_max_value_length] = ACTIONS(1503),
    [sym_data_max_label_length] = ACTIONS(1503),
    [sym_data_max_url_length] = ACTIONS(1503),
    [sym_max_last_login_info_length] = ACTIONS(1503),
    [sym_data_type_unknown] = ACTIONS(1503),
    [sym_data_type_string] = ACTIONS(1503),
    [sym_data_type_datetime] = ACTIONS(1503),
    [sym_data_type_date] = ACTIONS(1503),
    [sym_data_type_time] = ACTIONS(1503),
    [sym_data_type_image] = ACTIONS(1503),
    [sym_data_format_url] = ACTIONS(1503),
    [sym_data_format_phone] = ACTIONS(1503),
    [sym_data_format_email] = ACTIONS(1503),
    [sym_data_format_iso8601] = ACTIONS(1503),
    [sym_auditlog_priv] = ACTIONS(1503),
    [sym_configuration_priv] = ACTIONS(1503),
    [sym_deviceconfig_priv] = ACTIONS(1503),
    [sym_ftp_priv] = ACTIONS(1503),
    [sym_http_priv] = ACTIONS(1503),
    [sym_networkconfig_priv] = ACTIONS(1503),
    [sym_programport_priv] = ACTIONS(1503),
    [sym_remoteui_priv] = ACTIONS(1503),
    [sym_securitycontrol_priv] = ACTIONS(1503),
    [sym_softwaremanagement_priv] = ACTIONS(1503),
    [sym_terminal_priv] = ACTIONS(1503),
    [sym_tpadmin_priv] = ACTIONS(1503),
    [sym_user1_priv] = ACTIONS(1503),
    [sym_user2_priv] = ACTIONS(1503),
    [sym_user3_priv] = ACTIONS(1503),
    [sym_user4_priv] = ACTIONS(1503),
    [sym_usermanagement_priv] = ACTIONS(1503),
    [sym_valid_account] = ACTIONS(1503),
    [sym_err_name_invalid] = ACTIONS(1503),
    [sym_err_password_invalid] = ACTIONS(1503),
    [sym_err_invalid_account] = ACTIONS(1503),
    [sym_err_no_authorization] = ACTIONS(1503),
    [sym_err_invalid_parameter] = ACTIONS(1503),
    [sym_err_user_locked_out] = ACTIONS(1503),
    [sym_err_user_account_expired] = ACTIONS(1503),
    [sym_err_general_error] = ACTIONS(1503),
    [sym_err_authentication_error] = ACTIONS(1503),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1503),
    [sym_tls_validate_certificate] = ACTIONS(1503),
    [sym____reserved___] = ACTIONS(1503),
    [sym_date] = ACTIONS(1503),
    [sym_day] = ACTIONS(1503),
    [sym_time] = ACTIONS(1503),
    [sym_ldate] = ACTIONS(1503),
    [sym_push_device] = ACTIONS(1503),
    [sym_push_channel] = ACTIONS(1503),
    [sym_push_devchan] = ACTIONS(1503),
    [sym_release_device] = ACTIONS(1503),
    [sym_release_channel] = ACTIONS(1503),
    [sym_release_devchan] = ACTIONS(1503),
    [sym_master_slot] = ACTIONS(1503),
    [sym_get_pulse_time] = ACTIONS(1503),
    [sym_get_timer] = ACTIONS(1503),
    [sym_master_sn] = ACTIONS(1503),
    [sym_system_number] = ACTIONS(1503),
    [sym_button] = ACTIONS(1503),
    [sym_dv_channel] = ACTIONS(1503),
    [sym_data] = ACTIONS(1503),
    [sym_level] = ACTIONS(1503),
    [sym_timeline] = ACTIONS(1503),
    [sym_channel] = ACTIONS(1503),
    [sym_custom] = ACTIONS(1503),
    [sym_atoi] = ACTIONS(1503),
    [sym_atol] = ACTIONS(1503),
    [sym_atof] = ACTIONS(1503),
    [sym_itoa] = ACTIONS(1503),
    [sym_itohex] = ACTIONS(1503),
    [sym_hextoi] = ACTIONS(1503),
    [sym_ftoa] = ACTIONS(1503),
    [sym_find_string] = ACTIONS(1503),
    [sym_left_string] = ACTIONS(1503),
    [sym_right_string] = ACTIONS(1503),
    [sym_mid_string] = ACTIONS(1503),
    [sym_remove_string] = ACTIONS(1503),
    [sym_get_buffer_string] = ACTIONS(1503),
    [sym_get_buffer_char] = ACTIONS(1503),
    [sym_get_multi_buffer_string] = ACTIONS(1503),
    [sym_length_string] = ACTIONS(1503),
    [sym_max_length_string] = ACTIONS(1503),
    [sym_set_length_string] = ACTIONS(1503),
    [sym_compare_string] = ACTIONS(1503),
    [sym_upper_string] = ACTIONS(1503),
    [sym_lower_string] = ACTIONS(1503),
    [sym_get_last] = ACTIONS(1503),
    [sym_date_to_day] = ACTIONS(1503),
    [sym_date_to_month] = ACTIONS(1503),
    [sym_date_to_year] = ACTIONS(1503),
    [sym_time_to_hour] = ACTIONS(1503),
    [sym_time_to_minute] = ACTIONS(1503),
    [sym_time_to_second] = ACTIONS(1503),
    [sym_day_of_week] = ACTIONS(1503),
    [sym_set_pulse_time] = ACTIONS(1503),
    [sym_set_timer] = ACTIONS(1503),
    [sym_file_open] = ACTIONS(1503),
    [sym_file_close] = ACTIONS(1503),
    [sym_file_read] = ACTIONS(1503),
    [sym_file_read_line] = ACTIONS(1503),
    [sym_file_write] = ACTIONS(1503),
    [sym_file_write_line] = ACTIONS(1503),
    [sym_file_copy] = ACTIONS(1503),
    [sym_file_delete] = ACTIONS(1503),
    [sym_file_rename] = ACTIONS(1503),
    [sym_file_dir] = ACTIONS(1503),
    [sym_file_getdir] = ACTIONS(1503),
    [sym_file_setdir] = ACTIONS(1503),
    [sym_file_create_dir] = ACTIONS(1503),
    [sym_file_remove_dir] = ACTIONS(1503),
    [sym_file_seek] = ACTIONS(1503),
    [sym_length_array] = ACTIONS(1503),
    [sym_max_length_array] = ACTIONS(1503),
    [sym_set_length_array] = ACTIONS(1503),
    [sym_ip_client_open] = ACTIONS(1503),
    [sym_ip_client_close] = ACTIONS(1503),
    [sym_ip_server_open] = ACTIONS(1503),
    [sym_ip_server_close] = ACTIONS(1503),
    [sym_ip_bound_client_open] = ACTIONS(1503),
    [sym_ip_mc_server_open] = ACTIONS(1503),
    [sym_ip_set_option] = ACTIONS(1503),
    [sym_ssh_client_open] = ACTIONS(1503),
    [sym_ssh_client_close] = ACTIONS(1503),
    [sym_tls_client_open] = ACTIONS(1503),
    [sym_tls_client_close] = ACTIONS(1503),
    [sym_do_push] = ACTIONS(1503),
    [sym_do_push_timed] = ACTIONS(1503),
    [sym_do_release] = ACTIONS(1503),
    [sym_device_id] = ACTIONS(1503),
    [sym_device_info] = ACTIONS(1503),
    [sym_device_id_string] = ACTIONS(1503),
    [sym_device_standby] = ACTIONS(1503),
    [sym_device_wake] = ACTIONS(1503),
    [sym_timeline_create] = ACTIONS(1503),
    [sym_timeline_kill] = ACTIONS(1503),
    [sym_timeline_reload] = ACTIONS(1503),
    [sym_timeline_pause] = ACTIONS(1503),
    [sym_timeline_restart] = ACTIONS(1503),
    [sym_timeline_set] = ACTIONS(1503),
    [sym_timeline_get] = ACTIONS(1503),
    [sym_timeline_active] = ACTIONS(1503),
    [sym_timeline_dynamic_id] = ACTIONS(1503),
    [sym_variable_to_string] = ACTIONS(1503),
    [sym_string_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_string] = ACTIONS(1503),
    [sym_type_cast] = ACTIONS(1503),
    [sym_variable_to_xml] = ACTIONS(1503),
    [sym_xml_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_xml] = ACTIONS(1503),
    [sym_redirect_string] = ACTIONS(1503),
    [sym_random_number] = ACTIONS(1503),
    [sym_get_unique_id] = ACTIONS(1503),
    [sym_get_system_number] = ACTIONS(1503),
    [sym_set_system_number] = ACTIONS(1503),
    [sym_get_serial_number] = ACTIONS(1503),
    [sym_get_master_build] = ACTIONS(1503),
    [sym_set_validation_code] = ACTIONS(1503),
    [sym_reboot] = ACTIONS(1503),
    [sym_load_duet_module] = ACTIONS(1503),
    [sym_unload_duet_module] = ACTIONS(1503),
    [sym_duet_mem_size_set] = ACTIONS(1503),
    [sym_duet_mem_size_get] = ACTIONS(1503),
    [sym_dynamic_application_device] = ACTIONS(1503),
    [sym_dynamic_polled_port] = ACTIONS(1503),
    [sym_static_port_binding] = ACTIONS(1503),
    [sym_static_ip_binding] = ACTIONS(1503),
    [sym_get_dns_list] = ACTIONS(1503),
    [sym_set_dns_list] = ACTIONS(1503),
    [sym_get_ip_address] = ACTIONS(1503),
    [sym_set_ip_address] = ACTIONS(1503),
    [sym_get_url_list] = ACTIONS(1503),
    [sym_add_url_entry] = ACTIONS(1503),
    [sym_delete_url_entry] = ACTIONS(1503),
    [sym_abs_value] = ACTIONS(1503),
    [sym_max_value] = ACTIONS(1503),
    [sym_min_value] = ACTIONS(1503),
    [sym_exp_value] = ACTIONS(1503),
    [sym_log_value] = ACTIONS(1503),
    [sym_log10_value] = ACTIONS(1503),
    [sym_power_value] = ACTIONS(1503),
    [sym_sqrt_value] = ACTIONS(1503),
    [sym_raw_be] = ACTIONS(1503),
    [sym_raw_le] = ACTIONS(1503),
    [sym_internal_queue_size_set] = ACTIONS(1503),
    [sym_internal_queue_size_get] = ACTIONS(1503),
    [sym_internal_threshold_set] = ACTIONS(1503),
    [sym_internal_threshold_get] = ACTIONS(1503),
    [sym_rebuilt_event] = ACTIONS(1503),
    [sym_do_custom_event] = ACTIONS(1503),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1503),
    [sym_clkmgr_set_clk_source] = ACTIONS(1503),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1503),
    [sym_clkmgr_get_timezone] = ACTIONS(1503),
    [sym_clkmgr_set_timezone] = ACTIONS(1503),
    [sym_clkmgr_get_resync_period] = ACTIONS(1503),
    [sym_clkmgr_set_resync_period] = ACTIONS(1503),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_timeservers] = ACTIONS(1503),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_set_log_level] = ACTIONS(1503),
    [sym_get_log_level] = ACTIONS(1503),
    [sym_amx_log] = ACTIONS(1503),
    [sym_smtp_server_config_set] = ACTIONS(1503),
    [sym_smtp_server_config_get] = ACTIONS(1503),
    [sym_smtp_send] = ACTIONS(1503),
    [sym_validate_netlinx_account] = ACTIONS(1503),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1503),
    [sym_authenticate_certificate] = ACTIONS(1503),
    [sym_audit_netlinx_session_event] = ACTIONS(1503),
    [sym_audit_netlinx_generic_event] = ACTIONS(1503),
    [sym_get_available_flash_disk_space] = ACTIONS(1503),
    [sym_get_max_flash_disk_space] = ACTIONS(1503),
    [sym_data_create_feed] = ACTIONS(1503),
    [sym_data_delete_feed] = ACTIONS(1503),
    [sym_data_publish_feed] = ACTIONS(1503),
    [sym_data_get_published_feed] = ACTIONS(1503),
    [sym_data_add_record] = ACTIONS(1503),
    [sym_data_get_event_record] = ACTIONS(1503),
    [sym__wc_data_create_feed] = ACTIONS(1503),
    [sym__wc_data_add_record] = ACTIONS(1503),
    [sym__wc_data_get_event_record] = ACTIONS(1503),
    [sym_format] = ACTIONS(1503),
    [sym_astro_clock] = ACTIONS(1503),
    [sym_set_outdoor_temperature] = ACTIONS(1503),
    [sym_set_virtual_level_count] = ACTIONS(1503),
    [sym_set_virtual_channel_count] = ACTIONS(1503),
    [sym_set_virtual_port_count] = ACTIONS(1503),
    [sym__wc_encode_int] = ACTIONS(1503),
    [sym__wc_decode_int] = ACTIONS(1503),
    [sym__wc_int] = ACTIONS(1503),
    [sym__wc_to_ch_int] = ACTIONS(1503),
    [sym__ch_to_wc_int] = ACTIONS(1503),
    [sym__wc_find_string_int] = ACTIONS(1503),
    [sym__wc_left_string_int] = ACTIONS(1503),
    [sym__wc_lower_string_int] = ACTIONS(1503),
    [sym__wc_mid_string_int] = ACTIONS(1503),
    [sym__wc_remove_string_int] = ACTIONS(1503),
    [sym__wc_right_string_int] = ACTIONS(1503),
    [sym__wc_upper_string_int] = ACTIONS(1503),
    [sym__wc_compare_string_int] = ACTIONS(1503),
    [sym__wc_get_buffer_char_int] = ACTIONS(1503),
    [sym__wc_get_buffer_string_int] = ACTIONS(1503),
    [sym__wc_concat_string_int] = ACTIONS(1503),
    [sym__wc_file_open_int] = ACTIONS(1503),
    [sym__wc_file_close_int] = ACTIONS(1503),
    [sym__wc_file_read_int] = ACTIONS(1503),
    [sym__wc_file_read_line_int] = ACTIONS(1503),
    [sym__wc_file_write_int] = ACTIONS(1503),
    [sym__wc_file_write_line_int] = ACTIONS(1503),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1505),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1505),
    [aux_sym_preproc_else_token1] = ACTIONS(1505),
    [anon_sym_COMMA] = ACTIONS(1505),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1505),
    [anon_sym_RBRACE] = ACTIONS(1505),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1503),
    [anon_sym_TILDE] = ACTIONS(1505),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1505),
    [anon_sym_SQUOTE] = ACTIONS(1505),
    [sym_number_literal] = ACTIONS(1505),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(576)] = {
    [aux_sym_subscript_expression_repeat1] = STATE(576),
    [sym_identifier] = ACTIONS(1496),
    [sym_and] = ACTIONS(1496),
    [sym_or] = ACTIONS(1496),
    [sym_not] = ACTIONS(1496),
    [sym_xor] = ACTIONS(1496),
    [sym_band] = ACTIONS(1496),
    [sym_bor] = ACTIONS(1496),
    [sym_bxor] = ACTIONS(1496),
    [sym_bnot] = ACTIONS(1496),
    [sym_lshift] = ACTIONS(1496),
    [sym_rshift] = ACTIONS(1496),
    [sym___file__] = ACTIONS(1496),
    [sym___date__] = ACTIONS(1496),
    [sym___ldate__] = ACTIONS(1496),
    [sym___time__] = ACTIONS(1496),
    [sym___line__] = ACTIONS(1496),
    [sym___name__] = ACTIONS(1496),
    [sym___netlinx__] = ACTIONS(1496),
    [sym_netlinx_axi_version] = ACTIONS(1496),
    [sym_url_flg_tcp] = ACTIONS(1496),
    [sym_url_flg_accinfopresent] = ACTIONS(1496),
    [sym_url_flg_temp] = ACTIONS(1496),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1496),
    [sym_url_flg_stat_mask] = ACTIONS(1496),
    [sym_url_flg_stat_lookup] = ACTIONS(1496),
    [sym_url_flg_stat_connecting] = ACTIONS(1496),
    [sym_url_flg_stat_waiting] = ACTIONS(1496),
    [sym_url_flg_stat_connected] = ACTIONS(1496),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1496),
    [sym_first_virtual_device] = ACTIONS(1496),
    [sym_first_local_port] = ACTIONS(1496),
    [sym_dynamic_virtual_device] = ACTIONS(1496),
    [sym_timeline_once] = ACTIONS(1496),
    [sym_timeline_repeat] = ACTIONS(1496),
    [sym_timeline_absolute] = ACTIONS(1496),
    [sym_timeline_relative] = ACTIONS(1496),
    [sym_do_push_timed_infinite] = ACTIONS(1496),
    [sym_source_type_no_address] = ACTIONS(1496),
    [sym_source_type_neuron_id] = ACTIONS(1496),
    [sym_source_type_ip_address] = ACTIONS(1496),
    [sym_source_type_axlink] = ACTIONS(1496),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1496),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1496),
    [sym_source_type_ip_socket_address] = ACTIONS(1496),
    [sym_source_type_rs232] = ACTIONS(1496),
    [sym_source_type_internal] = ACTIONS(1496),
    [sym_source_type_mac_address] = ACTIONS(1496),
    [sym_source_type_ipv4_port] = ACTIONS(1496),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1496),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1496),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1496),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1496),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1496),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1496),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1496),
    [sym_true] = ACTIONS(1496),
    [sym_false] = ACTIONS(1496),
    [sym_file_read_only] = ACTIONS(1496),
    [sym_file_rw_new] = ACTIONS(1496),
    [sym_file_rw_append] = ACTIONS(1496),
    [sym_ip_tcp] = ACTIONS(1496),
    [sym_ip_udp] = ACTIONS(1496),
    [sym_ip_udp_2way] = ACTIONS(1496),
    [sym_xml_encode_types] = ACTIONS(1496),
    [sym_xml_encode_char_as_list] = ACTIONS(1496),
    [sym_xml_encode_le] = ACTIONS(1496),
    [sym_xml_decode_types] = ACTIONS(1496),
    [sym_xml_decode_no_preserve] = ACTIONS(1496),
    [sym_duet_dev_type_display_device] = ACTIONS(1496),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1496),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1496),
    [sym_duet_dev_type_io_device] = ACTIONS(1496),
    [sym_duet_dev_type_relay_device] = ACTIONS(1496),
    [sym_duet_dev_type_ups] = ACTIONS(1496),
    [sym_duet_dev_type_amplifier] = ACTIONS(1496),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1496),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1496),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1496),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1496),
    [sym_duet_dev_type_camera] = ACTIONS(1496),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1496),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1496),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1496),
    [sym_duet_dev_type_dss] = ACTIONS(1496),
    [sym_duet_dev_type_dvr] = ACTIONS(1496),
    [sym_duet_dev_type_disc_device] = ACTIONS(1496),
    [sym_duet_dev_type_document_camera] = ACTIONS(1496),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1496),
    [sym_duet_dev_type_hvac] = ACTIONS(1496),
    [sym_duet_dev_type_keypad] = ACTIONS(1496),
    [sym_duet_dev_type_light] = ACTIONS(1496),
    [sym_duet_dev_type_monitor] = ACTIONS(1496),
    [sym_duet_dev_type_motor] = ACTIONS(1496),
    [sym_duet_dev_type_multi_window] = ACTIONS(1496),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1496),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1496),
    [sym_duet_dev_type_receiver] = ACTIONS(1496),
    [sym_duet_dev_type_security_system] = ACTIONS(1496),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1496),
    [sym_duet_dev_type_settop_box] = ACTIONS(1496),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1496),
    [sym_duet_dev_type_switcher] = ACTIONS(1496),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1496),
    [sym_duet_dev_type_tv] = ACTIONS(1496),
    [sym_duet_dev_type_utility] = ACTIONS(1496),
    [sym_duet_dev_type_vcr] = ACTIONS(1496),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1496),
    [sym_duet_dev_type_video_processor] = ACTIONS(1496),
    [sym_duet_dev_type_video_projector] = ACTIONS(1496),
    [sym_duet_dev_type_video_wall] = ACTIONS(1496),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1496),
    [sym_duet_dev_type_weather] = ACTIONS(1496),
    [sym_duet_dev_not_polled] = ACTIONS(1496),
    [sym_duet_dev_polled] = ACTIONS(1496),
    [sym_clkmgr_mode_network] = ACTIONS(1496),
    [sym_clkmgr_mode_standalone] = ACTIONS(1496),
    [sym_normal_standby] = ACTIONS(1496),
    [sym_normal_wake] = ACTIONS(1496),
    [sym_ip_multicast_ttl_option] = ACTIONS(1496),
    [sym_ip_ttl_subnet] = ACTIONS(1496),
    [sym_ip_ttl_site] = ACTIONS(1496),
    [sym_ip_ttl_region] = ACTIONS(1496),
    [sym_ip_ttl_continent] = ACTIONS(1496),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1496),
    [sym_ip_nodelay_on] = ACTIONS(1496),
    [sym_ip_nodelay_off] = ACTIONS(1496),
    [sym_amx_error] = ACTIONS(1496),
    [sym_amx_warning] = ACTIONS(1496),
    [sym_amx_info] = ACTIONS(1496),
    [sym_amx_debug] = ACTIONS(1496),
    [sym_netlinx_login_success] = ACTIONS(1496),
    [sym_netlinx_login_fail] = ACTIONS(1496),
    [sym_netlinx_logout] = ACTIONS(1496),
    [sym_null_str] = ACTIONS(1496),
    [sym_smtp_address] = ACTIONS(1496),
    [sym_smtp_port_number] = ACTIONS(1496),
    [sym_smtp_username] = ACTIONS(1496),
    [sym_smtp_password] = ACTIONS(1496),
    [sym_smtp_require_tls] = ACTIONS(1496),
    [sym_smtp_from] = ACTIONS(1496),
    [sym_smtp_tls_true] = ACTIONS(1496),
    [sym_smtp_tls_false] = ACTIONS(1496),
    [sym_listview_on_row_select_event] = ACTIONS(1496),
    [sym_data_structure_unknown] = ACTIONS(1496),
    [sym_data_structure_datafeed] = ACTIONS(1496),
    [sym_data_structure_datarecordset] = ACTIONS(1496),
    [sym_data_structure_datarecord] = ACTIONS(1496),
    [sym_data_structure_datafield] = ACTIONS(1496),
    [sym_data_max_content_field_count] = ACTIONS(1496),
    [sym_data_max_metadata_field_count] = ACTIONS(1496),
    [sym_data_max_id_length] = ACTIONS(1496),
    [sym_data_max_name_length] = ACTIONS(1496),
    [sym_data_max_description_length] = ACTIONS(1496),
    [sym_data_max_type_length] = ACTIONS(1496),
    [sym_data_max_format_length] = ACTIONS(1496),
    [sym_data_max_source_length] = ACTIONS(1496),
    [sym_data_max_value_length] = ACTIONS(1496),
    [sym_data_max_label_length] = ACTIONS(1496),
    [sym_data_max_url_length] = ACTIONS(1496),
    [sym_max_last_login_info_length] = ACTIONS(1496),
    [sym_data_type_unknown] = ACTIONS(1496),
    [sym_data_type_string] = ACTIONS(1496),
    [sym_data_type_datetime] = ACTIONS(1496),
    [sym_data_type_date] = ACTIONS(1496),
    [sym_data_type_time] = ACTIONS(1496),
    [sym_data_type_image] = ACTIONS(1496),
    [sym_data_format_url] = ACTIONS(1496),
    [sym_data_format_phone] = ACTIONS(1496),
    [sym_data_format_email] = ACTIONS(1496),
    [sym_data_format_iso8601] = ACTIONS(1496),
    [sym_auditlog_priv] = ACTIONS(1496),
    [sym_configuration_priv] = ACTIONS(1496),
    [sym_deviceconfig_priv] = ACTIONS(1496),
    [sym_ftp_priv] = ACTIONS(1496),
    [sym_http_priv] = ACTIONS(1496),
    [sym_networkconfig_priv] = ACTIONS(1496),
    [sym_programport_priv] = ACTIONS(1496),
    [sym_remoteui_priv] = ACTIONS(1496),
    [sym_securitycontrol_priv] = ACTIONS(1496),
    [sym_softwaremanagement_priv] = ACTIONS(1496),
    [sym_terminal_priv] = ACTIONS(1496),
    [sym_tpadmin_priv] = ACTIONS(1496),
    [sym_user1_priv] = ACTIONS(1496),
    [sym_user2_priv] = ACTIONS(1496),
    [sym_user3_priv] = ACTIONS(1496),
    [sym_user4_priv] = ACTIONS(1496),
    [sym_usermanagement_priv] = ACTIONS(1496),
    [sym_valid_account] = ACTIONS(1496),
    [sym_err_name_invalid] = ACTIONS(1496),
    [sym_err_password_invalid] = ACTIONS(1496),
    [sym_err_invalid_account] = ACTIONS(1496),
    [sym_err_no_authorization] = ACTIONS(1496),
    [sym_err_invalid_parameter] = ACTIONS(1496),
    [sym_err_user_locked_out] = ACTIONS(1496),
    [sym_err_user_account_expired] = ACTIONS(1496),
    [sym_err_general_error] = ACTIONS(1496),
    [sym_err_authentication_error] = ACTIONS(1496),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1496),
    [sym_tls_validate_certificate] = ACTIONS(1496),
    [sym____reserved___] = ACTIONS(1496),
    [sym_date] = ACTIONS(1496),
    [sym_day] = ACTIONS(1496),
    [sym_time] = ACTIONS(1496),
    [sym_ldate] = ACTIONS(1496),
    [sym_push_device] = ACTIONS(1496),
    [sym_push_channel] = ACTIONS(1496),
    [sym_push_devchan] = ACTIONS(1496),
    [sym_release_device] = ACTIONS(1496),
    [sym_release_channel] = ACTIONS(1496),
    [sym_release_devchan] = ACTIONS(1496),
    [sym_master_slot] = ACTIONS(1496),
    [sym_get_pulse_time] = ACTIONS(1496),
    [sym_get_timer] = ACTIONS(1496),
    [sym_master_sn] = ACTIONS(1496),
    [sym_system_number] = ACTIONS(1496),
    [sym_button] = ACTIONS(1496),
    [sym_dv_channel] = ACTIONS(1496),
    [sym_data] = ACTIONS(1496),
    [sym_level] = ACTIONS(1496),
    [sym_timeline] = ACTIONS(1496),
    [sym_channel] = ACTIONS(1496),
    [sym_custom] = ACTIONS(1496),
    [sym_atoi] = ACTIONS(1496),
    [sym_atol] = ACTIONS(1496),
    [sym_atof] = ACTIONS(1496),
    [sym_itoa] = ACTIONS(1496),
    [sym_itohex] = ACTIONS(1496),
    [sym_hextoi] = ACTIONS(1496),
    [sym_ftoa] = ACTIONS(1496),
    [sym_find_string] = ACTIONS(1496),
    [sym_left_string] = ACTIONS(1496),
    [sym_right_string] = ACTIONS(1496),
    [sym_mid_string] = ACTIONS(1496),
    [sym_remove_string] = ACTIONS(1496),
    [sym_get_buffer_string] = ACTIONS(1496),
    [sym_get_buffer_char] = ACTIONS(1496),
    [sym_get_multi_buffer_string] = ACTIONS(1496),
    [sym_length_string] = ACTIONS(1496),
    [sym_max_length_string] = ACTIONS(1496),
    [sym_set_length_string] = ACTIONS(1496),
    [sym_compare_string] = ACTIONS(1496),
    [sym_upper_string] = ACTIONS(1496),
    [sym_lower_string] = ACTIONS(1496),
    [sym_get_last] = ACTIONS(1496),
    [sym_date_to_day] = ACTIONS(1496),
    [sym_date_to_month] = ACTIONS(1496),
    [sym_date_to_year] = ACTIONS(1496),
    [sym_time_to_hour] = ACTIONS(1496),
    [sym_time_to_minute] = ACTIONS(1496),
    [sym_time_to_second] = ACTIONS(1496),
    [sym_day_of_week] = ACTIONS(1496),
    [sym_set_pulse_time] = ACTIONS(1496),
    [sym_set_timer] = ACTIONS(1496),
    [sym_file_open] = ACTIONS(1496),
    [sym_file_close] = ACTIONS(1496),
    [sym_file_read] = ACTIONS(1496),
    [sym_file_read_line] = ACTIONS(1496),
    [sym_file_write] = ACTIONS(1496),
    [sym_file_write_line] = ACTIONS(1496),
    [sym_file_copy] = ACTIONS(1496),
    [sym_file_delete] = ACTIONS(1496),
    [sym_file_rename] = ACTIONS(1496),
    [sym_file_dir] = ACTIONS(1496),
    [sym_file_getdir] = ACTIONS(1496),
    [sym_file_setdir] = ACTIONS(1496),
    [sym_file_create_dir] = ACTIONS(1496),
    [sym_file_remove_dir] = ACTIONS(1496),
    [sym_file_seek] = ACTIONS(1496),
    [sym_length_array] = ACTIONS(1496),
    [sym_max_length_array] = ACTIONS(1496),
    [sym_set_length_array] = ACTIONS(1496),
    [sym_ip_client_open] = ACTIONS(1496),
    [sym_ip_client_close] = ACTIONS(1496),
    [sym_ip_server_open] = ACTIONS(1496),
    [sym_ip_server_close] = ACTIONS(1496),
    [sym_ip_bound_client_open] = ACTIONS(1496),
    [sym_ip_mc_server_open] = ACTIONS(1496),
    [sym_ip_set_option] = ACTIONS(1496),
    [sym_ssh_client_open] = ACTIONS(1496),
    [sym_ssh_client_close] = ACTIONS(1496),
    [sym_tls_client_open] = ACTIONS(1496),
    [sym_tls_client_close] = ACTIONS(1496),
    [sym_do_push] = ACTIONS(1496),
    [sym_do_push_timed] = ACTIONS(1496),
    [sym_do_release] = ACTIONS(1496),
    [sym_device_id] = ACTIONS(1496),
    [sym_device_info] = ACTIONS(1496),
    [sym_device_id_string] = ACTIONS(1496),
    [sym_device_standby] = ACTIONS(1496),
    [sym_device_wake] = ACTIONS(1496),
    [sym_timeline_create] = ACTIONS(1496),
    [sym_timeline_kill] = ACTIONS(1496),
    [sym_timeline_reload] = ACTIONS(1496),
    [sym_timeline_pause] = ACTIONS(1496),
    [sym_timeline_restart] = ACTIONS(1496),
    [sym_timeline_set] = ACTIONS(1496),
    [sym_timeline_get] = ACTIONS(1496),
    [sym_timeline_active] = ACTIONS(1496),
    [sym_timeline_dynamic_id] = ACTIONS(1496),
    [sym_variable_to_string] = ACTIONS(1496),
    [sym_string_to_variable] = ACTIONS(1496),
    [sym_length_variable_to_string] = ACTIONS(1496),
    [sym_type_cast] = ACTIONS(1496),
    [sym_variable_to_xml] = ACTIONS(1496),
    [sym_xml_to_variable] = ACTIONS(1496),
    [sym_length_variable_to_xml] = ACTIONS(1496),
    [sym_redirect_string] = ACTIONS(1496),
    [sym_random_number] = ACTIONS(1496),
    [sym_get_unique_id] = ACTIONS(1496),
    [sym_get_system_number] = ACTIONS(1496),
    [sym_set_system_number] = ACTIONS(1496),
    [sym_get_serial_number] = ACTIONS(1496),
    [sym_get_master_build] = ACTIONS(1496),
    [sym_set_validation_code] = ACTIONS(1496),
    [sym_reboot] = ACTIONS(1496),
    [sym_load_duet_module] = ACTIONS(1496),
    [sym_unload_duet_module] = ACTIONS(1496),
    [sym_duet_mem_size_set] = ACTIONS(1496),
    [sym_duet_mem_size_get] = ACTIONS(1496),
    [sym_dynamic_application_device] = ACTIONS(1496),
    [sym_dynamic_polled_port] = ACTIONS(1496),
    [sym_static_port_binding] = ACTIONS(1496),
    [sym_static_ip_binding] = ACTIONS(1496),
    [sym_get_dns_list] = ACTIONS(1496),
    [sym_set_dns_list] = ACTIONS(1496),
    [sym_get_ip_address] = ACTIONS(1496),
    [sym_set_ip_address] = ACTIONS(1496),
    [sym_get_url_list] = ACTIONS(1496),
    [sym_add_url_entry] = ACTIONS(1496),
    [sym_delete_url_entry] = ACTIONS(1496),
    [sym_abs_value] = ACTIONS(1496),
    [sym_max_value] = ACTIONS(1496),
    [sym_min_value] = ACTIONS(1496),
    [sym_exp_value] = ACTIONS(1496),
    [sym_log_value] = ACTIONS(1496),
    [sym_log10_value] = ACTIONS(1496),
    [sym_power_value] = ACTIONS(1496),
    [sym_sqrt_value] = ACTIONS(1496),
    [sym_raw_be] = ACTIONS(1496),
    [sym_raw_le] = ACTIONS(1496),
    [sym_internal_queue_size_set] = ACTIONS(1496),
    [sym_internal_queue_size_get] = ACTIONS(1496),
    [sym_internal_threshold_set] = ACTIONS(1496),
    [sym_internal_threshold_get] = ACTIONS(1496),
    [sym_rebuilt_event] = ACTIONS(1496),
    [sym_do_custom_event] = ACTIONS(1496),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1496),
    [sym_clkmgr_set_clk_source] = ACTIONS(1496),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1496),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1496),
    [sym_clkmgr_get_timezone] = ACTIONS(1496),
    [sym_clkmgr_set_timezone] = ACTIONS(1496),
    [sym_clkmgr_get_resync_period] = ACTIONS(1496),
    [sym_clkmgr_set_resync_period] = ACTIONS(1496),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1496),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1496),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1496),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1496),
    [sym_clkmgr_get_timeservers] = ACTIONS(1496),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1496),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1496),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1496),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1496),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1496),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1496),
    [sym_set_log_level] = ACTIONS(1496),
    [sym_get_log_level] = ACTIONS(1496),
    [sym_amx_log] = ACTIONS(1496),
    [sym_smtp_server_config_set] = ACTIONS(1496),
    [sym_smtp_server_config_get] = ACTIONS(1496),
    [sym_smtp_send] = ACTIONS(1496),
    [sym_validate_netlinx_account] = ACTIONS(1496),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1496),
    [sym_authenticate_certificate] = ACTIONS(1496),
    [sym_audit_netlinx_session_event] = ACTIONS(1496),
    [sym_audit_netlinx_generic_event] = ACTIONS(1496),
    [sym_get_available_flash_disk_space] = ACTIONS(1496),
    [sym_get_max_flash_disk_space] = ACTIONS(1496),
    [sym_data_create_feed] = ACTIONS(1496),
    [sym_data_delete_feed] = ACTIONS(1496),
    [sym_data_publish_feed] = ACTIONS(1496),
    [sym_data_get_published_feed] = ACTIONS(1496),
    [sym_data_add_record] = ACTIONS(1496),
    [sym_data_get_event_record] = ACTIONS(1496),
    [sym__wc_data_create_feed] = ACTIONS(1496),
    [sym__wc_data_add_record] = ACTIONS(1496),
    [sym__wc_data_get_event_record] = ACTIONS(1496),
    [sym_format] = ACTIONS(1496),
    [sym_astro_clock] = ACTIONS(1496),
    [sym_set_outdoor_temperature] = ACTIONS(1496),
    [sym_set_virtual_level_count] = ACTIONS(1496),
    [sym_set_virtual_channel_count] = ACTIONS(1496),
    [sym_set_virtual_port_count] = ACTIONS(1496),
    [sym__wc_encode_int] = ACTIONS(1496),
    [sym__wc_decode_int] = ACTIONS(1496),
    [sym__wc_int] = ACTIONS(1496),
    [sym__wc_to_ch_int] = ACTIONS(1496),
    [sym__ch_to_wc_int] = ACTIONS(1496),
    [sym__wc_find_string_int] = ACTIONS(1496),
    [sym__wc_left_string_int] = ACTIONS(1496),
    [sym__wc_lower_string_int] = ACTIONS(1496),
    [sym__wc_mid_string_int] = ACTIONS(1496),
    [sym__wc_remove_string_int] = ACTIONS(1496),
    [sym__wc_right_string_int] = ACTIONS(1496),
    [sym__wc_upper_string_int] = ACTIONS(1496),
    [sym__wc_compare_string_int] = ACTIONS(1496),
    [sym__wc_get_buffer_char_int] = ACTIONS(1496),
    [sym__wc_get_buffer_string_int] = ACTIONS(1496),
    [sym__wc_concat_string_int] = ACTIONS(1496),
    [sym__wc_file_open_int] = ACTIONS(1496),
    [sym__wc_file_close_int] = ACTIONS(1496),
    [sym__wc_file_read_int] = ACTIONS(1496),
    [sym__wc_file_read_line_int] = ACTIONS(1496),
    [sym__wc_file_write_int] = ACTIONS(1496),
    [sym__wc_file_write_line_int] = ACTIONS(1496),
    [anon_sym_EQ] = ACTIONS(1496),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1498),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1498),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1498),
    [aux_sym_preproc_else_token1] = ACTIONS(1498),
    [anon_sym_COMMA] = ACTIONS(1498),
    [anon_sym_LBRACK] = ACTIONS(2205),
    [anon_sym_LBRACE] = ACTIONS(1498),
    [anon_sym_RBRACE] = ACTIONS(1498),
    [anon_sym_COLON] = ACTIONS(1498),
    [anon_sym_LPAREN] = ACTIONS(1496),
    [anon_sym_BANG] = ACTIONS(1496),
    [anon_sym_TILDE] = ACTIONS(1498),
    [anon_sym_DASH] = ACTIONS(1496),
    [anon_sym_PLUS] = ACTIONS(1496),
    [anon_sym_STAR] = ACTIONS(1498),
    [anon_sym_SLASH] = ACTIONS(1496),
    [anon_sym_PERCENT] = ACTIONS(1498),
    [anon_sym_PIPE_PIPE] = ACTIONS(1498),
    [anon_sym_AMP_AMP] = ACTIONS(1498),
    [anon_sym_PIPE] = ACTIONS(1496),
    [anon_sym_CARET] = ACTIONS(1498),
    [anon_sym_AMP] = ACTIONS(1496),
    [anon_sym_EQ_EQ] = ACTIONS(1498),
    [anon_sym_BANG_EQ] = ACTIONS(1498),
    [anon_sym_LT_GT] = ACTIONS(1498),
    [anon_sym_GT] = ACTIONS(1496),
    [anon_sym_GT_EQ] = ACTIONS(1498),
    [anon_sym_LT_EQ] = ACTIONS(1498),
    [anon_sym_LT] = ACTIONS(1496),
    [anon_sym_LT_LT] = ACTIONS(1498),
    [anon_sym_GT_GT] = ACTIONS(1498),
    [anon_sym_DASH_DASH] = ACTIONS(1498),
    [anon_sym_PLUS_PLUS] = ACTIONS(1498),
    [anon_sym_DOT] = ACTIONS(1496),
    [anon_sym_DQUOTE] = ACTIONS(1498),
    [anon_sym_SQUOTE] = ACTIONS(1498),
    [sym_number_literal] = ACTIONS(1498),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(577)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1473),
    [sym_and] = ACTIONS(1473),
    [sym_or] = ACTIONS(1473),
    [sym_not] = ACTIONS(1473),
    [sym_xor] = ACTIONS(1473),
    [sym_band] = ACTIONS(1473),
    [sym_bor] = ACTIONS(1473),
    [sym_bxor] = ACTIONS(1473),
    [sym_bnot] = ACTIONS(1473),
    [sym_lshift] = ACTIONS(1473),
    [sym_rshift] = ACTIONS(1473),
    [sym___file__] = ACTIONS(1473),
    [sym___date__] = ACTIONS(1473),
    [sym___ldate__] = ACTIONS(1473),
    [sym___time__] = ACTIONS(1473),
    [sym___line__] = ACTIONS(1473),
    [sym___name__] = ACTIONS(1473),
    [sym___netlinx__] = ACTIONS(1473),
    [sym_netlinx_axi_version] = ACTIONS(1473),
    [sym_url_flg_tcp] = ACTIONS(1473),
    [sym_url_flg_accinfopresent] = ACTIONS(1473),
    [sym_url_flg_temp] = ACTIONS(1473),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1473),
    [sym_url_flg_stat_mask] = ACTIONS(1473),
    [sym_url_flg_stat_lookup] = ACTIONS(1473),
    [sym_url_flg_stat_connecting] = ACTIONS(1473),
    [sym_url_flg_stat_waiting] = ACTIONS(1473),
    [sym_url_flg_stat_connected] = ACTIONS(1473),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1473),
    [sym_first_virtual_device] = ACTIONS(1473),
    [sym_first_local_port] = ACTIONS(1473),
    [sym_dynamic_virtual_device] = ACTIONS(1473),
    [sym_timeline_once] = ACTIONS(1473),
    [sym_timeline_repeat] = ACTIONS(1473),
    [sym_timeline_absolute] = ACTIONS(1473),
    [sym_timeline_relative] = ACTIONS(1473),
    [sym_do_push_timed_infinite] = ACTIONS(1473),
    [sym_source_type_no_address] = ACTIONS(1473),
    [sym_source_type_neuron_id] = ACTIONS(1473),
    [sym_source_type_ip_address] = ACTIONS(1473),
    [sym_source_type_axlink] = ACTIONS(1473),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1473),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1473),
    [sym_source_type_ip_socket_address] = ACTIONS(1473),
    [sym_source_type_rs232] = ACTIONS(1473),
    [sym_source_type_internal] = ACTIONS(1473),
    [sym_source_type_mac_address] = ACTIONS(1473),
    [sym_source_type_ipv4_port] = ACTIONS(1473),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1473),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1473),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1473),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1473),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1473),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1473),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1473),
    [sym_true] = ACTIONS(1473),
    [sym_false] = ACTIONS(1473),
    [sym_file_read_only] = ACTIONS(1473),
    [sym_file_rw_new] = ACTIONS(1473),
    [sym_file_rw_append] = ACTIONS(1473),
    [sym_ip_tcp] = ACTIONS(1473),
    [sym_ip_udp] = ACTIONS(1473),
    [sym_ip_udp_2way] = ACTIONS(1473),
    [sym_xml_encode_types] = ACTIONS(1473),
    [sym_xml_encode_char_as_list] = ACTIONS(1473),
    [sym_xml_encode_le] = ACTIONS(1473),
    [sym_xml_decode_types] = ACTIONS(1473),
    [sym_xml_decode_no_preserve] = ACTIONS(1473),
    [sym_duet_dev_type_display_device] = ACTIONS(1473),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1473),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1473),
    [sym_duet_dev_type_io_device] = ACTIONS(1473),
    [sym_duet_dev_type_relay_device] = ACTIONS(1473),
    [sym_duet_dev_type_ups] = ACTIONS(1473),
    [sym_duet_dev_type_amplifier] = ACTIONS(1473),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1473),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1473),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1473),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1473),
    [sym_duet_dev_type_camera] = ACTIONS(1473),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1473),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1473),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1473),
    [sym_duet_dev_type_dss] = ACTIONS(1473),
    [sym_duet_dev_type_dvr] = ACTIONS(1473),
    [sym_duet_dev_type_disc_device] = ACTIONS(1473),
    [sym_duet_dev_type_document_camera] = ACTIONS(1473),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1473),
    [sym_duet_dev_type_hvac] = ACTIONS(1473),
    [sym_duet_dev_type_keypad] = ACTIONS(1473),
    [sym_duet_dev_type_light] = ACTIONS(1473),
    [sym_duet_dev_type_monitor] = ACTIONS(1473),
    [sym_duet_dev_type_motor] = ACTIONS(1473),
    [sym_duet_dev_type_multi_window] = ACTIONS(1473),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1473),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1473),
    [sym_duet_dev_type_receiver] = ACTIONS(1473),
    [sym_duet_dev_type_security_system] = ACTIONS(1473),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1473),
    [sym_duet_dev_type_settop_box] = ACTIONS(1473),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1473),
    [sym_duet_dev_type_switcher] = ACTIONS(1473),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1473),
    [sym_duet_dev_type_tv] = ACTIONS(1473),
    [sym_duet_dev_type_utility] = ACTIONS(1473),
    [sym_duet_dev_type_vcr] = ACTIONS(1473),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1473),
    [sym_duet_dev_type_video_processor] = ACTIONS(1473),
    [sym_duet_dev_type_video_projector] = ACTIONS(1473),
    [sym_duet_dev_type_video_wall] = ACTIONS(1473),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1473),
    [sym_duet_dev_type_weather] = ACTIONS(1473),
    [sym_duet_dev_not_polled] = ACTIONS(1473),
    [sym_duet_dev_polled] = ACTIONS(1473),
    [sym_clkmgr_mode_network] = ACTIONS(1473),
    [sym_clkmgr_mode_standalone] = ACTIONS(1473),
    [sym_normal_standby] = ACTIONS(1473),
    [sym_normal_wake] = ACTIONS(1473),
    [sym_ip_multicast_ttl_option] = ACTIONS(1473),
    [sym_ip_ttl_subnet] = ACTIONS(1473),
    [sym_ip_ttl_site] = ACTIONS(1473),
    [sym_ip_ttl_region] = ACTIONS(1473),
    [sym_ip_ttl_continent] = ACTIONS(1473),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1473),
    [sym_ip_nodelay_on] = ACTIONS(1473),
    [sym_ip_nodelay_off] = ACTIONS(1473),
    [sym_amx_error] = ACTIONS(1473),
    [sym_amx_warning] = ACTIONS(1473),
    [sym_amx_info] = ACTIONS(1473),
    [sym_amx_debug] = ACTIONS(1473),
    [sym_netlinx_login_success] = ACTIONS(1473),
    [sym_netlinx_login_fail] = ACTIONS(1473),
    [sym_netlinx_logout] = ACTIONS(1473),
    [sym_null_str] = ACTIONS(1473),
    [sym_smtp_address] = ACTIONS(1473),
    [sym_smtp_port_number] = ACTIONS(1473),
    [sym_smtp_username] = ACTIONS(1473),
    [sym_smtp_password] = ACTIONS(1473),
    [sym_smtp_require_tls] = ACTIONS(1473),
    [sym_smtp_from] = ACTIONS(1473),
    [sym_smtp_tls_true] = ACTIONS(1473),
    [sym_smtp_tls_false] = ACTIONS(1473),
    [sym_listview_on_row_select_event] = ACTIONS(1473),
    [sym_data_structure_unknown] = ACTIONS(1473),
    [sym_data_structure_datafeed] = ACTIONS(1473),
    [sym_data_structure_datarecordset] = ACTIONS(1473),
    [sym_data_structure_datarecord] = ACTIONS(1473),
    [sym_data_structure_datafield] = ACTIONS(1473),
    [sym_data_max_content_field_count] = ACTIONS(1473),
    [sym_data_max_metadata_field_count] = ACTIONS(1473),
    [sym_data_max_id_length] = ACTIONS(1473),
    [sym_data_max_name_length] = ACTIONS(1473),
    [sym_data_max_description_length] = ACTIONS(1473),
    [sym_data_max_type_length] = ACTIONS(1473),
    [sym_data_max_format_length] = ACTIONS(1473),
    [sym_data_max_source_length] = ACTIONS(1473),
    [sym_data_max_value_length] = ACTIONS(1473),
    [sym_data_max_label_length] = ACTIONS(1473),
    [sym_data_max_url_length] = ACTIONS(1473),
    [sym_max_last_login_info_length] = ACTIONS(1473),
    [sym_data_type_unknown] = ACTIONS(1473),
    [sym_data_type_string] = ACTIONS(1473),
    [sym_data_type_datetime] = ACTIONS(1473),
    [sym_data_type_date] = ACTIONS(1473),
    [sym_data_type_time] = ACTIONS(1473),
    [sym_data_type_image] = ACTIONS(1473),
    [sym_data_format_url] = ACTIONS(1473),
    [sym_data_format_phone] = ACTIONS(1473),
    [sym_data_format_email] = ACTIONS(1473),
    [sym_data_format_iso8601] = ACTIONS(1473),
    [sym_auditlog_priv] = ACTIONS(1473),
    [sym_configuration_priv] = ACTIONS(1473),
    [sym_deviceconfig_priv] = ACTIONS(1473),
    [sym_ftp_priv] = ACTIONS(1473),
    [sym_http_priv] = ACTIONS(1473),
    [sym_networkconfig_priv] = ACTIONS(1473),
    [sym_programport_priv] = ACTIONS(1473),
    [sym_remoteui_priv] = ACTIONS(1473),
    [sym_securitycontrol_priv] = ACTIONS(1473),
    [sym_softwaremanagement_priv] = ACTIONS(1473),
    [sym_terminal_priv] = ACTIONS(1473),
    [sym_tpadmin_priv] = ACTIONS(1473),
    [sym_user1_priv] = ACTIONS(1473),
    [sym_user2_priv] = ACTIONS(1473),
    [sym_user3_priv] = ACTIONS(1473),
    [sym_user4_priv] = ACTIONS(1473),
    [sym_usermanagement_priv] = ACTIONS(1473),
    [sym_valid_account] = ACTIONS(1473),
    [sym_err_name_invalid] = ACTIONS(1473),
    [sym_err_password_invalid] = ACTIONS(1473),
    [sym_err_invalid_account] = ACTIONS(1473),
    [sym_err_no_authorization] = ACTIONS(1473),
    [sym_err_invalid_parameter] = ACTIONS(1473),
    [sym_err_user_locked_out] = ACTIONS(1473),
    [sym_err_user_account_expired] = ACTIONS(1473),
    [sym_err_general_error] = ACTIONS(1473),
    [sym_err_authentication_error] = ACTIONS(1473),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1473),
    [sym_tls_validate_certificate] = ACTIONS(1473),
    [sym____reserved___] = ACTIONS(1473),
    [sym_date] = ACTIONS(1473),
    [sym_day] = ACTIONS(1473),
    [sym_time] = ACTIONS(1473),
    [sym_ldate] = ACTIONS(1473),
    [sym_push_device] = ACTIONS(1473),
    [sym_push_channel] = ACTIONS(1473),
    [sym_push_devchan] = ACTIONS(1473),
    [sym_release_device] = ACTIONS(1473),
    [sym_release_channel] = ACTIONS(1473),
    [sym_release_devchan] = ACTIONS(1473),
    [sym_master_slot] = ACTIONS(1473),
    [sym_get_pulse_time] = ACTIONS(1473),
    [sym_get_timer] = ACTIONS(1473),
    [sym_master_sn] = ACTIONS(1473),
    [sym_system_number] = ACTIONS(1473),
    [sym_button] = ACTIONS(1473),
    [sym_dv_channel] = ACTIONS(1473),
    [sym_data] = ACTIONS(1473),
    [sym_level] = ACTIONS(1473),
    [sym_timeline] = ACTIONS(1473),
    [sym_channel] = ACTIONS(1473),
    [sym_custom] = ACTIONS(1473),
    [sym_atoi] = ACTIONS(1473),
    [sym_atol] = ACTIONS(1473),
    [sym_atof] = ACTIONS(1473),
    [sym_itoa] = ACTIONS(1473),
    [sym_itohex] = ACTIONS(1473),
    [sym_hextoi] = ACTIONS(1473),
    [sym_ftoa] = ACTIONS(1473),
    [sym_find_string] = ACTIONS(1473),
    [sym_left_string] = ACTIONS(1473),
    [sym_right_string] = ACTIONS(1473),
    [sym_mid_string] = ACTIONS(1473),
    [sym_remove_string] = ACTIONS(1473),
    [sym_get_buffer_string] = ACTIONS(1473),
    [sym_get_buffer_char] = ACTIONS(1473),
    [sym_get_multi_buffer_string] = ACTIONS(1473),
    [sym_length_string] = ACTIONS(1473),
    [sym_max_length_string] = ACTIONS(1473),
    [sym_set_length_string] = ACTIONS(1473),
    [sym_compare_string] = ACTIONS(1473),
    [sym_upper_string] = ACTIONS(1473),
    [sym_lower_string] = ACTIONS(1473),
    [sym_get_last] = ACTIONS(1473),
    [sym_date_to_day] = ACTIONS(1473),
    [sym_date_to_month] = ACTIONS(1473),
    [sym_date_to_year] = ACTIONS(1473),
    [sym_time_to_hour] = ACTIONS(1473),
    [sym_time_to_minute] = ACTIONS(1473),
    [sym_time_to_second] = ACTIONS(1473),
    [sym_day_of_week] = ACTIONS(1473),
    [sym_set_pulse_time] = ACTIONS(1473),
    [sym_set_timer] = ACTIONS(1473),
    [sym_file_open] = ACTIONS(1473),
    [sym_file_close] = ACTIONS(1473),
    [sym_file_read] = ACTIONS(1473),
    [sym_file_read_line] = ACTIONS(1473),
    [sym_file_write] = ACTIONS(1473),
    [sym_file_write_line] = ACTIONS(1473),
    [sym_file_copy] = ACTIONS(1473),
    [sym_file_delete] = ACTIONS(1473),
    [sym_file_rename] = ACTIONS(1473),
    [sym_file_dir] = ACTIONS(1473),
    [sym_file_getdir] = ACTIONS(1473),
    [sym_file_setdir] = ACTIONS(1473),
    [sym_file_create_dir] = ACTIONS(1473),
    [sym_file_remove_dir] = ACTIONS(1473),
    [sym_file_seek] = ACTIONS(1473),
    [sym_length_array] = ACTIONS(1473),
    [sym_max_length_array] = ACTIONS(1473),
    [sym_set_length_array] = ACTIONS(1473),
    [sym_ip_client_open] = ACTIONS(1473),
    [sym_ip_client_close] = ACTIONS(1473),
    [sym_ip_server_open] = ACTIONS(1473),
    [sym_ip_server_close] = ACTIONS(1473),
    [sym_ip_bound_client_open] = ACTIONS(1473),
    [sym_ip_mc_server_open] = ACTIONS(1473),
    [sym_ip_set_option] = ACTIONS(1473),
    [sym_ssh_client_open] = ACTIONS(1473),
    [sym_ssh_client_close] = ACTIONS(1473),
    [sym_tls_client_open] = ACTIONS(1473),
    [sym_tls_client_close] = ACTIONS(1473),
    [sym_do_push] = ACTIONS(1473),
    [sym_do_push_timed] = ACTIONS(1473),
    [sym_do_release] = ACTIONS(1473),
    [sym_device_id] = ACTIONS(1473),
    [sym_device_info] = ACTIONS(1473),
    [sym_device_id_string] = ACTIONS(1473),
    [sym_device_standby] = ACTIONS(1473),
    [sym_device_wake] = ACTIONS(1473),
    [sym_timeline_create] = ACTIONS(1473),
    [sym_timeline_kill] = ACTIONS(1473),
    [sym_timeline_reload] = ACTIONS(1473),
    [sym_timeline_pause] = ACTIONS(1473),
    [sym_timeline_restart] = ACTIONS(1473),
    [sym_timeline_set] = ACTIONS(1473),
    [sym_timeline_get] = ACTIONS(1473),
    [sym_timeline_active] = ACTIONS(1473),
    [sym_timeline_dynamic_id] = ACTIONS(1473),
    [sym_variable_to_string] = ACTIONS(1473),
    [sym_string_to_variable] = ACTIONS(1473),
    [sym_length_variable_to_string] = ACTIONS(1473),
    [sym_type_cast] = ACTIONS(1473),
    [sym_variable_to_xml] = ACTIONS(1473),
    [sym_xml_to_variable] = ACTIONS(1473),
    [sym_length_variable_to_xml] = ACTIONS(1473),
    [sym_redirect_string] = ACTIONS(1473),
    [sym_random_number] = ACTIONS(1473),
    [sym_get_unique_id] = ACTIONS(1473),
    [sym_get_system_number] = ACTIONS(1473),
    [sym_set_system_number] = ACTIONS(1473),
    [sym_get_serial_number] = ACTIONS(1473),
    [sym_get_master_build] = ACTIONS(1473),
    [sym_set_validation_code] = ACTIONS(1473),
    [sym_reboot] = ACTIONS(1473),
    [sym_load_duet_module] = ACTIONS(1473),
    [sym_unload_duet_module] = ACTIONS(1473),
    [sym_duet_mem_size_set] = ACTIONS(1473),
    [sym_duet_mem_size_get] = ACTIONS(1473),
    [sym_dynamic_application_device] = ACTIONS(1473),
    [sym_dynamic_polled_port] = ACTIONS(1473),
    [sym_static_port_binding] = ACTIONS(1473),
    [sym_static_ip_binding] = ACTIONS(1473),
    [sym_get_dns_list] = ACTIONS(1473),
    [sym_set_dns_list] = ACTIONS(1473),
    [sym_get_ip_address] = ACTIONS(1473),
    [sym_set_ip_address] = ACTIONS(1473),
    [sym_get_url_list] = ACTIONS(1473),
    [sym_add_url_entry] = ACTIONS(1473),
    [sym_delete_url_entry] = ACTIONS(1473),
    [sym_abs_value] = ACTIONS(1473),
    [sym_max_value] = ACTIONS(1473),
    [sym_min_value] = ACTIONS(1473),
    [sym_exp_value] = ACTIONS(1473),
    [sym_log_value] = ACTIONS(1473),
    [sym_log10_value] = ACTIONS(1473),
    [sym_power_value] = ACTIONS(1473),
    [sym_sqrt_value] = ACTIONS(1473),
    [sym_raw_be] = ACTIONS(1473),
    [sym_raw_le] = ACTIONS(1473),
    [sym_internal_queue_size_set] = ACTIONS(1473),
    [sym_internal_queue_size_get] = ACTIONS(1473),
    [sym_internal_threshold_set] = ACTIONS(1473),
    [sym_internal_threshold_get] = ACTIONS(1473),
    [sym_rebuilt_event] = ACTIONS(1473),
    [sym_do_custom_event] = ACTIONS(1473),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1473),
    [sym_clkmgr_set_clk_source] = ACTIONS(1473),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1473),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1473),
    [sym_clkmgr_get_timezone] = ACTIONS(1473),
    [sym_clkmgr_set_timezone] = ACTIONS(1473),
    [sym_clkmgr_get_resync_period] = ACTIONS(1473),
    [sym_clkmgr_set_resync_period] = ACTIONS(1473),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1473),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1473),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1473),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1473),
    [sym_clkmgr_get_timeservers] = ACTIONS(1473),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1473),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1473),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1473),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1473),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1473),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1473),
    [sym_set_log_level] = ACTIONS(1473),
    [sym_get_log_level] = ACTIONS(1473),
    [sym_amx_log] = ACTIONS(1473),
    [sym_smtp_server_config_set] = ACTIONS(1473),
    [sym_smtp_server_config_get] = ACTIONS(1473),
    [sym_smtp_send] = ACTIONS(1473),
    [sym_validate_netlinx_account] = ACTIONS(1473),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1473),
    [sym_authenticate_certificate] = ACTIONS(1473),
    [sym_audit_netlinx_session_event] = ACTIONS(1473),
    [sym_audit_netlinx_generic_event] = ACTIONS(1473),
    [sym_get_available_flash_disk_space] = ACTIONS(1473),
    [sym_get_max_flash_disk_space] = ACTIONS(1473),
    [sym_data_create_feed] = ACTIONS(1473),
    [sym_data_delete_feed] = ACTIONS(1473),
    [sym_data_publish_feed] = ACTIONS(1473),
    [sym_data_get_published_feed] = ACTIONS(1473),
    [sym_data_add_record] = ACTIONS(1473),
    [sym_data_get_event_record] = ACTIONS(1473),
    [sym__wc_data_create_feed] = ACTIONS(1473),
    [sym__wc_data_add_record] = ACTIONS(1473),
    [sym__wc_data_get_event_record] = ACTIONS(1473),
    [sym_format] = ACTIONS(1473),
    [sym_astro_clock] = ACTIONS(1473),
    [sym_set_outdoor_temperature] = ACTIONS(1473),
    [sym_set_virtual_level_count] = ACTIONS(1473),
    [sym_set_virtual_channel_count] = ACTIONS(1473),
    [sym_set_virtual_port_count] = ACTIONS(1473),
    [sym__wc_encode_int] = ACTIONS(1473),
    [sym__wc_decode_int] = ACTIONS(1473),
    [sym__wc_int] = ACTIONS(1473),
    [sym__wc_to_ch_int] = ACTIONS(1473),
    [sym__ch_to_wc_int] = ACTIONS(1473),
    [sym__wc_find_string_int] = ACTIONS(1473),
    [sym__wc_left_string_int] = ACTIONS(1473),
    [sym__wc_lower_string_int] = ACTIONS(1473),
    [sym__wc_mid_string_int] = ACTIONS(1473),
    [sym__wc_remove_string_int] = ACTIONS(1473),
    [sym__wc_right_string_int] = ACTIONS(1473),
    [sym__wc_upper_string_int] = ACTIONS(1473),
    [sym__wc_compare_string_int] = ACTIONS(1473),
    [sym__wc_get_buffer_char_int] = ACTIONS(1473),
    [sym__wc_get_buffer_string_int] = ACTIONS(1473),
    [sym__wc_concat_string_int] = ACTIONS(1473),
    [sym__wc_file_open_int] = ACTIONS(1473),
    [sym__wc_file_close_int] = ACTIONS(1473),
    [sym__wc_file_read_int] = ACTIONS(1473),
    [sym__wc_file_read_line_int] = ACTIONS(1473),
    [sym__wc_file_write_int] = ACTIONS(1473),
    [sym__wc_file_write_line_int] = ACTIONS(1473),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1475),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1475),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1475),
    [aux_sym_preproc_else_token1] = ACTIONS(1475),
    [anon_sym_COMMA] = ACTIONS(1475),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1475),
    [anon_sym_RBRACE] = ACTIONS(1475),
    [anon_sym_COLON] = ACTIONS(1475),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1473),
    [anon_sym_TILDE] = ACTIONS(1475),
    [anon_sym_DASH] = ACTIONS(1473),
    [anon_sym_PLUS] = ACTIONS(1473),
    [anon_sym_STAR] = ACTIONS(1475),
    [anon_sym_SLASH] = ACTIONS(1473),
    [anon_sym_PERCENT] = ACTIONS(1475),
    [anon_sym_PIPE_PIPE] = ACTIONS(1475),
    [anon_sym_AMP_AMP] = ACTIONS(1475),
    [anon_sym_PIPE] = ACTIONS(1473),
    [anon_sym_CARET] = ACTIONS(1475),
    [anon_sym_AMP] = ACTIONS(1473),
    [anon_sym_EQ_EQ] = ACTIONS(1475),
    [anon_sym_BANG_EQ] = ACTIONS(1475),
    [anon_sym_LT_GT] = ACTIONS(1475),
    [anon_sym_GT] = ACTIONS(1473),
    [anon_sym_GT_EQ] = ACTIONS(1475),
    [anon_sym_LT_EQ] = ACTIONS(1475),
    [anon_sym_LT] = ACTIONS(1473),
    [anon_sym_LT_LT] = ACTIONS(1475),
    [anon_sym_GT_GT] = ACTIONS(1475),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1475),
    [anon_sym_SQUOTE] = ACTIONS(1475),
    [sym_number_literal] = ACTIONS(1475),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(578)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1477),
    [sym_and] = ACTIONS(1477),
    [sym_or] = ACTIONS(1477),
    [sym_not] = ACTIONS(1477),
    [sym_xor] = ACTIONS(1477),
    [sym_band] = ACTIONS(1477),
    [sym_bor] = ACTIONS(1477),
    [sym_bxor] = ACTIONS(1477),
    [sym_bnot] = ACTIONS(1477),
    [sym_lshift] = ACTIONS(1477),
    [sym_rshift] = ACTIONS(1477),
    [sym___file__] = ACTIONS(1477),
    [sym___date__] = ACTIONS(1477),
    [sym___ldate__] = ACTIONS(1477),
    [sym___time__] = ACTIONS(1477),
    [sym___line__] = ACTIONS(1477),
    [sym___name__] = ACTIONS(1477),
    [sym___netlinx__] = ACTIONS(1477),
    [sym_netlinx_axi_version] = ACTIONS(1477),
    [sym_url_flg_tcp] = ACTIONS(1477),
    [sym_url_flg_accinfopresent] = ACTIONS(1477),
    [sym_url_flg_temp] = ACTIONS(1477),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1477),
    [sym_url_flg_stat_mask] = ACTIONS(1477),
    [sym_url_flg_stat_lookup] = ACTIONS(1477),
    [sym_url_flg_stat_connecting] = ACTIONS(1477),
    [sym_url_flg_stat_waiting] = ACTIONS(1477),
    [sym_url_flg_stat_connected] = ACTIONS(1477),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1477),
    [sym_first_virtual_device] = ACTIONS(1477),
    [sym_first_local_port] = ACTIONS(1477),
    [sym_dynamic_virtual_device] = ACTIONS(1477),
    [sym_timeline_once] = ACTIONS(1477),
    [sym_timeline_repeat] = ACTIONS(1477),
    [sym_timeline_absolute] = ACTIONS(1477),
    [sym_timeline_relative] = ACTIONS(1477),
    [sym_do_push_timed_infinite] = ACTIONS(1477),
    [sym_source_type_no_address] = ACTIONS(1477),
    [sym_source_type_neuron_id] = ACTIONS(1477),
    [sym_source_type_ip_address] = ACTIONS(1477),
    [sym_source_type_axlink] = ACTIONS(1477),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1477),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1477),
    [sym_source_type_ip_socket_address] = ACTIONS(1477),
    [sym_source_type_rs232] = ACTIONS(1477),
    [sym_source_type_internal] = ACTIONS(1477),
    [sym_source_type_mac_address] = ACTIONS(1477),
    [sym_source_type_ipv4_port] = ACTIONS(1477),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1477),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1477),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1477),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1477),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1477),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1477),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1477),
    [sym_true] = ACTIONS(1477),
    [sym_false] = ACTIONS(1477),
    [sym_file_read_only] = ACTIONS(1477),
    [sym_file_rw_new] = ACTIONS(1477),
    [sym_file_rw_append] = ACTIONS(1477),
    [sym_ip_tcp] = ACTIONS(1477),
    [sym_ip_udp] = ACTIONS(1477),
    [sym_ip_udp_2way] = ACTIONS(1477),
    [sym_xml_encode_types] = ACTIONS(1477),
    [sym_xml_encode_char_as_list] = ACTIONS(1477),
    [sym_xml_encode_le] = ACTIONS(1477),
    [sym_xml_decode_types] = ACTIONS(1477),
    [sym_xml_decode_no_preserve] = ACTIONS(1477),
    [sym_duet_dev_type_display_device] = ACTIONS(1477),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1477),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1477),
    [sym_duet_dev_type_io_device] = ACTIONS(1477),
    [sym_duet_dev_type_relay_device] = ACTIONS(1477),
    [sym_duet_dev_type_ups] = ACTIONS(1477),
    [sym_duet_dev_type_amplifier] = ACTIONS(1477),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1477),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1477),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1477),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1477),
    [sym_duet_dev_type_camera] = ACTIONS(1477),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1477),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1477),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1477),
    [sym_duet_dev_type_dss] = ACTIONS(1477),
    [sym_duet_dev_type_dvr] = ACTIONS(1477),
    [sym_duet_dev_type_disc_device] = ACTIONS(1477),
    [sym_duet_dev_type_document_camera] = ACTIONS(1477),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1477),
    [sym_duet_dev_type_hvac] = ACTIONS(1477),
    [sym_duet_dev_type_keypad] = ACTIONS(1477),
    [sym_duet_dev_type_light] = ACTIONS(1477),
    [sym_duet_dev_type_monitor] = ACTIONS(1477),
    [sym_duet_dev_type_motor] = ACTIONS(1477),
    [sym_duet_dev_type_multi_window] = ACTIONS(1477),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1477),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1477),
    [sym_duet_dev_type_receiver] = ACTIONS(1477),
    [sym_duet_dev_type_security_system] = ACTIONS(1477),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1477),
    [sym_duet_dev_type_settop_box] = ACTIONS(1477),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1477),
    [sym_duet_dev_type_switcher] = ACTIONS(1477),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1477),
    [sym_duet_dev_type_tv] = ACTIONS(1477),
    [sym_duet_dev_type_utility] = ACTIONS(1477),
    [sym_duet_dev_type_vcr] = ACTIONS(1477),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1477),
    [sym_duet_dev_type_video_processor] = ACTIONS(1477),
    [sym_duet_dev_type_video_projector] = ACTIONS(1477),
    [sym_duet_dev_type_video_wall] = ACTIONS(1477),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1477),
    [sym_duet_dev_type_weather] = ACTIONS(1477),
    [sym_duet_dev_not_polled] = ACTIONS(1477),
    [sym_duet_dev_polled] = ACTIONS(1477),
    [sym_clkmgr_mode_network] = ACTIONS(1477),
    [sym_clkmgr_mode_standalone] = ACTIONS(1477),
    [sym_normal_standby] = ACTIONS(1477),
    [sym_normal_wake] = ACTIONS(1477),
    [sym_ip_multicast_ttl_option] = ACTIONS(1477),
    [sym_ip_ttl_subnet] = ACTIONS(1477),
    [sym_ip_ttl_site] = ACTIONS(1477),
    [sym_ip_ttl_region] = ACTIONS(1477),
    [sym_ip_ttl_continent] = ACTIONS(1477),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1477),
    [sym_ip_nodelay_on] = ACTIONS(1477),
    [sym_ip_nodelay_off] = ACTIONS(1477),
    [sym_amx_error] = ACTIONS(1477),
    [sym_amx_warning] = ACTIONS(1477),
    [sym_amx_info] = ACTIONS(1477),
    [sym_amx_debug] = ACTIONS(1477),
    [sym_netlinx_login_success] = ACTIONS(1477),
    [sym_netlinx_login_fail] = ACTIONS(1477),
    [sym_netlinx_logout] = ACTIONS(1477),
    [sym_null_str] = ACTIONS(1477),
    [sym_smtp_address] = ACTIONS(1477),
    [sym_smtp_port_number] = ACTIONS(1477),
    [sym_smtp_username] = ACTIONS(1477),
    [sym_smtp_password] = ACTIONS(1477),
    [sym_smtp_require_tls] = ACTIONS(1477),
    [sym_smtp_from] = ACTIONS(1477),
    [sym_smtp_tls_true] = ACTIONS(1477),
    [sym_smtp_tls_false] = ACTIONS(1477),
    [sym_listview_on_row_select_event] = ACTIONS(1477),
    [sym_data_structure_unknown] = ACTIONS(1477),
    [sym_data_structure_datafeed] = ACTIONS(1477),
    [sym_data_structure_datarecordset] = ACTIONS(1477),
    [sym_data_structure_datarecord] = ACTIONS(1477),
    [sym_data_structure_datafield] = ACTIONS(1477),
    [sym_data_max_content_field_count] = ACTIONS(1477),
    [sym_data_max_metadata_field_count] = ACTIONS(1477),
    [sym_data_max_id_length] = ACTIONS(1477),
    [sym_data_max_name_length] = ACTIONS(1477),
    [sym_data_max_description_length] = ACTIONS(1477),
    [sym_data_max_type_length] = ACTIONS(1477),
    [sym_data_max_format_length] = ACTIONS(1477),
    [sym_data_max_source_length] = ACTIONS(1477),
    [sym_data_max_value_length] = ACTIONS(1477),
    [sym_data_max_label_length] = ACTIONS(1477),
    [sym_data_max_url_length] = ACTIONS(1477),
    [sym_max_last_login_info_length] = ACTIONS(1477),
    [sym_data_type_unknown] = ACTIONS(1477),
    [sym_data_type_string] = ACTIONS(1477),
    [sym_data_type_datetime] = ACTIONS(1477),
    [sym_data_type_date] = ACTIONS(1477),
    [sym_data_type_time] = ACTIONS(1477),
    [sym_data_type_image] = ACTIONS(1477),
    [sym_data_format_url] = ACTIONS(1477),
    [sym_data_format_phone] = ACTIONS(1477),
    [sym_data_format_email] = ACTIONS(1477),
    [sym_data_format_iso8601] = ACTIONS(1477),
    [sym_auditlog_priv] = ACTIONS(1477),
    [sym_configuration_priv] = ACTIONS(1477),
    [sym_deviceconfig_priv] = ACTIONS(1477),
    [sym_ftp_priv] = ACTIONS(1477),
    [sym_http_priv] = ACTIONS(1477),
    [sym_networkconfig_priv] = ACTIONS(1477),
    [sym_programport_priv] = ACTIONS(1477),
    [sym_remoteui_priv] = ACTIONS(1477),
    [sym_securitycontrol_priv] = ACTIONS(1477),
    [sym_softwaremanagement_priv] = ACTIONS(1477),
    [sym_terminal_priv] = ACTIONS(1477),
    [sym_tpadmin_priv] = ACTIONS(1477),
    [sym_user1_priv] = ACTIONS(1477),
    [sym_user2_priv] = ACTIONS(1477),
    [sym_user3_priv] = ACTIONS(1477),
    [sym_user4_priv] = ACTIONS(1477),
    [sym_usermanagement_priv] = ACTIONS(1477),
    [sym_valid_account] = ACTIONS(1477),
    [sym_err_name_invalid] = ACTIONS(1477),
    [sym_err_password_invalid] = ACTIONS(1477),
    [sym_err_invalid_account] = ACTIONS(1477),
    [sym_err_no_authorization] = ACTIONS(1477),
    [sym_err_invalid_parameter] = ACTIONS(1477),
    [sym_err_user_locked_out] = ACTIONS(1477),
    [sym_err_user_account_expired] = ACTIONS(1477),
    [sym_err_general_error] = ACTIONS(1477),
    [sym_err_authentication_error] = ACTIONS(1477),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1477),
    [sym_tls_validate_certificate] = ACTIONS(1477),
    [sym____reserved___] = ACTIONS(1477),
    [sym_date] = ACTIONS(1477),
    [sym_day] = ACTIONS(1477),
    [sym_time] = ACTIONS(1477),
    [sym_ldate] = ACTIONS(1477),
    [sym_push_device] = ACTIONS(1477),
    [sym_push_channel] = ACTIONS(1477),
    [sym_push_devchan] = ACTIONS(1477),
    [sym_release_device] = ACTIONS(1477),
    [sym_release_channel] = ACTIONS(1477),
    [sym_release_devchan] = ACTIONS(1477),
    [sym_master_slot] = ACTIONS(1477),
    [sym_get_pulse_time] = ACTIONS(1477),
    [sym_get_timer] = ACTIONS(1477),
    [sym_master_sn] = ACTIONS(1477),
    [sym_system_number] = ACTIONS(1477),
    [sym_button] = ACTIONS(1477),
    [sym_dv_channel] = ACTIONS(1477),
    [sym_data] = ACTIONS(1477),
    [sym_level] = ACTIONS(1477),
    [sym_timeline] = ACTIONS(1477),
    [sym_channel] = ACTIONS(1477),
    [sym_custom] = ACTIONS(1477),
    [sym_atoi] = ACTIONS(1477),
    [sym_atol] = ACTIONS(1477),
    [sym_atof] = ACTIONS(1477),
    [sym_itoa] = ACTIONS(1477),
    [sym_itohex] = ACTIONS(1477),
    [sym_hextoi] = ACTIONS(1477),
    [sym_ftoa] = ACTIONS(1477),
    [sym_find_string] = ACTIONS(1477),
    [sym_left_string] = ACTIONS(1477),
    [sym_right_string] = ACTIONS(1477),
    [sym_mid_string] = ACTIONS(1477),
    [sym_remove_string] = ACTIONS(1477),
    [sym_get_buffer_string] = ACTIONS(1477),
    [sym_get_buffer_char] = ACTIONS(1477),
    [sym_get_multi_buffer_string] = ACTIONS(1477),
    [sym_length_string] = ACTIONS(1477),
    [sym_max_length_string] = ACTIONS(1477),
    [sym_set_length_string] = ACTIONS(1477),
    [sym_compare_string] = ACTIONS(1477),
    [sym_upper_string] = ACTIONS(1477),
    [sym_lower_string] = ACTIONS(1477),
    [sym_get_last] = ACTIONS(1477),
    [sym_date_to_day] = ACTIONS(1477),
    [sym_date_to_month] = ACTIONS(1477),
    [sym_date_to_year] = ACTIONS(1477),
    [sym_time_to_hour] = ACTIONS(1477),
    [sym_time_to_minute] = ACTIONS(1477),
    [sym_time_to_second] = ACTIONS(1477),
    [sym_day_of_week] = ACTIONS(1477),
    [sym_set_pulse_time] = ACTIONS(1477),
    [sym_set_timer] = ACTIONS(1477),
    [sym_file_open] = ACTIONS(1477),
    [sym_file_close] = ACTIONS(1477),
    [sym_file_read] = ACTIONS(1477),
    [sym_file_read_line] = ACTIONS(1477),
    [sym_file_write] = ACTIONS(1477),
    [sym_file_write_line] = ACTIONS(1477),
    [sym_file_copy] = ACTIONS(1477),
    [sym_file_delete] = ACTIONS(1477),
    [sym_file_rename] = ACTIONS(1477),
    [sym_file_dir] = ACTIONS(1477),
    [sym_file_getdir] = ACTIONS(1477),
    [sym_file_setdir] = ACTIONS(1477),
    [sym_file_create_dir] = ACTIONS(1477),
    [sym_file_remove_dir] = ACTIONS(1477),
    [sym_file_seek] = ACTIONS(1477),
    [sym_length_array] = ACTIONS(1477),
    [sym_max_length_array] = ACTIONS(1477),
    [sym_set_length_array] = ACTIONS(1477),
    [sym_ip_client_open] = ACTIONS(1477),
    [sym_ip_client_close] = ACTIONS(1477),
    [sym_ip_server_open] = ACTIONS(1477),
    [sym_ip_server_close] = ACTIONS(1477),
    [sym_ip_bound_client_open] = ACTIONS(1477),
    [sym_ip_mc_server_open] = ACTIONS(1477),
    [sym_ip_set_option] = ACTIONS(1477),
    [sym_ssh_client_open] = ACTIONS(1477),
    [sym_ssh_client_close] = ACTIONS(1477),
    [sym_tls_client_open] = ACTIONS(1477),
    [sym_tls_client_close] = ACTIONS(1477),
    [sym_do_push] = ACTIONS(1477),
    [sym_do_push_timed] = ACTIONS(1477),
    [sym_do_release] = ACTIONS(1477),
    [sym_device_id] = ACTIONS(1477),
    [sym_device_info] = ACTIONS(1477),
    [sym_device_id_string] = ACTIONS(1477),
    [sym_device_standby] = ACTIONS(1477),
    [sym_device_wake] = ACTIONS(1477),
    [sym_timeline_create] = ACTIONS(1477),
    [sym_timeline_kill] = ACTIONS(1477),
    [sym_timeline_reload] = ACTIONS(1477),
    [sym_timeline_pause] = ACTIONS(1477),
    [sym_timeline_restart] = ACTIONS(1477),
    [sym_timeline_set] = ACTIONS(1477),
    [sym_timeline_get] = ACTIONS(1477),
    [sym_timeline_active] = ACTIONS(1477),
    [sym_timeline_dynamic_id] = ACTIONS(1477),
    [sym_variable_to_string] = ACTIONS(1477),
    [sym_string_to_variable] = ACTIONS(1477),
    [sym_length_variable_to_string] = ACTIONS(1477),
    [sym_type_cast] = ACTIONS(1477),
    [sym_variable_to_xml] = ACTIONS(1477),
    [sym_xml_to_variable] = ACTIONS(1477),
    [sym_length_variable_to_xml] = ACTIONS(1477),
    [sym_redirect_string] = ACTIONS(1477),
    [sym_random_number] = ACTIONS(1477),
    [sym_get_unique_id] = ACTIONS(1477),
    [sym_get_system_number] = ACTIONS(1477),
    [sym_set_system_number] = ACTIONS(1477),
    [sym_get_serial_number] = ACTIONS(1477),
    [sym_get_master_build] = ACTIONS(1477),
    [sym_set_validation_code] = ACTIONS(1477),
    [sym_reboot] = ACTIONS(1477),
    [sym_load_duet_module] = ACTIONS(1477),
    [sym_unload_duet_module] = ACTIONS(1477),
    [sym_duet_mem_size_set] = ACTIONS(1477),
    [sym_duet_mem_size_get] = ACTIONS(1477),
    [sym_dynamic_application_device] = ACTIONS(1477),
    [sym_dynamic_polled_port] = ACTIONS(1477),
    [sym_static_port_binding] = ACTIONS(1477),
    [sym_static_ip_binding] = ACTIONS(1477),
    [sym_get_dns_list] = ACTIONS(1477),
    [sym_set_dns_list] = ACTIONS(1477),
    [sym_get_ip_address] = ACTIONS(1477),
    [sym_set_ip_address] = ACTIONS(1477),
    [sym_get_url_list] = ACTIONS(1477),
    [sym_add_url_entry] = ACTIONS(1477),
    [sym_delete_url_entry] = ACTIONS(1477),
    [sym_abs_value] = ACTIONS(1477),
    [sym_max_value] = ACTIONS(1477),
    [sym_min_value] = ACTIONS(1477),
    [sym_exp_value] = ACTIONS(1477),
    [sym_log_value] = ACTIONS(1477),
    [sym_log10_value] = ACTIONS(1477),
    [sym_power_value] = ACTIONS(1477),
    [sym_sqrt_value] = ACTIONS(1477),
    [sym_raw_be] = ACTIONS(1477),
    [sym_raw_le] = ACTIONS(1477),
    [sym_internal_queue_size_set] = ACTIONS(1477),
    [sym_internal_queue_size_get] = ACTIONS(1477),
    [sym_internal_threshold_set] = ACTIONS(1477),
    [sym_internal_threshold_get] = ACTIONS(1477),
    [sym_rebuilt_event] = ACTIONS(1477),
    [sym_do_custom_event] = ACTIONS(1477),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1477),
    [sym_clkmgr_set_clk_source] = ACTIONS(1477),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1477),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1477),
    [sym_clkmgr_get_timezone] = ACTIONS(1477),
    [sym_clkmgr_set_timezone] = ACTIONS(1477),
    [sym_clkmgr_get_resync_period] = ACTIONS(1477),
    [sym_clkmgr_set_resync_period] = ACTIONS(1477),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1477),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1477),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1477),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1477),
    [sym_clkmgr_get_timeservers] = ACTIONS(1477),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1477),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1477),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1477),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1477),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1477),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1477),
    [sym_set_log_level] = ACTIONS(1477),
    [sym_get_log_level] = ACTIONS(1477),
    [sym_amx_log] = ACTIONS(1477),
    [sym_smtp_server_config_set] = ACTIONS(1477),
    [sym_smtp_server_config_get] = ACTIONS(1477),
    [sym_smtp_send] = ACTIONS(1477),
    [sym_validate_netlinx_account] = ACTIONS(1477),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1477),
    [sym_authenticate_certificate] = ACTIONS(1477),
    [sym_audit_netlinx_session_event] = ACTIONS(1477),
    [sym_audit_netlinx_generic_event] = ACTIONS(1477),
    [sym_get_available_flash_disk_space] = ACTIONS(1477),
    [sym_get_max_flash_disk_space] = ACTIONS(1477),
    [sym_data_create_feed] = ACTIONS(1477),
    [sym_data_delete_feed] = ACTIONS(1477),
    [sym_data_publish_feed] = ACTIONS(1477),
    [sym_data_get_published_feed] = ACTIONS(1477),
    [sym_data_add_record] = ACTIONS(1477),
    [sym_data_get_event_record] = ACTIONS(1477),
    [sym__wc_data_create_feed] = ACTIONS(1477),
    [sym__wc_data_add_record] = ACTIONS(1477),
    [sym__wc_data_get_event_record] = ACTIONS(1477),
    [sym_format] = ACTIONS(1477),
    [sym_astro_clock] = ACTIONS(1477),
    [sym_set_outdoor_temperature] = ACTIONS(1477),
    [sym_set_virtual_level_count] = ACTIONS(1477),
    [sym_set_virtual_channel_count] = ACTIONS(1477),
    [sym_set_virtual_port_count] = ACTIONS(1477),
    [sym__wc_encode_int] = ACTIONS(1477),
    [sym__wc_decode_int] = ACTIONS(1477),
    [sym__wc_int] = ACTIONS(1477),
    [sym__wc_to_ch_int] = ACTIONS(1477),
    [sym__ch_to_wc_int] = ACTIONS(1477),
    [sym__wc_find_string_int] = ACTIONS(1477),
    [sym__wc_left_string_int] = ACTIONS(1477),
    [sym__wc_lower_string_int] = ACTIONS(1477),
    [sym__wc_mid_string_int] = ACTIONS(1477),
    [sym__wc_remove_string_int] = ACTIONS(1477),
    [sym__wc_right_string_int] = ACTIONS(1477),
    [sym__wc_upper_string_int] = ACTIONS(1477),
    [sym__wc_compare_string_int] = ACTIONS(1477),
    [sym__wc_get_buffer_char_int] = ACTIONS(1477),
    [sym__wc_get_buffer_string_int] = ACTIONS(1477),
    [sym__wc_concat_string_int] = ACTIONS(1477),
    [sym__wc_file_open_int] = ACTIONS(1477),
    [sym__wc_file_close_int] = ACTIONS(1477),
    [sym__wc_file_read_int] = ACTIONS(1477),
    [sym__wc_file_read_line_int] = ACTIONS(1477),
    [sym__wc_file_write_int] = ACTIONS(1477),
    [sym__wc_file_write_line_int] = ACTIONS(1477),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1479),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1479),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1479),
    [aux_sym_preproc_else_token1] = ACTIONS(1479),
    [anon_sym_COMMA] = ACTIONS(1479),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1479),
    [anon_sym_RBRACE] = ACTIONS(1479),
    [anon_sym_COLON] = ACTIONS(1479),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1477),
    [anon_sym_TILDE] = ACTIONS(1479),
    [anon_sym_DASH] = ACTIONS(1477),
    [anon_sym_PLUS] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(1479),
    [anon_sym_SLASH] = ACTIONS(1477),
    [anon_sym_PERCENT] = ACTIONS(1479),
    [anon_sym_PIPE_PIPE] = ACTIONS(1479),
    [anon_sym_AMP_AMP] = ACTIONS(1479),
    [anon_sym_PIPE] = ACTIONS(1477),
    [anon_sym_CARET] = ACTIONS(1479),
    [anon_sym_AMP] = ACTIONS(1477),
    [anon_sym_EQ_EQ] = ACTIONS(1479),
    [anon_sym_BANG_EQ] = ACTIONS(1479),
    [anon_sym_LT_GT] = ACTIONS(1479),
    [anon_sym_GT] = ACTIONS(1477),
    [anon_sym_GT_EQ] = ACTIONS(1479),
    [anon_sym_LT_EQ] = ACTIONS(1479),
    [anon_sym_LT] = ACTIONS(1477),
    [anon_sym_LT_LT] = ACTIONS(1479),
    [anon_sym_GT_GT] = ACTIONS(1479),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1479),
    [anon_sym_SQUOTE] = ACTIONS(1479),
    [sym_number_literal] = ACTIONS(1479),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(579)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_COMMA] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(580)] = {
    [sym_identifier] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [anon_sym_EQ] = ACTIONS(2208),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_else_token1] = ACTIONS(1469),
    [anon_sym_COMMA] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_RBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [sym_range_operator] = ACTIONS(2210),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(581)] = {
    [aux_sym_subscript_expression_repeat1] = STATE(576),
    [sym_identifier] = ACTIONS(1488),
    [sym_and] = ACTIONS(1488),
    [sym_or] = ACTIONS(1488),
    [sym_not] = ACTIONS(1488),
    [sym_xor] = ACTIONS(1488),
    [sym_band] = ACTIONS(1488),
    [sym_bor] = ACTIONS(1488),
    [sym_bxor] = ACTIONS(1488),
    [sym_bnot] = ACTIONS(1488),
    [sym_lshift] = ACTIONS(1488),
    [sym_rshift] = ACTIONS(1488),
    [sym___file__] = ACTIONS(1488),
    [sym___date__] = ACTIONS(1488),
    [sym___ldate__] = ACTIONS(1488),
    [sym___time__] = ACTIONS(1488),
    [sym___line__] = ACTIONS(1488),
    [sym___name__] = ACTIONS(1488),
    [sym___netlinx__] = ACTIONS(1488),
    [sym_netlinx_axi_version] = ACTIONS(1488),
    [sym_url_flg_tcp] = ACTIONS(1488),
    [sym_url_flg_accinfopresent] = ACTIONS(1488),
    [sym_url_flg_temp] = ACTIONS(1488),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1488),
    [sym_url_flg_stat_mask] = ACTIONS(1488),
    [sym_url_flg_stat_lookup] = ACTIONS(1488),
    [sym_url_flg_stat_connecting] = ACTIONS(1488),
    [sym_url_flg_stat_waiting] = ACTIONS(1488),
    [sym_url_flg_stat_connected] = ACTIONS(1488),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1488),
    [sym_first_virtual_device] = ACTIONS(1488),
    [sym_first_local_port] = ACTIONS(1488),
    [sym_dynamic_virtual_device] = ACTIONS(1488),
    [sym_timeline_once] = ACTIONS(1488),
    [sym_timeline_repeat] = ACTIONS(1488),
    [sym_timeline_absolute] = ACTIONS(1488),
    [sym_timeline_relative] = ACTIONS(1488),
    [sym_do_push_timed_infinite] = ACTIONS(1488),
    [sym_source_type_no_address] = ACTIONS(1488),
    [sym_source_type_neuron_id] = ACTIONS(1488),
    [sym_source_type_ip_address] = ACTIONS(1488),
    [sym_source_type_axlink] = ACTIONS(1488),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1488),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1488),
    [sym_source_type_ip_socket_address] = ACTIONS(1488),
    [sym_source_type_rs232] = ACTIONS(1488),
    [sym_source_type_internal] = ACTIONS(1488),
    [sym_source_type_mac_address] = ACTIONS(1488),
    [sym_source_type_ipv4_port] = ACTIONS(1488),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1488),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1488),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1488),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1488),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1488),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1488),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1488),
    [sym_true] = ACTIONS(1488),
    [sym_false] = ACTIONS(1488),
    [sym_file_read_only] = ACTIONS(1488),
    [sym_file_rw_new] = ACTIONS(1488),
    [sym_file_rw_append] = ACTIONS(1488),
    [sym_ip_tcp] = ACTIONS(1488),
    [sym_ip_udp] = ACTIONS(1488),
    [sym_ip_udp_2way] = ACTIONS(1488),
    [sym_xml_encode_types] = ACTIONS(1488),
    [sym_xml_encode_char_as_list] = ACTIONS(1488),
    [sym_xml_encode_le] = ACTIONS(1488),
    [sym_xml_decode_types] = ACTIONS(1488),
    [sym_xml_decode_no_preserve] = ACTIONS(1488),
    [sym_duet_dev_type_display_device] = ACTIONS(1488),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1488),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1488),
    [sym_duet_dev_type_io_device] = ACTIONS(1488),
    [sym_duet_dev_type_relay_device] = ACTIONS(1488),
    [sym_duet_dev_type_ups] = ACTIONS(1488),
    [sym_duet_dev_type_amplifier] = ACTIONS(1488),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1488),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1488),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1488),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1488),
    [sym_duet_dev_type_camera] = ACTIONS(1488),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1488),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1488),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1488),
    [sym_duet_dev_type_dss] = ACTIONS(1488),
    [sym_duet_dev_type_dvr] = ACTIONS(1488),
    [sym_duet_dev_type_disc_device] = ACTIONS(1488),
    [sym_duet_dev_type_document_camera] = ACTIONS(1488),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1488),
    [sym_duet_dev_type_hvac] = ACTIONS(1488),
    [sym_duet_dev_type_keypad] = ACTIONS(1488),
    [sym_duet_dev_type_light] = ACTIONS(1488),
    [sym_duet_dev_type_monitor] = ACTIONS(1488),
    [sym_duet_dev_type_motor] = ACTIONS(1488),
    [sym_duet_dev_type_multi_window] = ACTIONS(1488),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1488),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1488),
    [sym_duet_dev_type_receiver] = ACTIONS(1488),
    [sym_duet_dev_type_security_system] = ACTIONS(1488),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1488),
    [sym_duet_dev_type_settop_box] = ACTIONS(1488),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1488),
    [sym_duet_dev_type_switcher] = ACTIONS(1488),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1488),
    [sym_duet_dev_type_tv] = ACTIONS(1488),
    [sym_duet_dev_type_utility] = ACTIONS(1488),
    [sym_duet_dev_type_vcr] = ACTIONS(1488),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1488),
    [sym_duet_dev_type_video_processor] = ACTIONS(1488),
    [sym_duet_dev_type_video_projector] = ACTIONS(1488),
    [sym_duet_dev_type_video_wall] = ACTIONS(1488),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1488),
    [sym_duet_dev_type_weather] = ACTIONS(1488),
    [sym_duet_dev_not_polled] = ACTIONS(1488),
    [sym_duet_dev_polled] = ACTIONS(1488),
    [sym_clkmgr_mode_network] = ACTIONS(1488),
    [sym_clkmgr_mode_standalone] = ACTIONS(1488),
    [sym_normal_standby] = ACTIONS(1488),
    [sym_normal_wake] = ACTIONS(1488),
    [sym_ip_multicast_ttl_option] = ACTIONS(1488),
    [sym_ip_ttl_subnet] = ACTIONS(1488),
    [sym_ip_ttl_site] = ACTIONS(1488),
    [sym_ip_ttl_region] = ACTIONS(1488),
    [sym_ip_ttl_continent] = ACTIONS(1488),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1488),
    [sym_ip_nodelay_on] = ACTIONS(1488),
    [sym_ip_nodelay_off] = ACTIONS(1488),
    [sym_amx_error] = ACTIONS(1488),
    [sym_amx_warning] = ACTIONS(1488),
    [sym_amx_info] = ACTIONS(1488),
    [sym_amx_debug] = ACTIONS(1488),
    [sym_netlinx_login_success] = ACTIONS(1488),
    [sym_netlinx_login_fail] = ACTIONS(1488),
    [sym_netlinx_logout] = ACTIONS(1488),
    [sym_null_str] = ACTIONS(1488),
    [sym_smtp_address] = ACTIONS(1488),
    [sym_smtp_port_number] = ACTIONS(1488),
    [sym_smtp_username] = ACTIONS(1488),
    [sym_smtp_password] = ACTIONS(1488),
    [sym_smtp_require_tls] = ACTIONS(1488),
    [sym_smtp_from] = ACTIONS(1488),
    [sym_smtp_tls_true] = ACTIONS(1488),
    [sym_smtp_tls_false] = ACTIONS(1488),
    [sym_listview_on_row_select_event] = ACTIONS(1488),
    [sym_data_structure_unknown] = ACTIONS(1488),
    [sym_data_structure_datafeed] = ACTIONS(1488),
    [sym_data_structure_datarecordset] = ACTIONS(1488),
    [sym_data_structure_datarecord] = ACTIONS(1488),
    [sym_data_structure_datafield] = ACTIONS(1488),
    [sym_data_max_content_field_count] = ACTIONS(1488),
    [sym_data_max_metadata_field_count] = ACTIONS(1488),
    [sym_data_max_id_length] = ACTIONS(1488),
    [sym_data_max_name_length] = ACTIONS(1488),
    [sym_data_max_description_length] = ACTIONS(1488),
    [sym_data_max_type_length] = ACTIONS(1488),
    [sym_data_max_format_length] = ACTIONS(1488),
    [sym_data_max_source_length] = ACTIONS(1488),
    [sym_data_max_value_length] = ACTIONS(1488),
    [sym_data_max_label_length] = ACTIONS(1488),
    [sym_data_max_url_length] = ACTIONS(1488),
    [sym_max_last_login_info_length] = ACTIONS(1488),
    [sym_data_type_unknown] = ACTIONS(1488),
    [sym_data_type_string] = ACTIONS(1488),
    [sym_data_type_datetime] = ACTIONS(1488),
    [sym_data_type_date] = ACTIONS(1488),
    [sym_data_type_time] = ACTIONS(1488),
    [sym_data_type_image] = ACTIONS(1488),
    [sym_data_format_url] = ACTIONS(1488),
    [sym_data_format_phone] = ACTIONS(1488),
    [sym_data_format_email] = ACTIONS(1488),
    [sym_data_format_iso8601] = ACTIONS(1488),
    [sym_auditlog_priv] = ACTIONS(1488),
    [sym_configuration_priv] = ACTIONS(1488),
    [sym_deviceconfig_priv] = ACTIONS(1488),
    [sym_ftp_priv] = ACTIONS(1488),
    [sym_http_priv] = ACTIONS(1488),
    [sym_networkconfig_priv] = ACTIONS(1488),
    [sym_programport_priv] = ACTIONS(1488),
    [sym_remoteui_priv] = ACTIONS(1488),
    [sym_securitycontrol_priv] = ACTIONS(1488),
    [sym_softwaremanagement_priv] = ACTIONS(1488),
    [sym_terminal_priv] = ACTIONS(1488),
    [sym_tpadmin_priv] = ACTIONS(1488),
    [sym_user1_priv] = ACTIONS(1488),
    [sym_user2_priv] = ACTIONS(1488),
    [sym_user3_priv] = ACTIONS(1488),
    [sym_user4_priv] = ACTIONS(1488),
    [sym_usermanagement_priv] = ACTIONS(1488),
    [sym_valid_account] = ACTIONS(1488),
    [sym_err_name_invalid] = ACTIONS(1488),
    [sym_err_password_invalid] = ACTIONS(1488),
    [sym_err_invalid_account] = ACTIONS(1488),
    [sym_err_no_authorization] = ACTIONS(1488),
    [sym_err_invalid_parameter] = ACTIONS(1488),
    [sym_err_user_locked_out] = ACTIONS(1488),
    [sym_err_user_account_expired] = ACTIONS(1488),
    [sym_err_general_error] = ACTIONS(1488),
    [sym_err_authentication_error] = ACTIONS(1488),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1488),
    [sym_tls_validate_certificate] = ACTIONS(1488),
    [sym____reserved___] = ACTIONS(1488),
    [sym_date] = ACTIONS(1488),
    [sym_day] = ACTIONS(1488),
    [sym_time] = ACTIONS(1488),
    [sym_ldate] = ACTIONS(1488),
    [sym_push_device] = ACTIONS(1488),
    [sym_push_channel] = ACTIONS(1488),
    [sym_push_devchan] = ACTIONS(1488),
    [sym_release_device] = ACTIONS(1488),
    [sym_release_channel] = ACTIONS(1488),
    [sym_release_devchan] = ACTIONS(1488),
    [sym_master_slot] = ACTIONS(1488),
    [sym_get_pulse_time] = ACTIONS(1488),
    [sym_get_timer] = ACTIONS(1488),
    [sym_master_sn] = ACTIONS(1488),
    [sym_system_number] = ACTIONS(1488),
    [sym_button] = ACTIONS(1488),
    [sym_dv_channel] = ACTIONS(1488),
    [sym_data] = ACTIONS(1488),
    [sym_level] = ACTIONS(1488),
    [sym_timeline] = ACTIONS(1488),
    [sym_channel] = ACTIONS(1488),
    [sym_custom] = ACTIONS(1488),
    [sym_atoi] = ACTIONS(1488),
    [sym_atol] = ACTIONS(1488),
    [sym_atof] = ACTIONS(1488),
    [sym_itoa] = ACTIONS(1488),
    [sym_itohex] = ACTIONS(1488),
    [sym_hextoi] = ACTIONS(1488),
    [sym_ftoa] = ACTIONS(1488),
    [sym_find_string] = ACTIONS(1488),
    [sym_left_string] = ACTIONS(1488),
    [sym_right_string] = ACTIONS(1488),
    [sym_mid_string] = ACTIONS(1488),
    [sym_remove_string] = ACTIONS(1488),
    [sym_get_buffer_string] = ACTIONS(1488),
    [sym_get_buffer_char] = ACTIONS(1488),
    [sym_get_multi_buffer_string] = ACTIONS(1488),
    [sym_length_string] = ACTIONS(1488),
    [sym_max_length_string] = ACTIONS(1488),
    [sym_set_length_string] = ACTIONS(1488),
    [sym_compare_string] = ACTIONS(1488),
    [sym_upper_string] = ACTIONS(1488),
    [sym_lower_string] = ACTIONS(1488),
    [sym_get_last] = ACTIONS(1488),
    [sym_date_to_day] = ACTIONS(1488),
    [sym_date_to_month] = ACTIONS(1488),
    [sym_date_to_year] = ACTIONS(1488),
    [sym_time_to_hour] = ACTIONS(1488),
    [sym_time_to_minute] = ACTIONS(1488),
    [sym_time_to_second] = ACTIONS(1488),
    [sym_day_of_week] = ACTIONS(1488),
    [sym_set_pulse_time] = ACTIONS(1488),
    [sym_set_timer] = ACTIONS(1488),
    [sym_file_open] = ACTIONS(1488),
    [sym_file_close] = ACTIONS(1488),
    [sym_file_read] = ACTIONS(1488),
    [sym_file_read_line] = ACTIONS(1488),
    [sym_file_write] = ACTIONS(1488),
    [sym_file_write_line] = ACTIONS(1488),
    [sym_file_copy] = ACTIONS(1488),
    [sym_file_delete] = ACTIONS(1488),
    [sym_file_rename] = ACTIONS(1488),
    [sym_file_dir] = ACTIONS(1488),
    [sym_file_getdir] = ACTIONS(1488),
    [sym_file_setdir] = ACTIONS(1488),
    [sym_file_create_dir] = ACTIONS(1488),
    [sym_file_remove_dir] = ACTIONS(1488),
    [sym_file_seek] = ACTIONS(1488),
    [sym_length_array] = ACTIONS(1488),
    [sym_max_length_array] = ACTIONS(1488),
    [sym_set_length_array] = ACTIONS(1488),
    [sym_ip_client_open] = ACTIONS(1488),
    [sym_ip_client_close] = ACTIONS(1488),
    [sym_ip_server_open] = ACTIONS(1488),
    [sym_ip_server_close] = ACTIONS(1488),
    [sym_ip_bound_client_open] = ACTIONS(1488),
    [sym_ip_mc_server_open] = ACTIONS(1488),
    [sym_ip_set_option] = ACTIONS(1488),
    [sym_ssh_client_open] = ACTIONS(1488),
    [sym_ssh_client_close] = ACTIONS(1488),
    [sym_tls_client_open] = ACTIONS(1488),
    [sym_tls_client_close] = ACTIONS(1488),
    [sym_do_push] = ACTIONS(1488),
    [sym_do_push_timed] = ACTIONS(1488),
    [sym_do_release] = ACTIONS(1488),
    [sym_device_id] = ACTIONS(1488),
    [sym_device_info] = ACTIONS(1488),
    [sym_device_id_string] = ACTIONS(1488),
    [sym_device_standby] = ACTIONS(1488),
    [sym_device_wake] = ACTIONS(1488),
    [sym_timeline_create] = ACTIONS(1488),
    [sym_timeline_kill] = ACTIONS(1488),
    [sym_timeline_reload] = ACTIONS(1488),
    [sym_timeline_pause] = ACTIONS(1488),
    [sym_timeline_restart] = ACTIONS(1488),
    [sym_timeline_set] = ACTIONS(1488),
    [sym_timeline_get] = ACTIONS(1488),
    [sym_timeline_active] = ACTIONS(1488),
    [sym_timeline_dynamic_id] = ACTIONS(1488),
    [sym_variable_to_string] = ACTIONS(1488),
    [sym_string_to_variable] = ACTIONS(1488),
    [sym_length_variable_to_string] = ACTIONS(1488),
    [sym_type_cast] = ACTIONS(1488),
    [sym_variable_to_xml] = ACTIONS(1488),
    [sym_xml_to_variable] = ACTIONS(1488),
    [sym_length_variable_to_xml] = ACTIONS(1488),
    [sym_redirect_string] = ACTIONS(1488),
    [sym_random_number] = ACTIONS(1488),
    [sym_get_unique_id] = ACTIONS(1488),
    [sym_get_system_number] = ACTIONS(1488),
    [sym_set_system_number] = ACTIONS(1488),
    [sym_get_serial_number] = ACTIONS(1488),
    [sym_get_master_build] = ACTIONS(1488),
    [sym_set_validation_code] = ACTIONS(1488),
    [sym_reboot] = ACTIONS(1488),
    [sym_load_duet_module] = ACTIONS(1488),
    [sym_unload_duet_module] = ACTIONS(1488),
    [sym_duet_mem_size_set] = ACTIONS(1488),
    [sym_duet_mem_size_get] = ACTIONS(1488),
    [sym_dynamic_application_device] = ACTIONS(1488),
    [sym_dynamic_polled_port] = ACTIONS(1488),
    [sym_static_port_binding] = ACTIONS(1488),
    [sym_static_ip_binding] = ACTIONS(1488),
    [sym_get_dns_list] = ACTIONS(1488),
    [sym_set_dns_list] = ACTIONS(1488),
    [sym_get_ip_address] = ACTIONS(1488),
    [sym_set_ip_address] = ACTIONS(1488),
    [sym_get_url_list] = ACTIONS(1488),
    [sym_add_url_entry] = ACTIONS(1488),
    [sym_delete_url_entry] = ACTIONS(1488),
    [sym_abs_value] = ACTIONS(1488),
    [sym_max_value] = ACTIONS(1488),
    [sym_min_value] = ACTIONS(1488),
    [sym_exp_value] = ACTIONS(1488),
    [sym_log_value] = ACTIONS(1488),
    [sym_log10_value] = ACTIONS(1488),
    [sym_power_value] = ACTIONS(1488),
    [sym_sqrt_value] = ACTIONS(1488),
    [sym_raw_be] = ACTIONS(1488),
    [sym_raw_le] = ACTIONS(1488),
    [sym_internal_queue_size_set] = ACTIONS(1488),
    [sym_internal_queue_size_get] = ACTIONS(1488),
    [sym_internal_threshold_set] = ACTIONS(1488),
    [sym_internal_threshold_get] = ACTIONS(1488),
    [sym_rebuilt_event] = ACTIONS(1488),
    [sym_do_custom_event] = ACTIONS(1488),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1488),
    [sym_clkmgr_set_clk_source] = ACTIONS(1488),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1488),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1488),
    [sym_clkmgr_get_timezone] = ACTIONS(1488),
    [sym_clkmgr_set_timezone] = ACTIONS(1488),
    [sym_clkmgr_get_resync_period] = ACTIONS(1488),
    [sym_clkmgr_set_resync_period] = ACTIONS(1488),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1488),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1488),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1488),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1488),
    [sym_clkmgr_get_timeservers] = ACTIONS(1488),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1488),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1488),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1488),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1488),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1488),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1488),
    [sym_set_log_level] = ACTIONS(1488),
    [sym_get_log_level] = ACTIONS(1488),
    [sym_amx_log] = ACTIONS(1488),
    [sym_smtp_server_config_set] = ACTIONS(1488),
    [sym_smtp_server_config_get] = ACTIONS(1488),
    [sym_smtp_send] = ACTIONS(1488),
    [sym_validate_netlinx_account] = ACTIONS(1488),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1488),
    [sym_authenticate_certificate] = ACTIONS(1488),
    [sym_audit_netlinx_session_event] = ACTIONS(1488),
    [sym_audit_netlinx_generic_event] = ACTIONS(1488),
    [sym_get_available_flash_disk_space] = ACTIONS(1488),
    [sym_get_max_flash_disk_space] = ACTIONS(1488),
    [sym_data_create_feed] = ACTIONS(1488),
    [sym_data_delete_feed] = ACTIONS(1488),
    [sym_data_publish_feed] = ACTIONS(1488),
    [sym_data_get_published_feed] = ACTIONS(1488),
    [sym_data_add_record] = ACTIONS(1488),
    [sym_data_get_event_record] = ACTIONS(1488),
    [sym__wc_data_create_feed] = ACTIONS(1488),
    [sym__wc_data_add_record] = ACTIONS(1488),
    [sym__wc_data_get_event_record] = ACTIONS(1488),
    [sym_format] = ACTIONS(1488),
    [sym_astro_clock] = ACTIONS(1488),
    [sym_set_outdoor_temperature] = ACTIONS(1488),
    [sym_set_virtual_level_count] = ACTIONS(1488),
    [sym_set_virtual_channel_count] = ACTIONS(1488),
    [sym_set_virtual_port_count] = ACTIONS(1488),
    [sym__wc_encode_int] = ACTIONS(1488),
    [sym__wc_decode_int] = ACTIONS(1488),
    [sym__wc_int] = ACTIONS(1488),
    [sym__wc_to_ch_int] = ACTIONS(1488),
    [sym__ch_to_wc_int] = ACTIONS(1488),
    [sym__wc_find_string_int] = ACTIONS(1488),
    [sym__wc_left_string_int] = ACTIONS(1488),
    [sym__wc_lower_string_int] = ACTIONS(1488),
    [sym__wc_mid_string_int] = ACTIONS(1488),
    [sym__wc_remove_string_int] = ACTIONS(1488),
    [sym__wc_right_string_int] = ACTIONS(1488),
    [sym__wc_upper_string_int] = ACTIONS(1488),
    [sym__wc_compare_string_int] = ACTIONS(1488),
    [sym__wc_get_buffer_char_int] = ACTIONS(1488),
    [sym__wc_get_buffer_string_int] = ACTIONS(1488),
    [sym__wc_concat_string_int] = ACTIONS(1488),
    [sym__wc_file_open_int] = ACTIONS(1488),
    [sym__wc_file_close_int] = ACTIONS(1488),
    [sym__wc_file_read_int] = ACTIONS(1488),
    [sym__wc_file_read_line_int] = ACTIONS(1488),
    [sym__wc_file_write_int] = ACTIONS(1488),
    [sym__wc_file_write_line_int] = ACTIONS(1488),
    [anon_sym_EQ] = ACTIONS(1488),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1490),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1490),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1490),
    [aux_sym_preproc_else_token1] = ACTIONS(1490),
    [anon_sym_COMMA] = ACTIONS(1490),
    [anon_sym_LBRACK] = ACTIONS(1490),
    [anon_sym_LBRACE] = ACTIONS(1490),
    [anon_sym_RBRACE] = ACTIONS(1490),
    [anon_sym_COLON] = ACTIONS(1490),
    [anon_sym_LPAREN] = ACTIONS(1488),
    [anon_sym_BANG] = ACTIONS(1488),
    [anon_sym_TILDE] = ACTIONS(1490),
    [anon_sym_DASH] = ACTIONS(1488),
    [anon_sym_PLUS] = ACTIONS(1488),
    [anon_sym_STAR] = ACTIONS(1490),
    [anon_sym_SLASH] = ACTIONS(1488),
    [anon_sym_PERCENT] = ACTIONS(1490),
    [anon_sym_PIPE_PIPE] = ACTIONS(1490),
    [anon_sym_AMP_AMP] = ACTIONS(1490),
    [anon_sym_PIPE] = ACTIONS(1488),
    [anon_sym_CARET] = ACTIONS(1490),
    [anon_sym_AMP] = ACTIONS(1488),
    [anon_sym_EQ_EQ] = ACTIONS(1490),
    [anon_sym_BANG_EQ] = ACTIONS(1490),
    [anon_sym_LT_GT] = ACTIONS(1490),
    [anon_sym_GT] = ACTIONS(1488),
    [anon_sym_GT_EQ] = ACTIONS(1490),
    [anon_sym_LT_EQ] = ACTIONS(1490),
    [anon_sym_LT] = ACTIONS(1488),
    [anon_sym_LT_LT] = ACTIONS(1490),
    [anon_sym_GT_GT] = ACTIONS(1490),
    [anon_sym_DASH_DASH] = ACTIONS(1490),
    [anon_sym_PLUS_PLUS] = ACTIONS(1490),
    [anon_sym_DOT] = ACTIONS(1488),
    [anon_sym_DQUOTE] = ACTIONS(1490),
    [anon_sym_SQUOTE] = ACTIONS(1490),
    [sym_number_literal] = ACTIONS(1490),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(582)] = {
    [sym_identifier] = ACTIONS(1507),
    [sym_and] = ACTIONS(1507),
    [sym_or] = ACTIONS(1507),
    [sym_not] = ACTIONS(1507),
    [sym_xor] = ACTIONS(1507),
    [sym_band] = ACTIONS(1507),
    [sym_bor] = ACTIONS(1507),
    [sym_bxor] = ACTIONS(1507),
    [sym_bnot] = ACTIONS(1507),
    [sym_lshift] = ACTIONS(1507),
    [sym_rshift] = ACTIONS(1507),
    [sym___file__] = ACTIONS(1507),
    [sym___date__] = ACTIONS(1507),
    [sym___ldate__] = ACTIONS(1507),
    [sym___time__] = ACTIONS(1507),
    [sym___line__] = ACTIONS(1507),
    [sym___name__] = ACTIONS(1507),
    [sym___netlinx__] = ACTIONS(1507),
    [sym_netlinx_axi_version] = ACTIONS(1507),
    [sym_url_flg_tcp] = ACTIONS(1507),
    [sym_url_flg_accinfopresent] = ACTIONS(1507),
    [sym_url_flg_temp] = ACTIONS(1507),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1507),
    [sym_url_flg_stat_mask] = ACTIONS(1507),
    [sym_url_flg_stat_lookup] = ACTIONS(1507),
    [sym_url_flg_stat_connecting] = ACTIONS(1507),
    [sym_url_flg_stat_waiting] = ACTIONS(1507),
    [sym_url_flg_stat_connected] = ACTIONS(1507),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1507),
    [sym_first_virtual_device] = ACTIONS(1507),
    [sym_first_local_port] = ACTIONS(1507),
    [sym_dynamic_virtual_device] = ACTIONS(1507),
    [sym_timeline_once] = ACTIONS(1507),
    [sym_timeline_repeat] = ACTIONS(1507),
    [sym_timeline_absolute] = ACTIONS(1507),
    [sym_timeline_relative] = ACTIONS(1507),
    [sym_do_push_timed_infinite] = ACTIONS(1507),
    [sym_source_type_no_address] = ACTIONS(1507),
    [sym_source_type_neuron_id] = ACTIONS(1507),
    [sym_source_type_ip_address] = ACTIONS(1507),
    [sym_source_type_axlink] = ACTIONS(1507),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1507),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1507),
    [sym_source_type_ip_socket_address] = ACTIONS(1507),
    [sym_source_type_rs232] = ACTIONS(1507),
    [sym_source_type_internal] = ACTIONS(1507),
    [sym_source_type_mac_address] = ACTIONS(1507),
    [sym_source_type_ipv4_port] = ACTIONS(1507),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1507),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1507),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1507),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1507),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1507),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1507),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1507),
    [sym_true] = ACTIONS(1507),
    [sym_false] = ACTIONS(1507),
    [sym_file_read_only] = ACTIONS(1507),
    [sym_file_rw_new] = ACTIONS(1507),
    [sym_file_rw_append] = ACTIONS(1507),
    [sym_ip_tcp] = ACTIONS(1507),
    [sym_ip_udp] = ACTIONS(1507),
    [sym_ip_udp_2way] = ACTIONS(1507),
    [sym_xml_encode_types] = ACTIONS(1507),
    [sym_xml_encode_char_as_list] = ACTIONS(1507),
    [sym_xml_encode_le] = ACTIONS(1507),
    [sym_xml_decode_types] = ACTIONS(1507),
    [sym_xml_decode_no_preserve] = ACTIONS(1507),
    [sym_duet_dev_type_display_device] = ACTIONS(1507),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1507),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1507),
    [sym_duet_dev_type_io_device] = ACTIONS(1507),
    [sym_duet_dev_type_relay_device] = ACTIONS(1507),
    [sym_duet_dev_type_ups] = ACTIONS(1507),
    [sym_duet_dev_type_amplifier] = ACTIONS(1507),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1507),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1507),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1507),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1507),
    [sym_duet_dev_type_camera] = ACTIONS(1507),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1507),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1507),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1507),
    [sym_duet_dev_type_dss] = ACTIONS(1507),
    [sym_duet_dev_type_dvr] = ACTIONS(1507),
    [sym_duet_dev_type_disc_device] = ACTIONS(1507),
    [sym_duet_dev_type_document_camera] = ACTIONS(1507),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1507),
    [sym_duet_dev_type_hvac] = ACTIONS(1507),
    [sym_duet_dev_type_keypad] = ACTIONS(1507),
    [sym_duet_dev_type_light] = ACTIONS(1507),
    [sym_duet_dev_type_monitor] = ACTIONS(1507),
    [sym_duet_dev_type_motor] = ACTIONS(1507),
    [sym_duet_dev_type_multi_window] = ACTIONS(1507),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1507),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1507),
    [sym_duet_dev_type_receiver] = ACTIONS(1507),
    [sym_duet_dev_type_security_system] = ACTIONS(1507),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1507),
    [sym_duet_dev_type_settop_box] = ACTIONS(1507),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1507),
    [sym_duet_dev_type_switcher] = ACTIONS(1507),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1507),
    [sym_duet_dev_type_tv] = ACTIONS(1507),
    [sym_duet_dev_type_utility] = ACTIONS(1507),
    [sym_duet_dev_type_vcr] = ACTIONS(1507),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1507),
    [sym_duet_dev_type_video_processor] = ACTIONS(1507),
    [sym_duet_dev_type_video_projector] = ACTIONS(1507),
    [sym_duet_dev_type_video_wall] = ACTIONS(1507),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1507),
    [sym_duet_dev_type_weather] = ACTIONS(1507),
    [sym_duet_dev_not_polled] = ACTIONS(1507),
    [sym_duet_dev_polled] = ACTIONS(1507),
    [sym_clkmgr_mode_network] = ACTIONS(1507),
    [sym_clkmgr_mode_standalone] = ACTIONS(1507),
    [sym_normal_standby] = ACTIONS(1507),
    [sym_normal_wake] = ACTIONS(1507),
    [sym_ip_multicast_ttl_option] = ACTIONS(1507),
    [sym_ip_ttl_subnet] = ACTIONS(1507),
    [sym_ip_ttl_site] = ACTIONS(1507),
    [sym_ip_ttl_region] = ACTIONS(1507),
    [sym_ip_ttl_continent] = ACTIONS(1507),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1507),
    [sym_ip_nodelay_on] = ACTIONS(1507),
    [sym_ip_nodelay_off] = ACTIONS(1507),
    [sym_amx_error] = ACTIONS(1507),
    [sym_amx_warning] = ACTIONS(1507),
    [sym_amx_info] = ACTIONS(1507),
    [sym_amx_debug] = ACTIONS(1507),
    [sym_netlinx_login_success] = ACTIONS(1507),
    [sym_netlinx_login_fail] = ACTIONS(1507),
    [sym_netlinx_logout] = ACTIONS(1507),
    [sym_null_str] = ACTIONS(1507),
    [sym_smtp_address] = ACTIONS(1507),
    [sym_smtp_port_number] = ACTIONS(1507),
    [sym_smtp_username] = ACTIONS(1507),
    [sym_smtp_password] = ACTIONS(1507),
    [sym_smtp_require_tls] = ACTIONS(1507),
    [sym_smtp_from] = ACTIONS(1507),
    [sym_smtp_tls_true] = ACTIONS(1507),
    [sym_smtp_tls_false] = ACTIONS(1507),
    [sym_listview_on_row_select_event] = ACTIONS(1507),
    [sym_data_structure_unknown] = ACTIONS(1507),
    [sym_data_structure_datafeed] = ACTIONS(1507),
    [sym_data_structure_datarecordset] = ACTIONS(1507),
    [sym_data_structure_datarecord] = ACTIONS(1507),
    [sym_data_structure_datafield] = ACTIONS(1507),
    [sym_data_max_content_field_count] = ACTIONS(1507),
    [sym_data_max_metadata_field_count] = ACTIONS(1507),
    [sym_data_max_id_length] = ACTIONS(1507),
    [sym_data_max_name_length] = ACTIONS(1507),
    [sym_data_max_description_length] = ACTIONS(1507),
    [sym_data_max_type_length] = ACTIONS(1507),
    [sym_data_max_format_length] = ACTIONS(1507),
    [sym_data_max_source_length] = ACTIONS(1507),
    [sym_data_max_value_length] = ACTIONS(1507),
    [sym_data_max_label_length] = ACTIONS(1507),
    [sym_data_max_url_length] = ACTIONS(1507),
    [sym_max_last_login_info_length] = ACTIONS(1507),
    [sym_data_type_unknown] = ACTIONS(1507),
    [sym_data_type_string] = ACTIONS(1507),
    [sym_data_type_datetime] = ACTIONS(1507),
    [sym_data_type_date] = ACTIONS(1507),
    [sym_data_type_time] = ACTIONS(1507),
    [sym_data_type_image] = ACTIONS(1507),
    [sym_data_format_url] = ACTIONS(1507),
    [sym_data_format_phone] = ACTIONS(1507),
    [sym_data_format_email] = ACTIONS(1507),
    [sym_data_format_iso8601] = ACTIONS(1507),
    [sym_auditlog_priv] = ACTIONS(1507),
    [sym_configuration_priv] = ACTIONS(1507),
    [sym_deviceconfig_priv] = ACTIONS(1507),
    [sym_ftp_priv] = ACTIONS(1507),
    [sym_http_priv] = ACTIONS(1507),
    [sym_networkconfig_priv] = ACTIONS(1507),
    [sym_programport_priv] = ACTIONS(1507),
    [sym_remoteui_priv] = ACTIONS(1507),
    [sym_securitycontrol_priv] = ACTIONS(1507),
    [sym_softwaremanagement_priv] = ACTIONS(1507),
    [sym_terminal_priv] = ACTIONS(1507),
    [sym_tpadmin_priv] = ACTIONS(1507),
    [sym_user1_priv] = ACTIONS(1507),
    [sym_user2_priv] = ACTIONS(1507),
    [sym_user3_priv] = ACTIONS(1507),
    [sym_user4_priv] = ACTIONS(1507),
    [sym_usermanagement_priv] = ACTIONS(1507),
    [sym_valid_account] = ACTIONS(1507),
    [sym_err_name_invalid] = ACTIONS(1507),
    [sym_err_password_invalid] = ACTIONS(1507),
    [sym_err_invalid_account] = ACTIONS(1507),
    [sym_err_no_authorization] = ACTIONS(1507),
    [sym_err_invalid_parameter] = ACTIONS(1507),
    [sym_err_user_locked_out] = ACTIONS(1507),
    [sym_err_user_account_expired] = ACTIONS(1507),
    [sym_err_general_error] = ACTIONS(1507),
    [sym_err_authentication_error] = ACTIONS(1507),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1507),
    [sym_tls_validate_certificate] = ACTIONS(1507),
    [sym____reserved___] = ACTIONS(1507),
    [sym_date] = ACTIONS(1507),
    [sym_day] = ACTIONS(1507),
    [sym_time] = ACTIONS(1507),
    [sym_ldate] = ACTIONS(1507),
    [sym_push_device] = ACTIONS(1507),
    [sym_push_channel] = ACTIONS(1507),
    [sym_push_devchan] = ACTIONS(1507),
    [sym_release_device] = ACTIONS(1507),
    [sym_release_channel] = ACTIONS(1507),
    [sym_release_devchan] = ACTIONS(1507),
    [sym_master_slot] = ACTIONS(1507),
    [sym_get_pulse_time] = ACTIONS(1507),
    [sym_get_timer] = ACTIONS(1507),
    [sym_master_sn] = ACTIONS(1507),
    [sym_system_number] = ACTIONS(1507),
    [sym_button] = ACTIONS(1507),
    [sym_dv_channel] = ACTIONS(1507),
    [sym_data] = ACTIONS(1507),
    [sym_level] = ACTIONS(1507),
    [sym_timeline] = ACTIONS(1507),
    [sym_channel] = ACTIONS(1507),
    [sym_custom] = ACTIONS(1507),
    [sym_atoi] = ACTIONS(1507),
    [sym_atol] = ACTIONS(1507),
    [sym_atof] = ACTIONS(1507),
    [sym_itoa] = ACTIONS(1507),
    [sym_itohex] = ACTIONS(1507),
    [sym_hextoi] = ACTIONS(1507),
    [sym_ftoa] = ACTIONS(1507),
    [sym_find_string] = ACTIONS(1507),
    [sym_left_string] = ACTIONS(1507),
    [sym_right_string] = ACTIONS(1507),
    [sym_mid_string] = ACTIONS(1507),
    [sym_remove_string] = ACTIONS(1507),
    [sym_get_buffer_string] = ACTIONS(1507),
    [sym_get_buffer_char] = ACTIONS(1507),
    [sym_get_multi_buffer_string] = ACTIONS(1507),
    [sym_length_string] = ACTIONS(1507),
    [sym_max_length_string] = ACTIONS(1507),
    [sym_set_length_string] = ACTIONS(1507),
    [sym_compare_string] = ACTIONS(1507),
    [sym_upper_string] = ACTIONS(1507),
    [sym_lower_string] = ACTIONS(1507),
    [sym_get_last] = ACTIONS(1507),
    [sym_date_to_day] = ACTIONS(1507),
    [sym_date_to_month] = ACTIONS(1507),
    [sym_date_to_year] = ACTIONS(1507),
    [sym_time_to_hour] = ACTIONS(1507),
    [sym_time_to_minute] = ACTIONS(1507),
    [sym_time_to_second] = ACTIONS(1507),
    [sym_day_of_week] = ACTIONS(1507),
    [sym_set_pulse_time] = ACTIONS(1507),
    [sym_set_timer] = ACTIONS(1507),
    [sym_file_open] = ACTIONS(1507),
    [sym_file_close] = ACTIONS(1507),
    [sym_file_read] = ACTIONS(1507),
    [sym_file_read_line] = ACTIONS(1507),
    [sym_file_write] = ACTIONS(1507),
    [sym_file_write_line] = ACTIONS(1507),
    [sym_file_copy] = ACTIONS(1507),
    [sym_file_delete] = ACTIONS(1507),
    [sym_file_rename] = ACTIONS(1507),
    [sym_file_dir] = ACTIONS(1507),
    [sym_file_getdir] = ACTIONS(1507),
    [sym_file_setdir] = ACTIONS(1507),
    [sym_file_create_dir] = ACTIONS(1507),
    [sym_file_remove_dir] = ACTIONS(1507),
    [sym_file_seek] = ACTIONS(1507),
    [sym_length_array] = ACTIONS(1507),
    [sym_max_length_array] = ACTIONS(1507),
    [sym_set_length_array] = ACTIONS(1507),
    [sym_ip_client_open] = ACTIONS(1507),
    [sym_ip_client_close] = ACTIONS(1507),
    [sym_ip_server_open] = ACTIONS(1507),
    [sym_ip_server_close] = ACTIONS(1507),
    [sym_ip_bound_client_open] = ACTIONS(1507),
    [sym_ip_mc_server_open] = ACTIONS(1507),
    [sym_ip_set_option] = ACTIONS(1507),
    [sym_ssh_client_open] = ACTIONS(1507),
    [sym_ssh_client_close] = ACTIONS(1507),
    [sym_tls_client_open] = ACTIONS(1507),
    [sym_tls_client_close] = ACTIONS(1507),
    [sym_do_push] = ACTIONS(1507),
    [sym_do_push_timed] = ACTIONS(1507),
    [sym_do_release] = ACTIONS(1507),
    [sym_device_id] = ACTIONS(1507),
    [sym_device_info] = ACTIONS(1507),
    [sym_device_id_string] = ACTIONS(1507),
    [sym_device_standby] = ACTIONS(1507),
    [sym_device_wake] = ACTIONS(1507),
    [sym_timeline_create] = ACTIONS(1507),
    [sym_timeline_kill] = ACTIONS(1507),
    [sym_timeline_reload] = ACTIONS(1507),
    [sym_timeline_pause] = ACTIONS(1507),
    [sym_timeline_restart] = ACTIONS(1507),
    [sym_timeline_set] = ACTIONS(1507),
    [sym_timeline_get] = ACTIONS(1507),
    [sym_timeline_active] = ACTIONS(1507),
    [sym_timeline_dynamic_id] = ACTIONS(1507),
    [sym_variable_to_string] = ACTIONS(1507),
    [sym_string_to_variable] = ACTIONS(1507),
    [sym_length_variable_to_string] = ACTIONS(1507),
    [sym_type_cast] = ACTIONS(1507),
    [sym_variable_to_xml] = ACTIONS(1507),
    [sym_xml_to_variable] = ACTIONS(1507),
    [sym_length_variable_to_xml] = ACTIONS(1507),
    [sym_redirect_string] = ACTIONS(1507),
    [sym_random_number] = ACTIONS(1507),
    [sym_get_unique_id] = ACTIONS(1507),
    [sym_get_system_number] = ACTIONS(1507),
    [sym_set_system_number] = ACTIONS(1507),
    [sym_get_serial_number] = ACTIONS(1507),
    [sym_get_master_build] = ACTIONS(1507),
    [sym_set_validation_code] = ACTIONS(1507),
    [sym_reboot] = ACTIONS(1507),
    [sym_load_duet_module] = ACTIONS(1507),
    [sym_unload_duet_module] = ACTIONS(1507),
    [sym_duet_mem_size_set] = ACTIONS(1507),
    [sym_duet_mem_size_get] = ACTIONS(1507),
    [sym_dynamic_application_device] = ACTIONS(1507),
    [sym_dynamic_polled_port] = ACTIONS(1507),
    [sym_static_port_binding] = ACTIONS(1507),
    [sym_static_ip_binding] = ACTIONS(1507),
    [sym_get_dns_list] = ACTIONS(1507),
    [sym_set_dns_list] = ACTIONS(1507),
    [sym_get_ip_address] = ACTIONS(1507),
    [sym_set_ip_address] = ACTIONS(1507),
    [sym_get_url_list] = ACTIONS(1507),
    [sym_add_url_entry] = ACTIONS(1507),
    [sym_delete_url_entry] = ACTIONS(1507),
    [sym_abs_value] = ACTIONS(1507),
    [sym_max_value] = ACTIONS(1507),
    [sym_min_value] = ACTIONS(1507),
    [sym_exp_value] = ACTIONS(1507),
    [sym_log_value] = ACTIONS(1507),
    [sym_log10_value] = ACTIONS(1507),
    [sym_power_value] = ACTIONS(1507),
    [sym_sqrt_value] = ACTIONS(1507),
    [sym_raw_be] = ACTIONS(1507),
    [sym_raw_le] = ACTIONS(1507),
    [sym_internal_queue_size_set] = ACTIONS(1507),
    [sym_internal_queue_size_get] = ACTIONS(1507),
    [sym_internal_threshold_set] = ACTIONS(1507),
    [sym_internal_threshold_get] = ACTIONS(1507),
    [sym_rebuilt_event] = ACTIONS(1507),
    [sym_do_custom_event] = ACTIONS(1507),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1507),
    [sym_clkmgr_set_clk_source] = ACTIONS(1507),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1507),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1507),
    [sym_clkmgr_get_timezone] = ACTIONS(1507),
    [sym_clkmgr_set_timezone] = ACTIONS(1507),
    [sym_clkmgr_get_resync_period] = ACTIONS(1507),
    [sym_clkmgr_set_resync_period] = ACTIONS(1507),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1507),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1507),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1507),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1507),
    [sym_clkmgr_get_timeservers] = ACTIONS(1507),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1507),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1507),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1507),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1507),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1507),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1507),
    [sym_set_log_level] = ACTIONS(1507),
    [sym_get_log_level] = ACTIONS(1507),
    [sym_amx_log] = ACTIONS(1507),
    [sym_smtp_server_config_set] = ACTIONS(1507),
    [sym_smtp_server_config_get] = ACTIONS(1507),
    [sym_smtp_send] = ACTIONS(1507),
    [sym_validate_netlinx_account] = ACTIONS(1507),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1507),
    [sym_authenticate_certificate] = ACTIONS(1507),
    [sym_audit_netlinx_session_event] = ACTIONS(1507),
    [sym_audit_netlinx_generic_event] = ACTIONS(1507),
    [sym_get_available_flash_disk_space] = ACTIONS(1507),
    [sym_get_max_flash_disk_space] = ACTIONS(1507),
    [sym_data_create_feed] = ACTIONS(1507),
    [sym_data_delete_feed] = ACTIONS(1507),
    [sym_data_publish_feed] = ACTIONS(1507),
    [sym_data_get_published_feed] = ACTIONS(1507),
    [sym_data_add_record] = ACTIONS(1507),
    [sym_data_get_event_record] = ACTIONS(1507),
    [sym__wc_data_create_feed] = ACTIONS(1507),
    [sym__wc_data_add_record] = ACTIONS(1507),
    [sym__wc_data_get_event_record] = ACTIONS(1507),
    [sym_format] = ACTIONS(1507),
    [sym_astro_clock] = ACTIONS(1507),
    [sym_set_outdoor_temperature] = ACTIONS(1507),
    [sym_set_virtual_level_count] = ACTIONS(1507),
    [sym_set_virtual_channel_count] = ACTIONS(1507),
    [sym_set_virtual_port_count] = ACTIONS(1507),
    [sym__wc_encode_int] = ACTIONS(1507),
    [sym__wc_decode_int] = ACTIONS(1507),
    [sym__wc_int] = ACTIONS(1507),
    [sym__wc_to_ch_int] = ACTIONS(1507),
    [sym__ch_to_wc_int] = ACTIONS(1507),
    [sym__wc_find_string_int] = ACTIONS(1507),
    [sym__wc_left_string_int] = ACTIONS(1507),
    [sym__wc_lower_string_int] = ACTIONS(1507),
    [sym__wc_mid_string_int] = ACTIONS(1507),
    [sym__wc_remove_string_int] = ACTIONS(1507),
    [sym__wc_right_string_int] = ACTIONS(1507),
    [sym__wc_upper_string_int] = ACTIONS(1507),
    [sym__wc_compare_string_int] = ACTIONS(1507),
    [sym__wc_get_buffer_char_int] = ACTIONS(1507),
    [sym__wc_get_buffer_string_int] = ACTIONS(1507),
    [sym__wc_concat_string_int] = ACTIONS(1507),
    [sym__wc_file_open_int] = ACTIONS(1507),
    [sym__wc_file_close_int] = ACTIONS(1507),
    [sym__wc_file_read_int] = ACTIONS(1507),
    [sym__wc_file_read_line_int] = ACTIONS(1507),
    [sym__wc_file_write_int] = ACTIONS(1507),
    [sym__wc_file_write_line_int] = ACTIONS(1507),
    [anon_sym_EQ] = ACTIONS(1507),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1509),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1509),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1509),
    [aux_sym_preproc_else_token1] = ACTIONS(1509),
    [anon_sym_COMMA] = ACTIONS(1509),
    [anon_sym_LBRACK] = ACTIONS(1509),
    [anon_sym_LBRACE] = ACTIONS(1509),
    [anon_sym_RBRACE] = ACTIONS(1509),
    [anon_sym_COLON] = ACTIONS(1509),
    [anon_sym_LPAREN] = ACTIONS(1507),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1509),
    [anon_sym_DASH] = ACTIONS(1507),
    [anon_sym_PLUS] = ACTIONS(1507),
    [anon_sym_STAR] = ACTIONS(1509),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_PERCENT] = ACTIONS(1509),
    [anon_sym_PIPE_PIPE] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1509),
    [anon_sym_PIPE] = ACTIONS(1507),
    [anon_sym_CARET] = ACTIONS(1509),
    [anon_sym_AMP] = ACTIONS(1507),
    [anon_sym_EQ_EQ] = ACTIONS(1509),
    [anon_sym_BANG_EQ] = ACTIONS(1509),
    [anon_sym_LT_GT] = ACTIONS(1509),
    [anon_sym_GT] = ACTIONS(1507),
    [anon_sym_GT_EQ] = ACTIONS(1509),
    [anon_sym_LT_EQ] = ACTIONS(1509),
    [anon_sym_LT] = ACTIONS(1507),
    [anon_sym_LT_LT] = ACTIONS(1509),
    [anon_sym_GT_GT] = ACTIONS(1509),
    [anon_sym_DASH_DASH] = ACTIONS(1509),
    [anon_sym_PLUS_PLUS] = ACTIONS(1509),
    [anon_sym_DOT] = ACTIONS(1507),
    [sym_range_operator] = ACTIONS(1509),
    [anon_sym_DQUOTE] = ACTIONS(1509),
    [anon_sym_SQUOTE] = ACTIONS(1509),
    [sym_number_literal] = ACTIONS(1509),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(583)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1492),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(1492),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(1492),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(1492),
    [sym___date__] = ACTIONS(1492),
    [sym___ldate__] = ACTIONS(1492),
    [sym___time__] = ACTIONS(1492),
    [sym___line__] = ACTIONS(1492),
    [sym___name__] = ACTIONS(1492),
    [sym___netlinx__] = ACTIONS(1492),
    [sym_netlinx_axi_version] = ACTIONS(1492),
    [sym_url_flg_tcp] = ACTIONS(1492),
    [sym_url_flg_accinfopresent] = ACTIONS(1492),
    [sym_url_flg_temp] = ACTIONS(1492),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1492),
    [sym_url_flg_stat_mask] = ACTIONS(1492),
    [sym_url_flg_stat_lookup] = ACTIONS(1492),
    [sym_url_flg_stat_connecting] = ACTIONS(1492),
    [sym_url_flg_stat_waiting] = ACTIONS(1492),
    [sym_url_flg_stat_connected] = ACTIONS(1492),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1492),
    [sym_first_virtual_device] = ACTIONS(1492),
    [sym_first_local_port] = ACTIONS(1492),
    [sym_dynamic_virtual_device] = ACTIONS(1492),
    [sym_timeline_once] = ACTIONS(1492),
    [sym_timeline_repeat] = ACTIONS(1492),
    [sym_timeline_absolute] = ACTIONS(1492),
    [sym_timeline_relative] = ACTIONS(1492),
    [sym_do_push_timed_infinite] = ACTIONS(1492),
    [sym_source_type_no_address] = ACTIONS(1492),
    [sym_source_type_neuron_id] = ACTIONS(1492),
    [sym_source_type_ip_address] = ACTIONS(1492),
    [sym_source_type_axlink] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1492),
    [sym_source_type_ip_socket_address] = ACTIONS(1492),
    [sym_source_type_rs232] = ACTIONS(1492),
    [sym_source_type_internal] = ACTIONS(1492),
    [sym_source_type_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1492),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1492),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1492),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1492),
    [sym_true] = ACTIONS(1492),
    [sym_false] = ACTIONS(1492),
    [sym_file_read_only] = ACTIONS(1492),
    [sym_file_rw_new] = ACTIONS(1492),
    [sym_file_rw_append] = ACTIONS(1492),
    [sym_ip_tcp] = ACTIONS(1492),
    [sym_ip_udp] = ACTIONS(1492),
    [sym_ip_udp_2way] = ACTIONS(1492),
    [sym_xml_encode_types] = ACTIONS(1492),
    [sym_xml_encode_char_as_list] = ACTIONS(1492),
    [sym_xml_encode_le] = ACTIONS(1492),
    [sym_xml_decode_types] = ACTIONS(1492),
    [sym_xml_decode_no_preserve] = ACTIONS(1492),
    [sym_duet_dev_type_display_device] = ACTIONS(1492),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_io_device] = ACTIONS(1492),
    [sym_duet_dev_type_relay_device] = ACTIONS(1492),
    [sym_duet_dev_type_ups] = ACTIONS(1492),
    [sym_duet_dev_type_amplifier] = ACTIONS(1492),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1492),
    [sym_duet_dev_type_camera] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1492),
    [sym_duet_dev_type_dss] = ACTIONS(1492),
    [sym_duet_dev_type_dvr] = ACTIONS(1492),
    [sym_duet_dev_type_disc_device] = ACTIONS(1492),
    [sym_duet_dev_type_document_camera] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1492),
    [sym_duet_dev_type_hvac] = ACTIONS(1492),
    [sym_duet_dev_type_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_light] = ACTIONS(1492),
    [sym_duet_dev_type_monitor] = ACTIONS(1492),
    [sym_duet_dev_type_motor] = ACTIONS(1492),
    [sym_duet_dev_type_multi_window] = ACTIONS(1492),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1492),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1492),
    [sym_duet_dev_type_receiver] = ACTIONS(1492),
    [sym_duet_dev_type_security_system] = ACTIONS(1492),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1492),
    [sym_duet_dev_type_settop_box] = ACTIONS(1492),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1492),
    [sym_duet_dev_type_switcher] = ACTIONS(1492),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_tv] = ACTIONS(1492),
    [sym_duet_dev_type_utility] = ACTIONS(1492),
    [sym_duet_dev_type_vcr] = ACTIONS(1492),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_video_processor] = ACTIONS(1492),
    [sym_duet_dev_type_video_projector] = ACTIONS(1492),
    [sym_duet_dev_type_video_wall] = ACTIONS(1492),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1492),
    [sym_duet_dev_type_weather] = ACTIONS(1492),
    [sym_duet_dev_not_polled] = ACTIONS(1492),
    [sym_duet_dev_polled] = ACTIONS(1492),
    [sym_clkmgr_mode_network] = ACTIONS(1492),
    [sym_clkmgr_mode_standalone] = ACTIONS(1492),
    [sym_normal_standby] = ACTIONS(1492),
    [sym_normal_wake] = ACTIONS(1492),
    [sym_ip_multicast_ttl_option] = ACTIONS(1492),
    [sym_ip_ttl_subnet] = ACTIONS(1492),
    [sym_ip_ttl_site] = ACTIONS(1492),
    [sym_ip_ttl_region] = ACTIONS(1492),
    [sym_ip_ttl_continent] = ACTIONS(1492),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1492),
    [sym_ip_nodelay_on] = ACTIONS(1492),
    [sym_ip_nodelay_off] = ACTIONS(1492),
    [sym_amx_error] = ACTIONS(1492),
    [sym_amx_warning] = ACTIONS(1492),
    [sym_amx_info] = ACTIONS(1492),
    [sym_amx_debug] = ACTIONS(1492),
    [sym_netlinx_login_success] = ACTIONS(1492),
    [sym_netlinx_login_fail] = ACTIONS(1492),
    [sym_netlinx_logout] = ACTIONS(1492),
    [sym_null_str] = ACTIONS(1492),
    [sym_smtp_address] = ACTIONS(1492),
    [sym_smtp_port_number] = ACTIONS(1492),
    [sym_smtp_username] = ACTIONS(1492),
    [sym_smtp_password] = ACTIONS(1492),
    [sym_smtp_require_tls] = ACTIONS(1492),
    [sym_smtp_from] = ACTIONS(1492),
    [sym_smtp_tls_true] = ACTIONS(1492),
    [sym_smtp_tls_false] = ACTIONS(1492),
    [sym_listview_on_row_select_event] = ACTIONS(1492),
    [sym_data_structure_unknown] = ACTIONS(1492),
    [sym_data_structure_datafeed] = ACTIONS(1492),
    [sym_data_structure_datarecordset] = ACTIONS(1492),
    [sym_data_structure_datarecord] = ACTIONS(1492),
    [sym_data_structure_datafield] = ACTIONS(1492),
    [sym_data_max_content_field_count] = ACTIONS(1492),
    [sym_data_max_metadata_field_count] = ACTIONS(1492),
    [sym_data_max_id_length] = ACTIONS(1492),
    [sym_data_max_name_length] = ACTIONS(1492),
    [sym_data_max_description_length] = ACTIONS(1492),
    [sym_data_max_type_length] = ACTIONS(1492),
    [sym_data_max_format_length] = ACTIONS(1492),
    [sym_data_max_source_length] = ACTIONS(1492),
    [sym_data_max_value_length] = ACTIONS(1492),
    [sym_data_max_label_length] = ACTIONS(1492),
    [sym_data_max_url_length] = ACTIONS(1492),
    [sym_max_last_login_info_length] = ACTIONS(1492),
    [sym_data_type_unknown] = ACTIONS(1492),
    [sym_data_type_string] = ACTIONS(1492),
    [sym_data_type_datetime] = ACTIONS(1492),
    [sym_data_type_date] = ACTIONS(1492),
    [sym_data_type_time] = ACTIONS(1492),
    [sym_data_type_image] = ACTIONS(1492),
    [sym_data_format_url] = ACTIONS(1492),
    [sym_data_format_phone] = ACTIONS(1492),
    [sym_data_format_email] = ACTIONS(1492),
    [sym_data_format_iso8601] = ACTIONS(1492),
    [sym_auditlog_priv] = ACTIONS(1492),
    [sym_configuration_priv] = ACTIONS(1492),
    [sym_deviceconfig_priv] = ACTIONS(1492),
    [sym_ftp_priv] = ACTIONS(1492),
    [sym_http_priv] = ACTIONS(1492),
    [sym_networkconfig_priv] = ACTIONS(1492),
    [sym_programport_priv] = ACTIONS(1492),
    [sym_remoteui_priv] = ACTIONS(1492),
    [sym_securitycontrol_priv] = ACTIONS(1492),
    [sym_softwaremanagement_priv] = ACTIONS(1492),
    [sym_terminal_priv] = ACTIONS(1492),
    [sym_tpadmin_priv] = ACTIONS(1492),
    [sym_user1_priv] = ACTIONS(1492),
    [sym_user2_priv] = ACTIONS(1492),
    [sym_user3_priv] = ACTIONS(1492),
    [sym_user4_priv] = ACTIONS(1492),
    [sym_usermanagement_priv] = ACTIONS(1492),
    [sym_valid_account] = ACTIONS(1492),
    [sym_err_name_invalid] = ACTIONS(1492),
    [sym_err_password_invalid] = ACTIONS(1492),
    [sym_err_invalid_account] = ACTIONS(1492),
    [sym_err_no_authorization] = ACTIONS(1492),
    [sym_err_invalid_parameter] = ACTIONS(1492),
    [sym_err_user_locked_out] = ACTIONS(1492),
    [sym_err_user_account_expired] = ACTIONS(1492),
    [sym_err_general_error] = ACTIONS(1492),
    [sym_err_authentication_error] = ACTIONS(1492),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1492),
    [sym_tls_validate_certificate] = ACTIONS(1492),
    [sym____reserved___] = ACTIONS(1492),
    [sym_date] = ACTIONS(1492),
    [sym_day] = ACTIONS(1492),
    [sym_time] = ACTIONS(1492),
    [sym_ldate] = ACTIONS(1492),
    [sym_push_device] = ACTIONS(1492),
    [sym_push_channel] = ACTIONS(1492),
    [sym_push_devchan] = ACTIONS(1492),
    [sym_release_device] = ACTIONS(1492),
    [sym_release_channel] = ACTIONS(1492),
    [sym_release_devchan] = ACTIONS(1492),
    [sym_master_slot] = ACTIONS(1492),
    [sym_get_pulse_time] = ACTIONS(1492),
    [sym_get_timer] = ACTIONS(1492),
    [sym_master_sn] = ACTIONS(1492),
    [sym_system_number] = ACTIONS(1492),
    [sym_button] = ACTIONS(1492),
    [sym_dv_channel] = ACTIONS(1492),
    [sym_data] = ACTIONS(1492),
    [sym_level] = ACTIONS(1492),
    [sym_timeline] = ACTIONS(1492),
    [sym_channel] = ACTIONS(1492),
    [sym_custom] = ACTIONS(1492),
    [sym_atoi] = ACTIONS(1492),
    [sym_atol] = ACTIONS(1492),
    [sym_atof] = ACTIONS(1492),
    [sym_itoa] = ACTIONS(1492),
    [sym_itohex] = ACTIONS(1492),
    [sym_hextoi] = ACTIONS(1492),
    [sym_ftoa] = ACTIONS(1492),
    [sym_find_string] = ACTIONS(1492),
    [sym_left_string] = ACTIONS(1492),
    [sym_right_string] = ACTIONS(1492),
    [sym_mid_string] = ACTIONS(1492),
    [sym_remove_string] = ACTIONS(1492),
    [sym_get_buffer_string] = ACTIONS(1492),
    [sym_get_buffer_char] = ACTIONS(1492),
    [sym_get_multi_buffer_string] = ACTIONS(1492),
    [sym_length_string] = ACTIONS(1492),
    [sym_max_length_string] = ACTIONS(1492),
    [sym_set_length_string] = ACTIONS(1492),
    [sym_compare_string] = ACTIONS(1492),
    [sym_upper_string] = ACTIONS(1492),
    [sym_lower_string] = ACTIONS(1492),
    [sym_get_last] = ACTIONS(1492),
    [sym_date_to_day] = ACTIONS(1492),
    [sym_date_to_month] = ACTIONS(1492),
    [sym_date_to_year] = ACTIONS(1492),
    [sym_time_to_hour] = ACTIONS(1492),
    [sym_time_to_minute] = ACTIONS(1492),
    [sym_time_to_second] = ACTIONS(1492),
    [sym_day_of_week] = ACTIONS(1492),
    [sym_set_pulse_time] = ACTIONS(1492),
    [sym_set_timer] = ACTIONS(1492),
    [sym_file_open] = ACTIONS(1492),
    [sym_file_close] = ACTIONS(1492),
    [sym_file_read] = ACTIONS(1492),
    [sym_file_read_line] = ACTIONS(1492),
    [sym_file_write] = ACTIONS(1492),
    [sym_file_write_line] = ACTIONS(1492),
    [sym_file_copy] = ACTIONS(1492),
    [sym_file_delete] = ACTIONS(1492),
    [sym_file_rename] = ACTIONS(1492),
    [sym_file_dir] = ACTIONS(1492),
    [sym_file_getdir] = ACTIONS(1492),
    [sym_file_setdir] = ACTIONS(1492),
    [sym_file_create_dir] = ACTIONS(1492),
    [sym_file_remove_dir] = ACTIONS(1492),
    [sym_file_seek] = ACTIONS(1492),
    [sym_length_array] = ACTIONS(1492),
    [sym_max_length_array] = ACTIONS(1492),
    [sym_set_length_array] = ACTIONS(1492),
    [sym_ip_client_open] = ACTIONS(1492),
    [sym_ip_client_close] = ACTIONS(1492),
    [sym_ip_server_open] = ACTIONS(1492),
    [sym_ip_server_close] = ACTIONS(1492),
    [sym_ip_bound_client_open] = ACTIONS(1492),
    [sym_ip_mc_server_open] = ACTIONS(1492),
    [sym_ip_set_option] = ACTIONS(1492),
    [sym_ssh_client_open] = ACTIONS(1492),
    [sym_ssh_client_close] = ACTIONS(1492),
    [sym_tls_client_open] = ACTIONS(1492),
    [sym_tls_client_close] = ACTIONS(1492),
    [sym_do_push] = ACTIONS(1492),
    [sym_do_push_timed] = ACTIONS(1492),
    [sym_do_release] = ACTIONS(1492),
    [sym_device_id] = ACTIONS(1492),
    [sym_device_info] = ACTIONS(1492),
    [sym_device_id_string] = ACTIONS(1492),
    [sym_device_standby] = ACTIONS(1492),
    [sym_device_wake] = ACTIONS(1492),
    [sym_timeline_create] = ACTIONS(1492),
    [sym_timeline_kill] = ACTIONS(1492),
    [sym_timeline_reload] = ACTIONS(1492),
    [sym_timeline_pause] = ACTIONS(1492),
    [sym_timeline_restart] = ACTIONS(1492),
    [sym_timeline_set] = ACTIONS(1492),
    [sym_timeline_get] = ACTIONS(1492),
    [sym_timeline_active] = ACTIONS(1492),
    [sym_timeline_dynamic_id] = ACTIONS(1492),
    [sym_variable_to_string] = ACTIONS(1492),
    [sym_string_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_string] = ACTIONS(1492),
    [sym_type_cast] = ACTIONS(1492),
    [sym_variable_to_xml] = ACTIONS(1492),
    [sym_xml_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_xml] = ACTIONS(1492),
    [sym_redirect_string] = ACTIONS(1492),
    [sym_random_number] = ACTIONS(1492),
    [sym_get_unique_id] = ACTIONS(1492),
    [sym_get_system_number] = ACTIONS(1492),
    [sym_set_system_number] = ACTIONS(1492),
    [sym_get_serial_number] = ACTIONS(1492),
    [sym_get_master_build] = ACTIONS(1492),
    [sym_set_validation_code] = ACTIONS(1492),
    [sym_reboot] = ACTIONS(1492),
    [sym_load_duet_module] = ACTIONS(1492),
    [sym_unload_duet_module] = ACTIONS(1492),
    [sym_duet_mem_size_set] = ACTIONS(1492),
    [sym_duet_mem_size_get] = ACTIONS(1492),
    [sym_dynamic_application_device] = ACTIONS(1492),
    [sym_dynamic_polled_port] = ACTIONS(1492),
    [sym_static_port_binding] = ACTIONS(1492),
    [sym_static_ip_binding] = ACTIONS(1492),
    [sym_get_dns_list] = ACTIONS(1492),
    [sym_set_dns_list] = ACTIONS(1492),
    [sym_get_ip_address] = ACTIONS(1492),
    [sym_set_ip_address] = ACTIONS(1492),
    [sym_get_url_list] = ACTIONS(1492),
    [sym_add_url_entry] = ACTIONS(1492),
    [sym_delete_url_entry] = ACTIONS(1492),
    [sym_abs_value] = ACTIONS(1492),
    [sym_max_value] = ACTIONS(1492),
    [sym_min_value] = ACTIONS(1492),
    [sym_exp_value] = ACTIONS(1492),
    [sym_log_value] = ACTIONS(1492),
    [sym_log10_value] = ACTIONS(1492),
    [sym_power_value] = ACTIONS(1492),
    [sym_sqrt_value] = ACTIONS(1492),
    [sym_raw_be] = ACTIONS(1492),
    [sym_raw_le] = ACTIONS(1492),
    [sym_internal_queue_size_set] = ACTIONS(1492),
    [sym_internal_queue_size_get] = ACTIONS(1492),
    [sym_internal_threshold_set] = ACTIONS(1492),
    [sym_internal_threshold_get] = ACTIONS(1492),
    [sym_rebuilt_event] = ACTIONS(1492),
    [sym_do_custom_event] = ACTIONS(1492),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1492),
    [sym_clkmgr_set_clk_source] = ACTIONS(1492),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1492),
    [sym_clkmgr_get_timezone] = ACTIONS(1492),
    [sym_clkmgr_set_timezone] = ACTIONS(1492),
    [sym_clkmgr_get_resync_period] = ACTIONS(1492),
    [sym_clkmgr_set_resync_period] = ACTIONS(1492),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_timeservers] = ACTIONS(1492),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_set_log_level] = ACTIONS(1492),
    [sym_get_log_level] = ACTIONS(1492),
    [sym_amx_log] = ACTIONS(1492),
    [sym_smtp_server_config_set] = ACTIONS(1492),
    [sym_smtp_server_config_get] = ACTIONS(1492),
    [sym_smtp_send] = ACTIONS(1492),
    [sym_validate_netlinx_account] = ACTIONS(1492),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1492),
    [sym_authenticate_certificate] = ACTIONS(1492),
    [sym_audit_netlinx_session_event] = ACTIONS(1492),
    [sym_audit_netlinx_generic_event] = ACTIONS(1492),
    [sym_get_available_flash_disk_space] = ACTIONS(1492),
    [sym_get_max_flash_disk_space] = ACTIONS(1492),
    [sym_data_create_feed] = ACTIONS(1492),
    [sym_data_delete_feed] = ACTIONS(1492),
    [sym_data_publish_feed] = ACTIONS(1492),
    [sym_data_get_published_feed] = ACTIONS(1492),
    [sym_data_add_record] = ACTIONS(1492),
    [sym_data_get_event_record] = ACTIONS(1492),
    [sym__wc_data_create_feed] = ACTIONS(1492),
    [sym__wc_data_add_record] = ACTIONS(1492),
    [sym__wc_data_get_event_record] = ACTIONS(1492),
    [sym_format] = ACTIONS(1492),
    [sym_astro_clock] = ACTIONS(1492),
    [sym_set_outdoor_temperature] = ACTIONS(1492),
    [sym_set_virtual_level_count] = ACTIONS(1492),
    [sym_set_virtual_channel_count] = ACTIONS(1492),
    [sym_set_virtual_port_count] = ACTIONS(1492),
    [sym__wc_encode_int] = ACTIONS(1492),
    [sym__wc_decode_int] = ACTIONS(1492),
    [sym__wc_int] = ACTIONS(1492),
    [sym__wc_to_ch_int] = ACTIONS(1492),
    [sym__ch_to_wc_int] = ACTIONS(1492),
    [sym__wc_find_string_int] = ACTIONS(1492),
    [sym__wc_left_string_int] = ACTIONS(1492),
    [sym__wc_lower_string_int] = ACTIONS(1492),
    [sym__wc_mid_string_int] = ACTIONS(1492),
    [sym__wc_remove_string_int] = ACTIONS(1492),
    [sym__wc_right_string_int] = ACTIONS(1492),
    [sym__wc_upper_string_int] = ACTIONS(1492),
    [sym__wc_compare_string_int] = ACTIONS(1492),
    [sym__wc_get_buffer_char_int] = ACTIONS(1492),
    [sym__wc_get_buffer_string_int] = ACTIONS(1492),
    [sym__wc_concat_string_int] = ACTIONS(1492),
    [sym__wc_file_open_int] = ACTIONS(1492),
    [sym__wc_file_close_int] = ACTIONS(1492),
    [sym__wc_file_read_int] = ACTIONS(1492),
    [sym__wc_file_read_line_int] = ACTIONS(1492),
    [sym__wc_file_write_int] = ACTIONS(1492),
    [sym__wc_file_write_line_int] = ACTIONS(1492),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1494),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1494),
    [aux_sym_preproc_else_token1] = ACTIONS(1494),
    [anon_sym_COMMA] = ACTIONS(1494),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1494),
    [anon_sym_RBRACE] = ACTIONS(1494),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1492),
    [anon_sym_TILDE] = ACTIONS(1494),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1494),
    [anon_sym_SQUOTE] = ACTIONS(1494),
    [sym_number_literal] = ACTIONS(1494),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(584)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_COMMA] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(585)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1413),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_COMMA] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(586)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_COMMA] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(587)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_COMMA] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(588)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_COMMA] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(589)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1481),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(1481),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(1481),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(1481),
    [sym___date__] = ACTIONS(1481),
    [sym___ldate__] = ACTIONS(1481),
    [sym___time__] = ACTIONS(1481),
    [sym___line__] = ACTIONS(1481),
    [sym___name__] = ACTIONS(1481),
    [sym___netlinx__] = ACTIONS(1481),
    [sym_netlinx_axi_version] = ACTIONS(1481),
    [sym_url_flg_tcp] = ACTIONS(1481),
    [sym_url_flg_accinfopresent] = ACTIONS(1481),
    [sym_url_flg_temp] = ACTIONS(1481),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1481),
    [sym_url_flg_stat_mask] = ACTIONS(1481),
    [sym_url_flg_stat_lookup] = ACTIONS(1481),
    [sym_url_flg_stat_connecting] = ACTIONS(1481),
    [sym_url_flg_stat_waiting] = ACTIONS(1481),
    [sym_url_flg_stat_connected] = ACTIONS(1481),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1481),
    [sym_first_virtual_device] = ACTIONS(1481),
    [sym_first_local_port] = ACTIONS(1481),
    [sym_dynamic_virtual_device] = ACTIONS(1481),
    [sym_timeline_once] = ACTIONS(1481),
    [sym_timeline_repeat] = ACTIONS(1481),
    [sym_timeline_absolute] = ACTIONS(1481),
    [sym_timeline_relative] = ACTIONS(1481),
    [sym_do_push_timed_infinite] = ACTIONS(1481),
    [sym_source_type_no_address] = ACTIONS(1481),
    [sym_source_type_neuron_id] = ACTIONS(1481),
    [sym_source_type_ip_address] = ACTIONS(1481),
    [sym_source_type_axlink] = ACTIONS(1481),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1481),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1481),
    [sym_source_type_ip_socket_address] = ACTIONS(1481),
    [sym_source_type_rs232] = ACTIONS(1481),
    [sym_source_type_internal] = ACTIONS(1481),
    [sym_source_type_mac_address] = ACTIONS(1481),
    [sym_source_type_ipv4_port] = ACTIONS(1481),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1481),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1481),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1481),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1481),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1481),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1481),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1481),
    [sym_true] = ACTIONS(1481),
    [sym_false] = ACTIONS(1481),
    [sym_file_read_only] = ACTIONS(1481),
    [sym_file_rw_new] = ACTIONS(1481),
    [sym_file_rw_append] = ACTIONS(1481),
    [sym_ip_tcp] = ACTIONS(1481),
    [sym_ip_udp] = ACTIONS(1481),
    [sym_ip_udp_2way] = ACTIONS(1481),
    [sym_xml_encode_types] = ACTIONS(1481),
    [sym_xml_encode_char_as_list] = ACTIONS(1481),
    [sym_xml_encode_le] = ACTIONS(1481),
    [sym_xml_decode_types] = ACTIONS(1481),
    [sym_xml_decode_no_preserve] = ACTIONS(1481),
    [sym_duet_dev_type_display_device] = ACTIONS(1481),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1481),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1481),
    [sym_duet_dev_type_io_device] = ACTIONS(1481),
    [sym_duet_dev_type_relay_device] = ACTIONS(1481),
    [sym_duet_dev_type_ups] = ACTIONS(1481),
    [sym_duet_dev_type_amplifier] = ACTIONS(1481),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1481),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1481),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1481),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1481),
    [sym_duet_dev_type_camera] = ACTIONS(1481),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1481),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1481),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1481),
    [sym_duet_dev_type_dss] = ACTIONS(1481),
    [sym_duet_dev_type_dvr] = ACTIONS(1481),
    [sym_duet_dev_type_disc_device] = ACTIONS(1481),
    [sym_duet_dev_type_document_camera] = ACTIONS(1481),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1481),
    [sym_duet_dev_type_hvac] = ACTIONS(1481),
    [sym_duet_dev_type_keypad] = ACTIONS(1481),
    [sym_duet_dev_type_light] = ACTIONS(1481),
    [sym_duet_dev_type_monitor] = ACTIONS(1481),
    [sym_duet_dev_type_motor] = ACTIONS(1481),
    [sym_duet_dev_type_multi_window] = ACTIONS(1481),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1481),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1481),
    [sym_duet_dev_type_receiver] = ACTIONS(1481),
    [sym_duet_dev_type_security_system] = ACTIONS(1481),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1481),
    [sym_duet_dev_type_settop_box] = ACTIONS(1481),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1481),
    [sym_duet_dev_type_switcher] = ACTIONS(1481),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1481),
    [sym_duet_dev_type_tv] = ACTIONS(1481),
    [sym_duet_dev_type_utility] = ACTIONS(1481),
    [sym_duet_dev_type_vcr] = ACTIONS(1481),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1481),
    [sym_duet_dev_type_video_processor] = ACTIONS(1481),
    [sym_duet_dev_type_video_projector] = ACTIONS(1481),
    [sym_duet_dev_type_video_wall] = ACTIONS(1481),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1481),
    [sym_duet_dev_type_weather] = ACTIONS(1481),
    [sym_duet_dev_not_polled] = ACTIONS(1481),
    [sym_duet_dev_polled] = ACTIONS(1481),
    [sym_clkmgr_mode_network] = ACTIONS(1481),
    [sym_clkmgr_mode_standalone] = ACTIONS(1481),
    [sym_normal_standby] = ACTIONS(1481),
    [sym_normal_wake] = ACTIONS(1481),
    [sym_ip_multicast_ttl_option] = ACTIONS(1481),
    [sym_ip_ttl_subnet] = ACTIONS(1481),
    [sym_ip_ttl_site] = ACTIONS(1481),
    [sym_ip_ttl_region] = ACTIONS(1481),
    [sym_ip_ttl_continent] = ACTIONS(1481),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1481),
    [sym_ip_nodelay_on] = ACTIONS(1481),
    [sym_ip_nodelay_off] = ACTIONS(1481),
    [sym_amx_error] = ACTIONS(1481),
    [sym_amx_warning] = ACTIONS(1481),
    [sym_amx_info] = ACTIONS(1481),
    [sym_amx_debug] = ACTIONS(1481),
    [sym_netlinx_login_success] = ACTIONS(1481),
    [sym_netlinx_login_fail] = ACTIONS(1481),
    [sym_netlinx_logout] = ACTIONS(1481),
    [sym_null_str] = ACTIONS(1481),
    [sym_smtp_address] = ACTIONS(1481),
    [sym_smtp_port_number] = ACTIONS(1481),
    [sym_smtp_username] = ACTIONS(1481),
    [sym_smtp_password] = ACTIONS(1481),
    [sym_smtp_require_tls] = ACTIONS(1481),
    [sym_smtp_from] = ACTIONS(1481),
    [sym_smtp_tls_true] = ACTIONS(1481),
    [sym_smtp_tls_false] = ACTIONS(1481),
    [sym_listview_on_row_select_event] = ACTIONS(1481),
    [sym_data_structure_unknown] = ACTIONS(1481),
    [sym_data_structure_datafeed] = ACTIONS(1481),
    [sym_data_structure_datarecordset] = ACTIONS(1481),
    [sym_data_structure_datarecord] = ACTIONS(1481),
    [sym_data_structure_datafield] = ACTIONS(1481),
    [sym_data_max_content_field_count] = ACTIONS(1481),
    [sym_data_max_metadata_field_count] = ACTIONS(1481),
    [sym_data_max_id_length] = ACTIONS(1481),
    [sym_data_max_name_length] = ACTIONS(1481),
    [sym_data_max_description_length] = ACTIONS(1481),
    [sym_data_max_type_length] = ACTIONS(1481),
    [sym_data_max_format_length] = ACTIONS(1481),
    [sym_data_max_source_length] = ACTIONS(1481),
    [sym_data_max_value_length] = ACTIONS(1481),
    [sym_data_max_label_length] = ACTIONS(1481),
    [sym_data_max_url_length] = ACTIONS(1481),
    [sym_max_last_login_info_length] = ACTIONS(1481),
    [sym_data_type_unknown] = ACTIONS(1481),
    [sym_data_type_string] = ACTIONS(1481),
    [sym_data_type_datetime] = ACTIONS(1481),
    [sym_data_type_date] = ACTIONS(1481),
    [sym_data_type_time] = ACTIONS(1481),
    [sym_data_type_image] = ACTIONS(1481),
    [sym_data_format_url] = ACTIONS(1481),
    [sym_data_format_phone] = ACTIONS(1481),
    [sym_data_format_email] = ACTIONS(1481),
    [sym_data_format_iso8601] = ACTIONS(1481),
    [sym_auditlog_priv] = ACTIONS(1481),
    [sym_configuration_priv] = ACTIONS(1481),
    [sym_deviceconfig_priv] = ACTIONS(1481),
    [sym_ftp_priv] = ACTIONS(1481),
    [sym_http_priv] = ACTIONS(1481),
    [sym_networkconfig_priv] = ACTIONS(1481),
    [sym_programport_priv] = ACTIONS(1481),
    [sym_remoteui_priv] = ACTIONS(1481),
    [sym_securitycontrol_priv] = ACTIONS(1481),
    [sym_softwaremanagement_priv] = ACTIONS(1481),
    [sym_terminal_priv] = ACTIONS(1481),
    [sym_tpadmin_priv] = ACTIONS(1481),
    [sym_user1_priv] = ACTIONS(1481),
    [sym_user2_priv] = ACTIONS(1481),
    [sym_user3_priv] = ACTIONS(1481),
    [sym_user4_priv] = ACTIONS(1481),
    [sym_usermanagement_priv] = ACTIONS(1481),
    [sym_valid_account] = ACTIONS(1481),
    [sym_err_name_invalid] = ACTIONS(1481),
    [sym_err_password_invalid] = ACTIONS(1481),
    [sym_err_invalid_account] = ACTIONS(1481),
    [sym_err_no_authorization] = ACTIONS(1481),
    [sym_err_invalid_parameter] = ACTIONS(1481),
    [sym_err_user_locked_out] = ACTIONS(1481),
    [sym_err_user_account_expired] = ACTIONS(1481),
    [sym_err_general_error] = ACTIONS(1481),
    [sym_err_authentication_error] = ACTIONS(1481),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1481),
    [sym_tls_validate_certificate] = ACTIONS(1481),
    [sym____reserved___] = ACTIONS(1481),
    [sym_date] = ACTIONS(1481),
    [sym_day] = ACTIONS(1481),
    [sym_time] = ACTIONS(1481),
    [sym_ldate] = ACTIONS(1481),
    [sym_push_device] = ACTIONS(1481),
    [sym_push_channel] = ACTIONS(1481),
    [sym_push_devchan] = ACTIONS(1481),
    [sym_release_device] = ACTIONS(1481),
    [sym_release_channel] = ACTIONS(1481),
    [sym_release_devchan] = ACTIONS(1481),
    [sym_master_slot] = ACTIONS(1481),
    [sym_get_pulse_time] = ACTIONS(1481),
    [sym_get_timer] = ACTIONS(1481),
    [sym_master_sn] = ACTIONS(1481),
    [sym_system_number] = ACTIONS(1481),
    [sym_button] = ACTIONS(1481),
    [sym_dv_channel] = ACTIONS(1481),
    [sym_data] = ACTIONS(1481),
    [sym_level] = ACTIONS(1481),
    [sym_timeline] = ACTIONS(1481),
    [sym_channel] = ACTIONS(1481),
    [sym_custom] = ACTIONS(1481),
    [sym_atoi] = ACTIONS(1481),
    [sym_atol] = ACTIONS(1481),
    [sym_atof] = ACTIONS(1481),
    [sym_itoa] = ACTIONS(1481),
    [sym_itohex] = ACTIONS(1481),
    [sym_hextoi] = ACTIONS(1481),
    [sym_ftoa] = ACTIONS(1481),
    [sym_find_string] = ACTIONS(1481),
    [sym_left_string] = ACTIONS(1481),
    [sym_right_string] = ACTIONS(1481),
    [sym_mid_string] = ACTIONS(1481),
    [sym_remove_string] = ACTIONS(1481),
    [sym_get_buffer_string] = ACTIONS(1481),
    [sym_get_buffer_char] = ACTIONS(1481),
    [sym_get_multi_buffer_string] = ACTIONS(1481),
    [sym_length_string] = ACTIONS(1481),
    [sym_max_length_string] = ACTIONS(1481),
    [sym_set_length_string] = ACTIONS(1481),
    [sym_compare_string] = ACTIONS(1481),
    [sym_upper_string] = ACTIONS(1481),
    [sym_lower_string] = ACTIONS(1481),
    [sym_get_last] = ACTIONS(1481),
    [sym_date_to_day] = ACTIONS(1481),
    [sym_date_to_month] = ACTIONS(1481),
    [sym_date_to_year] = ACTIONS(1481),
    [sym_time_to_hour] = ACTIONS(1481),
    [sym_time_to_minute] = ACTIONS(1481),
    [sym_time_to_second] = ACTIONS(1481),
    [sym_day_of_week] = ACTIONS(1481),
    [sym_set_pulse_time] = ACTIONS(1481),
    [sym_set_timer] = ACTIONS(1481),
    [sym_file_open] = ACTIONS(1481),
    [sym_file_close] = ACTIONS(1481),
    [sym_file_read] = ACTIONS(1481),
    [sym_file_read_line] = ACTIONS(1481),
    [sym_file_write] = ACTIONS(1481),
    [sym_file_write_line] = ACTIONS(1481),
    [sym_file_copy] = ACTIONS(1481),
    [sym_file_delete] = ACTIONS(1481),
    [sym_file_rename] = ACTIONS(1481),
    [sym_file_dir] = ACTIONS(1481),
    [sym_file_getdir] = ACTIONS(1481),
    [sym_file_setdir] = ACTIONS(1481),
    [sym_file_create_dir] = ACTIONS(1481),
    [sym_file_remove_dir] = ACTIONS(1481),
    [sym_file_seek] = ACTIONS(1481),
    [sym_length_array] = ACTIONS(1481),
    [sym_max_length_array] = ACTIONS(1481),
    [sym_set_length_array] = ACTIONS(1481),
    [sym_ip_client_open] = ACTIONS(1481),
    [sym_ip_client_close] = ACTIONS(1481),
    [sym_ip_server_open] = ACTIONS(1481),
    [sym_ip_server_close] = ACTIONS(1481),
    [sym_ip_bound_client_open] = ACTIONS(1481),
    [sym_ip_mc_server_open] = ACTIONS(1481),
    [sym_ip_set_option] = ACTIONS(1481),
    [sym_ssh_client_open] = ACTIONS(1481),
    [sym_ssh_client_close] = ACTIONS(1481),
    [sym_tls_client_open] = ACTIONS(1481),
    [sym_tls_client_close] = ACTIONS(1481),
    [sym_do_push] = ACTIONS(1481),
    [sym_do_push_timed] = ACTIONS(1481),
    [sym_do_release] = ACTIONS(1481),
    [sym_device_id] = ACTIONS(1481),
    [sym_device_info] = ACTIONS(1481),
    [sym_device_id_string] = ACTIONS(1481),
    [sym_device_standby] = ACTIONS(1481),
    [sym_device_wake] = ACTIONS(1481),
    [sym_timeline_create] = ACTIONS(1481),
    [sym_timeline_kill] = ACTIONS(1481),
    [sym_timeline_reload] = ACTIONS(1481),
    [sym_timeline_pause] = ACTIONS(1481),
    [sym_timeline_restart] = ACTIONS(1481),
    [sym_timeline_set] = ACTIONS(1481),
    [sym_timeline_get] = ACTIONS(1481),
    [sym_timeline_active] = ACTIONS(1481),
    [sym_timeline_dynamic_id] = ACTIONS(1481),
    [sym_variable_to_string] = ACTIONS(1481),
    [sym_string_to_variable] = ACTIONS(1481),
    [sym_length_variable_to_string] = ACTIONS(1481),
    [sym_type_cast] = ACTIONS(1481),
    [sym_variable_to_xml] = ACTIONS(1481),
    [sym_xml_to_variable] = ACTIONS(1481),
    [sym_length_variable_to_xml] = ACTIONS(1481),
    [sym_redirect_string] = ACTIONS(1481),
    [sym_random_number] = ACTIONS(1481),
    [sym_get_unique_id] = ACTIONS(1481),
    [sym_get_system_number] = ACTIONS(1481),
    [sym_set_system_number] = ACTIONS(1481),
    [sym_get_serial_number] = ACTIONS(1481),
    [sym_get_master_build] = ACTIONS(1481),
    [sym_set_validation_code] = ACTIONS(1481),
    [sym_reboot] = ACTIONS(1481),
    [sym_load_duet_module] = ACTIONS(1481),
    [sym_unload_duet_module] = ACTIONS(1481),
    [sym_duet_mem_size_set] = ACTIONS(1481),
    [sym_duet_mem_size_get] = ACTIONS(1481),
    [sym_dynamic_application_device] = ACTIONS(1481),
    [sym_dynamic_polled_port] = ACTIONS(1481),
    [sym_static_port_binding] = ACTIONS(1481),
    [sym_static_ip_binding] = ACTIONS(1481),
    [sym_get_dns_list] = ACTIONS(1481),
    [sym_set_dns_list] = ACTIONS(1481),
    [sym_get_ip_address] = ACTIONS(1481),
    [sym_set_ip_address] = ACTIONS(1481),
    [sym_get_url_list] = ACTIONS(1481),
    [sym_add_url_entry] = ACTIONS(1481),
    [sym_delete_url_entry] = ACTIONS(1481),
    [sym_abs_value] = ACTIONS(1481),
    [sym_max_value] = ACTIONS(1481),
    [sym_min_value] = ACTIONS(1481),
    [sym_exp_value] = ACTIONS(1481),
    [sym_log_value] = ACTIONS(1481),
    [sym_log10_value] = ACTIONS(1481),
    [sym_power_value] = ACTIONS(1481),
    [sym_sqrt_value] = ACTIONS(1481),
    [sym_raw_be] = ACTIONS(1481),
    [sym_raw_le] = ACTIONS(1481),
    [sym_internal_queue_size_set] = ACTIONS(1481),
    [sym_internal_queue_size_get] = ACTIONS(1481),
    [sym_internal_threshold_set] = ACTIONS(1481),
    [sym_internal_threshold_get] = ACTIONS(1481),
    [sym_rebuilt_event] = ACTIONS(1481),
    [sym_do_custom_event] = ACTIONS(1481),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1481),
    [sym_clkmgr_set_clk_source] = ACTIONS(1481),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1481),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1481),
    [sym_clkmgr_get_timezone] = ACTIONS(1481),
    [sym_clkmgr_set_timezone] = ACTIONS(1481),
    [sym_clkmgr_get_resync_period] = ACTIONS(1481),
    [sym_clkmgr_set_resync_period] = ACTIONS(1481),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1481),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1481),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1481),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1481),
    [sym_clkmgr_get_timeservers] = ACTIONS(1481),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1481),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1481),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1481),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1481),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1481),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1481),
    [sym_set_log_level] = ACTIONS(1481),
    [sym_get_log_level] = ACTIONS(1481),
    [sym_amx_log] = ACTIONS(1481),
    [sym_smtp_server_config_set] = ACTIONS(1481),
    [sym_smtp_server_config_get] = ACTIONS(1481),
    [sym_smtp_send] = ACTIONS(1481),
    [sym_validate_netlinx_account] = ACTIONS(1481),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1481),
    [sym_authenticate_certificate] = ACTIONS(1481),
    [sym_audit_netlinx_session_event] = ACTIONS(1481),
    [sym_audit_netlinx_generic_event] = ACTIONS(1481),
    [sym_get_available_flash_disk_space] = ACTIONS(1481),
    [sym_get_max_flash_disk_space] = ACTIONS(1481),
    [sym_data_create_feed] = ACTIONS(1481),
    [sym_data_delete_feed] = ACTIONS(1481),
    [sym_data_publish_feed] = ACTIONS(1481),
    [sym_data_get_published_feed] = ACTIONS(1481),
    [sym_data_add_record] = ACTIONS(1481),
    [sym_data_get_event_record] = ACTIONS(1481),
    [sym__wc_data_create_feed] = ACTIONS(1481),
    [sym__wc_data_add_record] = ACTIONS(1481),
    [sym__wc_data_get_event_record] = ACTIONS(1481),
    [sym_format] = ACTIONS(1481),
    [sym_astro_clock] = ACTIONS(1481),
    [sym_set_outdoor_temperature] = ACTIONS(1481),
    [sym_set_virtual_level_count] = ACTIONS(1481),
    [sym_set_virtual_channel_count] = ACTIONS(1481),
    [sym_set_virtual_port_count] = ACTIONS(1481),
    [sym__wc_encode_int] = ACTIONS(1481),
    [sym__wc_decode_int] = ACTIONS(1481),
    [sym__wc_int] = ACTIONS(1481),
    [sym__wc_to_ch_int] = ACTIONS(1481),
    [sym__ch_to_wc_int] = ACTIONS(1481),
    [sym__wc_find_string_int] = ACTIONS(1481),
    [sym__wc_left_string_int] = ACTIONS(1481),
    [sym__wc_lower_string_int] = ACTIONS(1481),
    [sym__wc_mid_string_int] = ACTIONS(1481),
    [sym__wc_remove_string_int] = ACTIONS(1481),
    [sym__wc_right_string_int] = ACTIONS(1481),
    [sym__wc_upper_string_int] = ACTIONS(1481),
    [sym__wc_compare_string_int] = ACTIONS(1481),
    [sym__wc_get_buffer_char_int] = ACTIONS(1481),
    [sym__wc_get_buffer_string_int] = ACTIONS(1481),
    [sym__wc_concat_string_int] = ACTIONS(1481),
    [sym__wc_file_open_int] = ACTIONS(1481),
    [sym__wc_file_close_int] = ACTIONS(1481),
    [sym__wc_file_read_int] = ACTIONS(1481),
    [sym__wc_file_read_line_int] = ACTIONS(1481),
    [sym__wc_file_write_int] = ACTIONS(1481),
    [sym__wc_file_write_line_int] = ACTIONS(1481),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1483),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1483),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1483),
    [aux_sym_preproc_else_token1] = ACTIONS(1483),
    [anon_sym_COMMA] = ACTIONS(1483),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1483),
    [anon_sym_RBRACE] = ACTIONS(1483),
    [anon_sym_COLON] = ACTIONS(2212),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1481),
    [anon_sym_TILDE] = ACTIONS(1483),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1483),
    [anon_sym_SQUOTE] = ACTIONS(1483),
    [sym_number_literal] = ACTIONS(1483),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(590)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1413),
    [sym_rshift] = ACTIONS(1413),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_COMMA] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1415),
    [anon_sym_GT_GT] = ACTIONS(1415),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(591)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1413),
    [sym_rshift] = ACTIONS(1413),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_COMMA] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1415),
    [anon_sym_GT_GT] = ACTIONS(1415),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(592)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(1413),
    [sym_and] = ACTIONS(1413),
    [sym_or] = ACTIONS(1413),
    [sym_not] = ACTIONS(1413),
    [sym_xor] = ACTIONS(1413),
    [sym_band] = ACTIONS(1413),
    [sym_bor] = ACTIONS(1413),
    [sym_bxor] = ACTIONS(1413),
    [sym_bnot] = ACTIONS(1413),
    [sym_lshift] = ACTIONS(1413),
    [sym_rshift] = ACTIONS(1413),
    [sym___file__] = ACTIONS(1413),
    [sym___date__] = ACTIONS(1413),
    [sym___ldate__] = ACTIONS(1413),
    [sym___time__] = ACTIONS(1413),
    [sym___line__] = ACTIONS(1413),
    [sym___name__] = ACTIONS(1413),
    [sym___netlinx__] = ACTIONS(1413),
    [sym_netlinx_axi_version] = ACTIONS(1413),
    [sym_url_flg_tcp] = ACTIONS(1413),
    [sym_url_flg_accinfopresent] = ACTIONS(1413),
    [sym_url_flg_temp] = ACTIONS(1413),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1413),
    [sym_url_flg_stat_mask] = ACTIONS(1413),
    [sym_url_flg_stat_lookup] = ACTIONS(1413),
    [sym_url_flg_stat_connecting] = ACTIONS(1413),
    [sym_url_flg_stat_waiting] = ACTIONS(1413),
    [sym_url_flg_stat_connected] = ACTIONS(1413),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1413),
    [sym_first_virtual_device] = ACTIONS(1413),
    [sym_first_local_port] = ACTIONS(1413),
    [sym_dynamic_virtual_device] = ACTIONS(1413),
    [sym_timeline_once] = ACTIONS(1413),
    [sym_timeline_repeat] = ACTIONS(1413),
    [sym_timeline_absolute] = ACTIONS(1413),
    [sym_timeline_relative] = ACTIONS(1413),
    [sym_do_push_timed_infinite] = ACTIONS(1413),
    [sym_source_type_no_address] = ACTIONS(1413),
    [sym_source_type_neuron_id] = ACTIONS(1413),
    [sym_source_type_ip_address] = ACTIONS(1413),
    [sym_source_type_axlink] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1413),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1413),
    [sym_source_type_ip_socket_address] = ACTIONS(1413),
    [sym_source_type_rs232] = ACTIONS(1413),
    [sym_source_type_internal] = ACTIONS(1413),
    [sym_source_type_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1413),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1413),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1413),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1413),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1413),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1413),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1413),
    [sym_true] = ACTIONS(1413),
    [sym_false] = ACTIONS(1413),
    [sym_file_read_only] = ACTIONS(1413),
    [sym_file_rw_new] = ACTIONS(1413),
    [sym_file_rw_append] = ACTIONS(1413),
    [sym_ip_tcp] = ACTIONS(1413),
    [sym_ip_udp] = ACTIONS(1413),
    [sym_ip_udp_2way] = ACTIONS(1413),
    [sym_xml_encode_types] = ACTIONS(1413),
    [sym_xml_encode_char_as_list] = ACTIONS(1413),
    [sym_xml_encode_le] = ACTIONS(1413),
    [sym_xml_decode_types] = ACTIONS(1413),
    [sym_xml_decode_no_preserve] = ACTIONS(1413),
    [sym_duet_dev_type_display_device] = ACTIONS(1413),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1413),
    [sym_duet_dev_type_io_device] = ACTIONS(1413),
    [sym_duet_dev_type_relay_device] = ACTIONS(1413),
    [sym_duet_dev_type_ups] = ACTIONS(1413),
    [sym_duet_dev_type_amplifier] = ACTIONS(1413),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1413),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1413),
    [sym_duet_dev_type_camera] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1413),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1413),
    [sym_duet_dev_type_dss] = ACTIONS(1413),
    [sym_duet_dev_type_dvr] = ACTIONS(1413),
    [sym_duet_dev_type_disc_device] = ACTIONS(1413),
    [sym_duet_dev_type_document_camera] = ACTIONS(1413),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1413),
    [sym_duet_dev_type_hvac] = ACTIONS(1413),
    [sym_duet_dev_type_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_light] = ACTIONS(1413),
    [sym_duet_dev_type_monitor] = ACTIONS(1413),
    [sym_duet_dev_type_motor] = ACTIONS(1413),
    [sym_duet_dev_type_multi_window] = ACTIONS(1413),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1413),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1413),
    [sym_duet_dev_type_receiver] = ACTIONS(1413),
    [sym_duet_dev_type_security_system] = ACTIONS(1413),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1413),
    [sym_duet_dev_type_settop_box] = ACTIONS(1413),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1413),
    [sym_duet_dev_type_switcher] = ACTIONS(1413),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1413),
    [sym_duet_dev_type_tv] = ACTIONS(1413),
    [sym_duet_dev_type_utility] = ACTIONS(1413),
    [sym_duet_dev_type_vcr] = ACTIONS(1413),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1413),
    [sym_duet_dev_type_video_processor] = ACTIONS(1413),
    [sym_duet_dev_type_video_projector] = ACTIONS(1413),
    [sym_duet_dev_type_video_wall] = ACTIONS(1413),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1413),
    [sym_duet_dev_type_weather] = ACTIONS(1413),
    [sym_duet_dev_not_polled] = ACTIONS(1413),
    [sym_duet_dev_polled] = ACTIONS(1413),
    [sym_clkmgr_mode_network] = ACTIONS(1413),
    [sym_clkmgr_mode_standalone] = ACTIONS(1413),
    [sym_normal_standby] = ACTIONS(1413),
    [sym_normal_wake] = ACTIONS(1413),
    [sym_ip_multicast_ttl_option] = ACTIONS(1413),
    [sym_ip_ttl_subnet] = ACTIONS(1413),
    [sym_ip_ttl_site] = ACTIONS(1413),
    [sym_ip_ttl_region] = ACTIONS(1413),
    [sym_ip_ttl_continent] = ACTIONS(1413),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1413),
    [sym_ip_nodelay_on] = ACTIONS(1413),
    [sym_ip_nodelay_off] = ACTIONS(1413),
    [sym_amx_error] = ACTIONS(1413),
    [sym_amx_warning] = ACTIONS(1413),
    [sym_amx_info] = ACTIONS(1413),
    [sym_amx_debug] = ACTIONS(1413),
    [sym_netlinx_login_success] = ACTIONS(1413),
    [sym_netlinx_login_fail] = ACTIONS(1413),
    [sym_netlinx_logout] = ACTIONS(1413),
    [sym_null_str] = ACTIONS(1413),
    [sym_smtp_address] = ACTIONS(1413),
    [sym_smtp_port_number] = ACTIONS(1413),
    [sym_smtp_username] = ACTIONS(1413),
    [sym_smtp_password] = ACTIONS(1413),
    [sym_smtp_require_tls] = ACTIONS(1413),
    [sym_smtp_from] = ACTIONS(1413),
    [sym_smtp_tls_true] = ACTIONS(1413),
    [sym_smtp_tls_false] = ACTIONS(1413),
    [sym_listview_on_row_select_event] = ACTIONS(1413),
    [sym_data_structure_unknown] = ACTIONS(1413),
    [sym_data_structure_datafeed] = ACTIONS(1413),
    [sym_data_structure_datarecordset] = ACTIONS(1413),
    [sym_data_structure_datarecord] = ACTIONS(1413),
    [sym_data_structure_datafield] = ACTIONS(1413),
    [sym_data_max_content_field_count] = ACTIONS(1413),
    [sym_data_max_metadata_field_count] = ACTIONS(1413),
    [sym_data_max_id_length] = ACTIONS(1413),
    [sym_data_max_name_length] = ACTIONS(1413),
    [sym_data_max_description_length] = ACTIONS(1413),
    [sym_data_max_type_length] = ACTIONS(1413),
    [sym_data_max_format_length] = ACTIONS(1413),
    [sym_data_max_source_length] = ACTIONS(1413),
    [sym_data_max_value_length] = ACTIONS(1413),
    [sym_data_max_label_length] = ACTIONS(1413),
    [sym_data_max_url_length] = ACTIONS(1413),
    [sym_max_last_login_info_length] = ACTIONS(1413),
    [sym_data_type_unknown] = ACTIONS(1413),
    [sym_data_type_string] = ACTIONS(1413),
    [sym_data_type_datetime] = ACTIONS(1413),
    [sym_data_type_date] = ACTIONS(1413),
    [sym_data_type_time] = ACTIONS(1413),
    [sym_data_type_image] = ACTIONS(1413),
    [sym_data_format_url] = ACTIONS(1413),
    [sym_data_format_phone] = ACTIONS(1413),
    [sym_data_format_email] = ACTIONS(1413),
    [sym_data_format_iso8601] = ACTIONS(1413),
    [sym_auditlog_priv] = ACTIONS(1413),
    [sym_configuration_priv] = ACTIONS(1413),
    [sym_deviceconfig_priv] = ACTIONS(1413),
    [sym_ftp_priv] = ACTIONS(1413),
    [sym_http_priv] = ACTIONS(1413),
    [sym_networkconfig_priv] = ACTIONS(1413),
    [sym_programport_priv] = ACTIONS(1413),
    [sym_remoteui_priv] = ACTIONS(1413),
    [sym_securitycontrol_priv] = ACTIONS(1413),
    [sym_softwaremanagement_priv] = ACTIONS(1413),
    [sym_terminal_priv] = ACTIONS(1413),
    [sym_tpadmin_priv] = ACTIONS(1413),
    [sym_user1_priv] = ACTIONS(1413),
    [sym_user2_priv] = ACTIONS(1413),
    [sym_user3_priv] = ACTIONS(1413),
    [sym_user4_priv] = ACTIONS(1413),
    [sym_usermanagement_priv] = ACTIONS(1413),
    [sym_valid_account] = ACTIONS(1413),
    [sym_err_name_invalid] = ACTIONS(1413),
    [sym_err_password_invalid] = ACTIONS(1413),
    [sym_err_invalid_account] = ACTIONS(1413),
    [sym_err_no_authorization] = ACTIONS(1413),
    [sym_err_invalid_parameter] = ACTIONS(1413),
    [sym_err_user_locked_out] = ACTIONS(1413),
    [sym_err_user_account_expired] = ACTIONS(1413),
    [sym_err_general_error] = ACTIONS(1413),
    [sym_err_authentication_error] = ACTIONS(1413),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1413),
    [sym_tls_validate_certificate] = ACTIONS(1413),
    [sym____reserved___] = ACTIONS(1413),
    [sym_date] = ACTIONS(1413),
    [sym_day] = ACTIONS(1413),
    [sym_time] = ACTIONS(1413),
    [sym_ldate] = ACTIONS(1413),
    [sym_push_device] = ACTIONS(1413),
    [sym_push_channel] = ACTIONS(1413),
    [sym_push_devchan] = ACTIONS(1413),
    [sym_release_device] = ACTIONS(1413),
    [sym_release_channel] = ACTIONS(1413),
    [sym_release_devchan] = ACTIONS(1413),
    [sym_master_slot] = ACTIONS(1413),
    [sym_get_pulse_time] = ACTIONS(1413),
    [sym_get_timer] = ACTIONS(1413),
    [sym_master_sn] = ACTIONS(1413),
    [sym_system_number] = ACTIONS(1413),
    [sym_button] = ACTIONS(1413),
    [sym_dv_channel] = ACTIONS(1413),
    [sym_data] = ACTIONS(1413),
    [sym_level] = ACTIONS(1413),
    [sym_timeline] = ACTIONS(1413),
    [sym_channel] = ACTIONS(1413),
    [sym_custom] = ACTIONS(1413),
    [sym_atoi] = ACTIONS(1413),
    [sym_atol] = ACTIONS(1413),
    [sym_atof] = ACTIONS(1413),
    [sym_itoa] = ACTIONS(1413),
    [sym_itohex] = ACTIONS(1413),
    [sym_hextoi] = ACTIONS(1413),
    [sym_ftoa] = ACTIONS(1413),
    [sym_find_string] = ACTIONS(1413),
    [sym_left_string] = ACTIONS(1413),
    [sym_right_string] = ACTIONS(1413),
    [sym_mid_string] = ACTIONS(1413),
    [sym_remove_string] = ACTIONS(1413),
    [sym_get_buffer_string] = ACTIONS(1413),
    [sym_get_buffer_char] = ACTIONS(1413),
    [sym_get_multi_buffer_string] = ACTIONS(1413),
    [sym_length_string] = ACTIONS(1413),
    [sym_max_length_string] = ACTIONS(1413),
    [sym_set_length_string] = ACTIONS(1413),
    [sym_compare_string] = ACTIONS(1413),
    [sym_upper_string] = ACTIONS(1413),
    [sym_lower_string] = ACTIONS(1413),
    [sym_get_last] = ACTIONS(1413),
    [sym_date_to_day] = ACTIONS(1413),
    [sym_date_to_month] = ACTIONS(1413),
    [sym_date_to_year] = ACTIONS(1413),
    [sym_time_to_hour] = ACTIONS(1413),
    [sym_time_to_minute] = ACTIONS(1413),
    [sym_time_to_second] = ACTIONS(1413),
    [sym_day_of_week] = ACTIONS(1413),
    [sym_set_pulse_time] = ACTIONS(1413),
    [sym_set_timer] = ACTIONS(1413),
    [sym_file_open] = ACTIONS(1413),
    [sym_file_close] = ACTIONS(1413),
    [sym_file_read] = ACTIONS(1413),
    [sym_file_read_line] = ACTIONS(1413),
    [sym_file_write] = ACTIONS(1413),
    [sym_file_write_line] = ACTIONS(1413),
    [sym_file_copy] = ACTIONS(1413),
    [sym_file_delete] = ACTIONS(1413),
    [sym_file_rename] = ACTIONS(1413),
    [sym_file_dir] = ACTIONS(1413),
    [sym_file_getdir] = ACTIONS(1413),
    [sym_file_setdir] = ACTIONS(1413),
    [sym_file_create_dir] = ACTIONS(1413),
    [sym_file_remove_dir] = ACTIONS(1413),
    [sym_file_seek] = ACTIONS(1413),
    [sym_length_array] = ACTIONS(1413),
    [sym_max_length_array] = ACTIONS(1413),
    [sym_set_length_array] = ACTIONS(1413),
    [sym_ip_client_open] = ACTIONS(1413),
    [sym_ip_client_close] = ACTIONS(1413),
    [sym_ip_server_open] = ACTIONS(1413),
    [sym_ip_server_close] = ACTIONS(1413),
    [sym_ip_bound_client_open] = ACTIONS(1413),
    [sym_ip_mc_server_open] = ACTIONS(1413),
    [sym_ip_set_option] = ACTIONS(1413),
    [sym_ssh_client_open] = ACTIONS(1413),
    [sym_ssh_client_close] = ACTIONS(1413),
    [sym_tls_client_open] = ACTIONS(1413),
    [sym_tls_client_close] = ACTIONS(1413),
    [sym_do_push] = ACTIONS(1413),
    [sym_do_push_timed] = ACTIONS(1413),
    [sym_do_release] = ACTIONS(1413),
    [sym_device_id] = ACTIONS(1413),
    [sym_device_info] = ACTIONS(1413),
    [sym_device_id_string] = ACTIONS(1413),
    [sym_device_standby] = ACTIONS(1413),
    [sym_device_wake] = ACTIONS(1413),
    [sym_timeline_create] = ACTIONS(1413),
    [sym_timeline_kill] = ACTIONS(1413),
    [sym_timeline_reload] = ACTIONS(1413),
    [sym_timeline_pause] = ACTIONS(1413),
    [sym_timeline_restart] = ACTIONS(1413),
    [sym_timeline_set] = ACTIONS(1413),
    [sym_timeline_get] = ACTIONS(1413),
    [sym_timeline_active] = ACTIONS(1413),
    [sym_timeline_dynamic_id] = ACTIONS(1413),
    [sym_variable_to_string] = ACTIONS(1413),
    [sym_string_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_string] = ACTIONS(1413),
    [sym_type_cast] = ACTIONS(1413),
    [sym_variable_to_xml] = ACTIONS(1413),
    [sym_xml_to_variable] = ACTIONS(1413),
    [sym_length_variable_to_xml] = ACTIONS(1413),
    [sym_redirect_string] = ACTIONS(1413),
    [sym_random_number] = ACTIONS(1413),
    [sym_get_unique_id] = ACTIONS(1413),
    [sym_get_system_number] = ACTIONS(1413),
    [sym_set_system_number] = ACTIONS(1413),
    [sym_get_serial_number] = ACTIONS(1413),
    [sym_get_master_build] = ACTIONS(1413),
    [sym_set_validation_code] = ACTIONS(1413),
    [sym_reboot] = ACTIONS(1413),
    [sym_load_duet_module] = ACTIONS(1413),
    [sym_unload_duet_module] = ACTIONS(1413),
    [sym_duet_mem_size_set] = ACTIONS(1413),
    [sym_duet_mem_size_get] = ACTIONS(1413),
    [sym_dynamic_application_device] = ACTIONS(1413),
    [sym_dynamic_polled_port] = ACTIONS(1413),
    [sym_static_port_binding] = ACTIONS(1413),
    [sym_static_ip_binding] = ACTIONS(1413),
    [sym_get_dns_list] = ACTIONS(1413),
    [sym_set_dns_list] = ACTIONS(1413),
    [sym_get_ip_address] = ACTIONS(1413),
    [sym_set_ip_address] = ACTIONS(1413),
    [sym_get_url_list] = ACTIONS(1413),
    [sym_add_url_entry] = ACTIONS(1413),
    [sym_delete_url_entry] = ACTIONS(1413),
    [sym_abs_value] = ACTIONS(1413),
    [sym_max_value] = ACTIONS(1413),
    [sym_min_value] = ACTIONS(1413),
    [sym_exp_value] = ACTIONS(1413),
    [sym_log_value] = ACTIONS(1413),
    [sym_log10_value] = ACTIONS(1413),
    [sym_power_value] = ACTIONS(1413),
    [sym_sqrt_value] = ACTIONS(1413),
    [sym_raw_be] = ACTIONS(1413),
    [sym_raw_le] = ACTIONS(1413),
    [sym_internal_queue_size_set] = ACTIONS(1413),
    [sym_internal_queue_size_get] = ACTIONS(1413),
    [sym_internal_threshold_set] = ACTIONS(1413),
    [sym_internal_threshold_get] = ACTIONS(1413),
    [sym_rebuilt_event] = ACTIONS(1413),
    [sym_do_custom_event] = ACTIONS(1413),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1413),
    [sym_clkmgr_set_clk_source] = ACTIONS(1413),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1413),
    [sym_clkmgr_get_timezone] = ACTIONS(1413),
    [sym_clkmgr_set_timezone] = ACTIONS(1413),
    [sym_clkmgr_get_resync_period] = ACTIONS(1413),
    [sym_clkmgr_set_resync_period] = ACTIONS(1413),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1413),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_timeservers] = ACTIONS(1413),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1413),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1413),
    [sym_set_log_level] = ACTIONS(1413),
    [sym_get_log_level] = ACTIONS(1413),
    [sym_amx_log] = ACTIONS(1413),
    [sym_smtp_server_config_set] = ACTIONS(1413),
    [sym_smtp_server_config_get] = ACTIONS(1413),
    [sym_smtp_send] = ACTIONS(1413),
    [sym_validate_netlinx_account] = ACTIONS(1413),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1413),
    [sym_authenticate_certificate] = ACTIONS(1413),
    [sym_audit_netlinx_session_event] = ACTIONS(1413),
    [sym_audit_netlinx_generic_event] = ACTIONS(1413),
    [sym_get_available_flash_disk_space] = ACTIONS(1413),
    [sym_get_max_flash_disk_space] = ACTIONS(1413),
    [sym_data_create_feed] = ACTIONS(1413),
    [sym_data_delete_feed] = ACTIONS(1413),
    [sym_data_publish_feed] = ACTIONS(1413),
    [sym_data_get_published_feed] = ACTIONS(1413),
    [sym_data_add_record] = ACTIONS(1413),
    [sym_data_get_event_record] = ACTIONS(1413),
    [sym__wc_data_create_feed] = ACTIONS(1413),
    [sym__wc_data_add_record] = ACTIONS(1413),
    [sym__wc_data_get_event_record] = ACTIONS(1413),
    [sym_format] = ACTIONS(1413),
    [sym_astro_clock] = ACTIONS(1413),
    [sym_set_outdoor_temperature] = ACTIONS(1413),
    [sym_set_virtual_level_count] = ACTIONS(1413),
    [sym_set_virtual_channel_count] = ACTIONS(1413),
    [sym_set_virtual_port_count] = ACTIONS(1413),
    [sym__wc_encode_int] = ACTIONS(1413),
    [sym__wc_decode_int] = ACTIONS(1413),
    [sym__wc_int] = ACTIONS(1413),
    [sym__wc_to_ch_int] = ACTIONS(1413),
    [sym__ch_to_wc_int] = ACTIONS(1413),
    [sym__wc_find_string_int] = ACTIONS(1413),
    [sym__wc_left_string_int] = ACTIONS(1413),
    [sym__wc_lower_string_int] = ACTIONS(1413),
    [sym__wc_mid_string_int] = ACTIONS(1413),
    [sym__wc_remove_string_int] = ACTIONS(1413),
    [sym__wc_right_string_int] = ACTIONS(1413),
    [sym__wc_upper_string_int] = ACTIONS(1413),
    [sym__wc_compare_string_int] = ACTIONS(1413),
    [sym__wc_get_buffer_char_int] = ACTIONS(1413),
    [sym__wc_get_buffer_string_int] = ACTIONS(1413),
    [sym__wc_concat_string_int] = ACTIONS(1413),
    [sym__wc_file_open_int] = ACTIONS(1413),
    [sym__wc_file_close_int] = ACTIONS(1413),
    [sym__wc_file_read_int] = ACTIONS(1413),
    [sym__wc_file_read_line_int] = ACTIONS(1413),
    [sym__wc_file_write_int] = ACTIONS(1413),
    [sym__wc_file_write_line_int] = ACTIONS(1413),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1415),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1415),
    [aux_sym_preproc_else_token1] = ACTIONS(1415),
    [anon_sym_COMMA] = ACTIONS(1415),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(1415),
    [anon_sym_RBRACE] = ACTIONS(1415),
    [anon_sym_COLON] = ACTIONS(1415),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_STAR] = ACTIONS(1415),
    [anon_sym_SLASH] = ACTIONS(1413),
    [anon_sym_PERCENT] = ACTIONS(1415),
    [anon_sym_PIPE_PIPE] = ACTIONS(1415),
    [anon_sym_AMP_AMP] = ACTIONS(1415),
    [anon_sym_PIPE] = ACTIONS(1413),
    [anon_sym_CARET] = ACTIONS(1415),
    [anon_sym_AMP] = ACTIONS(1413),
    [anon_sym_EQ_EQ] = ACTIONS(1415),
    [anon_sym_BANG_EQ] = ACTIONS(1415),
    [anon_sym_LT_GT] = ACTIONS(1415),
    [anon_sym_GT] = ACTIONS(1413),
    [anon_sym_GT_EQ] = ACTIONS(1415),
    [anon_sym_LT_EQ] = ACTIONS(1415),
    [anon_sym_LT] = ACTIONS(1413),
    [anon_sym_LT_LT] = ACTIONS(1415),
    [anon_sym_GT_GT] = ACTIONS(1415),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(1415),
    [anon_sym_SQUOTE] = ACTIONS(1415),
    [sym_number_literal] = ACTIONS(1415),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(593)] = {
    [sym_identifier] = ACTIONS(1511),
    [sym_and] = ACTIONS(1511),
    [sym_or] = ACTIONS(1511),
    [sym_not] = ACTIONS(1511),
    [sym_xor] = ACTIONS(1511),
    [sym_band] = ACTIONS(1511),
    [sym_bor] = ACTIONS(1511),
    [sym_bxor] = ACTIONS(1511),
    [sym_bnot] = ACTIONS(1511),
    [sym_lshift] = ACTIONS(1511),
    [sym_rshift] = ACTIONS(1511),
    [sym___file__] = ACTIONS(1511),
    [sym___date__] = ACTIONS(1511),
    [sym___ldate__] = ACTIONS(1511),
    [sym___time__] = ACTIONS(1511),
    [sym___line__] = ACTIONS(1511),
    [sym___name__] = ACTIONS(1511),
    [sym___netlinx__] = ACTIONS(1511),
    [sym_netlinx_axi_version] = ACTIONS(1511),
    [sym_url_flg_tcp] = ACTIONS(1511),
    [sym_url_flg_accinfopresent] = ACTIONS(1511),
    [sym_url_flg_temp] = ACTIONS(1511),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1511),
    [sym_url_flg_stat_mask] = ACTIONS(1511),
    [sym_url_flg_stat_lookup] = ACTIONS(1511),
    [sym_url_flg_stat_connecting] = ACTIONS(1511),
    [sym_url_flg_stat_waiting] = ACTIONS(1511),
    [sym_url_flg_stat_connected] = ACTIONS(1511),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1511),
    [sym_first_virtual_device] = ACTIONS(1511),
    [sym_first_local_port] = ACTIONS(1511),
    [sym_dynamic_virtual_device] = ACTIONS(1511),
    [sym_timeline_once] = ACTIONS(1511),
    [sym_timeline_repeat] = ACTIONS(1511),
    [sym_timeline_absolute] = ACTIONS(1511),
    [sym_timeline_relative] = ACTIONS(1511),
    [sym_do_push_timed_infinite] = ACTIONS(1511),
    [sym_source_type_no_address] = ACTIONS(1511),
    [sym_source_type_neuron_id] = ACTIONS(1511),
    [sym_source_type_ip_address] = ACTIONS(1511),
    [sym_source_type_axlink] = ACTIONS(1511),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1511),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1511),
    [sym_source_type_ip_socket_address] = ACTIONS(1511),
    [sym_source_type_rs232] = ACTIONS(1511),
    [sym_source_type_internal] = ACTIONS(1511),
    [sym_source_type_mac_address] = ACTIONS(1511),
    [sym_source_type_ipv4_port] = ACTIONS(1511),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1511),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1511),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1511),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1511),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1511),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1511),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1511),
    [sym_true] = ACTIONS(1511),
    [sym_false] = ACTIONS(1511),
    [sym_file_read_only] = ACTIONS(1511),
    [sym_file_rw_new] = ACTIONS(1511),
    [sym_file_rw_append] = ACTIONS(1511),
    [sym_ip_tcp] = ACTIONS(1511),
    [sym_ip_udp] = ACTIONS(1511),
    [sym_ip_udp_2way] = ACTIONS(1511),
    [sym_xml_encode_types] = ACTIONS(1511),
    [sym_xml_encode_char_as_list] = ACTIONS(1511),
    [sym_xml_encode_le] = ACTIONS(1511),
    [sym_xml_decode_types] = ACTIONS(1511),
    [sym_xml_decode_no_preserve] = ACTIONS(1511),
    [sym_duet_dev_type_display_device] = ACTIONS(1511),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1511),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1511),
    [sym_duet_dev_type_io_device] = ACTIONS(1511),
    [sym_duet_dev_type_relay_device] = ACTIONS(1511),
    [sym_duet_dev_type_ups] = ACTIONS(1511),
    [sym_duet_dev_type_amplifier] = ACTIONS(1511),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1511),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1511),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1511),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1511),
    [sym_duet_dev_type_camera] = ACTIONS(1511),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1511),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1511),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1511),
    [sym_duet_dev_type_dss] = ACTIONS(1511),
    [sym_duet_dev_type_dvr] = ACTIONS(1511),
    [sym_duet_dev_type_disc_device] = ACTIONS(1511),
    [sym_duet_dev_type_document_camera] = ACTIONS(1511),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1511),
    [sym_duet_dev_type_hvac] = ACTIONS(1511),
    [sym_duet_dev_type_keypad] = ACTIONS(1511),
    [sym_duet_dev_type_light] = ACTIONS(1511),
    [sym_duet_dev_type_monitor] = ACTIONS(1511),
    [sym_duet_dev_type_motor] = ACTIONS(1511),
    [sym_duet_dev_type_multi_window] = ACTIONS(1511),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1511),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1511),
    [sym_duet_dev_type_receiver] = ACTIONS(1511),
    [sym_duet_dev_type_security_system] = ACTIONS(1511),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1511),
    [sym_duet_dev_type_settop_box] = ACTIONS(1511),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1511),
    [sym_duet_dev_type_switcher] = ACTIONS(1511),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1511),
    [sym_duet_dev_type_tv] = ACTIONS(1511),
    [sym_duet_dev_type_utility] = ACTIONS(1511),
    [sym_duet_dev_type_vcr] = ACTIONS(1511),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1511),
    [sym_duet_dev_type_video_processor] = ACTIONS(1511),
    [sym_duet_dev_type_video_projector] = ACTIONS(1511),
    [sym_duet_dev_type_video_wall] = ACTIONS(1511),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1511),
    [sym_duet_dev_type_weather] = ACTIONS(1511),
    [sym_duet_dev_not_polled] = ACTIONS(1511),
    [sym_duet_dev_polled] = ACTIONS(1511),
    [sym_clkmgr_mode_network] = ACTIONS(1511),
    [sym_clkmgr_mode_standalone] = ACTIONS(1511),
    [sym_normal_standby] = ACTIONS(1511),
    [sym_normal_wake] = ACTIONS(1511),
    [sym_ip_multicast_ttl_option] = ACTIONS(1511),
    [sym_ip_ttl_subnet] = ACTIONS(1511),
    [sym_ip_ttl_site] = ACTIONS(1511),
    [sym_ip_ttl_region] = ACTIONS(1511),
    [sym_ip_ttl_continent] = ACTIONS(1511),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1511),
    [sym_ip_nodelay_on] = ACTIONS(1511),
    [sym_ip_nodelay_off] = ACTIONS(1511),
    [sym_amx_error] = ACTIONS(1511),
    [sym_amx_warning] = ACTIONS(1511),
    [sym_amx_info] = ACTIONS(1511),
    [sym_amx_debug] = ACTIONS(1511),
    [sym_netlinx_login_success] = ACTIONS(1511),
    [sym_netlinx_login_fail] = ACTIONS(1511),
    [sym_netlinx_logout] = ACTIONS(1511),
    [sym_null_str] = ACTIONS(1511),
    [sym_smtp_address] = ACTIONS(1511),
    [sym_smtp_port_number] = ACTIONS(1511),
    [sym_smtp_username] = ACTIONS(1511),
    [sym_smtp_password] = ACTIONS(1511),
    [sym_smtp_require_tls] = ACTIONS(1511),
    [sym_smtp_from] = ACTIONS(1511),
    [sym_smtp_tls_true] = ACTIONS(1511),
    [sym_smtp_tls_false] = ACTIONS(1511),
    [sym_listview_on_row_select_event] = ACTIONS(1511),
    [sym_data_structure_unknown] = ACTIONS(1511),
    [sym_data_structure_datafeed] = ACTIONS(1511),
    [sym_data_structure_datarecordset] = ACTIONS(1511),
    [sym_data_structure_datarecord] = ACTIONS(1511),
    [sym_data_structure_datafield] = ACTIONS(1511),
    [sym_data_max_content_field_count] = ACTIONS(1511),
    [sym_data_max_metadata_field_count] = ACTIONS(1511),
    [sym_data_max_id_length] = ACTIONS(1511),
    [sym_data_max_name_length] = ACTIONS(1511),
    [sym_data_max_description_length] = ACTIONS(1511),
    [sym_data_max_type_length] = ACTIONS(1511),
    [sym_data_max_format_length] = ACTIONS(1511),
    [sym_data_max_source_length] = ACTIONS(1511),
    [sym_data_max_value_length] = ACTIONS(1511),
    [sym_data_max_label_length] = ACTIONS(1511),
    [sym_data_max_url_length] = ACTIONS(1511),
    [sym_max_last_login_info_length] = ACTIONS(1511),
    [sym_data_type_unknown] = ACTIONS(1511),
    [sym_data_type_string] = ACTIONS(1511),
    [sym_data_type_datetime] = ACTIONS(1511),
    [sym_data_type_date] = ACTIONS(1511),
    [sym_data_type_time] = ACTIONS(1511),
    [sym_data_type_image] = ACTIONS(1511),
    [sym_data_format_url] = ACTIONS(1511),
    [sym_data_format_phone] = ACTIONS(1511),
    [sym_data_format_email] = ACTIONS(1511),
    [sym_data_format_iso8601] = ACTIONS(1511),
    [sym_auditlog_priv] = ACTIONS(1511),
    [sym_configuration_priv] = ACTIONS(1511),
    [sym_deviceconfig_priv] = ACTIONS(1511),
    [sym_ftp_priv] = ACTIONS(1511),
    [sym_http_priv] = ACTIONS(1511),
    [sym_networkconfig_priv] = ACTIONS(1511),
    [sym_programport_priv] = ACTIONS(1511),
    [sym_remoteui_priv] = ACTIONS(1511),
    [sym_securitycontrol_priv] = ACTIONS(1511),
    [sym_softwaremanagement_priv] = ACTIONS(1511),
    [sym_terminal_priv] = ACTIONS(1511),
    [sym_tpadmin_priv] = ACTIONS(1511),
    [sym_user1_priv] = ACTIONS(1511),
    [sym_user2_priv] = ACTIONS(1511),
    [sym_user3_priv] = ACTIONS(1511),
    [sym_user4_priv] = ACTIONS(1511),
    [sym_usermanagement_priv] = ACTIONS(1511),
    [sym_valid_account] = ACTIONS(1511),
    [sym_err_name_invalid] = ACTIONS(1511),
    [sym_err_password_invalid] = ACTIONS(1511),
    [sym_err_invalid_account] = ACTIONS(1511),
    [sym_err_no_authorization] = ACTIONS(1511),
    [sym_err_invalid_parameter] = ACTIONS(1511),
    [sym_err_user_locked_out] = ACTIONS(1511),
    [sym_err_user_account_expired] = ACTIONS(1511),
    [sym_err_general_error] = ACTIONS(1511),
    [sym_err_authentication_error] = ACTIONS(1511),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1511),
    [sym_tls_validate_certificate] = ACTIONS(1511),
    [sym____reserved___] = ACTIONS(1511),
    [sym_date] = ACTIONS(1511),
    [sym_day] = ACTIONS(1511),
    [sym_time] = ACTIONS(1511),
    [sym_ldate] = ACTIONS(1511),
    [sym_push_device] = ACTIONS(1511),
    [sym_push_channel] = ACTIONS(1511),
    [sym_push_devchan] = ACTIONS(1511),
    [sym_release_device] = ACTIONS(1511),
    [sym_release_channel] = ACTIONS(1511),
    [sym_release_devchan] = ACTIONS(1511),
    [sym_master_slot] = ACTIONS(1511),
    [sym_get_pulse_time] = ACTIONS(1511),
    [sym_get_timer] = ACTIONS(1511),
    [sym_master_sn] = ACTIONS(1511),
    [sym_system_number] = ACTIONS(1511),
    [sym_button] = ACTIONS(1511),
    [sym_dv_channel] = ACTIONS(1511),
    [sym_data] = ACTIONS(1511),
    [sym_level] = ACTIONS(1511),
    [sym_timeline] = ACTIONS(1511),
    [sym_channel] = ACTIONS(1511),
    [sym_custom] = ACTIONS(1511),
    [sym_atoi] = ACTIONS(1511),
    [sym_atol] = ACTIONS(1511),
    [sym_atof] = ACTIONS(1511),
    [sym_itoa] = ACTIONS(1511),
    [sym_itohex] = ACTIONS(1511),
    [sym_hextoi] = ACTIONS(1511),
    [sym_ftoa] = ACTIONS(1511),
    [sym_find_string] = ACTIONS(1511),
    [sym_left_string] = ACTIONS(1511),
    [sym_right_string] = ACTIONS(1511),
    [sym_mid_string] = ACTIONS(1511),
    [sym_remove_string] = ACTIONS(1511),
    [sym_get_buffer_string] = ACTIONS(1511),
    [sym_get_buffer_char] = ACTIONS(1511),
    [sym_get_multi_buffer_string] = ACTIONS(1511),
    [sym_length_string] = ACTIONS(1511),
    [sym_max_length_string] = ACTIONS(1511),
    [sym_set_length_string] = ACTIONS(1511),
    [sym_compare_string] = ACTIONS(1511),
    [sym_upper_string] = ACTIONS(1511),
    [sym_lower_string] = ACTIONS(1511),
    [sym_get_last] = ACTIONS(1511),
    [sym_date_to_day] = ACTIONS(1511),
    [sym_date_to_month] = ACTIONS(1511),
    [sym_date_to_year] = ACTIONS(1511),
    [sym_time_to_hour] = ACTIONS(1511),
    [sym_time_to_minute] = ACTIONS(1511),
    [sym_time_to_second] = ACTIONS(1511),
    [sym_day_of_week] = ACTIONS(1511),
    [sym_set_pulse_time] = ACTIONS(1511),
    [sym_set_timer] = ACTIONS(1511),
    [sym_file_open] = ACTIONS(1511),
    [sym_file_close] = ACTIONS(1511),
    [sym_file_read] = ACTIONS(1511),
    [sym_file_read_line] = ACTIONS(1511),
    [sym_file_write] = ACTIONS(1511),
    [sym_file_write_line] = ACTIONS(1511),
    [sym_file_copy] = ACTIONS(1511),
    [sym_file_delete] = ACTIONS(1511),
    [sym_file_rename] = ACTIONS(1511),
    [sym_file_dir] = ACTIONS(1511),
    [sym_file_getdir] = ACTIONS(1511),
    [sym_file_setdir] = ACTIONS(1511),
    [sym_file_create_dir] = ACTIONS(1511),
    [sym_file_remove_dir] = ACTIONS(1511),
    [sym_file_seek] = ACTIONS(1511),
    [sym_length_array] = ACTIONS(1511),
    [sym_max_length_array] = ACTIONS(1511),
    [sym_set_length_array] = ACTIONS(1511),
    [sym_ip_client_open] = ACTIONS(1511),
    [sym_ip_client_close] = ACTIONS(1511),
    [sym_ip_server_open] = ACTIONS(1511),
    [sym_ip_server_close] = ACTIONS(1511),
    [sym_ip_bound_client_open] = ACTIONS(1511),
    [sym_ip_mc_server_open] = ACTIONS(1511),
    [sym_ip_set_option] = ACTIONS(1511),
    [sym_ssh_client_open] = ACTIONS(1511),
    [sym_ssh_client_close] = ACTIONS(1511),
    [sym_tls_client_open] = ACTIONS(1511),
    [sym_tls_client_close] = ACTIONS(1511),
    [sym_do_push] = ACTIONS(1511),
    [sym_do_push_timed] = ACTIONS(1511),
    [sym_do_release] = ACTIONS(1511),
    [sym_device_id] = ACTIONS(1511),
    [sym_device_info] = ACTIONS(1511),
    [sym_device_id_string] = ACTIONS(1511),
    [sym_device_standby] = ACTIONS(1511),
    [sym_device_wake] = ACTIONS(1511),
    [sym_timeline_create] = ACTIONS(1511),
    [sym_timeline_kill] = ACTIONS(1511),
    [sym_timeline_reload] = ACTIONS(1511),
    [sym_timeline_pause] = ACTIONS(1511),
    [sym_timeline_restart] = ACTIONS(1511),
    [sym_timeline_set] = ACTIONS(1511),
    [sym_timeline_get] = ACTIONS(1511),
    [sym_timeline_active] = ACTIONS(1511),
    [sym_timeline_dynamic_id] = ACTIONS(1511),
    [sym_variable_to_string] = ACTIONS(1511),
    [sym_string_to_variable] = ACTIONS(1511),
    [sym_length_variable_to_string] = ACTIONS(1511),
    [sym_type_cast] = ACTIONS(1511),
    [sym_variable_to_xml] = ACTIONS(1511),
    [sym_xml_to_variable] = ACTIONS(1511),
    [sym_length_variable_to_xml] = ACTIONS(1511),
    [sym_redirect_string] = ACTIONS(1511),
    [sym_random_number] = ACTIONS(1511),
    [sym_get_unique_id] = ACTIONS(1511),
    [sym_get_system_number] = ACTIONS(1511),
    [sym_set_system_number] = ACTIONS(1511),
    [sym_get_serial_number] = ACTIONS(1511),
    [sym_get_master_build] = ACTIONS(1511),
    [sym_set_validation_code] = ACTIONS(1511),
    [sym_reboot] = ACTIONS(1511),
    [sym_load_duet_module] = ACTIONS(1511),
    [sym_unload_duet_module] = ACTIONS(1511),
    [sym_duet_mem_size_set] = ACTIONS(1511),
    [sym_duet_mem_size_get] = ACTIONS(1511),
    [sym_dynamic_application_device] = ACTIONS(1511),
    [sym_dynamic_polled_port] = ACTIONS(1511),
    [sym_static_port_binding] = ACTIONS(1511),
    [sym_static_ip_binding] = ACTIONS(1511),
    [sym_get_dns_list] = ACTIONS(1511),
    [sym_set_dns_list] = ACTIONS(1511),
    [sym_get_ip_address] = ACTIONS(1511),
    [sym_set_ip_address] = ACTIONS(1511),
    [sym_get_url_list] = ACTIONS(1511),
    [sym_add_url_entry] = ACTIONS(1511),
    [sym_delete_url_entry] = ACTIONS(1511),
    [sym_abs_value] = ACTIONS(1511),
    [sym_max_value] = ACTIONS(1511),
    [sym_min_value] = ACTIONS(1511),
    [sym_exp_value] = ACTIONS(1511),
    [sym_log_value] = ACTIONS(1511),
    [sym_log10_value] = ACTIONS(1511),
    [sym_power_value] = ACTIONS(1511),
    [sym_sqrt_value] = ACTIONS(1511),
    [sym_raw_be] = ACTIONS(1511),
    [sym_raw_le] = ACTIONS(1511),
    [sym_internal_queue_size_set] = ACTIONS(1511),
    [sym_internal_queue_size_get] = ACTIONS(1511),
    [sym_internal_threshold_set] = ACTIONS(1511),
    [sym_internal_threshold_get] = ACTIONS(1511),
    [sym_rebuilt_event] = ACTIONS(1511),
    [sym_do_custom_event] = ACTIONS(1511),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1511),
    [sym_clkmgr_set_clk_source] = ACTIONS(1511),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1511),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1511),
    [sym_clkmgr_get_timezone] = ACTIONS(1511),
    [sym_clkmgr_set_timezone] = ACTIONS(1511),
    [sym_clkmgr_get_resync_period] = ACTIONS(1511),
    [sym_clkmgr_set_resync_period] = ACTIONS(1511),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1511),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1511),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1511),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1511),
    [sym_clkmgr_get_timeservers] = ACTIONS(1511),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1511),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1511),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1511),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1511),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1511),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1511),
    [sym_set_log_level] = ACTIONS(1511),
    [sym_get_log_level] = ACTIONS(1511),
    [sym_amx_log] = ACTIONS(1511),
    [sym_smtp_server_config_set] = ACTIONS(1511),
    [sym_smtp_server_config_get] = ACTIONS(1511),
    [sym_smtp_send] = ACTIONS(1511),
    [sym_validate_netlinx_account] = ACTIONS(1511),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1511),
    [sym_authenticate_certificate] = ACTIONS(1511),
    [sym_audit_netlinx_session_event] = ACTIONS(1511),
    [sym_audit_netlinx_generic_event] = ACTIONS(1511),
    [sym_get_available_flash_disk_space] = ACTIONS(1511),
    [sym_get_max_flash_disk_space] = ACTIONS(1511),
    [sym_data_create_feed] = ACTIONS(1511),
    [sym_data_delete_feed] = ACTIONS(1511),
    [sym_data_publish_feed] = ACTIONS(1511),
    [sym_data_get_published_feed] = ACTIONS(1511),
    [sym_data_add_record] = ACTIONS(1511),
    [sym_data_get_event_record] = ACTIONS(1511),
    [sym__wc_data_create_feed] = ACTIONS(1511),
    [sym__wc_data_add_record] = ACTIONS(1511),
    [sym__wc_data_get_event_record] = ACTIONS(1511),
    [sym_format] = ACTIONS(1511),
    [sym_astro_clock] = ACTIONS(1511),
    [sym_set_outdoor_temperature] = ACTIONS(1511),
    [sym_set_virtual_level_count] = ACTIONS(1511),
    [sym_set_virtual_channel_count] = ACTIONS(1511),
    [sym_set_virtual_port_count] = ACTIONS(1511),
    [sym__wc_encode_int] = ACTIONS(1511),
    [sym__wc_decode_int] = ACTIONS(1511),
    [sym__wc_int] = ACTIONS(1511),
    [sym__wc_to_ch_int] = ACTIONS(1511),
    [sym__ch_to_wc_int] = ACTIONS(1511),
    [sym__wc_find_string_int] = ACTIONS(1511),
    [sym__wc_left_string_int] = ACTIONS(1511),
    [sym__wc_lower_string_int] = ACTIONS(1511),
    [sym__wc_mid_string_int] = ACTIONS(1511),
    [sym__wc_remove_string_int] = ACTIONS(1511),
    [sym__wc_right_string_int] = ACTIONS(1511),
    [sym__wc_upper_string_int] = ACTIONS(1511),
    [sym__wc_compare_string_int] = ACTIONS(1511),
    [sym__wc_get_buffer_char_int] = ACTIONS(1511),
    [sym__wc_get_buffer_string_int] = ACTIONS(1511),
    [sym__wc_concat_string_int] = ACTIONS(1511),
    [sym__wc_file_open_int] = ACTIONS(1511),
    [sym__wc_file_close_int] = ACTIONS(1511),
    [sym__wc_file_read_int] = ACTIONS(1511),
    [sym__wc_file_read_line_int] = ACTIONS(1511),
    [sym__wc_file_write_int] = ACTIONS(1511),
    [sym__wc_file_write_line_int] = ACTIONS(1511),
    [anon_sym_EQ] = ACTIONS(1511),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1513),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1513),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1513),
    [aux_sym_preproc_else_token1] = ACTIONS(1513),
    [anon_sym_COMMA] = ACTIONS(1513),
    [anon_sym_LBRACK] = ACTIONS(1513),
    [anon_sym_LBRACE] = ACTIONS(1513),
    [anon_sym_RBRACE] = ACTIONS(1513),
    [anon_sym_COLON] = ACTIONS(1513),
    [anon_sym_LPAREN] = ACTIONS(1511),
    [anon_sym_BANG] = ACTIONS(1511),
    [anon_sym_TILDE] = ACTIONS(1513),
    [anon_sym_DASH] = ACTIONS(1511),
    [anon_sym_PLUS] = ACTIONS(1511),
    [anon_sym_STAR] = ACTIONS(1513),
    [anon_sym_SLASH] = ACTIONS(1511),
    [anon_sym_PERCENT] = ACTIONS(1513),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_AMP_AMP] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1511),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_AMP] = ACTIONS(1511),
    [anon_sym_EQ_EQ] = ACTIONS(1513),
    [anon_sym_BANG_EQ] = ACTIONS(1513),
    [anon_sym_LT_GT] = ACTIONS(1513),
    [anon_sym_GT] = ACTIONS(1511),
    [anon_sym_GT_EQ] = ACTIONS(1513),
    [anon_sym_LT_EQ] = ACTIONS(1513),
    [anon_sym_LT] = ACTIONS(1511),
    [anon_sym_LT_LT] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1513),
    [anon_sym_DASH_DASH] = ACTIONS(1513),
    [anon_sym_PLUS_PLUS] = ACTIONS(1513),
    [anon_sym_DOT] = ACTIONS(1511),
    [anon_sym_DQUOTE] = ACTIONS(1513),
    [anon_sym_SQUOTE] = ACTIONS(1513),
    [sym_number_literal] = ACTIONS(1513),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(594)] = {
    [sym_identifier] = ACTIONS(1539),
    [sym_and] = ACTIONS(1539),
    [sym_or] = ACTIONS(1539),
    [sym_not] = ACTIONS(1539),
    [sym_xor] = ACTIONS(1539),
    [sym_band] = ACTIONS(1539),
    [sym_bor] = ACTIONS(1539),
    [sym_bxor] = ACTIONS(1539),
    [sym_bnot] = ACTIONS(1539),
    [sym_lshift] = ACTIONS(1539),
    [sym_rshift] = ACTIONS(1539),
    [sym___file__] = ACTIONS(1539),
    [sym___date__] = ACTIONS(1539),
    [sym___ldate__] = ACTIONS(1539),
    [sym___time__] = ACTIONS(1539),
    [sym___line__] = ACTIONS(1539),
    [sym___name__] = ACTIONS(1539),
    [sym___netlinx__] = ACTIONS(1539),
    [sym_netlinx_axi_version] = ACTIONS(1539),
    [sym_url_flg_tcp] = ACTIONS(1539),
    [sym_url_flg_accinfopresent] = ACTIONS(1539),
    [sym_url_flg_temp] = ACTIONS(1539),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1539),
    [sym_url_flg_stat_mask] = ACTIONS(1539),
    [sym_url_flg_stat_lookup] = ACTIONS(1539),
    [sym_url_flg_stat_connecting] = ACTIONS(1539),
    [sym_url_flg_stat_waiting] = ACTIONS(1539),
    [sym_url_flg_stat_connected] = ACTIONS(1539),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1539),
    [sym_first_virtual_device] = ACTIONS(1539),
    [sym_first_local_port] = ACTIONS(1539),
    [sym_dynamic_virtual_device] = ACTIONS(1539),
    [sym_timeline_once] = ACTIONS(1539),
    [sym_timeline_repeat] = ACTIONS(1539),
    [sym_timeline_absolute] = ACTIONS(1539),
    [sym_timeline_relative] = ACTIONS(1539),
    [sym_do_push_timed_infinite] = ACTIONS(1539),
    [sym_source_type_no_address] = ACTIONS(1539),
    [sym_source_type_neuron_id] = ACTIONS(1539),
    [sym_source_type_ip_address] = ACTIONS(1539),
    [sym_source_type_axlink] = ACTIONS(1539),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1539),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1539),
    [sym_source_type_ip_socket_address] = ACTIONS(1539),
    [sym_source_type_rs232] = ACTIONS(1539),
    [sym_source_type_internal] = ACTIONS(1539),
    [sym_source_type_mac_address] = ACTIONS(1539),
    [sym_source_type_ipv4_port] = ACTIONS(1539),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1539),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1539),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1539),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1539),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1539),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1539),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1539),
    [sym_true] = ACTIONS(1539),
    [sym_false] = ACTIONS(1539),
    [sym_file_read_only] = ACTIONS(1539),
    [sym_file_rw_new] = ACTIONS(1539),
    [sym_file_rw_append] = ACTIONS(1539),
    [sym_ip_tcp] = ACTIONS(1539),
    [sym_ip_udp] = ACTIONS(1539),
    [sym_ip_udp_2way] = ACTIONS(1539),
    [sym_xml_encode_types] = ACTIONS(1539),
    [sym_xml_encode_char_as_list] = ACTIONS(1539),
    [sym_xml_encode_le] = ACTIONS(1539),
    [sym_xml_decode_types] = ACTIONS(1539),
    [sym_xml_decode_no_preserve] = ACTIONS(1539),
    [sym_duet_dev_type_display_device] = ACTIONS(1539),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1539),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1539),
    [sym_duet_dev_type_io_device] = ACTIONS(1539),
    [sym_duet_dev_type_relay_device] = ACTIONS(1539),
    [sym_duet_dev_type_ups] = ACTIONS(1539),
    [sym_duet_dev_type_amplifier] = ACTIONS(1539),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1539),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1539),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1539),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1539),
    [sym_duet_dev_type_camera] = ACTIONS(1539),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1539),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1539),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1539),
    [sym_duet_dev_type_dss] = ACTIONS(1539),
    [sym_duet_dev_type_dvr] = ACTIONS(1539),
    [sym_duet_dev_type_disc_device] = ACTIONS(1539),
    [sym_duet_dev_type_document_camera] = ACTIONS(1539),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1539),
    [sym_duet_dev_type_hvac] = ACTIONS(1539),
    [sym_duet_dev_type_keypad] = ACTIONS(1539),
    [sym_duet_dev_type_light] = ACTIONS(1539),
    [sym_duet_dev_type_monitor] = ACTIONS(1539),
    [sym_duet_dev_type_motor] = ACTIONS(1539),
    [sym_duet_dev_type_multi_window] = ACTIONS(1539),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1539),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1539),
    [sym_duet_dev_type_receiver] = ACTIONS(1539),
    [sym_duet_dev_type_security_system] = ACTIONS(1539),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1539),
    [sym_duet_dev_type_settop_box] = ACTIONS(1539),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1539),
    [sym_duet_dev_type_switcher] = ACTIONS(1539),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1539),
    [sym_duet_dev_type_tv] = ACTIONS(1539),
    [sym_duet_dev_type_utility] = ACTIONS(1539),
    [sym_duet_dev_type_vcr] = ACTIONS(1539),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1539),
    [sym_duet_dev_type_video_processor] = ACTIONS(1539),
    [sym_duet_dev_type_video_projector] = ACTIONS(1539),
    [sym_duet_dev_type_video_wall] = ACTIONS(1539),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1539),
    [sym_duet_dev_type_weather] = ACTIONS(1539),
    [sym_duet_dev_not_polled] = ACTIONS(1539),
    [sym_duet_dev_polled] = ACTIONS(1539),
    [sym_clkmgr_mode_network] = ACTIONS(1539),
    [sym_clkmgr_mode_standalone] = ACTIONS(1539),
    [sym_normal_standby] = ACTIONS(1539),
    [sym_normal_wake] = ACTIONS(1539),
    [sym_ip_multicast_ttl_option] = ACTIONS(1539),
    [sym_ip_ttl_subnet] = ACTIONS(1539),
    [sym_ip_ttl_site] = ACTIONS(1539),
    [sym_ip_ttl_region] = ACTIONS(1539),
    [sym_ip_ttl_continent] = ACTIONS(1539),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1539),
    [sym_ip_nodelay_on] = ACTIONS(1539),
    [sym_ip_nodelay_off] = ACTIONS(1539),
    [sym_amx_error] = ACTIONS(1539),
    [sym_amx_warning] = ACTIONS(1539),
    [sym_amx_info] = ACTIONS(1539),
    [sym_amx_debug] = ACTIONS(1539),
    [sym_netlinx_login_success] = ACTIONS(1539),
    [sym_netlinx_login_fail] = ACTIONS(1539),
    [sym_netlinx_logout] = ACTIONS(1539),
    [sym_null_str] = ACTIONS(1539),
    [sym_smtp_address] = ACTIONS(1539),
    [sym_smtp_port_number] = ACTIONS(1539),
    [sym_smtp_username] = ACTIONS(1539),
    [sym_smtp_password] = ACTIONS(1539),
    [sym_smtp_require_tls] = ACTIONS(1539),
    [sym_smtp_from] = ACTIONS(1539),
    [sym_smtp_tls_true] = ACTIONS(1539),
    [sym_smtp_tls_false] = ACTIONS(1539),
    [sym_listview_on_row_select_event] = ACTIONS(1539),
    [sym_data_structure_unknown] = ACTIONS(1539),
    [sym_data_structure_datafeed] = ACTIONS(1539),
    [sym_data_structure_datarecordset] = ACTIONS(1539),
    [sym_data_structure_datarecord] = ACTIONS(1539),
    [sym_data_structure_datafield] = ACTIONS(1539),
    [sym_data_max_content_field_count] = ACTIONS(1539),
    [sym_data_max_metadata_field_count] = ACTIONS(1539),
    [sym_data_max_id_length] = ACTIONS(1539),
    [sym_data_max_name_length] = ACTIONS(1539),
    [sym_data_max_description_length] = ACTIONS(1539),
    [sym_data_max_type_length] = ACTIONS(1539),
    [sym_data_max_format_length] = ACTIONS(1539),
    [sym_data_max_source_length] = ACTIONS(1539),
    [sym_data_max_value_length] = ACTIONS(1539),
    [sym_data_max_label_length] = ACTIONS(1539),
    [sym_data_max_url_length] = ACTIONS(1539),
    [sym_max_last_login_info_length] = ACTIONS(1539),
    [sym_data_type_unknown] = ACTIONS(1539),
    [sym_data_type_string] = ACTIONS(1539),
    [sym_data_type_datetime] = ACTIONS(1539),
    [sym_data_type_date] = ACTIONS(1539),
    [sym_data_type_time] = ACTIONS(1539),
    [sym_data_type_image] = ACTIONS(1539),
    [sym_data_format_url] = ACTIONS(1539),
    [sym_data_format_phone] = ACTIONS(1539),
    [sym_data_format_email] = ACTIONS(1539),
    [sym_data_format_iso8601] = ACTIONS(1539),
    [sym_auditlog_priv] = ACTIONS(1539),
    [sym_configuration_priv] = ACTIONS(1539),
    [sym_deviceconfig_priv] = ACTIONS(1539),
    [sym_ftp_priv] = ACTIONS(1539),
    [sym_http_priv] = ACTIONS(1539),
    [sym_networkconfig_priv] = ACTIONS(1539),
    [sym_programport_priv] = ACTIONS(1539),
    [sym_remoteui_priv] = ACTIONS(1539),
    [sym_securitycontrol_priv] = ACTIONS(1539),
    [sym_softwaremanagement_priv] = ACTIONS(1539),
    [sym_terminal_priv] = ACTIONS(1539),
    [sym_tpadmin_priv] = ACTIONS(1539),
    [sym_user1_priv] = ACTIONS(1539),
    [sym_user2_priv] = ACTIONS(1539),
    [sym_user3_priv] = ACTIONS(1539),
    [sym_user4_priv] = ACTIONS(1539),
    [sym_usermanagement_priv] = ACTIONS(1539),
    [sym_valid_account] = ACTIONS(1539),
    [sym_err_name_invalid] = ACTIONS(1539),
    [sym_err_password_invalid] = ACTIONS(1539),
    [sym_err_invalid_account] = ACTIONS(1539),
    [sym_err_no_authorization] = ACTIONS(1539),
    [sym_err_invalid_parameter] = ACTIONS(1539),
    [sym_err_user_locked_out] = ACTIONS(1539),
    [sym_err_user_account_expired] = ACTIONS(1539),
    [sym_err_general_error] = ACTIONS(1539),
    [sym_err_authentication_error] = ACTIONS(1539),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1539),
    [sym_tls_validate_certificate] = ACTIONS(1539),
    [sym____reserved___] = ACTIONS(1539),
    [sym_date] = ACTIONS(1539),
    [sym_day] = ACTIONS(1539),
    [sym_time] = ACTIONS(1539),
    [sym_ldate] = ACTIONS(1539),
    [sym_push_device] = ACTIONS(1539),
    [sym_push_channel] = ACTIONS(1539),
    [sym_push_devchan] = ACTIONS(1539),
    [sym_release_device] = ACTIONS(1539),
    [sym_release_channel] = ACTIONS(1539),
    [sym_release_devchan] = ACTIONS(1539),
    [sym_master_slot] = ACTIONS(1539),
    [sym_get_pulse_time] = ACTIONS(1539),
    [sym_get_timer] = ACTIONS(1539),
    [sym_master_sn] = ACTIONS(1539),
    [sym_system_number] = ACTIONS(1539),
    [sym_button] = ACTIONS(1539),
    [sym_dv_channel] = ACTIONS(1539),
    [sym_data] = ACTIONS(1539),
    [sym_level] = ACTIONS(1539),
    [sym_timeline] = ACTIONS(1539),
    [sym_channel] = ACTIONS(1539),
    [sym_custom] = ACTIONS(1539),
    [sym_atoi] = ACTIONS(1539),
    [sym_atol] = ACTIONS(1539),
    [sym_atof] = ACTIONS(1539),
    [sym_itoa] = ACTIONS(1539),
    [sym_itohex] = ACTIONS(1539),
    [sym_hextoi] = ACTIONS(1539),
    [sym_ftoa] = ACTIONS(1539),
    [sym_find_string] = ACTIONS(1539),
    [sym_left_string] = ACTIONS(1539),
    [sym_right_string] = ACTIONS(1539),
    [sym_mid_string] = ACTIONS(1539),
    [sym_remove_string] = ACTIONS(1539),
    [sym_get_buffer_string] = ACTIONS(1539),
    [sym_get_buffer_char] = ACTIONS(1539),
    [sym_get_multi_buffer_string] = ACTIONS(1539),
    [sym_length_string] = ACTIONS(1539),
    [sym_max_length_string] = ACTIONS(1539),
    [sym_set_length_string] = ACTIONS(1539),
    [sym_compare_string] = ACTIONS(1539),
    [sym_upper_string] = ACTIONS(1539),
    [sym_lower_string] = ACTIONS(1539),
    [sym_get_last] = ACTIONS(1539),
    [sym_date_to_day] = ACTIONS(1539),
    [sym_date_to_month] = ACTIONS(1539),
    [sym_date_to_year] = ACTIONS(1539),
    [sym_time_to_hour] = ACTIONS(1539),
    [sym_time_to_minute] = ACTIONS(1539),
    [sym_time_to_second] = ACTIONS(1539),
    [sym_day_of_week] = ACTIONS(1539),
    [sym_set_pulse_time] = ACTIONS(1539),
    [sym_set_timer] = ACTIONS(1539),
    [sym_file_open] = ACTIONS(1539),
    [sym_file_close] = ACTIONS(1539),
    [sym_file_read] = ACTIONS(1539),
    [sym_file_read_line] = ACTIONS(1539),
    [sym_file_write] = ACTIONS(1539),
    [sym_file_write_line] = ACTIONS(1539),
    [sym_file_copy] = ACTIONS(1539),
    [sym_file_delete] = ACTIONS(1539),
    [sym_file_rename] = ACTIONS(1539),
    [sym_file_dir] = ACTIONS(1539),
    [sym_file_getdir] = ACTIONS(1539),
    [sym_file_setdir] = ACTIONS(1539),
    [sym_file_create_dir] = ACTIONS(1539),
    [sym_file_remove_dir] = ACTIONS(1539),
    [sym_file_seek] = ACTIONS(1539),
    [sym_length_array] = ACTIONS(1539),
    [sym_max_length_array] = ACTIONS(1539),
    [sym_set_length_array] = ACTIONS(1539),
    [sym_ip_client_open] = ACTIONS(1539),
    [sym_ip_client_close] = ACTIONS(1539),
    [sym_ip_server_open] = ACTIONS(1539),
    [sym_ip_server_close] = ACTIONS(1539),
    [sym_ip_bound_client_open] = ACTIONS(1539),
    [sym_ip_mc_server_open] = ACTIONS(1539),
    [sym_ip_set_option] = ACTIONS(1539),
    [sym_ssh_client_open] = ACTIONS(1539),
    [sym_ssh_client_close] = ACTIONS(1539),
    [sym_tls_client_open] = ACTIONS(1539),
    [sym_tls_client_close] = ACTIONS(1539),
    [sym_do_push] = ACTIONS(1539),
    [sym_do_push_timed] = ACTIONS(1539),
    [sym_do_release] = ACTIONS(1539),
    [sym_device_id] = ACTIONS(1539),
    [sym_device_info] = ACTIONS(1539),
    [sym_device_id_string] = ACTIONS(1539),
    [sym_device_standby] = ACTIONS(1539),
    [sym_device_wake] = ACTIONS(1539),
    [sym_timeline_create] = ACTIONS(1539),
    [sym_timeline_kill] = ACTIONS(1539),
    [sym_timeline_reload] = ACTIONS(1539),
    [sym_timeline_pause] = ACTIONS(1539),
    [sym_timeline_restart] = ACTIONS(1539),
    [sym_timeline_set] = ACTIONS(1539),
    [sym_timeline_get] = ACTIONS(1539),
    [sym_timeline_active] = ACTIONS(1539),
    [sym_timeline_dynamic_id] = ACTIONS(1539),
    [sym_variable_to_string] = ACTIONS(1539),
    [sym_string_to_variable] = ACTIONS(1539),
    [sym_length_variable_to_string] = ACTIONS(1539),
    [sym_type_cast] = ACTIONS(1539),
    [sym_variable_to_xml] = ACTIONS(1539),
    [sym_xml_to_variable] = ACTIONS(1539),
    [sym_length_variable_to_xml] = ACTIONS(1539),
    [sym_redirect_string] = ACTIONS(1539),
    [sym_random_number] = ACTIONS(1539),
    [sym_get_unique_id] = ACTIONS(1539),
    [sym_get_system_number] = ACTIONS(1539),
    [sym_set_system_number] = ACTIONS(1539),
    [sym_get_serial_number] = ACTIONS(1539),
    [sym_get_master_build] = ACTIONS(1539),
    [sym_set_validation_code] = ACTIONS(1539),
    [sym_reboot] = ACTIONS(1539),
    [sym_load_duet_module] = ACTIONS(1539),
    [sym_unload_duet_module] = ACTIONS(1539),
    [sym_duet_mem_size_set] = ACTIONS(1539),
    [sym_duet_mem_size_get] = ACTIONS(1539),
    [sym_dynamic_application_device] = ACTIONS(1539),
    [sym_dynamic_polled_port] = ACTIONS(1539),
    [sym_static_port_binding] = ACTIONS(1539),
    [sym_static_ip_binding] = ACTIONS(1539),
    [sym_get_dns_list] = ACTIONS(1539),
    [sym_set_dns_list] = ACTIONS(1539),
    [sym_get_ip_address] = ACTIONS(1539),
    [sym_set_ip_address] = ACTIONS(1539),
    [sym_get_url_list] = ACTIONS(1539),
    [sym_add_url_entry] = ACTIONS(1539),
    [sym_delete_url_entry] = ACTIONS(1539),
    [sym_abs_value] = ACTIONS(1539),
    [sym_max_value] = ACTIONS(1539),
    [sym_min_value] = ACTIONS(1539),
    [sym_exp_value] = ACTIONS(1539),
    [sym_log_value] = ACTIONS(1539),
    [sym_log10_value] = ACTIONS(1539),
    [sym_power_value] = ACTIONS(1539),
    [sym_sqrt_value] = ACTIONS(1539),
    [sym_raw_be] = ACTIONS(1539),
    [sym_raw_le] = ACTIONS(1539),
    [sym_internal_queue_size_set] = ACTIONS(1539),
    [sym_internal_queue_size_get] = ACTIONS(1539),
    [sym_internal_threshold_set] = ACTIONS(1539),
    [sym_internal_threshold_get] = ACTIONS(1539),
    [sym_rebuilt_event] = ACTIONS(1539),
    [sym_do_custom_event] = ACTIONS(1539),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1539),
    [sym_clkmgr_set_clk_source] = ACTIONS(1539),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1539),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1539),
    [sym_clkmgr_get_timezone] = ACTIONS(1539),
    [sym_clkmgr_set_timezone] = ACTIONS(1539),
    [sym_clkmgr_get_resync_period] = ACTIONS(1539),
    [sym_clkmgr_set_resync_period] = ACTIONS(1539),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1539),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1539),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1539),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1539),
    [sym_clkmgr_get_timeservers] = ACTIONS(1539),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1539),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1539),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1539),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1539),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1539),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1539),
    [sym_set_log_level] = ACTIONS(1539),
    [sym_get_log_level] = ACTIONS(1539),
    [sym_amx_log] = ACTIONS(1539),
    [sym_smtp_server_config_set] = ACTIONS(1539),
    [sym_smtp_server_config_get] = ACTIONS(1539),
    [sym_smtp_send] = ACTIONS(1539),
    [sym_validate_netlinx_account] = ACTIONS(1539),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1539),
    [sym_authenticate_certificate] = ACTIONS(1539),
    [sym_audit_netlinx_session_event] = ACTIONS(1539),
    [sym_audit_netlinx_generic_event] = ACTIONS(1539),
    [sym_get_available_flash_disk_space] = ACTIONS(1539),
    [sym_get_max_flash_disk_space] = ACTIONS(1539),
    [sym_data_create_feed] = ACTIONS(1539),
    [sym_data_delete_feed] = ACTIONS(1539),
    [sym_data_publish_feed] = ACTIONS(1539),
    [sym_data_get_published_feed] = ACTIONS(1539),
    [sym_data_add_record] = ACTIONS(1539),
    [sym_data_get_event_record] = ACTIONS(1539),
    [sym__wc_data_create_feed] = ACTIONS(1539),
    [sym__wc_data_add_record] = ACTIONS(1539),
    [sym__wc_data_get_event_record] = ACTIONS(1539),
    [sym_format] = ACTIONS(1539),
    [sym_astro_clock] = ACTIONS(1539),
    [sym_set_outdoor_temperature] = ACTIONS(1539),
    [sym_set_virtual_level_count] = ACTIONS(1539),
    [sym_set_virtual_channel_count] = ACTIONS(1539),
    [sym_set_virtual_port_count] = ACTIONS(1539),
    [sym__wc_encode_int] = ACTIONS(1539),
    [sym__wc_decode_int] = ACTIONS(1539),
    [sym__wc_int] = ACTIONS(1539),
    [sym__wc_to_ch_int] = ACTIONS(1539),
    [sym__ch_to_wc_int] = ACTIONS(1539),
    [sym__wc_find_string_int] = ACTIONS(1539),
    [sym__wc_left_string_int] = ACTIONS(1539),
    [sym__wc_lower_string_int] = ACTIONS(1539),
    [sym__wc_mid_string_int] = ACTIONS(1539),
    [sym__wc_remove_string_int] = ACTIONS(1539),
    [sym__wc_right_string_int] = ACTIONS(1539),
    [sym__wc_upper_string_int] = ACTIONS(1539),
    [sym__wc_compare_string_int] = ACTIONS(1539),
    [sym__wc_get_buffer_char_int] = ACTIONS(1539),
    [sym__wc_get_buffer_string_int] = ACTIONS(1539),
    [sym__wc_concat_string_int] = ACTIONS(1539),
    [sym__wc_file_open_int] = ACTIONS(1539),
    [sym__wc_file_close_int] = ACTIONS(1539),
    [sym__wc_file_read_int] = ACTIONS(1539),
    [sym__wc_file_read_line_int] = ACTIONS(1539),
    [sym__wc_file_write_int] = ACTIONS(1539),
    [sym__wc_file_write_line_int] = ACTIONS(1539),
    [anon_sym_EQ] = ACTIONS(1539),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1541),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1541),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1541),
    [aux_sym_preproc_else_token1] = ACTIONS(1541),
    [anon_sym_COMMA] = ACTIONS(1541),
    [anon_sym_LBRACK] = ACTIONS(1541),
    [anon_sym_LBRACE] = ACTIONS(1541),
    [anon_sym_RBRACE] = ACTIONS(1541),
    [anon_sym_COLON] = ACTIONS(1541),
    [anon_sym_LPAREN] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1539),
    [anon_sym_TILDE] = ACTIONS(1541),
    [anon_sym_DASH] = ACTIONS(1539),
    [anon_sym_PLUS] = ACTIONS(1539),
    [anon_sym_STAR] = ACTIONS(1541),
    [anon_sym_SLASH] = ACTIONS(1539),
    [anon_sym_PERCENT] = ACTIONS(1541),
    [anon_sym_PIPE_PIPE] = ACTIONS(1541),
    [anon_sym_AMP_AMP] = ACTIONS(1541),
    [anon_sym_PIPE] = ACTIONS(1539),
    [anon_sym_CARET] = ACTIONS(1541),
    [anon_sym_AMP] = ACTIONS(1539),
    [anon_sym_EQ_EQ] = ACTIONS(1541),
    [anon_sym_BANG_EQ] = ACTIONS(1541),
    [anon_sym_LT_GT] = ACTIONS(1541),
    [anon_sym_GT] = ACTIONS(1539),
    [anon_sym_GT_EQ] = ACTIONS(1541),
    [anon_sym_LT_EQ] = ACTIONS(1541),
    [anon_sym_LT] = ACTIONS(1539),
    [anon_sym_LT_LT] = ACTIONS(1541),
    [anon_sym_GT_GT] = ACTIONS(1541),
    [anon_sym_DASH_DASH] = ACTIONS(1541),
    [anon_sym_PLUS_PLUS] = ACTIONS(1541),
    [anon_sym_DOT] = ACTIONS(1539),
    [anon_sym_DQUOTE] = ACTIONS(1541),
    [anon_sym_SQUOTE] = ACTIONS(1541),
    [sym_number_literal] = ACTIONS(1541),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(595)] = {
    [sym_identifier] = ACTIONS(1521),
    [sym_and] = ACTIONS(1521),
    [sym_or] = ACTIONS(1521),
    [sym_not] = ACTIONS(1521),
    [sym_xor] = ACTIONS(1521),
    [sym_band] = ACTIONS(1521),
    [sym_bor] = ACTIONS(1521),
    [sym_bxor] = ACTIONS(1521),
    [sym_bnot] = ACTIONS(1521),
    [sym_lshift] = ACTIONS(1521),
    [sym_rshift] = ACTIONS(1521),
    [sym___file__] = ACTIONS(1521),
    [sym___date__] = ACTIONS(1521),
    [sym___ldate__] = ACTIONS(1521),
    [sym___time__] = ACTIONS(1521),
    [sym___line__] = ACTIONS(1521),
    [sym___name__] = ACTIONS(1521),
    [sym___netlinx__] = ACTIONS(1521),
    [sym_netlinx_axi_version] = ACTIONS(1521),
    [sym_url_flg_tcp] = ACTIONS(1521),
    [sym_url_flg_accinfopresent] = ACTIONS(1521),
    [sym_url_flg_temp] = ACTIONS(1521),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1521),
    [sym_url_flg_stat_mask] = ACTIONS(1521),
    [sym_url_flg_stat_lookup] = ACTIONS(1521),
    [sym_url_flg_stat_connecting] = ACTIONS(1521),
    [sym_url_flg_stat_waiting] = ACTIONS(1521),
    [sym_url_flg_stat_connected] = ACTIONS(1521),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1521),
    [sym_first_virtual_device] = ACTIONS(1521),
    [sym_first_local_port] = ACTIONS(1521),
    [sym_dynamic_virtual_device] = ACTIONS(1521),
    [sym_timeline_once] = ACTIONS(1521),
    [sym_timeline_repeat] = ACTIONS(1521),
    [sym_timeline_absolute] = ACTIONS(1521),
    [sym_timeline_relative] = ACTIONS(1521),
    [sym_do_push_timed_infinite] = ACTIONS(1521),
    [sym_source_type_no_address] = ACTIONS(1521),
    [sym_source_type_neuron_id] = ACTIONS(1521),
    [sym_source_type_ip_address] = ACTIONS(1521),
    [sym_source_type_axlink] = ACTIONS(1521),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1521),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1521),
    [sym_source_type_ip_socket_address] = ACTIONS(1521),
    [sym_source_type_rs232] = ACTIONS(1521),
    [sym_source_type_internal] = ACTIONS(1521),
    [sym_source_type_mac_address] = ACTIONS(1521),
    [sym_source_type_ipv4_port] = ACTIONS(1521),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1521),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1521),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1521),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1521),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1521),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1521),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1521),
    [sym_true] = ACTIONS(1521),
    [sym_false] = ACTIONS(1521),
    [sym_file_read_only] = ACTIONS(1521),
    [sym_file_rw_new] = ACTIONS(1521),
    [sym_file_rw_append] = ACTIONS(1521),
    [sym_ip_tcp] = ACTIONS(1521),
    [sym_ip_udp] = ACTIONS(1521),
    [sym_ip_udp_2way] = ACTIONS(1521),
    [sym_xml_encode_types] = ACTIONS(1521),
    [sym_xml_encode_char_as_list] = ACTIONS(1521),
    [sym_xml_encode_le] = ACTIONS(1521),
    [sym_xml_decode_types] = ACTIONS(1521),
    [sym_xml_decode_no_preserve] = ACTIONS(1521),
    [sym_duet_dev_type_display_device] = ACTIONS(1521),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1521),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1521),
    [sym_duet_dev_type_io_device] = ACTIONS(1521),
    [sym_duet_dev_type_relay_device] = ACTIONS(1521),
    [sym_duet_dev_type_ups] = ACTIONS(1521),
    [sym_duet_dev_type_amplifier] = ACTIONS(1521),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1521),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1521),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1521),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1521),
    [sym_duet_dev_type_camera] = ACTIONS(1521),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1521),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1521),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1521),
    [sym_duet_dev_type_dss] = ACTIONS(1521),
    [sym_duet_dev_type_dvr] = ACTIONS(1521),
    [sym_duet_dev_type_disc_device] = ACTIONS(1521),
    [sym_duet_dev_type_document_camera] = ACTIONS(1521),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1521),
    [sym_duet_dev_type_hvac] = ACTIONS(1521),
    [sym_duet_dev_type_keypad] = ACTIONS(1521),
    [sym_duet_dev_type_light] = ACTIONS(1521),
    [sym_duet_dev_type_monitor] = ACTIONS(1521),
    [sym_duet_dev_type_motor] = ACTIONS(1521),
    [sym_duet_dev_type_multi_window] = ACTIONS(1521),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1521),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1521),
    [sym_duet_dev_type_receiver] = ACTIONS(1521),
    [sym_duet_dev_type_security_system] = ACTIONS(1521),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1521),
    [sym_duet_dev_type_settop_box] = ACTIONS(1521),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1521),
    [sym_duet_dev_type_switcher] = ACTIONS(1521),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1521),
    [sym_duet_dev_type_tv] = ACTIONS(1521),
    [sym_duet_dev_type_utility] = ACTIONS(1521),
    [sym_duet_dev_type_vcr] = ACTIONS(1521),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1521),
    [sym_duet_dev_type_video_processor] = ACTIONS(1521),
    [sym_duet_dev_type_video_projector] = ACTIONS(1521),
    [sym_duet_dev_type_video_wall] = ACTIONS(1521),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1521),
    [sym_duet_dev_type_weather] = ACTIONS(1521),
    [sym_duet_dev_not_polled] = ACTIONS(1521),
    [sym_duet_dev_polled] = ACTIONS(1521),
    [sym_clkmgr_mode_network] = ACTIONS(1521),
    [sym_clkmgr_mode_standalone] = ACTIONS(1521),
    [sym_normal_standby] = ACTIONS(1521),
    [sym_normal_wake] = ACTIONS(1521),
    [sym_ip_multicast_ttl_option] = ACTIONS(1521),
    [sym_ip_ttl_subnet] = ACTIONS(1521),
    [sym_ip_ttl_site] = ACTIONS(1521),
    [sym_ip_ttl_region] = ACTIONS(1521),
    [sym_ip_ttl_continent] = ACTIONS(1521),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1521),
    [sym_ip_nodelay_on] = ACTIONS(1521),
    [sym_ip_nodelay_off] = ACTIONS(1521),
    [sym_amx_error] = ACTIONS(1521),
    [sym_amx_warning] = ACTIONS(1521),
    [sym_amx_info] = ACTIONS(1521),
    [sym_amx_debug] = ACTIONS(1521),
    [sym_netlinx_login_success] = ACTIONS(1521),
    [sym_netlinx_login_fail] = ACTIONS(1521),
    [sym_netlinx_logout] = ACTIONS(1521),
    [sym_null_str] = ACTIONS(1521),
    [sym_smtp_address] = ACTIONS(1521),
    [sym_smtp_port_number] = ACTIONS(1521),
    [sym_smtp_username] = ACTIONS(1521),
    [sym_smtp_password] = ACTIONS(1521),
    [sym_smtp_require_tls] = ACTIONS(1521),
    [sym_smtp_from] = ACTIONS(1521),
    [sym_smtp_tls_true] = ACTIONS(1521),
    [sym_smtp_tls_false] = ACTIONS(1521),
    [sym_listview_on_row_select_event] = ACTIONS(1521),
    [sym_data_structure_unknown] = ACTIONS(1521),
    [sym_data_structure_datafeed] = ACTIONS(1521),
    [sym_data_structure_datarecordset] = ACTIONS(1521),
    [sym_data_structure_datarecord] = ACTIONS(1521),
    [sym_data_structure_datafield] = ACTIONS(1521),
    [sym_data_max_content_field_count] = ACTIONS(1521),
    [sym_data_max_metadata_field_count] = ACTIONS(1521),
    [sym_data_max_id_length] = ACTIONS(1521),
    [sym_data_max_name_length] = ACTIONS(1521),
    [sym_data_max_description_length] = ACTIONS(1521),
    [sym_data_max_type_length] = ACTIONS(1521),
    [sym_data_max_format_length] = ACTIONS(1521),
    [sym_data_max_source_length] = ACTIONS(1521),
    [sym_data_max_value_length] = ACTIONS(1521),
    [sym_data_max_label_length] = ACTIONS(1521),
    [sym_data_max_url_length] = ACTIONS(1521),
    [sym_max_last_login_info_length] = ACTIONS(1521),
    [sym_data_type_unknown] = ACTIONS(1521),
    [sym_data_type_string] = ACTIONS(1521),
    [sym_data_type_datetime] = ACTIONS(1521),
    [sym_data_type_date] = ACTIONS(1521),
    [sym_data_type_time] = ACTIONS(1521),
    [sym_data_type_image] = ACTIONS(1521),
    [sym_data_format_url] = ACTIONS(1521),
    [sym_data_format_phone] = ACTIONS(1521),
    [sym_data_format_email] = ACTIONS(1521),
    [sym_data_format_iso8601] = ACTIONS(1521),
    [sym_auditlog_priv] = ACTIONS(1521),
    [sym_configuration_priv] = ACTIONS(1521),
    [sym_deviceconfig_priv] = ACTIONS(1521),
    [sym_ftp_priv] = ACTIONS(1521),
    [sym_http_priv] = ACTIONS(1521),
    [sym_networkconfig_priv] = ACTIONS(1521),
    [sym_programport_priv] = ACTIONS(1521),
    [sym_remoteui_priv] = ACTIONS(1521),
    [sym_securitycontrol_priv] = ACTIONS(1521),
    [sym_softwaremanagement_priv] = ACTIONS(1521),
    [sym_terminal_priv] = ACTIONS(1521),
    [sym_tpadmin_priv] = ACTIONS(1521),
    [sym_user1_priv] = ACTIONS(1521),
    [sym_user2_priv] = ACTIONS(1521),
    [sym_user3_priv] = ACTIONS(1521),
    [sym_user4_priv] = ACTIONS(1521),
    [sym_usermanagement_priv] = ACTIONS(1521),
    [sym_valid_account] = ACTIONS(1521),
    [sym_err_name_invalid] = ACTIONS(1521),
    [sym_err_password_invalid] = ACTIONS(1521),
    [sym_err_invalid_account] = ACTIONS(1521),
    [sym_err_no_authorization] = ACTIONS(1521),
    [sym_err_invalid_parameter] = ACTIONS(1521),
    [sym_err_user_locked_out] = ACTIONS(1521),
    [sym_err_user_account_expired] = ACTIONS(1521),
    [sym_err_general_error] = ACTIONS(1521),
    [sym_err_authentication_error] = ACTIONS(1521),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1521),
    [sym_tls_validate_certificate] = ACTIONS(1521),
    [sym____reserved___] = ACTIONS(1521),
    [sym_date] = ACTIONS(1521),
    [sym_day] = ACTIONS(1521),
    [sym_time] = ACTIONS(1521),
    [sym_ldate] = ACTIONS(1521),
    [sym_push_device] = ACTIONS(1521),
    [sym_push_channel] = ACTIONS(1521),
    [sym_push_devchan] = ACTIONS(1521),
    [sym_release_device] = ACTIONS(1521),
    [sym_release_channel] = ACTIONS(1521),
    [sym_release_devchan] = ACTIONS(1521),
    [sym_master_slot] = ACTIONS(1521),
    [sym_get_pulse_time] = ACTIONS(1521),
    [sym_get_timer] = ACTIONS(1521),
    [sym_master_sn] = ACTIONS(1521),
    [sym_system_number] = ACTIONS(1521),
    [sym_button] = ACTIONS(1521),
    [sym_dv_channel] = ACTIONS(1521),
    [sym_data] = ACTIONS(1521),
    [sym_level] = ACTIONS(1521),
    [sym_timeline] = ACTIONS(1521),
    [sym_channel] = ACTIONS(1521),
    [sym_custom] = ACTIONS(1521),
    [sym_atoi] = ACTIONS(1521),
    [sym_atol] = ACTIONS(1521),
    [sym_atof] = ACTIONS(1521),
    [sym_itoa] = ACTIONS(1521),
    [sym_itohex] = ACTIONS(1521),
    [sym_hextoi] = ACTIONS(1521),
    [sym_ftoa] = ACTIONS(1521),
    [sym_find_string] = ACTIONS(1521),
    [sym_left_string] = ACTIONS(1521),
    [sym_right_string] = ACTIONS(1521),
    [sym_mid_string] = ACTIONS(1521),
    [sym_remove_string] = ACTIONS(1521),
    [sym_get_buffer_string] = ACTIONS(1521),
    [sym_get_buffer_char] = ACTIONS(1521),
    [sym_get_multi_buffer_string] = ACTIONS(1521),
    [sym_length_string] = ACTIONS(1521),
    [sym_max_length_string] = ACTIONS(1521),
    [sym_set_length_string] = ACTIONS(1521),
    [sym_compare_string] = ACTIONS(1521),
    [sym_upper_string] = ACTIONS(1521),
    [sym_lower_string] = ACTIONS(1521),
    [sym_get_last] = ACTIONS(1521),
    [sym_date_to_day] = ACTIONS(1521),
    [sym_date_to_month] = ACTIONS(1521),
    [sym_date_to_year] = ACTIONS(1521),
    [sym_time_to_hour] = ACTIONS(1521),
    [sym_time_to_minute] = ACTIONS(1521),
    [sym_time_to_second] = ACTIONS(1521),
    [sym_day_of_week] = ACTIONS(1521),
    [sym_set_pulse_time] = ACTIONS(1521),
    [sym_set_timer] = ACTIONS(1521),
    [sym_file_open] = ACTIONS(1521),
    [sym_file_close] = ACTIONS(1521),
    [sym_file_read] = ACTIONS(1521),
    [sym_file_read_line] = ACTIONS(1521),
    [sym_file_write] = ACTIONS(1521),
    [sym_file_write_line] = ACTIONS(1521),
    [sym_file_copy] = ACTIONS(1521),
    [sym_file_delete] = ACTIONS(1521),
    [sym_file_rename] = ACTIONS(1521),
    [sym_file_dir] = ACTIONS(1521),
    [sym_file_getdir] = ACTIONS(1521),
    [sym_file_setdir] = ACTIONS(1521),
    [sym_file_create_dir] = ACTIONS(1521),
    [sym_file_remove_dir] = ACTIONS(1521),
    [sym_file_seek] = ACTIONS(1521),
    [sym_length_array] = ACTIONS(1521),
    [sym_max_length_array] = ACTIONS(1521),
    [sym_set_length_array] = ACTIONS(1521),
    [sym_ip_client_open] = ACTIONS(1521),
    [sym_ip_client_close] = ACTIONS(1521),
    [sym_ip_server_open] = ACTIONS(1521),
    [sym_ip_server_close] = ACTIONS(1521),
    [sym_ip_bound_client_open] = ACTIONS(1521),
    [sym_ip_mc_server_open] = ACTIONS(1521),
    [sym_ip_set_option] = ACTIONS(1521),
    [sym_ssh_client_open] = ACTIONS(1521),
    [sym_ssh_client_close] = ACTIONS(1521),
    [sym_tls_client_open] = ACTIONS(1521),
    [sym_tls_client_close] = ACTIONS(1521),
    [sym_do_push] = ACTIONS(1521),
    [sym_do_push_timed] = ACTIONS(1521),
    [sym_do_release] = ACTIONS(1521),
    [sym_device_id] = ACTIONS(1521),
    [sym_device_info] = ACTIONS(1521),
    [sym_device_id_string] = ACTIONS(1521),
    [sym_device_standby] = ACTIONS(1521),
    [sym_device_wake] = ACTIONS(1521),
    [sym_timeline_create] = ACTIONS(1521),
    [sym_timeline_kill] = ACTIONS(1521),
    [sym_timeline_reload] = ACTIONS(1521),
    [sym_timeline_pause] = ACTIONS(1521),
    [sym_timeline_restart] = ACTIONS(1521),
    [sym_timeline_set] = ACTIONS(1521),
    [sym_timeline_get] = ACTIONS(1521),
    [sym_timeline_active] = ACTIONS(1521),
    [sym_timeline_dynamic_id] = ACTIONS(1521),
    [sym_variable_to_string] = ACTIONS(1521),
    [sym_string_to_variable] = ACTIONS(1521),
    [sym_length_variable_to_string] = ACTIONS(1521),
    [sym_type_cast] = ACTIONS(1521),
    [sym_variable_to_xml] = ACTIONS(1521),
    [sym_xml_to_variable] = ACTIONS(1521),
    [sym_length_variable_to_xml] = ACTIONS(1521),
    [sym_redirect_string] = ACTIONS(1521),
    [sym_random_number] = ACTIONS(1521),
    [sym_get_unique_id] = ACTIONS(1521),
    [sym_get_system_number] = ACTIONS(1521),
    [sym_set_system_number] = ACTIONS(1521),
    [sym_get_serial_number] = ACTIONS(1521),
    [sym_get_master_build] = ACTIONS(1521),
    [sym_set_validation_code] = ACTIONS(1521),
    [sym_reboot] = ACTIONS(1521),
    [sym_load_duet_module] = ACTIONS(1521),
    [sym_unload_duet_module] = ACTIONS(1521),
    [sym_duet_mem_size_set] = ACTIONS(1521),
    [sym_duet_mem_size_get] = ACTIONS(1521),
    [sym_dynamic_application_device] = ACTIONS(1521),
    [sym_dynamic_polled_port] = ACTIONS(1521),
    [sym_static_port_binding] = ACTIONS(1521),
    [sym_static_ip_binding] = ACTIONS(1521),
    [sym_get_dns_list] = ACTIONS(1521),
    [sym_set_dns_list] = ACTIONS(1521),
    [sym_get_ip_address] = ACTIONS(1521),
    [sym_set_ip_address] = ACTIONS(1521),
    [sym_get_url_list] = ACTIONS(1521),
    [sym_add_url_entry] = ACTIONS(1521),
    [sym_delete_url_entry] = ACTIONS(1521),
    [sym_abs_value] = ACTIONS(1521),
    [sym_max_value] = ACTIONS(1521),
    [sym_min_value] = ACTIONS(1521),
    [sym_exp_value] = ACTIONS(1521),
    [sym_log_value] = ACTIONS(1521),
    [sym_log10_value] = ACTIONS(1521),
    [sym_power_value] = ACTIONS(1521),
    [sym_sqrt_value] = ACTIONS(1521),
    [sym_raw_be] = ACTIONS(1521),
    [sym_raw_le] = ACTIONS(1521),
    [sym_internal_queue_size_set] = ACTIONS(1521),
    [sym_internal_queue_size_get] = ACTIONS(1521),
    [sym_internal_threshold_set] = ACTIONS(1521),
    [sym_internal_threshold_get] = ACTIONS(1521),
    [sym_rebuilt_event] = ACTIONS(1521),
    [sym_do_custom_event] = ACTIONS(1521),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1521),
    [sym_clkmgr_set_clk_source] = ACTIONS(1521),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1521),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1521),
    [sym_clkmgr_get_timezone] = ACTIONS(1521),
    [sym_clkmgr_set_timezone] = ACTIONS(1521),
    [sym_clkmgr_get_resync_period] = ACTIONS(1521),
    [sym_clkmgr_set_resync_period] = ACTIONS(1521),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1521),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1521),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1521),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1521),
    [sym_clkmgr_get_timeservers] = ACTIONS(1521),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1521),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1521),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1521),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1521),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1521),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1521),
    [sym_set_log_level] = ACTIONS(1521),
    [sym_get_log_level] = ACTIONS(1521),
    [sym_amx_log] = ACTIONS(1521),
    [sym_smtp_server_config_set] = ACTIONS(1521),
    [sym_smtp_server_config_get] = ACTIONS(1521),
    [sym_smtp_send] = ACTIONS(1521),
    [sym_validate_netlinx_account] = ACTIONS(1521),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1521),
    [sym_authenticate_certificate] = ACTIONS(1521),
    [sym_audit_netlinx_session_event] = ACTIONS(1521),
    [sym_audit_netlinx_generic_event] = ACTIONS(1521),
    [sym_get_available_flash_disk_space] = ACTIONS(1521),
    [sym_get_max_flash_disk_space] = ACTIONS(1521),
    [sym_data_create_feed] = ACTIONS(1521),
    [sym_data_delete_feed] = ACTIONS(1521),
    [sym_data_publish_feed] = ACTIONS(1521),
    [sym_data_get_published_feed] = ACTIONS(1521),
    [sym_data_add_record] = ACTIONS(1521),
    [sym_data_get_event_record] = ACTIONS(1521),
    [sym__wc_data_create_feed] = ACTIONS(1521),
    [sym__wc_data_add_record] = ACTIONS(1521),
    [sym__wc_data_get_event_record] = ACTIONS(1521),
    [sym_format] = ACTIONS(1521),
    [sym_astro_clock] = ACTIONS(1521),
    [sym_set_outdoor_temperature] = ACTIONS(1521),
    [sym_set_virtual_level_count] = ACTIONS(1521),
    [sym_set_virtual_channel_count] = ACTIONS(1521),
    [sym_set_virtual_port_count] = ACTIONS(1521),
    [sym__wc_encode_int] = ACTIONS(1521),
    [sym__wc_decode_int] = ACTIONS(1521),
    [sym__wc_int] = ACTIONS(1521),
    [sym__wc_to_ch_int] = ACTIONS(1521),
    [sym__ch_to_wc_int] = ACTIONS(1521),
    [sym__wc_find_string_int] = ACTIONS(1521),
    [sym__wc_left_string_int] = ACTIONS(1521),
    [sym__wc_lower_string_int] = ACTIONS(1521),
    [sym__wc_mid_string_int] = ACTIONS(1521),
    [sym__wc_remove_string_int] = ACTIONS(1521),
    [sym__wc_right_string_int] = ACTIONS(1521),
    [sym__wc_upper_string_int] = ACTIONS(1521),
    [sym__wc_compare_string_int] = ACTIONS(1521),
    [sym__wc_get_buffer_char_int] = ACTIONS(1521),
    [sym__wc_get_buffer_string_int] = ACTIONS(1521),
    [sym__wc_concat_string_int] = ACTIONS(1521),
    [sym__wc_file_open_int] = ACTIONS(1521),
    [sym__wc_file_close_int] = ACTIONS(1521),
    [sym__wc_file_read_int] = ACTIONS(1521),
    [sym__wc_file_read_line_int] = ACTIONS(1521),
    [sym__wc_file_write_int] = ACTIONS(1521),
    [sym__wc_file_write_line_int] = ACTIONS(1521),
    [anon_sym_EQ] = ACTIONS(1521),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1523),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1523),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1523),
    [aux_sym_preproc_else_token1] = ACTIONS(1523),
    [anon_sym_COMMA] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(1523),
    [anon_sym_LBRACE] = ACTIONS(1523),
    [anon_sym_RBRACE] = ACTIONS(1523),
    [anon_sym_COLON] = ACTIONS(1523),
    [anon_sym_LPAREN] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1521),
    [anon_sym_TILDE] = ACTIONS(1523),
    [anon_sym_DASH] = ACTIONS(1521),
    [anon_sym_PLUS] = ACTIONS(1521),
    [anon_sym_STAR] = ACTIONS(1523),
    [anon_sym_SLASH] = ACTIONS(1521),
    [anon_sym_PERCENT] = ACTIONS(1523),
    [anon_sym_PIPE_PIPE] = ACTIONS(1523),
    [anon_sym_AMP_AMP] = ACTIONS(1523),
    [anon_sym_PIPE] = ACTIONS(1521),
    [anon_sym_CARET] = ACTIONS(1523),
    [anon_sym_AMP] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1523),
    [anon_sym_BANG_EQ] = ACTIONS(1523),
    [anon_sym_LT_GT] = ACTIONS(1523),
    [anon_sym_GT] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1523),
    [anon_sym_LT_EQ] = ACTIONS(1523),
    [anon_sym_LT] = ACTIONS(1521),
    [anon_sym_LT_LT] = ACTIONS(1523),
    [anon_sym_GT_GT] = ACTIONS(1523),
    [anon_sym_DASH_DASH] = ACTIONS(1523),
    [anon_sym_PLUS_PLUS] = ACTIONS(1523),
    [anon_sym_DOT] = ACTIONS(1521),
    [anon_sym_DQUOTE] = ACTIONS(1523),
    [anon_sym_SQUOTE] = ACTIONS(1523),
    [sym_number_literal] = ACTIONS(1523),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(596)] = {
    [sym_identifier] = ACTIONS(1543),
    [sym_and] = ACTIONS(1543),
    [sym_or] = ACTIONS(1543),
    [sym_not] = ACTIONS(1543),
    [sym_xor] = ACTIONS(1543),
    [sym_band] = ACTIONS(1543),
    [sym_bor] = ACTIONS(1543),
    [sym_bxor] = ACTIONS(1543),
    [sym_bnot] = ACTIONS(1543),
    [sym_lshift] = ACTIONS(1543),
    [sym_rshift] = ACTIONS(1543),
    [sym___file__] = ACTIONS(1543),
    [sym___date__] = ACTIONS(1543),
    [sym___ldate__] = ACTIONS(1543),
    [sym___time__] = ACTIONS(1543),
    [sym___line__] = ACTIONS(1543),
    [sym___name__] = ACTIONS(1543),
    [sym___netlinx__] = ACTIONS(1543),
    [sym_netlinx_axi_version] = ACTIONS(1543),
    [sym_url_flg_tcp] = ACTIONS(1543),
    [sym_url_flg_accinfopresent] = ACTIONS(1543),
    [sym_url_flg_temp] = ACTIONS(1543),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1543),
    [sym_url_flg_stat_mask] = ACTIONS(1543),
    [sym_url_flg_stat_lookup] = ACTIONS(1543),
    [sym_url_flg_stat_connecting] = ACTIONS(1543),
    [sym_url_flg_stat_waiting] = ACTIONS(1543),
    [sym_url_flg_stat_connected] = ACTIONS(1543),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1543),
    [sym_first_virtual_device] = ACTIONS(1543),
    [sym_first_local_port] = ACTIONS(1543),
    [sym_dynamic_virtual_device] = ACTIONS(1543),
    [sym_timeline_once] = ACTIONS(1543),
    [sym_timeline_repeat] = ACTIONS(1543),
    [sym_timeline_absolute] = ACTIONS(1543),
    [sym_timeline_relative] = ACTIONS(1543),
    [sym_do_push_timed_infinite] = ACTIONS(1543),
    [sym_source_type_no_address] = ACTIONS(1543),
    [sym_source_type_neuron_id] = ACTIONS(1543),
    [sym_source_type_ip_address] = ACTIONS(1543),
    [sym_source_type_axlink] = ACTIONS(1543),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1543),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1543),
    [sym_source_type_ip_socket_address] = ACTIONS(1543),
    [sym_source_type_rs232] = ACTIONS(1543),
    [sym_source_type_internal] = ACTIONS(1543),
    [sym_source_type_mac_address] = ACTIONS(1543),
    [sym_source_type_ipv4_port] = ACTIONS(1543),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1543),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1543),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1543),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1543),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1543),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1543),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1543),
    [sym_true] = ACTIONS(1543),
    [sym_false] = ACTIONS(1543),
    [sym_file_read_only] = ACTIONS(1543),
    [sym_file_rw_new] = ACTIONS(1543),
    [sym_file_rw_append] = ACTIONS(1543),
    [sym_ip_tcp] = ACTIONS(1543),
    [sym_ip_udp] = ACTIONS(1543),
    [sym_ip_udp_2way] = ACTIONS(1543),
    [sym_xml_encode_types] = ACTIONS(1543),
    [sym_xml_encode_char_as_list] = ACTIONS(1543),
    [sym_xml_encode_le] = ACTIONS(1543),
    [sym_xml_decode_types] = ACTIONS(1543),
    [sym_xml_decode_no_preserve] = ACTIONS(1543),
    [sym_duet_dev_type_display_device] = ACTIONS(1543),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1543),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1543),
    [sym_duet_dev_type_io_device] = ACTIONS(1543),
    [sym_duet_dev_type_relay_device] = ACTIONS(1543),
    [sym_duet_dev_type_ups] = ACTIONS(1543),
    [sym_duet_dev_type_amplifier] = ACTIONS(1543),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1543),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1543),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1543),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1543),
    [sym_duet_dev_type_camera] = ACTIONS(1543),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1543),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1543),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1543),
    [sym_duet_dev_type_dss] = ACTIONS(1543),
    [sym_duet_dev_type_dvr] = ACTIONS(1543),
    [sym_duet_dev_type_disc_device] = ACTIONS(1543),
    [sym_duet_dev_type_document_camera] = ACTIONS(1543),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1543),
    [sym_duet_dev_type_hvac] = ACTIONS(1543),
    [sym_duet_dev_type_keypad] = ACTIONS(1543),
    [sym_duet_dev_type_light] = ACTIONS(1543),
    [sym_duet_dev_type_monitor] = ACTIONS(1543),
    [sym_duet_dev_type_motor] = ACTIONS(1543),
    [sym_duet_dev_type_multi_window] = ACTIONS(1543),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1543),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1543),
    [sym_duet_dev_type_receiver] = ACTIONS(1543),
    [sym_duet_dev_type_security_system] = ACTIONS(1543),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1543),
    [sym_duet_dev_type_settop_box] = ACTIONS(1543),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1543),
    [sym_duet_dev_type_switcher] = ACTIONS(1543),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1543),
    [sym_duet_dev_type_tv] = ACTIONS(1543),
    [sym_duet_dev_type_utility] = ACTIONS(1543),
    [sym_duet_dev_type_vcr] = ACTIONS(1543),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1543),
    [sym_duet_dev_type_video_processor] = ACTIONS(1543),
    [sym_duet_dev_type_video_projector] = ACTIONS(1543),
    [sym_duet_dev_type_video_wall] = ACTIONS(1543),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1543),
    [sym_duet_dev_type_weather] = ACTIONS(1543),
    [sym_duet_dev_not_polled] = ACTIONS(1543),
    [sym_duet_dev_polled] = ACTIONS(1543),
    [sym_clkmgr_mode_network] = ACTIONS(1543),
    [sym_clkmgr_mode_standalone] = ACTIONS(1543),
    [sym_normal_standby] = ACTIONS(1543),
    [sym_normal_wake] = ACTIONS(1543),
    [sym_ip_multicast_ttl_option] = ACTIONS(1543),
    [sym_ip_ttl_subnet] = ACTIONS(1543),
    [sym_ip_ttl_site] = ACTIONS(1543),
    [sym_ip_ttl_region] = ACTIONS(1543),
    [sym_ip_ttl_continent] = ACTIONS(1543),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1543),
    [sym_ip_nodelay_on] = ACTIONS(1543),
    [sym_ip_nodelay_off] = ACTIONS(1543),
    [sym_amx_error] = ACTIONS(1543),
    [sym_amx_warning] = ACTIONS(1543),
    [sym_amx_info] = ACTIONS(1543),
    [sym_amx_debug] = ACTIONS(1543),
    [sym_netlinx_login_success] = ACTIONS(1543),
    [sym_netlinx_login_fail] = ACTIONS(1543),
    [sym_netlinx_logout] = ACTIONS(1543),
    [sym_null_str] = ACTIONS(1543),
    [sym_smtp_address] = ACTIONS(1543),
    [sym_smtp_port_number] = ACTIONS(1543),
    [sym_smtp_username] = ACTIONS(1543),
    [sym_smtp_password] = ACTIONS(1543),
    [sym_smtp_require_tls] = ACTIONS(1543),
    [sym_smtp_from] = ACTIONS(1543),
    [sym_smtp_tls_true] = ACTIONS(1543),
    [sym_smtp_tls_false] = ACTIONS(1543),
    [sym_listview_on_row_select_event] = ACTIONS(1543),
    [sym_data_structure_unknown] = ACTIONS(1543),
    [sym_data_structure_datafeed] = ACTIONS(1543),
    [sym_data_structure_datarecordset] = ACTIONS(1543),
    [sym_data_structure_datarecord] = ACTIONS(1543),
    [sym_data_structure_datafield] = ACTIONS(1543),
    [sym_data_max_content_field_count] = ACTIONS(1543),
    [sym_data_max_metadata_field_count] = ACTIONS(1543),
    [sym_data_max_id_length] = ACTIONS(1543),
    [sym_data_max_name_length] = ACTIONS(1543),
    [sym_data_max_description_length] = ACTIONS(1543),
    [sym_data_max_type_length] = ACTIONS(1543),
    [sym_data_max_format_length] = ACTIONS(1543),
    [sym_data_max_source_length] = ACTIONS(1543),
    [sym_data_max_value_length] = ACTIONS(1543),
    [sym_data_max_label_length] = ACTIONS(1543),
    [sym_data_max_url_length] = ACTIONS(1543),
    [sym_max_last_login_info_length] = ACTIONS(1543),
    [sym_data_type_unknown] = ACTIONS(1543),
    [sym_data_type_string] = ACTIONS(1543),
    [sym_data_type_datetime] = ACTIONS(1543),
    [sym_data_type_date] = ACTIONS(1543),
    [sym_data_type_time] = ACTIONS(1543),
    [sym_data_type_image] = ACTIONS(1543),
    [sym_data_format_url] = ACTIONS(1543),
    [sym_data_format_phone] = ACTIONS(1543),
    [sym_data_format_email] = ACTIONS(1543),
    [sym_data_format_iso8601] = ACTIONS(1543),
    [sym_auditlog_priv] = ACTIONS(1543),
    [sym_configuration_priv] = ACTIONS(1543),
    [sym_deviceconfig_priv] = ACTIONS(1543),
    [sym_ftp_priv] = ACTIONS(1543),
    [sym_http_priv] = ACTIONS(1543),
    [sym_networkconfig_priv] = ACTIONS(1543),
    [sym_programport_priv] = ACTIONS(1543),
    [sym_remoteui_priv] = ACTIONS(1543),
    [sym_securitycontrol_priv] = ACTIONS(1543),
    [sym_softwaremanagement_priv] = ACTIONS(1543),
    [sym_terminal_priv] = ACTIONS(1543),
    [sym_tpadmin_priv] = ACTIONS(1543),
    [sym_user1_priv] = ACTIONS(1543),
    [sym_user2_priv] = ACTIONS(1543),
    [sym_user3_priv] = ACTIONS(1543),
    [sym_user4_priv] = ACTIONS(1543),
    [sym_usermanagement_priv] = ACTIONS(1543),
    [sym_valid_account] = ACTIONS(1543),
    [sym_err_name_invalid] = ACTIONS(1543),
    [sym_err_password_invalid] = ACTIONS(1543),
    [sym_err_invalid_account] = ACTIONS(1543),
    [sym_err_no_authorization] = ACTIONS(1543),
    [sym_err_invalid_parameter] = ACTIONS(1543),
    [sym_err_user_locked_out] = ACTIONS(1543),
    [sym_err_user_account_expired] = ACTIONS(1543),
    [sym_err_general_error] = ACTIONS(1543),
    [sym_err_authentication_error] = ACTIONS(1543),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1543),
    [sym_tls_validate_certificate] = ACTIONS(1543),
    [sym____reserved___] = ACTIONS(1543),
    [sym_date] = ACTIONS(1543),
    [sym_day] = ACTIONS(1543),
    [sym_time] = ACTIONS(1543),
    [sym_ldate] = ACTIONS(1543),
    [sym_push_device] = ACTIONS(1543),
    [sym_push_channel] = ACTIONS(1543),
    [sym_push_devchan] = ACTIONS(1543),
    [sym_release_device] = ACTIONS(1543),
    [sym_release_channel] = ACTIONS(1543),
    [sym_release_devchan] = ACTIONS(1543),
    [sym_master_slot] = ACTIONS(1543),
    [sym_get_pulse_time] = ACTIONS(1543),
    [sym_get_timer] = ACTIONS(1543),
    [sym_master_sn] = ACTIONS(1543),
    [sym_system_number] = ACTIONS(1543),
    [sym_button] = ACTIONS(1543),
    [sym_dv_channel] = ACTIONS(1543),
    [sym_data] = ACTIONS(1543),
    [sym_level] = ACTIONS(1543),
    [sym_timeline] = ACTIONS(1543),
    [sym_channel] = ACTIONS(1543),
    [sym_custom] = ACTIONS(1543),
    [sym_atoi] = ACTIONS(1543),
    [sym_atol] = ACTIONS(1543),
    [sym_atof] = ACTIONS(1543),
    [sym_itoa] = ACTIONS(1543),
    [sym_itohex] = ACTIONS(1543),
    [sym_hextoi] = ACTIONS(1543),
    [sym_ftoa] = ACTIONS(1543),
    [sym_find_string] = ACTIONS(1543),
    [sym_left_string] = ACTIONS(1543),
    [sym_right_string] = ACTIONS(1543),
    [sym_mid_string] = ACTIONS(1543),
    [sym_remove_string] = ACTIONS(1543),
    [sym_get_buffer_string] = ACTIONS(1543),
    [sym_get_buffer_char] = ACTIONS(1543),
    [sym_get_multi_buffer_string] = ACTIONS(1543),
    [sym_length_string] = ACTIONS(1543),
    [sym_max_length_string] = ACTIONS(1543),
    [sym_set_length_string] = ACTIONS(1543),
    [sym_compare_string] = ACTIONS(1543),
    [sym_upper_string] = ACTIONS(1543),
    [sym_lower_string] = ACTIONS(1543),
    [sym_get_last] = ACTIONS(1543),
    [sym_date_to_day] = ACTIONS(1543),
    [sym_date_to_month] = ACTIONS(1543),
    [sym_date_to_year] = ACTIONS(1543),
    [sym_time_to_hour] = ACTIONS(1543),
    [sym_time_to_minute] = ACTIONS(1543),
    [sym_time_to_second] = ACTIONS(1543),
    [sym_day_of_week] = ACTIONS(1543),
    [sym_set_pulse_time] = ACTIONS(1543),
    [sym_set_timer] = ACTIONS(1543),
    [sym_file_open] = ACTIONS(1543),
    [sym_file_close] = ACTIONS(1543),
    [sym_file_read] = ACTIONS(1543),
    [sym_file_read_line] = ACTIONS(1543),
    [sym_file_write] = ACTIONS(1543),
    [sym_file_write_line] = ACTIONS(1543),
    [sym_file_copy] = ACTIONS(1543),
    [sym_file_delete] = ACTIONS(1543),
    [sym_file_rename] = ACTIONS(1543),
    [sym_file_dir] = ACTIONS(1543),
    [sym_file_getdir] = ACTIONS(1543),
    [sym_file_setdir] = ACTIONS(1543),
    [sym_file_create_dir] = ACTIONS(1543),
    [sym_file_remove_dir] = ACTIONS(1543),
    [sym_file_seek] = ACTIONS(1543),
    [sym_length_array] = ACTIONS(1543),
    [sym_max_length_array] = ACTIONS(1543),
    [sym_set_length_array] = ACTIONS(1543),
    [sym_ip_client_open] = ACTIONS(1543),
    [sym_ip_client_close] = ACTIONS(1543),
    [sym_ip_server_open] = ACTIONS(1543),
    [sym_ip_server_close] = ACTIONS(1543),
    [sym_ip_bound_client_open] = ACTIONS(1543),
    [sym_ip_mc_server_open] = ACTIONS(1543),
    [sym_ip_set_option] = ACTIONS(1543),
    [sym_ssh_client_open] = ACTIONS(1543),
    [sym_ssh_client_close] = ACTIONS(1543),
    [sym_tls_client_open] = ACTIONS(1543),
    [sym_tls_client_close] = ACTIONS(1543),
    [sym_do_push] = ACTIONS(1543),
    [sym_do_push_timed] = ACTIONS(1543),
    [sym_do_release] = ACTIONS(1543),
    [sym_device_id] = ACTIONS(1543),
    [sym_device_info] = ACTIONS(1543),
    [sym_device_id_string] = ACTIONS(1543),
    [sym_device_standby] = ACTIONS(1543),
    [sym_device_wake] = ACTIONS(1543),
    [sym_timeline_create] = ACTIONS(1543),
    [sym_timeline_kill] = ACTIONS(1543),
    [sym_timeline_reload] = ACTIONS(1543),
    [sym_timeline_pause] = ACTIONS(1543),
    [sym_timeline_restart] = ACTIONS(1543),
    [sym_timeline_set] = ACTIONS(1543),
    [sym_timeline_get] = ACTIONS(1543),
    [sym_timeline_active] = ACTIONS(1543),
    [sym_timeline_dynamic_id] = ACTIONS(1543),
    [sym_variable_to_string] = ACTIONS(1543),
    [sym_string_to_variable] = ACTIONS(1543),
    [sym_length_variable_to_string] = ACTIONS(1543),
    [sym_type_cast] = ACTIONS(1543),
    [sym_variable_to_xml] = ACTIONS(1543),
    [sym_xml_to_variable] = ACTIONS(1543),
    [sym_length_variable_to_xml] = ACTIONS(1543),
    [sym_redirect_string] = ACTIONS(1543),
    [sym_random_number] = ACTIONS(1543),
    [sym_get_unique_id] = ACTIONS(1543),
    [sym_get_system_number] = ACTIONS(1543),
    [sym_set_system_number] = ACTIONS(1543),
    [sym_get_serial_number] = ACTIONS(1543),
    [sym_get_master_build] = ACTIONS(1543),
    [sym_set_validation_code] = ACTIONS(1543),
    [sym_reboot] = ACTIONS(1543),
    [sym_load_duet_module] = ACTIONS(1543),
    [sym_unload_duet_module] = ACTIONS(1543),
    [sym_duet_mem_size_set] = ACTIONS(1543),
    [sym_duet_mem_size_get] = ACTIONS(1543),
    [sym_dynamic_application_device] = ACTIONS(1543),
    [sym_dynamic_polled_port] = ACTIONS(1543),
    [sym_static_port_binding] = ACTIONS(1543),
    [sym_static_ip_binding] = ACTIONS(1543),
    [sym_get_dns_list] = ACTIONS(1543),
    [sym_set_dns_list] = ACTIONS(1543),
    [sym_get_ip_address] = ACTIONS(1543),
    [sym_set_ip_address] = ACTIONS(1543),
    [sym_get_url_list] = ACTIONS(1543),
    [sym_add_url_entry] = ACTIONS(1543),
    [sym_delete_url_entry] = ACTIONS(1543),
    [sym_abs_value] = ACTIONS(1543),
    [sym_max_value] = ACTIONS(1543),
    [sym_min_value] = ACTIONS(1543),
    [sym_exp_value] = ACTIONS(1543),
    [sym_log_value] = ACTIONS(1543),
    [sym_log10_value] = ACTIONS(1543),
    [sym_power_value] = ACTIONS(1543),
    [sym_sqrt_value] = ACTIONS(1543),
    [sym_raw_be] = ACTIONS(1543),
    [sym_raw_le] = ACTIONS(1543),
    [sym_internal_queue_size_set] = ACTIONS(1543),
    [sym_internal_queue_size_get] = ACTIONS(1543),
    [sym_internal_threshold_set] = ACTIONS(1543),
    [sym_internal_threshold_get] = ACTIONS(1543),
    [sym_rebuilt_event] = ACTIONS(1543),
    [sym_do_custom_event] = ACTIONS(1543),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1543),
    [sym_clkmgr_set_clk_source] = ACTIONS(1543),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1543),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1543),
    [sym_clkmgr_get_timezone] = ACTIONS(1543),
    [sym_clkmgr_set_timezone] = ACTIONS(1543),
    [sym_clkmgr_get_resync_period] = ACTIONS(1543),
    [sym_clkmgr_set_resync_period] = ACTIONS(1543),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1543),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1543),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1543),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1543),
    [sym_clkmgr_get_timeservers] = ACTIONS(1543),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1543),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1543),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1543),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1543),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1543),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1543),
    [sym_set_log_level] = ACTIONS(1543),
    [sym_get_log_level] = ACTIONS(1543),
    [sym_amx_log] = ACTIONS(1543),
    [sym_smtp_server_config_set] = ACTIONS(1543),
    [sym_smtp_server_config_get] = ACTIONS(1543),
    [sym_smtp_send] = ACTIONS(1543),
    [sym_validate_netlinx_account] = ACTIONS(1543),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1543),
    [sym_authenticate_certificate] = ACTIONS(1543),
    [sym_audit_netlinx_session_event] = ACTIONS(1543),
    [sym_audit_netlinx_generic_event] = ACTIONS(1543),
    [sym_get_available_flash_disk_space] = ACTIONS(1543),
    [sym_get_max_flash_disk_space] = ACTIONS(1543),
    [sym_data_create_feed] = ACTIONS(1543),
    [sym_data_delete_feed] = ACTIONS(1543),
    [sym_data_publish_feed] = ACTIONS(1543),
    [sym_data_get_published_feed] = ACTIONS(1543),
    [sym_data_add_record] = ACTIONS(1543),
    [sym_data_get_event_record] = ACTIONS(1543),
    [sym__wc_data_create_feed] = ACTIONS(1543),
    [sym__wc_data_add_record] = ACTIONS(1543),
    [sym__wc_data_get_event_record] = ACTIONS(1543),
    [sym_format] = ACTIONS(1543),
    [sym_astro_clock] = ACTIONS(1543),
    [sym_set_outdoor_temperature] = ACTIONS(1543),
    [sym_set_virtual_level_count] = ACTIONS(1543),
    [sym_set_virtual_channel_count] = ACTIONS(1543),
    [sym_set_virtual_port_count] = ACTIONS(1543),
    [sym__wc_encode_int] = ACTIONS(1543),
    [sym__wc_decode_int] = ACTIONS(1543),
    [sym__wc_int] = ACTIONS(1543),
    [sym__wc_to_ch_int] = ACTIONS(1543),
    [sym__ch_to_wc_int] = ACTIONS(1543),
    [sym__wc_find_string_int] = ACTIONS(1543),
    [sym__wc_left_string_int] = ACTIONS(1543),
    [sym__wc_lower_string_int] = ACTIONS(1543),
    [sym__wc_mid_string_int] = ACTIONS(1543),
    [sym__wc_remove_string_int] = ACTIONS(1543),
    [sym__wc_right_string_int] = ACTIONS(1543),
    [sym__wc_upper_string_int] = ACTIONS(1543),
    [sym__wc_compare_string_int] = ACTIONS(1543),
    [sym__wc_get_buffer_char_int] = ACTIONS(1543),
    [sym__wc_get_buffer_string_int] = ACTIONS(1543),
    [sym__wc_concat_string_int] = ACTIONS(1543),
    [sym__wc_file_open_int] = ACTIONS(1543),
    [sym__wc_file_close_int] = ACTIONS(1543),
    [sym__wc_file_read_int] = ACTIONS(1543),
    [sym__wc_file_read_line_int] = ACTIONS(1543),
    [sym__wc_file_write_int] = ACTIONS(1543),
    [sym__wc_file_write_line_int] = ACTIONS(1543),
    [anon_sym_EQ] = ACTIONS(1543),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1545),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1545),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1545),
    [aux_sym_preproc_else_token1] = ACTIONS(1545),
    [anon_sym_COMMA] = ACTIONS(1545),
    [anon_sym_LBRACK] = ACTIONS(1545),
    [anon_sym_LBRACE] = ACTIONS(1545),
    [anon_sym_RBRACE] = ACTIONS(1545),
    [anon_sym_COLON] = ACTIONS(1545),
    [anon_sym_LPAREN] = ACTIONS(1543),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1545),
    [anon_sym_DASH] = ACTIONS(1543),
    [anon_sym_PLUS] = ACTIONS(1543),
    [anon_sym_STAR] = ACTIONS(1545),
    [anon_sym_SLASH] = ACTIONS(1543),
    [anon_sym_PERCENT] = ACTIONS(1545),
    [anon_sym_PIPE_PIPE] = ACTIONS(1545),
    [anon_sym_AMP_AMP] = ACTIONS(1545),
    [anon_sym_PIPE] = ACTIONS(1543),
    [anon_sym_CARET] = ACTIONS(1545),
    [anon_sym_AMP] = ACTIONS(1543),
    [anon_sym_EQ_EQ] = ACTIONS(1545),
    [anon_sym_BANG_EQ] = ACTIONS(1545),
    [anon_sym_LT_GT] = ACTIONS(1545),
    [anon_sym_GT] = ACTIONS(1543),
    [anon_sym_GT_EQ] = ACTIONS(1545),
    [anon_sym_LT_EQ] = ACTIONS(1545),
    [anon_sym_LT] = ACTIONS(1543),
    [anon_sym_LT_LT] = ACTIONS(1545),
    [anon_sym_GT_GT] = ACTIONS(1545),
    [anon_sym_DASH_DASH] = ACTIONS(1545),
    [anon_sym_PLUS_PLUS] = ACTIONS(1545),
    [anon_sym_DOT] = ACTIONS(1543),
    [anon_sym_DQUOTE] = ACTIONS(1545),
    [anon_sym_SQUOTE] = ACTIONS(1545),
    [sym_number_literal] = ACTIONS(1545),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(597)] = {
    [sym_identifier] = ACTIONS(1547),
    [sym_and] = ACTIONS(1547),
    [sym_or] = ACTIONS(1547),
    [sym_not] = ACTIONS(1547),
    [sym_xor] = ACTIONS(1547),
    [sym_band] = ACTIONS(1547),
    [sym_bor] = ACTIONS(1547),
    [sym_bxor] = ACTIONS(1547),
    [sym_bnot] = ACTIONS(1547),
    [sym_lshift] = ACTIONS(1547),
    [sym_rshift] = ACTIONS(1547),
    [sym___file__] = ACTIONS(1547),
    [sym___date__] = ACTIONS(1547),
    [sym___ldate__] = ACTIONS(1547),
    [sym___time__] = ACTIONS(1547),
    [sym___line__] = ACTIONS(1547),
    [sym___name__] = ACTIONS(1547),
    [sym___netlinx__] = ACTIONS(1547),
    [sym_netlinx_axi_version] = ACTIONS(1547),
    [sym_url_flg_tcp] = ACTIONS(1547),
    [sym_url_flg_accinfopresent] = ACTIONS(1547),
    [sym_url_flg_temp] = ACTIONS(1547),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1547),
    [sym_url_flg_stat_mask] = ACTIONS(1547),
    [sym_url_flg_stat_lookup] = ACTIONS(1547),
    [sym_url_flg_stat_connecting] = ACTIONS(1547),
    [sym_url_flg_stat_waiting] = ACTIONS(1547),
    [sym_url_flg_stat_connected] = ACTIONS(1547),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1547),
    [sym_first_virtual_device] = ACTIONS(1547),
    [sym_first_local_port] = ACTIONS(1547),
    [sym_dynamic_virtual_device] = ACTIONS(1547),
    [sym_timeline_once] = ACTIONS(1547),
    [sym_timeline_repeat] = ACTIONS(1547),
    [sym_timeline_absolute] = ACTIONS(1547),
    [sym_timeline_relative] = ACTIONS(1547),
    [sym_do_push_timed_infinite] = ACTIONS(1547),
    [sym_source_type_no_address] = ACTIONS(1547),
    [sym_source_type_neuron_id] = ACTIONS(1547),
    [sym_source_type_ip_address] = ACTIONS(1547),
    [sym_source_type_axlink] = ACTIONS(1547),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1547),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1547),
    [sym_source_type_ip_socket_address] = ACTIONS(1547),
    [sym_source_type_rs232] = ACTIONS(1547),
    [sym_source_type_internal] = ACTIONS(1547),
    [sym_source_type_mac_address] = ACTIONS(1547),
    [sym_source_type_ipv4_port] = ACTIONS(1547),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1547),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1547),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1547),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1547),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1547),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1547),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1547),
    [sym_true] = ACTIONS(1547),
    [sym_false] = ACTIONS(1547),
    [sym_file_read_only] = ACTIONS(1547),
    [sym_file_rw_new] = ACTIONS(1547),
    [sym_file_rw_append] = ACTIONS(1547),
    [sym_ip_tcp] = ACTIONS(1547),
    [sym_ip_udp] = ACTIONS(1547),
    [sym_ip_udp_2way] = ACTIONS(1547),
    [sym_xml_encode_types] = ACTIONS(1547),
    [sym_xml_encode_char_as_list] = ACTIONS(1547),
    [sym_xml_encode_le] = ACTIONS(1547),
    [sym_xml_decode_types] = ACTIONS(1547),
    [sym_xml_decode_no_preserve] = ACTIONS(1547),
    [sym_duet_dev_type_display_device] = ACTIONS(1547),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1547),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1547),
    [sym_duet_dev_type_io_device] = ACTIONS(1547),
    [sym_duet_dev_type_relay_device] = ACTIONS(1547),
    [sym_duet_dev_type_ups] = ACTIONS(1547),
    [sym_duet_dev_type_amplifier] = ACTIONS(1547),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1547),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1547),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1547),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1547),
    [sym_duet_dev_type_camera] = ACTIONS(1547),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1547),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1547),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1547),
    [sym_duet_dev_type_dss] = ACTIONS(1547),
    [sym_duet_dev_type_dvr] = ACTIONS(1547),
    [sym_duet_dev_type_disc_device] = ACTIONS(1547),
    [sym_duet_dev_type_document_camera] = ACTIONS(1547),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1547),
    [sym_duet_dev_type_hvac] = ACTIONS(1547),
    [sym_duet_dev_type_keypad] = ACTIONS(1547),
    [sym_duet_dev_type_light] = ACTIONS(1547),
    [sym_duet_dev_type_monitor] = ACTIONS(1547),
    [sym_duet_dev_type_motor] = ACTIONS(1547),
    [sym_duet_dev_type_multi_window] = ACTIONS(1547),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1547),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1547),
    [sym_duet_dev_type_receiver] = ACTIONS(1547),
    [sym_duet_dev_type_security_system] = ACTIONS(1547),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1547),
    [sym_duet_dev_type_settop_box] = ACTIONS(1547),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1547),
    [sym_duet_dev_type_switcher] = ACTIONS(1547),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1547),
    [sym_duet_dev_type_tv] = ACTIONS(1547),
    [sym_duet_dev_type_utility] = ACTIONS(1547),
    [sym_duet_dev_type_vcr] = ACTIONS(1547),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1547),
    [sym_duet_dev_type_video_processor] = ACTIONS(1547),
    [sym_duet_dev_type_video_projector] = ACTIONS(1547),
    [sym_duet_dev_type_video_wall] = ACTIONS(1547),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1547),
    [sym_duet_dev_type_weather] = ACTIONS(1547),
    [sym_duet_dev_not_polled] = ACTIONS(1547),
    [sym_duet_dev_polled] = ACTIONS(1547),
    [sym_clkmgr_mode_network] = ACTIONS(1547),
    [sym_clkmgr_mode_standalone] = ACTIONS(1547),
    [sym_normal_standby] = ACTIONS(1547),
    [sym_normal_wake] = ACTIONS(1547),
    [sym_ip_multicast_ttl_option] = ACTIONS(1547),
    [sym_ip_ttl_subnet] = ACTIONS(1547),
    [sym_ip_ttl_site] = ACTIONS(1547),
    [sym_ip_ttl_region] = ACTIONS(1547),
    [sym_ip_ttl_continent] = ACTIONS(1547),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1547),
    [sym_ip_nodelay_on] = ACTIONS(1547),
    [sym_ip_nodelay_off] = ACTIONS(1547),
    [sym_amx_error] = ACTIONS(1547),
    [sym_amx_warning] = ACTIONS(1547),
    [sym_amx_info] = ACTIONS(1547),
    [sym_amx_debug] = ACTIONS(1547),
    [sym_netlinx_login_success] = ACTIONS(1547),
    [sym_netlinx_login_fail] = ACTIONS(1547),
    [sym_netlinx_logout] = ACTIONS(1547),
    [sym_null_str] = ACTIONS(1547),
    [sym_smtp_address] = ACTIONS(1547),
    [sym_smtp_port_number] = ACTIONS(1547),
    [sym_smtp_username] = ACTIONS(1547),
    [sym_smtp_password] = ACTIONS(1547),
    [sym_smtp_require_tls] = ACTIONS(1547),
    [sym_smtp_from] = ACTIONS(1547),
    [sym_smtp_tls_true] = ACTIONS(1547),
    [sym_smtp_tls_false] = ACTIONS(1547),
    [sym_listview_on_row_select_event] = ACTIONS(1547),
    [sym_data_structure_unknown] = ACTIONS(1547),
    [sym_data_structure_datafeed] = ACTIONS(1547),
    [sym_data_structure_datarecordset] = ACTIONS(1547),
    [sym_data_structure_datarecord] = ACTIONS(1547),
    [sym_data_structure_datafield] = ACTIONS(1547),
    [sym_data_max_content_field_count] = ACTIONS(1547),
    [sym_data_max_metadata_field_count] = ACTIONS(1547),
    [sym_data_max_id_length] = ACTIONS(1547),
    [sym_data_max_name_length] = ACTIONS(1547),
    [sym_data_max_description_length] = ACTIONS(1547),
    [sym_data_max_type_length] = ACTIONS(1547),
    [sym_data_max_format_length] = ACTIONS(1547),
    [sym_data_max_source_length] = ACTIONS(1547),
    [sym_data_max_value_length] = ACTIONS(1547),
    [sym_data_max_label_length] = ACTIONS(1547),
    [sym_data_max_url_length] = ACTIONS(1547),
    [sym_max_last_login_info_length] = ACTIONS(1547),
    [sym_data_type_unknown] = ACTIONS(1547),
    [sym_data_type_string] = ACTIONS(1547),
    [sym_data_type_datetime] = ACTIONS(1547),
    [sym_data_type_date] = ACTIONS(1547),
    [sym_data_type_time] = ACTIONS(1547),
    [sym_data_type_image] = ACTIONS(1547),
    [sym_data_format_url] = ACTIONS(1547),
    [sym_data_format_phone] = ACTIONS(1547),
    [sym_data_format_email] = ACTIONS(1547),
    [sym_data_format_iso8601] = ACTIONS(1547),
    [sym_auditlog_priv] = ACTIONS(1547),
    [sym_configuration_priv] = ACTIONS(1547),
    [sym_deviceconfig_priv] = ACTIONS(1547),
    [sym_ftp_priv] = ACTIONS(1547),
    [sym_http_priv] = ACTIONS(1547),
    [sym_networkconfig_priv] = ACTIONS(1547),
    [sym_programport_priv] = ACTIONS(1547),
    [sym_remoteui_priv] = ACTIONS(1547),
    [sym_securitycontrol_priv] = ACTIONS(1547),
    [sym_softwaremanagement_priv] = ACTIONS(1547),
    [sym_terminal_priv] = ACTIONS(1547),
    [sym_tpadmin_priv] = ACTIONS(1547),
    [sym_user1_priv] = ACTIONS(1547),
    [sym_user2_priv] = ACTIONS(1547),
    [sym_user3_priv] = ACTIONS(1547),
    [sym_user4_priv] = ACTIONS(1547),
    [sym_usermanagement_priv] = ACTIONS(1547),
    [sym_valid_account] = ACTIONS(1547),
    [sym_err_name_invalid] = ACTIONS(1547),
    [sym_err_password_invalid] = ACTIONS(1547),
    [sym_err_invalid_account] = ACTIONS(1547),
    [sym_err_no_authorization] = ACTIONS(1547),
    [sym_err_invalid_parameter] = ACTIONS(1547),
    [sym_err_user_locked_out] = ACTIONS(1547),
    [sym_err_user_account_expired] = ACTIONS(1547),
    [sym_err_general_error] = ACTIONS(1547),
    [sym_err_authentication_error] = ACTIONS(1547),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1547),
    [sym_tls_validate_certificate] = ACTIONS(1547),
    [sym____reserved___] = ACTIONS(1547),
    [sym_date] = ACTIONS(1547),
    [sym_day] = ACTIONS(1547),
    [sym_time] = ACTIONS(1547),
    [sym_ldate] = ACTIONS(1547),
    [sym_push_device] = ACTIONS(1547),
    [sym_push_channel] = ACTIONS(1547),
    [sym_push_devchan] = ACTIONS(1547),
    [sym_release_device] = ACTIONS(1547),
    [sym_release_channel] = ACTIONS(1547),
    [sym_release_devchan] = ACTIONS(1547),
    [sym_master_slot] = ACTIONS(1547),
    [sym_get_pulse_time] = ACTIONS(1547),
    [sym_get_timer] = ACTIONS(1547),
    [sym_master_sn] = ACTIONS(1547),
    [sym_system_number] = ACTIONS(1547),
    [sym_button] = ACTIONS(1547),
    [sym_dv_channel] = ACTIONS(1547),
    [sym_data] = ACTIONS(1547),
    [sym_level] = ACTIONS(1547),
    [sym_timeline] = ACTIONS(1547),
    [sym_channel] = ACTIONS(1547),
    [sym_custom] = ACTIONS(1547),
    [sym_atoi] = ACTIONS(1547),
    [sym_atol] = ACTIONS(1547),
    [sym_atof] = ACTIONS(1547),
    [sym_itoa] = ACTIONS(1547),
    [sym_itohex] = ACTIONS(1547),
    [sym_hextoi] = ACTIONS(1547),
    [sym_ftoa] = ACTIONS(1547),
    [sym_find_string] = ACTIONS(1547),
    [sym_left_string] = ACTIONS(1547),
    [sym_right_string] = ACTIONS(1547),
    [sym_mid_string] = ACTIONS(1547),
    [sym_remove_string] = ACTIONS(1547),
    [sym_get_buffer_string] = ACTIONS(1547),
    [sym_get_buffer_char] = ACTIONS(1547),
    [sym_get_multi_buffer_string] = ACTIONS(1547),
    [sym_length_string] = ACTIONS(1547),
    [sym_max_length_string] = ACTIONS(1547),
    [sym_set_length_string] = ACTIONS(1547),
    [sym_compare_string] = ACTIONS(1547),
    [sym_upper_string] = ACTIONS(1547),
    [sym_lower_string] = ACTIONS(1547),
    [sym_get_last] = ACTIONS(1547),
    [sym_date_to_day] = ACTIONS(1547),
    [sym_date_to_month] = ACTIONS(1547),
    [sym_date_to_year] = ACTIONS(1547),
    [sym_time_to_hour] = ACTIONS(1547),
    [sym_time_to_minute] = ACTIONS(1547),
    [sym_time_to_second] = ACTIONS(1547),
    [sym_day_of_week] = ACTIONS(1547),
    [sym_set_pulse_time] = ACTIONS(1547),
    [sym_set_timer] = ACTIONS(1547),
    [sym_file_open] = ACTIONS(1547),
    [sym_file_close] = ACTIONS(1547),
    [sym_file_read] = ACTIONS(1547),
    [sym_file_read_line] = ACTIONS(1547),
    [sym_file_write] = ACTIONS(1547),
    [sym_file_write_line] = ACTIONS(1547),
    [sym_file_copy] = ACTIONS(1547),
    [sym_file_delete] = ACTIONS(1547),
    [sym_file_rename] = ACTIONS(1547),
    [sym_file_dir] = ACTIONS(1547),
    [sym_file_getdir] = ACTIONS(1547),
    [sym_file_setdir] = ACTIONS(1547),
    [sym_file_create_dir] = ACTIONS(1547),
    [sym_file_remove_dir] = ACTIONS(1547),
    [sym_file_seek] = ACTIONS(1547),
    [sym_length_array] = ACTIONS(1547),
    [sym_max_length_array] = ACTIONS(1547),
    [sym_set_length_array] = ACTIONS(1547),
    [sym_ip_client_open] = ACTIONS(1547),
    [sym_ip_client_close] = ACTIONS(1547),
    [sym_ip_server_open] = ACTIONS(1547),
    [sym_ip_server_close] = ACTIONS(1547),
    [sym_ip_bound_client_open] = ACTIONS(1547),
    [sym_ip_mc_server_open] = ACTIONS(1547),
    [sym_ip_set_option] = ACTIONS(1547),
    [sym_ssh_client_open] = ACTIONS(1547),
    [sym_ssh_client_close] = ACTIONS(1547),
    [sym_tls_client_open] = ACTIONS(1547),
    [sym_tls_client_close] = ACTIONS(1547),
    [sym_do_push] = ACTIONS(1547),
    [sym_do_push_timed] = ACTIONS(1547),
    [sym_do_release] = ACTIONS(1547),
    [sym_device_id] = ACTIONS(1547),
    [sym_device_info] = ACTIONS(1547),
    [sym_device_id_string] = ACTIONS(1547),
    [sym_device_standby] = ACTIONS(1547),
    [sym_device_wake] = ACTIONS(1547),
    [sym_timeline_create] = ACTIONS(1547),
    [sym_timeline_kill] = ACTIONS(1547),
    [sym_timeline_reload] = ACTIONS(1547),
    [sym_timeline_pause] = ACTIONS(1547),
    [sym_timeline_restart] = ACTIONS(1547),
    [sym_timeline_set] = ACTIONS(1547),
    [sym_timeline_get] = ACTIONS(1547),
    [sym_timeline_active] = ACTIONS(1547),
    [sym_timeline_dynamic_id] = ACTIONS(1547),
    [sym_variable_to_string] = ACTIONS(1547),
    [sym_string_to_variable] = ACTIONS(1547),
    [sym_length_variable_to_string] = ACTIONS(1547),
    [sym_type_cast] = ACTIONS(1547),
    [sym_variable_to_xml] = ACTIONS(1547),
    [sym_xml_to_variable] = ACTIONS(1547),
    [sym_length_variable_to_xml] = ACTIONS(1547),
    [sym_redirect_string] = ACTIONS(1547),
    [sym_random_number] = ACTIONS(1547),
    [sym_get_unique_id] = ACTIONS(1547),
    [sym_get_system_number] = ACTIONS(1547),
    [sym_set_system_number] = ACTIONS(1547),
    [sym_get_serial_number] = ACTIONS(1547),
    [sym_get_master_build] = ACTIONS(1547),
    [sym_set_validation_code] = ACTIONS(1547),
    [sym_reboot] = ACTIONS(1547),
    [sym_load_duet_module] = ACTIONS(1547),
    [sym_unload_duet_module] = ACTIONS(1547),
    [sym_duet_mem_size_set] = ACTIONS(1547),
    [sym_duet_mem_size_get] = ACTIONS(1547),
    [sym_dynamic_application_device] = ACTIONS(1547),
    [sym_dynamic_polled_port] = ACTIONS(1547),
    [sym_static_port_binding] = ACTIONS(1547),
    [sym_static_ip_binding] = ACTIONS(1547),
    [sym_get_dns_list] = ACTIONS(1547),
    [sym_set_dns_list] = ACTIONS(1547),
    [sym_get_ip_address] = ACTIONS(1547),
    [sym_set_ip_address] = ACTIONS(1547),
    [sym_get_url_list] = ACTIONS(1547),
    [sym_add_url_entry] = ACTIONS(1547),
    [sym_delete_url_entry] = ACTIONS(1547),
    [sym_abs_value] = ACTIONS(1547),
    [sym_max_value] = ACTIONS(1547),
    [sym_min_value] = ACTIONS(1547),
    [sym_exp_value] = ACTIONS(1547),
    [sym_log_value] = ACTIONS(1547),
    [sym_log10_value] = ACTIONS(1547),
    [sym_power_value] = ACTIONS(1547),
    [sym_sqrt_value] = ACTIONS(1547),
    [sym_raw_be] = ACTIONS(1547),
    [sym_raw_le] = ACTIONS(1547),
    [sym_internal_queue_size_set] = ACTIONS(1547),
    [sym_internal_queue_size_get] = ACTIONS(1547),
    [sym_internal_threshold_set] = ACTIONS(1547),
    [sym_internal_threshold_get] = ACTIONS(1547),
    [sym_rebuilt_event] = ACTIONS(1547),
    [sym_do_custom_event] = ACTIONS(1547),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1547),
    [sym_clkmgr_set_clk_source] = ACTIONS(1547),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1547),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1547),
    [sym_clkmgr_get_timezone] = ACTIONS(1547),
    [sym_clkmgr_set_timezone] = ACTIONS(1547),
    [sym_clkmgr_get_resync_period] = ACTIONS(1547),
    [sym_clkmgr_set_resync_period] = ACTIONS(1547),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1547),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1547),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1547),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1547),
    [sym_clkmgr_get_timeservers] = ACTIONS(1547),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1547),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1547),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1547),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1547),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1547),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1547),
    [sym_set_log_level] = ACTIONS(1547),
    [sym_get_log_level] = ACTIONS(1547),
    [sym_amx_log] = ACTIONS(1547),
    [sym_smtp_server_config_set] = ACTIONS(1547),
    [sym_smtp_server_config_get] = ACTIONS(1547),
    [sym_smtp_send] = ACTIONS(1547),
    [sym_validate_netlinx_account] = ACTIONS(1547),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1547),
    [sym_authenticate_certificate] = ACTIONS(1547),
    [sym_audit_netlinx_session_event] = ACTIONS(1547),
    [sym_audit_netlinx_generic_event] = ACTIONS(1547),
    [sym_get_available_flash_disk_space] = ACTIONS(1547),
    [sym_get_max_flash_disk_space] = ACTIONS(1547),
    [sym_data_create_feed] = ACTIONS(1547),
    [sym_data_delete_feed] = ACTIONS(1547),
    [sym_data_publish_feed] = ACTIONS(1547),
    [sym_data_get_published_feed] = ACTIONS(1547),
    [sym_data_add_record] = ACTIONS(1547),
    [sym_data_get_event_record] = ACTIONS(1547),
    [sym__wc_data_create_feed] = ACTIONS(1547),
    [sym__wc_data_add_record] = ACTIONS(1547),
    [sym__wc_data_get_event_record] = ACTIONS(1547),
    [sym_format] = ACTIONS(1547),
    [sym_astro_clock] = ACTIONS(1547),
    [sym_set_outdoor_temperature] = ACTIONS(1547),
    [sym_set_virtual_level_count] = ACTIONS(1547),
    [sym_set_virtual_channel_count] = ACTIONS(1547),
    [sym_set_virtual_port_count] = ACTIONS(1547),
    [sym__wc_encode_int] = ACTIONS(1547),
    [sym__wc_decode_int] = ACTIONS(1547),
    [sym__wc_int] = ACTIONS(1547),
    [sym__wc_to_ch_int] = ACTIONS(1547),
    [sym__ch_to_wc_int] = ACTIONS(1547),
    [sym__wc_find_string_int] = ACTIONS(1547),
    [sym__wc_left_string_int] = ACTIONS(1547),
    [sym__wc_lower_string_int] = ACTIONS(1547),
    [sym__wc_mid_string_int] = ACTIONS(1547),
    [sym__wc_remove_string_int] = ACTIONS(1547),
    [sym__wc_right_string_int] = ACTIONS(1547),
    [sym__wc_upper_string_int] = ACTIONS(1547),
    [sym__wc_compare_string_int] = ACTIONS(1547),
    [sym__wc_get_buffer_char_int] = ACTIONS(1547),
    [sym__wc_get_buffer_string_int] = ACTIONS(1547),
    [sym__wc_concat_string_int] = ACTIONS(1547),
    [sym__wc_file_open_int] = ACTIONS(1547),
    [sym__wc_file_close_int] = ACTIONS(1547),
    [sym__wc_file_read_int] = ACTIONS(1547),
    [sym__wc_file_read_line_int] = ACTIONS(1547),
    [sym__wc_file_write_int] = ACTIONS(1547),
    [sym__wc_file_write_line_int] = ACTIONS(1547),
    [anon_sym_EQ] = ACTIONS(1547),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1549),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1549),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1549),
    [aux_sym_preproc_else_token1] = ACTIONS(1549),
    [anon_sym_COMMA] = ACTIONS(1549),
    [anon_sym_LBRACK] = ACTIONS(1549),
    [anon_sym_LBRACE] = ACTIONS(1549),
    [anon_sym_RBRACE] = ACTIONS(1549),
    [anon_sym_COLON] = ACTIONS(1549),
    [anon_sym_LPAREN] = ACTIONS(1547),
    [anon_sym_BANG] = ACTIONS(1547),
    [anon_sym_TILDE] = ACTIONS(1549),
    [anon_sym_DASH] = ACTIONS(1547),
    [anon_sym_PLUS] = ACTIONS(1547),
    [anon_sym_STAR] = ACTIONS(1549),
    [anon_sym_SLASH] = ACTIONS(1547),
    [anon_sym_PERCENT] = ACTIONS(1549),
    [anon_sym_PIPE_PIPE] = ACTIONS(1549),
    [anon_sym_AMP_AMP] = ACTIONS(1549),
    [anon_sym_PIPE] = ACTIONS(1547),
    [anon_sym_CARET] = ACTIONS(1549),
    [anon_sym_AMP] = ACTIONS(1547),
    [anon_sym_EQ_EQ] = ACTIONS(1549),
    [anon_sym_BANG_EQ] = ACTIONS(1549),
    [anon_sym_LT_GT] = ACTIONS(1549),
    [anon_sym_GT] = ACTIONS(1547),
    [anon_sym_GT_EQ] = ACTIONS(1549),
    [anon_sym_LT_EQ] = ACTIONS(1549),
    [anon_sym_LT] = ACTIONS(1547),
    [anon_sym_LT_LT] = ACTIONS(1549),
    [anon_sym_GT_GT] = ACTIONS(1549),
    [anon_sym_DASH_DASH] = ACTIONS(1549),
    [anon_sym_PLUS_PLUS] = ACTIONS(1549),
    [anon_sym_DOT] = ACTIONS(1547),
    [anon_sym_DQUOTE] = ACTIONS(1549),
    [anon_sym_SQUOTE] = ACTIONS(1549),
    [sym_number_literal] = ACTIONS(1549),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(598)] = {
    [sym_identifier] = ACTIONS(1551),
    [sym_and] = ACTIONS(1551),
    [sym_or] = ACTIONS(1551),
    [sym_not] = ACTIONS(1551),
    [sym_xor] = ACTIONS(1551),
    [sym_band] = ACTIONS(1551),
    [sym_bor] = ACTIONS(1551),
    [sym_bxor] = ACTIONS(1551),
    [sym_bnot] = ACTIONS(1551),
    [sym_lshift] = ACTIONS(1551),
    [sym_rshift] = ACTIONS(1551),
    [sym___file__] = ACTIONS(1551),
    [sym___date__] = ACTIONS(1551),
    [sym___ldate__] = ACTIONS(1551),
    [sym___time__] = ACTIONS(1551),
    [sym___line__] = ACTIONS(1551),
    [sym___name__] = ACTIONS(1551),
    [sym___netlinx__] = ACTIONS(1551),
    [sym_netlinx_axi_version] = ACTIONS(1551),
    [sym_url_flg_tcp] = ACTIONS(1551),
    [sym_url_flg_accinfopresent] = ACTIONS(1551),
    [sym_url_flg_temp] = ACTIONS(1551),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1551),
    [sym_url_flg_stat_mask] = ACTIONS(1551),
    [sym_url_flg_stat_lookup] = ACTIONS(1551),
    [sym_url_flg_stat_connecting] = ACTIONS(1551),
    [sym_url_flg_stat_waiting] = ACTIONS(1551),
    [sym_url_flg_stat_connected] = ACTIONS(1551),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1551),
    [sym_first_virtual_device] = ACTIONS(1551),
    [sym_first_local_port] = ACTIONS(1551),
    [sym_dynamic_virtual_device] = ACTIONS(1551),
    [sym_timeline_once] = ACTIONS(1551),
    [sym_timeline_repeat] = ACTIONS(1551),
    [sym_timeline_absolute] = ACTIONS(1551),
    [sym_timeline_relative] = ACTIONS(1551),
    [sym_do_push_timed_infinite] = ACTIONS(1551),
    [sym_source_type_no_address] = ACTIONS(1551),
    [sym_source_type_neuron_id] = ACTIONS(1551),
    [sym_source_type_ip_address] = ACTIONS(1551),
    [sym_source_type_axlink] = ACTIONS(1551),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1551),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1551),
    [sym_source_type_ip_socket_address] = ACTIONS(1551),
    [sym_source_type_rs232] = ACTIONS(1551),
    [sym_source_type_internal] = ACTIONS(1551),
    [sym_source_type_mac_address] = ACTIONS(1551),
    [sym_source_type_ipv4_port] = ACTIONS(1551),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1551),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1551),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1551),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1551),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1551),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1551),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1551),
    [sym_true] = ACTIONS(1551),
    [sym_false] = ACTIONS(1551),
    [sym_file_read_only] = ACTIONS(1551),
    [sym_file_rw_new] = ACTIONS(1551),
    [sym_file_rw_append] = ACTIONS(1551),
    [sym_ip_tcp] = ACTIONS(1551),
    [sym_ip_udp] = ACTIONS(1551),
    [sym_ip_udp_2way] = ACTIONS(1551),
    [sym_xml_encode_types] = ACTIONS(1551),
    [sym_xml_encode_char_as_list] = ACTIONS(1551),
    [sym_xml_encode_le] = ACTIONS(1551),
    [sym_xml_decode_types] = ACTIONS(1551),
    [sym_xml_decode_no_preserve] = ACTIONS(1551),
    [sym_duet_dev_type_display_device] = ACTIONS(1551),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1551),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1551),
    [sym_duet_dev_type_io_device] = ACTIONS(1551),
    [sym_duet_dev_type_relay_device] = ACTIONS(1551),
    [sym_duet_dev_type_ups] = ACTIONS(1551),
    [sym_duet_dev_type_amplifier] = ACTIONS(1551),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1551),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1551),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1551),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1551),
    [sym_duet_dev_type_camera] = ACTIONS(1551),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1551),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1551),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1551),
    [sym_duet_dev_type_dss] = ACTIONS(1551),
    [sym_duet_dev_type_dvr] = ACTIONS(1551),
    [sym_duet_dev_type_disc_device] = ACTIONS(1551),
    [sym_duet_dev_type_document_camera] = ACTIONS(1551),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1551),
    [sym_duet_dev_type_hvac] = ACTIONS(1551),
    [sym_duet_dev_type_keypad] = ACTIONS(1551),
    [sym_duet_dev_type_light] = ACTIONS(1551),
    [sym_duet_dev_type_monitor] = ACTIONS(1551),
    [sym_duet_dev_type_motor] = ACTIONS(1551),
    [sym_duet_dev_type_multi_window] = ACTIONS(1551),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1551),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1551),
    [sym_duet_dev_type_receiver] = ACTIONS(1551),
    [sym_duet_dev_type_security_system] = ACTIONS(1551),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1551),
    [sym_duet_dev_type_settop_box] = ACTIONS(1551),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1551),
    [sym_duet_dev_type_switcher] = ACTIONS(1551),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1551),
    [sym_duet_dev_type_tv] = ACTIONS(1551),
    [sym_duet_dev_type_utility] = ACTIONS(1551),
    [sym_duet_dev_type_vcr] = ACTIONS(1551),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1551),
    [sym_duet_dev_type_video_processor] = ACTIONS(1551),
    [sym_duet_dev_type_video_projector] = ACTIONS(1551),
    [sym_duet_dev_type_video_wall] = ACTIONS(1551),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1551),
    [sym_duet_dev_type_weather] = ACTIONS(1551),
    [sym_duet_dev_not_polled] = ACTIONS(1551),
    [sym_duet_dev_polled] = ACTIONS(1551),
    [sym_clkmgr_mode_network] = ACTIONS(1551),
    [sym_clkmgr_mode_standalone] = ACTIONS(1551),
    [sym_normal_standby] = ACTIONS(1551),
    [sym_normal_wake] = ACTIONS(1551),
    [sym_ip_multicast_ttl_option] = ACTIONS(1551),
    [sym_ip_ttl_subnet] = ACTIONS(1551),
    [sym_ip_ttl_site] = ACTIONS(1551),
    [sym_ip_ttl_region] = ACTIONS(1551),
    [sym_ip_ttl_continent] = ACTIONS(1551),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1551),
    [sym_ip_nodelay_on] = ACTIONS(1551),
    [sym_ip_nodelay_off] = ACTIONS(1551),
    [sym_amx_error] = ACTIONS(1551),
    [sym_amx_warning] = ACTIONS(1551),
    [sym_amx_info] = ACTIONS(1551),
    [sym_amx_debug] = ACTIONS(1551),
    [sym_netlinx_login_success] = ACTIONS(1551),
    [sym_netlinx_login_fail] = ACTIONS(1551),
    [sym_netlinx_logout] = ACTIONS(1551),
    [sym_null_str] = ACTIONS(1551),
    [sym_smtp_address] = ACTIONS(1551),
    [sym_smtp_port_number] = ACTIONS(1551),
    [sym_smtp_username] = ACTIONS(1551),
    [sym_smtp_password] = ACTIONS(1551),
    [sym_smtp_require_tls] = ACTIONS(1551),
    [sym_smtp_from] = ACTIONS(1551),
    [sym_smtp_tls_true] = ACTIONS(1551),
    [sym_smtp_tls_false] = ACTIONS(1551),
    [sym_listview_on_row_select_event] = ACTIONS(1551),
    [sym_data_structure_unknown] = ACTIONS(1551),
    [sym_data_structure_datafeed] = ACTIONS(1551),
    [sym_data_structure_datarecordset] = ACTIONS(1551),
    [sym_data_structure_datarecord] = ACTIONS(1551),
    [sym_data_structure_datafield] = ACTIONS(1551),
    [sym_data_max_content_field_count] = ACTIONS(1551),
    [sym_data_max_metadata_field_count] = ACTIONS(1551),
    [sym_data_max_id_length] = ACTIONS(1551),
    [sym_data_max_name_length] = ACTIONS(1551),
    [sym_data_max_description_length] = ACTIONS(1551),
    [sym_data_max_type_length] = ACTIONS(1551),
    [sym_data_max_format_length] = ACTIONS(1551),
    [sym_data_max_source_length] = ACTIONS(1551),
    [sym_data_max_value_length] = ACTIONS(1551),
    [sym_data_max_label_length] = ACTIONS(1551),
    [sym_data_max_url_length] = ACTIONS(1551),
    [sym_max_last_login_info_length] = ACTIONS(1551),
    [sym_data_type_unknown] = ACTIONS(1551),
    [sym_data_type_string] = ACTIONS(1551),
    [sym_data_type_datetime] = ACTIONS(1551),
    [sym_data_type_date] = ACTIONS(1551),
    [sym_data_type_time] = ACTIONS(1551),
    [sym_data_type_image] = ACTIONS(1551),
    [sym_data_format_url] = ACTIONS(1551),
    [sym_data_format_phone] = ACTIONS(1551),
    [sym_data_format_email] = ACTIONS(1551),
    [sym_data_format_iso8601] = ACTIONS(1551),
    [sym_auditlog_priv] = ACTIONS(1551),
    [sym_configuration_priv] = ACTIONS(1551),
    [sym_deviceconfig_priv] = ACTIONS(1551),
    [sym_ftp_priv] = ACTIONS(1551),
    [sym_http_priv] = ACTIONS(1551),
    [sym_networkconfig_priv] = ACTIONS(1551),
    [sym_programport_priv] = ACTIONS(1551),
    [sym_remoteui_priv] = ACTIONS(1551),
    [sym_securitycontrol_priv] = ACTIONS(1551),
    [sym_softwaremanagement_priv] = ACTIONS(1551),
    [sym_terminal_priv] = ACTIONS(1551),
    [sym_tpadmin_priv] = ACTIONS(1551),
    [sym_user1_priv] = ACTIONS(1551),
    [sym_user2_priv] = ACTIONS(1551),
    [sym_user3_priv] = ACTIONS(1551),
    [sym_user4_priv] = ACTIONS(1551),
    [sym_usermanagement_priv] = ACTIONS(1551),
    [sym_valid_account] = ACTIONS(1551),
    [sym_err_name_invalid] = ACTIONS(1551),
    [sym_err_password_invalid] = ACTIONS(1551),
    [sym_err_invalid_account] = ACTIONS(1551),
    [sym_err_no_authorization] = ACTIONS(1551),
    [sym_err_invalid_parameter] = ACTIONS(1551),
    [sym_err_user_locked_out] = ACTIONS(1551),
    [sym_err_user_account_expired] = ACTIONS(1551),
    [sym_err_general_error] = ACTIONS(1551),
    [sym_err_authentication_error] = ACTIONS(1551),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1551),
    [sym_tls_validate_certificate] = ACTIONS(1551),
    [sym____reserved___] = ACTIONS(1551),
    [sym_date] = ACTIONS(1551),
    [sym_day] = ACTIONS(1551),
    [sym_time] = ACTIONS(1551),
    [sym_ldate] = ACTIONS(1551),
    [sym_push_device] = ACTIONS(1551),
    [sym_push_channel] = ACTIONS(1551),
    [sym_push_devchan] = ACTIONS(1551),
    [sym_release_device] = ACTIONS(1551),
    [sym_release_channel] = ACTIONS(1551),
    [sym_release_devchan] = ACTIONS(1551),
    [sym_master_slot] = ACTIONS(1551),
    [sym_get_pulse_time] = ACTIONS(1551),
    [sym_get_timer] = ACTIONS(1551),
    [sym_master_sn] = ACTIONS(1551),
    [sym_system_number] = ACTIONS(1551),
    [sym_button] = ACTIONS(1551),
    [sym_dv_channel] = ACTIONS(1551),
    [sym_data] = ACTIONS(1551),
    [sym_level] = ACTIONS(1551),
    [sym_timeline] = ACTIONS(1551),
    [sym_channel] = ACTIONS(1551),
    [sym_custom] = ACTIONS(1551),
    [sym_atoi] = ACTIONS(1551),
    [sym_atol] = ACTIONS(1551),
    [sym_atof] = ACTIONS(1551),
    [sym_itoa] = ACTIONS(1551),
    [sym_itohex] = ACTIONS(1551),
    [sym_hextoi] = ACTIONS(1551),
    [sym_ftoa] = ACTIONS(1551),
    [sym_find_string] = ACTIONS(1551),
    [sym_left_string] = ACTIONS(1551),
    [sym_right_string] = ACTIONS(1551),
    [sym_mid_string] = ACTIONS(1551),
    [sym_remove_string] = ACTIONS(1551),
    [sym_get_buffer_string] = ACTIONS(1551),
    [sym_get_buffer_char] = ACTIONS(1551),
    [sym_get_multi_buffer_string] = ACTIONS(1551),
    [sym_length_string] = ACTIONS(1551),
    [sym_max_length_string] = ACTIONS(1551),
    [sym_set_length_string] = ACTIONS(1551),
    [sym_compare_string] = ACTIONS(1551),
    [sym_upper_string] = ACTIONS(1551),
    [sym_lower_string] = ACTIONS(1551),
    [sym_get_last] = ACTIONS(1551),
    [sym_date_to_day] = ACTIONS(1551),
    [sym_date_to_month] = ACTIONS(1551),
    [sym_date_to_year] = ACTIONS(1551),
    [sym_time_to_hour] = ACTIONS(1551),
    [sym_time_to_minute] = ACTIONS(1551),
    [sym_time_to_second] = ACTIONS(1551),
    [sym_day_of_week] = ACTIONS(1551),
    [sym_set_pulse_time] = ACTIONS(1551),
    [sym_set_timer] = ACTIONS(1551),
    [sym_file_open] = ACTIONS(1551),
    [sym_file_close] = ACTIONS(1551),
    [sym_file_read] = ACTIONS(1551),
    [sym_file_read_line] = ACTIONS(1551),
    [sym_file_write] = ACTIONS(1551),
    [sym_file_write_line] = ACTIONS(1551),
    [sym_file_copy] = ACTIONS(1551),
    [sym_file_delete] = ACTIONS(1551),
    [sym_file_rename] = ACTIONS(1551),
    [sym_file_dir] = ACTIONS(1551),
    [sym_file_getdir] = ACTIONS(1551),
    [sym_file_setdir] = ACTIONS(1551),
    [sym_file_create_dir] = ACTIONS(1551),
    [sym_file_remove_dir] = ACTIONS(1551),
    [sym_file_seek] = ACTIONS(1551),
    [sym_length_array] = ACTIONS(1551),
    [sym_max_length_array] = ACTIONS(1551),
    [sym_set_length_array] = ACTIONS(1551),
    [sym_ip_client_open] = ACTIONS(1551),
    [sym_ip_client_close] = ACTIONS(1551),
    [sym_ip_server_open] = ACTIONS(1551),
    [sym_ip_server_close] = ACTIONS(1551),
    [sym_ip_bound_client_open] = ACTIONS(1551),
    [sym_ip_mc_server_open] = ACTIONS(1551),
    [sym_ip_set_option] = ACTIONS(1551),
    [sym_ssh_client_open] = ACTIONS(1551),
    [sym_ssh_client_close] = ACTIONS(1551),
    [sym_tls_client_open] = ACTIONS(1551),
    [sym_tls_client_close] = ACTIONS(1551),
    [sym_do_push] = ACTIONS(1551),
    [sym_do_push_timed] = ACTIONS(1551),
    [sym_do_release] = ACTIONS(1551),
    [sym_device_id] = ACTIONS(1551),
    [sym_device_info] = ACTIONS(1551),
    [sym_device_id_string] = ACTIONS(1551),
    [sym_device_standby] = ACTIONS(1551),
    [sym_device_wake] = ACTIONS(1551),
    [sym_timeline_create] = ACTIONS(1551),
    [sym_timeline_kill] = ACTIONS(1551),
    [sym_timeline_reload] = ACTIONS(1551),
    [sym_timeline_pause] = ACTIONS(1551),
    [sym_timeline_restart] = ACTIONS(1551),
    [sym_timeline_set] = ACTIONS(1551),
    [sym_timeline_get] = ACTIONS(1551),
    [sym_timeline_active] = ACTIONS(1551),
    [sym_timeline_dynamic_id] = ACTIONS(1551),
    [sym_variable_to_string] = ACTIONS(1551),
    [sym_string_to_variable] = ACTIONS(1551),
    [sym_length_variable_to_string] = ACTIONS(1551),
    [sym_type_cast] = ACTIONS(1551),
    [sym_variable_to_xml] = ACTIONS(1551),
    [sym_xml_to_variable] = ACTIONS(1551),
    [sym_length_variable_to_xml] = ACTIONS(1551),
    [sym_redirect_string] = ACTIONS(1551),
    [sym_random_number] = ACTIONS(1551),
    [sym_get_unique_id] = ACTIONS(1551),
    [sym_get_system_number] = ACTIONS(1551),
    [sym_set_system_number] = ACTIONS(1551),
    [sym_get_serial_number] = ACTIONS(1551),
    [sym_get_master_build] = ACTIONS(1551),
    [sym_set_validation_code] = ACTIONS(1551),
    [sym_reboot] = ACTIONS(1551),
    [sym_load_duet_module] = ACTIONS(1551),
    [sym_unload_duet_module] = ACTIONS(1551),
    [sym_duet_mem_size_set] = ACTIONS(1551),
    [sym_duet_mem_size_get] = ACTIONS(1551),
    [sym_dynamic_application_device] = ACTIONS(1551),
    [sym_dynamic_polled_port] = ACTIONS(1551),
    [sym_static_port_binding] = ACTIONS(1551),
    [sym_static_ip_binding] = ACTIONS(1551),
    [sym_get_dns_list] = ACTIONS(1551),
    [sym_set_dns_list] = ACTIONS(1551),
    [sym_get_ip_address] = ACTIONS(1551),
    [sym_set_ip_address] = ACTIONS(1551),
    [sym_get_url_list] = ACTIONS(1551),
    [sym_add_url_entry] = ACTIONS(1551),
    [sym_delete_url_entry] = ACTIONS(1551),
    [sym_abs_value] = ACTIONS(1551),
    [sym_max_value] = ACTIONS(1551),
    [sym_min_value] = ACTIONS(1551),
    [sym_exp_value] = ACTIONS(1551),
    [sym_log_value] = ACTIONS(1551),
    [sym_log10_value] = ACTIONS(1551),
    [sym_power_value] = ACTIONS(1551),
    [sym_sqrt_value] = ACTIONS(1551),
    [sym_raw_be] = ACTIONS(1551),
    [sym_raw_le] = ACTIONS(1551),
    [sym_internal_queue_size_set] = ACTIONS(1551),
    [sym_internal_queue_size_get] = ACTIONS(1551),
    [sym_internal_threshold_set] = ACTIONS(1551),
    [sym_internal_threshold_get] = ACTIONS(1551),
    [sym_rebuilt_event] = ACTIONS(1551),
    [sym_do_custom_event] = ACTIONS(1551),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1551),
    [sym_clkmgr_set_clk_source] = ACTIONS(1551),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1551),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1551),
    [sym_clkmgr_get_timezone] = ACTIONS(1551),
    [sym_clkmgr_set_timezone] = ACTIONS(1551),
    [sym_clkmgr_get_resync_period] = ACTIONS(1551),
    [sym_clkmgr_set_resync_period] = ACTIONS(1551),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1551),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1551),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1551),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1551),
    [sym_clkmgr_get_timeservers] = ACTIONS(1551),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1551),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1551),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1551),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1551),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1551),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1551),
    [sym_set_log_level] = ACTIONS(1551),
    [sym_get_log_level] = ACTIONS(1551),
    [sym_amx_log] = ACTIONS(1551),
    [sym_smtp_server_config_set] = ACTIONS(1551),
    [sym_smtp_server_config_get] = ACTIONS(1551),
    [sym_smtp_send] = ACTIONS(1551),
    [sym_validate_netlinx_account] = ACTIONS(1551),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1551),
    [sym_authenticate_certificate] = ACTIONS(1551),
    [sym_audit_netlinx_session_event] = ACTIONS(1551),
    [sym_audit_netlinx_generic_event] = ACTIONS(1551),
    [sym_get_available_flash_disk_space] = ACTIONS(1551),
    [sym_get_max_flash_disk_space] = ACTIONS(1551),
    [sym_data_create_feed] = ACTIONS(1551),
    [sym_data_delete_feed] = ACTIONS(1551),
    [sym_data_publish_feed] = ACTIONS(1551),
    [sym_data_get_published_feed] = ACTIONS(1551),
    [sym_data_add_record] = ACTIONS(1551),
    [sym_data_get_event_record] = ACTIONS(1551),
    [sym__wc_data_create_feed] = ACTIONS(1551),
    [sym__wc_data_add_record] = ACTIONS(1551),
    [sym__wc_data_get_event_record] = ACTIONS(1551),
    [sym_format] = ACTIONS(1551),
    [sym_astro_clock] = ACTIONS(1551),
    [sym_set_outdoor_temperature] = ACTIONS(1551),
    [sym_set_virtual_level_count] = ACTIONS(1551),
    [sym_set_virtual_channel_count] = ACTIONS(1551),
    [sym_set_virtual_port_count] = ACTIONS(1551),
    [sym__wc_encode_int] = ACTIONS(1551),
    [sym__wc_decode_int] = ACTIONS(1551),
    [sym__wc_int] = ACTIONS(1551),
    [sym__wc_to_ch_int] = ACTIONS(1551),
    [sym__ch_to_wc_int] = ACTIONS(1551),
    [sym__wc_find_string_int] = ACTIONS(1551),
    [sym__wc_left_string_int] = ACTIONS(1551),
    [sym__wc_lower_string_int] = ACTIONS(1551),
    [sym__wc_mid_string_int] = ACTIONS(1551),
    [sym__wc_remove_string_int] = ACTIONS(1551),
    [sym__wc_right_string_int] = ACTIONS(1551),
    [sym__wc_upper_string_int] = ACTIONS(1551),
    [sym__wc_compare_string_int] = ACTIONS(1551),
    [sym__wc_get_buffer_char_int] = ACTIONS(1551),
    [sym__wc_get_buffer_string_int] = ACTIONS(1551),
    [sym__wc_concat_string_int] = ACTIONS(1551),
    [sym__wc_file_open_int] = ACTIONS(1551),
    [sym__wc_file_close_int] = ACTIONS(1551),
    [sym__wc_file_read_int] = ACTIONS(1551),
    [sym__wc_file_read_line_int] = ACTIONS(1551),
    [sym__wc_file_write_int] = ACTIONS(1551),
    [sym__wc_file_write_line_int] = ACTIONS(1551),
    [anon_sym_EQ] = ACTIONS(1551),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1553),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1553),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1553),
    [aux_sym_preproc_else_token1] = ACTIONS(1553),
    [anon_sym_COMMA] = ACTIONS(1553),
    [anon_sym_LBRACK] = ACTIONS(1553),
    [anon_sym_LBRACE] = ACTIONS(1553),
    [anon_sym_RBRACE] = ACTIONS(1553),
    [anon_sym_COLON] = ACTIONS(1553),
    [anon_sym_LPAREN] = ACTIONS(1551),
    [anon_sym_BANG] = ACTIONS(1551),
    [anon_sym_TILDE] = ACTIONS(1553),
    [anon_sym_DASH] = ACTIONS(1551),
    [anon_sym_PLUS] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(1553),
    [anon_sym_SLASH] = ACTIONS(1551),
    [anon_sym_PERCENT] = ACTIONS(1553),
    [anon_sym_PIPE_PIPE] = ACTIONS(1553),
    [anon_sym_AMP_AMP] = ACTIONS(1553),
    [anon_sym_PIPE] = ACTIONS(1551),
    [anon_sym_CARET] = ACTIONS(1553),
    [anon_sym_AMP] = ACTIONS(1551),
    [anon_sym_EQ_EQ] = ACTIONS(1553),
    [anon_sym_BANG_EQ] = ACTIONS(1553),
    [anon_sym_LT_GT] = ACTIONS(1553),
    [anon_sym_GT] = ACTIONS(1551),
    [anon_sym_GT_EQ] = ACTIONS(1553),
    [anon_sym_LT_EQ] = ACTIONS(1553),
    [anon_sym_LT] = ACTIONS(1551),
    [anon_sym_LT_LT] = ACTIONS(1553),
    [anon_sym_GT_GT] = ACTIONS(1553),
    [anon_sym_DASH_DASH] = ACTIONS(1553),
    [anon_sym_PLUS_PLUS] = ACTIONS(1553),
    [anon_sym_DOT] = ACTIONS(1551),
    [anon_sym_DQUOTE] = ACTIONS(1553),
    [anon_sym_SQUOTE] = ACTIONS(1553),
    [sym_number_literal] = ACTIONS(1553),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(599)] = {
    [sym_identifier] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [anon_sym_EQ] = ACTIONS(2215),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_else_token1] = ACTIONS(1469),
    [anon_sym_COMMA] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_RBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(600)] = {
    [sym_identifier] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [anon_sym_EQ] = ACTIONS(2208),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_else_token1] = ACTIONS(1469),
    [anon_sym_COMMA] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_RBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(601)] = {
    [sym_identifier] = ACTIONS(1555),
    [sym_and] = ACTIONS(1555),
    [sym_or] = ACTIONS(1555),
    [sym_not] = ACTIONS(1555),
    [sym_xor] = ACTIONS(1555),
    [sym_band] = ACTIONS(1555),
    [sym_bor] = ACTIONS(1555),
    [sym_bxor] = ACTIONS(1555),
    [sym_bnot] = ACTIONS(1555),
    [sym_lshift] = ACTIONS(1555),
    [sym_rshift] = ACTIONS(1555),
    [sym___file__] = ACTIONS(1555),
    [sym___date__] = ACTIONS(1555),
    [sym___ldate__] = ACTIONS(1555),
    [sym___time__] = ACTIONS(1555),
    [sym___line__] = ACTIONS(1555),
    [sym___name__] = ACTIONS(1555),
    [sym___netlinx__] = ACTIONS(1555),
    [sym_netlinx_axi_version] = ACTIONS(1555),
    [sym_url_flg_tcp] = ACTIONS(1555),
    [sym_url_flg_accinfopresent] = ACTIONS(1555),
    [sym_url_flg_temp] = ACTIONS(1555),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1555),
    [sym_url_flg_stat_mask] = ACTIONS(1555),
    [sym_url_flg_stat_lookup] = ACTIONS(1555),
    [sym_url_flg_stat_connecting] = ACTIONS(1555),
    [sym_url_flg_stat_waiting] = ACTIONS(1555),
    [sym_url_flg_stat_connected] = ACTIONS(1555),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1555),
    [sym_first_virtual_device] = ACTIONS(1555),
    [sym_first_local_port] = ACTIONS(1555),
    [sym_dynamic_virtual_device] = ACTIONS(1555),
    [sym_timeline_once] = ACTIONS(1555),
    [sym_timeline_repeat] = ACTIONS(1555),
    [sym_timeline_absolute] = ACTIONS(1555),
    [sym_timeline_relative] = ACTIONS(1555),
    [sym_do_push_timed_infinite] = ACTIONS(1555),
    [sym_source_type_no_address] = ACTIONS(1555),
    [sym_source_type_neuron_id] = ACTIONS(1555),
    [sym_source_type_ip_address] = ACTIONS(1555),
    [sym_source_type_axlink] = ACTIONS(1555),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1555),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1555),
    [sym_source_type_ip_socket_address] = ACTIONS(1555),
    [sym_source_type_rs232] = ACTIONS(1555),
    [sym_source_type_internal] = ACTIONS(1555),
    [sym_source_type_mac_address] = ACTIONS(1555),
    [sym_source_type_ipv4_port] = ACTIONS(1555),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1555),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1555),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1555),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1555),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1555),
    [sym_true] = ACTIONS(1555),
    [sym_false] = ACTIONS(1555),
    [sym_file_read_only] = ACTIONS(1555),
    [sym_file_rw_new] = ACTIONS(1555),
    [sym_file_rw_append] = ACTIONS(1555),
    [sym_ip_tcp] = ACTIONS(1555),
    [sym_ip_udp] = ACTIONS(1555),
    [sym_ip_udp_2way] = ACTIONS(1555),
    [sym_xml_encode_types] = ACTIONS(1555),
    [sym_xml_encode_char_as_list] = ACTIONS(1555),
    [sym_xml_encode_le] = ACTIONS(1555),
    [sym_xml_decode_types] = ACTIONS(1555),
    [sym_xml_decode_no_preserve] = ACTIONS(1555),
    [sym_duet_dev_type_display_device] = ACTIONS(1555),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1555),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1555),
    [sym_duet_dev_type_io_device] = ACTIONS(1555),
    [sym_duet_dev_type_relay_device] = ACTIONS(1555),
    [sym_duet_dev_type_ups] = ACTIONS(1555),
    [sym_duet_dev_type_amplifier] = ACTIONS(1555),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1555),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1555),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1555),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1555),
    [sym_duet_dev_type_camera] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1555),
    [sym_duet_dev_type_dss] = ACTIONS(1555),
    [sym_duet_dev_type_dvr] = ACTIONS(1555),
    [sym_duet_dev_type_disc_device] = ACTIONS(1555),
    [sym_duet_dev_type_document_camera] = ACTIONS(1555),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1555),
    [sym_duet_dev_type_hvac] = ACTIONS(1555),
    [sym_duet_dev_type_keypad] = ACTIONS(1555),
    [sym_duet_dev_type_light] = ACTIONS(1555),
    [sym_duet_dev_type_monitor] = ACTIONS(1555),
    [sym_duet_dev_type_motor] = ACTIONS(1555),
    [sym_duet_dev_type_multi_window] = ACTIONS(1555),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1555),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1555),
    [sym_duet_dev_type_receiver] = ACTIONS(1555),
    [sym_duet_dev_type_security_system] = ACTIONS(1555),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1555),
    [sym_duet_dev_type_settop_box] = ACTIONS(1555),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1555),
    [sym_duet_dev_type_switcher] = ACTIONS(1555),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1555),
    [sym_duet_dev_type_tv] = ACTIONS(1555),
    [sym_duet_dev_type_utility] = ACTIONS(1555),
    [sym_duet_dev_type_vcr] = ACTIONS(1555),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1555),
    [sym_duet_dev_type_video_processor] = ACTIONS(1555),
    [sym_duet_dev_type_video_projector] = ACTIONS(1555),
    [sym_duet_dev_type_video_wall] = ACTIONS(1555),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1555),
    [sym_duet_dev_type_weather] = ACTIONS(1555),
    [sym_duet_dev_not_polled] = ACTIONS(1555),
    [sym_duet_dev_polled] = ACTIONS(1555),
    [sym_clkmgr_mode_network] = ACTIONS(1555),
    [sym_clkmgr_mode_standalone] = ACTIONS(1555),
    [sym_normal_standby] = ACTIONS(1555),
    [sym_normal_wake] = ACTIONS(1555),
    [sym_ip_multicast_ttl_option] = ACTIONS(1555),
    [sym_ip_ttl_subnet] = ACTIONS(1555),
    [sym_ip_ttl_site] = ACTIONS(1555),
    [sym_ip_ttl_region] = ACTIONS(1555),
    [sym_ip_ttl_continent] = ACTIONS(1555),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1555),
    [sym_ip_nodelay_on] = ACTIONS(1555),
    [sym_ip_nodelay_off] = ACTIONS(1555),
    [sym_amx_error] = ACTIONS(1555),
    [sym_amx_warning] = ACTIONS(1555),
    [sym_amx_info] = ACTIONS(1555),
    [sym_amx_debug] = ACTIONS(1555),
    [sym_netlinx_login_success] = ACTIONS(1555),
    [sym_netlinx_login_fail] = ACTIONS(1555),
    [sym_netlinx_logout] = ACTIONS(1555),
    [sym_null_str] = ACTIONS(1555),
    [sym_smtp_address] = ACTIONS(1555),
    [sym_smtp_port_number] = ACTIONS(1555),
    [sym_smtp_username] = ACTIONS(1555),
    [sym_smtp_password] = ACTIONS(1555),
    [sym_smtp_require_tls] = ACTIONS(1555),
    [sym_smtp_from] = ACTIONS(1555),
    [sym_smtp_tls_true] = ACTIONS(1555),
    [sym_smtp_tls_false] = ACTIONS(1555),
    [sym_listview_on_row_select_event] = ACTIONS(1555),
    [sym_data_structure_unknown] = ACTIONS(1555),
    [sym_data_structure_datafeed] = ACTIONS(1555),
    [sym_data_structure_datarecordset] = ACTIONS(1555),
    [sym_data_structure_datarecord] = ACTIONS(1555),
    [sym_data_structure_datafield] = ACTIONS(1555),
    [sym_data_max_content_field_count] = ACTIONS(1555),
    [sym_data_max_metadata_field_count] = ACTIONS(1555),
    [sym_data_max_id_length] = ACTIONS(1555),
    [sym_data_max_name_length] = ACTIONS(1555),
    [sym_data_max_description_length] = ACTIONS(1555),
    [sym_data_max_type_length] = ACTIONS(1555),
    [sym_data_max_format_length] = ACTIONS(1555),
    [sym_data_max_source_length] = ACTIONS(1555),
    [sym_data_max_value_length] = ACTIONS(1555),
    [sym_data_max_label_length] = ACTIONS(1555),
    [sym_data_max_url_length] = ACTIONS(1555),
    [sym_max_last_login_info_length] = ACTIONS(1555),
    [sym_data_type_unknown] = ACTIONS(1555),
    [sym_data_type_string] = ACTIONS(1555),
    [sym_data_type_datetime] = ACTIONS(1555),
    [sym_data_type_date] = ACTIONS(1555),
    [sym_data_type_time] = ACTIONS(1555),
    [sym_data_type_image] = ACTIONS(1555),
    [sym_data_format_url] = ACTIONS(1555),
    [sym_data_format_phone] = ACTIONS(1555),
    [sym_data_format_email] = ACTIONS(1555),
    [sym_data_format_iso8601] = ACTIONS(1555),
    [sym_auditlog_priv] = ACTIONS(1555),
    [sym_configuration_priv] = ACTIONS(1555),
    [sym_deviceconfig_priv] = ACTIONS(1555),
    [sym_ftp_priv] = ACTIONS(1555),
    [sym_http_priv] = ACTIONS(1555),
    [sym_networkconfig_priv] = ACTIONS(1555),
    [sym_programport_priv] = ACTIONS(1555),
    [sym_remoteui_priv] = ACTIONS(1555),
    [sym_securitycontrol_priv] = ACTIONS(1555),
    [sym_softwaremanagement_priv] = ACTIONS(1555),
    [sym_terminal_priv] = ACTIONS(1555),
    [sym_tpadmin_priv] = ACTIONS(1555),
    [sym_user1_priv] = ACTIONS(1555),
    [sym_user2_priv] = ACTIONS(1555),
    [sym_user3_priv] = ACTIONS(1555),
    [sym_user4_priv] = ACTIONS(1555),
    [sym_usermanagement_priv] = ACTIONS(1555),
    [sym_valid_account] = ACTIONS(1555),
    [sym_err_name_invalid] = ACTIONS(1555),
    [sym_err_password_invalid] = ACTIONS(1555),
    [sym_err_invalid_account] = ACTIONS(1555),
    [sym_err_no_authorization] = ACTIONS(1555),
    [sym_err_invalid_parameter] = ACTIONS(1555),
    [sym_err_user_locked_out] = ACTIONS(1555),
    [sym_err_user_account_expired] = ACTIONS(1555),
    [sym_err_general_error] = ACTIONS(1555),
    [sym_err_authentication_error] = ACTIONS(1555),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1555),
    [sym_tls_validate_certificate] = ACTIONS(1555),
    [sym____reserved___] = ACTIONS(1555),
    [sym_date] = ACTIONS(1555),
    [sym_day] = ACTIONS(1555),
    [sym_time] = ACTIONS(1555),
    [sym_ldate] = ACTIONS(1555),
    [sym_push_device] = ACTIONS(1555),
    [sym_push_channel] = ACTIONS(1555),
    [sym_push_devchan] = ACTIONS(1555),
    [sym_release_device] = ACTIONS(1555),
    [sym_release_channel] = ACTIONS(1555),
    [sym_release_devchan] = ACTIONS(1555),
    [sym_master_slot] = ACTIONS(1555),
    [sym_get_pulse_time] = ACTIONS(1555),
    [sym_get_timer] = ACTIONS(1555),
    [sym_master_sn] = ACTIONS(1555),
    [sym_system_number] = ACTIONS(1555),
    [sym_button] = ACTIONS(1555),
    [sym_dv_channel] = ACTIONS(1555),
    [sym_data] = ACTIONS(1555),
    [sym_level] = ACTIONS(1555),
    [sym_timeline] = ACTIONS(1555),
    [sym_channel] = ACTIONS(1555),
    [sym_custom] = ACTIONS(1555),
    [sym_atoi] = ACTIONS(1555),
    [sym_atol] = ACTIONS(1555),
    [sym_atof] = ACTIONS(1555),
    [sym_itoa] = ACTIONS(1555),
    [sym_itohex] = ACTIONS(1555),
    [sym_hextoi] = ACTIONS(1555),
    [sym_ftoa] = ACTIONS(1555),
    [sym_find_string] = ACTIONS(1555),
    [sym_left_string] = ACTIONS(1555),
    [sym_right_string] = ACTIONS(1555),
    [sym_mid_string] = ACTIONS(1555),
    [sym_remove_string] = ACTIONS(1555),
    [sym_get_buffer_string] = ACTIONS(1555),
    [sym_get_buffer_char] = ACTIONS(1555),
    [sym_get_multi_buffer_string] = ACTIONS(1555),
    [sym_length_string] = ACTIONS(1555),
    [sym_max_length_string] = ACTIONS(1555),
    [sym_set_length_string] = ACTIONS(1555),
    [sym_compare_string] = ACTIONS(1555),
    [sym_upper_string] = ACTIONS(1555),
    [sym_lower_string] = ACTIONS(1555),
    [sym_get_last] = ACTIONS(1555),
    [sym_date_to_day] = ACTIONS(1555),
    [sym_date_to_month] = ACTIONS(1555),
    [sym_date_to_year] = ACTIONS(1555),
    [sym_time_to_hour] = ACTIONS(1555),
    [sym_time_to_minute] = ACTIONS(1555),
    [sym_time_to_second] = ACTIONS(1555),
    [sym_day_of_week] = ACTIONS(1555),
    [sym_set_pulse_time] = ACTIONS(1555),
    [sym_set_timer] = ACTIONS(1555),
    [sym_file_open] = ACTIONS(1555),
    [sym_file_close] = ACTIONS(1555),
    [sym_file_read] = ACTIONS(1555),
    [sym_file_read_line] = ACTIONS(1555),
    [sym_file_write] = ACTIONS(1555),
    [sym_file_write_line] = ACTIONS(1555),
    [sym_file_copy] = ACTIONS(1555),
    [sym_file_delete] = ACTIONS(1555),
    [sym_file_rename] = ACTIONS(1555),
    [sym_file_dir] = ACTIONS(1555),
    [sym_file_getdir] = ACTIONS(1555),
    [sym_file_setdir] = ACTIONS(1555),
    [sym_file_create_dir] = ACTIONS(1555),
    [sym_file_remove_dir] = ACTIONS(1555),
    [sym_file_seek] = ACTIONS(1555),
    [sym_length_array] = ACTIONS(1555),
    [sym_max_length_array] = ACTIONS(1555),
    [sym_set_length_array] = ACTIONS(1555),
    [sym_ip_client_open] = ACTIONS(1555),
    [sym_ip_client_close] = ACTIONS(1555),
    [sym_ip_server_open] = ACTIONS(1555),
    [sym_ip_server_close] = ACTIONS(1555),
    [sym_ip_bound_client_open] = ACTIONS(1555),
    [sym_ip_mc_server_open] = ACTIONS(1555),
    [sym_ip_set_option] = ACTIONS(1555),
    [sym_ssh_client_open] = ACTIONS(1555),
    [sym_ssh_client_close] = ACTIONS(1555),
    [sym_tls_client_open] = ACTIONS(1555),
    [sym_tls_client_close] = ACTIONS(1555),
    [sym_do_push] = ACTIONS(1555),
    [sym_do_push_timed] = ACTIONS(1555),
    [sym_do_release] = ACTIONS(1555),
    [sym_device_id] = ACTIONS(1555),
    [sym_device_info] = ACTIONS(1555),
    [sym_device_id_string] = ACTIONS(1555),
    [sym_device_standby] = ACTIONS(1555),
    [sym_device_wake] = ACTIONS(1555),
    [sym_timeline_create] = ACTIONS(1555),
    [sym_timeline_kill] = ACTIONS(1555),
    [sym_timeline_reload] = ACTIONS(1555),
    [sym_timeline_pause] = ACTIONS(1555),
    [sym_timeline_restart] = ACTIONS(1555),
    [sym_timeline_set] = ACTIONS(1555),
    [sym_timeline_get] = ACTIONS(1555),
    [sym_timeline_active] = ACTIONS(1555),
    [sym_timeline_dynamic_id] = ACTIONS(1555),
    [sym_variable_to_string] = ACTIONS(1555),
    [sym_string_to_variable] = ACTIONS(1555),
    [sym_length_variable_to_string] = ACTIONS(1555),
    [sym_type_cast] = ACTIONS(1555),
    [sym_variable_to_xml] = ACTIONS(1555),
    [sym_xml_to_variable] = ACTIONS(1555),
    [sym_length_variable_to_xml] = ACTIONS(1555),
    [sym_redirect_string] = ACTIONS(1555),
    [sym_random_number] = ACTIONS(1555),
    [sym_get_unique_id] = ACTIONS(1555),
    [sym_get_system_number] = ACTIONS(1555),
    [sym_set_system_number] = ACTIONS(1555),
    [sym_get_serial_number] = ACTIONS(1555),
    [sym_get_master_build] = ACTIONS(1555),
    [sym_set_validation_code] = ACTIONS(1555),
    [sym_reboot] = ACTIONS(1555),
    [sym_load_duet_module] = ACTIONS(1555),
    [sym_unload_duet_module] = ACTIONS(1555),
    [sym_duet_mem_size_set] = ACTIONS(1555),
    [sym_duet_mem_size_get] = ACTIONS(1555),
    [sym_dynamic_application_device] = ACTIONS(1555),
    [sym_dynamic_polled_port] = ACTIONS(1555),
    [sym_static_port_binding] = ACTIONS(1555),
    [sym_static_ip_binding] = ACTIONS(1555),
    [sym_get_dns_list] = ACTIONS(1555),
    [sym_set_dns_list] = ACTIONS(1555),
    [sym_get_ip_address] = ACTIONS(1555),
    [sym_set_ip_address] = ACTIONS(1555),
    [sym_get_url_list] = ACTIONS(1555),
    [sym_add_url_entry] = ACTIONS(1555),
    [sym_delete_url_entry] = ACTIONS(1555),
    [sym_abs_value] = ACTIONS(1555),
    [sym_max_value] = ACTIONS(1555),
    [sym_min_value] = ACTIONS(1555),
    [sym_exp_value] = ACTIONS(1555),
    [sym_log_value] = ACTIONS(1555),
    [sym_log10_value] = ACTIONS(1555),
    [sym_power_value] = ACTIONS(1555),
    [sym_sqrt_value] = ACTIONS(1555),
    [sym_raw_be] = ACTIONS(1555),
    [sym_raw_le] = ACTIONS(1555),
    [sym_internal_queue_size_set] = ACTIONS(1555),
    [sym_internal_queue_size_get] = ACTIONS(1555),
    [sym_internal_threshold_set] = ACTIONS(1555),
    [sym_internal_threshold_get] = ACTIONS(1555),
    [sym_rebuilt_event] = ACTIONS(1555),
    [sym_do_custom_event] = ACTIONS(1555),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1555),
    [sym_clkmgr_set_clk_source] = ACTIONS(1555),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1555),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1555),
    [sym_clkmgr_get_timezone] = ACTIONS(1555),
    [sym_clkmgr_set_timezone] = ACTIONS(1555),
    [sym_clkmgr_get_resync_period] = ACTIONS(1555),
    [sym_clkmgr_set_resync_period] = ACTIONS(1555),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1555),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1555),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1555),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1555),
    [sym_clkmgr_get_timeservers] = ACTIONS(1555),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1555),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1555),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1555),
    [sym_set_log_level] = ACTIONS(1555),
    [sym_get_log_level] = ACTIONS(1555),
    [sym_amx_log] = ACTIONS(1555),
    [sym_smtp_server_config_set] = ACTIONS(1555),
    [sym_smtp_server_config_get] = ACTIONS(1555),
    [sym_smtp_send] = ACTIONS(1555),
    [sym_validate_netlinx_account] = ACTIONS(1555),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1555),
    [sym_authenticate_certificate] = ACTIONS(1555),
    [sym_audit_netlinx_session_event] = ACTIONS(1555),
    [sym_audit_netlinx_generic_event] = ACTIONS(1555),
    [sym_get_available_flash_disk_space] = ACTIONS(1555),
    [sym_get_max_flash_disk_space] = ACTIONS(1555),
    [sym_data_create_feed] = ACTIONS(1555),
    [sym_data_delete_feed] = ACTIONS(1555),
    [sym_data_publish_feed] = ACTIONS(1555),
    [sym_data_get_published_feed] = ACTIONS(1555),
    [sym_data_add_record] = ACTIONS(1555),
    [sym_data_get_event_record] = ACTIONS(1555),
    [sym__wc_data_create_feed] = ACTIONS(1555),
    [sym__wc_data_add_record] = ACTIONS(1555),
    [sym__wc_data_get_event_record] = ACTIONS(1555),
    [sym_format] = ACTIONS(1555),
    [sym_astro_clock] = ACTIONS(1555),
    [sym_set_outdoor_temperature] = ACTIONS(1555),
    [sym_set_virtual_level_count] = ACTIONS(1555),
    [sym_set_virtual_channel_count] = ACTIONS(1555),
    [sym_set_virtual_port_count] = ACTIONS(1555),
    [sym__wc_encode_int] = ACTIONS(1555),
    [sym__wc_decode_int] = ACTIONS(1555),
    [sym__wc_int] = ACTIONS(1555),
    [sym__wc_to_ch_int] = ACTIONS(1555),
    [sym__ch_to_wc_int] = ACTIONS(1555),
    [sym__wc_find_string_int] = ACTIONS(1555),
    [sym__wc_left_string_int] = ACTIONS(1555),
    [sym__wc_lower_string_int] = ACTIONS(1555),
    [sym__wc_mid_string_int] = ACTIONS(1555),
    [sym__wc_remove_string_int] = ACTIONS(1555),
    [sym__wc_right_string_int] = ACTIONS(1555),
    [sym__wc_upper_string_int] = ACTIONS(1555),
    [sym__wc_compare_string_int] = ACTIONS(1555),
    [sym__wc_get_buffer_char_int] = ACTIONS(1555),
    [sym__wc_get_buffer_string_int] = ACTIONS(1555),
    [sym__wc_concat_string_int] = ACTIONS(1555),
    [sym__wc_file_open_int] = ACTIONS(1555),
    [sym__wc_file_close_int] = ACTIONS(1555),
    [sym__wc_file_read_int] = ACTIONS(1555),
    [sym__wc_file_read_line_int] = ACTIONS(1555),
    [sym__wc_file_write_int] = ACTIONS(1555),
    [sym__wc_file_write_line_int] = ACTIONS(1555),
    [anon_sym_EQ] = ACTIONS(1555),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1557),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1557),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1557),
    [aux_sym_preproc_else_token1] = ACTIONS(1557),
    [anon_sym_COMMA] = ACTIONS(1557),
    [anon_sym_LBRACK] = ACTIONS(1557),
    [anon_sym_LBRACE] = ACTIONS(1557),
    [anon_sym_RBRACE] = ACTIONS(1557),
    [anon_sym_COLON] = ACTIONS(1557),
    [anon_sym_LPAREN] = ACTIONS(1555),
    [anon_sym_BANG] = ACTIONS(1555),
    [anon_sym_TILDE] = ACTIONS(1557),
    [anon_sym_DASH] = ACTIONS(1555),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_STAR] = ACTIONS(1557),
    [anon_sym_SLASH] = ACTIONS(1555),
    [anon_sym_PERCENT] = ACTIONS(1557),
    [anon_sym_PIPE_PIPE] = ACTIONS(1557),
    [anon_sym_AMP_AMP] = ACTIONS(1557),
    [anon_sym_PIPE] = ACTIONS(1555),
    [anon_sym_CARET] = ACTIONS(1557),
    [anon_sym_AMP] = ACTIONS(1555),
    [anon_sym_EQ_EQ] = ACTIONS(1557),
    [anon_sym_BANG_EQ] = ACTIONS(1557),
    [anon_sym_LT_GT] = ACTIONS(1557),
    [anon_sym_GT] = ACTIONS(1555),
    [anon_sym_GT_EQ] = ACTIONS(1557),
    [anon_sym_LT_EQ] = ACTIONS(1557),
    [anon_sym_LT] = ACTIONS(1555),
    [anon_sym_LT_LT] = ACTIONS(1557),
    [anon_sym_GT_GT] = ACTIONS(1557),
    [anon_sym_DASH_DASH] = ACTIONS(1557),
    [anon_sym_PLUS_PLUS] = ACTIONS(1557),
    [anon_sym_DOT] = ACTIONS(1555),
    [anon_sym_DQUOTE] = ACTIONS(1557),
    [anon_sym_SQUOTE] = ACTIONS(1557),
    [sym_number_literal] = ACTIONS(1557),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(602)] = {
    [sym_identifier] = ACTIONS(1531),
    [sym_and] = ACTIONS(1531),
    [sym_or] = ACTIONS(1531),
    [sym_not] = ACTIONS(1531),
    [sym_xor] = ACTIONS(1531),
    [sym_band] = ACTIONS(1531),
    [sym_bor] = ACTIONS(1531),
    [sym_bxor] = ACTIONS(1531),
    [sym_bnot] = ACTIONS(1531),
    [sym_lshift] = ACTIONS(1531),
    [sym_rshift] = ACTIONS(1531),
    [sym___file__] = ACTIONS(1531),
    [sym___date__] = ACTIONS(1531),
    [sym___ldate__] = ACTIONS(1531),
    [sym___time__] = ACTIONS(1531),
    [sym___line__] = ACTIONS(1531),
    [sym___name__] = ACTIONS(1531),
    [sym___netlinx__] = ACTIONS(1531),
    [sym_netlinx_axi_version] = ACTIONS(1531),
    [sym_url_flg_tcp] = ACTIONS(1531),
    [sym_url_flg_accinfopresent] = ACTIONS(1531),
    [sym_url_flg_temp] = ACTIONS(1531),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1531),
    [sym_url_flg_stat_mask] = ACTIONS(1531),
    [sym_url_flg_stat_lookup] = ACTIONS(1531),
    [sym_url_flg_stat_connecting] = ACTIONS(1531),
    [sym_url_flg_stat_waiting] = ACTIONS(1531),
    [sym_url_flg_stat_connected] = ACTIONS(1531),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1531),
    [sym_first_virtual_device] = ACTIONS(1531),
    [sym_first_local_port] = ACTIONS(1531),
    [sym_dynamic_virtual_device] = ACTIONS(1531),
    [sym_timeline_once] = ACTIONS(1531),
    [sym_timeline_repeat] = ACTIONS(1531),
    [sym_timeline_absolute] = ACTIONS(1531),
    [sym_timeline_relative] = ACTIONS(1531),
    [sym_do_push_timed_infinite] = ACTIONS(1531),
    [sym_source_type_no_address] = ACTIONS(1531),
    [sym_source_type_neuron_id] = ACTIONS(1531),
    [sym_source_type_ip_address] = ACTIONS(1531),
    [sym_source_type_axlink] = ACTIONS(1531),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1531),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1531),
    [sym_source_type_ip_socket_address] = ACTIONS(1531),
    [sym_source_type_rs232] = ACTIONS(1531),
    [sym_source_type_internal] = ACTIONS(1531),
    [sym_source_type_mac_address] = ACTIONS(1531),
    [sym_source_type_ipv4_port] = ACTIONS(1531),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1531),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1531),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1531),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1531),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1531),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1531),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1531),
    [sym_true] = ACTIONS(1531),
    [sym_false] = ACTIONS(1531),
    [sym_file_read_only] = ACTIONS(1531),
    [sym_file_rw_new] = ACTIONS(1531),
    [sym_file_rw_append] = ACTIONS(1531),
    [sym_ip_tcp] = ACTIONS(1531),
    [sym_ip_udp] = ACTIONS(1531),
    [sym_ip_udp_2way] = ACTIONS(1531),
    [sym_xml_encode_types] = ACTIONS(1531),
    [sym_xml_encode_char_as_list] = ACTIONS(1531),
    [sym_xml_encode_le] = ACTIONS(1531),
    [sym_xml_decode_types] = ACTIONS(1531),
    [sym_xml_decode_no_preserve] = ACTIONS(1531),
    [sym_duet_dev_type_display_device] = ACTIONS(1531),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1531),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1531),
    [sym_duet_dev_type_io_device] = ACTIONS(1531),
    [sym_duet_dev_type_relay_device] = ACTIONS(1531),
    [sym_duet_dev_type_ups] = ACTIONS(1531),
    [sym_duet_dev_type_amplifier] = ACTIONS(1531),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1531),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1531),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1531),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1531),
    [sym_duet_dev_type_camera] = ACTIONS(1531),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1531),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1531),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1531),
    [sym_duet_dev_type_dss] = ACTIONS(1531),
    [sym_duet_dev_type_dvr] = ACTIONS(1531),
    [sym_duet_dev_type_disc_device] = ACTIONS(1531),
    [sym_duet_dev_type_document_camera] = ACTIONS(1531),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1531),
    [sym_duet_dev_type_hvac] = ACTIONS(1531),
    [sym_duet_dev_type_keypad] = ACTIONS(1531),
    [sym_duet_dev_type_light] = ACTIONS(1531),
    [sym_duet_dev_type_monitor] = ACTIONS(1531),
    [sym_duet_dev_type_motor] = ACTIONS(1531),
    [sym_duet_dev_type_multi_window] = ACTIONS(1531),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1531),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1531),
    [sym_duet_dev_type_receiver] = ACTIONS(1531),
    [sym_duet_dev_type_security_system] = ACTIONS(1531),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1531),
    [sym_duet_dev_type_settop_box] = ACTIONS(1531),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1531),
    [sym_duet_dev_type_switcher] = ACTIONS(1531),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1531),
    [sym_duet_dev_type_tv] = ACTIONS(1531),
    [sym_duet_dev_type_utility] = ACTIONS(1531),
    [sym_duet_dev_type_vcr] = ACTIONS(1531),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1531),
    [sym_duet_dev_type_video_processor] = ACTIONS(1531),
    [sym_duet_dev_type_video_projector] = ACTIONS(1531),
    [sym_duet_dev_type_video_wall] = ACTIONS(1531),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1531),
    [sym_duet_dev_type_weather] = ACTIONS(1531),
    [sym_duet_dev_not_polled] = ACTIONS(1531),
    [sym_duet_dev_polled] = ACTIONS(1531),
    [sym_clkmgr_mode_network] = ACTIONS(1531),
    [sym_clkmgr_mode_standalone] = ACTIONS(1531),
    [sym_normal_standby] = ACTIONS(1531),
    [sym_normal_wake] = ACTIONS(1531),
    [sym_ip_multicast_ttl_option] = ACTIONS(1531),
    [sym_ip_ttl_subnet] = ACTIONS(1531),
    [sym_ip_ttl_site] = ACTIONS(1531),
    [sym_ip_ttl_region] = ACTIONS(1531),
    [sym_ip_ttl_continent] = ACTIONS(1531),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1531),
    [sym_ip_nodelay_on] = ACTIONS(1531),
    [sym_ip_nodelay_off] = ACTIONS(1531),
    [sym_amx_error] = ACTIONS(1531),
    [sym_amx_warning] = ACTIONS(1531),
    [sym_amx_info] = ACTIONS(1531),
    [sym_amx_debug] = ACTIONS(1531),
    [sym_netlinx_login_success] = ACTIONS(1531),
    [sym_netlinx_login_fail] = ACTIONS(1531),
    [sym_netlinx_logout] = ACTIONS(1531),
    [sym_null_str] = ACTIONS(1531),
    [sym_smtp_address] = ACTIONS(1531),
    [sym_smtp_port_number] = ACTIONS(1531),
    [sym_smtp_username] = ACTIONS(1531),
    [sym_smtp_password] = ACTIONS(1531),
    [sym_smtp_require_tls] = ACTIONS(1531),
    [sym_smtp_from] = ACTIONS(1531),
    [sym_smtp_tls_true] = ACTIONS(1531),
    [sym_smtp_tls_false] = ACTIONS(1531),
    [sym_listview_on_row_select_event] = ACTIONS(1531),
    [sym_data_structure_unknown] = ACTIONS(1531),
    [sym_data_structure_datafeed] = ACTIONS(1531),
    [sym_data_structure_datarecordset] = ACTIONS(1531),
    [sym_data_structure_datarecord] = ACTIONS(1531),
    [sym_data_structure_datafield] = ACTIONS(1531),
    [sym_data_max_content_field_count] = ACTIONS(1531),
    [sym_data_max_metadata_field_count] = ACTIONS(1531),
    [sym_data_max_id_length] = ACTIONS(1531),
    [sym_data_max_name_length] = ACTIONS(1531),
    [sym_data_max_description_length] = ACTIONS(1531),
    [sym_data_max_type_length] = ACTIONS(1531),
    [sym_data_max_format_length] = ACTIONS(1531),
    [sym_data_max_source_length] = ACTIONS(1531),
    [sym_data_max_value_length] = ACTIONS(1531),
    [sym_data_max_label_length] = ACTIONS(1531),
    [sym_data_max_url_length] = ACTIONS(1531),
    [sym_max_last_login_info_length] = ACTIONS(1531),
    [sym_data_type_unknown] = ACTIONS(1531),
    [sym_data_type_string] = ACTIONS(1531),
    [sym_data_type_datetime] = ACTIONS(1531),
    [sym_data_type_date] = ACTIONS(1531),
    [sym_data_type_time] = ACTIONS(1531),
    [sym_data_type_image] = ACTIONS(1531),
    [sym_data_format_url] = ACTIONS(1531),
    [sym_data_format_phone] = ACTIONS(1531),
    [sym_data_format_email] = ACTIONS(1531),
    [sym_data_format_iso8601] = ACTIONS(1531),
    [sym_auditlog_priv] = ACTIONS(1531),
    [sym_configuration_priv] = ACTIONS(1531),
    [sym_deviceconfig_priv] = ACTIONS(1531),
    [sym_ftp_priv] = ACTIONS(1531),
    [sym_http_priv] = ACTIONS(1531),
    [sym_networkconfig_priv] = ACTIONS(1531),
    [sym_programport_priv] = ACTIONS(1531),
    [sym_remoteui_priv] = ACTIONS(1531),
    [sym_securitycontrol_priv] = ACTIONS(1531),
    [sym_softwaremanagement_priv] = ACTIONS(1531),
    [sym_terminal_priv] = ACTIONS(1531),
    [sym_tpadmin_priv] = ACTIONS(1531),
    [sym_user1_priv] = ACTIONS(1531),
    [sym_user2_priv] = ACTIONS(1531),
    [sym_user3_priv] = ACTIONS(1531),
    [sym_user4_priv] = ACTIONS(1531),
    [sym_usermanagement_priv] = ACTIONS(1531),
    [sym_valid_account] = ACTIONS(1531),
    [sym_err_name_invalid] = ACTIONS(1531),
    [sym_err_password_invalid] = ACTIONS(1531),
    [sym_err_invalid_account] = ACTIONS(1531),
    [sym_err_no_authorization] = ACTIONS(1531),
    [sym_err_invalid_parameter] = ACTIONS(1531),
    [sym_err_user_locked_out] = ACTIONS(1531),
    [sym_err_user_account_expired] = ACTIONS(1531),
    [sym_err_general_error] = ACTIONS(1531),
    [sym_err_authentication_error] = ACTIONS(1531),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1531),
    [sym_tls_validate_certificate] = ACTIONS(1531),
    [sym____reserved___] = ACTIONS(1531),
    [sym_date] = ACTIONS(1531),
    [sym_day] = ACTIONS(1531),
    [sym_time] = ACTIONS(1531),
    [sym_ldate] = ACTIONS(1531),
    [sym_push_device] = ACTIONS(1531),
    [sym_push_channel] = ACTIONS(1531),
    [sym_push_devchan] = ACTIONS(1531),
    [sym_release_device] = ACTIONS(1531),
    [sym_release_channel] = ACTIONS(1531),
    [sym_release_devchan] = ACTIONS(1531),
    [sym_master_slot] = ACTIONS(1531),
    [sym_get_pulse_time] = ACTIONS(1531),
    [sym_get_timer] = ACTIONS(1531),
    [sym_master_sn] = ACTIONS(1531),
    [sym_system_number] = ACTIONS(1531),
    [sym_button] = ACTIONS(1531),
    [sym_dv_channel] = ACTIONS(1531),
    [sym_data] = ACTIONS(1531),
    [sym_level] = ACTIONS(1531),
    [sym_timeline] = ACTIONS(1531),
    [sym_channel] = ACTIONS(1531),
    [sym_custom] = ACTIONS(1531),
    [sym_atoi] = ACTIONS(1531),
    [sym_atol] = ACTIONS(1531),
    [sym_atof] = ACTIONS(1531),
    [sym_itoa] = ACTIONS(1531),
    [sym_itohex] = ACTIONS(1531),
    [sym_hextoi] = ACTIONS(1531),
    [sym_ftoa] = ACTIONS(1531),
    [sym_find_string] = ACTIONS(1531),
    [sym_left_string] = ACTIONS(1531),
    [sym_right_string] = ACTIONS(1531),
    [sym_mid_string] = ACTIONS(1531),
    [sym_remove_string] = ACTIONS(1531),
    [sym_get_buffer_string] = ACTIONS(1531),
    [sym_get_buffer_char] = ACTIONS(1531),
    [sym_get_multi_buffer_string] = ACTIONS(1531),
    [sym_length_string] = ACTIONS(1531),
    [sym_max_length_string] = ACTIONS(1531),
    [sym_set_length_string] = ACTIONS(1531),
    [sym_compare_string] = ACTIONS(1531),
    [sym_upper_string] = ACTIONS(1531),
    [sym_lower_string] = ACTIONS(1531),
    [sym_get_last] = ACTIONS(1531),
    [sym_date_to_day] = ACTIONS(1531),
    [sym_date_to_month] = ACTIONS(1531),
    [sym_date_to_year] = ACTIONS(1531),
    [sym_time_to_hour] = ACTIONS(1531),
    [sym_time_to_minute] = ACTIONS(1531),
    [sym_time_to_second] = ACTIONS(1531),
    [sym_day_of_week] = ACTIONS(1531),
    [sym_set_pulse_time] = ACTIONS(1531),
    [sym_set_timer] = ACTIONS(1531),
    [sym_file_open] = ACTIONS(1531),
    [sym_file_close] = ACTIONS(1531),
    [sym_file_read] = ACTIONS(1531),
    [sym_file_read_line] = ACTIONS(1531),
    [sym_file_write] = ACTIONS(1531),
    [sym_file_write_line] = ACTIONS(1531),
    [sym_file_copy] = ACTIONS(1531),
    [sym_file_delete] = ACTIONS(1531),
    [sym_file_rename] = ACTIONS(1531),
    [sym_file_dir] = ACTIONS(1531),
    [sym_file_getdir] = ACTIONS(1531),
    [sym_file_setdir] = ACTIONS(1531),
    [sym_file_create_dir] = ACTIONS(1531),
    [sym_file_remove_dir] = ACTIONS(1531),
    [sym_file_seek] = ACTIONS(1531),
    [sym_length_array] = ACTIONS(1531),
    [sym_max_length_array] = ACTIONS(1531),
    [sym_set_length_array] = ACTIONS(1531),
    [sym_ip_client_open] = ACTIONS(1531),
    [sym_ip_client_close] = ACTIONS(1531),
    [sym_ip_server_open] = ACTIONS(1531),
    [sym_ip_server_close] = ACTIONS(1531),
    [sym_ip_bound_client_open] = ACTIONS(1531),
    [sym_ip_mc_server_open] = ACTIONS(1531),
    [sym_ip_set_option] = ACTIONS(1531),
    [sym_ssh_client_open] = ACTIONS(1531),
    [sym_ssh_client_close] = ACTIONS(1531),
    [sym_tls_client_open] = ACTIONS(1531),
    [sym_tls_client_close] = ACTIONS(1531),
    [sym_do_push] = ACTIONS(1531),
    [sym_do_push_timed] = ACTIONS(1531),
    [sym_do_release] = ACTIONS(1531),
    [sym_device_id] = ACTIONS(1531),
    [sym_device_info] = ACTIONS(1531),
    [sym_device_id_string] = ACTIONS(1531),
    [sym_device_standby] = ACTIONS(1531),
    [sym_device_wake] = ACTIONS(1531),
    [sym_timeline_create] = ACTIONS(1531),
    [sym_timeline_kill] = ACTIONS(1531),
    [sym_timeline_reload] = ACTIONS(1531),
    [sym_timeline_pause] = ACTIONS(1531),
    [sym_timeline_restart] = ACTIONS(1531),
    [sym_timeline_set] = ACTIONS(1531),
    [sym_timeline_get] = ACTIONS(1531),
    [sym_timeline_active] = ACTIONS(1531),
    [sym_timeline_dynamic_id] = ACTIONS(1531),
    [sym_variable_to_string] = ACTIONS(1531),
    [sym_string_to_variable] = ACTIONS(1531),
    [sym_length_variable_to_string] = ACTIONS(1531),
    [sym_type_cast] = ACTIONS(1531),
    [sym_variable_to_xml] = ACTIONS(1531),
    [sym_xml_to_variable] = ACTIONS(1531),
    [sym_length_variable_to_xml] = ACTIONS(1531),
    [sym_redirect_string] = ACTIONS(1531),
    [sym_random_number] = ACTIONS(1531),
    [sym_get_unique_id] = ACTIONS(1531),
    [sym_get_system_number] = ACTIONS(1531),
    [sym_set_system_number] = ACTIONS(1531),
    [sym_get_serial_number] = ACTIONS(1531),
    [sym_get_master_build] = ACTIONS(1531),
    [sym_set_validation_code] = ACTIONS(1531),
    [sym_reboot] = ACTIONS(1531),
    [sym_load_duet_module] = ACTIONS(1531),
    [sym_unload_duet_module] = ACTIONS(1531),
    [sym_duet_mem_size_set] = ACTIONS(1531),
    [sym_duet_mem_size_get] = ACTIONS(1531),
    [sym_dynamic_application_device] = ACTIONS(1531),
    [sym_dynamic_polled_port] = ACTIONS(1531),
    [sym_static_port_binding] = ACTIONS(1531),
    [sym_static_ip_binding] = ACTIONS(1531),
    [sym_get_dns_list] = ACTIONS(1531),
    [sym_set_dns_list] = ACTIONS(1531),
    [sym_get_ip_address] = ACTIONS(1531),
    [sym_set_ip_address] = ACTIONS(1531),
    [sym_get_url_list] = ACTIONS(1531),
    [sym_add_url_entry] = ACTIONS(1531),
    [sym_delete_url_entry] = ACTIONS(1531),
    [sym_abs_value] = ACTIONS(1531),
    [sym_max_value] = ACTIONS(1531),
    [sym_min_value] = ACTIONS(1531),
    [sym_exp_value] = ACTIONS(1531),
    [sym_log_value] = ACTIONS(1531),
    [sym_log10_value] = ACTIONS(1531),
    [sym_power_value] = ACTIONS(1531),
    [sym_sqrt_value] = ACTIONS(1531),
    [sym_raw_be] = ACTIONS(1531),
    [sym_raw_le] = ACTIONS(1531),
    [sym_internal_queue_size_set] = ACTIONS(1531),
    [sym_internal_queue_size_get] = ACTIONS(1531),
    [sym_internal_threshold_set] = ACTIONS(1531),
    [sym_internal_threshold_get] = ACTIONS(1531),
    [sym_rebuilt_event] = ACTIONS(1531),
    [sym_do_custom_event] = ACTIONS(1531),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1531),
    [sym_clkmgr_set_clk_source] = ACTIONS(1531),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1531),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1531),
    [sym_clkmgr_get_timezone] = ACTIONS(1531),
    [sym_clkmgr_set_timezone] = ACTIONS(1531),
    [sym_clkmgr_get_resync_period] = ACTIONS(1531),
    [sym_clkmgr_set_resync_period] = ACTIONS(1531),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1531),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1531),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1531),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1531),
    [sym_clkmgr_get_timeservers] = ACTIONS(1531),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1531),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1531),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1531),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1531),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1531),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1531),
    [sym_set_log_level] = ACTIONS(1531),
    [sym_get_log_level] = ACTIONS(1531),
    [sym_amx_log] = ACTIONS(1531),
    [sym_smtp_server_config_set] = ACTIONS(1531),
    [sym_smtp_server_config_get] = ACTIONS(1531),
    [sym_smtp_send] = ACTIONS(1531),
    [sym_validate_netlinx_account] = ACTIONS(1531),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1531),
    [sym_authenticate_certificate] = ACTIONS(1531),
    [sym_audit_netlinx_session_event] = ACTIONS(1531),
    [sym_audit_netlinx_generic_event] = ACTIONS(1531),
    [sym_get_available_flash_disk_space] = ACTIONS(1531),
    [sym_get_max_flash_disk_space] = ACTIONS(1531),
    [sym_data_create_feed] = ACTIONS(1531),
    [sym_data_delete_feed] = ACTIONS(1531),
    [sym_data_publish_feed] = ACTIONS(1531),
    [sym_data_get_published_feed] = ACTIONS(1531),
    [sym_data_add_record] = ACTIONS(1531),
    [sym_data_get_event_record] = ACTIONS(1531),
    [sym__wc_data_create_feed] = ACTIONS(1531),
    [sym__wc_data_add_record] = ACTIONS(1531),
    [sym__wc_data_get_event_record] = ACTIONS(1531),
    [sym_format] = ACTIONS(1531),
    [sym_astro_clock] = ACTIONS(1531),
    [sym_set_outdoor_temperature] = ACTIONS(1531),
    [sym_set_virtual_level_count] = ACTIONS(1531),
    [sym_set_virtual_channel_count] = ACTIONS(1531),
    [sym_set_virtual_port_count] = ACTIONS(1531),
    [sym__wc_encode_int] = ACTIONS(1531),
    [sym__wc_decode_int] = ACTIONS(1531),
    [sym__wc_int] = ACTIONS(1531),
    [sym__wc_to_ch_int] = ACTIONS(1531),
    [sym__ch_to_wc_int] = ACTIONS(1531),
    [sym__wc_find_string_int] = ACTIONS(1531),
    [sym__wc_left_string_int] = ACTIONS(1531),
    [sym__wc_lower_string_int] = ACTIONS(1531),
    [sym__wc_mid_string_int] = ACTIONS(1531),
    [sym__wc_remove_string_int] = ACTIONS(1531),
    [sym__wc_right_string_int] = ACTIONS(1531),
    [sym__wc_upper_string_int] = ACTIONS(1531),
    [sym__wc_compare_string_int] = ACTIONS(1531),
    [sym__wc_get_buffer_char_int] = ACTIONS(1531),
    [sym__wc_get_buffer_string_int] = ACTIONS(1531),
    [sym__wc_concat_string_int] = ACTIONS(1531),
    [sym__wc_file_open_int] = ACTIONS(1531),
    [sym__wc_file_close_int] = ACTIONS(1531),
    [sym__wc_file_read_int] = ACTIONS(1531),
    [sym__wc_file_read_line_int] = ACTIONS(1531),
    [sym__wc_file_write_int] = ACTIONS(1531),
    [sym__wc_file_write_line_int] = ACTIONS(1531),
    [anon_sym_EQ] = ACTIONS(1531),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1533),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1533),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1533),
    [aux_sym_preproc_else_token1] = ACTIONS(1533),
    [anon_sym_COMMA] = ACTIONS(1533),
    [anon_sym_LBRACK] = ACTIONS(1533),
    [anon_sym_LBRACE] = ACTIONS(1533),
    [anon_sym_RBRACE] = ACTIONS(1533),
    [anon_sym_COLON] = ACTIONS(1533),
    [anon_sym_LPAREN] = ACTIONS(1531),
    [anon_sym_BANG] = ACTIONS(1531),
    [anon_sym_TILDE] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(1533),
    [anon_sym_SLASH] = ACTIONS(1531),
    [anon_sym_PERCENT] = ACTIONS(1533),
    [anon_sym_PIPE_PIPE] = ACTIONS(1533),
    [anon_sym_AMP_AMP] = ACTIONS(1533),
    [anon_sym_PIPE] = ACTIONS(1531),
    [anon_sym_CARET] = ACTIONS(1533),
    [anon_sym_AMP] = ACTIONS(1531),
    [anon_sym_EQ_EQ] = ACTIONS(1533),
    [anon_sym_BANG_EQ] = ACTIONS(1533),
    [anon_sym_LT_GT] = ACTIONS(1533),
    [anon_sym_GT] = ACTIONS(1531),
    [anon_sym_GT_EQ] = ACTIONS(1533),
    [anon_sym_LT_EQ] = ACTIONS(1533),
    [anon_sym_LT] = ACTIONS(1531),
    [anon_sym_LT_LT] = ACTIONS(1533),
    [anon_sym_GT_GT] = ACTIONS(1533),
    [anon_sym_DASH_DASH] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1533),
    [anon_sym_DOT] = ACTIONS(1531),
    [anon_sym_DQUOTE] = ACTIONS(1533),
    [anon_sym_SQUOTE] = ACTIONS(1533),
    [sym_number_literal] = ACTIONS(1533),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(603)] = {
    [sym_identifier] = ACTIONS(1535),
    [sym_and] = ACTIONS(1535),
    [sym_or] = ACTIONS(1535),
    [sym_not] = ACTIONS(1535),
    [sym_xor] = ACTIONS(1535),
    [sym_band] = ACTIONS(1535),
    [sym_bor] = ACTIONS(1535),
    [sym_bxor] = ACTIONS(1535),
    [sym_bnot] = ACTIONS(1535),
    [sym_lshift] = ACTIONS(1535),
    [sym_rshift] = ACTIONS(1535),
    [sym___file__] = ACTIONS(1535),
    [sym___date__] = ACTIONS(1535),
    [sym___ldate__] = ACTIONS(1535),
    [sym___time__] = ACTIONS(1535),
    [sym___line__] = ACTIONS(1535),
    [sym___name__] = ACTIONS(1535),
    [sym___netlinx__] = ACTIONS(1535),
    [sym_netlinx_axi_version] = ACTIONS(1535),
    [sym_url_flg_tcp] = ACTIONS(1535),
    [sym_url_flg_accinfopresent] = ACTIONS(1535),
    [sym_url_flg_temp] = ACTIONS(1535),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1535),
    [sym_url_flg_stat_mask] = ACTIONS(1535),
    [sym_url_flg_stat_lookup] = ACTIONS(1535),
    [sym_url_flg_stat_connecting] = ACTIONS(1535),
    [sym_url_flg_stat_waiting] = ACTIONS(1535),
    [sym_url_flg_stat_connected] = ACTIONS(1535),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1535),
    [sym_first_virtual_device] = ACTIONS(1535),
    [sym_first_local_port] = ACTIONS(1535),
    [sym_dynamic_virtual_device] = ACTIONS(1535),
    [sym_timeline_once] = ACTIONS(1535),
    [sym_timeline_repeat] = ACTIONS(1535),
    [sym_timeline_absolute] = ACTIONS(1535),
    [sym_timeline_relative] = ACTIONS(1535),
    [sym_do_push_timed_infinite] = ACTIONS(1535),
    [sym_source_type_no_address] = ACTIONS(1535),
    [sym_source_type_neuron_id] = ACTIONS(1535),
    [sym_source_type_ip_address] = ACTIONS(1535),
    [sym_source_type_axlink] = ACTIONS(1535),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1535),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1535),
    [sym_source_type_ip_socket_address] = ACTIONS(1535),
    [sym_source_type_rs232] = ACTIONS(1535),
    [sym_source_type_internal] = ACTIONS(1535),
    [sym_source_type_mac_address] = ACTIONS(1535),
    [sym_source_type_ipv4_port] = ACTIONS(1535),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1535),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1535),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1535),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1535),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1535),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1535),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1535),
    [sym_true] = ACTIONS(1535),
    [sym_false] = ACTIONS(1535),
    [sym_file_read_only] = ACTIONS(1535),
    [sym_file_rw_new] = ACTIONS(1535),
    [sym_file_rw_append] = ACTIONS(1535),
    [sym_ip_tcp] = ACTIONS(1535),
    [sym_ip_udp] = ACTIONS(1535),
    [sym_ip_udp_2way] = ACTIONS(1535),
    [sym_xml_encode_types] = ACTIONS(1535),
    [sym_xml_encode_char_as_list] = ACTIONS(1535),
    [sym_xml_encode_le] = ACTIONS(1535),
    [sym_xml_decode_types] = ACTIONS(1535),
    [sym_xml_decode_no_preserve] = ACTIONS(1535),
    [sym_duet_dev_type_display_device] = ACTIONS(1535),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1535),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1535),
    [sym_duet_dev_type_io_device] = ACTIONS(1535),
    [sym_duet_dev_type_relay_device] = ACTIONS(1535),
    [sym_duet_dev_type_ups] = ACTIONS(1535),
    [sym_duet_dev_type_amplifier] = ACTIONS(1535),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1535),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1535),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1535),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1535),
    [sym_duet_dev_type_camera] = ACTIONS(1535),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1535),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1535),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1535),
    [sym_duet_dev_type_dss] = ACTIONS(1535),
    [sym_duet_dev_type_dvr] = ACTIONS(1535),
    [sym_duet_dev_type_disc_device] = ACTIONS(1535),
    [sym_duet_dev_type_document_camera] = ACTIONS(1535),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1535),
    [sym_duet_dev_type_hvac] = ACTIONS(1535),
    [sym_duet_dev_type_keypad] = ACTIONS(1535),
    [sym_duet_dev_type_light] = ACTIONS(1535),
    [sym_duet_dev_type_monitor] = ACTIONS(1535),
    [sym_duet_dev_type_motor] = ACTIONS(1535),
    [sym_duet_dev_type_multi_window] = ACTIONS(1535),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1535),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1535),
    [sym_duet_dev_type_receiver] = ACTIONS(1535),
    [sym_duet_dev_type_security_system] = ACTIONS(1535),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1535),
    [sym_duet_dev_type_settop_box] = ACTIONS(1535),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1535),
    [sym_duet_dev_type_switcher] = ACTIONS(1535),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1535),
    [sym_duet_dev_type_tv] = ACTIONS(1535),
    [sym_duet_dev_type_utility] = ACTIONS(1535),
    [sym_duet_dev_type_vcr] = ACTIONS(1535),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1535),
    [sym_duet_dev_type_video_processor] = ACTIONS(1535),
    [sym_duet_dev_type_video_projector] = ACTIONS(1535),
    [sym_duet_dev_type_video_wall] = ACTIONS(1535),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1535),
    [sym_duet_dev_type_weather] = ACTIONS(1535),
    [sym_duet_dev_not_polled] = ACTIONS(1535),
    [sym_duet_dev_polled] = ACTIONS(1535),
    [sym_clkmgr_mode_network] = ACTIONS(1535),
    [sym_clkmgr_mode_standalone] = ACTIONS(1535),
    [sym_normal_standby] = ACTIONS(1535),
    [sym_normal_wake] = ACTIONS(1535),
    [sym_ip_multicast_ttl_option] = ACTIONS(1535),
    [sym_ip_ttl_subnet] = ACTIONS(1535),
    [sym_ip_ttl_site] = ACTIONS(1535),
    [sym_ip_ttl_region] = ACTIONS(1535),
    [sym_ip_ttl_continent] = ACTIONS(1535),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1535),
    [sym_ip_nodelay_on] = ACTIONS(1535),
    [sym_ip_nodelay_off] = ACTIONS(1535),
    [sym_amx_error] = ACTIONS(1535),
    [sym_amx_warning] = ACTIONS(1535),
    [sym_amx_info] = ACTIONS(1535),
    [sym_amx_debug] = ACTIONS(1535),
    [sym_netlinx_login_success] = ACTIONS(1535),
    [sym_netlinx_login_fail] = ACTIONS(1535),
    [sym_netlinx_logout] = ACTIONS(1535),
    [sym_null_str] = ACTIONS(1535),
    [sym_smtp_address] = ACTIONS(1535),
    [sym_smtp_port_number] = ACTIONS(1535),
    [sym_smtp_username] = ACTIONS(1535),
    [sym_smtp_password] = ACTIONS(1535),
    [sym_smtp_require_tls] = ACTIONS(1535),
    [sym_smtp_from] = ACTIONS(1535),
    [sym_smtp_tls_true] = ACTIONS(1535),
    [sym_smtp_tls_false] = ACTIONS(1535),
    [sym_listview_on_row_select_event] = ACTIONS(1535),
    [sym_data_structure_unknown] = ACTIONS(1535),
    [sym_data_structure_datafeed] = ACTIONS(1535),
    [sym_data_structure_datarecordset] = ACTIONS(1535),
    [sym_data_structure_datarecord] = ACTIONS(1535),
    [sym_data_structure_datafield] = ACTIONS(1535),
    [sym_data_max_content_field_count] = ACTIONS(1535),
    [sym_data_max_metadata_field_count] = ACTIONS(1535),
    [sym_data_max_id_length] = ACTIONS(1535),
    [sym_data_max_name_length] = ACTIONS(1535),
    [sym_data_max_description_length] = ACTIONS(1535),
    [sym_data_max_type_length] = ACTIONS(1535),
    [sym_data_max_format_length] = ACTIONS(1535),
    [sym_data_max_source_length] = ACTIONS(1535),
    [sym_data_max_value_length] = ACTIONS(1535),
    [sym_data_max_label_length] = ACTIONS(1535),
    [sym_data_max_url_length] = ACTIONS(1535),
    [sym_max_last_login_info_length] = ACTIONS(1535),
    [sym_data_type_unknown] = ACTIONS(1535),
    [sym_data_type_string] = ACTIONS(1535),
    [sym_data_type_datetime] = ACTIONS(1535),
    [sym_data_type_date] = ACTIONS(1535),
    [sym_data_type_time] = ACTIONS(1535),
    [sym_data_type_image] = ACTIONS(1535),
    [sym_data_format_url] = ACTIONS(1535),
    [sym_data_format_phone] = ACTIONS(1535),
    [sym_data_format_email] = ACTIONS(1535),
    [sym_data_format_iso8601] = ACTIONS(1535),
    [sym_auditlog_priv] = ACTIONS(1535),
    [sym_configuration_priv] = ACTIONS(1535),
    [sym_deviceconfig_priv] = ACTIONS(1535),
    [sym_ftp_priv] = ACTIONS(1535),
    [sym_http_priv] = ACTIONS(1535),
    [sym_networkconfig_priv] = ACTIONS(1535),
    [sym_programport_priv] = ACTIONS(1535),
    [sym_remoteui_priv] = ACTIONS(1535),
    [sym_securitycontrol_priv] = ACTIONS(1535),
    [sym_softwaremanagement_priv] = ACTIONS(1535),
    [sym_terminal_priv] = ACTIONS(1535),
    [sym_tpadmin_priv] = ACTIONS(1535),
    [sym_user1_priv] = ACTIONS(1535),
    [sym_user2_priv] = ACTIONS(1535),
    [sym_user3_priv] = ACTIONS(1535),
    [sym_user4_priv] = ACTIONS(1535),
    [sym_usermanagement_priv] = ACTIONS(1535),
    [sym_valid_account] = ACTIONS(1535),
    [sym_err_name_invalid] = ACTIONS(1535),
    [sym_err_password_invalid] = ACTIONS(1535),
    [sym_err_invalid_account] = ACTIONS(1535),
    [sym_err_no_authorization] = ACTIONS(1535),
    [sym_err_invalid_parameter] = ACTIONS(1535),
    [sym_err_user_locked_out] = ACTIONS(1535),
    [sym_err_user_account_expired] = ACTIONS(1535),
    [sym_err_general_error] = ACTIONS(1535),
    [sym_err_authentication_error] = ACTIONS(1535),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1535),
    [sym_tls_validate_certificate] = ACTIONS(1535),
    [sym____reserved___] = ACTIONS(1535),
    [sym_date] = ACTIONS(1535),
    [sym_day] = ACTIONS(1535),
    [sym_time] = ACTIONS(1535),
    [sym_ldate] = ACTIONS(1535),
    [sym_push_device] = ACTIONS(1535),
    [sym_push_channel] = ACTIONS(1535),
    [sym_push_devchan] = ACTIONS(1535),
    [sym_release_device] = ACTIONS(1535),
    [sym_release_channel] = ACTIONS(1535),
    [sym_release_devchan] = ACTIONS(1535),
    [sym_master_slot] = ACTIONS(1535),
    [sym_get_pulse_time] = ACTIONS(1535),
    [sym_get_timer] = ACTIONS(1535),
    [sym_master_sn] = ACTIONS(1535),
    [sym_system_number] = ACTIONS(1535),
    [sym_button] = ACTIONS(1535),
    [sym_dv_channel] = ACTIONS(1535),
    [sym_data] = ACTIONS(1535),
    [sym_level] = ACTIONS(1535),
    [sym_timeline] = ACTIONS(1535),
    [sym_channel] = ACTIONS(1535),
    [sym_custom] = ACTIONS(1535),
    [sym_atoi] = ACTIONS(1535),
    [sym_atol] = ACTIONS(1535),
    [sym_atof] = ACTIONS(1535),
    [sym_itoa] = ACTIONS(1535),
    [sym_itohex] = ACTIONS(1535),
    [sym_hextoi] = ACTIONS(1535),
    [sym_ftoa] = ACTIONS(1535),
    [sym_find_string] = ACTIONS(1535),
    [sym_left_string] = ACTIONS(1535),
    [sym_right_string] = ACTIONS(1535),
    [sym_mid_string] = ACTIONS(1535),
    [sym_remove_string] = ACTIONS(1535),
    [sym_get_buffer_string] = ACTIONS(1535),
    [sym_get_buffer_char] = ACTIONS(1535),
    [sym_get_multi_buffer_string] = ACTIONS(1535),
    [sym_length_string] = ACTIONS(1535),
    [sym_max_length_string] = ACTIONS(1535),
    [sym_set_length_string] = ACTIONS(1535),
    [sym_compare_string] = ACTIONS(1535),
    [sym_upper_string] = ACTIONS(1535),
    [sym_lower_string] = ACTIONS(1535),
    [sym_get_last] = ACTIONS(1535),
    [sym_date_to_day] = ACTIONS(1535),
    [sym_date_to_month] = ACTIONS(1535),
    [sym_date_to_year] = ACTIONS(1535),
    [sym_time_to_hour] = ACTIONS(1535),
    [sym_time_to_minute] = ACTIONS(1535),
    [sym_time_to_second] = ACTIONS(1535),
    [sym_day_of_week] = ACTIONS(1535),
    [sym_set_pulse_time] = ACTIONS(1535),
    [sym_set_timer] = ACTIONS(1535),
    [sym_file_open] = ACTIONS(1535),
    [sym_file_close] = ACTIONS(1535),
    [sym_file_read] = ACTIONS(1535),
    [sym_file_read_line] = ACTIONS(1535),
    [sym_file_write] = ACTIONS(1535),
    [sym_file_write_line] = ACTIONS(1535),
    [sym_file_copy] = ACTIONS(1535),
    [sym_file_delete] = ACTIONS(1535),
    [sym_file_rename] = ACTIONS(1535),
    [sym_file_dir] = ACTIONS(1535),
    [sym_file_getdir] = ACTIONS(1535),
    [sym_file_setdir] = ACTIONS(1535),
    [sym_file_create_dir] = ACTIONS(1535),
    [sym_file_remove_dir] = ACTIONS(1535),
    [sym_file_seek] = ACTIONS(1535),
    [sym_length_array] = ACTIONS(1535),
    [sym_max_length_array] = ACTIONS(1535),
    [sym_set_length_array] = ACTIONS(1535),
    [sym_ip_client_open] = ACTIONS(1535),
    [sym_ip_client_close] = ACTIONS(1535),
    [sym_ip_server_open] = ACTIONS(1535),
    [sym_ip_server_close] = ACTIONS(1535),
    [sym_ip_bound_client_open] = ACTIONS(1535),
    [sym_ip_mc_server_open] = ACTIONS(1535),
    [sym_ip_set_option] = ACTIONS(1535),
    [sym_ssh_client_open] = ACTIONS(1535),
    [sym_ssh_client_close] = ACTIONS(1535),
    [sym_tls_client_open] = ACTIONS(1535),
    [sym_tls_client_close] = ACTIONS(1535),
    [sym_do_push] = ACTIONS(1535),
    [sym_do_push_timed] = ACTIONS(1535),
    [sym_do_release] = ACTIONS(1535),
    [sym_device_id] = ACTIONS(1535),
    [sym_device_info] = ACTIONS(1535),
    [sym_device_id_string] = ACTIONS(1535),
    [sym_device_standby] = ACTIONS(1535),
    [sym_device_wake] = ACTIONS(1535),
    [sym_timeline_create] = ACTIONS(1535),
    [sym_timeline_kill] = ACTIONS(1535),
    [sym_timeline_reload] = ACTIONS(1535),
    [sym_timeline_pause] = ACTIONS(1535),
    [sym_timeline_restart] = ACTIONS(1535),
    [sym_timeline_set] = ACTIONS(1535),
    [sym_timeline_get] = ACTIONS(1535),
    [sym_timeline_active] = ACTIONS(1535),
    [sym_timeline_dynamic_id] = ACTIONS(1535),
    [sym_variable_to_string] = ACTIONS(1535),
    [sym_string_to_variable] = ACTIONS(1535),
    [sym_length_variable_to_string] = ACTIONS(1535),
    [sym_type_cast] = ACTIONS(1535),
    [sym_variable_to_xml] = ACTIONS(1535),
    [sym_xml_to_variable] = ACTIONS(1535),
    [sym_length_variable_to_xml] = ACTIONS(1535),
    [sym_redirect_string] = ACTIONS(1535),
    [sym_random_number] = ACTIONS(1535),
    [sym_get_unique_id] = ACTIONS(1535),
    [sym_get_system_number] = ACTIONS(1535),
    [sym_set_system_number] = ACTIONS(1535),
    [sym_get_serial_number] = ACTIONS(1535),
    [sym_get_master_build] = ACTIONS(1535),
    [sym_set_validation_code] = ACTIONS(1535),
    [sym_reboot] = ACTIONS(1535),
    [sym_load_duet_module] = ACTIONS(1535),
    [sym_unload_duet_module] = ACTIONS(1535),
    [sym_duet_mem_size_set] = ACTIONS(1535),
    [sym_duet_mem_size_get] = ACTIONS(1535),
    [sym_dynamic_application_device] = ACTIONS(1535),
    [sym_dynamic_polled_port] = ACTIONS(1535),
    [sym_static_port_binding] = ACTIONS(1535),
    [sym_static_ip_binding] = ACTIONS(1535),
    [sym_get_dns_list] = ACTIONS(1535),
    [sym_set_dns_list] = ACTIONS(1535),
    [sym_get_ip_address] = ACTIONS(1535),
    [sym_set_ip_address] = ACTIONS(1535),
    [sym_get_url_list] = ACTIONS(1535),
    [sym_add_url_entry] = ACTIONS(1535),
    [sym_delete_url_entry] = ACTIONS(1535),
    [sym_abs_value] = ACTIONS(1535),
    [sym_max_value] = ACTIONS(1535),
    [sym_min_value] = ACTIONS(1535),
    [sym_exp_value] = ACTIONS(1535),
    [sym_log_value] = ACTIONS(1535),
    [sym_log10_value] = ACTIONS(1535),
    [sym_power_value] = ACTIONS(1535),
    [sym_sqrt_value] = ACTIONS(1535),
    [sym_raw_be] = ACTIONS(1535),
    [sym_raw_le] = ACTIONS(1535),
    [sym_internal_queue_size_set] = ACTIONS(1535),
    [sym_internal_queue_size_get] = ACTIONS(1535),
    [sym_internal_threshold_set] = ACTIONS(1535),
    [sym_internal_threshold_get] = ACTIONS(1535),
    [sym_rebuilt_event] = ACTIONS(1535),
    [sym_do_custom_event] = ACTIONS(1535),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1535),
    [sym_clkmgr_set_clk_source] = ACTIONS(1535),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1535),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1535),
    [sym_clkmgr_get_timezone] = ACTIONS(1535),
    [sym_clkmgr_set_timezone] = ACTIONS(1535),
    [sym_clkmgr_get_resync_period] = ACTIONS(1535),
    [sym_clkmgr_set_resync_period] = ACTIONS(1535),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1535),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1535),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1535),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1535),
    [sym_clkmgr_get_timeservers] = ACTIONS(1535),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1535),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1535),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1535),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1535),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1535),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1535),
    [sym_set_log_level] = ACTIONS(1535),
    [sym_get_log_level] = ACTIONS(1535),
    [sym_amx_log] = ACTIONS(1535),
    [sym_smtp_server_config_set] = ACTIONS(1535),
    [sym_smtp_server_config_get] = ACTIONS(1535),
    [sym_smtp_send] = ACTIONS(1535),
    [sym_validate_netlinx_account] = ACTIONS(1535),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1535),
    [sym_authenticate_certificate] = ACTIONS(1535),
    [sym_audit_netlinx_session_event] = ACTIONS(1535),
    [sym_audit_netlinx_generic_event] = ACTIONS(1535),
    [sym_get_available_flash_disk_space] = ACTIONS(1535),
    [sym_get_max_flash_disk_space] = ACTIONS(1535),
    [sym_data_create_feed] = ACTIONS(1535),
    [sym_data_delete_feed] = ACTIONS(1535),
    [sym_data_publish_feed] = ACTIONS(1535),
    [sym_data_get_published_feed] = ACTIONS(1535),
    [sym_data_add_record] = ACTIONS(1535),
    [sym_data_get_event_record] = ACTIONS(1535),
    [sym__wc_data_create_feed] = ACTIONS(1535),
    [sym__wc_data_add_record] = ACTIONS(1535),
    [sym__wc_data_get_event_record] = ACTIONS(1535),
    [sym_format] = ACTIONS(1535),
    [sym_astro_clock] = ACTIONS(1535),
    [sym_set_outdoor_temperature] = ACTIONS(1535),
    [sym_set_virtual_level_count] = ACTIONS(1535),
    [sym_set_virtual_channel_count] = ACTIONS(1535),
    [sym_set_virtual_port_count] = ACTIONS(1535),
    [sym__wc_encode_int] = ACTIONS(1535),
    [sym__wc_decode_int] = ACTIONS(1535),
    [sym__wc_int] = ACTIONS(1535),
    [sym__wc_to_ch_int] = ACTIONS(1535),
    [sym__ch_to_wc_int] = ACTIONS(1535),
    [sym__wc_find_string_int] = ACTIONS(1535),
    [sym__wc_left_string_int] = ACTIONS(1535),
    [sym__wc_lower_string_int] = ACTIONS(1535),
    [sym__wc_mid_string_int] = ACTIONS(1535),
    [sym__wc_remove_string_int] = ACTIONS(1535),
    [sym__wc_right_string_int] = ACTIONS(1535),
    [sym__wc_upper_string_int] = ACTIONS(1535),
    [sym__wc_compare_string_int] = ACTIONS(1535),
    [sym__wc_get_buffer_char_int] = ACTIONS(1535),
    [sym__wc_get_buffer_string_int] = ACTIONS(1535),
    [sym__wc_concat_string_int] = ACTIONS(1535),
    [sym__wc_file_open_int] = ACTIONS(1535),
    [sym__wc_file_close_int] = ACTIONS(1535),
    [sym__wc_file_read_int] = ACTIONS(1535),
    [sym__wc_file_read_line_int] = ACTIONS(1535),
    [sym__wc_file_write_int] = ACTIONS(1535),
    [sym__wc_file_write_line_int] = ACTIONS(1535),
    [anon_sym_EQ] = ACTIONS(1535),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1537),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1537),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1537),
    [aux_sym_preproc_else_token1] = ACTIONS(1537),
    [anon_sym_COMMA] = ACTIONS(1537),
    [anon_sym_LBRACK] = ACTIONS(1537),
    [anon_sym_LBRACE] = ACTIONS(1537),
    [anon_sym_RBRACE] = ACTIONS(1537),
    [anon_sym_COLON] = ACTIONS(1537),
    [anon_sym_LPAREN] = ACTIONS(1535),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1537),
    [anon_sym_DASH] = ACTIONS(1535),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_STAR] = ACTIONS(1537),
    [anon_sym_SLASH] = ACTIONS(1535),
    [anon_sym_PERCENT] = ACTIONS(1537),
    [anon_sym_PIPE_PIPE] = ACTIONS(1537),
    [anon_sym_AMP_AMP] = ACTIONS(1537),
    [anon_sym_PIPE] = ACTIONS(1535),
    [anon_sym_CARET] = ACTIONS(1537),
    [anon_sym_AMP] = ACTIONS(1535),
    [anon_sym_EQ_EQ] = ACTIONS(1537),
    [anon_sym_BANG_EQ] = ACTIONS(1537),
    [anon_sym_LT_GT] = ACTIONS(1537),
    [anon_sym_GT] = ACTIONS(1535),
    [anon_sym_GT_EQ] = ACTIONS(1537),
    [anon_sym_LT_EQ] = ACTIONS(1537),
    [anon_sym_LT] = ACTIONS(1535),
    [anon_sym_LT_LT] = ACTIONS(1537),
    [anon_sym_GT_GT] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DOT] = ACTIONS(1535),
    [anon_sym_DQUOTE] = ACTIONS(1537),
    [anon_sym_SQUOTE] = ACTIONS(1537),
    [sym_number_literal] = ACTIONS(1537),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(604)] = {
    [sym_identifier] = ACTIONS(1492),
    [sym_and] = ACTIONS(1492),
    [sym_or] = ACTIONS(1492),
    [sym_not] = ACTIONS(1492),
    [sym_xor] = ACTIONS(1492),
    [sym_band] = ACTIONS(1492),
    [sym_bor] = ACTIONS(1492),
    [sym_bxor] = ACTIONS(1492),
    [sym_bnot] = ACTIONS(1492),
    [sym_lshift] = ACTIONS(1492),
    [sym_rshift] = ACTIONS(1492),
    [sym___file__] = ACTIONS(1492),
    [sym___date__] = ACTIONS(1492),
    [sym___ldate__] = ACTIONS(1492),
    [sym___time__] = ACTIONS(1492),
    [sym___line__] = ACTIONS(1492),
    [sym___name__] = ACTIONS(1492),
    [sym___netlinx__] = ACTIONS(1492),
    [sym_netlinx_axi_version] = ACTIONS(1492),
    [sym_url_flg_tcp] = ACTIONS(1492),
    [sym_url_flg_accinfopresent] = ACTIONS(1492),
    [sym_url_flg_temp] = ACTIONS(1492),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1492),
    [sym_url_flg_stat_mask] = ACTIONS(1492),
    [sym_url_flg_stat_lookup] = ACTIONS(1492),
    [sym_url_flg_stat_connecting] = ACTIONS(1492),
    [sym_url_flg_stat_waiting] = ACTIONS(1492),
    [sym_url_flg_stat_connected] = ACTIONS(1492),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1492),
    [sym_first_virtual_device] = ACTIONS(1492),
    [sym_first_local_port] = ACTIONS(1492),
    [sym_dynamic_virtual_device] = ACTIONS(1492),
    [sym_timeline_once] = ACTIONS(1492),
    [sym_timeline_repeat] = ACTIONS(1492),
    [sym_timeline_absolute] = ACTIONS(1492),
    [sym_timeline_relative] = ACTIONS(1492),
    [sym_do_push_timed_infinite] = ACTIONS(1492),
    [sym_source_type_no_address] = ACTIONS(1492),
    [sym_source_type_neuron_id] = ACTIONS(1492),
    [sym_source_type_ip_address] = ACTIONS(1492),
    [sym_source_type_axlink] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1492),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1492),
    [sym_source_type_ip_socket_address] = ACTIONS(1492),
    [sym_source_type_rs232] = ACTIONS(1492),
    [sym_source_type_internal] = ACTIONS(1492),
    [sym_source_type_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1492),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1492),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1492),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1492),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1492),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1492),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1492),
    [sym_true] = ACTIONS(1492),
    [sym_false] = ACTIONS(1492),
    [sym_file_read_only] = ACTIONS(1492),
    [sym_file_rw_new] = ACTIONS(1492),
    [sym_file_rw_append] = ACTIONS(1492),
    [sym_ip_tcp] = ACTIONS(1492),
    [sym_ip_udp] = ACTIONS(1492),
    [sym_ip_udp_2way] = ACTIONS(1492),
    [sym_xml_encode_types] = ACTIONS(1492),
    [sym_xml_encode_char_as_list] = ACTIONS(1492),
    [sym_xml_encode_le] = ACTIONS(1492),
    [sym_xml_decode_types] = ACTIONS(1492),
    [sym_xml_decode_no_preserve] = ACTIONS(1492),
    [sym_duet_dev_type_display_device] = ACTIONS(1492),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1492),
    [sym_duet_dev_type_io_device] = ACTIONS(1492),
    [sym_duet_dev_type_relay_device] = ACTIONS(1492),
    [sym_duet_dev_type_ups] = ACTIONS(1492),
    [sym_duet_dev_type_amplifier] = ACTIONS(1492),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1492),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1492),
    [sym_duet_dev_type_camera] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1492),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1492),
    [sym_duet_dev_type_dss] = ACTIONS(1492),
    [sym_duet_dev_type_dvr] = ACTIONS(1492),
    [sym_duet_dev_type_disc_device] = ACTIONS(1492),
    [sym_duet_dev_type_document_camera] = ACTIONS(1492),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1492),
    [sym_duet_dev_type_hvac] = ACTIONS(1492),
    [sym_duet_dev_type_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_light] = ACTIONS(1492),
    [sym_duet_dev_type_monitor] = ACTIONS(1492),
    [sym_duet_dev_type_motor] = ACTIONS(1492),
    [sym_duet_dev_type_multi_window] = ACTIONS(1492),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1492),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1492),
    [sym_duet_dev_type_receiver] = ACTIONS(1492),
    [sym_duet_dev_type_security_system] = ACTIONS(1492),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1492),
    [sym_duet_dev_type_settop_box] = ACTIONS(1492),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1492),
    [sym_duet_dev_type_switcher] = ACTIONS(1492),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1492),
    [sym_duet_dev_type_tv] = ACTIONS(1492),
    [sym_duet_dev_type_utility] = ACTIONS(1492),
    [sym_duet_dev_type_vcr] = ACTIONS(1492),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1492),
    [sym_duet_dev_type_video_processor] = ACTIONS(1492),
    [sym_duet_dev_type_video_projector] = ACTIONS(1492),
    [sym_duet_dev_type_video_wall] = ACTIONS(1492),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1492),
    [sym_duet_dev_type_weather] = ACTIONS(1492),
    [sym_duet_dev_not_polled] = ACTIONS(1492),
    [sym_duet_dev_polled] = ACTIONS(1492),
    [sym_clkmgr_mode_network] = ACTIONS(1492),
    [sym_clkmgr_mode_standalone] = ACTIONS(1492),
    [sym_normal_standby] = ACTIONS(1492),
    [sym_normal_wake] = ACTIONS(1492),
    [sym_ip_multicast_ttl_option] = ACTIONS(1492),
    [sym_ip_ttl_subnet] = ACTIONS(1492),
    [sym_ip_ttl_site] = ACTIONS(1492),
    [sym_ip_ttl_region] = ACTIONS(1492),
    [sym_ip_ttl_continent] = ACTIONS(1492),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1492),
    [sym_ip_nodelay_on] = ACTIONS(1492),
    [sym_ip_nodelay_off] = ACTIONS(1492),
    [sym_amx_error] = ACTIONS(1492),
    [sym_amx_warning] = ACTIONS(1492),
    [sym_amx_info] = ACTIONS(1492),
    [sym_amx_debug] = ACTIONS(1492),
    [sym_netlinx_login_success] = ACTIONS(1492),
    [sym_netlinx_login_fail] = ACTIONS(1492),
    [sym_netlinx_logout] = ACTIONS(1492),
    [sym_null_str] = ACTIONS(1492),
    [sym_smtp_address] = ACTIONS(1492),
    [sym_smtp_port_number] = ACTIONS(1492),
    [sym_smtp_username] = ACTIONS(1492),
    [sym_smtp_password] = ACTIONS(1492),
    [sym_smtp_require_tls] = ACTIONS(1492),
    [sym_smtp_from] = ACTIONS(1492),
    [sym_smtp_tls_true] = ACTIONS(1492),
    [sym_smtp_tls_false] = ACTIONS(1492),
    [sym_listview_on_row_select_event] = ACTIONS(1492),
    [sym_data_structure_unknown] = ACTIONS(1492),
    [sym_data_structure_datafeed] = ACTIONS(1492),
    [sym_data_structure_datarecordset] = ACTIONS(1492),
    [sym_data_structure_datarecord] = ACTIONS(1492),
    [sym_data_structure_datafield] = ACTIONS(1492),
    [sym_data_max_content_field_count] = ACTIONS(1492),
    [sym_data_max_metadata_field_count] = ACTIONS(1492),
    [sym_data_max_id_length] = ACTIONS(1492),
    [sym_data_max_name_length] = ACTIONS(1492),
    [sym_data_max_description_length] = ACTIONS(1492),
    [sym_data_max_type_length] = ACTIONS(1492),
    [sym_data_max_format_length] = ACTIONS(1492),
    [sym_data_max_source_length] = ACTIONS(1492),
    [sym_data_max_value_length] = ACTIONS(1492),
    [sym_data_max_label_length] = ACTIONS(1492),
    [sym_data_max_url_length] = ACTIONS(1492),
    [sym_max_last_login_info_length] = ACTIONS(1492),
    [sym_data_type_unknown] = ACTIONS(1492),
    [sym_data_type_string] = ACTIONS(1492),
    [sym_data_type_datetime] = ACTIONS(1492),
    [sym_data_type_date] = ACTIONS(1492),
    [sym_data_type_time] = ACTIONS(1492),
    [sym_data_type_image] = ACTIONS(1492),
    [sym_data_format_url] = ACTIONS(1492),
    [sym_data_format_phone] = ACTIONS(1492),
    [sym_data_format_email] = ACTIONS(1492),
    [sym_data_format_iso8601] = ACTIONS(1492),
    [sym_auditlog_priv] = ACTIONS(1492),
    [sym_configuration_priv] = ACTIONS(1492),
    [sym_deviceconfig_priv] = ACTIONS(1492),
    [sym_ftp_priv] = ACTIONS(1492),
    [sym_http_priv] = ACTIONS(1492),
    [sym_networkconfig_priv] = ACTIONS(1492),
    [sym_programport_priv] = ACTIONS(1492),
    [sym_remoteui_priv] = ACTIONS(1492),
    [sym_securitycontrol_priv] = ACTIONS(1492),
    [sym_softwaremanagement_priv] = ACTIONS(1492),
    [sym_terminal_priv] = ACTIONS(1492),
    [sym_tpadmin_priv] = ACTIONS(1492),
    [sym_user1_priv] = ACTIONS(1492),
    [sym_user2_priv] = ACTIONS(1492),
    [sym_user3_priv] = ACTIONS(1492),
    [sym_user4_priv] = ACTIONS(1492),
    [sym_usermanagement_priv] = ACTIONS(1492),
    [sym_valid_account] = ACTIONS(1492),
    [sym_err_name_invalid] = ACTIONS(1492),
    [sym_err_password_invalid] = ACTIONS(1492),
    [sym_err_invalid_account] = ACTIONS(1492),
    [sym_err_no_authorization] = ACTIONS(1492),
    [sym_err_invalid_parameter] = ACTIONS(1492),
    [sym_err_user_locked_out] = ACTIONS(1492),
    [sym_err_user_account_expired] = ACTIONS(1492),
    [sym_err_general_error] = ACTIONS(1492),
    [sym_err_authentication_error] = ACTIONS(1492),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1492),
    [sym_tls_validate_certificate] = ACTIONS(1492),
    [sym____reserved___] = ACTIONS(1492),
    [sym_date] = ACTIONS(1492),
    [sym_day] = ACTIONS(1492),
    [sym_time] = ACTIONS(1492),
    [sym_ldate] = ACTIONS(1492),
    [sym_push_device] = ACTIONS(1492),
    [sym_push_channel] = ACTIONS(1492),
    [sym_push_devchan] = ACTIONS(1492),
    [sym_release_device] = ACTIONS(1492),
    [sym_release_channel] = ACTIONS(1492),
    [sym_release_devchan] = ACTIONS(1492),
    [sym_master_slot] = ACTIONS(1492),
    [sym_get_pulse_time] = ACTIONS(1492),
    [sym_get_timer] = ACTIONS(1492),
    [sym_master_sn] = ACTIONS(1492),
    [sym_system_number] = ACTIONS(1492),
    [sym_button] = ACTIONS(1492),
    [sym_dv_channel] = ACTIONS(1492),
    [sym_data] = ACTIONS(1492),
    [sym_level] = ACTIONS(1492),
    [sym_timeline] = ACTIONS(1492),
    [sym_channel] = ACTIONS(1492),
    [sym_custom] = ACTIONS(1492),
    [sym_atoi] = ACTIONS(1492),
    [sym_atol] = ACTIONS(1492),
    [sym_atof] = ACTIONS(1492),
    [sym_itoa] = ACTIONS(1492),
    [sym_itohex] = ACTIONS(1492),
    [sym_hextoi] = ACTIONS(1492),
    [sym_ftoa] = ACTIONS(1492),
    [sym_find_string] = ACTIONS(1492),
    [sym_left_string] = ACTIONS(1492),
    [sym_right_string] = ACTIONS(1492),
    [sym_mid_string] = ACTIONS(1492),
    [sym_remove_string] = ACTIONS(1492),
    [sym_get_buffer_string] = ACTIONS(1492),
    [sym_get_buffer_char] = ACTIONS(1492),
    [sym_get_multi_buffer_string] = ACTIONS(1492),
    [sym_length_string] = ACTIONS(1492),
    [sym_max_length_string] = ACTIONS(1492),
    [sym_set_length_string] = ACTIONS(1492),
    [sym_compare_string] = ACTIONS(1492),
    [sym_upper_string] = ACTIONS(1492),
    [sym_lower_string] = ACTIONS(1492),
    [sym_get_last] = ACTIONS(1492),
    [sym_date_to_day] = ACTIONS(1492),
    [sym_date_to_month] = ACTIONS(1492),
    [sym_date_to_year] = ACTIONS(1492),
    [sym_time_to_hour] = ACTIONS(1492),
    [sym_time_to_minute] = ACTIONS(1492),
    [sym_time_to_second] = ACTIONS(1492),
    [sym_day_of_week] = ACTIONS(1492),
    [sym_set_pulse_time] = ACTIONS(1492),
    [sym_set_timer] = ACTIONS(1492),
    [sym_file_open] = ACTIONS(1492),
    [sym_file_close] = ACTIONS(1492),
    [sym_file_read] = ACTIONS(1492),
    [sym_file_read_line] = ACTIONS(1492),
    [sym_file_write] = ACTIONS(1492),
    [sym_file_write_line] = ACTIONS(1492),
    [sym_file_copy] = ACTIONS(1492),
    [sym_file_delete] = ACTIONS(1492),
    [sym_file_rename] = ACTIONS(1492),
    [sym_file_dir] = ACTIONS(1492),
    [sym_file_getdir] = ACTIONS(1492),
    [sym_file_setdir] = ACTIONS(1492),
    [sym_file_create_dir] = ACTIONS(1492),
    [sym_file_remove_dir] = ACTIONS(1492),
    [sym_file_seek] = ACTIONS(1492),
    [sym_length_array] = ACTIONS(1492),
    [sym_max_length_array] = ACTIONS(1492),
    [sym_set_length_array] = ACTIONS(1492),
    [sym_ip_client_open] = ACTIONS(1492),
    [sym_ip_client_close] = ACTIONS(1492),
    [sym_ip_server_open] = ACTIONS(1492),
    [sym_ip_server_close] = ACTIONS(1492),
    [sym_ip_bound_client_open] = ACTIONS(1492),
    [sym_ip_mc_server_open] = ACTIONS(1492),
    [sym_ip_set_option] = ACTIONS(1492),
    [sym_ssh_client_open] = ACTIONS(1492),
    [sym_ssh_client_close] = ACTIONS(1492),
    [sym_tls_client_open] = ACTIONS(1492),
    [sym_tls_client_close] = ACTIONS(1492),
    [sym_do_push] = ACTIONS(1492),
    [sym_do_push_timed] = ACTIONS(1492),
    [sym_do_release] = ACTIONS(1492),
    [sym_device_id] = ACTIONS(1492),
    [sym_device_info] = ACTIONS(1492),
    [sym_device_id_string] = ACTIONS(1492),
    [sym_device_standby] = ACTIONS(1492),
    [sym_device_wake] = ACTIONS(1492),
    [sym_timeline_create] = ACTIONS(1492),
    [sym_timeline_kill] = ACTIONS(1492),
    [sym_timeline_reload] = ACTIONS(1492),
    [sym_timeline_pause] = ACTIONS(1492),
    [sym_timeline_restart] = ACTIONS(1492),
    [sym_timeline_set] = ACTIONS(1492),
    [sym_timeline_get] = ACTIONS(1492),
    [sym_timeline_active] = ACTIONS(1492),
    [sym_timeline_dynamic_id] = ACTIONS(1492),
    [sym_variable_to_string] = ACTIONS(1492),
    [sym_string_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_string] = ACTIONS(1492),
    [sym_type_cast] = ACTIONS(1492),
    [sym_variable_to_xml] = ACTIONS(1492),
    [sym_xml_to_variable] = ACTIONS(1492),
    [sym_length_variable_to_xml] = ACTIONS(1492),
    [sym_redirect_string] = ACTIONS(1492),
    [sym_random_number] = ACTIONS(1492),
    [sym_get_unique_id] = ACTIONS(1492),
    [sym_get_system_number] = ACTIONS(1492),
    [sym_set_system_number] = ACTIONS(1492),
    [sym_get_serial_number] = ACTIONS(1492),
    [sym_get_master_build] = ACTIONS(1492),
    [sym_set_validation_code] = ACTIONS(1492),
    [sym_reboot] = ACTIONS(1492),
    [sym_load_duet_module] = ACTIONS(1492),
    [sym_unload_duet_module] = ACTIONS(1492),
    [sym_duet_mem_size_set] = ACTIONS(1492),
    [sym_duet_mem_size_get] = ACTIONS(1492),
    [sym_dynamic_application_device] = ACTIONS(1492),
    [sym_dynamic_polled_port] = ACTIONS(1492),
    [sym_static_port_binding] = ACTIONS(1492),
    [sym_static_ip_binding] = ACTIONS(1492),
    [sym_get_dns_list] = ACTIONS(1492),
    [sym_set_dns_list] = ACTIONS(1492),
    [sym_get_ip_address] = ACTIONS(1492),
    [sym_set_ip_address] = ACTIONS(1492),
    [sym_get_url_list] = ACTIONS(1492),
    [sym_add_url_entry] = ACTIONS(1492),
    [sym_delete_url_entry] = ACTIONS(1492),
    [sym_abs_value] = ACTIONS(1492),
    [sym_max_value] = ACTIONS(1492),
    [sym_min_value] = ACTIONS(1492),
    [sym_exp_value] = ACTIONS(1492),
    [sym_log_value] = ACTIONS(1492),
    [sym_log10_value] = ACTIONS(1492),
    [sym_power_value] = ACTIONS(1492),
    [sym_sqrt_value] = ACTIONS(1492),
    [sym_raw_be] = ACTIONS(1492),
    [sym_raw_le] = ACTIONS(1492),
    [sym_internal_queue_size_set] = ACTIONS(1492),
    [sym_internal_queue_size_get] = ACTIONS(1492),
    [sym_internal_threshold_set] = ACTIONS(1492),
    [sym_internal_threshold_get] = ACTIONS(1492),
    [sym_rebuilt_event] = ACTIONS(1492),
    [sym_do_custom_event] = ACTIONS(1492),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1492),
    [sym_clkmgr_set_clk_source] = ACTIONS(1492),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1492),
    [sym_clkmgr_get_timezone] = ACTIONS(1492),
    [sym_clkmgr_set_timezone] = ACTIONS(1492),
    [sym_clkmgr_get_resync_period] = ACTIONS(1492),
    [sym_clkmgr_set_resync_period] = ACTIONS(1492),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1492),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_timeservers] = ACTIONS(1492),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1492),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1492),
    [sym_set_log_level] = ACTIONS(1492),
    [sym_get_log_level] = ACTIONS(1492),
    [sym_amx_log] = ACTIONS(1492),
    [sym_smtp_server_config_set] = ACTIONS(1492),
    [sym_smtp_server_config_get] = ACTIONS(1492),
    [sym_smtp_send] = ACTIONS(1492),
    [sym_validate_netlinx_account] = ACTIONS(1492),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1492),
    [sym_authenticate_certificate] = ACTIONS(1492),
    [sym_audit_netlinx_session_event] = ACTIONS(1492),
    [sym_audit_netlinx_generic_event] = ACTIONS(1492),
    [sym_get_available_flash_disk_space] = ACTIONS(1492),
    [sym_get_max_flash_disk_space] = ACTIONS(1492),
    [sym_data_create_feed] = ACTIONS(1492),
    [sym_data_delete_feed] = ACTIONS(1492),
    [sym_data_publish_feed] = ACTIONS(1492),
    [sym_data_get_published_feed] = ACTIONS(1492),
    [sym_data_add_record] = ACTIONS(1492),
    [sym_data_get_event_record] = ACTIONS(1492),
    [sym__wc_data_create_feed] = ACTIONS(1492),
    [sym__wc_data_add_record] = ACTIONS(1492),
    [sym__wc_data_get_event_record] = ACTIONS(1492),
    [sym_format] = ACTIONS(1492),
    [sym_astro_clock] = ACTIONS(1492),
    [sym_set_outdoor_temperature] = ACTIONS(1492),
    [sym_set_virtual_level_count] = ACTIONS(1492),
    [sym_set_virtual_channel_count] = ACTIONS(1492),
    [sym_set_virtual_port_count] = ACTIONS(1492),
    [sym__wc_encode_int] = ACTIONS(1492),
    [sym__wc_decode_int] = ACTIONS(1492),
    [sym__wc_int] = ACTIONS(1492),
    [sym__wc_to_ch_int] = ACTIONS(1492),
    [sym__ch_to_wc_int] = ACTIONS(1492),
    [sym__wc_find_string_int] = ACTIONS(1492),
    [sym__wc_left_string_int] = ACTIONS(1492),
    [sym__wc_lower_string_int] = ACTIONS(1492),
    [sym__wc_mid_string_int] = ACTIONS(1492),
    [sym__wc_remove_string_int] = ACTIONS(1492),
    [sym__wc_right_string_int] = ACTIONS(1492),
    [sym__wc_upper_string_int] = ACTIONS(1492),
    [sym__wc_compare_string_int] = ACTIONS(1492),
    [sym__wc_get_buffer_char_int] = ACTIONS(1492),
    [sym__wc_get_buffer_string_int] = ACTIONS(1492),
    [sym__wc_concat_string_int] = ACTIONS(1492),
    [sym__wc_file_open_int] = ACTIONS(1492),
    [sym__wc_file_close_int] = ACTIONS(1492),
    [sym__wc_file_read_int] = ACTIONS(1492),
    [sym__wc_file_read_line_int] = ACTIONS(1492),
    [sym__wc_file_write_int] = ACTIONS(1492),
    [sym__wc_file_write_line_int] = ACTIONS(1492),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1494),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1494),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1494),
    [aux_sym_preproc_else_token1] = ACTIONS(1494),
    [anon_sym_COMMA] = ACTIONS(1494),
    [anon_sym_LBRACK] = ACTIONS(1494),
    [anon_sym_LBRACE] = ACTIONS(1494),
    [anon_sym_RBRACE] = ACTIONS(1494),
    [anon_sym_COLON] = ACTIONS(1494),
    [anon_sym_LPAREN] = ACTIONS(1492),
    [anon_sym_BANG] = ACTIONS(1492),
    [anon_sym_TILDE] = ACTIONS(1494),
    [anon_sym_DASH] = ACTIONS(1492),
    [anon_sym_PLUS] = ACTIONS(1492),
    [anon_sym_STAR] = ACTIONS(1494),
    [anon_sym_SLASH] = ACTIONS(1492),
    [anon_sym_PERCENT] = ACTIONS(1494),
    [anon_sym_PIPE_PIPE] = ACTIONS(1494),
    [anon_sym_AMP_AMP] = ACTIONS(1494),
    [anon_sym_PIPE] = ACTIONS(1492),
    [anon_sym_CARET] = ACTIONS(1494),
    [anon_sym_AMP] = ACTIONS(1492),
    [anon_sym_EQ_EQ] = ACTIONS(1494),
    [anon_sym_BANG_EQ] = ACTIONS(1494),
    [anon_sym_LT_GT] = ACTIONS(1494),
    [anon_sym_GT] = ACTIONS(1492),
    [anon_sym_GT_EQ] = ACTIONS(1494),
    [anon_sym_LT_EQ] = ACTIONS(1494),
    [anon_sym_LT] = ACTIONS(1492),
    [anon_sym_LT_LT] = ACTIONS(1494),
    [anon_sym_GT_GT] = ACTIONS(1494),
    [anon_sym_DASH_DASH] = ACTIONS(1494),
    [anon_sym_PLUS_PLUS] = ACTIONS(1494),
    [anon_sym_DOT] = ACTIONS(1492),
    [anon_sym_DQUOTE] = ACTIONS(1494),
    [anon_sym_SQUOTE] = ACTIONS(1494),
    [sym_number_literal] = ACTIONS(1494),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(605)] = {
    [sym_identifier] = ACTIONS(1593),
    [sym_and] = ACTIONS(1593),
    [sym_or] = ACTIONS(1593),
    [sym_not] = ACTIONS(1593),
    [sym_xor] = ACTIONS(1593),
    [sym_band] = ACTIONS(1593),
    [sym_bor] = ACTIONS(1593),
    [sym_bxor] = ACTIONS(1593),
    [sym_bnot] = ACTIONS(1593),
    [sym_lshift] = ACTIONS(1593),
    [sym_rshift] = ACTIONS(1593),
    [sym___file__] = ACTIONS(1593),
    [sym___date__] = ACTIONS(1593),
    [sym___ldate__] = ACTIONS(1593),
    [sym___time__] = ACTIONS(1593),
    [sym___line__] = ACTIONS(1593),
    [sym___name__] = ACTIONS(1593),
    [sym___netlinx__] = ACTIONS(1593),
    [sym_netlinx_axi_version] = ACTIONS(1593),
    [sym_url_flg_tcp] = ACTIONS(1593),
    [sym_url_flg_accinfopresent] = ACTIONS(1593),
    [sym_url_flg_temp] = ACTIONS(1593),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1593),
    [sym_url_flg_stat_mask] = ACTIONS(1593),
    [sym_url_flg_stat_lookup] = ACTIONS(1593),
    [sym_url_flg_stat_connecting] = ACTIONS(1593),
    [sym_url_flg_stat_waiting] = ACTIONS(1593),
    [sym_url_flg_stat_connected] = ACTIONS(1593),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1593),
    [sym_first_virtual_device] = ACTIONS(1593),
    [sym_first_local_port] = ACTIONS(1593),
    [sym_dynamic_virtual_device] = ACTIONS(1593),
    [sym_timeline_once] = ACTIONS(1593),
    [sym_timeline_repeat] = ACTIONS(1593),
    [sym_timeline_absolute] = ACTIONS(1593),
    [sym_timeline_relative] = ACTIONS(1593),
    [sym_do_push_timed_infinite] = ACTIONS(1593),
    [sym_source_type_no_address] = ACTIONS(1593),
    [sym_source_type_neuron_id] = ACTIONS(1593),
    [sym_source_type_ip_address] = ACTIONS(1593),
    [sym_source_type_axlink] = ACTIONS(1593),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1593),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1593),
    [sym_source_type_ip_socket_address] = ACTIONS(1593),
    [sym_source_type_rs232] = ACTIONS(1593),
    [sym_source_type_internal] = ACTIONS(1593),
    [sym_source_type_mac_address] = ACTIONS(1593),
    [sym_source_type_ipv4_port] = ACTIONS(1593),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1593),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1593),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1593),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1593),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1593),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1593),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1593),
    [sym_true] = ACTIONS(1593),
    [sym_false] = ACTIONS(1593),
    [sym_file_read_only] = ACTIONS(1593),
    [sym_file_rw_new] = ACTIONS(1593),
    [sym_file_rw_append] = ACTIONS(1593),
    [sym_ip_tcp] = ACTIONS(1593),
    [sym_ip_udp] = ACTIONS(1593),
    [sym_ip_udp_2way] = ACTIONS(1593),
    [sym_xml_encode_types] = ACTIONS(1593),
    [sym_xml_encode_char_as_list] = ACTIONS(1593),
    [sym_xml_encode_le] = ACTIONS(1593),
    [sym_xml_decode_types] = ACTIONS(1593),
    [sym_xml_decode_no_preserve] = ACTIONS(1593),
    [sym_duet_dev_type_display_device] = ACTIONS(1593),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1593),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1593),
    [sym_duet_dev_type_io_device] = ACTIONS(1593),
    [sym_duet_dev_type_relay_device] = ACTIONS(1593),
    [sym_duet_dev_type_ups] = ACTIONS(1593),
    [sym_duet_dev_type_amplifier] = ACTIONS(1593),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1593),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1593),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1593),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1593),
    [sym_duet_dev_type_camera] = ACTIONS(1593),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1593),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1593),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1593),
    [sym_duet_dev_type_dss] = ACTIONS(1593),
    [sym_duet_dev_type_dvr] = ACTIONS(1593),
    [sym_duet_dev_type_disc_device] = ACTIONS(1593),
    [sym_duet_dev_type_document_camera] = ACTIONS(1593),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1593),
    [sym_duet_dev_type_hvac] = ACTIONS(1593),
    [sym_duet_dev_type_keypad] = ACTIONS(1593),
    [sym_duet_dev_type_light] = ACTIONS(1593),
    [sym_duet_dev_type_monitor] = ACTIONS(1593),
    [sym_duet_dev_type_motor] = ACTIONS(1593),
    [sym_duet_dev_type_multi_window] = ACTIONS(1593),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1593),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1593),
    [sym_duet_dev_type_receiver] = ACTIONS(1593),
    [sym_duet_dev_type_security_system] = ACTIONS(1593),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1593),
    [sym_duet_dev_type_settop_box] = ACTIONS(1593),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1593),
    [sym_duet_dev_type_switcher] = ACTIONS(1593),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1593),
    [sym_duet_dev_type_tv] = ACTIONS(1593),
    [sym_duet_dev_type_utility] = ACTIONS(1593),
    [sym_duet_dev_type_vcr] = ACTIONS(1593),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1593),
    [sym_duet_dev_type_video_processor] = ACTIONS(1593),
    [sym_duet_dev_type_video_projector] = ACTIONS(1593),
    [sym_duet_dev_type_video_wall] = ACTIONS(1593),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1593),
    [sym_duet_dev_type_weather] = ACTIONS(1593),
    [sym_duet_dev_not_polled] = ACTIONS(1593),
    [sym_duet_dev_polled] = ACTIONS(1593),
    [sym_clkmgr_mode_network] = ACTIONS(1593),
    [sym_clkmgr_mode_standalone] = ACTIONS(1593),
    [sym_normal_standby] = ACTIONS(1593),
    [sym_normal_wake] = ACTIONS(1593),
    [sym_ip_multicast_ttl_option] = ACTIONS(1593),
    [sym_ip_ttl_subnet] = ACTIONS(1593),
    [sym_ip_ttl_site] = ACTIONS(1593),
    [sym_ip_ttl_region] = ACTIONS(1593),
    [sym_ip_ttl_continent] = ACTIONS(1593),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1593),
    [sym_ip_nodelay_on] = ACTIONS(1593),
    [sym_ip_nodelay_off] = ACTIONS(1593),
    [sym_amx_error] = ACTIONS(1593),
    [sym_amx_warning] = ACTIONS(1593),
    [sym_amx_info] = ACTIONS(1593),
    [sym_amx_debug] = ACTIONS(1593),
    [sym_netlinx_login_success] = ACTIONS(1593),
    [sym_netlinx_login_fail] = ACTIONS(1593),
    [sym_netlinx_logout] = ACTIONS(1593),
    [sym_null_str] = ACTIONS(1593),
    [sym_smtp_address] = ACTIONS(1593),
    [sym_smtp_port_number] = ACTIONS(1593),
    [sym_smtp_username] = ACTIONS(1593),
    [sym_smtp_password] = ACTIONS(1593),
    [sym_smtp_require_tls] = ACTIONS(1593),
    [sym_smtp_from] = ACTIONS(1593),
    [sym_smtp_tls_true] = ACTIONS(1593),
    [sym_smtp_tls_false] = ACTIONS(1593),
    [sym_listview_on_row_select_event] = ACTIONS(1593),
    [sym_data_structure_unknown] = ACTIONS(1593),
    [sym_data_structure_datafeed] = ACTIONS(1593),
    [sym_data_structure_datarecordset] = ACTIONS(1593),
    [sym_data_structure_datarecord] = ACTIONS(1593),
    [sym_data_structure_datafield] = ACTIONS(1593),
    [sym_data_max_content_field_count] = ACTIONS(1593),
    [sym_data_max_metadata_field_count] = ACTIONS(1593),
    [sym_data_max_id_length] = ACTIONS(1593),
    [sym_data_max_name_length] = ACTIONS(1593),
    [sym_data_max_description_length] = ACTIONS(1593),
    [sym_data_max_type_length] = ACTIONS(1593),
    [sym_data_max_format_length] = ACTIONS(1593),
    [sym_data_max_source_length] = ACTIONS(1593),
    [sym_data_max_value_length] = ACTIONS(1593),
    [sym_data_max_label_length] = ACTIONS(1593),
    [sym_data_max_url_length] = ACTIONS(1593),
    [sym_max_last_login_info_length] = ACTIONS(1593),
    [sym_data_type_unknown] = ACTIONS(1593),
    [sym_data_type_string] = ACTIONS(1593),
    [sym_data_type_datetime] = ACTIONS(1593),
    [sym_data_type_date] = ACTIONS(1593),
    [sym_data_type_time] = ACTIONS(1593),
    [sym_data_type_image] = ACTIONS(1593),
    [sym_data_format_url] = ACTIONS(1593),
    [sym_data_format_phone] = ACTIONS(1593),
    [sym_data_format_email] = ACTIONS(1593),
    [sym_data_format_iso8601] = ACTIONS(1593),
    [sym_auditlog_priv] = ACTIONS(1593),
    [sym_configuration_priv] = ACTIONS(1593),
    [sym_deviceconfig_priv] = ACTIONS(1593),
    [sym_ftp_priv] = ACTIONS(1593),
    [sym_http_priv] = ACTIONS(1593),
    [sym_networkconfig_priv] = ACTIONS(1593),
    [sym_programport_priv] = ACTIONS(1593),
    [sym_remoteui_priv] = ACTIONS(1593),
    [sym_securitycontrol_priv] = ACTIONS(1593),
    [sym_softwaremanagement_priv] = ACTIONS(1593),
    [sym_terminal_priv] = ACTIONS(1593),
    [sym_tpadmin_priv] = ACTIONS(1593),
    [sym_user1_priv] = ACTIONS(1593),
    [sym_user2_priv] = ACTIONS(1593),
    [sym_user3_priv] = ACTIONS(1593),
    [sym_user4_priv] = ACTIONS(1593),
    [sym_usermanagement_priv] = ACTIONS(1593),
    [sym_valid_account] = ACTIONS(1593),
    [sym_err_name_invalid] = ACTIONS(1593),
    [sym_err_password_invalid] = ACTIONS(1593),
    [sym_err_invalid_account] = ACTIONS(1593),
    [sym_err_no_authorization] = ACTIONS(1593),
    [sym_err_invalid_parameter] = ACTIONS(1593),
    [sym_err_user_locked_out] = ACTIONS(1593),
    [sym_err_user_account_expired] = ACTIONS(1593),
    [sym_err_general_error] = ACTIONS(1593),
    [sym_err_authentication_error] = ACTIONS(1593),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1593),
    [sym_tls_validate_certificate] = ACTIONS(1593),
    [sym____reserved___] = ACTIONS(1593),
    [sym_date] = ACTIONS(1593),
    [sym_day] = ACTIONS(1593),
    [sym_time] = ACTIONS(1593),
    [sym_ldate] = ACTIONS(1593),
    [sym_push_device] = ACTIONS(1593),
    [sym_push_channel] = ACTIONS(1593),
    [sym_push_devchan] = ACTIONS(1593),
    [sym_release_device] = ACTIONS(1593),
    [sym_release_channel] = ACTIONS(1593),
    [sym_release_devchan] = ACTIONS(1593),
    [sym_master_slot] = ACTIONS(1593),
    [sym_get_pulse_time] = ACTIONS(1593),
    [sym_get_timer] = ACTIONS(1593),
    [sym_master_sn] = ACTIONS(1593),
    [sym_system_number] = ACTIONS(1593),
    [sym_button] = ACTIONS(1593),
    [sym_dv_channel] = ACTIONS(1593),
    [sym_data] = ACTIONS(1593),
    [sym_level] = ACTIONS(1593),
    [sym_timeline] = ACTIONS(1593),
    [sym_channel] = ACTIONS(1593),
    [sym_custom] = ACTIONS(1593),
    [sym_atoi] = ACTIONS(1593),
    [sym_atol] = ACTIONS(1593),
    [sym_atof] = ACTIONS(1593),
    [sym_itoa] = ACTIONS(1593),
    [sym_itohex] = ACTIONS(1593),
    [sym_hextoi] = ACTIONS(1593),
    [sym_ftoa] = ACTIONS(1593),
    [sym_find_string] = ACTIONS(1593),
    [sym_left_string] = ACTIONS(1593),
    [sym_right_string] = ACTIONS(1593),
    [sym_mid_string] = ACTIONS(1593),
    [sym_remove_string] = ACTIONS(1593),
    [sym_get_buffer_string] = ACTIONS(1593),
    [sym_get_buffer_char] = ACTIONS(1593),
    [sym_get_multi_buffer_string] = ACTIONS(1593),
    [sym_length_string] = ACTIONS(1593),
    [sym_max_length_string] = ACTIONS(1593),
    [sym_set_length_string] = ACTIONS(1593),
    [sym_compare_string] = ACTIONS(1593),
    [sym_upper_string] = ACTIONS(1593),
    [sym_lower_string] = ACTIONS(1593),
    [sym_get_last] = ACTIONS(1593),
    [sym_date_to_day] = ACTIONS(1593),
    [sym_date_to_month] = ACTIONS(1593),
    [sym_date_to_year] = ACTIONS(1593),
    [sym_time_to_hour] = ACTIONS(1593),
    [sym_time_to_minute] = ACTIONS(1593),
    [sym_time_to_second] = ACTIONS(1593),
    [sym_day_of_week] = ACTIONS(1593),
    [sym_set_pulse_time] = ACTIONS(1593),
    [sym_set_timer] = ACTIONS(1593),
    [sym_file_open] = ACTIONS(1593),
    [sym_file_close] = ACTIONS(1593),
    [sym_file_read] = ACTIONS(1593),
    [sym_file_read_line] = ACTIONS(1593),
    [sym_file_write] = ACTIONS(1593),
    [sym_file_write_line] = ACTIONS(1593),
    [sym_file_copy] = ACTIONS(1593),
    [sym_file_delete] = ACTIONS(1593),
    [sym_file_rename] = ACTIONS(1593),
    [sym_file_dir] = ACTIONS(1593),
    [sym_file_getdir] = ACTIONS(1593),
    [sym_file_setdir] = ACTIONS(1593),
    [sym_file_create_dir] = ACTIONS(1593),
    [sym_file_remove_dir] = ACTIONS(1593),
    [sym_file_seek] = ACTIONS(1593),
    [sym_length_array] = ACTIONS(1593),
    [sym_max_length_array] = ACTIONS(1593),
    [sym_set_length_array] = ACTIONS(1593),
    [sym_ip_client_open] = ACTIONS(1593),
    [sym_ip_client_close] = ACTIONS(1593),
    [sym_ip_server_open] = ACTIONS(1593),
    [sym_ip_server_close] = ACTIONS(1593),
    [sym_ip_bound_client_open] = ACTIONS(1593),
    [sym_ip_mc_server_open] = ACTIONS(1593),
    [sym_ip_set_option] = ACTIONS(1593),
    [sym_ssh_client_open] = ACTIONS(1593),
    [sym_ssh_client_close] = ACTIONS(1593),
    [sym_tls_client_open] = ACTIONS(1593),
    [sym_tls_client_close] = ACTIONS(1593),
    [sym_do_push] = ACTIONS(1593),
    [sym_do_push_timed] = ACTIONS(1593),
    [sym_do_release] = ACTIONS(1593),
    [sym_device_id] = ACTIONS(1593),
    [sym_device_info] = ACTIONS(1593),
    [sym_device_id_string] = ACTIONS(1593),
    [sym_device_standby] = ACTIONS(1593),
    [sym_device_wake] = ACTIONS(1593),
    [sym_timeline_create] = ACTIONS(1593),
    [sym_timeline_kill] = ACTIONS(1593),
    [sym_timeline_reload] = ACTIONS(1593),
    [sym_timeline_pause] = ACTIONS(1593),
    [sym_timeline_restart] = ACTIONS(1593),
    [sym_timeline_set] = ACTIONS(1593),
    [sym_timeline_get] = ACTIONS(1593),
    [sym_timeline_active] = ACTIONS(1593),
    [sym_timeline_dynamic_id] = ACTIONS(1593),
    [sym_variable_to_string] = ACTIONS(1593),
    [sym_string_to_variable] = ACTIONS(1593),
    [sym_length_variable_to_string] = ACTIONS(1593),
    [sym_type_cast] = ACTIONS(1593),
    [sym_variable_to_xml] = ACTIONS(1593),
    [sym_xml_to_variable] = ACTIONS(1593),
    [sym_length_variable_to_xml] = ACTIONS(1593),
    [sym_redirect_string] = ACTIONS(1593),
    [sym_random_number] = ACTIONS(1593),
    [sym_get_unique_id] = ACTIONS(1593),
    [sym_get_system_number] = ACTIONS(1593),
    [sym_set_system_number] = ACTIONS(1593),
    [sym_get_serial_number] = ACTIONS(1593),
    [sym_get_master_build] = ACTIONS(1593),
    [sym_set_validation_code] = ACTIONS(1593),
    [sym_reboot] = ACTIONS(1593),
    [sym_load_duet_module] = ACTIONS(1593),
    [sym_unload_duet_module] = ACTIONS(1593),
    [sym_duet_mem_size_set] = ACTIONS(1593),
    [sym_duet_mem_size_get] = ACTIONS(1593),
    [sym_dynamic_application_device] = ACTIONS(1593),
    [sym_dynamic_polled_port] = ACTIONS(1593),
    [sym_static_port_binding] = ACTIONS(1593),
    [sym_static_ip_binding] = ACTIONS(1593),
    [sym_get_dns_list] = ACTIONS(1593),
    [sym_set_dns_list] = ACTIONS(1593),
    [sym_get_ip_address] = ACTIONS(1593),
    [sym_set_ip_address] = ACTIONS(1593),
    [sym_get_url_list] = ACTIONS(1593),
    [sym_add_url_entry] = ACTIONS(1593),
    [sym_delete_url_entry] = ACTIONS(1593),
    [sym_abs_value] = ACTIONS(1593),
    [sym_max_value] = ACTIONS(1593),
    [sym_min_value] = ACTIONS(1593),
    [sym_exp_value] = ACTIONS(1593),
    [sym_log_value] = ACTIONS(1593),
    [sym_log10_value] = ACTIONS(1593),
    [sym_power_value] = ACTIONS(1593),
    [sym_sqrt_value] = ACTIONS(1593),
    [sym_raw_be] = ACTIONS(1593),
    [sym_raw_le] = ACTIONS(1593),
    [sym_internal_queue_size_set] = ACTIONS(1593),
    [sym_internal_queue_size_get] = ACTIONS(1593),
    [sym_internal_threshold_set] = ACTIONS(1593),
    [sym_internal_threshold_get] = ACTIONS(1593),
    [sym_rebuilt_event] = ACTIONS(1593),
    [sym_do_custom_event] = ACTIONS(1593),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1593),
    [sym_clkmgr_set_clk_source] = ACTIONS(1593),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1593),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1593),
    [sym_clkmgr_get_timezone] = ACTIONS(1593),
    [sym_clkmgr_set_timezone] = ACTIONS(1593),
    [sym_clkmgr_get_resync_period] = ACTIONS(1593),
    [sym_clkmgr_set_resync_period] = ACTIONS(1593),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1593),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1593),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1593),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1593),
    [sym_clkmgr_get_timeservers] = ACTIONS(1593),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1593),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1593),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1593),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1593),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1593),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1593),
    [sym_set_log_level] = ACTIONS(1593),
    [sym_get_log_level] = ACTIONS(1593),
    [sym_amx_log] = ACTIONS(1593),
    [sym_smtp_server_config_set] = ACTIONS(1593),
    [sym_smtp_server_config_get] = ACTIONS(1593),
    [sym_smtp_send] = ACTIONS(1593),
    [sym_validate_netlinx_account] = ACTIONS(1593),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1593),
    [sym_authenticate_certificate] = ACTIONS(1593),
    [sym_audit_netlinx_session_event] = ACTIONS(1593),
    [sym_audit_netlinx_generic_event] = ACTIONS(1593),
    [sym_get_available_flash_disk_space] = ACTIONS(1593),
    [sym_get_max_flash_disk_space] = ACTIONS(1593),
    [sym_data_create_feed] = ACTIONS(1593),
    [sym_data_delete_feed] = ACTIONS(1593),
    [sym_data_publish_feed] = ACTIONS(1593),
    [sym_data_get_published_feed] = ACTIONS(1593),
    [sym_data_add_record] = ACTIONS(1593),
    [sym_data_get_event_record] = ACTIONS(1593),
    [sym__wc_data_create_feed] = ACTIONS(1593),
    [sym__wc_data_add_record] = ACTIONS(1593),
    [sym__wc_data_get_event_record] = ACTIONS(1593),
    [sym_format] = ACTIONS(1593),
    [sym_astro_clock] = ACTIONS(1593),
    [sym_set_outdoor_temperature] = ACTIONS(1593),
    [sym_set_virtual_level_count] = ACTIONS(1593),
    [sym_set_virtual_channel_count] = ACTIONS(1593),
    [sym_set_virtual_port_count] = ACTIONS(1593),
    [sym__wc_encode_int] = ACTIONS(1593),
    [sym__wc_decode_int] = ACTIONS(1593),
    [sym__wc_int] = ACTIONS(1593),
    [sym__wc_to_ch_int] = ACTIONS(1593),
    [sym__ch_to_wc_int] = ACTIONS(1593),
    [sym__wc_find_string_int] = ACTIONS(1593),
    [sym__wc_left_string_int] = ACTIONS(1593),
    [sym__wc_lower_string_int] = ACTIONS(1593),
    [sym__wc_mid_string_int] = ACTIONS(1593),
    [sym__wc_remove_string_int] = ACTIONS(1593),
    [sym__wc_right_string_int] = ACTIONS(1593),
    [sym__wc_upper_string_int] = ACTIONS(1593),
    [sym__wc_compare_string_int] = ACTIONS(1593),
    [sym__wc_get_buffer_char_int] = ACTIONS(1593),
    [sym__wc_get_buffer_string_int] = ACTIONS(1593),
    [sym__wc_concat_string_int] = ACTIONS(1593),
    [sym__wc_file_open_int] = ACTIONS(1593),
    [sym__wc_file_close_int] = ACTIONS(1593),
    [sym__wc_file_read_int] = ACTIONS(1593),
    [sym__wc_file_read_line_int] = ACTIONS(1593),
    [sym__wc_file_write_int] = ACTIONS(1593),
    [sym__wc_file_write_line_int] = ACTIONS(1593),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1595),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1595),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1595),
    [aux_sym_preproc_else_token1] = ACTIONS(1595),
    [anon_sym_COMMA] = ACTIONS(1595),
    [anon_sym_LBRACK] = ACTIONS(1595),
    [anon_sym_LBRACE] = ACTIONS(1595),
    [anon_sym_RBRACE] = ACTIONS(1595),
    [anon_sym_COLON] = ACTIONS(1595),
    [anon_sym_LPAREN] = ACTIONS(1593),
    [anon_sym_BANG] = ACTIONS(1593),
    [anon_sym_TILDE] = ACTIONS(1595),
    [anon_sym_DASH] = ACTIONS(1593),
    [anon_sym_PLUS] = ACTIONS(1593),
    [anon_sym_STAR] = ACTIONS(1595),
    [anon_sym_SLASH] = ACTIONS(1593),
    [anon_sym_PERCENT] = ACTIONS(1595),
    [anon_sym_PIPE_PIPE] = ACTIONS(1595),
    [anon_sym_AMP_AMP] = ACTIONS(1595),
    [anon_sym_PIPE] = ACTIONS(1593),
    [anon_sym_CARET] = ACTIONS(1595),
    [anon_sym_AMP] = ACTIONS(1593),
    [anon_sym_EQ_EQ] = ACTIONS(1595),
    [anon_sym_BANG_EQ] = ACTIONS(1595),
    [anon_sym_LT_GT] = ACTIONS(1595),
    [anon_sym_GT] = ACTIONS(1593),
    [anon_sym_GT_EQ] = ACTIONS(1595),
    [anon_sym_LT_EQ] = ACTIONS(1595),
    [anon_sym_LT] = ACTIONS(1593),
    [anon_sym_LT_LT] = ACTIONS(1595),
    [anon_sym_GT_GT] = ACTIONS(1595),
    [anon_sym_DASH_DASH] = ACTIONS(1595),
    [anon_sym_PLUS_PLUS] = ACTIONS(1595),
    [anon_sym_DOT] = ACTIONS(1593),
    [anon_sym_DQUOTE] = ACTIONS(1595),
    [anon_sym_SQUOTE] = ACTIONS(1595),
    [sym_number_literal] = ACTIONS(1595),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(606)] = {
    [sym_identifier] = ACTIONS(1577),
    [sym_and] = ACTIONS(1577),
    [sym_or] = ACTIONS(1577),
    [sym_not] = ACTIONS(1577),
    [sym_xor] = ACTIONS(1577),
    [sym_band] = ACTIONS(1577),
    [sym_bor] = ACTIONS(1577),
    [sym_bxor] = ACTIONS(1577),
    [sym_bnot] = ACTIONS(1577),
    [sym_lshift] = ACTIONS(1577),
    [sym_rshift] = ACTIONS(1577),
    [sym___file__] = ACTIONS(1577),
    [sym___date__] = ACTIONS(1577),
    [sym___ldate__] = ACTIONS(1577),
    [sym___time__] = ACTIONS(1577),
    [sym___line__] = ACTIONS(1577),
    [sym___name__] = ACTIONS(1577),
    [sym___netlinx__] = ACTIONS(1577),
    [sym_netlinx_axi_version] = ACTIONS(1577),
    [sym_url_flg_tcp] = ACTIONS(1577),
    [sym_url_flg_accinfopresent] = ACTIONS(1577),
    [sym_url_flg_temp] = ACTIONS(1577),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1577),
    [sym_url_flg_stat_mask] = ACTIONS(1577),
    [sym_url_flg_stat_lookup] = ACTIONS(1577),
    [sym_url_flg_stat_connecting] = ACTIONS(1577),
    [sym_url_flg_stat_waiting] = ACTIONS(1577),
    [sym_url_flg_stat_connected] = ACTIONS(1577),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1577),
    [sym_first_virtual_device] = ACTIONS(1577),
    [sym_first_local_port] = ACTIONS(1577),
    [sym_dynamic_virtual_device] = ACTIONS(1577),
    [sym_timeline_once] = ACTIONS(1577),
    [sym_timeline_repeat] = ACTIONS(1577),
    [sym_timeline_absolute] = ACTIONS(1577),
    [sym_timeline_relative] = ACTIONS(1577),
    [sym_do_push_timed_infinite] = ACTIONS(1577),
    [sym_source_type_no_address] = ACTIONS(1577),
    [sym_source_type_neuron_id] = ACTIONS(1577),
    [sym_source_type_ip_address] = ACTIONS(1577),
    [sym_source_type_axlink] = ACTIONS(1577),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1577),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1577),
    [sym_source_type_ip_socket_address] = ACTIONS(1577),
    [sym_source_type_rs232] = ACTIONS(1577),
    [sym_source_type_internal] = ACTIONS(1577),
    [sym_source_type_mac_address] = ACTIONS(1577),
    [sym_source_type_ipv4_port] = ACTIONS(1577),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1577),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1577),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1577),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1577),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1577),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1577),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1577),
    [sym_true] = ACTIONS(1577),
    [sym_false] = ACTIONS(1577),
    [sym_file_read_only] = ACTIONS(1577),
    [sym_file_rw_new] = ACTIONS(1577),
    [sym_file_rw_append] = ACTIONS(1577),
    [sym_ip_tcp] = ACTIONS(1577),
    [sym_ip_udp] = ACTIONS(1577),
    [sym_ip_udp_2way] = ACTIONS(1577),
    [sym_xml_encode_types] = ACTIONS(1577),
    [sym_xml_encode_char_as_list] = ACTIONS(1577),
    [sym_xml_encode_le] = ACTIONS(1577),
    [sym_xml_decode_types] = ACTIONS(1577),
    [sym_xml_decode_no_preserve] = ACTIONS(1577),
    [sym_duet_dev_type_display_device] = ACTIONS(1577),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1577),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1577),
    [sym_duet_dev_type_io_device] = ACTIONS(1577),
    [sym_duet_dev_type_relay_device] = ACTIONS(1577),
    [sym_duet_dev_type_ups] = ACTIONS(1577),
    [sym_duet_dev_type_amplifier] = ACTIONS(1577),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1577),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1577),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1577),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1577),
    [sym_duet_dev_type_camera] = ACTIONS(1577),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1577),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1577),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1577),
    [sym_duet_dev_type_dss] = ACTIONS(1577),
    [sym_duet_dev_type_dvr] = ACTIONS(1577),
    [sym_duet_dev_type_disc_device] = ACTIONS(1577),
    [sym_duet_dev_type_document_camera] = ACTIONS(1577),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1577),
    [sym_duet_dev_type_hvac] = ACTIONS(1577),
    [sym_duet_dev_type_keypad] = ACTIONS(1577),
    [sym_duet_dev_type_light] = ACTIONS(1577),
    [sym_duet_dev_type_monitor] = ACTIONS(1577),
    [sym_duet_dev_type_motor] = ACTIONS(1577),
    [sym_duet_dev_type_multi_window] = ACTIONS(1577),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1577),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1577),
    [sym_duet_dev_type_receiver] = ACTIONS(1577),
    [sym_duet_dev_type_security_system] = ACTIONS(1577),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1577),
    [sym_duet_dev_type_settop_box] = ACTIONS(1577),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1577),
    [sym_duet_dev_type_switcher] = ACTIONS(1577),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1577),
    [sym_duet_dev_type_tv] = ACTIONS(1577),
    [sym_duet_dev_type_utility] = ACTIONS(1577),
    [sym_duet_dev_type_vcr] = ACTIONS(1577),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1577),
    [sym_duet_dev_type_video_processor] = ACTIONS(1577),
    [sym_duet_dev_type_video_projector] = ACTIONS(1577),
    [sym_duet_dev_type_video_wall] = ACTIONS(1577),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1577),
    [sym_duet_dev_type_weather] = ACTIONS(1577),
    [sym_duet_dev_not_polled] = ACTIONS(1577),
    [sym_duet_dev_polled] = ACTIONS(1577),
    [sym_clkmgr_mode_network] = ACTIONS(1577),
    [sym_clkmgr_mode_standalone] = ACTIONS(1577),
    [sym_normal_standby] = ACTIONS(1577),
    [sym_normal_wake] = ACTIONS(1577),
    [sym_ip_multicast_ttl_option] = ACTIONS(1577),
    [sym_ip_ttl_subnet] = ACTIONS(1577),
    [sym_ip_ttl_site] = ACTIONS(1577),
    [sym_ip_ttl_region] = ACTIONS(1577),
    [sym_ip_ttl_continent] = ACTIONS(1577),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1577),
    [sym_ip_nodelay_on] = ACTIONS(1577),
    [sym_ip_nodelay_off] = ACTIONS(1577),
    [sym_amx_error] = ACTIONS(1577),
    [sym_amx_warning] = ACTIONS(1577),
    [sym_amx_info] = ACTIONS(1577),
    [sym_amx_debug] = ACTIONS(1577),
    [sym_netlinx_login_success] = ACTIONS(1577),
    [sym_netlinx_login_fail] = ACTIONS(1577),
    [sym_netlinx_logout] = ACTIONS(1577),
    [sym_null_str] = ACTIONS(1577),
    [sym_smtp_address] = ACTIONS(1577),
    [sym_smtp_port_number] = ACTIONS(1577),
    [sym_smtp_username] = ACTIONS(1577),
    [sym_smtp_password] = ACTIONS(1577),
    [sym_smtp_require_tls] = ACTIONS(1577),
    [sym_smtp_from] = ACTIONS(1577),
    [sym_smtp_tls_true] = ACTIONS(1577),
    [sym_smtp_tls_false] = ACTIONS(1577),
    [sym_listview_on_row_select_event] = ACTIONS(1577),
    [sym_data_structure_unknown] = ACTIONS(1577),
    [sym_data_structure_datafeed] = ACTIONS(1577),
    [sym_data_structure_datarecordset] = ACTIONS(1577),
    [sym_data_structure_datarecord] = ACTIONS(1577),
    [sym_data_structure_datafield] = ACTIONS(1577),
    [sym_data_max_content_field_count] = ACTIONS(1577),
    [sym_data_max_metadata_field_count] = ACTIONS(1577),
    [sym_data_max_id_length] = ACTIONS(1577),
    [sym_data_max_name_length] = ACTIONS(1577),
    [sym_data_max_description_length] = ACTIONS(1577),
    [sym_data_max_type_length] = ACTIONS(1577),
    [sym_data_max_format_length] = ACTIONS(1577),
    [sym_data_max_source_length] = ACTIONS(1577),
    [sym_data_max_value_length] = ACTIONS(1577),
    [sym_data_max_label_length] = ACTIONS(1577),
    [sym_data_max_url_length] = ACTIONS(1577),
    [sym_max_last_login_info_length] = ACTIONS(1577),
    [sym_data_type_unknown] = ACTIONS(1577),
    [sym_data_type_string] = ACTIONS(1577),
    [sym_data_type_datetime] = ACTIONS(1577),
    [sym_data_type_date] = ACTIONS(1577),
    [sym_data_type_time] = ACTIONS(1577),
    [sym_data_type_image] = ACTIONS(1577),
    [sym_data_format_url] = ACTIONS(1577),
    [sym_data_format_phone] = ACTIONS(1577),
    [sym_data_format_email] = ACTIONS(1577),
    [sym_data_format_iso8601] = ACTIONS(1577),
    [sym_auditlog_priv] = ACTIONS(1577),
    [sym_configuration_priv] = ACTIONS(1577),
    [sym_deviceconfig_priv] = ACTIONS(1577),
    [sym_ftp_priv] = ACTIONS(1577),
    [sym_http_priv] = ACTIONS(1577),
    [sym_networkconfig_priv] = ACTIONS(1577),
    [sym_programport_priv] = ACTIONS(1577),
    [sym_remoteui_priv] = ACTIONS(1577),
    [sym_securitycontrol_priv] = ACTIONS(1577),
    [sym_softwaremanagement_priv] = ACTIONS(1577),
    [sym_terminal_priv] = ACTIONS(1577),
    [sym_tpadmin_priv] = ACTIONS(1577),
    [sym_user1_priv] = ACTIONS(1577),
    [sym_user2_priv] = ACTIONS(1577),
    [sym_user3_priv] = ACTIONS(1577),
    [sym_user4_priv] = ACTIONS(1577),
    [sym_usermanagement_priv] = ACTIONS(1577),
    [sym_valid_account] = ACTIONS(1577),
    [sym_err_name_invalid] = ACTIONS(1577),
    [sym_err_password_invalid] = ACTIONS(1577),
    [sym_err_invalid_account] = ACTIONS(1577),
    [sym_err_no_authorization] = ACTIONS(1577),
    [sym_err_invalid_parameter] = ACTIONS(1577),
    [sym_err_user_locked_out] = ACTIONS(1577),
    [sym_err_user_account_expired] = ACTIONS(1577),
    [sym_err_general_error] = ACTIONS(1577),
    [sym_err_authentication_error] = ACTIONS(1577),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1577),
    [sym_tls_validate_certificate] = ACTIONS(1577),
    [sym____reserved___] = ACTIONS(1577),
    [sym_date] = ACTIONS(1577),
    [sym_day] = ACTIONS(1577),
    [sym_time] = ACTIONS(1577),
    [sym_ldate] = ACTIONS(1577),
    [sym_push_device] = ACTIONS(1577),
    [sym_push_channel] = ACTIONS(1577),
    [sym_push_devchan] = ACTIONS(1577),
    [sym_release_device] = ACTIONS(1577),
    [sym_release_channel] = ACTIONS(1577),
    [sym_release_devchan] = ACTIONS(1577),
    [sym_master_slot] = ACTIONS(1577),
    [sym_get_pulse_time] = ACTIONS(1577),
    [sym_get_timer] = ACTIONS(1577),
    [sym_master_sn] = ACTIONS(1577),
    [sym_system_number] = ACTIONS(1577),
    [sym_button] = ACTIONS(1577),
    [sym_dv_channel] = ACTIONS(1577),
    [sym_data] = ACTIONS(1577),
    [sym_level] = ACTIONS(1577),
    [sym_timeline] = ACTIONS(1577),
    [sym_channel] = ACTIONS(1577),
    [sym_custom] = ACTIONS(1577),
    [sym_atoi] = ACTIONS(1577),
    [sym_atol] = ACTIONS(1577),
    [sym_atof] = ACTIONS(1577),
    [sym_itoa] = ACTIONS(1577),
    [sym_itohex] = ACTIONS(1577),
    [sym_hextoi] = ACTIONS(1577),
    [sym_ftoa] = ACTIONS(1577),
    [sym_find_string] = ACTIONS(1577),
    [sym_left_string] = ACTIONS(1577),
    [sym_right_string] = ACTIONS(1577),
    [sym_mid_string] = ACTIONS(1577),
    [sym_remove_string] = ACTIONS(1577),
    [sym_get_buffer_string] = ACTIONS(1577),
    [sym_get_buffer_char] = ACTIONS(1577),
    [sym_get_multi_buffer_string] = ACTIONS(1577),
    [sym_length_string] = ACTIONS(1577),
    [sym_max_length_string] = ACTIONS(1577),
    [sym_set_length_string] = ACTIONS(1577),
    [sym_compare_string] = ACTIONS(1577),
    [sym_upper_string] = ACTIONS(1577),
    [sym_lower_string] = ACTIONS(1577),
    [sym_get_last] = ACTIONS(1577),
    [sym_date_to_day] = ACTIONS(1577),
    [sym_date_to_month] = ACTIONS(1577),
    [sym_date_to_year] = ACTIONS(1577),
    [sym_time_to_hour] = ACTIONS(1577),
    [sym_time_to_minute] = ACTIONS(1577),
    [sym_time_to_second] = ACTIONS(1577),
    [sym_day_of_week] = ACTIONS(1577),
    [sym_set_pulse_time] = ACTIONS(1577),
    [sym_set_timer] = ACTIONS(1577),
    [sym_file_open] = ACTIONS(1577),
    [sym_file_close] = ACTIONS(1577),
    [sym_file_read] = ACTIONS(1577),
    [sym_file_read_line] = ACTIONS(1577),
    [sym_file_write] = ACTIONS(1577),
    [sym_file_write_line] = ACTIONS(1577),
    [sym_file_copy] = ACTIONS(1577),
    [sym_file_delete] = ACTIONS(1577),
    [sym_file_rename] = ACTIONS(1577),
    [sym_file_dir] = ACTIONS(1577),
    [sym_file_getdir] = ACTIONS(1577),
    [sym_file_setdir] = ACTIONS(1577),
    [sym_file_create_dir] = ACTIONS(1577),
    [sym_file_remove_dir] = ACTIONS(1577),
    [sym_file_seek] = ACTIONS(1577),
    [sym_length_array] = ACTIONS(1577),
    [sym_max_length_array] = ACTIONS(1577),
    [sym_set_length_array] = ACTIONS(1577),
    [sym_ip_client_open] = ACTIONS(1577),
    [sym_ip_client_close] = ACTIONS(1577),
    [sym_ip_server_open] = ACTIONS(1577),
    [sym_ip_server_close] = ACTIONS(1577),
    [sym_ip_bound_client_open] = ACTIONS(1577),
    [sym_ip_mc_server_open] = ACTIONS(1577),
    [sym_ip_set_option] = ACTIONS(1577),
    [sym_ssh_client_open] = ACTIONS(1577),
    [sym_ssh_client_close] = ACTIONS(1577),
    [sym_tls_client_open] = ACTIONS(1577),
    [sym_tls_client_close] = ACTIONS(1577),
    [sym_do_push] = ACTIONS(1577),
    [sym_do_push_timed] = ACTIONS(1577),
    [sym_do_release] = ACTIONS(1577),
    [sym_device_id] = ACTIONS(1577),
    [sym_device_info] = ACTIONS(1577),
    [sym_device_id_string] = ACTIONS(1577),
    [sym_device_standby] = ACTIONS(1577),
    [sym_device_wake] = ACTIONS(1577),
    [sym_timeline_create] = ACTIONS(1577),
    [sym_timeline_kill] = ACTIONS(1577),
    [sym_timeline_reload] = ACTIONS(1577),
    [sym_timeline_pause] = ACTIONS(1577),
    [sym_timeline_restart] = ACTIONS(1577),
    [sym_timeline_set] = ACTIONS(1577),
    [sym_timeline_get] = ACTIONS(1577),
    [sym_timeline_active] = ACTIONS(1577),
    [sym_timeline_dynamic_id] = ACTIONS(1577),
    [sym_variable_to_string] = ACTIONS(1577),
    [sym_string_to_variable] = ACTIONS(1577),
    [sym_length_variable_to_string] = ACTIONS(1577),
    [sym_type_cast] = ACTIONS(1577),
    [sym_variable_to_xml] = ACTIONS(1577),
    [sym_xml_to_variable] = ACTIONS(1577),
    [sym_length_variable_to_xml] = ACTIONS(1577),
    [sym_redirect_string] = ACTIONS(1577),
    [sym_random_number] = ACTIONS(1577),
    [sym_get_unique_id] = ACTIONS(1577),
    [sym_get_system_number] = ACTIONS(1577),
    [sym_set_system_number] = ACTIONS(1577),
    [sym_get_serial_number] = ACTIONS(1577),
    [sym_get_master_build] = ACTIONS(1577),
    [sym_set_validation_code] = ACTIONS(1577),
    [sym_reboot] = ACTIONS(1577),
    [sym_load_duet_module] = ACTIONS(1577),
    [sym_unload_duet_module] = ACTIONS(1577),
    [sym_duet_mem_size_set] = ACTIONS(1577),
    [sym_duet_mem_size_get] = ACTIONS(1577),
    [sym_dynamic_application_device] = ACTIONS(1577),
    [sym_dynamic_polled_port] = ACTIONS(1577),
    [sym_static_port_binding] = ACTIONS(1577),
    [sym_static_ip_binding] = ACTIONS(1577),
    [sym_get_dns_list] = ACTIONS(1577),
    [sym_set_dns_list] = ACTIONS(1577),
    [sym_get_ip_address] = ACTIONS(1577),
    [sym_set_ip_address] = ACTIONS(1577),
    [sym_get_url_list] = ACTIONS(1577),
    [sym_add_url_entry] = ACTIONS(1577),
    [sym_delete_url_entry] = ACTIONS(1577),
    [sym_abs_value] = ACTIONS(1577),
    [sym_max_value] = ACTIONS(1577),
    [sym_min_value] = ACTIONS(1577),
    [sym_exp_value] = ACTIONS(1577),
    [sym_log_value] = ACTIONS(1577),
    [sym_log10_value] = ACTIONS(1577),
    [sym_power_value] = ACTIONS(1577),
    [sym_sqrt_value] = ACTIONS(1577),
    [sym_raw_be] = ACTIONS(1577),
    [sym_raw_le] = ACTIONS(1577),
    [sym_internal_queue_size_set] = ACTIONS(1577),
    [sym_internal_queue_size_get] = ACTIONS(1577),
    [sym_internal_threshold_set] = ACTIONS(1577),
    [sym_internal_threshold_get] = ACTIONS(1577),
    [sym_rebuilt_event] = ACTIONS(1577),
    [sym_do_custom_event] = ACTIONS(1577),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1577),
    [sym_clkmgr_set_clk_source] = ACTIONS(1577),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1577),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1577),
    [sym_clkmgr_get_timezone] = ACTIONS(1577),
    [sym_clkmgr_set_timezone] = ACTIONS(1577),
    [sym_clkmgr_get_resync_period] = ACTIONS(1577),
    [sym_clkmgr_set_resync_period] = ACTIONS(1577),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1577),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1577),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1577),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1577),
    [sym_clkmgr_get_timeservers] = ACTIONS(1577),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1577),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1577),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1577),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1577),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1577),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1577),
    [sym_set_log_level] = ACTIONS(1577),
    [sym_get_log_level] = ACTIONS(1577),
    [sym_amx_log] = ACTIONS(1577),
    [sym_smtp_server_config_set] = ACTIONS(1577),
    [sym_smtp_server_config_get] = ACTIONS(1577),
    [sym_smtp_send] = ACTIONS(1577),
    [sym_validate_netlinx_account] = ACTIONS(1577),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1577),
    [sym_authenticate_certificate] = ACTIONS(1577),
    [sym_audit_netlinx_session_event] = ACTIONS(1577),
    [sym_audit_netlinx_generic_event] = ACTIONS(1577),
    [sym_get_available_flash_disk_space] = ACTIONS(1577),
    [sym_get_max_flash_disk_space] = ACTIONS(1577),
    [sym_data_create_feed] = ACTIONS(1577),
    [sym_data_delete_feed] = ACTIONS(1577),
    [sym_data_publish_feed] = ACTIONS(1577),
    [sym_data_get_published_feed] = ACTIONS(1577),
    [sym_data_add_record] = ACTIONS(1577),
    [sym_data_get_event_record] = ACTIONS(1577),
    [sym__wc_data_create_feed] = ACTIONS(1577),
    [sym__wc_data_add_record] = ACTIONS(1577),
    [sym__wc_data_get_event_record] = ACTIONS(1577),
    [sym_format] = ACTIONS(1577),
    [sym_astro_clock] = ACTIONS(1577),
    [sym_set_outdoor_temperature] = ACTIONS(1577),
    [sym_set_virtual_level_count] = ACTIONS(1577),
    [sym_set_virtual_channel_count] = ACTIONS(1577),
    [sym_set_virtual_port_count] = ACTIONS(1577),
    [sym__wc_encode_int] = ACTIONS(1577),
    [sym__wc_decode_int] = ACTIONS(1577),
    [sym__wc_int] = ACTIONS(1577),
    [sym__wc_to_ch_int] = ACTIONS(1577),
    [sym__ch_to_wc_int] = ACTIONS(1577),
    [sym__wc_find_string_int] = ACTIONS(1577),
    [sym__wc_left_string_int] = ACTIONS(1577),
    [sym__wc_lower_string_int] = ACTIONS(1577),
    [sym__wc_mid_string_int] = ACTIONS(1577),
    [sym__wc_remove_string_int] = ACTIONS(1577),
    [sym__wc_right_string_int] = ACTIONS(1577),
    [sym__wc_upper_string_int] = ACTIONS(1577),
    [sym__wc_compare_string_int] = ACTIONS(1577),
    [sym__wc_get_buffer_char_int] = ACTIONS(1577),
    [sym__wc_get_buffer_string_int] = ACTIONS(1577),
    [sym__wc_concat_string_int] = ACTIONS(1577),
    [sym__wc_file_open_int] = ACTIONS(1577),
    [sym__wc_file_close_int] = ACTIONS(1577),
    [sym__wc_file_read_int] = ACTIONS(1577),
    [sym__wc_file_read_line_int] = ACTIONS(1577),
    [sym__wc_file_write_int] = ACTIONS(1577),
    [sym__wc_file_write_line_int] = ACTIONS(1577),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1579),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1579),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1579),
    [aux_sym_preproc_else_token1] = ACTIONS(1579),
    [anon_sym_COMMA] = ACTIONS(1579),
    [anon_sym_LBRACK] = ACTIONS(1579),
    [anon_sym_LBRACE] = ACTIONS(1579),
    [anon_sym_RBRACE] = ACTIONS(1579),
    [anon_sym_COLON] = ACTIONS(1579),
    [anon_sym_LPAREN] = ACTIONS(1577),
    [anon_sym_BANG] = ACTIONS(1577),
    [anon_sym_TILDE] = ACTIONS(1579),
    [anon_sym_DASH] = ACTIONS(1577),
    [anon_sym_PLUS] = ACTIONS(1577),
    [anon_sym_STAR] = ACTIONS(1579),
    [anon_sym_SLASH] = ACTIONS(1577),
    [anon_sym_PERCENT] = ACTIONS(1579),
    [anon_sym_PIPE_PIPE] = ACTIONS(1579),
    [anon_sym_AMP_AMP] = ACTIONS(1579),
    [anon_sym_PIPE] = ACTIONS(1577),
    [anon_sym_CARET] = ACTIONS(1579),
    [anon_sym_AMP] = ACTIONS(1577),
    [anon_sym_EQ_EQ] = ACTIONS(1579),
    [anon_sym_BANG_EQ] = ACTIONS(1579),
    [anon_sym_LT_GT] = ACTIONS(1579),
    [anon_sym_GT] = ACTIONS(1577),
    [anon_sym_GT_EQ] = ACTIONS(1579),
    [anon_sym_LT_EQ] = ACTIONS(1579),
    [anon_sym_LT] = ACTIONS(1577),
    [anon_sym_LT_LT] = ACTIONS(1579),
    [anon_sym_GT_GT] = ACTIONS(1579),
    [anon_sym_DASH_DASH] = ACTIONS(1579),
    [anon_sym_PLUS_PLUS] = ACTIONS(1579),
    [anon_sym_DOT] = ACTIONS(1577),
    [anon_sym_DQUOTE] = ACTIONS(1579),
    [anon_sym_SQUOTE] = ACTIONS(1579),
    [sym_number_literal] = ACTIONS(1579),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(607)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2217),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2217),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2217),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2223),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2217),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(608)] = {
    [sym_identifier] = ACTIONS(1561),
    [sym_and] = ACTIONS(1561),
    [sym_or] = ACTIONS(1561),
    [sym_not] = ACTIONS(1561),
    [sym_xor] = ACTIONS(1561),
    [sym_band] = ACTIONS(1561),
    [sym_bor] = ACTIONS(1561),
    [sym_bxor] = ACTIONS(1561),
    [sym_bnot] = ACTIONS(1561),
    [sym_lshift] = ACTIONS(1561),
    [sym_rshift] = ACTIONS(1561),
    [sym___file__] = ACTIONS(1561),
    [sym___date__] = ACTIONS(1561),
    [sym___ldate__] = ACTIONS(1561),
    [sym___time__] = ACTIONS(1561),
    [sym___line__] = ACTIONS(1561),
    [sym___name__] = ACTIONS(1561),
    [sym___netlinx__] = ACTIONS(1561),
    [sym_netlinx_axi_version] = ACTIONS(1561),
    [sym_url_flg_tcp] = ACTIONS(1561),
    [sym_url_flg_accinfopresent] = ACTIONS(1561),
    [sym_url_flg_temp] = ACTIONS(1561),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1561),
    [sym_url_flg_stat_mask] = ACTIONS(1561),
    [sym_url_flg_stat_lookup] = ACTIONS(1561),
    [sym_url_flg_stat_connecting] = ACTIONS(1561),
    [sym_url_flg_stat_waiting] = ACTIONS(1561),
    [sym_url_flg_stat_connected] = ACTIONS(1561),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1561),
    [sym_first_virtual_device] = ACTIONS(1561),
    [sym_first_local_port] = ACTIONS(1561),
    [sym_dynamic_virtual_device] = ACTIONS(1561),
    [sym_timeline_once] = ACTIONS(1561),
    [sym_timeline_repeat] = ACTIONS(1561),
    [sym_timeline_absolute] = ACTIONS(1561),
    [sym_timeline_relative] = ACTIONS(1561),
    [sym_do_push_timed_infinite] = ACTIONS(1561),
    [sym_source_type_no_address] = ACTIONS(1561),
    [sym_source_type_neuron_id] = ACTIONS(1561),
    [sym_source_type_ip_address] = ACTIONS(1561),
    [sym_source_type_axlink] = ACTIONS(1561),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1561),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1561),
    [sym_source_type_ip_socket_address] = ACTIONS(1561),
    [sym_source_type_rs232] = ACTIONS(1561),
    [sym_source_type_internal] = ACTIONS(1561),
    [sym_source_type_mac_address] = ACTIONS(1561),
    [sym_source_type_ipv4_port] = ACTIONS(1561),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1561),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1561),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1561),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1561),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1561),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1561),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1561),
    [sym_true] = ACTIONS(1561),
    [sym_false] = ACTIONS(1561),
    [sym_file_read_only] = ACTIONS(1561),
    [sym_file_rw_new] = ACTIONS(1561),
    [sym_file_rw_append] = ACTIONS(1561),
    [sym_ip_tcp] = ACTIONS(1561),
    [sym_ip_udp] = ACTIONS(1561),
    [sym_ip_udp_2way] = ACTIONS(1561),
    [sym_xml_encode_types] = ACTIONS(1561),
    [sym_xml_encode_char_as_list] = ACTIONS(1561),
    [sym_xml_encode_le] = ACTIONS(1561),
    [sym_xml_decode_types] = ACTIONS(1561),
    [sym_xml_decode_no_preserve] = ACTIONS(1561),
    [sym_duet_dev_type_display_device] = ACTIONS(1561),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1561),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1561),
    [sym_duet_dev_type_io_device] = ACTIONS(1561),
    [sym_duet_dev_type_relay_device] = ACTIONS(1561),
    [sym_duet_dev_type_ups] = ACTIONS(1561),
    [sym_duet_dev_type_amplifier] = ACTIONS(1561),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1561),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1561),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1561),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1561),
    [sym_duet_dev_type_camera] = ACTIONS(1561),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1561),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1561),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1561),
    [sym_duet_dev_type_dss] = ACTIONS(1561),
    [sym_duet_dev_type_dvr] = ACTIONS(1561),
    [sym_duet_dev_type_disc_device] = ACTIONS(1561),
    [sym_duet_dev_type_document_camera] = ACTIONS(1561),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1561),
    [sym_duet_dev_type_hvac] = ACTIONS(1561),
    [sym_duet_dev_type_keypad] = ACTIONS(1561),
    [sym_duet_dev_type_light] = ACTIONS(1561),
    [sym_duet_dev_type_monitor] = ACTIONS(1561),
    [sym_duet_dev_type_motor] = ACTIONS(1561),
    [sym_duet_dev_type_multi_window] = ACTIONS(1561),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1561),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1561),
    [sym_duet_dev_type_receiver] = ACTIONS(1561),
    [sym_duet_dev_type_security_system] = ACTIONS(1561),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1561),
    [sym_duet_dev_type_settop_box] = ACTIONS(1561),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1561),
    [sym_duet_dev_type_switcher] = ACTIONS(1561),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1561),
    [sym_duet_dev_type_tv] = ACTIONS(1561),
    [sym_duet_dev_type_utility] = ACTIONS(1561),
    [sym_duet_dev_type_vcr] = ACTIONS(1561),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1561),
    [sym_duet_dev_type_video_processor] = ACTIONS(1561),
    [sym_duet_dev_type_video_projector] = ACTIONS(1561),
    [sym_duet_dev_type_video_wall] = ACTIONS(1561),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1561),
    [sym_duet_dev_type_weather] = ACTIONS(1561),
    [sym_duet_dev_not_polled] = ACTIONS(1561),
    [sym_duet_dev_polled] = ACTIONS(1561),
    [sym_clkmgr_mode_network] = ACTIONS(1561),
    [sym_clkmgr_mode_standalone] = ACTIONS(1561),
    [sym_normal_standby] = ACTIONS(1561),
    [sym_normal_wake] = ACTIONS(1561),
    [sym_ip_multicast_ttl_option] = ACTIONS(1561),
    [sym_ip_ttl_subnet] = ACTIONS(1561),
    [sym_ip_ttl_site] = ACTIONS(1561),
    [sym_ip_ttl_region] = ACTIONS(1561),
    [sym_ip_ttl_continent] = ACTIONS(1561),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1561),
    [sym_ip_nodelay_on] = ACTIONS(1561),
    [sym_ip_nodelay_off] = ACTIONS(1561),
    [sym_amx_error] = ACTIONS(1561),
    [sym_amx_warning] = ACTIONS(1561),
    [sym_amx_info] = ACTIONS(1561),
    [sym_amx_debug] = ACTIONS(1561),
    [sym_netlinx_login_success] = ACTIONS(1561),
    [sym_netlinx_login_fail] = ACTIONS(1561),
    [sym_netlinx_logout] = ACTIONS(1561),
    [sym_null_str] = ACTIONS(1561),
    [sym_smtp_address] = ACTIONS(1561),
    [sym_smtp_port_number] = ACTIONS(1561),
    [sym_smtp_username] = ACTIONS(1561),
    [sym_smtp_password] = ACTIONS(1561),
    [sym_smtp_require_tls] = ACTIONS(1561),
    [sym_smtp_from] = ACTIONS(1561),
    [sym_smtp_tls_true] = ACTIONS(1561),
    [sym_smtp_tls_false] = ACTIONS(1561),
    [sym_listview_on_row_select_event] = ACTIONS(1561),
    [sym_data_structure_unknown] = ACTIONS(1561),
    [sym_data_structure_datafeed] = ACTIONS(1561),
    [sym_data_structure_datarecordset] = ACTIONS(1561),
    [sym_data_structure_datarecord] = ACTIONS(1561),
    [sym_data_structure_datafield] = ACTIONS(1561),
    [sym_data_max_content_field_count] = ACTIONS(1561),
    [sym_data_max_metadata_field_count] = ACTIONS(1561),
    [sym_data_max_id_length] = ACTIONS(1561),
    [sym_data_max_name_length] = ACTIONS(1561),
    [sym_data_max_description_length] = ACTIONS(1561),
    [sym_data_max_type_length] = ACTIONS(1561),
    [sym_data_max_format_length] = ACTIONS(1561),
    [sym_data_max_source_length] = ACTIONS(1561),
    [sym_data_max_value_length] = ACTIONS(1561),
    [sym_data_max_label_length] = ACTIONS(1561),
    [sym_data_max_url_length] = ACTIONS(1561),
    [sym_max_last_login_info_length] = ACTIONS(1561),
    [sym_data_type_unknown] = ACTIONS(1561),
    [sym_data_type_string] = ACTIONS(1561),
    [sym_data_type_datetime] = ACTIONS(1561),
    [sym_data_type_date] = ACTIONS(1561),
    [sym_data_type_time] = ACTIONS(1561),
    [sym_data_type_image] = ACTIONS(1561),
    [sym_data_format_url] = ACTIONS(1561),
    [sym_data_format_phone] = ACTIONS(1561),
    [sym_data_format_email] = ACTIONS(1561),
    [sym_data_format_iso8601] = ACTIONS(1561),
    [sym_auditlog_priv] = ACTIONS(1561),
    [sym_configuration_priv] = ACTIONS(1561),
    [sym_deviceconfig_priv] = ACTIONS(1561),
    [sym_ftp_priv] = ACTIONS(1561),
    [sym_http_priv] = ACTIONS(1561),
    [sym_networkconfig_priv] = ACTIONS(1561),
    [sym_programport_priv] = ACTIONS(1561),
    [sym_remoteui_priv] = ACTIONS(1561),
    [sym_securitycontrol_priv] = ACTIONS(1561),
    [sym_softwaremanagement_priv] = ACTIONS(1561),
    [sym_terminal_priv] = ACTIONS(1561),
    [sym_tpadmin_priv] = ACTIONS(1561),
    [sym_user1_priv] = ACTIONS(1561),
    [sym_user2_priv] = ACTIONS(1561),
    [sym_user3_priv] = ACTIONS(1561),
    [sym_user4_priv] = ACTIONS(1561),
    [sym_usermanagement_priv] = ACTIONS(1561),
    [sym_valid_account] = ACTIONS(1561),
    [sym_err_name_invalid] = ACTIONS(1561),
    [sym_err_password_invalid] = ACTIONS(1561),
    [sym_err_invalid_account] = ACTIONS(1561),
    [sym_err_no_authorization] = ACTIONS(1561),
    [sym_err_invalid_parameter] = ACTIONS(1561),
    [sym_err_user_locked_out] = ACTIONS(1561),
    [sym_err_user_account_expired] = ACTIONS(1561),
    [sym_err_general_error] = ACTIONS(1561),
    [sym_err_authentication_error] = ACTIONS(1561),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1561),
    [sym_tls_validate_certificate] = ACTIONS(1561),
    [sym____reserved___] = ACTIONS(1561),
    [sym_date] = ACTIONS(1561),
    [sym_day] = ACTIONS(1561),
    [sym_time] = ACTIONS(1561),
    [sym_ldate] = ACTIONS(1561),
    [sym_push_device] = ACTIONS(1561),
    [sym_push_channel] = ACTIONS(1561),
    [sym_push_devchan] = ACTIONS(1561),
    [sym_release_device] = ACTIONS(1561),
    [sym_release_channel] = ACTIONS(1561),
    [sym_release_devchan] = ACTIONS(1561),
    [sym_master_slot] = ACTIONS(1561),
    [sym_get_pulse_time] = ACTIONS(1561),
    [sym_get_timer] = ACTIONS(1561),
    [sym_master_sn] = ACTIONS(1561),
    [sym_system_number] = ACTIONS(1561),
    [sym_button] = ACTIONS(1561),
    [sym_dv_channel] = ACTIONS(1561),
    [sym_data] = ACTIONS(1561),
    [sym_level] = ACTIONS(1561),
    [sym_timeline] = ACTIONS(1561),
    [sym_channel] = ACTIONS(1561),
    [sym_custom] = ACTIONS(1561),
    [sym_atoi] = ACTIONS(1561),
    [sym_atol] = ACTIONS(1561),
    [sym_atof] = ACTIONS(1561),
    [sym_itoa] = ACTIONS(1561),
    [sym_itohex] = ACTIONS(1561),
    [sym_hextoi] = ACTIONS(1561),
    [sym_ftoa] = ACTIONS(1561),
    [sym_find_string] = ACTIONS(1561),
    [sym_left_string] = ACTIONS(1561),
    [sym_right_string] = ACTIONS(1561),
    [sym_mid_string] = ACTIONS(1561),
    [sym_remove_string] = ACTIONS(1561),
    [sym_get_buffer_string] = ACTIONS(1561),
    [sym_get_buffer_char] = ACTIONS(1561),
    [sym_get_multi_buffer_string] = ACTIONS(1561),
    [sym_length_string] = ACTIONS(1561),
    [sym_max_length_string] = ACTIONS(1561),
    [sym_set_length_string] = ACTIONS(1561),
    [sym_compare_string] = ACTIONS(1561),
    [sym_upper_string] = ACTIONS(1561),
    [sym_lower_string] = ACTIONS(1561),
    [sym_get_last] = ACTIONS(1561),
    [sym_date_to_day] = ACTIONS(1561),
    [sym_date_to_month] = ACTIONS(1561),
    [sym_date_to_year] = ACTIONS(1561),
    [sym_time_to_hour] = ACTIONS(1561),
    [sym_time_to_minute] = ACTIONS(1561),
    [sym_time_to_second] = ACTIONS(1561),
    [sym_day_of_week] = ACTIONS(1561),
    [sym_set_pulse_time] = ACTIONS(1561),
    [sym_set_timer] = ACTIONS(1561),
    [sym_file_open] = ACTIONS(1561),
    [sym_file_close] = ACTIONS(1561),
    [sym_file_read] = ACTIONS(1561),
    [sym_file_read_line] = ACTIONS(1561),
    [sym_file_write] = ACTIONS(1561),
    [sym_file_write_line] = ACTIONS(1561),
    [sym_file_copy] = ACTIONS(1561),
    [sym_file_delete] = ACTIONS(1561),
    [sym_file_rename] = ACTIONS(1561),
    [sym_file_dir] = ACTIONS(1561),
    [sym_file_getdir] = ACTIONS(1561),
    [sym_file_setdir] = ACTIONS(1561),
    [sym_file_create_dir] = ACTIONS(1561),
    [sym_file_remove_dir] = ACTIONS(1561),
    [sym_file_seek] = ACTIONS(1561),
    [sym_length_array] = ACTIONS(1561),
    [sym_max_length_array] = ACTIONS(1561),
    [sym_set_length_array] = ACTIONS(1561),
    [sym_ip_client_open] = ACTIONS(1561),
    [sym_ip_client_close] = ACTIONS(1561),
    [sym_ip_server_open] = ACTIONS(1561),
    [sym_ip_server_close] = ACTIONS(1561),
    [sym_ip_bound_client_open] = ACTIONS(1561),
    [sym_ip_mc_server_open] = ACTIONS(1561),
    [sym_ip_set_option] = ACTIONS(1561),
    [sym_ssh_client_open] = ACTIONS(1561),
    [sym_ssh_client_close] = ACTIONS(1561),
    [sym_tls_client_open] = ACTIONS(1561),
    [sym_tls_client_close] = ACTIONS(1561),
    [sym_do_push] = ACTIONS(1561),
    [sym_do_push_timed] = ACTIONS(1561),
    [sym_do_release] = ACTIONS(1561),
    [sym_device_id] = ACTIONS(1561),
    [sym_device_info] = ACTIONS(1561),
    [sym_device_id_string] = ACTIONS(1561),
    [sym_device_standby] = ACTIONS(1561),
    [sym_device_wake] = ACTIONS(1561),
    [sym_timeline_create] = ACTIONS(1561),
    [sym_timeline_kill] = ACTIONS(1561),
    [sym_timeline_reload] = ACTIONS(1561),
    [sym_timeline_pause] = ACTIONS(1561),
    [sym_timeline_restart] = ACTIONS(1561),
    [sym_timeline_set] = ACTIONS(1561),
    [sym_timeline_get] = ACTIONS(1561),
    [sym_timeline_active] = ACTIONS(1561),
    [sym_timeline_dynamic_id] = ACTIONS(1561),
    [sym_variable_to_string] = ACTIONS(1561),
    [sym_string_to_variable] = ACTIONS(1561),
    [sym_length_variable_to_string] = ACTIONS(1561),
    [sym_type_cast] = ACTIONS(1561),
    [sym_variable_to_xml] = ACTIONS(1561),
    [sym_xml_to_variable] = ACTIONS(1561),
    [sym_length_variable_to_xml] = ACTIONS(1561),
    [sym_redirect_string] = ACTIONS(1561),
    [sym_random_number] = ACTIONS(1561),
    [sym_get_unique_id] = ACTIONS(1561),
    [sym_get_system_number] = ACTIONS(1561),
    [sym_set_system_number] = ACTIONS(1561),
    [sym_get_serial_number] = ACTIONS(1561),
    [sym_get_master_build] = ACTIONS(1561),
    [sym_set_validation_code] = ACTIONS(1561),
    [sym_reboot] = ACTIONS(1561),
    [sym_load_duet_module] = ACTIONS(1561),
    [sym_unload_duet_module] = ACTIONS(1561),
    [sym_duet_mem_size_set] = ACTIONS(1561),
    [sym_duet_mem_size_get] = ACTIONS(1561),
    [sym_dynamic_application_device] = ACTIONS(1561),
    [sym_dynamic_polled_port] = ACTIONS(1561),
    [sym_static_port_binding] = ACTIONS(1561),
    [sym_static_ip_binding] = ACTIONS(1561),
    [sym_get_dns_list] = ACTIONS(1561),
    [sym_set_dns_list] = ACTIONS(1561),
    [sym_get_ip_address] = ACTIONS(1561),
    [sym_set_ip_address] = ACTIONS(1561),
    [sym_get_url_list] = ACTIONS(1561),
    [sym_add_url_entry] = ACTIONS(1561),
    [sym_delete_url_entry] = ACTIONS(1561),
    [sym_abs_value] = ACTIONS(1561),
    [sym_max_value] = ACTIONS(1561),
    [sym_min_value] = ACTIONS(1561),
    [sym_exp_value] = ACTIONS(1561),
    [sym_log_value] = ACTIONS(1561),
    [sym_log10_value] = ACTIONS(1561),
    [sym_power_value] = ACTIONS(1561),
    [sym_sqrt_value] = ACTIONS(1561),
    [sym_raw_be] = ACTIONS(1561),
    [sym_raw_le] = ACTIONS(1561),
    [sym_internal_queue_size_set] = ACTIONS(1561),
    [sym_internal_queue_size_get] = ACTIONS(1561),
    [sym_internal_threshold_set] = ACTIONS(1561),
    [sym_internal_threshold_get] = ACTIONS(1561),
    [sym_rebuilt_event] = ACTIONS(1561),
    [sym_do_custom_event] = ACTIONS(1561),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1561),
    [sym_clkmgr_set_clk_source] = ACTIONS(1561),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1561),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1561),
    [sym_clkmgr_get_timezone] = ACTIONS(1561),
    [sym_clkmgr_set_timezone] = ACTIONS(1561),
    [sym_clkmgr_get_resync_period] = ACTIONS(1561),
    [sym_clkmgr_set_resync_period] = ACTIONS(1561),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1561),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1561),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1561),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1561),
    [sym_clkmgr_get_timeservers] = ACTIONS(1561),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1561),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1561),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1561),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1561),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1561),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1561),
    [sym_set_log_level] = ACTIONS(1561),
    [sym_get_log_level] = ACTIONS(1561),
    [sym_amx_log] = ACTIONS(1561),
    [sym_smtp_server_config_set] = ACTIONS(1561),
    [sym_smtp_server_config_get] = ACTIONS(1561),
    [sym_smtp_send] = ACTIONS(1561),
    [sym_validate_netlinx_account] = ACTIONS(1561),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1561),
    [sym_authenticate_certificate] = ACTIONS(1561),
    [sym_audit_netlinx_session_event] = ACTIONS(1561),
    [sym_audit_netlinx_generic_event] = ACTIONS(1561),
    [sym_get_available_flash_disk_space] = ACTIONS(1561),
    [sym_get_max_flash_disk_space] = ACTIONS(1561),
    [sym_data_create_feed] = ACTIONS(1561),
    [sym_data_delete_feed] = ACTIONS(1561),
    [sym_data_publish_feed] = ACTIONS(1561),
    [sym_data_get_published_feed] = ACTIONS(1561),
    [sym_data_add_record] = ACTIONS(1561),
    [sym_data_get_event_record] = ACTIONS(1561),
    [sym__wc_data_create_feed] = ACTIONS(1561),
    [sym__wc_data_add_record] = ACTIONS(1561),
    [sym__wc_data_get_event_record] = ACTIONS(1561),
    [sym_format] = ACTIONS(1561),
    [sym_astro_clock] = ACTIONS(1561),
    [sym_set_outdoor_temperature] = ACTIONS(1561),
    [sym_set_virtual_level_count] = ACTIONS(1561),
    [sym_set_virtual_channel_count] = ACTIONS(1561),
    [sym_set_virtual_port_count] = ACTIONS(1561),
    [sym__wc_encode_int] = ACTIONS(1561),
    [sym__wc_decode_int] = ACTIONS(1561),
    [sym__wc_int] = ACTIONS(1561),
    [sym__wc_to_ch_int] = ACTIONS(1561),
    [sym__ch_to_wc_int] = ACTIONS(1561),
    [sym__wc_find_string_int] = ACTIONS(1561),
    [sym__wc_left_string_int] = ACTIONS(1561),
    [sym__wc_lower_string_int] = ACTIONS(1561),
    [sym__wc_mid_string_int] = ACTIONS(1561),
    [sym__wc_remove_string_int] = ACTIONS(1561),
    [sym__wc_right_string_int] = ACTIONS(1561),
    [sym__wc_upper_string_int] = ACTIONS(1561),
    [sym__wc_compare_string_int] = ACTIONS(1561),
    [sym__wc_get_buffer_char_int] = ACTIONS(1561),
    [sym__wc_get_buffer_string_int] = ACTIONS(1561),
    [sym__wc_concat_string_int] = ACTIONS(1561),
    [sym__wc_file_open_int] = ACTIONS(1561),
    [sym__wc_file_close_int] = ACTIONS(1561),
    [sym__wc_file_read_int] = ACTIONS(1561),
    [sym__wc_file_read_line_int] = ACTIONS(1561),
    [sym__wc_file_write_int] = ACTIONS(1561),
    [sym__wc_file_write_line_int] = ACTIONS(1561),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1563),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1563),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1563),
    [aux_sym_preproc_else_token1] = ACTIONS(1563),
    [anon_sym_COMMA] = ACTIONS(1563),
    [anon_sym_LBRACK] = ACTIONS(1563),
    [anon_sym_LBRACE] = ACTIONS(1563),
    [anon_sym_RBRACE] = ACTIONS(1563),
    [anon_sym_COLON] = ACTIONS(1563),
    [anon_sym_LPAREN] = ACTIONS(1561),
    [anon_sym_BANG] = ACTIONS(1561),
    [anon_sym_TILDE] = ACTIONS(1563),
    [anon_sym_DASH] = ACTIONS(1561),
    [anon_sym_PLUS] = ACTIONS(1561),
    [anon_sym_STAR] = ACTIONS(1563),
    [anon_sym_SLASH] = ACTIONS(1561),
    [anon_sym_PERCENT] = ACTIONS(1563),
    [anon_sym_PIPE_PIPE] = ACTIONS(1563),
    [anon_sym_AMP_AMP] = ACTIONS(1563),
    [anon_sym_PIPE] = ACTIONS(1561),
    [anon_sym_CARET] = ACTIONS(1563),
    [anon_sym_AMP] = ACTIONS(1561),
    [anon_sym_EQ_EQ] = ACTIONS(1563),
    [anon_sym_BANG_EQ] = ACTIONS(1563),
    [anon_sym_LT_GT] = ACTIONS(1563),
    [anon_sym_GT] = ACTIONS(1561),
    [anon_sym_GT_EQ] = ACTIONS(1563),
    [anon_sym_LT_EQ] = ACTIONS(1563),
    [anon_sym_LT] = ACTIONS(1561),
    [anon_sym_LT_LT] = ACTIONS(1563),
    [anon_sym_GT_GT] = ACTIONS(1563),
    [anon_sym_DASH_DASH] = ACTIONS(1563),
    [anon_sym_PLUS_PLUS] = ACTIONS(1563),
    [anon_sym_DOT] = ACTIONS(1561),
    [anon_sym_DQUOTE] = ACTIONS(1563),
    [anon_sym_SQUOTE] = ACTIONS(1563),
    [sym_number_literal] = ACTIONS(1563),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(609)] = {
    [sym_identifier] = ACTIONS(1597),
    [sym_and] = ACTIONS(1597),
    [sym_or] = ACTIONS(1597),
    [sym_not] = ACTIONS(1597),
    [sym_xor] = ACTIONS(1597),
    [sym_band] = ACTIONS(1597),
    [sym_bor] = ACTIONS(1597),
    [sym_bxor] = ACTIONS(1597),
    [sym_bnot] = ACTIONS(1597),
    [sym_lshift] = ACTIONS(1597),
    [sym_rshift] = ACTIONS(1597),
    [sym___file__] = ACTIONS(1597),
    [sym___date__] = ACTIONS(1597),
    [sym___ldate__] = ACTIONS(1597),
    [sym___time__] = ACTIONS(1597),
    [sym___line__] = ACTIONS(1597),
    [sym___name__] = ACTIONS(1597),
    [sym___netlinx__] = ACTIONS(1597),
    [sym_netlinx_axi_version] = ACTIONS(1597),
    [sym_url_flg_tcp] = ACTIONS(1597),
    [sym_url_flg_accinfopresent] = ACTIONS(1597),
    [sym_url_flg_temp] = ACTIONS(1597),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1597),
    [sym_url_flg_stat_mask] = ACTIONS(1597),
    [sym_url_flg_stat_lookup] = ACTIONS(1597),
    [sym_url_flg_stat_connecting] = ACTIONS(1597),
    [sym_url_flg_stat_waiting] = ACTIONS(1597),
    [sym_url_flg_stat_connected] = ACTIONS(1597),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1597),
    [sym_first_virtual_device] = ACTIONS(1597),
    [sym_first_local_port] = ACTIONS(1597),
    [sym_dynamic_virtual_device] = ACTIONS(1597),
    [sym_timeline_once] = ACTIONS(1597),
    [sym_timeline_repeat] = ACTIONS(1597),
    [sym_timeline_absolute] = ACTIONS(1597),
    [sym_timeline_relative] = ACTIONS(1597),
    [sym_do_push_timed_infinite] = ACTIONS(1597),
    [sym_source_type_no_address] = ACTIONS(1597),
    [sym_source_type_neuron_id] = ACTIONS(1597),
    [sym_source_type_ip_address] = ACTIONS(1597),
    [sym_source_type_axlink] = ACTIONS(1597),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1597),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1597),
    [sym_source_type_ip_socket_address] = ACTIONS(1597),
    [sym_source_type_rs232] = ACTIONS(1597),
    [sym_source_type_internal] = ACTIONS(1597),
    [sym_source_type_mac_address] = ACTIONS(1597),
    [sym_source_type_ipv4_port] = ACTIONS(1597),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1597),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1597),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1597),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1597),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1597),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1597),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1597),
    [sym_true] = ACTIONS(1597),
    [sym_false] = ACTIONS(1597),
    [sym_file_read_only] = ACTIONS(1597),
    [sym_file_rw_new] = ACTIONS(1597),
    [sym_file_rw_append] = ACTIONS(1597),
    [sym_ip_tcp] = ACTIONS(1597),
    [sym_ip_udp] = ACTIONS(1597),
    [sym_ip_udp_2way] = ACTIONS(1597),
    [sym_xml_encode_types] = ACTIONS(1597),
    [sym_xml_encode_char_as_list] = ACTIONS(1597),
    [sym_xml_encode_le] = ACTIONS(1597),
    [sym_xml_decode_types] = ACTIONS(1597),
    [sym_xml_decode_no_preserve] = ACTIONS(1597),
    [sym_duet_dev_type_display_device] = ACTIONS(1597),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1597),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1597),
    [sym_duet_dev_type_io_device] = ACTIONS(1597),
    [sym_duet_dev_type_relay_device] = ACTIONS(1597),
    [sym_duet_dev_type_ups] = ACTIONS(1597),
    [sym_duet_dev_type_amplifier] = ACTIONS(1597),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1597),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1597),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1597),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1597),
    [sym_duet_dev_type_camera] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1597),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1597),
    [sym_duet_dev_type_dss] = ACTIONS(1597),
    [sym_duet_dev_type_dvr] = ACTIONS(1597),
    [sym_duet_dev_type_disc_device] = ACTIONS(1597),
    [sym_duet_dev_type_document_camera] = ACTIONS(1597),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1597),
    [sym_duet_dev_type_hvac] = ACTIONS(1597),
    [sym_duet_dev_type_keypad] = ACTIONS(1597),
    [sym_duet_dev_type_light] = ACTIONS(1597),
    [sym_duet_dev_type_monitor] = ACTIONS(1597),
    [sym_duet_dev_type_motor] = ACTIONS(1597),
    [sym_duet_dev_type_multi_window] = ACTIONS(1597),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1597),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1597),
    [sym_duet_dev_type_receiver] = ACTIONS(1597),
    [sym_duet_dev_type_security_system] = ACTIONS(1597),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1597),
    [sym_duet_dev_type_settop_box] = ACTIONS(1597),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1597),
    [sym_duet_dev_type_switcher] = ACTIONS(1597),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1597),
    [sym_duet_dev_type_tv] = ACTIONS(1597),
    [sym_duet_dev_type_utility] = ACTIONS(1597),
    [sym_duet_dev_type_vcr] = ACTIONS(1597),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1597),
    [sym_duet_dev_type_video_processor] = ACTIONS(1597),
    [sym_duet_dev_type_video_projector] = ACTIONS(1597),
    [sym_duet_dev_type_video_wall] = ACTIONS(1597),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1597),
    [sym_duet_dev_type_weather] = ACTIONS(1597),
    [sym_duet_dev_not_polled] = ACTIONS(1597),
    [sym_duet_dev_polled] = ACTIONS(1597),
    [sym_clkmgr_mode_network] = ACTIONS(1597),
    [sym_clkmgr_mode_standalone] = ACTIONS(1597),
    [sym_normal_standby] = ACTIONS(1597),
    [sym_normal_wake] = ACTIONS(1597),
    [sym_ip_multicast_ttl_option] = ACTIONS(1597),
    [sym_ip_ttl_subnet] = ACTIONS(1597),
    [sym_ip_ttl_site] = ACTIONS(1597),
    [sym_ip_ttl_region] = ACTIONS(1597),
    [sym_ip_ttl_continent] = ACTIONS(1597),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1597),
    [sym_ip_nodelay_on] = ACTIONS(1597),
    [sym_ip_nodelay_off] = ACTIONS(1597),
    [sym_amx_error] = ACTIONS(1597),
    [sym_amx_warning] = ACTIONS(1597),
    [sym_amx_info] = ACTIONS(1597),
    [sym_amx_debug] = ACTIONS(1597),
    [sym_netlinx_login_success] = ACTIONS(1597),
    [sym_netlinx_login_fail] = ACTIONS(1597),
    [sym_netlinx_logout] = ACTIONS(1597),
    [sym_null_str] = ACTIONS(1597),
    [sym_smtp_address] = ACTIONS(1597),
    [sym_smtp_port_number] = ACTIONS(1597),
    [sym_smtp_username] = ACTIONS(1597),
    [sym_smtp_password] = ACTIONS(1597),
    [sym_smtp_require_tls] = ACTIONS(1597),
    [sym_smtp_from] = ACTIONS(1597),
    [sym_smtp_tls_true] = ACTIONS(1597),
    [sym_smtp_tls_false] = ACTIONS(1597),
    [sym_listview_on_row_select_event] = ACTIONS(1597),
    [sym_data_structure_unknown] = ACTIONS(1597),
    [sym_data_structure_datafeed] = ACTIONS(1597),
    [sym_data_structure_datarecordset] = ACTIONS(1597),
    [sym_data_structure_datarecord] = ACTIONS(1597),
    [sym_data_structure_datafield] = ACTIONS(1597),
    [sym_data_max_content_field_count] = ACTIONS(1597),
    [sym_data_max_metadata_field_count] = ACTIONS(1597),
    [sym_data_max_id_length] = ACTIONS(1597),
    [sym_data_max_name_length] = ACTIONS(1597),
    [sym_data_max_description_length] = ACTIONS(1597),
    [sym_data_max_type_length] = ACTIONS(1597),
    [sym_data_max_format_length] = ACTIONS(1597),
    [sym_data_max_source_length] = ACTIONS(1597),
    [sym_data_max_value_length] = ACTIONS(1597),
    [sym_data_max_label_length] = ACTIONS(1597),
    [sym_data_max_url_length] = ACTIONS(1597),
    [sym_max_last_login_info_length] = ACTIONS(1597),
    [sym_data_type_unknown] = ACTIONS(1597),
    [sym_data_type_string] = ACTIONS(1597),
    [sym_data_type_datetime] = ACTIONS(1597),
    [sym_data_type_date] = ACTIONS(1597),
    [sym_data_type_time] = ACTIONS(1597),
    [sym_data_type_image] = ACTIONS(1597),
    [sym_data_format_url] = ACTIONS(1597),
    [sym_data_format_phone] = ACTIONS(1597),
    [sym_data_format_email] = ACTIONS(1597),
    [sym_data_format_iso8601] = ACTIONS(1597),
    [sym_auditlog_priv] = ACTIONS(1597),
    [sym_configuration_priv] = ACTIONS(1597),
    [sym_deviceconfig_priv] = ACTIONS(1597),
    [sym_ftp_priv] = ACTIONS(1597),
    [sym_http_priv] = ACTIONS(1597),
    [sym_networkconfig_priv] = ACTIONS(1597),
    [sym_programport_priv] = ACTIONS(1597),
    [sym_remoteui_priv] = ACTIONS(1597),
    [sym_securitycontrol_priv] = ACTIONS(1597),
    [sym_softwaremanagement_priv] = ACTIONS(1597),
    [sym_terminal_priv] = ACTIONS(1597),
    [sym_tpadmin_priv] = ACTIONS(1597),
    [sym_user1_priv] = ACTIONS(1597),
    [sym_user2_priv] = ACTIONS(1597),
    [sym_user3_priv] = ACTIONS(1597),
    [sym_user4_priv] = ACTIONS(1597),
    [sym_usermanagement_priv] = ACTIONS(1597),
    [sym_valid_account] = ACTIONS(1597),
    [sym_err_name_invalid] = ACTIONS(1597),
    [sym_err_password_invalid] = ACTIONS(1597),
    [sym_err_invalid_account] = ACTIONS(1597),
    [sym_err_no_authorization] = ACTIONS(1597),
    [sym_err_invalid_parameter] = ACTIONS(1597),
    [sym_err_user_locked_out] = ACTIONS(1597),
    [sym_err_user_account_expired] = ACTIONS(1597),
    [sym_err_general_error] = ACTIONS(1597),
    [sym_err_authentication_error] = ACTIONS(1597),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1597),
    [sym_tls_validate_certificate] = ACTIONS(1597),
    [sym____reserved___] = ACTIONS(1597),
    [sym_date] = ACTIONS(1597),
    [sym_day] = ACTIONS(1597),
    [sym_time] = ACTIONS(1597),
    [sym_ldate] = ACTIONS(1597),
    [sym_push_device] = ACTIONS(1597),
    [sym_push_channel] = ACTIONS(1597),
    [sym_push_devchan] = ACTIONS(1597),
    [sym_release_device] = ACTIONS(1597),
    [sym_release_channel] = ACTIONS(1597),
    [sym_release_devchan] = ACTIONS(1597),
    [sym_master_slot] = ACTIONS(1597),
    [sym_get_pulse_time] = ACTIONS(1597),
    [sym_get_timer] = ACTIONS(1597),
    [sym_master_sn] = ACTIONS(1597),
    [sym_system_number] = ACTIONS(1597),
    [sym_button] = ACTIONS(1597),
    [sym_dv_channel] = ACTIONS(1597),
    [sym_data] = ACTIONS(1597),
    [sym_level] = ACTIONS(1597),
    [sym_timeline] = ACTIONS(1597),
    [sym_channel] = ACTIONS(1597),
    [sym_custom] = ACTIONS(1597),
    [sym_atoi] = ACTIONS(1597),
    [sym_atol] = ACTIONS(1597),
    [sym_atof] = ACTIONS(1597),
    [sym_itoa] = ACTIONS(1597),
    [sym_itohex] = ACTIONS(1597),
    [sym_hextoi] = ACTIONS(1597),
    [sym_ftoa] = ACTIONS(1597),
    [sym_find_string] = ACTIONS(1597),
    [sym_left_string] = ACTIONS(1597),
    [sym_right_string] = ACTIONS(1597),
    [sym_mid_string] = ACTIONS(1597),
    [sym_remove_string] = ACTIONS(1597),
    [sym_get_buffer_string] = ACTIONS(1597),
    [sym_get_buffer_char] = ACTIONS(1597),
    [sym_get_multi_buffer_string] = ACTIONS(1597),
    [sym_length_string] = ACTIONS(1597),
    [sym_max_length_string] = ACTIONS(1597),
    [sym_set_length_string] = ACTIONS(1597),
    [sym_compare_string] = ACTIONS(1597),
    [sym_upper_string] = ACTIONS(1597),
    [sym_lower_string] = ACTIONS(1597),
    [sym_get_last] = ACTIONS(1597),
    [sym_date_to_day] = ACTIONS(1597),
    [sym_date_to_month] = ACTIONS(1597),
    [sym_date_to_year] = ACTIONS(1597),
    [sym_time_to_hour] = ACTIONS(1597),
    [sym_time_to_minute] = ACTIONS(1597),
    [sym_time_to_second] = ACTIONS(1597),
    [sym_day_of_week] = ACTIONS(1597),
    [sym_set_pulse_time] = ACTIONS(1597),
    [sym_set_timer] = ACTIONS(1597),
    [sym_file_open] = ACTIONS(1597),
    [sym_file_close] = ACTIONS(1597),
    [sym_file_read] = ACTIONS(1597),
    [sym_file_read_line] = ACTIONS(1597),
    [sym_file_write] = ACTIONS(1597),
    [sym_file_write_line] = ACTIONS(1597),
    [sym_file_copy] = ACTIONS(1597),
    [sym_file_delete] = ACTIONS(1597),
    [sym_file_rename] = ACTIONS(1597),
    [sym_file_dir] = ACTIONS(1597),
    [sym_file_getdir] = ACTIONS(1597),
    [sym_file_setdir] = ACTIONS(1597),
    [sym_file_create_dir] = ACTIONS(1597),
    [sym_file_remove_dir] = ACTIONS(1597),
    [sym_file_seek] = ACTIONS(1597),
    [sym_length_array] = ACTIONS(1597),
    [sym_max_length_array] = ACTIONS(1597),
    [sym_set_length_array] = ACTIONS(1597),
    [sym_ip_client_open] = ACTIONS(1597),
    [sym_ip_client_close] = ACTIONS(1597),
    [sym_ip_server_open] = ACTIONS(1597),
    [sym_ip_server_close] = ACTIONS(1597),
    [sym_ip_bound_client_open] = ACTIONS(1597),
    [sym_ip_mc_server_open] = ACTIONS(1597),
    [sym_ip_set_option] = ACTIONS(1597),
    [sym_ssh_client_open] = ACTIONS(1597),
    [sym_ssh_client_close] = ACTIONS(1597),
    [sym_tls_client_open] = ACTIONS(1597),
    [sym_tls_client_close] = ACTIONS(1597),
    [sym_do_push] = ACTIONS(1597),
    [sym_do_push_timed] = ACTIONS(1597),
    [sym_do_release] = ACTIONS(1597),
    [sym_device_id] = ACTIONS(1597),
    [sym_device_info] = ACTIONS(1597),
    [sym_device_id_string] = ACTIONS(1597),
    [sym_device_standby] = ACTIONS(1597),
    [sym_device_wake] = ACTIONS(1597),
    [sym_timeline_create] = ACTIONS(1597),
    [sym_timeline_kill] = ACTIONS(1597),
    [sym_timeline_reload] = ACTIONS(1597),
    [sym_timeline_pause] = ACTIONS(1597),
    [sym_timeline_restart] = ACTIONS(1597),
    [sym_timeline_set] = ACTIONS(1597),
    [sym_timeline_get] = ACTIONS(1597),
    [sym_timeline_active] = ACTIONS(1597),
    [sym_timeline_dynamic_id] = ACTIONS(1597),
    [sym_variable_to_string] = ACTIONS(1597),
    [sym_string_to_variable] = ACTIONS(1597),
    [sym_length_variable_to_string] = ACTIONS(1597),
    [sym_type_cast] = ACTIONS(1597),
    [sym_variable_to_xml] = ACTIONS(1597),
    [sym_xml_to_variable] = ACTIONS(1597),
    [sym_length_variable_to_xml] = ACTIONS(1597),
    [sym_redirect_string] = ACTIONS(1597),
    [sym_random_number] = ACTIONS(1597),
    [sym_get_unique_id] = ACTIONS(1597),
    [sym_get_system_number] = ACTIONS(1597),
    [sym_set_system_number] = ACTIONS(1597),
    [sym_get_serial_number] = ACTIONS(1597),
    [sym_get_master_build] = ACTIONS(1597),
    [sym_set_validation_code] = ACTIONS(1597),
    [sym_reboot] = ACTIONS(1597),
    [sym_load_duet_module] = ACTIONS(1597),
    [sym_unload_duet_module] = ACTIONS(1597),
    [sym_duet_mem_size_set] = ACTIONS(1597),
    [sym_duet_mem_size_get] = ACTIONS(1597),
    [sym_dynamic_application_device] = ACTIONS(1597),
    [sym_dynamic_polled_port] = ACTIONS(1597),
    [sym_static_port_binding] = ACTIONS(1597),
    [sym_static_ip_binding] = ACTIONS(1597),
    [sym_get_dns_list] = ACTIONS(1597),
    [sym_set_dns_list] = ACTIONS(1597),
    [sym_get_ip_address] = ACTIONS(1597),
    [sym_set_ip_address] = ACTIONS(1597),
    [sym_get_url_list] = ACTIONS(1597),
    [sym_add_url_entry] = ACTIONS(1597),
    [sym_delete_url_entry] = ACTIONS(1597),
    [sym_abs_value] = ACTIONS(1597),
    [sym_max_value] = ACTIONS(1597),
    [sym_min_value] = ACTIONS(1597),
    [sym_exp_value] = ACTIONS(1597),
    [sym_log_value] = ACTIONS(1597),
    [sym_log10_value] = ACTIONS(1597),
    [sym_power_value] = ACTIONS(1597),
    [sym_sqrt_value] = ACTIONS(1597),
    [sym_raw_be] = ACTIONS(1597),
    [sym_raw_le] = ACTIONS(1597),
    [sym_internal_queue_size_set] = ACTIONS(1597),
    [sym_internal_queue_size_get] = ACTIONS(1597),
    [sym_internal_threshold_set] = ACTIONS(1597),
    [sym_internal_threshold_get] = ACTIONS(1597),
    [sym_rebuilt_event] = ACTIONS(1597),
    [sym_do_custom_event] = ACTIONS(1597),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1597),
    [sym_clkmgr_set_clk_source] = ACTIONS(1597),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1597),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1597),
    [sym_clkmgr_get_timezone] = ACTIONS(1597),
    [sym_clkmgr_set_timezone] = ACTIONS(1597),
    [sym_clkmgr_get_resync_period] = ACTIONS(1597),
    [sym_clkmgr_set_resync_period] = ACTIONS(1597),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1597),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1597),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1597),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1597),
    [sym_clkmgr_get_timeservers] = ACTIONS(1597),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1597),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1597),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1597),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1597),
    [sym_set_log_level] = ACTIONS(1597),
    [sym_get_log_level] = ACTIONS(1597),
    [sym_amx_log] = ACTIONS(1597),
    [sym_smtp_server_config_set] = ACTIONS(1597),
    [sym_smtp_server_config_get] = ACTIONS(1597),
    [sym_smtp_send] = ACTIONS(1597),
    [sym_validate_netlinx_account] = ACTIONS(1597),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1597),
    [sym_authenticate_certificate] = ACTIONS(1597),
    [sym_audit_netlinx_session_event] = ACTIONS(1597),
    [sym_audit_netlinx_generic_event] = ACTIONS(1597),
    [sym_get_available_flash_disk_space] = ACTIONS(1597),
    [sym_get_max_flash_disk_space] = ACTIONS(1597),
    [sym_data_create_feed] = ACTIONS(1597),
    [sym_data_delete_feed] = ACTIONS(1597),
    [sym_data_publish_feed] = ACTIONS(1597),
    [sym_data_get_published_feed] = ACTIONS(1597),
    [sym_data_add_record] = ACTIONS(1597),
    [sym_data_get_event_record] = ACTIONS(1597),
    [sym__wc_data_create_feed] = ACTIONS(1597),
    [sym__wc_data_add_record] = ACTIONS(1597),
    [sym__wc_data_get_event_record] = ACTIONS(1597),
    [sym_format] = ACTIONS(1597),
    [sym_astro_clock] = ACTIONS(1597),
    [sym_set_outdoor_temperature] = ACTIONS(1597),
    [sym_set_virtual_level_count] = ACTIONS(1597),
    [sym_set_virtual_channel_count] = ACTIONS(1597),
    [sym_set_virtual_port_count] = ACTIONS(1597),
    [sym__wc_encode_int] = ACTIONS(1597),
    [sym__wc_decode_int] = ACTIONS(1597),
    [sym__wc_int] = ACTIONS(1597),
    [sym__wc_to_ch_int] = ACTIONS(1597),
    [sym__ch_to_wc_int] = ACTIONS(1597),
    [sym__wc_find_string_int] = ACTIONS(1597),
    [sym__wc_left_string_int] = ACTIONS(1597),
    [sym__wc_lower_string_int] = ACTIONS(1597),
    [sym__wc_mid_string_int] = ACTIONS(1597),
    [sym__wc_remove_string_int] = ACTIONS(1597),
    [sym__wc_right_string_int] = ACTIONS(1597),
    [sym__wc_upper_string_int] = ACTIONS(1597),
    [sym__wc_compare_string_int] = ACTIONS(1597),
    [sym__wc_get_buffer_char_int] = ACTIONS(1597),
    [sym__wc_get_buffer_string_int] = ACTIONS(1597),
    [sym__wc_concat_string_int] = ACTIONS(1597),
    [sym__wc_file_open_int] = ACTIONS(1597),
    [sym__wc_file_close_int] = ACTIONS(1597),
    [sym__wc_file_read_int] = ACTIONS(1597),
    [sym__wc_file_read_line_int] = ACTIONS(1597),
    [sym__wc_file_write_int] = ACTIONS(1597),
    [sym__wc_file_write_line_int] = ACTIONS(1597),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1599),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1599),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1599),
    [aux_sym_preproc_else_token1] = ACTIONS(1599),
    [anon_sym_COMMA] = ACTIONS(1599),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1599),
    [anon_sym_RBRACE] = ACTIONS(1599),
    [anon_sym_COLON] = ACTIONS(1599),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_BANG] = ACTIONS(1597),
    [anon_sym_TILDE] = ACTIONS(1599),
    [anon_sym_DASH] = ACTIONS(1597),
    [anon_sym_PLUS] = ACTIONS(1597),
    [anon_sym_STAR] = ACTIONS(1599),
    [anon_sym_SLASH] = ACTIONS(1597),
    [anon_sym_PERCENT] = ACTIONS(1599),
    [anon_sym_PIPE_PIPE] = ACTIONS(1599),
    [anon_sym_AMP_AMP] = ACTIONS(1599),
    [anon_sym_PIPE] = ACTIONS(1597),
    [anon_sym_CARET] = ACTIONS(1599),
    [anon_sym_AMP] = ACTIONS(1597),
    [anon_sym_EQ_EQ] = ACTIONS(1599),
    [anon_sym_BANG_EQ] = ACTIONS(1599),
    [anon_sym_LT_GT] = ACTIONS(1599),
    [anon_sym_GT] = ACTIONS(1597),
    [anon_sym_GT_EQ] = ACTIONS(1599),
    [anon_sym_LT_EQ] = ACTIONS(1599),
    [anon_sym_LT] = ACTIONS(1597),
    [anon_sym_LT_LT] = ACTIONS(1599),
    [anon_sym_GT_GT] = ACTIONS(1599),
    [anon_sym_DASH_DASH] = ACTIONS(1599),
    [anon_sym_PLUS_PLUS] = ACTIONS(1599),
    [anon_sym_DOT] = ACTIONS(1597),
    [anon_sym_DQUOTE] = ACTIONS(1599),
    [anon_sym_SQUOTE] = ACTIONS(1599),
    [sym_number_literal] = ACTIONS(1599),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(610)] = {
    [sym_identifier] = ACTIONS(1601),
    [sym_and] = ACTIONS(1601),
    [sym_or] = ACTIONS(1601),
    [sym_not] = ACTIONS(1601),
    [sym_xor] = ACTIONS(1601),
    [sym_band] = ACTIONS(1601),
    [sym_bor] = ACTIONS(1601),
    [sym_bxor] = ACTIONS(1601),
    [sym_bnot] = ACTIONS(1601),
    [sym_lshift] = ACTIONS(1601),
    [sym_rshift] = ACTIONS(1601),
    [sym___file__] = ACTIONS(1601),
    [sym___date__] = ACTIONS(1601),
    [sym___ldate__] = ACTIONS(1601),
    [sym___time__] = ACTIONS(1601),
    [sym___line__] = ACTIONS(1601),
    [sym___name__] = ACTIONS(1601),
    [sym___netlinx__] = ACTIONS(1601),
    [sym_netlinx_axi_version] = ACTIONS(1601),
    [sym_url_flg_tcp] = ACTIONS(1601),
    [sym_url_flg_accinfopresent] = ACTIONS(1601),
    [sym_url_flg_temp] = ACTIONS(1601),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1601),
    [sym_url_flg_stat_mask] = ACTIONS(1601),
    [sym_url_flg_stat_lookup] = ACTIONS(1601),
    [sym_url_flg_stat_connecting] = ACTIONS(1601),
    [sym_url_flg_stat_waiting] = ACTIONS(1601),
    [sym_url_flg_stat_connected] = ACTIONS(1601),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1601),
    [sym_first_virtual_device] = ACTIONS(1601),
    [sym_first_local_port] = ACTIONS(1601),
    [sym_dynamic_virtual_device] = ACTIONS(1601),
    [sym_timeline_once] = ACTIONS(1601),
    [sym_timeline_repeat] = ACTIONS(1601),
    [sym_timeline_absolute] = ACTIONS(1601),
    [sym_timeline_relative] = ACTIONS(1601),
    [sym_do_push_timed_infinite] = ACTIONS(1601),
    [sym_source_type_no_address] = ACTIONS(1601),
    [sym_source_type_neuron_id] = ACTIONS(1601),
    [sym_source_type_ip_address] = ACTIONS(1601),
    [sym_source_type_axlink] = ACTIONS(1601),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1601),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1601),
    [sym_source_type_ip_socket_address] = ACTIONS(1601),
    [sym_source_type_rs232] = ACTIONS(1601),
    [sym_source_type_internal] = ACTIONS(1601),
    [sym_source_type_mac_address] = ACTIONS(1601),
    [sym_source_type_ipv4_port] = ACTIONS(1601),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1601),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1601),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1601),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1601),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1601),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1601),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1601),
    [sym_true] = ACTIONS(1601),
    [sym_false] = ACTIONS(1601),
    [sym_file_read_only] = ACTIONS(1601),
    [sym_file_rw_new] = ACTIONS(1601),
    [sym_file_rw_append] = ACTIONS(1601),
    [sym_ip_tcp] = ACTIONS(1601),
    [sym_ip_udp] = ACTIONS(1601),
    [sym_ip_udp_2way] = ACTIONS(1601),
    [sym_xml_encode_types] = ACTIONS(1601),
    [sym_xml_encode_char_as_list] = ACTIONS(1601),
    [sym_xml_encode_le] = ACTIONS(1601),
    [sym_xml_decode_types] = ACTIONS(1601),
    [sym_xml_decode_no_preserve] = ACTIONS(1601),
    [sym_duet_dev_type_display_device] = ACTIONS(1601),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1601),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1601),
    [sym_duet_dev_type_io_device] = ACTIONS(1601),
    [sym_duet_dev_type_relay_device] = ACTIONS(1601),
    [sym_duet_dev_type_ups] = ACTIONS(1601),
    [sym_duet_dev_type_amplifier] = ACTIONS(1601),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1601),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1601),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1601),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1601),
    [sym_duet_dev_type_camera] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1601),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1601),
    [sym_duet_dev_type_dss] = ACTIONS(1601),
    [sym_duet_dev_type_dvr] = ACTIONS(1601),
    [sym_duet_dev_type_disc_device] = ACTIONS(1601),
    [sym_duet_dev_type_document_camera] = ACTIONS(1601),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1601),
    [sym_duet_dev_type_hvac] = ACTIONS(1601),
    [sym_duet_dev_type_keypad] = ACTIONS(1601),
    [sym_duet_dev_type_light] = ACTIONS(1601),
    [sym_duet_dev_type_monitor] = ACTIONS(1601),
    [sym_duet_dev_type_motor] = ACTIONS(1601),
    [sym_duet_dev_type_multi_window] = ACTIONS(1601),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1601),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1601),
    [sym_duet_dev_type_receiver] = ACTIONS(1601),
    [sym_duet_dev_type_security_system] = ACTIONS(1601),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1601),
    [sym_duet_dev_type_settop_box] = ACTIONS(1601),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1601),
    [sym_duet_dev_type_switcher] = ACTIONS(1601),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1601),
    [sym_duet_dev_type_tv] = ACTIONS(1601),
    [sym_duet_dev_type_utility] = ACTIONS(1601),
    [sym_duet_dev_type_vcr] = ACTIONS(1601),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1601),
    [sym_duet_dev_type_video_processor] = ACTIONS(1601),
    [sym_duet_dev_type_video_projector] = ACTIONS(1601),
    [sym_duet_dev_type_video_wall] = ACTIONS(1601),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1601),
    [sym_duet_dev_type_weather] = ACTIONS(1601),
    [sym_duet_dev_not_polled] = ACTIONS(1601),
    [sym_duet_dev_polled] = ACTIONS(1601),
    [sym_clkmgr_mode_network] = ACTIONS(1601),
    [sym_clkmgr_mode_standalone] = ACTIONS(1601),
    [sym_normal_standby] = ACTIONS(1601),
    [sym_normal_wake] = ACTIONS(1601),
    [sym_ip_multicast_ttl_option] = ACTIONS(1601),
    [sym_ip_ttl_subnet] = ACTIONS(1601),
    [sym_ip_ttl_site] = ACTIONS(1601),
    [sym_ip_ttl_region] = ACTIONS(1601),
    [sym_ip_ttl_continent] = ACTIONS(1601),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1601),
    [sym_ip_nodelay_on] = ACTIONS(1601),
    [sym_ip_nodelay_off] = ACTIONS(1601),
    [sym_amx_error] = ACTIONS(1601),
    [sym_amx_warning] = ACTIONS(1601),
    [sym_amx_info] = ACTIONS(1601),
    [sym_amx_debug] = ACTIONS(1601),
    [sym_netlinx_login_success] = ACTIONS(1601),
    [sym_netlinx_login_fail] = ACTIONS(1601),
    [sym_netlinx_logout] = ACTIONS(1601),
    [sym_null_str] = ACTIONS(1601),
    [sym_smtp_address] = ACTIONS(1601),
    [sym_smtp_port_number] = ACTIONS(1601),
    [sym_smtp_username] = ACTIONS(1601),
    [sym_smtp_password] = ACTIONS(1601),
    [sym_smtp_require_tls] = ACTIONS(1601),
    [sym_smtp_from] = ACTIONS(1601),
    [sym_smtp_tls_true] = ACTIONS(1601),
    [sym_smtp_tls_false] = ACTIONS(1601),
    [sym_listview_on_row_select_event] = ACTIONS(1601),
    [sym_data_structure_unknown] = ACTIONS(1601),
    [sym_data_structure_datafeed] = ACTIONS(1601),
    [sym_data_structure_datarecordset] = ACTIONS(1601),
    [sym_data_structure_datarecord] = ACTIONS(1601),
    [sym_data_structure_datafield] = ACTIONS(1601),
    [sym_data_max_content_field_count] = ACTIONS(1601),
    [sym_data_max_metadata_field_count] = ACTIONS(1601),
    [sym_data_max_id_length] = ACTIONS(1601),
    [sym_data_max_name_length] = ACTIONS(1601),
    [sym_data_max_description_length] = ACTIONS(1601),
    [sym_data_max_type_length] = ACTIONS(1601),
    [sym_data_max_format_length] = ACTIONS(1601),
    [sym_data_max_source_length] = ACTIONS(1601),
    [sym_data_max_value_length] = ACTIONS(1601),
    [sym_data_max_label_length] = ACTIONS(1601),
    [sym_data_max_url_length] = ACTIONS(1601),
    [sym_max_last_login_info_length] = ACTIONS(1601),
    [sym_data_type_unknown] = ACTIONS(1601),
    [sym_data_type_string] = ACTIONS(1601),
    [sym_data_type_datetime] = ACTIONS(1601),
    [sym_data_type_date] = ACTIONS(1601),
    [sym_data_type_time] = ACTIONS(1601),
    [sym_data_type_image] = ACTIONS(1601),
    [sym_data_format_url] = ACTIONS(1601),
    [sym_data_format_phone] = ACTIONS(1601),
    [sym_data_format_email] = ACTIONS(1601),
    [sym_data_format_iso8601] = ACTIONS(1601),
    [sym_auditlog_priv] = ACTIONS(1601),
    [sym_configuration_priv] = ACTIONS(1601),
    [sym_deviceconfig_priv] = ACTIONS(1601),
    [sym_ftp_priv] = ACTIONS(1601),
    [sym_http_priv] = ACTIONS(1601),
    [sym_networkconfig_priv] = ACTIONS(1601),
    [sym_programport_priv] = ACTIONS(1601),
    [sym_remoteui_priv] = ACTIONS(1601),
    [sym_securitycontrol_priv] = ACTIONS(1601),
    [sym_softwaremanagement_priv] = ACTIONS(1601),
    [sym_terminal_priv] = ACTIONS(1601),
    [sym_tpadmin_priv] = ACTIONS(1601),
    [sym_user1_priv] = ACTIONS(1601),
    [sym_user2_priv] = ACTIONS(1601),
    [sym_user3_priv] = ACTIONS(1601),
    [sym_user4_priv] = ACTIONS(1601),
    [sym_usermanagement_priv] = ACTIONS(1601),
    [sym_valid_account] = ACTIONS(1601),
    [sym_err_name_invalid] = ACTIONS(1601),
    [sym_err_password_invalid] = ACTIONS(1601),
    [sym_err_invalid_account] = ACTIONS(1601),
    [sym_err_no_authorization] = ACTIONS(1601),
    [sym_err_invalid_parameter] = ACTIONS(1601),
    [sym_err_user_locked_out] = ACTIONS(1601),
    [sym_err_user_account_expired] = ACTIONS(1601),
    [sym_err_general_error] = ACTIONS(1601),
    [sym_err_authentication_error] = ACTIONS(1601),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1601),
    [sym_tls_validate_certificate] = ACTIONS(1601),
    [sym____reserved___] = ACTIONS(1601),
    [sym_date] = ACTIONS(1601),
    [sym_day] = ACTIONS(1601),
    [sym_time] = ACTIONS(1601),
    [sym_ldate] = ACTIONS(1601),
    [sym_push_device] = ACTIONS(1601),
    [sym_push_channel] = ACTIONS(1601),
    [sym_push_devchan] = ACTIONS(1601),
    [sym_release_device] = ACTIONS(1601),
    [sym_release_channel] = ACTIONS(1601),
    [sym_release_devchan] = ACTIONS(1601),
    [sym_master_slot] = ACTIONS(1601),
    [sym_get_pulse_time] = ACTIONS(1601),
    [sym_get_timer] = ACTIONS(1601),
    [sym_master_sn] = ACTIONS(1601),
    [sym_system_number] = ACTIONS(1601),
    [sym_button] = ACTIONS(1601),
    [sym_dv_channel] = ACTIONS(1601),
    [sym_data] = ACTIONS(1601),
    [sym_level] = ACTIONS(1601),
    [sym_timeline] = ACTIONS(1601),
    [sym_channel] = ACTIONS(1601),
    [sym_custom] = ACTIONS(1601),
    [sym_atoi] = ACTIONS(1601),
    [sym_atol] = ACTIONS(1601),
    [sym_atof] = ACTIONS(1601),
    [sym_itoa] = ACTIONS(1601),
    [sym_itohex] = ACTIONS(1601),
    [sym_hextoi] = ACTIONS(1601),
    [sym_ftoa] = ACTIONS(1601),
    [sym_find_string] = ACTIONS(1601),
    [sym_left_string] = ACTIONS(1601),
    [sym_right_string] = ACTIONS(1601),
    [sym_mid_string] = ACTIONS(1601),
    [sym_remove_string] = ACTIONS(1601),
    [sym_get_buffer_string] = ACTIONS(1601),
    [sym_get_buffer_char] = ACTIONS(1601),
    [sym_get_multi_buffer_string] = ACTIONS(1601),
    [sym_length_string] = ACTIONS(1601),
    [sym_max_length_string] = ACTIONS(1601),
    [sym_set_length_string] = ACTIONS(1601),
    [sym_compare_string] = ACTIONS(1601),
    [sym_upper_string] = ACTIONS(1601),
    [sym_lower_string] = ACTIONS(1601),
    [sym_get_last] = ACTIONS(1601),
    [sym_date_to_day] = ACTIONS(1601),
    [sym_date_to_month] = ACTIONS(1601),
    [sym_date_to_year] = ACTIONS(1601),
    [sym_time_to_hour] = ACTIONS(1601),
    [sym_time_to_minute] = ACTIONS(1601),
    [sym_time_to_second] = ACTIONS(1601),
    [sym_day_of_week] = ACTIONS(1601),
    [sym_set_pulse_time] = ACTIONS(1601),
    [sym_set_timer] = ACTIONS(1601),
    [sym_file_open] = ACTIONS(1601),
    [sym_file_close] = ACTIONS(1601),
    [sym_file_read] = ACTIONS(1601),
    [sym_file_read_line] = ACTIONS(1601),
    [sym_file_write] = ACTIONS(1601),
    [sym_file_write_line] = ACTIONS(1601),
    [sym_file_copy] = ACTIONS(1601),
    [sym_file_delete] = ACTIONS(1601),
    [sym_file_rename] = ACTIONS(1601),
    [sym_file_dir] = ACTIONS(1601),
    [sym_file_getdir] = ACTIONS(1601),
    [sym_file_setdir] = ACTIONS(1601),
    [sym_file_create_dir] = ACTIONS(1601),
    [sym_file_remove_dir] = ACTIONS(1601),
    [sym_file_seek] = ACTIONS(1601),
    [sym_length_array] = ACTIONS(1601),
    [sym_max_length_array] = ACTIONS(1601),
    [sym_set_length_array] = ACTIONS(1601),
    [sym_ip_client_open] = ACTIONS(1601),
    [sym_ip_client_close] = ACTIONS(1601),
    [sym_ip_server_open] = ACTIONS(1601),
    [sym_ip_server_close] = ACTIONS(1601),
    [sym_ip_bound_client_open] = ACTIONS(1601),
    [sym_ip_mc_server_open] = ACTIONS(1601),
    [sym_ip_set_option] = ACTIONS(1601),
    [sym_ssh_client_open] = ACTIONS(1601),
    [sym_ssh_client_close] = ACTIONS(1601),
    [sym_tls_client_open] = ACTIONS(1601),
    [sym_tls_client_close] = ACTIONS(1601),
    [sym_do_push] = ACTIONS(1601),
    [sym_do_push_timed] = ACTIONS(1601),
    [sym_do_release] = ACTIONS(1601),
    [sym_device_id] = ACTIONS(1601),
    [sym_device_info] = ACTIONS(1601),
    [sym_device_id_string] = ACTIONS(1601),
    [sym_device_standby] = ACTIONS(1601),
    [sym_device_wake] = ACTIONS(1601),
    [sym_timeline_create] = ACTIONS(1601),
    [sym_timeline_kill] = ACTIONS(1601),
    [sym_timeline_reload] = ACTIONS(1601),
    [sym_timeline_pause] = ACTIONS(1601),
    [sym_timeline_restart] = ACTIONS(1601),
    [sym_timeline_set] = ACTIONS(1601),
    [sym_timeline_get] = ACTIONS(1601),
    [sym_timeline_active] = ACTIONS(1601),
    [sym_timeline_dynamic_id] = ACTIONS(1601),
    [sym_variable_to_string] = ACTIONS(1601),
    [sym_string_to_variable] = ACTIONS(1601),
    [sym_length_variable_to_string] = ACTIONS(1601),
    [sym_type_cast] = ACTIONS(1601),
    [sym_variable_to_xml] = ACTIONS(1601),
    [sym_xml_to_variable] = ACTIONS(1601),
    [sym_length_variable_to_xml] = ACTIONS(1601),
    [sym_redirect_string] = ACTIONS(1601),
    [sym_random_number] = ACTIONS(1601),
    [sym_get_unique_id] = ACTIONS(1601),
    [sym_get_system_number] = ACTIONS(1601),
    [sym_set_system_number] = ACTIONS(1601),
    [sym_get_serial_number] = ACTIONS(1601),
    [sym_get_master_build] = ACTIONS(1601),
    [sym_set_validation_code] = ACTIONS(1601),
    [sym_reboot] = ACTIONS(1601),
    [sym_load_duet_module] = ACTIONS(1601),
    [sym_unload_duet_module] = ACTIONS(1601),
    [sym_duet_mem_size_set] = ACTIONS(1601),
    [sym_duet_mem_size_get] = ACTIONS(1601),
    [sym_dynamic_application_device] = ACTIONS(1601),
    [sym_dynamic_polled_port] = ACTIONS(1601),
    [sym_static_port_binding] = ACTIONS(1601),
    [sym_static_ip_binding] = ACTIONS(1601),
    [sym_get_dns_list] = ACTIONS(1601),
    [sym_set_dns_list] = ACTIONS(1601),
    [sym_get_ip_address] = ACTIONS(1601),
    [sym_set_ip_address] = ACTIONS(1601),
    [sym_get_url_list] = ACTIONS(1601),
    [sym_add_url_entry] = ACTIONS(1601),
    [sym_delete_url_entry] = ACTIONS(1601),
    [sym_abs_value] = ACTIONS(1601),
    [sym_max_value] = ACTIONS(1601),
    [sym_min_value] = ACTIONS(1601),
    [sym_exp_value] = ACTIONS(1601),
    [sym_log_value] = ACTIONS(1601),
    [sym_log10_value] = ACTIONS(1601),
    [sym_power_value] = ACTIONS(1601),
    [sym_sqrt_value] = ACTIONS(1601),
    [sym_raw_be] = ACTIONS(1601),
    [sym_raw_le] = ACTIONS(1601),
    [sym_internal_queue_size_set] = ACTIONS(1601),
    [sym_internal_queue_size_get] = ACTIONS(1601),
    [sym_internal_threshold_set] = ACTIONS(1601),
    [sym_internal_threshold_get] = ACTIONS(1601),
    [sym_rebuilt_event] = ACTIONS(1601),
    [sym_do_custom_event] = ACTIONS(1601),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1601),
    [sym_clkmgr_set_clk_source] = ACTIONS(1601),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1601),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1601),
    [sym_clkmgr_get_timezone] = ACTIONS(1601),
    [sym_clkmgr_set_timezone] = ACTIONS(1601),
    [sym_clkmgr_get_resync_period] = ACTIONS(1601),
    [sym_clkmgr_set_resync_period] = ACTIONS(1601),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1601),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1601),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1601),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1601),
    [sym_clkmgr_get_timeservers] = ACTIONS(1601),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1601),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1601),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1601),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1601),
    [sym_set_log_level] = ACTIONS(1601),
    [sym_get_log_level] = ACTIONS(1601),
    [sym_amx_log] = ACTIONS(1601),
    [sym_smtp_server_config_set] = ACTIONS(1601),
    [sym_smtp_server_config_get] = ACTIONS(1601),
    [sym_smtp_send] = ACTIONS(1601),
    [sym_validate_netlinx_account] = ACTIONS(1601),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1601),
    [sym_authenticate_certificate] = ACTIONS(1601),
    [sym_audit_netlinx_session_event] = ACTIONS(1601),
    [sym_audit_netlinx_generic_event] = ACTIONS(1601),
    [sym_get_available_flash_disk_space] = ACTIONS(1601),
    [sym_get_max_flash_disk_space] = ACTIONS(1601),
    [sym_data_create_feed] = ACTIONS(1601),
    [sym_data_delete_feed] = ACTIONS(1601),
    [sym_data_publish_feed] = ACTIONS(1601),
    [sym_data_get_published_feed] = ACTIONS(1601),
    [sym_data_add_record] = ACTIONS(1601),
    [sym_data_get_event_record] = ACTIONS(1601),
    [sym__wc_data_create_feed] = ACTIONS(1601),
    [sym__wc_data_add_record] = ACTIONS(1601),
    [sym__wc_data_get_event_record] = ACTIONS(1601),
    [sym_format] = ACTIONS(1601),
    [sym_astro_clock] = ACTIONS(1601),
    [sym_set_outdoor_temperature] = ACTIONS(1601),
    [sym_set_virtual_level_count] = ACTIONS(1601),
    [sym_set_virtual_channel_count] = ACTIONS(1601),
    [sym_set_virtual_port_count] = ACTIONS(1601),
    [sym__wc_encode_int] = ACTIONS(1601),
    [sym__wc_decode_int] = ACTIONS(1601),
    [sym__wc_int] = ACTIONS(1601),
    [sym__wc_to_ch_int] = ACTIONS(1601),
    [sym__ch_to_wc_int] = ACTIONS(1601),
    [sym__wc_find_string_int] = ACTIONS(1601),
    [sym__wc_left_string_int] = ACTIONS(1601),
    [sym__wc_lower_string_int] = ACTIONS(1601),
    [sym__wc_mid_string_int] = ACTIONS(1601),
    [sym__wc_remove_string_int] = ACTIONS(1601),
    [sym__wc_right_string_int] = ACTIONS(1601),
    [sym__wc_upper_string_int] = ACTIONS(1601),
    [sym__wc_compare_string_int] = ACTIONS(1601),
    [sym__wc_get_buffer_char_int] = ACTIONS(1601),
    [sym__wc_get_buffer_string_int] = ACTIONS(1601),
    [sym__wc_concat_string_int] = ACTIONS(1601),
    [sym__wc_file_open_int] = ACTIONS(1601),
    [sym__wc_file_close_int] = ACTIONS(1601),
    [sym__wc_file_read_int] = ACTIONS(1601),
    [sym__wc_file_read_line_int] = ACTIONS(1601),
    [sym__wc_file_write_int] = ACTIONS(1601),
    [sym__wc_file_write_line_int] = ACTIONS(1601),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1603),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1603),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1603),
    [aux_sym_preproc_else_token1] = ACTIONS(1603),
    [anon_sym_COMMA] = ACTIONS(1603),
    [anon_sym_LBRACK] = ACTIONS(1603),
    [anon_sym_LBRACE] = ACTIONS(1603),
    [anon_sym_RBRACE] = ACTIONS(1603),
    [anon_sym_COLON] = ACTIONS(1603),
    [anon_sym_LPAREN] = ACTIONS(1601),
    [anon_sym_BANG] = ACTIONS(1601),
    [anon_sym_TILDE] = ACTIONS(1603),
    [anon_sym_DASH] = ACTIONS(1601),
    [anon_sym_PLUS] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1603),
    [anon_sym_SLASH] = ACTIONS(1601),
    [anon_sym_PERCENT] = ACTIONS(1603),
    [anon_sym_PIPE_PIPE] = ACTIONS(1603),
    [anon_sym_AMP_AMP] = ACTIONS(1603),
    [anon_sym_PIPE] = ACTIONS(1601),
    [anon_sym_CARET] = ACTIONS(1603),
    [anon_sym_AMP] = ACTIONS(1601),
    [anon_sym_EQ_EQ] = ACTIONS(1603),
    [anon_sym_BANG_EQ] = ACTIONS(1603),
    [anon_sym_LT_GT] = ACTIONS(1603),
    [anon_sym_GT] = ACTIONS(1601),
    [anon_sym_GT_EQ] = ACTIONS(1603),
    [anon_sym_LT_EQ] = ACTIONS(1603),
    [anon_sym_LT] = ACTIONS(1601),
    [anon_sym_LT_LT] = ACTIONS(1603),
    [anon_sym_GT_GT] = ACTIONS(1603),
    [anon_sym_DASH_DASH] = ACTIONS(1603),
    [anon_sym_PLUS_PLUS] = ACTIONS(1603),
    [anon_sym_DOT] = ACTIONS(1601),
    [anon_sym_DQUOTE] = ACTIONS(1603),
    [anon_sym_SQUOTE] = ACTIONS(1603),
    [sym_number_literal] = ACTIONS(1603),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(611)] = {
    [sym_identifier] = ACTIONS(1565),
    [sym_and] = ACTIONS(1565),
    [sym_or] = ACTIONS(1565),
    [sym_not] = ACTIONS(1565),
    [sym_xor] = ACTIONS(1565),
    [sym_band] = ACTIONS(1565),
    [sym_bor] = ACTIONS(1565),
    [sym_bxor] = ACTIONS(1565),
    [sym_bnot] = ACTIONS(1565),
    [sym_lshift] = ACTIONS(1565),
    [sym_rshift] = ACTIONS(1565),
    [sym___file__] = ACTIONS(1565),
    [sym___date__] = ACTIONS(1565),
    [sym___ldate__] = ACTIONS(1565),
    [sym___time__] = ACTIONS(1565),
    [sym___line__] = ACTIONS(1565),
    [sym___name__] = ACTIONS(1565),
    [sym___netlinx__] = ACTIONS(1565),
    [sym_netlinx_axi_version] = ACTIONS(1565),
    [sym_url_flg_tcp] = ACTIONS(1565),
    [sym_url_flg_accinfopresent] = ACTIONS(1565),
    [sym_url_flg_temp] = ACTIONS(1565),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1565),
    [sym_url_flg_stat_mask] = ACTIONS(1565),
    [sym_url_flg_stat_lookup] = ACTIONS(1565),
    [sym_url_flg_stat_connecting] = ACTIONS(1565),
    [sym_url_flg_stat_waiting] = ACTIONS(1565),
    [sym_url_flg_stat_connected] = ACTIONS(1565),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1565),
    [sym_first_virtual_device] = ACTIONS(1565),
    [sym_first_local_port] = ACTIONS(1565),
    [sym_dynamic_virtual_device] = ACTIONS(1565),
    [sym_timeline_once] = ACTIONS(1565),
    [sym_timeline_repeat] = ACTIONS(1565),
    [sym_timeline_absolute] = ACTIONS(1565),
    [sym_timeline_relative] = ACTIONS(1565),
    [sym_do_push_timed_infinite] = ACTIONS(1565),
    [sym_source_type_no_address] = ACTIONS(1565),
    [sym_source_type_neuron_id] = ACTIONS(1565),
    [sym_source_type_ip_address] = ACTIONS(1565),
    [sym_source_type_axlink] = ACTIONS(1565),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1565),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1565),
    [sym_source_type_ip_socket_address] = ACTIONS(1565),
    [sym_source_type_rs232] = ACTIONS(1565),
    [sym_source_type_internal] = ACTIONS(1565),
    [sym_source_type_mac_address] = ACTIONS(1565),
    [sym_source_type_ipv4_port] = ACTIONS(1565),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1565),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1565),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1565),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1565),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1565),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1565),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1565),
    [sym_true] = ACTIONS(1565),
    [sym_false] = ACTIONS(1565),
    [sym_file_read_only] = ACTIONS(1565),
    [sym_file_rw_new] = ACTIONS(1565),
    [sym_file_rw_append] = ACTIONS(1565),
    [sym_ip_tcp] = ACTIONS(1565),
    [sym_ip_udp] = ACTIONS(1565),
    [sym_ip_udp_2way] = ACTIONS(1565),
    [sym_xml_encode_types] = ACTIONS(1565),
    [sym_xml_encode_char_as_list] = ACTIONS(1565),
    [sym_xml_encode_le] = ACTIONS(1565),
    [sym_xml_decode_types] = ACTIONS(1565),
    [sym_xml_decode_no_preserve] = ACTIONS(1565),
    [sym_duet_dev_type_display_device] = ACTIONS(1565),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1565),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1565),
    [sym_duet_dev_type_io_device] = ACTIONS(1565),
    [sym_duet_dev_type_relay_device] = ACTIONS(1565),
    [sym_duet_dev_type_ups] = ACTIONS(1565),
    [sym_duet_dev_type_amplifier] = ACTIONS(1565),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1565),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1565),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1565),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1565),
    [sym_duet_dev_type_camera] = ACTIONS(1565),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1565),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1565),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1565),
    [sym_duet_dev_type_dss] = ACTIONS(1565),
    [sym_duet_dev_type_dvr] = ACTIONS(1565),
    [sym_duet_dev_type_disc_device] = ACTIONS(1565),
    [sym_duet_dev_type_document_camera] = ACTIONS(1565),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1565),
    [sym_duet_dev_type_hvac] = ACTIONS(1565),
    [sym_duet_dev_type_keypad] = ACTIONS(1565),
    [sym_duet_dev_type_light] = ACTIONS(1565),
    [sym_duet_dev_type_monitor] = ACTIONS(1565),
    [sym_duet_dev_type_motor] = ACTIONS(1565),
    [sym_duet_dev_type_multi_window] = ACTIONS(1565),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1565),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1565),
    [sym_duet_dev_type_receiver] = ACTIONS(1565),
    [sym_duet_dev_type_security_system] = ACTIONS(1565),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1565),
    [sym_duet_dev_type_settop_box] = ACTIONS(1565),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1565),
    [sym_duet_dev_type_switcher] = ACTIONS(1565),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1565),
    [sym_duet_dev_type_tv] = ACTIONS(1565),
    [sym_duet_dev_type_utility] = ACTIONS(1565),
    [sym_duet_dev_type_vcr] = ACTIONS(1565),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1565),
    [sym_duet_dev_type_video_processor] = ACTIONS(1565),
    [sym_duet_dev_type_video_projector] = ACTIONS(1565),
    [sym_duet_dev_type_video_wall] = ACTIONS(1565),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1565),
    [sym_duet_dev_type_weather] = ACTIONS(1565),
    [sym_duet_dev_not_polled] = ACTIONS(1565),
    [sym_duet_dev_polled] = ACTIONS(1565),
    [sym_clkmgr_mode_network] = ACTIONS(1565),
    [sym_clkmgr_mode_standalone] = ACTIONS(1565),
    [sym_normal_standby] = ACTIONS(1565),
    [sym_normal_wake] = ACTIONS(1565),
    [sym_ip_multicast_ttl_option] = ACTIONS(1565),
    [sym_ip_ttl_subnet] = ACTIONS(1565),
    [sym_ip_ttl_site] = ACTIONS(1565),
    [sym_ip_ttl_region] = ACTIONS(1565),
    [sym_ip_ttl_continent] = ACTIONS(1565),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1565),
    [sym_ip_nodelay_on] = ACTIONS(1565),
    [sym_ip_nodelay_off] = ACTIONS(1565),
    [sym_amx_error] = ACTIONS(1565),
    [sym_amx_warning] = ACTIONS(1565),
    [sym_amx_info] = ACTIONS(1565),
    [sym_amx_debug] = ACTIONS(1565),
    [sym_netlinx_login_success] = ACTIONS(1565),
    [sym_netlinx_login_fail] = ACTIONS(1565),
    [sym_netlinx_logout] = ACTIONS(1565),
    [sym_null_str] = ACTIONS(1565),
    [sym_smtp_address] = ACTIONS(1565),
    [sym_smtp_port_number] = ACTIONS(1565),
    [sym_smtp_username] = ACTIONS(1565),
    [sym_smtp_password] = ACTIONS(1565),
    [sym_smtp_require_tls] = ACTIONS(1565),
    [sym_smtp_from] = ACTIONS(1565),
    [sym_smtp_tls_true] = ACTIONS(1565),
    [sym_smtp_tls_false] = ACTIONS(1565),
    [sym_listview_on_row_select_event] = ACTIONS(1565),
    [sym_data_structure_unknown] = ACTIONS(1565),
    [sym_data_structure_datafeed] = ACTIONS(1565),
    [sym_data_structure_datarecordset] = ACTIONS(1565),
    [sym_data_structure_datarecord] = ACTIONS(1565),
    [sym_data_structure_datafield] = ACTIONS(1565),
    [sym_data_max_content_field_count] = ACTIONS(1565),
    [sym_data_max_metadata_field_count] = ACTIONS(1565),
    [sym_data_max_id_length] = ACTIONS(1565),
    [sym_data_max_name_length] = ACTIONS(1565),
    [sym_data_max_description_length] = ACTIONS(1565),
    [sym_data_max_type_length] = ACTIONS(1565),
    [sym_data_max_format_length] = ACTIONS(1565),
    [sym_data_max_source_length] = ACTIONS(1565),
    [sym_data_max_value_length] = ACTIONS(1565),
    [sym_data_max_label_length] = ACTIONS(1565),
    [sym_data_max_url_length] = ACTIONS(1565),
    [sym_max_last_login_info_length] = ACTIONS(1565),
    [sym_data_type_unknown] = ACTIONS(1565),
    [sym_data_type_string] = ACTIONS(1565),
    [sym_data_type_datetime] = ACTIONS(1565),
    [sym_data_type_date] = ACTIONS(1565),
    [sym_data_type_time] = ACTIONS(1565),
    [sym_data_type_image] = ACTIONS(1565),
    [sym_data_format_url] = ACTIONS(1565),
    [sym_data_format_phone] = ACTIONS(1565),
    [sym_data_format_email] = ACTIONS(1565),
    [sym_data_format_iso8601] = ACTIONS(1565),
    [sym_auditlog_priv] = ACTIONS(1565),
    [sym_configuration_priv] = ACTIONS(1565),
    [sym_deviceconfig_priv] = ACTIONS(1565),
    [sym_ftp_priv] = ACTIONS(1565),
    [sym_http_priv] = ACTIONS(1565),
    [sym_networkconfig_priv] = ACTIONS(1565),
    [sym_programport_priv] = ACTIONS(1565),
    [sym_remoteui_priv] = ACTIONS(1565),
    [sym_securitycontrol_priv] = ACTIONS(1565),
    [sym_softwaremanagement_priv] = ACTIONS(1565),
    [sym_terminal_priv] = ACTIONS(1565),
    [sym_tpadmin_priv] = ACTIONS(1565),
    [sym_user1_priv] = ACTIONS(1565),
    [sym_user2_priv] = ACTIONS(1565),
    [sym_user3_priv] = ACTIONS(1565),
    [sym_user4_priv] = ACTIONS(1565),
    [sym_usermanagement_priv] = ACTIONS(1565),
    [sym_valid_account] = ACTIONS(1565),
    [sym_err_name_invalid] = ACTIONS(1565),
    [sym_err_password_invalid] = ACTIONS(1565),
    [sym_err_invalid_account] = ACTIONS(1565),
    [sym_err_no_authorization] = ACTIONS(1565),
    [sym_err_invalid_parameter] = ACTIONS(1565),
    [sym_err_user_locked_out] = ACTIONS(1565),
    [sym_err_user_account_expired] = ACTIONS(1565),
    [sym_err_general_error] = ACTIONS(1565),
    [sym_err_authentication_error] = ACTIONS(1565),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1565),
    [sym_tls_validate_certificate] = ACTIONS(1565),
    [sym____reserved___] = ACTIONS(1565),
    [sym_date] = ACTIONS(1565),
    [sym_day] = ACTIONS(1565),
    [sym_time] = ACTIONS(1565),
    [sym_ldate] = ACTIONS(1565),
    [sym_push_device] = ACTIONS(1565),
    [sym_push_channel] = ACTIONS(1565),
    [sym_push_devchan] = ACTIONS(1565),
    [sym_release_device] = ACTIONS(1565),
    [sym_release_channel] = ACTIONS(1565),
    [sym_release_devchan] = ACTIONS(1565),
    [sym_master_slot] = ACTIONS(1565),
    [sym_get_pulse_time] = ACTIONS(1565),
    [sym_get_timer] = ACTIONS(1565),
    [sym_master_sn] = ACTIONS(1565),
    [sym_system_number] = ACTIONS(1565),
    [sym_button] = ACTIONS(1565),
    [sym_dv_channel] = ACTIONS(1565),
    [sym_data] = ACTIONS(1565),
    [sym_level] = ACTIONS(1565),
    [sym_timeline] = ACTIONS(1565),
    [sym_channel] = ACTIONS(1565),
    [sym_custom] = ACTIONS(1565),
    [sym_atoi] = ACTIONS(1565),
    [sym_atol] = ACTIONS(1565),
    [sym_atof] = ACTIONS(1565),
    [sym_itoa] = ACTIONS(1565),
    [sym_itohex] = ACTIONS(1565),
    [sym_hextoi] = ACTIONS(1565),
    [sym_ftoa] = ACTIONS(1565),
    [sym_find_string] = ACTIONS(1565),
    [sym_left_string] = ACTIONS(1565),
    [sym_right_string] = ACTIONS(1565),
    [sym_mid_string] = ACTIONS(1565),
    [sym_remove_string] = ACTIONS(1565),
    [sym_get_buffer_string] = ACTIONS(1565),
    [sym_get_buffer_char] = ACTIONS(1565),
    [sym_get_multi_buffer_string] = ACTIONS(1565),
    [sym_length_string] = ACTIONS(1565),
    [sym_max_length_string] = ACTIONS(1565),
    [sym_set_length_string] = ACTIONS(1565),
    [sym_compare_string] = ACTIONS(1565),
    [sym_upper_string] = ACTIONS(1565),
    [sym_lower_string] = ACTIONS(1565),
    [sym_get_last] = ACTIONS(1565),
    [sym_date_to_day] = ACTIONS(1565),
    [sym_date_to_month] = ACTIONS(1565),
    [sym_date_to_year] = ACTIONS(1565),
    [sym_time_to_hour] = ACTIONS(1565),
    [sym_time_to_minute] = ACTIONS(1565),
    [sym_time_to_second] = ACTIONS(1565),
    [sym_day_of_week] = ACTIONS(1565),
    [sym_set_pulse_time] = ACTIONS(1565),
    [sym_set_timer] = ACTIONS(1565),
    [sym_file_open] = ACTIONS(1565),
    [sym_file_close] = ACTIONS(1565),
    [sym_file_read] = ACTIONS(1565),
    [sym_file_read_line] = ACTIONS(1565),
    [sym_file_write] = ACTIONS(1565),
    [sym_file_write_line] = ACTIONS(1565),
    [sym_file_copy] = ACTIONS(1565),
    [sym_file_delete] = ACTIONS(1565),
    [sym_file_rename] = ACTIONS(1565),
    [sym_file_dir] = ACTIONS(1565),
    [sym_file_getdir] = ACTIONS(1565),
    [sym_file_setdir] = ACTIONS(1565),
    [sym_file_create_dir] = ACTIONS(1565),
    [sym_file_remove_dir] = ACTIONS(1565),
    [sym_file_seek] = ACTIONS(1565),
    [sym_length_array] = ACTIONS(1565),
    [sym_max_length_array] = ACTIONS(1565),
    [sym_set_length_array] = ACTIONS(1565),
    [sym_ip_client_open] = ACTIONS(1565),
    [sym_ip_client_close] = ACTIONS(1565),
    [sym_ip_server_open] = ACTIONS(1565),
    [sym_ip_server_close] = ACTIONS(1565),
    [sym_ip_bound_client_open] = ACTIONS(1565),
    [sym_ip_mc_server_open] = ACTIONS(1565),
    [sym_ip_set_option] = ACTIONS(1565),
    [sym_ssh_client_open] = ACTIONS(1565),
    [sym_ssh_client_close] = ACTIONS(1565),
    [sym_tls_client_open] = ACTIONS(1565),
    [sym_tls_client_close] = ACTIONS(1565),
    [sym_do_push] = ACTIONS(1565),
    [sym_do_push_timed] = ACTIONS(1565),
    [sym_do_release] = ACTIONS(1565),
    [sym_device_id] = ACTIONS(1565),
    [sym_device_info] = ACTIONS(1565),
    [sym_device_id_string] = ACTIONS(1565),
    [sym_device_standby] = ACTIONS(1565),
    [sym_device_wake] = ACTIONS(1565),
    [sym_timeline_create] = ACTIONS(1565),
    [sym_timeline_kill] = ACTIONS(1565),
    [sym_timeline_reload] = ACTIONS(1565),
    [sym_timeline_pause] = ACTIONS(1565),
    [sym_timeline_restart] = ACTIONS(1565),
    [sym_timeline_set] = ACTIONS(1565),
    [sym_timeline_get] = ACTIONS(1565),
    [sym_timeline_active] = ACTIONS(1565),
    [sym_timeline_dynamic_id] = ACTIONS(1565),
    [sym_variable_to_string] = ACTIONS(1565),
    [sym_string_to_variable] = ACTIONS(1565),
    [sym_length_variable_to_string] = ACTIONS(1565),
    [sym_type_cast] = ACTIONS(1565),
    [sym_variable_to_xml] = ACTIONS(1565),
    [sym_xml_to_variable] = ACTIONS(1565),
    [sym_length_variable_to_xml] = ACTIONS(1565),
    [sym_redirect_string] = ACTIONS(1565),
    [sym_random_number] = ACTIONS(1565),
    [sym_get_unique_id] = ACTIONS(1565),
    [sym_get_system_number] = ACTIONS(1565),
    [sym_set_system_number] = ACTIONS(1565),
    [sym_get_serial_number] = ACTIONS(1565),
    [sym_get_master_build] = ACTIONS(1565),
    [sym_set_validation_code] = ACTIONS(1565),
    [sym_reboot] = ACTIONS(1565),
    [sym_load_duet_module] = ACTIONS(1565),
    [sym_unload_duet_module] = ACTIONS(1565),
    [sym_duet_mem_size_set] = ACTIONS(1565),
    [sym_duet_mem_size_get] = ACTIONS(1565),
    [sym_dynamic_application_device] = ACTIONS(1565),
    [sym_dynamic_polled_port] = ACTIONS(1565),
    [sym_static_port_binding] = ACTIONS(1565),
    [sym_static_ip_binding] = ACTIONS(1565),
    [sym_get_dns_list] = ACTIONS(1565),
    [sym_set_dns_list] = ACTIONS(1565),
    [sym_get_ip_address] = ACTIONS(1565),
    [sym_set_ip_address] = ACTIONS(1565),
    [sym_get_url_list] = ACTIONS(1565),
    [sym_add_url_entry] = ACTIONS(1565),
    [sym_delete_url_entry] = ACTIONS(1565),
    [sym_abs_value] = ACTIONS(1565),
    [sym_max_value] = ACTIONS(1565),
    [sym_min_value] = ACTIONS(1565),
    [sym_exp_value] = ACTIONS(1565),
    [sym_log_value] = ACTIONS(1565),
    [sym_log10_value] = ACTIONS(1565),
    [sym_power_value] = ACTIONS(1565),
    [sym_sqrt_value] = ACTIONS(1565),
    [sym_raw_be] = ACTIONS(1565),
    [sym_raw_le] = ACTIONS(1565),
    [sym_internal_queue_size_set] = ACTIONS(1565),
    [sym_internal_queue_size_get] = ACTIONS(1565),
    [sym_internal_threshold_set] = ACTIONS(1565),
    [sym_internal_threshold_get] = ACTIONS(1565),
    [sym_rebuilt_event] = ACTIONS(1565),
    [sym_do_custom_event] = ACTIONS(1565),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1565),
    [sym_clkmgr_set_clk_source] = ACTIONS(1565),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1565),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1565),
    [sym_clkmgr_get_timezone] = ACTIONS(1565),
    [sym_clkmgr_set_timezone] = ACTIONS(1565),
    [sym_clkmgr_get_resync_period] = ACTIONS(1565),
    [sym_clkmgr_set_resync_period] = ACTIONS(1565),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1565),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1565),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1565),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1565),
    [sym_clkmgr_get_timeservers] = ACTIONS(1565),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1565),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1565),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1565),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1565),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1565),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1565),
    [sym_set_log_level] = ACTIONS(1565),
    [sym_get_log_level] = ACTIONS(1565),
    [sym_amx_log] = ACTIONS(1565),
    [sym_smtp_server_config_set] = ACTIONS(1565),
    [sym_smtp_server_config_get] = ACTIONS(1565),
    [sym_smtp_send] = ACTIONS(1565),
    [sym_validate_netlinx_account] = ACTIONS(1565),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1565),
    [sym_authenticate_certificate] = ACTIONS(1565),
    [sym_audit_netlinx_session_event] = ACTIONS(1565),
    [sym_audit_netlinx_generic_event] = ACTIONS(1565),
    [sym_get_available_flash_disk_space] = ACTIONS(1565),
    [sym_get_max_flash_disk_space] = ACTIONS(1565),
    [sym_data_create_feed] = ACTIONS(1565),
    [sym_data_delete_feed] = ACTIONS(1565),
    [sym_data_publish_feed] = ACTIONS(1565),
    [sym_data_get_published_feed] = ACTIONS(1565),
    [sym_data_add_record] = ACTIONS(1565),
    [sym_data_get_event_record] = ACTIONS(1565),
    [sym__wc_data_create_feed] = ACTIONS(1565),
    [sym__wc_data_add_record] = ACTIONS(1565),
    [sym__wc_data_get_event_record] = ACTIONS(1565),
    [sym_format] = ACTIONS(1565),
    [sym_astro_clock] = ACTIONS(1565),
    [sym_set_outdoor_temperature] = ACTIONS(1565),
    [sym_set_virtual_level_count] = ACTIONS(1565),
    [sym_set_virtual_channel_count] = ACTIONS(1565),
    [sym_set_virtual_port_count] = ACTIONS(1565),
    [sym__wc_encode_int] = ACTIONS(1565),
    [sym__wc_decode_int] = ACTIONS(1565),
    [sym__wc_int] = ACTIONS(1565),
    [sym__wc_to_ch_int] = ACTIONS(1565),
    [sym__ch_to_wc_int] = ACTIONS(1565),
    [sym__wc_find_string_int] = ACTIONS(1565),
    [sym__wc_left_string_int] = ACTIONS(1565),
    [sym__wc_lower_string_int] = ACTIONS(1565),
    [sym__wc_mid_string_int] = ACTIONS(1565),
    [sym__wc_remove_string_int] = ACTIONS(1565),
    [sym__wc_right_string_int] = ACTIONS(1565),
    [sym__wc_upper_string_int] = ACTIONS(1565),
    [sym__wc_compare_string_int] = ACTIONS(1565),
    [sym__wc_get_buffer_char_int] = ACTIONS(1565),
    [sym__wc_get_buffer_string_int] = ACTIONS(1565),
    [sym__wc_concat_string_int] = ACTIONS(1565),
    [sym__wc_file_open_int] = ACTIONS(1565),
    [sym__wc_file_close_int] = ACTIONS(1565),
    [sym__wc_file_read_int] = ACTIONS(1565),
    [sym__wc_file_read_line_int] = ACTIONS(1565),
    [sym__wc_file_write_int] = ACTIONS(1565),
    [sym__wc_file_write_line_int] = ACTIONS(1565),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1567),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1567),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1567),
    [aux_sym_preproc_else_token1] = ACTIONS(1567),
    [anon_sym_COMMA] = ACTIONS(1567),
    [anon_sym_LBRACK] = ACTIONS(1567),
    [anon_sym_LBRACE] = ACTIONS(1567),
    [anon_sym_RBRACE] = ACTIONS(1567),
    [anon_sym_COLON] = ACTIONS(1567),
    [anon_sym_LPAREN] = ACTIONS(1565),
    [anon_sym_BANG] = ACTIONS(1565),
    [anon_sym_TILDE] = ACTIONS(1567),
    [anon_sym_DASH] = ACTIONS(1565),
    [anon_sym_PLUS] = ACTIONS(1565),
    [anon_sym_STAR] = ACTIONS(1567),
    [anon_sym_SLASH] = ACTIONS(1565),
    [anon_sym_PERCENT] = ACTIONS(1567),
    [anon_sym_PIPE_PIPE] = ACTIONS(1567),
    [anon_sym_AMP_AMP] = ACTIONS(1567),
    [anon_sym_PIPE] = ACTIONS(1565),
    [anon_sym_CARET] = ACTIONS(1567),
    [anon_sym_AMP] = ACTIONS(1565),
    [anon_sym_EQ_EQ] = ACTIONS(1567),
    [anon_sym_BANG_EQ] = ACTIONS(1567),
    [anon_sym_LT_GT] = ACTIONS(1567),
    [anon_sym_GT] = ACTIONS(1565),
    [anon_sym_GT_EQ] = ACTIONS(1567),
    [anon_sym_LT_EQ] = ACTIONS(1567),
    [anon_sym_LT] = ACTIONS(1565),
    [anon_sym_LT_LT] = ACTIONS(1567),
    [anon_sym_GT_GT] = ACTIONS(1567),
    [anon_sym_DASH_DASH] = ACTIONS(1567),
    [anon_sym_PLUS_PLUS] = ACTIONS(1567),
    [anon_sym_DOT] = ACTIONS(1565),
    [anon_sym_DQUOTE] = ACTIONS(1567),
    [anon_sym_SQUOTE] = ACTIONS(1567),
    [sym_number_literal] = ACTIONS(1567),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(612)] = {
    [sym_identifier] = ACTIONS(1585),
    [sym_and] = ACTIONS(1585),
    [sym_or] = ACTIONS(1585),
    [sym_not] = ACTIONS(1585),
    [sym_xor] = ACTIONS(1585),
    [sym_band] = ACTIONS(1585),
    [sym_bor] = ACTIONS(1585),
    [sym_bxor] = ACTIONS(1585),
    [sym_bnot] = ACTIONS(1585),
    [sym_lshift] = ACTIONS(1585),
    [sym_rshift] = ACTIONS(1585),
    [sym___file__] = ACTIONS(1585),
    [sym___date__] = ACTIONS(1585),
    [sym___ldate__] = ACTIONS(1585),
    [sym___time__] = ACTIONS(1585),
    [sym___line__] = ACTIONS(1585),
    [sym___name__] = ACTIONS(1585),
    [sym___netlinx__] = ACTIONS(1585),
    [sym_netlinx_axi_version] = ACTIONS(1585),
    [sym_url_flg_tcp] = ACTIONS(1585),
    [sym_url_flg_accinfopresent] = ACTIONS(1585),
    [sym_url_flg_temp] = ACTIONS(1585),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1585),
    [sym_url_flg_stat_mask] = ACTIONS(1585),
    [sym_url_flg_stat_lookup] = ACTIONS(1585),
    [sym_url_flg_stat_connecting] = ACTIONS(1585),
    [sym_url_flg_stat_waiting] = ACTIONS(1585),
    [sym_url_flg_stat_connected] = ACTIONS(1585),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1585),
    [sym_first_virtual_device] = ACTIONS(1585),
    [sym_first_local_port] = ACTIONS(1585),
    [sym_dynamic_virtual_device] = ACTIONS(1585),
    [sym_timeline_once] = ACTIONS(1585),
    [sym_timeline_repeat] = ACTIONS(1585),
    [sym_timeline_absolute] = ACTIONS(1585),
    [sym_timeline_relative] = ACTIONS(1585),
    [sym_do_push_timed_infinite] = ACTIONS(1585),
    [sym_source_type_no_address] = ACTIONS(1585),
    [sym_source_type_neuron_id] = ACTIONS(1585),
    [sym_source_type_ip_address] = ACTIONS(1585),
    [sym_source_type_axlink] = ACTIONS(1585),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1585),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1585),
    [sym_source_type_ip_socket_address] = ACTIONS(1585),
    [sym_source_type_rs232] = ACTIONS(1585),
    [sym_source_type_internal] = ACTIONS(1585),
    [sym_source_type_mac_address] = ACTIONS(1585),
    [sym_source_type_ipv4_port] = ACTIONS(1585),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1585),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1585),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1585),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1585),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1585),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1585),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1585),
    [sym_true] = ACTIONS(1585),
    [sym_false] = ACTIONS(1585),
    [sym_file_read_only] = ACTIONS(1585),
    [sym_file_rw_new] = ACTIONS(1585),
    [sym_file_rw_append] = ACTIONS(1585),
    [sym_ip_tcp] = ACTIONS(1585),
    [sym_ip_udp] = ACTIONS(1585),
    [sym_ip_udp_2way] = ACTIONS(1585),
    [sym_xml_encode_types] = ACTIONS(1585),
    [sym_xml_encode_char_as_list] = ACTIONS(1585),
    [sym_xml_encode_le] = ACTIONS(1585),
    [sym_xml_decode_types] = ACTIONS(1585),
    [sym_xml_decode_no_preserve] = ACTIONS(1585),
    [sym_duet_dev_type_display_device] = ACTIONS(1585),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1585),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1585),
    [sym_duet_dev_type_io_device] = ACTIONS(1585),
    [sym_duet_dev_type_relay_device] = ACTIONS(1585),
    [sym_duet_dev_type_ups] = ACTIONS(1585),
    [sym_duet_dev_type_amplifier] = ACTIONS(1585),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1585),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1585),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1585),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1585),
    [sym_duet_dev_type_camera] = ACTIONS(1585),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1585),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1585),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1585),
    [sym_duet_dev_type_dss] = ACTIONS(1585),
    [sym_duet_dev_type_dvr] = ACTIONS(1585),
    [sym_duet_dev_type_disc_device] = ACTIONS(1585),
    [sym_duet_dev_type_document_camera] = ACTIONS(1585),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1585),
    [sym_duet_dev_type_hvac] = ACTIONS(1585),
    [sym_duet_dev_type_keypad] = ACTIONS(1585),
    [sym_duet_dev_type_light] = ACTIONS(1585),
    [sym_duet_dev_type_monitor] = ACTIONS(1585),
    [sym_duet_dev_type_motor] = ACTIONS(1585),
    [sym_duet_dev_type_multi_window] = ACTIONS(1585),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1585),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1585),
    [sym_duet_dev_type_receiver] = ACTIONS(1585),
    [sym_duet_dev_type_security_system] = ACTIONS(1585),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1585),
    [sym_duet_dev_type_settop_box] = ACTIONS(1585),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1585),
    [sym_duet_dev_type_switcher] = ACTIONS(1585),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1585),
    [sym_duet_dev_type_tv] = ACTIONS(1585),
    [sym_duet_dev_type_utility] = ACTIONS(1585),
    [sym_duet_dev_type_vcr] = ACTIONS(1585),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1585),
    [sym_duet_dev_type_video_processor] = ACTIONS(1585),
    [sym_duet_dev_type_video_projector] = ACTIONS(1585),
    [sym_duet_dev_type_video_wall] = ACTIONS(1585),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1585),
    [sym_duet_dev_type_weather] = ACTIONS(1585),
    [sym_duet_dev_not_polled] = ACTIONS(1585),
    [sym_duet_dev_polled] = ACTIONS(1585),
    [sym_clkmgr_mode_network] = ACTIONS(1585),
    [sym_clkmgr_mode_standalone] = ACTIONS(1585),
    [sym_normal_standby] = ACTIONS(1585),
    [sym_normal_wake] = ACTIONS(1585),
    [sym_ip_multicast_ttl_option] = ACTIONS(1585),
    [sym_ip_ttl_subnet] = ACTIONS(1585),
    [sym_ip_ttl_site] = ACTIONS(1585),
    [sym_ip_ttl_region] = ACTIONS(1585),
    [sym_ip_ttl_continent] = ACTIONS(1585),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1585),
    [sym_ip_nodelay_on] = ACTIONS(1585),
    [sym_ip_nodelay_off] = ACTIONS(1585),
    [sym_amx_error] = ACTIONS(1585),
    [sym_amx_warning] = ACTIONS(1585),
    [sym_amx_info] = ACTIONS(1585),
    [sym_amx_debug] = ACTIONS(1585),
    [sym_netlinx_login_success] = ACTIONS(1585),
    [sym_netlinx_login_fail] = ACTIONS(1585),
    [sym_netlinx_logout] = ACTIONS(1585),
    [sym_null_str] = ACTIONS(1585),
    [sym_smtp_address] = ACTIONS(1585),
    [sym_smtp_port_number] = ACTIONS(1585),
    [sym_smtp_username] = ACTIONS(1585),
    [sym_smtp_password] = ACTIONS(1585),
    [sym_smtp_require_tls] = ACTIONS(1585),
    [sym_smtp_from] = ACTIONS(1585),
    [sym_smtp_tls_true] = ACTIONS(1585),
    [sym_smtp_tls_false] = ACTIONS(1585),
    [sym_listview_on_row_select_event] = ACTIONS(1585),
    [sym_data_structure_unknown] = ACTIONS(1585),
    [sym_data_structure_datafeed] = ACTIONS(1585),
    [sym_data_structure_datarecordset] = ACTIONS(1585),
    [sym_data_structure_datarecord] = ACTIONS(1585),
    [sym_data_structure_datafield] = ACTIONS(1585),
    [sym_data_max_content_field_count] = ACTIONS(1585),
    [sym_data_max_metadata_field_count] = ACTIONS(1585),
    [sym_data_max_id_length] = ACTIONS(1585),
    [sym_data_max_name_length] = ACTIONS(1585),
    [sym_data_max_description_length] = ACTIONS(1585),
    [sym_data_max_type_length] = ACTIONS(1585),
    [sym_data_max_format_length] = ACTIONS(1585),
    [sym_data_max_source_length] = ACTIONS(1585),
    [sym_data_max_value_length] = ACTIONS(1585),
    [sym_data_max_label_length] = ACTIONS(1585),
    [sym_data_max_url_length] = ACTIONS(1585),
    [sym_max_last_login_info_length] = ACTIONS(1585),
    [sym_data_type_unknown] = ACTIONS(1585),
    [sym_data_type_string] = ACTIONS(1585),
    [sym_data_type_datetime] = ACTIONS(1585),
    [sym_data_type_date] = ACTIONS(1585),
    [sym_data_type_time] = ACTIONS(1585),
    [sym_data_type_image] = ACTIONS(1585),
    [sym_data_format_url] = ACTIONS(1585),
    [sym_data_format_phone] = ACTIONS(1585),
    [sym_data_format_email] = ACTIONS(1585),
    [sym_data_format_iso8601] = ACTIONS(1585),
    [sym_auditlog_priv] = ACTIONS(1585),
    [sym_configuration_priv] = ACTIONS(1585),
    [sym_deviceconfig_priv] = ACTIONS(1585),
    [sym_ftp_priv] = ACTIONS(1585),
    [sym_http_priv] = ACTIONS(1585),
    [sym_networkconfig_priv] = ACTIONS(1585),
    [sym_programport_priv] = ACTIONS(1585),
    [sym_remoteui_priv] = ACTIONS(1585),
    [sym_securitycontrol_priv] = ACTIONS(1585),
    [sym_softwaremanagement_priv] = ACTIONS(1585),
    [sym_terminal_priv] = ACTIONS(1585),
    [sym_tpadmin_priv] = ACTIONS(1585),
    [sym_user1_priv] = ACTIONS(1585),
    [sym_user2_priv] = ACTIONS(1585),
    [sym_user3_priv] = ACTIONS(1585),
    [sym_user4_priv] = ACTIONS(1585),
    [sym_usermanagement_priv] = ACTIONS(1585),
    [sym_valid_account] = ACTIONS(1585),
    [sym_err_name_invalid] = ACTIONS(1585),
    [sym_err_password_invalid] = ACTIONS(1585),
    [sym_err_invalid_account] = ACTIONS(1585),
    [sym_err_no_authorization] = ACTIONS(1585),
    [sym_err_invalid_parameter] = ACTIONS(1585),
    [sym_err_user_locked_out] = ACTIONS(1585),
    [sym_err_user_account_expired] = ACTIONS(1585),
    [sym_err_general_error] = ACTIONS(1585),
    [sym_err_authentication_error] = ACTIONS(1585),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1585),
    [sym_tls_validate_certificate] = ACTIONS(1585),
    [sym____reserved___] = ACTIONS(1585),
    [sym_date] = ACTIONS(1585),
    [sym_day] = ACTIONS(1585),
    [sym_time] = ACTIONS(1585),
    [sym_ldate] = ACTIONS(1585),
    [sym_push_device] = ACTIONS(1585),
    [sym_push_channel] = ACTIONS(1585),
    [sym_push_devchan] = ACTIONS(1585),
    [sym_release_device] = ACTIONS(1585),
    [sym_release_channel] = ACTIONS(1585),
    [sym_release_devchan] = ACTIONS(1585),
    [sym_master_slot] = ACTIONS(1585),
    [sym_get_pulse_time] = ACTIONS(1585),
    [sym_get_timer] = ACTIONS(1585),
    [sym_master_sn] = ACTIONS(1585),
    [sym_system_number] = ACTIONS(1585),
    [sym_button] = ACTIONS(1585),
    [sym_dv_channel] = ACTIONS(1585),
    [sym_data] = ACTIONS(1585),
    [sym_level] = ACTIONS(1585),
    [sym_timeline] = ACTIONS(1585),
    [sym_channel] = ACTIONS(1585),
    [sym_custom] = ACTIONS(1585),
    [sym_atoi] = ACTIONS(1585),
    [sym_atol] = ACTIONS(1585),
    [sym_atof] = ACTIONS(1585),
    [sym_itoa] = ACTIONS(1585),
    [sym_itohex] = ACTIONS(1585),
    [sym_hextoi] = ACTIONS(1585),
    [sym_ftoa] = ACTIONS(1585),
    [sym_find_string] = ACTIONS(1585),
    [sym_left_string] = ACTIONS(1585),
    [sym_right_string] = ACTIONS(1585),
    [sym_mid_string] = ACTIONS(1585),
    [sym_remove_string] = ACTIONS(1585),
    [sym_get_buffer_string] = ACTIONS(1585),
    [sym_get_buffer_char] = ACTIONS(1585),
    [sym_get_multi_buffer_string] = ACTIONS(1585),
    [sym_length_string] = ACTIONS(1585),
    [sym_max_length_string] = ACTIONS(1585),
    [sym_set_length_string] = ACTIONS(1585),
    [sym_compare_string] = ACTIONS(1585),
    [sym_upper_string] = ACTIONS(1585),
    [sym_lower_string] = ACTIONS(1585),
    [sym_get_last] = ACTIONS(1585),
    [sym_date_to_day] = ACTIONS(1585),
    [sym_date_to_month] = ACTIONS(1585),
    [sym_date_to_year] = ACTIONS(1585),
    [sym_time_to_hour] = ACTIONS(1585),
    [sym_time_to_minute] = ACTIONS(1585),
    [sym_time_to_second] = ACTIONS(1585),
    [sym_day_of_week] = ACTIONS(1585),
    [sym_set_pulse_time] = ACTIONS(1585),
    [sym_set_timer] = ACTIONS(1585),
    [sym_file_open] = ACTIONS(1585),
    [sym_file_close] = ACTIONS(1585),
    [sym_file_read] = ACTIONS(1585),
    [sym_file_read_line] = ACTIONS(1585),
    [sym_file_write] = ACTIONS(1585),
    [sym_file_write_line] = ACTIONS(1585),
    [sym_file_copy] = ACTIONS(1585),
    [sym_file_delete] = ACTIONS(1585),
    [sym_file_rename] = ACTIONS(1585),
    [sym_file_dir] = ACTIONS(1585),
    [sym_file_getdir] = ACTIONS(1585),
    [sym_file_setdir] = ACTIONS(1585),
    [sym_file_create_dir] = ACTIONS(1585),
    [sym_file_remove_dir] = ACTIONS(1585),
    [sym_file_seek] = ACTIONS(1585),
    [sym_length_array] = ACTIONS(1585),
    [sym_max_length_array] = ACTIONS(1585),
    [sym_set_length_array] = ACTIONS(1585),
    [sym_ip_client_open] = ACTIONS(1585),
    [sym_ip_client_close] = ACTIONS(1585),
    [sym_ip_server_open] = ACTIONS(1585),
    [sym_ip_server_close] = ACTIONS(1585),
    [sym_ip_bound_client_open] = ACTIONS(1585),
    [sym_ip_mc_server_open] = ACTIONS(1585),
    [sym_ip_set_option] = ACTIONS(1585),
    [sym_ssh_client_open] = ACTIONS(1585),
    [sym_ssh_client_close] = ACTIONS(1585),
    [sym_tls_client_open] = ACTIONS(1585),
    [sym_tls_client_close] = ACTIONS(1585),
    [sym_do_push] = ACTIONS(1585),
    [sym_do_push_timed] = ACTIONS(1585),
    [sym_do_release] = ACTIONS(1585),
    [sym_device_id] = ACTIONS(1585),
    [sym_device_info] = ACTIONS(1585),
    [sym_device_id_string] = ACTIONS(1585),
    [sym_device_standby] = ACTIONS(1585),
    [sym_device_wake] = ACTIONS(1585),
    [sym_timeline_create] = ACTIONS(1585),
    [sym_timeline_kill] = ACTIONS(1585),
    [sym_timeline_reload] = ACTIONS(1585),
    [sym_timeline_pause] = ACTIONS(1585),
    [sym_timeline_restart] = ACTIONS(1585),
    [sym_timeline_set] = ACTIONS(1585),
    [sym_timeline_get] = ACTIONS(1585),
    [sym_timeline_active] = ACTIONS(1585),
    [sym_timeline_dynamic_id] = ACTIONS(1585),
    [sym_variable_to_string] = ACTIONS(1585),
    [sym_string_to_variable] = ACTIONS(1585),
    [sym_length_variable_to_string] = ACTIONS(1585),
    [sym_type_cast] = ACTIONS(1585),
    [sym_variable_to_xml] = ACTIONS(1585),
    [sym_xml_to_variable] = ACTIONS(1585),
    [sym_length_variable_to_xml] = ACTIONS(1585),
    [sym_redirect_string] = ACTIONS(1585),
    [sym_random_number] = ACTIONS(1585),
    [sym_get_unique_id] = ACTIONS(1585),
    [sym_get_system_number] = ACTIONS(1585),
    [sym_set_system_number] = ACTIONS(1585),
    [sym_get_serial_number] = ACTIONS(1585),
    [sym_get_master_build] = ACTIONS(1585),
    [sym_set_validation_code] = ACTIONS(1585),
    [sym_reboot] = ACTIONS(1585),
    [sym_load_duet_module] = ACTIONS(1585),
    [sym_unload_duet_module] = ACTIONS(1585),
    [sym_duet_mem_size_set] = ACTIONS(1585),
    [sym_duet_mem_size_get] = ACTIONS(1585),
    [sym_dynamic_application_device] = ACTIONS(1585),
    [sym_dynamic_polled_port] = ACTIONS(1585),
    [sym_static_port_binding] = ACTIONS(1585),
    [sym_static_ip_binding] = ACTIONS(1585),
    [sym_get_dns_list] = ACTIONS(1585),
    [sym_set_dns_list] = ACTIONS(1585),
    [sym_get_ip_address] = ACTIONS(1585),
    [sym_set_ip_address] = ACTIONS(1585),
    [sym_get_url_list] = ACTIONS(1585),
    [sym_add_url_entry] = ACTIONS(1585),
    [sym_delete_url_entry] = ACTIONS(1585),
    [sym_abs_value] = ACTIONS(1585),
    [sym_max_value] = ACTIONS(1585),
    [sym_min_value] = ACTIONS(1585),
    [sym_exp_value] = ACTIONS(1585),
    [sym_log_value] = ACTIONS(1585),
    [sym_log10_value] = ACTIONS(1585),
    [sym_power_value] = ACTIONS(1585),
    [sym_sqrt_value] = ACTIONS(1585),
    [sym_raw_be] = ACTIONS(1585),
    [sym_raw_le] = ACTIONS(1585),
    [sym_internal_queue_size_set] = ACTIONS(1585),
    [sym_internal_queue_size_get] = ACTIONS(1585),
    [sym_internal_threshold_set] = ACTIONS(1585),
    [sym_internal_threshold_get] = ACTIONS(1585),
    [sym_rebuilt_event] = ACTIONS(1585),
    [sym_do_custom_event] = ACTIONS(1585),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1585),
    [sym_clkmgr_set_clk_source] = ACTIONS(1585),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1585),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1585),
    [sym_clkmgr_get_timezone] = ACTIONS(1585),
    [sym_clkmgr_set_timezone] = ACTIONS(1585),
    [sym_clkmgr_get_resync_period] = ACTIONS(1585),
    [sym_clkmgr_set_resync_period] = ACTIONS(1585),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1585),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1585),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1585),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1585),
    [sym_clkmgr_get_timeservers] = ACTIONS(1585),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1585),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1585),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1585),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1585),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1585),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1585),
    [sym_set_log_level] = ACTIONS(1585),
    [sym_get_log_level] = ACTIONS(1585),
    [sym_amx_log] = ACTIONS(1585),
    [sym_smtp_server_config_set] = ACTIONS(1585),
    [sym_smtp_server_config_get] = ACTIONS(1585),
    [sym_smtp_send] = ACTIONS(1585),
    [sym_validate_netlinx_account] = ACTIONS(1585),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1585),
    [sym_authenticate_certificate] = ACTIONS(1585),
    [sym_audit_netlinx_session_event] = ACTIONS(1585),
    [sym_audit_netlinx_generic_event] = ACTIONS(1585),
    [sym_get_available_flash_disk_space] = ACTIONS(1585),
    [sym_get_max_flash_disk_space] = ACTIONS(1585),
    [sym_data_create_feed] = ACTIONS(1585),
    [sym_data_delete_feed] = ACTIONS(1585),
    [sym_data_publish_feed] = ACTIONS(1585),
    [sym_data_get_published_feed] = ACTIONS(1585),
    [sym_data_add_record] = ACTIONS(1585),
    [sym_data_get_event_record] = ACTIONS(1585),
    [sym__wc_data_create_feed] = ACTIONS(1585),
    [sym__wc_data_add_record] = ACTIONS(1585),
    [sym__wc_data_get_event_record] = ACTIONS(1585),
    [sym_format] = ACTIONS(1585),
    [sym_astro_clock] = ACTIONS(1585),
    [sym_set_outdoor_temperature] = ACTIONS(1585),
    [sym_set_virtual_level_count] = ACTIONS(1585),
    [sym_set_virtual_channel_count] = ACTIONS(1585),
    [sym_set_virtual_port_count] = ACTIONS(1585),
    [sym__wc_encode_int] = ACTIONS(1585),
    [sym__wc_decode_int] = ACTIONS(1585),
    [sym__wc_int] = ACTIONS(1585),
    [sym__wc_to_ch_int] = ACTIONS(1585),
    [sym__ch_to_wc_int] = ACTIONS(1585),
    [sym__wc_find_string_int] = ACTIONS(1585),
    [sym__wc_left_string_int] = ACTIONS(1585),
    [sym__wc_lower_string_int] = ACTIONS(1585),
    [sym__wc_mid_string_int] = ACTIONS(1585),
    [sym__wc_remove_string_int] = ACTIONS(1585),
    [sym__wc_right_string_int] = ACTIONS(1585),
    [sym__wc_upper_string_int] = ACTIONS(1585),
    [sym__wc_compare_string_int] = ACTIONS(1585),
    [sym__wc_get_buffer_char_int] = ACTIONS(1585),
    [sym__wc_get_buffer_string_int] = ACTIONS(1585),
    [sym__wc_concat_string_int] = ACTIONS(1585),
    [sym__wc_file_open_int] = ACTIONS(1585),
    [sym__wc_file_close_int] = ACTIONS(1585),
    [sym__wc_file_read_int] = ACTIONS(1585),
    [sym__wc_file_read_line_int] = ACTIONS(1585),
    [sym__wc_file_write_int] = ACTIONS(1585),
    [sym__wc_file_write_line_int] = ACTIONS(1585),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1587),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1587),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1587),
    [aux_sym_preproc_else_token1] = ACTIONS(1587),
    [anon_sym_COMMA] = ACTIONS(1587),
    [anon_sym_LBRACK] = ACTIONS(1587),
    [anon_sym_LBRACE] = ACTIONS(1587),
    [anon_sym_RBRACE] = ACTIONS(1587),
    [anon_sym_COLON] = ACTIONS(1587),
    [anon_sym_LPAREN] = ACTIONS(1585),
    [anon_sym_BANG] = ACTIONS(1585),
    [anon_sym_TILDE] = ACTIONS(1587),
    [anon_sym_DASH] = ACTIONS(1585),
    [anon_sym_PLUS] = ACTIONS(1585),
    [anon_sym_STAR] = ACTIONS(1587),
    [anon_sym_SLASH] = ACTIONS(1585),
    [anon_sym_PERCENT] = ACTIONS(1587),
    [anon_sym_PIPE_PIPE] = ACTIONS(1587),
    [anon_sym_AMP_AMP] = ACTIONS(1587),
    [anon_sym_PIPE] = ACTIONS(1585),
    [anon_sym_CARET] = ACTIONS(1587),
    [anon_sym_AMP] = ACTIONS(1585),
    [anon_sym_EQ_EQ] = ACTIONS(1587),
    [anon_sym_BANG_EQ] = ACTIONS(1587),
    [anon_sym_LT_GT] = ACTIONS(1587),
    [anon_sym_GT] = ACTIONS(1585),
    [anon_sym_GT_EQ] = ACTIONS(1587),
    [anon_sym_LT_EQ] = ACTIONS(1587),
    [anon_sym_LT] = ACTIONS(1585),
    [anon_sym_LT_LT] = ACTIONS(1587),
    [anon_sym_GT_GT] = ACTIONS(1587),
    [anon_sym_DASH_DASH] = ACTIONS(1587),
    [anon_sym_PLUS_PLUS] = ACTIONS(1587),
    [anon_sym_DOT] = ACTIONS(1585),
    [anon_sym_DQUOTE] = ACTIONS(1587),
    [anon_sym_SQUOTE] = ACTIONS(1587),
    [sym_number_literal] = ACTIONS(1587),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(613)] = {
    [sym_identifier] = ACTIONS(1569),
    [sym_and] = ACTIONS(1569),
    [sym_or] = ACTIONS(1569),
    [sym_not] = ACTIONS(1569),
    [sym_xor] = ACTIONS(1569),
    [sym_band] = ACTIONS(1569),
    [sym_bor] = ACTIONS(1569),
    [sym_bxor] = ACTIONS(1569),
    [sym_bnot] = ACTIONS(1569),
    [sym_lshift] = ACTIONS(1569),
    [sym_rshift] = ACTIONS(1569),
    [sym___file__] = ACTIONS(1569),
    [sym___date__] = ACTIONS(1569),
    [sym___ldate__] = ACTIONS(1569),
    [sym___time__] = ACTIONS(1569),
    [sym___line__] = ACTIONS(1569),
    [sym___name__] = ACTIONS(1569),
    [sym___netlinx__] = ACTIONS(1569),
    [sym_netlinx_axi_version] = ACTIONS(1569),
    [sym_url_flg_tcp] = ACTIONS(1569),
    [sym_url_flg_accinfopresent] = ACTIONS(1569),
    [sym_url_flg_temp] = ACTIONS(1569),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1569),
    [sym_url_flg_stat_mask] = ACTIONS(1569),
    [sym_url_flg_stat_lookup] = ACTIONS(1569),
    [sym_url_flg_stat_connecting] = ACTIONS(1569),
    [sym_url_flg_stat_waiting] = ACTIONS(1569),
    [sym_url_flg_stat_connected] = ACTIONS(1569),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1569),
    [sym_first_virtual_device] = ACTIONS(1569),
    [sym_first_local_port] = ACTIONS(1569),
    [sym_dynamic_virtual_device] = ACTIONS(1569),
    [sym_timeline_once] = ACTIONS(1569),
    [sym_timeline_repeat] = ACTIONS(1569),
    [sym_timeline_absolute] = ACTIONS(1569),
    [sym_timeline_relative] = ACTIONS(1569),
    [sym_do_push_timed_infinite] = ACTIONS(1569),
    [sym_source_type_no_address] = ACTIONS(1569),
    [sym_source_type_neuron_id] = ACTIONS(1569),
    [sym_source_type_ip_address] = ACTIONS(1569),
    [sym_source_type_axlink] = ACTIONS(1569),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1569),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1569),
    [sym_source_type_ip_socket_address] = ACTIONS(1569),
    [sym_source_type_rs232] = ACTIONS(1569),
    [sym_source_type_internal] = ACTIONS(1569),
    [sym_source_type_mac_address] = ACTIONS(1569),
    [sym_source_type_ipv4_port] = ACTIONS(1569),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1569),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1569),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1569),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1569),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1569),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1569),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1569),
    [sym_true] = ACTIONS(1569),
    [sym_false] = ACTIONS(1569),
    [sym_file_read_only] = ACTIONS(1569),
    [sym_file_rw_new] = ACTIONS(1569),
    [sym_file_rw_append] = ACTIONS(1569),
    [sym_ip_tcp] = ACTIONS(1569),
    [sym_ip_udp] = ACTIONS(1569),
    [sym_ip_udp_2way] = ACTIONS(1569),
    [sym_xml_encode_types] = ACTIONS(1569),
    [sym_xml_encode_char_as_list] = ACTIONS(1569),
    [sym_xml_encode_le] = ACTIONS(1569),
    [sym_xml_decode_types] = ACTIONS(1569),
    [sym_xml_decode_no_preserve] = ACTIONS(1569),
    [sym_duet_dev_type_display_device] = ACTIONS(1569),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1569),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1569),
    [sym_duet_dev_type_io_device] = ACTIONS(1569),
    [sym_duet_dev_type_relay_device] = ACTIONS(1569),
    [sym_duet_dev_type_ups] = ACTIONS(1569),
    [sym_duet_dev_type_amplifier] = ACTIONS(1569),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1569),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1569),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1569),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1569),
    [sym_duet_dev_type_camera] = ACTIONS(1569),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1569),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1569),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1569),
    [sym_duet_dev_type_dss] = ACTIONS(1569),
    [sym_duet_dev_type_dvr] = ACTIONS(1569),
    [sym_duet_dev_type_disc_device] = ACTIONS(1569),
    [sym_duet_dev_type_document_camera] = ACTIONS(1569),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1569),
    [sym_duet_dev_type_hvac] = ACTIONS(1569),
    [sym_duet_dev_type_keypad] = ACTIONS(1569),
    [sym_duet_dev_type_light] = ACTIONS(1569),
    [sym_duet_dev_type_monitor] = ACTIONS(1569),
    [sym_duet_dev_type_motor] = ACTIONS(1569),
    [sym_duet_dev_type_multi_window] = ACTIONS(1569),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1569),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1569),
    [sym_duet_dev_type_receiver] = ACTIONS(1569),
    [sym_duet_dev_type_security_system] = ACTIONS(1569),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1569),
    [sym_duet_dev_type_settop_box] = ACTIONS(1569),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1569),
    [sym_duet_dev_type_switcher] = ACTIONS(1569),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1569),
    [sym_duet_dev_type_tv] = ACTIONS(1569),
    [sym_duet_dev_type_utility] = ACTIONS(1569),
    [sym_duet_dev_type_vcr] = ACTIONS(1569),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1569),
    [sym_duet_dev_type_video_processor] = ACTIONS(1569),
    [sym_duet_dev_type_video_projector] = ACTIONS(1569),
    [sym_duet_dev_type_video_wall] = ACTIONS(1569),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1569),
    [sym_duet_dev_type_weather] = ACTIONS(1569),
    [sym_duet_dev_not_polled] = ACTIONS(1569),
    [sym_duet_dev_polled] = ACTIONS(1569),
    [sym_clkmgr_mode_network] = ACTIONS(1569),
    [sym_clkmgr_mode_standalone] = ACTIONS(1569),
    [sym_normal_standby] = ACTIONS(1569),
    [sym_normal_wake] = ACTIONS(1569),
    [sym_ip_multicast_ttl_option] = ACTIONS(1569),
    [sym_ip_ttl_subnet] = ACTIONS(1569),
    [sym_ip_ttl_site] = ACTIONS(1569),
    [sym_ip_ttl_region] = ACTIONS(1569),
    [sym_ip_ttl_continent] = ACTIONS(1569),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1569),
    [sym_ip_nodelay_on] = ACTIONS(1569),
    [sym_ip_nodelay_off] = ACTIONS(1569),
    [sym_amx_error] = ACTIONS(1569),
    [sym_amx_warning] = ACTIONS(1569),
    [sym_amx_info] = ACTIONS(1569),
    [sym_amx_debug] = ACTIONS(1569),
    [sym_netlinx_login_success] = ACTIONS(1569),
    [sym_netlinx_login_fail] = ACTIONS(1569),
    [sym_netlinx_logout] = ACTIONS(1569),
    [sym_null_str] = ACTIONS(1569),
    [sym_smtp_address] = ACTIONS(1569),
    [sym_smtp_port_number] = ACTIONS(1569),
    [sym_smtp_username] = ACTIONS(1569),
    [sym_smtp_password] = ACTIONS(1569),
    [sym_smtp_require_tls] = ACTIONS(1569),
    [sym_smtp_from] = ACTIONS(1569),
    [sym_smtp_tls_true] = ACTIONS(1569),
    [sym_smtp_tls_false] = ACTIONS(1569),
    [sym_listview_on_row_select_event] = ACTIONS(1569),
    [sym_data_structure_unknown] = ACTIONS(1569),
    [sym_data_structure_datafeed] = ACTIONS(1569),
    [sym_data_structure_datarecordset] = ACTIONS(1569),
    [sym_data_structure_datarecord] = ACTIONS(1569),
    [sym_data_structure_datafield] = ACTIONS(1569),
    [sym_data_max_content_field_count] = ACTIONS(1569),
    [sym_data_max_metadata_field_count] = ACTIONS(1569),
    [sym_data_max_id_length] = ACTIONS(1569),
    [sym_data_max_name_length] = ACTIONS(1569),
    [sym_data_max_description_length] = ACTIONS(1569),
    [sym_data_max_type_length] = ACTIONS(1569),
    [sym_data_max_format_length] = ACTIONS(1569),
    [sym_data_max_source_length] = ACTIONS(1569),
    [sym_data_max_value_length] = ACTIONS(1569),
    [sym_data_max_label_length] = ACTIONS(1569),
    [sym_data_max_url_length] = ACTIONS(1569),
    [sym_max_last_login_info_length] = ACTIONS(1569),
    [sym_data_type_unknown] = ACTIONS(1569),
    [sym_data_type_string] = ACTIONS(1569),
    [sym_data_type_datetime] = ACTIONS(1569),
    [sym_data_type_date] = ACTIONS(1569),
    [sym_data_type_time] = ACTIONS(1569),
    [sym_data_type_image] = ACTIONS(1569),
    [sym_data_format_url] = ACTIONS(1569),
    [sym_data_format_phone] = ACTIONS(1569),
    [sym_data_format_email] = ACTIONS(1569),
    [sym_data_format_iso8601] = ACTIONS(1569),
    [sym_auditlog_priv] = ACTIONS(1569),
    [sym_configuration_priv] = ACTIONS(1569),
    [sym_deviceconfig_priv] = ACTIONS(1569),
    [sym_ftp_priv] = ACTIONS(1569),
    [sym_http_priv] = ACTIONS(1569),
    [sym_networkconfig_priv] = ACTIONS(1569),
    [sym_programport_priv] = ACTIONS(1569),
    [sym_remoteui_priv] = ACTIONS(1569),
    [sym_securitycontrol_priv] = ACTIONS(1569),
    [sym_softwaremanagement_priv] = ACTIONS(1569),
    [sym_terminal_priv] = ACTIONS(1569),
    [sym_tpadmin_priv] = ACTIONS(1569),
    [sym_user1_priv] = ACTIONS(1569),
    [sym_user2_priv] = ACTIONS(1569),
    [sym_user3_priv] = ACTIONS(1569),
    [sym_user4_priv] = ACTIONS(1569),
    [sym_usermanagement_priv] = ACTIONS(1569),
    [sym_valid_account] = ACTIONS(1569),
    [sym_err_name_invalid] = ACTIONS(1569),
    [sym_err_password_invalid] = ACTIONS(1569),
    [sym_err_invalid_account] = ACTIONS(1569),
    [sym_err_no_authorization] = ACTIONS(1569),
    [sym_err_invalid_parameter] = ACTIONS(1569),
    [sym_err_user_locked_out] = ACTIONS(1569),
    [sym_err_user_account_expired] = ACTIONS(1569),
    [sym_err_general_error] = ACTIONS(1569),
    [sym_err_authentication_error] = ACTIONS(1569),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1569),
    [sym_tls_validate_certificate] = ACTIONS(1569),
    [sym____reserved___] = ACTIONS(1569),
    [sym_date] = ACTIONS(1569),
    [sym_day] = ACTIONS(1569),
    [sym_time] = ACTIONS(1569),
    [sym_ldate] = ACTIONS(1569),
    [sym_push_device] = ACTIONS(1569),
    [sym_push_channel] = ACTIONS(1569),
    [sym_push_devchan] = ACTIONS(1569),
    [sym_release_device] = ACTIONS(1569),
    [sym_release_channel] = ACTIONS(1569),
    [sym_release_devchan] = ACTIONS(1569),
    [sym_master_slot] = ACTIONS(1569),
    [sym_get_pulse_time] = ACTIONS(1569),
    [sym_get_timer] = ACTIONS(1569),
    [sym_master_sn] = ACTIONS(1569),
    [sym_system_number] = ACTIONS(1569),
    [sym_button] = ACTIONS(1569),
    [sym_dv_channel] = ACTIONS(1569),
    [sym_data] = ACTIONS(1569),
    [sym_level] = ACTIONS(1569),
    [sym_timeline] = ACTIONS(1569),
    [sym_channel] = ACTIONS(1569),
    [sym_custom] = ACTIONS(1569),
    [sym_atoi] = ACTIONS(1569),
    [sym_atol] = ACTIONS(1569),
    [sym_atof] = ACTIONS(1569),
    [sym_itoa] = ACTIONS(1569),
    [sym_itohex] = ACTIONS(1569),
    [sym_hextoi] = ACTIONS(1569),
    [sym_ftoa] = ACTIONS(1569),
    [sym_find_string] = ACTIONS(1569),
    [sym_left_string] = ACTIONS(1569),
    [sym_right_string] = ACTIONS(1569),
    [sym_mid_string] = ACTIONS(1569),
    [sym_remove_string] = ACTIONS(1569),
    [sym_get_buffer_string] = ACTIONS(1569),
    [sym_get_buffer_char] = ACTIONS(1569),
    [sym_get_multi_buffer_string] = ACTIONS(1569),
    [sym_length_string] = ACTIONS(1569),
    [sym_max_length_string] = ACTIONS(1569),
    [sym_set_length_string] = ACTIONS(1569),
    [sym_compare_string] = ACTIONS(1569),
    [sym_upper_string] = ACTIONS(1569),
    [sym_lower_string] = ACTIONS(1569),
    [sym_get_last] = ACTIONS(1569),
    [sym_date_to_day] = ACTIONS(1569),
    [sym_date_to_month] = ACTIONS(1569),
    [sym_date_to_year] = ACTIONS(1569),
    [sym_time_to_hour] = ACTIONS(1569),
    [sym_time_to_minute] = ACTIONS(1569),
    [sym_time_to_second] = ACTIONS(1569),
    [sym_day_of_week] = ACTIONS(1569),
    [sym_set_pulse_time] = ACTIONS(1569),
    [sym_set_timer] = ACTIONS(1569),
    [sym_file_open] = ACTIONS(1569),
    [sym_file_close] = ACTIONS(1569),
    [sym_file_read] = ACTIONS(1569),
    [sym_file_read_line] = ACTIONS(1569),
    [sym_file_write] = ACTIONS(1569),
    [sym_file_write_line] = ACTIONS(1569),
    [sym_file_copy] = ACTIONS(1569),
    [sym_file_delete] = ACTIONS(1569),
    [sym_file_rename] = ACTIONS(1569),
    [sym_file_dir] = ACTIONS(1569),
    [sym_file_getdir] = ACTIONS(1569),
    [sym_file_setdir] = ACTIONS(1569),
    [sym_file_create_dir] = ACTIONS(1569),
    [sym_file_remove_dir] = ACTIONS(1569),
    [sym_file_seek] = ACTIONS(1569),
    [sym_length_array] = ACTIONS(1569),
    [sym_max_length_array] = ACTIONS(1569),
    [sym_set_length_array] = ACTIONS(1569),
    [sym_ip_client_open] = ACTIONS(1569),
    [sym_ip_client_close] = ACTIONS(1569),
    [sym_ip_server_open] = ACTIONS(1569),
    [sym_ip_server_close] = ACTIONS(1569),
    [sym_ip_bound_client_open] = ACTIONS(1569),
    [sym_ip_mc_server_open] = ACTIONS(1569),
    [sym_ip_set_option] = ACTIONS(1569),
    [sym_ssh_client_open] = ACTIONS(1569),
    [sym_ssh_client_close] = ACTIONS(1569),
    [sym_tls_client_open] = ACTIONS(1569),
    [sym_tls_client_close] = ACTIONS(1569),
    [sym_do_push] = ACTIONS(1569),
    [sym_do_push_timed] = ACTIONS(1569),
    [sym_do_release] = ACTIONS(1569),
    [sym_device_id] = ACTIONS(1569),
    [sym_device_info] = ACTIONS(1569),
    [sym_device_id_string] = ACTIONS(1569),
    [sym_device_standby] = ACTIONS(1569),
    [sym_device_wake] = ACTIONS(1569),
    [sym_timeline_create] = ACTIONS(1569),
    [sym_timeline_kill] = ACTIONS(1569),
    [sym_timeline_reload] = ACTIONS(1569),
    [sym_timeline_pause] = ACTIONS(1569),
    [sym_timeline_restart] = ACTIONS(1569),
    [sym_timeline_set] = ACTIONS(1569),
    [sym_timeline_get] = ACTIONS(1569),
    [sym_timeline_active] = ACTIONS(1569),
    [sym_timeline_dynamic_id] = ACTIONS(1569),
    [sym_variable_to_string] = ACTIONS(1569),
    [sym_string_to_variable] = ACTIONS(1569),
    [sym_length_variable_to_string] = ACTIONS(1569),
    [sym_type_cast] = ACTIONS(1569),
    [sym_variable_to_xml] = ACTIONS(1569),
    [sym_xml_to_variable] = ACTIONS(1569),
    [sym_length_variable_to_xml] = ACTIONS(1569),
    [sym_redirect_string] = ACTIONS(1569),
    [sym_random_number] = ACTIONS(1569),
    [sym_get_unique_id] = ACTIONS(1569),
    [sym_get_system_number] = ACTIONS(1569),
    [sym_set_system_number] = ACTIONS(1569),
    [sym_get_serial_number] = ACTIONS(1569),
    [sym_get_master_build] = ACTIONS(1569),
    [sym_set_validation_code] = ACTIONS(1569),
    [sym_reboot] = ACTIONS(1569),
    [sym_load_duet_module] = ACTIONS(1569),
    [sym_unload_duet_module] = ACTIONS(1569),
    [sym_duet_mem_size_set] = ACTIONS(1569),
    [sym_duet_mem_size_get] = ACTIONS(1569),
    [sym_dynamic_application_device] = ACTIONS(1569),
    [sym_dynamic_polled_port] = ACTIONS(1569),
    [sym_static_port_binding] = ACTIONS(1569),
    [sym_static_ip_binding] = ACTIONS(1569),
    [sym_get_dns_list] = ACTIONS(1569),
    [sym_set_dns_list] = ACTIONS(1569),
    [sym_get_ip_address] = ACTIONS(1569),
    [sym_set_ip_address] = ACTIONS(1569),
    [sym_get_url_list] = ACTIONS(1569),
    [sym_add_url_entry] = ACTIONS(1569),
    [sym_delete_url_entry] = ACTIONS(1569),
    [sym_abs_value] = ACTIONS(1569),
    [sym_max_value] = ACTIONS(1569),
    [sym_min_value] = ACTIONS(1569),
    [sym_exp_value] = ACTIONS(1569),
    [sym_log_value] = ACTIONS(1569),
    [sym_log10_value] = ACTIONS(1569),
    [sym_power_value] = ACTIONS(1569),
    [sym_sqrt_value] = ACTIONS(1569),
    [sym_raw_be] = ACTIONS(1569),
    [sym_raw_le] = ACTIONS(1569),
    [sym_internal_queue_size_set] = ACTIONS(1569),
    [sym_internal_queue_size_get] = ACTIONS(1569),
    [sym_internal_threshold_set] = ACTIONS(1569),
    [sym_internal_threshold_get] = ACTIONS(1569),
    [sym_rebuilt_event] = ACTIONS(1569),
    [sym_do_custom_event] = ACTIONS(1569),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1569),
    [sym_clkmgr_set_clk_source] = ACTIONS(1569),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1569),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1569),
    [sym_clkmgr_get_timezone] = ACTIONS(1569),
    [sym_clkmgr_set_timezone] = ACTIONS(1569),
    [sym_clkmgr_get_resync_period] = ACTIONS(1569),
    [sym_clkmgr_set_resync_period] = ACTIONS(1569),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1569),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1569),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1569),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1569),
    [sym_clkmgr_get_timeservers] = ACTIONS(1569),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1569),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1569),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1569),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1569),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1569),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1569),
    [sym_set_log_level] = ACTIONS(1569),
    [sym_get_log_level] = ACTIONS(1569),
    [sym_amx_log] = ACTIONS(1569),
    [sym_smtp_server_config_set] = ACTIONS(1569),
    [sym_smtp_server_config_get] = ACTIONS(1569),
    [sym_smtp_send] = ACTIONS(1569),
    [sym_validate_netlinx_account] = ACTIONS(1569),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1569),
    [sym_authenticate_certificate] = ACTIONS(1569),
    [sym_audit_netlinx_session_event] = ACTIONS(1569),
    [sym_audit_netlinx_generic_event] = ACTIONS(1569),
    [sym_get_available_flash_disk_space] = ACTIONS(1569),
    [sym_get_max_flash_disk_space] = ACTIONS(1569),
    [sym_data_create_feed] = ACTIONS(1569),
    [sym_data_delete_feed] = ACTIONS(1569),
    [sym_data_publish_feed] = ACTIONS(1569),
    [sym_data_get_published_feed] = ACTIONS(1569),
    [sym_data_add_record] = ACTIONS(1569),
    [sym_data_get_event_record] = ACTIONS(1569),
    [sym__wc_data_create_feed] = ACTIONS(1569),
    [sym__wc_data_add_record] = ACTIONS(1569),
    [sym__wc_data_get_event_record] = ACTIONS(1569),
    [sym_format] = ACTIONS(1569),
    [sym_astro_clock] = ACTIONS(1569),
    [sym_set_outdoor_temperature] = ACTIONS(1569),
    [sym_set_virtual_level_count] = ACTIONS(1569),
    [sym_set_virtual_channel_count] = ACTIONS(1569),
    [sym_set_virtual_port_count] = ACTIONS(1569),
    [sym__wc_encode_int] = ACTIONS(1569),
    [sym__wc_decode_int] = ACTIONS(1569),
    [sym__wc_int] = ACTIONS(1569),
    [sym__wc_to_ch_int] = ACTIONS(1569),
    [sym__ch_to_wc_int] = ACTIONS(1569),
    [sym__wc_find_string_int] = ACTIONS(1569),
    [sym__wc_left_string_int] = ACTIONS(1569),
    [sym__wc_lower_string_int] = ACTIONS(1569),
    [sym__wc_mid_string_int] = ACTIONS(1569),
    [sym__wc_remove_string_int] = ACTIONS(1569),
    [sym__wc_right_string_int] = ACTIONS(1569),
    [sym__wc_upper_string_int] = ACTIONS(1569),
    [sym__wc_compare_string_int] = ACTIONS(1569),
    [sym__wc_get_buffer_char_int] = ACTIONS(1569),
    [sym__wc_get_buffer_string_int] = ACTIONS(1569),
    [sym__wc_concat_string_int] = ACTIONS(1569),
    [sym__wc_file_open_int] = ACTIONS(1569),
    [sym__wc_file_close_int] = ACTIONS(1569),
    [sym__wc_file_read_int] = ACTIONS(1569),
    [sym__wc_file_read_line_int] = ACTIONS(1569),
    [sym__wc_file_write_int] = ACTIONS(1569),
    [sym__wc_file_write_line_int] = ACTIONS(1569),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1571),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1571),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1571),
    [aux_sym_preproc_else_token1] = ACTIONS(1571),
    [anon_sym_COMMA] = ACTIONS(1571),
    [anon_sym_LBRACK] = ACTIONS(1571),
    [anon_sym_LBRACE] = ACTIONS(1571),
    [anon_sym_RBRACE] = ACTIONS(1571),
    [anon_sym_COLON] = ACTIONS(1571),
    [anon_sym_LPAREN] = ACTIONS(1569),
    [anon_sym_BANG] = ACTIONS(1569),
    [anon_sym_TILDE] = ACTIONS(1571),
    [anon_sym_DASH] = ACTIONS(1569),
    [anon_sym_PLUS] = ACTIONS(1569),
    [anon_sym_STAR] = ACTIONS(1571),
    [anon_sym_SLASH] = ACTIONS(1569),
    [anon_sym_PERCENT] = ACTIONS(1571),
    [anon_sym_PIPE_PIPE] = ACTIONS(1571),
    [anon_sym_AMP_AMP] = ACTIONS(1571),
    [anon_sym_PIPE] = ACTIONS(1569),
    [anon_sym_CARET] = ACTIONS(1571),
    [anon_sym_AMP] = ACTIONS(1569),
    [anon_sym_EQ_EQ] = ACTIONS(1571),
    [anon_sym_BANG_EQ] = ACTIONS(1571),
    [anon_sym_LT_GT] = ACTIONS(1571),
    [anon_sym_GT] = ACTIONS(1569),
    [anon_sym_GT_EQ] = ACTIONS(1571),
    [anon_sym_LT_EQ] = ACTIONS(1571),
    [anon_sym_LT] = ACTIONS(1569),
    [anon_sym_LT_LT] = ACTIONS(1571),
    [anon_sym_GT_GT] = ACTIONS(1571),
    [anon_sym_DASH_DASH] = ACTIONS(1571),
    [anon_sym_PLUS_PLUS] = ACTIONS(1571),
    [anon_sym_DOT] = ACTIONS(1569),
    [anon_sym_DQUOTE] = ACTIONS(1571),
    [anon_sym_SQUOTE] = ACTIONS(1571),
    [sym_number_literal] = ACTIONS(1571),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(614)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2217),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2217),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2217),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2226),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2217),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(615)] = {
    [sym_identifier] = ACTIONS(1589),
    [sym_and] = ACTIONS(1589),
    [sym_or] = ACTIONS(1589),
    [sym_not] = ACTIONS(1589),
    [sym_xor] = ACTIONS(1589),
    [sym_band] = ACTIONS(1589),
    [sym_bor] = ACTIONS(1589),
    [sym_bxor] = ACTIONS(1589),
    [sym_bnot] = ACTIONS(1589),
    [sym_lshift] = ACTIONS(1589),
    [sym_rshift] = ACTIONS(1589),
    [sym___file__] = ACTIONS(1589),
    [sym___date__] = ACTIONS(1589),
    [sym___ldate__] = ACTIONS(1589),
    [sym___time__] = ACTIONS(1589),
    [sym___line__] = ACTIONS(1589),
    [sym___name__] = ACTIONS(1589),
    [sym___netlinx__] = ACTIONS(1589),
    [sym_netlinx_axi_version] = ACTIONS(1589),
    [sym_url_flg_tcp] = ACTIONS(1589),
    [sym_url_flg_accinfopresent] = ACTIONS(1589),
    [sym_url_flg_temp] = ACTIONS(1589),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1589),
    [sym_url_flg_stat_mask] = ACTIONS(1589),
    [sym_url_flg_stat_lookup] = ACTIONS(1589),
    [sym_url_flg_stat_connecting] = ACTIONS(1589),
    [sym_url_flg_stat_waiting] = ACTIONS(1589),
    [sym_url_flg_stat_connected] = ACTIONS(1589),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1589),
    [sym_first_virtual_device] = ACTIONS(1589),
    [sym_first_local_port] = ACTIONS(1589),
    [sym_dynamic_virtual_device] = ACTIONS(1589),
    [sym_timeline_once] = ACTIONS(1589),
    [sym_timeline_repeat] = ACTIONS(1589),
    [sym_timeline_absolute] = ACTIONS(1589),
    [sym_timeline_relative] = ACTIONS(1589),
    [sym_do_push_timed_infinite] = ACTIONS(1589),
    [sym_source_type_no_address] = ACTIONS(1589),
    [sym_source_type_neuron_id] = ACTIONS(1589),
    [sym_source_type_ip_address] = ACTIONS(1589),
    [sym_source_type_axlink] = ACTIONS(1589),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1589),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1589),
    [sym_source_type_ip_socket_address] = ACTIONS(1589),
    [sym_source_type_rs232] = ACTIONS(1589),
    [sym_source_type_internal] = ACTIONS(1589),
    [sym_source_type_mac_address] = ACTIONS(1589),
    [sym_source_type_ipv4_port] = ACTIONS(1589),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1589),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1589),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1589),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1589),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1589),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1589),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1589),
    [sym_true] = ACTIONS(1589),
    [sym_false] = ACTIONS(1589),
    [sym_file_read_only] = ACTIONS(1589),
    [sym_file_rw_new] = ACTIONS(1589),
    [sym_file_rw_append] = ACTIONS(1589),
    [sym_ip_tcp] = ACTIONS(1589),
    [sym_ip_udp] = ACTIONS(1589),
    [sym_ip_udp_2way] = ACTIONS(1589),
    [sym_xml_encode_types] = ACTIONS(1589),
    [sym_xml_encode_char_as_list] = ACTIONS(1589),
    [sym_xml_encode_le] = ACTIONS(1589),
    [sym_xml_decode_types] = ACTIONS(1589),
    [sym_xml_decode_no_preserve] = ACTIONS(1589),
    [sym_duet_dev_type_display_device] = ACTIONS(1589),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1589),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1589),
    [sym_duet_dev_type_io_device] = ACTIONS(1589),
    [sym_duet_dev_type_relay_device] = ACTIONS(1589),
    [sym_duet_dev_type_ups] = ACTIONS(1589),
    [sym_duet_dev_type_amplifier] = ACTIONS(1589),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1589),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1589),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1589),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1589),
    [sym_duet_dev_type_camera] = ACTIONS(1589),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1589),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1589),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1589),
    [sym_duet_dev_type_dss] = ACTIONS(1589),
    [sym_duet_dev_type_dvr] = ACTIONS(1589),
    [sym_duet_dev_type_disc_device] = ACTIONS(1589),
    [sym_duet_dev_type_document_camera] = ACTIONS(1589),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1589),
    [sym_duet_dev_type_hvac] = ACTIONS(1589),
    [sym_duet_dev_type_keypad] = ACTIONS(1589),
    [sym_duet_dev_type_light] = ACTIONS(1589),
    [sym_duet_dev_type_monitor] = ACTIONS(1589),
    [sym_duet_dev_type_motor] = ACTIONS(1589),
    [sym_duet_dev_type_multi_window] = ACTIONS(1589),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1589),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1589),
    [sym_duet_dev_type_receiver] = ACTIONS(1589),
    [sym_duet_dev_type_security_system] = ACTIONS(1589),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1589),
    [sym_duet_dev_type_settop_box] = ACTIONS(1589),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1589),
    [sym_duet_dev_type_switcher] = ACTIONS(1589),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1589),
    [sym_duet_dev_type_tv] = ACTIONS(1589),
    [sym_duet_dev_type_utility] = ACTIONS(1589),
    [sym_duet_dev_type_vcr] = ACTIONS(1589),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1589),
    [sym_duet_dev_type_video_processor] = ACTIONS(1589),
    [sym_duet_dev_type_video_projector] = ACTIONS(1589),
    [sym_duet_dev_type_video_wall] = ACTIONS(1589),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1589),
    [sym_duet_dev_type_weather] = ACTIONS(1589),
    [sym_duet_dev_not_polled] = ACTIONS(1589),
    [sym_duet_dev_polled] = ACTIONS(1589),
    [sym_clkmgr_mode_network] = ACTIONS(1589),
    [sym_clkmgr_mode_standalone] = ACTIONS(1589),
    [sym_normal_standby] = ACTIONS(1589),
    [sym_normal_wake] = ACTIONS(1589),
    [sym_ip_multicast_ttl_option] = ACTIONS(1589),
    [sym_ip_ttl_subnet] = ACTIONS(1589),
    [sym_ip_ttl_site] = ACTIONS(1589),
    [sym_ip_ttl_region] = ACTIONS(1589),
    [sym_ip_ttl_continent] = ACTIONS(1589),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1589),
    [sym_ip_nodelay_on] = ACTIONS(1589),
    [sym_ip_nodelay_off] = ACTIONS(1589),
    [sym_amx_error] = ACTIONS(1589),
    [sym_amx_warning] = ACTIONS(1589),
    [sym_amx_info] = ACTIONS(1589),
    [sym_amx_debug] = ACTIONS(1589),
    [sym_netlinx_login_success] = ACTIONS(1589),
    [sym_netlinx_login_fail] = ACTIONS(1589),
    [sym_netlinx_logout] = ACTIONS(1589),
    [sym_null_str] = ACTIONS(1589),
    [sym_smtp_address] = ACTIONS(1589),
    [sym_smtp_port_number] = ACTIONS(1589),
    [sym_smtp_username] = ACTIONS(1589),
    [sym_smtp_password] = ACTIONS(1589),
    [sym_smtp_require_tls] = ACTIONS(1589),
    [sym_smtp_from] = ACTIONS(1589),
    [sym_smtp_tls_true] = ACTIONS(1589),
    [sym_smtp_tls_false] = ACTIONS(1589),
    [sym_listview_on_row_select_event] = ACTIONS(1589),
    [sym_data_structure_unknown] = ACTIONS(1589),
    [sym_data_structure_datafeed] = ACTIONS(1589),
    [sym_data_structure_datarecordset] = ACTIONS(1589),
    [sym_data_structure_datarecord] = ACTIONS(1589),
    [sym_data_structure_datafield] = ACTIONS(1589),
    [sym_data_max_content_field_count] = ACTIONS(1589),
    [sym_data_max_metadata_field_count] = ACTIONS(1589),
    [sym_data_max_id_length] = ACTIONS(1589),
    [sym_data_max_name_length] = ACTIONS(1589),
    [sym_data_max_description_length] = ACTIONS(1589),
    [sym_data_max_type_length] = ACTIONS(1589),
    [sym_data_max_format_length] = ACTIONS(1589),
    [sym_data_max_source_length] = ACTIONS(1589),
    [sym_data_max_value_length] = ACTIONS(1589),
    [sym_data_max_label_length] = ACTIONS(1589),
    [sym_data_max_url_length] = ACTIONS(1589),
    [sym_max_last_login_info_length] = ACTIONS(1589),
    [sym_data_type_unknown] = ACTIONS(1589),
    [sym_data_type_string] = ACTIONS(1589),
    [sym_data_type_datetime] = ACTIONS(1589),
    [sym_data_type_date] = ACTIONS(1589),
    [sym_data_type_time] = ACTIONS(1589),
    [sym_data_type_image] = ACTIONS(1589),
    [sym_data_format_url] = ACTIONS(1589),
    [sym_data_format_phone] = ACTIONS(1589),
    [sym_data_format_email] = ACTIONS(1589),
    [sym_data_format_iso8601] = ACTIONS(1589),
    [sym_auditlog_priv] = ACTIONS(1589),
    [sym_configuration_priv] = ACTIONS(1589),
    [sym_deviceconfig_priv] = ACTIONS(1589),
    [sym_ftp_priv] = ACTIONS(1589),
    [sym_http_priv] = ACTIONS(1589),
    [sym_networkconfig_priv] = ACTIONS(1589),
    [sym_programport_priv] = ACTIONS(1589),
    [sym_remoteui_priv] = ACTIONS(1589),
    [sym_securitycontrol_priv] = ACTIONS(1589),
    [sym_softwaremanagement_priv] = ACTIONS(1589),
    [sym_terminal_priv] = ACTIONS(1589),
    [sym_tpadmin_priv] = ACTIONS(1589),
    [sym_user1_priv] = ACTIONS(1589),
    [sym_user2_priv] = ACTIONS(1589),
    [sym_user3_priv] = ACTIONS(1589),
    [sym_user4_priv] = ACTIONS(1589),
    [sym_usermanagement_priv] = ACTIONS(1589),
    [sym_valid_account] = ACTIONS(1589),
    [sym_err_name_invalid] = ACTIONS(1589),
    [sym_err_password_invalid] = ACTIONS(1589),
    [sym_err_invalid_account] = ACTIONS(1589),
    [sym_err_no_authorization] = ACTIONS(1589),
    [sym_err_invalid_parameter] = ACTIONS(1589),
    [sym_err_user_locked_out] = ACTIONS(1589),
    [sym_err_user_account_expired] = ACTIONS(1589),
    [sym_err_general_error] = ACTIONS(1589),
    [sym_err_authentication_error] = ACTIONS(1589),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1589),
    [sym_tls_validate_certificate] = ACTIONS(1589),
    [sym____reserved___] = ACTIONS(1589),
    [sym_date] = ACTIONS(1589),
    [sym_day] = ACTIONS(1589),
    [sym_time] = ACTIONS(1589),
    [sym_ldate] = ACTIONS(1589),
    [sym_push_device] = ACTIONS(1589),
    [sym_push_channel] = ACTIONS(1589),
    [sym_push_devchan] = ACTIONS(1589),
    [sym_release_device] = ACTIONS(1589),
    [sym_release_channel] = ACTIONS(1589),
    [sym_release_devchan] = ACTIONS(1589),
    [sym_master_slot] = ACTIONS(1589),
    [sym_get_pulse_time] = ACTIONS(1589),
    [sym_get_timer] = ACTIONS(1589),
    [sym_master_sn] = ACTIONS(1589),
    [sym_system_number] = ACTIONS(1589),
    [sym_button] = ACTIONS(1589),
    [sym_dv_channel] = ACTIONS(1589),
    [sym_data] = ACTIONS(1589),
    [sym_level] = ACTIONS(1589),
    [sym_timeline] = ACTIONS(1589),
    [sym_channel] = ACTIONS(1589),
    [sym_custom] = ACTIONS(1589),
    [sym_atoi] = ACTIONS(1589),
    [sym_atol] = ACTIONS(1589),
    [sym_atof] = ACTIONS(1589),
    [sym_itoa] = ACTIONS(1589),
    [sym_itohex] = ACTIONS(1589),
    [sym_hextoi] = ACTIONS(1589),
    [sym_ftoa] = ACTIONS(1589),
    [sym_find_string] = ACTIONS(1589),
    [sym_left_string] = ACTIONS(1589),
    [sym_right_string] = ACTIONS(1589),
    [sym_mid_string] = ACTIONS(1589),
    [sym_remove_string] = ACTIONS(1589),
    [sym_get_buffer_string] = ACTIONS(1589),
    [sym_get_buffer_char] = ACTIONS(1589),
    [sym_get_multi_buffer_string] = ACTIONS(1589),
    [sym_length_string] = ACTIONS(1589),
    [sym_max_length_string] = ACTIONS(1589),
    [sym_set_length_string] = ACTIONS(1589),
    [sym_compare_string] = ACTIONS(1589),
    [sym_upper_string] = ACTIONS(1589),
    [sym_lower_string] = ACTIONS(1589),
    [sym_get_last] = ACTIONS(1589),
    [sym_date_to_day] = ACTIONS(1589),
    [sym_date_to_month] = ACTIONS(1589),
    [sym_date_to_year] = ACTIONS(1589),
    [sym_time_to_hour] = ACTIONS(1589),
    [sym_time_to_minute] = ACTIONS(1589),
    [sym_time_to_second] = ACTIONS(1589),
    [sym_day_of_week] = ACTIONS(1589),
    [sym_set_pulse_time] = ACTIONS(1589),
    [sym_set_timer] = ACTIONS(1589),
    [sym_file_open] = ACTIONS(1589),
    [sym_file_close] = ACTIONS(1589),
    [sym_file_read] = ACTIONS(1589),
    [sym_file_read_line] = ACTIONS(1589),
    [sym_file_write] = ACTIONS(1589),
    [sym_file_write_line] = ACTIONS(1589),
    [sym_file_copy] = ACTIONS(1589),
    [sym_file_delete] = ACTIONS(1589),
    [sym_file_rename] = ACTIONS(1589),
    [sym_file_dir] = ACTIONS(1589),
    [sym_file_getdir] = ACTIONS(1589),
    [sym_file_setdir] = ACTIONS(1589),
    [sym_file_create_dir] = ACTIONS(1589),
    [sym_file_remove_dir] = ACTIONS(1589),
    [sym_file_seek] = ACTIONS(1589),
    [sym_length_array] = ACTIONS(1589),
    [sym_max_length_array] = ACTIONS(1589),
    [sym_set_length_array] = ACTIONS(1589),
    [sym_ip_client_open] = ACTIONS(1589),
    [sym_ip_client_close] = ACTIONS(1589),
    [sym_ip_server_open] = ACTIONS(1589),
    [sym_ip_server_close] = ACTIONS(1589),
    [sym_ip_bound_client_open] = ACTIONS(1589),
    [sym_ip_mc_server_open] = ACTIONS(1589),
    [sym_ip_set_option] = ACTIONS(1589),
    [sym_ssh_client_open] = ACTIONS(1589),
    [sym_ssh_client_close] = ACTIONS(1589),
    [sym_tls_client_open] = ACTIONS(1589),
    [sym_tls_client_close] = ACTIONS(1589),
    [sym_do_push] = ACTIONS(1589),
    [sym_do_push_timed] = ACTIONS(1589),
    [sym_do_release] = ACTIONS(1589),
    [sym_device_id] = ACTIONS(1589),
    [sym_device_info] = ACTIONS(1589),
    [sym_device_id_string] = ACTIONS(1589),
    [sym_device_standby] = ACTIONS(1589),
    [sym_device_wake] = ACTIONS(1589),
    [sym_timeline_create] = ACTIONS(1589),
    [sym_timeline_kill] = ACTIONS(1589),
    [sym_timeline_reload] = ACTIONS(1589),
    [sym_timeline_pause] = ACTIONS(1589),
    [sym_timeline_restart] = ACTIONS(1589),
    [sym_timeline_set] = ACTIONS(1589),
    [sym_timeline_get] = ACTIONS(1589),
    [sym_timeline_active] = ACTIONS(1589),
    [sym_timeline_dynamic_id] = ACTIONS(1589),
    [sym_variable_to_string] = ACTIONS(1589),
    [sym_string_to_variable] = ACTIONS(1589),
    [sym_length_variable_to_string] = ACTIONS(1589),
    [sym_type_cast] = ACTIONS(1589),
    [sym_variable_to_xml] = ACTIONS(1589),
    [sym_xml_to_variable] = ACTIONS(1589),
    [sym_length_variable_to_xml] = ACTIONS(1589),
    [sym_redirect_string] = ACTIONS(1589),
    [sym_random_number] = ACTIONS(1589),
    [sym_get_unique_id] = ACTIONS(1589),
    [sym_get_system_number] = ACTIONS(1589),
    [sym_set_system_number] = ACTIONS(1589),
    [sym_get_serial_number] = ACTIONS(1589),
    [sym_get_master_build] = ACTIONS(1589),
    [sym_set_validation_code] = ACTIONS(1589),
    [sym_reboot] = ACTIONS(1589),
    [sym_load_duet_module] = ACTIONS(1589),
    [sym_unload_duet_module] = ACTIONS(1589),
    [sym_duet_mem_size_set] = ACTIONS(1589),
    [sym_duet_mem_size_get] = ACTIONS(1589),
    [sym_dynamic_application_device] = ACTIONS(1589),
    [sym_dynamic_polled_port] = ACTIONS(1589),
    [sym_static_port_binding] = ACTIONS(1589),
    [sym_static_ip_binding] = ACTIONS(1589),
    [sym_get_dns_list] = ACTIONS(1589),
    [sym_set_dns_list] = ACTIONS(1589),
    [sym_get_ip_address] = ACTIONS(1589),
    [sym_set_ip_address] = ACTIONS(1589),
    [sym_get_url_list] = ACTIONS(1589),
    [sym_add_url_entry] = ACTIONS(1589),
    [sym_delete_url_entry] = ACTIONS(1589),
    [sym_abs_value] = ACTIONS(1589),
    [sym_max_value] = ACTIONS(1589),
    [sym_min_value] = ACTIONS(1589),
    [sym_exp_value] = ACTIONS(1589),
    [sym_log_value] = ACTIONS(1589),
    [sym_log10_value] = ACTIONS(1589),
    [sym_power_value] = ACTIONS(1589),
    [sym_sqrt_value] = ACTIONS(1589),
    [sym_raw_be] = ACTIONS(1589),
    [sym_raw_le] = ACTIONS(1589),
    [sym_internal_queue_size_set] = ACTIONS(1589),
    [sym_internal_queue_size_get] = ACTIONS(1589),
    [sym_internal_threshold_set] = ACTIONS(1589),
    [sym_internal_threshold_get] = ACTIONS(1589),
    [sym_rebuilt_event] = ACTIONS(1589),
    [sym_do_custom_event] = ACTIONS(1589),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1589),
    [sym_clkmgr_set_clk_source] = ACTIONS(1589),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1589),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1589),
    [sym_clkmgr_get_timezone] = ACTIONS(1589),
    [sym_clkmgr_set_timezone] = ACTIONS(1589),
    [sym_clkmgr_get_resync_period] = ACTIONS(1589),
    [sym_clkmgr_set_resync_period] = ACTIONS(1589),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1589),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1589),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1589),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1589),
    [sym_clkmgr_get_timeservers] = ACTIONS(1589),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1589),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1589),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1589),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1589),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1589),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1589),
    [sym_set_log_level] = ACTIONS(1589),
    [sym_get_log_level] = ACTIONS(1589),
    [sym_amx_log] = ACTIONS(1589),
    [sym_smtp_server_config_set] = ACTIONS(1589),
    [sym_smtp_server_config_get] = ACTIONS(1589),
    [sym_smtp_send] = ACTIONS(1589),
    [sym_validate_netlinx_account] = ACTIONS(1589),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1589),
    [sym_authenticate_certificate] = ACTIONS(1589),
    [sym_audit_netlinx_session_event] = ACTIONS(1589),
    [sym_audit_netlinx_generic_event] = ACTIONS(1589),
    [sym_get_available_flash_disk_space] = ACTIONS(1589),
    [sym_get_max_flash_disk_space] = ACTIONS(1589),
    [sym_data_create_feed] = ACTIONS(1589),
    [sym_data_delete_feed] = ACTIONS(1589),
    [sym_data_publish_feed] = ACTIONS(1589),
    [sym_data_get_published_feed] = ACTIONS(1589),
    [sym_data_add_record] = ACTIONS(1589),
    [sym_data_get_event_record] = ACTIONS(1589),
    [sym__wc_data_create_feed] = ACTIONS(1589),
    [sym__wc_data_add_record] = ACTIONS(1589),
    [sym__wc_data_get_event_record] = ACTIONS(1589),
    [sym_format] = ACTIONS(1589),
    [sym_astro_clock] = ACTIONS(1589),
    [sym_set_outdoor_temperature] = ACTIONS(1589),
    [sym_set_virtual_level_count] = ACTIONS(1589),
    [sym_set_virtual_channel_count] = ACTIONS(1589),
    [sym_set_virtual_port_count] = ACTIONS(1589),
    [sym__wc_encode_int] = ACTIONS(1589),
    [sym__wc_decode_int] = ACTIONS(1589),
    [sym__wc_int] = ACTIONS(1589),
    [sym__wc_to_ch_int] = ACTIONS(1589),
    [sym__ch_to_wc_int] = ACTIONS(1589),
    [sym__wc_find_string_int] = ACTIONS(1589),
    [sym__wc_left_string_int] = ACTIONS(1589),
    [sym__wc_lower_string_int] = ACTIONS(1589),
    [sym__wc_mid_string_int] = ACTIONS(1589),
    [sym__wc_remove_string_int] = ACTIONS(1589),
    [sym__wc_right_string_int] = ACTIONS(1589),
    [sym__wc_upper_string_int] = ACTIONS(1589),
    [sym__wc_compare_string_int] = ACTIONS(1589),
    [sym__wc_get_buffer_char_int] = ACTIONS(1589),
    [sym__wc_get_buffer_string_int] = ACTIONS(1589),
    [sym__wc_concat_string_int] = ACTIONS(1589),
    [sym__wc_file_open_int] = ACTIONS(1589),
    [sym__wc_file_close_int] = ACTIONS(1589),
    [sym__wc_file_read_int] = ACTIONS(1589),
    [sym__wc_file_read_line_int] = ACTIONS(1589),
    [sym__wc_file_write_int] = ACTIONS(1589),
    [sym__wc_file_write_line_int] = ACTIONS(1589),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1591),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1591),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1591),
    [aux_sym_preproc_else_token1] = ACTIONS(1591),
    [anon_sym_COMMA] = ACTIONS(1591),
    [anon_sym_LBRACK] = ACTIONS(1591),
    [anon_sym_LBRACE] = ACTIONS(1591),
    [anon_sym_RBRACE] = ACTIONS(1591),
    [anon_sym_COLON] = ACTIONS(1591),
    [anon_sym_LPAREN] = ACTIONS(1589),
    [anon_sym_BANG] = ACTIONS(1589),
    [anon_sym_TILDE] = ACTIONS(1591),
    [anon_sym_DASH] = ACTIONS(1589),
    [anon_sym_PLUS] = ACTIONS(1589),
    [anon_sym_STAR] = ACTIONS(1591),
    [anon_sym_SLASH] = ACTIONS(1589),
    [anon_sym_PERCENT] = ACTIONS(1591),
    [anon_sym_PIPE_PIPE] = ACTIONS(1591),
    [anon_sym_AMP_AMP] = ACTIONS(1591),
    [anon_sym_PIPE] = ACTIONS(1589),
    [anon_sym_CARET] = ACTIONS(1591),
    [anon_sym_AMP] = ACTIONS(1589),
    [anon_sym_EQ_EQ] = ACTIONS(1591),
    [anon_sym_BANG_EQ] = ACTIONS(1591),
    [anon_sym_LT_GT] = ACTIONS(1591),
    [anon_sym_GT] = ACTIONS(1589),
    [anon_sym_GT_EQ] = ACTIONS(1591),
    [anon_sym_LT_EQ] = ACTIONS(1591),
    [anon_sym_LT] = ACTIONS(1589),
    [anon_sym_LT_LT] = ACTIONS(1591),
    [anon_sym_GT_GT] = ACTIONS(1591),
    [anon_sym_DASH_DASH] = ACTIONS(1591),
    [anon_sym_PLUS_PLUS] = ACTIONS(1591),
    [anon_sym_DOT] = ACTIONS(1589),
    [anon_sym_DQUOTE] = ACTIONS(1591),
    [anon_sym_SQUOTE] = ACTIONS(1591),
    [sym_number_literal] = ACTIONS(1591),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(616)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2217),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2217),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2217),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2229),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2217),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(617)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2217),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2217),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2217),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2232),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2217),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(618)] = {
    [sym_identifier] = ACTIONS(1465),
    [sym_and] = ACTIONS(1465),
    [sym_or] = ACTIONS(1465),
    [sym_not] = ACTIONS(1465),
    [sym_xor] = ACTIONS(1465),
    [sym_band] = ACTIONS(1465),
    [sym_bor] = ACTIONS(1465),
    [sym_bxor] = ACTIONS(1465),
    [sym_bnot] = ACTIONS(1465),
    [sym_lshift] = ACTIONS(1465),
    [sym_rshift] = ACTIONS(1465),
    [sym___file__] = ACTIONS(1465),
    [sym___date__] = ACTIONS(1465),
    [sym___ldate__] = ACTIONS(1465),
    [sym___time__] = ACTIONS(1465),
    [sym___line__] = ACTIONS(1465),
    [sym___name__] = ACTIONS(1465),
    [sym___netlinx__] = ACTIONS(1465),
    [sym_netlinx_axi_version] = ACTIONS(1465),
    [sym_url_flg_tcp] = ACTIONS(1465),
    [sym_url_flg_accinfopresent] = ACTIONS(1465),
    [sym_url_flg_temp] = ACTIONS(1465),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1465),
    [sym_url_flg_stat_mask] = ACTIONS(1465),
    [sym_url_flg_stat_lookup] = ACTIONS(1465),
    [sym_url_flg_stat_connecting] = ACTIONS(1465),
    [sym_url_flg_stat_waiting] = ACTIONS(1465),
    [sym_url_flg_stat_connected] = ACTIONS(1465),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1465),
    [sym_first_virtual_device] = ACTIONS(1465),
    [sym_first_local_port] = ACTIONS(1465),
    [sym_dynamic_virtual_device] = ACTIONS(1465),
    [sym_timeline_once] = ACTIONS(1465),
    [sym_timeline_repeat] = ACTIONS(1465),
    [sym_timeline_absolute] = ACTIONS(1465),
    [sym_timeline_relative] = ACTIONS(1465),
    [sym_do_push_timed_infinite] = ACTIONS(1465),
    [sym_source_type_no_address] = ACTIONS(1465),
    [sym_source_type_neuron_id] = ACTIONS(1465),
    [sym_source_type_ip_address] = ACTIONS(1465),
    [sym_source_type_axlink] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1465),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1465),
    [sym_source_type_ip_socket_address] = ACTIONS(1465),
    [sym_source_type_rs232] = ACTIONS(1465),
    [sym_source_type_internal] = ACTIONS(1465),
    [sym_source_type_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1465),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1465),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1465),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1465),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1465),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1465),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1465),
    [sym_true] = ACTIONS(1465),
    [sym_false] = ACTIONS(1465),
    [sym_file_read_only] = ACTIONS(1465),
    [sym_file_rw_new] = ACTIONS(1465),
    [sym_file_rw_append] = ACTIONS(1465),
    [sym_ip_tcp] = ACTIONS(1465),
    [sym_ip_udp] = ACTIONS(1465),
    [sym_ip_udp_2way] = ACTIONS(1465),
    [sym_xml_encode_types] = ACTIONS(1465),
    [sym_xml_encode_char_as_list] = ACTIONS(1465),
    [sym_xml_encode_le] = ACTIONS(1465),
    [sym_xml_decode_types] = ACTIONS(1465),
    [sym_xml_decode_no_preserve] = ACTIONS(1465),
    [sym_duet_dev_type_display_device] = ACTIONS(1465),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1465),
    [sym_duet_dev_type_io_device] = ACTIONS(1465),
    [sym_duet_dev_type_relay_device] = ACTIONS(1465),
    [sym_duet_dev_type_ups] = ACTIONS(1465),
    [sym_duet_dev_type_amplifier] = ACTIONS(1465),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1465),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1465),
    [sym_duet_dev_type_camera] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1465),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1465),
    [sym_duet_dev_type_dss] = ACTIONS(1465),
    [sym_duet_dev_type_dvr] = ACTIONS(1465),
    [sym_duet_dev_type_disc_device] = ACTIONS(1465),
    [sym_duet_dev_type_document_camera] = ACTIONS(1465),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1465),
    [sym_duet_dev_type_hvac] = ACTIONS(1465),
    [sym_duet_dev_type_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_light] = ACTIONS(1465),
    [sym_duet_dev_type_monitor] = ACTIONS(1465),
    [sym_duet_dev_type_motor] = ACTIONS(1465),
    [sym_duet_dev_type_multi_window] = ACTIONS(1465),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1465),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1465),
    [sym_duet_dev_type_receiver] = ACTIONS(1465),
    [sym_duet_dev_type_security_system] = ACTIONS(1465),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1465),
    [sym_duet_dev_type_settop_box] = ACTIONS(1465),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1465),
    [sym_duet_dev_type_switcher] = ACTIONS(1465),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1465),
    [sym_duet_dev_type_tv] = ACTIONS(1465),
    [sym_duet_dev_type_utility] = ACTIONS(1465),
    [sym_duet_dev_type_vcr] = ACTIONS(1465),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1465),
    [sym_duet_dev_type_video_processor] = ACTIONS(1465),
    [sym_duet_dev_type_video_projector] = ACTIONS(1465),
    [sym_duet_dev_type_video_wall] = ACTIONS(1465),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1465),
    [sym_duet_dev_type_weather] = ACTIONS(1465),
    [sym_duet_dev_not_polled] = ACTIONS(1465),
    [sym_duet_dev_polled] = ACTIONS(1465),
    [sym_clkmgr_mode_network] = ACTIONS(1465),
    [sym_clkmgr_mode_standalone] = ACTIONS(1465),
    [sym_normal_standby] = ACTIONS(1465),
    [sym_normal_wake] = ACTIONS(1465),
    [sym_ip_multicast_ttl_option] = ACTIONS(1465),
    [sym_ip_ttl_subnet] = ACTIONS(1465),
    [sym_ip_ttl_site] = ACTIONS(1465),
    [sym_ip_ttl_region] = ACTIONS(1465),
    [sym_ip_ttl_continent] = ACTIONS(1465),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1465),
    [sym_ip_nodelay_on] = ACTIONS(1465),
    [sym_ip_nodelay_off] = ACTIONS(1465),
    [sym_amx_error] = ACTIONS(1465),
    [sym_amx_warning] = ACTIONS(1465),
    [sym_amx_info] = ACTIONS(1465),
    [sym_amx_debug] = ACTIONS(1465),
    [sym_netlinx_login_success] = ACTIONS(1465),
    [sym_netlinx_login_fail] = ACTIONS(1465),
    [sym_netlinx_logout] = ACTIONS(1465),
    [sym_null_str] = ACTIONS(1465),
    [sym_smtp_address] = ACTIONS(1465),
    [sym_smtp_port_number] = ACTIONS(1465),
    [sym_smtp_username] = ACTIONS(1465),
    [sym_smtp_password] = ACTIONS(1465),
    [sym_smtp_require_tls] = ACTIONS(1465),
    [sym_smtp_from] = ACTIONS(1465),
    [sym_smtp_tls_true] = ACTIONS(1465),
    [sym_smtp_tls_false] = ACTIONS(1465),
    [sym_listview_on_row_select_event] = ACTIONS(1465),
    [sym_data_structure_unknown] = ACTIONS(1465),
    [sym_data_structure_datafeed] = ACTIONS(1465),
    [sym_data_structure_datarecordset] = ACTIONS(1465),
    [sym_data_structure_datarecord] = ACTIONS(1465),
    [sym_data_structure_datafield] = ACTIONS(1465),
    [sym_data_max_content_field_count] = ACTIONS(1465),
    [sym_data_max_metadata_field_count] = ACTIONS(1465),
    [sym_data_max_id_length] = ACTIONS(1465),
    [sym_data_max_name_length] = ACTIONS(1465),
    [sym_data_max_description_length] = ACTIONS(1465),
    [sym_data_max_type_length] = ACTIONS(1465),
    [sym_data_max_format_length] = ACTIONS(1465),
    [sym_data_max_source_length] = ACTIONS(1465),
    [sym_data_max_value_length] = ACTIONS(1465),
    [sym_data_max_label_length] = ACTIONS(1465),
    [sym_data_max_url_length] = ACTIONS(1465),
    [sym_max_last_login_info_length] = ACTIONS(1465),
    [sym_data_type_unknown] = ACTIONS(1465),
    [sym_data_type_string] = ACTIONS(1465),
    [sym_data_type_datetime] = ACTIONS(1465),
    [sym_data_type_date] = ACTIONS(1465),
    [sym_data_type_time] = ACTIONS(1465),
    [sym_data_type_image] = ACTIONS(1465),
    [sym_data_format_url] = ACTIONS(1465),
    [sym_data_format_phone] = ACTIONS(1465),
    [sym_data_format_email] = ACTIONS(1465),
    [sym_data_format_iso8601] = ACTIONS(1465),
    [sym_auditlog_priv] = ACTIONS(1465),
    [sym_configuration_priv] = ACTIONS(1465),
    [sym_deviceconfig_priv] = ACTIONS(1465),
    [sym_ftp_priv] = ACTIONS(1465),
    [sym_http_priv] = ACTIONS(1465),
    [sym_networkconfig_priv] = ACTIONS(1465),
    [sym_programport_priv] = ACTIONS(1465),
    [sym_remoteui_priv] = ACTIONS(1465),
    [sym_securitycontrol_priv] = ACTIONS(1465),
    [sym_softwaremanagement_priv] = ACTIONS(1465),
    [sym_terminal_priv] = ACTIONS(1465),
    [sym_tpadmin_priv] = ACTIONS(1465),
    [sym_user1_priv] = ACTIONS(1465),
    [sym_user2_priv] = ACTIONS(1465),
    [sym_user3_priv] = ACTIONS(1465),
    [sym_user4_priv] = ACTIONS(1465),
    [sym_usermanagement_priv] = ACTIONS(1465),
    [sym_valid_account] = ACTIONS(1465),
    [sym_err_name_invalid] = ACTIONS(1465),
    [sym_err_password_invalid] = ACTIONS(1465),
    [sym_err_invalid_account] = ACTIONS(1465),
    [sym_err_no_authorization] = ACTIONS(1465),
    [sym_err_invalid_parameter] = ACTIONS(1465),
    [sym_err_user_locked_out] = ACTIONS(1465),
    [sym_err_user_account_expired] = ACTIONS(1465),
    [sym_err_general_error] = ACTIONS(1465),
    [sym_err_authentication_error] = ACTIONS(1465),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1465),
    [sym_tls_validate_certificate] = ACTIONS(1465),
    [sym____reserved___] = ACTIONS(1465),
    [sym_date] = ACTIONS(1465),
    [sym_day] = ACTIONS(1465),
    [sym_time] = ACTIONS(1465),
    [sym_ldate] = ACTIONS(1465),
    [sym_push_device] = ACTIONS(1465),
    [sym_push_channel] = ACTIONS(1465),
    [sym_push_devchan] = ACTIONS(1465),
    [sym_release_device] = ACTIONS(1465),
    [sym_release_channel] = ACTIONS(1465),
    [sym_release_devchan] = ACTIONS(1465),
    [sym_master_slot] = ACTIONS(1465),
    [sym_get_pulse_time] = ACTIONS(1465),
    [sym_get_timer] = ACTIONS(1465),
    [sym_master_sn] = ACTIONS(1465),
    [sym_system_number] = ACTIONS(1465),
    [sym_button] = ACTIONS(1465),
    [sym_dv_channel] = ACTIONS(1465),
    [sym_data] = ACTIONS(1465),
    [sym_level] = ACTIONS(1465),
    [sym_timeline] = ACTIONS(1465),
    [sym_channel] = ACTIONS(1465),
    [sym_custom] = ACTIONS(1465),
    [sym_atoi] = ACTIONS(1465),
    [sym_atol] = ACTIONS(1465),
    [sym_atof] = ACTIONS(1465),
    [sym_itoa] = ACTIONS(1465),
    [sym_itohex] = ACTIONS(1465),
    [sym_hextoi] = ACTIONS(1465),
    [sym_ftoa] = ACTIONS(1465),
    [sym_find_string] = ACTIONS(1465),
    [sym_left_string] = ACTIONS(1465),
    [sym_right_string] = ACTIONS(1465),
    [sym_mid_string] = ACTIONS(1465),
    [sym_remove_string] = ACTIONS(1465),
    [sym_get_buffer_string] = ACTIONS(1465),
    [sym_get_buffer_char] = ACTIONS(1465),
    [sym_get_multi_buffer_string] = ACTIONS(1465),
    [sym_length_string] = ACTIONS(1465),
    [sym_max_length_string] = ACTIONS(1465),
    [sym_set_length_string] = ACTIONS(1465),
    [sym_compare_string] = ACTIONS(1465),
    [sym_upper_string] = ACTIONS(1465),
    [sym_lower_string] = ACTIONS(1465),
    [sym_get_last] = ACTIONS(1465),
    [sym_date_to_day] = ACTIONS(1465),
    [sym_date_to_month] = ACTIONS(1465),
    [sym_date_to_year] = ACTIONS(1465),
    [sym_time_to_hour] = ACTIONS(1465),
    [sym_time_to_minute] = ACTIONS(1465),
    [sym_time_to_second] = ACTIONS(1465),
    [sym_day_of_week] = ACTIONS(1465),
    [sym_set_pulse_time] = ACTIONS(1465),
    [sym_set_timer] = ACTIONS(1465),
    [sym_file_open] = ACTIONS(1465),
    [sym_file_close] = ACTIONS(1465),
    [sym_file_read] = ACTIONS(1465),
    [sym_file_read_line] = ACTIONS(1465),
    [sym_file_write] = ACTIONS(1465),
    [sym_file_write_line] = ACTIONS(1465),
    [sym_file_copy] = ACTIONS(1465),
    [sym_file_delete] = ACTIONS(1465),
    [sym_file_rename] = ACTIONS(1465),
    [sym_file_dir] = ACTIONS(1465),
    [sym_file_getdir] = ACTIONS(1465),
    [sym_file_setdir] = ACTIONS(1465),
    [sym_file_create_dir] = ACTIONS(1465),
    [sym_file_remove_dir] = ACTIONS(1465),
    [sym_file_seek] = ACTIONS(1465),
    [sym_length_array] = ACTIONS(1465),
    [sym_max_length_array] = ACTIONS(1465),
    [sym_set_length_array] = ACTIONS(1465),
    [sym_ip_client_open] = ACTIONS(1465),
    [sym_ip_client_close] = ACTIONS(1465),
    [sym_ip_server_open] = ACTIONS(1465),
    [sym_ip_server_close] = ACTIONS(1465),
    [sym_ip_bound_client_open] = ACTIONS(1465),
    [sym_ip_mc_server_open] = ACTIONS(1465),
    [sym_ip_set_option] = ACTIONS(1465),
    [sym_ssh_client_open] = ACTIONS(1465),
    [sym_ssh_client_close] = ACTIONS(1465),
    [sym_tls_client_open] = ACTIONS(1465),
    [sym_tls_client_close] = ACTIONS(1465),
    [sym_do_push] = ACTIONS(1465),
    [sym_do_push_timed] = ACTIONS(1465),
    [sym_do_release] = ACTIONS(1465),
    [sym_device_id] = ACTIONS(1465),
    [sym_device_info] = ACTIONS(1465),
    [sym_device_id_string] = ACTIONS(1465),
    [sym_device_standby] = ACTIONS(1465),
    [sym_device_wake] = ACTIONS(1465),
    [sym_timeline_create] = ACTIONS(1465),
    [sym_timeline_kill] = ACTIONS(1465),
    [sym_timeline_reload] = ACTIONS(1465),
    [sym_timeline_pause] = ACTIONS(1465),
    [sym_timeline_restart] = ACTIONS(1465),
    [sym_timeline_set] = ACTIONS(1465),
    [sym_timeline_get] = ACTIONS(1465),
    [sym_timeline_active] = ACTIONS(1465),
    [sym_timeline_dynamic_id] = ACTIONS(1465),
    [sym_variable_to_string] = ACTIONS(1465),
    [sym_string_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_string] = ACTIONS(1465),
    [sym_type_cast] = ACTIONS(1465),
    [sym_variable_to_xml] = ACTIONS(1465),
    [sym_xml_to_variable] = ACTIONS(1465),
    [sym_length_variable_to_xml] = ACTIONS(1465),
    [sym_redirect_string] = ACTIONS(1465),
    [sym_random_number] = ACTIONS(1465),
    [sym_get_unique_id] = ACTIONS(1465),
    [sym_get_system_number] = ACTIONS(1465),
    [sym_set_system_number] = ACTIONS(1465),
    [sym_get_serial_number] = ACTIONS(1465),
    [sym_get_master_build] = ACTIONS(1465),
    [sym_set_validation_code] = ACTIONS(1465),
    [sym_reboot] = ACTIONS(1465),
    [sym_load_duet_module] = ACTIONS(1465),
    [sym_unload_duet_module] = ACTIONS(1465),
    [sym_duet_mem_size_set] = ACTIONS(1465),
    [sym_duet_mem_size_get] = ACTIONS(1465),
    [sym_dynamic_application_device] = ACTIONS(1465),
    [sym_dynamic_polled_port] = ACTIONS(1465),
    [sym_static_port_binding] = ACTIONS(1465),
    [sym_static_ip_binding] = ACTIONS(1465),
    [sym_get_dns_list] = ACTIONS(1465),
    [sym_set_dns_list] = ACTIONS(1465),
    [sym_get_ip_address] = ACTIONS(1465),
    [sym_set_ip_address] = ACTIONS(1465),
    [sym_get_url_list] = ACTIONS(1465),
    [sym_add_url_entry] = ACTIONS(1465),
    [sym_delete_url_entry] = ACTIONS(1465),
    [sym_abs_value] = ACTIONS(1465),
    [sym_max_value] = ACTIONS(1465),
    [sym_min_value] = ACTIONS(1465),
    [sym_exp_value] = ACTIONS(1465),
    [sym_log_value] = ACTIONS(1465),
    [sym_log10_value] = ACTIONS(1465),
    [sym_power_value] = ACTIONS(1465),
    [sym_sqrt_value] = ACTIONS(1465),
    [sym_raw_be] = ACTIONS(1465),
    [sym_raw_le] = ACTIONS(1465),
    [sym_internal_queue_size_set] = ACTIONS(1465),
    [sym_internal_queue_size_get] = ACTIONS(1465),
    [sym_internal_threshold_set] = ACTIONS(1465),
    [sym_internal_threshold_get] = ACTIONS(1465),
    [sym_rebuilt_event] = ACTIONS(1465),
    [sym_do_custom_event] = ACTIONS(1465),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1465),
    [sym_clkmgr_set_clk_source] = ACTIONS(1465),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1465),
    [sym_clkmgr_get_timezone] = ACTIONS(1465),
    [sym_clkmgr_set_timezone] = ACTIONS(1465),
    [sym_clkmgr_get_resync_period] = ACTIONS(1465),
    [sym_clkmgr_set_resync_period] = ACTIONS(1465),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1465),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_timeservers] = ACTIONS(1465),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1465),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1465),
    [sym_set_log_level] = ACTIONS(1465),
    [sym_get_log_level] = ACTIONS(1465),
    [sym_amx_log] = ACTIONS(1465),
    [sym_smtp_server_config_set] = ACTIONS(1465),
    [sym_smtp_server_config_get] = ACTIONS(1465),
    [sym_smtp_send] = ACTIONS(1465),
    [sym_validate_netlinx_account] = ACTIONS(1465),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1465),
    [sym_authenticate_certificate] = ACTIONS(1465),
    [sym_audit_netlinx_session_event] = ACTIONS(1465),
    [sym_audit_netlinx_generic_event] = ACTIONS(1465),
    [sym_get_available_flash_disk_space] = ACTIONS(1465),
    [sym_get_max_flash_disk_space] = ACTIONS(1465),
    [sym_data_create_feed] = ACTIONS(1465),
    [sym_data_delete_feed] = ACTIONS(1465),
    [sym_data_publish_feed] = ACTIONS(1465),
    [sym_data_get_published_feed] = ACTIONS(1465),
    [sym_data_add_record] = ACTIONS(1465),
    [sym_data_get_event_record] = ACTIONS(1465),
    [sym__wc_data_create_feed] = ACTIONS(1465),
    [sym__wc_data_add_record] = ACTIONS(1465),
    [sym__wc_data_get_event_record] = ACTIONS(1465),
    [sym_format] = ACTIONS(1465),
    [sym_astro_clock] = ACTIONS(1465),
    [sym_set_outdoor_temperature] = ACTIONS(1465),
    [sym_set_virtual_level_count] = ACTIONS(1465),
    [sym_set_virtual_channel_count] = ACTIONS(1465),
    [sym_set_virtual_port_count] = ACTIONS(1465),
    [sym__wc_encode_int] = ACTIONS(1465),
    [sym__wc_decode_int] = ACTIONS(1465),
    [sym__wc_int] = ACTIONS(1465),
    [sym__wc_to_ch_int] = ACTIONS(1465),
    [sym__ch_to_wc_int] = ACTIONS(1465),
    [sym__wc_find_string_int] = ACTIONS(1465),
    [sym__wc_left_string_int] = ACTIONS(1465),
    [sym__wc_lower_string_int] = ACTIONS(1465),
    [sym__wc_mid_string_int] = ACTIONS(1465),
    [sym__wc_remove_string_int] = ACTIONS(1465),
    [sym__wc_right_string_int] = ACTIONS(1465),
    [sym__wc_upper_string_int] = ACTIONS(1465),
    [sym__wc_compare_string_int] = ACTIONS(1465),
    [sym__wc_get_buffer_char_int] = ACTIONS(1465),
    [sym__wc_get_buffer_string_int] = ACTIONS(1465),
    [sym__wc_concat_string_int] = ACTIONS(1465),
    [sym__wc_file_open_int] = ACTIONS(1465),
    [sym__wc_file_close_int] = ACTIONS(1465),
    [sym__wc_file_read_int] = ACTIONS(1465),
    [sym__wc_file_read_line_int] = ACTIONS(1465),
    [sym__wc_file_write_int] = ACTIONS(1465),
    [sym__wc_file_write_line_int] = ACTIONS(1465),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1469),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1469),
    [aux_sym_preproc_else_token1] = ACTIONS(1469),
    [anon_sym_COMMA] = ACTIONS(1469),
    [anon_sym_LBRACK] = ACTIONS(1469),
    [anon_sym_LBRACE] = ACTIONS(1469),
    [anon_sym_RBRACE] = ACTIONS(1469),
    [anon_sym_COLON] = ACTIONS(1469),
    [anon_sym_LPAREN] = ACTIONS(1465),
    [anon_sym_BANG] = ACTIONS(1465),
    [anon_sym_TILDE] = ACTIONS(1469),
    [anon_sym_DASH] = ACTIONS(1465),
    [anon_sym_PLUS] = ACTIONS(1465),
    [anon_sym_STAR] = ACTIONS(1469),
    [anon_sym_SLASH] = ACTIONS(1465),
    [anon_sym_PERCENT] = ACTIONS(1469),
    [anon_sym_PIPE_PIPE] = ACTIONS(1469),
    [anon_sym_AMP_AMP] = ACTIONS(1469),
    [anon_sym_PIPE] = ACTIONS(1465),
    [anon_sym_CARET] = ACTIONS(1469),
    [anon_sym_AMP] = ACTIONS(1465),
    [anon_sym_EQ_EQ] = ACTIONS(1469),
    [anon_sym_BANG_EQ] = ACTIONS(1469),
    [anon_sym_LT_GT] = ACTIONS(1469),
    [anon_sym_GT] = ACTIONS(1465),
    [anon_sym_GT_EQ] = ACTIONS(1469),
    [anon_sym_LT_EQ] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(1465),
    [anon_sym_LT_LT] = ACTIONS(1469),
    [anon_sym_GT_GT] = ACTIONS(1469),
    [anon_sym_DASH_DASH] = ACTIONS(1469),
    [anon_sym_PLUS_PLUS] = ACTIONS(1469),
    [anon_sym_DOT] = ACTIONS(1465),
    [anon_sym_DQUOTE] = ACTIONS(1469),
    [anon_sym_SQUOTE] = ACTIONS(1469),
    [sym_number_literal] = ACTIONS(1469),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(619)] = {
    [sym_identifier] = ACTIONS(1503),
    [sym_and] = ACTIONS(1503),
    [sym_or] = ACTIONS(1503),
    [sym_not] = ACTIONS(1503),
    [sym_xor] = ACTIONS(1503),
    [sym_band] = ACTIONS(1503),
    [sym_bor] = ACTIONS(1503),
    [sym_bxor] = ACTIONS(1503),
    [sym_bnot] = ACTIONS(1503),
    [sym_lshift] = ACTIONS(1503),
    [sym_rshift] = ACTIONS(1503),
    [sym___file__] = ACTIONS(1503),
    [sym___date__] = ACTIONS(1503),
    [sym___ldate__] = ACTIONS(1503),
    [sym___time__] = ACTIONS(1503),
    [sym___line__] = ACTIONS(1503),
    [sym___name__] = ACTIONS(1503),
    [sym___netlinx__] = ACTIONS(1503),
    [sym_netlinx_axi_version] = ACTIONS(1503),
    [sym_url_flg_tcp] = ACTIONS(1503),
    [sym_url_flg_accinfopresent] = ACTIONS(1503),
    [sym_url_flg_temp] = ACTIONS(1503),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1503),
    [sym_url_flg_stat_mask] = ACTIONS(1503),
    [sym_url_flg_stat_lookup] = ACTIONS(1503),
    [sym_url_flg_stat_connecting] = ACTIONS(1503),
    [sym_url_flg_stat_waiting] = ACTIONS(1503),
    [sym_url_flg_stat_connected] = ACTIONS(1503),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1503),
    [sym_first_virtual_device] = ACTIONS(1503),
    [sym_first_local_port] = ACTIONS(1503),
    [sym_dynamic_virtual_device] = ACTIONS(1503),
    [sym_timeline_once] = ACTIONS(1503),
    [sym_timeline_repeat] = ACTIONS(1503),
    [sym_timeline_absolute] = ACTIONS(1503),
    [sym_timeline_relative] = ACTIONS(1503),
    [sym_do_push_timed_infinite] = ACTIONS(1503),
    [sym_source_type_no_address] = ACTIONS(1503),
    [sym_source_type_neuron_id] = ACTIONS(1503),
    [sym_source_type_ip_address] = ACTIONS(1503),
    [sym_source_type_axlink] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1503),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1503),
    [sym_source_type_ip_socket_address] = ACTIONS(1503),
    [sym_source_type_rs232] = ACTIONS(1503),
    [sym_source_type_internal] = ACTIONS(1503),
    [sym_source_type_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1503),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1503),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1503),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1503),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1503),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1503),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1503),
    [sym_true] = ACTIONS(1503),
    [sym_false] = ACTIONS(1503),
    [sym_file_read_only] = ACTIONS(1503),
    [sym_file_rw_new] = ACTIONS(1503),
    [sym_file_rw_append] = ACTIONS(1503),
    [sym_ip_tcp] = ACTIONS(1503),
    [sym_ip_udp] = ACTIONS(1503),
    [sym_ip_udp_2way] = ACTIONS(1503),
    [sym_xml_encode_types] = ACTIONS(1503),
    [sym_xml_encode_char_as_list] = ACTIONS(1503),
    [sym_xml_encode_le] = ACTIONS(1503),
    [sym_xml_decode_types] = ACTIONS(1503),
    [sym_xml_decode_no_preserve] = ACTIONS(1503),
    [sym_duet_dev_type_display_device] = ACTIONS(1503),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1503),
    [sym_duet_dev_type_io_device] = ACTIONS(1503),
    [sym_duet_dev_type_relay_device] = ACTIONS(1503),
    [sym_duet_dev_type_ups] = ACTIONS(1503),
    [sym_duet_dev_type_amplifier] = ACTIONS(1503),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1503),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1503),
    [sym_duet_dev_type_camera] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1503),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1503),
    [sym_duet_dev_type_dss] = ACTIONS(1503),
    [sym_duet_dev_type_dvr] = ACTIONS(1503),
    [sym_duet_dev_type_disc_device] = ACTIONS(1503),
    [sym_duet_dev_type_document_camera] = ACTIONS(1503),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1503),
    [sym_duet_dev_type_hvac] = ACTIONS(1503),
    [sym_duet_dev_type_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_light] = ACTIONS(1503),
    [sym_duet_dev_type_monitor] = ACTIONS(1503),
    [sym_duet_dev_type_motor] = ACTIONS(1503),
    [sym_duet_dev_type_multi_window] = ACTIONS(1503),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1503),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1503),
    [sym_duet_dev_type_receiver] = ACTIONS(1503),
    [sym_duet_dev_type_security_system] = ACTIONS(1503),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1503),
    [sym_duet_dev_type_settop_box] = ACTIONS(1503),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1503),
    [sym_duet_dev_type_switcher] = ACTIONS(1503),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1503),
    [sym_duet_dev_type_tv] = ACTIONS(1503),
    [sym_duet_dev_type_utility] = ACTIONS(1503),
    [sym_duet_dev_type_vcr] = ACTIONS(1503),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1503),
    [sym_duet_dev_type_video_processor] = ACTIONS(1503),
    [sym_duet_dev_type_video_projector] = ACTIONS(1503),
    [sym_duet_dev_type_video_wall] = ACTIONS(1503),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1503),
    [sym_duet_dev_type_weather] = ACTIONS(1503),
    [sym_duet_dev_not_polled] = ACTIONS(1503),
    [sym_duet_dev_polled] = ACTIONS(1503),
    [sym_clkmgr_mode_network] = ACTIONS(1503),
    [sym_clkmgr_mode_standalone] = ACTIONS(1503),
    [sym_normal_standby] = ACTIONS(1503),
    [sym_normal_wake] = ACTIONS(1503),
    [sym_ip_multicast_ttl_option] = ACTIONS(1503),
    [sym_ip_ttl_subnet] = ACTIONS(1503),
    [sym_ip_ttl_site] = ACTIONS(1503),
    [sym_ip_ttl_region] = ACTIONS(1503),
    [sym_ip_ttl_continent] = ACTIONS(1503),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1503),
    [sym_ip_nodelay_on] = ACTIONS(1503),
    [sym_ip_nodelay_off] = ACTIONS(1503),
    [sym_amx_error] = ACTIONS(1503),
    [sym_amx_warning] = ACTIONS(1503),
    [sym_amx_info] = ACTIONS(1503),
    [sym_amx_debug] = ACTIONS(1503),
    [sym_netlinx_login_success] = ACTIONS(1503),
    [sym_netlinx_login_fail] = ACTIONS(1503),
    [sym_netlinx_logout] = ACTIONS(1503),
    [sym_null_str] = ACTIONS(1503),
    [sym_smtp_address] = ACTIONS(1503),
    [sym_smtp_port_number] = ACTIONS(1503),
    [sym_smtp_username] = ACTIONS(1503),
    [sym_smtp_password] = ACTIONS(1503),
    [sym_smtp_require_tls] = ACTIONS(1503),
    [sym_smtp_from] = ACTIONS(1503),
    [sym_smtp_tls_true] = ACTIONS(1503),
    [sym_smtp_tls_false] = ACTIONS(1503),
    [sym_listview_on_row_select_event] = ACTIONS(1503),
    [sym_data_structure_unknown] = ACTIONS(1503),
    [sym_data_structure_datafeed] = ACTIONS(1503),
    [sym_data_structure_datarecordset] = ACTIONS(1503),
    [sym_data_structure_datarecord] = ACTIONS(1503),
    [sym_data_structure_datafield] = ACTIONS(1503),
    [sym_data_max_content_field_count] = ACTIONS(1503),
    [sym_data_max_metadata_field_count] = ACTIONS(1503),
    [sym_data_max_id_length] = ACTIONS(1503),
    [sym_data_max_name_length] = ACTIONS(1503),
    [sym_data_max_description_length] = ACTIONS(1503),
    [sym_data_max_type_length] = ACTIONS(1503),
    [sym_data_max_format_length] = ACTIONS(1503),
    [sym_data_max_source_length] = ACTIONS(1503),
    [sym_data_max_value_length] = ACTIONS(1503),
    [sym_data_max_label_length] = ACTIONS(1503),
    [sym_data_max_url_length] = ACTIONS(1503),
    [sym_max_last_login_info_length] = ACTIONS(1503),
    [sym_data_type_unknown] = ACTIONS(1503),
    [sym_data_type_string] = ACTIONS(1503),
    [sym_data_type_datetime] = ACTIONS(1503),
    [sym_data_type_date] = ACTIONS(1503),
    [sym_data_type_time] = ACTIONS(1503),
    [sym_data_type_image] = ACTIONS(1503),
    [sym_data_format_url] = ACTIONS(1503),
    [sym_data_format_phone] = ACTIONS(1503),
    [sym_data_format_email] = ACTIONS(1503),
    [sym_data_format_iso8601] = ACTIONS(1503),
    [sym_auditlog_priv] = ACTIONS(1503),
    [sym_configuration_priv] = ACTIONS(1503),
    [sym_deviceconfig_priv] = ACTIONS(1503),
    [sym_ftp_priv] = ACTIONS(1503),
    [sym_http_priv] = ACTIONS(1503),
    [sym_networkconfig_priv] = ACTIONS(1503),
    [sym_programport_priv] = ACTIONS(1503),
    [sym_remoteui_priv] = ACTIONS(1503),
    [sym_securitycontrol_priv] = ACTIONS(1503),
    [sym_softwaremanagement_priv] = ACTIONS(1503),
    [sym_terminal_priv] = ACTIONS(1503),
    [sym_tpadmin_priv] = ACTIONS(1503),
    [sym_user1_priv] = ACTIONS(1503),
    [sym_user2_priv] = ACTIONS(1503),
    [sym_user3_priv] = ACTIONS(1503),
    [sym_user4_priv] = ACTIONS(1503),
    [sym_usermanagement_priv] = ACTIONS(1503),
    [sym_valid_account] = ACTIONS(1503),
    [sym_err_name_invalid] = ACTIONS(1503),
    [sym_err_password_invalid] = ACTIONS(1503),
    [sym_err_invalid_account] = ACTIONS(1503),
    [sym_err_no_authorization] = ACTIONS(1503),
    [sym_err_invalid_parameter] = ACTIONS(1503),
    [sym_err_user_locked_out] = ACTIONS(1503),
    [sym_err_user_account_expired] = ACTIONS(1503),
    [sym_err_general_error] = ACTIONS(1503),
    [sym_err_authentication_error] = ACTIONS(1503),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1503),
    [sym_tls_validate_certificate] = ACTIONS(1503),
    [sym____reserved___] = ACTIONS(1503),
    [sym_date] = ACTIONS(1503),
    [sym_day] = ACTIONS(1503),
    [sym_time] = ACTIONS(1503),
    [sym_ldate] = ACTIONS(1503),
    [sym_push_device] = ACTIONS(1503),
    [sym_push_channel] = ACTIONS(1503),
    [sym_push_devchan] = ACTIONS(1503),
    [sym_release_device] = ACTIONS(1503),
    [sym_release_channel] = ACTIONS(1503),
    [sym_release_devchan] = ACTIONS(1503),
    [sym_master_slot] = ACTIONS(1503),
    [sym_get_pulse_time] = ACTIONS(1503),
    [sym_get_timer] = ACTIONS(1503),
    [sym_master_sn] = ACTIONS(1503),
    [sym_system_number] = ACTIONS(1503),
    [sym_button] = ACTIONS(1503),
    [sym_dv_channel] = ACTIONS(1503),
    [sym_data] = ACTIONS(1503),
    [sym_level] = ACTIONS(1503),
    [sym_timeline] = ACTIONS(1503),
    [sym_channel] = ACTIONS(1503),
    [sym_custom] = ACTIONS(1503),
    [sym_atoi] = ACTIONS(1503),
    [sym_atol] = ACTIONS(1503),
    [sym_atof] = ACTIONS(1503),
    [sym_itoa] = ACTIONS(1503),
    [sym_itohex] = ACTIONS(1503),
    [sym_hextoi] = ACTIONS(1503),
    [sym_ftoa] = ACTIONS(1503),
    [sym_find_string] = ACTIONS(1503),
    [sym_left_string] = ACTIONS(1503),
    [sym_right_string] = ACTIONS(1503),
    [sym_mid_string] = ACTIONS(1503),
    [sym_remove_string] = ACTIONS(1503),
    [sym_get_buffer_string] = ACTIONS(1503),
    [sym_get_buffer_char] = ACTIONS(1503),
    [sym_get_multi_buffer_string] = ACTIONS(1503),
    [sym_length_string] = ACTIONS(1503),
    [sym_max_length_string] = ACTIONS(1503),
    [sym_set_length_string] = ACTIONS(1503),
    [sym_compare_string] = ACTIONS(1503),
    [sym_upper_string] = ACTIONS(1503),
    [sym_lower_string] = ACTIONS(1503),
    [sym_get_last] = ACTIONS(1503),
    [sym_date_to_day] = ACTIONS(1503),
    [sym_date_to_month] = ACTIONS(1503),
    [sym_date_to_year] = ACTIONS(1503),
    [sym_time_to_hour] = ACTIONS(1503),
    [sym_time_to_minute] = ACTIONS(1503),
    [sym_time_to_second] = ACTIONS(1503),
    [sym_day_of_week] = ACTIONS(1503),
    [sym_set_pulse_time] = ACTIONS(1503),
    [sym_set_timer] = ACTIONS(1503),
    [sym_file_open] = ACTIONS(1503),
    [sym_file_close] = ACTIONS(1503),
    [sym_file_read] = ACTIONS(1503),
    [sym_file_read_line] = ACTIONS(1503),
    [sym_file_write] = ACTIONS(1503),
    [sym_file_write_line] = ACTIONS(1503),
    [sym_file_copy] = ACTIONS(1503),
    [sym_file_delete] = ACTIONS(1503),
    [sym_file_rename] = ACTIONS(1503),
    [sym_file_dir] = ACTIONS(1503),
    [sym_file_getdir] = ACTIONS(1503),
    [sym_file_setdir] = ACTIONS(1503),
    [sym_file_create_dir] = ACTIONS(1503),
    [sym_file_remove_dir] = ACTIONS(1503),
    [sym_file_seek] = ACTIONS(1503),
    [sym_length_array] = ACTIONS(1503),
    [sym_max_length_array] = ACTIONS(1503),
    [sym_set_length_array] = ACTIONS(1503),
    [sym_ip_client_open] = ACTIONS(1503),
    [sym_ip_client_close] = ACTIONS(1503),
    [sym_ip_server_open] = ACTIONS(1503),
    [sym_ip_server_close] = ACTIONS(1503),
    [sym_ip_bound_client_open] = ACTIONS(1503),
    [sym_ip_mc_server_open] = ACTIONS(1503),
    [sym_ip_set_option] = ACTIONS(1503),
    [sym_ssh_client_open] = ACTIONS(1503),
    [sym_ssh_client_close] = ACTIONS(1503),
    [sym_tls_client_open] = ACTIONS(1503),
    [sym_tls_client_close] = ACTIONS(1503),
    [sym_do_push] = ACTIONS(1503),
    [sym_do_push_timed] = ACTIONS(1503),
    [sym_do_release] = ACTIONS(1503),
    [sym_device_id] = ACTIONS(1503),
    [sym_device_info] = ACTIONS(1503),
    [sym_device_id_string] = ACTIONS(1503),
    [sym_device_standby] = ACTIONS(1503),
    [sym_device_wake] = ACTIONS(1503),
    [sym_timeline_create] = ACTIONS(1503),
    [sym_timeline_kill] = ACTIONS(1503),
    [sym_timeline_reload] = ACTIONS(1503),
    [sym_timeline_pause] = ACTIONS(1503),
    [sym_timeline_restart] = ACTIONS(1503),
    [sym_timeline_set] = ACTIONS(1503),
    [sym_timeline_get] = ACTIONS(1503),
    [sym_timeline_active] = ACTIONS(1503),
    [sym_timeline_dynamic_id] = ACTIONS(1503),
    [sym_variable_to_string] = ACTIONS(1503),
    [sym_string_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_string] = ACTIONS(1503),
    [sym_type_cast] = ACTIONS(1503),
    [sym_variable_to_xml] = ACTIONS(1503),
    [sym_xml_to_variable] = ACTIONS(1503),
    [sym_length_variable_to_xml] = ACTIONS(1503),
    [sym_redirect_string] = ACTIONS(1503),
    [sym_random_number] = ACTIONS(1503),
    [sym_get_unique_id] = ACTIONS(1503),
    [sym_get_system_number] = ACTIONS(1503),
    [sym_set_system_number] = ACTIONS(1503),
    [sym_get_serial_number] = ACTIONS(1503),
    [sym_get_master_build] = ACTIONS(1503),
    [sym_set_validation_code] = ACTIONS(1503),
    [sym_reboot] = ACTIONS(1503),
    [sym_load_duet_module] = ACTIONS(1503),
    [sym_unload_duet_module] = ACTIONS(1503),
    [sym_duet_mem_size_set] = ACTIONS(1503),
    [sym_duet_mem_size_get] = ACTIONS(1503),
    [sym_dynamic_application_device] = ACTIONS(1503),
    [sym_dynamic_polled_port] = ACTIONS(1503),
    [sym_static_port_binding] = ACTIONS(1503),
    [sym_static_ip_binding] = ACTIONS(1503),
    [sym_get_dns_list] = ACTIONS(1503),
    [sym_set_dns_list] = ACTIONS(1503),
    [sym_get_ip_address] = ACTIONS(1503),
    [sym_set_ip_address] = ACTIONS(1503),
    [sym_get_url_list] = ACTIONS(1503),
    [sym_add_url_entry] = ACTIONS(1503),
    [sym_delete_url_entry] = ACTIONS(1503),
    [sym_abs_value] = ACTIONS(1503),
    [sym_max_value] = ACTIONS(1503),
    [sym_min_value] = ACTIONS(1503),
    [sym_exp_value] = ACTIONS(1503),
    [sym_log_value] = ACTIONS(1503),
    [sym_log10_value] = ACTIONS(1503),
    [sym_power_value] = ACTIONS(1503),
    [sym_sqrt_value] = ACTIONS(1503),
    [sym_raw_be] = ACTIONS(1503),
    [sym_raw_le] = ACTIONS(1503),
    [sym_internal_queue_size_set] = ACTIONS(1503),
    [sym_internal_queue_size_get] = ACTIONS(1503),
    [sym_internal_threshold_set] = ACTIONS(1503),
    [sym_internal_threshold_get] = ACTIONS(1503),
    [sym_rebuilt_event] = ACTIONS(1503),
    [sym_do_custom_event] = ACTIONS(1503),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1503),
    [sym_clkmgr_set_clk_source] = ACTIONS(1503),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1503),
    [sym_clkmgr_get_timezone] = ACTIONS(1503),
    [sym_clkmgr_set_timezone] = ACTIONS(1503),
    [sym_clkmgr_get_resync_period] = ACTIONS(1503),
    [sym_clkmgr_set_resync_period] = ACTIONS(1503),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1503),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_timeservers] = ACTIONS(1503),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1503),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1503),
    [sym_set_log_level] = ACTIONS(1503),
    [sym_get_log_level] = ACTIONS(1503),
    [sym_amx_log] = ACTIONS(1503),
    [sym_smtp_server_config_set] = ACTIONS(1503),
    [sym_smtp_server_config_get] = ACTIONS(1503),
    [sym_smtp_send] = ACTIONS(1503),
    [sym_validate_netlinx_account] = ACTIONS(1503),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1503),
    [sym_authenticate_certificate] = ACTIONS(1503),
    [sym_audit_netlinx_session_event] = ACTIONS(1503),
    [sym_audit_netlinx_generic_event] = ACTIONS(1503),
    [sym_get_available_flash_disk_space] = ACTIONS(1503),
    [sym_get_max_flash_disk_space] = ACTIONS(1503),
    [sym_data_create_feed] = ACTIONS(1503),
    [sym_data_delete_feed] = ACTIONS(1503),
    [sym_data_publish_feed] = ACTIONS(1503),
    [sym_data_get_published_feed] = ACTIONS(1503),
    [sym_data_add_record] = ACTIONS(1503),
    [sym_data_get_event_record] = ACTIONS(1503),
    [sym__wc_data_create_feed] = ACTIONS(1503),
    [sym__wc_data_add_record] = ACTIONS(1503),
    [sym__wc_data_get_event_record] = ACTIONS(1503),
    [sym_format] = ACTIONS(1503),
    [sym_astro_clock] = ACTIONS(1503),
    [sym_set_outdoor_temperature] = ACTIONS(1503),
    [sym_set_virtual_level_count] = ACTIONS(1503),
    [sym_set_virtual_channel_count] = ACTIONS(1503),
    [sym_set_virtual_port_count] = ACTIONS(1503),
    [sym__wc_encode_int] = ACTIONS(1503),
    [sym__wc_decode_int] = ACTIONS(1503),
    [sym__wc_int] = ACTIONS(1503),
    [sym__wc_to_ch_int] = ACTIONS(1503),
    [sym__ch_to_wc_int] = ACTIONS(1503),
    [sym__wc_find_string_int] = ACTIONS(1503),
    [sym__wc_left_string_int] = ACTIONS(1503),
    [sym__wc_lower_string_int] = ACTIONS(1503),
    [sym__wc_mid_string_int] = ACTIONS(1503),
    [sym__wc_remove_string_int] = ACTIONS(1503),
    [sym__wc_right_string_int] = ACTIONS(1503),
    [sym__wc_upper_string_int] = ACTIONS(1503),
    [sym__wc_compare_string_int] = ACTIONS(1503),
    [sym__wc_get_buffer_char_int] = ACTIONS(1503),
    [sym__wc_get_buffer_string_int] = ACTIONS(1503),
    [sym__wc_concat_string_int] = ACTIONS(1503),
    [sym__wc_file_open_int] = ACTIONS(1503),
    [sym__wc_file_close_int] = ACTIONS(1503),
    [sym__wc_file_read_int] = ACTIONS(1503),
    [sym__wc_file_read_line_int] = ACTIONS(1503),
    [sym__wc_file_write_int] = ACTIONS(1503),
    [sym__wc_file_write_line_int] = ACTIONS(1503),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1505),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1505),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1505),
    [aux_sym_preproc_else_token1] = ACTIONS(1505),
    [anon_sym_COMMA] = ACTIONS(1505),
    [anon_sym_LBRACK] = ACTIONS(1505),
    [anon_sym_LBRACE] = ACTIONS(1505),
    [anon_sym_RBRACE] = ACTIONS(1505),
    [anon_sym_COLON] = ACTIONS(1505),
    [anon_sym_LPAREN] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1503),
    [anon_sym_TILDE] = ACTIONS(1505),
    [anon_sym_DASH] = ACTIONS(1503),
    [anon_sym_PLUS] = ACTIONS(1503),
    [anon_sym_STAR] = ACTIONS(1505),
    [anon_sym_SLASH] = ACTIONS(1503),
    [anon_sym_PERCENT] = ACTIONS(1505),
    [anon_sym_PIPE_PIPE] = ACTIONS(1505),
    [anon_sym_AMP_AMP] = ACTIONS(1505),
    [anon_sym_PIPE] = ACTIONS(1503),
    [anon_sym_CARET] = ACTIONS(1505),
    [anon_sym_AMP] = ACTIONS(1503),
    [anon_sym_EQ_EQ] = ACTIONS(1505),
    [anon_sym_BANG_EQ] = ACTIONS(1505),
    [anon_sym_LT_GT] = ACTIONS(1505),
    [anon_sym_GT] = ACTIONS(1503),
    [anon_sym_GT_EQ] = ACTIONS(1505),
    [anon_sym_LT_EQ] = ACTIONS(1505),
    [anon_sym_LT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1505),
    [anon_sym_GT_GT] = ACTIONS(1505),
    [anon_sym_DASH_DASH] = ACTIONS(1505),
    [anon_sym_PLUS_PLUS] = ACTIONS(1505),
    [anon_sym_DOT] = ACTIONS(1503),
    [anon_sym_DQUOTE] = ACTIONS(1505),
    [anon_sym_SQUOTE] = ACTIONS(1505),
    [sym_number_literal] = ACTIONS(1505),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(620)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2217),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2217),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2217),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2235),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2217),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(621)] = {
    [sym_identifier] = ACTIONS(1573),
    [sym_and] = ACTIONS(1573),
    [sym_or] = ACTIONS(1573),
    [sym_not] = ACTIONS(1573),
    [sym_xor] = ACTIONS(1573),
    [sym_band] = ACTIONS(1573),
    [sym_bor] = ACTIONS(1573),
    [sym_bxor] = ACTIONS(1573),
    [sym_bnot] = ACTIONS(1573),
    [sym_lshift] = ACTIONS(1573),
    [sym_rshift] = ACTIONS(1573),
    [sym___file__] = ACTIONS(1573),
    [sym___date__] = ACTIONS(1573),
    [sym___ldate__] = ACTIONS(1573),
    [sym___time__] = ACTIONS(1573),
    [sym___line__] = ACTIONS(1573),
    [sym___name__] = ACTIONS(1573),
    [sym___netlinx__] = ACTIONS(1573),
    [sym_netlinx_axi_version] = ACTIONS(1573),
    [sym_url_flg_tcp] = ACTIONS(1573),
    [sym_url_flg_accinfopresent] = ACTIONS(1573),
    [sym_url_flg_temp] = ACTIONS(1573),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1573),
    [sym_url_flg_stat_mask] = ACTIONS(1573),
    [sym_url_flg_stat_lookup] = ACTIONS(1573),
    [sym_url_flg_stat_connecting] = ACTIONS(1573),
    [sym_url_flg_stat_waiting] = ACTIONS(1573),
    [sym_url_flg_stat_connected] = ACTIONS(1573),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1573),
    [sym_first_virtual_device] = ACTIONS(1573),
    [sym_first_local_port] = ACTIONS(1573),
    [sym_dynamic_virtual_device] = ACTIONS(1573),
    [sym_timeline_once] = ACTIONS(1573),
    [sym_timeline_repeat] = ACTIONS(1573),
    [sym_timeline_absolute] = ACTIONS(1573),
    [sym_timeline_relative] = ACTIONS(1573),
    [sym_do_push_timed_infinite] = ACTIONS(1573),
    [sym_source_type_no_address] = ACTIONS(1573),
    [sym_source_type_neuron_id] = ACTIONS(1573),
    [sym_source_type_ip_address] = ACTIONS(1573),
    [sym_source_type_axlink] = ACTIONS(1573),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1573),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1573),
    [sym_source_type_ip_socket_address] = ACTIONS(1573),
    [sym_source_type_rs232] = ACTIONS(1573),
    [sym_source_type_internal] = ACTIONS(1573),
    [sym_source_type_mac_address] = ACTIONS(1573),
    [sym_source_type_ipv4_port] = ACTIONS(1573),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1573),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1573),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1573),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1573),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1573),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1573),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1573),
    [sym_true] = ACTIONS(1573),
    [sym_false] = ACTIONS(1573),
    [sym_file_read_only] = ACTIONS(1573),
    [sym_file_rw_new] = ACTIONS(1573),
    [sym_file_rw_append] = ACTIONS(1573),
    [sym_ip_tcp] = ACTIONS(1573),
    [sym_ip_udp] = ACTIONS(1573),
    [sym_ip_udp_2way] = ACTIONS(1573),
    [sym_xml_encode_types] = ACTIONS(1573),
    [sym_xml_encode_char_as_list] = ACTIONS(1573),
    [sym_xml_encode_le] = ACTIONS(1573),
    [sym_xml_decode_types] = ACTIONS(1573),
    [sym_xml_decode_no_preserve] = ACTIONS(1573),
    [sym_duet_dev_type_display_device] = ACTIONS(1573),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1573),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1573),
    [sym_duet_dev_type_io_device] = ACTIONS(1573),
    [sym_duet_dev_type_relay_device] = ACTIONS(1573),
    [sym_duet_dev_type_ups] = ACTIONS(1573),
    [sym_duet_dev_type_amplifier] = ACTIONS(1573),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1573),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1573),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1573),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1573),
    [sym_duet_dev_type_camera] = ACTIONS(1573),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1573),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1573),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1573),
    [sym_duet_dev_type_dss] = ACTIONS(1573),
    [sym_duet_dev_type_dvr] = ACTIONS(1573),
    [sym_duet_dev_type_disc_device] = ACTIONS(1573),
    [sym_duet_dev_type_document_camera] = ACTIONS(1573),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1573),
    [sym_duet_dev_type_hvac] = ACTIONS(1573),
    [sym_duet_dev_type_keypad] = ACTIONS(1573),
    [sym_duet_dev_type_light] = ACTIONS(1573),
    [sym_duet_dev_type_monitor] = ACTIONS(1573),
    [sym_duet_dev_type_motor] = ACTIONS(1573),
    [sym_duet_dev_type_multi_window] = ACTIONS(1573),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1573),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1573),
    [sym_duet_dev_type_receiver] = ACTIONS(1573),
    [sym_duet_dev_type_security_system] = ACTIONS(1573),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1573),
    [sym_duet_dev_type_settop_box] = ACTIONS(1573),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1573),
    [sym_duet_dev_type_switcher] = ACTIONS(1573),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1573),
    [sym_duet_dev_type_tv] = ACTIONS(1573),
    [sym_duet_dev_type_utility] = ACTIONS(1573),
    [sym_duet_dev_type_vcr] = ACTIONS(1573),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1573),
    [sym_duet_dev_type_video_processor] = ACTIONS(1573),
    [sym_duet_dev_type_video_projector] = ACTIONS(1573),
    [sym_duet_dev_type_video_wall] = ACTIONS(1573),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1573),
    [sym_duet_dev_type_weather] = ACTIONS(1573),
    [sym_duet_dev_not_polled] = ACTIONS(1573),
    [sym_duet_dev_polled] = ACTIONS(1573),
    [sym_clkmgr_mode_network] = ACTIONS(1573),
    [sym_clkmgr_mode_standalone] = ACTIONS(1573),
    [sym_normal_standby] = ACTIONS(1573),
    [sym_normal_wake] = ACTIONS(1573),
    [sym_ip_multicast_ttl_option] = ACTIONS(1573),
    [sym_ip_ttl_subnet] = ACTIONS(1573),
    [sym_ip_ttl_site] = ACTIONS(1573),
    [sym_ip_ttl_region] = ACTIONS(1573),
    [sym_ip_ttl_continent] = ACTIONS(1573),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1573),
    [sym_ip_nodelay_on] = ACTIONS(1573),
    [sym_ip_nodelay_off] = ACTIONS(1573),
    [sym_amx_error] = ACTIONS(1573),
    [sym_amx_warning] = ACTIONS(1573),
    [sym_amx_info] = ACTIONS(1573),
    [sym_amx_debug] = ACTIONS(1573),
    [sym_netlinx_login_success] = ACTIONS(1573),
    [sym_netlinx_login_fail] = ACTIONS(1573),
    [sym_netlinx_logout] = ACTIONS(1573),
    [sym_null_str] = ACTIONS(1573),
    [sym_smtp_address] = ACTIONS(1573),
    [sym_smtp_port_number] = ACTIONS(1573),
    [sym_smtp_username] = ACTIONS(1573),
    [sym_smtp_password] = ACTIONS(1573),
    [sym_smtp_require_tls] = ACTIONS(1573),
    [sym_smtp_from] = ACTIONS(1573),
    [sym_smtp_tls_true] = ACTIONS(1573),
    [sym_smtp_tls_false] = ACTIONS(1573),
    [sym_listview_on_row_select_event] = ACTIONS(1573),
    [sym_data_structure_unknown] = ACTIONS(1573),
    [sym_data_structure_datafeed] = ACTIONS(1573),
    [sym_data_structure_datarecordset] = ACTIONS(1573),
    [sym_data_structure_datarecord] = ACTIONS(1573),
    [sym_data_structure_datafield] = ACTIONS(1573),
    [sym_data_max_content_field_count] = ACTIONS(1573),
    [sym_data_max_metadata_field_count] = ACTIONS(1573),
    [sym_data_max_id_length] = ACTIONS(1573),
    [sym_data_max_name_length] = ACTIONS(1573),
    [sym_data_max_description_length] = ACTIONS(1573),
    [sym_data_max_type_length] = ACTIONS(1573),
    [sym_data_max_format_length] = ACTIONS(1573),
    [sym_data_max_source_length] = ACTIONS(1573),
    [sym_data_max_value_length] = ACTIONS(1573),
    [sym_data_max_label_length] = ACTIONS(1573),
    [sym_data_max_url_length] = ACTIONS(1573),
    [sym_max_last_login_info_length] = ACTIONS(1573),
    [sym_data_type_unknown] = ACTIONS(1573),
    [sym_data_type_string] = ACTIONS(1573),
    [sym_data_type_datetime] = ACTIONS(1573),
    [sym_data_type_date] = ACTIONS(1573),
    [sym_data_type_time] = ACTIONS(1573),
    [sym_data_type_image] = ACTIONS(1573),
    [sym_data_format_url] = ACTIONS(1573),
    [sym_data_format_phone] = ACTIONS(1573),
    [sym_data_format_email] = ACTIONS(1573),
    [sym_data_format_iso8601] = ACTIONS(1573),
    [sym_auditlog_priv] = ACTIONS(1573),
    [sym_configuration_priv] = ACTIONS(1573),
    [sym_deviceconfig_priv] = ACTIONS(1573),
    [sym_ftp_priv] = ACTIONS(1573),
    [sym_http_priv] = ACTIONS(1573),
    [sym_networkconfig_priv] = ACTIONS(1573),
    [sym_programport_priv] = ACTIONS(1573),
    [sym_remoteui_priv] = ACTIONS(1573),
    [sym_securitycontrol_priv] = ACTIONS(1573),
    [sym_softwaremanagement_priv] = ACTIONS(1573),
    [sym_terminal_priv] = ACTIONS(1573),
    [sym_tpadmin_priv] = ACTIONS(1573),
    [sym_user1_priv] = ACTIONS(1573),
    [sym_user2_priv] = ACTIONS(1573),
    [sym_user3_priv] = ACTIONS(1573),
    [sym_user4_priv] = ACTIONS(1573),
    [sym_usermanagement_priv] = ACTIONS(1573),
    [sym_valid_account] = ACTIONS(1573),
    [sym_err_name_invalid] = ACTIONS(1573),
    [sym_err_password_invalid] = ACTIONS(1573),
    [sym_err_invalid_account] = ACTIONS(1573),
    [sym_err_no_authorization] = ACTIONS(1573),
    [sym_err_invalid_parameter] = ACTIONS(1573),
    [sym_err_user_locked_out] = ACTIONS(1573),
    [sym_err_user_account_expired] = ACTIONS(1573),
    [sym_err_general_error] = ACTIONS(1573),
    [sym_err_authentication_error] = ACTIONS(1573),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1573),
    [sym_tls_validate_certificate] = ACTIONS(1573),
    [sym____reserved___] = ACTIONS(1573),
    [sym_date] = ACTIONS(1573),
    [sym_day] = ACTIONS(1573),
    [sym_time] = ACTIONS(1573),
    [sym_ldate] = ACTIONS(1573),
    [sym_push_device] = ACTIONS(1573),
    [sym_push_channel] = ACTIONS(1573),
    [sym_push_devchan] = ACTIONS(1573),
    [sym_release_device] = ACTIONS(1573),
    [sym_release_channel] = ACTIONS(1573),
    [sym_release_devchan] = ACTIONS(1573),
    [sym_master_slot] = ACTIONS(1573),
    [sym_get_pulse_time] = ACTIONS(1573),
    [sym_get_timer] = ACTIONS(1573),
    [sym_master_sn] = ACTIONS(1573),
    [sym_system_number] = ACTIONS(1573),
    [sym_button] = ACTIONS(1573),
    [sym_dv_channel] = ACTIONS(1573),
    [sym_data] = ACTIONS(1573),
    [sym_level] = ACTIONS(1573),
    [sym_timeline] = ACTIONS(1573),
    [sym_channel] = ACTIONS(1573),
    [sym_custom] = ACTIONS(1573),
    [sym_atoi] = ACTIONS(1573),
    [sym_atol] = ACTIONS(1573),
    [sym_atof] = ACTIONS(1573),
    [sym_itoa] = ACTIONS(1573),
    [sym_itohex] = ACTIONS(1573),
    [sym_hextoi] = ACTIONS(1573),
    [sym_ftoa] = ACTIONS(1573),
    [sym_find_string] = ACTIONS(1573),
    [sym_left_string] = ACTIONS(1573),
    [sym_right_string] = ACTIONS(1573),
    [sym_mid_string] = ACTIONS(1573),
    [sym_remove_string] = ACTIONS(1573),
    [sym_get_buffer_string] = ACTIONS(1573),
    [sym_get_buffer_char] = ACTIONS(1573),
    [sym_get_multi_buffer_string] = ACTIONS(1573),
    [sym_length_string] = ACTIONS(1573),
    [sym_max_length_string] = ACTIONS(1573),
    [sym_set_length_string] = ACTIONS(1573),
    [sym_compare_string] = ACTIONS(1573),
    [sym_upper_string] = ACTIONS(1573),
    [sym_lower_string] = ACTIONS(1573),
    [sym_get_last] = ACTIONS(1573),
    [sym_date_to_day] = ACTIONS(1573),
    [sym_date_to_month] = ACTIONS(1573),
    [sym_date_to_year] = ACTIONS(1573),
    [sym_time_to_hour] = ACTIONS(1573),
    [sym_time_to_minute] = ACTIONS(1573),
    [sym_time_to_second] = ACTIONS(1573),
    [sym_day_of_week] = ACTIONS(1573),
    [sym_set_pulse_time] = ACTIONS(1573),
    [sym_set_timer] = ACTIONS(1573),
    [sym_file_open] = ACTIONS(1573),
    [sym_file_close] = ACTIONS(1573),
    [sym_file_read] = ACTIONS(1573),
    [sym_file_read_line] = ACTIONS(1573),
    [sym_file_write] = ACTIONS(1573),
    [sym_file_write_line] = ACTIONS(1573),
    [sym_file_copy] = ACTIONS(1573),
    [sym_file_delete] = ACTIONS(1573),
    [sym_file_rename] = ACTIONS(1573),
    [sym_file_dir] = ACTIONS(1573),
    [sym_file_getdir] = ACTIONS(1573),
    [sym_file_setdir] = ACTIONS(1573),
    [sym_file_create_dir] = ACTIONS(1573),
    [sym_file_remove_dir] = ACTIONS(1573),
    [sym_file_seek] = ACTIONS(1573),
    [sym_length_array] = ACTIONS(1573),
    [sym_max_length_array] = ACTIONS(1573),
    [sym_set_length_array] = ACTIONS(1573),
    [sym_ip_client_open] = ACTIONS(1573),
    [sym_ip_client_close] = ACTIONS(1573),
    [sym_ip_server_open] = ACTIONS(1573),
    [sym_ip_server_close] = ACTIONS(1573),
    [sym_ip_bound_client_open] = ACTIONS(1573),
    [sym_ip_mc_server_open] = ACTIONS(1573),
    [sym_ip_set_option] = ACTIONS(1573),
    [sym_ssh_client_open] = ACTIONS(1573),
    [sym_ssh_client_close] = ACTIONS(1573),
    [sym_tls_client_open] = ACTIONS(1573),
    [sym_tls_client_close] = ACTIONS(1573),
    [sym_do_push] = ACTIONS(1573),
    [sym_do_push_timed] = ACTIONS(1573),
    [sym_do_release] = ACTIONS(1573),
    [sym_device_id] = ACTIONS(1573),
    [sym_device_info] = ACTIONS(1573),
    [sym_device_id_string] = ACTIONS(1573),
    [sym_device_standby] = ACTIONS(1573),
    [sym_device_wake] = ACTIONS(1573),
    [sym_timeline_create] = ACTIONS(1573),
    [sym_timeline_kill] = ACTIONS(1573),
    [sym_timeline_reload] = ACTIONS(1573),
    [sym_timeline_pause] = ACTIONS(1573),
    [sym_timeline_restart] = ACTIONS(1573),
    [sym_timeline_set] = ACTIONS(1573),
    [sym_timeline_get] = ACTIONS(1573),
    [sym_timeline_active] = ACTIONS(1573),
    [sym_timeline_dynamic_id] = ACTIONS(1573),
    [sym_variable_to_string] = ACTIONS(1573),
    [sym_string_to_variable] = ACTIONS(1573),
    [sym_length_variable_to_string] = ACTIONS(1573),
    [sym_type_cast] = ACTIONS(1573),
    [sym_variable_to_xml] = ACTIONS(1573),
    [sym_xml_to_variable] = ACTIONS(1573),
    [sym_length_variable_to_xml] = ACTIONS(1573),
    [sym_redirect_string] = ACTIONS(1573),
    [sym_random_number] = ACTIONS(1573),
    [sym_get_unique_id] = ACTIONS(1573),
    [sym_get_system_number] = ACTIONS(1573),
    [sym_set_system_number] = ACTIONS(1573),
    [sym_get_serial_number] = ACTIONS(1573),
    [sym_get_master_build] = ACTIONS(1573),
    [sym_set_validation_code] = ACTIONS(1573),
    [sym_reboot] = ACTIONS(1573),
    [sym_load_duet_module] = ACTIONS(1573),
    [sym_unload_duet_module] = ACTIONS(1573),
    [sym_duet_mem_size_set] = ACTIONS(1573),
    [sym_duet_mem_size_get] = ACTIONS(1573),
    [sym_dynamic_application_device] = ACTIONS(1573),
    [sym_dynamic_polled_port] = ACTIONS(1573),
    [sym_static_port_binding] = ACTIONS(1573),
    [sym_static_ip_binding] = ACTIONS(1573),
    [sym_get_dns_list] = ACTIONS(1573),
    [sym_set_dns_list] = ACTIONS(1573),
    [sym_get_ip_address] = ACTIONS(1573),
    [sym_set_ip_address] = ACTIONS(1573),
    [sym_get_url_list] = ACTIONS(1573),
    [sym_add_url_entry] = ACTIONS(1573),
    [sym_delete_url_entry] = ACTIONS(1573),
    [sym_abs_value] = ACTIONS(1573),
    [sym_max_value] = ACTIONS(1573),
    [sym_min_value] = ACTIONS(1573),
    [sym_exp_value] = ACTIONS(1573),
    [sym_log_value] = ACTIONS(1573),
    [sym_log10_value] = ACTIONS(1573),
    [sym_power_value] = ACTIONS(1573),
    [sym_sqrt_value] = ACTIONS(1573),
    [sym_raw_be] = ACTIONS(1573),
    [sym_raw_le] = ACTIONS(1573),
    [sym_internal_queue_size_set] = ACTIONS(1573),
    [sym_internal_queue_size_get] = ACTIONS(1573),
    [sym_internal_threshold_set] = ACTIONS(1573),
    [sym_internal_threshold_get] = ACTIONS(1573),
    [sym_rebuilt_event] = ACTIONS(1573),
    [sym_do_custom_event] = ACTIONS(1573),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1573),
    [sym_clkmgr_set_clk_source] = ACTIONS(1573),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1573),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1573),
    [sym_clkmgr_get_timezone] = ACTIONS(1573),
    [sym_clkmgr_set_timezone] = ACTIONS(1573),
    [sym_clkmgr_get_resync_period] = ACTIONS(1573),
    [sym_clkmgr_set_resync_period] = ACTIONS(1573),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1573),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1573),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1573),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1573),
    [sym_clkmgr_get_timeservers] = ACTIONS(1573),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1573),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1573),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1573),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1573),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1573),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1573),
    [sym_set_log_level] = ACTIONS(1573),
    [sym_get_log_level] = ACTIONS(1573),
    [sym_amx_log] = ACTIONS(1573),
    [sym_smtp_server_config_set] = ACTIONS(1573),
    [sym_smtp_server_config_get] = ACTIONS(1573),
    [sym_smtp_send] = ACTIONS(1573),
    [sym_validate_netlinx_account] = ACTIONS(1573),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1573),
    [sym_authenticate_certificate] = ACTIONS(1573),
    [sym_audit_netlinx_session_event] = ACTIONS(1573),
    [sym_audit_netlinx_generic_event] = ACTIONS(1573),
    [sym_get_available_flash_disk_space] = ACTIONS(1573),
    [sym_get_max_flash_disk_space] = ACTIONS(1573),
    [sym_data_create_feed] = ACTIONS(1573),
    [sym_data_delete_feed] = ACTIONS(1573),
    [sym_data_publish_feed] = ACTIONS(1573),
    [sym_data_get_published_feed] = ACTIONS(1573),
    [sym_data_add_record] = ACTIONS(1573),
    [sym_data_get_event_record] = ACTIONS(1573),
    [sym__wc_data_create_feed] = ACTIONS(1573),
    [sym__wc_data_add_record] = ACTIONS(1573),
    [sym__wc_data_get_event_record] = ACTIONS(1573),
    [sym_format] = ACTIONS(1573),
    [sym_astro_clock] = ACTIONS(1573),
    [sym_set_outdoor_temperature] = ACTIONS(1573),
    [sym_set_virtual_level_count] = ACTIONS(1573),
    [sym_set_virtual_channel_count] = ACTIONS(1573),
    [sym_set_virtual_port_count] = ACTIONS(1573),
    [sym__wc_encode_int] = ACTIONS(1573),
    [sym__wc_decode_int] = ACTIONS(1573),
    [sym__wc_int] = ACTIONS(1573),
    [sym__wc_to_ch_int] = ACTIONS(1573),
    [sym__ch_to_wc_int] = ACTIONS(1573),
    [sym__wc_find_string_int] = ACTIONS(1573),
    [sym__wc_left_string_int] = ACTIONS(1573),
    [sym__wc_lower_string_int] = ACTIONS(1573),
    [sym__wc_mid_string_int] = ACTIONS(1573),
    [sym__wc_remove_string_int] = ACTIONS(1573),
    [sym__wc_right_string_int] = ACTIONS(1573),
    [sym__wc_upper_string_int] = ACTIONS(1573),
    [sym__wc_compare_string_int] = ACTIONS(1573),
    [sym__wc_get_buffer_char_int] = ACTIONS(1573),
    [sym__wc_get_buffer_string_int] = ACTIONS(1573),
    [sym__wc_concat_string_int] = ACTIONS(1573),
    [sym__wc_file_open_int] = ACTIONS(1573),
    [sym__wc_file_close_int] = ACTIONS(1573),
    [sym__wc_file_read_int] = ACTIONS(1573),
    [sym__wc_file_read_line_int] = ACTIONS(1573),
    [sym__wc_file_write_int] = ACTIONS(1573),
    [sym__wc_file_write_line_int] = ACTIONS(1573),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1575),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1575),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1575),
    [aux_sym_preproc_else_token1] = ACTIONS(1575),
    [anon_sym_COMMA] = ACTIONS(1575),
    [anon_sym_LBRACK] = ACTIONS(1575),
    [anon_sym_LBRACE] = ACTIONS(1575),
    [anon_sym_RBRACE] = ACTIONS(1575),
    [anon_sym_COLON] = ACTIONS(1575),
    [anon_sym_LPAREN] = ACTIONS(1573),
    [anon_sym_BANG] = ACTIONS(1573),
    [anon_sym_TILDE] = ACTIONS(1575),
    [anon_sym_DASH] = ACTIONS(1573),
    [anon_sym_PLUS] = ACTIONS(1573),
    [anon_sym_STAR] = ACTIONS(1575),
    [anon_sym_SLASH] = ACTIONS(1573),
    [anon_sym_PERCENT] = ACTIONS(1575),
    [anon_sym_PIPE_PIPE] = ACTIONS(1575),
    [anon_sym_AMP_AMP] = ACTIONS(1575),
    [anon_sym_PIPE] = ACTIONS(1573),
    [anon_sym_CARET] = ACTIONS(1575),
    [anon_sym_AMP] = ACTIONS(1573),
    [anon_sym_EQ_EQ] = ACTIONS(1575),
    [anon_sym_BANG_EQ] = ACTIONS(1575),
    [anon_sym_LT_GT] = ACTIONS(1575),
    [anon_sym_GT] = ACTIONS(1573),
    [anon_sym_GT_EQ] = ACTIONS(1575),
    [anon_sym_LT_EQ] = ACTIONS(1575),
    [anon_sym_LT] = ACTIONS(1573),
    [anon_sym_LT_LT] = ACTIONS(1575),
    [anon_sym_GT_GT] = ACTIONS(1575),
    [anon_sym_DASH_DASH] = ACTIONS(1575),
    [anon_sym_PLUS_PLUS] = ACTIONS(1575),
    [anon_sym_DOT] = ACTIONS(1573),
    [anon_sym_DQUOTE] = ACTIONS(1575),
    [anon_sym_SQUOTE] = ACTIONS(1575),
    [sym_number_literal] = ACTIONS(1575),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(622)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2217),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2217),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2217),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2238),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2217),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(623)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2217),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2217),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2217),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2241),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2217),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(624)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2217),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2217),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2217),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2244),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2217),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(625)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2217),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2217),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2217),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2247),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2217),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(626)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2217),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2217),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2217),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2250),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2217),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(627)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2217),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2217),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2217),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2219),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2217),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(628)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2217),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2217),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2217),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2253),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2217),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(629)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2217),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2217),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2217),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2256),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2217),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(630)] = {
    [sym_identifier] = ACTIONS(1581),
    [sym_and] = ACTIONS(1581),
    [sym_or] = ACTIONS(1581),
    [sym_not] = ACTIONS(1581),
    [sym_xor] = ACTIONS(1581),
    [sym_band] = ACTIONS(1581),
    [sym_bor] = ACTIONS(1581),
    [sym_bxor] = ACTIONS(1581),
    [sym_bnot] = ACTIONS(1581),
    [sym_lshift] = ACTIONS(1581),
    [sym_rshift] = ACTIONS(1581),
    [sym___file__] = ACTIONS(1581),
    [sym___date__] = ACTIONS(1581),
    [sym___ldate__] = ACTIONS(1581),
    [sym___time__] = ACTIONS(1581),
    [sym___line__] = ACTIONS(1581),
    [sym___name__] = ACTIONS(1581),
    [sym___netlinx__] = ACTIONS(1581),
    [sym_netlinx_axi_version] = ACTIONS(1581),
    [sym_url_flg_tcp] = ACTIONS(1581),
    [sym_url_flg_accinfopresent] = ACTIONS(1581),
    [sym_url_flg_temp] = ACTIONS(1581),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1581),
    [sym_url_flg_stat_mask] = ACTIONS(1581),
    [sym_url_flg_stat_lookup] = ACTIONS(1581),
    [sym_url_flg_stat_connecting] = ACTIONS(1581),
    [sym_url_flg_stat_waiting] = ACTIONS(1581),
    [sym_url_flg_stat_connected] = ACTIONS(1581),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1581),
    [sym_first_virtual_device] = ACTIONS(1581),
    [sym_first_local_port] = ACTIONS(1581),
    [sym_dynamic_virtual_device] = ACTIONS(1581),
    [sym_timeline_once] = ACTIONS(1581),
    [sym_timeline_repeat] = ACTIONS(1581),
    [sym_timeline_absolute] = ACTIONS(1581),
    [sym_timeline_relative] = ACTIONS(1581),
    [sym_do_push_timed_infinite] = ACTIONS(1581),
    [sym_source_type_no_address] = ACTIONS(1581),
    [sym_source_type_neuron_id] = ACTIONS(1581),
    [sym_source_type_ip_address] = ACTIONS(1581),
    [sym_source_type_axlink] = ACTIONS(1581),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1581),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1581),
    [sym_source_type_ip_socket_address] = ACTIONS(1581),
    [sym_source_type_rs232] = ACTIONS(1581),
    [sym_source_type_internal] = ACTIONS(1581),
    [sym_source_type_mac_address] = ACTIONS(1581),
    [sym_source_type_ipv4_port] = ACTIONS(1581),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1581),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1581),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1581),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1581),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1581),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1581),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1581),
    [sym_true] = ACTIONS(1581),
    [sym_false] = ACTIONS(1581),
    [sym_file_read_only] = ACTIONS(1581),
    [sym_file_rw_new] = ACTIONS(1581),
    [sym_file_rw_append] = ACTIONS(1581),
    [sym_ip_tcp] = ACTIONS(1581),
    [sym_ip_udp] = ACTIONS(1581),
    [sym_ip_udp_2way] = ACTIONS(1581),
    [sym_xml_encode_types] = ACTIONS(1581),
    [sym_xml_encode_char_as_list] = ACTIONS(1581),
    [sym_xml_encode_le] = ACTIONS(1581),
    [sym_xml_decode_types] = ACTIONS(1581),
    [sym_xml_decode_no_preserve] = ACTIONS(1581),
    [sym_duet_dev_type_display_device] = ACTIONS(1581),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1581),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1581),
    [sym_duet_dev_type_io_device] = ACTIONS(1581),
    [sym_duet_dev_type_relay_device] = ACTIONS(1581),
    [sym_duet_dev_type_ups] = ACTIONS(1581),
    [sym_duet_dev_type_amplifier] = ACTIONS(1581),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1581),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1581),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1581),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1581),
    [sym_duet_dev_type_camera] = ACTIONS(1581),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1581),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1581),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1581),
    [sym_duet_dev_type_dss] = ACTIONS(1581),
    [sym_duet_dev_type_dvr] = ACTIONS(1581),
    [sym_duet_dev_type_disc_device] = ACTIONS(1581),
    [sym_duet_dev_type_document_camera] = ACTIONS(1581),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1581),
    [sym_duet_dev_type_hvac] = ACTIONS(1581),
    [sym_duet_dev_type_keypad] = ACTIONS(1581),
    [sym_duet_dev_type_light] = ACTIONS(1581),
    [sym_duet_dev_type_monitor] = ACTIONS(1581),
    [sym_duet_dev_type_motor] = ACTIONS(1581),
    [sym_duet_dev_type_multi_window] = ACTIONS(1581),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1581),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1581),
    [sym_duet_dev_type_receiver] = ACTIONS(1581),
    [sym_duet_dev_type_security_system] = ACTIONS(1581),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1581),
    [sym_duet_dev_type_settop_box] = ACTIONS(1581),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1581),
    [sym_duet_dev_type_switcher] = ACTIONS(1581),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1581),
    [sym_duet_dev_type_tv] = ACTIONS(1581),
    [sym_duet_dev_type_utility] = ACTIONS(1581),
    [sym_duet_dev_type_vcr] = ACTIONS(1581),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1581),
    [sym_duet_dev_type_video_processor] = ACTIONS(1581),
    [sym_duet_dev_type_video_projector] = ACTIONS(1581),
    [sym_duet_dev_type_video_wall] = ACTIONS(1581),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1581),
    [sym_duet_dev_type_weather] = ACTIONS(1581),
    [sym_duet_dev_not_polled] = ACTIONS(1581),
    [sym_duet_dev_polled] = ACTIONS(1581),
    [sym_clkmgr_mode_network] = ACTIONS(1581),
    [sym_clkmgr_mode_standalone] = ACTIONS(1581),
    [sym_normal_standby] = ACTIONS(1581),
    [sym_normal_wake] = ACTIONS(1581),
    [sym_ip_multicast_ttl_option] = ACTIONS(1581),
    [sym_ip_ttl_subnet] = ACTIONS(1581),
    [sym_ip_ttl_site] = ACTIONS(1581),
    [sym_ip_ttl_region] = ACTIONS(1581),
    [sym_ip_ttl_continent] = ACTIONS(1581),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1581),
    [sym_ip_nodelay_on] = ACTIONS(1581),
    [sym_ip_nodelay_off] = ACTIONS(1581),
    [sym_amx_error] = ACTIONS(1581),
    [sym_amx_warning] = ACTIONS(1581),
    [sym_amx_info] = ACTIONS(1581),
    [sym_amx_debug] = ACTIONS(1581),
    [sym_netlinx_login_success] = ACTIONS(1581),
    [sym_netlinx_login_fail] = ACTIONS(1581),
    [sym_netlinx_logout] = ACTIONS(1581),
    [sym_null_str] = ACTIONS(1581),
    [sym_smtp_address] = ACTIONS(1581),
    [sym_smtp_port_number] = ACTIONS(1581),
    [sym_smtp_username] = ACTIONS(1581),
    [sym_smtp_password] = ACTIONS(1581),
    [sym_smtp_require_tls] = ACTIONS(1581),
    [sym_smtp_from] = ACTIONS(1581),
    [sym_smtp_tls_true] = ACTIONS(1581),
    [sym_smtp_tls_false] = ACTIONS(1581),
    [sym_listview_on_row_select_event] = ACTIONS(1581),
    [sym_data_structure_unknown] = ACTIONS(1581),
    [sym_data_structure_datafeed] = ACTIONS(1581),
    [sym_data_structure_datarecordset] = ACTIONS(1581),
    [sym_data_structure_datarecord] = ACTIONS(1581),
    [sym_data_structure_datafield] = ACTIONS(1581),
    [sym_data_max_content_field_count] = ACTIONS(1581),
    [sym_data_max_metadata_field_count] = ACTIONS(1581),
    [sym_data_max_id_length] = ACTIONS(1581),
    [sym_data_max_name_length] = ACTIONS(1581),
    [sym_data_max_description_length] = ACTIONS(1581),
    [sym_data_max_type_length] = ACTIONS(1581),
    [sym_data_max_format_length] = ACTIONS(1581),
    [sym_data_max_source_length] = ACTIONS(1581),
    [sym_data_max_value_length] = ACTIONS(1581),
    [sym_data_max_label_length] = ACTIONS(1581),
    [sym_data_max_url_length] = ACTIONS(1581),
    [sym_max_last_login_info_length] = ACTIONS(1581),
    [sym_data_type_unknown] = ACTIONS(1581),
    [sym_data_type_string] = ACTIONS(1581),
    [sym_data_type_datetime] = ACTIONS(1581),
    [sym_data_type_date] = ACTIONS(1581),
    [sym_data_type_time] = ACTIONS(1581),
    [sym_data_type_image] = ACTIONS(1581),
    [sym_data_format_url] = ACTIONS(1581),
    [sym_data_format_phone] = ACTIONS(1581),
    [sym_data_format_email] = ACTIONS(1581),
    [sym_data_format_iso8601] = ACTIONS(1581),
    [sym_auditlog_priv] = ACTIONS(1581),
    [sym_configuration_priv] = ACTIONS(1581),
    [sym_deviceconfig_priv] = ACTIONS(1581),
    [sym_ftp_priv] = ACTIONS(1581),
    [sym_http_priv] = ACTIONS(1581),
    [sym_networkconfig_priv] = ACTIONS(1581),
    [sym_programport_priv] = ACTIONS(1581),
    [sym_remoteui_priv] = ACTIONS(1581),
    [sym_securitycontrol_priv] = ACTIONS(1581),
    [sym_softwaremanagement_priv] = ACTIONS(1581),
    [sym_terminal_priv] = ACTIONS(1581),
    [sym_tpadmin_priv] = ACTIONS(1581),
    [sym_user1_priv] = ACTIONS(1581),
    [sym_user2_priv] = ACTIONS(1581),
    [sym_user3_priv] = ACTIONS(1581),
    [sym_user4_priv] = ACTIONS(1581),
    [sym_usermanagement_priv] = ACTIONS(1581),
    [sym_valid_account] = ACTIONS(1581),
    [sym_err_name_invalid] = ACTIONS(1581),
    [sym_err_password_invalid] = ACTIONS(1581),
    [sym_err_invalid_account] = ACTIONS(1581),
    [sym_err_no_authorization] = ACTIONS(1581),
    [sym_err_invalid_parameter] = ACTIONS(1581),
    [sym_err_user_locked_out] = ACTIONS(1581),
    [sym_err_user_account_expired] = ACTIONS(1581),
    [sym_err_general_error] = ACTIONS(1581),
    [sym_err_authentication_error] = ACTIONS(1581),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1581),
    [sym_tls_validate_certificate] = ACTIONS(1581),
    [sym____reserved___] = ACTIONS(1581),
    [sym_date] = ACTIONS(1581),
    [sym_day] = ACTIONS(1581),
    [sym_time] = ACTIONS(1581),
    [sym_ldate] = ACTIONS(1581),
    [sym_push_device] = ACTIONS(1581),
    [sym_push_channel] = ACTIONS(1581),
    [sym_push_devchan] = ACTIONS(1581),
    [sym_release_device] = ACTIONS(1581),
    [sym_release_channel] = ACTIONS(1581),
    [sym_release_devchan] = ACTIONS(1581),
    [sym_master_slot] = ACTIONS(1581),
    [sym_get_pulse_time] = ACTIONS(1581),
    [sym_get_timer] = ACTIONS(1581),
    [sym_master_sn] = ACTIONS(1581),
    [sym_system_number] = ACTIONS(1581),
    [sym_button] = ACTIONS(1581),
    [sym_dv_channel] = ACTIONS(1581),
    [sym_data] = ACTIONS(1581),
    [sym_level] = ACTIONS(1581),
    [sym_timeline] = ACTIONS(1581),
    [sym_channel] = ACTIONS(1581),
    [sym_custom] = ACTIONS(1581),
    [sym_atoi] = ACTIONS(1581),
    [sym_atol] = ACTIONS(1581),
    [sym_atof] = ACTIONS(1581),
    [sym_itoa] = ACTIONS(1581),
    [sym_itohex] = ACTIONS(1581),
    [sym_hextoi] = ACTIONS(1581),
    [sym_ftoa] = ACTIONS(1581),
    [sym_find_string] = ACTIONS(1581),
    [sym_left_string] = ACTIONS(1581),
    [sym_right_string] = ACTIONS(1581),
    [sym_mid_string] = ACTIONS(1581),
    [sym_remove_string] = ACTIONS(1581),
    [sym_get_buffer_string] = ACTIONS(1581),
    [sym_get_buffer_char] = ACTIONS(1581),
    [sym_get_multi_buffer_string] = ACTIONS(1581),
    [sym_length_string] = ACTIONS(1581),
    [sym_max_length_string] = ACTIONS(1581),
    [sym_set_length_string] = ACTIONS(1581),
    [sym_compare_string] = ACTIONS(1581),
    [sym_upper_string] = ACTIONS(1581),
    [sym_lower_string] = ACTIONS(1581),
    [sym_get_last] = ACTIONS(1581),
    [sym_date_to_day] = ACTIONS(1581),
    [sym_date_to_month] = ACTIONS(1581),
    [sym_date_to_year] = ACTIONS(1581),
    [sym_time_to_hour] = ACTIONS(1581),
    [sym_time_to_minute] = ACTIONS(1581),
    [sym_time_to_second] = ACTIONS(1581),
    [sym_day_of_week] = ACTIONS(1581),
    [sym_set_pulse_time] = ACTIONS(1581),
    [sym_set_timer] = ACTIONS(1581),
    [sym_file_open] = ACTIONS(1581),
    [sym_file_close] = ACTIONS(1581),
    [sym_file_read] = ACTIONS(1581),
    [sym_file_read_line] = ACTIONS(1581),
    [sym_file_write] = ACTIONS(1581),
    [sym_file_write_line] = ACTIONS(1581),
    [sym_file_copy] = ACTIONS(1581),
    [sym_file_delete] = ACTIONS(1581),
    [sym_file_rename] = ACTIONS(1581),
    [sym_file_dir] = ACTIONS(1581),
    [sym_file_getdir] = ACTIONS(1581),
    [sym_file_setdir] = ACTIONS(1581),
    [sym_file_create_dir] = ACTIONS(1581),
    [sym_file_remove_dir] = ACTIONS(1581),
    [sym_file_seek] = ACTIONS(1581),
    [sym_length_array] = ACTIONS(1581),
    [sym_max_length_array] = ACTIONS(1581),
    [sym_set_length_array] = ACTIONS(1581),
    [sym_ip_client_open] = ACTIONS(1581),
    [sym_ip_client_close] = ACTIONS(1581),
    [sym_ip_server_open] = ACTIONS(1581),
    [sym_ip_server_close] = ACTIONS(1581),
    [sym_ip_bound_client_open] = ACTIONS(1581),
    [sym_ip_mc_server_open] = ACTIONS(1581),
    [sym_ip_set_option] = ACTIONS(1581),
    [sym_ssh_client_open] = ACTIONS(1581),
    [sym_ssh_client_close] = ACTIONS(1581),
    [sym_tls_client_open] = ACTIONS(1581),
    [sym_tls_client_close] = ACTIONS(1581),
    [sym_do_push] = ACTIONS(1581),
    [sym_do_push_timed] = ACTIONS(1581),
    [sym_do_release] = ACTIONS(1581),
    [sym_device_id] = ACTIONS(1581),
    [sym_device_info] = ACTIONS(1581),
    [sym_device_id_string] = ACTIONS(1581),
    [sym_device_standby] = ACTIONS(1581),
    [sym_device_wake] = ACTIONS(1581),
    [sym_timeline_create] = ACTIONS(1581),
    [sym_timeline_kill] = ACTIONS(1581),
    [sym_timeline_reload] = ACTIONS(1581),
    [sym_timeline_pause] = ACTIONS(1581),
    [sym_timeline_restart] = ACTIONS(1581),
    [sym_timeline_set] = ACTIONS(1581),
    [sym_timeline_get] = ACTIONS(1581),
    [sym_timeline_active] = ACTIONS(1581),
    [sym_timeline_dynamic_id] = ACTIONS(1581),
    [sym_variable_to_string] = ACTIONS(1581),
    [sym_string_to_variable] = ACTIONS(1581),
    [sym_length_variable_to_string] = ACTIONS(1581),
    [sym_type_cast] = ACTIONS(1581),
    [sym_variable_to_xml] = ACTIONS(1581),
    [sym_xml_to_variable] = ACTIONS(1581),
    [sym_length_variable_to_xml] = ACTIONS(1581),
    [sym_redirect_string] = ACTIONS(1581),
    [sym_random_number] = ACTIONS(1581),
    [sym_get_unique_id] = ACTIONS(1581),
    [sym_get_system_number] = ACTIONS(1581),
    [sym_set_system_number] = ACTIONS(1581),
    [sym_get_serial_number] = ACTIONS(1581),
    [sym_get_master_build] = ACTIONS(1581),
    [sym_set_validation_code] = ACTIONS(1581),
    [sym_reboot] = ACTIONS(1581),
    [sym_load_duet_module] = ACTIONS(1581),
    [sym_unload_duet_module] = ACTIONS(1581),
    [sym_duet_mem_size_set] = ACTIONS(1581),
    [sym_duet_mem_size_get] = ACTIONS(1581),
    [sym_dynamic_application_device] = ACTIONS(1581),
    [sym_dynamic_polled_port] = ACTIONS(1581),
    [sym_static_port_binding] = ACTIONS(1581),
    [sym_static_ip_binding] = ACTIONS(1581),
    [sym_get_dns_list] = ACTIONS(1581),
    [sym_set_dns_list] = ACTIONS(1581),
    [sym_get_ip_address] = ACTIONS(1581),
    [sym_set_ip_address] = ACTIONS(1581),
    [sym_get_url_list] = ACTIONS(1581),
    [sym_add_url_entry] = ACTIONS(1581),
    [sym_delete_url_entry] = ACTIONS(1581),
    [sym_abs_value] = ACTIONS(1581),
    [sym_max_value] = ACTIONS(1581),
    [sym_min_value] = ACTIONS(1581),
    [sym_exp_value] = ACTIONS(1581),
    [sym_log_value] = ACTIONS(1581),
    [sym_log10_value] = ACTIONS(1581),
    [sym_power_value] = ACTIONS(1581),
    [sym_sqrt_value] = ACTIONS(1581),
    [sym_raw_be] = ACTIONS(1581),
    [sym_raw_le] = ACTIONS(1581),
    [sym_internal_queue_size_set] = ACTIONS(1581),
    [sym_internal_queue_size_get] = ACTIONS(1581),
    [sym_internal_threshold_set] = ACTIONS(1581),
    [sym_internal_threshold_get] = ACTIONS(1581),
    [sym_rebuilt_event] = ACTIONS(1581),
    [sym_do_custom_event] = ACTIONS(1581),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1581),
    [sym_clkmgr_set_clk_source] = ACTIONS(1581),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1581),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1581),
    [sym_clkmgr_get_timezone] = ACTIONS(1581),
    [sym_clkmgr_set_timezone] = ACTIONS(1581),
    [sym_clkmgr_get_resync_period] = ACTIONS(1581),
    [sym_clkmgr_set_resync_period] = ACTIONS(1581),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1581),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1581),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1581),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1581),
    [sym_clkmgr_get_timeservers] = ACTIONS(1581),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1581),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1581),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1581),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1581),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1581),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1581),
    [sym_set_log_level] = ACTIONS(1581),
    [sym_get_log_level] = ACTIONS(1581),
    [sym_amx_log] = ACTIONS(1581),
    [sym_smtp_server_config_set] = ACTIONS(1581),
    [sym_smtp_server_config_get] = ACTIONS(1581),
    [sym_smtp_send] = ACTIONS(1581),
    [sym_validate_netlinx_account] = ACTIONS(1581),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1581),
    [sym_authenticate_certificate] = ACTIONS(1581),
    [sym_audit_netlinx_session_event] = ACTIONS(1581),
    [sym_audit_netlinx_generic_event] = ACTIONS(1581),
    [sym_get_available_flash_disk_space] = ACTIONS(1581),
    [sym_get_max_flash_disk_space] = ACTIONS(1581),
    [sym_data_create_feed] = ACTIONS(1581),
    [sym_data_delete_feed] = ACTIONS(1581),
    [sym_data_publish_feed] = ACTIONS(1581),
    [sym_data_get_published_feed] = ACTIONS(1581),
    [sym_data_add_record] = ACTIONS(1581),
    [sym_data_get_event_record] = ACTIONS(1581),
    [sym__wc_data_create_feed] = ACTIONS(1581),
    [sym__wc_data_add_record] = ACTIONS(1581),
    [sym__wc_data_get_event_record] = ACTIONS(1581),
    [sym_format] = ACTIONS(1581),
    [sym_astro_clock] = ACTIONS(1581),
    [sym_set_outdoor_temperature] = ACTIONS(1581),
    [sym_set_virtual_level_count] = ACTIONS(1581),
    [sym_set_virtual_channel_count] = ACTIONS(1581),
    [sym_set_virtual_port_count] = ACTIONS(1581),
    [sym__wc_encode_int] = ACTIONS(1581),
    [sym__wc_decode_int] = ACTIONS(1581),
    [sym__wc_int] = ACTIONS(1581),
    [sym__wc_to_ch_int] = ACTIONS(1581),
    [sym__ch_to_wc_int] = ACTIONS(1581),
    [sym__wc_find_string_int] = ACTIONS(1581),
    [sym__wc_left_string_int] = ACTIONS(1581),
    [sym__wc_lower_string_int] = ACTIONS(1581),
    [sym__wc_mid_string_int] = ACTIONS(1581),
    [sym__wc_remove_string_int] = ACTIONS(1581),
    [sym__wc_right_string_int] = ACTIONS(1581),
    [sym__wc_upper_string_int] = ACTIONS(1581),
    [sym__wc_compare_string_int] = ACTIONS(1581),
    [sym__wc_get_buffer_char_int] = ACTIONS(1581),
    [sym__wc_get_buffer_string_int] = ACTIONS(1581),
    [sym__wc_concat_string_int] = ACTIONS(1581),
    [sym__wc_file_open_int] = ACTIONS(1581),
    [sym__wc_file_close_int] = ACTIONS(1581),
    [sym__wc_file_read_int] = ACTIONS(1581),
    [sym__wc_file_read_line_int] = ACTIONS(1581),
    [sym__wc_file_write_int] = ACTIONS(1581),
    [sym__wc_file_write_line_int] = ACTIONS(1581),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(1583),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(1583),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(1583),
    [aux_sym_preproc_else_token1] = ACTIONS(1583),
    [anon_sym_COMMA] = ACTIONS(1583),
    [anon_sym_LBRACK] = ACTIONS(1583),
    [anon_sym_LBRACE] = ACTIONS(1583),
    [anon_sym_RBRACE] = ACTIONS(1583),
    [anon_sym_COLON] = ACTIONS(1583),
    [anon_sym_LPAREN] = ACTIONS(1581),
    [anon_sym_BANG] = ACTIONS(1581),
    [anon_sym_TILDE] = ACTIONS(1583),
    [anon_sym_DASH] = ACTIONS(1581),
    [anon_sym_PLUS] = ACTIONS(1581),
    [anon_sym_STAR] = ACTIONS(1583),
    [anon_sym_SLASH] = ACTIONS(1581),
    [anon_sym_PERCENT] = ACTIONS(1583),
    [anon_sym_PIPE_PIPE] = ACTIONS(1583),
    [anon_sym_AMP_AMP] = ACTIONS(1583),
    [anon_sym_PIPE] = ACTIONS(1581),
    [anon_sym_CARET] = ACTIONS(1583),
    [anon_sym_AMP] = ACTIONS(1581),
    [anon_sym_EQ_EQ] = ACTIONS(1583),
    [anon_sym_BANG_EQ] = ACTIONS(1583),
    [anon_sym_LT_GT] = ACTIONS(1583),
    [anon_sym_GT] = ACTIONS(1581),
    [anon_sym_GT_EQ] = ACTIONS(1583),
    [anon_sym_LT_EQ] = ACTIONS(1583),
    [anon_sym_LT] = ACTIONS(1581),
    [anon_sym_LT_LT] = ACTIONS(1583),
    [anon_sym_GT_GT] = ACTIONS(1583),
    [anon_sym_DASH_DASH] = ACTIONS(1583),
    [anon_sym_PLUS_PLUS] = ACTIONS(1583),
    [anon_sym_DOT] = ACTIONS(1581),
    [anon_sym_DQUOTE] = ACTIONS(1583),
    [anon_sym_SQUOTE] = ACTIONS(1583),
    [sym_number_literal] = ACTIONS(1583),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(631)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2259),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2259),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2259),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2259),
    [sym___date__] = ACTIONS(2259),
    [sym___ldate__] = ACTIONS(2259),
    [sym___time__] = ACTIONS(2259),
    [sym___line__] = ACTIONS(2259),
    [sym___name__] = ACTIONS(2259),
    [sym___netlinx__] = ACTIONS(2259),
    [sym_netlinx_axi_version] = ACTIONS(2259),
    [sym_url_flg_tcp] = ACTIONS(2259),
    [sym_url_flg_accinfopresent] = ACTIONS(2259),
    [sym_url_flg_temp] = ACTIONS(2259),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2259),
    [sym_url_flg_stat_mask] = ACTIONS(2259),
    [sym_url_flg_stat_lookup] = ACTIONS(2259),
    [sym_url_flg_stat_connecting] = ACTIONS(2259),
    [sym_url_flg_stat_waiting] = ACTIONS(2259),
    [sym_url_flg_stat_connected] = ACTIONS(2259),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2259),
    [sym_first_virtual_device] = ACTIONS(2259),
    [sym_first_local_port] = ACTIONS(2259),
    [sym_dynamic_virtual_device] = ACTIONS(2259),
    [sym_timeline_once] = ACTIONS(2259),
    [sym_timeline_repeat] = ACTIONS(2259),
    [sym_timeline_absolute] = ACTIONS(2259),
    [sym_timeline_relative] = ACTIONS(2259),
    [sym_do_push_timed_infinite] = ACTIONS(2259),
    [sym_source_type_no_address] = ACTIONS(2259),
    [sym_source_type_neuron_id] = ACTIONS(2259),
    [sym_source_type_ip_address] = ACTIONS(2259),
    [sym_source_type_axlink] = ACTIONS(2259),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2259),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2259),
    [sym_source_type_ip_socket_address] = ACTIONS(2259),
    [sym_source_type_rs232] = ACTIONS(2259),
    [sym_source_type_internal] = ACTIONS(2259),
    [sym_source_type_mac_address] = ACTIONS(2259),
    [sym_source_type_ipv4_port] = ACTIONS(2259),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2259),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2259),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2259),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2259),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2259),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2259),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2259),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2259),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2259),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2259),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2259),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2259),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2259),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2259),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2259),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2259),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2259),
    [sym_true] = ACTIONS(2259),
    [sym_false] = ACTIONS(2259),
    [sym_file_read_only] = ACTIONS(2259),
    [sym_file_rw_new] = ACTIONS(2259),
    [sym_file_rw_append] = ACTIONS(2259),
    [sym_ip_tcp] = ACTIONS(2259),
    [sym_ip_udp] = ACTIONS(2259),
    [sym_ip_udp_2way] = ACTIONS(2259),
    [sym_xml_encode_types] = ACTIONS(2259),
    [sym_xml_encode_char_as_list] = ACTIONS(2259),
    [sym_xml_encode_le] = ACTIONS(2259),
    [sym_xml_decode_types] = ACTIONS(2259),
    [sym_xml_decode_no_preserve] = ACTIONS(2259),
    [sym_duet_dev_type_display_device] = ACTIONS(2259),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2259),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2259),
    [sym_duet_dev_type_io_device] = ACTIONS(2259),
    [sym_duet_dev_type_relay_device] = ACTIONS(2259),
    [sym_duet_dev_type_ups] = ACTIONS(2259),
    [sym_duet_dev_type_amplifier] = ACTIONS(2259),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2259),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2259),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2259),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2259),
    [sym_duet_dev_type_camera] = ACTIONS(2259),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2259),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2259),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2259),
    [sym_duet_dev_type_dss] = ACTIONS(2259),
    [sym_duet_dev_type_dvr] = ACTIONS(2259),
    [sym_duet_dev_type_disc_device] = ACTIONS(2259),
    [sym_duet_dev_type_document_camera] = ACTIONS(2259),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2259),
    [sym_duet_dev_type_hvac] = ACTIONS(2259),
    [sym_duet_dev_type_keypad] = ACTIONS(2259),
    [sym_duet_dev_type_light] = ACTIONS(2259),
    [sym_duet_dev_type_monitor] = ACTIONS(2259),
    [sym_duet_dev_type_motor] = ACTIONS(2259),
    [sym_duet_dev_type_multi_window] = ACTIONS(2259),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2259),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2259),
    [sym_duet_dev_type_receiver] = ACTIONS(2259),
    [sym_duet_dev_type_security_system] = ACTIONS(2259),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2259),
    [sym_duet_dev_type_settop_box] = ACTIONS(2259),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2259),
    [sym_duet_dev_type_switcher] = ACTIONS(2259),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2259),
    [sym_duet_dev_type_tv] = ACTIONS(2259),
    [sym_duet_dev_type_utility] = ACTIONS(2259),
    [sym_duet_dev_type_vcr] = ACTIONS(2259),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2259),
    [sym_duet_dev_type_video_processor] = ACTIONS(2259),
    [sym_duet_dev_type_video_projector] = ACTIONS(2259),
    [sym_duet_dev_type_video_wall] = ACTIONS(2259),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2259),
    [sym_duet_dev_type_weather] = ACTIONS(2259),
    [sym_duet_dev_not_polled] = ACTIONS(2259),
    [sym_duet_dev_polled] = ACTIONS(2259),
    [sym_clkmgr_mode_network] = ACTIONS(2259),
    [sym_clkmgr_mode_standalone] = ACTIONS(2259),
    [sym_normal_standby] = ACTIONS(2259),
    [sym_normal_wake] = ACTIONS(2259),
    [sym_ip_multicast_ttl_option] = ACTIONS(2259),
    [sym_ip_ttl_subnet] = ACTIONS(2259),
    [sym_ip_ttl_site] = ACTIONS(2259),
    [sym_ip_ttl_region] = ACTIONS(2259),
    [sym_ip_ttl_continent] = ACTIONS(2259),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2259),
    [sym_ip_nodelay_on] = ACTIONS(2259),
    [sym_ip_nodelay_off] = ACTIONS(2259),
    [sym_amx_error] = ACTIONS(2259),
    [sym_amx_warning] = ACTIONS(2259),
    [sym_amx_info] = ACTIONS(2259),
    [sym_amx_debug] = ACTIONS(2259),
    [sym_netlinx_login_success] = ACTIONS(2259),
    [sym_netlinx_login_fail] = ACTIONS(2259),
    [sym_netlinx_logout] = ACTIONS(2259),
    [sym_null_str] = ACTIONS(2259),
    [sym_smtp_address] = ACTIONS(2259),
    [sym_smtp_port_number] = ACTIONS(2259),
    [sym_smtp_username] = ACTIONS(2259),
    [sym_smtp_password] = ACTIONS(2259),
    [sym_smtp_require_tls] = ACTIONS(2259),
    [sym_smtp_from] = ACTIONS(2259),
    [sym_smtp_tls_true] = ACTIONS(2259),
    [sym_smtp_tls_false] = ACTIONS(2259),
    [sym_listview_on_row_select_event] = ACTIONS(2259),
    [sym_data_structure_unknown] = ACTIONS(2259),
    [sym_data_structure_datafeed] = ACTIONS(2259),
    [sym_data_structure_datarecordset] = ACTIONS(2259),
    [sym_data_structure_datarecord] = ACTIONS(2259),
    [sym_data_structure_datafield] = ACTIONS(2259),
    [sym_data_max_content_field_count] = ACTIONS(2259),
    [sym_data_max_metadata_field_count] = ACTIONS(2259),
    [sym_data_max_id_length] = ACTIONS(2259),
    [sym_data_max_name_length] = ACTIONS(2259),
    [sym_data_max_description_length] = ACTIONS(2259),
    [sym_data_max_type_length] = ACTIONS(2259),
    [sym_data_max_format_length] = ACTIONS(2259),
    [sym_data_max_source_length] = ACTIONS(2259),
    [sym_data_max_value_length] = ACTIONS(2259),
    [sym_data_max_label_length] = ACTIONS(2259),
    [sym_data_max_url_length] = ACTIONS(2259),
    [sym_max_last_login_info_length] = ACTIONS(2259),
    [sym_data_type_unknown] = ACTIONS(2259),
    [sym_data_type_string] = ACTIONS(2259),
    [sym_data_type_datetime] = ACTIONS(2259),
    [sym_data_type_date] = ACTIONS(2259),
    [sym_data_type_time] = ACTIONS(2259),
    [sym_data_type_image] = ACTIONS(2259),
    [sym_data_format_url] = ACTIONS(2259),
    [sym_data_format_phone] = ACTIONS(2259),
    [sym_data_format_email] = ACTIONS(2259),
    [sym_data_format_iso8601] = ACTIONS(2259),
    [sym_auditlog_priv] = ACTIONS(2259),
    [sym_configuration_priv] = ACTIONS(2259),
    [sym_deviceconfig_priv] = ACTIONS(2259),
    [sym_ftp_priv] = ACTIONS(2259),
    [sym_http_priv] = ACTIONS(2259),
    [sym_networkconfig_priv] = ACTIONS(2259),
    [sym_programport_priv] = ACTIONS(2259),
    [sym_remoteui_priv] = ACTIONS(2259),
    [sym_securitycontrol_priv] = ACTIONS(2259),
    [sym_softwaremanagement_priv] = ACTIONS(2259),
    [sym_terminal_priv] = ACTIONS(2259),
    [sym_tpadmin_priv] = ACTIONS(2259),
    [sym_user1_priv] = ACTIONS(2259),
    [sym_user2_priv] = ACTIONS(2259),
    [sym_user3_priv] = ACTIONS(2259),
    [sym_user4_priv] = ACTIONS(2259),
    [sym_usermanagement_priv] = ACTIONS(2259),
    [sym_valid_account] = ACTIONS(2259),
    [sym_err_name_invalid] = ACTIONS(2259),
    [sym_err_password_invalid] = ACTIONS(2259),
    [sym_err_invalid_account] = ACTIONS(2259),
    [sym_err_no_authorization] = ACTIONS(2259),
    [sym_err_invalid_parameter] = ACTIONS(2259),
    [sym_err_user_locked_out] = ACTIONS(2259),
    [sym_err_user_account_expired] = ACTIONS(2259),
    [sym_err_general_error] = ACTIONS(2259),
    [sym_err_authentication_error] = ACTIONS(2259),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2259),
    [sym_tls_validate_certificate] = ACTIONS(2259),
    [sym____reserved___] = ACTIONS(2259),
    [sym_date] = ACTIONS(2259),
    [sym_day] = ACTIONS(2259),
    [sym_time] = ACTIONS(2259),
    [sym_ldate] = ACTIONS(2259),
    [sym_push_device] = ACTIONS(2259),
    [sym_push_channel] = ACTIONS(2259),
    [sym_push_devchan] = ACTIONS(2259),
    [sym_release_device] = ACTIONS(2259),
    [sym_release_channel] = ACTIONS(2259),
    [sym_release_devchan] = ACTIONS(2259),
    [sym_master_slot] = ACTIONS(2259),
    [sym_get_pulse_time] = ACTIONS(2259),
    [sym_get_timer] = ACTIONS(2259),
    [sym_master_sn] = ACTIONS(2259),
    [sym_system_number] = ACTIONS(2259),
    [sym_button] = ACTIONS(2259),
    [sym_dv_channel] = ACTIONS(2259),
    [sym_data] = ACTIONS(2259),
    [sym_level] = ACTIONS(2259),
    [sym_timeline] = ACTIONS(2259),
    [sym_channel] = ACTIONS(2259),
    [sym_custom] = ACTIONS(2259),
    [sym_atoi] = ACTIONS(2259),
    [sym_atol] = ACTIONS(2259),
    [sym_atof] = ACTIONS(2259),
    [sym_itoa] = ACTIONS(2259),
    [sym_itohex] = ACTIONS(2259),
    [sym_hextoi] = ACTIONS(2259),
    [sym_ftoa] = ACTIONS(2259),
    [sym_find_string] = ACTIONS(2259),
    [sym_left_string] = ACTIONS(2259),
    [sym_right_string] = ACTIONS(2259),
    [sym_mid_string] = ACTIONS(2259),
    [sym_remove_string] = ACTIONS(2259),
    [sym_get_buffer_string] = ACTIONS(2259),
    [sym_get_buffer_char] = ACTIONS(2259),
    [sym_get_multi_buffer_string] = ACTIONS(2259),
    [sym_length_string] = ACTIONS(2259),
    [sym_max_length_string] = ACTIONS(2259),
    [sym_set_length_string] = ACTIONS(2259),
    [sym_compare_string] = ACTIONS(2259),
    [sym_upper_string] = ACTIONS(2259),
    [sym_lower_string] = ACTIONS(2259),
    [sym_get_last] = ACTIONS(2259),
    [sym_date_to_day] = ACTIONS(2259),
    [sym_date_to_month] = ACTIONS(2259),
    [sym_date_to_year] = ACTIONS(2259),
    [sym_time_to_hour] = ACTIONS(2259),
    [sym_time_to_minute] = ACTIONS(2259),
    [sym_time_to_second] = ACTIONS(2259),
    [sym_day_of_week] = ACTIONS(2259),
    [sym_set_pulse_time] = ACTIONS(2259),
    [sym_set_timer] = ACTIONS(2259),
    [sym_file_open] = ACTIONS(2259),
    [sym_file_close] = ACTIONS(2259),
    [sym_file_read] = ACTIONS(2259),
    [sym_file_read_line] = ACTIONS(2259),
    [sym_file_write] = ACTIONS(2259),
    [sym_file_write_line] = ACTIONS(2259),
    [sym_file_copy] = ACTIONS(2259),
    [sym_file_delete] = ACTIONS(2259),
    [sym_file_rename] = ACTIONS(2259),
    [sym_file_dir] = ACTIONS(2259),
    [sym_file_getdir] = ACTIONS(2259),
    [sym_file_setdir] = ACTIONS(2259),
    [sym_file_create_dir] = ACTIONS(2259),
    [sym_file_remove_dir] = ACTIONS(2259),
    [sym_file_seek] = ACTIONS(2259),
    [sym_length_array] = ACTIONS(2259),
    [sym_max_length_array] = ACTIONS(2259),
    [sym_set_length_array] = ACTIONS(2259),
    [sym_ip_client_open] = ACTIONS(2259),
    [sym_ip_client_close] = ACTIONS(2259),
    [sym_ip_server_open] = ACTIONS(2259),
    [sym_ip_server_close] = ACTIONS(2259),
    [sym_ip_bound_client_open] = ACTIONS(2259),
    [sym_ip_mc_server_open] = ACTIONS(2259),
    [sym_ip_set_option] = ACTIONS(2259),
    [sym_ssh_client_open] = ACTIONS(2259),
    [sym_ssh_client_close] = ACTIONS(2259),
    [sym_tls_client_open] = ACTIONS(2259),
    [sym_tls_client_close] = ACTIONS(2259),
    [sym_do_push] = ACTIONS(2259),
    [sym_do_push_timed] = ACTIONS(2259),
    [sym_do_release] = ACTIONS(2259),
    [sym_device_id] = ACTIONS(2259),
    [sym_device_info] = ACTIONS(2259),
    [sym_device_id_string] = ACTIONS(2259),
    [sym_device_standby] = ACTIONS(2259),
    [sym_device_wake] = ACTIONS(2259),
    [sym_timeline_create] = ACTIONS(2259),
    [sym_timeline_kill] = ACTIONS(2259),
    [sym_timeline_reload] = ACTIONS(2259),
    [sym_timeline_pause] = ACTIONS(2259),
    [sym_timeline_restart] = ACTIONS(2259),
    [sym_timeline_set] = ACTIONS(2259),
    [sym_timeline_get] = ACTIONS(2259),
    [sym_timeline_active] = ACTIONS(2259),
    [sym_timeline_dynamic_id] = ACTIONS(2259),
    [sym_variable_to_string] = ACTIONS(2259),
    [sym_string_to_variable] = ACTIONS(2259),
    [sym_length_variable_to_string] = ACTIONS(2259),
    [sym_type_cast] = ACTIONS(2259),
    [sym_variable_to_xml] = ACTIONS(2259),
    [sym_xml_to_variable] = ACTIONS(2259),
    [sym_length_variable_to_xml] = ACTIONS(2259),
    [sym_redirect_string] = ACTIONS(2259),
    [sym_random_number] = ACTIONS(2259),
    [sym_get_unique_id] = ACTIONS(2259),
    [sym_get_system_number] = ACTIONS(2259),
    [sym_set_system_number] = ACTIONS(2259),
    [sym_get_serial_number] = ACTIONS(2259),
    [sym_get_master_build] = ACTIONS(2259),
    [sym_set_validation_code] = ACTIONS(2259),
    [sym_reboot] = ACTIONS(2259),
    [sym_load_duet_module] = ACTIONS(2259),
    [sym_unload_duet_module] = ACTIONS(2259),
    [sym_duet_mem_size_set] = ACTIONS(2259),
    [sym_duet_mem_size_get] = ACTIONS(2259),
    [sym_dynamic_application_device] = ACTIONS(2259),
    [sym_dynamic_polled_port] = ACTIONS(2259),
    [sym_static_port_binding] = ACTIONS(2259),
    [sym_static_ip_binding] = ACTIONS(2259),
    [sym_get_dns_list] = ACTIONS(2259),
    [sym_set_dns_list] = ACTIONS(2259),
    [sym_get_ip_address] = ACTIONS(2259),
    [sym_set_ip_address] = ACTIONS(2259),
    [sym_get_url_list] = ACTIONS(2259),
    [sym_add_url_entry] = ACTIONS(2259),
    [sym_delete_url_entry] = ACTIONS(2259),
    [sym_abs_value] = ACTIONS(2259),
    [sym_max_value] = ACTIONS(2259),
    [sym_min_value] = ACTIONS(2259),
    [sym_exp_value] = ACTIONS(2259),
    [sym_log_value] = ACTIONS(2259),
    [sym_log10_value] = ACTIONS(2259),
    [sym_power_value] = ACTIONS(2259),
    [sym_sqrt_value] = ACTIONS(2259),
    [sym_raw_be] = ACTIONS(2259),
    [sym_raw_le] = ACTIONS(2259),
    [sym_internal_queue_size_set] = ACTIONS(2259),
    [sym_internal_queue_size_get] = ACTIONS(2259),
    [sym_internal_threshold_set] = ACTIONS(2259),
    [sym_internal_threshold_get] = ACTIONS(2259),
    [sym_rebuilt_event] = ACTIONS(2259),
    [sym_do_custom_event] = ACTIONS(2259),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2259),
    [sym_clkmgr_set_clk_source] = ACTIONS(2259),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2259),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2259),
    [sym_clkmgr_get_timezone] = ACTIONS(2259),
    [sym_clkmgr_set_timezone] = ACTIONS(2259),
    [sym_clkmgr_get_resync_period] = ACTIONS(2259),
    [sym_clkmgr_set_resync_period] = ACTIONS(2259),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2259),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2259),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2259),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2259),
    [sym_clkmgr_get_timeservers] = ACTIONS(2259),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2259),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2259),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2259),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2259),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2259),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2259),
    [sym_set_log_level] = ACTIONS(2259),
    [sym_get_log_level] = ACTIONS(2259),
    [sym_amx_log] = ACTIONS(2259),
    [sym_smtp_server_config_set] = ACTIONS(2259),
    [sym_smtp_server_config_get] = ACTIONS(2259),
    [sym_smtp_send] = ACTIONS(2259),
    [sym_validate_netlinx_account] = ACTIONS(2259),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2259),
    [sym_authenticate_certificate] = ACTIONS(2259),
    [sym_audit_netlinx_session_event] = ACTIONS(2259),
    [sym_audit_netlinx_generic_event] = ACTIONS(2259),
    [sym_get_available_flash_disk_space] = ACTIONS(2259),
    [sym_get_max_flash_disk_space] = ACTIONS(2259),
    [sym_data_create_feed] = ACTIONS(2259),
    [sym_data_delete_feed] = ACTIONS(2259),
    [sym_data_publish_feed] = ACTIONS(2259),
    [sym_data_get_published_feed] = ACTIONS(2259),
    [sym_data_add_record] = ACTIONS(2259),
    [sym_data_get_event_record] = ACTIONS(2259),
    [sym__wc_data_create_feed] = ACTIONS(2259),
    [sym__wc_data_add_record] = ACTIONS(2259),
    [sym__wc_data_get_event_record] = ACTIONS(2259),
    [sym_format] = ACTIONS(2259),
    [sym_astro_clock] = ACTIONS(2259),
    [sym_set_outdoor_temperature] = ACTIONS(2259),
    [sym_set_virtual_level_count] = ACTIONS(2259),
    [sym_set_virtual_channel_count] = ACTIONS(2259),
    [sym_set_virtual_port_count] = ACTIONS(2259),
    [sym__wc_encode_int] = ACTIONS(2259),
    [sym__wc_decode_int] = ACTIONS(2259),
    [sym__wc_int] = ACTIONS(2259),
    [sym__wc_to_ch_int] = ACTIONS(2259),
    [sym__ch_to_wc_int] = ACTIONS(2259),
    [sym__wc_find_string_int] = ACTIONS(2259),
    [sym__wc_left_string_int] = ACTIONS(2259),
    [sym__wc_lower_string_int] = ACTIONS(2259),
    [sym__wc_mid_string_int] = ACTIONS(2259),
    [sym__wc_remove_string_int] = ACTIONS(2259),
    [sym__wc_right_string_int] = ACTIONS(2259),
    [sym__wc_upper_string_int] = ACTIONS(2259),
    [sym__wc_compare_string_int] = ACTIONS(2259),
    [sym__wc_get_buffer_char_int] = ACTIONS(2259),
    [sym__wc_get_buffer_string_int] = ACTIONS(2259),
    [sym__wc_concat_string_int] = ACTIONS(2259),
    [sym__wc_file_open_int] = ACTIONS(2259),
    [sym__wc_file_close_int] = ACTIONS(2259),
    [sym__wc_file_read_int] = ACTIONS(2259),
    [sym__wc_file_read_line_int] = ACTIONS(2259),
    [sym__wc_file_write_int] = ACTIONS(2259),
    [sym__wc_file_write_line_int] = ACTIONS(2259),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2261),
    [aux_sym_preproc_else_token1] = ACTIONS(2261),
    [anon_sym_COMMA] = ACTIONS(2263),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2261),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2259),
    [anon_sym_TILDE] = ACTIONS(2261),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2261),
    [anon_sym_SQUOTE] = ACTIONS(2261),
    [sym_number_literal] = ACTIONS(2261),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(632)] = {
    [sym_identifier] = ACTIONS(1555),
    [sym_if_keyword] = ACTIONS(1555),
    [sym_switch_keyword] = ACTIONS(1555),
    [sym_case_keyword] = ACTIONS(1555),
    [sym_default_keyword] = ACTIONS(1555),
    [sym_while_keyword] = ACTIONS(1555),
    [sym_for_keyword] = ACTIONS(1555),
    [sym_break_keyword] = ACTIONS(1555),
    [sym_continue_keyword] = ACTIONS(1555),
    [sym_return_keyword] = ACTIONS(1555),
    [sym_select_keyword] = ACTIONS(1555),
    [sym_active_keyword] = ACTIONS(1555),
    [aux_sym_on_keyword_token1] = ACTIONS(1555),
    [aux_sym_off_keyword_token1] = ACTIONS(1555),
    [sym_send_level_keyword] = ACTIONS(1555),
    [sym_send_string_keyword] = ACTIONS(1555),
    [sym_send_command_keyword] = ACTIONS(1555),
    [sym_clear_buffer_keyword] = ACTIONS(1555),
    [sym_create_buffer_keyword] = ACTIONS(1555),
    [sym_create_multi_buffer_keyword] = ACTIONS(1555),
    [sym_call_keyword] = ACTIONS(1555),
    [sym_system_call_keyword] = ACTIONS(1555),
    [sym_devchan_to_keyword] = ACTIONS(1555),
    [sym_devchan_min_to_keyword] = ACTIONS(1555),
    [sym_devchan_total_off_keyword] = ACTIONS(1555),
    [sym_devchan_pulse_keyword] = ACTIONS(1555),
    [sym_wait_keyword] = ACTIONS(1555),
    [sym_cancel_wait_keyword] = ACTIONS(1555),
    [sym_wait_until_keyword] = ACTIONS(1555),
    [sym_cancel_wait_until_keyword] = ACTIONS(1555),
    [sym_cancel_all_wait_keyword] = ACTIONS(1555),
    [sym_cancel_all_wait_until_keyword] = ACTIONS(1555),
    [sym_not] = ACTIONS(1555),
    [sym_bnot] = ACTIONS(1555),
    [sym___file__] = ACTIONS(1555),
    [sym___date__] = ACTIONS(1555),
    [sym___ldate__] = ACTIONS(1555),
    [sym___time__] = ACTIONS(1555),
    [sym___line__] = ACTIONS(1555),
    [sym___name__] = ACTIONS(1555),
    [sym___netlinx__] = ACTIONS(1555),
    [sym_netlinx_axi_version] = ACTIONS(1555),
    [sym_url_flg_tcp] = ACTIONS(1555),
    [sym_url_flg_accinfopresent] = ACTIONS(1555),
    [sym_url_flg_temp] = ACTIONS(1555),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(1555),
    [sym_url_flg_stat_mask] = ACTIONS(1555),
    [sym_url_flg_stat_lookup] = ACTIONS(1555),
    [sym_url_flg_stat_connecting] = ACTIONS(1555),
    [sym_url_flg_stat_waiting] = ACTIONS(1555),
    [sym_url_flg_stat_connected] = ACTIONS(1555),
    [sym_ip_addr_flg_dhcp] = ACTIONS(1555),
    [sym_first_virtual_device] = ACTIONS(1555),
    [sym_first_local_port] = ACTIONS(1555),
    [sym_dynamic_virtual_device] = ACTIONS(1555),
    [sym_timeline_once] = ACTIONS(1555),
    [sym_timeline_repeat] = ACTIONS(1555),
    [sym_timeline_absolute] = ACTIONS(1555),
    [sym_timeline_relative] = ACTIONS(1555),
    [sym_do_push_timed_infinite] = ACTIONS(1555),
    [sym_source_type_no_address] = ACTIONS(1555),
    [sym_source_type_neuron_id] = ACTIONS(1555),
    [sym_source_type_ip_address] = ACTIONS(1555),
    [sym_source_type_axlink] = ACTIONS(1555),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(1555),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(1555),
    [sym_source_type_ip_socket_address] = ACTIONS(1555),
    [sym_source_type_rs232] = ACTIONS(1555),
    [sym_source_type_internal] = ACTIONS(1555),
    [sym_source_type_mac_address] = ACTIONS(1555),
    [sym_source_type_ipv4_port] = ACTIONS(1555),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(1555),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(1555),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(1555),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(1555),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(1555),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(1555),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(1555),
    [sym_true] = ACTIONS(1555),
    [sym_false] = ACTIONS(1555),
    [sym_file_read_only] = ACTIONS(1555),
    [sym_file_rw_new] = ACTIONS(1555),
    [sym_file_rw_append] = ACTIONS(1555),
    [sym_ip_tcp] = ACTIONS(1555),
    [sym_ip_udp] = ACTIONS(1555),
    [sym_ip_udp_2way] = ACTIONS(1555),
    [sym_xml_encode_types] = ACTIONS(1555),
    [sym_xml_encode_char_as_list] = ACTIONS(1555),
    [sym_xml_encode_le] = ACTIONS(1555),
    [sym_xml_decode_types] = ACTIONS(1555),
    [sym_xml_decode_no_preserve] = ACTIONS(1555),
    [sym_duet_dev_type_display_device] = ACTIONS(1555),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(1555),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(1555),
    [sym_duet_dev_type_io_device] = ACTIONS(1555),
    [sym_duet_dev_type_relay_device] = ACTIONS(1555),
    [sym_duet_dev_type_ups] = ACTIONS(1555),
    [sym_duet_dev_type_amplifier] = ACTIONS(1555),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(1555),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(1555),
    [sym_duet_dev_type_audio_processor] = ACTIONS(1555),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(1555),
    [sym_duet_dev_type_camera] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(1555),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(1555),
    [sym_duet_dev_type_dss] = ACTIONS(1555),
    [sym_duet_dev_type_dvr] = ACTIONS(1555),
    [sym_duet_dev_type_disc_device] = ACTIONS(1555),
    [sym_duet_dev_type_document_camera] = ACTIONS(1555),
    [sym_duet_dev_type_audio_tape] = ACTIONS(1555),
    [sym_duet_dev_type_hvac] = ACTIONS(1555),
    [sym_duet_dev_type_keypad] = ACTIONS(1555),
    [sym_duet_dev_type_light] = ACTIONS(1555),
    [sym_duet_dev_type_monitor] = ACTIONS(1555),
    [sym_duet_dev_type_motor] = ACTIONS(1555),
    [sym_duet_dev_type_multi_window] = ACTIONS(1555),
    [sym_duet_dev_type_pool_spa] = ACTIONS(1555),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(1555),
    [sym_duet_dev_type_receiver] = ACTIONS(1555),
    [sym_duet_dev_type_security_system] = ACTIONS(1555),
    [sym_duet_dev_type_sensor_device] = ACTIONS(1555),
    [sym_duet_dev_type_settop_box] = ACTIONS(1555),
    [sym_duet_dev_type_slide_projector] = ACTIONS(1555),
    [sym_duet_dev_type_switcher] = ACTIONS(1555),
    [sym_duet_dev_type_text_keypad] = ACTIONS(1555),
    [sym_duet_dev_type_tv] = ACTIONS(1555),
    [sym_duet_dev_type_utility] = ACTIONS(1555),
    [sym_duet_dev_type_vcr] = ACTIONS(1555),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(1555),
    [sym_duet_dev_type_video_processor] = ACTIONS(1555),
    [sym_duet_dev_type_video_projector] = ACTIONS(1555),
    [sym_duet_dev_type_video_wall] = ACTIONS(1555),
    [sym_duet_dev_type_volume_controller] = ACTIONS(1555),
    [sym_duet_dev_type_weather] = ACTIONS(1555),
    [sym_duet_dev_not_polled] = ACTIONS(1555),
    [sym_duet_dev_polled] = ACTIONS(1555),
    [sym_clkmgr_mode_network] = ACTIONS(1555),
    [sym_clkmgr_mode_standalone] = ACTIONS(1555),
    [sym_normal_standby] = ACTIONS(1555),
    [sym_normal_wake] = ACTIONS(1555),
    [sym_ip_multicast_ttl_option] = ACTIONS(1555),
    [sym_ip_ttl_subnet] = ACTIONS(1555),
    [sym_ip_ttl_site] = ACTIONS(1555),
    [sym_ip_ttl_region] = ACTIONS(1555),
    [sym_ip_ttl_continent] = ACTIONS(1555),
    [sym_ip_tcp_nodelay_option] = ACTIONS(1555),
    [sym_ip_nodelay_on] = ACTIONS(1555),
    [sym_ip_nodelay_off] = ACTIONS(1555),
    [sym_amx_error] = ACTIONS(1555),
    [sym_amx_warning] = ACTIONS(1555),
    [sym_amx_info] = ACTIONS(1555),
    [sym_amx_debug] = ACTIONS(1555),
    [sym_netlinx_login_success] = ACTIONS(1555),
    [sym_netlinx_login_fail] = ACTIONS(1555),
    [sym_netlinx_logout] = ACTIONS(1555),
    [sym_null_str] = ACTIONS(1555),
    [sym_smtp_address] = ACTIONS(1555),
    [sym_smtp_port_number] = ACTIONS(1555),
    [sym_smtp_username] = ACTIONS(1555),
    [sym_smtp_password] = ACTIONS(1555),
    [sym_smtp_require_tls] = ACTIONS(1555),
    [sym_smtp_from] = ACTIONS(1555),
    [sym_smtp_tls_true] = ACTIONS(1555),
    [sym_smtp_tls_false] = ACTIONS(1555),
    [sym_listview_on_row_select_event] = ACTIONS(1555),
    [sym_data_structure_unknown] = ACTIONS(1555),
    [sym_data_structure_datafeed] = ACTIONS(1555),
    [sym_data_structure_datarecordset] = ACTIONS(1555),
    [sym_data_structure_datarecord] = ACTIONS(1555),
    [sym_data_structure_datafield] = ACTIONS(1555),
    [sym_data_max_content_field_count] = ACTIONS(1555),
    [sym_data_max_metadata_field_count] = ACTIONS(1555),
    [sym_data_max_id_length] = ACTIONS(1555),
    [sym_data_max_name_length] = ACTIONS(1555),
    [sym_data_max_description_length] = ACTIONS(1555),
    [sym_data_max_type_length] = ACTIONS(1555),
    [sym_data_max_format_length] = ACTIONS(1555),
    [sym_data_max_source_length] = ACTIONS(1555),
    [sym_data_max_value_length] = ACTIONS(1555),
    [sym_data_max_label_length] = ACTIONS(1555),
    [sym_data_max_url_length] = ACTIONS(1555),
    [sym_max_last_login_info_length] = ACTIONS(1555),
    [sym_data_type_unknown] = ACTIONS(1555),
    [sym_data_type_string] = ACTIONS(1555),
    [sym_data_type_datetime] = ACTIONS(1555),
    [sym_data_type_date] = ACTIONS(1555),
    [sym_data_type_time] = ACTIONS(1555),
    [sym_data_type_image] = ACTIONS(1555),
    [sym_data_format_url] = ACTIONS(1555),
    [sym_data_format_phone] = ACTIONS(1555),
    [sym_data_format_email] = ACTIONS(1555),
    [sym_data_format_iso8601] = ACTIONS(1555),
    [sym_auditlog_priv] = ACTIONS(1555),
    [sym_configuration_priv] = ACTIONS(1555),
    [sym_deviceconfig_priv] = ACTIONS(1555),
    [sym_ftp_priv] = ACTIONS(1555),
    [sym_http_priv] = ACTIONS(1555),
    [sym_networkconfig_priv] = ACTIONS(1555),
    [sym_programport_priv] = ACTIONS(1555),
    [sym_remoteui_priv] = ACTIONS(1555),
    [sym_securitycontrol_priv] = ACTIONS(1555),
    [sym_softwaremanagement_priv] = ACTIONS(1555),
    [sym_terminal_priv] = ACTIONS(1555),
    [sym_tpadmin_priv] = ACTIONS(1555),
    [sym_user1_priv] = ACTIONS(1555),
    [sym_user2_priv] = ACTIONS(1555),
    [sym_user3_priv] = ACTIONS(1555),
    [sym_user4_priv] = ACTIONS(1555),
    [sym_usermanagement_priv] = ACTIONS(1555),
    [sym_valid_account] = ACTIONS(1555),
    [sym_err_name_invalid] = ACTIONS(1555),
    [sym_err_password_invalid] = ACTIONS(1555),
    [sym_err_invalid_account] = ACTIONS(1555),
    [sym_err_no_authorization] = ACTIONS(1555),
    [sym_err_invalid_parameter] = ACTIONS(1555),
    [sym_err_user_locked_out] = ACTIONS(1555),
    [sym_err_user_account_expired] = ACTIONS(1555),
    [sym_err_general_error] = ACTIONS(1555),
    [sym_err_authentication_error] = ACTIONS(1555),
    [sym_tls_ignore_certificate_errors] = ACTIONS(1555),
    [sym_tls_validate_certificate] = ACTIONS(1555),
    [sym____reserved___] = ACTIONS(1555),
    [sym_date] = ACTIONS(1555),
    [sym_day] = ACTIONS(1555),
    [sym_time] = ACTIONS(1555),
    [sym_ldate] = ACTIONS(1555),
    [sym_push_device] = ACTIONS(1555),
    [sym_push_channel] = ACTIONS(1555),
    [sym_push_devchan] = ACTIONS(1555),
    [sym_release_device] = ACTIONS(1555),
    [sym_release_channel] = ACTIONS(1555),
    [sym_release_devchan] = ACTIONS(1555),
    [sym_master_slot] = ACTIONS(1555),
    [sym_get_pulse_time] = ACTIONS(1555),
    [sym_get_timer] = ACTIONS(1555),
    [sym_master_sn] = ACTIONS(1555),
    [sym_system_number] = ACTIONS(1555),
    [sym_button] = ACTIONS(1555),
    [sym_dv_channel] = ACTIONS(1555),
    [sym_data] = ACTIONS(1555),
    [sym_level] = ACTIONS(1555),
    [sym_timeline] = ACTIONS(1555),
    [sym_channel] = ACTIONS(1555),
    [sym_custom] = ACTIONS(1555),
    [sym_atoi] = ACTIONS(1555),
    [sym_atol] = ACTIONS(1555),
    [sym_atof] = ACTIONS(1555),
    [sym_itoa] = ACTIONS(1555),
    [sym_itohex] = ACTIONS(1555),
    [sym_hextoi] = ACTIONS(1555),
    [sym_ftoa] = ACTIONS(1555),
    [sym_find_string] = ACTIONS(1555),
    [sym_left_string] = ACTIONS(1555),
    [sym_right_string] = ACTIONS(1555),
    [sym_mid_string] = ACTIONS(1555),
    [sym_remove_string] = ACTIONS(1555),
    [sym_get_buffer_string] = ACTIONS(1555),
    [sym_get_buffer_char] = ACTIONS(1555),
    [sym_get_multi_buffer_string] = ACTIONS(1555),
    [sym_length_string] = ACTIONS(1555),
    [sym_max_length_string] = ACTIONS(1555),
    [sym_set_length_string] = ACTIONS(1555),
    [sym_compare_string] = ACTIONS(1555),
    [sym_upper_string] = ACTIONS(1555),
    [sym_lower_string] = ACTIONS(1555),
    [sym_get_last] = ACTIONS(1555),
    [sym_date_to_day] = ACTIONS(1555),
    [sym_date_to_month] = ACTIONS(1555),
    [sym_date_to_year] = ACTIONS(1555),
    [sym_time_to_hour] = ACTIONS(1555),
    [sym_time_to_minute] = ACTIONS(1555),
    [sym_time_to_second] = ACTIONS(1555),
    [sym_day_of_week] = ACTIONS(1555),
    [sym_set_pulse_time] = ACTIONS(1555),
    [sym_set_timer] = ACTIONS(1555),
    [sym_file_open] = ACTIONS(1555),
    [sym_file_close] = ACTIONS(1555),
    [sym_file_read] = ACTIONS(1555),
    [sym_file_read_line] = ACTIONS(1555),
    [sym_file_write] = ACTIONS(1555),
    [sym_file_write_line] = ACTIONS(1555),
    [sym_file_copy] = ACTIONS(1555),
    [sym_file_delete] = ACTIONS(1555),
    [sym_file_rename] = ACTIONS(1555),
    [sym_file_dir] = ACTIONS(1555),
    [sym_file_getdir] = ACTIONS(1555),
    [sym_file_setdir] = ACTIONS(1555),
    [sym_file_create_dir] = ACTIONS(1555),
    [sym_file_remove_dir] = ACTIONS(1555),
    [sym_file_seek] = ACTIONS(1555),
    [sym_length_array] = ACTIONS(1555),
    [sym_max_length_array] = ACTIONS(1555),
    [sym_set_length_array] = ACTIONS(1555),
    [sym_ip_client_open] = ACTIONS(1555),
    [sym_ip_client_close] = ACTIONS(1555),
    [sym_ip_server_open] = ACTIONS(1555),
    [sym_ip_server_close] = ACTIONS(1555),
    [sym_ip_bound_client_open] = ACTIONS(1555),
    [sym_ip_mc_server_open] = ACTIONS(1555),
    [sym_ip_set_option] = ACTIONS(1555),
    [sym_ssh_client_open] = ACTIONS(1555),
    [sym_ssh_client_close] = ACTIONS(1555),
    [sym_tls_client_open] = ACTIONS(1555),
    [sym_tls_client_close] = ACTIONS(1555),
    [sym_do_push] = ACTIONS(1555),
    [sym_do_push_timed] = ACTIONS(1555),
    [sym_do_release] = ACTIONS(1555),
    [sym_device_id] = ACTIONS(1555),
    [sym_device_info] = ACTIONS(1555),
    [sym_device_id_string] = ACTIONS(1555),
    [sym_device_standby] = ACTIONS(1555),
    [sym_device_wake] = ACTIONS(1555),
    [sym_timeline_create] = ACTIONS(1555),
    [sym_timeline_kill] = ACTIONS(1555),
    [sym_timeline_reload] = ACTIONS(1555),
    [sym_timeline_pause] = ACTIONS(1555),
    [sym_timeline_restart] = ACTIONS(1555),
    [sym_timeline_set] = ACTIONS(1555),
    [sym_timeline_get] = ACTIONS(1555),
    [sym_timeline_active] = ACTIONS(1555),
    [sym_timeline_dynamic_id] = ACTIONS(1555),
    [sym_variable_to_string] = ACTIONS(1555),
    [sym_string_to_variable] = ACTIONS(1555),
    [sym_length_variable_to_string] = ACTIONS(1555),
    [sym_type_cast] = ACTIONS(1555),
    [sym_variable_to_xml] = ACTIONS(1555),
    [sym_xml_to_variable] = ACTIONS(1555),
    [sym_length_variable_to_xml] = ACTIONS(1555),
    [sym_redirect_string] = ACTIONS(1555),
    [sym_random_number] = ACTIONS(1555),
    [sym_get_unique_id] = ACTIONS(1555),
    [sym_get_system_number] = ACTIONS(1555),
    [sym_set_system_number] = ACTIONS(1555),
    [sym_get_serial_number] = ACTIONS(1555),
    [sym_get_master_build] = ACTIONS(1555),
    [sym_set_validation_code] = ACTIONS(1555),
    [sym_reboot] = ACTIONS(1555),
    [sym_load_duet_module] = ACTIONS(1555),
    [sym_unload_duet_module] = ACTIONS(1555),
    [sym_duet_mem_size_set] = ACTIONS(1555),
    [sym_duet_mem_size_get] = ACTIONS(1555),
    [sym_dynamic_application_device] = ACTIONS(1555),
    [sym_dynamic_polled_port] = ACTIONS(1555),
    [sym_static_port_binding] = ACTIONS(1555),
    [sym_static_ip_binding] = ACTIONS(1555),
    [sym_get_dns_list] = ACTIONS(1555),
    [sym_set_dns_list] = ACTIONS(1555),
    [sym_get_ip_address] = ACTIONS(1555),
    [sym_set_ip_address] = ACTIONS(1555),
    [sym_get_url_list] = ACTIONS(1555),
    [sym_add_url_entry] = ACTIONS(1555),
    [sym_delete_url_entry] = ACTIONS(1555),
    [sym_abs_value] = ACTIONS(1555),
    [sym_max_value] = ACTIONS(1555),
    [sym_min_value] = ACTIONS(1555),
    [sym_exp_value] = ACTIONS(1555),
    [sym_log_value] = ACTIONS(1555),
    [sym_log10_value] = ACTIONS(1555),
    [sym_power_value] = ACTIONS(1555),
    [sym_sqrt_value] = ACTIONS(1555),
    [sym_raw_be] = ACTIONS(1555),
    [sym_raw_le] = ACTIONS(1555),
    [sym_internal_queue_size_set] = ACTIONS(1555),
    [sym_internal_queue_size_get] = ACTIONS(1555),
    [sym_internal_threshold_set] = ACTIONS(1555),
    [sym_internal_threshold_get] = ACTIONS(1555),
    [sym_rebuilt_event] = ACTIONS(1555),
    [sym_do_custom_event] = ACTIONS(1555),
    [sym_clkmgr_is_network_sourced] = ACTIONS(1555),
    [sym_clkmgr_set_clk_source] = ACTIONS(1555),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(1555),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(1555),
    [sym_clkmgr_get_timezone] = ACTIONS(1555),
    [sym_clkmgr_set_timezone] = ACTIONS(1555),
    [sym_clkmgr_get_resync_period] = ACTIONS(1555),
    [sym_clkmgr_set_resync_period] = ACTIONS(1555),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(1555),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(1555),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(1555),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(1555),
    [sym_clkmgr_get_timeservers] = ACTIONS(1555),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(1555),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(1555),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(1555),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(1555),
    [sym_set_log_level] = ACTIONS(1555),
    [sym_get_log_level] = ACTIONS(1555),
    [sym_amx_log] = ACTIONS(1555),
    [sym_smtp_server_config_set] = ACTIONS(1555),
    [sym_smtp_server_config_get] = ACTIONS(1555),
    [sym_smtp_send] = ACTIONS(1555),
    [sym_validate_netlinx_account] = ACTIONS(1555),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(1555),
    [sym_authenticate_certificate] = ACTIONS(1555),
    [sym_audit_netlinx_session_event] = ACTIONS(1555),
    [sym_audit_netlinx_generic_event] = ACTIONS(1555),
    [sym_get_available_flash_disk_space] = ACTIONS(1555),
    [sym_get_max_flash_disk_space] = ACTIONS(1555),
    [sym_data_create_feed] = ACTIONS(1555),
    [sym_data_delete_feed] = ACTIONS(1555),
    [sym_data_publish_feed] = ACTIONS(1555),
    [sym_data_get_published_feed] = ACTIONS(1555),
    [sym_data_add_record] = ACTIONS(1555),
    [sym_data_get_event_record] = ACTIONS(1555),
    [sym__wc_data_create_feed] = ACTIONS(1555),
    [sym__wc_data_add_record] = ACTIONS(1555),
    [sym__wc_data_get_event_record] = ACTIONS(1555),
    [sym_format] = ACTIONS(1555),
    [sym_astro_clock] = ACTIONS(1555),
    [sym_set_outdoor_temperature] = ACTIONS(1555),
    [sym_set_virtual_level_count] = ACTIONS(1555),
    [sym_set_virtual_channel_count] = ACTIONS(1555),
    [sym_set_virtual_port_count] = ACTIONS(1555),
    [sym__wc_encode_int] = ACTIONS(1555),
    [sym__wc_decode_int] = ACTIONS(1555),
    [sym__wc_int] = ACTIONS(1555),
    [sym__wc_to_ch_int] = ACTIONS(1555),
    [sym__ch_to_wc_int] = ACTIONS(1555),
    [sym__wc_find_string_int] = ACTIONS(1555),
    [sym__wc_left_string_int] = ACTIONS(1555),
    [sym__wc_lower_string_int] = ACTIONS(1555),
    [sym__wc_mid_string_int] = ACTIONS(1555),
    [sym__wc_remove_string_int] = ACTIONS(1555),
    [sym__wc_right_string_int] = ACTIONS(1555),
    [sym__wc_upper_string_int] = ACTIONS(1555),
    [sym__wc_compare_string_int] = ACTIONS(1555),
    [sym__wc_get_buffer_char_int] = ACTIONS(1555),
    [sym__wc_get_buffer_string_int] = ACTIONS(1555),
    [sym__wc_concat_string_int] = ACTIONS(1555),
    [sym__wc_file_open_int] = ACTIONS(1555),
    [sym__wc_file_close_int] = ACTIONS(1555),
    [sym__wc_file_read_int] = ACTIONS(1555),
    [sym__wc_file_read_line_int] = ACTIONS(1555),
    [sym__wc_file_write_int] = ACTIONS(1555),
    [sym__wc_file_write_line_int] = ACTIONS(1555),
    [anon_sym_LBRACK] = ACTIONS(1557),
    [anon_sym_LBRACE] = ACTIONS(1557),
    [anon_sym_COLON] = ACTIONS(1557),
    [anon_sym_LPAREN] = ACTIONS(1555),
    [anon_sym_BANG] = ACTIONS(1557),
    [anon_sym_TILDE] = ACTIONS(1557),
    [anon_sym_DASH] = ACTIONS(1555),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_DASH_DASH] = ACTIONS(1557),
    [anon_sym_PLUS_PLUS] = ACTIONS(1557),
    [anon_sym_DQUOTE] = ACTIONS(1557),
    [anon_sym_SQUOTE] = ACTIONS(1557),
    [sym_number_literal] = ACTIONS(1557),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(633)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2266),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2266),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2266),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2266),
    [sym___date__] = ACTIONS(2266),
    [sym___ldate__] = ACTIONS(2266),
    [sym___time__] = ACTIONS(2266),
    [sym___line__] = ACTIONS(2266),
    [sym___name__] = ACTIONS(2266),
    [sym___netlinx__] = ACTIONS(2266),
    [sym_netlinx_axi_version] = ACTIONS(2266),
    [sym_url_flg_tcp] = ACTIONS(2266),
    [sym_url_flg_accinfopresent] = ACTIONS(2266),
    [sym_url_flg_temp] = ACTIONS(2266),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2266),
    [sym_url_flg_stat_mask] = ACTIONS(2266),
    [sym_url_flg_stat_lookup] = ACTIONS(2266),
    [sym_url_flg_stat_connecting] = ACTIONS(2266),
    [sym_url_flg_stat_waiting] = ACTIONS(2266),
    [sym_url_flg_stat_connected] = ACTIONS(2266),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2266),
    [sym_first_virtual_device] = ACTIONS(2266),
    [sym_first_local_port] = ACTIONS(2266),
    [sym_dynamic_virtual_device] = ACTIONS(2266),
    [sym_timeline_once] = ACTIONS(2266),
    [sym_timeline_repeat] = ACTIONS(2266),
    [sym_timeline_absolute] = ACTIONS(2266),
    [sym_timeline_relative] = ACTIONS(2266),
    [sym_do_push_timed_infinite] = ACTIONS(2266),
    [sym_source_type_no_address] = ACTIONS(2266),
    [sym_source_type_neuron_id] = ACTIONS(2266),
    [sym_source_type_ip_address] = ACTIONS(2266),
    [sym_source_type_axlink] = ACTIONS(2266),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2266),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2266),
    [sym_source_type_ip_socket_address] = ACTIONS(2266),
    [sym_source_type_rs232] = ACTIONS(2266),
    [sym_source_type_internal] = ACTIONS(2266),
    [sym_source_type_mac_address] = ACTIONS(2266),
    [sym_source_type_ipv4_port] = ACTIONS(2266),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2266),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2266),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2266),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2266),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2266),
    [sym_true] = ACTIONS(2266),
    [sym_false] = ACTIONS(2266),
    [sym_file_read_only] = ACTIONS(2266),
    [sym_file_rw_new] = ACTIONS(2266),
    [sym_file_rw_append] = ACTIONS(2266),
    [sym_ip_tcp] = ACTIONS(2266),
    [sym_ip_udp] = ACTIONS(2266),
    [sym_ip_udp_2way] = ACTIONS(2266),
    [sym_xml_encode_types] = ACTIONS(2266),
    [sym_xml_encode_char_as_list] = ACTIONS(2266),
    [sym_xml_encode_le] = ACTIONS(2266),
    [sym_xml_decode_types] = ACTIONS(2266),
    [sym_xml_decode_no_preserve] = ACTIONS(2266),
    [sym_duet_dev_type_display_device] = ACTIONS(2266),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2266),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2266),
    [sym_duet_dev_type_io_device] = ACTIONS(2266),
    [sym_duet_dev_type_relay_device] = ACTIONS(2266),
    [sym_duet_dev_type_ups] = ACTIONS(2266),
    [sym_duet_dev_type_amplifier] = ACTIONS(2266),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2266),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2266),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2266),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2266),
    [sym_duet_dev_type_camera] = ACTIONS(2266),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2266),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2266),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2266),
    [sym_duet_dev_type_dss] = ACTIONS(2266),
    [sym_duet_dev_type_dvr] = ACTIONS(2266),
    [sym_duet_dev_type_disc_device] = ACTIONS(2266),
    [sym_duet_dev_type_document_camera] = ACTIONS(2266),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2266),
    [sym_duet_dev_type_hvac] = ACTIONS(2266),
    [sym_duet_dev_type_keypad] = ACTIONS(2266),
    [sym_duet_dev_type_light] = ACTIONS(2266),
    [sym_duet_dev_type_monitor] = ACTIONS(2266),
    [sym_duet_dev_type_motor] = ACTIONS(2266),
    [sym_duet_dev_type_multi_window] = ACTIONS(2266),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2266),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2266),
    [sym_duet_dev_type_receiver] = ACTIONS(2266),
    [sym_duet_dev_type_security_system] = ACTIONS(2266),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2266),
    [sym_duet_dev_type_settop_box] = ACTIONS(2266),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2266),
    [sym_duet_dev_type_switcher] = ACTIONS(2266),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2266),
    [sym_duet_dev_type_tv] = ACTIONS(2266),
    [sym_duet_dev_type_utility] = ACTIONS(2266),
    [sym_duet_dev_type_vcr] = ACTIONS(2266),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2266),
    [sym_duet_dev_type_video_processor] = ACTIONS(2266),
    [sym_duet_dev_type_video_projector] = ACTIONS(2266),
    [sym_duet_dev_type_video_wall] = ACTIONS(2266),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2266),
    [sym_duet_dev_type_weather] = ACTIONS(2266),
    [sym_duet_dev_not_polled] = ACTIONS(2266),
    [sym_duet_dev_polled] = ACTIONS(2266),
    [sym_clkmgr_mode_network] = ACTIONS(2266),
    [sym_clkmgr_mode_standalone] = ACTIONS(2266),
    [sym_normal_standby] = ACTIONS(2266),
    [sym_normal_wake] = ACTIONS(2266),
    [sym_ip_multicast_ttl_option] = ACTIONS(2266),
    [sym_ip_ttl_subnet] = ACTIONS(2266),
    [sym_ip_ttl_site] = ACTIONS(2266),
    [sym_ip_ttl_region] = ACTIONS(2266),
    [sym_ip_ttl_continent] = ACTIONS(2266),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2266),
    [sym_ip_nodelay_on] = ACTIONS(2266),
    [sym_ip_nodelay_off] = ACTIONS(2266),
    [sym_amx_error] = ACTIONS(2266),
    [sym_amx_warning] = ACTIONS(2266),
    [sym_amx_info] = ACTIONS(2266),
    [sym_amx_debug] = ACTIONS(2266),
    [sym_netlinx_login_success] = ACTIONS(2266),
    [sym_netlinx_login_fail] = ACTIONS(2266),
    [sym_netlinx_logout] = ACTIONS(2266),
    [sym_null_str] = ACTIONS(2266),
    [sym_smtp_address] = ACTIONS(2266),
    [sym_smtp_port_number] = ACTIONS(2266),
    [sym_smtp_username] = ACTIONS(2266),
    [sym_smtp_password] = ACTIONS(2266),
    [sym_smtp_require_tls] = ACTIONS(2266),
    [sym_smtp_from] = ACTIONS(2266),
    [sym_smtp_tls_true] = ACTIONS(2266),
    [sym_smtp_tls_false] = ACTIONS(2266),
    [sym_listview_on_row_select_event] = ACTIONS(2266),
    [sym_data_structure_unknown] = ACTIONS(2266),
    [sym_data_structure_datafeed] = ACTIONS(2266),
    [sym_data_structure_datarecordset] = ACTIONS(2266),
    [sym_data_structure_datarecord] = ACTIONS(2266),
    [sym_data_structure_datafield] = ACTIONS(2266),
    [sym_data_max_content_field_count] = ACTIONS(2266),
    [sym_data_max_metadata_field_count] = ACTIONS(2266),
    [sym_data_max_id_length] = ACTIONS(2266),
    [sym_data_max_name_length] = ACTIONS(2266),
    [sym_data_max_description_length] = ACTIONS(2266),
    [sym_data_max_type_length] = ACTIONS(2266),
    [sym_data_max_format_length] = ACTIONS(2266),
    [sym_data_max_source_length] = ACTIONS(2266),
    [sym_data_max_value_length] = ACTIONS(2266),
    [sym_data_max_label_length] = ACTIONS(2266),
    [sym_data_max_url_length] = ACTIONS(2266),
    [sym_max_last_login_info_length] = ACTIONS(2266),
    [sym_data_type_unknown] = ACTIONS(2266),
    [sym_data_type_string] = ACTIONS(2266),
    [sym_data_type_datetime] = ACTIONS(2266),
    [sym_data_type_date] = ACTIONS(2266),
    [sym_data_type_time] = ACTIONS(2266),
    [sym_data_type_image] = ACTIONS(2266),
    [sym_data_format_url] = ACTIONS(2266),
    [sym_data_format_phone] = ACTIONS(2266),
    [sym_data_format_email] = ACTIONS(2266),
    [sym_data_format_iso8601] = ACTIONS(2266),
    [sym_auditlog_priv] = ACTIONS(2266),
    [sym_configuration_priv] = ACTIONS(2266),
    [sym_deviceconfig_priv] = ACTIONS(2266),
    [sym_ftp_priv] = ACTIONS(2266),
    [sym_http_priv] = ACTIONS(2266),
    [sym_networkconfig_priv] = ACTIONS(2266),
    [sym_programport_priv] = ACTIONS(2266),
    [sym_remoteui_priv] = ACTIONS(2266),
    [sym_securitycontrol_priv] = ACTIONS(2266),
    [sym_softwaremanagement_priv] = ACTIONS(2266),
    [sym_terminal_priv] = ACTIONS(2266),
    [sym_tpadmin_priv] = ACTIONS(2266),
    [sym_user1_priv] = ACTIONS(2266),
    [sym_user2_priv] = ACTIONS(2266),
    [sym_user3_priv] = ACTIONS(2266),
    [sym_user4_priv] = ACTIONS(2266),
    [sym_usermanagement_priv] = ACTIONS(2266),
    [sym_valid_account] = ACTIONS(2266),
    [sym_err_name_invalid] = ACTIONS(2266),
    [sym_err_password_invalid] = ACTIONS(2266),
    [sym_err_invalid_account] = ACTIONS(2266),
    [sym_err_no_authorization] = ACTIONS(2266),
    [sym_err_invalid_parameter] = ACTIONS(2266),
    [sym_err_user_locked_out] = ACTIONS(2266),
    [sym_err_user_account_expired] = ACTIONS(2266),
    [sym_err_general_error] = ACTIONS(2266),
    [sym_err_authentication_error] = ACTIONS(2266),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2266),
    [sym_tls_validate_certificate] = ACTIONS(2266),
    [sym____reserved___] = ACTIONS(2266),
    [sym_date] = ACTIONS(2266),
    [sym_day] = ACTIONS(2266),
    [sym_time] = ACTIONS(2266),
    [sym_ldate] = ACTIONS(2266),
    [sym_push_device] = ACTIONS(2266),
    [sym_push_channel] = ACTIONS(2266),
    [sym_push_devchan] = ACTIONS(2266),
    [sym_release_device] = ACTIONS(2266),
    [sym_release_channel] = ACTIONS(2266),
    [sym_release_devchan] = ACTIONS(2266),
    [sym_master_slot] = ACTIONS(2266),
    [sym_get_pulse_time] = ACTIONS(2266),
    [sym_get_timer] = ACTIONS(2266),
    [sym_master_sn] = ACTIONS(2266),
    [sym_system_number] = ACTIONS(2266),
    [sym_button] = ACTIONS(2266),
    [sym_dv_channel] = ACTIONS(2266),
    [sym_data] = ACTIONS(2266),
    [sym_level] = ACTIONS(2266),
    [sym_timeline] = ACTIONS(2266),
    [sym_channel] = ACTIONS(2266),
    [sym_custom] = ACTIONS(2266),
    [sym_atoi] = ACTIONS(2266),
    [sym_atol] = ACTIONS(2266),
    [sym_atof] = ACTIONS(2266),
    [sym_itoa] = ACTIONS(2266),
    [sym_itohex] = ACTIONS(2266),
    [sym_hextoi] = ACTIONS(2266),
    [sym_ftoa] = ACTIONS(2266),
    [sym_find_string] = ACTIONS(2266),
    [sym_left_string] = ACTIONS(2266),
    [sym_right_string] = ACTIONS(2266),
    [sym_mid_string] = ACTIONS(2266),
    [sym_remove_string] = ACTIONS(2266),
    [sym_get_buffer_string] = ACTIONS(2266),
    [sym_get_buffer_char] = ACTIONS(2266),
    [sym_get_multi_buffer_string] = ACTIONS(2266),
    [sym_length_string] = ACTIONS(2266),
    [sym_max_length_string] = ACTIONS(2266),
    [sym_set_length_string] = ACTIONS(2266),
    [sym_compare_string] = ACTIONS(2266),
    [sym_upper_string] = ACTIONS(2266),
    [sym_lower_string] = ACTIONS(2266),
    [sym_get_last] = ACTIONS(2266),
    [sym_date_to_day] = ACTIONS(2266),
    [sym_date_to_month] = ACTIONS(2266),
    [sym_date_to_year] = ACTIONS(2266),
    [sym_time_to_hour] = ACTIONS(2266),
    [sym_time_to_minute] = ACTIONS(2266),
    [sym_time_to_second] = ACTIONS(2266),
    [sym_day_of_week] = ACTIONS(2266),
    [sym_set_pulse_time] = ACTIONS(2266),
    [sym_set_timer] = ACTIONS(2266),
    [sym_file_open] = ACTIONS(2266),
    [sym_file_close] = ACTIONS(2266),
    [sym_file_read] = ACTIONS(2266),
    [sym_file_read_line] = ACTIONS(2266),
    [sym_file_write] = ACTIONS(2266),
    [sym_file_write_line] = ACTIONS(2266),
    [sym_file_copy] = ACTIONS(2266),
    [sym_file_delete] = ACTIONS(2266),
    [sym_file_rename] = ACTIONS(2266),
    [sym_file_dir] = ACTIONS(2266),
    [sym_file_getdir] = ACTIONS(2266),
    [sym_file_setdir] = ACTIONS(2266),
    [sym_file_create_dir] = ACTIONS(2266),
    [sym_file_remove_dir] = ACTIONS(2266),
    [sym_file_seek] = ACTIONS(2266),
    [sym_length_array] = ACTIONS(2266),
    [sym_max_length_array] = ACTIONS(2266),
    [sym_set_length_array] = ACTIONS(2266),
    [sym_ip_client_open] = ACTIONS(2266),
    [sym_ip_client_close] = ACTIONS(2266),
    [sym_ip_server_open] = ACTIONS(2266),
    [sym_ip_server_close] = ACTIONS(2266),
    [sym_ip_bound_client_open] = ACTIONS(2266),
    [sym_ip_mc_server_open] = ACTIONS(2266),
    [sym_ip_set_option] = ACTIONS(2266),
    [sym_ssh_client_open] = ACTIONS(2266),
    [sym_ssh_client_close] = ACTIONS(2266),
    [sym_tls_client_open] = ACTIONS(2266),
    [sym_tls_client_close] = ACTIONS(2266),
    [sym_do_push] = ACTIONS(2266),
    [sym_do_push_timed] = ACTIONS(2266),
    [sym_do_release] = ACTIONS(2266),
    [sym_device_id] = ACTIONS(2266),
    [sym_device_info] = ACTIONS(2266),
    [sym_device_id_string] = ACTIONS(2266),
    [sym_device_standby] = ACTIONS(2266),
    [sym_device_wake] = ACTIONS(2266),
    [sym_timeline_create] = ACTIONS(2266),
    [sym_timeline_kill] = ACTIONS(2266),
    [sym_timeline_reload] = ACTIONS(2266),
    [sym_timeline_pause] = ACTIONS(2266),
    [sym_timeline_restart] = ACTIONS(2266),
    [sym_timeline_set] = ACTIONS(2266),
    [sym_timeline_get] = ACTIONS(2266),
    [sym_timeline_active] = ACTIONS(2266),
    [sym_timeline_dynamic_id] = ACTIONS(2266),
    [sym_variable_to_string] = ACTIONS(2266),
    [sym_string_to_variable] = ACTIONS(2266),
    [sym_length_variable_to_string] = ACTIONS(2266),
    [sym_type_cast] = ACTIONS(2266),
    [sym_variable_to_xml] = ACTIONS(2266),
    [sym_xml_to_variable] = ACTIONS(2266),
    [sym_length_variable_to_xml] = ACTIONS(2266),
    [sym_redirect_string] = ACTIONS(2266),
    [sym_random_number] = ACTIONS(2266),
    [sym_get_unique_id] = ACTIONS(2266),
    [sym_get_system_number] = ACTIONS(2266),
    [sym_set_system_number] = ACTIONS(2266),
    [sym_get_serial_number] = ACTIONS(2266),
    [sym_get_master_build] = ACTIONS(2266),
    [sym_set_validation_code] = ACTIONS(2266),
    [sym_reboot] = ACTIONS(2266),
    [sym_load_duet_module] = ACTIONS(2266),
    [sym_unload_duet_module] = ACTIONS(2266),
    [sym_duet_mem_size_set] = ACTIONS(2266),
    [sym_duet_mem_size_get] = ACTIONS(2266),
    [sym_dynamic_application_device] = ACTIONS(2266),
    [sym_dynamic_polled_port] = ACTIONS(2266),
    [sym_static_port_binding] = ACTIONS(2266),
    [sym_static_ip_binding] = ACTIONS(2266),
    [sym_get_dns_list] = ACTIONS(2266),
    [sym_set_dns_list] = ACTIONS(2266),
    [sym_get_ip_address] = ACTIONS(2266),
    [sym_set_ip_address] = ACTIONS(2266),
    [sym_get_url_list] = ACTIONS(2266),
    [sym_add_url_entry] = ACTIONS(2266),
    [sym_delete_url_entry] = ACTIONS(2266),
    [sym_abs_value] = ACTIONS(2266),
    [sym_max_value] = ACTIONS(2266),
    [sym_min_value] = ACTIONS(2266),
    [sym_exp_value] = ACTIONS(2266),
    [sym_log_value] = ACTIONS(2266),
    [sym_log10_value] = ACTIONS(2266),
    [sym_power_value] = ACTIONS(2266),
    [sym_sqrt_value] = ACTIONS(2266),
    [sym_raw_be] = ACTIONS(2266),
    [sym_raw_le] = ACTIONS(2266),
    [sym_internal_queue_size_set] = ACTIONS(2266),
    [sym_internal_queue_size_get] = ACTIONS(2266),
    [sym_internal_threshold_set] = ACTIONS(2266),
    [sym_internal_threshold_get] = ACTIONS(2266),
    [sym_rebuilt_event] = ACTIONS(2266),
    [sym_do_custom_event] = ACTIONS(2266),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2266),
    [sym_clkmgr_set_clk_source] = ACTIONS(2266),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2266),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2266),
    [sym_clkmgr_get_timezone] = ACTIONS(2266),
    [sym_clkmgr_set_timezone] = ACTIONS(2266),
    [sym_clkmgr_get_resync_period] = ACTIONS(2266),
    [sym_clkmgr_set_resync_period] = ACTIONS(2266),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2266),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2266),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2266),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2266),
    [sym_clkmgr_get_timeservers] = ACTIONS(2266),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2266),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2266),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2266),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2266),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2266),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2266),
    [sym_set_log_level] = ACTIONS(2266),
    [sym_get_log_level] = ACTIONS(2266),
    [sym_amx_log] = ACTIONS(2266),
    [sym_smtp_server_config_set] = ACTIONS(2266),
    [sym_smtp_server_config_get] = ACTIONS(2266),
    [sym_smtp_send] = ACTIONS(2266),
    [sym_validate_netlinx_account] = ACTIONS(2266),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2266),
    [sym_authenticate_certificate] = ACTIONS(2266),
    [sym_audit_netlinx_session_event] = ACTIONS(2266),
    [sym_audit_netlinx_generic_event] = ACTIONS(2266),
    [sym_get_available_flash_disk_space] = ACTIONS(2266),
    [sym_get_max_flash_disk_space] = ACTIONS(2266),
    [sym_data_create_feed] = ACTIONS(2266),
    [sym_data_delete_feed] = ACTIONS(2266),
    [sym_data_publish_feed] = ACTIONS(2266),
    [sym_data_get_published_feed] = ACTIONS(2266),
    [sym_data_add_record] = ACTIONS(2266),
    [sym_data_get_event_record] = ACTIONS(2266),
    [sym__wc_data_create_feed] = ACTIONS(2266),
    [sym__wc_data_add_record] = ACTIONS(2266),
    [sym__wc_data_get_event_record] = ACTIONS(2266),
    [sym_format] = ACTIONS(2266),
    [sym_astro_clock] = ACTIONS(2266),
    [sym_set_outdoor_temperature] = ACTIONS(2266),
    [sym_set_virtual_level_count] = ACTIONS(2266),
    [sym_set_virtual_channel_count] = ACTIONS(2266),
    [sym_set_virtual_port_count] = ACTIONS(2266),
    [sym__wc_encode_int] = ACTIONS(2266),
    [sym__wc_decode_int] = ACTIONS(2266),
    [sym__wc_int] = ACTIONS(2266),
    [sym__wc_to_ch_int] = ACTIONS(2266),
    [sym__ch_to_wc_int] = ACTIONS(2266),
    [sym__wc_find_string_int] = ACTIONS(2266),
    [sym__wc_left_string_int] = ACTIONS(2266),
    [sym__wc_lower_string_int] = ACTIONS(2266),
    [sym__wc_mid_string_int] = ACTIONS(2266),
    [sym__wc_remove_string_int] = ACTIONS(2266),
    [sym__wc_right_string_int] = ACTIONS(2266),
    [sym__wc_upper_string_int] = ACTIONS(2266),
    [sym__wc_compare_string_int] = ACTIONS(2266),
    [sym__wc_get_buffer_char_int] = ACTIONS(2266),
    [sym__wc_get_buffer_string_int] = ACTIONS(2266),
    [sym__wc_concat_string_int] = ACTIONS(2266),
    [sym__wc_file_open_int] = ACTIONS(2266),
    [sym__wc_file_close_int] = ACTIONS(2266),
    [sym__wc_file_read_int] = ACTIONS(2266),
    [sym__wc_file_read_line_int] = ACTIONS(2266),
    [sym__wc_file_write_int] = ACTIONS(2266),
    [sym__wc_file_write_line_int] = ACTIONS(2266),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2268),
    [aux_sym_preproc_else_token1] = ACTIONS(2268),
    [anon_sym_COMMA] = ACTIONS(2270),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2268),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2266),
    [anon_sym_TILDE] = ACTIONS(2268),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2268),
    [anon_sym_SQUOTE] = ACTIONS(2268),
    [sym_number_literal] = ACTIONS(2268),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(634)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2273),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2273),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2273),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2273),
    [sym___date__] = ACTIONS(2273),
    [sym___ldate__] = ACTIONS(2273),
    [sym___time__] = ACTIONS(2273),
    [sym___line__] = ACTIONS(2273),
    [sym___name__] = ACTIONS(2273),
    [sym___netlinx__] = ACTIONS(2273),
    [sym_netlinx_axi_version] = ACTIONS(2273),
    [sym_url_flg_tcp] = ACTIONS(2273),
    [sym_url_flg_accinfopresent] = ACTIONS(2273),
    [sym_url_flg_temp] = ACTIONS(2273),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2273),
    [sym_url_flg_stat_mask] = ACTIONS(2273),
    [sym_url_flg_stat_lookup] = ACTIONS(2273),
    [sym_url_flg_stat_connecting] = ACTIONS(2273),
    [sym_url_flg_stat_waiting] = ACTIONS(2273),
    [sym_url_flg_stat_connected] = ACTIONS(2273),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2273),
    [sym_first_virtual_device] = ACTIONS(2273),
    [sym_first_local_port] = ACTIONS(2273),
    [sym_dynamic_virtual_device] = ACTIONS(2273),
    [sym_timeline_once] = ACTIONS(2273),
    [sym_timeline_repeat] = ACTIONS(2273),
    [sym_timeline_absolute] = ACTIONS(2273),
    [sym_timeline_relative] = ACTIONS(2273),
    [sym_do_push_timed_infinite] = ACTIONS(2273),
    [sym_source_type_no_address] = ACTIONS(2273),
    [sym_source_type_neuron_id] = ACTIONS(2273),
    [sym_source_type_ip_address] = ACTIONS(2273),
    [sym_source_type_axlink] = ACTIONS(2273),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2273),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2273),
    [sym_source_type_ip_socket_address] = ACTIONS(2273),
    [sym_source_type_rs232] = ACTIONS(2273),
    [sym_source_type_internal] = ACTIONS(2273),
    [sym_source_type_mac_address] = ACTIONS(2273),
    [sym_source_type_ipv4_port] = ACTIONS(2273),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2273),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2273),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2273),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2273),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2273),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2273),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2273),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2273),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2273),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2273),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2273),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2273),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2273),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2273),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2273),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2273),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2273),
    [sym_true] = ACTIONS(2273),
    [sym_false] = ACTIONS(2273),
    [sym_file_read_only] = ACTIONS(2273),
    [sym_file_rw_new] = ACTIONS(2273),
    [sym_file_rw_append] = ACTIONS(2273),
    [sym_ip_tcp] = ACTIONS(2273),
    [sym_ip_udp] = ACTIONS(2273),
    [sym_ip_udp_2way] = ACTIONS(2273),
    [sym_xml_encode_types] = ACTIONS(2273),
    [sym_xml_encode_char_as_list] = ACTIONS(2273),
    [sym_xml_encode_le] = ACTIONS(2273),
    [sym_xml_decode_types] = ACTIONS(2273),
    [sym_xml_decode_no_preserve] = ACTIONS(2273),
    [sym_duet_dev_type_display_device] = ACTIONS(2273),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2273),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2273),
    [sym_duet_dev_type_io_device] = ACTIONS(2273),
    [sym_duet_dev_type_relay_device] = ACTIONS(2273),
    [sym_duet_dev_type_ups] = ACTIONS(2273),
    [sym_duet_dev_type_amplifier] = ACTIONS(2273),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2273),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2273),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2273),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2273),
    [sym_duet_dev_type_camera] = ACTIONS(2273),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2273),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2273),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2273),
    [sym_duet_dev_type_dss] = ACTIONS(2273),
    [sym_duet_dev_type_dvr] = ACTIONS(2273),
    [sym_duet_dev_type_disc_device] = ACTIONS(2273),
    [sym_duet_dev_type_document_camera] = ACTIONS(2273),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2273),
    [sym_duet_dev_type_hvac] = ACTIONS(2273),
    [sym_duet_dev_type_keypad] = ACTIONS(2273),
    [sym_duet_dev_type_light] = ACTIONS(2273),
    [sym_duet_dev_type_monitor] = ACTIONS(2273),
    [sym_duet_dev_type_motor] = ACTIONS(2273),
    [sym_duet_dev_type_multi_window] = ACTIONS(2273),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2273),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2273),
    [sym_duet_dev_type_receiver] = ACTIONS(2273),
    [sym_duet_dev_type_security_system] = ACTIONS(2273),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2273),
    [sym_duet_dev_type_settop_box] = ACTIONS(2273),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2273),
    [sym_duet_dev_type_switcher] = ACTIONS(2273),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2273),
    [sym_duet_dev_type_tv] = ACTIONS(2273),
    [sym_duet_dev_type_utility] = ACTIONS(2273),
    [sym_duet_dev_type_vcr] = ACTIONS(2273),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2273),
    [sym_duet_dev_type_video_processor] = ACTIONS(2273),
    [sym_duet_dev_type_video_projector] = ACTIONS(2273),
    [sym_duet_dev_type_video_wall] = ACTIONS(2273),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2273),
    [sym_duet_dev_type_weather] = ACTIONS(2273),
    [sym_duet_dev_not_polled] = ACTIONS(2273),
    [sym_duet_dev_polled] = ACTIONS(2273),
    [sym_clkmgr_mode_network] = ACTIONS(2273),
    [sym_clkmgr_mode_standalone] = ACTIONS(2273),
    [sym_normal_standby] = ACTIONS(2273),
    [sym_normal_wake] = ACTIONS(2273),
    [sym_ip_multicast_ttl_option] = ACTIONS(2273),
    [sym_ip_ttl_subnet] = ACTIONS(2273),
    [sym_ip_ttl_site] = ACTIONS(2273),
    [sym_ip_ttl_region] = ACTIONS(2273),
    [sym_ip_ttl_continent] = ACTIONS(2273),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2273),
    [sym_ip_nodelay_on] = ACTIONS(2273),
    [sym_ip_nodelay_off] = ACTIONS(2273),
    [sym_amx_error] = ACTIONS(2273),
    [sym_amx_warning] = ACTIONS(2273),
    [sym_amx_info] = ACTIONS(2273),
    [sym_amx_debug] = ACTIONS(2273),
    [sym_netlinx_login_success] = ACTIONS(2273),
    [sym_netlinx_login_fail] = ACTIONS(2273),
    [sym_netlinx_logout] = ACTIONS(2273),
    [sym_null_str] = ACTIONS(2273),
    [sym_smtp_address] = ACTIONS(2273),
    [sym_smtp_port_number] = ACTIONS(2273),
    [sym_smtp_username] = ACTIONS(2273),
    [sym_smtp_password] = ACTIONS(2273),
    [sym_smtp_require_tls] = ACTIONS(2273),
    [sym_smtp_from] = ACTIONS(2273),
    [sym_smtp_tls_true] = ACTIONS(2273),
    [sym_smtp_tls_false] = ACTIONS(2273),
    [sym_listview_on_row_select_event] = ACTIONS(2273),
    [sym_data_structure_unknown] = ACTIONS(2273),
    [sym_data_structure_datafeed] = ACTIONS(2273),
    [sym_data_structure_datarecordset] = ACTIONS(2273),
    [sym_data_structure_datarecord] = ACTIONS(2273),
    [sym_data_structure_datafield] = ACTIONS(2273),
    [sym_data_max_content_field_count] = ACTIONS(2273),
    [sym_data_max_metadata_field_count] = ACTIONS(2273),
    [sym_data_max_id_length] = ACTIONS(2273),
    [sym_data_max_name_length] = ACTIONS(2273),
    [sym_data_max_description_length] = ACTIONS(2273),
    [sym_data_max_type_length] = ACTIONS(2273),
    [sym_data_max_format_length] = ACTIONS(2273),
    [sym_data_max_source_length] = ACTIONS(2273),
    [sym_data_max_value_length] = ACTIONS(2273),
    [sym_data_max_label_length] = ACTIONS(2273),
    [sym_data_max_url_length] = ACTIONS(2273),
    [sym_max_last_login_info_length] = ACTIONS(2273),
    [sym_data_type_unknown] = ACTIONS(2273),
    [sym_data_type_string] = ACTIONS(2273),
    [sym_data_type_datetime] = ACTIONS(2273),
    [sym_data_type_date] = ACTIONS(2273),
    [sym_data_type_time] = ACTIONS(2273),
    [sym_data_type_image] = ACTIONS(2273),
    [sym_data_format_url] = ACTIONS(2273),
    [sym_data_format_phone] = ACTIONS(2273),
    [sym_data_format_email] = ACTIONS(2273),
    [sym_data_format_iso8601] = ACTIONS(2273),
    [sym_auditlog_priv] = ACTIONS(2273),
    [sym_configuration_priv] = ACTIONS(2273),
    [sym_deviceconfig_priv] = ACTIONS(2273),
    [sym_ftp_priv] = ACTIONS(2273),
    [sym_http_priv] = ACTIONS(2273),
    [sym_networkconfig_priv] = ACTIONS(2273),
    [sym_programport_priv] = ACTIONS(2273),
    [sym_remoteui_priv] = ACTIONS(2273),
    [sym_securitycontrol_priv] = ACTIONS(2273),
    [sym_softwaremanagement_priv] = ACTIONS(2273),
    [sym_terminal_priv] = ACTIONS(2273),
    [sym_tpadmin_priv] = ACTIONS(2273),
    [sym_user1_priv] = ACTIONS(2273),
    [sym_user2_priv] = ACTIONS(2273),
    [sym_user3_priv] = ACTIONS(2273),
    [sym_user4_priv] = ACTIONS(2273),
    [sym_usermanagement_priv] = ACTIONS(2273),
    [sym_valid_account] = ACTIONS(2273),
    [sym_err_name_invalid] = ACTIONS(2273),
    [sym_err_password_invalid] = ACTIONS(2273),
    [sym_err_invalid_account] = ACTIONS(2273),
    [sym_err_no_authorization] = ACTIONS(2273),
    [sym_err_invalid_parameter] = ACTIONS(2273),
    [sym_err_user_locked_out] = ACTIONS(2273),
    [sym_err_user_account_expired] = ACTIONS(2273),
    [sym_err_general_error] = ACTIONS(2273),
    [sym_err_authentication_error] = ACTIONS(2273),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2273),
    [sym_tls_validate_certificate] = ACTIONS(2273),
    [sym____reserved___] = ACTIONS(2273),
    [sym_date] = ACTIONS(2273),
    [sym_day] = ACTIONS(2273),
    [sym_time] = ACTIONS(2273),
    [sym_ldate] = ACTIONS(2273),
    [sym_push_device] = ACTIONS(2273),
    [sym_push_channel] = ACTIONS(2273),
    [sym_push_devchan] = ACTIONS(2273),
    [sym_release_device] = ACTIONS(2273),
    [sym_release_channel] = ACTIONS(2273),
    [sym_release_devchan] = ACTIONS(2273),
    [sym_master_slot] = ACTIONS(2273),
    [sym_get_pulse_time] = ACTIONS(2273),
    [sym_get_timer] = ACTIONS(2273),
    [sym_master_sn] = ACTIONS(2273),
    [sym_system_number] = ACTIONS(2273),
    [sym_button] = ACTIONS(2273),
    [sym_dv_channel] = ACTIONS(2273),
    [sym_data] = ACTIONS(2273),
    [sym_level] = ACTIONS(2273),
    [sym_timeline] = ACTIONS(2273),
    [sym_channel] = ACTIONS(2273),
    [sym_custom] = ACTIONS(2273),
    [sym_atoi] = ACTIONS(2273),
    [sym_atol] = ACTIONS(2273),
    [sym_atof] = ACTIONS(2273),
    [sym_itoa] = ACTIONS(2273),
    [sym_itohex] = ACTIONS(2273),
    [sym_hextoi] = ACTIONS(2273),
    [sym_ftoa] = ACTIONS(2273),
    [sym_find_string] = ACTIONS(2273),
    [sym_left_string] = ACTIONS(2273),
    [sym_right_string] = ACTIONS(2273),
    [sym_mid_string] = ACTIONS(2273),
    [sym_remove_string] = ACTIONS(2273),
    [sym_get_buffer_string] = ACTIONS(2273),
    [sym_get_buffer_char] = ACTIONS(2273),
    [sym_get_multi_buffer_string] = ACTIONS(2273),
    [sym_length_string] = ACTIONS(2273),
    [sym_max_length_string] = ACTIONS(2273),
    [sym_set_length_string] = ACTIONS(2273),
    [sym_compare_string] = ACTIONS(2273),
    [sym_upper_string] = ACTIONS(2273),
    [sym_lower_string] = ACTIONS(2273),
    [sym_get_last] = ACTIONS(2273),
    [sym_date_to_day] = ACTIONS(2273),
    [sym_date_to_month] = ACTIONS(2273),
    [sym_date_to_year] = ACTIONS(2273),
    [sym_time_to_hour] = ACTIONS(2273),
    [sym_time_to_minute] = ACTIONS(2273),
    [sym_time_to_second] = ACTIONS(2273),
    [sym_day_of_week] = ACTIONS(2273),
    [sym_set_pulse_time] = ACTIONS(2273),
    [sym_set_timer] = ACTIONS(2273),
    [sym_file_open] = ACTIONS(2273),
    [sym_file_close] = ACTIONS(2273),
    [sym_file_read] = ACTIONS(2273),
    [sym_file_read_line] = ACTIONS(2273),
    [sym_file_write] = ACTIONS(2273),
    [sym_file_write_line] = ACTIONS(2273),
    [sym_file_copy] = ACTIONS(2273),
    [sym_file_delete] = ACTIONS(2273),
    [sym_file_rename] = ACTIONS(2273),
    [sym_file_dir] = ACTIONS(2273),
    [sym_file_getdir] = ACTIONS(2273),
    [sym_file_setdir] = ACTIONS(2273),
    [sym_file_create_dir] = ACTIONS(2273),
    [sym_file_remove_dir] = ACTIONS(2273),
    [sym_file_seek] = ACTIONS(2273),
    [sym_length_array] = ACTIONS(2273),
    [sym_max_length_array] = ACTIONS(2273),
    [sym_set_length_array] = ACTIONS(2273),
    [sym_ip_client_open] = ACTIONS(2273),
    [sym_ip_client_close] = ACTIONS(2273),
    [sym_ip_server_open] = ACTIONS(2273),
    [sym_ip_server_close] = ACTIONS(2273),
    [sym_ip_bound_client_open] = ACTIONS(2273),
    [sym_ip_mc_server_open] = ACTIONS(2273),
    [sym_ip_set_option] = ACTIONS(2273),
    [sym_ssh_client_open] = ACTIONS(2273),
    [sym_ssh_client_close] = ACTIONS(2273),
    [sym_tls_client_open] = ACTIONS(2273),
    [sym_tls_client_close] = ACTIONS(2273),
    [sym_do_push] = ACTIONS(2273),
    [sym_do_push_timed] = ACTIONS(2273),
    [sym_do_release] = ACTIONS(2273),
    [sym_device_id] = ACTIONS(2273),
    [sym_device_info] = ACTIONS(2273),
    [sym_device_id_string] = ACTIONS(2273),
    [sym_device_standby] = ACTIONS(2273),
    [sym_device_wake] = ACTIONS(2273),
    [sym_timeline_create] = ACTIONS(2273),
    [sym_timeline_kill] = ACTIONS(2273),
    [sym_timeline_reload] = ACTIONS(2273),
    [sym_timeline_pause] = ACTIONS(2273),
    [sym_timeline_restart] = ACTIONS(2273),
    [sym_timeline_set] = ACTIONS(2273),
    [sym_timeline_get] = ACTIONS(2273),
    [sym_timeline_active] = ACTIONS(2273),
    [sym_timeline_dynamic_id] = ACTIONS(2273),
    [sym_variable_to_string] = ACTIONS(2273),
    [sym_string_to_variable] = ACTIONS(2273),
    [sym_length_variable_to_string] = ACTIONS(2273),
    [sym_type_cast] = ACTIONS(2273),
    [sym_variable_to_xml] = ACTIONS(2273),
    [sym_xml_to_variable] = ACTIONS(2273),
    [sym_length_variable_to_xml] = ACTIONS(2273),
    [sym_redirect_string] = ACTIONS(2273),
    [sym_random_number] = ACTIONS(2273),
    [sym_get_unique_id] = ACTIONS(2273),
    [sym_get_system_number] = ACTIONS(2273),
    [sym_set_system_number] = ACTIONS(2273),
    [sym_get_serial_number] = ACTIONS(2273),
    [sym_get_master_build] = ACTIONS(2273),
    [sym_set_validation_code] = ACTIONS(2273),
    [sym_reboot] = ACTIONS(2273),
    [sym_load_duet_module] = ACTIONS(2273),
    [sym_unload_duet_module] = ACTIONS(2273),
    [sym_duet_mem_size_set] = ACTIONS(2273),
    [sym_duet_mem_size_get] = ACTIONS(2273),
    [sym_dynamic_application_device] = ACTIONS(2273),
    [sym_dynamic_polled_port] = ACTIONS(2273),
    [sym_static_port_binding] = ACTIONS(2273),
    [sym_static_ip_binding] = ACTIONS(2273),
    [sym_get_dns_list] = ACTIONS(2273),
    [sym_set_dns_list] = ACTIONS(2273),
    [sym_get_ip_address] = ACTIONS(2273),
    [sym_set_ip_address] = ACTIONS(2273),
    [sym_get_url_list] = ACTIONS(2273),
    [sym_add_url_entry] = ACTIONS(2273),
    [sym_delete_url_entry] = ACTIONS(2273),
    [sym_abs_value] = ACTIONS(2273),
    [sym_max_value] = ACTIONS(2273),
    [sym_min_value] = ACTIONS(2273),
    [sym_exp_value] = ACTIONS(2273),
    [sym_log_value] = ACTIONS(2273),
    [sym_log10_value] = ACTIONS(2273),
    [sym_power_value] = ACTIONS(2273),
    [sym_sqrt_value] = ACTIONS(2273),
    [sym_raw_be] = ACTIONS(2273),
    [sym_raw_le] = ACTIONS(2273),
    [sym_internal_queue_size_set] = ACTIONS(2273),
    [sym_internal_queue_size_get] = ACTIONS(2273),
    [sym_internal_threshold_set] = ACTIONS(2273),
    [sym_internal_threshold_get] = ACTIONS(2273),
    [sym_rebuilt_event] = ACTIONS(2273),
    [sym_do_custom_event] = ACTIONS(2273),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2273),
    [sym_clkmgr_set_clk_source] = ACTIONS(2273),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2273),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2273),
    [sym_clkmgr_get_timezone] = ACTIONS(2273),
    [sym_clkmgr_set_timezone] = ACTIONS(2273),
    [sym_clkmgr_get_resync_period] = ACTIONS(2273),
    [sym_clkmgr_set_resync_period] = ACTIONS(2273),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2273),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2273),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2273),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2273),
    [sym_clkmgr_get_timeservers] = ACTIONS(2273),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2273),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2273),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2273),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2273),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2273),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2273),
    [sym_set_log_level] = ACTIONS(2273),
    [sym_get_log_level] = ACTIONS(2273),
    [sym_amx_log] = ACTIONS(2273),
    [sym_smtp_server_config_set] = ACTIONS(2273),
    [sym_smtp_server_config_get] = ACTIONS(2273),
    [sym_smtp_send] = ACTIONS(2273),
    [sym_validate_netlinx_account] = ACTIONS(2273),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2273),
    [sym_authenticate_certificate] = ACTIONS(2273),
    [sym_audit_netlinx_session_event] = ACTIONS(2273),
    [sym_audit_netlinx_generic_event] = ACTIONS(2273),
    [sym_get_available_flash_disk_space] = ACTIONS(2273),
    [sym_get_max_flash_disk_space] = ACTIONS(2273),
    [sym_data_create_feed] = ACTIONS(2273),
    [sym_data_delete_feed] = ACTIONS(2273),
    [sym_data_publish_feed] = ACTIONS(2273),
    [sym_data_get_published_feed] = ACTIONS(2273),
    [sym_data_add_record] = ACTIONS(2273),
    [sym_data_get_event_record] = ACTIONS(2273),
    [sym__wc_data_create_feed] = ACTIONS(2273),
    [sym__wc_data_add_record] = ACTIONS(2273),
    [sym__wc_data_get_event_record] = ACTIONS(2273),
    [sym_format] = ACTIONS(2273),
    [sym_astro_clock] = ACTIONS(2273),
    [sym_set_outdoor_temperature] = ACTIONS(2273),
    [sym_set_virtual_level_count] = ACTIONS(2273),
    [sym_set_virtual_channel_count] = ACTIONS(2273),
    [sym_set_virtual_port_count] = ACTIONS(2273),
    [sym__wc_encode_int] = ACTIONS(2273),
    [sym__wc_decode_int] = ACTIONS(2273),
    [sym__wc_int] = ACTIONS(2273),
    [sym__wc_to_ch_int] = ACTIONS(2273),
    [sym__ch_to_wc_int] = ACTIONS(2273),
    [sym__wc_find_string_int] = ACTIONS(2273),
    [sym__wc_left_string_int] = ACTIONS(2273),
    [sym__wc_lower_string_int] = ACTIONS(2273),
    [sym__wc_mid_string_int] = ACTIONS(2273),
    [sym__wc_remove_string_int] = ACTIONS(2273),
    [sym__wc_right_string_int] = ACTIONS(2273),
    [sym__wc_upper_string_int] = ACTIONS(2273),
    [sym__wc_compare_string_int] = ACTIONS(2273),
    [sym__wc_get_buffer_char_int] = ACTIONS(2273),
    [sym__wc_get_buffer_string_int] = ACTIONS(2273),
    [sym__wc_concat_string_int] = ACTIONS(2273),
    [sym__wc_file_open_int] = ACTIONS(2273),
    [sym__wc_file_close_int] = ACTIONS(2273),
    [sym__wc_file_read_int] = ACTIONS(2273),
    [sym__wc_file_read_line_int] = ACTIONS(2273),
    [sym__wc_file_write_int] = ACTIONS(2273),
    [sym__wc_file_write_line_int] = ACTIONS(2273),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2276),
    [aux_sym_preproc_else_token1] = ACTIONS(2276),
    [anon_sym_COMMA] = ACTIONS(2263),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2276),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2273),
    [anon_sym_TILDE] = ACTIONS(2276),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2276),
    [anon_sym_SQUOTE] = ACTIONS(2276),
    [sym_number_literal] = ACTIONS(2276),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(635)] = {
    [sym_argument_list] = STATE(593),
    [aux_sym_subscript_expression_repeat1] = STATE(581),
    [sym_identifier] = ACTIONS(2279),
    [sym_and] = ACTIONS(2185),
    [sym_or] = ACTIONS(2187),
    [sym_not] = ACTIONS(2279),
    [sym_xor] = ACTIONS(2189),
    [sym_band] = ACTIONS(2191),
    [sym_bor] = ACTIONS(2193),
    [sym_bxor] = ACTIONS(2189),
    [sym_bnot] = ACTIONS(2279),
    [sym_lshift] = ACTIONS(2163),
    [sym_rshift] = ACTIONS(2163),
    [sym___file__] = ACTIONS(2279),
    [sym___date__] = ACTIONS(2279),
    [sym___ldate__] = ACTIONS(2279),
    [sym___time__] = ACTIONS(2279),
    [sym___line__] = ACTIONS(2279),
    [sym___name__] = ACTIONS(2279),
    [sym___netlinx__] = ACTIONS(2279),
    [sym_netlinx_axi_version] = ACTIONS(2279),
    [sym_url_flg_tcp] = ACTIONS(2279),
    [sym_url_flg_accinfopresent] = ACTIONS(2279),
    [sym_url_flg_temp] = ACTIONS(2279),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2279),
    [sym_url_flg_stat_mask] = ACTIONS(2279),
    [sym_url_flg_stat_lookup] = ACTIONS(2279),
    [sym_url_flg_stat_connecting] = ACTIONS(2279),
    [sym_url_flg_stat_waiting] = ACTIONS(2279),
    [sym_url_flg_stat_connected] = ACTIONS(2279),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2279),
    [sym_first_virtual_device] = ACTIONS(2279),
    [sym_first_local_port] = ACTIONS(2279),
    [sym_dynamic_virtual_device] = ACTIONS(2279),
    [sym_timeline_once] = ACTIONS(2279),
    [sym_timeline_repeat] = ACTIONS(2279),
    [sym_timeline_absolute] = ACTIONS(2279),
    [sym_timeline_relative] = ACTIONS(2279),
    [sym_do_push_timed_infinite] = ACTIONS(2279),
    [sym_source_type_no_address] = ACTIONS(2279),
    [sym_source_type_neuron_id] = ACTIONS(2279),
    [sym_source_type_ip_address] = ACTIONS(2279),
    [sym_source_type_axlink] = ACTIONS(2279),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2279),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2279),
    [sym_source_type_ip_socket_address] = ACTIONS(2279),
    [sym_source_type_rs232] = ACTIONS(2279),
    [sym_source_type_internal] = ACTIONS(2279),
    [sym_source_type_mac_address] = ACTIONS(2279),
    [sym_source_type_ipv4_port] = ACTIONS(2279),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2279),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2279),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2279),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2279),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2279),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2279),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2279),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2279),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2279),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2279),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2279),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2279),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2279),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2279),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2279),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2279),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2279),
    [sym_true] = ACTIONS(2279),
    [sym_false] = ACTIONS(2279),
    [sym_file_read_only] = ACTIONS(2279),
    [sym_file_rw_new] = ACTIONS(2279),
    [sym_file_rw_append] = ACTIONS(2279),
    [sym_ip_tcp] = ACTIONS(2279),
    [sym_ip_udp] = ACTIONS(2279),
    [sym_ip_udp_2way] = ACTIONS(2279),
    [sym_xml_encode_types] = ACTIONS(2279),
    [sym_xml_encode_char_as_list] = ACTIONS(2279),
    [sym_xml_encode_le] = ACTIONS(2279),
    [sym_xml_decode_types] = ACTIONS(2279),
    [sym_xml_decode_no_preserve] = ACTIONS(2279),
    [sym_duet_dev_type_display_device] = ACTIONS(2279),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2279),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2279),
    [sym_duet_dev_type_io_device] = ACTIONS(2279),
    [sym_duet_dev_type_relay_device] = ACTIONS(2279),
    [sym_duet_dev_type_ups] = ACTIONS(2279),
    [sym_duet_dev_type_amplifier] = ACTIONS(2279),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2279),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2279),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2279),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2279),
    [sym_duet_dev_type_camera] = ACTIONS(2279),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2279),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2279),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2279),
    [sym_duet_dev_type_dss] = ACTIONS(2279),
    [sym_duet_dev_type_dvr] = ACTIONS(2279),
    [sym_duet_dev_type_disc_device] = ACTIONS(2279),
    [sym_duet_dev_type_document_camera] = ACTIONS(2279),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2279),
    [sym_duet_dev_type_hvac] = ACTIONS(2279),
    [sym_duet_dev_type_keypad] = ACTIONS(2279),
    [sym_duet_dev_type_light] = ACTIONS(2279),
    [sym_duet_dev_type_monitor] = ACTIONS(2279),
    [sym_duet_dev_type_motor] = ACTIONS(2279),
    [sym_duet_dev_type_multi_window] = ACTIONS(2279),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2279),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2279),
    [sym_duet_dev_type_receiver] = ACTIONS(2279),
    [sym_duet_dev_type_security_system] = ACTIONS(2279),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2279),
    [sym_duet_dev_type_settop_box] = ACTIONS(2279),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2279),
    [sym_duet_dev_type_switcher] = ACTIONS(2279),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2279),
    [sym_duet_dev_type_tv] = ACTIONS(2279),
    [sym_duet_dev_type_utility] = ACTIONS(2279),
    [sym_duet_dev_type_vcr] = ACTIONS(2279),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2279),
    [sym_duet_dev_type_video_processor] = ACTIONS(2279),
    [sym_duet_dev_type_video_projector] = ACTIONS(2279),
    [sym_duet_dev_type_video_wall] = ACTIONS(2279),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2279),
    [sym_duet_dev_type_weather] = ACTIONS(2279),
    [sym_duet_dev_not_polled] = ACTIONS(2279),
    [sym_duet_dev_polled] = ACTIONS(2279),
    [sym_clkmgr_mode_network] = ACTIONS(2279),
    [sym_clkmgr_mode_standalone] = ACTIONS(2279),
    [sym_normal_standby] = ACTIONS(2279),
    [sym_normal_wake] = ACTIONS(2279),
    [sym_ip_multicast_ttl_option] = ACTIONS(2279),
    [sym_ip_ttl_subnet] = ACTIONS(2279),
    [sym_ip_ttl_site] = ACTIONS(2279),
    [sym_ip_ttl_region] = ACTIONS(2279),
    [sym_ip_ttl_continent] = ACTIONS(2279),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2279),
    [sym_ip_nodelay_on] = ACTIONS(2279),
    [sym_ip_nodelay_off] = ACTIONS(2279),
    [sym_amx_error] = ACTIONS(2279),
    [sym_amx_warning] = ACTIONS(2279),
    [sym_amx_info] = ACTIONS(2279),
    [sym_amx_debug] = ACTIONS(2279),
    [sym_netlinx_login_success] = ACTIONS(2279),
    [sym_netlinx_login_fail] = ACTIONS(2279),
    [sym_netlinx_logout] = ACTIONS(2279),
    [sym_null_str] = ACTIONS(2279),
    [sym_smtp_address] = ACTIONS(2279),
    [sym_smtp_port_number] = ACTIONS(2279),
    [sym_smtp_username] = ACTIONS(2279),
    [sym_smtp_password] = ACTIONS(2279),
    [sym_smtp_require_tls] = ACTIONS(2279),
    [sym_smtp_from] = ACTIONS(2279),
    [sym_smtp_tls_true] = ACTIONS(2279),
    [sym_smtp_tls_false] = ACTIONS(2279),
    [sym_listview_on_row_select_event] = ACTIONS(2279),
    [sym_data_structure_unknown] = ACTIONS(2279),
    [sym_data_structure_datafeed] = ACTIONS(2279),
    [sym_data_structure_datarecordset] = ACTIONS(2279),
    [sym_data_structure_datarecord] = ACTIONS(2279),
    [sym_data_structure_datafield] = ACTIONS(2279),
    [sym_data_max_content_field_count] = ACTIONS(2279),
    [sym_data_max_metadata_field_count] = ACTIONS(2279),
    [sym_data_max_id_length] = ACTIONS(2279),
    [sym_data_max_name_length] = ACTIONS(2279),
    [sym_data_max_description_length] = ACTIONS(2279),
    [sym_data_max_type_length] = ACTIONS(2279),
    [sym_data_max_format_length] = ACTIONS(2279),
    [sym_data_max_source_length] = ACTIONS(2279),
    [sym_data_max_value_length] = ACTIONS(2279),
    [sym_data_max_label_length] = ACTIONS(2279),
    [sym_data_max_url_length] = ACTIONS(2279),
    [sym_max_last_login_info_length] = ACTIONS(2279),
    [sym_data_type_unknown] = ACTIONS(2279),
    [sym_data_type_string] = ACTIONS(2279),
    [sym_data_type_datetime] = ACTIONS(2279),
    [sym_data_type_date] = ACTIONS(2279),
    [sym_data_type_time] = ACTIONS(2279),
    [sym_data_type_image] = ACTIONS(2279),
    [sym_data_format_url] = ACTIONS(2279),
    [sym_data_format_phone] = ACTIONS(2279),
    [sym_data_format_email] = ACTIONS(2279),
    [sym_data_format_iso8601] = ACTIONS(2279),
    [sym_auditlog_priv] = ACTIONS(2279),
    [sym_configuration_priv] = ACTIONS(2279),
    [sym_deviceconfig_priv] = ACTIONS(2279),
    [sym_ftp_priv] = ACTIONS(2279),
    [sym_http_priv] = ACTIONS(2279),
    [sym_networkconfig_priv] = ACTIONS(2279),
    [sym_programport_priv] = ACTIONS(2279),
    [sym_remoteui_priv] = ACTIONS(2279),
    [sym_securitycontrol_priv] = ACTIONS(2279),
    [sym_softwaremanagement_priv] = ACTIONS(2279),
    [sym_terminal_priv] = ACTIONS(2279),
    [sym_tpadmin_priv] = ACTIONS(2279),
    [sym_user1_priv] = ACTIONS(2279),
    [sym_user2_priv] = ACTIONS(2279),
    [sym_user3_priv] = ACTIONS(2279),
    [sym_user4_priv] = ACTIONS(2279),
    [sym_usermanagement_priv] = ACTIONS(2279),
    [sym_valid_account] = ACTIONS(2279),
    [sym_err_name_invalid] = ACTIONS(2279),
    [sym_err_password_invalid] = ACTIONS(2279),
    [sym_err_invalid_account] = ACTIONS(2279),
    [sym_err_no_authorization] = ACTIONS(2279),
    [sym_err_invalid_parameter] = ACTIONS(2279),
    [sym_err_user_locked_out] = ACTIONS(2279),
    [sym_err_user_account_expired] = ACTIONS(2279),
    [sym_err_general_error] = ACTIONS(2279),
    [sym_err_authentication_error] = ACTIONS(2279),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2279),
    [sym_tls_validate_certificate] = ACTIONS(2279),
    [sym____reserved___] = ACTIONS(2279),
    [sym_date] = ACTIONS(2279),
    [sym_day] = ACTIONS(2279),
    [sym_time] = ACTIONS(2279),
    [sym_ldate] = ACTIONS(2279),
    [sym_push_device] = ACTIONS(2279),
    [sym_push_channel] = ACTIONS(2279),
    [sym_push_devchan] = ACTIONS(2279),
    [sym_release_device] = ACTIONS(2279),
    [sym_release_channel] = ACTIONS(2279),
    [sym_release_devchan] = ACTIONS(2279),
    [sym_master_slot] = ACTIONS(2279),
    [sym_get_pulse_time] = ACTIONS(2279),
    [sym_get_timer] = ACTIONS(2279),
    [sym_master_sn] = ACTIONS(2279),
    [sym_system_number] = ACTIONS(2279),
    [sym_button] = ACTIONS(2279),
    [sym_dv_channel] = ACTIONS(2279),
    [sym_data] = ACTIONS(2279),
    [sym_level] = ACTIONS(2279),
    [sym_timeline] = ACTIONS(2279),
    [sym_channel] = ACTIONS(2279),
    [sym_custom] = ACTIONS(2279),
    [sym_atoi] = ACTIONS(2279),
    [sym_atol] = ACTIONS(2279),
    [sym_atof] = ACTIONS(2279),
    [sym_itoa] = ACTIONS(2279),
    [sym_itohex] = ACTIONS(2279),
    [sym_hextoi] = ACTIONS(2279),
    [sym_ftoa] = ACTIONS(2279),
    [sym_find_string] = ACTIONS(2279),
    [sym_left_string] = ACTIONS(2279),
    [sym_right_string] = ACTIONS(2279),
    [sym_mid_string] = ACTIONS(2279),
    [sym_remove_string] = ACTIONS(2279),
    [sym_get_buffer_string] = ACTIONS(2279),
    [sym_get_buffer_char] = ACTIONS(2279),
    [sym_get_multi_buffer_string] = ACTIONS(2279),
    [sym_length_string] = ACTIONS(2279),
    [sym_max_length_string] = ACTIONS(2279),
    [sym_set_length_string] = ACTIONS(2279),
    [sym_compare_string] = ACTIONS(2279),
    [sym_upper_string] = ACTIONS(2279),
    [sym_lower_string] = ACTIONS(2279),
    [sym_get_last] = ACTIONS(2279),
    [sym_date_to_day] = ACTIONS(2279),
    [sym_date_to_month] = ACTIONS(2279),
    [sym_date_to_year] = ACTIONS(2279),
    [sym_time_to_hour] = ACTIONS(2279),
    [sym_time_to_minute] = ACTIONS(2279),
    [sym_time_to_second] = ACTIONS(2279),
    [sym_day_of_week] = ACTIONS(2279),
    [sym_set_pulse_time] = ACTIONS(2279),
    [sym_set_timer] = ACTIONS(2279),
    [sym_file_open] = ACTIONS(2279),
    [sym_file_close] = ACTIONS(2279),
    [sym_file_read] = ACTIONS(2279),
    [sym_file_read_line] = ACTIONS(2279),
    [sym_file_write] = ACTIONS(2279),
    [sym_file_write_line] = ACTIONS(2279),
    [sym_file_copy] = ACTIONS(2279),
    [sym_file_delete] = ACTIONS(2279),
    [sym_file_rename] = ACTIONS(2279),
    [sym_file_dir] = ACTIONS(2279),
    [sym_file_getdir] = ACTIONS(2279),
    [sym_file_setdir] = ACTIONS(2279),
    [sym_file_create_dir] = ACTIONS(2279),
    [sym_file_remove_dir] = ACTIONS(2279),
    [sym_file_seek] = ACTIONS(2279),
    [sym_length_array] = ACTIONS(2279),
    [sym_max_length_array] = ACTIONS(2279),
    [sym_set_length_array] = ACTIONS(2279),
    [sym_ip_client_open] = ACTIONS(2279),
    [sym_ip_client_close] = ACTIONS(2279),
    [sym_ip_server_open] = ACTIONS(2279),
    [sym_ip_server_close] = ACTIONS(2279),
    [sym_ip_bound_client_open] = ACTIONS(2279),
    [sym_ip_mc_server_open] = ACTIONS(2279),
    [sym_ip_set_option] = ACTIONS(2279),
    [sym_ssh_client_open] = ACTIONS(2279),
    [sym_ssh_client_close] = ACTIONS(2279),
    [sym_tls_client_open] = ACTIONS(2279),
    [sym_tls_client_close] = ACTIONS(2279),
    [sym_do_push] = ACTIONS(2279),
    [sym_do_push_timed] = ACTIONS(2279),
    [sym_do_release] = ACTIONS(2279),
    [sym_device_id] = ACTIONS(2279),
    [sym_device_info] = ACTIONS(2279),
    [sym_device_id_string] = ACTIONS(2279),
    [sym_device_standby] = ACTIONS(2279),
    [sym_device_wake] = ACTIONS(2279),
    [sym_timeline_create] = ACTIONS(2279),
    [sym_timeline_kill] = ACTIONS(2279),
    [sym_timeline_reload] = ACTIONS(2279),
    [sym_timeline_pause] = ACTIONS(2279),
    [sym_timeline_restart] = ACTIONS(2279),
    [sym_timeline_set] = ACTIONS(2279),
    [sym_timeline_get] = ACTIONS(2279),
    [sym_timeline_active] = ACTIONS(2279),
    [sym_timeline_dynamic_id] = ACTIONS(2279),
    [sym_variable_to_string] = ACTIONS(2279),
    [sym_string_to_variable] = ACTIONS(2279),
    [sym_length_variable_to_string] = ACTIONS(2279),
    [sym_type_cast] = ACTIONS(2279),
    [sym_variable_to_xml] = ACTIONS(2279),
    [sym_xml_to_variable] = ACTIONS(2279),
    [sym_length_variable_to_xml] = ACTIONS(2279),
    [sym_redirect_string] = ACTIONS(2279),
    [sym_random_number] = ACTIONS(2279),
    [sym_get_unique_id] = ACTIONS(2279),
    [sym_get_system_number] = ACTIONS(2279),
    [sym_set_system_number] = ACTIONS(2279),
    [sym_get_serial_number] = ACTIONS(2279),
    [sym_get_master_build] = ACTIONS(2279),
    [sym_set_validation_code] = ACTIONS(2279),
    [sym_reboot] = ACTIONS(2279),
    [sym_load_duet_module] = ACTIONS(2279),
    [sym_unload_duet_module] = ACTIONS(2279),
    [sym_duet_mem_size_set] = ACTIONS(2279),
    [sym_duet_mem_size_get] = ACTIONS(2279),
    [sym_dynamic_application_device] = ACTIONS(2279),
    [sym_dynamic_polled_port] = ACTIONS(2279),
    [sym_static_port_binding] = ACTIONS(2279),
    [sym_static_ip_binding] = ACTIONS(2279),
    [sym_get_dns_list] = ACTIONS(2279),
    [sym_set_dns_list] = ACTIONS(2279),
    [sym_get_ip_address] = ACTIONS(2279),
    [sym_set_ip_address] = ACTIONS(2279),
    [sym_get_url_list] = ACTIONS(2279),
    [sym_add_url_entry] = ACTIONS(2279),
    [sym_delete_url_entry] = ACTIONS(2279),
    [sym_abs_value] = ACTIONS(2279),
    [sym_max_value] = ACTIONS(2279),
    [sym_min_value] = ACTIONS(2279),
    [sym_exp_value] = ACTIONS(2279),
    [sym_log_value] = ACTIONS(2279),
    [sym_log10_value] = ACTIONS(2279),
    [sym_power_value] = ACTIONS(2279),
    [sym_sqrt_value] = ACTIONS(2279),
    [sym_raw_be] = ACTIONS(2279),
    [sym_raw_le] = ACTIONS(2279),
    [sym_internal_queue_size_set] = ACTIONS(2279),
    [sym_internal_queue_size_get] = ACTIONS(2279),
    [sym_internal_threshold_set] = ACTIONS(2279),
    [sym_internal_threshold_get] = ACTIONS(2279),
    [sym_rebuilt_event] = ACTIONS(2279),
    [sym_do_custom_event] = ACTIONS(2279),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2279),
    [sym_clkmgr_set_clk_source] = ACTIONS(2279),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2279),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2279),
    [sym_clkmgr_get_timezone] = ACTIONS(2279),
    [sym_clkmgr_set_timezone] = ACTIONS(2279),
    [sym_clkmgr_get_resync_period] = ACTIONS(2279),
    [sym_clkmgr_set_resync_period] = ACTIONS(2279),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2279),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2279),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2279),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2279),
    [sym_clkmgr_get_timeservers] = ACTIONS(2279),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2279),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2279),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2279),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2279),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2279),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2279),
    [sym_set_log_level] = ACTIONS(2279),
    [sym_get_log_level] = ACTIONS(2279),
    [sym_amx_log] = ACTIONS(2279),
    [sym_smtp_server_config_set] = ACTIONS(2279),
    [sym_smtp_server_config_get] = ACTIONS(2279),
    [sym_smtp_send] = ACTIONS(2279),
    [sym_validate_netlinx_account] = ACTIONS(2279),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2279),
    [sym_authenticate_certificate] = ACTIONS(2279),
    [sym_audit_netlinx_session_event] = ACTIONS(2279),
    [sym_audit_netlinx_generic_event] = ACTIONS(2279),
    [sym_get_available_flash_disk_space] = ACTIONS(2279),
    [sym_get_max_flash_disk_space] = ACTIONS(2279),
    [sym_data_create_feed] = ACTIONS(2279),
    [sym_data_delete_feed] = ACTIONS(2279),
    [sym_data_publish_feed] = ACTIONS(2279),
    [sym_data_get_published_feed] = ACTIONS(2279),
    [sym_data_add_record] = ACTIONS(2279),
    [sym_data_get_event_record] = ACTIONS(2279),
    [sym__wc_data_create_feed] = ACTIONS(2279),
    [sym__wc_data_add_record] = ACTIONS(2279),
    [sym__wc_data_get_event_record] = ACTIONS(2279),
    [sym_format] = ACTIONS(2279),
    [sym_astro_clock] = ACTIONS(2279),
    [sym_set_outdoor_temperature] = ACTIONS(2279),
    [sym_set_virtual_level_count] = ACTIONS(2279),
    [sym_set_virtual_channel_count] = ACTIONS(2279),
    [sym_set_virtual_port_count] = ACTIONS(2279),
    [sym__wc_encode_int] = ACTIONS(2279),
    [sym__wc_decode_int] = ACTIONS(2279),
    [sym__wc_int] = ACTIONS(2279),
    [sym__wc_to_ch_int] = ACTIONS(2279),
    [sym__ch_to_wc_int] = ACTIONS(2279),
    [sym__wc_find_string_int] = ACTIONS(2279),
    [sym__wc_left_string_int] = ACTIONS(2279),
    [sym__wc_lower_string_int] = ACTIONS(2279),
    [sym__wc_mid_string_int] = ACTIONS(2279),
    [sym__wc_remove_string_int] = ACTIONS(2279),
    [sym__wc_right_string_int] = ACTIONS(2279),
    [sym__wc_upper_string_int] = ACTIONS(2279),
    [sym__wc_compare_string_int] = ACTIONS(2279),
    [sym__wc_get_buffer_char_int] = ACTIONS(2279),
    [sym__wc_get_buffer_string_int] = ACTIONS(2279),
    [sym__wc_concat_string_int] = ACTIONS(2279),
    [sym__wc_file_open_int] = ACTIONS(2279),
    [sym__wc_file_close_int] = ACTIONS(2279),
    [sym__wc_file_read_int] = ACTIONS(2279),
    [sym__wc_file_read_line_int] = ACTIONS(2279),
    [sym__wc_file_write_int] = ACTIONS(2279),
    [sym__wc_file_write_line_int] = ACTIONS(2279),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2281),
    [aux_sym_preproc_else_token1] = ACTIONS(2281),
    [anon_sym_LBRACK] = ACTIONS(2165),
    [anon_sym_LBRACE] = ACTIONS(2281),
    [anon_sym_COLON] = ACTIONS(2195),
    [anon_sym_LPAREN] = ACTIONS(2167),
    [anon_sym_BANG] = ACTIONS(2279),
    [anon_sym_TILDE] = ACTIONS(2281),
    [anon_sym_DASH] = ACTIONS(2169),
    [anon_sym_PLUS] = ACTIONS(2169),
    [anon_sym_STAR] = ACTIONS(2171),
    [anon_sym_SLASH] = ACTIONS(2173),
    [anon_sym_PERCENT] = ACTIONS(2171),
    [anon_sym_PIPE_PIPE] = ACTIONS(2197),
    [anon_sym_AMP_AMP] = ACTIONS(2199),
    [anon_sym_PIPE] = ACTIONS(2193),
    [anon_sym_CARET] = ACTIONS(2201),
    [anon_sym_AMP] = ACTIONS(2191),
    [anon_sym_EQ_EQ] = ACTIONS(2203),
    [anon_sym_BANG_EQ] = ACTIONS(2203),
    [anon_sym_LT_GT] = ACTIONS(2203),
    [anon_sym_GT] = ACTIONS(2175),
    [anon_sym_GT_EQ] = ACTIONS(2177),
    [anon_sym_LT_EQ] = ACTIONS(2177),
    [anon_sym_LT] = ACTIONS(2175),
    [anon_sym_LT_LT] = ACTIONS(2179),
    [anon_sym_GT_GT] = ACTIONS(2179),
    [anon_sym_DASH_DASH] = ACTIONS(2181),
    [anon_sym_PLUS_PLUS] = ACTIONS(2181),
    [anon_sym_DOT] = ACTIONS(2183),
    [anon_sym_DQUOTE] = ACTIONS(2281),
    [anon_sym_SQUOTE] = ACTIONS(2281),
    [sym_number_literal] = ACTIONS(2281),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(636)] = {
    [sym_preproc_if_defined_in_initializer_list] = STATE(645),
    [sym_preproc_if_not_defined_in_initializer_list] = STATE(645),
    [sym_preproc_if_defined_in_initializer_list_no_comma] = STATE(934),
    [sym_preproc_if_not_defined_in_initializer_list_no_comma] = STATE(934),
    [sym_expression] = STATE(628),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(907),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_initializer_list_repeat1] = STATE(645),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2293),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2295),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_RBRACE] = ACTIONS(2301),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(637)] = {
    [sym_preproc_if_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_not_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_defined_in_initializer_list_no_comma] = STATE(960),
    [sym_preproc_if_not_defined_in_initializer_list_no_comma] = STATE(960),
    [sym_expression] = STATE(622),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(913),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_initializer_list_repeat1] = STATE(643),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2293),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2295),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_RBRACE] = ACTIONS(2315),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(638)] = {
    [sym_preproc_else_in_initializer_list] = STATE(2262),
    [sym_preproc_else_in_initializer_list_no_comma] = STATE(2263),
    [sym_expression] = STATE(634),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(944),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = STATE(653),
    [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = STATE(658),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LF] = ACTIONS(2317),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2319),
    [aux_sym_preproc_else_token1] = ACTIONS(2321),
    [anon_sym_COMMA] = ACTIONS(2323),
    [anon_sym_LBRACK] = ACTIONS(2325),
    [anon_sym_LBRACE] = ACTIONS(2327),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2285),
    [anon_sym_TILDE] = ACTIONS(2285),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2329),
    [anon_sym_PLUS_PLUS] = ACTIONS(2329),
    [anon_sym_DQUOTE] = ACTIONS(2331),
    [anon_sym_SQUOTE] = ACTIONS(2333),
    [sym_number_literal] = ACTIONS(2335),
    [sym_comment] = ACTIONS(287),
  },
  [STATE(639)] = {
    [sym_preproc_if_defined_in_initializer_list] = STATE(641),
    [sym_preproc_if_not_defined_in_initializer_list] = STATE(641),
    [sym_preproc_if_defined_in_initializer_list_no_comma] = STATE(958),
    [sym_preproc_if_not_defined_in_initializer_list_no_comma] = STATE(958),
    [sym_expression] = STATE(614),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(906),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_initializer_list_repeat1] = STATE(641),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2293),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2295),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_RBRACE] = ACTIONS(2337),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(640)] = {
    [sym_preproc_else_in_initializer_list] = STATE(2074),
    [sym_preproc_else_in_initializer_list_no_comma] = STATE(2257),
    [sym_expression] = STATE(634),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(944),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = STATE(655),
    [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = STATE(659),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LF] = ACTIONS(2339),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2341),
    [aux_sym_preproc_else_token1] = ACTIONS(2321),
    [anon_sym_COMMA] = ACTIONS(2323),
    [anon_sym_LBRACK] = ACTIONS(2325),
    [anon_sym_LBRACE] = ACTIONS(2327),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2285),
    [anon_sym_TILDE] = ACTIONS(2285),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2329),
    [anon_sym_PLUS_PLUS] = ACTIONS(2329),
    [anon_sym_DQUOTE] = ACTIONS(2331),
    [anon_sym_SQUOTE] = ACTIONS(2333),
    [sym_number_literal] = ACTIONS(2335),
    [sym_comment] = ACTIONS(287),
  },
  [STATE(641)] = {
    [sym_preproc_if_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_not_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_defined_in_initializer_list_no_comma] = STATE(948),
    [sym_preproc_if_not_defined_in_initializer_list_no_comma] = STATE(948),
    [sym_expression] = STATE(616),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(909),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_initializer_list_repeat1] = STATE(643),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2293),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2295),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_RBRACE] = ACTIONS(2343),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(642)] = {
    [sym_preproc_if_defined_in_initializer_list] = STATE(637),
    [sym_preproc_if_not_defined_in_initializer_list] = STATE(637),
    [sym_preproc_if_defined_in_initializer_list_no_comma] = STATE(927),
    [sym_preproc_if_not_defined_in_initializer_list_no_comma] = STATE(927),
    [sym_expression] = STATE(620),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(912),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_initializer_list_repeat1] = STATE(637),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2293),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2295),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_RBRACE] = ACTIONS(2345),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(643)] = {
    [sym_preproc_if_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_not_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_defined_in_initializer_list_no_comma] = STATE(643),
    [sym_preproc_if_not_defined_in_initializer_list_no_comma] = STATE(643),
    [sym_expression] = STATE(627),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(916),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_initializer_list_repeat1] = STATE(643),
    [sym_identifier] = ACTIONS(2347),
    [sym_not] = ACTIONS(2350),
    [sym_bnot] = ACTIONS(2350),
    [sym___file__] = ACTIONS(2353),
    [sym___date__] = ACTIONS(2353),
    [sym___ldate__] = ACTIONS(2353),
    [sym___time__] = ACTIONS(2353),
    [sym___line__] = ACTIONS(2353),
    [sym___name__] = ACTIONS(2353),
    [sym___netlinx__] = ACTIONS(2356),
    [sym_netlinx_axi_version] = ACTIONS(2356),
    [sym_url_flg_tcp] = ACTIONS(2356),
    [sym_url_flg_accinfopresent] = ACTIONS(2356),
    [sym_url_flg_temp] = ACTIONS(2356),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2356),
    [sym_url_flg_stat_mask] = ACTIONS(2356),
    [sym_url_flg_stat_lookup] = ACTIONS(2356),
    [sym_url_flg_stat_connecting] = ACTIONS(2356),
    [sym_url_flg_stat_waiting] = ACTIONS(2356),
    [sym_url_flg_stat_connected] = ACTIONS(2356),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2356),
    [sym_first_virtual_device] = ACTIONS(2356),
    [sym_first_local_port] = ACTIONS(2356),
    [sym_dynamic_virtual_device] = ACTIONS(2356),
    [sym_timeline_once] = ACTIONS(2356),
    [sym_timeline_repeat] = ACTIONS(2356),
    [sym_timeline_absolute] = ACTIONS(2356),
    [sym_timeline_relative] = ACTIONS(2356),
    [sym_do_push_timed_infinite] = ACTIONS(2356),
    [sym_source_type_no_address] = ACTIONS(2356),
    [sym_source_type_neuron_id] = ACTIONS(2356),
    [sym_source_type_ip_address] = ACTIONS(2356),
    [sym_source_type_axlink] = ACTIONS(2356),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2356),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2356),
    [sym_source_type_ip_socket_address] = ACTIONS(2356),
    [sym_source_type_rs232] = ACTIONS(2356),
    [sym_source_type_internal] = ACTIONS(2356),
    [sym_source_type_mac_address] = ACTIONS(2356),
    [sym_source_type_ipv4_port] = ACTIONS(2356),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2356),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2356),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2356),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2356),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2356),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2356),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2356),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2356),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2356),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2356),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2356),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2356),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2356),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2356),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2356),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2356),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2356),
    [sym_true] = ACTIONS(2356),
    [sym_false] = ACTIONS(2356),
    [sym_file_read_only] = ACTIONS(2356),
    [sym_file_rw_new] = ACTIONS(2356),
    [sym_file_rw_append] = ACTIONS(2356),
    [sym_ip_tcp] = ACTIONS(2356),
    [sym_ip_udp] = ACTIONS(2356),
    [sym_ip_udp_2way] = ACTIONS(2356),
    [sym_xml_encode_types] = ACTIONS(2356),
    [sym_xml_encode_char_as_list] = ACTIONS(2356),
    [sym_xml_encode_le] = ACTIONS(2356),
    [sym_xml_decode_types] = ACTIONS(2356),
    [sym_xml_decode_no_preserve] = ACTIONS(2356),
    [sym_duet_dev_type_display_device] = ACTIONS(2356),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2356),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2356),
    [sym_duet_dev_type_io_device] = ACTIONS(2356),
    [sym_duet_dev_type_relay_device] = ACTIONS(2356),
    [sym_duet_dev_type_ups] = ACTIONS(2356),
    [sym_duet_dev_type_amplifier] = ACTIONS(2356),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2356),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2356),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2356),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2356),
    [sym_duet_dev_type_camera] = ACTIONS(2356),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2356),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2356),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2356),
    [sym_duet_dev_type_dss] = ACTIONS(2356),
    [sym_duet_dev_type_dvr] = ACTIONS(2356),
    [sym_duet_dev_type_disc_device] = ACTIONS(2356),
    [sym_duet_dev_type_document_camera] = ACTIONS(2356),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2356),
    [sym_duet_dev_type_hvac] = ACTIONS(2356),
    [sym_duet_dev_type_keypad] = ACTIONS(2356),
    [sym_duet_dev_type_light] = ACTIONS(2356),
    [sym_duet_dev_type_monitor] = ACTIONS(2356),
    [sym_duet_dev_type_motor] = ACTIONS(2356),
    [sym_duet_dev_type_multi_window] = ACTIONS(2356),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2356),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2356),
    [sym_duet_dev_type_receiver] = ACTIONS(2356),
    [sym_duet_dev_type_security_system] = ACTIONS(2356),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2356),
    [sym_duet_dev_type_settop_box] = ACTIONS(2356),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2356),
    [sym_duet_dev_type_switcher] = ACTIONS(2356),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2356),
    [sym_duet_dev_type_tv] = ACTIONS(2356),
    [sym_duet_dev_type_utility] = ACTIONS(2356),
    [sym_duet_dev_type_vcr] = ACTIONS(2356),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2356),
    [sym_duet_dev_type_video_processor] = ACTIONS(2356),
    [sym_duet_dev_type_video_projector] = ACTIONS(2356),
    [sym_duet_dev_type_video_wall] = ACTIONS(2356),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2356),
    [sym_duet_dev_type_weather] = ACTIONS(2356),
    [sym_duet_dev_not_polled] = ACTIONS(2356),
    [sym_duet_dev_polled] = ACTIONS(2356),
    [sym_clkmgr_mode_network] = ACTIONS(2356),
    [sym_clkmgr_mode_standalone] = ACTIONS(2356),
    [sym_normal_standby] = ACTIONS(2356),
    [sym_normal_wake] = ACTIONS(2356),
    [sym_ip_multicast_ttl_option] = ACTIONS(2356),
    [sym_ip_ttl_subnet] = ACTIONS(2356),
    [sym_ip_ttl_site] = ACTIONS(2356),
    [sym_ip_ttl_region] = ACTIONS(2356),
    [sym_ip_ttl_continent] = ACTIONS(2356),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2356),
    [sym_ip_nodelay_on] = ACTIONS(2356),
    [sym_ip_nodelay_off] = ACTIONS(2356),
    [sym_amx_error] = ACTIONS(2356),
    [sym_amx_warning] = ACTIONS(2356),
    [sym_amx_info] = ACTIONS(2356),
    [sym_amx_debug] = ACTIONS(2356),
    [sym_netlinx_login_success] = ACTIONS(2356),
    [sym_netlinx_login_fail] = ACTIONS(2356),
    [sym_netlinx_logout] = ACTIONS(2356),
    [sym_null_str] = ACTIONS(2356),
    [sym_smtp_address] = ACTIONS(2356),
    [sym_smtp_port_number] = ACTIONS(2356),
    [sym_smtp_username] = ACTIONS(2356),
    [sym_smtp_password] = ACTIONS(2356),
    [sym_smtp_require_tls] = ACTIONS(2356),
    [sym_smtp_from] = ACTIONS(2356),
    [sym_smtp_tls_true] = ACTIONS(2356),
    [sym_smtp_tls_false] = ACTIONS(2356),
    [sym_listview_on_row_select_event] = ACTIONS(2356),
    [sym_data_structure_unknown] = ACTIONS(2356),
    [sym_data_structure_datafeed] = ACTIONS(2356),
    [sym_data_structure_datarecordset] = ACTIONS(2356),
    [sym_data_structure_datarecord] = ACTIONS(2356),
    [sym_data_structure_datafield] = ACTIONS(2356),
    [sym_data_max_content_field_count] = ACTIONS(2356),
    [sym_data_max_metadata_field_count] = ACTIONS(2356),
    [sym_data_max_id_length] = ACTIONS(2356),
    [sym_data_max_name_length] = ACTIONS(2356),
    [sym_data_max_description_length] = ACTIONS(2356),
    [sym_data_max_type_length] = ACTIONS(2356),
    [sym_data_max_format_length] = ACTIONS(2356),
    [sym_data_max_source_length] = ACTIONS(2356),
    [sym_data_max_value_length] = ACTIONS(2356),
    [sym_data_max_label_length] = ACTIONS(2356),
    [sym_data_max_url_length] = ACTIONS(2356),
    [sym_max_last_login_info_length] = ACTIONS(2356),
    [sym_data_type_unknown] = ACTIONS(2356),
    [sym_data_type_string] = ACTIONS(2356),
    [sym_data_type_datetime] = ACTIONS(2356),
    [sym_data_type_date] = ACTIONS(2356),
    [sym_data_type_time] = ACTIONS(2356),
    [sym_data_type_image] = ACTIONS(2356),
    [sym_data_format_url] = ACTIONS(2356),
    [sym_data_format_phone] = ACTIONS(2356),
    [sym_data_format_email] = ACTIONS(2356),
    [sym_data_format_iso8601] = ACTIONS(2356),
    [sym_auditlog_priv] = ACTIONS(2356),
    [sym_configuration_priv] = ACTIONS(2356),
    [sym_deviceconfig_priv] = ACTIONS(2356),
    [sym_ftp_priv] = ACTIONS(2356),
    [sym_http_priv] = ACTIONS(2356),
    [sym_networkconfig_priv] = ACTIONS(2356),
    [sym_programport_priv] = ACTIONS(2356),
    [sym_remoteui_priv] = ACTIONS(2356),
    [sym_securitycontrol_priv] = ACTIONS(2356),
    [sym_softwaremanagement_priv] = ACTIONS(2356),
    [sym_terminal_priv] = ACTIONS(2356),
    [sym_tpadmin_priv] = ACTIONS(2356),
    [sym_user1_priv] = ACTIONS(2356),
    [sym_user2_priv] = ACTIONS(2356),
    [sym_user3_priv] = ACTIONS(2356),
    [sym_user4_priv] = ACTIONS(2356),
    [sym_usermanagement_priv] = ACTIONS(2356),
    [sym_valid_account] = ACTIONS(2356),
    [sym_err_name_invalid] = ACTIONS(2356),
    [sym_err_password_invalid] = ACTIONS(2356),
    [sym_err_invalid_account] = ACTIONS(2356),
    [sym_err_no_authorization] = ACTIONS(2356),
    [sym_err_invalid_parameter] = ACTIONS(2356),
    [sym_err_user_locked_out] = ACTIONS(2356),
    [sym_err_user_account_expired] = ACTIONS(2356),
    [sym_err_general_error] = ACTIONS(2356),
    [sym_err_authentication_error] = ACTIONS(2356),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2356),
    [sym_tls_validate_certificate] = ACTIONS(2356),
    [sym____reserved___] = ACTIONS(2359),
    [sym_date] = ACTIONS(2359),
    [sym_day] = ACTIONS(2359),
    [sym_time] = ACTIONS(2359),
    [sym_ldate] = ACTIONS(2359),
    [sym_push_device] = ACTIONS(2359),
    [sym_push_channel] = ACTIONS(2359),
    [sym_push_devchan] = ACTIONS(2359),
    [sym_release_device] = ACTIONS(2359),
    [sym_release_channel] = ACTIONS(2359),
    [sym_release_devchan] = ACTIONS(2359),
    [sym_master_slot] = ACTIONS(2359),
    [sym_get_pulse_time] = ACTIONS(2359),
    [sym_get_timer] = ACTIONS(2359),
    [sym_master_sn] = ACTIONS(2359),
    [sym_system_number] = ACTIONS(2359),
    [sym_button] = ACTIONS(2359),
    [sym_dv_channel] = ACTIONS(2359),
    [sym_data] = ACTIONS(2359),
    [sym_level] = ACTIONS(2359),
    [sym_timeline] = ACTIONS(2359),
    [sym_channel] = ACTIONS(2359),
    [sym_custom] = ACTIONS(2359),
    [sym_atoi] = ACTIONS(2362),
    [sym_atol] = ACTIONS(2362),
    [sym_atof] = ACTIONS(2362),
    [sym_itoa] = ACTIONS(2362),
    [sym_itohex] = ACTIONS(2362),
    [sym_hextoi] = ACTIONS(2362),
    [sym_ftoa] = ACTIONS(2362),
    [sym_find_string] = ACTIONS(2362),
    [sym_left_string] = ACTIONS(2362),
    [sym_right_string] = ACTIONS(2362),
    [sym_mid_string] = ACTIONS(2362),
    [sym_remove_string] = ACTIONS(2362),
    [sym_get_buffer_string] = ACTIONS(2362),
    [sym_get_buffer_char] = ACTIONS(2362),
    [sym_get_multi_buffer_string] = ACTIONS(2362),
    [sym_length_string] = ACTIONS(2362),
    [sym_max_length_string] = ACTIONS(2362),
    [sym_set_length_string] = ACTIONS(2362),
    [sym_compare_string] = ACTIONS(2362),
    [sym_upper_string] = ACTIONS(2362),
    [sym_lower_string] = ACTIONS(2362),
    [sym_get_last] = ACTIONS(2362),
    [sym_date_to_day] = ACTIONS(2362),
    [sym_date_to_month] = ACTIONS(2362),
    [sym_date_to_year] = ACTIONS(2362),
    [sym_time_to_hour] = ACTIONS(2362),
    [sym_time_to_minute] = ACTIONS(2362),
    [sym_time_to_second] = ACTIONS(2362),
    [sym_day_of_week] = ACTIONS(2362),
    [sym_set_pulse_time] = ACTIONS(2362),
    [sym_set_timer] = ACTIONS(2362),
    [sym_file_open] = ACTIONS(2362),
    [sym_file_close] = ACTIONS(2362),
    [sym_file_read] = ACTIONS(2362),
    [sym_file_read_line] = ACTIONS(2362),
    [sym_file_write] = ACTIONS(2362),
    [sym_file_write_line] = ACTIONS(2362),
    [sym_file_copy] = ACTIONS(2362),
    [sym_file_delete] = ACTIONS(2362),
    [sym_file_rename] = ACTIONS(2362),
    [sym_file_dir] = ACTIONS(2362),
    [sym_file_getdir] = ACTIONS(2362),
    [sym_file_setdir] = ACTIONS(2362),
    [sym_file_create_dir] = ACTIONS(2362),
    [sym_file_remove_dir] = ACTIONS(2362),
    [sym_file_seek] = ACTIONS(2362),
    [sym_length_array] = ACTIONS(2362),
    [sym_max_length_array] = ACTIONS(2362),
    [sym_set_length_array] = ACTIONS(2362),
    [sym_ip_client_open] = ACTIONS(2362),
    [sym_ip_client_close] = ACTIONS(2362),
    [sym_ip_server_open] = ACTIONS(2362),
    [sym_ip_server_close] = ACTIONS(2362),
    [sym_ip_bound_client_open] = ACTIONS(2362),
    [sym_ip_mc_server_open] = ACTIONS(2362),
    [sym_ip_set_option] = ACTIONS(2362),
    [sym_ssh_client_open] = ACTIONS(2362),
    [sym_ssh_client_close] = ACTIONS(2362),
    [sym_tls_client_open] = ACTIONS(2362),
    [sym_tls_client_close] = ACTIONS(2362),
    [sym_do_push] = ACTIONS(2362),
    [sym_do_push_timed] = ACTIONS(2362),
    [sym_do_release] = ACTIONS(2362),
    [sym_device_id] = ACTIONS(2362),
    [sym_device_info] = ACTIONS(2362),
    [sym_device_id_string] = ACTIONS(2362),
    [sym_device_standby] = ACTIONS(2362),
    [sym_device_wake] = ACTIONS(2362),
    [sym_timeline_create] = ACTIONS(2362),
    [sym_timeline_kill] = ACTIONS(2362),
    [sym_timeline_reload] = ACTIONS(2362),
    [sym_timeline_pause] = ACTIONS(2362),
    [sym_timeline_restart] = ACTIONS(2362),
    [sym_timeline_set] = ACTIONS(2362),
    [sym_timeline_get] = ACTIONS(2362),
    [sym_timeline_active] = ACTIONS(2362),
    [sym_timeline_dynamic_id] = ACTIONS(2362),
    [sym_variable_to_string] = ACTIONS(2362),
    [sym_string_to_variable] = ACTIONS(2362),
    [sym_length_variable_to_string] = ACTIONS(2362),
    [sym_type_cast] = ACTIONS(2362),
    [sym_variable_to_xml] = ACTIONS(2362),
    [sym_xml_to_variable] = ACTIONS(2362),
    [sym_length_variable_to_xml] = ACTIONS(2362),
    [sym_redirect_string] = ACTIONS(2362),
    [sym_random_number] = ACTIONS(2362),
    [sym_get_unique_id] = ACTIONS(2362),
    [sym_get_system_number] = ACTIONS(2362),
    [sym_set_system_number] = ACTIONS(2362),
    [sym_get_serial_number] = ACTIONS(2362),
    [sym_get_master_build] = ACTIONS(2362),
    [sym_set_validation_code] = ACTIONS(2362),
    [sym_reboot] = ACTIONS(2362),
    [sym_load_duet_module] = ACTIONS(2362),
    [sym_unload_duet_module] = ACTIONS(2362),
    [sym_duet_mem_size_set] = ACTIONS(2362),
    [sym_duet_mem_size_get] = ACTIONS(2362),
    [sym_dynamic_application_device] = ACTIONS(2362),
    [sym_dynamic_polled_port] = ACTIONS(2362),
    [sym_static_port_binding] = ACTIONS(2362),
    [sym_static_ip_binding] = ACTIONS(2362),
    [sym_get_dns_list] = ACTIONS(2362),
    [sym_set_dns_list] = ACTIONS(2362),
    [sym_get_ip_address] = ACTIONS(2362),
    [sym_set_ip_address] = ACTIONS(2362),
    [sym_get_url_list] = ACTIONS(2362),
    [sym_add_url_entry] = ACTIONS(2362),
    [sym_delete_url_entry] = ACTIONS(2362),
    [sym_abs_value] = ACTIONS(2362),
    [sym_max_value] = ACTIONS(2362),
    [sym_min_value] = ACTIONS(2362),
    [sym_exp_value] = ACTIONS(2362),
    [sym_log_value] = ACTIONS(2362),
    [sym_log10_value] = ACTIONS(2362),
    [sym_power_value] = ACTIONS(2362),
    [sym_sqrt_value] = ACTIONS(2362),
    [sym_raw_be] = ACTIONS(2362),
    [sym_raw_le] = ACTIONS(2362),
    [sym_internal_queue_size_set] = ACTIONS(2362),
    [sym_internal_queue_size_get] = ACTIONS(2362),
    [sym_internal_threshold_set] = ACTIONS(2362),
    [sym_internal_threshold_get] = ACTIONS(2362),
    [sym_rebuilt_event] = ACTIONS(2362),
    [sym_do_custom_event] = ACTIONS(2362),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2362),
    [sym_clkmgr_set_clk_source] = ACTIONS(2362),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2362),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2362),
    [sym_clkmgr_get_timezone] = ACTIONS(2362),
    [sym_clkmgr_set_timezone] = ACTIONS(2362),
    [sym_clkmgr_get_resync_period] = ACTIONS(2362),
    [sym_clkmgr_set_resync_period] = ACTIONS(2362),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2362),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2362),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2362),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2362),
    [sym_clkmgr_get_timeservers] = ACTIONS(2362),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2362),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2362),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2362),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2362),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2362),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2362),
    [sym_set_log_level] = ACTIONS(2362),
    [sym_get_log_level] = ACTIONS(2362),
    [sym_amx_log] = ACTIONS(2362),
    [sym_smtp_server_config_set] = ACTIONS(2362),
    [sym_smtp_server_config_get] = ACTIONS(2362),
    [sym_smtp_send] = ACTIONS(2362),
    [sym_validate_netlinx_account] = ACTIONS(2362),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2362),
    [sym_authenticate_certificate] = ACTIONS(2362),
    [sym_audit_netlinx_session_event] = ACTIONS(2362),
    [sym_audit_netlinx_generic_event] = ACTIONS(2362),
    [sym_get_available_flash_disk_space] = ACTIONS(2362),
    [sym_get_max_flash_disk_space] = ACTIONS(2362),
    [sym_data_create_feed] = ACTIONS(2362),
    [sym_data_delete_feed] = ACTIONS(2362),
    [sym_data_publish_feed] = ACTIONS(2362),
    [sym_data_get_published_feed] = ACTIONS(2362),
    [sym_data_add_record] = ACTIONS(2362),
    [sym_data_get_event_record] = ACTIONS(2362),
    [sym__wc_data_create_feed] = ACTIONS(2362),
    [sym__wc_data_add_record] = ACTIONS(2362),
    [sym__wc_data_get_event_record] = ACTIONS(2362),
    [sym_format] = ACTIONS(2362),
    [sym_astro_clock] = ACTIONS(2362),
    [sym_set_outdoor_temperature] = ACTIONS(2362),
    [sym_set_virtual_level_count] = ACTIONS(2362),
    [sym_set_virtual_channel_count] = ACTIONS(2362),
    [sym_set_virtual_port_count] = ACTIONS(2362),
    [sym__wc_encode_int] = ACTIONS(2362),
    [sym__wc_decode_int] = ACTIONS(2362),
    [sym__wc_int] = ACTIONS(2362),
    [sym__wc_to_ch_int] = ACTIONS(2362),
    [sym__ch_to_wc_int] = ACTIONS(2362),
    [sym__wc_find_string_int] = ACTIONS(2362),
    [sym__wc_left_string_int] = ACTIONS(2362),
    [sym__wc_lower_string_int] = ACTIONS(2362),
    [sym__wc_mid_string_int] = ACTIONS(2362),
    [sym__wc_remove_string_int] = ACTIONS(2362),
    [sym__wc_right_string_int] = ACTIONS(2362),
    [sym__wc_upper_string_int] = ACTIONS(2362),
    [sym__wc_compare_string_int] = ACTIONS(2362),
    [sym__wc_get_buffer_char_int] = ACTIONS(2362),
    [sym__wc_get_buffer_string_int] = ACTIONS(2362),
    [sym__wc_concat_string_int] = ACTIONS(2362),
    [sym__wc_file_open_int] = ACTIONS(2362),
    [sym__wc_file_close_int] = ACTIONS(2362),
    [sym__wc_file_read_int] = ACTIONS(2362),
    [sym__wc_file_read_line_int] = ACTIONS(2362),
    [sym__wc_file_write_int] = ACTIONS(2362),
    [sym__wc_file_write_line_int] = ACTIONS(2362),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2365),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2368),
    [anon_sym_LBRACK] = ACTIONS(2371),
    [anon_sym_LBRACE] = ACTIONS(2374),
    [anon_sym_RBRACE] = ACTIONS(2377),
    [anon_sym_LPAREN] = ACTIONS(2379),
    [anon_sym_BANG] = ACTIONS(2382),
    [anon_sym_TILDE] = ACTIONS(2382),
    [anon_sym_DASH] = ACTIONS(2350),
    [anon_sym_PLUS] = ACTIONS(2350),
    [anon_sym_DASH_DASH] = ACTIONS(2385),
    [anon_sym_PLUS_PLUS] = ACTIONS(2385),
    [anon_sym_DQUOTE] = ACTIONS(2388),
    [anon_sym_SQUOTE] = ACTIONS(2391),
    [sym_number_literal] = ACTIONS(2394),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(644)] = {
    [sym_preproc_if_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_not_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_defined_in_initializer_list_no_comma] = STATE(925),
    [sym_preproc_if_not_defined_in_initializer_list_no_comma] = STATE(925),
    [sym_expression] = STATE(607),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(911),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_initializer_list_repeat1] = STATE(643),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2293),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2295),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_RBRACE] = ACTIONS(2397),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(645)] = {
    [sym_preproc_if_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_not_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_defined_in_initializer_list_no_comma] = STATE(952),
    [sym_preproc_if_not_defined_in_initializer_list_no_comma] = STATE(952),
    [sym_expression] = STATE(629),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(908),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_initializer_list_repeat1] = STATE(643),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2293),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2295),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_RBRACE] = ACTIONS(2399),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(646)] = {
    [sym_preproc_if_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_not_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_defined_in_initializer_list_no_comma] = STATE(947),
    [sym_preproc_if_not_defined_in_initializer_list_no_comma] = STATE(947),
    [sym_expression] = STATE(624),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(915),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_initializer_list_repeat1] = STATE(643),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2293),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2295),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_RBRACE] = ACTIONS(2401),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(647)] = {
    [sym_preproc_if_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_not_defined_in_initializer_list] = STATE(643),
    [sym_preproc_if_defined_in_initializer_list_no_comma] = STATE(953),
    [sym_preproc_if_not_defined_in_initializer_list_no_comma] = STATE(953),
    [sym_expression] = STATE(626),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(918),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_initializer_list_repeat1] = STATE(643),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2293),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2295),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_RBRACE] = ACTIONS(2403),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(648)] = {
    [sym_preproc_if_defined_in_initializer_list] = STATE(646),
    [sym_preproc_if_not_defined_in_initializer_list] = STATE(646),
    [sym_preproc_if_defined_in_initializer_list_no_comma] = STATE(945),
    [sym_preproc_if_not_defined_in_initializer_list_no_comma] = STATE(945),
    [sym_expression] = STATE(623),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(914),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_initializer_list_repeat1] = STATE(646),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2293),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2295),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_RBRACE] = ACTIONS(2405),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(649)] = {
    [sym_preproc_if_defined_in_initializer_list] = STATE(644),
    [sym_preproc_if_not_defined_in_initializer_list] = STATE(644),
    [sym_preproc_if_defined_in_initializer_list_no_comma] = STATE(938),
    [sym_preproc_if_not_defined_in_initializer_list_no_comma] = STATE(938),
    [sym_expression] = STATE(617),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(910),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_initializer_list_repeat1] = STATE(644),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2293),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2295),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_RBRACE] = ACTIONS(2407),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(650)] = {
    [sym_preproc_if_defined_in_initializer_list] = STATE(647),
    [sym_preproc_if_not_defined_in_initializer_list] = STATE(647),
    [sym_preproc_if_defined_in_initializer_list_no_comma] = STATE(951),
    [sym_preproc_if_not_defined_in_initializer_list_no_comma] = STATE(951),
    [sym_expression] = STATE(625),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(917),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_initializer_list_repeat1] = STATE(647),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2293),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2295),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_RBRACE] = ACTIONS(2409),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(651)] = {
    [sym_preproc_else_in_initializer_list] = STATE(2084),
    [sym_preproc_else_in_initializer_list_no_comma] = STATE(2085),
    [sym_expression] = STATE(634),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(944),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = STATE(654),
    [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = STATE(661),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2411),
    [aux_sym_preproc_else_token1] = ACTIONS(2413),
    [anon_sym_COMMA] = ACTIONS(2415),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(652)] = {
    [sym_preproc_else_in_initializer_list] = STATE(2142),
    [sym_preproc_else_in_initializer_list_no_comma] = STATE(2143),
    [sym_expression] = STATE(634),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(944),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = STATE(656),
    [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = STATE(660),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2417),
    [aux_sym_preproc_else_token1] = ACTIONS(2413),
    [anon_sym_COMMA] = ACTIONS(2415),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(653)] = {
    [sym_preproc_else_in_initializer_list] = STATE(2084),
    [sym_expression] = STATE(631),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(946),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = STATE(657),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2419),
    [aux_sym_preproc_else_token1] = ACTIONS(2421),
    [anon_sym_COMMA] = ACTIONS(2415),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(654)] = {
    [sym_preproc_else_in_initializer_list] = STATE(2128),
    [sym_expression] = STATE(631),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(946),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = STATE(657),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2423),
    [aux_sym_preproc_else_token1] = ACTIONS(2421),
    [anon_sym_COMMA] = ACTIONS(2415),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(655)] = {
    [sym_preproc_else_in_initializer_list] = STATE(2142),
    [sym_expression] = STATE(631),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(946),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = STATE(657),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2425),
    [aux_sym_preproc_else_token1] = ACTIONS(2421),
    [anon_sym_COMMA] = ACTIONS(2415),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(656)] = {
    [sym_preproc_else_in_initializer_list] = STATE(2190),
    [sym_expression] = STATE(631),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(946),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = STATE(657),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2427),
    [aux_sym_preproc_else_token1] = ACTIONS(2421),
    [anon_sym_COMMA] = ACTIONS(2415),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(657)] = {
    [sym_expression] = STATE(631),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(946),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = STATE(657),
    [sym_identifier] = ACTIONS(2429),
    [sym_not] = ACTIONS(2432),
    [sym_bnot] = ACTIONS(2432),
    [sym___file__] = ACTIONS(2435),
    [sym___date__] = ACTIONS(2435),
    [sym___ldate__] = ACTIONS(2435),
    [sym___time__] = ACTIONS(2435),
    [sym___line__] = ACTIONS(2435),
    [sym___name__] = ACTIONS(2435),
    [sym___netlinx__] = ACTIONS(2438),
    [sym_netlinx_axi_version] = ACTIONS(2438),
    [sym_url_flg_tcp] = ACTIONS(2438),
    [sym_url_flg_accinfopresent] = ACTIONS(2438),
    [sym_url_flg_temp] = ACTIONS(2438),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2438),
    [sym_url_flg_stat_mask] = ACTIONS(2438),
    [sym_url_flg_stat_lookup] = ACTIONS(2438),
    [sym_url_flg_stat_connecting] = ACTIONS(2438),
    [sym_url_flg_stat_waiting] = ACTIONS(2438),
    [sym_url_flg_stat_connected] = ACTIONS(2438),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2438),
    [sym_first_virtual_device] = ACTIONS(2438),
    [sym_first_local_port] = ACTIONS(2438),
    [sym_dynamic_virtual_device] = ACTIONS(2438),
    [sym_timeline_once] = ACTIONS(2438),
    [sym_timeline_repeat] = ACTIONS(2438),
    [sym_timeline_absolute] = ACTIONS(2438),
    [sym_timeline_relative] = ACTIONS(2438),
    [sym_do_push_timed_infinite] = ACTIONS(2438),
    [sym_source_type_no_address] = ACTIONS(2438),
    [sym_source_type_neuron_id] = ACTIONS(2438),
    [sym_source_type_ip_address] = ACTIONS(2438),
    [sym_source_type_axlink] = ACTIONS(2438),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2438),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2438),
    [sym_source_type_ip_socket_address] = ACTIONS(2438),
    [sym_source_type_rs232] = ACTIONS(2438),
    [sym_source_type_internal] = ACTIONS(2438),
    [sym_source_type_mac_address] = ACTIONS(2438),
    [sym_source_type_ipv4_port] = ACTIONS(2438),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2438),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2438),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2438),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2438),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2438),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2438),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2438),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2438),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2438),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2438),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2438),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2438),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2438),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2438),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2438),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2438),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2438),
    [sym_true] = ACTIONS(2438),
    [sym_false] = ACTIONS(2438),
    [sym_file_read_only] = ACTIONS(2438),
    [sym_file_rw_new] = ACTIONS(2438),
    [sym_file_rw_append] = ACTIONS(2438),
    [sym_ip_tcp] = ACTIONS(2438),
    [sym_ip_udp] = ACTIONS(2438),
    [sym_ip_udp_2way] = ACTIONS(2438),
    [sym_xml_encode_types] = ACTIONS(2438),
    [sym_xml_encode_char_as_list] = ACTIONS(2438),
    [sym_xml_encode_le] = ACTIONS(2438),
    [sym_xml_decode_types] = ACTIONS(2438),
    [sym_xml_decode_no_preserve] = ACTIONS(2438),
    [sym_duet_dev_type_display_device] = ACTIONS(2438),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2438),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2438),
    [sym_duet_dev_type_io_device] = ACTIONS(2438),
    [sym_duet_dev_type_relay_device] = ACTIONS(2438),
    [sym_duet_dev_type_ups] = ACTIONS(2438),
    [sym_duet_dev_type_amplifier] = ACTIONS(2438),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2438),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2438),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2438),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2438),
    [sym_duet_dev_type_camera] = ACTIONS(2438),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2438),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2438),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2438),
    [sym_duet_dev_type_dss] = ACTIONS(2438),
    [sym_duet_dev_type_dvr] = ACTIONS(2438),
    [sym_duet_dev_type_disc_device] = ACTIONS(2438),
    [sym_duet_dev_type_document_camera] = ACTIONS(2438),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2438),
    [sym_duet_dev_type_hvac] = ACTIONS(2438),
    [sym_duet_dev_type_keypad] = ACTIONS(2438),
    [sym_duet_dev_type_light] = ACTIONS(2438),
    [sym_duet_dev_type_monitor] = ACTIONS(2438),
    [sym_duet_dev_type_motor] = ACTIONS(2438),
    [sym_duet_dev_type_multi_window] = ACTIONS(2438),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2438),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2438),
    [sym_duet_dev_type_receiver] = ACTIONS(2438),
    [sym_duet_dev_type_security_system] = ACTIONS(2438),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2438),
    [sym_duet_dev_type_settop_box] = ACTIONS(2438),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2438),
    [sym_duet_dev_type_switcher] = ACTIONS(2438),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2438),
    [sym_duet_dev_type_tv] = ACTIONS(2438),
    [sym_duet_dev_type_utility] = ACTIONS(2438),
    [sym_duet_dev_type_vcr] = ACTIONS(2438),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2438),
    [sym_duet_dev_type_video_processor] = ACTIONS(2438),
    [sym_duet_dev_type_video_projector] = ACTIONS(2438),
    [sym_duet_dev_type_video_wall] = ACTIONS(2438),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2438),
    [sym_duet_dev_type_weather] = ACTIONS(2438),
    [sym_duet_dev_not_polled] = ACTIONS(2438),
    [sym_duet_dev_polled] = ACTIONS(2438),
    [sym_clkmgr_mode_network] = ACTIONS(2438),
    [sym_clkmgr_mode_standalone] = ACTIONS(2438),
    [sym_normal_standby] = ACTIONS(2438),
    [sym_normal_wake] = ACTIONS(2438),
    [sym_ip_multicast_ttl_option] = ACTIONS(2438),
    [sym_ip_ttl_subnet] = ACTIONS(2438),
    [sym_ip_ttl_site] = ACTIONS(2438),
    [sym_ip_ttl_region] = ACTIONS(2438),
    [sym_ip_ttl_continent] = ACTIONS(2438),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2438),
    [sym_ip_nodelay_on] = ACTIONS(2438),
    [sym_ip_nodelay_off] = ACTIONS(2438),
    [sym_amx_error] = ACTIONS(2438),
    [sym_amx_warning] = ACTIONS(2438),
    [sym_amx_info] = ACTIONS(2438),
    [sym_amx_debug] = ACTIONS(2438),
    [sym_netlinx_login_success] = ACTIONS(2438),
    [sym_netlinx_login_fail] = ACTIONS(2438),
    [sym_netlinx_logout] = ACTIONS(2438),
    [sym_null_str] = ACTIONS(2438),
    [sym_smtp_address] = ACTIONS(2438),
    [sym_smtp_port_number] = ACTIONS(2438),
    [sym_smtp_username] = ACTIONS(2438),
    [sym_smtp_password] = ACTIONS(2438),
    [sym_smtp_require_tls] = ACTIONS(2438),
    [sym_smtp_from] = ACTIONS(2438),
    [sym_smtp_tls_true] = ACTIONS(2438),
    [sym_smtp_tls_false] = ACTIONS(2438),
    [sym_listview_on_row_select_event] = ACTIONS(2438),
    [sym_data_structure_unknown] = ACTIONS(2438),
    [sym_data_structure_datafeed] = ACTIONS(2438),
    [sym_data_structure_datarecordset] = ACTIONS(2438),
    [sym_data_structure_datarecord] = ACTIONS(2438),
    [sym_data_structure_datafield] = ACTIONS(2438),
    [sym_data_max_content_field_count] = ACTIONS(2438),
    [sym_data_max_metadata_field_count] = ACTIONS(2438),
    [sym_data_max_id_length] = ACTIONS(2438),
    [sym_data_max_name_length] = ACTIONS(2438),
    [sym_data_max_description_length] = ACTIONS(2438),
    [sym_data_max_type_length] = ACTIONS(2438),
    [sym_data_max_format_length] = ACTIONS(2438),
    [sym_data_max_source_length] = ACTIONS(2438),
    [sym_data_max_value_length] = ACTIONS(2438),
    [sym_data_max_label_length] = ACTIONS(2438),
    [sym_data_max_url_length] = ACTIONS(2438),
    [sym_max_last_login_info_length] = ACTIONS(2438),
    [sym_data_type_unknown] = ACTIONS(2438),
    [sym_data_type_string] = ACTIONS(2438),
    [sym_data_type_datetime] = ACTIONS(2438),
    [sym_data_type_date] = ACTIONS(2438),
    [sym_data_type_time] = ACTIONS(2438),
    [sym_data_type_image] = ACTIONS(2438),
    [sym_data_format_url] = ACTIONS(2438),
    [sym_data_format_phone] = ACTIONS(2438),
    [sym_data_format_email] = ACTIONS(2438),
    [sym_data_format_iso8601] = ACTIONS(2438),
    [sym_auditlog_priv] = ACTIONS(2438),
    [sym_configuration_priv] = ACTIONS(2438),
    [sym_deviceconfig_priv] = ACTIONS(2438),
    [sym_ftp_priv] = ACTIONS(2438),
    [sym_http_priv] = ACTIONS(2438),
    [sym_networkconfig_priv] = ACTIONS(2438),
    [sym_programport_priv] = ACTIONS(2438),
    [sym_remoteui_priv] = ACTIONS(2438),
    [sym_securitycontrol_priv] = ACTIONS(2438),
    [sym_softwaremanagement_priv] = ACTIONS(2438),
    [sym_terminal_priv] = ACTIONS(2438),
    [sym_tpadmin_priv] = ACTIONS(2438),
    [sym_user1_priv] = ACTIONS(2438),
    [sym_user2_priv] = ACTIONS(2438),
    [sym_user3_priv] = ACTIONS(2438),
    [sym_user4_priv] = ACTIONS(2438),
    [sym_usermanagement_priv] = ACTIONS(2438),
    [sym_valid_account] = ACTIONS(2438),
    [sym_err_name_invalid] = ACTIONS(2438),
    [sym_err_password_invalid] = ACTIONS(2438),
    [sym_err_invalid_account] = ACTIONS(2438),
    [sym_err_no_authorization] = ACTIONS(2438),
    [sym_err_invalid_parameter] = ACTIONS(2438),
    [sym_err_user_locked_out] = ACTIONS(2438),
    [sym_err_user_account_expired] = ACTIONS(2438),
    [sym_err_general_error] = ACTIONS(2438),
    [sym_err_authentication_error] = ACTIONS(2438),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2438),
    [sym_tls_validate_certificate] = ACTIONS(2438),
    [sym____reserved___] = ACTIONS(2441),
    [sym_date] = ACTIONS(2441),
    [sym_day] = ACTIONS(2441),
    [sym_time] = ACTIONS(2441),
    [sym_ldate] = ACTIONS(2441),
    [sym_push_device] = ACTIONS(2441),
    [sym_push_channel] = ACTIONS(2441),
    [sym_push_devchan] = ACTIONS(2441),
    [sym_release_device] = ACTIONS(2441),
    [sym_release_channel] = ACTIONS(2441),
    [sym_release_devchan] = ACTIONS(2441),
    [sym_master_slot] = ACTIONS(2441),
    [sym_get_pulse_time] = ACTIONS(2441),
    [sym_get_timer] = ACTIONS(2441),
    [sym_master_sn] = ACTIONS(2441),
    [sym_system_number] = ACTIONS(2441),
    [sym_button] = ACTIONS(2441),
    [sym_dv_channel] = ACTIONS(2441),
    [sym_data] = ACTIONS(2441),
    [sym_level] = ACTIONS(2441),
    [sym_timeline] = ACTIONS(2441),
    [sym_channel] = ACTIONS(2441),
    [sym_custom] = ACTIONS(2441),
    [sym_atoi] = ACTIONS(2444),
    [sym_atol] = ACTIONS(2444),
    [sym_atof] = ACTIONS(2444),
    [sym_itoa] = ACTIONS(2444),
    [sym_itohex] = ACTIONS(2444),
    [sym_hextoi] = ACTIONS(2444),
    [sym_ftoa] = ACTIONS(2444),
    [sym_find_string] = ACTIONS(2444),
    [sym_left_string] = ACTIONS(2444),
    [sym_right_string] = ACTIONS(2444),
    [sym_mid_string] = ACTIONS(2444),
    [sym_remove_string] = ACTIONS(2444),
    [sym_get_buffer_string] = ACTIONS(2444),
    [sym_get_buffer_char] = ACTIONS(2444),
    [sym_get_multi_buffer_string] = ACTIONS(2444),
    [sym_length_string] = ACTIONS(2444),
    [sym_max_length_string] = ACTIONS(2444),
    [sym_set_length_string] = ACTIONS(2444),
    [sym_compare_string] = ACTIONS(2444),
    [sym_upper_string] = ACTIONS(2444),
    [sym_lower_string] = ACTIONS(2444),
    [sym_get_last] = ACTIONS(2444),
    [sym_date_to_day] = ACTIONS(2444),
    [sym_date_to_month] = ACTIONS(2444),
    [sym_date_to_year] = ACTIONS(2444),
    [sym_time_to_hour] = ACTIONS(2444),
    [sym_time_to_minute] = ACTIONS(2444),
    [sym_time_to_second] = ACTIONS(2444),
    [sym_day_of_week] = ACTIONS(2444),
    [sym_set_pulse_time] = ACTIONS(2444),
    [sym_set_timer] = ACTIONS(2444),
    [sym_file_open] = ACTIONS(2444),
    [sym_file_close] = ACTIONS(2444),
    [sym_file_read] = ACTIONS(2444),
    [sym_file_read_line] = ACTIONS(2444),
    [sym_file_write] = ACTIONS(2444),
    [sym_file_write_line] = ACTIONS(2444),
    [sym_file_copy] = ACTIONS(2444),
    [sym_file_delete] = ACTIONS(2444),
    [sym_file_rename] = ACTIONS(2444),
    [sym_file_dir] = ACTIONS(2444),
    [sym_file_getdir] = ACTIONS(2444),
    [sym_file_setdir] = ACTIONS(2444),
    [sym_file_create_dir] = ACTIONS(2444),
    [sym_file_remove_dir] = ACTIONS(2444),
    [sym_file_seek] = ACTIONS(2444),
    [sym_length_array] = ACTIONS(2444),
    [sym_max_length_array] = ACTIONS(2444),
    [sym_set_length_array] = ACTIONS(2444),
    [sym_ip_client_open] = ACTIONS(2444),
    [sym_ip_client_close] = ACTIONS(2444),
    [sym_ip_server_open] = ACTIONS(2444),
    [sym_ip_server_close] = ACTIONS(2444),
    [sym_ip_bound_client_open] = ACTIONS(2444),
    [sym_ip_mc_server_open] = ACTIONS(2444),
    [sym_ip_set_option] = ACTIONS(2444),
    [sym_ssh_client_open] = ACTIONS(2444),
    [sym_ssh_client_close] = ACTIONS(2444),
    [sym_tls_client_open] = ACTIONS(2444),
    [sym_tls_client_close] = ACTIONS(2444),
    [sym_do_push] = ACTIONS(2444),
    [sym_do_push_timed] = ACTIONS(2444),
    [sym_do_release] = ACTIONS(2444),
    [sym_device_id] = ACTIONS(2444),
    [sym_device_info] = ACTIONS(2444),
    [sym_device_id_string] = ACTIONS(2444),
    [sym_device_standby] = ACTIONS(2444),
    [sym_device_wake] = ACTIONS(2444),
    [sym_timeline_create] = ACTIONS(2444),
    [sym_timeline_kill] = ACTIONS(2444),
    [sym_timeline_reload] = ACTIONS(2444),
    [sym_timeline_pause] = ACTIONS(2444),
    [sym_timeline_restart] = ACTIONS(2444),
    [sym_timeline_set] = ACTIONS(2444),
    [sym_timeline_get] = ACTIONS(2444),
    [sym_timeline_active] = ACTIONS(2444),
    [sym_timeline_dynamic_id] = ACTIONS(2444),
    [sym_variable_to_string] = ACTIONS(2444),
    [sym_string_to_variable] = ACTIONS(2444),
    [sym_length_variable_to_string] = ACTIONS(2444),
    [sym_type_cast] = ACTIONS(2444),
    [sym_variable_to_xml] = ACTIONS(2444),
    [sym_xml_to_variable] = ACTIONS(2444),
    [sym_length_variable_to_xml] = ACTIONS(2444),
    [sym_redirect_string] = ACTIONS(2444),
    [sym_random_number] = ACTIONS(2444),
    [sym_get_unique_id] = ACTIONS(2444),
    [sym_get_system_number] = ACTIONS(2444),
    [sym_set_system_number] = ACTIONS(2444),
    [sym_get_serial_number] = ACTIONS(2444),
    [sym_get_master_build] = ACTIONS(2444),
    [sym_set_validation_code] = ACTIONS(2444),
    [sym_reboot] = ACTIONS(2444),
    [sym_load_duet_module] = ACTIONS(2444),
    [sym_unload_duet_module] = ACTIONS(2444),
    [sym_duet_mem_size_set] = ACTIONS(2444),
    [sym_duet_mem_size_get] = ACTIONS(2444),
    [sym_dynamic_application_device] = ACTIONS(2444),
    [sym_dynamic_polled_port] = ACTIONS(2444),
    [sym_static_port_binding] = ACTIONS(2444),
    [sym_static_ip_binding] = ACTIONS(2444),
    [sym_get_dns_list] = ACTIONS(2444),
    [sym_set_dns_list] = ACTIONS(2444),
    [sym_get_ip_address] = ACTIONS(2444),
    [sym_set_ip_address] = ACTIONS(2444),
    [sym_get_url_list] = ACTIONS(2444),
    [sym_add_url_entry] = ACTIONS(2444),
    [sym_delete_url_entry] = ACTIONS(2444),
    [sym_abs_value] = ACTIONS(2444),
    [sym_max_value] = ACTIONS(2444),
    [sym_min_value] = ACTIONS(2444),
    [sym_exp_value] = ACTIONS(2444),
    [sym_log_value] = ACTIONS(2444),
    [sym_log10_value] = ACTIONS(2444),
    [sym_power_value] = ACTIONS(2444),
    [sym_sqrt_value] = ACTIONS(2444),
    [sym_raw_be] = ACTIONS(2444),
    [sym_raw_le] = ACTIONS(2444),
    [sym_internal_queue_size_set] = ACTIONS(2444),
    [sym_internal_queue_size_get] = ACTIONS(2444),
    [sym_internal_threshold_set] = ACTIONS(2444),
    [sym_internal_threshold_get] = ACTIONS(2444),
    [sym_rebuilt_event] = ACTIONS(2444),
    [sym_do_custom_event] = ACTIONS(2444),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2444),
    [sym_clkmgr_set_clk_source] = ACTIONS(2444),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2444),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2444),
    [sym_clkmgr_get_timezone] = ACTIONS(2444),
    [sym_clkmgr_set_timezone] = ACTIONS(2444),
    [sym_clkmgr_get_resync_period] = ACTIONS(2444),
    [sym_clkmgr_set_resync_period] = ACTIONS(2444),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2444),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2444),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2444),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2444),
    [sym_clkmgr_get_timeservers] = ACTIONS(2444),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2444),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2444),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2444),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2444),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2444),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2444),
    [sym_set_log_level] = ACTIONS(2444),
    [sym_get_log_level] = ACTIONS(2444),
    [sym_amx_log] = ACTIONS(2444),
    [sym_smtp_server_config_set] = ACTIONS(2444),
    [sym_smtp_server_config_get] = ACTIONS(2444),
    [sym_smtp_send] = ACTIONS(2444),
    [sym_validate_netlinx_account] = ACTIONS(2444),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2444),
    [sym_authenticate_certificate] = ACTIONS(2444),
    [sym_audit_netlinx_session_event] = ACTIONS(2444),
    [sym_audit_netlinx_generic_event] = ACTIONS(2444),
    [sym_get_available_flash_disk_space] = ACTIONS(2444),
    [sym_get_max_flash_disk_space] = ACTIONS(2444),
    [sym_data_create_feed] = ACTIONS(2444),
    [sym_data_delete_feed] = ACTIONS(2444),
    [sym_data_publish_feed] = ACTIONS(2444),
    [sym_data_get_published_feed] = ACTIONS(2444),
    [sym_data_add_record] = ACTIONS(2444),
    [sym_data_get_event_record] = ACTIONS(2444),
    [sym__wc_data_create_feed] = ACTIONS(2444),
    [sym__wc_data_add_record] = ACTIONS(2444),
    [sym__wc_data_get_event_record] = ACTIONS(2444),
    [sym_format] = ACTIONS(2444),
    [sym_astro_clock] = ACTIONS(2444),
    [sym_set_outdoor_temperature] = ACTIONS(2444),
    [sym_set_virtual_level_count] = ACTIONS(2444),
    [sym_set_virtual_channel_count] = ACTIONS(2444),
    [sym_set_virtual_port_count] = ACTIONS(2444),
    [sym__wc_encode_int] = ACTIONS(2444),
    [sym__wc_decode_int] = ACTIONS(2444),
    [sym__wc_int] = ACTIONS(2444),
    [sym__wc_to_ch_int] = ACTIONS(2444),
    [sym__ch_to_wc_int] = ACTIONS(2444),
    [sym__wc_find_string_int] = ACTIONS(2444),
    [sym__wc_left_string_int] = ACTIONS(2444),
    [sym__wc_lower_string_int] = ACTIONS(2444),
    [sym__wc_mid_string_int] = ACTIONS(2444),
    [sym__wc_remove_string_int] = ACTIONS(2444),
    [sym__wc_right_string_int] = ACTIONS(2444),
    [sym__wc_upper_string_int] = ACTIONS(2444),
    [sym__wc_compare_string_int] = ACTIONS(2444),
    [sym__wc_get_buffer_char_int] = ACTIONS(2444),
    [sym__wc_get_buffer_string_int] = ACTIONS(2444),
    [sym__wc_concat_string_int] = ACTIONS(2444),
    [sym__wc_file_open_int] = ACTIONS(2444),
    [sym__wc_file_close_int] = ACTIONS(2444),
    [sym__wc_file_read_int] = ACTIONS(2444),
    [sym__wc_file_read_line_int] = ACTIONS(2444),
    [sym__wc_file_write_int] = ACTIONS(2444),
    [sym__wc_file_write_line_int] = ACTIONS(2444),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2268),
    [aux_sym_preproc_else_token1] = ACTIONS(2268),
    [anon_sym_COMMA] = ACTIONS(2447),
    [anon_sym_LBRACK] = ACTIONS(2450),
    [anon_sym_LBRACE] = ACTIONS(2453),
    [anon_sym_LPAREN] = ACTIONS(2456),
    [anon_sym_BANG] = ACTIONS(2459),
    [anon_sym_TILDE] = ACTIONS(2459),
    [anon_sym_DASH] = ACTIONS(2432),
    [anon_sym_PLUS] = ACTIONS(2432),
    [anon_sym_DASH_DASH] = ACTIONS(2462),
    [anon_sym_PLUS_PLUS] = ACTIONS(2462),
    [anon_sym_DQUOTE] = ACTIONS(2465),
    [anon_sym_SQUOTE] = ACTIONS(2468),
    [sym_number_literal] = ACTIONS(2471),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(658)] = {
    [sym_preproc_else_in_initializer_list_no_comma] = STATE(2085),
    [sym_expression] = STATE(635),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(663),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = STATE(663),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2474),
    [aux_sym_preproc_else_token1] = ACTIONS(2476),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(659)] = {
    [sym_preproc_else_in_initializer_list_no_comma] = STATE(2143),
    [sym_expression] = STATE(635),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(663),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = STATE(663),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2478),
    [aux_sym_preproc_else_token1] = ACTIONS(2476),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(660)] = {
    [sym_preproc_else_in_initializer_list_no_comma] = STATE(2196),
    [sym_expression] = STATE(635),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(663),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = STATE(663),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2480),
    [aux_sym_preproc_else_token1] = ACTIONS(2476),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(661)] = {
    [sym_preproc_else_in_initializer_list_no_comma] = STATE(2138),
    [sym_expression] = STATE(635),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(663),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = STATE(663),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2482),
    [aux_sym_preproc_else_token1] = ACTIONS(2476),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(662)] = {
    [sym_expression] = STATE(634),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(944),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = STATE(669),
    [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = STATE(671),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2484),
    [anon_sym_COMMA] = ACTIONS(2415),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(663)] = {
    [sym_expression] = STATE(635),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(663),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = STATE(663),
    [sym_identifier] = ACTIONS(2487),
    [sym_not] = ACTIONS(2490),
    [sym_bnot] = ACTIONS(2490),
    [sym___file__] = ACTIONS(2493),
    [sym___date__] = ACTIONS(2493),
    [sym___ldate__] = ACTIONS(2493),
    [sym___time__] = ACTIONS(2493),
    [sym___line__] = ACTIONS(2493),
    [sym___name__] = ACTIONS(2493),
    [sym___netlinx__] = ACTIONS(2496),
    [sym_netlinx_axi_version] = ACTIONS(2496),
    [sym_url_flg_tcp] = ACTIONS(2496),
    [sym_url_flg_accinfopresent] = ACTIONS(2496),
    [sym_url_flg_temp] = ACTIONS(2496),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2496),
    [sym_url_flg_stat_mask] = ACTIONS(2496),
    [sym_url_flg_stat_lookup] = ACTIONS(2496),
    [sym_url_flg_stat_connecting] = ACTIONS(2496),
    [sym_url_flg_stat_waiting] = ACTIONS(2496),
    [sym_url_flg_stat_connected] = ACTIONS(2496),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2496),
    [sym_first_virtual_device] = ACTIONS(2496),
    [sym_first_local_port] = ACTIONS(2496),
    [sym_dynamic_virtual_device] = ACTIONS(2496),
    [sym_timeline_once] = ACTIONS(2496),
    [sym_timeline_repeat] = ACTIONS(2496),
    [sym_timeline_absolute] = ACTIONS(2496),
    [sym_timeline_relative] = ACTIONS(2496),
    [sym_do_push_timed_infinite] = ACTIONS(2496),
    [sym_source_type_no_address] = ACTIONS(2496),
    [sym_source_type_neuron_id] = ACTIONS(2496),
    [sym_source_type_ip_address] = ACTIONS(2496),
    [sym_source_type_axlink] = ACTIONS(2496),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2496),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2496),
    [sym_source_type_ip_socket_address] = ACTIONS(2496),
    [sym_source_type_rs232] = ACTIONS(2496),
    [sym_source_type_internal] = ACTIONS(2496),
    [sym_source_type_mac_address] = ACTIONS(2496),
    [sym_source_type_ipv4_port] = ACTIONS(2496),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2496),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2496),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2496),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2496),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2496),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2496),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2496),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2496),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2496),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2496),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2496),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2496),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2496),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2496),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2496),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2496),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2496),
    [sym_true] = ACTIONS(2496),
    [sym_false] = ACTIONS(2496),
    [sym_file_read_only] = ACTIONS(2496),
    [sym_file_rw_new] = ACTIONS(2496),
    [sym_file_rw_append] = ACTIONS(2496),
    [sym_ip_tcp] = ACTIONS(2496),
    [sym_ip_udp] = ACTIONS(2496),
    [sym_ip_udp_2way] = ACTIONS(2496),
    [sym_xml_encode_types] = ACTIONS(2496),
    [sym_xml_encode_char_as_list] = ACTIONS(2496),
    [sym_xml_encode_le] = ACTIONS(2496),
    [sym_xml_decode_types] = ACTIONS(2496),
    [sym_xml_decode_no_preserve] = ACTIONS(2496),
    [sym_duet_dev_type_display_device] = ACTIONS(2496),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2496),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2496),
    [sym_duet_dev_type_io_device] = ACTIONS(2496),
    [sym_duet_dev_type_relay_device] = ACTIONS(2496),
    [sym_duet_dev_type_ups] = ACTIONS(2496),
    [sym_duet_dev_type_amplifier] = ACTIONS(2496),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2496),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2496),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2496),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2496),
    [sym_duet_dev_type_camera] = ACTIONS(2496),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2496),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2496),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2496),
    [sym_duet_dev_type_dss] = ACTIONS(2496),
    [sym_duet_dev_type_dvr] = ACTIONS(2496),
    [sym_duet_dev_type_disc_device] = ACTIONS(2496),
    [sym_duet_dev_type_document_camera] = ACTIONS(2496),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2496),
    [sym_duet_dev_type_hvac] = ACTIONS(2496),
    [sym_duet_dev_type_keypad] = ACTIONS(2496),
    [sym_duet_dev_type_light] = ACTIONS(2496),
    [sym_duet_dev_type_monitor] = ACTIONS(2496),
    [sym_duet_dev_type_motor] = ACTIONS(2496),
    [sym_duet_dev_type_multi_window] = ACTIONS(2496),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2496),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2496),
    [sym_duet_dev_type_receiver] = ACTIONS(2496),
    [sym_duet_dev_type_security_system] = ACTIONS(2496),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2496),
    [sym_duet_dev_type_settop_box] = ACTIONS(2496),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2496),
    [sym_duet_dev_type_switcher] = ACTIONS(2496),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2496),
    [sym_duet_dev_type_tv] = ACTIONS(2496),
    [sym_duet_dev_type_utility] = ACTIONS(2496),
    [sym_duet_dev_type_vcr] = ACTIONS(2496),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2496),
    [sym_duet_dev_type_video_processor] = ACTIONS(2496),
    [sym_duet_dev_type_video_projector] = ACTIONS(2496),
    [sym_duet_dev_type_video_wall] = ACTIONS(2496),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2496),
    [sym_duet_dev_type_weather] = ACTIONS(2496),
    [sym_duet_dev_not_polled] = ACTIONS(2496),
    [sym_duet_dev_polled] = ACTIONS(2496),
    [sym_clkmgr_mode_network] = ACTIONS(2496),
    [sym_clkmgr_mode_standalone] = ACTIONS(2496),
    [sym_normal_standby] = ACTIONS(2496),
    [sym_normal_wake] = ACTIONS(2496),
    [sym_ip_multicast_ttl_option] = ACTIONS(2496),
    [sym_ip_ttl_subnet] = ACTIONS(2496),
    [sym_ip_ttl_site] = ACTIONS(2496),
    [sym_ip_ttl_region] = ACTIONS(2496),
    [sym_ip_ttl_continent] = ACTIONS(2496),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2496),
    [sym_ip_nodelay_on] = ACTIONS(2496),
    [sym_ip_nodelay_off] = ACTIONS(2496),
    [sym_amx_error] = ACTIONS(2496),
    [sym_amx_warning] = ACTIONS(2496),
    [sym_amx_info] = ACTIONS(2496),
    [sym_amx_debug] = ACTIONS(2496),
    [sym_netlinx_login_success] = ACTIONS(2496),
    [sym_netlinx_login_fail] = ACTIONS(2496),
    [sym_netlinx_logout] = ACTIONS(2496),
    [sym_null_str] = ACTIONS(2496),
    [sym_smtp_address] = ACTIONS(2496),
    [sym_smtp_port_number] = ACTIONS(2496),
    [sym_smtp_username] = ACTIONS(2496),
    [sym_smtp_password] = ACTIONS(2496),
    [sym_smtp_require_tls] = ACTIONS(2496),
    [sym_smtp_from] = ACTIONS(2496),
    [sym_smtp_tls_true] = ACTIONS(2496),
    [sym_smtp_tls_false] = ACTIONS(2496),
    [sym_listview_on_row_select_event] = ACTIONS(2496),
    [sym_data_structure_unknown] = ACTIONS(2496),
    [sym_data_structure_datafeed] = ACTIONS(2496),
    [sym_data_structure_datarecordset] = ACTIONS(2496),
    [sym_data_structure_datarecord] = ACTIONS(2496),
    [sym_data_structure_datafield] = ACTIONS(2496),
    [sym_data_max_content_field_count] = ACTIONS(2496),
    [sym_data_max_metadata_field_count] = ACTIONS(2496),
    [sym_data_max_id_length] = ACTIONS(2496),
    [sym_data_max_name_length] = ACTIONS(2496),
    [sym_data_max_description_length] = ACTIONS(2496),
    [sym_data_max_type_length] = ACTIONS(2496),
    [sym_data_max_format_length] = ACTIONS(2496),
    [sym_data_max_source_length] = ACTIONS(2496),
    [sym_data_max_value_length] = ACTIONS(2496),
    [sym_data_max_label_length] = ACTIONS(2496),
    [sym_data_max_url_length] = ACTIONS(2496),
    [sym_max_last_login_info_length] = ACTIONS(2496),
    [sym_data_type_unknown] = ACTIONS(2496),
    [sym_data_type_string] = ACTIONS(2496),
    [sym_data_type_datetime] = ACTIONS(2496),
    [sym_data_type_date] = ACTIONS(2496),
    [sym_data_type_time] = ACTIONS(2496),
    [sym_data_type_image] = ACTIONS(2496),
    [sym_data_format_url] = ACTIONS(2496),
    [sym_data_format_phone] = ACTIONS(2496),
    [sym_data_format_email] = ACTIONS(2496),
    [sym_data_format_iso8601] = ACTIONS(2496),
    [sym_auditlog_priv] = ACTIONS(2496),
    [sym_configuration_priv] = ACTIONS(2496),
    [sym_deviceconfig_priv] = ACTIONS(2496),
    [sym_ftp_priv] = ACTIONS(2496),
    [sym_http_priv] = ACTIONS(2496),
    [sym_networkconfig_priv] = ACTIONS(2496),
    [sym_programport_priv] = ACTIONS(2496),
    [sym_remoteui_priv] = ACTIONS(2496),
    [sym_securitycontrol_priv] = ACTIONS(2496),
    [sym_softwaremanagement_priv] = ACTIONS(2496),
    [sym_terminal_priv] = ACTIONS(2496),
    [sym_tpadmin_priv] = ACTIONS(2496),
    [sym_user1_priv] = ACTIONS(2496),
    [sym_user2_priv] = ACTIONS(2496),
    [sym_user3_priv] = ACTIONS(2496),
    [sym_user4_priv] = ACTIONS(2496),
    [sym_usermanagement_priv] = ACTIONS(2496),
    [sym_valid_account] = ACTIONS(2496),
    [sym_err_name_invalid] = ACTIONS(2496),
    [sym_err_password_invalid] = ACTIONS(2496),
    [sym_err_invalid_account] = ACTIONS(2496),
    [sym_err_no_authorization] = ACTIONS(2496),
    [sym_err_invalid_parameter] = ACTIONS(2496),
    [sym_err_user_locked_out] = ACTIONS(2496),
    [sym_err_user_account_expired] = ACTIONS(2496),
    [sym_err_general_error] = ACTIONS(2496),
    [sym_err_authentication_error] = ACTIONS(2496),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2496),
    [sym_tls_validate_certificate] = ACTIONS(2496),
    [sym____reserved___] = ACTIONS(2499),
    [sym_date] = ACTIONS(2499),
    [sym_day] = ACTIONS(2499),
    [sym_time] = ACTIONS(2499),
    [sym_ldate] = ACTIONS(2499),
    [sym_push_device] = ACTIONS(2499),
    [sym_push_channel] = ACTIONS(2499),
    [sym_push_devchan] = ACTIONS(2499),
    [sym_release_device] = ACTIONS(2499),
    [sym_release_channel] = ACTIONS(2499),
    [sym_release_devchan] = ACTIONS(2499),
    [sym_master_slot] = ACTIONS(2499),
    [sym_get_pulse_time] = ACTIONS(2499),
    [sym_get_timer] = ACTIONS(2499),
    [sym_master_sn] = ACTIONS(2499),
    [sym_system_number] = ACTIONS(2499),
    [sym_button] = ACTIONS(2499),
    [sym_dv_channel] = ACTIONS(2499),
    [sym_data] = ACTIONS(2499),
    [sym_level] = ACTIONS(2499),
    [sym_timeline] = ACTIONS(2499),
    [sym_channel] = ACTIONS(2499),
    [sym_custom] = ACTIONS(2499),
    [sym_atoi] = ACTIONS(2502),
    [sym_atol] = ACTIONS(2502),
    [sym_atof] = ACTIONS(2502),
    [sym_itoa] = ACTIONS(2502),
    [sym_itohex] = ACTIONS(2502),
    [sym_hextoi] = ACTIONS(2502),
    [sym_ftoa] = ACTIONS(2502),
    [sym_find_string] = ACTIONS(2502),
    [sym_left_string] = ACTIONS(2502),
    [sym_right_string] = ACTIONS(2502),
    [sym_mid_string] = ACTIONS(2502),
    [sym_remove_string] = ACTIONS(2502),
    [sym_get_buffer_string] = ACTIONS(2502),
    [sym_get_buffer_char] = ACTIONS(2502),
    [sym_get_multi_buffer_string] = ACTIONS(2502),
    [sym_length_string] = ACTIONS(2502),
    [sym_max_length_string] = ACTIONS(2502),
    [sym_set_length_string] = ACTIONS(2502),
    [sym_compare_string] = ACTIONS(2502),
    [sym_upper_string] = ACTIONS(2502),
    [sym_lower_string] = ACTIONS(2502),
    [sym_get_last] = ACTIONS(2502),
    [sym_date_to_day] = ACTIONS(2502),
    [sym_date_to_month] = ACTIONS(2502),
    [sym_date_to_year] = ACTIONS(2502),
    [sym_time_to_hour] = ACTIONS(2502),
    [sym_time_to_minute] = ACTIONS(2502),
    [sym_time_to_second] = ACTIONS(2502),
    [sym_day_of_week] = ACTIONS(2502),
    [sym_set_pulse_time] = ACTIONS(2502),
    [sym_set_timer] = ACTIONS(2502),
    [sym_file_open] = ACTIONS(2502),
    [sym_file_close] = ACTIONS(2502),
    [sym_file_read] = ACTIONS(2502),
    [sym_file_read_line] = ACTIONS(2502),
    [sym_file_write] = ACTIONS(2502),
    [sym_file_write_line] = ACTIONS(2502),
    [sym_file_copy] = ACTIONS(2502),
    [sym_file_delete] = ACTIONS(2502),
    [sym_file_rename] = ACTIONS(2502),
    [sym_file_dir] = ACTIONS(2502),
    [sym_file_getdir] = ACTIONS(2502),
    [sym_file_setdir] = ACTIONS(2502),
    [sym_file_create_dir] = ACTIONS(2502),
    [sym_file_remove_dir] = ACTIONS(2502),
    [sym_file_seek] = ACTIONS(2502),
    [sym_length_array] = ACTIONS(2502),
    [sym_max_length_array] = ACTIONS(2502),
    [sym_set_length_array] = ACTIONS(2502),
    [sym_ip_client_open] = ACTIONS(2502),
    [sym_ip_client_close] = ACTIONS(2502),
    [sym_ip_server_open] = ACTIONS(2502),
    [sym_ip_server_close] = ACTIONS(2502),
    [sym_ip_bound_client_open] = ACTIONS(2502),
    [sym_ip_mc_server_open] = ACTIONS(2502),
    [sym_ip_set_option] = ACTIONS(2502),
    [sym_ssh_client_open] = ACTIONS(2502),
    [sym_ssh_client_close] = ACTIONS(2502),
    [sym_tls_client_open] = ACTIONS(2502),
    [sym_tls_client_close] = ACTIONS(2502),
    [sym_do_push] = ACTIONS(2502),
    [sym_do_push_timed] = ACTIONS(2502),
    [sym_do_release] = ACTIONS(2502),
    [sym_device_id] = ACTIONS(2502),
    [sym_device_info] = ACTIONS(2502),
    [sym_device_id_string] = ACTIONS(2502),
    [sym_device_standby] = ACTIONS(2502),
    [sym_device_wake] = ACTIONS(2502),
    [sym_timeline_create] = ACTIONS(2502),
    [sym_timeline_kill] = ACTIONS(2502),
    [sym_timeline_reload] = ACTIONS(2502),
    [sym_timeline_pause] = ACTIONS(2502),
    [sym_timeline_restart] = ACTIONS(2502),
    [sym_timeline_set] = ACTIONS(2502),
    [sym_timeline_get] = ACTIONS(2502),
    [sym_timeline_active] = ACTIONS(2502),
    [sym_timeline_dynamic_id] = ACTIONS(2502),
    [sym_variable_to_string] = ACTIONS(2502),
    [sym_string_to_variable] = ACTIONS(2502),
    [sym_length_variable_to_string] = ACTIONS(2502),
    [sym_type_cast] = ACTIONS(2502),
    [sym_variable_to_xml] = ACTIONS(2502),
    [sym_xml_to_variable] = ACTIONS(2502),
    [sym_length_variable_to_xml] = ACTIONS(2502),
    [sym_redirect_string] = ACTIONS(2502),
    [sym_random_number] = ACTIONS(2502),
    [sym_get_unique_id] = ACTIONS(2502),
    [sym_get_system_number] = ACTIONS(2502),
    [sym_set_system_number] = ACTIONS(2502),
    [sym_get_serial_number] = ACTIONS(2502),
    [sym_get_master_build] = ACTIONS(2502),
    [sym_set_validation_code] = ACTIONS(2502),
    [sym_reboot] = ACTIONS(2502),
    [sym_load_duet_module] = ACTIONS(2502),
    [sym_unload_duet_module] = ACTIONS(2502),
    [sym_duet_mem_size_set] = ACTIONS(2502),
    [sym_duet_mem_size_get] = ACTIONS(2502),
    [sym_dynamic_application_device] = ACTIONS(2502),
    [sym_dynamic_polled_port] = ACTIONS(2502),
    [sym_static_port_binding] = ACTIONS(2502),
    [sym_static_ip_binding] = ACTIONS(2502),
    [sym_get_dns_list] = ACTIONS(2502),
    [sym_set_dns_list] = ACTIONS(2502),
    [sym_get_ip_address] = ACTIONS(2502),
    [sym_set_ip_address] = ACTIONS(2502),
    [sym_get_url_list] = ACTIONS(2502),
    [sym_add_url_entry] = ACTIONS(2502),
    [sym_delete_url_entry] = ACTIONS(2502),
    [sym_abs_value] = ACTIONS(2502),
    [sym_max_value] = ACTIONS(2502),
    [sym_min_value] = ACTIONS(2502),
    [sym_exp_value] = ACTIONS(2502),
    [sym_log_value] = ACTIONS(2502),
    [sym_log10_value] = ACTIONS(2502),
    [sym_power_value] = ACTIONS(2502),
    [sym_sqrt_value] = ACTIONS(2502),
    [sym_raw_be] = ACTIONS(2502),
    [sym_raw_le] = ACTIONS(2502),
    [sym_internal_queue_size_set] = ACTIONS(2502),
    [sym_internal_queue_size_get] = ACTIONS(2502),
    [sym_internal_threshold_set] = ACTIONS(2502),
    [sym_internal_threshold_get] = ACTIONS(2502),
    [sym_rebuilt_event] = ACTIONS(2502),
    [sym_do_custom_event] = ACTIONS(2502),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2502),
    [sym_clkmgr_set_clk_source] = ACTIONS(2502),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2502),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2502),
    [sym_clkmgr_get_timezone] = ACTIONS(2502),
    [sym_clkmgr_set_timezone] = ACTIONS(2502),
    [sym_clkmgr_get_resync_period] = ACTIONS(2502),
    [sym_clkmgr_set_resync_period] = ACTIONS(2502),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2502),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2502),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2502),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2502),
    [sym_clkmgr_get_timeservers] = ACTIONS(2502),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2502),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2502),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2502),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2502),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2502),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2502),
    [sym_set_log_level] = ACTIONS(2502),
    [sym_get_log_level] = ACTIONS(2502),
    [sym_amx_log] = ACTIONS(2502),
    [sym_smtp_server_config_set] = ACTIONS(2502),
    [sym_smtp_server_config_get] = ACTIONS(2502),
    [sym_smtp_send] = ACTIONS(2502),
    [sym_validate_netlinx_account] = ACTIONS(2502),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2502),
    [sym_authenticate_certificate] = ACTIONS(2502),
    [sym_audit_netlinx_session_event] = ACTIONS(2502),
    [sym_audit_netlinx_generic_event] = ACTIONS(2502),
    [sym_get_available_flash_disk_space] = ACTIONS(2502),
    [sym_get_max_flash_disk_space] = ACTIONS(2502),
    [sym_data_create_feed] = ACTIONS(2502),
    [sym_data_delete_feed] = ACTIONS(2502),
    [sym_data_publish_feed] = ACTIONS(2502),
    [sym_data_get_published_feed] = ACTIONS(2502),
    [sym_data_add_record] = ACTIONS(2502),
    [sym_data_get_event_record] = ACTIONS(2502),
    [sym__wc_data_create_feed] = ACTIONS(2502),
    [sym__wc_data_add_record] = ACTIONS(2502),
    [sym__wc_data_get_event_record] = ACTIONS(2502),
    [sym_format] = ACTIONS(2502),
    [sym_astro_clock] = ACTIONS(2502),
    [sym_set_outdoor_temperature] = ACTIONS(2502),
    [sym_set_virtual_level_count] = ACTIONS(2502),
    [sym_set_virtual_channel_count] = ACTIONS(2502),
    [sym_set_virtual_port_count] = ACTIONS(2502),
    [sym__wc_encode_int] = ACTIONS(2502),
    [sym__wc_decode_int] = ACTIONS(2502),
    [sym__wc_int] = ACTIONS(2502),
    [sym__wc_to_ch_int] = ACTIONS(2502),
    [sym__ch_to_wc_int] = ACTIONS(2502),
    [sym__wc_find_string_int] = ACTIONS(2502),
    [sym__wc_left_string_int] = ACTIONS(2502),
    [sym__wc_lower_string_int] = ACTIONS(2502),
    [sym__wc_mid_string_int] = ACTIONS(2502),
    [sym__wc_remove_string_int] = ACTIONS(2502),
    [sym__wc_right_string_int] = ACTIONS(2502),
    [sym__wc_upper_string_int] = ACTIONS(2502),
    [sym__wc_compare_string_int] = ACTIONS(2502),
    [sym__wc_get_buffer_char_int] = ACTIONS(2502),
    [sym__wc_get_buffer_string_int] = ACTIONS(2502),
    [sym__wc_concat_string_int] = ACTIONS(2502),
    [sym__wc_file_open_int] = ACTIONS(2502),
    [sym__wc_file_close_int] = ACTIONS(2502),
    [sym__wc_file_read_int] = ACTIONS(2502),
    [sym__wc_file_read_line_int] = ACTIONS(2502),
    [sym__wc_file_write_int] = ACTIONS(2502),
    [sym__wc_file_write_line_int] = ACTIONS(2502),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2505),
    [aux_sym_preproc_else_token1] = ACTIONS(2505),
    [anon_sym_LBRACK] = ACTIONS(2507),
    [anon_sym_LBRACE] = ACTIONS(2510),
    [anon_sym_LPAREN] = ACTIONS(2513),
    [anon_sym_BANG] = ACTIONS(2516),
    [anon_sym_TILDE] = ACTIONS(2516),
    [anon_sym_DASH] = ACTIONS(2490),
    [anon_sym_PLUS] = ACTIONS(2490),
    [anon_sym_DASH_DASH] = ACTIONS(2519),
    [anon_sym_PLUS_PLUS] = ACTIONS(2519),
    [anon_sym_DQUOTE] = ACTIONS(2522),
    [anon_sym_SQUOTE] = ACTIONS(2525),
    [sym_number_literal] = ACTIONS(2528),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(664)] = {
    [sym_expression] = STATE(1140),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1920),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_initializer_list] = STATE(1920),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(2531),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_SEMI] = ACTIONS(2533),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(2533),
  },
  [STATE(665)] = {
    [sym_expression] = STATE(633),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(959),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2535),
    [sym_not] = ACTIONS(2538),
    [sym_bnot] = ACTIONS(2538),
    [sym___file__] = ACTIONS(2541),
    [sym___date__] = ACTIONS(2541),
    [sym___ldate__] = ACTIONS(2541),
    [sym___time__] = ACTIONS(2541),
    [sym___line__] = ACTIONS(2541),
    [sym___name__] = ACTIONS(2541),
    [sym___netlinx__] = ACTIONS(2544),
    [sym_netlinx_axi_version] = ACTIONS(2544),
    [sym_url_flg_tcp] = ACTIONS(2544),
    [sym_url_flg_accinfopresent] = ACTIONS(2544),
    [sym_url_flg_temp] = ACTIONS(2544),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2544),
    [sym_url_flg_stat_mask] = ACTIONS(2544),
    [sym_url_flg_stat_lookup] = ACTIONS(2544),
    [sym_url_flg_stat_connecting] = ACTIONS(2544),
    [sym_url_flg_stat_waiting] = ACTIONS(2544),
    [sym_url_flg_stat_connected] = ACTIONS(2544),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2544),
    [sym_first_virtual_device] = ACTIONS(2544),
    [sym_first_local_port] = ACTIONS(2544),
    [sym_dynamic_virtual_device] = ACTIONS(2544),
    [sym_timeline_once] = ACTIONS(2544),
    [sym_timeline_repeat] = ACTIONS(2544),
    [sym_timeline_absolute] = ACTIONS(2544),
    [sym_timeline_relative] = ACTIONS(2544),
    [sym_do_push_timed_infinite] = ACTIONS(2544),
    [sym_source_type_no_address] = ACTIONS(2544),
    [sym_source_type_neuron_id] = ACTIONS(2544),
    [sym_source_type_ip_address] = ACTIONS(2544),
    [sym_source_type_axlink] = ACTIONS(2544),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2544),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2544),
    [sym_source_type_ip_socket_address] = ACTIONS(2544),
    [sym_source_type_rs232] = ACTIONS(2544),
    [sym_source_type_internal] = ACTIONS(2544),
    [sym_source_type_mac_address] = ACTIONS(2544),
    [sym_source_type_ipv4_port] = ACTIONS(2544),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2544),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2544),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2544),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2544),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2544),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2544),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2544),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2544),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2544),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2544),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2544),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2544),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2544),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2544),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2544),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2544),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2544),
    [sym_true] = ACTIONS(2544),
    [sym_false] = ACTIONS(2544),
    [sym_file_read_only] = ACTIONS(2544),
    [sym_file_rw_new] = ACTIONS(2544),
    [sym_file_rw_append] = ACTIONS(2544),
    [sym_ip_tcp] = ACTIONS(2544),
    [sym_ip_udp] = ACTIONS(2544),
    [sym_ip_udp_2way] = ACTIONS(2544),
    [sym_xml_encode_types] = ACTIONS(2544),
    [sym_xml_encode_char_as_list] = ACTIONS(2544),
    [sym_xml_encode_le] = ACTIONS(2544),
    [sym_xml_decode_types] = ACTIONS(2544),
    [sym_xml_decode_no_preserve] = ACTIONS(2544),
    [sym_duet_dev_type_display_device] = ACTIONS(2544),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2544),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2544),
    [sym_duet_dev_type_io_device] = ACTIONS(2544),
    [sym_duet_dev_type_relay_device] = ACTIONS(2544),
    [sym_duet_dev_type_ups] = ACTIONS(2544),
    [sym_duet_dev_type_amplifier] = ACTIONS(2544),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2544),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2544),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2544),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2544),
    [sym_duet_dev_type_camera] = ACTIONS(2544),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2544),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2544),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2544),
    [sym_duet_dev_type_dss] = ACTIONS(2544),
    [sym_duet_dev_type_dvr] = ACTIONS(2544),
    [sym_duet_dev_type_disc_device] = ACTIONS(2544),
    [sym_duet_dev_type_document_camera] = ACTIONS(2544),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2544),
    [sym_duet_dev_type_hvac] = ACTIONS(2544),
    [sym_duet_dev_type_keypad] = ACTIONS(2544),
    [sym_duet_dev_type_light] = ACTIONS(2544),
    [sym_duet_dev_type_monitor] = ACTIONS(2544),
    [sym_duet_dev_type_motor] = ACTIONS(2544),
    [sym_duet_dev_type_multi_window] = ACTIONS(2544),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2544),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2544),
    [sym_duet_dev_type_receiver] = ACTIONS(2544),
    [sym_duet_dev_type_security_system] = ACTIONS(2544),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2544),
    [sym_duet_dev_type_settop_box] = ACTIONS(2544),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2544),
    [sym_duet_dev_type_switcher] = ACTIONS(2544),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2544),
    [sym_duet_dev_type_tv] = ACTIONS(2544),
    [sym_duet_dev_type_utility] = ACTIONS(2544),
    [sym_duet_dev_type_vcr] = ACTIONS(2544),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2544),
    [sym_duet_dev_type_video_processor] = ACTIONS(2544),
    [sym_duet_dev_type_video_projector] = ACTIONS(2544),
    [sym_duet_dev_type_video_wall] = ACTIONS(2544),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2544),
    [sym_duet_dev_type_weather] = ACTIONS(2544),
    [sym_duet_dev_not_polled] = ACTIONS(2544),
    [sym_duet_dev_polled] = ACTIONS(2544),
    [sym_clkmgr_mode_network] = ACTIONS(2544),
    [sym_clkmgr_mode_standalone] = ACTIONS(2544),
    [sym_normal_standby] = ACTIONS(2544),
    [sym_normal_wake] = ACTIONS(2544),
    [sym_ip_multicast_ttl_option] = ACTIONS(2544),
    [sym_ip_ttl_subnet] = ACTIONS(2544),
    [sym_ip_ttl_site] = ACTIONS(2544),
    [sym_ip_ttl_region] = ACTIONS(2544),
    [sym_ip_ttl_continent] = ACTIONS(2544),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2544),
    [sym_ip_nodelay_on] = ACTIONS(2544),
    [sym_ip_nodelay_off] = ACTIONS(2544),
    [sym_amx_error] = ACTIONS(2544),
    [sym_amx_warning] = ACTIONS(2544),
    [sym_amx_info] = ACTIONS(2544),
    [sym_amx_debug] = ACTIONS(2544),
    [sym_netlinx_login_success] = ACTIONS(2544),
    [sym_netlinx_login_fail] = ACTIONS(2544),
    [sym_netlinx_logout] = ACTIONS(2544),
    [sym_null_str] = ACTIONS(2544),
    [sym_smtp_address] = ACTIONS(2544),
    [sym_smtp_port_number] = ACTIONS(2544),
    [sym_smtp_username] = ACTIONS(2544),
    [sym_smtp_password] = ACTIONS(2544),
    [sym_smtp_require_tls] = ACTIONS(2544),
    [sym_smtp_from] = ACTIONS(2544),
    [sym_smtp_tls_true] = ACTIONS(2544),
    [sym_smtp_tls_false] = ACTIONS(2544),
    [sym_listview_on_row_select_event] = ACTIONS(2544),
    [sym_data_structure_unknown] = ACTIONS(2544),
    [sym_data_structure_datafeed] = ACTIONS(2544),
    [sym_data_structure_datarecordset] = ACTIONS(2544),
    [sym_data_structure_datarecord] = ACTIONS(2544),
    [sym_data_structure_datafield] = ACTIONS(2544),
    [sym_data_max_content_field_count] = ACTIONS(2544),
    [sym_data_max_metadata_field_count] = ACTIONS(2544),
    [sym_data_max_id_length] = ACTIONS(2544),
    [sym_data_max_name_length] = ACTIONS(2544),
    [sym_data_max_description_length] = ACTIONS(2544),
    [sym_data_max_type_length] = ACTIONS(2544),
    [sym_data_max_format_length] = ACTIONS(2544),
    [sym_data_max_source_length] = ACTIONS(2544),
    [sym_data_max_value_length] = ACTIONS(2544),
    [sym_data_max_label_length] = ACTIONS(2544),
    [sym_data_max_url_length] = ACTIONS(2544),
    [sym_max_last_login_info_length] = ACTIONS(2544),
    [sym_data_type_unknown] = ACTIONS(2544),
    [sym_data_type_string] = ACTIONS(2544),
    [sym_data_type_datetime] = ACTIONS(2544),
    [sym_data_type_date] = ACTIONS(2544),
    [sym_data_type_time] = ACTIONS(2544),
    [sym_data_type_image] = ACTIONS(2544),
    [sym_data_format_url] = ACTIONS(2544),
    [sym_data_format_phone] = ACTIONS(2544),
    [sym_data_format_email] = ACTIONS(2544),
    [sym_data_format_iso8601] = ACTIONS(2544),
    [sym_auditlog_priv] = ACTIONS(2544),
    [sym_configuration_priv] = ACTIONS(2544),
    [sym_deviceconfig_priv] = ACTIONS(2544),
    [sym_ftp_priv] = ACTIONS(2544),
    [sym_http_priv] = ACTIONS(2544),
    [sym_networkconfig_priv] = ACTIONS(2544),
    [sym_programport_priv] = ACTIONS(2544),
    [sym_remoteui_priv] = ACTIONS(2544),
    [sym_securitycontrol_priv] = ACTIONS(2544),
    [sym_softwaremanagement_priv] = ACTIONS(2544),
    [sym_terminal_priv] = ACTIONS(2544),
    [sym_tpadmin_priv] = ACTIONS(2544),
    [sym_user1_priv] = ACTIONS(2544),
    [sym_user2_priv] = ACTIONS(2544),
    [sym_user3_priv] = ACTIONS(2544),
    [sym_user4_priv] = ACTIONS(2544),
    [sym_usermanagement_priv] = ACTIONS(2544),
    [sym_valid_account] = ACTIONS(2544),
    [sym_err_name_invalid] = ACTIONS(2544),
    [sym_err_password_invalid] = ACTIONS(2544),
    [sym_err_invalid_account] = ACTIONS(2544),
    [sym_err_no_authorization] = ACTIONS(2544),
    [sym_err_invalid_parameter] = ACTIONS(2544),
    [sym_err_user_locked_out] = ACTIONS(2544),
    [sym_err_user_account_expired] = ACTIONS(2544),
    [sym_err_general_error] = ACTIONS(2544),
    [sym_err_authentication_error] = ACTIONS(2544),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2544),
    [sym_tls_validate_certificate] = ACTIONS(2544),
    [sym____reserved___] = ACTIONS(2547),
    [sym_date] = ACTIONS(2547),
    [sym_day] = ACTIONS(2547),
    [sym_time] = ACTIONS(2547),
    [sym_ldate] = ACTIONS(2547),
    [sym_push_device] = ACTIONS(2547),
    [sym_push_channel] = ACTIONS(2547),
    [sym_push_devchan] = ACTIONS(2547),
    [sym_release_device] = ACTIONS(2547),
    [sym_release_channel] = ACTIONS(2547),
    [sym_release_devchan] = ACTIONS(2547),
    [sym_master_slot] = ACTIONS(2547),
    [sym_get_pulse_time] = ACTIONS(2547),
    [sym_get_timer] = ACTIONS(2547),
    [sym_master_sn] = ACTIONS(2547),
    [sym_system_number] = ACTIONS(2547),
    [sym_button] = ACTIONS(2547),
    [sym_dv_channel] = ACTIONS(2547),
    [sym_data] = ACTIONS(2547),
    [sym_level] = ACTIONS(2547),
    [sym_timeline] = ACTIONS(2547),
    [sym_channel] = ACTIONS(2547),
    [sym_custom] = ACTIONS(2547),
    [sym_atoi] = ACTIONS(2550),
    [sym_atol] = ACTIONS(2550),
    [sym_atof] = ACTIONS(2550),
    [sym_itoa] = ACTIONS(2550),
    [sym_itohex] = ACTIONS(2550),
    [sym_hextoi] = ACTIONS(2550),
    [sym_ftoa] = ACTIONS(2550),
    [sym_find_string] = ACTIONS(2550),
    [sym_left_string] = ACTIONS(2550),
    [sym_right_string] = ACTIONS(2550),
    [sym_mid_string] = ACTIONS(2550),
    [sym_remove_string] = ACTIONS(2550),
    [sym_get_buffer_string] = ACTIONS(2550),
    [sym_get_buffer_char] = ACTIONS(2550),
    [sym_get_multi_buffer_string] = ACTIONS(2550),
    [sym_length_string] = ACTIONS(2550),
    [sym_max_length_string] = ACTIONS(2550),
    [sym_set_length_string] = ACTIONS(2550),
    [sym_compare_string] = ACTIONS(2550),
    [sym_upper_string] = ACTIONS(2550),
    [sym_lower_string] = ACTIONS(2550),
    [sym_get_last] = ACTIONS(2550),
    [sym_date_to_day] = ACTIONS(2550),
    [sym_date_to_month] = ACTIONS(2550),
    [sym_date_to_year] = ACTIONS(2550),
    [sym_time_to_hour] = ACTIONS(2550),
    [sym_time_to_minute] = ACTIONS(2550),
    [sym_time_to_second] = ACTIONS(2550),
    [sym_day_of_week] = ACTIONS(2550),
    [sym_set_pulse_time] = ACTIONS(2550),
    [sym_set_timer] = ACTIONS(2550),
    [sym_file_open] = ACTIONS(2550),
    [sym_file_close] = ACTIONS(2550),
    [sym_file_read] = ACTIONS(2550),
    [sym_file_read_line] = ACTIONS(2550),
    [sym_file_write] = ACTIONS(2550),
    [sym_file_write_line] = ACTIONS(2550),
    [sym_file_copy] = ACTIONS(2550),
    [sym_file_delete] = ACTIONS(2550),
    [sym_file_rename] = ACTIONS(2550),
    [sym_file_dir] = ACTIONS(2550),
    [sym_file_getdir] = ACTIONS(2550),
    [sym_file_setdir] = ACTIONS(2550),
    [sym_file_create_dir] = ACTIONS(2550),
    [sym_file_remove_dir] = ACTIONS(2550),
    [sym_file_seek] = ACTIONS(2550),
    [sym_length_array] = ACTIONS(2550),
    [sym_max_length_array] = ACTIONS(2550),
    [sym_set_length_array] = ACTIONS(2550),
    [sym_ip_client_open] = ACTIONS(2550),
    [sym_ip_client_close] = ACTIONS(2550),
    [sym_ip_server_open] = ACTIONS(2550),
    [sym_ip_server_close] = ACTIONS(2550),
    [sym_ip_bound_client_open] = ACTIONS(2550),
    [sym_ip_mc_server_open] = ACTIONS(2550),
    [sym_ip_set_option] = ACTIONS(2550),
    [sym_ssh_client_open] = ACTIONS(2550),
    [sym_ssh_client_close] = ACTIONS(2550),
    [sym_tls_client_open] = ACTIONS(2550),
    [sym_tls_client_close] = ACTIONS(2550),
    [sym_do_push] = ACTIONS(2550),
    [sym_do_push_timed] = ACTIONS(2550),
    [sym_do_release] = ACTIONS(2550),
    [sym_device_id] = ACTIONS(2550),
    [sym_device_info] = ACTIONS(2550),
    [sym_device_id_string] = ACTIONS(2550),
    [sym_device_standby] = ACTIONS(2550),
    [sym_device_wake] = ACTIONS(2550),
    [sym_timeline_create] = ACTIONS(2550),
    [sym_timeline_kill] = ACTIONS(2550),
    [sym_timeline_reload] = ACTIONS(2550),
    [sym_timeline_pause] = ACTIONS(2550),
    [sym_timeline_restart] = ACTIONS(2550),
    [sym_timeline_set] = ACTIONS(2550),
    [sym_timeline_get] = ACTIONS(2550),
    [sym_timeline_active] = ACTIONS(2550),
    [sym_timeline_dynamic_id] = ACTIONS(2550),
    [sym_variable_to_string] = ACTIONS(2550),
    [sym_string_to_variable] = ACTIONS(2550),
    [sym_length_variable_to_string] = ACTIONS(2550),
    [sym_type_cast] = ACTIONS(2550),
    [sym_variable_to_xml] = ACTIONS(2550),
    [sym_xml_to_variable] = ACTIONS(2550),
    [sym_length_variable_to_xml] = ACTIONS(2550),
    [sym_redirect_string] = ACTIONS(2550),
    [sym_random_number] = ACTIONS(2550),
    [sym_get_unique_id] = ACTIONS(2550),
    [sym_get_system_number] = ACTIONS(2550),
    [sym_set_system_number] = ACTIONS(2550),
    [sym_get_serial_number] = ACTIONS(2550),
    [sym_get_master_build] = ACTIONS(2550),
    [sym_set_validation_code] = ACTIONS(2550),
    [sym_reboot] = ACTIONS(2550),
    [sym_load_duet_module] = ACTIONS(2550),
    [sym_unload_duet_module] = ACTIONS(2550),
    [sym_duet_mem_size_set] = ACTIONS(2550),
    [sym_duet_mem_size_get] = ACTIONS(2550),
    [sym_dynamic_application_device] = ACTIONS(2550),
    [sym_dynamic_polled_port] = ACTIONS(2550),
    [sym_static_port_binding] = ACTIONS(2550),
    [sym_static_ip_binding] = ACTIONS(2550),
    [sym_get_dns_list] = ACTIONS(2550),
    [sym_set_dns_list] = ACTIONS(2550),
    [sym_get_ip_address] = ACTIONS(2550),
    [sym_set_ip_address] = ACTIONS(2550),
    [sym_get_url_list] = ACTIONS(2550),
    [sym_add_url_entry] = ACTIONS(2550),
    [sym_delete_url_entry] = ACTIONS(2550),
    [sym_abs_value] = ACTIONS(2550),
    [sym_max_value] = ACTIONS(2550),
    [sym_min_value] = ACTIONS(2550),
    [sym_exp_value] = ACTIONS(2550),
    [sym_log_value] = ACTIONS(2550),
    [sym_log10_value] = ACTIONS(2550),
    [sym_power_value] = ACTIONS(2550),
    [sym_sqrt_value] = ACTIONS(2550),
    [sym_raw_be] = ACTIONS(2550),
    [sym_raw_le] = ACTIONS(2550),
    [sym_internal_queue_size_set] = ACTIONS(2550),
    [sym_internal_queue_size_get] = ACTIONS(2550),
    [sym_internal_threshold_set] = ACTIONS(2550),
    [sym_internal_threshold_get] = ACTIONS(2550),
    [sym_rebuilt_event] = ACTIONS(2550),
    [sym_do_custom_event] = ACTIONS(2550),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2550),
    [sym_clkmgr_set_clk_source] = ACTIONS(2550),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2550),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2550),
    [sym_clkmgr_get_timezone] = ACTIONS(2550),
    [sym_clkmgr_set_timezone] = ACTIONS(2550),
    [sym_clkmgr_get_resync_period] = ACTIONS(2550),
    [sym_clkmgr_set_resync_period] = ACTIONS(2550),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2550),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2550),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2550),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2550),
    [sym_clkmgr_get_timeservers] = ACTIONS(2550),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2550),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2550),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2550),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2550),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2550),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2550),
    [sym_set_log_level] = ACTIONS(2550),
    [sym_get_log_level] = ACTIONS(2550),
    [sym_amx_log] = ACTIONS(2550),
    [sym_smtp_server_config_set] = ACTIONS(2550),
    [sym_smtp_server_config_get] = ACTIONS(2550),
    [sym_smtp_send] = ACTIONS(2550),
    [sym_validate_netlinx_account] = ACTIONS(2550),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2550),
    [sym_authenticate_certificate] = ACTIONS(2550),
    [sym_audit_netlinx_session_event] = ACTIONS(2550),
    [sym_audit_netlinx_generic_event] = ACTIONS(2550),
    [sym_get_available_flash_disk_space] = ACTIONS(2550),
    [sym_get_max_flash_disk_space] = ACTIONS(2550),
    [sym_data_create_feed] = ACTIONS(2550),
    [sym_data_delete_feed] = ACTIONS(2550),
    [sym_data_publish_feed] = ACTIONS(2550),
    [sym_data_get_published_feed] = ACTIONS(2550),
    [sym_data_add_record] = ACTIONS(2550),
    [sym_data_get_event_record] = ACTIONS(2550),
    [sym__wc_data_create_feed] = ACTIONS(2550),
    [sym__wc_data_add_record] = ACTIONS(2550),
    [sym__wc_data_get_event_record] = ACTIONS(2550),
    [sym_format] = ACTIONS(2550),
    [sym_astro_clock] = ACTIONS(2550),
    [sym_set_outdoor_temperature] = ACTIONS(2550),
    [sym_set_virtual_level_count] = ACTIONS(2550),
    [sym_set_virtual_channel_count] = ACTIONS(2550),
    [sym_set_virtual_port_count] = ACTIONS(2550),
    [sym__wc_encode_int] = ACTIONS(2550),
    [sym__wc_decode_int] = ACTIONS(2550),
    [sym__wc_int] = ACTIONS(2550),
    [sym__wc_to_ch_int] = ACTIONS(2550),
    [sym__ch_to_wc_int] = ACTIONS(2550),
    [sym__wc_find_string_int] = ACTIONS(2550),
    [sym__wc_left_string_int] = ACTIONS(2550),
    [sym__wc_lower_string_int] = ACTIONS(2550),
    [sym__wc_mid_string_int] = ACTIONS(2550),
    [sym__wc_remove_string_int] = ACTIONS(2550),
    [sym__wc_right_string_int] = ACTIONS(2550),
    [sym__wc_upper_string_int] = ACTIONS(2550),
    [sym__wc_compare_string_int] = ACTIONS(2550),
    [sym__wc_get_buffer_char_int] = ACTIONS(2550),
    [sym__wc_get_buffer_string_int] = ACTIONS(2550),
    [sym__wc_concat_string_int] = ACTIONS(2550),
    [sym__wc_file_open_int] = ACTIONS(2550),
    [sym__wc_file_close_int] = ACTIONS(2550),
    [sym__wc_file_read_int] = ACTIONS(2550),
    [sym__wc_file_read_line_int] = ACTIONS(2550),
    [sym__wc_file_write_int] = ACTIONS(2550),
    [sym__wc_file_write_line_int] = ACTIONS(2550),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2261),
    [aux_sym_preproc_else_token1] = ACTIONS(2261),
    [anon_sym_COMMA] = ACTIONS(2261),
    [anon_sym_LBRACK] = ACTIONS(2553),
    [anon_sym_LBRACE] = ACTIONS(2556),
    [anon_sym_LPAREN] = ACTIONS(2559),
    [anon_sym_BANG] = ACTIONS(2562),
    [anon_sym_TILDE] = ACTIONS(2562),
    [anon_sym_DASH] = ACTIONS(2538),
    [anon_sym_PLUS] = ACTIONS(2538),
    [anon_sym_DASH_DASH] = ACTIONS(2565),
    [anon_sym_PLUS_PLUS] = ACTIONS(2565),
    [anon_sym_DQUOTE] = ACTIONS(2568),
    [anon_sym_SQUOTE] = ACTIONS(2571),
    [sym_number_literal] = ACTIONS(2574),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(666)] = {
    [sym_expression] = STATE(1101),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2004),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_initializer_list] = STATE(2004),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(2531),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_SEMI] = ACTIONS(2577),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(2577),
  },
  [STATE(667)] = {
    [sym_expression] = STATE(631),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(946),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = STATE(669),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2579),
    [anon_sym_COMMA] = ACTIONS(2415),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(668)] = {
    [sym_expression] = STATE(1144),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(2042),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_initializer_list] = STATE(2042),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(2531),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_SEMI] = ACTIONS(2581),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
    [sym__automatic_semicolon] = ACTIONS(2581),
  },
  [STATE(669)] = {
    [sym_expression] = STATE(631),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(946),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_repeat1] = STATE(657),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2583),
    [anon_sym_COMMA] = ACTIONS(2415),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(670)] = {
    [sym_expression] = STATE(635),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(671),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = STATE(671),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2585),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(671)] = {
    [sym_expression] = STATE(635),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(663),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1] = STATE(663),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2587),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(672)] = {
    [sym_compound_statement] = STATE(1811),
    [sym_expression] = STATE(1148),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_RPAREN] = ACTIONS(2597),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(673)] = {
    [sym_compound_statement] = STATE(1816),
    [sym_expression] = STATE(1146),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_RPAREN] = ACTIONS(2605),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(674)] = {
    [sym__for_statement_body] = STATE(2188),
    [sym_expression] = STATE(1165),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2149),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_SEMI] = ACTIONS(2607),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(675)] = {
    [sym_compound_statement] = STATE(1839),
    [sym_expression] = STATE(1150),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_RPAREN] = ACTIONS(2609),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(676)] = {
    [sym_compound_statement] = STATE(1874),
    [sym_expression] = STATE(1111),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_RPAREN] = ACTIONS(2611),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(677)] = {
    [sym__for_statement_body] = STATE(2141),
    [sym_expression] = STATE(1165),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2149),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_SEMI] = ACTIONS(2607),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(678)] = {
    [sym_compound_statement] = STATE(1872),
    [sym_expression] = STATE(1152),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_RPAREN] = ACTIONS(2613),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(679)] = {
    [sym_compound_statement] = STATE(2052),
    [sym_expression] = STATE(1210),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2052),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(680)] = {
    [sym_compound_statement] = STATE(2148),
    [sym_expression] = STATE(1173),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2148),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(681)] = {
    [sym_compound_statement] = STATE(1813),
    [sym_expression] = STATE(1142),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_RPAREN] = ACTIONS(2615),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(682)] = {
    [sym_compound_statement] = STATE(2185),
    [sym_expression] = STATE(1184),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2185),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(683)] = {
    [sym_compound_statement] = STATE(2100),
    [sym_expression] = STATE(1222),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2100),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(684)] = {
    [sym_compound_statement] = STATE(2214),
    [sym_expression] = STATE(1198),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2214),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(685)] = {
    [sym_compound_statement] = STATE(2055),
    [sym_expression] = STATE(1201),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2055),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(686)] = {
    [sym_compound_statement] = STATE(2080),
    [sym_expression] = STATE(1204),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2080),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(687)] = {
    [sym__for_statement_body] = STATE(2122),
    [sym_expression] = STATE(1165),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2149),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_SEMI] = ACTIONS(2607),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(688)] = {
    [sym_expression] = STATE(1215),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_initializer_list] = STATE(1121),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(2617),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(689)] = {
    [sym_expression] = STATE(583),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(604),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(690)] = {
    [sym_expression] = STATE(575),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_initializer_list] = STATE(619),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LBRACE] = ACTIONS(2299),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(691)] = {
    [sym_expression] = STATE(1186),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2244),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_RPAREN] = ACTIONS(2623),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(692)] = {
    [sym_expression] = STATE(1195),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2256),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_SEMI] = ACTIONS(2625),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(693)] = {
    [sym_expression] = STATE(1193),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2099),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_RPAREN] = ACTIONS(2627),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(694)] = {
    [sym_expression] = STATE(163),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_initializer_list] = STATE(189),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LBRACE] = ACTIONS(2641),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(695)] = {
    [sym_expression] = STATE(156),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_initializer_list] = STATE(195),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LBRACE] = ACTIONS(2641),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(696)] = {
    [sym_expression] = STATE(1056),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_initializer_list] = STATE(1121),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(2617),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(697)] = {
    [sym_expression] = STATE(1153),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_initializer_list] = STATE(1226),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(2531),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(698)] = {
    [sym_expression] = STATE(67),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_initializer_list] = STATE(130),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LBRACE] = ACTIONS(2665),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(699)] = {
    [sym_expression] = STATE(78),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_initializer_list] = STATE(110),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LBRACE] = ACTIONS(2665),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(700)] = {
    [sym_expression] = STATE(1119),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_initializer_list] = STATE(1259),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(2531),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(701)] = {
    [sym_compound_statement] = STATE(1950),
    [sym_expression] = STATE(1220),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(155),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(702)] = {
    [sym_expression] = STATE(1094),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_initializer_list] = STATE(1115),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(2617),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(703)] = {
    [sym_expression] = STATE(1170),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2159),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_SEMI] = ACTIONS(2677),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(704)] = {
    [sym_expression] = STATE(1159),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2072),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_RPAREN] = ACTIONS(2679),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(705)] = {
    [sym_expression] = STATE(1160),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2075),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_RPAREN] = ACTIONS(2681),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(706)] = {
    [sym_expression] = STATE(1130),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_initializer_list] = STATE(1834),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LBRACE] = ACTIONS(2531),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(707)] = {
    [sym_expression] = STATE(94),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_initializer_list] = STATE(137),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LBRACE] = ACTIONS(2695),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(708)] = {
    [sym_expression] = STATE(1205),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_initializer_list] = STATE(1115),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LBRACE] = ACTIONS(2617),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(709)] = {
    [sym_expression] = STATE(84),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_initializer_list] = STATE(142),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LBRACE] = ACTIONS(2695),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(710)] = {
    [sym_expression] = STATE(1281),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2034),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(711)] = {
    [sym_expression] = STATE(1241),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_RBRACK] = ACTIONS(2707),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(712)] = {
    [sym_expression] = STATE(1281),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2038),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(713)] = {
    [sym_expression] = STATE(1254),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_RBRACK] = ACTIONS(2709),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(714)] = {
    [sym_expression] = STATE(1244),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_RBRACK] = ACTIONS(2711),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(715)] = {
    [sym_expression] = STATE(1246),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_RBRACK] = ACTIONS(2713),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(716)] = {
    [sym_expression] = STATE(1247),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_RBRACK] = ACTIONS(2715),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(717)] = {
    [sym_expression] = STATE(1110),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_comma_expression] = STATE(1980),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(718)] = {
    [sym_expression] = STATE(1232),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_RBRACK] = ACTIONS(2717),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(719)] = {
    [sym_expression] = STATE(1231),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_RBRACK] = ACTIONS(2719),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(720)] = {
    [sym_expression] = STATE(1255),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_RBRACK] = ACTIONS(2721),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(721)] = {
    [sym_expression] = STATE(1229),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_RBRACK] = ACTIONS(2723),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(722)] = {
    [sym_expression] = STATE(1120),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(1996),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(723)] = {
    [sym_expression] = STATE(1250),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_RBRACK] = ACTIONS(2725),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(724)] = {
    [sym_expression] = STATE(1252),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_RBRACK] = ACTIONS(2727),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(725)] = {
    [sym_expression] = STATE(1257),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_RBRACK] = ACTIONS(2729),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(726)] = {
    [sym_expression] = STATE(1281),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(1921),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(727)] = {
    [sym_expression] = STATE(1281),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(1922),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(728)] = {
    [sym_expression] = STATE(1249),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_RBRACK] = ACTIONS(2731),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(729)] = {
    [sym_expression] = STATE(1281),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2045),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(730)] = {
    [sym_expression] = STATE(1281),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_comma_expression] = STATE(2046),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(731)] = {
    [sym_expression] = STATE(1085),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(732)] = {
    [sym_expression] = STATE(1239),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(733)] = {
    [sym_expression] = STATE(1192),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(734)] = {
    [sym_expression] = STATE(1196),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(735)] = {
    [sym_expression] = STATE(161),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(736)] = {
    [sym_expression] = STATE(92),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(737)] = {
    [sym_expression] = STATE(590),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(738)] = {
    [sym_expression] = STATE(152),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(739)] = {
    [sym_expression] = STATE(165),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(740)] = {
    [sym_expression] = STATE(167),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(741)] = {
    [sym_expression] = STATE(168),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(742)] = {
    [sym_expression] = STATE(169),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(743)] = {
    [sym_expression] = STATE(170),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(744)] = {
    [sym_expression] = STATE(164),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(745)] = {
    [sym_expression] = STATE(153),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(746)] = {
    [sym_expression] = STATE(158),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(747)] = {
    [sym_expression] = STATE(154),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(748)] = {
    [sym_expression] = STATE(591),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(749)] = {
    [sym_expression] = STATE(592),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(750)] = {
    [sym_expression] = STATE(574),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(751)] = {
    [sym_expression] = STATE(1256),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(752)] = {
    [sym_expression] = STATE(166),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(753)] = {
    [sym_expression] = STATE(579),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(754)] = {
    [sym_expression] = STATE(64),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(755)] = {
    [sym_expression] = STATE(68),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(756)] = {
    [sym_expression] = STATE(69),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(757)] = {
    [sym_expression] = STATE(70),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(758)] = {
    [sym_expression] = STATE(59),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(759)] = {
    [sym_expression] = STATE(71),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(760)] = {
    [sym_expression] = STATE(72),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(761)] = {
    [sym_expression] = STATE(73),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(762)] = {
    [sym_expression] = STATE(74),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(763)] = {
    [sym_expression] = STATE(75),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(764)] = {
    [sym_expression] = STATE(76),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(765)] = {
    [sym_expression] = STATE(1295),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(766)] = {
    [sym_expression] = STATE(1289),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(767)] = {
    [sym_expression] = STATE(1113),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(768)] = {
    [sym_expression] = STATE(1114),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(769)] = {
    [sym_expression] = STATE(1116),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(770)] = {
    [sym_expression] = STATE(1117),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(771)] = {
    [sym_expression] = STATE(1263),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(772)] = {
    [sym_expression] = STATE(1272),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(773)] = {
    [sym_expression] = STATE(1200),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(774)] = {
    [sym_expression] = STATE(1043),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(775)] = {
    [sym_expression] = STATE(1125),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(776)] = {
    [sym_expression] = STATE(1242),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(777)] = {
    [sym_expression] = STATE(65),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(778)] = {
    [sym_expression] = STATE(1136),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(779)] = {
    [sym_expression] = STATE(1169),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(780)] = {
    [sym_expression] = STATE(1104),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(781)] = {
    [sym_expression] = STATE(1128),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2733),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(782)] = {
    [sym_expression] = STATE(1225),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(783)] = {
    [sym_expression] = STATE(1235),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(784)] = {
    [sym_expression] = STATE(1161),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(785)] = {
    [sym_expression] = STATE(151),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(786)] = {
    [sym_expression] = STATE(150),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(787)] = {
    [sym_expression] = STATE(1046),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(788)] = {
    [sym_expression] = STATE(1141),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2735),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(789)] = {
    [sym_expression] = STATE(1185),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(790)] = {
    [sym_expression] = STATE(1167),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(791)] = {
    [sym_expression] = STATE(1168),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(792)] = {
    [sym_expression] = STATE(1065),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(793)] = {
    [sym_expression] = STATE(1237),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(794)] = {
    [sym_expression] = STATE(61),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(795)] = {
    [sym_expression] = STATE(584),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(796)] = {
    [sym_expression] = STATE(585),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(797)] = {
    [sym_expression] = STATE(1127),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(798)] = {
    [sym_expression] = STATE(86),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(799)] = {
    [sym_expression] = STATE(1081),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(800)] = {
    [sym_expression] = STATE(1164),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(801)] = {
    [sym_expression] = STATE(57),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(802)] = {
    [sym_expression] = STATE(58),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(803)] = {
    [sym_expression] = STATE(91),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(804)] = {
    [sym_expression] = STATE(1236),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(805)] = {
    [sym_expression] = STATE(1145),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2737),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(806)] = {
    [sym_expression] = STATE(1197),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(807)] = {
    [sym_expression] = STATE(1194),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(808)] = {
    [sym_expression] = STATE(60),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(809)] = {
    [sym_expression] = STATE(1097),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(810)] = {
    [sym_expression] = STATE(1166),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(811)] = {
    [sym_expression] = STATE(1284),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(812)] = {
    [sym_expression] = STATE(1238),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(813)] = {
    [sym_expression] = STATE(1086),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(814)] = {
    [sym_expression] = STATE(1070),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(815)] = {
    [sym_expression] = STATE(1087),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(816)] = {
    [sym_expression] = STATE(1079),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(817)] = {
    [sym_expression] = STATE(1224),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(818)] = {
    [sym_expression] = STATE(1105),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(819)] = {
    [sym_expression] = STATE(1051),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(820)] = {
    [sym_expression] = STATE(1106),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(821)] = {
    [sym_expression] = STATE(1216),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(822)] = {
    [sym_expression] = STATE(577),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(823)] = {
    [sym_expression] = STATE(1053),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(824)] = {
    [sym_expression] = STATE(1147),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2739),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(825)] = {
    [sym_expression] = STATE(1044),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(826)] = {
    [sym_expression] = STATE(1107),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(827)] = {
    [sym_expression] = STATE(1294),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(828)] = {
    [sym_expression] = STATE(160),
    [sym_compiler_variable] = STATE(182),
    [sym_system_variable] = STATE(175),
    [sym_system_constant] = STATE(182),
    [sym_system_function] = STATE(1946),
    [sym_assignment_expression] = STATE(182),
    [sym_unary_expression] = STATE(182),
    [sym_binary_expression] = STATE(182),
    [sym_update_expression] = STATE(182),
    [sym_subscript_expression] = STATE(176),
    [sym_call_expression] = STATE(176),
    [sym_field_expression] = STATE(176),
    [sym_parenthesized_expression] = STATE(176),
    [sym_devchan_expression] = STATE(159),
    [sym_devchan_range_expression] = STATE(182),
    [sym_string_expression] = STATE(182),
    [sym_device_literal] = STATE(182),
    [sym_string_literal] = STATE(182),
    [sym_identifier] = ACTIONS(2629),
    [sym_not] = ACTIONS(2631),
    [sym_bnot] = ACTIONS(2631),
    [sym___file__] = ACTIONS(2633),
    [sym___date__] = ACTIONS(2633),
    [sym___ldate__] = ACTIONS(2633),
    [sym___time__] = ACTIONS(2633),
    [sym___line__] = ACTIONS(2633),
    [sym___name__] = ACTIONS(2633),
    [sym___netlinx__] = ACTIONS(2635),
    [sym_netlinx_axi_version] = ACTIONS(2635),
    [sym_url_flg_tcp] = ACTIONS(2635),
    [sym_url_flg_accinfopresent] = ACTIONS(2635),
    [sym_url_flg_temp] = ACTIONS(2635),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2635),
    [sym_url_flg_stat_mask] = ACTIONS(2635),
    [sym_url_flg_stat_lookup] = ACTIONS(2635),
    [sym_url_flg_stat_connecting] = ACTIONS(2635),
    [sym_url_flg_stat_waiting] = ACTIONS(2635),
    [sym_url_flg_stat_connected] = ACTIONS(2635),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2635),
    [sym_first_virtual_device] = ACTIONS(2635),
    [sym_first_local_port] = ACTIONS(2635),
    [sym_dynamic_virtual_device] = ACTIONS(2635),
    [sym_timeline_once] = ACTIONS(2635),
    [sym_timeline_repeat] = ACTIONS(2635),
    [sym_timeline_absolute] = ACTIONS(2635),
    [sym_timeline_relative] = ACTIONS(2635),
    [sym_do_push_timed_infinite] = ACTIONS(2635),
    [sym_source_type_no_address] = ACTIONS(2635),
    [sym_source_type_neuron_id] = ACTIONS(2635),
    [sym_source_type_ip_address] = ACTIONS(2635),
    [sym_source_type_axlink] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2635),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2635),
    [sym_source_type_ip_socket_address] = ACTIONS(2635),
    [sym_source_type_rs232] = ACTIONS(2635),
    [sym_source_type_internal] = ACTIONS(2635),
    [sym_source_type_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2635),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2635),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2635),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2635),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2635),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2635),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2635),
    [sym_true] = ACTIONS(2635),
    [sym_false] = ACTIONS(2635),
    [sym_file_read_only] = ACTIONS(2635),
    [sym_file_rw_new] = ACTIONS(2635),
    [sym_file_rw_append] = ACTIONS(2635),
    [sym_ip_tcp] = ACTIONS(2635),
    [sym_ip_udp] = ACTIONS(2635),
    [sym_ip_udp_2way] = ACTIONS(2635),
    [sym_xml_encode_types] = ACTIONS(2635),
    [sym_xml_encode_char_as_list] = ACTIONS(2635),
    [sym_xml_encode_le] = ACTIONS(2635),
    [sym_xml_decode_types] = ACTIONS(2635),
    [sym_xml_decode_no_preserve] = ACTIONS(2635),
    [sym_duet_dev_type_display_device] = ACTIONS(2635),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2635),
    [sym_duet_dev_type_io_device] = ACTIONS(2635),
    [sym_duet_dev_type_relay_device] = ACTIONS(2635),
    [sym_duet_dev_type_ups] = ACTIONS(2635),
    [sym_duet_dev_type_amplifier] = ACTIONS(2635),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2635),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2635),
    [sym_duet_dev_type_camera] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2635),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2635),
    [sym_duet_dev_type_dss] = ACTIONS(2635),
    [sym_duet_dev_type_dvr] = ACTIONS(2635),
    [sym_duet_dev_type_disc_device] = ACTIONS(2635),
    [sym_duet_dev_type_document_camera] = ACTIONS(2635),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2635),
    [sym_duet_dev_type_hvac] = ACTIONS(2635),
    [sym_duet_dev_type_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_light] = ACTIONS(2635),
    [sym_duet_dev_type_monitor] = ACTIONS(2635),
    [sym_duet_dev_type_motor] = ACTIONS(2635),
    [sym_duet_dev_type_multi_window] = ACTIONS(2635),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2635),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2635),
    [sym_duet_dev_type_receiver] = ACTIONS(2635),
    [sym_duet_dev_type_security_system] = ACTIONS(2635),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2635),
    [sym_duet_dev_type_settop_box] = ACTIONS(2635),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2635),
    [sym_duet_dev_type_switcher] = ACTIONS(2635),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2635),
    [sym_duet_dev_type_tv] = ACTIONS(2635),
    [sym_duet_dev_type_utility] = ACTIONS(2635),
    [sym_duet_dev_type_vcr] = ACTIONS(2635),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2635),
    [sym_duet_dev_type_video_processor] = ACTIONS(2635),
    [sym_duet_dev_type_video_projector] = ACTIONS(2635),
    [sym_duet_dev_type_video_wall] = ACTIONS(2635),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2635),
    [sym_duet_dev_type_weather] = ACTIONS(2635),
    [sym_duet_dev_not_polled] = ACTIONS(2635),
    [sym_duet_dev_polled] = ACTIONS(2635),
    [sym_clkmgr_mode_network] = ACTIONS(2635),
    [sym_clkmgr_mode_standalone] = ACTIONS(2635),
    [sym_normal_standby] = ACTIONS(2635),
    [sym_normal_wake] = ACTIONS(2635),
    [sym_ip_multicast_ttl_option] = ACTIONS(2635),
    [sym_ip_ttl_subnet] = ACTIONS(2635),
    [sym_ip_ttl_site] = ACTIONS(2635),
    [sym_ip_ttl_region] = ACTIONS(2635),
    [sym_ip_ttl_continent] = ACTIONS(2635),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2635),
    [sym_ip_nodelay_on] = ACTIONS(2635),
    [sym_ip_nodelay_off] = ACTIONS(2635),
    [sym_amx_error] = ACTIONS(2635),
    [sym_amx_warning] = ACTIONS(2635),
    [sym_amx_info] = ACTIONS(2635),
    [sym_amx_debug] = ACTIONS(2635),
    [sym_netlinx_login_success] = ACTIONS(2635),
    [sym_netlinx_login_fail] = ACTIONS(2635),
    [sym_netlinx_logout] = ACTIONS(2635),
    [sym_null_str] = ACTIONS(2635),
    [sym_smtp_address] = ACTIONS(2635),
    [sym_smtp_port_number] = ACTIONS(2635),
    [sym_smtp_username] = ACTIONS(2635),
    [sym_smtp_password] = ACTIONS(2635),
    [sym_smtp_require_tls] = ACTIONS(2635),
    [sym_smtp_from] = ACTIONS(2635),
    [sym_smtp_tls_true] = ACTIONS(2635),
    [sym_smtp_tls_false] = ACTIONS(2635),
    [sym_listview_on_row_select_event] = ACTIONS(2635),
    [sym_data_structure_unknown] = ACTIONS(2635),
    [sym_data_structure_datafeed] = ACTIONS(2635),
    [sym_data_structure_datarecordset] = ACTIONS(2635),
    [sym_data_structure_datarecord] = ACTIONS(2635),
    [sym_data_structure_datafield] = ACTIONS(2635),
    [sym_data_max_content_field_count] = ACTIONS(2635),
    [sym_data_max_metadata_field_count] = ACTIONS(2635),
    [sym_data_max_id_length] = ACTIONS(2635),
    [sym_data_max_name_length] = ACTIONS(2635),
    [sym_data_max_description_length] = ACTIONS(2635),
    [sym_data_max_type_length] = ACTIONS(2635),
    [sym_data_max_format_length] = ACTIONS(2635),
    [sym_data_max_source_length] = ACTIONS(2635),
    [sym_data_max_value_length] = ACTIONS(2635),
    [sym_data_max_label_length] = ACTIONS(2635),
    [sym_data_max_url_length] = ACTIONS(2635),
    [sym_max_last_login_info_length] = ACTIONS(2635),
    [sym_data_type_unknown] = ACTIONS(2635),
    [sym_data_type_string] = ACTIONS(2635),
    [sym_data_type_datetime] = ACTIONS(2635),
    [sym_data_type_date] = ACTIONS(2635),
    [sym_data_type_time] = ACTIONS(2635),
    [sym_data_type_image] = ACTIONS(2635),
    [sym_data_format_url] = ACTIONS(2635),
    [sym_data_format_phone] = ACTIONS(2635),
    [sym_data_format_email] = ACTIONS(2635),
    [sym_data_format_iso8601] = ACTIONS(2635),
    [sym_auditlog_priv] = ACTIONS(2635),
    [sym_configuration_priv] = ACTIONS(2635),
    [sym_deviceconfig_priv] = ACTIONS(2635),
    [sym_ftp_priv] = ACTIONS(2635),
    [sym_http_priv] = ACTIONS(2635),
    [sym_networkconfig_priv] = ACTIONS(2635),
    [sym_programport_priv] = ACTIONS(2635),
    [sym_remoteui_priv] = ACTIONS(2635),
    [sym_securitycontrol_priv] = ACTIONS(2635),
    [sym_softwaremanagement_priv] = ACTIONS(2635),
    [sym_terminal_priv] = ACTIONS(2635),
    [sym_tpadmin_priv] = ACTIONS(2635),
    [sym_user1_priv] = ACTIONS(2635),
    [sym_user2_priv] = ACTIONS(2635),
    [sym_user3_priv] = ACTIONS(2635),
    [sym_user4_priv] = ACTIONS(2635),
    [sym_usermanagement_priv] = ACTIONS(2635),
    [sym_valid_account] = ACTIONS(2635),
    [sym_err_name_invalid] = ACTIONS(2635),
    [sym_err_password_invalid] = ACTIONS(2635),
    [sym_err_invalid_account] = ACTIONS(2635),
    [sym_err_no_authorization] = ACTIONS(2635),
    [sym_err_invalid_parameter] = ACTIONS(2635),
    [sym_err_user_locked_out] = ACTIONS(2635),
    [sym_err_user_account_expired] = ACTIONS(2635),
    [sym_err_general_error] = ACTIONS(2635),
    [sym_err_authentication_error] = ACTIONS(2635),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2635),
    [sym_tls_validate_certificate] = ACTIONS(2635),
    [sym____reserved___] = ACTIONS(2637),
    [sym_date] = ACTIONS(2637),
    [sym_day] = ACTIONS(2637),
    [sym_time] = ACTIONS(2637),
    [sym_ldate] = ACTIONS(2637),
    [sym_push_device] = ACTIONS(2637),
    [sym_push_channel] = ACTIONS(2637),
    [sym_push_devchan] = ACTIONS(2637),
    [sym_release_device] = ACTIONS(2637),
    [sym_release_channel] = ACTIONS(2637),
    [sym_release_devchan] = ACTIONS(2637),
    [sym_master_slot] = ACTIONS(2637),
    [sym_get_pulse_time] = ACTIONS(2637),
    [sym_get_timer] = ACTIONS(2637),
    [sym_master_sn] = ACTIONS(2637),
    [sym_system_number] = ACTIONS(2637),
    [sym_button] = ACTIONS(2637),
    [sym_dv_channel] = ACTIONS(2637),
    [sym_data] = ACTIONS(2637),
    [sym_level] = ACTIONS(2637),
    [sym_timeline] = ACTIONS(2637),
    [sym_channel] = ACTIONS(2637),
    [sym_custom] = ACTIONS(2637),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2639),
    [anon_sym_LPAREN] = ACTIONS(2643),
    [anon_sym_BANG] = ACTIONS(2645),
    [anon_sym_TILDE] = ACTIONS(2645),
    [anon_sym_DASH] = ACTIONS(2631),
    [anon_sym_PLUS] = ACTIONS(2631),
    [anon_sym_DASH_DASH] = ACTIONS(2647),
    [anon_sym_PLUS_PLUS] = ACTIONS(2647),
    [anon_sym_DQUOTE] = ACTIONS(2649),
    [anon_sym_SQUOTE] = ACTIONS(1649),
    [sym_number_literal] = ACTIONS(2651),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(829)] = {
    [sym_expression] = STATE(1149),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2741),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(830)] = {
    [sym_expression] = STATE(1108),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(831)] = {
    [sym_expression] = STATE(1283),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(832)] = {
    [sym_expression] = STATE(63),
    [sym_compiler_variable] = STATE(112),
    [sym_system_variable] = STATE(88),
    [sym_system_constant] = STATE(112),
    [sym_system_function] = STATE(1959),
    [sym_assignment_expression] = STATE(112),
    [sym_unary_expression] = STATE(112),
    [sym_binary_expression] = STATE(112),
    [sym_update_expression] = STATE(112),
    [sym_subscript_expression] = STATE(89),
    [sym_call_expression] = STATE(89),
    [sym_field_expression] = STATE(89),
    [sym_parenthesized_expression] = STATE(89),
    [sym_devchan_expression] = STATE(62),
    [sym_devchan_range_expression] = STATE(112),
    [sym_string_expression] = STATE(112),
    [sym_device_literal] = STATE(112),
    [sym_string_literal] = STATE(112),
    [sym_identifier] = ACTIONS(2653),
    [sym_not] = ACTIONS(2655),
    [sym_bnot] = ACTIONS(2655),
    [sym___file__] = ACTIONS(2657),
    [sym___date__] = ACTIONS(2657),
    [sym___ldate__] = ACTIONS(2657),
    [sym___time__] = ACTIONS(2657),
    [sym___line__] = ACTIONS(2657),
    [sym___name__] = ACTIONS(2657),
    [sym___netlinx__] = ACTIONS(2659),
    [sym_netlinx_axi_version] = ACTIONS(2659),
    [sym_url_flg_tcp] = ACTIONS(2659),
    [sym_url_flg_accinfopresent] = ACTIONS(2659),
    [sym_url_flg_temp] = ACTIONS(2659),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2659),
    [sym_url_flg_stat_mask] = ACTIONS(2659),
    [sym_url_flg_stat_lookup] = ACTIONS(2659),
    [sym_url_flg_stat_connecting] = ACTIONS(2659),
    [sym_url_flg_stat_waiting] = ACTIONS(2659),
    [sym_url_flg_stat_connected] = ACTIONS(2659),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2659),
    [sym_first_virtual_device] = ACTIONS(2659),
    [sym_first_local_port] = ACTIONS(2659),
    [sym_dynamic_virtual_device] = ACTIONS(2659),
    [sym_timeline_once] = ACTIONS(2659),
    [sym_timeline_repeat] = ACTIONS(2659),
    [sym_timeline_absolute] = ACTIONS(2659),
    [sym_timeline_relative] = ACTIONS(2659),
    [sym_do_push_timed_infinite] = ACTIONS(2659),
    [sym_source_type_no_address] = ACTIONS(2659),
    [sym_source_type_neuron_id] = ACTIONS(2659),
    [sym_source_type_ip_address] = ACTIONS(2659),
    [sym_source_type_axlink] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2659),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2659),
    [sym_source_type_ip_socket_address] = ACTIONS(2659),
    [sym_source_type_rs232] = ACTIONS(2659),
    [sym_source_type_internal] = ACTIONS(2659),
    [sym_source_type_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2659),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2659),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2659),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2659),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2659),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2659),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2659),
    [sym_true] = ACTIONS(2659),
    [sym_false] = ACTIONS(2659),
    [sym_file_read_only] = ACTIONS(2659),
    [sym_file_rw_new] = ACTIONS(2659),
    [sym_file_rw_append] = ACTIONS(2659),
    [sym_ip_tcp] = ACTIONS(2659),
    [sym_ip_udp] = ACTIONS(2659),
    [sym_ip_udp_2way] = ACTIONS(2659),
    [sym_xml_encode_types] = ACTIONS(2659),
    [sym_xml_encode_char_as_list] = ACTIONS(2659),
    [sym_xml_encode_le] = ACTIONS(2659),
    [sym_xml_decode_types] = ACTIONS(2659),
    [sym_xml_decode_no_preserve] = ACTIONS(2659),
    [sym_duet_dev_type_display_device] = ACTIONS(2659),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2659),
    [sym_duet_dev_type_io_device] = ACTIONS(2659),
    [sym_duet_dev_type_relay_device] = ACTIONS(2659),
    [sym_duet_dev_type_ups] = ACTIONS(2659),
    [sym_duet_dev_type_amplifier] = ACTIONS(2659),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2659),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2659),
    [sym_duet_dev_type_camera] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2659),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2659),
    [sym_duet_dev_type_dss] = ACTIONS(2659),
    [sym_duet_dev_type_dvr] = ACTIONS(2659),
    [sym_duet_dev_type_disc_device] = ACTIONS(2659),
    [sym_duet_dev_type_document_camera] = ACTIONS(2659),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2659),
    [sym_duet_dev_type_hvac] = ACTIONS(2659),
    [sym_duet_dev_type_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_light] = ACTIONS(2659),
    [sym_duet_dev_type_monitor] = ACTIONS(2659),
    [sym_duet_dev_type_motor] = ACTIONS(2659),
    [sym_duet_dev_type_multi_window] = ACTIONS(2659),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2659),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2659),
    [sym_duet_dev_type_receiver] = ACTIONS(2659),
    [sym_duet_dev_type_security_system] = ACTIONS(2659),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2659),
    [sym_duet_dev_type_settop_box] = ACTIONS(2659),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2659),
    [sym_duet_dev_type_switcher] = ACTIONS(2659),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2659),
    [sym_duet_dev_type_tv] = ACTIONS(2659),
    [sym_duet_dev_type_utility] = ACTIONS(2659),
    [sym_duet_dev_type_vcr] = ACTIONS(2659),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2659),
    [sym_duet_dev_type_video_processor] = ACTIONS(2659),
    [sym_duet_dev_type_video_projector] = ACTIONS(2659),
    [sym_duet_dev_type_video_wall] = ACTIONS(2659),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2659),
    [sym_duet_dev_type_weather] = ACTIONS(2659),
    [sym_duet_dev_not_polled] = ACTIONS(2659),
    [sym_duet_dev_polled] = ACTIONS(2659),
    [sym_clkmgr_mode_network] = ACTIONS(2659),
    [sym_clkmgr_mode_standalone] = ACTIONS(2659),
    [sym_normal_standby] = ACTIONS(2659),
    [sym_normal_wake] = ACTIONS(2659),
    [sym_ip_multicast_ttl_option] = ACTIONS(2659),
    [sym_ip_ttl_subnet] = ACTIONS(2659),
    [sym_ip_ttl_site] = ACTIONS(2659),
    [sym_ip_ttl_region] = ACTIONS(2659),
    [sym_ip_ttl_continent] = ACTIONS(2659),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2659),
    [sym_ip_nodelay_on] = ACTIONS(2659),
    [sym_ip_nodelay_off] = ACTIONS(2659),
    [sym_amx_error] = ACTIONS(2659),
    [sym_amx_warning] = ACTIONS(2659),
    [sym_amx_info] = ACTIONS(2659),
    [sym_amx_debug] = ACTIONS(2659),
    [sym_netlinx_login_success] = ACTIONS(2659),
    [sym_netlinx_login_fail] = ACTIONS(2659),
    [sym_netlinx_logout] = ACTIONS(2659),
    [sym_null_str] = ACTIONS(2659),
    [sym_smtp_address] = ACTIONS(2659),
    [sym_smtp_port_number] = ACTIONS(2659),
    [sym_smtp_username] = ACTIONS(2659),
    [sym_smtp_password] = ACTIONS(2659),
    [sym_smtp_require_tls] = ACTIONS(2659),
    [sym_smtp_from] = ACTIONS(2659),
    [sym_smtp_tls_true] = ACTIONS(2659),
    [sym_smtp_tls_false] = ACTIONS(2659),
    [sym_listview_on_row_select_event] = ACTIONS(2659),
    [sym_data_structure_unknown] = ACTIONS(2659),
    [sym_data_structure_datafeed] = ACTIONS(2659),
    [sym_data_structure_datarecordset] = ACTIONS(2659),
    [sym_data_structure_datarecord] = ACTIONS(2659),
    [sym_data_structure_datafield] = ACTIONS(2659),
    [sym_data_max_content_field_count] = ACTIONS(2659),
    [sym_data_max_metadata_field_count] = ACTIONS(2659),
    [sym_data_max_id_length] = ACTIONS(2659),
    [sym_data_max_name_length] = ACTIONS(2659),
    [sym_data_max_description_length] = ACTIONS(2659),
    [sym_data_max_type_length] = ACTIONS(2659),
    [sym_data_max_format_length] = ACTIONS(2659),
    [sym_data_max_source_length] = ACTIONS(2659),
    [sym_data_max_value_length] = ACTIONS(2659),
    [sym_data_max_label_length] = ACTIONS(2659),
    [sym_data_max_url_length] = ACTIONS(2659),
    [sym_max_last_login_info_length] = ACTIONS(2659),
    [sym_data_type_unknown] = ACTIONS(2659),
    [sym_data_type_string] = ACTIONS(2659),
    [sym_data_type_datetime] = ACTIONS(2659),
    [sym_data_type_date] = ACTIONS(2659),
    [sym_data_type_time] = ACTIONS(2659),
    [sym_data_type_image] = ACTIONS(2659),
    [sym_data_format_url] = ACTIONS(2659),
    [sym_data_format_phone] = ACTIONS(2659),
    [sym_data_format_email] = ACTIONS(2659),
    [sym_data_format_iso8601] = ACTIONS(2659),
    [sym_auditlog_priv] = ACTIONS(2659),
    [sym_configuration_priv] = ACTIONS(2659),
    [sym_deviceconfig_priv] = ACTIONS(2659),
    [sym_ftp_priv] = ACTIONS(2659),
    [sym_http_priv] = ACTIONS(2659),
    [sym_networkconfig_priv] = ACTIONS(2659),
    [sym_programport_priv] = ACTIONS(2659),
    [sym_remoteui_priv] = ACTIONS(2659),
    [sym_securitycontrol_priv] = ACTIONS(2659),
    [sym_softwaremanagement_priv] = ACTIONS(2659),
    [sym_terminal_priv] = ACTIONS(2659),
    [sym_tpadmin_priv] = ACTIONS(2659),
    [sym_user1_priv] = ACTIONS(2659),
    [sym_user2_priv] = ACTIONS(2659),
    [sym_user3_priv] = ACTIONS(2659),
    [sym_user4_priv] = ACTIONS(2659),
    [sym_usermanagement_priv] = ACTIONS(2659),
    [sym_valid_account] = ACTIONS(2659),
    [sym_err_name_invalid] = ACTIONS(2659),
    [sym_err_password_invalid] = ACTIONS(2659),
    [sym_err_invalid_account] = ACTIONS(2659),
    [sym_err_no_authorization] = ACTIONS(2659),
    [sym_err_invalid_parameter] = ACTIONS(2659),
    [sym_err_user_locked_out] = ACTIONS(2659),
    [sym_err_user_account_expired] = ACTIONS(2659),
    [sym_err_general_error] = ACTIONS(2659),
    [sym_err_authentication_error] = ACTIONS(2659),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2659),
    [sym_tls_validate_certificate] = ACTIONS(2659),
    [sym____reserved___] = ACTIONS(2661),
    [sym_date] = ACTIONS(2661),
    [sym_day] = ACTIONS(2661),
    [sym_time] = ACTIONS(2661),
    [sym_ldate] = ACTIONS(2661),
    [sym_push_device] = ACTIONS(2661),
    [sym_push_channel] = ACTIONS(2661),
    [sym_push_devchan] = ACTIONS(2661),
    [sym_release_device] = ACTIONS(2661),
    [sym_release_channel] = ACTIONS(2661),
    [sym_release_devchan] = ACTIONS(2661),
    [sym_master_slot] = ACTIONS(2661),
    [sym_get_pulse_time] = ACTIONS(2661),
    [sym_get_timer] = ACTIONS(2661),
    [sym_master_sn] = ACTIONS(2661),
    [sym_system_number] = ACTIONS(2661),
    [sym_button] = ACTIONS(2661),
    [sym_dv_channel] = ACTIONS(2661),
    [sym_data] = ACTIONS(2661),
    [sym_level] = ACTIONS(2661),
    [sym_timeline] = ACTIONS(2661),
    [sym_channel] = ACTIONS(2661),
    [sym_custom] = ACTIONS(2661),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2663),
    [anon_sym_LPAREN] = ACTIONS(2667),
    [anon_sym_BANG] = ACTIONS(2669),
    [anon_sym_TILDE] = ACTIONS(2669),
    [anon_sym_DASH] = ACTIONS(2655),
    [anon_sym_PLUS] = ACTIONS(2655),
    [anon_sym_DASH_DASH] = ACTIONS(2671),
    [anon_sym_PLUS_PLUS] = ACTIONS(2671),
    [anon_sym_DQUOTE] = ACTIONS(2673),
    [anon_sym_SQUOTE] = ACTIONS(1405),
    [sym_number_literal] = ACTIONS(2675),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(833)] = {
    [sym_expression] = STATE(1141),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2157),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(834)] = {
    [sym_expression] = STATE(95),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(835)] = {
    [sym_expression] = STATE(1109),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(836)] = {
    [sym_expression] = STATE(96),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(837)] = {
    [sym_expression] = STATE(99),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(838)] = {
    [sym_expression] = STATE(100),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(839)] = {
    [sym_expression] = STATE(1287),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(840)] = {
    [sym_expression] = STATE(1151),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2743),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(841)] = {
    [sym_expression] = STATE(1291),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(842)] = {
    [sym_expression] = STATE(1285),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(843)] = {
    [sym_expression] = STATE(1172),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(844)] = {
    [sym_expression] = STATE(586),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(845)] = {
    [sym_expression] = STATE(587),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(846)] = {
    [sym_expression] = STATE(101),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(847)] = {
    [sym_expression] = STATE(1293),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(848)] = {
    [sym_expression] = STATE(1176),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(849)] = {
    [sym_expression] = STATE(1177),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(850)] = {
    [sym_expression] = STATE(1243),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(851)] = {
    [sym_expression] = STATE(1178),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(852)] = {
    [sym_expression] = STATE(1179),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(853)] = {
    [sym_expression] = STATE(1180),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(854)] = {
    [sym_expression] = STATE(1043),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(855)] = {
    [sym_expression] = STATE(588),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(856)] = {
    [sym_expression] = STATE(1206),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(857)] = {
    [sym_expression] = STATE(1207),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(858)] = {
    [sym_expression] = STATE(1208),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(859)] = {
    [sym_expression] = STATE(1209),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(860)] = {
    [sym_expression] = STATE(1158),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(861)] = {
    [sym_expression] = STATE(1211),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(862)] = {
    [sym_expression] = STATE(1212),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(863)] = {
    [sym_expression] = STATE(1044),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(864)] = {
    [sym_expression] = STATE(1213),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(865)] = {
    [sym_expression] = STATE(1214),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(866)] = {
    [sym_expression] = STATE(102),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(867)] = {
    [sym_expression] = STATE(1046),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1268),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1269),
    [sym_call_expression] = STATE(1269),
    [sym_field_expression] = STATE(1269),
    [sym_parenthesized_expression] = STATE(1269),
    [sym_devchan_expression] = STATE(1218),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2145),
    [sym_not] = ACTIONS(2147),
    [sym_bnot] = ACTIONS(2147),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2155),
    [anon_sym_TILDE] = ACTIONS(2155),
    [anon_sym_DASH] = ACTIONS(2147),
    [anon_sym_PLUS] = ACTIONS(2147),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DQUOTE] = ACTIONS(2621),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(868)] = {
    [sym_expression] = STATE(1282),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(869)] = {
    [sym_expression] = STATE(1288),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(870)] = {
    [sym_expression] = STATE(1183),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(871)] = {
    [sym_expression] = STATE(589),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(872)] = {
    [sym_expression] = STATE(1199),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(873)] = {
    [sym_expression] = STATE(107),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(874)] = {
    [sym_expression] = STATE(1292),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(875)] = {
    [sym_expression] = STATE(1187),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(876)] = {
    [sym_expression] = STATE(1188),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(877)] = {
    [sym_expression] = STATE(1248),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(878)] = {
    [sym_expression] = STATE(1189),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(879)] = {
    [sym_expression] = STATE(1190),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(880)] = {
    [sym_expression] = STATE(1191),
    [sym_compiler_variable] = STATE(1262),
    [sym_system_variable] = STATE(1162),
    [sym_system_constant] = STATE(1262),
    [sym_system_function] = STATE(1989),
    [sym_assignment_expression] = STATE(1262),
    [sym_unary_expression] = STATE(1262),
    [sym_binary_expression] = STATE(1262),
    [sym_update_expression] = STATE(1262),
    [sym_subscript_expression] = STATE(1163),
    [sym_call_expression] = STATE(1163),
    [sym_field_expression] = STATE(1163),
    [sym_parenthesized_expression] = STATE(1163),
    [sym_devchan_expression] = STATE(1138),
    [sym_devchan_range_expression] = STATE(1262),
    [sym_string_expression] = STATE(1262),
    [sym_device_literal] = STATE(1262),
    [sym_string_literal] = STATE(1262),
    [sym_identifier] = ACTIONS(2143),
    [sym_not] = ACTIONS(129),
    [sym_bnot] = ACTIONS(129),
    [sym___file__] = ACTIONS(131),
    [sym___date__] = ACTIONS(131),
    [sym___ldate__] = ACTIONS(131),
    [sym___time__] = ACTIONS(131),
    [sym___line__] = ACTIONS(131),
    [sym___name__] = ACTIONS(131),
    [sym___netlinx__] = ACTIONS(133),
    [sym_netlinx_axi_version] = ACTIONS(133),
    [sym_url_flg_tcp] = ACTIONS(133),
    [sym_url_flg_accinfopresent] = ACTIONS(133),
    [sym_url_flg_temp] = ACTIONS(133),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(133),
    [sym_url_flg_stat_mask] = ACTIONS(133),
    [sym_url_flg_stat_lookup] = ACTIONS(133),
    [sym_url_flg_stat_connecting] = ACTIONS(133),
    [sym_url_flg_stat_waiting] = ACTIONS(133),
    [sym_url_flg_stat_connected] = ACTIONS(133),
    [sym_ip_addr_flg_dhcp] = ACTIONS(133),
    [sym_first_virtual_device] = ACTIONS(133),
    [sym_first_local_port] = ACTIONS(133),
    [sym_dynamic_virtual_device] = ACTIONS(133),
    [sym_timeline_once] = ACTIONS(133),
    [sym_timeline_repeat] = ACTIONS(133),
    [sym_timeline_absolute] = ACTIONS(133),
    [sym_timeline_relative] = ACTIONS(133),
    [sym_do_push_timed_infinite] = ACTIONS(133),
    [sym_source_type_no_address] = ACTIONS(133),
    [sym_source_type_neuron_id] = ACTIONS(133),
    [sym_source_type_ip_address] = ACTIONS(133),
    [sym_source_type_axlink] = ACTIONS(133),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(133),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(133),
    [sym_source_type_ip_socket_address] = ACTIONS(133),
    [sym_source_type_rs232] = ACTIONS(133),
    [sym_source_type_internal] = ACTIONS(133),
    [sym_source_type_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(133),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(133),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(133),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(133),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(133),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(133),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(133),
    [sym_true] = ACTIONS(133),
    [sym_false] = ACTIONS(133),
    [sym_file_read_only] = ACTIONS(133),
    [sym_file_rw_new] = ACTIONS(133),
    [sym_file_rw_append] = ACTIONS(133),
    [sym_ip_tcp] = ACTIONS(133),
    [sym_ip_udp] = ACTIONS(133),
    [sym_ip_udp_2way] = ACTIONS(133),
    [sym_xml_encode_types] = ACTIONS(133),
    [sym_xml_encode_char_as_list] = ACTIONS(133),
    [sym_xml_encode_le] = ACTIONS(133),
    [sym_xml_decode_types] = ACTIONS(133),
    [sym_xml_decode_no_preserve] = ACTIONS(133),
    [sym_duet_dev_type_display_device] = ACTIONS(133),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(133),
    [sym_duet_dev_type_io_device] = ACTIONS(133),
    [sym_duet_dev_type_relay_device] = ACTIONS(133),
    [sym_duet_dev_type_ups] = ACTIONS(133),
    [sym_duet_dev_type_amplifier] = ACTIONS(133),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(133),
    [sym_duet_dev_type_audio_processor] = ACTIONS(133),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(133),
    [sym_duet_dev_type_camera] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(133),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(133),
    [sym_duet_dev_type_dss] = ACTIONS(133),
    [sym_duet_dev_type_dvr] = ACTIONS(133),
    [sym_duet_dev_type_disc_device] = ACTIONS(133),
    [sym_duet_dev_type_document_camera] = ACTIONS(133),
    [sym_duet_dev_type_audio_tape] = ACTIONS(133),
    [sym_duet_dev_type_hvac] = ACTIONS(133),
    [sym_duet_dev_type_keypad] = ACTIONS(133),
    [sym_duet_dev_type_light] = ACTIONS(133),
    [sym_duet_dev_type_monitor] = ACTIONS(133),
    [sym_duet_dev_type_motor] = ACTIONS(133),
    [sym_duet_dev_type_multi_window] = ACTIONS(133),
    [sym_duet_dev_type_pool_spa] = ACTIONS(133),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(133),
    [sym_duet_dev_type_receiver] = ACTIONS(133),
    [sym_duet_dev_type_security_system] = ACTIONS(133),
    [sym_duet_dev_type_sensor_device] = ACTIONS(133),
    [sym_duet_dev_type_settop_box] = ACTIONS(133),
    [sym_duet_dev_type_slide_projector] = ACTIONS(133),
    [sym_duet_dev_type_switcher] = ACTIONS(133),
    [sym_duet_dev_type_text_keypad] = ACTIONS(133),
    [sym_duet_dev_type_tv] = ACTIONS(133),
    [sym_duet_dev_type_utility] = ACTIONS(133),
    [sym_duet_dev_type_vcr] = ACTIONS(133),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(133),
    [sym_duet_dev_type_video_processor] = ACTIONS(133),
    [sym_duet_dev_type_video_projector] = ACTIONS(133),
    [sym_duet_dev_type_video_wall] = ACTIONS(133),
    [sym_duet_dev_type_volume_controller] = ACTIONS(133),
    [sym_duet_dev_type_weather] = ACTIONS(133),
    [sym_duet_dev_not_polled] = ACTIONS(133),
    [sym_duet_dev_polled] = ACTIONS(133),
    [sym_clkmgr_mode_network] = ACTIONS(133),
    [sym_clkmgr_mode_standalone] = ACTIONS(133),
    [sym_normal_standby] = ACTIONS(133),
    [sym_normal_wake] = ACTIONS(133),
    [sym_ip_multicast_ttl_option] = ACTIONS(133),
    [sym_ip_ttl_subnet] = ACTIONS(133),
    [sym_ip_ttl_site] = ACTIONS(133),
    [sym_ip_ttl_region] = ACTIONS(133),
    [sym_ip_ttl_continent] = ACTIONS(133),
    [sym_ip_tcp_nodelay_option] = ACTIONS(133),
    [sym_ip_nodelay_on] = ACTIONS(133),
    [sym_ip_nodelay_off] = ACTIONS(133),
    [sym_amx_error] = ACTIONS(133),
    [sym_amx_warning] = ACTIONS(133),
    [sym_amx_info] = ACTIONS(133),
    [sym_amx_debug] = ACTIONS(133),
    [sym_netlinx_login_success] = ACTIONS(133),
    [sym_netlinx_login_fail] = ACTIONS(133),
    [sym_netlinx_logout] = ACTIONS(133),
    [sym_null_str] = ACTIONS(133),
    [sym_smtp_address] = ACTIONS(133),
    [sym_smtp_port_number] = ACTIONS(133),
    [sym_smtp_username] = ACTIONS(133),
    [sym_smtp_password] = ACTIONS(133),
    [sym_smtp_require_tls] = ACTIONS(133),
    [sym_smtp_from] = ACTIONS(133),
    [sym_smtp_tls_true] = ACTIONS(133),
    [sym_smtp_tls_false] = ACTIONS(133),
    [sym_listview_on_row_select_event] = ACTIONS(133),
    [sym_data_structure_unknown] = ACTIONS(133),
    [sym_data_structure_datafeed] = ACTIONS(133),
    [sym_data_structure_datarecordset] = ACTIONS(133),
    [sym_data_structure_datarecord] = ACTIONS(133),
    [sym_data_structure_datafield] = ACTIONS(133),
    [sym_data_max_content_field_count] = ACTIONS(133),
    [sym_data_max_metadata_field_count] = ACTIONS(133),
    [sym_data_max_id_length] = ACTIONS(133),
    [sym_data_max_name_length] = ACTIONS(133),
    [sym_data_max_description_length] = ACTIONS(133),
    [sym_data_max_type_length] = ACTIONS(133),
    [sym_data_max_format_length] = ACTIONS(133),
    [sym_data_max_source_length] = ACTIONS(133),
    [sym_data_max_value_length] = ACTIONS(133),
    [sym_data_max_label_length] = ACTIONS(133),
    [sym_data_max_url_length] = ACTIONS(133),
    [sym_max_last_login_info_length] = ACTIONS(133),
    [sym_data_type_unknown] = ACTIONS(133),
    [sym_data_type_string] = ACTIONS(133),
    [sym_data_type_datetime] = ACTIONS(133),
    [sym_data_type_date] = ACTIONS(133),
    [sym_data_type_time] = ACTIONS(133),
    [sym_data_type_image] = ACTIONS(133),
    [sym_data_format_url] = ACTIONS(133),
    [sym_data_format_phone] = ACTIONS(133),
    [sym_data_format_email] = ACTIONS(133),
    [sym_data_format_iso8601] = ACTIONS(133),
    [sym_auditlog_priv] = ACTIONS(133),
    [sym_configuration_priv] = ACTIONS(133),
    [sym_deviceconfig_priv] = ACTIONS(133),
    [sym_ftp_priv] = ACTIONS(133),
    [sym_http_priv] = ACTIONS(133),
    [sym_networkconfig_priv] = ACTIONS(133),
    [sym_programport_priv] = ACTIONS(133),
    [sym_remoteui_priv] = ACTIONS(133),
    [sym_securitycontrol_priv] = ACTIONS(133),
    [sym_softwaremanagement_priv] = ACTIONS(133),
    [sym_terminal_priv] = ACTIONS(133),
    [sym_tpadmin_priv] = ACTIONS(133),
    [sym_user1_priv] = ACTIONS(133),
    [sym_user2_priv] = ACTIONS(133),
    [sym_user3_priv] = ACTIONS(133),
    [sym_user4_priv] = ACTIONS(133),
    [sym_usermanagement_priv] = ACTIONS(133),
    [sym_valid_account] = ACTIONS(133),
    [sym_err_name_invalid] = ACTIONS(133),
    [sym_err_password_invalid] = ACTIONS(133),
    [sym_err_invalid_account] = ACTIONS(133),
    [sym_err_no_authorization] = ACTIONS(133),
    [sym_err_invalid_parameter] = ACTIONS(133),
    [sym_err_user_locked_out] = ACTIONS(133),
    [sym_err_user_account_expired] = ACTIONS(133),
    [sym_err_general_error] = ACTIONS(133),
    [sym_err_authentication_error] = ACTIONS(133),
    [sym_tls_ignore_certificate_errors] = ACTIONS(133),
    [sym_tls_validate_certificate] = ACTIONS(133),
    [sym____reserved___] = ACTIONS(137),
    [sym_date] = ACTIONS(137),
    [sym_day] = ACTIONS(137),
    [sym_time] = ACTIONS(137),
    [sym_ldate] = ACTIONS(137),
    [sym_push_device] = ACTIONS(137),
    [sym_push_channel] = ACTIONS(137),
    [sym_push_devchan] = ACTIONS(137),
    [sym_release_device] = ACTIONS(137),
    [sym_release_channel] = ACTIONS(137),
    [sym_release_devchan] = ACTIONS(137),
    [sym_master_slot] = ACTIONS(137),
    [sym_get_pulse_time] = ACTIONS(137),
    [sym_get_timer] = ACTIONS(137),
    [sym_master_sn] = ACTIONS(137),
    [sym_system_number] = ACTIONS(137),
    [sym_button] = ACTIONS(137),
    [sym_dv_channel] = ACTIONS(137),
    [sym_data] = ACTIONS(137),
    [sym_level] = ACTIONS(137),
    [sym_timeline] = ACTIONS(137),
    [sym_channel] = ACTIONS(137),
    [sym_custom] = ACTIONS(137),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(153),
    [anon_sym_LPAREN] = ACTIONS(157),
    [anon_sym_BANG] = ACTIONS(159),
    [anon_sym_TILDE] = ACTIONS(159),
    [anon_sym_DASH] = ACTIONS(129),
    [anon_sym_PLUS] = ACTIONS(129),
    [anon_sym_DASH_DASH] = ACTIONS(161),
    [anon_sym_PLUS_PLUS] = ACTIONS(161),
    [anon_sym_DQUOTE] = ACTIONS(163),
    [anon_sym_SQUOTE] = ACTIONS(165),
    [sym_number_literal] = ACTIONS(167),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(881)] = {
    [sym_expression] = STATE(104),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(882)] = {
    [sym_expression] = STATE(1297),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(883)] = {
    [sym_expression] = STATE(1251),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(884)] = {
    [sym_expression] = STATE(81),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(885)] = {
    [sym_expression] = STATE(1296),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(886)] = {
    [sym_expression] = STATE(1253),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(887)] = {
    [sym_expression] = STATE(578),
    [sym_compiler_variable] = STATE(618),
    [sym_system_variable] = STATE(599),
    [sym_system_constant] = STATE(618),
    [sym_system_function] = STATE(1923),
    [sym_assignment_expression] = STATE(618),
    [sym_unary_expression] = STATE(618),
    [sym_binary_expression] = STATE(618),
    [sym_update_expression] = STATE(618),
    [sym_subscript_expression] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_field_expression] = STATE(600),
    [sym_parenthesized_expression] = STATE(600),
    [sym_devchan_expression] = STATE(580),
    [sym_devchan_range_expression] = STATE(618),
    [sym_string_expression] = STATE(618),
    [sym_device_literal] = STATE(618),
    [sym_string_literal] = STATE(618),
    [sym_identifier] = ACTIONS(2283),
    [sym_not] = ACTIONS(2285),
    [sym_bnot] = ACTIONS(2285),
    [sym___file__] = ACTIONS(2287),
    [sym___date__] = ACTIONS(2287),
    [sym___ldate__] = ACTIONS(2287),
    [sym___time__] = ACTIONS(2287),
    [sym___line__] = ACTIONS(2287),
    [sym___name__] = ACTIONS(2287),
    [sym___netlinx__] = ACTIONS(2289),
    [sym_netlinx_axi_version] = ACTIONS(2289),
    [sym_url_flg_tcp] = ACTIONS(2289),
    [sym_url_flg_accinfopresent] = ACTIONS(2289),
    [sym_url_flg_temp] = ACTIONS(2289),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2289),
    [sym_url_flg_stat_mask] = ACTIONS(2289),
    [sym_url_flg_stat_lookup] = ACTIONS(2289),
    [sym_url_flg_stat_connecting] = ACTIONS(2289),
    [sym_url_flg_stat_waiting] = ACTIONS(2289),
    [sym_url_flg_stat_connected] = ACTIONS(2289),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2289),
    [sym_first_virtual_device] = ACTIONS(2289),
    [sym_first_local_port] = ACTIONS(2289),
    [sym_dynamic_virtual_device] = ACTIONS(2289),
    [sym_timeline_once] = ACTIONS(2289),
    [sym_timeline_repeat] = ACTIONS(2289),
    [sym_timeline_absolute] = ACTIONS(2289),
    [sym_timeline_relative] = ACTIONS(2289),
    [sym_do_push_timed_infinite] = ACTIONS(2289),
    [sym_source_type_no_address] = ACTIONS(2289),
    [sym_source_type_neuron_id] = ACTIONS(2289),
    [sym_source_type_ip_address] = ACTIONS(2289),
    [sym_source_type_axlink] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2289),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2289),
    [sym_source_type_ip_socket_address] = ACTIONS(2289),
    [sym_source_type_rs232] = ACTIONS(2289),
    [sym_source_type_internal] = ACTIONS(2289),
    [sym_source_type_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2289),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2289),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2289),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2289),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2289),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2289),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2289),
    [sym_true] = ACTIONS(2289),
    [sym_false] = ACTIONS(2289),
    [sym_file_read_only] = ACTIONS(2289),
    [sym_file_rw_new] = ACTIONS(2289),
    [sym_file_rw_append] = ACTIONS(2289),
    [sym_ip_tcp] = ACTIONS(2289),
    [sym_ip_udp] = ACTIONS(2289),
    [sym_ip_udp_2way] = ACTIONS(2289),
    [sym_xml_encode_types] = ACTIONS(2289),
    [sym_xml_encode_char_as_list] = ACTIONS(2289),
    [sym_xml_encode_le] = ACTIONS(2289),
    [sym_xml_decode_types] = ACTIONS(2289),
    [sym_xml_decode_no_preserve] = ACTIONS(2289),
    [sym_duet_dev_type_display_device] = ACTIONS(2289),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2289),
    [sym_duet_dev_type_io_device] = ACTIONS(2289),
    [sym_duet_dev_type_relay_device] = ACTIONS(2289),
    [sym_duet_dev_type_ups] = ACTIONS(2289),
    [sym_duet_dev_type_amplifier] = ACTIONS(2289),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2289),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2289),
    [sym_duet_dev_type_camera] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2289),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2289),
    [sym_duet_dev_type_dss] = ACTIONS(2289),
    [sym_duet_dev_type_dvr] = ACTIONS(2289),
    [sym_duet_dev_type_disc_device] = ACTIONS(2289),
    [sym_duet_dev_type_document_camera] = ACTIONS(2289),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2289),
    [sym_duet_dev_type_hvac] = ACTIONS(2289),
    [sym_duet_dev_type_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_light] = ACTIONS(2289),
    [sym_duet_dev_type_monitor] = ACTIONS(2289),
    [sym_duet_dev_type_motor] = ACTIONS(2289),
    [sym_duet_dev_type_multi_window] = ACTIONS(2289),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2289),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2289),
    [sym_duet_dev_type_receiver] = ACTIONS(2289),
    [sym_duet_dev_type_security_system] = ACTIONS(2289),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2289),
    [sym_duet_dev_type_settop_box] = ACTIONS(2289),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2289),
    [sym_duet_dev_type_switcher] = ACTIONS(2289),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2289),
    [sym_duet_dev_type_tv] = ACTIONS(2289),
    [sym_duet_dev_type_utility] = ACTIONS(2289),
    [sym_duet_dev_type_vcr] = ACTIONS(2289),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2289),
    [sym_duet_dev_type_video_processor] = ACTIONS(2289),
    [sym_duet_dev_type_video_projector] = ACTIONS(2289),
    [sym_duet_dev_type_video_wall] = ACTIONS(2289),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2289),
    [sym_duet_dev_type_weather] = ACTIONS(2289),
    [sym_duet_dev_not_polled] = ACTIONS(2289),
    [sym_duet_dev_polled] = ACTIONS(2289),
    [sym_clkmgr_mode_network] = ACTIONS(2289),
    [sym_clkmgr_mode_standalone] = ACTIONS(2289),
    [sym_normal_standby] = ACTIONS(2289),
    [sym_normal_wake] = ACTIONS(2289),
    [sym_ip_multicast_ttl_option] = ACTIONS(2289),
    [sym_ip_ttl_subnet] = ACTIONS(2289),
    [sym_ip_ttl_site] = ACTIONS(2289),
    [sym_ip_ttl_region] = ACTIONS(2289),
    [sym_ip_ttl_continent] = ACTIONS(2289),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2289),
    [sym_ip_nodelay_on] = ACTIONS(2289),
    [sym_ip_nodelay_off] = ACTIONS(2289),
    [sym_amx_error] = ACTIONS(2289),
    [sym_amx_warning] = ACTIONS(2289),
    [sym_amx_info] = ACTIONS(2289),
    [sym_amx_debug] = ACTIONS(2289),
    [sym_netlinx_login_success] = ACTIONS(2289),
    [sym_netlinx_login_fail] = ACTIONS(2289),
    [sym_netlinx_logout] = ACTIONS(2289),
    [sym_null_str] = ACTIONS(2289),
    [sym_smtp_address] = ACTIONS(2289),
    [sym_smtp_port_number] = ACTIONS(2289),
    [sym_smtp_username] = ACTIONS(2289),
    [sym_smtp_password] = ACTIONS(2289),
    [sym_smtp_require_tls] = ACTIONS(2289),
    [sym_smtp_from] = ACTIONS(2289),
    [sym_smtp_tls_true] = ACTIONS(2289),
    [sym_smtp_tls_false] = ACTIONS(2289),
    [sym_listview_on_row_select_event] = ACTIONS(2289),
    [sym_data_structure_unknown] = ACTIONS(2289),
    [sym_data_structure_datafeed] = ACTIONS(2289),
    [sym_data_structure_datarecordset] = ACTIONS(2289),
    [sym_data_structure_datarecord] = ACTIONS(2289),
    [sym_data_structure_datafield] = ACTIONS(2289),
    [sym_data_max_content_field_count] = ACTIONS(2289),
    [sym_data_max_metadata_field_count] = ACTIONS(2289),
    [sym_data_max_id_length] = ACTIONS(2289),
    [sym_data_max_name_length] = ACTIONS(2289),
    [sym_data_max_description_length] = ACTIONS(2289),
    [sym_data_max_type_length] = ACTIONS(2289),
    [sym_data_max_format_length] = ACTIONS(2289),
    [sym_data_max_source_length] = ACTIONS(2289),
    [sym_data_max_value_length] = ACTIONS(2289),
    [sym_data_max_label_length] = ACTIONS(2289),
    [sym_data_max_url_length] = ACTIONS(2289),
    [sym_max_last_login_info_length] = ACTIONS(2289),
    [sym_data_type_unknown] = ACTIONS(2289),
    [sym_data_type_string] = ACTIONS(2289),
    [sym_data_type_datetime] = ACTIONS(2289),
    [sym_data_type_date] = ACTIONS(2289),
    [sym_data_type_time] = ACTIONS(2289),
    [sym_data_type_image] = ACTIONS(2289),
    [sym_data_format_url] = ACTIONS(2289),
    [sym_data_format_phone] = ACTIONS(2289),
    [sym_data_format_email] = ACTIONS(2289),
    [sym_data_format_iso8601] = ACTIONS(2289),
    [sym_auditlog_priv] = ACTIONS(2289),
    [sym_configuration_priv] = ACTIONS(2289),
    [sym_deviceconfig_priv] = ACTIONS(2289),
    [sym_ftp_priv] = ACTIONS(2289),
    [sym_http_priv] = ACTIONS(2289),
    [sym_networkconfig_priv] = ACTIONS(2289),
    [sym_programport_priv] = ACTIONS(2289),
    [sym_remoteui_priv] = ACTIONS(2289),
    [sym_securitycontrol_priv] = ACTIONS(2289),
    [sym_softwaremanagement_priv] = ACTIONS(2289),
    [sym_terminal_priv] = ACTIONS(2289),
    [sym_tpadmin_priv] = ACTIONS(2289),
    [sym_user1_priv] = ACTIONS(2289),
    [sym_user2_priv] = ACTIONS(2289),
    [sym_user3_priv] = ACTIONS(2289),
    [sym_user4_priv] = ACTIONS(2289),
    [sym_usermanagement_priv] = ACTIONS(2289),
    [sym_valid_account] = ACTIONS(2289),
    [sym_err_name_invalid] = ACTIONS(2289),
    [sym_err_password_invalid] = ACTIONS(2289),
    [sym_err_invalid_account] = ACTIONS(2289),
    [sym_err_no_authorization] = ACTIONS(2289),
    [sym_err_invalid_parameter] = ACTIONS(2289),
    [sym_err_user_locked_out] = ACTIONS(2289),
    [sym_err_user_account_expired] = ACTIONS(2289),
    [sym_err_general_error] = ACTIONS(2289),
    [sym_err_authentication_error] = ACTIONS(2289),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2289),
    [sym_tls_validate_certificate] = ACTIONS(2289),
    [sym____reserved___] = ACTIONS(2291),
    [sym_date] = ACTIONS(2291),
    [sym_day] = ACTIONS(2291),
    [sym_time] = ACTIONS(2291),
    [sym_ldate] = ACTIONS(2291),
    [sym_push_device] = ACTIONS(2291),
    [sym_push_channel] = ACTIONS(2291),
    [sym_push_devchan] = ACTIONS(2291),
    [sym_release_device] = ACTIONS(2291),
    [sym_release_channel] = ACTIONS(2291),
    [sym_release_devchan] = ACTIONS(2291),
    [sym_master_slot] = ACTIONS(2291),
    [sym_get_pulse_time] = ACTIONS(2291),
    [sym_get_timer] = ACTIONS(2291),
    [sym_master_sn] = ACTIONS(2291),
    [sym_system_number] = ACTIONS(2291),
    [sym_button] = ACTIONS(2291),
    [sym_dv_channel] = ACTIONS(2291),
    [sym_data] = ACTIONS(2291),
    [sym_level] = ACTIONS(2291),
    [sym_timeline] = ACTIONS(2291),
    [sym_channel] = ACTIONS(2291),
    [sym_custom] = ACTIONS(2291),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2297),
    [anon_sym_LPAREN] = ACTIONS(2303),
    [anon_sym_BANG] = ACTIONS(2305),
    [anon_sym_TILDE] = ACTIONS(2305),
    [anon_sym_DASH] = ACTIONS(2285),
    [anon_sym_PLUS] = ACTIONS(2285),
    [anon_sym_DASH_DASH] = ACTIONS(2307),
    [anon_sym_PLUS_PLUS] = ACTIONS(2307),
    [anon_sym_DQUOTE] = ACTIONS(2309),
    [anon_sym_SQUOTE] = ACTIONS(2311),
    [sym_number_literal] = ACTIONS(2313),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(888)] = {
    [sym_expression] = STATE(1286),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(889)] = {
    [sym_expression] = STATE(1258),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(890)] = {
    [sym_expression] = STATE(82),
    [sym_compiler_variable] = STATE(138),
    [sym_system_variable] = STATE(115),
    [sym_system_constant] = STATE(138),
    [sym_system_function] = STATE(2025),
    [sym_assignment_expression] = STATE(138),
    [sym_unary_expression] = STATE(138),
    [sym_binary_expression] = STATE(138),
    [sym_update_expression] = STATE(138),
    [sym_subscript_expression] = STATE(116),
    [sym_call_expression] = STATE(116),
    [sym_field_expression] = STATE(116),
    [sym_parenthesized_expression] = STATE(116),
    [sym_devchan_expression] = STATE(90),
    [sym_devchan_range_expression] = STATE(138),
    [sym_string_expression] = STATE(138),
    [sym_device_literal] = STATE(138),
    [sym_string_literal] = STATE(138),
    [sym_identifier] = ACTIONS(2683),
    [sym_not] = ACTIONS(2685),
    [sym_bnot] = ACTIONS(2685),
    [sym___file__] = ACTIONS(2687),
    [sym___date__] = ACTIONS(2687),
    [sym___ldate__] = ACTIONS(2687),
    [sym___time__] = ACTIONS(2687),
    [sym___line__] = ACTIONS(2687),
    [sym___name__] = ACTIONS(2687),
    [sym___netlinx__] = ACTIONS(2689),
    [sym_netlinx_axi_version] = ACTIONS(2689),
    [sym_url_flg_tcp] = ACTIONS(2689),
    [sym_url_flg_accinfopresent] = ACTIONS(2689),
    [sym_url_flg_temp] = ACTIONS(2689),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2689),
    [sym_url_flg_stat_mask] = ACTIONS(2689),
    [sym_url_flg_stat_lookup] = ACTIONS(2689),
    [sym_url_flg_stat_connecting] = ACTIONS(2689),
    [sym_url_flg_stat_waiting] = ACTIONS(2689),
    [sym_url_flg_stat_connected] = ACTIONS(2689),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2689),
    [sym_first_virtual_device] = ACTIONS(2689),
    [sym_first_local_port] = ACTIONS(2689),
    [sym_dynamic_virtual_device] = ACTIONS(2689),
    [sym_timeline_once] = ACTIONS(2689),
    [sym_timeline_repeat] = ACTIONS(2689),
    [sym_timeline_absolute] = ACTIONS(2689),
    [sym_timeline_relative] = ACTIONS(2689),
    [sym_do_push_timed_infinite] = ACTIONS(2689),
    [sym_source_type_no_address] = ACTIONS(2689),
    [sym_source_type_neuron_id] = ACTIONS(2689),
    [sym_source_type_ip_address] = ACTIONS(2689),
    [sym_source_type_axlink] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2689),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2689),
    [sym_source_type_ip_socket_address] = ACTIONS(2689),
    [sym_source_type_rs232] = ACTIONS(2689),
    [sym_source_type_internal] = ACTIONS(2689),
    [sym_source_type_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2689),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2689),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2689),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2689),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2689),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2689),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2689),
    [sym_true] = ACTIONS(2689),
    [sym_false] = ACTIONS(2689),
    [sym_file_read_only] = ACTIONS(2689),
    [sym_file_rw_new] = ACTIONS(2689),
    [sym_file_rw_append] = ACTIONS(2689),
    [sym_ip_tcp] = ACTIONS(2689),
    [sym_ip_udp] = ACTIONS(2689),
    [sym_ip_udp_2way] = ACTIONS(2689),
    [sym_xml_encode_types] = ACTIONS(2689),
    [sym_xml_encode_char_as_list] = ACTIONS(2689),
    [sym_xml_encode_le] = ACTIONS(2689),
    [sym_xml_decode_types] = ACTIONS(2689),
    [sym_xml_decode_no_preserve] = ACTIONS(2689),
    [sym_duet_dev_type_display_device] = ACTIONS(2689),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2689),
    [sym_duet_dev_type_io_device] = ACTIONS(2689),
    [sym_duet_dev_type_relay_device] = ACTIONS(2689),
    [sym_duet_dev_type_ups] = ACTIONS(2689),
    [sym_duet_dev_type_amplifier] = ACTIONS(2689),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2689),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2689),
    [sym_duet_dev_type_camera] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2689),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2689),
    [sym_duet_dev_type_dss] = ACTIONS(2689),
    [sym_duet_dev_type_dvr] = ACTIONS(2689),
    [sym_duet_dev_type_disc_device] = ACTIONS(2689),
    [sym_duet_dev_type_document_camera] = ACTIONS(2689),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2689),
    [sym_duet_dev_type_hvac] = ACTIONS(2689),
    [sym_duet_dev_type_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_light] = ACTIONS(2689),
    [sym_duet_dev_type_monitor] = ACTIONS(2689),
    [sym_duet_dev_type_motor] = ACTIONS(2689),
    [sym_duet_dev_type_multi_window] = ACTIONS(2689),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2689),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2689),
    [sym_duet_dev_type_receiver] = ACTIONS(2689),
    [sym_duet_dev_type_security_system] = ACTIONS(2689),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2689),
    [sym_duet_dev_type_settop_box] = ACTIONS(2689),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2689),
    [sym_duet_dev_type_switcher] = ACTIONS(2689),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2689),
    [sym_duet_dev_type_tv] = ACTIONS(2689),
    [sym_duet_dev_type_utility] = ACTIONS(2689),
    [sym_duet_dev_type_vcr] = ACTIONS(2689),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2689),
    [sym_duet_dev_type_video_processor] = ACTIONS(2689),
    [sym_duet_dev_type_video_projector] = ACTIONS(2689),
    [sym_duet_dev_type_video_wall] = ACTIONS(2689),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2689),
    [sym_duet_dev_type_weather] = ACTIONS(2689),
    [sym_duet_dev_not_polled] = ACTIONS(2689),
    [sym_duet_dev_polled] = ACTIONS(2689),
    [sym_clkmgr_mode_network] = ACTIONS(2689),
    [sym_clkmgr_mode_standalone] = ACTIONS(2689),
    [sym_normal_standby] = ACTIONS(2689),
    [sym_normal_wake] = ACTIONS(2689),
    [sym_ip_multicast_ttl_option] = ACTIONS(2689),
    [sym_ip_ttl_subnet] = ACTIONS(2689),
    [sym_ip_ttl_site] = ACTIONS(2689),
    [sym_ip_ttl_region] = ACTIONS(2689),
    [sym_ip_ttl_continent] = ACTIONS(2689),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2689),
    [sym_ip_nodelay_on] = ACTIONS(2689),
    [sym_ip_nodelay_off] = ACTIONS(2689),
    [sym_amx_error] = ACTIONS(2689),
    [sym_amx_warning] = ACTIONS(2689),
    [sym_amx_info] = ACTIONS(2689),
    [sym_amx_debug] = ACTIONS(2689),
    [sym_netlinx_login_success] = ACTIONS(2689),
    [sym_netlinx_login_fail] = ACTIONS(2689),
    [sym_netlinx_logout] = ACTIONS(2689),
    [sym_null_str] = ACTIONS(2689),
    [sym_smtp_address] = ACTIONS(2689),
    [sym_smtp_port_number] = ACTIONS(2689),
    [sym_smtp_username] = ACTIONS(2689),
    [sym_smtp_password] = ACTIONS(2689),
    [sym_smtp_require_tls] = ACTIONS(2689),
    [sym_smtp_from] = ACTIONS(2689),
    [sym_smtp_tls_true] = ACTIONS(2689),
    [sym_smtp_tls_false] = ACTIONS(2689),
    [sym_listview_on_row_select_event] = ACTIONS(2689),
    [sym_data_structure_unknown] = ACTIONS(2689),
    [sym_data_structure_datafeed] = ACTIONS(2689),
    [sym_data_structure_datarecordset] = ACTIONS(2689),
    [sym_data_structure_datarecord] = ACTIONS(2689),
    [sym_data_structure_datafield] = ACTIONS(2689),
    [sym_data_max_content_field_count] = ACTIONS(2689),
    [sym_data_max_metadata_field_count] = ACTIONS(2689),
    [sym_data_max_id_length] = ACTIONS(2689),
    [sym_data_max_name_length] = ACTIONS(2689),
    [sym_data_max_description_length] = ACTIONS(2689),
    [sym_data_max_type_length] = ACTIONS(2689),
    [sym_data_max_format_length] = ACTIONS(2689),
    [sym_data_max_source_length] = ACTIONS(2689),
    [sym_data_max_value_length] = ACTIONS(2689),
    [sym_data_max_label_length] = ACTIONS(2689),
    [sym_data_max_url_length] = ACTIONS(2689),
    [sym_max_last_login_info_length] = ACTIONS(2689),
    [sym_data_type_unknown] = ACTIONS(2689),
    [sym_data_type_string] = ACTIONS(2689),
    [sym_data_type_datetime] = ACTIONS(2689),
    [sym_data_type_date] = ACTIONS(2689),
    [sym_data_type_time] = ACTIONS(2689),
    [sym_data_type_image] = ACTIONS(2689),
    [sym_data_format_url] = ACTIONS(2689),
    [sym_data_format_phone] = ACTIONS(2689),
    [sym_data_format_email] = ACTIONS(2689),
    [sym_data_format_iso8601] = ACTIONS(2689),
    [sym_auditlog_priv] = ACTIONS(2689),
    [sym_configuration_priv] = ACTIONS(2689),
    [sym_deviceconfig_priv] = ACTIONS(2689),
    [sym_ftp_priv] = ACTIONS(2689),
    [sym_http_priv] = ACTIONS(2689),
    [sym_networkconfig_priv] = ACTIONS(2689),
    [sym_programport_priv] = ACTIONS(2689),
    [sym_remoteui_priv] = ACTIONS(2689),
    [sym_securitycontrol_priv] = ACTIONS(2689),
    [sym_softwaremanagement_priv] = ACTIONS(2689),
    [sym_terminal_priv] = ACTIONS(2689),
    [sym_tpadmin_priv] = ACTIONS(2689),
    [sym_user1_priv] = ACTIONS(2689),
    [sym_user2_priv] = ACTIONS(2689),
    [sym_user3_priv] = ACTIONS(2689),
    [sym_user4_priv] = ACTIONS(2689),
    [sym_usermanagement_priv] = ACTIONS(2689),
    [sym_valid_account] = ACTIONS(2689),
    [sym_err_name_invalid] = ACTIONS(2689),
    [sym_err_password_invalid] = ACTIONS(2689),
    [sym_err_invalid_account] = ACTIONS(2689),
    [sym_err_no_authorization] = ACTIONS(2689),
    [sym_err_invalid_parameter] = ACTIONS(2689),
    [sym_err_user_locked_out] = ACTIONS(2689),
    [sym_err_user_account_expired] = ACTIONS(2689),
    [sym_err_general_error] = ACTIONS(2689),
    [sym_err_authentication_error] = ACTIONS(2689),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2689),
    [sym_tls_validate_certificate] = ACTIONS(2689),
    [sym____reserved___] = ACTIONS(2691),
    [sym_date] = ACTIONS(2691),
    [sym_day] = ACTIONS(2691),
    [sym_time] = ACTIONS(2691),
    [sym_ldate] = ACTIONS(2691),
    [sym_push_device] = ACTIONS(2691),
    [sym_push_channel] = ACTIONS(2691),
    [sym_push_devchan] = ACTIONS(2691),
    [sym_release_device] = ACTIONS(2691),
    [sym_release_channel] = ACTIONS(2691),
    [sym_release_devchan] = ACTIONS(2691),
    [sym_master_slot] = ACTIONS(2691),
    [sym_get_pulse_time] = ACTIONS(2691),
    [sym_get_timer] = ACTIONS(2691),
    [sym_master_sn] = ACTIONS(2691),
    [sym_system_number] = ACTIONS(2691),
    [sym_button] = ACTIONS(2691),
    [sym_dv_channel] = ACTIONS(2691),
    [sym_data] = ACTIONS(2691),
    [sym_level] = ACTIONS(2691),
    [sym_timeline] = ACTIONS(2691),
    [sym_channel] = ACTIONS(2691),
    [sym_custom] = ACTIONS(2691),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2693),
    [anon_sym_LPAREN] = ACTIONS(2697),
    [anon_sym_BANG] = ACTIONS(2699),
    [anon_sym_TILDE] = ACTIONS(2699),
    [anon_sym_DASH] = ACTIONS(2685),
    [anon_sym_PLUS] = ACTIONS(2685),
    [anon_sym_DASH_DASH] = ACTIONS(2701),
    [anon_sym_PLUS_PLUS] = ACTIONS(2701),
    [anon_sym_DQUOTE] = ACTIONS(2703),
    [anon_sym_SQUOTE] = ACTIONS(1461),
    [sym_number_literal] = ACTIONS(2705),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(891)] = {
    [sym_expression] = STATE(1290),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(892)] = {
    [sym_expression] = STATE(1264),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(893)] = {
    [sym_expression] = STATE(1265),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(894)] = {
    [sym_expression] = STATE(1266),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(895)] = {
    [sym_expression] = STATE(1267),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(896)] = {
    [sym_expression] = STATE(1271),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(897)] = {
    [sym_expression] = STATE(1223),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(898)] = {
    [sym_expression] = STATE(1273),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(899)] = {
    [sym_expression] = STATE(1274),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(900)] = {
    [sym_expression] = STATE(1275),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(901)] = {
    [sym_expression] = STATE(1276),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(902)] = {
    [sym_expression] = STATE(1277),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(903)] = {
    [sym_expression] = STATE(1280),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(904)] = {
    [sym_expression] = STATE(1227),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(905)] = {
    [sym_expression] = STATE(1202),
    [sym_compiler_variable] = STATE(1154),
    [sym_system_variable] = STATE(1129),
    [sym_system_constant] = STATE(1154),
    [sym_system_function] = STATE(1913),
    [sym_assignment_expression] = STATE(1154),
    [sym_unary_expression] = STATE(1154),
    [sym_binary_expression] = STATE(1154),
    [sym_update_expression] = STATE(1154),
    [sym_subscript_expression] = STATE(1102),
    [sym_call_expression] = STATE(1102),
    [sym_field_expression] = STATE(1102),
    [sym_parenthesized_expression] = STATE(1102),
    [sym_devchan_expression] = STATE(1090),
    [sym_devchan_range_expression] = STATE(1154),
    [sym_string_expression] = STATE(1154),
    [sym_device_literal] = STATE(1154),
    [sym_string_literal] = STATE(1154),
    [sym_identifier] = ACTIONS(2589),
    [sym_not] = ACTIONS(2591),
    [sym_bnot] = ACTIONS(2591),
    [sym___file__] = ACTIONS(2149),
    [sym___date__] = ACTIONS(2149),
    [sym___ldate__] = ACTIONS(2149),
    [sym___time__] = ACTIONS(2149),
    [sym___line__] = ACTIONS(2149),
    [sym___name__] = ACTIONS(2149),
    [sym___netlinx__] = ACTIONS(2151),
    [sym_netlinx_axi_version] = ACTIONS(2151),
    [sym_url_flg_tcp] = ACTIONS(2151),
    [sym_url_flg_accinfopresent] = ACTIONS(2151),
    [sym_url_flg_temp] = ACTIONS(2151),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2151),
    [sym_url_flg_stat_mask] = ACTIONS(2151),
    [sym_url_flg_stat_lookup] = ACTIONS(2151),
    [sym_url_flg_stat_connecting] = ACTIONS(2151),
    [sym_url_flg_stat_waiting] = ACTIONS(2151),
    [sym_url_flg_stat_connected] = ACTIONS(2151),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2151),
    [sym_first_virtual_device] = ACTIONS(2151),
    [sym_first_local_port] = ACTIONS(2151),
    [sym_dynamic_virtual_device] = ACTIONS(2151),
    [sym_timeline_once] = ACTIONS(2151),
    [sym_timeline_repeat] = ACTIONS(2151),
    [sym_timeline_absolute] = ACTIONS(2151),
    [sym_timeline_relative] = ACTIONS(2151),
    [sym_do_push_timed_infinite] = ACTIONS(2151),
    [sym_source_type_no_address] = ACTIONS(2151),
    [sym_source_type_neuron_id] = ACTIONS(2151),
    [sym_source_type_ip_address] = ACTIONS(2151),
    [sym_source_type_axlink] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2151),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2151),
    [sym_source_type_ip_socket_address] = ACTIONS(2151),
    [sym_source_type_rs232] = ACTIONS(2151),
    [sym_source_type_internal] = ACTIONS(2151),
    [sym_source_type_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2151),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2151),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2151),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2151),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2151),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2151),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2151),
    [sym_true] = ACTIONS(2151),
    [sym_false] = ACTIONS(2151),
    [sym_file_read_only] = ACTIONS(2151),
    [sym_file_rw_new] = ACTIONS(2151),
    [sym_file_rw_append] = ACTIONS(2151),
    [sym_ip_tcp] = ACTIONS(2151),
    [sym_ip_udp] = ACTIONS(2151),
    [sym_ip_udp_2way] = ACTIONS(2151),
    [sym_xml_encode_types] = ACTIONS(2151),
    [sym_xml_encode_char_as_list] = ACTIONS(2151),
    [sym_xml_encode_le] = ACTIONS(2151),
    [sym_xml_decode_types] = ACTIONS(2151),
    [sym_xml_decode_no_preserve] = ACTIONS(2151),
    [sym_duet_dev_type_display_device] = ACTIONS(2151),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2151),
    [sym_duet_dev_type_io_device] = ACTIONS(2151),
    [sym_duet_dev_type_relay_device] = ACTIONS(2151),
    [sym_duet_dev_type_ups] = ACTIONS(2151),
    [sym_duet_dev_type_amplifier] = ACTIONS(2151),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2151),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2151),
    [sym_duet_dev_type_camera] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2151),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2151),
    [sym_duet_dev_type_dss] = ACTIONS(2151),
    [sym_duet_dev_type_dvr] = ACTIONS(2151),
    [sym_duet_dev_type_disc_device] = ACTIONS(2151),
    [sym_duet_dev_type_document_camera] = ACTIONS(2151),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2151),
    [sym_duet_dev_type_hvac] = ACTIONS(2151),
    [sym_duet_dev_type_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_light] = ACTIONS(2151),
    [sym_duet_dev_type_monitor] = ACTIONS(2151),
    [sym_duet_dev_type_motor] = ACTIONS(2151),
    [sym_duet_dev_type_multi_window] = ACTIONS(2151),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2151),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2151),
    [sym_duet_dev_type_receiver] = ACTIONS(2151),
    [sym_duet_dev_type_security_system] = ACTIONS(2151),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2151),
    [sym_duet_dev_type_settop_box] = ACTIONS(2151),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2151),
    [sym_duet_dev_type_switcher] = ACTIONS(2151),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2151),
    [sym_duet_dev_type_tv] = ACTIONS(2151),
    [sym_duet_dev_type_utility] = ACTIONS(2151),
    [sym_duet_dev_type_vcr] = ACTIONS(2151),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2151),
    [sym_duet_dev_type_video_processor] = ACTIONS(2151),
    [sym_duet_dev_type_video_projector] = ACTIONS(2151),
    [sym_duet_dev_type_video_wall] = ACTIONS(2151),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2151),
    [sym_duet_dev_type_weather] = ACTIONS(2151),
    [sym_duet_dev_not_polled] = ACTIONS(2151),
    [sym_duet_dev_polled] = ACTIONS(2151),
    [sym_clkmgr_mode_network] = ACTIONS(2151),
    [sym_clkmgr_mode_standalone] = ACTIONS(2151),
    [sym_normal_standby] = ACTIONS(2151),
    [sym_normal_wake] = ACTIONS(2151),
    [sym_ip_multicast_ttl_option] = ACTIONS(2151),
    [sym_ip_ttl_subnet] = ACTIONS(2151),
    [sym_ip_ttl_site] = ACTIONS(2151),
    [sym_ip_ttl_region] = ACTIONS(2151),
    [sym_ip_ttl_continent] = ACTIONS(2151),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2151),
    [sym_ip_nodelay_on] = ACTIONS(2151),
    [sym_ip_nodelay_off] = ACTIONS(2151),
    [sym_amx_error] = ACTIONS(2151),
    [sym_amx_warning] = ACTIONS(2151),
    [sym_amx_info] = ACTIONS(2151),
    [sym_amx_debug] = ACTIONS(2151),
    [sym_netlinx_login_success] = ACTIONS(2151),
    [sym_netlinx_login_fail] = ACTIONS(2151),
    [sym_netlinx_logout] = ACTIONS(2151),
    [sym_null_str] = ACTIONS(2151),
    [sym_smtp_address] = ACTIONS(2151),
    [sym_smtp_port_number] = ACTIONS(2151),
    [sym_smtp_username] = ACTIONS(2151),
    [sym_smtp_password] = ACTIONS(2151),
    [sym_smtp_require_tls] = ACTIONS(2151),
    [sym_smtp_from] = ACTIONS(2151),
    [sym_smtp_tls_true] = ACTIONS(2151),
    [sym_smtp_tls_false] = ACTIONS(2151),
    [sym_listview_on_row_select_event] = ACTIONS(2151),
    [sym_data_structure_unknown] = ACTIONS(2151),
    [sym_data_structure_datafeed] = ACTIONS(2151),
    [sym_data_structure_datarecordset] = ACTIONS(2151),
    [sym_data_structure_datarecord] = ACTIONS(2151),
    [sym_data_structure_datafield] = ACTIONS(2151),
    [sym_data_max_content_field_count] = ACTIONS(2151),
    [sym_data_max_metadata_field_count] = ACTIONS(2151),
    [sym_data_max_id_length] = ACTIONS(2151),
    [sym_data_max_name_length] = ACTIONS(2151),
    [sym_data_max_description_length] = ACTIONS(2151),
    [sym_data_max_type_length] = ACTIONS(2151),
    [sym_data_max_format_length] = ACTIONS(2151),
    [sym_data_max_source_length] = ACTIONS(2151),
    [sym_data_max_value_length] = ACTIONS(2151),
    [sym_data_max_label_length] = ACTIONS(2151),
    [sym_data_max_url_length] = ACTIONS(2151),
    [sym_max_last_login_info_length] = ACTIONS(2151),
    [sym_data_type_unknown] = ACTIONS(2151),
    [sym_data_type_string] = ACTIONS(2151),
    [sym_data_type_datetime] = ACTIONS(2151),
    [sym_data_type_date] = ACTIONS(2151),
    [sym_data_type_time] = ACTIONS(2151),
    [sym_data_type_image] = ACTIONS(2151),
    [sym_data_format_url] = ACTIONS(2151),
    [sym_data_format_phone] = ACTIONS(2151),
    [sym_data_format_email] = ACTIONS(2151),
    [sym_data_format_iso8601] = ACTIONS(2151),
    [sym_auditlog_priv] = ACTIONS(2151),
    [sym_configuration_priv] = ACTIONS(2151),
    [sym_deviceconfig_priv] = ACTIONS(2151),
    [sym_ftp_priv] = ACTIONS(2151),
    [sym_http_priv] = ACTIONS(2151),
    [sym_networkconfig_priv] = ACTIONS(2151),
    [sym_programport_priv] = ACTIONS(2151),
    [sym_remoteui_priv] = ACTIONS(2151),
    [sym_securitycontrol_priv] = ACTIONS(2151),
    [sym_softwaremanagement_priv] = ACTIONS(2151),
    [sym_terminal_priv] = ACTIONS(2151),
    [sym_tpadmin_priv] = ACTIONS(2151),
    [sym_user1_priv] = ACTIONS(2151),
    [sym_user2_priv] = ACTIONS(2151),
    [sym_user3_priv] = ACTIONS(2151),
    [sym_user4_priv] = ACTIONS(2151),
    [sym_usermanagement_priv] = ACTIONS(2151),
    [sym_valid_account] = ACTIONS(2151),
    [sym_err_name_invalid] = ACTIONS(2151),
    [sym_err_password_invalid] = ACTIONS(2151),
    [sym_err_invalid_account] = ACTIONS(2151),
    [sym_err_no_authorization] = ACTIONS(2151),
    [sym_err_invalid_parameter] = ACTIONS(2151),
    [sym_err_user_locked_out] = ACTIONS(2151),
    [sym_err_user_account_expired] = ACTIONS(2151),
    [sym_err_general_error] = ACTIONS(2151),
    [sym_err_authentication_error] = ACTIONS(2151),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2151),
    [sym_tls_validate_certificate] = ACTIONS(2151),
    [sym____reserved___] = ACTIONS(2153),
    [sym_date] = ACTIONS(2153),
    [sym_day] = ACTIONS(2153),
    [sym_time] = ACTIONS(2153),
    [sym_ldate] = ACTIONS(2153),
    [sym_push_device] = ACTIONS(2153),
    [sym_push_channel] = ACTIONS(2153),
    [sym_push_devchan] = ACTIONS(2153),
    [sym_release_device] = ACTIONS(2153),
    [sym_release_channel] = ACTIONS(2153),
    [sym_release_devchan] = ACTIONS(2153),
    [sym_master_slot] = ACTIONS(2153),
    [sym_get_pulse_time] = ACTIONS(2153),
    [sym_get_timer] = ACTIONS(2153),
    [sym_master_sn] = ACTIONS(2153),
    [sym_system_number] = ACTIONS(2153),
    [sym_button] = ACTIONS(2153),
    [sym_dv_channel] = ACTIONS(2153),
    [sym_data] = ACTIONS(2153),
    [sym_level] = ACTIONS(2153),
    [sym_timeline] = ACTIONS(2153),
    [sym_channel] = ACTIONS(2153),
    [sym_custom] = ACTIONS(2153),
    [sym_atoi] = ACTIONS(139),
    [sym_atol] = ACTIONS(139),
    [sym_atof] = ACTIONS(139),
    [sym_itoa] = ACTIONS(139),
    [sym_itohex] = ACTIONS(139),
    [sym_hextoi] = ACTIONS(139),
    [sym_ftoa] = ACTIONS(139),
    [sym_find_string] = ACTIONS(139),
    [sym_left_string] = ACTIONS(139),
    [sym_right_string] = ACTIONS(139),
    [sym_mid_string] = ACTIONS(139),
    [sym_remove_string] = ACTIONS(139),
    [sym_get_buffer_string] = ACTIONS(139),
    [sym_get_buffer_char] = ACTIONS(139),
    [sym_get_multi_buffer_string] = ACTIONS(139),
    [sym_length_string] = ACTIONS(139),
    [sym_max_length_string] = ACTIONS(139),
    [sym_set_length_string] = ACTIONS(139),
    [sym_compare_string] = ACTIONS(139),
    [sym_upper_string] = ACTIONS(139),
    [sym_lower_string] = ACTIONS(139),
    [sym_get_last] = ACTIONS(139),
    [sym_date_to_day] = ACTIONS(139),
    [sym_date_to_month] = ACTIONS(139),
    [sym_date_to_year] = ACTIONS(139),
    [sym_time_to_hour] = ACTIONS(139),
    [sym_time_to_minute] = ACTIONS(139),
    [sym_time_to_second] = ACTIONS(139),
    [sym_day_of_week] = ACTIONS(139),
    [sym_set_pulse_time] = ACTIONS(139),
    [sym_set_timer] = ACTIONS(139),
    [sym_file_open] = ACTIONS(139),
    [sym_file_close] = ACTIONS(139),
    [sym_file_read] = ACTIONS(139),
    [sym_file_read_line] = ACTIONS(139),
    [sym_file_write] = ACTIONS(139),
    [sym_file_write_line] = ACTIONS(139),
    [sym_file_copy] = ACTIONS(139),
    [sym_file_delete] = ACTIONS(139),
    [sym_file_rename] = ACTIONS(139),
    [sym_file_dir] = ACTIONS(139),
    [sym_file_getdir] = ACTIONS(139),
    [sym_file_setdir] = ACTIONS(139),
    [sym_file_create_dir] = ACTIONS(139),
    [sym_file_remove_dir] = ACTIONS(139),
    [sym_file_seek] = ACTIONS(139),
    [sym_length_array] = ACTIONS(139),
    [sym_max_length_array] = ACTIONS(139),
    [sym_set_length_array] = ACTIONS(139),
    [sym_ip_client_open] = ACTIONS(139),
    [sym_ip_client_close] = ACTIONS(139),
    [sym_ip_server_open] = ACTIONS(139),
    [sym_ip_server_close] = ACTIONS(139),
    [sym_ip_bound_client_open] = ACTIONS(139),
    [sym_ip_mc_server_open] = ACTIONS(139),
    [sym_ip_set_option] = ACTIONS(139),
    [sym_ssh_client_open] = ACTIONS(139),
    [sym_ssh_client_close] = ACTIONS(139),
    [sym_tls_client_open] = ACTIONS(139),
    [sym_tls_client_close] = ACTIONS(139),
    [sym_do_push] = ACTIONS(139),
    [sym_do_push_timed] = ACTIONS(139),
    [sym_do_release] = ACTIONS(139),
    [sym_device_id] = ACTIONS(139),
    [sym_device_info] = ACTIONS(139),
    [sym_device_id_string] = ACTIONS(139),
    [sym_device_standby] = ACTIONS(139),
    [sym_device_wake] = ACTIONS(139),
    [sym_timeline_create] = ACTIONS(139),
    [sym_timeline_kill] = ACTIONS(139),
    [sym_timeline_reload] = ACTIONS(139),
    [sym_timeline_pause] = ACTIONS(139),
    [sym_timeline_restart] = ACTIONS(139),
    [sym_timeline_set] = ACTIONS(139),
    [sym_timeline_get] = ACTIONS(139),
    [sym_timeline_active] = ACTIONS(139),
    [sym_timeline_dynamic_id] = ACTIONS(139),
    [sym_variable_to_string] = ACTIONS(139),
    [sym_string_to_variable] = ACTIONS(139),
    [sym_length_variable_to_string] = ACTIONS(139),
    [sym_type_cast] = ACTIONS(139),
    [sym_variable_to_xml] = ACTIONS(139),
    [sym_xml_to_variable] = ACTIONS(139),
    [sym_length_variable_to_xml] = ACTIONS(139),
    [sym_redirect_string] = ACTIONS(139),
    [sym_random_number] = ACTIONS(139),
    [sym_get_unique_id] = ACTIONS(139),
    [sym_get_system_number] = ACTIONS(139),
    [sym_set_system_number] = ACTIONS(139),
    [sym_get_serial_number] = ACTIONS(139),
    [sym_get_master_build] = ACTIONS(139),
    [sym_set_validation_code] = ACTIONS(139),
    [sym_reboot] = ACTIONS(139),
    [sym_load_duet_module] = ACTIONS(139),
    [sym_unload_duet_module] = ACTIONS(139),
    [sym_duet_mem_size_set] = ACTIONS(139),
    [sym_duet_mem_size_get] = ACTIONS(139),
    [sym_dynamic_application_device] = ACTIONS(139),
    [sym_dynamic_polled_port] = ACTIONS(139),
    [sym_static_port_binding] = ACTIONS(139),
    [sym_static_ip_binding] = ACTIONS(139),
    [sym_get_dns_list] = ACTIONS(139),
    [sym_set_dns_list] = ACTIONS(139),
    [sym_get_ip_address] = ACTIONS(139),
    [sym_set_ip_address] = ACTIONS(139),
    [sym_get_url_list] = ACTIONS(139),
    [sym_add_url_entry] = ACTIONS(139),
    [sym_delete_url_entry] = ACTIONS(139),
    [sym_abs_value] = ACTIONS(139),
    [sym_max_value] = ACTIONS(139),
    [sym_min_value] = ACTIONS(139),
    [sym_exp_value] = ACTIONS(139),
    [sym_log_value] = ACTIONS(139),
    [sym_log10_value] = ACTIONS(139),
    [sym_power_value] = ACTIONS(139),
    [sym_sqrt_value] = ACTIONS(139),
    [sym_raw_be] = ACTIONS(139),
    [sym_raw_le] = ACTIONS(139),
    [sym_internal_queue_size_set] = ACTIONS(139),
    [sym_internal_queue_size_get] = ACTIONS(139),
    [sym_internal_threshold_set] = ACTIONS(139),
    [sym_internal_threshold_get] = ACTIONS(139),
    [sym_rebuilt_event] = ACTIONS(139),
    [sym_do_custom_event] = ACTIONS(139),
    [sym_clkmgr_is_network_sourced] = ACTIONS(139),
    [sym_clkmgr_set_clk_source] = ACTIONS(139),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(139),
    [sym_clkmgr_get_timezone] = ACTIONS(139),
    [sym_clkmgr_set_timezone] = ACTIONS(139),
    [sym_clkmgr_get_resync_period] = ACTIONS(139),
    [sym_clkmgr_set_resync_period] = ACTIONS(139),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(139),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_timeservers] = ACTIONS(139),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(139),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(139),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(139),
    [sym_set_log_level] = ACTIONS(139),
    [sym_get_log_level] = ACTIONS(139),
    [sym_amx_log] = ACTIONS(139),
    [sym_smtp_server_config_set] = ACTIONS(139),
    [sym_smtp_server_config_get] = ACTIONS(139),
    [sym_smtp_send] = ACTIONS(139),
    [sym_validate_netlinx_account] = ACTIONS(139),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(139),
    [sym_authenticate_certificate] = ACTIONS(139),
    [sym_audit_netlinx_session_event] = ACTIONS(139),
    [sym_audit_netlinx_generic_event] = ACTIONS(139),
    [sym_get_available_flash_disk_space] = ACTIONS(139),
    [sym_get_max_flash_disk_space] = ACTIONS(139),
    [sym_data_create_feed] = ACTIONS(139),
    [sym_data_delete_feed] = ACTIONS(139),
    [sym_data_publish_feed] = ACTIONS(139),
    [sym_data_get_published_feed] = ACTIONS(139),
    [sym_data_add_record] = ACTIONS(139),
    [sym_data_get_event_record] = ACTIONS(139),
    [sym__wc_data_create_feed] = ACTIONS(139),
    [sym__wc_data_add_record] = ACTIONS(139),
    [sym__wc_data_get_event_record] = ACTIONS(139),
    [sym_format] = ACTIONS(139),
    [sym_astro_clock] = ACTIONS(139),
    [sym_set_outdoor_temperature] = ACTIONS(139),
    [sym_set_virtual_level_count] = ACTIONS(139),
    [sym_set_virtual_channel_count] = ACTIONS(139),
    [sym_set_virtual_port_count] = ACTIONS(139),
    [sym__wc_encode_int] = ACTIONS(139),
    [sym__wc_decode_int] = ACTIONS(139),
    [sym__wc_int] = ACTIONS(139),
    [sym__wc_to_ch_int] = ACTIONS(139),
    [sym__ch_to_wc_int] = ACTIONS(139),
    [sym__wc_find_string_int] = ACTIONS(139),
    [sym__wc_left_string_int] = ACTIONS(139),
    [sym__wc_lower_string_int] = ACTIONS(139),
    [sym__wc_mid_string_int] = ACTIONS(139),
    [sym__wc_remove_string_int] = ACTIONS(139),
    [sym__wc_right_string_int] = ACTIONS(139),
    [sym__wc_upper_string_int] = ACTIONS(139),
    [sym__wc_compare_string_int] = ACTIONS(139),
    [sym__wc_get_buffer_char_int] = ACTIONS(139),
    [sym__wc_get_buffer_string_int] = ACTIONS(139),
    [sym__wc_concat_string_int] = ACTIONS(139),
    [sym__wc_file_open_int] = ACTIONS(139),
    [sym__wc_file_close_int] = ACTIONS(139),
    [sym__wc_file_read_int] = ACTIONS(139),
    [sym__wc_file_read_line_int] = ACTIONS(139),
    [sym__wc_file_write_int] = ACTIONS(139),
    [sym__wc_file_write_line_int] = ACTIONS(139),
    [anon_sym_LBRACK] = ACTIONS(2593),
    [anon_sym_LPAREN] = ACTIONS(2595),
    [anon_sym_BANG] = ACTIONS(2599),
    [anon_sym_TILDE] = ACTIONS(2599),
    [anon_sym_DASH] = ACTIONS(2591),
    [anon_sym_PLUS] = ACTIONS(2591),
    [anon_sym_DASH_DASH] = ACTIONS(2601),
    [anon_sym_PLUS_PLUS] = ACTIONS(2601),
    [anon_sym_DQUOTE] = ACTIONS(2603),
    [anon_sym_SQUOTE] = ACTIONS(2159),
    [sym_number_literal] = ACTIONS(2161),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(906)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2226),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(907)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2253),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(908)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2256),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(909)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2229),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(910)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2232),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(911)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2223),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(912)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2235),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(913)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2238),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(914)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2241),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(915)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2244),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(916)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2219),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(917)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2247),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(918)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_COMMA] = ACTIONS(2221),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2250),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(919)] = {
    [sym_identifier] = ACTIONS(2745),
    [sym_not] = ACTIONS(2745),
    [sym_bnot] = ACTIONS(2745),
    [sym___file__] = ACTIONS(2745),
    [sym___date__] = ACTIONS(2745),
    [sym___ldate__] = ACTIONS(2745),
    [sym___time__] = ACTIONS(2745),
    [sym___line__] = ACTIONS(2745),
    [sym___name__] = ACTIONS(2745),
    [sym___netlinx__] = ACTIONS(2745),
    [sym_netlinx_axi_version] = ACTIONS(2745),
    [sym_url_flg_tcp] = ACTIONS(2745),
    [sym_url_flg_accinfopresent] = ACTIONS(2745),
    [sym_url_flg_temp] = ACTIONS(2745),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2745),
    [sym_url_flg_stat_mask] = ACTIONS(2745),
    [sym_url_flg_stat_lookup] = ACTIONS(2745),
    [sym_url_flg_stat_connecting] = ACTIONS(2745),
    [sym_url_flg_stat_waiting] = ACTIONS(2745),
    [sym_url_flg_stat_connected] = ACTIONS(2745),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2745),
    [sym_first_virtual_device] = ACTIONS(2745),
    [sym_first_local_port] = ACTIONS(2745),
    [sym_dynamic_virtual_device] = ACTIONS(2745),
    [sym_timeline_once] = ACTIONS(2745),
    [sym_timeline_repeat] = ACTIONS(2745),
    [sym_timeline_absolute] = ACTIONS(2745),
    [sym_timeline_relative] = ACTIONS(2745),
    [sym_do_push_timed_infinite] = ACTIONS(2745),
    [sym_source_type_no_address] = ACTIONS(2745),
    [sym_source_type_neuron_id] = ACTIONS(2745),
    [sym_source_type_ip_address] = ACTIONS(2745),
    [sym_source_type_axlink] = ACTIONS(2745),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2745),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2745),
    [sym_source_type_ip_socket_address] = ACTIONS(2745),
    [sym_source_type_rs232] = ACTIONS(2745),
    [sym_source_type_internal] = ACTIONS(2745),
    [sym_source_type_mac_address] = ACTIONS(2745),
    [sym_source_type_ipv4_port] = ACTIONS(2745),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2745),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2745),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2745),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2745),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2745),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2745),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2745),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2745),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2745),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2745),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2745),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2745),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2745),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2745),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2745),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2745),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2745),
    [sym_true] = ACTIONS(2745),
    [sym_false] = ACTIONS(2745),
    [sym_file_read_only] = ACTIONS(2745),
    [sym_file_rw_new] = ACTIONS(2745),
    [sym_file_rw_append] = ACTIONS(2745),
    [sym_ip_tcp] = ACTIONS(2745),
    [sym_ip_udp] = ACTIONS(2745),
    [sym_ip_udp_2way] = ACTIONS(2745),
    [sym_xml_encode_types] = ACTIONS(2745),
    [sym_xml_encode_char_as_list] = ACTIONS(2745),
    [sym_xml_encode_le] = ACTIONS(2745),
    [sym_xml_decode_types] = ACTIONS(2745),
    [sym_xml_decode_no_preserve] = ACTIONS(2745),
    [sym_duet_dev_type_display_device] = ACTIONS(2745),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2745),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2745),
    [sym_duet_dev_type_io_device] = ACTIONS(2745),
    [sym_duet_dev_type_relay_device] = ACTIONS(2745),
    [sym_duet_dev_type_ups] = ACTIONS(2745),
    [sym_duet_dev_type_amplifier] = ACTIONS(2745),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2745),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2745),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2745),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2745),
    [sym_duet_dev_type_camera] = ACTIONS(2745),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2745),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2745),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2745),
    [sym_duet_dev_type_dss] = ACTIONS(2745),
    [sym_duet_dev_type_dvr] = ACTIONS(2745),
    [sym_duet_dev_type_disc_device] = ACTIONS(2745),
    [sym_duet_dev_type_document_camera] = ACTIONS(2745),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2745),
    [sym_duet_dev_type_hvac] = ACTIONS(2745),
    [sym_duet_dev_type_keypad] = ACTIONS(2745),
    [sym_duet_dev_type_light] = ACTIONS(2745),
    [sym_duet_dev_type_monitor] = ACTIONS(2745),
    [sym_duet_dev_type_motor] = ACTIONS(2745),
    [sym_duet_dev_type_multi_window] = ACTIONS(2745),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2745),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2745),
    [sym_duet_dev_type_receiver] = ACTIONS(2745),
    [sym_duet_dev_type_security_system] = ACTIONS(2745),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2745),
    [sym_duet_dev_type_settop_box] = ACTIONS(2745),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2745),
    [sym_duet_dev_type_switcher] = ACTIONS(2745),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2745),
    [sym_duet_dev_type_tv] = ACTIONS(2745),
    [sym_duet_dev_type_utility] = ACTIONS(2745),
    [sym_duet_dev_type_vcr] = ACTIONS(2745),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2745),
    [sym_duet_dev_type_video_processor] = ACTIONS(2745),
    [sym_duet_dev_type_video_projector] = ACTIONS(2745),
    [sym_duet_dev_type_video_wall] = ACTIONS(2745),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2745),
    [sym_duet_dev_type_weather] = ACTIONS(2745),
    [sym_duet_dev_not_polled] = ACTIONS(2745),
    [sym_duet_dev_polled] = ACTIONS(2745),
    [sym_clkmgr_mode_network] = ACTIONS(2745),
    [sym_clkmgr_mode_standalone] = ACTIONS(2745),
    [sym_normal_standby] = ACTIONS(2745),
    [sym_normal_wake] = ACTIONS(2745),
    [sym_ip_multicast_ttl_option] = ACTIONS(2745),
    [sym_ip_ttl_subnet] = ACTIONS(2745),
    [sym_ip_ttl_site] = ACTIONS(2745),
    [sym_ip_ttl_region] = ACTIONS(2745),
    [sym_ip_ttl_continent] = ACTIONS(2745),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2745),
    [sym_ip_nodelay_on] = ACTIONS(2745),
    [sym_ip_nodelay_off] = ACTIONS(2745),
    [sym_amx_error] = ACTIONS(2745),
    [sym_amx_warning] = ACTIONS(2745),
    [sym_amx_info] = ACTIONS(2745),
    [sym_amx_debug] = ACTIONS(2745),
    [sym_netlinx_login_success] = ACTIONS(2745),
    [sym_netlinx_login_fail] = ACTIONS(2745),
    [sym_netlinx_logout] = ACTIONS(2745),
    [sym_null_str] = ACTIONS(2745),
    [sym_smtp_address] = ACTIONS(2745),
    [sym_smtp_port_number] = ACTIONS(2745),
    [sym_smtp_username] = ACTIONS(2745),
    [sym_smtp_password] = ACTIONS(2745),
    [sym_smtp_require_tls] = ACTIONS(2745),
    [sym_smtp_from] = ACTIONS(2745),
    [sym_smtp_tls_true] = ACTIONS(2745),
    [sym_smtp_tls_false] = ACTIONS(2745),
    [sym_listview_on_row_select_event] = ACTIONS(2745),
    [sym_data_structure_unknown] = ACTIONS(2745),
    [sym_data_structure_datafeed] = ACTIONS(2745),
    [sym_data_structure_datarecordset] = ACTIONS(2745),
    [sym_data_structure_datarecord] = ACTIONS(2745),
    [sym_data_structure_datafield] = ACTIONS(2745),
    [sym_data_max_content_field_count] = ACTIONS(2745),
    [sym_data_max_metadata_field_count] = ACTIONS(2745),
    [sym_data_max_id_length] = ACTIONS(2745),
    [sym_data_max_name_length] = ACTIONS(2745),
    [sym_data_max_description_length] = ACTIONS(2745),
    [sym_data_max_type_length] = ACTIONS(2745),
    [sym_data_max_format_length] = ACTIONS(2745),
    [sym_data_max_source_length] = ACTIONS(2745),
    [sym_data_max_value_length] = ACTIONS(2745),
    [sym_data_max_label_length] = ACTIONS(2745),
    [sym_data_max_url_length] = ACTIONS(2745),
    [sym_max_last_login_info_length] = ACTIONS(2745),
    [sym_data_type_unknown] = ACTIONS(2745),
    [sym_data_type_string] = ACTIONS(2745),
    [sym_data_type_datetime] = ACTIONS(2745),
    [sym_data_type_date] = ACTIONS(2745),
    [sym_data_type_time] = ACTIONS(2745),
    [sym_data_type_image] = ACTIONS(2745),
    [sym_data_format_url] = ACTIONS(2745),
    [sym_data_format_phone] = ACTIONS(2745),
    [sym_data_format_email] = ACTIONS(2745),
    [sym_data_format_iso8601] = ACTIONS(2745),
    [sym_auditlog_priv] = ACTIONS(2745),
    [sym_configuration_priv] = ACTIONS(2745),
    [sym_deviceconfig_priv] = ACTIONS(2745),
    [sym_ftp_priv] = ACTIONS(2745),
    [sym_http_priv] = ACTIONS(2745),
    [sym_networkconfig_priv] = ACTIONS(2745),
    [sym_programport_priv] = ACTIONS(2745),
    [sym_remoteui_priv] = ACTIONS(2745),
    [sym_securitycontrol_priv] = ACTIONS(2745),
    [sym_softwaremanagement_priv] = ACTIONS(2745),
    [sym_terminal_priv] = ACTIONS(2745),
    [sym_tpadmin_priv] = ACTIONS(2745),
    [sym_user1_priv] = ACTIONS(2745),
    [sym_user2_priv] = ACTIONS(2745),
    [sym_user3_priv] = ACTIONS(2745),
    [sym_user4_priv] = ACTIONS(2745),
    [sym_usermanagement_priv] = ACTIONS(2745),
    [sym_valid_account] = ACTIONS(2745),
    [sym_err_name_invalid] = ACTIONS(2745),
    [sym_err_password_invalid] = ACTIONS(2745),
    [sym_err_invalid_account] = ACTIONS(2745),
    [sym_err_no_authorization] = ACTIONS(2745),
    [sym_err_invalid_parameter] = ACTIONS(2745),
    [sym_err_user_locked_out] = ACTIONS(2745),
    [sym_err_user_account_expired] = ACTIONS(2745),
    [sym_err_general_error] = ACTIONS(2745),
    [sym_err_authentication_error] = ACTIONS(2745),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2745),
    [sym_tls_validate_certificate] = ACTIONS(2745),
    [sym____reserved___] = ACTIONS(2745),
    [sym_date] = ACTIONS(2745),
    [sym_day] = ACTIONS(2745),
    [sym_time] = ACTIONS(2745),
    [sym_ldate] = ACTIONS(2745),
    [sym_push_device] = ACTIONS(2745),
    [sym_push_channel] = ACTIONS(2745),
    [sym_push_devchan] = ACTIONS(2745),
    [sym_release_device] = ACTIONS(2745),
    [sym_release_channel] = ACTIONS(2745),
    [sym_release_devchan] = ACTIONS(2745),
    [sym_master_slot] = ACTIONS(2745),
    [sym_get_pulse_time] = ACTIONS(2745),
    [sym_get_timer] = ACTIONS(2745),
    [sym_master_sn] = ACTIONS(2745),
    [sym_system_number] = ACTIONS(2745),
    [sym_button] = ACTIONS(2745),
    [sym_dv_channel] = ACTIONS(2745),
    [sym_data] = ACTIONS(2745),
    [sym_level] = ACTIONS(2745),
    [sym_timeline] = ACTIONS(2745),
    [sym_channel] = ACTIONS(2745),
    [sym_custom] = ACTIONS(2745),
    [sym_atoi] = ACTIONS(2745),
    [sym_atol] = ACTIONS(2745),
    [sym_atof] = ACTIONS(2745),
    [sym_itoa] = ACTIONS(2745),
    [sym_itohex] = ACTIONS(2745),
    [sym_hextoi] = ACTIONS(2745),
    [sym_ftoa] = ACTIONS(2745),
    [sym_find_string] = ACTIONS(2745),
    [sym_left_string] = ACTIONS(2745),
    [sym_right_string] = ACTIONS(2745),
    [sym_mid_string] = ACTIONS(2745),
    [sym_remove_string] = ACTIONS(2745),
    [sym_get_buffer_string] = ACTIONS(2745),
    [sym_get_buffer_char] = ACTIONS(2745),
    [sym_get_multi_buffer_string] = ACTIONS(2745),
    [sym_length_string] = ACTIONS(2745),
    [sym_max_length_string] = ACTIONS(2745),
    [sym_set_length_string] = ACTIONS(2745),
    [sym_compare_string] = ACTIONS(2745),
    [sym_upper_string] = ACTIONS(2745),
    [sym_lower_string] = ACTIONS(2745),
    [sym_get_last] = ACTIONS(2745),
    [sym_date_to_day] = ACTIONS(2745),
    [sym_date_to_month] = ACTIONS(2745),
    [sym_date_to_year] = ACTIONS(2745),
    [sym_time_to_hour] = ACTIONS(2745),
    [sym_time_to_minute] = ACTIONS(2745),
    [sym_time_to_second] = ACTIONS(2745),
    [sym_day_of_week] = ACTIONS(2745),
    [sym_set_pulse_time] = ACTIONS(2745),
    [sym_set_timer] = ACTIONS(2745),
    [sym_file_open] = ACTIONS(2745),
    [sym_file_close] = ACTIONS(2745),
    [sym_file_read] = ACTIONS(2745),
    [sym_file_read_line] = ACTIONS(2745),
    [sym_file_write] = ACTIONS(2745),
    [sym_file_write_line] = ACTIONS(2745),
    [sym_file_copy] = ACTIONS(2745),
    [sym_file_delete] = ACTIONS(2745),
    [sym_file_rename] = ACTIONS(2745),
    [sym_file_dir] = ACTIONS(2745),
    [sym_file_getdir] = ACTIONS(2745),
    [sym_file_setdir] = ACTIONS(2745),
    [sym_file_create_dir] = ACTIONS(2745),
    [sym_file_remove_dir] = ACTIONS(2745),
    [sym_file_seek] = ACTIONS(2745),
    [sym_length_array] = ACTIONS(2745),
    [sym_max_length_array] = ACTIONS(2745),
    [sym_set_length_array] = ACTIONS(2745),
    [sym_ip_client_open] = ACTIONS(2745),
    [sym_ip_client_close] = ACTIONS(2745),
    [sym_ip_server_open] = ACTIONS(2745),
    [sym_ip_server_close] = ACTIONS(2745),
    [sym_ip_bound_client_open] = ACTIONS(2745),
    [sym_ip_mc_server_open] = ACTIONS(2745),
    [sym_ip_set_option] = ACTIONS(2745),
    [sym_ssh_client_open] = ACTIONS(2745),
    [sym_ssh_client_close] = ACTIONS(2745),
    [sym_tls_client_open] = ACTIONS(2745),
    [sym_tls_client_close] = ACTIONS(2745),
    [sym_do_push] = ACTIONS(2745),
    [sym_do_push_timed] = ACTIONS(2745),
    [sym_do_release] = ACTIONS(2745),
    [sym_device_id] = ACTIONS(2745),
    [sym_device_info] = ACTIONS(2745),
    [sym_device_id_string] = ACTIONS(2745),
    [sym_device_standby] = ACTIONS(2745),
    [sym_device_wake] = ACTIONS(2745),
    [sym_timeline_create] = ACTIONS(2745),
    [sym_timeline_kill] = ACTIONS(2745),
    [sym_timeline_reload] = ACTIONS(2745),
    [sym_timeline_pause] = ACTIONS(2745),
    [sym_timeline_restart] = ACTIONS(2745),
    [sym_timeline_set] = ACTIONS(2745),
    [sym_timeline_get] = ACTIONS(2745),
    [sym_timeline_active] = ACTIONS(2745),
    [sym_timeline_dynamic_id] = ACTIONS(2745),
    [sym_variable_to_string] = ACTIONS(2745),
    [sym_string_to_variable] = ACTIONS(2745),
    [sym_length_variable_to_string] = ACTIONS(2745),
    [sym_type_cast] = ACTIONS(2745),
    [sym_variable_to_xml] = ACTIONS(2745),
    [sym_xml_to_variable] = ACTIONS(2745),
    [sym_length_variable_to_xml] = ACTIONS(2745),
    [sym_redirect_string] = ACTIONS(2745),
    [sym_random_number] = ACTIONS(2745),
    [sym_get_unique_id] = ACTIONS(2745),
    [sym_get_system_number] = ACTIONS(2745),
    [sym_set_system_number] = ACTIONS(2745),
    [sym_get_serial_number] = ACTIONS(2745),
    [sym_get_master_build] = ACTIONS(2745),
    [sym_set_validation_code] = ACTIONS(2745),
    [sym_reboot] = ACTIONS(2745),
    [sym_load_duet_module] = ACTIONS(2745),
    [sym_unload_duet_module] = ACTIONS(2745),
    [sym_duet_mem_size_set] = ACTIONS(2745),
    [sym_duet_mem_size_get] = ACTIONS(2745),
    [sym_dynamic_application_device] = ACTIONS(2745),
    [sym_dynamic_polled_port] = ACTIONS(2745),
    [sym_static_port_binding] = ACTIONS(2745),
    [sym_static_ip_binding] = ACTIONS(2745),
    [sym_get_dns_list] = ACTIONS(2745),
    [sym_set_dns_list] = ACTIONS(2745),
    [sym_get_ip_address] = ACTIONS(2745),
    [sym_set_ip_address] = ACTIONS(2745),
    [sym_get_url_list] = ACTIONS(2745),
    [sym_add_url_entry] = ACTIONS(2745),
    [sym_delete_url_entry] = ACTIONS(2745),
    [sym_abs_value] = ACTIONS(2745),
    [sym_max_value] = ACTIONS(2745),
    [sym_min_value] = ACTIONS(2745),
    [sym_exp_value] = ACTIONS(2745),
    [sym_log_value] = ACTIONS(2745),
    [sym_log10_value] = ACTIONS(2745),
    [sym_power_value] = ACTIONS(2745),
    [sym_sqrt_value] = ACTIONS(2745),
    [sym_raw_be] = ACTIONS(2745),
    [sym_raw_le] = ACTIONS(2745),
    [sym_internal_queue_size_set] = ACTIONS(2745),
    [sym_internal_queue_size_get] = ACTIONS(2745),
    [sym_internal_threshold_set] = ACTIONS(2745),
    [sym_internal_threshold_get] = ACTIONS(2745),
    [sym_rebuilt_event] = ACTIONS(2745),
    [sym_do_custom_event] = ACTIONS(2745),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2745),
    [sym_clkmgr_set_clk_source] = ACTIONS(2745),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2745),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2745),
    [sym_clkmgr_get_timezone] = ACTIONS(2745),
    [sym_clkmgr_set_timezone] = ACTIONS(2745),
    [sym_clkmgr_get_resync_period] = ACTIONS(2745),
    [sym_clkmgr_set_resync_period] = ACTIONS(2745),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2745),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2745),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2745),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2745),
    [sym_clkmgr_get_timeservers] = ACTIONS(2745),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2745),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2745),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2745),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2745),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2745),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2745),
    [sym_set_log_level] = ACTIONS(2745),
    [sym_get_log_level] = ACTIONS(2745),
    [sym_amx_log] = ACTIONS(2745),
    [sym_smtp_server_config_set] = ACTIONS(2745),
    [sym_smtp_server_config_get] = ACTIONS(2745),
    [sym_smtp_send] = ACTIONS(2745),
    [sym_validate_netlinx_account] = ACTIONS(2745),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2745),
    [sym_authenticate_certificate] = ACTIONS(2745),
    [sym_audit_netlinx_session_event] = ACTIONS(2745),
    [sym_audit_netlinx_generic_event] = ACTIONS(2745),
    [sym_get_available_flash_disk_space] = ACTIONS(2745),
    [sym_get_max_flash_disk_space] = ACTIONS(2745),
    [sym_data_create_feed] = ACTIONS(2745),
    [sym_data_delete_feed] = ACTIONS(2745),
    [sym_data_publish_feed] = ACTIONS(2745),
    [sym_data_get_published_feed] = ACTIONS(2745),
    [sym_data_add_record] = ACTIONS(2745),
    [sym_data_get_event_record] = ACTIONS(2745),
    [sym__wc_data_create_feed] = ACTIONS(2745),
    [sym__wc_data_add_record] = ACTIONS(2745),
    [sym__wc_data_get_event_record] = ACTIONS(2745),
    [sym_format] = ACTIONS(2745),
    [sym_astro_clock] = ACTIONS(2745),
    [sym_set_outdoor_temperature] = ACTIONS(2745),
    [sym_set_virtual_level_count] = ACTIONS(2745),
    [sym_set_virtual_channel_count] = ACTIONS(2745),
    [sym_set_virtual_port_count] = ACTIONS(2745),
    [sym__wc_encode_int] = ACTIONS(2745),
    [sym__wc_decode_int] = ACTIONS(2745),
    [sym__wc_int] = ACTIONS(2745),
    [sym__wc_to_ch_int] = ACTIONS(2745),
    [sym__ch_to_wc_int] = ACTIONS(2745),
    [sym__wc_find_string_int] = ACTIONS(2745),
    [sym__wc_left_string_int] = ACTIONS(2745),
    [sym__wc_lower_string_int] = ACTIONS(2745),
    [sym__wc_mid_string_int] = ACTIONS(2745),
    [sym__wc_remove_string_int] = ACTIONS(2745),
    [sym__wc_right_string_int] = ACTIONS(2745),
    [sym__wc_upper_string_int] = ACTIONS(2745),
    [sym__wc_compare_string_int] = ACTIONS(2745),
    [sym__wc_get_buffer_char_int] = ACTIONS(2745),
    [sym__wc_get_buffer_string_int] = ACTIONS(2745),
    [sym__wc_concat_string_int] = ACTIONS(2745),
    [sym__wc_file_open_int] = ACTIONS(2745),
    [sym__wc_file_close_int] = ACTIONS(2745),
    [sym__wc_file_read_int] = ACTIONS(2745),
    [sym__wc_file_read_line_int] = ACTIONS(2745),
    [sym__wc_file_write_int] = ACTIONS(2745),
    [sym__wc_file_write_line_int] = ACTIONS(2745),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2748),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2748),
    [anon_sym_LBRACK] = ACTIONS(2748),
    [anon_sym_LBRACE] = ACTIONS(2748),
    [anon_sym_RBRACE] = ACTIONS(2748),
    [anon_sym_LPAREN] = ACTIONS(2745),
    [anon_sym_BANG] = ACTIONS(2748),
    [anon_sym_TILDE] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2745),
    [anon_sym_PLUS] = ACTIONS(2745),
    [anon_sym_DASH_DASH] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2748),
    [anon_sym_DQUOTE] = ACTIONS(2748),
    [anon_sym_SQUOTE] = ACTIONS(2748),
    [sym_number_literal] = ACTIONS(2748),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(920)] = {
    [sym_identifier] = ACTIONS(2751),
    [sym_not] = ACTIONS(2751),
    [sym_bnot] = ACTIONS(2751),
    [sym___file__] = ACTIONS(2751),
    [sym___date__] = ACTIONS(2751),
    [sym___ldate__] = ACTIONS(2751),
    [sym___time__] = ACTIONS(2751),
    [sym___line__] = ACTIONS(2751),
    [sym___name__] = ACTIONS(2751),
    [sym___netlinx__] = ACTIONS(2751),
    [sym_netlinx_axi_version] = ACTIONS(2751),
    [sym_url_flg_tcp] = ACTIONS(2751),
    [sym_url_flg_accinfopresent] = ACTIONS(2751),
    [sym_url_flg_temp] = ACTIONS(2751),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2751),
    [sym_url_flg_stat_mask] = ACTIONS(2751),
    [sym_url_flg_stat_lookup] = ACTIONS(2751),
    [sym_url_flg_stat_connecting] = ACTIONS(2751),
    [sym_url_flg_stat_waiting] = ACTIONS(2751),
    [sym_url_flg_stat_connected] = ACTIONS(2751),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2751),
    [sym_first_virtual_device] = ACTIONS(2751),
    [sym_first_local_port] = ACTIONS(2751),
    [sym_dynamic_virtual_device] = ACTIONS(2751),
    [sym_timeline_once] = ACTIONS(2751),
    [sym_timeline_repeat] = ACTIONS(2751),
    [sym_timeline_absolute] = ACTIONS(2751),
    [sym_timeline_relative] = ACTIONS(2751),
    [sym_do_push_timed_infinite] = ACTIONS(2751),
    [sym_source_type_no_address] = ACTIONS(2751),
    [sym_source_type_neuron_id] = ACTIONS(2751),
    [sym_source_type_ip_address] = ACTIONS(2751),
    [sym_source_type_axlink] = ACTIONS(2751),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2751),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2751),
    [sym_source_type_ip_socket_address] = ACTIONS(2751),
    [sym_source_type_rs232] = ACTIONS(2751),
    [sym_source_type_internal] = ACTIONS(2751),
    [sym_source_type_mac_address] = ACTIONS(2751),
    [sym_source_type_ipv4_port] = ACTIONS(2751),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2751),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2751),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2751),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2751),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2751),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2751),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2751),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2751),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2751),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2751),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2751),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2751),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2751),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2751),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2751),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2751),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2751),
    [sym_true] = ACTIONS(2751),
    [sym_false] = ACTIONS(2751),
    [sym_file_read_only] = ACTIONS(2751),
    [sym_file_rw_new] = ACTIONS(2751),
    [sym_file_rw_append] = ACTIONS(2751),
    [sym_ip_tcp] = ACTIONS(2751),
    [sym_ip_udp] = ACTIONS(2751),
    [sym_ip_udp_2way] = ACTIONS(2751),
    [sym_xml_encode_types] = ACTIONS(2751),
    [sym_xml_encode_char_as_list] = ACTIONS(2751),
    [sym_xml_encode_le] = ACTIONS(2751),
    [sym_xml_decode_types] = ACTIONS(2751),
    [sym_xml_decode_no_preserve] = ACTIONS(2751),
    [sym_duet_dev_type_display_device] = ACTIONS(2751),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2751),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2751),
    [sym_duet_dev_type_io_device] = ACTIONS(2751),
    [sym_duet_dev_type_relay_device] = ACTIONS(2751),
    [sym_duet_dev_type_ups] = ACTIONS(2751),
    [sym_duet_dev_type_amplifier] = ACTIONS(2751),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2751),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2751),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2751),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2751),
    [sym_duet_dev_type_camera] = ACTIONS(2751),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2751),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2751),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2751),
    [sym_duet_dev_type_dss] = ACTIONS(2751),
    [sym_duet_dev_type_dvr] = ACTIONS(2751),
    [sym_duet_dev_type_disc_device] = ACTIONS(2751),
    [sym_duet_dev_type_document_camera] = ACTIONS(2751),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2751),
    [sym_duet_dev_type_hvac] = ACTIONS(2751),
    [sym_duet_dev_type_keypad] = ACTIONS(2751),
    [sym_duet_dev_type_light] = ACTIONS(2751),
    [sym_duet_dev_type_monitor] = ACTIONS(2751),
    [sym_duet_dev_type_motor] = ACTIONS(2751),
    [sym_duet_dev_type_multi_window] = ACTIONS(2751),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2751),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2751),
    [sym_duet_dev_type_receiver] = ACTIONS(2751),
    [sym_duet_dev_type_security_system] = ACTIONS(2751),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2751),
    [sym_duet_dev_type_settop_box] = ACTIONS(2751),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2751),
    [sym_duet_dev_type_switcher] = ACTIONS(2751),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2751),
    [sym_duet_dev_type_tv] = ACTIONS(2751),
    [sym_duet_dev_type_utility] = ACTIONS(2751),
    [sym_duet_dev_type_vcr] = ACTIONS(2751),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2751),
    [sym_duet_dev_type_video_processor] = ACTIONS(2751),
    [sym_duet_dev_type_video_projector] = ACTIONS(2751),
    [sym_duet_dev_type_video_wall] = ACTIONS(2751),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2751),
    [sym_duet_dev_type_weather] = ACTIONS(2751),
    [sym_duet_dev_not_polled] = ACTIONS(2751),
    [sym_duet_dev_polled] = ACTIONS(2751),
    [sym_clkmgr_mode_network] = ACTIONS(2751),
    [sym_clkmgr_mode_standalone] = ACTIONS(2751),
    [sym_normal_standby] = ACTIONS(2751),
    [sym_normal_wake] = ACTIONS(2751),
    [sym_ip_multicast_ttl_option] = ACTIONS(2751),
    [sym_ip_ttl_subnet] = ACTIONS(2751),
    [sym_ip_ttl_site] = ACTIONS(2751),
    [sym_ip_ttl_region] = ACTIONS(2751),
    [sym_ip_ttl_continent] = ACTIONS(2751),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2751),
    [sym_ip_nodelay_on] = ACTIONS(2751),
    [sym_ip_nodelay_off] = ACTIONS(2751),
    [sym_amx_error] = ACTIONS(2751),
    [sym_amx_warning] = ACTIONS(2751),
    [sym_amx_info] = ACTIONS(2751),
    [sym_amx_debug] = ACTIONS(2751),
    [sym_netlinx_login_success] = ACTIONS(2751),
    [sym_netlinx_login_fail] = ACTIONS(2751),
    [sym_netlinx_logout] = ACTIONS(2751),
    [sym_null_str] = ACTIONS(2751),
    [sym_smtp_address] = ACTIONS(2751),
    [sym_smtp_port_number] = ACTIONS(2751),
    [sym_smtp_username] = ACTIONS(2751),
    [sym_smtp_password] = ACTIONS(2751),
    [sym_smtp_require_tls] = ACTIONS(2751),
    [sym_smtp_from] = ACTIONS(2751),
    [sym_smtp_tls_true] = ACTIONS(2751),
    [sym_smtp_tls_false] = ACTIONS(2751),
    [sym_listview_on_row_select_event] = ACTIONS(2751),
    [sym_data_structure_unknown] = ACTIONS(2751),
    [sym_data_structure_datafeed] = ACTIONS(2751),
    [sym_data_structure_datarecordset] = ACTIONS(2751),
    [sym_data_structure_datarecord] = ACTIONS(2751),
    [sym_data_structure_datafield] = ACTIONS(2751),
    [sym_data_max_content_field_count] = ACTIONS(2751),
    [sym_data_max_metadata_field_count] = ACTIONS(2751),
    [sym_data_max_id_length] = ACTIONS(2751),
    [sym_data_max_name_length] = ACTIONS(2751),
    [sym_data_max_description_length] = ACTIONS(2751),
    [sym_data_max_type_length] = ACTIONS(2751),
    [sym_data_max_format_length] = ACTIONS(2751),
    [sym_data_max_source_length] = ACTIONS(2751),
    [sym_data_max_value_length] = ACTIONS(2751),
    [sym_data_max_label_length] = ACTIONS(2751),
    [sym_data_max_url_length] = ACTIONS(2751),
    [sym_max_last_login_info_length] = ACTIONS(2751),
    [sym_data_type_unknown] = ACTIONS(2751),
    [sym_data_type_string] = ACTIONS(2751),
    [sym_data_type_datetime] = ACTIONS(2751),
    [sym_data_type_date] = ACTIONS(2751),
    [sym_data_type_time] = ACTIONS(2751),
    [sym_data_type_image] = ACTIONS(2751),
    [sym_data_format_url] = ACTIONS(2751),
    [sym_data_format_phone] = ACTIONS(2751),
    [sym_data_format_email] = ACTIONS(2751),
    [sym_data_format_iso8601] = ACTIONS(2751),
    [sym_auditlog_priv] = ACTIONS(2751),
    [sym_configuration_priv] = ACTIONS(2751),
    [sym_deviceconfig_priv] = ACTIONS(2751),
    [sym_ftp_priv] = ACTIONS(2751),
    [sym_http_priv] = ACTIONS(2751),
    [sym_networkconfig_priv] = ACTIONS(2751),
    [sym_programport_priv] = ACTIONS(2751),
    [sym_remoteui_priv] = ACTIONS(2751),
    [sym_securitycontrol_priv] = ACTIONS(2751),
    [sym_softwaremanagement_priv] = ACTIONS(2751),
    [sym_terminal_priv] = ACTIONS(2751),
    [sym_tpadmin_priv] = ACTIONS(2751),
    [sym_user1_priv] = ACTIONS(2751),
    [sym_user2_priv] = ACTIONS(2751),
    [sym_user3_priv] = ACTIONS(2751),
    [sym_user4_priv] = ACTIONS(2751),
    [sym_usermanagement_priv] = ACTIONS(2751),
    [sym_valid_account] = ACTIONS(2751),
    [sym_err_name_invalid] = ACTIONS(2751),
    [sym_err_password_invalid] = ACTIONS(2751),
    [sym_err_invalid_account] = ACTIONS(2751),
    [sym_err_no_authorization] = ACTIONS(2751),
    [sym_err_invalid_parameter] = ACTIONS(2751),
    [sym_err_user_locked_out] = ACTIONS(2751),
    [sym_err_user_account_expired] = ACTIONS(2751),
    [sym_err_general_error] = ACTIONS(2751),
    [sym_err_authentication_error] = ACTIONS(2751),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2751),
    [sym_tls_validate_certificate] = ACTIONS(2751),
    [sym____reserved___] = ACTIONS(2751),
    [sym_date] = ACTIONS(2751),
    [sym_day] = ACTIONS(2751),
    [sym_time] = ACTIONS(2751),
    [sym_ldate] = ACTIONS(2751),
    [sym_push_device] = ACTIONS(2751),
    [sym_push_channel] = ACTIONS(2751),
    [sym_push_devchan] = ACTIONS(2751),
    [sym_release_device] = ACTIONS(2751),
    [sym_release_channel] = ACTIONS(2751),
    [sym_release_devchan] = ACTIONS(2751),
    [sym_master_slot] = ACTIONS(2751),
    [sym_get_pulse_time] = ACTIONS(2751),
    [sym_get_timer] = ACTIONS(2751),
    [sym_master_sn] = ACTIONS(2751),
    [sym_system_number] = ACTIONS(2751),
    [sym_button] = ACTIONS(2751),
    [sym_dv_channel] = ACTIONS(2751),
    [sym_data] = ACTIONS(2751),
    [sym_level] = ACTIONS(2751),
    [sym_timeline] = ACTIONS(2751),
    [sym_channel] = ACTIONS(2751),
    [sym_custom] = ACTIONS(2751),
    [sym_atoi] = ACTIONS(2751),
    [sym_atol] = ACTIONS(2751),
    [sym_atof] = ACTIONS(2751),
    [sym_itoa] = ACTIONS(2751),
    [sym_itohex] = ACTIONS(2751),
    [sym_hextoi] = ACTIONS(2751),
    [sym_ftoa] = ACTIONS(2751),
    [sym_find_string] = ACTIONS(2751),
    [sym_left_string] = ACTIONS(2751),
    [sym_right_string] = ACTIONS(2751),
    [sym_mid_string] = ACTIONS(2751),
    [sym_remove_string] = ACTIONS(2751),
    [sym_get_buffer_string] = ACTIONS(2751),
    [sym_get_buffer_char] = ACTIONS(2751),
    [sym_get_multi_buffer_string] = ACTIONS(2751),
    [sym_length_string] = ACTIONS(2751),
    [sym_max_length_string] = ACTIONS(2751),
    [sym_set_length_string] = ACTIONS(2751),
    [sym_compare_string] = ACTIONS(2751),
    [sym_upper_string] = ACTIONS(2751),
    [sym_lower_string] = ACTIONS(2751),
    [sym_get_last] = ACTIONS(2751),
    [sym_date_to_day] = ACTIONS(2751),
    [sym_date_to_month] = ACTIONS(2751),
    [sym_date_to_year] = ACTIONS(2751),
    [sym_time_to_hour] = ACTIONS(2751),
    [sym_time_to_minute] = ACTIONS(2751),
    [sym_time_to_second] = ACTIONS(2751),
    [sym_day_of_week] = ACTIONS(2751),
    [sym_set_pulse_time] = ACTIONS(2751),
    [sym_set_timer] = ACTIONS(2751),
    [sym_file_open] = ACTIONS(2751),
    [sym_file_close] = ACTIONS(2751),
    [sym_file_read] = ACTIONS(2751),
    [sym_file_read_line] = ACTIONS(2751),
    [sym_file_write] = ACTIONS(2751),
    [sym_file_write_line] = ACTIONS(2751),
    [sym_file_copy] = ACTIONS(2751),
    [sym_file_delete] = ACTIONS(2751),
    [sym_file_rename] = ACTIONS(2751),
    [sym_file_dir] = ACTIONS(2751),
    [sym_file_getdir] = ACTIONS(2751),
    [sym_file_setdir] = ACTIONS(2751),
    [sym_file_create_dir] = ACTIONS(2751),
    [sym_file_remove_dir] = ACTIONS(2751),
    [sym_file_seek] = ACTIONS(2751),
    [sym_length_array] = ACTIONS(2751),
    [sym_max_length_array] = ACTIONS(2751),
    [sym_set_length_array] = ACTIONS(2751),
    [sym_ip_client_open] = ACTIONS(2751),
    [sym_ip_client_close] = ACTIONS(2751),
    [sym_ip_server_open] = ACTIONS(2751),
    [sym_ip_server_close] = ACTIONS(2751),
    [sym_ip_bound_client_open] = ACTIONS(2751),
    [sym_ip_mc_server_open] = ACTIONS(2751),
    [sym_ip_set_option] = ACTIONS(2751),
    [sym_ssh_client_open] = ACTIONS(2751),
    [sym_ssh_client_close] = ACTIONS(2751),
    [sym_tls_client_open] = ACTIONS(2751),
    [sym_tls_client_close] = ACTIONS(2751),
    [sym_do_push] = ACTIONS(2751),
    [sym_do_push_timed] = ACTIONS(2751),
    [sym_do_release] = ACTIONS(2751),
    [sym_device_id] = ACTIONS(2751),
    [sym_device_info] = ACTIONS(2751),
    [sym_device_id_string] = ACTIONS(2751),
    [sym_device_standby] = ACTIONS(2751),
    [sym_device_wake] = ACTIONS(2751),
    [sym_timeline_create] = ACTIONS(2751),
    [sym_timeline_kill] = ACTIONS(2751),
    [sym_timeline_reload] = ACTIONS(2751),
    [sym_timeline_pause] = ACTIONS(2751),
    [sym_timeline_restart] = ACTIONS(2751),
    [sym_timeline_set] = ACTIONS(2751),
    [sym_timeline_get] = ACTIONS(2751),
    [sym_timeline_active] = ACTIONS(2751),
    [sym_timeline_dynamic_id] = ACTIONS(2751),
    [sym_variable_to_string] = ACTIONS(2751),
    [sym_string_to_variable] = ACTIONS(2751),
    [sym_length_variable_to_string] = ACTIONS(2751),
    [sym_type_cast] = ACTIONS(2751),
    [sym_variable_to_xml] = ACTIONS(2751),
    [sym_xml_to_variable] = ACTIONS(2751),
    [sym_length_variable_to_xml] = ACTIONS(2751),
    [sym_redirect_string] = ACTIONS(2751),
    [sym_random_number] = ACTIONS(2751),
    [sym_get_unique_id] = ACTIONS(2751),
    [sym_get_system_number] = ACTIONS(2751),
    [sym_set_system_number] = ACTIONS(2751),
    [sym_get_serial_number] = ACTIONS(2751),
    [sym_get_master_build] = ACTIONS(2751),
    [sym_set_validation_code] = ACTIONS(2751),
    [sym_reboot] = ACTIONS(2751),
    [sym_load_duet_module] = ACTIONS(2751),
    [sym_unload_duet_module] = ACTIONS(2751),
    [sym_duet_mem_size_set] = ACTIONS(2751),
    [sym_duet_mem_size_get] = ACTIONS(2751),
    [sym_dynamic_application_device] = ACTIONS(2751),
    [sym_dynamic_polled_port] = ACTIONS(2751),
    [sym_static_port_binding] = ACTIONS(2751),
    [sym_static_ip_binding] = ACTIONS(2751),
    [sym_get_dns_list] = ACTIONS(2751),
    [sym_set_dns_list] = ACTIONS(2751),
    [sym_get_ip_address] = ACTIONS(2751),
    [sym_set_ip_address] = ACTIONS(2751),
    [sym_get_url_list] = ACTIONS(2751),
    [sym_add_url_entry] = ACTIONS(2751),
    [sym_delete_url_entry] = ACTIONS(2751),
    [sym_abs_value] = ACTIONS(2751),
    [sym_max_value] = ACTIONS(2751),
    [sym_min_value] = ACTIONS(2751),
    [sym_exp_value] = ACTIONS(2751),
    [sym_log_value] = ACTIONS(2751),
    [sym_log10_value] = ACTIONS(2751),
    [sym_power_value] = ACTIONS(2751),
    [sym_sqrt_value] = ACTIONS(2751),
    [sym_raw_be] = ACTIONS(2751),
    [sym_raw_le] = ACTIONS(2751),
    [sym_internal_queue_size_set] = ACTIONS(2751),
    [sym_internal_queue_size_get] = ACTIONS(2751),
    [sym_internal_threshold_set] = ACTIONS(2751),
    [sym_internal_threshold_get] = ACTIONS(2751),
    [sym_rebuilt_event] = ACTIONS(2751),
    [sym_do_custom_event] = ACTIONS(2751),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2751),
    [sym_clkmgr_set_clk_source] = ACTIONS(2751),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2751),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2751),
    [sym_clkmgr_get_timezone] = ACTIONS(2751),
    [sym_clkmgr_set_timezone] = ACTIONS(2751),
    [sym_clkmgr_get_resync_period] = ACTIONS(2751),
    [sym_clkmgr_set_resync_period] = ACTIONS(2751),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2751),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2751),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2751),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2751),
    [sym_clkmgr_get_timeservers] = ACTIONS(2751),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2751),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2751),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2751),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2751),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2751),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2751),
    [sym_set_log_level] = ACTIONS(2751),
    [sym_get_log_level] = ACTIONS(2751),
    [sym_amx_log] = ACTIONS(2751),
    [sym_smtp_server_config_set] = ACTIONS(2751),
    [sym_smtp_server_config_get] = ACTIONS(2751),
    [sym_smtp_send] = ACTIONS(2751),
    [sym_validate_netlinx_account] = ACTIONS(2751),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2751),
    [sym_authenticate_certificate] = ACTIONS(2751),
    [sym_audit_netlinx_session_event] = ACTIONS(2751),
    [sym_audit_netlinx_generic_event] = ACTIONS(2751),
    [sym_get_available_flash_disk_space] = ACTIONS(2751),
    [sym_get_max_flash_disk_space] = ACTIONS(2751),
    [sym_data_create_feed] = ACTIONS(2751),
    [sym_data_delete_feed] = ACTIONS(2751),
    [sym_data_publish_feed] = ACTIONS(2751),
    [sym_data_get_published_feed] = ACTIONS(2751),
    [sym_data_add_record] = ACTIONS(2751),
    [sym_data_get_event_record] = ACTIONS(2751),
    [sym__wc_data_create_feed] = ACTIONS(2751),
    [sym__wc_data_add_record] = ACTIONS(2751),
    [sym__wc_data_get_event_record] = ACTIONS(2751),
    [sym_format] = ACTIONS(2751),
    [sym_astro_clock] = ACTIONS(2751),
    [sym_set_outdoor_temperature] = ACTIONS(2751),
    [sym_set_virtual_level_count] = ACTIONS(2751),
    [sym_set_virtual_channel_count] = ACTIONS(2751),
    [sym_set_virtual_port_count] = ACTIONS(2751),
    [sym__wc_encode_int] = ACTIONS(2751),
    [sym__wc_decode_int] = ACTIONS(2751),
    [sym__wc_int] = ACTIONS(2751),
    [sym__wc_to_ch_int] = ACTIONS(2751),
    [sym__ch_to_wc_int] = ACTIONS(2751),
    [sym__wc_find_string_int] = ACTIONS(2751),
    [sym__wc_left_string_int] = ACTIONS(2751),
    [sym__wc_lower_string_int] = ACTIONS(2751),
    [sym__wc_mid_string_int] = ACTIONS(2751),
    [sym__wc_remove_string_int] = ACTIONS(2751),
    [sym__wc_right_string_int] = ACTIONS(2751),
    [sym__wc_upper_string_int] = ACTIONS(2751),
    [sym__wc_compare_string_int] = ACTIONS(2751),
    [sym__wc_get_buffer_char_int] = ACTIONS(2751),
    [sym__wc_get_buffer_string_int] = ACTIONS(2751),
    [sym__wc_concat_string_int] = ACTIONS(2751),
    [sym__wc_file_open_int] = ACTIONS(2751),
    [sym__wc_file_close_int] = ACTIONS(2751),
    [sym__wc_file_read_int] = ACTIONS(2751),
    [sym__wc_file_read_line_int] = ACTIONS(2751),
    [sym__wc_file_write_int] = ACTIONS(2751),
    [sym__wc_file_write_line_int] = ACTIONS(2751),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2377),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2377),
    [anon_sym_LBRACK] = ACTIONS(2377),
    [anon_sym_LBRACE] = ACTIONS(2377),
    [anon_sym_RBRACE] = ACTIONS(2377),
    [anon_sym_LPAREN] = ACTIONS(2751),
    [anon_sym_BANG] = ACTIONS(2377),
    [anon_sym_TILDE] = ACTIONS(2377),
    [anon_sym_DASH] = ACTIONS(2751),
    [anon_sym_PLUS] = ACTIONS(2751),
    [anon_sym_DASH_DASH] = ACTIONS(2377),
    [anon_sym_PLUS_PLUS] = ACTIONS(2377),
    [anon_sym_DQUOTE] = ACTIONS(2377),
    [anon_sym_SQUOTE] = ACTIONS(2377),
    [sym_number_literal] = ACTIONS(2377),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(921)] = {
    [sym_identifier] = ACTIONS(2753),
    [sym_not] = ACTIONS(2753),
    [sym_bnot] = ACTIONS(2753),
    [sym___file__] = ACTIONS(2753),
    [sym___date__] = ACTIONS(2753),
    [sym___ldate__] = ACTIONS(2753),
    [sym___time__] = ACTIONS(2753),
    [sym___line__] = ACTIONS(2753),
    [sym___name__] = ACTIONS(2753),
    [sym___netlinx__] = ACTIONS(2753),
    [sym_netlinx_axi_version] = ACTIONS(2753),
    [sym_url_flg_tcp] = ACTIONS(2753),
    [sym_url_flg_accinfopresent] = ACTIONS(2753),
    [sym_url_flg_temp] = ACTIONS(2753),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2753),
    [sym_url_flg_stat_mask] = ACTIONS(2753),
    [sym_url_flg_stat_lookup] = ACTIONS(2753),
    [sym_url_flg_stat_connecting] = ACTIONS(2753),
    [sym_url_flg_stat_waiting] = ACTIONS(2753),
    [sym_url_flg_stat_connected] = ACTIONS(2753),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2753),
    [sym_first_virtual_device] = ACTIONS(2753),
    [sym_first_local_port] = ACTIONS(2753),
    [sym_dynamic_virtual_device] = ACTIONS(2753),
    [sym_timeline_once] = ACTIONS(2753),
    [sym_timeline_repeat] = ACTIONS(2753),
    [sym_timeline_absolute] = ACTIONS(2753),
    [sym_timeline_relative] = ACTIONS(2753),
    [sym_do_push_timed_infinite] = ACTIONS(2753),
    [sym_source_type_no_address] = ACTIONS(2753),
    [sym_source_type_neuron_id] = ACTIONS(2753),
    [sym_source_type_ip_address] = ACTIONS(2753),
    [sym_source_type_axlink] = ACTIONS(2753),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2753),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2753),
    [sym_source_type_ip_socket_address] = ACTIONS(2753),
    [sym_source_type_rs232] = ACTIONS(2753),
    [sym_source_type_internal] = ACTIONS(2753),
    [sym_source_type_mac_address] = ACTIONS(2753),
    [sym_source_type_ipv4_port] = ACTIONS(2753),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2753),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2753),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2753),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2753),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2753),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2753),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2753),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2753),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2753),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2753),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2753),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2753),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2753),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2753),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2753),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2753),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2753),
    [sym_true] = ACTIONS(2753),
    [sym_false] = ACTIONS(2753),
    [sym_file_read_only] = ACTIONS(2753),
    [sym_file_rw_new] = ACTIONS(2753),
    [sym_file_rw_append] = ACTIONS(2753),
    [sym_ip_tcp] = ACTIONS(2753),
    [sym_ip_udp] = ACTIONS(2753),
    [sym_ip_udp_2way] = ACTIONS(2753),
    [sym_xml_encode_types] = ACTIONS(2753),
    [sym_xml_encode_char_as_list] = ACTIONS(2753),
    [sym_xml_encode_le] = ACTIONS(2753),
    [sym_xml_decode_types] = ACTIONS(2753),
    [sym_xml_decode_no_preserve] = ACTIONS(2753),
    [sym_duet_dev_type_display_device] = ACTIONS(2753),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2753),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2753),
    [sym_duet_dev_type_io_device] = ACTIONS(2753),
    [sym_duet_dev_type_relay_device] = ACTIONS(2753),
    [sym_duet_dev_type_ups] = ACTIONS(2753),
    [sym_duet_dev_type_amplifier] = ACTIONS(2753),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2753),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2753),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2753),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2753),
    [sym_duet_dev_type_camera] = ACTIONS(2753),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2753),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2753),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2753),
    [sym_duet_dev_type_dss] = ACTIONS(2753),
    [sym_duet_dev_type_dvr] = ACTIONS(2753),
    [sym_duet_dev_type_disc_device] = ACTIONS(2753),
    [sym_duet_dev_type_document_camera] = ACTIONS(2753),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2753),
    [sym_duet_dev_type_hvac] = ACTIONS(2753),
    [sym_duet_dev_type_keypad] = ACTIONS(2753),
    [sym_duet_dev_type_light] = ACTIONS(2753),
    [sym_duet_dev_type_monitor] = ACTIONS(2753),
    [sym_duet_dev_type_motor] = ACTIONS(2753),
    [sym_duet_dev_type_multi_window] = ACTIONS(2753),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2753),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2753),
    [sym_duet_dev_type_receiver] = ACTIONS(2753),
    [sym_duet_dev_type_security_system] = ACTIONS(2753),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2753),
    [sym_duet_dev_type_settop_box] = ACTIONS(2753),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2753),
    [sym_duet_dev_type_switcher] = ACTIONS(2753),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2753),
    [sym_duet_dev_type_tv] = ACTIONS(2753),
    [sym_duet_dev_type_utility] = ACTIONS(2753),
    [sym_duet_dev_type_vcr] = ACTIONS(2753),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2753),
    [sym_duet_dev_type_video_processor] = ACTIONS(2753),
    [sym_duet_dev_type_video_projector] = ACTIONS(2753),
    [sym_duet_dev_type_video_wall] = ACTIONS(2753),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2753),
    [sym_duet_dev_type_weather] = ACTIONS(2753),
    [sym_duet_dev_not_polled] = ACTIONS(2753),
    [sym_duet_dev_polled] = ACTIONS(2753),
    [sym_clkmgr_mode_network] = ACTIONS(2753),
    [sym_clkmgr_mode_standalone] = ACTIONS(2753),
    [sym_normal_standby] = ACTIONS(2753),
    [sym_normal_wake] = ACTIONS(2753),
    [sym_ip_multicast_ttl_option] = ACTIONS(2753),
    [sym_ip_ttl_subnet] = ACTIONS(2753),
    [sym_ip_ttl_site] = ACTIONS(2753),
    [sym_ip_ttl_region] = ACTIONS(2753),
    [sym_ip_ttl_continent] = ACTIONS(2753),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2753),
    [sym_ip_nodelay_on] = ACTIONS(2753),
    [sym_ip_nodelay_off] = ACTIONS(2753),
    [sym_amx_error] = ACTIONS(2753),
    [sym_amx_warning] = ACTIONS(2753),
    [sym_amx_info] = ACTIONS(2753),
    [sym_amx_debug] = ACTIONS(2753),
    [sym_netlinx_login_success] = ACTIONS(2753),
    [sym_netlinx_login_fail] = ACTIONS(2753),
    [sym_netlinx_logout] = ACTIONS(2753),
    [sym_null_str] = ACTIONS(2753),
    [sym_smtp_address] = ACTIONS(2753),
    [sym_smtp_port_number] = ACTIONS(2753),
    [sym_smtp_username] = ACTIONS(2753),
    [sym_smtp_password] = ACTIONS(2753),
    [sym_smtp_require_tls] = ACTIONS(2753),
    [sym_smtp_from] = ACTIONS(2753),
    [sym_smtp_tls_true] = ACTIONS(2753),
    [sym_smtp_tls_false] = ACTIONS(2753),
    [sym_listview_on_row_select_event] = ACTIONS(2753),
    [sym_data_structure_unknown] = ACTIONS(2753),
    [sym_data_structure_datafeed] = ACTIONS(2753),
    [sym_data_structure_datarecordset] = ACTIONS(2753),
    [sym_data_structure_datarecord] = ACTIONS(2753),
    [sym_data_structure_datafield] = ACTIONS(2753),
    [sym_data_max_content_field_count] = ACTIONS(2753),
    [sym_data_max_metadata_field_count] = ACTIONS(2753),
    [sym_data_max_id_length] = ACTIONS(2753),
    [sym_data_max_name_length] = ACTIONS(2753),
    [sym_data_max_description_length] = ACTIONS(2753),
    [sym_data_max_type_length] = ACTIONS(2753),
    [sym_data_max_format_length] = ACTIONS(2753),
    [sym_data_max_source_length] = ACTIONS(2753),
    [sym_data_max_value_length] = ACTIONS(2753),
    [sym_data_max_label_length] = ACTIONS(2753),
    [sym_data_max_url_length] = ACTIONS(2753),
    [sym_max_last_login_info_length] = ACTIONS(2753),
    [sym_data_type_unknown] = ACTIONS(2753),
    [sym_data_type_string] = ACTIONS(2753),
    [sym_data_type_datetime] = ACTIONS(2753),
    [sym_data_type_date] = ACTIONS(2753),
    [sym_data_type_time] = ACTIONS(2753),
    [sym_data_type_image] = ACTIONS(2753),
    [sym_data_format_url] = ACTIONS(2753),
    [sym_data_format_phone] = ACTIONS(2753),
    [sym_data_format_email] = ACTIONS(2753),
    [sym_data_format_iso8601] = ACTIONS(2753),
    [sym_auditlog_priv] = ACTIONS(2753),
    [sym_configuration_priv] = ACTIONS(2753),
    [sym_deviceconfig_priv] = ACTIONS(2753),
    [sym_ftp_priv] = ACTIONS(2753),
    [sym_http_priv] = ACTIONS(2753),
    [sym_networkconfig_priv] = ACTIONS(2753),
    [sym_programport_priv] = ACTIONS(2753),
    [sym_remoteui_priv] = ACTIONS(2753),
    [sym_securitycontrol_priv] = ACTIONS(2753),
    [sym_softwaremanagement_priv] = ACTIONS(2753),
    [sym_terminal_priv] = ACTIONS(2753),
    [sym_tpadmin_priv] = ACTIONS(2753),
    [sym_user1_priv] = ACTIONS(2753),
    [sym_user2_priv] = ACTIONS(2753),
    [sym_user3_priv] = ACTIONS(2753),
    [sym_user4_priv] = ACTIONS(2753),
    [sym_usermanagement_priv] = ACTIONS(2753),
    [sym_valid_account] = ACTIONS(2753),
    [sym_err_name_invalid] = ACTIONS(2753),
    [sym_err_password_invalid] = ACTIONS(2753),
    [sym_err_invalid_account] = ACTIONS(2753),
    [sym_err_no_authorization] = ACTIONS(2753),
    [sym_err_invalid_parameter] = ACTIONS(2753),
    [sym_err_user_locked_out] = ACTIONS(2753),
    [sym_err_user_account_expired] = ACTIONS(2753),
    [sym_err_general_error] = ACTIONS(2753),
    [sym_err_authentication_error] = ACTIONS(2753),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2753),
    [sym_tls_validate_certificate] = ACTIONS(2753),
    [sym____reserved___] = ACTIONS(2753),
    [sym_date] = ACTIONS(2753),
    [sym_day] = ACTIONS(2753),
    [sym_time] = ACTIONS(2753),
    [sym_ldate] = ACTIONS(2753),
    [sym_push_device] = ACTIONS(2753),
    [sym_push_channel] = ACTIONS(2753),
    [sym_push_devchan] = ACTIONS(2753),
    [sym_release_device] = ACTIONS(2753),
    [sym_release_channel] = ACTIONS(2753),
    [sym_release_devchan] = ACTIONS(2753),
    [sym_master_slot] = ACTIONS(2753),
    [sym_get_pulse_time] = ACTIONS(2753),
    [sym_get_timer] = ACTIONS(2753),
    [sym_master_sn] = ACTIONS(2753),
    [sym_system_number] = ACTIONS(2753),
    [sym_button] = ACTIONS(2753),
    [sym_dv_channel] = ACTIONS(2753),
    [sym_data] = ACTIONS(2753),
    [sym_level] = ACTIONS(2753),
    [sym_timeline] = ACTIONS(2753),
    [sym_channel] = ACTIONS(2753),
    [sym_custom] = ACTIONS(2753),
    [sym_atoi] = ACTIONS(2753),
    [sym_atol] = ACTIONS(2753),
    [sym_atof] = ACTIONS(2753),
    [sym_itoa] = ACTIONS(2753),
    [sym_itohex] = ACTIONS(2753),
    [sym_hextoi] = ACTIONS(2753),
    [sym_ftoa] = ACTIONS(2753),
    [sym_find_string] = ACTIONS(2753),
    [sym_left_string] = ACTIONS(2753),
    [sym_right_string] = ACTIONS(2753),
    [sym_mid_string] = ACTIONS(2753),
    [sym_remove_string] = ACTIONS(2753),
    [sym_get_buffer_string] = ACTIONS(2753),
    [sym_get_buffer_char] = ACTIONS(2753),
    [sym_get_multi_buffer_string] = ACTIONS(2753),
    [sym_length_string] = ACTIONS(2753),
    [sym_max_length_string] = ACTIONS(2753),
    [sym_set_length_string] = ACTIONS(2753),
    [sym_compare_string] = ACTIONS(2753),
    [sym_upper_string] = ACTIONS(2753),
    [sym_lower_string] = ACTIONS(2753),
    [sym_get_last] = ACTIONS(2753),
    [sym_date_to_day] = ACTIONS(2753),
    [sym_date_to_month] = ACTIONS(2753),
    [sym_date_to_year] = ACTIONS(2753),
    [sym_time_to_hour] = ACTIONS(2753),
    [sym_time_to_minute] = ACTIONS(2753),
    [sym_time_to_second] = ACTIONS(2753),
    [sym_day_of_week] = ACTIONS(2753),
    [sym_set_pulse_time] = ACTIONS(2753),
    [sym_set_timer] = ACTIONS(2753),
    [sym_file_open] = ACTIONS(2753),
    [sym_file_close] = ACTIONS(2753),
    [sym_file_read] = ACTIONS(2753),
    [sym_file_read_line] = ACTIONS(2753),
    [sym_file_write] = ACTIONS(2753),
    [sym_file_write_line] = ACTIONS(2753),
    [sym_file_copy] = ACTIONS(2753),
    [sym_file_delete] = ACTIONS(2753),
    [sym_file_rename] = ACTIONS(2753),
    [sym_file_dir] = ACTIONS(2753),
    [sym_file_getdir] = ACTIONS(2753),
    [sym_file_setdir] = ACTIONS(2753),
    [sym_file_create_dir] = ACTIONS(2753),
    [sym_file_remove_dir] = ACTIONS(2753),
    [sym_file_seek] = ACTIONS(2753),
    [sym_length_array] = ACTIONS(2753),
    [sym_max_length_array] = ACTIONS(2753),
    [sym_set_length_array] = ACTIONS(2753),
    [sym_ip_client_open] = ACTIONS(2753),
    [sym_ip_client_close] = ACTIONS(2753),
    [sym_ip_server_open] = ACTIONS(2753),
    [sym_ip_server_close] = ACTIONS(2753),
    [sym_ip_bound_client_open] = ACTIONS(2753),
    [sym_ip_mc_server_open] = ACTIONS(2753),
    [sym_ip_set_option] = ACTIONS(2753),
    [sym_ssh_client_open] = ACTIONS(2753),
    [sym_ssh_client_close] = ACTIONS(2753),
    [sym_tls_client_open] = ACTIONS(2753),
    [sym_tls_client_close] = ACTIONS(2753),
    [sym_do_push] = ACTIONS(2753),
    [sym_do_push_timed] = ACTIONS(2753),
    [sym_do_release] = ACTIONS(2753),
    [sym_device_id] = ACTIONS(2753),
    [sym_device_info] = ACTIONS(2753),
    [sym_device_id_string] = ACTIONS(2753),
    [sym_device_standby] = ACTIONS(2753),
    [sym_device_wake] = ACTIONS(2753),
    [sym_timeline_create] = ACTIONS(2753),
    [sym_timeline_kill] = ACTIONS(2753),
    [sym_timeline_reload] = ACTIONS(2753),
    [sym_timeline_pause] = ACTIONS(2753),
    [sym_timeline_restart] = ACTIONS(2753),
    [sym_timeline_set] = ACTIONS(2753),
    [sym_timeline_get] = ACTIONS(2753),
    [sym_timeline_active] = ACTIONS(2753),
    [sym_timeline_dynamic_id] = ACTIONS(2753),
    [sym_variable_to_string] = ACTIONS(2753),
    [sym_string_to_variable] = ACTIONS(2753),
    [sym_length_variable_to_string] = ACTIONS(2753),
    [sym_type_cast] = ACTIONS(2753),
    [sym_variable_to_xml] = ACTIONS(2753),
    [sym_xml_to_variable] = ACTIONS(2753),
    [sym_length_variable_to_xml] = ACTIONS(2753),
    [sym_redirect_string] = ACTIONS(2753),
    [sym_random_number] = ACTIONS(2753),
    [sym_get_unique_id] = ACTIONS(2753),
    [sym_get_system_number] = ACTIONS(2753),
    [sym_set_system_number] = ACTIONS(2753),
    [sym_get_serial_number] = ACTIONS(2753),
    [sym_get_master_build] = ACTIONS(2753),
    [sym_set_validation_code] = ACTIONS(2753),
    [sym_reboot] = ACTIONS(2753),
    [sym_load_duet_module] = ACTIONS(2753),
    [sym_unload_duet_module] = ACTIONS(2753),
    [sym_duet_mem_size_set] = ACTIONS(2753),
    [sym_duet_mem_size_get] = ACTIONS(2753),
    [sym_dynamic_application_device] = ACTIONS(2753),
    [sym_dynamic_polled_port] = ACTIONS(2753),
    [sym_static_port_binding] = ACTIONS(2753),
    [sym_static_ip_binding] = ACTIONS(2753),
    [sym_get_dns_list] = ACTIONS(2753),
    [sym_set_dns_list] = ACTIONS(2753),
    [sym_get_ip_address] = ACTIONS(2753),
    [sym_set_ip_address] = ACTIONS(2753),
    [sym_get_url_list] = ACTIONS(2753),
    [sym_add_url_entry] = ACTIONS(2753),
    [sym_delete_url_entry] = ACTIONS(2753),
    [sym_abs_value] = ACTIONS(2753),
    [sym_max_value] = ACTIONS(2753),
    [sym_min_value] = ACTIONS(2753),
    [sym_exp_value] = ACTIONS(2753),
    [sym_log_value] = ACTIONS(2753),
    [sym_log10_value] = ACTIONS(2753),
    [sym_power_value] = ACTIONS(2753),
    [sym_sqrt_value] = ACTIONS(2753),
    [sym_raw_be] = ACTIONS(2753),
    [sym_raw_le] = ACTIONS(2753),
    [sym_internal_queue_size_set] = ACTIONS(2753),
    [sym_internal_queue_size_get] = ACTIONS(2753),
    [sym_internal_threshold_set] = ACTIONS(2753),
    [sym_internal_threshold_get] = ACTIONS(2753),
    [sym_rebuilt_event] = ACTIONS(2753),
    [sym_do_custom_event] = ACTIONS(2753),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2753),
    [sym_clkmgr_set_clk_source] = ACTIONS(2753),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2753),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2753),
    [sym_clkmgr_get_timezone] = ACTIONS(2753),
    [sym_clkmgr_set_timezone] = ACTIONS(2753),
    [sym_clkmgr_get_resync_period] = ACTIONS(2753),
    [sym_clkmgr_set_resync_period] = ACTIONS(2753),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2753),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2753),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2753),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2753),
    [sym_clkmgr_get_timeservers] = ACTIONS(2753),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2753),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2753),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2753),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2753),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2753),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2753),
    [sym_set_log_level] = ACTIONS(2753),
    [sym_get_log_level] = ACTIONS(2753),
    [sym_amx_log] = ACTIONS(2753),
    [sym_smtp_server_config_set] = ACTIONS(2753),
    [sym_smtp_server_config_get] = ACTIONS(2753),
    [sym_smtp_send] = ACTIONS(2753),
    [sym_validate_netlinx_account] = ACTIONS(2753),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2753),
    [sym_authenticate_certificate] = ACTIONS(2753),
    [sym_audit_netlinx_session_event] = ACTIONS(2753),
    [sym_audit_netlinx_generic_event] = ACTIONS(2753),
    [sym_get_available_flash_disk_space] = ACTIONS(2753),
    [sym_get_max_flash_disk_space] = ACTIONS(2753),
    [sym_data_create_feed] = ACTIONS(2753),
    [sym_data_delete_feed] = ACTIONS(2753),
    [sym_data_publish_feed] = ACTIONS(2753),
    [sym_data_get_published_feed] = ACTIONS(2753),
    [sym_data_add_record] = ACTIONS(2753),
    [sym_data_get_event_record] = ACTIONS(2753),
    [sym__wc_data_create_feed] = ACTIONS(2753),
    [sym__wc_data_add_record] = ACTIONS(2753),
    [sym__wc_data_get_event_record] = ACTIONS(2753),
    [sym_format] = ACTIONS(2753),
    [sym_astro_clock] = ACTIONS(2753),
    [sym_set_outdoor_temperature] = ACTIONS(2753),
    [sym_set_virtual_level_count] = ACTIONS(2753),
    [sym_set_virtual_channel_count] = ACTIONS(2753),
    [sym_set_virtual_port_count] = ACTIONS(2753),
    [sym__wc_encode_int] = ACTIONS(2753),
    [sym__wc_decode_int] = ACTIONS(2753),
    [sym__wc_int] = ACTIONS(2753),
    [sym__wc_to_ch_int] = ACTIONS(2753),
    [sym__ch_to_wc_int] = ACTIONS(2753),
    [sym__wc_find_string_int] = ACTIONS(2753),
    [sym__wc_left_string_int] = ACTIONS(2753),
    [sym__wc_lower_string_int] = ACTIONS(2753),
    [sym__wc_mid_string_int] = ACTIONS(2753),
    [sym__wc_remove_string_int] = ACTIONS(2753),
    [sym__wc_right_string_int] = ACTIONS(2753),
    [sym__wc_upper_string_int] = ACTIONS(2753),
    [sym__wc_compare_string_int] = ACTIONS(2753),
    [sym__wc_get_buffer_char_int] = ACTIONS(2753),
    [sym__wc_get_buffer_string_int] = ACTIONS(2753),
    [sym__wc_concat_string_int] = ACTIONS(2753),
    [sym__wc_file_open_int] = ACTIONS(2753),
    [sym__wc_file_close_int] = ACTIONS(2753),
    [sym__wc_file_read_int] = ACTIONS(2753),
    [sym__wc_file_read_line_int] = ACTIONS(2753),
    [sym__wc_file_write_int] = ACTIONS(2753),
    [sym__wc_file_write_line_int] = ACTIONS(2753),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2755),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2755),
    [anon_sym_LBRACK] = ACTIONS(2755),
    [anon_sym_LBRACE] = ACTIONS(2755),
    [anon_sym_RBRACE] = ACTIONS(2755),
    [anon_sym_LPAREN] = ACTIONS(2753),
    [anon_sym_BANG] = ACTIONS(2755),
    [anon_sym_TILDE] = ACTIONS(2755),
    [anon_sym_DASH] = ACTIONS(2753),
    [anon_sym_PLUS] = ACTIONS(2753),
    [anon_sym_DASH_DASH] = ACTIONS(2755),
    [anon_sym_PLUS_PLUS] = ACTIONS(2755),
    [anon_sym_DQUOTE] = ACTIONS(2755),
    [anon_sym_SQUOTE] = ACTIONS(2755),
    [sym_number_literal] = ACTIONS(2755),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(922)] = {
    [sym_identifier] = ACTIONS(2757),
    [sym_not] = ACTIONS(2757),
    [sym_bnot] = ACTIONS(2757),
    [sym___file__] = ACTIONS(2757),
    [sym___date__] = ACTIONS(2757),
    [sym___ldate__] = ACTIONS(2757),
    [sym___time__] = ACTIONS(2757),
    [sym___line__] = ACTIONS(2757),
    [sym___name__] = ACTIONS(2757),
    [sym___netlinx__] = ACTIONS(2757),
    [sym_netlinx_axi_version] = ACTIONS(2757),
    [sym_url_flg_tcp] = ACTIONS(2757),
    [sym_url_flg_accinfopresent] = ACTIONS(2757),
    [sym_url_flg_temp] = ACTIONS(2757),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2757),
    [sym_url_flg_stat_mask] = ACTIONS(2757),
    [sym_url_flg_stat_lookup] = ACTIONS(2757),
    [sym_url_flg_stat_connecting] = ACTIONS(2757),
    [sym_url_flg_stat_waiting] = ACTIONS(2757),
    [sym_url_flg_stat_connected] = ACTIONS(2757),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2757),
    [sym_first_virtual_device] = ACTIONS(2757),
    [sym_first_local_port] = ACTIONS(2757),
    [sym_dynamic_virtual_device] = ACTIONS(2757),
    [sym_timeline_once] = ACTIONS(2757),
    [sym_timeline_repeat] = ACTIONS(2757),
    [sym_timeline_absolute] = ACTIONS(2757),
    [sym_timeline_relative] = ACTIONS(2757),
    [sym_do_push_timed_infinite] = ACTIONS(2757),
    [sym_source_type_no_address] = ACTIONS(2757),
    [sym_source_type_neuron_id] = ACTIONS(2757),
    [sym_source_type_ip_address] = ACTIONS(2757),
    [sym_source_type_axlink] = ACTIONS(2757),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2757),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2757),
    [sym_source_type_ip_socket_address] = ACTIONS(2757),
    [sym_source_type_rs232] = ACTIONS(2757),
    [sym_source_type_internal] = ACTIONS(2757),
    [sym_source_type_mac_address] = ACTIONS(2757),
    [sym_source_type_ipv4_port] = ACTIONS(2757),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2757),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2757),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2757),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2757),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2757),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2757),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2757),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2757),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2757),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2757),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2757),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2757),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2757),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2757),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2757),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2757),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2757),
    [sym_true] = ACTIONS(2757),
    [sym_false] = ACTIONS(2757),
    [sym_file_read_only] = ACTIONS(2757),
    [sym_file_rw_new] = ACTIONS(2757),
    [sym_file_rw_append] = ACTIONS(2757),
    [sym_ip_tcp] = ACTIONS(2757),
    [sym_ip_udp] = ACTIONS(2757),
    [sym_ip_udp_2way] = ACTIONS(2757),
    [sym_xml_encode_types] = ACTIONS(2757),
    [sym_xml_encode_char_as_list] = ACTIONS(2757),
    [sym_xml_encode_le] = ACTIONS(2757),
    [sym_xml_decode_types] = ACTIONS(2757),
    [sym_xml_decode_no_preserve] = ACTIONS(2757),
    [sym_duet_dev_type_display_device] = ACTIONS(2757),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2757),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2757),
    [sym_duet_dev_type_io_device] = ACTIONS(2757),
    [sym_duet_dev_type_relay_device] = ACTIONS(2757),
    [sym_duet_dev_type_ups] = ACTIONS(2757),
    [sym_duet_dev_type_amplifier] = ACTIONS(2757),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2757),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2757),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2757),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2757),
    [sym_duet_dev_type_camera] = ACTIONS(2757),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2757),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2757),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2757),
    [sym_duet_dev_type_dss] = ACTIONS(2757),
    [sym_duet_dev_type_dvr] = ACTIONS(2757),
    [sym_duet_dev_type_disc_device] = ACTIONS(2757),
    [sym_duet_dev_type_document_camera] = ACTIONS(2757),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2757),
    [sym_duet_dev_type_hvac] = ACTIONS(2757),
    [sym_duet_dev_type_keypad] = ACTIONS(2757),
    [sym_duet_dev_type_light] = ACTIONS(2757),
    [sym_duet_dev_type_monitor] = ACTIONS(2757),
    [sym_duet_dev_type_motor] = ACTIONS(2757),
    [sym_duet_dev_type_multi_window] = ACTIONS(2757),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2757),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2757),
    [sym_duet_dev_type_receiver] = ACTIONS(2757),
    [sym_duet_dev_type_security_system] = ACTIONS(2757),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2757),
    [sym_duet_dev_type_settop_box] = ACTIONS(2757),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2757),
    [sym_duet_dev_type_switcher] = ACTIONS(2757),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2757),
    [sym_duet_dev_type_tv] = ACTIONS(2757),
    [sym_duet_dev_type_utility] = ACTIONS(2757),
    [sym_duet_dev_type_vcr] = ACTIONS(2757),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2757),
    [sym_duet_dev_type_video_processor] = ACTIONS(2757),
    [sym_duet_dev_type_video_projector] = ACTIONS(2757),
    [sym_duet_dev_type_video_wall] = ACTIONS(2757),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2757),
    [sym_duet_dev_type_weather] = ACTIONS(2757),
    [sym_duet_dev_not_polled] = ACTIONS(2757),
    [sym_duet_dev_polled] = ACTIONS(2757),
    [sym_clkmgr_mode_network] = ACTIONS(2757),
    [sym_clkmgr_mode_standalone] = ACTIONS(2757),
    [sym_normal_standby] = ACTIONS(2757),
    [sym_normal_wake] = ACTIONS(2757),
    [sym_ip_multicast_ttl_option] = ACTIONS(2757),
    [sym_ip_ttl_subnet] = ACTIONS(2757),
    [sym_ip_ttl_site] = ACTIONS(2757),
    [sym_ip_ttl_region] = ACTIONS(2757),
    [sym_ip_ttl_continent] = ACTIONS(2757),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2757),
    [sym_ip_nodelay_on] = ACTIONS(2757),
    [sym_ip_nodelay_off] = ACTIONS(2757),
    [sym_amx_error] = ACTIONS(2757),
    [sym_amx_warning] = ACTIONS(2757),
    [sym_amx_info] = ACTIONS(2757),
    [sym_amx_debug] = ACTIONS(2757),
    [sym_netlinx_login_success] = ACTIONS(2757),
    [sym_netlinx_login_fail] = ACTIONS(2757),
    [sym_netlinx_logout] = ACTIONS(2757),
    [sym_null_str] = ACTIONS(2757),
    [sym_smtp_address] = ACTIONS(2757),
    [sym_smtp_port_number] = ACTIONS(2757),
    [sym_smtp_username] = ACTIONS(2757),
    [sym_smtp_password] = ACTIONS(2757),
    [sym_smtp_require_tls] = ACTIONS(2757),
    [sym_smtp_from] = ACTIONS(2757),
    [sym_smtp_tls_true] = ACTIONS(2757),
    [sym_smtp_tls_false] = ACTIONS(2757),
    [sym_listview_on_row_select_event] = ACTIONS(2757),
    [sym_data_structure_unknown] = ACTIONS(2757),
    [sym_data_structure_datafeed] = ACTIONS(2757),
    [sym_data_structure_datarecordset] = ACTIONS(2757),
    [sym_data_structure_datarecord] = ACTIONS(2757),
    [sym_data_structure_datafield] = ACTIONS(2757),
    [sym_data_max_content_field_count] = ACTIONS(2757),
    [sym_data_max_metadata_field_count] = ACTIONS(2757),
    [sym_data_max_id_length] = ACTIONS(2757),
    [sym_data_max_name_length] = ACTIONS(2757),
    [sym_data_max_description_length] = ACTIONS(2757),
    [sym_data_max_type_length] = ACTIONS(2757),
    [sym_data_max_format_length] = ACTIONS(2757),
    [sym_data_max_source_length] = ACTIONS(2757),
    [sym_data_max_value_length] = ACTIONS(2757),
    [sym_data_max_label_length] = ACTIONS(2757),
    [sym_data_max_url_length] = ACTIONS(2757),
    [sym_max_last_login_info_length] = ACTIONS(2757),
    [sym_data_type_unknown] = ACTIONS(2757),
    [sym_data_type_string] = ACTIONS(2757),
    [sym_data_type_datetime] = ACTIONS(2757),
    [sym_data_type_date] = ACTIONS(2757),
    [sym_data_type_time] = ACTIONS(2757),
    [sym_data_type_image] = ACTIONS(2757),
    [sym_data_format_url] = ACTIONS(2757),
    [sym_data_format_phone] = ACTIONS(2757),
    [sym_data_format_email] = ACTIONS(2757),
    [sym_data_format_iso8601] = ACTIONS(2757),
    [sym_auditlog_priv] = ACTIONS(2757),
    [sym_configuration_priv] = ACTIONS(2757),
    [sym_deviceconfig_priv] = ACTIONS(2757),
    [sym_ftp_priv] = ACTIONS(2757),
    [sym_http_priv] = ACTIONS(2757),
    [sym_networkconfig_priv] = ACTIONS(2757),
    [sym_programport_priv] = ACTIONS(2757),
    [sym_remoteui_priv] = ACTIONS(2757),
    [sym_securitycontrol_priv] = ACTIONS(2757),
    [sym_softwaremanagement_priv] = ACTIONS(2757),
    [sym_terminal_priv] = ACTIONS(2757),
    [sym_tpadmin_priv] = ACTIONS(2757),
    [sym_user1_priv] = ACTIONS(2757),
    [sym_user2_priv] = ACTIONS(2757),
    [sym_user3_priv] = ACTIONS(2757),
    [sym_user4_priv] = ACTIONS(2757),
    [sym_usermanagement_priv] = ACTIONS(2757),
    [sym_valid_account] = ACTIONS(2757),
    [sym_err_name_invalid] = ACTIONS(2757),
    [sym_err_password_invalid] = ACTIONS(2757),
    [sym_err_invalid_account] = ACTIONS(2757),
    [sym_err_no_authorization] = ACTIONS(2757),
    [sym_err_invalid_parameter] = ACTIONS(2757),
    [sym_err_user_locked_out] = ACTIONS(2757),
    [sym_err_user_account_expired] = ACTIONS(2757),
    [sym_err_general_error] = ACTIONS(2757),
    [sym_err_authentication_error] = ACTIONS(2757),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2757),
    [sym_tls_validate_certificate] = ACTIONS(2757),
    [sym____reserved___] = ACTIONS(2757),
    [sym_date] = ACTIONS(2757),
    [sym_day] = ACTIONS(2757),
    [sym_time] = ACTIONS(2757),
    [sym_ldate] = ACTIONS(2757),
    [sym_push_device] = ACTIONS(2757),
    [sym_push_channel] = ACTIONS(2757),
    [sym_push_devchan] = ACTIONS(2757),
    [sym_release_device] = ACTIONS(2757),
    [sym_release_channel] = ACTIONS(2757),
    [sym_release_devchan] = ACTIONS(2757),
    [sym_master_slot] = ACTIONS(2757),
    [sym_get_pulse_time] = ACTIONS(2757),
    [sym_get_timer] = ACTIONS(2757),
    [sym_master_sn] = ACTIONS(2757),
    [sym_system_number] = ACTIONS(2757),
    [sym_button] = ACTIONS(2757),
    [sym_dv_channel] = ACTIONS(2757),
    [sym_data] = ACTIONS(2757),
    [sym_level] = ACTIONS(2757),
    [sym_timeline] = ACTIONS(2757),
    [sym_channel] = ACTIONS(2757),
    [sym_custom] = ACTIONS(2757),
    [sym_atoi] = ACTIONS(2757),
    [sym_atol] = ACTIONS(2757),
    [sym_atof] = ACTIONS(2757),
    [sym_itoa] = ACTIONS(2757),
    [sym_itohex] = ACTIONS(2757),
    [sym_hextoi] = ACTIONS(2757),
    [sym_ftoa] = ACTIONS(2757),
    [sym_find_string] = ACTIONS(2757),
    [sym_left_string] = ACTIONS(2757),
    [sym_right_string] = ACTIONS(2757),
    [sym_mid_string] = ACTIONS(2757),
    [sym_remove_string] = ACTIONS(2757),
    [sym_get_buffer_string] = ACTIONS(2757),
    [sym_get_buffer_char] = ACTIONS(2757),
    [sym_get_multi_buffer_string] = ACTIONS(2757),
    [sym_length_string] = ACTIONS(2757),
    [sym_max_length_string] = ACTIONS(2757),
    [sym_set_length_string] = ACTIONS(2757),
    [sym_compare_string] = ACTIONS(2757),
    [sym_upper_string] = ACTIONS(2757),
    [sym_lower_string] = ACTIONS(2757),
    [sym_get_last] = ACTIONS(2757),
    [sym_date_to_day] = ACTIONS(2757),
    [sym_date_to_month] = ACTIONS(2757),
    [sym_date_to_year] = ACTIONS(2757),
    [sym_time_to_hour] = ACTIONS(2757),
    [sym_time_to_minute] = ACTIONS(2757),
    [sym_time_to_second] = ACTIONS(2757),
    [sym_day_of_week] = ACTIONS(2757),
    [sym_set_pulse_time] = ACTIONS(2757),
    [sym_set_timer] = ACTIONS(2757),
    [sym_file_open] = ACTIONS(2757),
    [sym_file_close] = ACTIONS(2757),
    [sym_file_read] = ACTIONS(2757),
    [sym_file_read_line] = ACTIONS(2757),
    [sym_file_write] = ACTIONS(2757),
    [sym_file_write_line] = ACTIONS(2757),
    [sym_file_copy] = ACTIONS(2757),
    [sym_file_delete] = ACTIONS(2757),
    [sym_file_rename] = ACTIONS(2757),
    [sym_file_dir] = ACTIONS(2757),
    [sym_file_getdir] = ACTIONS(2757),
    [sym_file_setdir] = ACTIONS(2757),
    [sym_file_create_dir] = ACTIONS(2757),
    [sym_file_remove_dir] = ACTIONS(2757),
    [sym_file_seek] = ACTIONS(2757),
    [sym_length_array] = ACTIONS(2757),
    [sym_max_length_array] = ACTIONS(2757),
    [sym_set_length_array] = ACTIONS(2757),
    [sym_ip_client_open] = ACTIONS(2757),
    [sym_ip_client_close] = ACTIONS(2757),
    [sym_ip_server_open] = ACTIONS(2757),
    [sym_ip_server_close] = ACTIONS(2757),
    [sym_ip_bound_client_open] = ACTIONS(2757),
    [sym_ip_mc_server_open] = ACTIONS(2757),
    [sym_ip_set_option] = ACTIONS(2757),
    [sym_ssh_client_open] = ACTIONS(2757),
    [sym_ssh_client_close] = ACTIONS(2757),
    [sym_tls_client_open] = ACTIONS(2757),
    [sym_tls_client_close] = ACTIONS(2757),
    [sym_do_push] = ACTIONS(2757),
    [sym_do_push_timed] = ACTIONS(2757),
    [sym_do_release] = ACTIONS(2757),
    [sym_device_id] = ACTIONS(2757),
    [sym_device_info] = ACTIONS(2757),
    [sym_device_id_string] = ACTIONS(2757),
    [sym_device_standby] = ACTIONS(2757),
    [sym_device_wake] = ACTIONS(2757),
    [sym_timeline_create] = ACTIONS(2757),
    [sym_timeline_kill] = ACTIONS(2757),
    [sym_timeline_reload] = ACTIONS(2757),
    [sym_timeline_pause] = ACTIONS(2757),
    [sym_timeline_restart] = ACTIONS(2757),
    [sym_timeline_set] = ACTIONS(2757),
    [sym_timeline_get] = ACTIONS(2757),
    [sym_timeline_active] = ACTIONS(2757),
    [sym_timeline_dynamic_id] = ACTIONS(2757),
    [sym_variable_to_string] = ACTIONS(2757),
    [sym_string_to_variable] = ACTIONS(2757),
    [sym_length_variable_to_string] = ACTIONS(2757),
    [sym_type_cast] = ACTIONS(2757),
    [sym_variable_to_xml] = ACTIONS(2757),
    [sym_xml_to_variable] = ACTIONS(2757),
    [sym_length_variable_to_xml] = ACTIONS(2757),
    [sym_redirect_string] = ACTIONS(2757),
    [sym_random_number] = ACTIONS(2757),
    [sym_get_unique_id] = ACTIONS(2757),
    [sym_get_system_number] = ACTIONS(2757),
    [sym_set_system_number] = ACTIONS(2757),
    [sym_get_serial_number] = ACTIONS(2757),
    [sym_get_master_build] = ACTIONS(2757),
    [sym_set_validation_code] = ACTIONS(2757),
    [sym_reboot] = ACTIONS(2757),
    [sym_load_duet_module] = ACTIONS(2757),
    [sym_unload_duet_module] = ACTIONS(2757),
    [sym_duet_mem_size_set] = ACTIONS(2757),
    [sym_duet_mem_size_get] = ACTIONS(2757),
    [sym_dynamic_application_device] = ACTIONS(2757),
    [sym_dynamic_polled_port] = ACTIONS(2757),
    [sym_static_port_binding] = ACTIONS(2757),
    [sym_static_ip_binding] = ACTIONS(2757),
    [sym_get_dns_list] = ACTIONS(2757),
    [sym_set_dns_list] = ACTIONS(2757),
    [sym_get_ip_address] = ACTIONS(2757),
    [sym_set_ip_address] = ACTIONS(2757),
    [sym_get_url_list] = ACTIONS(2757),
    [sym_add_url_entry] = ACTIONS(2757),
    [sym_delete_url_entry] = ACTIONS(2757),
    [sym_abs_value] = ACTIONS(2757),
    [sym_max_value] = ACTIONS(2757),
    [sym_min_value] = ACTIONS(2757),
    [sym_exp_value] = ACTIONS(2757),
    [sym_log_value] = ACTIONS(2757),
    [sym_log10_value] = ACTIONS(2757),
    [sym_power_value] = ACTIONS(2757),
    [sym_sqrt_value] = ACTIONS(2757),
    [sym_raw_be] = ACTIONS(2757),
    [sym_raw_le] = ACTIONS(2757),
    [sym_internal_queue_size_set] = ACTIONS(2757),
    [sym_internal_queue_size_get] = ACTIONS(2757),
    [sym_internal_threshold_set] = ACTIONS(2757),
    [sym_internal_threshold_get] = ACTIONS(2757),
    [sym_rebuilt_event] = ACTIONS(2757),
    [sym_do_custom_event] = ACTIONS(2757),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2757),
    [sym_clkmgr_set_clk_source] = ACTIONS(2757),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2757),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2757),
    [sym_clkmgr_get_timezone] = ACTIONS(2757),
    [sym_clkmgr_set_timezone] = ACTIONS(2757),
    [sym_clkmgr_get_resync_period] = ACTIONS(2757),
    [sym_clkmgr_set_resync_period] = ACTIONS(2757),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2757),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2757),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2757),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2757),
    [sym_clkmgr_get_timeservers] = ACTIONS(2757),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2757),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2757),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2757),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2757),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2757),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2757),
    [sym_set_log_level] = ACTIONS(2757),
    [sym_get_log_level] = ACTIONS(2757),
    [sym_amx_log] = ACTIONS(2757),
    [sym_smtp_server_config_set] = ACTIONS(2757),
    [sym_smtp_server_config_get] = ACTIONS(2757),
    [sym_smtp_send] = ACTIONS(2757),
    [sym_validate_netlinx_account] = ACTIONS(2757),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2757),
    [sym_authenticate_certificate] = ACTIONS(2757),
    [sym_audit_netlinx_session_event] = ACTIONS(2757),
    [sym_audit_netlinx_generic_event] = ACTIONS(2757),
    [sym_get_available_flash_disk_space] = ACTIONS(2757),
    [sym_get_max_flash_disk_space] = ACTIONS(2757),
    [sym_data_create_feed] = ACTIONS(2757),
    [sym_data_delete_feed] = ACTIONS(2757),
    [sym_data_publish_feed] = ACTIONS(2757),
    [sym_data_get_published_feed] = ACTIONS(2757),
    [sym_data_add_record] = ACTIONS(2757),
    [sym_data_get_event_record] = ACTIONS(2757),
    [sym__wc_data_create_feed] = ACTIONS(2757),
    [sym__wc_data_add_record] = ACTIONS(2757),
    [sym__wc_data_get_event_record] = ACTIONS(2757),
    [sym_format] = ACTIONS(2757),
    [sym_astro_clock] = ACTIONS(2757),
    [sym_set_outdoor_temperature] = ACTIONS(2757),
    [sym_set_virtual_level_count] = ACTIONS(2757),
    [sym_set_virtual_channel_count] = ACTIONS(2757),
    [sym_set_virtual_port_count] = ACTIONS(2757),
    [sym__wc_encode_int] = ACTIONS(2757),
    [sym__wc_decode_int] = ACTIONS(2757),
    [sym__wc_int] = ACTIONS(2757),
    [sym__wc_to_ch_int] = ACTIONS(2757),
    [sym__ch_to_wc_int] = ACTIONS(2757),
    [sym__wc_find_string_int] = ACTIONS(2757),
    [sym__wc_left_string_int] = ACTIONS(2757),
    [sym__wc_lower_string_int] = ACTIONS(2757),
    [sym__wc_mid_string_int] = ACTIONS(2757),
    [sym__wc_remove_string_int] = ACTIONS(2757),
    [sym__wc_right_string_int] = ACTIONS(2757),
    [sym__wc_upper_string_int] = ACTIONS(2757),
    [sym__wc_compare_string_int] = ACTIONS(2757),
    [sym__wc_get_buffer_char_int] = ACTIONS(2757),
    [sym__wc_get_buffer_string_int] = ACTIONS(2757),
    [sym__wc_concat_string_int] = ACTIONS(2757),
    [sym__wc_file_open_int] = ACTIONS(2757),
    [sym__wc_file_close_int] = ACTIONS(2757),
    [sym__wc_file_read_int] = ACTIONS(2757),
    [sym__wc_file_read_line_int] = ACTIONS(2757),
    [sym__wc_file_write_int] = ACTIONS(2757),
    [sym__wc_file_write_line_int] = ACTIONS(2757),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2759),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2759),
    [anon_sym_LBRACK] = ACTIONS(2759),
    [anon_sym_LBRACE] = ACTIONS(2759),
    [anon_sym_RBRACE] = ACTIONS(2759),
    [anon_sym_LPAREN] = ACTIONS(2757),
    [anon_sym_BANG] = ACTIONS(2759),
    [anon_sym_TILDE] = ACTIONS(2759),
    [anon_sym_DASH] = ACTIONS(2757),
    [anon_sym_PLUS] = ACTIONS(2757),
    [anon_sym_DASH_DASH] = ACTIONS(2759),
    [anon_sym_PLUS_PLUS] = ACTIONS(2759),
    [anon_sym_DQUOTE] = ACTIONS(2759),
    [anon_sym_SQUOTE] = ACTIONS(2759),
    [sym_number_literal] = ACTIONS(2759),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(923)] = {
    [sym_identifier] = ACTIONS(2761),
    [sym_not] = ACTIONS(2761),
    [sym_bnot] = ACTIONS(2761),
    [sym___file__] = ACTIONS(2761),
    [sym___date__] = ACTIONS(2761),
    [sym___ldate__] = ACTIONS(2761),
    [sym___time__] = ACTIONS(2761),
    [sym___line__] = ACTIONS(2761),
    [sym___name__] = ACTIONS(2761),
    [sym___netlinx__] = ACTIONS(2761),
    [sym_netlinx_axi_version] = ACTIONS(2761),
    [sym_url_flg_tcp] = ACTIONS(2761),
    [sym_url_flg_accinfopresent] = ACTIONS(2761),
    [sym_url_flg_temp] = ACTIONS(2761),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2761),
    [sym_url_flg_stat_mask] = ACTIONS(2761),
    [sym_url_flg_stat_lookup] = ACTIONS(2761),
    [sym_url_flg_stat_connecting] = ACTIONS(2761),
    [sym_url_flg_stat_waiting] = ACTIONS(2761),
    [sym_url_flg_stat_connected] = ACTIONS(2761),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2761),
    [sym_first_virtual_device] = ACTIONS(2761),
    [sym_first_local_port] = ACTIONS(2761),
    [sym_dynamic_virtual_device] = ACTIONS(2761),
    [sym_timeline_once] = ACTIONS(2761),
    [sym_timeline_repeat] = ACTIONS(2761),
    [sym_timeline_absolute] = ACTIONS(2761),
    [sym_timeline_relative] = ACTIONS(2761),
    [sym_do_push_timed_infinite] = ACTIONS(2761),
    [sym_source_type_no_address] = ACTIONS(2761),
    [sym_source_type_neuron_id] = ACTIONS(2761),
    [sym_source_type_ip_address] = ACTIONS(2761),
    [sym_source_type_axlink] = ACTIONS(2761),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2761),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2761),
    [sym_source_type_ip_socket_address] = ACTIONS(2761),
    [sym_source_type_rs232] = ACTIONS(2761),
    [sym_source_type_internal] = ACTIONS(2761),
    [sym_source_type_mac_address] = ACTIONS(2761),
    [sym_source_type_ipv4_port] = ACTIONS(2761),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2761),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2761),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2761),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2761),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2761),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2761),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2761),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2761),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2761),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2761),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2761),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2761),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2761),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2761),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2761),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2761),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2761),
    [sym_true] = ACTIONS(2761),
    [sym_false] = ACTIONS(2761),
    [sym_file_read_only] = ACTIONS(2761),
    [sym_file_rw_new] = ACTIONS(2761),
    [sym_file_rw_append] = ACTIONS(2761),
    [sym_ip_tcp] = ACTIONS(2761),
    [sym_ip_udp] = ACTIONS(2761),
    [sym_ip_udp_2way] = ACTIONS(2761),
    [sym_xml_encode_types] = ACTIONS(2761),
    [sym_xml_encode_char_as_list] = ACTIONS(2761),
    [sym_xml_encode_le] = ACTIONS(2761),
    [sym_xml_decode_types] = ACTIONS(2761),
    [sym_xml_decode_no_preserve] = ACTIONS(2761),
    [sym_duet_dev_type_display_device] = ACTIONS(2761),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2761),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2761),
    [sym_duet_dev_type_io_device] = ACTIONS(2761),
    [sym_duet_dev_type_relay_device] = ACTIONS(2761),
    [sym_duet_dev_type_ups] = ACTIONS(2761),
    [sym_duet_dev_type_amplifier] = ACTIONS(2761),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2761),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2761),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2761),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2761),
    [sym_duet_dev_type_camera] = ACTIONS(2761),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2761),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2761),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2761),
    [sym_duet_dev_type_dss] = ACTIONS(2761),
    [sym_duet_dev_type_dvr] = ACTIONS(2761),
    [sym_duet_dev_type_disc_device] = ACTIONS(2761),
    [sym_duet_dev_type_document_camera] = ACTIONS(2761),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2761),
    [sym_duet_dev_type_hvac] = ACTIONS(2761),
    [sym_duet_dev_type_keypad] = ACTIONS(2761),
    [sym_duet_dev_type_light] = ACTIONS(2761),
    [sym_duet_dev_type_monitor] = ACTIONS(2761),
    [sym_duet_dev_type_motor] = ACTIONS(2761),
    [sym_duet_dev_type_multi_window] = ACTIONS(2761),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2761),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2761),
    [sym_duet_dev_type_receiver] = ACTIONS(2761),
    [sym_duet_dev_type_security_system] = ACTIONS(2761),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2761),
    [sym_duet_dev_type_settop_box] = ACTIONS(2761),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2761),
    [sym_duet_dev_type_switcher] = ACTIONS(2761),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2761),
    [sym_duet_dev_type_tv] = ACTIONS(2761),
    [sym_duet_dev_type_utility] = ACTIONS(2761),
    [sym_duet_dev_type_vcr] = ACTIONS(2761),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2761),
    [sym_duet_dev_type_video_processor] = ACTIONS(2761),
    [sym_duet_dev_type_video_projector] = ACTIONS(2761),
    [sym_duet_dev_type_video_wall] = ACTIONS(2761),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2761),
    [sym_duet_dev_type_weather] = ACTIONS(2761),
    [sym_duet_dev_not_polled] = ACTIONS(2761),
    [sym_duet_dev_polled] = ACTIONS(2761),
    [sym_clkmgr_mode_network] = ACTIONS(2761),
    [sym_clkmgr_mode_standalone] = ACTIONS(2761),
    [sym_normal_standby] = ACTIONS(2761),
    [sym_normal_wake] = ACTIONS(2761),
    [sym_ip_multicast_ttl_option] = ACTIONS(2761),
    [sym_ip_ttl_subnet] = ACTIONS(2761),
    [sym_ip_ttl_site] = ACTIONS(2761),
    [sym_ip_ttl_region] = ACTIONS(2761),
    [sym_ip_ttl_continent] = ACTIONS(2761),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2761),
    [sym_ip_nodelay_on] = ACTIONS(2761),
    [sym_ip_nodelay_off] = ACTIONS(2761),
    [sym_amx_error] = ACTIONS(2761),
    [sym_amx_warning] = ACTIONS(2761),
    [sym_amx_info] = ACTIONS(2761),
    [sym_amx_debug] = ACTIONS(2761),
    [sym_netlinx_login_success] = ACTIONS(2761),
    [sym_netlinx_login_fail] = ACTIONS(2761),
    [sym_netlinx_logout] = ACTIONS(2761),
    [sym_null_str] = ACTIONS(2761),
    [sym_smtp_address] = ACTIONS(2761),
    [sym_smtp_port_number] = ACTIONS(2761),
    [sym_smtp_username] = ACTIONS(2761),
    [sym_smtp_password] = ACTIONS(2761),
    [sym_smtp_require_tls] = ACTIONS(2761),
    [sym_smtp_from] = ACTIONS(2761),
    [sym_smtp_tls_true] = ACTIONS(2761),
    [sym_smtp_tls_false] = ACTIONS(2761),
    [sym_listview_on_row_select_event] = ACTIONS(2761),
    [sym_data_structure_unknown] = ACTIONS(2761),
    [sym_data_structure_datafeed] = ACTIONS(2761),
    [sym_data_structure_datarecordset] = ACTIONS(2761),
    [sym_data_structure_datarecord] = ACTIONS(2761),
    [sym_data_structure_datafield] = ACTIONS(2761),
    [sym_data_max_content_field_count] = ACTIONS(2761),
    [sym_data_max_metadata_field_count] = ACTIONS(2761),
    [sym_data_max_id_length] = ACTIONS(2761),
    [sym_data_max_name_length] = ACTIONS(2761),
    [sym_data_max_description_length] = ACTIONS(2761),
    [sym_data_max_type_length] = ACTIONS(2761),
    [sym_data_max_format_length] = ACTIONS(2761),
    [sym_data_max_source_length] = ACTIONS(2761),
    [sym_data_max_value_length] = ACTIONS(2761),
    [sym_data_max_label_length] = ACTIONS(2761),
    [sym_data_max_url_length] = ACTIONS(2761),
    [sym_max_last_login_info_length] = ACTIONS(2761),
    [sym_data_type_unknown] = ACTIONS(2761),
    [sym_data_type_string] = ACTIONS(2761),
    [sym_data_type_datetime] = ACTIONS(2761),
    [sym_data_type_date] = ACTIONS(2761),
    [sym_data_type_time] = ACTIONS(2761),
    [sym_data_type_image] = ACTIONS(2761),
    [sym_data_format_url] = ACTIONS(2761),
    [sym_data_format_phone] = ACTIONS(2761),
    [sym_data_format_email] = ACTIONS(2761),
    [sym_data_format_iso8601] = ACTIONS(2761),
    [sym_auditlog_priv] = ACTIONS(2761),
    [sym_configuration_priv] = ACTIONS(2761),
    [sym_deviceconfig_priv] = ACTIONS(2761),
    [sym_ftp_priv] = ACTIONS(2761),
    [sym_http_priv] = ACTIONS(2761),
    [sym_networkconfig_priv] = ACTIONS(2761),
    [sym_programport_priv] = ACTIONS(2761),
    [sym_remoteui_priv] = ACTIONS(2761),
    [sym_securitycontrol_priv] = ACTIONS(2761),
    [sym_softwaremanagement_priv] = ACTIONS(2761),
    [sym_terminal_priv] = ACTIONS(2761),
    [sym_tpadmin_priv] = ACTIONS(2761),
    [sym_user1_priv] = ACTIONS(2761),
    [sym_user2_priv] = ACTIONS(2761),
    [sym_user3_priv] = ACTIONS(2761),
    [sym_user4_priv] = ACTIONS(2761),
    [sym_usermanagement_priv] = ACTIONS(2761),
    [sym_valid_account] = ACTIONS(2761),
    [sym_err_name_invalid] = ACTIONS(2761),
    [sym_err_password_invalid] = ACTIONS(2761),
    [sym_err_invalid_account] = ACTIONS(2761),
    [sym_err_no_authorization] = ACTIONS(2761),
    [sym_err_invalid_parameter] = ACTIONS(2761),
    [sym_err_user_locked_out] = ACTIONS(2761),
    [sym_err_user_account_expired] = ACTIONS(2761),
    [sym_err_general_error] = ACTIONS(2761),
    [sym_err_authentication_error] = ACTIONS(2761),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2761),
    [sym_tls_validate_certificate] = ACTIONS(2761),
    [sym____reserved___] = ACTIONS(2761),
    [sym_date] = ACTIONS(2761),
    [sym_day] = ACTIONS(2761),
    [sym_time] = ACTIONS(2761),
    [sym_ldate] = ACTIONS(2761),
    [sym_push_device] = ACTIONS(2761),
    [sym_push_channel] = ACTIONS(2761),
    [sym_push_devchan] = ACTIONS(2761),
    [sym_release_device] = ACTIONS(2761),
    [sym_release_channel] = ACTIONS(2761),
    [sym_release_devchan] = ACTIONS(2761),
    [sym_master_slot] = ACTIONS(2761),
    [sym_get_pulse_time] = ACTIONS(2761),
    [sym_get_timer] = ACTIONS(2761),
    [sym_master_sn] = ACTIONS(2761),
    [sym_system_number] = ACTIONS(2761),
    [sym_button] = ACTIONS(2761),
    [sym_dv_channel] = ACTIONS(2761),
    [sym_data] = ACTIONS(2761),
    [sym_level] = ACTIONS(2761),
    [sym_timeline] = ACTIONS(2761),
    [sym_channel] = ACTIONS(2761),
    [sym_custom] = ACTIONS(2761),
    [sym_atoi] = ACTIONS(2761),
    [sym_atol] = ACTIONS(2761),
    [sym_atof] = ACTIONS(2761),
    [sym_itoa] = ACTIONS(2761),
    [sym_itohex] = ACTIONS(2761),
    [sym_hextoi] = ACTIONS(2761),
    [sym_ftoa] = ACTIONS(2761),
    [sym_find_string] = ACTIONS(2761),
    [sym_left_string] = ACTIONS(2761),
    [sym_right_string] = ACTIONS(2761),
    [sym_mid_string] = ACTIONS(2761),
    [sym_remove_string] = ACTIONS(2761),
    [sym_get_buffer_string] = ACTIONS(2761),
    [sym_get_buffer_char] = ACTIONS(2761),
    [sym_get_multi_buffer_string] = ACTIONS(2761),
    [sym_length_string] = ACTIONS(2761),
    [sym_max_length_string] = ACTIONS(2761),
    [sym_set_length_string] = ACTIONS(2761),
    [sym_compare_string] = ACTIONS(2761),
    [sym_upper_string] = ACTIONS(2761),
    [sym_lower_string] = ACTIONS(2761),
    [sym_get_last] = ACTIONS(2761),
    [sym_date_to_day] = ACTIONS(2761),
    [sym_date_to_month] = ACTIONS(2761),
    [sym_date_to_year] = ACTIONS(2761),
    [sym_time_to_hour] = ACTIONS(2761),
    [sym_time_to_minute] = ACTIONS(2761),
    [sym_time_to_second] = ACTIONS(2761),
    [sym_day_of_week] = ACTIONS(2761),
    [sym_set_pulse_time] = ACTIONS(2761),
    [sym_set_timer] = ACTIONS(2761),
    [sym_file_open] = ACTIONS(2761),
    [sym_file_close] = ACTIONS(2761),
    [sym_file_read] = ACTIONS(2761),
    [sym_file_read_line] = ACTIONS(2761),
    [sym_file_write] = ACTIONS(2761),
    [sym_file_write_line] = ACTIONS(2761),
    [sym_file_copy] = ACTIONS(2761),
    [sym_file_delete] = ACTIONS(2761),
    [sym_file_rename] = ACTIONS(2761),
    [sym_file_dir] = ACTIONS(2761),
    [sym_file_getdir] = ACTIONS(2761),
    [sym_file_setdir] = ACTIONS(2761),
    [sym_file_create_dir] = ACTIONS(2761),
    [sym_file_remove_dir] = ACTIONS(2761),
    [sym_file_seek] = ACTIONS(2761),
    [sym_length_array] = ACTIONS(2761),
    [sym_max_length_array] = ACTIONS(2761),
    [sym_set_length_array] = ACTIONS(2761),
    [sym_ip_client_open] = ACTIONS(2761),
    [sym_ip_client_close] = ACTIONS(2761),
    [sym_ip_server_open] = ACTIONS(2761),
    [sym_ip_server_close] = ACTIONS(2761),
    [sym_ip_bound_client_open] = ACTIONS(2761),
    [sym_ip_mc_server_open] = ACTIONS(2761),
    [sym_ip_set_option] = ACTIONS(2761),
    [sym_ssh_client_open] = ACTIONS(2761),
    [sym_ssh_client_close] = ACTIONS(2761),
    [sym_tls_client_open] = ACTIONS(2761),
    [sym_tls_client_close] = ACTIONS(2761),
    [sym_do_push] = ACTIONS(2761),
    [sym_do_push_timed] = ACTIONS(2761),
    [sym_do_release] = ACTIONS(2761),
    [sym_device_id] = ACTIONS(2761),
    [sym_device_info] = ACTIONS(2761),
    [sym_device_id_string] = ACTIONS(2761),
    [sym_device_standby] = ACTIONS(2761),
    [sym_device_wake] = ACTIONS(2761),
    [sym_timeline_create] = ACTIONS(2761),
    [sym_timeline_kill] = ACTIONS(2761),
    [sym_timeline_reload] = ACTIONS(2761),
    [sym_timeline_pause] = ACTIONS(2761),
    [sym_timeline_restart] = ACTIONS(2761),
    [sym_timeline_set] = ACTIONS(2761),
    [sym_timeline_get] = ACTIONS(2761),
    [sym_timeline_active] = ACTIONS(2761),
    [sym_timeline_dynamic_id] = ACTIONS(2761),
    [sym_variable_to_string] = ACTIONS(2761),
    [sym_string_to_variable] = ACTIONS(2761),
    [sym_length_variable_to_string] = ACTIONS(2761),
    [sym_type_cast] = ACTIONS(2761),
    [sym_variable_to_xml] = ACTIONS(2761),
    [sym_xml_to_variable] = ACTIONS(2761),
    [sym_length_variable_to_xml] = ACTIONS(2761),
    [sym_redirect_string] = ACTIONS(2761),
    [sym_random_number] = ACTIONS(2761),
    [sym_get_unique_id] = ACTIONS(2761),
    [sym_get_system_number] = ACTIONS(2761),
    [sym_set_system_number] = ACTIONS(2761),
    [sym_get_serial_number] = ACTIONS(2761),
    [sym_get_master_build] = ACTIONS(2761),
    [sym_set_validation_code] = ACTIONS(2761),
    [sym_reboot] = ACTIONS(2761),
    [sym_load_duet_module] = ACTIONS(2761),
    [sym_unload_duet_module] = ACTIONS(2761),
    [sym_duet_mem_size_set] = ACTIONS(2761),
    [sym_duet_mem_size_get] = ACTIONS(2761),
    [sym_dynamic_application_device] = ACTIONS(2761),
    [sym_dynamic_polled_port] = ACTIONS(2761),
    [sym_static_port_binding] = ACTIONS(2761),
    [sym_static_ip_binding] = ACTIONS(2761),
    [sym_get_dns_list] = ACTIONS(2761),
    [sym_set_dns_list] = ACTIONS(2761),
    [sym_get_ip_address] = ACTIONS(2761),
    [sym_set_ip_address] = ACTIONS(2761),
    [sym_get_url_list] = ACTIONS(2761),
    [sym_add_url_entry] = ACTIONS(2761),
    [sym_delete_url_entry] = ACTIONS(2761),
    [sym_abs_value] = ACTIONS(2761),
    [sym_max_value] = ACTIONS(2761),
    [sym_min_value] = ACTIONS(2761),
    [sym_exp_value] = ACTIONS(2761),
    [sym_log_value] = ACTIONS(2761),
    [sym_log10_value] = ACTIONS(2761),
    [sym_power_value] = ACTIONS(2761),
    [sym_sqrt_value] = ACTIONS(2761),
    [sym_raw_be] = ACTIONS(2761),
    [sym_raw_le] = ACTIONS(2761),
    [sym_internal_queue_size_set] = ACTIONS(2761),
    [sym_internal_queue_size_get] = ACTIONS(2761),
    [sym_internal_threshold_set] = ACTIONS(2761),
    [sym_internal_threshold_get] = ACTIONS(2761),
    [sym_rebuilt_event] = ACTIONS(2761),
    [sym_do_custom_event] = ACTIONS(2761),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2761),
    [sym_clkmgr_set_clk_source] = ACTIONS(2761),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2761),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2761),
    [sym_clkmgr_get_timezone] = ACTIONS(2761),
    [sym_clkmgr_set_timezone] = ACTIONS(2761),
    [sym_clkmgr_get_resync_period] = ACTIONS(2761),
    [sym_clkmgr_set_resync_period] = ACTIONS(2761),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2761),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2761),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2761),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2761),
    [sym_clkmgr_get_timeservers] = ACTIONS(2761),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2761),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2761),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2761),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2761),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2761),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2761),
    [sym_set_log_level] = ACTIONS(2761),
    [sym_get_log_level] = ACTIONS(2761),
    [sym_amx_log] = ACTIONS(2761),
    [sym_smtp_server_config_set] = ACTIONS(2761),
    [sym_smtp_server_config_get] = ACTIONS(2761),
    [sym_smtp_send] = ACTIONS(2761),
    [sym_validate_netlinx_account] = ACTIONS(2761),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2761),
    [sym_authenticate_certificate] = ACTIONS(2761),
    [sym_audit_netlinx_session_event] = ACTIONS(2761),
    [sym_audit_netlinx_generic_event] = ACTIONS(2761),
    [sym_get_available_flash_disk_space] = ACTIONS(2761),
    [sym_get_max_flash_disk_space] = ACTIONS(2761),
    [sym_data_create_feed] = ACTIONS(2761),
    [sym_data_delete_feed] = ACTIONS(2761),
    [sym_data_publish_feed] = ACTIONS(2761),
    [sym_data_get_published_feed] = ACTIONS(2761),
    [sym_data_add_record] = ACTIONS(2761),
    [sym_data_get_event_record] = ACTIONS(2761),
    [sym__wc_data_create_feed] = ACTIONS(2761),
    [sym__wc_data_add_record] = ACTIONS(2761),
    [sym__wc_data_get_event_record] = ACTIONS(2761),
    [sym_format] = ACTIONS(2761),
    [sym_astro_clock] = ACTIONS(2761),
    [sym_set_outdoor_temperature] = ACTIONS(2761),
    [sym_set_virtual_level_count] = ACTIONS(2761),
    [sym_set_virtual_channel_count] = ACTIONS(2761),
    [sym_set_virtual_port_count] = ACTIONS(2761),
    [sym__wc_encode_int] = ACTIONS(2761),
    [sym__wc_decode_int] = ACTIONS(2761),
    [sym__wc_int] = ACTIONS(2761),
    [sym__wc_to_ch_int] = ACTIONS(2761),
    [sym__ch_to_wc_int] = ACTIONS(2761),
    [sym__wc_find_string_int] = ACTIONS(2761),
    [sym__wc_left_string_int] = ACTIONS(2761),
    [sym__wc_lower_string_int] = ACTIONS(2761),
    [sym__wc_mid_string_int] = ACTIONS(2761),
    [sym__wc_remove_string_int] = ACTIONS(2761),
    [sym__wc_right_string_int] = ACTIONS(2761),
    [sym__wc_upper_string_int] = ACTIONS(2761),
    [sym__wc_compare_string_int] = ACTIONS(2761),
    [sym__wc_get_buffer_char_int] = ACTIONS(2761),
    [sym__wc_get_buffer_string_int] = ACTIONS(2761),
    [sym__wc_concat_string_int] = ACTIONS(2761),
    [sym__wc_file_open_int] = ACTIONS(2761),
    [sym__wc_file_close_int] = ACTIONS(2761),
    [sym__wc_file_read_int] = ACTIONS(2761),
    [sym__wc_file_read_line_int] = ACTIONS(2761),
    [sym__wc_file_write_int] = ACTIONS(2761),
    [sym__wc_file_write_line_int] = ACTIONS(2761),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2763),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2763),
    [anon_sym_LBRACK] = ACTIONS(2763),
    [anon_sym_LBRACE] = ACTIONS(2763),
    [anon_sym_RBRACE] = ACTIONS(2763),
    [anon_sym_LPAREN] = ACTIONS(2761),
    [anon_sym_BANG] = ACTIONS(2763),
    [anon_sym_TILDE] = ACTIONS(2763),
    [anon_sym_DASH] = ACTIONS(2761),
    [anon_sym_PLUS] = ACTIONS(2761),
    [anon_sym_DASH_DASH] = ACTIONS(2763),
    [anon_sym_PLUS_PLUS] = ACTIONS(2763),
    [anon_sym_DQUOTE] = ACTIONS(2763),
    [anon_sym_SQUOTE] = ACTIONS(2763),
    [sym_number_literal] = ACTIONS(2763),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(924)] = {
    [sym_identifier] = ACTIONS(2765),
    [sym_not] = ACTIONS(2765),
    [sym_bnot] = ACTIONS(2765),
    [sym___file__] = ACTIONS(2765),
    [sym___date__] = ACTIONS(2765),
    [sym___ldate__] = ACTIONS(2765),
    [sym___time__] = ACTIONS(2765),
    [sym___line__] = ACTIONS(2765),
    [sym___name__] = ACTIONS(2765),
    [sym___netlinx__] = ACTIONS(2765),
    [sym_netlinx_axi_version] = ACTIONS(2765),
    [sym_url_flg_tcp] = ACTIONS(2765),
    [sym_url_flg_accinfopresent] = ACTIONS(2765),
    [sym_url_flg_temp] = ACTIONS(2765),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2765),
    [sym_url_flg_stat_mask] = ACTIONS(2765),
    [sym_url_flg_stat_lookup] = ACTIONS(2765),
    [sym_url_flg_stat_connecting] = ACTIONS(2765),
    [sym_url_flg_stat_waiting] = ACTIONS(2765),
    [sym_url_flg_stat_connected] = ACTIONS(2765),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2765),
    [sym_first_virtual_device] = ACTIONS(2765),
    [sym_first_local_port] = ACTIONS(2765),
    [sym_dynamic_virtual_device] = ACTIONS(2765),
    [sym_timeline_once] = ACTIONS(2765),
    [sym_timeline_repeat] = ACTIONS(2765),
    [sym_timeline_absolute] = ACTIONS(2765),
    [sym_timeline_relative] = ACTIONS(2765),
    [sym_do_push_timed_infinite] = ACTIONS(2765),
    [sym_source_type_no_address] = ACTIONS(2765),
    [sym_source_type_neuron_id] = ACTIONS(2765),
    [sym_source_type_ip_address] = ACTIONS(2765),
    [sym_source_type_axlink] = ACTIONS(2765),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2765),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2765),
    [sym_source_type_ip_socket_address] = ACTIONS(2765),
    [sym_source_type_rs232] = ACTIONS(2765),
    [sym_source_type_internal] = ACTIONS(2765),
    [sym_source_type_mac_address] = ACTIONS(2765),
    [sym_source_type_ipv4_port] = ACTIONS(2765),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2765),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2765),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2765),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2765),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2765),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2765),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2765),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2765),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2765),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2765),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2765),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2765),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2765),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2765),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2765),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2765),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2765),
    [sym_true] = ACTIONS(2765),
    [sym_false] = ACTIONS(2765),
    [sym_file_read_only] = ACTIONS(2765),
    [sym_file_rw_new] = ACTIONS(2765),
    [sym_file_rw_append] = ACTIONS(2765),
    [sym_ip_tcp] = ACTIONS(2765),
    [sym_ip_udp] = ACTIONS(2765),
    [sym_ip_udp_2way] = ACTIONS(2765),
    [sym_xml_encode_types] = ACTIONS(2765),
    [sym_xml_encode_char_as_list] = ACTIONS(2765),
    [sym_xml_encode_le] = ACTIONS(2765),
    [sym_xml_decode_types] = ACTIONS(2765),
    [sym_xml_decode_no_preserve] = ACTIONS(2765),
    [sym_duet_dev_type_display_device] = ACTIONS(2765),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2765),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2765),
    [sym_duet_dev_type_io_device] = ACTIONS(2765),
    [sym_duet_dev_type_relay_device] = ACTIONS(2765),
    [sym_duet_dev_type_ups] = ACTIONS(2765),
    [sym_duet_dev_type_amplifier] = ACTIONS(2765),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2765),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2765),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2765),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2765),
    [sym_duet_dev_type_camera] = ACTIONS(2765),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2765),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2765),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2765),
    [sym_duet_dev_type_dss] = ACTIONS(2765),
    [sym_duet_dev_type_dvr] = ACTIONS(2765),
    [sym_duet_dev_type_disc_device] = ACTIONS(2765),
    [sym_duet_dev_type_document_camera] = ACTIONS(2765),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2765),
    [sym_duet_dev_type_hvac] = ACTIONS(2765),
    [sym_duet_dev_type_keypad] = ACTIONS(2765),
    [sym_duet_dev_type_light] = ACTIONS(2765),
    [sym_duet_dev_type_monitor] = ACTIONS(2765),
    [sym_duet_dev_type_motor] = ACTIONS(2765),
    [sym_duet_dev_type_multi_window] = ACTIONS(2765),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2765),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2765),
    [sym_duet_dev_type_receiver] = ACTIONS(2765),
    [sym_duet_dev_type_security_system] = ACTIONS(2765),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2765),
    [sym_duet_dev_type_settop_box] = ACTIONS(2765),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2765),
    [sym_duet_dev_type_switcher] = ACTIONS(2765),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2765),
    [sym_duet_dev_type_tv] = ACTIONS(2765),
    [sym_duet_dev_type_utility] = ACTIONS(2765),
    [sym_duet_dev_type_vcr] = ACTIONS(2765),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2765),
    [sym_duet_dev_type_video_processor] = ACTIONS(2765),
    [sym_duet_dev_type_video_projector] = ACTIONS(2765),
    [sym_duet_dev_type_video_wall] = ACTIONS(2765),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2765),
    [sym_duet_dev_type_weather] = ACTIONS(2765),
    [sym_duet_dev_not_polled] = ACTIONS(2765),
    [sym_duet_dev_polled] = ACTIONS(2765),
    [sym_clkmgr_mode_network] = ACTIONS(2765),
    [sym_clkmgr_mode_standalone] = ACTIONS(2765),
    [sym_normal_standby] = ACTIONS(2765),
    [sym_normal_wake] = ACTIONS(2765),
    [sym_ip_multicast_ttl_option] = ACTIONS(2765),
    [sym_ip_ttl_subnet] = ACTIONS(2765),
    [sym_ip_ttl_site] = ACTIONS(2765),
    [sym_ip_ttl_region] = ACTIONS(2765),
    [sym_ip_ttl_continent] = ACTIONS(2765),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2765),
    [sym_ip_nodelay_on] = ACTIONS(2765),
    [sym_ip_nodelay_off] = ACTIONS(2765),
    [sym_amx_error] = ACTIONS(2765),
    [sym_amx_warning] = ACTIONS(2765),
    [sym_amx_info] = ACTIONS(2765),
    [sym_amx_debug] = ACTIONS(2765),
    [sym_netlinx_login_success] = ACTIONS(2765),
    [sym_netlinx_login_fail] = ACTIONS(2765),
    [sym_netlinx_logout] = ACTIONS(2765),
    [sym_null_str] = ACTIONS(2765),
    [sym_smtp_address] = ACTIONS(2765),
    [sym_smtp_port_number] = ACTIONS(2765),
    [sym_smtp_username] = ACTIONS(2765),
    [sym_smtp_password] = ACTIONS(2765),
    [sym_smtp_require_tls] = ACTIONS(2765),
    [sym_smtp_from] = ACTIONS(2765),
    [sym_smtp_tls_true] = ACTIONS(2765),
    [sym_smtp_tls_false] = ACTIONS(2765),
    [sym_listview_on_row_select_event] = ACTIONS(2765),
    [sym_data_structure_unknown] = ACTIONS(2765),
    [sym_data_structure_datafeed] = ACTIONS(2765),
    [sym_data_structure_datarecordset] = ACTIONS(2765),
    [sym_data_structure_datarecord] = ACTIONS(2765),
    [sym_data_structure_datafield] = ACTIONS(2765),
    [sym_data_max_content_field_count] = ACTIONS(2765),
    [sym_data_max_metadata_field_count] = ACTIONS(2765),
    [sym_data_max_id_length] = ACTIONS(2765),
    [sym_data_max_name_length] = ACTIONS(2765),
    [sym_data_max_description_length] = ACTIONS(2765),
    [sym_data_max_type_length] = ACTIONS(2765),
    [sym_data_max_format_length] = ACTIONS(2765),
    [sym_data_max_source_length] = ACTIONS(2765),
    [sym_data_max_value_length] = ACTIONS(2765),
    [sym_data_max_label_length] = ACTIONS(2765),
    [sym_data_max_url_length] = ACTIONS(2765),
    [sym_max_last_login_info_length] = ACTIONS(2765),
    [sym_data_type_unknown] = ACTIONS(2765),
    [sym_data_type_string] = ACTIONS(2765),
    [sym_data_type_datetime] = ACTIONS(2765),
    [sym_data_type_date] = ACTIONS(2765),
    [sym_data_type_time] = ACTIONS(2765),
    [sym_data_type_image] = ACTIONS(2765),
    [sym_data_format_url] = ACTIONS(2765),
    [sym_data_format_phone] = ACTIONS(2765),
    [sym_data_format_email] = ACTIONS(2765),
    [sym_data_format_iso8601] = ACTIONS(2765),
    [sym_auditlog_priv] = ACTIONS(2765),
    [sym_configuration_priv] = ACTIONS(2765),
    [sym_deviceconfig_priv] = ACTIONS(2765),
    [sym_ftp_priv] = ACTIONS(2765),
    [sym_http_priv] = ACTIONS(2765),
    [sym_networkconfig_priv] = ACTIONS(2765),
    [sym_programport_priv] = ACTIONS(2765),
    [sym_remoteui_priv] = ACTIONS(2765),
    [sym_securitycontrol_priv] = ACTIONS(2765),
    [sym_softwaremanagement_priv] = ACTIONS(2765),
    [sym_terminal_priv] = ACTIONS(2765),
    [sym_tpadmin_priv] = ACTIONS(2765),
    [sym_user1_priv] = ACTIONS(2765),
    [sym_user2_priv] = ACTIONS(2765),
    [sym_user3_priv] = ACTIONS(2765),
    [sym_user4_priv] = ACTIONS(2765),
    [sym_usermanagement_priv] = ACTIONS(2765),
    [sym_valid_account] = ACTIONS(2765),
    [sym_err_name_invalid] = ACTIONS(2765),
    [sym_err_password_invalid] = ACTIONS(2765),
    [sym_err_invalid_account] = ACTIONS(2765),
    [sym_err_no_authorization] = ACTIONS(2765),
    [sym_err_invalid_parameter] = ACTIONS(2765),
    [sym_err_user_locked_out] = ACTIONS(2765),
    [sym_err_user_account_expired] = ACTIONS(2765),
    [sym_err_general_error] = ACTIONS(2765),
    [sym_err_authentication_error] = ACTIONS(2765),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2765),
    [sym_tls_validate_certificate] = ACTIONS(2765),
    [sym____reserved___] = ACTIONS(2765),
    [sym_date] = ACTIONS(2765),
    [sym_day] = ACTIONS(2765),
    [sym_time] = ACTIONS(2765),
    [sym_ldate] = ACTIONS(2765),
    [sym_push_device] = ACTIONS(2765),
    [sym_push_channel] = ACTIONS(2765),
    [sym_push_devchan] = ACTIONS(2765),
    [sym_release_device] = ACTIONS(2765),
    [sym_release_channel] = ACTIONS(2765),
    [sym_release_devchan] = ACTIONS(2765),
    [sym_master_slot] = ACTIONS(2765),
    [sym_get_pulse_time] = ACTIONS(2765),
    [sym_get_timer] = ACTIONS(2765),
    [sym_master_sn] = ACTIONS(2765),
    [sym_system_number] = ACTIONS(2765),
    [sym_button] = ACTIONS(2765),
    [sym_dv_channel] = ACTIONS(2765),
    [sym_data] = ACTIONS(2765),
    [sym_level] = ACTIONS(2765),
    [sym_timeline] = ACTIONS(2765),
    [sym_channel] = ACTIONS(2765),
    [sym_custom] = ACTIONS(2765),
    [sym_atoi] = ACTIONS(2765),
    [sym_atol] = ACTIONS(2765),
    [sym_atof] = ACTIONS(2765),
    [sym_itoa] = ACTIONS(2765),
    [sym_itohex] = ACTIONS(2765),
    [sym_hextoi] = ACTIONS(2765),
    [sym_ftoa] = ACTIONS(2765),
    [sym_find_string] = ACTIONS(2765),
    [sym_left_string] = ACTIONS(2765),
    [sym_right_string] = ACTIONS(2765),
    [sym_mid_string] = ACTIONS(2765),
    [sym_remove_string] = ACTIONS(2765),
    [sym_get_buffer_string] = ACTIONS(2765),
    [sym_get_buffer_char] = ACTIONS(2765),
    [sym_get_multi_buffer_string] = ACTIONS(2765),
    [sym_length_string] = ACTIONS(2765),
    [sym_max_length_string] = ACTIONS(2765),
    [sym_set_length_string] = ACTIONS(2765),
    [sym_compare_string] = ACTIONS(2765),
    [sym_upper_string] = ACTIONS(2765),
    [sym_lower_string] = ACTIONS(2765),
    [sym_get_last] = ACTIONS(2765),
    [sym_date_to_day] = ACTIONS(2765),
    [sym_date_to_month] = ACTIONS(2765),
    [sym_date_to_year] = ACTIONS(2765),
    [sym_time_to_hour] = ACTIONS(2765),
    [sym_time_to_minute] = ACTIONS(2765),
    [sym_time_to_second] = ACTIONS(2765),
    [sym_day_of_week] = ACTIONS(2765),
    [sym_set_pulse_time] = ACTIONS(2765),
    [sym_set_timer] = ACTIONS(2765),
    [sym_file_open] = ACTIONS(2765),
    [sym_file_close] = ACTIONS(2765),
    [sym_file_read] = ACTIONS(2765),
    [sym_file_read_line] = ACTIONS(2765),
    [sym_file_write] = ACTIONS(2765),
    [sym_file_write_line] = ACTIONS(2765),
    [sym_file_copy] = ACTIONS(2765),
    [sym_file_delete] = ACTIONS(2765),
    [sym_file_rename] = ACTIONS(2765),
    [sym_file_dir] = ACTIONS(2765),
    [sym_file_getdir] = ACTIONS(2765),
    [sym_file_setdir] = ACTIONS(2765),
    [sym_file_create_dir] = ACTIONS(2765),
    [sym_file_remove_dir] = ACTIONS(2765),
    [sym_file_seek] = ACTIONS(2765),
    [sym_length_array] = ACTIONS(2765),
    [sym_max_length_array] = ACTIONS(2765),
    [sym_set_length_array] = ACTIONS(2765),
    [sym_ip_client_open] = ACTIONS(2765),
    [sym_ip_client_close] = ACTIONS(2765),
    [sym_ip_server_open] = ACTIONS(2765),
    [sym_ip_server_close] = ACTIONS(2765),
    [sym_ip_bound_client_open] = ACTIONS(2765),
    [sym_ip_mc_server_open] = ACTIONS(2765),
    [sym_ip_set_option] = ACTIONS(2765),
    [sym_ssh_client_open] = ACTIONS(2765),
    [sym_ssh_client_close] = ACTIONS(2765),
    [sym_tls_client_open] = ACTIONS(2765),
    [sym_tls_client_close] = ACTIONS(2765),
    [sym_do_push] = ACTIONS(2765),
    [sym_do_push_timed] = ACTIONS(2765),
    [sym_do_release] = ACTIONS(2765),
    [sym_device_id] = ACTIONS(2765),
    [sym_device_info] = ACTIONS(2765),
    [sym_device_id_string] = ACTIONS(2765),
    [sym_device_standby] = ACTIONS(2765),
    [sym_device_wake] = ACTIONS(2765),
    [sym_timeline_create] = ACTIONS(2765),
    [sym_timeline_kill] = ACTIONS(2765),
    [sym_timeline_reload] = ACTIONS(2765),
    [sym_timeline_pause] = ACTIONS(2765),
    [sym_timeline_restart] = ACTIONS(2765),
    [sym_timeline_set] = ACTIONS(2765),
    [sym_timeline_get] = ACTIONS(2765),
    [sym_timeline_active] = ACTIONS(2765),
    [sym_timeline_dynamic_id] = ACTIONS(2765),
    [sym_variable_to_string] = ACTIONS(2765),
    [sym_string_to_variable] = ACTIONS(2765),
    [sym_length_variable_to_string] = ACTIONS(2765),
    [sym_type_cast] = ACTIONS(2765),
    [sym_variable_to_xml] = ACTIONS(2765),
    [sym_xml_to_variable] = ACTIONS(2765),
    [sym_length_variable_to_xml] = ACTIONS(2765),
    [sym_redirect_string] = ACTIONS(2765),
    [sym_random_number] = ACTIONS(2765),
    [sym_get_unique_id] = ACTIONS(2765),
    [sym_get_system_number] = ACTIONS(2765),
    [sym_set_system_number] = ACTIONS(2765),
    [sym_get_serial_number] = ACTIONS(2765),
    [sym_get_master_build] = ACTIONS(2765),
    [sym_set_validation_code] = ACTIONS(2765),
    [sym_reboot] = ACTIONS(2765),
    [sym_load_duet_module] = ACTIONS(2765),
    [sym_unload_duet_module] = ACTIONS(2765),
    [sym_duet_mem_size_set] = ACTIONS(2765),
    [sym_duet_mem_size_get] = ACTIONS(2765),
    [sym_dynamic_application_device] = ACTIONS(2765),
    [sym_dynamic_polled_port] = ACTIONS(2765),
    [sym_static_port_binding] = ACTIONS(2765),
    [sym_static_ip_binding] = ACTIONS(2765),
    [sym_get_dns_list] = ACTIONS(2765),
    [sym_set_dns_list] = ACTIONS(2765),
    [sym_get_ip_address] = ACTIONS(2765),
    [sym_set_ip_address] = ACTIONS(2765),
    [sym_get_url_list] = ACTIONS(2765),
    [sym_add_url_entry] = ACTIONS(2765),
    [sym_delete_url_entry] = ACTIONS(2765),
    [sym_abs_value] = ACTIONS(2765),
    [sym_max_value] = ACTIONS(2765),
    [sym_min_value] = ACTIONS(2765),
    [sym_exp_value] = ACTIONS(2765),
    [sym_log_value] = ACTIONS(2765),
    [sym_log10_value] = ACTIONS(2765),
    [sym_power_value] = ACTIONS(2765),
    [sym_sqrt_value] = ACTIONS(2765),
    [sym_raw_be] = ACTIONS(2765),
    [sym_raw_le] = ACTIONS(2765),
    [sym_internal_queue_size_set] = ACTIONS(2765),
    [sym_internal_queue_size_get] = ACTIONS(2765),
    [sym_internal_threshold_set] = ACTIONS(2765),
    [sym_internal_threshold_get] = ACTIONS(2765),
    [sym_rebuilt_event] = ACTIONS(2765),
    [sym_do_custom_event] = ACTIONS(2765),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2765),
    [sym_clkmgr_set_clk_source] = ACTIONS(2765),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2765),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2765),
    [sym_clkmgr_get_timezone] = ACTIONS(2765),
    [sym_clkmgr_set_timezone] = ACTIONS(2765),
    [sym_clkmgr_get_resync_period] = ACTIONS(2765),
    [sym_clkmgr_set_resync_period] = ACTIONS(2765),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2765),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2765),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2765),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2765),
    [sym_clkmgr_get_timeservers] = ACTIONS(2765),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2765),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2765),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2765),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2765),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2765),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2765),
    [sym_set_log_level] = ACTIONS(2765),
    [sym_get_log_level] = ACTIONS(2765),
    [sym_amx_log] = ACTIONS(2765),
    [sym_smtp_server_config_set] = ACTIONS(2765),
    [sym_smtp_server_config_get] = ACTIONS(2765),
    [sym_smtp_send] = ACTIONS(2765),
    [sym_validate_netlinx_account] = ACTIONS(2765),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2765),
    [sym_authenticate_certificate] = ACTIONS(2765),
    [sym_audit_netlinx_session_event] = ACTIONS(2765),
    [sym_audit_netlinx_generic_event] = ACTIONS(2765),
    [sym_get_available_flash_disk_space] = ACTIONS(2765),
    [sym_get_max_flash_disk_space] = ACTIONS(2765),
    [sym_data_create_feed] = ACTIONS(2765),
    [sym_data_delete_feed] = ACTIONS(2765),
    [sym_data_publish_feed] = ACTIONS(2765),
    [sym_data_get_published_feed] = ACTIONS(2765),
    [sym_data_add_record] = ACTIONS(2765),
    [sym_data_get_event_record] = ACTIONS(2765),
    [sym__wc_data_create_feed] = ACTIONS(2765),
    [sym__wc_data_add_record] = ACTIONS(2765),
    [sym__wc_data_get_event_record] = ACTIONS(2765),
    [sym_format] = ACTIONS(2765),
    [sym_astro_clock] = ACTIONS(2765),
    [sym_set_outdoor_temperature] = ACTIONS(2765),
    [sym_set_virtual_level_count] = ACTIONS(2765),
    [sym_set_virtual_channel_count] = ACTIONS(2765),
    [sym_set_virtual_port_count] = ACTIONS(2765),
    [sym__wc_encode_int] = ACTIONS(2765),
    [sym__wc_decode_int] = ACTIONS(2765),
    [sym__wc_int] = ACTIONS(2765),
    [sym__wc_to_ch_int] = ACTIONS(2765),
    [sym__ch_to_wc_int] = ACTIONS(2765),
    [sym__wc_find_string_int] = ACTIONS(2765),
    [sym__wc_left_string_int] = ACTIONS(2765),
    [sym__wc_lower_string_int] = ACTIONS(2765),
    [sym__wc_mid_string_int] = ACTIONS(2765),
    [sym__wc_remove_string_int] = ACTIONS(2765),
    [sym__wc_right_string_int] = ACTIONS(2765),
    [sym__wc_upper_string_int] = ACTIONS(2765),
    [sym__wc_compare_string_int] = ACTIONS(2765),
    [sym__wc_get_buffer_char_int] = ACTIONS(2765),
    [sym__wc_get_buffer_string_int] = ACTIONS(2765),
    [sym__wc_concat_string_int] = ACTIONS(2765),
    [sym__wc_file_open_int] = ACTIONS(2765),
    [sym__wc_file_close_int] = ACTIONS(2765),
    [sym__wc_file_read_int] = ACTIONS(2765),
    [sym__wc_file_read_line_int] = ACTIONS(2765),
    [sym__wc_file_write_int] = ACTIONS(2765),
    [sym__wc_file_write_line_int] = ACTIONS(2765),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2768),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2768),
    [anon_sym_LBRACK] = ACTIONS(2768),
    [anon_sym_LBRACE] = ACTIONS(2768),
    [anon_sym_RBRACE] = ACTIONS(2768),
    [anon_sym_LPAREN] = ACTIONS(2765),
    [anon_sym_BANG] = ACTIONS(2768),
    [anon_sym_TILDE] = ACTIONS(2768),
    [anon_sym_DASH] = ACTIONS(2765),
    [anon_sym_PLUS] = ACTIONS(2765),
    [anon_sym_DASH_DASH] = ACTIONS(2768),
    [anon_sym_PLUS_PLUS] = ACTIONS(2768),
    [anon_sym_DQUOTE] = ACTIONS(2768),
    [anon_sym_SQUOTE] = ACTIONS(2768),
    [sym_number_literal] = ACTIONS(2768),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(925)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2223),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(926)] = {
    [sym_identifier] = ACTIONS(2771),
    [sym_not] = ACTIONS(2771),
    [sym_bnot] = ACTIONS(2771),
    [sym___file__] = ACTIONS(2771),
    [sym___date__] = ACTIONS(2771),
    [sym___ldate__] = ACTIONS(2771),
    [sym___time__] = ACTIONS(2771),
    [sym___line__] = ACTIONS(2771),
    [sym___name__] = ACTIONS(2771),
    [sym___netlinx__] = ACTIONS(2771),
    [sym_netlinx_axi_version] = ACTIONS(2771),
    [sym_url_flg_tcp] = ACTIONS(2771),
    [sym_url_flg_accinfopresent] = ACTIONS(2771),
    [sym_url_flg_temp] = ACTIONS(2771),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2771),
    [sym_url_flg_stat_mask] = ACTIONS(2771),
    [sym_url_flg_stat_lookup] = ACTIONS(2771),
    [sym_url_flg_stat_connecting] = ACTIONS(2771),
    [sym_url_flg_stat_waiting] = ACTIONS(2771),
    [sym_url_flg_stat_connected] = ACTIONS(2771),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2771),
    [sym_first_virtual_device] = ACTIONS(2771),
    [sym_first_local_port] = ACTIONS(2771),
    [sym_dynamic_virtual_device] = ACTIONS(2771),
    [sym_timeline_once] = ACTIONS(2771),
    [sym_timeline_repeat] = ACTIONS(2771),
    [sym_timeline_absolute] = ACTIONS(2771),
    [sym_timeline_relative] = ACTIONS(2771),
    [sym_do_push_timed_infinite] = ACTIONS(2771),
    [sym_source_type_no_address] = ACTIONS(2771),
    [sym_source_type_neuron_id] = ACTIONS(2771),
    [sym_source_type_ip_address] = ACTIONS(2771),
    [sym_source_type_axlink] = ACTIONS(2771),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2771),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2771),
    [sym_source_type_ip_socket_address] = ACTIONS(2771),
    [sym_source_type_rs232] = ACTIONS(2771),
    [sym_source_type_internal] = ACTIONS(2771),
    [sym_source_type_mac_address] = ACTIONS(2771),
    [sym_source_type_ipv4_port] = ACTIONS(2771),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2771),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2771),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2771),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2771),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2771),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2771),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2771),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2771),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2771),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2771),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2771),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2771),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2771),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2771),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2771),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2771),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2771),
    [sym_true] = ACTIONS(2771),
    [sym_false] = ACTIONS(2771),
    [sym_file_read_only] = ACTIONS(2771),
    [sym_file_rw_new] = ACTIONS(2771),
    [sym_file_rw_append] = ACTIONS(2771),
    [sym_ip_tcp] = ACTIONS(2771),
    [sym_ip_udp] = ACTIONS(2771),
    [sym_ip_udp_2way] = ACTIONS(2771),
    [sym_xml_encode_types] = ACTIONS(2771),
    [sym_xml_encode_char_as_list] = ACTIONS(2771),
    [sym_xml_encode_le] = ACTIONS(2771),
    [sym_xml_decode_types] = ACTIONS(2771),
    [sym_xml_decode_no_preserve] = ACTIONS(2771),
    [sym_duet_dev_type_display_device] = ACTIONS(2771),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2771),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2771),
    [sym_duet_dev_type_io_device] = ACTIONS(2771),
    [sym_duet_dev_type_relay_device] = ACTIONS(2771),
    [sym_duet_dev_type_ups] = ACTIONS(2771),
    [sym_duet_dev_type_amplifier] = ACTIONS(2771),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2771),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2771),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2771),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2771),
    [sym_duet_dev_type_camera] = ACTIONS(2771),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2771),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2771),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2771),
    [sym_duet_dev_type_dss] = ACTIONS(2771),
    [sym_duet_dev_type_dvr] = ACTIONS(2771),
    [sym_duet_dev_type_disc_device] = ACTIONS(2771),
    [sym_duet_dev_type_document_camera] = ACTIONS(2771),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2771),
    [sym_duet_dev_type_hvac] = ACTIONS(2771),
    [sym_duet_dev_type_keypad] = ACTIONS(2771),
    [sym_duet_dev_type_light] = ACTIONS(2771),
    [sym_duet_dev_type_monitor] = ACTIONS(2771),
    [sym_duet_dev_type_motor] = ACTIONS(2771),
    [sym_duet_dev_type_multi_window] = ACTIONS(2771),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2771),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2771),
    [sym_duet_dev_type_receiver] = ACTIONS(2771),
    [sym_duet_dev_type_security_system] = ACTIONS(2771),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2771),
    [sym_duet_dev_type_settop_box] = ACTIONS(2771),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2771),
    [sym_duet_dev_type_switcher] = ACTIONS(2771),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2771),
    [sym_duet_dev_type_tv] = ACTIONS(2771),
    [sym_duet_dev_type_utility] = ACTIONS(2771),
    [sym_duet_dev_type_vcr] = ACTIONS(2771),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2771),
    [sym_duet_dev_type_video_processor] = ACTIONS(2771),
    [sym_duet_dev_type_video_projector] = ACTIONS(2771),
    [sym_duet_dev_type_video_wall] = ACTIONS(2771),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2771),
    [sym_duet_dev_type_weather] = ACTIONS(2771),
    [sym_duet_dev_not_polled] = ACTIONS(2771),
    [sym_duet_dev_polled] = ACTIONS(2771),
    [sym_clkmgr_mode_network] = ACTIONS(2771),
    [sym_clkmgr_mode_standalone] = ACTIONS(2771),
    [sym_normal_standby] = ACTIONS(2771),
    [sym_normal_wake] = ACTIONS(2771),
    [sym_ip_multicast_ttl_option] = ACTIONS(2771),
    [sym_ip_ttl_subnet] = ACTIONS(2771),
    [sym_ip_ttl_site] = ACTIONS(2771),
    [sym_ip_ttl_region] = ACTIONS(2771),
    [sym_ip_ttl_continent] = ACTIONS(2771),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2771),
    [sym_ip_nodelay_on] = ACTIONS(2771),
    [sym_ip_nodelay_off] = ACTIONS(2771),
    [sym_amx_error] = ACTIONS(2771),
    [sym_amx_warning] = ACTIONS(2771),
    [sym_amx_info] = ACTIONS(2771),
    [sym_amx_debug] = ACTIONS(2771),
    [sym_netlinx_login_success] = ACTIONS(2771),
    [sym_netlinx_login_fail] = ACTIONS(2771),
    [sym_netlinx_logout] = ACTIONS(2771),
    [sym_null_str] = ACTIONS(2771),
    [sym_smtp_address] = ACTIONS(2771),
    [sym_smtp_port_number] = ACTIONS(2771),
    [sym_smtp_username] = ACTIONS(2771),
    [sym_smtp_password] = ACTIONS(2771),
    [sym_smtp_require_tls] = ACTIONS(2771),
    [sym_smtp_from] = ACTIONS(2771),
    [sym_smtp_tls_true] = ACTIONS(2771),
    [sym_smtp_tls_false] = ACTIONS(2771),
    [sym_listview_on_row_select_event] = ACTIONS(2771),
    [sym_data_structure_unknown] = ACTIONS(2771),
    [sym_data_structure_datafeed] = ACTIONS(2771),
    [sym_data_structure_datarecordset] = ACTIONS(2771),
    [sym_data_structure_datarecord] = ACTIONS(2771),
    [sym_data_structure_datafield] = ACTIONS(2771),
    [sym_data_max_content_field_count] = ACTIONS(2771),
    [sym_data_max_metadata_field_count] = ACTIONS(2771),
    [sym_data_max_id_length] = ACTIONS(2771),
    [sym_data_max_name_length] = ACTIONS(2771),
    [sym_data_max_description_length] = ACTIONS(2771),
    [sym_data_max_type_length] = ACTIONS(2771),
    [sym_data_max_format_length] = ACTIONS(2771),
    [sym_data_max_source_length] = ACTIONS(2771),
    [sym_data_max_value_length] = ACTIONS(2771),
    [sym_data_max_label_length] = ACTIONS(2771),
    [sym_data_max_url_length] = ACTIONS(2771),
    [sym_max_last_login_info_length] = ACTIONS(2771),
    [sym_data_type_unknown] = ACTIONS(2771),
    [sym_data_type_string] = ACTIONS(2771),
    [sym_data_type_datetime] = ACTIONS(2771),
    [sym_data_type_date] = ACTIONS(2771),
    [sym_data_type_time] = ACTIONS(2771),
    [sym_data_type_image] = ACTIONS(2771),
    [sym_data_format_url] = ACTIONS(2771),
    [sym_data_format_phone] = ACTIONS(2771),
    [sym_data_format_email] = ACTIONS(2771),
    [sym_data_format_iso8601] = ACTIONS(2771),
    [sym_auditlog_priv] = ACTIONS(2771),
    [sym_configuration_priv] = ACTIONS(2771),
    [sym_deviceconfig_priv] = ACTIONS(2771),
    [sym_ftp_priv] = ACTIONS(2771),
    [sym_http_priv] = ACTIONS(2771),
    [sym_networkconfig_priv] = ACTIONS(2771),
    [sym_programport_priv] = ACTIONS(2771),
    [sym_remoteui_priv] = ACTIONS(2771),
    [sym_securitycontrol_priv] = ACTIONS(2771),
    [sym_softwaremanagement_priv] = ACTIONS(2771),
    [sym_terminal_priv] = ACTIONS(2771),
    [sym_tpadmin_priv] = ACTIONS(2771),
    [sym_user1_priv] = ACTIONS(2771),
    [sym_user2_priv] = ACTIONS(2771),
    [sym_user3_priv] = ACTIONS(2771),
    [sym_user4_priv] = ACTIONS(2771),
    [sym_usermanagement_priv] = ACTIONS(2771),
    [sym_valid_account] = ACTIONS(2771),
    [sym_err_name_invalid] = ACTIONS(2771),
    [sym_err_password_invalid] = ACTIONS(2771),
    [sym_err_invalid_account] = ACTIONS(2771),
    [sym_err_no_authorization] = ACTIONS(2771),
    [sym_err_invalid_parameter] = ACTIONS(2771),
    [sym_err_user_locked_out] = ACTIONS(2771),
    [sym_err_user_account_expired] = ACTIONS(2771),
    [sym_err_general_error] = ACTIONS(2771),
    [sym_err_authentication_error] = ACTIONS(2771),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2771),
    [sym_tls_validate_certificate] = ACTIONS(2771),
    [sym____reserved___] = ACTIONS(2771),
    [sym_date] = ACTIONS(2771),
    [sym_day] = ACTIONS(2771),
    [sym_time] = ACTIONS(2771),
    [sym_ldate] = ACTIONS(2771),
    [sym_push_device] = ACTIONS(2771),
    [sym_push_channel] = ACTIONS(2771),
    [sym_push_devchan] = ACTIONS(2771),
    [sym_release_device] = ACTIONS(2771),
    [sym_release_channel] = ACTIONS(2771),
    [sym_release_devchan] = ACTIONS(2771),
    [sym_master_slot] = ACTIONS(2771),
    [sym_get_pulse_time] = ACTIONS(2771),
    [sym_get_timer] = ACTIONS(2771),
    [sym_master_sn] = ACTIONS(2771),
    [sym_system_number] = ACTIONS(2771),
    [sym_button] = ACTIONS(2771),
    [sym_dv_channel] = ACTIONS(2771),
    [sym_data] = ACTIONS(2771),
    [sym_level] = ACTIONS(2771),
    [sym_timeline] = ACTIONS(2771),
    [sym_channel] = ACTIONS(2771),
    [sym_custom] = ACTIONS(2771),
    [sym_atoi] = ACTIONS(2771),
    [sym_atol] = ACTIONS(2771),
    [sym_atof] = ACTIONS(2771),
    [sym_itoa] = ACTIONS(2771),
    [sym_itohex] = ACTIONS(2771),
    [sym_hextoi] = ACTIONS(2771),
    [sym_ftoa] = ACTIONS(2771),
    [sym_find_string] = ACTIONS(2771),
    [sym_left_string] = ACTIONS(2771),
    [sym_right_string] = ACTIONS(2771),
    [sym_mid_string] = ACTIONS(2771),
    [sym_remove_string] = ACTIONS(2771),
    [sym_get_buffer_string] = ACTIONS(2771),
    [sym_get_buffer_char] = ACTIONS(2771),
    [sym_get_multi_buffer_string] = ACTIONS(2771),
    [sym_length_string] = ACTIONS(2771),
    [sym_max_length_string] = ACTIONS(2771),
    [sym_set_length_string] = ACTIONS(2771),
    [sym_compare_string] = ACTIONS(2771),
    [sym_upper_string] = ACTIONS(2771),
    [sym_lower_string] = ACTIONS(2771),
    [sym_get_last] = ACTIONS(2771),
    [sym_date_to_day] = ACTIONS(2771),
    [sym_date_to_month] = ACTIONS(2771),
    [sym_date_to_year] = ACTIONS(2771),
    [sym_time_to_hour] = ACTIONS(2771),
    [sym_time_to_minute] = ACTIONS(2771),
    [sym_time_to_second] = ACTIONS(2771),
    [sym_day_of_week] = ACTIONS(2771),
    [sym_set_pulse_time] = ACTIONS(2771),
    [sym_set_timer] = ACTIONS(2771),
    [sym_file_open] = ACTIONS(2771),
    [sym_file_close] = ACTIONS(2771),
    [sym_file_read] = ACTIONS(2771),
    [sym_file_read_line] = ACTIONS(2771),
    [sym_file_write] = ACTIONS(2771),
    [sym_file_write_line] = ACTIONS(2771),
    [sym_file_copy] = ACTIONS(2771),
    [sym_file_delete] = ACTIONS(2771),
    [sym_file_rename] = ACTIONS(2771),
    [sym_file_dir] = ACTIONS(2771),
    [sym_file_getdir] = ACTIONS(2771),
    [sym_file_setdir] = ACTIONS(2771),
    [sym_file_create_dir] = ACTIONS(2771),
    [sym_file_remove_dir] = ACTIONS(2771),
    [sym_file_seek] = ACTIONS(2771),
    [sym_length_array] = ACTIONS(2771),
    [sym_max_length_array] = ACTIONS(2771),
    [sym_set_length_array] = ACTIONS(2771),
    [sym_ip_client_open] = ACTIONS(2771),
    [sym_ip_client_close] = ACTIONS(2771),
    [sym_ip_server_open] = ACTIONS(2771),
    [sym_ip_server_close] = ACTIONS(2771),
    [sym_ip_bound_client_open] = ACTIONS(2771),
    [sym_ip_mc_server_open] = ACTIONS(2771),
    [sym_ip_set_option] = ACTIONS(2771),
    [sym_ssh_client_open] = ACTIONS(2771),
    [sym_ssh_client_close] = ACTIONS(2771),
    [sym_tls_client_open] = ACTIONS(2771),
    [sym_tls_client_close] = ACTIONS(2771),
    [sym_do_push] = ACTIONS(2771),
    [sym_do_push_timed] = ACTIONS(2771),
    [sym_do_release] = ACTIONS(2771),
    [sym_device_id] = ACTIONS(2771),
    [sym_device_info] = ACTIONS(2771),
    [sym_device_id_string] = ACTIONS(2771),
    [sym_device_standby] = ACTIONS(2771),
    [sym_device_wake] = ACTIONS(2771),
    [sym_timeline_create] = ACTIONS(2771),
    [sym_timeline_kill] = ACTIONS(2771),
    [sym_timeline_reload] = ACTIONS(2771),
    [sym_timeline_pause] = ACTIONS(2771),
    [sym_timeline_restart] = ACTIONS(2771),
    [sym_timeline_set] = ACTIONS(2771),
    [sym_timeline_get] = ACTIONS(2771),
    [sym_timeline_active] = ACTIONS(2771),
    [sym_timeline_dynamic_id] = ACTIONS(2771),
    [sym_variable_to_string] = ACTIONS(2771),
    [sym_string_to_variable] = ACTIONS(2771),
    [sym_length_variable_to_string] = ACTIONS(2771),
    [sym_type_cast] = ACTIONS(2771),
    [sym_variable_to_xml] = ACTIONS(2771),
    [sym_xml_to_variable] = ACTIONS(2771),
    [sym_length_variable_to_xml] = ACTIONS(2771),
    [sym_redirect_string] = ACTIONS(2771),
    [sym_random_number] = ACTIONS(2771),
    [sym_get_unique_id] = ACTIONS(2771),
    [sym_get_system_number] = ACTIONS(2771),
    [sym_set_system_number] = ACTIONS(2771),
    [sym_get_serial_number] = ACTIONS(2771),
    [sym_get_master_build] = ACTIONS(2771),
    [sym_set_validation_code] = ACTIONS(2771),
    [sym_reboot] = ACTIONS(2771),
    [sym_load_duet_module] = ACTIONS(2771),
    [sym_unload_duet_module] = ACTIONS(2771),
    [sym_duet_mem_size_set] = ACTIONS(2771),
    [sym_duet_mem_size_get] = ACTIONS(2771),
    [sym_dynamic_application_device] = ACTIONS(2771),
    [sym_dynamic_polled_port] = ACTIONS(2771),
    [sym_static_port_binding] = ACTIONS(2771),
    [sym_static_ip_binding] = ACTIONS(2771),
    [sym_get_dns_list] = ACTIONS(2771),
    [sym_set_dns_list] = ACTIONS(2771),
    [sym_get_ip_address] = ACTIONS(2771),
    [sym_set_ip_address] = ACTIONS(2771),
    [sym_get_url_list] = ACTIONS(2771),
    [sym_add_url_entry] = ACTIONS(2771),
    [sym_delete_url_entry] = ACTIONS(2771),
    [sym_abs_value] = ACTIONS(2771),
    [sym_max_value] = ACTIONS(2771),
    [sym_min_value] = ACTIONS(2771),
    [sym_exp_value] = ACTIONS(2771),
    [sym_log_value] = ACTIONS(2771),
    [sym_log10_value] = ACTIONS(2771),
    [sym_power_value] = ACTIONS(2771),
    [sym_sqrt_value] = ACTIONS(2771),
    [sym_raw_be] = ACTIONS(2771),
    [sym_raw_le] = ACTIONS(2771),
    [sym_internal_queue_size_set] = ACTIONS(2771),
    [sym_internal_queue_size_get] = ACTIONS(2771),
    [sym_internal_threshold_set] = ACTIONS(2771),
    [sym_internal_threshold_get] = ACTIONS(2771),
    [sym_rebuilt_event] = ACTIONS(2771),
    [sym_do_custom_event] = ACTIONS(2771),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2771),
    [sym_clkmgr_set_clk_source] = ACTIONS(2771),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2771),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2771),
    [sym_clkmgr_get_timezone] = ACTIONS(2771),
    [sym_clkmgr_set_timezone] = ACTIONS(2771),
    [sym_clkmgr_get_resync_period] = ACTIONS(2771),
    [sym_clkmgr_set_resync_period] = ACTIONS(2771),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2771),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2771),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2771),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2771),
    [sym_clkmgr_get_timeservers] = ACTIONS(2771),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2771),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2771),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2771),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2771),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2771),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2771),
    [sym_set_log_level] = ACTIONS(2771),
    [sym_get_log_level] = ACTIONS(2771),
    [sym_amx_log] = ACTIONS(2771),
    [sym_smtp_server_config_set] = ACTIONS(2771),
    [sym_smtp_server_config_get] = ACTIONS(2771),
    [sym_smtp_send] = ACTIONS(2771),
    [sym_validate_netlinx_account] = ACTIONS(2771),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2771),
    [sym_authenticate_certificate] = ACTIONS(2771),
    [sym_audit_netlinx_session_event] = ACTIONS(2771),
    [sym_audit_netlinx_generic_event] = ACTIONS(2771),
    [sym_get_available_flash_disk_space] = ACTIONS(2771),
    [sym_get_max_flash_disk_space] = ACTIONS(2771),
    [sym_data_create_feed] = ACTIONS(2771),
    [sym_data_delete_feed] = ACTIONS(2771),
    [sym_data_publish_feed] = ACTIONS(2771),
    [sym_data_get_published_feed] = ACTIONS(2771),
    [sym_data_add_record] = ACTIONS(2771),
    [sym_data_get_event_record] = ACTIONS(2771),
    [sym__wc_data_create_feed] = ACTIONS(2771),
    [sym__wc_data_add_record] = ACTIONS(2771),
    [sym__wc_data_get_event_record] = ACTIONS(2771),
    [sym_format] = ACTIONS(2771),
    [sym_astro_clock] = ACTIONS(2771),
    [sym_set_outdoor_temperature] = ACTIONS(2771),
    [sym_set_virtual_level_count] = ACTIONS(2771),
    [sym_set_virtual_channel_count] = ACTIONS(2771),
    [sym_set_virtual_port_count] = ACTIONS(2771),
    [sym__wc_encode_int] = ACTIONS(2771),
    [sym__wc_decode_int] = ACTIONS(2771),
    [sym__wc_int] = ACTIONS(2771),
    [sym__wc_to_ch_int] = ACTIONS(2771),
    [sym__ch_to_wc_int] = ACTIONS(2771),
    [sym__wc_find_string_int] = ACTIONS(2771),
    [sym__wc_left_string_int] = ACTIONS(2771),
    [sym__wc_lower_string_int] = ACTIONS(2771),
    [sym__wc_mid_string_int] = ACTIONS(2771),
    [sym__wc_remove_string_int] = ACTIONS(2771),
    [sym__wc_right_string_int] = ACTIONS(2771),
    [sym__wc_upper_string_int] = ACTIONS(2771),
    [sym__wc_compare_string_int] = ACTIONS(2771),
    [sym__wc_get_buffer_char_int] = ACTIONS(2771),
    [sym__wc_get_buffer_string_int] = ACTIONS(2771),
    [sym__wc_concat_string_int] = ACTIONS(2771),
    [sym__wc_file_open_int] = ACTIONS(2771),
    [sym__wc_file_close_int] = ACTIONS(2771),
    [sym__wc_file_read_int] = ACTIONS(2771),
    [sym__wc_file_read_line_int] = ACTIONS(2771),
    [sym__wc_file_write_int] = ACTIONS(2771),
    [sym__wc_file_write_line_int] = ACTIONS(2771),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2774),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2774),
    [anon_sym_LBRACK] = ACTIONS(2774),
    [anon_sym_LBRACE] = ACTIONS(2774),
    [anon_sym_RBRACE] = ACTIONS(2774),
    [anon_sym_LPAREN] = ACTIONS(2771),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_DASH] = ACTIONS(2771),
    [anon_sym_PLUS] = ACTIONS(2771),
    [anon_sym_DASH_DASH] = ACTIONS(2774),
    [anon_sym_PLUS_PLUS] = ACTIONS(2774),
    [anon_sym_DQUOTE] = ACTIONS(2774),
    [anon_sym_SQUOTE] = ACTIONS(2774),
    [sym_number_literal] = ACTIONS(2774),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(927)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2235),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(928)] = {
    [sym_identifier] = ACTIONS(2777),
    [sym_not] = ACTIONS(2777),
    [sym_bnot] = ACTIONS(2777),
    [sym___file__] = ACTIONS(2777),
    [sym___date__] = ACTIONS(2777),
    [sym___ldate__] = ACTIONS(2777),
    [sym___time__] = ACTIONS(2777),
    [sym___line__] = ACTIONS(2777),
    [sym___name__] = ACTIONS(2777),
    [sym___netlinx__] = ACTIONS(2777),
    [sym_netlinx_axi_version] = ACTIONS(2777),
    [sym_url_flg_tcp] = ACTIONS(2777),
    [sym_url_flg_accinfopresent] = ACTIONS(2777),
    [sym_url_flg_temp] = ACTIONS(2777),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2777),
    [sym_url_flg_stat_mask] = ACTIONS(2777),
    [sym_url_flg_stat_lookup] = ACTIONS(2777),
    [sym_url_flg_stat_connecting] = ACTIONS(2777),
    [sym_url_flg_stat_waiting] = ACTIONS(2777),
    [sym_url_flg_stat_connected] = ACTIONS(2777),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2777),
    [sym_first_virtual_device] = ACTIONS(2777),
    [sym_first_local_port] = ACTIONS(2777),
    [sym_dynamic_virtual_device] = ACTIONS(2777),
    [sym_timeline_once] = ACTIONS(2777),
    [sym_timeline_repeat] = ACTIONS(2777),
    [sym_timeline_absolute] = ACTIONS(2777),
    [sym_timeline_relative] = ACTIONS(2777),
    [sym_do_push_timed_infinite] = ACTIONS(2777),
    [sym_source_type_no_address] = ACTIONS(2777),
    [sym_source_type_neuron_id] = ACTIONS(2777),
    [sym_source_type_ip_address] = ACTIONS(2777),
    [sym_source_type_axlink] = ACTIONS(2777),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2777),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2777),
    [sym_source_type_ip_socket_address] = ACTIONS(2777),
    [sym_source_type_rs232] = ACTIONS(2777),
    [sym_source_type_internal] = ACTIONS(2777),
    [sym_source_type_mac_address] = ACTIONS(2777),
    [sym_source_type_ipv4_port] = ACTIONS(2777),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2777),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2777),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2777),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2777),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2777),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2777),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2777),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2777),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2777),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2777),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2777),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2777),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2777),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2777),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2777),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2777),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2777),
    [sym_true] = ACTIONS(2777),
    [sym_false] = ACTIONS(2777),
    [sym_file_read_only] = ACTIONS(2777),
    [sym_file_rw_new] = ACTIONS(2777),
    [sym_file_rw_append] = ACTIONS(2777),
    [sym_ip_tcp] = ACTIONS(2777),
    [sym_ip_udp] = ACTIONS(2777),
    [sym_ip_udp_2way] = ACTIONS(2777),
    [sym_xml_encode_types] = ACTIONS(2777),
    [sym_xml_encode_char_as_list] = ACTIONS(2777),
    [sym_xml_encode_le] = ACTIONS(2777),
    [sym_xml_decode_types] = ACTIONS(2777),
    [sym_xml_decode_no_preserve] = ACTIONS(2777),
    [sym_duet_dev_type_display_device] = ACTIONS(2777),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2777),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2777),
    [sym_duet_dev_type_io_device] = ACTIONS(2777),
    [sym_duet_dev_type_relay_device] = ACTIONS(2777),
    [sym_duet_dev_type_ups] = ACTIONS(2777),
    [sym_duet_dev_type_amplifier] = ACTIONS(2777),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2777),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2777),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2777),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2777),
    [sym_duet_dev_type_camera] = ACTIONS(2777),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2777),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2777),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2777),
    [sym_duet_dev_type_dss] = ACTIONS(2777),
    [sym_duet_dev_type_dvr] = ACTIONS(2777),
    [sym_duet_dev_type_disc_device] = ACTIONS(2777),
    [sym_duet_dev_type_document_camera] = ACTIONS(2777),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2777),
    [sym_duet_dev_type_hvac] = ACTIONS(2777),
    [sym_duet_dev_type_keypad] = ACTIONS(2777),
    [sym_duet_dev_type_light] = ACTIONS(2777),
    [sym_duet_dev_type_monitor] = ACTIONS(2777),
    [sym_duet_dev_type_motor] = ACTIONS(2777),
    [sym_duet_dev_type_multi_window] = ACTIONS(2777),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2777),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2777),
    [sym_duet_dev_type_receiver] = ACTIONS(2777),
    [sym_duet_dev_type_security_system] = ACTIONS(2777),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2777),
    [sym_duet_dev_type_settop_box] = ACTIONS(2777),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2777),
    [sym_duet_dev_type_switcher] = ACTIONS(2777),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2777),
    [sym_duet_dev_type_tv] = ACTIONS(2777),
    [sym_duet_dev_type_utility] = ACTIONS(2777),
    [sym_duet_dev_type_vcr] = ACTIONS(2777),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2777),
    [sym_duet_dev_type_video_processor] = ACTIONS(2777),
    [sym_duet_dev_type_video_projector] = ACTIONS(2777),
    [sym_duet_dev_type_video_wall] = ACTIONS(2777),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2777),
    [sym_duet_dev_type_weather] = ACTIONS(2777),
    [sym_duet_dev_not_polled] = ACTIONS(2777),
    [sym_duet_dev_polled] = ACTIONS(2777),
    [sym_clkmgr_mode_network] = ACTIONS(2777),
    [sym_clkmgr_mode_standalone] = ACTIONS(2777),
    [sym_normal_standby] = ACTIONS(2777),
    [sym_normal_wake] = ACTIONS(2777),
    [sym_ip_multicast_ttl_option] = ACTIONS(2777),
    [sym_ip_ttl_subnet] = ACTIONS(2777),
    [sym_ip_ttl_site] = ACTIONS(2777),
    [sym_ip_ttl_region] = ACTIONS(2777),
    [sym_ip_ttl_continent] = ACTIONS(2777),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2777),
    [sym_ip_nodelay_on] = ACTIONS(2777),
    [sym_ip_nodelay_off] = ACTIONS(2777),
    [sym_amx_error] = ACTIONS(2777),
    [sym_amx_warning] = ACTIONS(2777),
    [sym_amx_info] = ACTIONS(2777),
    [sym_amx_debug] = ACTIONS(2777),
    [sym_netlinx_login_success] = ACTIONS(2777),
    [sym_netlinx_login_fail] = ACTIONS(2777),
    [sym_netlinx_logout] = ACTIONS(2777),
    [sym_null_str] = ACTIONS(2777),
    [sym_smtp_address] = ACTIONS(2777),
    [sym_smtp_port_number] = ACTIONS(2777),
    [sym_smtp_username] = ACTIONS(2777),
    [sym_smtp_password] = ACTIONS(2777),
    [sym_smtp_require_tls] = ACTIONS(2777),
    [sym_smtp_from] = ACTIONS(2777),
    [sym_smtp_tls_true] = ACTIONS(2777),
    [sym_smtp_tls_false] = ACTIONS(2777),
    [sym_listview_on_row_select_event] = ACTIONS(2777),
    [sym_data_structure_unknown] = ACTIONS(2777),
    [sym_data_structure_datafeed] = ACTIONS(2777),
    [sym_data_structure_datarecordset] = ACTIONS(2777),
    [sym_data_structure_datarecord] = ACTIONS(2777),
    [sym_data_structure_datafield] = ACTIONS(2777),
    [sym_data_max_content_field_count] = ACTIONS(2777),
    [sym_data_max_metadata_field_count] = ACTIONS(2777),
    [sym_data_max_id_length] = ACTIONS(2777),
    [sym_data_max_name_length] = ACTIONS(2777),
    [sym_data_max_description_length] = ACTIONS(2777),
    [sym_data_max_type_length] = ACTIONS(2777),
    [sym_data_max_format_length] = ACTIONS(2777),
    [sym_data_max_source_length] = ACTIONS(2777),
    [sym_data_max_value_length] = ACTIONS(2777),
    [sym_data_max_label_length] = ACTIONS(2777),
    [sym_data_max_url_length] = ACTIONS(2777),
    [sym_max_last_login_info_length] = ACTIONS(2777),
    [sym_data_type_unknown] = ACTIONS(2777),
    [sym_data_type_string] = ACTIONS(2777),
    [sym_data_type_datetime] = ACTIONS(2777),
    [sym_data_type_date] = ACTIONS(2777),
    [sym_data_type_time] = ACTIONS(2777),
    [sym_data_type_image] = ACTIONS(2777),
    [sym_data_format_url] = ACTIONS(2777),
    [sym_data_format_phone] = ACTIONS(2777),
    [sym_data_format_email] = ACTIONS(2777),
    [sym_data_format_iso8601] = ACTIONS(2777),
    [sym_auditlog_priv] = ACTIONS(2777),
    [sym_configuration_priv] = ACTIONS(2777),
    [sym_deviceconfig_priv] = ACTIONS(2777),
    [sym_ftp_priv] = ACTIONS(2777),
    [sym_http_priv] = ACTIONS(2777),
    [sym_networkconfig_priv] = ACTIONS(2777),
    [sym_programport_priv] = ACTIONS(2777),
    [sym_remoteui_priv] = ACTIONS(2777),
    [sym_securitycontrol_priv] = ACTIONS(2777),
    [sym_softwaremanagement_priv] = ACTIONS(2777),
    [sym_terminal_priv] = ACTIONS(2777),
    [sym_tpadmin_priv] = ACTIONS(2777),
    [sym_user1_priv] = ACTIONS(2777),
    [sym_user2_priv] = ACTIONS(2777),
    [sym_user3_priv] = ACTIONS(2777),
    [sym_user4_priv] = ACTIONS(2777),
    [sym_usermanagement_priv] = ACTIONS(2777),
    [sym_valid_account] = ACTIONS(2777),
    [sym_err_name_invalid] = ACTIONS(2777),
    [sym_err_password_invalid] = ACTIONS(2777),
    [sym_err_invalid_account] = ACTIONS(2777),
    [sym_err_no_authorization] = ACTIONS(2777),
    [sym_err_invalid_parameter] = ACTIONS(2777),
    [sym_err_user_locked_out] = ACTIONS(2777),
    [sym_err_user_account_expired] = ACTIONS(2777),
    [sym_err_general_error] = ACTIONS(2777),
    [sym_err_authentication_error] = ACTIONS(2777),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2777),
    [sym_tls_validate_certificate] = ACTIONS(2777),
    [sym____reserved___] = ACTIONS(2777),
    [sym_date] = ACTIONS(2777),
    [sym_day] = ACTIONS(2777),
    [sym_time] = ACTIONS(2777),
    [sym_ldate] = ACTIONS(2777),
    [sym_push_device] = ACTIONS(2777),
    [sym_push_channel] = ACTIONS(2777),
    [sym_push_devchan] = ACTIONS(2777),
    [sym_release_device] = ACTIONS(2777),
    [sym_release_channel] = ACTIONS(2777),
    [sym_release_devchan] = ACTIONS(2777),
    [sym_master_slot] = ACTIONS(2777),
    [sym_get_pulse_time] = ACTIONS(2777),
    [sym_get_timer] = ACTIONS(2777),
    [sym_master_sn] = ACTIONS(2777),
    [sym_system_number] = ACTIONS(2777),
    [sym_button] = ACTIONS(2777),
    [sym_dv_channel] = ACTIONS(2777),
    [sym_data] = ACTIONS(2777),
    [sym_level] = ACTIONS(2777),
    [sym_timeline] = ACTIONS(2777),
    [sym_channel] = ACTIONS(2777),
    [sym_custom] = ACTIONS(2777),
    [sym_atoi] = ACTIONS(2777),
    [sym_atol] = ACTIONS(2777),
    [sym_atof] = ACTIONS(2777),
    [sym_itoa] = ACTIONS(2777),
    [sym_itohex] = ACTIONS(2777),
    [sym_hextoi] = ACTIONS(2777),
    [sym_ftoa] = ACTIONS(2777),
    [sym_find_string] = ACTIONS(2777),
    [sym_left_string] = ACTIONS(2777),
    [sym_right_string] = ACTIONS(2777),
    [sym_mid_string] = ACTIONS(2777),
    [sym_remove_string] = ACTIONS(2777),
    [sym_get_buffer_string] = ACTIONS(2777),
    [sym_get_buffer_char] = ACTIONS(2777),
    [sym_get_multi_buffer_string] = ACTIONS(2777),
    [sym_length_string] = ACTIONS(2777),
    [sym_max_length_string] = ACTIONS(2777),
    [sym_set_length_string] = ACTIONS(2777),
    [sym_compare_string] = ACTIONS(2777),
    [sym_upper_string] = ACTIONS(2777),
    [sym_lower_string] = ACTIONS(2777),
    [sym_get_last] = ACTIONS(2777),
    [sym_date_to_day] = ACTIONS(2777),
    [sym_date_to_month] = ACTIONS(2777),
    [sym_date_to_year] = ACTIONS(2777),
    [sym_time_to_hour] = ACTIONS(2777),
    [sym_time_to_minute] = ACTIONS(2777),
    [sym_time_to_second] = ACTIONS(2777),
    [sym_day_of_week] = ACTIONS(2777),
    [sym_set_pulse_time] = ACTIONS(2777),
    [sym_set_timer] = ACTIONS(2777),
    [sym_file_open] = ACTIONS(2777),
    [sym_file_close] = ACTIONS(2777),
    [sym_file_read] = ACTIONS(2777),
    [sym_file_read_line] = ACTIONS(2777),
    [sym_file_write] = ACTIONS(2777),
    [sym_file_write_line] = ACTIONS(2777),
    [sym_file_copy] = ACTIONS(2777),
    [sym_file_delete] = ACTIONS(2777),
    [sym_file_rename] = ACTIONS(2777),
    [sym_file_dir] = ACTIONS(2777),
    [sym_file_getdir] = ACTIONS(2777),
    [sym_file_setdir] = ACTIONS(2777),
    [sym_file_create_dir] = ACTIONS(2777),
    [sym_file_remove_dir] = ACTIONS(2777),
    [sym_file_seek] = ACTIONS(2777),
    [sym_length_array] = ACTIONS(2777),
    [sym_max_length_array] = ACTIONS(2777),
    [sym_set_length_array] = ACTIONS(2777),
    [sym_ip_client_open] = ACTIONS(2777),
    [sym_ip_client_close] = ACTIONS(2777),
    [sym_ip_server_open] = ACTIONS(2777),
    [sym_ip_server_close] = ACTIONS(2777),
    [sym_ip_bound_client_open] = ACTIONS(2777),
    [sym_ip_mc_server_open] = ACTIONS(2777),
    [sym_ip_set_option] = ACTIONS(2777),
    [sym_ssh_client_open] = ACTIONS(2777),
    [sym_ssh_client_close] = ACTIONS(2777),
    [sym_tls_client_open] = ACTIONS(2777),
    [sym_tls_client_close] = ACTIONS(2777),
    [sym_do_push] = ACTIONS(2777),
    [sym_do_push_timed] = ACTIONS(2777),
    [sym_do_release] = ACTIONS(2777),
    [sym_device_id] = ACTIONS(2777),
    [sym_device_info] = ACTIONS(2777),
    [sym_device_id_string] = ACTIONS(2777),
    [sym_device_standby] = ACTIONS(2777),
    [sym_device_wake] = ACTIONS(2777),
    [sym_timeline_create] = ACTIONS(2777),
    [sym_timeline_kill] = ACTIONS(2777),
    [sym_timeline_reload] = ACTIONS(2777),
    [sym_timeline_pause] = ACTIONS(2777),
    [sym_timeline_restart] = ACTIONS(2777),
    [sym_timeline_set] = ACTIONS(2777),
    [sym_timeline_get] = ACTIONS(2777),
    [sym_timeline_active] = ACTIONS(2777),
    [sym_timeline_dynamic_id] = ACTIONS(2777),
    [sym_variable_to_string] = ACTIONS(2777),
    [sym_string_to_variable] = ACTIONS(2777),
    [sym_length_variable_to_string] = ACTIONS(2777),
    [sym_type_cast] = ACTIONS(2777),
    [sym_variable_to_xml] = ACTIONS(2777),
    [sym_xml_to_variable] = ACTIONS(2777),
    [sym_length_variable_to_xml] = ACTIONS(2777),
    [sym_redirect_string] = ACTIONS(2777),
    [sym_random_number] = ACTIONS(2777),
    [sym_get_unique_id] = ACTIONS(2777),
    [sym_get_system_number] = ACTIONS(2777),
    [sym_set_system_number] = ACTIONS(2777),
    [sym_get_serial_number] = ACTIONS(2777),
    [sym_get_master_build] = ACTIONS(2777),
    [sym_set_validation_code] = ACTIONS(2777),
    [sym_reboot] = ACTIONS(2777),
    [sym_load_duet_module] = ACTIONS(2777),
    [sym_unload_duet_module] = ACTIONS(2777),
    [sym_duet_mem_size_set] = ACTIONS(2777),
    [sym_duet_mem_size_get] = ACTIONS(2777),
    [sym_dynamic_application_device] = ACTIONS(2777),
    [sym_dynamic_polled_port] = ACTIONS(2777),
    [sym_static_port_binding] = ACTIONS(2777),
    [sym_static_ip_binding] = ACTIONS(2777),
    [sym_get_dns_list] = ACTIONS(2777),
    [sym_set_dns_list] = ACTIONS(2777),
    [sym_get_ip_address] = ACTIONS(2777),
    [sym_set_ip_address] = ACTIONS(2777),
    [sym_get_url_list] = ACTIONS(2777),
    [sym_add_url_entry] = ACTIONS(2777),
    [sym_delete_url_entry] = ACTIONS(2777),
    [sym_abs_value] = ACTIONS(2777),
    [sym_max_value] = ACTIONS(2777),
    [sym_min_value] = ACTIONS(2777),
    [sym_exp_value] = ACTIONS(2777),
    [sym_log_value] = ACTIONS(2777),
    [sym_log10_value] = ACTIONS(2777),
    [sym_power_value] = ACTIONS(2777),
    [sym_sqrt_value] = ACTIONS(2777),
    [sym_raw_be] = ACTIONS(2777),
    [sym_raw_le] = ACTIONS(2777),
    [sym_internal_queue_size_set] = ACTIONS(2777),
    [sym_internal_queue_size_get] = ACTIONS(2777),
    [sym_internal_threshold_set] = ACTIONS(2777),
    [sym_internal_threshold_get] = ACTIONS(2777),
    [sym_rebuilt_event] = ACTIONS(2777),
    [sym_do_custom_event] = ACTIONS(2777),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2777),
    [sym_clkmgr_set_clk_source] = ACTIONS(2777),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2777),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2777),
    [sym_clkmgr_get_timezone] = ACTIONS(2777),
    [sym_clkmgr_set_timezone] = ACTIONS(2777),
    [sym_clkmgr_get_resync_period] = ACTIONS(2777),
    [sym_clkmgr_set_resync_period] = ACTIONS(2777),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2777),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2777),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2777),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2777),
    [sym_clkmgr_get_timeservers] = ACTIONS(2777),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2777),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2777),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2777),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2777),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2777),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2777),
    [sym_set_log_level] = ACTIONS(2777),
    [sym_get_log_level] = ACTIONS(2777),
    [sym_amx_log] = ACTIONS(2777),
    [sym_smtp_server_config_set] = ACTIONS(2777),
    [sym_smtp_server_config_get] = ACTIONS(2777),
    [sym_smtp_send] = ACTIONS(2777),
    [sym_validate_netlinx_account] = ACTIONS(2777),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2777),
    [sym_authenticate_certificate] = ACTIONS(2777),
    [sym_audit_netlinx_session_event] = ACTIONS(2777),
    [sym_audit_netlinx_generic_event] = ACTIONS(2777),
    [sym_get_available_flash_disk_space] = ACTIONS(2777),
    [sym_get_max_flash_disk_space] = ACTIONS(2777),
    [sym_data_create_feed] = ACTIONS(2777),
    [sym_data_delete_feed] = ACTIONS(2777),
    [sym_data_publish_feed] = ACTIONS(2777),
    [sym_data_get_published_feed] = ACTIONS(2777),
    [sym_data_add_record] = ACTIONS(2777),
    [sym_data_get_event_record] = ACTIONS(2777),
    [sym__wc_data_create_feed] = ACTIONS(2777),
    [sym__wc_data_add_record] = ACTIONS(2777),
    [sym__wc_data_get_event_record] = ACTIONS(2777),
    [sym_format] = ACTIONS(2777),
    [sym_astro_clock] = ACTIONS(2777),
    [sym_set_outdoor_temperature] = ACTIONS(2777),
    [sym_set_virtual_level_count] = ACTIONS(2777),
    [sym_set_virtual_channel_count] = ACTIONS(2777),
    [sym_set_virtual_port_count] = ACTIONS(2777),
    [sym__wc_encode_int] = ACTIONS(2777),
    [sym__wc_decode_int] = ACTIONS(2777),
    [sym__wc_int] = ACTIONS(2777),
    [sym__wc_to_ch_int] = ACTIONS(2777),
    [sym__ch_to_wc_int] = ACTIONS(2777),
    [sym__wc_find_string_int] = ACTIONS(2777),
    [sym__wc_left_string_int] = ACTIONS(2777),
    [sym__wc_lower_string_int] = ACTIONS(2777),
    [sym__wc_mid_string_int] = ACTIONS(2777),
    [sym__wc_remove_string_int] = ACTIONS(2777),
    [sym__wc_right_string_int] = ACTIONS(2777),
    [sym__wc_upper_string_int] = ACTIONS(2777),
    [sym__wc_compare_string_int] = ACTIONS(2777),
    [sym__wc_get_buffer_char_int] = ACTIONS(2777),
    [sym__wc_get_buffer_string_int] = ACTIONS(2777),
    [sym__wc_concat_string_int] = ACTIONS(2777),
    [sym__wc_file_open_int] = ACTIONS(2777),
    [sym__wc_file_close_int] = ACTIONS(2777),
    [sym__wc_file_read_int] = ACTIONS(2777),
    [sym__wc_file_read_line_int] = ACTIONS(2777),
    [sym__wc_file_write_int] = ACTIONS(2777),
    [sym__wc_file_write_line_int] = ACTIONS(2777),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2779),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2779),
    [anon_sym_LBRACK] = ACTIONS(2779),
    [anon_sym_LBRACE] = ACTIONS(2779),
    [anon_sym_RBRACE] = ACTIONS(2779),
    [anon_sym_LPAREN] = ACTIONS(2777),
    [anon_sym_BANG] = ACTIONS(2779),
    [anon_sym_TILDE] = ACTIONS(2779),
    [anon_sym_DASH] = ACTIONS(2777),
    [anon_sym_PLUS] = ACTIONS(2777),
    [anon_sym_DASH_DASH] = ACTIONS(2779),
    [anon_sym_PLUS_PLUS] = ACTIONS(2779),
    [anon_sym_DQUOTE] = ACTIONS(2779),
    [anon_sym_SQUOTE] = ACTIONS(2779),
    [sym_number_literal] = ACTIONS(2779),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(929)] = {
    [sym_identifier] = ACTIONS(2781),
    [sym_not] = ACTIONS(2781),
    [sym_bnot] = ACTIONS(2781),
    [sym___file__] = ACTIONS(2781),
    [sym___date__] = ACTIONS(2781),
    [sym___ldate__] = ACTIONS(2781),
    [sym___time__] = ACTIONS(2781),
    [sym___line__] = ACTIONS(2781),
    [sym___name__] = ACTIONS(2781),
    [sym___netlinx__] = ACTIONS(2781),
    [sym_netlinx_axi_version] = ACTIONS(2781),
    [sym_url_flg_tcp] = ACTIONS(2781),
    [sym_url_flg_accinfopresent] = ACTIONS(2781),
    [sym_url_flg_temp] = ACTIONS(2781),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2781),
    [sym_url_flg_stat_mask] = ACTIONS(2781),
    [sym_url_flg_stat_lookup] = ACTIONS(2781),
    [sym_url_flg_stat_connecting] = ACTIONS(2781),
    [sym_url_flg_stat_waiting] = ACTIONS(2781),
    [sym_url_flg_stat_connected] = ACTIONS(2781),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2781),
    [sym_first_virtual_device] = ACTIONS(2781),
    [sym_first_local_port] = ACTIONS(2781),
    [sym_dynamic_virtual_device] = ACTIONS(2781),
    [sym_timeline_once] = ACTIONS(2781),
    [sym_timeline_repeat] = ACTIONS(2781),
    [sym_timeline_absolute] = ACTIONS(2781),
    [sym_timeline_relative] = ACTIONS(2781),
    [sym_do_push_timed_infinite] = ACTIONS(2781),
    [sym_source_type_no_address] = ACTIONS(2781),
    [sym_source_type_neuron_id] = ACTIONS(2781),
    [sym_source_type_ip_address] = ACTIONS(2781),
    [sym_source_type_axlink] = ACTIONS(2781),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2781),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2781),
    [sym_source_type_ip_socket_address] = ACTIONS(2781),
    [sym_source_type_rs232] = ACTIONS(2781),
    [sym_source_type_internal] = ACTIONS(2781),
    [sym_source_type_mac_address] = ACTIONS(2781),
    [sym_source_type_ipv4_port] = ACTIONS(2781),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2781),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2781),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2781),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2781),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2781),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2781),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2781),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2781),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2781),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2781),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2781),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2781),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2781),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2781),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2781),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2781),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2781),
    [sym_true] = ACTIONS(2781),
    [sym_false] = ACTIONS(2781),
    [sym_file_read_only] = ACTIONS(2781),
    [sym_file_rw_new] = ACTIONS(2781),
    [sym_file_rw_append] = ACTIONS(2781),
    [sym_ip_tcp] = ACTIONS(2781),
    [sym_ip_udp] = ACTIONS(2781),
    [sym_ip_udp_2way] = ACTIONS(2781),
    [sym_xml_encode_types] = ACTIONS(2781),
    [sym_xml_encode_char_as_list] = ACTIONS(2781),
    [sym_xml_encode_le] = ACTIONS(2781),
    [sym_xml_decode_types] = ACTIONS(2781),
    [sym_xml_decode_no_preserve] = ACTIONS(2781),
    [sym_duet_dev_type_display_device] = ACTIONS(2781),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2781),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2781),
    [sym_duet_dev_type_io_device] = ACTIONS(2781),
    [sym_duet_dev_type_relay_device] = ACTIONS(2781),
    [sym_duet_dev_type_ups] = ACTIONS(2781),
    [sym_duet_dev_type_amplifier] = ACTIONS(2781),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2781),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2781),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2781),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2781),
    [sym_duet_dev_type_camera] = ACTIONS(2781),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2781),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2781),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2781),
    [sym_duet_dev_type_dss] = ACTIONS(2781),
    [sym_duet_dev_type_dvr] = ACTIONS(2781),
    [sym_duet_dev_type_disc_device] = ACTIONS(2781),
    [sym_duet_dev_type_document_camera] = ACTIONS(2781),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2781),
    [sym_duet_dev_type_hvac] = ACTIONS(2781),
    [sym_duet_dev_type_keypad] = ACTIONS(2781),
    [sym_duet_dev_type_light] = ACTIONS(2781),
    [sym_duet_dev_type_monitor] = ACTIONS(2781),
    [sym_duet_dev_type_motor] = ACTIONS(2781),
    [sym_duet_dev_type_multi_window] = ACTIONS(2781),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2781),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2781),
    [sym_duet_dev_type_receiver] = ACTIONS(2781),
    [sym_duet_dev_type_security_system] = ACTIONS(2781),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2781),
    [sym_duet_dev_type_settop_box] = ACTIONS(2781),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2781),
    [sym_duet_dev_type_switcher] = ACTIONS(2781),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2781),
    [sym_duet_dev_type_tv] = ACTIONS(2781),
    [sym_duet_dev_type_utility] = ACTIONS(2781),
    [sym_duet_dev_type_vcr] = ACTIONS(2781),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2781),
    [sym_duet_dev_type_video_processor] = ACTIONS(2781),
    [sym_duet_dev_type_video_projector] = ACTIONS(2781),
    [sym_duet_dev_type_video_wall] = ACTIONS(2781),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2781),
    [sym_duet_dev_type_weather] = ACTIONS(2781),
    [sym_duet_dev_not_polled] = ACTIONS(2781),
    [sym_duet_dev_polled] = ACTIONS(2781),
    [sym_clkmgr_mode_network] = ACTIONS(2781),
    [sym_clkmgr_mode_standalone] = ACTIONS(2781),
    [sym_normal_standby] = ACTIONS(2781),
    [sym_normal_wake] = ACTIONS(2781),
    [sym_ip_multicast_ttl_option] = ACTIONS(2781),
    [sym_ip_ttl_subnet] = ACTIONS(2781),
    [sym_ip_ttl_site] = ACTIONS(2781),
    [sym_ip_ttl_region] = ACTIONS(2781),
    [sym_ip_ttl_continent] = ACTIONS(2781),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2781),
    [sym_ip_nodelay_on] = ACTIONS(2781),
    [sym_ip_nodelay_off] = ACTIONS(2781),
    [sym_amx_error] = ACTIONS(2781),
    [sym_amx_warning] = ACTIONS(2781),
    [sym_amx_info] = ACTIONS(2781),
    [sym_amx_debug] = ACTIONS(2781),
    [sym_netlinx_login_success] = ACTIONS(2781),
    [sym_netlinx_login_fail] = ACTIONS(2781),
    [sym_netlinx_logout] = ACTIONS(2781),
    [sym_null_str] = ACTIONS(2781),
    [sym_smtp_address] = ACTIONS(2781),
    [sym_smtp_port_number] = ACTIONS(2781),
    [sym_smtp_username] = ACTIONS(2781),
    [sym_smtp_password] = ACTIONS(2781),
    [sym_smtp_require_tls] = ACTIONS(2781),
    [sym_smtp_from] = ACTIONS(2781),
    [sym_smtp_tls_true] = ACTIONS(2781),
    [sym_smtp_tls_false] = ACTIONS(2781),
    [sym_listview_on_row_select_event] = ACTIONS(2781),
    [sym_data_structure_unknown] = ACTIONS(2781),
    [sym_data_structure_datafeed] = ACTIONS(2781),
    [sym_data_structure_datarecordset] = ACTIONS(2781),
    [sym_data_structure_datarecord] = ACTIONS(2781),
    [sym_data_structure_datafield] = ACTIONS(2781),
    [sym_data_max_content_field_count] = ACTIONS(2781),
    [sym_data_max_metadata_field_count] = ACTIONS(2781),
    [sym_data_max_id_length] = ACTIONS(2781),
    [sym_data_max_name_length] = ACTIONS(2781),
    [sym_data_max_description_length] = ACTIONS(2781),
    [sym_data_max_type_length] = ACTIONS(2781),
    [sym_data_max_format_length] = ACTIONS(2781),
    [sym_data_max_source_length] = ACTIONS(2781),
    [sym_data_max_value_length] = ACTIONS(2781),
    [sym_data_max_label_length] = ACTIONS(2781),
    [sym_data_max_url_length] = ACTIONS(2781),
    [sym_max_last_login_info_length] = ACTIONS(2781),
    [sym_data_type_unknown] = ACTIONS(2781),
    [sym_data_type_string] = ACTIONS(2781),
    [sym_data_type_datetime] = ACTIONS(2781),
    [sym_data_type_date] = ACTIONS(2781),
    [sym_data_type_time] = ACTIONS(2781),
    [sym_data_type_image] = ACTIONS(2781),
    [sym_data_format_url] = ACTIONS(2781),
    [sym_data_format_phone] = ACTIONS(2781),
    [sym_data_format_email] = ACTIONS(2781),
    [sym_data_format_iso8601] = ACTIONS(2781),
    [sym_auditlog_priv] = ACTIONS(2781),
    [sym_configuration_priv] = ACTIONS(2781),
    [sym_deviceconfig_priv] = ACTIONS(2781),
    [sym_ftp_priv] = ACTIONS(2781),
    [sym_http_priv] = ACTIONS(2781),
    [sym_networkconfig_priv] = ACTIONS(2781),
    [sym_programport_priv] = ACTIONS(2781),
    [sym_remoteui_priv] = ACTIONS(2781),
    [sym_securitycontrol_priv] = ACTIONS(2781),
    [sym_softwaremanagement_priv] = ACTIONS(2781),
    [sym_terminal_priv] = ACTIONS(2781),
    [sym_tpadmin_priv] = ACTIONS(2781),
    [sym_user1_priv] = ACTIONS(2781),
    [sym_user2_priv] = ACTIONS(2781),
    [sym_user3_priv] = ACTIONS(2781),
    [sym_user4_priv] = ACTIONS(2781),
    [sym_usermanagement_priv] = ACTIONS(2781),
    [sym_valid_account] = ACTIONS(2781),
    [sym_err_name_invalid] = ACTIONS(2781),
    [sym_err_password_invalid] = ACTIONS(2781),
    [sym_err_invalid_account] = ACTIONS(2781),
    [sym_err_no_authorization] = ACTIONS(2781),
    [sym_err_invalid_parameter] = ACTIONS(2781),
    [sym_err_user_locked_out] = ACTIONS(2781),
    [sym_err_user_account_expired] = ACTIONS(2781),
    [sym_err_general_error] = ACTIONS(2781),
    [sym_err_authentication_error] = ACTIONS(2781),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2781),
    [sym_tls_validate_certificate] = ACTIONS(2781),
    [sym____reserved___] = ACTIONS(2781),
    [sym_date] = ACTIONS(2781),
    [sym_day] = ACTIONS(2781),
    [sym_time] = ACTIONS(2781),
    [sym_ldate] = ACTIONS(2781),
    [sym_push_device] = ACTIONS(2781),
    [sym_push_channel] = ACTIONS(2781),
    [sym_push_devchan] = ACTIONS(2781),
    [sym_release_device] = ACTIONS(2781),
    [sym_release_channel] = ACTIONS(2781),
    [sym_release_devchan] = ACTIONS(2781),
    [sym_master_slot] = ACTIONS(2781),
    [sym_get_pulse_time] = ACTIONS(2781),
    [sym_get_timer] = ACTIONS(2781),
    [sym_master_sn] = ACTIONS(2781),
    [sym_system_number] = ACTIONS(2781),
    [sym_button] = ACTIONS(2781),
    [sym_dv_channel] = ACTIONS(2781),
    [sym_data] = ACTIONS(2781),
    [sym_level] = ACTIONS(2781),
    [sym_timeline] = ACTIONS(2781),
    [sym_channel] = ACTIONS(2781),
    [sym_custom] = ACTIONS(2781),
    [sym_atoi] = ACTIONS(2781),
    [sym_atol] = ACTIONS(2781),
    [sym_atof] = ACTIONS(2781),
    [sym_itoa] = ACTIONS(2781),
    [sym_itohex] = ACTIONS(2781),
    [sym_hextoi] = ACTIONS(2781),
    [sym_ftoa] = ACTIONS(2781),
    [sym_find_string] = ACTIONS(2781),
    [sym_left_string] = ACTIONS(2781),
    [sym_right_string] = ACTIONS(2781),
    [sym_mid_string] = ACTIONS(2781),
    [sym_remove_string] = ACTIONS(2781),
    [sym_get_buffer_string] = ACTIONS(2781),
    [sym_get_buffer_char] = ACTIONS(2781),
    [sym_get_multi_buffer_string] = ACTIONS(2781),
    [sym_length_string] = ACTIONS(2781),
    [sym_max_length_string] = ACTIONS(2781),
    [sym_set_length_string] = ACTIONS(2781),
    [sym_compare_string] = ACTIONS(2781),
    [sym_upper_string] = ACTIONS(2781),
    [sym_lower_string] = ACTIONS(2781),
    [sym_get_last] = ACTIONS(2781),
    [sym_date_to_day] = ACTIONS(2781),
    [sym_date_to_month] = ACTIONS(2781),
    [sym_date_to_year] = ACTIONS(2781),
    [sym_time_to_hour] = ACTIONS(2781),
    [sym_time_to_minute] = ACTIONS(2781),
    [sym_time_to_second] = ACTIONS(2781),
    [sym_day_of_week] = ACTIONS(2781),
    [sym_set_pulse_time] = ACTIONS(2781),
    [sym_set_timer] = ACTIONS(2781),
    [sym_file_open] = ACTIONS(2781),
    [sym_file_close] = ACTIONS(2781),
    [sym_file_read] = ACTIONS(2781),
    [sym_file_read_line] = ACTIONS(2781),
    [sym_file_write] = ACTIONS(2781),
    [sym_file_write_line] = ACTIONS(2781),
    [sym_file_copy] = ACTIONS(2781),
    [sym_file_delete] = ACTIONS(2781),
    [sym_file_rename] = ACTIONS(2781),
    [sym_file_dir] = ACTIONS(2781),
    [sym_file_getdir] = ACTIONS(2781),
    [sym_file_setdir] = ACTIONS(2781),
    [sym_file_create_dir] = ACTIONS(2781),
    [sym_file_remove_dir] = ACTIONS(2781),
    [sym_file_seek] = ACTIONS(2781),
    [sym_length_array] = ACTIONS(2781),
    [sym_max_length_array] = ACTIONS(2781),
    [sym_set_length_array] = ACTIONS(2781),
    [sym_ip_client_open] = ACTIONS(2781),
    [sym_ip_client_close] = ACTIONS(2781),
    [sym_ip_server_open] = ACTIONS(2781),
    [sym_ip_server_close] = ACTIONS(2781),
    [sym_ip_bound_client_open] = ACTIONS(2781),
    [sym_ip_mc_server_open] = ACTIONS(2781),
    [sym_ip_set_option] = ACTIONS(2781),
    [sym_ssh_client_open] = ACTIONS(2781),
    [sym_ssh_client_close] = ACTIONS(2781),
    [sym_tls_client_open] = ACTIONS(2781),
    [sym_tls_client_close] = ACTIONS(2781),
    [sym_do_push] = ACTIONS(2781),
    [sym_do_push_timed] = ACTIONS(2781),
    [sym_do_release] = ACTIONS(2781),
    [sym_device_id] = ACTIONS(2781),
    [sym_device_info] = ACTIONS(2781),
    [sym_device_id_string] = ACTIONS(2781),
    [sym_device_standby] = ACTIONS(2781),
    [sym_device_wake] = ACTIONS(2781),
    [sym_timeline_create] = ACTIONS(2781),
    [sym_timeline_kill] = ACTIONS(2781),
    [sym_timeline_reload] = ACTIONS(2781),
    [sym_timeline_pause] = ACTIONS(2781),
    [sym_timeline_restart] = ACTIONS(2781),
    [sym_timeline_set] = ACTIONS(2781),
    [sym_timeline_get] = ACTIONS(2781),
    [sym_timeline_active] = ACTIONS(2781),
    [sym_timeline_dynamic_id] = ACTIONS(2781),
    [sym_variable_to_string] = ACTIONS(2781),
    [sym_string_to_variable] = ACTIONS(2781),
    [sym_length_variable_to_string] = ACTIONS(2781),
    [sym_type_cast] = ACTIONS(2781),
    [sym_variable_to_xml] = ACTIONS(2781),
    [sym_xml_to_variable] = ACTIONS(2781),
    [sym_length_variable_to_xml] = ACTIONS(2781),
    [sym_redirect_string] = ACTIONS(2781),
    [sym_random_number] = ACTIONS(2781),
    [sym_get_unique_id] = ACTIONS(2781),
    [sym_get_system_number] = ACTIONS(2781),
    [sym_set_system_number] = ACTIONS(2781),
    [sym_get_serial_number] = ACTIONS(2781),
    [sym_get_master_build] = ACTIONS(2781),
    [sym_set_validation_code] = ACTIONS(2781),
    [sym_reboot] = ACTIONS(2781),
    [sym_load_duet_module] = ACTIONS(2781),
    [sym_unload_duet_module] = ACTIONS(2781),
    [sym_duet_mem_size_set] = ACTIONS(2781),
    [sym_duet_mem_size_get] = ACTIONS(2781),
    [sym_dynamic_application_device] = ACTIONS(2781),
    [sym_dynamic_polled_port] = ACTIONS(2781),
    [sym_static_port_binding] = ACTIONS(2781),
    [sym_static_ip_binding] = ACTIONS(2781),
    [sym_get_dns_list] = ACTIONS(2781),
    [sym_set_dns_list] = ACTIONS(2781),
    [sym_get_ip_address] = ACTIONS(2781),
    [sym_set_ip_address] = ACTIONS(2781),
    [sym_get_url_list] = ACTIONS(2781),
    [sym_add_url_entry] = ACTIONS(2781),
    [sym_delete_url_entry] = ACTIONS(2781),
    [sym_abs_value] = ACTIONS(2781),
    [sym_max_value] = ACTIONS(2781),
    [sym_min_value] = ACTIONS(2781),
    [sym_exp_value] = ACTIONS(2781),
    [sym_log_value] = ACTIONS(2781),
    [sym_log10_value] = ACTIONS(2781),
    [sym_power_value] = ACTIONS(2781),
    [sym_sqrt_value] = ACTIONS(2781),
    [sym_raw_be] = ACTIONS(2781),
    [sym_raw_le] = ACTIONS(2781),
    [sym_internal_queue_size_set] = ACTIONS(2781),
    [sym_internal_queue_size_get] = ACTIONS(2781),
    [sym_internal_threshold_set] = ACTIONS(2781),
    [sym_internal_threshold_get] = ACTIONS(2781),
    [sym_rebuilt_event] = ACTIONS(2781),
    [sym_do_custom_event] = ACTIONS(2781),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2781),
    [sym_clkmgr_set_clk_source] = ACTIONS(2781),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2781),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2781),
    [sym_clkmgr_get_timezone] = ACTIONS(2781),
    [sym_clkmgr_set_timezone] = ACTIONS(2781),
    [sym_clkmgr_get_resync_period] = ACTIONS(2781),
    [sym_clkmgr_set_resync_period] = ACTIONS(2781),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2781),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2781),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2781),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2781),
    [sym_clkmgr_get_timeservers] = ACTIONS(2781),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2781),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2781),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2781),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2781),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2781),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2781),
    [sym_set_log_level] = ACTIONS(2781),
    [sym_get_log_level] = ACTIONS(2781),
    [sym_amx_log] = ACTIONS(2781),
    [sym_smtp_server_config_set] = ACTIONS(2781),
    [sym_smtp_server_config_get] = ACTIONS(2781),
    [sym_smtp_send] = ACTIONS(2781),
    [sym_validate_netlinx_account] = ACTIONS(2781),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2781),
    [sym_authenticate_certificate] = ACTIONS(2781),
    [sym_audit_netlinx_session_event] = ACTIONS(2781),
    [sym_audit_netlinx_generic_event] = ACTIONS(2781),
    [sym_get_available_flash_disk_space] = ACTIONS(2781),
    [sym_get_max_flash_disk_space] = ACTIONS(2781),
    [sym_data_create_feed] = ACTIONS(2781),
    [sym_data_delete_feed] = ACTIONS(2781),
    [sym_data_publish_feed] = ACTIONS(2781),
    [sym_data_get_published_feed] = ACTIONS(2781),
    [sym_data_add_record] = ACTIONS(2781),
    [sym_data_get_event_record] = ACTIONS(2781),
    [sym__wc_data_create_feed] = ACTIONS(2781),
    [sym__wc_data_add_record] = ACTIONS(2781),
    [sym__wc_data_get_event_record] = ACTIONS(2781),
    [sym_format] = ACTIONS(2781),
    [sym_astro_clock] = ACTIONS(2781),
    [sym_set_outdoor_temperature] = ACTIONS(2781),
    [sym_set_virtual_level_count] = ACTIONS(2781),
    [sym_set_virtual_channel_count] = ACTIONS(2781),
    [sym_set_virtual_port_count] = ACTIONS(2781),
    [sym__wc_encode_int] = ACTIONS(2781),
    [sym__wc_decode_int] = ACTIONS(2781),
    [sym__wc_int] = ACTIONS(2781),
    [sym__wc_to_ch_int] = ACTIONS(2781),
    [sym__ch_to_wc_int] = ACTIONS(2781),
    [sym__wc_find_string_int] = ACTIONS(2781),
    [sym__wc_left_string_int] = ACTIONS(2781),
    [sym__wc_lower_string_int] = ACTIONS(2781),
    [sym__wc_mid_string_int] = ACTIONS(2781),
    [sym__wc_remove_string_int] = ACTIONS(2781),
    [sym__wc_right_string_int] = ACTIONS(2781),
    [sym__wc_upper_string_int] = ACTIONS(2781),
    [sym__wc_compare_string_int] = ACTIONS(2781),
    [sym__wc_get_buffer_char_int] = ACTIONS(2781),
    [sym__wc_get_buffer_string_int] = ACTIONS(2781),
    [sym__wc_concat_string_int] = ACTIONS(2781),
    [sym__wc_file_open_int] = ACTIONS(2781),
    [sym__wc_file_close_int] = ACTIONS(2781),
    [sym__wc_file_read_int] = ACTIONS(2781),
    [sym__wc_file_read_line_int] = ACTIONS(2781),
    [sym__wc_file_write_int] = ACTIONS(2781),
    [sym__wc_file_write_line_int] = ACTIONS(2781),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2783),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2783),
    [anon_sym_LBRACK] = ACTIONS(2783),
    [anon_sym_LBRACE] = ACTIONS(2783),
    [anon_sym_RBRACE] = ACTIONS(2783),
    [anon_sym_LPAREN] = ACTIONS(2781),
    [anon_sym_BANG] = ACTIONS(2783),
    [anon_sym_TILDE] = ACTIONS(2783),
    [anon_sym_DASH] = ACTIONS(2781),
    [anon_sym_PLUS] = ACTIONS(2781),
    [anon_sym_DASH_DASH] = ACTIONS(2783),
    [anon_sym_PLUS_PLUS] = ACTIONS(2783),
    [anon_sym_DQUOTE] = ACTIONS(2783),
    [anon_sym_SQUOTE] = ACTIONS(2783),
    [sym_number_literal] = ACTIONS(2783),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(930)] = {
    [sym_identifier] = ACTIONS(2785),
    [sym_not] = ACTIONS(2785),
    [sym_bnot] = ACTIONS(2785),
    [sym___file__] = ACTIONS(2785),
    [sym___date__] = ACTIONS(2785),
    [sym___ldate__] = ACTIONS(2785),
    [sym___time__] = ACTIONS(2785),
    [sym___line__] = ACTIONS(2785),
    [sym___name__] = ACTIONS(2785),
    [sym___netlinx__] = ACTIONS(2785),
    [sym_netlinx_axi_version] = ACTIONS(2785),
    [sym_url_flg_tcp] = ACTIONS(2785),
    [sym_url_flg_accinfopresent] = ACTIONS(2785),
    [sym_url_flg_temp] = ACTIONS(2785),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2785),
    [sym_url_flg_stat_mask] = ACTIONS(2785),
    [sym_url_flg_stat_lookup] = ACTIONS(2785),
    [sym_url_flg_stat_connecting] = ACTIONS(2785),
    [sym_url_flg_stat_waiting] = ACTIONS(2785),
    [sym_url_flg_stat_connected] = ACTIONS(2785),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2785),
    [sym_first_virtual_device] = ACTIONS(2785),
    [sym_first_local_port] = ACTIONS(2785),
    [sym_dynamic_virtual_device] = ACTIONS(2785),
    [sym_timeline_once] = ACTIONS(2785),
    [sym_timeline_repeat] = ACTIONS(2785),
    [sym_timeline_absolute] = ACTIONS(2785),
    [sym_timeline_relative] = ACTIONS(2785),
    [sym_do_push_timed_infinite] = ACTIONS(2785),
    [sym_source_type_no_address] = ACTIONS(2785),
    [sym_source_type_neuron_id] = ACTIONS(2785),
    [sym_source_type_ip_address] = ACTIONS(2785),
    [sym_source_type_axlink] = ACTIONS(2785),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2785),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2785),
    [sym_source_type_ip_socket_address] = ACTIONS(2785),
    [sym_source_type_rs232] = ACTIONS(2785),
    [sym_source_type_internal] = ACTIONS(2785),
    [sym_source_type_mac_address] = ACTIONS(2785),
    [sym_source_type_ipv4_port] = ACTIONS(2785),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2785),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2785),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2785),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2785),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2785),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2785),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2785),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2785),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2785),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2785),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2785),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2785),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2785),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2785),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2785),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2785),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2785),
    [sym_true] = ACTIONS(2785),
    [sym_false] = ACTIONS(2785),
    [sym_file_read_only] = ACTIONS(2785),
    [sym_file_rw_new] = ACTIONS(2785),
    [sym_file_rw_append] = ACTIONS(2785),
    [sym_ip_tcp] = ACTIONS(2785),
    [sym_ip_udp] = ACTIONS(2785),
    [sym_ip_udp_2way] = ACTIONS(2785),
    [sym_xml_encode_types] = ACTIONS(2785),
    [sym_xml_encode_char_as_list] = ACTIONS(2785),
    [sym_xml_encode_le] = ACTIONS(2785),
    [sym_xml_decode_types] = ACTIONS(2785),
    [sym_xml_decode_no_preserve] = ACTIONS(2785),
    [sym_duet_dev_type_display_device] = ACTIONS(2785),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2785),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2785),
    [sym_duet_dev_type_io_device] = ACTIONS(2785),
    [sym_duet_dev_type_relay_device] = ACTIONS(2785),
    [sym_duet_dev_type_ups] = ACTIONS(2785),
    [sym_duet_dev_type_amplifier] = ACTIONS(2785),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2785),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2785),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2785),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2785),
    [sym_duet_dev_type_camera] = ACTIONS(2785),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2785),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2785),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2785),
    [sym_duet_dev_type_dss] = ACTIONS(2785),
    [sym_duet_dev_type_dvr] = ACTIONS(2785),
    [sym_duet_dev_type_disc_device] = ACTIONS(2785),
    [sym_duet_dev_type_document_camera] = ACTIONS(2785),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2785),
    [sym_duet_dev_type_hvac] = ACTIONS(2785),
    [sym_duet_dev_type_keypad] = ACTIONS(2785),
    [sym_duet_dev_type_light] = ACTIONS(2785),
    [sym_duet_dev_type_monitor] = ACTIONS(2785),
    [sym_duet_dev_type_motor] = ACTIONS(2785),
    [sym_duet_dev_type_multi_window] = ACTIONS(2785),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2785),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2785),
    [sym_duet_dev_type_receiver] = ACTIONS(2785),
    [sym_duet_dev_type_security_system] = ACTIONS(2785),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2785),
    [sym_duet_dev_type_settop_box] = ACTIONS(2785),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2785),
    [sym_duet_dev_type_switcher] = ACTIONS(2785),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2785),
    [sym_duet_dev_type_tv] = ACTIONS(2785),
    [sym_duet_dev_type_utility] = ACTIONS(2785),
    [sym_duet_dev_type_vcr] = ACTIONS(2785),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2785),
    [sym_duet_dev_type_video_processor] = ACTIONS(2785),
    [sym_duet_dev_type_video_projector] = ACTIONS(2785),
    [sym_duet_dev_type_video_wall] = ACTIONS(2785),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2785),
    [sym_duet_dev_type_weather] = ACTIONS(2785),
    [sym_duet_dev_not_polled] = ACTIONS(2785),
    [sym_duet_dev_polled] = ACTIONS(2785),
    [sym_clkmgr_mode_network] = ACTIONS(2785),
    [sym_clkmgr_mode_standalone] = ACTIONS(2785),
    [sym_normal_standby] = ACTIONS(2785),
    [sym_normal_wake] = ACTIONS(2785),
    [sym_ip_multicast_ttl_option] = ACTIONS(2785),
    [sym_ip_ttl_subnet] = ACTIONS(2785),
    [sym_ip_ttl_site] = ACTIONS(2785),
    [sym_ip_ttl_region] = ACTIONS(2785),
    [sym_ip_ttl_continent] = ACTIONS(2785),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2785),
    [sym_ip_nodelay_on] = ACTIONS(2785),
    [sym_ip_nodelay_off] = ACTIONS(2785),
    [sym_amx_error] = ACTIONS(2785),
    [sym_amx_warning] = ACTIONS(2785),
    [sym_amx_info] = ACTIONS(2785),
    [sym_amx_debug] = ACTIONS(2785),
    [sym_netlinx_login_success] = ACTIONS(2785),
    [sym_netlinx_login_fail] = ACTIONS(2785),
    [sym_netlinx_logout] = ACTIONS(2785),
    [sym_null_str] = ACTIONS(2785),
    [sym_smtp_address] = ACTIONS(2785),
    [sym_smtp_port_number] = ACTIONS(2785),
    [sym_smtp_username] = ACTIONS(2785),
    [sym_smtp_password] = ACTIONS(2785),
    [sym_smtp_require_tls] = ACTIONS(2785),
    [sym_smtp_from] = ACTIONS(2785),
    [sym_smtp_tls_true] = ACTIONS(2785),
    [sym_smtp_tls_false] = ACTIONS(2785),
    [sym_listview_on_row_select_event] = ACTIONS(2785),
    [sym_data_structure_unknown] = ACTIONS(2785),
    [sym_data_structure_datafeed] = ACTIONS(2785),
    [sym_data_structure_datarecordset] = ACTIONS(2785),
    [sym_data_structure_datarecord] = ACTIONS(2785),
    [sym_data_structure_datafield] = ACTIONS(2785),
    [sym_data_max_content_field_count] = ACTIONS(2785),
    [sym_data_max_metadata_field_count] = ACTIONS(2785),
    [sym_data_max_id_length] = ACTIONS(2785),
    [sym_data_max_name_length] = ACTIONS(2785),
    [sym_data_max_description_length] = ACTIONS(2785),
    [sym_data_max_type_length] = ACTIONS(2785),
    [sym_data_max_format_length] = ACTIONS(2785),
    [sym_data_max_source_length] = ACTIONS(2785),
    [sym_data_max_value_length] = ACTIONS(2785),
    [sym_data_max_label_length] = ACTIONS(2785),
    [sym_data_max_url_length] = ACTIONS(2785),
    [sym_max_last_login_info_length] = ACTIONS(2785),
    [sym_data_type_unknown] = ACTIONS(2785),
    [sym_data_type_string] = ACTIONS(2785),
    [sym_data_type_datetime] = ACTIONS(2785),
    [sym_data_type_date] = ACTIONS(2785),
    [sym_data_type_time] = ACTIONS(2785),
    [sym_data_type_image] = ACTIONS(2785),
    [sym_data_format_url] = ACTIONS(2785),
    [sym_data_format_phone] = ACTIONS(2785),
    [sym_data_format_email] = ACTIONS(2785),
    [sym_data_format_iso8601] = ACTIONS(2785),
    [sym_auditlog_priv] = ACTIONS(2785),
    [sym_configuration_priv] = ACTIONS(2785),
    [sym_deviceconfig_priv] = ACTIONS(2785),
    [sym_ftp_priv] = ACTIONS(2785),
    [sym_http_priv] = ACTIONS(2785),
    [sym_networkconfig_priv] = ACTIONS(2785),
    [sym_programport_priv] = ACTIONS(2785),
    [sym_remoteui_priv] = ACTIONS(2785),
    [sym_securitycontrol_priv] = ACTIONS(2785),
    [sym_softwaremanagement_priv] = ACTIONS(2785),
    [sym_terminal_priv] = ACTIONS(2785),
    [sym_tpadmin_priv] = ACTIONS(2785),
    [sym_user1_priv] = ACTIONS(2785),
    [sym_user2_priv] = ACTIONS(2785),
    [sym_user3_priv] = ACTIONS(2785),
    [sym_user4_priv] = ACTIONS(2785),
    [sym_usermanagement_priv] = ACTIONS(2785),
    [sym_valid_account] = ACTIONS(2785),
    [sym_err_name_invalid] = ACTIONS(2785),
    [sym_err_password_invalid] = ACTIONS(2785),
    [sym_err_invalid_account] = ACTIONS(2785),
    [sym_err_no_authorization] = ACTIONS(2785),
    [sym_err_invalid_parameter] = ACTIONS(2785),
    [sym_err_user_locked_out] = ACTIONS(2785),
    [sym_err_user_account_expired] = ACTIONS(2785),
    [sym_err_general_error] = ACTIONS(2785),
    [sym_err_authentication_error] = ACTIONS(2785),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2785),
    [sym_tls_validate_certificate] = ACTIONS(2785),
    [sym____reserved___] = ACTIONS(2785),
    [sym_date] = ACTIONS(2785),
    [sym_day] = ACTIONS(2785),
    [sym_time] = ACTIONS(2785),
    [sym_ldate] = ACTIONS(2785),
    [sym_push_device] = ACTIONS(2785),
    [sym_push_channel] = ACTIONS(2785),
    [sym_push_devchan] = ACTIONS(2785),
    [sym_release_device] = ACTIONS(2785),
    [sym_release_channel] = ACTIONS(2785),
    [sym_release_devchan] = ACTIONS(2785),
    [sym_master_slot] = ACTIONS(2785),
    [sym_get_pulse_time] = ACTIONS(2785),
    [sym_get_timer] = ACTIONS(2785),
    [sym_master_sn] = ACTIONS(2785),
    [sym_system_number] = ACTIONS(2785),
    [sym_button] = ACTIONS(2785),
    [sym_dv_channel] = ACTIONS(2785),
    [sym_data] = ACTIONS(2785),
    [sym_level] = ACTIONS(2785),
    [sym_timeline] = ACTIONS(2785),
    [sym_channel] = ACTIONS(2785),
    [sym_custom] = ACTIONS(2785),
    [sym_atoi] = ACTIONS(2785),
    [sym_atol] = ACTIONS(2785),
    [sym_atof] = ACTIONS(2785),
    [sym_itoa] = ACTIONS(2785),
    [sym_itohex] = ACTIONS(2785),
    [sym_hextoi] = ACTIONS(2785),
    [sym_ftoa] = ACTIONS(2785),
    [sym_find_string] = ACTIONS(2785),
    [sym_left_string] = ACTIONS(2785),
    [sym_right_string] = ACTIONS(2785),
    [sym_mid_string] = ACTIONS(2785),
    [sym_remove_string] = ACTIONS(2785),
    [sym_get_buffer_string] = ACTIONS(2785),
    [sym_get_buffer_char] = ACTIONS(2785),
    [sym_get_multi_buffer_string] = ACTIONS(2785),
    [sym_length_string] = ACTIONS(2785),
    [sym_max_length_string] = ACTIONS(2785),
    [sym_set_length_string] = ACTIONS(2785),
    [sym_compare_string] = ACTIONS(2785),
    [sym_upper_string] = ACTIONS(2785),
    [sym_lower_string] = ACTIONS(2785),
    [sym_get_last] = ACTIONS(2785),
    [sym_date_to_day] = ACTIONS(2785),
    [sym_date_to_month] = ACTIONS(2785),
    [sym_date_to_year] = ACTIONS(2785),
    [sym_time_to_hour] = ACTIONS(2785),
    [sym_time_to_minute] = ACTIONS(2785),
    [sym_time_to_second] = ACTIONS(2785),
    [sym_day_of_week] = ACTIONS(2785),
    [sym_set_pulse_time] = ACTIONS(2785),
    [sym_set_timer] = ACTIONS(2785),
    [sym_file_open] = ACTIONS(2785),
    [sym_file_close] = ACTIONS(2785),
    [sym_file_read] = ACTIONS(2785),
    [sym_file_read_line] = ACTIONS(2785),
    [sym_file_write] = ACTIONS(2785),
    [sym_file_write_line] = ACTIONS(2785),
    [sym_file_copy] = ACTIONS(2785),
    [sym_file_delete] = ACTIONS(2785),
    [sym_file_rename] = ACTIONS(2785),
    [sym_file_dir] = ACTIONS(2785),
    [sym_file_getdir] = ACTIONS(2785),
    [sym_file_setdir] = ACTIONS(2785),
    [sym_file_create_dir] = ACTIONS(2785),
    [sym_file_remove_dir] = ACTIONS(2785),
    [sym_file_seek] = ACTIONS(2785),
    [sym_length_array] = ACTIONS(2785),
    [sym_max_length_array] = ACTIONS(2785),
    [sym_set_length_array] = ACTIONS(2785),
    [sym_ip_client_open] = ACTIONS(2785),
    [sym_ip_client_close] = ACTIONS(2785),
    [sym_ip_server_open] = ACTIONS(2785),
    [sym_ip_server_close] = ACTIONS(2785),
    [sym_ip_bound_client_open] = ACTIONS(2785),
    [sym_ip_mc_server_open] = ACTIONS(2785),
    [sym_ip_set_option] = ACTIONS(2785),
    [sym_ssh_client_open] = ACTIONS(2785),
    [sym_ssh_client_close] = ACTIONS(2785),
    [sym_tls_client_open] = ACTIONS(2785),
    [sym_tls_client_close] = ACTIONS(2785),
    [sym_do_push] = ACTIONS(2785),
    [sym_do_push_timed] = ACTIONS(2785),
    [sym_do_release] = ACTIONS(2785),
    [sym_device_id] = ACTIONS(2785),
    [sym_device_info] = ACTIONS(2785),
    [sym_device_id_string] = ACTIONS(2785),
    [sym_device_standby] = ACTIONS(2785),
    [sym_device_wake] = ACTIONS(2785),
    [sym_timeline_create] = ACTIONS(2785),
    [sym_timeline_kill] = ACTIONS(2785),
    [sym_timeline_reload] = ACTIONS(2785),
    [sym_timeline_pause] = ACTIONS(2785),
    [sym_timeline_restart] = ACTIONS(2785),
    [sym_timeline_set] = ACTIONS(2785),
    [sym_timeline_get] = ACTIONS(2785),
    [sym_timeline_active] = ACTIONS(2785),
    [sym_timeline_dynamic_id] = ACTIONS(2785),
    [sym_variable_to_string] = ACTIONS(2785),
    [sym_string_to_variable] = ACTIONS(2785),
    [sym_length_variable_to_string] = ACTIONS(2785),
    [sym_type_cast] = ACTIONS(2785),
    [sym_variable_to_xml] = ACTIONS(2785),
    [sym_xml_to_variable] = ACTIONS(2785),
    [sym_length_variable_to_xml] = ACTIONS(2785),
    [sym_redirect_string] = ACTIONS(2785),
    [sym_random_number] = ACTIONS(2785),
    [sym_get_unique_id] = ACTIONS(2785),
    [sym_get_system_number] = ACTIONS(2785),
    [sym_set_system_number] = ACTIONS(2785),
    [sym_get_serial_number] = ACTIONS(2785),
    [sym_get_master_build] = ACTIONS(2785),
    [sym_set_validation_code] = ACTIONS(2785),
    [sym_reboot] = ACTIONS(2785),
    [sym_load_duet_module] = ACTIONS(2785),
    [sym_unload_duet_module] = ACTIONS(2785),
    [sym_duet_mem_size_set] = ACTIONS(2785),
    [sym_duet_mem_size_get] = ACTIONS(2785),
    [sym_dynamic_application_device] = ACTIONS(2785),
    [sym_dynamic_polled_port] = ACTIONS(2785),
    [sym_static_port_binding] = ACTIONS(2785),
    [sym_static_ip_binding] = ACTIONS(2785),
    [sym_get_dns_list] = ACTIONS(2785),
    [sym_set_dns_list] = ACTIONS(2785),
    [sym_get_ip_address] = ACTIONS(2785),
    [sym_set_ip_address] = ACTIONS(2785),
    [sym_get_url_list] = ACTIONS(2785),
    [sym_add_url_entry] = ACTIONS(2785),
    [sym_delete_url_entry] = ACTIONS(2785),
    [sym_abs_value] = ACTIONS(2785),
    [sym_max_value] = ACTIONS(2785),
    [sym_min_value] = ACTIONS(2785),
    [sym_exp_value] = ACTIONS(2785),
    [sym_log_value] = ACTIONS(2785),
    [sym_log10_value] = ACTIONS(2785),
    [sym_power_value] = ACTIONS(2785),
    [sym_sqrt_value] = ACTIONS(2785),
    [sym_raw_be] = ACTIONS(2785),
    [sym_raw_le] = ACTIONS(2785),
    [sym_internal_queue_size_set] = ACTIONS(2785),
    [sym_internal_queue_size_get] = ACTIONS(2785),
    [sym_internal_threshold_set] = ACTIONS(2785),
    [sym_internal_threshold_get] = ACTIONS(2785),
    [sym_rebuilt_event] = ACTIONS(2785),
    [sym_do_custom_event] = ACTIONS(2785),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2785),
    [sym_clkmgr_set_clk_source] = ACTIONS(2785),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2785),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2785),
    [sym_clkmgr_get_timezone] = ACTIONS(2785),
    [sym_clkmgr_set_timezone] = ACTIONS(2785),
    [sym_clkmgr_get_resync_period] = ACTIONS(2785),
    [sym_clkmgr_set_resync_period] = ACTIONS(2785),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2785),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2785),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2785),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2785),
    [sym_clkmgr_get_timeservers] = ACTIONS(2785),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2785),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2785),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2785),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2785),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2785),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2785),
    [sym_set_log_level] = ACTIONS(2785),
    [sym_get_log_level] = ACTIONS(2785),
    [sym_amx_log] = ACTIONS(2785),
    [sym_smtp_server_config_set] = ACTIONS(2785),
    [sym_smtp_server_config_get] = ACTIONS(2785),
    [sym_smtp_send] = ACTIONS(2785),
    [sym_validate_netlinx_account] = ACTIONS(2785),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2785),
    [sym_authenticate_certificate] = ACTIONS(2785),
    [sym_audit_netlinx_session_event] = ACTIONS(2785),
    [sym_audit_netlinx_generic_event] = ACTIONS(2785),
    [sym_get_available_flash_disk_space] = ACTIONS(2785),
    [sym_get_max_flash_disk_space] = ACTIONS(2785),
    [sym_data_create_feed] = ACTIONS(2785),
    [sym_data_delete_feed] = ACTIONS(2785),
    [sym_data_publish_feed] = ACTIONS(2785),
    [sym_data_get_published_feed] = ACTIONS(2785),
    [sym_data_add_record] = ACTIONS(2785),
    [sym_data_get_event_record] = ACTIONS(2785),
    [sym__wc_data_create_feed] = ACTIONS(2785),
    [sym__wc_data_add_record] = ACTIONS(2785),
    [sym__wc_data_get_event_record] = ACTIONS(2785),
    [sym_format] = ACTIONS(2785),
    [sym_astro_clock] = ACTIONS(2785),
    [sym_set_outdoor_temperature] = ACTIONS(2785),
    [sym_set_virtual_level_count] = ACTIONS(2785),
    [sym_set_virtual_channel_count] = ACTIONS(2785),
    [sym_set_virtual_port_count] = ACTIONS(2785),
    [sym__wc_encode_int] = ACTIONS(2785),
    [sym__wc_decode_int] = ACTIONS(2785),
    [sym__wc_int] = ACTIONS(2785),
    [sym__wc_to_ch_int] = ACTIONS(2785),
    [sym__ch_to_wc_int] = ACTIONS(2785),
    [sym__wc_find_string_int] = ACTIONS(2785),
    [sym__wc_left_string_int] = ACTIONS(2785),
    [sym__wc_lower_string_int] = ACTIONS(2785),
    [sym__wc_mid_string_int] = ACTIONS(2785),
    [sym__wc_remove_string_int] = ACTIONS(2785),
    [sym__wc_right_string_int] = ACTIONS(2785),
    [sym__wc_upper_string_int] = ACTIONS(2785),
    [sym__wc_compare_string_int] = ACTIONS(2785),
    [sym__wc_get_buffer_char_int] = ACTIONS(2785),
    [sym__wc_get_buffer_string_int] = ACTIONS(2785),
    [sym__wc_concat_string_int] = ACTIONS(2785),
    [sym__wc_file_open_int] = ACTIONS(2785),
    [sym__wc_file_close_int] = ACTIONS(2785),
    [sym__wc_file_read_int] = ACTIONS(2785),
    [sym__wc_file_read_line_int] = ACTIONS(2785),
    [sym__wc_file_write_int] = ACTIONS(2785),
    [sym__wc_file_write_line_int] = ACTIONS(2785),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2787),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2787),
    [anon_sym_LBRACK] = ACTIONS(2787),
    [anon_sym_LBRACE] = ACTIONS(2787),
    [anon_sym_RBRACE] = ACTIONS(2787),
    [anon_sym_LPAREN] = ACTIONS(2785),
    [anon_sym_BANG] = ACTIONS(2787),
    [anon_sym_TILDE] = ACTIONS(2787),
    [anon_sym_DASH] = ACTIONS(2785),
    [anon_sym_PLUS] = ACTIONS(2785),
    [anon_sym_DASH_DASH] = ACTIONS(2787),
    [anon_sym_PLUS_PLUS] = ACTIONS(2787),
    [anon_sym_DQUOTE] = ACTIONS(2787),
    [anon_sym_SQUOTE] = ACTIONS(2787),
    [sym_number_literal] = ACTIONS(2787),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(931)] = {
    [sym_identifier] = ACTIONS(2789),
    [sym_not] = ACTIONS(2789),
    [sym_bnot] = ACTIONS(2789),
    [sym___file__] = ACTIONS(2789),
    [sym___date__] = ACTIONS(2789),
    [sym___ldate__] = ACTIONS(2789),
    [sym___time__] = ACTIONS(2789),
    [sym___line__] = ACTIONS(2789),
    [sym___name__] = ACTIONS(2789),
    [sym___netlinx__] = ACTIONS(2789),
    [sym_netlinx_axi_version] = ACTIONS(2789),
    [sym_url_flg_tcp] = ACTIONS(2789),
    [sym_url_flg_accinfopresent] = ACTIONS(2789),
    [sym_url_flg_temp] = ACTIONS(2789),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2789),
    [sym_url_flg_stat_mask] = ACTIONS(2789),
    [sym_url_flg_stat_lookup] = ACTIONS(2789),
    [sym_url_flg_stat_connecting] = ACTIONS(2789),
    [sym_url_flg_stat_waiting] = ACTIONS(2789),
    [sym_url_flg_stat_connected] = ACTIONS(2789),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2789),
    [sym_first_virtual_device] = ACTIONS(2789),
    [sym_first_local_port] = ACTIONS(2789),
    [sym_dynamic_virtual_device] = ACTIONS(2789),
    [sym_timeline_once] = ACTIONS(2789),
    [sym_timeline_repeat] = ACTIONS(2789),
    [sym_timeline_absolute] = ACTIONS(2789),
    [sym_timeline_relative] = ACTIONS(2789),
    [sym_do_push_timed_infinite] = ACTIONS(2789),
    [sym_source_type_no_address] = ACTIONS(2789),
    [sym_source_type_neuron_id] = ACTIONS(2789),
    [sym_source_type_ip_address] = ACTIONS(2789),
    [sym_source_type_axlink] = ACTIONS(2789),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2789),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2789),
    [sym_source_type_ip_socket_address] = ACTIONS(2789),
    [sym_source_type_rs232] = ACTIONS(2789),
    [sym_source_type_internal] = ACTIONS(2789),
    [sym_source_type_mac_address] = ACTIONS(2789),
    [sym_source_type_ipv4_port] = ACTIONS(2789),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2789),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2789),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2789),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2789),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2789),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2789),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2789),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2789),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2789),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2789),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2789),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2789),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2789),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2789),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2789),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2789),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2789),
    [sym_true] = ACTIONS(2789),
    [sym_false] = ACTIONS(2789),
    [sym_file_read_only] = ACTIONS(2789),
    [sym_file_rw_new] = ACTIONS(2789),
    [sym_file_rw_append] = ACTIONS(2789),
    [sym_ip_tcp] = ACTIONS(2789),
    [sym_ip_udp] = ACTIONS(2789),
    [sym_ip_udp_2way] = ACTIONS(2789),
    [sym_xml_encode_types] = ACTIONS(2789),
    [sym_xml_encode_char_as_list] = ACTIONS(2789),
    [sym_xml_encode_le] = ACTIONS(2789),
    [sym_xml_decode_types] = ACTIONS(2789),
    [sym_xml_decode_no_preserve] = ACTIONS(2789),
    [sym_duet_dev_type_display_device] = ACTIONS(2789),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2789),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2789),
    [sym_duet_dev_type_io_device] = ACTIONS(2789),
    [sym_duet_dev_type_relay_device] = ACTIONS(2789),
    [sym_duet_dev_type_ups] = ACTIONS(2789),
    [sym_duet_dev_type_amplifier] = ACTIONS(2789),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2789),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2789),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2789),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2789),
    [sym_duet_dev_type_camera] = ACTIONS(2789),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2789),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2789),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2789),
    [sym_duet_dev_type_dss] = ACTIONS(2789),
    [sym_duet_dev_type_dvr] = ACTIONS(2789),
    [sym_duet_dev_type_disc_device] = ACTIONS(2789),
    [sym_duet_dev_type_document_camera] = ACTIONS(2789),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2789),
    [sym_duet_dev_type_hvac] = ACTIONS(2789),
    [sym_duet_dev_type_keypad] = ACTIONS(2789),
    [sym_duet_dev_type_light] = ACTIONS(2789),
    [sym_duet_dev_type_monitor] = ACTIONS(2789),
    [sym_duet_dev_type_motor] = ACTIONS(2789),
    [sym_duet_dev_type_multi_window] = ACTIONS(2789),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2789),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2789),
    [sym_duet_dev_type_receiver] = ACTIONS(2789),
    [sym_duet_dev_type_security_system] = ACTIONS(2789),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2789),
    [sym_duet_dev_type_settop_box] = ACTIONS(2789),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2789),
    [sym_duet_dev_type_switcher] = ACTIONS(2789),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2789),
    [sym_duet_dev_type_tv] = ACTIONS(2789),
    [sym_duet_dev_type_utility] = ACTIONS(2789),
    [sym_duet_dev_type_vcr] = ACTIONS(2789),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2789),
    [sym_duet_dev_type_video_processor] = ACTIONS(2789),
    [sym_duet_dev_type_video_projector] = ACTIONS(2789),
    [sym_duet_dev_type_video_wall] = ACTIONS(2789),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2789),
    [sym_duet_dev_type_weather] = ACTIONS(2789),
    [sym_duet_dev_not_polled] = ACTIONS(2789),
    [sym_duet_dev_polled] = ACTIONS(2789),
    [sym_clkmgr_mode_network] = ACTIONS(2789),
    [sym_clkmgr_mode_standalone] = ACTIONS(2789),
    [sym_normal_standby] = ACTIONS(2789),
    [sym_normal_wake] = ACTIONS(2789),
    [sym_ip_multicast_ttl_option] = ACTIONS(2789),
    [sym_ip_ttl_subnet] = ACTIONS(2789),
    [sym_ip_ttl_site] = ACTIONS(2789),
    [sym_ip_ttl_region] = ACTIONS(2789),
    [sym_ip_ttl_continent] = ACTIONS(2789),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2789),
    [sym_ip_nodelay_on] = ACTIONS(2789),
    [sym_ip_nodelay_off] = ACTIONS(2789),
    [sym_amx_error] = ACTIONS(2789),
    [sym_amx_warning] = ACTIONS(2789),
    [sym_amx_info] = ACTIONS(2789),
    [sym_amx_debug] = ACTIONS(2789),
    [sym_netlinx_login_success] = ACTIONS(2789),
    [sym_netlinx_login_fail] = ACTIONS(2789),
    [sym_netlinx_logout] = ACTIONS(2789),
    [sym_null_str] = ACTIONS(2789),
    [sym_smtp_address] = ACTIONS(2789),
    [sym_smtp_port_number] = ACTIONS(2789),
    [sym_smtp_username] = ACTIONS(2789),
    [sym_smtp_password] = ACTIONS(2789),
    [sym_smtp_require_tls] = ACTIONS(2789),
    [sym_smtp_from] = ACTIONS(2789),
    [sym_smtp_tls_true] = ACTIONS(2789),
    [sym_smtp_tls_false] = ACTIONS(2789),
    [sym_listview_on_row_select_event] = ACTIONS(2789),
    [sym_data_structure_unknown] = ACTIONS(2789),
    [sym_data_structure_datafeed] = ACTIONS(2789),
    [sym_data_structure_datarecordset] = ACTIONS(2789),
    [sym_data_structure_datarecord] = ACTIONS(2789),
    [sym_data_structure_datafield] = ACTIONS(2789),
    [sym_data_max_content_field_count] = ACTIONS(2789),
    [sym_data_max_metadata_field_count] = ACTIONS(2789),
    [sym_data_max_id_length] = ACTIONS(2789),
    [sym_data_max_name_length] = ACTIONS(2789),
    [sym_data_max_description_length] = ACTIONS(2789),
    [sym_data_max_type_length] = ACTIONS(2789),
    [sym_data_max_format_length] = ACTIONS(2789),
    [sym_data_max_source_length] = ACTIONS(2789),
    [sym_data_max_value_length] = ACTIONS(2789),
    [sym_data_max_label_length] = ACTIONS(2789),
    [sym_data_max_url_length] = ACTIONS(2789),
    [sym_max_last_login_info_length] = ACTIONS(2789),
    [sym_data_type_unknown] = ACTIONS(2789),
    [sym_data_type_string] = ACTIONS(2789),
    [sym_data_type_datetime] = ACTIONS(2789),
    [sym_data_type_date] = ACTIONS(2789),
    [sym_data_type_time] = ACTIONS(2789),
    [sym_data_type_image] = ACTIONS(2789),
    [sym_data_format_url] = ACTIONS(2789),
    [sym_data_format_phone] = ACTIONS(2789),
    [sym_data_format_email] = ACTIONS(2789),
    [sym_data_format_iso8601] = ACTIONS(2789),
    [sym_auditlog_priv] = ACTIONS(2789),
    [sym_configuration_priv] = ACTIONS(2789),
    [sym_deviceconfig_priv] = ACTIONS(2789),
    [sym_ftp_priv] = ACTIONS(2789),
    [sym_http_priv] = ACTIONS(2789),
    [sym_networkconfig_priv] = ACTIONS(2789),
    [sym_programport_priv] = ACTIONS(2789),
    [sym_remoteui_priv] = ACTIONS(2789),
    [sym_securitycontrol_priv] = ACTIONS(2789),
    [sym_softwaremanagement_priv] = ACTIONS(2789),
    [sym_terminal_priv] = ACTIONS(2789),
    [sym_tpadmin_priv] = ACTIONS(2789),
    [sym_user1_priv] = ACTIONS(2789),
    [sym_user2_priv] = ACTIONS(2789),
    [sym_user3_priv] = ACTIONS(2789),
    [sym_user4_priv] = ACTIONS(2789),
    [sym_usermanagement_priv] = ACTIONS(2789),
    [sym_valid_account] = ACTIONS(2789),
    [sym_err_name_invalid] = ACTIONS(2789),
    [sym_err_password_invalid] = ACTIONS(2789),
    [sym_err_invalid_account] = ACTIONS(2789),
    [sym_err_no_authorization] = ACTIONS(2789),
    [sym_err_invalid_parameter] = ACTIONS(2789),
    [sym_err_user_locked_out] = ACTIONS(2789),
    [sym_err_user_account_expired] = ACTIONS(2789),
    [sym_err_general_error] = ACTIONS(2789),
    [sym_err_authentication_error] = ACTIONS(2789),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2789),
    [sym_tls_validate_certificate] = ACTIONS(2789),
    [sym____reserved___] = ACTIONS(2789),
    [sym_date] = ACTIONS(2789),
    [sym_day] = ACTIONS(2789),
    [sym_time] = ACTIONS(2789),
    [sym_ldate] = ACTIONS(2789),
    [sym_push_device] = ACTIONS(2789),
    [sym_push_channel] = ACTIONS(2789),
    [sym_push_devchan] = ACTIONS(2789),
    [sym_release_device] = ACTIONS(2789),
    [sym_release_channel] = ACTIONS(2789),
    [sym_release_devchan] = ACTIONS(2789),
    [sym_master_slot] = ACTIONS(2789),
    [sym_get_pulse_time] = ACTIONS(2789),
    [sym_get_timer] = ACTIONS(2789),
    [sym_master_sn] = ACTIONS(2789),
    [sym_system_number] = ACTIONS(2789),
    [sym_button] = ACTIONS(2789),
    [sym_dv_channel] = ACTIONS(2789),
    [sym_data] = ACTIONS(2789),
    [sym_level] = ACTIONS(2789),
    [sym_timeline] = ACTIONS(2789),
    [sym_channel] = ACTIONS(2789),
    [sym_custom] = ACTIONS(2789),
    [sym_atoi] = ACTIONS(2789),
    [sym_atol] = ACTIONS(2789),
    [sym_atof] = ACTIONS(2789),
    [sym_itoa] = ACTIONS(2789),
    [sym_itohex] = ACTIONS(2789),
    [sym_hextoi] = ACTIONS(2789),
    [sym_ftoa] = ACTIONS(2789),
    [sym_find_string] = ACTIONS(2789),
    [sym_left_string] = ACTIONS(2789),
    [sym_right_string] = ACTIONS(2789),
    [sym_mid_string] = ACTIONS(2789),
    [sym_remove_string] = ACTIONS(2789),
    [sym_get_buffer_string] = ACTIONS(2789),
    [sym_get_buffer_char] = ACTIONS(2789),
    [sym_get_multi_buffer_string] = ACTIONS(2789),
    [sym_length_string] = ACTIONS(2789),
    [sym_max_length_string] = ACTIONS(2789),
    [sym_set_length_string] = ACTIONS(2789),
    [sym_compare_string] = ACTIONS(2789),
    [sym_upper_string] = ACTIONS(2789),
    [sym_lower_string] = ACTIONS(2789),
    [sym_get_last] = ACTIONS(2789),
    [sym_date_to_day] = ACTIONS(2789),
    [sym_date_to_month] = ACTIONS(2789),
    [sym_date_to_year] = ACTIONS(2789),
    [sym_time_to_hour] = ACTIONS(2789),
    [sym_time_to_minute] = ACTIONS(2789),
    [sym_time_to_second] = ACTIONS(2789),
    [sym_day_of_week] = ACTIONS(2789),
    [sym_set_pulse_time] = ACTIONS(2789),
    [sym_set_timer] = ACTIONS(2789),
    [sym_file_open] = ACTIONS(2789),
    [sym_file_close] = ACTIONS(2789),
    [sym_file_read] = ACTIONS(2789),
    [sym_file_read_line] = ACTIONS(2789),
    [sym_file_write] = ACTIONS(2789),
    [sym_file_write_line] = ACTIONS(2789),
    [sym_file_copy] = ACTIONS(2789),
    [sym_file_delete] = ACTIONS(2789),
    [sym_file_rename] = ACTIONS(2789),
    [sym_file_dir] = ACTIONS(2789),
    [sym_file_getdir] = ACTIONS(2789),
    [sym_file_setdir] = ACTIONS(2789),
    [sym_file_create_dir] = ACTIONS(2789),
    [sym_file_remove_dir] = ACTIONS(2789),
    [sym_file_seek] = ACTIONS(2789),
    [sym_length_array] = ACTIONS(2789),
    [sym_max_length_array] = ACTIONS(2789),
    [sym_set_length_array] = ACTIONS(2789),
    [sym_ip_client_open] = ACTIONS(2789),
    [sym_ip_client_close] = ACTIONS(2789),
    [sym_ip_server_open] = ACTIONS(2789),
    [sym_ip_server_close] = ACTIONS(2789),
    [sym_ip_bound_client_open] = ACTIONS(2789),
    [sym_ip_mc_server_open] = ACTIONS(2789),
    [sym_ip_set_option] = ACTIONS(2789),
    [sym_ssh_client_open] = ACTIONS(2789),
    [sym_ssh_client_close] = ACTIONS(2789),
    [sym_tls_client_open] = ACTIONS(2789),
    [sym_tls_client_close] = ACTIONS(2789),
    [sym_do_push] = ACTIONS(2789),
    [sym_do_push_timed] = ACTIONS(2789),
    [sym_do_release] = ACTIONS(2789),
    [sym_device_id] = ACTIONS(2789),
    [sym_device_info] = ACTIONS(2789),
    [sym_device_id_string] = ACTIONS(2789),
    [sym_device_standby] = ACTIONS(2789),
    [sym_device_wake] = ACTIONS(2789),
    [sym_timeline_create] = ACTIONS(2789),
    [sym_timeline_kill] = ACTIONS(2789),
    [sym_timeline_reload] = ACTIONS(2789),
    [sym_timeline_pause] = ACTIONS(2789),
    [sym_timeline_restart] = ACTIONS(2789),
    [sym_timeline_set] = ACTIONS(2789),
    [sym_timeline_get] = ACTIONS(2789),
    [sym_timeline_active] = ACTIONS(2789),
    [sym_timeline_dynamic_id] = ACTIONS(2789),
    [sym_variable_to_string] = ACTIONS(2789),
    [sym_string_to_variable] = ACTIONS(2789),
    [sym_length_variable_to_string] = ACTIONS(2789),
    [sym_type_cast] = ACTIONS(2789),
    [sym_variable_to_xml] = ACTIONS(2789),
    [sym_xml_to_variable] = ACTIONS(2789),
    [sym_length_variable_to_xml] = ACTIONS(2789),
    [sym_redirect_string] = ACTIONS(2789),
    [sym_random_number] = ACTIONS(2789),
    [sym_get_unique_id] = ACTIONS(2789),
    [sym_get_system_number] = ACTIONS(2789),
    [sym_set_system_number] = ACTIONS(2789),
    [sym_get_serial_number] = ACTIONS(2789),
    [sym_get_master_build] = ACTIONS(2789),
    [sym_set_validation_code] = ACTIONS(2789),
    [sym_reboot] = ACTIONS(2789),
    [sym_load_duet_module] = ACTIONS(2789),
    [sym_unload_duet_module] = ACTIONS(2789),
    [sym_duet_mem_size_set] = ACTIONS(2789),
    [sym_duet_mem_size_get] = ACTIONS(2789),
    [sym_dynamic_application_device] = ACTIONS(2789),
    [sym_dynamic_polled_port] = ACTIONS(2789),
    [sym_static_port_binding] = ACTIONS(2789),
    [sym_static_ip_binding] = ACTIONS(2789),
    [sym_get_dns_list] = ACTIONS(2789),
    [sym_set_dns_list] = ACTIONS(2789),
    [sym_get_ip_address] = ACTIONS(2789),
    [sym_set_ip_address] = ACTIONS(2789),
    [sym_get_url_list] = ACTIONS(2789),
    [sym_add_url_entry] = ACTIONS(2789),
    [sym_delete_url_entry] = ACTIONS(2789),
    [sym_abs_value] = ACTIONS(2789),
    [sym_max_value] = ACTIONS(2789),
    [sym_min_value] = ACTIONS(2789),
    [sym_exp_value] = ACTIONS(2789),
    [sym_log_value] = ACTIONS(2789),
    [sym_log10_value] = ACTIONS(2789),
    [sym_power_value] = ACTIONS(2789),
    [sym_sqrt_value] = ACTIONS(2789),
    [sym_raw_be] = ACTIONS(2789),
    [sym_raw_le] = ACTIONS(2789),
    [sym_internal_queue_size_set] = ACTIONS(2789),
    [sym_internal_queue_size_get] = ACTIONS(2789),
    [sym_internal_threshold_set] = ACTIONS(2789),
    [sym_internal_threshold_get] = ACTIONS(2789),
    [sym_rebuilt_event] = ACTIONS(2789),
    [sym_do_custom_event] = ACTIONS(2789),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2789),
    [sym_clkmgr_set_clk_source] = ACTIONS(2789),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2789),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2789),
    [sym_clkmgr_get_timezone] = ACTIONS(2789),
    [sym_clkmgr_set_timezone] = ACTIONS(2789),
    [sym_clkmgr_get_resync_period] = ACTIONS(2789),
    [sym_clkmgr_set_resync_period] = ACTIONS(2789),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2789),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2789),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2789),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2789),
    [sym_clkmgr_get_timeservers] = ACTIONS(2789),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2789),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2789),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2789),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2789),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2789),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2789),
    [sym_set_log_level] = ACTIONS(2789),
    [sym_get_log_level] = ACTIONS(2789),
    [sym_amx_log] = ACTIONS(2789),
    [sym_smtp_server_config_set] = ACTIONS(2789),
    [sym_smtp_server_config_get] = ACTIONS(2789),
    [sym_smtp_send] = ACTIONS(2789),
    [sym_validate_netlinx_account] = ACTIONS(2789),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2789),
    [sym_authenticate_certificate] = ACTIONS(2789),
    [sym_audit_netlinx_session_event] = ACTIONS(2789),
    [sym_audit_netlinx_generic_event] = ACTIONS(2789),
    [sym_get_available_flash_disk_space] = ACTIONS(2789),
    [sym_get_max_flash_disk_space] = ACTIONS(2789),
    [sym_data_create_feed] = ACTIONS(2789),
    [sym_data_delete_feed] = ACTIONS(2789),
    [sym_data_publish_feed] = ACTIONS(2789),
    [sym_data_get_published_feed] = ACTIONS(2789),
    [sym_data_add_record] = ACTIONS(2789),
    [sym_data_get_event_record] = ACTIONS(2789),
    [sym__wc_data_create_feed] = ACTIONS(2789),
    [sym__wc_data_add_record] = ACTIONS(2789),
    [sym__wc_data_get_event_record] = ACTIONS(2789),
    [sym_format] = ACTIONS(2789),
    [sym_astro_clock] = ACTIONS(2789),
    [sym_set_outdoor_temperature] = ACTIONS(2789),
    [sym_set_virtual_level_count] = ACTIONS(2789),
    [sym_set_virtual_channel_count] = ACTIONS(2789),
    [sym_set_virtual_port_count] = ACTIONS(2789),
    [sym__wc_encode_int] = ACTIONS(2789),
    [sym__wc_decode_int] = ACTIONS(2789),
    [sym__wc_int] = ACTIONS(2789),
    [sym__wc_to_ch_int] = ACTIONS(2789),
    [sym__ch_to_wc_int] = ACTIONS(2789),
    [sym__wc_find_string_int] = ACTIONS(2789),
    [sym__wc_left_string_int] = ACTIONS(2789),
    [sym__wc_lower_string_int] = ACTIONS(2789),
    [sym__wc_mid_string_int] = ACTIONS(2789),
    [sym__wc_remove_string_int] = ACTIONS(2789),
    [sym__wc_right_string_int] = ACTIONS(2789),
    [sym__wc_upper_string_int] = ACTIONS(2789),
    [sym__wc_compare_string_int] = ACTIONS(2789),
    [sym__wc_get_buffer_char_int] = ACTIONS(2789),
    [sym__wc_get_buffer_string_int] = ACTIONS(2789),
    [sym__wc_concat_string_int] = ACTIONS(2789),
    [sym__wc_file_open_int] = ACTIONS(2789),
    [sym__wc_file_close_int] = ACTIONS(2789),
    [sym__wc_file_read_int] = ACTIONS(2789),
    [sym__wc_file_read_line_int] = ACTIONS(2789),
    [sym__wc_file_write_int] = ACTIONS(2789),
    [sym__wc_file_write_line_int] = ACTIONS(2789),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2791),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2791),
    [anon_sym_LBRACK] = ACTIONS(2791),
    [anon_sym_LBRACE] = ACTIONS(2791),
    [anon_sym_RBRACE] = ACTIONS(2791),
    [anon_sym_LPAREN] = ACTIONS(2789),
    [anon_sym_BANG] = ACTIONS(2791),
    [anon_sym_TILDE] = ACTIONS(2791),
    [anon_sym_DASH] = ACTIONS(2789),
    [anon_sym_PLUS] = ACTIONS(2789),
    [anon_sym_DASH_DASH] = ACTIONS(2791),
    [anon_sym_PLUS_PLUS] = ACTIONS(2791),
    [anon_sym_DQUOTE] = ACTIONS(2791),
    [anon_sym_SQUOTE] = ACTIONS(2791),
    [sym_number_literal] = ACTIONS(2791),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(932)] = {
    [sym_identifier] = ACTIONS(2793),
    [sym_not] = ACTIONS(2793),
    [sym_bnot] = ACTIONS(2793),
    [sym___file__] = ACTIONS(2793),
    [sym___date__] = ACTIONS(2793),
    [sym___ldate__] = ACTIONS(2793),
    [sym___time__] = ACTIONS(2793),
    [sym___line__] = ACTIONS(2793),
    [sym___name__] = ACTIONS(2793),
    [sym___netlinx__] = ACTIONS(2793),
    [sym_netlinx_axi_version] = ACTIONS(2793),
    [sym_url_flg_tcp] = ACTIONS(2793),
    [sym_url_flg_accinfopresent] = ACTIONS(2793),
    [sym_url_flg_temp] = ACTIONS(2793),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2793),
    [sym_url_flg_stat_mask] = ACTIONS(2793),
    [sym_url_flg_stat_lookup] = ACTIONS(2793),
    [sym_url_flg_stat_connecting] = ACTIONS(2793),
    [sym_url_flg_stat_waiting] = ACTIONS(2793),
    [sym_url_flg_stat_connected] = ACTIONS(2793),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2793),
    [sym_first_virtual_device] = ACTIONS(2793),
    [sym_first_local_port] = ACTIONS(2793),
    [sym_dynamic_virtual_device] = ACTIONS(2793),
    [sym_timeline_once] = ACTIONS(2793),
    [sym_timeline_repeat] = ACTIONS(2793),
    [sym_timeline_absolute] = ACTIONS(2793),
    [sym_timeline_relative] = ACTIONS(2793),
    [sym_do_push_timed_infinite] = ACTIONS(2793),
    [sym_source_type_no_address] = ACTIONS(2793),
    [sym_source_type_neuron_id] = ACTIONS(2793),
    [sym_source_type_ip_address] = ACTIONS(2793),
    [sym_source_type_axlink] = ACTIONS(2793),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2793),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2793),
    [sym_source_type_ip_socket_address] = ACTIONS(2793),
    [sym_source_type_rs232] = ACTIONS(2793),
    [sym_source_type_internal] = ACTIONS(2793),
    [sym_source_type_mac_address] = ACTIONS(2793),
    [sym_source_type_ipv4_port] = ACTIONS(2793),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2793),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2793),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2793),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2793),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2793),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2793),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2793),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2793),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2793),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2793),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2793),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2793),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2793),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2793),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2793),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2793),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2793),
    [sym_true] = ACTIONS(2793),
    [sym_false] = ACTIONS(2793),
    [sym_file_read_only] = ACTIONS(2793),
    [sym_file_rw_new] = ACTIONS(2793),
    [sym_file_rw_append] = ACTIONS(2793),
    [sym_ip_tcp] = ACTIONS(2793),
    [sym_ip_udp] = ACTIONS(2793),
    [sym_ip_udp_2way] = ACTIONS(2793),
    [sym_xml_encode_types] = ACTIONS(2793),
    [sym_xml_encode_char_as_list] = ACTIONS(2793),
    [sym_xml_encode_le] = ACTIONS(2793),
    [sym_xml_decode_types] = ACTIONS(2793),
    [sym_xml_decode_no_preserve] = ACTIONS(2793),
    [sym_duet_dev_type_display_device] = ACTIONS(2793),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2793),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2793),
    [sym_duet_dev_type_io_device] = ACTIONS(2793),
    [sym_duet_dev_type_relay_device] = ACTIONS(2793),
    [sym_duet_dev_type_ups] = ACTIONS(2793),
    [sym_duet_dev_type_amplifier] = ACTIONS(2793),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2793),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2793),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2793),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2793),
    [sym_duet_dev_type_camera] = ACTIONS(2793),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2793),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2793),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2793),
    [sym_duet_dev_type_dss] = ACTIONS(2793),
    [sym_duet_dev_type_dvr] = ACTIONS(2793),
    [sym_duet_dev_type_disc_device] = ACTIONS(2793),
    [sym_duet_dev_type_document_camera] = ACTIONS(2793),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2793),
    [sym_duet_dev_type_hvac] = ACTIONS(2793),
    [sym_duet_dev_type_keypad] = ACTIONS(2793),
    [sym_duet_dev_type_light] = ACTIONS(2793),
    [sym_duet_dev_type_monitor] = ACTIONS(2793),
    [sym_duet_dev_type_motor] = ACTIONS(2793),
    [sym_duet_dev_type_multi_window] = ACTIONS(2793),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2793),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2793),
    [sym_duet_dev_type_receiver] = ACTIONS(2793),
    [sym_duet_dev_type_security_system] = ACTIONS(2793),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2793),
    [sym_duet_dev_type_settop_box] = ACTIONS(2793),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2793),
    [sym_duet_dev_type_switcher] = ACTIONS(2793),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2793),
    [sym_duet_dev_type_tv] = ACTIONS(2793),
    [sym_duet_dev_type_utility] = ACTIONS(2793),
    [sym_duet_dev_type_vcr] = ACTIONS(2793),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2793),
    [sym_duet_dev_type_video_processor] = ACTIONS(2793),
    [sym_duet_dev_type_video_projector] = ACTIONS(2793),
    [sym_duet_dev_type_video_wall] = ACTIONS(2793),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2793),
    [sym_duet_dev_type_weather] = ACTIONS(2793),
    [sym_duet_dev_not_polled] = ACTIONS(2793),
    [sym_duet_dev_polled] = ACTIONS(2793),
    [sym_clkmgr_mode_network] = ACTIONS(2793),
    [sym_clkmgr_mode_standalone] = ACTIONS(2793),
    [sym_normal_standby] = ACTIONS(2793),
    [sym_normal_wake] = ACTIONS(2793),
    [sym_ip_multicast_ttl_option] = ACTIONS(2793),
    [sym_ip_ttl_subnet] = ACTIONS(2793),
    [sym_ip_ttl_site] = ACTIONS(2793),
    [sym_ip_ttl_region] = ACTIONS(2793),
    [sym_ip_ttl_continent] = ACTIONS(2793),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2793),
    [sym_ip_nodelay_on] = ACTIONS(2793),
    [sym_ip_nodelay_off] = ACTIONS(2793),
    [sym_amx_error] = ACTIONS(2793),
    [sym_amx_warning] = ACTIONS(2793),
    [sym_amx_info] = ACTIONS(2793),
    [sym_amx_debug] = ACTIONS(2793),
    [sym_netlinx_login_success] = ACTIONS(2793),
    [sym_netlinx_login_fail] = ACTIONS(2793),
    [sym_netlinx_logout] = ACTIONS(2793),
    [sym_null_str] = ACTIONS(2793),
    [sym_smtp_address] = ACTIONS(2793),
    [sym_smtp_port_number] = ACTIONS(2793),
    [sym_smtp_username] = ACTIONS(2793),
    [sym_smtp_password] = ACTIONS(2793),
    [sym_smtp_require_tls] = ACTIONS(2793),
    [sym_smtp_from] = ACTIONS(2793),
    [sym_smtp_tls_true] = ACTIONS(2793),
    [sym_smtp_tls_false] = ACTIONS(2793),
    [sym_listview_on_row_select_event] = ACTIONS(2793),
    [sym_data_structure_unknown] = ACTIONS(2793),
    [sym_data_structure_datafeed] = ACTIONS(2793),
    [sym_data_structure_datarecordset] = ACTIONS(2793),
    [sym_data_structure_datarecord] = ACTIONS(2793),
    [sym_data_structure_datafield] = ACTIONS(2793),
    [sym_data_max_content_field_count] = ACTIONS(2793),
    [sym_data_max_metadata_field_count] = ACTIONS(2793),
    [sym_data_max_id_length] = ACTIONS(2793),
    [sym_data_max_name_length] = ACTIONS(2793),
    [sym_data_max_description_length] = ACTIONS(2793),
    [sym_data_max_type_length] = ACTIONS(2793),
    [sym_data_max_format_length] = ACTIONS(2793),
    [sym_data_max_source_length] = ACTIONS(2793),
    [sym_data_max_value_length] = ACTIONS(2793),
    [sym_data_max_label_length] = ACTIONS(2793),
    [sym_data_max_url_length] = ACTIONS(2793),
    [sym_max_last_login_info_length] = ACTIONS(2793),
    [sym_data_type_unknown] = ACTIONS(2793),
    [sym_data_type_string] = ACTIONS(2793),
    [sym_data_type_datetime] = ACTIONS(2793),
    [sym_data_type_date] = ACTIONS(2793),
    [sym_data_type_time] = ACTIONS(2793),
    [sym_data_type_image] = ACTIONS(2793),
    [sym_data_format_url] = ACTIONS(2793),
    [sym_data_format_phone] = ACTIONS(2793),
    [sym_data_format_email] = ACTIONS(2793),
    [sym_data_format_iso8601] = ACTIONS(2793),
    [sym_auditlog_priv] = ACTIONS(2793),
    [sym_configuration_priv] = ACTIONS(2793),
    [sym_deviceconfig_priv] = ACTIONS(2793),
    [sym_ftp_priv] = ACTIONS(2793),
    [sym_http_priv] = ACTIONS(2793),
    [sym_networkconfig_priv] = ACTIONS(2793),
    [sym_programport_priv] = ACTIONS(2793),
    [sym_remoteui_priv] = ACTIONS(2793),
    [sym_securitycontrol_priv] = ACTIONS(2793),
    [sym_softwaremanagement_priv] = ACTIONS(2793),
    [sym_terminal_priv] = ACTIONS(2793),
    [sym_tpadmin_priv] = ACTIONS(2793),
    [sym_user1_priv] = ACTIONS(2793),
    [sym_user2_priv] = ACTIONS(2793),
    [sym_user3_priv] = ACTIONS(2793),
    [sym_user4_priv] = ACTIONS(2793),
    [sym_usermanagement_priv] = ACTIONS(2793),
    [sym_valid_account] = ACTIONS(2793),
    [sym_err_name_invalid] = ACTIONS(2793),
    [sym_err_password_invalid] = ACTIONS(2793),
    [sym_err_invalid_account] = ACTIONS(2793),
    [sym_err_no_authorization] = ACTIONS(2793),
    [sym_err_invalid_parameter] = ACTIONS(2793),
    [sym_err_user_locked_out] = ACTIONS(2793),
    [sym_err_user_account_expired] = ACTIONS(2793),
    [sym_err_general_error] = ACTIONS(2793),
    [sym_err_authentication_error] = ACTIONS(2793),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2793),
    [sym_tls_validate_certificate] = ACTIONS(2793),
    [sym____reserved___] = ACTIONS(2793),
    [sym_date] = ACTIONS(2793),
    [sym_day] = ACTIONS(2793),
    [sym_time] = ACTIONS(2793),
    [sym_ldate] = ACTIONS(2793),
    [sym_push_device] = ACTIONS(2793),
    [sym_push_channel] = ACTIONS(2793),
    [sym_push_devchan] = ACTIONS(2793),
    [sym_release_device] = ACTIONS(2793),
    [sym_release_channel] = ACTIONS(2793),
    [sym_release_devchan] = ACTIONS(2793),
    [sym_master_slot] = ACTIONS(2793),
    [sym_get_pulse_time] = ACTIONS(2793),
    [sym_get_timer] = ACTIONS(2793),
    [sym_master_sn] = ACTIONS(2793),
    [sym_system_number] = ACTIONS(2793),
    [sym_button] = ACTIONS(2793),
    [sym_dv_channel] = ACTIONS(2793),
    [sym_data] = ACTIONS(2793),
    [sym_level] = ACTIONS(2793),
    [sym_timeline] = ACTIONS(2793),
    [sym_channel] = ACTIONS(2793),
    [sym_custom] = ACTIONS(2793),
    [sym_atoi] = ACTIONS(2793),
    [sym_atol] = ACTIONS(2793),
    [sym_atof] = ACTIONS(2793),
    [sym_itoa] = ACTIONS(2793),
    [sym_itohex] = ACTIONS(2793),
    [sym_hextoi] = ACTIONS(2793),
    [sym_ftoa] = ACTIONS(2793),
    [sym_find_string] = ACTIONS(2793),
    [sym_left_string] = ACTIONS(2793),
    [sym_right_string] = ACTIONS(2793),
    [sym_mid_string] = ACTIONS(2793),
    [sym_remove_string] = ACTIONS(2793),
    [sym_get_buffer_string] = ACTIONS(2793),
    [sym_get_buffer_char] = ACTIONS(2793),
    [sym_get_multi_buffer_string] = ACTIONS(2793),
    [sym_length_string] = ACTIONS(2793),
    [sym_max_length_string] = ACTIONS(2793),
    [sym_set_length_string] = ACTIONS(2793),
    [sym_compare_string] = ACTIONS(2793),
    [sym_upper_string] = ACTIONS(2793),
    [sym_lower_string] = ACTIONS(2793),
    [sym_get_last] = ACTIONS(2793),
    [sym_date_to_day] = ACTIONS(2793),
    [sym_date_to_month] = ACTIONS(2793),
    [sym_date_to_year] = ACTIONS(2793),
    [sym_time_to_hour] = ACTIONS(2793),
    [sym_time_to_minute] = ACTIONS(2793),
    [sym_time_to_second] = ACTIONS(2793),
    [sym_day_of_week] = ACTIONS(2793),
    [sym_set_pulse_time] = ACTIONS(2793),
    [sym_set_timer] = ACTIONS(2793),
    [sym_file_open] = ACTIONS(2793),
    [sym_file_close] = ACTIONS(2793),
    [sym_file_read] = ACTIONS(2793),
    [sym_file_read_line] = ACTIONS(2793),
    [sym_file_write] = ACTIONS(2793),
    [sym_file_write_line] = ACTIONS(2793),
    [sym_file_copy] = ACTIONS(2793),
    [sym_file_delete] = ACTIONS(2793),
    [sym_file_rename] = ACTIONS(2793),
    [sym_file_dir] = ACTIONS(2793),
    [sym_file_getdir] = ACTIONS(2793),
    [sym_file_setdir] = ACTIONS(2793),
    [sym_file_create_dir] = ACTIONS(2793),
    [sym_file_remove_dir] = ACTIONS(2793),
    [sym_file_seek] = ACTIONS(2793),
    [sym_length_array] = ACTIONS(2793),
    [sym_max_length_array] = ACTIONS(2793),
    [sym_set_length_array] = ACTIONS(2793),
    [sym_ip_client_open] = ACTIONS(2793),
    [sym_ip_client_close] = ACTIONS(2793),
    [sym_ip_server_open] = ACTIONS(2793),
    [sym_ip_server_close] = ACTIONS(2793),
    [sym_ip_bound_client_open] = ACTIONS(2793),
    [sym_ip_mc_server_open] = ACTIONS(2793),
    [sym_ip_set_option] = ACTIONS(2793),
    [sym_ssh_client_open] = ACTIONS(2793),
    [sym_ssh_client_close] = ACTIONS(2793),
    [sym_tls_client_open] = ACTIONS(2793),
    [sym_tls_client_close] = ACTIONS(2793),
    [sym_do_push] = ACTIONS(2793),
    [sym_do_push_timed] = ACTIONS(2793),
    [sym_do_release] = ACTIONS(2793),
    [sym_device_id] = ACTIONS(2793),
    [sym_device_info] = ACTIONS(2793),
    [sym_device_id_string] = ACTIONS(2793),
    [sym_device_standby] = ACTIONS(2793),
    [sym_device_wake] = ACTIONS(2793),
    [sym_timeline_create] = ACTIONS(2793),
    [sym_timeline_kill] = ACTIONS(2793),
    [sym_timeline_reload] = ACTIONS(2793),
    [sym_timeline_pause] = ACTIONS(2793),
    [sym_timeline_restart] = ACTIONS(2793),
    [sym_timeline_set] = ACTIONS(2793),
    [sym_timeline_get] = ACTIONS(2793),
    [sym_timeline_active] = ACTIONS(2793),
    [sym_timeline_dynamic_id] = ACTIONS(2793),
    [sym_variable_to_string] = ACTIONS(2793),
    [sym_string_to_variable] = ACTIONS(2793),
    [sym_length_variable_to_string] = ACTIONS(2793),
    [sym_type_cast] = ACTIONS(2793),
    [sym_variable_to_xml] = ACTIONS(2793),
    [sym_xml_to_variable] = ACTIONS(2793),
    [sym_length_variable_to_xml] = ACTIONS(2793),
    [sym_redirect_string] = ACTIONS(2793),
    [sym_random_number] = ACTIONS(2793),
    [sym_get_unique_id] = ACTIONS(2793),
    [sym_get_system_number] = ACTIONS(2793),
    [sym_set_system_number] = ACTIONS(2793),
    [sym_get_serial_number] = ACTIONS(2793),
    [sym_get_master_build] = ACTIONS(2793),
    [sym_set_validation_code] = ACTIONS(2793),
    [sym_reboot] = ACTIONS(2793),
    [sym_load_duet_module] = ACTIONS(2793),
    [sym_unload_duet_module] = ACTIONS(2793),
    [sym_duet_mem_size_set] = ACTIONS(2793),
    [sym_duet_mem_size_get] = ACTIONS(2793),
    [sym_dynamic_application_device] = ACTIONS(2793),
    [sym_dynamic_polled_port] = ACTIONS(2793),
    [sym_static_port_binding] = ACTIONS(2793),
    [sym_static_ip_binding] = ACTIONS(2793),
    [sym_get_dns_list] = ACTIONS(2793),
    [sym_set_dns_list] = ACTIONS(2793),
    [sym_get_ip_address] = ACTIONS(2793),
    [sym_set_ip_address] = ACTIONS(2793),
    [sym_get_url_list] = ACTIONS(2793),
    [sym_add_url_entry] = ACTIONS(2793),
    [sym_delete_url_entry] = ACTIONS(2793),
    [sym_abs_value] = ACTIONS(2793),
    [sym_max_value] = ACTIONS(2793),
    [sym_min_value] = ACTIONS(2793),
    [sym_exp_value] = ACTIONS(2793),
    [sym_log_value] = ACTIONS(2793),
    [sym_log10_value] = ACTIONS(2793),
    [sym_power_value] = ACTIONS(2793),
    [sym_sqrt_value] = ACTIONS(2793),
    [sym_raw_be] = ACTIONS(2793),
    [sym_raw_le] = ACTIONS(2793),
    [sym_internal_queue_size_set] = ACTIONS(2793),
    [sym_internal_queue_size_get] = ACTIONS(2793),
    [sym_internal_threshold_set] = ACTIONS(2793),
    [sym_internal_threshold_get] = ACTIONS(2793),
    [sym_rebuilt_event] = ACTIONS(2793),
    [sym_do_custom_event] = ACTIONS(2793),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2793),
    [sym_clkmgr_set_clk_source] = ACTIONS(2793),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2793),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2793),
    [sym_clkmgr_get_timezone] = ACTIONS(2793),
    [sym_clkmgr_set_timezone] = ACTIONS(2793),
    [sym_clkmgr_get_resync_period] = ACTIONS(2793),
    [sym_clkmgr_set_resync_period] = ACTIONS(2793),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2793),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2793),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2793),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2793),
    [sym_clkmgr_get_timeservers] = ACTIONS(2793),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2793),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2793),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2793),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2793),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2793),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2793),
    [sym_set_log_level] = ACTIONS(2793),
    [sym_get_log_level] = ACTIONS(2793),
    [sym_amx_log] = ACTIONS(2793),
    [sym_smtp_server_config_set] = ACTIONS(2793),
    [sym_smtp_server_config_get] = ACTIONS(2793),
    [sym_smtp_send] = ACTIONS(2793),
    [sym_validate_netlinx_account] = ACTIONS(2793),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2793),
    [sym_authenticate_certificate] = ACTIONS(2793),
    [sym_audit_netlinx_session_event] = ACTIONS(2793),
    [sym_audit_netlinx_generic_event] = ACTIONS(2793),
    [sym_get_available_flash_disk_space] = ACTIONS(2793),
    [sym_get_max_flash_disk_space] = ACTIONS(2793),
    [sym_data_create_feed] = ACTIONS(2793),
    [sym_data_delete_feed] = ACTIONS(2793),
    [sym_data_publish_feed] = ACTIONS(2793),
    [sym_data_get_published_feed] = ACTIONS(2793),
    [sym_data_add_record] = ACTIONS(2793),
    [sym_data_get_event_record] = ACTIONS(2793),
    [sym__wc_data_create_feed] = ACTIONS(2793),
    [sym__wc_data_add_record] = ACTIONS(2793),
    [sym__wc_data_get_event_record] = ACTIONS(2793),
    [sym_format] = ACTIONS(2793),
    [sym_astro_clock] = ACTIONS(2793),
    [sym_set_outdoor_temperature] = ACTIONS(2793),
    [sym_set_virtual_level_count] = ACTIONS(2793),
    [sym_set_virtual_channel_count] = ACTIONS(2793),
    [sym_set_virtual_port_count] = ACTIONS(2793),
    [sym__wc_encode_int] = ACTIONS(2793),
    [sym__wc_decode_int] = ACTIONS(2793),
    [sym__wc_int] = ACTIONS(2793),
    [sym__wc_to_ch_int] = ACTIONS(2793),
    [sym__ch_to_wc_int] = ACTIONS(2793),
    [sym__wc_find_string_int] = ACTIONS(2793),
    [sym__wc_left_string_int] = ACTIONS(2793),
    [sym__wc_lower_string_int] = ACTIONS(2793),
    [sym__wc_mid_string_int] = ACTIONS(2793),
    [sym__wc_remove_string_int] = ACTIONS(2793),
    [sym__wc_right_string_int] = ACTIONS(2793),
    [sym__wc_upper_string_int] = ACTIONS(2793),
    [sym__wc_compare_string_int] = ACTIONS(2793),
    [sym__wc_get_buffer_char_int] = ACTIONS(2793),
    [sym__wc_get_buffer_string_int] = ACTIONS(2793),
    [sym__wc_concat_string_int] = ACTIONS(2793),
    [sym__wc_file_open_int] = ACTIONS(2793),
    [sym__wc_file_close_int] = ACTIONS(2793),
    [sym__wc_file_read_int] = ACTIONS(2793),
    [sym__wc_file_read_line_int] = ACTIONS(2793),
    [sym__wc_file_write_int] = ACTIONS(2793),
    [sym__wc_file_write_line_int] = ACTIONS(2793),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2795),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2795),
    [anon_sym_LBRACK] = ACTIONS(2795),
    [anon_sym_LBRACE] = ACTIONS(2795),
    [anon_sym_RBRACE] = ACTIONS(2795),
    [anon_sym_LPAREN] = ACTIONS(2793),
    [anon_sym_BANG] = ACTIONS(2795),
    [anon_sym_TILDE] = ACTIONS(2795),
    [anon_sym_DASH] = ACTIONS(2793),
    [anon_sym_PLUS] = ACTIONS(2793),
    [anon_sym_DASH_DASH] = ACTIONS(2795),
    [anon_sym_PLUS_PLUS] = ACTIONS(2795),
    [anon_sym_DQUOTE] = ACTIONS(2795),
    [anon_sym_SQUOTE] = ACTIONS(2795),
    [sym_number_literal] = ACTIONS(2795),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(933)] = {
    [sym_identifier] = ACTIONS(2797),
    [sym_not] = ACTIONS(2797),
    [sym_bnot] = ACTIONS(2797),
    [sym___file__] = ACTIONS(2797),
    [sym___date__] = ACTIONS(2797),
    [sym___ldate__] = ACTIONS(2797),
    [sym___time__] = ACTIONS(2797),
    [sym___line__] = ACTIONS(2797),
    [sym___name__] = ACTIONS(2797),
    [sym___netlinx__] = ACTIONS(2797),
    [sym_netlinx_axi_version] = ACTIONS(2797),
    [sym_url_flg_tcp] = ACTIONS(2797),
    [sym_url_flg_accinfopresent] = ACTIONS(2797),
    [sym_url_flg_temp] = ACTIONS(2797),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2797),
    [sym_url_flg_stat_mask] = ACTIONS(2797),
    [sym_url_flg_stat_lookup] = ACTIONS(2797),
    [sym_url_flg_stat_connecting] = ACTIONS(2797),
    [sym_url_flg_stat_waiting] = ACTIONS(2797),
    [sym_url_flg_stat_connected] = ACTIONS(2797),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2797),
    [sym_first_virtual_device] = ACTIONS(2797),
    [sym_first_local_port] = ACTIONS(2797),
    [sym_dynamic_virtual_device] = ACTIONS(2797),
    [sym_timeline_once] = ACTIONS(2797),
    [sym_timeline_repeat] = ACTIONS(2797),
    [sym_timeline_absolute] = ACTIONS(2797),
    [sym_timeline_relative] = ACTIONS(2797),
    [sym_do_push_timed_infinite] = ACTIONS(2797),
    [sym_source_type_no_address] = ACTIONS(2797),
    [sym_source_type_neuron_id] = ACTIONS(2797),
    [sym_source_type_ip_address] = ACTIONS(2797),
    [sym_source_type_axlink] = ACTIONS(2797),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2797),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2797),
    [sym_source_type_ip_socket_address] = ACTIONS(2797),
    [sym_source_type_rs232] = ACTIONS(2797),
    [sym_source_type_internal] = ACTIONS(2797),
    [sym_source_type_mac_address] = ACTIONS(2797),
    [sym_source_type_ipv4_port] = ACTIONS(2797),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2797),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2797),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2797),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2797),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2797),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2797),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2797),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2797),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2797),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2797),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2797),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2797),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2797),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2797),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2797),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2797),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2797),
    [sym_true] = ACTIONS(2797),
    [sym_false] = ACTIONS(2797),
    [sym_file_read_only] = ACTIONS(2797),
    [sym_file_rw_new] = ACTIONS(2797),
    [sym_file_rw_append] = ACTIONS(2797),
    [sym_ip_tcp] = ACTIONS(2797),
    [sym_ip_udp] = ACTIONS(2797),
    [sym_ip_udp_2way] = ACTIONS(2797),
    [sym_xml_encode_types] = ACTIONS(2797),
    [sym_xml_encode_char_as_list] = ACTIONS(2797),
    [sym_xml_encode_le] = ACTIONS(2797),
    [sym_xml_decode_types] = ACTIONS(2797),
    [sym_xml_decode_no_preserve] = ACTIONS(2797),
    [sym_duet_dev_type_display_device] = ACTIONS(2797),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2797),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2797),
    [sym_duet_dev_type_io_device] = ACTIONS(2797),
    [sym_duet_dev_type_relay_device] = ACTIONS(2797),
    [sym_duet_dev_type_ups] = ACTIONS(2797),
    [sym_duet_dev_type_amplifier] = ACTIONS(2797),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2797),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2797),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2797),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2797),
    [sym_duet_dev_type_camera] = ACTIONS(2797),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2797),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2797),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2797),
    [sym_duet_dev_type_dss] = ACTIONS(2797),
    [sym_duet_dev_type_dvr] = ACTIONS(2797),
    [sym_duet_dev_type_disc_device] = ACTIONS(2797),
    [sym_duet_dev_type_document_camera] = ACTIONS(2797),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2797),
    [sym_duet_dev_type_hvac] = ACTIONS(2797),
    [sym_duet_dev_type_keypad] = ACTIONS(2797),
    [sym_duet_dev_type_light] = ACTIONS(2797),
    [sym_duet_dev_type_monitor] = ACTIONS(2797),
    [sym_duet_dev_type_motor] = ACTIONS(2797),
    [sym_duet_dev_type_multi_window] = ACTIONS(2797),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2797),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2797),
    [sym_duet_dev_type_receiver] = ACTIONS(2797),
    [sym_duet_dev_type_security_system] = ACTIONS(2797),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2797),
    [sym_duet_dev_type_settop_box] = ACTIONS(2797),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2797),
    [sym_duet_dev_type_switcher] = ACTIONS(2797),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2797),
    [sym_duet_dev_type_tv] = ACTIONS(2797),
    [sym_duet_dev_type_utility] = ACTIONS(2797),
    [sym_duet_dev_type_vcr] = ACTIONS(2797),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2797),
    [sym_duet_dev_type_video_processor] = ACTIONS(2797),
    [sym_duet_dev_type_video_projector] = ACTIONS(2797),
    [sym_duet_dev_type_video_wall] = ACTIONS(2797),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2797),
    [sym_duet_dev_type_weather] = ACTIONS(2797),
    [sym_duet_dev_not_polled] = ACTIONS(2797),
    [sym_duet_dev_polled] = ACTIONS(2797),
    [sym_clkmgr_mode_network] = ACTIONS(2797),
    [sym_clkmgr_mode_standalone] = ACTIONS(2797),
    [sym_normal_standby] = ACTIONS(2797),
    [sym_normal_wake] = ACTIONS(2797),
    [sym_ip_multicast_ttl_option] = ACTIONS(2797),
    [sym_ip_ttl_subnet] = ACTIONS(2797),
    [sym_ip_ttl_site] = ACTIONS(2797),
    [sym_ip_ttl_region] = ACTIONS(2797),
    [sym_ip_ttl_continent] = ACTIONS(2797),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2797),
    [sym_ip_nodelay_on] = ACTIONS(2797),
    [sym_ip_nodelay_off] = ACTIONS(2797),
    [sym_amx_error] = ACTIONS(2797),
    [sym_amx_warning] = ACTIONS(2797),
    [sym_amx_info] = ACTIONS(2797),
    [sym_amx_debug] = ACTIONS(2797),
    [sym_netlinx_login_success] = ACTIONS(2797),
    [sym_netlinx_login_fail] = ACTIONS(2797),
    [sym_netlinx_logout] = ACTIONS(2797),
    [sym_null_str] = ACTIONS(2797),
    [sym_smtp_address] = ACTIONS(2797),
    [sym_smtp_port_number] = ACTIONS(2797),
    [sym_smtp_username] = ACTIONS(2797),
    [sym_smtp_password] = ACTIONS(2797),
    [sym_smtp_require_tls] = ACTIONS(2797),
    [sym_smtp_from] = ACTIONS(2797),
    [sym_smtp_tls_true] = ACTIONS(2797),
    [sym_smtp_tls_false] = ACTIONS(2797),
    [sym_listview_on_row_select_event] = ACTIONS(2797),
    [sym_data_structure_unknown] = ACTIONS(2797),
    [sym_data_structure_datafeed] = ACTIONS(2797),
    [sym_data_structure_datarecordset] = ACTIONS(2797),
    [sym_data_structure_datarecord] = ACTIONS(2797),
    [sym_data_structure_datafield] = ACTIONS(2797),
    [sym_data_max_content_field_count] = ACTIONS(2797),
    [sym_data_max_metadata_field_count] = ACTIONS(2797),
    [sym_data_max_id_length] = ACTIONS(2797),
    [sym_data_max_name_length] = ACTIONS(2797),
    [sym_data_max_description_length] = ACTIONS(2797),
    [sym_data_max_type_length] = ACTIONS(2797),
    [sym_data_max_format_length] = ACTIONS(2797),
    [sym_data_max_source_length] = ACTIONS(2797),
    [sym_data_max_value_length] = ACTIONS(2797),
    [sym_data_max_label_length] = ACTIONS(2797),
    [sym_data_max_url_length] = ACTIONS(2797),
    [sym_max_last_login_info_length] = ACTIONS(2797),
    [sym_data_type_unknown] = ACTIONS(2797),
    [sym_data_type_string] = ACTIONS(2797),
    [sym_data_type_datetime] = ACTIONS(2797),
    [sym_data_type_date] = ACTIONS(2797),
    [sym_data_type_time] = ACTIONS(2797),
    [sym_data_type_image] = ACTIONS(2797),
    [sym_data_format_url] = ACTIONS(2797),
    [sym_data_format_phone] = ACTIONS(2797),
    [sym_data_format_email] = ACTIONS(2797),
    [sym_data_format_iso8601] = ACTIONS(2797),
    [sym_auditlog_priv] = ACTIONS(2797),
    [sym_configuration_priv] = ACTIONS(2797),
    [sym_deviceconfig_priv] = ACTIONS(2797),
    [sym_ftp_priv] = ACTIONS(2797),
    [sym_http_priv] = ACTIONS(2797),
    [sym_networkconfig_priv] = ACTIONS(2797),
    [sym_programport_priv] = ACTIONS(2797),
    [sym_remoteui_priv] = ACTIONS(2797),
    [sym_securitycontrol_priv] = ACTIONS(2797),
    [sym_softwaremanagement_priv] = ACTIONS(2797),
    [sym_terminal_priv] = ACTIONS(2797),
    [sym_tpadmin_priv] = ACTIONS(2797),
    [sym_user1_priv] = ACTIONS(2797),
    [sym_user2_priv] = ACTIONS(2797),
    [sym_user3_priv] = ACTIONS(2797),
    [sym_user4_priv] = ACTIONS(2797),
    [sym_usermanagement_priv] = ACTIONS(2797),
    [sym_valid_account] = ACTIONS(2797),
    [sym_err_name_invalid] = ACTIONS(2797),
    [sym_err_password_invalid] = ACTIONS(2797),
    [sym_err_invalid_account] = ACTIONS(2797),
    [sym_err_no_authorization] = ACTIONS(2797),
    [sym_err_invalid_parameter] = ACTIONS(2797),
    [sym_err_user_locked_out] = ACTIONS(2797),
    [sym_err_user_account_expired] = ACTIONS(2797),
    [sym_err_general_error] = ACTIONS(2797),
    [sym_err_authentication_error] = ACTIONS(2797),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2797),
    [sym_tls_validate_certificate] = ACTIONS(2797),
    [sym____reserved___] = ACTIONS(2797),
    [sym_date] = ACTIONS(2797),
    [sym_day] = ACTIONS(2797),
    [sym_time] = ACTIONS(2797),
    [sym_ldate] = ACTIONS(2797),
    [sym_push_device] = ACTIONS(2797),
    [sym_push_channel] = ACTIONS(2797),
    [sym_push_devchan] = ACTIONS(2797),
    [sym_release_device] = ACTIONS(2797),
    [sym_release_channel] = ACTIONS(2797),
    [sym_release_devchan] = ACTIONS(2797),
    [sym_master_slot] = ACTIONS(2797),
    [sym_get_pulse_time] = ACTIONS(2797),
    [sym_get_timer] = ACTIONS(2797),
    [sym_master_sn] = ACTIONS(2797),
    [sym_system_number] = ACTIONS(2797),
    [sym_button] = ACTIONS(2797),
    [sym_dv_channel] = ACTIONS(2797),
    [sym_data] = ACTIONS(2797),
    [sym_level] = ACTIONS(2797),
    [sym_timeline] = ACTIONS(2797),
    [sym_channel] = ACTIONS(2797),
    [sym_custom] = ACTIONS(2797),
    [sym_atoi] = ACTIONS(2797),
    [sym_atol] = ACTIONS(2797),
    [sym_atof] = ACTIONS(2797),
    [sym_itoa] = ACTIONS(2797),
    [sym_itohex] = ACTIONS(2797),
    [sym_hextoi] = ACTIONS(2797),
    [sym_ftoa] = ACTIONS(2797),
    [sym_find_string] = ACTIONS(2797),
    [sym_left_string] = ACTIONS(2797),
    [sym_right_string] = ACTIONS(2797),
    [sym_mid_string] = ACTIONS(2797),
    [sym_remove_string] = ACTIONS(2797),
    [sym_get_buffer_string] = ACTIONS(2797),
    [sym_get_buffer_char] = ACTIONS(2797),
    [sym_get_multi_buffer_string] = ACTIONS(2797),
    [sym_length_string] = ACTIONS(2797),
    [sym_max_length_string] = ACTIONS(2797),
    [sym_set_length_string] = ACTIONS(2797),
    [sym_compare_string] = ACTIONS(2797),
    [sym_upper_string] = ACTIONS(2797),
    [sym_lower_string] = ACTIONS(2797),
    [sym_get_last] = ACTIONS(2797),
    [sym_date_to_day] = ACTIONS(2797),
    [sym_date_to_month] = ACTIONS(2797),
    [sym_date_to_year] = ACTIONS(2797),
    [sym_time_to_hour] = ACTIONS(2797),
    [sym_time_to_minute] = ACTIONS(2797),
    [sym_time_to_second] = ACTIONS(2797),
    [sym_day_of_week] = ACTIONS(2797),
    [sym_set_pulse_time] = ACTIONS(2797),
    [sym_set_timer] = ACTIONS(2797),
    [sym_file_open] = ACTIONS(2797),
    [sym_file_close] = ACTIONS(2797),
    [sym_file_read] = ACTIONS(2797),
    [sym_file_read_line] = ACTIONS(2797),
    [sym_file_write] = ACTIONS(2797),
    [sym_file_write_line] = ACTIONS(2797),
    [sym_file_copy] = ACTIONS(2797),
    [sym_file_delete] = ACTIONS(2797),
    [sym_file_rename] = ACTIONS(2797),
    [sym_file_dir] = ACTIONS(2797),
    [sym_file_getdir] = ACTIONS(2797),
    [sym_file_setdir] = ACTIONS(2797),
    [sym_file_create_dir] = ACTIONS(2797),
    [sym_file_remove_dir] = ACTIONS(2797),
    [sym_file_seek] = ACTIONS(2797),
    [sym_length_array] = ACTIONS(2797),
    [sym_max_length_array] = ACTIONS(2797),
    [sym_set_length_array] = ACTIONS(2797),
    [sym_ip_client_open] = ACTIONS(2797),
    [sym_ip_client_close] = ACTIONS(2797),
    [sym_ip_server_open] = ACTIONS(2797),
    [sym_ip_server_close] = ACTIONS(2797),
    [sym_ip_bound_client_open] = ACTIONS(2797),
    [sym_ip_mc_server_open] = ACTIONS(2797),
    [sym_ip_set_option] = ACTIONS(2797),
    [sym_ssh_client_open] = ACTIONS(2797),
    [sym_ssh_client_close] = ACTIONS(2797),
    [sym_tls_client_open] = ACTIONS(2797),
    [sym_tls_client_close] = ACTIONS(2797),
    [sym_do_push] = ACTIONS(2797),
    [sym_do_push_timed] = ACTIONS(2797),
    [sym_do_release] = ACTIONS(2797),
    [sym_device_id] = ACTIONS(2797),
    [sym_device_info] = ACTIONS(2797),
    [sym_device_id_string] = ACTIONS(2797),
    [sym_device_standby] = ACTIONS(2797),
    [sym_device_wake] = ACTIONS(2797),
    [sym_timeline_create] = ACTIONS(2797),
    [sym_timeline_kill] = ACTIONS(2797),
    [sym_timeline_reload] = ACTIONS(2797),
    [sym_timeline_pause] = ACTIONS(2797),
    [sym_timeline_restart] = ACTIONS(2797),
    [sym_timeline_set] = ACTIONS(2797),
    [sym_timeline_get] = ACTIONS(2797),
    [sym_timeline_active] = ACTIONS(2797),
    [sym_timeline_dynamic_id] = ACTIONS(2797),
    [sym_variable_to_string] = ACTIONS(2797),
    [sym_string_to_variable] = ACTIONS(2797),
    [sym_length_variable_to_string] = ACTIONS(2797),
    [sym_type_cast] = ACTIONS(2797),
    [sym_variable_to_xml] = ACTIONS(2797),
    [sym_xml_to_variable] = ACTIONS(2797),
    [sym_length_variable_to_xml] = ACTIONS(2797),
    [sym_redirect_string] = ACTIONS(2797),
    [sym_random_number] = ACTIONS(2797),
    [sym_get_unique_id] = ACTIONS(2797),
    [sym_get_system_number] = ACTIONS(2797),
    [sym_set_system_number] = ACTIONS(2797),
    [sym_get_serial_number] = ACTIONS(2797),
    [sym_get_master_build] = ACTIONS(2797),
    [sym_set_validation_code] = ACTIONS(2797),
    [sym_reboot] = ACTIONS(2797),
    [sym_load_duet_module] = ACTIONS(2797),
    [sym_unload_duet_module] = ACTIONS(2797),
    [sym_duet_mem_size_set] = ACTIONS(2797),
    [sym_duet_mem_size_get] = ACTIONS(2797),
    [sym_dynamic_application_device] = ACTIONS(2797),
    [sym_dynamic_polled_port] = ACTIONS(2797),
    [sym_static_port_binding] = ACTIONS(2797),
    [sym_static_ip_binding] = ACTIONS(2797),
    [sym_get_dns_list] = ACTIONS(2797),
    [sym_set_dns_list] = ACTIONS(2797),
    [sym_get_ip_address] = ACTIONS(2797),
    [sym_set_ip_address] = ACTIONS(2797),
    [sym_get_url_list] = ACTIONS(2797),
    [sym_add_url_entry] = ACTIONS(2797),
    [sym_delete_url_entry] = ACTIONS(2797),
    [sym_abs_value] = ACTIONS(2797),
    [sym_max_value] = ACTIONS(2797),
    [sym_min_value] = ACTIONS(2797),
    [sym_exp_value] = ACTIONS(2797),
    [sym_log_value] = ACTIONS(2797),
    [sym_log10_value] = ACTIONS(2797),
    [sym_power_value] = ACTIONS(2797),
    [sym_sqrt_value] = ACTIONS(2797),
    [sym_raw_be] = ACTIONS(2797),
    [sym_raw_le] = ACTIONS(2797),
    [sym_internal_queue_size_set] = ACTIONS(2797),
    [sym_internal_queue_size_get] = ACTIONS(2797),
    [sym_internal_threshold_set] = ACTIONS(2797),
    [sym_internal_threshold_get] = ACTIONS(2797),
    [sym_rebuilt_event] = ACTIONS(2797),
    [sym_do_custom_event] = ACTIONS(2797),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2797),
    [sym_clkmgr_set_clk_source] = ACTIONS(2797),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2797),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2797),
    [sym_clkmgr_get_timezone] = ACTIONS(2797),
    [sym_clkmgr_set_timezone] = ACTIONS(2797),
    [sym_clkmgr_get_resync_period] = ACTIONS(2797),
    [sym_clkmgr_set_resync_period] = ACTIONS(2797),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2797),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2797),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2797),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2797),
    [sym_clkmgr_get_timeservers] = ACTIONS(2797),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2797),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2797),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2797),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2797),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2797),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2797),
    [sym_set_log_level] = ACTIONS(2797),
    [sym_get_log_level] = ACTIONS(2797),
    [sym_amx_log] = ACTIONS(2797),
    [sym_smtp_server_config_set] = ACTIONS(2797),
    [sym_smtp_server_config_get] = ACTIONS(2797),
    [sym_smtp_send] = ACTIONS(2797),
    [sym_validate_netlinx_account] = ACTIONS(2797),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2797),
    [sym_authenticate_certificate] = ACTIONS(2797),
    [sym_audit_netlinx_session_event] = ACTIONS(2797),
    [sym_audit_netlinx_generic_event] = ACTIONS(2797),
    [sym_get_available_flash_disk_space] = ACTIONS(2797),
    [sym_get_max_flash_disk_space] = ACTIONS(2797),
    [sym_data_create_feed] = ACTIONS(2797),
    [sym_data_delete_feed] = ACTIONS(2797),
    [sym_data_publish_feed] = ACTIONS(2797),
    [sym_data_get_published_feed] = ACTIONS(2797),
    [sym_data_add_record] = ACTIONS(2797),
    [sym_data_get_event_record] = ACTIONS(2797),
    [sym__wc_data_create_feed] = ACTIONS(2797),
    [sym__wc_data_add_record] = ACTIONS(2797),
    [sym__wc_data_get_event_record] = ACTIONS(2797),
    [sym_format] = ACTIONS(2797),
    [sym_astro_clock] = ACTIONS(2797),
    [sym_set_outdoor_temperature] = ACTIONS(2797),
    [sym_set_virtual_level_count] = ACTIONS(2797),
    [sym_set_virtual_channel_count] = ACTIONS(2797),
    [sym_set_virtual_port_count] = ACTIONS(2797),
    [sym__wc_encode_int] = ACTIONS(2797),
    [sym__wc_decode_int] = ACTIONS(2797),
    [sym__wc_int] = ACTIONS(2797),
    [sym__wc_to_ch_int] = ACTIONS(2797),
    [sym__ch_to_wc_int] = ACTIONS(2797),
    [sym__wc_find_string_int] = ACTIONS(2797),
    [sym__wc_left_string_int] = ACTIONS(2797),
    [sym__wc_lower_string_int] = ACTIONS(2797),
    [sym__wc_mid_string_int] = ACTIONS(2797),
    [sym__wc_remove_string_int] = ACTIONS(2797),
    [sym__wc_right_string_int] = ACTIONS(2797),
    [sym__wc_upper_string_int] = ACTIONS(2797),
    [sym__wc_compare_string_int] = ACTIONS(2797),
    [sym__wc_get_buffer_char_int] = ACTIONS(2797),
    [sym__wc_get_buffer_string_int] = ACTIONS(2797),
    [sym__wc_concat_string_int] = ACTIONS(2797),
    [sym__wc_file_open_int] = ACTIONS(2797),
    [sym__wc_file_close_int] = ACTIONS(2797),
    [sym__wc_file_read_int] = ACTIONS(2797),
    [sym__wc_file_read_line_int] = ACTIONS(2797),
    [sym__wc_file_write_int] = ACTIONS(2797),
    [sym__wc_file_write_line_int] = ACTIONS(2797),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2799),
    [aux_sym_preproc_else_token1] = ACTIONS(2799),
    [anon_sym_COMMA] = ACTIONS(2799),
    [anon_sym_LBRACK] = ACTIONS(2799),
    [anon_sym_LBRACE] = ACTIONS(2799),
    [anon_sym_LPAREN] = ACTIONS(2797),
    [anon_sym_BANG] = ACTIONS(2799),
    [anon_sym_TILDE] = ACTIONS(2799),
    [anon_sym_DASH] = ACTIONS(2797),
    [anon_sym_PLUS] = ACTIONS(2797),
    [anon_sym_DASH_DASH] = ACTIONS(2799),
    [anon_sym_PLUS_PLUS] = ACTIONS(2799),
    [anon_sym_DQUOTE] = ACTIONS(2799),
    [anon_sym_SQUOTE] = ACTIONS(2799),
    [sym_number_literal] = ACTIONS(2799),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(934)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2253),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(935)] = {
    [sym_identifier] = ACTIONS(2801),
    [sym_not] = ACTIONS(2801),
    [sym_bnot] = ACTIONS(2801),
    [sym___file__] = ACTIONS(2801),
    [sym___date__] = ACTIONS(2801),
    [sym___ldate__] = ACTIONS(2801),
    [sym___time__] = ACTIONS(2801),
    [sym___line__] = ACTIONS(2801),
    [sym___name__] = ACTIONS(2801),
    [sym___netlinx__] = ACTIONS(2801),
    [sym_netlinx_axi_version] = ACTIONS(2801),
    [sym_url_flg_tcp] = ACTIONS(2801),
    [sym_url_flg_accinfopresent] = ACTIONS(2801),
    [sym_url_flg_temp] = ACTIONS(2801),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2801),
    [sym_url_flg_stat_mask] = ACTIONS(2801),
    [sym_url_flg_stat_lookup] = ACTIONS(2801),
    [sym_url_flg_stat_connecting] = ACTIONS(2801),
    [sym_url_flg_stat_waiting] = ACTIONS(2801),
    [sym_url_flg_stat_connected] = ACTIONS(2801),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2801),
    [sym_first_virtual_device] = ACTIONS(2801),
    [sym_first_local_port] = ACTIONS(2801),
    [sym_dynamic_virtual_device] = ACTIONS(2801),
    [sym_timeline_once] = ACTIONS(2801),
    [sym_timeline_repeat] = ACTIONS(2801),
    [sym_timeline_absolute] = ACTIONS(2801),
    [sym_timeline_relative] = ACTIONS(2801),
    [sym_do_push_timed_infinite] = ACTIONS(2801),
    [sym_source_type_no_address] = ACTIONS(2801),
    [sym_source_type_neuron_id] = ACTIONS(2801),
    [sym_source_type_ip_address] = ACTIONS(2801),
    [sym_source_type_axlink] = ACTIONS(2801),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2801),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2801),
    [sym_source_type_ip_socket_address] = ACTIONS(2801),
    [sym_source_type_rs232] = ACTIONS(2801),
    [sym_source_type_internal] = ACTIONS(2801),
    [sym_source_type_mac_address] = ACTIONS(2801),
    [sym_source_type_ipv4_port] = ACTIONS(2801),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2801),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2801),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2801),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2801),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2801),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2801),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2801),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2801),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2801),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2801),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2801),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2801),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2801),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2801),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2801),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2801),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2801),
    [sym_true] = ACTIONS(2801),
    [sym_false] = ACTIONS(2801),
    [sym_file_read_only] = ACTIONS(2801),
    [sym_file_rw_new] = ACTIONS(2801),
    [sym_file_rw_append] = ACTIONS(2801),
    [sym_ip_tcp] = ACTIONS(2801),
    [sym_ip_udp] = ACTIONS(2801),
    [sym_ip_udp_2way] = ACTIONS(2801),
    [sym_xml_encode_types] = ACTIONS(2801),
    [sym_xml_encode_char_as_list] = ACTIONS(2801),
    [sym_xml_encode_le] = ACTIONS(2801),
    [sym_xml_decode_types] = ACTIONS(2801),
    [sym_xml_decode_no_preserve] = ACTIONS(2801),
    [sym_duet_dev_type_display_device] = ACTIONS(2801),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2801),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2801),
    [sym_duet_dev_type_io_device] = ACTIONS(2801),
    [sym_duet_dev_type_relay_device] = ACTIONS(2801),
    [sym_duet_dev_type_ups] = ACTIONS(2801),
    [sym_duet_dev_type_amplifier] = ACTIONS(2801),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2801),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2801),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2801),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2801),
    [sym_duet_dev_type_camera] = ACTIONS(2801),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2801),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2801),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2801),
    [sym_duet_dev_type_dss] = ACTIONS(2801),
    [sym_duet_dev_type_dvr] = ACTIONS(2801),
    [sym_duet_dev_type_disc_device] = ACTIONS(2801),
    [sym_duet_dev_type_document_camera] = ACTIONS(2801),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2801),
    [sym_duet_dev_type_hvac] = ACTIONS(2801),
    [sym_duet_dev_type_keypad] = ACTIONS(2801),
    [sym_duet_dev_type_light] = ACTIONS(2801),
    [sym_duet_dev_type_monitor] = ACTIONS(2801),
    [sym_duet_dev_type_motor] = ACTIONS(2801),
    [sym_duet_dev_type_multi_window] = ACTIONS(2801),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2801),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2801),
    [sym_duet_dev_type_receiver] = ACTIONS(2801),
    [sym_duet_dev_type_security_system] = ACTIONS(2801),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2801),
    [sym_duet_dev_type_settop_box] = ACTIONS(2801),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2801),
    [sym_duet_dev_type_switcher] = ACTIONS(2801),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2801),
    [sym_duet_dev_type_tv] = ACTIONS(2801),
    [sym_duet_dev_type_utility] = ACTIONS(2801),
    [sym_duet_dev_type_vcr] = ACTIONS(2801),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2801),
    [sym_duet_dev_type_video_processor] = ACTIONS(2801),
    [sym_duet_dev_type_video_projector] = ACTIONS(2801),
    [sym_duet_dev_type_video_wall] = ACTIONS(2801),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2801),
    [sym_duet_dev_type_weather] = ACTIONS(2801),
    [sym_duet_dev_not_polled] = ACTIONS(2801),
    [sym_duet_dev_polled] = ACTIONS(2801),
    [sym_clkmgr_mode_network] = ACTIONS(2801),
    [sym_clkmgr_mode_standalone] = ACTIONS(2801),
    [sym_normal_standby] = ACTIONS(2801),
    [sym_normal_wake] = ACTIONS(2801),
    [sym_ip_multicast_ttl_option] = ACTIONS(2801),
    [sym_ip_ttl_subnet] = ACTIONS(2801),
    [sym_ip_ttl_site] = ACTIONS(2801),
    [sym_ip_ttl_region] = ACTIONS(2801),
    [sym_ip_ttl_continent] = ACTIONS(2801),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2801),
    [sym_ip_nodelay_on] = ACTIONS(2801),
    [sym_ip_nodelay_off] = ACTIONS(2801),
    [sym_amx_error] = ACTIONS(2801),
    [sym_amx_warning] = ACTIONS(2801),
    [sym_amx_info] = ACTIONS(2801),
    [sym_amx_debug] = ACTIONS(2801),
    [sym_netlinx_login_success] = ACTIONS(2801),
    [sym_netlinx_login_fail] = ACTIONS(2801),
    [sym_netlinx_logout] = ACTIONS(2801),
    [sym_null_str] = ACTIONS(2801),
    [sym_smtp_address] = ACTIONS(2801),
    [sym_smtp_port_number] = ACTIONS(2801),
    [sym_smtp_username] = ACTIONS(2801),
    [sym_smtp_password] = ACTIONS(2801),
    [sym_smtp_require_tls] = ACTIONS(2801),
    [sym_smtp_from] = ACTIONS(2801),
    [sym_smtp_tls_true] = ACTIONS(2801),
    [sym_smtp_tls_false] = ACTIONS(2801),
    [sym_listview_on_row_select_event] = ACTIONS(2801),
    [sym_data_structure_unknown] = ACTIONS(2801),
    [sym_data_structure_datafeed] = ACTIONS(2801),
    [sym_data_structure_datarecordset] = ACTIONS(2801),
    [sym_data_structure_datarecord] = ACTIONS(2801),
    [sym_data_structure_datafield] = ACTIONS(2801),
    [sym_data_max_content_field_count] = ACTIONS(2801),
    [sym_data_max_metadata_field_count] = ACTIONS(2801),
    [sym_data_max_id_length] = ACTIONS(2801),
    [sym_data_max_name_length] = ACTIONS(2801),
    [sym_data_max_description_length] = ACTIONS(2801),
    [sym_data_max_type_length] = ACTIONS(2801),
    [sym_data_max_format_length] = ACTIONS(2801),
    [sym_data_max_source_length] = ACTIONS(2801),
    [sym_data_max_value_length] = ACTIONS(2801),
    [sym_data_max_label_length] = ACTIONS(2801),
    [sym_data_max_url_length] = ACTIONS(2801),
    [sym_max_last_login_info_length] = ACTIONS(2801),
    [sym_data_type_unknown] = ACTIONS(2801),
    [sym_data_type_string] = ACTIONS(2801),
    [sym_data_type_datetime] = ACTIONS(2801),
    [sym_data_type_date] = ACTIONS(2801),
    [sym_data_type_time] = ACTIONS(2801),
    [sym_data_type_image] = ACTIONS(2801),
    [sym_data_format_url] = ACTIONS(2801),
    [sym_data_format_phone] = ACTIONS(2801),
    [sym_data_format_email] = ACTIONS(2801),
    [sym_data_format_iso8601] = ACTIONS(2801),
    [sym_auditlog_priv] = ACTIONS(2801),
    [sym_configuration_priv] = ACTIONS(2801),
    [sym_deviceconfig_priv] = ACTIONS(2801),
    [sym_ftp_priv] = ACTIONS(2801),
    [sym_http_priv] = ACTIONS(2801),
    [sym_networkconfig_priv] = ACTIONS(2801),
    [sym_programport_priv] = ACTIONS(2801),
    [sym_remoteui_priv] = ACTIONS(2801),
    [sym_securitycontrol_priv] = ACTIONS(2801),
    [sym_softwaremanagement_priv] = ACTIONS(2801),
    [sym_terminal_priv] = ACTIONS(2801),
    [sym_tpadmin_priv] = ACTIONS(2801),
    [sym_user1_priv] = ACTIONS(2801),
    [sym_user2_priv] = ACTIONS(2801),
    [sym_user3_priv] = ACTIONS(2801),
    [sym_user4_priv] = ACTIONS(2801),
    [sym_usermanagement_priv] = ACTIONS(2801),
    [sym_valid_account] = ACTIONS(2801),
    [sym_err_name_invalid] = ACTIONS(2801),
    [sym_err_password_invalid] = ACTIONS(2801),
    [sym_err_invalid_account] = ACTIONS(2801),
    [sym_err_no_authorization] = ACTIONS(2801),
    [sym_err_invalid_parameter] = ACTIONS(2801),
    [sym_err_user_locked_out] = ACTIONS(2801),
    [sym_err_user_account_expired] = ACTIONS(2801),
    [sym_err_general_error] = ACTIONS(2801),
    [sym_err_authentication_error] = ACTIONS(2801),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2801),
    [sym_tls_validate_certificate] = ACTIONS(2801),
    [sym____reserved___] = ACTIONS(2801),
    [sym_date] = ACTIONS(2801),
    [sym_day] = ACTIONS(2801),
    [sym_time] = ACTIONS(2801),
    [sym_ldate] = ACTIONS(2801),
    [sym_push_device] = ACTIONS(2801),
    [sym_push_channel] = ACTIONS(2801),
    [sym_push_devchan] = ACTIONS(2801),
    [sym_release_device] = ACTIONS(2801),
    [sym_release_channel] = ACTIONS(2801),
    [sym_release_devchan] = ACTIONS(2801),
    [sym_master_slot] = ACTIONS(2801),
    [sym_get_pulse_time] = ACTIONS(2801),
    [sym_get_timer] = ACTIONS(2801),
    [sym_master_sn] = ACTIONS(2801),
    [sym_system_number] = ACTIONS(2801),
    [sym_button] = ACTIONS(2801),
    [sym_dv_channel] = ACTIONS(2801),
    [sym_data] = ACTIONS(2801),
    [sym_level] = ACTIONS(2801),
    [sym_timeline] = ACTIONS(2801),
    [sym_channel] = ACTIONS(2801),
    [sym_custom] = ACTIONS(2801),
    [sym_atoi] = ACTIONS(2801),
    [sym_atol] = ACTIONS(2801),
    [sym_atof] = ACTIONS(2801),
    [sym_itoa] = ACTIONS(2801),
    [sym_itohex] = ACTIONS(2801),
    [sym_hextoi] = ACTIONS(2801),
    [sym_ftoa] = ACTIONS(2801),
    [sym_find_string] = ACTIONS(2801),
    [sym_left_string] = ACTIONS(2801),
    [sym_right_string] = ACTIONS(2801),
    [sym_mid_string] = ACTIONS(2801),
    [sym_remove_string] = ACTIONS(2801),
    [sym_get_buffer_string] = ACTIONS(2801),
    [sym_get_buffer_char] = ACTIONS(2801),
    [sym_get_multi_buffer_string] = ACTIONS(2801),
    [sym_length_string] = ACTIONS(2801),
    [sym_max_length_string] = ACTIONS(2801),
    [sym_set_length_string] = ACTIONS(2801),
    [sym_compare_string] = ACTIONS(2801),
    [sym_upper_string] = ACTIONS(2801),
    [sym_lower_string] = ACTIONS(2801),
    [sym_get_last] = ACTIONS(2801),
    [sym_date_to_day] = ACTIONS(2801),
    [sym_date_to_month] = ACTIONS(2801),
    [sym_date_to_year] = ACTIONS(2801),
    [sym_time_to_hour] = ACTIONS(2801),
    [sym_time_to_minute] = ACTIONS(2801),
    [sym_time_to_second] = ACTIONS(2801),
    [sym_day_of_week] = ACTIONS(2801),
    [sym_set_pulse_time] = ACTIONS(2801),
    [sym_set_timer] = ACTIONS(2801),
    [sym_file_open] = ACTIONS(2801),
    [sym_file_close] = ACTIONS(2801),
    [sym_file_read] = ACTIONS(2801),
    [sym_file_read_line] = ACTIONS(2801),
    [sym_file_write] = ACTIONS(2801),
    [sym_file_write_line] = ACTIONS(2801),
    [sym_file_copy] = ACTIONS(2801),
    [sym_file_delete] = ACTIONS(2801),
    [sym_file_rename] = ACTIONS(2801),
    [sym_file_dir] = ACTIONS(2801),
    [sym_file_getdir] = ACTIONS(2801),
    [sym_file_setdir] = ACTIONS(2801),
    [sym_file_create_dir] = ACTIONS(2801),
    [sym_file_remove_dir] = ACTIONS(2801),
    [sym_file_seek] = ACTIONS(2801),
    [sym_length_array] = ACTIONS(2801),
    [sym_max_length_array] = ACTIONS(2801),
    [sym_set_length_array] = ACTIONS(2801),
    [sym_ip_client_open] = ACTIONS(2801),
    [sym_ip_client_close] = ACTIONS(2801),
    [sym_ip_server_open] = ACTIONS(2801),
    [sym_ip_server_close] = ACTIONS(2801),
    [sym_ip_bound_client_open] = ACTIONS(2801),
    [sym_ip_mc_server_open] = ACTIONS(2801),
    [sym_ip_set_option] = ACTIONS(2801),
    [sym_ssh_client_open] = ACTIONS(2801),
    [sym_ssh_client_close] = ACTIONS(2801),
    [sym_tls_client_open] = ACTIONS(2801),
    [sym_tls_client_close] = ACTIONS(2801),
    [sym_do_push] = ACTIONS(2801),
    [sym_do_push_timed] = ACTIONS(2801),
    [sym_do_release] = ACTIONS(2801),
    [sym_device_id] = ACTIONS(2801),
    [sym_device_info] = ACTIONS(2801),
    [sym_device_id_string] = ACTIONS(2801),
    [sym_device_standby] = ACTIONS(2801),
    [sym_device_wake] = ACTIONS(2801),
    [sym_timeline_create] = ACTIONS(2801),
    [sym_timeline_kill] = ACTIONS(2801),
    [sym_timeline_reload] = ACTIONS(2801),
    [sym_timeline_pause] = ACTIONS(2801),
    [sym_timeline_restart] = ACTIONS(2801),
    [sym_timeline_set] = ACTIONS(2801),
    [sym_timeline_get] = ACTIONS(2801),
    [sym_timeline_active] = ACTIONS(2801),
    [sym_timeline_dynamic_id] = ACTIONS(2801),
    [sym_variable_to_string] = ACTIONS(2801),
    [sym_string_to_variable] = ACTIONS(2801),
    [sym_length_variable_to_string] = ACTIONS(2801),
    [sym_type_cast] = ACTIONS(2801),
    [sym_variable_to_xml] = ACTIONS(2801),
    [sym_xml_to_variable] = ACTIONS(2801),
    [sym_length_variable_to_xml] = ACTIONS(2801),
    [sym_redirect_string] = ACTIONS(2801),
    [sym_random_number] = ACTIONS(2801),
    [sym_get_unique_id] = ACTIONS(2801),
    [sym_get_system_number] = ACTIONS(2801),
    [sym_set_system_number] = ACTIONS(2801),
    [sym_get_serial_number] = ACTIONS(2801),
    [sym_get_master_build] = ACTIONS(2801),
    [sym_set_validation_code] = ACTIONS(2801),
    [sym_reboot] = ACTIONS(2801),
    [sym_load_duet_module] = ACTIONS(2801),
    [sym_unload_duet_module] = ACTIONS(2801),
    [sym_duet_mem_size_set] = ACTIONS(2801),
    [sym_duet_mem_size_get] = ACTIONS(2801),
    [sym_dynamic_application_device] = ACTIONS(2801),
    [sym_dynamic_polled_port] = ACTIONS(2801),
    [sym_static_port_binding] = ACTIONS(2801),
    [sym_static_ip_binding] = ACTIONS(2801),
    [sym_get_dns_list] = ACTIONS(2801),
    [sym_set_dns_list] = ACTIONS(2801),
    [sym_get_ip_address] = ACTIONS(2801),
    [sym_set_ip_address] = ACTIONS(2801),
    [sym_get_url_list] = ACTIONS(2801),
    [sym_add_url_entry] = ACTIONS(2801),
    [sym_delete_url_entry] = ACTIONS(2801),
    [sym_abs_value] = ACTIONS(2801),
    [sym_max_value] = ACTIONS(2801),
    [sym_min_value] = ACTIONS(2801),
    [sym_exp_value] = ACTIONS(2801),
    [sym_log_value] = ACTIONS(2801),
    [sym_log10_value] = ACTIONS(2801),
    [sym_power_value] = ACTIONS(2801),
    [sym_sqrt_value] = ACTIONS(2801),
    [sym_raw_be] = ACTIONS(2801),
    [sym_raw_le] = ACTIONS(2801),
    [sym_internal_queue_size_set] = ACTIONS(2801),
    [sym_internal_queue_size_get] = ACTIONS(2801),
    [sym_internal_threshold_set] = ACTIONS(2801),
    [sym_internal_threshold_get] = ACTIONS(2801),
    [sym_rebuilt_event] = ACTIONS(2801),
    [sym_do_custom_event] = ACTIONS(2801),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2801),
    [sym_clkmgr_set_clk_source] = ACTIONS(2801),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2801),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2801),
    [sym_clkmgr_get_timezone] = ACTIONS(2801),
    [sym_clkmgr_set_timezone] = ACTIONS(2801),
    [sym_clkmgr_get_resync_period] = ACTIONS(2801),
    [sym_clkmgr_set_resync_period] = ACTIONS(2801),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2801),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2801),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2801),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2801),
    [sym_clkmgr_get_timeservers] = ACTIONS(2801),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2801),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2801),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2801),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2801),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2801),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2801),
    [sym_set_log_level] = ACTIONS(2801),
    [sym_get_log_level] = ACTIONS(2801),
    [sym_amx_log] = ACTIONS(2801),
    [sym_smtp_server_config_set] = ACTIONS(2801),
    [sym_smtp_server_config_get] = ACTIONS(2801),
    [sym_smtp_send] = ACTIONS(2801),
    [sym_validate_netlinx_account] = ACTIONS(2801),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2801),
    [sym_authenticate_certificate] = ACTIONS(2801),
    [sym_audit_netlinx_session_event] = ACTIONS(2801),
    [sym_audit_netlinx_generic_event] = ACTIONS(2801),
    [sym_get_available_flash_disk_space] = ACTIONS(2801),
    [sym_get_max_flash_disk_space] = ACTIONS(2801),
    [sym_data_create_feed] = ACTIONS(2801),
    [sym_data_delete_feed] = ACTIONS(2801),
    [sym_data_publish_feed] = ACTIONS(2801),
    [sym_data_get_published_feed] = ACTIONS(2801),
    [sym_data_add_record] = ACTIONS(2801),
    [sym_data_get_event_record] = ACTIONS(2801),
    [sym__wc_data_create_feed] = ACTIONS(2801),
    [sym__wc_data_add_record] = ACTIONS(2801),
    [sym__wc_data_get_event_record] = ACTIONS(2801),
    [sym_format] = ACTIONS(2801),
    [sym_astro_clock] = ACTIONS(2801),
    [sym_set_outdoor_temperature] = ACTIONS(2801),
    [sym_set_virtual_level_count] = ACTIONS(2801),
    [sym_set_virtual_channel_count] = ACTIONS(2801),
    [sym_set_virtual_port_count] = ACTIONS(2801),
    [sym__wc_encode_int] = ACTIONS(2801),
    [sym__wc_decode_int] = ACTIONS(2801),
    [sym__wc_int] = ACTIONS(2801),
    [sym__wc_to_ch_int] = ACTIONS(2801),
    [sym__ch_to_wc_int] = ACTIONS(2801),
    [sym__wc_find_string_int] = ACTIONS(2801),
    [sym__wc_left_string_int] = ACTIONS(2801),
    [sym__wc_lower_string_int] = ACTIONS(2801),
    [sym__wc_mid_string_int] = ACTIONS(2801),
    [sym__wc_remove_string_int] = ACTIONS(2801),
    [sym__wc_right_string_int] = ACTIONS(2801),
    [sym__wc_upper_string_int] = ACTIONS(2801),
    [sym__wc_compare_string_int] = ACTIONS(2801),
    [sym__wc_get_buffer_char_int] = ACTIONS(2801),
    [sym__wc_get_buffer_string_int] = ACTIONS(2801),
    [sym__wc_concat_string_int] = ACTIONS(2801),
    [sym__wc_file_open_int] = ACTIONS(2801),
    [sym__wc_file_close_int] = ACTIONS(2801),
    [sym__wc_file_read_int] = ACTIONS(2801),
    [sym__wc_file_read_line_int] = ACTIONS(2801),
    [sym__wc_file_write_int] = ACTIONS(2801),
    [sym__wc_file_write_line_int] = ACTIONS(2801),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2803),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2803),
    [anon_sym_LBRACK] = ACTIONS(2803),
    [anon_sym_LBRACE] = ACTIONS(2803),
    [anon_sym_RBRACE] = ACTIONS(2803),
    [anon_sym_LPAREN] = ACTIONS(2801),
    [anon_sym_BANG] = ACTIONS(2803),
    [anon_sym_TILDE] = ACTIONS(2803),
    [anon_sym_DASH] = ACTIONS(2801),
    [anon_sym_PLUS] = ACTIONS(2801),
    [anon_sym_DASH_DASH] = ACTIONS(2803),
    [anon_sym_PLUS_PLUS] = ACTIONS(2803),
    [anon_sym_DQUOTE] = ACTIONS(2803),
    [anon_sym_SQUOTE] = ACTIONS(2803),
    [sym_number_literal] = ACTIONS(2803),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(936)] = {
    [sym_identifier] = ACTIONS(2805),
    [sym_not] = ACTIONS(2805),
    [sym_bnot] = ACTIONS(2805),
    [sym___file__] = ACTIONS(2805),
    [sym___date__] = ACTIONS(2805),
    [sym___ldate__] = ACTIONS(2805),
    [sym___time__] = ACTIONS(2805),
    [sym___line__] = ACTIONS(2805),
    [sym___name__] = ACTIONS(2805),
    [sym___netlinx__] = ACTIONS(2805),
    [sym_netlinx_axi_version] = ACTIONS(2805),
    [sym_url_flg_tcp] = ACTIONS(2805),
    [sym_url_flg_accinfopresent] = ACTIONS(2805),
    [sym_url_flg_temp] = ACTIONS(2805),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2805),
    [sym_url_flg_stat_mask] = ACTIONS(2805),
    [sym_url_flg_stat_lookup] = ACTIONS(2805),
    [sym_url_flg_stat_connecting] = ACTIONS(2805),
    [sym_url_flg_stat_waiting] = ACTIONS(2805),
    [sym_url_flg_stat_connected] = ACTIONS(2805),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2805),
    [sym_first_virtual_device] = ACTIONS(2805),
    [sym_first_local_port] = ACTIONS(2805),
    [sym_dynamic_virtual_device] = ACTIONS(2805),
    [sym_timeline_once] = ACTIONS(2805),
    [sym_timeline_repeat] = ACTIONS(2805),
    [sym_timeline_absolute] = ACTIONS(2805),
    [sym_timeline_relative] = ACTIONS(2805),
    [sym_do_push_timed_infinite] = ACTIONS(2805),
    [sym_source_type_no_address] = ACTIONS(2805),
    [sym_source_type_neuron_id] = ACTIONS(2805),
    [sym_source_type_ip_address] = ACTIONS(2805),
    [sym_source_type_axlink] = ACTIONS(2805),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2805),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2805),
    [sym_source_type_ip_socket_address] = ACTIONS(2805),
    [sym_source_type_rs232] = ACTIONS(2805),
    [sym_source_type_internal] = ACTIONS(2805),
    [sym_source_type_mac_address] = ACTIONS(2805),
    [sym_source_type_ipv4_port] = ACTIONS(2805),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2805),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2805),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2805),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2805),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2805),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2805),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2805),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2805),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2805),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2805),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2805),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2805),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2805),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2805),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2805),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2805),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2805),
    [sym_true] = ACTIONS(2805),
    [sym_false] = ACTIONS(2805),
    [sym_file_read_only] = ACTIONS(2805),
    [sym_file_rw_new] = ACTIONS(2805),
    [sym_file_rw_append] = ACTIONS(2805),
    [sym_ip_tcp] = ACTIONS(2805),
    [sym_ip_udp] = ACTIONS(2805),
    [sym_ip_udp_2way] = ACTIONS(2805),
    [sym_xml_encode_types] = ACTIONS(2805),
    [sym_xml_encode_char_as_list] = ACTIONS(2805),
    [sym_xml_encode_le] = ACTIONS(2805),
    [sym_xml_decode_types] = ACTIONS(2805),
    [sym_xml_decode_no_preserve] = ACTIONS(2805),
    [sym_duet_dev_type_display_device] = ACTIONS(2805),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2805),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2805),
    [sym_duet_dev_type_io_device] = ACTIONS(2805),
    [sym_duet_dev_type_relay_device] = ACTIONS(2805),
    [sym_duet_dev_type_ups] = ACTIONS(2805),
    [sym_duet_dev_type_amplifier] = ACTIONS(2805),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2805),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2805),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2805),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2805),
    [sym_duet_dev_type_camera] = ACTIONS(2805),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2805),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2805),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2805),
    [sym_duet_dev_type_dss] = ACTIONS(2805),
    [sym_duet_dev_type_dvr] = ACTIONS(2805),
    [sym_duet_dev_type_disc_device] = ACTIONS(2805),
    [sym_duet_dev_type_document_camera] = ACTIONS(2805),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2805),
    [sym_duet_dev_type_hvac] = ACTIONS(2805),
    [sym_duet_dev_type_keypad] = ACTIONS(2805),
    [sym_duet_dev_type_light] = ACTIONS(2805),
    [sym_duet_dev_type_monitor] = ACTIONS(2805),
    [sym_duet_dev_type_motor] = ACTIONS(2805),
    [sym_duet_dev_type_multi_window] = ACTIONS(2805),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2805),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2805),
    [sym_duet_dev_type_receiver] = ACTIONS(2805),
    [sym_duet_dev_type_security_system] = ACTIONS(2805),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2805),
    [sym_duet_dev_type_settop_box] = ACTIONS(2805),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2805),
    [sym_duet_dev_type_switcher] = ACTIONS(2805),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2805),
    [sym_duet_dev_type_tv] = ACTIONS(2805),
    [sym_duet_dev_type_utility] = ACTIONS(2805),
    [sym_duet_dev_type_vcr] = ACTIONS(2805),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2805),
    [sym_duet_dev_type_video_processor] = ACTIONS(2805),
    [sym_duet_dev_type_video_projector] = ACTIONS(2805),
    [sym_duet_dev_type_video_wall] = ACTIONS(2805),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2805),
    [sym_duet_dev_type_weather] = ACTIONS(2805),
    [sym_duet_dev_not_polled] = ACTIONS(2805),
    [sym_duet_dev_polled] = ACTIONS(2805),
    [sym_clkmgr_mode_network] = ACTIONS(2805),
    [sym_clkmgr_mode_standalone] = ACTIONS(2805),
    [sym_normal_standby] = ACTIONS(2805),
    [sym_normal_wake] = ACTIONS(2805),
    [sym_ip_multicast_ttl_option] = ACTIONS(2805),
    [sym_ip_ttl_subnet] = ACTIONS(2805),
    [sym_ip_ttl_site] = ACTIONS(2805),
    [sym_ip_ttl_region] = ACTIONS(2805),
    [sym_ip_ttl_continent] = ACTIONS(2805),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2805),
    [sym_ip_nodelay_on] = ACTIONS(2805),
    [sym_ip_nodelay_off] = ACTIONS(2805),
    [sym_amx_error] = ACTIONS(2805),
    [sym_amx_warning] = ACTIONS(2805),
    [sym_amx_info] = ACTIONS(2805),
    [sym_amx_debug] = ACTIONS(2805),
    [sym_netlinx_login_success] = ACTIONS(2805),
    [sym_netlinx_login_fail] = ACTIONS(2805),
    [sym_netlinx_logout] = ACTIONS(2805),
    [sym_null_str] = ACTIONS(2805),
    [sym_smtp_address] = ACTIONS(2805),
    [sym_smtp_port_number] = ACTIONS(2805),
    [sym_smtp_username] = ACTIONS(2805),
    [sym_smtp_password] = ACTIONS(2805),
    [sym_smtp_require_tls] = ACTIONS(2805),
    [sym_smtp_from] = ACTIONS(2805),
    [sym_smtp_tls_true] = ACTIONS(2805),
    [sym_smtp_tls_false] = ACTIONS(2805),
    [sym_listview_on_row_select_event] = ACTIONS(2805),
    [sym_data_structure_unknown] = ACTIONS(2805),
    [sym_data_structure_datafeed] = ACTIONS(2805),
    [sym_data_structure_datarecordset] = ACTIONS(2805),
    [sym_data_structure_datarecord] = ACTIONS(2805),
    [sym_data_structure_datafield] = ACTIONS(2805),
    [sym_data_max_content_field_count] = ACTIONS(2805),
    [sym_data_max_metadata_field_count] = ACTIONS(2805),
    [sym_data_max_id_length] = ACTIONS(2805),
    [sym_data_max_name_length] = ACTIONS(2805),
    [sym_data_max_description_length] = ACTIONS(2805),
    [sym_data_max_type_length] = ACTIONS(2805),
    [sym_data_max_format_length] = ACTIONS(2805),
    [sym_data_max_source_length] = ACTIONS(2805),
    [sym_data_max_value_length] = ACTIONS(2805),
    [sym_data_max_label_length] = ACTIONS(2805),
    [sym_data_max_url_length] = ACTIONS(2805),
    [sym_max_last_login_info_length] = ACTIONS(2805),
    [sym_data_type_unknown] = ACTIONS(2805),
    [sym_data_type_string] = ACTIONS(2805),
    [sym_data_type_datetime] = ACTIONS(2805),
    [sym_data_type_date] = ACTIONS(2805),
    [sym_data_type_time] = ACTIONS(2805),
    [sym_data_type_image] = ACTIONS(2805),
    [sym_data_format_url] = ACTIONS(2805),
    [sym_data_format_phone] = ACTIONS(2805),
    [sym_data_format_email] = ACTIONS(2805),
    [sym_data_format_iso8601] = ACTIONS(2805),
    [sym_auditlog_priv] = ACTIONS(2805),
    [sym_configuration_priv] = ACTIONS(2805),
    [sym_deviceconfig_priv] = ACTIONS(2805),
    [sym_ftp_priv] = ACTIONS(2805),
    [sym_http_priv] = ACTIONS(2805),
    [sym_networkconfig_priv] = ACTIONS(2805),
    [sym_programport_priv] = ACTIONS(2805),
    [sym_remoteui_priv] = ACTIONS(2805),
    [sym_securitycontrol_priv] = ACTIONS(2805),
    [sym_softwaremanagement_priv] = ACTIONS(2805),
    [sym_terminal_priv] = ACTIONS(2805),
    [sym_tpadmin_priv] = ACTIONS(2805),
    [sym_user1_priv] = ACTIONS(2805),
    [sym_user2_priv] = ACTIONS(2805),
    [sym_user3_priv] = ACTIONS(2805),
    [sym_user4_priv] = ACTIONS(2805),
    [sym_usermanagement_priv] = ACTIONS(2805),
    [sym_valid_account] = ACTIONS(2805),
    [sym_err_name_invalid] = ACTIONS(2805),
    [sym_err_password_invalid] = ACTIONS(2805),
    [sym_err_invalid_account] = ACTIONS(2805),
    [sym_err_no_authorization] = ACTIONS(2805),
    [sym_err_invalid_parameter] = ACTIONS(2805),
    [sym_err_user_locked_out] = ACTIONS(2805),
    [sym_err_user_account_expired] = ACTIONS(2805),
    [sym_err_general_error] = ACTIONS(2805),
    [sym_err_authentication_error] = ACTIONS(2805),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2805),
    [sym_tls_validate_certificate] = ACTIONS(2805),
    [sym____reserved___] = ACTIONS(2805),
    [sym_date] = ACTIONS(2805),
    [sym_day] = ACTIONS(2805),
    [sym_time] = ACTIONS(2805),
    [sym_ldate] = ACTIONS(2805),
    [sym_push_device] = ACTIONS(2805),
    [sym_push_channel] = ACTIONS(2805),
    [sym_push_devchan] = ACTIONS(2805),
    [sym_release_device] = ACTIONS(2805),
    [sym_release_channel] = ACTIONS(2805),
    [sym_release_devchan] = ACTIONS(2805),
    [sym_master_slot] = ACTIONS(2805),
    [sym_get_pulse_time] = ACTIONS(2805),
    [sym_get_timer] = ACTIONS(2805),
    [sym_master_sn] = ACTIONS(2805),
    [sym_system_number] = ACTIONS(2805),
    [sym_button] = ACTIONS(2805),
    [sym_dv_channel] = ACTIONS(2805),
    [sym_data] = ACTIONS(2805),
    [sym_level] = ACTIONS(2805),
    [sym_timeline] = ACTIONS(2805),
    [sym_channel] = ACTIONS(2805),
    [sym_custom] = ACTIONS(2805),
    [sym_atoi] = ACTIONS(2805),
    [sym_atol] = ACTIONS(2805),
    [sym_atof] = ACTIONS(2805),
    [sym_itoa] = ACTIONS(2805),
    [sym_itohex] = ACTIONS(2805),
    [sym_hextoi] = ACTIONS(2805),
    [sym_ftoa] = ACTIONS(2805),
    [sym_find_string] = ACTIONS(2805),
    [sym_left_string] = ACTIONS(2805),
    [sym_right_string] = ACTIONS(2805),
    [sym_mid_string] = ACTIONS(2805),
    [sym_remove_string] = ACTIONS(2805),
    [sym_get_buffer_string] = ACTIONS(2805),
    [sym_get_buffer_char] = ACTIONS(2805),
    [sym_get_multi_buffer_string] = ACTIONS(2805),
    [sym_length_string] = ACTIONS(2805),
    [sym_max_length_string] = ACTIONS(2805),
    [sym_set_length_string] = ACTIONS(2805),
    [sym_compare_string] = ACTIONS(2805),
    [sym_upper_string] = ACTIONS(2805),
    [sym_lower_string] = ACTIONS(2805),
    [sym_get_last] = ACTIONS(2805),
    [sym_date_to_day] = ACTIONS(2805),
    [sym_date_to_month] = ACTIONS(2805),
    [sym_date_to_year] = ACTIONS(2805),
    [sym_time_to_hour] = ACTIONS(2805),
    [sym_time_to_minute] = ACTIONS(2805),
    [sym_time_to_second] = ACTIONS(2805),
    [sym_day_of_week] = ACTIONS(2805),
    [sym_set_pulse_time] = ACTIONS(2805),
    [sym_set_timer] = ACTIONS(2805),
    [sym_file_open] = ACTIONS(2805),
    [sym_file_close] = ACTIONS(2805),
    [sym_file_read] = ACTIONS(2805),
    [sym_file_read_line] = ACTIONS(2805),
    [sym_file_write] = ACTIONS(2805),
    [sym_file_write_line] = ACTIONS(2805),
    [sym_file_copy] = ACTIONS(2805),
    [sym_file_delete] = ACTIONS(2805),
    [sym_file_rename] = ACTIONS(2805),
    [sym_file_dir] = ACTIONS(2805),
    [sym_file_getdir] = ACTIONS(2805),
    [sym_file_setdir] = ACTIONS(2805),
    [sym_file_create_dir] = ACTIONS(2805),
    [sym_file_remove_dir] = ACTIONS(2805),
    [sym_file_seek] = ACTIONS(2805),
    [sym_length_array] = ACTIONS(2805),
    [sym_max_length_array] = ACTIONS(2805),
    [sym_set_length_array] = ACTIONS(2805),
    [sym_ip_client_open] = ACTIONS(2805),
    [sym_ip_client_close] = ACTIONS(2805),
    [sym_ip_server_open] = ACTIONS(2805),
    [sym_ip_server_close] = ACTIONS(2805),
    [sym_ip_bound_client_open] = ACTIONS(2805),
    [sym_ip_mc_server_open] = ACTIONS(2805),
    [sym_ip_set_option] = ACTIONS(2805),
    [sym_ssh_client_open] = ACTIONS(2805),
    [sym_ssh_client_close] = ACTIONS(2805),
    [sym_tls_client_open] = ACTIONS(2805),
    [sym_tls_client_close] = ACTIONS(2805),
    [sym_do_push] = ACTIONS(2805),
    [sym_do_push_timed] = ACTIONS(2805),
    [sym_do_release] = ACTIONS(2805),
    [sym_device_id] = ACTIONS(2805),
    [sym_device_info] = ACTIONS(2805),
    [sym_device_id_string] = ACTIONS(2805),
    [sym_device_standby] = ACTIONS(2805),
    [sym_device_wake] = ACTIONS(2805),
    [sym_timeline_create] = ACTIONS(2805),
    [sym_timeline_kill] = ACTIONS(2805),
    [sym_timeline_reload] = ACTIONS(2805),
    [sym_timeline_pause] = ACTIONS(2805),
    [sym_timeline_restart] = ACTIONS(2805),
    [sym_timeline_set] = ACTIONS(2805),
    [sym_timeline_get] = ACTIONS(2805),
    [sym_timeline_active] = ACTIONS(2805),
    [sym_timeline_dynamic_id] = ACTIONS(2805),
    [sym_variable_to_string] = ACTIONS(2805),
    [sym_string_to_variable] = ACTIONS(2805),
    [sym_length_variable_to_string] = ACTIONS(2805),
    [sym_type_cast] = ACTIONS(2805),
    [sym_variable_to_xml] = ACTIONS(2805),
    [sym_xml_to_variable] = ACTIONS(2805),
    [sym_length_variable_to_xml] = ACTIONS(2805),
    [sym_redirect_string] = ACTIONS(2805),
    [sym_random_number] = ACTIONS(2805),
    [sym_get_unique_id] = ACTIONS(2805),
    [sym_get_system_number] = ACTIONS(2805),
    [sym_set_system_number] = ACTIONS(2805),
    [sym_get_serial_number] = ACTIONS(2805),
    [sym_get_master_build] = ACTIONS(2805),
    [sym_set_validation_code] = ACTIONS(2805),
    [sym_reboot] = ACTIONS(2805),
    [sym_load_duet_module] = ACTIONS(2805),
    [sym_unload_duet_module] = ACTIONS(2805),
    [sym_duet_mem_size_set] = ACTIONS(2805),
    [sym_duet_mem_size_get] = ACTIONS(2805),
    [sym_dynamic_application_device] = ACTIONS(2805),
    [sym_dynamic_polled_port] = ACTIONS(2805),
    [sym_static_port_binding] = ACTIONS(2805),
    [sym_static_ip_binding] = ACTIONS(2805),
    [sym_get_dns_list] = ACTIONS(2805),
    [sym_set_dns_list] = ACTIONS(2805),
    [sym_get_ip_address] = ACTIONS(2805),
    [sym_set_ip_address] = ACTIONS(2805),
    [sym_get_url_list] = ACTIONS(2805),
    [sym_add_url_entry] = ACTIONS(2805),
    [sym_delete_url_entry] = ACTIONS(2805),
    [sym_abs_value] = ACTIONS(2805),
    [sym_max_value] = ACTIONS(2805),
    [sym_min_value] = ACTIONS(2805),
    [sym_exp_value] = ACTIONS(2805),
    [sym_log_value] = ACTIONS(2805),
    [sym_log10_value] = ACTIONS(2805),
    [sym_power_value] = ACTIONS(2805),
    [sym_sqrt_value] = ACTIONS(2805),
    [sym_raw_be] = ACTIONS(2805),
    [sym_raw_le] = ACTIONS(2805),
    [sym_internal_queue_size_set] = ACTIONS(2805),
    [sym_internal_queue_size_get] = ACTIONS(2805),
    [sym_internal_threshold_set] = ACTIONS(2805),
    [sym_internal_threshold_get] = ACTIONS(2805),
    [sym_rebuilt_event] = ACTIONS(2805),
    [sym_do_custom_event] = ACTIONS(2805),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2805),
    [sym_clkmgr_set_clk_source] = ACTIONS(2805),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2805),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2805),
    [sym_clkmgr_get_timezone] = ACTIONS(2805),
    [sym_clkmgr_set_timezone] = ACTIONS(2805),
    [sym_clkmgr_get_resync_period] = ACTIONS(2805),
    [sym_clkmgr_set_resync_period] = ACTIONS(2805),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2805),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2805),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2805),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2805),
    [sym_clkmgr_get_timeservers] = ACTIONS(2805),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2805),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2805),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2805),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2805),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2805),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2805),
    [sym_set_log_level] = ACTIONS(2805),
    [sym_get_log_level] = ACTIONS(2805),
    [sym_amx_log] = ACTIONS(2805),
    [sym_smtp_server_config_set] = ACTIONS(2805),
    [sym_smtp_server_config_get] = ACTIONS(2805),
    [sym_smtp_send] = ACTIONS(2805),
    [sym_validate_netlinx_account] = ACTIONS(2805),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2805),
    [sym_authenticate_certificate] = ACTIONS(2805),
    [sym_audit_netlinx_session_event] = ACTIONS(2805),
    [sym_audit_netlinx_generic_event] = ACTIONS(2805),
    [sym_get_available_flash_disk_space] = ACTIONS(2805),
    [sym_get_max_flash_disk_space] = ACTIONS(2805),
    [sym_data_create_feed] = ACTIONS(2805),
    [sym_data_delete_feed] = ACTIONS(2805),
    [sym_data_publish_feed] = ACTIONS(2805),
    [sym_data_get_published_feed] = ACTIONS(2805),
    [sym_data_add_record] = ACTIONS(2805),
    [sym_data_get_event_record] = ACTIONS(2805),
    [sym__wc_data_create_feed] = ACTIONS(2805),
    [sym__wc_data_add_record] = ACTIONS(2805),
    [sym__wc_data_get_event_record] = ACTIONS(2805),
    [sym_format] = ACTIONS(2805),
    [sym_astro_clock] = ACTIONS(2805),
    [sym_set_outdoor_temperature] = ACTIONS(2805),
    [sym_set_virtual_level_count] = ACTIONS(2805),
    [sym_set_virtual_channel_count] = ACTIONS(2805),
    [sym_set_virtual_port_count] = ACTIONS(2805),
    [sym__wc_encode_int] = ACTIONS(2805),
    [sym__wc_decode_int] = ACTIONS(2805),
    [sym__wc_int] = ACTIONS(2805),
    [sym__wc_to_ch_int] = ACTIONS(2805),
    [sym__ch_to_wc_int] = ACTIONS(2805),
    [sym__wc_find_string_int] = ACTIONS(2805),
    [sym__wc_left_string_int] = ACTIONS(2805),
    [sym__wc_lower_string_int] = ACTIONS(2805),
    [sym__wc_mid_string_int] = ACTIONS(2805),
    [sym__wc_remove_string_int] = ACTIONS(2805),
    [sym__wc_right_string_int] = ACTIONS(2805),
    [sym__wc_upper_string_int] = ACTIONS(2805),
    [sym__wc_compare_string_int] = ACTIONS(2805),
    [sym__wc_get_buffer_char_int] = ACTIONS(2805),
    [sym__wc_get_buffer_string_int] = ACTIONS(2805),
    [sym__wc_concat_string_int] = ACTIONS(2805),
    [sym__wc_file_open_int] = ACTIONS(2805),
    [sym__wc_file_close_int] = ACTIONS(2805),
    [sym__wc_file_read_int] = ACTIONS(2805),
    [sym__wc_file_read_line_int] = ACTIONS(2805),
    [sym__wc_file_write_int] = ACTIONS(2805),
    [sym__wc_file_write_line_int] = ACTIONS(2805),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2808),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2808),
    [anon_sym_LBRACK] = ACTIONS(2808),
    [anon_sym_LBRACE] = ACTIONS(2808),
    [anon_sym_RBRACE] = ACTIONS(2808),
    [anon_sym_LPAREN] = ACTIONS(2805),
    [anon_sym_BANG] = ACTIONS(2808),
    [anon_sym_TILDE] = ACTIONS(2808),
    [anon_sym_DASH] = ACTIONS(2805),
    [anon_sym_PLUS] = ACTIONS(2805),
    [anon_sym_DASH_DASH] = ACTIONS(2808),
    [anon_sym_PLUS_PLUS] = ACTIONS(2808),
    [anon_sym_DQUOTE] = ACTIONS(2808),
    [anon_sym_SQUOTE] = ACTIONS(2808),
    [sym_number_literal] = ACTIONS(2808),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(937)] = {
    [sym_identifier] = ACTIONS(2811),
    [sym_not] = ACTIONS(2811),
    [sym_bnot] = ACTIONS(2811),
    [sym___file__] = ACTIONS(2811),
    [sym___date__] = ACTIONS(2811),
    [sym___ldate__] = ACTIONS(2811),
    [sym___time__] = ACTIONS(2811),
    [sym___line__] = ACTIONS(2811),
    [sym___name__] = ACTIONS(2811),
    [sym___netlinx__] = ACTIONS(2811),
    [sym_netlinx_axi_version] = ACTIONS(2811),
    [sym_url_flg_tcp] = ACTIONS(2811),
    [sym_url_flg_accinfopresent] = ACTIONS(2811),
    [sym_url_flg_temp] = ACTIONS(2811),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2811),
    [sym_url_flg_stat_mask] = ACTIONS(2811),
    [sym_url_flg_stat_lookup] = ACTIONS(2811),
    [sym_url_flg_stat_connecting] = ACTIONS(2811),
    [sym_url_flg_stat_waiting] = ACTIONS(2811),
    [sym_url_flg_stat_connected] = ACTIONS(2811),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2811),
    [sym_first_virtual_device] = ACTIONS(2811),
    [sym_first_local_port] = ACTIONS(2811),
    [sym_dynamic_virtual_device] = ACTIONS(2811),
    [sym_timeline_once] = ACTIONS(2811),
    [sym_timeline_repeat] = ACTIONS(2811),
    [sym_timeline_absolute] = ACTIONS(2811),
    [sym_timeline_relative] = ACTIONS(2811),
    [sym_do_push_timed_infinite] = ACTIONS(2811),
    [sym_source_type_no_address] = ACTIONS(2811),
    [sym_source_type_neuron_id] = ACTIONS(2811),
    [sym_source_type_ip_address] = ACTIONS(2811),
    [sym_source_type_axlink] = ACTIONS(2811),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2811),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2811),
    [sym_source_type_ip_socket_address] = ACTIONS(2811),
    [sym_source_type_rs232] = ACTIONS(2811),
    [sym_source_type_internal] = ACTIONS(2811),
    [sym_source_type_mac_address] = ACTIONS(2811),
    [sym_source_type_ipv4_port] = ACTIONS(2811),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2811),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2811),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2811),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2811),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2811),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2811),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2811),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2811),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2811),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2811),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2811),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2811),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2811),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2811),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2811),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2811),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2811),
    [sym_true] = ACTIONS(2811),
    [sym_false] = ACTIONS(2811),
    [sym_file_read_only] = ACTIONS(2811),
    [sym_file_rw_new] = ACTIONS(2811),
    [sym_file_rw_append] = ACTIONS(2811),
    [sym_ip_tcp] = ACTIONS(2811),
    [sym_ip_udp] = ACTIONS(2811),
    [sym_ip_udp_2way] = ACTIONS(2811),
    [sym_xml_encode_types] = ACTIONS(2811),
    [sym_xml_encode_char_as_list] = ACTIONS(2811),
    [sym_xml_encode_le] = ACTIONS(2811),
    [sym_xml_decode_types] = ACTIONS(2811),
    [sym_xml_decode_no_preserve] = ACTIONS(2811),
    [sym_duet_dev_type_display_device] = ACTIONS(2811),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2811),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2811),
    [sym_duet_dev_type_io_device] = ACTIONS(2811),
    [sym_duet_dev_type_relay_device] = ACTIONS(2811),
    [sym_duet_dev_type_ups] = ACTIONS(2811),
    [sym_duet_dev_type_amplifier] = ACTIONS(2811),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2811),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2811),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2811),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2811),
    [sym_duet_dev_type_camera] = ACTIONS(2811),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2811),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2811),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2811),
    [sym_duet_dev_type_dss] = ACTIONS(2811),
    [sym_duet_dev_type_dvr] = ACTIONS(2811),
    [sym_duet_dev_type_disc_device] = ACTIONS(2811),
    [sym_duet_dev_type_document_camera] = ACTIONS(2811),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2811),
    [sym_duet_dev_type_hvac] = ACTIONS(2811),
    [sym_duet_dev_type_keypad] = ACTIONS(2811),
    [sym_duet_dev_type_light] = ACTIONS(2811),
    [sym_duet_dev_type_monitor] = ACTIONS(2811),
    [sym_duet_dev_type_motor] = ACTIONS(2811),
    [sym_duet_dev_type_multi_window] = ACTIONS(2811),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2811),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2811),
    [sym_duet_dev_type_receiver] = ACTIONS(2811),
    [sym_duet_dev_type_security_system] = ACTIONS(2811),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2811),
    [sym_duet_dev_type_settop_box] = ACTIONS(2811),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2811),
    [sym_duet_dev_type_switcher] = ACTIONS(2811),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2811),
    [sym_duet_dev_type_tv] = ACTIONS(2811),
    [sym_duet_dev_type_utility] = ACTIONS(2811),
    [sym_duet_dev_type_vcr] = ACTIONS(2811),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2811),
    [sym_duet_dev_type_video_processor] = ACTIONS(2811),
    [sym_duet_dev_type_video_projector] = ACTIONS(2811),
    [sym_duet_dev_type_video_wall] = ACTIONS(2811),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2811),
    [sym_duet_dev_type_weather] = ACTIONS(2811),
    [sym_duet_dev_not_polled] = ACTIONS(2811),
    [sym_duet_dev_polled] = ACTIONS(2811),
    [sym_clkmgr_mode_network] = ACTIONS(2811),
    [sym_clkmgr_mode_standalone] = ACTIONS(2811),
    [sym_normal_standby] = ACTIONS(2811),
    [sym_normal_wake] = ACTIONS(2811),
    [sym_ip_multicast_ttl_option] = ACTIONS(2811),
    [sym_ip_ttl_subnet] = ACTIONS(2811),
    [sym_ip_ttl_site] = ACTIONS(2811),
    [sym_ip_ttl_region] = ACTIONS(2811),
    [sym_ip_ttl_continent] = ACTIONS(2811),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2811),
    [sym_ip_nodelay_on] = ACTIONS(2811),
    [sym_ip_nodelay_off] = ACTIONS(2811),
    [sym_amx_error] = ACTIONS(2811),
    [sym_amx_warning] = ACTIONS(2811),
    [sym_amx_info] = ACTIONS(2811),
    [sym_amx_debug] = ACTIONS(2811),
    [sym_netlinx_login_success] = ACTIONS(2811),
    [sym_netlinx_login_fail] = ACTIONS(2811),
    [sym_netlinx_logout] = ACTIONS(2811),
    [sym_null_str] = ACTIONS(2811),
    [sym_smtp_address] = ACTIONS(2811),
    [sym_smtp_port_number] = ACTIONS(2811),
    [sym_smtp_username] = ACTIONS(2811),
    [sym_smtp_password] = ACTIONS(2811),
    [sym_smtp_require_tls] = ACTIONS(2811),
    [sym_smtp_from] = ACTIONS(2811),
    [sym_smtp_tls_true] = ACTIONS(2811),
    [sym_smtp_tls_false] = ACTIONS(2811),
    [sym_listview_on_row_select_event] = ACTIONS(2811),
    [sym_data_structure_unknown] = ACTIONS(2811),
    [sym_data_structure_datafeed] = ACTIONS(2811),
    [sym_data_structure_datarecordset] = ACTIONS(2811),
    [sym_data_structure_datarecord] = ACTIONS(2811),
    [sym_data_structure_datafield] = ACTIONS(2811),
    [sym_data_max_content_field_count] = ACTIONS(2811),
    [sym_data_max_metadata_field_count] = ACTIONS(2811),
    [sym_data_max_id_length] = ACTIONS(2811),
    [sym_data_max_name_length] = ACTIONS(2811),
    [sym_data_max_description_length] = ACTIONS(2811),
    [sym_data_max_type_length] = ACTIONS(2811),
    [sym_data_max_format_length] = ACTIONS(2811),
    [sym_data_max_source_length] = ACTIONS(2811),
    [sym_data_max_value_length] = ACTIONS(2811),
    [sym_data_max_label_length] = ACTIONS(2811),
    [sym_data_max_url_length] = ACTIONS(2811),
    [sym_max_last_login_info_length] = ACTIONS(2811),
    [sym_data_type_unknown] = ACTIONS(2811),
    [sym_data_type_string] = ACTIONS(2811),
    [sym_data_type_datetime] = ACTIONS(2811),
    [sym_data_type_date] = ACTIONS(2811),
    [sym_data_type_time] = ACTIONS(2811),
    [sym_data_type_image] = ACTIONS(2811),
    [sym_data_format_url] = ACTIONS(2811),
    [sym_data_format_phone] = ACTIONS(2811),
    [sym_data_format_email] = ACTIONS(2811),
    [sym_data_format_iso8601] = ACTIONS(2811),
    [sym_auditlog_priv] = ACTIONS(2811),
    [sym_configuration_priv] = ACTIONS(2811),
    [sym_deviceconfig_priv] = ACTIONS(2811),
    [sym_ftp_priv] = ACTIONS(2811),
    [sym_http_priv] = ACTIONS(2811),
    [sym_networkconfig_priv] = ACTIONS(2811),
    [sym_programport_priv] = ACTIONS(2811),
    [sym_remoteui_priv] = ACTIONS(2811),
    [sym_securitycontrol_priv] = ACTIONS(2811),
    [sym_softwaremanagement_priv] = ACTIONS(2811),
    [sym_terminal_priv] = ACTIONS(2811),
    [sym_tpadmin_priv] = ACTIONS(2811),
    [sym_user1_priv] = ACTIONS(2811),
    [sym_user2_priv] = ACTIONS(2811),
    [sym_user3_priv] = ACTIONS(2811),
    [sym_user4_priv] = ACTIONS(2811),
    [sym_usermanagement_priv] = ACTIONS(2811),
    [sym_valid_account] = ACTIONS(2811),
    [sym_err_name_invalid] = ACTIONS(2811),
    [sym_err_password_invalid] = ACTIONS(2811),
    [sym_err_invalid_account] = ACTIONS(2811),
    [sym_err_no_authorization] = ACTIONS(2811),
    [sym_err_invalid_parameter] = ACTIONS(2811),
    [sym_err_user_locked_out] = ACTIONS(2811),
    [sym_err_user_account_expired] = ACTIONS(2811),
    [sym_err_general_error] = ACTIONS(2811),
    [sym_err_authentication_error] = ACTIONS(2811),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2811),
    [sym_tls_validate_certificate] = ACTIONS(2811),
    [sym____reserved___] = ACTIONS(2811),
    [sym_date] = ACTIONS(2811),
    [sym_day] = ACTIONS(2811),
    [sym_time] = ACTIONS(2811),
    [sym_ldate] = ACTIONS(2811),
    [sym_push_device] = ACTIONS(2811),
    [sym_push_channel] = ACTIONS(2811),
    [sym_push_devchan] = ACTIONS(2811),
    [sym_release_device] = ACTIONS(2811),
    [sym_release_channel] = ACTIONS(2811),
    [sym_release_devchan] = ACTIONS(2811),
    [sym_master_slot] = ACTIONS(2811),
    [sym_get_pulse_time] = ACTIONS(2811),
    [sym_get_timer] = ACTIONS(2811),
    [sym_master_sn] = ACTIONS(2811),
    [sym_system_number] = ACTIONS(2811),
    [sym_button] = ACTIONS(2811),
    [sym_dv_channel] = ACTIONS(2811),
    [sym_data] = ACTIONS(2811),
    [sym_level] = ACTIONS(2811),
    [sym_timeline] = ACTIONS(2811),
    [sym_channel] = ACTIONS(2811),
    [sym_custom] = ACTIONS(2811),
    [sym_atoi] = ACTIONS(2811),
    [sym_atol] = ACTIONS(2811),
    [sym_atof] = ACTIONS(2811),
    [sym_itoa] = ACTIONS(2811),
    [sym_itohex] = ACTIONS(2811),
    [sym_hextoi] = ACTIONS(2811),
    [sym_ftoa] = ACTIONS(2811),
    [sym_find_string] = ACTIONS(2811),
    [sym_left_string] = ACTIONS(2811),
    [sym_right_string] = ACTIONS(2811),
    [sym_mid_string] = ACTIONS(2811),
    [sym_remove_string] = ACTIONS(2811),
    [sym_get_buffer_string] = ACTIONS(2811),
    [sym_get_buffer_char] = ACTIONS(2811),
    [sym_get_multi_buffer_string] = ACTIONS(2811),
    [sym_length_string] = ACTIONS(2811),
    [sym_max_length_string] = ACTIONS(2811),
    [sym_set_length_string] = ACTIONS(2811),
    [sym_compare_string] = ACTIONS(2811),
    [sym_upper_string] = ACTIONS(2811),
    [sym_lower_string] = ACTIONS(2811),
    [sym_get_last] = ACTIONS(2811),
    [sym_date_to_day] = ACTIONS(2811),
    [sym_date_to_month] = ACTIONS(2811),
    [sym_date_to_year] = ACTIONS(2811),
    [sym_time_to_hour] = ACTIONS(2811),
    [sym_time_to_minute] = ACTIONS(2811),
    [sym_time_to_second] = ACTIONS(2811),
    [sym_day_of_week] = ACTIONS(2811),
    [sym_set_pulse_time] = ACTIONS(2811),
    [sym_set_timer] = ACTIONS(2811),
    [sym_file_open] = ACTIONS(2811),
    [sym_file_close] = ACTIONS(2811),
    [sym_file_read] = ACTIONS(2811),
    [sym_file_read_line] = ACTIONS(2811),
    [sym_file_write] = ACTIONS(2811),
    [sym_file_write_line] = ACTIONS(2811),
    [sym_file_copy] = ACTIONS(2811),
    [sym_file_delete] = ACTIONS(2811),
    [sym_file_rename] = ACTIONS(2811),
    [sym_file_dir] = ACTIONS(2811),
    [sym_file_getdir] = ACTIONS(2811),
    [sym_file_setdir] = ACTIONS(2811),
    [sym_file_create_dir] = ACTIONS(2811),
    [sym_file_remove_dir] = ACTIONS(2811),
    [sym_file_seek] = ACTIONS(2811),
    [sym_length_array] = ACTIONS(2811),
    [sym_max_length_array] = ACTIONS(2811),
    [sym_set_length_array] = ACTIONS(2811),
    [sym_ip_client_open] = ACTIONS(2811),
    [sym_ip_client_close] = ACTIONS(2811),
    [sym_ip_server_open] = ACTIONS(2811),
    [sym_ip_server_close] = ACTIONS(2811),
    [sym_ip_bound_client_open] = ACTIONS(2811),
    [sym_ip_mc_server_open] = ACTIONS(2811),
    [sym_ip_set_option] = ACTIONS(2811),
    [sym_ssh_client_open] = ACTIONS(2811),
    [sym_ssh_client_close] = ACTIONS(2811),
    [sym_tls_client_open] = ACTIONS(2811),
    [sym_tls_client_close] = ACTIONS(2811),
    [sym_do_push] = ACTIONS(2811),
    [sym_do_push_timed] = ACTIONS(2811),
    [sym_do_release] = ACTIONS(2811),
    [sym_device_id] = ACTIONS(2811),
    [sym_device_info] = ACTIONS(2811),
    [sym_device_id_string] = ACTIONS(2811),
    [sym_device_standby] = ACTIONS(2811),
    [sym_device_wake] = ACTIONS(2811),
    [sym_timeline_create] = ACTIONS(2811),
    [sym_timeline_kill] = ACTIONS(2811),
    [sym_timeline_reload] = ACTIONS(2811),
    [sym_timeline_pause] = ACTIONS(2811),
    [sym_timeline_restart] = ACTIONS(2811),
    [sym_timeline_set] = ACTIONS(2811),
    [sym_timeline_get] = ACTIONS(2811),
    [sym_timeline_active] = ACTIONS(2811),
    [sym_timeline_dynamic_id] = ACTIONS(2811),
    [sym_variable_to_string] = ACTIONS(2811),
    [sym_string_to_variable] = ACTIONS(2811),
    [sym_length_variable_to_string] = ACTIONS(2811),
    [sym_type_cast] = ACTIONS(2811),
    [sym_variable_to_xml] = ACTIONS(2811),
    [sym_xml_to_variable] = ACTIONS(2811),
    [sym_length_variable_to_xml] = ACTIONS(2811),
    [sym_redirect_string] = ACTIONS(2811),
    [sym_random_number] = ACTIONS(2811),
    [sym_get_unique_id] = ACTIONS(2811),
    [sym_get_system_number] = ACTIONS(2811),
    [sym_set_system_number] = ACTIONS(2811),
    [sym_get_serial_number] = ACTIONS(2811),
    [sym_get_master_build] = ACTIONS(2811),
    [sym_set_validation_code] = ACTIONS(2811),
    [sym_reboot] = ACTIONS(2811),
    [sym_load_duet_module] = ACTIONS(2811),
    [sym_unload_duet_module] = ACTIONS(2811),
    [sym_duet_mem_size_set] = ACTIONS(2811),
    [sym_duet_mem_size_get] = ACTIONS(2811),
    [sym_dynamic_application_device] = ACTIONS(2811),
    [sym_dynamic_polled_port] = ACTIONS(2811),
    [sym_static_port_binding] = ACTIONS(2811),
    [sym_static_ip_binding] = ACTIONS(2811),
    [sym_get_dns_list] = ACTIONS(2811),
    [sym_set_dns_list] = ACTIONS(2811),
    [sym_get_ip_address] = ACTIONS(2811),
    [sym_set_ip_address] = ACTIONS(2811),
    [sym_get_url_list] = ACTIONS(2811),
    [sym_add_url_entry] = ACTIONS(2811),
    [sym_delete_url_entry] = ACTIONS(2811),
    [sym_abs_value] = ACTIONS(2811),
    [sym_max_value] = ACTIONS(2811),
    [sym_min_value] = ACTIONS(2811),
    [sym_exp_value] = ACTIONS(2811),
    [sym_log_value] = ACTIONS(2811),
    [sym_log10_value] = ACTIONS(2811),
    [sym_power_value] = ACTIONS(2811),
    [sym_sqrt_value] = ACTIONS(2811),
    [sym_raw_be] = ACTIONS(2811),
    [sym_raw_le] = ACTIONS(2811),
    [sym_internal_queue_size_set] = ACTIONS(2811),
    [sym_internal_queue_size_get] = ACTIONS(2811),
    [sym_internal_threshold_set] = ACTIONS(2811),
    [sym_internal_threshold_get] = ACTIONS(2811),
    [sym_rebuilt_event] = ACTIONS(2811),
    [sym_do_custom_event] = ACTIONS(2811),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2811),
    [sym_clkmgr_set_clk_source] = ACTIONS(2811),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2811),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2811),
    [sym_clkmgr_get_timezone] = ACTIONS(2811),
    [sym_clkmgr_set_timezone] = ACTIONS(2811),
    [sym_clkmgr_get_resync_period] = ACTIONS(2811),
    [sym_clkmgr_set_resync_period] = ACTIONS(2811),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2811),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2811),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2811),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2811),
    [sym_clkmgr_get_timeservers] = ACTIONS(2811),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2811),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2811),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2811),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2811),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2811),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2811),
    [sym_set_log_level] = ACTIONS(2811),
    [sym_get_log_level] = ACTIONS(2811),
    [sym_amx_log] = ACTIONS(2811),
    [sym_smtp_server_config_set] = ACTIONS(2811),
    [sym_smtp_server_config_get] = ACTIONS(2811),
    [sym_smtp_send] = ACTIONS(2811),
    [sym_validate_netlinx_account] = ACTIONS(2811),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2811),
    [sym_authenticate_certificate] = ACTIONS(2811),
    [sym_audit_netlinx_session_event] = ACTIONS(2811),
    [sym_audit_netlinx_generic_event] = ACTIONS(2811),
    [sym_get_available_flash_disk_space] = ACTIONS(2811),
    [sym_get_max_flash_disk_space] = ACTIONS(2811),
    [sym_data_create_feed] = ACTIONS(2811),
    [sym_data_delete_feed] = ACTIONS(2811),
    [sym_data_publish_feed] = ACTIONS(2811),
    [sym_data_get_published_feed] = ACTIONS(2811),
    [sym_data_add_record] = ACTIONS(2811),
    [sym_data_get_event_record] = ACTIONS(2811),
    [sym__wc_data_create_feed] = ACTIONS(2811),
    [sym__wc_data_add_record] = ACTIONS(2811),
    [sym__wc_data_get_event_record] = ACTIONS(2811),
    [sym_format] = ACTIONS(2811),
    [sym_astro_clock] = ACTIONS(2811),
    [sym_set_outdoor_temperature] = ACTIONS(2811),
    [sym_set_virtual_level_count] = ACTIONS(2811),
    [sym_set_virtual_channel_count] = ACTIONS(2811),
    [sym_set_virtual_port_count] = ACTIONS(2811),
    [sym__wc_encode_int] = ACTIONS(2811),
    [sym__wc_decode_int] = ACTIONS(2811),
    [sym__wc_int] = ACTIONS(2811),
    [sym__wc_to_ch_int] = ACTIONS(2811),
    [sym__ch_to_wc_int] = ACTIONS(2811),
    [sym__wc_find_string_int] = ACTIONS(2811),
    [sym__wc_left_string_int] = ACTIONS(2811),
    [sym__wc_lower_string_int] = ACTIONS(2811),
    [sym__wc_mid_string_int] = ACTIONS(2811),
    [sym__wc_remove_string_int] = ACTIONS(2811),
    [sym__wc_right_string_int] = ACTIONS(2811),
    [sym__wc_upper_string_int] = ACTIONS(2811),
    [sym__wc_compare_string_int] = ACTIONS(2811),
    [sym__wc_get_buffer_char_int] = ACTIONS(2811),
    [sym__wc_get_buffer_string_int] = ACTIONS(2811),
    [sym__wc_concat_string_int] = ACTIONS(2811),
    [sym__wc_file_open_int] = ACTIONS(2811),
    [sym__wc_file_close_int] = ACTIONS(2811),
    [sym__wc_file_read_int] = ACTIONS(2811),
    [sym__wc_file_read_line_int] = ACTIONS(2811),
    [sym__wc_file_write_int] = ACTIONS(2811),
    [sym__wc_file_write_line_int] = ACTIONS(2811),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2813),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2813),
    [anon_sym_LBRACK] = ACTIONS(2813),
    [anon_sym_LBRACE] = ACTIONS(2813),
    [anon_sym_RBRACE] = ACTIONS(2813),
    [anon_sym_LPAREN] = ACTIONS(2811),
    [anon_sym_BANG] = ACTIONS(2813),
    [anon_sym_TILDE] = ACTIONS(2813),
    [anon_sym_DASH] = ACTIONS(2811),
    [anon_sym_PLUS] = ACTIONS(2811),
    [anon_sym_DASH_DASH] = ACTIONS(2813),
    [anon_sym_PLUS_PLUS] = ACTIONS(2813),
    [anon_sym_DQUOTE] = ACTIONS(2813),
    [anon_sym_SQUOTE] = ACTIONS(2813),
    [sym_number_literal] = ACTIONS(2813),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(938)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2232),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(939)] = {
    [sym_identifier] = ACTIONS(2815),
    [sym_not] = ACTIONS(2815),
    [sym_bnot] = ACTIONS(2815),
    [sym___file__] = ACTIONS(2815),
    [sym___date__] = ACTIONS(2815),
    [sym___ldate__] = ACTIONS(2815),
    [sym___time__] = ACTIONS(2815),
    [sym___line__] = ACTIONS(2815),
    [sym___name__] = ACTIONS(2815),
    [sym___netlinx__] = ACTIONS(2815),
    [sym_netlinx_axi_version] = ACTIONS(2815),
    [sym_url_flg_tcp] = ACTIONS(2815),
    [sym_url_flg_accinfopresent] = ACTIONS(2815),
    [sym_url_flg_temp] = ACTIONS(2815),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2815),
    [sym_url_flg_stat_mask] = ACTIONS(2815),
    [sym_url_flg_stat_lookup] = ACTIONS(2815),
    [sym_url_flg_stat_connecting] = ACTIONS(2815),
    [sym_url_flg_stat_waiting] = ACTIONS(2815),
    [sym_url_flg_stat_connected] = ACTIONS(2815),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2815),
    [sym_first_virtual_device] = ACTIONS(2815),
    [sym_first_local_port] = ACTIONS(2815),
    [sym_dynamic_virtual_device] = ACTIONS(2815),
    [sym_timeline_once] = ACTIONS(2815),
    [sym_timeline_repeat] = ACTIONS(2815),
    [sym_timeline_absolute] = ACTIONS(2815),
    [sym_timeline_relative] = ACTIONS(2815),
    [sym_do_push_timed_infinite] = ACTIONS(2815),
    [sym_source_type_no_address] = ACTIONS(2815),
    [sym_source_type_neuron_id] = ACTIONS(2815),
    [sym_source_type_ip_address] = ACTIONS(2815),
    [sym_source_type_axlink] = ACTIONS(2815),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2815),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2815),
    [sym_source_type_ip_socket_address] = ACTIONS(2815),
    [sym_source_type_rs232] = ACTIONS(2815),
    [sym_source_type_internal] = ACTIONS(2815),
    [sym_source_type_mac_address] = ACTIONS(2815),
    [sym_source_type_ipv4_port] = ACTIONS(2815),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2815),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2815),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2815),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2815),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2815),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2815),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2815),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2815),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2815),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2815),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2815),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2815),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2815),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2815),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2815),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2815),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2815),
    [sym_true] = ACTIONS(2815),
    [sym_false] = ACTIONS(2815),
    [sym_file_read_only] = ACTIONS(2815),
    [sym_file_rw_new] = ACTIONS(2815),
    [sym_file_rw_append] = ACTIONS(2815),
    [sym_ip_tcp] = ACTIONS(2815),
    [sym_ip_udp] = ACTIONS(2815),
    [sym_ip_udp_2way] = ACTIONS(2815),
    [sym_xml_encode_types] = ACTIONS(2815),
    [sym_xml_encode_char_as_list] = ACTIONS(2815),
    [sym_xml_encode_le] = ACTIONS(2815),
    [sym_xml_decode_types] = ACTIONS(2815),
    [sym_xml_decode_no_preserve] = ACTIONS(2815),
    [sym_duet_dev_type_display_device] = ACTIONS(2815),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2815),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2815),
    [sym_duet_dev_type_io_device] = ACTIONS(2815),
    [sym_duet_dev_type_relay_device] = ACTIONS(2815),
    [sym_duet_dev_type_ups] = ACTIONS(2815),
    [sym_duet_dev_type_amplifier] = ACTIONS(2815),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2815),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2815),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2815),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2815),
    [sym_duet_dev_type_camera] = ACTIONS(2815),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2815),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2815),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2815),
    [sym_duet_dev_type_dss] = ACTIONS(2815),
    [sym_duet_dev_type_dvr] = ACTIONS(2815),
    [sym_duet_dev_type_disc_device] = ACTIONS(2815),
    [sym_duet_dev_type_document_camera] = ACTIONS(2815),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2815),
    [sym_duet_dev_type_hvac] = ACTIONS(2815),
    [sym_duet_dev_type_keypad] = ACTIONS(2815),
    [sym_duet_dev_type_light] = ACTIONS(2815),
    [sym_duet_dev_type_monitor] = ACTIONS(2815),
    [sym_duet_dev_type_motor] = ACTIONS(2815),
    [sym_duet_dev_type_multi_window] = ACTIONS(2815),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2815),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2815),
    [sym_duet_dev_type_receiver] = ACTIONS(2815),
    [sym_duet_dev_type_security_system] = ACTIONS(2815),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2815),
    [sym_duet_dev_type_settop_box] = ACTIONS(2815),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2815),
    [sym_duet_dev_type_switcher] = ACTIONS(2815),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2815),
    [sym_duet_dev_type_tv] = ACTIONS(2815),
    [sym_duet_dev_type_utility] = ACTIONS(2815),
    [sym_duet_dev_type_vcr] = ACTIONS(2815),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2815),
    [sym_duet_dev_type_video_processor] = ACTIONS(2815),
    [sym_duet_dev_type_video_projector] = ACTIONS(2815),
    [sym_duet_dev_type_video_wall] = ACTIONS(2815),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2815),
    [sym_duet_dev_type_weather] = ACTIONS(2815),
    [sym_duet_dev_not_polled] = ACTIONS(2815),
    [sym_duet_dev_polled] = ACTIONS(2815),
    [sym_clkmgr_mode_network] = ACTIONS(2815),
    [sym_clkmgr_mode_standalone] = ACTIONS(2815),
    [sym_normal_standby] = ACTIONS(2815),
    [sym_normal_wake] = ACTIONS(2815),
    [sym_ip_multicast_ttl_option] = ACTIONS(2815),
    [sym_ip_ttl_subnet] = ACTIONS(2815),
    [sym_ip_ttl_site] = ACTIONS(2815),
    [sym_ip_ttl_region] = ACTIONS(2815),
    [sym_ip_ttl_continent] = ACTIONS(2815),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2815),
    [sym_ip_nodelay_on] = ACTIONS(2815),
    [sym_ip_nodelay_off] = ACTIONS(2815),
    [sym_amx_error] = ACTIONS(2815),
    [sym_amx_warning] = ACTIONS(2815),
    [sym_amx_info] = ACTIONS(2815),
    [sym_amx_debug] = ACTIONS(2815),
    [sym_netlinx_login_success] = ACTIONS(2815),
    [sym_netlinx_login_fail] = ACTIONS(2815),
    [sym_netlinx_logout] = ACTIONS(2815),
    [sym_null_str] = ACTIONS(2815),
    [sym_smtp_address] = ACTIONS(2815),
    [sym_smtp_port_number] = ACTIONS(2815),
    [sym_smtp_username] = ACTIONS(2815),
    [sym_smtp_password] = ACTIONS(2815),
    [sym_smtp_require_tls] = ACTIONS(2815),
    [sym_smtp_from] = ACTIONS(2815),
    [sym_smtp_tls_true] = ACTIONS(2815),
    [sym_smtp_tls_false] = ACTIONS(2815),
    [sym_listview_on_row_select_event] = ACTIONS(2815),
    [sym_data_structure_unknown] = ACTIONS(2815),
    [sym_data_structure_datafeed] = ACTIONS(2815),
    [sym_data_structure_datarecordset] = ACTIONS(2815),
    [sym_data_structure_datarecord] = ACTIONS(2815),
    [sym_data_structure_datafield] = ACTIONS(2815),
    [sym_data_max_content_field_count] = ACTIONS(2815),
    [sym_data_max_metadata_field_count] = ACTIONS(2815),
    [sym_data_max_id_length] = ACTIONS(2815),
    [sym_data_max_name_length] = ACTIONS(2815),
    [sym_data_max_description_length] = ACTIONS(2815),
    [sym_data_max_type_length] = ACTIONS(2815),
    [sym_data_max_format_length] = ACTIONS(2815),
    [sym_data_max_source_length] = ACTIONS(2815),
    [sym_data_max_value_length] = ACTIONS(2815),
    [sym_data_max_label_length] = ACTIONS(2815),
    [sym_data_max_url_length] = ACTIONS(2815),
    [sym_max_last_login_info_length] = ACTIONS(2815),
    [sym_data_type_unknown] = ACTIONS(2815),
    [sym_data_type_string] = ACTIONS(2815),
    [sym_data_type_datetime] = ACTIONS(2815),
    [sym_data_type_date] = ACTIONS(2815),
    [sym_data_type_time] = ACTIONS(2815),
    [sym_data_type_image] = ACTIONS(2815),
    [sym_data_format_url] = ACTIONS(2815),
    [sym_data_format_phone] = ACTIONS(2815),
    [sym_data_format_email] = ACTIONS(2815),
    [sym_data_format_iso8601] = ACTIONS(2815),
    [sym_auditlog_priv] = ACTIONS(2815),
    [sym_configuration_priv] = ACTIONS(2815),
    [sym_deviceconfig_priv] = ACTIONS(2815),
    [sym_ftp_priv] = ACTIONS(2815),
    [sym_http_priv] = ACTIONS(2815),
    [sym_networkconfig_priv] = ACTIONS(2815),
    [sym_programport_priv] = ACTIONS(2815),
    [sym_remoteui_priv] = ACTIONS(2815),
    [sym_securitycontrol_priv] = ACTIONS(2815),
    [sym_softwaremanagement_priv] = ACTIONS(2815),
    [sym_terminal_priv] = ACTIONS(2815),
    [sym_tpadmin_priv] = ACTIONS(2815),
    [sym_user1_priv] = ACTIONS(2815),
    [sym_user2_priv] = ACTIONS(2815),
    [sym_user3_priv] = ACTIONS(2815),
    [sym_user4_priv] = ACTIONS(2815),
    [sym_usermanagement_priv] = ACTIONS(2815),
    [sym_valid_account] = ACTIONS(2815),
    [sym_err_name_invalid] = ACTIONS(2815),
    [sym_err_password_invalid] = ACTIONS(2815),
    [sym_err_invalid_account] = ACTIONS(2815),
    [sym_err_no_authorization] = ACTIONS(2815),
    [sym_err_invalid_parameter] = ACTIONS(2815),
    [sym_err_user_locked_out] = ACTIONS(2815),
    [sym_err_user_account_expired] = ACTIONS(2815),
    [sym_err_general_error] = ACTIONS(2815),
    [sym_err_authentication_error] = ACTIONS(2815),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2815),
    [sym_tls_validate_certificate] = ACTIONS(2815),
    [sym____reserved___] = ACTIONS(2815),
    [sym_date] = ACTIONS(2815),
    [sym_day] = ACTIONS(2815),
    [sym_time] = ACTIONS(2815),
    [sym_ldate] = ACTIONS(2815),
    [sym_push_device] = ACTIONS(2815),
    [sym_push_channel] = ACTIONS(2815),
    [sym_push_devchan] = ACTIONS(2815),
    [sym_release_device] = ACTIONS(2815),
    [sym_release_channel] = ACTIONS(2815),
    [sym_release_devchan] = ACTIONS(2815),
    [sym_master_slot] = ACTIONS(2815),
    [sym_get_pulse_time] = ACTIONS(2815),
    [sym_get_timer] = ACTIONS(2815),
    [sym_master_sn] = ACTIONS(2815),
    [sym_system_number] = ACTIONS(2815),
    [sym_button] = ACTIONS(2815),
    [sym_dv_channel] = ACTIONS(2815),
    [sym_data] = ACTIONS(2815),
    [sym_level] = ACTIONS(2815),
    [sym_timeline] = ACTIONS(2815),
    [sym_channel] = ACTIONS(2815),
    [sym_custom] = ACTIONS(2815),
    [sym_atoi] = ACTIONS(2815),
    [sym_atol] = ACTIONS(2815),
    [sym_atof] = ACTIONS(2815),
    [sym_itoa] = ACTIONS(2815),
    [sym_itohex] = ACTIONS(2815),
    [sym_hextoi] = ACTIONS(2815),
    [sym_ftoa] = ACTIONS(2815),
    [sym_find_string] = ACTIONS(2815),
    [sym_left_string] = ACTIONS(2815),
    [sym_right_string] = ACTIONS(2815),
    [sym_mid_string] = ACTIONS(2815),
    [sym_remove_string] = ACTIONS(2815),
    [sym_get_buffer_string] = ACTIONS(2815),
    [sym_get_buffer_char] = ACTIONS(2815),
    [sym_get_multi_buffer_string] = ACTIONS(2815),
    [sym_length_string] = ACTIONS(2815),
    [sym_max_length_string] = ACTIONS(2815),
    [sym_set_length_string] = ACTIONS(2815),
    [sym_compare_string] = ACTIONS(2815),
    [sym_upper_string] = ACTIONS(2815),
    [sym_lower_string] = ACTIONS(2815),
    [sym_get_last] = ACTIONS(2815),
    [sym_date_to_day] = ACTIONS(2815),
    [sym_date_to_month] = ACTIONS(2815),
    [sym_date_to_year] = ACTIONS(2815),
    [sym_time_to_hour] = ACTIONS(2815),
    [sym_time_to_minute] = ACTIONS(2815),
    [sym_time_to_second] = ACTIONS(2815),
    [sym_day_of_week] = ACTIONS(2815),
    [sym_set_pulse_time] = ACTIONS(2815),
    [sym_set_timer] = ACTIONS(2815),
    [sym_file_open] = ACTIONS(2815),
    [sym_file_close] = ACTIONS(2815),
    [sym_file_read] = ACTIONS(2815),
    [sym_file_read_line] = ACTIONS(2815),
    [sym_file_write] = ACTIONS(2815),
    [sym_file_write_line] = ACTIONS(2815),
    [sym_file_copy] = ACTIONS(2815),
    [sym_file_delete] = ACTIONS(2815),
    [sym_file_rename] = ACTIONS(2815),
    [sym_file_dir] = ACTIONS(2815),
    [sym_file_getdir] = ACTIONS(2815),
    [sym_file_setdir] = ACTIONS(2815),
    [sym_file_create_dir] = ACTIONS(2815),
    [sym_file_remove_dir] = ACTIONS(2815),
    [sym_file_seek] = ACTIONS(2815),
    [sym_length_array] = ACTIONS(2815),
    [sym_max_length_array] = ACTIONS(2815),
    [sym_set_length_array] = ACTIONS(2815),
    [sym_ip_client_open] = ACTIONS(2815),
    [sym_ip_client_close] = ACTIONS(2815),
    [sym_ip_server_open] = ACTIONS(2815),
    [sym_ip_server_close] = ACTIONS(2815),
    [sym_ip_bound_client_open] = ACTIONS(2815),
    [sym_ip_mc_server_open] = ACTIONS(2815),
    [sym_ip_set_option] = ACTIONS(2815),
    [sym_ssh_client_open] = ACTIONS(2815),
    [sym_ssh_client_close] = ACTIONS(2815),
    [sym_tls_client_open] = ACTIONS(2815),
    [sym_tls_client_close] = ACTIONS(2815),
    [sym_do_push] = ACTIONS(2815),
    [sym_do_push_timed] = ACTIONS(2815),
    [sym_do_release] = ACTIONS(2815),
    [sym_device_id] = ACTIONS(2815),
    [sym_device_info] = ACTIONS(2815),
    [sym_device_id_string] = ACTIONS(2815),
    [sym_device_standby] = ACTIONS(2815),
    [sym_device_wake] = ACTIONS(2815),
    [sym_timeline_create] = ACTIONS(2815),
    [sym_timeline_kill] = ACTIONS(2815),
    [sym_timeline_reload] = ACTIONS(2815),
    [sym_timeline_pause] = ACTIONS(2815),
    [sym_timeline_restart] = ACTIONS(2815),
    [sym_timeline_set] = ACTIONS(2815),
    [sym_timeline_get] = ACTIONS(2815),
    [sym_timeline_active] = ACTIONS(2815),
    [sym_timeline_dynamic_id] = ACTIONS(2815),
    [sym_variable_to_string] = ACTIONS(2815),
    [sym_string_to_variable] = ACTIONS(2815),
    [sym_length_variable_to_string] = ACTIONS(2815),
    [sym_type_cast] = ACTIONS(2815),
    [sym_variable_to_xml] = ACTIONS(2815),
    [sym_xml_to_variable] = ACTIONS(2815),
    [sym_length_variable_to_xml] = ACTIONS(2815),
    [sym_redirect_string] = ACTIONS(2815),
    [sym_random_number] = ACTIONS(2815),
    [sym_get_unique_id] = ACTIONS(2815),
    [sym_get_system_number] = ACTIONS(2815),
    [sym_set_system_number] = ACTIONS(2815),
    [sym_get_serial_number] = ACTIONS(2815),
    [sym_get_master_build] = ACTIONS(2815),
    [sym_set_validation_code] = ACTIONS(2815),
    [sym_reboot] = ACTIONS(2815),
    [sym_load_duet_module] = ACTIONS(2815),
    [sym_unload_duet_module] = ACTIONS(2815),
    [sym_duet_mem_size_set] = ACTIONS(2815),
    [sym_duet_mem_size_get] = ACTIONS(2815),
    [sym_dynamic_application_device] = ACTIONS(2815),
    [sym_dynamic_polled_port] = ACTIONS(2815),
    [sym_static_port_binding] = ACTIONS(2815),
    [sym_static_ip_binding] = ACTIONS(2815),
    [sym_get_dns_list] = ACTIONS(2815),
    [sym_set_dns_list] = ACTIONS(2815),
    [sym_get_ip_address] = ACTIONS(2815),
    [sym_set_ip_address] = ACTIONS(2815),
    [sym_get_url_list] = ACTIONS(2815),
    [sym_add_url_entry] = ACTIONS(2815),
    [sym_delete_url_entry] = ACTIONS(2815),
    [sym_abs_value] = ACTIONS(2815),
    [sym_max_value] = ACTIONS(2815),
    [sym_min_value] = ACTIONS(2815),
    [sym_exp_value] = ACTIONS(2815),
    [sym_log_value] = ACTIONS(2815),
    [sym_log10_value] = ACTIONS(2815),
    [sym_power_value] = ACTIONS(2815),
    [sym_sqrt_value] = ACTIONS(2815),
    [sym_raw_be] = ACTIONS(2815),
    [sym_raw_le] = ACTIONS(2815),
    [sym_internal_queue_size_set] = ACTIONS(2815),
    [sym_internal_queue_size_get] = ACTIONS(2815),
    [sym_internal_threshold_set] = ACTIONS(2815),
    [sym_internal_threshold_get] = ACTIONS(2815),
    [sym_rebuilt_event] = ACTIONS(2815),
    [sym_do_custom_event] = ACTIONS(2815),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2815),
    [sym_clkmgr_set_clk_source] = ACTIONS(2815),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2815),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2815),
    [sym_clkmgr_get_timezone] = ACTIONS(2815),
    [sym_clkmgr_set_timezone] = ACTIONS(2815),
    [sym_clkmgr_get_resync_period] = ACTIONS(2815),
    [sym_clkmgr_set_resync_period] = ACTIONS(2815),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2815),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2815),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2815),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2815),
    [sym_clkmgr_get_timeservers] = ACTIONS(2815),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2815),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2815),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2815),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2815),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2815),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2815),
    [sym_set_log_level] = ACTIONS(2815),
    [sym_get_log_level] = ACTIONS(2815),
    [sym_amx_log] = ACTIONS(2815),
    [sym_smtp_server_config_set] = ACTIONS(2815),
    [sym_smtp_server_config_get] = ACTIONS(2815),
    [sym_smtp_send] = ACTIONS(2815),
    [sym_validate_netlinx_account] = ACTIONS(2815),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2815),
    [sym_authenticate_certificate] = ACTIONS(2815),
    [sym_audit_netlinx_session_event] = ACTIONS(2815),
    [sym_audit_netlinx_generic_event] = ACTIONS(2815),
    [sym_get_available_flash_disk_space] = ACTIONS(2815),
    [sym_get_max_flash_disk_space] = ACTIONS(2815),
    [sym_data_create_feed] = ACTIONS(2815),
    [sym_data_delete_feed] = ACTIONS(2815),
    [sym_data_publish_feed] = ACTIONS(2815),
    [sym_data_get_published_feed] = ACTIONS(2815),
    [sym_data_add_record] = ACTIONS(2815),
    [sym_data_get_event_record] = ACTIONS(2815),
    [sym__wc_data_create_feed] = ACTIONS(2815),
    [sym__wc_data_add_record] = ACTIONS(2815),
    [sym__wc_data_get_event_record] = ACTIONS(2815),
    [sym_format] = ACTIONS(2815),
    [sym_astro_clock] = ACTIONS(2815),
    [sym_set_outdoor_temperature] = ACTIONS(2815),
    [sym_set_virtual_level_count] = ACTIONS(2815),
    [sym_set_virtual_channel_count] = ACTIONS(2815),
    [sym_set_virtual_port_count] = ACTIONS(2815),
    [sym__wc_encode_int] = ACTIONS(2815),
    [sym__wc_decode_int] = ACTIONS(2815),
    [sym__wc_int] = ACTIONS(2815),
    [sym__wc_to_ch_int] = ACTIONS(2815),
    [sym__ch_to_wc_int] = ACTIONS(2815),
    [sym__wc_find_string_int] = ACTIONS(2815),
    [sym__wc_left_string_int] = ACTIONS(2815),
    [sym__wc_lower_string_int] = ACTIONS(2815),
    [sym__wc_mid_string_int] = ACTIONS(2815),
    [sym__wc_remove_string_int] = ACTIONS(2815),
    [sym__wc_right_string_int] = ACTIONS(2815),
    [sym__wc_upper_string_int] = ACTIONS(2815),
    [sym__wc_compare_string_int] = ACTIONS(2815),
    [sym__wc_get_buffer_char_int] = ACTIONS(2815),
    [sym__wc_get_buffer_string_int] = ACTIONS(2815),
    [sym__wc_concat_string_int] = ACTIONS(2815),
    [sym__wc_file_open_int] = ACTIONS(2815),
    [sym__wc_file_close_int] = ACTIONS(2815),
    [sym__wc_file_read_int] = ACTIONS(2815),
    [sym__wc_file_read_line_int] = ACTIONS(2815),
    [sym__wc_file_write_int] = ACTIONS(2815),
    [sym__wc_file_write_line_int] = ACTIONS(2815),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2817),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2817),
    [anon_sym_LBRACK] = ACTIONS(2817),
    [anon_sym_LBRACE] = ACTIONS(2817),
    [anon_sym_RBRACE] = ACTIONS(2817),
    [anon_sym_LPAREN] = ACTIONS(2815),
    [anon_sym_BANG] = ACTIONS(2817),
    [anon_sym_TILDE] = ACTIONS(2817),
    [anon_sym_DASH] = ACTIONS(2815),
    [anon_sym_PLUS] = ACTIONS(2815),
    [anon_sym_DASH_DASH] = ACTIONS(2817),
    [anon_sym_PLUS_PLUS] = ACTIONS(2817),
    [anon_sym_DQUOTE] = ACTIONS(2817),
    [anon_sym_SQUOTE] = ACTIONS(2817),
    [sym_number_literal] = ACTIONS(2817),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(940)] = {
    [sym_identifier] = ACTIONS(2819),
    [sym_not] = ACTIONS(2819),
    [sym_bnot] = ACTIONS(2819),
    [sym___file__] = ACTIONS(2819),
    [sym___date__] = ACTIONS(2819),
    [sym___ldate__] = ACTIONS(2819),
    [sym___time__] = ACTIONS(2819),
    [sym___line__] = ACTIONS(2819),
    [sym___name__] = ACTIONS(2819),
    [sym___netlinx__] = ACTIONS(2819),
    [sym_netlinx_axi_version] = ACTIONS(2819),
    [sym_url_flg_tcp] = ACTIONS(2819),
    [sym_url_flg_accinfopresent] = ACTIONS(2819),
    [sym_url_flg_temp] = ACTIONS(2819),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2819),
    [sym_url_flg_stat_mask] = ACTIONS(2819),
    [sym_url_flg_stat_lookup] = ACTIONS(2819),
    [sym_url_flg_stat_connecting] = ACTIONS(2819),
    [sym_url_flg_stat_waiting] = ACTIONS(2819),
    [sym_url_flg_stat_connected] = ACTIONS(2819),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2819),
    [sym_first_virtual_device] = ACTIONS(2819),
    [sym_first_local_port] = ACTIONS(2819),
    [sym_dynamic_virtual_device] = ACTIONS(2819),
    [sym_timeline_once] = ACTIONS(2819),
    [sym_timeline_repeat] = ACTIONS(2819),
    [sym_timeline_absolute] = ACTIONS(2819),
    [sym_timeline_relative] = ACTIONS(2819),
    [sym_do_push_timed_infinite] = ACTIONS(2819),
    [sym_source_type_no_address] = ACTIONS(2819),
    [sym_source_type_neuron_id] = ACTIONS(2819),
    [sym_source_type_ip_address] = ACTIONS(2819),
    [sym_source_type_axlink] = ACTIONS(2819),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2819),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2819),
    [sym_source_type_ip_socket_address] = ACTIONS(2819),
    [sym_source_type_rs232] = ACTIONS(2819),
    [sym_source_type_internal] = ACTIONS(2819),
    [sym_source_type_mac_address] = ACTIONS(2819),
    [sym_source_type_ipv4_port] = ACTIONS(2819),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2819),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2819),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2819),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2819),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2819),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2819),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2819),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2819),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2819),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2819),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2819),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2819),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2819),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2819),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2819),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2819),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2819),
    [sym_true] = ACTIONS(2819),
    [sym_false] = ACTIONS(2819),
    [sym_file_read_only] = ACTIONS(2819),
    [sym_file_rw_new] = ACTIONS(2819),
    [sym_file_rw_append] = ACTIONS(2819),
    [sym_ip_tcp] = ACTIONS(2819),
    [sym_ip_udp] = ACTIONS(2819),
    [sym_ip_udp_2way] = ACTIONS(2819),
    [sym_xml_encode_types] = ACTIONS(2819),
    [sym_xml_encode_char_as_list] = ACTIONS(2819),
    [sym_xml_encode_le] = ACTIONS(2819),
    [sym_xml_decode_types] = ACTIONS(2819),
    [sym_xml_decode_no_preserve] = ACTIONS(2819),
    [sym_duet_dev_type_display_device] = ACTIONS(2819),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2819),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2819),
    [sym_duet_dev_type_io_device] = ACTIONS(2819),
    [sym_duet_dev_type_relay_device] = ACTIONS(2819),
    [sym_duet_dev_type_ups] = ACTIONS(2819),
    [sym_duet_dev_type_amplifier] = ACTIONS(2819),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2819),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2819),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2819),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2819),
    [sym_duet_dev_type_camera] = ACTIONS(2819),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2819),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2819),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2819),
    [sym_duet_dev_type_dss] = ACTIONS(2819),
    [sym_duet_dev_type_dvr] = ACTIONS(2819),
    [sym_duet_dev_type_disc_device] = ACTIONS(2819),
    [sym_duet_dev_type_document_camera] = ACTIONS(2819),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2819),
    [sym_duet_dev_type_hvac] = ACTIONS(2819),
    [sym_duet_dev_type_keypad] = ACTIONS(2819),
    [sym_duet_dev_type_light] = ACTIONS(2819),
    [sym_duet_dev_type_monitor] = ACTIONS(2819),
    [sym_duet_dev_type_motor] = ACTIONS(2819),
    [sym_duet_dev_type_multi_window] = ACTIONS(2819),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2819),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2819),
    [sym_duet_dev_type_receiver] = ACTIONS(2819),
    [sym_duet_dev_type_security_system] = ACTIONS(2819),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2819),
    [sym_duet_dev_type_settop_box] = ACTIONS(2819),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2819),
    [sym_duet_dev_type_switcher] = ACTIONS(2819),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2819),
    [sym_duet_dev_type_tv] = ACTIONS(2819),
    [sym_duet_dev_type_utility] = ACTIONS(2819),
    [sym_duet_dev_type_vcr] = ACTIONS(2819),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2819),
    [sym_duet_dev_type_video_processor] = ACTIONS(2819),
    [sym_duet_dev_type_video_projector] = ACTIONS(2819),
    [sym_duet_dev_type_video_wall] = ACTIONS(2819),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2819),
    [sym_duet_dev_type_weather] = ACTIONS(2819),
    [sym_duet_dev_not_polled] = ACTIONS(2819),
    [sym_duet_dev_polled] = ACTIONS(2819),
    [sym_clkmgr_mode_network] = ACTIONS(2819),
    [sym_clkmgr_mode_standalone] = ACTIONS(2819),
    [sym_normal_standby] = ACTIONS(2819),
    [sym_normal_wake] = ACTIONS(2819),
    [sym_ip_multicast_ttl_option] = ACTIONS(2819),
    [sym_ip_ttl_subnet] = ACTIONS(2819),
    [sym_ip_ttl_site] = ACTIONS(2819),
    [sym_ip_ttl_region] = ACTIONS(2819),
    [sym_ip_ttl_continent] = ACTIONS(2819),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2819),
    [sym_ip_nodelay_on] = ACTIONS(2819),
    [sym_ip_nodelay_off] = ACTIONS(2819),
    [sym_amx_error] = ACTIONS(2819),
    [sym_amx_warning] = ACTIONS(2819),
    [sym_amx_info] = ACTIONS(2819),
    [sym_amx_debug] = ACTIONS(2819),
    [sym_netlinx_login_success] = ACTIONS(2819),
    [sym_netlinx_login_fail] = ACTIONS(2819),
    [sym_netlinx_logout] = ACTIONS(2819),
    [sym_null_str] = ACTIONS(2819),
    [sym_smtp_address] = ACTIONS(2819),
    [sym_smtp_port_number] = ACTIONS(2819),
    [sym_smtp_username] = ACTIONS(2819),
    [sym_smtp_password] = ACTIONS(2819),
    [sym_smtp_require_tls] = ACTIONS(2819),
    [sym_smtp_from] = ACTIONS(2819),
    [sym_smtp_tls_true] = ACTIONS(2819),
    [sym_smtp_tls_false] = ACTIONS(2819),
    [sym_listview_on_row_select_event] = ACTIONS(2819),
    [sym_data_structure_unknown] = ACTIONS(2819),
    [sym_data_structure_datafeed] = ACTIONS(2819),
    [sym_data_structure_datarecordset] = ACTIONS(2819),
    [sym_data_structure_datarecord] = ACTIONS(2819),
    [sym_data_structure_datafield] = ACTIONS(2819),
    [sym_data_max_content_field_count] = ACTIONS(2819),
    [sym_data_max_metadata_field_count] = ACTIONS(2819),
    [sym_data_max_id_length] = ACTIONS(2819),
    [sym_data_max_name_length] = ACTIONS(2819),
    [sym_data_max_description_length] = ACTIONS(2819),
    [sym_data_max_type_length] = ACTIONS(2819),
    [sym_data_max_format_length] = ACTIONS(2819),
    [sym_data_max_source_length] = ACTIONS(2819),
    [sym_data_max_value_length] = ACTIONS(2819),
    [sym_data_max_label_length] = ACTIONS(2819),
    [sym_data_max_url_length] = ACTIONS(2819),
    [sym_max_last_login_info_length] = ACTIONS(2819),
    [sym_data_type_unknown] = ACTIONS(2819),
    [sym_data_type_string] = ACTIONS(2819),
    [sym_data_type_datetime] = ACTIONS(2819),
    [sym_data_type_date] = ACTIONS(2819),
    [sym_data_type_time] = ACTIONS(2819),
    [sym_data_type_image] = ACTIONS(2819),
    [sym_data_format_url] = ACTIONS(2819),
    [sym_data_format_phone] = ACTIONS(2819),
    [sym_data_format_email] = ACTIONS(2819),
    [sym_data_format_iso8601] = ACTIONS(2819),
    [sym_auditlog_priv] = ACTIONS(2819),
    [sym_configuration_priv] = ACTIONS(2819),
    [sym_deviceconfig_priv] = ACTIONS(2819),
    [sym_ftp_priv] = ACTIONS(2819),
    [sym_http_priv] = ACTIONS(2819),
    [sym_networkconfig_priv] = ACTIONS(2819),
    [sym_programport_priv] = ACTIONS(2819),
    [sym_remoteui_priv] = ACTIONS(2819),
    [sym_securitycontrol_priv] = ACTIONS(2819),
    [sym_softwaremanagement_priv] = ACTIONS(2819),
    [sym_terminal_priv] = ACTIONS(2819),
    [sym_tpadmin_priv] = ACTIONS(2819),
    [sym_user1_priv] = ACTIONS(2819),
    [sym_user2_priv] = ACTIONS(2819),
    [sym_user3_priv] = ACTIONS(2819),
    [sym_user4_priv] = ACTIONS(2819),
    [sym_usermanagement_priv] = ACTIONS(2819),
    [sym_valid_account] = ACTIONS(2819),
    [sym_err_name_invalid] = ACTIONS(2819),
    [sym_err_password_invalid] = ACTIONS(2819),
    [sym_err_invalid_account] = ACTIONS(2819),
    [sym_err_no_authorization] = ACTIONS(2819),
    [sym_err_invalid_parameter] = ACTIONS(2819),
    [sym_err_user_locked_out] = ACTIONS(2819),
    [sym_err_user_account_expired] = ACTIONS(2819),
    [sym_err_general_error] = ACTIONS(2819),
    [sym_err_authentication_error] = ACTIONS(2819),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2819),
    [sym_tls_validate_certificate] = ACTIONS(2819),
    [sym____reserved___] = ACTIONS(2819),
    [sym_date] = ACTIONS(2819),
    [sym_day] = ACTIONS(2819),
    [sym_time] = ACTIONS(2819),
    [sym_ldate] = ACTIONS(2819),
    [sym_push_device] = ACTIONS(2819),
    [sym_push_channel] = ACTIONS(2819),
    [sym_push_devchan] = ACTIONS(2819),
    [sym_release_device] = ACTIONS(2819),
    [sym_release_channel] = ACTIONS(2819),
    [sym_release_devchan] = ACTIONS(2819),
    [sym_master_slot] = ACTIONS(2819),
    [sym_get_pulse_time] = ACTIONS(2819),
    [sym_get_timer] = ACTIONS(2819),
    [sym_master_sn] = ACTIONS(2819),
    [sym_system_number] = ACTIONS(2819),
    [sym_button] = ACTIONS(2819),
    [sym_dv_channel] = ACTIONS(2819),
    [sym_data] = ACTIONS(2819),
    [sym_level] = ACTIONS(2819),
    [sym_timeline] = ACTIONS(2819),
    [sym_channel] = ACTIONS(2819),
    [sym_custom] = ACTIONS(2819),
    [sym_atoi] = ACTIONS(2819),
    [sym_atol] = ACTIONS(2819),
    [sym_atof] = ACTIONS(2819),
    [sym_itoa] = ACTIONS(2819),
    [sym_itohex] = ACTIONS(2819),
    [sym_hextoi] = ACTIONS(2819),
    [sym_ftoa] = ACTIONS(2819),
    [sym_find_string] = ACTIONS(2819),
    [sym_left_string] = ACTIONS(2819),
    [sym_right_string] = ACTIONS(2819),
    [sym_mid_string] = ACTIONS(2819),
    [sym_remove_string] = ACTIONS(2819),
    [sym_get_buffer_string] = ACTIONS(2819),
    [sym_get_buffer_char] = ACTIONS(2819),
    [sym_get_multi_buffer_string] = ACTIONS(2819),
    [sym_length_string] = ACTIONS(2819),
    [sym_max_length_string] = ACTIONS(2819),
    [sym_set_length_string] = ACTIONS(2819),
    [sym_compare_string] = ACTIONS(2819),
    [sym_upper_string] = ACTIONS(2819),
    [sym_lower_string] = ACTIONS(2819),
    [sym_get_last] = ACTIONS(2819),
    [sym_date_to_day] = ACTIONS(2819),
    [sym_date_to_month] = ACTIONS(2819),
    [sym_date_to_year] = ACTIONS(2819),
    [sym_time_to_hour] = ACTIONS(2819),
    [sym_time_to_minute] = ACTIONS(2819),
    [sym_time_to_second] = ACTIONS(2819),
    [sym_day_of_week] = ACTIONS(2819),
    [sym_set_pulse_time] = ACTIONS(2819),
    [sym_set_timer] = ACTIONS(2819),
    [sym_file_open] = ACTIONS(2819),
    [sym_file_close] = ACTIONS(2819),
    [sym_file_read] = ACTIONS(2819),
    [sym_file_read_line] = ACTIONS(2819),
    [sym_file_write] = ACTIONS(2819),
    [sym_file_write_line] = ACTIONS(2819),
    [sym_file_copy] = ACTIONS(2819),
    [sym_file_delete] = ACTIONS(2819),
    [sym_file_rename] = ACTIONS(2819),
    [sym_file_dir] = ACTIONS(2819),
    [sym_file_getdir] = ACTIONS(2819),
    [sym_file_setdir] = ACTIONS(2819),
    [sym_file_create_dir] = ACTIONS(2819),
    [sym_file_remove_dir] = ACTIONS(2819),
    [sym_file_seek] = ACTIONS(2819),
    [sym_length_array] = ACTIONS(2819),
    [sym_max_length_array] = ACTIONS(2819),
    [sym_set_length_array] = ACTIONS(2819),
    [sym_ip_client_open] = ACTIONS(2819),
    [sym_ip_client_close] = ACTIONS(2819),
    [sym_ip_server_open] = ACTIONS(2819),
    [sym_ip_server_close] = ACTIONS(2819),
    [sym_ip_bound_client_open] = ACTIONS(2819),
    [sym_ip_mc_server_open] = ACTIONS(2819),
    [sym_ip_set_option] = ACTIONS(2819),
    [sym_ssh_client_open] = ACTIONS(2819),
    [sym_ssh_client_close] = ACTIONS(2819),
    [sym_tls_client_open] = ACTIONS(2819),
    [sym_tls_client_close] = ACTIONS(2819),
    [sym_do_push] = ACTIONS(2819),
    [sym_do_push_timed] = ACTIONS(2819),
    [sym_do_release] = ACTIONS(2819),
    [sym_device_id] = ACTIONS(2819),
    [sym_device_info] = ACTIONS(2819),
    [sym_device_id_string] = ACTIONS(2819),
    [sym_device_standby] = ACTIONS(2819),
    [sym_device_wake] = ACTIONS(2819),
    [sym_timeline_create] = ACTIONS(2819),
    [sym_timeline_kill] = ACTIONS(2819),
    [sym_timeline_reload] = ACTIONS(2819),
    [sym_timeline_pause] = ACTIONS(2819),
    [sym_timeline_restart] = ACTIONS(2819),
    [sym_timeline_set] = ACTIONS(2819),
    [sym_timeline_get] = ACTIONS(2819),
    [sym_timeline_active] = ACTIONS(2819),
    [sym_timeline_dynamic_id] = ACTIONS(2819),
    [sym_variable_to_string] = ACTIONS(2819),
    [sym_string_to_variable] = ACTIONS(2819),
    [sym_length_variable_to_string] = ACTIONS(2819),
    [sym_type_cast] = ACTIONS(2819),
    [sym_variable_to_xml] = ACTIONS(2819),
    [sym_xml_to_variable] = ACTIONS(2819),
    [sym_length_variable_to_xml] = ACTIONS(2819),
    [sym_redirect_string] = ACTIONS(2819),
    [sym_random_number] = ACTIONS(2819),
    [sym_get_unique_id] = ACTIONS(2819),
    [sym_get_system_number] = ACTIONS(2819),
    [sym_set_system_number] = ACTIONS(2819),
    [sym_get_serial_number] = ACTIONS(2819),
    [sym_get_master_build] = ACTIONS(2819),
    [sym_set_validation_code] = ACTIONS(2819),
    [sym_reboot] = ACTIONS(2819),
    [sym_load_duet_module] = ACTIONS(2819),
    [sym_unload_duet_module] = ACTIONS(2819),
    [sym_duet_mem_size_set] = ACTIONS(2819),
    [sym_duet_mem_size_get] = ACTIONS(2819),
    [sym_dynamic_application_device] = ACTIONS(2819),
    [sym_dynamic_polled_port] = ACTIONS(2819),
    [sym_static_port_binding] = ACTIONS(2819),
    [sym_static_ip_binding] = ACTIONS(2819),
    [sym_get_dns_list] = ACTIONS(2819),
    [sym_set_dns_list] = ACTIONS(2819),
    [sym_get_ip_address] = ACTIONS(2819),
    [sym_set_ip_address] = ACTIONS(2819),
    [sym_get_url_list] = ACTIONS(2819),
    [sym_add_url_entry] = ACTIONS(2819),
    [sym_delete_url_entry] = ACTIONS(2819),
    [sym_abs_value] = ACTIONS(2819),
    [sym_max_value] = ACTIONS(2819),
    [sym_min_value] = ACTIONS(2819),
    [sym_exp_value] = ACTIONS(2819),
    [sym_log_value] = ACTIONS(2819),
    [sym_log10_value] = ACTIONS(2819),
    [sym_power_value] = ACTIONS(2819),
    [sym_sqrt_value] = ACTIONS(2819),
    [sym_raw_be] = ACTIONS(2819),
    [sym_raw_le] = ACTIONS(2819),
    [sym_internal_queue_size_set] = ACTIONS(2819),
    [sym_internal_queue_size_get] = ACTIONS(2819),
    [sym_internal_threshold_set] = ACTIONS(2819),
    [sym_internal_threshold_get] = ACTIONS(2819),
    [sym_rebuilt_event] = ACTIONS(2819),
    [sym_do_custom_event] = ACTIONS(2819),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2819),
    [sym_clkmgr_set_clk_source] = ACTIONS(2819),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2819),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2819),
    [sym_clkmgr_get_timezone] = ACTIONS(2819),
    [sym_clkmgr_set_timezone] = ACTIONS(2819),
    [sym_clkmgr_get_resync_period] = ACTIONS(2819),
    [sym_clkmgr_set_resync_period] = ACTIONS(2819),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2819),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2819),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2819),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2819),
    [sym_clkmgr_get_timeservers] = ACTIONS(2819),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2819),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2819),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2819),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2819),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2819),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2819),
    [sym_set_log_level] = ACTIONS(2819),
    [sym_get_log_level] = ACTIONS(2819),
    [sym_amx_log] = ACTIONS(2819),
    [sym_smtp_server_config_set] = ACTIONS(2819),
    [sym_smtp_server_config_get] = ACTIONS(2819),
    [sym_smtp_send] = ACTIONS(2819),
    [sym_validate_netlinx_account] = ACTIONS(2819),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2819),
    [sym_authenticate_certificate] = ACTIONS(2819),
    [sym_audit_netlinx_session_event] = ACTIONS(2819),
    [sym_audit_netlinx_generic_event] = ACTIONS(2819),
    [sym_get_available_flash_disk_space] = ACTIONS(2819),
    [sym_get_max_flash_disk_space] = ACTIONS(2819),
    [sym_data_create_feed] = ACTIONS(2819),
    [sym_data_delete_feed] = ACTIONS(2819),
    [sym_data_publish_feed] = ACTIONS(2819),
    [sym_data_get_published_feed] = ACTIONS(2819),
    [sym_data_add_record] = ACTIONS(2819),
    [sym_data_get_event_record] = ACTIONS(2819),
    [sym__wc_data_create_feed] = ACTIONS(2819),
    [sym__wc_data_add_record] = ACTIONS(2819),
    [sym__wc_data_get_event_record] = ACTIONS(2819),
    [sym_format] = ACTIONS(2819),
    [sym_astro_clock] = ACTIONS(2819),
    [sym_set_outdoor_temperature] = ACTIONS(2819),
    [sym_set_virtual_level_count] = ACTIONS(2819),
    [sym_set_virtual_channel_count] = ACTIONS(2819),
    [sym_set_virtual_port_count] = ACTIONS(2819),
    [sym__wc_encode_int] = ACTIONS(2819),
    [sym__wc_decode_int] = ACTIONS(2819),
    [sym__wc_int] = ACTIONS(2819),
    [sym__wc_to_ch_int] = ACTIONS(2819),
    [sym__ch_to_wc_int] = ACTIONS(2819),
    [sym__wc_find_string_int] = ACTIONS(2819),
    [sym__wc_left_string_int] = ACTIONS(2819),
    [sym__wc_lower_string_int] = ACTIONS(2819),
    [sym__wc_mid_string_int] = ACTIONS(2819),
    [sym__wc_remove_string_int] = ACTIONS(2819),
    [sym__wc_right_string_int] = ACTIONS(2819),
    [sym__wc_upper_string_int] = ACTIONS(2819),
    [sym__wc_compare_string_int] = ACTIONS(2819),
    [sym__wc_get_buffer_char_int] = ACTIONS(2819),
    [sym__wc_get_buffer_string_int] = ACTIONS(2819),
    [sym__wc_concat_string_int] = ACTIONS(2819),
    [sym__wc_file_open_int] = ACTIONS(2819),
    [sym__wc_file_close_int] = ACTIONS(2819),
    [sym__wc_file_read_int] = ACTIONS(2819),
    [sym__wc_file_read_line_int] = ACTIONS(2819),
    [sym__wc_file_write_int] = ACTIONS(2819),
    [sym__wc_file_write_line_int] = ACTIONS(2819),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2821),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2821),
    [anon_sym_LBRACK] = ACTIONS(2821),
    [anon_sym_LBRACE] = ACTIONS(2821),
    [anon_sym_RBRACE] = ACTIONS(2821),
    [anon_sym_LPAREN] = ACTIONS(2819),
    [anon_sym_BANG] = ACTIONS(2821),
    [anon_sym_TILDE] = ACTIONS(2821),
    [anon_sym_DASH] = ACTIONS(2819),
    [anon_sym_PLUS] = ACTIONS(2819),
    [anon_sym_DASH_DASH] = ACTIONS(2821),
    [anon_sym_PLUS_PLUS] = ACTIONS(2821),
    [anon_sym_DQUOTE] = ACTIONS(2821),
    [anon_sym_SQUOTE] = ACTIONS(2821),
    [sym_number_literal] = ACTIONS(2821),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(941)] = {
    [sym_identifier] = ACTIONS(2823),
    [sym_not] = ACTIONS(2823),
    [sym_bnot] = ACTIONS(2823),
    [sym___file__] = ACTIONS(2823),
    [sym___date__] = ACTIONS(2823),
    [sym___ldate__] = ACTIONS(2823),
    [sym___time__] = ACTIONS(2823),
    [sym___line__] = ACTIONS(2823),
    [sym___name__] = ACTIONS(2823),
    [sym___netlinx__] = ACTIONS(2823),
    [sym_netlinx_axi_version] = ACTIONS(2823),
    [sym_url_flg_tcp] = ACTIONS(2823),
    [sym_url_flg_accinfopresent] = ACTIONS(2823),
    [sym_url_flg_temp] = ACTIONS(2823),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2823),
    [sym_url_flg_stat_mask] = ACTIONS(2823),
    [sym_url_flg_stat_lookup] = ACTIONS(2823),
    [sym_url_flg_stat_connecting] = ACTIONS(2823),
    [sym_url_flg_stat_waiting] = ACTIONS(2823),
    [sym_url_flg_stat_connected] = ACTIONS(2823),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2823),
    [sym_first_virtual_device] = ACTIONS(2823),
    [sym_first_local_port] = ACTIONS(2823),
    [sym_dynamic_virtual_device] = ACTIONS(2823),
    [sym_timeline_once] = ACTIONS(2823),
    [sym_timeline_repeat] = ACTIONS(2823),
    [sym_timeline_absolute] = ACTIONS(2823),
    [sym_timeline_relative] = ACTIONS(2823),
    [sym_do_push_timed_infinite] = ACTIONS(2823),
    [sym_source_type_no_address] = ACTIONS(2823),
    [sym_source_type_neuron_id] = ACTIONS(2823),
    [sym_source_type_ip_address] = ACTIONS(2823),
    [sym_source_type_axlink] = ACTIONS(2823),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2823),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2823),
    [sym_source_type_ip_socket_address] = ACTIONS(2823),
    [sym_source_type_rs232] = ACTIONS(2823),
    [sym_source_type_internal] = ACTIONS(2823),
    [sym_source_type_mac_address] = ACTIONS(2823),
    [sym_source_type_ipv4_port] = ACTIONS(2823),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2823),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2823),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2823),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2823),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2823),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2823),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2823),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2823),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2823),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2823),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2823),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2823),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2823),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2823),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2823),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2823),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2823),
    [sym_true] = ACTIONS(2823),
    [sym_false] = ACTIONS(2823),
    [sym_file_read_only] = ACTIONS(2823),
    [sym_file_rw_new] = ACTIONS(2823),
    [sym_file_rw_append] = ACTIONS(2823),
    [sym_ip_tcp] = ACTIONS(2823),
    [sym_ip_udp] = ACTIONS(2823),
    [sym_ip_udp_2way] = ACTIONS(2823),
    [sym_xml_encode_types] = ACTIONS(2823),
    [sym_xml_encode_char_as_list] = ACTIONS(2823),
    [sym_xml_encode_le] = ACTIONS(2823),
    [sym_xml_decode_types] = ACTIONS(2823),
    [sym_xml_decode_no_preserve] = ACTIONS(2823),
    [sym_duet_dev_type_display_device] = ACTIONS(2823),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2823),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2823),
    [sym_duet_dev_type_io_device] = ACTIONS(2823),
    [sym_duet_dev_type_relay_device] = ACTIONS(2823),
    [sym_duet_dev_type_ups] = ACTIONS(2823),
    [sym_duet_dev_type_amplifier] = ACTIONS(2823),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2823),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2823),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2823),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2823),
    [sym_duet_dev_type_camera] = ACTIONS(2823),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2823),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2823),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2823),
    [sym_duet_dev_type_dss] = ACTIONS(2823),
    [sym_duet_dev_type_dvr] = ACTIONS(2823),
    [sym_duet_dev_type_disc_device] = ACTIONS(2823),
    [sym_duet_dev_type_document_camera] = ACTIONS(2823),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2823),
    [sym_duet_dev_type_hvac] = ACTIONS(2823),
    [sym_duet_dev_type_keypad] = ACTIONS(2823),
    [sym_duet_dev_type_light] = ACTIONS(2823),
    [sym_duet_dev_type_monitor] = ACTIONS(2823),
    [sym_duet_dev_type_motor] = ACTIONS(2823),
    [sym_duet_dev_type_multi_window] = ACTIONS(2823),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2823),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2823),
    [sym_duet_dev_type_receiver] = ACTIONS(2823),
    [sym_duet_dev_type_security_system] = ACTIONS(2823),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2823),
    [sym_duet_dev_type_settop_box] = ACTIONS(2823),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2823),
    [sym_duet_dev_type_switcher] = ACTIONS(2823),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2823),
    [sym_duet_dev_type_tv] = ACTIONS(2823),
    [sym_duet_dev_type_utility] = ACTIONS(2823),
    [sym_duet_dev_type_vcr] = ACTIONS(2823),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2823),
    [sym_duet_dev_type_video_processor] = ACTIONS(2823),
    [sym_duet_dev_type_video_projector] = ACTIONS(2823),
    [sym_duet_dev_type_video_wall] = ACTIONS(2823),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2823),
    [sym_duet_dev_type_weather] = ACTIONS(2823),
    [sym_duet_dev_not_polled] = ACTIONS(2823),
    [sym_duet_dev_polled] = ACTIONS(2823),
    [sym_clkmgr_mode_network] = ACTIONS(2823),
    [sym_clkmgr_mode_standalone] = ACTIONS(2823),
    [sym_normal_standby] = ACTIONS(2823),
    [sym_normal_wake] = ACTIONS(2823),
    [sym_ip_multicast_ttl_option] = ACTIONS(2823),
    [sym_ip_ttl_subnet] = ACTIONS(2823),
    [sym_ip_ttl_site] = ACTIONS(2823),
    [sym_ip_ttl_region] = ACTIONS(2823),
    [sym_ip_ttl_continent] = ACTIONS(2823),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2823),
    [sym_ip_nodelay_on] = ACTIONS(2823),
    [sym_ip_nodelay_off] = ACTIONS(2823),
    [sym_amx_error] = ACTIONS(2823),
    [sym_amx_warning] = ACTIONS(2823),
    [sym_amx_info] = ACTIONS(2823),
    [sym_amx_debug] = ACTIONS(2823),
    [sym_netlinx_login_success] = ACTIONS(2823),
    [sym_netlinx_login_fail] = ACTIONS(2823),
    [sym_netlinx_logout] = ACTIONS(2823),
    [sym_null_str] = ACTIONS(2823),
    [sym_smtp_address] = ACTIONS(2823),
    [sym_smtp_port_number] = ACTIONS(2823),
    [sym_smtp_username] = ACTIONS(2823),
    [sym_smtp_password] = ACTIONS(2823),
    [sym_smtp_require_tls] = ACTIONS(2823),
    [sym_smtp_from] = ACTIONS(2823),
    [sym_smtp_tls_true] = ACTIONS(2823),
    [sym_smtp_tls_false] = ACTIONS(2823),
    [sym_listview_on_row_select_event] = ACTIONS(2823),
    [sym_data_structure_unknown] = ACTIONS(2823),
    [sym_data_structure_datafeed] = ACTIONS(2823),
    [sym_data_structure_datarecordset] = ACTIONS(2823),
    [sym_data_structure_datarecord] = ACTIONS(2823),
    [sym_data_structure_datafield] = ACTIONS(2823),
    [sym_data_max_content_field_count] = ACTIONS(2823),
    [sym_data_max_metadata_field_count] = ACTIONS(2823),
    [sym_data_max_id_length] = ACTIONS(2823),
    [sym_data_max_name_length] = ACTIONS(2823),
    [sym_data_max_description_length] = ACTIONS(2823),
    [sym_data_max_type_length] = ACTIONS(2823),
    [sym_data_max_format_length] = ACTIONS(2823),
    [sym_data_max_source_length] = ACTIONS(2823),
    [sym_data_max_value_length] = ACTIONS(2823),
    [sym_data_max_label_length] = ACTIONS(2823),
    [sym_data_max_url_length] = ACTIONS(2823),
    [sym_max_last_login_info_length] = ACTIONS(2823),
    [sym_data_type_unknown] = ACTIONS(2823),
    [sym_data_type_string] = ACTIONS(2823),
    [sym_data_type_datetime] = ACTIONS(2823),
    [sym_data_type_date] = ACTIONS(2823),
    [sym_data_type_time] = ACTIONS(2823),
    [sym_data_type_image] = ACTIONS(2823),
    [sym_data_format_url] = ACTIONS(2823),
    [sym_data_format_phone] = ACTIONS(2823),
    [sym_data_format_email] = ACTIONS(2823),
    [sym_data_format_iso8601] = ACTIONS(2823),
    [sym_auditlog_priv] = ACTIONS(2823),
    [sym_configuration_priv] = ACTIONS(2823),
    [sym_deviceconfig_priv] = ACTIONS(2823),
    [sym_ftp_priv] = ACTIONS(2823),
    [sym_http_priv] = ACTIONS(2823),
    [sym_networkconfig_priv] = ACTIONS(2823),
    [sym_programport_priv] = ACTIONS(2823),
    [sym_remoteui_priv] = ACTIONS(2823),
    [sym_securitycontrol_priv] = ACTIONS(2823),
    [sym_softwaremanagement_priv] = ACTIONS(2823),
    [sym_terminal_priv] = ACTIONS(2823),
    [sym_tpadmin_priv] = ACTIONS(2823),
    [sym_user1_priv] = ACTIONS(2823),
    [sym_user2_priv] = ACTIONS(2823),
    [sym_user3_priv] = ACTIONS(2823),
    [sym_user4_priv] = ACTIONS(2823),
    [sym_usermanagement_priv] = ACTIONS(2823),
    [sym_valid_account] = ACTIONS(2823),
    [sym_err_name_invalid] = ACTIONS(2823),
    [sym_err_password_invalid] = ACTIONS(2823),
    [sym_err_invalid_account] = ACTIONS(2823),
    [sym_err_no_authorization] = ACTIONS(2823),
    [sym_err_invalid_parameter] = ACTIONS(2823),
    [sym_err_user_locked_out] = ACTIONS(2823),
    [sym_err_user_account_expired] = ACTIONS(2823),
    [sym_err_general_error] = ACTIONS(2823),
    [sym_err_authentication_error] = ACTIONS(2823),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2823),
    [sym_tls_validate_certificate] = ACTIONS(2823),
    [sym____reserved___] = ACTIONS(2823),
    [sym_date] = ACTIONS(2823),
    [sym_day] = ACTIONS(2823),
    [sym_time] = ACTIONS(2823),
    [sym_ldate] = ACTIONS(2823),
    [sym_push_device] = ACTIONS(2823),
    [sym_push_channel] = ACTIONS(2823),
    [sym_push_devchan] = ACTIONS(2823),
    [sym_release_device] = ACTIONS(2823),
    [sym_release_channel] = ACTIONS(2823),
    [sym_release_devchan] = ACTIONS(2823),
    [sym_master_slot] = ACTIONS(2823),
    [sym_get_pulse_time] = ACTIONS(2823),
    [sym_get_timer] = ACTIONS(2823),
    [sym_master_sn] = ACTIONS(2823),
    [sym_system_number] = ACTIONS(2823),
    [sym_button] = ACTIONS(2823),
    [sym_dv_channel] = ACTIONS(2823),
    [sym_data] = ACTIONS(2823),
    [sym_level] = ACTIONS(2823),
    [sym_timeline] = ACTIONS(2823),
    [sym_channel] = ACTIONS(2823),
    [sym_custom] = ACTIONS(2823),
    [sym_atoi] = ACTIONS(2823),
    [sym_atol] = ACTIONS(2823),
    [sym_atof] = ACTIONS(2823),
    [sym_itoa] = ACTIONS(2823),
    [sym_itohex] = ACTIONS(2823),
    [sym_hextoi] = ACTIONS(2823),
    [sym_ftoa] = ACTIONS(2823),
    [sym_find_string] = ACTIONS(2823),
    [sym_left_string] = ACTIONS(2823),
    [sym_right_string] = ACTIONS(2823),
    [sym_mid_string] = ACTIONS(2823),
    [sym_remove_string] = ACTIONS(2823),
    [sym_get_buffer_string] = ACTIONS(2823),
    [sym_get_buffer_char] = ACTIONS(2823),
    [sym_get_multi_buffer_string] = ACTIONS(2823),
    [sym_length_string] = ACTIONS(2823),
    [sym_max_length_string] = ACTIONS(2823),
    [sym_set_length_string] = ACTIONS(2823),
    [sym_compare_string] = ACTIONS(2823),
    [sym_upper_string] = ACTIONS(2823),
    [sym_lower_string] = ACTIONS(2823),
    [sym_get_last] = ACTIONS(2823),
    [sym_date_to_day] = ACTIONS(2823),
    [sym_date_to_month] = ACTIONS(2823),
    [sym_date_to_year] = ACTIONS(2823),
    [sym_time_to_hour] = ACTIONS(2823),
    [sym_time_to_minute] = ACTIONS(2823),
    [sym_time_to_second] = ACTIONS(2823),
    [sym_day_of_week] = ACTIONS(2823),
    [sym_set_pulse_time] = ACTIONS(2823),
    [sym_set_timer] = ACTIONS(2823),
    [sym_file_open] = ACTIONS(2823),
    [sym_file_close] = ACTIONS(2823),
    [sym_file_read] = ACTIONS(2823),
    [sym_file_read_line] = ACTIONS(2823),
    [sym_file_write] = ACTIONS(2823),
    [sym_file_write_line] = ACTIONS(2823),
    [sym_file_copy] = ACTIONS(2823),
    [sym_file_delete] = ACTIONS(2823),
    [sym_file_rename] = ACTIONS(2823),
    [sym_file_dir] = ACTIONS(2823),
    [sym_file_getdir] = ACTIONS(2823),
    [sym_file_setdir] = ACTIONS(2823),
    [sym_file_create_dir] = ACTIONS(2823),
    [sym_file_remove_dir] = ACTIONS(2823),
    [sym_file_seek] = ACTIONS(2823),
    [sym_length_array] = ACTIONS(2823),
    [sym_max_length_array] = ACTIONS(2823),
    [sym_set_length_array] = ACTIONS(2823),
    [sym_ip_client_open] = ACTIONS(2823),
    [sym_ip_client_close] = ACTIONS(2823),
    [sym_ip_server_open] = ACTIONS(2823),
    [sym_ip_server_close] = ACTIONS(2823),
    [sym_ip_bound_client_open] = ACTIONS(2823),
    [sym_ip_mc_server_open] = ACTIONS(2823),
    [sym_ip_set_option] = ACTIONS(2823),
    [sym_ssh_client_open] = ACTIONS(2823),
    [sym_ssh_client_close] = ACTIONS(2823),
    [sym_tls_client_open] = ACTIONS(2823),
    [sym_tls_client_close] = ACTIONS(2823),
    [sym_do_push] = ACTIONS(2823),
    [sym_do_push_timed] = ACTIONS(2823),
    [sym_do_release] = ACTIONS(2823),
    [sym_device_id] = ACTIONS(2823),
    [sym_device_info] = ACTIONS(2823),
    [sym_device_id_string] = ACTIONS(2823),
    [sym_device_standby] = ACTIONS(2823),
    [sym_device_wake] = ACTIONS(2823),
    [sym_timeline_create] = ACTIONS(2823),
    [sym_timeline_kill] = ACTIONS(2823),
    [sym_timeline_reload] = ACTIONS(2823),
    [sym_timeline_pause] = ACTIONS(2823),
    [sym_timeline_restart] = ACTIONS(2823),
    [sym_timeline_set] = ACTIONS(2823),
    [sym_timeline_get] = ACTIONS(2823),
    [sym_timeline_active] = ACTIONS(2823),
    [sym_timeline_dynamic_id] = ACTIONS(2823),
    [sym_variable_to_string] = ACTIONS(2823),
    [sym_string_to_variable] = ACTIONS(2823),
    [sym_length_variable_to_string] = ACTIONS(2823),
    [sym_type_cast] = ACTIONS(2823),
    [sym_variable_to_xml] = ACTIONS(2823),
    [sym_xml_to_variable] = ACTIONS(2823),
    [sym_length_variable_to_xml] = ACTIONS(2823),
    [sym_redirect_string] = ACTIONS(2823),
    [sym_random_number] = ACTIONS(2823),
    [sym_get_unique_id] = ACTIONS(2823),
    [sym_get_system_number] = ACTIONS(2823),
    [sym_set_system_number] = ACTIONS(2823),
    [sym_get_serial_number] = ACTIONS(2823),
    [sym_get_master_build] = ACTIONS(2823),
    [sym_set_validation_code] = ACTIONS(2823),
    [sym_reboot] = ACTIONS(2823),
    [sym_load_duet_module] = ACTIONS(2823),
    [sym_unload_duet_module] = ACTIONS(2823),
    [sym_duet_mem_size_set] = ACTIONS(2823),
    [sym_duet_mem_size_get] = ACTIONS(2823),
    [sym_dynamic_application_device] = ACTIONS(2823),
    [sym_dynamic_polled_port] = ACTIONS(2823),
    [sym_static_port_binding] = ACTIONS(2823),
    [sym_static_ip_binding] = ACTIONS(2823),
    [sym_get_dns_list] = ACTIONS(2823),
    [sym_set_dns_list] = ACTIONS(2823),
    [sym_get_ip_address] = ACTIONS(2823),
    [sym_set_ip_address] = ACTIONS(2823),
    [sym_get_url_list] = ACTIONS(2823),
    [sym_add_url_entry] = ACTIONS(2823),
    [sym_delete_url_entry] = ACTIONS(2823),
    [sym_abs_value] = ACTIONS(2823),
    [sym_max_value] = ACTIONS(2823),
    [sym_min_value] = ACTIONS(2823),
    [sym_exp_value] = ACTIONS(2823),
    [sym_log_value] = ACTIONS(2823),
    [sym_log10_value] = ACTIONS(2823),
    [sym_power_value] = ACTIONS(2823),
    [sym_sqrt_value] = ACTIONS(2823),
    [sym_raw_be] = ACTIONS(2823),
    [sym_raw_le] = ACTIONS(2823),
    [sym_internal_queue_size_set] = ACTIONS(2823),
    [sym_internal_queue_size_get] = ACTIONS(2823),
    [sym_internal_threshold_set] = ACTIONS(2823),
    [sym_internal_threshold_get] = ACTIONS(2823),
    [sym_rebuilt_event] = ACTIONS(2823),
    [sym_do_custom_event] = ACTIONS(2823),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2823),
    [sym_clkmgr_set_clk_source] = ACTIONS(2823),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2823),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2823),
    [sym_clkmgr_get_timezone] = ACTIONS(2823),
    [sym_clkmgr_set_timezone] = ACTIONS(2823),
    [sym_clkmgr_get_resync_period] = ACTIONS(2823),
    [sym_clkmgr_set_resync_period] = ACTIONS(2823),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2823),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2823),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2823),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2823),
    [sym_clkmgr_get_timeservers] = ACTIONS(2823),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2823),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2823),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2823),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2823),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2823),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2823),
    [sym_set_log_level] = ACTIONS(2823),
    [sym_get_log_level] = ACTIONS(2823),
    [sym_amx_log] = ACTIONS(2823),
    [sym_smtp_server_config_set] = ACTIONS(2823),
    [sym_smtp_server_config_get] = ACTIONS(2823),
    [sym_smtp_send] = ACTIONS(2823),
    [sym_validate_netlinx_account] = ACTIONS(2823),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2823),
    [sym_authenticate_certificate] = ACTIONS(2823),
    [sym_audit_netlinx_session_event] = ACTIONS(2823),
    [sym_audit_netlinx_generic_event] = ACTIONS(2823),
    [sym_get_available_flash_disk_space] = ACTIONS(2823),
    [sym_get_max_flash_disk_space] = ACTIONS(2823),
    [sym_data_create_feed] = ACTIONS(2823),
    [sym_data_delete_feed] = ACTIONS(2823),
    [sym_data_publish_feed] = ACTIONS(2823),
    [sym_data_get_published_feed] = ACTIONS(2823),
    [sym_data_add_record] = ACTIONS(2823),
    [sym_data_get_event_record] = ACTIONS(2823),
    [sym__wc_data_create_feed] = ACTIONS(2823),
    [sym__wc_data_add_record] = ACTIONS(2823),
    [sym__wc_data_get_event_record] = ACTIONS(2823),
    [sym_format] = ACTIONS(2823),
    [sym_astro_clock] = ACTIONS(2823),
    [sym_set_outdoor_temperature] = ACTIONS(2823),
    [sym_set_virtual_level_count] = ACTIONS(2823),
    [sym_set_virtual_channel_count] = ACTIONS(2823),
    [sym_set_virtual_port_count] = ACTIONS(2823),
    [sym__wc_encode_int] = ACTIONS(2823),
    [sym__wc_decode_int] = ACTIONS(2823),
    [sym__wc_int] = ACTIONS(2823),
    [sym__wc_to_ch_int] = ACTIONS(2823),
    [sym__ch_to_wc_int] = ACTIONS(2823),
    [sym__wc_find_string_int] = ACTIONS(2823),
    [sym__wc_left_string_int] = ACTIONS(2823),
    [sym__wc_lower_string_int] = ACTIONS(2823),
    [sym__wc_mid_string_int] = ACTIONS(2823),
    [sym__wc_remove_string_int] = ACTIONS(2823),
    [sym__wc_right_string_int] = ACTIONS(2823),
    [sym__wc_upper_string_int] = ACTIONS(2823),
    [sym__wc_compare_string_int] = ACTIONS(2823),
    [sym__wc_get_buffer_char_int] = ACTIONS(2823),
    [sym__wc_get_buffer_string_int] = ACTIONS(2823),
    [sym__wc_concat_string_int] = ACTIONS(2823),
    [sym__wc_file_open_int] = ACTIONS(2823),
    [sym__wc_file_close_int] = ACTIONS(2823),
    [sym__wc_file_read_int] = ACTIONS(2823),
    [sym__wc_file_read_line_int] = ACTIONS(2823),
    [sym__wc_file_write_int] = ACTIONS(2823),
    [sym__wc_file_write_line_int] = ACTIONS(2823),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2825),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2825),
    [anon_sym_LBRACK] = ACTIONS(2825),
    [anon_sym_LBRACE] = ACTIONS(2825),
    [anon_sym_RBRACE] = ACTIONS(2825),
    [anon_sym_LPAREN] = ACTIONS(2823),
    [anon_sym_BANG] = ACTIONS(2825),
    [anon_sym_TILDE] = ACTIONS(2825),
    [anon_sym_DASH] = ACTIONS(2823),
    [anon_sym_PLUS] = ACTIONS(2823),
    [anon_sym_DASH_DASH] = ACTIONS(2825),
    [anon_sym_PLUS_PLUS] = ACTIONS(2825),
    [anon_sym_DQUOTE] = ACTIONS(2825),
    [anon_sym_SQUOTE] = ACTIONS(2825),
    [sym_number_literal] = ACTIONS(2825),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(942)] = {
    [sym_identifier] = ACTIONS(2827),
    [sym_not] = ACTIONS(2827),
    [sym_bnot] = ACTIONS(2827),
    [sym___file__] = ACTIONS(2827),
    [sym___date__] = ACTIONS(2827),
    [sym___ldate__] = ACTIONS(2827),
    [sym___time__] = ACTIONS(2827),
    [sym___line__] = ACTIONS(2827),
    [sym___name__] = ACTIONS(2827),
    [sym___netlinx__] = ACTIONS(2827),
    [sym_netlinx_axi_version] = ACTIONS(2827),
    [sym_url_flg_tcp] = ACTIONS(2827),
    [sym_url_flg_accinfopresent] = ACTIONS(2827),
    [sym_url_flg_temp] = ACTIONS(2827),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2827),
    [sym_url_flg_stat_mask] = ACTIONS(2827),
    [sym_url_flg_stat_lookup] = ACTIONS(2827),
    [sym_url_flg_stat_connecting] = ACTIONS(2827),
    [sym_url_flg_stat_waiting] = ACTIONS(2827),
    [sym_url_flg_stat_connected] = ACTIONS(2827),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2827),
    [sym_first_virtual_device] = ACTIONS(2827),
    [sym_first_local_port] = ACTIONS(2827),
    [sym_dynamic_virtual_device] = ACTIONS(2827),
    [sym_timeline_once] = ACTIONS(2827),
    [sym_timeline_repeat] = ACTIONS(2827),
    [sym_timeline_absolute] = ACTIONS(2827),
    [sym_timeline_relative] = ACTIONS(2827),
    [sym_do_push_timed_infinite] = ACTIONS(2827),
    [sym_source_type_no_address] = ACTIONS(2827),
    [sym_source_type_neuron_id] = ACTIONS(2827),
    [sym_source_type_ip_address] = ACTIONS(2827),
    [sym_source_type_axlink] = ACTIONS(2827),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2827),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2827),
    [sym_source_type_ip_socket_address] = ACTIONS(2827),
    [sym_source_type_rs232] = ACTIONS(2827),
    [sym_source_type_internal] = ACTIONS(2827),
    [sym_source_type_mac_address] = ACTIONS(2827),
    [sym_source_type_ipv4_port] = ACTIONS(2827),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2827),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2827),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2827),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2827),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2827),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2827),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2827),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2827),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2827),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2827),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2827),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2827),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2827),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2827),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2827),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2827),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2827),
    [sym_true] = ACTIONS(2827),
    [sym_false] = ACTIONS(2827),
    [sym_file_read_only] = ACTIONS(2827),
    [sym_file_rw_new] = ACTIONS(2827),
    [sym_file_rw_append] = ACTIONS(2827),
    [sym_ip_tcp] = ACTIONS(2827),
    [sym_ip_udp] = ACTIONS(2827),
    [sym_ip_udp_2way] = ACTIONS(2827),
    [sym_xml_encode_types] = ACTIONS(2827),
    [sym_xml_encode_char_as_list] = ACTIONS(2827),
    [sym_xml_encode_le] = ACTIONS(2827),
    [sym_xml_decode_types] = ACTIONS(2827),
    [sym_xml_decode_no_preserve] = ACTIONS(2827),
    [sym_duet_dev_type_display_device] = ACTIONS(2827),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2827),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2827),
    [sym_duet_dev_type_io_device] = ACTIONS(2827),
    [sym_duet_dev_type_relay_device] = ACTIONS(2827),
    [sym_duet_dev_type_ups] = ACTIONS(2827),
    [sym_duet_dev_type_amplifier] = ACTIONS(2827),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2827),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2827),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2827),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2827),
    [sym_duet_dev_type_camera] = ACTIONS(2827),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2827),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2827),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2827),
    [sym_duet_dev_type_dss] = ACTIONS(2827),
    [sym_duet_dev_type_dvr] = ACTIONS(2827),
    [sym_duet_dev_type_disc_device] = ACTIONS(2827),
    [sym_duet_dev_type_document_camera] = ACTIONS(2827),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2827),
    [sym_duet_dev_type_hvac] = ACTIONS(2827),
    [sym_duet_dev_type_keypad] = ACTIONS(2827),
    [sym_duet_dev_type_light] = ACTIONS(2827),
    [sym_duet_dev_type_monitor] = ACTIONS(2827),
    [sym_duet_dev_type_motor] = ACTIONS(2827),
    [sym_duet_dev_type_multi_window] = ACTIONS(2827),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2827),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2827),
    [sym_duet_dev_type_receiver] = ACTIONS(2827),
    [sym_duet_dev_type_security_system] = ACTIONS(2827),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2827),
    [sym_duet_dev_type_settop_box] = ACTIONS(2827),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2827),
    [sym_duet_dev_type_switcher] = ACTIONS(2827),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2827),
    [sym_duet_dev_type_tv] = ACTIONS(2827),
    [sym_duet_dev_type_utility] = ACTIONS(2827),
    [sym_duet_dev_type_vcr] = ACTIONS(2827),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2827),
    [sym_duet_dev_type_video_processor] = ACTIONS(2827),
    [sym_duet_dev_type_video_projector] = ACTIONS(2827),
    [sym_duet_dev_type_video_wall] = ACTIONS(2827),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2827),
    [sym_duet_dev_type_weather] = ACTIONS(2827),
    [sym_duet_dev_not_polled] = ACTIONS(2827),
    [sym_duet_dev_polled] = ACTIONS(2827),
    [sym_clkmgr_mode_network] = ACTIONS(2827),
    [sym_clkmgr_mode_standalone] = ACTIONS(2827),
    [sym_normal_standby] = ACTIONS(2827),
    [sym_normal_wake] = ACTIONS(2827),
    [sym_ip_multicast_ttl_option] = ACTIONS(2827),
    [sym_ip_ttl_subnet] = ACTIONS(2827),
    [sym_ip_ttl_site] = ACTIONS(2827),
    [sym_ip_ttl_region] = ACTIONS(2827),
    [sym_ip_ttl_continent] = ACTIONS(2827),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2827),
    [sym_ip_nodelay_on] = ACTIONS(2827),
    [sym_ip_nodelay_off] = ACTIONS(2827),
    [sym_amx_error] = ACTIONS(2827),
    [sym_amx_warning] = ACTIONS(2827),
    [sym_amx_info] = ACTIONS(2827),
    [sym_amx_debug] = ACTIONS(2827),
    [sym_netlinx_login_success] = ACTIONS(2827),
    [sym_netlinx_login_fail] = ACTIONS(2827),
    [sym_netlinx_logout] = ACTIONS(2827),
    [sym_null_str] = ACTIONS(2827),
    [sym_smtp_address] = ACTIONS(2827),
    [sym_smtp_port_number] = ACTIONS(2827),
    [sym_smtp_username] = ACTIONS(2827),
    [sym_smtp_password] = ACTIONS(2827),
    [sym_smtp_require_tls] = ACTIONS(2827),
    [sym_smtp_from] = ACTIONS(2827),
    [sym_smtp_tls_true] = ACTIONS(2827),
    [sym_smtp_tls_false] = ACTIONS(2827),
    [sym_listview_on_row_select_event] = ACTIONS(2827),
    [sym_data_structure_unknown] = ACTIONS(2827),
    [sym_data_structure_datafeed] = ACTIONS(2827),
    [sym_data_structure_datarecordset] = ACTIONS(2827),
    [sym_data_structure_datarecord] = ACTIONS(2827),
    [sym_data_structure_datafield] = ACTIONS(2827),
    [sym_data_max_content_field_count] = ACTIONS(2827),
    [sym_data_max_metadata_field_count] = ACTIONS(2827),
    [sym_data_max_id_length] = ACTIONS(2827),
    [sym_data_max_name_length] = ACTIONS(2827),
    [sym_data_max_description_length] = ACTIONS(2827),
    [sym_data_max_type_length] = ACTIONS(2827),
    [sym_data_max_format_length] = ACTIONS(2827),
    [sym_data_max_source_length] = ACTIONS(2827),
    [sym_data_max_value_length] = ACTIONS(2827),
    [sym_data_max_label_length] = ACTIONS(2827),
    [sym_data_max_url_length] = ACTIONS(2827),
    [sym_max_last_login_info_length] = ACTIONS(2827),
    [sym_data_type_unknown] = ACTIONS(2827),
    [sym_data_type_string] = ACTIONS(2827),
    [sym_data_type_datetime] = ACTIONS(2827),
    [sym_data_type_date] = ACTIONS(2827),
    [sym_data_type_time] = ACTIONS(2827),
    [sym_data_type_image] = ACTIONS(2827),
    [sym_data_format_url] = ACTIONS(2827),
    [sym_data_format_phone] = ACTIONS(2827),
    [sym_data_format_email] = ACTIONS(2827),
    [sym_data_format_iso8601] = ACTIONS(2827),
    [sym_auditlog_priv] = ACTIONS(2827),
    [sym_configuration_priv] = ACTIONS(2827),
    [sym_deviceconfig_priv] = ACTIONS(2827),
    [sym_ftp_priv] = ACTIONS(2827),
    [sym_http_priv] = ACTIONS(2827),
    [sym_networkconfig_priv] = ACTIONS(2827),
    [sym_programport_priv] = ACTIONS(2827),
    [sym_remoteui_priv] = ACTIONS(2827),
    [sym_securitycontrol_priv] = ACTIONS(2827),
    [sym_softwaremanagement_priv] = ACTIONS(2827),
    [sym_terminal_priv] = ACTIONS(2827),
    [sym_tpadmin_priv] = ACTIONS(2827),
    [sym_user1_priv] = ACTIONS(2827),
    [sym_user2_priv] = ACTIONS(2827),
    [sym_user3_priv] = ACTIONS(2827),
    [sym_user4_priv] = ACTIONS(2827),
    [sym_usermanagement_priv] = ACTIONS(2827),
    [sym_valid_account] = ACTIONS(2827),
    [sym_err_name_invalid] = ACTIONS(2827),
    [sym_err_password_invalid] = ACTIONS(2827),
    [sym_err_invalid_account] = ACTIONS(2827),
    [sym_err_no_authorization] = ACTIONS(2827),
    [sym_err_invalid_parameter] = ACTIONS(2827),
    [sym_err_user_locked_out] = ACTIONS(2827),
    [sym_err_user_account_expired] = ACTIONS(2827),
    [sym_err_general_error] = ACTIONS(2827),
    [sym_err_authentication_error] = ACTIONS(2827),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2827),
    [sym_tls_validate_certificate] = ACTIONS(2827),
    [sym____reserved___] = ACTIONS(2827),
    [sym_date] = ACTIONS(2827),
    [sym_day] = ACTIONS(2827),
    [sym_time] = ACTIONS(2827),
    [sym_ldate] = ACTIONS(2827),
    [sym_push_device] = ACTIONS(2827),
    [sym_push_channel] = ACTIONS(2827),
    [sym_push_devchan] = ACTIONS(2827),
    [sym_release_device] = ACTIONS(2827),
    [sym_release_channel] = ACTIONS(2827),
    [sym_release_devchan] = ACTIONS(2827),
    [sym_master_slot] = ACTIONS(2827),
    [sym_get_pulse_time] = ACTIONS(2827),
    [sym_get_timer] = ACTIONS(2827),
    [sym_master_sn] = ACTIONS(2827),
    [sym_system_number] = ACTIONS(2827),
    [sym_button] = ACTIONS(2827),
    [sym_dv_channel] = ACTIONS(2827),
    [sym_data] = ACTIONS(2827),
    [sym_level] = ACTIONS(2827),
    [sym_timeline] = ACTIONS(2827),
    [sym_channel] = ACTIONS(2827),
    [sym_custom] = ACTIONS(2827),
    [sym_atoi] = ACTIONS(2827),
    [sym_atol] = ACTIONS(2827),
    [sym_atof] = ACTIONS(2827),
    [sym_itoa] = ACTIONS(2827),
    [sym_itohex] = ACTIONS(2827),
    [sym_hextoi] = ACTIONS(2827),
    [sym_ftoa] = ACTIONS(2827),
    [sym_find_string] = ACTIONS(2827),
    [sym_left_string] = ACTIONS(2827),
    [sym_right_string] = ACTIONS(2827),
    [sym_mid_string] = ACTIONS(2827),
    [sym_remove_string] = ACTIONS(2827),
    [sym_get_buffer_string] = ACTIONS(2827),
    [sym_get_buffer_char] = ACTIONS(2827),
    [sym_get_multi_buffer_string] = ACTIONS(2827),
    [sym_length_string] = ACTIONS(2827),
    [sym_max_length_string] = ACTIONS(2827),
    [sym_set_length_string] = ACTIONS(2827),
    [sym_compare_string] = ACTIONS(2827),
    [sym_upper_string] = ACTIONS(2827),
    [sym_lower_string] = ACTIONS(2827),
    [sym_get_last] = ACTIONS(2827),
    [sym_date_to_day] = ACTIONS(2827),
    [sym_date_to_month] = ACTIONS(2827),
    [sym_date_to_year] = ACTIONS(2827),
    [sym_time_to_hour] = ACTIONS(2827),
    [sym_time_to_minute] = ACTIONS(2827),
    [sym_time_to_second] = ACTIONS(2827),
    [sym_day_of_week] = ACTIONS(2827),
    [sym_set_pulse_time] = ACTIONS(2827),
    [sym_set_timer] = ACTIONS(2827),
    [sym_file_open] = ACTIONS(2827),
    [sym_file_close] = ACTIONS(2827),
    [sym_file_read] = ACTIONS(2827),
    [sym_file_read_line] = ACTIONS(2827),
    [sym_file_write] = ACTIONS(2827),
    [sym_file_write_line] = ACTIONS(2827),
    [sym_file_copy] = ACTIONS(2827),
    [sym_file_delete] = ACTIONS(2827),
    [sym_file_rename] = ACTIONS(2827),
    [sym_file_dir] = ACTIONS(2827),
    [sym_file_getdir] = ACTIONS(2827),
    [sym_file_setdir] = ACTIONS(2827),
    [sym_file_create_dir] = ACTIONS(2827),
    [sym_file_remove_dir] = ACTIONS(2827),
    [sym_file_seek] = ACTIONS(2827),
    [sym_length_array] = ACTIONS(2827),
    [sym_max_length_array] = ACTIONS(2827),
    [sym_set_length_array] = ACTIONS(2827),
    [sym_ip_client_open] = ACTIONS(2827),
    [sym_ip_client_close] = ACTIONS(2827),
    [sym_ip_server_open] = ACTIONS(2827),
    [sym_ip_server_close] = ACTIONS(2827),
    [sym_ip_bound_client_open] = ACTIONS(2827),
    [sym_ip_mc_server_open] = ACTIONS(2827),
    [sym_ip_set_option] = ACTIONS(2827),
    [sym_ssh_client_open] = ACTIONS(2827),
    [sym_ssh_client_close] = ACTIONS(2827),
    [sym_tls_client_open] = ACTIONS(2827),
    [sym_tls_client_close] = ACTIONS(2827),
    [sym_do_push] = ACTIONS(2827),
    [sym_do_push_timed] = ACTIONS(2827),
    [sym_do_release] = ACTIONS(2827),
    [sym_device_id] = ACTIONS(2827),
    [sym_device_info] = ACTIONS(2827),
    [sym_device_id_string] = ACTIONS(2827),
    [sym_device_standby] = ACTIONS(2827),
    [sym_device_wake] = ACTIONS(2827),
    [sym_timeline_create] = ACTIONS(2827),
    [sym_timeline_kill] = ACTIONS(2827),
    [sym_timeline_reload] = ACTIONS(2827),
    [sym_timeline_pause] = ACTIONS(2827),
    [sym_timeline_restart] = ACTIONS(2827),
    [sym_timeline_set] = ACTIONS(2827),
    [sym_timeline_get] = ACTIONS(2827),
    [sym_timeline_active] = ACTIONS(2827),
    [sym_timeline_dynamic_id] = ACTIONS(2827),
    [sym_variable_to_string] = ACTIONS(2827),
    [sym_string_to_variable] = ACTIONS(2827),
    [sym_length_variable_to_string] = ACTIONS(2827),
    [sym_type_cast] = ACTIONS(2827),
    [sym_variable_to_xml] = ACTIONS(2827),
    [sym_xml_to_variable] = ACTIONS(2827),
    [sym_length_variable_to_xml] = ACTIONS(2827),
    [sym_redirect_string] = ACTIONS(2827),
    [sym_random_number] = ACTIONS(2827),
    [sym_get_unique_id] = ACTIONS(2827),
    [sym_get_system_number] = ACTIONS(2827),
    [sym_set_system_number] = ACTIONS(2827),
    [sym_get_serial_number] = ACTIONS(2827),
    [sym_get_master_build] = ACTIONS(2827),
    [sym_set_validation_code] = ACTIONS(2827),
    [sym_reboot] = ACTIONS(2827),
    [sym_load_duet_module] = ACTIONS(2827),
    [sym_unload_duet_module] = ACTIONS(2827),
    [sym_duet_mem_size_set] = ACTIONS(2827),
    [sym_duet_mem_size_get] = ACTIONS(2827),
    [sym_dynamic_application_device] = ACTIONS(2827),
    [sym_dynamic_polled_port] = ACTIONS(2827),
    [sym_static_port_binding] = ACTIONS(2827),
    [sym_static_ip_binding] = ACTIONS(2827),
    [sym_get_dns_list] = ACTIONS(2827),
    [sym_set_dns_list] = ACTIONS(2827),
    [sym_get_ip_address] = ACTIONS(2827),
    [sym_set_ip_address] = ACTIONS(2827),
    [sym_get_url_list] = ACTIONS(2827),
    [sym_add_url_entry] = ACTIONS(2827),
    [sym_delete_url_entry] = ACTIONS(2827),
    [sym_abs_value] = ACTIONS(2827),
    [sym_max_value] = ACTIONS(2827),
    [sym_min_value] = ACTIONS(2827),
    [sym_exp_value] = ACTIONS(2827),
    [sym_log_value] = ACTIONS(2827),
    [sym_log10_value] = ACTIONS(2827),
    [sym_power_value] = ACTIONS(2827),
    [sym_sqrt_value] = ACTIONS(2827),
    [sym_raw_be] = ACTIONS(2827),
    [sym_raw_le] = ACTIONS(2827),
    [sym_internal_queue_size_set] = ACTIONS(2827),
    [sym_internal_queue_size_get] = ACTIONS(2827),
    [sym_internal_threshold_set] = ACTIONS(2827),
    [sym_internal_threshold_get] = ACTIONS(2827),
    [sym_rebuilt_event] = ACTIONS(2827),
    [sym_do_custom_event] = ACTIONS(2827),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2827),
    [sym_clkmgr_set_clk_source] = ACTIONS(2827),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2827),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2827),
    [sym_clkmgr_get_timezone] = ACTIONS(2827),
    [sym_clkmgr_set_timezone] = ACTIONS(2827),
    [sym_clkmgr_get_resync_period] = ACTIONS(2827),
    [sym_clkmgr_set_resync_period] = ACTIONS(2827),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2827),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2827),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2827),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2827),
    [sym_clkmgr_get_timeservers] = ACTIONS(2827),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2827),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2827),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2827),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2827),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2827),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2827),
    [sym_set_log_level] = ACTIONS(2827),
    [sym_get_log_level] = ACTIONS(2827),
    [sym_amx_log] = ACTIONS(2827),
    [sym_smtp_server_config_set] = ACTIONS(2827),
    [sym_smtp_server_config_get] = ACTIONS(2827),
    [sym_smtp_send] = ACTIONS(2827),
    [sym_validate_netlinx_account] = ACTIONS(2827),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2827),
    [sym_authenticate_certificate] = ACTIONS(2827),
    [sym_audit_netlinx_session_event] = ACTIONS(2827),
    [sym_audit_netlinx_generic_event] = ACTIONS(2827),
    [sym_get_available_flash_disk_space] = ACTIONS(2827),
    [sym_get_max_flash_disk_space] = ACTIONS(2827),
    [sym_data_create_feed] = ACTIONS(2827),
    [sym_data_delete_feed] = ACTIONS(2827),
    [sym_data_publish_feed] = ACTIONS(2827),
    [sym_data_get_published_feed] = ACTIONS(2827),
    [sym_data_add_record] = ACTIONS(2827),
    [sym_data_get_event_record] = ACTIONS(2827),
    [sym__wc_data_create_feed] = ACTIONS(2827),
    [sym__wc_data_add_record] = ACTIONS(2827),
    [sym__wc_data_get_event_record] = ACTIONS(2827),
    [sym_format] = ACTIONS(2827),
    [sym_astro_clock] = ACTIONS(2827),
    [sym_set_outdoor_temperature] = ACTIONS(2827),
    [sym_set_virtual_level_count] = ACTIONS(2827),
    [sym_set_virtual_channel_count] = ACTIONS(2827),
    [sym_set_virtual_port_count] = ACTIONS(2827),
    [sym__wc_encode_int] = ACTIONS(2827),
    [sym__wc_decode_int] = ACTIONS(2827),
    [sym__wc_int] = ACTIONS(2827),
    [sym__wc_to_ch_int] = ACTIONS(2827),
    [sym__ch_to_wc_int] = ACTIONS(2827),
    [sym__wc_find_string_int] = ACTIONS(2827),
    [sym__wc_left_string_int] = ACTIONS(2827),
    [sym__wc_lower_string_int] = ACTIONS(2827),
    [sym__wc_mid_string_int] = ACTIONS(2827),
    [sym__wc_remove_string_int] = ACTIONS(2827),
    [sym__wc_right_string_int] = ACTIONS(2827),
    [sym__wc_upper_string_int] = ACTIONS(2827),
    [sym__wc_compare_string_int] = ACTIONS(2827),
    [sym__wc_get_buffer_char_int] = ACTIONS(2827),
    [sym__wc_get_buffer_string_int] = ACTIONS(2827),
    [sym__wc_concat_string_int] = ACTIONS(2827),
    [sym__wc_file_open_int] = ACTIONS(2827),
    [sym__wc_file_close_int] = ACTIONS(2827),
    [sym__wc_file_read_int] = ACTIONS(2827),
    [sym__wc_file_read_line_int] = ACTIONS(2827),
    [sym__wc_file_write_int] = ACTIONS(2827),
    [sym__wc_file_write_line_int] = ACTIONS(2827),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2829),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2829),
    [anon_sym_LBRACK] = ACTIONS(2829),
    [anon_sym_LBRACE] = ACTIONS(2829),
    [anon_sym_RBRACE] = ACTIONS(2829),
    [anon_sym_LPAREN] = ACTIONS(2827),
    [anon_sym_BANG] = ACTIONS(2829),
    [anon_sym_TILDE] = ACTIONS(2829),
    [anon_sym_DASH] = ACTIONS(2827),
    [anon_sym_PLUS] = ACTIONS(2827),
    [anon_sym_DASH_DASH] = ACTIONS(2829),
    [anon_sym_PLUS_PLUS] = ACTIONS(2829),
    [anon_sym_DQUOTE] = ACTIONS(2829),
    [anon_sym_SQUOTE] = ACTIONS(2829),
    [sym_number_literal] = ACTIONS(2829),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(943)] = {
    [sym_identifier] = ACTIONS(2266),
    [sym_not] = ACTIONS(2266),
    [sym_bnot] = ACTIONS(2266),
    [sym___file__] = ACTIONS(2266),
    [sym___date__] = ACTIONS(2266),
    [sym___ldate__] = ACTIONS(2266),
    [sym___time__] = ACTIONS(2266),
    [sym___line__] = ACTIONS(2266),
    [sym___name__] = ACTIONS(2266),
    [sym___netlinx__] = ACTIONS(2266),
    [sym_netlinx_axi_version] = ACTIONS(2266),
    [sym_url_flg_tcp] = ACTIONS(2266),
    [sym_url_flg_accinfopresent] = ACTIONS(2266),
    [sym_url_flg_temp] = ACTIONS(2266),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2266),
    [sym_url_flg_stat_mask] = ACTIONS(2266),
    [sym_url_flg_stat_lookup] = ACTIONS(2266),
    [sym_url_flg_stat_connecting] = ACTIONS(2266),
    [sym_url_flg_stat_waiting] = ACTIONS(2266),
    [sym_url_flg_stat_connected] = ACTIONS(2266),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2266),
    [sym_first_virtual_device] = ACTIONS(2266),
    [sym_first_local_port] = ACTIONS(2266),
    [sym_dynamic_virtual_device] = ACTIONS(2266),
    [sym_timeline_once] = ACTIONS(2266),
    [sym_timeline_repeat] = ACTIONS(2266),
    [sym_timeline_absolute] = ACTIONS(2266),
    [sym_timeline_relative] = ACTIONS(2266),
    [sym_do_push_timed_infinite] = ACTIONS(2266),
    [sym_source_type_no_address] = ACTIONS(2266),
    [sym_source_type_neuron_id] = ACTIONS(2266),
    [sym_source_type_ip_address] = ACTIONS(2266),
    [sym_source_type_axlink] = ACTIONS(2266),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2266),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2266),
    [sym_source_type_ip_socket_address] = ACTIONS(2266),
    [sym_source_type_rs232] = ACTIONS(2266),
    [sym_source_type_internal] = ACTIONS(2266),
    [sym_source_type_mac_address] = ACTIONS(2266),
    [sym_source_type_ipv4_port] = ACTIONS(2266),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2266),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2266),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2266),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2266),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2266),
    [sym_true] = ACTIONS(2266),
    [sym_false] = ACTIONS(2266),
    [sym_file_read_only] = ACTIONS(2266),
    [sym_file_rw_new] = ACTIONS(2266),
    [sym_file_rw_append] = ACTIONS(2266),
    [sym_ip_tcp] = ACTIONS(2266),
    [sym_ip_udp] = ACTIONS(2266),
    [sym_ip_udp_2way] = ACTIONS(2266),
    [sym_xml_encode_types] = ACTIONS(2266),
    [sym_xml_encode_char_as_list] = ACTIONS(2266),
    [sym_xml_encode_le] = ACTIONS(2266),
    [sym_xml_decode_types] = ACTIONS(2266),
    [sym_xml_decode_no_preserve] = ACTIONS(2266),
    [sym_duet_dev_type_display_device] = ACTIONS(2266),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2266),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2266),
    [sym_duet_dev_type_io_device] = ACTIONS(2266),
    [sym_duet_dev_type_relay_device] = ACTIONS(2266),
    [sym_duet_dev_type_ups] = ACTIONS(2266),
    [sym_duet_dev_type_amplifier] = ACTIONS(2266),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2266),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2266),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2266),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2266),
    [sym_duet_dev_type_camera] = ACTIONS(2266),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2266),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2266),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2266),
    [sym_duet_dev_type_dss] = ACTIONS(2266),
    [sym_duet_dev_type_dvr] = ACTIONS(2266),
    [sym_duet_dev_type_disc_device] = ACTIONS(2266),
    [sym_duet_dev_type_document_camera] = ACTIONS(2266),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2266),
    [sym_duet_dev_type_hvac] = ACTIONS(2266),
    [sym_duet_dev_type_keypad] = ACTIONS(2266),
    [sym_duet_dev_type_light] = ACTIONS(2266),
    [sym_duet_dev_type_monitor] = ACTIONS(2266),
    [sym_duet_dev_type_motor] = ACTIONS(2266),
    [sym_duet_dev_type_multi_window] = ACTIONS(2266),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2266),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2266),
    [sym_duet_dev_type_receiver] = ACTIONS(2266),
    [sym_duet_dev_type_security_system] = ACTIONS(2266),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2266),
    [sym_duet_dev_type_settop_box] = ACTIONS(2266),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2266),
    [sym_duet_dev_type_switcher] = ACTIONS(2266),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2266),
    [sym_duet_dev_type_tv] = ACTIONS(2266),
    [sym_duet_dev_type_utility] = ACTIONS(2266),
    [sym_duet_dev_type_vcr] = ACTIONS(2266),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2266),
    [sym_duet_dev_type_video_processor] = ACTIONS(2266),
    [sym_duet_dev_type_video_projector] = ACTIONS(2266),
    [sym_duet_dev_type_video_wall] = ACTIONS(2266),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2266),
    [sym_duet_dev_type_weather] = ACTIONS(2266),
    [sym_duet_dev_not_polled] = ACTIONS(2266),
    [sym_duet_dev_polled] = ACTIONS(2266),
    [sym_clkmgr_mode_network] = ACTIONS(2266),
    [sym_clkmgr_mode_standalone] = ACTIONS(2266),
    [sym_normal_standby] = ACTIONS(2266),
    [sym_normal_wake] = ACTIONS(2266),
    [sym_ip_multicast_ttl_option] = ACTIONS(2266),
    [sym_ip_ttl_subnet] = ACTIONS(2266),
    [sym_ip_ttl_site] = ACTIONS(2266),
    [sym_ip_ttl_region] = ACTIONS(2266),
    [sym_ip_ttl_continent] = ACTIONS(2266),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2266),
    [sym_ip_nodelay_on] = ACTIONS(2266),
    [sym_ip_nodelay_off] = ACTIONS(2266),
    [sym_amx_error] = ACTIONS(2266),
    [sym_amx_warning] = ACTIONS(2266),
    [sym_amx_info] = ACTIONS(2266),
    [sym_amx_debug] = ACTIONS(2266),
    [sym_netlinx_login_success] = ACTIONS(2266),
    [sym_netlinx_login_fail] = ACTIONS(2266),
    [sym_netlinx_logout] = ACTIONS(2266),
    [sym_null_str] = ACTIONS(2266),
    [sym_smtp_address] = ACTIONS(2266),
    [sym_smtp_port_number] = ACTIONS(2266),
    [sym_smtp_username] = ACTIONS(2266),
    [sym_smtp_password] = ACTIONS(2266),
    [sym_smtp_require_tls] = ACTIONS(2266),
    [sym_smtp_from] = ACTIONS(2266),
    [sym_smtp_tls_true] = ACTIONS(2266),
    [sym_smtp_tls_false] = ACTIONS(2266),
    [sym_listview_on_row_select_event] = ACTIONS(2266),
    [sym_data_structure_unknown] = ACTIONS(2266),
    [sym_data_structure_datafeed] = ACTIONS(2266),
    [sym_data_structure_datarecordset] = ACTIONS(2266),
    [sym_data_structure_datarecord] = ACTIONS(2266),
    [sym_data_structure_datafield] = ACTIONS(2266),
    [sym_data_max_content_field_count] = ACTIONS(2266),
    [sym_data_max_metadata_field_count] = ACTIONS(2266),
    [sym_data_max_id_length] = ACTIONS(2266),
    [sym_data_max_name_length] = ACTIONS(2266),
    [sym_data_max_description_length] = ACTIONS(2266),
    [sym_data_max_type_length] = ACTIONS(2266),
    [sym_data_max_format_length] = ACTIONS(2266),
    [sym_data_max_source_length] = ACTIONS(2266),
    [sym_data_max_value_length] = ACTIONS(2266),
    [sym_data_max_label_length] = ACTIONS(2266),
    [sym_data_max_url_length] = ACTIONS(2266),
    [sym_max_last_login_info_length] = ACTIONS(2266),
    [sym_data_type_unknown] = ACTIONS(2266),
    [sym_data_type_string] = ACTIONS(2266),
    [sym_data_type_datetime] = ACTIONS(2266),
    [sym_data_type_date] = ACTIONS(2266),
    [sym_data_type_time] = ACTIONS(2266),
    [sym_data_type_image] = ACTIONS(2266),
    [sym_data_format_url] = ACTIONS(2266),
    [sym_data_format_phone] = ACTIONS(2266),
    [sym_data_format_email] = ACTIONS(2266),
    [sym_data_format_iso8601] = ACTIONS(2266),
    [sym_auditlog_priv] = ACTIONS(2266),
    [sym_configuration_priv] = ACTIONS(2266),
    [sym_deviceconfig_priv] = ACTIONS(2266),
    [sym_ftp_priv] = ACTIONS(2266),
    [sym_http_priv] = ACTIONS(2266),
    [sym_networkconfig_priv] = ACTIONS(2266),
    [sym_programport_priv] = ACTIONS(2266),
    [sym_remoteui_priv] = ACTIONS(2266),
    [sym_securitycontrol_priv] = ACTIONS(2266),
    [sym_softwaremanagement_priv] = ACTIONS(2266),
    [sym_terminal_priv] = ACTIONS(2266),
    [sym_tpadmin_priv] = ACTIONS(2266),
    [sym_user1_priv] = ACTIONS(2266),
    [sym_user2_priv] = ACTIONS(2266),
    [sym_user3_priv] = ACTIONS(2266),
    [sym_user4_priv] = ACTIONS(2266),
    [sym_usermanagement_priv] = ACTIONS(2266),
    [sym_valid_account] = ACTIONS(2266),
    [sym_err_name_invalid] = ACTIONS(2266),
    [sym_err_password_invalid] = ACTIONS(2266),
    [sym_err_invalid_account] = ACTIONS(2266),
    [sym_err_no_authorization] = ACTIONS(2266),
    [sym_err_invalid_parameter] = ACTIONS(2266),
    [sym_err_user_locked_out] = ACTIONS(2266),
    [sym_err_user_account_expired] = ACTIONS(2266),
    [sym_err_general_error] = ACTIONS(2266),
    [sym_err_authentication_error] = ACTIONS(2266),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2266),
    [sym_tls_validate_certificate] = ACTIONS(2266),
    [sym____reserved___] = ACTIONS(2266),
    [sym_date] = ACTIONS(2266),
    [sym_day] = ACTIONS(2266),
    [sym_time] = ACTIONS(2266),
    [sym_ldate] = ACTIONS(2266),
    [sym_push_device] = ACTIONS(2266),
    [sym_push_channel] = ACTIONS(2266),
    [sym_push_devchan] = ACTIONS(2266),
    [sym_release_device] = ACTIONS(2266),
    [sym_release_channel] = ACTIONS(2266),
    [sym_release_devchan] = ACTIONS(2266),
    [sym_master_slot] = ACTIONS(2266),
    [sym_get_pulse_time] = ACTIONS(2266),
    [sym_get_timer] = ACTIONS(2266),
    [sym_master_sn] = ACTIONS(2266),
    [sym_system_number] = ACTIONS(2266),
    [sym_button] = ACTIONS(2266),
    [sym_dv_channel] = ACTIONS(2266),
    [sym_data] = ACTIONS(2266),
    [sym_level] = ACTIONS(2266),
    [sym_timeline] = ACTIONS(2266),
    [sym_channel] = ACTIONS(2266),
    [sym_custom] = ACTIONS(2266),
    [sym_atoi] = ACTIONS(2266),
    [sym_atol] = ACTIONS(2266),
    [sym_atof] = ACTIONS(2266),
    [sym_itoa] = ACTIONS(2266),
    [sym_itohex] = ACTIONS(2266),
    [sym_hextoi] = ACTIONS(2266),
    [sym_ftoa] = ACTIONS(2266),
    [sym_find_string] = ACTIONS(2266),
    [sym_left_string] = ACTIONS(2266),
    [sym_right_string] = ACTIONS(2266),
    [sym_mid_string] = ACTIONS(2266),
    [sym_remove_string] = ACTIONS(2266),
    [sym_get_buffer_string] = ACTIONS(2266),
    [sym_get_buffer_char] = ACTIONS(2266),
    [sym_get_multi_buffer_string] = ACTIONS(2266),
    [sym_length_string] = ACTIONS(2266),
    [sym_max_length_string] = ACTIONS(2266),
    [sym_set_length_string] = ACTIONS(2266),
    [sym_compare_string] = ACTIONS(2266),
    [sym_upper_string] = ACTIONS(2266),
    [sym_lower_string] = ACTIONS(2266),
    [sym_get_last] = ACTIONS(2266),
    [sym_date_to_day] = ACTIONS(2266),
    [sym_date_to_month] = ACTIONS(2266),
    [sym_date_to_year] = ACTIONS(2266),
    [sym_time_to_hour] = ACTIONS(2266),
    [sym_time_to_minute] = ACTIONS(2266),
    [sym_time_to_second] = ACTIONS(2266),
    [sym_day_of_week] = ACTIONS(2266),
    [sym_set_pulse_time] = ACTIONS(2266),
    [sym_set_timer] = ACTIONS(2266),
    [sym_file_open] = ACTIONS(2266),
    [sym_file_close] = ACTIONS(2266),
    [sym_file_read] = ACTIONS(2266),
    [sym_file_read_line] = ACTIONS(2266),
    [sym_file_write] = ACTIONS(2266),
    [sym_file_write_line] = ACTIONS(2266),
    [sym_file_copy] = ACTIONS(2266),
    [sym_file_delete] = ACTIONS(2266),
    [sym_file_rename] = ACTIONS(2266),
    [sym_file_dir] = ACTIONS(2266),
    [sym_file_getdir] = ACTIONS(2266),
    [sym_file_setdir] = ACTIONS(2266),
    [sym_file_create_dir] = ACTIONS(2266),
    [sym_file_remove_dir] = ACTIONS(2266),
    [sym_file_seek] = ACTIONS(2266),
    [sym_length_array] = ACTIONS(2266),
    [sym_max_length_array] = ACTIONS(2266),
    [sym_set_length_array] = ACTIONS(2266),
    [sym_ip_client_open] = ACTIONS(2266),
    [sym_ip_client_close] = ACTIONS(2266),
    [sym_ip_server_open] = ACTIONS(2266),
    [sym_ip_server_close] = ACTIONS(2266),
    [sym_ip_bound_client_open] = ACTIONS(2266),
    [sym_ip_mc_server_open] = ACTIONS(2266),
    [sym_ip_set_option] = ACTIONS(2266),
    [sym_ssh_client_open] = ACTIONS(2266),
    [sym_ssh_client_close] = ACTIONS(2266),
    [sym_tls_client_open] = ACTIONS(2266),
    [sym_tls_client_close] = ACTIONS(2266),
    [sym_do_push] = ACTIONS(2266),
    [sym_do_push_timed] = ACTIONS(2266),
    [sym_do_release] = ACTIONS(2266),
    [sym_device_id] = ACTIONS(2266),
    [sym_device_info] = ACTIONS(2266),
    [sym_device_id_string] = ACTIONS(2266),
    [sym_device_standby] = ACTIONS(2266),
    [sym_device_wake] = ACTIONS(2266),
    [sym_timeline_create] = ACTIONS(2266),
    [sym_timeline_kill] = ACTIONS(2266),
    [sym_timeline_reload] = ACTIONS(2266),
    [sym_timeline_pause] = ACTIONS(2266),
    [sym_timeline_restart] = ACTIONS(2266),
    [sym_timeline_set] = ACTIONS(2266),
    [sym_timeline_get] = ACTIONS(2266),
    [sym_timeline_active] = ACTIONS(2266),
    [sym_timeline_dynamic_id] = ACTIONS(2266),
    [sym_variable_to_string] = ACTIONS(2266),
    [sym_string_to_variable] = ACTIONS(2266),
    [sym_length_variable_to_string] = ACTIONS(2266),
    [sym_type_cast] = ACTIONS(2266),
    [sym_variable_to_xml] = ACTIONS(2266),
    [sym_xml_to_variable] = ACTIONS(2266),
    [sym_length_variable_to_xml] = ACTIONS(2266),
    [sym_redirect_string] = ACTIONS(2266),
    [sym_random_number] = ACTIONS(2266),
    [sym_get_unique_id] = ACTIONS(2266),
    [sym_get_system_number] = ACTIONS(2266),
    [sym_set_system_number] = ACTIONS(2266),
    [sym_get_serial_number] = ACTIONS(2266),
    [sym_get_master_build] = ACTIONS(2266),
    [sym_set_validation_code] = ACTIONS(2266),
    [sym_reboot] = ACTIONS(2266),
    [sym_load_duet_module] = ACTIONS(2266),
    [sym_unload_duet_module] = ACTIONS(2266),
    [sym_duet_mem_size_set] = ACTIONS(2266),
    [sym_duet_mem_size_get] = ACTIONS(2266),
    [sym_dynamic_application_device] = ACTIONS(2266),
    [sym_dynamic_polled_port] = ACTIONS(2266),
    [sym_static_port_binding] = ACTIONS(2266),
    [sym_static_ip_binding] = ACTIONS(2266),
    [sym_get_dns_list] = ACTIONS(2266),
    [sym_set_dns_list] = ACTIONS(2266),
    [sym_get_ip_address] = ACTIONS(2266),
    [sym_set_ip_address] = ACTIONS(2266),
    [sym_get_url_list] = ACTIONS(2266),
    [sym_add_url_entry] = ACTIONS(2266),
    [sym_delete_url_entry] = ACTIONS(2266),
    [sym_abs_value] = ACTIONS(2266),
    [sym_max_value] = ACTIONS(2266),
    [sym_min_value] = ACTIONS(2266),
    [sym_exp_value] = ACTIONS(2266),
    [sym_log_value] = ACTIONS(2266),
    [sym_log10_value] = ACTIONS(2266),
    [sym_power_value] = ACTIONS(2266),
    [sym_sqrt_value] = ACTIONS(2266),
    [sym_raw_be] = ACTIONS(2266),
    [sym_raw_le] = ACTIONS(2266),
    [sym_internal_queue_size_set] = ACTIONS(2266),
    [sym_internal_queue_size_get] = ACTIONS(2266),
    [sym_internal_threshold_set] = ACTIONS(2266),
    [sym_internal_threshold_get] = ACTIONS(2266),
    [sym_rebuilt_event] = ACTIONS(2266),
    [sym_do_custom_event] = ACTIONS(2266),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2266),
    [sym_clkmgr_set_clk_source] = ACTIONS(2266),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2266),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2266),
    [sym_clkmgr_get_timezone] = ACTIONS(2266),
    [sym_clkmgr_set_timezone] = ACTIONS(2266),
    [sym_clkmgr_get_resync_period] = ACTIONS(2266),
    [sym_clkmgr_set_resync_period] = ACTIONS(2266),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2266),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2266),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2266),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2266),
    [sym_clkmgr_get_timeservers] = ACTIONS(2266),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2266),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2266),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2266),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2266),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2266),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2266),
    [sym_set_log_level] = ACTIONS(2266),
    [sym_get_log_level] = ACTIONS(2266),
    [sym_amx_log] = ACTIONS(2266),
    [sym_smtp_server_config_set] = ACTIONS(2266),
    [sym_smtp_server_config_get] = ACTIONS(2266),
    [sym_smtp_send] = ACTIONS(2266),
    [sym_validate_netlinx_account] = ACTIONS(2266),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2266),
    [sym_authenticate_certificate] = ACTIONS(2266),
    [sym_audit_netlinx_session_event] = ACTIONS(2266),
    [sym_audit_netlinx_generic_event] = ACTIONS(2266),
    [sym_get_available_flash_disk_space] = ACTIONS(2266),
    [sym_get_max_flash_disk_space] = ACTIONS(2266),
    [sym_data_create_feed] = ACTIONS(2266),
    [sym_data_delete_feed] = ACTIONS(2266),
    [sym_data_publish_feed] = ACTIONS(2266),
    [sym_data_get_published_feed] = ACTIONS(2266),
    [sym_data_add_record] = ACTIONS(2266),
    [sym_data_get_event_record] = ACTIONS(2266),
    [sym__wc_data_create_feed] = ACTIONS(2266),
    [sym__wc_data_add_record] = ACTIONS(2266),
    [sym__wc_data_get_event_record] = ACTIONS(2266),
    [sym_format] = ACTIONS(2266),
    [sym_astro_clock] = ACTIONS(2266),
    [sym_set_outdoor_temperature] = ACTIONS(2266),
    [sym_set_virtual_level_count] = ACTIONS(2266),
    [sym_set_virtual_channel_count] = ACTIONS(2266),
    [sym_set_virtual_port_count] = ACTIONS(2266),
    [sym__wc_encode_int] = ACTIONS(2266),
    [sym__wc_decode_int] = ACTIONS(2266),
    [sym__wc_int] = ACTIONS(2266),
    [sym__wc_to_ch_int] = ACTIONS(2266),
    [sym__ch_to_wc_int] = ACTIONS(2266),
    [sym__wc_find_string_int] = ACTIONS(2266),
    [sym__wc_left_string_int] = ACTIONS(2266),
    [sym__wc_lower_string_int] = ACTIONS(2266),
    [sym__wc_mid_string_int] = ACTIONS(2266),
    [sym__wc_remove_string_int] = ACTIONS(2266),
    [sym__wc_right_string_int] = ACTIONS(2266),
    [sym__wc_upper_string_int] = ACTIONS(2266),
    [sym__wc_compare_string_int] = ACTIONS(2266),
    [sym__wc_get_buffer_char_int] = ACTIONS(2266),
    [sym__wc_get_buffer_string_int] = ACTIONS(2266),
    [sym__wc_concat_string_int] = ACTIONS(2266),
    [sym__wc_file_open_int] = ACTIONS(2266),
    [sym__wc_file_close_int] = ACTIONS(2266),
    [sym__wc_file_read_int] = ACTIONS(2266),
    [sym__wc_file_read_line_int] = ACTIONS(2266),
    [sym__wc_file_write_int] = ACTIONS(2266),
    [sym__wc_file_write_line_int] = ACTIONS(2266),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2268),
    [aux_sym_preproc_else_token1] = ACTIONS(2268),
    [anon_sym_COMMA] = ACTIONS(2268),
    [anon_sym_LBRACK] = ACTIONS(2268),
    [anon_sym_LBRACE] = ACTIONS(2268),
    [anon_sym_LPAREN] = ACTIONS(2266),
    [anon_sym_BANG] = ACTIONS(2268),
    [anon_sym_TILDE] = ACTIONS(2268),
    [anon_sym_DASH] = ACTIONS(2266),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_DASH_DASH] = ACTIONS(2268),
    [anon_sym_PLUS_PLUS] = ACTIONS(2268),
    [anon_sym_DQUOTE] = ACTIONS(2268),
    [anon_sym_SQUOTE] = ACTIONS(2268),
    [sym_number_literal] = ACTIONS(2268),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(944)] = {
    [sym_identifier] = ACTIONS(2273),
    [sym_not] = ACTIONS(2273),
    [sym_bnot] = ACTIONS(2273),
    [sym___file__] = ACTIONS(2273),
    [sym___date__] = ACTIONS(2273),
    [sym___ldate__] = ACTIONS(2273),
    [sym___time__] = ACTIONS(2273),
    [sym___line__] = ACTIONS(2273),
    [sym___name__] = ACTIONS(2273),
    [sym___netlinx__] = ACTIONS(2273),
    [sym_netlinx_axi_version] = ACTIONS(2273),
    [sym_url_flg_tcp] = ACTIONS(2273),
    [sym_url_flg_accinfopresent] = ACTIONS(2273),
    [sym_url_flg_temp] = ACTIONS(2273),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2273),
    [sym_url_flg_stat_mask] = ACTIONS(2273),
    [sym_url_flg_stat_lookup] = ACTIONS(2273),
    [sym_url_flg_stat_connecting] = ACTIONS(2273),
    [sym_url_flg_stat_waiting] = ACTIONS(2273),
    [sym_url_flg_stat_connected] = ACTIONS(2273),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2273),
    [sym_first_virtual_device] = ACTIONS(2273),
    [sym_first_local_port] = ACTIONS(2273),
    [sym_dynamic_virtual_device] = ACTIONS(2273),
    [sym_timeline_once] = ACTIONS(2273),
    [sym_timeline_repeat] = ACTIONS(2273),
    [sym_timeline_absolute] = ACTIONS(2273),
    [sym_timeline_relative] = ACTIONS(2273),
    [sym_do_push_timed_infinite] = ACTIONS(2273),
    [sym_source_type_no_address] = ACTIONS(2273),
    [sym_source_type_neuron_id] = ACTIONS(2273),
    [sym_source_type_ip_address] = ACTIONS(2273),
    [sym_source_type_axlink] = ACTIONS(2273),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2273),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2273),
    [sym_source_type_ip_socket_address] = ACTIONS(2273),
    [sym_source_type_rs232] = ACTIONS(2273),
    [sym_source_type_internal] = ACTIONS(2273),
    [sym_source_type_mac_address] = ACTIONS(2273),
    [sym_source_type_ipv4_port] = ACTIONS(2273),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2273),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2273),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2273),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2273),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2273),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2273),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2273),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2273),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2273),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2273),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2273),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2273),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2273),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2273),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2273),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2273),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2273),
    [sym_true] = ACTIONS(2273),
    [sym_false] = ACTIONS(2273),
    [sym_file_read_only] = ACTIONS(2273),
    [sym_file_rw_new] = ACTIONS(2273),
    [sym_file_rw_append] = ACTIONS(2273),
    [sym_ip_tcp] = ACTIONS(2273),
    [sym_ip_udp] = ACTIONS(2273),
    [sym_ip_udp_2way] = ACTIONS(2273),
    [sym_xml_encode_types] = ACTIONS(2273),
    [sym_xml_encode_char_as_list] = ACTIONS(2273),
    [sym_xml_encode_le] = ACTIONS(2273),
    [sym_xml_decode_types] = ACTIONS(2273),
    [sym_xml_decode_no_preserve] = ACTIONS(2273),
    [sym_duet_dev_type_display_device] = ACTIONS(2273),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2273),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2273),
    [sym_duet_dev_type_io_device] = ACTIONS(2273),
    [sym_duet_dev_type_relay_device] = ACTIONS(2273),
    [sym_duet_dev_type_ups] = ACTIONS(2273),
    [sym_duet_dev_type_amplifier] = ACTIONS(2273),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2273),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2273),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2273),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2273),
    [sym_duet_dev_type_camera] = ACTIONS(2273),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2273),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2273),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2273),
    [sym_duet_dev_type_dss] = ACTIONS(2273),
    [sym_duet_dev_type_dvr] = ACTIONS(2273),
    [sym_duet_dev_type_disc_device] = ACTIONS(2273),
    [sym_duet_dev_type_document_camera] = ACTIONS(2273),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2273),
    [sym_duet_dev_type_hvac] = ACTIONS(2273),
    [sym_duet_dev_type_keypad] = ACTIONS(2273),
    [sym_duet_dev_type_light] = ACTIONS(2273),
    [sym_duet_dev_type_monitor] = ACTIONS(2273),
    [sym_duet_dev_type_motor] = ACTIONS(2273),
    [sym_duet_dev_type_multi_window] = ACTIONS(2273),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2273),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2273),
    [sym_duet_dev_type_receiver] = ACTIONS(2273),
    [sym_duet_dev_type_security_system] = ACTIONS(2273),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2273),
    [sym_duet_dev_type_settop_box] = ACTIONS(2273),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2273),
    [sym_duet_dev_type_switcher] = ACTIONS(2273),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2273),
    [sym_duet_dev_type_tv] = ACTIONS(2273),
    [sym_duet_dev_type_utility] = ACTIONS(2273),
    [sym_duet_dev_type_vcr] = ACTIONS(2273),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2273),
    [sym_duet_dev_type_video_processor] = ACTIONS(2273),
    [sym_duet_dev_type_video_projector] = ACTIONS(2273),
    [sym_duet_dev_type_video_wall] = ACTIONS(2273),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2273),
    [sym_duet_dev_type_weather] = ACTIONS(2273),
    [sym_duet_dev_not_polled] = ACTIONS(2273),
    [sym_duet_dev_polled] = ACTIONS(2273),
    [sym_clkmgr_mode_network] = ACTIONS(2273),
    [sym_clkmgr_mode_standalone] = ACTIONS(2273),
    [sym_normal_standby] = ACTIONS(2273),
    [sym_normal_wake] = ACTIONS(2273),
    [sym_ip_multicast_ttl_option] = ACTIONS(2273),
    [sym_ip_ttl_subnet] = ACTIONS(2273),
    [sym_ip_ttl_site] = ACTIONS(2273),
    [sym_ip_ttl_region] = ACTIONS(2273),
    [sym_ip_ttl_continent] = ACTIONS(2273),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2273),
    [sym_ip_nodelay_on] = ACTIONS(2273),
    [sym_ip_nodelay_off] = ACTIONS(2273),
    [sym_amx_error] = ACTIONS(2273),
    [sym_amx_warning] = ACTIONS(2273),
    [sym_amx_info] = ACTIONS(2273),
    [sym_amx_debug] = ACTIONS(2273),
    [sym_netlinx_login_success] = ACTIONS(2273),
    [sym_netlinx_login_fail] = ACTIONS(2273),
    [sym_netlinx_logout] = ACTIONS(2273),
    [sym_null_str] = ACTIONS(2273),
    [sym_smtp_address] = ACTIONS(2273),
    [sym_smtp_port_number] = ACTIONS(2273),
    [sym_smtp_username] = ACTIONS(2273),
    [sym_smtp_password] = ACTIONS(2273),
    [sym_smtp_require_tls] = ACTIONS(2273),
    [sym_smtp_from] = ACTIONS(2273),
    [sym_smtp_tls_true] = ACTIONS(2273),
    [sym_smtp_tls_false] = ACTIONS(2273),
    [sym_listview_on_row_select_event] = ACTIONS(2273),
    [sym_data_structure_unknown] = ACTIONS(2273),
    [sym_data_structure_datafeed] = ACTIONS(2273),
    [sym_data_structure_datarecordset] = ACTIONS(2273),
    [sym_data_structure_datarecord] = ACTIONS(2273),
    [sym_data_structure_datafield] = ACTIONS(2273),
    [sym_data_max_content_field_count] = ACTIONS(2273),
    [sym_data_max_metadata_field_count] = ACTIONS(2273),
    [sym_data_max_id_length] = ACTIONS(2273),
    [sym_data_max_name_length] = ACTIONS(2273),
    [sym_data_max_description_length] = ACTIONS(2273),
    [sym_data_max_type_length] = ACTIONS(2273),
    [sym_data_max_format_length] = ACTIONS(2273),
    [sym_data_max_source_length] = ACTIONS(2273),
    [sym_data_max_value_length] = ACTIONS(2273),
    [sym_data_max_label_length] = ACTIONS(2273),
    [sym_data_max_url_length] = ACTIONS(2273),
    [sym_max_last_login_info_length] = ACTIONS(2273),
    [sym_data_type_unknown] = ACTIONS(2273),
    [sym_data_type_string] = ACTIONS(2273),
    [sym_data_type_datetime] = ACTIONS(2273),
    [sym_data_type_date] = ACTIONS(2273),
    [sym_data_type_time] = ACTIONS(2273),
    [sym_data_type_image] = ACTIONS(2273),
    [sym_data_format_url] = ACTIONS(2273),
    [sym_data_format_phone] = ACTIONS(2273),
    [sym_data_format_email] = ACTIONS(2273),
    [sym_data_format_iso8601] = ACTIONS(2273),
    [sym_auditlog_priv] = ACTIONS(2273),
    [sym_configuration_priv] = ACTIONS(2273),
    [sym_deviceconfig_priv] = ACTIONS(2273),
    [sym_ftp_priv] = ACTIONS(2273),
    [sym_http_priv] = ACTIONS(2273),
    [sym_networkconfig_priv] = ACTIONS(2273),
    [sym_programport_priv] = ACTIONS(2273),
    [sym_remoteui_priv] = ACTIONS(2273),
    [sym_securitycontrol_priv] = ACTIONS(2273),
    [sym_softwaremanagement_priv] = ACTIONS(2273),
    [sym_terminal_priv] = ACTIONS(2273),
    [sym_tpadmin_priv] = ACTIONS(2273),
    [sym_user1_priv] = ACTIONS(2273),
    [sym_user2_priv] = ACTIONS(2273),
    [sym_user3_priv] = ACTIONS(2273),
    [sym_user4_priv] = ACTIONS(2273),
    [sym_usermanagement_priv] = ACTIONS(2273),
    [sym_valid_account] = ACTIONS(2273),
    [sym_err_name_invalid] = ACTIONS(2273),
    [sym_err_password_invalid] = ACTIONS(2273),
    [sym_err_invalid_account] = ACTIONS(2273),
    [sym_err_no_authorization] = ACTIONS(2273),
    [sym_err_invalid_parameter] = ACTIONS(2273),
    [sym_err_user_locked_out] = ACTIONS(2273),
    [sym_err_user_account_expired] = ACTIONS(2273),
    [sym_err_general_error] = ACTIONS(2273),
    [sym_err_authentication_error] = ACTIONS(2273),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2273),
    [sym_tls_validate_certificate] = ACTIONS(2273),
    [sym____reserved___] = ACTIONS(2273),
    [sym_date] = ACTIONS(2273),
    [sym_day] = ACTIONS(2273),
    [sym_time] = ACTIONS(2273),
    [sym_ldate] = ACTIONS(2273),
    [sym_push_device] = ACTIONS(2273),
    [sym_push_channel] = ACTIONS(2273),
    [sym_push_devchan] = ACTIONS(2273),
    [sym_release_device] = ACTIONS(2273),
    [sym_release_channel] = ACTIONS(2273),
    [sym_release_devchan] = ACTIONS(2273),
    [sym_master_slot] = ACTIONS(2273),
    [sym_get_pulse_time] = ACTIONS(2273),
    [sym_get_timer] = ACTIONS(2273),
    [sym_master_sn] = ACTIONS(2273),
    [sym_system_number] = ACTIONS(2273),
    [sym_button] = ACTIONS(2273),
    [sym_dv_channel] = ACTIONS(2273),
    [sym_data] = ACTIONS(2273),
    [sym_level] = ACTIONS(2273),
    [sym_timeline] = ACTIONS(2273),
    [sym_channel] = ACTIONS(2273),
    [sym_custom] = ACTIONS(2273),
    [sym_atoi] = ACTIONS(2273),
    [sym_atol] = ACTIONS(2273),
    [sym_atof] = ACTIONS(2273),
    [sym_itoa] = ACTIONS(2273),
    [sym_itohex] = ACTIONS(2273),
    [sym_hextoi] = ACTIONS(2273),
    [sym_ftoa] = ACTIONS(2273),
    [sym_find_string] = ACTIONS(2273),
    [sym_left_string] = ACTIONS(2273),
    [sym_right_string] = ACTIONS(2273),
    [sym_mid_string] = ACTIONS(2273),
    [sym_remove_string] = ACTIONS(2273),
    [sym_get_buffer_string] = ACTIONS(2273),
    [sym_get_buffer_char] = ACTIONS(2273),
    [sym_get_multi_buffer_string] = ACTIONS(2273),
    [sym_length_string] = ACTIONS(2273),
    [sym_max_length_string] = ACTIONS(2273),
    [sym_set_length_string] = ACTIONS(2273),
    [sym_compare_string] = ACTIONS(2273),
    [sym_upper_string] = ACTIONS(2273),
    [sym_lower_string] = ACTIONS(2273),
    [sym_get_last] = ACTIONS(2273),
    [sym_date_to_day] = ACTIONS(2273),
    [sym_date_to_month] = ACTIONS(2273),
    [sym_date_to_year] = ACTIONS(2273),
    [sym_time_to_hour] = ACTIONS(2273),
    [sym_time_to_minute] = ACTIONS(2273),
    [sym_time_to_second] = ACTIONS(2273),
    [sym_day_of_week] = ACTIONS(2273),
    [sym_set_pulse_time] = ACTIONS(2273),
    [sym_set_timer] = ACTIONS(2273),
    [sym_file_open] = ACTIONS(2273),
    [sym_file_close] = ACTIONS(2273),
    [sym_file_read] = ACTIONS(2273),
    [sym_file_read_line] = ACTIONS(2273),
    [sym_file_write] = ACTIONS(2273),
    [sym_file_write_line] = ACTIONS(2273),
    [sym_file_copy] = ACTIONS(2273),
    [sym_file_delete] = ACTIONS(2273),
    [sym_file_rename] = ACTIONS(2273),
    [sym_file_dir] = ACTIONS(2273),
    [sym_file_getdir] = ACTIONS(2273),
    [sym_file_setdir] = ACTIONS(2273),
    [sym_file_create_dir] = ACTIONS(2273),
    [sym_file_remove_dir] = ACTIONS(2273),
    [sym_file_seek] = ACTIONS(2273),
    [sym_length_array] = ACTIONS(2273),
    [sym_max_length_array] = ACTIONS(2273),
    [sym_set_length_array] = ACTIONS(2273),
    [sym_ip_client_open] = ACTIONS(2273),
    [sym_ip_client_close] = ACTIONS(2273),
    [sym_ip_server_open] = ACTIONS(2273),
    [sym_ip_server_close] = ACTIONS(2273),
    [sym_ip_bound_client_open] = ACTIONS(2273),
    [sym_ip_mc_server_open] = ACTIONS(2273),
    [sym_ip_set_option] = ACTIONS(2273),
    [sym_ssh_client_open] = ACTIONS(2273),
    [sym_ssh_client_close] = ACTIONS(2273),
    [sym_tls_client_open] = ACTIONS(2273),
    [sym_tls_client_close] = ACTIONS(2273),
    [sym_do_push] = ACTIONS(2273),
    [sym_do_push_timed] = ACTIONS(2273),
    [sym_do_release] = ACTIONS(2273),
    [sym_device_id] = ACTIONS(2273),
    [sym_device_info] = ACTIONS(2273),
    [sym_device_id_string] = ACTIONS(2273),
    [sym_device_standby] = ACTIONS(2273),
    [sym_device_wake] = ACTIONS(2273),
    [sym_timeline_create] = ACTIONS(2273),
    [sym_timeline_kill] = ACTIONS(2273),
    [sym_timeline_reload] = ACTIONS(2273),
    [sym_timeline_pause] = ACTIONS(2273),
    [sym_timeline_restart] = ACTIONS(2273),
    [sym_timeline_set] = ACTIONS(2273),
    [sym_timeline_get] = ACTIONS(2273),
    [sym_timeline_active] = ACTIONS(2273),
    [sym_timeline_dynamic_id] = ACTIONS(2273),
    [sym_variable_to_string] = ACTIONS(2273),
    [sym_string_to_variable] = ACTIONS(2273),
    [sym_length_variable_to_string] = ACTIONS(2273),
    [sym_type_cast] = ACTIONS(2273),
    [sym_variable_to_xml] = ACTIONS(2273),
    [sym_xml_to_variable] = ACTIONS(2273),
    [sym_length_variable_to_xml] = ACTIONS(2273),
    [sym_redirect_string] = ACTIONS(2273),
    [sym_random_number] = ACTIONS(2273),
    [sym_get_unique_id] = ACTIONS(2273),
    [sym_get_system_number] = ACTIONS(2273),
    [sym_set_system_number] = ACTIONS(2273),
    [sym_get_serial_number] = ACTIONS(2273),
    [sym_get_master_build] = ACTIONS(2273),
    [sym_set_validation_code] = ACTIONS(2273),
    [sym_reboot] = ACTIONS(2273),
    [sym_load_duet_module] = ACTIONS(2273),
    [sym_unload_duet_module] = ACTIONS(2273),
    [sym_duet_mem_size_set] = ACTIONS(2273),
    [sym_duet_mem_size_get] = ACTIONS(2273),
    [sym_dynamic_application_device] = ACTIONS(2273),
    [sym_dynamic_polled_port] = ACTIONS(2273),
    [sym_static_port_binding] = ACTIONS(2273),
    [sym_static_ip_binding] = ACTIONS(2273),
    [sym_get_dns_list] = ACTIONS(2273),
    [sym_set_dns_list] = ACTIONS(2273),
    [sym_get_ip_address] = ACTIONS(2273),
    [sym_set_ip_address] = ACTIONS(2273),
    [sym_get_url_list] = ACTIONS(2273),
    [sym_add_url_entry] = ACTIONS(2273),
    [sym_delete_url_entry] = ACTIONS(2273),
    [sym_abs_value] = ACTIONS(2273),
    [sym_max_value] = ACTIONS(2273),
    [sym_min_value] = ACTIONS(2273),
    [sym_exp_value] = ACTIONS(2273),
    [sym_log_value] = ACTIONS(2273),
    [sym_log10_value] = ACTIONS(2273),
    [sym_power_value] = ACTIONS(2273),
    [sym_sqrt_value] = ACTIONS(2273),
    [sym_raw_be] = ACTIONS(2273),
    [sym_raw_le] = ACTIONS(2273),
    [sym_internal_queue_size_set] = ACTIONS(2273),
    [sym_internal_queue_size_get] = ACTIONS(2273),
    [sym_internal_threshold_set] = ACTIONS(2273),
    [sym_internal_threshold_get] = ACTIONS(2273),
    [sym_rebuilt_event] = ACTIONS(2273),
    [sym_do_custom_event] = ACTIONS(2273),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2273),
    [sym_clkmgr_set_clk_source] = ACTIONS(2273),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2273),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2273),
    [sym_clkmgr_get_timezone] = ACTIONS(2273),
    [sym_clkmgr_set_timezone] = ACTIONS(2273),
    [sym_clkmgr_get_resync_period] = ACTIONS(2273),
    [sym_clkmgr_set_resync_period] = ACTIONS(2273),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2273),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2273),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2273),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2273),
    [sym_clkmgr_get_timeservers] = ACTIONS(2273),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2273),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2273),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2273),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2273),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2273),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2273),
    [sym_set_log_level] = ACTIONS(2273),
    [sym_get_log_level] = ACTIONS(2273),
    [sym_amx_log] = ACTIONS(2273),
    [sym_smtp_server_config_set] = ACTIONS(2273),
    [sym_smtp_server_config_get] = ACTIONS(2273),
    [sym_smtp_send] = ACTIONS(2273),
    [sym_validate_netlinx_account] = ACTIONS(2273),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2273),
    [sym_authenticate_certificate] = ACTIONS(2273),
    [sym_audit_netlinx_session_event] = ACTIONS(2273),
    [sym_audit_netlinx_generic_event] = ACTIONS(2273),
    [sym_get_available_flash_disk_space] = ACTIONS(2273),
    [sym_get_max_flash_disk_space] = ACTIONS(2273),
    [sym_data_create_feed] = ACTIONS(2273),
    [sym_data_delete_feed] = ACTIONS(2273),
    [sym_data_publish_feed] = ACTIONS(2273),
    [sym_data_get_published_feed] = ACTIONS(2273),
    [sym_data_add_record] = ACTIONS(2273),
    [sym_data_get_event_record] = ACTIONS(2273),
    [sym__wc_data_create_feed] = ACTIONS(2273),
    [sym__wc_data_add_record] = ACTIONS(2273),
    [sym__wc_data_get_event_record] = ACTIONS(2273),
    [sym_format] = ACTIONS(2273),
    [sym_astro_clock] = ACTIONS(2273),
    [sym_set_outdoor_temperature] = ACTIONS(2273),
    [sym_set_virtual_level_count] = ACTIONS(2273),
    [sym_set_virtual_channel_count] = ACTIONS(2273),
    [sym_set_virtual_port_count] = ACTIONS(2273),
    [sym__wc_encode_int] = ACTIONS(2273),
    [sym__wc_decode_int] = ACTIONS(2273),
    [sym__wc_int] = ACTIONS(2273),
    [sym__wc_to_ch_int] = ACTIONS(2273),
    [sym__ch_to_wc_int] = ACTIONS(2273),
    [sym__wc_find_string_int] = ACTIONS(2273),
    [sym__wc_left_string_int] = ACTIONS(2273),
    [sym__wc_lower_string_int] = ACTIONS(2273),
    [sym__wc_mid_string_int] = ACTIONS(2273),
    [sym__wc_remove_string_int] = ACTIONS(2273),
    [sym__wc_right_string_int] = ACTIONS(2273),
    [sym__wc_upper_string_int] = ACTIONS(2273),
    [sym__wc_compare_string_int] = ACTIONS(2273),
    [sym__wc_get_buffer_char_int] = ACTIONS(2273),
    [sym__wc_get_buffer_string_int] = ACTIONS(2273),
    [sym__wc_concat_string_int] = ACTIONS(2273),
    [sym__wc_file_open_int] = ACTIONS(2273),
    [sym__wc_file_close_int] = ACTIONS(2273),
    [sym__wc_file_read_int] = ACTIONS(2273),
    [sym__wc_file_read_line_int] = ACTIONS(2273),
    [sym__wc_file_write_int] = ACTIONS(2273),
    [sym__wc_file_write_line_int] = ACTIONS(2273),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2276),
    [aux_sym_preproc_else_token1] = ACTIONS(2276),
    [anon_sym_COMMA] = ACTIONS(2263),
    [anon_sym_LBRACK] = ACTIONS(2276),
    [anon_sym_LBRACE] = ACTIONS(2276),
    [anon_sym_LPAREN] = ACTIONS(2273),
    [anon_sym_BANG] = ACTIONS(2276),
    [anon_sym_TILDE] = ACTIONS(2276),
    [anon_sym_DASH] = ACTIONS(2273),
    [anon_sym_PLUS] = ACTIONS(2273),
    [anon_sym_DASH_DASH] = ACTIONS(2276),
    [anon_sym_PLUS_PLUS] = ACTIONS(2276),
    [anon_sym_DQUOTE] = ACTIONS(2276),
    [anon_sym_SQUOTE] = ACTIONS(2276),
    [sym_number_literal] = ACTIONS(2276),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(945)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2241),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(946)] = {
    [sym_identifier] = ACTIONS(2259),
    [sym_not] = ACTIONS(2259),
    [sym_bnot] = ACTIONS(2259),
    [sym___file__] = ACTIONS(2259),
    [sym___date__] = ACTIONS(2259),
    [sym___ldate__] = ACTIONS(2259),
    [sym___time__] = ACTIONS(2259),
    [sym___line__] = ACTIONS(2259),
    [sym___name__] = ACTIONS(2259),
    [sym___netlinx__] = ACTIONS(2259),
    [sym_netlinx_axi_version] = ACTIONS(2259),
    [sym_url_flg_tcp] = ACTIONS(2259),
    [sym_url_flg_accinfopresent] = ACTIONS(2259),
    [sym_url_flg_temp] = ACTIONS(2259),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2259),
    [sym_url_flg_stat_mask] = ACTIONS(2259),
    [sym_url_flg_stat_lookup] = ACTIONS(2259),
    [sym_url_flg_stat_connecting] = ACTIONS(2259),
    [sym_url_flg_stat_waiting] = ACTIONS(2259),
    [sym_url_flg_stat_connected] = ACTIONS(2259),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2259),
    [sym_first_virtual_device] = ACTIONS(2259),
    [sym_first_local_port] = ACTIONS(2259),
    [sym_dynamic_virtual_device] = ACTIONS(2259),
    [sym_timeline_once] = ACTIONS(2259),
    [sym_timeline_repeat] = ACTIONS(2259),
    [sym_timeline_absolute] = ACTIONS(2259),
    [sym_timeline_relative] = ACTIONS(2259),
    [sym_do_push_timed_infinite] = ACTIONS(2259),
    [sym_source_type_no_address] = ACTIONS(2259),
    [sym_source_type_neuron_id] = ACTIONS(2259),
    [sym_source_type_ip_address] = ACTIONS(2259),
    [sym_source_type_axlink] = ACTIONS(2259),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2259),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2259),
    [sym_source_type_ip_socket_address] = ACTIONS(2259),
    [sym_source_type_rs232] = ACTIONS(2259),
    [sym_source_type_internal] = ACTIONS(2259),
    [sym_source_type_mac_address] = ACTIONS(2259),
    [sym_source_type_ipv4_port] = ACTIONS(2259),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2259),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2259),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2259),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2259),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2259),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2259),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2259),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2259),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2259),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2259),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2259),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2259),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2259),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2259),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2259),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2259),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2259),
    [sym_true] = ACTIONS(2259),
    [sym_false] = ACTIONS(2259),
    [sym_file_read_only] = ACTIONS(2259),
    [sym_file_rw_new] = ACTIONS(2259),
    [sym_file_rw_append] = ACTIONS(2259),
    [sym_ip_tcp] = ACTIONS(2259),
    [sym_ip_udp] = ACTIONS(2259),
    [sym_ip_udp_2way] = ACTIONS(2259),
    [sym_xml_encode_types] = ACTIONS(2259),
    [sym_xml_encode_char_as_list] = ACTIONS(2259),
    [sym_xml_encode_le] = ACTIONS(2259),
    [sym_xml_decode_types] = ACTIONS(2259),
    [sym_xml_decode_no_preserve] = ACTIONS(2259),
    [sym_duet_dev_type_display_device] = ACTIONS(2259),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2259),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2259),
    [sym_duet_dev_type_io_device] = ACTIONS(2259),
    [sym_duet_dev_type_relay_device] = ACTIONS(2259),
    [sym_duet_dev_type_ups] = ACTIONS(2259),
    [sym_duet_dev_type_amplifier] = ACTIONS(2259),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2259),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2259),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2259),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2259),
    [sym_duet_dev_type_camera] = ACTIONS(2259),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2259),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2259),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2259),
    [sym_duet_dev_type_dss] = ACTIONS(2259),
    [sym_duet_dev_type_dvr] = ACTIONS(2259),
    [sym_duet_dev_type_disc_device] = ACTIONS(2259),
    [sym_duet_dev_type_document_camera] = ACTIONS(2259),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2259),
    [sym_duet_dev_type_hvac] = ACTIONS(2259),
    [sym_duet_dev_type_keypad] = ACTIONS(2259),
    [sym_duet_dev_type_light] = ACTIONS(2259),
    [sym_duet_dev_type_monitor] = ACTIONS(2259),
    [sym_duet_dev_type_motor] = ACTIONS(2259),
    [sym_duet_dev_type_multi_window] = ACTIONS(2259),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2259),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2259),
    [sym_duet_dev_type_receiver] = ACTIONS(2259),
    [sym_duet_dev_type_security_system] = ACTIONS(2259),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2259),
    [sym_duet_dev_type_settop_box] = ACTIONS(2259),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2259),
    [sym_duet_dev_type_switcher] = ACTIONS(2259),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2259),
    [sym_duet_dev_type_tv] = ACTIONS(2259),
    [sym_duet_dev_type_utility] = ACTIONS(2259),
    [sym_duet_dev_type_vcr] = ACTIONS(2259),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2259),
    [sym_duet_dev_type_video_processor] = ACTIONS(2259),
    [sym_duet_dev_type_video_projector] = ACTIONS(2259),
    [sym_duet_dev_type_video_wall] = ACTIONS(2259),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2259),
    [sym_duet_dev_type_weather] = ACTIONS(2259),
    [sym_duet_dev_not_polled] = ACTIONS(2259),
    [sym_duet_dev_polled] = ACTIONS(2259),
    [sym_clkmgr_mode_network] = ACTIONS(2259),
    [sym_clkmgr_mode_standalone] = ACTIONS(2259),
    [sym_normal_standby] = ACTIONS(2259),
    [sym_normal_wake] = ACTIONS(2259),
    [sym_ip_multicast_ttl_option] = ACTIONS(2259),
    [sym_ip_ttl_subnet] = ACTIONS(2259),
    [sym_ip_ttl_site] = ACTIONS(2259),
    [sym_ip_ttl_region] = ACTIONS(2259),
    [sym_ip_ttl_continent] = ACTIONS(2259),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2259),
    [sym_ip_nodelay_on] = ACTIONS(2259),
    [sym_ip_nodelay_off] = ACTIONS(2259),
    [sym_amx_error] = ACTIONS(2259),
    [sym_amx_warning] = ACTIONS(2259),
    [sym_amx_info] = ACTIONS(2259),
    [sym_amx_debug] = ACTIONS(2259),
    [sym_netlinx_login_success] = ACTIONS(2259),
    [sym_netlinx_login_fail] = ACTIONS(2259),
    [sym_netlinx_logout] = ACTIONS(2259),
    [sym_null_str] = ACTIONS(2259),
    [sym_smtp_address] = ACTIONS(2259),
    [sym_smtp_port_number] = ACTIONS(2259),
    [sym_smtp_username] = ACTIONS(2259),
    [sym_smtp_password] = ACTIONS(2259),
    [sym_smtp_require_tls] = ACTIONS(2259),
    [sym_smtp_from] = ACTIONS(2259),
    [sym_smtp_tls_true] = ACTIONS(2259),
    [sym_smtp_tls_false] = ACTIONS(2259),
    [sym_listview_on_row_select_event] = ACTIONS(2259),
    [sym_data_structure_unknown] = ACTIONS(2259),
    [sym_data_structure_datafeed] = ACTIONS(2259),
    [sym_data_structure_datarecordset] = ACTIONS(2259),
    [sym_data_structure_datarecord] = ACTIONS(2259),
    [sym_data_structure_datafield] = ACTIONS(2259),
    [sym_data_max_content_field_count] = ACTIONS(2259),
    [sym_data_max_metadata_field_count] = ACTIONS(2259),
    [sym_data_max_id_length] = ACTIONS(2259),
    [sym_data_max_name_length] = ACTIONS(2259),
    [sym_data_max_description_length] = ACTIONS(2259),
    [sym_data_max_type_length] = ACTIONS(2259),
    [sym_data_max_format_length] = ACTIONS(2259),
    [sym_data_max_source_length] = ACTIONS(2259),
    [sym_data_max_value_length] = ACTIONS(2259),
    [sym_data_max_label_length] = ACTIONS(2259),
    [sym_data_max_url_length] = ACTIONS(2259),
    [sym_max_last_login_info_length] = ACTIONS(2259),
    [sym_data_type_unknown] = ACTIONS(2259),
    [sym_data_type_string] = ACTIONS(2259),
    [sym_data_type_datetime] = ACTIONS(2259),
    [sym_data_type_date] = ACTIONS(2259),
    [sym_data_type_time] = ACTIONS(2259),
    [sym_data_type_image] = ACTIONS(2259),
    [sym_data_format_url] = ACTIONS(2259),
    [sym_data_format_phone] = ACTIONS(2259),
    [sym_data_format_email] = ACTIONS(2259),
    [sym_data_format_iso8601] = ACTIONS(2259),
    [sym_auditlog_priv] = ACTIONS(2259),
    [sym_configuration_priv] = ACTIONS(2259),
    [sym_deviceconfig_priv] = ACTIONS(2259),
    [sym_ftp_priv] = ACTIONS(2259),
    [sym_http_priv] = ACTIONS(2259),
    [sym_networkconfig_priv] = ACTIONS(2259),
    [sym_programport_priv] = ACTIONS(2259),
    [sym_remoteui_priv] = ACTIONS(2259),
    [sym_securitycontrol_priv] = ACTIONS(2259),
    [sym_softwaremanagement_priv] = ACTIONS(2259),
    [sym_terminal_priv] = ACTIONS(2259),
    [sym_tpadmin_priv] = ACTIONS(2259),
    [sym_user1_priv] = ACTIONS(2259),
    [sym_user2_priv] = ACTIONS(2259),
    [sym_user3_priv] = ACTIONS(2259),
    [sym_user4_priv] = ACTIONS(2259),
    [sym_usermanagement_priv] = ACTIONS(2259),
    [sym_valid_account] = ACTIONS(2259),
    [sym_err_name_invalid] = ACTIONS(2259),
    [sym_err_password_invalid] = ACTIONS(2259),
    [sym_err_invalid_account] = ACTIONS(2259),
    [sym_err_no_authorization] = ACTIONS(2259),
    [sym_err_invalid_parameter] = ACTIONS(2259),
    [sym_err_user_locked_out] = ACTIONS(2259),
    [sym_err_user_account_expired] = ACTIONS(2259),
    [sym_err_general_error] = ACTIONS(2259),
    [sym_err_authentication_error] = ACTIONS(2259),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2259),
    [sym_tls_validate_certificate] = ACTIONS(2259),
    [sym____reserved___] = ACTIONS(2259),
    [sym_date] = ACTIONS(2259),
    [sym_day] = ACTIONS(2259),
    [sym_time] = ACTIONS(2259),
    [sym_ldate] = ACTIONS(2259),
    [sym_push_device] = ACTIONS(2259),
    [sym_push_channel] = ACTIONS(2259),
    [sym_push_devchan] = ACTIONS(2259),
    [sym_release_device] = ACTIONS(2259),
    [sym_release_channel] = ACTIONS(2259),
    [sym_release_devchan] = ACTIONS(2259),
    [sym_master_slot] = ACTIONS(2259),
    [sym_get_pulse_time] = ACTIONS(2259),
    [sym_get_timer] = ACTIONS(2259),
    [sym_master_sn] = ACTIONS(2259),
    [sym_system_number] = ACTIONS(2259),
    [sym_button] = ACTIONS(2259),
    [sym_dv_channel] = ACTIONS(2259),
    [sym_data] = ACTIONS(2259),
    [sym_level] = ACTIONS(2259),
    [sym_timeline] = ACTIONS(2259),
    [sym_channel] = ACTIONS(2259),
    [sym_custom] = ACTIONS(2259),
    [sym_atoi] = ACTIONS(2259),
    [sym_atol] = ACTIONS(2259),
    [sym_atof] = ACTIONS(2259),
    [sym_itoa] = ACTIONS(2259),
    [sym_itohex] = ACTIONS(2259),
    [sym_hextoi] = ACTIONS(2259),
    [sym_ftoa] = ACTIONS(2259),
    [sym_find_string] = ACTIONS(2259),
    [sym_left_string] = ACTIONS(2259),
    [sym_right_string] = ACTIONS(2259),
    [sym_mid_string] = ACTIONS(2259),
    [sym_remove_string] = ACTIONS(2259),
    [sym_get_buffer_string] = ACTIONS(2259),
    [sym_get_buffer_char] = ACTIONS(2259),
    [sym_get_multi_buffer_string] = ACTIONS(2259),
    [sym_length_string] = ACTIONS(2259),
    [sym_max_length_string] = ACTIONS(2259),
    [sym_set_length_string] = ACTIONS(2259),
    [sym_compare_string] = ACTIONS(2259),
    [sym_upper_string] = ACTIONS(2259),
    [sym_lower_string] = ACTIONS(2259),
    [sym_get_last] = ACTIONS(2259),
    [sym_date_to_day] = ACTIONS(2259),
    [sym_date_to_month] = ACTIONS(2259),
    [sym_date_to_year] = ACTIONS(2259),
    [sym_time_to_hour] = ACTIONS(2259),
    [sym_time_to_minute] = ACTIONS(2259),
    [sym_time_to_second] = ACTIONS(2259),
    [sym_day_of_week] = ACTIONS(2259),
    [sym_set_pulse_time] = ACTIONS(2259),
    [sym_set_timer] = ACTIONS(2259),
    [sym_file_open] = ACTIONS(2259),
    [sym_file_close] = ACTIONS(2259),
    [sym_file_read] = ACTIONS(2259),
    [sym_file_read_line] = ACTIONS(2259),
    [sym_file_write] = ACTIONS(2259),
    [sym_file_write_line] = ACTIONS(2259),
    [sym_file_copy] = ACTIONS(2259),
    [sym_file_delete] = ACTIONS(2259),
    [sym_file_rename] = ACTIONS(2259),
    [sym_file_dir] = ACTIONS(2259),
    [sym_file_getdir] = ACTIONS(2259),
    [sym_file_setdir] = ACTIONS(2259),
    [sym_file_create_dir] = ACTIONS(2259),
    [sym_file_remove_dir] = ACTIONS(2259),
    [sym_file_seek] = ACTIONS(2259),
    [sym_length_array] = ACTIONS(2259),
    [sym_max_length_array] = ACTIONS(2259),
    [sym_set_length_array] = ACTIONS(2259),
    [sym_ip_client_open] = ACTIONS(2259),
    [sym_ip_client_close] = ACTIONS(2259),
    [sym_ip_server_open] = ACTIONS(2259),
    [sym_ip_server_close] = ACTIONS(2259),
    [sym_ip_bound_client_open] = ACTIONS(2259),
    [sym_ip_mc_server_open] = ACTIONS(2259),
    [sym_ip_set_option] = ACTIONS(2259),
    [sym_ssh_client_open] = ACTIONS(2259),
    [sym_ssh_client_close] = ACTIONS(2259),
    [sym_tls_client_open] = ACTIONS(2259),
    [sym_tls_client_close] = ACTIONS(2259),
    [sym_do_push] = ACTIONS(2259),
    [sym_do_push_timed] = ACTIONS(2259),
    [sym_do_release] = ACTIONS(2259),
    [sym_device_id] = ACTIONS(2259),
    [sym_device_info] = ACTIONS(2259),
    [sym_device_id_string] = ACTIONS(2259),
    [sym_device_standby] = ACTIONS(2259),
    [sym_device_wake] = ACTIONS(2259),
    [sym_timeline_create] = ACTIONS(2259),
    [sym_timeline_kill] = ACTIONS(2259),
    [sym_timeline_reload] = ACTIONS(2259),
    [sym_timeline_pause] = ACTIONS(2259),
    [sym_timeline_restart] = ACTIONS(2259),
    [sym_timeline_set] = ACTIONS(2259),
    [sym_timeline_get] = ACTIONS(2259),
    [sym_timeline_active] = ACTIONS(2259),
    [sym_timeline_dynamic_id] = ACTIONS(2259),
    [sym_variable_to_string] = ACTIONS(2259),
    [sym_string_to_variable] = ACTIONS(2259),
    [sym_length_variable_to_string] = ACTIONS(2259),
    [sym_type_cast] = ACTIONS(2259),
    [sym_variable_to_xml] = ACTIONS(2259),
    [sym_xml_to_variable] = ACTIONS(2259),
    [sym_length_variable_to_xml] = ACTIONS(2259),
    [sym_redirect_string] = ACTIONS(2259),
    [sym_random_number] = ACTIONS(2259),
    [sym_get_unique_id] = ACTIONS(2259),
    [sym_get_system_number] = ACTIONS(2259),
    [sym_set_system_number] = ACTIONS(2259),
    [sym_get_serial_number] = ACTIONS(2259),
    [sym_get_master_build] = ACTIONS(2259),
    [sym_set_validation_code] = ACTIONS(2259),
    [sym_reboot] = ACTIONS(2259),
    [sym_load_duet_module] = ACTIONS(2259),
    [sym_unload_duet_module] = ACTIONS(2259),
    [sym_duet_mem_size_set] = ACTIONS(2259),
    [sym_duet_mem_size_get] = ACTIONS(2259),
    [sym_dynamic_application_device] = ACTIONS(2259),
    [sym_dynamic_polled_port] = ACTIONS(2259),
    [sym_static_port_binding] = ACTIONS(2259),
    [sym_static_ip_binding] = ACTIONS(2259),
    [sym_get_dns_list] = ACTIONS(2259),
    [sym_set_dns_list] = ACTIONS(2259),
    [sym_get_ip_address] = ACTIONS(2259),
    [sym_set_ip_address] = ACTIONS(2259),
    [sym_get_url_list] = ACTIONS(2259),
    [sym_add_url_entry] = ACTIONS(2259),
    [sym_delete_url_entry] = ACTIONS(2259),
    [sym_abs_value] = ACTIONS(2259),
    [sym_max_value] = ACTIONS(2259),
    [sym_min_value] = ACTIONS(2259),
    [sym_exp_value] = ACTIONS(2259),
    [sym_log_value] = ACTIONS(2259),
    [sym_log10_value] = ACTIONS(2259),
    [sym_power_value] = ACTIONS(2259),
    [sym_sqrt_value] = ACTIONS(2259),
    [sym_raw_be] = ACTIONS(2259),
    [sym_raw_le] = ACTIONS(2259),
    [sym_internal_queue_size_set] = ACTIONS(2259),
    [sym_internal_queue_size_get] = ACTIONS(2259),
    [sym_internal_threshold_set] = ACTIONS(2259),
    [sym_internal_threshold_get] = ACTIONS(2259),
    [sym_rebuilt_event] = ACTIONS(2259),
    [sym_do_custom_event] = ACTIONS(2259),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2259),
    [sym_clkmgr_set_clk_source] = ACTIONS(2259),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2259),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2259),
    [sym_clkmgr_get_timezone] = ACTIONS(2259),
    [sym_clkmgr_set_timezone] = ACTIONS(2259),
    [sym_clkmgr_get_resync_period] = ACTIONS(2259),
    [sym_clkmgr_set_resync_period] = ACTIONS(2259),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2259),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2259),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2259),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2259),
    [sym_clkmgr_get_timeservers] = ACTIONS(2259),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2259),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2259),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2259),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2259),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2259),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2259),
    [sym_set_log_level] = ACTIONS(2259),
    [sym_get_log_level] = ACTIONS(2259),
    [sym_amx_log] = ACTIONS(2259),
    [sym_smtp_server_config_set] = ACTIONS(2259),
    [sym_smtp_server_config_get] = ACTIONS(2259),
    [sym_smtp_send] = ACTIONS(2259),
    [sym_validate_netlinx_account] = ACTIONS(2259),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2259),
    [sym_authenticate_certificate] = ACTIONS(2259),
    [sym_audit_netlinx_session_event] = ACTIONS(2259),
    [sym_audit_netlinx_generic_event] = ACTIONS(2259),
    [sym_get_available_flash_disk_space] = ACTIONS(2259),
    [sym_get_max_flash_disk_space] = ACTIONS(2259),
    [sym_data_create_feed] = ACTIONS(2259),
    [sym_data_delete_feed] = ACTIONS(2259),
    [sym_data_publish_feed] = ACTIONS(2259),
    [sym_data_get_published_feed] = ACTIONS(2259),
    [sym_data_add_record] = ACTIONS(2259),
    [sym_data_get_event_record] = ACTIONS(2259),
    [sym__wc_data_create_feed] = ACTIONS(2259),
    [sym__wc_data_add_record] = ACTIONS(2259),
    [sym__wc_data_get_event_record] = ACTIONS(2259),
    [sym_format] = ACTIONS(2259),
    [sym_astro_clock] = ACTIONS(2259),
    [sym_set_outdoor_temperature] = ACTIONS(2259),
    [sym_set_virtual_level_count] = ACTIONS(2259),
    [sym_set_virtual_channel_count] = ACTIONS(2259),
    [sym_set_virtual_port_count] = ACTIONS(2259),
    [sym__wc_encode_int] = ACTIONS(2259),
    [sym__wc_decode_int] = ACTIONS(2259),
    [sym__wc_int] = ACTIONS(2259),
    [sym__wc_to_ch_int] = ACTIONS(2259),
    [sym__ch_to_wc_int] = ACTIONS(2259),
    [sym__wc_find_string_int] = ACTIONS(2259),
    [sym__wc_left_string_int] = ACTIONS(2259),
    [sym__wc_lower_string_int] = ACTIONS(2259),
    [sym__wc_mid_string_int] = ACTIONS(2259),
    [sym__wc_remove_string_int] = ACTIONS(2259),
    [sym__wc_right_string_int] = ACTIONS(2259),
    [sym__wc_upper_string_int] = ACTIONS(2259),
    [sym__wc_compare_string_int] = ACTIONS(2259),
    [sym__wc_get_buffer_char_int] = ACTIONS(2259),
    [sym__wc_get_buffer_string_int] = ACTIONS(2259),
    [sym__wc_concat_string_int] = ACTIONS(2259),
    [sym__wc_file_open_int] = ACTIONS(2259),
    [sym__wc_file_close_int] = ACTIONS(2259),
    [sym__wc_file_read_int] = ACTIONS(2259),
    [sym__wc_file_read_line_int] = ACTIONS(2259),
    [sym__wc_file_write_int] = ACTIONS(2259),
    [sym__wc_file_write_line_int] = ACTIONS(2259),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2261),
    [aux_sym_preproc_else_token1] = ACTIONS(2261),
    [anon_sym_COMMA] = ACTIONS(2263),
    [anon_sym_LBRACK] = ACTIONS(2261),
    [anon_sym_LBRACE] = ACTIONS(2261),
    [anon_sym_LPAREN] = ACTIONS(2259),
    [anon_sym_BANG] = ACTIONS(2261),
    [anon_sym_TILDE] = ACTIONS(2261),
    [anon_sym_DASH] = ACTIONS(2259),
    [anon_sym_PLUS] = ACTIONS(2259),
    [anon_sym_DASH_DASH] = ACTIONS(2261),
    [anon_sym_PLUS_PLUS] = ACTIONS(2261),
    [anon_sym_DQUOTE] = ACTIONS(2261),
    [anon_sym_SQUOTE] = ACTIONS(2261),
    [sym_number_literal] = ACTIONS(2261),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(947)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2244),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(948)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2229),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(949)] = {
    [sym_identifier] = ACTIONS(2831),
    [sym_not] = ACTIONS(2831),
    [sym_bnot] = ACTIONS(2831),
    [sym___file__] = ACTIONS(2831),
    [sym___date__] = ACTIONS(2831),
    [sym___ldate__] = ACTIONS(2831),
    [sym___time__] = ACTIONS(2831),
    [sym___line__] = ACTIONS(2831),
    [sym___name__] = ACTIONS(2831),
    [sym___netlinx__] = ACTIONS(2831),
    [sym_netlinx_axi_version] = ACTIONS(2831),
    [sym_url_flg_tcp] = ACTIONS(2831),
    [sym_url_flg_accinfopresent] = ACTIONS(2831),
    [sym_url_flg_temp] = ACTIONS(2831),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2831),
    [sym_url_flg_stat_mask] = ACTIONS(2831),
    [sym_url_flg_stat_lookup] = ACTIONS(2831),
    [sym_url_flg_stat_connecting] = ACTIONS(2831),
    [sym_url_flg_stat_waiting] = ACTIONS(2831),
    [sym_url_flg_stat_connected] = ACTIONS(2831),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2831),
    [sym_first_virtual_device] = ACTIONS(2831),
    [sym_first_local_port] = ACTIONS(2831),
    [sym_dynamic_virtual_device] = ACTIONS(2831),
    [sym_timeline_once] = ACTIONS(2831),
    [sym_timeline_repeat] = ACTIONS(2831),
    [sym_timeline_absolute] = ACTIONS(2831),
    [sym_timeline_relative] = ACTIONS(2831),
    [sym_do_push_timed_infinite] = ACTIONS(2831),
    [sym_source_type_no_address] = ACTIONS(2831),
    [sym_source_type_neuron_id] = ACTIONS(2831),
    [sym_source_type_ip_address] = ACTIONS(2831),
    [sym_source_type_axlink] = ACTIONS(2831),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2831),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2831),
    [sym_source_type_ip_socket_address] = ACTIONS(2831),
    [sym_source_type_rs232] = ACTIONS(2831),
    [sym_source_type_internal] = ACTIONS(2831),
    [sym_source_type_mac_address] = ACTIONS(2831),
    [sym_source_type_ipv4_port] = ACTIONS(2831),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2831),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2831),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2831),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2831),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2831),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2831),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2831),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2831),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2831),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2831),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2831),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2831),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2831),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2831),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2831),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2831),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2831),
    [sym_true] = ACTIONS(2831),
    [sym_false] = ACTIONS(2831),
    [sym_file_read_only] = ACTIONS(2831),
    [sym_file_rw_new] = ACTIONS(2831),
    [sym_file_rw_append] = ACTIONS(2831),
    [sym_ip_tcp] = ACTIONS(2831),
    [sym_ip_udp] = ACTIONS(2831),
    [sym_ip_udp_2way] = ACTIONS(2831),
    [sym_xml_encode_types] = ACTIONS(2831),
    [sym_xml_encode_char_as_list] = ACTIONS(2831),
    [sym_xml_encode_le] = ACTIONS(2831),
    [sym_xml_decode_types] = ACTIONS(2831),
    [sym_xml_decode_no_preserve] = ACTIONS(2831),
    [sym_duet_dev_type_display_device] = ACTIONS(2831),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2831),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2831),
    [sym_duet_dev_type_io_device] = ACTIONS(2831),
    [sym_duet_dev_type_relay_device] = ACTIONS(2831),
    [sym_duet_dev_type_ups] = ACTIONS(2831),
    [sym_duet_dev_type_amplifier] = ACTIONS(2831),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2831),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2831),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2831),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2831),
    [sym_duet_dev_type_camera] = ACTIONS(2831),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2831),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2831),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2831),
    [sym_duet_dev_type_dss] = ACTIONS(2831),
    [sym_duet_dev_type_dvr] = ACTIONS(2831),
    [sym_duet_dev_type_disc_device] = ACTIONS(2831),
    [sym_duet_dev_type_document_camera] = ACTIONS(2831),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2831),
    [sym_duet_dev_type_hvac] = ACTIONS(2831),
    [sym_duet_dev_type_keypad] = ACTIONS(2831),
    [sym_duet_dev_type_light] = ACTIONS(2831),
    [sym_duet_dev_type_monitor] = ACTIONS(2831),
    [sym_duet_dev_type_motor] = ACTIONS(2831),
    [sym_duet_dev_type_multi_window] = ACTIONS(2831),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2831),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2831),
    [sym_duet_dev_type_receiver] = ACTIONS(2831),
    [sym_duet_dev_type_security_system] = ACTIONS(2831),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2831),
    [sym_duet_dev_type_settop_box] = ACTIONS(2831),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2831),
    [sym_duet_dev_type_switcher] = ACTIONS(2831),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2831),
    [sym_duet_dev_type_tv] = ACTIONS(2831),
    [sym_duet_dev_type_utility] = ACTIONS(2831),
    [sym_duet_dev_type_vcr] = ACTIONS(2831),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2831),
    [sym_duet_dev_type_video_processor] = ACTIONS(2831),
    [sym_duet_dev_type_video_projector] = ACTIONS(2831),
    [sym_duet_dev_type_video_wall] = ACTIONS(2831),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2831),
    [sym_duet_dev_type_weather] = ACTIONS(2831),
    [sym_duet_dev_not_polled] = ACTIONS(2831),
    [sym_duet_dev_polled] = ACTIONS(2831),
    [sym_clkmgr_mode_network] = ACTIONS(2831),
    [sym_clkmgr_mode_standalone] = ACTIONS(2831),
    [sym_normal_standby] = ACTIONS(2831),
    [sym_normal_wake] = ACTIONS(2831),
    [sym_ip_multicast_ttl_option] = ACTIONS(2831),
    [sym_ip_ttl_subnet] = ACTIONS(2831),
    [sym_ip_ttl_site] = ACTIONS(2831),
    [sym_ip_ttl_region] = ACTIONS(2831),
    [sym_ip_ttl_continent] = ACTIONS(2831),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2831),
    [sym_ip_nodelay_on] = ACTIONS(2831),
    [sym_ip_nodelay_off] = ACTIONS(2831),
    [sym_amx_error] = ACTIONS(2831),
    [sym_amx_warning] = ACTIONS(2831),
    [sym_amx_info] = ACTIONS(2831),
    [sym_amx_debug] = ACTIONS(2831),
    [sym_netlinx_login_success] = ACTIONS(2831),
    [sym_netlinx_login_fail] = ACTIONS(2831),
    [sym_netlinx_logout] = ACTIONS(2831),
    [sym_null_str] = ACTIONS(2831),
    [sym_smtp_address] = ACTIONS(2831),
    [sym_smtp_port_number] = ACTIONS(2831),
    [sym_smtp_username] = ACTIONS(2831),
    [sym_smtp_password] = ACTIONS(2831),
    [sym_smtp_require_tls] = ACTIONS(2831),
    [sym_smtp_from] = ACTIONS(2831),
    [sym_smtp_tls_true] = ACTIONS(2831),
    [sym_smtp_tls_false] = ACTIONS(2831),
    [sym_listview_on_row_select_event] = ACTIONS(2831),
    [sym_data_structure_unknown] = ACTIONS(2831),
    [sym_data_structure_datafeed] = ACTIONS(2831),
    [sym_data_structure_datarecordset] = ACTIONS(2831),
    [sym_data_structure_datarecord] = ACTIONS(2831),
    [sym_data_structure_datafield] = ACTIONS(2831),
    [sym_data_max_content_field_count] = ACTIONS(2831),
    [sym_data_max_metadata_field_count] = ACTIONS(2831),
    [sym_data_max_id_length] = ACTIONS(2831),
    [sym_data_max_name_length] = ACTIONS(2831),
    [sym_data_max_description_length] = ACTIONS(2831),
    [sym_data_max_type_length] = ACTIONS(2831),
    [sym_data_max_format_length] = ACTIONS(2831),
    [sym_data_max_source_length] = ACTIONS(2831),
    [sym_data_max_value_length] = ACTIONS(2831),
    [sym_data_max_label_length] = ACTIONS(2831),
    [sym_data_max_url_length] = ACTIONS(2831),
    [sym_max_last_login_info_length] = ACTIONS(2831),
    [sym_data_type_unknown] = ACTIONS(2831),
    [sym_data_type_string] = ACTIONS(2831),
    [sym_data_type_datetime] = ACTIONS(2831),
    [sym_data_type_date] = ACTIONS(2831),
    [sym_data_type_time] = ACTIONS(2831),
    [sym_data_type_image] = ACTIONS(2831),
    [sym_data_format_url] = ACTIONS(2831),
    [sym_data_format_phone] = ACTIONS(2831),
    [sym_data_format_email] = ACTIONS(2831),
    [sym_data_format_iso8601] = ACTIONS(2831),
    [sym_auditlog_priv] = ACTIONS(2831),
    [sym_configuration_priv] = ACTIONS(2831),
    [sym_deviceconfig_priv] = ACTIONS(2831),
    [sym_ftp_priv] = ACTIONS(2831),
    [sym_http_priv] = ACTIONS(2831),
    [sym_networkconfig_priv] = ACTIONS(2831),
    [sym_programport_priv] = ACTIONS(2831),
    [sym_remoteui_priv] = ACTIONS(2831),
    [sym_securitycontrol_priv] = ACTIONS(2831),
    [sym_softwaremanagement_priv] = ACTIONS(2831),
    [sym_terminal_priv] = ACTIONS(2831),
    [sym_tpadmin_priv] = ACTIONS(2831),
    [sym_user1_priv] = ACTIONS(2831),
    [sym_user2_priv] = ACTIONS(2831),
    [sym_user3_priv] = ACTIONS(2831),
    [sym_user4_priv] = ACTIONS(2831),
    [sym_usermanagement_priv] = ACTIONS(2831),
    [sym_valid_account] = ACTIONS(2831),
    [sym_err_name_invalid] = ACTIONS(2831),
    [sym_err_password_invalid] = ACTIONS(2831),
    [sym_err_invalid_account] = ACTIONS(2831),
    [sym_err_no_authorization] = ACTIONS(2831),
    [sym_err_invalid_parameter] = ACTIONS(2831),
    [sym_err_user_locked_out] = ACTIONS(2831),
    [sym_err_user_account_expired] = ACTIONS(2831),
    [sym_err_general_error] = ACTIONS(2831),
    [sym_err_authentication_error] = ACTIONS(2831),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2831),
    [sym_tls_validate_certificate] = ACTIONS(2831),
    [sym____reserved___] = ACTIONS(2831),
    [sym_date] = ACTIONS(2831),
    [sym_day] = ACTIONS(2831),
    [sym_time] = ACTIONS(2831),
    [sym_ldate] = ACTIONS(2831),
    [sym_push_device] = ACTIONS(2831),
    [sym_push_channel] = ACTIONS(2831),
    [sym_push_devchan] = ACTIONS(2831),
    [sym_release_device] = ACTIONS(2831),
    [sym_release_channel] = ACTIONS(2831),
    [sym_release_devchan] = ACTIONS(2831),
    [sym_master_slot] = ACTIONS(2831),
    [sym_get_pulse_time] = ACTIONS(2831),
    [sym_get_timer] = ACTIONS(2831),
    [sym_master_sn] = ACTIONS(2831),
    [sym_system_number] = ACTIONS(2831),
    [sym_button] = ACTIONS(2831),
    [sym_dv_channel] = ACTIONS(2831),
    [sym_data] = ACTIONS(2831),
    [sym_level] = ACTIONS(2831),
    [sym_timeline] = ACTIONS(2831),
    [sym_channel] = ACTIONS(2831),
    [sym_custom] = ACTIONS(2831),
    [sym_atoi] = ACTIONS(2831),
    [sym_atol] = ACTIONS(2831),
    [sym_atof] = ACTIONS(2831),
    [sym_itoa] = ACTIONS(2831),
    [sym_itohex] = ACTIONS(2831),
    [sym_hextoi] = ACTIONS(2831),
    [sym_ftoa] = ACTIONS(2831),
    [sym_find_string] = ACTIONS(2831),
    [sym_left_string] = ACTIONS(2831),
    [sym_right_string] = ACTIONS(2831),
    [sym_mid_string] = ACTIONS(2831),
    [sym_remove_string] = ACTIONS(2831),
    [sym_get_buffer_string] = ACTIONS(2831),
    [sym_get_buffer_char] = ACTIONS(2831),
    [sym_get_multi_buffer_string] = ACTIONS(2831),
    [sym_length_string] = ACTIONS(2831),
    [sym_max_length_string] = ACTIONS(2831),
    [sym_set_length_string] = ACTIONS(2831),
    [sym_compare_string] = ACTIONS(2831),
    [sym_upper_string] = ACTIONS(2831),
    [sym_lower_string] = ACTIONS(2831),
    [sym_get_last] = ACTIONS(2831),
    [sym_date_to_day] = ACTIONS(2831),
    [sym_date_to_month] = ACTIONS(2831),
    [sym_date_to_year] = ACTIONS(2831),
    [sym_time_to_hour] = ACTIONS(2831),
    [sym_time_to_minute] = ACTIONS(2831),
    [sym_time_to_second] = ACTIONS(2831),
    [sym_day_of_week] = ACTIONS(2831),
    [sym_set_pulse_time] = ACTIONS(2831),
    [sym_set_timer] = ACTIONS(2831),
    [sym_file_open] = ACTIONS(2831),
    [sym_file_close] = ACTIONS(2831),
    [sym_file_read] = ACTIONS(2831),
    [sym_file_read_line] = ACTIONS(2831),
    [sym_file_write] = ACTIONS(2831),
    [sym_file_write_line] = ACTIONS(2831),
    [sym_file_copy] = ACTIONS(2831),
    [sym_file_delete] = ACTIONS(2831),
    [sym_file_rename] = ACTIONS(2831),
    [sym_file_dir] = ACTIONS(2831),
    [sym_file_getdir] = ACTIONS(2831),
    [sym_file_setdir] = ACTIONS(2831),
    [sym_file_create_dir] = ACTIONS(2831),
    [sym_file_remove_dir] = ACTIONS(2831),
    [sym_file_seek] = ACTIONS(2831),
    [sym_length_array] = ACTIONS(2831),
    [sym_max_length_array] = ACTIONS(2831),
    [sym_set_length_array] = ACTIONS(2831),
    [sym_ip_client_open] = ACTIONS(2831),
    [sym_ip_client_close] = ACTIONS(2831),
    [sym_ip_server_open] = ACTIONS(2831),
    [sym_ip_server_close] = ACTIONS(2831),
    [sym_ip_bound_client_open] = ACTIONS(2831),
    [sym_ip_mc_server_open] = ACTIONS(2831),
    [sym_ip_set_option] = ACTIONS(2831),
    [sym_ssh_client_open] = ACTIONS(2831),
    [sym_ssh_client_close] = ACTIONS(2831),
    [sym_tls_client_open] = ACTIONS(2831),
    [sym_tls_client_close] = ACTIONS(2831),
    [sym_do_push] = ACTIONS(2831),
    [sym_do_push_timed] = ACTIONS(2831),
    [sym_do_release] = ACTIONS(2831),
    [sym_device_id] = ACTIONS(2831),
    [sym_device_info] = ACTIONS(2831),
    [sym_device_id_string] = ACTIONS(2831),
    [sym_device_standby] = ACTIONS(2831),
    [sym_device_wake] = ACTIONS(2831),
    [sym_timeline_create] = ACTIONS(2831),
    [sym_timeline_kill] = ACTIONS(2831),
    [sym_timeline_reload] = ACTIONS(2831),
    [sym_timeline_pause] = ACTIONS(2831),
    [sym_timeline_restart] = ACTIONS(2831),
    [sym_timeline_set] = ACTIONS(2831),
    [sym_timeline_get] = ACTIONS(2831),
    [sym_timeline_active] = ACTIONS(2831),
    [sym_timeline_dynamic_id] = ACTIONS(2831),
    [sym_variable_to_string] = ACTIONS(2831),
    [sym_string_to_variable] = ACTIONS(2831),
    [sym_length_variable_to_string] = ACTIONS(2831),
    [sym_type_cast] = ACTIONS(2831),
    [sym_variable_to_xml] = ACTIONS(2831),
    [sym_xml_to_variable] = ACTIONS(2831),
    [sym_length_variable_to_xml] = ACTIONS(2831),
    [sym_redirect_string] = ACTIONS(2831),
    [sym_random_number] = ACTIONS(2831),
    [sym_get_unique_id] = ACTIONS(2831),
    [sym_get_system_number] = ACTIONS(2831),
    [sym_set_system_number] = ACTIONS(2831),
    [sym_get_serial_number] = ACTIONS(2831),
    [sym_get_master_build] = ACTIONS(2831),
    [sym_set_validation_code] = ACTIONS(2831),
    [sym_reboot] = ACTIONS(2831),
    [sym_load_duet_module] = ACTIONS(2831),
    [sym_unload_duet_module] = ACTIONS(2831),
    [sym_duet_mem_size_set] = ACTIONS(2831),
    [sym_duet_mem_size_get] = ACTIONS(2831),
    [sym_dynamic_application_device] = ACTIONS(2831),
    [sym_dynamic_polled_port] = ACTIONS(2831),
    [sym_static_port_binding] = ACTIONS(2831),
    [sym_static_ip_binding] = ACTIONS(2831),
    [sym_get_dns_list] = ACTIONS(2831),
    [sym_set_dns_list] = ACTIONS(2831),
    [sym_get_ip_address] = ACTIONS(2831),
    [sym_set_ip_address] = ACTIONS(2831),
    [sym_get_url_list] = ACTIONS(2831),
    [sym_add_url_entry] = ACTIONS(2831),
    [sym_delete_url_entry] = ACTIONS(2831),
    [sym_abs_value] = ACTIONS(2831),
    [sym_max_value] = ACTIONS(2831),
    [sym_min_value] = ACTIONS(2831),
    [sym_exp_value] = ACTIONS(2831),
    [sym_log_value] = ACTIONS(2831),
    [sym_log10_value] = ACTIONS(2831),
    [sym_power_value] = ACTIONS(2831),
    [sym_sqrt_value] = ACTIONS(2831),
    [sym_raw_be] = ACTIONS(2831),
    [sym_raw_le] = ACTIONS(2831),
    [sym_internal_queue_size_set] = ACTIONS(2831),
    [sym_internal_queue_size_get] = ACTIONS(2831),
    [sym_internal_threshold_set] = ACTIONS(2831),
    [sym_internal_threshold_get] = ACTIONS(2831),
    [sym_rebuilt_event] = ACTIONS(2831),
    [sym_do_custom_event] = ACTIONS(2831),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2831),
    [sym_clkmgr_set_clk_source] = ACTIONS(2831),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2831),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2831),
    [sym_clkmgr_get_timezone] = ACTIONS(2831),
    [sym_clkmgr_set_timezone] = ACTIONS(2831),
    [sym_clkmgr_get_resync_period] = ACTIONS(2831),
    [sym_clkmgr_set_resync_period] = ACTIONS(2831),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2831),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2831),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2831),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2831),
    [sym_clkmgr_get_timeservers] = ACTIONS(2831),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2831),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2831),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2831),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2831),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2831),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2831),
    [sym_set_log_level] = ACTIONS(2831),
    [sym_get_log_level] = ACTIONS(2831),
    [sym_amx_log] = ACTIONS(2831),
    [sym_smtp_server_config_set] = ACTIONS(2831),
    [sym_smtp_server_config_get] = ACTIONS(2831),
    [sym_smtp_send] = ACTIONS(2831),
    [sym_validate_netlinx_account] = ACTIONS(2831),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2831),
    [sym_authenticate_certificate] = ACTIONS(2831),
    [sym_audit_netlinx_session_event] = ACTIONS(2831),
    [sym_audit_netlinx_generic_event] = ACTIONS(2831),
    [sym_get_available_flash_disk_space] = ACTIONS(2831),
    [sym_get_max_flash_disk_space] = ACTIONS(2831),
    [sym_data_create_feed] = ACTIONS(2831),
    [sym_data_delete_feed] = ACTIONS(2831),
    [sym_data_publish_feed] = ACTIONS(2831),
    [sym_data_get_published_feed] = ACTIONS(2831),
    [sym_data_add_record] = ACTIONS(2831),
    [sym_data_get_event_record] = ACTIONS(2831),
    [sym__wc_data_create_feed] = ACTIONS(2831),
    [sym__wc_data_add_record] = ACTIONS(2831),
    [sym__wc_data_get_event_record] = ACTIONS(2831),
    [sym_format] = ACTIONS(2831),
    [sym_astro_clock] = ACTIONS(2831),
    [sym_set_outdoor_temperature] = ACTIONS(2831),
    [sym_set_virtual_level_count] = ACTIONS(2831),
    [sym_set_virtual_channel_count] = ACTIONS(2831),
    [sym_set_virtual_port_count] = ACTIONS(2831),
    [sym__wc_encode_int] = ACTIONS(2831),
    [sym__wc_decode_int] = ACTIONS(2831),
    [sym__wc_int] = ACTIONS(2831),
    [sym__wc_to_ch_int] = ACTIONS(2831),
    [sym__ch_to_wc_int] = ACTIONS(2831),
    [sym__wc_find_string_int] = ACTIONS(2831),
    [sym__wc_left_string_int] = ACTIONS(2831),
    [sym__wc_lower_string_int] = ACTIONS(2831),
    [sym__wc_mid_string_int] = ACTIONS(2831),
    [sym__wc_remove_string_int] = ACTIONS(2831),
    [sym__wc_right_string_int] = ACTIONS(2831),
    [sym__wc_upper_string_int] = ACTIONS(2831),
    [sym__wc_compare_string_int] = ACTIONS(2831),
    [sym__wc_get_buffer_char_int] = ACTIONS(2831),
    [sym__wc_get_buffer_string_int] = ACTIONS(2831),
    [sym__wc_concat_string_int] = ACTIONS(2831),
    [sym__wc_file_open_int] = ACTIONS(2831),
    [sym__wc_file_close_int] = ACTIONS(2831),
    [sym__wc_file_read_int] = ACTIONS(2831),
    [sym__wc_file_read_line_int] = ACTIONS(2831),
    [sym__wc_file_write_int] = ACTIONS(2831),
    [sym__wc_file_write_line_int] = ACTIONS(2831),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2833),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2833),
    [anon_sym_LBRACK] = ACTIONS(2833),
    [anon_sym_LBRACE] = ACTIONS(2833),
    [anon_sym_RBRACE] = ACTIONS(2833),
    [anon_sym_LPAREN] = ACTIONS(2831),
    [anon_sym_BANG] = ACTIONS(2833),
    [anon_sym_TILDE] = ACTIONS(2833),
    [anon_sym_DASH] = ACTIONS(2831),
    [anon_sym_PLUS] = ACTIONS(2831),
    [anon_sym_DASH_DASH] = ACTIONS(2833),
    [anon_sym_PLUS_PLUS] = ACTIONS(2833),
    [anon_sym_DQUOTE] = ACTIONS(2833),
    [anon_sym_SQUOTE] = ACTIONS(2833),
    [sym_number_literal] = ACTIONS(2833),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(950)] = {
    [sym_identifier] = ACTIONS(2835),
    [sym_not] = ACTIONS(2835),
    [sym_bnot] = ACTIONS(2835),
    [sym___file__] = ACTIONS(2835),
    [sym___date__] = ACTIONS(2835),
    [sym___ldate__] = ACTIONS(2835),
    [sym___time__] = ACTIONS(2835),
    [sym___line__] = ACTIONS(2835),
    [sym___name__] = ACTIONS(2835),
    [sym___netlinx__] = ACTIONS(2835),
    [sym_netlinx_axi_version] = ACTIONS(2835),
    [sym_url_flg_tcp] = ACTIONS(2835),
    [sym_url_flg_accinfopresent] = ACTIONS(2835),
    [sym_url_flg_temp] = ACTIONS(2835),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2835),
    [sym_url_flg_stat_mask] = ACTIONS(2835),
    [sym_url_flg_stat_lookup] = ACTIONS(2835),
    [sym_url_flg_stat_connecting] = ACTIONS(2835),
    [sym_url_flg_stat_waiting] = ACTIONS(2835),
    [sym_url_flg_stat_connected] = ACTIONS(2835),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2835),
    [sym_first_virtual_device] = ACTIONS(2835),
    [sym_first_local_port] = ACTIONS(2835),
    [sym_dynamic_virtual_device] = ACTIONS(2835),
    [sym_timeline_once] = ACTIONS(2835),
    [sym_timeline_repeat] = ACTIONS(2835),
    [sym_timeline_absolute] = ACTIONS(2835),
    [sym_timeline_relative] = ACTIONS(2835),
    [sym_do_push_timed_infinite] = ACTIONS(2835),
    [sym_source_type_no_address] = ACTIONS(2835),
    [sym_source_type_neuron_id] = ACTIONS(2835),
    [sym_source_type_ip_address] = ACTIONS(2835),
    [sym_source_type_axlink] = ACTIONS(2835),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2835),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2835),
    [sym_source_type_ip_socket_address] = ACTIONS(2835),
    [sym_source_type_rs232] = ACTIONS(2835),
    [sym_source_type_internal] = ACTIONS(2835),
    [sym_source_type_mac_address] = ACTIONS(2835),
    [sym_source_type_ipv4_port] = ACTIONS(2835),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2835),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2835),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2835),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2835),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2835),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2835),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2835),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2835),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2835),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2835),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2835),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2835),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2835),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2835),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2835),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2835),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2835),
    [sym_true] = ACTIONS(2835),
    [sym_false] = ACTIONS(2835),
    [sym_file_read_only] = ACTIONS(2835),
    [sym_file_rw_new] = ACTIONS(2835),
    [sym_file_rw_append] = ACTIONS(2835),
    [sym_ip_tcp] = ACTIONS(2835),
    [sym_ip_udp] = ACTIONS(2835),
    [sym_ip_udp_2way] = ACTIONS(2835),
    [sym_xml_encode_types] = ACTIONS(2835),
    [sym_xml_encode_char_as_list] = ACTIONS(2835),
    [sym_xml_encode_le] = ACTIONS(2835),
    [sym_xml_decode_types] = ACTIONS(2835),
    [sym_xml_decode_no_preserve] = ACTIONS(2835),
    [sym_duet_dev_type_display_device] = ACTIONS(2835),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2835),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2835),
    [sym_duet_dev_type_io_device] = ACTIONS(2835),
    [sym_duet_dev_type_relay_device] = ACTIONS(2835),
    [sym_duet_dev_type_ups] = ACTIONS(2835),
    [sym_duet_dev_type_amplifier] = ACTIONS(2835),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2835),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2835),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2835),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2835),
    [sym_duet_dev_type_camera] = ACTIONS(2835),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2835),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2835),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2835),
    [sym_duet_dev_type_dss] = ACTIONS(2835),
    [sym_duet_dev_type_dvr] = ACTIONS(2835),
    [sym_duet_dev_type_disc_device] = ACTIONS(2835),
    [sym_duet_dev_type_document_camera] = ACTIONS(2835),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2835),
    [sym_duet_dev_type_hvac] = ACTIONS(2835),
    [sym_duet_dev_type_keypad] = ACTIONS(2835),
    [sym_duet_dev_type_light] = ACTIONS(2835),
    [sym_duet_dev_type_monitor] = ACTIONS(2835),
    [sym_duet_dev_type_motor] = ACTIONS(2835),
    [sym_duet_dev_type_multi_window] = ACTIONS(2835),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2835),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2835),
    [sym_duet_dev_type_receiver] = ACTIONS(2835),
    [sym_duet_dev_type_security_system] = ACTIONS(2835),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2835),
    [sym_duet_dev_type_settop_box] = ACTIONS(2835),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2835),
    [sym_duet_dev_type_switcher] = ACTIONS(2835),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2835),
    [sym_duet_dev_type_tv] = ACTIONS(2835),
    [sym_duet_dev_type_utility] = ACTIONS(2835),
    [sym_duet_dev_type_vcr] = ACTIONS(2835),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2835),
    [sym_duet_dev_type_video_processor] = ACTIONS(2835),
    [sym_duet_dev_type_video_projector] = ACTIONS(2835),
    [sym_duet_dev_type_video_wall] = ACTIONS(2835),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2835),
    [sym_duet_dev_type_weather] = ACTIONS(2835),
    [sym_duet_dev_not_polled] = ACTIONS(2835),
    [sym_duet_dev_polled] = ACTIONS(2835),
    [sym_clkmgr_mode_network] = ACTIONS(2835),
    [sym_clkmgr_mode_standalone] = ACTIONS(2835),
    [sym_normal_standby] = ACTIONS(2835),
    [sym_normal_wake] = ACTIONS(2835),
    [sym_ip_multicast_ttl_option] = ACTIONS(2835),
    [sym_ip_ttl_subnet] = ACTIONS(2835),
    [sym_ip_ttl_site] = ACTIONS(2835),
    [sym_ip_ttl_region] = ACTIONS(2835),
    [sym_ip_ttl_continent] = ACTIONS(2835),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2835),
    [sym_ip_nodelay_on] = ACTIONS(2835),
    [sym_ip_nodelay_off] = ACTIONS(2835),
    [sym_amx_error] = ACTIONS(2835),
    [sym_amx_warning] = ACTIONS(2835),
    [sym_amx_info] = ACTIONS(2835),
    [sym_amx_debug] = ACTIONS(2835),
    [sym_netlinx_login_success] = ACTIONS(2835),
    [sym_netlinx_login_fail] = ACTIONS(2835),
    [sym_netlinx_logout] = ACTIONS(2835),
    [sym_null_str] = ACTIONS(2835),
    [sym_smtp_address] = ACTIONS(2835),
    [sym_smtp_port_number] = ACTIONS(2835),
    [sym_smtp_username] = ACTIONS(2835),
    [sym_smtp_password] = ACTIONS(2835),
    [sym_smtp_require_tls] = ACTIONS(2835),
    [sym_smtp_from] = ACTIONS(2835),
    [sym_smtp_tls_true] = ACTIONS(2835),
    [sym_smtp_tls_false] = ACTIONS(2835),
    [sym_listview_on_row_select_event] = ACTIONS(2835),
    [sym_data_structure_unknown] = ACTIONS(2835),
    [sym_data_structure_datafeed] = ACTIONS(2835),
    [sym_data_structure_datarecordset] = ACTIONS(2835),
    [sym_data_structure_datarecord] = ACTIONS(2835),
    [sym_data_structure_datafield] = ACTIONS(2835),
    [sym_data_max_content_field_count] = ACTIONS(2835),
    [sym_data_max_metadata_field_count] = ACTIONS(2835),
    [sym_data_max_id_length] = ACTIONS(2835),
    [sym_data_max_name_length] = ACTIONS(2835),
    [sym_data_max_description_length] = ACTIONS(2835),
    [sym_data_max_type_length] = ACTIONS(2835),
    [sym_data_max_format_length] = ACTIONS(2835),
    [sym_data_max_source_length] = ACTIONS(2835),
    [sym_data_max_value_length] = ACTIONS(2835),
    [sym_data_max_label_length] = ACTIONS(2835),
    [sym_data_max_url_length] = ACTIONS(2835),
    [sym_max_last_login_info_length] = ACTIONS(2835),
    [sym_data_type_unknown] = ACTIONS(2835),
    [sym_data_type_string] = ACTIONS(2835),
    [sym_data_type_datetime] = ACTIONS(2835),
    [sym_data_type_date] = ACTIONS(2835),
    [sym_data_type_time] = ACTIONS(2835),
    [sym_data_type_image] = ACTIONS(2835),
    [sym_data_format_url] = ACTIONS(2835),
    [sym_data_format_phone] = ACTIONS(2835),
    [sym_data_format_email] = ACTIONS(2835),
    [sym_data_format_iso8601] = ACTIONS(2835),
    [sym_auditlog_priv] = ACTIONS(2835),
    [sym_configuration_priv] = ACTIONS(2835),
    [sym_deviceconfig_priv] = ACTIONS(2835),
    [sym_ftp_priv] = ACTIONS(2835),
    [sym_http_priv] = ACTIONS(2835),
    [sym_networkconfig_priv] = ACTIONS(2835),
    [sym_programport_priv] = ACTIONS(2835),
    [sym_remoteui_priv] = ACTIONS(2835),
    [sym_securitycontrol_priv] = ACTIONS(2835),
    [sym_softwaremanagement_priv] = ACTIONS(2835),
    [sym_terminal_priv] = ACTIONS(2835),
    [sym_tpadmin_priv] = ACTIONS(2835),
    [sym_user1_priv] = ACTIONS(2835),
    [sym_user2_priv] = ACTIONS(2835),
    [sym_user3_priv] = ACTIONS(2835),
    [sym_user4_priv] = ACTIONS(2835),
    [sym_usermanagement_priv] = ACTIONS(2835),
    [sym_valid_account] = ACTIONS(2835),
    [sym_err_name_invalid] = ACTIONS(2835),
    [sym_err_password_invalid] = ACTIONS(2835),
    [sym_err_invalid_account] = ACTIONS(2835),
    [sym_err_no_authorization] = ACTIONS(2835),
    [sym_err_invalid_parameter] = ACTIONS(2835),
    [sym_err_user_locked_out] = ACTIONS(2835),
    [sym_err_user_account_expired] = ACTIONS(2835),
    [sym_err_general_error] = ACTIONS(2835),
    [sym_err_authentication_error] = ACTIONS(2835),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2835),
    [sym_tls_validate_certificate] = ACTIONS(2835),
    [sym____reserved___] = ACTIONS(2835),
    [sym_date] = ACTIONS(2835),
    [sym_day] = ACTIONS(2835),
    [sym_time] = ACTIONS(2835),
    [sym_ldate] = ACTIONS(2835),
    [sym_push_device] = ACTIONS(2835),
    [sym_push_channel] = ACTIONS(2835),
    [sym_push_devchan] = ACTIONS(2835),
    [sym_release_device] = ACTIONS(2835),
    [sym_release_channel] = ACTIONS(2835),
    [sym_release_devchan] = ACTIONS(2835),
    [sym_master_slot] = ACTIONS(2835),
    [sym_get_pulse_time] = ACTIONS(2835),
    [sym_get_timer] = ACTIONS(2835),
    [sym_master_sn] = ACTIONS(2835),
    [sym_system_number] = ACTIONS(2835),
    [sym_button] = ACTIONS(2835),
    [sym_dv_channel] = ACTIONS(2835),
    [sym_data] = ACTIONS(2835),
    [sym_level] = ACTIONS(2835),
    [sym_timeline] = ACTIONS(2835),
    [sym_channel] = ACTIONS(2835),
    [sym_custom] = ACTIONS(2835),
    [sym_atoi] = ACTIONS(2835),
    [sym_atol] = ACTIONS(2835),
    [sym_atof] = ACTIONS(2835),
    [sym_itoa] = ACTIONS(2835),
    [sym_itohex] = ACTIONS(2835),
    [sym_hextoi] = ACTIONS(2835),
    [sym_ftoa] = ACTIONS(2835),
    [sym_find_string] = ACTIONS(2835),
    [sym_left_string] = ACTIONS(2835),
    [sym_right_string] = ACTIONS(2835),
    [sym_mid_string] = ACTIONS(2835),
    [sym_remove_string] = ACTIONS(2835),
    [sym_get_buffer_string] = ACTIONS(2835),
    [sym_get_buffer_char] = ACTIONS(2835),
    [sym_get_multi_buffer_string] = ACTIONS(2835),
    [sym_length_string] = ACTIONS(2835),
    [sym_max_length_string] = ACTIONS(2835),
    [sym_set_length_string] = ACTIONS(2835),
    [sym_compare_string] = ACTIONS(2835),
    [sym_upper_string] = ACTIONS(2835),
    [sym_lower_string] = ACTIONS(2835),
    [sym_get_last] = ACTIONS(2835),
    [sym_date_to_day] = ACTIONS(2835),
    [sym_date_to_month] = ACTIONS(2835),
    [sym_date_to_year] = ACTIONS(2835),
    [sym_time_to_hour] = ACTIONS(2835),
    [sym_time_to_minute] = ACTIONS(2835),
    [sym_time_to_second] = ACTIONS(2835),
    [sym_day_of_week] = ACTIONS(2835),
    [sym_set_pulse_time] = ACTIONS(2835),
    [sym_set_timer] = ACTIONS(2835),
    [sym_file_open] = ACTIONS(2835),
    [sym_file_close] = ACTIONS(2835),
    [sym_file_read] = ACTIONS(2835),
    [sym_file_read_line] = ACTIONS(2835),
    [sym_file_write] = ACTIONS(2835),
    [sym_file_write_line] = ACTIONS(2835),
    [sym_file_copy] = ACTIONS(2835),
    [sym_file_delete] = ACTIONS(2835),
    [sym_file_rename] = ACTIONS(2835),
    [sym_file_dir] = ACTIONS(2835),
    [sym_file_getdir] = ACTIONS(2835),
    [sym_file_setdir] = ACTIONS(2835),
    [sym_file_create_dir] = ACTIONS(2835),
    [sym_file_remove_dir] = ACTIONS(2835),
    [sym_file_seek] = ACTIONS(2835),
    [sym_length_array] = ACTIONS(2835),
    [sym_max_length_array] = ACTIONS(2835),
    [sym_set_length_array] = ACTIONS(2835),
    [sym_ip_client_open] = ACTIONS(2835),
    [sym_ip_client_close] = ACTIONS(2835),
    [sym_ip_server_open] = ACTIONS(2835),
    [sym_ip_server_close] = ACTIONS(2835),
    [sym_ip_bound_client_open] = ACTIONS(2835),
    [sym_ip_mc_server_open] = ACTIONS(2835),
    [sym_ip_set_option] = ACTIONS(2835),
    [sym_ssh_client_open] = ACTIONS(2835),
    [sym_ssh_client_close] = ACTIONS(2835),
    [sym_tls_client_open] = ACTIONS(2835),
    [sym_tls_client_close] = ACTIONS(2835),
    [sym_do_push] = ACTIONS(2835),
    [sym_do_push_timed] = ACTIONS(2835),
    [sym_do_release] = ACTIONS(2835),
    [sym_device_id] = ACTIONS(2835),
    [sym_device_info] = ACTIONS(2835),
    [sym_device_id_string] = ACTIONS(2835),
    [sym_device_standby] = ACTIONS(2835),
    [sym_device_wake] = ACTIONS(2835),
    [sym_timeline_create] = ACTIONS(2835),
    [sym_timeline_kill] = ACTIONS(2835),
    [sym_timeline_reload] = ACTIONS(2835),
    [sym_timeline_pause] = ACTIONS(2835),
    [sym_timeline_restart] = ACTIONS(2835),
    [sym_timeline_set] = ACTIONS(2835),
    [sym_timeline_get] = ACTIONS(2835),
    [sym_timeline_active] = ACTIONS(2835),
    [sym_timeline_dynamic_id] = ACTIONS(2835),
    [sym_variable_to_string] = ACTIONS(2835),
    [sym_string_to_variable] = ACTIONS(2835),
    [sym_length_variable_to_string] = ACTIONS(2835),
    [sym_type_cast] = ACTIONS(2835),
    [sym_variable_to_xml] = ACTIONS(2835),
    [sym_xml_to_variable] = ACTIONS(2835),
    [sym_length_variable_to_xml] = ACTIONS(2835),
    [sym_redirect_string] = ACTIONS(2835),
    [sym_random_number] = ACTIONS(2835),
    [sym_get_unique_id] = ACTIONS(2835),
    [sym_get_system_number] = ACTIONS(2835),
    [sym_set_system_number] = ACTIONS(2835),
    [sym_get_serial_number] = ACTIONS(2835),
    [sym_get_master_build] = ACTIONS(2835),
    [sym_set_validation_code] = ACTIONS(2835),
    [sym_reboot] = ACTIONS(2835),
    [sym_load_duet_module] = ACTIONS(2835),
    [sym_unload_duet_module] = ACTIONS(2835),
    [sym_duet_mem_size_set] = ACTIONS(2835),
    [sym_duet_mem_size_get] = ACTIONS(2835),
    [sym_dynamic_application_device] = ACTIONS(2835),
    [sym_dynamic_polled_port] = ACTIONS(2835),
    [sym_static_port_binding] = ACTIONS(2835),
    [sym_static_ip_binding] = ACTIONS(2835),
    [sym_get_dns_list] = ACTIONS(2835),
    [sym_set_dns_list] = ACTIONS(2835),
    [sym_get_ip_address] = ACTIONS(2835),
    [sym_set_ip_address] = ACTIONS(2835),
    [sym_get_url_list] = ACTIONS(2835),
    [sym_add_url_entry] = ACTIONS(2835),
    [sym_delete_url_entry] = ACTIONS(2835),
    [sym_abs_value] = ACTIONS(2835),
    [sym_max_value] = ACTIONS(2835),
    [sym_min_value] = ACTIONS(2835),
    [sym_exp_value] = ACTIONS(2835),
    [sym_log_value] = ACTIONS(2835),
    [sym_log10_value] = ACTIONS(2835),
    [sym_power_value] = ACTIONS(2835),
    [sym_sqrt_value] = ACTIONS(2835),
    [sym_raw_be] = ACTIONS(2835),
    [sym_raw_le] = ACTIONS(2835),
    [sym_internal_queue_size_set] = ACTIONS(2835),
    [sym_internal_queue_size_get] = ACTIONS(2835),
    [sym_internal_threshold_set] = ACTIONS(2835),
    [sym_internal_threshold_get] = ACTIONS(2835),
    [sym_rebuilt_event] = ACTIONS(2835),
    [sym_do_custom_event] = ACTIONS(2835),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2835),
    [sym_clkmgr_set_clk_source] = ACTIONS(2835),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2835),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2835),
    [sym_clkmgr_get_timezone] = ACTIONS(2835),
    [sym_clkmgr_set_timezone] = ACTIONS(2835),
    [sym_clkmgr_get_resync_period] = ACTIONS(2835),
    [sym_clkmgr_set_resync_period] = ACTIONS(2835),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2835),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2835),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2835),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2835),
    [sym_clkmgr_get_timeservers] = ACTIONS(2835),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2835),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2835),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2835),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2835),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2835),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2835),
    [sym_set_log_level] = ACTIONS(2835),
    [sym_get_log_level] = ACTIONS(2835),
    [sym_amx_log] = ACTIONS(2835),
    [sym_smtp_server_config_set] = ACTIONS(2835),
    [sym_smtp_server_config_get] = ACTIONS(2835),
    [sym_smtp_send] = ACTIONS(2835),
    [sym_validate_netlinx_account] = ACTIONS(2835),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2835),
    [sym_authenticate_certificate] = ACTIONS(2835),
    [sym_audit_netlinx_session_event] = ACTIONS(2835),
    [sym_audit_netlinx_generic_event] = ACTIONS(2835),
    [sym_get_available_flash_disk_space] = ACTIONS(2835),
    [sym_get_max_flash_disk_space] = ACTIONS(2835),
    [sym_data_create_feed] = ACTIONS(2835),
    [sym_data_delete_feed] = ACTIONS(2835),
    [sym_data_publish_feed] = ACTIONS(2835),
    [sym_data_get_published_feed] = ACTIONS(2835),
    [sym_data_add_record] = ACTIONS(2835),
    [sym_data_get_event_record] = ACTIONS(2835),
    [sym__wc_data_create_feed] = ACTIONS(2835),
    [sym__wc_data_add_record] = ACTIONS(2835),
    [sym__wc_data_get_event_record] = ACTIONS(2835),
    [sym_format] = ACTIONS(2835),
    [sym_astro_clock] = ACTIONS(2835),
    [sym_set_outdoor_temperature] = ACTIONS(2835),
    [sym_set_virtual_level_count] = ACTIONS(2835),
    [sym_set_virtual_channel_count] = ACTIONS(2835),
    [sym_set_virtual_port_count] = ACTIONS(2835),
    [sym__wc_encode_int] = ACTIONS(2835),
    [sym__wc_decode_int] = ACTIONS(2835),
    [sym__wc_int] = ACTIONS(2835),
    [sym__wc_to_ch_int] = ACTIONS(2835),
    [sym__ch_to_wc_int] = ACTIONS(2835),
    [sym__wc_find_string_int] = ACTIONS(2835),
    [sym__wc_left_string_int] = ACTIONS(2835),
    [sym__wc_lower_string_int] = ACTIONS(2835),
    [sym__wc_mid_string_int] = ACTIONS(2835),
    [sym__wc_remove_string_int] = ACTIONS(2835),
    [sym__wc_right_string_int] = ACTIONS(2835),
    [sym__wc_upper_string_int] = ACTIONS(2835),
    [sym__wc_compare_string_int] = ACTIONS(2835),
    [sym__wc_get_buffer_char_int] = ACTIONS(2835),
    [sym__wc_get_buffer_string_int] = ACTIONS(2835),
    [sym__wc_concat_string_int] = ACTIONS(2835),
    [sym__wc_file_open_int] = ACTIONS(2835),
    [sym__wc_file_close_int] = ACTIONS(2835),
    [sym__wc_file_read_int] = ACTIONS(2835),
    [sym__wc_file_read_line_int] = ACTIONS(2835),
    [sym__wc_file_write_int] = ACTIONS(2835),
    [sym__wc_file_write_line_int] = ACTIONS(2835),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2837),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2837),
    [anon_sym_LBRACK] = ACTIONS(2837),
    [anon_sym_LBRACE] = ACTIONS(2837),
    [anon_sym_RBRACE] = ACTIONS(2837),
    [anon_sym_LPAREN] = ACTIONS(2835),
    [anon_sym_BANG] = ACTIONS(2837),
    [anon_sym_TILDE] = ACTIONS(2837),
    [anon_sym_DASH] = ACTIONS(2835),
    [anon_sym_PLUS] = ACTIONS(2835),
    [anon_sym_DASH_DASH] = ACTIONS(2837),
    [anon_sym_PLUS_PLUS] = ACTIONS(2837),
    [anon_sym_DQUOTE] = ACTIONS(2837),
    [anon_sym_SQUOTE] = ACTIONS(2837),
    [sym_number_literal] = ACTIONS(2837),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(951)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2247),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(952)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2256),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(953)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2250),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(954)] = {
    [sym_identifier] = ACTIONS(2839),
    [sym_not] = ACTIONS(2839),
    [sym_bnot] = ACTIONS(2839),
    [sym___file__] = ACTIONS(2839),
    [sym___date__] = ACTIONS(2839),
    [sym___ldate__] = ACTIONS(2839),
    [sym___time__] = ACTIONS(2839),
    [sym___line__] = ACTIONS(2839),
    [sym___name__] = ACTIONS(2839),
    [sym___netlinx__] = ACTIONS(2839),
    [sym_netlinx_axi_version] = ACTIONS(2839),
    [sym_url_flg_tcp] = ACTIONS(2839),
    [sym_url_flg_accinfopresent] = ACTIONS(2839),
    [sym_url_flg_temp] = ACTIONS(2839),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2839),
    [sym_url_flg_stat_mask] = ACTIONS(2839),
    [sym_url_flg_stat_lookup] = ACTIONS(2839),
    [sym_url_flg_stat_connecting] = ACTIONS(2839),
    [sym_url_flg_stat_waiting] = ACTIONS(2839),
    [sym_url_flg_stat_connected] = ACTIONS(2839),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2839),
    [sym_first_virtual_device] = ACTIONS(2839),
    [sym_first_local_port] = ACTIONS(2839),
    [sym_dynamic_virtual_device] = ACTIONS(2839),
    [sym_timeline_once] = ACTIONS(2839),
    [sym_timeline_repeat] = ACTIONS(2839),
    [sym_timeline_absolute] = ACTIONS(2839),
    [sym_timeline_relative] = ACTIONS(2839),
    [sym_do_push_timed_infinite] = ACTIONS(2839),
    [sym_source_type_no_address] = ACTIONS(2839),
    [sym_source_type_neuron_id] = ACTIONS(2839),
    [sym_source_type_ip_address] = ACTIONS(2839),
    [sym_source_type_axlink] = ACTIONS(2839),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2839),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2839),
    [sym_source_type_ip_socket_address] = ACTIONS(2839),
    [sym_source_type_rs232] = ACTIONS(2839),
    [sym_source_type_internal] = ACTIONS(2839),
    [sym_source_type_mac_address] = ACTIONS(2839),
    [sym_source_type_ipv4_port] = ACTIONS(2839),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2839),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2839),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2839),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2839),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2839),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2839),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2839),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2839),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2839),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2839),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2839),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2839),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2839),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2839),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2839),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2839),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2839),
    [sym_true] = ACTIONS(2839),
    [sym_false] = ACTIONS(2839),
    [sym_file_read_only] = ACTIONS(2839),
    [sym_file_rw_new] = ACTIONS(2839),
    [sym_file_rw_append] = ACTIONS(2839),
    [sym_ip_tcp] = ACTIONS(2839),
    [sym_ip_udp] = ACTIONS(2839),
    [sym_ip_udp_2way] = ACTIONS(2839),
    [sym_xml_encode_types] = ACTIONS(2839),
    [sym_xml_encode_char_as_list] = ACTIONS(2839),
    [sym_xml_encode_le] = ACTIONS(2839),
    [sym_xml_decode_types] = ACTIONS(2839),
    [sym_xml_decode_no_preserve] = ACTIONS(2839),
    [sym_duet_dev_type_display_device] = ACTIONS(2839),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2839),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2839),
    [sym_duet_dev_type_io_device] = ACTIONS(2839),
    [sym_duet_dev_type_relay_device] = ACTIONS(2839),
    [sym_duet_dev_type_ups] = ACTIONS(2839),
    [sym_duet_dev_type_amplifier] = ACTIONS(2839),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2839),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2839),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2839),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2839),
    [sym_duet_dev_type_camera] = ACTIONS(2839),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2839),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2839),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2839),
    [sym_duet_dev_type_dss] = ACTIONS(2839),
    [sym_duet_dev_type_dvr] = ACTIONS(2839),
    [sym_duet_dev_type_disc_device] = ACTIONS(2839),
    [sym_duet_dev_type_document_camera] = ACTIONS(2839),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2839),
    [sym_duet_dev_type_hvac] = ACTIONS(2839),
    [sym_duet_dev_type_keypad] = ACTIONS(2839),
    [sym_duet_dev_type_light] = ACTIONS(2839),
    [sym_duet_dev_type_monitor] = ACTIONS(2839),
    [sym_duet_dev_type_motor] = ACTIONS(2839),
    [sym_duet_dev_type_multi_window] = ACTIONS(2839),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2839),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2839),
    [sym_duet_dev_type_receiver] = ACTIONS(2839),
    [sym_duet_dev_type_security_system] = ACTIONS(2839),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2839),
    [sym_duet_dev_type_settop_box] = ACTIONS(2839),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2839),
    [sym_duet_dev_type_switcher] = ACTIONS(2839),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2839),
    [sym_duet_dev_type_tv] = ACTIONS(2839),
    [sym_duet_dev_type_utility] = ACTIONS(2839),
    [sym_duet_dev_type_vcr] = ACTIONS(2839),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2839),
    [sym_duet_dev_type_video_processor] = ACTIONS(2839),
    [sym_duet_dev_type_video_projector] = ACTIONS(2839),
    [sym_duet_dev_type_video_wall] = ACTIONS(2839),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2839),
    [sym_duet_dev_type_weather] = ACTIONS(2839),
    [sym_duet_dev_not_polled] = ACTIONS(2839),
    [sym_duet_dev_polled] = ACTIONS(2839),
    [sym_clkmgr_mode_network] = ACTIONS(2839),
    [sym_clkmgr_mode_standalone] = ACTIONS(2839),
    [sym_normal_standby] = ACTIONS(2839),
    [sym_normal_wake] = ACTIONS(2839),
    [sym_ip_multicast_ttl_option] = ACTIONS(2839),
    [sym_ip_ttl_subnet] = ACTIONS(2839),
    [sym_ip_ttl_site] = ACTIONS(2839),
    [sym_ip_ttl_region] = ACTIONS(2839),
    [sym_ip_ttl_continent] = ACTIONS(2839),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2839),
    [sym_ip_nodelay_on] = ACTIONS(2839),
    [sym_ip_nodelay_off] = ACTIONS(2839),
    [sym_amx_error] = ACTIONS(2839),
    [sym_amx_warning] = ACTIONS(2839),
    [sym_amx_info] = ACTIONS(2839),
    [sym_amx_debug] = ACTIONS(2839),
    [sym_netlinx_login_success] = ACTIONS(2839),
    [sym_netlinx_login_fail] = ACTIONS(2839),
    [sym_netlinx_logout] = ACTIONS(2839),
    [sym_null_str] = ACTIONS(2839),
    [sym_smtp_address] = ACTIONS(2839),
    [sym_smtp_port_number] = ACTIONS(2839),
    [sym_smtp_username] = ACTIONS(2839),
    [sym_smtp_password] = ACTIONS(2839),
    [sym_smtp_require_tls] = ACTIONS(2839),
    [sym_smtp_from] = ACTIONS(2839),
    [sym_smtp_tls_true] = ACTIONS(2839),
    [sym_smtp_tls_false] = ACTIONS(2839),
    [sym_listview_on_row_select_event] = ACTIONS(2839),
    [sym_data_structure_unknown] = ACTIONS(2839),
    [sym_data_structure_datafeed] = ACTIONS(2839),
    [sym_data_structure_datarecordset] = ACTIONS(2839),
    [sym_data_structure_datarecord] = ACTIONS(2839),
    [sym_data_structure_datafield] = ACTIONS(2839),
    [sym_data_max_content_field_count] = ACTIONS(2839),
    [sym_data_max_metadata_field_count] = ACTIONS(2839),
    [sym_data_max_id_length] = ACTIONS(2839),
    [sym_data_max_name_length] = ACTIONS(2839),
    [sym_data_max_description_length] = ACTIONS(2839),
    [sym_data_max_type_length] = ACTIONS(2839),
    [sym_data_max_format_length] = ACTIONS(2839),
    [sym_data_max_source_length] = ACTIONS(2839),
    [sym_data_max_value_length] = ACTIONS(2839),
    [sym_data_max_label_length] = ACTIONS(2839),
    [sym_data_max_url_length] = ACTIONS(2839),
    [sym_max_last_login_info_length] = ACTIONS(2839),
    [sym_data_type_unknown] = ACTIONS(2839),
    [sym_data_type_string] = ACTIONS(2839),
    [sym_data_type_datetime] = ACTIONS(2839),
    [sym_data_type_date] = ACTIONS(2839),
    [sym_data_type_time] = ACTIONS(2839),
    [sym_data_type_image] = ACTIONS(2839),
    [sym_data_format_url] = ACTIONS(2839),
    [sym_data_format_phone] = ACTIONS(2839),
    [sym_data_format_email] = ACTIONS(2839),
    [sym_data_format_iso8601] = ACTIONS(2839),
    [sym_auditlog_priv] = ACTIONS(2839),
    [sym_configuration_priv] = ACTIONS(2839),
    [sym_deviceconfig_priv] = ACTIONS(2839),
    [sym_ftp_priv] = ACTIONS(2839),
    [sym_http_priv] = ACTIONS(2839),
    [sym_networkconfig_priv] = ACTIONS(2839),
    [sym_programport_priv] = ACTIONS(2839),
    [sym_remoteui_priv] = ACTIONS(2839),
    [sym_securitycontrol_priv] = ACTIONS(2839),
    [sym_softwaremanagement_priv] = ACTIONS(2839),
    [sym_terminal_priv] = ACTIONS(2839),
    [sym_tpadmin_priv] = ACTIONS(2839),
    [sym_user1_priv] = ACTIONS(2839),
    [sym_user2_priv] = ACTIONS(2839),
    [sym_user3_priv] = ACTIONS(2839),
    [sym_user4_priv] = ACTIONS(2839),
    [sym_usermanagement_priv] = ACTIONS(2839),
    [sym_valid_account] = ACTIONS(2839),
    [sym_err_name_invalid] = ACTIONS(2839),
    [sym_err_password_invalid] = ACTIONS(2839),
    [sym_err_invalid_account] = ACTIONS(2839),
    [sym_err_no_authorization] = ACTIONS(2839),
    [sym_err_invalid_parameter] = ACTIONS(2839),
    [sym_err_user_locked_out] = ACTIONS(2839),
    [sym_err_user_account_expired] = ACTIONS(2839),
    [sym_err_general_error] = ACTIONS(2839),
    [sym_err_authentication_error] = ACTIONS(2839),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2839),
    [sym_tls_validate_certificate] = ACTIONS(2839),
    [sym____reserved___] = ACTIONS(2839),
    [sym_date] = ACTIONS(2839),
    [sym_day] = ACTIONS(2839),
    [sym_time] = ACTIONS(2839),
    [sym_ldate] = ACTIONS(2839),
    [sym_push_device] = ACTIONS(2839),
    [sym_push_channel] = ACTIONS(2839),
    [sym_push_devchan] = ACTIONS(2839),
    [sym_release_device] = ACTIONS(2839),
    [sym_release_channel] = ACTIONS(2839),
    [sym_release_devchan] = ACTIONS(2839),
    [sym_master_slot] = ACTIONS(2839),
    [sym_get_pulse_time] = ACTIONS(2839),
    [sym_get_timer] = ACTIONS(2839),
    [sym_master_sn] = ACTIONS(2839),
    [sym_system_number] = ACTIONS(2839),
    [sym_button] = ACTIONS(2839),
    [sym_dv_channel] = ACTIONS(2839),
    [sym_data] = ACTIONS(2839),
    [sym_level] = ACTIONS(2839),
    [sym_timeline] = ACTIONS(2839),
    [sym_channel] = ACTIONS(2839),
    [sym_custom] = ACTIONS(2839),
    [sym_atoi] = ACTIONS(2839),
    [sym_atol] = ACTIONS(2839),
    [sym_atof] = ACTIONS(2839),
    [sym_itoa] = ACTIONS(2839),
    [sym_itohex] = ACTIONS(2839),
    [sym_hextoi] = ACTIONS(2839),
    [sym_ftoa] = ACTIONS(2839),
    [sym_find_string] = ACTIONS(2839),
    [sym_left_string] = ACTIONS(2839),
    [sym_right_string] = ACTIONS(2839),
    [sym_mid_string] = ACTIONS(2839),
    [sym_remove_string] = ACTIONS(2839),
    [sym_get_buffer_string] = ACTIONS(2839),
    [sym_get_buffer_char] = ACTIONS(2839),
    [sym_get_multi_buffer_string] = ACTIONS(2839),
    [sym_length_string] = ACTIONS(2839),
    [sym_max_length_string] = ACTIONS(2839),
    [sym_set_length_string] = ACTIONS(2839),
    [sym_compare_string] = ACTIONS(2839),
    [sym_upper_string] = ACTIONS(2839),
    [sym_lower_string] = ACTIONS(2839),
    [sym_get_last] = ACTIONS(2839),
    [sym_date_to_day] = ACTIONS(2839),
    [sym_date_to_month] = ACTIONS(2839),
    [sym_date_to_year] = ACTIONS(2839),
    [sym_time_to_hour] = ACTIONS(2839),
    [sym_time_to_minute] = ACTIONS(2839),
    [sym_time_to_second] = ACTIONS(2839),
    [sym_day_of_week] = ACTIONS(2839),
    [sym_set_pulse_time] = ACTIONS(2839),
    [sym_set_timer] = ACTIONS(2839),
    [sym_file_open] = ACTIONS(2839),
    [sym_file_close] = ACTIONS(2839),
    [sym_file_read] = ACTIONS(2839),
    [sym_file_read_line] = ACTIONS(2839),
    [sym_file_write] = ACTIONS(2839),
    [sym_file_write_line] = ACTIONS(2839),
    [sym_file_copy] = ACTIONS(2839),
    [sym_file_delete] = ACTIONS(2839),
    [sym_file_rename] = ACTIONS(2839),
    [sym_file_dir] = ACTIONS(2839),
    [sym_file_getdir] = ACTIONS(2839),
    [sym_file_setdir] = ACTIONS(2839),
    [sym_file_create_dir] = ACTIONS(2839),
    [sym_file_remove_dir] = ACTIONS(2839),
    [sym_file_seek] = ACTIONS(2839),
    [sym_length_array] = ACTIONS(2839),
    [sym_max_length_array] = ACTIONS(2839),
    [sym_set_length_array] = ACTIONS(2839),
    [sym_ip_client_open] = ACTIONS(2839),
    [sym_ip_client_close] = ACTIONS(2839),
    [sym_ip_server_open] = ACTIONS(2839),
    [sym_ip_server_close] = ACTIONS(2839),
    [sym_ip_bound_client_open] = ACTIONS(2839),
    [sym_ip_mc_server_open] = ACTIONS(2839),
    [sym_ip_set_option] = ACTIONS(2839),
    [sym_ssh_client_open] = ACTIONS(2839),
    [sym_ssh_client_close] = ACTIONS(2839),
    [sym_tls_client_open] = ACTIONS(2839),
    [sym_tls_client_close] = ACTIONS(2839),
    [sym_do_push] = ACTIONS(2839),
    [sym_do_push_timed] = ACTIONS(2839),
    [sym_do_release] = ACTIONS(2839),
    [sym_device_id] = ACTIONS(2839),
    [sym_device_info] = ACTIONS(2839),
    [sym_device_id_string] = ACTIONS(2839),
    [sym_device_standby] = ACTIONS(2839),
    [sym_device_wake] = ACTIONS(2839),
    [sym_timeline_create] = ACTIONS(2839),
    [sym_timeline_kill] = ACTIONS(2839),
    [sym_timeline_reload] = ACTIONS(2839),
    [sym_timeline_pause] = ACTIONS(2839),
    [sym_timeline_restart] = ACTIONS(2839),
    [sym_timeline_set] = ACTIONS(2839),
    [sym_timeline_get] = ACTIONS(2839),
    [sym_timeline_active] = ACTIONS(2839),
    [sym_timeline_dynamic_id] = ACTIONS(2839),
    [sym_variable_to_string] = ACTIONS(2839),
    [sym_string_to_variable] = ACTIONS(2839),
    [sym_length_variable_to_string] = ACTIONS(2839),
    [sym_type_cast] = ACTIONS(2839),
    [sym_variable_to_xml] = ACTIONS(2839),
    [sym_xml_to_variable] = ACTIONS(2839),
    [sym_length_variable_to_xml] = ACTIONS(2839),
    [sym_redirect_string] = ACTIONS(2839),
    [sym_random_number] = ACTIONS(2839),
    [sym_get_unique_id] = ACTIONS(2839),
    [sym_get_system_number] = ACTIONS(2839),
    [sym_set_system_number] = ACTIONS(2839),
    [sym_get_serial_number] = ACTIONS(2839),
    [sym_get_master_build] = ACTIONS(2839),
    [sym_set_validation_code] = ACTIONS(2839),
    [sym_reboot] = ACTIONS(2839),
    [sym_load_duet_module] = ACTIONS(2839),
    [sym_unload_duet_module] = ACTIONS(2839),
    [sym_duet_mem_size_set] = ACTIONS(2839),
    [sym_duet_mem_size_get] = ACTIONS(2839),
    [sym_dynamic_application_device] = ACTIONS(2839),
    [sym_dynamic_polled_port] = ACTIONS(2839),
    [sym_static_port_binding] = ACTIONS(2839),
    [sym_static_ip_binding] = ACTIONS(2839),
    [sym_get_dns_list] = ACTIONS(2839),
    [sym_set_dns_list] = ACTIONS(2839),
    [sym_get_ip_address] = ACTIONS(2839),
    [sym_set_ip_address] = ACTIONS(2839),
    [sym_get_url_list] = ACTIONS(2839),
    [sym_add_url_entry] = ACTIONS(2839),
    [sym_delete_url_entry] = ACTIONS(2839),
    [sym_abs_value] = ACTIONS(2839),
    [sym_max_value] = ACTIONS(2839),
    [sym_min_value] = ACTIONS(2839),
    [sym_exp_value] = ACTIONS(2839),
    [sym_log_value] = ACTIONS(2839),
    [sym_log10_value] = ACTIONS(2839),
    [sym_power_value] = ACTIONS(2839),
    [sym_sqrt_value] = ACTIONS(2839),
    [sym_raw_be] = ACTIONS(2839),
    [sym_raw_le] = ACTIONS(2839),
    [sym_internal_queue_size_set] = ACTIONS(2839),
    [sym_internal_queue_size_get] = ACTIONS(2839),
    [sym_internal_threshold_set] = ACTIONS(2839),
    [sym_internal_threshold_get] = ACTIONS(2839),
    [sym_rebuilt_event] = ACTIONS(2839),
    [sym_do_custom_event] = ACTIONS(2839),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2839),
    [sym_clkmgr_set_clk_source] = ACTIONS(2839),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2839),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2839),
    [sym_clkmgr_get_timezone] = ACTIONS(2839),
    [sym_clkmgr_set_timezone] = ACTIONS(2839),
    [sym_clkmgr_get_resync_period] = ACTIONS(2839),
    [sym_clkmgr_set_resync_period] = ACTIONS(2839),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2839),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2839),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2839),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2839),
    [sym_clkmgr_get_timeservers] = ACTIONS(2839),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2839),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2839),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2839),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2839),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2839),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2839),
    [sym_set_log_level] = ACTIONS(2839),
    [sym_get_log_level] = ACTIONS(2839),
    [sym_amx_log] = ACTIONS(2839),
    [sym_smtp_server_config_set] = ACTIONS(2839),
    [sym_smtp_server_config_get] = ACTIONS(2839),
    [sym_smtp_send] = ACTIONS(2839),
    [sym_validate_netlinx_account] = ACTIONS(2839),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2839),
    [sym_authenticate_certificate] = ACTIONS(2839),
    [sym_audit_netlinx_session_event] = ACTIONS(2839),
    [sym_audit_netlinx_generic_event] = ACTIONS(2839),
    [sym_get_available_flash_disk_space] = ACTIONS(2839),
    [sym_get_max_flash_disk_space] = ACTIONS(2839),
    [sym_data_create_feed] = ACTIONS(2839),
    [sym_data_delete_feed] = ACTIONS(2839),
    [sym_data_publish_feed] = ACTIONS(2839),
    [sym_data_get_published_feed] = ACTIONS(2839),
    [sym_data_add_record] = ACTIONS(2839),
    [sym_data_get_event_record] = ACTIONS(2839),
    [sym__wc_data_create_feed] = ACTIONS(2839),
    [sym__wc_data_add_record] = ACTIONS(2839),
    [sym__wc_data_get_event_record] = ACTIONS(2839),
    [sym_format] = ACTIONS(2839),
    [sym_astro_clock] = ACTIONS(2839),
    [sym_set_outdoor_temperature] = ACTIONS(2839),
    [sym_set_virtual_level_count] = ACTIONS(2839),
    [sym_set_virtual_channel_count] = ACTIONS(2839),
    [sym_set_virtual_port_count] = ACTIONS(2839),
    [sym__wc_encode_int] = ACTIONS(2839),
    [sym__wc_decode_int] = ACTIONS(2839),
    [sym__wc_int] = ACTIONS(2839),
    [sym__wc_to_ch_int] = ACTIONS(2839),
    [sym__ch_to_wc_int] = ACTIONS(2839),
    [sym__wc_find_string_int] = ACTIONS(2839),
    [sym__wc_left_string_int] = ACTIONS(2839),
    [sym__wc_lower_string_int] = ACTIONS(2839),
    [sym__wc_mid_string_int] = ACTIONS(2839),
    [sym__wc_remove_string_int] = ACTIONS(2839),
    [sym__wc_right_string_int] = ACTIONS(2839),
    [sym__wc_upper_string_int] = ACTIONS(2839),
    [sym__wc_compare_string_int] = ACTIONS(2839),
    [sym__wc_get_buffer_char_int] = ACTIONS(2839),
    [sym__wc_get_buffer_string_int] = ACTIONS(2839),
    [sym__wc_concat_string_int] = ACTIONS(2839),
    [sym__wc_file_open_int] = ACTIONS(2839),
    [sym__wc_file_close_int] = ACTIONS(2839),
    [sym__wc_file_read_int] = ACTIONS(2839),
    [sym__wc_file_read_line_int] = ACTIONS(2839),
    [sym__wc_file_write_int] = ACTIONS(2839),
    [sym__wc_file_write_line_int] = ACTIONS(2839),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2841),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2841),
    [anon_sym_LBRACK] = ACTIONS(2841),
    [anon_sym_LBRACE] = ACTIONS(2841),
    [anon_sym_RBRACE] = ACTIONS(2841),
    [anon_sym_LPAREN] = ACTIONS(2839),
    [anon_sym_BANG] = ACTIONS(2841),
    [anon_sym_TILDE] = ACTIONS(2841),
    [anon_sym_DASH] = ACTIONS(2839),
    [anon_sym_PLUS] = ACTIONS(2839),
    [anon_sym_DASH_DASH] = ACTIONS(2841),
    [anon_sym_PLUS_PLUS] = ACTIONS(2841),
    [anon_sym_DQUOTE] = ACTIONS(2841),
    [anon_sym_SQUOTE] = ACTIONS(2841),
    [sym_number_literal] = ACTIONS(2841),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(955)] = {
    [sym_identifier] = ACTIONS(2843),
    [sym_not] = ACTIONS(2843),
    [sym_bnot] = ACTIONS(2843),
    [sym___file__] = ACTIONS(2843),
    [sym___date__] = ACTIONS(2843),
    [sym___ldate__] = ACTIONS(2843),
    [sym___time__] = ACTIONS(2843),
    [sym___line__] = ACTIONS(2843),
    [sym___name__] = ACTIONS(2843),
    [sym___netlinx__] = ACTIONS(2843),
    [sym_netlinx_axi_version] = ACTIONS(2843),
    [sym_url_flg_tcp] = ACTIONS(2843),
    [sym_url_flg_accinfopresent] = ACTIONS(2843),
    [sym_url_flg_temp] = ACTIONS(2843),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2843),
    [sym_url_flg_stat_mask] = ACTIONS(2843),
    [sym_url_flg_stat_lookup] = ACTIONS(2843),
    [sym_url_flg_stat_connecting] = ACTIONS(2843),
    [sym_url_flg_stat_waiting] = ACTIONS(2843),
    [sym_url_flg_stat_connected] = ACTIONS(2843),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2843),
    [sym_first_virtual_device] = ACTIONS(2843),
    [sym_first_local_port] = ACTIONS(2843),
    [sym_dynamic_virtual_device] = ACTIONS(2843),
    [sym_timeline_once] = ACTIONS(2843),
    [sym_timeline_repeat] = ACTIONS(2843),
    [sym_timeline_absolute] = ACTIONS(2843),
    [sym_timeline_relative] = ACTIONS(2843),
    [sym_do_push_timed_infinite] = ACTIONS(2843),
    [sym_source_type_no_address] = ACTIONS(2843),
    [sym_source_type_neuron_id] = ACTIONS(2843),
    [sym_source_type_ip_address] = ACTIONS(2843),
    [sym_source_type_axlink] = ACTIONS(2843),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2843),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2843),
    [sym_source_type_ip_socket_address] = ACTIONS(2843),
    [sym_source_type_rs232] = ACTIONS(2843),
    [sym_source_type_internal] = ACTIONS(2843),
    [sym_source_type_mac_address] = ACTIONS(2843),
    [sym_source_type_ipv4_port] = ACTIONS(2843),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2843),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2843),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2843),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2843),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2843),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2843),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2843),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2843),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2843),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2843),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2843),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2843),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2843),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2843),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2843),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2843),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2843),
    [sym_true] = ACTIONS(2843),
    [sym_false] = ACTIONS(2843),
    [sym_file_read_only] = ACTIONS(2843),
    [sym_file_rw_new] = ACTIONS(2843),
    [sym_file_rw_append] = ACTIONS(2843),
    [sym_ip_tcp] = ACTIONS(2843),
    [sym_ip_udp] = ACTIONS(2843),
    [sym_ip_udp_2way] = ACTIONS(2843),
    [sym_xml_encode_types] = ACTIONS(2843),
    [sym_xml_encode_char_as_list] = ACTIONS(2843),
    [sym_xml_encode_le] = ACTIONS(2843),
    [sym_xml_decode_types] = ACTIONS(2843),
    [sym_xml_decode_no_preserve] = ACTIONS(2843),
    [sym_duet_dev_type_display_device] = ACTIONS(2843),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2843),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2843),
    [sym_duet_dev_type_io_device] = ACTIONS(2843),
    [sym_duet_dev_type_relay_device] = ACTIONS(2843),
    [sym_duet_dev_type_ups] = ACTIONS(2843),
    [sym_duet_dev_type_amplifier] = ACTIONS(2843),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2843),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2843),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2843),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2843),
    [sym_duet_dev_type_camera] = ACTIONS(2843),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2843),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2843),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2843),
    [sym_duet_dev_type_dss] = ACTIONS(2843),
    [sym_duet_dev_type_dvr] = ACTIONS(2843),
    [sym_duet_dev_type_disc_device] = ACTIONS(2843),
    [sym_duet_dev_type_document_camera] = ACTIONS(2843),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2843),
    [sym_duet_dev_type_hvac] = ACTIONS(2843),
    [sym_duet_dev_type_keypad] = ACTIONS(2843),
    [sym_duet_dev_type_light] = ACTIONS(2843),
    [sym_duet_dev_type_monitor] = ACTIONS(2843),
    [sym_duet_dev_type_motor] = ACTIONS(2843),
    [sym_duet_dev_type_multi_window] = ACTIONS(2843),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2843),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2843),
    [sym_duet_dev_type_receiver] = ACTIONS(2843),
    [sym_duet_dev_type_security_system] = ACTIONS(2843),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2843),
    [sym_duet_dev_type_settop_box] = ACTIONS(2843),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2843),
    [sym_duet_dev_type_switcher] = ACTIONS(2843),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2843),
    [sym_duet_dev_type_tv] = ACTIONS(2843),
    [sym_duet_dev_type_utility] = ACTIONS(2843),
    [sym_duet_dev_type_vcr] = ACTIONS(2843),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2843),
    [sym_duet_dev_type_video_processor] = ACTIONS(2843),
    [sym_duet_dev_type_video_projector] = ACTIONS(2843),
    [sym_duet_dev_type_video_wall] = ACTIONS(2843),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2843),
    [sym_duet_dev_type_weather] = ACTIONS(2843),
    [sym_duet_dev_not_polled] = ACTIONS(2843),
    [sym_duet_dev_polled] = ACTIONS(2843),
    [sym_clkmgr_mode_network] = ACTIONS(2843),
    [sym_clkmgr_mode_standalone] = ACTIONS(2843),
    [sym_normal_standby] = ACTIONS(2843),
    [sym_normal_wake] = ACTIONS(2843),
    [sym_ip_multicast_ttl_option] = ACTIONS(2843),
    [sym_ip_ttl_subnet] = ACTIONS(2843),
    [sym_ip_ttl_site] = ACTIONS(2843),
    [sym_ip_ttl_region] = ACTIONS(2843),
    [sym_ip_ttl_continent] = ACTIONS(2843),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2843),
    [sym_ip_nodelay_on] = ACTIONS(2843),
    [sym_ip_nodelay_off] = ACTIONS(2843),
    [sym_amx_error] = ACTIONS(2843),
    [sym_amx_warning] = ACTIONS(2843),
    [sym_amx_info] = ACTIONS(2843),
    [sym_amx_debug] = ACTIONS(2843),
    [sym_netlinx_login_success] = ACTIONS(2843),
    [sym_netlinx_login_fail] = ACTIONS(2843),
    [sym_netlinx_logout] = ACTIONS(2843),
    [sym_null_str] = ACTIONS(2843),
    [sym_smtp_address] = ACTIONS(2843),
    [sym_smtp_port_number] = ACTIONS(2843),
    [sym_smtp_username] = ACTIONS(2843),
    [sym_smtp_password] = ACTIONS(2843),
    [sym_smtp_require_tls] = ACTIONS(2843),
    [sym_smtp_from] = ACTIONS(2843),
    [sym_smtp_tls_true] = ACTIONS(2843),
    [sym_smtp_tls_false] = ACTIONS(2843),
    [sym_listview_on_row_select_event] = ACTIONS(2843),
    [sym_data_structure_unknown] = ACTIONS(2843),
    [sym_data_structure_datafeed] = ACTIONS(2843),
    [sym_data_structure_datarecordset] = ACTIONS(2843),
    [sym_data_structure_datarecord] = ACTIONS(2843),
    [sym_data_structure_datafield] = ACTIONS(2843),
    [sym_data_max_content_field_count] = ACTIONS(2843),
    [sym_data_max_metadata_field_count] = ACTIONS(2843),
    [sym_data_max_id_length] = ACTIONS(2843),
    [sym_data_max_name_length] = ACTIONS(2843),
    [sym_data_max_description_length] = ACTIONS(2843),
    [sym_data_max_type_length] = ACTIONS(2843),
    [sym_data_max_format_length] = ACTIONS(2843),
    [sym_data_max_source_length] = ACTIONS(2843),
    [sym_data_max_value_length] = ACTIONS(2843),
    [sym_data_max_label_length] = ACTIONS(2843),
    [sym_data_max_url_length] = ACTIONS(2843),
    [sym_max_last_login_info_length] = ACTIONS(2843),
    [sym_data_type_unknown] = ACTIONS(2843),
    [sym_data_type_string] = ACTIONS(2843),
    [sym_data_type_datetime] = ACTIONS(2843),
    [sym_data_type_date] = ACTIONS(2843),
    [sym_data_type_time] = ACTIONS(2843),
    [sym_data_type_image] = ACTIONS(2843),
    [sym_data_format_url] = ACTIONS(2843),
    [sym_data_format_phone] = ACTIONS(2843),
    [sym_data_format_email] = ACTIONS(2843),
    [sym_data_format_iso8601] = ACTIONS(2843),
    [sym_auditlog_priv] = ACTIONS(2843),
    [sym_configuration_priv] = ACTIONS(2843),
    [sym_deviceconfig_priv] = ACTIONS(2843),
    [sym_ftp_priv] = ACTIONS(2843),
    [sym_http_priv] = ACTIONS(2843),
    [sym_networkconfig_priv] = ACTIONS(2843),
    [sym_programport_priv] = ACTIONS(2843),
    [sym_remoteui_priv] = ACTIONS(2843),
    [sym_securitycontrol_priv] = ACTIONS(2843),
    [sym_softwaremanagement_priv] = ACTIONS(2843),
    [sym_terminal_priv] = ACTIONS(2843),
    [sym_tpadmin_priv] = ACTIONS(2843),
    [sym_user1_priv] = ACTIONS(2843),
    [sym_user2_priv] = ACTIONS(2843),
    [sym_user3_priv] = ACTIONS(2843),
    [sym_user4_priv] = ACTIONS(2843),
    [sym_usermanagement_priv] = ACTIONS(2843),
    [sym_valid_account] = ACTIONS(2843),
    [sym_err_name_invalid] = ACTIONS(2843),
    [sym_err_password_invalid] = ACTIONS(2843),
    [sym_err_invalid_account] = ACTIONS(2843),
    [sym_err_no_authorization] = ACTIONS(2843),
    [sym_err_invalid_parameter] = ACTIONS(2843),
    [sym_err_user_locked_out] = ACTIONS(2843),
    [sym_err_user_account_expired] = ACTIONS(2843),
    [sym_err_general_error] = ACTIONS(2843),
    [sym_err_authentication_error] = ACTIONS(2843),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2843),
    [sym_tls_validate_certificate] = ACTIONS(2843),
    [sym____reserved___] = ACTIONS(2843),
    [sym_date] = ACTIONS(2843),
    [sym_day] = ACTIONS(2843),
    [sym_time] = ACTIONS(2843),
    [sym_ldate] = ACTIONS(2843),
    [sym_push_device] = ACTIONS(2843),
    [sym_push_channel] = ACTIONS(2843),
    [sym_push_devchan] = ACTIONS(2843),
    [sym_release_device] = ACTIONS(2843),
    [sym_release_channel] = ACTIONS(2843),
    [sym_release_devchan] = ACTIONS(2843),
    [sym_master_slot] = ACTIONS(2843),
    [sym_get_pulse_time] = ACTIONS(2843),
    [sym_get_timer] = ACTIONS(2843),
    [sym_master_sn] = ACTIONS(2843),
    [sym_system_number] = ACTIONS(2843),
    [sym_button] = ACTIONS(2843),
    [sym_dv_channel] = ACTIONS(2843),
    [sym_data] = ACTIONS(2843),
    [sym_level] = ACTIONS(2843),
    [sym_timeline] = ACTIONS(2843),
    [sym_channel] = ACTIONS(2843),
    [sym_custom] = ACTIONS(2843),
    [sym_atoi] = ACTIONS(2843),
    [sym_atol] = ACTIONS(2843),
    [sym_atof] = ACTIONS(2843),
    [sym_itoa] = ACTIONS(2843),
    [sym_itohex] = ACTIONS(2843),
    [sym_hextoi] = ACTIONS(2843),
    [sym_ftoa] = ACTIONS(2843),
    [sym_find_string] = ACTIONS(2843),
    [sym_left_string] = ACTIONS(2843),
    [sym_right_string] = ACTIONS(2843),
    [sym_mid_string] = ACTIONS(2843),
    [sym_remove_string] = ACTIONS(2843),
    [sym_get_buffer_string] = ACTIONS(2843),
    [sym_get_buffer_char] = ACTIONS(2843),
    [sym_get_multi_buffer_string] = ACTIONS(2843),
    [sym_length_string] = ACTIONS(2843),
    [sym_max_length_string] = ACTIONS(2843),
    [sym_set_length_string] = ACTIONS(2843),
    [sym_compare_string] = ACTIONS(2843),
    [sym_upper_string] = ACTIONS(2843),
    [sym_lower_string] = ACTIONS(2843),
    [sym_get_last] = ACTIONS(2843),
    [sym_date_to_day] = ACTIONS(2843),
    [sym_date_to_month] = ACTIONS(2843),
    [sym_date_to_year] = ACTIONS(2843),
    [sym_time_to_hour] = ACTIONS(2843),
    [sym_time_to_minute] = ACTIONS(2843),
    [sym_time_to_second] = ACTIONS(2843),
    [sym_day_of_week] = ACTIONS(2843),
    [sym_set_pulse_time] = ACTIONS(2843),
    [sym_set_timer] = ACTIONS(2843),
    [sym_file_open] = ACTIONS(2843),
    [sym_file_close] = ACTIONS(2843),
    [sym_file_read] = ACTIONS(2843),
    [sym_file_read_line] = ACTIONS(2843),
    [sym_file_write] = ACTIONS(2843),
    [sym_file_write_line] = ACTIONS(2843),
    [sym_file_copy] = ACTIONS(2843),
    [sym_file_delete] = ACTIONS(2843),
    [sym_file_rename] = ACTIONS(2843),
    [sym_file_dir] = ACTIONS(2843),
    [sym_file_getdir] = ACTIONS(2843),
    [sym_file_setdir] = ACTIONS(2843),
    [sym_file_create_dir] = ACTIONS(2843),
    [sym_file_remove_dir] = ACTIONS(2843),
    [sym_file_seek] = ACTIONS(2843),
    [sym_length_array] = ACTIONS(2843),
    [sym_max_length_array] = ACTIONS(2843),
    [sym_set_length_array] = ACTIONS(2843),
    [sym_ip_client_open] = ACTIONS(2843),
    [sym_ip_client_close] = ACTIONS(2843),
    [sym_ip_server_open] = ACTIONS(2843),
    [sym_ip_server_close] = ACTIONS(2843),
    [sym_ip_bound_client_open] = ACTIONS(2843),
    [sym_ip_mc_server_open] = ACTIONS(2843),
    [sym_ip_set_option] = ACTIONS(2843),
    [sym_ssh_client_open] = ACTIONS(2843),
    [sym_ssh_client_close] = ACTIONS(2843),
    [sym_tls_client_open] = ACTIONS(2843),
    [sym_tls_client_close] = ACTIONS(2843),
    [sym_do_push] = ACTIONS(2843),
    [sym_do_push_timed] = ACTIONS(2843),
    [sym_do_release] = ACTIONS(2843),
    [sym_device_id] = ACTIONS(2843),
    [sym_device_info] = ACTIONS(2843),
    [sym_device_id_string] = ACTIONS(2843),
    [sym_device_standby] = ACTIONS(2843),
    [sym_device_wake] = ACTIONS(2843),
    [sym_timeline_create] = ACTIONS(2843),
    [sym_timeline_kill] = ACTIONS(2843),
    [sym_timeline_reload] = ACTIONS(2843),
    [sym_timeline_pause] = ACTIONS(2843),
    [sym_timeline_restart] = ACTIONS(2843),
    [sym_timeline_set] = ACTIONS(2843),
    [sym_timeline_get] = ACTIONS(2843),
    [sym_timeline_active] = ACTIONS(2843),
    [sym_timeline_dynamic_id] = ACTIONS(2843),
    [sym_variable_to_string] = ACTIONS(2843),
    [sym_string_to_variable] = ACTIONS(2843),
    [sym_length_variable_to_string] = ACTIONS(2843),
    [sym_type_cast] = ACTIONS(2843),
    [sym_variable_to_xml] = ACTIONS(2843),
    [sym_xml_to_variable] = ACTIONS(2843),
    [sym_length_variable_to_xml] = ACTIONS(2843),
    [sym_redirect_string] = ACTIONS(2843),
    [sym_random_number] = ACTIONS(2843),
    [sym_get_unique_id] = ACTIONS(2843),
    [sym_get_system_number] = ACTIONS(2843),
    [sym_set_system_number] = ACTIONS(2843),
    [sym_get_serial_number] = ACTIONS(2843),
    [sym_get_master_build] = ACTIONS(2843),
    [sym_set_validation_code] = ACTIONS(2843),
    [sym_reboot] = ACTIONS(2843),
    [sym_load_duet_module] = ACTIONS(2843),
    [sym_unload_duet_module] = ACTIONS(2843),
    [sym_duet_mem_size_set] = ACTIONS(2843),
    [sym_duet_mem_size_get] = ACTIONS(2843),
    [sym_dynamic_application_device] = ACTIONS(2843),
    [sym_dynamic_polled_port] = ACTIONS(2843),
    [sym_static_port_binding] = ACTIONS(2843),
    [sym_static_ip_binding] = ACTIONS(2843),
    [sym_get_dns_list] = ACTIONS(2843),
    [sym_set_dns_list] = ACTIONS(2843),
    [sym_get_ip_address] = ACTIONS(2843),
    [sym_set_ip_address] = ACTIONS(2843),
    [sym_get_url_list] = ACTIONS(2843),
    [sym_add_url_entry] = ACTIONS(2843),
    [sym_delete_url_entry] = ACTIONS(2843),
    [sym_abs_value] = ACTIONS(2843),
    [sym_max_value] = ACTIONS(2843),
    [sym_min_value] = ACTIONS(2843),
    [sym_exp_value] = ACTIONS(2843),
    [sym_log_value] = ACTIONS(2843),
    [sym_log10_value] = ACTIONS(2843),
    [sym_power_value] = ACTIONS(2843),
    [sym_sqrt_value] = ACTIONS(2843),
    [sym_raw_be] = ACTIONS(2843),
    [sym_raw_le] = ACTIONS(2843),
    [sym_internal_queue_size_set] = ACTIONS(2843),
    [sym_internal_queue_size_get] = ACTIONS(2843),
    [sym_internal_threshold_set] = ACTIONS(2843),
    [sym_internal_threshold_get] = ACTIONS(2843),
    [sym_rebuilt_event] = ACTIONS(2843),
    [sym_do_custom_event] = ACTIONS(2843),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2843),
    [sym_clkmgr_set_clk_source] = ACTIONS(2843),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2843),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2843),
    [sym_clkmgr_get_timezone] = ACTIONS(2843),
    [sym_clkmgr_set_timezone] = ACTIONS(2843),
    [sym_clkmgr_get_resync_period] = ACTIONS(2843),
    [sym_clkmgr_set_resync_period] = ACTIONS(2843),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2843),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2843),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2843),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2843),
    [sym_clkmgr_get_timeservers] = ACTIONS(2843),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2843),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2843),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2843),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2843),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2843),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2843),
    [sym_set_log_level] = ACTIONS(2843),
    [sym_get_log_level] = ACTIONS(2843),
    [sym_amx_log] = ACTIONS(2843),
    [sym_smtp_server_config_set] = ACTIONS(2843),
    [sym_smtp_server_config_get] = ACTIONS(2843),
    [sym_smtp_send] = ACTIONS(2843),
    [sym_validate_netlinx_account] = ACTIONS(2843),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2843),
    [sym_authenticate_certificate] = ACTIONS(2843),
    [sym_audit_netlinx_session_event] = ACTIONS(2843),
    [sym_audit_netlinx_generic_event] = ACTIONS(2843),
    [sym_get_available_flash_disk_space] = ACTIONS(2843),
    [sym_get_max_flash_disk_space] = ACTIONS(2843),
    [sym_data_create_feed] = ACTIONS(2843),
    [sym_data_delete_feed] = ACTIONS(2843),
    [sym_data_publish_feed] = ACTIONS(2843),
    [sym_data_get_published_feed] = ACTIONS(2843),
    [sym_data_add_record] = ACTIONS(2843),
    [sym_data_get_event_record] = ACTIONS(2843),
    [sym__wc_data_create_feed] = ACTIONS(2843),
    [sym__wc_data_add_record] = ACTIONS(2843),
    [sym__wc_data_get_event_record] = ACTIONS(2843),
    [sym_format] = ACTIONS(2843),
    [sym_astro_clock] = ACTIONS(2843),
    [sym_set_outdoor_temperature] = ACTIONS(2843),
    [sym_set_virtual_level_count] = ACTIONS(2843),
    [sym_set_virtual_channel_count] = ACTIONS(2843),
    [sym_set_virtual_port_count] = ACTIONS(2843),
    [sym__wc_encode_int] = ACTIONS(2843),
    [sym__wc_decode_int] = ACTIONS(2843),
    [sym__wc_int] = ACTIONS(2843),
    [sym__wc_to_ch_int] = ACTIONS(2843),
    [sym__ch_to_wc_int] = ACTIONS(2843),
    [sym__wc_find_string_int] = ACTIONS(2843),
    [sym__wc_left_string_int] = ACTIONS(2843),
    [sym__wc_lower_string_int] = ACTIONS(2843),
    [sym__wc_mid_string_int] = ACTIONS(2843),
    [sym__wc_remove_string_int] = ACTIONS(2843),
    [sym__wc_right_string_int] = ACTIONS(2843),
    [sym__wc_upper_string_int] = ACTIONS(2843),
    [sym__wc_compare_string_int] = ACTIONS(2843),
    [sym__wc_get_buffer_char_int] = ACTIONS(2843),
    [sym__wc_get_buffer_string_int] = ACTIONS(2843),
    [sym__wc_concat_string_int] = ACTIONS(2843),
    [sym__wc_file_open_int] = ACTIONS(2843),
    [sym__wc_file_close_int] = ACTIONS(2843),
    [sym__wc_file_read_int] = ACTIONS(2843),
    [sym__wc_file_read_line_int] = ACTIONS(2843),
    [sym__wc_file_write_int] = ACTIONS(2843),
    [sym__wc_file_write_line_int] = ACTIONS(2843),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2845),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2845),
    [anon_sym_LBRACK] = ACTIONS(2845),
    [anon_sym_LBRACE] = ACTIONS(2845),
    [anon_sym_RBRACE] = ACTIONS(2845),
    [anon_sym_LPAREN] = ACTIONS(2843),
    [anon_sym_BANG] = ACTIONS(2845),
    [anon_sym_TILDE] = ACTIONS(2845),
    [anon_sym_DASH] = ACTIONS(2843),
    [anon_sym_PLUS] = ACTIONS(2843),
    [anon_sym_DASH_DASH] = ACTIONS(2845),
    [anon_sym_PLUS_PLUS] = ACTIONS(2845),
    [anon_sym_DQUOTE] = ACTIONS(2845),
    [anon_sym_SQUOTE] = ACTIONS(2845),
    [sym_number_literal] = ACTIONS(2845),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(956)] = {
    [sym_identifier] = ACTIONS(2847),
    [sym_not] = ACTIONS(2847),
    [sym_bnot] = ACTIONS(2847),
    [sym___file__] = ACTIONS(2847),
    [sym___date__] = ACTIONS(2847),
    [sym___ldate__] = ACTIONS(2847),
    [sym___time__] = ACTIONS(2847),
    [sym___line__] = ACTIONS(2847),
    [sym___name__] = ACTIONS(2847),
    [sym___netlinx__] = ACTIONS(2847),
    [sym_netlinx_axi_version] = ACTIONS(2847),
    [sym_url_flg_tcp] = ACTIONS(2847),
    [sym_url_flg_accinfopresent] = ACTIONS(2847),
    [sym_url_flg_temp] = ACTIONS(2847),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2847),
    [sym_url_flg_stat_mask] = ACTIONS(2847),
    [sym_url_flg_stat_lookup] = ACTIONS(2847),
    [sym_url_flg_stat_connecting] = ACTIONS(2847),
    [sym_url_flg_stat_waiting] = ACTIONS(2847),
    [sym_url_flg_stat_connected] = ACTIONS(2847),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2847),
    [sym_first_virtual_device] = ACTIONS(2847),
    [sym_first_local_port] = ACTIONS(2847),
    [sym_dynamic_virtual_device] = ACTIONS(2847),
    [sym_timeline_once] = ACTIONS(2847),
    [sym_timeline_repeat] = ACTIONS(2847),
    [sym_timeline_absolute] = ACTIONS(2847),
    [sym_timeline_relative] = ACTIONS(2847),
    [sym_do_push_timed_infinite] = ACTIONS(2847),
    [sym_source_type_no_address] = ACTIONS(2847),
    [sym_source_type_neuron_id] = ACTIONS(2847),
    [sym_source_type_ip_address] = ACTIONS(2847),
    [sym_source_type_axlink] = ACTIONS(2847),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2847),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2847),
    [sym_source_type_ip_socket_address] = ACTIONS(2847),
    [sym_source_type_rs232] = ACTIONS(2847),
    [sym_source_type_internal] = ACTIONS(2847),
    [sym_source_type_mac_address] = ACTIONS(2847),
    [sym_source_type_ipv4_port] = ACTIONS(2847),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2847),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2847),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2847),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2847),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2847),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2847),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2847),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2847),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2847),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2847),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2847),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2847),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2847),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2847),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2847),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2847),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2847),
    [sym_true] = ACTIONS(2847),
    [sym_false] = ACTIONS(2847),
    [sym_file_read_only] = ACTIONS(2847),
    [sym_file_rw_new] = ACTIONS(2847),
    [sym_file_rw_append] = ACTIONS(2847),
    [sym_ip_tcp] = ACTIONS(2847),
    [sym_ip_udp] = ACTIONS(2847),
    [sym_ip_udp_2way] = ACTIONS(2847),
    [sym_xml_encode_types] = ACTIONS(2847),
    [sym_xml_encode_char_as_list] = ACTIONS(2847),
    [sym_xml_encode_le] = ACTIONS(2847),
    [sym_xml_decode_types] = ACTIONS(2847),
    [sym_xml_decode_no_preserve] = ACTIONS(2847),
    [sym_duet_dev_type_display_device] = ACTIONS(2847),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2847),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2847),
    [sym_duet_dev_type_io_device] = ACTIONS(2847),
    [sym_duet_dev_type_relay_device] = ACTIONS(2847),
    [sym_duet_dev_type_ups] = ACTIONS(2847),
    [sym_duet_dev_type_amplifier] = ACTIONS(2847),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2847),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2847),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2847),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2847),
    [sym_duet_dev_type_camera] = ACTIONS(2847),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2847),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2847),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2847),
    [sym_duet_dev_type_dss] = ACTIONS(2847),
    [sym_duet_dev_type_dvr] = ACTIONS(2847),
    [sym_duet_dev_type_disc_device] = ACTIONS(2847),
    [sym_duet_dev_type_document_camera] = ACTIONS(2847),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2847),
    [sym_duet_dev_type_hvac] = ACTIONS(2847),
    [sym_duet_dev_type_keypad] = ACTIONS(2847),
    [sym_duet_dev_type_light] = ACTIONS(2847),
    [sym_duet_dev_type_monitor] = ACTIONS(2847),
    [sym_duet_dev_type_motor] = ACTIONS(2847),
    [sym_duet_dev_type_multi_window] = ACTIONS(2847),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2847),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2847),
    [sym_duet_dev_type_receiver] = ACTIONS(2847),
    [sym_duet_dev_type_security_system] = ACTIONS(2847),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2847),
    [sym_duet_dev_type_settop_box] = ACTIONS(2847),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2847),
    [sym_duet_dev_type_switcher] = ACTIONS(2847),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2847),
    [sym_duet_dev_type_tv] = ACTIONS(2847),
    [sym_duet_dev_type_utility] = ACTIONS(2847),
    [sym_duet_dev_type_vcr] = ACTIONS(2847),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2847),
    [sym_duet_dev_type_video_processor] = ACTIONS(2847),
    [sym_duet_dev_type_video_projector] = ACTIONS(2847),
    [sym_duet_dev_type_video_wall] = ACTIONS(2847),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2847),
    [sym_duet_dev_type_weather] = ACTIONS(2847),
    [sym_duet_dev_not_polled] = ACTIONS(2847),
    [sym_duet_dev_polled] = ACTIONS(2847),
    [sym_clkmgr_mode_network] = ACTIONS(2847),
    [sym_clkmgr_mode_standalone] = ACTIONS(2847),
    [sym_normal_standby] = ACTIONS(2847),
    [sym_normal_wake] = ACTIONS(2847),
    [sym_ip_multicast_ttl_option] = ACTIONS(2847),
    [sym_ip_ttl_subnet] = ACTIONS(2847),
    [sym_ip_ttl_site] = ACTIONS(2847),
    [sym_ip_ttl_region] = ACTIONS(2847),
    [sym_ip_ttl_continent] = ACTIONS(2847),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2847),
    [sym_ip_nodelay_on] = ACTIONS(2847),
    [sym_ip_nodelay_off] = ACTIONS(2847),
    [sym_amx_error] = ACTIONS(2847),
    [sym_amx_warning] = ACTIONS(2847),
    [sym_amx_info] = ACTIONS(2847),
    [sym_amx_debug] = ACTIONS(2847),
    [sym_netlinx_login_success] = ACTIONS(2847),
    [sym_netlinx_login_fail] = ACTIONS(2847),
    [sym_netlinx_logout] = ACTIONS(2847),
    [sym_null_str] = ACTIONS(2847),
    [sym_smtp_address] = ACTIONS(2847),
    [sym_smtp_port_number] = ACTIONS(2847),
    [sym_smtp_username] = ACTIONS(2847),
    [sym_smtp_password] = ACTIONS(2847),
    [sym_smtp_require_tls] = ACTIONS(2847),
    [sym_smtp_from] = ACTIONS(2847),
    [sym_smtp_tls_true] = ACTIONS(2847),
    [sym_smtp_tls_false] = ACTIONS(2847),
    [sym_listview_on_row_select_event] = ACTIONS(2847),
    [sym_data_structure_unknown] = ACTIONS(2847),
    [sym_data_structure_datafeed] = ACTIONS(2847),
    [sym_data_structure_datarecordset] = ACTIONS(2847),
    [sym_data_structure_datarecord] = ACTIONS(2847),
    [sym_data_structure_datafield] = ACTIONS(2847),
    [sym_data_max_content_field_count] = ACTIONS(2847),
    [sym_data_max_metadata_field_count] = ACTIONS(2847),
    [sym_data_max_id_length] = ACTIONS(2847),
    [sym_data_max_name_length] = ACTIONS(2847),
    [sym_data_max_description_length] = ACTIONS(2847),
    [sym_data_max_type_length] = ACTIONS(2847),
    [sym_data_max_format_length] = ACTIONS(2847),
    [sym_data_max_source_length] = ACTIONS(2847),
    [sym_data_max_value_length] = ACTIONS(2847),
    [sym_data_max_label_length] = ACTIONS(2847),
    [sym_data_max_url_length] = ACTIONS(2847),
    [sym_max_last_login_info_length] = ACTIONS(2847),
    [sym_data_type_unknown] = ACTIONS(2847),
    [sym_data_type_string] = ACTIONS(2847),
    [sym_data_type_datetime] = ACTIONS(2847),
    [sym_data_type_date] = ACTIONS(2847),
    [sym_data_type_time] = ACTIONS(2847),
    [sym_data_type_image] = ACTIONS(2847),
    [sym_data_format_url] = ACTIONS(2847),
    [sym_data_format_phone] = ACTIONS(2847),
    [sym_data_format_email] = ACTIONS(2847),
    [sym_data_format_iso8601] = ACTIONS(2847),
    [sym_auditlog_priv] = ACTIONS(2847),
    [sym_configuration_priv] = ACTIONS(2847),
    [sym_deviceconfig_priv] = ACTIONS(2847),
    [sym_ftp_priv] = ACTIONS(2847),
    [sym_http_priv] = ACTIONS(2847),
    [sym_networkconfig_priv] = ACTIONS(2847),
    [sym_programport_priv] = ACTIONS(2847),
    [sym_remoteui_priv] = ACTIONS(2847),
    [sym_securitycontrol_priv] = ACTIONS(2847),
    [sym_softwaremanagement_priv] = ACTIONS(2847),
    [sym_terminal_priv] = ACTIONS(2847),
    [sym_tpadmin_priv] = ACTIONS(2847),
    [sym_user1_priv] = ACTIONS(2847),
    [sym_user2_priv] = ACTIONS(2847),
    [sym_user3_priv] = ACTIONS(2847),
    [sym_user4_priv] = ACTIONS(2847),
    [sym_usermanagement_priv] = ACTIONS(2847),
    [sym_valid_account] = ACTIONS(2847),
    [sym_err_name_invalid] = ACTIONS(2847),
    [sym_err_password_invalid] = ACTIONS(2847),
    [sym_err_invalid_account] = ACTIONS(2847),
    [sym_err_no_authorization] = ACTIONS(2847),
    [sym_err_invalid_parameter] = ACTIONS(2847),
    [sym_err_user_locked_out] = ACTIONS(2847),
    [sym_err_user_account_expired] = ACTIONS(2847),
    [sym_err_general_error] = ACTIONS(2847),
    [sym_err_authentication_error] = ACTIONS(2847),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2847),
    [sym_tls_validate_certificate] = ACTIONS(2847),
    [sym____reserved___] = ACTIONS(2847),
    [sym_date] = ACTIONS(2847),
    [sym_day] = ACTIONS(2847),
    [sym_time] = ACTIONS(2847),
    [sym_ldate] = ACTIONS(2847),
    [sym_push_device] = ACTIONS(2847),
    [sym_push_channel] = ACTIONS(2847),
    [sym_push_devchan] = ACTIONS(2847),
    [sym_release_device] = ACTIONS(2847),
    [sym_release_channel] = ACTIONS(2847),
    [sym_release_devchan] = ACTIONS(2847),
    [sym_master_slot] = ACTIONS(2847),
    [sym_get_pulse_time] = ACTIONS(2847),
    [sym_get_timer] = ACTIONS(2847),
    [sym_master_sn] = ACTIONS(2847),
    [sym_system_number] = ACTIONS(2847),
    [sym_button] = ACTIONS(2847),
    [sym_dv_channel] = ACTIONS(2847),
    [sym_data] = ACTIONS(2847),
    [sym_level] = ACTIONS(2847),
    [sym_timeline] = ACTIONS(2847),
    [sym_channel] = ACTIONS(2847),
    [sym_custom] = ACTIONS(2847),
    [sym_atoi] = ACTIONS(2847),
    [sym_atol] = ACTIONS(2847),
    [sym_atof] = ACTIONS(2847),
    [sym_itoa] = ACTIONS(2847),
    [sym_itohex] = ACTIONS(2847),
    [sym_hextoi] = ACTIONS(2847),
    [sym_ftoa] = ACTIONS(2847),
    [sym_find_string] = ACTIONS(2847),
    [sym_left_string] = ACTIONS(2847),
    [sym_right_string] = ACTIONS(2847),
    [sym_mid_string] = ACTIONS(2847),
    [sym_remove_string] = ACTIONS(2847),
    [sym_get_buffer_string] = ACTIONS(2847),
    [sym_get_buffer_char] = ACTIONS(2847),
    [sym_get_multi_buffer_string] = ACTIONS(2847),
    [sym_length_string] = ACTIONS(2847),
    [sym_max_length_string] = ACTIONS(2847),
    [sym_set_length_string] = ACTIONS(2847),
    [sym_compare_string] = ACTIONS(2847),
    [sym_upper_string] = ACTIONS(2847),
    [sym_lower_string] = ACTIONS(2847),
    [sym_get_last] = ACTIONS(2847),
    [sym_date_to_day] = ACTIONS(2847),
    [sym_date_to_month] = ACTIONS(2847),
    [sym_date_to_year] = ACTIONS(2847),
    [sym_time_to_hour] = ACTIONS(2847),
    [sym_time_to_minute] = ACTIONS(2847),
    [sym_time_to_second] = ACTIONS(2847),
    [sym_day_of_week] = ACTIONS(2847),
    [sym_set_pulse_time] = ACTIONS(2847),
    [sym_set_timer] = ACTIONS(2847),
    [sym_file_open] = ACTIONS(2847),
    [sym_file_close] = ACTIONS(2847),
    [sym_file_read] = ACTIONS(2847),
    [sym_file_read_line] = ACTIONS(2847),
    [sym_file_write] = ACTIONS(2847),
    [sym_file_write_line] = ACTIONS(2847),
    [sym_file_copy] = ACTIONS(2847),
    [sym_file_delete] = ACTIONS(2847),
    [sym_file_rename] = ACTIONS(2847),
    [sym_file_dir] = ACTIONS(2847),
    [sym_file_getdir] = ACTIONS(2847),
    [sym_file_setdir] = ACTIONS(2847),
    [sym_file_create_dir] = ACTIONS(2847),
    [sym_file_remove_dir] = ACTIONS(2847),
    [sym_file_seek] = ACTIONS(2847),
    [sym_length_array] = ACTIONS(2847),
    [sym_max_length_array] = ACTIONS(2847),
    [sym_set_length_array] = ACTIONS(2847),
    [sym_ip_client_open] = ACTIONS(2847),
    [sym_ip_client_close] = ACTIONS(2847),
    [sym_ip_server_open] = ACTIONS(2847),
    [sym_ip_server_close] = ACTIONS(2847),
    [sym_ip_bound_client_open] = ACTIONS(2847),
    [sym_ip_mc_server_open] = ACTIONS(2847),
    [sym_ip_set_option] = ACTIONS(2847),
    [sym_ssh_client_open] = ACTIONS(2847),
    [sym_ssh_client_close] = ACTIONS(2847),
    [sym_tls_client_open] = ACTIONS(2847),
    [sym_tls_client_close] = ACTIONS(2847),
    [sym_do_push] = ACTIONS(2847),
    [sym_do_push_timed] = ACTIONS(2847),
    [sym_do_release] = ACTIONS(2847),
    [sym_device_id] = ACTIONS(2847),
    [sym_device_info] = ACTIONS(2847),
    [sym_device_id_string] = ACTIONS(2847),
    [sym_device_standby] = ACTIONS(2847),
    [sym_device_wake] = ACTIONS(2847),
    [sym_timeline_create] = ACTIONS(2847),
    [sym_timeline_kill] = ACTIONS(2847),
    [sym_timeline_reload] = ACTIONS(2847),
    [sym_timeline_pause] = ACTIONS(2847),
    [sym_timeline_restart] = ACTIONS(2847),
    [sym_timeline_set] = ACTIONS(2847),
    [sym_timeline_get] = ACTIONS(2847),
    [sym_timeline_active] = ACTIONS(2847),
    [sym_timeline_dynamic_id] = ACTIONS(2847),
    [sym_variable_to_string] = ACTIONS(2847),
    [sym_string_to_variable] = ACTIONS(2847),
    [sym_length_variable_to_string] = ACTIONS(2847),
    [sym_type_cast] = ACTIONS(2847),
    [sym_variable_to_xml] = ACTIONS(2847),
    [sym_xml_to_variable] = ACTIONS(2847),
    [sym_length_variable_to_xml] = ACTIONS(2847),
    [sym_redirect_string] = ACTIONS(2847),
    [sym_random_number] = ACTIONS(2847),
    [sym_get_unique_id] = ACTIONS(2847),
    [sym_get_system_number] = ACTIONS(2847),
    [sym_set_system_number] = ACTIONS(2847),
    [sym_get_serial_number] = ACTIONS(2847),
    [sym_get_master_build] = ACTIONS(2847),
    [sym_set_validation_code] = ACTIONS(2847),
    [sym_reboot] = ACTIONS(2847),
    [sym_load_duet_module] = ACTIONS(2847),
    [sym_unload_duet_module] = ACTIONS(2847),
    [sym_duet_mem_size_set] = ACTIONS(2847),
    [sym_duet_mem_size_get] = ACTIONS(2847),
    [sym_dynamic_application_device] = ACTIONS(2847),
    [sym_dynamic_polled_port] = ACTIONS(2847),
    [sym_static_port_binding] = ACTIONS(2847),
    [sym_static_ip_binding] = ACTIONS(2847),
    [sym_get_dns_list] = ACTIONS(2847),
    [sym_set_dns_list] = ACTIONS(2847),
    [sym_get_ip_address] = ACTIONS(2847),
    [sym_set_ip_address] = ACTIONS(2847),
    [sym_get_url_list] = ACTIONS(2847),
    [sym_add_url_entry] = ACTIONS(2847),
    [sym_delete_url_entry] = ACTIONS(2847),
    [sym_abs_value] = ACTIONS(2847),
    [sym_max_value] = ACTIONS(2847),
    [sym_min_value] = ACTIONS(2847),
    [sym_exp_value] = ACTIONS(2847),
    [sym_log_value] = ACTIONS(2847),
    [sym_log10_value] = ACTIONS(2847),
    [sym_power_value] = ACTIONS(2847),
    [sym_sqrt_value] = ACTIONS(2847),
    [sym_raw_be] = ACTIONS(2847),
    [sym_raw_le] = ACTIONS(2847),
    [sym_internal_queue_size_set] = ACTIONS(2847),
    [sym_internal_queue_size_get] = ACTIONS(2847),
    [sym_internal_threshold_set] = ACTIONS(2847),
    [sym_internal_threshold_get] = ACTIONS(2847),
    [sym_rebuilt_event] = ACTIONS(2847),
    [sym_do_custom_event] = ACTIONS(2847),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2847),
    [sym_clkmgr_set_clk_source] = ACTIONS(2847),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2847),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2847),
    [sym_clkmgr_get_timezone] = ACTIONS(2847),
    [sym_clkmgr_set_timezone] = ACTIONS(2847),
    [sym_clkmgr_get_resync_period] = ACTIONS(2847),
    [sym_clkmgr_set_resync_period] = ACTIONS(2847),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2847),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2847),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2847),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2847),
    [sym_clkmgr_get_timeservers] = ACTIONS(2847),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2847),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2847),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2847),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2847),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2847),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2847),
    [sym_set_log_level] = ACTIONS(2847),
    [sym_get_log_level] = ACTIONS(2847),
    [sym_amx_log] = ACTIONS(2847),
    [sym_smtp_server_config_set] = ACTIONS(2847),
    [sym_smtp_server_config_get] = ACTIONS(2847),
    [sym_smtp_send] = ACTIONS(2847),
    [sym_validate_netlinx_account] = ACTIONS(2847),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2847),
    [sym_authenticate_certificate] = ACTIONS(2847),
    [sym_audit_netlinx_session_event] = ACTIONS(2847),
    [sym_audit_netlinx_generic_event] = ACTIONS(2847),
    [sym_get_available_flash_disk_space] = ACTIONS(2847),
    [sym_get_max_flash_disk_space] = ACTIONS(2847),
    [sym_data_create_feed] = ACTIONS(2847),
    [sym_data_delete_feed] = ACTIONS(2847),
    [sym_data_publish_feed] = ACTIONS(2847),
    [sym_data_get_published_feed] = ACTIONS(2847),
    [sym_data_add_record] = ACTIONS(2847),
    [sym_data_get_event_record] = ACTIONS(2847),
    [sym__wc_data_create_feed] = ACTIONS(2847),
    [sym__wc_data_add_record] = ACTIONS(2847),
    [sym__wc_data_get_event_record] = ACTIONS(2847),
    [sym_format] = ACTIONS(2847),
    [sym_astro_clock] = ACTIONS(2847),
    [sym_set_outdoor_temperature] = ACTIONS(2847),
    [sym_set_virtual_level_count] = ACTIONS(2847),
    [sym_set_virtual_channel_count] = ACTIONS(2847),
    [sym_set_virtual_port_count] = ACTIONS(2847),
    [sym__wc_encode_int] = ACTIONS(2847),
    [sym__wc_decode_int] = ACTIONS(2847),
    [sym__wc_int] = ACTIONS(2847),
    [sym__wc_to_ch_int] = ACTIONS(2847),
    [sym__ch_to_wc_int] = ACTIONS(2847),
    [sym__wc_find_string_int] = ACTIONS(2847),
    [sym__wc_left_string_int] = ACTIONS(2847),
    [sym__wc_lower_string_int] = ACTIONS(2847),
    [sym__wc_mid_string_int] = ACTIONS(2847),
    [sym__wc_remove_string_int] = ACTIONS(2847),
    [sym__wc_right_string_int] = ACTIONS(2847),
    [sym__wc_upper_string_int] = ACTIONS(2847),
    [sym__wc_compare_string_int] = ACTIONS(2847),
    [sym__wc_get_buffer_char_int] = ACTIONS(2847),
    [sym__wc_get_buffer_string_int] = ACTIONS(2847),
    [sym__wc_concat_string_int] = ACTIONS(2847),
    [sym__wc_file_open_int] = ACTIONS(2847),
    [sym__wc_file_close_int] = ACTIONS(2847),
    [sym__wc_file_read_int] = ACTIONS(2847),
    [sym__wc_file_read_line_int] = ACTIONS(2847),
    [sym__wc_file_write_int] = ACTIONS(2847),
    [sym__wc_file_write_line_int] = ACTIONS(2847),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2849),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2849),
    [anon_sym_LBRACK] = ACTIONS(2849),
    [anon_sym_LBRACE] = ACTIONS(2849),
    [anon_sym_RBRACE] = ACTIONS(2849),
    [anon_sym_LPAREN] = ACTIONS(2847),
    [anon_sym_BANG] = ACTIONS(2849),
    [anon_sym_TILDE] = ACTIONS(2849),
    [anon_sym_DASH] = ACTIONS(2847),
    [anon_sym_PLUS] = ACTIONS(2847),
    [anon_sym_DASH_DASH] = ACTIONS(2849),
    [anon_sym_PLUS_PLUS] = ACTIONS(2849),
    [anon_sym_DQUOTE] = ACTIONS(2849),
    [anon_sym_SQUOTE] = ACTIONS(2849),
    [sym_number_literal] = ACTIONS(2849),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(957)] = {
    [sym_identifier] = ACTIONS(2851),
    [sym_not] = ACTIONS(2851),
    [sym_bnot] = ACTIONS(2851),
    [sym___file__] = ACTIONS(2851),
    [sym___date__] = ACTIONS(2851),
    [sym___ldate__] = ACTIONS(2851),
    [sym___time__] = ACTIONS(2851),
    [sym___line__] = ACTIONS(2851),
    [sym___name__] = ACTIONS(2851),
    [sym___netlinx__] = ACTIONS(2851),
    [sym_netlinx_axi_version] = ACTIONS(2851),
    [sym_url_flg_tcp] = ACTIONS(2851),
    [sym_url_flg_accinfopresent] = ACTIONS(2851),
    [sym_url_flg_temp] = ACTIONS(2851),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2851),
    [sym_url_flg_stat_mask] = ACTIONS(2851),
    [sym_url_flg_stat_lookup] = ACTIONS(2851),
    [sym_url_flg_stat_connecting] = ACTIONS(2851),
    [sym_url_flg_stat_waiting] = ACTIONS(2851),
    [sym_url_flg_stat_connected] = ACTIONS(2851),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2851),
    [sym_first_virtual_device] = ACTIONS(2851),
    [sym_first_local_port] = ACTIONS(2851),
    [sym_dynamic_virtual_device] = ACTIONS(2851),
    [sym_timeline_once] = ACTIONS(2851),
    [sym_timeline_repeat] = ACTIONS(2851),
    [sym_timeline_absolute] = ACTIONS(2851),
    [sym_timeline_relative] = ACTIONS(2851),
    [sym_do_push_timed_infinite] = ACTIONS(2851),
    [sym_source_type_no_address] = ACTIONS(2851),
    [sym_source_type_neuron_id] = ACTIONS(2851),
    [sym_source_type_ip_address] = ACTIONS(2851),
    [sym_source_type_axlink] = ACTIONS(2851),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2851),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2851),
    [sym_source_type_ip_socket_address] = ACTIONS(2851),
    [sym_source_type_rs232] = ACTIONS(2851),
    [sym_source_type_internal] = ACTIONS(2851),
    [sym_source_type_mac_address] = ACTIONS(2851),
    [sym_source_type_ipv4_port] = ACTIONS(2851),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2851),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2851),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2851),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2851),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2851),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2851),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2851),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2851),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2851),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2851),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2851),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2851),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2851),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2851),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2851),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2851),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2851),
    [sym_true] = ACTIONS(2851),
    [sym_false] = ACTIONS(2851),
    [sym_file_read_only] = ACTIONS(2851),
    [sym_file_rw_new] = ACTIONS(2851),
    [sym_file_rw_append] = ACTIONS(2851),
    [sym_ip_tcp] = ACTIONS(2851),
    [sym_ip_udp] = ACTIONS(2851),
    [sym_ip_udp_2way] = ACTIONS(2851),
    [sym_xml_encode_types] = ACTIONS(2851),
    [sym_xml_encode_char_as_list] = ACTIONS(2851),
    [sym_xml_encode_le] = ACTIONS(2851),
    [sym_xml_decode_types] = ACTIONS(2851),
    [sym_xml_decode_no_preserve] = ACTIONS(2851),
    [sym_duet_dev_type_display_device] = ACTIONS(2851),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2851),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2851),
    [sym_duet_dev_type_io_device] = ACTIONS(2851),
    [sym_duet_dev_type_relay_device] = ACTIONS(2851),
    [sym_duet_dev_type_ups] = ACTIONS(2851),
    [sym_duet_dev_type_amplifier] = ACTIONS(2851),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2851),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2851),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2851),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2851),
    [sym_duet_dev_type_camera] = ACTIONS(2851),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2851),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2851),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2851),
    [sym_duet_dev_type_dss] = ACTIONS(2851),
    [sym_duet_dev_type_dvr] = ACTIONS(2851),
    [sym_duet_dev_type_disc_device] = ACTIONS(2851),
    [sym_duet_dev_type_document_camera] = ACTIONS(2851),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2851),
    [sym_duet_dev_type_hvac] = ACTIONS(2851),
    [sym_duet_dev_type_keypad] = ACTIONS(2851),
    [sym_duet_dev_type_light] = ACTIONS(2851),
    [sym_duet_dev_type_monitor] = ACTIONS(2851),
    [sym_duet_dev_type_motor] = ACTIONS(2851),
    [sym_duet_dev_type_multi_window] = ACTIONS(2851),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2851),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2851),
    [sym_duet_dev_type_receiver] = ACTIONS(2851),
    [sym_duet_dev_type_security_system] = ACTIONS(2851),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2851),
    [sym_duet_dev_type_settop_box] = ACTIONS(2851),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2851),
    [sym_duet_dev_type_switcher] = ACTIONS(2851),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2851),
    [sym_duet_dev_type_tv] = ACTIONS(2851),
    [sym_duet_dev_type_utility] = ACTIONS(2851),
    [sym_duet_dev_type_vcr] = ACTIONS(2851),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2851),
    [sym_duet_dev_type_video_processor] = ACTIONS(2851),
    [sym_duet_dev_type_video_projector] = ACTIONS(2851),
    [sym_duet_dev_type_video_wall] = ACTIONS(2851),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2851),
    [sym_duet_dev_type_weather] = ACTIONS(2851),
    [sym_duet_dev_not_polled] = ACTIONS(2851),
    [sym_duet_dev_polled] = ACTIONS(2851),
    [sym_clkmgr_mode_network] = ACTIONS(2851),
    [sym_clkmgr_mode_standalone] = ACTIONS(2851),
    [sym_normal_standby] = ACTIONS(2851),
    [sym_normal_wake] = ACTIONS(2851),
    [sym_ip_multicast_ttl_option] = ACTIONS(2851),
    [sym_ip_ttl_subnet] = ACTIONS(2851),
    [sym_ip_ttl_site] = ACTIONS(2851),
    [sym_ip_ttl_region] = ACTIONS(2851),
    [sym_ip_ttl_continent] = ACTIONS(2851),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2851),
    [sym_ip_nodelay_on] = ACTIONS(2851),
    [sym_ip_nodelay_off] = ACTIONS(2851),
    [sym_amx_error] = ACTIONS(2851),
    [sym_amx_warning] = ACTIONS(2851),
    [sym_amx_info] = ACTIONS(2851),
    [sym_amx_debug] = ACTIONS(2851),
    [sym_netlinx_login_success] = ACTIONS(2851),
    [sym_netlinx_login_fail] = ACTIONS(2851),
    [sym_netlinx_logout] = ACTIONS(2851),
    [sym_null_str] = ACTIONS(2851),
    [sym_smtp_address] = ACTIONS(2851),
    [sym_smtp_port_number] = ACTIONS(2851),
    [sym_smtp_username] = ACTIONS(2851),
    [sym_smtp_password] = ACTIONS(2851),
    [sym_smtp_require_tls] = ACTIONS(2851),
    [sym_smtp_from] = ACTIONS(2851),
    [sym_smtp_tls_true] = ACTIONS(2851),
    [sym_smtp_tls_false] = ACTIONS(2851),
    [sym_listview_on_row_select_event] = ACTIONS(2851),
    [sym_data_structure_unknown] = ACTIONS(2851),
    [sym_data_structure_datafeed] = ACTIONS(2851),
    [sym_data_structure_datarecordset] = ACTIONS(2851),
    [sym_data_structure_datarecord] = ACTIONS(2851),
    [sym_data_structure_datafield] = ACTIONS(2851),
    [sym_data_max_content_field_count] = ACTIONS(2851),
    [sym_data_max_metadata_field_count] = ACTIONS(2851),
    [sym_data_max_id_length] = ACTIONS(2851),
    [sym_data_max_name_length] = ACTIONS(2851),
    [sym_data_max_description_length] = ACTIONS(2851),
    [sym_data_max_type_length] = ACTIONS(2851),
    [sym_data_max_format_length] = ACTIONS(2851),
    [sym_data_max_source_length] = ACTIONS(2851),
    [sym_data_max_value_length] = ACTIONS(2851),
    [sym_data_max_label_length] = ACTIONS(2851),
    [sym_data_max_url_length] = ACTIONS(2851),
    [sym_max_last_login_info_length] = ACTIONS(2851),
    [sym_data_type_unknown] = ACTIONS(2851),
    [sym_data_type_string] = ACTIONS(2851),
    [sym_data_type_datetime] = ACTIONS(2851),
    [sym_data_type_date] = ACTIONS(2851),
    [sym_data_type_time] = ACTIONS(2851),
    [sym_data_type_image] = ACTIONS(2851),
    [sym_data_format_url] = ACTIONS(2851),
    [sym_data_format_phone] = ACTIONS(2851),
    [sym_data_format_email] = ACTIONS(2851),
    [sym_data_format_iso8601] = ACTIONS(2851),
    [sym_auditlog_priv] = ACTIONS(2851),
    [sym_configuration_priv] = ACTIONS(2851),
    [sym_deviceconfig_priv] = ACTIONS(2851),
    [sym_ftp_priv] = ACTIONS(2851),
    [sym_http_priv] = ACTIONS(2851),
    [sym_networkconfig_priv] = ACTIONS(2851),
    [sym_programport_priv] = ACTIONS(2851),
    [sym_remoteui_priv] = ACTIONS(2851),
    [sym_securitycontrol_priv] = ACTIONS(2851),
    [sym_softwaremanagement_priv] = ACTIONS(2851),
    [sym_terminal_priv] = ACTIONS(2851),
    [sym_tpadmin_priv] = ACTIONS(2851),
    [sym_user1_priv] = ACTIONS(2851),
    [sym_user2_priv] = ACTIONS(2851),
    [sym_user3_priv] = ACTIONS(2851),
    [sym_user4_priv] = ACTIONS(2851),
    [sym_usermanagement_priv] = ACTIONS(2851),
    [sym_valid_account] = ACTIONS(2851),
    [sym_err_name_invalid] = ACTIONS(2851),
    [sym_err_password_invalid] = ACTIONS(2851),
    [sym_err_invalid_account] = ACTIONS(2851),
    [sym_err_no_authorization] = ACTIONS(2851),
    [sym_err_invalid_parameter] = ACTIONS(2851),
    [sym_err_user_locked_out] = ACTIONS(2851),
    [sym_err_user_account_expired] = ACTIONS(2851),
    [sym_err_general_error] = ACTIONS(2851),
    [sym_err_authentication_error] = ACTIONS(2851),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2851),
    [sym_tls_validate_certificate] = ACTIONS(2851),
    [sym____reserved___] = ACTIONS(2851),
    [sym_date] = ACTIONS(2851),
    [sym_day] = ACTIONS(2851),
    [sym_time] = ACTIONS(2851),
    [sym_ldate] = ACTIONS(2851),
    [sym_push_device] = ACTIONS(2851),
    [sym_push_channel] = ACTIONS(2851),
    [sym_push_devchan] = ACTIONS(2851),
    [sym_release_device] = ACTIONS(2851),
    [sym_release_channel] = ACTIONS(2851),
    [sym_release_devchan] = ACTIONS(2851),
    [sym_master_slot] = ACTIONS(2851),
    [sym_get_pulse_time] = ACTIONS(2851),
    [sym_get_timer] = ACTIONS(2851),
    [sym_master_sn] = ACTIONS(2851),
    [sym_system_number] = ACTIONS(2851),
    [sym_button] = ACTIONS(2851),
    [sym_dv_channel] = ACTIONS(2851),
    [sym_data] = ACTIONS(2851),
    [sym_level] = ACTIONS(2851),
    [sym_timeline] = ACTIONS(2851),
    [sym_channel] = ACTIONS(2851),
    [sym_custom] = ACTIONS(2851),
    [sym_atoi] = ACTIONS(2851),
    [sym_atol] = ACTIONS(2851),
    [sym_atof] = ACTIONS(2851),
    [sym_itoa] = ACTIONS(2851),
    [sym_itohex] = ACTIONS(2851),
    [sym_hextoi] = ACTIONS(2851),
    [sym_ftoa] = ACTIONS(2851),
    [sym_find_string] = ACTIONS(2851),
    [sym_left_string] = ACTIONS(2851),
    [sym_right_string] = ACTIONS(2851),
    [sym_mid_string] = ACTIONS(2851),
    [sym_remove_string] = ACTIONS(2851),
    [sym_get_buffer_string] = ACTIONS(2851),
    [sym_get_buffer_char] = ACTIONS(2851),
    [sym_get_multi_buffer_string] = ACTIONS(2851),
    [sym_length_string] = ACTIONS(2851),
    [sym_max_length_string] = ACTIONS(2851),
    [sym_set_length_string] = ACTIONS(2851),
    [sym_compare_string] = ACTIONS(2851),
    [sym_upper_string] = ACTIONS(2851),
    [sym_lower_string] = ACTIONS(2851),
    [sym_get_last] = ACTIONS(2851),
    [sym_date_to_day] = ACTIONS(2851),
    [sym_date_to_month] = ACTIONS(2851),
    [sym_date_to_year] = ACTIONS(2851),
    [sym_time_to_hour] = ACTIONS(2851),
    [sym_time_to_minute] = ACTIONS(2851),
    [sym_time_to_second] = ACTIONS(2851),
    [sym_day_of_week] = ACTIONS(2851),
    [sym_set_pulse_time] = ACTIONS(2851),
    [sym_set_timer] = ACTIONS(2851),
    [sym_file_open] = ACTIONS(2851),
    [sym_file_close] = ACTIONS(2851),
    [sym_file_read] = ACTIONS(2851),
    [sym_file_read_line] = ACTIONS(2851),
    [sym_file_write] = ACTIONS(2851),
    [sym_file_write_line] = ACTIONS(2851),
    [sym_file_copy] = ACTIONS(2851),
    [sym_file_delete] = ACTIONS(2851),
    [sym_file_rename] = ACTIONS(2851),
    [sym_file_dir] = ACTIONS(2851),
    [sym_file_getdir] = ACTIONS(2851),
    [sym_file_setdir] = ACTIONS(2851),
    [sym_file_create_dir] = ACTIONS(2851),
    [sym_file_remove_dir] = ACTIONS(2851),
    [sym_file_seek] = ACTIONS(2851),
    [sym_length_array] = ACTIONS(2851),
    [sym_max_length_array] = ACTIONS(2851),
    [sym_set_length_array] = ACTIONS(2851),
    [sym_ip_client_open] = ACTIONS(2851),
    [sym_ip_client_close] = ACTIONS(2851),
    [sym_ip_server_open] = ACTIONS(2851),
    [sym_ip_server_close] = ACTIONS(2851),
    [sym_ip_bound_client_open] = ACTIONS(2851),
    [sym_ip_mc_server_open] = ACTIONS(2851),
    [sym_ip_set_option] = ACTIONS(2851),
    [sym_ssh_client_open] = ACTIONS(2851),
    [sym_ssh_client_close] = ACTIONS(2851),
    [sym_tls_client_open] = ACTIONS(2851),
    [sym_tls_client_close] = ACTIONS(2851),
    [sym_do_push] = ACTIONS(2851),
    [sym_do_push_timed] = ACTIONS(2851),
    [sym_do_release] = ACTIONS(2851),
    [sym_device_id] = ACTIONS(2851),
    [sym_device_info] = ACTIONS(2851),
    [sym_device_id_string] = ACTIONS(2851),
    [sym_device_standby] = ACTIONS(2851),
    [sym_device_wake] = ACTIONS(2851),
    [sym_timeline_create] = ACTIONS(2851),
    [sym_timeline_kill] = ACTIONS(2851),
    [sym_timeline_reload] = ACTIONS(2851),
    [sym_timeline_pause] = ACTIONS(2851),
    [sym_timeline_restart] = ACTIONS(2851),
    [sym_timeline_set] = ACTIONS(2851),
    [sym_timeline_get] = ACTIONS(2851),
    [sym_timeline_active] = ACTIONS(2851),
    [sym_timeline_dynamic_id] = ACTIONS(2851),
    [sym_variable_to_string] = ACTIONS(2851),
    [sym_string_to_variable] = ACTIONS(2851),
    [sym_length_variable_to_string] = ACTIONS(2851),
    [sym_type_cast] = ACTIONS(2851),
    [sym_variable_to_xml] = ACTIONS(2851),
    [sym_xml_to_variable] = ACTIONS(2851),
    [sym_length_variable_to_xml] = ACTIONS(2851),
    [sym_redirect_string] = ACTIONS(2851),
    [sym_random_number] = ACTIONS(2851),
    [sym_get_unique_id] = ACTIONS(2851),
    [sym_get_system_number] = ACTIONS(2851),
    [sym_set_system_number] = ACTIONS(2851),
    [sym_get_serial_number] = ACTIONS(2851),
    [sym_get_master_build] = ACTIONS(2851),
    [sym_set_validation_code] = ACTIONS(2851),
    [sym_reboot] = ACTIONS(2851),
    [sym_load_duet_module] = ACTIONS(2851),
    [sym_unload_duet_module] = ACTIONS(2851),
    [sym_duet_mem_size_set] = ACTIONS(2851),
    [sym_duet_mem_size_get] = ACTIONS(2851),
    [sym_dynamic_application_device] = ACTIONS(2851),
    [sym_dynamic_polled_port] = ACTIONS(2851),
    [sym_static_port_binding] = ACTIONS(2851),
    [sym_static_ip_binding] = ACTIONS(2851),
    [sym_get_dns_list] = ACTIONS(2851),
    [sym_set_dns_list] = ACTIONS(2851),
    [sym_get_ip_address] = ACTIONS(2851),
    [sym_set_ip_address] = ACTIONS(2851),
    [sym_get_url_list] = ACTIONS(2851),
    [sym_add_url_entry] = ACTIONS(2851),
    [sym_delete_url_entry] = ACTIONS(2851),
    [sym_abs_value] = ACTIONS(2851),
    [sym_max_value] = ACTIONS(2851),
    [sym_min_value] = ACTIONS(2851),
    [sym_exp_value] = ACTIONS(2851),
    [sym_log_value] = ACTIONS(2851),
    [sym_log10_value] = ACTIONS(2851),
    [sym_power_value] = ACTIONS(2851),
    [sym_sqrt_value] = ACTIONS(2851),
    [sym_raw_be] = ACTIONS(2851),
    [sym_raw_le] = ACTIONS(2851),
    [sym_internal_queue_size_set] = ACTIONS(2851),
    [sym_internal_queue_size_get] = ACTIONS(2851),
    [sym_internal_threshold_set] = ACTIONS(2851),
    [sym_internal_threshold_get] = ACTIONS(2851),
    [sym_rebuilt_event] = ACTIONS(2851),
    [sym_do_custom_event] = ACTIONS(2851),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2851),
    [sym_clkmgr_set_clk_source] = ACTIONS(2851),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2851),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2851),
    [sym_clkmgr_get_timezone] = ACTIONS(2851),
    [sym_clkmgr_set_timezone] = ACTIONS(2851),
    [sym_clkmgr_get_resync_period] = ACTIONS(2851),
    [sym_clkmgr_set_resync_period] = ACTIONS(2851),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2851),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2851),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2851),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2851),
    [sym_clkmgr_get_timeservers] = ACTIONS(2851),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2851),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2851),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2851),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2851),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2851),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2851),
    [sym_set_log_level] = ACTIONS(2851),
    [sym_get_log_level] = ACTIONS(2851),
    [sym_amx_log] = ACTIONS(2851),
    [sym_smtp_server_config_set] = ACTIONS(2851),
    [sym_smtp_server_config_get] = ACTIONS(2851),
    [sym_smtp_send] = ACTIONS(2851),
    [sym_validate_netlinx_account] = ACTIONS(2851),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2851),
    [sym_authenticate_certificate] = ACTIONS(2851),
    [sym_audit_netlinx_session_event] = ACTIONS(2851),
    [sym_audit_netlinx_generic_event] = ACTIONS(2851),
    [sym_get_available_flash_disk_space] = ACTIONS(2851),
    [sym_get_max_flash_disk_space] = ACTIONS(2851),
    [sym_data_create_feed] = ACTIONS(2851),
    [sym_data_delete_feed] = ACTIONS(2851),
    [sym_data_publish_feed] = ACTIONS(2851),
    [sym_data_get_published_feed] = ACTIONS(2851),
    [sym_data_add_record] = ACTIONS(2851),
    [sym_data_get_event_record] = ACTIONS(2851),
    [sym__wc_data_create_feed] = ACTIONS(2851),
    [sym__wc_data_add_record] = ACTIONS(2851),
    [sym__wc_data_get_event_record] = ACTIONS(2851),
    [sym_format] = ACTIONS(2851),
    [sym_astro_clock] = ACTIONS(2851),
    [sym_set_outdoor_temperature] = ACTIONS(2851),
    [sym_set_virtual_level_count] = ACTIONS(2851),
    [sym_set_virtual_channel_count] = ACTIONS(2851),
    [sym_set_virtual_port_count] = ACTIONS(2851),
    [sym__wc_encode_int] = ACTIONS(2851),
    [sym__wc_decode_int] = ACTIONS(2851),
    [sym__wc_int] = ACTIONS(2851),
    [sym__wc_to_ch_int] = ACTIONS(2851),
    [sym__ch_to_wc_int] = ACTIONS(2851),
    [sym__wc_find_string_int] = ACTIONS(2851),
    [sym__wc_left_string_int] = ACTIONS(2851),
    [sym__wc_lower_string_int] = ACTIONS(2851),
    [sym__wc_mid_string_int] = ACTIONS(2851),
    [sym__wc_remove_string_int] = ACTIONS(2851),
    [sym__wc_right_string_int] = ACTIONS(2851),
    [sym__wc_upper_string_int] = ACTIONS(2851),
    [sym__wc_compare_string_int] = ACTIONS(2851),
    [sym__wc_get_buffer_char_int] = ACTIONS(2851),
    [sym__wc_get_buffer_string_int] = ACTIONS(2851),
    [sym__wc_concat_string_int] = ACTIONS(2851),
    [sym__wc_file_open_int] = ACTIONS(2851),
    [sym__wc_file_close_int] = ACTIONS(2851),
    [sym__wc_file_read_int] = ACTIONS(2851),
    [sym__wc_file_read_line_int] = ACTIONS(2851),
    [sym__wc_file_write_int] = ACTIONS(2851),
    [sym__wc_file_write_line_int] = ACTIONS(2851),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2853),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2853),
    [anon_sym_LBRACK] = ACTIONS(2853),
    [anon_sym_LBRACE] = ACTIONS(2853),
    [anon_sym_RBRACE] = ACTIONS(2853),
    [anon_sym_LPAREN] = ACTIONS(2851),
    [anon_sym_BANG] = ACTIONS(2853),
    [anon_sym_TILDE] = ACTIONS(2853),
    [anon_sym_DASH] = ACTIONS(2851),
    [anon_sym_PLUS] = ACTIONS(2851),
    [anon_sym_DASH_DASH] = ACTIONS(2853),
    [anon_sym_PLUS_PLUS] = ACTIONS(2853),
    [anon_sym_DQUOTE] = ACTIONS(2853),
    [anon_sym_SQUOTE] = ACTIONS(2853),
    [sym_number_literal] = ACTIONS(2853),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(958)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2226),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(959)] = {
    [sym_identifier] = ACTIONS(2266),
    [sym_not] = ACTIONS(2266),
    [sym_bnot] = ACTIONS(2266),
    [sym___file__] = ACTIONS(2266),
    [sym___date__] = ACTIONS(2266),
    [sym___ldate__] = ACTIONS(2266),
    [sym___time__] = ACTIONS(2266),
    [sym___line__] = ACTIONS(2266),
    [sym___name__] = ACTIONS(2266),
    [sym___netlinx__] = ACTIONS(2266),
    [sym_netlinx_axi_version] = ACTIONS(2266),
    [sym_url_flg_tcp] = ACTIONS(2266),
    [sym_url_flg_accinfopresent] = ACTIONS(2266),
    [sym_url_flg_temp] = ACTIONS(2266),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2266),
    [sym_url_flg_stat_mask] = ACTIONS(2266),
    [sym_url_flg_stat_lookup] = ACTIONS(2266),
    [sym_url_flg_stat_connecting] = ACTIONS(2266),
    [sym_url_flg_stat_waiting] = ACTIONS(2266),
    [sym_url_flg_stat_connected] = ACTIONS(2266),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2266),
    [sym_first_virtual_device] = ACTIONS(2266),
    [sym_first_local_port] = ACTIONS(2266),
    [sym_dynamic_virtual_device] = ACTIONS(2266),
    [sym_timeline_once] = ACTIONS(2266),
    [sym_timeline_repeat] = ACTIONS(2266),
    [sym_timeline_absolute] = ACTIONS(2266),
    [sym_timeline_relative] = ACTIONS(2266),
    [sym_do_push_timed_infinite] = ACTIONS(2266),
    [sym_source_type_no_address] = ACTIONS(2266),
    [sym_source_type_neuron_id] = ACTIONS(2266),
    [sym_source_type_ip_address] = ACTIONS(2266),
    [sym_source_type_axlink] = ACTIONS(2266),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2266),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2266),
    [sym_source_type_ip_socket_address] = ACTIONS(2266),
    [sym_source_type_rs232] = ACTIONS(2266),
    [sym_source_type_internal] = ACTIONS(2266),
    [sym_source_type_mac_address] = ACTIONS(2266),
    [sym_source_type_ipv4_port] = ACTIONS(2266),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2266),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2266),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2266),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2266),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2266),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2266),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2266),
    [sym_true] = ACTIONS(2266),
    [sym_false] = ACTIONS(2266),
    [sym_file_read_only] = ACTIONS(2266),
    [sym_file_rw_new] = ACTIONS(2266),
    [sym_file_rw_append] = ACTIONS(2266),
    [sym_ip_tcp] = ACTIONS(2266),
    [sym_ip_udp] = ACTIONS(2266),
    [sym_ip_udp_2way] = ACTIONS(2266),
    [sym_xml_encode_types] = ACTIONS(2266),
    [sym_xml_encode_char_as_list] = ACTIONS(2266),
    [sym_xml_encode_le] = ACTIONS(2266),
    [sym_xml_decode_types] = ACTIONS(2266),
    [sym_xml_decode_no_preserve] = ACTIONS(2266),
    [sym_duet_dev_type_display_device] = ACTIONS(2266),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2266),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2266),
    [sym_duet_dev_type_io_device] = ACTIONS(2266),
    [sym_duet_dev_type_relay_device] = ACTIONS(2266),
    [sym_duet_dev_type_ups] = ACTIONS(2266),
    [sym_duet_dev_type_amplifier] = ACTIONS(2266),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2266),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2266),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2266),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2266),
    [sym_duet_dev_type_camera] = ACTIONS(2266),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2266),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2266),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2266),
    [sym_duet_dev_type_dss] = ACTIONS(2266),
    [sym_duet_dev_type_dvr] = ACTIONS(2266),
    [sym_duet_dev_type_disc_device] = ACTIONS(2266),
    [sym_duet_dev_type_document_camera] = ACTIONS(2266),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2266),
    [sym_duet_dev_type_hvac] = ACTIONS(2266),
    [sym_duet_dev_type_keypad] = ACTIONS(2266),
    [sym_duet_dev_type_light] = ACTIONS(2266),
    [sym_duet_dev_type_monitor] = ACTIONS(2266),
    [sym_duet_dev_type_motor] = ACTIONS(2266),
    [sym_duet_dev_type_multi_window] = ACTIONS(2266),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2266),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2266),
    [sym_duet_dev_type_receiver] = ACTIONS(2266),
    [sym_duet_dev_type_security_system] = ACTIONS(2266),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2266),
    [sym_duet_dev_type_settop_box] = ACTIONS(2266),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2266),
    [sym_duet_dev_type_switcher] = ACTIONS(2266),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2266),
    [sym_duet_dev_type_tv] = ACTIONS(2266),
    [sym_duet_dev_type_utility] = ACTIONS(2266),
    [sym_duet_dev_type_vcr] = ACTIONS(2266),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2266),
    [sym_duet_dev_type_video_processor] = ACTIONS(2266),
    [sym_duet_dev_type_video_projector] = ACTIONS(2266),
    [sym_duet_dev_type_video_wall] = ACTIONS(2266),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2266),
    [sym_duet_dev_type_weather] = ACTIONS(2266),
    [sym_duet_dev_not_polled] = ACTIONS(2266),
    [sym_duet_dev_polled] = ACTIONS(2266),
    [sym_clkmgr_mode_network] = ACTIONS(2266),
    [sym_clkmgr_mode_standalone] = ACTIONS(2266),
    [sym_normal_standby] = ACTIONS(2266),
    [sym_normal_wake] = ACTIONS(2266),
    [sym_ip_multicast_ttl_option] = ACTIONS(2266),
    [sym_ip_ttl_subnet] = ACTIONS(2266),
    [sym_ip_ttl_site] = ACTIONS(2266),
    [sym_ip_ttl_region] = ACTIONS(2266),
    [sym_ip_ttl_continent] = ACTIONS(2266),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2266),
    [sym_ip_nodelay_on] = ACTIONS(2266),
    [sym_ip_nodelay_off] = ACTIONS(2266),
    [sym_amx_error] = ACTIONS(2266),
    [sym_amx_warning] = ACTIONS(2266),
    [sym_amx_info] = ACTIONS(2266),
    [sym_amx_debug] = ACTIONS(2266),
    [sym_netlinx_login_success] = ACTIONS(2266),
    [sym_netlinx_login_fail] = ACTIONS(2266),
    [sym_netlinx_logout] = ACTIONS(2266),
    [sym_null_str] = ACTIONS(2266),
    [sym_smtp_address] = ACTIONS(2266),
    [sym_smtp_port_number] = ACTIONS(2266),
    [sym_smtp_username] = ACTIONS(2266),
    [sym_smtp_password] = ACTIONS(2266),
    [sym_smtp_require_tls] = ACTIONS(2266),
    [sym_smtp_from] = ACTIONS(2266),
    [sym_smtp_tls_true] = ACTIONS(2266),
    [sym_smtp_tls_false] = ACTIONS(2266),
    [sym_listview_on_row_select_event] = ACTIONS(2266),
    [sym_data_structure_unknown] = ACTIONS(2266),
    [sym_data_structure_datafeed] = ACTIONS(2266),
    [sym_data_structure_datarecordset] = ACTIONS(2266),
    [sym_data_structure_datarecord] = ACTIONS(2266),
    [sym_data_structure_datafield] = ACTIONS(2266),
    [sym_data_max_content_field_count] = ACTIONS(2266),
    [sym_data_max_metadata_field_count] = ACTIONS(2266),
    [sym_data_max_id_length] = ACTIONS(2266),
    [sym_data_max_name_length] = ACTIONS(2266),
    [sym_data_max_description_length] = ACTIONS(2266),
    [sym_data_max_type_length] = ACTIONS(2266),
    [sym_data_max_format_length] = ACTIONS(2266),
    [sym_data_max_source_length] = ACTIONS(2266),
    [sym_data_max_value_length] = ACTIONS(2266),
    [sym_data_max_label_length] = ACTIONS(2266),
    [sym_data_max_url_length] = ACTIONS(2266),
    [sym_max_last_login_info_length] = ACTIONS(2266),
    [sym_data_type_unknown] = ACTIONS(2266),
    [sym_data_type_string] = ACTIONS(2266),
    [sym_data_type_datetime] = ACTIONS(2266),
    [sym_data_type_date] = ACTIONS(2266),
    [sym_data_type_time] = ACTIONS(2266),
    [sym_data_type_image] = ACTIONS(2266),
    [sym_data_format_url] = ACTIONS(2266),
    [sym_data_format_phone] = ACTIONS(2266),
    [sym_data_format_email] = ACTIONS(2266),
    [sym_data_format_iso8601] = ACTIONS(2266),
    [sym_auditlog_priv] = ACTIONS(2266),
    [sym_configuration_priv] = ACTIONS(2266),
    [sym_deviceconfig_priv] = ACTIONS(2266),
    [sym_ftp_priv] = ACTIONS(2266),
    [sym_http_priv] = ACTIONS(2266),
    [sym_networkconfig_priv] = ACTIONS(2266),
    [sym_programport_priv] = ACTIONS(2266),
    [sym_remoteui_priv] = ACTIONS(2266),
    [sym_securitycontrol_priv] = ACTIONS(2266),
    [sym_softwaremanagement_priv] = ACTIONS(2266),
    [sym_terminal_priv] = ACTIONS(2266),
    [sym_tpadmin_priv] = ACTIONS(2266),
    [sym_user1_priv] = ACTIONS(2266),
    [sym_user2_priv] = ACTIONS(2266),
    [sym_user3_priv] = ACTIONS(2266),
    [sym_user4_priv] = ACTIONS(2266),
    [sym_usermanagement_priv] = ACTIONS(2266),
    [sym_valid_account] = ACTIONS(2266),
    [sym_err_name_invalid] = ACTIONS(2266),
    [sym_err_password_invalid] = ACTIONS(2266),
    [sym_err_invalid_account] = ACTIONS(2266),
    [sym_err_no_authorization] = ACTIONS(2266),
    [sym_err_invalid_parameter] = ACTIONS(2266),
    [sym_err_user_locked_out] = ACTIONS(2266),
    [sym_err_user_account_expired] = ACTIONS(2266),
    [sym_err_general_error] = ACTIONS(2266),
    [sym_err_authentication_error] = ACTIONS(2266),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2266),
    [sym_tls_validate_certificate] = ACTIONS(2266),
    [sym____reserved___] = ACTIONS(2266),
    [sym_date] = ACTIONS(2266),
    [sym_day] = ACTIONS(2266),
    [sym_time] = ACTIONS(2266),
    [sym_ldate] = ACTIONS(2266),
    [sym_push_device] = ACTIONS(2266),
    [sym_push_channel] = ACTIONS(2266),
    [sym_push_devchan] = ACTIONS(2266),
    [sym_release_device] = ACTIONS(2266),
    [sym_release_channel] = ACTIONS(2266),
    [sym_release_devchan] = ACTIONS(2266),
    [sym_master_slot] = ACTIONS(2266),
    [sym_get_pulse_time] = ACTIONS(2266),
    [sym_get_timer] = ACTIONS(2266),
    [sym_master_sn] = ACTIONS(2266),
    [sym_system_number] = ACTIONS(2266),
    [sym_button] = ACTIONS(2266),
    [sym_dv_channel] = ACTIONS(2266),
    [sym_data] = ACTIONS(2266),
    [sym_level] = ACTIONS(2266),
    [sym_timeline] = ACTIONS(2266),
    [sym_channel] = ACTIONS(2266),
    [sym_custom] = ACTIONS(2266),
    [sym_atoi] = ACTIONS(2266),
    [sym_atol] = ACTIONS(2266),
    [sym_atof] = ACTIONS(2266),
    [sym_itoa] = ACTIONS(2266),
    [sym_itohex] = ACTIONS(2266),
    [sym_hextoi] = ACTIONS(2266),
    [sym_ftoa] = ACTIONS(2266),
    [sym_find_string] = ACTIONS(2266),
    [sym_left_string] = ACTIONS(2266),
    [sym_right_string] = ACTIONS(2266),
    [sym_mid_string] = ACTIONS(2266),
    [sym_remove_string] = ACTIONS(2266),
    [sym_get_buffer_string] = ACTIONS(2266),
    [sym_get_buffer_char] = ACTIONS(2266),
    [sym_get_multi_buffer_string] = ACTIONS(2266),
    [sym_length_string] = ACTIONS(2266),
    [sym_max_length_string] = ACTIONS(2266),
    [sym_set_length_string] = ACTIONS(2266),
    [sym_compare_string] = ACTIONS(2266),
    [sym_upper_string] = ACTIONS(2266),
    [sym_lower_string] = ACTIONS(2266),
    [sym_get_last] = ACTIONS(2266),
    [sym_date_to_day] = ACTIONS(2266),
    [sym_date_to_month] = ACTIONS(2266),
    [sym_date_to_year] = ACTIONS(2266),
    [sym_time_to_hour] = ACTIONS(2266),
    [sym_time_to_minute] = ACTIONS(2266),
    [sym_time_to_second] = ACTIONS(2266),
    [sym_day_of_week] = ACTIONS(2266),
    [sym_set_pulse_time] = ACTIONS(2266),
    [sym_set_timer] = ACTIONS(2266),
    [sym_file_open] = ACTIONS(2266),
    [sym_file_close] = ACTIONS(2266),
    [sym_file_read] = ACTIONS(2266),
    [sym_file_read_line] = ACTIONS(2266),
    [sym_file_write] = ACTIONS(2266),
    [sym_file_write_line] = ACTIONS(2266),
    [sym_file_copy] = ACTIONS(2266),
    [sym_file_delete] = ACTIONS(2266),
    [sym_file_rename] = ACTIONS(2266),
    [sym_file_dir] = ACTIONS(2266),
    [sym_file_getdir] = ACTIONS(2266),
    [sym_file_setdir] = ACTIONS(2266),
    [sym_file_create_dir] = ACTIONS(2266),
    [sym_file_remove_dir] = ACTIONS(2266),
    [sym_file_seek] = ACTIONS(2266),
    [sym_length_array] = ACTIONS(2266),
    [sym_max_length_array] = ACTIONS(2266),
    [sym_set_length_array] = ACTIONS(2266),
    [sym_ip_client_open] = ACTIONS(2266),
    [sym_ip_client_close] = ACTIONS(2266),
    [sym_ip_server_open] = ACTIONS(2266),
    [sym_ip_server_close] = ACTIONS(2266),
    [sym_ip_bound_client_open] = ACTIONS(2266),
    [sym_ip_mc_server_open] = ACTIONS(2266),
    [sym_ip_set_option] = ACTIONS(2266),
    [sym_ssh_client_open] = ACTIONS(2266),
    [sym_ssh_client_close] = ACTIONS(2266),
    [sym_tls_client_open] = ACTIONS(2266),
    [sym_tls_client_close] = ACTIONS(2266),
    [sym_do_push] = ACTIONS(2266),
    [sym_do_push_timed] = ACTIONS(2266),
    [sym_do_release] = ACTIONS(2266),
    [sym_device_id] = ACTIONS(2266),
    [sym_device_info] = ACTIONS(2266),
    [sym_device_id_string] = ACTIONS(2266),
    [sym_device_standby] = ACTIONS(2266),
    [sym_device_wake] = ACTIONS(2266),
    [sym_timeline_create] = ACTIONS(2266),
    [sym_timeline_kill] = ACTIONS(2266),
    [sym_timeline_reload] = ACTIONS(2266),
    [sym_timeline_pause] = ACTIONS(2266),
    [sym_timeline_restart] = ACTIONS(2266),
    [sym_timeline_set] = ACTIONS(2266),
    [sym_timeline_get] = ACTIONS(2266),
    [sym_timeline_active] = ACTIONS(2266),
    [sym_timeline_dynamic_id] = ACTIONS(2266),
    [sym_variable_to_string] = ACTIONS(2266),
    [sym_string_to_variable] = ACTIONS(2266),
    [sym_length_variable_to_string] = ACTIONS(2266),
    [sym_type_cast] = ACTIONS(2266),
    [sym_variable_to_xml] = ACTIONS(2266),
    [sym_xml_to_variable] = ACTIONS(2266),
    [sym_length_variable_to_xml] = ACTIONS(2266),
    [sym_redirect_string] = ACTIONS(2266),
    [sym_random_number] = ACTIONS(2266),
    [sym_get_unique_id] = ACTIONS(2266),
    [sym_get_system_number] = ACTIONS(2266),
    [sym_set_system_number] = ACTIONS(2266),
    [sym_get_serial_number] = ACTIONS(2266),
    [sym_get_master_build] = ACTIONS(2266),
    [sym_set_validation_code] = ACTIONS(2266),
    [sym_reboot] = ACTIONS(2266),
    [sym_load_duet_module] = ACTIONS(2266),
    [sym_unload_duet_module] = ACTIONS(2266),
    [sym_duet_mem_size_set] = ACTIONS(2266),
    [sym_duet_mem_size_get] = ACTIONS(2266),
    [sym_dynamic_application_device] = ACTIONS(2266),
    [sym_dynamic_polled_port] = ACTIONS(2266),
    [sym_static_port_binding] = ACTIONS(2266),
    [sym_static_ip_binding] = ACTIONS(2266),
    [sym_get_dns_list] = ACTIONS(2266),
    [sym_set_dns_list] = ACTIONS(2266),
    [sym_get_ip_address] = ACTIONS(2266),
    [sym_set_ip_address] = ACTIONS(2266),
    [sym_get_url_list] = ACTIONS(2266),
    [sym_add_url_entry] = ACTIONS(2266),
    [sym_delete_url_entry] = ACTIONS(2266),
    [sym_abs_value] = ACTIONS(2266),
    [sym_max_value] = ACTIONS(2266),
    [sym_min_value] = ACTIONS(2266),
    [sym_exp_value] = ACTIONS(2266),
    [sym_log_value] = ACTIONS(2266),
    [sym_log10_value] = ACTIONS(2266),
    [sym_power_value] = ACTIONS(2266),
    [sym_sqrt_value] = ACTIONS(2266),
    [sym_raw_be] = ACTIONS(2266),
    [sym_raw_le] = ACTIONS(2266),
    [sym_internal_queue_size_set] = ACTIONS(2266),
    [sym_internal_queue_size_get] = ACTIONS(2266),
    [sym_internal_threshold_set] = ACTIONS(2266),
    [sym_internal_threshold_get] = ACTIONS(2266),
    [sym_rebuilt_event] = ACTIONS(2266),
    [sym_do_custom_event] = ACTIONS(2266),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2266),
    [sym_clkmgr_set_clk_source] = ACTIONS(2266),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2266),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2266),
    [sym_clkmgr_get_timezone] = ACTIONS(2266),
    [sym_clkmgr_set_timezone] = ACTIONS(2266),
    [sym_clkmgr_get_resync_period] = ACTIONS(2266),
    [sym_clkmgr_set_resync_period] = ACTIONS(2266),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2266),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2266),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2266),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2266),
    [sym_clkmgr_get_timeservers] = ACTIONS(2266),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2266),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2266),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2266),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2266),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2266),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2266),
    [sym_set_log_level] = ACTIONS(2266),
    [sym_get_log_level] = ACTIONS(2266),
    [sym_amx_log] = ACTIONS(2266),
    [sym_smtp_server_config_set] = ACTIONS(2266),
    [sym_smtp_server_config_get] = ACTIONS(2266),
    [sym_smtp_send] = ACTIONS(2266),
    [sym_validate_netlinx_account] = ACTIONS(2266),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2266),
    [sym_authenticate_certificate] = ACTIONS(2266),
    [sym_audit_netlinx_session_event] = ACTIONS(2266),
    [sym_audit_netlinx_generic_event] = ACTIONS(2266),
    [sym_get_available_flash_disk_space] = ACTIONS(2266),
    [sym_get_max_flash_disk_space] = ACTIONS(2266),
    [sym_data_create_feed] = ACTIONS(2266),
    [sym_data_delete_feed] = ACTIONS(2266),
    [sym_data_publish_feed] = ACTIONS(2266),
    [sym_data_get_published_feed] = ACTIONS(2266),
    [sym_data_add_record] = ACTIONS(2266),
    [sym_data_get_event_record] = ACTIONS(2266),
    [sym__wc_data_create_feed] = ACTIONS(2266),
    [sym__wc_data_add_record] = ACTIONS(2266),
    [sym__wc_data_get_event_record] = ACTIONS(2266),
    [sym_format] = ACTIONS(2266),
    [sym_astro_clock] = ACTIONS(2266),
    [sym_set_outdoor_temperature] = ACTIONS(2266),
    [sym_set_virtual_level_count] = ACTIONS(2266),
    [sym_set_virtual_channel_count] = ACTIONS(2266),
    [sym_set_virtual_port_count] = ACTIONS(2266),
    [sym__wc_encode_int] = ACTIONS(2266),
    [sym__wc_decode_int] = ACTIONS(2266),
    [sym__wc_int] = ACTIONS(2266),
    [sym__wc_to_ch_int] = ACTIONS(2266),
    [sym__ch_to_wc_int] = ACTIONS(2266),
    [sym__wc_find_string_int] = ACTIONS(2266),
    [sym__wc_left_string_int] = ACTIONS(2266),
    [sym__wc_lower_string_int] = ACTIONS(2266),
    [sym__wc_mid_string_int] = ACTIONS(2266),
    [sym__wc_remove_string_int] = ACTIONS(2266),
    [sym__wc_right_string_int] = ACTIONS(2266),
    [sym__wc_upper_string_int] = ACTIONS(2266),
    [sym__wc_compare_string_int] = ACTIONS(2266),
    [sym__wc_get_buffer_char_int] = ACTIONS(2266),
    [sym__wc_get_buffer_string_int] = ACTIONS(2266),
    [sym__wc_concat_string_int] = ACTIONS(2266),
    [sym__wc_file_open_int] = ACTIONS(2266),
    [sym__wc_file_close_int] = ACTIONS(2266),
    [sym__wc_file_read_int] = ACTIONS(2266),
    [sym__wc_file_read_line_int] = ACTIONS(2266),
    [sym__wc_file_write_int] = ACTIONS(2266),
    [sym__wc_file_write_line_int] = ACTIONS(2266),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2268),
    [aux_sym_preproc_else_token1] = ACTIONS(2268),
    [anon_sym_COMMA] = ACTIONS(2270),
    [anon_sym_LBRACK] = ACTIONS(2268),
    [anon_sym_LBRACE] = ACTIONS(2268),
    [anon_sym_LPAREN] = ACTIONS(2266),
    [anon_sym_BANG] = ACTIONS(2268),
    [anon_sym_TILDE] = ACTIONS(2268),
    [anon_sym_DASH] = ACTIONS(2266),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_DASH_DASH] = ACTIONS(2268),
    [anon_sym_PLUS_PLUS] = ACTIONS(2268),
    [anon_sym_DQUOTE] = ACTIONS(2268),
    [anon_sym_SQUOTE] = ACTIONS(2268),
    [sym_number_literal] = ACTIONS(2268),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(960)] = {
    [sym_identifier] = ACTIONS(2217),
    [sym_not] = ACTIONS(2217),
    [sym_bnot] = ACTIONS(2217),
    [sym___file__] = ACTIONS(2217),
    [sym___date__] = ACTIONS(2217),
    [sym___ldate__] = ACTIONS(2217),
    [sym___time__] = ACTIONS(2217),
    [sym___line__] = ACTIONS(2217),
    [sym___name__] = ACTIONS(2217),
    [sym___netlinx__] = ACTIONS(2217),
    [sym_netlinx_axi_version] = ACTIONS(2217),
    [sym_url_flg_tcp] = ACTIONS(2217),
    [sym_url_flg_accinfopresent] = ACTIONS(2217),
    [sym_url_flg_temp] = ACTIONS(2217),
    [sym_url_flg_stat_prgnetlinx] = ACTIONS(2217),
    [sym_url_flg_stat_mask] = ACTIONS(2217),
    [sym_url_flg_stat_lookup] = ACTIONS(2217),
    [sym_url_flg_stat_connecting] = ACTIONS(2217),
    [sym_url_flg_stat_waiting] = ACTIONS(2217),
    [sym_url_flg_stat_connected] = ACTIONS(2217),
    [sym_ip_addr_flg_dhcp] = ACTIONS(2217),
    [sym_first_virtual_device] = ACTIONS(2217),
    [sym_first_local_port] = ACTIONS(2217),
    [sym_dynamic_virtual_device] = ACTIONS(2217),
    [sym_timeline_once] = ACTIONS(2217),
    [sym_timeline_repeat] = ACTIONS(2217),
    [sym_timeline_absolute] = ACTIONS(2217),
    [sym_timeline_relative] = ACTIONS(2217),
    [sym_do_push_timed_infinite] = ACTIONS(2217),
    [sym_source_type_no_address] = ACTIONS(2217),
    [sym_source_type_neuron_id] = ACTIONS(2217),
    [sym_source_type_ip_address] = ACTIONS(2217),
    [sym_source_type_axlink] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_icsp] = ACTIONS(2217),
    [sym_source_type_neuron_subnode_pl] = ACTIONS(2217),
    [sym_source_type_ip_socket_address] = ACTIONS(2217),
    [sym_source_type_rs232] = ACTIONS(2217),
    [sym_source_type_internal] = ACTIONS(2217),
    [sym_source_type_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_address] = ACTIONS(2217),
    [sym_source_type_ipv4_port_mac_ipv6] = ACTIONS(2217),
    [sym_internal_queue_size_index_interpreter] = ACTIONS(2217),
    [sym_internal_queue_size_index_notification_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_connection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_route_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_device_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_diagnostic_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_tcp_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ipconnection_mgr] = ACTIONS(2217),
    [sym_internal_queue_size_index_message_dispatcher] = ACTIONS(2217),
    [sym_internal_queue_size_index_axlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_phastlink_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsplontalk_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icsp232_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_icspip_tx] = ACTIONS(2217),
    [sym_internal_queue_size_index_ni_device] = ACTIONS(2217),
    [sym_true] = ACTIONS(2217),
    [sym_false] = ACTIONS(2217),
    [sym_file_read_only] = ACTIONS(2217),
    [sym_file_rw_new] = ACTIONS(2217),
    [sym_file_rw_append] = ACTIONS(2217),
    [sym_ip_tcp] = ACTIONS(2217),
    [sym_ip_udp] = ACTIONS(2217),
    [sym_ip_udp_2way] = ACTIONS(2217),
    [sym_xml_encode_types] = ACTIONS(2217),
    [sym_xml_encode_char_as_list] = ACTIONS(2217),
    [sym_xml_encode_le] = ACTIONS(2217),
    [sym_xml_decode_types] = ACTIONS(2217),
    [sym_xml_decode_no_preserve] = ACTIONS(2217),
    [sym_duet_dev_type_display_device] = ACTIONS(2217),
    [sym_duet_dev_type_rfidsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_lightsystem_device] = ACTIONS(2217),
    [sym_duet_dev_type_io_device] = ACTIONS(2217),
    [sym_duet_dev_type_relay_device] = ACTIONS(2217),
    [sym_duet_dev_type_ups] = ACTIONS(2217),
    [sym_duet_dev_type_amplifier] = ACTIONS(2217),
    [sym_duet_dev_type_audio_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_mixer] = ACTIONS(2217),
    [sym_duet_dev_type_audio_processor] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tuner_device] = ACTIONS(2217),
    [sym_duet_dev_type_camera] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_encoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_decoder] = ACTIONS(2217),
    [sym_duet_dev_type_digital_media_server] = ACTIONS(2217),
    [sym_duet_dev_type_dss] = ACTIONS(2217),
    [sym_duet_dev_type_dvr] = ACTIONS(2217),
    [sym_duet_dev_type_disc_device] = ACTIONS(2217),
    [sym_duet_dev_type_document_camera] = ACTIONS(2217),
    [sym_duet_dev_type_audio_tape] = ACTIONS(2217),
    [sym_duet_dev_type_hvac] = ACTIONS(2217),
    [sym_duet_dev_type_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_light] = ACTIONS(2217),
    [sym_duet_dev_type_monitor] = ACTIONS(2217),
    [sym_duet_dev_type_motor] = ACTIONS(2217),
    [sym_duet_dev_type_multi_window] = ACTIONS(2217),
    [sym_duet_dev_type_pool_spa] = ACTIONS(2217),
    [sym_duet_dev_type_preamp_surround_sound_processor] = ACTIONS(2217),
    [sym_duet_dev_type_receiver] = ACTIONS(2217),
    [sym_duet_dev_type_security_system] = ACTIONS(2217),
    [sym_duet_dev_type_sensor_device] = ACTIONS(2217),
    [sym_duet_dev_type_settop_box] = ACTIONS(2217),
    [sym_duet_dev_type_slide_projector] = ACTIONS(2217),
    [sym_duet_dev_type_switcher] = ACTIONS(2217),
    [sym_duet_dev_type_text_keypad] = ACTIONS(2217),
    [sym_duet_dev_type_tv] = ACTIONS(2217),
    [sym_duet_dev_type_utility] = ACTIONS(2217),
    [sym_duet_dev_type_vcr] = ACTIONS(2217),
    [sym_duet_dev_type_video_conferencer] = ACTIONS(2217),
    [sym_duet_dev_type_video_processor] = ACTIONS(2217),
    [sym_duet_dev_type_video_projector] = ACTIONS(2217),
    [sym_duet_dev_type_video_wall] = ACTIONS(2217),
    [sym_duet_dev_type_volume_controller] = ACTIONS(2217),
    [sym_duet_dev_type_weather] = ACTIONS(2217),
    [sym_duet_dev_not_polled] = ACTIONS(2217),
    [sym_duet_dev_polled] = ACTIONS(2217),
    [sym_clkmgr_mode_network] = ACTIONS(2217),
    [sym_clkmgr_mode_standalone] = ACTIONS(2217),
    [sym_normal_standby] = ACTIONS(2217),
    [sym_normal_wake] = ACTIONS(2217),
    [sym_ip_multicast_ttl_option] = ACTIONS(2217),
    [sym_ip_ttl_subnet] = ACTIONS(2217),
    [sym_ip_ttl_site] = ACTIONS(2217),
    [sym_ip_ttl_region] = ACTIONS(2217),
    [sym_ip_ttl_continent] = ACTIONS(2217),
    [sym_ip_tcp_nodelay_option] = ACTIONS(2217),
    [sym_ip_nodelay_on] = ACTIONS(2217),
    [sym_ip_nodelay_off] = ACTIONS(2217),
    [sym_amx_error] = ACTIONS(2217),
    [sym_amx_warning] = ACTIONS(2217),
    [sym_amx_info] = ACTIONS(2217),
    [sym_amx_debug] = ACTIONS(2217),
    [sym_netlinx_login_success] = ACTIONS(2217),
    [sym_netlinx_login_fail] = ACTIONS(2217),
    [sym_netlinx_logout] = ACTIONS(2217),
    [sym_null_str] = ACTIONS(2217),
    [sym_smtp_address] = ACTIONS(2217),
    [sym_smtp_port_number] = ACTIONS(2217),
    [sym_smtp_username] = ACTIONS(2217),
    [sym_smtp_password] = ACTIONS(2217),
    [sym_smtp_require_tls] = ACTIONS(2217),
    [sym_smtp_from] = ACTIONS(2217),
    [sym_smtp_tls_true] = ACTIONS(2217),
    [sym_smtp_tls_false] = ACTIONS(2217),
    [sym_listview_on_row_select_event] = ACTIONS(2217),
    [sym_data_structure_unknown] = ACTIONS(2217),
    [sym_data_structure_datafeed] = ACTIONS(2217),
    [sym_data_structure_datarecordset] = ACTIONS(2217),
    [sym_data_structure_datarecord] = ACTIONS(2217),
    [sym_data_structure_datafield] = ACTIONS(2217),
    [sym_data_max_content_field_count] = ACTIONS(2217),
    [sym_data_max_metadata_field_count] = ACTIONS(2217),
    [sym_data_max_id_length] = ACTIONS(2217),
    [sym_data_max_name_length] = ACTIONS(2217),
    [sym_data_max_description_length] = ACTIONS(2217),
    [sym_data_max_type_length] = ACTIONS(2217),
    [sym_data_max_format_length] = ACTIONS(2217),
    [sym_data_max_source_length] = ACTIONS(2217),
    [sym_data_max_value_length] = ACTIONS(2217),
    [sym_data_max_label_length] = ACTIONS(2217),
    [sym_data_max_url_length] = ACTIONS(2217),
    [sym_max_last_login_info_length] = ACTIONS(2217),
    [sym_data_type_unknown] = ACTIONS(2217),
    [sym_data_type_string] = ACTIONS(2217),
    [sym_data_type_datetime] = ACTIONS(2217),
    [sym_data_type_date] = ACTIONS(2217),
    [sym_data_type_time] = ACTIONS(2217),
    [sym_data_type_image] = ACTIONS(2217),
    [sym_data_format_url] = ACTIONS(2217),
    [sym_data_format_phone] = ACTIONS(2217),
    [sym_data_format_email] = ACTIONS(2217),
    [sym_data_format_iso8601] = ACTIONS(2217),
    [sym_auditlog_priv] = ACTIONS(2217),
    [sym_configuration_priv] = ACTIONS(2217),
    [sym_deviceconfig_priv] = ACTIONS(2217),
    [sym_ftp_priv] = ACTIONS(2217),
    [sym_http_priv] = ACTIONS(2217),
    [sym_networkconfig_priv] = ACTIONS(2217),
    [sym_programport_priv] = ACTIONS(2217),
    [sym_remoteui_priv] = ACTIONS(2217),
    [sym_securitycontrol_priv] = ACTIONS(2217),
    [sym_softwaremanagement_priv] = ACTIONS(2217),
    [sym_terminal_priv] = ACTIONS(2217),
    [sym_tpadmin_priv] = ACTIONS(2217),
    [sym_user1_priv] = ACTIONS(2217),
    [sym_user2_priv] = ACTIONS(2217),
    [sym_user3_priv] = ACTIONS(2217),
    [sym_user4_priv] = ACTIONS(2217),
    [sym_usermanagement_priv] = ACTIONS(2217),
    [sym_valid_account] = ACTIONS(2217),
    [sym_err_name_invalid] = ACTIONS(2217),
    [sym_err_password_invalid] = ACTIONS(2217),
    [sym_err_invalid_account] = ACTIONS(2217),
    [sym_err_no_authorization] = ACTIONS(2217),
    [sym_err_invalid_parameter] = ACTIONS(2217),
    [sym_err_user_locked_out] = ACTIONS(2217),
    [sym_err_user_account_expired] = ACTIONS(2217),
    [sym_err_general_error] = ACTIONS(2217),
    [sym_err_authentication_error] = ACTIONS(2217),
    [sym_tls_ignore_certificate_errors] = ACTIONS(2217),
    [sym_tls_validate_certificate] = ACTIONS(2217),
    [sym____reserved___] = ACTIONS(2217),
    [sym_date] = ACTIONS(2217),
    [sym_day] = ACTIONS(2217),
    [sym_time] = ACTIONS(2217),
    [sym_ldate] = ACTIONS(2217),
    [sym_push_device] = ACTIONS(2217),
    [sym_push_channel] = ACTIONS(2217),
    [sym_push_devchan] = ACTIONS(2217),
    [sym_release_device] = ACTIONS(2217),
    [sym_release_channel] = ACTIONS(2217),
    [sym_release_devchan] = ACTIONS(2217),
    [sym_master_slot] = ACTIONS(2217),
    [sym_get_pulse_time] = ACTIONS(2217),
    [sym_get_timer] = ACTIONS(2217),
    [sym_master_sn] = ACTIONS(2217),
    [sym_system_number] = ACTIONS(2217),
    [sym_button] = ACTIONS(2217),
    [sym_dv_channel] = ACTIONS(2217),
    [sym_data] = ACTIONS(2217),
    [sym_level] = ACTIONS(2217),
    [sym_timeline] = ACTIONS(2217),
    [sym_channel] = ACTIONS(2217),
    [sym_custom] = ACTIONS(2217),
    [sym_atoi] = ACTIONS(2217),
    [sym_atol] = ACTIONS(2217),
    [sym_atof] = ACTIONS(2217),
    [sym_itoa] = ACTIONS(2217),
    [sym_itohex] = ACTIONS(2217),
    [sym_hextoi] = ACTIONS(2217),
    [sym_ftoa] = ACTIONS(2217),
    [sym_find_string] = ACTIONS(2217),
    [sym_left_string] = ACTIONS(2217),
    [sym_right_string] = ACTIONS(2217),
    [sym_mid_string] = ACTIONS(2217),
    [sym_remove_string] = ACTIONS(2217),
    [sym_get_buffer_string] = ACTIONS(2217),
    [sym_get_buffer_char] = ACTIONS(2217),
    [sym_get_multi_buffer_string] = ACTIONS(2217),
    [sym_length_string] = ACTIONS(2217),
    [sym_max_length_string] = ACTIONS(2217),
    [sym_set_length_string] = ACTIONS(2217),
    [sym_compare_string] = ACTIONS(2217),
    [sym_upper_string] = ACTIONS(2217),
    [sym_lower_string] = ACTIONS(2217),
    [sym_get_last] = ACTIONS(2217),
    [sym_date_to_day] = ACTIONS(2217),
    [sym_date_to_month] = ACTIONS(2217),
    [sym_date_to_year] = ACTIONS(2217),
    [sym_time_to_hour] = ACTIONS(2217),
    [sym_time_to_minute] = ACTIONS(2217),
    [sym_time_to_second] = ACTIONS(2217),
    [sym_day_of_week] = ACTIONS(2217),
    [sym_set_pulse_time] = ACTIONS(2217),
    [sym_set_timer] = ACTIONS(2217),
    [sym_file_open] = ACTIONS(2217),
    [sym_file_close] = ACTIONS(2217),
    [sym_file_read] = ACTIONS(2217),
    [sym_file_read_line] = ACTIONS(2217),
    [sym_file_write] = ACTIONS(2217),
    [sym_file_write_line] = ACTIONS(2217),
    [sym_file_copy] = ACTIONS(2217),
    [sym_file_delete] = ACTIONS(2217),
    [sym_file_rename] = ACTIONS(2217),
    [sym_file_dir] = ACTIONS(2217),
    [sym_file_getdir] = ACTIONS(2217),
    [sym_file_setdir] = ACTIONS(2217),
    [sym_file_create_dir] = ACTIONS(2217),
    [sym_file_remove_dir] = ACTIONS(2217),
    [sym_file_seek] = ACTIONS(2217),
    [sym_length_array] = ACTIONS(2217),
    [sym_max_length_array] = ACTIONS(2217),
    [sym_set_length_array] = ACTIONS(2217),
    [sym_ip_client_open] = ACTIONS(2217),
    [sym_ip_client_close] = ACTIONS(2217),
    [sym_ip_server_open] = ACTIONS(2217),
    [sym_ip_server_close] = ACTIONS(2217),
    [sym_ip_bound_client_open] = ACTIONS(2217),
    [sym_ip_mc_server_open] = ACTIONS(2217),
    [sym_ip_set_option] = ACTIONS(2217),
    [sym_ssh_client_open] = ACTIONS(2217),
    [sym_ssh_client_close] = ACTIONS(2217),
    [sym_tls_client_open] = ACTIONS(2217),
    [sym_tls_client_close] = ACTIONS(2217),
    [sym_do_push] = ACTIONS(2217),
    [sym_do_push_timed] = ACTIONS(2217),
    [sym_do_release] = ACTIONS(2217),
    [sym_device_id] = ACTIONS(2217),
    [sym_device_info] = ACTIONS(2217),
    [sym_device_id_string] = ACTIONS(2217),
    [sym_device_standby] = ACTIONS(2217),
    [sym_device_wake] = ACTIONS(2217),
    [sym_timeline_create] = ACTIONS(2217),
    [sym_timeline_kill] = ACTIONS(2217),
    [sym_timeline_reload] = ACTIONS(2217),
    [sym_timeline_pause] = ACTIONS(2217),
    [sym_timeline_restart] = ACTIONS(2217),
    [sym_timeline_set] = ACTIONS(2217),
    [sym_timeline_get] = ACTIONS(2217),
    [sym_timeline_active] = ACTIONS(2217),
    [sym_timeline_dynamic_id] = ACTIONS(2217),
    [sym_variable_to_string] = ACTIONS(2217),
    [sym_string_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_string] = ACTIONS(2217),
    [sym_type_cast] = ACTIONS(2217),
    [sym_variable_to_xml] = ACTIONS(2217),
    [sym_xml_to_variable] = ACTIONS(2217),
    [sym_length_variable_to_xml] = ACTIONS(2217),
    [sym_redirect_string] = ACTIONS(2217),
    [sym_random_number] = ACTIONS(2217),
    [sym_get_unique_id] = ACTIONS(2217),
    [sym_get_system_number] = ACTIONS(2217),
    [sym_set_system_number] = ACTIONS(2217),
    [sym_get_serial_number] = ACTIONS(2217),
    [sym_get_master_build] = ACTIONS(2217),
    [sym_set_validation_code] = ACTIONS(2217),
    [sym_reboot] = ACTIONS(2217),
    [sym_load_duet_module] = ACTIONS(2217),
    [sym_unload_duet_module] = ACTIONS(2217),
    [sym_duet_mem_size_set] = ACTIONS(2217),
    [sym_duet_mem_size_get] = ACTIONS(2217),
    [sym_dynamic_application_device] = ACTIONS(2217),
    [sym_dynamic_polled_port] = ACTIONS(2217),
    [sym_static_port_binding] = ACTIONS(2217),
    [sym_static_ip_binding] = ACTIONS(2217),
    [sym_get_dns_list] = ACTIONS(2217),
    [sym_set_dns_list] = ACTIONS(2217),
    [sym_get_ip_address] = ACTIONS(2217),
    [sym_set_ip_address] = ACTIONS(2217),
    [sym_get_url_list] = ACTIONS(2217),
    [sym_add_url_entry] = ACTIONS(2217),
    [sym_delete_url_entry] = ACTIONS(2217),
    [sym_abs_value] = ACTIONS(2217),
    [sym_max_value] = ACTIONS(2217),
    [sym_min_value] = ACTIONS(2217),
    [sym_exp_value] = ACTIONS(2217),
    [sym_log_value] = ACTIONS(2217),
    [sym_log10_value] = ACTIONS(2217),
    [sym_power_value] = ACTIONS(2217),
    [sym_sqrt_value] = ACTIONS(2217),
    [sym_raw_be] = ACTIONS(2217),
    [sym_raw_le] = ACTIONS(2217),
    [sym_internal_queue_size_set] = ACTIONS(2217),
    [sym_internal_queue_size_get] = ACTIONS(2217),
    [sym_internal_threshold_set] = ACTIONS(2217),
    [sym_internal_threshold_get] = ACTIONS(2217),
    [sym_rebuilt_event] = ACTIONS(2217),
    [sym_do_custom_event] = ACTIONS(2217),
    [sym_clkmgr_is_network_sourced] = ACTIONS(2217),
    [sym_clkmgr_set_clk_source] = ACTIONS(2217),
    [sym_clkmgr_is_daylightsavings_on] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_mode] = ACTIONS(2217),
    [sym_clkmgr_get_timezone] = ACTIONS(2217),
    [sym_clkmgr_set_timezone] = ACTIONS(2217),
    [sym_clkmgr_get_resync_period] = ACTIONS(2217),
    [sym_clkmgr_set_resync_period] = ACTIONS(2217),
    [sym_clkmgr_get_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_set_daylightsavings_offset] = ACTIONS(2217),
    [sym_clkmgr_get_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_set_active_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_timeservers] = ACTIONS(2217),
    [sym_clkmgr_add_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_delete_userdefined_timeserver] = ACTIONS(2217),
    [sym_clkmgr_get_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_start_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_get_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_clkmgr_set_end_daylightsavings_rule] = ACTIONS(2217),
    [sym_set_log_level] = ACTIONS(2217),
    [sym_get_log_level] = ACTIONS(2217),
    [sym_amx_log] = ACTIONS(2217),
    [sym_smtp_server_config_set] = ACTIONS(2217),
    [sym_smtp_server_config_get] = ACTIONS(2217),
    [sym_smtp_send] = ACTIONS(2217),
    [sym_validate_netlinx_account] = ACTIONS(2217),
    [sym_validate_netlinx_account_with_permission] = ACTIONS(2217),
    [sym_authenticate_certificate] = ACTIONS(2217),
    [sym_audit_netlinx_session_event] = ACTIONS(2217),
    [sym_audit_netlinx_generic_event] = ACTIONS(2217),
    [sym_get_available_flash_disk_space] = ACTIONS(2217),
    [sym_get_max_flash_disk_space] = ACTIONS(2217),
    [sym_data_create_feed] = ACTIONS(2217),
    [sym_data_delete_feed] = ACTIONS(2217),
    [sym_data_publish_feed] = ACTIONS(2217),
    [sym_data_get_published_feed] = ACTIONS(2217),
    [sym_data_add_record] = ACTIONS(2217),
    [sym_data_get_event_record] = ACTIONS(2217),
    [sym__wc_data_create_feed] = ACTIONS(2217),
    [sym__wc_data_add_record] = ACTIONS(2217),
    [sym__wc_data_get_event_record] = ACTIONS(2217),
    [sym_format] = ACTIONS(2217),
    [sym_astro_clock] = ACTIONS(2217),
    [sym_set_outdoor_temperature] = ACTIONS(2217),
    [sym_set_virtual_level_count] = ACTIONS(2217),
    [sym_set_virtual_channel_count] = ACTIONS(2217),
    [sym_set_virtual_port_count] = ACTIONS(2217),
    [sym__wc_encode_int] = ACTIONS(2217),
    [sym__wc_decode_int] = ACTIONS(2217),
    [sym__wc_int] = ACTIONS(2217),
    [sym__wc_to_ch_int] = ACTIONS(2217),
    [sym__ch_to_wc_int] = ACTIONS(2217),
    [sym__wc_find_string_int] = ACTIONS(2217),
    [sym__wc_left_string_int] = ACTIONS(2217),
    [sym__wc_lower_string_int] = ACTIONS(2217),
    [sym__wc_mid_string_int] = ACTIONS(2217),
    [sym__wc_remove_string_int] = ACTIONS(2217),
    [sym__wc_right_string_int] = ACTIONS(2217),
    [sym__wc_upper_string_int] = ACTIONS(2217),
    [sym__wc_compare_string_int] = ACTIONS(2217),
    [sym__wc_get_buffer_char_int] = ACTIONS(2217),
    [sym__wc_get_buffer_string_int] = ACTIONS(2217),
    [sym__wc_concat_string_int] = ACTIONS(2217),
    [sym__wc_file_open_int] = ACTIONS(2217),
    [sym__wc_file_close_int] = ACTIONS(2217),
    [sym__wc_file_read_int] = ACTIONS(2217),
    [sym__wc_file_read_line_int] = ACTIONS(2217),
    [sym__wc_file_write_int] = ACTIONS(2217),
    [sym__wc_file_write_line_int] = ACTIONS(2217),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2219),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2219),
    [anon_sym_LBRACK] = ACTIONS(2219),
    [anon_sym_LBRACE] = ACTIONS(2219),
    [anon_sym_RBRACE] = ACTIONS(2238),
    [anon_sym_LPAREN] = ACTIONS(2217),
    [anon_sym_BANG] = ACTIONS(2219),
    [anon_sym_TILDE] = ACTIONS(2219),
    [anon_sym_DASH] = ACTIONS(2217),
    [anon_sym_PLUS] = ACTIONS(2217),
    [anon_sym_DASH_DASH] = ACTIONS(2219),
    [anon_sym_PLUS_PLUS] = ACTIONS(2219),
    [anon_sym_DQUOTE] = ACTIONS(2219),
    [anon_sym_SQUOTE] = ACTIONS(2219),
    [sym_number_literal] = ACTIONS(2219),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(961)] = {
    [sym_identifier] = ACTIONS(1601),
    [sym_char_keyword] = ACTIONS(1601),
    [sym_widechar_keyword] = ACTIONS(1601),
    [sym_integer_keyword] = ACTIONS(1601),
    [sym_sinteger_keyword] = ACTIONS(1601),
    [sym_long_keyword] = ACTIONS(1601),
    [sym_slong_keyword] = ACTIONS(1601),
    [sym_float_keyword] = ACTIONS(1601),
    [sym_double_keyword] = ACTIONS(1601),
    [sym_variant_keyword] = ACTIONS(1601),
    [sym_variantarray_keyword] = ACTIONS(1601),
    [sym_local_var_keyword] = ACTIONS(1601),
    [sym_stack_var_keyword] = ACTIONS(1601),
    [sym_constant_keyword] = ACTIONS(1601),
    [sym_volatile_keyword] = ACTIONS(1601),
    [sym_non_volatile_keyword] = ACTIONS(1601),
    [sym_persistent_keyword] = ACTIONS(1601),
    [sym_dev_keyword] = ACTIONS(1601),
    [sym_devlev_keyword] = ACTIONS(1601),
    [sym_devchan_keyword] = ACTIONS(1601),
    [sym_and] = ACTIONS(1601),
    [sym_or] = ACTIONS(1601),
    [sym_xor] = ACTIONS(1601),
    [sym_band] = ACTIONS(1601),
    [sym_bor] = ACTIONS(1601),
    [sym_bxor] = ACTIONS(1601),
    [sym_lshift] = ACTIONS(1601),
    [sym_rshift] = ACTIONS(1601),
    [sym_tdata] = ACTIONS(1601),
    [sym_tchannel] = ACTIONS(1601),
    [sym_tlevel] = ACTIONS(1601),
    [sym_tbutton] = ACTIONS(1601),
    [sym_ttimeline] = ACTIONS(1601),
    [sym_tcustom] = ACTIONS(1601),
    [sym_url_struct] = ACTIONS(1601),
    [sym_dns_struct] = ACTIONS(1601),
    [sym_ip_address_struct] = ACTIONS(1601),
    [sym_dev_info_struct] = ACTIONS(1601),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1601),
    [sym_data_feed] = ACTIONS(1601),
    [sym_data_field] = ACTIONS(1601),
    [sym_data_record] = ACTIONS(1601),
    [sym_wc_data_feed] = ACTIONS(1601),
    [sym_wc_data_field] = ACTIONS(1601),
    [sym_wc_data_record] = ACTIONS(1601),
    [sym_last_login_info] = ACTIONS(1601),
    [anon_sym_COMMA] = ACTIONS(1603),
    [anon_sym_LBRACK] = ACTIONS(1603),
    [anon_sym_RBRACK] = ACTIONS(1603),
    [anon_sym_LBRACE] = ACTIONS(1603),
    [anon_sym_COLON] = ACTIONS(1603),
    [anon_sym_LPAREN] = ACTIONS(1601),
    [anon_sym_RPAREN] = ACTIONS(1603),
    [anon_sym_SEMI] = ACTIONS(1603),
    [anon_sym_DASH] = ACTIONS(1601),
    [anon_sym_PLUS] = ACTIONS(1601),
    [anon_sym_STAR] = ACTIONS(1603),
    [anon_sym_SLASH] = ACTIONS(1601),
    [anon_sym_PERCENT] = ACTIONS(1603),
    [anon_sym_PIPE_PIPE] = ACTIONS(1603),
    [anon_sym_AMP_AMP] = ACTIONS(1603),
    [anon_sym_PIPE] = ACTIONS(1601),
    [anon_sym_CARET] = ACTIONS(1603),
    [anon_sym_AMP] = ACTIONS(1601),
    [anon_sym_EQ_EQ] = ACTIONS(1603),
    [anon_sym_BANG_EQ] = ACTIONS(1603),
    [anon_sym_LT_GT] = ACTIONS(1603),
    [anon_sym_GT] = ACTIONS(1601),
    [anon_sym_GT_EQ] = ACTIONS(1603),
    [anon_sym_LT_EQ] = ACTIONS(1603),
    [anon_sym_LT] = ACTIONS(1601),
    [anon_sym_LT_LT] = ACTIONS(1603),
    [anon_sym_GT_GT] = ACTIONS(1603),
    [anon_sym_DASH_DASH] = ACTIONS(1603),
    [anon_sym_PLUS_PLUS] = ACTIONS(1603),
    [anon_sym_DOT] = ACTIONS(1603),
    [anon_sym_DQUOTE] = ACTIONS(1603),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(962)] = {
    [sym_identifier] = ACTIONS(1597),
    [sym_char_keyword] = ACTIONS(1597),
    [sym_widechar_keyword] = ACTIONS(1597),
    [sym_integer_keyword] = ACTIONS(1597),
    [sym_sinteger_keyword] = ACTIONS(1597),
    [sym_long_keyword] = ACTIONS(1597),
    [sym_slong_keyword] = ACTIONS(1597),
    [sym_float_keyword] = ACTIONS(1597),
    [sym_double_keyword] = ACTIONS(1597),
    [sym_variant_keyword] = ACTIONS(1597),
    [sym_variantarray_keyword] = ACTIONS(1597),
    [sym_local_var_keyword] = ACTIONS(1597),
    [sym_stack_var_keyword] = ACTIONS(1597),
    [sym_constant_keyword] = ACTIONS(1597),
    [sym_volatile_keyword] = ACTIONS(1597),
    [sym_non_volatile_keyword] = ACTIONS(1597),
    [sym_persistent_keyword] = ACTIONS(1597),
    [sym_dev_keyword] = ACTIONS(1597),
    [sym_devlev_keyword] = ACTIONS(1597),
    [sym_devchan_keyword] = ACTIONS(1597),
    [sym_and] = ACTIONS(1597),
    [sym_or] = ACTIONS(1597),
    [sym_xor] = ACTIONS(1597),
    [sym_band] = ACTIONS(1597),
    [sym_bor] = ACTIONS(1597),
    [sym_bxor] = ACTIONS(1597),
    [sym_lshift] = ACTIONS(1597),
    [sym_rshift] = ACTIONS(1597),
    [sym_tdata] = ACTIONS(1597),
    [sym_tchannel] = ACTIONS(1597),
    [sym_tlevel] = ACTIONS(1597),
    [sym_tbutton] = ACTIONS(1597),
    [sym_ttimeline] = ACTIONS(1597),
    [sym_tcustom] = ACTIONS(1597),
    [sym_url_struct] = ACTIONS(1597),
    [sym_dns_struct] = ACTIONS(1597),
    [sym_ip_address_struct] = ACTIONS(1597),
    [sym_dev_info_struct] = ACTIONS(1597),
    [sym_clkmgr_timeserver_struct] = ACTIONS(1597),
    [sym_data_feed] = ACTIONS(1597),
    [sym_data_field] = ACTIONS(1597),
    [sym_data_record] = ACTIONS(1597),
    [sym_wc_data_feed] = ACTIONS(1597),
    [sym_wc_data_field] = ACTIONS(1597),
    [sym_wc_data_record] = ACTIONS(1597),
    [sym_last_login_info] = ACTIONS(1597),
    [anon_sym_COMMA] = ACTIONS(1599),
    [anon_sym_LBRACK] = ACTIONS(1599),
    [anon_sym_RBRACK] = ACTIONS(1599),
    [anon_sym_LBRACE] = ACTIONS(1599),
    [anon_sym_COLON] = ACTIONS(1599),
    [anon_sym_LPAREN] = ACTIONS(1597),
    [anon_sym_RPAREN] = ACTIONS(1599),
    [anon_sym_SEMI] = ACTIONS(1599),
    [anon_sym_DASH] = ACTIONS(1597),
    [anon_sym_PLUS] = ACTIONS(1597),
    [anon_sym_STAR] = ACTIONS(1599),
    [anon_sym_SLASH] = ACTIONS(1597),
    [anon_sym_PERCENT] = ACTIONS(1599),
    [anon_sym_PIPE_PIPE] = ACTIONS(1599),
    [anon_sym_AMP_AMP] = ACTIONS(1599),
    [anon_sym_PIPE] = ACTIONS(1597),
    [anon_sym_CARET] = ACTIONS(1599),
    [anon_sym_AMP] = ACTIONS(1597),
    [anon_sym_EQ_EQ] = ACTIONS(1599),
    [anon_sym_BANG_EQ] = ACTIONS(1599),
    [anon_sym_LT_GT] = ACTIONS(1599),
    [anon_sym_GT] = ACTIONS(1597),
    [anon_sym_GT_EQ] = ACTIONS(1599),
    [anon_sym_LT_EQ] = ACTIONS(1599),
    [anon_sym_LT] = ACTIONS(1597),
    [anon_sym_LT_LT] = ACTIONS(1599),
    [anon_sym_GT_GT] = ACTIONS(1599),
    [anon_sym_DASH_DASH] = ACTIONS(1599),
    [anon_sym_PLUS_PLUS] = ACTIONS(1599),
    [anon_sym_DOT] = ACTIONS(1599),
    [anon_sym_DQUOTE] = ACTIONS(1599),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(963)] = {
    [sym_preproc_define] = STATE(966),
    [sym_preproc_warn] = STATE(966),
    [sym_preproc_disable_warning] = STATE(966),
    [sym_preproc_if_defined_in_field_declaration_list] = STATE(966),
    [sym_preproc_if_not_defined_in_field_declaration_list] = STATE(966),
    [sym_preproc_else_in_field_declaration_list] = STATE(2095),
    [sym_storage_class_specifier] = STATE(1007),
    [sym_type_qualifier] = STATE(1007),
    [sym_type_specifier] = STATE(1382),
    [sym__field_declaration_list_item] = STATE(966),
    [sym_field_declaration] = STATE(966),
    [sym_intrinsic_type] = STATE(1447),
    [sym_primitive_type] = STATE(1450),
    [sym_structured_type] = STATE(1450),
    [sym__declaration_modifiers] = STATE(1007),
    [sym__declaration_specifiers] = STATE(1459),
    [sym_system_type] = STATE(1447),
    [aux_sym_preproc_if_defined_in_field_declaration_list_repeat1] = STATE(966),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1007),
    [sym_identifier] = ACTIONS(2855),
    [sym_char_keyword] = ACTIONS(2857),
    [sym_widechar_keyword] = ACTIONS(2857),
    [sym_integer_keyword] = ACTIONS(2857),
    [sym_sinteger_keyword] = ACTIONS(2857),
    [sym_long_keyword] = ACTIONS(2857),
    [sym_slong_keyword] = ACTIONS(2857),
    [sym_float_keyword] = ACTIONS(2857),
    [sym_double_keyword] = ACTIONS(2857),
    [sym_variant_keyword] = ACTIONS(2857),
    [sym_variantarray_keyword] = ACTIONS(2857),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(2859),
    [sym_devlev_keyword] = ACTIONS(2859),
    [sym_devchan_keyword] = ACTIONS(2859),
    [sym_tdata] = ACTIONS(2861),
    [sym_tchannel] = ACTIONS(2861),
    [sym_tlevel] = ACTIONS(2861),
    [sym_tbutton] = ACTIONS(2861),
    [sym_ttimeline] = ACTIONS(2861),
    [sym_tcustom] = ACTIONS(2861),
    [sym_url_struct] = ACTIONS(2861),
    [sym_dns_struct] = ACTIONS(2861),
    [sym_ip_address_struct] = ACTIONS(2861),
    [sym_dev_info_struct] = ACTIONS(2861),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2861),
    [sym_data_feed] = ACTIONS(2861),
    [sym_data_field] = ACTIONS(2861),
    [sym_data_record] = ACTIONS(2861),
    [sym_wc_data_feed] = ACTIONS(2861),
    [sym_wc_data_field] = ACTIONS(2861),
    [sym_wc_data_record] = ACTIONS(2861),
    [sym_last_login_info] = ACTIONS(2861),
    [aux_sym_preproc_define_token1] = ACTIONS(2863),
    [aux_sym_preproc_warn_token1] = ACTIONS(2865),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2867),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2869),
    [anon_sym_LF] = ACTIONS(2871),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2873),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2875),
    [aux_sym_preproc_else_token1] = ACTIONS(2877),
    [sym_comment] = ACTIONS(287),
  },
  [STATE(964)] = {
    [sym_preproc_define] = STATE(967),
    [sym_preproc_warn] = STATE(967),
    [sym_preproc_disable_warning] = STATE(967),
    [sym_preproc_if_defined_in_field_declaration_list] = STATE(967),
    [sym_preproc_if_not_defined_in_field_declaration_list] = STATE(967),
    [sym_preproc_else_in_field_declaration_list] = STATE(2246),
    [sym_storage_class_specifier] = STATE(1007),
    [sym_type_qualifier] = STATE(1007),
    [sym_type_specifier] = STATE(1382),
    [sym__field_declaration_list_item] = STATE(967),
    [sym_field_declaration] = STATE(967),
    [sym_intrinsic_type] = STATE(1447),
    [sym_primitive_type] = STATE(1450),
    [sym_structured_type] = STATE(1450),
    [sym__declaration_modifiers] = STATE(1007),
    [sym__declaration_specifiers] = STATE(1459),
    [sym_system_type] = STATE(1447),
    [aux_sym_preproc_if_defined_in_field_declaration_list_repeat1] = STATE(967),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1007),
    [sym_identifier] = ACTIONS(2855),
    [sym_char_keyword] = ACTIONS(2857),
    [sym_widechar_keyword] = ACTIONS(2857),
    [sym_integer_keyword] = ACTIONS(2857),
    [sym_sinteger_keyword] = ACTIONS(2857),
    [sym_long_keyword] = ACTIONS(2857),
    [sym_slong_keyword] = ACTIONS(2857),
    [sym_float_keyword] = ACTIONS(2857),
    [sym_double_keyword] = ACTIONS(2857),
    [sym_variant_keyword] = ACTIONS(2857),
    [sym_variantarray_keyword] = ACTIONS(2857),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(2859),
    [sym_devlev_keyword] = ACTIONS(2859),
    [sym_devchan_keyword] = ACTIONS(2859),
    [sym_tdata] = ACTIONS(2861),
    [sym_tchannel] = ACTIONS(2861),
    [sym_tlevel] = ACTIONS(2861),
    [sym_tbutton] = ACTIONS(2861),
    [sym_ttimeline] = ACTIONS(2861),
    [sym_tcustom] = ACTIONS(2861),
    [sym_url_struct] = ACTIONS(2861),
    [sym_dns_struct] = ACTIONS(2861),
    [sym_ip_address_struct] = ACTIONS(2861),
    [sym_dev_info_struct] = ACTIONS(2861),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2861),
    [sym_data_feed] = ACTIONS(2861),
    [sym_data_field] = ACTIONS(2861),
    [sym_data_record] = ACTIONS(2861),
    [sym_wc_data_feed] = ACTIONS(2861),
    [sym_wc_data_field] = ACTIONS(2861),
    [sym_wc_data_record] = ACTIONS(2861),
    [sym_last_login_info] = ACTIONS(2861),
    [aux_sym_preproc_define_token1] = ACTIONS(2863),
    [aux_sym_preproc_warn_token1] = ACTIONS(2865),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2867),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2869),
    [anon_sym_LF] = ACTIONS(2879),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2881),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2875),
    [aux_sym_preproc_else_token1] = ACTIONS(2877),
    [sym_comment] = ACTIONS(287),
  },
  [STATE(965)] = {
    [sym_preproc_define] = STATE(969),
    [sym_preproc_warn] = STATE(969),
    [sym_preproc_disable_warning] = STATE(969),
    [sym_preproc_if_defined_in_field_declaration_list] = STATE(969),
    [sym_preproc_if_not_defined_in_field_declaration_list] = STATE(969),
    [sym_preproc_else_in_field_declaration_list] = STATE(2234),
    [sym_storage_class_specifier] = STATE(1007),
    [sym_type_qualifier] = STATE(1007),
    [sym_type_specifier] = STATE(1382),
    [sym__field_declaration_list_item] = STATE(969),
    [sym_field_declaration] = STATE(969),
    [sym_intrinsic_type] = STATE(1447),
    [sym_primitive_type] = STATE(1450),
    [sym_structured_type] = STATE(1450),
    [sym__declaration_modifiers] = STATE(1007),
    [sym__declaration_specifiers] = STATE(1459),
    [sym_system_type] = STATE(1447),
    [aux_sym_preproc_if_defined_in_field_declaration_list_repeat1] = STATE(969),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1007),
    [sym_identifier] = ACTIONS(2855),
    [sym_char_keyword] = ACTIONS(2857),
    [sym_widechar_keyword] = ACTIONS(2857),
    [sym_integer_keyword] = ACTIONS(2857),
    [sym_sinteger_keyword] = ACTIONS(2857),
    [sym_long_keyword] = ACTIONS(2857),
    [sym_slong_keyword] = ACTIONS(2857),
    [sym_float_keyword] = ACTIONS(2857),
    [sym_double_keyword] = ACTIONS(2857),
    [sym_variant_keyword] = ACTIONS(2857),
    [sym_variantarray_keyword] = ACTIONS(2857),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(2859),
    [sym_devlev_keyword] = ACTIONS(2859),
    [sym_devchan_keyword] = ACTIONS(2859),
    [sym_tdata] = ACTIONS(2861),
    [sym_tchannel] = ACTIONS(2861),
    [sym_tlevel] = ACTIONS(2861),
    [sym_tbutton] = ACTIONS(2861),
    [sym_ttimeline] = ACTIONS(2861),
    [sym_tcustom] = ACTIONS(2861),
    [sym_url_struct] = ACTIONS(2861),
    [sym_dns_struct] = ACTIONS(2861),
    [sym_ip_address_struct] = ACTIONS(2861),
    [sym_dev_info_struct] = ACTIONS(2861),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2861),
    [sym_data_feed] = ACTIONS(2861),
    [sym_data_field] = ACTIONS(2861),
    [sym_data_record] = ACTIONS(2861),
    [sym_wc_data_feed] = ACTIONS(2861),
    [sym_wc_data_field] = ACTIONS(2861),
    [sym_wc_data_record] = ACTIONS(2861),
    [sym_last_login_info] = ACTIONS(2861),
    [aux_sym_preproc_define_token1] = ACTIONS(2883),
    [aux_sym_preproc_warn_token1] = ACTIONS(2885),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2887),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2889),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2891),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2893),
    [aux_sym_preproc_else_token1] = ACTIONS(2895),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(966)] = {
    [sym_preproc_define] = STATE(971),
    [sym_preproc_warn] = STATE(971),
    [sym_preproc_disable_warning] = STATE(971),
    [sym_preproc_if_defined_in_field_declaration_list] = STATE(971),
    [sym_preproc_if_not_defined_in_field_declaration_list] = STATE(971),
    [sym_preproc_else_in_field_declaration_list] = STATE(2234),
    [sym_storage_class_specifier] = STATE(1007),
    [sym_type_qualifier] = STATE(1007),
    [sym_type_specifier] = STATE(1382),
    [sym__field_declaration_list_item] = STATE(971),
    [sym_field_declaration] = STATE(971),
    [sym_intrinsic_type] = STATE(1447),
    [sym_primitive_type] = STATE(1450),
    [sym_structured_type] = STATE(1450),
    [sym__declaration_modifiers] = STATE(1007),
    [sym__declaration_specifiers] = STATE(1459),
    [sym_system_type] = STATE(1447),
    [aux_sym_preproc_if_defined_in_field_declaration_list_repeat1] = STATE(971),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1007),
    [sym_identifier] = ACTIONS(2855),
    [sym_char_keyword] = ACTIONS(2857),
    [sym_widechar_keyword] = ACTIONS(2857),
    [sym_integer_keyword] = ACTIONS(2857),
    [sym_sinteger_keyword] = ACTIONS(2857),
    [sym_long_keyword] = ACTIONS(2857),
    [sym_slong_keyword] = ACTIONS(2857),
    [sym_float_keyword] = ACTIONS(2857),
    [sym_double_keyword] = ACTIONS(2857),
    [sym_variant_keyword] = ACTIONS(2857),
    [sym_variantarray_keyword] = ACTIONS(2857),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(2859),
    [sym_devlev_keyword] = ACTIONS(2859),
    [sym_devchan_keyword] = ACTIONS(2859),
    [sym_tdata] = ACTIONS(2861),
    [sym_tchannel] = ACTIONS(2861),
    [sym_tlevel] = ACTIONS(2861),
    [sym_tbutton] = ACTIONS(2861),
    [sym_ttimeline] = ACTIONS(2861),
    [sym_tcustom] = ACTIONS(2861),
    [sym_url_struct] = ACTIONS(2861),
    [sym_dns_struct] = ACTIONS(2861),
    [sym_ip_address_struct] = ACTIONS(2861),
    [sym_dev_info_struct] = ACTIONS(2861),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2861),
    [sym_data_feed] = ACTIONS(2861),
    [sym_data_field] = ACTIONS(2861),
    [sym_data_record] = ACTIONS(2861),
    [sym_wc_data_feed] = ACTIONS(2861),
    [sym_wc_data_field] = ACTIONS(2861),
    [sym_wc_data_record] = ACTIONS(2861),
    [sym_last_login_info] = ACTIONS(2861),
    [aux_sym_preproc_define_token1] = ACTIONS(2883),
    [aux_sym_preproc_warn_token1] = ACTIONS(2885),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2887),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2889),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2891),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2893),
    [aux_sym_preproc_else_token1] = ACTIONS(2895),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(967)] = {
    [sym_preproc_define] = STATE(971),
    [sym_preproc_warn] = STATE(971),
    [sym_preproc_disable_warning] = STATE(971),
    [sym_preproc_if_defined_in_field_declaration_list] = STATE(971),
    [sym_preproc_if_not_defined_in_field_declaration_list] = STATE(971),
    [sym_preproc_else_in_field_declaration_list] = STATE(2220),
    [sym_storage_class_specifier] = STATE(1007),
    [sym_type_qualifier] = STATE(1007),
    [sym_type_specifier] = STATE(1382),
    [sym__field_declaration_list_item] = STATE(971),
    [sym_field_declaration] = STATE(971),
    [sym_intrinsic_type] = STATE(1447),
    [sym_primitive_type] = STATE(1450),
    [sym_structured_type] = STATE(1450),
    [sym__declaration_modifiers] = STATE(1007),
    [sym__declaration_specifiers] = STATE(1459),
    [sym_system_type] = STATE(1447),
    [aux_sym_preproc_if_defined_in_field_declaration_list_repeat1] = STATE(971),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1007),
    [sym_identifier] = ACTIONS(2855),
    [sym_char_keyword] = ACTIONS(2857),
    [sym_widechar_keyword] = ACTIONS(2857),
    [sym_integer_keyword] = ACTIONS(2857),
    [sym_sinteger_keyword] = ACTIONS(2857),
    [sym_long_keyword] = ACTIONS(2857),
    [sym_slong_keyword] = ACTIONS(2857),
    [sym_float_keyword] = ACTIONS(2857),
    [sym_double_keyword] = ACTIONS(2857),
    [sym_variant_keyword] = ACTIONS(2857),
    [sym_variantarray_keyword] = ACTIONS(2857),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(2859),
    [sym_devlev_keyword] = ACTIONS(2859),
    [sym_devchan_keyword] = ACTIONS(2859),
    [sym_tdata] = ACTIONS(2861),
    [sym_tchannel] = ACTIONS(2861),
    [sym_tlevel] = ACTIONS(2861),
    [sym_tbutton] = ACTIONS(2861),
    [sym_ttimeline] = ACTIONS(2861),
    [sym_tcustom] = ACTIONS(2861),
    [sym_url_struct] = ACTIONS(2861),
    [sym_dns_struct] = ACTIONS(2861),
    [sym_ip_address_struct] = ACTIONS(2861),
    [sym_dev_info_struct] = ACTIONS(2861),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2861),
    [sym_data_feed] = ACTIONS(2861),
    [sym_data_field] = ACTIONS(2861),
    [sym_data_record] = ACTIONS(2861),
    [sym_wc_data_feed] = ACTIONS(2861),
    [sym_wc_data_field] = ACTIONS(2861),
    [sym_wc_data_record] = ACTIONS(2861),
    [sym_last_login_info] = ACTIONS(2861),
    [aux_sym_preproc_define_token1] = ACTIONS(2883),
    [aux_sym_preproc_warn_token1] = ACTIONS(2885),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2887),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2889),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2897),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2893),
    [aux_sym_preproc_else_token1] = ACTIONS(2895),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(968)] = {
    [sym_preproc_define] = STATE(970),
    [sym_preproc_warn] = STATE(970),
    [sym_preproc_disable_warning] = STATE(970),
    [sym_preproc_if_defined_in_field_declaration_list] = STATE(970),
    [sym_preproc_if_not_defined_in_field_declaration_list] = STATE(970),
    [sym_preproc_else_in_field_declaration_list] = STATE(2220),
    [sym_storage_class_specifier] = STATE(1007),
    [sym_type_qualifier] = STATE(1007),
    [sym_type_specifier] = STATE(1382),
    [sym__field_declaration_list_item] = STATE(970),
    [sym_field_declaration] = STATE(970),
    [sym_intrinsic_type] = STATE(1447),
    [sym_primitive_type] = STATE(1450),
    [sym_structured_type] = STATE(1450),
    [sym__declaration_modifiers] = STATE(1007),
    [sym__declaration_specifiers] = STATE(1459),
    [sym_system_type] = STATE(1447),
    [aux_sym_preproc_if_defined_in_field_declaration_list_repeat1] = STATE(970),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1007),
    [sym_identifier] = ACTIONS(2855),
    [sym_char_keyword] = ACTIONS(2857),
    [sym_widechar_keyword] = ACTIONS(2857),
    [sym_integer_keyword] = ACTIONS(2857),
    [sym_sinteger_keyword] = ACTIONS(2857),
    [sym_long_keyword] = ACTIONS(2857),
    [sym_slong_keyword] = ACTIONS(2857),
    [sym_float_keyword] = ACTIONS(2857),
    [sym_double_keyword] = ACTIONS(2857),
    [sym_variant_keyword] = ACTIONS(2857),
    [sym_variantarray_keyword] = ACTIONS(2857),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(2859),
    [sym_devlev_keyword] = ACTIONS(2859),
    [sym_devchan_keyword] = ACTIONS(2859),
    [sym_tdata] = ACTIONS(2861),
    [sym_tchannel] = ACTIONS(2861),
    [sym_tlevel] = ACTIONS(2861),
    [sym_tbutton] = ACTIONS(2861),
    [sym_ttimeline] = ACTIONS(2861),
    [sym_tcustom] = ACTIONS(2861),
    [sym_url_struct] = ACTIONS(2861),
    [sym_dns_struct] = ACTIONS(2861),
    [sym_ip_address_struct] = ACTIONS(2861),
    [sym_dev_info_struct] = ACTIONS(2861),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2861),
    [sym_data_feed] = ACTIONS(2861),
    [sym_data_field] = ACTIONS(2861),
    [sym_data_record] = ACTIONS(2861),
    [sym_wc_data_feed] = ACTIONS(2861),
    [sym_wc_data_field] = ACTIONS(2861),
    [sym_wc_data_record] = ACTIONS(2861),
    [sym_last_login_info] = ACTIONS(2861),
    [aux_sym_preproc_define_token1] = ACTIONS(2883),
    [aux_sym_preproc_warn_token1] = ACTIONS(2885),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2887),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2889),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2897),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2893),
    [aux_sym_preproc_else_token1] = ACTIONS(2895),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(969)] = {
    [sym_preproc_define] = STATE(971),
    [sym_preproc_warn] = STATE(971),
    [sym_preproc_disable_warning] = STATE(971),
    [sym_preproc_if_defined_in_field_declaration_list] = STATE(971),
    [sym_preproc_if_not_defined_in_field_declaration_list] = STATE(971),
    [sym_preproc_else_in_field_declaration_list] = STATE(2184),
    [sym_storage_class_specifier] = STATE(1007),
    [sym_type_qualifier] = STATE(1007),
    [sym_type_specifier] = STATE(1382),
    [sym__field_declaration_list_item] = STATE(971),
    [sym_field_declaration] = STATE(971),
    [sym_intrinsic_type] = STATE(1447),
    [sym_primitive_type] = STATE(1450),
    [sym_structured_type] = STATE(1450),
    [sym__declaration_modifiers] = STATE(1007),
    [sym__declaration_specifiers] = STATE(1459),
    [sym_system_type] = STATE(1447),
    [aux_sym_preproc_if_defined_in_field_declaration_list_repeat1] = STATE(971),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1007),
    [sym_identifier] = ACTIONS(2855),
    [sym_char_keyword] = ACTIONS(2857),
    [sym_widechar_keyword] = ACTIONS(2857),
    [sym_integer_keyword] = ACTIONS(2857),
    [sym_sinteger_keyword] = ACTIONS(2857),
    [sym_long_keyword] = ACTIONS(2857),
    [sym_slong_keyword] = ACTIONS(2857),
    [sym_float_keyword] = ACTIONS(2857),
    [sym_double_keyword] = ACTIONS(2857),
    [sym_variant_keyword] = ACTIONS(2857),
    [sym_variantarray_keyword] = ACTIONS(2857),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(2859),
    [sym_devlev_keyword] = ACTIONS(2859),
    [sym_devchan_keyword] = ACTIONS(2859),
    [sym_tdata] = ACTIONS(2861),
    [sym_tchannel] = ACTIONS(2861),
    [sym_tlevel] = ACTIONS(2861),
    [sym_tbutton] = ACTIONS(2861),
    [sym_ttimeline] = ACTIONS(2861),
    [sym_tcustom] = ACTIONS(2861),
    [sym_url_struct] = ACTIONS(2861),
    [sym_dns_struct] = ACTIONS(2861),
    [sym_ip_address_struct] = ACTIONS(2861),
    [sym_dev_info_struct] = ACTIONS(2861),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2861),
    [sym_data_feed] = ACTIONS(2861),
    [sym_data_field] = ACTIONS(2861),
    [sym_data_record] = ACTIONS(2861),
    [sym_wc_data_feed] = ACTIONS(2861),
    [sym_wc_data_field] = ACTIONS(2861),
    [sym_wc_data_record] = ACTIONS(2861),
    [sym_last_login_info] = ACTIONS(2861),
    [aux_sym_preproc_define_token1] = ACTIONS(2883),
    [aux_sym_preproc_warn_token1] = ACTIONS(2885),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2887),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2889),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2899),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2893),
    [aux_sym_preproc_else_token1] = ACTIONS(2895),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(970)] = {
    [sym_preproc_define] = STATE(971),
    [sym_preproc_warn] = STATE(971),
    [sym_preproc_disable_warning] = STATE(971),
    [sym_preproc_if_defined_in_field_declaration_list] = STATE(971),
    [sym_preproc_if_not_defined_in_field_declaration_list] = STATE(971),
    [sym_preproc_else_in_field_declaration_list] = STATE(2174),
    [sym_storage_class_specifier] = STATE(1007),
    [sym_type_qualifier] = STATE(1007),
    [sym_type_specifier] = STATE(1382),
    [sym__field_declaration_list_item] = STATE(971),
    [sym_field_declaration] = STATE(971),
    [sym_intrinsic_type] = STATE(1447),
    [sym_primitive_type] = STATE(1450),
    [sym_structured_type] = STATE(1450),
    [sym__declaration_modifiers] = STATE(1007),
    [sym__declaration_specifiers] = STATE(1459),
    [sym_system_type] = STATE(1447),
    [aux_sym_preproc_if_defined_in_field_declaration_list_repeat1] = STATE(971),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1007),
    [sym_identifier] = ACTIONS(2855),
    [sym_char_keyword] = ACTIONS(2857),
    [sym_widechar_keyword] = ACTIONS(2857),
    [sym_integer_keyword] = ACTIONS(2857),
    [sym_sinteger_keyword] = ACTIONS(2857),
    [sym_long_keyword] = ACTIONS(2857),
    [sym_slong_keyword] = ACTIONS(2857),
    [sym_float_keyword] = ACTIONS(2857),
    [sym_double_keyword] = ACTIONS(2857),
    [sym_variant_keyword] = ACTIONS(2857),
    [sym_variantarray_keyword] = ACTIONS(2857),
    [sym_local_var_keyword] = ACTIONS(49),
    [sym_stack_var_keyword] = ACTIONS(49),
    [sym_constant_keyword] = ACTIONS(51),
    [sym_volatile_keyword] = ACTIONS(51),
    [sym_non_volatile_keyword] = ACTIONS(51),
    [sym_persistent_keyword] = ACTIONS(51),
    [sym_dev_keyword] = ACTIONS(2859),
    [sym_devlev_keyword] = ACTIONS(2859),
    [sym_devchan_keyword] = ACTIONS(2859),
    [sym_tdata] = ACTIONS(2861),
    [sym_tchannel] = ACTIONS(2861),
    [sym_tlevel] = ACTIONS(2861),
    [sym_tbutton] = ACTIONS(2861),
    [sym_ttimeline] = ACTIONS(2861),
    [sym_tcustom] = ACTIONS(2861),
    [sym_url_struct] = ACTIONS(2861),
    [sym_dns_struct] = ACTIONS(2861),
    [sym_ip_address_struct] = ACTIONS(2861),
    [sym_dev_info_struct] = ACTIONS(2861),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2861),
    [sym_data_feed] = ACTIONS(2861),
    [sym_data_field] = ACTIONS(2861),
    [sym_data_record] = ACTIONS(2861),
    [sym_wc_data_feed] = ACTIONS(2861),
    [sym_wc_data_field] = ACTIONS(2861),
    [sym_wc_data_record] = ACTIONS(2861),
    [sym_last_login_info] = ACTIONS(2861),
    [aux_sym_preproc_define_token1] = ACTIONS(2883),
    [aux_sym_preproc_warn_token1] = ACTIONS(2885),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2887),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2889),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2901),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2893),
    [aux_sym_preproc_else_token1] = ACTIONS(2895),
    [sym_comment] = ACTIONS(3),
  },
  [STATE(971)] = {
    [sym_preproc_define] = STATE(971),
    [sym_preproc_warn] = STATE(971),
    [sym_preproc_disable_warning] = STATE(971),
    [sym_preproc_if_defined_in_field_declaration_list] = STATE(971),
    [sym_preproc_if_not_defined_in_field_declaration_list] = STATE(971),
    [sym_storage_class_specifier] = STATE(1007),
    [sym_type_qualifier] = STATE(1007),
    [sym_type_specifier] = STATE(1382),
    [sym__field_declaration_list_item] = STATE(971),
    [sym_field_declaration] = STATE(971),
    [sym_intrinsic_type] = STATE(1447),
    [sym_primitive_type] = STATE(1450),
    [sym_structured_type] = STATE(1450),
    [sym__declaration_modifiers] = STATE(1007),
    [sym__declaration_specifiers] = STATE(1459),
    [sym_system_type] = STATE(1447),
    [aux_sym_preproc_if_defined_in_field_declaration_list_repeat1] = STATE(971),
    [aux_sym__declaration_specifiers_repeat1] = STATE(1007),
    [sym_identifier] = ACTIONS(2903),
    [sym_char_keyword] = ACTIONS(2906),
    [sym_widechar_keyword] = ACTIONS(2906),
    [sym_integer_keyword] = ACTIONS(2906),
    [sym_sinteger_keyword] = ACTIONS(2906),
    [sym_long_keyword] = ACTIONS(2906),
    [sym_slong_keyword] = ACTIONS(2906),
    [sym_float_keyword] = ACTIONS(2906),
    [sym_double_keyword] = ACTIONS(2906),
    [sym_variant_keyword] = ACTIONS(2906),
    [sym_variantarray_keyword] = ACTIONS(2906),
    [sym_local_var_keyword] = ACTIONS(2909),
    [sym_stack_var_keyword] = ACTIONS(2909),
    [sym_constant_keyword] = ACTIONS(2912),
    [sym_volatile_keyword] = ACTIONS(2912),
    [sym_non_volatile_keyword] = ACTIONS(2912),
    [sym_persistent_keyword] = ACTIONS(2912),
    [sym_dev_keyword] = ACTIONS(2915),
    [sym_devlev_keyword] = ACTIONS(2915),
    [sym_devchan_keyword] = ACTIONS(2915),
    [sym_tdata] = ACTIONS(2918),
    [sym_tchannel] = ACTIONS(2918),
    [sym_tlevel] = ACTIONS(2918),
    [sym_tbutton] = ACTIONS(2918),
    [sym_ttimeline] = ACTIONS(2918),
    [sym_tcustom] = ACTIONS(2918),
    [sym_url_struct] = ACTIONS(2918),
    [sym_dns_struct] = ACTIONS(2918),
    [sym_ip_address_struct] = ACTIONS(2918),
    [sym_dev_info_struct] = ACTIONS(2918),
    [sym_clkmgr_timeserver_struct] = ACTIONS(2918),
    [sym_data_feed] = ACTIONS(2918),
    [sym_data_field] = ACTIONS(2918),
    [sym_data_record] = ACTIONS(2918),
    [sym_wc_data_feed] = ACTIONS(2918),
    [sym_wc_data_field] = ACTIONS(2918),
    [sym_wc_data_record] = ACTIONS(2918),
    [sym_last_login_info] = ACTIONS(2918),
    [aux_sym_preproc_define_token1] = ACTIONS(2921),
    [aux_sym_preproc_warn_token1] = ACTIONS(2924),
    [aux_sym_preproc_disable_warning_token1] = ACTIONS(2927),
    [aux_sym_preproc_if_defined_token1] = ACTIONS(2930),
    [aux_sym_preproc_if_defined_token2] = ACTIONS(2933),
    [aux_sym_preproc_if_not_defined_token1] = ACTIONS(2935),
    [aux_sym_preproc_else_token1] = ACTIONS(2933),
    [anon_sym_RBRACE] = ACTIONS(2933),
    [sym_comment] = ACTIONS(3),
  },
};

static const uint16_t ts_small_parse_table[] = {
  [0] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2855), 1,
      sym_identifier,
    ACTIONS(2883), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(2885), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(2887), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(2889), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(2893), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(2938), 1,
      anon_sym_RBRACE,
    STATE(1382), 1,
      sym_type_specifier,
    STATE(1459), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1447), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1450), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(2859), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1007), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(971), 8,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_field_declaration_list,
      sym_preproc_if_not_defined_in_field_declaration_list,
      sym__field_declaration_list_item,
      sym_field_declaration,
      aux_sym_preproc_if_defined_in_field_declaration_list_repeat1,
    ACTIONS(2857), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(2861), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [102] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2855), 1,
      sym_identifier,
    ACTIONS(2883), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(2885), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(2887), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(2889), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(2893), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(2940), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1382), 1,
      sym_type_specifier,
    STATE(1459), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1447), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1450), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(2859), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1007), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(971), 8,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_field_declaration_list,
      sym_preproc_if_not_defined_in_field_declaration_list,
      sym__field_declaration_list_item,
      sym_field_declaration,
      aux_sym_preproc_if_defined_in_field_declaration_list_repeat1,
    ACTIONS(2857), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(2861), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [204] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2855), 1,
      sym_identifier,
    ACTIONS(2883), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(2885), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(2887), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(2889), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(2893), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(2942), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1382), 1,
      sym_type_specifier,
    STATE(1459), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1447), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1450), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(2859), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1007), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(973), 8,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_field_declaration_list,
      sym_preproc_if_not_defined_in_field_declaration_list,
      sym__field_declaration_list_item,
      sym_field_declaration,
      aux_sym_preproc_if_defined_in_field_declaration_list_repeat1,
    ACTIONS(2857), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(2861), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [306] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2855), 1,
      sym_identifier,
    ACTIONS(2883), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(2885), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(2887), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(2889), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(2893), 1,
      aux_sym_preproc_if_not_defined_token1,
    STATE(1382), 1,
      sym_type_specifier,
    STATE(1459), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1447), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1450), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(2859), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1007), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(972), 8,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_field_declaration_list,
      sym_preproc_if_not_defined_in_field_declaration_list,
      sym__field_declaration_list_item,
      sym_field_declaration,
      aux_sym_preproc_if_defined_in_field_declaration_list_repeat1,
    ACTIONS(2857), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(2861), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [405] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1048), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    STATE(529), 1,
      sym_compound_statement,
    STATE(987), 1,
      sym_parameter_list,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(988), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [499] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    STATE(327), 1,
      sym_compound_statement,
    STATE(998), 1,
      sym_parameter_list,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(999), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [593] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    STATE(396), 1,
      sym_compound_statement,
    STATE(980), 1,
      sym_parameter_list,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(981), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [687] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    ACTIONS(2948), 1,
      anon_sym_RPAREN,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1431), 1,
      sym__declaration_specifiers,
    STATE(1815), 1,
      sym_parameter_declaration,
    STATE(2078), 1,
      sym_compound_statement,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1008), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [773] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(416), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(990), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [861] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(417), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1004), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [949] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(275), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1004), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [1037] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    ACTIONS(2950), 1,
      anon_sym_RPAREN,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1431), 1,
      sym__declaration_specifiers,
    STATE(1806), 1,
      sym_parameter_declaration,
    STATE(2117), 1,
      sym_compound_statement,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1008), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [1123] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(418), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(986), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [1211] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    ACTIONS(2952), 1,
      anon_sym_RPAREN,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1431), 1,
      sym__declaration_specifiers,
    STATE(1814), 1,
      sym_parameter_declaration,
    STATE(2164), 1,
      sym_compound_statement,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1008), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [1297] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(440), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1004), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [1385] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1048), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(536), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(991), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [1473] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1048), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(537), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1004), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [1561] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1048), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(538), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(992), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [1649] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(439), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1004), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [1737] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1048), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(569), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1004), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [1825] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1048), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(540), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1004), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [1913] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1048), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(541), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(995), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [2001] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(441), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(996), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [2089] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1048), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(526), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1004), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [2177] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(451), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1004), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [2265] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_RPAREN,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1431), 1,
      sym__declaration_specifiers,
    STATE(1819), 1,
      sym_parameter_declaration,
    STATE(2116), 1,
      sym_compound_statement,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1008), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [2351] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(379), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1001), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [2439] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(381), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1004), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [2527] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(391), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1002), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [2615] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(329), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1004), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [2703] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(335), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1004), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [2791] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(336), 1,
      sym_compound_statement,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(982), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [2879] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2956), 1,
      sym_identifier,
    ACTIONS(2974), 1,
      anon_sym_LBRACE,
    STATE(1019), 1,
      sym_storage_class_specifier,
    STATE(1020), 1,
      sym_type_qualifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1455), 1,
      sym__declaration_specifiers,
    ACTIONS(2962), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1004), 2,
      sym_declaration,
      aux_sym_function_definition_repeat1,
    STATE(1008), 2,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(2968), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(2965), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    ACTIONS(2959), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(2971), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [2964] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1507), 10,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
    ACTIONS(1509), 41,
      sym_button_event_keyword,
      sym_channel_event_keyword,
      sym_push_keyword,
      sym_release_keyword,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_LBRACE,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_range_operator,
      anon_sym_DQUOTE,
  [3023] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(1371), 1,
      sym_type_specifier,
    STATE(1431), 1,
      sym__declaration_specifiers,
    STATE(1963), 1,
      sym_parameter_declaration,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1008), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [3100] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2855), 1,
      sym_identifier,
    STATE(1374), 1,
      sym_type_specifier,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1447), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1450), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(2859), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1041), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(2857), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(2861), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [3171] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2944), 1,
      sym_identifier,
    STATE(1369), 1,
      sym_type_specifier,
    ACTIONS(49), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    STATE(1440), 2,
      sym_intrinsic_type,
      sym_system_type,
    STATE(1444), 2,
      sym_primitive_type,
      sym_structured_type,
    ACTIONS(53), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(51), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1041), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(135), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [3242] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2978), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(2976), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [3296] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2980), 1,
      sym_identifier,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    STATE(1799), 2,
      sym_init_declarator,
      sym__declaration_declarator,
    ACTIONS(2982), 37,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [3362] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2980), 1,
      sym_identifier,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    STATE(1800), 2,
      sym_init_declarator,
      sym__declaration_declarator,
    ACTIONS(2982), 37,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [3428] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1695), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(1697), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [3482] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(2986), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    STATE(1726), 2,
      sym_init_declarator,
      sym__declaration_declarator,
    ACTIONS(2982), 37,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [3548] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2990), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(2988), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [3602] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2994), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(2992), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [3656] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1707), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(1709), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [3710] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1703), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(1705), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [3764] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1711), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(1713), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [3818] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(2996), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    STATE(1788), 2,
      sym_init_declarator,
      sym__declaration_declarator,
    ACTIONS(2982), 37,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [3884] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(2996), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    STATE(1793), 2,
      sym_init_declarator,
      sym__declaration_declarator,
    ACTIONS(2982), 37,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [3950] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3000), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(2998), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [4004] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3004), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(3002), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [4058] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(3006), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    STATE(1734), 2,
      sym_init_declarator,
      sym__declaration_declarator,
    ACTIONS(2982), 37,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [4124] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(3006), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    STATE(1735), 2,
      sym_init_declarator,
      sym__declaration_declarator,
    ACTIONS(2982), 37,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [4190] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3010), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(3008), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [4244] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3014), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(3012), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [4298] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3018), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(3016), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [4352] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(2986), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    STATE(1804), 2,
      sym_init_declarator,
      sym__declaration_declarator,
    ACTIONS(2982), 37,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [4418] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2994), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(2992), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [4472] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3022), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(3020), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [4526] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3026), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(3024), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [4580] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3030), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(3028), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [4634] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3030), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(3028), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [4688] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3034), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(3032), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [4742] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3038), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(3036), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [4796] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3042), 8,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(3040), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [4850] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3046), 1,
      anon_sym_EQ,
    ACTIONS(3048), 1,
      anon_sym_LPAREN,
    ACTIONS(3044), 7,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_identifier,
    ACTIONS(1465), 15,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1469), 20,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [4908] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1923), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
    ACTIONS(1921), 39,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      anon_sym_LPAREN,
      sym_identifier,
  [4959] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1893), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
    ACTIONS(1891), 39,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      anon_sym_LPAREN,
      sym_identifier,
  [5010] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2135), 4,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
    ACTIONS(2133), 39,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      anon_sym_LPAREN,
      sym_identifier,
  [5061] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3053), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    ACTIONS(3056), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1041), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(3051), 32,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [5115] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3059), 1,
      anon_sym_LBRACK,
    STATE(1042), 1,
      aux_sym_subscript_expression_repeat1,
    ACTIONS(1496), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1498), 29,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [5167] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(1477), 7,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1479), 26,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DQUOTE,
  [5227] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(1413), 7,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1415), 26,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DQUOTE,
  [5287] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1042), 1,
      aux_sym_subscript_expression_repeat1,
    ACTIONS(1488), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1490), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [5337] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(1473), 7,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1475), 26,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DQUOTE,
  [5397] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1551), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1553), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [5444] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1783), 1,
      anon_sym_LBRACE,
    ACTIONS(1781), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [5491] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1783), 1,
      anon_sym_LBRACE,
    ACTIONS(1781), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [5538] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1795), 1,
      anon_sym_LBRACE,
    ACTIONS(1793), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [5585] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1413), 4,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1415), 23,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [5650] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3076), 39,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      anon_sym_LPAREN,
      sym_identifier,
  [5695] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1413), 6,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1415), 23,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [5758] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1775), 1,
      anon_sym_LBRACE,
    ACTIONS(1773), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [5805] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1791), 1,
      anon_sym_LBRACE,
    ACTIONS(1789), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [5852] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(1505), 4,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
      anon_sym_SEMI,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [5939] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1775), 1,
      anon_sym_LBRACE,
    ACTIONS(1773), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [5986] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3102), 1,
      sym_identifier,
    STATE(306), 1,
      sym_function_definition,
    STATE(2036), 1,
      sym_type_specifier,
    STATE(2235), 1,
      sym_array_return_type,
    STATE(1967), 2,
      sym_primitive_type,
      sym_structured_type,
    STATE(1981), 2,
      sym_intrinsic_type,
      sym_system_type,
    ACTIONS(3106), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(3104), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(3108), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [6047] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3110), 1,
      sym_identifier,
    STATE(306), 1,
      sym_function_declaration,
    STATE(2037), 1,
      sym_type_specifier,
    STATE(2237), 1,
      sym_array_return_type,
    STATE(1967), 2,
      sym_primitive_type,
      sym_structured_type,
    STATE(1981), 2,
      sym_intrinsic_type,
      sym_system_type,
    ACTIONS(3106), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(3104), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(3108), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [6108] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1779), 1,
      anon_sym_LBRACE,
    ACTIONS(1777), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [6155] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3112), 1,
      sym_identifier,
    STATE(402), 1,
      sym_function_definition,
    STATE(2047), 1,
      sym_type_specifier,
    STATE(2121), 1,
      sym_array_return_type,
    STATE(1967), 2,
      sym_primitive_type,
      sym_structured_type,
    STATE(1981), 2,
      sym_intrinsic_type,
      sym_system_type,
    ACTIONS(3106), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(3104), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(3108), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [6216] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3114), 39,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      anon_sym_LPAREN,
      sym_identifier,
  [6261] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1779), 1,
      anon_sym_LBRACE,
    ACTIONS(1777), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [6308] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1771), 1,
      anon_sym_LBRACE,
    ACTIONS(1769), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [6355] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(1413), 2,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 17,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
  [6426] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1555), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1557), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [6473] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1771), 1,
      anon_sym_LBRACE,
    ACTIONS(1769), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [6520] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1717), 1,
      anon_sym_LBRACE,
    ACTIONS(1715), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [6567] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1531), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1533), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [6614] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1413), 1,
      anon_sym_PIPE,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 13,
      sym_and,
      sym_or,
      sym_xor,
      sym_bor,
      sym_bxor,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
  [6691] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1771), 1,
      anon_sym_LBRACE,
    ACTIONS(1769), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [6738] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1795), 1,
      anon_sym_LBRACE,
    ACTIONS(1793), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [6785] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1791), 1,
      anon_sym_LBRACE,
    ACTIONS(1789), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [6832] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1795), 1,
      anon_sym_LBRACE,
    ACTIONS(1793), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [6879] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1775), 1,
      anon_sym_LBRACE,
    ACTIONS(1773), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [6926] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1543), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1545), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [6973] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1771), 1,
      anon_sym_LBRACE,
    ACTIONS(1769), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [7020] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1795), 1,
      anon_sym_LBRACE,
    ACTIONS(1793), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [7067] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1413), 1,
      anon_sym_PIPE,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 10,
      sym_and,
      sym_or,
      sym_bor,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [7146] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1779), 1,
      anon_sym_LBRACE,
    ACTIONS(1777), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [7193] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3116), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(1483), 4,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
      anon_sym_SEMI,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [7280] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1783), 1,
      anon_sym_LBRACE,
    ACTIONS(1781), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [7327] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1783), 1,
      anon_sym_LBRACE,
    ACTIONS(1781), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [7374] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3119), 1,
      sym_identifier,
    STATE(402), 1,
      sym_function_declaration,
    STATE(1936), 1,
      sym_type_specifier,
    STATE(2166), 1,
      sym_array_return_type,
    STATE(1967), 2,
      sym_primitive_type,
      sym_structured_type,
    STATE(1981), 2,
      sym_intrinsic_type,
      sym_system_type,
    ACTIONS(3106), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(3104), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(3108), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [7435] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 9,
      sym_and,
      sym_or,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [7516] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 7,
      sym_or,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
  [7599] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(1413), 2,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 14,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
  [7672] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1535), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1537), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [7719] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1767), 1,
      anon_sym_LBRACE,
    ACTIONS(1765), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [7766] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3121), 1,
      anon_sym_EQ,
    ACTIONS(3123), 1,
      sym_range_operator,
    ACTIONS(1465), 9,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
    ACTIONS(1469), 28,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
  [7817] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1511), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1513), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [7864] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1521), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1523), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [7911] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1547), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1549), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [7958] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(1494), 4,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RPAREN,
      anon_sym_SEMI,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [8045] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3125), 1,
      sym_identifier,
    STATE(548), 1,
      sym_function_definition,
    STATE(2029), 1,
      sym_type_specifier,
    STATE(2221), 1,
      sym_array_return_type,
    STATE(1967), 2,
      sym_primitive_type,
      sym_structured_type,
    STATE(1981), 2,
      sym_intrinsic_type,
      sym_system_type,
    ACTIONS(3106), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(3104), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(3108), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [8106] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3127), 1,
      sym_identifier,
    STATE(548), 1,
      sym_function_declaration,
    STATE(2030), 1,
      sym_type_specifier,
    STATE(2223), 1,
      sym_array_return_type,
    STATE(1967), 2,
      sym_primitive_type,
      sym_structured_type,
    STATE(1981), 2,
      sym_intrinsic_type,
      sym_system_type,
    ACTIONS(3106), 3,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
    ACTIONS(3104), 10,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
    ACTIONS(3108), 18,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
  [8167] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1413), 4,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 19,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [8234] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1775), 1,
      anon_sym_LBRACE,
    ACTIONS(1773), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [8281] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1539), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1541), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [8328] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1779), 1,
      anon_sym_LBRACE,
    ACTIONS(1777), 38,
      sym_char_keyword,
      sym_widechar_keyword,
      sym_integer_keyword,
      sym_sinteger_keyword,
      sym_long_keyword,
      sym_slong_keyword,
      sym_float_keyword,
      sym_double_keyword,
      sym_variant_keyword,
      sym_variantarray_keyword,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      sym_dev_keyword,
      sym_devlev_keyword,
      sym_devchan_keyword,
      sym_tdata,
      sym_tchannel,
      sym_tlevel,
      sym_tbutton,
      sym_ttimeline,
      sym_tcustom,
      sym_url_struct,
      sym_dns_struct,
      sym_ip_address_struct,
      sym_dev_info_struct,
      sym_clkmgr_timeserver_struct,
      sym_data_feed,
      sym_data_field,
      sym_data_record,
      sym_wc_data_feed,
      sym_wc_data_field,
      sym_wc_data_record,
      sym_last_login_info,
      sym_identifier,
  [8375] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3141), 1,
      anon_sym_COMMA,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3149), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [8463] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3121), 1,
      anon_sym_EQ,
    ACTIONS(1465), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1469), 29,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [8511] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1561), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1563), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [8557] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 8,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [8637] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 6,
      sym__automatic_semicolon,
      sym_or,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
  [8719] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1413), 1,
      anon_sym_PIPE,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 12,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_bor,
      sym_bxor,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
  [8795] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(1413), 2,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 13,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
  [8867] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1413), 1,
      anon_sym_PIPE,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 9,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_bor,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [8945] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(1413), 4,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1415), 22,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [9009] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3141), 1,
      anon_sym_COMMA,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3171), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [9097] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(3175), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    STATE(1859), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [9187] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1585), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1587), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [9233] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(1413), 6,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1415), 22,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [9295] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(1413), 7,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1415), 24,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [9353] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1492), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1494), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [9399] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(1413), 2,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 16,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
  [9469] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(1413), 4,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 18,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [9535] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3177), 1,
      anon_sym_LBRACK,
    STATE(1118), 1,
      aux_sym_subscript_expression_repeat1,
    ACTIONS(1496), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1498), 27,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [9585] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(1505), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [9671] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3171), 2,
      anon_sym_RPAREN,
      anon_sym_SEMI,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [9759] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1503), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1505), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [9805] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1573), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1575), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [9851] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3141), 1,
      anon_sym_COMMA,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    ACTIONS(3182), 1,
      anon_sym_COLON,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3184), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [9939] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3141), 1,
      anon_sym_COMMA,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    ACTIONS(3182), 1,
      anon_sym_COLON,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3186), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [10027] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(1473), 7,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1475), 24,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [10085] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1581), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1583), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [10131] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(1477), 7,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1479), 24,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [10189] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3196), 1,
      sym_bor,
    ACTIONS(3200), 1,
      anon_sym_COMMA,
    ACTIONS(3202), 1,
      anon_sym_COLON,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3210), 1,
      anon_sym_PIPE,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    ACTIONS(3220), 1,
      anon_sym_DQUOTE,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    STATE(1871), 1,
      aux_sym_string_expression_repeat1,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3188), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3190), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3192), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [10279] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3222), 1,
      anon_sym_EQ,
    ACTIONS(1465), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1469), 29,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [10327] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3224), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [10413] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3141), 1,
      anon_sym_COMMA,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    ACTIONS(3182), 1,
      anon_sym_COLON,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3226), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [10501] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1577), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1579), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [10547] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1589), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1591), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [10593] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1507), 10,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
    ACTIONS(1509), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_range_operator,
  [10639] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1118), 1,
      aux_sym_subscript_expression_repeat1,
    ACTIONS(1488), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1490), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [10687] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    ACTIONS(3228), 1,
      anon_sym_COLON,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(1483), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [10773] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3141), 1,
      anon_sym_COMMA,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3184), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [10861] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3046), 1,
      anon_sym_EQ,
    ACTIONS(3231), 1,
      sym_range_operator,
    ACTIONS(1465), 9,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
    ACTIONS(1469), 27,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
  [10911] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3141), 1,
      anon_sym_COMMA,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3186), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [10999] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3141), 1,
      anon_sym_COMMA,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3233), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [11087] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3196), 1,
      sym_bor,
    ACTIONS(3200), 1,
      anon_sym_COMMA,
    ACTIONS(3202), 1,
      anon_sym_COLON,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3210), 1,
      anon_sym_PIPE,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    ACTIONS(3235), 1,
      anon_sym_DQUOTE,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    STATE(1876), 1,
      aux_sym_string_expression_repeat1,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3188), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3190), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3192), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [11177] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(3237), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    STATE(1865), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [11267] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3141), 1,
      anon_sym_COMMA,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3226), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [11355] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3141), 1,
      anon_sym_COMMA,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3239), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [11443] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3196), 1,
      sym_bor,
    ACTIONS(3200), 1,
      anon_sym_COMMA,
    ACTIONS(3202), 1,
      anon_sym_COLON,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3210), 1,
      anon_sym_PIPE,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    ACTIONS(3241), 1,
      anon_sym_DQUOTE,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    STATE(1824), 1,
      aux_sym_string_expression_repeat1,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3188), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3190), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3192), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [11533] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(3243), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    STATE(1830), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [11623] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3196), 1,
      sym_bor,
    ACTIONS(3200), 1,
      anon_sym_COMMA,
    ACTIONS(3202), 1,
      anon_sym_COLON,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3210), 1,
      anon_sym_PIPE,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    ACTIONS(3245), 1,
      anon_sym_DQUOTE,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    STATE(1855), 1,
      aux_sym_string_expression_repeat1,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3188), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3190), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3192), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [11713] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(3247), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    STATE(1821), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [11803] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3196), 1,
      sym_bor,
    ACTIONS(3200), 1,
      anon_sym_COMMA,
    ACTIONS(3202), 1,
      anon_sym_COLON,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3210), 1,
      anon_sym_PIPE,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    ACTIONS(3249), 1,
      anon_sym_DQUOTE,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    STATE(1836), 1,
      aux_sym_string_expression_repeat1,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3188), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3190), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3192), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [11893] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(3251), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    STATE(1848), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [11983] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3196), 1,
      sym_bor,
    ACTIONS(3200), 1,
      anon_sym_COMMA,
    ACTIONS(3202), 1,
      anon_sym_COLON,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3210), 1,
      anon_sym_PIPE,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    ACTIONS(3253), 1,
      anon_sym_DQUOTE,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    STATE(1868), 1,
      aux_sym_string_expression_repeat1,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3188), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3190), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3192), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [12073] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(3255), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    STATE(1807), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [12163] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(1494), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [12249] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1465), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1469), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [12295] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1565), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1567), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [12341] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1593), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1595), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [12387] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1569), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1571), 30,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [12433] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1413), 1,
      anon_sym_PIPE,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3192), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 8,
      sym_and,
      sym_or,
      sym_bor,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_DQUOTE,
  [12510] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3257), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [12597] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3259), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [12684] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3261), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [12769] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3263), 1,
      anon_sym_EQ,
    ACTIONS(1465), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1469), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [12816] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3046), 1,
      anon_sym_EQ,
    ACTIONS(1465), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1469), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [12863] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3196), 1,
      sym_bor,
    ACTIONS(3202), 1,
      anon_sym_COLON,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3210), 1,
      anon_sym_PIPE,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3188), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3190), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3265), 2,
      anon_sym_COMMA,
      anon_sym_DQUOTE,
    ACTIONS(3192), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [12948] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3267), 1,
      anon_sym_SEMI,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [13035] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3269), 1,
      anon_sym_COMMA,
    ACTIONS(3271), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [13122] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3269), 1,
      anon_sym_COMMA,
    ACTIONS(3273), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [13209] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3275), 1,
      anon_sym_COMMA,
    ACTIONS(3277), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [13296] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3279), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [13381] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3281), 1,
      anon_sym_SEMI,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [13468] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1521), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1523), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [13513] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3283), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [13598] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3285), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [13685] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1535), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1537), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [13730] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1531), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1533), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [13775] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3287), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [13860] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3289), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [13945] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3291), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [14030] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3293), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [14115] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3295), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [14200] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1543), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1545), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [14245] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1547), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1549), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [14290] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3297), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [14375] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3299), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [14462] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3301), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [14547] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3303), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [14634] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3305), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [14719] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3307), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [14804] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3309), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [14889] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3311), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [14974] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3313), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [15059] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3269), 1,
      anon_sym_COMMA,
    ACTIONS(3315), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [15146] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3317), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [15233] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3319), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [15318] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3321), 1,
      anon_sym_SEMI,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [15405] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3269), 1,
      anon_sym_COMMA,
    ACTIONS(3323), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [15492] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3325), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [15577] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3327), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [15664] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3329), 1,
      anon_sym_COMMA,
    ACTIONS(3331), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [15751] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3135), 1,
      sym_band,
    ACTIONS(3137), 1,
      sym_bor,
    ACTIONS(3143), 1,
      anon_sym_LBRACK,
    ACTIONS(3145), 1,
      anon_sym_COLON,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    ACTIONS(3155), 1,
      anon_sym_SLASH,
    ACTIONS(3157), 1,
      anon_sym_PIPE,
    ACTIONS(3159), 1,
      anon_sym_AMP,
    ACTIONS(3169), 1,
      anon_sym_DOT,
    STATE(1135), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1203), 1,
      sym_argument_list,
    ACTIONS(3129), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3131), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3151), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3153), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3163), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3165), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3167), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3333), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3133), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3161), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3139), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [15836] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3335), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [15923] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3337), 1,
      anon_sym_COMMA,
    ACTIONS(3339), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [16010] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1511), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1513), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [16055] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3341), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [16142] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3196), 1,
      sym_bor,
    ACTIONS(3202), 1,
      anon_sym_COLON,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3210), 1,
      anon_sym_PIPE,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(1494), 2,
      anon_sym_COMMA,
      anon_sym_DQUOTE,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3188), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3190), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3192), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [16227] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3196), 1,
      sym_bor,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3210), 1,
      anon_sym_PIPE,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3192), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 7,
      sym_and,
      sym_or,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_DQUOTE,
  [16306] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3196), 1,
      sym_bor,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3210), 1,
      anon_sym_PIPE,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3188), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3192), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 5,
      sym_or,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PIPE_PIPE,
      anon_sym_DQUOTE,
  [16387] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1413), 1,
      anon_sym_PIPE,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 11,
      sym_and,
      sym_or,
      sym_xor,
      sym_bor,
      sym_bxor,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_DQUOTE,
  [16462] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(1413), 2,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 12,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_DQUOTE,
  [16533] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3343), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [16620] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1413), 4,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1415), 21,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DQUOTE,
  [16683] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1413), 6,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1415), 21,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DQUOTE,
  [16744] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(1413), 2,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 15,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_DQUOTE,
  [16813] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(1413), 4,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1415), 17,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_DQUOTE,
  [16878] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3196), 1,
      sym_bor,
    ACTIONS(3202), 1,
      anon_sym_COLON,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3210), 1,
      anon_sym_PIPE,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(1505), 2,
      anon_sym_COMMA,
      anon_sym_DQUOTE,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3188), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3190), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3192), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [16963] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3194), 1,
      sym_band,
    ACTIONS(3196), 1,
      sym_bor,
    ACTIONS(3208), 1,
      anon_sym_SLASH,
    ACTIONS(3210), 1,
      anon_sym_PIPE,
    ACTIONS(3212), 1,
      anon_sym_AMP,
    ACTIONS(3345), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(1483), 2,
      anon_sym_COMMA,
      anon_sym_DQUOTE,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3188), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3190), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3204), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3206), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3216), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3218), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3192), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3214), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3198), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [17048] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1555), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1557), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [17093] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3123), 1,
      sym_range_operator,
    ACTIONS(3348), 1,
      anon_sym_EQ,
    ACTIONS(1465), 9,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_DOT,
    ACTIONS(1469), 26,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DQUOTE,
  [17142] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1539), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1541), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [17187] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3350), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [17272] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1551), 9,
      anon_sym_EQ,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1553), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [17317] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3180), 1,
      anon_sym_COMMA,
    ACTIONS(3352), 1,
      anon_sym_RPAREN,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [17404] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3354), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [17488] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3356), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [17572] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3358), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [17656] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1492), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1494), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [17700] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3360), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [17784] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1561), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1563), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [17828] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3362), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [17912] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1593), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1595), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [17956] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3364), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [18040] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3366), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [18124] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1577), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1579), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [18168] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1569), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1571), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [18212] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3368), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [18296] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3370), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [18380] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3372), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [18464] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3374), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [18548] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3376), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [18632] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1597), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1599), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [18676] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [18760] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3380), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [18844] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3382), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [18928] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3384), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [19012] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1581), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1583), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [19056] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3386), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [19140] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3388), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [19224] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3390), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [19308] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3392), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [19392] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3394), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [19476] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3396), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [19560] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3398), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [19644] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3400), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [19728] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3402), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [19812] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3404), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [19896] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3406), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [19980] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3408), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [20064] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3410), 1,
      anon_sym_RBRACK,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [20148] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1503), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1505), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [20192] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1573), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1575), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [20236] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1565), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1567), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [20280] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1465), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1469), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [20324] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3412), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [20408] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3414), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [20492] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3416), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [20576] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3269), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [20660] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3418), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [20744] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3420), 1,
      anon_sym_EQ,
    ACTIONS(1465), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1469), 27,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [20790] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3348), 1,
      anon_sym_EQ,
    ACTIONS(1465), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1469), 27,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DQUOTE,
  [20836] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1585), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1587), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [20880] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3422), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [20964] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3424), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [21048] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3426), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [21132] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3428), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [21216] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3430), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [21300] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3432), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [21384] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3434), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [21468] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1601), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1603), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [21512] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1589), 8,
      anon_sym_LPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(1591), 28,
      sym__automatic_semicolon,
      sym_and,
      sym_or,
      sym_xor,
      sym_band,
      sym_bor,
      sym_bxor,
      sym_lshift,
      sym_rshift,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_SEMI,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
  [21556] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3436), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [21640] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3141), 1,
      anon_sym_COMMA,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [21724] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3438), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [21805] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3441), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [21886] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3444), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [21967] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3446), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [22048] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3448), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [22129] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3450), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [22210] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3453), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [22291] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3455), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [22372] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3458), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [22453] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3460), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [22534] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3463), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [22615] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3465), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [22696] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3467), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [22777] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3470), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [22858] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3472), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [22939] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_LBRACK,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    ACTIONS(3068), 1,
      anon_sym_DOT,
    ACTIONS(3074), 1,
      anon_sym_SLASH,
    ACTIONS(3084), 1,
      sym_band,
    ACTIONS(3086), 1,
      sym_bor,
    ACTIONS(3092), 1,
      anon_sym_PIPE,
    ACTIONS(3094), 1,
      anon_sym_AMP,
    ACTIONS(3474), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      aux_sym_subscript_expression_repeat1,
    STATE(1091), 1,
      sym_argument_list,
    ACTIONS(3066), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3070), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3072), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3078), 2,
      sym_and,
      anon_sym_AMP_AMP,
    ACTIONS(3080), 2,
      sym_or,
      anon_sym_PIPE_PIPE,
    ACTIONS(3098), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3100), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3082), 3,
      sym_xor,
      sym_bxor,
      anon_sym_CARET,
    ACTIONS(3096), 3,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_LT_GT,
    ACTIONS(3088), 4,
      sym_lshift,
      sym_rshift,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [23020] = 15,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(3478), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3480), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3482), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3484), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3486), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3488), 1,
      anon_sym_LF,
    ACTIONS(3490), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(3492), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3494), 1,
      aux_sym_preproc_else_token1,
    STATE(1304), 1,
      aux_sym_preproc_if_defined_in_data_event_block_repeat2,
    STATE(2071), 1,
      sym_data_event_type,
    STATE(2195), 1,
      sym_preproc_else_in_data_event_block,
    ACTIONS(3476), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1312), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [23080] = 15,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(3478), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3480), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3482), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3484), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3486), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3492), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3494), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3496), 1,
      anon_sym_LF,
    ACTIONS(3498), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1300), 1,
      aux_sym_preproc_if_defined_in_data_event_block_repeat2,
    STATE(2071), 1,
      sym_data_event_type,
    STATE(2086), 1,
      sym_preproc_else_in_data_event_block,
    ACTIONS(3476), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1312), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [23140] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3510), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3512), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(3514), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3516), 1,
      aux_sym_preproc_else_token1,
    STATE(1307), 1,
      aux_sym_preproc_if_defined_in_data_event_block_repeat2,
    STATE(2071), 1,
      sym_data_event_type,
    STATE(2119), 1,
      sym_preproc_else_in_data_event_block,
    ACTIONS(3500), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1312), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [23197] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3510), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3514), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3516), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3518), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1302), 1,
      aux_sym_preproc_if_defined_in_data_event_block_repeat2,
    STATE(2071), 1,
      sym_data_event_type,
    STATE(2098), 1,
      sym_preproc_else_in_data_event_block,
    ACTIONS(3500), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1312), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [23254] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3510), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3514), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3516), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3520), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1307), 1,
      aux_sym_preproc_if_defined_in_data_event_block_repeat2,
    STATE(2065), 1,
      sym_preproc_else_in_data_event_block,
    STATE(2071), 1,
      sym_data_event_type,
    ACTIONS(3500), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1312), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [23311] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3510), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3514), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3516), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3522), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1307), 1,
      aux_sym_preproc_if_defined_in_data_event_block_repeat2,
    STATE(2071), 1,
      sym_data_event_type,
    STATE(2073), 1,
      sym_preproc_else_in_data_event_block,
    ACTIONS(3500), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1312), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [23368] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3510), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3514), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3516), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3518), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1307), 1,
      aux_sym_preproc_if_defined_in_data_event_block_repeat2,
    STATE(2071), 1,
      sym_data_event_type,
    STATE(2098), 1,
      sym_preproc_else_in_data_event_block,
    ACTIONS(3500), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1312), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [23425] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3510), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3512), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(3514), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3516), 1,
      aux_sym_preproc_else_token1,
    STATE(1303), 1,
      aux_sym_preproc_if_defined_in_data_event_block_repeat2,
    STATE(2071), 1,
      sym_data_event_type,
    STATE(2119), 1,
      sym_preproc_else_in_data_event_block,
    ACTIONS(3500), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1312), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [23482] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3527), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3530), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3533), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3536), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3539), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3544), 1,
      aux_sym_preproc_if_not_defined_token1,
    STATE(2071), 1,
      sym_data_event_type,
    ACTIONS(3542), 3,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    ACTIONS(3524), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1306), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [23532] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3550), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3553), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3556), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3559), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3562), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3567), 1,
      aux_sym_preproc_if_not_defined_token1,
    STATE(1307), 1,
      aux_sym_preproc_if_defined_in_data_event_block_repeat2,
    STATE(2071), 1,
      sym_data_event_type,
    ACTIONS(3565), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    ACTIONS(3547), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1312), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [23584] = 17,
    ACTIONS(141), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(3570), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3572), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3574), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3576), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3578), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3580), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3582), 1,
      anon_sym_LF,
    ACTIONS(3584), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(3586), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3588), 1,
      aux_sym_preproc_else_token1,
    STATE(1322), 1,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat2,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2079), 1,
      sym_preproc_else_in_channel_event_block,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1334), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [23645] = 17,
    ACTIONS(141), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(3570), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3572), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3574), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3576), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3578), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3580), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3586), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3588), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3590), 1,
      anon_sym_LF,
    ACTIONS(3592), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1319), 1,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat2,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2243), 1,
      sym_preproc_else_in_channel_event_block,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1334), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [23706] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3510), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3514), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3594), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1311), 1,
      aux_sym_preproc_if_defined_in_data_event_block_repeat2,
    STATE(2071), 1,
      sym_data_event_type,
    ACTIONS(3500), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1312), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [23757] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3510), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3514), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3596), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1307), 1,
      aux_sym_preproc_if_defined_in_data_event_block_repeat2,
    STATE(2071), 1,
      sym_data_event_type,
    ACTIONS(3500), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1312), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [23808] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3601), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3604), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3607), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3610), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3613), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3618), 1,
      aux_sym_preproc_if_not_defined_token1,
    STATE(2071), 1,
      sym_data_event_type,
    ACTIONS(3616), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    ACTIONS(3598), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1306), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [23857] = 16,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(3478), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3480), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3482), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3484), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3623), 1,
      sym_hold_keyword,
    ACTIONS(3625), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3627), 1,
      anon_sym_LF,
    ACTIONS(3629), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(3631), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3633), 1,
      aux_sym_preproc_else_token1,
    STATE(1328), 1,
      aux_sym_preproc_if_defined_in_button_event_block_repeat2,
    STATE(2076), 1,
      sym_button_event_type,
    STATE(2125), 1,
      sym_preproc_else_in_button_event_block,
    ACTIONS(3621), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1342), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [23915] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(145), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(147), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3635), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3637), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3641), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3643), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(3645), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3647), 1,
      aux_sym_preproc_else_token1,
    STATE(1325), 1,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat2,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2132), 1,
      sym_preproc_else_in_channel_event_block,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1334), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [23973] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3510), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3514), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3649), 1,
      anon_sym_RBRACE,
    STATE(2071), 1,
      sym_data_event_type,
    ACTIONS(3500), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1306), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [24021] = 16,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(3478), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3480), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3482), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3484), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3623), 1,
      sym_hold_keyword,
    ACTIONS(3625), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3631), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3633), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3651), 1,
      anon_sym_LF,
    ACTIONS(3653), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1330), 1,
      aux_sym_preproc_if_defined_in_button_event_block_repeat2,
    STATE(2076), 1,
      sym_button_event_type,
    STATE(2109), 1,
      sym_preproc_else_in_button_event_block,
    ACTIONS(3621), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1342), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [24079] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(145), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(147), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3635), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3637), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3641), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3645), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3647), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3655), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1324), 1,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat2,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2217), 1,
      sym_preproc_else_in_channel_event_block,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1334), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [24137] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3510), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3514), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3657), 1,
      anon_sym_RBRACE,
    STATE(2071), 1,
      sym_data_event_type,
    ACTIONS(3500), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1320), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [24185] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(145), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(147), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3635), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3637), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3641), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3645), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3647), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3655), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1325), 1,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat2,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2217), 1,
      sym_preproc_else_in_channel_event_block,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1334), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [24243] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3510), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3514), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3659), 1,
      anon_sym_RBRACE,
    STATE(2071), 1,
      sym_data_event_type,
    ACTIONS(3500), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1306), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [24291] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3510), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3514), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3661), 1,
      anon_sym_RBRACE,
    STATE(2071), 1,
      sym_data_event_type,
    ACTIONS(3500), 7,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
    STATE(1315), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_data_event_block,
      sym_preproc_if_not_defined_in_data_event_block,
      sym__data_event_block_item,
      sym_data_event_handler,
      aux_sym_preproc_if_defined_in_data_event_block_repeat1,
  [24339] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(145), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(147), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3635), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3637), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3641), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3645), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3647), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3663), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1325), 1,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat2,
    STATE(2060), 1,
      sym_preproc_else_in_channel_event_block,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1334), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [24397] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(145), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(147), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3635), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3637), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3641), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3645), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3647), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3663), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1314), 1,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat2,
    STATE(2060), 1,
      sym_preproc_else_in_channel_event_block,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1334), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [24455] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(145), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(147), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3635), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3637), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3641), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3645), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3647), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3665), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1325), 1,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat2,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2129), 1,
      sym_preproc_else_in_channel_event_block,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1334), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [24513] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3667), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3670), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3673), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3676), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3679), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3682), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3685), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3690), 1,
      aux_sym_preproc_if_not_defined_token1,
    STATE(1325), 1,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat2,
    STATE(2062), 1,
      sym_channel_event_type,
    ACTIONS(3688), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1334), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [24566] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3695), 1,
      sym_hold_keyword,
    ACTIONS(3697), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3699), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(3701), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3703), 1,
      aux_sym_preproc_else_token1,
    STATE(1332), 1,
      aux_sym_preproc_if_defined_in_button_event_block_repeat2,
    STATE(2066), 1,
      sym_preproc_else_in_button_event_block,
    STATE(2076), 1,
      sym_button_event_type,
    ACTIONS(3693), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1342), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [24621] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3705), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3708), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3711), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3714), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3717), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3720), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3723), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3728), 1,
      aux_sym_preproc_if_not_defined_token1,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    ACTIONS(3726), 3,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    STATE(1327), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [24672] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3695), 1,
      sym_hold_keyword,
    ACTIONS(3697), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3699), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(3701), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3703), 1,
      aux_sym_preproc_else_token1,
    STATE(1333), 1,
      aux_sym_preproc_if_defined_in_button_event_block_repeat2,
    STATE(2066), 1,
      sym_preproc_else_in_button_event_block,
    STATE(2076), 1,
      sym_button_event_type,
    ACTIONS(3693), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1342), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [24727] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3695), 1,
      sym_hold_keyword,
    ACTIONS(3697), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3701), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3703), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3731), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1331), 1,
      aux_sym_preproc_if_defined_in_button_event_block_repeat2,
    STATE(2076), 1,
      sym_button_event_type,
    STATE(2096), 1,
      sym_preproc_else_in_button_event_block,
    ACTIONS(3693), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1342), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [24782] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3695), 1,
      sym_hold_keyword,
    ACTIONS(3697), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3701), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3703), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3731), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1333), 1,
      aux_sym_preproc_if_defined_in_button_event_block_repeat2,
    STATE(2076), 1,
      sym_button_event_type,
    STATE(2096), 1,
      sym_preproc_else_in_button_event_block,
    ACTIONS(3693), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1342), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [24837] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3695), 1,
      sym_hold_keyword,
    ACTIONS(3697), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3701), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3703), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3733), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1333), 1,
      aux_sym_preproc_if_defined_in_button_event_block_repeat2,
    STATE(2076), 1,
      sym_button_event_type,
    STATE(2118), 1,
      sym_preproc_else_in_button_event_block,
    ACTIONS(3693), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1342), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [24892] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3695), 1,
      sym_hold_keyword,
    ACTIONS(3697), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3701), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3703), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3735), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1333), 1,
      aux_sym_preproc_if_defined_in_button_event_block_repeat2,
    STATE(2076), 1,
      sym_button_event_type,
    STATE(2107), 1,
      sym_preproc_else_in_button_event_block,
    ACTIONS(3693), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1342), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [24947] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3740), 1,
      sym_hold_keyword,
    ACTIONS(3743), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3746), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3749), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3752), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3755), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3760), 1,
      aux_sym_preproc_if_not_defined_token1,
    STATE(1333), 1,
      aux_sym_preproc_if_defined_in_button_event_block_repeat2,
    STATE(2076), 1,
      sym_button_event_type,
    ACTIONS(3737), 2,
      sym_push_keyword,
      sym_release_keyword,
    ACTIONS(3758), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1342), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [24997] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3763), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3766), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3769), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3772), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3775), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3778), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3781), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3786), 1,
      aux_sym_preproc_if_not_defined_token1,
    STATE(2062), 1,
      sym_channel_event_type,
    ACTIONS(3784), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1327), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [25047] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3792), 1,
      sym_hold_keyword,
    ACTIONS(3795), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3798), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3801), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3804), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3807), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3812), 1,
      aux_sym_preproc_if_not_defined_token1,
    STATE(2076), 1,
      sym_button_event_type,
    ACTIONS(3789), 2,
      sym_push_keyword,
      sym_release_keyword,
    ACTIONS(3810), 3,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
    STATE(1335), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [25095] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(145), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(147), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3635), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3637), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3641), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3645), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3815), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1337), 1,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat2,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1334), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [25147] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(145), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(147), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3635), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3637), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3641), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3645), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3817), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1325), 1,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat2,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1334), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [25199] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3695), 1,
      sym_hold_keyword,
    ACTIONS(3697), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3701), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3819), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1341), 1,
      aux_sym_preproc_if_defined_in_button_event_block_repeat2,
    STATE(2076), 1,
      sym_button_event_type,
    ACTIONS(3693), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1342), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [25248] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(145), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(147), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3635), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3637), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3641), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3645), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3821), 1,
      anon_sym_RBRACE,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1340), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [25297] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(145), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(147), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3635), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3637), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3641), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3645), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3823), 1,
      anon_sym_RBRACE,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1327), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [25346] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3695), 1,
      sym_hold_keyword,
    ACTIONS(3697), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3701), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3825), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1333), 1,
      aux_sym_preproc_if_defined_in_button_event_block_repeat2,
    STATE(2076), 1,
      sym_button_event_type,
    ACTIONS(3693), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1342), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [25395] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3830), 1,
      sym_hold_keyword,
    ACTIONS(3833), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3836), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3839), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3842), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3845), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3850), 1,
      aux_sym_preproc_if_not_defined_token1,
    STATE(2076), 1,
      sym_button_event_type,
    ACTIONS(3827), 2,
      sym_push_keyword,
      sym_release_keyword,
    ACTIONS(3848), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1335), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [25442] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(145), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(147), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3635), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3637), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3641), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3645), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3853), 1,
      anon_sym_RBRACE,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1327), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [25491] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(145), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(147), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3635), 1,
      aux_sym_on_keyword_token1,
    ACTIONS(3637), 1,
      aux_sym_off_keyword_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3641), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3645), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3855), 1,
      anon_sym_RBRACE,
    STATE(2062), 1,
      sym_channel_event_type,
    STATE(2194), 2,
      sym_on_keyword,
      sym_off_keyword,
    STATE(1343), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_channel_event_block,
      sym_preproc_if_not_defined_in_channel_event_block,
      sym__channel_event_block_item,
      sym_channel_event_handler,
      aux_sym_preproc_if_defined_in_channel_event_block_repeat1,
  [25540] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3695), 1,
      sym_hold_keyword,
    ACTIONS(3697), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3701), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3857), 1,
      anon_sym_RBRACE,
    STATE(2076), 1,
      sym_button_event_type,
    ACTIONS(3693), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1335), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [25586] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3695), 1,
      sym_hold_keyword,
    ACTIONS(3697), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3701), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3859), 1,
      anon_sym_RBRACE,
    STATE(2076), 1,
      sym_button_event_type,
    ACTIONS(3693), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1335), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [25632] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3695), 1,
      sym_hold_keyword,
    ACTIONS(3697), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3701), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3861), 1,
      anon_sym_RBRACE,
    STATE(2076), 1,
      sym_button_event_type,
    ACTIONS(3693), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1346), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [25678] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3502), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3504), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3506), 1,
      aux_sym_preproc_warn_token1,
    ACTIONS(3508), 1,
      aux_sym_preproc_disable_warning_token1,
    ACTIONS(3695), 1,
      sym_hold_keyword,
    ACTIONS(3697), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3701), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3863), 1,
      anon_sym_RBRACE,
    STATE(2076), 1,
      sym_button_event_type,
    ACTIONS(3693), 2,
      sym_push_keyword,
      sym_release_keyword,
    STATE(1345), 9,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_warn,
      sym_preproc_disable_warning,
      sym_preproc_if_defined_in_button_event_block,
      sym_preproc_if_not_defined_in_button_event_block,
      sym__button_event_block_item,
      sym_button_event_handler,
      aux_sym_preproc_if_defined_in_button_event_block_repeat1,
  [25724] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1695), 19,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [25749] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1711), 19,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [25774] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1669), 19,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [25799] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1665), 19,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [25824] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1699), 19,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [25849] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1707), 19,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [25874] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1703), 19,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [25899] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3865), 16,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [25921] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3867), 16,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [25943] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3869), 16,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [25965] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3871), 16,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [25987] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3873), 16,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26009] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3875), 16,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26031] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3877), 16,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26053] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3879), 16,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26075] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3881), 16,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26097] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3883), 16,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26119] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3885), 16,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26141] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3887), 16,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26163] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3889), 16,
      sym_online_keyword,
      sym_offline_keyword,
      sym_onerror_keyword,
      sym_string_keyword,
      sym_command_keyword,
      sym_standby_keyword,
      sym_awake_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26185] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3891), 2,
      anon_sym_LPAREN,
      sym_identifier,
    ACTIONS(3893), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    ACTIONS(3897), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(3895), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1372), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
  [26214] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3051), 2,
      anon_sym_LPAREN,
      sym_identifier,
    ACTIONS(3899), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    ACTIONS(3905), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(3902), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1370), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
  [26243] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3893), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    ACTIONS(3907), 2,
      anon_sym_LPAREN,
      sym_identifier,
    ACTIONS(3909), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(3895), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1373), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
  [26272] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3893), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    ACTIONS(3911), 2,
      anon_sym_LPAREN,
      sym_identifier,
    ACTIONS(3913), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(3895), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1370), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
  [26301] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3893), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    ACTIONS(3915), 2,
      anon_sym_LPAREN,
      sym_identifier,
    ACTIONS(3917), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(3895), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1370), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
  [26330] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3891), 2,
      anon_sym_LPAREN,
      sym_identifier,
    ACTIONS(3897), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3919), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    ACTIONS(3921), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1376), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
  [26358] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3051), 2,
      anon_sym_LPAREN,
      sym_identifier,
    ACTIONS(3905), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3923), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    ACTIONS(3926), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1375), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
  [26386] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3911), 2,
      anon_sym_LPAREN,
      sym_identifier,
    ACTIONS(3913), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3919), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    ACTIONS(3921), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1375), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
  [26414] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(3929), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    ACTIONS(3931), 2,
      anon_sym_EQ,
      anon_sym_LBRACK,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    STATE(1721), 2,
      sym_init_declarator,
      sym__declaration_declarator,
    ACTIONS(3933), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [26450] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(3929), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    ACTIONS(3931), 2,
      anon_sym_EQ,
      anon_sym_LBRACK,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    STATE(1710), 2,
      sym_init_declarator,
      sym__declaration_declarator,
    ACTIONS(3933), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [26486] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(3929), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    ACTIONS(3931), 2,
      anon_sym_EQ,
      anon_sym_LBRACK,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    STATE(1739), 2,
      sym_init_declarator,
      sym__declaration_declarator,
    ACTIONS(3933), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [26522] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(3929), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1742), 1,
      sym_init_declarator,
    STATE(1743), 1,
      sym__declaration_declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    ACTIONS(3931), 2,
      anon_sym_EQ,
      anon_sym_LBRACK,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    ACTIONS(3933), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [26560] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(3929), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1713), 1,
      sym_init_declarator,
    STATE(1714), 1,
      sym__declaration_declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    ACTIONS(3931), 2,
      anon_sym_EQ,
      anon_sym_LBRACK,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    ACTIONS(3933), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [26598] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3907), 2,
      anon_sym_LPAREN,
      sym_identifier,
    ACTIONS(3909), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3919), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    ACTIONS(3921), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1383), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
  [26626] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3915), 2,
      anon_sym_LPAREN,
      sym_identifier,
    ACTIONS(3917), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3919), 2,
      sym_local_var_keyword,
      sym_stack_var_keyword,
    ACTIONS(3921), 4,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
    STATE(1375), 4,
      sym_storage_class_specifier,
      sym_type_qualifier,
      sym__declaration_modifiers,
      aux_sym__declaration_specifiers_repeat1,
  [26654] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(3929), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    ACTIONS(3931), 2,
      anon_sym_EQ,
      anon_sym_LBRACK,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    STATE(1796), 2,
      sym_init_declarator,
      sym__declaration_declarator,
    ACTIONS(3933), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [26690] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(3929), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1728), 1,
      sym_init_declarator,
    STATE(1730), 1,
      sym__declaration_declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    ACTIONS(3931), 2,
      anon_sym_EQ,
      anon_sym_LBRACK,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    ACTIONS(3933), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [26728] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(3929), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1753), 1,
      sym__declaration_declarator,
    STATE(1764), 1,
      sym_init_declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    ACTIONS(3931), 2,
      anon_sym_EQ,
      anon_sym_LBRACK,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    ACTIONS(3933), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [26766] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3935), 12,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26784] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3937), 1,
      sym_release_keyword,
    ACTIONS(3939), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3941), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3943), 1,
      anon_sym_LBRACE,
    STATE(1406), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_release_event_declarator,
      sym_preproc_if_not_defined_in_release_event_declarator,
      sym_release_event_declarator,
      aux_sym__release_event_declarator_list_repeat1,
  [26814] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3945), 1,
      sym_custom_event_keyword,
    ACTIONS(3948), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3951), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3954), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3957), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3960), 1,
      anon_sym_LBRACE,
    STATE(1389), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_custom_event_declarator,
      sym_preproc_if_not_defined_in_custom_event_declarator,
      sym_custom_event_declarator,
      aux_sym__custom_event_declarator_list_repeat1,
  [26844] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3962), 1,
      sym_push_keyword,
    ACTIONS(3965), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3968), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3971), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3974), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3977), 1,
      anon_sym_LBRACE,
    STATE(1390), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_push_event_declarator,
      sym_preproc_if_not_defined_in_push_event_declarator,
      sym_push_event_declarator,
      aux_sym__push_event_declarator_list_repeat1,
  [26874] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3979), 1,
      sym_button_event_keyword,
    ACTIONS(3981), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3983), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(3985), 1,
      anon_sym_LBRACE,
    STATE(1418), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_button_event_declarator,
      sym_preproc_if_not_defined_in_button_event_declarator,
      sym_button_event_declarator,
      aux_sym__button_event_declarator_list_repeat1,
  [26904] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3987), 12,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26922] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3989), 12,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26940] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3991), 12,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26958] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3993), 12,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26976] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3995), 12,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [26994] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3997), 1,
      sym_data_event_keyword,
    ACTIONS(3999), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4001), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4003), 1,
      anon_sym_LBRACE,
    STATE(1417), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_data_event_declarator,
      sym_preproc_if_not_defined_in_data_event_declarator,
      sym_data_event_declarator,
      aux_sym__data_event_declarator_list_repeat1,
  [27024] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4005), 1,
      sym_level_event_keyword,
    ACTIONS(4007), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4009), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4011), 1,
      anon_sym_LBRACE,
    STATE(1402), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_level_event_declarator,
      sym_preproc_if_not_defined_in_level_event_declarator,
      sym_level_event_declarator,
      aux_sym__level_event_declarator_list_repeat1,
  [27054] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4013), 1,
      sym_custom_event_keyword,
    ACTIONS(4015), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4017), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4019), 1,
      anon_sym_LBRACE,
    STATE(1423), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_custom_event_declarator,
      sym_preproc_if_not_defined_in_custom_event_declarator,
      sym_custom_event_declarator,
      aux_sym__custom_event_declarator_list_repeat1,
  [27084] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4021), 1,
      sym_channel_event_keyword,
    ACTIONS(4023), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4025), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4027), 1,
      anon_sym_LBRACE,
    STATE(1408), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_channel_event_declarator,
      sym_preproc_if_not_defined_in_channel_event_declarator,
      sym_channel_event_declarator,
      aux_sym__channel_event_declarator_list_repeat1,
  [27114] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4029), 12,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27132] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4031), 1,
      sym_level_event_keyword,
    ACTIONS(4034), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4037), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(4040), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4043), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4046), 1,
      anon_sym_LBRACE,
    STATE(1402), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_level_event_declarator,
      sym_preproc_if_not_defined_in_level_event_declarator,
      sym_level_event_declarator,
      aux_sym__level_event_declarator_list_repeat1,
  [27162] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4021), 1,
      sym_channel_event_keyword,
    ACTIONS(4023), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4025), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4048), 1,
      anon_sym_LBRACE,
    STATE(1400), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_channel_event_declarator,
      sym_preproc_if_not_defined_in_channel_event_declarator,
      sym_channel_event_declarator,
      aux_sym__channel_event_declarator_list_repeat1,
  [27192] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4050), 1,
      sym_push_keyword,
    ACTIONS(4052), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4054), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4056), 1,
      anon_sym_LBRACE,
    STATE(1390), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_push_event_declarator,
      sym_preproc_if_not_defined_in_push_event_declarator,
      sym_push_event_declarator,
      aux_sym__push_event_declarator_list_repeat1,
  [27222] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4058), 1,
      sym_data_event_keyword,
    ACTIONS(4061), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4064), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(4067), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4070), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4073), 1,
      anon_sym_LBRACE,
    STATE(1405), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_data_event_declarator,
      sym_preproc_if_not_defined_in_data_event_declarator,
      sym_data_event_declarator,
      aux_sym__data_event_declarator_list_repeat1,
  [27252] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3937), 1,
      sym_release_keyword,
    ACTIONS(3939), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3941), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4075), 1,
      anon_sym_LBRACE,
    STATE(1421), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_release_event_declarator,
      sym_preproc_if_not_defined_in_release_event_declarator,
      sym_release_event_declarator,
      aux_sym__release_event_declarator_list_repeat1,
  [27282] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4077), 12,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27300] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4079), 1,
      sym_channel_event_keyword,
    ACTIONS(4082), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4085), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(4088), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4091), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4094), 1,
      anon_sym_LBRACE,
    STATE(1408), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_channel_event_declarator,
      sym_preproc_if_not_defined_in_channel_event_declarator,
      sym_channel_event_declarator,
      aux_sym__channel_event_declarator_list_repeat1,
  [27330] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4096), 1,
      sym_timeline_event_keyword,
    ACTIONS(4099), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4102), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(4105), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4108), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4111), 1,
      anon_sym_LBRACE,
    STATE(1409), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_timeline_event_declarator,
      sym_preproc_if_not_defined_in_timeline_event_declarator,
      sym_timeline_event_declarator,
      aux_sym__timeline_event_declarator_list_repeat1,
  [27360] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4113), 1,
      sym_timeline_event_keyword,
    ACTIONS(4115), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4117), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4119), 1,
      anon_sym_LBRACE,
    STATE(1409), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_timeline_event_declarator,
      sym_preproc_if_not_defined_in_timeline_event_declarator,
      sym_timeline_event_declarator,
      aux_sym__timeline_event_declarator_list_repeat1,
  [27390] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4121), 12,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27408] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4123), 12,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27426] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4125), 12,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27444] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4127), 12,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27462] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4113), 1,
      sym_timeline_event_keyword,
    ACTIONS(4115), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4117), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4129), 1,
      anon_sym_LBRACE,
    STATE(1410), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_timeline_event_declarator,
      sym_preproc_if_not_defined_in_timeline_event_declarator,
      sym_timeline_event_declarator,
      aux_sym__timeline_event_declarator_list_repeat1,
  [27492] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4131), 12,
      sym_push_keyword,
      sym_release_keyword,
      sym_hold_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27510] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3997), 1,
      sym_data_event_keyword,
    ACTIONS(3999), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4001), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4133), 1,
      anon_sym_LBRACE,
    STATE(1405), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_data_event_declarator,
      sym_preproc_if_not_defined_in_data_event_declarator,
      sym_data_event_declarator,
      aux_sym__data_event_declarator_list_repeat1,
  [27540] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4135), 1,
      sym_button_event_keyword,
    ACTIONS(4138), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4141), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(4144), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4147), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4150), 1,
      anon_sym_LBRACE,
    STATE(1418), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_button_event_declarator,
      sym_preproc_if_not_defined_in_button_event_declarator,
      sym_button_event_declarator,
      aux_sym__button_event_declarator_list_repeat1,
  [27570] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4050), 1,
      sym_push_keyword,
    ACTIONS(4052), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4054), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4152), 1,
      anon_sym_LBRACE,
    STATE(1404), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_push_event_declarator,
      sym_preproc_if_not_defined_in_push_event_declarator,
      sym_push_event_declarator,
      aux_sym__push_event_declarator_list_repeat1,
  [27600] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(3979), 1,
      sym_button_event_keyword,
    ACTIONS(3981), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(3983), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4154), 1,
      anon_sym_LBRACE,
    STATE(1391), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_button_event_declarator,
      sym_preproc_if_not_defined_in_button_event_declarator,
      sym_button_event_declarator,
      aux_sym__button_event_declarator_list_repeat1,
  [27630] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4156), 1,
      sym_release_keyword,
    ACTIONS(4159), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4162), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(4165), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4168), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4171), 1,
      anon_sym_LBRACE,
    STATE(1421), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_release_event_declarator,
      sym_preproc_if_not_defined_in_release_event_declarator,
      sym_release_event_declarator,
      aux_sym__release_event_declarator_list_repeat1,
  [27660] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4005), 1,
      sym_level_event_keyword,
    ACTIONS(4007), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4009), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4173), 1,
      anon_sym_LBRACE,
    STATE(1398), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_level_event_declarator,
      sym_preproc_if_not_defined_in_level_event_declarator,
      sym_level_event_declarator,
      aux_sym__level_event_declarator_list_repeat1,
  [27690] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(143), 1,
      aux_sym_preproc_define_token1,
    ACTIONS(3639), 1,
      aux_sym_preproc_include_token1,
    ACTIONS(4013), 1,
      sym_custom_event_keyword,
    ACTIONS(4015), 1,
      aux_sym_preproc_if_defined_token1,
    ACTIONS(4017), 1,
      aux_sym_preproc_if_not_defined_token1,
    ACTIONS(4175), 1,
      anon_sym_LBRACE,
    STATE(1389), 6,
      sym_preproc_include,
      sym_preproc_define,
      sym_preproc_if_defined_in_custom_event_declarator,
      sym_preproc_if_not_defined_in_custom_event_declarator,
      sym_custom_event_declarator,
      aux_sym__custom_event_declarator_list_repeat1,
  [27720] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4179), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(4177), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [27739] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4181), 11,
      aux_sym_on_keyword_token1,
      aux_sym_off_keyword_token1,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27756] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4183), 11,
      aux_sym_on_keyword_token1,
      aux_sym_off_keyword_token1,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27773] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4185), 11,
      aux_sym_on_keyword_token1,
      aux_sym_off_keyword_token1,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27790] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4189), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(4187), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [27809] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4193), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(4191), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [27828] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4195), 11,
      aux_sym_on_keyword_token1,
      aux_sym_off_keyword_token1,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27845] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4197), 1,
      sym_identifier,
    ACTIONS(4201), 1,
      anon_sym_LBRACK,
    ACTIONS(4203), 1,
      anon_sym_LPAREN,
    STATE(1696), 1,
      sym__declarator,
    STATE(1860), 1,
      sym_abstract_array_declarator,
    STATE(1862), 1,
      sym__abstract_declarator,
    ACTIONS(4199), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1705), 3,
      sym_array_declarator,
      sym_function_declarator,
      sym_parenthesized_declarator,
  [27876] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4205), 11,
      aux_sym_on_keyword_token1,
      aux_sym_off_keyword_token1,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27893] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4207), 11,
      aux_sym_on_keyword_token1,
      aux_sym_off_keyword_token1,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27910] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4209), 11,
      aux_sym_on_keyword_token1,
      aux_sym_off_keyword_token1,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27927] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4211), 11,
      aux_sym_on_keyword_token1,
      aux_sym_off_keyword_token1,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27944] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4213), 11,
      aux_sym_on_keyword_token1,
      aux_sym_off_keyword_token1,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27961] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4215), 11,
      aux_sym_on_keyword_token1,
      aux_sym_off_keyword_token1,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27978] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4217), 11,
      aux_sym_on_keyword_token1,
      aux_sym_off_keyword_token1,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [27995] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4219), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(3044), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [28014] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4223), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(4221), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [28033] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4225), 11,
      aux_sym_on_keyword_token1,
      aux_sym_off_keyword_token1,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [28050] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4227), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(3114), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [28069] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4229), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(3076), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [28088] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4233), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
    ACTIONS(4231), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [28107] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4235), 11,
      aux_sym_on_keyword_token1,
      aux_sym_off_keyword_token1,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_warn_token1,
      aux_sym_preproc_disable_warning_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_RBRACE,
  [28124] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4179), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(4177), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [28142] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4223), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(4221), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [28160] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4193), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(4191), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [28178] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4229), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3076), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [28196] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4233), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(4231), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [28214] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4189), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(4187), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [28232] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4227), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3114), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [28250] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4219), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
    ACTIONS(3044), 8,
      sym_local_var_keyword,
      sym_stack_var_keyword,
      sym_constant_keyword,
      sym_volatile_keyword,
      sym_non_volatile_keyword,
      sym_persistent_keyword,
      anon_sym_LPAREN,
      sym_identifier,
  [28268] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(4237), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1802), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym__declaration_declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
  [28297] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(4239), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1794), 1,
      sym_init_declarator,
    STATE(1797), 1,
      sym__declaration_declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
  [28326] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(4241), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1736), 1,
      sym_init_declarator,
    STATE(1737), 1,
      sym__declaration_declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
  [28355] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(4243), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1719), 1,
      sym_init_declarator,
    STATE(1771), 1,
      sym__declaration_declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
  [28384] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_LPAREN,
    ACTIONS(3929), 1,
      sym_identifier,
    STATE(1705), 1,
      sym_function_declarator,
    STATE(1708), 1,
      sym__declarator,
    STATE(1842), 1,
      sym__function_declaration_declarator,
    STATE(1664), 2,
      sym_array_declarator,
      sym_parenthesized_declarator,
    STATE(1826), 2,
      sym_init_declarator,
      sym__declaration_declarator,
  [28411] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4245), 1,
      sym_identifier,
    ACTIONS(4247), 1,
      anon_sym_LPAREN,
    ACTIONS(4249), 1,
      anon_sym_SEMI,
    ACTIONS(4251), 1,
      sym__automatic_semicolon,
    STATE(1505), 1,
      sym__field_declarator,
    STATE(1892), 1,
      sym__field_declaration_declarator,
    STATE(1706), 3,
      sym_array_field_declarator,
      sym_parenthesized_field_declarator,
      sym_function_field_declarator,
  [28438] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4253), 8,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28452] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4255), 8,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28466] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4257), 8,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28480] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4259), 8,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28494] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4261), 8,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28508] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4263), 8,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28522] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4265), 8,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28536] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4267), 8,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28550] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4269), 8,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28564] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4271), 8,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28578] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4273), 8,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28592] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4275), 8,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28606] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4277), 8,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28620] = 8,
    ACTIONS(79), 1,
      sym_button_event_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4279), 1,
      anon_sym_LF,
    ACTIONS(4281), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4283), 1,
      aux_sym_preproc_else_token1,
    STATE(1549), 1,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat2,
    STATE(2108), 1,
      sym_preproc_else_in_button_event_declarator,
    STATE(1671), 2,
      sym_button_event_declarator,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [28646] = 8,
    ACTIONS(79), 1,
      sym_button_event_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4283), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4285), 1,
      anon_sym_LF,
    ACTIONS(4287), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1526), 1,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat2,
    STATE(2133), 1,
      sym_preproc_else_in_button_event_declarator,
    STATE(1671), 2,
      sym_button_event_declarator,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [28672] = 8,
    ACTIONS(83), 1,
      sym_level_event_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4289), 1,
      anon_sym_LF,
    ACTIONS(4291), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4293), 1,
      aux_sym_preproc_else_token1,
    STATE(1528), 1,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat2,
    STATE(2212), 1,
      sym_preproc_else_in_level_event_declarator,
    STATE(1670), 2,
      sym_level_event_declarator,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [28698] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4295), 8,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28712] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4297), 8,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28726] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4299), 8,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28740] = 8,
    ACTIONS(85), 1,
      sym_data_event_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4301), 1,
      anon_sym_LF,
    ACTIONS(4303), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4305), 1,
      aux_sym_preproc_else_token1,
    STATE(1515), 1,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat2,
    STATE(2103), 1,
      sym_preproc_else_in_data_event_declarator,
    STATE(1673), 2,
      sym_data_event_declarator,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [28766] = 8,
    ACTIONS(85), 1,
      sym_data_event_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4305), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4307), 1,
      anon_sym_LF,
    ACTIONS(4309), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1517), 1,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat2,
    STATE(2165), 1,
      sym_preproc_else_in_data_event_declarator,
    STATE(1673), 2,
      sym_data_event_declarator,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [28792] = 8,
    ACTIONS(81), 1,
      sym_channel_event_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4311), 1,
      anon_sym_LF,
    ACTIONS(4313), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4315), 1,
      aux_sym_preproc_else_token1,
    STATE(1532), 1,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2,
    STATE(2203), 1,
      sym_preproc_else_in_channel_event_declarator,
    STATE(1675), 2,
      sym_channel_event_declarator,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [28818] = 8,
    ACTIONS(81), 1,
      sym_channel_event_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4315), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4317), 1,
      anon_sym_LF,
    ACTIONS(4319), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1534), 1,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2,
    STATE(2063), 1,
      sym_preproc_else_in_channel_event_declarator,
    STATE(1675), 2,
      sym_channel_event_declarator,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [28844] = 8,
    ACTIONS(89), 1,
      sym_custom_event_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4321), 1,
      anon_sym_LF,
    ACTIONS(4323), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4325), 1,
      aux_sym_preproc_else_token1,
    STATE(1536), 1,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2,
    STATE(2120), 1,
      sym_preproc_else_in_custom_event_declarator,
    STATE(1682), 2,
      sym_custom_event_declarator,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [28870] = 8,
    ACTIONS(89), 1,
      sym_custom_event_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4325), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4327), 1,
      anon_sym_LF,
    ACTIONS(4329), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1538), 1,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2,
    STATE(2215), 1,
      sym_preproc_else_in_custom_event_declarator,
    STATE(1682), 2,
      sym_custom_event_declarator,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [28896] = 8,
    ACTIONS(87), 1,
      sym_timeline_event_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4331), 1,
      anon_sym_LF,
    ACTIONS(4333), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4335), 1,
      aux_sym_preproc_else_token1,
    STATE(1519), 1,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2,
    STATE(2154), 1,
      sym_preproc_else_in_timeline_event_declarator,
    STATE(1692), 2,
      sym_timeline_event_declarator,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [28922] = 8,
    ACTIONS(91), 1,
      sym_push_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4337), 1,
      anon_sym_LF,
    ACTIONS(4339), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4341), 1,
      aux_sym_preproc_else_token1,
    STATE(1540), 1,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat2,
    STATE(2248), 1,
      sym_preproc_else_in_push_event_declarator,
    STATE(1691), 2,
      sym_push_event_declarator,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [28948] = 8,
    ACTIONS(91), 1,
      sym_push_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4341), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4343), 1,
      anon_sym_LF,
    ACTIONS(4345), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1542), 1,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat2,
    STATE(2059), 1,
      sym_preproc_else_in_push_event_declarator,
    STATE(1691), 2,
      sym_push_event_declarator,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [28974] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4347), 8,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [28988] = 8,
    ACTIONS(87), 1,
      sym_timeline_event_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4335), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4349), 1,
      anon_sym_LF,
    ACTIONS(4351), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1521), 1,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2,
    STATE(2056), 1,
      sym_preproc_else_in_timeline_event_declarator,
    STATE(1692), 2,
      sym_timeline_event_declarator,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [29014] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4353), 8,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [29028] = 8,
    ACTIONS(93), 1,
      sym_release_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4355), 1,
      anon_sym_LF,
    ACTIONS(4357), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4359), 1,
      aux_sym_preproc_else_token1,
    STATE(1544), 1,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat2,
    STATE(2068), 1,
      sym_preproc_else_in_release_event_declarator,
    STATE(1686), 2,
      sym_release_event_declarator,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [29054] = 8,
    ACTIONS(93), 1,
      sym_release_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4359), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4361), 1,
      anon_sym_LF,
    ACTIONS(4363), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1546), 1,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat2,
    STATE(2081), 1,
      sym_preproc_else_in_release_event_declarator,
    STATE(1686), 2,
      sym_release_event_declarator,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [29080] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4365), 8,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [29094] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4367), 8,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [29108] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4369), 8,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [29122] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4371), 8,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [29136] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4373), 8,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_if_not_defined_token1,
      aux_sym_preproc_else_token1,
      anon_sym_LBRACE,
  [29150] = 8,
    ACTIONS(83), 1,
      sym_level_event_keyword,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4293), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4375), 1,
      anon_sym_LF,
    ACTIONS(4377), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1530), 1,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat2,
    STATE(2250), 1,
      sym_preproc_else_in_level_event_declarator,
    STATE(1670), 2,
      sym_level_event_declarator,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [29176] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3997), 1,
      sym_data_event_keyword,
    ACTIONS(4379), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4381), 1,
      aux_sym_preproc_else_token1,
    STATE(1616), 1,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat2,
    STATE(2186), 1,
      sym_preproc_else_in_data_event_declarator,
    STATE(1673), 2,
      sym_data_event_declarator,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [29199] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4383), 1,
      sym_identifier,
    ACTIONS(4385), 1,
      anon_sym_LPAREN,
    ACTIONS(4387), 1,
      anon_sym_RPAREN,
    STATE(1718), 1,
      sym__field_declarator,
    STATE(1829), 3,
      sym_array_field_declarator,
      sym_parenthesized_field_declarator,
      sym_function_field_declarator,
  [29220] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3979), 1,
      sym_button_event_keyword,
    ACTIONS(4389), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4391), 1,
      aux_sym_preproc_else_token1,
    STATE(1560), 1,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat2,
    STATE(2131), 1,
      sym_preproc_else_in_button_event_declarator,
    STATE(1671), 2,
      sym_button_event_declarator,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [29243] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3979), 1,
      sym_button_event_keyword,
    ACTIONS(4391), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4393), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1560), 1,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat2,
    STATE(2146), 1,
      sym_preproc_else_in_button_event_declarator,
    STATE(1671), 2,
      sym_button_event_declarator,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [29266] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4005), 1,
      sym_level_event_keyword,
    ACTIONS(4395), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4397), 1,
      aux_sym_preproc_else_token1,
    STATE(1583), 1,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat2,
    STATE(2182), 1,
      sym_preproc_else_in_level_event_declarator,
    STATE(1670), 2,
      sym_level_event_declarator,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [29289] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4005), 1,
      sym_level_event_keyword,
    ACTIONS(4397), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4399), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1583), 1,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat2,
    STATE(2200), 1,
      sym_preproc_else_in_level_event_declarator,
    STATE(1670), 2,
      sym_level_event_declarator,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [29312] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4401), 1,
      anon_sym_COMMA,
    ACTIONS(4403), 1,
      anon_sym_LBRACK,
    ACTIONS(4405), 1,
      anon_sym_LPAREN,
    STATE(1672), 1,
      sym_parameter_list,
    STATE(1751), 1,
      aux_sym__field_declaration_declarator_repeat1,
    ACTIONS(4407), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [29335] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4021), 1,
      sym_channel_event_keyword,
    ACTIONS(4409), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4411), 1,
      aux_sym_preproc_else_token1,
    STATE(1620), 1,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2,
    STATE(2113), 1,
      sym_preproc_else_in_channel_event_declarator,
    STATE(1675), 2,
      sym_channel_event_declarator,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [29358] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4021), 1,
      sym_channel_event_keyword,
    ACTIONS(4411), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4413), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1620), 1,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2,
    STATE(2153), 1,
      sym_preproc_else_in_channel_event_declarator,
    STATE(1675), 2,
      sym_channel_event_declarator,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [29381] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4013), 1,
      sym_custom_event_keyword,
    ACTIONS(4415), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4417), 1,
      aux_sym_preproc_else_token1,
    STATE(1632), 1,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2,
    STATE(2264), 1,
      sym_preproc_else_in_custom_event_declarator,
    STATE(1682), 2,
      sym_custom_event_declarator,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [29404] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4013), 1,
      sym_custom_event_keyword,
    ACTIONS(4417), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4419), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1632), 1,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2,
    STATE(2251), 1,
      sym_preproc_else_in_custom_event_declarator,
    STATE(1682), 2,
      sym_custom_event_declarator,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [29427] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4050), 1,
      sym_push_keyword,
    ACTIONS(4421), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4423), 1,
      aux_sym_preproc_else_token1,
    STATE(1552), 1,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat2,
    STATE(2140), 1,
      sym_preproc_else_in_push_event_declarator,
    STATE(1691), 2,
      sym_push_event_declarator,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [29450] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4050), 1,
      sym_push_keyword,
    ACTIONS(4423), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4425), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1552), 1,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat2,
    STATE(2161), 1,
      sym_preproc_else_in_push_event_declarator,
    STATE(1691), 2,
      sym_push_event_declarator,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [29473] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3937), 1,
      sym_release_keyword,
    ACTIONS(4427), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4429), 1,
      aux_sym_preproc_else_token1,
    STATE(1558), 1,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat2,
    STATE(2187), 1,
      sym_preproc_else_in_release_event_declarator,
    STATE(1686), 2,
      sym_release_event_declarator,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [29496] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3937), 1,
      sym_release_keyword,
    ACTIONS(4429), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4431), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1558), 1,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat2,
    STATE(2259), 1,
      sym_preproc_else_in_release_event_declarator,
    STATE(1686), 2,
      sym_release_event_declarator,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [29519] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3997), 1,
      sym_data_event_keyword,
    ACTIONS(4381), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4433), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1499), 1,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat2,
    STATE(2236), 1,
      sym_preproc_else_in_data_event_declarator,
    STATE(1673), 2,
      sym_data_event_declarator,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [29542] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3997), 1,
      sym_data_event_keyword,
    ACTIONS(4381), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4433), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1616), 1,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat2,
    STATE(2236), 1,
      sym_preproc_else_in_data_event_declarator,
    STATE(1673), 2,
      sym_data_event_declarator,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [29565] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3997), 1,
      sym_data_event_keyword,
    ACTIONS(4381), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4435), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1547), 1,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat2,
    STATE(2156), 1,
      sym_preproc_else_in_data_event_declarator,
    STATE(1673), 2,
      sym_data_event_declarator,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [29588] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3997), 1,
      sym_data_event_keyword,
    ACTIONS(4381), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4435), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1616), 1,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat2,
    STATE(2156), 1,
      sym_preproc_else_in_data_event_declarator,
    STATE(1673), 2,
      sym_data_event_declarator,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [29611] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4113), 1,
      sym_timeline_event_keyword,
    ACTIONS(4437), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4439), 1,
      aux_sym_preproc_else_token1,
    STATE(1548), 1,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2,
    STATE(2163), 1,
      sym_preproc_else_in_timeline_event_declarator,
    STATE(1692), 2,
      sym_timeline_event_declarator,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [29634] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4113), 1,
      sym_timeline_event_keyword,
    ACTIONS(4437), 1,
      aux_sym_preproc_if_defined_token2,
    ACTIONS(4439), 1,
      aux_sym_preproc_else_token1,
    STATE(1660), 1,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2,
    STATE(2163), 1,
      sym_preproc_else_in_timeline_event_declarator,
    STATE(1692), 2,
      sym_timeline_event_declarator,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [29657] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4113), 1,
      sym_timeline_event_keyword,
    ACTIONS(4439), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4441), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1523), 1,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2,
    STATE(2126), 1,
      sym_preproc_else_in_timeline_event_declarator,
    STATE(1692), 2,
      sym_timeline_event_declarator,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [29680] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4113), 1,
      sym_timeline_event_keyword,
    ACTIONS(4439), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4441), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1660), 1,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2,
    STATE(2126), 1,
      sym_preproc_else_in_timeline_event_declarator,
    STATE(1692), 2,
      sym_timeline_event_declarator,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [29703] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3979), 1,
      sym_button_event_keyword,
    ACTIONS(4391), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4443), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1501), 1,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat2,
    STATE(2157), 1,
      sym_preproc_else_in_button_event_declarator,
    STATE(1671), 2,
      sym_button_event_declarator,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [29726] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4113), 1,
      sym_timeline_event_keyword,
    ACTIONS(4439), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4445), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1660), 1,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2,
    STATE(2260), 1,
      sym_preproc_else_in_timeline_event_declarator,
    STATE(1692), 2,
      sym_timeline_event_declarator,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [29749] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3979), 1,
      sym_button_event_keyword,
    ACTIONS(4391), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4447), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1502), 1,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat2,
    STATE(2058), 1,
      sym_preproc_else_in_button_event_declarator,
    STATE(1671), 2,
      sym_button_event_declarator,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [29772] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4383), 1,
      sym_identifier,
    ACTIONS(4385), 1,
      anon_sym_LPAREN,
    ACTIONS(4449), 1,
      anon_sym_RPAREN,
    STATE(1761), 1,
      sym__field_declarator,
    STATE(1829), 3,
      sym_array_field_declarator,
      sym_parenthesized_field_declarator,
      sym_function_field_declarator,
  [29793] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3979), 1,
      sym_button_event_keyword,
    ACTIONS(4391), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4447), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1560), 1,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat2,
    STATE(2058), 1,
      sym_preproc_else_in_button_event_declarator,
    STATE(1671), 2,
      sym_button_event_declarator,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [29816] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4005), 1,
      sym_level_event_keyword,
    ACTIONS(4397), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4451), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1503), 1,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat2,
    STATE(2134), 1,
      sym_preproc_else_in_level_event_declarator,
    STATE(1670), 2,
      sym_level_event_declarator,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [29839] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4005), 1,
      sym_level_event_keyword,
    ACTIONS(4397), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4451), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1583), 1,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat2,
    STATE(2134), 1,
      sym_preproc_else_in_level_event_declarator,
    STATE(1670), 2,
      sym_level_event_declarator,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [29862] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4005), 1,
      sym_level_event_keyword,
    ACTIONS(4397), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4453), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1504), 1,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat2,
    STATE(2162), 1,
      sym_preproc_else_in_level_event_declarator,
    STATE(1670), 2,
      sym_level_event_declarator,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [29885] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4005), 1,
      sym_level_event_keyword,
    ACTIONS(4397), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4453), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1583), 1,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat2,
    STATE(2162), 1,
      sym_preproc_else_in_level_event_declarator,
    STATE(1670), 2,
      sym_level_event_declarator,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [29908] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4021), 1,
      sym_channel_event_keyword,
    ACTIONS(4411), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4455), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1506), 1,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2,
    STATE(2123), 1,
      sym_preproc_else_in_channel_event_declarator,
    STATE(1675), 2,
      sym_channel_event_declarator,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [29931] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4021), 1,
      sym_channel_event_keyword,
    ACTIONS(4411), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4455), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1620), 1,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2,
    STATE(2123), 1,
      sym_preproc_else_in_channel_event_declarator,
    STATE(1675), 2,
      sym_channel_event_declarator,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [29954] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4021), 1,
      sym_channel_event_keyword,
    ACTIONS(4411), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4457), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1507), 1,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2,
    STATE(2209), 1,
      sym_preproc_else_in_channel_event_declarator,
    STATE(1675), 2,
      sym_channel_event_declarator,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [29977] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4021), 1,
      sym_channel_event_keyword,
    ACTIONS(4411), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4457), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1620), 1,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2,
    STATE(2209), 1,
      sym_preproc_else_in_channel_event_declarator,
    STATE(1675), 2,
      sym_channel_event_declarator,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [30000] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4013), 1,
      sym_custom_event_keyword,
    ACTIONS(4417), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4459), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1508), 1,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2,
    STATE(2155), 1,
      sym_preproc_else_in_custom_event_declarator,
    STATE(1682), 2,
      sym_custom_event_declarator,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [30023] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4013), 1,
      sym_custom_event_keyword,
    ACTIONS(4417), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4459), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1632), 1,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2,
    STATE(2155), 1,
      sym_preproc_else_in_custom_event_declarator,
    STATE(1682), 2,
      sym_custom_event_declarator,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [30046] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4013), 1,
      sym_custom_event_keyword,
    ACTIONS(4417), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4461), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1509), 1,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2,
    STATE(2127), 1,
      sym_preproc_else_in_custom_event_declarator,
    STATE(1682), 2,
      sym_custom_event_declarator,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [30069] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4013), 1,
      sym_custom_event_keyword,
    ACTIONS(4417), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4461), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1632), 1,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2,
    STATE(2127), 1,
      sym_preproc_else_in_custom_event_declarator,
    STATE(1682), 2,
      sym_custom_event_declarator,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [30092] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4050), 1,
      sym_push_keyword,
    ACTIONS(4423), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4463), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1510), 1,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat2,
    STATE(2077), 1,
      sym_preproc_else_in_push_event_declarator,
    STATE(1691), 2,
      sym_push_event_declarator,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [30115] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4050), 1,
      sym_push_keyword,
    ACTIONS(4423), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4463), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1552), 1,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat2,
    STATE(2077), 1,
      sym_preproc_else_in_push_event_declarator,
    STATE(1691), 2,
      sym_push_event_declarator,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [30138] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4050), 1,
      sym_push_keyword,
    ACTIONS(4423), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4465), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1511), 1,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat2,
    STATE(2057), 1,
      sym_preproc_else_in_push_event_declarator,
    STATE(1691), 2,
      sym_push_event_declarator,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [30161] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4050), 1,
      sym_push_keyword,
    ACTIONS(4423), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4465), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1552), 1,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat2,
    STATE(2057), 1,
      sym_preproc_else_in_push_event_declarator,
    STATE(1691), 2,
      sym_push_event_declarator,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [30184] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3937), 1,
      sym_release_keyword,
    ACTIONS(4429), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4467), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1512), 1,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat2,
    STATE(2111), 1,
      sym_preproc_else_in_release_event_declarator,
    STATE(1686), 2,
      sym_release_event_declarator,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [30207] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3937), 1,
      sym_release_keyword,
    ACTIONS(4429), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4467), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1558), 1,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat2,
    STATE(2111), 1,
      sym_preproc_else_in_release_event_declarator,
    STATE(1686), 2,
      sym_release_event_declarator,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [30230] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3937), 1,
      sym_release_keyword,
    ACTIONS(4429), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4469), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1513), 1,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat2,
    STATE(2158), 1,
      sym_preproc_else_in_release_event_declarator,
    STATE(1686), 2,
      sym_release_event_declarator,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [30253] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3937), 1,
      sym_release_keyword,
    ACTIONS(4429), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4469), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1558), 1,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat2,
    STATE(2158), 1,
      sym_preproc_else_in_release_event_declarator,
    STATE(1686), 2,
      sym_release_event_declarator,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [30276] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3997), 1,
      sym_data_event_keyword,
    ACTIONS(4381), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4471), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1616), 1,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat2,
    STATE(2205), 1,
      sym_preproc_else_in_data_event_declarator,
    STATE(1673), 2,
      sym_data_event_declarator,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [30299] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4113), 1,
      sym_timeline_event_keyword,
    ACTIONS(4439), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4473), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1660), 1,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2,
    STATE(2233), 1,
      sym_preproc_else_in_timeline_event_declarator,
    STATE(1692), 2,
      sym_timeline_event_declarator,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [30322] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3979), 1,
      sym_button_event_keyword,
    ACTIONS(4391), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(4443), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1560), 1,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat2,
    STATE(2157), 1,
      sym_preproc_else_in_button_event_declarator,
    STATE(1671), 2,
      sym_button_event_declarator,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [30345] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4475), 6,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30357] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4477), 6,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30369] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4479), 1,
      sym_push_keyword,
    STATE(1552), 1,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat2,
    ACTIONS(4482), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1691), 2,
      sym_push_event_declarator,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [30387] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4484), 6,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30399] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4486), 6,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30411] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4488), 6,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30423] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4490), 6,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30435] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4492), 6,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30447] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4494), 1,
      sym_release_keyword,
    STATE(1558), 1,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat2,
    ACTIONS(4497), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1686), 2,
      sym_release_event_declarator,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [30465] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4499), 6,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30477] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4501), 1,
      sym_button_event_keyword,
    STATE(1560), 1,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat2,
    ACTIONS(4504), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1671), 2,
      sym_button_event_declarator,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [30495] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4506), 6,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30507] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4510), 1,
      anon_sym_LPAREN,
    ACTIONS(4508), 5,
      sym__automatic_semicolon,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [30521] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4516), 1,
      anon_sym_LPAREN,
    ACTIONS(4512), 2,
      anon_sym_EQ,
      anon_sym_LBRACK,
    ACTIONS(4514), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [30537] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1921), 1,
      anon_sym_LPAREN,
    ACTIONS(1923), 5,
      sym__automatic_semicolon,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [30551] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4518), 6,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30563] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4520), 6,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30575] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4245), 1,
      sym_identifier,
    ACTIONS(4247), 1,
      anon_sym_LPAREN,
    STATE(1627), 1,
      sym__field_declarator,
    STATE(1706), 3,
      sym_array_field_declarator,
      sym_parenthesized_field_declarator,
      sym_function_field_declarator,
  [30593] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4522), 6,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30605] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4524), 6,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30617] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4526), 6,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30629] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4528), 6,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30641] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4530), 6,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30653] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4532), 6,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30665] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4534), 6,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30677] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4536), 6,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30689] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4538), 6,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30701] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4540), 6,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30713] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4542), 6,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30725] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4544), 6,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30737] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4546), 6,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30749] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4548), 6,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30761] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4550), 6,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30773] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4552), 1,
      sym_level_event_keyword,
    STATE(1583), 1,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat2,
    ACTIONS(4555), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1670), 2,
      sym_level_event_declarator,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [30791] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4557), 6,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30803] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4559), 6,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30815] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4561), 6,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30827] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4563), 6,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30839] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2133), 1,
      anon_sym_LPAREN,
    ACTIONS(2135), 5,
      sym__automatic_semicolon,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [30853] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4565), 6,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30865] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4567), 6,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30877] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4569), 6,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30889] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4571), 6,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30901] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4573), 6,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30913] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4575), 6,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30925] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4577), 6,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30937] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4579), 6,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30949] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4581), 6,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30961] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1891), 1,
      anon_sym_LPAREN,
    ACTIONS(1893), 5,
      sym__automatic_semicolon,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [30975] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4583), 6,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30987] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4585), 6,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [30999] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4587), 6,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31011] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4589), 6,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31023] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4591), 6,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31035] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4593), 6,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31047] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4595), 6,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31059] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4597), 6,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31071] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4599), 6,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31083] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4601), 6,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31095] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4603), 6,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31107] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4605), 6,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31119] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4607), 6,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31131] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4609), 6,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31143] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4611), 6,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31155] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4613), 6,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31167] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4615), 6,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31179] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4617), 1,
      sym_data_event_keyword,
    STATE(1616), 1,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat2,
    ACTIONS(4620), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1673), 2,
      sym_data_event_declarator,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [31197] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4622), 6,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31209] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4624), 6,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31221] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4626), 6,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31233] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4628), 1,
      sym_channel_event_keyword,
    STATE(1620), 1,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2,
    ACTIONS(4631), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1675), 2,
      sym_channel_event_declarator,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [31251] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4633), 6,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31263] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4635), 6,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31275] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4637), 6,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31287] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4639), 6,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31299] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4641), 6,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31311] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4643), 6,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31323] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4403), 1,
      anon_sym_LBRACK,
    ACTIONS(4405), 1,
      anon_sym_LPAREN,
    STATE(1672), 1,
      sym_parameter_list,
    ACTIONS(4645), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [31341] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4647), 6,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31353] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4649), 6,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31365] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4651), 6,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31377] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4653), 6,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31389] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4655), 1,
      sym_custom_event_keyword,
    STATE(1632), 1,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2,
    ACTIONS(4658), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1682), 2,
      sym_custom_event_declarator,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [31407] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4660), 6,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31419] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4662), 6,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31431] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4664), 6,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31443] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4666), 6,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31455] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4668), 6,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31467] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4670), 6,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31479] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4674), 1,
      anon_sym_LPAREN,
    ACTIONS(4672), 5,
      sym__automatic_semicolon,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [31493] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4676), 6,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31505] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4197), 1,
      sym_identifier,
    ACTIONS(4203), 1,
      anon_sym_LPAREN,
    STATE(1720), 1,
      sym__declarator,
    STATE(1705), 3,
      sym_array_declarator,
      sym_function_declarator,
      sym_parenthesized_declarator,
  [31523] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4678), 6,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31535] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4680), 6,
      sym_button_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31547] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4682), 6,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31559] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4684), 6,
      sym_level_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31571] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4686), 6,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31583] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4688), 6,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31595] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4690), 6,
      sym_channel_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31607] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4692), 6,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31619] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4694), 6,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31631] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4696), 6,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31643] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4698), 6,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31655] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4700), 6,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31667] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4702), 6,
      sym_release_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31679] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4706), 1,
      anon_sym_LPAREN,
    ACTIONS(4704), 5,
      sym__automatic_semicolon,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [31693] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4708), 6,
      sym_data_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31705] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4710), 6,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31717] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4712), 6,
      sym_push_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31729] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4714), 6,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31741] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4716), 1,
      sym_timeline_event_keyword,
    STATE(1660), 1,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2,
    ACTIONS(4719), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1692), 2,
      sym_timeline_event_declarator,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [31759] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4721), 6,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31771] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4197), 1,
      sym_identifier,
    ACTIONS(4203), 1,
      anon_sym_LPAREN,
    STATE(1795), 1,
      sym__declarator,
    STATE(1705), 3,
      sym_array_declarator,
      sym_function_declarator,
      sym_parenthesized_declarator,
  [31789] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4723), 6,
      sym_timeline_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31801] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4725), 1,
      anon_sym_LPAREN,
    ACTIONS(3931), 2,
      anon_sym_EQ,
      anon_sym_LBRACK,
    ACTIONS(3933), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [31817] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4727), 6,
      sym_custom_event_keyword,
      aux_sym_preproc_include_token1,
      aux_sym_preproc_define_token1,
      aux_sym_preproc_if_defined_token1,
      aux_sym_preproc_if_not_defined_token1,
      anon_sym_LBRACE,
  [31829] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4729), 1,
      sym_button_event_keyword,
    ACTIONS(4732), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1666), 2,
      sym_button_event_declarator,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [31844] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3997), 1,
      sym_data_event_keyword,
    ACTIONS(4734), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1674), 1,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat2,
    STATE(1673), 2,
      sym_data_event_declarator,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [31861] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3979), 1,
      sym_button_event_keyword,
    ACTIONS(4736), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1701), 1,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat2,
    STATE(1671), 2,
      sym_button_event_declarator,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [31878] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4005), 1,
      sym_level_event_keyword,
    ACTIONS(4738), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1700), 1,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat2,
    STATE(1670), 2,
      sym_level_event_declarator,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [31895] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4740), 1,
      sym_level_event_keyword,
    ACTIONS(4743), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1697), 2,
      sym_level_event_declarator,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [31910] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4745), 1,
      sym_button_event_keyword,
    ACTIONS(4748), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1666), 2,
      sym_button_event_declarator,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [31925] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4752), 1,
      anon_sym_LPAREN,
    ACTIONS(4750), 4,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [31938] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4754), 1,
      sym_data_event_keyword,
    ACTIONS(4757), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1679), 2,
      sym_data_event_declarator,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [31953] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3997), 1,
      sym_data_event_keyword,
    ACTIONS(4759), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1616), 1,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat2,
    STATE(1673), 2,
      sym_data_event_declarator,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [31970] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4761), 1,
      sym_channel_event_keyword,
    ACTIONS(4764), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1678), 2,
      sym_channel_event_declarator,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [31985] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4021), 1,
      sym_channel_event_keyword,
    ACTIONS(4766), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1620), 1,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2,
    STATE(1675), 2,
      sym_channel_event_declarator,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [32002] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4768), 1,
      sym_push_keyword,
    ACTIONS(4771), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1677), 2,
      sym_push_event_declarator,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [32017] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4773), 1,
      sym_channel_event_keyword,
    ACTIONS(4776), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1678), 2,
      sym_channel_event_declarator,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [32032] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4778), 1,
      sym_data_event_keyword,
    ACTIONS(4781), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1679), 2,
      sym_data_event_declarator,
      aux_sym_preproc_if_defined_in_data_event_declarator_repeat1,
  [32047] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4783), 1,
      sym_release_keyword,
    ACTIONS(4786), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1680), 2,
      sym_release_event_declarator,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [32062] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4013), 1,
      sym_custom_event_keyword,
    ACTIONS(4788), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1699), 1,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2,
    STATE(1682), 2,
      sym_custom_event_declarator,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [32079] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4790), 1,
      sym_custom_event_keyword,
    ACTIONS(4793), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1703), 2,
      sym_custom_event_declarator,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [32094] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4797), 1,
      anon_sym_LPAREN,
    ACTIONS(4795), 4,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [32107] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4801), 1,
      anon_sym_LPAREN,
    ACTIONS(4799), 4,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [32120] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3937), 1,
      sym_release_keyword,
    ACTIONS(4803), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1702), 1,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat2,
    STATE(1686), 2,
      sym_release_event_declarator,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [32137] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4805), 1,
      sym_release_keyword,
    ACTIONS(4808), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1680), 2,
      sym_release_event_declarator,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [32152] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4113), 1,
      sym_timeline_event_keyword,
    ACTIONS(4810), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1695), 1,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2,
    STATE(1692), 2,
      sym_timeline_event_declarator,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [32169] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4050), 1,
      sym_push_keyword,
    ACTIONS(4812), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1552), 1,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat2,
    STATE(1691), 2,
      sym_push_event_declarator,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [32186] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4050), 1,
      sym_push_keyword,
    ACTIONS(4814), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1688), 1,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat2,
    STATE(1691), 2,
      sym_push_event_declarator,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [32203] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4818), 1,
      anon_sym_LPAREN,
    ACTIONS(4816), 4,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [32216] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4820), 1,
      sym_push_keyword,
    ACTIONS(4823), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1677), 2,
      sym_push_event_declarator,
      aux_sym_preproc_if_defined_in_push_event_declarator_repeat1,
  [32231] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4825), 1,
      sym_timeline_event_keyword,
    ACTIONS(4828), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1698), 2,
      sym_timeline_event_declarator,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [32246] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4832), 1,
      anon_sym_LPAREN,
    ACTIONS(4830), 4,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [32259] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4836), 1,
      anon_sym_LPAREN,
    ACTIONS(4834), 4,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [32272] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4113), 1,
      sym_timeline_event_keyword,
    ACTIONS(4838), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1660), 1,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2,
    STATE(1692), 2,
      sym_timeline_event_declarator,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [32289] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    ACTIONS(4842), 1,
      anon_sym_LBRACK,
    STATE(1772), 1,
      sym_parameter_list,
    ACTIONS(4840), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [32306] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4844), 1,
      sym_level_event_keyword,
    ACTIONS(4847), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1697), 2,
      sym_level_event_declarator,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [32321] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4849), 1,
      sym_timeline_event_keyword,
    ACTIONS(4852), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1698), 2,
      sym_timeline_event_declarator,
      aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1,
  [32336] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4013), 1,
      sym_custom_event_keyword,
    ACTIONS(4854), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1632), 1,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2,
    STATE(1682), 2,
      sym_custom_event_declarator,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [32353] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4005), 1,
      sym_level_event_keyword,
    ACTIONS(4856), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1583), 1,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat2,
    STATE(1670), 2,
      sym_level_event_declarator,
      aux_sym_preproc_if_defined_in_level_event_declarator_repeat1,
  [32370] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3979), 1,
      sym_button_event_keyword,
    ACTIONS(4858), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1560), 1,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat2,
    STATE(1671), 2,
      sym_button_event_declarator,
      aux_sym_preproc_if_defined_in_button_event_declarator_repeat1,
  [32387] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3937), 1,
      sym_release_keyword,
    ACTIONS(4860), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1558), 1,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat2,
    STATE(1686), 2,
      sym_release_event_declarator,
      aux_sym_preproc_if_defined_in_release_event_declarator_repeat1,
  [32404] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4862), 1,
      sym_custom_event_keyword,
    ACTIONS(4865), 2,
      aux_sym_preproc_if_defined_token2,
      aux_sym_preproc_else_token1,
    STATE(1703), 2,
      sym_custom_event_declarator,
      aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1,
  [32419] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4021), 1,
      sym_channel_event_keyword,
    ACTIONS(4867), 1,
      aux_sym_preproc_if_defined_token2,
    STATE(1676), 1,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2,
    STATE(1675), 2,
      sym_channel_event_declarator,
      aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1,
  [32436] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4725), 1,
      anon_sym_LPAREN,
    ACTIONS(3931), 4,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [32449] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4871), 1,
      anon_sym_LPAREN,
    ACTIONS(4869), 4,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_SEMI,
  [32462] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4873), 1,
      anon_sym_SQUOTE,
    ACTIONS(4875), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4878), 1,
      sym_escape_sequence,
    STATE(1707), 1,
      aux_sym_string_literal_repeat1,
  [32478] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4405), 1,
      anon_sym_LPAREN,
    ACTIONS(4881), 1,
      anon_sym_EQ,
    ACTIONS(4883), 1,
      anon_sym_LBRACK,
    STATE(1563), 1,
      sym_parameter_list,
  [32494] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    STATE(984), 1,
      sym_parameter_list,
    ACTIONS(4219), 2,
      anon_sym_LBRACK,
      sym_identifier,
  [32508] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    STATE(1722), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(4887), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [32522] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4889), 1,
      anon_sym_SEMI,
    ACTIONS(4891), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [32538] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4893), 1,
      anon_sym_SEMI,
    ACTIONS(4895), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [32554] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4897), 1,
      anon_sym_SEMI,
    ACTIONS(4899), 1,
      sym__automatic_semicolon,
    STATE(1723), 1,
      aux_sym_declaration_repeat1,
  [32570] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4901), 1,
      anon_sym_SEMI,
    ACTIONS(4903), 1,
      sym__automatic_semicolon,
    STATE(1724), 1,
      aux_sym_declaration_repeat1,
  [32586] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4905), 1,
      anon_sym_SEMI,
    ACTIONS(4907), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [32602] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4909), 1,
      anon_sym_SEMI,
    ACTIONS(4911), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [32618] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4913), 1,
      anon_sym_SEMI,
    ACTIONS(4915), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [32634] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    ACTIONS(4917), 1,
      anon_sym_LBRACK,
    ACTIONS(4919), 1,
      anon_sym_RPAREN,
    STATE(1857), 1,
      sym_parameter_list,
  [32650] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4921), 1,
      anon_sym_SEMI,
    ACTIONS(4923), 1,
      sym__automatic_semicolon,
    STATE(1746), 1,
      aux_sym_declaration_repeat1,
  [32666] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    ACTIONS(4842), 1,
      anon_sym_LBRACK,
    ACTIONS(4925), 1,
      anon_sym_RPAREN,
    STATE(1772), 1,
      sym_parameter_list,
  [32682] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    STATE(1756), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(4927), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [32696] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4929), 1,
      anon_sym_SEMI,
    ACTIONS(4931), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [32712] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4933), 1,
      anon_sym_SEMI,
    ACTIONS(4935), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [32728] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4937), 1,
      anon_sym_SEMI,
    ACTIONS(4939), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [32744] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4941), 1,
      anon_sym_SEMI,
    ACTIONS(4943), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [32760] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    STATE(1754), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(4945), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [32774] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4947), 1,
      anon_sym_SEMI,
    ACTIONS(4949), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [32790] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4951), 1,
      anon_sym_SEMI,
    ACTIONS(4953), 1,
      sym__automatic_semicolon,
    STATE(1759), 1,
      aux_sym_declaration_repeat1,
  [32806] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4955), 1,
      anon_sym_SQUOTE,
    ACTIONS(4957), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4959), 1,
      sym_escape_sequence,
    STATE(1733), 1,
      aux_sym_string_literal_repeat1,
  [32822] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4961), 1,
      anon_sym_SEMI,
    ACTIONS(4963), 1,
      sym__automatic_semicolon,
    STATE(1760), 1,
      aux_sym_declaration_repeat1,
  [32838] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4965), 1,
      anon_sym_SEMI,
    ACTIONS(4967), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [32854] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4969), 1,
      anon_sym_SEMI,
    ACTIONS(4971), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [32870] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4973), 1,
      anon_sym_SQUOTE,
    ACTIONS(4975), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4977), 1,
      sym_escape_sequence,
    STATE(1707), 1,
      aux_sym_string_literal_repeat1,
  [32886] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    STATE(1740), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(4979), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [32900] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    STATE(1741), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(4979), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [32914] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4981), 1,
      anon_sym_SEMI,
    ACTIONS(4983), 1,
      sym__automatic_semicolon,
    STATE(1744), 1,
      aux_sym_declaration_repeat1,
  [32930] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4985), 1,
      anon_sym_SEMI,
    ACTIONS(4987), 1,
      sym__automatic_semicolon,
    STATE(1745), 1,
      aux_sym_declaration_repeat1,
  [32946] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4405), 1,
      anon_sym_LPAREN,
    STATE(1864), 1,
      sym_parameter_list,
    ACTIONS(4219), 2,
      anon_sym_LBRACK,
      sym_identifier,
  [32960] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    STATE(1747), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(4989), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [32974] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4991), 1,
      anon_sym_SEMI,
    ACTIONS(4993), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [32990] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4995), 1,
      anon_sym_SEMI,
    ACTIONS(4997), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33006] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(4999), 1,
      anon_sym_SEMI,
    ACTIONS(5001), 1,
      sym__automatic_semicolon,
    STATE(1748), 1,
      aux_sym_declaration_repeat1,
  [33022] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5003), 1,
      anon_sym_SEMI,
    ACTIONS(5005), 1,
      sym__automatic_semicolon,
    STATE(1749), 1,
      aux_sym_declaration_repeat1,
  [33038] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5007), 1,
      anon_sym_SEMI,
    ACTIONS(5009), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33054] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5011), 1,
      anon_sym_SEMI,
    ACTIONS(5013), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33070] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5015), 1,
      anon_sym_SEMI,
    ACTIONS(5017), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33086] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5019), 1,
      anon_sym_SEMI,
    ACTIONS(5021), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33102] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5023), 1,
      anon_sym_SEMI,
    ACTIONS(5025), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33118] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5027), 1,
      anon_sym_SEMI,
    ACTIONS(5029), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33134] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(5031), 1,
      anon_sym_SQUOTE,
    ACTIONS(5033), 1,
      aux_sym_string_literal_token1,
    ACTIONS(5035), 1,
      sym_escape_sequence,
    STATE(1752), 1,
      aux_sym_string_literal_repeat1,
  [33150] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4401), 1,
      anon_sym_COMMA,
    STATE(1755), 1,
      aux_sym__field_declaration_declarator_repeat1,
    ACTIONS(5037), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33164] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4975), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4977), 1,
      sym_escape_sequence,
    ACTIONS(5039), 1,
      anon_sym_SQUOTE,
    STATE(1707), 1,
      aux_sym_string_literal_repeat1,
  [33180] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5041), 1,
      anon_sym_SEMI,
    ACTIONS(5043), 1,
      sym__automatic_semicolon,
    STATE(1789), 1,
      aux_sym_declaration_repeat1,
  [33196] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5045), 1,
      anon_sym_SEMI,
    ACTIONS(5047), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33212] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5049), 1,
      anon_sym_COMMA,
    STATE(1755), 1,
      aux_sym__field_declaration_declarator_repeat1,
    ACTIONS(5052), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33226] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5054), 1,
      anon_sym_SEMI,
    ACTIONS(5056), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33242] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    STATE(1951), 1,
      sym_argument_list,
    ACTIONS(5058), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33256] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4975), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4977), 1,
      sym_escape_sequence,
    ACTIONS(5060), 1,
      anon_sym_SQUOTE,
    STATE(1707), 1,
      aux_sym_string_literal_repeat1,
  [33272] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5062), 1,
      anon_sym_SEMI,
    ACTIONS(5064), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33288] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5066), 1,
      anon_sym_SEMI,
    ACTIONS(5068), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33304] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    ACTIONS(4917), 1,
      anon_sym_LBRACK,
    ACTIONS(5070), 1,
      anon_sym_RPAREN,
    STATE(1857), 1,
      sym_parameter_list,
  [33320] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(5072), 1,
      anon_sym_SQUOTE,
    ACTIONS(5074), 1,
      aux_sym_string_literal_token1,
    ACTIONS(5076), 1,
      sym_escape_sequence,
    STATE(1785), 1,
      aux_sym_string_literal_repeat1,
  [33336] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5078), 1,
      anon_sym_SEMI,
    ACTIONS(5080), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33352] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5082), 1,
      anon_sym_SEMI,
    ACTIONS(5084), 1,
      sym__automatic_semicolon,
    STATE(1787), 1,
      aux_sym_declaration_repeat1,
  [33368] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(5086), 1,
      anon_sym_SQUOTE,
    ACTIONS(5088), 1,
      aux_sym_string_literal_token1,
    ACTIONS(5090), 1,
      sym_escape_sequence,
    STATE(1766), 1,
      aux_sym_string_literal_repeat1,
  [33384] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4975), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4977), 1,
      sym_escape_sequence,
    ACTIONS(5092), 1,
      anon_sym_SQUOTE,
    STATE(1707), 1,
      aux_sym_string_literal_repeat1,
  [33400] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(5094), 1,
      anon_sym_SQUOTE,
    ACTIONS(5096), 1,
      aux_sym_string_literal_token1,
    ACTIONS(5098), 1,
      sym_escape_sequence,
    STATE(1768), 1,
      aux_sym_string_literal_repeat1,
  [33416] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4975), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4977), 1,
      sym_escape_sequence,
    ACTIONS(5100), 1,
      anon_sym_SQUOTE,
    STATE(1707), 1,
      aux_sym_string_literal_repeat1,
  [33432] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(5102), 1,
      anon_sym_SQUOTE,
    ACTIONS(5104), 1,
      aux_sym_string_literal_token1,
    ACTIONS(5106), 1,
      sym_escape_sequence,
    STATE(1770), 1,
      aux_sym_string_literal_repeat1,
  [33448] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4975), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4977), 1,
      sym_escape_sequence,
    ACTIONS(5108), 1,
      anon_sym_SQUOTE,
    STATE(1707), 1,
      aux_sym_string_literal_repeat1,
  [33464] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5110), 1,
      anon_sym_SEMI,
    ACTIONS(5112), 1,
      sym__automatic_semicolon,
    STATE(1763), 1,
      aux_sym_declaration_repeat1,
  [33480] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4516), 1,
      anon_sym_LPAREN,
    ACTIONS(4512), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [33492] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5114), 1,
      anon_sym_SEMI,
    ACTIONS(5116), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33508] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    STATE(989), 1,
      sym_parameter_list,
    ACTIONS(4219), 2,
      anon_sym_LBRACK,
      sym_identifier,
  [33522] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4405), 1,
      anon_sym_LPAREN,
    STATE(1818), 1,
      sym_parameter_list,
    ACTIONS(4219), 2,
      anon_sym_LBRACK,
      sym_identifier,
  [33536] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    STATE(1955), 1,
      sym_argument_list,
    ACTIONS(5118), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33550] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    STATE(1956), 1,
      sym_argument_list,
    ACTIONS(5120), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33564] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(5122), 1,
      anon_sym_SQUOTE,
    ACTIONS(5124), 1,
      aux_sym_string_literal_token1,
    ACTIONS(5126), 1,
      sym_escape_sequence,
    STATE(1786), 1,
      aux_sym_string_literal_repeat1,
  [33580] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5128), 1,
      anon_sym_COMMA,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(5131), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33594] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4674), 1,
      anon_sym_LPAREN,
    ACTIONS(4672), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [33606] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4706), 1,
      anon_sym_LPAREN,
    ACTIONS(4704), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [33618] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4510), 1,
      anon_sym_LPAREN,
    ACTIONS(4508), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [33630] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    STATE(1894), 1,
      sym_argument_list,
    ACTIONS(5133), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33644] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    STATE(1895), 1,
      sym_argument_list,
    ACTIONS(5135), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33658] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4975), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4977), 1,
      sym_escape_sequence,
    ACTIONS(5137), 1,
      anon_sym_SQUOTE,
    STATE(1707), 1,
      aux_sym_string_literal_repeat1,
  [33674] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4975), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4977), 1,
      sym_escape_sequence,
    ACTIONS(5139), 1,
      anon_sym_SQUOTE,
    STATE(1707), 1,
      aux_sym_string_literal_repeat1,
  [33690] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5141), 1,
      anon_sym_SEMI,
    ACTIONS(5143), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33706] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    STATE(1711), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(5145), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33720] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5147), 1,
      anon_sym_SEMI,
    ACTIONS(5149), 1,
      sym__automatic_semicolon,
    STATE(1779), 1,
      aux_sym_declaration_repeat1,
  [33736] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    STATE(1000), 1,
      sym_parameter_list,
    ACTIONS(4219), 2,
      anon_sym_LBRACK,
      sym_identifier,
  [33750] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4405), 1,
      anon_sym_LPAREN,
    STATE(1870), 1,
      sym_parameter_list,
    ACTIONS(4219), 2,
      anon_sym_LBRACK,
      sym_identifier,
  [33764] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(5151), 1,
      anon_sym_SQUOTE,
    ACTIONS(5153), 1,
      aux_sym_string_literal_token1,
    ACTIONS(5155), 1,
      sym_escape_sequence,
    STATE(1798), 1,
      aux_sym_string_literal_repeat1,
  [33780] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    STATE(1712), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(5145), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33794] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5157), 1,
      anon_sym_SEMI,
    ACTIONS(5159), 1,
      sym__automatic_semicolon,
    STATE(1715), 1,
      aux_sym_declaration_repeat1,
  [33810] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    ACTIONS(4842), 1,
      anon_sym_LBRACK,
    ACTIONS(5161), 1,
      anon_sym_RPAREN,
    STATE(1772), 1,
      sym_parameter_list,
  [33826] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    STATE(1773), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(5163), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33840] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5165), 1,
      anon_sym_SEMI,
    ACTIONS(5167), 1,
      sym__automatic_semicolon,
    STATE(1716), 1,
      aux_sym_declaration_repeat1,
  [33856] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(4975), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4977), 1,
      sym_escape_sequence,
    ACTIONS(5169), 1,
      anon_sym_SQUOTE,
    STATE(1707), 1,
      aux_sym_string_literal_repeat1,
  [33872] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    STATE(1725), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(5171), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33886] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    STATE(1727), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(5171), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33900] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    STATE(1945), 1,
      sym_argument_list,
    ACTIONS(5173), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33914] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5175), 1,
      anon_sym_SEMI,
    ACTIONS(5177), 1,
      sym__automatic_semicolon,
    STATE(1731), 1,
      aux_sym_declaration_repeat1,
  [33930] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    ACTIONS(5179), 1,
      anon_sym_SEMI,
    ACTIONS(5181), 1,
      sym__automatic_semicolon,
    STATE(1732), 1,
      aux_sym_declaration_repeat1,
  [33946] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_COMMA,
    STATE(1717), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(4945), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [33960] = 5,
    ACTIONS(287), 1,
      sym_comment,
    ACTIONS(5183), 1,
      anon_sym_SQUOTE,
    ACTIONS(5185), 1,
      aux_sym_string_literal_token1,
    ACTIONS(5187), 1,
      sym_escape_sequence,
    STATE(1758), 1,
      aux_sym_string_literal_repeat1,
  [33976] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5189), 1,
      anon_sym_COMMA,
    ACTIONS(5191), 1,
      anon_sym_RPAREN,
    STATE(1867), 1,
      aux_sym_parameter_list_repeat1,
  [33989] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(5193), 1,
      anon_sym_RPAREN,
    STATE(1875), 1,
      aux_sym_argument_list_repeat1,
  [34002] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5195), 1,
      sym_preproc_arg,
    ACTIONS(5197), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34013] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5199), 1,
      anon_sym_LBRACK,
    STATE(1496), 1,
      sym_channel_event_devchan_reference,
    STATE(1497), 1,
      sym_devchan_expression,
  [34026] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5201), 1,
      anon_sym_COMMA,
    ACTIONS(5204), 1,
      anon_sym_RPAREN,
    STATE(1810), 1,
      aux_sym_parameter_list_repeat1,
  [34039] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(3247), 1,
      anon_sym_RPAREN,
    STATE(1821), 1,
      aux_sym_argument_list_repeat1,
  [34052] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2159), 1,
      anon_sym_SQUOTE,
    STATE(305), 1,
      sym_call_definition,
    STATE(977), 1,
      sym_string_literal,
  [34065] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(3237), 1,
      anon_sym_RPAREN,
    STATE(1865), 1,
      aux_sym_argument_list_repeat1,
  [34078] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5189), 1,
      anon_sym_COMMA,
    ACTIONS(5206), 1,
      anon_sym_RPAREN,
    STATE(1823), 1,
      aux_sym_parameter_list_repeat1,
  [34091] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5189), 1,
      anon_sym_COMMA,
    ACTIONS(5208), 1,
      anon_sym_RPAREN,
    STATE(1866), 1,
      aux_sym_parameter_list_repeat1,
  [34104] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(3243), 1,
      anon_sym_RPAREN,
    STATE(1830), 1,
      aux_sym_argument_list_repeat1,
  [34117] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3231), 1,
      sym_range_operator,
    ACTIONS(5210), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34128] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5212), 1,
      anon_sym_EQ,
    ACTIONS(5214), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34139] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5189), 1,
      anon_sym_COMMA,
    ACTIONS(5216), 1,
      anon_sym_RPAREN,
    STATE(1835), 1,
      aux_sym_parameter_list_repeat1,
  [34152] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5218), 1,
      anon_sym_SQUOTE,
    STATE(307), 1,
      sym_module_definition,
    STATE(2238), 1,
      sym_string_literal,
  [34165] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(5220), 1,
      anon_sym_RPAREN,
    STATE(1875), 1,
      aux_sym_argument_list_repeat1,
  [34178] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(153), 1,
      anon_sym_LBRACK,
    STATE(1877), 1,
      sym_devchan_expression,
    STATE(1942), 1,
      sym_devchan_range_expression,
  [34191] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5189), 1,
      anon_sym_COMMA,
    ACTIONS(5222), 1,
      anon_sym_RPAREN,
    STATE(1810), 1,
      aux_sym_parameter_list_repeat1,
  [34204] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3200), 1,
      anon_sym_COMMA,
    ACTIONS(5224), 1,
      anon_sym_DQUOTE,
    STATE(1837), 1,
      aux_sym_string_expression_repeat1,
  [34217] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5226), 1,
      anon_sym_LBRACK,
    STATE(1460), 1,
      sym_level_event_devlev_reference,
    STATE(1461), 1,
      sym_devlev_expression,
  [34230] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5228), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [34239] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5230), 1,
      anon_sym_LBRACK,
    STATE(1494), 1,
      sym_button_event_devchan_reference,
    STATE(1495), 1,
      sym_devchan_expression,
  [34252] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5232), 1,
      sym_preproc_arg,
    ACTIONS(5234), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34263] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4871), 1,
      anon_sym_LPAREN,
    ACTIONS(4869), 2,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [34274] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(5236), 1,
      anon_sym_RPAREN,
    STATE(1875), 1,
      aux_sym_argument_list_repeat1,
  [34287] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5218), 1,
      anon_sym_SQUOTE,
    STATE(404), 1,
      sym_module_definition,
    STATE(2168), 1,
      sym_string_literal,
  [34300] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5238), 1,
      anon_sym_EQ,
    ACTIONS(5240), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34311] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5242), 1,
      sym_preproc_arg,
    ACTIONS(5244), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34322] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3224), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [34331] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5189), 1,
      anon_sym_COMMA,
    ACTIONS(5246), 1,
      anon_sym_RPAREN,
    STATE(1810), 1,
      aux_sym_parameter_list_repeat1,
  [34344] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3200), 1,
      anon_sym_COMMA,
    ACTIONS(5248), 1,
      anon_sym_DQUOTE,
    STATE(1837), 1,
      aux_sym_string_expression_repeat1,
  [34357] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3265), 1,
      anon_sym_DQUOTE,
    ACTIONS(5250), 1,
      anon_sym_COMMA,
    STATE(1837), 1,
      aux_sym_string_expression_repeat1,
  [34370] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5253), 1,
      anon_sym_LBRACK,
    STATE(1488), 1,
      sym_release_event_devchan_reference,
    STATE(1490), 1,
      sym_devchan_expression,
  [34383] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(3251), 1,
      anon_sym_RPAREN,
    STATE(1848), 1,
      aux_sym_argument_list_repeat1,
  [34396] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2159), 1,
      anon_sym_SQUOTE,
    STATE(401), 1,
      sym_call_definition,
    STATE(978), 1,
      sym_string_literal,
  [34409] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5255), 1,
      sym_preproc_arg,
    ACTIONS(5257), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34420] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5259), 3,
      sym__automatic_semicolon,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [34429] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5261), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [34438] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4818), 1,
      anon_sym_LPAREN,
    ACTIONS(4816), 2,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [34449] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5263), 1,
      anon_sym_LBRACK,
    STATE(1471), 1,
      sym_push_event_devchan_reference,
    STATE(1472), 1,
      sym_devchan_expression,
  [34462] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5265), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [34471] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2159), 1,
      anon_sym_SQUOTE,
    STATE(547), 1,
      sym_call_definition,
    STATE(976), 1,
      sym_string_literal,
  [34484] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(5267), 1,
      anon_sym_RPAREN,
    STATE(1875), 1,
      aux_sym_argument_list_repeat1,
  [34497] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4832), 1,
      anon_sym_LPAREN,
    ACTIONS(4830), 2,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [34508] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5269), 1,
      sym_preproc_arg,
    ACTIONS(5271), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34519] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(153), 1,
      anon_sym_LBRACK,
    STATE(1863), 1,
      sym_devchan_expression,
    STATE(1890), 1,
      sym_devchan_range_expression,
  [34532] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5218), 1,
      anon_sym_SQUOTE,
    STATE(549), 1,
      sym_module_definition,
    STATE(2224), 1,
      sym_string_literal,
  [34545] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4801), 1,
      anon_sym_LPAREN,
    ACTIONS(4799), 2,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [34556] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5273), 1,
      anon_sym_EQ,
    ACTIONS(5275), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34567] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3200), 1,
      anon_sym_COMMA,
    ACTIONS(5277), 1,
      anon_sym_DQUOTE,
    STATE(1837), 1,
      aux_sym_string_expression_repeat1,
  [34580] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4797), 1,
      anon_sym_LPAREN,
    ACTIONS(4795), 2,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [34591] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4752), 1,
      anon_sym_LPAREN,
    ACTIONS(4750), 2,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [34602] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5279), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [34611] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(5281), 1,
      anon_sym_RPAREN,
    STATE(1875), 1,
      aux_sym_argument_list_repeat1,
  [34624] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5283), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [34633] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5285), 3,
      anon_sym_COMMA,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [34642] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5287), 1,
      anon_sym_LBRACK,
    ACTIONS(4840), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [34653] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3231), 1,
      sym_range_operator,
    ACTIONS(5289), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34664] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5291), 1,
      anon_sym_EQ,
    ACTIONS(5293), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34675] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(5295), 1,
      anon_sym_RPAREN,
    STATE(1875), 1,
      aux_sym_argument_list_repeat1,
  [34688] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5189), 1,
      anon_sym_COMMA,
    ACTIONS(5297), 1,
      anon_sym_RPAREN,
    STATE(1810), 1,
      aux_sym_parameter_list_repeat1,
  [34701] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5189), 1,
      anon_sym_COMMA,
    ACTIONS(5299), 1,
      anon_sym_RPAREN,
    STATE(1810), 1,
      aux_sym_parameter_list_repeat1,
  [34714] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3200), 1,
      anon_sym_COMMA,
    ACTIONS(5301), 1,
      anon_sym_DQUOTE,
    STATE(1837), 1,
      aux_sym_string_expression_repeat1,
  [34727] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4836), 1,
      anon_sym_LPAREN,
    ACTIONS(4834), 2,
      anon_sym_LBRACK,
      anon_sym_RPAREN,
  [34738] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5303), 1,
      anon_sym_EQ,
    ACTIONS(5305), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34749] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3200), 1,
      anon_sym_COMMA,
    ACTIONS(5307), 1,
      anon_sym_DQUOTE,
    STATE(1837), 1,
      aux_sym_string_expression_repeat1,
  [34762] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(3255), 1,
      anon_sym_RPAREN,
    STATE(1807), 1,
      aux_sym_argument_list_repeat1,
  [34775] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5309), 1,
      anon_sym_EQ,
    ACTIONS(5311), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34786] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3173), 1,
      anon_sym_COMMA,
    ACTIONS(3175), 1,
      anon_sym_RPAREN,
    STATE(1859), 1,
      aux_sym_argument_list_repeat1,
  [34799] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3350), 1,
      anon_sym_RPAREN,
    ACTIONS(5313), 1,
      anon_sym_COMMA,
    STATE(1875), 1,
      aux_sym_argument_list_repeat1,
  [34812] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3200), 1,
      anon_sym_COMMA,
    ACTIONS(5316), 1,
      anon_sym_DQUOTE,
    STATE(1837), 1,
      aux_sym_string_expression_repeat1,
  [34825] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3231), 1,
      sym_range_operator,
    ACTIONS(5318), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34836] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(153), 1,
      anon_sym_LBRACK,
    STATE(1817), 1,
      sym_devchan_expression,
    STATE(1992), 1,
      sym_devchan_range_expression,
  [34849] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5320), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34857] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5322), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34865] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5324), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34873] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5326), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34881] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5328), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34889] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5330), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34897] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5332), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34905] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5334), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34913] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    STATE(1434), 1,
      sym_compound_statement,
  [34923] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1979), 1,
      sym_string_literal,
  [34933] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5336), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34941] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5289), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34949] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2693), 1,
      anon_sym_LBRACK,
    STATE(139), 1,
      sym_devchan_expression,
  [34959] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5338), 1,
      anon_sym_SEMI,
    ACTIONS(5340), 1,
      sym__automatic_semicolon,
  [34969] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5342), 1,
      anon_sym_LBRACE,
    STATE(432), 1,
      sym_data_event_block,
  [34979] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5344), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34987] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5346), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [34995] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5348), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35003] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5350), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35011] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1048), 1,
      anon_sym_LBRACE,
    STATE(516), 1,
      sym_compound_statement,
  [35021] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5352), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35029] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5354), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35037] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1935), 1,
      sym_string_literal,
  [35047] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5356), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35055] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5358), 1,
      anon_sym_LBRACE,
    STATE(312), 1,
      sym_data_event_block,
  [35065] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    STATE(315), 1,
      sym_compound_statement,
  [35075] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5360), 1,
      anon_sym_LBRACE,
    STATE(333), 1,
      sym_button_event_block,
  [35085] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    STATE(272), 1,
      sym_compound_statement,
  [35095] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5362), 1,
      anon_sym_LBRACE,
    STATE(279), 1,
      sym_channel_event_block,
  [35105] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    STATE(281), 1,
      sym_compound_statement,
  [35115] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5364), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35123] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    STATE(282), 1,
      sym_compound_statement,
  [35133] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    STATE(283), 1,
      sym_compound_statement,
  [35143] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5366), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35151] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3064), 1,
      anon_sym_LPAREN,
    STATE(1091), 1,
      sym_argument_list,
  [35161] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3186), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35169] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5368), 1,
      anon_sym_SQUOTE,
    STATE(285), 1,
      sym_string_literal,
  [35179] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5368), 1,
      anon_sym_SQUOTE,
    STATE(244), 1,
      sym_string_literal,
  [35189] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5370), 1,
      anon_sym_LBRACE,
    STATE(434), 1,
      sym_button_event_block,
  [35199] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1048), 1,
      anon_sym_LBRACE,
    STATE(476), 1,
      sym_compound_statement,
  [35209] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5372), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35217] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3233), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35225] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5374), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35233] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5376), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35241] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2167), 1,
      anon_sym_LPAREN,
    STATE(593), 1,
      sym_argument_list,
  [35251] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5378), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35259] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5380), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35267] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5382), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35275] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5384), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35283] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5386), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35291] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5388), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35299] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5390), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35307] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5392), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35315] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5394), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35323] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5396), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35331] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2297), 1,
      anon_sym_LBRACK,
    STATE(621), 1,
      sym_devchan_expression,
  [35341] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5398), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35349] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5400), 1,
      sym_identifier,
    ACTIONS(5402), 1,
      anon_sym_LBRACK,
  [35359] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5404), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35367] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    STATE(435), 1,
      sym_compound_statement,
  [35377] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    STATE(433), 1,
      sym_compound_statement,
  [35387] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5406), 1,
      anon_sym_LBRACE,
    STATE(436), 1,
      sym_channel_event_block,
  [35397] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    STATE(438), 1,
      sym_compound_statement,
  [35407] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5318), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35415] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2593), 1,
      anon_sym_LBRACK,
    STATE(1122), 1,
      sym_devchan_expression,
  [35425] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    STATE(437), 1,
      sym_compound_statement,
  [35435] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5408), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35443] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1621), 1,
      anon_sym_LPAREN,
    STATE(171), 1,
      sym_argument_list,
  [35453] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5410), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35461] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5412), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35469] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5414), 1,
      anon_sym_LPAREN,
    STATE(535), 1,
      sym_parenthesized_expression,
  [35479] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3350), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [35487] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5416), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35495] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2639), 1,
      anon_sym_LBRACK,
    STATE(183), 1,
      sym_devchan_expression,
  [35505] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    STATE(298), 1,
      sym_compound_statement,
  [35515] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5418), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35523] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5420), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35531] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5422), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35539] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5424), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35547] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5426), 1,
      sym_repeat_keyword,
    STATE(2094), 1,
      sym_button_event_hold_repeat,
  [35557] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1377), 1,
      anon_sym_LPAREN,
    STATE(80), 1,
      sym_argument_list,
  [35567] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5428), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35575] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    STATE(445), 1,
      sym_compound_statement,
  [35585] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5430), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35593] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5204), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [35601] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2663), 1,
      anon_sym_LBRACK,
    STATE(124), 1,
      sym_devchan_expression,
  [35611] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(153), 1,
      anon_sym_LBRACK,
    STATE(1260), 1,
      sym_devchan_expression,
  [35621] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5432), 1,
      anon_sym_LBRACE,
    STATE(1411), 1,
      sym_compound_statement,
  [35631] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4233), 2,
      anon_sym_LBRACK,
      sym_identifier,
  [35639] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5434), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35647] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5414), 1,
      anon_sym_LPAREN,
    STATE(556), 1,
      sym_parenthesized_expression,
  [35657] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5414), 1,
      anon_sym_LPAREN,
    STATE(1918), 1,
      sym_parenthesized_expression,
  [35667] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5436), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35675] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1776), 1,
      sym_string_literal,
  [35685] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1777), 1,
      sym_string_literal,
  [35695] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1929), 1,
      sym_string_literal,
  [35705] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1930), 1,
      sym_string_literal,
  [35715] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1931), 1,
      sym_string_literal,
  [35725] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1932), 1,
      sym_string_literal,
  [35735] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5438), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35743] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5440), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35751] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3171), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35759] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4223), 2,
      anon_sym_LBRACK,
      sym_identifier,
  [35767] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    STATE(993), 1,
      sym_parameter_list,
  [35777] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4405), 1,
      anon_sym_LPAREN,
    STATE(1832), 1,
      sym_parameter_list,
  [35787] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    STATE(1968), 1,
      sym_argument_list,
  [35797] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5432), 1,
      anon_sym_LBRACE,
    STATE(1361), 1,
      sym_compound_statement,
  [35807] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    STATE(994), 1,
      sym_parameter_list,
  [35817] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4405), 1,
      anon_sym_LPAREN,
    STATE(1873), 1,
      sym_parameter_list,
  [35827] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    STATE(1926), 1,
      sym_argument_list,
  [35837] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    STATE(1203), 1,
      sym_argument_list,
  [35847] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3184), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35855] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5442), 1,
      anon_sym_LBRACK,
    STATE(1476), 1,
      sym_data_event_device_reference,
  [35865] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5210), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35873] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(155), 1,
      anon_sym_LBRACE,
    STATE(382), 1,
      sym_compound_statement,
  [35883] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5444), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35891] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5414), 1,
      anon_sym_LPAREN,
    STATE(530), 1,
      sym_parenthesized_expression,
  [35901] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3171), 2,
      anon_sym_RPAREN,
      anon_sym_SEMI,
  [35909] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5414), 1,
      anon_sym_LPAREN,
    STATE(1953), 1,
      sym_parenthesized_expression,
  [35919] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5414), 1,
      anon_sym_LPAREN,
    STATE(561), 1,
      sym_parenthesized_expression,
  [35929] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5414), 1,
      anon_sym_LPAREN,
    STATE(2032), 1,
      sym_parenthesized_expression,
  [35939] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5446), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35947] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5414), 1,
      anon_sym_LPAREN,
    STATE(562), 1,
      sym_parenthesized_expression,
  [35957] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1801), 1,
      sym_string_literal,
  [35967] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1757), 1,
      sym_string_literal,
  [35977] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3149), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [35985] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1783), 1,
      sym_string_literal,
  [35995] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1784), 1,
      sym_string_literal,
  [36005] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1971), 1,
      sym_string_literal,
  [36015] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1880), 1,
      sym_string_literal,
  [36025] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1881), 1,
      sym_string_literal,
  [36035] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1883), 1,
      sym_string_literal,
  [36045] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5448), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [36053] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    STATE(248), 1,
      sym_compound_statement,
  [36063] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2946), 1,
      anon_sym_LPAREN,
    STATE(1003), 1,
      sym_parameter_list,
  [36073] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4405), 1,
      anon_sym_LPAREN,
    STATE(1854), 1,
      sym_parameter_list,
  [36083] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3147), 1,
      anon_sym_LPAREN,
    STATE(1909), 1,
      sym_argument_list,
  [36093] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5450), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [36101] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5452), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [36109] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1925), 1,
      sym_string_literal,
  [36119] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1927), 1,
      sym_string_literal,
  [36129] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5454), 1,
      anon_sym_LBRACK,
    STATE(1462), 1,
      sym_timeline_event_id_reference,
  [36139] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5456), 1,
      anon_sym_SQUOTE,
    STATE(458), 1,
      sym_string_literal,
  [36149] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5458), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [36157] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5460), 1,
      anon_sym_LBRACE,
    STATE(1882), 1,
      sym_field_declaration_list,
  [36167] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1947), 1,
      sym_string_literal,
  [36177] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1433), 1,
      anon_sym_LPAREN,
    STATE(131), 1,
      sym_argument_list,
  [36187] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1919), 1,
      sym_string_literal,
  [36197] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3226), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [36205] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5414), 1,
      anon_sym_LPAREN,
    STATE(2114), 1,
      sym_parenthesized_expression,
  [36215] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5402), 1,
      anon_sym_LBRACK,
    ACTIONS(5462), 1,
      sym_identifier,
  [36225] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5402), 1,
      anon_sym_LBRACK,
    ACTIONS(5464), 1,
      sym_identifier,
  [36235] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5456), 1,
      anon_sym_SQUOTE,
    STATE(274), 1,
      sym_string_literal,
  [36245] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(323), 1,
      anon_sym_LBRACE,
    STATE(254), 1,
      sym_compound_statement,
  [36255] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5414), 1,
      anon_sym_LPAREN,
    STATE(2176), 1,
      sym_parenthesized_expression,
  [36265] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5466), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [36273] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5414), 1,
      anon_sym_LPAREN,
    STATE(2181), 1,
      sym_parenthesized_expression,
  [36283] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5402), 1,
      anon_sym_LBRACK,
    ACTIONS(5468), 1,
      sym_identifier,
  [36293] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5402), 1,
      anon_sym_LBRACK,
    ACTIONS(5470), 1,
      sym_identifier,
  [36303] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5472), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [36311] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5474), 1,
      anon_sym_LBRACK,
    STATE(1463), 1,
      sym_custom_event_reference,
  [36321] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(165), 1,
      anon_sym_SQUOTE,
    STATE(1885), 1,
      sym_string_literal,
  [36331] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4189), 2,
      anon_sym_LBRACK,
      sym_identifier,
  [36339] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3239), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [36347] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4179), 2,
      anon_sym_LBRACK,
      sym_identifier,
  [36355] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4193), 2,
      anon_sym_LBRACK,
      sym_identifier,
  [36363] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5476), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [36371] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5478), 2,
      sym__automatic_semicolon,
      anon_sym_SEMI,
  [36379] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5402), 1,
      anon_sym_LBRACK,
    ACTIONS(5480), 1,
      sym_identifier,
  [36389] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5414), 1,
      anon_sym_LPAREN,
    STATE(557), 1,
      sym_parenthesized_expression,
  [36399] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5482), 1,
      anon_sym_COLON,
  [36406] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5484), 1,
      anon_sym_COLON,
  [36413] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5486), 1,
      aux_sym_preproc_if_defined_token2,
  [36420] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3343), 1,
      anon_sym_RPAREN,
  [36427] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5488), 1,
      aux_sym_preproc_if_defined_token2,
  [36434] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5490), 1,
      aux_sym_preproc_if_defined_token2,
  [36441] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3335), 1,
      anon_sym_RPAREN,
  [36448] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5492), 1,
      aux_sym_preproc_if_defined_token2,
  [36455] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5494), 1,
      aux_sym_preproc_if_defined_token2,
  [36462] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5496), 1,
      aux_sym_preproc_if_defined_token2,
  [36469] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5498), 1,
      aux_sym_preproc_if_defined_token2,
  [36476] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5500), 1,
      aux_sym_preproc_if_defined_token2,
  [36483] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5502), 1,
      sym_identifier,
  [36490] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5504), 1,
      anon_sym_COLON,
  [36497] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5506), 1,
      aux_sym_preproc_if_defined_token2,
  [36504] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5508), 1,
      anon_sym_LPAREN,
  [36511] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5510), 1,
      aux_sym_preproc_if_defined_token2,
  [36518] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5512), 1,
      aux_sym_preproc_if_defined_token2,
  [36525] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5514), 1,
      aux_sym_preproc_if_defined_token2,
  [36532] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5516), 1,
      aux_sym_preproc_if_defined_token2,
  [36539] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5518), 1,
      sym_identifier,
  [36546] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5520), 1,
      sym_identifier,
  [36553] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5522), 1,
      anon_sym_COLON,
  [36560] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3257), 1,
      anon_sym_RPAREN,
  [36567] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5524), 1,
      aux_sym_preproc_if_defined_token2,
  [36574] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5526), 1,
      aux_sym_preproc_if_defined_token2,
  [36581] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3259), 1,
      anon_sym_RPAREN,
  [36588] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5528), 1,
      anon_sym_COLON,
  [36595] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5530), 1,
      aux_sym_preproc_if_defined_token2,
  [36602] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5208), 1,
      anon_sym_RPAREN,
  [36609] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5532), 1,
      aux_sym_preproc_if_defined_token2,
  [36616] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3341), 1,
      anon_sym_RPAREN,
  [36623] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5534), 1,
      aux_sym_preproc_if_defined_token2,
  [36630] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5536), 1,
      sym_identifier,
  [36637] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5538), 1,
      sym_identifier,
  [36644] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5540), 1,
      aux_sym_preproc_if_defined_token2,
  [36651] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5542), 1,
      aux_sym_preproc_if_defined_token2,
  [36658] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5544), 1,
      aux_sym_preproc_if_defined_token2,
  [36665] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5546), 1,
      sym_identifier,
  [36672] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5548), 1,
      anon_sym_EQ,
  [36679] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5550), 1,
      anon_sym_EQ,
  [36686] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5552), 1,
      sym_identifier,
  [36693] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5554), 1,
      sym_identifier,
  [36700] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5556), 1,
      anon_sym_RBRACK,
  [36707] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5558), 1,
      sym_identifier,
  [36714] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5560), 1,
      anon_sym_RBRACK,
  [36721] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5562), 1,
      aux_sym_preproc_if_defined_token2,
  [36728] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5564), 1,
      aux_sym_preproc_if_defined_token2,
  [36735] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5566), 1,
      sym_identifier,
  [36742] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5568), 1,
      aux_sym_preproc_if_defined_token2,
  [36749] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3317), 1,
      anon_sym_RPAREN,
  [36756] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3352), 1,
      anon_sym_RPAREN,
  [36763] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5570), 1,
      sym_number_literal,
  [36770] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5572), 1,
      sym_identifier,
  [36777] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5574), 1,
      aux_sym_preproc_if_defined_token2,
  [36784] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5576), 1,
      sym_number_literal,
  [36791] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5578), 1,
      sym_identifier,
  [36798] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5580), 1,
      sym_identifier,
  [36805] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5582), 1,
      aux_sym_preproc_if_defined_token2,
  [36812] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5584), 1,
      aux_sym_preproc_if_defined_token2,
  [36819] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5586), 1,
      aux_sym_preproc_if_defined_token2,
  [36826] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1599), 1,
      sym_identifier,
  [36833] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5588), 1,
      aux_sym_preproc_if_defined_token2,
  [36840] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5590), 1,
      aux_sym_preproc_if_defined_token2,
  [36847] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5592), 1,
      aux_sym_preproc_if_defined_token2,
  [36854] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5594), 1,
      anon_sym_COLON,
  [36861] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5596), 1,
      aux_sym_preproc_if_defined_token2,
  [36868] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5216), 1,
      anon_sym_RPAREN,
  [36875] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5191), 1,
      anon_sym_RPAREN,
  [36882] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5598), 1,
      aux_sym_preproc_if_defined_token2,
  [36889] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5600), 1,
      aux_sym_preproc_if_defined_token2,
  [36896] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5602), 1,
      aux_sym_preproc_if_defined_token2,
  [36903] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5480), 1,
      sym_identifier,
  [36910] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5604), 1,
      anon_sym_RPAREN,
  [36917] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5606), 1,
      aux_sym_preproc_if_defined_token2,
  [36924] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5608), 1,
      sym_identifier,
  [36931] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5610), 1,
      aux_sym_preproc_if_defined_token2,
  [36938] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5612), 1,
      aux_sym_preproc_if_defined_token2,
  [36945] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5614), 1,
      aux_sym_preproc_if_defined_token2,
  [36952] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5616), 1,
      aux_sym_preproc_if_defined_token2,
  [36959] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5618), 1,
      aux_sym_preproc_if_defined_token2,
  [36966] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5620), 1,
      anon_sym_COLON,
  [36973] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5622), 1,
      aux_sym_preproc_if_defined_token2,
  [36980] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5624), 1,
      aux_sym_preproc_if_defined_token2,
  [36987] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5626), 1,
      aux_sym_preproc_if_defined_token2,
  [36994] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5628), 1,
      aux_sym_preproc_if_defined_token2,
  [37001] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5630), 1,
      aux_sym_preproc_if_defined_token2,
  [37008] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5632), 1,
      aux_sym_preproc_if_defined_token2,
  [37015] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5634), 1,
      anon_sym_COLON,
  [37022] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5636), 1,
      aux_sym_preproc_if_defined_token2,
  [37029] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5638), 1,
      sym_identifier,
  [37036] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5640), 1,
      aux_sym_preproc_if_defined_token2,
  [37043] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5642), 1,
      anon_sym_RPAREN,
  [37050] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5644), 1,
      aux_sym_preproc_if_defined_token2,
  [37057] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5646), 1,
      aux_sym_preproc_if_defined_token2,
  [37064] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5648), 1,
      sym_identifier,
  [37071] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5650), 1,
      sym_identifier,
  [37078] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5652), 1,
      aux_sym_preproc_if_defined_token2,
  [37085] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5654), 1,
      sym_identifier,
  [37092] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3285), 1,
      anon_sym_RPAREN,
  [37099] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3267), 1,
      anon_sym_SEMI,
  [37106] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5656), 1,
      anon_sym_COLON,
  [37113] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5658), 1,
      sym_identifier,
  [37120] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5660), 1,
      sym_identifier,
  [37127] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5662), 1,
      aux_sym_preproc_if_defined_token2,
  [37134] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5664), 1,
      aux_sym_preproc_if_defined_token2,
  [37141] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5666), 1,
      aux_sym_preproc_if_defined_token2,
  [37148] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5668), 1,
      aux_sym_preproc_if_defined_token2,
  [37155] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5670), 1,
      aux_sym_preproc_if_defined_token2,
  [37162] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5672), 1,
      aux_sym_preproc_if_defined_token2,
  [37169] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3281), 1,
      anon_sym_SEMI,
  [37176] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5674), 1,
      aux_sym_preproc_if_defined_token2,
  [37183] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5676), 1,
      aux_sym_preproc_if_defined_token2,
  [37190] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5678), 1,
      aux_sym_preproc_if_defined_token2,
  [37197] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5680), 1,
      aux_sym_preproc_if_defined_token2,
  [37204] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5206), 1,
      anon_sym_RPAREN,
  [37211] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5682), 1,
      aux_sym_preproc_if_defined_token2,
  [37218] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5400), 1,
      sym_identifier,
  [37225] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5684), 1,
      anon_sym_LBRACK,
  [37232] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5686), 1,
      sym_identifier,
  [37239] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5688), 1,
      sym_identifier,
  [37246] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5690), 1,
      aux_sym_preproc_if_defined_token2,
  [37253] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5692), 1,
      sym_number_literal,
  [37260] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5694), 1,
      sym_identifier,
  [37267] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5696), 1,
      sym_identifier,
  [37274] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5698), 1,
      aux_sym_preproc_if_defined_token2,
  [37281] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5700), 1,
      anon_sym_LBRACK,
  [37288] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5702), 1,
      anon_sym_COLON,
  [37295] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5704), 1,
      sym_identifier,
  [37302] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5706), 1,
      sym_identifier,
  [37309] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5708), 1,
      sym_identifier,
  [37316] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5710), 1,
      sym_identifier,
  [37323] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5712), 1,
      anon_sym_COLON,
  [37330] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5714), 1,
      aux_sym_preproc_if_defined_token2,
  [37337] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5716), 1,
      sym_identifier,
  [37344] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5718), 1,
      aux_sym_preproc_if_defined_token2,
  [37351] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3299), 1,
      anon_sym_RPAREN,
  [37358] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5720), 1,
      aux_sym_preproc_if_defined_token2,
  [37365] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5722), 1,
      aux_sym_preproc_if_defined_token2,
  [37372] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5724), 1,
      anon_sym_RPAREN,
  [37379] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5726), 1,
      sym_identifier,
  [37386] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5728), 1,
      aux_sym_preproc_if_defined_token2,
  [37393] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5730), 1,
      anon_sym_COLON,
  [37400] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5732), 1,
      sym_identifier,
  [37407] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5734), 1,
      anon_sym_COLON,
  [37414] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5736), 1,
      anon_sym_COLON,
  [37421] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5738), 1,
      aux_sym_preproc_if_defined_token2,
  [37428] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5740), 1,
      aux_sym_preproc_if_defined_token2,
  [37435] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5742), 1,
      sym_identifier,
  [37442] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5744), 1,
      anon_sym_LPAREN,
  [37449] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5746), 1,
      sym_number_literal,
  [37456] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5748), 1,
      aux_sym_preproc_if_defined_token2,
  [37463] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5750), 1,
      sym_identifier,
  [37470] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5752), 1,
      sym_identifier,
  [37477] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5754), 1,
      aux_sym_preproc_if_defined_token2,
  [37484] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5756), 1,
      anon_sym_LBRACK,
  [37491] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5758), 1,
      aux_sym_preproc_if_defined_token2,
  [37498] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5760), 1,
      aux_sym_preproc_if_defined_token2,
  [37505] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5762), 1,
      aux_sym_preproc_if_defined_token2,
  [37512] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5764), 1,
      sym_identifier,
  [37519] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5766), 1,
      aux_sym_preproc_if_defined_token2,
  [37526] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5768), 1,
      sym_number_literal,
  [37533] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5770), 1,
      anon_sym_EQ,
  [37540] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5772), 1,
      aux_sym_preproc_if_defined_token2,
  [37547] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5774), 1,
      aux_sym_preproc_if_defined_token2,
  [37554] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3327), 1,
      anon_sym_RPAREN,
  [37561] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5776), 1,
      aux_sym_preproc_if_defined_token2,
  [37568] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5778), 1,
      aux_sym_preproc_if_defined_token2,
  [37575] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5780), 1,
      aux_sym_preproc_if_defined_token2,
  [37582] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5782), 1,
      sym_identifier,
  [37589] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5784), 1,
      sym_identifier,
  [37596] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5786), 1,
      aux_sym_preproc_if_defined_token2,
  [37603] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5462), 1,
      sym_identifier,
  [37610] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5788), 1,
      anon_sym_COLON,
  [37617] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5464), 1,
      sym_identifier,
  [37624] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5790), 1,
      sym_identifier,
  [37631] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5792), 1,
      anon_sym_COLON,
  [37638] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5794), 1,
      sym_identifier,
  [37645] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5796), 1,
      aux_sym_preproc_if_defined_token2,
  [37652] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5798), 1,
      sym_identifier,
  [37659] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5800), 1,
      sym_identifier,
  [37666] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5802), 1,
      sym_identifier,
  [37673] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5804), 1,
      sym_identifier,
  [37680] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5806), 1,
      sym_identifier,
  [37687] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5808), 1,
      aux_sym_preproc_if_defined_token2,
  [37694] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5810), 1,
      aux_sym_preproc_if_defined_token2,
  [37701] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5468), 1,
      sym_identifier,
  [37708] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5812), 1,
      aux_sym_preproc_if_defined_token2,
  [37715] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5470), 1,
      sym_identifier,
  [37722] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5814), 1,
      sym_identifier,
  [37729] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5816), 1,
      aux_sym_preproc_if_defined_token2,
  [37736] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5818), 1,
      ts_builtin_sym_end,
  [37743] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5820), 1,
      sym_identifier,
  [37750] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5822), 1,
      sym_identifier,
  [37757] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5824), 1,
      aux_sym_preproc_if_defined_token2,
  [37764] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3303), 1,
      anon_sym_RPAREN,
  [37771] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5826), 1,
      anon_sym_LBRACK,
  [37778] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5828), 1,
      aux_sym_preproc_if_defined_token2,
  [37785] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5830), 1,
      anon_sym_LPAREN,
  [37792] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5832), 1,
      aux_sym_preproc_if_defined_token2,
  [37799] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5834), 1,
      sym_identifier,
  [37806] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5836), 1,
      aux_sym_preproc_if_defined_token2,
  [37813] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5838), 1,
      aux_sym_preproc_if_defined_token2,
  [37820] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5840), 1,
      anon_sym_LPAREN,
  [37827] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5842), 1,
      sym_identifier,
  [37834] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1603), 1,
      sym_identifier,
  [37841] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5844), 1,
      aux_sym_preproc_if_defined_token2,
  [37848] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_SEMI,
  [37855] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5846), 1,
      aux_sym_preproc_if_defined_token2,
  [37862] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5848), 1,
      anon_sym_EQ,
  [37869] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5850), 1,
      aux_sym_preproc_if_defined_token2,
  [37876] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5852), 1,
      aux_sym_preproc_if_defined_token2,
  [37883] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5854), 1,
      aux_sym_preproc_if_defined_token2,
  [37890] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5856), 1,
      aux_sym_preproc_if_defined_token2,
  [37897] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5858), 1,
      aux_sym_preproc_if_defined_token2,
  [37904] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5860), 1,
      aux_sym_preproc_if_defined_token2,
};

static const uint32_t ts_small_parse_table_map[] = {
  [SMALL_STATE(972)] = 0,
  [SMALL_STATE(973)] = 102,
  [SMALL_STATE(974)] = 204,
  [SMALL_STATE(975)] = 306,
  [SMALL_STATE(976)] = 405,
  [SMALL_STATE(977)] = 499,
  [SMALL_STATE(978)] = 593,
  [SMALL_STATE(979)] = 687,
  [SMALL_STATE(980)] = 773,
  [SMALL_STATE(981)] = 861,
  [SMALL_STATE(982)] = 949,
  [SMALL_STATE(983)] = 1037,
  [SMALL_STATE(984)] = 1123,
  [SMALL_STATE(985)] = 1211,
  [SMALL_STATE(986)] = 1297,
  [SMALL_STATE(987)] = 1385,
  [SMALL_STATE(988)] = 1473,
  [SMALL_STATE(989)] = 1561,
  [SMALL_STATE(990)] = 1649,
  [SMALL_STATE(991)] = 1737,
  [SMALL_STATE(992)] = 1825,
  [SMALL_STATE(993)] = 1913,
  [SMALL_STATE(994)] = 2001,
  [SMALL_STATE(995)] = 2089,
  [SMALL_STATE(996)] = 2177,
  [SMALL_STATE(997)] = 2265,
  [SMALL_STATE(998)] = 2351,
  [SMALL_STATE(999)] = 2439,
  [SMALL_STATE(1000)] = 2527,
  [SMALL_STATE(1001)] = 2615,
  [SMALL_STATE(1002)] = 2703,
  [SMALL_STATE(1003)] = 2791,
  [SMALL_STATE(1004)] = 2879,
  [SMALL_STATE(1005)] = 2964,
  [SMALL_STATE(1006)] = 3023,
  [SMALL_STATE(1007)] = 3100,
  [SMALL_STATE(1008)] = 3171,
  [SMALL_STATE(1009)] = 3242,
  [SMALL_STATE(1010)] = 3296,
  [SMALL_STATE(1011)] = 3362,
  [SMALL_STATE(1012)] = 3428,
  [SMALL_STATE(1013)] = 3482,
  [SMALL_STATE(1014)] = 3548,
  [SMALL_STATE(1015)] = 3602,
  [SMALL_STATE(1016)] = 3656,
  [SMALL_STATE(1017)] = 3710,
  [SMALL_STATE(1018)] = 3764,
  [SMALL_STATE(1019)] = 3818,
  [SMALL_STATE(1020)] = 3884,
  [SMALL_STATE(1021)] = 3950,
  [SMALL_STATE(1022)] = 4004,
  [SMALL_STATE(1023)] = 4058,
  [SMALL_STATE(1024)] = 4124,
  [SMALL_STATE(1025)] = 4190,
  [SMALL_STATE(1026)] = 4244,
  [SMALL_STATE(1027)] = 4298,
  [SMALL_STATE(1028)] = 4352,
  [SMALL_STATE(1029)] = 4418,
  [SMALL_STATE(1030)] = 4472,
  [SMALL_STATE(1031)] = 4526,
  [SMALL_STATE(1032)] = 4580,
  [SMALL_STATE(1033)] = 4634,
  [SMALL_STATE(1034)] = 4688,
  [SMALL_STATE(1035)] = 4742,
  [SMALL_STATE(1036)] = 4796,
  [SMALL_STATE(1037)] = 4850,
  [SMALL_STATE(1038)] = 4908,
  [SMALL_STATE(1039)] = 4959,
  [SMALL_STATE(1040)] = 5010,
  [SMALL_STATE(1041)] = 5061,
  [SMALL_STATE(1042)] = 5115,
  [SMALL_STATE(1043)] = 5167,
  [SMALL_STATE(1044)] = 5227,
  [SMALL_STATE(1045)] = 5287,
  [SMALL_STATE(1046)] = 5337,
  [SMALL_STATE(1047)] = 5397,
  [SMALL_STATE(1048)] = 5444,
  [SMALL_STATE(1049)] = 5491,
  [SMALL_STATE(1050)] = 5538,
  [SMALL_STATE(1051)] = 5585,
  [SMALL_STATE(1052)] = 5650,
  [SMALL_STATE(1053)] = 5695,
  [SMALL_STATE(1054)] = 5758,
  [SMALL_STATE(1055)] = 5805,
  [SMALL_STATE(1056)] = 5852,
  [SMALL_STATE(1057)] = 5939,
  [SMALL_STATE(1058)] = 5986,
  [SMALL_STATE(1059)] = 6047,
  [SMALL_STATE(1060)] = 6108,
  [SMALL_STATE(1061)] = 6155,
  [SMALL_STATE(1062)] = 6216,
  [SMALL_STATE(1063)] = 6261,
  [SMALL_STATE(1064)] = 6308,
  [SMALL_STATE(1065)] = 6355,
  [SMALL_STATE(1066)] = 6426,
  [SMALL_STATE(1067)] = 6473,
  [SMALL_STATE(1068)] = 6520,
  [SMALL_STATE(1069)] = 6567,
  [SMALL_STATE(1070)] = 6614,
  [SMALL_STATE(1071)] = 6691,
  [SMALL_STATE(1072)] = 6738,
  [SMALL_STATE(1073)] = 6785,
  [SMALL_STATE(1074)] = 6832,
  [SMALL_STATE(1075)] = 6879,
  [SMALL_STATE(1076)] = 6926,
  [SMALL_STATE(1077)] = 6973,
  [SMALL_STATE(1078)] = 7020,
  [SMALL_STATE(1079)] = 7067,
  [SMALL_STATE(1080)] = 7146,
  [SMALL_STATE(1081)] = 7193,
  [SMALL_STATE(1082)] = 7280,
  [SMALL_STATE(1083)] = 7327,
  [SMALL_STATE(1084)] = 7374,
  [SMALL_STATE(1085)] = 7435,
  [SMALL_STATE(1086)] = 7516,
  [SMALL_STATE(1087)] = 7599,
  [SMALL_STATE(1088)] = 7672,
  [SMALL_STATE(1089)] = 7719,
  [SMALL_STATE(1090)] = 7766,
  [SMALL_STATE(1091)] = 7817,
  [SMALL_STATE(1092)] = 7864,
  [SMALL_STATE(1093)] = 7911,
  [SMALL_STATE(1094)] = 7958,
  [SMALL_STATE(1095)] = 8045,
  [SMALL_STATE(1096)] = 8106,
  [SMALL_STATE(1097)] = 8167,
  [SMALL_STATE(1098)] = 8234,
  [SMALL_STATE(1099)] = 8281,
  [SMALL_STATE(1100)] = 8328,
  [SMALL_STATE(1101)] = 8375,
  [SMALL_STATE(1102)] = 8463,
  [SMALL_STATE(1103)] = 8511,
  [SMALL_STATE(1104)] = 8557,
  [SMALL_STATE(1105)] = 8637,
  [SMALL_STATE(1106)] = 8719,
  [SMALL_STATE(1107)] = 8795,
  [SMALL_STATE(1108)] = 8867,
  [SMALL_STATE(1109)] = 8945,
  [SMALL_STATE(1110)] = 9009,
  [SMALL_STATE(1111)] = 9097,
  [SMALL_STATE(1112)] = 9187,
  [SMALL_STATE(1113)] = 9233,
  [SMALL_STATE(1114)] = 9295,
  [SMALL_STATE(1115)] = 9353,
  [SMALL_STATE(1116)] = 9399,
  [SMALL_STATE(1117)] = 9469,
  [SMALL_STATE(1118)] = 9535,
  [SMALL_STATE(1119)] = 9585,
  [SMALL_STATE(1120)] = 9671,
  [SMALL_STATE(1121)] = 9759,
  [SMALL_STATE(1122)] = 9805,
  [SMALL_STATE(1123)] = 9851,
  [SMALL_STATE(1124)] = 9939,
  [SMALL_STATE(1125)] = 10027,
  [SMALL_STATE(1126)] = 10085,
  [SMALL_STATE(1127)] = 10131,
  [SMALL_STATE(1128)] = 10189,
  [SMALL_STATE(1129)] = 10279,
  [SMALL_STATE(1130)] = 10327,
  [SMALL_STATE(1131)] = 10413,
  [SMALL_STATE(1132)] = 10501,
  [SMALL_STATE(1133)] = 10547,
  [SMALL_STATE(1134)] = 10593,
  [SMALL_STATE(1135)] = 10639,
  [SMALL_STATE(1136)] = 10687,
  [SMALL_STATE(1137)] = 10773,
  [SMALL_STATE(1138)] = 10861,
  [SMALL_STATE(1139)] = 10911,
  [SMALL_STATE(1140)] = 10999,
  [SMALL_STATE(1141)] = 11087,
  [SMALL_STATE(1142)] = 11177,
  [SMALL_STATE(1143)] = 11267,
  [SMALL_STATE(1144)] = 11355,
  [SMALL_STATE(1145)] = 11443,
  [SMALL_STATE(1146)] = 11533,
  [SMALL_STATE(1147)] = 11623,
  [SMALL_STATE(1148)] = 11713,
  [SMALL_STATE(1149)] = 11803,
  [SMALL_STATE(1150)] = 11893,
  [SMALL_STATE(1151)] = 11983,
  [SMALL_STATE(1152)] = 12073,
  [SMALL_STATE(1153)] = 12163,
  [SMALL_STATE(1154)] = 12249,
  [SMALL_STATE(1155)] = 12295,
  [SMALL_STATE(1156)] = 12341,
  [SMALL_STATE(1157)] = 12387,
  [SMALL_STATE(1158)] = 12433,
  [SMALL_STATE(1159)] = 12510,
  [SMALL_STATE(1160)] = 12597,
  [SMALL_STATE(1161)] = 12684,
  [SMALL_STATE(1162)] = 12769,
  [SMALL_STATE(1163)] = 12816,
  [SMALL_STATE(1164)] = 12863,
  [SMALL_STATE(1165)] = 12948,
  [SMALL_STATE(1166)] = 13035,
  [SMALL_STATE(1167)] = 13122,
  [SMALL_STATE(1168)] = 13209,
  [SMALL_STATE(1169)] = 13296,
  [SMALL_STATE(1170)] = 13381,
  [SMALL_STATE(1171)] = 13468,
  [SMALL_STATE(1172)] = 13513,
  [SMALL_STATE(1173)] = 13598,
  [SMALL_STATE(1174)] = 13685,
  [SMALL_STATE(1175)] = 13730,
  [SMALL_STATE(1176)] = 13775,
  [SMALL_STATE(1177)] = 13860,
  [SMALL_STATE(1178)] = 13945,
  [SMALL_STATE(1179)] = 14030,
  [SMALL_STATE(1180)] = 14115,
  [SMALL_STATE(1181)] = 14200,
  [SMALL_STATE(1182)] = 14245,
  [SMALL_STATE(1183)] = 14290,
  [SMALL_STATE(1184)] = 14375,
  [SMALL_STATE(1185)] = 14462,
  [SMALL_STATE(1186)] = 14547,
  [SMALL_STATE(1187)] = 14634,
  [SMALL_STATE(1188)] = 14719,
  [SMALL_STATE(1189)] = 14804,
  [SMALL_STATE(1190)] = 14889,
  [SMALL_STATE(1191)] = 14974,
  [SMALL_STATE(1192)] = 15059,
  [SMALL_STATE(1193)] = 15146,
  [SMALL_STATE(1194)] = 15233,
  [SMALL_STATE(1195)] = 15318,
  [SMALL_STATE(1196)] = 15405,
  [SMALL_STATE(1197)] = 15492,
  [SMALL_STATE(1198)] = 15577,
  [SMALL_STATE(1199)] = 15664,
  [SMALL_STATE(1200)] = 15751,
  [SMALL_STATE(1201)] = 15836,
  [SMALL_STATE(1202)] = 15923,
  [SMALL_STATE(1203)] = 16010,
  [SMALL_STATE(1204)] = 16055,
  [SMALL_STATE(1205)] = 16142,
  [SMALL_STATE(1206)] = 16227,
  [SMALL_STATE(1207)] = 16306,
  [SMALL_STATE(1208)] = 16387,
  [SMALL_STATE(1209)] = 16462,
  [SMALL_STATE(1210)] = 16533,
  [SMALL_STATE(1211)] = 16620,
  [SMALL_STATE(1212)] = 16683,
  [SMALL_STATE(1213)] = 16744,
  [SMALL_STATE(1214)] = 16813,
  [SMALL_STATE(1215)] = 16878,
  [SMALL_STATE(1216)] = 16963,
  [SMALL_STATE(1217)] = 17048,
  [SMALL_STATE(1218)] = 17093,
  [SMALL_STATE(1219)] = 17142,
  [SMALL_STATE(1220)] = 17187,
  [SMALL_STATE(1221)] = 17272,
  [SMALL_STATE(1222)] = 17317,
  [SMALL_STATE(1223)] = 17404,
  [SMALL_STATE(1224)] = 17488,
  [SMALL_STATE(1225)] = 17572,
  [SMALL_STATE(1226)] = 17656,
  [SMALL_STATE(1227)] = 17700,
  [SMALL_STATE(1228)] = 17784,
  [SMALL_STATE(1229)] = 17828,
  [SMALL_STATE(1230)] = 17912,
  [SMALL_STATE(1231)] = 17956,
  [SMALL_STATE(1232)] = 18040,
  [SMALL_STATE(1233)] = 18124,
  [SMALL_STATE(1234)] = 18168,
  [SMALL_STATE(1235)] = 18212,
  [SMALL_STATE(1236)] = 18296,
  [SMALL_STATE(1237)] = 18380,
  [SMALL_STATE(1238)] = 18464,
  [SMALL_STATE(1239)] = 18548,
  [SMALL_STATE(1240)] = 18632,
  [SMALL_STATE(1241)] = 18676,
  [SMALL_STATE(1242)] = 18760,
  [SMALL_STATE(1243)] = 18844,
  [SMALL_STATE(1244)] = 18928,
  [SMALL_STATE(1245)] = 19012,
  [SMALL_STATE(1246)] = 19056,
  [SMALL_STATE(1247)] = 19140,
  [SMALL_STATE(1248)] = 19224,
  [SMALL_STATE(1249)] = 19308,
  [SMALL_STATE(1250)] = 19392,
  [SMALL_STATE(1251)] = 19476,
  [SMALL_STATE(1252)] = 19560,
  [SMALL_STATE(1253)] = 19644,
  [SMALL_STATE(1254)] = 19728,
  [SMALL_STATE(1255)] = 19812,
  [SMALL_STATE(1256)] = 19896,
  [SMALL_STATE(1257)] = 19980,
  [SMALL_STATE(1258)] = 20064,
  [SMALL_STATE(1259)] = 20148,
  [SMALL_STATE(1260)] = 20192,
  [SMALL_STATE(1261)] = 20236,
  [SMALL_STATE(1262)] = 20280,
  [SMALL_STATE(1263)] = 20324,
  [SMALL_STATE(1264)] = 20408,
  [SMALL_STATE(1265)] = 20492,
  [SMALL_STATE(1266)] = 20576,
  [SMALL_STATE(1267)] = 20660,
  [SMALL_STATE(1268)] = 20744,
  [SMALL_STATE(1269)] = 20790,
  [SMALL_STATE(1270)] = 20836,
  [SMALL_STATE(1271)] = 20880,
  [SMALL_STATE(1272)] = 20964,
  [SMALL_STATE(1273)] = 21048,
  [SMALL_STATE(1274)] = 21132,
  [SMALL_STATE(1275)] = 21216,
  [SMALL_STATE(1276)] = 21300,
  [SMALL_STATE(1277)] = 21384,
  [SMALL_STATE(1278)] = 21468,
  [SMALL_STATE(1279)] = 21512,
  [SMALL_STATE(1280)] = 21556,
  [SMALL_STATE(1281)] = 21640,
  [SMALL_STATE(1282)] = 21724,
  [SMALL_STATE(1283)] = 21805,
  [SMALL_STATE(1284)] = 21886,
  [SMALL_STATE(1285)] = 21967,
  [SMALL_STATE(1286)] = 22048,
  [SMALL_STATE(1287)] = 22129,
  [SMALL_STATE(1288)] = 22210,
  [SMALL_STATE(1289)] = 22291,
  [SMALL_STATE(1290)] = 22372,
  [SMALL_STATE(1291)] = 22453,
  [SMALL_STATE(1292)] = 22534,
  [SMALL_STATE(1293)] = 22615,
  [SMALL_STATE(1294)] = 22696,
  [SMALL_STATE(1295)] = 22777,
  [SMALL_STATE(1296)] = 22858,
  [SMALL_STATE(1297)] = 22939,
  [SMALL_STATE(1298)] = 23020,
  [SMALL_STATE(1299)] = 23080,
  [SMALL_STATE(1300)] = 23140,
  [SMALL_STATE(1301)] = 23197,
  [SMALL_STATE(1302)] = 23254,
  [SMALL_STATE(1303)] = 23311,
  [SMALL_STATE(1304)] = 23368,
  [SMALL_STATE(1305)] = 23425,
  [SMALL_STATE(1306)] = 23482,
  [SMALL_STATE(1307)] = 23532,
  [SMALL_STATE(1308)] = 23584,
  [SMALL_STATE(1309)] = 23645,
  [SMALL_STATE(1310)] = 23706,
  [SMALL_STATE(1311)] = 23757,
  [SMALL_STATE(1312)] = 23808,
  [SMALL_STATE(1313)] = 23857,
  [SMALL_STATE(1314)] = 23915,
  [SMALL_STATE(1315)] = 23973,
  [SMALL_STATE(1316)] = 24021,
  [SMALL_STATE(1317)] = 24079,
  [SMALL_STATE(1318)] = 24137,
  [SMALL_STATE(1319)] = 24185,
  [SMALL_STATE(1320)] = 24243,
  [SMALL_STATE(1321)] = 24291,
  [SMALL_STATE(1322)] = 24339,
  [SMALL_STATE(1323)] = 24397,
  [SMALL_STATE(1324)] = 24455,
  [SMALL_STATE(1325)] = 24513,
  [SMALL_STATE(1326)] = 24566,
  [SMALL_STATE(1327)] = 24621,
  [SMALL_STATE(1328)] = 24672,
  [SMALL_STATE(1329)] = 24727,
  [SMALL_STATE(1330)] = 24782,
  [SMALL_STATE(1331)] = 24837,
  [SMALL_STATE(1332)] = 24892,
  [SMALL_STATE(1333)] = 24947,
  [SMALL_STATE(1334)] = 24997,
  [SMALL_STATE(1335)] = 25047,
  [SMALL_STATE(1336)] = 25095,
  [SMALL_STATE(1337)] = 25147,
  [SMALL_STATE(1338)] = 25199,
  [SMALL_STATE(1339)] = 25248,
  [SMALL_STATE(1340)] = 25297,
  [SMALL_STATE(1341)] = 25346,
  [SMALL_STATE(1342)] = 25395,
  [SMALL_STATE(1343)] = 25442,
  [SMALL_STATE(1344)] = 25491,
  [SMALL_STATE(1345)] = 25540,
  [SMALL_STATE(1346)] = 25586,
  [SMALL_STATE(1347)] = 25632,
  [SMALL_STATE(1348)] = 25678,
  [SMALL_STATE(1349)] = 25724,
  [SMALL_STATE(1350)] = 25749,
  [SMALL_STATE(1351)] = 25774,
  [SMALL_STATE(1352)] = 25799,
  [SMALL_STATE(1353)] = 25824,
  [SMALL_STATE(1354)] = 25849,
  [SMALL_STATE(1355)] = 25874,
  [SMALL_STATE(1356)] = 25899,
  [SMALL_STATE(1357)] = 25921,
  [SMALL_STATE(1358)] = 25943,
  [SMALL_STATE(1359)] = 25965,
  [SMALL_STATE(1360)] = 25987,
  [SMALL_STATE(1361)] = 26009,
  [SMALL_STATE(1362)] = 26031,
  [SMALL_STATE(1363)] = 26053,
  [SMALL_STATE(1364)] = 26075,
  [SMALL_STATE(1365)] = 26097,
  [SMALL_STATE(1366)] = 26119,
  [SMALL_STATE(1367)] = 26141,
  [SMALL_STATE(1368)] = 26163,
  [SMALL_STATE(1369)] = 26185,
  [SMALL_STATE(1370)] = 26214,
  [SMALL_STATE(1371)] = 26243,
  [SMALL_STATE(1372)] = 26272,
  [SMALL_STATE(1373)] = 26301,
  [SMALL_STATE(1374)] = 26330,
  [SMALL_STATE(1375)] = 26358,
  [SMALL_STATE(1376)] = 26386,
  [SMALL_STATE(1377)] = 26414,
  [SMALL_STATE(1378)] = 26450,
  [SMALL_STATE(1379)] = 26486,
  [SMALL_STATE(1380)] = 26522,
  [SMALL_STATE(1381)] = 26560,
  [SMALL_STATE(1382)] = 26598,
  [SMALL_STATE(1383)] = 26626,
  [SMALL_STATE(1384)] = 26654,
  [SMALL_STATE(1385)] = 26690,
  [SMALL_STATE(1386)] = 26728,
  [SMALL_STATE(1387)] = 26766,
  [SMALL_STATE(1388)] = 26784,
  [SMALL_STATE(1389)] = 26814,
  [SMALL_STATE(1390)] = 26844,
  [SMALL_STATE(1391)] = 26874,
  [SMALL_STATE(1392)] = 26904,
  [SMALL_STATE(1393)] = 26922,
  [SMALL_STATE(1394)] = 26940,
  [SMALL_STATE(1395)] = 26958,
  [SMALL_STATE(1396)] = 26976,
  [SMALL_STATE(1397)] = 26994,
  [SMALL_STATE(1398)] = 27024,
  [SMALL_STATE(1399)] = 27054,
  [SMALL_STATE(1400)] = 27084,
  [SMALL_STATE(1401)] = 27114,
  [SMALL_STATE(1402)] = 27132,
  [SMALL_STATE(1403)] = 27162,
  [SMALL_STATE(1404)] = 27192,
  [SMALL_STATE(1405)] = 27222,
  [SMALL_STATE(1406)] = 27252,
  [SMALL_STATE(1407)] = 27282,
  [SMALL_STATE(1408)] = 27300,
  [SMALL_STATE(1409)] = 27330,
  [SMALL_STATE(1410)] = 27360,
  [SMALL_STATE(1411)] = 27390,
  [SMALL_STATE(1412)] = 27408,
  [SMALL_STATE(1413)] = 27426,
  [SMALL_STATE(1414)] = 27444,
  [SMALL_STATE(1415)] = 27462,
  [SMALL_STATE(1416)] = 27492,
  [SMALL_STATE(1417)] = 27510,
  [SMALL_STATE(1418)] = 27540,
  [SMALL_STATE(1419)] = 27570,
  [SMALL_STATE(1420)] = 27600,
  [SMALL_STATE(1421)] = 27630,
  [SMALL_STATE(1422)] = 27660,
  [SMALL_STATE(1423)] = 27690,
  [SMALL_STATE(1424)] = 27720,
  [SMALL_STATE(1425)] = 27739,
  [SMALL_STATE(1426)] = 27756,
  [SMALL_STATE(1427)] = 27773,
  [SMALL_STATE(1428)] = 27790,
  [SMALL_STATE(1429)] = 27809,
  [SMALL_STATE(1430)] = 27828,
  [SMALL_STATE(1431)] = 27845,
  [SMALL_STATE(1432)] = 27876,
  [SMALL_STATE(1433)] = 27893,
  [SMALL_STATE(1434)] = 27910,
  [SMALL_STATE(1435)] = 27927,
  [SMALL_STATE(1436)] = 27944,
  [SMALL_STATE(1437)] = 27961,
  [SMALL_STATE(1438)] = 27978,
  [SMALL_STATE(1439)] = 27995,
  [SMALL_STATE(1440)] = 28014,
  [SMALL_STATE(1441)] = 28033,
  [SMALL_STATE(1442)] = 28050,
  [SMALL_STATE(1443)] = 28069,
  [SMALL_STATE(1444)] = 28088,
  [SMALL_STATE(1445)] = 28107,
  [SMALL_STATE(1446)] = 28124,
  [SMALL_STATE(1447)] = 28142,
  [SMALL_STATE(1448)] = 28160,
  [SMALL_STATE(1449)] = 28178,
  [SMALL_STATE(1450)] = 28196,
  [SMALL_STATE(1451)] = 28214,
  [SMALL_STATE(1452)] = 28232,
  [SMALL_STATE(1453)] = 28250,
  [SMALL_STATE(1454)] = 28268,
  [SMALL_STATE(1455)] = 28297,
  [SMALL_STATE(1456)] = 28326,
  [SMALL_STATE(1457)] = 28355,
  [SMALL_STATE(1458)] = 28384,
  [SMALL_STATE(1459)] = 28411,
  [SMALL_STATE(1460)] = 28438,
  [SMALL_STATE(1461)] = 28452,
  [SMALL_STATE(1462)] = 28466,
  [SMALL_STATE(1463)] = 28480,
  [SMALL_STATE(1464)] = 28494,
  [SMALL_STATE(1465)] = 28508,
  [SMALL_STATE(1466)] = 28522,
  [SMALL_STATE(1467)] = 28536,
  [SMALL_STATE(1468)] = 28550,
  [SMALL_STATE(1469)] = 28564,
  [SMALL_STATE(1470)] = 28578,
  [SMALL_STATE(1471)] = 28592,
  [SMALL_STATE(1472)] = 28606,
  [SMALL_STATE(1473)] = 28620,
  [SMALL_STATE(1474)] = 28646,
  [SMALL_STATE(1475)] = 28672,
  [SMALL_STATE(1476)] = 28698,
  [SMALL_STATE(1477)] = 28712,
  [SMALL_STATE(1478)] = 28726,
  [SMALL_STATE(1479)] = 28740,
  [SMALL_STATE(1480)] = 28766,
  [SMALL_STATE(1481)] = 28792,
  [SMALL_STATE(1482)] = 28818,
  [SMALL_STATE(1483)] = 28844,
  [SMALL_STATE(1484)] = 28870,
  [SMALL_STATE(1485)] = 28896,
  [SMALL_STATE(1486)] = 28922,
  [SMALL_STATE(1487)] = 28948,
  [SMALL_STATE(1488)] = 28974,
  [SMALL_STATE(1489)] = 28988,
  [SMALL_STATE(1490)] = 29014,
  [SMALL_STATE(1491)] = 29028,
  [SMALL_STATE(1492)] = 29054,
  [SMALL_STATE(1493)] = 29080,
  [SMALL_STATE(1494)] = 29094,
  [SMALL_STATE(1495)] = 29108,
  [SMALL_STATE(1496)] = 29122,
  [SMALL_STATE(1497)] = 29136,
  [SMALL_STATE(1498)] = 29150,
  [SMALL_STATE(1499)] = 29176,
  [SMALL_STATE(1500)] = 29199,
  [SMALL_STATE(1501)] = 29220,
  [SMALL_STATE(1502)] = 29243,
  [SMALL_STATE(1503)] = 29266,
  [SMALL_STATE(1504)] = 29289,
  [SMALL_STATE(1505)] = 29312,
  [SMALL_STATE(1506)] = 29335,
  [SMALL_STATE(1507)] = 29358,
  [SMALL_STATE(1508)] = 29381,
  [SMALL_STATE(1509)] = 29404,
  [SMALL_STATE(1510)] = 29427,
  [SMALL_STATE(1511)] = 29450,
  [SMALL_STATE(1512)] = 29473,
  [SMALL_STATE(1513)] = 29496,
  [SMALL_STATE(1514)] = 29519,
  [SMALL_STATE(1515)] = 29542,
  [SMALL_STATE(1516)] = 29565,
  [SMALL_STATE(1517)] = 29588,
  [SMALL_STATE(1518)] = 29611,
  [SMALL_STATE(1519)] = 29634,
  [SMALL_STATE(1520)] = 29657,
  [SMALL_STATE(1521)] = 29680,
  [SMALL_STATE(1522)] = 29703,
  [SMALL_STATE(1523)] = 29726,
  [SMALL_STATE(1524)] = 29749,
  [SMALL_STATE(1525)] = 29772,
  [SMALL_STATE(1526)] = 29793,
  [SMALL_STATE(1527)] = 29816,
  [SMALL_STATE(1528)] = 29839,
  [SMALL_STATE(1529)] = 29862,
  [SMALL_STATE(1530)] = 29885,
  [SMALL_STATE(1531)] = 29908,
  [SMALL_STATE(1532)] = 29931,
  [SMALL_STATE(1533)] = 29954,
  [SMALL_STATE(1534)] = 29977,
  [SMALL_STATE(1535)] = 30000,
  [SMALL_STATE(1536)] = 30023,
  [SMALL_STATE(1537)] = 30046,
  [SMALL_STATE(1538)] = 30069,
  [SMALL_STATE(1539)] = 30092,
  [SMALL_STATE(1540)] = 30115,
  [SMALL_STATE(1541)] = 30138,
  [SMALL_STATE(1542)] = 30161,
  [SMALL_STATE(1543)] = 30184,
  [SMALL_STATE(1544)] = 30207,
  [SMALL_STATE(1545)] = 30230,
  [SMALL_STATE(1546)] = 30253,
  [SMALL_STATE(1547)] = 30276,
  [SMALL_STATE(1548)] = 30299,
  [SMALL_STATE(1549)] = 30322,
  [SMALL_STATE(1550)] = 30345,
  [SMALL_STATE(1551)] = 30357,
  [SMALL_STATE(1552)] = 30369,
  [SMALL_STATE(1553)] = 30387,
  [SMALL_STATE(1554)] = 30399,
  [SMALL_STATE(1555)] = 30411,
  [SMALL_STATE(1556)] = 30423,
  [SMALL_STATE(1557)] = 30435,
  [SMALL_STATE(1558)] = 30447,
  [SMALL_STATE(1559)] = 30465,
  [SMALL_STATE(1560)] = 30477,
  [SMALL_STATE(1561)] = 30495,
  [SMALL_STATE(1562)] = 30507,
  [SMALL_STATE(1563)] = 30521,
  [SMALL_STATE(1564)] = 30537,
  [SMALL_STATE(1565)] = 30551,
  [SMALL_STATE(1566)] = 30563,
  [SMALL_STATE(1567)] = 30575,
  [SMALL_STATE(1568)] = 30593,
  [SMALL_STATE(1569)] = 30605,
  [SMALL_STATE(1570)] = 30617,
  [SMALL_STATE(1571)] = 30629,
  [SMALL_STATE(1572)] = 30641,
  [SMALL_STATE(1573)] = 30653,
  [SMALL_STATE(1574)] = 30665,
  [SMALL_STATE(1575)] = 30677,
  [SMALL_STATE(1576)] = 30689,
  [SMALL_STATE(1577)] = 30701,
  [SMALL_STATE(1578)] = 30713,
  [SMALL_STATE(1579)] = 30725,
  [SMALL_STATE(1580)] = 30737,
  [SMALL_STATE(1581)] = 30749,
  [SMALL_STATE(1582)] = 30761,
  [SMALL_STATE(1583)] = 30773,
  [SMALL_STATE(1584)] = 30791,
  [SMALL_STATE(1585)] = 30803,
  [SMALL_STATE(1586)] = 30815,
  [SMALL_STATE(1587)] = 30827,
  [SMALL_STATE(1588)] = 30839,
  [SMALL_STATE(1589)] = 30853,
  [SMALL_STATE(1590)] = 30865,
  [SMALL_STATE(1591)] = 30877,
  [SMALL_STATE(1592)] = 30889,
  [SMALL_STATE(1593)] = 30901,
  [SMALL_STATE(1594)] = 30913,
  [SMALL_STATE(1595)] = 30925,
  [SMALL_STATE(1596)] = 30937,
  [SMALL_STATE(1597)] = 30949,
  [SMALL_STATE(1598)] = 30961,
  [SMALL_STATE(1599)] = 30975,
  [SMALL_STATE(1600)] = 30987,
  [SMALL_STATE(1601)] = 30999,
  [SMALL_STATE(1602)] = 31011,
  [SMALL_STATE(1603)] = 31023,
  [SMALL_STATE(1604)] = 31035,
  [SMALL_STATE(1605)] = 31047,
  [SMALL_STATE(1606)] = 31059,
  [SMALL_STATE(1607)] = 31071,
  [SMALL_STATE(1608)] = 31083,
  [SMALL_STATE(1609)] = 31095,
  [SMALL_STATE(1610)] = 31107,
  [SMALL_STATE(1611)] = 31119,
  [SMALL_STATE(1612)] = 31131,
  [SMALL_STATE(1613)] = 31143,
  [SMALL_STATE(1614)] = 31155,
  [SMALL_STATE(1615)] = 31167,
  [SMALL_STATE(1616)] = 31179,
  [SMALL_STATE(1617)] = 31197,
  [SMALL_STATE(1618)] = 31209,
  [SMALL_STATE(1619)] = 31221,
  [SMALL_STATE(1620)] = 31233,
  [SMALL_STATE(1621)] = 31251,
  [SMALL_STATE(1622)] = 31263,
  [SMALL_STATE(1623)] = 31275,
  [SMALL_STATE(1624)] = 31287,
  [SMALL_STATE(1625)] = 31299,
  [SMALL_STATE(1626)] = 31311,
  [SMALL_STATE(1627)] = 31323,
  [SMALL_STATE(1628)] = 31341,
  [SMALL_STATE(1629)] = 31353,
  [SMALL_STATE(1630)] = 31365,
  [SMALL_STATE(1631)] = 31377,
  [SMALL_STATE(1632)] = 31389,
  [SMALL_STATE(1633)] = 31407,
  [SMALL_STATE(1634)] = 31419,
  [SMALL_STATE(1635)] = 31431,
  [SMALL_STATE(1636)] = 31443,
  [SMALL_STATE(1637)] = 31455,
  [SMALL_STATE(1638)] = 31467,
  [SMALL_STATE(1639)] = 31479,
  [SMALL_STATE(1640)] = 31493,
  [SMALL_STATE(1641)] = 31505,
  [SMALL_STATE(1642)] = 31523,
  [SMALL_STATE(1643)] = 31535,
  [SMALL_STATE(1644)] = 31547,
  [SMALL_STATE(1645)] = 31559,
  [SMALL_STATE(1646)] = 31571,
  [SMALL_STATE(1647)] = 31583,
  [SMALL_STATE(1648)] = 31595,
  [SMALL_STATE(1649)] = 31607,
  [SMALL_STATE(1650)] = 31619,
  [SMALL_STATE(1651)] = 31631,
  [SMALL_STATE(1652)] = 31643,
  [SMALL_STATE(1653)] = 31655,
  [SMALL_STATE(1654)] = 31667,
  [SMALL_STATE(1655)] = 31679,
  [SMALL_STATE(1656)] = 31693,
  [SMALL_STATE(1657)] = 31705,
  [SMALL_STATE(1658)] = 31717,
  [SMALL_STATE(1659)] = 31729,
  [SMALL_STATE(1660)] = 31741,
  [SMALL_STATE(1661)] = 31759,
  [SMALL_STATE(1662)] = 31771,
  [SMALL_STATE(1663)] = 31789,
  [SMALL_STATE(1664)] = 31801,
  [SMALL_STATE(1665)] = 31817,
  [SMALL_STATE(1666)] = 31829,
  [SMALL_STATE(1667)] = 31844,
  [SMALL_STATE(1668)] = 31861,
  [SMALL_STATE(1669)] = 31878,
  [SMALL_STATE(1670)] = 31895,
  [SMALL_STATE(1671)] = 31910,
  [SMALL_STATE(1672)] = 31925,
  [SMALL_STATE(1673)] = 31938,
  [SMALL_STATE(1674)] = 31953,
  [SMALL_STATE(1675)] = 31970,
  [SMALL_STATE(1676)] = 31985,
  [SMALL_STATE(1677)] = 32002,
  [SMALL_STATE(1678)] = 32017,
  [SMALL_STATE(1679)] = 32032,
  [SMALL_STATE(1680)] = 32047,
  [SMALL_STATE(1681)] = 32062,
  [SMALL_STATE(1682)] = 32079,
  [SMALL_STATE(1683)] = 32094,
  [SMALL_STATE(1684)] = 32107,
  [SMALL_STATE(1685)] = 32120,
  [SMALL_STATE(1686)] = 32137,
  [SMALL_STATE(1687)] = 32152,
  [SMALL_STATE(1688)] = 32169,
  [SMALL_STATE(1689)] = 32186,
  [SMALL_STATE(1690)] = 32203,
  [SMALL_STATE(1691)] = 32216,
  [SMALL_STATE(1692)] = 32231,
  [SMALL_STATE(1693)] = 32246,
  [SMALL_STATE(1694)] = 32259,
  [SMALL_STATE(1695)] = 32272,
  [SMALL_STATE(1696)] = 32289,
  [SMALL_STATE(1697)] = 32306,
  [SMALL_STATE(1698)] = 32321,
  [SMALL_STATE(1699)] = 32336,
  [SMALL_STATE(1700)] = 32353,
  [SMALL_STATE(1701)] = 32370,
  [SMALL_STATE(1702)] = 32387,
  [SMALL_STATE(1703)] = 32404,
  [SMALL_STATE(1704)] = 32419,
  [SMALL_STATE(1705)] = 32436,
  [SMALL_STATE(1706)] = 32449,
  [SMALL_STATE(1707)] = 32462,
  [SMALL_STATE(1708)] = 32478,
  [SMALL_STATE(1709)] = 32494,
  [SMALL_STATE(1710)] = 32508,
  [SMALL_STATE(1711)] = 32522,
  [SMALL_STATE(1712)] = 32538,
  [SMALL_STATE(1713)] = 32554,
  [SMALL_STATE(1714)] = 32570,
  [SMALL_STATE(1715)] = 32586,
  [SMALL_STATE(1716)] = 32602,
  [SMALL_STATE(1717)] = 32618,
  [SMALL_STATE(1718)] = 32634,
  [SMALL_STATE(1719)] = 32650,
  [SMALL_STATE(1720)] = 32666,
  [SMALL_STATE(1721)] = 32682,
  [SMALL_STATE(1722)] = 32696,
  [SMALL_STATE(1723)] = 32712,
  [SMALL_STATE(1724)] = 32728,
  [SMALL_STATE(1725)] = 32744,
  [SMALL_STATE(1726)] = 32760,
  [SMALL_STATE(1727)] = 32774,
  [SMALL_STATE(1728)] = 32790,
  [SMALL_STATE(1729)] = 32806,
  [SMALL_STATE(1730)] = 32822,
  [SMALL_STATE(1731)] = 32838,
  [SMALL_STATE(1732)] = 32854,
  [SMALL_STATE(1733)] = 32870,
  [SMALL_STATE(1734)] = 32886,
  [SMALL_STATE(1735)] = 32900,
  [SMALL_STATE(1736)] = 32914,
  [SMALL_STATE(1737)] = 32930,
  [SMALL_STATE(1738)] = 32946,
  [SMALL_STATE(1739)] = 32960,
  [SMALL_STATE(1740)] = 32974,
  [SMALL_STATE(1741)] = 32990,
  [SMALL_STATE(1742)] = 33006,
  [SMALL_STATE(1743)] = 33022,
  [SMALL_STATE(1744)] = 33038,
  [SMALL_STATE(1745)] = 33054,
  [SMALL_STATE(1746)] = 33070,
  [SMALL_STATE(1747)] = 33086,
  [SMALL_STATE(1748)] = 33102,
  [SMALL_STATE(1749)] = 33118,
  [SMALL_STATE(1750)] = 33134,
  [SMALL_STATE(1751)] = 33150,
  [SMALL_STATE(1752)] = 33164,
  [SMALL_STATE(1753)] = 33180,
  [SMALL_STATE(1754)] = 33196,
  [SMALL_STATE(1755)] = 33212,
  [SMALL_STATE(1756)] = 33226,
  [SMALL_STATE(1757)] = 33242,
  [SMALL_STATE(1758)] = 33256,
  [SMALL_STATE(1759)] = 33272,
  [SMALL_STATE(1760)] = 33288,
  [SMALL_STATE(1761)] = 33304,
  [SMALL_STATE(1762)] = 33320,
  [SMALL_STATE(1763)] = 33336,
  [SMALL_STATE(1764)] = 33352,
  [SMALL_STATE(1765)] = 33368,
  [SMALL_STATE(1766)] = 33384,
  [SMALL_STATE(1767)] = 33400,
  [SMALL_STATE(1768)] = 33416,
  [SMALL_STATE(1769)] = 33432,
  [SMALL_STATE(1770)] = 33448,
  [SMALL_STATE(1771)] = 33464,
  [SMALL_STATE(1772)] = 33480,
  [SMALL_STATE(1773)] = 33492,
  [SMALL_STATE(1774)] = 33508,
  [SMALL_STATE(1775)] = 33522,
  [SMALL_STATE(1776)] = 33536,
  [SMALL_STATE(1777)] = 33550,
  [SMALL_STATE(1778)] = 33564,
  [SMALL_STATE(1779)] = 33580,
  [SMALL_STATE(1780)] = 33594,
  [SMALL_STATE(1781)] = 33606,
  [SMALL_STATE(1782)] = 33618,
  [SMALL_STATE(1783)] = 33630,
  [SMALL_STATE(1784)] = 33644,
  [SMALL_STATE(1785)] = 33658,
  [SMALL_STATE(1786)] = 33674,
  [SMALL_STATE(1787)] = 33690,
  [SMALL_STATE(1788)] = 33706,
  [SMALL_STATE(1789)] = 33720,
  [SMALL_STATE(1790)] = 33736,
  [SMALL_STATE(1791)] = 33750,
  [SMALL_STATE(1792)] = 33764,
  [SMALL_STATE(1793)] = 33780,
  [SMALL_STATE(1794)] = 33794,
  [SMALL_STATE(1795)] = 33810,
  [SMALL_STATE(1796)] = 33826,
  [SMALL_STATE(1797)] = 33840,
  [SMALL_STATE(1798)] = 33856,
  [SMALL_STATE(1799)] = 33872,
  [SMALL_STATE(1800)] = 33886,
  [SMALL_STATE(1801)] = 33900,
  [SMALL_STATE(1802)] = 33914,
  [SMALL_STATE(1803)] = 33930,
  [SMALL_STATE(1804)] = 33946,
  [SMALL_STATE(1805)] = 33960,
  [SMALL_STATE(1806)] = 33976,
  [SMALL_STATE(1807)] = 33989,
  [SMALL_STATE(1808)] = 34002,
  [SMALL_STATE(1809)] = 34013,
  [SMALL_STATE(1810)] = 34026,
  [SMALL_STATE(1811)] = 34039,
  [SMALL_STATE(1812)] = 34052,
  [SMALL_STATE(1813)] = 34065,
  [SMALL_STATE(1814)] = 34078,
  [SMALL_STATE(1815)] = 34091,
  [SMALL_STATE(1816)] = 34104,
  [SMALL_STATE(1817)] = 34117,
  [SMALL_STATE(1818)] = 34128,
  [SMALL_STATE(1819)] = 34139,
  [SMALL_STATE(1820)] = 34152,
  [SMALL_STATE(1821)] = 34165,
  [SMALL_STATE(1822)] = 34178,
  [SMALL_STATE(1823)] = 34191,
  [SMALL_STATE(1824)] = 34204,
  [SMALL_STATE(1825)] = 34217,
  [SMALL_STATE(1826)] = 34230,
  [SMALL_STATE(1827)] = 34239,
  [SMALL_STATE(1828)] = 34252,
  [SMALL_STATE(1829)] = 34263,
  [SMALL_STATE(1830)] = 34274,
  [SMALL_STATE(1831)] = 34287,
  [SMALL_STATE(1832)] = 34300,
  [SMALL_STATE(1833)] = 34311,
  [SMALL_STATE(1834)] = 34322,
  [SMALL_STATE(1835)] = 34331,
  [SMALL_STATE(1836)] = 34344,
  [SMALL_STATE(1837)] = 34357,
  [SMALL_STATE(1838)] = 34370,
  [SMALL_STATE(1839)] = 34383,
  [SMALL_STATE(1840)] = 34396,
  [SMALL_STATE(1841)] = 34409,
  [SMALL_STATE(1842)] = 34420,
  [SMALL_STATE(1843)] = 34429,
  [SMALL_STATE(1844)] = 34438,
  [SMALL_STATE(1845)] = 34449,
  [SMALL_STATE(1846)] = 34462,
  [SMALL_STATE(1847)] = 34471,
  [SMALL_STATE(1848)] = 34484,
  [SMALL_STATE(1849)] = 34497,
  [SMALL_STATE(1850)] = 34508,
  [SMALL_STATE(1851)] = 34519,
  [SMALL_STATE(1852)] = 34532,
  [SMALL_STATE(1853)] = 34545,
  [SMALL_STATE(1854)] = 34556,
  [SMALL_STATE(1855)] = 34567,
  [SMALL_STATE(1856)] = 34580,
  [SMALL_STATE(1857)] = 34591,
  [SMALL_STATE(1858)] = 34602,
  [SMALL_STATE(1859)] = 34611,
  [SMALL_STATE(1860)] = 34624,
  [SMALL_STATE(1861)] = 34633,
  [SMALL_STATE(1862)] = 34642,
  [SMALL_STATE(1863)] = 34653,
  [SMALL_STATE(1864)] = 34664,
  [SMALL_STATE(1865)] = 34675,
  [SMALL_STATE(1866)] = 34688,
  [SMALL_STATE(1867)] = 34701,
  [SMALL_STATE(1868)] = 34714,
  [SMALL_STATE(1869)] = 34727,
  [SMALL_STATE(1870)] = 34738,
  [SMALL_STATE(1871)] = 34749,
  [SMALL_STATE(1872)] = 34762,
  [SMALL_STATE(1873)] = 34775,
  [SMALL_STATE(1874)] = 34786,
  [SMALL_STATE(1875)] = 34799,
  [SMALL_STATE(1876)] = 34812,
  [SMALL_STATE(1877)] = 34825,
  [SMALL_STATE(1878)] = 34836,
  [SMALL_STATE(1879)] = 34849,
  [SMALL_STATE(1880)] = 34857,
  [SMALL_STATE(1881)] = 34865,
  [SMALL_STATE(1882)] = 34873,
  [SMALL_STATE(1883)] = 34881,
  [SMALL_STATE(1884)] = 34889,
  [SMALL_STATE(1885)] = 34897,
  [SMALL_STATE(1886)] = 34905,
  [SMALL_STATE(1887)] = 34913,
  [SMALL_STATE(1888)] = 34923,
  [SMALL_STATE(1889)] = 34933,
  [SMALL_STATE(1890)] = 34941,
  [SMALL_STATE(1891)] = 34949,
  [SMALL_STATE(1892)] = 34959,
  [SMALL_STATE(1893)] = 34969,
  [SMALL_STATE(1894)] = 34979,
  [SMALL_STATE(1895)] = 34987,
  [SMALL_STATE(1896)] = 34995,
  [SMALL_STATE(1897)] = 35003,
  [SMALL_STATE(1898)] = 35011,
  [SMALL_STATE(1899)] = 35021,
  [SMALL_STATE(1900)] = 35029,
  [SMALL_STATE(1901)] = 35037,
  [SMALL_STATE(1902)] = 35047,
  [SMALL_STATE(1903)] = 35055,
  [SMALL_STATE(1904)] = 35065,
  [SMALL_STATE(1905)] = 35075,
  [SMALL_STATE(1906)] = 35085,
  [SMALL_STATE(1907)] = 35095,
  [SMALL_STATE(1908)] = 35105,
  [SMALL_STATE(1909)] = 35115,
  [SMALL_STATE(1910)] = 35123,
  [SMALL_STATE(1911)] = 35133,
  [SMALL_STATE(1912)] = 35143,
  [SMALL_STATE(1913)] = 35151,
  [SMALL_STATE(1914)] = 35161,
  [SMALL_STATE(1915)] = 35169,
  [SMALL_STATE(1916)] = 35179,
  [SMALL_STATE(1917)] = 35189,
  [SMALL_STATE(1918)] = 35199,
  [SMALL_STATE(1919)] = 35209,
  [SMALL_STATE(1920)] = 35217,
  [SMALL_STATE(1921)] = 35225,
  [SMALL_STATE(1922)] = 35233,
  [SMALL_STATE(1923)] = 35241,
  [SMALL_STATE(1924)] = 35251,
  [SMALL_STATE(1925)] = 35259,
  [SMALL_STATE(1926)] = 35267,
  [SMALL_STATE(1927)] = 35275,
  [SMALL_STATE(1928)] = 35283,
  [SMALL_STATE(1929)] = 35291,
  [SMALL_STATE(1930)] = 35299,
  [SMALL_STATE(1931)] = 35307,
  [SMALL_STATE(1932)] = 35315,
  [SMALL_STATE(1933)] = 35323,
  [SMALL_STATE(1934)] = 35331,
  [SMALL_STATE(1935)] = 35341,
  [SMALL_STATE(1936)] = 35349,
  [SMALL_STATE(1937)] = 35359,
  [SMALL_STATE(1938)] = 35367,
  [SMALL_STATE(1939)] = 35377,
  [SMALL_STATE(1940)] = 35387,
  [SMALL_STATE(1941)] = 35397,
  [SMALL_STATE(1942)] = 35407,
  [SMALL_STATE(1943)] = 35415,
  [SMALL_STATE(1944)] = 35425,
  [SMALL_STATE(1945)] = 35435,
  [SMALL_STATE(1946)] = 35443,
  [SMALL_STATE(1947)] = 35453,
  [SMALL_STATE(1948)] = 35461,
  [SMALL_STATE(1949)] = 35469,
  [SMALL_STATE(1950)] = 35479,
  [SMALL_STATE(1951)] = 35487,
  [SMALL_STATE(1952)] = 35495,
  [SMALL_STATE(1953)] = 35505,
  [SMALL_STATE(1954)] = 35515,
  [SMALL_STATE(1955)] = 35523,
  [SMALL_STATE(1956)] = 35531,
  [SMALL_STATE(1957)] = 35539,
  [SMALL_STATE(1958)] = 35547,
  [SMALL_STATE(1959)] = 35557,
  [SMALL_STATE(1960)] = 35567,
  [SMALL_STATE(1961)] = 35575,
  [SMALL_STATE(1962)] = 35585,
  [SMALL_STATE(1963)] = 35593,
  [SMALL_STATE(1964)] = 35601,
  [SMALL_STATE(1965)] = 35611,
  [SMALL_STATE(1966)] = 35621,
  [SMALL_STATE(1967)] = 35631,
  [SMALL_STATE(1968)] = 35639,
  [SMALL_STATE(1969)] = 35647,
  [SMALL_STATE(1970)] = 35657,
  [SMALL_STATE(1971)] = 35667,
  [SMALL_STATE(1972)] = 35675,
  [SMALL_STATE(1973)] = 35685,
  [SMALL_STATE(1974)] = 35695,
  [SMALL_STATE(1975)] = 35705,
  [SMALL_STATE(1976)] = 35715,
  [SMALL_STATE(1977)] = 35725,
  [SMALL_STATE(1978)] = 35735,
  [SMALL_STATE(1979)] = 35743,
  [SMALL_STATE(1980)] = 35751,
  [SMALL_STATE(1981)] = 35759,
  [SMALL_STATE(1982)] = 35767,
  [SMALL_STATE(1983)] = 35777,
  [SMALL_STATE(1984)] = 35787,
  [SMALL_STATE(1985)] = 35797,
  [SMALL_STATE(1986)] = 35807,
  [SMALL_STATE(1987)] = 35817,
  [SMALL_STATE(1988)] = 35827,
  [SMALL_STATE(1989)] = 35837,
  [SMALL_STATE(1990)] = 35847,
  [SMALL_STATE(1991)] = 35855,
  [SMALL_STATE(1992)] = 35865,
  [SMALL_STATE(1993)] = 35873,
  [SMALL_STATE(1994)] = 35883,
  [SMALL_STATE(1995)] = 35891,
  [SMALL_STATE(1996)] = 35901,
  [SMALL_STATE(1997)] = 35909,
  [SMALL_STATE(1998)] = 35919,
  [SMALL_STATE(1999)] = 35929,
  [SMALL_STATE(2000)] = 35939,
  [SMALL_STATE(2001)] = 35947,
  [SMALL_STATE(2002)] = 35957,
  [SMALL_STATE(2003)] = 35967,
  [SMALL_STATE(2004)] = 35977,
  [SMALL_STATE(2005)] = 35985,
  [SMALL_STATE(2006)] = 35995,
  [SMALL_STATE(2007)] = 36005,
  [SMALL_STATE(2008)] = 36015,
  [SMALL_STATE(2009)] = 36025,
  [SMALL_STATE(2010)] = 36035,
  [SMALL_STATE(2011)] = 36045,
  [SMALL_STATE(2012)] = 36053,
  [SMALL_STATE(2013)] = 36063,
  [SMALL_STATE(2014)] = 36073,
  [SMALL_STATE(2015)] = 36083,
  [SMALL_STATE(2016)] = 36093,
  [SMALL_STATE(2017)] = 36101,
  [SMALL_STATE(2018)] = 36109,
  [SMALL_STATE(2019)] = 36119,
  [SMALL_STATE(2020)] = 36129,
  [SMALL_STATE(2021)] = 36139,
  [SMALL_STATE(2022)] = 36149,
  [SMALL_STATE(2023)] = 36157,
  [SMALL_STATE(2024)] = 36167,
  [SMALL_STATE(2025)] = 36177,
  [SMALL_STATE(2026)] = 36187,
  [SMALL_STATE(2027)] = 36197,
  [SMALL_STATE(2028)] = 36205,
  [SMALL_STATE(2029)] = 36215,
  [SMALL_STATE(2030)] = 36225,
  [SMALL_STATE(2031)] = 36235,
  [SMALL_STATE(2032)] = 36245,
  [SMALL_STATE(2033)] = 36255,
  [SMALL_STATE(2034)] = 36265,
  [SMALL_STATE(2035)] = 36273,
  [SMALL_STATE(2036)] = 36283,
  [SMALL_STATE(2037)] = 36293,
  [SMALL_STATE(2038)] = 36303,
  [SMALL_STATE(2039)] = 36311,
  [SMALL_STATE(2040)] = 36321,
  [SMALL_STATE(2041)] = 36331,
  [SMALL_STATE(2042)] = 36339,
  [SMALL_STATE(2043)] = 36347,
  [SMALL_STATE(2044)] = 36355,
  [SMALL_STATE(2045)] = 36363,
  [SMALL_STATE(2046)] = 36371,
  [SMALL_STATE(2047)] = 36379,
  [SMALL_STATE(2048)] = 36389,
  [SMALL_STATE(2049)] = 36399,
  [SMALL_STATE(2050)] = 36406,
  [SMALL_STATE(2051)] = 36413,
  [SMALL_STATE(2052)] = 36420,
  [SMALL_STATE(2053)] = 36427,
  [SMALL_STATE(2054)] = 36434,
  [SMALL_STATE(2055)] = 36441,
  [SMALL_STATE(2056)] = 36448,
  [SMALL_STATE(2057)] = 36455,
  [SMALL_STATE(2058)] = 36462,
  [SMALL_STATE(2059)] = 36469,
  [SMALL_STATE(2060)] = 36476,
  [SMALL_STATE(2061)] = 36483,
  [SMALL_STATE(2062)] = 36490,
  [SMALL_STATE(2063)] = 36497,
  [SMALL_STATE(2064)] = 36504,
  [SMALL_STATE(2065)] = 36511,
  [SMALL_STATE(2066)] = 36518,
  [SMALL_STATE(2067)] = 36525,
  [SMALL_STATE(2068)] = 36532,
  [SMALL_STATE(2069)] = 36539,
  [SMALL_STATE(2070)] = 36546,
  [SMALL_STATE(2071)] = 36553,
  [SMALL_STATE(2072)] = 36560,
  [SMALL_STATE(2073)] = 36567,
  [SMALL_STATE(2074)] = 36574,
  [SMALL_STATE(2075)] = 36581,
  [SMALL_STATE(2076)] = 36588,
  [SMALL_STATE(2077)] = 36595,
  [SMALL_STATE(2078)] = 36602,
  [SMALL_STATE(2079)] = 36609,
  [SMALL_STATE(2080)] = 36616,
  [SMALL_STATE(2081)] = 36623,
  [SMALL_STATE(2082)] = 36630,
  [SMALL_STATE(2083)] = 36637,
  [SMALL_STATE(2084)] = 36644,
  [SMALL_STATE(2085)] = 36651,
  [SMALL_STATE(2086)] = 36658,
  [SMALL_STATE(2087)] = 36665,
  [SMALL_STATE(2088)] = 36672,
  [SMALL_STATE(2089)] = 36679,
  [SMALL_STATE(2090)] = 36686,
  [SMALL_STATE(2091)] = 36693,
  [SMALL_STATE(2092)] = 36700,
  [SMALL_STATE(2093)] = 36707,
  [SMALL_STATE(2094)] = 36714,
  [SMALL_STATE(2095)] = 36721,
  [SMALL_STATE(2096)] = 36728,
  [SMALL_STATE(2097)] = 36735,
  [SMALL_STATE(2098)] = 36742,
  [SMALL_STATE(2099)] = 36749,
  [SMALL_STATE(2100)] = 36756,
  [SMALL_STATE(2101)] = 36763,
  [SMALL_STATE(2102)] = 36770,
  [SMALL_STATE(2103)] = 36777,
  [SMALL_STATE(2104)] = 36784,
  [SMALL_STATE(2105)] = 36791,
  [SMALL_STATE(2106)] = 36798,
  [SMALL_STATE(2107)] = 36805,
  [SMALL_STATE(2108)] = 36812,
  [SMALL_STATE(2109)] = 36819,
  [SMALL_STATE(2110)] = 36826,
  [SMALL_STATE(2111)] = 36833,
  [SMALL_STATE(2112)] = 36840,
  [SMALL_STATE(2113)] = 36847,
  [SMALL_STATE(2114)] = 36854,
  [SMALL_STATE(2115)] = 36861,
  [SMALL_STATE(2116)] = 36868,
  [SMALL_STATE(2117)] = 36875,
  [SMALL_STATE(2118)] = 36882,
  [SMALL_STATE(2119)] = 36889,
  [SMALL_STATE(2120)] = 36896,
  [SMALL_STATE(2121)] = 36903,
  [SMALL_STATE(2122)] = 36910,
  [SMALL_STATE(2123)] = 36917,
  [SMALL_STATE(2124)] = 36924,
  [SMALL_STATE(2125)] = 36931,
  [SMALL_STATE(2126)] = 36938,
  [SMALL_STATE(2127)] = 36945,
  [SMALL_STATE(2128)] = 36952,
  [SMALL_STATE(2129)] = 36959,
  [SMALL_STATE(2130)] = 36966,
  [SMALL_STATE(2131)] = 36973,
  [SMALL_STATE(2132)] = 36980,
  [SMALL_STATE(2133)] = 36987,
  [SMALL_STATE(2134)] = 36994,
  [SMALL_STATE(2135)] = 37001,
  [SMALL_STATE(2136)] = 37008,
  [SMALL_STATE(2137)] = 37015,
  [SMALL_STATE(2138)] = 37022,
  [SMALL_STATE(2139)] = 37029,
  [SMALL_STATE(2140)] = 37036,
  [SMALL_STATE(2141)] = 37043,
  [SMALL_STATE(2142)] = 37050,
  [SMALL_STATE(2143)] = 37057,
  [SMALL_STATE(2144)] = 37064,
  [SMALL_STATE(2145)] = 37071,
  [SMALL_STATE(2146)] = 37078,
  [SMALL_STATE(2147)] = 37085,
  [SMALL_STATE(2148)] = 37092,
  [SMALL_STATE(2149)] = 37099,
  [SMALL_STATE(2150)] = 37106,
  [SMALL_STATE(2151)] = 37113,
  [SMALL_STATE(2152)] = 37120,
  [SMALL_STATE(2153)] = 37127,
  [SMALL_STATE(2154)] = 37134,
  [SMALL_STATE(2155)] = 37141,
  [SMALL_STATE(2156)] = 37148,
  [SMALL_STATE(2157)] = 37155,
  [SMALL_STATE(2158)] = 37162,
  [SMALL_STATE(2159)] = 37169,
  [SMALL_STATE(2160)] = 37176,
  [SMALL_STATE(2161)] = 37183,
  [SMALL_STATE(2162)] = 37190,
  [SMALL_STATE(2163)] = 37197,
  [SMALL_STATE(2164)] = 37204,
  [SMALL_STATE(2165)] = 37211,
  [SMALL_STATE(2166)] = 37218,
  [SMALL_STATE(2167)] = 37225,
  [SMALL_STATE(2168)] = 37232,
  [SMALL_STATE(2169)] = 37239,
  [SMALL_STATE(2170)] = 37246,
  [SMALL_STATE(2171)] = 37253,
  [SMALL_STATE(2172)] = 37260,
  [SMALL_STATE(2173)] = 37267,
  [SMALL_STATE(2174)] = 37274,
  [SMALL_STATE(2175)] = 37281,
  [SMALL_STATE(2176)] = 37288,
  [SMALL_STATE(2177)] = 37295,
  [SMALL_STATE(2178)] = 37302,
  [SMALL_STATE(2179)] = 37309,
  [SMALL_STATE(2180)] = 37316,
  [SMALL_STATE(2181)] = 37323,
  [SMALL_STATE(2182)] = 37330,
  [SMALL_STATE(2183)] = 37337,
  [SMALL_STATE(2184)] = 37344,
  [SMALL_STATE(2185)] = 37351,
  [SMALL_STATE(2186)] = 37358,
  [SMALL_STATE(2187)] = 37365,
  [SMALL_STATE(2188)] = 37372,
  [SMALL_STATE(2189)] = 37379,
  [SMALL_STATE(2190)] = 37386,
  [SMALL_STATE(2191)] = 37393,
  [SMALL_STATE(2192)] = 37400,
  [SMALL_STATE(2193)] = 37407,
  [SMALL_STATE(2194)] = 37414,
  [SMALL_STATE(2195)] = 37421,
  [SMALL_STATE(2196)] = 37428,
  [SMALL_STATE(2197)] = 37435,
  [SMALL_STATE(2198)] = 37442,
  [SMALL_STATE(2199)] = 37449,
  [SMALL_STATE(2200)] = 37456,
  [SMALL_STATE(2201)] = 37463,
  [SMALL_STATE(2202)] = 37470,
  [SMALL_STATE(2203)] = 37477,
  [SMALL_STATE(2204)] = 37484,
  [SMALL_STATE(2205)] = 37491,
  [SMALL_STATE(2206)] = 37498,
  [SMALL_STATE(2207)] = 37505,
  [SMALL_STATE(2208)] = 37512,
  [SMALL_STATE(2209)] = 37519,
  [SMALL_STATE(2210)] = 37526,
  [SMALL_STATE(2211)] = 37533,
  [SMALL_STATE(2212)] = 37540,
  [SMALL_STATE(2213)] = 37547,
  [SMALL_STATE(2214)] = 37554,
  [SMALL_STATE(2215)] = 37561,
  [SMALL_STATE(2216)] = 37568,
  [SMALL_STATE(2217)] = 37575,
  [SMALL_STATE(2218)] = 37582,
  [SMALL_STATE(2219)] = 37589,
  [SMALL_STATE(2220)] = 37596,
  [SMALL_STATE(2221)] = 37603,
  [SMALL_STATE(2222)] = 37610,
  [SMALL_STATE(2223)] = 37617,
  [SMALL_STATE(2224)] = 37624,
  [SMALL_STATE(2225)] = 37631,
  [SMALL_STATE(2226)] = 37638,
  [SMALL_STATE(2227)] = 37645,
  [SMALL_STATE(2228)] = 37652,
  [SMALL_STATE(2229)] = 37659,
  [SMALL_STATE(2230)] = 37666,
  [SMALL_STATE(2231)] = 37673,
  [SMALL_STATE(2232)] = 37680,
  [SMALL_STATE(2233)] = 37687,
  [SMALL_STATE(2234)] = 37694,
  [SMALL_STATE(2235)] = 37701,
  [SMALL_STATE(2236)] = 37708,
  [SMALL_STATE(2237)] = 37715,
  [SMALL_STATE(2238)] = 37722,
  [SMALL_STATE(2239)] = 37729,
  [SMALL_STATE(2240)] = 37736,
  [SMALL_STATE(2241)] = 37743,
  [SMALL_STATE(2242)] = 37750,
  [SMALL_STATE(2243)] = 37757,
  [SMALL_STATE(2244)] = 37764,
  [SMALL_STATE(2245)] = 37771,
  [SMALL_STATE(2246)] = 37778,
  [SMALL_STATE(2247)] = 37785,
  [SMALL_STATE(2248)] = 37792,
  [SMALL_STATE(2249)] = 37799,
  [SMALL_STATE(2250)] = 37806,
  [SMALL_STATE(2251)] = 37813,
  [SMALL_STATE(2252)] = 37820,
  [SMALL_STATE(2253)] = 37827,
  [SMALL_STATE(2254)] = 37834,
  [SMALL_STATE(2255)] = 37841,
  [SMALL_STATE(2256)] = 37848,
  [SMALL_STATE(2257)] = 37855,
  [SMALL_STATE(2258)] = 37862,
  [SMALL_STATE(2259)] = 37869,
  [SMALL_STATE(2260)] = 37876,
  [SMALL_STATE(2261)] = 37883,
  [SMALL_STATE(2262)] = 37890,
  [SMALL_STATE(2263)] = 37897,
  [SMALL_STATE(2264)] = 37904,
};

static const TSParseActionEntry ts_parse_actions[] = {
  [0] = {.entry = {.count = 0, .reusable = false}},
  [1] = {.entry = {.count = 1, .reusable = false}}, RECOVER(),
  [3] = {.entry = {.count = 1, .reusable = true}}, SHIFT_EXTRA(),
  [5] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 0, 0, 0),
  [7] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1037),
  [9] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2258),
  [11] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2211),
  [13] = {.entry = {.count = 1, .reusable = false}}, SHIFT(429),
  [15] = {.entry = {.count = 1, .reusable = false}}, SHIFT(397),
  [17] = {.entry = {.count = 1, .reusable = false}}, SHIFT(400),
  [19] = {.entry = {.count = 1, .reusable = false}}, SHIFT(405),
  [21] = {.entry = {.count = 1, .reusable = false}}, SHIFT(407),
  [23] = {.entry = {.count = 1, .reusable = false}}, SHIFT(411),
  [25] = {.entry = {.count = 1, .reusable = false}}, SHIFT(412),
  [27] = {.entry = {.count = 1, .reusable = false}}, SHIFT(413),
  [29] = {.entry = {.count = 1, .reusable = false}}, SHIFT(414),
  [31] = {.entry = {.count = 1, .reusable = false}}, SHIFT(420),
  [33] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1840),
  [35] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1061),
  [37] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1084),
  [39] = {.entry = {.count = 1, .reusable = false}}, SHIFT(422),
  [41] = {.entry = {.count = 1, .reusable = false}}, SHIFT(425),
  [43] = {.entry = {.count = 1, .reusable = false}}, SHIFT(430),
  [45] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1831),
  [47] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1428),
  [49] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1062),
  [51] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1052),
  [53] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1424),
  [55] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1995),
  [57] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1997),
  [59] = {.entry = {.count = 1, .reusable = false}}, SHIFT(811),
  [61] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2225),
  [63] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1949),
  [65] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2198),
  [67] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1960),
  [69] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1962),
  [71] = {.entry = {.count = 1, .reusable = false}}, SHIFT(666),
  [73] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1993),
  [75] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2033),
  [77] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2232),
  [79] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1827),
  [81] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1809),
  [83] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1825),
  [85] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1991),
  [87] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2020),
  [89] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2039),
  [91] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1845),
  [93] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1838),
  [95] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2167),
  [97] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2204),
  [99] = {.entry = {.count = 1, .reusable = false}}, SHIFT(771),
  [101] = {.entry = {.count = 1, .reusable = false}}, SHIFT(772),
  [103] = {.entry = {.count = 1, .reusable = false}}, SHIFT(782),
  [105] = {.entry = {.count = 1, .reusable = false}}, SHIFT(784),
  [107] = {.entry = {.count = 1, .reusable = false}}, SHIFT(710),
  [109] = {.entry = {.count = 1, .reusable = false}}, SHIFT(712),
  [111] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2002),
  [113] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2003),
  [115] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2175),
  [117] = {.entry = {.count = 1, .reusable = false}}, SHIFT(794),
  [119] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2026),
  [121] = {.entry = {.count = 1, .reusable = false}}, SHIFT(808),
  [123] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1901),
  [125] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1924),
  [127] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1889),
  [129] = {.entry = {.count = 1, .reusable = false}}, SHIFT(775),
  [131] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1261),
  [133] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1230),
  [135] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1429),
  [137] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1171),
  [139] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2064),
  [141] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1888),
  [143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2228),
  [145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2040),
  [147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2101),
  [149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2091),
  [151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2087),
  [153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(783),
  [155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(48),
  [157] = {.entry = {.count = 1, .reusable = false}}, SHIFT(679),
  [159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(775),
  [161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(797),
  [163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(781),
  [165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1792),
  [167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1262),
  [169] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2088),
  [171] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2089),
  [173] = {.entry = {.count = 1, .reusable = false}}, SHIFT(363),
  [175] = {.entry = {.count = 1, .reusable = false}}, SHIFT(364),
  [177] = {.entry = {.count = 1, .reusable = false}}, SHIFT(365),
  [179] = {.entry = {.count = 1, .reusable = false}}, SHIFT(366),
  [181] = {.entry = {.count = 1, .reusable = false}}, SHIFT(367),
  [183] = {.entry = {.count = 1, .reusable = false}}, SHIFT(368),
  [185] = {.entry = {.count = 1, .reusable = false}}, SHIFT(369),
  [187] = {.entry = {.count = 1, .reusable = false}}, SHIFT(370),
  [189] = {.entry = {.count = 1, .reusable = false}}, SHIFT(373),
  [191] = {.entry = {.count = 1, .reusable = false}}, SHIFT(374),
  [193] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1812),
  [195] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1058),
  [197] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1059),
  [199] = {.entry = {.count = 1, .reusable = false}}, SHIFT(375),
  [201] = {.entry = {.count = 1, .reusable = false}}, SHIFT(339),
  [203] = {.entry = {.count = 1, .reusable = false}}, SHIFT(318),
  [205] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1820),
  [207] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1998),
  [209] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1999),
  [211] = {.entry = {.count = 1, .reusable = false}}, SHIFT(869),
  [213] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2150),
  [215] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2001),
  [217] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2252),
  [219] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1994),
  [221] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2011),
  [223] = {.entry = {.count = 1, .reusable = false}}, SHIFT(668),
  [225] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2012),
  [227] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2035),
  [229] = {.entry = {.count = 1, .reusable = false}}, SHIFT(904),
  [231] = {.entry = {.count = 1, .reusable = false}}, SHIFT(896),
  [233] = {.entry = {.count = 1, .reusable = false}}, SHIFT(897),
  [235] = {.entry = {.count = 1, .reusable = false}}, SHIFT(870),
  [237] = {.entry = {.count = 1, .reusable = false}}, SHIFT(729),
  [239] = {.entry = {.count = 1, .reusable = false}}, SHIFT(730),
  [241] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2005),
  [243] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2006),
  [245] = {.entry = {.count = 1, .reusable = false}}, SHIFT(801),
  [247] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2007),
  [249] = {.entry = {.count = 1, .reusable = false}}, SHIFT(802),
  [251] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2008),
  [253] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2016),
  [255] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2017),
  [257] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2009),
  [259] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2169),
  [261] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2010),
  [263] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2171),
  [265] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2172),
  [267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(8),
  [269] = {.entry = {.count = 1, .reusable = false}}, SHIFT(403),
  [271] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2173),
  [273] = {.entry = {.count = 1, .reusable = false}}, SHIFT(28),
  [275] = {.entry = {.count = 1, .reusable = false}}, SHIFT(783),
  [277] = {.entry = {.count = 1, .reusable = false}}, SHIFT(47),
  [279] = {.entry = {.count = 1, .reusable = false}}, SHIFT(797),
  [281] = {.entry = {.count = 1, .reusable = false}}, SHIFT(781),
  [283] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1792),
  [285] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1262),
  [287] = {.entry = {.count = 1, .reusable = false}}, SHIFT_EXTRA(),
  [289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(11),
  [291] = {.entry = {.count = 1, .reusable = false}}, SHIFT(406),
  [293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(19),
  [295] = {.entry = {.count = 1, .reusable = false}}, SHIFT(555),
  [297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(14),
  [299] = {.entry = {.count = 1, .reusable = false}}, SHIFT(525),
  [301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(22),
  [303] = {.entry = {.count = 1, .reusable = false}}, SHIFT(351),
  [305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(20),
  [307] = {.entry = {.count = 1, .reusable = false}}, SHIFT(350),
  [309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2169),
  [311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2010),
  [313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2171),
  [315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2172),
  [317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(421),
  [319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2173),
  [321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(28),
  [323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(47),
  [325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(448),
  [327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(424),
  [329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(450),
  [331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(523),
  [333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(568),
  [335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(546),
  [337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(534),
  [339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(345),
  [341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(310),
  [343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(356),
  [345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(270),
  [347] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1037),
  [350] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2088),
  [353] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2089),
  [356] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(363),
  [359] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(364),
  [362] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(365),
  [365] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(366),
  [368] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(367),
  [371] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(368),
  [374] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(369),
  [377] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(370),
  [380] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(373),
  [383] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(374),
  [386] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1812),
  [389] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1058),
  [392] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1059),
  [395] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(375),
  [398] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(339),
  [401] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(318),
  [404] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1820),
  [407] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1428),
  [410] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1062),
  [413] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1052),
  [416] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1424),
  [419] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1998),
  [422] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1999),
  [425] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(869),
  [428] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2150),
  [431] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2001),
  [434] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2252),
  [437] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1994),
  [440] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2011),
  [443] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(668),
  [446] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2012),
  [449] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2035),
  [452] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2232),
  [455] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1827),
  [458] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1809),
  [461] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1825),
  [464] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1991),
  [467] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2020),
  [470] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2039),
  [473] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1845),
  [476] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1838),
  [479] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2167),
  [482] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2204),
  [485] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(904),
  [488] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(896),
  [491] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(897),
  [494] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(870),
  [497] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(729),
  [500] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(730),
  [503] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2005),
  [506] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2006),
  [509] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2175),
  [512] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(801),
  [515] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2007),
  [518] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(802),
  [521] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2008),
  [524] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2016),
  [527] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2017),
  [530] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(775),
  [533] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1261),
  [536] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1230),
  [539] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1429),
  [542] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1171),
  [545] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2064),
  [548] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2009),
  [551] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2169),
  [554] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2010),
  [557] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2171),
  [560] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2172),
  [563] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0),
  [565] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2173),
  [568] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(783),
  [571] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(47),
  [574] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(679),
  [577] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(775),
  [580] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(797),
  [583] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(781),
  [586] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1792),
  [589] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1262),
  [592] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 1, 0, 0),
  [594] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else, 1, 20, 0),
  [596] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2258),
  [599] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2211),
  [602] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(429),
  [605] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(397),
  [608] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(400),
  [611] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(405),
  [614] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(407),
  [617] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(411),
  [620] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(412),
  [623] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(413),
  [626] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(414),
  [629] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(420),
  [632] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1840),
  [635] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1061),
  [638] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1084),
  [641] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(422),
  [644] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(425),
  [647] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(430),
  [650] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1831),
  [653] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1995),
  [656] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1997),
  [659] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(811),
  [662] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2225),
  [665] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1949),
  [668] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2198),
  [671] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1960),
  [674] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1962),
  [677] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(666),
  [680] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1993),
  [683] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2033),
  [686] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(771),
  [689] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(772),
  [692] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(782),
  [695] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(784),
  [698] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(710),
  [701] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(712),
  [704] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2002),
  [707] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2003),
  [710] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(794),
  [713] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2026),
  [716] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(808),
  [719] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1901),
  [722] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1924),
  [725] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1889),
  [728] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(1888),
  [731] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2228),
  [734] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2040),
  [737] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2101),
  [740] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2091),
  [743] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(2087),
  [746] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(48),
  [749] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else, 2, 20, 0),
  [751] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 2, 0, 0),
  [753] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_statement, 2, 0, 0),
  [755] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 3, 0, 20),
  [757] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_statement, 3, 0, 20),
  [759] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 3, 0, 0),
  [761] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_statement, 3, 0, 0),
  [763] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 4, 0, 20),
  [765] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_statement, 4, 0, 20),
  [767] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1037),
  [770] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0),
  [772] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1428),
  [775] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1062),
  [778] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1052),
  [781] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1424),
  [784] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1998),
  [787] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1999),
  [790] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2001),
  [793] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2252),
  [796] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1994),
  [799] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2011),
  [802] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(668),
  [805] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2012),
  [808] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2035),
  [811] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2167),
  [814] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2204),
  [817] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(904),
  [820] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(896),
  [823] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(897),
  [826] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(870),
  [829] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(729),
  [832] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(730),
  [835] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2005),
  [838] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2006),
  [841] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2175),
  [844] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(801),
  [847] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2007),
  [850] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(802),
  [853] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2008),
  [856] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2016),
  [859] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2017),
  [862] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(775),
  [865] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1261),
  [868] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1230),
  [871] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1429),
  [874] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1171),
  [877] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2064),
  [880] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0),
  [882] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(783),
  [885] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(47),
  [888] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(679),
  [891] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(775),
  [894] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(797),
  [897] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(781),
  [900] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1792),
  [903] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1262),
  [906] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1995),
  [909] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1997),
  [912] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1949),
  [915] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2198),
  [918] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1960),
  [921] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1962),
  [924] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(666),
  [927] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1993),
  [930] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2033),
  [933] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(771),
  [936] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(772),
  [939] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(782),
  [942] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(784),
  [945] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(710),
  [948] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(712),
  [951] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2002),
  [954] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2003),
  [957] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(794),
  [960] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2026),
  [963] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(808),
  [966] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1901),
  [969] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1924),
  [972] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1889),
  [975] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(48),
  [978] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1847),
  [980] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1095),
  [982] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1096),
  [984] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1852),
  [986] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1969),
  [988] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1970),
  [990] = {.entry = {.count = 1, .reusable = false}}, SHIFT(842),
  [992] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2222),
  [994] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2048),
  [996] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2247),
  [998] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1896),
  [1000] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1897),
  [1002] = {.entry = {.count = 1, .reusable = false}}, SHIFT(664),
  [1004] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1898),
  [1006] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2028),
  [1008] = {.entry = {.count = 1, .reusable = false}}, SHIFT(903),
  [1010] = {.entry = {.count = 1, .reusable = false}}, SHIFT(892),
  [1012] = {.entry = {.count = 1, .reusable = false}}, SHIFT(893),
  [1014] = {.entry = {.count = 1, .reusable = false}}, SHIFT(843),
  [1016] = {.entry = {.count = 1, .reusable = false}}, SHIFT(726),
  [1018] = {.entry = {.count = 1, .reusable = false}}, SHIFT(727),
  [1020] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1972),
  [1022] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1973),
  [1024] = {.entry = {.count = 1, .reusable = false}}, SHIFT(785),
  [1026] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1974),
  [1028] = {.entry = {.count = 1, .reusable = false}}, SHIFT(786),
  [1030] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1975),
  [1032] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1899),
  [1034] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1900),
  [1036] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1976),
  [1038] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2102),
  [1040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1977),
  [1042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2104),
  [1044] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2105),
  [1046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2106),
  [1048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(44),
  [1050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(196),
  [1052] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1352),
  [1054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(481),
  [1056] = {.entry = {.count = 1, .reusable = true}}, SHIFT(518),
  [1058] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1037),
  [1061] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1847),
  [1064] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1095),
  [1067] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1096),
  [1070] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1852),
  [1073] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1428),
  [1076] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1062),
  [1079] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1052),
  [1082] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1424),
  [1085] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1969),
  [1088] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1970),
  [1091] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(842),
  [1094] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2222),
  [1097] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2048),
  [1100] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2247),
  [1103] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1896),
  [1106] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1897),
  [1109] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(664),
  [1112] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1898),
  [1115] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2028),
  [1118] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2232),
  [1121] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2167),
  [1124] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2204),
  [1127] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(903),
  [1130] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(892),
  [1133] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(893),
  [1136] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(843),
  [1139] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(726),
  [1142] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(727),
  [1145] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1972),
  [1148] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1973),
  [1151] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2175),
  [1154] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(785),
  [1157] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1974),
  [1160] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(786),
  [1163] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1975),
  [1166] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1899),
  [1169] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1900),
  [1172] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(775),
  [1175] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1261),
  [1178] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1230),
  [1181] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1429),
  [1184] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1171),
  [1187] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2064),
  [1190] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1976),
  [1193] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2102),
  [1196] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1977),
  [1199] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2104),
  [1202] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2105),
  [1205] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2106),
  [1208] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(783),
  [1211] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(44),
  [1214] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0),
  [1216] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(679),
  [1219] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(775),
  [1222] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(797),
  [1225] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(781),
  [1228] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1792),
  [1231] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_compound_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1262),
  [1234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(266),
  [1236] = {.entry = {.count = 1, .reusable = true}}, SHIFT(251),
  [1238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(197),
  [1240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1351),
  [1242] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_string_expression, 2, 0, 0), SHIFT(1269),
  [1245] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_expression, 2, 0, 0),
  [1247] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_string_expression, 2, 0, 0), SHIFT(867),
  [1250] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_string_expression, 2, 0, 0), SHIFT(1155),
  [1253] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_string_expression, 2, 0, 0), SHIFT(1156),
  [1256] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_string_expression, 2, 0, 0), SHIFT(1092),
  [1259] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_string_expression, 2, 0, 0), SHIFT(2064),
  [1262] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_expression, 2, 0, 0),
  [1264] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_string_expression, 2, 0, 0), SHIFT(894),
  [1267] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_string_expression, 2, 0, 0), SHIFT(682),
  [1270] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_string_expression, 2, 0, 0), SHIFT(867),
  [1273] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_string_expression, 2, 0, 0), SHIFT(854),
  [1276] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_string_expression, 2, 0, 0), SHIFT(573),
  [1279] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_string_expression, 2, 0, 0), SHIFT(1762),
  [1282] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_string_expression, 2, 0, 0), SHIFT(1154),
  [1285] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1969),
  [1288] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1970),
  [1291] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2048),
  [1294] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2247),
  [1297] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1896),
  [1300] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1897),
  [1303] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(664),
  [1306] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1898),
  [1309] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2028),
  [1312] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(903),
  [1315] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(892),
  [1318] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(893),
  [1321] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(843),
  [1324] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(726),
  [1327] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(727),
  [1330] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1972),
  [1333] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1973),
  [1336] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(785),
  [1339] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1974),
  [1342] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(786),
  [1345] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1975),
  [1348] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1899),
  [1351] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1900),
  [1354] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(44),
  [1357] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_wait_statement, 2, 0, 9),
  [1359] = {.entry = {.count = 1, .reusable = false}}, SHIFT(755),
  [1361] = {.entry = {.count = 1, .reusable = false}}, SHIFT(756),
  [1363] = {.entry = {.count = 1, .reusable = false}}, SHIFT(757),
  [1365] = {.entry = {.count = 1, .reusable = false}}, SHIFT(758),
  [1367] = {.entry = {.count = 1, .reusable = false}}, SHIFT(759),
  [1369] = {.entry = {.count = 1, .reusable = false}}, SHIFT(760),
  [1371] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_wait_statement, 2, 0, 9),
  [1373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(725),
  [1375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(891),
  [1377] = {.entry = {.count = 1, .reusable = false}}, SHIFT(678),
  [1379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(262),
  [1381] = {.entry = {.count = 1, .reusable = false}}, SHIFT(761),
  [1383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(762),
  [1385] = {.entry = {.count = 1, .reusable = false}}, SHIFT(762),
  [1387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(756),
  [1389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(755),
  [1391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(757),
  [1393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(763),
  [1395] = {.entry = {.count = 1, .reusable = false}}, SHIFT(764),
  [1397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(764),
  [1399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(760),
  [1401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(118),
  [1403] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2061),
  [1405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1767),
  [1407] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_wait_until_statement, 2, 0, 10),
  [1409] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_wait_until_statement, 2, 0, 10),
  [1411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(264),
  [1413] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_binary_expression, 3, 0, 29),
  [1415] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_binary_expression, 3, 0, 29),
  [1417] = {.entry = {.count = 1, .reusable = false}}, SHIFT(834),
  [1419] = {.entry = {.count = 1, .reusable = false}}, SHIFT(836),
  [1421] = {.entry = {.count = 1, .reusable = false}}, SHIFT(837),
  [1423] = {.entry = {.count = 1, .reusable = false}}, SHIFT(838),
  [1425] = {.entry = {.count = 1, .reusable = false}}, SHIFT(846),
  [1427] = {.entry = {.count = 1, .reusable = false}}, SHIFT(866),
  [1429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(716),
  [1431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(874),
  [1433] = {.entry = {.count = 1, .reusable = false}}, SHIFT(673),
  [1435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(341),
  [1437] = {.entry = {.count = 1, .reusable = false}}, SHIFT(873),
  [1439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(881),
  [1441] = {.entry = {.count = 1, .reusable = false}}, SHIFT(881),
  [1443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(836),
  [1445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(834),
  [1447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(837),
  [1449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(884),
  [1451] = {.entry = {.count = 1, .reusable = false}}, SHIFT(890),
  [1453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(890),
  [1455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(866),
  [1457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(135),
  [1459] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2093),
  [1461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1729),
  [1463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(321),
  [1465] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression, 1, 0, 0),
  [1467] = {.entry = {.count = 1, .reusable = false}}, SHIFT(698),
  [1469] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression, 1, 0, 0),
  [1471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1964),
  [1473] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unary_expression, 2, 0, 11),
  [1475] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unary_expression, 2, 0, 11),
  [1477] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_update_expression, 2, 0, 11),
  [1479] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_update_expression, 2, 0, 11),
  [1481] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_device_literal, 5, 46, 73),
  [1483] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73),
  [1485] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73), SHIFT(841),
  [1488] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_subscript_expression, 2, 0, 15),
  [1490] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subscript_expression, 2, 0, 15),
  [1492] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_assignment_expression, 3, 0, 29),
  [1494] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment_expression, 3, 0, 29),
  [1496] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_subscript_expression_repeat1, 2, 0, 36),
  [1498] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_subscript_expression_repeat1, 2, 0, 36),
  [1500] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_subscript_expression_repeat1, 2, 0, 36), SHIFT_REPEAT(725),
  [1503] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_assignment_expression, 3, 0, 37),
  [1505] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment_expression, 3, 0, 37),
  [1507] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_devchan_expression, 5, 28, 68),
  [1509] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_devchan_expression, 5, 28, 68),
  [1511] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_expression, 2, 0, 14),
  [1513] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_expression, 2, 0, 14),
  [1515] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_subscript_expression_repeat1, 2, 0, 36), SHIFT_REPEAT(716),
  [1518] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73), SHIFT(827),
  [1521] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_system_variable, 1, 0, 0),
  [1523] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_system_variable, 1, 0, 0),
  [1525] = {.entry = {.count = 1, .reusable = false}}, SHIFT(699),
  [1527] = {.entry = {.count = 1, .reusable = false}}, SHIFT(707),
  [1529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1891),
  [1531] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_subscript_expression_repeat1, 2, 0, 0),
  [1533] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_subscript_expression_repeat1, 2, 0, 0),
  [1535] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 2, 0, 0),
  [1537] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 2, 0, 0),
  [1539] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_expression, 3, 0, 35),
  [1541] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_expression, 3, 0, 35),
  [1543] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_subscript_expression_repeat1, 3, 0, 56),
  [1545] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_subscript_expression_repeat1, 3, 0, 56),
  [1547] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 3, 0, 0),
  [1549] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 3, 0, 0),
  [1551] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 4, 0, 0),
  [1553] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 4, 0, 0),
  [1555] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_expression, 3, 0, 0),
  [1557] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_expression, 3, 0, 0),
  [1559] = {.entry = {.count = 1, .reusable = false}}, SHIFT(709),
  [1561] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_update_expression, 2, 0, 13),
  [1563] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_update_expression, 2, 0, 13),
  [1565] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_compiler_variable, 1, 0, 0),
  [1567] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_compiler_variable, 1, 0, 0),
  [1569] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_initializer_list, 2, 0, 0),
  [1571] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_initializer_list, 2, 0, 0),
  [1573] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_devchan_range_expression, 3, 16, 38),
  [1575] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_devchan_range_expression, 3, 16, 38),
  [1577] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_initializer_list, 3, 0, 0),
  [1579] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_initializer_list, 3, 0, 0),
  [1581] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_expression, 4, 0, 0),
  [1583] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_expression, 4, 0, 0),
  [1585] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_expression, 3, 0, 0),
  [1587] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_expression, 3, 0, 0),
  [1589] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_initializer_list, 4, 0, 0),
  [1591] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_initializer_list, 4, 0, 0),
  [1593] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_system_constant, 1, 0, 0),
  [1595] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_system_constant, 1, 0, 0),
  [1597] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_literal, 2, 0, 0),
  [1599] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_literal, 2, 0, 0),
  [1601] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_literal, 3, 0, 0),
  [1603] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_literal, 3, 0, 0),
  [1605] = {.entry = {.count = 1, .reusable = false}}, SHIFT(738),
  [1607] = {.entry = {.count = 1, .reusable = false}}, SHIFT(739),
  [1609] = {.entry = {.count = 1, .reusable = false}}, SHIFT(740),
  [1611] = {.entry = {.count = 1, .reusable = false}}, SHIFT(741),
  [1613] = {.entry = {.count = 1, .reusable = false}}, SHIFT(742),
  [1615] = {.entry = {.count = 1, .reusable = false}}, SHIFT(743),
  [1617] = {.entry = {.count = 1, .reusable = true}}, SHIFT(724),
  [1619] = {.entry = {.count = 1, .reusable = true}}, SHIFT(888),
  [1621] = {.entry = {.count = 1, .reusable = false}}, SHIFT(675),
  [1623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(478),
  [1625] = {.entry = {.count = 1, .reusable = false}}, SHIFT(744),
  [1627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(745),
  [1629] = {.entry = {.count = 1, .reusable = false}}, SHIFT(745),
  [1631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(739),
  [1633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(738),
  [1635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(740),
  [1637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(746),
  [1639] = {.entry = {.count = 1, .reusable = false}}, SHIFT(747),
  [1641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(747),
  [1643] = {.entry = {.count = 1, .reusable = true}}, SHIFT(743),
  [1645] = {.entry = {.count = 1, .reusable = true}}, SHIFT(185),
  [1647] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2226),
  [1649] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1765),
  [1651] = {.entry = {.count = 1, .reusable = true}}, SHIFT(475),
  [1653] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_subscript_expression_repeat1, 2, 0, 36), SHIFT_REPEAT(724),
  [1656] = {.entry = {.count = 1, .reusable = false}}, SHIFT(694),
  [1658] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1952),
  [1660] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73), SHIFT(839),
  [1663] = {.entry = {.count = 1, .reusable = false}}, SHIFT(695),
  [1665] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_compound_statement, 3, 0, 0),
  [1667] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_compound_statement, 3, 0, 0),
  [1669] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_compound_statement, 2, 0, 0),
  [1671] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_compound_statement, 2, 0, 0),
  [1673] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_wait_statement, 3, 0, 23),
  [1675] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_wait_statement, 3, 0, 23),
  [1677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(213),
  [1679] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_wait_until_statement, 3, 0, 25),
  [1681] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_wait_until_statement, 3, 0, 25),
  [1683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(214),
  [1685] = {.entry = {.count = 1, .reusable = true}}, SHIFT(273),
  [1687] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 3, 0, 18),
  [1689] = {.entry = {.count = 1, .reusable = false}}, SHIFT(566),
  [1691] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 3, 0, 18),
  [1693] = {.entry = {.count = 1, .reusable = true}}, SHIFT(308),
  [1695] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_define, 4, 0, 47),
  [1697] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_define, 4, 0, 47),
  [1699] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_include, 3, 0, 26),
  [1701] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_include, 3, 0, 26),
  [1703] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_warn, 3, 0, 27),
  [1705] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_warn, 3, 0, 27),
  [1707] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_define, 3, 0, 24),
  [1709] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_define, 3, 0, 24),
  [1711] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_disable_warning, 3, 0, 28),
  [1713] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_disable_warning, 3, 0, 28),
  [1715] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 4, 0, 49),
  [1717] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 4, 0, 49),
  [1719] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 4, 0, 44),
  [1721] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 4, 0, 44),
  [1723] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_active_statement, 4, 0, 45),
  [1725] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_active_statement, 4, 0, 45),
  [1727] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_statement, 4, 0, 46),
  [1729] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_statement, 4, 0, 46),
  [1731] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_system_call_statement, 4, 0, 46),
  [1733] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_system_call_statement, 4, 0, 46),
  [1735] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_wait_statement, 4, 0, 23),
  [1737] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_wait_statement, 4, 0, 23),
  [1739] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_wait_until_statement, 4, 0, 25),
  [1741] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_wait_until_statement, 4, 0, 25),
  [1743] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_else_clause, 2, 0, 0),
  [1745] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_else_clause, 2, 0, 0),
  [1747] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_statement, 5, 0, 63),
  [1749] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_statement, 5, 0, 63),
  [1751] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_send_string_statement, 5, 0, 66),
  [1753] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_send_string_statement, 5, 0, 66),
  [1755] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_send_command_statement, 5, 0, 66),
  [1757] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_send_command_statement, 5, 0, 66),
  [1759] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_send_level_statement, 7, 0, 86),
  [1761] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_send_level_statement, 7, 0, 86),
  [1763] = {.entry = {.count = 1, .reusable = false}}, SHIFT(531),
  [1765] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 3, 0, 30),
  [1767] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 3, 0, 30),
  [1769] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 3, 0, 32),
  [1771] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 3, 0, 32),
  [1773] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 4, 0, 50),
  [1775] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 4, 0, 50),
  [1777] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 4, 0, 54),
  [1779] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 4, 0, 54),
  [1781] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 4, 0, 55),
  [1783] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 4, 0, 55),
  [1785] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_devchan_operation_statement, 3, 0, 33),
  [1787] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_devchan_operation_statement, 3, 0, 33),
  [1789] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 5, 0, 70),
  [1791] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 5, 0, 70),
  [1793] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 5, 0, 72),
  [1795] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 5, 0, 72),
  [1797] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module_name, 3, 0, 0),
  [1799] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module_name, 3, 0, 0),
  [1801] = {.entry = {.count = 1, .reusable = false}}, SHIFT(985),
  [1803] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_break_statement, 2, 0, 0),
  [1805] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break_statement, 2, 0, 0),
  [1807] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_continue_statement, 2, 0, 0),
  [1809] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_continue_statement, 2, 0, 0),
  [1811] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return_statement, 2, 0, 0),
  [1813] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_statement, 2, 0, 0),
  [1815] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_select_statement, 2, 0, 3),
  [1817] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_select_statement, 2, 0, 3),
  [1819] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_cancel_all_wait_statement, 2, 0, 0),
  [1821] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_cancel_all_wait_statement, 2, 0, 0),
  [1823] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_cancel_all_wait_until_statement, 2, 0, 0),
  [1825] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_cancel_all_wait_until_statement, 2, 0, 0),
  [1827] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_statement, 1, 0, 0),
  [1829] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_statement, 1, 0, 0),
  [1831] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_statement, 2, 0, 0),
  [1833] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_statement, 2, 0, 0),
  [1835] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_switch_statement, 3, 0, 19),
  [1837] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_switch_statement, 3, 0, 19),
  [1839] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_while_statement, 3, 0, 19),
  [1841] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_statement, 3, 0, 19),
  [1843] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return_statement, 3, 0, 0),
  [1845] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_statement, 3, 0, 0),
  [1847] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_clear_buffer_statement, 3, 0, 0),
  [1849] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_clear_buffer_statement, 3, 0, 0),
  [1851] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_create_buffer_statement, 3, 0, 0),
  [1853] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_create_buffer_statement, 3, 0, 0),
  [1855] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_create_multi_buffer_statement, 3, 0, 0),
  [1857] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_create_multi_buffer_statement, 3, 0, 0),
  [1859] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_statement, 3, 0, 22),
  [1861] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_statement, 3, 0, 22),
  [1863] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_system_call_statement, 3, 0, 22),
  [1865] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_system_call_statement, 3, 0, 22),
  [1867] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_wait_statement, 3, 0, 9),
  [1869] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_wait_statement, 3, 0, 9),
  [1871] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_cancel_wait_statement, 3, 0, 24),
  [1873] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_cancel_wait_statement, 3, 0, 24),
  [1875] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_wait_until_statement, 3, 0, 10),
  [1877] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_wait_until_statement, 3, 0, 10),
  [1879] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_cancel_wait_until_statement, 3, 0, 24),
  [1881] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_cancel_wait_until_statement, 3, 0, 24),
  [1883] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined, 5, 20, 67),
  [1885] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined, 5, 20, 67),
  [1887] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined, 5, 20, 24),
  [1889] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined, 5, 20, 24),
  [1891] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameter_list, 3, 0, 0),
  [1893] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_list, 3, 0, 0),
  [1895] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_level_event_definition, 2, 0, 3),
  [1897] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_level_event_definition, 2, 0, 3),
  [1899] = {.entry = {.count = 1, .reusable = false}}, SHIFT(997),
  [1901] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 5, 0, 74),
  [1903] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 5, 0, 74),
  [1905] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declaration, 5, 0, 43),
  [1907] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 5, 0, 43),
  [1909] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined, 6, 20, 82),
  [1911] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined, 6, 20, 82),
  [1913] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined, 6, 20, 82),
  [1915] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined, 6, 20, 82),
  [1917] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_channel_event_definition, 2, 0, 3),
  [1919] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_channel_event_definition, 2, 0, 3),
  [1921] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameter_list, 4, 0, 0),
  [1923] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_list, 4, 0, 0),
  [1925] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_custom_event_definition, 2, 0, 3),
  [1927] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_custom_event_definition, 2, 0, 3),
  [1929] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_push_event_definition, 2, 0, 3),
  [1931] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_push_event_definition, 2, 0, 3),
  [1933] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_release_event_definition, 2, 0, 3),
  [1935] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_release_event_definition, 2, 0, 3),
  [1937] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_program_name, 3, 0, 0),
  [1939] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program_name, 3, 0, 0),
  [1941] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declaration, 6, 0, 60),
  [1943] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 6, 0, 60),
  [1945] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_data_event_block, 2, 0, 0),
  [1947] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_event_block, 2, 0, 0),
  [1949] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_button_event_block, 2, 0, 0),
  [1951] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_button_event_block, 2, 0, 0),
  [1953] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_channel_event_block, 2, 0, 0),
  [1955] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_channel_event_block, 2, 0, 0),
  [1957] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module_name, 4, 0, 39),
  [1959] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module_name, 4, 0, 39),
  [1961] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_data_event_block, 3, 0, 0),
  [1963] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_event_block, 3, 0, 0),
  [1965] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_button_event_block, 3, 0, 0),
  [1967] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_button_event_block, 3, 0, 0),
  [1969] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_channel_event_block, 3, 0, 0),
  [1971] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_channel_event_block, 3, 0, 0),
  [1973] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_call, 2, 0, 0),
  [1975] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_call, 2, 0, 0),
  [1977] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_function, 2, 0, 0),
  [1979] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_function, 2, 0, 0),
  [1981] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_module, 2, 0, 0),
  [1983] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_module, 2, 0, 0),
  [1985] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined, 4, 20, 48),
  [1987] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined, 4, 20, 48),
  [1989] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined, 4, 20, 24),
  [1991] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined, 4, 20, 24),
  [1993] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_data_event_definition, 2, 0, 3),
  [1995] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_event_definition, 2, 0, 3),
  [1997] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_timeline_event_definition, 2, 0, 3),
  [1999] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_timeline_event_definition, 2, 0, 3),
  [2001] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined, 4, 20, 48),
  [2003] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined, 4, 20, 48),
  [2005] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_program_section, 1, 0, 0),
  [2007] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_program_section, 1, 0, 0),
  [2009] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_section, 1, 0, 0),
  [2011] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_section, 1, 0, 0),
  [2013] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_definition, 2, 0, 0),
  [2015] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_definition, 2, 0, 0),
  [2017] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_event_definition, 1, 0, 0),
  [2019] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_event_definition, 1, 0, 0),
  [2021] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_definition, 2, 0, 17),
  [2023] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_definition, 2, 0, 17),
  [2025] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_definition, 4, 0, 57),
  [2027] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_definition, 4, 0, 57),
  [2029] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declaration, 3, 0, 43),
  [2031] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 3, 0, 43),
  [2033] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_button_event_definition, 2, 0, 3),
  [2035] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_button_event_definition, 2, 0, 3),
  [2037] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 4, 0, 57),
  [2039] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 4, 0, 57),
  [2041] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 4, 0, 59),
  [2043] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 4, 0, 59),
  [2045] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declaration, 4, 0, 60),
  [2047] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declaration, 4, 0, 60),
  [2049] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module_definition, 4, 0, 61),
  [2051] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module_definition, 4, 0, 61),
  [2053] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_event_section, 1, 0, 0),
  [2055] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_event_section, 1, 0, 0),
  [2057] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined, 4, 20, 24),
  [2059] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined, 4, 20, 24),
  [2061] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined, 3, 20, 24),
  [2063] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined, 3, 20, 24),
  [2065] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined, 3, 20, 24),
  [2067] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined, 3, 20, 24),
  [2069] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined, 5, 20, 67),
  [2071] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined, 5, 20, 67),
  [2073] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined, 5, 20, 24),
  [2075] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined, 5, 20, 24),
  [2077] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_device_section, 1, 0, 0),
  [2079] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_device_section, 1, 0, 0),
  [2081] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_combine_section, 1, 0, 0),
  [2083] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_combine_section, 1, 0, 0),
  [2085] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_constant_section, 1, 0, 0),
  [2087] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_constant_section, 1, 0, 0),
  [2089] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_type_section, 1, 0, 0),
  [2091] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_type_section, 1, 0, 0),
  [2093] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_variable_section, 1, 0, 0),
  [2095] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_variable_section, 1, 0, 0),
  [2097] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_system_variable_section, 1, 0, 0),
  [2099] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_system_variable_section, 1, 0, 0),
  [2101] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_connect_level_section, 1, 0, 0),
  [2103] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_connect_level_section, 1, 0, 0),
  [2105] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_latching_section, 1, 0, 0),
  [2107] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_latching_section, 1, 0, 0),
  [2109] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_mutually_exclusive_section, 1, 0, 0),
  [2111] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_mutually_exclusive_section, 1, 0, 0),
  [2113] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_toggling_section, 1, 0, 0),
  [2115] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_toggling_section, 1, 0, 0),
  [2117] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_define_start_section, 1, 0, 0),
  [2119] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_define_start_section, 1, 0, 0),
  [2121] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_definition, 3, 0, 41),
  [2123] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_definition, 3, 0, 41),
  [2125] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_definition, 3, 0, 42),
  [2127] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_definition, 3, 0, 42),
  [2129] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 3, 0, 41),
  [2131] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 3, 0, 41),
  [2133] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameter_list, 2, 0, 0),
  [2135] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_list, 2, 0, 0),
  [2137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(495),
  [2139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(493),
  [2141] = {.entry = {.count = 1, .reusable = false}}, SHIFT(559),
  [2143] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1163),
  [2145] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1269),
  [2147] = {.entry = {.count = 1, .reusable = false}}, SHIFT(867),
  [2149] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1155),
  [2151] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1156),
  [2153] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1092),
  [2155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(867),
  [2157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(573),
  [2159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1762),
  [2161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1154),
  [2163] = {.entry = {.count = 1, .reusable = false}}, SHIFT(737),
  [2165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(723),
  [2167] = {.entry = {.count = 1, .reusable = false}}, SHIFT(672),
  [2169] = {.entry = {.count = 1, .reusable = false}}, SHIFT(748),
  [2171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(749),
  [2173] = {.entry = {.count = 1, .reusable = false}}, SHIFT(749),
  [2175] = {.entry = {.count = 1, .reusable = false}}, SHIFT(753),
  [2177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(753),
  [2179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(737),
  [2181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(608),
  [2183] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2124),
  [2185] = {.entry = {.count = 1, .reusable = false}}, SHIFT(795),
  [2187] = {.entry = {.count = 1, .reusable = false}}, SHIFT(796),
  [2189] = {.entry = {.count = 1, .reusable = false}}, SHIFT(844),
  [2191] = {.entry = {.count = 1, .reusable = false}}, SHIFT(845),
  [2193] = {.entry = {.count = 1, .reusable = false}}, SHIFT(855),
  [2195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(885),
  [2197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(796),
  [2199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(795),
  [2201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(844),
  [2203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(750),
  [2205] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_subscript_expression_repeat1, 2, 0, 36), SHIFT_REPEAT(723),
  [2208] = {.entry = {.count = 1, .reusable = false}}, SHIFT(689),
  [2210] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1934),
  [2212] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73), SHIFT(831),
  [2215] = {.entry = {.count = 1, .reusable = false}}, SHIFT(690),
  [2217] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0),
  [2219] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0),
  [2221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(920),
  [2223] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0), SHIFT(1279),
  [2226] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0), SHIFT(606),
  [2229] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0), SHIFT(615),
  [2232] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0), SHIFT(1233),
  [2235] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0), SHIFT(1132),
  [2238] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0), SHIFT(1133),
  [2241] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0), SHIFT(147),
  [2244] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0), SHIFT(148),
  [2247] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0), SHIFT(187),
  [2250] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0), SHIFT(184),
  [2253] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0), SHIFT(125),
  [2256] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 1, 0, 0), SHIFT(129),
  [2259] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0),
  [2261] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0),
  [2263] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(943),
  [2266] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0),
  [2268] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0),
  [2270] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(933),
  [2273] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 1, 0, 0),
  [2276] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 1, 0, 0),
  [2279] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 1, 0, 0),
  [2281] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 1, 0, 0),
  [2283] = {.entry = {.count = 1, .reusable = false}}, SHIFT(600),
  [2285] = {.entry = {.count = 1, .reusable = false}}, SHIFT(822),
  [2287] = {.entry = {.count = 1, .reusable = false}}, SHIFT(611),
  [2289] = {.entry = {.count = 1, .reusable = false}}, SHIFT(605),
  [2291] = {.entry = {.count = 1, .reusable = false}}, SHIFT(595),
  [2293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2179),
  [2295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2189),
  [2297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(900),
  [2299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(639),
  [2301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(123),
  [2303] = {.entry = {.count = 1, .reusable = false}}, SHIFT(684),
  [2305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(822),
  [2307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(887),
  [2309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(824),
  [2311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1805),
  [2313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(618),
  [2315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1132),
  [2317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(651),
  [2319] = {.entry = {.count = 1, .reusable = false}}, SHIFT(919),
  [2321] = {.entry = {.count = 1, .reusable = false}}, SHIFT(662),
  [2323] = {.entry = {.count = 1, .reusable = false}}, SHIFT(665),
  [2325] = {.entry = {.count = 1, .reusable = false}}, SHIFT(900),
  [2327] = {.entry = {.count = 1, .reusable = false}}, SHIFT(639),
  [2329] = {.entry = {.count = 1, .reusable = false}}, SHIFT(887),
  [2331] = {.entry = {.count = 1, .reusable = false}}, SHIFT(824),
  [2333] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1805),
  [2335] = {.entry = {.count = 1, .reusable = false}}, SHIFT(618),
  [2337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(613),
  [2339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(652),
  [2341] = {.entry = {.count = 1, .reusable = false}}, SHIFT(926),
  [2343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(606),
  [2345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1157),
  [2347] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(600),
  [2350] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(822),
  [2353] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(611),
  [2356] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(605),
  [2359] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(595),
  [2362] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2064),
  [2365] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2179),
  [2368] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2189),
  [2371] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(900),
  [2374] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(639),
  [2377] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0),
  [2379] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(684),
  [2382] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(822),
  [2385] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(887),
  [2388] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(824),
  [2391] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1805),
  [2394] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(618),
  [2397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1233),
  [2399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(125),
  [2401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(147),
  [2403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(187),
  [2405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(144),
  [2407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1234),
  [2409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(188),
  [2411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(924),
  [2413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(662),
  [2415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(665),
  [2417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(936),
  [2419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(957),
  [2421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(667),
  [2423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(931),
  [2425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(954),
  [2427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(940),
  [2429] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(600),
  [2432] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(822),
  [2435] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(611),
  [2438] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(605),
  [2441] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(595),
  [2444] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2064),
  [2447] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(665),
  [2450] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(900),
  [2453] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(639),
  [2456] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(684),
  [2459] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(822),
  [2462] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(887),
  [2465] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(824),
  [2468] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1805),
  [2471] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(618),
  [2474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(923),
  [2476] = {.entry = {.count = 1, .reusable = true}}, SHIFT(670),
  [2478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(921),
  [2480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(941),
  [2482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(932),
  [2484] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_preproc_else_in_initializer_list, 1, 20, 0), REDUCE(sym_preproc_else_in_initializer_list_no_comma, 1, -1, 0),
  [2487] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(600),
  [2490] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(822),
  [2493] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(611),
  [2496] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(605),
  [2499] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(595),
  [2502] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(2064),
  [2505] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0),
  [2507] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(900),
  [2510] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(639),
  [2513] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(684),
  [2516] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(822),
  [2519] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(887),
  [2522] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(824),
  [2525] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(1805),
  [2528] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(618),
  [2531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(649),
  [2533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(515),
  [2535] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(600),
  [2538] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(822),
  [2541] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(611),
  [2544] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(605),
  [2547] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(595),
  [2550] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(2064),
  [2553] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(900),
  [2556] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(639),
  [2559] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(684),
  [2562] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(822),
  [2565] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(887),
  [2568] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(824),
  [2571] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(1805),
  [2574] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 1, 0, 0), SHIFT_REPEAT(618),
  [2577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(378),
  [2579] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_initializer_list, 1, 20, 0),
  [2581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(247),
  [2583] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_initializer_list, 2, 20, 0),
  [2585] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_initializer_list_no_comma, 1, -1, 0),
  [2587] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_initializer_list_no_comma, 2, -1, 0),
  [2589] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1102),
  [2591] = {.entry = {.count = 1, .reusable = false}}, SHIFT(787),
  [2593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(894),
  [2595] = {.entry = {.count = 1, .reusable = false}}, SHIFT(682),
  [2597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(603),
  [2599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(787),
  [2601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(774),
  [2603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(788),
  [2605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(122),
  [2607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(703),
  [2609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(180),
  [2611] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1174),
  [2613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(98),
  [2615] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1088),
  [2617] = {.entry = {.count = 1, .reusable = true}}, SHIFT(642),
  [2619] = {.entry = {.count = 1, .reusable = true}}, SHIFT(854),
  [2621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(833),
  [2623] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 2, 0, 0),
  [2625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(705),
  [2627] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 4, 0, 77),
  [2629] = {.entry = {.count = 1, .reusable = false}}, SHIFT(176),
  [2631] = {.entry = {.count = 1, .reusable = false}}, SHIFT(828),
  [2633] = {.entry = {.count = 1, .reusable = false}}, SHIFT(190),
  [2635] = {.entry = {.count = 1, .reusable = false}}, SHIFT(191),
  [2637] = {.entry = {.count = 1, .reusable = false}}, SHIFT(174),
  [2639] = {.entry = {.count = 1, .reusable = true}}, SHIFT(901),
  [2641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(650),
  [2643] = {.entry = {.count = 1, .reusable = false}}, SHIFT(685),
  [2645] = {.entry = {.count = 1, .reusable = true}}, SHIFT(828),
  [2647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(735),
  [2649] = {.entry = {.count = 1, .reusable = true}}, SHIFT(829),
  [2651] = {.entry = {.count = 1, .reusable = true}}, SHIFT(182),
  [2653] = {.entry = {.count = 1, .reusable = false}}, SHIFT(89),
  [2655] = {.entry = {.count = 1, .reusable = false}}, SHIFT(832),
  [2657] = {.entry = {.count = 1, .reusable = false}}, SHIFT(120),
  [2659] = {.entry = {.count = 1, .reusable = false}}, SHIFT(132),
  [2661] = {.entry = {.count = 1, .reusable = false}}, SHIFT(87),
  [2663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(902),
  [2665] = {.entry = {.count = 1, .reusable = true}}, SHIFT(636),
  [2667] = {.entry = {.count = 1, .reusable = false}}, SHIFT(686),
  [2669] = {.entry = {.count = 1, .reusable = true}}, SHIFT(832),
  [2671] = {.entry = {.count = 1, .reusable = true}}, SHIFT(754),
  [2673] = {.entry = {.count = 1, .reusable = true}}, SHIFT(840),
  [2675] = {.entry = {.count = 1, .reusable = true}}, SHIFT(112),
  [2677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(691),
  [2679] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 3, 0, 10),
  [2681] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 3, 0, 64),
  [2683] = {.entry = {.count = 1, .reusable = false}}, SHIFT(116),
  [2685] = {.entry = {.count = 1, .reusable = false}}, SHIFT(803),
  [2687] = {.entry = {.count = 1, .reusable = false}}, SHIFT(140),
  [2689] = {.entry = {.count = 1, .reusable = false}}, SHIFT(141),
  [2691] = {.entry = {.count = 1, .reusable = false}}, SHIFT(128),
  [2693] = {.entry = {.count = 1, .reusable = true}}, SHIFT(898),
  [2695] = {.entry = {.count = 1, .reusable = true}}, SHIFT(648),
  [2697] = {.entry = {.count = 1, .reusable = false}}, SHIFT(683),
  [2699] = {.entry = {.count = 1, .reusable = true}}, SHIFT(803),
  [2701] = {.entry = {.count = 1, .reusable = true}}, SHIFT(736),
  [2703] = {.entry = {.count = 1, .reusable = true}}, SHIFT(805),
  [2705] = {.entry = {.count = 1, .reusable = true}}, SHIFT(138),
  [2707] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1069),
  [2709] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1693),
  [2711] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1781),
  [2713] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1849),
  [2715] = {.entry = {.count = 1, .reusable = true}}, SHIFT(121),
  [2717] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1175),
  [2719] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1846),
  [2721] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1655),
  [2723] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2201),
  [2725] = {.entry = {.count = 1, .reusable = true}}, SHIFT(602),
  [2727] = {.entry = {.count = 1, .reusable = true}}, SHIFT(181),
  [2729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(97),
  [2731] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1861),
  [2733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(572),
  [2735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(571),
  [2737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(51),
  [2739] = {.entry = {.count = 1, .reusable = true}}, SHIFT(570),
  [2741] = {.entry = {.count = 1, .reusable = true}}, SHIFT(149),
  [2743] = {.entry = {.count = 1, .reusable = true}}, SHIFT(50),
  [2745] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 3, 20, 24), REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 3, -1, 24),
  [2748] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 3, 20, 24), REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 3, -1, 24),
  [2751] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0),
  [2753] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 4, -1, 24),
  [2755] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 4, -1, 24),
  [2757] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 6, -1, 82),
  [2759] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 6, -1, 82),
  [2761] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 4, -1, 24),
  [2763] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 4, -1, 24),
  [2765] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 4, 20, 24), REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 4, -1, 24),
  [2768] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 4, 20, 24), REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 4, -1, 24),
  [2771] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 3, 20, 24), REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 3, -1, 24),
  [2774] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 3, 20, 24), REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 3, -1, 24),
  [2777] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 4, 20, 48),
  [2779] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 4, 20, 48),
  [2781] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 5, 20, 67),
  [2783] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 5, 20, 67),
  [2785] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 5, -1, 67),
  [2787] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 5, -1, 67),
  [2789] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 5, 20, 24),
  [2791] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 5, 20, 24),
  [2793] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 5, -1, 24),
  [2795] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 5, -1, 24),
  [2797] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 3, 0, 0),
  [2799] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_initializer_list_repeat1, 3, 0, 0),
  [2801] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 5, 20, 67),
  [2803] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 5, 20, 67),
  [2805] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 4, 20, 24), REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 4, -1, 24),
  [2808] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 4, 20, 24), REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 4, -1, 24),
  [2811] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 4, 20, 48),
  [2813] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 4, 20, 48),
  [2815] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 5, -1, 67),
  [2817] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 5, -1, 67),
  [2819] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 5, 20, 24),
  [2821] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 5, 20, 24),
  [2823] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 5, -1, 24),
  [2825] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 5, -1, 24),
  [2827] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 4, -1, 48),
  [2829] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 4, -1, 48),
  [2831] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 4, -1, 48),
  [2833] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list_no_comma, 4, -1, 48),
  [2835] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 6, 20, 82),
  [2837] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 6, 20, 82),
  [2839] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 4, 20, 24),
  [2841] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 4, 20, 24),
  [2843] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 6, -1, 82),
  [2845] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_initializer_list_no_comma, 6, -1, 82),
  [2847] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 6, 20, 82),
  [2849] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_initializer_list, 6, 20, 82),
  [2851] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 4, 20, 24),
  [2853] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_initializer_list, 4, 20, 24),
  [2855] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1453),
  [2857] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1451),
  [2859] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1446),
  [2861] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1448),
  [2863] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2208),
  [2865] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2024),
  [2867] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2210),
  [2869] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2147),
  [2871] = {.entry = {.count = 1, .reusable = true}}, SHIFT(965),
  [2873] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1031),
  [2875] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2090),
  [2877] = {.entry = {.count = 1, .reusable = false}}, SHIFT(974),
  [2879] = {.entry = {.count = 1, .reusable = true}}, SHIFT(968),
  [2881] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1027),
  [2883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2208),
  [2885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2024),
  [2887] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2210),
  [2889] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2147),
  [2891] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1035),
  [2893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2090),
  [2895] = {.entry = {.count = 1, .reusable = true}}, SHIFT(974),
  [2897] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1030),
  [2899] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1022),
  [2901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1036),
  [2903] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1453),
  [2906] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1451),
  [2909] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1062),
  [2912] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1052),
  [2915] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1446),
  [2918] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_defined_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1448),
  [2921] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2208),
  [2924] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2024),
  [2927] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2210),
  [2930] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2147),
  [2933] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_field_declaration_list_repeat1, 2, 0, 0),
  [2935] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2090),
  [2938] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1902),
  [2940] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_field_declaration_list, 2, 20, 0),
  [2942] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_field_declaration_list, 1, 20, 0),
  [2944] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1439),
  [2946] = {.entry = {.count = 1, .reusable = false}}, SHIFT(983),
  [2948] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1588),
  [2950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1040),
  [2952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(394),
  [2954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(431),
  [2956] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(1439),
  [2959] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(1428),
  [2962] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(1062),
  [2965] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(1052),
  [2968] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(1424),
  [2971] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(1429),
  [2974] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_function_definition_repeat1, 2, 0, 0),
  [2976] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_field_declaration_list, 5, 20, 67),
  [2978] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_field_declaration_list, 5, 20, 67),
  [2980] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1377),
  [2982] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__declaration_modifiers, 1, 0, 0),
  [2984] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1662),
  [2986] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1384),
  [2988] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_field_declaration_list, 4, 20, 48),
  [2990] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_field_declaration_list, 4, 20, 48),
  [2992] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 2, 0, 40),
  [2994] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 2, 0, 40),
  [2996] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1378),
  [2998] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_field_declaration_list, 5, 20, 67),
  [3000] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_field_declaration_list, 5, 20, 67),
  [3002] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_field_declaration_list, 5, 20, 24),
  [3004] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_field_declaration_list, 5, 20, 24),
  [3006] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1379),
  [3008] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_field_declaration_list, 6, 20, 82),
  [3010] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_field_declaration_list, 6, 20, 82),
  [3012] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_field_declaration_list, 6, 20, 82),
  [3014] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_field_declaration_list, 6, 20, 82),
  [3016] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_field_declaration_list, 3, 20, 24),
  [3018] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_field_declaration_list, 3, 20, 24),
  [3020] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_field_declaration_list, 4, 20, 24),
  [3022] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_field_declaration_list, 4, 20, 24),
  [3024] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_field_declaration_list, 3, 20, 24),
  [3026] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_field_declaration_list, 3, 20, 24),
  [3028] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 3, 0, 78),
  [3030] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 3, 0, 78),
  [3032] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_field_declaration_list, 4, 20, 48),
  [3034] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_field_declaration_list, 4, 20, 48),
  [3036] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_not_defined_in_field_declaration_list, 4, 20, 24),
  [3038] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_field_declaration_list, 4, 20, 24),
  [3040] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_defined_in_field_declaration_list, 5, 20, 24),
  [3042] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_field_declaration_list, 5, 20, 24),
  [3044] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_specifier, 1, 0, 1),
  [3046] = {.entry = {.count = 1, .reusable = false}}, SHIFT(697),
  [3048] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_type_specifier, 1, 0, 1), REDUCE(sym_expression, 1, 0, 0),
  [3051] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0),
  [3053] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0), SHIFT_REPEAT(1062),
  [3056] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0), SHIFT_REPEAT(1052),
  [3059] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_subscript_expression_repeat1, 2, 0, 36), SHIFT_REPEAT(711),
  [3062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(711),
  [3064] = {.entry = {.count = 1, .reusable = false}}, SHIFT(681),
  [3066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1103),
  [3068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2180),
  [3070] = {.entry = {.count = 1, .reusable = false}}, SHIFT(823),
  [3072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(825),
  [3074] = {.entry = {.count = 1, .reusable = false}}, SHIFT(825),
  [3076] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_qualifier, 1, 0, 0),
  [3078] = {.entry = {.count = 1, .reusable = true}}, SHIFT(731),
  [3080] = {.entry = {.count = 1, .reusable = true}}, SHIFT(813),
  [3082] = {.entry = {.count = 1, .reusable = true}}, SHIFT(814),
  [3084] = {.entry = {.count = 1, .reusable = true}}, SHIFT(815),
  [3086] = {.entry = {.count = 1, .reusable = true}}, SHIFT(816),
  [3088] = {.entry = {.count = 1, .reusable = true}}, SHIFT(819),
  [3090] = {.entry = {.count = 1, .reusable = true}}, SHIFT(847),
  [3092] = {.entry = {.count = 1, .reusable = false}}, SHIFT(816),
  [3094] = {.entry = {.count = 1, .reusable = false}}, SHIFT(815),
  [3096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(792),
  [3098] = {.entry = {.count = 1, .reusable = false}}, SHIFT(809),
  [3100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(809),
  [3102] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1790),
  [3104] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2041),
  [3106] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2043),
  [3108] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2044),
  [3110] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1791),
  [3112] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1709),
  [3114] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_storage_class_specifier, 1, 0, 0),
  [3116] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73), SHIFT(799),
  [3119] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1738),
  [3121] = {.entry = {.count = 1, .reusable = false}}, SHIFT(702),
  [3123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1943),
  [3125] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1774),
  [3127] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1775),
  [3129] = {.entry = {.count = 1, .reusable = true}}, SHIFT(780),
  [3131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(818),
  [3133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(820),
  [3135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(826),
  [3137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(830),
  [3139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(835),
  [3141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(717),
  [3143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(718),
  [3145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(765),
  [3147] = {.entry = {.count = 1, .reusable = false}}, SHIFT(676),
  [3149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(288),
  [3151] = {.entry = {.count = 1, .reusable = false}}, SHIFT(767),
  [3153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(768),
  [3155] = {.entry = {.count = 1, .reusable = false}}, SHIFT(768),
  [3157] = {.entry = {.count = 1, .reusable = false}}, SHIFT(830),
  [3159] = {.entry = {.count = 1, .reusable = false}}, SHIFT(826),
  [3161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(769),
  [3163] = {.entry = {.count = 1, .reusable = false}}, SHIFT(770),
  [3165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(770),
  [3167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1228),
  [3169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2178),
  [3171] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_comma_expression, 3, 0, 34),
  [3173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(701),
  [3175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1182),
  [3177] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_subscript_expression_repeat1, 2, 0, 36), SHIFT_REPEAT(718),
  [3180] = {.entry = {.count = 1, .reusable = true}}, SHIFT(722),
  [3182] = {.entry = {.count = 1, .reusable = true}}, SHIFT(766),
  [3184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(390),
  [3186] = {.entry = {.count = 1, .reusable = true}}, SHIFT(519),
  [3188] = {.entry = {.count = 1, .reusable = true}}, SHIFT(856),
  [3190] = {.entry = {.count = 1, .reusable = true}}, SHIFT(857),
  [3192] = {.entry = {.count = 1, .reusable = true}}, SHIFT(858),
  [3194] = {.entry = {.count = 1, .reusable = true}}, SHIFT(859),
  [3196] = {.entry = {.count = 1, .reusable = true}}, SHIFT(860),
  [3198] = {.entry = {.count = 1, .reusable = true}}, SHIFT(861),
  [3200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(800),
  [3202] = {.entry = {.count = 1, .reusable = true}}, SHIFT(882),
  [3204] = {.entry = {.count = 1, .reusable = false}}, SHIFT(862),
  [3206] = {.entry = {.count = 1, .reusable = true}}, SHIFT(863),
  [3208] = {.entry = {.count = 1, .reusable = false}}, SHIFT(863),
  [3210] = {.entry = {.count = 1, .reusable = false}}, SHIFT(860),
  [3212] = {.entry = {.count = 1, .reusable = false}}, SHIFT(859),
  [3214] = {.entry = {.count = 1, .reusable = true}}, SHIFT(864),
  [3216] = {.entry = {.count = 1, .reusable = false}}, SHIFT(865),
  [3218] = {.entry = {.count = 1, .reusable = true}}, SHIFT(865),
  [3220] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1270),
  [3222] = {.entry = {.count = 1, .reusable = false}}, SHIFT(696),
  [3224] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_init_declarator, 3, 0, 52),
  [3226] = {.entry = {.count = 1, .reusable = true}}, SHIFT(253),
  [3228] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73), SHIFT(766),
  [3231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1965),
  [3233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(488),
  [3235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1112),
  [3237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1093),
  [3239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(256),
  [3241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(136),
  [3243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(111),
  [3245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(612),
  [3247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(597),
  [3249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(186),
  [3251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(172),
  [3253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(127),
  [3255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(106),
  [3257] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 4, 0, 75),
  [3259] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 4, 0, 76),
  [3261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(293),
  [3263] = {.entry = {.count = 1, .reusable = false}}, SHIFT(700),
  [3265] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_string_expression_repeat1, 2, 0, 0),
  [3267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(692),
  [3269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(850),
  [3271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1464),
  [3273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1465),
  [3275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(817),
  [3277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1466),
  [3279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(362),
  [3281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(704),
  [3283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(496),
  [3285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(632),
  [3287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(512),
  [3289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(491),
  [3291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(527),
  [3293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(533),
  [3295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(470),
  [3297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(257),
  [3299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1066),
  [3301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(452),
  [3303] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 3, 0, 62),
  [3305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(217),
  [3307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(218),
  [3309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(276),
  [3311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(286),
  [3313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(219),
  [3315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1469),
  [3317] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 5, 0, 85),
  [3319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(457),
  [3321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(693),
  [3323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1470),
  [3325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(353),
  [3327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(601),
  [3329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(751),
  [3331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1478),
  [3333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(354),
  [3335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(179),
  [3337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1958),
  [3339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2050),
  [3341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(109),
  [3343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1217),
  [3345] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73), SHIFT(868),
  [3348] = {.entry = {.count = 1, .reusable = false}}, SHIFT(708),
  [3350] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_argument_list_repeat1, 2, 0, 0),
  [3352] = {.entry = {.count = 1, .reusable = true}}, SHIFT(117),
  [3354] = {.entry = {.count = 1, .reusable = true}}, SHIFT(876),
  [3356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1477),
  [3358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(773),
  [3360] = {.entry = {.count = 1, .reusable = true}}, SHIFT(899),
  [3362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2229),
  [3364] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1858),
  [3366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1181),
  [3368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(776),
  [3370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(779),
  [3372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1467),
  [3374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1468),
  [3376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(872),
  [3378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1076),
  [3380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1134),
  [3382] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1005),
  [3384] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1782),
  [3386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1869),
  [3388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(119),
  [3390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(85),
  [3392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1843),
  [3394] = {.entry = {.count = 1, .reusable = true}}, SHIFT(596),
  [3396] = {.entry = {.count = 1, .reusable = true}}, SHIFT(582),
  [3398] = {.entry = {.count = 1, .reusable = true}}, SHIFT(178),
  [3400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(157),
  [3402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1694),
  [3404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1562),
  [3406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1493),
  [3408] = {.entry = {.count = 1, .reusable = true}}, SHIFT(105),
  [3410] = {.entry = {.count = 1, .reusable = true}}, SHIFT(79),
  [3412] = {.entry = {.count = 1, .reusable = true}}, SHIFT(804),
  [3414] = {.entry = {.count = 1, .reusable = true}}, SHIFT(848),
  [3416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(849),
  [3418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(853),
  [3420] = {.entry = {.count = 1, .reusable = false}}, SHIFT(688),
  [3422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(875),
  [3424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(806),
  [3426] = {.entry = {.count = 1, .reusable = true}}, SHIFT(877),
  [3428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(880),
  [3430] = {.entry = {.count = 1, .reusable = true}}, SHIFT(883),
  [3432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(886),
  [3434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(889),
  [3436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(895),
  [3438] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73), SHIFT(821),
  [3441] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73), SHIFT(871),
  [3444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(38),
  [3446] = {.entry = {.count = 1, .reusable = true}}, SHIFT(54),
  [3448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(752),
  [3450] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73), SHIFT(752),
  [3453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(32),
  [3455] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73), SHIFT(778),
  [3458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(777),
  [3460] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73), SHIFT(777),
  [3463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(798),
  [3465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(799),
  [3467] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_device_literal, 5, 46, 73), SHIFT(798),
  [3470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(778),
  [3472] = {.entry = {.count = 1, .reusable = true}}, SHIFT(871),
  [3474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(821),
  [3476] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2049),
  [3478] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2018),
  [3480] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2197),
  [3482] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2019),
  [3484] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2199),
  [3486] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2219),
  [3488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1301),
  [3490] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1356),
  [3492] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2177),
  [3494] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1310),
  [3496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1305),
  [3498] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1368),
  [3500] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2049),
  [3502] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2018),
  [3504] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2197),
  [3506] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2019),
  [3508] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2199),
  [3510] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2219),
  [3512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1359),
  [3514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2177),
  [3516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1310),
  [3518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1366),
  [3520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1362),
  [3522] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1364),
  [3524] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2049),
  [3527] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2018),
  [3530] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2197),
  [3533] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2019),
  [3536] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2199),
  [3539] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2219),
  [3542] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat1, 2, 0, 0),
  [3544] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2177),
  [3547] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2049),
  [3550] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2018),
  [3553] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2197),
  [3556] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2019),
  [3559] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2199),
  [3562] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2219),
  [3565] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 2, 0, 0),
  [3567] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2177),
  [3570] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2130),
  [3572] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2137),
  [3574] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2228),
  [3576] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2040),
  [3578] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2101),
  [3580] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2139),
  [3582] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1323),
  [3584] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1432),
  [3586] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2097),
  [3588] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1336),
  [3590] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1317),
  [3592] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1437),
  [3594] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_data_event_block, 1, 20, 0),
  [3596] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_data_event_block, 2, 20, 0),
  [3598] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 1, 0, 0), SHIFT(2049),
  [3601] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 1, 0, 0), SHIFT(2018),
  [3604] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 1, 0, 0), SHIFT(2197),
  [3607] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 1, 0, 0), SHIFT(2019),
  [3610] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 1, 0, 0), SHIFT(2199),
  [3613] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 1, 0, 0), SHIFT(2219),
  [3616] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 1, 0, 0),
  [3618] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_block_repeat2, 1, 0, 0), SHIFT(2177),
  [3621] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2191),
  [3623] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2245),
  [3625] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2249),
  [3627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1326),
  [3629] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1407),
  [3631] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2253),
  [3633] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1338),
  [3635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2130),
  [3637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2137),
  [3639] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1888),
  [3641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2139),
  [3643] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1436),
  [3645] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2097),
  [3647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1336),
  [3649] = {.entry = {.count = 1, .reusable = true}}, SHIFT(427),
  [3651] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1329),
  [3653] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1416),
  [3655] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1435),
  [3657] = {.entry = {.count = 1, .reusable = true}}, SHIFT(289),
  [3659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(295),
  [3661] = {.entry = {.count = 1, .reusable = true}}, SHIFT(408),
  [3663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1441),
  [3665] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1433),
  [3667] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2130),
  [3670] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2137),
  [3673] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(1888),
  [3676] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2228),
  [3679] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2040),
  [3682] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2101),
  [3685] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2139),
  [3688] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 2, 0, 0),
  [3690] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2097),
  [3693] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2191),
  [3695] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2245),
  [3697] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2249),
  [3699] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1414),
  [3701] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2253),
  [3703] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1338),
  [3705] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2130),
  [3708] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2137),
  [3711] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(1888),
  [3714] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2228),
  [3717] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2040),
  [3720] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2101),
  [3723] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2139),
  [3726] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat1, 2, 0, 0),
  [3728] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2097),
  [3731] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1387),
  [3733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1396),
  [3735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1394),
  [3737] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2191),
  [3740] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2245),
  [3743] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2018),
  [3746] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2197),
  [3749] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2019),
  [3752] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2199),
  [3755] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2249),
  [3758] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 2, 0, 0),
  [3760] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 2, 0, 0), SHIFT_REPEAT(2253),
  [3763] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 1, 0, 0), SHIFT(2130),
  [3766] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 1, 0, 0), SHIFT(2137),
  [3769] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 1, 0, 0), SHIFT(1888),
  [3772] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 1, 0, 0), SHIFT(2228),
  [3775] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 1, 0, 0), SHIFT(2040),
  [3778] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 1, 0, 0), SHIFT(2101),
  [3781] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 1, 0, 0), SHIFT(2139),
  [3784] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 1, 0, 0),
  [3786] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_block_repeat2, 1, 0, 0), SHIFT(2097),
  [3789] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2191),
  [3792] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2245),
  [3795] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2018),
  [3798] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2197),
  [3801] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2019),
  [3804] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2199),
  [3807] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2249),
  [3810] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat1, 2, 0, 0),
  [3812] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat1, 2, 0, 0), SHIFT_REPEAT(2253),
  [3815] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_channel_event_block, 1, 20, 0),
  [3817] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_channel_event_block, 2, 20, 0),
  [3819] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_button_event_block, 1, 20, 0),
  [3821] = {.entry = {.count = 1, .reusable = true}}, SHIFT(291),
  [3823] = {.entry = {.count = 1, .reusable = true}}, SHIFT(297),
  [3825] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_button_event_block, 2, 20, 0),
  [3827] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 1, 0, 0), SHIFT(2191),
  [3830] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 1, 0, 0), SHIFT(2245),
  [3833] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 1, 0, 0), SHIFT(2018),
  [3836] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 1, 0, 0), SHIFT(2197),
  [3839] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 1, 0, 0), SHIFT(2019),
  [3842] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 1, 0, 0), SHIFT(2199),
  [3845] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 1, 0, 0), SHIFT(2249),
  [3848] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 1, 0, 0),
  [3850] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_block_repeat2, 1, 0, 0), SHIFT(2253),
  [3853] = {.entry = {.count = 1, .reusable = true}}, SHIFT(459),
  [3855] = {.entry = {.count = 1, .reusable = true}}, SHIFT(410),
  [3857] = {.entry = {.count = 1, .reusable = true}}, SHIFT(296),
  [3859] = {.entry = {.count = 1, .reusable = true}}, SHIFT(428),
  [3861] = {.entry = {.count = 1, .reusable = true}}, SHIFT(409),
  [3863] = {.entry = {.count = 1, .reusable = true}}, SHIFT(290),
  [3865] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_data_event_block, 3, 20, 24),
  [3867] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_data_event_block, 5, 20, 67),
  [3869] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_data_event_block, 4, 20, 48),
  [3871] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_data_event_block, 4, 20, 24),
  [3873] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_data_event_block, 4, 20, 48),
  [3875] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_event_handler, 3, 0, 69),
  [3877] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_data_event_block, 5, 20, 24),
  [3879] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_data_event_block, 5, 20, 67),
  [3881] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_data_event_block, 5, 20, 24),
  [3883] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_data_event_block, 6, 20, 82),
  [3885] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_data_event_block, 4, 20, 24),
  [3887] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_data_event_block, 6, 20, 82),
  [3889] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_data_event_block, 3, 20, 24),
  [3891] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__declaration_specifiers, 2, 0, 16),
  [3893] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1442),
  [3895] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1443),
  [3897] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_specifiers, 2, 0, 16),
  [3899] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0), SHIFT_REPEAT(1442),
  [3902] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0), SHIFT_REPEAT(1443),
  [3905] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0),
  [3907] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__declaration_specifiers, 1, 0, 2),
  [3909] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_specifiers, 1, 0, 2),
  [3911] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__declaration_specifiers, 3, 0, 16),
  [3913] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_specifiers, 3, 0, 16),
  [3915] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__declaration_specifiers, 2, 0, 2),
  [3917] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_specifiers, 2, 0, 2),
  [3919] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1452),
  [3921] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1449),
  [3923] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0), SHIFT_REPEAT(1452),
  [3926] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0), SHIFT_REPEAT(1449),
  [3929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1664),
  [3931] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declarator, 1, 0, 0),
  [3933] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_declarator, 1, 0, 0),
  [3935] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_button_event_block, 4, 20, 24),
  [3937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1838),
  [3939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2241),
  [3941] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2242),
  [3943] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__release_event_declarator_list, 1, 0, 0),
  [3945] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__custom_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2039),
  [3948] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__custom_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1888),
  [3951] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__custom_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2228),
  [3954] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__custom_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2202),
  [3957] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__custom_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2218),
  [3960] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__custom_event_declarator_list_repeat1, 2, 0, 0),
  [3962] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__push_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1845),
  [3965] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__push_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1888),
  [3968] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__push_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2228),
  [3971] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__push_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2230),
  [3974] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__push_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2231),
  [3977] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__push_event_declarator_list_repeat1, 2, 0, 0),
  [3979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1827),
  [3981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2144),
  [3983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2145),
  [3985] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__button_event_declarator_list, 2, 0, 0),
  [3987] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_button_event_block, 4, 20, 48),
  [3989] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_button_event_block, 5, 20, 67),
  [3991] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_button_event_block, 5, 20, 24),
  [3993] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_button_event_block, 5, 20, 67),
  [3995] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_button_event_block, 5, 20, 24),
  [3997] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1991),
  [3999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2069),
  [4001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2070),
  [4003] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__data_event_declarator_list, 1, 0, 0),
  [4005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1825),
  [4007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2151),
  [4009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2152),
  [4011] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__level_event_declarator_list, 2, 0, 0),
  [4013] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2039),
  [4015] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2202),
  [4017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2218),
  [4019] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__custom_event_declarator_list, 1, 0, 0),
  [4021] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1809),
  [4023] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2183),
  [4025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2192),
  [4027] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__channel_event_declarator_list, 2, 0, 0),
  [4029] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_button_event_block, 4, 20, 48),
  [4031] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__level_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1825),
  [4034] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__level_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1888),
  [4037] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__level_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2228),
  [4040] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__level_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2151),
  [4043] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__level_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2152),
  [4046] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__level_event_declarator_list_repeat1, 2, 0, 0),
  [4048] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__channel_event_declarator_list, 1, 0, 0),
  [4050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1845),
  [4052] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2230),
  [4054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2231),
  [4056] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__push_event_declarator_list, 2, 0, 0),
  [4058] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__data_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1991),
  [4061] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__data_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1888),
  [4064] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__data_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2228),
  [4067] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__data_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2069),
  [4070] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__data_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2070),
  [4073] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__data_event_declarator_list_repeat1, 2, 0, 0),
  [4075] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__release_event_declarator_list, 2, 0, 0),
  [4077] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_button_event_block, 3, 20, 24),
  [4079] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__channel_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1809),
  [4082] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__channel_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1888),
  [4085] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__channel_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2228),
  [4088] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__channel_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2183),
  [4091] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__channel_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2192),
  [4094] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__channel_event_declarator_list_repeat1, 2, 0, 0),
  [4096] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__timeline_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2020),
  [4099] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__timeline_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1888),
  [4102] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__timeline_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2228),
  [4105] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__timeline_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2082),
  [4108] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__timeline_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2083),
  [4111] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__timeline_event_declarator_list_repeat1, 2, 0, 0),
  [4113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2020),
  [4115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2082),
  [4117] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2083),
  [4119] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__timeline_event_declarator_list, 2, 0, 0),
  [4121] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_button_event_handler, 3, 0, 69),
  [4123] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_button_event_block, 6, 20, 82),
  [4125] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_button_event_block, 6, 20, 82),
  [4127] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_button_event_block, 4, 20, 24),
  [4129] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__timeline_event_declarator_list, 1, 0, 0),
  [4131] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_button_event_block, 3, 20, 24),
  [4133] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__data_event_declarator_list, 2, 0, 0),
  [4135] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__button_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1827),
  [4138] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__button_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1888),
  [4141] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__button_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2228),
  [4144] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__button_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2144),
  [4147] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__button_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2145),
  [4150] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__button_event_declarator_list_repeat1, 2, 0, 0),
  [4152] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__push_event_declarator_list, 1, 0, 0),
  [4154] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__button_event_declarator_list, 1, 0, 0),
  [4156] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__release_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1838),
  [4159] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__release_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1888),
  [4162] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__release_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2228),
  [4165] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__release_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2241),
  [4168] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__release_event_declarator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(2242),
  [4171] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__release_event_declarator_list_repeat1, 2, 0, 0),
  [4173] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__level_event_declarator_list, 1, 0, 0),
  [4175] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__custom_event_declarator_list, 2, 0, 0),
  [4177] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_structured_type, 1, 0, 0),
  [4179] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_structured_type, 1, 0, 0),
  [4181] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_channel_event_block, 4, 20, 48),
  [4183] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_channel_event_block, 6, 20, 82),
  [4185] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_channel_event_block, 6, 20, 82),
  [4187] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_primitive_type, 1, 0, 0),
  [4189] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_primitive_type, 1, 0, 0),
  [4191] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_system_type, 1, 0, 0),
  [4193] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_system_type, 1, 0, 0),
  [4195] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_channel_event_block, 5, 20, 67),
  [4197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1705),
  [4199] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_declaration, 1, 0, 40),
  [4201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(719),
  [4203] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1641),
  [4205] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_channel_event_block, 3, 20, 24),
  [4207] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_channel_event_block, 5, 20, 24),
  [4209] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_channel_event_handler, 3, 0, 69),
  [4211] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_channel_event_block, 4, 20, 24),
  [4213] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_channel_event_block, 5, 20, 24),
  [4215] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_channel_event_block, 3, 20, 24),
  [4217] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_channel_event_block, 4, 20, 48),
  [4219] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_specifier, 1, 0, 1),
  [4221] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_specifier, 1, 0, 0),
  [4223] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_specifier, 1, 0, 0),
  [4225] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_channel_event_block, 4, 20, 24),
  [4227] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_storage_class_specifier, 1, 0, 0),
  [4229] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_qualifier, 1, 0, 0),
  [4231] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_intrinsic_type, 1, 0, 0),
  [4233] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_intrinsic_type, 1, 0, 0),
  [4235] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_channel_event_block, 5, 20, 67),
  [4237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1385),
  [4239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1381),
  [4241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1380),
  [4243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1386),
  [4245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1684),
  [4247] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1500),
  [4249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1029),
  [4251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1015),
  [4253] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_level_event_declarator, 2, 0, 5),
  [4255] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_level_event_devlev_reference, 1, 0, 0),
  [4257] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_timeline_event_declarator, 2, 0, 7),
  [4259] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_custom_event_declarator, 2, 0, 8),
  [4261] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_button_event_devchan_reference, 3, 0, 0),
  [4263] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_channel_event_devchan_reference, 3, 0, 0),
  [4265] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_level_event_devlev_reference, 3, 0, 0),
  [4267] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_event_device_reference, 3, 0, 0),
  [4269] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_timeline_event_id_reference, 3, 0, 0),
  [4271] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_push_event_devchan_reference, 3, 0, 0),
  [4273] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_release_event_devchan_reference, 3, 0, 0),
  [4275] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_push_event_declarator, 2, 0, 4),
  [4277] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_push_event_devchan_reference, 1, 0, 0),
  [4279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1522),
  [4281] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1584),
  [4283] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1668),
  [4285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1524),
  [4287] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1589),
  [4289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1527),
  [4291] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1592),
  [4293] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1669),
  [4295] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_event_declarator, 2, 0, 6),
  [4297] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_devlev_expression, 5, 28, 80),
  [4299] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_custom_event_reference, 5, 0, 81),
  [4301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1514),
  [4303] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1566),
  [4305] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1667),
  [4307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1516),
  [4309] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1569),
  [4311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1531),
  [4313] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1610),
  [4315] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1704),
  [4317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1533),
  [4319] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1619),
  [4321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1535),
  [4323] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1621),
  [4325] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1681),
  [4327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1537),
  [4329] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1623),
  [4331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1518),
  [4333] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1571),
  [4335] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1687),
  [4337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1539),
  [4339] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1626),
  [4341] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1689),
  [4343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1541),
  [4345] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1629),
  [4347] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_release_event_declarator, 2, 0, 4),
  [4349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1520),
  [4351] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1581),
  [4353] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_release_event_devchan_reference, 1, 0, 0),
  [4355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1543),
  [4357] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1631),
  [4359] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1685),
  [4361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1545),
  [4363] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1635),
  [4365] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_custom_event_reference, 7, 0, 87),
  [4367] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_button_event_declarator, 2, 0, 4),
  [4369] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_button_event_devchan_reference, 1, 0, 0),
  [4371] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_channel_event_declarator, 2, 0, 4),
  [4373] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_channel_event_devchan_reference, 1, 0, 0),
  [4375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1529),
  [4377] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1599),
  [4379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1574),
  [4381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1667),
  [4383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1853),
  [4385] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1525),
  [4387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1683),
  [4389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1587),
  [4391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1668),
  [4393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1593),
  [4395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1595),
  [4397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1669),
  [4399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1597),
  [4401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1567),
  [4403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(713),
  [4405] = {.entry = {.count = 1, .reusable = false}}, SHIFT(979),
  [4407] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__field_declaration_declarator, 1, 0, 53),
  [4409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1603),
  [4411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1704),
  [4413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1605),
  [4415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1665),
  [4417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1681),
  [4419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1608),
  [4421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1611),
  [4423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1689),
  [4425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1613),
  [4427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1615),
  [4429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1685),
  [4431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1618),
  [4433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1582),
  [4435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1640),
  [4437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1657),
  [4439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1687),
  [4441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1661),
  [4443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1628),
  [4445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1580),
  [4447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1565),
  [4449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1856),
  [4451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1570),
  [4453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1585),
  [4455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1600),
  [4457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1622),
  [4459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1625),
  [4461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1633),
  [4463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1658),
  [4465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1553),
  [4467] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1556),
  [4469] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1559),
  [4471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1576),
  [4473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1578),
  [4475] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_channel_event_declarator, 4, 20, 48),
  [4477] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_push_event_declarator, 4, 20, 48),
  [4479] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_push_event_declarator_repeat2, 2, 0, 0), SHIFT_REPEAT(1845),
  [4482] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_push_event_declarator_repeat2, 2, 0, 0),
  [4484] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_push_event_declarator, 4, 20, 24),
  [4486] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_button_event_declarator, 4, 20, 48),
  [4488] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_push_event_declarator, 4, 20, 48),
  [4490] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_release_event_declarator, 4, 20, 24),
  [4492] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_release_event_declarator, 4, 20, 48),
  [4494] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_release_event_declarator_repeat2, 2, 0, 0), SHIFT_REPEAT(1838),
  [4497] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_release_event_declarator_repeat2, 2, 0, 0),
  [4499] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_release_event_declarator, 4, 20, 24),
  [4501] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_declarator_repeat2, 2, 0, 0), SHIFT_REPEAT(1827),
  [4504] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_declarator_repeat2, 2, 0, 0),
  [4506] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_release_event_declarator, 4, 20, 48),
  [4508] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_declarator, 4, 0, 71),
  [4510] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_declarator, 4, 0, 71),
  [4512] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declarator, 2, 0, 31),
  [4514] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_declaration_declarator, 2, 0, 31),
  [4516] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declarator, 2, 0, 31),
  [4518] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_button_event_declarator, 4, 20, 24),
  [4520] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_data_event_declarator, 3, 20, 24),
  [4522] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_button_event_declarator, 4, 20, 48),
  [4524] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_data_event_declarator, 3, 20, 24),
  [4526] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_level_event_declarator, 4, 20, 24),
  [4528] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_timeline_event_declarator, 3, 20, 24),
  [4530] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_level_event_declarator, 4, 20, 48),
  [4532] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_data_event_declarator, 5, 20, 67),
  [4534] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_data_event_declarator, 5, 20, 24),
  [4536] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_data_event_declarator, 5, 20, 67),
  [4538] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_data_event_declarator, 5, 20, 24),
  [4540] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_timeline_event_declarator, 5, 20, 67),
  [4542] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_timeline_event_declarator, 5, 20, 24),
  [4544] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_timeline_event_declarator, 5, 20, 67),
  [4546] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_timeline_event_declarator, 5, 20, 24),
  [4548] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_timeline_event_declarator, 3, 20, 24),
  [4550] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_data_event_declarator, 4, 20, 24),
  [4552] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_level_event_declarator_repeat2, 2, 0, 0), SHIFT_REPEAT(1825),
  [4555] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_level_event_declarator_repeat2, 2, 0, 0),
  [4557] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_button_event_declarator, 3, 20, 24),
  [4559] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_level_event_declarator, 4, 20, 24),
  [4561] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_button_event_declarator, 5, 20, 67),
  [4563] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_button_event_declarator, 5, 20, 24),
  [4565] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_button_event_declarator, 3, 20, 24),
  [4567] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_level_event_declarator, 4, 20, 48),
  [4569] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_button_event_declarator, 5, 20, 67),
  [4571] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_level_event_declarator, 3, 20, 24),
  [4573] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_button_event_declarator, 5, 20, 24),
  [4575] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_level_event_declarator, 5, 20, 67),
  [4577] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_level_event_declarator, 5, 20, 24),
  [4579] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_level_event_declarator, 5, 20, 67),
  [4581] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_level_event_declarator, 5, 20, 24),
  [4583] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_level_event_declarator, 3, 20, 24),
  [4585] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_channel_event_declarator, 4, 20, 24),
  [4587] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_data_event_declarator, 4, 20, 48),
  [4589] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_channel_event_declarator, 5, 20, 67),
  [4591] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_channel_event_declarator, 5, 20, 24),
  [4593] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_channel_event_declarator, 5, 20, 67),
  [4595] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_channel_event_declarator, 5, 20, 24),
  [4597] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_custom_event_declarator, 5, 20, 67),
  [4599] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_custom_event_declarator, 5, 20, 67),
  [4601] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_custom_event_declarator, 5, 20, 24),
  [4603] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_push_event_declarator, 5, 20, 67),
  [4605] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_channel_event_declarator, 3, 20, 24),
  [4607] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_push_event_declarator, 5, 20, 24),
  [4609] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_push_event_declarator, 5, 20, 67),
  [4611] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_push_event_declarator, 5, 20, 24),
  [4613] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_release_event_declarator, 5, 20, 67),
  [4615] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_release_event_declarator, 5, 20, 24),
  [4617] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_declarator_repeat2, 2, 0, 0), SHIFT_REPEAT(1991),
  [4620] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_declarator_repeat2, 2, 0, 0),
  [4622] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_release_event_declarator, 5, 20, 67),
  [4624] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_release_event_declarator, 5, 20, 24),
  [4626] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_channel_event_declarator, 3, 20, 24),
  [4628] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2, 2, 0, 0), SHIFT_REPEAT(1809),
  [4631] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2, 2, 0, 0),
  [4633] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_custom_event_declarator, 3, 20, 24),
  [4635] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_channel_event_declarator, 4, 20, 24),
  [4637] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_custom_event_declarator, 3, 20, 24),
  [4639] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_channel_event_declarator, 4, 20, 48),
  [4641] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_custom_event_declarator, 4, 20, 24),
  [4643] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_push_event_declarator, 3, 20, 24),
  [4645] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__field_declaration_declarator_repeat1, 2, 0, 30),
  [4647] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_button_event_declarator, 4, 20, 24),
  [4649] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_push_event_declarator, 3, 20, 24),
  [4651] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_custom_event_declarator, 4, 20, 48),
  [4653] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_release_event_declarator, 3, 20, 24),
  [4655] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2, 2, 0, 0), SHIFT_REPEAT(2039),
  [4658] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2, 2, 0, 0),
  [4660] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_custom_event_declarator, 4, 20, 24),
  [4662] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_data_event_declarator, 6, 20, 82),
  [4664] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_release_event_declarator, 3, 20, 24),
  [4666] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_data_event_declarator, 6, 20, 82),
  [4668] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_timeline_event_declarator, 6, 20, 82),
  [4670] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_timeline_event_declarator, 6, 20, 82),
  [4672] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_declarator, 3, -10, 0),
  [4674] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_declarator, 3, -10, 0),
  [4676] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_data_event_declarator, 4, 20, 24),
  [4678] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_button_event_declarator, 6, 20, 82),
  [4680] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_button_event_declarator, 6, 20, 82),
  [4682] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_level_event_declarator, 6, 20, 82),
  [4684] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_level_event_declarator, 6, 20, 82),
  [4686] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_custom_event_declarator, 4, 20, 48),
  [4688] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_channel_event_declarator, 6, 20, 82),
  [4690] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_channel_event_declarator, 6, 20, 82),
  [4692] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_custom_event_declarator, 6, 20, 82),
  [4694] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_custom_event_declarator, 6, 20, 82),
  [4696] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_push_event_declarator, 6, 20, 82),
  [4698] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_push_event_declarator, 6, 20, 82),
  [4700] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_release_event_declarator, 6, 20, 82),
  [4702] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_release_event_declarator, 6, 20, 82),
  [4704] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_declarator, 3, 0, 53),
  [4706] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_declarator, 3, 0, 53),
  [4708] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_data_event_declarator, 4, 20, 48),
  [4710] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_timeline_event_declarator, 4, 20, 24),
  [4712] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_push_event_declarator, 4, 20, 24),
  [4714] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_timeline_event_declarator, 4, 20, 48),
  [4716] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2, 2, 0, 0), SHIFT_REPEAT(2020),
  [4719] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2, 2, 0, 0),
  [4721] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_timeline_event_declarator, 4, 20, 24),
  [4723] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_not_defined_in_timeline_event_declarator, 4, 20, 48),
  [4725] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__declarator, 1, 0, 0),
  [4727] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_defined_in_custom_event_declarator, 5, 20, 24),
  [4729] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_declarator_repeat1, 2, 0, 0), SHIFT_REPEAT(1827),
  [4732] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_declarator_repeat1, 2, 0, 0),
  [4734] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_data_event_declarator, 1, 20, 0),
  [4736] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_button_event_declarator, 1, 20, 0),
  [4738] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_level_event_declarator, 1, 20, 0),
  [4740] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_level_event_declarator_repeat2, 1, 0, 0), SHIFT(1825),
  [4743] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_level_event_declarator_repeat2, 1, 0, 0),
  [4745] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_declarator_repeat2, 1, 0, 0), SHIFT(1827),
  [4748] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_button_event_declarator_repeat2, 1, 0, 0),
  [4750] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_field_declarator, 2, 0, 31),
  [4752] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_field_declarator, 2, 0, 31),
  [4754] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_declarator_repeat2, 1, 0, 0), SHIFT(1991),
  [4757] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_declarator_repeat2, 1, 0, 0),
  [4759] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_data_event_declarator, 2, 20, 0),
  [4761] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2, 1, 0, 0), SHIFT(1809),
  [4764] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_declarator_repeat2, 1, 0, 0),
  [4766] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_channel_event_declarator, 2, 20, 0),
  [4768] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_push_event_declarator_repeat1, 2, 0, 0), SHIFT_REPEAT(1845),
  [4771] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_push_event_declarator_repeat1, 2, 0, 0),
  [4773] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1, 2, 0, 0), SHIFT_REPEAT(1809),
  [4776] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_channel_event_declarator_repeat1, 2, 0, 0),
  [4778] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_declarator_repeat1, 2, 0, 0), SHIFT_REPEAT(1991),
  [4781] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_data_event_declarator_repeat1, 2, 0, 0),
  [4783] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_release_event_declarator_repeat1, 2, 0, 0), SHIFT_REPEAT(1838),
  [4786] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_release_event_declarator_repeat1, 2, 0, 0),
  [4788] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_custom_event_declarator, 1, 20, 0),
  [4790] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2, 1, 0, 0), SHIFT(2039),
  [4793] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_custom_event_declarator_repeat2, 1, 0, 0),
  [4795] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_field_declarator, 2, -10, 0),
  [4797] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_field_declarator, 2, -10, 0),
  [4799] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__field_declarator, 1, 0, 65),
  [4801] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__field_declarator, 1, 0, 65),
  [4803] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_release_event_declarator, 1, 20, 0),
  [4805] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_release_event_declarator_repeat2, 1, 0, 0), SHIFT(1838),
  [4808] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_release_event_declarator_repeat2, 1, 0, 0),
  [4810] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_timeline_event_declarator, 1, 20, 0),
  [4812] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_push_event_declarator, 2, 20, 0),
  [4814] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_push_event_declarator, 1, 20, 0),
  [4816] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_field_declarator, 3, -10, 0),
  [4818] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_field_declarator, 3, -10, 0),
  [4820] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_push_event_declarator_repeat2, 1, 0, 0), SHIFT(1845),
  [4823] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_push_event_declarator_repeat2, 1, 0, 0),
  [4825] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2, 1, 0, 0), SHIFT(2020),
  [4828] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat2, 1, 0, 0),
  [4830] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_field_declarator, 3, 0, 53),
  [4832] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_field_declarator, 3, 0, 53),
  [4834] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_field_declarator, 4, 0, 71),
  [4836] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_field_declarator, 4, 0, 71),
  [4838] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_timeline_event_declarator, 2, 20, 0),
  [4840] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_declaration, 2, 0, 32),
  [4842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(714),
  [4844] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_level_event_declarator_repeat1, 2, 0, 0), SHIFT_REPEAT(1825),
  [4847] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_level_event_declarator_repeat1, 2, 0, 0),
  [4849] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1, 2, 0, 0), SHIFT_REPEAT(2020),
  [4852] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_timeline_event_declarator_repeat1, 2, 0, 0),
  [4854] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_custom_event_declarator, 2, 20, 0),
  [4856] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_level_event_declarator, 2, 20, 0),
  [4858] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_button_event_declarator, 2, 20, 0),
  [4860] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_release_event_declarator, 2, 20, 0),
  [4862] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1, 2, 0, 0), SHIFT_REPEAT(2039),
  [4865] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_defined_in_custom_event_declarator_repeat1, 2, 0, 0),
  [4867] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_channel_event_declarator, 1, 20, 0),
  [4869] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__field_declarator, 1, 0, 0),
  [4871] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__field_declarator, 1, 0, 0),
  [4873] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_string_literal_repeat1, 2, 0, 0),
  [4875] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_string_literal_repeat1, 2, 0, 0), SHIFT_REPEAT(1707),
  [4878] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_string_literal_repeat1, 2, 0, 0), SHIFT_REPEAT(1707),
  [4881] = {.entry = {.count = 1, .reusable = true}}, SHIFT(706),
  [4883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(720),
  [4885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1458),
  [4887] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1068),
  [4889] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1098),
  [4891] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1075),
  [4893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1057),
  [4895] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1054),
  [4897] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1063),
  [4899] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1060),
  [4901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1080),
  [4903] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1100),
  [4905] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1083),
  [4907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1082),
  [4909] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1049),
  [4911] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1048),
  [4913] = {.entry = {.count = 1, .reusable = true}}, SHIFT(301),
  [4915] = {.entry = {.count = 1, .reusable = true}}, SHIFT(300),
  [4917] = {.entry = {.count = 1, .reusable = true}}, SHIFT(715),
  [4919] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1690),
  [4921] = {.entry = {.count = 1, .reusable = true}}, SHIFT(385),
  [4923] = {.entry = {.count = 1, .reusable = true}}, SHIFT(384),
  [4925] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1780),
  [4927] = {.entry = {.count = 1, .reusable = true}}, SHIFT(497),
  [4929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1073),
  [4931] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1055),
  [4933] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1072),
  [4935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1050),
  [4937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1078),
  [4939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1074),
  [4941] = {.entry = {.count = 1, .reusable = true}}, SHIFT(499),
  [4943] = {.entry = {.count = 1, .reusable = true}}, SHIFT(498),
  [4945] = {.entry = {.count = 1, .reusable = true}}, SHIFT(383),
  [4947] = {.entry = {.count = 1, .reusable = true}}, SHIFT(501),
  [4949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(500),
  [4951] = {.entry = {.count = 1, .reusable = true}}, SHIFT(503),
  [4953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(502),
  [4955] = {.entry = {.count = 1, .reusable = false}}, SHIFT(145),
  [4957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1733),
  [4959] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1733),
  [4961] = {.entry = {.count = 1, .reusable = true}}, SHIFT(505),
  [4963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(504),
  [4965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(507),
  [4967] = {.entry = {.count = 1, .reusable = true}}, SHIFT(506),
  [4969] = {.entry = {.count = 1, .reusable = true}}, SHIFT(509),
  [4971] = {.entry = {.count = 1, .reusable = true}}, SHIFT(508),
  [4973] = {.entry = {.count = 1, .reusable = false}}, SHIFT(146),
  [4975] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1707),
  [4977] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1707),
  [4979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(221),
  [4981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(223),
  [4983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(222),
  [4985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(225),
  [4987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(224),
  [4989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(208),
  [4991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(227),
  [4993] = {.entry = {.count = 1, .reusable = true}}, SHIFT(226),
  [4995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(229),
  [4997] = {.entry = {.count = 1, .reusable = true}}, SHIFT(228),
  [4999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(231),
  [5001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(230),
  [5003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(233),
  [5005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(232),
  [5007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(235),
  [5009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(234),
  [5011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(237),
  [5013] = {.entry = {.count = 1, .reusable = true}}, SHIFT(267),
  [5015] = {.entry = {.count = 1, .reusable = true}}, SHIFT(326),
  [5017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(325),
  [5019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(239),
  [5021] = {.entry = {.count = 1, .reusable = true}}, SHIFT(238),
  [5023] = {.entry = {.count = 1, .reusable = true}}, SHIFT(241),
  [5025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(240),
  [5027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(243),
  [5029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(242),
  [5031] = {.entry = {.count = 1, .reusable = false}}, SHIFT(453),
  [5033] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1752),
  [5035] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1752),
  [5037] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__field_declaration_declarator, 2, 0, 79),
  [5039] = {.entry = {.count = 1, .reusable = false}}, SHIFT(454),
  [5041] = {.entry = {.count = 1, .reusable = true}}, SHIFT(324),
  [5043] = {.entry = {.count = 1, .reusable = true}}, SHIFT(320),
  [5045] = {.entry = {.count = 1, .reusable = true}}, SHIFT(303),
  [5047] = {.entry = {.count = 1, .reusable = true}}, SHIFT(302),
  [5049] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__field_declaration_declarator_repeat1, 2, 0, 51), SHIFT_REPEAT(1567),
  [5052] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__field_declaration_declarator_repeat1, 2, 0, 51),
  [5054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(465),
  [5056] = {.entry = {.count = 1, .reusable = true}}, SHIFT(464),
  [5058] = {.entry = {.count = 1, .reusable = true}}, SHIFT(314),
  [5060] = {.entry = {.count = 1, .reusable = false}}, SHIFT(610),
  [5062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(467),
  [5064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(466),
  [5066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(469),
  [5068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(468),
  [5070] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1844),
  [5072] = {.entry = {.count = 1, .reusable = false}}, SHIFT(962),
  [5074] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1785),
  [5076] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1785),
  [5078] = {.entry = {.count = 1, .reusable = true}}, SHIFT(330),
  [5080] = {.entry = {.count = 1, .reusable = true}}, SHIFT(328),
  [5082] = {.entry = {.count = 1, .reusable = true}}, SHIFT(317),
  [5084] = {.entry = {.count = 1, .reusable = true}}, SHIFT(304),
  [5086] = {.entry = {.count = 1, .reusable = false}}, SHIFT(194),
  [5088] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1766),
  [5090] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1766),
  [5092] = {.entry = {.count = 1, .reusable = false}}, SHIFT(192),
  [5094] = {.entry = {.count = 1, .reusable = false}}, SHIFT(133),
  [5096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1768),
  [5098] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1768),
  [5100] = {.entry = {.count = 1, .reusable = false}}, SHIFT(134),
  [5102] = {.entry = {.count = 1, .reusable = false}}, SHIFT(371),
  [5104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1770),
  [5106] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1770),
  [5108] = {.entry = {.count = 1, .reusable = false}}, SHIFT(372),
  [5110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(387),
  [5112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(386),
  [5114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(357),
  [5116] = {.entry = {.count = 1, .reusable = true}}, SHIFT(355),
  [5118] = {.entry = {.count = 1, .reusable = true}}, SHIFT(473),
  [5120] = {.entry = {.count = 1, .reusable = true}}, SHIFT(474),
  [5122] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2110),
  [5124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1786),
  [5126] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1786),
  [5128] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_repeat1, 2, 0, 51), SHIFT_REPEAT(1458),
  [5131] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_declaration_repeat1, 2, 0, 51),
  [5133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(260),
  [5135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(261),
  [5137] = {.entry = {.count = 1, .reusable = false}}, SHIFT(961),
  [5139] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2254),
  [5141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(359),
  [5143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(358),
  [5145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1089),
  [5147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(361),
  [5149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(360),
  [5151] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1240),
  [5153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1798),
  [5155] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1798),
  [5157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1064),
  [5159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1071),
  [5161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1639),
  [5163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(299),
  [5165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1077),
  [5167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1067),
  [5169] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1278),
  [5171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(482),
  [5173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(313),
  [5175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(484),
  [5177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(483),
  [5179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(486),
  [5181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(485),
  [5183] = {.entry = {.count = 1, .reusable = false}}, SHIFT(609),
  [5185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1758),
  [5187] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1758),
  [5189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1006),
  [5191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1039),
  [5193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(108),
  [5195] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1937),
  [5197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1354),
  [5199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(790),
  [5201] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameter_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1006),
  [5204] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_parameter_list_repeat1, 2, 0, 0),
  [5206] = {.entry = {.count = 1, .reusable = true}}, SHIFT(271),
  [5208] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1598),
  [5210] = {.entry = {.count = 1, .reusable = true}}, SHIFT(388),
  [5212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(851),
  [5214] = {.entry = {.count = 1, .reusable = true}}, SHIFT(544),
  [5216] = {.entry = {.count = 1, .reusable = true}}, SHIFT(442),
  [5218] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1778),
  [5220] = {.entry = {.count = 1, .reusable = true}}, SHIFT(598),
  [5222] = {.entry = {.count = 1, .reusable = true}}, SHIFT(280),
  [5224] = {.entry = {.count = 1, .reusable = true}}, SHIFT(143),
  [5226] = {.entry = {.count = 1, .reusable = true}}, SHIFT(791),
  [5228] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_declaration_repeat1, 2, 0, 30),
  [5230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(810),
  [5232] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1954),
  [5234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1016),
  [5236] = {.entry = {.count = 1, .reusable = true}}, SHIFT(114),
  [5238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(852),
  [5240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(542),
  [5242] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1957),
  [5244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(551),
  [5246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(446),
  [5248] = {.entry = {.count = 1, .reusable = true}}, SHIFT(193),
  [5250] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_string_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(800),
  [5253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(734),
  [5255] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1879),
  [5257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(347),
  [5259] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_declarator, 1, 0, 12),
  [5261] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_array_declarator, 4, 0, 71),
  [5263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(733),
  [5265] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_array_declarator, 2, 0, 0),
  [5267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(177),
  [5269] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2000),
  [5271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(206),
  [5273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(879),
  [5275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(337),
  [5277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(630),
  [5279] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_array_declarator, 3, 0, 84),
  [5281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1221),
  [5283] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__abstract_declarator, 1, 0, 0),
  [5285] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_array_declarator, 3, 0, 53),
  [5287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(728),
  [5289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(236),
  [5291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(789),
  [5293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(419),
  [5295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1047),
  [5297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1564),
  [5299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1038),
  [5301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(126),
  [5303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(878),
  [5305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(332),
  [5307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1245),
  [5309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(807),
  [5311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(443),
  [5313] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(701),
  [5316] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1126),
  [5318] = {.entry = {.count = 1, .reusable = true}}, SHIFT(487),
  [5320] = {.entry = {.count = 1, .reusable = true}}, SHIFT(343),
  [5322] = {.entry = {.count = 1, .reusable = true}}, SHIFT(265),
  [5324] = {.entry = {.count = 1, .reusable = true}}, SHIFT(346),
  [5326] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 3, 0, 21),
  [5328] = {.entry = {.count = 1, .reusable = true}}, SHIFT(348),
  [5330] = {.entry = {.count = 1, .reusable = true}}, SHIFT(349),
  [5332] = {.entry = {.count = 1, .reusable = true}}, SHIFT(205),
  [5334] = {.entry = {.count = 1, .reusable = true}}, SHIFT(207),
  [5336] = {.entry = {.count = 1, .reusable = true}}, SHIFT(376),
  [5338] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1033),
  [5340] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1032),
  [5342] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1321),
  [5344] = {.entry = {.count = 1, .reusable = true}}, SHIFT(211),
  [5346] = {.entry = {.count = 1, .reusable = true}}, SHIFT(212),
  [5348] = {.entry = {.count = 1, .reusable = true}}, SHIFT(513),
  [5350] = {.entry = {.count = 1, .reusable = true}}, SHIFT(514),
  [5352] = {.entry = {.count = 1, .reusable = true}}, SHIFT(463),
  [5354] = {.entry = {.count = 1, .reusable = true}}, SHIFT(517),
  [5356] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 3, 0, 0),
  [5358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1318),
  [5360] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1348),
  [5362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1339),
  [5364] = {.entry = {.count = 1, .reusable = true}}, SHIFT(338),
  [5366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(563),
  [5368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1769),
  [5370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1347),
  [5372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(331),
  [5374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(471),
  [5376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(472),
  [5378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(284),
  [5380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1353),
  [5382] = {.entry = {.count = 1, .reusable = true}}, SHIFT(444),
  [5384] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1355),
  [5386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1350),
  [5388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(477),
  [5390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(480),
  [5392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(550),
  [5394] = {.entry = {.count = 1, .reusable = true}}, SHIFT(552),
  [5396] = {.entry = {.count = 1, .reusable = true}}, SHIFT(553),
  [5398] = {.entry = {.count = 1, .reusable = true}}, SHIFT(342),
  [5400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1987),
  [5402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(721),
  [5404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1349),
  [5406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1344),
  [5408] = {.entry = {.count = 1, .reusable = true}}, SHIFT(393),
  [5410] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1017),
  [5412] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1018),
  [5414] = {.entry = {.count = 1, .reusable = false}}, SHIFT(680),
  [5416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(334),
  [5418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1012),
  [5420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(520),
  [5422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(492),
  [5424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(564),
  [5426] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2092),
  [5428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(380),
  [5430] = {.entry = {.count = 1, .reusable = true}}, SHIFT(377),
  [5432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(49),
  [5434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(543),
  [5436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(263),
  [5438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(395),
  [5440] = {.entry = {.count = 1, .reusable = true}}, SHIFT(204),
  [5442] = {.entry = {.count = 1, .reusable = true}}, SHIFT(793),
  [5444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(245),
  [5446] = {.entry = {.count = 1, .reusable = true}}, SHIFT(203),
  [5448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(246),
  [5450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(249),
  [5452] = {.entry = {.count = 1, .reusable = true}}, SHIFT(250),
  [5454] = {.entry = {.count = 1, .reusable = true}}, SHIFT(812),
  [5456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1750),
  [5458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(322),
  [5460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(975),
  [5462] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1982),
  [5464] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1983),
  [5466] = {.entry = {.count = 1, .reusable = true}}, SHIFT(294),
  [5468] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2013),
  [5470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2014),
  [5472] = {.entry = {.count = 1, .reusable = true}}, SHIFT(268),
  [5474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(732),
  [5476] = {.entry = {.count = 1, .reusable = true}}, SHIFT(258),
  [5478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(259),
  [5480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1986),
  [5482] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_data_event_type, 1, 0, 0),
  [5484] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_button_event_type, 4, 0, 83),
  [5486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(524),
  [5488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(277),
  [5490] = {.entry = {.count = 1, .reusable = true}}, SHIFT(278),
  [5492] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1663),
  [5494] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1612),
  [5496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1591),
  [5498] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1555),
  [5500] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1445),
  [5502] = {.entry = {.count = 1, .reusable = true}}, SHIFT(103),
  [5504] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1887),
  [5506] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1624),
  [5508] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_system_function, 1, 0, 0),
  [5510] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1365),
  [5512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1393),
  [5514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(449),
  [5516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1557),
  [5518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1479),
  [5520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1480),
  [5522] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1985),
  [5524] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1367),
  [5526] = {.entry = {.count = 1, .reusable = true}}, SHIFT(928),
  [5528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1966),
  [5530] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1609),
  [5532] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1425),
  [5534] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1561),
  [5536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1485),
  [5538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1489),
  [5540] = {.entry = {.count = 1, .reusable = true}}, SHIFT(929),
  [5542] = {.entry = {.count = 1, .reusable = true}}, SHIFT(930),
  [5544] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1360),
  [5546] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3),
  [5548] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1915),
  [5550] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1916),
  [5552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(963),
  [5554] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2),
  [5556] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_button_event_hold_repeat, 1, 0, 0),
  [5558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(113),
  [5560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2193),
  [5562] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1014),
  [5564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1395),
  [5566] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1308),
  [5568] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1363),
  [5570] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1886),
  [5572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1833),
  [5574] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1601),
  [5576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1933),
  [5578] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4),
  [5580] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5),
  [5582] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1412),
  [5584] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1554),
  [5586] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1401),
  [5588] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1614),
  [5590] = {.entry = {.count = 1, .reusable = true}}, SHIFT(539),
  [5592] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1647),
  [5594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(558),
  [5596] = {.entry = {.count = 1, .reusable = true}}, SHIFT(545),
  [5598] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1413),
  [5600] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1357),
  [5602] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1630),
  [5604] = {.entry = {.count = 1, .reusable = true}}, SHIFT(560),
  [5606] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1602),
  [5608] = {.entry = {.count = 1, .reusable = true}}, SHIFT(594),
  [5610] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1392),
  [5612] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1579),
  [5614] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1607),
  [5616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(950),
  [5618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1426),
  [5620] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_on_keyword, 1, 0, 0),
  [5622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1642),
  [5624] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1427),
  [5626] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1568),
  [5628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1594),
  [5630] = {.entry = {.count = 1, .reusable = true}}, SHIFT(528),
  [5632] = {.entry = {.count = 1, .reusable = true}}, SHIFT(521),
  [5634] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_off_keyword, 1, 0, 0),
  [5636] = {.entry = {.count = 1, .reusable = true}}, SHIFT(955),
  [5638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1309),
  [5640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1651),
  [5642] = {.entry = {.count = 1, .reusable = true}}, SHIFT(532),
  [5644] = {.entry = {.count = 1, .reusable = true}}, SHIFT(935),
  [5646] = {.entry = {.count = 1, .reusable = true}}, SHIFT(939),
  [5648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1473),
  [5650] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1474),
  [5652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1643),
  [5654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(964),
  [5656] = {.entry = {.count = 1, .reusable = true}}, SHIFT(31),
  [5658] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1475),
  [5660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1498),
  [5662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1648),
  [5664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1659),
  [5666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1606),
  [5668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1575),
  [5670] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1586),
  [5672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1617),
  [5674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(455),
  [5676] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1652),
  [5678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1596),
  [5680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1577),
  [5682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1656),
  [5684] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_devchan_on_keyword, 1, 0, 0),
  [5686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1988),
  [5688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1841),
  [5690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(447),
  [5692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1884),
  [5694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(7),
  [5696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6),
  [5698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1025),
  [5700] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_devchan_operation, 1, 0, 0),
  [5702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(554),
  [5704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1299),
  [5706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1219),
  [5708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(638),
  [5710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1099),
  [5712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(565),
  [5714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1644),
  [5716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1481),
  [5718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1026),
  [5720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1634),
  [5722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1653),
  [5724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(567),
  [5726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(640),
  [5728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(956),
  [5730] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_button_event_type, 1, 0, 0),
  [5732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1482),
  [5734] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_button_event_type, 6, 0, 83),
  [5736] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_channel_event_type, 1, 0, 0),
  [5738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1358),
  [5740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(922),
  [5742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1808),
  [5744] = {.entry = {.count = 1, .reusable = false}}, SHIFT(677),
  [5746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1928),
  [5748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1645),
  [5750] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_return_type, 3, 0, 0),
  [5752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1483),
  [5754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1550),
  [5756] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_devchan_off_keyword, 1, 0, 0),
  [5758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1636),
  [5760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(423),
  [5762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(352),
  [5764] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1828),
  [5766] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1604),
  [5768] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1948),
  [5770] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2031),
  [5772] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1572),
  [5774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(426),
  [5776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1646),
  [5778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(269),
  [5780] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1430),
  [5782] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1484),
  [5784] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1298),
  [5786] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1009),
  [5788] = {.entry = {.count = 1, .reusable = true}}, SHIFT(52),
  [5790] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1984),
  [5792] = {.entry = {.count = 1, .reusable = true}}, SHIFT(40),
  [5794] = {.entry = {.count = 1, .reusable = true}}, SHIFT(173),
  [5796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(309),
  [5798] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1850),
  [5800] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_return_type, 4, 0, 58),
  [5802] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1486),
  [5804] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1487),
  [5806] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2023),
  [5808] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1637),
  [5810] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1021),
  [5812] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1573),
  [5814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2015),
  [5816] = {.entry = {.count = 1, .reusable = true}}, SHIFT(456),
  [5818] = {.entry = {.count = 1, .reusable = true}},  ACCEPT_INPUT(),
  [5820] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1491),
  [5822] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1492),
  [5824] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1438),
  [5826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(905),
  [5828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1034),
  [5830] = {.entry = {.count = 1, .reusable = false}}, SHIFT(687),
  [5832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1551),
  [5834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1313),
  [5836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1590),
  [5838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1650),
  [5840] = {.entry = {.count = 1, .reusable = false}}, SHIFT(674),
  [5842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1316),
  [5844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(316),
  [5846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(949),
  [5848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2021),
  [5850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1654),
  [5852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1638),
  [5854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(522),
  [5856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(937),
  [5858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(942),
  [5860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1649),
};

enum ts_external_scanner_symbol_identifiers {
  ts_external_token__automatic_semicolon = 0,
};

static const TSSymbol ts_external_scanner_symbol_map[EXTERNAL_TOKEN_COUNT] = {
  [ts_external_token__automatic_semicolon] = sym__automatic_semicolon,
};

static const bool ts_external_scanner_states[2][EXTERNAL_TOKEN_COUNT] = {
  [1] = {
    [ts_external_token__automatic_semicolon] = true,
  },
};

#ifdef __cplusplus
extern "C" {
#endif
void *tree_sitter_netlinx_external_scanner_create(void);
void tree_sitter_netlinx_external_scanner_destroy(void *);
bool tree_sitter_netlinx_external_scanner_scan(void *, TSLexer *, const bool *);
unsigned tree_sitter_netlinx_external_scanner_serialize(void *, char *);
void tree_sitter_netlinx_external_scanner_deserialize(void *, const char *, unsigned);

#ifdef TREE_SITTER_HIDE_SYMBOLS
#define TS_PUBLIC
#elif defined(_WIN32)
#define TS_PUBLIC __declspec(dllexport)
#else
#define TS_PUBLIC __attribute__((visibility("default")))
#endif

TS_PUBLIC const TSLanguage *tree_sitter_netlinx(void) {
  static const TSLanguage language = {
    .abi_version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .state_count = STATE_COUNT,
    .large_state_count = LARGE_STATE_COUNT,
    .production_id_count = PRODUCTION_ID_COUNT,
    .field_count = FIELD_COUNT,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .parse_table = &ts_parse_table[0][0],
    .small_parse_table = ts_small_parse_table,
    .small_parse_table_map = ts_small_parse_table_map,
    .parse_actions = ts_parse_actions,
    .symbol_names = ts_symbol_names,
    .field_names = ts_field_names,
    .field_map_slices = ts_field_map_slices,
    .field_map_entries = ts_field_map_entries,
    .symbol_metadata = ts_symbol_metadata,
    .public_symbol_map = ts_symbol_map,
    .alias_map = ts_non_terminal_alias_map,
    .alias_sequences = &ts_alias_sequences[0][0],
    .lex_modes = (const void*)ts_lex_modes,
    .lex_fn = ts_lex,
    .keyword_lex_fn = ts_lex_keywords,
    .keyword_capture_token = sym_identifier,
    .external_scanner = {
      &ts_external_scanner_states[0][0],
      ts_external_scanner_symbol_map,
      tree_sitter_netlinx_external_scanner_create,
      tree_sitter_netlinx_external_scanner_destroy,
      tree_sitter_netlinx_external_scanner_scan,
      tree_sitter_netlinx_external_scanner_serialize,
      tree_sitter_netlinx_external_scanner_deserialize,
    },
    .primary_state_ids = ts_primary_state_ids,
  };
  return &language;
}
#ifdef __cplusplus
}
#endif
